!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_A1B2070 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS2070* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB, 0.33944E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.35173E+00_JPRB, 0.35120E+00_JPRB, 0.35070E+00_JPRB, 0.35026E+00_JPRB, 0.34986E+00_JPRB, 0.34957E+00_JPRB, 0.34933E+00_JPRB ,&
&  0.34898E+00_JPRB, 0.34869E+00_JPRB, 0.34830E+00_JPRB, 0.34790E+00_JPRB, 0.34754E+00_JPRB, 0.34713E+00_JPRB, 0.34663E+00_JPRB ,&
&  0.34602E+00_JPRB, 0.34547E+00_JPRB, 0.34483E+00_JPRB, 0.34402E+00_JPRB, 0.34319E+00_JPRB, 0.34231E+00_JPRB, 0.34125E+00_JPRB ,&
&  0.34001E+00_JPRB, 0.33882E+00_JPRB, 0.33754E+00_JPRB, 0.33619E+00_JPRB, 0.33477E+00_JPRB, 0.33321E+00_JPRB, 0.33183E+00_JPRB ,&
&  0.33081E+00_JPRB, 0.32972E+00_JPRB, 0.32869E+00_JPRB, 0.32764E+00_JPRB, 0.32683E+00_JPRB, 0.32627E+00_JPRB, 0.32567E+00_JPRB ,&
&  0.32514E+00_JPRB, 0.32474E+00_JPRB, 0.32482E+00_JPRB, 0.32508E+00_JPRB, 0.32545E+00_JPRB, 0.32624E+00_JPRB, 0.32748E+00_JPRB ,&
&  0.32853E+00_JPRB, 0.32945E+00_JPRB, 0.33056E+00_JPRB, 0.33178E+00_JPRB, 0.33307E+00_JPRB, 0.33438E+00_JPRB, 0.33556E+00_JPRB ,&
&  0.33645E+00_JPRB, 0.33719E+00_JPRB, 0.33800E+00_JPRB, 0.33865E+00_JPRB, 0.33901E+00_JPRB, 0.33936E+00_JPRB, 0.33974E+00_JPRB ,&
&  0.34002E+00_JPRB, 0.34017E+00_JPRB, 0.34040E+00_JPRB, 0.34080E+00_JPRB, 0.34115E+00_JPRB, 0.34154E+00_JPRB, 0.34213E+00_JPRB ,&
&  0.34294E+00_JPRB, 0.34354E+00_JPRB, 0.34425E+00_JPRB, 0.34520E+00_JPRB, 0.34644E+00_JPRB, 0.34793E+00_JPRB, 0.34911E+00_JPRB ,&
&  0.35036E+00_JPRB, 0.35167E+00_JPRB, 0.35306E+00_JPRB, 0.35404E+00_JPRB, 0.35509E+00_JPRB, 0.35605E+00_JPRB, 0.35697E+00_JPRB ,&
&  0.35721E+00_JPRB, 0.35762E+00_JPRB, 0.35813E+00_JPRB, 0.35859E+00_JPRB, 0.35820E+00_JPRB, 0.35796E+00_JPRB, 0.35778E+00_JPRB ,&
&  0.35806E+00_JPRB, 0.35729E+00_JPRB, 0.35692E+00_JPRB, 0.35641E+00_JPRB, 0.35616E+00_JPRB, 0.35541E+00_JPRB, 0.35486E+00_JPRB ,&
&  0.35429E+00_JPRB, 0.35386E+00_JPRB, 0.35331E+00_JPRB, 0.35276E+00_JPRB, 0.35219E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.36380E+00_JPRB, 0.36286E+00_JPRB, 0.36183E+00_JPRB, 0.36075E+00_JPRB, 0.35964E+00_JPRB, 0.35882E+00_JPRB, 0.35798E+00_JPRB ,&
&  0.35708E+00_JPRB, 0.35635E+00_JPRB, 0.35546E+00_JPRB, 0.35472E+00_JPRB, 0.35407E+00_JPRB, 0.35385E+00_JPRB, 0.35333E+00_JPRB ,&
&  0.35271E+00_JPRB, 0.35176E+00_JPRB, 0.35094E+00_JPRB, 0.34947E+00_JPRB, 0.34782E+00_JPRB, 0.34558E+00_JPRB, 0.34348E+00_JPRB ,&
&  0.34096E+00_JPRB, 0.33804E+00_JPRB, 0.33546E+00_JPRB, 0.33294E+00_JPRB, 0.33040E+00_JPRB, 0.32778E+00_JPRB, 0.32592E+00_JPRB ,&
&  0.32474E+00_JPRB, 0.32312E+00_JPRB, 0.32173E+00_JPRB, 0.32048E+00_JPRB, 0.31902E+00_JPRB, 0.31806E+00_JPRB, 0.31720E+00_JPRB ,&
&  0.31555E+00_JPRB, 0.31394E+00_JPRB, 0.31408E+00_JPRB, 0.31450E+00_JPRB, 0.31546E+00_JPRB, 0.31738E+00_JPRB, 0.32097E+00_JPRB ,&
&  0.32566E+00_JPRB, 0.33014E+00_JPRB, 0.33492E+00_JPRB, 0.33921E+00_JPRB, 0.34270E+00_JPRB, 0.34544E+00_JPRB, 0.34768E+00_JPRB ,&
&  0.34910E+00_JPRB, 0.35015E+00_JPRB, 0.35073E+00_JPRB, 0.35078E+00_JPRB, 0.35022E+00_JPRB, 0.35003E+00_JPRB, 0.35011E+00_JPRB ,&
&  0.35032E+00_JPRB, 0.34918E+00_JPRB, 0.34883E+00_JPRB, 0.34912E+00_JPRB, 0.34992E+00_JPRB, 0.35066E+00_JPRB, 0.35228E+00_JPRB ,&
&  0.35464E+00_JPRB, 0.35758E+00_JPRB, 0.36025E+00_JPRB, 0.36381E+00_JPRB, 0.36801E+00_JPRB, 0.37287E+00_JPRB, 0.37577E+00_JPRB ,&
&  0.37936E+00_JPRB, 0.38315E+00_JPRB, 0.38716E+00_JPRB, 0.38810E+00_JPRB, 0.38916E+00_JPRB, 0.38996E+00_JPRB, 0.39099E+00_JPRB ,&
&  0.38942E+00_JPRB, 0.38884E+00_JPRB, 0.38848E+00_JPRB, 0.38907E+00_JPRB, 0.38712E+00_JPRB, 0.38617E+00_JPRB, 0.38519E+00_JPRB ,&
&  0.38564E+00_JPRB, 0.38261E+00_JPRB, 0.38090E+00_JPRB, 0.37876E+00_JPRB, 0.37751E+00_JPRB, 0.37452E+00_JPRB, 0.37217E+00_JPRB ,&
&  0.37017E+00_JPRB, 0.36893E+00_JPRB, 0.36737E+00_JPRB, 0.36602E+00_JPRB, 0.36494E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.37551E+00_JPRB, 0.37312E+00_JPRB, 0.36961E+00_JPRB, 0.36521E+00_JPRB, 0.36122E+00_JPRB, 0.35826E+00_JPRB, 0.35516E+00_JPRB ,&
&  0.35270E+00_JPRB, 0.35058E+00_JPRB, 0.34914E+00_JPRB, 0.34810E+00_JPRB, 0.34707E+00_JPRB, 0.34765E+00_JPRB, 0.34833E+00_JPRB ,&
&  0.34892E+00_JPRB, 0.34881E+00_JPRB, 0.34998E+00_JPRB, 0.34963E+00_JPRB, 0.34868E+00_JPRB, 0.34635E+00_JPRB, 0.34301E+00_JPRB ,&
&  0.33908E+00_JPRB, 0.33454E+00_JPRB, 0.32924E+00_JPRB, 0.32318E+00_JPRB, 0.31743E+00_JPRB, 0.29775E+00_JPRB, 0.29292E+00_JPRB ,&
&  0.29377E+00_JPRB, 0.29060E+00_JPRB, 0.28822E+00_JPRB, 0.28706E+00_JPRB, 0.28590E+00_JPRB, 0.28395E+00_JPRB, 0.28178E+00_JPRB ,&
&  0.26682E+00_JPRB, 0.26554E+00_JPRB, 0.26355E+00_JPRB, 0.26243E+00_JPRB, 0.26482E+00_JPRB, 0.27346E+00_JPRB, 0.28705E+00_JPRB ,&
&  0.31124E+00_JPRB, 0.32160E+00_JPRB, 0.33028E+00_JPRB, 0.34065E+00_JPRB, 0.34872E+00_JPRB, 0.35469E+00_JPRB, 0.36050E+00_JPRB ,&
&  0.36534E+00_JPRB, 0.37003E+00_JPRB, 0.37347E+00_JPRB, 0.37529E+00_JPRB, 0.37545E+00_JPRB, 0.37537E+00_JPRB, 0.37514E+00_JPRB ,&
&  0.37492E+00_JPRB, 0.37466E+00_JPRB, 0.37484E+00_JPRB, 0.37477E+00_JPRB, 0.37433E+00_JPRB, 0.37463E+00_JPRB, 0.37724E+00_JPRB ,&
&  0.38124E+00_JPRB, 0.38745E+00_JPRB, 0.39283E+00_JPRB, 0.40048E+00_JPRB, 0.40743E+00_JPRB, 0.41632E+00_JPRB, 0.41976E+00_JPRB ,&
&  0.42405E+00_JPRB, 0.42676E+00_JPRB, 0.43045E+00_JPRB, 0.43035E+00_JPRB, 0.43013E+00_JPRB, 0.42920E+00_JPRB, 0.42917E+00_JPRB ,&
&  0.42668E+00_JPRB, 0.42490E+00_JPRB, 0.42255E+00_JPRB, 0.42211E+00_JPRB, 0.41919E+00_JPRB, 0.41835E+00_JPRB, 0.41638E+00_JPRB ,&
&  0.41703E+00_JPRB, 0.41350E+00_JPRB, 0.41204E+00_JPRB, 0.40783E+00_JPRB, 0.40438E+00_JPRB, 0.39820E+00_JPRB, 0.39287E+00_JPRB ,&
&  0.38835E+00_JPRB, 0.38492E+00_JPRB, 0.38208E+00_JPRB, 0.37967E+00_JPRB, 0.37743E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.38283E+00_JPRB, 0.37630E+00_JPRB, 0.36914E+00_JPRB, 0.36316E+00_JPRB, 0.35772E+00_JPRB, 0.35378E+00_JPRB, 0.35007E+00_JPRB ,&
&  0.34730E+00_JPRB, 0.34468E+00_JPRB, 0.34281E+00_JPRB, 0.34115E+00_JPRB, 0.33999E+00_JPRB, 0.33946E+00_JPRB, 0.33904E+00_JPRB ,&
&  0.33924E+00_JPRB, 0.33923E+00_JPRB, 0.33982E+00_JPRB, 0.33988E+00_JPRB, 0.34032E+00_JPRB, 0.33888E+00_JPRB, 0.33562E+00_JPRB ,&
&  0.33163E+00_JPRB, 0.32709E+00_JPRB, 0.31104E+00_JPRB, 0.30509E+00_JPRB, 0.29178E+00_JPRB, 0.28214E+00_JPRB, 0.27652E+00_JPRB ,&
&  0.26686E+00_JPRB, 0.26173E+00_JPRB, 0.25208E+00_JPRB, 0.24632E+00_JPRB, 0.24455E+00_JPRB, 0.23760E+00_JPRB, 0.23460E+00_JPRB ,&
&  0.22983E+00_JPRB, 0.22156E+00_JPRB, 0.21836E+00_JPRB, 0.21708E+00_JPRB, 0.21993E+00_JPRB, 0.22743E+00_JPRB, 0.24092E+00_JPRB ,&
&  0.26646E+00_JPRB, 0.29487E+00_JPRB, 0.33362E+00_JPRB, 0.34473E+00_JPRB, 0.34654E+00_JPRB, 0.35535E+00_JPRB, 0.36238E+00_JPRB ,&
&  0.36693E+00_JPRB, 0.37700E+00_JPRB, 0.38832E+00_JPRB, 0.39256E+00_JPRB, 0.39567E+00_JPRB, 0.39682E+00_JPRB, 0.39761E+00_JPRB ,&
&  0.39843E+00_JPRB, 0.40250E+00_JPRB, 0.40507E+00_JPRB, 0.40780E+00_JPRB, 0.41057E+00_JPRB, 0.41333E+00_JPRB, 0.41921E+00_JPRB ,&
&  0.42422E+00_JPRB, 0.43533E+00_JPRB, 0.44435E+00_JPRB, 0.45854E+00_JPRB, 0.46422E+00_JPRB, 0.47121E+00_JPRB, 0.47278E+00_JPRB ,&
&  0.47586E+00_JPRB, 0.47742E+00_JPRB, 0.47860E+00_JPRB, 0.47426E+00_JPRB, 0.47314E+00_JPRB, 0.47577E+00_JPRB, 0.47434E+00_JPRB ,&
&  0.47221E+00_JPRB, 0.47052E+00_JPRB, 0.46830E+00_JPRB, 0.46638E+00_JPRB, 0.46489E+00_JPRB, 0.46480E+00_JPRB, 0.46355E+00_JPRB ,&
&  0.46473E+00_JPRB, 0.46216E+00_JPRB, 0.45943E+00_JPRB, 0.45419E+00_JPRB, 0.44747E+00_JPRB, 0.43821E+00_JPRB, 0.42674E+00_JPRB ,&
&  0.41691E+00_JPRB, 0.40935E+00_JPRB, 0.40136E+00_JPRB, 0.39523E+00_JPRB, 0.38886E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.36711E+00_JPRB, 0.35891E+00_JPRB, 0.35220E+00_JPRB, 0.34752E+00_JPRB, 0.34353E+00_JPRB, 0.34106E+00_JPRB, 0.33859E+00_JPRB ,&
&  0.33737E+00_JPRB, 0.33577E+00_JPRB, 0.33510E+00_JPRB, 0.33422E+00_JPRB, 0.33410E+00_JPRB, 0.33433E+00_JPRB, 0.33513E+00_JPRB ,&
&  0.33620E+00_JPRB, 0.33682E+00_JPRB, 0.33766E+00_JPRB, 0.33655E+00_JPRB, 0.33557E+00_JPRB, 0.33367E+00_JPRB, 0.33194E+00_JPRB ,&
&  0.32735E+00_JPRB, 0.32176E+00_JPRB, 0.31018E+00_JPRB, 0.30350E+00_JPRB, 0.29153E+00_JPRB, 0.28287E+00_JPRB, 0.27449E+00_JPRB ,&
&  0.26796E+00_JPRB, 0.25277E+00_JPRB, 0.25645E+00_JPRB, 0.24674E+00_JPRB, 0.23732E+00_JPRB, 0.22785E+00_JPRB, 0.21695E+00_JPRB ,&
&  0.21265E+00_JPRB, 0.20194E+00_JPRB, 0.19807E+00_JPRB, 0.19817E+00_JPRB, 0.19997E+00_JPRB, 0.20797E+00_JPRB, 0.22042E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.30814E+00_JPRB, 0.32960E+00_JPRB, 0.33947E+00_JPRB, 0.33734E+00_JPRB, 0.33861E+00_JPRB, 0.33622E+00_JPRB ,&
&  0.34695E+00_JPRB, 0.36092E+00_JPRB, 0.37590E+00_JPRB, 0.38542E+00_JPRB, 0.39755E+00_JPRB, 0.40429E+00_JPRB, 0.40991E+00_JPRB ,&
&  0.41647E+00_JPRB, 0.42704E+00_JPRB, 0.43662E+00_JPRB, 0.44680E+00_JPRB, 0.45990E+00_JPRB, 0.47318E+00_JPRB, 0.48839E+00_JPRB ,&
&  0.49709E+00_JPRB, 0.51493E+00_JPRB, 0.52068E+00_JPRB, 0.52862E+00_JPRB, 0.52909E+00_JPRB, 0.52735E+00_JPRB, 0.52728E+00_JPRB ,&
&  0.52932E+00_JPRB, 0.53172E+00_JPRB, 0.53341E+00_JPRB, 0.53179E+00_JPRB, 0.53095E+00_JPRB, 0.52325E+00_JPRB, 0.51985E+00_JPRB ,&
&  0.51865E+00_JPRB, 0.51637E+00_JPRB, 0.51342E+00_JPRB, 0.50890E+00_JPRB, 0.50313E+00_JPRB, 0.49653E+00_JPRB, 0.49238E+00_JPRB ,&
&  0.48716E+00_JPRB, 0.48101E+00_JPRB, 0.47393E+00_JPRB, 0.46699E+00_JPRB, 0.45957E+00_JPRB, 0.45186E+00_JPRB, 0.43908E+00_JPRB ,&
&  0.42306E+00_JPRB, 0.40938E+00_JPRB, 0.39651E+00_JPRB, 0.38652E+00_JPRB, 0.37628E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.35459E+00_JPRB, 0.34722E+00_JPRB, 0.34125E+00_JPRB, 0.33819E+00_JPRB, 0.33503E+00_JPRB, 0.33384E+00_JPRB, 0.33176E+00_JPRB ,&
&  0.33070E+00_JPRB, 0.32948E+00_JPRB, 0.32896E+00_JPRB, 0.32835E+00_JPRB, 0.32835E+00_JPRB, 0.32847E+00_JPRB, 0.32933E+00_JPRB ,&
&  0.33036E+00_JPRB, 0.33218E+00_JPRB, 0.33409E+00_JPRB, 0.33425E+00_JPRB, 0.33430E+00_JPRB, 0.33332E+00_JPRB, 0.33184E+00_JPRB ,&
&  0.32797E+00_JPRB, 0.32270E+00_JPRB, 0.31147E+00_JPRB, 0.30311E+00_JPRB, 0.29577E+00_JPRB, 0.28569E+00_JPRB, 0.28346E+00_JPRB ,&
&  0.28040E+00_JPRB, 0.27297E+00_JPRB, 0.26203E+00_JPRB, 0.25755E+00_JPRB, 0.25077E+00_JPRB, 0.22658E+00_JPRB, 0.22654E+00_JPRB ,&
&  0.21943E+00_JPRB, 0.20354E+00_JPRB, 0.19883E+00_JPRB, 0.19555E+00_JPRB, 0.19685E+00_JPRB, 0.20385E+00_JPRB, 0.21496E+00_JPRB ,&
&  0.23898E+00_JPRB, 0.31238E+00_JPRB, 0.33033E+00_JPRB, 0.33739E+00_JPRB, 0.33121E+00_JPRB, 0.33476E+00_JPRB, 0.33560E+00_JPRB ,&
&  0.34364E+00_JPRB, 0.35083E+00_JPRB, 0.36006E+00_JPRB, 0.36754E+00_JPRB, 0.37560E+00_JPRB, 0.37911E+00_JPRB, 0.38742E+00_JPRB ,&
&  0.40299E+00_JPRB, 0.42176E+00_JPRB, 0.43883E+00_JPRB, 0.46126E+00_JPRB, 0.49074E+00_JPRB, 0.54363E+00_JPRB, 0.59908E+00_JPRB ,&
&  0.60367E+00_JPRB, 0.59715E+00_JPRB, 0.59366E+00_JPRB, 0.58752E+00_JPRB, 0.58571E+00_JPRB, 0.58460E+00_JPRB, 0.58744E+00_JPRB ,&
&  0.59480E+00_JPRB, 0.60170E+00_JPRB, 0.59125E+00_JPRB, 0.56912E+00_JPRB, 0.55966E+00_JPRB, 0.54396E+00_JPRB, 0.54053E+00_JPRB ,&
&  0.53590E+00_JPRB, 0.54267E+00_JPRB, 0.53745E+00_JPRB, 0.52860E+00_JPRB, 0.52006E+00_JPRB, 0.51048E+00_JPRB, 0.50372E+00_JPRB ,&
&  0.49584E+00_JPRB, 0.48637E+00_JPRB, 0.47653E+00_JPRB, 0.46859E+00_JPRB, 0.46169E+00_JPRB, 0.45613E+00_JPRB, 0.44750E+00_JPRB ,&
&  0.43032E+00_JPRB, 0.40868E+00_JPRB, 0.39211E+00_JPRB, 0.37652E+00_JPRB, 0.36398E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.34871E+00_JPRB, 0.34431E+00_JPRB, 0.34174E+00_JPRB, 0.34176E+00_JPRB, 0.34114E+00_JPRB, 0.34122E+00_JPRB, 0.33893E+00_JPRB ,&
&  0.33673E+00_JPRB, 0.33439E+00_JPRB, 0.33231E+00_JPRB, 0.33028E+00_JPRB, 0.32897E+00_JPRB, 0.32759E+00_JPRB, 0.32667E+00_JPRB ,&
&  0.32588E+00_JPRB, 0.32177E+00_JPRB, 0.32048E+00_JPRB, 0.32233E+00_JPRB, 0.32488E+00_JPRB, 0.32685E+00_JPRB, 0.32928E+00_JPRB ,&
&  0.32715E+00_JPRB, 0.32236E+00_JPRB, 0.31555E+00_JPRB, 0.30640E+00_JPRB, 0.29920E+00_JPRB, 0.29616E+00_JPRB, 0.29121E+00_JPRB ,&
&  0.28173E+00_JPRB, 0.27378E+00_JPRB, 0.27725E+00_JPRB, 0.27463E+00_JPRB, 0.25909E+00_JPRB, 0.25149E+00_JPRB, 0.24078E+00_JPRB ,&
&  0.22612E+00_JPRB, 0.20651E+00_JPRB, 0.20194E+00_JPRB, 0.20001E+00_JPRB, 0.20446E+00_JPRB, 0.21213E+00_JPRB, 0.22913E+00_JPRB ,&
&  0.25917E+00_JPRB, 0.31743E+00_JPRB, 0.33309E+00_JPRB, 0.34170E+00_JPRB, 0.34622E+00_JPRB, 0.34694E+00_JPRB, 0.34757E+00_JPRB ,&
&  0.35339E+00_JPRB, 0.35853E+00_JPRB, 0.36662E+00_JPRB, 0.37564E+00_JPRB, 0.38743E+00_JPRB, 0.39716E+00_JPRB, 0.40387E+00_JPRB ,&
&  0.42773E+00_JPRB, 0.43851E+00_JPRB, 0.44805E+00_JPRB, 0.48619E+00_JPRB, 0.56014E+00_JPRB, 0.64049E+00_JPRB, 0.67674E+00_JPRB ,&
&  0.67270E+00_JPRB, 0.66494E+00_JPRB, 0.63986E+00_JPRB, 0.62492E+00_JPRB, 0.61814E+00_JPRB, 0.61649E+00_JPRB, 0.61825E+00_JPRB ,&
&  0.62365E+00_JPRB, 0.63623E+00_JPRB, 0.68238E+00_JPRB, 0.64640E+00_JPRB, 0.62440E+00_JPRB, 0.60230E+00_JPRB, 0.57775E+00_JPRB ,&
&  0.55897E+00_JPRB, 0.54852E+00_JPRB, 0.54157E+00_JPRB, 0.53227E+00_JPRB, 0.52297E+00_JPRB, 0.51503E+00_JPRB, 0.51211E+00_JPRB ,&
&  0.50452E+00_JPRB, 0.49389E+00_JPRB, 0.47947E+00_JPRB, 0.47160E+00_JPRB, 0.46438E+00_JPRB, 0.45749E+00_JPRB, 0.45010E+00_JPRB ,&
&  0.43241E+00_JPRB, 0.40821E+00_JPRB, 0.38904E+00_JPRB, 0.37203E+00_JPRB, 0.35827E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.35219E+00_JPRB, 0.35044E+00_JPRB, 0.35213E+00_JPRB, 0.35699E+00_JPRB, 0.36081E+00_JPRB, 0.35590E+00_JPRB, 0.34984E+00_JPRB ,&
&  0.34564E+00_JPRB, 0.34403E+00_JPRB, 0.34329E+00_JPRB, 0.34065E+00_JPRB, 0.33857E+00_JPRB, 0.33638E+00_JPRB, 0.33411E+00_JPRB ,&
&  0.33167E+00_JPRB, 0.32890E+00_JPRB, 0.32560E+00_JPRB, 0.32084E+00_JPRB, 0.31843E+00_JPRB, 0.31960E+00_JPRB, 0.32308E+00_JPRB ,&
&  0.32628E+00_JPRB, 0.32578E+00_JPRB, 0.32289E+00_JPRB, 0.31520E+00_JPRB, 0.30650E+00_JPRB, 0.29952E+00_JPRB, 0.29881E+00_JPRB ,&
&  0.29425E+00_JPRB, 0.28595E+00_JPRB, 0.27697E+00_JPRB, 0.28055E+00_JPRB, 0.28157E+00_JPRB, 0.26557E+00_JPRB, 0.25029E+00_JPRB ,&
&  0.23463E+00_JPRB, 0.22044E+00_JPRB, 0.21398E+00_JPRB, 0.21374E+00_JPRB, 0.22158E+00_JPRB, 0.23644E+00_JPRB, 0.25735E+00_JPRB ,&
&  0.28984E+00_JPRB, 0.31904E+00_JPRB, 0.33303E+00_JPRB, 0.34440E+00_JPRB, 0.34672E+00_JPRB, 0.34579E+00_JPRB, 0.34479E+00_JPRB ,&
&  0.34673E+00_JPRB, 0.35019E+00_JPRB, 0.35613E+00_JPRB, 0.36101E+00_JPRB, 0.37240E+00_JPRB, 0.39296E+00_JPRB, 0.43545E+00_JPRB ,&
&  0.54778E+00_JPRB, 0.55093E+00_JPRB, 0.49600E+00_JPRB, 0.53819E+00_JPRB, 0.61617E+00_JPRB, 0.68816E+00_JPRB, 0.74180E+00_JPRB ,&
&  0.74976E+00_JPRB, 0.71192E+00_JPRB, 0.67905E+00_JPRB, 0.66058E+00_JPRB, 0.64790E+00_JPRB, 0.63786E+00_JPRB, 0.63431E+00_JPRB ,&
&  0.63892E+00_JPRB, 0.65395E+00_JPRB, 0.73562E+00_JPRB, 0.64520E+00_JPRB, 0.59419E+00_JPRB, 0.57461E+00_JPRB, 0.55313E+00_JPRB ,&
&  0.53501E+00_JPRB, 0.52610E+00_JPRB, 0.52699E+00_JPRB, 0.53093E+00_JPRB, 0.50976E+00_JPRB, 0.49077E+00_JPRB, 0.47685E+00_JPRB ,&
&  0.47172E+00_JPRB, 0.46360E+00_JPRB, 0.45795E+00_JPRB, 0.45931E+00_JPRB, 0.45623E+00_JPRB, 0.44492E+00_JPRB, 0.43805E+00_JPRB ,&
&  0.42608E+00_JPRB, 0.40493E+00_JPRB, 0.39045E+00_JPRB, 0.37624E+00_JPRB, 0.36135E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.35041E+00_JPRB, 0.35915E+00_JPRB, 0.37146E+00_JPRB, 0.38004E+00_JPRB, 0.38249E+00_JPRB, 0.36606E+00_JPRB, 0.34934E+00_JPRB ,&
&  0.34009E+00_JPRB, 0.33645E+00_JPRB, 0.33730E+00_JPRB, 0.33774E+00_JPRB, 0.33764E+00_JPRB, 0.33777E+00_JPRB, 0.33735E+00_JPRB ,&
&  0.33416E+00_JPRB, 0.32959E+00_JPRB, 0.32525E+00_JPRB, 0.32106E+00_JPRB, 0.31785E+00_JPRB, 0.31719E+00_JPRB, 0.31857E+00_JPRB ,&
&  0.32131E+00_JPRB, 0.32269E+00_JPRB, 0.32212E+00_JPRB, 0.31759E+00_JPRB, 0.31393E+00_JPRB, 0.31081E+00_JPRB, 0.30498E+00_JPRB ,&
&  0.29498E+00_JPRB, 0.29359E+00_JPRB, 0.29078E+00_JPRB, 0.28736E+00_JPRB, 0.28877E+00_JPRB, 0.27836E+00_JPRB, 0.26236E+00_JPRB ,&
&  0.24622E+00_JPRB, 0.23423E+00_JPRB, 0.23223E+00_JPRB, 0.23982E+00_JPRB, 0.25775E+00_JPRB, 0.28372E+00_JPRB, 0.29746E+00_JPRB ,&
&  0.30991E+00_JPRB, 0.32107E+00_JPRB, 0.32872E+00_JPRB, 0.33591E+00_JPRB, 0.34123E+00_JPRB, 0.34367E+00_JPRB, 0.34417E+00_JPRB ,&
&  0.34335E+00_JPRB, 0.34345E+00_JPRB, 0.34532E+00_JPRB, 0.33098E+00_JPRB, 0.34111E+00_JPRB, 0.40446E+00_JPRB, 0.46114E+00_JPRB ,&
&  0.53194E+00_JPRB, 0.54707E+00_JPRB, 0.56093E+00_JPRB, 0.61383E+00_JPRB, 0.66740E+00_JPRB, 0.71416E+00_JPRB, 0.75379E+00_JPRB ,&
&  0.76642E+00_JPRB, 0.72529E+00_JPRB, 0.68452E+00_JPRB, 0.67015E+00_JPRB, 0.66394E+00_JPRB, 0.65623E+00_JPRB, 0.63186E+00_JPRB ,&
&  0.61384E+00_JPRB, 0.58341E+00_JPRB, 0.54474E+00_JPRB, 0.51081E+00_JPRB, 0.50561E+00_JPRB, 0.51966E+00_JPRB, 0.54056E+00_JPRB ,&
&  0.54159E+00_JPRB, 0.53941E+00_JPRB, 0.53867E+00_JPRB, 0.54336E+00_JPRB, 0.49792E+00_JPRB, 0.46831E+00_JPRB, 0.44781E+00_JPRB ,&
&  0.43835E+00_JPRB, 0.43232E+00_JPRB, 0.42815E+00_JPRB, 0.43559E+00_JPRB, 0.43552E+00_JPRB, 0.43021E+00_JPRB, 0.41387E+00_JPRB ,&
&  0.39231E+00_JPRB, 0.37500E+00_JPRB, 0.36253E+00_JPRB, 0.35392E+00_JPRB, 0.35054E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.38961E+00_JPRB, 0.39679E+00_JPRB, 0.40044E+00_JPRB, 0.40317E+00_JPRB, 0.39869E+00_JPRB, 0.39063E+00_JPRB, 0.38187E+00_JPRB ,&
&  0.38138E+00_JPRB, 0.37226E+00_JPRB, 0.36376E+00_JPRB, 0.35135E+00_JPRB, 0.34068E+00_JPRB, 0.33158E+00_JPRB, 0.32998E+00_JPRB ,&
&  0.33200E+00_JPRB, 0.33425E+00_JPRB, 0.33263E+00_JPRB, 0.32728E+00_JPRB, 0.32183E+00_JPRB, 0.31880E+00_JPRB, 0.31889E+00_JPRB ,&
&  0.32056E+00_JPRB, 0.32292E+00_JPRB, 0.32366E+00_JPRB, 0.32224E+00_JPRB, 0.32046E+00_JPRB, 0.31797E+00_JPRB, 0.31442E+00_JPRB ,&
&  0.30757E+00_JPRB, 0.30315E+00_JPRB, 0.30062E+00_JPRB, 0.29937E+00_JPRB, 0.30113E+00_JPRB, 0.29117E+00_JPRB, 0.27546E+00_JPRB ,&
&  0.25976E+00_JPRB, 0.25464E+00_JPRB, 0.26403E+00_JPRB, 0.29221E+00_JPRB, 0.30983E+00_JPRB, 0.32424E+00_JPRB, 0.33562E+00_JPRB ,&
&  0.33473E+00_JPRB, 0.33264E+00_JPRB, 0.33779E+00_JPRB, 0.35214E+00_JPRB, 0.35493E+00_JPRB, 0.35573E+00_JPRB, 0.35365E+00_JPRB ,&
&  0.35031E+00_JPRB, 0.34986E+00_JPRB, 0.35028E+00_JPRB, 0.34172E+00_JPRB, 0.39657E+00_JPRB, 0.50111E+00_JPRB, 0.52168E+00_JPRB ,&
&  0.58040E+00_JPRB, 0.61989E+00_JPRB, 0.67545E+00_JPRB, 0.69016E+00_JPRB, 0.67721E+00_JPRB, 0.69938E+00_JPRB, 0.75804E+00_JPRB ,&
&  0.77137E+00_JPRB, 0.74978E+00_JPRB, 0.72036E+00_JPRB, 0.70940E+00_JPRB, 0.67910E+00_JPRB, 0.64790E+00_JPRB, 0.61990E+00_JPRB ,&
&  0.61341E+00_JPRB, 0.59784E+00_JPRB, 0.56811E+00_JPRB, 0.53389E+00_JPRB, 0.52143E+00_JPRB, 0.52031E+00_JPRB, 0.53027E+00_JPRB ,&
&  0.53429E+00_JPRB, 0.54279E+00_JPRB, 0.54825E+00_JPRB, 0.55357E+00_JPRB, 0.51309E+00_JPRB, 0.46572E+00_JPRB, 0.43530E+00_JPRB ,&
&  0.41574E+00_JPRB, 0.40195E+00_JPRB, 0.38956E+00_JPRB, 0.39500E+00_JPRB, 0.40461E+00_JPRB, 0.40487E+00_JPRB, 0.38621E+00_JPRB ,&
&  0.37139E+00_JPRB, 0.36391E+00_JPRB, 0.36113E+00_JPRB, 0.37085E+00_JPRB, 0.38177E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.41501E+00_JPRB, 0.42370E+00_JPRB, 0.42344E+00_JPRB, 0.42096E+00_JPRB, 0.42185E+00_JPRB, 0.41491E+00_JPRB, 0.39988E+00_JPRB ,&
&  0.39734E+00_JPRB, 0.37674E+00_JPRB, 0.36709E+00_JPRB, 0.35799E+00_JPRB, 0.35107E+00_JPRB, 0.34248E+00_JPRB, 0.33029E+00_JPRB ,&
&  0.32669E+00_JPRB, 0.32998E+00_JPRB, 0.33585E+00_JPRB, 0.33557E+00_JPRB, 0.33177E+00_JPRB, 0.32717E+00_JPRB, 0.32502E+00_JPRB ,&
&  0.32522E+00_JPRB, 0.32731E+00_JPRB, 0.33047E+00_JPRB, 0.32891E+00_JPRB, 0.32516E+00_JPRB, 0.32201E+00_JPRB, 0.32198E+00_JPRB ,&
&  0.31841E+00_JPRB, 0.31257E+00_JPRB, 0.31253E+00_JPRB, 0.31221E+00_JPRB, 0.31437E+00_JPRB, 0.31539E+00_JPRB, 0.30304E+00_JPRB ,&
&  0.28529E+00_JPRB, 0.29124E+00_JPRB, 0.31989E+00_JPRB, 0.33912E+00_JPRB, 0.35653E+00_JPRB, 0.36427E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.37406E+00_JPRB, 0.37917E+00_JPRB, 0.38167E+00_JPRB, 0.38423E+00_JPRB, 0.38458E+00_JPRB, 0.38278E+00_JPRB, 0.37907E+00_JPRB ,&
&  0.37652E+00_JPRB, 0.35010E+00_JPRB, 0.34388E+00_JPRB, 0.36553E+00_JPRB, 0.50779E+00_JPRB, 0.52770E+00_JPRB, 0.54724E+00_JPRB ,&
&  0.61079E+00_JPRB, 0.64981E+00_JPRB, 0.70478E+00_JPRB, 0.73962E+00_JPRB, 0.74109E+00_JPRB, 0.74585E+00_JPRB, 0.79277E+00_JPRB ,&
&  0.82537E+00_JPRB, 0.81842E+00_JPRB, 0.79383E+00_JPRB, 0.75908E+00_JPRB, 0.70710E+00_JPRB, 0.68453E+00_JPRB, 0.66986E+00_JPRB ,&
&  0.67293E+00_JPRB, 0.66831E+00_JPRB, 0.63647E+00_JPRB, 0.59729E+00_JPRB, 0.57012E+00_JPRB, 0.57044E+00_JPRB, 0.57473E+00_JPRB ,&
&  0.57592E+00_JPRB, 0.58407E+00_JPRB, 0.58004E+00_JPRB, 0.57002E+00_JPRB, 0.54292E+00_JPRB, 0.51558E+00_JPRB, 0.49958E+00_JPRB ,&
&  0.46132E+00_JPRB, 0.42346E+00_JPRB, 0.39323E+00_JPRB, 0.39135E+00_JPRB, 0.40064E+00_JPRB, 0.40027E+00_JPRB, 0.38822E+00_JPRB ,&
&  0.38511E+00_JPRB, 0.38111E+00_JPRB, 0.37947E+00_JPRB, 0.39126E+00_JPRB, 0.40141E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.42246E+00_JPRB, 0.42979E+00_JPRB, 0.42974E+00_JPRB, 0.42735E+00_JPRB, 0.43364E+00_JPRB, 0.42793E+00_JPRB, 0.42781E+00_JPRB ,&
&  0.44491E+00_JPRB, 0.43479E+00_JPRB, 0.45081E+00_JPRB, 0.39657E+00_JPRB, 0.36559E+00_JPRB, 0.35420E+00_JPRB, 0.34132E+00_JPRB ,&
&  0.33745E+00_JPRB, 0.34153E+00_JPRB, 0.34340E+00_JPRB, 0.34281E+00_JPRB, 0.34178E+00_JPRB, 0.33540E+00_JPRB, 0.33236E+00_JPRB ,&
&  0.33293E+00_JPRB, 0.33605E+00_JPRB, 0.34000E+00_JPRB, 0.33811E+00_JPRB, 0.33475E+00_JPRB, 0.33215E+00_JPRB, 0.32949E+00_JPRB ,&
&  0.32514E+00_JPRB, 0.32508E+00_JPRB, 0.32616E+00_JPRB, 0.32516E+00_JPRB, 0.32635E+00_JPRB, 0.32970E+00_JPRB, 0.32990E+00_JPRB ,&
&  0.33647E+00_JPRB, 0.33804E+00_JPRB, 0.36602E+00_JPRB, 0.38542E+00_JPRB, 0.39652E+00_JPRB, 0.39980E+00_JPRB, 0.40284E+00_JPRB ,&
&  0.40959E+00_JPRB, 0.41866E+00_JPRB, 0.42855E+00_JPRB, 0.43625E+00_JPRB, 0.43682E+00_JPRB, 0.43452E+00_JPRB, 0.42896E+00_JPRB ,&
&  0.42488E+00_JPRB, 0.37518E+00_JPRB, 0.42144E+00_JPRB, 0.44845E+00_JPRB, 0.55374E+00_JPRB, 0.67008E+00_JPRB, 0.66274E+00_JPRB ,&
&  0.80167E+00_JPRB, 0.77160E+00_JPRB, 0.79827E+00_JPRB, 0.79863E+00_JPRB, 0.79825E+00_JPRB, 0.82059E+00_JPRB, 0.86552E+00_JPRB ,&
&  0.94212E+00_JPRB, 0.90427E+00_JPRB, 0.90517E+00_JPRB, 0.83212E+00_JPRB, 0.78330E+00_JPRB, 0.77819E+00_JPRB, 0.76785E+00_JPRB ,&
&  0.76856E+00_JPRB, 0.76747E+00_JPRB, 0.75194E+00_JPRB, 0.67317E+00_JPRB, 0.65219E+00_JPRB, 0.64376E+00_JPRB, 0.60722E+00_JPRB ,&
&  0.61728E+00_JPRB, 0.61183E+00_JPRB, 0.58289E+00_JPRB, 0.60395E+00_JPRB, 0.56010E+00_JPRB, 0.53326E+00_JPRB, 0.52823E+00_JPRB ,&
&  0.51262E+00_JPRB, 0.45582E+00_JPRB, 0.43088E+00_JPRB, 0.42387E+00_JPRB, 0.41333E+00_JPRB, 0.40384E+00_JPRB, 0.40423E+00_JPRB ,&
&  0.39320E+00_JPRB, 0.39485E+00_JPRB, 0.40896E+00_JPRB, 0.41121E+00_JPRB, 0.41375E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.43179E+00_JPRB, 0.43230E+00_JPRB, 0.43248E+00_JPRB, 0.43386E+00_JPRB, 0.44903E+00_JPRB, 0.45584E+00_JPRB, 0.45592E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.46891E+00_JPRB, 0.46785E+00_JPRB, 0.46163E+00_JPRB, 0.45209E+00_JPRB, 0.41425E+00_JPRB, 0.35957E+00_JPRB ,&
&  0.34102E+00_JPRB, 0.34281E+00_JPRB, 0.34920E+00_JPRB, 0.35028E+00_JPRB, 0.34651E+00_JPRB, 0.34118E+00_JPRB, 0.33845E+00_JPRB ,&
&  0.34100E+00_JPRB, 0.34455E+00_JPRB, 0.34889E+00_JPRB, 0.34993E+00_JPRB, 0.34748E+00_JPRB, 0.34615E+00_JPRB, 0.34430E+00_JPRB ,&
&  0.33819E+00_JPRB, 0.33770E+00_JPRB, 0.33808E+00_JPRB, 0.33485E+00_JPRB, 0.34702E+00_JPRB, 0.35529E+00_JPRB, 0.35774E+00_JPRB ,&
&  0.37499E+00_JPRB, 0.39879E+00_JPRB, 0.41614E+00_JPRB, 0.43253E+00_JPRB, 0.43673E+00_JPRB, 0.43959E+00_JPRB, 0.44915E+00_JPRB ,&
&  0.46548E+00_JPRB, 0.48099E+00_JPRB, 0.49556E+00_JPRB, 0.50685E+00_JPRB, 0.51154E+00_JPRB, 0.49847E+00_JPRB, 0.51262E+00_JPRB ,&
&  0.51377E+00_JPRB, 0.53790E+00_JPRB, 0.59503E+00_JPRB, 0.59510E+00_JPRB, 0.68892E+00_JPRB, 0.76034E+00_JPRB, 0.75160E+00_JPRB ,&
&  0.81164E+00_JPRB, 0.78837E+00_JPRB, 0.88110E+00_JPRB, 0.94587E+00_JPRB, 0.10058E+01_JPRB, 0.10315E+01_JPRB, 0.10696E+01_JPRB ,&
&  0.11649E+01_JPRB, 0.12326E+01_JPRB, 0.10719E+01_JPRB, 0.99417E+00_JPRB, 0.93052E+00_JPRB, 0.92774E+00_JPRB, 0.88732E+00_JPRB ,&
&  0.89112E+00_JPRB, 0.91153E+00_JPRB, 0.89344E+00_JPRB, 0.86504E+00_JPRB, 0.80838E+00_JPRB, 0.75022E+00_JPRB, 0.69658E+00_JPRB ,&
&  0.66433E+00_JPRB, 0.60179E+00_JPRB, 0.57004E+00_JPRB, 0.57101E+00_JPRB, 0.56818E+00_JPRB, 0.52224E+00_JPRB, 0.53232E+00_JPRB ,&
&  0.54551E+00_JPRB, 0.50660E+00_JPRB, 0.47360E+00_JPRB, 0.44710E+00_JPRB, 0.43803E+00_JPRB, 0.42539E+00_JPRB, 0.40238E+00_JPRB ,&
&  0.38680E+00_JPRB, 0.40697E+00_JPRB, 0.42084E+00_JPRB, 0.42472E+00_JPRB, 0.42867E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.44585E+00_JPRB, 0.44092E+00_JPRB, 0.43572E+00_JPRB, 0.44156E+00_JPRB, 0.45911E+00_JPRB, 0.46603E+00_JPRB, 0.47833E+00_JPRB ,&
&  0.48585E+00_JPRB, 0.48649E+00_JPRB, 0.48660E+00_JPRB, 0.47558E+00_JPRB, 0.46858E+00_JPRB, 0.46266E+00_JPRB, 0.42969E+00_JPRB ,&
&  0.38645E+00_JPRB, 0.37290E+00_JPRB, 0.36072E+00_JPRB, 0.35810E+00_JPRB, 0.35700E+00_JPRB, 0.35119E+00_JPRB, 0.34770E+00_JPRB ,&
&  0.35147E+00_JPRB, 0.35306E+00_JPRB, 0.35511E+00_JPRB, 0.35666E+00_JPRB, 0.35707E+00_JPRB, 0.35907E+00_JPRB, 0.35853E+00_JPRB ,&
&  0.35152E+00_JPRB, 0.35160E+00_JPRB, 0.35491E+00_JPRB, 0.35341E+00_JPRB, 0.37068E+00_JPRB, 0.39000E+00_JPRB, 0.40507E+00_JPRB ,&
&  0.42975E+00_JPRB, 0.45185E+00_JPRB, 0.46569E+00_JPRB, 0.47662E+00_JPRB, 0.48003E+00_JPRB, 0.49304E+00_JPRB, 0.51516E+00_JPRB ,&
&  0.54273E+00_JPRB, 0.56256E+00_JPRB, 0.57838E+00_JPRB, 0.59518E+00_JPRB, 0.59061E+00_JPRB, 0.59862E+00_JPRB, 0.63913E+00_JPRB ,&
&  0.69160E+00_JPRB, 0.76225E+00_JPRB, 0.78855E+00_JPRB, 0.83005E+00_JPRB, 0.91516E+00_JPRB, 0.93697E+00_JPRB, 0.88943E+00_JPRB ,&
&  0.91896E+00_JPRB, 0.88331E+00_JPRB, 0.11735E+01_JPRB, 0.12017E+01_JPRB, 0.11833E+01_JPRB, 0.12360E+01_JPRB, 0.12153E+01_JPRB ,&
&  0.12235E+01_JPRB, 0.11698E+01_JPRB, 0.11054E+01_JPRB, 0.10737E+01_JPRB, 0.10516E+01_JPRB, 0.11081E+01_JPRB, 0.10559E+01_JPRB ,&
&  0.10710E+01_JPRB, 0.10645E+01_JPRB, 0.91706E+00_JPRB, 0.80155E+00_JPRB, 0.69162E+00_JPRB, 0.72009E+00_JPRB, 0.69056E+00_JPRB ,&
&  0.65535E+00_JPRB, 0.56779E+00_JPRB, 0.59040E+00_JPRB, 0.59661E+00_JPRB, 0.60395E+00_JPRB, 0.62210E+00_JPRB, 0.61887E+00_JPRB ,&
&  0.61499E+00_JPRB, 0.61360E+00_JPRB, 0.54444E+00_JPRB, 0.49656E+00_JPRB, 0.47568E+00_JPRB, 0.45234E+00_JPRB, 0.41664E+00_JPRB ,&
&  0.40741E+00_JPRB, 0.42976E+00_JPRB, 0.44539E+00_JPRB, 0.45079E+00_JPRB, 0.45040E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.46643E+00_JPRB, 0.45837E+00_JPRB, 0.44883E+00_JPRB, 0.45739E+00_JPRB, 0.47162E+00_JPRB, 0.47661E+00_JPRB, 0.48926E+00_JPRB ,&
&  0.50207E+00_JPRB, 0.50350E+00_JPRB, 0.49375E+00_JPRB, 0.48453E+00_JPRB, 0.47886E+00_JPRB, 0.47472E+00_JPRB, 0.47353E+00_JPRB ,&
&  0.42561E+00_JPRB, 0.39714E+00_JPRB, 0.36934E+00_JPRB, 0.36483E+00_JPRB, 0.38301E+00_JPRB, 0.37254E+00_JPRB, 0.36990E+00_JPRB ,&
&  0.36639E+00_JPRB, 0.36809E+00_JPRB, 0.36459E+00_JPRB, 0.36309E+00_JPRB, 0.36368E+00_JPRB, 0.36726E+00_JPRB, 0.37169E+00_JPRB ,&
&  0.36875E+00_JPRB, 0.37612E+00_JPRB, 0.38739E+00_JPRB, 0.38947E+00_JPRB, 0.41063E+00_JPRB, 0.43176E+00_JPRB, 0.46136E+00_JPRB ,&
&  0.49108E+00_JPRB, 0.50306E+00_JPRB, 0.51002E+00_JPRB, 0.52397E+00_JPRB, 0.54059E+00_JPRB, 0.57207E+00_JPRB, 0.61051E+00_JPRB ,&
&  0.65111E+00_JPRB, 0.68218E+00_JPRB, 0.70253E+00_JPRB, 0.71317E+00_JPRB, 0.70089E+00_JPRB, 0.73106E+00_JPRB, 0.79867E+00_JPRB ,&
&  0.91229E+00_JPRB, 0.95257E+00_JPRB, 0.95594E+00_JPRB, 0.11766E+01_JPRB, 0.13194E+01_JPRB, 0.13003E+01_JPRB, 0.11609E+01_JPRB ,&
&  0.10820E+01_JPRB, 0.10076E+01_JPRB, 0.11162E+01_JPRB, 0.11946E+01_JPRB, 0.11699E+01_JPRB, 0.12752E+01_JPRB, 0.12305E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.10492E+01_JPRB, 0.10107E+01_JPRB, 0.10630E+01_JPRB, 0.10630E+01_JPRB, 0.11406E+01_JPRB, 0.11395E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.98641E+00_JPRB, 0.74156E+00_JPRB, 0.68157E+00_JPRB, 0.54885E+00_JPRB, 0.55711E+00_JPRB, 0.67418E+00_JPRB ,&
&  0.67169E+00_JPRB, 0.67137E+00_JPRB, 0.66283E+00_JPRB, 0.66793E+00_JPRB, 0.67313E+00_JPRB, 0.77684E+00_JPRB, 0.80506E+00_JPRB ,&
&  0.70857E+00_JPRB, 0.70278E+00_JPRB, 0.66577E+00_JPRB, 0.61856E+00_JPRB, 0.57404E+00_JPRB, 0.52044E+00_JPRB, 0.48054E+00_JPRB ,&
&  0.47127E+00_JPRB, 0.47114E+00_JPRB, 0.47857E+00_JPRB, 0.47656E+00_JPRB, 0.47001E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.49759E+00_JPRB, 0.49011E+00_JPRB, 0.47638E+00_JPRB, 0.47965E+00_JPRB, 0.48669E+00_JPRB, 0.49366E+00_JPRB, 0.50721E+00_JPRB ,&
&  0.52047E+00_JPRB, 0.51512E+00_JPRB, 0.50101E+00_JPRB, 0.49562E+00_JPRB, 0.49137E+00_JPRB, 0.48400E+00_JPRB, 0.48236E+00_JPRB ,&
&  0.48685E+00_JPRB, 0.45672E+00_JPRB, 0.41408E+00_JPRB, 0.38098E+00_JPRB, 0.38986E+00_JPRB, 0.38828E+00_JPRB, 0.39571E+00_JPRB ,&
&  0.39105E+00_JPRB, 0.40044E+00_JPRB, 0.39149E+00_JPRB, 0.38673E+00_JPRB, 0.38440E+00_JPRB, 0.38927E+00_JPRB, 0.40118E+00_JPRB ,&
&  0.41670E+00_JPRB, 0.44403E+00_JPRB, 0.47169E+00_JPRB, 0.47877E+00_JPRB, 0.50358E+00_JPRB, 0.49597E+00_JPRB, 0.52848E+00_JPRB ,&
&  0.55502E+00_JPRB, 0.56109E+00_JPRB, 0.56777E+00_JPRB, 0.59198E+00_JPRB, 0.63107E+00_JPRB, 0.67997E+00_JPRB, 0.74182E+00_JPRB ,&
&  0.80123E+00_JPRB, 0.82612E+00_JPRB, 0.80682E+00_JPRB, 0.79688E+00_JPRB, 0.82838E+00_JPRB, 0.89980E+00_JPRB, 0.90757E+00_JPRB ,&
&  0.96493E+00_JPRB, 0.97911E+00_JPRB, 0.93344E+00_JPRB, 0.12358E+01_JPRB, 0.15009E+01_JPRB, 0.15276E+01_JPRB, 0.14226E+01_JPRB ,&
&  0.13286E+01_JPRB, 0.12405E+01_JPRB, 0.13085E+01_JPRB, 0.13284E+01_JPRB, 0.13166E+01_JPRB, 0.13029E+01_JPRB, 0.12194E+01_JPRB ,&
&  0.10833E+01_JPRB, 0.10165E+01_JPRB, 0.10348E+01_JPRB, 0.10529E+01_JPRB, 0.10831E+01_JPRB, 0.10560E+01_JPRB, 0.10933E+01_JPRB ,&
&  0.11423E+01_JPRB, 0.66769E+00_JPRB, 0.54630E+00_JPRB, 0.62093E+00_JPRB, 0.53729E+00_JPRB, 0.54807E+00_JPRB, 0.61397E+00_JPRB ,&
&  0.60682E+00_JPRB, 0.61134E+00_JPRB, 0.67887E+00_JPRB, 0.72942E+00_JPRB, 0.81120E+00_JPRB, 0.95315E+00_JPRB, 0.94677E+00_JPRB ,&
&  0.89531E+00_JPRB, 0.82310E+00_JPRB, 0.82992E+00_JPRB, 0.79530E+00_JPRB, 0.72424E+00_JPRB, 0.63260E+00_JPRB, 0.56501E+00_JPRB ,&
&  0.53378E+00_JPRB, 0.52613E+00_JPRB, 0.52287E+00_JPRB, 0.51362E+00_JPRB, 0.50059E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.54673E+00_JPRB, 0.53641E+00_JPRB, 0.50996E+00_JPRB, 0.50336E+00_JPRB, 0.51294E+00_JPRB, 0.52874E+00_JPRB, 0.54154E+00_JPRB ,&
&  0.54217E+00_JPRB, 0.53010E+00_JPRB, 0.51920E+00_JPRB, 0.51487E+00_JPRB, 0.50616E+00_JPRB, 0.49208E+00_JPRB, 0.48520E+00_JPRB ,&
&  0.48247E+00_JPRB, 0.47818E+00_JPRB, 0.43619E+00_JPRB, 0.41579E+00_JPRB, 0.39314E+00_JPRB, 0.39956E+00_JPRB, 0.40952E+00_JPRB ,&
&  0.42226E+00_JPRB, 0.41979E+00_JPRB, 0.41348E+00_JPRB, 0.41484E+00_JPRB, 0.41636E+00_JPRB, 0.42643E+00_JPRB, 0.46944E+00_JPRB ,&
&  0.51976E+00_JPRB, 0.57215E+00_JPRB, 0.58011E+00_JPRB, 0.60277E+00_JPRB, 0.59489E+00_JPRB, 0.57397E+00_JPRB, 0.62137E+00_JPRB ,&
&  0.63787E+00_JPRB, 0.63984E+00_JPRB, 0.64600E+00_JPRB, 0.67898E+00_JPRB, 0.72725E+00_JPRB, 0.76239E+00_JPRB, 0.81404E+00_JPRB ,&
&  0.84668E+00_JPRB, 0.85594E+00_JPRB, 0.84417E+00_JPRB, 0.81934E+00_JPRB, 0.86596E+00_JPRB, 0.89403E+00_JPRB, 0.84251E+00_JPRB ,&
&  0.85736E+00_JPRB, 0.76772E+00_JPRB, 0.64021E+00_JPRB, 0.94497E+00_JPRB, 0.14997E+01_JPRB, 0.15932E+01_JPRB, 0.13521E+01_JPRB ,&
&  0.13837E+01_JPRB, 0.13701E+01_JPRB, 0.14748E+01_JPRB, 0.15586E+01_JPRB, 0.15069E+01_JPRB, 0.13781E+01_JPRB, 0.12528E+01_JPRB ,&
&  0.11275E+01_JPRB, 0.10917E+01_JPRB, 0.11353E+01_JPRB, 0.13300E+01_JPRB, 0.13641E+01_JPRB, 0.13548E+01_JPRB, 0.13030E+01_JPRB ,&
&  0.10898E+01_JPRB, 0.91451E+00_JPRB, 0.65540E+00_JPRB, 0.66846E+00_JPRB, 0.53896E+00_JPRB, 0.54130E+00_JPRB, 0.60253E+00_JPRB ,&
&  0.59111E+00_JPRB, 0.63164E+00_JPRB, 0.70534E+00_JPRB, 0.77425E+00_JPRB, 0.10179E+01_JPRB, 0.11657E+01_JPRB, 0.12071E+01_JPRB ,&
&  0.11559E+01_JPRB, 0.10142E+01_JPRB, 0.10254E+01_JPRB, 0.94566E+00_JPRB, 0.88037E+00_JPRB, 0.77420E+00_JPRB, 0.68223E+00_JPRB ,&
&  0.62691E+00_JPRB, 0.60574E+00_JPRB, 0.59117E+00_JPRB, 0.57352E+00_JPRB, 0.55333E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.57410E+00_JPRB, 0.54905E+00_JPRB, 0.50808E+00_JPRB, 0.50407E+00_JPRB, 0.51906E+00_JPRB, 0.54927E+00_JPRB, 0.56738E+00_JPRB ,&
&  0.56316E+00_JPRB, 0.55006E+00_JPRB, 0.53700E+00_JPRB, 0.52915E+00_JPRB, 0.51408E+00_JPRB, 0.49360E+00_JPRB, 0.48522E+00_JPRB ,&
&  0.47840E+00_JPRB, 0.46364E+00_JPRB, 0.42766E+00_JPRB, 0.40682E+00_JPRB, 0.38907E+00_JPRB, 0.38985E+00_JPRB, 0.40316E+00_JPRB ,&
&  0.41638E+00_JPRB, 0.42678E+00_JPRB, 0.44082E+00_JPRB, 0.45139E+00_JPRB, 0.45875E+00_JPRB, 0.49773E+00_JPRB, 0.57901E+00_JPRB ,&
&  0.72683E+00_JPRB, 0.68481E+00_JPRB, 0.70170E+00_JPRB, 0.67448E+00_JPRB, 0.67096E+00_JPRB, 0.67672E+00_JPRB, 0.67200E+00_JPRB ,&
&  0.64728E+00_JPRB, 0.63571E+00_JPRB, 0.65024E+00_JPRB, 0.68319E+00_JPRB, 0.75384E+00_JPRB, 0.81702E+00_JPRB, 0.90384E+00_JPRB ,&
&  0.93878E+00_JPRB, 0.91432E+00_JPRB, 0.85848E+00_JPRB, 0.84897E+00_JPRB, 0.90567E+00_JPRB, 0.90111E+00_JPRB, 0.86601E+00_JPRB ,&
&  0.73765E+00_JPRB, 0.65318E+00_JPRB, 0.74488E+00_JPRB, 0.95906E+00_JPRB, 0.13288E+01_JPRB, 0.15978E+01_JPRB, 0.15101E+01_JPRB ,&
&  0.15122E+01_JPRB, 0.14877E+01_JPRB, 0.15071E+01_JPRB, 0.12344E+01_JPRB, 0.13556E+01_JPRB, 0.12857E+01_JPRB, 0.12096E+01_JPRB ,&
&  0.11528E+01_JPRB, 0.11774E+01_JPRB, 0.13109E+01_JPRB, 0.15482E+01_JPRB, 0.17039E+01_JPRB, 0.17751E+01_JPRB, 0.13379E+01_JPRB ,&
&  0.10249E+01_JPRB, 0.10923E+01_JPRB, 0.98214E+00_JPRB, 0.81760E+00_JPRB, 0.69198E+00_JPRB, 0.70329E+00_JPRB, 0.69602E+00_JPRB ,&
&  0.69517E+00_JPRB, 0.70878E+00_JPRB, 0.78901E+00_JPRB, 0.97833E+00_JPRB, 0.13069E+01_JPRB, 0.15388E+01_JPRB, 0.15180E+01_JPRB ,&
&  0.13465E+01_JPRB, 0.13508E+01_JPRB, 0.12292E+01_JPRB, 0.10728E+01_JPRB, 0.98892E+00_JPRB, 0.90024E+00_JPRB, 0.80947E+00_JPRB ,&
&  0.73696E+00_JPRB, 0.70175E+00_JPRB, 0.67548E+00_JPRB, 0.64221E+00_JPRB, 0.59940E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.60122E+00_JPRB, 0.57394E+00_JPRB, 0.53464E+00_JPRB, 0.53205E+00_JPRB, 0.55707E+00_JPRB, 0.58723E+00_JPRB, 0.59406E+00_JPRB ,&
&  0.57228E+00_JPRB, 0.55631E+00_JPRB, 0.54179E+00_JPRB, 0.54031E+00_JPRB, 0.53087E+00_JPRB, 0.50616E+00_JPRB, 0.48965E+00_JPRB ,&
&  0.48616E+00_JPRB, 0.45592E+00_JPRB, 0.41982E+00_JPRB, 0.41702E+00_JPRB, 0.41088E+00_JPRB, 0.38876E+00_JPRB, 0.39515E+00_JPRB ,&
&  0.41138E+00_JPRB, 0.43187E+00_JPRB, 0.45480E+00_JPRB, 0.49148E+00_JPRB, 0.54627E+00_JPRB, 0.63610E+00_JPRB, 0.74907E+00_JPRB ,&
&  0.76866E+00_JPRB, 0.77115E+00_JPRB, 0.79765E+00_JPRB, 0.77046E+00_JPRB, 0.71061E+00_JPRB, 0.67911E+00_JPRB, 0.66628E+00_JPRB ,&
&  0.64591E+00_JPRB, 0.63842E+00_JPRB, 0.67754E+00_JPRB, 0.75481E+00_JPRB, 0.86252E+00_JPRB, 0.93246E+00_JPRB, 0.98687E+00_JPRB ,&
&  0.95319E+00_JPRB, 0.94364E+00_JPRB, 0.93076E+00_JPRB, 0.10088E+01_JPRB, 0.78035E+00_JPRB, 0.74387E+00_JPRB, 0.71687E+00_JPRB ,&
&  0.80852E+00_JPRB, 0.77166E+00_JPRB, 0.77876E+00_JPRB, 0.99516E+00_JPRB, 0.10071E+01_JPRB, 0.12295E+01_JPRB, 0.14793E+01_JPRB ,&
&  0.16096E+01_JPRB, 0.16247E+01_JPRB, 0.16054E+01_JPRB, 0.10011E+01_JPRB, 0.93247E+00_JPRB, 0.12128E+01_JPRB, 0.12850E+01_JPRB ,&
&  0.12921E+01_JPRB, 0.14421E+01_JPRB, 0.16465E+01_JPRB, 0.19686E+01_JPRB, 0.15193E+01_JPRB, 0.12924E+01_JPRB, 0.99718E+00_JPRB ,&
&  0.97536E+00_JPRB, 0.90396E+00_JPRB, 0.10577E+01_JPRB, 0.96455E+00_JPRB, 0.96223E+00_JPRB, 0.92231E+00_JPRB, 0.84962E+00_JPRB ,&
&  0.85234E+00_JPRB, 0.88993E+00_JPRB, 0.10208E+01_JPRB, 0.13559E+01_JPRB, 0.17485E+01_JPRB, 0.17662E+01_JPRB, 0.19045E+01_JPRB ,&
&  0.18277E+01_JPRB, 0.15953E+01_JPRB, 0.13192E+01_JPRB, 0.11865E+01_JPRB, 0.10959E+01_JPRB, 0.10235E+01_JPRB, 0.94389E+00_JPRB ,&
&  0.85635E+00_JPRB, 0.79157E+00_JPRB, 0.74022E+00_JPRB, 0.68681E+00_JPRB, 0.63455E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.64869E+00_JPRB, 0.61986E+00_JPRB, 0.57433E+00_JPRB, 0.57741E+00_JPRB, 0.60960E+00_JPRB, 0.62311E+00_JPRB, 0.60378E+00_JPRB ,&
&  0.58548E+00_JPRB, 0.56266E+00_JPRB, 0.55273E+00_JPRB, 0.54416E+00_JPRB, 0.53223E+00_JPRB, 0.52776E+00_JPRB, 0.50935E+00_JPRB ,&
&  0.50873E+00_JPRB, 0.49300E+00_JPRB, 0.44621E+00_JPRB, 0.42235E+00_JPRB, 0.42687E+00_JPRB, 0.40337E+00_JPRB, 0.38974E+00_JPRB ,&
&  0.41946E+00_JPRB, 0.45432E+00_JPRB, 0.50193E+00_JPRB, 0.57024E+00_JPRB, 0.66420E+00_JPRB, 0.82712E+00_JPRB, 0.91322E+00_JPRB ,&
&  0.87053E+00_JPRB, 0.83227E+00_JPRB, 0.75961E+00_JPRB, 0.69341E+00_JPRB, 0.64275E+00_JPRB, 0.62022E+00_JPRB, 0.61281E+00_JPRB ,&
&  0.61523E+00_JPRB, 0.63769E+00_JPRB, 0.70911E+00_JPRB, 0.81331E+00_JPRB, 0.91816E+00_JPRB, 0.95624E+00_JPRB, 0.95787E+00_JPRB ,&
&  0.98871E+00_JPRB, 0.10627E+01_JPRB, 0.10760E+01_JPRB, 0.10120E+01_JPRB, 0.90808E+00_JPRB, 0.76900E+00_JPRB, 0.76727E+00_JPRB ,&
&  0.80791E+00_JPRB, 0.74945E+00_JPRB, 0.79087E+00_JPRB, 0.10066E+01_JPRB, 0.11629E+01_JPRB, 0.12423E+01_JPRB, 0.15091E+01_JPRB ,&
&  0.12407E+01_JPRB, 0.99658E+00_JPRB, 0.84742E+00_JPRB, 0.71203E+00_JPRB, 0.10414E+01_JPRB, 0.14694E+01_JPRB, 0.16292E+01_JPRB ,&
&  0.17069E+01_JPRB, 0.18725E+01_JPRB, 0.20995E+01_JPRB, 0.19188E+01_JPRB, 0.14104E+01_JPRB, 0.11431E+01_JPRB, 0.13950E+01_JPRB ,&
&  0.14664E+01_JPRB, 0.15213E+01_JPRB, 0.14682E+01_JPRB, 0.12612E+01_JPRB, 0.10304E+01_JPRB, 0.11615E+01_JPRB, 0.11428E+01_JPRB ,&
&  0.11566E+01_JPRB, 0.11846E+01_JPRB, 0.15058E+01_JPRB, 0.19342E+01_JPRB, 0.21530E+01_JPRB, 0.24022E+01_JPRB, 0.24693E+01_JPRB ,&
&  0.20982E+01_JPRB, 0.18459E+01_JPRB, 0.15433E+01_JPRB, 0.13169E+01_JPRB, 0.11850E+01_JPRB, 0.11021E+01_JPRB, 0.10354E+01_JPRB ,&
&  0.94966E+00_JPRB, 0.88697E+00_JPRB, 0.82356E+00_JPRB, 0.74482E+00_JPRB, 0.68352E+00_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.71316E+00_JPRB, 0.67202E+00_JPRB, 0.61939E+00_JPRB, 0.62246E+00_JPRB, 0.65827E+00_JPRB, 0.64581E+00_JPRB, 0.61236E+00_JPRB ,&
&  0.60290E+00_JPRB, 0.57882E+00_JPRB, 0.56619E+00_JPRB, 0.55022E+00_JPRB, 0.53434E+00_JPRB, 0.52510E+00_JPRB, 0.52312E+00_JPRB ,&
&  0.54028E+00_JPRB, 0.56525E+00_JPRB, 0.50788E+00_JPRB, 0.46972E+00_JPRB, 0.45368E+00_JPRB, 0.43572E+00_JPRB, 0.40235E+00_JPRB ,&
&  0.44557E+00_JPRB, 0.50911E+00_JPRB, 0.57822E+00_JPRB, 0.67895E+00_JPRB, 0.82887E+00_JPRB, 0.87700E+00_JPRB, 0.87439E+00_JPRB ,&
&  0.84862E+00_JPRB, 0.79363E+00_JPRB, 0.72049E+00_JPRB, 0.65559E+00_JPRB, 0.62260E+00_JPRB, 0.61414E+00_JPRB, 0.61429E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.70315E+00_JPRB, 0.78541E+00_JPRB, 0.88285E+00_JPRB, 0.98118E+00_JPRB, 0.10328E+01_JPRB, 0.10611E+01_JPRB ,&
&  0.11185E+01_JPRB, 0.11835E+01_JPRB, 0.11965E+01_JPRB, 0.11467E+01_JPRB, 0.11037E+01_JPRB, 0.94806E+00_JPRB, 0.10832E+01_JPRB ,&
&  0.10337E+01_JPRB, 0.10813E+01_JPRB, 0.10324E+01_JPRB, 0.12996E+01_JPRB, 0.12308E+01_JPRB, 0.12142E+01_JPRB, 0.13620E+01_JPRB ,&
&  0.10549E+01_JPRB, 0.97629E+00_JPRB, 0.11266E+01_JPRB, 0.13037E+01_JPRB, 0.14321E+01_JPRB, 0.15995E+01_JPRB, 0.23209E+01_JPRB ,&
&  0.19558E+01_JPRB, 0.22359E+01_JPRB, 0.24435E+01_JPRB, 0.23651E+01_JPRB, 0.10748E+01_JPRB, 0.82453E+00_JPRB, 0.13126E+01_JPRB ,&
&  0.14559E+01_JPRB, 0.99645E+00_JPRB, 0.75451E+00_JPRB, 0.94426E+00_JPRB, 0.73654E+00_JPRB, 0.87212E+00_JPRB, 0.12236E+01_JPRB ,&
&  0.13966E+01_JPRB, 0.16091E+01_JPRB, 0.22355E+01_JPRB, 0.24270E+01_JPRB, 0.23791E+01_JPRB, 0.30006E+01_JPRB, 0.25736E+01_JPRB ,&
&  0.22442E+01_JPRB, 0.18001E+01_JPRB, 0.16499E+01_JPRB, 0.14394E+01_JPRB, 0.12914E+01_JPRB, 0.12050E+01_JPRB, 0.11296E+01_JPRB ,&
&  0.10436E+01_JPRB, 0.96969E+00_JPRB, 0.89946E+00_JPRB, 0.82319E+00_JPRB, 0.76150E+00_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.75980E+00_JPRB, 0.71813E+00_JPRB, 0.67474E+00_JPRB, 0.67220E+00_JPRB, 0.69458E+00_JPRB, 0.65306E+00_JPRB, 0.62831E+00_JPRB ,&
&  0.60321E+00_JPRB, 0.58831E+00_JPRB, 0.56924E+00_JPRB, 0.54717E+00_JPRB, 0.52928E+00_JPRB, 0.52965E+00_JPRB, 0.53761E+00_JPRB ,&
&  0.55377E+00_JPRB, 0.60210E+00_JPRB, 0.62796E+00_JPRB, 0.59191E+00_JPRB, 0.56132E+00_JPRB, 0.51781E+00_JPRB, 0.48481E+00_JPRB ,&
&  0.52670E+00_JPRB, 0.61975E+00_JPRB, 0.68731E+00_JPRB, 0.74262E+00_JPRB, 0.79259E+00_JPRB, 0.77977E+00_JPRB, 0.80316E+00_JPRB ,&
&  0.79720E+00_JPRB, 0.74333E+00_JPRB, 0.68890E+00_JPRB, 0.64607E+00_JPRB, 0.62976E+00_JPRB, 0.62960E+00_JPRB, 0.66058E+00_JPRB ,&
&  0.71209E+00_JPRB, 0.77073E+00_JPRB, 0.83400E+00_JPRB, 0.95151E+00_JPRB, 0.10747E+01_JPRB, 0.11315E+01_JPRB, 0.11807E+01_JPRB ,&
&  0.12268E+01_JPRB, 0.12501E+01_JPRB, 0.12601E+01_JPRB, 0.12790E+01_JPRB, 0.13774E+01_JPRB, 0.12418E+01_JPRB, 0.10508E+01_JPRB ,&
&  0.95621E+00_JPRB, 0.10965E+01_JPRB, 0.15432E+01_JPRB, 0.14884E+01_JPRB, 0.12494E+01_JPRB, 0.12734E+01_JPRB, 0.12531E+01_JPRB ,&
&  0.12216E+01_JPRB, 0.14345E+01_JPRB, 0.16335E+01_JPRB, 0.18383E+01_JPRB, 0.19821E+01_JPRB, 0.16205E+01_JPRB, 0.22683E+01_JPRB ,&
&  0.24325E+01_JPRB, 0.23766E+01_JPRB, 0.17741E+01_JPRB, 0.12226E+01_JPRB, 0.12351E+01_JPRB, 0.93308E+00_JPRB, 0.66116E+00_JPRB ,&
&  0.62437E+00_JPRB, 0.59884E+00_JPRB, 0.55164E+00_JPRB, 0.58296E+00_JPRB, 0.59256E+00_JPRB, 0.80492E+00_JPRB, 0.12245E+01_JPRB ,&
&  0.17869E+01_JPRB, 0.22600E+01_JPRB, 0.25796E+01_JPRB, 0.26357E+01_JPRB, 0.25327E+01_JPRB, 0.29622E+01_JPRB, 0.26675E+01_JPRB ,&
&  0.19015E+01_JPRB, 0.16560E+01_JPRB, 0.15150E+01_JPRB, 0.14453E+01_JPRB, 0.13763E+01_JPRB, 0.13082E+01_JPRB, 0.12143E+01_JPRB ,&
&  0.11117E+01_JPRB, 0.10374E+01_JPRB, 0.97148E+00_JPRB, 0.88510E+00_JPRB, 0.81528E+00_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.80175E+00_JPRB, 0.76497E+00_JPRB, 0.75035E+00_JPRB, 0.73424E+00_JPRB, 0.72283E+00_JPRB, 0.66338E+00_JPRB, 0.64246E+00_JPRB ,&
&  0.60176E+00_JPRB, 0.58593E+00_JPRB, 0.55579E+00_JPRB, 0.53837E+00_JPRB, 0.52729E+00_JPRB, 0.53879E+00_JPRB, 0.56170E+00_JPRB ,&
&  0.58917E+00_JPRB, 0.66632E+00_JPRB, 0.77442E+00_JPRB, 0.81441E+00_JPRB, 0.83803E+00_JPRB, 0.78705E+00_JPRB, 0.72765E+00_JPRB ,&
&  0.71070E+00_JPRB, 0.80172E+00_JPRB, 0.80108E+00_JPRB, 0.79444E+00_JPRB, 0.80031E+00_JPRB, 0.78215E+00_JPRB, 0.76260E+00_JPRB ,&
&  0.75517E+00_JPRB, 0.71500E+00_JPRB, 0.66288E+00_JPRB, 0.64196E+00_JPRB, 0.64364E+00_JPRB, 0.66683E+00_JPRB, 0.70875E+00_JPRB ,&
&  0.77950E+00_JPRB, 0.81649E+00_JPRB, 0.89733E+00_JPRB, 0.10381E+01_JPRB, 0.11509E+01_JPRB, 0.11956E+01_JPRB, 0.12601E+01_JPRB ,&
&  0.13180E+01_JPRB, 0.13174E+01_JPRB, 0.13023E+01_JPRB, 0.12678E+01_JPRB, 0.10990E+01_JPRB, 0.92116E+00_JPRB, 0.11070E+01_JPRB ,&
&  0.12722E+01_JPRB, 0.15796E+01_JPRB, 0.17588E+01_JPRB, 0.18571E+01_JPRB, 0.14085E+01_JPRB, 0.13001E+01_JPRB, 0.13063E+01_JPRB ,&
&  0.14683E+01_JPRB, 0.23012E+01_JPRB, 0.20884E+01_JPRB, 0.19004E+01_JPRB, 0.23418E+01_JPRB, 0.19538E+01_JPRB, 0.20694E+01_JPRB ,&
&  0.24962E+01_JPRB, 0.23409E+01_JPRB, 0.18003E+01_JPRB, 0.13870E+01_JPRB, 0.17265E+01_JPRB, 0.16813E+01_JPRB, 0.84384E+00_JPRB ,&
&  0.68455E+00_JPRB, 0.63522E+00_JPRB, 0.59675E+00_JPRB, 0.57065E+00_JPRB, 0.58363E+00_JPRB, 0.84680E+00_JPRB, 0.15652E+01_JPRB ,&
&  0.20985E+01_JPRB, 0.27667E+01_JPRB, 0.28620E+01_JPRB, 0.27849E+01_JPRB, 0.25691E+01_JPRB, 0.26691E+01_JPRB, 0.23407E+01_JPRB ,&
&  0.19373E+01_JPRB, 0.17213E+01_JPRB, 0.15613E+01_JPRB, 0.14823E+01_JPRB, 0.14136E+01_JPRB, 0.13464E+01_JPRB, 0.12632E+01_JPRB ,&
&  0.11678E+01_JPRB, 0.10886E+01_JPRB, 0.10167E+01_JPRB, 0.92638E+00_JPRB, 0.85251E+00_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.83354E+00_JPRB, 0.81035E+00_JPRB, 0.81128E+00_JPRB, 0.80030E+00_JPRB, 0.75850E+00_JPRB, 0.70197E+00_JPRB, 0.64638E+00_JPRB ,&
&  0.60681E+00_JPRB, 0.57681E+00_JPRB, 0.55380E+00_JPRB, 0.54253E+00_JPRB, 0.54117E+00_JPRB, 0.55917E+00_JPRB, 0.58626E+00_JPRB ,&
&  0.64449E+00_JPRB, 0.74830E+00_JPRB, 0.86872E+00_JPRB, 0.10775E+01_JPRB, 0.11206E+01_JPRB, 0.10417E+01_JPRB, 0.10869E+01_JPRB ,&
&  0.10639E+01_JPRB, 0.10541E+01_JPRB, 0.98620E+00_JPRB, 0.90279E+00_JPRB, 0.84615E+00_JPRB, 0.81726E+00_JPRB, 0.78645E+00_JPRB ,&
&  0.73611E+00_JPRB, 0.71146E+00_JPRB, 0.67137E+00_JPRB, 0.67892E+00_JPRB, 0.69725E+00_JPRB, 0.72691E+00_JPRB, 0.77981E+00_JPRB ,&
&  0.80759E+00_JPRB, 0.83466E+00_JPRB, 0.95098E+00_JPRB, 0.10875E+01_JPRB, 0.11826E+01_JPRB, 0.12587E+01_JPRB, 0.13472E+01_JPRB ,&
&  0.13717E+01_JPRB, 0.13653E+01_JPRB, 0.13267E+01_JPRB, 0.12705E+01_JPRB, 0.10432E+01_JPRB, 0.11882E+01_JPRB, 0.12393E+01_JPRB ,&
&  0.13743E+01_JPRB, 0.15517E+01_JPRB, 0.16182E+01_JPRB, 0.17334E+01_JPRB, 0.15020E+01_JPRB, 0.13472E+01_JPRB, 0.13209E+01_JPRB ,&
&  0.17850E+01_JPRB, 0.19232E+01_JPRB, 0.17302E+01_JPRB, 0.18763E+01_JPRB, 0.23138E+01_JPRB, 0.29344E+01_JPRB, 0.18907E+01_JPRB ,&
&  0.16757E+01_JPRB, 0.21559E+01_JPRB, 0.18006E+01_JPRB, 0.15762E+01_JPRB, 0.18546E+01_JPRB, 0.21868E+01_JPRB, 0.16207E+01_JPRB ,&
&  0.83716E+00_JPRB, 0.68025E+00_JPRB, 0.70234E+00_JPRB, 0.75478E+00_JPRB, 0.75467E+00_JPRB, 0.11097E+01_JPRB, 0.27375E+01_JPRB ,&
&  0.29046E+01_JPRB, 0.33086E+01_JPRB, 0.30923E+01_JPRB, 0.27695E+01_JPRB, 0.25700E+01_JPRB, 0.25098E+01_JPRB, 0.22613E+01_JPRB ,&
&  0.19309E+01_JPRB, 0.17389E+01_JPRB, 0.15872E+01_JPRB, 0.14798E+01_JPRB, 0.13832E+01_JPRB, 0.12908E+01_JPRB, 0.12034E+01_JPRB ,&
&  0.11386E+01_JPRB, 0.10713E+01_JPRB, 0.10125E+01_JPRB, 0.93823E+00_JPRB, 0.87345E+00_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.84362E+00_JPRB, 0.82845E+00_JPRB, 0.83134E+00_JPRB, 0.81970E+00_JPRB, 0.79882E+00_JPRB, 0.77348E+00_JPRB, 0.70070E+00_JPRB ,&
&  0.62913E+00_JPRB, 0.58169E+00_JPRB, 0.56104E+00_JPRB, 0.55730E+00_JPRB, 0.56772E+00_JPRB, 0.59186E+00_JPRB, 0.61892E+00_JPRB ,&
&  0.71404E+00_JPRB, 0.81022E+00_JPRB, 0.96967E+00_JPRB, 0.11871E+01_JPRB, 0.12953E+01_JPRB, 0.11754E+01_JPRB, 0.12366E+01_JPRB ,&
&  0.14254E+01_JPRB, 0.12987E+01_JPRB, 0.11461E+01_JPRB, 0.10689E+01_JPRB, 0.10031E+01_JPRB, 0.94712E+00_JPRB, 0.87658E+00_JPRB ,&
&  0.80940E+00_JPRB, 0.80675E+00_JPRB, 0.76625E+00_JPRB, 0.77061E+00_JPRB, 0.77714E+00_JPRB, 0.82050E+00_JPRB, 0.81309E+00_JPRB ,&
&  0.81218E+00_JPRB, 0.89081E+00_JPRB, 0.10053E+01_JPRB, 0.11129E+01_JPRB, 0.12126E+01_JPRB, 0.13031E+01_JPRB, 0.14084E+01_JPRB ,&
&  0.14588E+01_JPRB, 0.14327E+01_JPRB, 0.13655E+01_JPRB, 0.12449E+01_JPRB, 0.12205E+01_JPRB, 0.12406E+01_JPRB, 0.13343E+01_JPRB ,&
&  0.13862E+01_JPRB, 0.14978E+01_JPRB, 0.15489E+01_JPRB, 0.15435E+01_JPRB, 0.14362E+01_JPRB, 0.13595E+01_JPRB, 0.13662E+01_JPRB ,&
&  0.16862E+01_JPRB, 0.17335E+01_JPRB, 0.15718E+01_JPRB, 0.19616E+01_JPRB, 0.24636E+01_JPRB, 0.28442E+01_JPRB, 0.26350E+01_JPRB ,&
&  0.22285E+01_JPRB, 0.18331E+01_JPRB, 0.16714E+01_JPRB, 0.18573E+01_JPRB, 0.20236E+01_JPRB, 0.21890E+01_JPRB, 0.26937E+01_JPRB ,&
&  0.27587E+01_JPRB, 0.17170E+01_JPRB, 0.14669E+01_JPRB, 0.18774E+01_JPRB, 0.16172E+01_JPRB, 0.16854E+01_JPRB, 0.23698E+01_JPRB ,&
&  0.29484E+01_JPRB, 0.33298E+01_JPRB, 0.31588E+01_JPRB, 0.27667E+01_JPRB, 0.26050E+01_JPRB, 0.23991E+01_JPRB, 0.21772E+01_JPRB ,&
&  0.18633E+01_JPRB, 0.16246E+01_JPRB, 0.14621E+01_JPRB, 0.13443E+01_JPRB, 0.12345E+01_JPRB, 0.11463E+01_JPRB, 0.10971E+01_JPRB ,&
&  0.10583E+01_JPRB, 0.10136E+01_JPRB, 0.97400E+00_JPRB, 0.91887E+00_JPRB, 0.86913E+00_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.84171E+00_JPRB, 0.83586E+00_JPRB, 0.83639E+00_JPRB, 0.83308E+00_JPRB, 0.86699E+00_JPRB, 0.86853E+00_JPRB, 0.79764E+00_JPRB ,&
&  0.72266E+00_JPRB, 0.60731E+00_JPRB, 0.58805E+00_JPRB, 0.59391E+00_JPRB, 0.61104E+00_JPRB, 0.63985E+00_JPRB, 0.70169E+00_JPRB ,&
&  0.79701E+00_JPRB, 0.89466E+00_JPRB, 0.10947E+01_JPRB, 0.13348E+01_JPRB, 0.15772E+01_JPRB, 0.18635E+01_JPRB, 0.13107E+01_JPRB ,&
&  0.13006E+01_JPRB, 0.12913E+01_JPRB, 0.11922E+01_JPRB, 0.11820E+01_JPRB, 0.11455E+01_JPRB, 0.11224E+01_JPRB, 0.99689E+00_JPRB ,&
&  0.91959E+00_JPRB, 0.91185E+00_JPRB, 0.86941E+00_JPRB, 0.83830E+00_JPRB, 0.86191E+00_JPRB, 0.86978E+00_JPRB, 0.82540E+00_JPRB ,&
&  0.87032E+00_JPRB, 0.95012E+00_JPRB, 0.10486E+01_JPRB, 0.11431E+01_JPRB, 0.12447E+01_JPRB, 0.13561E+01_JPRB, 0.14323E+01_JPRB ,&
&  0.15122E+01_JPRB, 0.15081E+01_JPRB, 0.14380E+01_JPRB, 0.12956E+01_JPRB, 0.12845E+01_JPRB, 0.12921E+01_JPRB, 0.13074E+01_JPRB ,&
&  0.12362E+01_JPRB, 0.11900E+01_JPRB, 0.13475E+01_JPRB, 0.14267E+01_JPRB, 0.13716E+01_JPRB, 0.12760E+01_JPRB, 0.12989E+01_JPRB ,&
&  0.16295E+01_JPRB, 0.18701E+01_JPRB, 0.21315E+01_JPRB, 0.22914E+01_JPRB, 0.26407E+01_JPRB, 0.28630E+01_JPRB, 0.29471E+01_JPRB ,&
&  0.29638E+01_JPRB, 0.25245E+01_JPRB, 0.22719E+01_JPRB, 0.22766E+01_JPRB, 0.22239E+01_JPRB, 0.23128E+01_JPRB, 0.25956E+01_JPRB ,&
&  0.29964E+01_JPRB, 0.35988E+01_JPRB, 0.36861E+01_JPRB, 0.30479E+01_JPRB, 0.27084E+01_JPRB, 0.23346E+01_JPRB, 0.24897E+01_JPRB ,&
&  0.29098E+01_JPRB, 0.29064E+01_JPRB, 0.28175E+01_JPRB, 0.29427E+01_JPRB, 0.23572E+01_JPRB, 0.20985E+01_JPRB, 0.18847E+01_JPRB ,&
&  0.16924E+01_JPRB, 0.14859E+01_JPRB, 0.13352E+01_JPRB, 0.12155E+01_JPRB, 0.10943E+01_JPRB, 0.10257E+01_JPRB, 0.10037E+01_JPRB ,&
&  0.97698E+00_JPRB, 0.95584E+00_JPRB, 0.92865E+00_JPRB, 0.89167E+00_JPRB, 0.86214E+00_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.84140E+00_JPRB, 0.83117E+00_JPRB, 0.83245E+00_JPRB, 0.83996E+00_JPRB, 0.91864E+00_JPRB, 0.94445E+00_JPRB, 0.93578E+00_JPRB ,&
&  0.91210E+00_JPRB, 0.66211E+00_JPRB, 0.62956E+00_JPRB, 0.63731E+00_JPRB, 0.66501E+00_JPRB, 0.70956E+00_JPRB, 0.78934E+00_JPRB ,&
&  0.86997E+00_JPRB, 0.99141E+00_JPRB, 0.12048E+01_JPRB, 0.14801E+01_JPRB, 0.17028E+01_JPRB, 0.18201E+01_JPRB, 0.17208E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.13632E+01_JPRB, 0.12525E+01_JPRB, 0.13170E+01_JPRB, 0.13108E+01_JPRB, 0.12664E+01_JPRB, 0.11117E+01_JPRB ,&
&  0.10183E+01_JPRB, 0.10021E+01_JPRB, 0.98416E+00_JPRB, 0.99541E+00_JPRB, 0.98537E+00_JPRB, 0.92072E+00_JPRB, 0.90897E+00_JPRB ,&
&  0.96113E+00_JPRB, 0.10540E+01_JPRB, 0.11565E+01_JPRB, 0.12389E+01_JPRB, 0.13468E+01_JPRB, 0.14454E+01_JPRB, 0.15091E+01_JPRB ,&
&  0.15814E+01_JPRB, 0.15839E+01_JPRB, 0.14638E+01_JPRB, 0.13950E+01_JPRB, 0.13577E+01_JPRB, 0.13623E+01_JPRB, 0.13468E+01_JPRB ,&
&  0.12553E+01_JPRB, 0.11835E+01_JPRB, 0.12699E+01_JPRB, 0.12637E+01_JPRB, 0.12139E+01_JPRB, 0.12816E+01_JPRB, 0.13804E+01_JPRB ,&
&  0.17832E+01_JPRB, 0.20011E+01_JPRB, 0.27178E+01_JPRB, 0.25111E+01_JPRB, 0.25584E+01_JPRB, 0.28368E+01_JPRB, 0.27452E+01_JPRB ,&
&  0.24541E+01_JPRB, 0.24897E+01_JPRB, 0.23697E+01_JPRB, 0.24735E+01_JPRB, 0.25011E+01_JPRB, 0.26380E+01_JPRB, 0.27190E+01_JPRB ,&
&  0.29155E+01_JPRB, 0.32112E+01_JPRB, 0.35631E+01_JPRB, 0.29424E+01_JPRB, 0.26452E+01_JPRB, 0.24399E+01_JPRB, 0.25723E+01_JPRB ,&
&  0.27643E+01_JPRB, 0.27319E+01_JPRB, 0.28339E+01_JPRB, 0.26697E+01_JPRB, 0.20216E+01_JPRB, 0.18098E+01_JPRB, 0.16346E+01_JPRB ,&
&  0.14907E+01_JPRB, 0.13365E+01_JPRB, 0.12127E+01_JPRB, 0.10984E+01_JPRB, 0.10135E+01_JPRB, 0.97732E+00_JPRB, 0.96053E+00_JPRB ,&
&  0.94197E+00_JPRB, 0.92776E+00_JPRB, 0.91161E+00_JPRB, 0.88264E+00_JPRB, 0.85850E+00_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.85392E+00_JPRB, 0.85186E+00_JPRB, 0.86003E+00_JPRB, 0.85838E+00_JPRB, 0.93972E+00_JPRB, 0.10526E+01_JPRB, 0.11215E+01_JPRB ,&
&  0.10683E+01_JPRB, 0.69535E+00_JPRB, 0.67482E+00_JPRB, 0.68685E+00_JPRB, 0.71671E+00_JPRB, 0.78334E+00_JPRB, 0.86540E+00_JPRB ,&
&  0.97403E+00_JPRB, 0.11190E+01_JPRB, 0.13417E+01_JPRB, 0.16145E+01_JPRB, 0.18039E+01_JPRB, 0.20082E+01_JPRB, 0.19919E+01_JPRB ,&
&  0.14211E+01_JPRB, 0.12047E+01_JPRB, 0.12935E+01_JPRB, 0.13622E+01_JPRB, 0.13234E+01_JPRB, 0.13475E+01_JPRB, 0.13355E+01_JPRB ,&
&  0.12508E+01_JPRB, 0.11667E+01_JPRB, 0.11752E+01_JPRB, 0.11711E+01_JPRB, 0.10413E+01_JPRB, 0.10237E+01_JPRB, 0.10423E+01_JPRB ,&
&  0.11057E+01_JPRB, 0.11940E+01_JPRB, 0.12867E+01_JPRB, 0.13626E+01_JPRB, 0.14716E+01_JPRB, 0.15492E+01_JPRB, 0.16177E+01_JPRB ,&
&  0.16415E+01_JPRB, 0.16542E+01_JPRB, 0.15589E+01_JPRB, 0.14509E+01_JPRB, 0.14178E+01_JPRB, 0.13930E+01_JPRB, 0.13420E+01_JPRB ,&
&  0.12940E+01_JPRB, 0.13156E+01_JPRB, 0.13262E+01_JPRB, 0.12858E+01_JPRB, 0.12632E+01_JPRB, 0.14763E+01_JPRB, 0.16950E+01_JPRB ,&
&  0.19880E+01_JPRB, 0.23528E+01_JPRB, 0.31357E+01_JPRB, 0.24492E+01_JPRB, 0.24935E+01_JPRB, 0.26614E+01_JPRB, 0.26288E+01_JPRB ,&
&  0.24505E+01_JPRB, 0.25113E+01_JPRB, 0.24894E+01_JPRB, 0.25375E+01_JPRB, 0.26347E+01_JPRB, 0.26059E+01_JPRB, 0.30737E+01_JPRB ,&
&  0.29191E+01_JPRB, 0.33399E+01_JPRB, 0.30948E+01_JPRB, 0.29144E+01_JPRB, 0.24774E+01_JPRB, 0.24024E+01_JPRB, 0.23481E+01_JPRB ,&
&  0.24763E+01_JPRB, 0.24421E+01_JPRB, 0.22962E+01_JPRB, 0.21176E+01_JPRB, 0.17150E+01_JPRB, 0.15652E+01_JPRB, 0.14171E+01_JPRB ,&
&  0.12735E+01_JPRB, 0.11633E+01_JPRB, 0.11034E+01_JPRB, 0.10498E+01_JPRB, 0.10201E+01_JPRB, 0.99382E+00_JPRB, 0.96650E+00_JPRB ,&
&  0.94166E+00_JPRB, 0.92431E+00_JPRB, 0.90796E+00_JPRB, 0.88211E+00_JPRB, 0.86340E+00_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.85647E+00_JPRB, 0.84466E+00_JPRB, 0.83710E+00_JPRB, 0.80282E+00_JPRB, 0.78708E+00_JPRB, 0.78937E+00_JPRB, 0.77073E+00_JPRB ,&
&  0.74304E+00_JPRB, 0.71095E+00_JPRB, 0.71626E+00_JPRB, 0.74642E+00_JPRB, 0.79170E+00_JPRB, 0.86773E+00_JPRB, 0.96826E+00_JPRB ,&
&  0.10989E+01_JPRB, 0.12214E+01_JPRB, 0.14132E+01_JPRB, 0.15991E+01_JPRB, 0.17412E+01_JPRB, 0.18421E+01_JPRB, 0.18585E+01_JPRB ,&
&  0.14914E+01_JPRB, 0.11322E+01_JPRB, 0.13714E+01_JPRB, 0.14944E+01_JPRB, 0.14597E+01_JPRB, 0.15144E+01_JPRB, 0.14940E+01_JPRB ,&
&  0.13450E+01_JPRB, 0.13128E+01_JPRB, 0.13361E+01_JPRB, 0.12637E+01_JPRB, 0.11274E+01_JPRB, 0.11490E+01_JPRB, 0.12020E+01_JPRB ,&
&  0.12792E+01_JPRB, 0.13442E+01_JPRB, 0.14230E+01_JPRB, 0.15029E+01_JPRB, 0.15870E+01_JPRB, 0.16903E+01_JPRB, 0.17540E+01_JPRB ,&
&  0.17586E+01_JPRB, 0.17425E+01_JPRB, 0.16996E+01_JPRB, 0.15718E+01_JPRB, 0.14880E+01_JPRB, 0.14449E+01_JPRB, 0.13929E+01_JPRB ,&
&  0.13812E+01_JPRB, 0.13705E+01_JPRB, 0.14323E+01_JPRB, 0.15788E+01_JPRB, 0.18184E+01_JPRB, 0.18213E+01_JPRB, 0.20818E+01_JPRB ,&
&  0.22762E+01_JPRB, 0.27033E+01_JPRB, 0.28309E+01_JPRB, 0.31947E+01_JPRB, 0.20018E+01_JPRB, 0.23152E+01_JPRB, 0.21669E+01_JPRB ,&
&  0.25470E+01_JPRB, 0.24299E+01_JPRB, 0.24755E+01_JPRB, 0.26078E+01_JPRB, 0.28903E+01_JPRB, 0.28652E+01_JPRB, 0.30073E+01_JPRB ,&
&  0.29621E+01_JPRB, 0.28827E+01_JPRB, 0.26202E+01_JPRB, 0.25632E+01_JPRB, 0.24094E+01_JPRB, 0.25014E+01_JPRB, 0.21857E+01_JPRB ,&
&  0.21872E+01_JPRB, 0.20971E+01_JPRB, 0.19890E+01_JPRB, 0.18822E+01_JPRB, 0.15464E+01_JPRB, 0.13050E+01_JPRB, 0.11755E+01_JPRB ,&
&  0.11169E+01_JPRB, 0.10793E+01_JPRB, 0.10392E+01_JPRB, 0.10114E+01_JPRB, 0.99866E+00_JPRB, 0.98591E+00_JPRB, 0.96933E+00_JPRB ,&
&  0.94942E+00_JPRB, 0.93021E+00_JPRB, 0.91295E+00_JPRB, 0.89070E+00_JPRB, 0.87306E+00_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.83819E+00_JPRB, 0.81982E+00_JPRB, 0.80527E+00_JPRB, 0.78456E+00_JPRB, 0.77082E+00_JPRB, 0.75186E+00_JPRB, 0.74547E+00_JPRB ,&
&  0.74110E+00_JPRB, 0.75194E+00_JPRB, 0.76070E+00_JPRB, 0.78807E+00_JPRB, 0.82574E+00_JPRB, 0.89260E+00_JPRB, 0.95769E+00_JPRB ,&
&  0.10344E+01_JPRB, 0.11386E+01_JPRB, 0.13371E+01_JPRB, 0.15795E+01_JPRB, 0.17590E+01_JPRB, 0.18462E+01_JPRB, 0.17304E+01_JPRB ,&
&  0.16090E+01_JPRB, 0.15726E+01_JPRB, 0.15031E+01_JPRB, 0.12617E+01_JPRB, 0.12148E+01_JPRB, 0.14959E+01_JPRB, 0.14318E+01_JPRB ,&
&  0.13446E+01_JPRB, 0.13254E+01_JPRB, 0.12980E+01_JPRB, 0.12689E+01_JPRB, 0.12459E+01_JPRB, 0.12768E+01_JPRB, 0.13314E+01_JPRB ,&
&  0.13977E+01_JPRB, 0.14620E+01_JPRB, 0.15531E+01_JPRB, 0.16440E+01_JPRB, 0.17390E+01_JPRB, 0.18743E+01_JPRB, 0.19395E+01_JPRB ,&
&  0.19239E+01_JPRB, 0.19057E+01_JPRB, 0.18533E+01_JPRB, 0.17309E+01_JPRB, 0.15936E+01_JPRB, 0.15706E+01_JPRB, 0.15878E+01_JPRB ,&
&  0.16233E+01_JPRB, 0.16832E+01_JPRB, 0.17663E+01_JPRB, 0.19967E+01_JPRB, 0.22078E+01_JPRB, 0.20942E+01_JPRB, 0.21264E+01_JPRB ,&
&  0.24478E+01_JPRB, 0.28228E+01_JPRB, 0.25047E+01_JPRB, 0.29262E+01_JPRB, 0.16955E+01_JPRB, 0.20787E+01_JPRB, 0.24087E+01_JPRB ,&
&  0.22911E+01_JPRB, 0.23159E+01_JPRB, 0.24849E+01_JPRB, 0.27506E+01_JPRB, 0.30420E+01_JPRB, 0.28392E+01_JPRB, 0.28128E+01_JPRB ,&
&  0.26253E+01_JPRB, 0.23136E+01_JPRB, 0.22195E+01_JPRB, 0.21715E+01_JPRB, 0.22418E+01_JPRB, 0.22493E+01_JPRB, 0.18335E+01_JPRB ,&
&  0.16143E+01_JPRB, 0.18204E+01_JPRB, 0.18690E+01_JPRB, 0.18068E+01_JPRB, 0.14194E+01_JPRB, 0.11010E+01_JPRB, 0.10576E+01_JPRB ,&
&  0.10503E+01_JPRB, 0.10145E+01_JPRB, 0.99587E+00_JPRB, 0.98751E+00_JPRB, 0.98660E+00_JPRB, 0.97884E+00_JPRB, 0.96945E+00_JPRB ,&
&  0.95789E+00_JPRB, 0.93602E+00_JPRB, 0.91502E+00_JPRB, 0.89012E+00_JPRB, 0.86309E+00_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.80731E+00_JPRB, 0.79468E+00_JPRB, 0.78212E+00_JPRB, 0.76564E+00_JPRB, 0.72639E+00_JPRB, 0.70312E+00_JPRB, 0.69748E+00_JPRB ,&
&  0.70964E+00_JPRB, 0.72981E+00_JPRB, 0.74474E+00_JPRB, 0.75242E+00_JPRB, 0.77990E+00_JPRB, 0.79693E+00_JPRB, 0.82680E+00_JPRB ,&
&  0.88506E+00_JPRB, 0.98070E+00_JPRB, 0.11614E+01_JPRB, 0.14487E+01_JPRB, 0.17170E+01_JPRB, 0.19595E+01_JPRB, 0.18264E+01_JPRB ,&
&  0.16914E+01_JPRB, 0.16162E+01_JPRB, 0.16012E+01_JPRB, 0.16050E+01_JPRB, 0.14323E+01_JPRB, 0.15571E+01_JPRB, 0.16271E+01_JPRB ,&
&  0.16439E+01_JPRB, 0.15465E+01_JPRB, 0.14216E+01_JPRB, 0.13894E+01_JPRB, 0.13709E+01_JPRB, 0.13930E+01_JPRB, 0.14432E+01_JPRB ,&
&  0.15063E+01_JPRB, 0.15732E+01_JPRB, 0.16803E+01_JPRB, 0.18130E+01_JPRB, 0.19718E+01_JPRB, 0.21171E+01_JPRB, 0.21721E+01_JPRB ,&
&  0.21614E+01_JPRB, 0.21465E+01_JPRB, 0.21062E+01_JPRB, 0.19716E+01_JPRB, 0.19095E+01_JPRB, 0.19395E+01_JPRB, 0.20166E+01_JPRB ,&
&  0.21195E+01_JPRB, 0.20960E+01_JPRB, 0.22103E+01_JPRB, 0.24673E+01_JPRB, 0.25342E+01_JPRB, 0.24624E+01_JPRB, 0.24473E+01_JPRB ,&
&  0.26437E+01_JPRB, 0.29143E+01_JPRB, 0.19728E+01_JPRB, 0.23515E+01_JPRB, 0.28072E+01_JPRB, 0.23732E+01_JPRB, 0.22197E+01_JPRB ,&
&  0.22249E+01_JPRB, 0.23113E+01_JPRB, 0.25145E+01_JPRB, 0.27677E+01_JPRB, 0.30468E+01_JPRB, 0.32428E+01_JPRB, 0.24009E+01_JPRB ,&
&  0.22024E+01_JPRB, 0.19173E+01_JPRB, 0.18625E+01_JPRB, 0.18829E+01_JPRB, 0.19392E+01_JPRB, 0.18700E+01_JPRB, 0.15362E+01_JPRB ,&
&  0.14416E+01_JPRB, 0.16007E+01_JPRB, 0.16039E+01_JPRB, 0.16873E+01_JPRB, 0.16058E+01_JPRB, 0.10049E+01_JPRB, 0.10220E+01_JPRB ,&
&  0.10093E+01_JPRB, 0.98313E+00_JPRB, 0.97348E+00_JPRB, 0.96856E+00_JPRB, 0.96830E+00_JPRB, 0.96456E+00_JPRB, 0.95951E+00_JPRB ,&
&  0.94316E+00_JPRB, 0.91641E+00_JPRB, 0.88789E+00_JPRB, 0.85870E+00_JPRB, 0.83067E+00_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.78697E+00_JPRB, 0.77516E+00_JPRB, 0.76092E+00_JPRB, 0.72827E+00_JPRB, 0.68950E+00_JPRB, 0.67227E+00_JPRB, 0.66319E+00_JPRB ,&
&  0.66615E+00_JPRB, 0.68560E+00_JPRB, 0.69742E+00_JPRB, 0.69483E+00_JPRB, 0.69138E+00_JPRB, 0.70620E+00_JPRB, 0.75113E+00_JPRB ,&
&  0.80858E+00_JPRB, 0.88071E+00_JPRB, 0.10174E+01_JPRB, 0.13023E+01_JPRB, 0.16615E+01_JPRB, 0.19457E+01_JPRB, 0.18971E+01_JPRB ,&
&  0.17659E+01_JPRB, 0.17692E+01_JPRB, 0.17856E+01_JPRB, 0.18299E+01_JPRB, 0.18596E+01_JPRB, 0.19349E+01_JPRB, 0.20010E+01_JPRB ,&
&  0.17796E+01_JPRB, 0.14738E+01_JPRB, 0.14234E+01_JPRB, 0.14192E+01_JPRB, 0.14500E+01_JPRB, 0.14886E+01_JPRB, 0.15423E+01_JPRB ,&
&  0.16097E+01_JPRB, 0.17089E+01_JPRB, 0.18485E+01_JPRB, 0.20618E+01_JPRB, 0.22860E+01_JPRB, 0.24187E+01_JPRB, 0.24860E+01_JPRB ,&
&  0.25260E+01_JPRB, 0.25502E+01_JPRB, 0.26155E+01_JPRB, 0.23596E+01_JPRB, 0.24447E+01_JPRB, 0.25187E+01_JPRB, 0.25931E+01_JPRB ,&
&  0.25740E+01_JPRB, 0.24996E+01_JPRB, 0.26947E+01_JPRB, 0.28436E+01_JPRB, 0.29890E+01_JPRB, 0.29741E+01_JPRB, 0.28858E+01_JPRB ,&
&  0.29972E+01_JPRB, 0.27229E+01_JPRB, 0.17715E+01_JPRB, 0.17341E+01_JPRB, 0.18786E+01_JPRB, 0.17324E+01_JPRB, 0.21088E+01_JPRB ,&
&  0.20897E+01_JPRB, 0.21656E+01_JPRB, 0.22670E+01_JPRB, 0.23899E+01_JPRB, 0.25185E+01_JPRB, 0.25736E+01_JPRB, 0.20743E+01_JPRB ,&
&  0.17808E+01_JPRB, 0.16150E+01_JPRB, 0.15802E+01_JPRB, 0.15458E+01_JPRB, 0.15176E+01_JPRB, 0.14372E+01_JPRB, 0.13856E+01_JPRB ,&
&  0.13763E+01_JPRB, 0.13539E+01_JPRB, 0.12899E+01_JPRB, 0.12196E+01_JPRB, 0.10925E+01_JPRB, 0.98693E+00_JPRB, 0.99297E+00_JPRB ,&
&  0.98736E+00_JPRB, 0.95791E+00_JPRB, 0.95223E+00_JPRB, 0.94384E+00_JPRB, 0.94013E+00_JPRB, 0.93297E+00_JPRB, 0.92058E+00_JPRB ,&
&  0.89811E+00_JPRB, 0.87470E+00_JPRB, 0.85096E+00_JPRB, 0.82388E+00_JPRB, 0.80010E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.77518E+00_JPRB, 0.76211E+00_JPRB, 0.74008E+00_JPRB, 0.71104E+00_JPRB, 0.70980E+00_JPRB, 0.71446E+00_JPRB, 0.69957E+00_JPRB ,&
&  0.69062E+00_JPRB, 0.68454E+00_JPRB, 0.70044E+00_JPRB, 0.70320E+00_JPRB, 0.70787E+00_JPRB, 0.72383E+00_JPRB, 0.74320E+00_JPRB ,&
&  0.76180E+00_JPRB, 0.77675E+00_JPRB, 0.84561E+00_JPRB, 0.99057E+00_JPRB, 0.12645E+01_JPRB, 0.16267E+01_JPRB, 0.18873E+01_JPRB ,&
&  0.20386E+01_JPRB, 0.21746E+01_JPRB, 0.22993E+01_JPRB, 0.24031E+01_JPRB, 0.22651E+01_JPRB, 0.23233E+01_JPRB, 0.25183E+01_JPRB ,&
&  0.18693E+01_JPRB, 0.13836E+01_JPRB, 0.15006E+01_JPRB, 0.14073E+01_JPRB, 0.14544E+01_JPRB, 0.15495E+01_JPRB, 0.16307E+01_JPRB ,&
&  0.17406E+01_JPRB, 0.18698E+01_JPRB, 0.20290E+01_JPRB, 0.22472E+01_JPRB, 0.24757E+01_JPRB, 0.26377E+01_JPRB, 0.27652E+01_JPRB ,&
&  0.28714E+01_JPRB, 0.29824E+01_JPRB, 0.31448E+01_JPRB, 0.31276E+01_JPRB, 0.30600E+01_JPRB, 0.31716E+01_JPRB, 0.32271E+01_JPRB ,&
&  0.33056E+01_JPRB, 0.33078E+01_JPRB, 0.28942E+01_JPRB, 0.30117E+01_JPRB, 0.33985E+01_JPRB, 0.32164E+01_JPRB, 0.31653E+01_JPRB ,&
&  0.33840E+01_JPRB, 0.28035E+01_JPRB, 0.16201E+01_JPRB, 0.15408E+01_JPRB, 0.16755E+01_JPRB, 0.18235E+01_JPRB, 0.19697E+01_JPRB ,&
&  0.20214E+01_JPRB, 0.20555E+01_JPRB, 0.20358E+01_JPRB, 0.20339E+01_JPRB, 0.19437E+01_JPRB, 0.18316E+01_JPRB, 0.17109E+01_JPRB ,&
&  0.15007E+01_JPRB, 0.14394E+01_JPRB, 0.13840E+01_JPRB, 0.13409E+01_JPRB, 0.13347E+01_JPRB, 0.12862E+01_JPRB, 0.12510E+01_JPRB ,&
&  0.13427E+01_JPRB, 0.13719E+01_JPRB, 0.14898E+01_JPRB, 0.12599E+01_JPRB, 0.10292E+01_JPRB, 0.99769E+00_JPRB, 0.98303E+00_JPRB ,&
&  0.10057E+01_JPRB, 0.98645E+00_JPRB, 0.96957E+00_JPRB, 0.95226E+00_JPRB, 0.93910E+00_JPRB, 0.91666E+00_JPRB, 0.88556E+00_JPRB ,&
&  0.85385E+00_JPRB, 0.83023E+00_JPRB, 0.81585E+00_JPRB, 0.80001E+00_JPRB, 0.78098E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.75325E+00_JPRB, 0.72320E+00_JPRB, 0.71756E+00_JPRB, 0.75714E+00_JPRB, 0.73393E+00_JPRB, 0.75702E+00_JPRB, 0.75098E+00_JPRB ,&
&  0.75159E+00_JPRB, 0.75039E+00_JPRB, 0.76700E+00_JPRB, 0.78860E+00_JPRB, 0.81633E+00_JPRB, 0.83937E+00_JPRB, 0.84815E+00_JPRB ,&
&  0.85964E+00_JPRB, 0.87095E+00_JPRB, 0.88959E+00_JPRB, 0.92446E+00_JPRB, 0.99000E+00_JPRB, 0.11010E+01_JPRB, 0.13191E+01_JPRB ,&
&  0.15435E+01_JPRB, 0.16634E+01_JPRB, 0.17101E+01_JPRB, 0.17147E+01_JPRB, 0.19081E+01_JPRB, 0.23824E+01_JPRB, 0.27374E+01_JPRB ,&
&  0.16687E+01_JPRB, 0.14815E+01_JPRB, 0.13639E+01_JPRB, 0.12536E+01_JPRB, 0.13232E+01_JPRB, 0.15202E+01_JPRB, 0.16360E+01_JPRB ,&
&  0.17116E+01_JPRB, 0.18354E+01_JPRB, 0.19879E+01_JPRB, 0.21767E+01_JPRB, 0.23439E+01_JPRB, 0.25125E+01_JPRB, 0.26872E+01_JPRB ,&
&  0.28740E+01_JPRB, 0.31120E+01_JPRB, 0.33603E+01_JPRB, 0.36228E+01_JPRB, 0.36035E+01_JPRB, 0.40384E+01_JPRB, 0.41350E+01_JPRB ,&
&  0.42007E+01_JPRB, 0.40064E+01_JPRB, 0.34630E+01_JPRB, 0.35348E+01_JPRB, 0.37943E+01_JPRB, 0.37531E+01_JPRB, 0.34676E+01_JPRB ,&
&  0.34519E+01_JPRB, 0.25913E+01_JPRB, 0.17301E+01_JPRB, 0.16977E+01_JPRB, 0.17072E+01_JPRB, 0.17728E+01_JPRB, 0.17153E+01_JPRB ,&
&  0.17783E+01_JPRB, 0.17534E+01_JPRB, 0.17008E+01_JPRB, 0.16223E+01_JPRB, 0.15399E+01_JPRB, 0.15199E+01_JPRB, 0.15078E+01_JPRB ,&
&  0.13811E+01_JPRB, 0.13411E+01_JPRB, 0.12849E+01_JPRB, 0.12766E+01_JPRB, 0.13178E+01_JPRB, 0.11313E+01_JPRB, 0.11917E+01_JPRB ,&
&  0.12477E+01_JPRB, 0.13058E+01_JPRB, 0.11688E+01_JPRB, 0.13519E+01_JPRB, 0.10240E+01_JPRB, 0.95475E+00_JPRB, 0.96787E+00_JPRB ,&
&  0.99858E+00_JPRB, 0.96320E+00_JPRB, 0.94890E+00_JPRB, 0.93422E+00_JPRB, 0.92809E+00_JPRB, 0.90722E+00_JPRB, 0.85787E+00_JPRB ,&
&  0.82006E+00_JPRB, 0.79716E+00_JPRB, 0.78841E+00_JPRB, 0.77871E+00_JPRB, 0.77282E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.71913E+00_JPRB, 0.71177E+00_JPRB, 0.77209E+00_JPRB, 0.73729E+00_JPRB, 0.73816E+00_JPRB, 0.75660E+00_JPRB, 0.77657E+00_JPRB ,&
&  0.78319E+00_JPRB, 0.79882E+00_JPRB, 0.82701E+00_JPRB, 0.87038E+00_JPRB, 0.91175E+00_JPRB, 0.94346E+00_JPRB, 0.97850E+00_JPRB ,&
&  0.10054E+01_JPRB, 0.10213E+01_JPRB, 0.10304E+01_JPRB, 0.10398E+01_JPRB, 0.10697E+01_JPRB, 0.11045E+01_JPRB, 0.11583E+01_JPRB ,&
&  0.12294E+01_JPRB, 0.12505E+01_JPRB, 0.12551E+01_JPRB, 0.12945E+01_JPRB, 0.13396E+01_JPRB, 0.14760E+01_JPRB, 0.17476E+01_JPRB ,&
&  0.13841E+01_JPRB, 0.14619E+01_JPRB, 0.14000E+01_JPRB, 0.13344E+01_JPRB, 0.13946E+01_JPRB, 0.13856E+01_JPRB, 0.14620E+01_JPRB ,&
&  0.15988E+01_JPRB, 0.16412E+01_JPRB, 0.17316E+01_JPRB, 0.18172E+01_JPRB, 0.19239E+01_JPRB, 0.20647E+01_JPRB, 0.22064E+01_JPRB ,&
&  0.23843E+01_JPRB, 0.25998E+01_JPRB, 0.27684E+01_JPRB, 0.30359E+01_JPRB, 0.32961E+01_JPRB, 0.35743E+01_JPRB, 0.37252E+01_JPRB ,&
&  0.38205E+01_JPRB, 0.38870E+01_JPRB, 0.35900E+01_JPRB, 0.36171E+01_JPRB, 0.36287E+01_JPRB, 0.34515E+01_JPRB, 0.30235E+01_JPRB ,&
&  0.25804E+01_JPRB, 0.21671E+01_JPRB, 0.18830E+01_JPRB, 0.17319E+01_JPRB, 0.16730E+01_JPRB, 0.15913E+01_JPRB, 0.15433E+01_JPRB ,&
&  0.15339E+01_JPRB, 0.14588E+01_JPRB, 0.13765E+01_JPRB, 0.12848E+01_JPRB, 0.12468E+01_JPRB, 0.12537E+01_JPRB, 0.12611E+01_JPRB ,&
&  0.12160E+01_JPRB, 0.11790E+01_JPRB, 0.11772E+01_JPRB, 0.12418E+01_JPRB, 0.13570E+01_JPRB, 0.17440E+01_JPRB, 0.14141E+01_JPRB ,&
&  0.10893E+01_JPRB, 0.99765E+00_JPRB, 0.77362E+00_JPRB, 0.10173E+01_JPRB, 0.90530E+00_JPRB, 0.88307E+00_JPRB, 0.91936E+00_JPRB ,&
&  0.93466E+00_JPRB, 0.92889E+00_JPRB, 0.96194E+00_JPRB, 0.92404E+00_JPRB, 0.89055E+00_JPRB, 0.87207E+00_JPRB, 0.86633E+00_JPRB ,&
&  0.81518E+00_JPRB, 0.76276E+00_JPRB, 0.71099E+00_JPRB, 0.71752E+00_JPRB, 0.68017E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.63815E+00_JPRB, 0.68300E+00_JPRB, 0.78017E+00_JPRB, 0.71725E+00_JPRB, 0.71880E+00_JPRB, 0.73862E+00_JPRB, 0.76765E+00_JPRB ,&
&  0.79822E+00_JPRB, 0.83646E+00_JPRB, 0.87712E+00_JPRB, 0.93232E+00_JPRB, 0.98103E+00_JPRB, 0.10231E+01_JPRB, 0.10628E+01_JPRB ,&
&  0.10987E+01_JPRB, 0.11296E+01_JPRB, 0.11594E+01_JPRB, 0.11750E+01_JPRB, 0.11876E+01_JPRB, 0.12154E+01_JPRB, 0.12506E+01_JPRB ,&
&  0.13182E+01_JPRB, 0.13532E+01_JPRB, 0.13966E+01_JPRB, 0.14645E+01_JPRB, 0.15160E+01_JPRB, 0.15748E+01_JPRB, 0.13945E+01_JPRB ,&
&  0.15752E+01_JPRB, 0.14463E+01_JPRB, 0.13999E+01_JPRB, 0.13915E+01_JPRB, 0.13703E+01_JPRB, 0.13587E+01_JPRB, 0.13779E+01_JPRB ,&
&  0.14496E+01_JPRB, 0.14281E+01_JPRB, 0.14600E+01_JPRB, 0.14622E+01_JPRB, 0.15291E+01_JPRB, 0.16524E+01_JPRB, 0.17373E+01_JPRB ,&
&  0.18879E+01_JPRB, 0.20707E+01_JPRB, 0.22686E+01_JPRB, 0.24231E+01_JPRB, 0.25601E+01_JPRB, 0.25702E+01_JPRB, 0.25758E+01_JPRB ,&
&  0.26528E+01_JPRB, 0.28137E+01_JPRB, 0.27704E+01_JPRB, 0.28475E+01_JPRB, 0.28393E+01_JPRB, 0.26064E+01_JPRB, 0.20542E+01_JPRB ,&
&  0.17814E+01_JPRB, 0.18271E+01_JPRB, 0.16014E+01_JPRB, 0.18132E+01_JPRB, 0.16176E+01_JPRB, 0.14386E+01_JPRB, 0.13972E+01_JPRB ,&
&  0.13133E+01_JPRB, 0.12152E+01_JPRB, 0.11364E+01_JPRB, 0.10250E+01_JPRB, 0.10170E+01_JPRB, 0.10252E+01_JPRB, 0.10396E+01_JPRB ,&
&  0.99698E+00_JPRB, 0.96074E+00_JPRB, 0.95060E+00_JPRB, 0.10453E+01_JPRB, 0.14025E+01_JPRB, 0.14669E+01_JPRB, 0.16418E+01_JPRB ,&
&  0.97609E+00_JPRB, 0.62070E+00_JPRB, 0.71155E+00_JPRB, 0.77070E+00_JPRB, 0.75524E+00_JPRB, 0.75482E+00_JPRB, 0.81690E+00_JPRB ,&
&  0.89022E+00_JPRB, 0.89016E+00_JPRB, 0.86422E+00_JPRB, 0.86490E+00_JPRB, 0.85737E+00_JPRB, 0.85673E+00_JPRB, 0.77684E+00_JPRB ,&
&  0.75138E+00_JPRB, 0.69540E+00_JPRB, 0.66329E+00_JPRB, 0.64872E+00_JPRB, 0.61707E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.61309E+00_JPRB, 0.69264E+00_JPRB, 0.76589E+00_JPRB, 0.70309E+00_JPRB, 0.71692E+00_JPRB, 0.74344E+00_JPRB, 0.78077E+00_JPRB ,&
&  0.82282E+00_JPRB, 0.87127E+00_JPRB, 0.92144E+00_JPRB, 0.98002E+00_JPRB, 0.10267E+01_JPRB, 0.10698E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.11624E+01_JPRB, 0.11827E+01_JPRB, 0.12217E+01_JPRB, 0.12341E+01_JPRB, 0.12623E+01_JPRB, 0.12724E+01_JPRB, 0.12969E+01_JPRB ,&
&  0.13607E+01_JPRB, 0.14041E+01_JPRB, 0.14536E+01_JPRB, 0.14969E+01_JPRB, 0.15092E+01_JPRB, 0.15947E+01_JPRB, 0.12927E+01_JPRB ,&
&  0.15536E+01_JPRB, 0.14420E+01_JPRB, 0.13734E+01_JPRB, 0.13959E+01_JPRB, 0.13803E+01_JPRB, 0.13899E+01_JPRB, 0.13932E+01_JPRB ,&
&  0.13939E+01_JPRB, 0.14107E+01_JPRB, 0.14400E+01_JPRB, 0.14317E+01_JPRB, 0.13733E+01_JPRB, 0.14486E+01_JPRB, 0.15299E+01_JPRB ,&
&  0.16317E+01_JPRB, 0.17572E+01_JPRB, 0.18558E+01_JPRB, 0.19174E+01_JPRB, 0.19488E+01_JPRB, 0.19103E+01_JPRB, 0.18692E+01_JPRB ,&
&  0.18212E+01_JPRB, 0.18461E+01_JPRB, 0.17527E+01_JPRB, 0.18959E+01_JPRB, 0.21038E+01_JPRB, 0.17851E+01_JPRB, 0.13607E+01_JPRB ,&
&  0.11719E+01_JPRB, 0.14380E+01_JPRB, 0.16434E+01_JPRB, 0.17974E+01_JPRB, 0.15161E+01_JPRB, 0.13680E+01_JPRB, 0.12630E+01_JPRB ,&
&  0.11396E+01_JPRB, 0.10282E+01_JPRB, 0.94680E+00_JPRB, 0.84956E+00_JPRB, 0.81162E+00_JPRB, 0.80579E+00_JPRB, 0.81280E+00_JPRB ,&
&  0.79350E+00_JPRB, 0.77822E+00_JPRB, 0.78648E+00_JPRB, 0.83027E+00_JPRB, 0.97578E+00_JPRB, 0.10854E+01_JPRB, 0.18738E+01_JPRB ,&
&  0.10315E+01_JPRB, 0.72569E+00_JPRB, 0.10967E+01_JPRB, 0.49505E+00_JPRB, 0.59514E+00_JPRB, 0.63995E+00_JPRB, 0.68229E+00_JPRB ,&
&  0.64201E+00_JPRB, 0.81548E+00_JPRB, 0.11624E+01_JPRB, 0.11883E+01_JPRB, 0.12457E+01_JPRB, 0.13203E+01_JPRB, 0.80093E+00_JPRB ,&
&  0.65477E+00_JPRB, 0.64643E+00_JPRB, 0.65094E+00_JPRB, 0.60192E+00_JPRB, 0.59581E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.58537E+00_JPRB, 0.64806E+00_JPRB, 0.72208E+00_JPRB, 0.77474E+00_JPRB, 0.76092E+00_JPRB, 0.76537E+00_JPRB, 0.78765E+00_JPRB ,&
&  0.81644E+00_JPRB, 0.85256E+00_JPRB, 0.88929E+00_JPRB, 0.93696E+00_JPRB, 0.96989E+00_JPRB, 0.10115E+01_JPRB, 0.10700E+01_JPRB ,&
&  0.11431E+01_JPRB, 0.11677E+01_JPRB, 0.11981E+01_JPRB, 0.12328E+01_JPRB, 0.12651E+01_JPRB, 0.12988E+01_JPRB, 0.13319E+01_JPRB ,&
&  0.13511E+01_JPRB, 0.13851E+01_JPRB, 0.14109E+01_JPRB, 0.14793E+01_JPRB, 0.15286E+01_JPRB, 0.17112E+01_JPRB, 0.12747E+01_JPRB ,&
&  0.14968E+01_JPRB, 0.14129E+01_JPRB, 0.13376E+01_JPRB, 0.13690E+01_JPRB, 0.13701E+01_JPRB, 0.13589E+01_JPRB, 0.13525E+01_JPRB ,&
&  0.14049E+01_JPRB, 0.14333E+01_JPRB, 0.14569E+01_JPRB, 0.15934E+01_JPRB, 0.15878E+01_JPRB, 0.15963E+01_JPRB, 0.17520E+01_JPRB ,&
&  0.19018E+01_JPRB, 0.20322E+01_JPRB, 0.20830E+01_JPRB, 0.21592E+01_JPRB, 0.21637E+01_JPRB, 0.21219E+01_JPRB, 0.20645E+01_JPRB ,&
&  0.19929E+01_JPRB, 0.19813E+01_JPRB, 0.20509E+01_JPRB, 0.22594E+01_JPRB, 0.23381E+01_JPRB, 0.20713E+01_JPRB, 0.16876E+01_JPRB ,&
&  0.12299E+01_JPRB, 0.10775E+01_JPRB, 0.14688E+01_JPRB, 0.16587E+01_JPRB, 0.14218E+01_JPRB, 0.13036E+01_JPRB, 0.11228E+01_JPRB ,&
&  0.10315E+01_JPRB, 0.91913E+00_JPRB, 0.84998E+00_JPRB, 0.76679E+00_JPRB, 0.72605E+00_JPRB, 0.71929E+00_JPRB, 0.75503E+00_JPRB ,&
&  0.77194E+00_JPRB, 0.75233E+00_JPRB, 0.75071E+00_JPRB, 0.76474E+00_JPRB, 0.91348E+00_JPRB, 0.11756E+01_JPRB, 0.12088E+01_JPRB ,&
&  0.12385E+01_JPRB, 0.87151E+00_JPRB, 0.75356E+00_JPRB, 0.77038E+00_JPRB, 0.58293E+00_JPRB, 0.61448E+00_JPRB, 0.66695E+00_JPRB ,&
&  0.85024E+00_JPRB, 0.93117E+00_JPRB, 0.12647E+01_JPRB, 0.34713E+01_JPRB, 0.39812E+01_JPRB, 0.49775E+01_JPRB, 0.20018E+01_JPRB ,&
&  0.94726E+00_JPRB, 0.64065E+00_JPRB, 0.57508E+00_JPRB, 0.57607E+00_JPRB, 0.56818E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.55519E+00_JPRB, 0.60456E+00_JPRB, 0.64633E+00_JPRB, 0.70602E+00_JPRB, 0.72989E+00_JPRB, 0.73954E+00_JPRB, 0.74279E+00_JPRB ,&
&  0.73480E+00_JPRB, 0.74262E+00_JPRB, 0.76687E+00_JPRB, 0.82584E+00_JPRB, 0.88849E+00_JPRB, 0.95106E+00_JPRB, 0.10205E+01_JPRB ,&
&  0.11091E+01_JPRB, 0.11766E+01_JPRB, 0.11755E+01_JPRB, 0.11950E+01_JPRB, 0.12401E+01_JPRB, 0.12844E+01_JPRB, 0.13057E+01_JPRB ,&
&  0.13184E+01_JPRB, 0.13577E+01_JPRB, 0.14054E+01_JPRB, 0.14780E+01_JPRB, 0.15687E+01_JPRB, 0.16529E+01_JPRB, 0.12159E+01_JPRB ,&
&  0.14543E+01_JPRB, 0.13757E+01_JPRB, 0.13584E+01_JPRB, 0.13812E+01_JPRB, 0.13324E+01_JPRB, 0.12620E+01_JPRB, 0.12708E+01_JPRB ,&
&  0.13581E+01_JPRB, 0.14799E+01_JPRB, 0.16009E+01_JPRB, 0.17022E+01_JPRB, 0.18927E+01_JPRB, 0.19342E+01_JPRB, 0.22209E+01_JPRB ,&
&  0.24669E+01_JPRB, 0.26226E+01_JPRB, 0.26985E+01_JPRB, 0.27707E+01_JPRB, 0.28598E+01_JPRB, 0.29846E+01_JPRB, 0.31290E+01_JPRB ,&
&  0.32171E+01_JPRB, 0.32662E+01_JPRB, 0.33544E+01_JPRB, 0.34462E+01_JPRB, 0.30900E+01_JPRB, 0.32954E+01_JPRB, 0.26386E+01_JPRB ,&
&  0.17330E+01_JPRB, 0.13708E+01_JPRB, 0.15832E+01_JPRB, 0.15692E+01_JPRB, 0.13909E+01_JPRB, 0.12497E+01_JPRB, 0.11048E+01_JPRB ,&
&  0.96116E+00_JPRB, 0.86495E+00_JPRB, 0.79891E+00_JPRB, 0.73674E+00_JPRB, 0.68668E+00_JPRB, 0.67691E+00_JPRB, 0.68600E+00_JPRB ,&
&  0.69462E+00_JPRB, 0.69319E+00_JPRB, 0.69461E+00_JPRB, 0.73679E+00_JPRB, 0.85889E+00_JPRB, 0.11626E+01_JPRB, 0.16328E+01_JPRB ,&
&  0.19184E+01_JPRB, 0.92655E+00_JPRB, 0.74848E+00_JPRB, 0.70309E+00_JPRB, 0.69721E+00_JPRB, 0.72733E+00_JPRB, 0.77657E+00_JPRB ,&
&  0.92678E+00_JPRB, 0.11883E+01_JPRB, 0.10991E+01_JPRB, 0.12871E+01_JPRB, 0.25569E+01_JPRB, 0.27767E+01_JPRB, 0.16106E+01_JPRB ,&
&  0.10254E+01_JPRB, 0.73202E+00_JPRB, 0.59747E+00_JPRB, 0.55595E+00_JPRB, 0.54579E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.56046E+00_JPRB, 0.57595E+00_JPRB, 0.59836E+00_JPRB, 0.62548E+00_JPRB, 0.62474E+00_JPRB, 0.62814E+00_JPRB, 0.61733E+00_JPRB ,&
&  0.61640E+00_JPRB, 0.63668E+00_JPRB, 0.68702E+00_JPRB, 0.77263E+00_JPRB, 0.83642E+00_JPRB, 0.92347E+00_JPRB, 0.10438E+01_JPRB ,&
&  0.10948E+01_JPRB, 0.10901E+01_JPRB, 0.10921E+01_JPRB, 0.11089E+01_JPRB, 0.11382E+01_JPRB, 0.11774E+01_JPRB, 0.12141E+01_JPRB ,&
&  0.12546E+01_JPRB, 0.13391E+01_JPRB, 0.14150E+01_JPRB, 0.15172E+01_JPRB, 0.16159E+01_JPRB, 0.16763E+01_JPRB, 0.17883E+01_JPRB ,&
&  0.10524E+01_JPRB, 0.14323E+01_JPRB, 0.14113E+01_JPRB, 0.13803E+01_JPRB, 0.12042E+01_JPRB, 0.11012E+01_JPRB, 0.11587E+01_JPRB ,&
&  0.13817E+01_JPRB, 0.16154E+01_JPRB, 0.19222E+01_JPRB, 0.19217E+01_JPRB, 0.21673E+01_JPRB, 0.23452E+01_JPRB, 0.26857E+01_JPRB ,&
&  0.28310E+01_JPRB, 0.29707E+01_JPRB, 0.31448E+01_JPRB, 0.33402E+01_JPRB, 0.35491E+01_JPRB, 0.37997E+01_JPRB, 0.40323E+01_JPRB ,&
&  0.42168E+01_JPRB, 0.43094E+01_JPRB, 0.45020E+01_JPRB, 0.40549E+01_JPRB, 0.35370E+01_JPRB, 0.37419E+01_JPRB, 0.36038E+01_JPRB ,&
&  0.29500E+01_JPRB, 0.19978E+01_JPRB, 0.18458E+01_JPRB, 0.17683E+01_JPRB, 0.15086E+01_JPRB, 0.13459E+01_JPRB, 0.11844E+01_JPRB ,&
&  0.99786E+00_JPRB, 0.82991E+00_JPRB, 0.75904E+00_JPRB, 0.70200E+00_JPRB, 0.67174E+00_JPRB, 0.68313E+00_JPRB, 0.71545E+00_JPRB ,&
&  0.72388E+00_JPRB, 0.72155E+00_JPRB, 0.71240E+00_JPRB, 0.73104E+00_JPRB, 0.79262E+00_JPRB, 0.85161E+00_JPRB, 0.89988E+00_JPRB ,&
&  0.86380E+00_JPRB, 0.82058E+00_JPRB, 0.80165E+00_JPRB, 0.80584E+00_JPRB, 0.81615E+00_JPRB, 0.81921E+00_JPRB, 0.84655E+00_JPRB ,&
&  0.85709E+00_JPRB, 0.84958E+00_JPRB, 0.93585E+00_JPRB, 0.11389E+01_JPRB, 0.12198E+01_JPRB, 0.12934E+01_JPRB, 0.10714E+01_JPRB ,&
&  0.86255E+00_JPRB, 0.73239E+00_JPRB, 0.64845E+00_JPRB, 0.59359E+00_JPRB, 0.54961E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.55320E+00_JPRB, 0.55271E+00_JPRB, 0.55103E+00_JPRB, 0.56410E+00_JPRB, 0.56490E+00_JPRB, 0.56749E+00_JPRB, 0.56477E+00_JPRB ,&
&  0.59128E+00_JPRB, 0.64330E+00_JPRB, 0.72312E+00_JPRB, 0.81463E+00_JPRB, 0.88293E+00_JPRB, 0.96584E+00_JPRB, 0.10110E+01_JPRB ,&
&  0.10168E+01_JPRB, 0.10073E+01_JPRB, 0.10109E+01_JPRB, 0.10412E+01_JPRB, 0.10663E+01_JPRB, 0.11068E+01_JPRB, 0.11523E+01_JPRB ,&
&  0.12019E+01_JPRB, 0.12728E+01_JPRB, 0.13659E+01_JPRB, 0.14777E+01_JPRB, 0.16063E+01_JPRB, 0.17040E+01_JPRB, 0.17840E+01_JPRB ,&
&  0.70583E+00_JPRB, 0.96770E+00_JPRB, 0.13491E+01_JPRB, 0.14129E+01_JPRB, 0.10990E+01_JPRB, 0.98644E+00_JPRB, 0.93243E+00_JPRB ,&
&  0.11486E+01_JPRB, 0.15105E+01_JPRB, 0.18109E+01_JPRB, 0.21983E+01_JPRB, 0.23116E+01_JPRB, 0.26010E+01_JPRB, 0.28184E+01_JPRB ,&
&  0.29002E+01_JPRB, 0.30128E+01_JPRB, 0.32118E+01_JPRB, 0.34576E+01_JPRB, 0.36992E+01_JPRB, 0.39220E+01_JPRB, 0.41605E+01_JPRB ,&
&  0.44180E+01_JPRB, 0.46012E+01_JPRB, 0.48079E+01_JPRB, 0.40854E+01_JPRB, 0.42626E+01_JPRB, 0.45293E+01_JPRB, 0.51064E+01_JPRB ,&
&  0.41551E+01_JPRB, 0.25096E+01_JPRB, 0.18949E+01_JPRB, 0.18894E+01_JPRB, 0.16286E+01_JPRB, 0.14705E+01_JPRB, 0.12507E+01_JPRB ,&
&  0.10290E+01_JPRB, 0.87462E+00_JPRB, 0.79751E+00_JPRB, 0.77382E+00_JPRB, 0.76854E+00_JPRB, 0.78809E+00_JPRB, 0.80646E+00_JPRB ,&
&  0.80446E+00_JPRB, 0.80594E+00_JPRB, 0.79690E+00_JPRB, 0.79410E+00_JPRB, 0.84385E+00_JPRB, 0.89189E+00_JPRB, 0.91591E+00_JPRB ,&
&  0.91521E+00_JPRB, 0.91021E+00_JPRB, 0.91704E+00_JPRB, 0.93357E+00_JPRB, 0.95756E+00_JPRB, 0.96434E+00_JPRB, 0.96235E+00_JPRB ,&
&  0.86638E+00_JPRB, 0.81643E+00_JPRB, 0.80622E+00_JPRB, 0.79176E+00_JPRB, 0.79238E+00_JPRB, 0.82195E+00_JPRB, 0.77380E+00_JPRB ,&
&  0.74415E+00_JPRB, 0.73340E+00_JPRB, 0.70425E+00_JPRB, 0.63774E+00_JPRB, 0.56880E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.57461E+00_JPRB, 0.57138E+00_JPRB, 0.56158E+00_JPRB, 0.56347E+00_JPRB, 0.55613E+00_JPRB, 0.56404E+00_JPRB, 0.57937E+00_JPRB ,&
&  0.61152E+00_JPRB, 0.66987E+00_JPRB, 0.74360E+00_JPRB, 0.80962E+00_JPRB, 0.88551E+00_JPRB, 0.94577E+00_JPRB, 0.95600E+00_JPRB ,&
&  0.94600E+00_JPRB, 0.94966E+00_JPRB, 0.94822E+00_JPRB, 0.98002E+00_JPRB, 0.10104E+01_JPRB, 0.10539E+01_JPRB, 0.11082E+01_JPRB ,&
&  0.11745E+01_JPRB, 0.12605E+01_JPRB, 0.13689E+01_JPRB, 0.14979E+01_JPRB, 0.16750E+01_JPRB, 0.18810E+01_JPRB, 0.20967E+01_JPRB ,&
&  0.14682E+01_JPRB, 0.77999E+00_JPRB, 0.13439E+01_JPRB, 0.15033E+01_JPRB, 0.11666E+01_JPRB, 0.10374E+01_JPRB, 0.88949E+00_JPRB ,&
&  0.89559E+00_JPRB, 0.13385E+01_JPRB, 0.17048E+01_JPRB, 0.20753E+01_JPRB, 0.22518E+01_JPRB, 0.25180E+01_JPRB, 0.27135E+01_JPRB ,&
&  0.28176E+01_JPRB, 0.29333E+01_JPRB, 0.30714E+01_JPRB, 0.32302E+01_JPRB, 0.34281E+01_JPRB, 0.36319E+01_JPRB, 0.38837E+01_JPRB ,&
&  0.41871E+01_JPRB, 0.44293E+01_JPRB, 0.46092E+01_JPRB, 0.39874E+01_JPRB, 0.42971E+01_JPRB, 0.41067E+01_JPRB, 0.39010E+01_JPRB ,&
&  0.34137E+01_JPRB, 0.20905E+01_JPRB, 0.15514E+01_JPRB, 0.18270E+01_JPRB, 0.16595E+01_JPRB, 0.12543E+01_JPRB, 0.13002E+01_JPRB ,&
&  0.11173E+01_JPRB, 0.95610E+00_JPRB, 0.91910E+00_JPRB, 0.92300E+00_JPRB, 0.88437E+00_JPRB, 0.87571E+00_JPRB, 0.86425E+00_JPRB ,&
&  0.89471E+00_JPRB, 0.88962E+00_JPRB, 0.87216E+00_JPRB, 0.88374E+00_JPRB, 0.91452E+00_JPRB, 0.95738E+00_JPRB, 0.98636E+00_JPRB ,&
&  0.10114E+01_JPRB, 0.10270E+01_JPRB, 0.10360E+01_JPRB, 0.10577E+01_JPRB, 0.10736E+01_JPRB, 0.10082E+01_JPRB, 0.97947E+00_JPRB ,&
&  0.90642E+00_JPRB, 0.88407E+00_JPRB, 0.86453E+00_JPRB, 0.84758E+00_JPRB, 0.81779E+00_JPRB, 0.82139E+00_JPRB, 0.84838E+00_JPRB ,&
&  0.87851E+00_JPRB, 0.88277E+00_JPRB, 0.81119E+00_JPRB, 0.68068E+00_JPRB, 0.60050E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.58011E+00_JPRB, 0.56416E+00_JPRB, 0.55474E+00_JPRB, 0.57080E+00_JPRB, 0.56850E+00_JPRB, 0.57566E+00_JPRB, 0.59554E+00_JPRB ,&
&  0.62792E+00_JPRB, 0.67182E+00_JPRB, 0.70777E+00_JPRB, 0.75764E+00_JPRB, 0.82280E+00_JPRB, 0.88358E+00_JPRB, 0.89534E+00_JPRB ,&
&  0.88717E+00_JPRB, 0.88612E+00_JPRB, 0.89649E+00_JPRB, 0.91286E+00_JPRB, 0.94927E+00_JPRB, 0.99255E+00_JPRB, 0.10387E+01_JPRB ,&
&  0.11054E+01_JPRB, 0.11986E+01_JPRB, 0.12991E+01_JPRB, 0.14151E+01_JPRB, 0.15664E+01_JPRB, 0.18101E+01_JPRB, 0.21620E+01_JPRB ,&
&  0.31257E+01_JPRB, 0.31526E+01_JPRB, 0.11863E+01_JPRB, 0.15222E+01_JPRB, 0.13616E+01_JPRB, 0.11701E+01_JPRB, 0.94084E+00_JPRB ,&
&  0.88676E+00_JPRB, 0.11376E+01_JPRB, 0.15366E+01_JPRB, 0.19427E+01_JPRB, 0.19925E+01_JPRB, 0.22070E+01_JPRB, 0.23902E+01_JPRB ,&
&  0.25988E+01_JPRB, 0.27417E+01_JPRB, 0.28162E+01_JPRB, 0.28540E+01_JPRB, 0.29458E+01_JPRB, 0.30906E+01_JPRB, 0.33487E+01_JPRB ,&
&  0.37428E+01_JPRB, 0.41824E+01_JPRB, 0.45261E+01_JPRB, 0.42159E+01_JPRB, 0.44262E+01_JPRB, 0.38558E+01_JPRB, 0.33867E+01_JPRB ,&
&  0.25667E+01_JPRB, 0.21162E+01_JPRB, 0.19282E+01_JPRB, 0.18914E+01_JPRB, 0.16118E+01_JPRB, 0.14130E+01_JPRB, 0.14811E+01_JPRB ,&
&  0.13524E+01_JPRB, 0.11816E+01_JPRB, 0.10728E+01_JPRB, 0.10066E+01_JPRB, 0.97829E+00_JPRB, 0.95369E+00_JPRB, 0.95019E+00_JPRB ,&
&  0.97518E+00_JPRB, 0.98127E+00_JPRB, 0.95721E+00_JPRB, 0.98248E+00_JPRB, 0.10086E+01_JPRB, 0.10406E+01_JPRB, 0.10643E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.11154E+01_JPRB, 0.11269E+01_JPRB, 0.11258E+01_JPRB, 0.10565E+01_JPRB, 0.98715E+00_JPRB, 0.99014E+00_JPRB ,&
&  0.94335E+00_JPRB, 0.94900E+00_JPRB, 0.96185E+00_JPRB, 0.97077E+00_JPRB, 0.94753E+00_JPRB, 0.96073E+00_JPRB, 0.96287E+00_JPRB ,&
&  0.99551E+00_JPRB, 0.10230E+01_JPRB, 0.92730E+00_JPRB, 0.71208E+00_JPRB, 0.62028E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.62592E+00_JPRB, 0.60791E+00_JPRB, 0.60266E+00_JPRB, 0.61029E+00_JPRB, 0.61360E+00_JPRB, 0.60102E+00_JPRB, 0.60232E+00_JPRB ,&
&  0.61486E+00_JPRB, 0.63363E+00_JPRB, 0.63415E+00_JPRB, 0.66797E+00_JPRB, 0.74514E+00_JPRB, 0.82348E+00_JPRB, 0.83281E+00_JPRB ,&
&  0.82489E+00_JPRB, 0.81434E+00_JPRB, 0.81028E+00_JPRB, 0.82809E+00_JPRB, 0.84901E+00_JPRB, 0.90180E+00_JPRB, 0.95730E+00_JPRB ,&
&  0.10147E+01_JPRB, 0.10778E+01_JPRB, 0.11320E+01_JPRB, 0.11805E+01_JPRB, 0.12741E+01_JPRB, 0.14338E+01_JPRB, 0.18592E+01_JPRB ,&
&  0.26429E+01_JPRB, 0.49427E+01_JPRB, 0.14611E+01_JPRB, 0.15069E+01_JPRB, 0.14694E+01_JPRB, 0.13139E+01_JPRB, 0.11589E+01_JPRB ,&
&  0.11205E+01_JPRB, 0.10684E+01_JPRB, 0.13862E+01_JPRB, 0.17735E+01_JPRB, 0.17327E+01_JPRB, 0.18012E+01_JPRB, 0.19137E+01_JPRB ,&
&  0.21802E+01_JPRB, 0.23771E+01_JPRB, 0.25045E+01_JPRB, 0.24969E+01_JPRB, 0.24707E+01_JPRB, 0.24487E+01_JPRB, 0.25188E+01_JPRB ,&
&  0.28398E+01_JPRB, 0.34546E+01_JPRB, 0.41201E+01_JPRB, 0.42777E+01_JPRB, 0.46331E+01_JPRB, 0.40495E+01_JPRB, 0.36517E+01_JPRB ,&
&  0.29607E+01_JPRB, 0.26297E+01_JPRB, 0.25440E+01_JPRB, 0.22857E+01_JPRB, 0.18535E+01_JPRB, 0.16535E+01_JPRB, 0.16468E+01_JPRB ,&
&  0.15497E+01_JPRB, 0.13811E+01_JPRB, 0.12085E+01_JPRB, 0.10790E+01_JPRB, 0.10512E+01_JPRB, 0.10727E+01_JPRB, 0.10448E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.11125E+01_JPRB, 0.10862E+01_JPRB, 0.11275E+01_JPRB, 0.11447E+01_JPRB, 0.11474E+01_JPRB, 0.11461E+01_JPRB ,&
&  0.11450E+01_JPRB, 0.11689E+01_JPRB, 0.11734E+01_JPRB, 0.11225E+01_JPRB, 0.10219E+01_JPRB, 0.95877E+00_JPRB, 0.95917E+00_JPRB ,&
&  0.96371E+00_JPRB, 0.99431E+00_JPRB, 0.10087E+01_JPRB, 0.98246E+00_JPRB, 0.10212E+01_JPRB, 0.10135E+01_JPRB, 0.10115E+01_JPRB ,&
&  0.10320E+01_JPRB, 0.10895E+01_JPRB, 0.11202E+01_JPRB, 0.81201E+00_JPRB, 0.68920E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.71108E+00_JPRB, 0.68270E+00_JPRB, 0.65879E+00_JPRB, 0.63865E+00_JPRB, 0.62553E+00_JPRB, 0.62871E+00_JPRB, 0.63309E+00_JPRB ,&
&  0.63034E+00_JPRB, 0.63431E+00_JPRB, 0.63038E+00_JPRB, 0.63489E+00_JPRB, 0.65650E+00_JPRB, 0.70234E+00_JPRB, 0.75324E+00_JPRB ,&
&  0.75713E+00_JPRB, 0.72838E+00_JPRB, 0.72380E+00_JPRB, 0.72057E+00_JPRB, 0.74221E+00_JPRB, 0.77274E+00_JPRB, 0.82869E+00_JPRB ,&
&  0.90487E+00_JPRB, 0.96815E+00_JPRB, 0.99690E+00_JPRB, 0.10058E+01_JPRB, 0.10352E+01_JPRB, 0.10927E+01_JPRB, 0.14696E+01_JPRB ,&
&  0.22781E+01_JPRB, 0.47485E+01_JPRB, 0.33960E+01_JPRB, 0.23223E+01_JPRB, 0.16068E+01_JPRB, 0.13616E+01_JPRB, 0.13228E+01_JPRB ,&
&  0.14600E+01_JPRB, 0.14144E+01_JPRB, 0.15529E+01_JPRB, 0.15901E+01_JPRB, 0.15135E+01_JPRB, 0.14368E+01_JPRB, 0.15070E+01_JPRB ,&
&  0.17176E+01_JPRB, 0.19243E+01_JPRB, 0.20745E+01_JPRB, 0.21136E+01_JPRB, 0.20682E+01_JPRB, 0.20232E+01_JPRB, 0.19898E+01_JPRB ,&
&  0.19860E+01_JPRB, 0.22376E+01_JPRB, 0.31403E+01_JPRB, 0.41523E+01_JPRB, 0.47471E+01_JPRB, 0.43812E+01_JPRB, 0.41252E+01_JPRB ,&
&  0.39809E+01_JPRB, 0.34824E+01_JPRB, 0.31559E+01_JPRB, 0.26602E+01_JPRB, 0.20849E+01_JPRB, 0.21068E+01_JPRB, 0.17779E+01_JPRB ,&
&  0.16341E+01_JPRB, 0.15042E+01_JPRB, 0.13506E+01_JPRB, 0.11858E+01_JPRB, 0.11677E+01_JPRB, 0.11705E+01_JPRB, 0.11327E+01_JPRB ,&
&  0.11618E+01_JPRB, 0.12015E+01_JPRB, 0.11784E+01_JPRB, 0.11851E+01_JPRB, 0.11868E+01_JPRB, 0.11942E+01_JPRB, 0.11930E+01_JPRB ,&
&  0.11886E+01_JPRB, 0.12096E+01_JPRB, 0.11358E+01_JPRB, 0.10583E+01_JPRB, 0.10104E+01_JPRB, 0.95431E+00_JPRB, 0.93585E+00_JPRB ,&
&  0.97087E+00_JPRB, 0.10422E+01_JPRB, 0.10490E+01_JPRB, 0.10542E+01_JPRB, 0.10369E+01_JPRB, 0.10387E+01_JPRB, 0.99880E+00_JPRB ,&
&  0.98357E+00_JPRB, 0.95931E+00_JPRB, 0.96753E+00_JPRB, 0.88549E+00_JPRB, 0.77893E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.76889E+00_JPRB, 0.74011E+00_JPRB, 0.68105E+00_JPRB, 0.63230E+00_JPRB, 0.62212E+00_JPRB, 0.62806E+00_JPRB, 0.64113E+00_JPRB ,&
&  0.64822E+00_JPRB, 0.65150E+00_JPRB, 0.64934E+00_JPRB, 0.64153E+00_JPRB, 0.64437E+00_JPRB, 0.62613E+00_JPRB, 0.64767E+00_JPRB ,&
&  0.67754E+00_JPRB, 0.68238E+00_JPRB, 0.69618E+00_JPRB, 0.67438E+00_JPRB, 0.67373E+00_JPRB, 0.67005E+00_JPRB, 0.69968E+00_JPRB ,&
&  0.76741E+00_JPRB, 0.85291E+00_JPRB, 0.90396E+00_JPRB, 0.91811E+00_JPRB, 0.93356E+00_JPRB, 0.99159E+00_JPRB, 0.12372E+01_JPRB ,&
&  0.19099E+01_JPRB, 0.40783E+01_JPRB, 0.19508E+01_JPRB, 0.28341E+01_JPRB, 0.19188E+01_JPRB, 0.14719E+01_JPRB, 0.12150E+01_JPRB ,&
&  0.13030E+01_JPRB, 0.15817E+01_JPRB, 0.14138E+01_JPRB, 0.13714E+01_JPRB, 0.13153E+01_JPRB, 0.12771E+01_JPRB, 0.12720E+01_JPRB ,&
&  0.13395E+01_JPRB, 0.15152E+01_JPRB, 0.16905E+01_JPRB, 0.17700E+01_JPRB, 0.17267E+01_JPRB, 0.16275E+01_JPRB, 0.15986E+01_JPRB ,&
&  0.15849E+01_JPRB, 0.16113E+01_JPRB, 0.19950E+01_JPRB, 0.26250E+01_JPRB, 0.41121E+01_JPRB, 0.46439E+01_JPRB, 0.47883E+01_JPRB ,&
&  0.49025E+01_JPRB, 0.44685E+01_JPRB, 0.37330E+01_JPRB, 0.30598E+01_JPRB, 0.24846E+01_JPRB, 0.23582E+01_JPRB, 0.19875E+01_JPRB ,&
&  0.17525E+01_JPRB, 0.15935E+01_JPRB, 0.14486E+01_JPRB, 0.13175E+01_JPRB, 0.12486E+01_JPRB, 0.12271E+01_JPRB, 0.11952E+01_JPRB ,&
&  0.12379E+01_JPRB, 0.12942E+01_JPRB, 0.12757E+01_JPRB, 0.12309E+01_JPRB, 0.12084E+01_JPRB, 0.12049E+01_JPRB, 0.11961E+01_JPRB ,&
&  0.12001E+01_JPRB, 0.12265E+01_JPRB, 0.11512E+01_JPRB, 0.10379E+01_JPRB, 0.10056E+01_JPRB, 0.99357E+00_JPRB, 0.97921E+00_JPRB ,&
&  0.10388E+01_JPRB, 0.10999E+01_JPRB, 0.10963E+01_JPRB, 0.10560E+01_JPRB, 0.10537E+01_JPRB, 0.10916E+01_JPRB, 0.10257E+01_JPRB ,&
&  0.97685E+00_JPRB, 0.94199E+00_JPRB, 0.91448E+00_JPRB, 0.88324E+00_JPRB, 0.83316E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.77748E+00_JPRB, 0.72433E+00_JPRB, 0.66378E+00_JPRB, 0.63372E+00_JPRB, 0.63569E+00_JPRB, 0.62320E+00_JPRB, 0.63330E+00_JPRB ,&
&  0.65754E+00_JPRB, 0.66115E+00_JPRB, 0.66161E+00_JPRB, 0.65058E+00_JPRB, 0.64017E+00_JPRB, 0.61863E+00_JPRB, 0.60071E+00_JPRB ,&
&  0.62124E+00_JPRB, 0.67677E+00_JPRB, 0.70088E+00_JPRB, 0.68932E+00_JPRB, 0.67448E+00_JPRB, 0.65442E+00_JPRB, 0.65000E+00_JPRB ,&
&  0.67717E+00_JPRB, 0.73516E+00_JPRB, 0.79737E+00_JPRB, 0.82945E+00_JPRB, 0.84871E+00_JPRB, 0.87660E+00_JPRB, 0.10327E+01_JPRB ,&
&  0.15071E+01_JPRB, 0.39999E+01_JPRB, 0.17022E+01_JPRB, 0.29781E+01_JPRB, 0.21708E+01_JPRB, 0.16852E+01_JPRB, 0.12854E+01_JPRB ,&
&  0.12422E+01_JPRB, 0.13664E+01_JPRB, 0.12846E+01_JPRB, 0.12568E+01_JPRB, 0.12076E+01_JPRB, 0.11846E+01_JPRB, 0.11941E+01_JPRB ,&
&  0.12062E+01_JPRB, 0.12281E+01_JPRB, 0.13057E+01_JPRB, 0.13752E+01_JPRB, 0.13845E+01_JPRB, 0.13298E+01_JPRB, 0.12748E+01_JPRB ,&
&  0.12481E+01_JPRB, 0.12566E+01_JPRB, 0.14439E+01_JPRB, 0.19168E+01_JPRB, 0.29496E+01_JPRB, 0.42992E+01_JPRB, 0.57253E+01_JPRB ,&
&  0.62047E+01_JPRB, 0.56772E+01_JPRB, 0.45610E+01_JPRB, 0.37284E+01_JPRB, 0.31197E+01_JPRB, 0.28579E+01_JPRB, 0.24199E+01_JPRB ,&
&  0.20675E+01_JPRB, 0.18274E+01_JPRB, 0.16379E+01_JPRB, 0.15167E+01_JPRB, 0.14074E+01_JPRB, 0.13724E+01_JPRB, 0.13373E+01_JPRB ,&
&  0.13498E+01_JPRB, 0.14317E+01_JPRB, 0.14299E+01_JPRB, 0.13605E+01_JPRB, 0.12682E+01_JPRB, 0.12256E+01_JPRB, 0.11842E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11985E+01_JPRB, 0.11708E+01_JPRB, 0.10875E+01_JPRB, 0.10415E+01_JPRB, 0.10330E+01_JPRB, 0.10326E+01_JPRB ,&
&  0.10992E+01_JPRB, 0.11504E+01_JPRB, 0.11415E+01_JPRB, 0.11179E+01_JPRB, 0.10866E+01_JPRB, 0.11311E+01_JPRB, 0.10606E+01_JPRB ,&
&  0.98695E+00_JPRB, 0.94589E+00_JPRB, 0.90939E+00_JPRB, 0.85869E+00_JPRB, 0.83609E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.79134E+00_JPRB, 0.72439E+00_JPRB, 0.67795E+00_JPRB, 0.64620E+00_JPRB, 0.64503E+00_JPRB, 0.63816E+00_JPRB, 0.63580E+00_JPRB ,&
&  0.65516E+00_JPRB, 0.66710E+00_JPRB, 0.67999E+00_JPRB, 0.66910E+00_JPRB, 0.64727E+00_JPRB, 0.64160E+00_JPRB, 0.62768E+00_JPRB ,&
&  0.62233E+00_JPRB, 0.66244E+00_JPRB, 0.70161E+00_JPRB, 0.71124E+00_JPRB, 0.69443E+00_JPRB, 0.65939E+00_JPRB, 0.64219E+00_JPRB ,&
&  0.64826E+00_JPRB, 0.66791E+00_JPRB, 0.70152E+00_JPRB, 0.73716E+00_JPRB, 0.76530E+00_JPRB, 0.78968E+00_JPRB, 0.83863E+00_JPRB ,&
&  0.10421E+01_JPRB, 0.14414E+01_JPRB, 0.22976E+01_JPRB, 0.28579E+01_JPRB, 0.22977E+01_JPRB, 0.17855E+01_JPRB, 0.14371E+01_JPRB ,&
&  0.13297E+01_JPRB, 0.12634E+01_JPRB, 0.12064E+01_JPRB, 0.11540E+01_JPRB, 0.10980E+01_JPRB, 0.10627E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10576E+01_JPRB, 0.10448E+01_JPRB, 0.10325E+01_JPRB, 0.10385E+01_JPRB, 0.10470E+01_JPRB, 0.10402E+01_JPRB, 0.10225E+01_JPRB ,&
&  0.10168E+01_JPRB, 0.10264E+01_JPRB, 0.11029E+01_JPRB, 0.14409E+01_JPRB, 0.24765E+01_JPRB, 0.38118E+01_JPRB, 0.57775E+01_JPRB ,&
&  0.69960E+01_JPRB, 0.63193E+01_JPRB, 0.52443E+01_JPRB, 0.43550E+01_JPRB, 0.37135E+01_JPRB, 0.33555E+01_JPRB, 0.29819E+01_JPRB ,&
&  0.26302E+01_JPRB, 0.22801E+01_JPRB, 0.19916E+01_JPRB, 0.18127E+01_JPRB, 0.16234E+01_JPRB, 0.15369E+01_JPRB, 0.15393E+01_JPRB ,&
&  0.15012E+01_JPRB, 0.15006E+01_JPRB, 0.15128E+01_JPRB, 0.15640E+01_JPRB, 0.14689E+01_JPRB, 0.13579E+01_JPRB, 0.12559E+01_JPRB ,&
&  0.11792E+01_JPRB, 0.11771E+01_JPRB, 0.11529E+01_JPRB, 0.11343E+01_JPRB, 0.11260E+01_JPRB, 0.11170E+01_JPRB, 0.11115E+01_JPRB ,&
&  0.11271E+01_JPRB, 0.11578E+01_JPRB, 0.11662E+01_JPRB, 0.11585E+01_JPRB, 0.10800E+01_JPRB, 0.11448E+01_JPRB, 0.10746E+01_JPRB ,&
&  0.10047E+01_JPRB, 0.95909E+00_JPRB, 0.92576E+00_JPRB, 0.88489E+00_JPRB, 0.86084E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.86662E+00_JPRB, 0.80105E+00_JPRB, 0.73657E+00_JPRB, 0.69564E+00_JPRB, 0.66490E+00_JPRB, 0.65201E+00_JPRB, 0.64438E+00_JPRB ,&
&  0.65199E+00_JPRB, 0.66034E+00_JPRB, 0.67565E+00_JPRB, 0.67360E+00_JPRB, 0.65506E+00_JPRB, 0.65900E+00_JPRB, 0.67743E+00_JPRB ,&
&  0.67529E+00_JPRB, 0.68491E+00_JPRB, 0.70178E+00_JPRB, 0.69195E+00_JPRB, 0.67176E+00_JPRB, 0.65042E+00_JPRB, 0.63256E+00_JPRB ,&
&  0.63489E+00_JPRB, 0.64063E+00_JPRB, 0.65421E+00_JPRB, 0.67736E+00_JPRB, 0.70883E+00_JPRB, 0.73892E+00_JPRB, 0.76295E+00_JPRB ,&
&  0.82089E+00_JPRB, 0.14572E+01_JPRB, 0.20844E+01_JPRB, 0.24684E+01_JPRB, 0.22588E+01_JPRB, 0.17889E+01_JPRB, 0.15356E+01_JPRB ,&
&  0.13148E+01_JPRB, 0.12045E+01_JPRB, 0.11354E+01_JPRB, 0.10843E+01_JPRB, 0.10182E+01_JPRB, 0.98224E+00_JPRB, 0.97275E+00_JPRB ,&
&  0.95292E+00_JPRB, 0.91451E+00_JPRB, 0.87161E+00_JPRB, 0.83870E+00_JPRB, 0.83350E+00_JPRB, 0.84102E+00_JPRB, 0.85011E+00_JPRB ,&
&  0.85922E+00_JPRB, 0.87412E+00_JPRB, 0.92234E+00_JPRB, 0.10629E+01_JPRB, 0.18315E+01_JPRB, 0.23120E+01_JPRB, 0.43115E+01_JPRB ,&
&  0.53534E+01_JPRB, 0.46299E+01_JPRB, 0.41594E+01_JPRB, 0.38199E+01_JPRB, 0.34516E+01_JPRB, 0.31216E+01_JPRB, 0.29097E+01_JPRB ,&
&  0.26935E+01_JPRB, 0.24494E+01_JPRB, 0.22445E+01_JPRB, 0.20955E+01_JPRB, 0.19525E+01_JPRB, 0.18460E+01_JPRB, 0.17153E+01_JPRB ,&
&  0.16648E+01_JPRB, 0.15957E+01_JPRB, 0.15698E+01_JPRB, 0.16109E+01_JPRB, 0.16361E+01_JPRB, 0.15428E+01_JPRB, 0.14244E+01_JPRB ,&
&  0.13003E+01_JPRB, 0.12301E+01_JPRB, 0.11828E+01_JPRB, 0.11622E+01_JPRB, 0.11761E+01_JPRB, 0.11897E+01_JPRB, 0.11853E+01_JPRB ,&
&  0.11542E+01_JPRB, 0.11472E+01_JPRB, 0.11724E+01_JPRB, 0.11487E+01_JPRB, 0.10789E+01_JPRB, 0.11508E+01_JPRB, 0.10902E+01_JPRB ,&
&  0.10183E+01_JPRB, 0.95605E+00_JPRB, 0.91692E+00_JPRB, 0.87966E+00_JPRB, 0.89735E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.90943E+00_JPRB, 0.86553E+00_JPRB, 0.81193E+00_JPRB, 0.76978E+00_JPRB, 0.73338E+00_JPRB, 0.70828E+00_JPRB, 0.70183E+00_JPRB ,&
&  0.69524E+00_JPRB, 0.68520E+00_JPRB, 0.69138E+00_JPRB, 0.69071E+00_JPRB, 0.68913E+00_JPRB, 0.68788E+00_JPRB, 0.70480E+00_JPRB ,&
&  0.71803E+00_JPRB, 0.71803E+00_JPRB, 0.71224E+00_JPRB, 0.69783E+00_JPRB, 0.68073E+00_JPRB, 0.66081E+00_JPRB, 0.64784E+00_JPRB ,&
&  0.64033E+00_JPRB, 0.64074E+00_JPRB, 0.64736E+00_JPRB, 0.66075E+00_JPRB, 0.68599E+00_JPRB, 0.71827E+00_JPRB, 0.74905E+00_JPRB ,&
&  0.75404E+00_JPRB, 0.99498E+00_JPRB, 0.13485E+01_JPRB, 0.19460E+01_JPRB, 0.20992E+01_JPRB, 0.17956E+01_JPRB, 0.15330E+01_JPRB ,&
&  0.13347E+01_JPRB, 0.11986E+01_JPRB, 0.11062E+01_JPRB, 0.10437E+01_JPRB, 0.97996E+00_JPRB, 0.94616E+00_JPRB, 0.91732E+00_JPRB ,&
&  0.88573E+00_JPRB, 0.84832E+00_JPRB, 0.80402E+00_JPRB, 0.77448E+00_JPRB, 0.76881E+00_JPRB, 0.77704E+00_JPRB, 0.77933E+00_JPRB ,&
&  0.77765E+00_JPRB, 0.78889E+00_JPRB, 0.80496E+00_JPRB, 0.84841E+00_JPRB, 0.11733E+01_JPRB, 0.15351E+01_JPRB, 0.23682E+01_JPRB ,&
&  0.24268E+01_JPRB, 0.23046E+01_JPRB, 0.22891E+01_JPRB, 0.22667E+01_JPRB, 0.23002E+01_JPRB, 0.23186E+01_JPRB, 0.23603E+01_JPRB ,&
&  0.23861E+01_JPRB, 0.23558E+01_JPRB, 0.22794E+01_JPRB, 0.22188E+01_JPRB, 0.21598E+01_JPRB, 0.20936E+01_JPRB, 0.19674E+01_JPRB ,&
&  0.18662E+01_JPRB, 0.17824E+01_JPRB, 0.17252E+01_JPRB, 0.16684E+01_JPRB, 0.16594E+01_JPRB, 0.16147E+01_JPRB, 0.15263E+01_JPRB ,&
&  0.14088E+01_JPRB, 0.12969E+01_JPRB, 0.12498E+01_JPRB, 0.12283E+01_JPRB, 0.12107E+01_JPRB, 0.11877E+01_JPRB, 0.11851E+01_JPRB ,&
&  0.11917E+01_JPRB, 0.11879E+01_JPRB, 0.11734E+01_JPRB, 0.11443E+01_JPRB, 0.10822E+01_JPRB, 0.11407E+01_JPRB, 0.10980E+01_JPRB ,&
&  0.10384E+01_JPRB, 0.97059E+00_JPRB, 0.91111E+00_JPRB, 0.87882E+00_JPRB, 0.92319E+00_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.99527E+00_JPRB, 0.94899E+00_JPRB, 0.89922E+00_JPRB, 0.83592E+00_JPRB, 0.78525E+00_JPRB, 0.76250E+00_JPRB, 0.74878E+00_JPRB ,&
&  0.73007E+00_JPRB, 0.71246E+00_JPRB, 0.71139E+00_JPRB, 0.71854E+00_JPRB, 0.72516E+00_JPRB, 0.72894E+00_JPRB, 0.72463E+00_JPRB ,&
&  0.72376E+00_JPRB, 0.72702E+00_JPRB, 0.71843E+00_JPRB, 0.70407E+00_JPRB, 0.68851E+00_JPRB, 0.67533E+00_JPRB, 0.66518E+00_JPRB ,&
&  0.65935E+00_JPRB, 0.66181E+00_JPRB, 0.66381E+00_JPRB, 0.67458E+00_JPRB, 0.69465E+00_JPRB, 0.71884E+00_JPRB, 0.75004E+00_JPRB ,&
&  0.75985E+00_JPRB, 0.72954E+00_JPRB, 0.96452E+00_JPRB, 0.12912E+01_JPRB, 0.15570E+01_JPRB, 0.15828E+01_JPRB, 0.14553E+01_JPRB ,&
&  0.13439E+01_JPRB, 0.12202E+01_JPRB, 0.11234E+01_JPRB, 0.10443E+01_JPRB, 0.97674E+00_JPRB, 0.92350E+00_JPRB, 0.88519E+00_JPRB ,&
&  0.84604E+00_JPRB, 0.81380E+00_JPRB, 0.78322E+00_JPRB, 0.75629E+00_JPRB, 0.74926E+00_JPRB, 0.74541E+00_JPRB, 0.74150E+00_JPRB ,&
&  0.73917E+00_JPRB, 0.74076E+00_JPRB, 0.74494E+00_JPRB, 0.75607E+00_JPRB, 0.80362E+00_JPRB, 0.91024E+00_JPRB, 0.11480E+01_JPRB ,&
&  0.12770E+01_JPRB, 0.13524E+01_JPRB, 0.13760E+01_JPRB, 0.13775E+01_JPRB, 0.14099E+01_JPRB, 0.14874E+01_JPRB, 0.16087E+01_JPRB ,&
&  0.17560E+01_JPRB, 0.18877E+01_JPRB, 0.19885E+01_JPRB, 0.20502E+01_JPRB, 0.20601E+01_JPRB, 0.20306E+01_JPRB, 0.19727E+01_JPRB ,&
&  0.19087E+01_JPRB, 0.18106E+01_JPRB, 0.17132E+01_JPRB, 0.16328E+01_JPRB, 0.15873E+01_JPRB, 0.15559E+01_JPRB, 0.15018E+01_JPRB ,&
&  0.14225E+01_JPRB, 0.13309E+01_JPRB, 0.12647E+01_JPRB, 0.12288E+01_JPRB, 0.12088E+01_JPRB, 0.11860E+01_JPRB, 0.11798E+01_JPRB ,&
&  0.11749E+01_JPRB, 0.11740E+01_JPRB, 0.11260E+01_JPRB, 0.11128E+01_JPRB, 0.11488E+01_JPRB, 0.11082E+01_JPRB, 0.10703E+01_JPRB ,&
&  0.10231E+01_JPRB, 0.97416E+00_JPRB, 0.92858E+00_JPRB, 0.89177E+00_JPRB, 0.98457E+00_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.92439E+00_JPRB, 0.91949E+00_JPRB, 0.88594E+00_JPRB, 0.83812E+00_JPRB, 0.79233E+00_JPRB, 0.75533E+00_JPRB, 0.73619E+00_JPRB ,&
&  0.72214E+00_JPRB, 0.71316E+00_JPRB, 0.71929E+00_JPRB, 0.73403E+00_JPRB, 0.75138E+00_JPRB, 0.75752E+00_JPRB, 0.75346E+00_JPRB ,&
&  0.74120E+00_JPRB, 0.72129E+00_JPRB, 0.70754E+00_JPRB, 0.69881E+00_JPRB, 0.68762E+00_JPRB, 0.68180E+00_JPRB, 0.67874E+00_JPRB ,&
&  0.68103E+00_JPRB, 0.68575E+00_JPRB, 0.68844E+00_JPRB, 0.69758E+00_JPRB, 0.71327E+00_JPRB, 0.73747E+00_JPRB, 0.76317E+00_JPRB ,&
&  0.76960E+00_JPRB, 0.67382E+00_JPRB, 0.76546E+00_JPRB, 0.96837E+00_JPRB, 0.11848E+01_JPRB, 0.12992E+01_JPRB, 0.13053E+01_JPRB ,&
&  0.12537E+01_JPRB, 0.11928E+01_JPRB, 0.11222E+01_JPRB, 0.10432E+01_JPRB, 0.96926E+00_JPRB, 0.90683E+00_JPRB, 0.85078E+00_JPRB ,&
&  0.80779E+00_JPRB, 0.77734E+00_JPRB, 0.75967E+00_JPRB, 0.74729E+00_JPRB, 0.73709E+00_JPRB, 0.71890E+00_JPRB, 0.70892E+00_JPRB ,&
&  0.70956E+00_JPRB, 0.71221E+00_JPRB, 0.71642E+00_JPRB, 0.72307E+00_JPRB, 0.74730E+00_JPRB, 0.78440E+00_JPRB, 0.83024E+00_JPRB ,&
&  0.87881E+00_JPRB, 0.92490E+00_JPRB, 0.95110E+00_JPRB, 0.96359E+00_JPRB, 0.97187E+00_JPRB, 0.10302E+01_JPRB, 0.10891E+01_JPRB ,&
&  0.11754E+01_JPRB, 0.13143E+01_JPRB, 0.14779E+01_JPRB, 0.15824E+01_JPRB, 0.16044E+01_JPRB, 0.15661E+01_JPRB, 0.14998E+01_JPRB ,&
&  0.14511E+01_JPRB, 0.13976E+01_JPRB, 0.13428E+01_JPRB, 0.12962E+01_JPRB, 0.12712E+01_JPRB, 0.12752E+01_JPRB, 0.12812E+01_JPRB ,&
&  0.12784E+01_JPRB, 0.12553E+01_JPRB, 0.12265E+01_JPRB, 0.12157E+01_JPRB, 0.12115E+01_JPRB, 0.11956E+01_JPRB, 0.11715E+01_JPRB ,&
&  0.11445E+01_JPRB, 0.11205E+01_JPRB, 0.10979E+01_JPRB, 0.11002E+01_JPRB, 0.10657E+01_JPRB, 0.10399E+01_JPRB, 0.10075E+01_JPRB ,&
&  0.97847E+00_JPRB, 0.94555E+00_JPRB, 0.90425E+00_JPRB, 0.86758E+00_JPRB, 0.85770E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.82234E+00_JPRB, 0.85849E+00_JPRB, 0.86402E+00_JPRB, 0.83627E+00_JPRB, 0.80210E+00_JPRB, 0.77796E+00_JPRB, 0.76158E+00_JPRB ,&
&  0.75641E+00_JPRB, 0.75099E+00_JPRB, 0.75499E+00_JPRB, 0.76819E+00_JPRB, 0.78239E+00_JPRB, 0.78582E+00_JPRB, 0.76865E+00_JPRB ,&
&  0.74501E+00_JPRB, 0.72499E+00_JPRB, 0.71357E+00_JPRB, 0.70783E+00_JPRB, 0.70154E+00_JPRB, 0.69821E+00_JPRB, 0.70585E+00_JPRB ,&
&  0.71533E+00_JPRB, 0.72712E+00_JPRB, 0.74036E+00_JPRB, 0.75182E+00_JPRB, 0.76107E+00_JPRB, 0.77035E+00_JPRB, 0.79167E+00_JPRB ,&
&  0.80484E+00_JPRB, 0.63526E+00_JPRB, 0.67004E+00_JPRB, 0.80568E+00_JPRB, 0.94112E+00_JPRB, 0.10985E+01_JPRB, 0.11714E+01_JPRB ,&
&  0.11621E+01_JPRB, 0.11229E+01_JPRB, 0.10689E+01_JPRB, 0.10087E+01_JPRB, 0.94472E+00_JPRB, 0.88791E+00_JPRB, 0.83541E+00_JPRB ,&
&  0.79912E+00_JPRB, 0.77169E+00_JPRB, 0.75457E+00_JPRB, 0.73591E+00_JPRB, 0.72395E+00_JPRB, 0.71520E+00_JPRB, 0.70788E+00_JPRB ,&
&  0.70272E+00_JPRB, 0.69675E+00_JPRB, 0.69825E+00_JPRB, 0.70472E+00_JPRB, 0.70767E+00_JPRB, 0.71775E+00_JPRB, 0.71844E+00_JPRB ,&
&  0.74116E+00_JPRB, 0.76992E+00_JPRB, 0.78458E+00_JPRB, 0.79046E+00_JPRB, 0.80101E+00_JPRB, 0.82196E+00_JPRB, 0.85119E+00_JPRB ,&
&  0.88734E+00_JPRB, 0.95866E+00_JPRB, 0.10374E+01_JPRB, 0.11107E+01_JPRB, 0.11539E+01_JPRB, 0.11612E+01_JPRB, 0.11486E+01_JPRB ,&
&  0.11497E+01_JPRB, 0.11423E+01_JPRB, 0.11265E+01_JPRB, 0.11034E+01_JPRB, 0.10734E+01_JPRB, 0.10594E+01_JPRB, 0.10693E+01_JPRB ,&
&  0.10913E+01_JPRB, 0.11188E+01_JPRB, 0.11343E+01_JPRB, 0.11513E+01_JPRB, 0.11479E+01_JPRB, 0.11249E+01_JPRB, 0.11010E+01_JPRB ,&
&  0.10736E+01_JPRB, 0.10392E+01_JPRB, 0.10083E+01_JPRB, 0.93360E+00_JPRB, 0.97767E+00_JPRB, 0.97172E+00_JPRB, 0.95036E+00_JPRB ,&
&  0.93101E+00_JPRB, 0.89492E+00_JPRB, 0.87002E+00_JPRB, 0.70963E+00_JPRB, 0.78286E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.77996E+00_JPRB, 0.81245E+00_JPRB, 0.82461E+00_JPRB, 0.81836E+00_JPRB, 0.79989E+00_JPRB, 0.78867E+00_JPRB, 0.78051E+00_JPRB ,&
&  0.77509E+00_JPRB, 0.77072E+00_JPRB, 0.77981E+00_JPRB, 0.78687E+00_JPRB, 0.78905E+00_JPRB, 0.78031E+00_JPRB, 0.77009E+00_JPRB ,&
&  0.75437E+00_JPRB, 0.73538E+00_JPRB, 0.72136E+00_JPRB, 0.70936E+00_JPRB, 0.70725E+00_JPRB, 0.71466E+00_JPRB, 0.73029E+00_JPRB ,&
&  0.75081E+00_JPRB, 0.76535E+00_JPRB, 0.77755E+00_JPRB, 0.79084E+00_JPRB, 0.79444E+00_JPRB, 0.79601E+00_JPRB, 0.80075E+00_JPRB ,&
&  0.80896E+00_JPRB, 0.62439E+00_JPRB, 0.66963E+00_JPRB, 0.73360E+00_JPRB, 0.79825E+00_JPRB, 0.89576E+00_JPRB, 0.97713E+00_JPRB ,&
&  0.10056E+01_JPRB, 0.97224E+00_JPRB, 0.94325E+00_JPRB, 0.90876E+00_JPRB, 0.87074E+00_JPRB, 0.83708E+00_JPRB, 0.80042E+00_JPRB ,&
&  0.77276E+00_JPRB, 0.75385E+00_JPRB, 0.74512E+00_JPRB, 0.73716E+00_JPRB, 0.72815E+00_JPRB, 0.71651E+00_JPRB, 0.70293E+00_JPRB ,&
&  0.68914E+00_JPRB, 0.68176E+00_JPRB, 0.67717E+00_JPRB, 0.67370E+00_JPRB, 0.67599E+00_JPRB, 0.68090E+00_JPRB, 0.68377E+00_JPRB ,&
&  0.69388E+00_JPRB, 0.70415E+00_JPRB, 0.71009E+00_JPRB, 0.71130E+00_JPRB, 0.71284E+00_JPRB, 0.71817E+00_JPRB, 0.73834E+00_JPRB ,&
&  0.75678E+00_JPRB, 0.78683E+00_JPRB, 0.82588E+00_JPRB, 0.87932E+00_JPRB, 0.92267E+00_JPRB, 0.94284E+00_JPRB, 0.95181E+00_JPRB ,&
&  0.95451E+00_JPRB, 0.94567E+00_JPRB, 0.92578E+00_JPRB, 0.90609E+00_JPRB, 0.89063E+00_JPRB, 0.88560E+00_JPRB, 0.88468E+00_JPRB ,&
&  0.89459E+00_JPRB, 0.91204E+00_JPRB, 0.92756E+00_JPRB, 0.94759E+00_JPRB, 0.96716E+00_JPRB, 0.96672E+00_JPRB, 0.93902E+00_JPRB ,&
&  0.91447E+00_JPRB, 0.90021E+00_JPRB, 0.88905E+00_JPRB, 0.89905E+00_JPRB, 0.89656E+00_JPRB, 0.88640E+00_JPRB, 0.86345E+00_JPRB ,&
&  0.84649E+00_JPRB, 0.83205E+00_JPRB, 0.63891E+00_JPRB, 0.72516E+00_JPRB, 0.74664E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.77936E+00_JPRB, 0.77610E+00_JPRB, 0.77093E+00_JPRB, 0.77570E+00_JPRB, 0.78080E+00_JPRB, 0.78114E+00_JPRB, 0.78125E+00_JPRB ,&
&  0.77924E+00_JPRB, 0.77586E+00_JPRB, 0.78395E+00_JPRB, 0.79109E+00_JPRB, 0.79156E+00_JPRB, 0.78445E+00_JPRB, 0.77691E+00_JPRB ,&
&  0.77113E+00_JPRB, 0.76215E+00_JPRB, 0.75320E+00_JPRB, 0.74955E+00_JPRB, 0.75200E+00_JPRB, 0.76574E+00_JPRB, 0.78668E+00_JPRB ,&
&  0.80229E+00_JPRB, 0.80148E+00_JPRB, 0.80521E+00_JPRB, 0.81664E+00_JPRB, 0.81976E+00_JPRB, 0.82212E+00_JPRB, 0.82970E+00_JPRB ,&
&  0.71174E+00_JPRB, 0.58727E+00_JPRB, 0.65289E+00_JPRB, 0.71233E+00_JPRB, 0.75897E+00_JPRB, 0.81333E+00_JPRB, 0.87768E+00_JPRB ,&
&  0.89888E+00_JPRB, 0.88491E+00_JPRB, 0.86943E+00_JPRB, 0.84886E+00_JPRB, 0.82379E+00_JPRB, 0.80661E+00_JPRB, 0.79305E+00_JPRB ,&
&  0.78987E+00_JPRB, 0.77907E+00_JPRB, 0.76232E+00_JPRB, 0.74866E+00_JPRB, 0.73705E+00_JPRB, 0.72194E+00_JPRB, 0.70721E+00_JPRB ,&
&  0.69184E+00_JPRB, 0.67859E+00_JPRB, 0.67435E+00_JPRB, 0.67219E+00_JPRB, 0.67439E+00_JPRB, 0.67430E+00_JPRB, 0.66925E+00_JPRB ,&
&  0.66604E+00_JPRB, 0.66821E+00_JPRB, 0.67776E+00_JPRB, 0.68683E+00_JPRB, 0.69069E+00_JPRB, 0.69089E+00_JPRB, 0.70151E+00_JPRB ,&
&  0.71137E+00_JPRB, 0.72821E+00_JPRB, 0.74927E+00_JPRB, 0.78122E+00_JPRB, 0.80673E+00_JPRB, 0.81834E+00_JPRB, 0.82576E+00_JPRB ,&
&  0.83528E+00_JPRB, 0.83078E+00_JPRB, 0.81411E+00_JPRB, 0.80148E+00_JPRB, 0.79539E+00_JPRB, 0.78738E+00_JPRB, 0.78368E+00_JPRB ,&
&  0.78990E+00_JPRB, 0.79706E+00_JPRB, 0.79840E+00_JPRB, 0.80652E+00_JPRB, 0.81856E+00_JPRB, 0.82938E+00_JPRB, 0.83559E+00_JPRB ,&
&  0.81861E+00_JPRB, 0.81980E+00_JPRB, 0.81830E+00_JPRB, 0.82573E+00_JPRB, 0.83386E+00_JPRB, 0.82054E+00_JPRB, 0.80699E+00_JPRB ,&
&  0.79463E+00_JPRB, 0.77087E+00_JPRB, 0.76908E+00_JPRB, 0.76851E+00_JPRB, 0.77041E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.73970E+00_JPRB, 0.73112E+00_JPRB, 0.72562E+00_JPRB, 0.73472E+00_JPRB, 0.75081E+00_JPRB, 0.76366E+00_JPRB, 0.76538E+00_JPRB ,&
&  0.76653E+00_JPRB, 0.77189E+00_JPRB, 0.78051E+00_JPRB, 0.78264E+00_JPRB, 0.77532E+00_JPRB, 0.75822E+00_JPRB, 0.74922E+00_JPRB ,&
&  0.74672E+00_JPRB, 0.74747E+00_JPRB, 0.74725E+00_JPRB, 0.75151E+00_JPRB, 0.77098E+00_JPRB, 0.80149E+00_JPRB, 0.83039E+00_JPRB ,&
&  0.84461E+00_JPRB, 0.83917E+00_JPRB, 0.83299E+00_JPRB, 0.82802E+00_JPRB, 0.82693E+00_JPRB, 0.83198E+00_JPRB, 0.83320E+00_JPRB ,&
&  0.82545E+00_JPRB, 0.67938E+00_JPRB, 0.73604E+00_JPRB, 0.75760E+00_JPRB, 0.78258E+00_JPRB, 0.81211E+00_JPRB, 0.85047E+00_JPRB ,&
&  0.85778E+00_JPRB, 0.84718E+00_JPRB, 0.82636E+00_JPRB, 0.81145E+00_JPRB, 0.79530E+00_JPRB, 0.78988E+00_JPRB, 0.78652E+00_JPRB ,&
&  0.78452E+00_JPRB, 0.78141E+00_JPRB, 0.75946E+00_JPRB, 0.72411E+00_JPRB, 0.70450E+00_JPRB, 0.69237E+00_JPRB, 0.68216E+00_JPRB ,&
&  0.67262E+00_JPRB, 0.65989E+00_JPRB, 0.64754E+00_JPRB, 0.63728E+00_JPRB, 0.63847E+00_JPRB, 0.63647E+00_JPRB, 0.62377E+00_JPRB ,&
&  0.61853E+00_JPRB, 0.61660E+00_JPRB, 0.62452E+00_JPRB, 0.63562E+00_JPRB, 0.64068E+00_JPRB, 0.63710E+00_JPRB, 0.64738E+00_JPRB ,&
&  0.67054E+00_JPRB, 0.68633E+00_JPRB, 0.69989E+00_JPRB, 0.71654E+00_JPRB, 0.72579E+00_JPRB, 0.72717E+00_JPRB, 0.73049E+00_JPRB ,&
&  0.73943E+00_JPRB, 0.73948E+00_JPRB, 0.73170E+00_JPRB, 0.72584E+00_JPRB, 0.72752E+00_JPRB, 0.72893E+00_JPRB, 0.73858E+00_JPRB ,&
&  0.74770E+00_JPRB, 0.75125E+00_JPRB, 0.74793E+00_JPRB, 0.75090E+00_JPRB, 0.75736E+00_JPRB, 0.76797E+00_JPRB, 0.77455E+00_JPRB ,&
&  0.78123E+00_JPRB, 0.77648E+00_JPRB, 0.76283E+00_JPRB, 0.76224E+00_JPRB, 0.76640E+00_JPRB, 0.76306E+00_JPRB, 0.74953E+00_JPRB ,&
&  0.73758E+00_JPRB, 0.72827E+00_JPRB, 0.73015E+00_JPRB, 0.73139E+00_JPRB, 0.73620E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.73072E+00_JPRB, 0.72675E+00_JPRB, 0.72535E+00_JPRB, 0.73622E+00_JPRB, 0.75385E+00_JPRB, 0.75868E+00_JPRB, 0.76097E+00_JPRB ,&
&  0.77408E+00_JPRB, 0.78397E+00_JPRB, 0.78383E+00_JPRB, 0.78232E+00_JPRB, 0.77617E+00_JPRB, 0.76207E+00_JPRB, 0.75115E+00_JPRB ,&
&  0.75617E+00_JPRB, 0.76160E+00_JPRB, 0.76360E+00_JPRB, 0.78744E+00_JPRB, 0.81905E+00_JPRB, 0.85665E+00_JPRB, 0.88215E+00_JPRB ,&
&  0.89124E+00_JPRB, 0.88390E+00_JPRB, 0.88016E+00_JPRB, 0.87485E+00_JPRB, 0.86947E+00_JPRB, 0.87122E+00_JPRB, 0.86574E+00_JPRB ,&
&  0.85586E+00_JPRB, 0.76403E+00_JPRB, 0.79404E+00_JPRB, 0.80371E+00_JPRB, 0.81967E+00_JPRB, 0.84406E+00_JPRB, 0.87307E+00_JPRB ,&
&  0.87374E+00_JPRB, 0.86830E+00_JPRB, 0.85732E+00_JPRB, 0.83578E+00_JPRB, 0.81798E+00_JPRB, 0.80787E+00_JPRB, 0.79091E+00_JPRB ,&
&  0.76356E+00_JPRB, 0.72649E+00_JPRB, 0.69368E+00_JPRB, 0.67129E+00_JPRB, 0.66108E+00_JPRB, 0.65832E+00_JPRB, 0.65374E+00_JPRB ,&
&  0.65169E+00_JPRB, 0.65008E+00_JPRB, 0.63947E+00_JPRB, 0.62990E+00_JPRB, 0.61998E+00_JPRB, 0.61327E+00_JPRB, 0.60025E+00_JPRB ,&
&  0.59219E+00_JPRB, 0.58138E+00_JPRB, 0.58078E+00_JPRB, 0.60098E+00_JPRB, 0.62249E+00_JPRB, 0.62789E+00_JPRB, 0.64155E+00_JPRB ,&
&  0.66010E+00_JPRB, 0.66840E+00_JPRB, 0.67035E+00_JPRB, 0.66769E+00_JPRB, 0.66026E+00_JPRB, 0.65971E+00_JPRB, 0.66809E+00_JPRB ,&
&  0.67602E+00_JPRB, 0.68664E+00_JPRB, 0.69087E+00_JPRB, 0.69587E+00_JPRB, 0.70288E+00_JPRB, 0.70830E+00_JPRB, 0.71918E+00_JPRB ,&
&  0.72480E+00_JPRB, 0.72871E+00_JPRB, 0.73975E+00_JPRB, 0.75119E+00_JPRB, 0.75636E+00_JPRB, 0.75899E+00_JPRB, 0.76852E+00_JPRB ,&
&  0.77250E+00_JPRB, 0.77074E+00_JPRB, 0.76178E+00_JPRB, 0.74899E+00_JPRB, 0.73987E+00_JPRB, 0.72666E+00_JPRB, 0.70649E+00_JPRB ,&
&  0.68842E+00_JPRB, 0.68597E+00_JPRB, 0.69507E+00_JPRB, 0.70503E+00_JPRB, 0.72184E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.71876E+00_JPRB, 0.73006E+00_JPRB, 0.74302E+00_JPRB, 0.75567E+00_JPRB, 0.76891E+00_JPRB, 0.76870E+00_JPRB, 0.77050E+00_JPRB ,&
&  0.79048E+00_JPRB, 0.79418E+00_JPRB, 0.79398E+00_JPRB, 0.78562E+00_JPRB, 0.77942E+00_JPRB, 0.77926E+00_JPRB, 0.77018E+00_JPRB ,&
&  0.76553E+00_JPRB, 0.77165E+00_JPRB, 0.78976E+00_JPRB, 0.81763E+00_JPRB, 0.84486E+00_JPRB, 0.88078E+00_JPRB, 0.90739E+00_JPRB ,&
&  0.91159E+00_JPRB, 0.90072E+00_JPRB, 0.90083E+00_JPRB, 0.90403E+00_JPRB, 0.90409E+00_JPRB, 0.88708E+00_JPRB, 0.88272E+00_JPRB ,&
&  0.86259E+00_JPRB, 0.86978E+00_JPRB, 0.73960E+00_JPRB, 0.80959E+00_JPRB, 0.85380E+00_JPRB, 0.88859E+00_JPRB, 0.89972E+00_JPRB ,&
&  0.89200E+00_JPRB, 0.87910E+00_JPRB, 0.85521E+00_JPRB, 0.82817E+00_JPRB, 0.81844E+00_JPRB, 0.79814E+00_JPRB, 0.77488E+00_JPRB ,&
&  0.72617E+00_JPRB, 0.66394E+00_JPRB, 0.63220E+00_JPRB, 0.62065E+00_JPRB, 0.61492E+00_JPRB, 0.61611E+00_JPRB, 0.62135E+00_JPRB ,&
&  0.62496E+00_JPRB, 0.62549E+00_JPRB, 0.61450E+00_JPRB, 0.59803E+00_JPRB, 0.57821E+00_JPRB, 0.56767E+00_JPRB, 0.56534E+00_JPRB ,&
&  0.56212E+00_JPRB, 0.54899E+00_JPRB, 0.55234E+00_JPRB, 0.58262E+00_JPRB, 0.60803E+00_JPRB, 0.62645E+00_JPRB, 0.63498E+00_JPRB ,&
&  0.64117E+00_JPRB, 0.63537E+00_JPRB, 0.63192E+00_JPRB, 0.62273E+00_JPRB, 0.59906E+00_JPRB, 0.59252E+00_JPRB, 0.61358E+00_JPRB ,&
&  0.63759E+00_JPRB, 0.65139E+00_JPRB, 0.65970E+00_JPRB, 0.67222E+00_JPRB, 0.68561E+00_JPRB, 0.69315E+00_JPRB, 0.69928E+00_JPRB ,&
&  0.70361E+00_JPRB, 0.70791E+00_JPRB, 0.72135E+00_JPRB, 0.73658E+00_JPRB, 0.74367E+00_JPRB, 0.74500E+00_JPRB, 0.74517E+00_JPRB ,&
&  0.74252E+00_JPRB, 0.73505E+00_JPRB, 0.72629E+00_JPRB, 0.71805E+00_JPRB, 0.69804E+00_JPRB, 0.67411E+00_JPRB, 0.65558E+00_JPRB ,&
&  0.64570E+00_JPRB, 0.65538E+00_JPRB, 0.66292E+00_JPRB, 0.67543E+00_JPRB, 0.70202E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.75750E+00_JPRB, 0.77123E+00_JPRB, 0.78261E+00_JPRB, 0.80343E+00_JPRB, 0.82233E+00_JPRB, 0.81967E+00_JPRB, 0.81295E+00_JPRB ,&
&  0.80850E+00_JPRB, 0.80445E+00_JPRB, 0.79772E+00_JPRB, 0.78842E+00_JPRB, 0.78217E+00_JPRB, 0.78173E+00_JPRB, 0.78230E+00_JPRB ,&
&  0.78369E+00_JPRB, 0.80360E+00_JPRB, 0.83642E+00_JPRB, 0.86216E+00_JPRB, 0.89036E+00_JPRB, 0.93357E+00_JPRB, 0.95398E+00_JPRB ,&
&  0.93724E+00_JPRB, 0.93616E+00_JPRB, 0.92527E+00_JPRB, 0.91070E+00_JPRB, 0.91254E+00_JPRB, 0.90330E+00_JPRB, 0.87516E+00_JPRB ,&
&  0.86867E+00_JPRB, 0.86180E+00_JPRB, 0.85759E+00_JPRB, 0.88619E+00_JPRB, 0.90892E+00_JPRB, 0.90934E+00_JPRB, 0.90633E+00_JPRB ,&
&  0.88415E+00_JPRB, 0.84717E+00_JPRB, 0.81189E+00_JPRB, 0.79460E+00_JPRB, 0.79257E+00_JPRB, 0.77247E+00_JPRB, 0.74433E+00_JPRB ,&
&  0.68673E+00_JPRB, 0.63948E+00_JPRB, 0.61335E+00_JPRB, 0.59778E+00_JPRB, 0.59269E+00_JPRB, 0.58994E+00_JPRB, 0.58548E+00_JPRB ,&
&  0.59086E+00_JPRB, 0.58575E+00_JPRB, 0.55793E+00_JPRB, 0.52862E+00_JPRB, 0.52346E+00_JPRB, 0.53251E+00_JPRB, 0.55303E+00_JPRB ,&
&  0.56824E+00_JPRB, 0.56350E+00_JPRB, 0.57046E+00_JPRB, 0.57867E+00_JPRB, 0.59651E+00_JPRB, 0.61447E+00_JPRB, 0.61587E+00_JPRB ,&
&  0.61613E+00_JPRB, 0.61107E+00_JPRB, 0.59800E+00_JPRB, 0.58919E+00_JPRB, 0.57769E+00_JPRB, 0.57288E+00_JPRB, 0.58403E+00_JPRB ,&
&  0.60143E+00_JPRB, 0.61126E+00_JPRB, 0.62202E+00_JPRB, 0.63572E+00_JPRB, 0.64712E+00_JPRB, 0.65562E+00_JPRB, 0.66210E+00_JPRB ,&
&  0.67221E+00_JPRB, 0.68692E+00_JPRB, 0.70205E+00_JPRB, 0.71540E+00_JPRB, 0.71553E+00_JPRB, 0.70122E+00_JPRB, 0.67566E+00_JPRB ,&
&  0.65864E+00_JPRB, 0.64543E+00_JPRB, 0.64251E+00_JPRB, 0.64856E+00_JPRB, 0.64385E+00_JPRB, 0.62429E+00_JPRB, 0.61426E+00_JPRB ,&
&  0.62478E+00_JPRB, 0.64862E+00_JPRB, 0.67129E+00_JPRB, 0.70206E+00_JPRB, 0.73431E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.79433E+00_JPRB, 0.82565E+00_JPRB, 0.84961E+00_JPRB, 0.85798E+00_JPRB, 0.85566E+00_JPRB, 0.84701E+00_JPRB, 0.82475E+00_JPRB ,&
&  0.80053E+00_JPRB, 0.78356E+00_JPRB, 0.76547E+00_JPRB, 0.73914E+00_JPRB, 0.71734E+00_JPRB, 0.71295E+00_JPRB, 0.72564E+00_JPRB ,&
&  0.77192E+00_JPRB, 0.81300E+00_JPRB, 0.82966E+00_JPRB, 0.84340E+00_JPRB, 0.89329E+00_JPRB, 0.93513E+00_JPRB, 0.94417E+00_JPRB ,&
&  0.93886E+00_JPRB, 0.94031E+00_JPRB, 0.90381E+00_JPRB, 0.87069E+00_JPRB, 0.87086E+00_JPRB, 0.87620E+00_JPRB, 0.85780E+00_JPRB ,&
&  0.88333E+00_JPRB, 0.90068E+00_JPRB, 0.89502E+00_JPRB, 0.87107E+00_JPRB, 0.86422E+00_JPRB, 0.85386E+00_JPRB, 0.84163E+00_JPRB ,&
&  0.81961E+00_JPRB, 0.77146E+00_JPRB, 0.74100E+00_JPRB, 0.73900E+00_JPRB, 0.73186E+00_JPRB, 0.72468E+00_JPRB, 0.71127E+00_JPRB ,&
&  0.67114E+00_JPRB, 0.63520E+00_JPRB, 0.60809E+00_JPRB, 0.58947E+00_JPRB, 0.57775E+00_JPRB, 0.56856E+00_JPRB, 0.55887E+00_JPRB ,&
&  0.55047E+00_JPRB, 0.52290E+00_JPRB, 0.49439E+00_JPRB, 0.48797E+00_JPRB, 0.49362E+00_JPRB, 0.50125E+00_JPRB, 0.52935E+00_JPRB ,&
&  0.55448E+00_JPRB, 0.55293E+00_JPRB, 0.54951E+00_JPRB, 0.55236E+00_JPRB, 0.56163E+00_JPRB, 0.56246E+00_JPRB, 0.55248E+00_JPRB ,&
&  0.54752E+00_JPRB, 0.54515E+00_JPRB, 0.53556E+00_JPRB, 0.52837E+00_JPRB, 0.52769E+00_JPRB, 0.53153E+00_JPRB, 0.54568E+00_JPRB ,&
&  0.55438E+00_JPRB, 0.55558E+00_JPRB, 0.56904E+00_JPRB, 0.58608E+00_JPRB, 0.59648E+00_JPRB, 0.60477E+00_JPRB, 0.61085E+00_JPRB ,&
&  0.62460E+00_JPRB, 0.64425E+00_JPRB, 0.65940E+00_JPRB, 0.66163E+00_JPRB, 0.63876E+00_JPRB, 0.61451E+00_JPRB, 0.59550E+00_JPRB ,&
&  0.58717E+00_JPRB, 0.59161E+00_JPRB, 0.59939E+00_JPRB, 0.60696E+00_JPRB, 0.61379E+00_JPRB, 0.61843E+00_JPRB, 0.61785E+00_JPRB ,&
&  0.63749E+00_JPRB, 0.66851E+00_JPRB, 0.69349E+00_JPRB, 0.72324E+00_JPRB, 0.75433E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.86027E+00_JPRB, 0.87769E+00_JPRB, 0.88923E+00_JPRB, 0.88970E+00_JPRB, 0.87400E+00_JPRB, 0.84724E+00_JPRB, 0.82443E+00_JPRB ,&
&  0.79846E+00_JPRB, 0.77322E+00_JPRB, 0.74926E+00_JPRB, 0.71647E+00_JPRB, 0.69886E+00_JPRB, 0.70236E+00_JPRB, 0.75146E+00_JPRB ,&
&  0.81691E+00_JPRB, 0.82196E+00_JPRB, 0.81614E+00_JPRB, 0.84154E+00_JPRB, 0.89400E+00_JPRB, 0.92577E+00_JPRB, 0.94655E+00_JPRB ,&
&  0.94884E+00_JPRB, 0.91553E+00_JPRB, 0.88256E+00_JPRB, 0.86949E+00_JPRB, 0.89313E+00_JPRB, 0.91078E+00_JPRB, 0.91553E+00_JPRB ,&
&  0.91097E+00_JPRB, 0.89007E+00_JPRB, 0.87159E+00_JPRB, 0.85590E+00_JPRB, 0.86018E+00_JPRB, 0.83513E+00_JPRB, 0.78272E+00_JPRB ,&
&  0.71626E+00_JPRB, 0.68970E+00_JPRB, 0.67804E+00_JPRB, 0.66362E+00_JPRB, 0.65284E+00_JPRB, 0.65322E+00_JPRB, 0.64431E+00_JPRB ,&
&  0.63083E+00_JPRB, 0.61498E+00_JPRB, 0.59973E+00_JPRB, 0.58188E+00_JPRB, 0.56778E+00_JPRB, 0.54778E+00_JPRB, 0.52904E+00_JPRB ,&
&  0.51166E+00_JPRB, 0.49002E+00_JPRB, 0.47010E+00_JPRB, 0.45611E+00_JPRB, 0.44989E+00_JPRB, 0.46071E+00_JPRB, 0.50145E+00_JPRB ,&
&  0.53311E+00_JPRB, 0.53016E+00_JPRB, 0.52791E+00_JPRB, 0.53223E+00_JPRB, 0.52544E+00_JPRB, 0.51524E+00_JPRB, 0.50960E+00_JPRB ,&
&  0.50437E+00_JPRB, 0.50034E+00_JPRB, 0.49862E+00_JPRB, 0.50056E+00_JPRB, 0.50280E+00_JPRB, 0.50095E+00_JPRB, 0.50307E+00_JPRB ,&
&  0.50732E+00_JPRB, 0.51243E+00_JPRB, 0.52946E+00_JPRB, 0.55389E+00_JPRB, 0.57477E+00_JPRB, 0.58987E+00_JPRB, 0.60384E+00_JPRB ,&
&  0.62517E+00_JPRB, 0.63824E+00_JPRB, 0.62575E+00_JPRB, 0.61399E+00_JPRB, 0.60807E+00_JPRB, 0.61344E+00_JPRB, 0.62405E+00_JPRB ,&
&  0.64466E+00_JPRB, 0.65734E+00_JPRB, 0.66353E+00_JPRB, 0.66493E+00_JPRB, 0.66805E+00_JPRB, 0.66888E+00_JPRB, 0.68146E+00_JPRB ,&
&  0.71531E+00_JPRB, 0.74445E+00_JPRB, 0.75160E+00_JPRB, 0.78397E+00_JPRB, 0.82548E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.89892E+00_JPRB, 0.90292E+00_JPRB, 0.89289E+00_JPRB, 0.88838E+00_JPRB, 0.87775E+00_JPRB, 0.86408E+00_JPRB, 0.84183E+00_JPRB ,&
&  0.81573E+00_JPRB, 0.78769E+00_JPRB, 0.76102E+00_JPRB, 0.73915E+00_JPRB, 0.74297E+00_JPRB, 0.77552E+00_JPRB, 0.80306E+00_JPRB ,&
&  0.79228E+00_JPRB, 0.78857E+00_JPRB, 0.80575E+00_JPRB, 0.83642E+00_JPRB, 0.86967E+00_JPRB, 0.90026E+00_JPRB, 0.92198E+00_JPRB ,&
&  0.93952E+00_JPRB, 0.92129E+00_JPRB, 0.92539E+00_JPRB, 0.92777E+00_JPRB, 0.92173E+00_JPRB, 0.90351E+00_JPRB, 0.87449E+00_JPRB ,&
&  0.84589E+00_JPRB, 0.81866E+00_JPRB, 0.78505E+00_JPRB, 0.72676E+00_JPRB, 0.79981E+00_JPRB, 0.72424E+00_JPRB, 0.69600E+00_JPRB ,&
&  0.66857E+00_JPRB, 0.65024E+00_JPRB, 0.62968E+00_JPRB, 0.60748E+00_JPRB, 0.59992E+00_JPRB, 0.59544E+00_JPRB, 0.58754E+00_JPRB ,&
&  0.58099E+00_JPRB, 0.57372E+00_JPRB, 0.56264E+00_JPRB, 0.54873E+00_JPRB, 0.53571E+00_JPRB, 0.52184E+00_JPRB, 0.50905E+00_JPRB ,&
&  0.49042E+00_JPRB, 0.46304E+00_JPRB, 0.43423E+00_JPRB, 0.41299E+00_JPRB, 0.40190E+00_JPRB, 0.41138E+00_JPRB, 0.44480E+00_JPRB ,&
&  0.47903E+00_JPRB, 0.48484E+00_JPRB, 0.49464E+00_JPRB, 0.49761E+00_JPRB, 0.48896E+00_JPRB, 0.47844E+00_JPRB, 0.46051E+00_JPRB ,&
&  0.44711E+00_JPRB, 0.44506E+00_JPRB, 0.44617E+00_JPRB, 0.45030E+00_JPRB, 0.45540E+00_JPRB, 0.46246E+00_JPRB, 0.47232E+00_JPRB ,&
&  0.48133E+00_JPRB, 0.48458E+00_JPRB, 0.49695E+00_JPRB, 0.52375E+00_JPRB, 0.55485E+00_JPRB, 0.58643E+00_JPRB, 0.61359E+00_JPRB ,&
&  0.63223E+00_JPRB, 0.64541E+00_JPRB, 0.65382E+00_JPRB, 0.66902E+00_JPRB, 0.68473E+00_JPRB, 0.70045E+00_JPRB, 0.70853E+00_JPRB ,&
&  0.70607E+00_JPRB, 0.70988E+00_JPRB, 0.72738E+00_JPRB, 0.75581E+00_JPRB, 0.77562E+00_JPRB, 0.80941E+00_JPRB, 0.83049E+00_JPRB ,&
&  0.85277E+00_JPRB, 0.85836E+00_JPRB, 0.85280E+00_JPRB, 0.86518E+00_JPRB, 0.88174E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.88370E+00_JPRB, 0.89411E+00_JPRB, 0.89184E+00_JPRB, 0.89067E+00_JPRB, 0.89233E+00_JPRB, 0.88531E+00_JPRB, 0.87150E+00_JPRB ,&
&  0.85752E+00_JPRB, 0.83103E+00_JPRB, 0.80045E+00_JPRB, 0.78661E+00_JPRB, 0.79384E+00_JPRB, 0.79354E+00_JPRB, 0.77499E+00_JPRB ,&
&  0.77346E+00_JPRB, 0.78728E+00_JPRB, 0.81029E+00_JPRB, 0.83584E+00_JPRB, 0.87012E+00_JPRB, 0.90565E+00_JPRB, 0.93210E+00_JPRB ,&
&  0.94262E+00_JPRB, 0.94088E+00_JPRB, 0.93330E+00_JPRB, 0.90660E+00_JPRB, 0.87077E+00_JPRB, 0.84321E+00_JPRB, 0.82228E+00_JPRB ,&
&  0.79643E+00_JPRB, 0.74792E+00_JPRB, 0.67902E+00_JPRB, 0.67329E+00_JPRB, 0.61039E+00_JPRB, 0.59120E+00_JPRB, 0.61371E+00_JPRB ,&
&  0.62913E+00_JPRB, 0.61126E+00_JPRB, 0.58728E+00_JPRB, 0.56739E+00_JPRB, 0.56350E+00_JPRB, 0.56209E+00_JPRB, 0.55694E+00_JPRB ,&
&  0.55304E+00_JPRB, 0.54628E+00_JPRB, 0.53215E+00_JPRB, 0.51924E+00_JPRB, 0.50707E+00_JPRB, 0.49486E+00_JPRB, 0.48639E+00_JPRB ,&
&  0.45780E+00_JPRB, 0.41420E+00_JPRB, 0.38167E+00_JPRB, 0.36736E+00_JPRB, 0.35938E+00_JPRB, 0.36312E+00_JPRB, 0.37620E+00_JPRB ,&
&  0.38699E+00_JPRB, 0.38794E+00_JPRB, 0.38250E+00_JPRB, 0.36952E+00_JPRB, 0.34882E+00_JPRB, 0.30440E+00_JPRB, 0.27855E+00_JPRB ,&
&  0.29480E+00_JPRB, 0.32748E+00_JPRB, 0.36172E+00_JPRB, 0.37254E+00_JPRB, 0.38177E+00_JPRB, 0.38369E+00_JPRB, 0.36859E+00_JPRB ,&
&  0.33979E+00_JPRB, 0.31108E+00_JPRB, 0.30587E+00_JPRB, 0.30809E+00_JPRB, 0.31325E+00_JPRB, 0.32244E+00_JPRB, 0.33849E+00_JPRB ,&
&  0.36272E+00_JPRB, 0.40063E+00_JPRB, 0.42296E+00_JPRB, 0.42567E+00_JPRB, 0.41841E+00_JPRB, 0.40715E+00_JPRB, 0.40488E+00_JPRB ,&
&  0.41903E+00_JPRB, 0.47927E+00_JPRB, 0.60059E+00_JPRB, 0.74677E+00_JPRB, 0.74765E+00_JPRB, 0.79205E+00_JPRB, 0.82671E+00_JPRB ,&
&  0.85650E+00_JPRB, 0.86989E+00_JPRB, 0.85666E+00_JPRB, 0.85763E+00_JPRB, 0.86390E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.84300E+00_JPRB, 0.86294E+00_JPRB, 0.87384E+00_JPRB, 0.88783E+00_JPRB, 0.88812E+00_JPRB, 0.85807E+00_JPRB, 0.84371E+00_JPRB ,&
&  0.84841E+00_JPRB, 0.84343E+00_JPRB, 0.83230E+00_JPRB, 0.82525E+00_JPRB, 0.81620E+00_JPRB, 0.80637E+00_JPRB, 0.79822E+00_JPRB ,&
&  0.79001E+00_JPRB, 0.78496E+00_JPRB, 0.79187E+00_JPRB, 0.80569E+00_JPRB, 0.81702E+00_JPRB, 0.82317E+00_JPRB, 0.82817E+00_JPRB ,&
&  0.82779E+00_JPRB, 0.79483E+00_JPRB, 0.74963E+00_JPRB, 0.71672E+00_JPRB, 0.70037E+00_JPRB, 0.68653E+00_JPRB, 0.65312E+00_JPRB ,&
&  0.58446E+00_JPRB, 0.46334E+00_JPRB, 0.40797E+00_JPRB, 0.44953E+00_JPRB, 0.56580E+00_JPRB, 0.53178E+00_JPRB, 0.55290E+00_JPRB ,&
&  0.58375E+00_JPRB, 0.59977E+00_JPRB, 0.59564E+00_JPRB, 0.58521E+00_JPRB, 0.58181E+00_JPRB, 0.58553E+00_JPRB, 0.58243E+00_JPRB ,&
&  0.57926E+00_JPRB, 0.56348E+00_JPRB, 0.55402E+00_JPRB, 0.54285E+00_JPRB, 0.52496E+00_JPRB, 0.49912E+00_JPRB, 0.48325E+00_JPRB ,&
&  0.44444E+00_JPRB, 0.40624E+00_JPRB, 0.39484E+00_JPRB, 0.39501E+00_JPRB, 0.36395E+00_JPRB, 0.35123E+00_JPRB, 0.33588E+00_JPRB ,&
&  0.29655E+00_JPRB, 0.27522E+00_JPRB, 0.24351E+00_JPRB, 0.22368E+00_JPRB, 0.20918E+00_JPRB, 0.19468E+00_JPRB, 0.19250E+00_JPRB ,&
&  0.19754E+00_JPRB, 0.21036E+00_JPRB, 0.23138E+00_JPRB, 0.26431E+00_JPRB, 0.29395E+00_JPRB, 0.25320E+00_JPRB, 0.23586E+00_JPRB ,&
&  0.22151E+00_JPRB, 0.21838E+00_JPRB, 0.22440E+00_JPRB, 0.23136E+00_JPRB, 0.23838E+00_JPRB, 0.24659E+00_JPRB, 0.25460E+00_JPRB ,&
&  0.26036E+00_JPRB, 0.26336E+00_JPRB, 0.26405E+00_JPRB, 0.26328E+00_JPRB, 0.26201E+00_JPRB, 0.26265E+00_JPRB, 0.26608E+00_JPRB ,&
&  0.27254E+00_JPRB, 0.28501E+00_JPRB, 0.30894E+00_JPRB, 0.34431E+00_JPRB, 0.39032E+00_JPRB, 0.45071E+00_JPRB, 0.54264E+00_JPRB ,&
&  0.66728E+00_JPRB, 0.83117E+00_JPRB, 0.87665E+00_JPRB, 0.88574E+00_JPRB, 0.85461E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.84826E+00_JPRB, 0.84569E+00_JPRB, 0.85321E+00_JPRB, 0.84296E+00_JPRB, 0.82404E+00_JPRB, 0.81518E+00_JPRB, 0.83227E+00_JPRB ,&
&  0.85206E+00_JPRB, 0.82253E+00_JPRB, 0.80076E+00_JPRB, 0.78147E+00_JPRB, 0.76760E+00_JPRB, 0.75365E+00_JPRB, 0.73169E+00_JPRB ,&
&  0.71169E+00_JPRB, 0.70435E+00_JPRB, 0.70240E+00_JPRB, 0.69735E+00_JPRB, 0.68712E+00_JPRB, 0.67711E+00_JPRB, 0.65948E+00_JPRB ,&
&  0.62650E+00_JPRB, 0.59182E+00_JPRB, 0.57388E+00_JPRB, 0.55029E+00_JPRB, 0.53356E+00_JPRB, 0.51415E+00_JPRB, 0.48654E+00_JPRB ,&
&  0.44120E+00_JPRB, 0.38331E+00_JPRB, 0.34852E+00_JPRB, 0.38553E+00_JPRB, 0.47913E+00_JPRB, 0.47238E+00_JPRB, 0.49220E+00_JPRB ,&
&  0.52709E+00_JPRB, 0.54837E+00_JPRB, 0.55908E+00_JPRB, 0.56267E+00_JPRB, 0.57215E+00_JPRB, 0.57824E+00_JPRB, 0.57489E+00_JPRB ,&
&  0.56003E+00_JPRB, 0.50783E+00_JPRB, 0.46479E+00_JPRB, 0.36112E+00_JPRB, 0.29026E+00_JPRB, 0.25281E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.20353E+00_JPRB, 0.19215E+00_JPRB, 0.18960E+00_JPRB, 0.18863E+00_JPRB, 0.19401E+00_JPRB, 0.20017E+00_JPRB, 0.20009E+00_JPRB ,&
&  0.19588E+00_JPRB, 0.18764E+00_JPRB, 0.17617E+00_JPRB, 0.16939E+00_JPRB, 0.16577E+00_JPRB, 0.16587E+00_JPRB, 0.16782E+00_JPRB ,&
&  0.16908E+00_JPRB, 0.17278E+00_JPRB, 0.18531E+00_JPRB, 0.19610E+00_JPRB, 0.20958E+00_JPRB, 0.19218E+00_JPRB, 0.17771E+00_JPRB ,&
&  0.17241E+00_JPRB, 0.17715E+00_JPRB, 0.18427E+00_JPRB, 0.18960E+00_JPRB, 0.19521E+00_JPRB, 0.19986E+00_JPRB, 0.20447E+00_JPRB ,&
&  0.20913E+00_JPRB, 0.21240E+00_JPRB, 0.21394E+00_JPRB, 0.21564E+00_JPRB, 0.21718E+00_JPRB, 0.21952E+00_JPRB, 0.22239E+00_JPRB ,&
&  0.22733E+00_JPRB, 0.23684E+00_JPRB, 0.24920E+00_JPRB, 0.26967E+00_JPRB, 0.28887E+00_JPRB, 0.30794E+00_JPRB, 0.33012E+00_JPRB ,&
&  0.35078E+00_JPRB, 0.43146E+00_JPRB, 0.62484E+00_JPRB, 0.94483E+00_JPRB, 0.93846E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.79638E+00_JPRB, 0.80872E+00_JPRB, 0.82561E+00_JPRB, 0.80221E+00_JPRB, 0.78705E+00_JPRB, 0.79042E+00_JPRB, 0.79124E+00_JPRB ,&
&  0.76799E+00_JPRB, 0.73246E+00_JPRB, 0.68304E+00_JPRB, 0.64200E+00_JPRB, 0.57729E+00_JPRB, 0.49389E+00_JPRB, 0.45618E+00_JPRB ,&
&  0.42165E+00_JPRB, 0.41797E+00_JPRB, 0.42009E+00_JPRB, 0.43618E+00_JPRB, 0.44494E+00_JPRB, 0.44155E+00_JPRB, 0.41969E+00_JPRB ,&
&  0.39267E+00_JPRB, 0.36582E+00_JPRB, 0.34828E+00_JPRB, 0.33463E+00_JPRB, 0.33451E+00_JPRB, 0.33266E+00_JPRB, 0.32615E+00_JPRB ,&
&  0.31794E+00_JPRB, 0.32267E+00_JPRB, 0.34029E+00_JPRB, 0.36799E+00_JPRB, 0.41649E+00_JPRB, 0.39637E+00_JPRB, 0.40365E+00_JPRB ,&
&  0.43394E+00_JPRB, 0.45579E+00_JPRB, 0.46457E+00_JPRB, 0.46782E+00_JPRB, 0.46085E+00_JPRB, 0.48475E+00_JPRB, 0.42836E+00_JPRB ,&
&  0.32161E+00_JPRB, 0.26254E+00_JPRB, 0.22023E+00_JPRB, 0.19272E+00_JPRB, 0.17398E+00_JPRB, 0.16295E+00_JPRB, 0.15063E+00_JPRB ,&
&  0.14485E+00_JPRB, 0.14048E+00_JPRB, 0.13948E+00_JPRB, 0.14125E+00_JPRB, 0.14393E+00_JPRB, 0.14730E+00_JPRB, 0.14844E+00_JPRB ,&
&  0.15037E+00_JPRB, 0.15039E+00_JPRB, 0.14953E+00_JPRB, 0.14933E+00_JPRB, 0.14821E+00_JPRB, 0.14876E+00_JPRB, 0.14853E+00_JPRB ,&
&  0.14823E+00_JPRB, 0.14797E+00_JPRB, 0.14957E+00_JPRB, 0.15028E+00_JPRB, 0.15104E+00_JPRB, 0.14799E+00_JPRB, 0.14642E+00_JPRB ,&
&  0.14675E+00_JPRB, 0.14893E+00_JPRB, 0.15242E+00_JPRB, 0.15641E+00_JPRB, 0.16191E+00_JPRB, 0.16624E+00_JPRB, 0.17121E+00_JPRB ,&
&  0.17551E+00_JPRB, 0.17969E+00_JPRB, 0.18378E+00_JPRB, 0.18797E+00_JPRB, 0.19250E+00_JPRB, 0.19705E+00_JPRB, 0.20179E+00_JPRB ,&
&  0.20674E+00_JPRB, 0.21518E+00_JPRB, 0.22555E+00_JPRB, 0.24355E+00_JPRB, 0.26771E+00_JPRB, 0.29408E+00_JPRB, 0.34066E+00_JPRB ,&
&  0.39938E+00_JPRB, 0.62805E+00_JPRB, 0.92969E+00_JPRB, 0.76069E+00_JPRB, 0.77831E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.78701E+00_JPRB, 0.80536E+00_JPRB, 0.82143E+00_JPRB, 0.81881E+00_JPRB, 0.80155E+00_JPRB, 0.74400E+00_JPRB, 0.68067E+00_JPRB ,&
&  0.60213E+00_JPRB, 0.53633E+00_JPRB, 0.46698E+00_JPRB, 0.40711E+00_JPRB, 0.35669E+00_JPRB, 0.31789E+00_JPRB, 0.29421E+00_JPRB ,&
&  0.27739E+00_JPRB, 0.27000E+00_JPRB, 0.26698E+00_JPRB, 0.27156E+00_JPRB, 0.27382E+00_JPRB, 0.27932E+00_JPRB, 0.27373E+00_JPRB ,&
&  0.26681E+00_JPRB, 0.25900E+00_JPRB, 0.25695E+00_JPRB, 0.26561E+00_JPRB, 0.27898E+00_JPRB, 0.29193E+00_JPRB, 0.30881E+00_JPRB ,&
&  0.32714E+00_JPRB, 0.33304E+00_JPRB, 0.34747E+00_JPRB, 0.35111E+00_JPRB, 0.33663E+00_JPRB, 0.33849E+00_JPRB, 0.34443E+00_JPRB ,&
&  0.35776E+00_JPRB, 0.37636E+00_JPRB, 0.38702E+00_JPRB, 0.40696E+00_JPRB, 0.35346E+00_JPRB, 0.29345E+00_JPRB, 0.24142E+00_JPRB ,&
&  0.20261E+00_JPRB, 0.18043E+00_JPRB, 0.16459E+00_JPRB, 0.15656E+00_JPRB, 0.14861E+00_JPRB, 0.14428E+00_JPRB, 0.13971E+00_JPRB ,&
&  0.13766E+00_JPRB, 0.13761E+00_JPRB, 0.13711E+00_JPRB, 0.13681E+00_JPRB, 0.13725E+00_JPRB, 0.13796E+00_JPRB, 0.13738E+00_JPRB ,&
&  0.13664E+00_JPRB, 0.13634E+00_JPRB, 0.13583E+00_JPRB, 0.13539E+00_JPRB, 0.13448E+00_JPRB, 0.13422E+00_JPRB, 0.13242E+00_JPRB ,&
&  0.13140E+00_JPRB, 0.12987E+00_JPRB, 0.12898E+00_JPRB, 0.12889E+00_JPRB, 0.12924E+00_JPRB, 0.12960E+00_JPRB, 0.13025E+00_JPRB ,&
&  0.13152E+00_JPRB, 0.13266E+00_JPRB, 0.13459E+00_JPRB, 0.13643E+00_JPRB, 0.13941E+00_JPRB, 0.14205E+00_JPRB, 0.14576E+00_JPRB ,&
&  0.14971E+00_JPRB, 0.15447E+00_JPRB, 0.15996E+00_JPRB, 0.16615E+00_JPRB, 0.17140E+00_JPRB, 0.17655E+00_JPRB, 0.18214E+00_JPRB ,&
&  0.18654E+00_JPRB, 0.19396E+00_JPRB, 0.20232E+00_JPRB, 0.21617E+00_JPRB, 0.24512E+00_JPRB, 0.26858E+00_JPRB, 0.31697E+00_JPRB ,&
&  0.39770E+00_JPRB, 0.65097E+00_JPRB, 0.88358E+00_JPRB, 0.73710E+00_JPRB, 0.76857E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.66559E+00_JPRB, 0.69672E+00_JPRB, 0.72082E+00_JPRB, 0.72282E+00_JPRB, 0.72149E+00_JPRB, 0.68414E+00_JPRB, 0.62604E+00_JPRB ,&
&  0.51972E+00_JPRB, 0.45019E+00_JPRB, 0.40607E+00_JPRB, 0.36445E+00_JPRB, 0.32679E+00_JPRB, 0.30127E+00_JPRB, 0.28395E+00_JPRB ,&
&  0.27084E+00_JPRB, 0.26362E+00_JPRB, 0.25815E+00_JPRB, 0.25356E+00_JPRB, 0.24858E+00_JPRB, 0.24378E+00_JPRB, 0.23515E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22653E+00_JPRB, 0.22503E+00_JPRB, 0.22747E+00_JPRB, 0.23439E+00_JPRB, 0.24122E+00_JPRB, 0.25182E+00_JPRB ,&
&  0.26448E+00_JPRB, 0.27864E+00_JPRB, 0.28696E+00_JPRB, 0.29052E+00_JPRB, 0.29359E+00_JPRB, 0.29201E+00_JPRB, 0.26977E+00_JPRB ,&
&  0.27466E+00_JPRB, 0.29685E+00_JPRB, 0.31514E+00_JPRB, 0.32456E+00_JPRB, 0.27962E+00_JPRB, 0.23836E+00_JPRB, 0.19768E+00_JPRB ,&
&  0.17845E+00_JPRB, 0.16291E+00_JPRB, 0.15472E+00_JPRB, 0.14847E+00_JPRB, 0.14372E+00_JPRB, 0.14109E+00_JPRB, 0.13876E+00_JPRB ,&
&  0.13673E+00_JPRB, 0.13595E+00_JPRB, 0.13457E+00_JPRB, 0.13402E+00_JPRB, 0.13307E+00_JPRB, 0.13293E+00_JPRB, 0.13168E+00_JPRB ,&
&  0.13038E+00_JPRB, 0.12932E+00_JPRB, 0.12861E+00_JPRB, 0.12822E+00_JPRB, 0.12732E+00_JPRB, 0.12639E+00_JPRB, 0.12436E+00_JPRB ,&
&  0.12290E+00_JPRB, 0.12149E+00_JPRB, 0.12041E+00_JPRB, 0.12047E+00_JPRB, 0.12100E+00_JPRB, 0.12290E+00_JPRB, 0.12391E+00_JPRB ,&
&  0.12406E+00_JPRB, 0.12408E+00_JPRB, 0.12402E+00_JPRB, 0.12473E+00_JPRB, 0.12580E+00_JPRB, 0.12772E+00_JPRB, 0.13041E+00_JPRB ,&
&  0.13367E+00_JPRB, 0.13870E+00_JPRB, 0.14372E+00_JPRB, 0.14932E+00_JPRB, 0.15453E+00_JPRB, 0.15963E+00_JPRB, 0.16637E+00_JPRB ,&
&  0.17326E+00_JPRB, 0.18138E+00_JPRB, 0.19346E+00_JPRB, 0.20627E+00_JPRB, 0.22938E+00_JPRB, 0.24891E+00_JPRB, 0.27984E+00_JPRB ,&
&  0.34962E+00_JPRB, 0.49582E+00_JPRB, 0.58179E+00_JPRB, 0.59422E+00_JPRB, 0.62764E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.48475E+00_JPRB, 0.49638E+00_JPRB, 0.51154E+00_JPRB, 0.52075E+00_JPRB, 0.53104E+00_JPRB, 0.49852E+00_JPRB, 0.47287E+00_JPRB ,&
&  0.42021E+00_JPRB, 0.39004E+00_JPRB, 0.36492E+00_JPRB, 0.34186E+00_JPRB, 0.32131E+00_JPRB, 0.30297E+00_JPRB, 0.28573E+00_JPRB ,&
&  0.27469E+00_JPRB, 0.26809E+00_JPRB, 0.25912E+00_JPRB, 0.24903E+00_JPRB, 0.23823E+00_JPRB, 0.22879E+00_JPRB, 0.22396E+00_JPRB ,&
&  0.21930E+00_JPRB, 0.21596E+00_JPRB, 0.21461E+00_JPRB, 0.21468E+00_JPRB, 0.21656E+00_JPRB, 0.21675E+00_JPRB, 0.21740E+00_JPRB ,&
&  0.21420E+00_JPRB, 0.22144E+00_JPRB, 0.23100E+00_JPRB, 0.23537E+00_JPRB, 0.24350E+00_JPRB, 0.24221E+00_JPRB, 0.23750E+00_JPRB ,&
&  0.23030E+00_JPRB, 0.22189E+00_JPRB, 0.21233E+00_JPRB, 0.20050E+00_JPRB, 0.19008E+00_JPRB, 0.18243E+00_JPRB, 0.17485E+00_JPRB ,&
&  0.17033E+00_JPRB, 0.16475E+00_JPRB, 0.15809E+00_JPRB, 0.15232E+00_JPRB, 0.14731E+00_JPRB, 0.14320E+00_JPRB, 0.13944E+00_JPRB ,&
&  0.13672E+00_JPRB, 0.13513E+00_JPRB, 0.13360E+00_JPRB, 0.13321E+00_JPRB, 0.13207E+00_JPRB, 0.13092E+00_JPRB, 0.12977E+00_JPRB ,&
&  0.12913E+00_JPRB, 0.12782E+00_JPRB, 0.12677E+00_JPRB, 0.12582E+00_JPRB, 0.12439E+00_JPRB, 0.12317E+00_JPRB, 0.12179E+00_JPRB ,&
&  0.12060E+00_JPRB, 0.11997E+00_JPRB, 0.11959E+00_JPRB, 0.11948E+00_JPRB, 0.11973E+00_JPRB, 0.11966E+00_JPRB, 0.11989E+00_JPRB ,&
&  0.12017E+00_JPRB, 0.12060E+00_JPRB, 0.12054E+00_JPRB, 0.12084E+00_JPRB, 0.12169E+00_JPRB, 0.12288E+00_JPRB, 0.12583E+00_JPRB ,&
&  0.12923E+00_JPRB, 0.13358E+00_JPRB, 0.13832E+00_JPRB, 0.14371E+00_JPRB, 0.14913E+00_JPRB, 0.15450E+00_JPRB, 0.16038E+00_JPRB ,&
&  0.16807E+00_JPRB, 0.17501E+00_JPRB, 0.18312E+00_JPRB, 0.19547E+00_JPRB, 0.22238E+00_JPRB, 0.23695E+00_JPRB, 0.25162E+00_JPRB ,&
&  0.27758E+00_JPRB, 0.31277E+00_JPRB, 0.41631E+00_JPRB, 0.44887E+00_JPRB, 0.46630E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.22880E+00_JPRB, 0.22882E+00_JPRB, 0.22949E+00_JPRB, 0.23581E+00_JPRB, 0.24689E+00_JPRB, 0.32431E+00_JPRB, 0.31527E+00_JPRB ,&
&  0.30793E+00_JPRB, 0.27986E+00_JPRB, 0.27643E+00_JPRB, 0.26943E+00_JPRB, 0.25997E+00_JPRB, 0.25237E+00_JPRB, 0.24792E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.23765E+00_JPRB, 0.23467E+00_JPRB, 0.23153E+00_JPRB, 0.22842E+00_JPRB, 0.22485E+00_JPRB, 0.22211E+00_JPRB ,&
&  0.21956E+00_JPRB, 0.21779E+00_JPRB, 0.21584E+00_JPRB, 0.21191E+00_JPRB, 0.21021E+00_JPRB, 0.20690E+00_JPRB, 0.20196E+00_JPRB ,&
&  0.19549E+00_JPRB, 0.19207E+00_JPRB, 0.18827E+00_JPRB, 0.18488E+00_JPRB, 0.18788E+00_JPRB, 0.18696E+00_JPRB, 0.18481E+00_JPRB ,&
&  0.18124E+00_JPRB, 0.17755E+00_JPRB, 0.17274E+00_JPRB, 0.16813E+00_JPRB, 0.16429E+00_JPRB, 0.16116E+00_JPRB, 0.15846E+00_JPRB ,&
&  0.15632E+00_JPRB, 0.15453E+00_JPRB, 0.15202E+00_JPRB, 0.14973E+00_JPRB, 0.14731E+00_JPRB, 0.14518E+00_JPRB, 0.14297E+00_JPRB ,&
&  0.14122E+00_JPRB, 0.13957E+00_JPRB, 0.13786E+00_JPRB, 0.13611E+00_JPRB, 0.13471E+00_JPRB, 0.13290E+00_JPRB, 0.13154E+00_JPRB ,&
&  0.13035E+00_JPRB, 0.12884E+00_JPRB, 0.12772E+00_JPRB, 0.12635E+00_JPRB, 0.12533E+00_JPRB, 0.12449E+00_JPRB, 0.12367E+00_JPRB ,&
&  0.12259E+00_JPRB, 0.12186E+00_JPRB, 0.12158E+00_JPRB, 0.12135E+00_JPRB, 0.12101E+00_JPRB, 0.12105E+00_JPRB, 0.12136E+00_JPRB ,&
&  0.12181E+00_JPRB, 0.12234E+00_JPRB, 0.12323E+00_JPRB, 0.12428E+00_JPRB, 0.12586E+00_JPRB, 0.12797E+00_JPRB, 0.13131E+00_JPRB ,&
&  0.13494E+00_JPRB, 0.13897E+00_JPRB, 0.14310E+00_JPRB, 0.14766E+00_JPRB, 0.15231E+00_JPRB, 0.15702E+00_JPRB, 0.16241E+00_JPRB ,&
&  0.16827E+00_JPRB, 0.17283E+00_JPRB, 0.17709E+00_JPRB, 0.18160E+00_JPRB, 0.19110E+00_JPRB, 0.19817E+00_JPRB, 0.20350E+00_JPRB ,&
&  0.20953E+00_JPRB, 0.21774E+00_JPRB, 0.22244E+00_JPRB, 0.22563E+00_JPRB, 0.22743E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.20139E+00_JPRB, 0.20408E+00_JPRB, 0.20589E+00_JPRB, 0.20704E+00_JPRB, 0.20677E+00_JPRB, 0.20769E+00_JPRB, 0.20864E+00_JPRB ,&
&  0.20990E+00_JPRB, 0.21150E+00_JPRB, 0.21265E+00_JPRB, 0.21370E+00_JPRB, 0.21420E+00_JPRB, 0.21421E+00_JPRB, 0.21423E+00_JPRB ,&
&  0.21403E+00_JPRB, 0.21379E+00_JPRB, 0.21025E+00_JPRB, 0.20708E+00_JPRB, 0.20479E+00_JPRB, 0.20348E+00_JPRB, 0.20233E+00_JPRB ,&
&  0.20139E+00_JPRB, 0.20014E+00_JPRB, 0.19869E+00_JPRB, 0.19655E+00_JPRB, 0.19326E+00_JPRB, 0.19005E+00_JPRB, 0.18673E+00_JPRB ,&
&  0.18373E+00_JPRB, 0.18131E+00_JPRB, 0.17907E+00_JPRB, 0.17682E+00_JPRB, 0.17549E+00_JPRB, 0.17394E+00_JPRB, 0.17234E+00_JPRB ,&
&  0.17030E+00_JPRB, 0.16809E+00_JPRB, 0.16600E+00_JPRB, 0.16385E+00_JPRB, 0.16179E+00_JPRB, 0.15975E+00_JPRB, 0.15826E+00_JPRB ,&
&  0.15671E+00_JPRB, 0.15553E+00_JPRB, 0.15424E+00_JPRB, 0.15305E+00_JPRB, 0.15181E+00_JPRB, 0.15063E+00_JPRB, 0.14926E+00_JPRB ,&
&  0.14832E+00_JPRB, 0.14734E+00_JPRB, 0.14634E+00_JPRB, 0.14542E+00_JPRB, 0.14450E+00_JPRB, 0.14360E+00_JPRB, 0.14259E+00_JPRB ,&
&  0.14167E+00_JPRB, 0.14107E+00_JPRB, 0.14044E+00_JPRB, 0.13974E+00_JPRB, 0.13910E+00_JPRB, 0.13865E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13790E+00_JPRB, 0.13786E+00_JPRB, 0.13803E+00_JPRB, 0.13819E+00_JPRB, 0.13836E+00_JPRB, 0.13875E+00_JPRB, 0.13933E+00_JPRB ,&
&  0.14003E+00_JPRB, 0.14078E+00_JPRB, 0.14174E+00_JPRB, 0.14312E+00_JPRB, 0.14464E+00_JPRB, 0.14639E+00_JPRB, 0.14830E+00_JPRB ,&
&  0.15038E+00_JPRB, 0.15260E+00_JPRB, 0.15493E+00_JPRB, 0.15722E+00_JPRB, 0.15969E+00_JPRB, 0.16215E+00_JPRB, 0.16453E+00_JPRB ,&
&  0.16702E+00_JPRB, 0.16948E+00_JPRB, 0.17202E+00_JPRB, 0.17435E+00_JPRB, 0.17795E+00_JPRB, 0.18030E+00_JPRB, 0.18274E+00_JPRB ,&
&  0.18498E+00_JPRB, 0.18758E+00_JPRB, 0.19020E+00_JPRB, 0.19345E+00_JPRB, 0.19667E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB, 0.16600E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB, 0.38368E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.40393E+00_JPRB, 0.40266E+00_JPRB, 0.40110E+00_JPRB, 0.39975E+00_JPRB, 0.39804E+00_JPRB, 0.39670E+00_JPRB, 0.39534E+00_JPRB ,&
&  0.39416E+00_JPRB, 0.39287E+00_JPRB, 0.39180E+00_JPRB, 0.39042E+00_JPRB, 0.38915E+00_JPRB, 0.38795E+00_JPRB, 0.38687E+00_JPRB ,&
&  0.38575E+00_JPRB, 0.38475E+00_JPRB, 0.38375E+00_JPRB, 0.38267E+00_JPRB, 0.38178E+00_JPRB, 0.38067E+00_JPRB, 0.37912E+00_JPRB ,&
&  0.37652E+00_JPRB, 0.37364E+00_JPRB, 0.37112E+00_JPRB, 0.36943E+00_JPRB, 0.36822E+00_JPRB, 0.36723E+00_JPRB, 0.36637E+00_JPRB ,&
&  0.36585E+00_JPRB, 0.36569E+00_JPRB, 0.36570E+00_JPRB, 0.36558E+00_JPRB, 0.36573E+00_JPRB, 0.36665E+00_JPRB, 0.36721E+00_JPRB ,&
&  0.36749E+00_JPRB, 0.36779E+00_JPRB, 0.36865E+00_JPRB, 0.36936E+00_JPRB, 0.37008E+00_JPRB, 0.37093E+00_JPRB, 0.37240E+00_JPRB ,&
&  0.37387E+00_JPRB, 0.37523E+00_JPRB, 0.37675E+00_JPRB, 0.37829E+00_JPRB, 0.37981E+00_JPRB, 0.38136E+00_JPRB, 0.38334E+00_JPRB ,&
&  0.38544E+00_JPRB, 0.38729E+00_JPRB, 0.38890E+00_JPRB, 0.39015E+00_JPRB, 0.39119E+00_JPRB, 0.39164E+00_JPRB, 0.39189E+00_JPRB ,&
&  0.39164E+00_JPRB, 0.39197E+00_JPRB, 0.39185E+00_JPRB, 0.39178E+00_JPRB, 0.39129E+00_JPRB, 0.39156E+00_JPRB, 0.39156E+00_JPRB ,&
&  0.39215E+00_JPRB, 0.39240E+00_JPRB, 0.39310E+00_JPRB, 0.39374E+00_JPRB, 0.39495E+00_JPRB, 0.39639E+00_JPRB, 0.39800E+00_JPRB ,&
&  0.39982E+00_JPRB, 0.40182E+00_JPRB, 0.40417E+00_JPRB, 0.40541E+00_JPRB, 0.40664E+00_JPRB, 0.40789E+00_JPRB, 0.40935E+00_JPRB ,&
&  0.41041E+00_JPRB, 0.41142E+00_JPRB, 0.41252E+00_JPRB, 0.41335E+00_JPRB, 0.41374E+00_JPRB, 0.41398E+00_JPRB, 0.41422E+00_JPRB ,&
&  0.41415E+00_JPRB, 0.41406E+00_JPRB, 0.41387E+00_JPRB, 0.41375E+00_JPRB, 0.41307E+00_JPRB, 0.41217E+00_JPRB, 0.41107E+00_JPRB ,&
&  0.41017E+00_JPRB, 0.40896E+00_JPRB, 0.40782E+00_JPRB, 0.40657E+00_JPRB, 0.40538E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.44743E+00_JPRB, 0.44505E+00_JPRB, 0.44255E+00_JPRB, 0.44023E+00_JPRB, 0.43698E+00_JPRB, 0.43368E+00_JPRB, 0.43029E+00_JPRB ,&
&  0.42687E+00_JPRB, 0.42293E+00_JPRB, 0.41880E+00_JPRB, 0.41444E+00_JPRB, 0.40997E+00_JPRB, 0.40583E+00_JPRB, 0.40243E+00_JPRB ,&
&  0.39940E+00_JPRB, 0.39610E+00_JPRB, 0.39332E+00_JPRB, 0.38968E+00_JPRB, 0.38596E+00_JPRB, 0.38273E+00_JPRB, 0.38068E+00_JPRB ,&
&  0.37709E+00_JPRB, 0.37274E+00_JPRB, 0.36814E+00_JPRB, 0.36312E+00_JPRB, 0.35840E+00_JPRB, 0.35369E+00_JPRB, 0.34890E+00_JPRB ,&
&  0.34426E+00_JPRB, 0.34082E+00_JPRB, 0.33783E+00_JPRB, 0.33484E+00_JPRB, 0.33188E+00_JPRB, 0.33134E+00_JPRB, 0.33064E+00_JPRB ,&
&  0.33008E+00_JPRB, 0.33032E+00_JPRB, 0.33302E+00_JPRB, 0.33551E+00_JPRB, 0.33808E+00_JPRB, 0.34072E+00_JPRB, 0.34503E+00_JPRB ,&
&  0.34905E+00_JPRB, 0.35382E+00_JPRB, 0.35968E+00_JPRB, 0.36535E+00_JPRB, 0.36984E+00_JPRB, 0.37301E+00_JPRB, 0.37711E+00_JPRB ,&
&  0.37987E+00_JPRB, 0.38353E+00_JPRB, 0.38598E+00_JPRB, 0.38699E+00_JPRB, 0.38657E+00_JPRB, 0.38620E+00_JPRB, 0.38644E+00_JPRB ,&
&  0.38612E+00_JPRB, 0.38773E+00_JPRB, 0.38926E+00_JPRB, 0.39081E+00_JPRB, 0.39112E+00_JPRB, 0.39336E+00_JPRB, 0.39516E+00_JPRB ,&
&  0.39871E+00_JPRB, 0.40378E+00_JPRB, 0.40900E+00_JPRB, 0.41478E+00_JPRB, 0.42195E+00_JPRB, 0.43043E+00_JPRB, 0.43714E+00_JPRB ,&
&  0.44368E+00_JPRB, 0.44893E+00_JPRB, 0.45303E+00_JPRB, 0.45495E+00_JPRB, 0.45678E+00_JPRB, 0.45858E+00_JPRB, 0.46109E+00_JPRB ,&
&  0.46211E+00_JPRB, 0.46337E+00_JPRB, 0.46467E+00_JPRB, 0.46562E+00_JPRB, 0.46498E+00_JPRB, 0.46421E+00_JPRB, 0.46387E+00_JPRB ,&
&  0.46330E+00_JPRB, 0.46251E+00_JPRB, 0.46177E+00_JPRB, 0.46141E+00_JPRB, 0.46058E+00_JPRB, 0.45949E+00_JPRB, 0.45838E+00_JPRB ,&
&  0.45732E+00_JPRB, 0.45563E+00_JPRB, 0.45364E+00_JPRB, 0.45167E+00_JPRB, 0.44987E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.46393E+00_JPRB, 0.46342E+00_JPRB, 0.46278E+00_JPRB, 0.46197E+00_JPRB, 0.46009E+00_JPRB, 0.45785E+00_JPRB, 0.45498E+00_JPRB ,&
&  0.45150E+00_JPRB, 0.44659E+00_JPRB, 0.44146E+00_JPRB, 0.43603E+00_JPRB, 0.43097E+00_JPRB, 0.42676E+00_JPRB, 0.42281E+00_JPRB ,&
&  0.41851E+00_JPRB, 0.41279E+00_JPRB, 0.40614E+00_JPRB, 0.39746E+00_JPRB, 0.38938E+00_JPRB, 0.38249E+00_JPRB, 0.37498E+00_JPRB ,&
&  0.36398E+00_JPRB, 0.35183E+00_JPRB, 0.34100E+00_JPRB, 0.33005E+00_JPRB, 0.32195E+00_JPRB, 0.30005E+00_JPRB, 0.29306E+00_JPRB ,&
&  0.28822E+00_JPRB, 0.28344E+00_JPRB, 0.28139E+00_JPRB, 0.27999E+00_JPRB, 0.28118E+00_JPRB, 0.27784E+00_JPRB, 0.27601E+00_JPRB ,&
&  0.26588E+00_JPRB, 0.26728E+00_JPRB, 0.26992E+00_JPRB, 0.27328E+00_JPRB, 0.28010E+00_JPRB, 0.28803E+00_JPRB, 0.30053E+00_JPRB ,&
&  0.32298E+00_JPRB, 0.33295E+00_JPRB, 0.34785E+00_JPRB, 0.35939E+00_JPRB, 0.36611E+00_JPRB, 0.36970E+00_JPRB, 0.37360E+00_JPRB ,&
&  0.37782E+00_JPRB, 0.38395E+00_JPRB, 0.38754E+00_JPRB, 0.38995E+00_JPRB, 0.38968E+00_JPRB, 0.38840E+00_JPRB, 0.38781E+00_JPRB ,&
&  0.38661E+00_JPRB, 0.38798E+00_JPRB, 0.39080E+00_JPRB, 0.39708E+00_JPRB, 0.40595E+00_JPRB, 0.41748E+00_JPRB, 0.42982E+00_JPRB ,&
&  0.44259E+00_JPRB, 0.45693E+00_JPRB, 0.46881E+00_JPRB, 0.48018E+00_JPRB, 0.49207E+00_JPRB, 0.50202E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51581E+00_JPRB, 0.51705E+00_JPRB, 0.51391E+00_JPRB, 0.51010E+00_JPRB, 0.50545E+00_JPRB, 0.50211E+00_JPRB, 0.50129E+00_JPRB ,&
&  0.50046E+00_JPRB, 0.50035E+00_JPRB, 0.50073E+00_JPRB, 0.49873E+00_JPRB, 0.49636E+00_JPRB, 0.49341E+00_JPRB, 0.49050E+00_JPRB ,&
&  0.48707E+00_JPRB, 0.48409E+00_JPRB, 0.48141E+00_JPRB, 0.47888E+00_JPRB, 0.47674E+00_JPRB, 0.47380E+00_JPRB, 0.47140E+00_JPRB ,&
&  0.46892E+00_JPRB, 0.46812E+00_JPRB, 0.46666E+00_JPRB, 0.46558E+00_JPRB, 0.46447E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.43604E+00_JPRB, 0.43625E+00_JPRB, 0.43709E+00_JPRB, 0.43752E+00_JPRB, 0.43876E+00_JPRB, 0.43956E+00_JPRB, 0.44071E+00_JPRB ,&
&  0.44129E+00_JPRB, 0.44189E+00_JPRB, 0.44150E+00_JPRB, 0.44056E+00_JPRB, 0.43867E+00_JPRB, 0.43688E+00_JPRB, 0.43479E+00_JPRB ,&
&  0.43157E+00_JPRB, 0.42473E+00_JPRB, 0.41500E+00_JPRB, 0.40397E+00_JPRB, 0.39134E+00_JPRB, 0.37752E+00_JPRB, 0.36171E+00_JPRB ,&
&  0.34507E+00_JPRB, 0.32942E+00_JPRB, 0.30961E+00_JPRB, 0.30144E+00_JPRB, 0.28883E+00_JPRB, 0.27907E+00_JPRB, 0.27209E+00_JPRB ,&
&  0.26375E+00_JPRB, 0.25795E+00_JPRB, 0.25211E+00_JPRB, 0.24615E+00_JPRB, 0.24226E+00_JPRB, 0.23552E+00_JPRB, 0.23221E+00_JPRB ,&
&  0.22992E+00_JPRB, 0.22890E+00_JPRB, 0.22915E+00_JPRB, 0.23103E+00_JPRB, 0.23524E+00_JPRB, 0.24066E+00_JPRB, 0.25120E+00_JPRB ,&
&  0.27459E+00_JPRB, 0.30820E+00_JPRB, 0.35927E+00_JPRB, 0.36926E+00_JPRB, 0.36847E+00_JPRB, 0.37555E+00_JPRB, 0.38317E+00_JPRB ,&
&  0.39045E+00_JPRB, 0.39640E+00_JPRB, 0.40107E+00_JPRB, 0.40358E+00_JPRB, 0.40679E+00_JPRB, 0.40945E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41536E+00_JPRB, 0.42261E+00_JPRB, 0.43288E+00_JPRB, 0.44527E+00_JPRB, 0.45845E+00_JPRB, 0.46911E+00_JPRB, 0.47900E+00_JPRB ,&
&  0.48695E+00_JPRB, 0.49610E+00_JPRB, 0.50877E+00_JPRB, 0.52267E+00_JPRB, 0.53693E+00_JPRB, 0.54507E+00_JPRB, 0.55166E+00_JPRB ,&
&  0.55545E+00_JPRB, 0.55369E+00_JPRB, 0.54735E+00_JPRB, 0.53976E+00_JPRB, 0.53403E+00_JPRB, 0.53619E+00_JPRB, 0.53441E+00_JPRB ,&
&  0.53308E+00_JPRB, 0.52756E+00_JPRB, 0.51905E+00_JPRB, 0.50857E+00_JPRB, 0.50071E+00_JPRB, 0.49356E+00_JPRB, 0.48891E+00_JPRB ,&
&  0.48480E+00_JPRB, 0.48072E+00_JPRB, 0.47546E+00_JPRB, 0.46863E+00_JPRB, 0.46169E+00_JPRB, 0.45445E+00_JPRB, 0.44875E+00_JPRB ,&
&  0.44368E+00_JPRB, 0.44069E+00_JPRB, 0.43817E+00_JPRB, 0.43697E+00_JPRB, 0.43588E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.41866E+00_JPRB, 0.42250E+00_JPRB, 0.42643E+00_JPRB, 0.42930E+00_JPRB, 0.43170E+00_JPRB, 0.43357E+00_JPRB, 0.43423E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43443E+00_JPRB, 0.43468E+00_JPRB, 0.43477E+00_JPRB, 0.43516E+00_JPRB, 0.43607E+00_JPRB, 0.43672E+00_JPRB ,&
&  0.43722E+00_JPRB, 0.43224E+00_JPRB, 0.42168E+00_JPRB, 0.40958E+00_JPRB, 0.39657E+00_JPRB, 0.38024E+00_JPRB, 0.36059E+00_JPRB ,&
&  0.34274E+00_JPRB, 0.32950E+00_JPRB, 0.31498E+00_JPRB, 0.30534E+00_JPRB, 0.29483E+00_JPRB, 0.28881E+00_JPRB, 0.28035E+00_JPRB ,&
&  0.27757E+00_JPRB, 0.25754E+00_JPRB, 0.25477E+00_JPRB, 0.24213E+00_JPRB, 0.23250E+00_JPRB, 0.22463E+00_JPRB, 0.21888E+00_JPRB ,&
&  0.21726E+00_JPRB, 0.21515E+00_JPRB, 0.21524E+00_JPRB, 0.21804E+00_JPRB, 0.22169E+00_JPRB, 0.22945E+00_JPRB, 0.23837E+00_JPRB ,&
&  0.25931E+00_JPRB, 0.32833E+00_JPRB, 0.35351E+00_JPRB, 0.36160E+00_JPRB, 0.36691E+00_JPRB, 0.37954E+00_JPRB, 0.39128E+00_JPRB ,&
&  0.40598E+00_JPRB, 0.41475E+00_JPRB, 0.42234E+00_JPRB, 0.42600E+00_JPRB, 0.43926E+00_JPRB, 0.45105E+00_JPRB, 0.46105E+00_JPRB ,&
&  0.47168E+00_JPRB, 0.48208E+00_JPRB, 0.49427E+00_JPRB, 0.50132E+00_JPRB, 0.50160E+00_JPRB, 0.49940E+00_JPRB, 0.49696E+00_JPRB ,&
&  0.49856E+00_JPRB, 0.50194E+00_JPRB, 0.51719E+00_JPRB, 0.53797E+00_JPRB, 0.56177E+00_JPRB, 0.57829E+00_JPRB, 0.58612E+00_JPRB ,&
&  0.59042E+00_JPRB, 0.59135E+00_JPRB, 0.59554E+00_JPRB, 0.60299E+00_JPRB, 0.59646E+00_JPRB, 0.58370E+00_JPRB, 0.57585E+00_JPRB ,&
&  0.56796E+00_JPRB, 0.55042E+00_JPRB, 0.53592E+00_JPRB, 0.52306E+00_JPRB, 0.51310E+00_JPRB, 0.50419E+00_JPRB, 0.49595E+00_JPRB ,&
&  0.48605E+00_JPRB, 0.47588E+00_JPRB, 0.46446E+00_JPRB, 0.45002E+00_JPRB, 0.43766E+00_JPRB, 0.42506E+00_JPRB, 0.41764E+00_JPRB ,&
&  0.41147E+00_JPRB, 0.41025E+00_JPRB, 0.40920E+00_JPRB, 0.41184E+00_JPRB, 0.41386E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.40742E+00_JPRB, 0.41667E+00_JPRB, 0.42440E+00_JPRB, 0.42969E+00_JPRB, 0.43305E+00_JPRB, 0.43519E+00_JPRB, 0.43485E+00_JPRB ,&
&  0.43422E+00_JPRB, 0.43270E+00_JPRB, 0.43219E+00_JPRB, 0.43125E+00_JPRB, 0.43158E+00_JPRB, 0.43103E+00_JPRB, 0.43089E+00_JPRB ,&
&  0.43113E+00_JPRB, 0.43019E+00_JPRB, 0.42490E+00_JPRB, 0.41433E+00_JPRB, 0.40110E+00_JPRB, 0.38644E+00_JPRB, 0.36595E+00_JPRB ,&
&  0.34589E+00_JPRB, 0.33313E+00_JPRB, 0.32118E+00_JPRB, 0.31119E+00_JPRB, 0.30529E+00_JPRB, 0.29781E+00_JPRB, 0.29642E+00_JPRB ,&
&  0.29420E+00_JPRB, 0.28700E+00_JPRB, 0.27797E+00_JPRB, 0.26866E+00_JPRB, 0.25877E+00_JPRB, 0.23139E+00_JPRB, 0.22744E+00_JPRB ,&
&  0.22031E+00_JPRB, 0.21409E+00_JPRB, 0.21256E+00_JPRB, 0.21354E+00_JPRB, 0.21637E+00_JPRB, 0.22496E+00_JPRB, 0.23614E+00_JPRB ,&
&  0.25290E+00_JPRB, 0.33235E+00_JPRB, 0.34968E+00_JPRB, 0.36079E+00_JPRB, 0.37827E+00_JPRB, 0.39583E+00_JPRB, 0.41062E+00_JPRB ,&
&  0.42391E+00_JPRB, 0.43448E+00_JPRB, 0.44827E+00_JPRB, 0.46084E+00_JPRB, 0.47499E+00_JPRB, 0.48780E+00_JPRB, 0.49637E+00_JPRB ,&
&  0.50348E+00_JPRB, 0.50383E+00_JPRB, 0.52117E+00_JPRB, 0.52751E+00_JPRB, 0.52327E+00_JPRB, 0.53538E+00_JPRB, 0.54102E+00_JPRB ,&
&  0.53770E+00_JPRB, 0.53059E+00_JPRB, 0.54138E+00_JPRB, 0.57005E+00_JPRB, 0.61122E+00_JPRB, 0.64316E+00_JPRB, 0.65172E+00_JPRB ,&
&  0.65416E+00_JPRB, 0.64911E+00_JPRB, 0.64329E+00_JPRB, 0.64213E+00_JPRB, 0.62455E+00_JPRB, 0.59856E+00_JPRB, 0.58015E+00_JPRB ,&
&  0.56481E+00_JPRB, 0.56206E+00_JPRB, 0.55310E+00_JPRB, 0.54275E+00_JPRB, 0.53093E+00_JPRB, 0.51889E+00_JPRB, 0.50228E+00_JPRB ,&
&  0.47936E+00_JPRB, 0.45415E+00_JPRB, 0.43431E+00_JPRB, 0.41680E+00_JPRB, 0.40697E+00_JPRB, 0.39605E+00_JPRB, 0.39096E+00_JPRB ,&
&  0.38605E+00_JPRB, 0.38631E+00_JPRB, 0.38708E+00_JPRB, 0.39210E+00_JPRB, 0.39798E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.40852E+00_JPRB, 0.42317E+00_JPRB, 0.43522E+00_JPRB, 0.44243E+00_JPRB, 0.44399E+00_JPRB, 0.44457E+00_JPRB, 0.44325E+00_JPRB ,&
&  0.44245E+00_JPRB, 0.44133E+00_JPRB, 0.43969E+00_JPRB, 0.43634E+00_JPRB, 0.43313E+00_JPRB, 0.42895E+00_JPRB, 0.42640E+00_JPRB ,&
&  0.42338E+00_JPRB, 0.41664E+00_JPRB, 0.41197E+00_JPRB, 0.40697E+00_JPRB, 0.39820E+00_JPRB, 0.38638E+00_JPRB, 0.36951E+00_JPRB ,&
&  0.35059E+00_JPRB, 0.33650E+00_JPRB, 0.32702E+00_JPRB, 0.31730E+00_JPRB, 0.30923E+00_JPRB, 0.30401E+00_JPRB, 0.30141E+00_JPRB ,&
&  0.30097E+00_JPRB, 0.29673E+00_JPRB, 0.29380E+00_JPRB, 0.28167E+00_JPRB, 0.26640E+00_JPRB, 0.26063E+00_JPRB, 0.24382E+00_JPRB ,&
&  0.22881E+00_JPRB, 0.21782E+00_JPRB, 0.21518E+00_JPRB, 0.21547E+00_JPRB, 0.22004E+00_JPRB, 0.22992E+00_JPRB, 0.25181E+00_JPRB ,&
&  0.28921E+00_JPRB, 0.35189E+00_JPRB, 0.36526E+00_JPRB, 0.38250E+00_JPRB, 0.40148E+00_JPRB, 0.41972E+00_JPRB, 0.43285E+00_JPRB ,&
&  0.44233E+00_JPRB, 0.45100E+00_JPRB, 0.46640E+00_JPRB, 0.48189E+00_JPRB, 0.50812E+00_JPRB, 0.53328E+00_JPRB, 0.55442E+00_JPRB ,&
&  0.57412E+00_JPRB, 0.54983E+00_JPRB, 0.55337E+00_JPRB, 0.55795E+00_JPRB, 0.57788E+00_JPRB, 0.59422E+00_JPRB, 0.58743E+00_JPRB ,&
&  0.58502E+00_JPRB, 0.60801E+00_JPRB, 0.61088E+00_JPRB, 0.62688E+00_JPRB, 0.66254E+00_JPRB, 0.70136E+00_JPRB, 0.71597E+00_JPRB ,&
&  0.70222E+00_JPRB, 0.70367E+00_JPRB, 0.74224E+00_JPRB, 0.70767E+00_JPRB, 0.68633E+00_JPRB, 0.65134E+00_JPRB, 0.61023E+00_JPRB ,&
&  0.58810E+00_JPRB, 0.57575E+00_JPRB, 0.56954E+00_JPRB, 0.56134E+00_JPRB, 0.54818E+00_JPRB, 0.52946E+00_JPRB, 0.49779E+00_JPRB ,&
&  0.45804E+00_JPRB, 0.42951E+00_JPRB, 0.40940E+00_JPRB, 0.39632E+00_JPRB, 0.38732E+00_JPRB, 0.37749E+00_JPRB, 0.37043E+00_JPRB ,&
&  0.36583E+00_JPRB, 0.36746E+00_JPRB, 0.37381E+00_JPRB, 0.38546E+00_JPRB, 0.39748E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.42618E+00_JPRB, 0.44119E+00_JPRB, 0.45014E+00_JPRB, 0.45055E+00_JPRB, 0.44394E+00_JPRB, 0.43198E+00_JPRB, 0.42268E+00_JPRB ,&
&  0.41984E+00_JPRB, 0.42043E+00_JPRB, 0.42225E+00_JPRB, 0.41987E+00_JPRB, 0.41791E+00_JPRB, 0.41618E+00_JPRB, 0.41557E+00_JPRB ,&
&  0.41486E+00_JPRB, 0.41427E+00_JPRB, 0.41316E+00_JPRB, 0.40643E+00_JPRB, 0.39680E+00_JPRB, 0.38466E+00_JPRB, 0.37074E+00_JPRB ,&
&  0.35620E+00_JPRB, 0.34467E+00_JPRB, 0.33530E+00_JPRB, 0.32790E+00_JPRB, 0.31927E+00_JPRB, 0.31436E+00_JPRB, 0.31109E+00_JPRB ,&
&  0.30867E+00_JPRB, 0.30870E+00_JPRB, 0.30604E+00_JPRB, 0.29985E+00_JPRB, 0.28731E+00_JPRB, 0.27209E+00_JPRB, 0.25472E+00_JPRB ,&
&  0.24070E+00_JPRB, 0.23197E+00_JPRB, 0.22702E+00_JPRB, 0.22651E+00_JPRB, 0.23372E+00_JPRB, 0.24982E+00_JPRB, 0.28336E+00_JPRB ,&
&  0.34737E+00_JPRB, 0.37063E+00_JPRB, 0.38337E+00_JPRB, 0.39995E+00_JPRB, 0.41524E+00_JPRB, 0.42755E+00_JPRB, 0.43863E+00_JPRB ,&
&  0.44718E+00_JPRB, 0.45556E+00_JPRB, 0.46710E+00_JPRB, 0.47697E+00_JPRB, 0.49612E+00_JPRB, 0.50728E+00_JPRB, 0.55710E+00_JPRB ,&
&  0.66465E+00_JPRB, 0.66713E+00_JPRB, 0.61312E+00_JPRB, 0.61942E+00_JPRB, 0.64109E+00_JPRB, 0.66253E+00_JPRB, 0.68552E+00_JPRB ,&
&  0.70314E+00_JPRB, 0.70931E+00_JPRB, 0.71261E+00_JPRB, 0.71281E+00_JPRB, 0.72520E+00_JPRB, 0.74063E+00_JPRB, 0.74368E+00_JPRB ,&
&  0.73787E+00_JPRB, 0.75964E+00_JPRB, 0.81634E+00_JPRB, 0.68660E+00_JPRB, 0.62774E+00_JPRB, 0.61079E+00_JPRB, 0.58662E+00_JPRB ,&
&  0.57186E+00_JPRB, 0.56862E+00_JPRB, 0.57565E+00_JPRB, 0.57597E+00_JPRB, 0.54193E+00_JPRB, 0.49640E+00_JPRB, 0.44941E+00_JPRB ,&
&  0.42391E+00_JPRB, 0.41064E+00_JPRB, 0.39587E+00_JPRB, 0.38576E+00_JPRB, 0.37744E+00_JPRB, 0.36813E+00_JPRB, 0.35859E+00_JPRB ,&
&  0.35928E+00_JPRB, 0.36646E+00_JPRB, 0.37815E+00_JPRB, 0.39856E+00_JPRB, 0.41402E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.42429E+00_JPRB, 0.44512E+00_JPRB, 0.45161E+00_JPRB, 0.44176E+00_JPRB, 0.43002E+00_JPRB, 0.40522E+00_JPRB, 0.38272E+00_JPRB ,&
&  0.37397E+00_JPRB, 0.36910E+00_JPRB, 0.36861E+00_JPRB, 0.36879E+00_JPRB, 0.37090E+00_JPRB, 0.37521E+00_JPRB, 0.37955E+00_JPRB ,&
&  0.38096E+00_JPRB, 0.38191E+00_JPRB, 0.38384E+00_JPRB, 0.38621E+00_JPRB, 0.38359E+00_JPRB, 0.37466E+00_JPRB, 0.36422E+00_JPRB ,&
&  0.35073E+00_JPRB, 0.34131E+00_JPRB, 0.33466E+00_JPRB, 0.32955E+00_JPRB, 0.32624E+00_JPRB, 0.32561E+00_JPRB, 0.32649E+00_JPRB ,&
&  0.32502E+00_JPRB, 0.32357E+00_JPRB, 0.32347E+00_JPRB, 0.32034E+00_JPRB, 0.30886E+00_JPRB, 0.29135E+00_JPRB, 0.26976E+00_JPRB ,&
&  0.25475E+00_JPRB, 0.24881E+00_JPRB, 0.25054E+00_JPRB, 0.26084E+00_JPRB, 0.28942E+00_JPRB, 0.34351E+00_JPRB, 0.36522E+00_JPRB ,&
&  0.38377E+00_JPRB, 0.39559E+00_JPRB, 0.40705E+00_JPRB, 0.42220E+00_JPRB, 0.43484E+00_JPRB, 0.44459E+00_JPRB, 0.45382E+00_JPRB ,&
&  0.46706E+00_JPRB, 0.48046E+00_JPRB, 0.49871E+00_JPRB, 0.46404E+00_JPRB, 0.47536E+00_JPRB, 0.55387E+00_JPRB, 0.59612E+00_JPRB ,&
&  0.65938E+00_JPRB, 0.67574E+00_JPRB, 0.67983E+00_JPRB, 0.73144E+00_JPRB, 0.75263E+00_JPRB, 0.78339E+00_JPRB, 0.79026E+00_JPRB ,&
&  0.78514E+00_JPRB, 0.77035E+00_JPRB, 0.75460E+00_JPRB, 0.74032E+00_JPRB, 0.74799E+00_JPRB, 0.77064E+00_JPRB, 0.76037E+00_JPRB ,&
&  0.74842E+00_JPRB, 0.74074E+00_JPRB, 0.68507E+00_JPRB, 0.60196E+00_JPRB, 0.57217E+00_JPRB, 0.56752E+00_JPRB, 0.57362E+00_JPRB ,&
&  0.57033E+00_JPRB, 0.57302E+00_JPRB, 0.57610E+00_JPRB, 0.57065E+00_JPRB, 0.51165E+00_JPRB, 0.45728E+00_JPRB, 0.41304E+00_JPRB ,&
&  0.38981E+00_JPRB, 0.38506E+00_JPRB, 0.37673E+00_JPRB, 0.37640E+00_JPRB, 0.37068E+00_JPRB, 0.36610E+00_JPRB, 0.35897E+00_JPRB ,&
&  0.35838E+00_JPRB, 0.36388E+00_JPRB, 0.37983E+00_JPRB, 0.39897E+00_JPRB, 0.41186E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.45192E+00_JPRB, 0.45679E+00_JPRB, 0.44574E+00_JPRB, 0.43819E+00_JPRB, 0.42700E+00_JPRB, 0.41378E+00_JPRB, 0.39781E+00_JPRB ,&
&  0.39732E+00_JPRB, 0.37722E+00_JPRB, 0.36218E+00_JPRB, 0.34899E+00_JPRB, 0.34137E+00_JPRB, 0.33739E+00_JPRB, 0.34158E+00_JPRB ,&
&  0.35185E+00_JPRB, 0.36523E+00_JPRB, 0.37335E+00_JPRB, 0.37334E+00_JPRB, 0.36886E+00_JPRB, 0.35891E+00_JPRB, 0.35162E+00_JPRB ,&
&  0.34138E+00_JPRB, 0.33614E+00_JPRB, 0.33658E+00_JPRB, 0.33949E+00_JPRB, 0.34263E+00_JPRB, 0.34246E+00_JPRB, 0.33863E+00_JPRB ,&
&  0.33302E+00_JPRB, 0.33004E+00_JPRB, 0.33226E+00_JPRB, 0.33497E+00_JPRB, 0.32901E+00_JPRB, 0.30732E+00_JPRB, 0.28523E+00_JPRB ,&
&  0.26969E+00_JPRB, 0.26904E+00_JPRB, 0.29224E+00_JPRB, 0.34854E+00_JPRB, 0.37043E+00_JPRB, 0.38609E+00_JPRB, 0.39736E+00_JPRB ,&
&  0.39321E+00_JPRB, 0.39779E+00_JPRB, 0.41979E+00_JPRB, 0.46144E+00_JPRB, 0.46967E+00_JPRB, 0.47793E+00_JPRB, 0.48620E+00_JPRB ,&
&  0.50460E+00_JPRB, 0.50447E+00_JPRB, 0.51838E+00_JPRB, 0.50998E+00_JPRB, 0.60700E+00_JPRB, 0.80248E+00_JPRB, 0.74875E+00_JPRB ,&
&  0.77950E+00_JPRB, 0.79094E+00_JPRB, 0.84637E+00_JPRB, 0.86753E+00_JPRB, 0.86201E+00_JPRB, 0.86277E+00_JPRB, 0.85526E+00_JPRB ,&
&  0.82073E+00_JPRB, 0.80118E+00_JPRB, 0.77308E+00_JPRB, 0.76962E+00_JPRB, 0.76731E+00_JPRB, 0.77508E+00_JPRB, 0.77305E+00_JPRB ,&
&  0.76497E+00_JPRB, 0.75204E+00_JPRB, 0.70188E+00_JPRB, 0.63569E+00_JPRB, 0.60155E+00_JPRB, 0.57247E+00_JPRB, 0.55438E+00_JPRB ,&
&  0.54423E+00_JPRB, 0.55463E+00_JPRB, 0.57000E+00_JPRB, 0.56953E+00_JPRB, 0.51841E+00_JPRB, 0.45722E+00_JPRB, 0.40506E+00_JPRB ,&
&  0.37379E+00_JPRB, 0.36508E+00_JPRB, 0.34995E+00_JPRB, 0.34439E+00_JPRB, 0.35051E+00_JPRB, 0.35899E+00_JPRB, 0.36207E+00_JPRB ,&
&  0.36588E+00_JPRB, 0.37532E+00_JPRB, 0.39950E+00_JPRB, 0.42567E+00_JPRB, 0.44184E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.45161E+00_JPRB, 0.45784E+00_JPRB, 0.45744E+00_JPRB, 0.45922E+00_JPRB, 0.45824E+00_JPRB, 0.44362E+00_JPRB, 0.41799E+00_JPRB ,&
&  0.42161E+00_JPRB, 0.37615E+00_JPRB, 0.36089E+00_JPRB, 0.35214E+00_JPRB, 0.35065E+00_JPRB, 0.34543E+00_JPRB, 0.33681E+00_JPRB ,&
&  0.33821E+00_JPRB, 0.34881E+00_JPRB, 0.36584E+00_JPRB, 0.37356E+00_JPRB, 0.37173E+00_JPRB, 0.36218E+00_JPRB, 0.34878E+00_JPRB ,&
&  0.33647E+00_JPRB, 0.33339E+00_JPRB, 0.33918E+00_JPRB, 0.34796E+00_JPRB, 0.35346E+00_JPRB, 0.35242E+00_JPRB, 0.34900E+00_JPRB ,&
&  0.34470E+00_JPRB, 0.34045E+00_JPRB, 0.33982E+00_JPRB, 0.34150E+00_JPRB, 0.34167E+00_JPRB, 0.33476E+00_JPRB, 0.31580E+00_JPRB ,&
&  0.29876E+00_JPRB, 0.30709E+00_JPRB, 0.36365E+00_JPRB, 0.39223E+00_JPRB, 0.41245E+00_JPRB, 0.42251E+00_JPRB, 0.43268E+00_JPRB ,&
&  0.44627E+00_JPRB, 0.46236E+00_JPRB, 0.48179E+00_JPRB, 0.50581E+00_JPRB, 0.52067E+00_JPRB, 0.53321E+00_JPRB, 0.54513E+00_JPRB ,&
&  0.58472E+00_JPRB, 0.50709E+00_JPRB, 0.48718E+00_JPRB, 0.52411E+00_JPRB, 0.76849E+00_JPRB, 0.85079E+00_JPRB, 0.87187E+00_JPRB ,&
&  0.92306E+00_JPRB, 0.88996E+00_JPRB, 0.90880E+00_JPRB, 0.94550E+00_JPRB, 0.96336E+00_JPRB, 0.96534E+00_JPRB, 0.95850E+00_JPRB ,&
&  0.93822E+00_JPRB, 0.90934E+00_JPRB, 0.85903E+00_JPRB, 0.82427E+00_JPRB, 0.80087E+00_JPRB, 0.80941E+00_JPRB, 0.79801E+00_JPRB ,&
&  0.77524E+00_JPRB, 0.76508E+00_JPRB, 0.71755E+00_JPRB, 0.66394E+00_JPRB, 0.62289E+00_JPRB, 0.59596E+00_JPRB, 0.57541E+00_JPRB ,&
&  0.56168E+00_JPRB, 0.56475E+00_JPRB, 0.57436E+00_JPRB, 0.58036E+00_JPRB, 0.55073E+00_JPRB, 0.50166E+00_JPRB, 0.45614E+00_JPRB ,&
&  0.40747E+00_JPRB, 0.38399E+00_JPRB, 0.35792E+00_JPRB, 0.35269E+00_JPRB, 0.36387E+00_JPRB, 0.37735E+00_JPRB, 0.38938E+00_JPRB ,&
&  0.40424E+00_JPRB, 0.41508E+00_JPRB, 0.42617E+00_JPRB, 0.44170E+00_JPRB, 0.44666E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.45135E+00_JPRB, 0.46369E+00_JPRB, 0.47191E+00_JPRB, 0.48164E+00_JPRB, 0.48422E+00_JPRB, 0.46131E+00_JPRB, 0.44978E+00_JPRB ,&
&  0.48606E+00_JPRB, 0.43699E+00_JPRB, 0.45702E+00_JPRB, 0.39510E+00_JPRB, 0.37086E+00_JPRB, 0.36395E+00_JPRB, 0.35166E+00_JPRB ,&
&  0.34928E+00_JPRB, 0.35682E+00_JPRB, 0.36564E+00_JPRB, 0.37321E+00_JPRB, 0.37593E+00_JPRB, 0.36426E+00_JPRB, 0.35192E+00_JPRB ,&
&  0.33817E+00_JPRB, 0.33443E+00_JPRB, 0.34114E+00_JPRB, 0.34895E+00_JPRB, 0.35608E+00_JPRB, 0.35787E+00_JPRB, 0.35515E+00_JPRB ,&
&  0.34842E+00_JPRB, 0.34563E+00_JPRB, 0.34793E+00_JPRB, 0.34841E+00_JPRB, 0.35068E+00_JPRB, 0.34799E+00_JPRB, 0.34582E+00_JPRB ,&
&  0.34843E+00_JPRB, 0.35668E+00_JPRB, 0.39686E+00_JPRB, 0.42489E+00_JPRB, 0.44323E+00_JPRB, 0.45214E+00_JPRB, 0.46601E+00_JPRB ,&
&  0.47974E+00_JPRB, 0.49697E+00_JPRB, 0.52379E+00_JPRB, 0.55647E+00_JPRB, 0.58377E+00_JPRB, 0.60264E+00_JPRB, 0.62488E+00_JPRB ,&
&  0.67203E+00_JPRB, 0.55460E+00_JPRB, 0.63664E+00_JPRB, 0.65062E+00_JPRB, 0.80213E+00_JPRB, 0.98068E+00_JPRB, 0.95820E+00_JPRB ,&
&  0.10816E+01_JPRB, 0.10342E+01_JPRB, 0.10500E+01_JPRB, 0.10494E+01_JPRB, 0.10628E+01_JPRB, 0.10723E+01_JPRB, 0.10740E+01_JPRB ,&
&  0.11075E+01_JPRB, 0.99755E+00_JPRB, 0.98489E+00_JPRB, 0.92553E+00_JPRB, 0.88328E+00_JPRB, 0.87764E+00_JPRB, 0.84018E+00_JPRB ,&
&  0.81041E+00_JPRB, 0.79466E+00_JPRB, 0.76376E+00_JPRB, 0.67900E+00_JPRB, 0.65323E+00_JPRB, 0.63121E+00_JPRB, 0.58390E+00_JPRB ,&
&  0.56631E+00_JPRB, 0.54735E+00_JPRB, 0.52923E+00_JPRB, 0.56177E+00_JPRB, 0.54634E+00_JPRB, 0.50315E+00_JPRB, 0.45719E+00_JPRB ,&
&  0.41937E+00_JPRB, 0.39258E+00_JPRB, 0.40149E+00_JPRB, 0.40474E+00_JPRB, 0.40053E+00_JPRB, 0.40305E+00_JPRB, 0.43078E+00_JPRB ,&
&  0.42952E+00_JPRB, 0.43818E+00_JPRB, 0.45792E+00_JPRB, 0.45962E+00_JPRB, 0.45301E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.46013E+00_JPRB, 0.47675E+00_JPRB, 0.48373E+00_JPRB, 0.49799E+00_JPRB, 0.50574E+00_JPRB, 0.48898E+00_JPRB, 0.46480E+00_JPRB ,&
&  0.47240E+00_JPRB, 0.47069E+00_JPRB, 0.46988E+00_JPRB, 0.47066E+00_JPRB, 0.47267E+00_JPRB, 0.43149E+00_JPRB, 0.36296E+00_JPRB ,&
&  0.34138E+00_JPRB, 0.34696E+00_JPRB, 0.36163E+00_JPRB, 0.37192E+00_JPRB, 0.37247E+00_JPRB, 0.36534E+00_JPRB, 0.35834E+00_JPRB ,&
&  0.35153E+00_JPRB, 0.34675E+00_JPRB, 0.34987E+00_JPRB, 0.35300E+00_JPRB, 0.35541E+00_JPRB, 0.36009E+00_JPRB, 0.36215E+00_JPRB ,&
&  0.35636E+00_JPRB, 0.35577E+00_JPRB, 0.35529E+00_JPRB, 0.34992E+00_JPRB, 0.36219E+00_JPRB, 0.36100E+00_JPRB, 0.36041E+00_JPRB ,&
&  0.37517E+00_JPRB, 0.39887E+00_JPRB, 0.42235E+00_JPRB, 0.45096E+00_JPRB, 0.47405E+00_JPRB, 0.48859E+00_JPRB, 0.50493E+00_JPRB ,&
&  0.52675E+00_JPRB, 0.55704E+00_JPRB, 0.59255E+00_JPRB, 0.63331E+00_JPRB, 0.67802E+00_JPRB, 0.66667E+00_JPRB, 0.73204E+00_JPRB ,&
&  0.74144E+00_JPRB, 0.78330E+00_JPRB, 0.80220E+00_JPRB, 0.74883E+00_JPRB, 0.90395E+00_JPRB, 0.10487E+01_JPRB, 0.10279E+01_JPRB ,&
&  0.10699E+01_JPRB, 0.10065E+01_JPRB, 0.10948E+01_JPRB, 0.11894E+01_JPRB, 0.12643E+01_JPRB, 0.12421E+01_JPRB, 0.12170E+01_JPRB ,&
&  0.12737E+01_JPRB, 0.12766E+01_JPRB, 0.11163E+01_JPRB, 0.10554E+01_JPRB, 0.99297E+00_JPRB, 0.97397E+00_JPRB, 0.91073E+00_JPRB ,&
&  0.87887E+00_JPRB, 0.86662E+00_JPRB, 0.83200E+00_JPRB, 0.78836E+00_JPRB, 0.73710E+00_JPRB, 0.69111E+00_JPRB, 0.64809E+00_JPRB ,&
&  0.59800E+00_JPRB, 0.53425E+00_JPRB, 0.50637E+00_JPRB, 0.51388E+00_JPRB, 0.53165E+00_JPRB, 0.47895E+00_JPRB, 0.43103E+00_JPRB ,&
&  0.41495E+00_JPRB, 0.41713E+00_JPRB, 0.44867E+00_JPRB, 0.44855E+00_JPRB, 0.44171E+00_JPRB, 0.43488E+00_JPRB, 0.42199E+00_JPRB ,&
&  0.41461E+00_JPRB, 0.44877E+00_JPRB, 0.46060E+00_JPRB, 0.45894E+00_JPRB, 0.45498E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.47912E+00_JPRB, 0.50087E+00_JPRB, 0.50538E+00_JPRB, 0.52027E+00_JPRB, 0.53270E+00_JPRB, 0.51073E+00_JPRB, 0.50110E+00_JPRB ,&
&  0.49843E+00_JPRB, 0.49392E+00_JPRB, 0.48644E+00_JPRB, 0.48283E+00_JPRB, 0.48151E+00_JPRB, 0.47311E+00_JPRB, 0.42531E+00_JPRB ,&
&  0.37104E+00_JPRB, 0.36673E+00_JPRB, 0.36276E+00_JPRB, 0.37143E+00_JPRB, 0.38041E+00_JPRB, 0.37423E+00_JPRB, 0.36886E+00_JPRB ,&
&  0.36875E+00_JPRB, 0.36390E+00_JPRB, 0.36074E+00_JPRB, 0.36128E+00_JPRB, 0.36516E+00_JPRB, 0.37295E+00_JPRB, 0.37638E+00_JPRB ,&
&  0.36768E+00_JPRB, 0.36254E+00_JPRB, 0.35973E+00_JPRB, 0.36000E+00_JPRB, 0.37207E+00_JPRB, 0.37769E+00_JPRB, 0.38242E+00_JPRB ,&
&  0.39786E+00_JPRB, 0.42081E+00_JPRB, 0.44477E+00_JPRB, 0.47956E+00_JPRB, 0.51253E+00_JPRB, 0.54026E+00_JPRB, 0.57352E+00_JPRB ,&
&  0.63037E+00_JPRB, 0.68495E+00_JPRB, 0.71589E+00_JPRB, 0.75841E+00_JPRB, 0.79001E+00_JPRB, 0.82899E+00_JPRB, 0.86632E+00_JPRB ,&
&  0.88958E+00_JPRB, 0.90319E+00_JPRB, 0.87036E+00_JPRB, 0.90073E+00_JPRB, 0.10543E+01_JPRB, 0.11405E+01_JPRB, 0.11538E+01_JPRB ,&
&  0.12154E+01_JPRB, 0.11468E+01_JPRB, 0.14171E+01_JPRB, 0.14870E+01_JPRB, 0.14242E+01_JPRB, 0.14305E+01_JPRB, 0.13447E+01_JPRB ,&
&  0.13331E+01_JPRB, 0.12656E+01_JPRB, 0.11908E+01_JPRB, 0.11329E+01_JPRB, 0.10795E+01_JPRB, 0.10889E+01_JPRB, 0.10272E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.98748E+00_JPRB, 0.85280E+00_JPRB, 0.75893E+00_JPRB, 0.66485E+00_JPRB, 0.69727E+00_JPRB, 0.67041E+00_JPRB ,&
&  0.61099E+00_JPRB, 0.52372E+00_JPRB, 0.53777E+00_JPRB, 0.55079E+00_JPRB, 0.58086E+00_JPRB, 0.56322E+00_JPRB, 0.48310E+00_JPRB ,&
&  0.45212E+00_JPRB, 0.46985E+00_JPRB, 0.47166E+00_JPRB, 0.47351E+00_JPRB, 0.47315E+00_JPRB, 0.46692E+00_JPRB, 0.43245E+00_JPRB ,&
&  0.42255E+00_JPRB, 0.45208E+00_JPRB, 0.46480E+00_JPRB, 0.46474E+00_JPRB, 0.46911E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.50442E+00_JPRB, 0.53207E+00_JPRB, 0.54258E+00_JPRB, 0.54994E+00_JPRB, 0.55986E+00_JPRB, 0.53682E+00_JPRB, 0.52746E+00_JPRB ,&
&  0.52867E+00_JPRB, 0.52156E+00_JPRB, 0.50444E+00_JPRB, 0.49925E+00_JPRB, 0.49611E+00_JPRB, 0.48723E+00_JPRB, 0.47317E+00_JPRB ,&
&  0.40064E+00_JPRB, 0.38137E+00_JPRB, 0.36105E+00_JPRB, 0.36431E+00_JPRB, 0.39854E+00_JPRB, 0.39160E+00_JPRB, 0.38850E+00_JPRB ,&
&  0.38574E+00_JPRB, 0.38402E+00_JPRB, 0.37641E+00_JPRB, 0.37452E+00_JPRB, 0.37837E+00_JPRB, 0.39024E+00_JPRB, 0.40233E+00_JPRB ,&
&  0.39492E+00_JPRB, 0.38315E+00_JPRB, 0.37690E+00_JPRB, 0.37723E+00_JPRB, 0.38785E+00_JPRB, 0.40007E+00_JPRB, 0.41473E+00_JPRB ,&
&  0.43358E+00_JPRB, 0.45361E+00_JPRB, 0.48718E+00_JPRB, 0.53630E+00_JPRB, 0.57877E+00_JPRB, 0.61774E+00_JPRB, 0.68663E+00_JPRB ,&
&  0.78296E+00_JPRB, 0.85681E+00_JPRB, 0.89793E+00_JPRB, 0.93397E+00_JPRB, 0.93831E+00_JPRB, 0.97825E+00_JPRB, 0.98602E+00_JPRB ,&
&  0.10451E+01_JPRB, 0.10000E+01_JPRB, 0.97202E+00_JPRB, 0.11455E+01_JPRB, 0.13348E+01_JPRB, 0.13949E+01_JPRB, 0.13632E+01_JPRB ,&
&  0.13714E+01_JPRB, 0.13143E+01_JPRB, 0.14095E+01_JPRB, 0.15027E+01_JPRB, 0.14514E+01_JPRB, 0.15041E+01_JPRB, 0.13933E+01_JPRB ,&
&  0.12488E+01_JPRB, 0.11650E+01_JPRB, 0.11323E+01_JPRB, 0.11365E+01_JPRB, 0.10949E+01_JPRB, 0.11497E+01_JPRB, 0.11305E+01_JPRB ,&
&  0.11452E+01_JPRB, 0.10123E+01_JPRB, 0.77020E+00_JPRB, 0.71347E+00_JPRB, 0.58184E+00_JPRB, 0.59621E+00_JPRB, 0.72046E+00_JPRB ,&
&  0.69758E+00_JPRB, 0.63644E+00_JPRB, 0.62763E+00_JPRB, 0.64460E+00_JPRB, 0.68372E+00_JPRB, 0.75132E+00_JPRB, 0.64687E+00_JPRB ,&
&  0.54157E+00_JPRB, 0.53159E+00_JPRB, 0.53476E+00_JPRB, 0.52777E+00_JPRB, 0.52048E+00_JPRB, 0.51975E+00_JPRB, 0.50797E+00_JPRB ,&
&  0.49535E+00_JPRB, 0.48921E+00_JPRB, 0.49000E+00_JPRB, 0.48911E+00_JPRB, 0.49695E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.53413E+00_JPRB, 0.55847E+00_JPRB, 0.57430E+00_JPRB, 0.57697E+00_JPRB, 0.58728E+00_JPRB, 0.57218E+00_JPRB, 0.55735E+00_JPRB ,&
&  0.55487E+00_JPRB, 0.54438E+00_JPRB, 0.52364E+00_JPRB, 0.51781E+00_JPRB, 0.51112E+00_JPRB, 0.49819E+00_JPRB, 0.48424E+00_JPRB ,&
&  0.47999E+00_JPRB, 0.42978E+00_JPRB, 0.39163E+00_JPRB, 0.36955E+00_JPRB, 0.39373E+00_JPRB, 0.39276E+00_JPRB, 0.39986E+00_JPRB ,&
&  0.39684E+00_JPRB, 0.40657E+00_JPRB, 0.40362E+00_JPRB, 0.40050E+00_JPRB, 0.40837E+00_JPRB, 0.42374E+00_JPRB, 0.43714E+00_JPRB ,&
&  0.43656E+00_JPRB, 0.43267E+00_JPRB, 0.42510E+00_JPRB, 0.42741E+00_JPRB, 0.44852E+00_JPRB, 0.44747E+00_JPRB, 0.47121E+00_JPRB ,&
&  0.49127E+00_JPRB, 0.51270E+00_JPRB, 0.54689E+00_JPRB, 0.60264E+00_JPRB, 0.65680E+00_JPRB, 0.71355E+00_JPRB, 0.80825E+00_JPRB ,&
&  0.90203E+00_JPRB, 0.94252E+00_JPRB, 0.94453E+00_JPRB, 0.96940E+00_JPRB, 0.10442E+01_JPRB, 0.11342E+01_JPRB, 0.11091E+01_JPRB ,&
&  0.10859E+01_JPRB, 0.99601E+00_JPRB, 0.91752E+00_JPRB, 0.11902E+01_JPRB, 0.14452E+01_JPRB, 0.14722E+01_JPRB, 0.14475E+01_JPRB ,&
&  0.14783E+01_JPRB, 0.14905E+01_JPRB, 0.15988E+01_JPRB, 0.16297E+01_JPRB, 0.16166E+01_JPRB, 0.15668E+01_JPRB, 0.14097E+01_JPRB ,&
&  0.12338E+01_JPRB, 0.11537E+01_JPRB, 0.11874E+01_JPRB, 0.12058E+01_JPRB, 0.12142E+01_JPRB, 0.11679E+01_JPRB, 0.12430E+01_JPRB ,&
&  0.13322E+01_JPRB, 0.76072E+00_JPRB, 0.61528E+00_JPRB, 0.70821E+00_JPRB, 0.61676E+00_JPRB, 0.62586E+00_JPRB, 0.69409E+00_JPRB ,&
&  0.66542E+00_JPRB, 0.64657E+00_JPRB, 0.71067E+00_JPRB, 0.73566E+00_JPRB, 0.80701E+00_JPRB, 0.90864E+00_JPRB, 0.80189E+00_JPRB ,&
&  0.70243E+00_JPRB, 0.65175E+00_JPRB, 0.65112E+00_JPRB, 0.63055E+00_JPRB, 0.62990E+00_JPRB, 0.61489E+00_JPRB, 0.60082E+00_JPRB ,&
&  0.57732E+00_JPRB, 0.56131E+00_JPRB, 0.54473E+00_JPRB, 0.53693E+00_JPRB, 0.53475E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.58997E+00_JPRB, 0.60731E+00_JPRB, 0.61475E+00_JPRB, 0.61758E+00_JPRB, 0.62375E+00_JPRB, 0.61807E+00_JPRB, 0.59711E+00_JPRB ,&
&  0.58005E+00_JPRB, 0.56469E+00_JPRB, 0.54330E+00_JPRB, 0.53609E+00_JPRB, 0.52388E+00_JPRB, 0.50894E+00_JPRB, 0.49566E+00_JPRB ,&
&  0.49118E+00_JPRB, 0.47210E+00_JPRB, 0.41232E+00_JPRB, 0.38811E+00_JPRB, 0.36984E+00_JPRB, 0.38227E+00_JPRB, 0.39384E+00_JPRB ,&
&  0.41117E+00_JPRB, 0.42014E+00_JPRB, 0.42604E+00_JPRB, 0.43797E+00_JPRB, 0.45871E+00_JPRB, 0.47230E+00_JPRB, 0.49606E+00_JPRB ,&
&  0.50153E+00_JPRB, 0.50832E+00_JPRB, 0.48792E+00_JPRB, 0.49958E+00_JPRB, 0.52356E+00_JPRB, 0.52344E+00_JPRB, 0.56812E+00_JPRB ,&
&  0.59876E+00_JPRB, 0.63364E+00_JPRB, 0.66893E+00_JPRB, 0.69819E+00_JPRB, 0.73608E+00_JPRB, 0.76654E+00_JPRB, 0.84159E+00_JPRB ,&
&  0.89682E+00_JPRB, 0.91777E+00_JPRB, 0.90422E+00_JPRB, 0.91032E+00_JPRB, 0.95846E+00_JPRB, 0.10153E+01_JPRB, 0.10155E+01_JPRB ,&
&  0.10348E+01_JPRB, 0.86506E+00_JPRB, 0.65287E+00_JPRB, 0.99489E+00_JPRB, 0.14419E+01_JPRB, 0.14270E+01_JPRB, 0.12594E+01_JPRB ,&
&  0.13649E+01_JPRB, 0.14664E+01_JPRB, 0.16484E+01_JPRB, 0.17062E+01_JPRB, 0.16840E+01_JPRB, 0.16003E+01_JPRB, 0.14957E+01_JPRB ,&
&  0.13836E+01_JPRB, 0.13996E+01_JPRB, 0.15090E+01_JPRB, 0.16901E+01_JPRB, 0.17159E+01_JPRB, 0.16985E+01_JPRB, 0.16416E+01_JPRB ,&
&  0.14031E+01_JPRB, 0.11696E+01_JPRB, 0.80632E+00_JPRB, 0.82916E+00_JPRB, 0.65749E+00_JPRB, 0.65279E+00_JPRB, 0.71278E+00_JPRB ,&
&  0.68507E+00_JPRB, 0.72013E+00_JPRB, 0.79688E+00_JPRB, 0.84485E+00_JPRB, 0.10436E+01_JPRB, 0.10619E+01_JPRB, 0.99362E+00_JPRB ,&
&  0.87580E+00_JPRB, 0.76468E+00_JPRB, 0.77981E+00_JPRB, 0.76772E+00_JPRB, 0.77005E+00_JPRB, 0.74359E+00_JPRB, 0.71423E+00_JPRB ,&
&  0.67834E+00_JPRB, 0.64901E+00_JPRB, 0.62765E+00_JPRB, 0.60474E+00_JPRB, 0.58843E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.64902E+00_JPRB, 0.64645E+00_JPRB, 0.63581E+00_JPRB, 0.62562E+00_JPRB, 0.62323E+00_JPRB, 0.61829E+00_JPRB, 0.60597E+00_JPRB ,&
&  0.59788E+00_JPRB, 0.58066E+00_JPRB, 0.56319E+00_JPRB, 0.54799E+00_JPRB, 0.53757E+00_JPRB, 0.52269E+00_JPRB, 0.51128E+00_JPRB ,&
&  0.50556E+00_JPRB, 0.46351E+00_JPRB, 0.39595E+00_JPRB, 0.36698E+00_JPRB, 0.35217E+00_JPRB, 0.36194E+00_JPRB, 0.38222E+00_JPRB ,&
&  0.40668E+00_JPRB, 0.42401E+00_JPRB, 0.45168E+00_JPRB, 0.48885E+00_JPRB, 0.53145E+00_JPRB, 0.56640E+00_JPRB, 0.59123E+00_JPRB ,&
&  0.66527E+00_JPRB, 0.60719E+00_JPRB, 0.62020E+00_JPRB, 0.60837E+00_JPRB, 0.64535E+00_JPRB, 0.66630E+00_JPRB, 0.67329E+00_JPRB ,&
&  0.67674E+00_JPRB, 0.68177E+00_JPRB, 0.69021E+00_JPRB, 0.69268E+00_JPRB, 0.70892E+00_JPRB, 0.75331E+00_JPRB, 0.84766E+00_JPRB ,&
&  0.92514E+00_JPRB, 0.91333E+00_JPRB, 0.87343E+00_JPRB, 0.90638E+00_JPRB, 0.97208E+00_JPRB, 0.10110E+01_JPRB, 0.10120E+01_JPRB ,&
&  0.96979E+00_JPRB, 0.89091E+00_JPRB, 0.10480E+01_JPRB, 0.12251E+01_JPRB, 0.14557E+01_JPRB, 0.15103E+01_JPRB, 0.14204E+01_JPRB ,&
&  0.14986E+01_JPRB, 0.15413E+01_JPRB, 0.16367E+01_JPRB, 0.13369E+01_JPRB, 0.15119E+01_JPRB, 0.15148E+01_JPRB, 0.15051E+01_JPRB ,&
&  0.15106E+01_JPRB, 0.16304E+01_JPRB, 0.17736E+01_JPRB, 0.20345E+01_JPRB, 0.22239E+01_JPRB, 0.23511E+01_JPRB, 0.17842E+01_JPRB ,&
&  0.13704E+01_JPRB, 0.14393E+01_JPRB, 0.12927E+01_JPRB, 0.10468E+01_JPRB, 0.85667E+00_JPRB, 0.85365E+00_JPRB, 0.83551E+00_JPRB ,&
&  0.82406E+00_JPRB, 0.85944E+00_JPRB, 0.92058E+00_JPRB, 0.11260E+01_JPRB, 0.13454E+01_JPRB, 0.13763E+01_JPRB, 0.12340E+01_JPRB ,&
&  0.10237E+01_JPRB, 0.10255E+01_JPRB, 0.97866E+00_JPRB, 0.93689E+00_JPRB, 0.93867E+00_JPRB, 0.89877E+00_JPRB, 0.83528E+00_JPRB ,&
&  0.77168E+00_JPRB, 0.73227E+00_JPRB, 0.70324E+00_JPRB, 0.67520E+00_JPRB, 0.65469E+00_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.69264E+00_JPRB, 0.68366E+00_JPRB, 0.66689E+00_JPRB, 0.65245E+00_JPRB, 0.63984E+00_JPRB, 0.62848E+00_JPRB, 0.61680E+00_JPRB ,&
&  0.61611E+00_JPRB, 0.58311E+00_JPRB, 0.57033E+00_JPRB, 0.55910E+00_JPRB, 0.55069E+00_JPRB, 0.54020E+00_JPRB, 0.53356E+00_JPRB ,&
&  0.52717E+00_JPRB, 0.45075E+00_JPRB, 0.38215E+00_JPRB, 0.37609E+00_JPRB, 0.37404E+00_JPRB, 0.36637E+00_JPRB, 0.38183E+00_JPRB ,&
&  0.40611E+00_JPRB, 0.43505E+00_JPRB, 0.47680E+00_JPRB, 0.55755E+00_JPRB, 0.63779E+00_JPRB, 0.70293E+00_JPRB, 0.75820E+00_JPRB ,&
&  0.74848E+00_JPRB, 0.76142E+00_JPRB, 0.79384E+00_JPRB, 0.76192E+00_JPRB, 0.73348E+00_JPRB, 0.71932E+00_JPRB, 0.71866E+00_JPRB ,&
&  0.71866E+00_JPRB, 0.70912E+00_JPRB, 0.71407E+00_JPRB, 0.72637E+00_JPRB, 0.75168E+00_JPRB, 0.81138E+00_JPRB, 0.89623E+00_JPRB ,&
&  0.93552E+00_JPRB, 0.90891E+00_JPRB, 0.89226E+00_JPRB, 0.97088E+00_JPRB, 0.90393E+00_JPRB, 0.86743E+00_JPRB, 0.84066E+00_JPRB ,&
&  0.10581E+01_JPRB, 0.11974E+01_JPRB, 0.11949E+01_JPRB, 0.13245E+01_JPRB, 0.12346E+01_JPRB, 0.13243E+01_JPRB, 0.15256E+01_JPRB ,&
&  0.16458E+01_JPRB, 0.16598E+01_JPRB, 0.17094E+01_JPRB, 0.11824E+01_JPRB, 0.10936E+01_JPRB, 0.14988E+01_JPRB, 0.16069E+01_JPRB ,&
&  0.16504E+01_JPRB, 0.18972E+01_JPRB, 0.22312E+01_JPRB, 0.26314E+01_JPRB, 0.20058E+01_JPRB, 0.17322E+01_JPRB, 0.13056E+01_JPRB ,&
&  0.12540E+01_JPRB, 0.11346E+01_JPRB, 0.12731E+01_JPRB, 0.11683E+01_JPRB, 0.11284E+01_JPRB, 0.10815E+01_JPRB, 0.10041E+01_JPRB ,&
&  0.99503E+00_JPRB, 0.99874E+00_JPRB, 0.11447E+01_JPRB, 0.14862E+01_JPRB, 0.17689E+01_JPRB, 0.16328E+01_JPRB, 0.15696E+01_JPRB ,&
&  0.14826E+01_JPRB, 0.13748E+01_JPRB, 0.12162E+01_JPRB, 0.11683E+01_JPRB, 0.10975E+01_JPRB, 0.10103E+01_JPRB, 0.91147E+00_JPRB ,&
&  0.83227E+00_JPRB, 0.78769E+00_JPRB, 0.74797E+00_JPRB, 0.71887E+00_JPRB, 0.70184E+00_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.73545E+00_JPRB, 0.71944E+00_JPRB, 0.70217E+00_JPRB, 0.68412E+00_JPRB, 0.66783E+00_JPRB, 0.65157E+00_JPRB, 0.64021E+00_JPRB ,&
&  0.64326E+00_JPRB, 0.60613E+00_JPRB, 0.57616E+00_JPRB, 0.56457E+00_JPRB, 0.56107E+00_JPRB, 0.55962E+00_JPRB, 0.55921E+00_JPRB ,&
&  0.55428E+00_JPRB, 0.49965E+00_JPRB, 0.40752E+00_JPRB, 0.38290E+00_JPRB, 0.39665E+00_JPRB, 0.38694E+00_JPRB, 0.38564E+00_JPRB ,&
&  0.42083E+00_JPRB, 0.47008E+00_JPRB, 0.54666E+00_JPRB, 0.67400E+00_JPRB, 0.78364E+00_JPRB, 0.87467E+00_JPRB, 0.92893E+00_JPRB ,&
&  0.93066E+00_JPRB, 0.86819E+00_JPRB, 0.79212E+00_JPRB, 0.72191E+00_JPRB, 0.68910E+00_JPRB, 0.67779E+00_JPRB, 0.68675E+00_JPRB ,&
&  0.70221E+00_JPRB, 0.71157E+00_JPRB, 0.72331E+00_JPRB, 0.73048E+00_JPRB, 0.77430E+00_JPRB, 0.81523E+00_JPRB, 0.88308E+00_JPRB ,&
&  0.91333E+00_JPRB, 0.90418E+00_JPRB, 0.93633E+00_JPRB, 0.10080E+01_JPRB, 0.10033E+01_JPRB, 0.93695E+00_JPRB, 0.91603E+00_JPRB ,&
&  0.10203E+01_JPRB, 0.11352E+01_JPRB, 0.12303E+01_JPRB, 0.13342E+01_JPRB, 0.14516E+01_JPRB, 0.15210E+01_JPRB, 0.17170E+01_JPRB ,&
&  0.14592E+01_JPRB, 0.11777E+01_JPRB, 0.99025E+00_JPRB, 0.86296E+00_JPRB, 0.12266E+01_JPRB, 0.17806E+01_JPRB, 0.20379E+01_JPRB ,&
&  0.22740E+01_JPRB, 0.25366E+01_JPRB, 0.28242E+01_JPRB, 0.25330E+01_JPRB, 0.19365E+01_JPRB, 0.14825E+01_JPRB, 0.17501E+01_JPRB ,&
&  0.17941E+01_JPRB, 0.17514E+01_JPRB, 0.16089E+01_JPRB, 0.14050E+01_JPRB, 0.11143E+01_JPRB, 0.12601E+01_JPRB, 0.12099E+01_JPRB ,&
&  0.11588E+01_JPRB, 0.11474E+01_JPRB, 0.15759E+01_JPRB, 0.20265E+01_JPRB, 0.21069E+01_JPRB, 0.20898E+01_JPRB, 0.20376E+01_JPRB ,&
&  0.18411E+01_JPRB, 0.17193E+01_JPRB, 0.15152E+01_JPRB, 0.13289E+01_JPRB, 0.11744E+01_JPRB, 0.10394E+01_JPRB, 0.93209E+00_JPRB ,&
&  0.86564E+00_JPRB, 0.82154E+00_JPRB, 0.77822E+00_JPRB, 0.75915E+00_JPRB, 0.74615E+00_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.79912E+00_JPRB, 0.77642E+00_JPRB, 0.75537E+00_JPRB, 0.73580E+00_JPRB, 0.71563E+00_JPRB, 0.68580E+00_JPRB, 0.67527E+00_JPRB ,&
&  0.66979E+00_JPRB, 0.63319E+00_JPRB, 0.59882E+00_JPRB, 0.58593E+00_JPRB, 0.59257E+00_JPRB, 0.59593E+00_JPRB, 0.59191E+00_JPRB ,&
&  0.58584E+00_JPRB, 0.58521E+00_JPRB, 0.46827E+00_JPRB, 0.42175E+00_JPRB, 0.41949E+00_JPRB, 0.41353E+00_JPRB, 0.39889E+00_JPRB ,&
&  0.45724E+00_JPRB, 0.54566E+00_JPRB, 0.64896E+00_JPRB, 0.79274E+00_JPRB, 0.96852E+00_JPRB, 0.95719E+00_JPRB, 0.95027E+00_JPRB ,&
&  0.95937E+00_JPRB, 0.90604E+00_JPRB, 0.82689E+00_JPRB, 0.76639E+00_JPRB, 0.73364E+00_JPRB, 0.72374E+00_JPRB, 0.73217E+00_JPRB ,&
&  0.74973E+00_JPRB, 0.76222E+00_JPRB, 0.76963E+00_JPRB, 0.79331E+00_JPRB, 0.80301E+00_JPRB, 0.83541E+00_JPRB, 0.90062E+00_JPRB ,&
&  0.92107E+00_JPRB, 0.90483E+00_JPRB, 0.94222E+00_JPRB, 0.10525E+01_JPRB, 0.11350E+01_JPRB, 0.10499E+01_JPRB, 0.12335E+01_JPRB ,&
&  0.11530E+01_JPRB, 0.12998E+01_JPRB, 0.13868E+01_JPRB, 0.16565E+01_JPRB, 0.17172E+01_JPRB, 0.16184E+01_JPRB, 0.16845E+01_JPRB ,&
&  0.13865E+01_JPRB, 0.12474E+01_JPRB, 0.13955E+01_JPRB, 0.15864E+01_JPRB, 0.17339E+01_JPRB, 0.18795E+01_JPRB, 0.28862E+01_JPRB ,&
&  0.24858E+01_JPRB, 0.27991E+01_JPRB, 0.30236E+01_JPRB, 0.29417E+01_JPRB, 0.14109E+01_JPRB, 0.10331E+01_JPRB, 0.14783E+01_JPRB ,&
&  0.15981E+01_JPRB, 0.11067E+01_JPRB, 0.80210E+00_JPRB, 0.97191E+00_JPRB, 0.74084E+00_JPRB, 0.88361E+00_JPRB, 0.11647E+01_JPRB ,&
&  0.12462E+01_JPRB, 0.13663E+01_JPRB, 0.19665E+01_JPRB, 0.22987E+01_JPRB, 0.23833E+01_JPRB, 0.27658E+01_JPRB, 0.23513E+01_JPRB ,&
&  0.21453E+01_JPRB, 0.18295E+01_JPRB, 0.17177E+01_JPRB, 0.14744E+01_JPRB, 0.12765E+01_JPRB, 0.11425E+01_JPRB, 0.10400E+01_JPRB ,&
&  0.96450E+00_JPRB, 0.90086E+00_JPRB, 0.85763E+00_JPRB, 0.83560E+00_JPRB, 0.81940E+00_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.88472E+00_JPRB, 0.84973E+00_JPRB, 0.81859E+00_JPRB, 0.79298E+00_JPRB, 0.76055E+00_JPRB, 0.72983E+00_JPRB, 0.71968E+00_JPRB ,&
&  0.69867E+00_JPRB, 0.66248E+00_JPRB, 0.63710E+00_JPRB, 0.61523E+00_JPRB, 0.61597E+00_JPRB, 0.61920E+00_JPRB, 0.60539E+00_JPRB ,&
&  0.58732E+00_JPRB, 0.57705E+00_JPRB, 0.53603E+00_JPRB, 0.49442E+00_JPRB, 0.47204E+00_JPRB, 0.45745E+00_JPRB, 0.46816E+00_JPRB ,&
&  0.55990E+00_JPRB, 0.71370E+00_JPRB, 0.82409E+00_JPRB, 0.93006E+00_JPRB, 0.10067E+01_JPRB, 0.97583E+00_JPRB, 0.10332E+01_JPRB ,&
&  0.10555E+01_JPRB, 0.10158E+01_JPRB, 0.93556E+00_JPRB, 0.86155E+00_JPRB, 0.82047E+00_JPRB, 0.79849E+00_JPRB, 0.78564E+00_JPRB ,&
&  0.79082E+00_JPRB, 0.80737E+00_JPRB, 0.81937E+00_JPRB, 0.81385E+00_JPRB, 0.81129E+00_JPRB, 0.84874E+00_JPRB, 0.91386E+00_JPRB ,&
&  0.92971E+00_JPRB, 0.91279E+00_JPRB, 0.96803E+00_JPRB, 0.10536E+01_JPRB, 0.12639E+01_JPRB, 0.13213E+01_JPRB, 0.11017E+01_JPRB ,&
&  0.96483E+00_JPRB, 0.10669E+01_JPRB, 0.14807E+01_JPRB, 0.16392E+01_JPRB, 0.16258E+01_JPRB, 0.15944E+01_JPRB, 0.14996E+01_JPRB ,&
&  0.14924E+01_JPRB, 0.17769E+01_JPRB, 0.19625E+01_JPRB, 0.22412E+01_JPRB, 0.23661E+01_JPRB, 0.18884E+01_JPRB, 0.25844E+01_JPRB ,&
&  0.28134E+01_JPRB, 0.28068E+01_JPRB, 0.22614E+01_JPRB, 0.15618E+01_JPRB, 0.15607E+01_JPRB, 0.11297E+01_JPRB, 0.76324E+00_JPRB ,&
&  0.71338E+00_JPRB, 0.68007E+00_JPRB, 0.61601E+00_JPRB, 0.62331E+00_JPRB, 0.60687E+00_JPRB, 0.79185E+00_JPRB, 0.11118E+01_JPRB ,&
&  0.14738E+01_JPRB, 0.18424E+01_JPRB, 0.22833E+01_JPRB, 0.25679E+01_JPRB, 0.24740E+01_JPRB, 0.27440E+01_JPRB, 0.25523E+01_JPRB ,&
&  0.20382E+01_JPRB, 0.18141E+01_JPRB, 0.16562E+01_JPRB, 0.14562E+01_JPRB, 0.13071E+01_JPRB, 0.12097E+01_JPRB, 0.11360E+01_JPRB ,&
&  0.10626E+01_JPRB, 0.99906E+00_JPRB, 0.95744E+00_JPRB, 0.93417E+00_JPRB, 0.91384E+00_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.95330E+00_JPRB, 0.91544E+00_JPRB, 0.88274E+00_JPRB, 0.85575E+00_JPRB, 0.82116E+00_JPRB, 0.78993E+00_JPRB, 0.77062E+00_JPRB ,&
&  0.73516E+00_JPRB, 0.69396E+00_JPRB, 0.66387E+00_JPRB, 0.64292E+00_JPRB, 0.63612E+00_JPRB, 0.63447E+00_JPRB, 0.62288E+00_JPRB ,&
&  0.60506E+00_JPRB, 0.59012E+00_JPRB, 0.58156E+00_JPRB, 0.57107E+00_JPRB, 0.57533E+00_JPRB, 0.58667E+00_JPRB, 0.64213E+00_JPRB ,&
&  0.79056E+00_JPRB, 0.99088E+00_JPRB, 0.10761E+01_JPRB, 0.11461E+01_JPRB, 0.12078E+01_JPRB, 0.12117E+01_JPRB, 0.12101E+01_JPRB ,&
&  0.11884E+01_JPRB, 0.11380E+01_JPRB, 0.10425E+01_JPRB, 0.94685E+00_JPRB, 0.88123E+00_JPRB, 0.84778E+00_JPRB, 0.83443E+00_JPRB ,&
&  0.83258E+00_JPRB, 0.85811E+00_JPRB, 0.85375E+00_JPRB, 0.83100E+00_JPRB, 0.82300E+00_JPRB, 0.87692E+00_JPRB, 0.93479E+00_JPRB ,&
&  0.94995E+00_JPRB, 0.98002E+00_JPRB, 0.10548E+01_JPRB, 0.10928E+01_JPRB, 0.10526E+01_JPRB, 0.88758E+00_JPRB, 0.97443E+00_JPRB ,&
&  0.10904E+01_JPRB, 0.13471E+01_JPRB, 0.14082E+01_JPRB, 0.17663E+01_JPRB, 0.17643E+01_JPRB, 0.16063E+01_JPRB, 0.16038E+01_JPRB ,&
&  0.17514E+01_JPRB, 0.23951E+01_JPRB, 0.21254E+01_JPRB, 0.21411E+01_JPRB, 0.26443E+01_JPRB, 0.21603E+01_JPRB, 0.23382E+01_JPRB ,&
&  0.27625E+01_JPRB, 0.25769E+01_JPRB, 0.21344E+01_JPRB, 0.15820E+01_JPRB, 0.19281E+01_JPRB, 0.17890E+01_JPRB, 0.90492E+00_JPRB ,&
&  0.74149E+00_JPRB, 0.68695E+00_JPRB, 0.64535E+00_JPRB, 0.61895E+00_JPRB, 0.61273E+00_JPRB, 0.86990E+00_JPRB, 0.14506E+01_JPRB ,&
&  0.16700E+01_JPRB, 0.23007E+01_JPRB, 0.26809E+01_JPRB, 0.28086E+01_JPRB, 0.24971E+01_JPRB, 0.23031E+01_JPRB, 0.21301E+01_JPRB ,&
&  0.19078E+01_JPRB, 0.18495E+01_JPRB, 0.16663E+01_JPRB, 0.15268E+01_JPRB, 0.14064E+01_JPRB, 0.13089E+01_JPRB, 0.12223E+01_JPRB ,&
&  0.11398E+01_JPRB, 0.10831E+01_JPRB, 0.10399E+01_JPRB, 0.10217E+01_JPRB, 0.98974E+00_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.10036E+01_JPRB, 0.97429E+00_JPRB, 0.93299E+00_JPRB, 0.90162E+00_JPRB, 0.86657E+00_JPRB, 0.82897E+00_JPRB, 0.81473E+00_JPRB ,&
&  0.78240E+00_JPRB, 0.73689E+00_JPRB, 0.69760E+00_JPRB, 0.67986E+00_JPRB, 0.66640E+00_JPRB, 0.64963E+00_JPRB, 0.63512E+00_JPRB ,&
&  0.61815E+00_JPRB, 0.60879E+00_JPRB, 0.61819E+00_JPRB, 0.69524E+00_JPRB, 0.74437E+00_JPRB, 0.78496E+00_JPRB, 0.85614E+00_JPRB ,&
&  0.11235E+01_JPRB, 0.12519E+01_JPRB, 0.12573E+01_JPRB, 0.12797E+01_JPRB, 0.12594E+01_JPRB, 0.12337E+01_JPRB, 0.12069E+01_JPRB ,&
&  0.11632E+01_JPRB, 0.11227E+01_JPRB, 0.10499E+01_JPRB, 0.97661E+00_JPRB, 0.91761E+00_JPRB, 0.89336E+00_JPRB, 0.88650E+00_JPRB ,&
&  0.90584E+00_JPRB, 0.89507E+00_JPRB, 0.87143E+00_JPRB, 0.83553E+00_JPRB, 0.83768E+00_JPRB, 0.89970E+00_JPRB, 0.96045E+00_JPRB ,&
&  0.98954E+00_JPRB, 0.10686E+01_JPRB, 0.11219E+01_JPRB, 0.10995E+01_JPRB, 0.90874E+00_JPRB, 0.10156E+01_JPRB, 0.10575E+01_JPRB ,&
&  0.11135E+01_JPRB, 0.11827E+01_JPRB, 0.11807E+01_JPRB, 0.14662E+01_JPRB, 0.16830E+01_JPRB, 0.15856E+01_JPRB, 0.15411E+01_JPRB ,&
&  0.20047E+01_JPRB, 0.20614E+01_JPRB, 0.19203E+01_JPRB, 0.20354E+01_JPRB, 0.22927E+01_JPRB, 0.29184E+01_JPRB, 0.19607E+01_JPRB ,&
&  0.17916E+01_JPRB, 0.21527E+01_JPRB, 0.18697E+01_JPRB, 0.14504E+01_JPRB, 0.17191E+01_JPRB, 0.20459E+01_JPRB, 0.16624E+01_JPRB ,&
&  0.85518E+00_JPRB, 0.69634E+00_JPRB, 0.73963E+00_JPRB, 0.81130E+00_JPRB, 0.74892E+00_JPRB, 0.10729E+01_JPRB, 0.24524E+01_JPRB ,&
&  0.23376E+01_JPRB, 0.28400E+01_JPRB, 0.29087E+01_JPRB, 0.27396E+01_JPRB, 0.23771E+01_JPRB, 0.20911E+01_JPRB, 0.20552E+01_JPRB ,&
&  0.18647E+01_JPRB, 0.17667E+01_JPRB, 0.16865E+01_JPRB, 0.16042E+01_JPRB, 0.14992E+01_JPRB, 0.13750E+01_JPRB, 0.12689E+01_JPRB ,&
&  0.11896E+01_JPRB, 0.11375E+01_JPRB, 0.10920E+01_JPRB, 0.10602E+01_JPRB, 0.10268E+01_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.10604E+01_JPRB, 0.10131E+01_JPRB, 0.96619E+00_JPRB, 0.93607E+00_JPRB, 0.89540E+00_JPRB, 0.85882E+00_JPRB, 0.86905E+00_JPRB ,&
&  0.84696E+00_JPRB, 0.79171E+00_JPRB, 0.73265E+00_JPRB, 0.70743E+00_JPRB, 0.68744E+00_JPRB, 0.66567E+00_JPRB, 0.65229E+00_JPRB ,&
&  0.64025E+00_JPRB, 0.63849E+00_JPRB, 0.64474E+00_JPRB, 0.71830E+00_JPRB, 0.79467E+00_JPRB, 0.78090E+00_JPRB, 0.90933E+00_JPRB ,&
&  0.12923E+01_JPRB, 0.13950E+01_JPRB, 0.13580E+01_JPRB, 0.13517E+01_JPRB, 0.13192E+01_JPRB, 0.12867E+01_JPRB, 0.12351E+01_JPRB ,&
&  0.11337E+01_JPRB, 0.10812E+01_JPRB, 0.10030E+01_JPRB, 0.97449E+00_JPRB, 0.95357E+00_JPRB, 0.93476E+00_JPRB, 0.93295E+00_JPRB ,&
&  0.90415E+00_JPRB, 0.87993E+00_JPRB, 0.85064E+00_JPRB, 0.82005E+00_JPRB, 0.85626E+00_JPRB, 0.93757E+00_JPRB, 0.10113E+01_JPRB ,&
&  0.10719E+01_JPRB, 0.11503E+01_JPRB, 0.11575E+01_JPRB, 0.10574E+01_JPRB, 0.10245E+01_JPRB, 0.10230E+01_JPRB, 0.10668E+01_JPRB ,&
&  0.10620E+01_JPRB, 0.11336E+01_JPRB, 0.12375E+01_JPRB, 0.13678E+01_JPRB, 0.15214E+01_JPRB, 0.15386E+01_JPRB, 0.15535E+01_JPRB ,&
&  0.18242E+01_JPRB, 0.18449E+01_JPRB, 0.17211E+01_JPRB, 0.19214E+01_JPRB, 0.21459E+01_JPRB, 0.26815E+01_JPRB, 0.26591E+01_JPRB ,&
&  0.21657E+01_JPRB, 0.16881E+01_JPRB, 0.15181E+01_JPRB, 0.15019E+01_JPRB, 0.16410E+01_JPRB, 0.19085E+01_JPRB, 0.25729E+01_JPRB ,&
&  0.25944E+01_JPRB, 0.17823E+01_JPRB, 0.15073E+01_JPRB, 0.19577E+01_JPRB, 0.17034E+01_JPRB, 0.16130E+01_JPRB, 0.21133E+01_JPRB ,&
&  0.26131E+01_JPRB, 0.29738E+01_JPRB, 0.29080E+01_JPRB, 0.26082E+01_JPRB, 0.23825E+01_JPRB, 0.21042E+01_JPRB, 0.20302E+01_JPRB ,&
&  0.18966E+01_JPRB, 0.17778E+01_JPRB, 0.17374E+01_JPRB, 0.16731E+01_JPRB, 0.15473E+01_JPRB, 0.13985E+01_JPRB, 0.12724E+01_JPRB ,&
&  0.12008E+01_JPRB, 0.11587E+01_JPRB, 0.11280E+01_JPRB, 0.11076E+01_JPRB, 0.10875E+01_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.10869E+01_JPRB, 0.10283E+01_JPRB, 0.98782E+00_JPRB, 0.95320E+00_JPRB, 0.90864E+00_JPRB, 0.89138E+00_JPRB, 0.93797E+00_JPRB ,&
&  0.93411E+00_JPRB, 0.83560E+00_JPRB, 0.77957E+00_JPRB, 0.73403E+00_JPRB, 0.70225E+00_JPRB, 0.68311E+00_JPRB, 0.67380E+00_JPRB ,&
&  0.66510E+00_JPRB, 0.66802E+00_JPRB, 0.67335E+00_JPRB, 0.75420E+00_JPRB, 0.90542E+00_JPRB, 0.11470E+01_JPRB, 0.93975E+00_JPRB ,&
&  0.13482E+01_JPRB, 0.16028E+01_JPRB, 0.15274E+01_JPRB, 0.14200E+01_JPRB, 0.13130E+01_JPRB, 0.13217E+01_JPRB, 0.12278E+01_JPRB ,&
&  0.10922E+01_JPRB, 0.10243E+01_JPRB, 0.96555E+00_JPRB, 0.93606E+00_JPRB, 0.93176E+00_JPRB, 0.91707E+00_JPRB, 0.88375E+00_JPRB ,&
&  0.86116E+00_JPRB, 0.84347E+00_JPRB, 0.82890E+00_JPRB, 0.84091E+00_JPRB, 0.89534E+00_JPRB, 0.97835E+00_JPRB, 0.10734E+01_JPRB ,&
&  0.11789E+01_JPRB, 0.12219E+01_JPRB, 0.11936E+01_JPRB, 0.10785E+01_JPRB, 0.10657E+01_JPRB, 0.10549E+01_JPRB, 0.10593E+01_JPRB ,&
&  0.10213E+01_JPRB, 0.99241E+00_JPRB, 0.11626E+01_JPRB, 0.13504E+01_JPRB, 0.14490E+01_JPRB, 0.14082E+01_JPRB, 0.14142E+01_JPRB ,&
&  0.17250E+01_JPRB, 0.19959E+01_JPRB, 0.22033E+01_JPRB, 0.21376E+01_JPRB, 0.23814E+01_JPRB, 0.26818E+01_JPRB, 0.29127E+01_JPRB ,&
&  0.27790E+01_JPRB, 0.20921E+01_JPRB, 0.17618E+01_JPRB, 0.16138E+01_JPRB, 0.15846E+01_JPRB, 0.18323E+01_JPRB, 0.23758E+01_JPRB ,&
&  0.30040E+01_JPRB, 0.39027E+01_JPRB, 0.39218E+01_JPRB, 0.31288E+01_JPRB, 0.27431E+01_JPRB, 0.24573E+01_JPRB, 0.22883E+01_JPRB ,&
&  0.28940E+01_JPRB, 0.29881E+01_JPRB, 0.28559E+01_JPRB, 0.31490E+01_JPRB, 0.25198E+01_JPRB, 0.21744E+01_JPRB, 0.20136E+01_JPRB ,&
&  0.19322E+01_JPRB, 0.18320E+01_JPRB, 0.17373E+01_JPRB, 0.16353E+01_JPRB, 0.14766E+01_JPRB, 0.13352E+01_JPRB, 0.12486E+01_JPRB ,&
&  0.12002E+01_JPRB, 0.11741E+01_JPRB, 0.11518E+01_JPRB, 0.11406E+01_JPRB, 0.11266E+01_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.10759E+01_JPRB, 0.10227E+01_JPRB, 0.96557E+00_JPRB, 0.94344E+00_JPRB, 0.92278E+00_JPRB, 0.92722E+00_JPRB, 0.95676E+00_JPRB ,&
&  0.10402E+01_JPRB, 0.88621E+00_JPRB, 0.79900E+00_JPRB, 0.73708E+00_JPRB, 0.70921E+00_JPRB, 0.70268E+00_JPRB, 0.69702E+00_JPRB ,&
&  0.69876E+00_JPRB, 0.71076E+00_JPRB, 0.74676E+00_JPRB, 0.80520E+00_JPRB, 0.96928E+00_JPRB, 0.12063E+01_JPRB, 0.12647E+01_JPRB ,&
&  0.14990E+01_JPRB, 0.20600E+01_JPRB, 0.16804E+01_JPRB, 0.14548E+01_JPRB, 0.13647E+01_JPRB, 0.14827E+01_JPRB, 0.12986E+01_JPRB ,&
&  0.10990E+01_JPRB, 0.10488E+01_JPRB, 0.10257E+01_JPRB, 0.94339E+00_JPRB, 0.91180E+00_JPRB, 0.87795E+00_JPRB, 0.86875E+00_JPRB ,&
&  0.85841E+00_JPRB, 0.84903E+00_JPRB, 0.84084E+00_JPRB, 0.87728E+00_JPRB, 0.94934E+00_JPRB, 0.10426E+01_JPRB, 0.11797E+01_JPRB ,&
&  0.13039E+01_JPRB, 0.12687E+01_JPRB, 0.11680E+01_JPRB, 0.11145E+01_JPRB, 0.10944E+01_JPRB, 0.11002E+01_JPRB, 0.11069E+01_JPRB ,&
&  0.10815E+01_JPRB, 0.10465E+01_JPRB, 0.11478E+01_JPRB, 0.12026E+01_JPRB, 0.12322E+01_JPRB, 0.13396E+01_JPRB, 0.13745E+01_JPRB ,&
&  0.17271E+01_JPRB, 0.18650E+01_JPRB, 0.24068E+01_JPRB, 0.22175E+01_JPRB, 0.24947E+01_JPRB, 0.27315E+01_JPRB, 0.25723E+01_JPRB ,&
&  0.21357E+01_JPRB, 0.18993E+01_JPRB, 0.16291E+01_JPRB, 0.15595E+01_JPRB, 0.17030E+01_JPRB, 0.20916E+01_JPRB, 0.25134E+01_JPRB ,&
&  0.29936E+01_JPRB, 0.34944E+01_JPRB, 0.39047E+01_JPRB, 0.31651E+01_JPRB, 0.28518E+01_JPRB, 0.26569E+01_JPRB, 0.26637E+01_JPRB ,&
&  0.30505E+01_JPRB, 0.31382E+01_JPRB, 0.32106E+01_JPRB, 0.31213E+01_JPRB, 0.23773E+01_JPRB, 0.20612E+01_JPRB, 0.19111E+01_JPRB ,&
&  0.18206E+01_JPRB, 0.17247E+01_JPRB, 0.15713E+01_JPRB, 0.14441E+01_JPRB, 0.13491E+01_JPRB, 0.12816E+01_JPRB, 0.12166E+01_JPRB ,&
&  0.11783E+01_JPRB, 0.11449E+01_JPRB, 0.11287E+01_JPRB, 0.11280E+01_JPRB, 0.11125E+01_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.10293E+01_JPRB, 0.99156E+00_JPRB, 0.97111E+00_JPRB, 0.97014E+00_JPRB, 0.96485E+00_JPRB, 0.10020E+01_JPRB, 0.10518E+01_JPRB ,&
&  0.11817E+01_JPRB, 0.87586E+00_JPRB, 0.77760E+00_JPRB, 0.74184E+00_JPRB, 0.73006E+00_JPRB, 0.72306E+00_JPRB, 0.72735E+00_JPRB ,&
&  0.73755E+00_JPRB, 0.77309E+00_JPRB, 0.83450E+00_JPRB, 0.92841E+00_JPRB, 0.11410E+01_JPRB, 0.14461E+01_JPRB, 0.16908E+01_JPRB ,&
&  0.18688E+01_JPRB, 0.18538E+01_JPRB, 0.17014E+01_JPRB, 0.14544E+01_JPRB, 0.13601E+01_JPRB, 0.14343E+01_JPRB, 0.13976E+01_JPRB ,&
&  0.12411E+01_JPRB, 0.11115E+01_JPRB, 0.10379E+01_JPRB, 0.94918E+00_JPRB, 0.90227E+00_JPRB, 0.88775E+00_JPRB, 0.87522E+00_JPRB ,&
&  0.85296E+00_JPRB, 0.83947E+00_JPRB, 0.84972E+00_JPRB, 0.91832E+00_JPRB, 0.10096E+01_JPRB, 0.11458E+01_JPRB, 0.13495E+01_JPRB ,&
&  0.13877E+01_JPRB, 0.12958E+01_JPRB, 0.12127E+01_JPRB, 0.11268E+01_JPRB, 0.11228E+01_JPRB, 0.11302E+01_JPRB, 0.11336E+01_JPRB ,&
&  0.11261E+01_JPRB, 0.11601E+01_JPRB, 0.11974E+01_JPRB, 0.12074E+01_JPRB, 0.11856E+01_JPRB, 0.13626E+01_JPRB, 0.14657E+01_JPRB ,&
&  0.17286E+01_JPRB, 0.20099E+01_JPRB, 0.26072E+01_JPRB, 0.21268E+01_JPRB, 0.25560E+01_JPRB, 0.26197E+01_JPRB, 0.23643E+01_JPRB ,&
&  0.19780E+01_JPRB, 0.17591E+01_JPRB, 0.16137E+01_JPRB, 0.16362E+01_JPRB, 0.19618E+01_JPRB, 0.23706E+01_JPRB, 0.32017E+01_JPRB ,&
&  0.32132E+01_JPRB, 0.35981E+01_JPRB, 0.33958E+01_JPRB, 0.33139E+01_JPRB, 0.29174E+01_JPRB, 0.27640E+01_JPRB, 0.27228E+01_JPRB ,&
&  0.29565E+01_JPRB, 0.29515E+01_JPRB, 0.27538E+01_JPRB, 0.25154E+01_JPRB, 0.20352E+01_JPRB, 0.17823E+01_JPRB, 0.16638E+01_JPRB ,&
&  0.15456E+01_JPRB, 0.14239E+01_JPRB, 0.13338E+01_JPRB, 0.12982E+01_JPRB, 0.12535E+01_JPRB, 0.12057E+01_JPRB, 0.11644E+01_JPRB ,&
&  0.11412E+01_JPRB, 0.11205E+01_JPRB, 0.11072E+01_JPRB, 0.10954E+01_JPRB, 0.10727E+01_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.97219E+00_JPRB, 0.94845E+00_JPRB, 0.94078E+00_JPRB, 0.92179E+00_JPRB, 0.90662E+00_JPRB, 0.90609E+00_JPRB, 0.89386E+00_JPRB ,&
&  0.83712E+00_JPRB, 0.78770E+00_JPRB, 0.77012E+00_JPRB, 0.76672E+00_JPRB, 0.75756E+00_JPRB, 0.74587E+00_JPRB, 0.74980E+00_JPRB ,&
&  0.77685E+00_JPRB, 0.83520E+00_JPRB, 0.92630E+00_JPRB, 0.10940E+01_JPRB, 0.13161E+01_JPRB, 0.15948E+01_JPRB, 0.18587E+01_JPRB ,&
&  0.18640E+01_JPRB, 0.13541E+01_JPRB, 0.15204E+01_JPRB, 0.15068E+01_JPRB, 0.13502E+01_JPRB, 0.13614E+01_JPRB, 0.13455E+01_JPRB ,&
&  0.11607E+01_JPRB, 0.10953E+01_JPRB, 0.10672E+01_JPRB, 0.10146E+01_JPRB, 0.91632E+00_JPRB, 0.89495E+00_JPRB, 0.87010E+00_JPRB ,&
&  0.85666E+00_JPRB, 0.85846E+00_JPRB, 0.89637E+00_JPRB, 0.10093E+01_JPRB, 0.11543E+01_JPRB, 0.13697E+01_JPRB, 0.14876E+01_JPRB ,&
&  0.14279E+01_JPRB, 0.13722E+01_JPRB, 0.13390E+01_JPRB, 0.12435E+01_JPRB, 0.12250E+01_JPRB, 0.12375E+01_JPRB, 0.12409E+01_JPRB ,&
&  0.12384E+01_JPRB, 0.12364E+01_JPRB, 0.12909E+01_JPRB, 0.13922E+01_JPRB, 0.14877E+01_JPRB, 0.14439E+01_JPRB, 0.16478E+01_JPRB ,&
&  0.18636E+01_JPRB, 0.22615E+01_JPRB, 0.25247E+01_JPRB, 0.29128E+01_JPRB, 0.21171E+01_JPRB, 0.23172E+01_JPRB, 0.19245E+01_JPRB ,&
&  0.19029E+01_JPRB, 0.17570E+01_JPRB, 0.18323E+01_JPRB, 0.20169E+01_JPRB, 0.24900E+01_JPRB, 0.29030E+01_JPRB, 0.36470E+01_JPRB ,&
&  0.32827E+01_JPRB, 0.31568E+01_JPRB, 0.30192E+01_JPRB, 0.31889E+01_JPRB, 0.29685E+01_JPRB, 0.29782E+01_JPRB, 0.27211E+01_JPRB ,&
&  0.27035E+01_JPRB, 0.24602E+01_JPRB, 0.22021E+01_JPRB, 0.20440E+01_JPRB, 0.17240E+01_JPRB, 0.14361E+01_JPRB, 0.12980E+01_JPRB ,&
&  0.12158E+01_JPRB, 0.11997E+01_JPRB, 0.11903E+01_JPRB, 0.11818E+01_JPRB, 0.11607E+01_JPRB, 0.11365E+01_JPRB, 0.11165E+01_JPRB ,&
&  0.11004E+01_JPRB, 0.10807E+01_JPRB, 0.10649E+01_JPRB, 0.10464E+01_JPRB, 0.10146E+01_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.92270E+00_JPRB, 0.90061E+00_JPRB, 0.89225E+00_JPRB, 0.87850E+00_JPRB, 0.86827E+00_JPRB, 0.85858E+00_JPRB, 0.84463E+00_JPRB ,&
&  0.81860E+00_JPRB, 0.80485E+00_JPRB, 0.80023E+00_JPRB, 0.79211E+00_JPRB, 0.77002E+00_JPRB, 0.76203E+00_JPRB, 0.78613E+00_JPRB ,&
&  0.84961E+00_JPRB, 0.95567E+00_JPRB, 0.10911E+01_JPRB, 0.12451E+01_JPRB, 0.14560E+01_JPRB, 0.16195E+01_JPRB, 0.16769E+01_JPRB ,&
&  0.17962E+01_JPRB, 0.18364E+01_JPRB, 0.16199E+01_JPRB, 0.11953E+01_JPRB, 0.10836E+01_JPRB, 0.12682E+01_JPRB, 0.11816E+01_JPRB ,&
&  0.10884E+01_JPRB, 0.10471E+01_JPRB, 0.10162E+01_JPRB, 0.98296E+00_JPRB, 0.94018E+00_JPRB, 0.91750E+00_JPRB, 0.90509E+00_JPRB ,&
&  0.90990E+00_JPRB, 0.94818E+00_JPRB, 0.10551E+01_JPRB, 0.12560E+01_JPRB, 0.14530E+01_JPRB, 0.15921E+01_JPRB, 0.15917E+01_JPRB ,&
&  0.15778E+01_JPRB, 0.16009E+01_JPRB, 0.15926E+01_JPRB, 0.14983E+01_JPRB, 0.14283E+01_JPRB, 0.14440E+01_JPRB, 0.14615E+01_JPRB ,&
&  0.14737E+01_JPRB, 0.14824E+01_JPRB, 0.14979E+01_JPRB, 0.15901E+01_JPRB, 0.16647E+01_JPRB, 0.15845E+01_JPRB, 0.16989E+01_JPRB ,&
&  0.20611E+01_JPRB, 0.25854E+01_JPRB, 0.25820E+01_JPRB, 0.30846E+01_JPRB, 0.18115E+01_JPRB, 0.20291E+01_JPRB, 0.20605E+01_JPRB ,&
&  0.18282E+01_JPRB, 0.19520E+01_JPRB, 0.21314E+01_JPRB, 0.24245E+01_JPRB, 0.28883E+01_JPRB, 0.31085E+01_JPRB, 0.34037E+01_JPRB ,&
&  0.29634E+01_JPRB, 0.26853E+01_JPRB, 0.26537E+01_JPRB, 0.27714E+01_JPRB, 0.27871E+01_JPRB, 0.27198E+01_JPRB, 0.23180E+01_JPRB ,&
&  0.19844E+01_JPRB, 0.20364E+01_JPRB, 0.18347E+01_JPRB, 0.17855E+01_JPRB, 0.14525E+01_JPRB, 0.11640E+01_JPRB, 0.11483E+01_JPRB ,&
&  0.11321E+01_JPRB, 0.11109E+01_JPRB, 0.11071E+01_JPRB, 0.10913E+01_JPRB, 0.10769E+01_JPRB, 0.10645E+01_JPRB, 0.10497E+01_JPRB ,&
&  0.10383E+01_JPRB, 0.10274E+01_JPRB, 0.10116E+01_JPRB, 0.98822E+00_JPRB, 0.95682E+00_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.88362E+00_JPRB, 0.86762E+00_JPRB, 0.86645E+00_JPRB, 0.84553E+00_JPRB, 0.82387E+00_JPRB, 0.81961E+00_JPRB, 0.82308E+00_JPRB ,&
&  0.81461E+00_JPRB, 0.80769E+00_JPRB, 0.79567E+00_JPRB, 0.78572E+00_JPRB, 0.78839E+00_JPRB, 0.81917E+00_JPRB, 0.87880E+00_JPRB ,&
&  0.96056E+00_JPRB, 0.10819E+01_JPRB, 0.12376E+01_JPRB, 0.13851E+01_JPRB, 0.15640E+01_JPRB, 0.16455E+01_JPRB, 0.16235E+01_JPRB ,&
&  0.16786E+01_JPRB, 0.17490E+01_JPRB, 0.17337E+01_JPRB, 0.17100E+01_JPRB, 0.14016E+01_JPRB, 0.14704E+01_JPRB, 0.15143E+01_JPRB ,&
&  0.14420E+01_JPRB, 0.12067E+01_JPRB, 0.10635E+01_JPRB, 0.10298E+01_JPRB, 0.99402E+00_JPRB, 0.99309E+00_JPRB, 0.10243E+01_JPRB ,&
&  0.10907E+01_JPRB, 0.12070E+01_JPRB, 0.13888E+01_JPRB, 0.15886E+01_JPRB, 0.17208E+01_JPRB, 0.17815E+01_JPRB, 0.18044E+01_JPRB ,&
&  0.18410E+01_JPRB, 0.18882E+01_JPRB, 0.19367E+01_JPRB, 0.18757E+01_JPRB, 0.18550E+01_JPRB, 0.18527E+01_JPRB, 0.19120E+01_JPRB ,&
&  0.19724E+01_JPRB, 0.18954E+01_JPRB, 0.19049E+01_JPRB, 0.20313E+01_JPRB, 0.20545E+01_JPRB, 0.20350E+01_JPRB, 0.21184E+01_JPRB ,&
&  0.24706E+01_JPRB, 0.30547E+01_JPRB, 0.24154E+01_JPRB, 0.25086E+01_JPRB, 0.28367E+01_JPRB, 0.22946E+01_JPRB, 0.19361E+01_JPRB ,&
&  0.19457E+01_JPRB, 0.20925E+01_JPRB, 0.22671E+01_JPRB, 0.25396E+01_JPRB, 0.29651E+01_JPRB, 0.34703E+01_JPRB, 0.29211E+01_JPRB ,&
&  0.25512E+01_JPRB, 0.22885E+01_JPRB, 0.22910E+01_JPRB, 0.23176E+01_JPRB, 0.22934E+01_JPRB, 0.22184E+01_JPRB, 0.19290E+01_JPRB ,&
&  0.17271E+01_JPRB, 0.17756E+01_JPRB, 0.16919E+01_JPRB, 0.16322E+01_JPRB, 0.16337E+01_JPRB, 0.11527E+01_JPRB, 0.11134E+01_JPRB ,&
&  0.10509E+01_JPRB, 0.10449E+01_JPRB, 0.10435E+01_JPRB, 0.10336E+01_JPRB, 0.10205E+01_JPRB, 0.10032E+01_JPRB, 0.99112E+00_JPRB ,&
&  0.98095E+00_JPRB, 0.96532E+00_JPRB, 0.95155E+00_JPRB, 0.93797E+00_JPRB, 0.90783E+00_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.85707E+00_JPRB, 0.84914E+00_JPRB, 0.84530E+00_JPRB, 0.82988E+00_JPRB, 0.80933E+00_JPRB, 0.80732E+00_JPRB, 0.80122E+00_JPRB ,&
&  0.78237E+00_JPRB, 0.76328E+00_JPRB, 0.74112E+00_JPRB, 0.75053E+00_JPRB, 0.77515E+00_JPRB, 0.82616E+00_JPRB, 0.88916E+00_JPRB ,&
&  0.97089E+00_JPRB, 0.10922E+01_JPRB, 0.12574E+01_JPRB, 0.13882E+01_JPRB, 0.15683E+01_JPRB, 0.16521E+01_JPRB, 0.16963E+01_JPRB ,&
&  0.17658E+01_JPRB, 0.18467E+01_JPRB, 0.19088E+01_JPRB, 0.19372E+01_JPRB, 0.19013E+01_JPRB, 0.18978E+01_JPRB, 0.19513E+01_JPRB ,&
&  0.16258E+01_JPRB, 0.12447E+01_JPRB, 0.11524E+01_JPRB, 0.11178E+01_JPRB, 0.11263E+01_JPRB, 0.11605E+01_JPRB, 0.12440E+01_JPRB ,&
&  0.13732E+01_JPRB, 0.15203E+01_JPRB, 0.17056E+01_JPRB, 0.18494E+01_JPRB, 0.19662E+01_JPRB, 0.20537E+01_JPRB, 0.21167E+01_JPRB ,&
&  0.22184E+01_JPRB, 0.23284E+01_JPRB, 0.24890E+01_JPRB, 0.23937E+01_JPRB, 0.26222E+01_JPRB, 0.26170E+01_JPRB, 0.26378E+01_JPRB ,&
&  0.25697E+01_JPRB, 0.23983E+01_JPRB, 0.24781E+01_JPRB, 0.26042E+01_JPRB, 0.27314E+01_JPRB, 0.27536E+01_JPRB, 0.27207E+01_JPRB ,&
&  0.30576E+01_JPRB, 0.31514E+01_JPRB, 0.23332E+01_JPRB, 0.20234E+01_JPRB, 0.20466E+01_JPRB, 0.16906E+01_JPRB, 0.19002E+01_JPRB ,&
&  0.19153E+01_JPRB, 0.20381E+01_JPRB, 0.21762E+01_JPRB, 0.24222E+01_JPRB, 0.26970E+01_JPRB, 0.28762E+01_JPRB, 0.25208E+01_JPRB ,&
&  0.20661E+01_JPRB, 0.18737E+01_JPRB, 0.18417E+01_JPRB, 0.18074E+01_JPRB, 0.18008E+01_JPRB, 0.17814E+01_JPRB, 0.17041E+01_JPRB ,&
&  0.16152E+01_JPRB, 0.14954E+01_JPRB, 0.14181E+01_JPRB, 0.12208E+01_JPRB, 0.10972E+01_JPRB, 0.10491E+01_JPRB, 0.98947E+00_JPRB ,&
&  0.97952E+00_JPRB, 0.99692E+00_JPRB, 0.99936E+00_JPRB, 0.99127E+00_JPRB, 0.98443E+00_JPRB, 0.96362E+00_JPRB, 0.95228E+00_JPRB ,&
&  0.94621E+00_JPRB, 0.93048E+00_JPRB, 0.91451E+00_JPRB, 0.90325E+00_JPRB, 0.87670E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.81611E+00_JPRB, 0.80067E+00_JPRB, 0.79371E+00_JPRB, 0.78800E+00_JPRB, 0.76928E+00_JPRB, 0.75489E+00_JPRB, 0.74562E+00_JPRB ,&
&  0.71097E+00_JPRB, 0.70738E+00_JPRB, 0.69229E+00_JPRB, 0.69956E+00_JPRB, 0.72835E+00_JPRB, 0.75681E+00_JPRB, 0.80226E+00_JPRB ,&
&  0.84540E+00_JPRB, 0.90454E+00_JPRB, 0.10009E+01_JPRB, 0.12022E+01_JPRB, 0.14285E+01_JPRB, 0.15971E+01_JPRB, 0.17961E+01_JPRB ,&
&  0.18763E+01_JPRB, 0.19457E+01_JPRB, 0.20855E+01_JPRB, 0.22702E+01_JPRB, 0.23725E+01_JPRB, 0.24419E+01_JPRB, 0.23766E+01_JPRB ,&
&  0.17648E+01_JPRB, 0.12198E+01_JPRB, 0.12427E+01_JPRB, 0.11801E+01_JPRB, 0.12427E+01_JPRB, 0.13249E+01_JPRB, 0.14198E+01_JPRB ,&
&  0.15658E+01_JPRB, 0.17176E+01_JPRB, 0.18747E+01_JPRB, 0.20036E+01_JPRB, 0.21514E+01_JPRB, 0.23094E+01_JPRB, 0.24037E+01_JPRB ,&
&  0.24954E+01_JPRB, 0.25915E+01_JPRB, 0.26710E+01_JPRB, 0.26239E+01_JPRB, 0.27743E+01_JPRB, 0.29710E+01_JPRB, 0.31359E+01_JPRB ,&
&  0.32931E+01_JPRB, 0.33064E+01_JPRB, 0.27832E+01_JPRB, 0.28922E+01_JPRB, 0.33000E+01_JPRB, 0.30834E+01_JPRB, 0.31610E+01_JPRB ,&
&  0.36704E+01_JPRB, 0.37633E+01_JPRB, 0.21266E+01_JPRB, 0.18047E+01_JPRB, 0.18067E+01_JPRB, 0.17358E+01_JPRB, 0.17604E+01_JPRB ,&
&  0.17837E+01_JPRB, 0.19076E+01_JPRB, 0.20635E+01_JPRB, 0.21755E+01_JPRB, 0.22192E+01_JPRB, 0.20857E+01_JPRB, 0.19435E+01_JPRB ,&
&  0.16462E+01_JPRB, 0.15020E+01_JPRB, 0.15016E+01_JPRB, 0.15201E+01_JPRB, 0.15516E+01_JPRB, 0.15938E+01_JPRB, 0.14666E+01_JPRB ,&
&  0.14237E+01_JPRB, 0.13950E+01_JPRB, 0.15178E+01_JPRB, 0.13010E+01_JPRB, 0.10148E+01_JPRB, 0.95984E+00_JPRB, 0.91856E+00_JPRB ,&
&  0.95780E+00_JPRB, 0.97954E+00_JPRB, 0.98840E+00_JPRB, 0.99217E+00_JPRB, 0.98862E+00_JPRB, 0.97499E+00_JPRB, 0.95855E+00_JPRB ,&
&  0.93807E+00_JPRB, 0.91488E+00_JPRB, 0.88895E+00_JPRB, 0.86310E+00_JPRB, 0.83903E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.72872E+00_JPRB, 0.71337E+00_JPRB, 0.71419E+00_JPRB, 0.68429E+00_JPRB, 0.70123E+00_JPRB, 0.67944E+00_JPRB, 0.70987E+00_JPRB ,&
&  0.68485E+00_JPRB, 0.68011E+00_JPRB, 0.67968E+00_JPRB, 0.69219E+00_JPRB, 0.70513E+00_JPRB, 0.72356E+00_JPRB, 0.73874E+00_JPRB ,&
&  0.76237E+00_JPRB, 0.80876E+00_JPRB, 0.87464E+00_JPRB, 0.95145E+00_JPRB, 0.10514E+01_JPRB, 0.11515E+01_JPRB, 0.12398E+01_JPRB ,&
&  0.13586E+01_JPRB, 0.14617E+01_JPRB, 0.15226E+01_JPRB, 0.16799E+01_JPRB, 0.21189E+01_JPRB, 0.26570E+01_JPRB, 0.27147E+01_JPRB ,&
&  0.17458E+01_JPRB, 0.12893E+01_JPRB, 0.11787E+01_JPRB, 0.11124E+01_JPRB, 0.11977E+01_JPRB, 0.13814E+01_JPRB, 0.14754E+01_JPRB ,&
&  0.15725E+01_JPRB, 0.16904E+01_JPRB, 0.18238E+01_JPRB, 0.19692E+01_JPRB, 0.21443E+01_JPRB, 0.23172E+01_JPRB, 0.24169E+01_JPRB ,&
&  0.24422E+01_JPRB, 0.25044E+01_JPRB, 0.25293E+01_JPRB, 0.27210E+01_JPRB, 0.30643E+01_JPRB, 0.34691E+01_JPRB, 0.37762E+01_JPRB ,&
&  0.39402E+01_JPRB, 0.38725E+01_JPRB, 0.32895E+01_JPRB, 0.33454E+01_JPRB, 0.35337E+01_JPRB, 0.34628E+01_JPRB, 0.33315E+01_JPRB ,&
&  0.36657E+01_JPRB, 0.34325E+01_JPRB, 0.21757E+01_JPRB, 0.19363E+01_JPRB, 0.18050E+01_JPRB, 0.17005E+01_JPRB, 0.15884E+01_JPRB ,&
&  0.16654E+01_JPRB, 0.17786E+01_JPRB, 0.18540E+01_JPRB, 0.18473E+01_JPRB, 0.17757E+01_JPRB, 0.16582E+01_JPRB, 0.16207E+01_JPRB ,&
&  0.14722E+01_JPRB, 0.14265E+01_JPRB, 0.14255E+01_JPRB, 0.14138E+01_JPRB, 0.15804E+01_JPRB, 0.14172E+01_JPRB, 0.14080E+01_JPRB ,&
&  0.12389E+01_JPRB, 0.12185E+01_JPRB, 0.10446E+01_JPRB, 0.12427E+01_JPRB, 0.97294E+00_JPRB, 0.84342E+00_JPRB, 0.86253E+00_JPRB ,&
&  0.88146E+00_JPRB, 0.91132E+00_JPRB, 0.90693E+00_JPRB, 0.91461E+00_JPRB, 0.91968E+00_JPRB, 0.90795E+00_JPRB, 0.90007E+00_JPRB ,&
&  0.88388E+00_JPRB, 0.86356E+00_JPRB, 0.82713E+00_JPRB, 0.79488E+00_JPRB, 0.76258E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.70787E+00_JPRB, 0.71059E+00_JPRB, 0.69387E+00_JPRB, 0.66421E+00_JPRB, 0.64388E+00_JPRB, 0.63126E+00_JPRB, 0.63956E+00_JPRB ,&
&  0.64932E+00_JPRB, 0.67096E+00_JPRB, 0.68890E+00_JPRB, 0.70810E+00_JPRB, 0.71759E+00_JPRB, 0.72478E+00_JPRB, 0.74175E+00_JPRB ,&
&  0.76432E+00_JPRB, 0.78632E+00_JPRB, 0.81460E+00_JPRB, 0.85312E+00_JPRB, 0.88972E+00_JPRB, 0.93591E+00_JPRB, 0.95660E+00_JPRB ,&
&  0.95716E+00_JPRB, 0.98714E+00_JPRB, 0.10130E+01_JPRB, 0.10707E+01_JPRB, 0.13018E+01_JPRB, 0.15449E+01_JPRB, 0.18233E+01_JPRB ,&
&  0.14118E+01_JPRB, 0.14176E+01_JPRB, 0.12814E+01_JPRB, 0.11979E+01_JPRB, 0.12444E+01_JPRB, 0.12446E+01_JPRB, 0.12947E+01_JPRB ,&
&  0.13667E+01_JPRB, 0.14150E+01_JPRB, 0.14588E+01_JPRB, 0.15129E+01_JPRB, 0.15694E+01_JPRB, 0.16781E+01_JPRB, 0.16499E+01_JPRB ,&
&  0.16462E+01_JPRB, 0.17293E+01_JPRB, 0.18336E+01_JPRB, 0.21247E+01_JPRB, 0.25498E+01_JPRB, 0.29506E+01_JPRB, 0.29979E+01_JPRB ,&
&  0.30443E+01_JPRB, 0.32730E+01_JPRB, 0.32055E+01_JPRB, 0.32418E+01_JPRB, 0.31951E+01_JPRB, 0.30758E+01_JPRB, 0.30753E+01_JPRB ,&
&  0.29198E+01_JPRB, 0.25976E+01_JPRB, 0.22617E+01_JPRB, 0.19328E+01_JPRB, 0.17129E+01_JPRB, 0.15299E+01_JPRB, 0.14802E+01_JPRB ,&
&  0.14900E+01_JPRB, 0.15574E+01_JPRB, 0.15888E+01_JPRB, 0.15212E+01_JPRB, 0.14625E+01_JPRB, 0.13852E+01_JPRB, 0.13846E+01_JPRB ,&
&  0.13604E+01_JPRB, 0.12577E+01_JPRB, 0.12242E+01_JPRB, 0.12682E+01_JPRB, 0.13452E+01_JPRB, 0.20965E+01_JPRB, 0.18953E+01_JPRB ,&
&  0.11365E+01_JPRB, 0.93377E+00_JPRB, 0.70151E+00_JPRB, 0.10065E+01_JPRB, 0.78808E+00_JPRB, 0.75559E+00_JPRB, 0.77865E+00_JPRB ,&
&  0.79882E+00_JPRB, 0.81437E+00_JPRB, 0.82440E+00_JPRB, 0.81961E+00_JPRB, 0.81814E+00_JPRB, 0.81719E+00_JPRB, 0.80133E+00_JPRB ,&
&  0.78816E+00_JPRB, 0.77028E+00_JPRB, 0.76514E+00_JPRB, 0.73145E+00_JPRB, 0.71457E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.68053E+00_JPRB, 0.67995E+00_JPRB, 0.65462E+00_JPRB, 0.61792E+00_JPRB, 0.60134E+00_JPRB, 0.60856E+00_JPRB, 0.62502E+00_JPRB ,&
&  0.64103E+00_JPRB, 0.66227E+00_JPRB, 0.68693E+00_JPRB, 0.71114E+00_JPRB, 0.73290E+00_JPRB, 0.75645E+00_JPRB, 0.78448E+00_JPRB ,&
&  0.81109E+00_JPRB, 0.84368E+00_JPRB, 0.87704E+00_JPRB, 0.91105E+00_JPRB, 0.94109E+00_JPRB, 0.99306E+00_JPRB, 0.10308E+01_JPRB ,&
&  0.10887E+01_JPRB, 0.11466E+01_JPRB, 0.12184E+01_JPRB, 0.12905E+01_JPRB, 0.13821E+01_JPRB, 0.14528E+01_JPRB, 0.12901E+01_JPRB ,&
&  0.15114E+01_JPRB, 0.14417E+01_JPRB, 0.13275E+01_JPRB, 0.12413E+01_JPRB, 0.11779E+01_JPRB, 0.11328E+01_JPRB, 0.10858E+01_JPRB ,&
&  0.10670E+01_JPRB, 0.10515E+01_JPRB, 0.10189E+01_JPRB, 0.99533E+00_JPRB, 0.99290E+00_JPRB, 0.10751E+01_JPRB, 0.10916E+01_JPRB ,&
&  0.11991E+01_JPRB, 0.13816E+01_JPRB, 0.15456E+01_JPRB, 0.18139E+01_JPRB, 0.20436E+01_JPRB, 0.22101E+01_JPRB, 0.21256E+01_JPRB ,&
&  0.21607E+01_JPRB, 0.23836E+01_JPRB, 0.23652E+01_JPRB, 0.24433E+01_JPRB, 0.24223E+01_JPRB, 0.23009E+01_JPRB, 0.22210E+01_JPRB ,&
&  0.19807E+01_JPRB, 0.18828E+01_JPRB, 0.18750E+01_JPRB, 0.19918E+01_JPRB, 0.16804E+01_JPRB, 0.14696E+01_JPRB, 0.14463E+01_JPRB ,&
&  0.13708E+01_JPRB, 0.13937E+01_JPRB, 0.13965E+01_JPRB, 0.13145E+01_JPRB, 0.12488E+01_JPRB, 0.11955E+01_JPRB, 0.12012E+01_JPRB ,&
&  0.12039E+01_JPRB, 0.10911E+01_JPRB, 0.10875E+01_JPRB, 0.11923E+01_JPRB, 0.13800E+01_JPRB, 0.14785E+01_JPRB, 0.17765E+01_JPRB ,&
&  0.10734E+01_JPRB, 0.65587E+00_JPRB, 0.68854E+00_JPRB, 0.76204E+00_JPRB, 0.67831E+00_JPRB, 0.65019E+00_JPRB, 0.69656E+00_JPRB ,&
&  0.73784E+00_JPRB, 0.76261E+00_JPRB, 0.75215E+00_JPRB, 0.77587E+00_JPRB, 0.81602E+00_JPRB, 0.80670E+00_JPRB, 0.75206E+00_JPRB ,&
&  0.75952E+00_JPRB, 0.72340E+00_JPRB, 0.70499E+00_JPRB, 0.69696E+00_JPRB, 0.69088E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.64701E+00_JPRB, 0.63700E+00_JPRB, 0.61051E+00_JPRB, 0.57520E+00_JPRB, 0.56217E+00_JPRB, 0.57950E+00_JPRB, 0.60541E+00_JPRB ,&
&  0.62652E+00_JPRB, 0.65172E+00_JPRB, 0.67745E+00_JPRB, 0.70578E+00_JPRB, 0.73657E+00_JPRB, 0.76559E+00_JPRB, 0.78798E+00_JPRB ,&
&  0.80422E+00_JPRB, 0.82620E+00_JPRB, 0.84631E+00_JPRB, 0.86384E+00_JPRB, 0.87235E+00_JPRB, 0.87720E+00_JPRB, 0.88237E+00_JPRB ,&
&  0.92228E+00_JPRB, 0.96507E+00_JPRB, 0.10214E+01_JPRB, 0.10799E+01_JPRB, 0.11243E+01_JPRB, 0.11434E+01_JPRB, 0.96632E+00_JPRB ,&
&  0.13759E+01_JPRB, 0.13940E+01_JPRB, 0.13105E+01_JPRB, 0.12209E+01_JPRB, 0.11290E+01_JPRB, 0.10601E+01_JPRB, 0.99157E+00_JPRB ,&
&  0.93202E+00_JPRB, 0.97506E+00_JPRB, 0.95377E+00_JPRB, 0.96482E+00_JPRB, 0.91833E+00_JPRB, 0.98972E+00_JPRB, 0.10802E+01_JPRB ,&
&  0.12508E+01_JPRB, 0.14411E+01_JPRB, 0.16105E+01_JPRB, 0.17637E+01_JPRB, 0.18377E+01_JPRB, 0.18980E+01_JPRB, 0.18089E+01_JPRB ,&
&  0.18207E+01_JPRB, 0.18312E+01_JPRB, 0.17250E+01_JPRB, 0.18035E+01_JPRB, 0.18783E+01_JPRB, 0.16198E+01_JPRB, 0.14349E+01_JPRB ,&
&  0.13019E+01_JPRB, 0.14466E+01_JPRB, 0.18447E+01_JPRB, 0.19963E+01_JPRB, 0.16852E+01_JPRB, 0.15484E+01_JPRB, 0.14329E+01_JPRB ,&
&  0.13067E+01_JPRB, 0.12855E+01_JPRB, 0.12523E+01_JPRB, 0.11845E+01_JPRB, 0.10809E+01_JPRB, 0.10232E+01_JPRB, 0.10307E+01_JPRB ,&
&  0.10413E+01_JPRB, 0.95278E+00_JPRB, 0.91299E+00_JPRB, 0.10156E+01_JPRB, 0.10959E+01_JPRB, 0.10370E+01_JPRB, 0.16737E+01_JPRB ,&
&  0.10708E+01_JPRB, 0.77900E+00_JPRB, 0.10463E+01_JPRB, 0.50465E+00_JPRB, 0.57970E+00_JPRB, 0.55160E+00_JPRB, 0.57115E+00_JPRB ,&
&  0.49126E+00_JPRB, 0.57581E+00_JPRB, 0.93814E+00_JPRB, 0.12560E+01_JPRB, 0.14427E+01_JPRB, 0.14348E+01_JPRB, 0.83322E+00_JPRB ,&
&  0.73703E+00_JPRB, 0.71268E+00_JPRB, 0.69522E+00_JPRB, 0.67364E+00_JPRB, 0.65017E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.60970E+00_JPRB, 0.57608E+00_JPRB, 0.56250E+00_JPRB, 0.55370E+00_JPRB, 0.54437E+00_JPRB, 0.56410E+00_JPRB, 0.58052E+00_JPRB ,&
&  0.60466E+00_JPRB, 0.63370E+00_JPRB, 0.66858E+00_JPRB, 0.69937E+00_JPRB, 0.74230E+00_JPRB, 0.78429E+00_JPRB, 0.81770E+00_JPRB ,&
&  0.84504E+00_JPRB, 0.86689E+00_JPRB, 0.88518E+00_JPRB, 0.89448E+00_JPRB, 0.89971E+00_JPRB, 0.90328E+00_JPRB, 0.90474E+00_JPRB ,&
&  0.90553E+00_JPRB, 0.93182E+00_JPRB, 0.98258E+00_JPRB, 0.10412E+01_JPRB, 0.10807E+01_JPRB, 0.11367E+01_JPRB, 0.75337E+00_JPRB ,&
&  0.12008E+01_JPRB, 0.12551E+01_JPRB, 0.12442E+01_JPRB, 0.11713E+01_JPRB, 0.10925E+01_JPRB, 0.10303E+01_JPRB, 0.98392E+00_JPRB ,&
&  0.95846E+00_JPRB, 0.99338E+00_JPRB, 0.99192E+00_JPRB, 0.11170E+01_JPRB, 0.11896E+01_JPRB, 0.12021E+01_JPRB, 0.13521E+01_JPRB ,&
&  0.14767E+01_JPRB, 0.16707E+01_JPRB, 0.17993E+01_JPRB, 0.18987E+01_JPRB, 0.19770E+01_JPRB, 0.20462E+01_JPRB, 0.20784E+01_JPRB ,&
&  0.20978E+01_JPRB, 0.20532E+01_JPRB, 0.19941E+01_JPRB, 0.21150E+01_JPRB, 0.21694E+01_JPRB, 0.18621E+01_JPRB, 0.16976E+01_JPRB ,&
&  0.12468E+01_JPRB, 0.11209E+01_JPRB, 0.16233E+01_JPRB, 0.19288E+01_JPRB, 0.17320E+01_JPRB, 0.16233E+01_JPRB, 0.14394E+01_JPRB ,&
&  0.12589E+01_JPRB, 0.11657E+01_JPRB, 0.11072E+01_JPRB, 0.10404E+01_JPRB, 0.95868E+00_JPRB, 0.90354E+00_JPRB, 0.88062E+00_JPRB ,&
&  0.85731E+00_JPRB, 0.77335E+00_JPRB, 0.73838E+00_JPRB, 0.78555E+00_JPRB, 0.82526E+00_JPRB, 0.96207E+00_JPRB, 0.99729E+00_JPRB ,&
&  0.11706E+01_JPRB, 0.10551E+01_JPRB, 0.82256E+00_JPRB, 0.75401E+00_JPRB, 0.56592E+00_JPRB, 0.51285E+00_JPRB, 0.50056E+00_JPRB ,&
&  0.52003E+00_JPRB, 0.47370E+00_JPRB, 0.97034E+00_JPRB, 0.37618E+01_JPRB, 0.37125E+01_JPRB, 0.47028E+01_JPRB, 0.24640E+01_JPRB ,&
&  0.11778E+01_JPRB, 0.74032E+00_JPRB, 0.64804E+00_JPRB, 0.64042E+00_JPRB, 0.62293E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.54658E+00_JPRB, 0.52616E+00_JPRB, 0.50667E+00_JPRB, 0.51690E+00_JPRB, 0.52565E+00_JPRB, 0.53331E+00_JPRB, 0.56169E+00_JPRB ,&
&  0.61992E+00_JPRB, 0.63494E+00_JPRB, 0.66265E+00_JPRB, 0.70787E+00_JPRB, 0.77410E+00_JPRB, 0.82756E+00_JPRB, 0.87717E+00_JPRB ,&
&  0.92759E+00_JPRB, 0.96050E+00_JPRB, 0.96363E+00_JPRB, 0.95705E+00_JPRB, 0.96275E+00_JPRB, 0.97447E+00_JPRB, 0.99505E+00_JPRB ,&
&  0.10031E+01_JPRB, 0.10306E+01_JPRB, 0.10810E+01_JPRB, 0.11386E+01_JPRB, 0.11899E+01_JPRB, 0.11883E+01_JPRB, 0.76799E+00_JPRB ,&
&  0.98363E+00_JPRB, 0.10632E+01_JPRB, 0.11084E+01_JPRB, 0.10938E+01_JPRB, 0.10293E+01_JPRB, 0.96086E+00_JPRB, 0.96407E+00_JPRB ,&
&  0.10377E+01_JPRB, 0.11397E+01_JPRB, 0.12257E+01_JPRB, 0.12964E+01_JPRB, 0.15168E+01_JPRB, 0.15451E+01_JPRB, 0.18225E+01_JPRB ,&
&  0.19753E+01_JPRB, 0.20928E+01_JPRB, 0.22527E+01_JPRB, 0.23509E+01_JPRB, 0.24766E+01_JPRB, 0.26258E+01_JPRB, 0.27672E+01_JPRB ,&
&  0.28390E+01_JPRB, 0.28447E+01_JPRB, 0.29511E+01_JPRB, 0.32788E+01_JPRB, 0.30335E+01_JPRB, 0.34031E+01_JPRB, 0.27639E+01_JPRB ,&
&  0.18383E+01_JPRB, 0.15228E+01_JPRB, 0.17880E+01_JPRB, 0.18951E+01_JPRB, 0.17558E+01_JPRB, 0.16779E+01_JPRB, 0.15037E+01_JPRB ,&
&  0.12589E+01_JPRB, 0.11007E+01_JPRB, 0.10311E+01_JPRB, 0.97236E+00_JPRB, 0.90629E+00_JPRB, 0.83952E+00_JPRB, 0.78565E+00_JPRB ,&
&  0.72334E+00_JPRB, 0.67660E+00_JPRB, 0.69334E+00_JPRB, 0.70053E+00_JPRB, 0.74299E+00_JPRB, 0.92197E+00_JPRB, 0.13210E+01_JPRB ,&
&  0.20766E+01_JPRB, 0.11265E+01_JPRB, 0.87573E+00_JPRB, 0.75114E+00_JPRB, 0.62728E+00_JPRB, 0.56114E+00_JPRB, 0.52266E+00_JPRB ,&
&  0.51701E+00_JPRB, 0.59518E+00_JPRB, 0.72600E+00_JPRB, 0.14348E+01_JPRB, 0.25789E+01_JPRB, 0.26357E+01_JPRB, 0.17830E+01_JPRB ,&
&  0.10920E+01_JPRB, 0.75994E+00_JPRB, 0.60999E+00_JPRB, 0.57568E+00_JPRB, 0.55149E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.51448E+00_JPRB, 0.50119E+00_JPRB, 0.50067E+00_JPRB, 0.49977E+00_JPRB, 0.53677E+00_JPRB, 0.56146E+00_JPRB, 0.58415E+00_JPRB ,&
&  0.61906E+00_JPRB, 0.64138E+00_JPRB, 0.67484E+00_JPRB, 0.73692E+00_JPRB, 0.80468E+00_JPRB, 0.86177E+00_JPRB, 0.89202E+00_JPRB ,&
&  0.91106E+00_JPRB, 0.91236E+00_JPRB, 0.90705E+00_JPRB, 0.89876E+00_JPRB, 0.92026E+00_JPRB, 0.93391E+00_JPRB, 0.97358E+00_JPRB ,&
&  0.10289E+01_JPRB, 0.10877E+01_JPRB, 0.11586E+01_JPRB, 0.12242E+01_JPRB, 0.12941E+01_JPRB, 0.13471E+01_JPRB, 0.13560E+01_JPRB ,&
&  0.64585E+00_JPRB, 0.90423E+00_JPRB, 0.96186E+00_JPRB, 0.95975E+00_JPRB, 0.91306E+00_JPRB, 0.90467E+00_JPRB, 0.10110E+01_JPRB ,&
&  0.11613E+01_JPRB, 0.12420E+01_JPRB, 0.14227E+01_JPRB, 0.14793E+01_JPRB, 0.18268E+01_JPRB, 0.19516E+01_JPRB, 0.21978E+01_JPRB ,&
&  0.23769E+01_JPRB, 0.25126E+01_JPRB, 0.26908E+01_JPRB, 0.28831E+01_JPRB, 0.30967E+01_JPRB, 0.33327E+01_JPRB, 0.36227E+01_JPRB ,&
&  0.39019E+01_JPRB, 0.42540E+01_JPRB, 0.47125E+01_JPRB, 0.47310E+01_JPRB, 0.44021E+01_JPRB, 0.47415E+01_JPRB, 0.44786E+01_JPRB ,&
&  0.37417E+01_JPRB, 0.26291E+01_JPRB, 0.24748E+01_JPRB, 0.21584E+01_JPRB, 0.18891E+01_JPRB, 0.17645E+01_JPRB, 0.15609E+01_JPRB ,&
&  0.13014E+01_JPRB, 0.11515E+01_JPRB, 0.10408E+01_JPRB, 0.97235E+00_JPRB, 0.88765E+00_JPRB, 0.82374E+00_JPRB, 0.77565E+00_JPRB ,&
&  0.74079E+00_JPRB, 0.75245E+00_JPRB, 0.76889E+00_JPRB, 0.73834E+00_JPRB, 0.71436E+00_JPRB, 0.76598E+00_JPRB, 0.88528E+00_JPRB ,&
&  0.10347E+01_JPRB, 0.91595E+00_JPRB, 0.81351E+00_JPRB, 0.80691E+00_JPRB, 0.76734E+00_JPRB, 0.72356E+00_JPRB, 0.70034E+00_JPRB ,&
&  0.69775E+00_JPRB, 0.67611E+00_JPRB, 0.76860E+00_JPRB, 0.11685E+01_JPRB, 0.15281E+01_JPRB, 0.17625E+01_JPRB, 0.14587E+01_JPRB ,&
&  0.10435E+01_JPRB, 0.77926E+00_JPRB, 0.62508E+00_JPRB, 0.58325E+00_JPRB, 0.56020E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.56153E+00_JPRB, 0.54946E+00_JPRB, 0.54934E+00_JPRB, 0.54575E+00_JPRB, 0.57953E+00_JPRB, 0.61781E+00_JPRB, 0.62878E+00_JPRB ,&
&  0.62496E+00_JPRB, 0.64406E+00_JPRB, 0.66257E+00_JPRB, 0.69592E+00_JPRB, 0.76466E+00_JPRB, 0.80076E+00_JPRB, 0.84294E+00_JPRB ,&
&  0.88383E+00_JPRB, 0.88423E+00_JPRB, 0.85941E+00_JPRB, 0.85266E+00_JPRB, 0.88459E+00_JPRB, 0.91508E+00_JPRB, 0.96486E+00_JPRB ,&
&  0.10334E+01_JPRB, 0.11164E+01_JPRB, 0.12187E+01_JPRB, 0.13401E+01_JPRB, 0.14780E+01_JPRB, 0.15988E+01_JPRB, 0.16240E+01_JPRB ,&
&  0.70050E+00_JPRB, 0.68603E+00_JPRB, 0.84614E+00_JPRB, 0.85488E+00_JPRB, 0.82049E+00_JPRB, 0.87298E+00_JPRB, 0.98255E+00_JPRB ,&
&  0.10813E+01_JPRB, 0.10936E+01_JPRB, 0.12642E+01_JPRB, 0.17593E+01_JPRB, 0.20182E+01_JPRB, 0.22232E+01_JPRB, 0.24382E+01_JPRB ,&
&  0.26097E+01_JPRB, 0.27948E+01_JPRB, 0.29604E+01_JPRB, 0.31082E+01_JPRB, 0.32936E+01_JPRB, 0.35334E+01_JPRB, 0.38534E+01_JPRB ,&
&  0.42638E+01_JPRB, 0.48634E+01_JPRB, 0.54485E+01_JPRB, 0.50515E+01_JPRB, 0.52028E+01_JPRB, 0.55239E+01_JPRB, 0.62856E+01_JPRB ,&
&  0.55847E+01_JPRB, 0.37806E+01_JPRB, 0.28369E+01_JPRB, 0.23347E+01_JPRB, 0.18077E+01_JPRB, 0.16185E+01_JPRB, 0.14227E+01_JPRB ,&
&  0.12720E+01_JPRB, 0.11702E+01_JPRB, 0.10812E+01_JPRB, 0.99709E+00_JPRB, 0.89173E+00_JPRB, 0.84344E+00_JPRB, 0.81553E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.86612E+00_JPRB, 0.85343E+00_JPRB, 0.83451E+00_JPRB, 0.79947E+00_JPRB, 0.82302E+00_JPRB, 0.88343E+00_JPRB ,&
&  0.92698E+00_JPRB, 0.94261E+00_JPRB, 0.95349E+00_JPRB, 0.96519E+00_JPRB, 0.97648E+00_JPRB, 0.97973E+00_JPRB, 0.94879E+00_JPRB ,&
&  0.90239E+00_JPRB, 0.85788E+00_JPRB, 0.85110E+00_JPRB, 0.83826E+00_JPRB, 0.98644E+00_JPRB, 0.12817E+01_JPRB, 0.13173E+01_JPRB ,&
&  0.11242E+01_JPRB, 0.86786E+00_JPRB, 0.70574E+00_JPRB, 0.65499E+00_JPRB, 0.62898E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.67465E+00_JPRB, 0.63636E+00_JPRB, 0.62633E+00_JPRB, 0.60945E+00_JPRB, 0.62364E+00_JPRB, 0.65104E+00_JPRB, 0.67096E+00_JPRB ,&
&  0.67659E+00_JPRB, 0.67563E+00_JPRB, 0.67700E+00_JPRB, 0.69779E+00_JPRB, 0.72696E+00_JPRB, 0.75935E+00_JPRB, 0.79944E+00_JPRB ,&
&  0.82641E+00_JPRB, 0.83896E+00_JPRB, 0.81413E+00_JPRB, 0.78093E+00_JPRB, 0.79625E+00_JPRB, 0.82697E+00_JPRB, 0.87301E+00_JPRB ,&
&  0.93391E+00_JPRB, 0.10054E+01_JPRB, 0.11083E+01_JPRB, 0.12794E+01_JPRB, 0.15401E+01_JPRB, 0.18549E+01_JPRB, 0.21810E+01_JPRB ,&
&  0.17343E+01_JPRB, 0.82246E+00_JPRB, 0.85866E+00_JPRB, 0.94104E+00_JPRB, 0.86463E+00_JPRB, 0.92019E+00_JPRB, 0.96321E+00_JPRB ,&
&  0.99310E+00_JPRB, 0.10684E+01_JPRB, 0.11323E+01_JPRB, 0.14098E+01_JPRB, 0.18708E+01_JPRB, 0.21471E+01_JPRB, 0.23808E+01_JPRB ,&
&  0.25672E+01_JPRB, 0.27013E+01_JPRB, 0.27873E+01_JPRB, 0.28757E+01_JPRB, 0.30331E+01_JPRB, 0.32741E+01_JPRB, 0.35951E+01_JPRB ,&
&  0.40338E+01_JPRB, 0.45999E+01_JPRB, 0.52760E+01_JPRB, 0.48739E+01_JPRB, 0.55278E+01_JPRB, 0.55299E+01_JPRB, 0.55966E+01_JPRB ,&
&  0.50909E+01_JPRB, 0.33685E+01_JPRB, 0.24902E+01_JPRB, 0.22331E+01_JPRB, 0.16976E+01_JPRB, 0.12429E+01_JPRB, 0.12747E+01_JPRB ,&
&  0.12116E+01_JPRB, 0.11589E+01_JPRB, 0.11285E+01_JPRB, 0.10219E+01_JPRB, 0.92581E+00_JPRB, 0.88898E+00_JPRB, 0.93434E+00_JPRB ,&
&  0.98621E+00_JPRB, 0.10199E+01_JPRB, 0.98950E+00_JPRB, 0.97833E+00_JPRB, 0.94566E+00_JPRB, 0.96711E+00_JPRB, 0.10062E+01_JPRB ,&
&  0.10396E+01_JPRB, 0.10740E+01_JPRB, 0.10940E+01_JPRB, 0.11243E+01_JPRB, 0.11613E+01_JPRB, 0.11054E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.10211E+01_JPRB, 0.10094E+01_JPRB, 0.95437E+00_JPRB, 0.92480E+00_JPRB, 0.89229E+00_JPRB, 0.91898E+00_JPRB, 0.10183E+01_JPRB ,&
&  0.99788E+00_JPRB, 0.88028E+00_JPRB, 0.76327E+00_JPRB, 0.73246E+00_JPRB, 0.73659E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.77586E+00_JPRB, 0.69271E+00_JPRB, 0.64561E+00_JPRB, 0.64080E+00_JPRB, 0.62775E+00_JPRB, 0.63428E+00_JPRB, 0.64409E+00_JPRB ,&
&  0.64190E+00_JPRB, 0.66181E+00_JPRB, 0.65656E+00_JPRB, 0.67180E+00_JPRB, 0.69268E+00_JPRB, 0.70242E+00_JPRB, 0.73065E+00_JPRB ,&
&  0.75775E+00_JPRB, 0.78553E+00_JPRB, 0.80852E+00_JPRB, 0.77405E+00_JPRB, 0.76956E+00_JPRB, 0.77646E+00_JPRB, 0.80077E+00_JPRB ,&
&  0.84799E+00_JPRB, 0.91221E+00_JPRB, 0.98651E+00_JPRB, 0.10798E+01_JPRB, 0.12184E+01_JPRB, 0.14703E+01_JPRB, 0.19423E+01_JPRB ,&
&  0.31160E+01_JPRB, 0.25647E+01_JPRB, 0.84481E+00_JPRB, 0.10647E+01_JPRB, 0.10477E+01_JPRB, 0.10515E+01_JPRB, 0.92764E+00_JPRB ,&
&  0.98686E+00_JPRB, 0.10888E+01_JPRB, 0.10705E+01_JPRB, 0.11277E+01_JPRB, 0.13854E+01_JPRB, 0.17588E+01_JPRB, 0.20487E+01_JPRB ,&
&  0.22411E+01_JPRB, 0.23825E+01_JPRB, 0.24440E+01_JPRB, 0.25037E+01_JPRB, 0.25911E+01_JPRB, 0.27595E+01_JPRB, 0.30267E+01_JPRB ,&
&  0.34200E+01_JPRB, 0.39816E+01_JPRB, 0.47313E+01_JPRB, 0.47134E+01_JPRB, 0.58353E+01_JPRB, 0.55829E+01_JPRB, 0.51735E+01_JPRB ,&
&  0.42055E+01_JPRB, 0.33145E+01_JPRB, 0.27214E+01_JPRB, 0.21280E+01_JPRB, 0.14997E+01_JPRB, 0.11872E+01_JPRB, 0.11718E+01_JPRB ,&
&  0.11509E+01_JPRB, 0.10278E+01_JPRB, 0.10023E+01_JPRB, 0.98615E+00_JPRB, 0.93265E+00_JPRB, 0.99248E+00_JPRB, 0.10871E+01_JPRB ,&
&  0.11328E+01_JPRB, 0.11307E+01_JPRB, 0.11200E+01_JPRB, 0.11018E+01_JPRB, 0.10862E+01_JPRB, 0.10901E+01_JPRB, 0.11203E+01_JPRB ,&
&  0.11355E+01_JPRB, 0.11537E+01_JPRB, 0.11562E+01_JPRB, 0.11833E+01_JPRB, 0.12106E+01_JPRB, 0.11458E+01_JPRB, 0.10627E+01_JPRB ,&
&  0.10511E+01_JPRB, 0.10675E+01_JPRB, 0.10413E+01_JPRB, 0.10225E+01_JPRB, 0.96753E+00_JPRB, 0.92107E+00_JPRB, 0.90848E+00_JPRB ,&
&  0.98423E+00_JPRB, 0.91751E+00_JPRB, 0.85828E+00_JPRB, 0.78937E+00_JPRB, 0.80852E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.82172E+00_JPRB, 0.77011E+00_JPRB, 0.68585E+00_JPRB, 0.65627E+00_JPRB, 0.65045E+00_JPRB, 0.63778E+00_JPRB, 0.62034E+00_JPRB ,&
&  0.59778E+00_JPRB, 0.59005E+00_JPRB, 0.61683E+00_JPRB, 0.64915E+00_JPRB, 0.67680E+00_JPRB, 0.68287E+00_JPRB, 0.69669E+00_JPRB ,&
&  0.70517E+00_JPRB, 0.72841E+00_JPRB, 0.77204E+00_JPRB, 0.78308E+00_JPRB, 0.77201E+00_JPRB, 0.76654E+00_JPRB, 0.77126E+00_JPRB ,&
&  0.78503E+00_JPRB, 0.80758E+00_JPRB, 0.85357E+00_JPRB, 0.90684E+00_JPRB, 0.98750E+00_JPRB, 0.11277E+01_JPRB, 0.14302E+01_JPRB ,&
&  0.20837E+01_JPRB, 0.36622E+01_JPRB, 0.98332E+00_JPRB, 0.12990E+01_JPRB, 0.12583E+01_JPRB, 0.11577E+01_JPRB, 0.10770E+01_JPRB ,&
&  0.10640E+01_JPRB, 0.10760E+01_JPRB, 0.11066E+01_JPRB, 0.10775E+01_JPRB, 0.99354E+00_JPRB, 0.12033E+01_JPRB, 0.15185E+01_JPRB ,&
&  0.18110E+01_JPRB, 0.19589E+01_JPRB, 0.20185E+01_JPRB, 0.20121E+01_JPRB, 0.20442E+01_JPRB, 0.21151E+01_JPRB, 0.22700E+01_JPRB ,&
&  0.25461E+01_JPRB, 0.29912E+01_JPRB, 0.36600E+01_JPRB, 0.39747E+01_JPRB, 0.53427E+01_JPRB, 0.56788E+01_JPRB, 0.54085E+01_JPRB ,&
&  0.43131E+01_JPRB, 0.38144E+01_JPRB, 0.32989E+01_JPRB, 0.23420E+01_JPRB, 0.16380E+01_JPRB, 0.12876E+01_JPRB, 0.12092E+01_JPRB ,&
&  0.11591E+01_JPRB, 0.10421E+01_JPRB, 0.10283E+01_JPRB, 0.10033E+01_JPRB, 0.10156E+01_JPRB, 0.10805E+01_JPRB, 0.11577E+01_JPRB ,&
&  0.11752E+01_JPRB, 0.11802E+01_JPRB, 0.11605E+01_JPRB, 0.11464E+01_JPRB, 0.11459E+01_JPRB, 0.11504E+01_JPRB, 0.11833E+01_JPRB ,&
&  0.11965E+01_JPRB, 0.12049E+01_JPRB, 0.11987E+01_JPRB, 0.11660E+01_JPRB, 0.11709E+01_JPRB, 0.11308E+01_JPRB, 0.10761E+01_JPRB ,&
&  0.10815E+01_JPRB, 0.11024E+01_JPRB, 0.10914E+01_JPRB, 0.10485E+01_JPRB, 0.10680E+01_JPRB, 0.10285E+01_JPRB, 0.98623E+00_JPRB ,&
&  0.10138E+01_JPRB, 0.10207E+01_JPRB, 0.99138E+00_JPRB, 0.86277E+00_JPRB, 0.86810E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.86007E+00_JPRB, 0.81376E+00_JPRB, 0.75493E+00_JPRB, 0.69040E+00_JPRB, 0.69059E+00_JPRB, 0.69802E+00_JPRB, 0.66792E+00_JPRB ,&
&  0.61709E+00_JPRB, 0.61260E+00_JPRB, 0.62135E+00_JPRB, 0.64475E+00_JPRB, 0.66432E+00_JPRB, 0.67207E+00_JPRB, 0.70178E+00_JPRB ,&
&  0.69574E+00_JPRB, 0.69116E+00_JPRB, 0.71410E+00_JPRB, 0.74949E+00_JPRB, 0.75398E+00_JPRB, 0.75352E+00_JPRB, 0.76864E+00_JPRB ,&
&  0.77517E+00_JPRB, 0.76504E+00_JPRB, 0.76558E+00_JPRB, 0.78535E+00_JPRB, 0.83861E+00_JPRB, 0.93394E+00_JPRB, 0.11124E+01_JPRB ,&
&  0.17306E+01_JPRB, 0.40062E+01_JPRB, 0.27363E+01_JPRB, 0.22873E+01_JPRB, 0.16469E+01_JPRB, 0.13226E+01_JPRB, 0.11961E+01_JPRB ,&
&  0.12782E+01_JPRB, 0.12389E+01_JPRB, 0.12622E+01_JPRB, 0.10709E+01_JPRB, 0.93619E+00_JPRB, 0.91073E+00_JPRB, 0.10742E+01_JPRB ,&
&  0.12617E+01_JPRB, 0.14266E+01_JPRB, 0.15463E+01_JPRB, 0.15594E+01_JPRB, 0.15750E+01_JPRB, 0.16255E+01_JPRB, 0.16966E+01_JPRB ,&
&  0.18480E+01_JPRB, 0.21443E+01_JPRB, 0.26591E+01_JPRB, 0.33388E+01_JPRB, 0.41568E+01_JPRB, 0.49808E+01_JPRB, 0.57899E+01_JPRB ,&
&  0.54984E+01_JPRB, 0.47607E+01_JPRB, 0.38321E+01_JPRB, 0.25897E+01_JPRB, 0.17284E+01_JPRB, 0.15735E+01_JPRB, 0.12559E+01_JPRB ,&
&  0.11751E+01_JPRB, 0.10926E+01_JPRB, 0.10908E+01_JPRB, 0.11123E+01_JPRB, 0.11456E+01_JPRB, 0.11938E+01_JPRB, 0.12298E+01_JPRB ,&
&  0.12355E+01_JPRB, 0.12169E+01_JPRB, 0.12037E+01_JPRB, 0.11853E+01_JPRB, 0.11759E+01_JPRB, 0.11737E+01_JPRB, 0.11835E+01_JPRB ,&
&  0.11908E+01_JPRB, 0.12086E+01_JPRB, 0.11559E+01_JPRB, 0.11417E+01_JPRB, 0.11377E+01_JPRB, 0.11142E+01_JPRB, 0.10591E+01_JPRB ,&
&  0.10749E+01_JPRB, 0.11213E+01_JPRB, 0.11375E+01_JPRB, 0.11214E+01_JPRB, 0.11191E+01_JPRB, 0.11257E+01_JPRB, 0.10811E+01_JPRB ,&
&  0.10769E+01_JPRB, 0.10518E+01_JPRB, 0.97801E+00_JPRB, 0.93028E+00_JPRB, 0.91867E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.91010E+00_JPRB, 0.87294E+00_JPRB, 0.82561E+00_JPRB, 0.76842E+00_JPRB, 0.73805E+00_JPRB, 0.74103E+00_JPRB, 0.75363E+00_JPRB ,&
&  0.72598E+00_JPRB, 0.70310E+00_JPRB, 0.69451E+00_JPRB, 0.70930E+00_JPRB, 0.73412E+00_JPRB, 0.74425E+00_JPRB, 0.74940E+00_JPRB ,&
&  0.73827E+00_JPRB, 0.73176E+00_JPRB, 0.71725E+00_JPRB, 0.72176E+00_JPRB, 0.77027E+00_JPRB, 0.76998E+00_JPRB, 0.75942E+00_JPRB ,&
&  0.77745E+00_JPRB, 0.77244E+00_JPRB, 0.75218E+00_JPRB, 0.75234E+00_JPRB, 0.76939E+00_JPRB, 0.82952E+00_JPRB, 0.96315E+00_JPRB ,&
&  0.16099E+01_JPRB, 0.37167E+01_JPRB, 0.18887E+01_JPRB, 0.26023E+01_JPRB, 0.18101E+01_JPRB, 0.14699E+01_JPRB, 0.11891E+01_JPRB ,&
&  0.12094E+01_JPRB, 0.14272E+01_JPRB, 0.12252E+01_JPRB, 0.10901E+01_JPRB, 0.95392E+00_JPRB, 0.89080E+00_JPRB, 0.95929E+00_JPRB ,&
&  0.10586E+01_JPRB, 0.11086E+01_JPRB, 0.11713E+01_JPRB, 0.12120E+01_JPRB, 0.11973E+01_JPRB, 0.12051E+01_JPRB, 0.12557E+01_JPRB ,&
&  0.13284E+01_JPRB, 0.14884E+01_JPRB, 0.18214E+01_JPRB, 0.23403E+01_JPRB, 0.35056E+01_JPRB, 0.47720E+01_JPRB, 0.61538E+01_JPRB ,&
&  0.61969E+01_JPRB, 0.60400E+01_JPRB, 0.44673E+01_JPRB, 0.28966E+01_JPRB, 0.19603E+01_JPRB, 0.16474E+01_JPRB, 0.13789E+01_JPRB ,&
&  0.12497E+01_JPRB, 0.11976E+01_JPRB, 0.11563E+01_JPRB, 0.11683E+01_JPRB, 0.12540E+01_JPRB, 0.13104E+01_JPRB, 0.12935E+01_JPRB ,&
&  0.12803E+01_JPRB, 0.12465E+01_JPRB, 0.12259E+01_JPRB, 0.11989E+01_JPRB, 0.11972E+01_JPRB, 0.11913E+01_JPRB, 0.11778E+01_JPRB ,&
&  0.11713E+01_JPRB, 0.11822E+01_JPRB, 0.11427E+01_JPRB, 0.11437E+01_JPRB, 0.11412E+01_JPRB, 0.11244E+01_JPRB, 0.10726E+01_JPRB ,&
&  0.10928E+01_JPRB, 0.11193E+01_JPRB, 0.11254E+01_JPRB, 0.11080E+01_JPRB, 0.11111E+01_JPRB, 0.11649E+01_JPRB, 0.11293E+01_JPRB ,&
&  0.11189E+01_JPRB, 0.10872E+01_JPRB, 0.10190E+01_JPRB, 0.98336E+00_JPRB, 0.96611E+00_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.96003E+00_JPRB, 0.91468E+00_JPRB, 0.87920E+00_JPRB, 0.83387E+00_JPRB, 0.80048E+00_JPRB, 0.78205E+00_JPRB, 0.78498E+00_JPRB ,&
&  0.77648E+00_JPRB, 0.75004E+00_JPRB, 0.73361E+00_JPRB, 0.73585E+00_JPRB, 0.73707E+00_JPRB, 0.74675E+00_JPRB, 0.75450E+00_JPRB ,&
&  0.75011E+00_JPRB, 0.73825E+00_JPRB, 0.73305E+00_JPRB, 0.73434E+00_JPRB, 0.75952E+00_JPRB, 0.79704E+00_JPRB, 0.80580E+00_JPRB ,&
&  0.79426E+00_JPRB, 0.78584E+00_JPRB, 0.76508E+00_JPRB, 0.74697E+00_JPRB, 0.74762E+00_JPRB, 0.78174E+00_JPRB, 0.88965E+00_JPRB ,&
&  0.15154E+01_JPRB, 0.40091E+01_JPRB, 0.17875E+01_JPRB, 0.27398E+01_JPRB, 0.19188E+01_JPRB, 0.15453E+01_JPRB, 0.12415E+01_JPRB ,&
&  0.12437E+01_JPRB, 0.13689E+01_JPRB, 0.12487E+01_JPRB, 0.11334E+01_JPRB, 0.10208E+01_JPRB, 0.95224E+00_JPRB, 0.93519E+00_JPRB ,&
&  0.99421E+00_JPRB, 0.10139E+01_JPRB, 0.10141E+01_JPRB, 0.10300E+01_JPRB, 0.10116E+01_JPRB, 0.10012E+01_JPRB, 0.10153E+01_JPRB ,&
&  0.10223E+01_JPRB, 0.10571E+01_JPRB, 0.12206E+01_JPRB, 0.18501E+01_JPRB, 0.27674E+01_JPRB, 0.45179E+01_JPRB, 0.70626E+01_JPRB ,&
&  0.68642E+01_JPRB, 0.70974E+01_JPRB, 0.50154E+01_JPRB, 0.32582E+01_JPRB, 0.21514E+01_JPRB, 0.17219E+01_JPRB, 0.15452E+01_JPRB ,&
&  0.13953E+01_JPRB, 0.12911E+01_JPRB, 0.12283E+01_JPRB, 0.12267E+01_JPRB, 0.12844E+01_JPRB, 0.13043E+01_JPRB, 0.13201E+01_JPRB ,&
&  0.13099E+01_JPRB, 0.12446E+01_JPRB, 0.12117E+01_JPRB, 0.11951E+01_JPRB, 0.11994E+01_JPRB, 0.11934E+01_JPRB, 0.11771E+01_JPRB ,&
&  0.11643E+01_JPRB, 0.11580E+01_JPRB, 0.11341E+01_JPRB, 0.11479E+01_JPRB, 0.11593E+01_JPRB, 0.11466E+01_JPRB, 0.10913E+01_JPRB ,&
&  0.10983E+01_JPRB, 0.11127E+01_JPRB, 0.11047E+01_JPRB, 0.11134E+01_JPRB, 0.11165E+01_JPRB, 0.11784E+01_JPRB, 0.11324E+01_JPRB ,&
&  0.11189E+01_JPRB, 0.10916E+01_JPRB, 0.10383E+01_JPRB, 0.10115E+01_JPRB, 0.10013E+01_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.99772E+00_JPRB, 0.96487E+00_JPRB, 0.93969E+00_JPRB, 0.90488E+00_JPRB, 0.85706E+00_JPRB, 0.83238E+00_JPRB, 0.81645E+00_JPRB ,&
&  0.80802E+00_JPRB, 0.78124E+00_JPRB, 0.76185E+00_JPRB, 0.76131E+00_JPRB, 0.76724E+00_JPRB, 0.76205E+00_JPRB, 0.76201E+00_JPRB ,&
&  0.76951E+00_JPRB, 0.77262E+00_JPRB, 0.76283E+00_JPRB, 0.75164E+00_JPRB, 0.74911E+00_JPRB, 0.78164E+00_JPRB, 0.79709E+00_JPRB ,&
&  0.80281E+00_JPRB, 0.79104E+00_JPRB, 0.76730E+00_JPRB, 0.73838E+00_JPRB, 0.73198E+00_JPRB, 0.74885E+00_JPRB, 0.83018E+00_JPRB ,&
&  0.13193E+01_JPRB, 0.16593E+01_JPRB, 0.28241E+01_JPRB, 0.28321E+01_JPRB, 0.20111E+01_JPRB, 0.16281E+01_JPRB, 0.14246E+01_JPRB ,&
&  0.14151E+01_JPRB, 0.13597E+01_JPRB, 0.12511E+01_JPRB, 0.11602E+01_JPRB, 0.10777E+01_JPRB, 0.10094E+01_JPRB, 0.96199E+00_JPRB ,&
&  0.95095E+00_JPRB, 0.95401E+00_JPRB, 0.95582E+00_JPRB, 0.95882E+00_JPRB, 0.95578E+00_JPRB, 0.95541E+00_JPRB, 0.95606E+00_JPRB ,&
&  0.95545E+00_JPRB, 0.95219E+00_JPRB, 0.98572E+00_JPRB, 0.12546E+01_JPRB, 0.23350E+01_JPRB, 0.39675E+01_JPRB, 0.62803E+01_JPRB ,&
&  0.72708E+01_JPRB, 0.67809E+01_JPRB, 0.47214E+01_JPRB, 0.33333E+01_JPRB, 0.22828E+01_JPRB, 0.17355E+01_JPRB, 0.15688E+01_JPRB ,&
&  0.14941E+01_JPRB, 0.14225E+01_JPRB, 0.13210E+01_JPRB, 0.12915E+01_JPRB, 0.12879E+01_JPRB, 0.12709E+01_JPRB, 0.12707E+01_JPRB ,&
&  0.12569E+01_JPRB, 0.12462E+01_JPRB, 0.12251E+01_JPRB, 0.12037E+01_JPRB, 0.11813E+01_JPRB, 0.11572E+01_JPRB, 0.11415E+01_JPRB ,&
&  0.11308E+01_JPRB, 0.11152E+01_JPRB, 0.11042E+01_JPRB, 0.11341E+01_JPRB, 0.11921E+01_JPRB, 0.11984E+01_JPRB, 0.11705E+01_JPRB ,&
&  0.11211E+01_JPRB, 0.10978E+01_JPRB, 0.10890E+01_JPRB, 0.11106E+01_JPRB, 0.10933E+01_JPRB, 0.11860E+01_JPRB, 0.11501E+01_JPRB ,&
&  0.11304E+01_JPRB, 0.11074E+01_JPRB, 0.10650E+01_JPRB, 0.10274E+01_JPRB, 0.10067E+01_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.10081E+01_JPRB, 0.96944E+00_JPRB, 0.94381E+00_JPRB, 0.92671E+00_JPRB, 0.89668E+00_JPRB, 0.86524E+00_JPRB, 0.85722E+00_JPRB ,&
&  0.85450E+00_JPRB, 0.83530E+00_JPRB, 0.81187E+00_JPRB, 0.78209E+00_JPRB, 0.77449E+00_JPRB, 0.77957E+00_JPRB, 0.78574E+00_JPRB ,&
&  0.78606E+00_JPRB, 0.78734E+00_JPRB, 0.78587E+00_JPRB, 0.78271E+00_JPRB, 0.77912E+00_JPRB, 0.79886E+00_JPRB, 0.80661E+00_JPRB ,&
&  0.79781E+00_JPRB, 0.79750E+00_JPRB, 0.77514E+00_JPRB, 0.75194E+00_JPRB, 0.74149E+00_JPRB, 0.73149E+00_JPRB, 0.74139E+00_JPRB ,&
&  0.10647E+01_JPRB, 0.18738E+01_JPRB, 0.26946E+01_JPRB, 0.24957E+01_JPRB, 0.19763E+01_JPRB, 0.15923E+01_JPRB, 0.14889E+01_JPRB ,&
&  0.14203E+01_JPRB, 0.13488E+01_JPRB, 0.12508E+01_JPRB, 0.11718E+01_JPRB, 0.11036E+01_JPRB, 0.10504E+01_JPRB, 0.99841E+00_JPRB ,&
&  0.97078E+00_JPRB, 0.96372E+00_JPRB, 0.94601E+00_JPRB, 0.93663E+00_JPRB, 0.93481E+00_JPRB, 0.94187E+00_JPRB, 0.94989E+00_JPRB ,&
&  0.95446E+00_JPRB, 0.95277E+00_JPRB, 0.95708E+00_JPRB, 0.10495E+01_JPRB, 0.16784E+01_JPRB, 0.21422E+01_JPRB, 0.40741E+01_JPRB ,&
&  0.57937E+01_JPRB, 0.46892E+01_JPRB, 0.35477E+01_JPRB, 0.28705E+01_JPRB, 0.22788E+01_JPRB, 0.17693E+01_JPRB, 0.15286E+01_JPRB ,&
&  0.14385E+01_JPRB, 0.13859E+01_JPRB, 0.13136E+01_JPRB, 0.12691E+01_JPRB, 0.12344E+01_JPRB, 0.12149E+01_JPRB, 0.12034E+01_JPRB ,&
&  0.11789E+01_JPRB, 0.11761E+01_JPRB, 0.11702E+01_JPRB, 0.11640E+01_JPRB, 0.11427E+01_JPRB, 0.11233E+01_JPRB, 0.11065E+01_JPRB ,&
&  0.10940E+01_JPRB, 0.10743E+01_JPRB, 0.10644E+01_JPRB, 0.10923E+01_JPRB, 0.11776E+01_JPRB, 0.12392E+01_JPRB, 0.12316E+01_JPRB ,&
&  0.11687E+01_JPRB, 0.11142E+01_JPRB, 0.11131E+01_JPRB, 0.11065E+01_JPRB, 0.11113E+01_JPRB, 0.12242E+01_JPRB, 0.11771E+01_JPRB ,&
&  0.11434E+01_JPRB, 0.11265E+01_JPRB, 0.10964E+01_JPRB, 0.10580E+01_JPRB, 0.10356E+01_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.10474E+01_JPRB, 0.10042E+01_JPRB, 0.96798E+00_JPRB, 0.93416E+00_JPRB, 0.92307E+00_JPRB, 0.90245E+00_JPRB, 0.87721E+00_JPRB ,&
&  0.86388E+00_JPRB, 0.86332E+00_JPRB, 0.85647E+00_JPRB, 0.83934E+00_JPRB, 0.82399E+00_JPRB, 0.81801E+00_JPRB, 0.81789E+00_JPRB ,&
&  0.81661E+00_JPRB, 0.81535E+00_JPRB, 0.81236E+00_JPRB, 0.80369E+00_JPRB, 0.80576E+00_JPRB, 0.81392E+00_JPRB, 0.82175E+00_JPRB ,&
&  0.80275E+00_JPRB, 0.78723E+00_JPRB, 0.77464E+00_JPRB, 0.76187E+00_JPRB, 0.75812E+00_JPRB, 0.74079E+00_JPRB, 0.72171E+00_JPRB ,&
&  0.85509E+00_JPRB, 0.13140E+01_JPRB, 0.18260E+01_JPRB, 0.22362E+01_JPRB, 0.19618E+01_JPRB, 0.16616E+01_JPRB, 0.15279E+01_JPRB ,&
&  0.14285E+01_JPRB, 0.13129E+01_JPRB, 0.12216E+01_JPRB, 0.11594E+01_JPRB, 0.11248E+01_JPRB, 0.10806E+01_JPRB, 0.10142E+01_JPRB ,&
&  0.96700E+00_JPRB, 0.94164E+00_JPRB, 0.93590E+00_JPRB, 0.93736E+00_JPRB, 0.93349E+00_JPRB, 0.94835E+00_JPRB, 0.96130E+00_JPRB ,&
&  0.96771E+00_JPRB, 0.96065E+00_JPRB, 0.94714E+00_JPRB, 0.95449E+00_JPRB, 0.11784E+01_JPRB, 0.13878E+01_JPRB, 0.20439E+01_JPRB ,&
&  0.23845E+01_JPRB, 0.25538E+01_JPRB, 0.24415E+01_JPRB, 0.23265E+01_JPRB, 0.21229E+01_JPRB, 0.17754E+01_JPRB, 0.15028E+01_JPRB ,&
&  0.13558E+01_JPRB, 0.12962E+01_JPRB, 0.12431E+01_JPRB, 0.12178E+01_JPRB, 0.11915E+01_JPRB, 0.11660E+01_JPRB, 0.11536E+01_JPRB ,&
&  0.11409E+01_JPRB, 0.11279E+01_JPRB, 0.11162E+01_JPRB, 0.11196E+01_JPRB, 0.11043E+01_JPRB, 0.10729E+01_JPRB, 0.10481E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.10242E+01_JPRB, 0.10361E+01_JPRB, 0.10756E+01_JPRB, 0.11166E+01_JPRB, 0.11453E+01_JPRB, 0.11733E+01_JPRB ,&
&  0.11932E+01_JPRB, 0.11754E+01_JPRB, 0.11463E+01_JPRB, 0.11431E+01_JPRB, 0.11434E+01_JPRB, 0.12342E+01_JPRB, 0.11907E+01_JPRB ,&
&  0.11621E+01_JPRB, 0.11409E+01_JPRB, 0.11152E+01_JPRB, 0.10649E+01_JPRB, 0.10838E+01_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.10503E+01_JPRB, 0.10300E+01_JPRB, 0.10042E+01_JPRB, 0.96553E+00_JPRB, 0.94864E+00_JPRB, 0.93248E+00_JPRB, 0.91466E+00_JPRB ,&
&  0.90571E+00_JPRB, 0.89746E+00_JPRB, 0.89136E+00_JPRB, 0.87709E+00_JPRB, 0.86236E+00_JPRB, 0.85311E+00_JPRB, 0.84168E+00_JPRB ,&
&  0.84178E+00_JPRB, 0.84159E+00_JPRB, 0.83306E+00_JPRB, 0.82408E+00_JPRB, 0.81981E+00_JPRB, 0.81644E+00_JPRB, 0.81979E+00_JPRB ,&
&  0.81046E+00_JPRB, 0.79880E+00_JPRB, 0.79733E+00_JPRB, 0.78874E+00_JPRB, 0.77511E+00_JPRB, 0.76400E+00_JPRB, 0.73434E+00_JPRB ,&
&  0.75254E+00_JPRB, 0.87497E+00_JPRB, 0.12486E+01_JPRB, 0.16707E+01_JPRB, 0.16518E+01_JPRB, 0.15994E+01_JPRB, 0.14814E+01_JPRB ,&
&  0.13733E+01_JPRB, 0.12760E+01_JPRB, 0.11681E+01_JPRB, 0.10970E+01_JPRB, 0.10761E+01_JPRB, 0.10514E+01_JPRB, 0.10208E+01_JPRB ,&
&  0.98549E+00_JPRB, 0.95453E+00_JPRB, 0.93727E+00_JPRB, 0.94376E+00_JPRB, 0.94002E+00_JPRB, 0.94353E+00_JPRB, 0.95246E+00_JPRB ,&
&  0.95277E+00_JPRB, 0.94809E+00_JPRB, 0.92866E+00_JPRB, 0.91340E+00_JPRB, 0.90704E+00_JPRB, 0.93094E+00_JPRB, 0.11011E+01_JPRB ,&
&  0.13513E+01_JPRB, 0.14806E+01_JPRB, 0.15510E+01_JPRB, 0.16312E+01_JPRB, 0.16732E+01_JPRB, 0.16232E+01_JPRB, 0.14774E+01_JPRB ,&
&  0.13565E+01_JPRB, 0.12808E+01_JPRB, 0.12089E+01_JPRB, 0.12043E+01_JPRB, 0.11820E+01_JPRB, 0.11512E+01_JPRB, 0.11200E+01_JPRB ,&
&  0.10873E+01_JPRB, 0.10771E+01_JPRB, 0.10562E+01_JPRB, 0.10420E+01_JPRB, 0.10271E+01_JPRB, 0.10057E+01_JPRB, 0.98268E+00_JPRB ,&
&  0.97326E+00_JPRB, 0.96610E+00_JPRB, 0.97608E+00_JPRB, 0.10081E+01_JPRB, 0.10485E+01_JPRB, 0.10771E+01_JPRB, 0.11070E+01_JPRB ,&
&  0.11314E+01_JPRB, 0.11577E+01_JPRB, 0.11253E+01_JPRB, 0.11130E+01_JPRB, 0.12177E+01_JPRB, 0.12137E+01_JPRB, 0.11725E+01_JPRB ,&
&  0.11460E+01_JPRB, 0.11204E+01_JPRB, 0.11031E+01_JPRB, 0.10439E+01_JPRB, 0.10306E+01_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.93791E+00_JPRB, 0.96255E+00_JPRB, 0.95487E+00_JPRB, 0.92722E+00_JPRB, 0.91184E+00_JPRB, 0.90618E+00_JPRB, 0.90563E+00_JPRB ,&
&  0.90260E+00_JPRB, 0.89293E+00_JPRB, 0.89811E+00_JPRB, 0.90435E+00_JPRB, 0.90123E+00_JPRB, 0.88840E+00_JPRB, 0.87871E+00_JPRB ,&
&  0.87533E+00_JPRB, 0.87815E+00_JPRB, 0.88225E+00_JPRB, 0.87137E+00_JPRB, 0.84938E+00_JPRB, 0.83458E+00_JPRB, 0.82167E+00_JPRB ,&
&  0.81971E+00_JPRB, 0.81912E+00_JPRB, 0.81642E+00_JPRB, 0.81730E+00_JPRB, 0.79570E+00_JPRB, 0.77151E+00_JPRB, 0.75195E+00_JPRB ,&
&  0.73976E+00_JPRB, 0.68872E+00_JPRB, 0.87153E+00_JPRB, 0.11509E+01_JPRB, 0.12874E+01_JPRB, 0.12841E+01_JPRB, 0.12032E+01_JPRB ,&
&  0.11511E+01_JPRB, 0.11064E+01_JPRB, 0.10613E+01_JPRB, 0.10132E+01_JPRB, 0.98530E+00_JPRB, 0.96869E+00_JPRB, 0.95550E+00_JPRB ,&
&  0.94257E+00_JPRB, 0.92810E+00_JPRB, 0.91838E+00_JPRB, 0.90022E+00_JPRB, 0.88508E+00_JPRB, 0.88229E+00_JPRB, 0.88376E+00_JPRB ,&
&  0.88950E+00_JPRB, 0.88639E+00_JPRB, 0.88568E+00_JPRB, 0.88342E+00_JPRB, 0.86125E+00_JPRB, 0.85817E+00_JPRB, 0.87977E+00_JPRB ,&
&  0.95733E+00_JPRB, 0.10220E+01_JPRB, 0.10656E+01_JPRB, 0.11604E+01_JPRB, 0.12455E+01_JPRB, 0.13163E+01_JPRB, 0.13124E+01_JPRB ,&
&  0.12703E+01_JPRB, 0.12326E+01_JPRB, 0.11917E+01_JPRB, 0.11729E+01_JPRB, 0.11502E+01_JPRB, 0.11068E+01_JPRB, 0.10617E+01_JPRB ,&
&  0.10368E+01_JPRB, 0.10189E+01_JPRB, 0.99318E+00_JPRB, 0.97320E+00_JPRB, 0.95145E+00_JPRB, 0.92345E+00_JPRB, 0.89938E+00_JPRB ,&
&  0.88271E+00_JPRB, 0.87854E+00_JPRB, 0.88361E+00_JPRB, 0.90434E+00_JPRB, 0.93537E+00_JPRB, 0.97213E+00_JPRB, 0.10092E+01_JPRB ,&
&  0.10430E+01_JPRB, 0.10680E+01_JPRB, 0.10985E+01_JPRB, 0.11462E+01_JPRB, 0.11457E+01_JPRB, 0.11358E+01_JPRB, 0.11369E+01_JPRB ,&
&  0.11187E+01_JPRB, 0.10813E+01_JPRB, 0.10411E+01_JPRB, 0.97529E+00_JPRB, 0.88812E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.89447E+00_JPRB, 0.91031E+00_JPRB, 0.90864E+00_JPRB, 0.89827E+00_JPRB, 0.89003E+00_JPRB, 0.88562E+00_JPRB, 0.88999E+00_JPRB ,&
&  0.88946E+00_JPRB, 0.88448E+00_JPRB, 0.88760E+00_JPRB, 0.88957E+00_JPRB, 0.89083E+00_JPRB, 0.88720E+00_JPRB, 0.89094E+00_JPRB ,&
&  0.89077E+00_JPRB, 0.89168E+00_JPRB, 0.89427E+00_JPRB, 0.88742E+00_JPRB, 0.86695E+00_JPRB, 0.85230E+00_JPRB, 0.83789E+00_JPRB ,&
&  0.83654E+00_JPRB, 0.83588E+00_JPRB, 0.83611E+00_JPRB, 0.82951E+00_JPRB, 0.80898E+00_JPRB, 0.79110E+00_JPRB, 0.77980E+00_JPRB ,&
&  0.77957E+00_JPRB, 0.64319E+00_JPRB, 0.72625E+00_JPRB, 0.95357E+00_JPRB, 0.10609E+01_JPRB, 0.10254E+01_JPRB, 0.10100E+01_JPRB ,&
&  0.10136E+01_JPRB, 0.10106E+01_JPRB, 0.99102E+00_JPRB, 0.95766E+00_JPRB, 0.91142E+00_JPRB, 0.89577E+00_JPRB, 0.87406E+00_JPRB ,&
&  0.85459E+00_JPRB, 0.84670E+00_JPRB, 0.83962E+00_JPRB, 0.83823E+00_JPRB, 0.82725E+00_JPRB, 0.81491E+00_JPRB, 0.81518E+00_JPRB ,&
&  0.82355E+00_JPRB, 0.82507E+00_JPRB, 0.83165E+00_JPRB, 0.83756E+00_JPRB, 0.83203E+00_JPRB, 0.84612E+00_JPRB, 0.85557E+00_JPRB ,&
&  0.87435E+00_JPRB, 0.89208E+00_JPRB, 0.90711E+00_JPRB, 0.93676E+00_JPRB, 0.98753E+00_JPRB, 0.10748E+01_JPRB, 0.11281E+01_JPRB ,&
&  0.11275E+01_JPRB, 0.11104E+01_JPRB, 0.11060E+01_JPRB, 0.10990E+01_JPRB, 0.10821E+01_JPRB, 0.10497E+01_JPRB, 0.10144E+01_JPRB ,&
&  0.98933E+00_JPRB, 0.96462E+00_JPRB, 0.94123E+00_JPRB, 0.91564E+00_JPRB, 0.88956E+00_JPRB, 0.86547E+00_JPRB, 0.84561E+00_JPRB ,&
&  0.83249E+00_JPRB, 0.82703E+00_JPRB, 0.83549E+00_JPRB, 0.85711E+00_JPRB, 0.88584E+00_JPRB, 0.91877E+00_JPRB, 0.95300E+00_JPRB ,&
&  0.99127E+00_JPRB, 0.10305E+01_JPRB, 0.10692E+01_JPRB, 0.10265E+01_JPRB, 0.10878E+01_JPRB, 0.10973E+01_JPRB, 0.10993E+01_JPRB ,&
&  0.10847E+01_JPRB, 0.10416E+01_JPRB, 0.97891E+00_JPRB, 0.77893E+00_JPRB, 0.85768E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.87853E+00_JPRB, 0.88398E+00_JPRB, 0.88701E+00_JPRB, 0.87497E+00_JPRB, 0.85967E+00_JPRB, 0.85430E+00_JPRB, 0.86160E+00_JPRB ,&
&  0.85793E+00_JPRB, 0.84500E+00_JPRB, 0.84977E+00_JPRB, 0.85292E+00_JPRB, 0.85925E+00_JPRB, 0.86178E+00_JPRB, 0.86466E+00_JPRB ,&
&  0.86772E+00_JPRB, 0.87093E+00_JPRB, 0.86880E+00_JPRB, 0.87689E+00_JPRB, 0.86923E+00_JPRB, 0.86338E+00_JPRB, 0.86029E+00_JPRB ,&
&  0.85177E+00_JPRB, 0.84304E+00_JPRB, 0.83565E+00_JPRB, 0.82529E+00_JPRB, 0.81717E+00_JPRB, 0.80742E+00_JPRB, 0.79160E+00_JPRB ,&
&  0.80264E+00_JPRB, 0.65278E+00_JPRB, 0.74031E+00_JPRB, 0.89345E+00_JPRB, 0.89584E+00_JPRB, 0.88511E+00_JPRB, 0.89844E+00_JPRB ,&
&  0.91930E+00_JPRB, 0.92945E+00_JPRB, 0.92126E+00_JPRB, 0.89592E+00_JPRB, 0.84597E+00_JPRB, 0.82155E+00_JPRB, 0.80750E+00_JPRB ,&
&  0.79187E+00_JPRB, 0.78019E+00_JPRB, 0.77301E+00_JPRB, 0.77451E+00_JPRB, 0.77478E+00_JPRB, 0.77178E+00_JPRB, 0.76816E+00_JPRB ,&
&  0.77127E+00_JPRB, 0.78084E+00_JPRB, 0.78724E+00_JPRB, 0.79408E+00_JPRB, 0.80043E+00_JPRB, 0.79656E+00_JPRB, 0.79544E+00_JPRB ,&
&  0.79591E+00_JPRB, 0.80075E+00_JPRB, 0.80459E+00_JPRB, 0.81754E+00_JPRB, 0.84197E+00_JPRB, 0.89739E+00_JPRB, 0.94830E+00_JPRB ,&
&  0.96835E+00_JPRB, 0.97261E+00_JPRB, 0.97420E+00_JPRB, 0.97453E+00_JPRB, 0.97633E+00_JPRB, 0.97107E+00_JPRB, 0.95329E+00_JPRB ,&
&  0.93187E+00_JPRB, 0.90579E+00_JPRB, 0.88608E+00_JPRB, 0.87066E+00_JPRB, 0.85388E+00_JPRB, 0.84191E+00_JPRB, 0.83766E+00_JPRB ,&
&  0.82883E+00_JPRB, 0.81652E+00_JPRB, 0.81972E+00_JPRB, 0.83582E+00_JPRB, 0.85220E+00_JPRB, 0.86581E+00_JPRB, 0.88906E+00_JPRB ,&
&  0.92758E+00_JPRB, 0.96717E+00_JPRB, 0.99958E+00_JPRB, 0.10314E+01_JPRB, 0.10453E+01_JPRB, 0.10488E+01_JPRB, 0.10408E+01_JPRB ,&
&  0.10204E+01_JPRB, 0.99674E+00_JPRB, 0.73961E+00_JPRB, 0.84198E+00_JPRB, 0.86699E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.88204E+00_JPRB, 0.87245E+00_JPRB, 0.86082E+00_JPRB, 0.84427E+00_JPRB, 0.82779E+00_JPRB, 0.81834E+00_JPRB, 0.82411E+00_JPRB ,&
&  0.82293E+00_JPRB, 0.81106E+00_JPRB, 0.82109E+00_JPRB, 0.83399E+00_JPRB, 0.84419E+00_JPRB, 0.85338E+00_JPRB, 0.85606E+00_JPRB ,&
&  0.85920E+00_JPRB, 0.85596E+00_JPRB, 0.85911E+00_JPRB, 0.87134E+00_JPRB, 0.89003E+00_JPRB, 0.89168E+00_JPRB, 0.88859E+00_JPRB ,&
&  0.86621E+00_JPRB, 0.85500E+00_JPRB, 0.84474E+00_JPRB, 0.83224E+00_JPRB, 0.82077E+00_JPRB, 0.81637E+00_JPRB, 0.81830E+00_JPRB ,&
&  0.70402E+00_JPRB, 0.60998E+00_JPRB, 0.72726E+00_JPRB, 0.82294E+00_JPRB, 0.82289E+00_JPRB, 0.82014E+00_JPRB, 0.83719E+00_JPRB ,&
&  0.84327E+00_JPRB, 0.84291E+00_JPRB, 0.84117E+00_JPRB, 0.83093E+00_JPRB, 0.79979E+00_JPRB, 0.76527E+00_JPRB, 0.74712E+00_JPRB ,&
&  0.73639E+00_JPRB, 0.74125E+00_JPRB, 0.74325E+00_JPRB, 0.75066E+00_JPRB, 0.75089E+00_JPRB, 0.74391E+00_JPRB, 0.74222E+00_JPRB ,&
&  0.74263E+00_JPRB, 0.75215E+00_JPRB, 0.76108E+00_JPRB, 0.76770E+00_JPRB, 0.77015E+00_JPRB, 0.77059E+00_JPRB, 0.76719E+00_JPRB ,&
&  0.76501E+00_JPRB, 0.76412E+00_JPRB, 0.76278E+00_JPRB, 0.76852E+00_JPRB, 0.78212E+00_JPRB, 0.80850E+00_JPRB, 0.84684E+00_JPRB ,&
&  0.87160E+00_JPRB, 0.88128E+00_JPRB, 0.88624E+00_JPRB, 0.88472E+00_JPRB, 0.89230E+00_JPRB, 0.89497E+00_JPRB, 0.88217E+00_JPRB ,&
&  0.86538E+00_JPRB, 0.85197E+00_JPRB, 0.84850E+00_JPRB, 0.84427E+00_JPRB, 0.84226E+00_JPRB, 0.83840E+00_JPRB, 0.83030E+00_JPRB ,&
&  0.82078E+00_JPRB, 0.80738E+00_JPRB, 0.79769E+00_JPRB, 0.79868E+00_JPRB, 0.81086E+00_JPRB, 0.82159E+00_JPRB, 0.84277E+00_JPRB ,&
&  0.87855E+00_JPRB, 0.92047E+00_JPRB, 0.95512E+00_JPRB, 0.99076E+00_JPRB, 0.10106E+01_JPRB, 0.10183E+01_JPRB, 0.10090E+01_JPRB ,&
&  0.98798E+00_JPRB, 0.94765E+00_JPRB, 0.93154E+00_JPRB, 0.90624E+00_JPRB, 0.89306E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.84067E+00_JPRB, 0.83348E+00_JPRB, 0.81928E+00_JPRB, 0.79383E+00_JPRB, 0.76069E+00_JPRB, 0.76895E+00_JPRB, 0.78024E+00_JPRB ,&
&  0.77360E+00_JPRB, 0.76405E+00_JPRB, 0.77465E+00_JPRB, 0.79990E+00_JPRB, 0.82435E+00_JPRB, 0.83819E+00_JPRB, 0.84010E+00_JPRB ,&
&  0.83469E+00_JPRB, 0.83634E+00_JPRB, 0.84455E+00_JPRB, 0.86181E+00_JPRB, 0.86110E+00_JPRB, 0.84321E+00_JPRB, 0.83220E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82742E+00_JPRB, 0.82495E+00_JPRB, 0.80826E+00_JPRB, 0.79737E+00_JPRB, 0.79447E+00_JPRB, 0.79490E+00_JPRB ,&
&  0.79087E+00_JPRB, 0.65194E+00_JPRB, 0.76068E+00_JPRB, 0.76925E+00_JPRB, 0.77736E+00_JPRB, 0.76892E+00_JPRB, 0.76850E+00_JPRB ,&
&  0.77259E+00_JPRB, 0.78788E+00_JPRB, 0.78072E+00_JPRB, 0.75712E+00_JPRB, 0.73217E+00_JPRB, 0.70490E+00_JPRB, 0.68638E+00_JPRB ,&
&  0.67708E+00_JPRB, 0.68016E+00_JPRB, 0.69039E+00_JPRB, 0.70130E+00_JPRB, 0.71281E+00_JPRB, 0.71469E+00_JPRB, 0.71865E+00_JPRB ,&
&  0.72115E+00_JPRB, 0.72582E+00_JPRB, 0.72783E+00_JPRB, 0.73296E+00_JPRB, 0.73470E+00_JPRB, 0.73125E+00_JPRB, 0.72354E+00_JPRB ,&
&  0.71255E+00_JPRB, 0.70658E+00_JPRB, 0.70714E+00_JPRB, 0.71104E+00_JPRB, 0.72301E+00_JPRB, 0.74519E+00_JPRB, 0.77151E+00_JPRB ,&
&  0.79859E+00_JPRB, 0.81617E+00_JPRB, 0.82592E+00_JPRB, 0.82815E+00_JPRB, 0.83480E+00_JPRB, 0.84136E+00_JPRB, 0.83673E+00_JPRB ,&
&  0.82420E+00_JPRB, 0.81059E+00_JPRB, 0.80258E+00_JPRB, 0.79396E+00_JPRB, 0.78991E+00_JPRB, 0.78682E+00_JPRB, 0.78044E+00_JPRB ,&
&  0.77204E+00_JPRB, 0.77100E+00_JPRB, 0.77324E+00_JPRB, 0.76355E+00_JPRB, 0.76645E+00_JPRB, 0.77829E+00_JPRB, 0.79916E+00_JPRB ,&
&  0.82507E+00_JPRB, 0.85872E+00_JPRB, 0.89151E+00_JPRB, 0.91773E+00_JPRB, 0.93256E+00_JPRB, 0.93606E+00_JPRB, 0.92884E+00_JPRB ,&
&  0.91021E+00_JPRB, 0.88505E+00_JPRB, 0.86618E+00_JPRB, 0.85580E+00_JPRB, 0.85608E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.79694E+00_JPRB, 0.79733E+00_JPRB, 0.78707E+00_JPRB, 0.74929E+00_JPRB, 0.72059E+00_JPRB, 0.72992E+00_JPRB, 0.73834E+00_JPRB ,&
&  0.73774E+00_JPRB, 0.75280E+00_JPRB, 0.77794E+00_JPRB, 0.80334E+00_JPRB, 0.82029E+00_JPRB, 0.81995E+00_JPRB, 0.82366E+00_JPRB ,&
&  0.82084E+00_JPRB, 0.81854E+00_JPRB, 0.82838E+00_JPRB, 0.83328E+00_JPRB, 0.82258E+00_JPRB, 0.80163E+00_JPRB, 0.79941E+00_JPRB ,&
&  0.80560E+00_JPRB, 0.80132E+00_JPRB, 0.78318E+00_JPRB, 0.77094E+00_JPRB, 0.76110E+00_JPRB, 0.76659E+00_JPRB, 0.77377E+00_JPRB ,&
&  0.76837E+00_JPRB, 0.72099E+00_JPRB, 0.75913E+00_JPRB, 0.76859E+00_JPRB, 0.77309E+00_JPRB, 0.75858E+00_JPRB, 0.76204E+00_JPRB ,&
&  0.77228E+00_JPRB, 0.77839E+00_JPRB, 0.75829E+00_JPRB, 0.73556E+00_JPRB, 0.71548E+00_JPRB, 0.69863E+00_JPRB, 0.67128E+00_JPRB ,&
&  0.66105E+00_JPRB, 0.65806E+00_JPRB, 0.67165E+00_JPRB, 0.68615E+00_JPRB, 0.70293E+00_JPRB, 0.70992E+00_JPRB, 0.72050E+00_JPRB ,&
&  0.72252E+00_JPRB, 0.71663E+00_JPRB, 0.71110E+00_JPRB, 0.70663E+00_JPRB, 0.70546E+00_JPRB, 0.69936E+00_JPRB, 0.69026E+00_JPRB ,&
&  0.68270E+00_JPRB, 0.68148E+00_JPRB, 0.68690E+00_JPRB, 0.69702E+00_JPRB, 0.71104E+00_JPRB, 0.73520E+00_JPRB, 0.75613E+00_JPRB ,&
&  0.78054E+00_JPRB, 0.79061E+00_JPRB, 0.79835E+00_JPRB, 0.81171E+00_JPRB, 0.81861E+00_JPRB, 0.81564E+00_JPRB, 0.81113E+00_JPRB ,&
&  0.80464E+00_JPRB, 0.79508E+00_JPRB, 0.79641E+00_JPRB, 0.79236E+00_JPRB, 0.79014E+00_JPRB, 0.77949E+00_JPRB, 0.76278E+00_JPRB ,&
&  0.75634E+00_JPRB, 0.75203E+00_JPRB, 0.74417E+00_JPRB, 0.74187E+00_JPRB, 0.73977E+00_JPRB, 0.75841E+00_JPRB, 0.79105E+00_JPRB ,&
&  0.80932E+00_JPRB, 0.83357E+00_JPRB, 0.86779E+00_JPRB, 0.88950E+00_JPRB, 0.88857E+00_JPRB, 0.88619E+00_JPRB, 0.87316E+00_JPRB ,&
&  0.85325E+00_JPRB, 0.83919E+00_JPRB, 0.82639E+00_JPRB, 0.82166E+00_JPRB, 0.81754E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.75208E+00_JPRB, 0.77385E+00_JPRB, 0.74153E+00_JPRB, 0.72407E+00_JPRB, 0.70355E+00_JPRB, 0.69771E+00_JPRB, 0.68831E+00_JPRB ,&
&  0.69924E+00_JPRB, 0.71663E+00_JPRB, 0.74054E+00_JPRB, 0.74848E+00_JPRB, 0.75519E+00_JPRB, 0.75784E+00_JPRB, 0.77444E+00_JPRB ,&
&  0.76948E+00_JPRB, 0.76105E+00_JPRB, 0.76403E+00_JPRB, 0.75362E+00_JPRB, 0.74485E+00_JPRB, 0.74992E+00_JPRB, 0.75921E+00_JPRB ,&
&  0.76952E+00_JPRB, 0.76077E+00_JPRB, 0.74334E+00_JPRB, 0.71687E+00_JPRB, 0.70442E+00_JPRB, 0.70176E+00_JPRB, 0.70940E+00_JPRB ,&
&  0.71498E+00_JPRB, 0.74094E+00_JPRB, 0.67663E+00_JPRB, 0.73262E+00_JPRB, 0.73902E+00_JPRB, 0.73537E+00_JPRB, 0.73287E+00_JPRB ,&
&  0.73677E+00_JPRB, 0.72594E+00_JPRB, 0.71704E+00_JPRB, 0.70863E+00_JPRB, 0.68594E+00_JPRB, 0.66778E+00_JPRB, 0.64869E+00_JPRB ,&
&  0.63179E+00_JPRB, 0.62146E+00_JPRB, 0.62690E+00_JPRB, 0.64492E+00_JPRB, 0.66329E+00_JPRB, 0.67346E+00_JPRB, 0.67953E+00_JPRB ,&
&  0.68268E+00_JPRB, 0.67525E+00_JPRB, 0.66516E+00_JPRB, 0.66505E+00_JPRB, 0.67121E+00_JPRB, 0.67577E+00_JPRB, 0.67844E+00_JPRB ,&
&  0.67840E+00_JPRB, 0.65627E+00_JPRB, 0.64432E+00_JPRB, 0.66146E+00_JPRB, 0.68094E+00_JPRB, 0.70367E+00_JPRB, 0.73314E+00_JPRB ,&
&  0.75609E+00_JPRB, 0.76548E+00_JPRB, 0.76604E+00_JPRB, 0.77257E+00_JPRB, 0.77802E+00_JPRB, 0.78564E+00_JPRB, 0.79142E+00_JPRB ,&
&  0.77860E+00_JPRB, 0.77447E+00_JPRB, 0.77441E+00_JPRB, 0.77351E+00_JPRB, 0.76315E+00_JPRB, 0.73867E+00_JPRB, 0.70534E+00_JPRB ,&
&  0.68995E+00_JPRB, 0.67491E+00_JPRB, 0.66407E+00_JPRB, 0.66064E+00_JPRB, 0.66179E+00_JPRB, 0.68833E+00_JPRB, 0.72695E+00_JPRB ,&
&  0.76545E+00_JPRB, 0.79633E+00_JPRB, 0.82293E+00_JPRB, 0.84714E+00_JPRB, 0.84560E+00_JPRB, 0.82323E+00_JPRB, 0.81285E+00_JPRB ,&
&  0.79704E+00_JPRB, 0.78007E+00_JPRB, 0.76847E+00_JPRB, 0.75950E+00_JPRB, 0.75019E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.75293E+00_JPRB, 0.75469E+00_JPRB, 0.74039E+00_JPRB, 0.73053E+00_JPRB, 0.71337E+00_JPRB, 0.69068E+00_JPRB, 0.67111E+00_JPRB ,&
&  0.67297E+00_JPRB, 0.68868E+00_JPRB, 0.70373E+00_JPRB, 0.71454E+00_JPRB, 0.72989E+00_JPRB, 0.73286E+00_JPRB, 0.73899E+00_JPRB ,&
&  0.73431E+00_JPRB, 0.69703E+00_JPRB, 0.69108E+00_JPRB, 0.69865E+00_JPRB, 0.70051E+00_JPRB, 0.70676E+00_JPRB, 0.71786E+00_JPRB ,&
&  0.72801E+00_JPRB, 0.72340E+00_JPRB, 0.71518E+00_JPRB, 0.68550E+00_JPRB, 0.67876E+00_JPRB, 0.68361E+00_JPRB, 0.68848E+00_JPRB ,&
&  0.69851E+00_JPRB, 0.71539E+00_JPRB, 0.71791E+00_JPRB, 0.72599E+00_JPRB, 0.73384E+00_JPRB, 0.73719E+00_JPRB, 0.73334E+00_JPRB ,&
&  0.70680E+00_JPRB, 0.68036E+00_JPRB, 0.66934E+00_JPRB, 0.65197E+00_JPRB, 0.62979E+00_JPRB, 0.61426E+00_JPRB, 0.60676E+00_JPRB ,&
&  0.60286E+00_JPRB, 0.60812E+00_JPRB, 0.61439E+00_JPRB, 0.62843E+00_JPRB, 0.63371E+00_JPRB, 0.63782E+00_JPRB, 0.64317E+00_JPRB ,&
&  0.64292E+00_JPRB, 0.63909E+00_JPRB, 0.63944E+00_JPRB, 0.64940E+00_JPRB, 0.65520E+00_JPRB, 0.65262E+00_JPRB, 0.64782E+00_JPRB ,&
&  0.63502E+00_JPRB, 0.62249E+00_JPRB, 0.63073E+00_JPRB, 0.64676E+00_JPRB, 0.65505E+00_JPRB, 0.68257E+00_JPRB, 0.71792E+00_JPRB ,&
&  0.74809E+00_JPRB, 0.76727E+00_JPRB, 0.76731E+00_JPRB, 0.76242E+00_JPRB, 0.75928E+00_JPRB, 0.76582E+00_JPRB, 0.77736E+00_JPRB ,&
&  0.76068E+00_JPRB, 0.75127E+00_JPRB, 0.74909E+00_JPRB, 0.72944E+00_JPRB, 0.69230E+00_JPRB, 0.65960E+00_JPRB, 0.63573E+00_JPRB ,&
&  0.61689E+00_JPRB, 0.60629E+00_JPRB, 0.61001E+00_JPRB, 0.61312E+00_JPRB, 0.62531E+00_JPRB, 0.66112E+00_JPRB, 0.69897E+00_JPRB ,&
&  0.74018E+00_JPRB, 0.76151E+00_JPRB, 0.76860E+00_JPRB, 0.77700E+00_JPRB, 0.78289E+00_JPRB, 0.77154E+00_JPRB, 0.76188E+00_JPRB ,&
&  0.75877E+00_JPRB, 0.75005E+00_JPRB, 0.74067E+00_JPRB, 0.73129E+00_JPRB, 0.73917E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.74646E+00_JPRB, 0.73954E+00_JPRB, 0.72664E+00_JPRB, 0.70027E+00_JPRB, 0.68228E+00_JPRB, 0.65501E+00_JPRB, 0.63968E+00_JPRB ,&
&  0.64211E+00_JPRB, 0.64371E+00_JPRB, 0.64884E+00_JPRB, 0.65792E+00_JPRB, 0.65551E+00_JPRB, 0.65055E+00_JPRB, 0.64561E+00_JPRB ,&
&  0.64838E+00_JPRB, 0.64880E+00_JPRB, 0.65728E+00_JPRB, 0.67266E+00_JPRB, 0.68889E+00_JPRB, 0.69827E+00_JPRB, 0.70842E+00_JPRB ,&
&  0.71620E+00_JPRB, 0.71017E+00_JPRB, 0.70566E+00_JPRB, 0.67490E+00_JPRB, 0.65700E+00_JPRB, 0.66201E+00_JPRB, 0.66939E+00_JPRB ,&
&  0.67932E+00_JPRB, 0.68970E+00_JPRB, 0.69893E+00_JPRB, 0.70470E+00_JPRB, 0.69064E+00_JPRB, 0.68639E+00_JPRB, 0.67527E+00_JPRB ,&
&  0.63161E+00_JPRB, 0.58902E+00_JPRB, 0.56593E+00_JPRB, 0.55795E+00_JPRB, 0.55238E+00_JPRB, 0.55827E+00_JPRB, 0.56618E+00_JPRB ,&
&  0.57980E+00_JPRB, 0.59215E+00_JPRB, 0.59519E+00_JPRB, 0.59810E+00_JPRB, 0.59949E+00_JPRB, 0.59866E+00_JPRB, 0.59936E+00_JPRB ,&
&  0.59707E+00_JPRB, 0.59598E+00_JPRB, 0.60354E+00_JPRB, 0.60425E+00_JPRB, 0.59538E+00_JPRB, 0.59260E+00_JPRB, 0.59053E+00_JPRB ,&
&  0.58862E+00_JPRB, 0.58696E+00_JPRB, 0.58423E+00_JPRB, 0.59480E+00_JPRB, 0.62093E+00_JPRB, 0.65090E+00_JPRB, 0.67682E+00_JPRB ,&
&  0.69213E+00_JPRB, 0.70472E+00_JPRB, 0.70787E+00_JPRB, 0.70943E+00_JPRB, 0.70872E+00_JPRB, 0.69963E+00_JPRB, 0.69603E+00_JPRB ,&
&  0.68830E+00_JPRB, 0.66916E+00_JPRB, 0.64334E+00_JPRB, 0.60328E+00_JPRB, 0.56406E+00_JPRB, 0.54126E+00_JPRB, 0.54029E+00_JPRB ,&
&  0.55216E+00_JPRB, 0.56458E+00_JPRB, 0.57087E+00_JPRB, 0.57870E+00_JPRB, 0.59303E+00_JPRB, 0.61039E+00_JPRB, 0.62898E+00_JPRB ,&
&  0.64064E+00_JPRB, 0.64085E+00_JPRB, 0.63870E+00_JPRB, 0.64690E+00_JPRB, 0.65791E+00_JPRB, 0.67589E+00_JPRB, 0.69562E+00_JPRB ,&
&  0.71287E+00_JPRB, 0.71271E+00_JPRB, 0.71332E+00_JPRB, 0.71491E+00_JPRB, 0.73519E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.73957E+00_JPRB, 0.73880E+00_JPRB, 0.71061E+00_JPRB, 0.67902E+00_JPRB, 0.66046E+00_JPRB, 0.63989E+00_JPRB, 0.62333E+00_JPRB ,&
&  0.62168E+00_JPRB, 0.61869E+00_JPRB, 0.61603E+00_JPRB, 0.61277E+00_JPRB, 0.61053E+00_JPRB, 0.60723E+00_JPRB, 0.59508E+00_JPRB ,&
&  0.59060E+00_JPRB, 0.59699E+00_JPRB, 0.61920E+00_JPRB, 0.64414E+00_JPRB, 0.66417E+00_JPRB, 0.69044E+00_JPRB, 0.70187E+00_JPRB ,&
&  0.71402E+00_JPRB, 0.72678E+00_JPRB, 0.72892E+00_JPRB, 0.69562E+00_JPRB, 0.66825E+00_JPRB, 0.66930E+00_JPRB, 0.68051E+00_JPRB ,&
&  0.68091E+00_JPRB, 0.68389E+00_JPRB, 0.68672E+00_JPRB, 0.68309E+00_JPRB, 0.66213E+00_JPRB, 0.62374E+00_JPRB, 0.59321E+00_JPRB ,&
&  0.56750E+00_JPRB, 0.54746E+00_JPRB, 0.54081E+00_JPRB, 0.52870E+00_JPRB, 0.52645E+00_JPRB, 0.53652E+00_JPRB, 0.55047E+00_JPRB ,&
&  0.56506E+00_JPRB, 0.57184E+00_JPRB, 0.57887E+00_JPRB, 0.57510E+00_JPRB, 0.57078E+00_JPRB, 0.57370E+00_JPRB, 0.57368E+00_JPRB ,&
&  0.57327E+00_JPRB, 0.56417E+00_JPRB, 0.56140E+00_JPRB, 0.55451E+00_JPRB, 0.55065E+00_JPRB, 0.55105E+00_JPRB, 0.55411E+00_JPRB ,&
&  0.55570E+00_JPRB, 0.56169E+00_JPRB, 0.56370E+00_JPRB, 0.56615E+00_JPRB, 0.57571E+00_JPRB, 0.59239E+00_JPRB, 0.60903E+00_JPRB ,&
&  0.62154E+00_JPRB, 0.63145E+00_JPRB, 0.63905E+00_JPRB, 0.64498E+00_JPRB, 0.64494E+00_JPRB, 0.64031E+00_JPRB, 0.62647E+00_JPRB ,&
&  0.59618E+00_JPRB, 0.56362E+00_JPRB, 0.53871E+00_JPRB, 0.51877E+00_JPRB, 0.51517E+00_JPRB, 0.52607E+00_JPRB, 0.54356E+00_JPRB ,&
&  0.55679E+00_JPRB, 0.56036E+00_JPRB, 0.55385E+00_JPRB, 0.55162E+00_JPRB, 0.55586E+00_JPRB, 0.56494E+00_JPRB, 0.57204E+00_JPRB ,&
&  0.57108E+00_JPRB, 0.56801E+00_JPRB, 0.57220E+00_JPRB, 0.57978E+00_JPRB, 0.59443E+00_JPRB, 0.61271E+00_JPRB, 0.63232E+00_JPRB ,&
&  0.65172E+00_JPRB, 0.66672E+00_JPRB, 0.68601E+00_JPRB, 0.70559E+00_JPRB, 0.73111E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.72072E+00_JPRB, 0.69350E+00_JPRB, 0.66464E+00_JPRB, 0.63992E+00_JPRB, 0.62829E+00_JPRB, 0.61093E+00_JPRB, 0.59625E+00_JPRB ,&
&  0.59528E+00_JPRB, 0.60286E+00_JPRB, 0.59766E+00_JPRB, 0.59111E+00_JPRB, 0.58148E+00_JPRB, 0.57371E+00_JPRB, 0.56470E+00_JPRB ,&
&  0.56063E+00_JPRB, 0.57185E+00_JPRB, 0.59729E+00_JPRB, 0.60978E+00_JPRB, 0.61582E+00_JPRB, 0.64625E+00_JPRB, 0.67777E+00_JPRB ,&
&  0.68733E+00_JPRB, 0.70160E+00_JPRB, 0.70573E+00_JPRB, 0.68003E+00_JPRB, 0.66325E+00_JPRB, 0.65676E+00_JPRB, 0.66052E+00_JPRB ,&
&  0.65580E+00_JPRB, 0.65985E+00_JPRB, 0.65134E+00_JPRB, 0.58195E+00_JPRB, 0.58067E+00_JPRB, 0.55005E+00_JPRB, 0.53918E+00_JPRB ,&
&  0.53328E+00_JPRB, 0.53789E+00_JPRB, 0.53340E+00_JPRB, 0.52371E+00_JPRB, 0.51990E+00_JPRB, 0.52332E+00_JPRB, 0.52529E+00_JPRB ,&
&  0.52152E+00_JPRB, 0.51130E+00_JPRB, 0.51595E+00_JPRB, 0.52234E+00_JPRB, 0.52601E+00_JPRB, 0.53316E+00_JPRB, 0.54375E+00_JPRB ,&
&  0.53350E+00_JPRB, 0.52398E+00_JPRB, 0.51544E+00_JPRB, 0.51607E+00_JPRB, 0.52302E+00_JPRB, 0.53750E+00_JPRB, 0.54234E+00_JPRB ,&
&  0.54040E+00_JPRB, 0.53946E+00_JPRB, 0.54086E+00_JPRB, 0.54740E+00_JPRB, 0.56206E+00_JPRB, 0.57665E+00_JPRB, 0.57784E+00_JPRB ,&
&  0.57910E+00_JPRB, 0.58392E+00_JPRB, 0.58930E+00_JPRB, 0.59084E+00_JPRB, 0.58661E+00_JPRB, 0.57134E+00_JPRB, 0.55371E+00_JPRB ,&
&  0.53707E+00_JPRB, 0.51962E+00_JPRB, 0.50505E+00_JPRB, 0.50075E+00_JPRB, 0.51445E+00_JPRB, 0.54416E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56495E+00_JPRB, 0.56699E+00_JPRB, 0.57002E+00_JPRB, 0.57056E+00_JPRB, 0.57205E+00_JPRB, 0.56969E+00_JPRB, 0.56264E+00_JPRB ,&
&  0.55108E+00_JPRB, 0.54018E+00_JPRB, 0.53814E+00_JPRB, 0.54326E+00_JPRB, 0.54918E+00_JPRB, 0.56214E+00_JPRB, 0.57940E+00_JPRB ,&
&  0.60176E+00_JPRB, 0.64007E+00_JPRB, 0.67220E+00_JPRB, 0.69708E+00_JPRB, 0.71508E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.69740E+00_JPRB, 0.67144E+00_JPRB, 0.65410E+00_JPRB, 0.64058E+00_JPRB, 0.62239E+00_JPRB, 0.60847E+00_JPRB, 0.60282E+00_JPRB ,&
&  0.61091E+00_JPRB, 0.60926E+00_JPRB, 0.59916E+00_JPRB, 0.59070E+00_JPRB, 0.57463E+00_JPRB, 0.56033E+00_JPRB, 0.55451E+00_JPRB ,&
&  0.56286E+00_JPRB, 0.57886E+00_JPRB, 0.59860E+00_JPRB, 0.60637E+00_JPRB, 0.60548E+00_JPRB, 0.61393E+00_JPRB, 0.63329E+00_JPRB ,&
&  0.66207E+00_JPRB, 0.68220E+00_JPRB, 0.68901E+00_JPRB, 0.67539E+00_JPRB, 0.65750E+00_JPRB, 0.64918E+00_JPRB, 0.63489E+00_JPRB ,&
&  0.61813E+00_JPRB, 0.62115E+00_JPRB, 0.55407E+00_JPRB, 0.54642E+00_JPRB, 0.52332E+00_JPRB, 0.50862E+00_JPRB, 0.50457E+00_JPRB ,&
&  0.51726E+00_JPRB, 0.52077E+00_JPRB, 0.51196E+00_JPRB, 0.49946E+00_JPRB, 0.49119E+00_JPRB, 0.48028E+00_JPRB, 0.47035E+00_JPRB ,&
&  0.46371E+00_JPRB, 0.46651E+00_JPRB, 0.47557E+00_JPRB, 0.48491E+00_JPRB, 0.49581E+00_JPRB, 0.50629E+00_JPRB, 0.50286E+00_JPRB ,&
&  0.49117E+00_JPRB, 0.48925E+00_JPRB, 0.49657E+00_JPRB, 0.51172E+00_JPRB, 0.52826E+00_JPRB, 0.53593E+00_JPRB, 0.53928E+00_JPRB ,&
&  0.53083E+00_JPRB, 0.51373E+00_JPRB, 0.49912E+00_JPRB, 0.48712E+00_JPRB, 0.46046E+00_JPRB, 0.39768E+00_JPRB, 0.37512E+00_JPRB ,&
&  0.38635E+00_JPRB, 0.44473E+00_JPRB, 0.51425E+00_JPRB, 0.50654E+00_JPRB, 0.50300E+00_JPRB, 0.48858E+00_JPRB, 0.45157E+00_JPRB ,&
&  0.40812E+00_JPRB, 0.34693E+00_JPRB, 0.32626E+00_JPRB, 0.32304E+00_JPRB, 0.32259E+00_JPRB, 0.32909E+00_JPRB, 0.34165E+00_JPRB ,&
&  0.35815E+00_JPRB, 0.38426E+00_JPRB, 0.40127E+00_JPRB, 0.40751E+00_JPRB, 0.40553E+00_JPRB, 0.38931E+00_JPRB, 0.37521E+00_JPRB ,&
&  0.36616E+00_JPRB, 0.39152E+00_JPRB, 0.45093E+00_JPRB, 0.53341E+00_JPRB, 0.53374E+00_JPRB, 0.55951E+00_JPRB, 0.58500E+00_JPRB ,&
&  0.60918E+00_JPRB, 0.64932E+00_JPRB, 0.67538E+00_JPRB, 0.68100E+00_JPRB, 0.69935E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.65681E+00_JPRB, 0.64478E+00_JPRB, 0.63657E+00_JPRB, 0.63076E+00_JPRB, 0.62388E+00_JPRB, 0.62256E+00_JPRB, 0.62178E+00_JPRB ,&
&  0.61586E+00_JPRB, 0.60727E+00_JPRB, 0.60804E+00_JPRB, 0.59078E+00_JPRB, 0.56624E+00_JPRB, 0.54980E+00_JPRB, 0.54750E+00_JPRB ,&
&  0.55128E+00_JPRB, 0.55835E+00_JPRB, 0.56733E+00_JPRB, 0.57569E+00_JPRB, 0.58053E+00_JPRB, 0.59228E+00_JPRB, 0.61319E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.65109E+00_JPRB, 0.65511E+00_JPRB, 0.64685E+00_JPRB, 0.63441E+00_JPRB, 0.62160E+00_JPRB, 0.61049E+00_JPRB ,&
&  0.60082E+00_JPRB, 0.49915E+00_JPRB, 0.43202E+00_JPRB, 0.43902E+00_JPRB, 0.50680E+00_JPRB, 0.49049E+00_JPRB, 0.48894E+00_JPRB ,&
&  0.50131E+00_JPRB, 0.50095E+00_JPRB, 0.48742E+00_JPRB, 0.47023E+00_JPRB, 0.45426E+00_JPRB, 0.44285E+00_JPRB, 0.43476E+00_JPRB ,&
&  0.43120E+00_JPRB, 0.43807E+00_JPRB, 0.44599E+00_JPRB, 0.45824E+00_JPRB, 0.46106E+00_JPRB, 0.44804E+00_JPRB, 0.43238E+00_JPRB ,&
&  0.41740E+00_JPRB, 0.41203E+00_JPRB, 0.41455E+00_JPRB, 0.42147E+00_JPRB, 0.43056E+00_JPRB, 0.43767E+00_JPRB, 0.43550E+00_JPRB ,&
&  0.40707E+00_JPRB, 0.38390E+00_JPRB, 0.35568E+00_JPRB, 0.32905E+00_JPRB, 0.30594E+00_JPRB, 0.29047E+00_JPRB, 0.28247E+00_JPRB ,&
&  0.28072E+00_JPRB, 0.28869E+00_JPRB, 0.31048E+00_JPRB, 0.35965E+00_JPRB, 0.42499E+00_JPRB, 0.34576E+00_JPRB, 0.30686E+00_JPRB ,&
&  0.28561E+00_JPRB, 0.27240E+00_JPRB, 0.26733E+00_JPRB, 0.26819E+00_JPRB, 0.27258E+00_JPRB, 0.27898E+00_JPRB, 0.28403E+00_JPRB ,&
&  0.28807E+00_JPRB, 0.29146E+00_JPRB, 0.29305E+00_JPRB, 0.29373E+00_JPRB, 0.29508E+00_JPRB, 0.29616E+00_JPRB, 0.29504E+00_JPRB ,&
&  0.29371E+00_JPRB, 0.30036E+00_JPRB, 0.31851E+00_JPRB, 0.34699E+00_JPRB, 0.38007E+00_JPRB, 0.41609E+00_JPRB, 0.46848E+00_JPRB ,&
&  0.53392E+00_JPRB, 0.60770E+00_JPRB, 0.62633E+00_JPRB, 0.63984E+00_JPRB, 0.65053E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.59831E+00_JPRB, 0.60197E+00_JPRB, 0.60601E+00_JPRB, 0.60822E+00_JPRB, 0.60275E+00_JPRB, 0.59535E+00_JPRB, 0.59068E+00_JPRB ,&
&  0.59580E+00_JPRB, 0.59580E+00_JPRB, 0.57245E+00_JPRB, 0.55428E+00_JPRB, 0.54390E+00_JPRB, 0.54023E+00_JPRB, 0.53724E+00_JPRB ,&
&  0.53098E+00_JPRB, 0.52874E+00_JPRB, 0.52978E+00_JPRB, 0.54082E+00_JPRB, 0.55001E+00_JPRB, 0.55802E+00_JPRB, 0.56535E+00_JPRB ,&
&  0.56041E+00_JPRB, 0.57266E+00_JPRB, 0.59410E+00_JPRB, 0.59309E+00_JPRB, 0.59018E+00_JPRB, 0.58284E+00_JPRB, 0.56826E+00_JPRB ,&
&  0.51967E+00_JPRB, 0.45560E+00_JPRB, 0.39537E+00_JPRB, 0.40738E+00_JPRB, 0.48328E+00_JPRB, 0.49139E+00_JPRB, 0.47677E+00_JPRB ,&
&  0.48735E+00_JPRB, 0.48277E+00_JPRB, 0.46480E+00_JPRB, 0.45140E+00_JPRB, 0.43586E+00_JPRB, 0.42575E+00_JPRB, 0.41850E+00_JPRB ,&
&  0.41549E+00_JPRB, 0.41458E+00_JPRB, 0.39914E+00_JPRB, 0.34995E+00_JPRB, 0.32182E+00_JPRB, 0.30436E+00_JPRB, 0.28639E+00_JPRB ,&
&  0.27213E+00_JPRB, 0.26331E+00_JPRB, 0.26196E+00_JPRB, 0.26224E+00_JPRB, 0.26732E+00_JPRB, 0.26983E+00_JPRB, 0.27215E+00_JPRB ,&
&  0.27225E+00_JPRB, 0.27123E+00_JPRB, 0.26722E+00_JPRB, 0.26271E+00_JPRB, 0.25953E+00_JPRB, 0.25915E+00_JPRB, 0.25882E+00_JPRB ,&
&  0.26155E+00_JPRB, 0.26551E+00_JPRB, 0.28237E+00_JPRB, 0.30026E+00_JPRB, 0.29823E+00_JPRB, 0.28307E+00_JPRB, 0.26864E+00_JPRB ,&
&  0.25848E+00_JPRB, 0.25339E+00_JPRB, 0.25005E+00_JPRB, 0.24950E+00_JPRB, 0.25257E+00_JPRB, 0.25699E+00_JPRB, 0.25813E+00_JPRB ,&
&  0.25792E+00_JPRB, 0.25855E+00_JPRB, 0.25858E+00_JPRB, 0.25903E+00_JPRB, 0.25965E+00_JPRB, 0.26228E+00_JPRB, 0.26731E+00_JPRB ,&
&  0.27319E+00_JPRB, 0.28380E+00_JPRB, 0.29579E+00_JPRB, 0.31187E+00_JPRB, 0.32367E+00_JPRB, 0.33255E+00_JPRB, 0.34452E+00_JPRB ,&
&  0.36256E+00_JPRB, 0.40160E+00_JPRB, 0.47053E+00_JPRB, 0.59575E+00_JPRB, 0.59002E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.56607E+00_JPRB, 0.57750E+00_JPRB, 0.58973E+00_JPRB, 0.58155E+00_JPRB, 0.56805E+00_JPRB, 0.56046E+00_JPRB, 0.56026E+00_JPRB ,&
&  0.56003E+00_JPRB, 0.55708E+00_JPRB, 0.54497E+00_JPRB, 0.53172E+00_JPRB, 0.51356E+00_JPRB, 0.46564E+00_JPRB, 0.44789E+00_JPRB ,&
&  0.43037E+00_JPRB, 0.43297E+00_JPRB, 0.43727E+00_JPRB, 0.44362E+00_JPRB, 0.45728E+00_JPRB, 0.46796E+00_JPRB, 0.46967E+00_JPRB ,&
&  0.45943E+00_JPRB, 0.44102E+00_JPRB, 0.42321E+00_JPRB, 0.40952E+00_JPRB, 0.40912E+00_JPRB, 0.40903E+00_JPRB, 0.40193E+00_JPRB ,&
&  0.38846E+00_JPRB, 0.38259E+00_JPRB, 0.38246E+00_JPRB, 0.40229E+00_JPRB, 0.45376E+00_JPRB, 0.47261E+00_JPRB, 0.44800E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.44750E+00_JPRB, 0.44343E+00_JPRB, 0.44463E+00_JPRB, 0.43432E+00_JPRB, 0.41409E+00_JPRB, 0.38020E+00_JPRB ,&
&  0.33705E+00_JPRB, 0.31469E+00_JPRB, 0.29348E+00_JPRB, 0.27322E+00_JPRB, 0.25733E+00_JPRB, 0.24812E+00_JPRB, 0.24024E+00_JPRB ,&
&  0.23512E+00_JPRB, 0.23077E+00_JPRB, 0.22793E+00_JPRB, 0.22788E+00_JPRB, 0.22954E+00_JPRB, 0.23040E+00_JPRB, 0.23147E+00_JPRB ,&
&  0.23304E+00_JPRB, 0.23395E+00_JPRB, 0.23448E+00_JPRB, 0.23589E+00_JPRB, 0.23743E+00_JPRB, 0.24023E+00_JPRB, 0.24399E+00_JPRB ,&
&  0.24867E+00_JPRB, 0.25244E+00_JPRB, 0.25606E+00_JPRB, 0.26245E+00_JPRB, 0.26306E+00_JPRB, 0.26178E+00_JPRB, 0.25381E+00_JPRB ,&
&  0.24797E+00_JPRB, 0.24542E+00_JPRB, 0.24275E+00_JPRB, 0.24205E+00_JPRB, 0.24325E+00_JPRB, 0.24531E+00_JPRB, 0.24738E+00_JPRB ,&
&  0.24883E+00_JPRB, 0.24888E+00_JPRB, 0.24799E+00_JPRB, 0.24784E+00_JPRB, 0.24837E+00_JPRB, 0.24852E+00_JPRB, 0.25227E+00_JPRB ,&
&  0.26044E+00_JPRB, 0.27240E+00_JPRB, 0.28569E+00_JPRB, 0.30379E+00_JPRB, 0.32084E+00_JPRB, 0.33745E+00_JPRB, 0.36516E+00_JPRB ,&
&  0.40420E+00_JPRB, 0.51133E+00_JPRB, 0.58033E+00_JPRB, 0.52091E+00_JPRB, 0.54553E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.53833E+00_JPRB, 0.54596E+00_JPRB, 0.55682E+00_JPRB, 0.56149E+00_JPRB, 0.56950E+00_JPRB, 0.55789E+00_JPRB, 0.55070E+00_JPRB ,&
&  0.52043E+00_JPRB, 0.49872E+00_JPRB, 0.46782E+00_JPRB, 0.42958E+00_JPRB, 0.40378E+00_JPRB, 0.38389E+00_JPRB, 0.37656E+00_JPRB ,&
&  0.36759E+00_JPRB, 0.36213E+00_JPRB, 0.35641E+00_JPRB, 0.35406E+00_JPRB, 0.35226E+00_JPRB, 0.35303E+00_JPRB, 0.35248E+00_JPRB ,&
&  0.34866E+00_JPRB, 0.34300E+00_JPRB, 0.33626E+00_JPRB, 0.33897E+00_JPRB, 0.35183E+00_JPRB, 0.36665E+00_JPRB, 0.38421E+00_JPRB ,&
&  0.40454E+00_JPRB, 0.41512E+00_JPRB, 0.42538E+00_JPRB, 0.42939E+00_JPRB, 0.41393E+00_JPRB, 0.41653E+00_JPRB, 0.41981E+00_JPRB ,&
&  0.42781E+00_JPRB, 0.43416E+00_JPRB, 0.43094E+00_JPRB, 0.42607E+00_JPRB, 0.37486E+00_JPRB, 0.34210E+00_JPRB, 0.31341E+00_JPRB ,&
&  0.29197E+00_JPRB, 0.27825E+00_JPRB, 0.26634E+00_JPRB, 0.25615E+00_JPRB, 0.24556E+00_JPRB, 0.23880E+00_JPRB, 0.23373E+00_JPRB ,&
&  0.22966E+00_JPRB, 0.22794E+00_JPRB, 0.22632E+00_JPRB, 0.22589E+00_JPRB, 0.22640E+00_JPRB, 0.22584E+00_JPRB, 0.22586E+00_JPRB ,&
&  0.22574E+00_JPRB, 0.22514E+00_JPRB, 0.22533E+00_JPRB, 0.22669E+00_JPRB, 0.22875E+00_JPRB, 0.23212E+00_JPRB, 0.23315E+00_JPRB ,&
&  0.23418E+00_JPRB, 0.23492E+00_JPRB, 0.23500E+00_JPRB, 0.23701E+00_JPRB, 0.23811E+00_JPRB, 0.23827E+00_JPRB, 0.23668E+00_JPRB ,&
&  0.23500E+00_JPRB, 0.23371E+00_JPRB, 0.23368E+00_JPRB, 0.23426E+00_JPRB, 0.23584E+00_JPRB, 0.23827E+00_JPRB, 0.23878E+00_JPRB ,&
&  0.23969E+00_JPRB, 0.23924E+00_JPRB, 0.23877E+00_JPRB, 0.24081E+00_JPRB, 0.24317E+00_JPRB, 0.24575E+00_JPRB, 0.24919E+00_JPRB ,&
&  0.25090E+00_JPRB, 0.25822E+00_JPRB, 0.27174E+00_JPRB, 0.29264E+00_JPRB, 0.32585E+00_JPRB, 0.34253E+00_JPRB, 0.38195E+00_JPRB ,&
&  0.44146E+00_JPRB, 0.58211E+00_JPRB, 0.64651E+00_JPRB, 0.50675E+00_JPRB, 0.52278E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.47989E+00_JPRB, 0.49223E+00_JPRB, 0.50917E+00_JPRB, 0.51972E+00_JPRB, 0.53355E+00_JPRB, 0.53605E+00_JPRB, 0.52993E+00_JPRB ,&
&  0.51174E+00_JPRB, 0.48292E+00_JPRB, 0.46641E+00_JPRB, 0.44547E+00_JPRB, 0.42420E+00_JPRB, 0.40259E+00_JPRB, 0.39290E+00_JPRB ,&
&  0.38118E+00_JPRB, 0.36775E+00_JPRB, 0.35412E+00_JPRB, 0.34228E+00_JPRB, 0.33090E+00_JPRB, 0.32042E+00_JPRB, 0.31444E+00_JPRB ,&
&  0.30828E+00_JPRB, 0.30786E+00_JPRB, 0.30728E+00_JPRB, 0.31138E+00_JPRB, 0.32280E+00_JPRB, 0.34157E+00_JPRB, 0.36749E+00_JPRB ,&
&  0.40619E+00_JPRB, 0.41099E+00_JPRB, 0.39632E+00_JPRB, 0.39006E+00_JPRB, 0.38267E+00_JPRB, 0.37645E+00_JPRB, 0.36425E+00_JPRB ,&
&  0.37168E+00_JPRB, 0.38254E+00_JPRB, 0.39789E+00_JPRB, 0.38577E+00_JPRB, 0.33992E+00_JPRB, 0.32009E+00_JPRB, 0.29641E+00_JPRB ,&
&  0.28462E+00_JPRB, 0.27452E+00_JPRB, 0.26640E+00_JPRB, 0.25840E+00_JPRB, 0.24961E+00_JPRB, 0.24433E+00_JPRB, 0.23857E+00_JPRB ,&
&  0.23463E+00_JPRB, 0.23317E+00_JPRB, 0.23080E+00_JPRB, 0.23099E+00_JPRB, 0.23099E+00_JPRB, 0.23032E+00_JPRB, 0.22974E+00_JPRB ,&
&  0.22862E+00_JPRB, 0.22741E+00_JPRB, 0.22691E+00_JPRB, 0.22754E+00_JPRB, 0.22835E+00_JPRB, 0.22916E+00_JPRB, 0.22698E+00_JPRB ,&
&  0.22573E+00_JPRB, 0.22449E+00_JPRB, 0.22438E+00_JPRB, 0.22582E+00_JPRB, 0.22630E+00_JPRB, 0.22727E+00_JPRB, 0.22749E+00_JPRB ,&
&  0.22767E+00_JPRB, 0.22832E+00_JPRB, 0.22800E+00_JPRB, 0.22881E+00_JPRB, 0.22911E+00_JPRB, 0.23031E+00_JPRB, 0.23107E+00_JPRB ,&
&  0.23190E+00_JPRB, 0.23401E+00_JPRB, 0.23624E+00_JPRB, 0.23951E+00_JPRB, 0.24394E+00_JPRB, 0.24676E+00_JPRB, 0.25135E+00_JPRB ,&
&  0.25669E+00_JPRB, 0.26580E+00_JPRB, 0.28353E+00_JPRB, 0.30338E+00_JPRB, 0.32824E+00_JPRB, 0.33946E+00_JPRB, 0.34696E+00_JPRB ,&
&  0.35891E+00_JPRB, 0.39107E+00_JPRB, 0.44180E+00_JPRB, 0.46546E+00_JPRB, 0.47200E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.47450E+00_JPRB, 0.47653E+00_JPRB, 0.47799E+00_JPRB, 0.47514E+00_JPRB, 0.47457E+00_JPRB, 0.47470E+00_JPRB, 0.47198E+00_JPRB ,&
&  0.46393E+00_JPRB, 0.46172E+00_JPRB, 0.45666E+00_JPRB, 0.44912E+00_JPRB, 0.44161E+00_JPRB, 0.42932E+00_JPRB, 0.41284E+00_JPRB ,&
&  0.39080E+00_JPRB, 0.37125E+00_JPRB, 0.35669E+00_JPRB, 0.34296E+00_JPRB, 0.32957E+00_JPRB, 0.31649E+00_JPRB, 0.31162E+00_JPRB ,&
&  0.30718E+00_JPRB, 0.30542E+00_JPRB, 0.30503E+00_JPRB, 0.30838E+00_JPRB, 0.31590E+00_JPRB, 0.32529E+00_JPRB, 0.33564E+00_JPRB ,&
&  0.34711E+00_JPRB, 0.35788E+00_JPRB, 0.35906E+00_JPRB, 0.35153E+00_JPRB, 0.34461E+00_JPRB, 0.34035E+00_JPRB, 0.33704E+00_JPRB ,&
&  0.33314E+00_JPRB, 0.32494E+00_JPRB, 0.31541E+00_JPRB, 0.30529E+00_JPRB, 0.29510E+00_JPRB, 0.28766E+00_JPRB, 0.27893E+00_JPRB ,&
&  0.27251E+00_JPRB, 0.26655E+00_JPRB, 0.26010E+00_JPRB, 0.25453E+00_JPRB, 0.25074E+00_JPRB, 0.24763E+00_JPRB, 0.24445E+00_JPRB ,&
&  0.24296E+00_JPRB, 0.24148E+00_JPRB, 0.23986E+00_JPRB, 0.23832E+00_JPRB, 0.23634E+00_JPRB, 0.23487E+00_JPRB, 0.23354E+00_JPRB ,&
&  0.23255E+00_JPRB, 0.23166E+00_JPRB, 0.23112E+00_JPRB, 0.23055E+00_JPRB, 0.23006E+00_JPRB, 0.22898E+00_JPRB, 0.22773E+00_JPRB ,&
&  0.22698E+00_JPRB, 0.22681E+00_JPRB, 0.22603E+00_JPRB, 0.22564E+00_JPRB, 0.22531E+00_JPRB, 0.22452E+00_JPRB, 0.22505E+00_JPRB ,&
&  0.22632E+00_JPRB, 0.22774E+00_JPRB, 0.22796E+00_JPRB, 0.22904E+00_JPRB, 0.23032E+00_JPRB, 0.23195E+00_JPRB, 0.23460E+00_JPRB ,&
&  0.23659E+00_JPRB, 0.23867E+00_JPRB, 0.24131E+00_JPRB, 0.24356E+00_JPRB, 0.24714E+00_JPRB, 0.25156E+00_JPRB, 0.25702E+00_JPRB ,&
&  0.26449E+00_JPRB, 0.27350E+00_JPRB, 0.28636E+00_JPRB, 0.30214E+00_JPRB, 0.32501E+00_JPRB, 0.33303E+00_JPRB, 0.34052E+00_JPRB ,&
&  0.34630E+00_JPRB, 0.35756E+00_JPRB, 0.44022E+00_JPRB, 0.45480E+00_JPRB, 0.46976E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.34989E+00_JPRB, 0.35994E+00_JPRB, 0.36816E+00_JPRB, 0.37586E+00_JPRB, 0.37506E+00_JPRB, 0.43988E+00_JPRB, 0.41948E+00_JPRB ,&
&  0.40968E+00_JPRB, 0.38247E+00_JPRB, 0.38415E+00_JPRB, 0.38026E+00_JPRB, 0.37407E+00_JPRB, 0.36736E+00_JPRB, 0.36161E+00_JPRB ,&
&  0.35580E+00_JPRB, 0.34822E+00_JPRB, 0.34372E+00_JPRB, 0.33795E+00_JPRB, 0.33238E+00_JPRB, 0.32672E+00_JPRB, 0.32332E+00_JPRB ,&
&  0.32077E+00_JPRB, 0.31932E+00_JPRB, 0.31821E+00_JPRB, 0.31657E+00_JPRB, 0.31867E+00_JPRB, 0.31961E+00_JPRB, 0.31750E+00_JPRB ,&
&  0.31342E+00_JPRB, 0.30966E+00_JPRB, 0.30512E+00_JPRB, 0.30052E+00_JPRB, 0.29787E+00_JPRB, 0.29542E+00_JPRB, 0.29283E+00_JPRB ,&
&  0.28928E+00_JPRB, 0.28530E+00_JPRB, 0.28113E+00_JPRB, 0.27776E+00_JPRB, 0.27449E+00_JPRB, 0.27106E+00_JPRB, 0.26779E+00_JPRB ,&
&  0.26434E+00_JPRB, 0.26090E+00_JPRB, 0.25745E+00_JPRB, 0.25524E+00_JPRB, 0.25339E+00_JPRB, 0.25169E+00_JPRB, 0.25014E+00_JPRB ,&
&  0.24953E+00_JPRB, 0.24879E+00_JPRB, 0.24786E+00_JPRB, 0.24668E+00_JPRB, 0.24562E+00_JPRB, 0.24446E+00_JPRB, 0.24354E+00_JPRB ,&
&  0.24343E+00_JPRB, 0.24284E+00_JPRB, 0.24222E+00_JPRB, 0.24142E+00_JPRB, 0.24104E+00_JPRB, 0.24018E+00_JPRB, 0.23911E+00_JPRB ,&
&  0.23815E+00_JPRB, 0.23727E+00_JPRB, 0.23709E+00_JPRB, 0.23685E+00_JPRB, 0.23660E+00_JPRB, 0.23639E+00_JPRB, 0.23681E+00_JPRB ,&
&  0.23734E+00_JPRB, 0.23800E+00_JPRB, 0.23908E+00_JPRB, 0.24035E+00_JPRB, 0.24147E+00_JPRB, 0.24258E+00_JPRB, 0.24414E+00_JPRB ,&
&  0.24617E+00_JPRB, 0.24821E+00_JPRB, 0.25050E+00_JPRB, 0.25253E+00_JPRB, 0.25609E+00_JPRB, 0.25994E+00_JPRB, 0.26420E+00_JPRB ,&
&  0.26938E+00_JPRB, 0.27653E+00_JPRB, 0.28486E+00_JPRB, 0.29244E+00_JPRB, 0.30042E+00_JPRB, 0.30852E+00_JPRB, 0.31518E+00_JPRB ,&
&  0.32094E+00_JPRB, 0.32668E+00_JPRB, 0.33181E+00_JPRB, 0.33757E+00_JPRB, 0.34384E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.30789E+00_JPRB, 0.31081E+00_JPRB, 0.31317E+00_JPRB, 0.31477E+00_JPRB, 0.31470E+00_JPRB, 0.31616E+00_JPRB, 0.31632E+00_JPRB ,&
&  0.31624E+00_JPRB, 0.31593E+00_JPRB, 0.31667E+00_JPRB, 0.31746E+00_JPRB, 0.31796E+00_JPRB, 0.31826E+00_JPRB, 0.31831E+00_JPRB ,&
&  0.31778E+00_JPRB, 0.31675E+00_JPRB, 0.31245E+00_JPRB, 0.31006E+00_JPRB, 0.30861E+00_JPRB, 0.30786E+00_JPRB, 0.30744E+00_JPRB ,&
&  0.30651E+00_JPRB, 0.30544E+00_JPRB, 0.30377E+00_JPRB, 0.30356E+00_JPRB, 0.30274E+00_JPRB, 0.30166E+00_JPRB, 0.30012E+00_JPRB ,&
&  0.29860E+00_JPRB, 0.29741E+00_JPRB, 0.29596E+00_JPRB, 0.29447E+00_JPRB, 0.29269E+00_JPRB, 0.29049E+00_JPRB, 0.28767E+00_JPRB ,&
&  0.28478E+00_JPRB, 0.28185E+00_JPRB, 0.27942E+00_JPRB, 0.27714E+00_JPRB, 0.27517E+00_JPRB, 0.27356E+00_JPRB, 0.27238E+00_JPRB ,&
&  0.27106E+00_JPRB, 0.26955E+00_JPRB, 0.26802E+00_JPRB, 0.26673E+00_JPRB, 0.26547E+00_JPRB, 0.26436E+00_JPRB, 0.26379E+00_JPRB ,&
&  0.26345E+00_JPRB, 0.26305E+00_JPRB, 0.26255E+00_JPRB, 0.26209E+00_JPRB, 0.26152E+00_JPRB, 0.26099E+00_JPRB, 0.26046E+00_JPRB ,&
&  0.26001E+00_JPRB, 0.25976E+00_JPRB, 0.25937E+00_JPRB, 0.25897E+00_JPRB, 0.25872E+00_JPRB, 0.25830E+00_JPRB, 0.25783E+00_JPRB ,&
&  0.25730E+00_JPRB, 0.25702E+00_JPRB, 0.25670E+00_JPRB, 0.25630E+00_JPRB, 0.25592E+00_JPRB, 0.25566E+00_JPRB, 0.25567E+00_JPRB ,&
&  0.25563E+00_JPRB, 0.25553E+00_JPRB, 0.25559E+00_JPRB, 0.25637E+00_JPRB, 0.25720E+00_JPRB, 0.25820E+00_JPRB, 0.25933E+00_JPRB ,&
&  0.26078E+00_JPRB, 0.26231E+00_JPRB, 0.26403E+00_JPRB, 0.26568E+00_JPRB, 0.26790E+00_JPRB, 0.27011E+00_JPRB, 0.27232E+00_JPRB ,&
&  0.27444E+00_JPRB, 0.27665E+00_JPRB, 0.27890E+00_JPRB, 0.28120E+00_JPRB, 0.28427E+00_JPRB, 0.28678E+00_JPRB, 0.28960E+00_JPRB ,&
&  0.29231E+00_JPRB, 0.29535E+00_JPRB, 0.29840E+00_JPRB, 0.30109E+00_JPRB, 0.30381E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB, 0.27968E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB, 0.38962E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.39650E+00_JPRB, 0.39655E+00_JPRB, 0.39662E+00_JPRB, 0.39679E+00_JPRB, 0.39670E+00_JPRB, 0.39644E+00_JPRB, 0.39599E+00_JPRB ,&
&  0.39547E+00_JPRB, 0.39490E+00_JPRB, 0.39408E+00_JPRB, 0.39326E+00_JPRB, 0.39246E+00_JPRB, 0.39166E+00_JPRB, 0.39053E+00_JPRB ,&
&  0.38928E+00_JPRB, 0.38782E+00_JPRB, 0.38634E+00_JPRB, 0.38501E+00_JPRB, 0.38430E+00_JPRB, 0.38378E+00_JPRB, 0.38342E+00_JPRB ,&
&  0.38302E+00_JPRB, 0.38273E+00_JPRB, 0.38246E+00_JPRB, 0.38228E+00_JPRB, 0.38225E+00_JPRB, 0.38208E+00_JPRB, 0.38194E+00_JPRB ,&
&  0.38179E+00_JPRB, 0.38176E+00_JPRB, 0.38156E+00_JPRB, 0.38139E+00_JPRB, 0.38117E+00_JPRB, 0.38125E+00_JPRB, 0.38124E+00_JPRB ,&
&  0.38138E+00_JPRB, 0.38142E+00_JPRB, 0.38147E+00_JPRB, 0.38148E+00_JPRB, 0.38156E+00_JPRB, 0.38150E+00_JPRB, 0.38141E+00_JPRB ,&
&  0.38136E+00_JPRB, 0.38148E+00_JPRB, 0.38203E+00_JPRB, 0.38286E+00_JPRB, 0.38369E+00_JPRB, 0.38460E+00_JPRB, 0.38548E+00_JPRB ,&
&  0.38637E+00_JPRB, 0.38710E+00_JPRB, 0.38808E+00_JPRB, 0.38901E+00_JPRB, 0.39007E+00_JPRB, 0.39103E+00_JPRB, 0.39208E+00_JPRB ,&
&  0.39306E+00_JPRB, 0.39427E+00_JPRB, 0.39572E+00_JPRB, 0.39728E+00_JPRB, 0.39877E+00_JPRB, 0.40010E+00_JPRB, 0.40153E+00_JPRB ,&
&  0.40302E+00_JPRB, 0.40445E+00_JPRB, 0.40551E+00_JPRB, 0.40669E+00_JPRB, 0.40791E+00_JPRB, 0.40912E+00_JPRB, 0.40998E+00_JPRB ,&
&  0.41089E+00_JPRB, 0.41189E+00_JPRB, 0.41292E+00_JPRB, 0.41321E+00_JPRB, 0.41342E+00_JPRB, 0.41350E+00_JPRB, 0.41332E+00_JPRB ,&
&  0.41260E+00_JPRB, 0.41183E+00_JPRB, 0.41105E+00_JPRB, 0.41021E+00_JPRB, 0.40907E+00_JPRB, 0.40778E+00_JPRB, 0.40630E+00_JPRB ,&
&  0.40475E+00_JPRB, 0.40289E+00_JPRB, 0.40113E+00_JPRB, 0.39988E+00_JPRB, 0.39910E+00_JPRB, 0.39827E+00_JPRB, 0.39746E+00_JPRB ,&
&  0.39676E+00_JPRB, 0.39659E+00_JPRB, 0.39657E+00_JPRB, 0.39654E+00_JPRB, 0.39658E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.40068E+00_JPRB, 0.40094E+00_JPRB, 0.40139E+00_JPRB, 0.40159E+00_JPRB, 0.40093E+00_JPRB, 0.40032E+00_JPRB, 0.39968E+00_JPRB ,&
&  0.39936E+00_JPRB, 0.39906E+00_JPRB, 0.39828E+00_JPRB, 0.39719E+00_JPRB, 0.39612E+00_JPRB, 0.39503E+00_JPRB, 0.39383E+00_JPRB ,&
&  0.39275E+00_JPRB, 0.39160E+00_JPRB, 0.39016E+00_JPRB, 0.38747E+00_JPRB, 0.38520E+00_JPRB, 0.38374E+00_JPRB, 0.38349E+00_JPRB ,&
&  0.38293E+00_JPRB, 0.38210E+00_JPRB, 0.38060E+00_JPRB, 0.37888E+00_JPRB, 0.37702E+00_JPRB, 0.37522E+00_JPRB, 0.37334E+00_JPRB ,&
&  0.37173E+00_JPRB, 0.37026E+00_JPRB, 0.36917E+00_JPRB, 0.36804E+00_JPRB, 0.36830E+00_JPRB, 0.36859E+00_JPRB, 0.36887E+00_JPRB ,&
&  0.36914E+00_JPRB, 0.36916E+00_JPRB, 0.36997E+00_JPRB, 0.37080E+00_JPRB, 0.37165E+00_JPRB, 0.37284E+00_JPRB, 0.37337E+00_JPRB ,&
&  0.37321E+00_JPRB, 0.37327E+00_JPRB, 0.37499E+00_JPRB, 0.37622E+00_JPRB, 0.37711E+00_JPRB, 0.37784E+00_JPRB, 0.37959E+00_JPRB ,&
&  0.38128E+00_JPRB, 0.38290E+00_JPRB, 0.38473E+00_JPRB, 0.38730E+00_JPRB, 0.38953E+00_JPRB, 0.39191E+00_JPRB, 0.39460E+00_JPRB ,&
&  0.39723E+00_JPRB, 0.40006E+00_JPRB, 0.40297E+00_JPRB, 0.40586E+00_JPRB, 0.40850E+00_JPRB, 0.41077E+00_JPRB, 0.41342E+00_JPRB ,&
&  0.41624E+00_JPRB, 0.41948E+00_JPRB, 0.42145E+00_JPRB, 0.42358E+00_JPRB, 0.42596E+00_JPRB, 0.42933E+00_JPRB, 0.43061E+00_JPRB ,&
&  0.43180E+00_JPRB, 0.43289E+00_JPRB, 0.43412E+00_JPRB, 0.43463E+00_JPRB, 0.43514E+00_JPRB, 0.43566E+00_JPRB, 0.43602E+00_JPRB ,&
&  0.43588E+00_JPRB, 0.43510E+00_JPRB, 0.43370E+00_JPRB, 0.43171E+00_JPRB, 0.42934E+00_JPRB, 0.42700E+00_JPRB, 0.42417E+00_JPRB ,&
&  0.42166E+00_JPRB, 0.41843E+00_JPRB, 0.41534E+00_JPRB, 0.41330E+00_JPRB, 0.41211E+00_JPRB, 0.40974E+00_JPRB, 0.40714E+00_JPRB ,&
&  0.40486E+00_JPRB, 0.40314E+00_JPRB, 0.40227E+00_JPRB, 0.40147E+00_JPRB, 0.40107E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.40508E+00_JPRB, 0.40534E+00_JPRB, 0.40557E+00_JPRB, 0.40552E+00_JPRB, 0.40452E+00_JPRB, 0.40373E+00_JPRB, 0.40285E+00_JPRB ,&
&  0.40210E+00_JPRB, 0.40101E+00_JPRB, 0.40013E+00_JPRB, 0.39941E+00_JPRB, 0.39875E+00_JPRB, 0.39837E+00_JPRB, 0.39794E+00_JPRB ,&
&  0.39776E+00_JPRB, 0.39725E+00_JPRB, 0.39674E+00_JPRB, 0.39527E+00_JPRB, 0.39265E+00_JPRB, 0.38894E+00_JPRB, 0.38558E+00_JPRB ,&
&  0.38216E+00_JPRB, 0.37880E+00_JPRB, 0.37499E+00_JPRB, 0.37102E+00_JPRB, 0.36738E+00_JPRB, 0.34528E+00_JPRB, 0.33848E+00_JPRB ,&
&  0.33846E+00_JPRB, 0.33896E+00_JPRB, 0.34168E+00_JPRB, 0.34544E+00_JPRB, 0.34884E+00_JPRB, 0.34428E+00_JPRB, 0.34259E+00_JPRB ,&
&  0.32843E+00_JPRB, 0.32975E+00_JPRB, 0.33119E+00_JPRB, 0.33297E+00_JPRB, 0.33705E+00_JPRB, 0.34080E+00_JPRB, 0.34732E+00_JPRB ,&
&  0.36189E+00_JPRB, 0.36424E+00_JPRB, 0.36654E+00_JPRB, 0.36929E+00_JPRB, 0.37168E+00_JPRB, 0.37384E+00_JPRB, 0.37636E+00_JPRB ,&
&  0.37814E+00_JPRB, 0.38012E+00_JPRB, 0.38238E+00_JPRB, 0.38516E+00_JPRB, 0.38854E+00_JPRB, 0.39229E+00_JPRB, 0.39627E+00_JPRB ,&
&  0.40011E+00_JPRB, 0.40463E+00_JPRB, 0.40933E+00_JPRB, 0.41413E+00_JPRB, 0.41894E+00_JPRB, 0.42365E+00_JPRB, 0.42868E+00_JPRB ,&
&  0.43330E+00_JPRB, 0.43820E+00_JPRB, 0.44157E+00_JPRB, 0.44527E+00_JPRB, 0.44771E+00_JPRB, 0.45072E+00_JPRB, 0.45144E+00_JPRB ,&
&  0.45244E+00_JPRB, 0.45121E+00_JPRB, 0.45102E+00_JPRB, 0.45013E+00_JPRB, 0.44930E+00_JPRB, 0.44890E+00_JPRB, 0.44816E+00_JPRB ,&
&  0.44779E+00_JPRB, 0.44686E+00_JPRB, 0.44590E+00_JPRB, 0.44406E+00_JPRB, 0.44107E+00_JPRB, 0.43748E+00_JPRB, 0.43375E+00_JPRB ,&
&  0.43127E+00_JPRB, 0.42968E+00_JPRB, 0.42852E+00_JPRB, 0.42784E+00_JPRB, 0.42603E+00_JPRB, 0.42313E+00_JPRB, 0.41953E+00_JPRB ,&
&  0.41633E+00_JPRB, 0.41292E+00_JPRB, 0.40990E+00_JPRB, 0.40741E+00_JPRB, 0.40590E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.41101E+00_JPRB, 0.41179E+00_JPRB, 0.41230E+00_JPRB, 0.41263E+00_JPRB, 0.41191E+00_JPRB, 0.41128E+00_JPRB, 0.41043E+00_JPRB ,&
&  0.40949E+00_JPRB, 0.40856E+00_JPRB, 0.40756E+00_JPRB, 0.40680E+00_JPRB, 0.40580E+00_JPRB, 0.40515E+00_JPRB, 0.40391E+00_JPRB ,&
&  0.40341E+00_JPRB, 0.40207E+00_JPRB, 0.40044E+00_JPRB, 0.39697E+00_JPRB, 0.39357E+00_JPRB, 0.38953E+00_JPRB, 0.38332E+00_JPRB ,&
&  0.37664E+00_JPRB, 0.37063E+00_JPRB, 0.35333E+00_JPRB, 0.34810E+00_JPRB, 0.33879E+00_JPRB, 0.33353E+00_JPRB, 0.33040E+00_JPRB ,&
&  0.32689E+00_JPRB, 0.32473E+00_JPRB, 0.31839E+00_JPRB, 0.31260E+00_JPRB, 0.30768E+00_JPRB, 0.29964E+00_JPRB, 0.29504E+00_JPRB ,&
&  0.28983E+00_JPRB, 0.28739E+00_JPRB, 0.28826E+00_JPRB, 0.29134E+00_JPRB, 0.29897E+00_JPRB, 0.30825E+00_JPRB, 0.32059E+00_JPRB ,&
&  0.33731E+00_JPRB, 0.35210E+00_JPRB, 0.36641E+00_JPRB, 0.36713E+00_JPRB, 0.37088E+00_JPRB, 0.37423E+00_JPRB, 0.37636E+00_JPRB ,&
&  0.37328E+00_JPRB, 0.37258E+00_JPRB, 0.37946E+00_JPRB, 0.38228E+00_JPRB, 0.38588E+00_JPRB, 0.39018E+00_JPRB, 0.39470E+00_JPRB ,&
&  0.39967E+00_JPRB, 0.40532E+00_JPRB, 0.41141E+00_JPRB, 0.41851E+00_JPRB, 0.42722E+00_JPRB, 0.43547E+00_JPRB, 0.44227E+00_JPRB ,&
&  0.44800E+00_JPRB, 0.45521E+00_JPRB, 0.46016E+00_JPRB, 0.46487E+00_JPRB, 0.46930E+00_JPRB, 0.47608E+00_JPRB, 0.48143E+00_JPRB ,&
&  0.48481E+00_JPRB, 0.48322E+00_JPRB, 0.47893E+00_JPRB, 0.46991E+00_JPRB, 0.46548E+00_JPRB, 0.46921E+00_JPRB, 0.46734E+00_JPRB ,&
&  0.46641E+00_JPRB, 0.46391E+00_JPRB, 0.46181E+00_JPRB, 0.45869E+00_JPRB, 0.45540E+00_JPRB, 0.45067E+00_JPRB, 0.44542E+00_JPRB ,&
&  0.44137E+00_JPRB, 0.44005E+00_JPRB, 0.43903E+00_JPRB, 0.43814E+00_JPRB, 0.43511E+00_JPRB, 0.43185E+00_JPRB, 0.42827E+00_JPRB ,&
&  0.42463E+00_JPRB, 0.42066E+00_JPRB, 0.41663E+00_JPRB, 0.41344E+00_JPRB, 0.41152E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.42300E+00_JPRB, 0.42365E+00_JPRB, 0.42317E+00_JPRB, 0.42299E+00_JPRB, 0.42196E+00_JPRB, 0.42117E+00_JPRB, 0.41976E+00_JPRB ,&
&  0.41826E+00_JPRB, 0.41676E+00_JPRB, 0.41517E+00_JPRB, 0.41365E+00_JPRB, 0.41201E+00_JPRB, 0.41047E+00_JPRB, 0.40946E+00_JPRB ,&
&  0.40845E+00_JPRB, 0.40653E+00_JPRB, 0.40336E+00_JPRB, 0.39935E+00_JPRB, 0.39584E+00_JPRB, 0.39173E+00_JPRB, 0.38713E+00_JPRB ,&
&  0.38202E+00_JPRB, 0.37534E+00_JPRB, 0.36423E+00_JPRB, 0.35886E+00_JPRB, 0.35130E+00_JPRB, 0.34773E+00_JPRB, 0.34291E+00_JPRB ,&
&  0.34204E+00_JPRB, 0.31843E+00_JPRB, 0.31690E+00_JPRB, 0.30727E+00_JPRB, 0.30004E+00_JPRB, 0.29453E+00_JPRB, 0.28423E+00_JPRB ,&
&  0.27970E+00_JPRB, 0.27407E+00_JPRB, 0.27196E+00_JPRB, 0.27465E+00_JPRB, 0.27987E+00_JPRB, 0.29070E+00_JPRB, 0.30501E+00_JPRB ,&
&  0.32324E+00_JPRB, 0.35941E+00_JPRB, 0.36803E+00_JPRB, 0.37366E+00_JPRB, 0.37915E+00_JPRB, 0.38306E+00_JPRB, 0.37761E+00_JPRB ,&
&  0.37580E+00_JPRB, 0.37375E+00_JPRB, 0.37458E+00_JPRB, 0.37797E+00_JPRB, 0.38446E+00_JPRB, 0.38721E+00_JPRB, 0.39169E+00_JPRB ,&
&  0.39634E+00_JPRB, 0.40368E+00_JPRB, 0.41167E+00_JPRB, 0.42237E+00_JPRB, 0.43716E+00_JPRB, 0.45155E+00_JPRB, 0.46419E+00_JPRB ,&
&  0.47354E+00_JPRB, 0.47865E+00_JPRB, 0.47959E+00_JPRB, 0.48443E+00_JPRB, 0.49158E+00_JPRB, 0.50316E+00_JPRB, 0.51040E+00_JPRB ,&
&  0.51522E+00_JPRB, 0.51968E+00_JPRB, 0.51859E+00_JPRB, 0.51444E+00_JPRB, 0.50522E+00_JPRB, 0.49607E+00_JPRB, 0.49153E+00_JPRB ,&
&  0.49298E+00_JPRB, 0.48919E+00_JPRB, 0.48344E+00_JPRB, 0.47536E+00_JPRB, 0.47017E+00_JPRB, 0.46354E+00_JPRB, 0.45731E+00_JPRB ,&
&  0.45239E+00_JPRB, 0.45003E+00_JPRB, 0.44668E+00_JPRB, 0.44453E+00_JPRB, 0.44043E+00_JPRB, 0.43761E+00_JPRB, 0.43511E+00_JPRB ,&
&  0.43163E+00_JPRB, 0.42880E+00_JPRB, 0.42495E+00_JPRB, 0.42286E+00_JPRB, 0.42301E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.43662E+00_JPRB, 0.43660E+00_JPRB, 0.43628E+00_JPRB, 0.43565E+00_JPRB, 0.43469E+00_JPRB, 0.43435E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43352E+00_JPRB, 0.43284E+00_JPRB, 0.43174E+00_JPRB, 0.43058E+00_JPRB, 0.42915E+00_JPRB, 0.42816E+00_JPRB, 0.42706E+00_JPRB ,&
&  0.42488E+00_JPRB, 0.42138E+00_JPRB, 0.41651E+00_JPRB, 0.41162E+00_JPRB, 0.40692E+00_JPRB, 0.40376E+00_JPRB, 0.39919E+00_JPRB ,&
&  0.39454E+00_JPRB, 0.38836E+00_JPRB, 0.37896E+00_JPRB, 0.37176E+00_JPRB, 0.36918E+00_JPRB, 0.36362E+00_JPRB, 0.36372E+00_JPRB ,&
&  0.36119E+00_JPRB, 0.35722E+00_JPRB, 0.34685E+00_JPRB, 0.34307E+00_JPRB, 0.33653E+00_JPRB, 0.30276E+00_JPRB, 0.29460E+00_JPRB ,&
&  0.28364E+00_JPRB, 0.27207E+00_JPRB, 0.26770E+00_JPRB, 0.26652E+00_JPRB, 0.26932E+00_JPRB, 0.28048E+00_JPRB, 0.29592E+00_JPRB ,&
&  0.31410E+00_JPRB, 0.36120E+00_JPRB, 0.37397E+00_JPRB, 0.38471E+00_JPRB, 0.39292E+00_JPRB, 0.39737E+00_JPRB, 0.39605E+00_JPRB ,&
&  0.39452E+00_JPRB, 0.39399E+00_JPRB, 0.39696E+00_JPRB, 0.40192E+00_JPRB, 0.40434E+00_JPRB, 0.40147E+00_JPRB, 0.40189E+00_JPRB ,&
&  0.39848E+00_JPRB, 0.40292E+00_JPRB, 0.41276E+00_JPRB, 0.42649E+00_JPRB, 0.44378E+00_JPRB, 0.45591E+00_JPRB, 0.47275E+00_JPRB ,&
&  0.48738E+00_JPRB, 0.49615E+00_JPRB, 0.51085E+00_JPRB, 0.52545E+00_JPRB, 0.53437E+00_JPRB, 0.53977E+00_JPRB, 0.54040E+00_JPRB ,&
&  0.53833E+00_JPRB, 0.55105E+00_JPRB, 0.55808E+00_JPRB, 0.55422E+00_JPRB, 0.54637E+00_JPRB, 0.53295E+00_JPRB, 0.52039E+00_JPRB ,&
&  0.51442E+00_JPRB, 0.51625E+00_JPRB, 0.51056E+00_JPRB, 0.49614E+00_JPRB, 0.48503E+00_JPRB, 0.47576E+00_JPRB, 0.46830E+00_JPRB ,&
&  0.46281E+00_JPRB, 0.45970E+00_JPRB, 0.45510E+00_JPRB, 0.45169E+00_JPRB, 0.44838E+00_JPRB, 0.44582E+00_JPRB, 0.44428E+00_JPRB ,&
&  0.44269E+00_JPRB, 0.44228E+00_JPRB, 0.44086E+00_JPRB, 0.43873E+00_JPRB, 0.43821E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.45218E+00_JPRB, 0.45386E+00_JPRB, 0.45529E+00_JPRB, 0.45541E+00_JPRB, 0.45584E+00_JPRB, 0.45648E+00_JPRB, 0.45469E+00_JPRB ,&
&  0.45073E+00_JPRB, 0.44759E+00_JPRB, 0.44379E+00_JPRB, 0.44265E+00_JPRB, 0.44154E+00_JPRB, 0.44067E+00_JPRB, 0.43992E+00_JPRB ,&
&  0.43758E+00_JPRB, 0.42860E+00_JPRB, 0.42260E+00_JPRB, 0.41978E+00_JPRB, 0.41685E+00_JPRB, 0.41530E+00_JPRB, 0.41359E+00_JPRB ,&
&  0.40884E+00_JPRB, 0.40214E+00_JPRB, 0.39556E+00_JPRB, 0.38434E+00_JPRB, 0.37606E+00_JPRB, 0.37585E+00_JPRB, 0.37928E+00_JPRB ,&
&  0.37545E+00_JPRB, 0.36414E+00_JPRB, 0.36404E+00_JPRB, 0.36129E+00_JPRB, 0.34704E+00_JPRB, 0.34251E+00_JPRB, 0.31469E+00_JPRB ,&
&  0.29457E+00_JPRB, 0.27799E+00_JPRB, 0.27201E+00_JPRB, 0.26833E+00_JPRB, 0.27246E+00_JPRB, 0.28304E+00_JPRB, 0.30759E+00_JPRB ,&
&  0.33453E+00_JPRB, 0.36400E+00_JPRB, 0.37471E+00_JPRB, 0.38793E+00_JPRB, 0.39855E+00_JPRB, 0.41231E+00_JPRB, 0.42472E+00_JPRB ,&
&  0.43146E+00_JPRB, 0.43610E+00_JPRB, 0.44205E+00_JPRB, 0.44547E+00_JPRB, 0.45330E+00_JPRB, 0.45906E+00_JPRB, 0.46008E+00_JPRB ,&
&  0.46511E+00_JPRB, 0.46529E+00_JPRB, 0.47169E+00_JPRB, 0.47953E+00_JPRB, 0.48964E+00_JPRB, 0.49403E+00_JPRB, 0.49813E+00_JPRB ,&
&  0.50509E+00_JPRB, 0.52452E+00_JPRB, 0.55189E+00_JPRB, 0.58553E+00_JPRB, 0.59363E+00_JPRB, 0.58923E+00_JPRB, 0.57320E+00_JPRB ,&
&  0.55460E+00_JPRB, 0.57106E+00_JPRB, 0.65090E+00_JPRB, 0.63611E+00_JPRB, 0.62390E+00_JPRB, 0.60910E+00_JPRB, 0.57901E+00_JPRB ,&
&  0.54491E+00_JPRB, 0.53119E+00_JPRB, 0.51974E+00_JPRB, 0.50047E+00_JPRB, 0.48521E+00_JPRB, 0.47370E+00_JPRB, 0.46689E+00_JPRB ,&
&  0.46279E+00_JPRB, 0.46243E+00_JPRB, 0.45944E+00_JPRB, 0.45790E+00_JPRB, 0.45487E+00_JPRB, 0.45192E+00_JPRB, 0.45142E+00_JPRB ,&
&  0.45082E+00_JPRB, 0.45186E+00_JPRB, 0.45305E+00_JPRB, 0.45356E+00_JPRB, 0.45323E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.46814E+00_JPRB, 0.47464E+00_JPRB, 0.47969E+00_JPRB, 0.48818E+00_JPRB, 0.49571E+00_JPRB, 0.49544E+00_JPRB, 0.49006E+00_JPRB ,&
&  0.47841E+00_JPRB, 0.46425E+00_JPRB, 0.45497E+00_JPRB, 0.44781E+00_JPRB, 0.44394E+00_JPRB, 0.44205E+00_JPRB, 0.44020E+00_JPRB ,&
&  0.43866E+00_JPRB, 0.43676E+00_JPRB, 0.43173E+00_JPRB, 0.42425E+00_JPRB, 0.41938E+00_JPRB, 0.41827E+00_JPRB, 0.41756E+00_JPRB ,&
&  0.41539E+00_JPRB, 0.41081E+00_JPRB, 0.40464E+00_JPRB, 0.39394E+00_JPRB, 0.38501E+00_JPRB, 0.38656E+00_JPRB, 0.38213E+00_JPRB ,&
&  0.37742E+00_JPRB, 0.37462E+00_JPRB, 0.37238E+00_JPRB, 0.36868E+00_JPRB, 0.36441E+00_JPRB, 0.35382E+00_JPRB, 0.33202E+00_JPRB ,&
&  0.30860E+00_JPRB, 0.28974E+00_JPRB, 0.27838E+00_JPRB, 0.27519E+00_JPRB, 0.28184E+00_JPRB, 0.29960E+00_JPRB, 0.32615E+00_JPRB ,&
&  0.36046E+00_JPRB, 0.37579E+00_JPRB, 0.39146E+00_JPRB, 0.41000E+00_JPRB, 0.42606E+00_JPRB, 0.44460E+00_JPRB, 0.46071E+00_JPRB ,&
&  0.47599E+00_JPRB, 0.49595E+00_JPRB, 0.52048E+00_JPRB, 0.55023E+00_JPRB, 0.58736E+00_JPRB, 0.57057E+00_JPRB, 0.59362E+00_JPRB ,&
&  0.67539E+00_JPRB, 0.67666E+00_JPRB, 0.64780E+00_JPRB, 0.61432E+00_JPRB, 0.59161E+00_JPRB, 0.58201E+00_JPRB, 0.58120E+00_JPRB ,&
&  0.58449E+00_JPRB, 0.59478E+00_JPRB, 0.61416E+00_JPRB, 0.63764E+00_JPRB, 0.64299E+00_JPRB, 0.63647E+00_JPRB, 0.61295E+00_JPRB ,&
&  0.59249E+00_JPRB, 0.61146E+00_JPRB, 0.78467E+00_JPRB, 0.71009E+00_JPRB, 0.64555E+00_JPRB, 0.61412E+00_JPRB, 0.57684E+00_JPRB ,&
&  0.54945E+00_JPRB, 0.52925E+00_JPRB, 0.51013E+00_JPRB, 0.49620E+00_JPRB, 0.47768E+00_JPRB, 0.45939E+00_JPRB, 0.44372E+00_JPRB ,&
&  0.43956E+00_JPRB, 0.43981E+00_JPRB, 0.44142E+00_JPRB, 0.44333E+00_JPRB, 0.44291E+00_JPRB, 0.44437E+00_JPRB, 0.44712E+00_JPRB ,&
&  0.44622E+00_JPRB, 0.44726E+00_JPRB, 0.45218E+00_JPRB, 0.45737E+00_JPRB, 0.46247E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.45628E+00_JPRB, 0.47510E+00_JPRB, 0.50180E+00_JPRB, 0.51844E+00_JPRB, 0.52610E+00_JPRB, 0.51227E+00_JPRB, 0.49684E+00_JPRB ,&
&  0.48029E+00_JPRB, 0.45708E+00_JPRB, 0.44485E+00_JPRB, 0.43710E+00_JPRB, 0.43245E+00_JPRB, 0.42905E+00_JPRB, 0.42633E+00_JPRB ,&
&  0.42405E+00_JPRB, 0.42152E+00_JPRB, 0.42037E+00_JPRB, 0.42171E+00_JPRB, 0.42298E+00_JPRB, 0.42095E+00_JPRB, 0.41743E+00_JPRB ,&
&  0.41434E+00_JPRB, 0.41043E+00_JPRB, 0.40211E+00_JPRB, 0.39087E+00_JPRB, 0.38713E+00_JPRB, 0.39297E+00_JPRB, 0.39131E+00_JPRB ,&
&  0.38641E+00_JPRB, 0.38016E+00_JPRB, 0.37889E+00_JPRB, 0.37930E+00_JPRB, 0.37557E+00_JPRB, 0.36699E+00_JPRB, 0.34741E+00_JPRB ,&
&  0.32170E+00_JPRB, 0.30135E+00_JPRB, 0.29475E+00_JPRB, 0.30588E+00_JPRB, 0.32880E+00_JPRB, 0.36417E+00_JPRB, 0.37782E+00_JPRB ,&
&  0.38968E+00_JPRB, 0.40317E+00_JPRB, 0.41713E+00_JPRB, 0.43754E+00_JPRB, 0.46068E+00_JPRB, 0.48086E+00_JPRB, 0.49901E+00_JPRB ,&
&  0.52520E+00_JPRB, 0.55221E+00_JPRB, 0.58649E+00_JPRB, 0.55571E+00_JPRB, 0.57049E+00_JPRB, 0.61815E+00_JPRB, 0.64227E+00_JPRB ,&
&  0.65502E+00_JPRB, 0.67943E+00_JPRB, 0.69562E+00_JPRB, 0.71767E+00_JPRB, 0.70488E+00_JPRB, 0.68642E+00_JPRB, 0.66714E+00_JPRB ,&
&  0.66075E+00_JPRB, 0.65360E+00_JPRB, 0.65443E+00_JPRB, 0.67709E+00_JPRB, 0.69847E+00_JPRB, 0.69743E+00_JPRB, 0.67640E+00_JPRB ,&
&  0.64425E+00_JPRB, 0.59648E+00_JPRB, 0.55326E+00_JPRB, 0.52553E+00_JPRB, 0.52497E+00_JPRB, 0.53624E+00_JPRB, 0.53974E+00_JPRB ,&
&  0.53585E+00_JPRB, 0.53073E+00_JPRB, 0.51984E+00_JPRB, 0.51117E+00_JPRB, 0.47561E+00_JPRB, 0.45019E+00_JPRB, 0.43047E+00_JPRB ,&
&  0.42407E+00_JPRB, 0.42139E+00_JPRB, 0.42164E+00_JPRB, 0.42901E+00_JPRB, 0.42909E+00_JPRB, 0.43150E+00_JPRB, 0.43352E+00_JPRB ,&
&  0.43252E+00_JPRB, 0.43108E+00_JPRB, 0.43451E+00_JPRB, 0.43915E+00_JPRB, 0.44593E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.51194E+00_JPRB, 0.52665E+00_JPRB, 0.53917E+00_JPRB, 0.54653E+00_JPRB, 0.53765E+00_JPRB, 0.53216E+00_JPRB, 0.52711E+00_JPRB ,&
&  0.52758E+00_JPRB, 0.48287E+00_JPRB, 0.45600E+00_JPRB, 0.43488E+00_JPRB, 0.42002E+00_JPRB, 0.40610E+00_JPRB, 0.40156E+00_JPRB ,&
&  0.40238E+00_JPRB, 0.40507E+00_JPRB, 0.40707E+00_JPRB, 0.40862E+00_JPRB, 0.41021E+00_JPRB, 0.41051E+00_JPRB, 0.41179E+00_JPRB ,&
&  0.41421E+00_JPRB, 0.41407E+00_JPRB, 0.40616E+00_JPRB, 0.39942E+00_JPRB, 0.39786E+00_JPRB, 0.39946E+00_JPRB, 0.39614E+00_JPRB ,&
&  0.39190E+00_JPRB, 0.38749E+00_JPRB, 0.38574E+00_JPRB, 0.38677E+00_JPRB, 0.38454E+00_JPRB, 0.37679E+00_JPRB, 0.35491E+00_JPRB ,&
&  0.32717E+00_JPRB, 0.31787E+00_JPRB, 0.33280E+00_JPRB, 0.37744E+00_JPRB, 0.39237E+00_JPRB, 0.40242E+00_JPRB, 0.41547E+00_JPRB ,&
&  0.41214E+00_JPRB, 0.41432E+00_JPRB, 0.43258E+00_JPRB, 0.47585E+00_JPRB, 0.49639E+00_JPRB, 0.51425E+00_JPRB, 0.53390E+00_JPRB ,&
&  0.56682E+00_JPRB, 0.59152E+00_JPRB, 0.61935E+00_JPRB, 0.58677E+00_JPRB, 0.63985E+00_JPRB, 0.77392E+00_JPRB, 0.74693E+00_JPRB ,&
&  0.74426E+00_JPRB, 0.74820E+00_JPRB, 0.78041E+00_JPRB, 0.79584E+00_JPRB, 0.78586E+00_JPRB, 0.76629E+00_JPRB, 0.75425E+00_JPRB ,&
&  0.73692E+00_JPRB, 0.72788E+00_JPRB, 0.73259E+00_JPRB, 0.76634E+00_JPRB, 0.76249E+00_JPRB, 0.74733E+00_JPRB, 0.72564E+00_JPRB ,&
&  0.68968E+00_JPRB, 0.63990E+00_JPRB, 0.59412E+00_JPRB, 0.56046E+00_JPRB, 0.55195E+00_JPRB, 0.55374E+00_JPRB, 0.55052E+00_JPRB ,&
&  0.53962E+00_JPRB, 0.53838E+00_JPRB, 0.54085E+00_JPRB, 0.54040E+00_JPRB, 0.50393E+00_JPRB, 0.46266E+00_JPRB, 0.43219E+00_JPRB ,&
&  0.41089E+00_JPRB, 0.39895E+00_JPRB, 0.39014E+00_JPRB, 0.39526E+00_JPRB, 0.40564E+00_JPRB, 0.41762E+00_JPRB, 0.41892E+00_JPRB ,&
&  0.42227E+00_JPRB, 0.42571E+00_JPRB, 0.43792E+00_JPRB, 0.46295E+00_JPRB, 0.48904E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.53955E+00_JPRB, 0.55821E+00_JPRB, 0.57349E+00_JPRB, 0.56932E+00_JPRB, 0.55788E+00_JPRB, 0.54748E+00_JPRB, 0.53912E+00_JPRB ,&
&  0.54029E+00_JPRB, 0.47125E+00_JPRB, 0.44447E+00_JPRB, 0.42816E+00_JPRB, 0.41856E+00_JPRB, 0.40119E+00_JPRB, 0.38606E+00_JPRB ,&
&  0.38352E+00_JPRB, 0.39011E+00_JPRB, 0.40041E+00_JPRB, 0.40693E+00_JPRB, 0.41247E+00_JPRB, 0.41612E+00_JPRB, 0.41771E+00_JPRB ,&
&  0.41976E+00_JPRB, 0.42180E+00_JPRB, 0.41789E+00_JPRB, 0.41230E+00_JPRB, 0.40874E+00_JPRB, 0.40772E+00_JPRB, 0.40688E+00_JPRB ,&
&  0.40499E+00_JPRB, 0.40196E+00_JPRB, 0.39524E+00_JPRB, 0.39255E+00_JPRB, 0.39044E+00_JPRB, 0.39177E+00_JPRB, 0.38970E+00_JPRB ,&
&  0.34378E+00_JPRB, 0.34774E+00_JPRB, 0.38975E+00_JPRB, 0.39770E+00_JPRB, 0.40948E+00_JPRB, 0.41624E+00_JPRB, 0.42248E+00_JPRB ,&
&  0.43337E+00_JPRB, 0.45365E+00_JPRB, 0.48186E+00_JPRB, 0.51475E+00_JPRB, 0.54135E+00_JPRB, 0.56077E+00_JPRB, 0.57716E+00_JPRB ,&
&  0.61786E+00_JPRB, 0.56948E+00_JPRB, 0.55113E+00_JPRB, 0.59886E+00_JPRB, 0.71848E+00_JPRB, 0.78222E+00_JPRB, 0.83397E+00_JPRB ,&
&  0.87948E+00_JPRB, 0.87012E+00_JPRB, 0.86296E+00_JPRB, 0.86884E+00_JPRB, 0.86401E+00_JPRB, 0.85297E+00_JPRB, 0.86174E+00_JPRB ,&
&  0.85267E+00_JPRB, 0.85995E+00_JPRB, 0.87568E+00_JPRB, 0.88565E+00_JPRB, 0.84862E+00_JPRB, 0.82099E+00_JPRB, 0.78983E+00_JPRB ,&
&  0.75220E+00_JPRB, 0.70883E+00_JPRB, 0.65993E+00_JPRB, 0.62117E+00_JPRB, 0.59424E+00_JPRB, 0.59216E+00_JPRB, 0.58830E+00_JPRB ,&
&  0.57665E+00_JPRB, 0.57040E+00_JPRB, 0.56233E+00_JPRB, 0.55421E+00_JPRB, 0.52989E+00_JPRB, 0.50598E+00_JPRB, 0.49319E+00_JPRB ,&
&  0.45544E+00_JPRB, 0.43164E+00_JPRB, 0.40206E+00_JPRB, 0.40118E+00_JPRB, 0.41847E+00_JPRB, 0.43422E+00_JPRB, 0.44374E+00_JPRB ,&
&  0.46152E+00_JPRB, 0.47256E+00_JPRB, 0.47806E+00_JPRB, 0.49080E+00_JPRB, 0.51260E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.55463E+00_JPRB, 0.56615E+00_JPRB, 0.57264E+00_JPRB, 0.57035E+00_JPRB, 0.57197E+00_JPRB, 0.56644E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.60305E+00_JPRB, 0.55677E+00_JPRB, 0.59086E+00_JPRB, 0.48735E+00_JPRB, 0.43911E+00_JPRB, 0.41495E+00_JPRB, 0.39593E+00_JPRB ,&
&  0.39138E+00_JPRB, 0.39691E+00_JPRB, 0.40222E+00_JPRB, 0.40891E+00_JPRB, 0.42001E+00_JPRB, 0.42053E+00_JPRB, 0.42068E+00_JPRB ,&
&  0.42486E+00_JPRB, 0.42871E+00_JPRB, 0.43403E+00_JPRB, 0.42449E+00_JPRB, 0.41996E+00_JPRB, 0.41799E+00_JPRB, 0.42165E+00_JPRB ,&
&  0.40851E+00_JPRB, 0.40553E+00_JPRB, 0.41174E+00_JPRB, 0.40907E+00_JPRB, 0.40766E+00_JPRB, 0.40638E+00_JPRB, 0.40896E+00_JPRB ,&
&  0.41651E+00_JPRB, 0.39769E+00_JPRB, 0.41993E+00_JPRB, 0.42874E+00_JPRB, 0.43535E+00_JPRB, 0.44291E+00_JPRB, 0.44928E+00_JPRB ,&
&  0.46046E+00_JPRB, 0.47941E+00_JPRB, 0.50719E+00_JPRB, 0.53605E+00_JPRB, 0.56911E+00_JPRB, 0.59780E+00_JPRB, 0.61943E+00_JPRB ,&
&  0.66945E+00_JPRB, 0.58950E+00_JPRB, 0.68519E+00_JPRB, 0.73387E+00_JPRB, 0.84447E+00_JPRB, 0.98856E+00_JPRB, 0.10324E+01_JPRB ,&
&  0.11551E+01_JPRB, 0.11094E+01_JPRB, 0.10835E+01_JPRB, 0.10489E+01_JPRB, 0.10099E+01_JPRB, 0.98675E+00_JPRB, 0.97526E+00_JPRB ,&
&  0.10030E+01_JPRB, 0.10195E+01_JPRB, 0.10878E+01_JPRB, 0.10510E+01_JPRB, 0.96687E+00_JPRB, 0.91586E+00_JPRB, 0.86378E+00_JPRB ,&
&  0.82094E+00_JPRB, 0.79299E+00_JPRB, 0.76393E+00_JPRB, 0.69565E+00_JPRB, 0.67858E+00_JPRB, 0.66821E+00_JPRB, 0.62815E+00_JPRB ,&
&  0.60886E+00_JPRB, 0.57884E+00_JPRB, 0.54489E+00_JPRB, 0.55124E+00_JPRB, 0.52518E+00_JPRB, 0.50566E+00_JPRB, 0.50112E+00_JPRB ,&
&  0.48192E+00_JPRB, 0.44882E+00_JPRB, 0.45035E+00_JPRB, 0.45909E+00_JPRB, 0.45689E+00_JPRB, 0.45868E+00_JPRB, 0.48325E+00_JPRB ,&
&  0.49002E+00_JPRB, 0.50176E+00_JPRB, 0.52879E+00_JPRB, 0.53568E+00_JPRB, 0.54732E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.56333E+00_JPRB, 0.55985E+00_JPRB, 0.56007E+00_JPRB, 0.56391E+00_JPRB, 0.57871E+00_JPRB, 0.60435E+00_JPRB, 0.59352E+00_JPRB ,&
&  0.61718E+00_JPRB, 0.61905E+00_JPRB, 0.60815E+00_JPRB, 0.59527E+00_JPRB, 0.57000E+00_JPRB, 0.49702E+00_JPRB, 0.41302E+00_JPRB ,&
&  0.38855E+00_JPRB, 0.39318E+00_JPRB, 0.40440E+00_JPRB, 0.41000E+00_JPRB, 0.41763E+00_JPRB, 0.41764E+00_JPRB, 0.41939E+00_JPRB ,&
&  0.42536E+00_JPRB, 0.43155E+00_JPRB, 0.43652E+00_JPRB, 0.43652E+00_JPRB, 0.42892E+00_JPRB, 0.43233E+00_JPRB, 0.44339E+00_JPRB ,&
&  0.42292E+00_JPRB, 0.41169E+00_JPRB, 0.40832E+00_JPRB, 0.40060E+00_JPRB, 0.41542E+00_JPRB, 0.42206E+00_JPRB, 0.42948E+00_JPRB ,&
&  0.43883E+00_JPRB, 0.44811E+00_JPRB, 0.45132E+00_JPRB, 0.45800E+00_JPRB, 0.46829E+00_JPRB, 0.48599E+00_JPRB, 0.50401E+00_JPRB ,&
&  0.52511E+00_JPRB, 0.54670E+00_JPRB, 0.57398E+00_JPRB, 0.61009E+00_JPRB, 0.65595E+00_JPRB, 0.65873E+00_JPRB, 0.70950E+00_JPRB ,&
&  0.75612E+00_JPRB, 0.84666E+00_JPRB, 0.95372E+00_JPRB, 0.92663E+00_JPRB, 0.10487E+01_JPRB, 0.11578E+01_JPRB, 0.11617E+01_JPRB ,&
&  0.12169E+01_JPRB, 0.11679E+01_JPRB, 0.12026E+01_JPRB, 0.12355E+01_JPRB, 0.12582E+01_JPRB, 0.12397E+01_JPRB, 0.11814E+01_JPRB ,&
&  0.11666E+01_JPRB, 0.13070E+01_JPRB, 0.13176E+01_JPRB, 0.12374E+01_JPRB, 0.11142E+01_JPRB, 0.10389E+01_JPRB, 0.95750E+00_JPRB ,&
&  0.93072E+00_JPRB, 0.93449E+00_JPRB, 0.90127E+00_JPRB, 0.86559E+00_JPRB, 0.80560E+00_JPRB, 0.74225E+00_JPRB, 0.69232E+00_JPRB ,&
&  0.63599E+00_JPRB, 0.56219E+00_JPRB, 0.52779E+00_JPRB, 0.52133E+00_JPRB, 0.51742E+00_JPRB, 0.49128E+00_JPRB, 0.48549E+00_JPRB ,&
&  0.48661E+00_JPRB, 0.48168E+00_JPRB, 0.50163E+00_JPRB, 0.50173E+00_JPRB, 0.49944E+00_JPRB, 0.49453E+00_JPRB, 0.48166E+00_JPRB ,&
&  0.47516E+00_JPRB, 0.50876E+00_JPRB, 0.53447E+00_JPRB, 0.55115E+00_JPRB, 0.55919E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.56430E+00_JPRB, 0.55863E+00_JPRB, 0.54831E+00_JPRB, 0.55618E+00_JPRB, 0.58440E+00_JPRB, 0.61111E+00_JPRB, 0.62663E+00_JPRB ,&
&  0.62459E+00_JPRB, 0.62305E+00_JPRB, 0.61690E+00_JPRB, 0.60706E+00_JPRB, 0.58805E+00_JPRB, 0.56632E+00_JPRB, 0.50635E+00_JPRB ,&
&  0.43051E+00_JPRB, 0.41942E+00_JPRB, 0.40160E+00_JPRB, 0.40469E+00_JPRB, 0.41947E+00_JPRB, 0.41991E+00_JPRB, 0.42202E+00_JPRB ,&
&  0.42900E+00_JPRB, 0.43488E+00_JPRB, 0.43941E+00_JPRB, 0.43752E+00_JPRB, 0.43499E+00_JPRB, 0.45678E+00_JPRB, 0.45332E+00_JPRB ,&
&  0.42460E+00_JPRB, 0.40750E+00_JPRB, 0.40393E+00_JPRB, 0.40772E+00_JPRB, 0.42042E+00_JPRB, 0.43648E+00_JPRB, 0.44561E+00_JPRB ,&
&  0.45428E+00_JPRB, 0.46391E+00_JPRB, 0.47059E+00_JPRB, 0.48498E+00_JPRB, 0.50648E+00_JPRB, 0.53369E+00_JPRB, 0.55847E+00_JPRB ,&
&  0.57858E+00_JPRB, 0.60430E+00_JPRB, 0.64002E+00_JPRB, 0.68839E+00_JPRB, 0.72380E+00_JPRB, 0.76272E+00_JPRB, 0.83031E+00_JPRB ,&
&  0.89681E+00_JPRB, 0.99556E+00_JPRB, 0.10568E+01_JPRB, 0.11457E+01_JPRB, 0.12869E+01_JPRB, 0.13236E+01_JPRB, 0.13136E+01_JPRB ,&
&  0.13308E+01_JPRB, 0.12874E+01_JPRB, 0.14966E+01_JPRB, 0.15283E+01_JPRB, 0.14879E+01_JPRB, 0.14874E+01_JPRB, 0.13929E+01_JPRB ,&
&  0.13282E+01_JPRB, 0.13070E+01_JPRB, 0.12776E+01_JPRB, 0.12377E+01_JPRB, 0.11886E+01_JPRB, 0.11908E+01_JPRB, 0.11131E+01_JPRB ,&
&  0.11142E+01_JPRB, 0.10862E+01_JPRB, 0.92277E+00_JPRB, 0.80579E+00_JPRB, 0.70009E+00_JPRB, 0.70959E+00_JPRB, 0.69199E+00_JPRB ,&
&  0.63338E+00_JPRB, 0.54747E+00_JPRB, 0.54230E+00_JPRB, 0.53297E+00_JPRB, 0.53698E+00_JPRB, 0.54020E+00_JPRB, 0.52238E+00_JPRB ,&
&  0.52831E+00_JPRB, 0.55089E+00_JPRB, 0.55217E+00_JPRB, 0.56400E+00_JPRB, 0.55723E+00_JPRB, 0.53916E+00_JPRB, 0.48580E+00_JPRB ,&
&  0.47832E+00_JPRB, 0.50475E+00_JPRB, 0.52493E+00_JPRB, 0.54568E+00_JPRB, 0.55991E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.57863E+00_JPRB, 0.57651E+00_JPRB, 0.56772E+00_JPRB, 0.57352E+00_JPRB, 0.59919E+00_JPRB, 0.62571E+00_JPRB, 0.64054E+00_JPRB ,&
&  0.63902E+00_JPRB, 0.64022E+00_JPRB, 0.62974E+00_JPRB, 0.61839E+00_JPRB, 0.60506E+00_JPRB, 0.58241E+00_JPRB, 0.57931E+00_JPRB ,&
&  0.47482E+00_JPRB, 0.43799E+00_JPRB, 0.39441E+00_JPRB, 0.38923E+00_JPRB, 0.42530E+00_JPRB, 0.42661E+00_JPRB, 0.43209E+00_JPRB ,&
&  0.43372E+00_JPRB, 0.44254E+00_JPRB, 0.44142E+00_JPRB, 0.43992E+00_JPRB, 0.45205E+00_JPRB, 0.47385E+00_JPRB, 0.46111E+00_JPRB ,&
&  0.43857E+00_JPRB, 0.42258E+00_JPRB, 0.42144E+00_JPRB, 0.42360E+00_JPRB, 0.43280E+00_JPRB, 0.44925E+00_JPRB, 0.46228E+00_JPRB ,&
&  0.47451E+00_JPRB, 0.49923E+00_JPRB, 0.52114E+00_JPRB, 0.54827E+00_JPRB, 0.57991E+00_JPRB, 0.62250E+00_JPRB, 0.65641E+00_JPRB ,&
&  0.69177E+00_JPRB, 0.73476E+00_JPRB, 0.78209E+00_JPRB, 0.83889E+00_JPRB, 0.86774E+00_JPRB, 0.93714E+00_JPRB, 0.10143E+01_JPRB ,&
&  0.11612E+01_JPRB, 0.12140E+01_JPRB, 0.12435E+01_JPRB, 0.15060E+01_JPRB, 0.16320E+01_JPRB, 0.16239E+01_JPRB, 0.15128E+01_JPRB ,&
&  0.14625E+01_JPRB, 0.14210E+01_JPRB, 0.14587E+01_JPRB, 0.15185E+01_JPRB, 0.15124E+01_JPRB, 0.15546E+01_JPRB, 0.14777E+01_JPRB ,&
&  0.13320E+01_JPRB, 0.12666E+01_JPRB, 0.12325E+01_JPRB, 0.12447E+01_JPRB, 0.11915E+01_JPRB, 0.12105E+01_JPRB, 0.11668E+01_JPRB ,&
&  0.11815E+01_JPRB, 0.10242E+01_JPRB, 0.77714E+00_JPRB, 0.71246E+00_JPRB, 0.59025E+00_JPRB, 0.60358E+00_JPRB, 0.72854E+00_JPRB ,&
&  0.70553E+00_JPRB, 0.63755E+00_JPRB, 0.59112E+00_JPRB, 0.56585E+00_JPRB, 0.56326E+00_JPRB, 0.63396E+00_JPRB, 0.62773E+00_JPRB ,&
&  0.58026E+00_JPRB, 0.60847E+00_JPRB, 0.64230E+00_JPRB, 0.66386E+00_JPRB, 0.65372E+00_JPRB, 0.63205E+00_JPRB, 0.57951E+00_JPRB ,&
&  0.56979E+00_JPRB, 0.56343E+00_JPRB, 0.56617E+00_JPRB, 0.57352E+00_JPRB, 0.57758E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.61966E+00_JPRB, 0.61549E+00_JPRB, 0.60881E+00_JPRB, 0.61364E+00_JPRB, 0.63102E+00_JPRB, 0.65335E+00_JPRB, 0.66695E+00_JPRB ,&
&  0.66198E+00_JPRB, 0.65905E+00_JPRB, 0.64623E+00_JPRB, 0.63287E+00_JPRB, 0.61926E+00_JPRB, 0.59959E+00_JPRB, 0.58728E+00_JPRB ,&
&  0.58735E+00_JPRB, 0.49274E+00_JPRB, 0.42405E+00_JPRB, 0.38752E+00_JPRB, 0.40807E+00_JPRB, 0.41704E+00_JPRB, 0.43190E+00_JPRB ,&
&  0.43266E+00_JPRB, 0.45108E+00_JPRB, 0.45133E+00_JPRB, 0.45291E+00_JPRB, 0.48352E+00_JPRB, 0.49113E+00_JPRB, 0.48132E+00_JPRB ,&
&  0.47730E+00_JPRB, 0.46229E+00_JPRB, 0.45851E+00_JPRB, 0.46695E+00_JPRB, 0.48419E+00_JPRB, 0.47507E+00_JPRB, 0.50155E+00_JPRB ,&
&  0.53340E+00_JPRB, 0.56729E+00_JPRB, 0.59743E+00_JPRB, 0.64268E+00_JPRB, 0.69970E+00_JPRB, 0.74637E+00_JPRB, 0.79551E+00_JPRB ,&
&  0.84282E+00_JPRB, 0.87488E+00_JPRB, 0.88906E+00_JPRB, 0.93348E+00_JPRB, 0.10101E+01_JPRB, 0.11466E+01_JPRB, 0.11541E+01_JPRB ,&
&  0.12113E+01_JPRB, 0.12503E+01_JPRB, 0.12100E+01_JPRB, 0.15295E+01_JPRB, 0.17459E+01_JPRB, 0.17456E+01_JPRB, 0.16804E+01_JPRB ,&
&  0.16424E+01_JPRB, 0.16076E+01_JPRB, 0.16852E+01_JPRB, 0.16990E+01_JPRB, 0.16954E+01_JPRB, 0.16579E+01_JPRB, 0.15288E+01_JPRB ,&
&  0.13796E+01_JPRB, 0.12835E+01_JPRB, 0.12360E+01_JPRB, 0.11897E+01_JPRB, 0.11476E+01_JPRB, 0.11115E+01_JPRB, 0.11516E+01_JPRB ,&
&  0.12434E+01_JPRB, 0.73734E+00_JPRB, 0.60267E+00_JPRB, 0.67894E+00_JPRB, 0.58673E+00_JPRB, 0.60026E+00_JPRB, 0.66362E+00_JPRB ,&
&  0.65088E+00_JPRB, 0.63820E+00_JPRB, 0.65079E+00_JPRB, 0.63413E+00_JPRB, 0.65195E+00_JPRB, 0.78229E+00_JPRB, 0.79201E+00_JPRB ,&
&  0.74664E+00_JPRB, 0.71391E+00_JPRB, 0.76213E+00_JPRB, 0.76693E+00_JPRB, 0.76328E+00_JPRB, 0.74374E+00_JPRB, 0.70602E+00_JPRB ,&
&  0.67379E+00_JPRB, 0.65671E+00_JPRB, 0.64109E+00_JPRB, 0.63063E+00_JPRB, 0.62401E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.69682E+00_JPRB, 0.69300E+00_JPRB, 0.68944E+00_JPRB, 0.68575E+00_JPRB, 0.69050E+00_JPRB, 0.70648E+00_JPRB, 0.71349E+00_JPRB ,&
&  0.70378E+00_JPRB, 0.69381E+00_JPRB, 0.67441E+00_JPRB, 0.65660E+00_JPRB, 0.63984E+00_JPRB, 0.62404E+00_JPRB, 0.61063E+00_JPRB ,&
&  0.59675E+00_JPRB, 0.55075E+00_JPRB, 0.45493E+00_JPRB, 0.41008E+00_JPRB, 0.38490E+00_JPRB, 0.40100E+00_JPRB, 0.41960E+00_JPRB ,&
&  0.43241E+00_JPRB, 0.44833E+00_JPRB, 0.45866E+00_JPRB, 0.49678E+00_JPRB, 0.54869E+00_JPRB, 0.53237E+00_JPRB, 0.54668E+00_JPRB ,&
&  0.54817E+00_JPRB, 0.53340E+00_JPRB, 0.50923E+00_JPRB, 0.53148E+00_JPRB, 0.53094E+00_JPRB, 0.52964E+00_JPRB, 0.60161E+00_JPRB ,&
&  0.65350E+00_JPRB, 0.69202E+00_JPRB, 0.73278E+00_JPRB, 0.77603E+00_JPRB, 0.81803E+00_JPRB, 0.82927E+00_JPRB, 0.85988E+00_JPRB ,&
&  0.90093E+00_JPRB, 0.93492E+00_JPRB, 0.97404E+00_JPRB, 0.99558E+00_JPRB, 0.10508E+01_JPRB, 0.11235E+01_JPRB, 0.11202E+01_JPRB ,&
&  0.11626E+01_JPRB, 0.10521E+01_JPRB, 0.84504E+00_JPRB, 0.11367E+01_JPRB, 0.16884E+01_JPRB, 0.18091E+01_JPRB, 0.16883E+01_JPRB ,&
&  0.18004E+01_JPRB, 0.18272E+01_JPRB, 0.19631E+01_JPRB, 0.20677E+01_JPRB, 0.19899E+01_JPRB, 0.18589E+01_JPRB, 0.17037E+01_JPRB ,&
&  0.15225E+01_JPRB, 0.14005E+01_JPRB, 0.13485E+01_JPRB, 0.14242E+01_JPRB, 0.13869E+01_JPRB, 0.13960E+01_JPRB, 0.13977E+01_JPRB ,&
&  0.12287E+01_JPRB, 0.10272E+01_JPRB, 0.74602E+00_JPRB, 0.76485E+00_JPRB, 0.61781E+00_JPRB, 0.62217E+00_JPRB, 0.68038E+00_JPRB ,&
&  0.65893E+00_JPRB, 0.69306E+00_JPRB, 0.71444E+00_JPRB, 0.71184E+00_JPRB, 0.84364E+00_JPRB, 0.97170E+00_JPRB, 0.10758E+01_JPRB ,&
&  0.10502E+01_JPRB, 0.90459E+00_JPRB, 0.94640E+00_JPRB, 0.92139E+00_JPRB, 0.92448E+00_JPRB, 0.90128E+00_JPRB, 0.86843E+00_JPRB ,&
&  0.82726E+00_JPRB, 0.78323E+00_JPRB, 0.74691E+00_JPRB, 0.71987E+00_JPRB, 0.70462E+00_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.74791E+00_JPRB, 0.73231E+00_JPRB, 0.72250E+00_JPRB, 0.71862E+00_JPRB, 0.72363E+00_JPRB, 0.73668E+00_JPRB, 0.73633E+00_JPRB ,&
&  0.72732E+00_JPRB, 0.71126E+00_JPRB, 0.69019E+00_JPRB, 0.66951E+00_JPRB, 0.65173E+00_JPRB, 0.63858E+00_JPRB, 0.62745E+00_JPRB ,&
&  0.60826E+00_JPRB, 0.53355E+00_JPRB, 0.43420E+00_JPRB, 0.38868E+00_JPRB, 0.36968E+00_JPRB, 0.37590E+00_JPRB, 0.40514E+00_JPRB ,&
&  0.43425E+00_JPRB, 0.45855E+00_JPRB, 0.49560E+00_JPRB, 0.55693E+00_JPRB, 0.61064E+00_JPRB, 0.62169E+00_JPRB, 0.64119E+00_JPRB ,&
&  0.71038E+00_JPRB, 0.60048E+00_JPRB, 0.60382E+00_JPRB, 0.60946E+00_JPRB, 0.67302E+00_JPRB, 0.72284E+00_JPRB, 0.71760E+00_JPRB ,&
&  0.70838E+00_JPRB, 0.73670E+00_JPRB, 0.76949E+00_JPRB, 0.80938E+00_JPRB, 0.84167E+00_JPRB, 0.89161E+00_JPRB, 0.99306E+00_JPRB ,&
&  0.10776E+01_JPRB, 0.10974E+01_JPRB, 0.10476E+01_JPRB, 0.10193E+01_JPRB, 0.10920E+01_JPRB, 0.11463E+01_JPRB, 0.11275E+01_JPRB ,&
&  0.10211E+01_JPRB, 0.95963E+00_JPRB, 0.10987E+01_JPRB, 0.13559E+01_JPRB, 0.16730E+01_JPRB, 0.19180E+01_JPRB, 0.19549E+01_JPRB ,&
&  0.20325E+01_JPRB, 0.20184E+01_JPRB, 0.20805E+01_JPRB, 0.19043E+01_JPRB, 0.20483E+01_JPRB, 0.19837E+01_JPRB, 0.18968E+01_JPRB ,&
&  0.18507E+01_JPRB, 0.18306E+01_JPRB, 0.17702E+01_JPRB, 0.18232E+01_JPRB, 0.19526E+01_JPRB, 0.20020E+01_JPRB, 0.15905E+01_JPRB ,&
&  0.12507E+01_JPRB, 0.12544E+01_JPRB, 0.10820E+01_JPRB, 0.93611E+00_JPRB, 0.81232E+00_JPRB, 0.81425E+00_JPRB, 0.80605E+00_JPRB ,&
&  0.79645E+00_JPRB, 0.80322E+00_JPRB, 0.79272E+00_JPRB, 0.86453E+00_JPRB, 0.10047E+01_JPRB, 0.11627E+01_JPRB, 0.12776E+01_JPRB ,&
&  0.11753E+01_JPRB, 0.12373E+01_JPRB, 0.11954E+01_JPRB, 0.11390E+01_JPRB, 0.11522E+01_JPRB, 0.11283E+01_JPRB, 0.10697E+01_JPRB ,&
&  0.99015E+00_JPRB, 0.92282E+00_JPRB, 0.86528E+00_JPRB, 0.81476E+00_JPRB, 0.77497E+00_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.86461E+00_JPRB, 0.83379E+00_JPRB, 0.81161E+00_JPRB, 0.80025E+00_JPRB, 0.80159E+00_JPRB, 0.80954E+00_JPRB, 0.79766E+00_JPRB ,&
&  0.76654E+00_JPRB, 0.73603E+00_JPRB, 0.71292E+00_JPRB, 0.69394E+00_JPRB, 0.67311E+00_JPRB, 0.65554E+00_JPRB, 0.64321E+00_JPRB ,&
&  0.63289E+00_JPRB, 0.52032E+00_JPRB, 0.41496E+00_JPRB, 0.39341E+00_JPRB, 0.38212E+00_JPRB, 0.36874E+00_JPRB, 0.38984E+00_JPRB ,&
&  0.42962E+00_JPRB, 0.48163E+00_JPRB, 0.53864E+00_JPRB, 0.64105E+00_JPRB, 0.70059E+00_JPRB, 0.74131E+00_JPRB, 0.76465E+00_JPRB ,&
&  0.73569E+00_JPRB, 0.72991E+00_JPRB, 0.77243E+00_JPRB, 0.77575E+00_JPRB, 0.76568E+00_JPRB, 0.77259E+00_JPRB, 0.75734E+00_JPRB ,&
&  0.75739E+00_JPRB, 0.77881E+00_JPRB, 0.81988E+00_JPRB, 0.87964E+00_JPRB, 0.95479E+00_JPRB, 0.10500E+01_JPRB, 0.11707E+01_JPRB ,&
&  0.11845E+01_JPRB, 0.11378E+01_JPRB, 0.10222E+01_JPRB, 0.10197E+01_JPRB, 0.95797E+00_JPRB, 0.96816E+00_JPRB, 0.95738E+00_JPRB ,&
&  0.11743E+01_JPRB, 0.12337E+01_JPRB, 0.12014E+01_JPRB, 0.14679E+01_JPRB, 0.14723E+01_JPRB, 0.16547E+01_JPRB, 0.20763E+01_JPRB ,&
&  0.21756E+01_JPRB, 0.21828E+01_JPRB, 0.22306E+01_JPRB, 0.17850E+01_JPRB, 0.15817E+01_JPRB, 0.22022E+01_JPRB, 0.23499E+01_JPRB ,&
&  0.24568E+01_JPRB, 0.25912E+01_JPRB, 0.26036E+01_JPRB, 0.27354E+01_JPRB, 0.22884E+01_JPRB, 0.18505E+01_JPRB, 0.13648E+01_JPRB ,&
&  0.12629E+01_JPRB, 0.11549E+01_JPRB, 0.12336E+01_JPRB, 0.10971E+01_JPRB, 0.10629E+01_JPRB, 0.98420E+00_JPRB, 0.92469E+00_JPRB ,&
&  0.92911E+00_JPRB, 0.92212E+00_JPRB, 0.97412E+00_JPRB, 0.10882E+01_JPRB, 0.12531E+01_JPRB, 0.12696E+01_JPRB, 0.14472E+01_JPRB ,&
&  0.15064E+01_JPRB, 0.14611E+01_JPRB, 0.13022E+01_JPRB, 0.13023E+01_JPRB, 0.12546E+01_JPRB, 0.12129E+01_JPRB, 0.11627E+01_JPRB ,&
&  0.10998E+01_JPRB, 0.10394E+01_JPRB, 0.99087E+00_JPRB, 0.94398E+00_JPRB, 0.90212E+00_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.97093E+00_JPRB, 0.93936E+00_JPRB, 0.91367E+00_JPRB, 0.89905E+00_JPRB, 0.89031E+00_JPRB, 0.87434E+00_JPRB, 0.83671E+00_JPRB ,&
&  0.78928E+00_JPRB, 0.75159E+00_JPRB, 0.71767E+00_JPRB, 0.69444E+00_JPRB, 0.66856E+00_JPRB, 0.66003E+00_JPRB, 0.66139E+00_JPRB ,&
&  0.65275E+00_JPRB, 0.57427E+00_JPRB, 0.44585E+00_JPRB, 0.40709E+00_JPRB, 0.40523E+00_JPRB, 0.38744E+00_JPRB, 0.38915E+00_JPRB ,&
&  0.44993E+00_JPRB, 0.52819E+00_JPRB, 0.61193E+00_JPRB, 0.74752E+00_JPRB, 0.81458E+00_JPRB, 0.87885E+00_JPRB, 0.90186E+00_JPRB ,&
&  0.91485E+00_JPRB, 0.83596E+00_JPRB, 0.79004E+00_JPRB, 0.75125E+00_JPRB, 0.72219E+00_JPRB, 0.70678E+00_JPRB, 0.72543E+00_JPRB ,&
&  0.75190E+00_JPRB, 0.81273E+00_JPRB, 0.88486E+00_JPRB, 0.92348E+00_JPRB, 0.10030E+01_JPRB, 0.10749E+01_JPRB, 0.11416E+01_JPRB ,&
&  0.11449E+01_JPRB, 0.11079E+01_JPRB, 0.10765E+01_JPRB, 0.11372E+01_JPRB, 0.11339E+01_JPRB, 0.10576E+01_JPRB, 0.10745E+01_JPRB ,&
&  0.12018E+01_JPRB, 0.12130E+01_JPRB, 0.12651E+01_JPRB, 0.14444E+01_JPRB, 0.17074E+01_JPRB, 0.17942E+01_JPRB, 0.20706E+01_JPRB ,&
&  0.17250E+01_JPRB, 0.15232E+01_JPRB, 0.13270E+01_JPRB, 0.11788E+01_JPRB, 0.16030E+01_JPRB, 0.27372E+01_JPRB, 0.29457E+01_JPRB ,&
&  0.29869E+01_JPRB, 0.31932E+01_JPRB, 0.32714E+01_JPRB, 0.28341E+01_JPRB, 0.22629E+01_JPRB, 0.17743E+01_JPRB, 0.20902E+01_JPRB ,&
&  0.20543E+01_JPRB, 0.21825E+01_JPRB, 0.19246E+01_JPRB, 0.14856E+01_JPRB, 0.11788E+01_JPRB, 0.12893E+01_JPRB, 0.12381E+01_JPRB ,&
&  0.11970E+01_JPRB, 0.11941E+01_JPRB, 0.14803E+01_JPRB, 0.14991E+01_JPRB, 0.15024E+01_JPRB, 0.16274E+01_JPRB, 0.17599E+01_JPRB ,&
&  0.16181E+01_JPRB, 0.15603E+01_JPRB, 0.14923E+01_JPRB, 0.13994E+01_JPRB, 0.13321E+01_JPRB, 0.12639E+01_JPRB, 0.12009E+01_JPRB ,&
&  0.11539E+01_JPRB, 0.11128E+01_JPRB, 0.10682E+01_JPRB, 0.10357E+01_JPRB, 0.10024E+01_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.10471E+01_JPRB, 0.10163E+01_JPRB, 0.99168E+00_JPRB, 0.96667E+00_JPRB, 0.93942E+00_JPRB, 0.90109E+00_JPRB, 0.85179E+00_JPRB ,&
&  0.80213E+00_JPRB, 0.75855E+00_JPRB, 0.72546E+00_JPRB, 0.70113E+00_JPRB, 0.67502E+00_JPRB, 0.66348E+00_JPRB, 0.66407E+00_JPRB ,&
&  0.66170E+00_JPRB, 0.66113E+00_JPRB, 0.50961E+00_JPRB, 0.44327E+00_JPRB, 0.43483E+00_JPRB, 0.42869E+00_JPRB, 0.41828E+00_JPRB ,&
&  0.49963E+00_JPRB, 0.60021E+00_JPRB, 0.67276E+00_JPRB, 0.81507E+00_JPRB, 0.97511E+00_JPRB, 0.92874E+00_JPRB, 0.89676E+00_JPRB ,&
&  0.90900E+00_JPRB, 0.85537E+00_JPRB, 0.79589E+00_JPRB, 0.76353E+00_JPRB, 0.74483E+00_JPRB, 0.75297E+00_JPRB, 0.78254E+00_JPRB ,&
&  0.84392E+00_JPRB, 0.92480E+00_JPRB, 0.94666E+00_JPRB, 0.98792E+00_JPRB, 0.10341E+01_JPRB, 0.11208E+01_JPRB, 0.11898E+01_JPRB ,&
&  0.11540E+01_JPRB, 0.11190E+01_JPRB, 0.11503E+01_JPRB, 0.12772E+01_JPRB, 0.12870E+01_JPRB, 0.12207E+01_JPRB, 0.13557E+01_JPRB ,&
&  0.13526E+01_JPRB, 0.14760E+01_JPRB, 0.14465E+01_JPRB, 0.16107E+01_JPRB, 0.17395E+01_JPRB, 0.17346E+01_JPRB, 0.18496E+01_JPRB ,&
&  0.14613E+01_JPRB, 0.13052E+01_JPRB, 0.13798E+01_JPRB, 0.15858E+01_JPRB, 0.18754E+01_JPRB, 0.24147E+01_JPRB, 0.36918E+01_JPRB ,&
&  0.30986E+01_JPRB, 0.33033E+01_JPRB, 0.33004E+01_JPRB, 0.31020E+01_JPRB, 0.16779E+01_JPRB, 0.13407E+01_JPRB, 0.21618E+01_JPRB ,&
&  0.23875E+01_JPRB, 0.14903E+01_JPRB, 0.10204E+01_JPRB, 0.11911E+01_JPRB, 0.91369E+00_JPRB, 0.10781E+01_JPRB, 0.14143E+01_JPRB ,&
&  0.15137E+01_JPRB, 0.15939E+01_JPRB, 0.19832E+01_JPRB, 0.17829E+01_JPRB, 0.17199E+01_JPRB, 0.22764E+01_JPRB, 0.20087E+01_JPRB ,&
&  0.18532E+01_JPRB, 0.16504E+01_JPRB, 0.16223E+01_JPRB, 0.14432E+01_JPRB, 0.13354E+01_JPRB, 0.12716E+01_JPRB, 0.12360E+01_JPRB ,&
&  0.12167E+01_JPRB, 0.11934E+01_JPRB, 0.11645E+01_JPRB, 0.11259E+01_JPRB, 0.10823E+01_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.10789E+01_JPRB, 0.10558E+01_JPRB, 0.10366E+01_JPRB, 0.10104E+01_JPRB, 0.97236E+00_JPRB, 0.92183E+00_JPRB, 0.86753E+00_JPRB ,&
&  0.82463E+00_JPRB, 0.77640E+00_JPRB, 0.74029E+00_JPRB, 0.71315E+00_JPRB, 0.68840E+00_JPRB, 0.67253E+00_JPRB, 0.66573E+00_JPRB ,&
&  0.65423E+00_JPRB, 0.64594E+00_JPRB, 0.59459E+00_JPRB, 0.53294E+00_JPRB, 0.52012E+00_JPRB, 0.52206E+00_JPRB, 0.52844E+00_JPRB ,&
&  0.61530E+00_JPRB, 0.71246E+00_JPRB, 0.79754E+00_JPRB, 0.89957E+00_JPRB, 0.95828E+00_JPRB, 0.90331E+00_JPRB, 0.92570E+00_JPRB ,&
&  0.91907E+00_JPRB, 0.88505E+00_JPRB, 0.83569E+00_JPRB, 0.82381E+00_JPRB, 0.83177E+00_JPRB, 0.85294E+00_JPRB, 0.88618E+00_JPRB ,&
&  0.93458E+00_JPRB, 0.97974E+00_JPRB, 0.98164E+00_JPRB, 0.98391E+00_JPRB, 0.10604E+01_JPRB, 0.11692E+01_JPRB, 0.12260E+01_JPRB ,&
&  0.11916E+01_JPRB, 0.11982E+01_JPRB, 0.12406E+01_JPRB, 0.13619E+01_JPRB, 0.15379E+01_JPRB, 0.15398E+01_JPRB, 0.13149E+01_JPRB ,&
&  0.11706E+01_JPRB, 0.12695E+01_JPRB, 0.16023E+01_JPRB, 0.15702E+01_JPRB, 0.16002E+01_JPRB, 0.16567E+01_JPRB, 0.17043E+01_JPRB ,&
&  0.16609E+01_JPRB, 0.16954E+01_JPRB, 0.17767E+01_JPRB, 0.20990E+01_JPRB, 0.24757E+01_JPRB, 0.25527E+01_JPRB, 0.36454E+01_JPRB ,&
&  0.36599E+01_JPRB, 0.33220E+01_JPRB, 0.25065E+01_JPRB, 0.18543E+01_JPRB, 0.18438E+01_JPRB, 0.14752E+01_JPRB, 0.99500E+00_JPRB ,&
&  0.94966E+00_JPRB, 0.89792E+00_JPRB, 0.78841E+00_JPRB, 0.77470E+00_JPRB, 0.72829E+00_JPRB, 0.95125E+00_JPRB, 0.14030E+01_JPRB ,&
&  0.19225E+01_JPRB, 0.22373E+01_JPRB, 0.23040E+01_JPRB, 0.21136E+01_JPRB, 0.19335E+01_JPRB, 0.24991E+01_JPRB, 0.22580E+01_JPRB ,&
&  0.17579E+01_JPRB, 0.16245E+01_JPRB, 0.15496E+01_JPRB, 0.14252E+01_JPRB, 0.13596E+01_JPRB, 0.13056E+01_JPRB, 0.12688E+01_JPRB ,&
&  0.12364E+01_JPRB, 0.12099E+01_JPRB, 0.11809E+01_JPRB, 0.11432E+01_JPRB, 0.11096E+01_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.11001E+01_JPRB, 0.10827E+01_JPRB, 0.10600E+01_JPRB, 0.10295E+01_JPRB, 0.98614E+00_JPRB, 0.93850E+00_JPRB, 0.89344E+00_JPRB ,&
&  0.86169E+00_JPRB, 0.81713E+00_JPRB, 0.77462E+00_JPRB, 0.73991E+00_JPRB, 0.71024E+00_JPRB, 0.68803E+00_JPRB, 0.67747E+00_JPRB ,&
&  0.67068E+00_JPRB, 0.65719E+00_JPRB, 0.66398E+00_JPRB, 0.64077E+00_JPRB, 0.68629E+00_JPRB, 0.72831E+00_JPRB, 0.77155E+00_JPRB ,&
&  0.85143E+00_JPRB, 0.93158E+00_JPRB, 0.10452E+01_JPRB, 0.10839E+01_JPRB, 0.11070E+01_JPRB, 0.10828E+01_JPRB, 0.10409E+01_JPRB ,&
&  0.10035E+01_JPRB, 0.99881E+00_JPRB, 0.97166E+00_JPRB, 0.95806E+00_JPRB, 0.94579E+00_JPRB, 0.94266E+00_JPRB, 0.95567E+00_JPRB ,&
&  0.98223E+00_JPRB, 0.10164E+01_JPRB, 0.98780E+00_JPRB, 0.10145E+01_JPRB, 0.11043E+01_JPRB, 0.12000E+01_JPRB, 0.12652E+01_JPRB ,&
&  0.12840E+01_JPRB, 0.12962E+01_JPRB, 0.13553E+01_JPRB, 0.14632E+01_JPRB, 0.13808E+01_JPRB, 0.11659E+01_JPRB, 0.12748E+01_JPRB ,&
&  0.13682E+01_JPRB, 0.15689E+01_JPRB, 0.15400E+01_JPRB, 0.17825E+01_JPRB, 0.17276E+01_JPRB, 0.16318E+01_JPRB, 0.17764E+01_JPRB ,&
&  0.19513E+01_JPRB, 0.24910E+01_JPRB, 0.21878E+01_JPRB, 0.22131E+01_JPRB, 0.32436E+01_JPRB, 0.30939E+01_JPRB, 0.33471E+01_JPRB ,&
&  0.36224E+01_JPRB, 0.29671E+01_JPRB, 0.23692E+01_JPRB, 0.20629E+01_JPRB, 0.27582E+01_JPRB, 0.26611E+01_JPRB, 0.12950E+01_JPRB ,&
&  0.10401E+01_JPRB, 0.90871E+00_JPRB, 0.82030E+00_JPRB, 0.74640E+00_JPRB, 0.71109E+00_JPRB, 0.10006E+01_JPRB, 0.18507E+01_JPRB ,&
&  0.19555E+01_JPRB, 0.28436E+01_JPRB, 0.28535E+01_JPRB, 0.25707E+01_JPRB, 0.22348E+01_JPRB, 0.24273E+01_JPRB, 0.20918E+01_JPRB ,&
&  0.17746E+01_JPRB, 0.16729E+01_JPRB, 0.15165E+01_JPRB, 0.13993E+01_JPRB, 0.13002E+01_JPRB, 0.12443E+01_JPRB, 0.12221E+01_JPRB ,&
&  0.12018E+01_JPRB, 0.11818E+01_JPRB, 0.11642E+01_JPRB, 0.11420E+01_JPRB, 0.11210E+01_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.10891E+01_JPRB, 0.10625E+01_JPRB, 0.10411E+01_JPRB, 0.10121E+01_JPRB, 0.98243E+00_JPRB, 0.95197E+00_JPRB, 0.91727E+00_JPRB ,&
&  0.89109E+00_JPRB, 0.84853E+00_JPRB, 0.80523E+00_JPRB, 0.76396E+00_JPRB, 0.72846E+00_JPRB, 0.70206E+00_JPRB, 0.69063E+00_JPRB ,&
&  0.68263E+00_JPRB, 0.68126E+00_JPRB, 0.69021E+00_JPRB, 0.75532E+00_JPRB, 0.85270E+00_JPRB, 0.90796E+00_JPRB, 0.10377E+01_JPRB ,&
&  0.11968E+01_JPRB, 0.13109E+01_JPRB, 0.14469E+01_JPRB, 0.14494E+01_JPRB, 0.14093E+01_JPRB, 0.13341E+01_JPRB, 0.12577E+01_JPRB ,&
&  0.11990E+01_JPRB, 0.11898E+01_JPRB, 0.11343E+01_JPRB, 0.10846E+01_JPRB, 0.10435E+01_JPRB, 0.10243E+01_JPRB, 0.10246E+01_JPRB ,&
&  0.10602E+01_JPRB, 0.10331E+01_JPRB, 0.10113E+01_JPRB, 0.10497E+01_JPRB, 0.11314E+01_JPRB, 0.12427E+01_JPRB, 0.13219E+01_JPRB ,&
&  0.13447E+01_JPRB, 0.13411E+01_JPRB, 0.13909E+01_JPRB, 0.14726E+01_JPRB, 0.12168E+01_JPRB, 0.13840E+01_JPRB, 0.13869E+01_JPRB ,&
&  0.14041E+01_JPRB, 0.14720E+01_JPRB, 0.14521E+01_JPRB, 0.16741E+01_JPRB, 0.17449E+01_JPRB, 0.16527E+01_JPRB, 0.17673E+01_JPRB ,&
&  0.22859E+01_JPRB, 0.22164E+01_JPRB, 0.21453E+01_JPRB, 0.25446E+01_JPRB, 0.32312E+01_JPRB, 0.46023E+01_JPRB, 0.29703E+01_JPRB ,&
&  0.24992E+01_JPRB, 0.27655E+01_JPRB, 0.23839E+01_JPRB, 0.21435E+01_JPRB, 0.26408E+01_JPRB, 0.30268E+01_JPRB, 0.25138E+01_JPRB ,&
&  0.11830E+01_JPRB, 0.88002E+00_JPRB, 0.91343E+00_JPRB, 0.94000E+00_JPRB, 0.90030E+00_JPRB, 0.12233E+01_JPRB, 0.28145E+01_JPRB ,&
&  0.24396E+01_JPRB, 0.31579E+01_JPRB, 0.30054E+01_JPRB, 0.26147E+01_JPRB, 0.23049E+01_JPRB, 0.22697E+01_JPRB, 0.20729E+01_JPRB ,&
&  0.17095E+01_JPRB, 0.15282E+01_JPRB, 0.13783E+01_JPRB, 0.12798E+01_JPRB, 0.12299E+01_JPRB, 0.11948E+01_JPRB, 0.11761E+01_JPRB ,&
&  0.11589E+01_JPRB, 0.11441E+01_JPRB, 0.11357E+01_JPRB, 0.11286E+01_JPRB, 0.11140E+01_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.10494E+01_JPRB, 0.10270E+01_JPRB, 0.10059E+01_JPRB, 0.99305E+00_JPRB, 0.97805E+00_JPRB, 0.94487E+00_JPRB, 0.91292E+00_JPRB ,&
&  0.88562E+00_JPRB, 0.84399E+00_JPRB, 0.80755E+00_JPRB, 0.76716E+00_JPRB, 0.73370E+00_JPRB, 0.71060E+00_JPRB, 0.69919E+00_JPRB ,&
&  0.70133E+00_JPRB, 0.70232E+00_JPRB, 0.71370E+00_JPRB, 0.80997E+00_JPRB, 0.97193E+00_JPRB, 0.10224E+01_JPRB, 0.11768E+01_JPRB ,&
&  0.17244E+01_JPRB, 0.19368E+01_JPRB, 0.18401E+01_JPRB, 0.17813E+01_JPRB, 0.17384E+01_JPRB, 0.16757E+01_JPRB, 0.15725E+01_JPRB ,&
&  0.14325E+01_JPRB, 0.13390E+01_JPRB, 0.12212E+01_JPRB, 0.11432E+01_JPRB, 0.10958E+01_JPRB, 0.10752E+01_JPRB, 0.10826E+01_JPRB ,&
&  0.10726E+01_JPRB, 0.10510E+01_JPRB, 0.10358E+01_JPRB, 0.10669E+01_JPRB, 0.11616E+01_JPRB, 0.12670E+01_JPRB, 0.13522E+01_JPRB ,&
&  0.13690E+01_JPRB, 0.13633E+01_JPRB, 0.13759E+01_JPRB, 0.13505E+01_JPRB, 0.13699E+01_JPRB, 0.13961E+01_JPRB, 0.14396E+01_JPRB ,&
&  0.14358E+01_JPRB, 0.15249E+01_JPRB, 0.16231E+01_JPRB, 0.16553E+01_JPRB, 0.16846E+01_JPRB, 0.17218E+01_JPRB, 0.18768E+01_JPRB ,&
&  0.22270E+01_JPRB, 0.22773E+01_JPRB, 0.24128E+01_JPRB, 0.30756E+01_JPRB, 0.35006E+01_JPRB, 0.39722E+01_JPRB, 0.36201E+01_JPRB ,&
&  0.29228E+01_JPRB, 0.22049E+01_JPRB, 0.20185E+01_JPRB, 0.22030E+01_JPRB, 0.24800E+01_JPRB, 0.26150E+01_JPRB, 0.35611E+01_JPRB ,&
&  0.35083E+01_JPRB, 0.23791E+01_JPRB, 0.20016E+01_JPRB, 0.25280E+01_JPRB, 0.21493E+01_JPRB, 0.19297E+01_JPRB, 0.23616E+01_JPRB ,&
&  0.26839E+01_JPRB, 0.31643E+01_JPRB, 0.29088E+01_JPRB, 0.24208E+01_JPRB, 0.23121E+01_JPRB, 0.21307E+01_JPRB, 0.18672E+01_JPRB ,&
&  0.15697E+01_JPRB, 0.14351E+01_JPRB, 0.13403E+01_JPRB, 0.12511E+01_JPRB, 0.11839E+01_JPRB, 0.11491E+01_JPRB, 0.11306E+01_JPRB ,&
&  0.11099E+01_JPRB, 0.11009E+01_JPRB, 0.10980E+01_JPRB, 0.10914E+01_JPRB, 0.10702E+01_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.99936E+00_JPRB, 0.98804E+00_JPRB, 0.98456E+00_JPRB, 0.95500E+00_JPRB, 0.91820E+00_JPRB, 0.88552E+00_JPRB, 0.87062E+00_JPRB ,&
&  0.85291E+00_JPRB, 0.81963E+00_JPRB, 0.78571E+00_JPRB, 0.75497E+00_JPRB, 0.72460E+00_JPRB, 0.70835E+00_JPRB, 0.70297E+00_JPRB ,&
&  0.70996E+00_JPRB, 0.72362E+00_JPRB, 0.77929E+00_JPRB, 0.91683E+00_JPRB, 0.11036E+01_JPRB, 0.14057E+01_JPRB, 0.12878E+01_JPRB ,&
&  0.18221E+01_JPRB, 0.23634E+01_JPRB, 0.22626E+01_JPRB, 0.21502E+01_JPRB, 0.20374E+01_JPRB, 0.19493E+01_JPRB, 0.17346E+01_JPRB ,&
&  0.15295E+01_JPRB, 0.14063E+01_JPRB, 0.12714E+01_JPRB, 0.11838E+01_JPRB, 0.11456E+01_JPRB, 0.11290E+01_JPRB, 0.10990E+01_JPRB ,&
&  0.10797E+01_JPRB, 0.10536E+01_JPRB, 0.10409E+01_JPRB, 0.10724E+01_JPRB, 0.11436E+01_JPRB, 0.12607E+01_JPRB, 0.13390E+01_JPRB ,&
&  0.13747E+01_JPRB, 0.13739E+01_JPRB, 0.13642E+01_JPRB, 0.12912E+01_JPRB, 0.13354E+01_JPRB, 0.13808E+01_JPRB, 0.14379E+01_JPRB ,&
&  0.14086E+01_JPRB, 0.13526E+01_JPRB, 0.15168E+01_JPRB, 0.16727E+01_JPRB, 0.17302E+01_JPRB, 0.17128E+01_JPRB, 0.18479E+01_JPRB ,&
&  0.22932E+01_JPRB, 0.27015E+01_JPRB, 0.33822E+01_JPRB, 0.35994E+01_JPRB, 0.37095E+01_JPRB, 0.37481E+01_JPRB, 0.36309E+01_JPRB ,&
&  0.35254E+01_JPRB, 0.27905E+01_JPRB, 0.24397E+01_JPRB, 0.23281E+01_JPRB, 0.23065E+01_JPRB, 0.24487E+01_JPRB, 0.31780E+01_JPRB ,&
&  0.38584E+01_JPRB, 0.42376E+01_JPRB, 0.43163E+01_JPRB, 0.36698E+01_JPRB, 0.31488E+01_JPRB, 0.26735E+01_JPRB, 0.24968E+01_JPRB ,&
&  0.29396E+01_JPRB, 0.30140E+01_JPRB, 0.27204E+01_JPRB, 0.29001E+01_JPRB, 0.22857E+01_JPRB, 0.20946E+01_JPRB, 0.18124E+01_JPRB ,&
&  0.15912E+01_JPRB, 0.14344E+01_JPRB, 0.13116E+01_JPRB, 0.12247E+01_JPRB, 0.11738E+01_JPRB, 0.11135E+01_JPRB, 0.10712E+01_JPRB ,&
&  0.10465E+01_JPRB, 0.10433E+01_JPRB, 0.10410E+01_JPRB, 0.10338E+01_JPRB, 0.10139E+01_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.98524E+00_JPRB, 0.97877E+00_JPRB, 0.96349E+00_JPRB, 0.92396E+00_JPRB, 0.89355E+00_JPRB, 0.87363E+00_JPRB, 0.83933E+00_JPRB ,&
&  0.81644E+00_JPRB, 0.79390E+00_JPRB, 0.76475E+00_JPRB, 0.73703E+00_JPRB, 0.71892E+00_JPRB, 0.70352E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.74868E+00_JPRB, 0.82226E+00_JPRB, 0.95158E+00_JPRB, 0.10858E+01_JPRB, 0.12866E+01_JPRB, 0.16256E+01_JPRB, 0.18187E+01_JPRB ,&
&  0.19567E+01_JPRB, 0.28538E+01_JPRB, 0.24665E+01_JPRB, 0.22824E+01_JPRB, 0.21400E+01_JPRB, 0.19848E+01_JPRB, 0.17283E+01_JPRB ,&
&  0.14683E+01_JPRB, 0.13687E+01_JPRB, 0.13016E+01_JPRB, 0.12350E+01_JPRB, 0.12021E+01_JPRB, 0.11363E+01_JPRB, 0.11271E+01_JPRB ,&
&  0.11072E+01_JPRB, 0.10917E+01_JPRB, 0.10731E+01_JPRB, 0.11079E+01_JPRB, 0.11896E+01_JPRB, 0.12806E+01_JPRB, 0.13629E+01_JPRB ,&
&  0.13964E+01_JPRB, 0.13850E+01_JPRB, 0.13284E+01_JPRB, 0.13024E+01_JPRB, 0.13019E+01_JPRB, 0.13615E+01_JPRB, 0.14357E+01_JPRB ,&
&  0.14544E+01_JPRB, 0.14110E+01_JPRB, 0.15059E+01_JPRB, 0.15537E+01_JPRB, 0.15949E+01_JPRB, 0.17699E+01_JPRB, 0.19021E+01_JPRB ,&
&  0.23769E+01_JPRB, 0.24802E+01_JPRB, 0.35907E+01_JPRB, 0.33514E+01_JPRB, 0.32975E+01_JPRB, 0.34109E+01_JPRB, 0.31503E+01_JPRB ,&
&  0.26595E+01_JPRB, 0.24534E+01_JPRB, 0.22225E+01_JPRB, 0.21903E+01_JPRB, 0.23153E+01_JPRB, 0.26244E+01_JPRB, 0.32017E+01_JPRB ,&
&  0.33641E+01_JPRB, 0.32524E+01_JPRB, 0.32838E+01_JPRB, 0.27774E+01_JPRB, 0.26642E+01_JPRB, 0.24745E+01_JPRB, 0.24056E+01_JPRB ,&
&  0.28662E+01_JPRB, 0.28996E+01_JPRB, 0.28258E+01_JPRB, 0.26658E+01_JPRB, 0.20669E+01_JPRB, 0.18466E+01_JPRB, 0.16023E+01_JPRB ,&
&  0.14393E+01_JPRB, 0.13081E+01_JPRB, 0.12343E+01_JPRB, 0.11712E+01_JPRB, 0.11141E+01_JPRB, 0.10556E+01_JPRB, 0.10233E+01_JPRB ,&
&  0.10034E+01_JPRB, 0.99757E+00_JPRB, 0.10001E+01_JPRB, 0.99451E+00_JPRB, 0.98726E+00_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.10161E+01_JPRB, 0.10234E+01_JPRB, 0.10274E+01_JPRB, 0.10560E+01_JPRB, 0.11001E+01_JPRB, 0.11667E+01_JPRB, 0.11994E+01_JPRB ,&
&  0.10570E+01_JPRB, 0.76999E+00_JPRB, 0.75037E+00_JPRB, 0.73299E+00_JPRB, 0.71707E+00_JPRB, 0.72110E+00_JPRB, 0.76899E+00_JPRB ,&
&  0.84494E+00_JPRB, 0.94256E+00_JPRB, 0.10554E+01_JPRB, 0.12260E+01_JPRB, 0.14775E+01_JPRB, 0.19385E+01_JPRB, 0.23588E+01_JPRB ,&
&  0.23149E+01_JPRB, 0.24708E+01_JPRB, 0.25195E+01_JPRB, 0.21992E+01_JPRB, 0.20099E+01_JPRB, 0.18843E+01_JPRB, 0.17679E+01_JPRB ,&
&  0.15383E+01_JPRB, 0.14184E+01_JPRB, 0.13902E+01_JPRB, 0.13776E+01_JPRB, 0.11861E+01_JPRB, 0.11682E+01_JPRB, 0.11659E+01_JPRB ,&
&  0.11662E+01_JPRB, 0.11747E+01_JPRB, 0.11832E+01_JPRB, 0.12302E+01_JPRB, 0.13005E+01_JPRB, 0.13802E+01_JPRB, 0.14416E+01_JPRB ,&
&  0.14637E+01_JPRB, 0.14522E+01_JPRB, 0.14130E+01_JPRB, 0.13230E+01_JPRB, 0.13193E+01_JPRB, 0.13690E+01_JPRB, 0.14358E+01_JPRB ,&
&  0.14784E+01_JPRB, 0.15352E+01_JPRB, 0.15736E+01_JPRB, 0.16008E+01_JPRB, 0.16020E+01_JPRB, 0.18902E+01_JPRB, 0.20809E+01_JPRB ,&
&  0.23882E+01_JPRB, 0.27677E+01_JPRB, 0.38669E+01_JPRB, 0.28414E+01_JPRB, 0.29472E+01_JPRB, 0.30068E+01_JPRB, 0.27858E+01_JPRB ,&
&  0.23742E+01_JPRB, 0.22148E+01_JPRB, 0.20354E+01_JPRB, 0.20386E+01_JPRB, 0.23183E+01_JPRB, 0.26580E+01_JPRB, 0.33397E+01_JPRB ,&
&  0.28225E+01_JPRB, 0.28508E+01_JPRB, 0.24892E+01_JPRB, 0.24789E+01_JPRB, 0.24053E+01_JPRB, 0.22642E+01_JPRB, 0.21274E+01_JPRB ,&
&  0.25065E+01_JPRB, 0.23982E+01_JPRB, 0.21870E+01_JPRB, 0.19565E+01_JPRB, 0.15587E+01_JPRB, 0.13902E+01_JPRB, 0.12338E+01_JPRB ,&
&  0.11666E+01_JPRB, 0.11373E+01_JPRB, 0.11303E+01_JPRB, 0.10925E+01_JPRB, 0.10656E+01_JPRB, 0.10345E+01_JPRB, 0.10180E+01_JPRB ,&
&  0.10045E+01_JPRB, 0.99957E+00_JPRB, 0.99822E+00_JPRB, 0.10019E+01_JPRB, 0.10065E+01_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.10777E+01_JPRB, 0.10653E+01_JPRB, 0.10556E+01_JPRB, 0.10484E+01_JPRB, 0.10379E+01_JPRB, 0.10066E+01_JPRB, 0.94173E+00_JPRB ,&
&  0.85109E+00_JPRB, 0.78129E+00_JPRB, 0.76536E+00_JPRB, 0.75672E+00_JPRB, 0.76094E+00_JPRB, 0.79264E+00_JPRB, 0.85205E+00_JPRB ,&
&  0.91724E+00_JPRB, 0.10286E+01_JPRB, 0.11830E+01_JPRB, 0.13874E+01_JPRB, 0.16659E+01_JPRB, 0.21056E+01_JPRB, 0.25652E+01_JPRB ,&
&  0.25523E+01_JPRB, 0.18785E+01_JPRB, 0.21232E+01_JPRB, 0.20751E+01_JPRB, 0.19412E+01_JPRB, 0.17970E+01_JPRB, 0.16210E+01_JPRB ,&
&  0.14401E+01_JPRB, 0.13866E+01_JPRB, 0.13635E+01_JPRB, 0.13131E+01_JPRB, 0.12217E+01_JPRB, 0.12522E+01_JPRB, 0.12873E+01_JPRB ,&
&  0.13227E+01_JPRB, 0.13503E+01_JPRB, 0.13561E+01_JPRB, 0.14045E+01_JPRB, 0.14510E+01_JPRB, 0.15247E+01_JPRB, 0.16065E+01_JPRB ,&
&  0.16376E+01_JPRB, 0.16426E+01_JPRB, 0.16252E+01_JPRB, 0.15171E+01_JPRB, 0.14877E+01_JPRB, 0.15508E+01_JPRB, 0.16112E+01_JPRB ,&
&  0.16496E+01_JPRB, 0.16376E+01_JPRB, 0.17170E+01_JPRB, 0.19149E+01_JPRB, 0.21601E+01_JPRB, 0.20689E+01_JPRB, 0.24116E+01_JPRB ,&
&  0.26540E+01_JPRB, 0.31226E+01_JPRB, 0.30440E+01_JPRB, 0.34828E+01_JPRB, 0.22886E+01_JPRB, 0.24654E+01_JPRB, 0.21847E+01_JPRB ,&
&  0.21980E+01_JPRB, 0.19817E+01_JPRB, 0.19297E+01_JPRB, 0.20988E+01_JPRB, 0.24911E+01_JPRB, 0.29697E+01_JPRB, 0.29635E+01_JPRB ,&
&  0.22359E+01_JPRB, 0.20814E+01_JPRB, 0.21491E+01_JPRB, 0.23585E+01_JPRB, 0.23351E+01_JPRB, 0.22330E+01_JPRB, 0.19304E+01_JPRB ,&
&  0.19634E+01_JPRB, 0.18247E+01_JPRB, 0.16936E+01_JPRB, 0.15105E+01_JPRB, 0.11859E+01_JPRB, 0.10605E+01_JPRB, 0.10528E+01_JPRB ,&
&  0.10742E+01_JPRB, 0.10952E+01_JPRB, 0.11007E+01_JPRB, 0.10973E+01_JPRB, 0.10750E+01_JPRB, 0.10533E+01_JPRB, 0.10394E+01_JPRB ,&
&  0.10321E+01_JPRB, 0.10389E+01_JPRB, 0.10472E+01_JPRB, 0.10546E+01_JPRB, 0.10680E+01_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.10537E+01_JPRB, 0.10274E+01_JPRB, 0.10011E+01_JPRB, 0.97697E+00_JPRB, 0.95682E+00_JPRB, 0.91046E+00_JPRB, 0.85857E+00_JPRB ,&
&  0.82484E+00_JPRB, 0.80686E+00_JPRB, 0.80583E+00_JPRB, 0.81345E+00_JPRB, 0.83925E+00_JPRB, 0.86948E+00_JPRB, 0.92514E+00_JPRB ,&
&  0.10148E+01_JPRB, 0.11430E+01_JPRB, 0.13190E+01_JPRB, 0.15369E+01_JPRB, 0.18154E+01_JPRB, 0.21394E+01_JPRB, 0.23460E+01_JPRB ,&
&  0.25375E+01_JPRB, 0.25545E+01_JPRB, 0.21995E+01_JPRB, 0.16172E+01_JPRB, 0.15098E+01_JPRB, 0.15689E+01_JPRB, 0.14606E+01_JPRB ,&
&  0.14429E+01_JPRB, 0.14025E+01_JPRB, 0.13589E+01_JPRB, 0.13632E+01_JPRB, 0.13749E+01_JPRB, 0.14291E+01_JPRB, 0.14736E+01_JPRB ,&
&  0.15063E+01_JPRB, 0.15335E+01_JPRB, 0.15718E+01_JPRB, 0.16494E+01_JPRB, 0.17184E+01_JPRB, 0.18015E+01_JPRB, 0.18665E+01_JPRB ,&
&  0.19071E+01_JPRB, 0.19674E+01_JPRB, 0.19602E+01_JPRB, 0.18970E+01_JPRB, 0.18315E+01_JPRB, 0.18928E+01_JPRB, 0.19722E+01_JPRB ,&
&  0.20387E+01_JPRB, 0.20631E+01_JPRB, 0.21226E+01_JPRB, 0.23777E+01_JPRB, 0.26038E+01_JPRB, 0.24187E+01_JPRB, 0.25260E+01_JPRB ,&
&  0.29491E+01_JPRB, 0.32542E+01_JPRB, 0.24771E+01_JPRB, 0.29290E+01_JPRB, 0.17286E+01_JPRB, 0.21051E+01_JPRB, 0.21986E+01_JPRB ,&
&  0.19905E+01_JPRB, 0.19552E+01_JPRB, 0.19948E+01_JPRB, 0.22508E+01_JPRB, 0.26326E+01_JPRB, 0.28342E+01_JPRB, 0.24444E+01_JPRB ,&
&  0.18707E+01_JPRB, 0.18064E+01_JPRB, 0.19026E+01_JPRB, 0.20189E+01_JPRB, 0.20272E+01_JPRB, 0.18769E+01_JPRB, 0.14567E+01_JPRB ,&
&  0.11924E+01_JPRB, 0.14422E+01_JPRB, 0.14614E+01_JPRB, 0.14615E+01_JPRB, 0.11461E+01_JPRB, 0.98943E+00_JPRB, 0.10472E+01_JPRB ,&
&  0.11013E+01_JPRB, 0.11343E+01_JPRB, 0.11417E+01_JPRB, 0.11287E+01_JPRB, 0.11042E+01_JPRB, 0.10835E+01_JPRB, 0.10758E+01_JPRB ,&
&  0.10752E+01_JPRB, 0.10697E+01_JPRB, 0.10726E+01_JPRB, 0.10777E+01_JPRB, 0.10739E+01_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.96718E+00_JPRB, 0.93516E+00_JPRB, 0.91285E+00_JPRB, 0.89522E+00_JPRB, 0.88515E+00_JPRB, 0.86093E+00_JPRB, 0.82827E+00_JPRB ,&
&  0.80961E+00_JPRB, 0.80289E+00_JPRB, 0.81437E+00_JPRB, 0.84531E+00_JPRB, 0.88944E+00_JPRB, 0.93290E+00_JPRB, 0.10155E+01_JPRB ,&
&  0.11164E+01_JPRB, 0.12654E+01_JPRB, 0.14581E+01_JPRB, 0.17385E+01_JPRB, 0.19729E+01_JPRB, 0.21932E+01_JPRB, 0.22437E+01_JPRB ,&
&  0.23395E+01_JPRB, 0.23509E+01_JPRB, 0.22259E+01_JPRB, 0.20788E+01_JPRB, 0.16652E+01_JPRB, 0.16835E+01_JPRB, 0.18487E+01_JPRB ,&
&  0.19279E+01_JPRB, 0.16862E+01_JPRB, 0.15147E+01_JPRB, 0.15401E+01_JPRB, 0.15564E+01_JPRB, 0.16203E+01_JPRB, 0.16778E+01_JPRB ,&
&  0.17271E+01_JPRB, 0.17807E+01_JPRB, 0.18360E+01_JPRB, 0.19181E+01_JPRB, 0.20068E+01_JPRB, 0.20602E+01_JPRB, 0.21142E+01_JPRB ,&
&  0.21463E+01_JPRB, 0.21884E+01_JPRB, 0.22241E+01_JPRB, 0.22251E+01_JPRB, 0.22851E+01_JPRB, 0.23800E+01_JPRB, 0.25465E+01_JPRB ,&
&  0.27160E+01_JPRB, 0.27069E+01_JPRB, 0.28982E+01_JPRB, 0.32154E+01_JPRB, 0.32623E+01_JPRB, 0.31421E+01_JPRB, 0.31239E+01_JPRB ,&
&  0.34331E+01_JPRB, 0.37027E+01_JPRB, 0.20389E+01_JPRB, 0.22311E+01_JPRB, 0.25533E+01_JPRB, 0.21761E+01_JPRB, 0.19512E+01_JPRB ,&
&  0.19334E+01_JPRB, 0.19655E+01_JPRB, 0.20966E+01_JPRB, 0.22792E+01_JPRB, 0.25932E+01_JPRB, 0.31483E+01_JPRB, 0.21863E+01_JPRB ,&
&  0.15792E+01_JPRB, 0.14895E+01_JPRB, 0.15422E+01_JPRB, 0.16014E+01_JPRB, 0.16024E+01_JPRB, 0.14140E+01_JPRB, 0.10934E+01_JPRB ,&
&  0.10206E+01_JPRB, 0.13978E+01_JPRB, 0.15254E+01_JPRB, 0.16610E+01_JPRB, 0.14884E+01_JPRB, 0.98283E+00_JPRB, 0.10774E+01_JPRB ,&
&  0.11157E+01_JPRB, 0.11318E+01_JPRB, 0.11064E+01_JPRB, 0.10971E+01_JPRB, 0.10897E+01_JPRB, 0.10792E+01_JPRB, 0.10774E+01_JPRB ,&
&  0.10729E+01_JPRB, 0.10733E+01_JPRB, 0.10748E+01_JPRB, 0.10469E+01_JPRB, 0.10023E+01_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.88110E+00_JPRB, 0.85551E+00_JPRB, 0.84125E+00_JPRB, 0.82225E+00_JPRB, 0.81146E+00_JPRB, 0.79865E+00_JPRB, 0.79287E+00_JPRB ,&
&  0.79143E+00_JPRB, 0.79587E+00_JPRB, 0.81590E+00_JPRB, 0.84915E+00_JPRB, 0.89656E+00_JPRB, 0.96090E+00_JPRB, 0.10747E+01_JPRB ,&
&  0.11897E+01_JPRB, 0.13664E+01_JPRB, 0.15834E+01_JPRB, 0.18380E+01_JPRB, 0.20279E+01_JPRB, 0.20971E+01_JPRB, 0.22147E+01_JPRB ,&
&  0.22952E+01_JPRB, 0.23235E+01_JPRB, 0.22956E+01_JPRB, 0.22422E+01_JPRB, 0.21579E+01_JPRB, 0.21862E+01_JPRB, 0.22962E+01_JPRB ,&
&  0.21100E+01_JPRB, 0.16940E+01_JPRB, 0.16313E+01_JPRB, 0.16745E+01_JPRB, 0.17371E+01_JPRB, 0.17813E+01_JPRB, 0.18580E+01_JPRB ,&
&  0.19237E+01_JPRB, 0.19766E+01_JPRB, 0.20373E+01_JPRB, 0.21065E+01_JPRB, 0.22144E+01_JPRB, 0.22866E+01_JPRB, 0.23777E+01_JPRB ,&
&  0.24578E+01_JPRB, 0.25427E+01_JPRB, 0.26879E+01_JPRB, 0.25734E+01_JPRB, 0.28406E+01_JPRB, 0.30426E+01_JPRB, 0.32345E+01_JPRB ,&
&  0.33124E+01_JPRB, 0.33012E+01_JPRB, 0.35764E+01_JPRB, 0.37920E+01_JPRB, 0.39455E+01_JPRB, 0.39871E+01_JPRB, 0.38728E+01_JPRB ,&
&  0.40220E+01_JPRB, 0.37290E+01_JPRB, 0.20261E+01_JPRB, 0.17516E+01_JPRB, 0.18575E+01_JPRB, 0.16421E+01_JPRB, 0.18906E+01_JPRB ,&
&  0.18905E+01_JPRB, 0.19593E+01_JPRB, 0.20619E+01_JPRB, 0.22352E+01_JPRB, 0.24269E+01_JPRB, 0.28788E+01_JPRB, 0.19847E+01_JPRB ,&
&  0.12462E+01_JPRB, 0.11568E+01_JPRB, 0.11625E+01_JPRB, 0.11940E+01_JPRB, 0.12369E+01_JPRB, 0.11685E+01_JPRB, 0.10796E+01_JPRB ,&
&  0.11211E+01_JPRB, 0.11809E+01_JPRB, 0.12329E+01_JPRB, 0.11608E+01_JPRB, 0.98010E+00_JPRB, 0.93708E+00_JPRB, 0.98191E+00_JPRB ,&
&  0.10159E+01_JPRB, 0.10336E+01_JPRB, 0.10271E+01_JPRB, 0.10307E+01_JPRB, 0.10214E+01_JPRB, 0.10154E+01_JPRB, 0.10097E+01_JPRB ,&
&  0.99840E+00_JPRB, 0.99281E+00_JPRB, 0.97548E+00_JPRB, 0.93419E+00_JPRB, 0.90284E+00_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.77584E+00_JPRB, 0.77689E+00_JPRB, 0.76778E+00_JPRB, 0.75946E+00_JPRB, 0.75609E+00_JPRB, 0.73865E+00_JPRB, 0.76004E+00_JPRB ,&
&  0.75603E+00_JPRB, 0.77812E+00_JPRB, 0.80148E+00_JPRB, 0.82627E+00_JPRB, 0.87402E+00_JPRB, 0.92390E+00_JPRB, 0.99499E+00_JPRB ,&
&  0.11179E+01_JPRB, 0.13169E+01_JPRB, 0.15713E+01_JPRB, 0.18878E+01_JPRB, 0.20015E+01_JPRB, 0.20776E+01_JPRB, 0.22012E+01_JPRB ,&
&  0.23531E+01_JPRB, 0.24777E+01_JPRB, 0.25517E+01_JPRB, 0.26424E+01_JPRB, 0.27333E+01_JPRB, 0.26864E+01_JPRB, 0.25675E+01_JPRB ,&
&  0.21372E+01_JPRB, 0.16562E+01_JPRB, 0.18085E+01_JPRB, 0.17174E+01_JPRB, 0.18301E+01_JPRB, 0.19300E+01_JPRB, 0.20102E+01_JPRB ,&
&  0.20948E+01_JPRB, 0.21570E+01_JPRB, 0.22421E+01_JPRB, 0.23418E+01_JPRB, 0.24746E+01_JPRB, 0.26046E+01_JPRB, 0.27461E+01_JPRB ,&
&  0.28559E+01_JPRB, 0.29649E+01_JPRB, 0.30489E+01_JPRB, 0.30152E+01_JPRB, 0.32270E+01_JPRB, 0.35111E+01_JPRB, 0.37754E+01_JPRB ,&
&  0.39875E+01_JPRB, 0.39762E+01_JPRB, 0.35790E+01_JPRB, 0.37564E+01_JPRB, 0.40349E+01_JPRB, 0.38649E+01_JPRB, 0.40119E+01_JPRB ,&
&  0.44254E+01_JPRB, 0.38806E+01_JPRB, 0.18833E+01_JPRB, 0.16071E+01_JPRB, 0.16855E+01_JPRB, 0.17274E+01_JPRB, 0.17645E+01_JPRB ,&
&  0.17796E+01_JPRB, 0.18166E+01_JPRB, 0.18877E+01_JPRB, 0.19696E+01_JPRB, 0.19760E+01_JPRB, 0.20035E+01_JPRB, 0.16537E+01_JPRB ,&
&  0.10595E+01_JPRB, 0.10189E+01_JPRB, 0.10705E+01_JPRB, 0.11279E+01_JPRB, 0.12483E+01_JPRB, 0.12180E+01_JPRB, 0.10455E+01_JPRB ,&
&  0.11287E+01_JPRB, 0.12801E+01_JPRB, 0.15030E+01_JPRB, 0.13075E+01_JPRB, 0.91216E+00_JPRB, 0.86134E+00_JPRB, 0.86310E+00_JPRB ,&
&  0.89331E+00_JPRB, 0.89601E+00_JPRB, 0.91141E+00_JPRB, 0.92077E+00_JPRB, 0.93428E+00_JPRB, 0.91965E+00_JPRB, 0.90232E+00_JPRB ,&
&  0.88052E+00_JPRB, 0.86109E+00_JPRB, 0.84096E+00_JPRB, 0.81040E+00_JPRB, 0.78806E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.68398E+00_JPRB, 0.65244E+00_JPRB, 0.62336E+00_JPRB, 0.62309E+00_JPRB, 0.62753E+00_JPRB, 0.61324E+00_JPRB, 0.64492E+00_JPRB ,&
&  0.61948E+00_JPRB, 0.64426E+00_JPRB, 0.65778E+00_JPRB, 0.65838E+00_JPRB, 0.70115E+00_JPRB, 0.74529E+00_JPRB, 0.80211E+00_JPRB ,&
&  0.88061E+00_JPRB, 0.10041E+01_JPRB, 0.11598E+01_JPRB, 0.13547E+01_JPRB, 0.15913E+01_JPRB, 0.17071E+01_JPRB, 0.18245E+01_JPRB ,&
&  0.19898E+01_JPRB, 0.22082E+01_JPRB, 0.25186E+01_JPRB, 0.28727E+01_JPRB, 0.32945E+01_JPRB, 0.31469E+01_JPRB, 0.26863E+01_JPRB ,&
&  0.20272E+01_JPRB, 0.16839E+01_JPRB, 0.16171E+01_JPRB, 0.15276E+01_JPRB, 0.16626E+01_JPRB, 0.19239E+01_JPRB, 0.20655E+01_JPRB ,&
&  0.21632E+01_JPRB, 0.22737E+01_JPRB, 0.23959E+01_JPRB, 0.25309E+01_JPRB, 0.26783E+01_JPRB, 0.28457E+01_JPRB, 0.29448E+01_JPRB ,&
&  0.30099E+01_JPRB, 0.30788E+01_JPRB, 0.31437E+01_JPRB, 0.33392E+01_JPRB, 0.35982E+01_JPRB, 0.39355E+01_JPRB, 0.40376E+01_JPRB ,&
&  0.40680E+01_JPRB, 0.39819E+01_JPRB, 0.34623E+01_JPRB, 0.34367E+01_JPRB, 0.35379E+01_JPRB, 0.35723E+01_JPRB, 0.36318E+01_JPRB ,&
&  0.39641E+01_JPRB, 0.31299E+01_JPRB, 0.18793E+01_JPRB, 0.17940E+01_JPRB, 0.17785E+01_JPRB, 0.18029E+01_JPRB, 0.16014E+01_JPRB ,&
&  0.15965E+01_JPRB, 0.15974E+01_JPRB, 0.16164E+01_JPRB, 0.15842E+01_JPRB, 0.15796E+01_JPRB, 0.15473E+01_JPRB, 0.14072E+01_JPRB ,&
&  0.11517E+01_JPRB, 0.10820E+01_JPRB, 0.10345E+01_JPRB, 0.10245E+01_JPRB, 0.12268E+01_JPRB, 0.11865E+01_JPRB, 0.11401E+01_JPRB ,&
&  0.10582E+01_JPRB, 0.11598E+01_JPRB, 0.10328E+01_JPRB, 0.12506E+01_JPRB, 0.84578E+00_JPRB, 0.76400E+00_JPRB, 0.74997E+00_JPRB ,&
&  0.77261E+00_JPRB, 0.78179E+00_JPRB, 0.79527E+00_JPRB, 0.78398E+00_JPRB, 0.79847E+00_JPRB, 0.82798E+00_JPRB, 0.83008E+00_JPRB ,&
&  0.78489E+00_JPRB, 0.76889E+00_JPRB, 0.73907E+00_JPRB, 0.70374E+00_JPRB, 0.69605E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.58491E+00_JPRB, 0.57723E+00_JPRB, 0.57458E+00_JPRB, 0.57074E+00_JPRB, 0.57971E+00_JPRB, 0.58181E+00_JPRB, 0.58376E+00_JPRB ,&
&  0.58433E+00_JPRB, 0.59723E+00_JPRB, 0.60738E+00_JPRB, 0.62244E+00_JPRB, 0.63932E+00_JPRB, 0.67474E+00_JPRB, 0.71474E+00_JPRB ,&
&  0.76176E+00_JPRB, 0.81516E+00_JPRB, 0.87451E+00_JPRB, 0.96075E+00_JPRB, 0.10821E+01_JPRB, 0.11694E+01_JPRB, 0.12653E+01_JPRB ,&
&  0.13927E+01_JPRB, 0.15854E+01_JPRB, 0.18017E+01_JPRB, 0.20942E+01_JPRB, 0.24319E+01_JPRB, 0.25184E+01_JPRB, 0.23482E+01_JPRB ,&
&  0.18022E+01_JPRB, 0.18122E+01_JPRB, 0.17203E+01_JPRB, 0.15831E+01_JPRB, 0.16368E+01_JPRB, 0.16188E+01_JPRB, 0.16913E+01_JPRB ,&
&  0.17900E+01_JPRB, 0.18299E+01_JPRB, 0.18537E+01_JPRB, 0.19080E+01_JPRB, 0.19548E+01_JPRB, 0.19976E+01_JPRB, 0.19918E+01_JPRB ,&
&  0.20022E+01_JPRB, 0.21046E+01_JPRB, 0.24688E+01_JPRB, 0.29241E+01_JPRB, 0.32565E+01_JPRB, 0.34183E+01_JPRB, 0.32617E+01_JPRB ,&
&  0.30231E+01_JPRB, 0.28766E+01_JPRB, 0.27090E+01_JPRB, 0.26752E+01_JPRB, 0.27540E+01_JPRB, 0.27918E+01_JPRB, 0.29752E+01_JPRB ,&
&  0.28091E+01_JPRB, 0.22949E+01_JPRB, 0.20342E+01_JPRB, 0.18653E+01_JPRB, 0.17841E+01_JPRB, 0.16839E+01_JPRB, 0.15544E+01_JPRB ,&
&  0.15620E+01_JPRB, 0.14992E+01_JPRB, 0.14487E+01_JPRB, 0.13445E+01_JPRB, 0.12454E+01_JPRB, 0.11773E+01_JPRB, 0.11331E+01_JPRB ,&
&  0.10283E+01_JPRB, 0.94865E+00_JPRB, 0.93160E+00_JPRB, 0.10069E+01_JPRB, 0.11661E+01_JPRB, 0.17840E+01_JPRB, 0.16263E+01_JPRB ,&
&  0.99716E+00_JPRB, 0.85663E+00_JPRB, 0.65021E+00_JPRB, 0.85735E+00_JPRB, 0.70764E+00_JPRB, 0.67642E+00_JPRB, 0.67651E+00_JPRB ,&
&  0.68164E+00_JPRB, 0.67821E+00_JPRB, 0.65833E+00_JPRB, 0.65456E+00_JPRB, 0.66416E+00_JPRB, 0.67270E+00_JPRB, 0.66263E+00_JPRB ,&
&  0.62572E+00_JPRB, 0.60868E+00_JPRB, 0.63533E+00_JPRB, 0.59518E+00_JPRB, 0.58725E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.53682E+00_JPRB, 0.53710E+00_JPRB, 0.55165E+00_JPRB, 0.56553E+00_JPRB, 0.57368E+00_JPRB, 0.58070E+00_JPRB, 0.59184E+00_JPRB ,&
&  0.59906E+00_JPRB, 0.60899E+00_JPRB, 0.62024E+00_JPRB, 0.63974E+00_JPRB, 0.66756E+00_JPRB, 0.70907E+00_JPRB, 0.75645E+00_JPRB ,&
&  0.81475E+00_JPRB, 0.87771E+00_JPRB, 0.94900E+00_JPRB, 0.10271E+01_JPRB, 0.11241E+01_JPRB, 0.12599E+01_JPRB, 0.14454E+01_JPRB ,&
&  0.15516E+01_JPRB, 0.16870E+01_JPRB, 0.18542E+01_JPRB, 0.20012E+01_JPRB, 0.20308E+01_JPRB, 0.19805E+01_JPRB, 0.16260E+01_JPRB ,&
&  0.19155E+01_JPRB, 0.18194E+01_JPRB, 0.16798E+01_JPRB, 0.15366E+01_JPRB, 0.14360E+01_JPRB, 0.13565E+01_JPRB, 0.12935E+01_JPRB ,&
&  0.12792E+01_JPRB, 0.12375E+01_JPRB, 0.11758E+01_JPRB, 0.11328E+01_JPRB, 0.11159E+01_JPRB, 0.11399E+01_JPRB, 0.11341E+01_JPRB ,&
&  0.12601E+01_JPRB, 0.15454E+01_JPRB, 0.19171E+01_JPRB, 0.23711E+01_JPRB, 0.26317E+01_JPRB, 0.26083E+01_JPRB, 0.21803E+01_JPRB ,&
&  0.19297E+01_JPRB, 0.18363E+01_JPRB, 0.18162E+01_JPRB, 0.19231E+01_JPRB, 0.19334E+01_JPRB, 0.20258E+01_JPRB, 0.20798E+01_JPRB ,&
&  0.20544E+01_JPRB, 0.17925E+01_JPRB, 0.17350E+01_JPRB, 0.20103E+01_JPRB, 0.18294E+01_JPRB, 0.16365E+01_JPRB, 0.16006E+01_JPRB ,&
&  0.15002E+01_JPRB, 0.14331E+01_JPRB, 0.13904E+01_JPRB, 0.12938E+01_JPRB, 0.11802E+01_JPRB, 0.10830E+01_JPRB, 0.10376E+01_JPRB ,&
&  0.95397E+00_JPRB, 0.86331E+00_JPRB, 0.81684E+00_JPRB, 0.86250E+00_JPRB, 0.10439E+01_JPRB, 0.12440E+01_JPRB, 0.17415E+01_JPRB ,&
&  0.99979E+00_JPRB, 0.62981E+00_JPRB, 0.65013E+00_JPRB, 0.68018E+00_JPRB, 0.62974E+00_JPRB, 0.61106E+00_JPRB, 0.66724E+00_JPRB ,&
&  0.73463E+00_JPRB, 0.80233E+00_JPRB, 0.79767E+00_JPRB, 0.68891E+00_JPRB, 0.64614E+00_JPRB, 0.62525E+00_JPRB, 0.60037E+00_JPRB ,&
&  0.58054E+00_JPRB, 0.54397E+00_JPRB, 0.55782E+00_JPRB, 0.54184E+00_JPRB, 0.53992E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.50582E+00_JPRB, 0.50467E+00_JPRB, 0.51333E+00_JPRB, 0.52176E+00_JPRB, 0.52386E+00_JPRB, 0.52614E+00_JPRB, 0.52833E+00_JPRB ,&
&  0.53171E+00_JPRB, 0.53196E+00_JPRB, 0.53452E+00_JPRB, 0.54193E+00_JPRB, 0.55800E+00_JPRB, 0.59316E+00_JPRB, 0.63427E+00_JPRB ,&
&  0.67079E+00_JPRB, 0.70541E+00_JPRB, 0.73993E+00_JPRB, 0.77033E+00_JPRB, 0.80258E+00_JPRB, 0.82656E+00_JPRB, 0.88999E+00_JPRB ,&
&  0.97131E+00_JPRB, 0.10690E+01_JPRB, 0.11703E+01_JPRB, 0.12495E+01_JPRB, 0.13227E+01_JPRB, 0.12939E+01_JPRB, 0.10768E+01_JPRB ,&
&  0.16820E+01_JPRB, 0.16630E+01_JPRB, 0.15324E+01_JPRB, 0.13874E+01_JPRB, 0.12813E+01_JPRB, 0.11854E+01_JPRB, 0.10863E+01_JPRB ,&
&  0.99331E+00_JPRB, 0.10393E+01_JPRB, 0.10026E+01_JPRB, 0.10593E+01_JPRB, 0.97630E+00_JPRB, 0.10419E+01_JPRB, 0.12506E+01_JPRB ,&
&  0.14277E+01_JPRB, 0.16191E+01_JPRB, 0.18552E+01_JPRB, 0.20912E+01_JPRB, 0.22285E+01_JPRB, 0.21894E+01_JPRB, 0.19597E+01_JPRB ,&
&  0.18293E+01_JPRB, 0.17916E+01_JPRB, 0.17287E+01_JPRB, 0.18872E+01_JPRB, 0.18111E+01_JPRB, 0.16893E+01_JPRB, 0.17112E+01_JPRB ,&
&  0.15100E+01_JPRB, 0.15050E+01_JPRB, 0.16853E+01_JPRB, 0.20125E+01_JPRB, 0.18368E+01_JPRB, 0.17069E+01_JPRB, 0.16399E+01_JPRB ,&
&  0.15253E+01_JPRB, 0.14282E+01_JPRB, 0.13571E+01_JPRB, 0.12590E+01_JPRB, 0.11519E+01_JPRB, 0.10498E+01_JPRB, 0.96527E+00_JPRB ,&
&  0.89608E+00_JPRB, 0.81673E+00_JPRB, 0.76099E+00_JPRB, 0.74985E+00_JPRB, 0.80489E+00_JPRB, 0.88111E+00_JPRB, 0.14556E+01_JPRB ,&
&  0.99960E+00_JPRB, 0.79848E+00_JPRB, 0.86175E+00_JPRB, 0.44010E+00_JPRB, 0.52306E+00_JPRB, 0.55507E+00_JPRB, 0.66720E+00_JPRB ,&
&  0.82874E+00_JPRB, 0.13200E+01_JPRB, 0.20364E+01_JPRB, 0.18315E+01_JPRB, 0.13990E+01_JPRB, 0.90209E+00_JPRB, 0.54449E+00_JPRB ,&
&  0.52450E+00_JPRB, 0.52009E+00_JPRB, 0.52523E+00_JPRB, 0.51700E+00_JPRB, 0.51357E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.46056E+00_JPRB, 0.45171E+00_JPRB, 0.46053E+00_JPRB, 0.47047E+00_JPRB, 0.46656E+00_JPRB, 0.46607E+00_JPRB, 0.46087E+00_JPRB ,&
&  0.45135E+00_JPRB, 0.44486E+00_JPRB, 0.46707E+00_JPRB, 0.50896E+00_JPRB, 0.56720E+00_JPRB, 0.64279E+00_JPRB, 0.71776E+00_JPRB ,&
&  0.76734E+00_JPRB, 0.82475E+00_JPRB, 0.87476E+00_JPRB, 0.91538E+00_JPRB, 0.91949E+00_JPRB, 0.92102E+00_JPRB, 0.90637E+00_JPRB ,&
&  0.89836E+00_JPRB, 0.91729E+00_JPRB, 0.97320E+00_JPRB, 0.10305E+01_JPRB, 0.11331E+01_JPRB, 0.12665E+01_JPRB, 0.85511E+00_JPRB ,&
&  0.13781E+01_JPRB, 0.13921E+01_JPRB, 0.13393E+01_JPRB, 0.12251E+01_JPRB, 0.11628E+01_JPRB, 0.10623E+01_JPRB, 0.98336E+00_JPRB ,&
&  0.99218E+00_JPRB, 0.10595E+01_JPRB, 0.10959E+01_JPRB, 0.12523E+01_JPRB, 0.14218E+01_JPRB, 0.15354E+01_JPRB, 0.17832E+01_JPRB ,&
&  0.20025E+01_JPRB, 0.22806E+01_JPRB, 0.25815E+01_JPRB, 0.27866E+01_JPRB, 0.29554E+01_JPRB, 0.30331E+01_JPRB, 0.30312E+01_JPRB ,&
&  0.28406E+01_JPRB, 0.27130E+01_JPRB, 0.28169E+01_JPRB, 0.30470E+01_JPRB, 0.26530E+01_JPRB, 0.24773E+01_JPRB, 0.20661E+01_JPRB ,&
&  0.15206E+01_JPRB, 0.13696E+01_JPRB, 0.15506E+01_JPRB, 0.18499E+01_JPRB, 0.18248E+01_JPRB, 0.17161E+01_JPRB, 0.15578E+01_JPRB ,&
&  0.14355E+01_JPRB, 0.13092E+01_JPRB, 0.12165E+01_JPRB, 0.11302E+01_JPRB, 0.10466E+01_JPRB, 0.96026E+00_JPRB, 0.87216E+00_JPRB ,&
&  0.82572E+00_JPRB, 0.77363E+00_JPRB, 0.71641E+00_JPRB, 0.71572E+00_JPRB, 0.76134E+00_JPRB, 0.90948E+00_JPRB, 0.98525E+00_JPRB ,&
&  0.11051E+01_JPRB, 0.80917E+00_JPRB, 0.69263E+00_JPRB, 0.65464E+00_JPRB, 0.52517E+00_JPRB, 0.55286E+00_JPRB, 0.71893E+00_JPRB ,&
&  0.11301E+01_JPRB, 0.16655E+01_JPRB, 0.28067E+01_JPRB, 0.50339E+01_JPRB, 0.27866E+01_JPRB, 0.21512E+01_JPRB, 0.52783E+00_JPRB ,&
&  0.50638E+00_JPRB, 0.50811E+00_JPRB, 0.50559E+00_JPRB, 0.48931E+00_JPRB, 0.48423E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.44765E+00_JPRB, 0.43961E+00_JPRB, 0.44885E+00_JPRB, 0.44693E+00_JPRB, 0.45020E+00_JPRB, 0.45699E+00_JPRB, 0.46099E+00_JPRB ,&
&  0.46482E+00_JPRB, 0.48137E+00_JPRB, 0.57212E+00_JPRB, 0.69730E+00_JPRB, 0.81606E+00_JPRB, 0.90666E+00_JPRB, 0.94799E+00_JPRB ,&
&  0.98947E+00_JPRB, 0.10105E+01_JPRB, 0.10053E+01_JPRB, 0.99382E+00_JPRB, 0.10108E+01_JPRB, 0.10613E+01_JPRB, 0.11026E+01_JPRB ,&
&  0.11411E+01_JPRB, 0.11793E+01_JPRB, 0.12223E+01_JPRB, 0.12894E+01_JPRB, 0.13566E+01_JPRB, 0.13099E+01_JPRB, 0.83285E+00_JPRB ,&
&  0.11691E+01_JPRB, 0.12439E+01_JPRB, 0.12619E+01_JPRB, 0.11769E+01_JPRB, 0.10940E+01_JPRB, 0.10497E+01_JPRB, 0.11350E+01_JPRB ,&
&  0.12607E+01_JPRB, 0.13463E+01_JPRB, 0.13916E+01_JPRB, 0.14828E+01_JPRB, 0.20825E+01_JPRB, 0.23179E+01_JPRB, 0.27619E+01_JPRB ,&
&  0.30864E+01_JPRB, 0.34316E+01_JPRB, 0.36831E+01_JPRB, 0.39580E+01_JPRB, 0.41883E+01_JPRB, 0.43538E+01_JPRB, 0.45856E+01_JPRB ,&
&  0.46729E+01_JPRB, 0.47859E+01_JPRB, 0.48095E+01_JPRB, 0.46917E+01_JPRB, 0.38061E+01_JPRB, 0.40314E+01_JPRB, 0.27048E+01_JPRB ,&
&  0.18863E+01_JPRB, 0.16031E+01_JPRB, 0.17613E+01_JPRB, 0.17890E+01_JPRB, 0.16505E+01_JPRB, 0.14811E+01_JPRB, 0.12789E+01_JPRB ,&
&  0.11862E+01_JPRB, 0.11256E+01_JPRB, 0.10736E+01_JPRB, 0.10058E+01_JPRB, 0.96708E+00_JPRB, 0.90044E+00_JPRB, 0.84823E+00_JPRB ,&
&  0.82643E+00_JPRB, 0.78073E+00_JPRB, 0.72714E+00_JPRB, 0.73857E+00_JPRB, 0.81285E+00_JPRB, 0.10387E+01_JPRB, 0.14555E+01_JPRB ,&
&  0.15974E+01_JPRB, 0.88573E+00_JPRB, 0.76981E+00_JPRB, 0.68418E+00_JPRB, 0.64377E+00_JPRB, 0.66273E+00_JPRB, 0.74169E+00_JPRB ,&
&  0.95974E+00_JPRB, 0.16974E+01_JPRB, 0.25792E+01_JPRB, 0.19486E+01_JPRB, 0.22154E+01_JPRB, 0.15259E+01_JPRB, 0.62768E+00_JPRB ,&
&  0.49256E+00_JPRB, 0.48002E+00_JPRB, 0.48532E+00_JPRB, 0.47605E+00_JPRB, 0.46323E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.49849E+00_JPRB, 0.48748E+00_JPRB, 0.48623E+00_JPRB, 0.49618E+00_JPRB, 0.50240E+00_JPRB, 0.52704E+00_JPRB, 0.55767E+00_JPRB ,&
&  0.57390E+00_JPRB, 0.64501E+00_JPRB, 0.80457E+00_JPRB, 0.94125E+00_JPRB, 0.99997E+00_JPRB, 0.10425E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.99579E+00_JPRB, 0.97201E+00_JPRB, 0.93149E+00_JPRB, 0.93439E+00_JPRB, 0.96126E+00_JPRB, 0.10098E+01_JPRB, 0.10732E+01_JPRB ,&
&  0.11449E+01_JPRB, 0.12172E+01_JPRB, 0.12799E+01_JPRB, 0.13387E+01_JPRB, 0.14197E+01_JPRB, 0.14154E+01_JPRB, 0.13587E+01_JPRB ,&
&  0.70574E+00_JPRB, 0.11713E+01_JPRB, 0.11981E+01_JPRB, 0.11404E+01_JPRB, 0.10870E+01_JPRB, 0.11164E+01_JPRB, 0.13183E+01_JPRB ,&
&  0.15320E+01_JPRB, 0.16187E+01_JPRB, 0.18807E+01_JPRB, 0.20100E+01_JPRB, 0.26268E+01_JPRB, 0.30078E+01_JPRB, 0.33657E+01_JPRB ,&
&  0.37039E+01_JPRB, 0.39591E+01_JPRB, 0.42337E+01_JPRB, 0.45043E+01_JPRB, 0.47709E+01_JPRB, 0.50031E+01_JPRB, 0.52692E+01_JPRB ,&
&  0.55819E+01_JPRB, 0.60085E+01_JPRB, 0.64591E+01_JPRB, 0.62378E+01_JPRB, 0.52722E+01_JPRB, 0.49126E+01_JPRB, 0.38444E+01_JPRB ,&
&  0.25376E+01_JPRB, 0.15703E+01_JPRB, 0.15605E+01_JPRB, 0.16320E+01_JPRB, 0.14761E+01_JPRB, 0.13249E+01_JPRB, 0.11860E+01_JPRB ,&
&  0.11031E+01_JPRB, 0.10375E+01_JPRB, 0.95280E+00_JPRB, 0.94619E+00_JPRB, 0.95718E+00_JPRB, 0.92309E+00_JPRB, 0.88735E+00_JPRB ,&
&  0.87909E+00_JPRB, 0.84837E+00_JPRB, 0.81188E+00_JPRB, 0.79243E+00_JPRB, 0.81592E+00_JPRB, 0.89130E+00_JPRB, 0.95717E+00_JPRB ,&
&  0.98566E+00_JPRB, 0.88230E+00_JPRB, 0.80473E+00_JPRB, 0.79355E+00_JPRB, 0.78036E+00_JPRB, 0.77403E+00_JPRB, 0.76363E+00_JPRB ,&
&  0.81192E+00_JPRB, 0.97215E+00_JPRB, 0.13662E+01_JPRB, 0.17616E+01_JPRB, 0.14148E+01_JPRB, 0.11177E+01_JPRB, 0.64354E+00_JPRB ,&
&  0.51717E+00_JPRB, 0.49991E+00_JPRB, 0.51776E+00_JPRB, 0.52164E+00_JPRB, 0.52121E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.60783E+00_JPRB, 0.59510E+00_JPRB, 0.57699E+00_JPRB, 0.59257E+00_JPRB, 0.62558E+00_JPRB, 0.66831E+00_JPRB, 0.68700E+00_JPRB ,&
&  0.71165E+00_JPRB, 0.81439E+00_JPRB, 0.95050E+00_JPRB, 0.10714E+01_JPRB, 0.10947E+01_JPRB, 0.10657E+01_JPRB, 0.97132E+00_JPRB ,&
&  0.94953E+00_JPRB, 0.92739E+00_JPRB, 0.91631E+00_JPRB, 0.90614E+00_JPRB, 0.91895E+00_JPRB, 0.96572E+00_JPRB, 0.10398E+01_JPRB ,&
&  0.11347E+01_JPRB, 0.12377E+01_JPRB, 0.13532E+01_JPRB, 0.14780E+01_JPRB, 0.15913E+01_JPRB, 0.16355E+01_JPRB, 0.15208E+01_JPRB ,&
&  0.64665E+00_JPRB, 0.79282E+00_JPRB, 0.11660E+01_JPRB, 0.11801E+01_JPRB, 0.11661E+01_JPRB, 0.12214E+01_JPRB, 0.13991E+01_JPRB ,&
&  0.15818E+01_JPRB, 0.16799E+01_JPRB, 0.19558E+01_JPRB, 0.24699E+01_JPRB, 0.27636E+01_JPRB, 0.31765E+01_JPRB, 0.34275E+01_JPRB ,&
&  0.36839E+01_JPRB, 0.39403E+01_JPRB, 0.42198E+01_JPRB, 0.45070E+01_JPRB, 0.47938E+01_JPRB, 0.50615E+01_JPRB, 0.53979E+01_JPRB ,&
&  0.58472E+01_JPRB, 0.63448E+01_JPRB, 0.67657E+01_JPRB, 0.61150E+01_JPRB, 0.64843E+01_JPRB, 0.56511E+01_JPRB, 0.48451E+01_JPRB ,&
&  0.27970E+01_JPRB, 0.16156E+01_JPRB, 0.15980E+01_JPRB, 0.16724E+01_JPRB, 0.14455E+01_JPRB, 0.12550E+01_JPRB, 0.10615E+01_JPRB ,&
&  0.98294E+00_JPRB, 0.90508E+00_JPRB, 0.92298E+00_JPRB, 0.95125E+00_JPRB, 0.93490E+00_JPRB, 0.96804E+00_JPRB, 0.91506E+00_JPRB ,&
&  0.91958E+00_JPRB, 0.92638E+00_JPRB, 0.91165E+00_JPRB, 0.91461E+00_JPRB, 0.91848E+00_JPRB, 0.92852E+00_JPRB, 0.91931E+00_JPRB ,&
&  0.92303E+00_JPRB, 0.91018E+00_JPRB, 0.90551E+00_JPRB, 0.90788E+00_JPRB, 0.89835E+00_JPRB, 0.88047E+00_JPRB, 0.83380E+00_JPRB ,&
&  0.78577E+00_JPRB, 0.79543E+00_JPRB, 0.89551E+00_JPRB, 0.10271E+01_JPRB, 0.97709E+00_JPRB, 0.84438E+00_JPRB, 0.67262E+00_JPRB ,&
&  0.65843E+00_JPRB, 0.67937E+00_JPRB, 0.68055E+00_JPRB, 0.66097E+00_JPRB, 0.63730E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.72483E+00_JPRB, 0.69816E+00_JPRB, 0.65939E+00_JPRB, 0.65046E+00_JPRB, 0.70350E+00_JPRB, 0.75186E+00_JPRB, 0.75423E+00_JPRB ,&
&  0.76343E+00_JPRB, 0.82970E+00_JPRB, 0.94966E+00_JPRB, 0.10450E+01_JPRB, 0.10768E+01_JPRB, 0.10702E+01_JPRB, 0.98116E+00_JPRB ,&
&  0.94861E+00_JPRB, 0.92835E+00_JPRB, 0.90368E+00_JPRB, 0.89399E+00_JPRB, 0.89554E+00_JPRB, 0.91054E+00_JPRB, 0.94620E+00_JPRB ,&
&  0.10007E+01_JPRB, 0.10932E+01_JPRB, 0.12382E+01_JPRB, 0.14434E+01_JPRB, 0.17421E+01_JPRB, 0.20909E+01_JPRB, 0.23929E+01_JPRB ,&
&  0.18013E+01_JPRB, 0.76759E+00_JPRB, 0.90369E+00_JPRB, 0.12612E+01_JPRB, 0.12245E+01_JPRB, 0.13083E+01_JPRB, 0.13390E+01_JPRB ,&
&  0.14538E+01_JPRB, 0.16823E+01_JPRB, 0.19290E+01_JPRB, 0.23954E+01_JPRB, 0.27213E+01_JPRB, 0.30636E+01_JPRB, 0.32331E+01_JPRB ,&
&  0.33995E+01_JPRB, 0.35976E+01_JPRB, 0.38005E+01_JPRB, 0.40114E+01_JPRB, 0.42453E+01_JPRB, 0.45055E+01_JPRB, 0.48881E+01_JPRB ,&
&  0.53724E+01_JPRB, 0.58487E+01_JPRB, 0.61813E+01_JPRB, 0.53250E+01_JPRB, 0.56216E+01_JPRB, 0.44236E+01_JPRB, 0.32829E+01_JPRB ,&
&  0.24956E+01_JPRB, 0.19491E+01_JPRB, 0.17563E+01_JPRB, 0.17529E+01_JPRB, 0.14727E+01_JPRB, 0.11172E+01_JPRB, 0.10520E+01_JPRB ,&
&  0.94025E+00_JPRB, 0.90813E+00_JPRB, 0.92724E+00_JPRB, 0.88228E+00_JPRB, 0.94099E+00_JPRB, 0.96229E+00_JPRB, 0.97512E+00_JPRB ,&
&  0.95688E+00_JPRB, 0.97653E+00_JPRB, 0.10004E+01_JPRB, 0.10405E+01_JPRB, 0.10277E+01_JPRB, 0.10245E+01_JPRB, 0.99982E+00_JPRB ,&
&  0.10018E+01_JPRB, 0.10070E+01_JPRB, 0.10151E+01_JPRB, 0.10288E+01_JPRB, 0.10280E+01_JPRB, 0.94854E+00_JPRB, 0.89556E+00_JPRB ,&
&  0.86654E+00_JPRB, 0.85345E+00_JPRB, 0.81941E+00_JPRB, 0.85864E+00_JPRB, 0.86223E+00_JPRB, 0.86894E+00_JPRB, 0.90128E+00_JPRB ,&
&  0.91319E+00_JPRB, 0.91378E+00_JPRB, 0.86113E+00_JPRB, 0.80315E+00_JPRB, 0.76668E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.79377E+00_JPRB, 0.74424E+00_JPRB, 0.71084E+00_JPRB, 0.72008E+00_JPRB, 0.76969E+00_JPRB, 0.82063E+00_JPRB, 0.81789E+00_JPRB ,&
&  0.80302E+00_JPRB, 0.82463E+00_JPRB, 0.89648E+00_JPRB, 0.95970E+00_JPRB, 0.96970E+00_JPRB, 0.97852E+00_JPRB, 0.10015E+01_JPRB ,&
&  0.95520E+00_JPRB, 0.92489E+00_JPRB, 0.91074E+00_JPRB, 0.89114E+00_JPRB, 0.89097E+00_JPRB, 0.88689E+00_JPRB, 0.89369E+00_JPRB ,&
&  0.92188E+00_JPRB, 0.96558E+00_JPRB, 0.10543E+01_JPRB, 0.11914E+01_JPRB, 0.14068E+01_JPRB, 0.18374E+01_JPRB, 0.25004E+01_JPRB ,&
&  0.35652E+01_JPRB, 0.23541E+01_JPRB, 0.87046E+00_JPRB, 0.11787E+01_JPRB, 0.13083E+01_JPRB, 0.13920E+01_JPRB, 0.12604E+01_JPRB ,&
&  0.13404E+01_JPRB, 0.14744E+01_JPRB, 0.16655E+01_JPRB, 0.21605E+01_JPRB, 0.24904E+01_JPRB, 0.27359E+01_JPRB, 0.28965E+01_JPRB ,&
&  0.29632E+01_JPRB, 0.30771E+01_JPRB, 0.31809E+01_JPRB, 0.32748E+01_JPRB, 0.33885E+01_JPRB, 0.35608E+01_JPRB, 0.38452E+01_JPRB ,&
&  0.43762E+01_JPRB, 0.50360E+01_JPRB, 0.55918E+01_JPRB, 0.48791E+01_JPRB, 0.48900E+01_JPRB, 0.37325E+01_JPRB, 0.30495E+01_JPRB ,&
&  0.25997E+01_JPRB, 0.25048E+01_JPRB, 0.23332E+01_JPRB, 0.19069E+01_JPRB, 0.15152E+01_JPRB, 0.12726E+01_JPRB, 0.12474E+01_JPRB ,&
&  0.10500E+01_JPRB, 0.95267E+00_JPRB, 0.93938E+00_JPRB, 0.97420E+00_JPRB, 0.98966E+00_JPRB, 0.99877E+00_JPRB, 0.99980E+00_JPRB ,&
&  0.10206E+01_JPRB, 0.10218E+01_JPRB, 0.10675E+01_JPRB, 0.10974E+01_JPRB, 0.11187E+01_JPRB, 0.11567E+01_JPRB, 0.11499E+01_JPRB ,&
&  0.11309E+01_JPRB, 0.11193E+01_JPRB, 0.11203E+01_JPRB, 0.11285E+01_JPRB, 0.10711E+01_JPRB, 0.98775E+00_JPRB, 0.98031E+00_JPRB ,&
&  0.97026E+00_JPRB, 0.96309E+00_JPRB, 0.92482E+00_JPRB, 0.93137E+00_JPRB, 0.93702E+00_JPRB, 0.99496E+00_JPRB, 0.10656E+01_JPRB ,&
&  0.10956E+01_JPRB, 0.10644E+01_JPRB, 0.99110E+00_JPRB, 0.85976E+00_JPRB, 0.84400E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.85727E+00_JPRB, 0.84063E+00_JPRB, 0.81053E+00_JPRB, 0.78970E+00_JPRB, 0.81268E+00_JPRB, 0.84805E+00_JPRB, 0.84719E+00_JPRB ,&
&  0.82172E+00_JPRB, 0.80655E+00_JPRB, 0.84315E+00_JPRB, 0.88283E+00_JPRB, 0.88120E+00_JPRB, 0.90066E+00_JPRB, 0.91156E+00_JPRB ,&
&  0.94675E+00_JPRB, 0.94199E+00_JPRB, 0.92244E+00_JPRB, 0.89624E+00_JPRB, 0.89360E+00_JPRB, 0.88044E+00_JPRB, 0.88052E+00_JPRB ,&
&  0.89366E+00_JPRB, 0.91660E+00_JPRB, 0.96394E+00_JPRB, 0.10391E+01_JPRB, 0.11294E+01_JPRB, 0.13168E+01_JPRB, 0.17020E+01_JPRB ,&
&  0.21739E+01_JPRB, 0.28087E+01_JPRB, 0.10012E+01_JPRB, 0.13176E+01_JPRB, 0.16141E+01_JPRB, 0.15684E+01_JPRB, 0.14325E+01_JPRB ,&
&  0.13794E+01_JPRB, 0.13219E+01_JPRB, 0.13977E+01_JPRB, 0.17087E+01_JPRB, 0.20292E+01_JPRB, 0.22455E+01_JPRB, 0.22485E+01_JPRB ,&
&  0.23116E+01_JPRB, 0.24401E+01_JPRB, 0.26525E+01_JPRB, 0.27108E+01_JPRB, 0.27146E+01_JPRB, 0.26998E+01_JPRB, 0.28155E+01_JPRB ,&
&  0.31847E+01_JPRB, 0.38310E+01_JPRB, 0.47848E+01_JPRB, 0.44833E+01_JPRB, 0.44825E+01_JPRB, 0.38874E+01_JPRB, 0.34013E+01_JPRB ,&
&  0.30137E+01_JPRB, 0.29577E+01_JPRB, 0.27448E+01_JPRB, 0.21992E+01_JPRB, 0.16932E+01_JPRB, 0.14695E+01_JPRB, 0.14131E+01_JPRB ,&
&  0.12172E+01_JPRB, 0.10872E+01_JPRB, 0.10722E+01_JPRB, 0.10710E+01_JPRB, 0.11056E+01_JPRB, 0.10875E+01_JPRB, 0.10594E+01_JPRB ,&
&  0.10789E+01_JPRB, 0.10547E+01_JPRB, 0.10771E+01_JPRB, 0.11166E+01_JPRB, 0.11562E+01_JPRB, 0.12135E+01_JPRB, 0.12383E+01_JPRB ,&
&  0.12280E+01_JPRB, 0.12241E+01_JPRB, 0.12051E+01_JPRB, 0.11477E+01_JPRB, 0.10714E+01_JPRB, 0.10310E+01_JPRB, 0.10239E+01_JPRB ,&
&  0.10339E+01_JPRB, 0.10404E+01_JPRB, 0.10061E+01_JPRB, 0.95876E+00_JPRB, 0.99270E+00_JPRB, 0.10043E+01_JPRB, 0.10905E+01_JPRB ,&
&  0.11962E+01_JPRB, 0.12615E+01_JPRB, 0.12097E+01_JPRB, 0.93249E+00_JPRB, 0.91589E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.95877E+00_JPRB, 0.91864E+00_JPRB, 0.89363E+00_JPRB, 0.86832E+00_JPRB, 0.84671E+00_JPRB, 0.87912E+00_JPRB, 0.87219E+00_JPRB ,&
&  0.84916E+00_JPRB, 0.81757E+00_JPRB, 0.83452E+00_JPRB, 0.88377E+00_JPRB, 0.88805E+00_JPRB, 0.86972E+00_JPRB, 0.85687E+00_JPRB ,&
&  0.85632E+00_JPRB, 0.87251E+00_JPRB, 0.87571E+00_JPRB, 0.88952E+00_JPRB, 0.89243E+00_JPRB, 0.88358E+00_JPRB, 0.87880E+00_JPRB ,&
&  0.89560E+00_JPRB, 0.90989E+00_JPRB, 0.93021E+00_JPRB, 0.95121E+00_JPRB, 0.98551E+00_JPRB, 0.10251E+01_JPRB, 0.12092E+01_JPRB ,&
&  0.14628E+01_JPRB, 0.23316E+01_JPRB, 0.19035E+01_JPRB, 0.22615E+01_JPRB, 0.21385E+01_JPRB, 0.19529E+01_JPRB, 0.17023E+01_JPRB ,&
&  0.17282E+01_JPRB, 0.15955E+01_JPRB, 0.16077E+01_JPRB, 0.14308E+01_JPRB, 0.14958E+01_JPRB, 0.16293E+01_JPRB, 0.16186E+01_JPRB ,&
&  0.15651E+01_JPRB, 0.15501E+01_JPRB, 0.18220E+01_JPRB, 0.19601E+01_JPRB, 0.20245E+01_JPRB, 0.20453E+01_JPRB, 0.20795E+01_JPRB ,&
&  0.22722E+01_JPRB, 0.26225E+01_JPRB, 0.35292E+01_JPRB, 0.42269E+01_JPRB, 0.41269E+01_JPRB, 0.40310E+01_JPRB, 0.39327E+01_JPRB ,&
&  0.39620E+01_JPRB, 0.35303E+01_JPRB, 0.30981E+01_JPRB, 0.24827E+01_JPRB, 0.19140E+01_JPRB, 0.18195E+01_JPRB, 0.15467E+01_JPRB ,&
&  0.13450E+01_JPRB, 0.12138E+01_JPRB, 0.11931E+01_JPRB, 0.11888E+01_JPRB, 0.12030E+01_JPRB, 0.11676E+01_JPRB, 0.11164E+01_JPRB ,&
&  0.10990E+01_JPRB, 0.11037E+01_JPRB, 0.10856E+01_JPRB, 0.11106E+01_JPRB, 0.11333E+01_JPRB, 0.11709E+01_JPRB, 0.12030E+01_JPRB ,&
&  0.12369E+01_JPRB, 0.12574E+01_JPRB, 0.12061E+01_JPRB, 0.11559E+01_JPRB, 0.10966E+01_JPRB, 0.10609E+01_JPRB, 0.10319E+01_JPRB ,&
&  0.10461E+01_JPRB, 0.10714E+01_JPRB, 0.10663E+01_JPRB, 0.10231E+01_JPRB, 0.99752E+00_JPRB, 0.98703E+00_JPRB, 0.99091E+00_JPRB ,&
&  0.10114E+01_JPRB, 0.10661E+01_JPRB, 0.10570E+01_JPRB, 0.10021E+01_JPRB, 0.98664E+00_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.10230E+01_JPRB, 0.10024E+01_JPRB, 0.95510E+00_JPRB, 0.94247E+00_JPRB, 0.90660E+00_JPRB, 0.92992E+00_JPRB, 0.93521E+00_JPRB ,&
&  0.90265E+00_JPRB, 0.87060E+00_JPRB, 0.84764E+00_JPRB, 0.86843E+00_JPRB, 0.89170E+00_JPRB, 0.88043E+00_JPRB, 0.84246E+00_JPRB ,&
&  0.82945E+00_JPRB, 0.82593E+00_JPRB, 0.82417E+00_JPRB, 0.82889E+00_JPRB, 0.84662E+00_JPRB, 0.87253E+00_JPRB, 0.87225E+00_JPRB ,&
&  0.86972E+00_JPRB, 0.90332E+00_JPRB, 0.93706E+00_JPRB, 0.94164E+00_JPRB, 0.94089E+00_JPRB, 0.94440E+00_JPRB, 0.10272E+01_JPRB ,&
&  0.11552E+01_JPRB, 0.24067E+01_JPRB, 0.17123E+01_JPRB, 0.29115E+01_JPRB, 0.23267E+01_JPRB, 0.20668E+01_JPRB, 0.16854E+01_JPRB ,&
&  0.15518E+01_JPRB, 0.16512E+01_JPRB, 0.14301E+01_JPRB, 0.12721E+01_JPRB, 0.11968E+01_JPRB, 0.12017E+01_JPRB, 0.11702E+01_JPRB ,&
&  0.10667E+01_JPRB, 0.10675E+01_JPRB, 0.11599E+01_JPRB, 0.12973E+01_JPRB, 0.13974E+01_JPRB, 0.14546E+01_JPRB, 0.15358E+01_JPRB ,&
&  0.16653E+01_JPRB, 0.18337E+01_JPRB, 0.24319E+01_JPRB, 0.32268E+01_JPRB, 0.39620E+01_JPRB, 0.42089E+01_JPRB, 0.45618E+01_JPRB ,&
&  0.46900E+01_JPRB, 0.42697E+01_JPRB, 0.34990E+01_JPRB, 0.27564E+01_JPRB, 0.22376E+01_JPRB, 0.20492E+01_JPRB, 0.18260E+01_JPRB ,&
&  0.16109E+01_JPRB, 0.14138E+01_JPRB, 0.13640E+01_JPRB, 0.13201E+01_JPRB, 0.13253E+01_JPRB, 0.12846E+01_JPRB, 0.11961E+01_JPRB ,&
&  0.11272E+01_JPRB, 0.11126E+01_JPRB, 0.11240E+01_JPRB, 0.11403E+01_JPRB, 0.11642E+01_JPRB, 0.11664E+01_JPRB, 0.11695E+01_JPRB ,&
&  0.12004E+01_JPRB, 0.12395E+01_JPRB, 0.12105E+01_JPRB, 0.11977E+01_JPRB, 0.11509E+01_JPRB, 0.11128E+01_JPRB, 0.10950E+01_JPRB ,&
&  0.11258E+01_JPRB, 0.11194E+01_JPRB, 0.10773E+01_JPRB, 0.10258E+01_JPRB, 0.99942E+00_JPRB, 0.10064E+01_JPRB, 0.97166E+00_JPRB ,&
&  0.97068E+00_JPRB, 0.10115E+01_JPRB, 0.10588E+01_JPRB, 0.10237E+01_JPRB, 0.10205E+01_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.10470E+01_JPRB, 0.10437E+01_JPRB, 0.98435E+00_JPRB, 0.96445E+00_JPRB, 0.93386E+00_JPRB, 0.92366E+00_JPRB, 0.93385E+00_JPRB ,&
&  0.92550E+00_JPRB, 0.90145E+00_JPRB, 0.87082E+00_JPRB, 0.85763E+00_JPRB, 0.87067E+00_JPRB, 0.88422E+00_JPRB, 0.85197E+00_JPRB ,&
&  0.81477E+00_JPRB, 0.80074E+00_JPRB, 0.79209E+00_JPRB, 0.79455E+00_JPRB, 0.80704E+00_JPRB, 0.82465E+00_JPRB, 0.83577E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.87368E+00_JPRB, 0.92182E+00_JPRB, 0.94288E+00_JPRB, 0.93268E+00_JPRB, 0.93668E+00_JPRB, 0.99194E+00_JPRB ,&
&  0.11729E+01_JPRB, 0.30634E+01_JPRB, 0.18604E+01_JPRB, 0.30035E+01_JPRB, 0.23856E+01_JPRB, 0.20453E+01_JPRB, 0.16434E+01_JPRB ,&
&  0.15203E+01_JPRB, 0.14794E+01_JPRB, 0.13419E+01_JPRB, 0.12201E+01_JPRB, 0.11389E+01_JPRB, 0.10855E+01_JPRB, 0.10416E+01_JPRB ,&
&  0.99191E+00_JPRB, 0.96198E+00_JPRB, 0.96836E+00_JPRB, 0.10427E+01_JPRB, 0.11030E+01_JPRB, 0.11434E+01_JPRB, 0.11906E+01_JPRB ,&
&  0.12712E+01_JPRB, 0.13845E+01_JPRB, 0.17414E+01_JPRB, 0.27366E+01_JPRB, 0.35250E+01_JPRB, 0.42409E+01_JPRB, 0.61930E+01_JPRB ,&
&  0.59329E+01_JPRB, 0.54001E+01_JPRB, 0.39354E+01_JPRB, 0.30771E+01_JPRB, 0.25186E+01_JPRB, 0.22013E+01_JPRB, 0.20577E+01_JPRB ,&
&  0.19082E+01_JPRB, 0.16972E+01_JPRB, 0.16023E+01_JPRB, 0.15352E+01_JPRB, 0.14642E+01_JPRB, 0.13865E+01_JPRB, 0.12978E+01_JPRB ,&
&  0.12285E+01_JPRB, 0.11801E+01_JPRB, 0.11863E+01_JPRB, 0.11957E+01_JPRB, 0.12036E+01_JPRB, 0.11866E+01_JPRB, 0.11750E+01_JPRB ,&
&  0.11849E+01_JPRB, 0.12166E+01_JPRB, 0.12017E+01_JPRB, 0.12063E+01_JPRB, 0.11905E+01_JPRB, 0.11593E+01_JPRB, 0.11388E+01_JPRB ,&
&  0.11823E+01_JPRB, 0.11787E+01_JPRB, 0.11109E+01_JPRB, 0.10700E+01_JPRB, 0.10229E+01_JPRB, 0.10410E+01_JPRB, 0.98941E+00_JPRB ,&
&  0.96651E+00_JPRB, 0.96085E+00_JPRB, 0.10262E+01_JPRB, 0.10330E+01_JPRB, 0.10493E+01_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.10579E+01_JPRB, 0.10422E+01_JPRB, 0.10120E+01_JPRB, 0.96424E+00_JPRB, 0.94522E+00_JPRB, 0.92432E+00_JPRB, 0.91305E+00_JPRB ,&
&  0.90953E+00_JPRB, 0.89657E+00_JPRB, 0.88313E+00_JPRB, 0.86288E+00_JPRB, 0.85487E+00_JPRB, 0.86676E+00_JPRB, 0.86133E+00_JPRB ,&
&  0.84307E+00_JPRB, 0.81902E+00_JPRB, 0.81019E+00_JPRB, 0.80035E+00_JPRB, 0.79001E+00_JPRB, 0.79065E+00_JPRB, 0.80182E+00_JPRB ,&
&  0.80875E+00_JPRB, 0.82829E+00_JPRB, 0.86044E+00_JPRB, 0.89012E+00_JPRB, 0.89146E+00_JPRB, 0.89445E+00_JPRB, 0.96561E+00_JPRB ,&
&  0.12124E+01_JPRB, 0.12408E+01_JPRB, 0.25321E+01_JPRB, 0.28878E+01_JPRB, 0.24112E+01_JPRB, 0.20916E+01_JPRB, 0.17732E+01_JPRB ,&
&  0.16121E+01_JPRB, 0.14081E+01_JPRB, 0.12463E+01_JPRB, 0.11343E+01_JPRB, 0.10763E+01_JPRB, 0.10087E+01_JPRB, 0.98433E+00_JPRB ,&
&  0.97005E+00_JPRB, 0.94661E+00_JPRB, 0.94040E+00_JPRB, 0.96061E+00_JPRB, 0.97316E+00_JPRB, 0.98903E+00_JPRB, 0.10154E+01_JPRB ,&
&  0.10705E+01_JPRB, 0.11418E+01_JPRB, 0.13244E+01_JPRB, 0.19674E+01_JPRB, 0.32059E+01_JPRB, 0.44154E+01_JPRB, 0.61689E+01_JPRB ,&
&  0.69832E+01_JPRB, 0.63074E+01_JPRB, 0.43894E+01_JPRB, 0.33380E+01_JPRB, 0.27729E+01_JPRB, 0.24001E+01_JPRB, 0.22217E+01_JPRB ,&
&  0.20987E+01_JPRB, 0.19363E+01_JPRB, 0.18081E+01_JPRB, 0.17816E+01_JPRB, 0.16540E+01_JPRB, 0.15157E+01_JPRB, 0.13970E+01_JPRB ,&
&  0.13320E+01_JPRB, 0.12812E+01_JPRB, 0.12304E+01_JPRB, 0.11961E+01_JPRB, 0.11993E+01_JPRB, 0.11910E+01_JPRB, 0.11736E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.11850E+01_JPRB, 0.11790E+01_JPRB, 0.11771E+01_JPRB, 0.11994E+01_JPRB, 0.12026E+01_JPRB, 0.12024E+01_JPRB ,&
&  0.11923E+01_JPRB, 0.11960E+01_JPRB, 0.11508E+01_JPRB, 0.11176E+01_JPRB, 0.10461E+01_JPRB, 0.10814E+01_JPRB, 0.10255E+01_JPRB ,&
&  0.98471E+00_JPRB, 0.95479E+00_JPRB, 0.96688E+00_JPRB, 0.10180E+01_JPRB, 0.10559E+01_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.10336E+01_JPRB, 0.10214E+01_JPRB, 0.10023E+01_JPRB, 0.97533E+00_JPRB, 0.95165E+00_JPRB, 0.93915E+00_JPRB, 0.92619E+00_JPRB ,&
&  0.92639E+00_JPRB, 0.92755E+00_JPRB, 0.91837E+00_JPRB, 0.89248E+00_JPRB, 0.87085E+00_JPRB, 0.86635E+00_JPRB, 0.87088E+00_JPRB ,&
&  0.86456E+00_JPRB, 0.85347E+00_JPRB, 0.85627E+00_JPRB, 0.83881E+00_JPRB, 0.81698E+00_JPRB, 0.80314E+00_JPRB, 0.79866E+00_JPRB ,&
&  0.80436E+00_JPRB, 0.81561E+00_JPRB, 0.84233E+00_JPRB, 0.86828E+00_JPRB, 0.87157E+00_JPRB, 0.86592E+00_JPRB, 0.86710E+00_JPRB ,&
&  0.10949E+01_JPRB, 0.17383E+01_JPRB, 0.27505E+01_JPRB, 0.26133E+01_JPRB, 0.22752E+01_JPRB, 0.19359E+01_JPRB, 0.17685E+01_JPRB ,&
&  0.15307E+01_JPRB, 0.13856E+01_JPRB, 0.12415E+01_JPRB, 0.11181E+01_JPRB, 0.10330E+01_JPRB, 0.98432E+00_JPRB, 0.95978E+00_JPRB ,&
&  0.93187E+00_JPRB, 0.91387E+00_JPRB, 0.91178E+00_JPRB, 0.91460E+00_JPRB, 0.91768E+00_JPRB, 0.91393E+00_JPRB, 0.91954E+00_JPRB ,&
&  0.95994E+00_JPRB, 0.10197E+01_JPRB, 0.11175E+01_JPRB, 0.14537E+01_JPRB, 0.26629E+01_JPRB, 0.33529E+01_JPRB, 0.52621E+01_JPRB ,&
&  0.71941E+01_JPRB, 0.59465E+01_JPRB, 0.45103E+01_JPRB, 0.35525E+01_JPRB, 0.29654E+01_JPRB, 0.25948E+01_JPRB, 0.23755E+01_JPRB ,&
&  0.21586E+01_JPRB, 0.20268E+01_JPRB, 0.19064E+01_JPRB, 0.17818E+01_JPRB, 0.16928E+01_JPRB, 0.15687E+01_JPRB, 0.14574E+01_JPRB ,&
&  0.14149E+01_JPRB, 0.13551E+01_JPRB, 0.12895E+01_JPRB, 0.12181E+01_JPRB, 0.11931E+01_JPRB, 0.11810E+01_JPRB, 0.11671E+01_JPRB ,&
&  0.11646E+01_JPRB, 0.11613E+01_JPRB, 0.11535E+01_JPRB, 0.11558E+01_JPRB, 0.11928E+01_JPRB, 0.12251E+01_JPRB, 0.12187E+01_JPRB ,&
&  0.11716E+01_JPRB, 0.11591E+01_JPRB, 0.11743E+01_JPRB, 0.11281E+01_JPRB, 0.10848E+01_JPRB, 0.11489E+01_JPRB, 0.10776E+01_JPRB ,&
&  0.10115E+01_JPRB, 0.97564E+00_JPRB, 0.95309E+00_JPRB, 0.96391E+00_JPRB, 0.10274E+01_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.10258E+01_JPRB, 0.10193E+01_JPRB, 0.10020E+01_JPRB, 0.98689E+00_JPRB, 0.98127E+00_JPRB, 0.96692E+00_JPRB, 0.95663E+00_JPRB ,&
&  0.95165E+00_JPRB, 0.94059E+00_JPRB, 0.93208E+00_JPRB, 0.91779E+00_JPRB, 0.89923E+00_JPRB, 0.88309E+00_JPRB, 0.87722E+00_JPRB ,&
&  0.87694E+00_JPRB, 0.86767E+00_JPRB, 0.87168E+00_JPRB, 0.87312E+00_JPRB, 0.86988E+00_JPRB, 0.85785E+00_JPRB, 0.84578E+00_JPRB ,&
&  0.83842E+00_JPRB, 0.82450E+00_JPRB, 0.82119E+00_JPRB, 0.84242E+00_JPRB, 0.86126E+00_JPRB, 0.86462E+00_JPRB, 0.85625E+00_JPRB ,&
&  0.95349E+00_JPRB, 0.14349E+01_JPRB, 0.20576E+01_JPRB, 0.22109E+01_JPRB, 0.20761E+01_JPRB, 0.18577E+01_JPRB, 0.16371E+01_JPRB ,&
&  0.14478E+01_JPRB, 0.12969E+01_JPRB, 0.12054E+01_JPRB, 0.11320E+01_JPRB, 0.10620E+01_JPRB, 0.10047E+01_JPRB, 0.96154E+00_JPRB ,&
&  0.91930E+00_JPRB, 0.90016E+00_JPRB, 0.88877E+00_JPRB, 0.88458E+00_JPRB, 0.88113E+00_JPRB, 0.87800E+00_JPRB, 0.88596E+00_JPRB ,&
&  0.91099E+00_JPRB, 0.95019E+00_JPRB, 0.10049E+01_JPRB, 0.11171E+01_JPRB, 0.16638E+01_JPRB, 0.25925E+01_JPRB, 0.38303E+01_JPRB ,&
&  0.41901E+01_JPRB, 0.42712E+01_JPRB, 0.38607E+01_JPRB, 0.32692E+01_JPRB, 0.27880E+01_JPRB, 0.24491E+01_JPRB, 0.22299E+01_JPRB ,&
&  0.20230E+01_JPRB, 0.18774E+01_JPRB, 0.17730E+01_JPRB, 0.16406E+01_JPRB, 0.15080E+01_JPRB, 0.14279E+01_JPRB, 0.14258E+01_JPRB ,&
&  0.14449E+01_JPRB, 0.14111E+01_JPRB, 0.13376E+01_JPRB, 0.12493E+01_JPRB, 0.11639E+01_JPRB, 0.11265E+01_JPRB, 0.11125E+01_JPRB ,&
&  0.11010E+01_JPRB, 0.11075E+01_JPRB, 0.11308E+01_JPRB, 0.11582E+01_JPRB, 0.11754E+01_JPRB, 0.11810E+01_JPRB, 0.11771E+01_JPRB ,&
&  0.11879E+01_JPRB, 0.11749E+01_JPRB, 0.11588E+01_JPRB, 0.11480E+01_JPRB, 0.11122E+01_JPRB, 0.11680E+01_JPRB, 0.11177E+01_JPRB ,&
&  0.10477E+01_JPRB, 0.99852E+00_JPRB, 0.97324E+00_JPRB, 0.95948E+00_JPRB, 0.10067E+01_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.10350E+01_JPRB, 0.10284E+01_JPRB, 0.10112E+01_JPRB, 0.99777E+00_JPRB, 0.98003E+00_JPRB, 0.96362E+00_JPRB, 0.94956E+00_JPRB ,&
&  0.94252E+00_JPRB, 0.92666E+00_JPRB, 0.91856E+00_JPRB, 0.91034E+00_JPRB, 0.90644E+00_JPRB, 0.89686E+00_JPRB, 0.88235E+00_JPRB ,&
&  0.86716E+00_JPRB, 0.85866E+00_JPRB, 0.85604E+00_JPRB, 0.86010E+00_JPRB, 0.87002E+00_JPRB, 0.87654E+00_JPRB, 0.87154E+00_JPRB ,&
&  0.86202E+00_JPRB, 0.84690E+00_JPRB, 0.82868E+00_JPRB, 0.84326E+00_JPRB, 0.85964E+00_JPRB, 0.86473E+00_JPRB, 0.85252E+00_JPRB ,&
&  0.90088E+00_JPRB, 0.10836E+01_JPRB, 0.14660E+01_JPRB, 0.15924E+01_JPRB, 0.15986E+01_JPRB, 0.16202E+01_JPRB, 0.15070E+01_JPRB ,&
&  0.13905E+01_JPRB, 0.12540E+01_JPRB, 0.11401E+01_JPRB, 0.10812E+01_JPRB, 0.10438E+01_JPRB, 0.99537E+00_JPRB, 0.95601E+00_JPRB ,&
&  0.90664E+00_JPRB, 0.88172E+00_JPRB, 0.86548E+00_JPRB, 0.85906E+00_JPRB, 0.85587E+00_JPRB, 0.85653E+00_JPRB, 0.86314E+00_JPRB ,&
&  0.87446E+00_JPRB, 0.89299E+00_JPRB, 0.92465E+00_JPRB, 0.97738E+00_JPRB, 0.10972E+01_JPRB, 0.14819E+01_JPRB, 0.20851E+01_JPRB ,&
&  0.24985E+01_JPRB, 0.28082E+01_JPRB, 0.30194E+01_JPRB, 0.29919E+01_JPRB, 0.27149E+01_JPRB, 0.23909E+01_JPRB, 0.21496E+01_JPRB ,&
&  0.19494E+01_JPRB, 0.17574E+01_JPRB, 0.15967E+01_JPRB, 0.14620E+01_JPRB, 0.13430E+01_JPRB, 0.12926E+01_JPRB, 0.13374E+01_JPRB ,&
&  0.13656E+01_JPRB, 0.13492E+01_JPRB, 0.12887E+01_JPRB, 0.12157E+01_JPRB, 0.11314E+01_JPRB, 0.10816E+01_JPRB, 0.10669E+01_JPRB ,&
&  0.10662E+01_JPRB, 0.10775E+01_JPRB, 0.10952E+01_JPRB, 0.11178E+01_JPRB, 0.11381E+01_JPRB, 0.11489E+01_JPRB, 0.11507E+01_JPRB ,&
&  0.11583E+01_JPRB, 0.11599E+01_JPRB, 0.11013E+01_JPRB, 0.10794E+01_JPRB, 0.11600E+01_JPRB, 0.11423E+01_JPRB, 0.11194E+01_JPRB ,&
&  0.10748E+01_JPRB, 0.10127E+01_JPRB, 0.97006E+00_JPRB, 0.94000E+00_JPRB, 0.99268E+00_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.90036E+00_JPRB, 0.92701E+00_JPRB, 0.93681E+00_JPRB, 0.93880E+00_JPRB, 0.92888E+00_JPRB, 0.91262E+00_JPRB, 0.89895E+00_JPRB ,&
&  0.89341E+00_JPRB, 0.87956E+00_JPRB, 0.87369E+00_JPRB, 0.87127E+00_JPRB, 0.87444E+00_JPRB, 0.88144E+00_JPRB, 0.88096E+00_JPRB ,&
&  0.87115E+00_JPRB, 0.85672E+00_JPRB, 0.84709E+00_JPRB, 0.84654E+00_JPRB, 0.85632E+00_JPRB, 0.86704E+00_JPRB, 0.87397E+00_JPRB ,&
&  0.87645E+00_JPRB, 0.86937E+00_JPRB, 0.86529E+00_JPRB, 0.87471E+00_JPRB, 0.88312E+00_JPRB, 0.87543E+00_JPRB, 0.85627E+00_JPRB ,&
&  0.86820E+00_JPRB, 0.86383E+00_JPRB, 0.10096E+01_JPRB, 0.11532E+01_JPRB, 0.12714E+01_JPRB, 0.13170E+01_JPRB, 0.13032E+01_JPRB ,&
&  0.12419E+01_JPRB, 0.11754E+01_JPRB, 0.10964E+01_JPRB, 0.10305E+01_JPRB, 0.98180E+00_JPRB, 0.94965E+00_JPRB, 0.92899E+00_JPRB ,&
&  0.89915E+00_JPRB, 0.87047E+00_JPRB, 0.84569E+00_JPRB, 0.83180E+00_JPRB, 0.82671E+00_JPRB, 0.82225E+00_JPRB, 0.81822E+00_JPRB ,&
&  0.81435E+00_JPRB, 0.81617E+00_JPRB, 0.83519E+00_JPRB, 0.87202E+00_JPRB, 0.92907E+00_JPRB, 0.10738E+01_JPRB, 0.13419E+01_JPRB ,&
&  0.15950E+01_JPRB, 0.18624E+01_JPRB, 0.21253E+01_JPRB, 0.23298E+01_JPRB, 0.23307E+01_JPRB, 0.22063E+01_JPRB, 0.20330E+01_JPRB ,&
&  0.18302E+01_JPRB, 0.15883E+01_JPRB, 0.13730E+01_JPRB, 0.12475E+01_JPRB, 0.12048E+01_JPRB, 0.12134E+01_JPRB, 0.12584E+01_JPRB ,&
&  0.12776E+01_JPRB, 0.12581E+01_JPRB, 0.11931E+01_JPRB, 0.11273E+01_JPRB, 0.10789E+01_JPRB, 0.10371E+01_JPRB, 0.10229E+01_JPRB ,&
&  0.10172E+01_JPRB, 0.10224E+01_JPRB, 0.10377E+01_JPRB, 0.10535E+01_JPRB, 0.10760E+01_JPRB, 0.10906E+01_JPRB, 0.10919E+01_JPRB ,&
&  0.10886E+01_JPRB, 0.10816E+01_JPRB, 0.10749E+01_JPRB, 0.10899E+01_JPRB, 0.10731E+01_JPRB, 0.10622E+01_JPRB, 0.10492E+01_JPRB ,&
&  0.10246E+01_JPRB, 0.98249E+00_JPRB, 0.93688E+00_JPRB, 0.90935E+00_JPRB, 0.84565E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.84027E+00_JPRB, 0.86540E+00_JPRB, 0.87705E+00_JPRB, 0.87990E+00_JPRB, 0.87864E+00_JPRB, 0.86703E+00_JPRB, 0.85270E+00_JPRB ,&
&  0.83562E+00_JPRB, 0.82910E+00_JPRB, 0.82716E+00_JPRB, 0.83203E+00_JPRB, 0.83799E+00_JPRB, 0.84658E+00_JPRB, 0.85861E+00_JPRB ,&
&  0.87009E+00_JPRB, 0.87537E+00_JPRB, 0.87942E+00_JPRB, 0.87738E+00_JPRB, 0.87548E+00_JPRB, 0.87756E+00_JPRB, 0.88151E+00_JPRB ,&
&  0.89000E+00_JPRB, 0.89240E+00_JPRB, 0.88829E+00_JPRB, 0.89651E+00_JPRB, 0.91008E+00_JPRB, 0.89994E+00_JPRB, 0.88134E+00_JPRB ,&
&  0.87436E+00_JPRB, 0.76306E+00_JPRB, 0.79672E+00_JPRB, 0.92910E+00_JPRB, 0.10408E+01_JPRB, 0.11448E+01_JPRB, 0.11706E+01_JPRB ,&
&  0.11423E+01_JPRB, 0.10884E+01_JPRB, 0.10344E+01_JPRB, 0.98317E+00_JPRB, 0.94722E+00_JPRB, 0.91036E+00_JPRB, 0.88502E+00_JPRB ,&
&  0.86834E+00_JPRB, 0.85019E+00_JPRB, 0.82434E+00_JPRB, 0.80281E+00_JPRB, 0.78521E+00_JPRB, 0.76784E+00_JPRB, 0.75812E+00_JPRB ,&
&  0.75837E+00_JPRB, 0.77004E+00_JPRB, 0.78776E+00_JPRB, 0.81557E+00_JPRB, 0.86451E+00_JPRB, 0.93555E+00_JPRB, 0.10543E+01_JPRB ,&
&  0.11922E+01_JPRB, 0.13448E+01_JPRB, 0.14935E+01_JPRB, 0.16263E+01_JPRB, 0.17380E+01_JPRB, 0.17546E+01_JPRB, 0.16663E+01_JPRB ,&
&  0.15058E+01_JPRB, 0.13680E+01_JPRB, 0.12615E+01_JPRB, 0.11903E+01_JPRB, 0.11754E+01_JPRB, 0.11924E+01_JPRB, 0.12038E+01_JPRB ,&
&  0.11859E+01_JPRB, 0.11389E+01_JPRB, 0.10772E+01_JPRB, 0.10208E+01_JPRB, 0.98761E+00_JPRB, 0.96529E+00_JPRB, 0.94409E+00_JPRB ,&
&  0.92697E+00_JPRB, 0.92016E+00_JPRB, 0.92465E+00_JPRB, 0.94256E+00_JPRB, 0.96750E+00_JPRB, 0.99425E+00_JPRB, 0.10069E+01_JPRB ,&
&  0.10132E+01_JPRB, 0.10184E+01_JPRB, 0.10316E+01_JPRB, 0.97313E+00_JPRB, 0.10054E+01_JPRB, 0.10095E+01_JPRB, 0.10055E+01_JPRB ,&
&  0.98552E+00_JPRB, 0.95622E+00_JPRB, 0.91685E+00_JPRB, 0.76461E+00_JPRB, 0.81441E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.81605E+00_JPRB, 0.82337E+00_JPRB, 0.82945E+00_JPRB, 0.82697E+00_JPRB, 0.82400E+00_JPRB, 0.81723E+00_JPRB, 0.80263E+00_JPRB ,&
&  0.78871E+00_JPRB, 0.78378E+00_JPRB, 0.78469E+00_JPRB, 0.78674E+00_JPRB, 0.79027E+00_JPRB, 0.79670E+00_JPRB, 0.80805E+00_JPRB ,&
&  0.82164E+00_JPRB, 0.83573E+00_JPRB, 0.85221E+00_JPRB, 0.86721E+00_JPRB, 0.87003E+00_JPRB, 0.86603E+00_JPRB, 0.86534E+00_JPRB ,&
&  0.87074E+00_JPRB, 0.86918E+00_JPRB, 0.86186E+00_JPRB, 0.86358E+00_JPRB, 0.87231E+00_JPRB, 0.86408E+00_JPRB, 0.84644E+00_JPRB ,&
&  0.83696E+00_JPRB, 0.69949E+00_JPRB, 0.74063E+00_JPRB, 0.81406E+00_JPRB, 0.90128E+00_JPRB, 0.98197E+00_JPRB, 0.10158E+01_JPRB ,&
&  0.10067E+01_JPRB, 0.96685E+00_JPRB, 0.93126E+00_JPRB, 0.88891E+00_JPRB, 0.86786E+00_JPRB, 0.85308E+00_JPRB, 0.83577E+00_JPRB ,&
&  0.81056E+00_JPRB, 0.78426E+00_JPRB, 0.75522E+00_JPRB, 0.72942E+00_JPRB, 0.70904E+00_JPRB, 0.69906E+00_JPRB, 0.69561E+00_JPRB ,&
&  0.69555E+00_JPRB, 0.70649E+00_JPRB, 0.71363E+00_JPRB, 0.72745E+00_JPRB, 0.75235E+00_JPRB, 0.79334E+00_JPRB, 0.85278E+00_JPRB ,&
&  0.93609E+00_JPRB, 0.10366E+01_JPRB, 0.11356E+01_JPRB, 0.12064E+01_JPRB, 0.12515E+01_JPRB, 0.12858E+01_JPRB, 0.12830E+01_JPRB ,&
&  0.12322E+01_JPRB, 0.11773E+01_JPRB, 0.11635E+01_JPRB, 0.11567E+01_JPRB, 0.11372E+01_JPRB, 0.11150E+01_JPRB, 0.10808E+01_JPRB ,&
&  0.10331E+01_JPRB, 0.97947E+00_JPRB, 0.93193E+00_JPRB, 0.88933E+00_JPRB, 0.86198E+00_JPRB, 0.84653E+00_JPRB, 0.82279E+00_JPRB ,&
&  0.80712E+00_JPRB, 0.80371E+00_JPRB, 0.81191E+00_JPRB, 0.83341E+00_JPRB, 0.86267E+00_JPRB, 0.88985E+00_JPRB, 0.90502E+00_JPRB ,&
&  0.90989E+00_JPRB, 0.91529E+00_JPRB, 0.91922E+00_JPRB, 0.94063E+00_JPRB, 0.94405E+00_JPRB, 0.94367E+00_JPRB, 0.93939E+00_JPRB ,&
&  0.93207E+00_JPRB, 0.92369E+00_JPRB, 0.72076E+00_JPRB, 0.78434E+00_JPRB, 0.79790E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.83107E+00_JPRB, 0.81329E+00_JPRB, 0.80414E+00_JPRB, 0.79886E+00_JPRB, 0.79490E+00_JPRB, 0.78850E+00_JPRB, 0.77558E+00_JPRB ,&
&  0.76234E+00_JPRB, 0.75572E+00_JPRB, 0.74980E+00_JPRB, 0.74210E+00_JPRB, 0.74417E+00_JPRB, 0.75074E+00_JPRB, 0.76217E+00_JPRB ,&
&  0.77451E+00_JPRB, 0.79007E+00_JPRB, 0.80485E+00_JPRB, 0.82323E+00_JPRB, 0.83494E+00_JPRB, 0.84275E+00_JPRB, 0.84649E+00_JPRB ,&
&  0.84883E+00_JPRB, 0.84877E+00_JPRB, 0.84069E+00_JPRB, 0.83022E+00_JPRB, 0.82747E+00_JPRB, 0.82945E+00_JPRB, 0.82482E+00_JPRB ,&
&  0.72648E+00_JPRB, 0.64240E+00_JPRB, 0.69999E+00_JPRB, 0.76667E+00_JPRB, 0.83298E+00_JPRB, 0.88919E+00_JPRB, 0.91457E+00_JPRB ,&
&  0.90207E+00_JPRB, 0.87049E+00_JPRB, 0.84073E+00_JPRB, 0.80577E+00_JPRB, 0.79080E+00_JPRB, 0.78445E+00_JPRB, 0.77504E+00_JPRB ,&
&  0.76482E+00_JPRB, 0.75251E+00_JPRB, 0.73647E+00_JPRB, 0.71151E+00_JPRB, 0.69362E+00_JPRB, 0.68068E+00_JPRB, 0.67786E+00_JPRB ,&
&  0.67961E+00_JPRB, 0.68331E+00_JPRB, 0.68696E+00_JPRB, 0.69048E+00_JPRB, 0.69949E+00_JPRB, 0.71588E+00_JPRB, 0.73867E+00_JPRB ,&
&  0.77536E+00_JPRB, 0.82161E+00_JPRB, 0.88006E+00_JPRB, 0.92149E+00_JPRB, 0.94587E+00_JPRB, 0.96837E+00_JPRB, 0.99858E+00_JPRB ,&
&  0.10185E+01_JPRB, 0.10143E+01_JPRB, 0.98585E+00_JPRB, 0.97044E+00_JPRB, 0.96451E+00_JPRB, 0.95431E+00_JPRB, 0.93188E+00_JPRB ,&
&  0.89518E+00_JPRB, 0.85089E+00_JPRB, 0.80561E+00_JPRB, 0.77742E+00_JPRB, 0.76928E+00_JPRB, 0.76575E+00_JPRB, 0.75798E+00_JPRB ,&
&  0.75093E+00_JPRB, 0.74926E+00_JPRB, 0.75486E+00_JPRB, 0.76683E+00_JPRB, 0.78290E+00_JPRB, 0.80206E+00_JPRB, 0.81698E+00_JPRB ,&
&  0.83065E+00_JPRB, 0.84359E+00_JPRB, 0.85315E+00_JPRB, 0.86876E+00_JPRB, 0.88475E+00_JPRB, 0.89165E+00_JPRB, 0.88602E+00_JPRB ,&
&  0.88250E+00_JPRB, 0.86985E+00_JPRB, 0.85943E+00_JPRB, 0.85427E+00_JPRB, 0.84585E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.79710E+00_JPRB, 0.77908E+00_JPRB, 0.76878E+00_JPRB, 0.76462E+00_JPRB, 0.75602E+00_JPRB, 0.73704E+00_JPRB, 0.72131E+00_JPRB ,&
&  0.70680E+00_JPRB, 0.69888E+00_JPRB, 0.69559E+00_JPRB, 0.69500E+00_JPRB, 0.70114E+00_JPRB, 0.70710E+00_JPRB, 0.71439E+00_JPRB ,&
&  0.72614E+00_JPRB, 0.74475E+00_JPRB, 0.76052E+00_JPRB, 0.77472E+00_JPRB, 0.78719E+00_JPRB, 0.79940E+00_JPRB, 0.80221E+00_JPRB ,&
&  0.80434E+00_JPRB, 0.81142E+00_JPRB, 0.81241E+00_JPRB, 0.79919E+00_JPRB, 0.78553E+00_JPRB, 0.78854E+00_JPRB, 0.78596E+00_JPRB ,&
&  0.77663E+00_JPRB, 0.66258E+00_JPRB, 0.71797E+00_JPRB, 0.74475E+00_JPRB, 0.77545E+00_JPRB, 0.80483E+00_JPRB, 0.82057E+00_JPRB ,&
&  0.81633E+00_JPRB, 0.79766E+00_JPRB, 0.77596E+00_JPRB, 0.74764E+00_JPRB, 0.73917E+00_JPRB, 0.73204E+00_JPRB, 0.72280E+00_JPRB ,&
&  0.71670E+00_JPRB, 0.71068E+00_JPRB, 0.70206E+00_JPRB, 0.69019E+00_JPRB, 0.67500E+00_JPRB, 0.66373E+00_JPRB, 0.65371E+00_JPRB ,&
&  0.65078E+00_JPRB, 0.64305E+00_JPRB, 0.63610E+00_JPRB, 0.63684E+00_JPRB, 0.64907E+00_JPRB, 0.65666E+00_JPRB, 0.66093E+00_JPRB ,&
&  0.66823E+00_JPRB, 0.68082E+00_JPRB, 0.70145E+00_JPRB, 0.72732E+00_JPRB, 0.74844E+00_JPRB, 0.76443E+00_JPRB, 0.78635E+00_JPRB ,&
&  0.81959E+00_JPRB, 0.84295E+00_JPRB, 0.84449E+00_JPRB, 0.82741E+00_JPRB, 0.81367E+00_JPRB, 0.80743E+00_JPRB, 0.78920E+00_JPRB ,&
&  0.76025E+00_JPRB, 0.73765E+00_JPRB, 0.72643E+00_JPRB, 0.72017E+00_JPRB, 0.71318E+00_JPRB, 0.70598E+00_JPRB, 0.70486E+00_JPRB ,&
&  0.70585E+00_JPRB, 0.70539E+00_JPRB, 0.70411E+00_JPRB, 0.70686E+00_JPRB, 0.71298E+00_JPRB, 0.72087E+00_JPRB, 0.73163E+00_JPRB ,&
&  0.74733E+00_JPRB, 0.76163E+00_JPRB, 0.76920E+00_JPRB, 0.77904E+00_JPRB, 0.79148E+00_JPRB, 0.79679E+00_JPRB, 0.81015E+00_JPRB ,&
&  0.80947E+00_JPRB, 0.80437E+00_JPRB, 0.80217E+00_JPRB, 0.80703E+00_JPRB, 0.80871E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.76138E+00_JPRB, 0.74148E+00_JPRB, 0.72960E+00_JPRB, 0.71812E+00_JPRB, 0.71009E+00_JPRB, 0.69235E+00_JPRB, 0.68226E+00_JPRB ,&
&  0.67569E+00_JPRB, 0.67150E+00_JPRB, 0.67065E+00_JPRB, 0.67097E+00_JPRB, 0.68133E+00_JPRB, 0.68813E+00_JPRB, 0.69089E+00_JPRB ,&
&  0.69878E+00_JPRB, 0.71669E+00_JPRB, 0.73330E+00_JPRB, 0.74709E+00_JPRB, 0.75778E+00_JPRB, 0.76529E+00_JPRB, 0.76481E+00_JPRB ,&
&  0.76819E+00_JPRB, 0.77586E+00_JPRB, 0.77948E+00_JPRB, 0.76884E+00_JPRB, 0.75549E+00_JPRB, 0.74844E+00_JPRB, 0.74902E+00_JPRB ,&
&  0.73844E+00_JPRB, 0.69590E+00_JPRB, 0.71131E+00_JPRB, 0.72339E+00_JPRB, 0.73770E+00_JPRB, 0.74628E+00_JPRB, 0.75523E+00_JPRB ,&
&  0.75760E+00_JPRB, 0.74043E+00_JPRB, 0.71795E+00_JPRB, 0.70150E+00_JPRB, 0.70491E+00_JPRB, 0.70225E+00_JPRB, 0.69261E+00_JPRB ,&
&  0.68809E+00_JPRB, 0.68426E+00_JPRB, 0.67662E+00_JPRB, 0.66352E+00_JPRB, 0.65169E+00_JPRB, 0.64349E+00_JPRB, 0.63784E+00_JPRB ,&
&  0.62615E+00_JPRB, 0.61658E+00_JPRB, 0.61144E+00_JPRB, 0.60575E+00_JPRB, 0.60120E+00_JPRB, 0.60620E+00_JPRB, 0.61135E+00_JPRB ,&
&  0.61396E+00_JPRB, 0.61925E+00_JPRB, 0.63143E+00_JPRB, 0.65015E+00_JPRB, 0.66711E+00_JPRB, 0.68514E+00_JPRB, 0.70509E+00_JPRB ,&
&  0.72602E+00_JPRB, 0.74574E+00_JPRB, 0.75419E+00_JPRB, 0.75953E+00_JPRB, 0.75321E+00_JPRB, 0.74177E+00_JPRB, 0.71935E+00_JPRB ,&
&  0.69424E+00_JPRB, 0.68591E+00_JPRB, 0.68362E+00_JPRB, 0.68665E+00_JPRB, 0.68292E+00_JPRB, 0.67931E+00_JPRB, 0.67793E+00_JPRB ,&
&  0.67429E+00_JPRB, 0.67077E+00_JPRB, 0.66552E+00_JPRB, 0.66071E+00_JPRB, 0.66383E+00_JPRB, 0.66939E+00_JPRB, 0.68245E+00_JPRB ,&
&  0.70028E+00_JPRB, 0.71939E+00_JPRB, 0.73881E+00_JPRB, 0.75132E+00_JPRB, 0.75496E+00_JPRB, 0.75710E+00_JPRB, 0.75983E+00_JPRB ,&
&  0.76039E+00_JPRB, 0.75800E+00_JPRB, 0.76329E+00_JPRB, 0.76722E+00_JPRB, 0.77156E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.70605E+00_JPRB, 0.68096E+00_JPRB, 0.66286E+00_JPRB, 0.65625E+00_JPRB, 0.65279E+00_JPRB, 0.64073E+00_JPRB, 0.63543E+00_JPRB ,&
&  0.64132E+00_JPRB, 0.64300E+00_JPRB, 0.64996E+00_JPRB, 0.64954E+00_JPRB, 0.64883E+00_JPRB, 0.65152E+00_JPRB, 0.64397E+00_JPRB ,&
&  0.64412E+00_JPRB, 0.65335E+00_JPRB, 0.66428E+00_JPRB, 0.67905E+00_JPRB, 0.69147E+00_JPRB, 0.70083E+00_JPRB, 0.70865E+00_JPRB ,&
&  0.71172E+00_JPRB, 0.71817E+00_JPRB, 0.72165E+00_JPRB, 0.71719E+00_JPRB, 0.70574E+00_JPRB, 0.69665E+00_JPRB, 0.69484E+00_JPRB ,&
&  0.69320E+00_JPRB, 0.68848E+00_JPRB, 0.64414E+00_JPRB, 0.66843E+00_JPRB, 0.67554E+00_JPRB, 0.67962E+00_JPRB, 0.68262E+00_JPRB ,&
&  0.67812E+00_JPRB, 0.66367E+00_JPRB, 0.65631E+00_JPRB, 0.64324E+00_JPRB, 0.64164E+00_JPRB, 0.64757E+00_JPRB, 0.64571E+00_JPRB ,&
&  0.64087E+00_JPRB, 0.62999E+00_JPRB, 0.61575E+00_JPRB, 0.60483E+00_JPRB, 0.60198E+00_JPRB, 0.59957E+00_JPRB, 0.59137E+00_JPRB ,&
&  0.58592E+00_JPRB, 0.58521E+00_JPRB, 0.58362E+00_JPRB, 0.57762E+00_JPRB, 0.56266E+00_JPRB, 0.54928E+00_JPRB, 0.54771E+00_JPRB ,&
&  0.55314E+00_JPRB, 0.56278E+00_JPRB, 0.57991E+00_JPRB, 0.60095E+00_JPRB, 0.62256E+00_JPRB, 0.64244E+00_JPRB, 0.65985E+00_JPRB ,&
&  0.66665E+00_JPRB, 0.66956E+00_JPRB, 0.67337E+00_JPRB, 0.68002E+00_JPRB, 0.68244E+00_JPRB, 0.67470E+00_JPRB, 0.65345E+00_JPRB ,&
&  0.63035E+00_JPRB, 0.62302E+00_JPRB, 0.62556E+00_JPRB, 0.63333E+00_JPRB, 0.63685E+00_JPRB, 0.63601E+00_JPRB, 0.62979E+00_JPRB ,&
&  0.62628E+00_JPRB, 0.61824E+00_JPRB, 0.60919E+00_JPRB, 0.60460E+00_JPRB, 0.60404E+00_JPRB, 0.60984E+00_JPRB, 0.62718E+00_JPRB ,&
&  0.64834E+00_JPRB, 0.67194E+00_JPRB, 0.69270E+00_JPRB, 0.69980E+00_JPRB, 0.70324E+00_JPRB, 0.70159E+00_JPRB, 0.69195E+00_JPRB ,&
&  0.69575E+00_JPRB, 0.71439E+00_JPRB, 0.72363E+00_JPRB, 0.72472E+00_JPRB, 0.72184E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.65373E+00_JPRB, 0.62512E+00_JPRB, 0.61390E+00_JPRB, 0.61586E+00_JPRB, 0.61928E+00_JPRB, 0.61498E+00_JPRB, 0.62219E+00_JPRB ,&
&  0.63327E+00_JPRB, 0.64169E+00_JPRB, 0.64635E+00_JPRB, 0.64254E+00_JPRB, 0.64001E+00_JPRB, 0.63698E+00_JPRB, 0.62206E+00_JPRB ,&
&  0.61706E+00_JPRB, 0.61533E+00_JPRB, 0.61964E+00_JPRB, 0.62708E+00_JPRB, 0.63530E+00_JPRB, 0.64792E+00_JPRB, 0.65728E+00_JPRB ,&
&  0.66337E+00_JPRB, 0.66713E+00_JPRB, 0.67129E+00_JPRB, 0.67141E+00_JPRB, 0.66765E+00_JPRB, 0.65815E+00_JPRB, 0.65080E+00_JPRB ,&
&  0.64633E+00_JPRB, 0.63492E+00_JPRB, 0.62427E+00_JPRB, 0.62277E+00_JPRB, 0.62566E+00_JPRB, 0.62853E+00_JPRB, 0.62813E+00_JPRB ,&
&  0.61590E+00_JPRB, 0.59972E+00_JPRB, 0.59840E+00_JPRB, 0.59785E+00_JPRB, 0.60264E+00_JPRB, 0.61459E+00_JPRB, 0.61783E+00_JPRB ,&
&  0.61423E+00_JPRB, 0.59970E+00_JPRB, 0.58025E+00_JPRB, 0.57644E+00_JPRB, 0.57840E+00_JPRB, 0.57104E+00_JPRB, 0.56402E+00_JPRB ,&
&  0.56290E+00_JPRB, 0.56013E+00_JPRB, 0.55425E+00_JPRB, 0.54605E+00_JPRB, 0.53305E+00_JPRB, 0.51636E+00_JPRB, 0.50828E+00_JPRB ,&
&  0.50918E+00_JPRB, 0.52719E+00_JPRB, 0.55766E+00_JPRB, 0.59079E+00_JPRB, 0.62121E+00_JPRB, 0.63548E+00_JPRB, 0.63396E+00_JPRB ,&
&  0.61957E+00_JPRB, 0.61442E+00_JPRB, 0.61851E+00_JPRB, 0.62340E+00_JPRB, 0.62562E+00_JPRB, 0.61573E+00_JPRB, 0.59803E+00_JPRB ,&
&  0.58472E+00_JPRB, 0.57518E+00_JPRB, 0.57398E+00_JPRB, 0.57847E+00_JPRB, 0.58292E+00_JPRB, 0.58394E+00_JPRB, 0.57835E+00_JPRB ,&
&  0.57291E+00_JPRB, 0.56681E+00_JPRB, 0.56273E+00_JPRB, 0.56485E+00_JPRB, 0.56933E+00_JPRB, 0.57411E+00_JPRB, 0.58752E+00_JPRB ,&
&  0.60711E+00_JPRB, 0.62644E+00_JPRB, 0.63077E+00_JPRB, 0.62618E+00_JPRB, 0.62523E+00_JPRB, 0.62199E+00_JPRB, 0.61891E+00_JPRB ,&
&  0.62868E+00_JPRB, 0.65078E+00_JPRB, 0.66654E+00_JPRB, 0.67496E+00_JPRB, 0.67472E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.57717E+00_JPRB, 0.56043E+00_JPRB, 0.56115E+00_JPRB, 0.56867E+00_JPRB, 0.57291E+00_JPRB, 0.58073E+00_JPRB, 0.60019E+00_JPRB ,&
&  0.61135E+00_JPRB, 0.61836E+00_JPRB, 0.62108E+00_JPRB, 0.61596E+00_JPRB, 0.61512E+00_JPRB, 0.60791E+00_JPRB, 0.59507E+00_JPRB ,&
&  0.58968E+00_JPRB, 0.58860E+00_JPRB, 0.59027E+00_JPRB, 0.59424E+00_JPRB, 0.59671E+00_JPRB, 0.59988E+00_JPRB, 0.60348E+00_JPRB ,&
&  0.60545E+00_JPRB, 0.60549E+00_JPRB, 0.60459E+00_JPRB, 0.60474E+00_JPRB, 0.60365E+00_JPRB, 0.59578E+00_JPRB, 0.58683E+00_JPRB ,&
&  0.57721E+00_JPRB, 0.56790E+00_JPRB, 0.56500E+00_JPRB, 0.56905E+00_JPRB, 0.57374E+00_JPRB, 0.57647E+00_JPRB, 0.57397E+00_JPRB ,&
&  0.56946E+00_JPRB, 0.56747E+00_JPRB, 0.57045E+00_JPRB, 0.57734E+00_JPRB, 0.58568E+00_JPRB, 0.59509E+00_JPRB, 0.60003E+00_JPRB ,&
&  0.59422E+00_JPRB, 0.57631E+00_JPRB, 0.56536E+00_JPRB, 0.56461E+00_JPRB, 0.56025E+00_JPRB, 0.54487E+00_JPRB, 0.53431E+00_JPRB ,&
&  0.52953E+00_JPRB, 0.52063E+00_JPRB, 0.51125E+00_JPRB, 0.49859E+00_JPRB, 0.48703E+00_JPRB, 0.48263E+00_JPRB, 0.48174E+00_JPRB ,&
&  0.48622E+00_JPRB, 0.50723E+00_JPRB, 0.54354E+00_JPRB, 0.57759E+00_JPRB, 0.59164E+00_JPRB, 0.58429E+00_JPRB, 0.56587E+00_JPRB ,&
&  0.55624E+00_JPRB, 0.55136E+00_JPRB, 0.55191E+00_JPRB, 0.55191E+00_JPRB, 0.54926E+00_JPRB, 0.54091E+00_JPRB, 0.53350E+00_JPRB ,&
&  0.52304E+00_JPRB, 0.51170E+00_JPRB, 0.50723E+00_JPRB, 0.50445E+00_JPRB, 0.50146E+00_JPRB, 0.50167E+00_JPRB, 0.50119E+00_JPRB ,&
&  0.50223E+00_JPRB, 0.50638E+00_JPRB, 0.51775E+00_JPRB, 0.52348E+00_JPRB, 0.52353E+00_JPRB, 0.51322E+00_JPRB, 0.51488E+00_JPRB ,&
&  0.51842E+00_JPRB, 0.51647E+00_JPRB, 0.51179E+00_JPRB, 0.51151E+00_JPRB, 0.51137E+00_JPRB, 0.51320E+00_JPRB, 0.52500E+00_JPRB ,&
&  0.55186E+00_JPRB, 0.57516E+00_JPRB, 0.59412E+00_JPRB, 0.59923E+00_JPRB, 0.59480E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.52067E+00_JPRB, 0.52598E+00_JPRB, 0.53922E+00_JPRB, 0.55011E+00_JPRB, 0.55367E+00_JPRB, 0.56339E+00_JPRB, 0.57458E+00_JPRB ,&
&  0.58337E+00_JPRB, 0.58481E+00_JPRB, 0.58099E+00_JPRB, 0.57097E+00_JPRB, 0.55915E+00_JPRB, 0.54932E+00_JPRB, 0.54551E+00_JPRB ,&
&  0.54774E+00_JPRB, 0.55075E+00_JPRB, 0.55625E+00_JPRB, 0.55936E+00_JPRB, 0.56321E+00_JPRB, 0.56227E+00_JPRB, 0.56300E+00_JPRB ,&
&  0.56510E+00_JPRB, 0.56632E+00_JPRB, 0.56685E+00_JPRB, 0.56307E+00_JPRB, 0.55997E+00_JPRB, 0.55719E+00_JPRB, 0.54819E+00_JPRB ,&
&  0.53830E+00_JPRB, 0.53176E+00_JPRB, 0.53555E+00_JPRB, 0.54671E+00_JPRB, 0.55372E+00_JPRB, 0.55057E+00_JPRB, 0.54891E+00_JPRB ,&
&  0.54911E+00_JPRB, 0.55430E+00_JPRB, 0.56269E+00_JPRB, 0.57481E+00_JPRB, 0.58297E+00_JPRB, 0.59371E+00_JPRB, 0.59421E+00_JPRB ,&
&  0.58770E+00_JPRB, 0.57838E+00_JPRB, 0.57530E+00_JPRB, 0.57374E+00_JPRB, 0.54384E+00_JPRB, 0.51121E+00_JPRB, 0.49529E+00_JPRB ,&
&  0.48673E+00_JPRB, 0.47613E+00_JPRB, 0.46030E+00_JPRB, 0.44932E+00_JPRB, 0.44835E+00_JPRB, 0.45344E+00_JPRB, 0.46521E+00_JPRB ,&
&  0.47497E+00_JPRB, 0.48907E+00_JPRB, 0.50525E+00_JPRB, 0.51569E+00_JPRB, 0.51562E+00_JPRB, 0.50271E+00_JPRB, 0.49038E+00_JPRB ,&
&  0.48269E+00_JPRB, 0.48008E+00_JPRB, 0.47820E+00_JPRB, 0.47613E+00_JPRB, 0.47445E+00_JPRB, 0.47397E+00_JPRB, 0.47303E+00_JPRB ,&
&  0.46887E+00_JPRB, 0.46186E+00_JPRB, 0.45332E+00_JPRB, 0.44861E+00_JPRB, 0.44803E+00_JPRB, 0.44840E+00_JPRB, 0.45036E+00_JPRB ,&
&  0.45936E+00_JPRB, 0.47079E+00_JPRB, 0.47970E+00_JPRB, 0.48309E+00_JPRB, 0.47943E+00_JPRB, 0.47112E+00_JPRB, 0.46628E+00_JPRB ,&
&  0.46286E+00_JPRB, 0.46047E+00_JPRB, 0.45718E+00_JPRB, 0.45574E+00_JPRB, 0.45573E+00_JPRB, 0.46342E+00_JPRB, 0.47960E+00_JPRB ,&
&  0.49935E+00_JPRB, 0.52482E+00_JPRB, 0.53608E+00_JPRB, 0.53432E+00_JPRB, 0.52698E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.49021E+00_JPRB, 0.50317E+00_JPRB, 0.51944E+00_JPRB, 0.53080E+00_JPRB, 0.53562E+00_JPRB, 0.54090E+00_JPRB, 0.54326E+00_JPRB ,&
&  0.53767E+00_JPRB, 0.53523E+00_JPRB, 0.52967E+00_JPRB, 0.52188E+00_JPRB, 0.51512E+00_JPRB, 0.51180E+00_JPRB, 0.51227E+00_JPRB ,&
&  0.51441E+00_JPRB, 0.51626E+00_JPRB, 0.51285E+00_JPRB, 0.51124E+00_JPRB, 0.51542E+00_JPRB, 0.52107E+00_JPRB, 0.52480E+00_JPRB ,&
&  0.52649E+00_JPRB, 0.52819E+00_JPRB, 0.53311E+00_JPRB, 0.53618E+00_JPRB, 0.53621E+00_JPRB, 0.52942E+00_JPRB, 0.52423E+00_JPRB ,&
&  0.51897E+00_JPRB, 0.51700E+00_JPRB, 0.52303E+00_JPRB, 0.51530E+00_JPRB, 0.52979E+00_JPRB, 0.53142E+00_JPRB, 0.53661E+00_JPRB ,&
&  0.54455E+00_JPRB, 0.55819E+00_JPRB, 0.56906E+00_JPRB, 0.57733E+00_JPRB, 0.57968E+00_JPRB, 0.58262E+00_JPRB, 0.57255E+00_JPRB ,&
&  0.55870E+00_JPRB, 0.54007E+00_JPRB, 0.52120E+00_JPRB, 0.50038E+00_JPRB, 0.47996E+00_JPRB, 0.46427E+00_JPRB, 0.45687E+00_JPRB ,&
&  0.44780E+00_JPRB, 0.43380E+00_JPRB, 0.42039E+00_JPRB, 0.41527E+00_JPRB, 0.41999E+00_JPRB, 0.43018E+00_JPRB, 0.44595E+00_JPRB ,&
&  0.45628E+00_JPRB, 0.45966E+00_JPRB, 0.45678E+00_JPRB, 0.45278E+00_JPRB, 0.44848E+00_JPRB, 0.44071E+00_JPRB, 0.43601E+00_JPRB ,&
&  0.43224E+00_JPRB, 0.42650E+00_JPRB, 0.42351E+00_JPRB, 0.42499E+00_JPRB, 0.42795E+00_JPRB, 0.43004E+00_JPRB, 0.42886E+00_JPRB ,&
&  0.42632E+00_JPRB, 0.42169E+00_JPRB, 0.41366E+00_JPRB, 0.40975E+00_JPRB, 0.41059E+00_JPRB, 0.41519E+00_JPRB, 0.42265E+00_JPRB ,&
&  0.42506E+00_JPRB, 0.43131E+00_JPRB, 0.43728E+00_JPRB, 0.43928E+00_JPRB, 0.44169E+00_JPRB, 0.44348E+00_JPRB, 0.44340E+00_JPRB ,&
&  0.44153E+00_JPRB, 0.43646E+00_JPRB, 0.42970E+00_JPRB, 0.42720E+00_JPRB, 0.42880E+00_JPRB, 0.43606E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.45474E+00_JPRB, 0.46895E+00_JPRB, 0.47954E+00_JPRB, 0.48181E+00_JPRB, 0.48396E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.46848E+00_JPRB, 0.48029E+00_JPRB, 0.49407E+00_JPRB, 0.50346E+00_JPRB, 0.50229E+00_JPRB, 0.50206E+00_JPRB, 0.50796E+00_JPRB ,&
&  0.50129E+00_JPRB, 0.49373E+00_JPRB, 0.48776E+00_JPRB, 0.48186E+00_JPRB, 0.48001E+00_JPRB, 0.47893E+00_JPRB, 0.48210E+00_JPRB ,&
&  0.48443E+00_JPRB, 0.48353E+00_JPRB, 0.48215E+00_JPRB, 0.48322E+00_JPRB, 0.49018E+00_JPRB, 0.49876E+00_JPRB, 0.50592E+00_JPRB ,&
&  0.50747E+00_JPRB, 0.51125E+00_JPRB, 0.51723E+00_JPRB, 0.52229E+00_JPRB, 0.52477E+00_JPRB, 0.52614E+00_JPRB, 0.52471E+00_JPRB ,&
&  0.52185E+00_JPRB, 0.51878E+00_JPRB, 0.50167E+00_JPRB, 0.51448E+00_JPRB, 0.51898E+00_JPRB, 0.51962E+00_JPRB, 0.52451E+00_JPRB ,&
&  0.53209E+00_JPRB, 0.54116E+00_JPRB, 0.54561E+00_JPRB, 0.54704E+00_JPRB, 0.54215E+00_JPRB, 0.53250E+00_JPRB, 0.50787E+00_JPRB ,&
&  0.48722E+00_JPRB, 0.46588E+00_JPRB, 0.45249E+00_JPRB, 0.44421E+00_JPRB, 0.43754E+00_JPRB, 0.43193E+00_JPRB, 0.42690E+00_JPRB ,&
&  0.41994E+00_JPRB, 0.41375E+00_JPRB, 0.40990E+00_JPRB, 0.40977E+00_JPRB, 0.41372E+00_JPRB, 0.42450E+00_JPRB, 0.43787E+00_JPRB ,&
&  0.44384E+00_JPRB, 0.43615E+00_JPRB, 0.42808E+00_JPRB, 0.41896E+00_JPRB, 0.40374E+00_JPRB, 0.37707E+00_JPRB, 0.36027E+00_JPRB ,&
&  0.35646E+00_JPRB, 0.36906E+00_JPRB, 0.38993E+00_JPRB, 0.39620E+00_JPRB, 0.40179E+00_JPRB, 0.39922E+00_JPRB, 0.39363E+00_JPRB ,&
&  0.38399E+00_JPRB, 0.36103E+00_JPRB, 0.34912E+00_JPRB, 0.34583E+00_JPRB, 0.34385E+00_JPRB, 0.34467E+00_JPRB, 0.34746E+00_JPRB ,&
&  0.35206E+00_JPRB, 0.35906E+00_JPRB, 0.36396E+00_JPRB, 0.36591E+00_JPRB, 0.36885E+00_JPRB, 0.36913E+00_JPRB, 0.36806E+00_JPRB ,&
&  0.36471E+00_JPRB, 0.37194E+00_JPRB, 0.39092E+00_JPRB, 0.41869E+00_JPRB, 0.42529E+00_JPRB, 0.43331E+00_JPRB, 0.44214E+00_JPRB ,&
&  0.44766E+00_JPRB, 0.45315E+00_JPRB, 0.45385E+00_JPRB, 0.45455E+00_JPRB, 0.45957E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.44312E+00_JPRB, 0.45025E+00_JPRB, 0.45899E+00_JPRB, 0.47030E+00_JPRB, 0.47275E+00_JPRB, 0.47431E+00_JPRB, 0.48325E+00_JPRB ,&
&  0.48325E+00_JPRB, 0.48004E+00_JPRB, 0.47686E+00_JPRB, 0.47735E+00_JPRB, 0.47984E+00_JPRB, 0.48256E+00_JPRB, 0.48251E+00_JPRB ,&
&  0.47367E+00_JPRB, 0.46868E+00_JPRB, 0.47173E+00_JPRB, 0.47505E+00_JPRB, 0.47985E+00_JPRB, 0.48634E+00_JPRB, 0.49209E+00_JPRB ,&
&  0.49531E+00_JPRB, 0.49913E+00_JPRB, 0.50190E+00_JPRB, 0.50834E+00_JPRB, 0.51801E+00_JPRB, 0.52470E+00_JPRB, 0.53058E+00_JPRB ,&
&  0.51496E+00_JPRB, 0.46693E+00_JPRB, 0.43679E+00_JPRB, 0.44275E+00_JPRB, 0.49034E+00_JPRB, 0.50100E+00_JPRB, 0.50183E+00_JPRB ,&
&  0.50474E+00_JPRB, 0.51225E+00_JPRB, 0.51187E+00_JPRB, 0.50616E+00_JPRB, 0.49413E+00_JPRB, 0.47481E+00_JPRB, 0.45138E+00_JPRB ,&
&  0.43583E+00_JPRB, 0.42573E+00_JPRB, 0.41887E+00_JPRB, 0.41194E+00_JPRB, 0.40837E+00_JPRB, 0.40335E+00_JPRB, 0.39552E+00_JPRB ,&
&  0.39237E+00_JPRB, 0.39127E+00_JPRB, 0.39254E+00_JPRB, 0.39385E+00_JPRB, 0.39547E+00_JPRB, 0.39633E+00_JPRB, 0.39200E+00_JPRB ,&
&  0.38004E+00_JPRB, 0.36402E+00_JPRB, 0.34890E+00_JPRB, 0.33244E+00_JPRB, 0.31852E+00_JPRB, 0.31013E+00_JPRB, 0.30328E+00_JPRB ,&
&  0.30080E+00_JPRB, 0.30736E+00_JPRB, 0.32565E+00_JPRB, 0.34616E+00_JPRB, 0.37615E+00_JPRB, 0.35061E+00_JPRB, 0.32702E+00_JPRB ,&
&  0.31593E+00_JPRB, 0.31039E+00_JPRB, 0.30727E+00_JPRB, 0.30730E+00_JPRB, 0.30896E+00_JPRB, 0.30942E+00_JPRB, 0.30956E+00_JPRB ,&
&  0.31110E+00_JPRB, 0.31382E+00_JPRB, 0.31578E+00_JPRB, 0.31754E+00_JPRB, 0.32022E+00_JPRB, 0.32324E+00_JPRB, 0.32484E+00_JPRB ,&
&  0.32576E+00_JPRB, 0.33098E+00_JPRB, 0.34109E+00_JPRB, 0.35285E+00_JPRB, 0.36433E+00_JPRB, 0.37625E+00_JPRB, 0.39350E+00_JPRB ,&
&  0.41468E+00_JPRB, 0.43947E+00_JPRB, 0.43961E+00_JPRB, 0.44044E+00_JPRB, 0.44047E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.42924E+00_JPRB, 0.44018E+00_JPRB, 0.44572E+00_JPRB, 0.45129E+00_JPRB, 0.45374E+00_JPRB, 0.45527E+00_JPRB, 0.46333E+00_JPRB ,&
&  0.46820E+00_JPRB, 0.47212E+00_JPRB, 0.47465E+00_JPRB, 0.47615E+00_JPRB, 0.47251E+00_JPRB, 0.46625E+00_JPRB, 0.46465E+00_JPRB ,&
&  0.46359E+00_JPRB, 0.46412E+00_JPRB, 0.46322E+00_JPRB, 0.46326E+00_JPRB, 0.46381E+00_JPRB, 0.46855E+00_JPRB, 0.47551E+00_JPRB ,&
&  0.47712E+00_JPRB, 0.48141E+00_JPRB, 0.49448E+00_JPRB, 0.50221E+00_JPRB, 0.50705E+00_JPRB, 0.50627E+00_JPRB, 0.50009E+00_JPRB ,&
&  0.47138E+00_JPRB, 0.44304E+00_JPRB, 0.41640E+00_JPRB, 0.41765E+00_JPRB, 0.46952E+00_JPRB, 0.47538E+00_JPRB, 0.47508E+00_JPRB ,&
&  0.47360E+00_JPRB, 0.47773E+00_JPRB, 0.47795E+00_JPRB, 0.46323E+00_JPRB, 0.45253E+00_JPRB, 0.43803E+00_JPRB, 0.42534E+00_JPRB ,&
&  0.41404E+00_JPRB, 0.40652E+00_JPRB, 0.39293E+00_JPRB, 0.36507E+00_JPRB, 0.34806E+00_JPRB, 0.33929E+00_JPRB, 0.32539E+00_JPRB ,&
&  0.31478E+00_JPRB, 0.31134E+00_JPRB, 0.31052E+00_JPRB, 0.30945E+00_JPRB, 0.31344E+00_JPRB, 0.31548E+00_JPRB, 0.31369E+00_JPRB ,&
&  0.30976E+00_JPRB, 0.30653E+00_JPRB, 0.30255E+00_JPRB, 0.29723E+00_JPRB, 0.29450E+00_JPRB, 0.29384E+00_JPRB, 0.29460E+00_JPRB ,&
&  0.29907E+00_JPRB, 0.30559E+00_JPRB, 0.31975E+00_JPRB, 0.32960E+00_JPRB, 0.32473E+00_JPRB, 0.31272E+00_JPRB, 0.30240E+00_JPRB ,&
&  0.29534E+00_JPRB, 0.29249E+00_JPRB, 0.29028E+00_JPRB, 0.28796E+00_JPRB, 0.28857E+00_JPRB, 0.29075E+00_JPRB, 0.29218E+00_JPRB ,&
&  0.29240E+00_JPRB, 0.29438E+00_JPRB, 0.29560E+00_JPRB, 0.29700E+00_JPRB, 0.29821E+00_JPRB, 0.30016E+00_JPRB, 0.30278E+00_JPRB ,&
&  0.30516E+00_JPRB, 0.31063E+00_JPRB, 0.31659E+00_JPRB, 0.32336E+00_JPRB, 0.32767E+00_JPRB, 0.33094E+00_JPRB, 0.33771E+00_JPRB ,&
&  0.34390E+00_JPRB, 0.35957E+00_JPRB, 0.39120E+00_JPRB, 0.42675E+00_JPRB, 0.41929E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.41660E+00_JPRB, 0.42554E+00_JPRB, 0.43333E+00_JPRB, 0.43944E+00_JPRB, 0.44346E+00_JPRB, 0.44441E+00_JPRB, 0.44734E+00_JPRB ,&
&  0.45507E+00_JPRB, 0.45968E+00_JPRB, 0.46333E+00_JPRB, 0.46246E+00_JPRB, 0.46198E+00_JPRB, 0.44379E+00_JPRB, 0.43966E+00_JPRB ,&
&  0.43342E+00_JPRB, 0.42905E+00_JPRB, 0.42289E+00_JPRB, 0.42272E+00_JPRB, 0.42241E+00_JPRB, 0.42453E+00_JPRB, 0.42537E+00_JPRB ,&
&  0.42594E+00_JPRB, 0.42458E+00_JPRB, 0.42162E+00_JPRB, 0.41984E+00_JPRB, 0.42101E+00_JPRB, 0.42218E+00_JPRB, 0.41905E+00_JPRB ,&
&  0.41454E+00_JPRB, 0.41341E+00_JPRB, 0.41649E+00_JPRB, 0.42319E+00_JPRB, 0.46284E+00_JPRB, 0.45713E+00_JPRB, 0.45513E+00_JPRB ,&
&  0.45180E+00_JPRB, 0.44752E+00_JPRB, 0.44680E+00_JPRB, 0.43995E+00_JPRB, 0.43018E+00_JPRB, 0.40770E+00_JPRB, 0.39529E+00_JPRB ,&
&  0.36114E+00_JPRB, 0.34256E+00_JPRB, 0.32625E+00_JPRB, 0.31568E+00_JPRB, 0.30493E+00_JPRB, 0.29868E+00_JPRB, 0.29317E+00_JPRB ,&
&  0.28897E+00_JPRB, 0.28563E+00_JPRB, 0.28264E+00_JPRB, 0.28328E+00_JPRB, 0.28514E+00_JPRB, 0.28605E+00_JPRB, 0.28700E+00_JPRB ,&
&  0.28756E+00_JPRB, 0.28810E+00_JPRB, 0.28877E+00_JPRB, 0.29026E+00_JPRB, 0.29222E+00_JPRB, 0.29481E+00_JPRB, 0.29811E+00_JPRB ,&
&  0.30342E+00_JPRB, 0.30752E+00_JPRB, 0.30897E+00_JPRB, 0.31181E+00_JPRB, 0.31146E+00_JPRB, 0.30523E+00_JPRB, 0.29833E+00_JPRB ,&
&  0.29103E+00_JPRB, 0.28722E+00_JPRB, 0.28345E+00_JPRB, 0.28077E+00_JPRB, 0.28011E+00_JPRB, 0.28002E+00_JPRB, 0.28204E+00_JPRB ,&
&  0.28429E+00_JPRB, 0.28633E+00_JPRB, 0.28778E+00_JPRB, 0.28934E+00_JPRB, 0.29134E+00_JPRB, 0.29283E+00_JPRB, 0.29475E+00_JPRB ,&
&  0.29723E+00_JPRB, 0.30056E+00_JPRB, 0.30555E+00_JPRB, 0.31135E+00_JPRB, 0.31898E+00_JPRB, 0.33002E+00_JPRB, 0.34905E+00_JPRB ,&
&  0.37356E+00_JPRB, 0.42817E+00_JPRB, 0.43230E+00_JPRB, 0.40336E+00_JPRB, 0.40970E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.39310E+00_JPRB, 0.40026E+00_JPRB, 0.40735E+00_JPRB, 0.41501E+00_JPRB, 0.42461E+00_JPRB, 0.42886E+00_JPRB, 0.43086E+00_JPRB ,&
&  0.42665E+00_JPRB, 0.42236E+00_JPRB, 0.41171E+00_JPRB, 0.40244E+00_JPRB, 0.39312E+00_JPRB, 0.38570E+00_JPRB, 0.37976E+00_JPRB ,&
&  0.37603E+00_JPRB, 0.37319E+00_JPRB, 0.36859E+00_JPRB, 0.36513E+00_JPRB, 0.36195E+00_JPRB, 0.36213E+00_JPRB, 0.36262E+00_JPRB ,&
&  0.36225E+00_JPRB, 0.36271E+00_JPRB, 0.36124E+00_JPRB, 0.36644E+00_JPRB, 0.37574E+00_JPRB, 0.38690E+00_JPRB, 0.40149E+00_JPRB ,&
&  0.42187E+00_JPRB, 0.43281E+00_JPRB, 0.44837E+00_JPRB, 0.44851E+00_JPRB, 0.44675E+00_JPRB, 0.44305E+00_JPRB, 0.43995E+00_JPRB ,&
&  0.43522E+00_JPRB, 0.42836E+00_JPRB, 0.42320E+00_JPRB, 0.41660E+00_JPRB, 0.38659E+00_JPRB, 0.36342E+00_JPRB, 0.34554E+00_JPRB ,&
&  0.33045E+00_JPRB, 0.31620E+00_JPRB, 0.30348E+00_JPRB, 0.29713E+00_JPRB, 0.29174E+00_JPRB, 0.28854E+00_JPRB, 0.28518E+00_JPRB ,&
&  0.28159E+00_JPRB, 0.28045E+00_JPRB, 0.27853E+00_JPRB, 0.27780E+00_JPRB, 0.27892E+00_JPRB, 0.27878E+00_JPRB, 0.27954E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28162E+00_JPRB, 0.28455E+00_JPRB, 0.28673E+00_JPRB, 0.28894E+00_JPRB, 0.29097E+00_JPRB, 0.29195E+00_JPRB ,&
&  0.29394E+00_JPRB, 0.29567E+00_JPRB, 0.29561E+00_JPRB, 0.29481E+00_JPRB, 0.29288E+00_JPRB, 0.28845E+00_JPRB, 0.28409E+00_JPRB ,&
&  0.27941E+00_JPRB, 0.27603E+00_JPRB, 0.27482E+00_JPRB, 0.27373E+00_JPRB, 0.27460E+00_JPRB, 0.27534E+00_JPRB, 0.27615E+00_JPRB ,&
&  0.27809E+00_JPRB, 0.27995E+00_JPRB, 0.28217E+00_JPRB, 0.28540E+00_JPRB, 0.28779E+00_JPRB, 0.28979E+00_JPRB, 0.29229E+00_JPRB ,&
&  0.29230E+00_JPRB, 0.29676E+00_JPRB, 0.30244E+00_JPRB, 0.31016E+00_JPRB, 0.32401E+00_JPRB, 0.33428E+00_JPRB, 0.35737E+00_JPRB ,&
&  0.39138E+00_JPRB, 0.45553E+00_JPRB, 0.44843E+00_JPRB, 0.38219E+00_JPRB, 0.38660E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.38086E+00_JPRB, 0.38416E+00_JPRB, 0.38752E+00_JPRB, 0.39169E+00_JPRB, 0.39679E+00_JPRB, 0.40144E+00_JPRB, 0.40581E+00_JPRB ,&
&  0.40863E+00_JPRB, 0.40605E+00_JPRB, 0.40701E+00_JPRB, 0.40222E+00_JPRB, 0.39745E+00_JPRB, 0.39124E+00_JPRB, 0.38505E+00_JPRB ,&
&  0.37872E+00_JPRB, 0.36964E+00_JPRB, 0.35996E+00_JPRB, 0.35145E+00_JPRB, 0.34421E+00_JPRB, 0.33868E+00_JPRB, 0.33438E+00_JPRB ,&
&  0.33138E+00_JPRB, 0.33200E+00_JPRB, 0.33410E+00_JPRB, 0.34018E+00_JPRB, 0.35038E+00_JPRB, 0.36282E+00_JPRB, 0.38184E+00_JPRB ,&
&  0.40851E+00_JPRB, 0.41067E+00_JPRB, 0.41643E+00_JPRB, 0.41561E+00_JPRB, 0.41152E+00_JPRB, 0.40979E+00_JPRB, 0.40638E+00_JPRB ,&
&  0.40700E+00_JPRB, 0.39845E+00_JPRB, 0.40812E+00_JPRB, 0.39061E+00_JPRB, 0.36273E+00_JPRB, 0.34731E+00_JPRB, 0.32881E+00_JPRB ,&
&  0.31569E+00_JPRB, 0.30449E+00_JPRB, 0.29615E+00_JPRB, 0.29290E+00_JPRB, 0.28886E+00_JPRB, 0.28702E+00_JPRB, 0.28302E+00_JPRB ,&
&  0.27896E+00_JPRB, 0.27710E+00_JPRB, 0.27387E+00_JPRB, 0.27376E+00_JPRB, 0.27394E+00_JPRB, 0.27334E+00_JPRB, 0.27345E+00_JPRB ,&
&  0.27390E+00_JPRB, 0.27513E+00_JPRB, 0.27734E+00_JPRB, 0.27932E+00_JPRB, 0.28051E+00_JPRB, 0.28120E+00_JPRB, 0.28105E+00_JPRB ,&
&  0.28159E+00_JPRB, 0.28128E+00_JPRB, 0.28038E+00_JPRB, 0.27898E+00_JPRB, 0.27679E+00_JPRB, 0.27449E+00_JPRB, 0.27227E+00_JPRB ,&
&  0.27043E+00_JPRB, 0.26967E+00_JPRB, 0.26900E+00_JPRB, 0.26912E+00_JPRB, 0.26964E+00_JPRB, 0.27146E+00_JPRB, 0.27315E+00_JPRB ,&
&  0.27512E+00_JPRB, 0.27793E+00_JPRB, 0.28039E+00_JPRB, 0.28341E+00_JPRB, 0.28611E+00_JPRB, 0.28755E+00_JPRB, 0.29179E+00_JPRB ,&
&  0.29730E+00_JPRB, 0.30491E+00_JPRB, 0.31379E+00_JPRB, 0.32218E+00_JPRB, 0.33274E+00_JPRB, 0.34811E+00_JPRB, 0.36875E+00_JPRB ,&
&  0.38942E+00_JPRB, 0.41071E+00_JPRB, 0.40397E+00_JPRB, 0.37765E+00_JPRB, 0.37722E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.38005E+00_JPRB, 0.37865E+00_JPRB, 0.37954E+00_JPRB, 0.38095E+00_JPRB, 0.38521E+00_JPRB, 0.39098E+00_JPRB, 0.39723E+00_JPRB ,&
&  0.39695E+00_JPRB, 0.40032E+00_JPRB, 0.39927E+00_JPRB, 0.39623E+00_JPRB, 0.39130E+00_JPRB, 0.38627E+00_JPRB, 0.38154E+00_JPRB ,&
&  0.37482E+00_JPRB, 0.36618E+00_JPRB, 0.35775E+00_JPRB, 0.34910E+00_JPRB, 0.34090E+00_JPRB, 0.33240E+00_JPRB, 0.32767E+00_JPRB ,&
&  0.32429E+00_JPRB, 0.32344E+00_JPRB, 0.32547E+00_JPRB, 0.32806E+00_JPRB, 0.33339E+00_JPRB, 0.34069E+00_JPRB, 0.35043E+00_JPRB ,&
&  0.35981E+00_JPRB, 0.37069E+00_JPRB, 0.37641E+00_JPRB, 0.37628E+00_JPRB, 0.37616E+00_JPRB, 0.37416E+00_JPRB, 0.37075E+00_JPRB ,&
&  0.36403E+00_JPRB, 0.35653E+00_JPRB, 0.34772E+00_JPRB, 0.33841E+00_JPRB, 0.32890E+00_JPRB, 0.32017E+00_JPRB, 0.31205E+00_JPRB ,&
&  0.30547E+00_JPRB, 0.29989E+00_JPRB, 0.29472E+00_JPRB, 0.29037E+00_JPRB, 0.28701E+00_JPRB, 0.28454E+00_JPRB, 0.28139E+00_JPRB ,&
&  0.27881E+00_JPRB, 0.27652E+00_JPRB, 0.27377E+00_JPRB, 0.27190E+00_JPRB, 0.27022E+00_JPRB, 0.26913E+00_JPRB, 0.26821E+00_JPRB ,&
&  0.26821E+00_JPRB, 0.26822E+00_JPRB, 0.26902E+00_JPRB, 0.26983E+00_JPRB, 0.27075E+00_JPRB, 0.27080E+00_JPRB, 0.27064E+00_JPRB ,&
&  0.27025E+00_JPRB, 0.26985E+00_JPRB, 0.26832E+00_JPRB, 0.26712E+00_JPRB, 0.26627E+00_JPRB, 0.26533E+00_JPRB, 0.26523E+00_JPRB ,&
&  0.26604E+00_JPRB, 0.26688E+00_JPRB, 0.26753E+00_JPRB, 0.26920E+00_JPRB, 0.27085E+00_JPRB, 0.27329E+00_JPRB, 0.27625E+00_JPRB ,&
&  0.27840E+00_JPRB, 0.28093E+00_JPRB, 0.28375E+00_JPRB, 0.28702E+00_JPRB, 0.29094E+00_JPRB, 0.29499E+00_JPRB, 0.29978E+00_JPRB ,&
&  0.30519E+00_JPRB, 0.31068E+00_JPRB, 0.31684E+00_JPRB, 0.32365E+00_JPRB, 0.33186E+00_JPRB, 0.33786E+00_JPRB, 0.34725E+00_JPRB ,&
&  0.35435E+00_JPRB, 0.35138E+00_JPRB, 0.39178E+00_JPRB, 0.38540E+00_JPRB, 0.38143E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.32315E+00_JPRB, 0.32962E+00_JPRB, 0.33398E+00_JPRB, 0.33828E+00_JPRB, 0.33151E+00_JPRB, 0.36880E+00_JPRB, 0.36707E+00_JPRB ,&
&  0.36806E+00_JPRB, 0.35610E+00_JPRB, 0.35756E+00_JPRB, 0.35675E+00_JPRB, 0.35418E+00_JPRB, 0.35027E+00_JPRB, 0.34661E+00_JPRB ,&
&  0.34337E+00_JPRB, 0.33880E+00_JPRB, 0.33340E+00_JPRB, 0.32874E+00_JPRB, 0.32506E+00_JPRB, 0.32191E+00_JPRB, 0.31921E+00_JPRB ,&
&  0.31857E+00_JPRB, 0.31987E+00_JPRB, 0.32235E+00_JPRB, 0.32074E+00_JPRB, 0.32145E+00_JPRB, 0.32313E+00_JPRB, 0.32525E+00_JPRB ,&
&  0.32720E+00_JPRB, 0.32896E+00_JPRB, 0.32897E+00_JPRB, 0.32720E+00_JPRB, 0.32631E+00_JPRB, 0.32412E+00_JPRB, 0.32205E+00_JPRB ,&
&  0.31945E+00_JPRB, 0.31660E+00_JPRB, 0.31321E+00_JPRB, 0.31050E+00_JPRB, 0.30750E+00_JPRB, 0.30384E+00_JPRB, 0.30033E+00_JPRB ,&
&  0.29733E+00_JPRB, 0.29462E+00_JPRB, 0.29158E+00_JPRB, 0.28875E+00_JPRB, 0.28605E+00_JPRB, 0.28330E+00_JPRB, 0.28082E+00_JPRB ,&
&  0.27892E+00_JPRB, 0.27731E+00_JPRB, 0.27573E+00_JPRB, 0.27406E+00_JPRB, 0.27245E+00_JPRB, 0.27089E+00_JPRB, 0.26965E+00_JPRB ,&
&  0.26955E+00_JPRB, 0.26886E+00_JPRB, 0.26820E+00_JPRB, 0.26788E+00_JPRB, 0.26802E+00_JPRB, 0.26755E+00_JPRB, 0.26738E+00_JPRB ,&
&  0.26737E+00_JPRB, 0.26744E+00_JPRB, 0.26768E+00_JPRB, 0.26800E+00_JPRB, 0.26843E+00_JPRB, 0.26901E+00_JPRB, 0.26993E+00_JPRB ,&
&  0.27103E+00_JPRB, 0.27237E+00_JPRB, 0.27381E+00_JPRB, 0.27528E+00_JPRB, 0.27679E+00_JPRB, 0.27838E+00_JPRB, 0.28018E+00_JPRB ,&
&  0.28198E+00_JPRB, 0.28373E+00_JPRB, 0.28543E+00_JPRB, 0.28709E+00_JPRB, 0.28988E+00_JPRB, 0.29255E+00_JPRB, 0.29540E+00_JPRB ,&
&  0.29848E+00_JPRB, 0.30209E+00_JPRB, 0.30590E+00_JPRB, 0.31002E+00_JPRB, 0.31469E+00_JPRB, 0.31590E+00_JPRB, 0.31681E+00_JPRB ,&
&  0.31726E+00_JPRB, 0.32023E+00_JPRB, 0.32180E+00_JPRB, 0.32112E+00_JPRB, 0.32051E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.29605E+00_JPRB, 0.29612E+00_JPRB, 0.29612E+00_JPRB, 0.29608E+00_JPRB, 0.29454E+00_JPRB, 0.29492E+00_JPRB, 0.29533E+00_JPRB ,&
&  0.29608E+00_JPRB, 0.29769E+00_JPRB, 0.29951E+00_JPRB, 0.30087E+00_JPRB, 0.30181E+00_JPRB, 0.30239E+00_JPRB, 0.30283E+00_JPRB ,&
&  0.30290E+00_JPRB, 0.30318E+00_JPRB, 0.29959E+00_JPRB, 0.29830E+00_JPRB, 0.29786E+00_JPRB, 0.29785E+00_JPRB, 0.29805E+00_JPRB ,&
&  0.29817E+00_JPRB, 0.29814E+00_JPRB, 0.29756E+00_JPRB, 0.29788E+00_JPRB, 0.29823E+00_JPRB, 0.29846E+00_JPRB, 0.29871E+00_JPRB ,&
&  0.29895E+00_JPRB, 0.29916E+00_JPRB, 0.29910E+00_JPRB, 0.29886E+00_JPRB, 0.29874E+00_JPRB, 0.29826E+00_JPRB, 0.29740E+00_JPRB ,&
&  0.29634E+00_JPRB, 0.29501E+00_JPRB, 0.29369E+00_JPRB, 0.29248E+00_JPRB, 0.29144E+00_JPRB, 0.29055E+00_JPRB, 0.28969E+00_JPRB ,&
&  0.28864E+00_JPRB, 0.28725E+00_JPRB, 0.28579E+00_JPRB, 0.28436E+00_JPRB, 0.28298E+00_JPRB, 0.28174E+00_JPRB, 0.28041E+00_JPRB ,&
&  0.27941E+00_JPRB, 0.27855E+00_JPRB, 0.27772E+00_JPRB, 0.27700E+00_JPRB, 0.27629E+00_JPRB, 0.27561E+00_JPRB, 0.27497E+00_JPRB ,&
&  0.27409E+00_JPRB, 0.27362E+00_JPRB, 0.27309E+00_JPRB, 0.27287E+00_JPRB, 0.27296E+00_JPRB, 0.27302E+00_JPRB, 0.27318E+00_JPRB ,&
&  0.27341E+00_JPRB, 0.27369E+00_JPRB, 0.27384E+00_JPRB, 0.27407E+00_JPRB, 0.27442E+00_JPRB, 0.27486E+00_JPRB, 0.27539E+00_JPRB ,&
&  0.27597E+00_JPRB, 0.27659E+00_JPRB, 0.27728E+00_JPRB, 0.27804E+00_JPRB, 0.27884E+00_JPRB, 0.27972E+00_JPRB, 0.28055E+00_JPRB ,&
&  0.28139E+00_JPRB, 0.28217E+00_JPRB, 0.28296E+00_JPRB, 0.28356E+00_JPRB, 0.28444E+00_JPRB, 0.28529E+00_JPRB, 0.28631E+00_JPRB ,&
&  0.28733E+00_JPRB, 0.28842E+00_JPRB, 0.28937E+00_JPRB, 0.29041E+00_JPRB, 0.29157E+00_JPRB, 0.29132E+00_JPRB, 0.29118E+00_JPRB ,&
&  0.29109E+00_JPRB, 0.29132E+00_JPRB, 0.29122E+00_JPRB, 0.29152E+00_JPRB, 0.29239E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB, 0.28020E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB, 0.49620E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.50032E+00_JPRB, 0.50058E+00_JPRB, 0.50073E+00_JPRB, 0.50078E+00_JPRB, 0.50082E+00_JPRB, 0.50106E+00_JPRB, 0.50122E+00_JPRB ,&
&  0.50145E+00_JPRB, 0.50172E+00_JPRB, 0.50231E+00_JPRB, 0.50258E+00_JPRB, 0.50285E+00_JPRB, 0.50295E+00_JPRB, 0.50309E+00_JPRB ,&
&  0.50315E+00_JPRB, 0.50319E+00_JPRB, 0.50311E+00_JPRB, 0.50314E+00_JPRB, 0.50323E+00_JPRB, 0.50345E+00_JPRB, 0.50363E+00_JPRB ,&
&  0.50347E+00_JPRB, 0.50298E+00_JPRB, 0.50261E+00_JPRB, 0.50210E+00_JPRB, 0.50145E+00_JPRB, 0.50040E+00_JPRB, 0.49911E+00_JPRB ,&
&  0.49767E+00_JPRB, 0.49631E+00_JPRB, 0.49517E+00_JPRB, 0.49407E+00_JPRB, 0.49325E+00_JPRB, 0.49257E+00_JPRB, 0.49213E+00_JPRB ,&
&  0.49153E+00_JPRB, 0.49123E+00_JPRB, 0.49095E+00_JPRB, 0.49076E+00_JPRB, 0.49022E+00_JPRB, 0.48980E+00_JPRB, 0.48925E+00_JPRB ,&
&  0.48868E+00_JPRB, 0.48787E+00_JPRB, 0.48726E+00_JPRB, 0.48653E+00_JPRB, 0.48574E+00_JPRB, 0.48446E+00_JPRB, 0.48333E+00_JPRB ,&
&  0.48224E+00_JPRB, 0.48112E+00_JPRB, 0.47986E+00_JPRB, 0.47879E+00_JPRB, 0.47803E+00_JPRB, 0.47730E+00_JPRB, 0.47639E+00_JPRB ,&
&  0.47615E+00_JPRB, 0.47649E+00_JPRB, 0.47677E+00_JPRB, 0.47683E+00_JPRB, 0.47729E+00_JPRB, 0.47787E+00_JPRB, 0.47851E+00_JPRB ,&
&  0.47910E+00_JPRB, 0.47997E+00_JPRB, 0.48123E+00_JPRB, 0.48240E+00_JPRB, 0.48360E+00_JPRB, 0.48482E+00_JPRB, 0.48623E+00_JPRB ,&
&  0.48753E+00_JPRB, 0.48881E+00_JPRB, 0.48995E+00_JPRB, 0.49118E+00_JPRB, 0.49198E+00_JPRB, 0.49269E+00_JPRB, 0.49303E+00_JPRB ,&
&  0.49342E+00_JPRB, 0.49366E+00_JPRB, 0.49395E+00_JPRB, 0.49418E+00_JPRB, 0.49462E+00_JPRB, 0.49497E+00_JPRB, 0.49537E+00_JPRB ,&
&  0.49578E+00_JPRB, 0.49635E+00_JPRB, 0.49679E+00_JPRB, 0.49718E+00_JPRB, 0.49755E+00_JPRB, 0.49813E+00_JPRB, 0.49858E+00_JPRB ,&
&  0.49894E+00_JPRB, 0.49934E+00_JPRB, 0.49972E+00_JPRB, 0.50004E+00_JPRB, 0.50020E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.49498E+00_JPRB, 0.49676E+00_JPRB, 0.49780E+00_JPRB, 0.49878E+00_JPRB, 0.49973E+00_JPRB, 0.50122E+00_JPRB, 0.50240E+00_JPRB ,&
&  0.50362E+00_JPRB, 0.50530E+00_JPRB, 0.50758E+00_JPRB, 0.50904E+00_JPRB, 0.51016E+00_JPRB, 0.51085E+00_JPRB, 0.51146E+00_JPRB ,&
&  0.51157E+00_JPRB, 0.51127E+00_JPRB, 0.51062E+00_JPRB, 0.51013E+00_JPRB, 0.50962E+00_JPRB, 0.50913E+00_JPRB, 0.50842E+00_JPRB ,&
&  0.50700E+00_JPRB, 0.50516E+00_JPRB, 0.50313E+00_JPRB, 0.50014E+00_JPRB, 0.49678E+00_JPRB, 0.49315E+00_JPRB, 0.48922E+00_JPRB ,&
&  0.48521E+00_JPRB, 0.48199E+00_JPRB, 0.47890E+00_JPRB, 0.47594E+00_JPRB, 0.47467E+00_JPRB, 0.47356E+00_JPRB, 0.47241E+00_JPRB ,&
&  0.47125E+00_JPRB, 0.46959E+00_JPRB, 0.46922E+00_JPRB, 0.46914E+00_JPRB, 0.46943E+00_JPRB, 0.47055E+00_JPRB, 0.47154E+00_JPRB ,&
&  0.47217E+00_JPRB, 0.47212E+00_JPRB, 0.47172E+00_JPRB, 0.47167E+00_JPRB, 0.47172E+00_JPRB, 0.47139E+00_JPRB, 0.47042E+00_JPRB ,&
&  0.46971E+00_JPRB, 0.46850E+00_JPRB, 0.46705E+00_JPRB, 0.46538E+00_JPRB, 0.46440E+00_JPRB, 0.46319E+00_JPRB, 0.46187E+00_JPRB ,&
&  0.46104E+00_JPRB, 0.46080E+00_JPRB, 0.46074E+00_JPRB, 0.46078E+00_JPRB, 0.46144E+00_JPRB, 0.46237E+00_JPRB, 0.46330E+00_JPRB ,&
&  0.46432E+00_JPRB, 0.46617E+00_JPRB, 0.46836E+00_JPRB, 0.47023E+00_JPRB, 0.47202E+00_JPRB, 0.47374E+00_JPRB, 0.47584E+00_JPRB ,&
&  0.47735E+00_JPRB, 0.47845E+00_JPRB, 0.47870E+00_JPRB, 0.47965E+00_JPRB, 0.48034E+00_JPRB, 0.48143E+00_JPRB, 0.48194E+00_JPRB ,&
&  0.48312E+00_JPRB, 0.48364E+00_JPRB, 0.48387E+00_JPRB, 0.48422E+00_JPRB, 0.48472E+00_JPRB, 0.48507E+00_JPRB, 0.48508E+00_JPRB ,&
&  0.48507E+00_JPRB, 0.48521E+00_JPRB, 0.48529E+00_JPRB, 0.48532E+00_JPRB, 0.48577E+00_JPRB, 0.48640E+00_JPRB, 0.48713E+00_JPRB ,&
&  0.48801E+00_JPRB, 0.48945E+00_JPRB, 0.49113E+00_JPRB, 0.49237E+00_JPRB, 0.49353E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.48862E+00_JPRB, 0.49004E+00_JPRB, 0.49108E+00_JPRB, 0.49275E+00_JPRB, 0.49587E+00_JPRB, 0.49948E+00_JPRB, 0.50175E+00_JPRB ,&
&  0.50479E+00_JPRB, 0.50839E+00_JPRB, 0.51233E+00_JPRB, 0.51480E+00_JPRB, 0.51669E+00_JPRB, 0.51769E+00_JPRB, 0.51773E+00_JPRB ,&
&  0.51666E+00_JPRB, 0.51514E+00_JPRB, 0.51377E+00_JPRB, 0.51228E+00_JPRB, 0.51070E+00_JPRB, 0.50895E+00_JPRB, 0.50537E+00_JPRB ,&
&  0.50212E+00_JPRB, 0.50022E+00_JPRB, 0.49554E+00_JPRB, 0.48997E+00_JPRB, 0.48531E+00_JPRB, 0.45180E+00_JPRB, 0.44024E+00_JPRB ,&
&  0.43830E+00_JPRB, 0.43554E+00_JPRB, 0.43639E+00_JPRB, 0.43775E+00_JPRB, 0.44325E+00_JPRB, 0.43893E+00_JPRB, 0.43617E+00_JPRB ,&
&  0.41659E+00_JPRB, 0.41613E+00_JPRB, 0.41280E+00_JPRB, 0.41110E+00_JPRB, 0.41322E+00_JPRB, 0.41540E+00_JPRB, 0.42278E+00_JPRB ,&
&  0.44153E+00_JPRB, 0.44723E+00_JPRB, 0.44998E+00_JPRB, 0.45185E+00_JPRB, 0.45365E+00_JPRB, 0.45465E+00_JPRB, 0.45556E+00_JPRB ,&
&  0.45601E+00_JPRB, 0.45591E+00_JPRB, 0.45591E+00_JPRB, 0.45543E+00_JPRB, 0.45482E+00_JPRB, 0.45371E+00_JPRB, 0.45289E+00_JPRB ,&
&  0.45234E+00_JPRB, 0.45267E+00_JPRB, 0.45358E+00_JPRB, 0.45438E+00_JPRB, 0.45570E+00_JPRB, 0.45651E+00_JPRB, 0.45727E+00_JPRB ,&
&  0.45785E+00_JPRB, 0.45845E+00_JPRB, 0.45957E+00_JPRB, 0.46089E+00_JPRB, 0.46256E+00_JPRB, 0.46327E+00_JPRB, 0.46469E+00_JPRB ,&
&  0.46576E+00_JPRB, 0.46734E+00_JPRB, 0.46842E+00_JPRB, 0.47087E+00_JPRB, 0.47236E+00_JPRB, 0.47503E+00_JPRB, 0.47718E+00_JPRB ,&
&  0.48030E+00_JPRB, 0.48216E+00_JPRB, 0.48393E+00_JPRB, 0.48504E+00_JPRB, 0.48620E+00_JPRB, 0.48698E+00_JPRB, 0.48706E+00_JPRB ,&
&  0.48697E+00_JPRB, 0.48613E+00_JPRB, 0.48525E+00_JPRB, 0.48403E+00_JPRB, 0.48321E+00_JPRB, 0.48216E+00_JPRB, 0.48149E+00_JPRB ,&
&  0.48055E+00_JPRB, 0.48108E+00_JPRB, 0.48271E+00_JPRB, 0.48495E+00_JPRB, 0.48686E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.49937E+00_JPRB, 0.49935E+00_JPRB, 0.49877E+00_JPRB, 0.50041E+00_JPRB, 0.50265E+00_JPRB, 0.50712E+00_JPRB, 0.50801E+00_JPRB ,&
&  0.51213E+00_JPRB, 0.51624E+00_JPRB, 0.52066E+00_JPRB, 0.52251E+00_JPRB, 0.52397E+00_JPRB, 0.52379E+00_JPRB, 0.52215E+00_JPRB ,&
&  0.52002E+00_JPRB, 0.51647E+00_JPRB, 0.51354E+00_JPRB, 0.50976E+00_JPRB, 0.50570E+00_JPRB, 0.50328E+00_JPRB, 0.50018E+00_JPRB ,&
&  0.49848E+00_JPRB, 0.49563E+00_JPRB, 0.46431E+00_JPRB, 0.45371E+00_JPRB, 0.43743E+00_JPRB, 0.42800E+00_JPRB, 0.42293E+00_JPRB ,&
&  0.41724E+00_JPRB, 0.41168E+00_JPRB, 0.40427E+00_JPRB, 0.39427E+00_JPRB, 0.38257E+00_JPRB, 0.36636E+00_JPRB, 0.35879E+00_JPRB ,&
&  0.35200E+00_JPRB, 0.35045E+00_JPRB, 0.35174E+00_JPRB, 0.35595E+00_JPRB, 0.36472E+00_JPRB, 0.37444E+00_JPRB, 0.38683E+00_JPRB ,&
&  0.40425E+00_JPRB, 0.42167E+00_JPRB, 0.43125E+00_JPRB, 0.42974E+00_JPRB, 0.44097E+00_JPRB, 0.44651E+00_JPRB, 0.44950E+00_JPRB ,&
&  0.45074E+00_JPRB, 0.44915E+00_JPRB, 0.45279E+00_JPRB, 0.45278E+00_JPRB, 0.45291E+00_JPRB, 0.45344E+00_JPRB, 0.45447E+00_JPRB ,&
&  0.45670E+00_JPRB, 0.45896E+00_JPRB, 0.46049E+00_JPRB, 0.46172E+00_JPRB, 0.46217E+00_JPRB, 0.46191E+00_JPRB, 0.46149E+00_JPRB ,&
&  0.46120E+00_JPRB, 0.46102E+00_JPRB, 0.46107E+00_JPRB, 0.46143E+00_JPRB, 0.46263E+00_JPRB, 0.46332E+00_JPRB, 0.46349E+00_JPRB ,&
&  0.46452E+00_JPRB, 0.46620E+00_JPRB, 0.46807E+00_JPRB, 0.46735E+00_JPRB, 0.46969E+00_JPRB, 0.47655E+00_JPRB, 0.48009E+00_JPRB ,&
&  0.48619E+00_JPRB, 0.49090E+00_JPRB, 0.49643E+00_JPRB, 0.50065E+00_JPRB, 0.50417E+00_JPRB, 0.50556E+00_JPRB, 0.50499E+00_JPRB ,&
&  0.50406E+00_JPRB, 0.50239E+00_JPRB, 0.50126E+00_JPRB, 0.49898E+00_JPRB, 0.49720E+00_JPRB, 0.49467E+00_JPRB, 0.49342E+00_JPRB ,&
&  0.49211E+00_JPRB, 0.49238E+00_JPRB, 0.49151E+00_JPRB, 0.49425E+00_JPRB, 0.49669E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.51496E+00_JPRB, 0.51451E+00_JPRB, 0.51508E+00_JPRB, 0.51919E+00_JPRB, 0.52689E+00_JPRB, 0.53572E+00_JPRB, 0.53460E+00_JPRB ,&
&  0.53440E+00_JPRB, 0.53163E+00_JPRB, 0.53287E+00_JPRB, 0.53191E+00_JPRB, 0.53022E+00_JPRB, 0.52730E+00_JPRB, 0.52477E+00_JPRB ,&
&  0.52237E+00_JPRB, 0.51617E+00_JPRB, 0.50853E+00_JPRB, 0.50614E+00_JPRB, 0.50672E+00_JPRB, 0.50682E+00_JPRB, 0.50364E+00_JPRB ,&
&  0.49840E+00_JPRB, 0.48700E+00_JPRB, 0.46258E+00_JPRB, 0.44503E+00_JPRB, 0.42790E+00_JPRB, 0.42162E+00_JPRB, 0.41103E+00_JPRB ,&
&  0.41467E+00_JPRB, 0.38109E+00_JPRB, 0.38151E+00_JPRB, 0.36710E+00_JPRB, 0.35622E+00_JPRB, 0.34430E+00_JPRB, 0.33228E+00_JPRB ,&
&  0.32861E+00_JPRB, 0.32443E+00_JPRB, 0.32391E+00_JPRB, 0.33008E+00_JPRB, 0.33800E+00_JPRB, 0.35171E+00_JPRB, 0.36719E+00_JPRB ,&
&  0.37998E+00_JPRB, 0.41938E+00_JPRB, 0.43076E+00_JPRB, 0.43513E+00_JPRB, 0.44580E+00_JPRB, 0.45285E+00_JPRB, 0.45382E+00_JPRB ,&
&  0.45405E+00_JPRB, 0.45293E+00_JPRB, 0.45702E+00_JPRB, 0.45951E+00_JPRB, 0.46411E+00_JPRB, 0.46407E+00_JPRB, 0.46426E+00_JPRB ,&
&  0.46661E+00_JPRB, 0.46930E+00_JPRB, 0.47246E+00_JPRB, 0.47512E+00_JPRB, 0.47753E+00_JPRB, 0.47907E+00_JPRB, 0.47984E+00_JPRB ,&
&  0.47988E+00_JPRB, 0.47932E+00_JPRB, 0.47873E+00_JPRB, 0.47925E+00_JPRB, 0.47972E+00_JPRB, 0.48206E+00_JPRB, 0.48305E+00_JPRB ,&
&  0.48405E+00_JPRB, 0.48420E+00_JPRB, 0.48357E+00_JPRB, 0.48432E+00_JPRB, 0.48729E+00_JPRB, 0.49023E+00_JPRB, 0.49550E+00_JPRB ,&
&  0.50372E+00_JPRB, 0.50916E+00_JPRB, 0.51550E+00_JPRB, 0.52096E+00_JPRB, 0.52543E+00_JPRB, 0.52583E+00_JPRB, 0.52378E+00_JPRB ,&
&  0.52219E+00_JPRB, 0.52031E+00_JPRB, 0.51836E+00_JPRB, 0.51515E+00_JPRB, 0.51195E+00_JPRB, 0.50807E+00_JPRB, 0.50544E+00_JPRB ,&
&  0.50377E+00_JPRB, 0.50497E+00_JPRB, 0.50404E+00_JPRB, 0.50718E+00_JPRB, 0.51117E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.52811E+00_JPRB, 0.52977E+00_JPRB, 0.53475E+00_JPRB, 0.54101E+00_JPRB, 0.54773E+00_JPRB, 0.55506E+00_JPRB, 0.55795E+00_JPRB ,&
&  0.55981E+00_JPRB, 0.55687E+00_JPRB, 0.55237E+00_JPRB, 0.54029E+00_JPRB, 0.53218E+00_JPRB, 0.52822E+00_JPRB, 0.52348E+00_JPRB ,&
&  0.51833E+00_JPRB, 0.51531E+00_JPRB, 0.51326E+00_JPRB, 0.51256E+00_JPRB, 0.50970E+00_JPRB, 0.50789E+00_JPRB, 0.50027E+00_JPRB ,&
&  0.48529E+00_JPRB, 0.47176E+00_JPRB, 0.45115E+00_JPRB, 0.43306E+00_JPRB, 0.42077E+00_JPRB, 0.41018E+00_JPRB, 0.40960E+00_JPRB ,&
&  0.40737E+00_JPRB, 0.40596E+00_JPRB, 0.40418E+00_JPRB, 0.40231E+00_JPRB, 0.40429E+00_JPRB, 0.34516E+00_JPRB, 0.33060E+00_JPRB ,&
&  0.32107E+00_JPRB, 0.31414E+00_JPRB, 0.31255E+00_JPRB, 0.31314E+00_JPRB, 0.31877E+00_JPRB, 0.33597E+00_JPRB, 0.35944E+00_JPRB ,&
&  0.37733E+00_JPRB, 0.43082E+00_JPRB, 0.44072E+00_JPRB, 0.45004E+00_JPRB, 0.46464E+00_JPRB, 0.47419E+00_JPRB, 0.47812E+00_JPRB ,&
&  0.48473E+00_JPRB, 0.49555E+00_JPRB, 0.50767E+00_JPRB, 0.51862E+00_JPRB, 0.51842E+00_JPRB, 0.52045E+00_JPRB, 0.51859E+00_JPRB ,&
&  0.51273E+00_JPRB, 0.50520E+00_JPRB, 0.50283E+00_JPRB, 0.50188E+00_JPRB, 0.50121E+00_JPRB, 0.50151E+00_JPRB, 0.50033E+00_JPRB ,&
&  0.50219E+00_JPRB, 0.50196E+00_JPRB, 0.50653E+00_JPRB, 0.50822E+00_JPRB, 0.51175E+00_JPRB, 0.51610E+00_JPRB, 0.51839E+00_JPRB ,&
&  0.51834E+00_JPRB, 0.51858E+00_JPRB, 0.51497E+00_JPRB, 0.50937E+00_JPRB, 0.50745E+00_JPRB, 0.50516E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.51129E+00_JPRB, 0.51966E+00_JPRB, 0.52398E+00_JPRB, 0.52877E+00_JPRB, 0.53300E+00_JPRB, 0.53323E+00_JPRB, 0.53384E+00_JPRB ,&
&  0.53367E+00_JPRB, 0.53409E+00_JPRB, 0.53258E+00_JPRB, 0.52964E+00_JPRB, 0.52469E+00_JPRB, 0.51923E+00_JPRB, 0.51421E+00_JPRB ,&
&  0.51319E+00_JPRB, 0.51533E+00_JPRB, 0.51532E+00_JPRB, 0.51811E+00_JPRB, 0.52310E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.54066E+00_JPRB, 0.54375E+00_JPRB, 0.55119E+00_JPRB, 0.55732E+00_JPRB, 0.56580E+00_JPRB, 0.57270E+00_JPRB, 0.57535E+00_JPRB ,&
&  0.58339E+00_JPRB, 0.57926E+00_JPRB, 0.56570E+00_JPRB, 0.54848E+00_JPRB, 0.53099E+00_JPRB, 0.51964E+00_JPRB, 0.51441E+00_JPRB ,&
&  0.51401E+00_JPRB, 0.50861E+00_JPRB, 0.50500E+00_JPRB, 0.50337E+00_JPRB, 0.50057E+00_JPRB, 0.49788E+00_JPRB, 0.49085E+00_JPRB ,&
&  0.47380E+00_JPRB, 0.46168E+00_JPRB, 0.44875E+00_JPRB, 0.43169E+00_JPRB, 0.41708E+00_JPRB, 0.40931E+00_JPRB, 0.41204E+00_JPRB ,&
&  0.41378E+00_JPRB, 0.40684E+00_JPRB, 0.40575E+00_JPRB, 0.40322E+00_JPRB, 0.39860E+00_JPRB, 0.39514E+00_JPRB, 0.34676E+00_JPRB ,&
&  0.33061E+00_JPRB, 0.31618E+00_JPRB, 0.31102E+00_JPRB, 0.31139E+00_JPRB, 0.32322E+00_JPRB, 0.34268E+00_JPRB, 0.37785E+00_JPRB ,&
&  0.41226E+00_JPRB, 0.44338E+00_JPRB, 0.46319E+00_JPRB, 0.47861E+00_JPRB, 0.49505E+00_JPRB, 0.51052E+00_JPRB, 0.52116E+00_JPRB ,&
&  0.53770E+00_JPRB, 0.55891E+00_JPRB, 0.57758E+00_JPRB, 0.59433E+00_JPRB, 0.60492E+00_JPRB, 0.60698E+00_JPRB, 0.60850E+00_JPRB ,&
&  0.60162E+00_JPRB, 0.55163E+00_JPRB, 0.53084E+00_JPRB, 0.51807E+00_JPRB, 0.50980E+00_JPRB, 0.50956E+00_JPRB, 0.50696E+00_JPRB ,&
&  0.51059E+00_JPRB, 0.51619E+00_JPRB, 0.51851E+00_JPRB, 0.52112E+00_JPRB, 0.52384E+00_JPRB, 0.52481E+00_JPRB, 0.52895E+00_JPRB ,&
&  0.53683E+00_JPRB, 0.54865E+00_JPRB, 0.57045E+00_JPRB, 0.55582E+00_JPRB, 0.53772E+00_JPRB, 0.52029E+00_JPRB, 0.51113E+00_JPRB ,&
&  0.50924E+00_JPRB, 0.51254E+00_JPRB, 0.51914E+00_JPRB, 0.52207E+00_JPRB, 0.52529E+00_JPRB, 0.52888E+00_JPRB, 0.53798E+00_JPRB ,&
&  0.54051E+00_JPRB, 0.54317E+00_JPRB, 0.54370E+00_JPRB, 0.54268E+00_JPRB, 0.53749E+00_JPRB, 0.53195E+00_JPRB, 0.52763E+00_JPRB ,&
&  0.52734E+00_JPRB, 0.52934E+00_JPRB, 0.53072E+00_JPRB, 0.53322E+00_JPRB, 0.53718E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.54519E+00_JPRB, 0.55392E+00_JPRB, 0.56815E+00_JPRB, 0.57880E+00_JPRB, 0.58893E+00_JPRB, 0.58084E+00_JPRB, 0.57154E+00_JPRB ,&
&  0.56933E+00_JPRB, 0.56633E+00_JPRB, 0.55557E+00_JPRB, 0.54226E+00_JPRB, 0.52443E+00_JPRB, 0.51067E+00_JPRB, 0.50580E+00_JPRB ,&
&  0.50534E+00_JPRB, 0.50193E+00_JPRB, 0.49732E+00_JPRB, 0.48924E+00_JPRB, 0.48144E+00_JPRB, 0.47983E+00_JPRB, 0.47489E+00_JPRB ,&
&  0.46399E+00_JPRB, 0.45614E+00_JPRB, 0.44761E+00_JPRB, 0.43605E+00_JPRB, 0.42460E+00_JPRB, 0.41984E+00_JPRB, 0.41810E+00_JPRB ,&
&  0.41657E+00_JPRB, 0.41661E+00_JPRB, 0.41836E+00_JPRB, 0.41263E+00_JPRB, 0.41242E+00_JPRB, 0.41419E+00_JPRB, 0.37899E+00_JPRB ,&
&  0.34667E+00_JPRB, 0.32547E+00_JPRB, 0.31400E+00_JPRB, 0.31439E+00_JPRB, 0.33083E+00_JPRB, 0.36446E+00_JPRB, 0.40462E+00_JPRB ,&
&  0.44440E+00_JPRB, 0.47782E+00_JPRB, 0.50406E+00_JPRB, 0.52099E+00_JPRB, 0.53429E+00_JPRB, 0.55037E+00_JPRB, 0.56825E+00_JPRB ,&
&  0.59153E+00_JPRB, 0.62158E+00_JPRB, 0.64712E+00_JPRB, 0.67329E+00_JPRB, 0.69031E+00_JPRB, 0.66067E+00_JPRB, 0.70128E+00_JPRB ,&
&  0.84849E+00_JPRB, 0.70034E+00_JPRB, 0.58722E+00_JPRB, 0.54453E+00_JPRB, 0.53704E+00_JPRB, 0.54185E+00_JPRB, 0.54869E+00_JPRB ,&
&  0.55730E+00_JPRB, 0.55889E+00_JPRB, 0.55494E+00_JPRB, 0.55098E+00_JPRB, 0.55245E+00_JPRB, 0.55781E+00_JPRB, 0.57361E+00_JPRB ,&
&  0.59989E+00_JPRB, 0.71857E+00_JPRB, 0.91858E+00_JPRB, 0.69384E+00_JPRB, 0.60991E+00_JPRB, 0.55395E+00_JPRB, 0.51127E+00_JPRB ,&
&  0.49444E+00_JPRB, 0.49099E+00_JPRB, 0.49606E+00_JPRB, 0.49975E+00_JPRB, 0.49932E+00_JPRB, 0.50220E+00_JPRB, 0.50784E+00_JPRB ,&
&  0.51926E+00_JPRB, 0.53122E+00_JPRB, 0.53888E+00_JPRB, 0.54451E+00_JPRB, 0.54369E+00_JPRB, 0.53828E+00_JPRB, 0.53172E+00_JPRB ,&
&  0.53435E+00_JPRB, 0.53966E+00_JPRB, 0.54340E+00_JPRB, 0.54485E+00_JPRB, 0.54402E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.54541E+00_JPRB, 0.57207E+00_JPRB, 0.60296E+00_JPRB, 0.60488E+00_JPRB, 0.60340E+00_JPRB, 0.57148E+00_JPRB, 0.53784E+00_JPRB ,&
&  0.51638E+00_JPRB, 0.50557E+00_JPRB, 0.50175E+00_JPRB, 0.49399E+00_JPRB, 0.48722E+00_JPRB, 0.48411E+00_JPRB, 0.48172E+00_JPRB ,&
&  0.47998E+00_JPRB, 0.47766E+00_JPRB, 0.47454E+00_JPRB, 0.46815E+00_JPRB, 0.46503E+00_JPRB, 0.46762E+00_JPRB, 0.46540E+00_JPRB ,&
&  0.45861E+00_JPRB, 0.45512E+00_JPRB, 0.44410E+00_JPRB, 0.42983E+00_JPRB, 0.42313E+00_JPRB, 0.42444E+00_JPRB, 0.42977E+00_JPRB ,&
&  0.42567E+00_JPRB, 0.42046E+00_JPRB, 0.42692E+00_JPRB, 0.43634E+00_JPRB, 0.42869E+00_JPRB, 0.42715E+00_JPRB, 0.38938E+00_JPRB ,&
&  0.35322E+00_JPRB, 0.33162E+00_JPRB, 0.32602E+00_JPRB, 0.34942E+00_JPRB, 0.39422E+00_JPRB, 0.45362E+00_JPRB, 0.48749E+00_JPRB ,&
&  0.51052E+00_JPRB, 0.53568E+00_JPRB, 0.55821E+00_JPRB, 0.58129E+00_JPRB, 0.61159E+00_JPRB, 0.64239E+00_JPRB, 0.67817E+00_JPRB ,&
&  0.70501E+00_JPRB, 0.71920E+00_JPRB, 0.71602E+00_JPRB, 0.64832E+00_JPRB, 0.63306E+00_JPRB, 0.62866E+00_JPRB, 0.68041E+00_JPRB ,&
&  0.73561E+00_JPRB, 0.67742E+00_JPRB, 0.63692E+00_JPRB, 0.63631E+00_JPRB, 0.63210E+00_JPRB, 0.64129E+00_JPRB, 0.64351E+00_JPRB ,&
&  0.63157E+00_JPRB, 0.61048E+00_JPRB, 0.58914E+00_JPRB, 0.57318E+00_JPRB, 0.57521E+00_JPRB, 0.59167E+00_JPRB, 0.61012E+00_JPRB ,&
&  0.64499E+00_JPRB, 0.72337E+00_JPRB, 0.71508E+00_JPRB, 0.55736E+00_JPRB, 0.53352E+00_JPRB, 0.52298E+00_JPRB, 0.51176E+00_JPRB ,&
&  0.49362E+00_JPRB, 0.48298E+00_JPRB, 0.48257E+00_JPRB, 0.48341E+00_JPRB, 0.47767E+00_JPRB, 0.47225E+00_JPRB, 0.47067E+00_JPRB ,&
&  0.47459E+00_JPRB, 0.48379E+00_JPRB, 0.49620E+00_JPRB, 0.51972E+00_JPRB, 0.53269E+00_JPRB, 0.53596E+00_JPRB, 0.52935E+00_JPRB ,&
&  0.52550E+00_JPRB, 0.52792E+00_JPRB, 0.52835E+00_JPRB, 0.52591E+00_JPRB, 0.53075E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.63189E+00_JPRB, 0.63532E+00_JPRB, 0.62545E+00_JPRB, 0.62210E+00_JPRB, 0.59907E+00_JPRB, 0.58434E+00_JPRB, 0.55547E+00_JPRB ,&
&  0.54738E+00_JPRB, 0.52133E+00_JPRB, 0.49402E+00_JPRB, 0.46484E+00_JPRB, 0.45042E+00_JPRB, 0.44510E+00_JPRB, 0.44730E+00_JPRB ,&
&  0.45295E+00_JPRB, 0.45951E+00_JPRB, 0.46309E+00_JPRB, 0.45643E+00_JPRB, 0.45907E+00_JPRB, 0.46180E+00_JPRB, 0.46045E+00_JPRB ,&
&  0.45755E+00_JPRB, 0.45711E+00_JPRB, 0.45066E+00_JPRB, 0.44022E+00_JPRB, 0.43415E+00_JPRB, 0.43218E+00_JPRB, 0.42796E+00_JPRB ,&
&  0.42009E+00_JPRB, 0.41940E+00_JPRB, 0.43179E+00_JPRB, 0.45617E+00_JPRB, 0.43839E+00_JPRB, 0.43352E+00_JPRB, 0.38854E+00_JPRB ,&
&  0.35511E+00_JPRB, 0.35074E+00_JPRB, 0.37731E+00_JPRB, 0.44377E+00_JPRB, 0.47009E+00_JPRB, 0.50264E+00_JPRB, 0.52225E+00_JPRB ,&
&  0.52167E+00_JPRB, 0.53717E+00_JPRB, 0.57940E+00_JPRB, 0.65849E+00_JPRB, 0.68944E+00_JPRB, 0.73001E+00_JPRB, 0.76980E+00_JPRB ,&
&  0.79884E+00_JPRB, 0.81668E+00_JPRB, 0.82176E+00_JPRB, 0.77895E+00_JPRB, 0.79438E+00_JPRB, 0.85451E+00_JPRB, 0.75165E+00_JPRB ,&
&  0.74262E+00_JPRB, 0.71197E+00_JPRB, 0.71430E+00_JPRB, 0.75164E+00_JPRB, 0.71858E+00_JPRB, 0.72758E+00_JPRB, 0.71047E+00_JPRB ,&
&  0.66188E+00_JPRB, 0.63451E+00_JPRB, 0.61532E+00_JPRB, 0.61052E+00_JPRB, 0.61319E+00_JPRB, 0.62899E+00_JPRB, 0.65458E+00_JPRB ,&
&  0.68940E+00_JPRB, 0.70353E+00_JPRB, 0.66074E+00_JPRB, 0.56825E+00_JPRB, 0.53403E+00_JPRB, 0.51876E+00_JPRB, 0.50722E+00_JPRB ,&
&  0.49443E+00_JPRB, 0.48655E+00_JPRB, 0.48716E+00_JPRB, 0.48653E+00_JPRB, 0.47963E+00_JPRB, 0.46592E+00_JPRB, 0.45423E+00_JPRB ,&
&  0.44934E+00_JPRB, 0.45059E+00_JPRB, 0.45117E+00_JPRB, 0.47271E+00_JPRB, 0.49847E+00_JPRB, 0.51994E+00_JPRB, 0.52229E+00_JPRB ,&
&  0.52371E+00_JPRB, 0.53426E+00_JPRB, 0.54758E+00_JPRB, 0.57713E+00_JPRB, 0.60191E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.64838E+00_JPRB, 0.65417E+00_JPRB, 0.65142E+00_JPRB, 0.63948E+00_JPRB, 0.62764E+00_JPRB, 0.59693E+00_JPRB, 0.52870E+00_JPRB ,&
&  0.52831E+00_JPRB, 0.48259E+00_JPRB, 0.46100E+00_JPRB, 0.44769E+00_JPRB, 0.44567E+00_JPRB, 0.43895E+00_JPRB, 0.43520E+00_JPRB ,&
&  0.43558E+00_JPRB, 0.44652E+00_JPRB, 0.45527E+00_JPRB, 0.45227E+00_JPRB, 0.45596E+00_JPRB, 0.45964E+00_JPRB, 0.45851E+00_JPRB ,&
&  0.46162E+00_JPRB, 0.46484E+00_JPRB, 0.46482E+00_JPRB, 0.45896E+00_JPRB, 0.44663E+00_JPRB, 0.43777E+00_JPRB, 0.42833E+00_JPRB ,&
&  0.42202E+00_JPRB, 0.42273E+00_JPRB, 0.43298E+00_JPRB, 0.45590E+00_JPRB, 0.44665E+00_JPRB, 0.43555E+00_JPRB, 0.41921E+00_JPRB ,&
&  0.37381E+00_JPRB, 0.39148E+00_JPRB, 0.45653E+00_JPRB, 0.48311E+00_JPRB, 0.51032E+00_JPRB, 0.54270E+00_JPRB, 0.58773E+00_JPRB ,&
&  0.62688E+00_JPRB, 0.66801E+00_JPRB, 0.71369E+00_JPRB, 0.76592E+00_JPRB, 0.82316E+00_JPRB, 0.87327E+00_JPRB, 0.90471E+00_JPRB ,&
&  0.94030E+00_JPRB, 0.77974E+00_JPRB, 0.76294E+00_JPRB, 0.88714E+00_JPRB, 0.10831E+01_JPRB, 0.90690E+00_JPRB, 0.84948E+00_JPRB ,&
&  0.82059E+00_JPRB, 0.77922E+00_JPRB, 0.76261E+00_JPRB, 0.80348E+00_JPRB, 0.80516E+00_JPRB, 0.82933E+00_JPRB, 0.84750E+00_JPRB ,&
&  0.77717E+00_JPRB, 0.73705E+00_JPRB, 0.70339E+00_JPRB, 0.70190E+00_JPRB, 0.71318E+00_JPRB, 0.72905E+00_JPRB, 0.73487E+00_JPRB ,&
&  0.72841E+00_JPRB, 0.71180E+00_JPRB, 0.67405E+00_JPRB, 0.62011E+00_JPRB, 0.58148E+00_JPRB, 0.55890E+00_JPRB, 0.54137E+00_JPRB ,&
&  0.52575E+00_JPRB, 0.50944E+00_JPRB, 0.49587E+00_JPRB, 0.48806E+00_JPRB, 0.48504E+00_JPRB, 0.48677E+00_JPRB, 0.49891E+00_JPRB ,&
&  0.48405E+00_JPRB, 0.47544E+00_JPRB, 0.46407E+00_JPRB, 0.47702E+00_JPRB, 0.50567E+00_JPRB, 0.53202E+00_JPRB, 0.54714E+00_JPRB ,&
&  0.57006E+00_JPRB, 0.58159E+00_JPRB, 0.57626E+00_JPRB, 0.59347E+00_JPRB, 0.61355E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.66116E+00_JPRB, 0.67141E+00_JPRB, 0.66797E+00_JPRB, 0.66751E+00_JPRB, 0.65332E+00_JPRB, 0.60053E+00_JPRB, 0.55284E+00_JPRB ,&
&  0.61191E+00_JPRB, 0.54105E+00_JPRB, 0.56282E+00_JPRB, 0.48149E+00_JPRB, 0.45625E+00_JPRB, 0.44884E+00_JPRB, 0.43184E+00_JPRB ,&
&  0.41334E+00_JPRB, 0.42232E+00_JPRB, 0.43444E+00_JPRB, 0.43818E+00_JPRB, 0.44387E+00_JPRB, 0.45104E+00_JPRB, 0.45071E+00_JPRB ,&
&  0.45495E+00_JPRB, 0.46387E+00_JPRB, 0.47499E+00_JPRB, 0.46626E+00_JPRB, 0.45341E+00_JPRB, 0.44664E+00_JPRB, 0.43837E+00_JPRB ,&
&  0.42519E+00_JPRB, 0.41954E+00_JPRB, 0.42975E+00_JPRB, 0.44937E+00_JPRB, 0.45929E+00_JPRB, 0.44288E+00_JPRB, 0.43398E+00_JPRB ,&
&  0.43391E+00_JPRB, 0.43878E+00_JPRB, 0.48889E+00_JPRB, 0.51742E+00_JPRB, 0.55516E+00_JPRB, 0.60339E+00_JPRB, 0.65096E+00_JPRB ,&
&  0.70557E+00_JPRB, 0.76260E+00_JPRB, 0.82424E+00_JPRB, 0.89523E+00_JPRB, 0.95963E+00_JPRB, 0.10220E+01_JPRB, 0.10726E+01_JPRB ,&
&  0.11377E+01_JPRB, 0.86948E+00_JPRB, 0.10556E+01_JPRB, 0.11384E+01_JPRB, 0.13077E+01_JPRB, 0.12686E+01_JPRB, 0.11509E+01_JPRB ,&
&  0.11833E+01_JPRB, 0.10436E+01_JPRB, 0.95643E+00_JPRB, 0.88496E+00_JPRB, 0.87131E+00_JPRB, 0.91676E+00_JPRB, 0.96113E+00_JPRB ,&
&  0.10253E+01_JPRB, 0.99777E+00_JPRB, 0.94109E+00_JPRB, 0.93074E+00_JPRB, 0.93062E+00_JPRB, 0.92641E+00_JPRB, 0.90105E+00_JPRB ,&
&  0.85825E+00_JPRB, 0.80725E+00_JPRB, 0.75484E+00_JPRB, 0.68866E+00_JPRB, 0.65470E+00_JPRB, 0.62534E+00_JPRB, 0.59315E+00_JPRB ,&
&  0.57417E+00_JPRB, 0.53983E+00_JPRB, 0.50418E+00_JPRB, 0.48309E+00_JPRB, 0.46719E+00_JPRB, 0.46287E+00_JPRB, 0.47368E+00_JPRB ,&
&  0.47868E+00_JPRB, 0.47740E+00_JPRB, 0.51189E+00_JPRB, 0.53861E+00_JPRB, 0.55423E+00_JPRB, 0.57139E+00_JPRB, 0.61924E+00_JPRB ,&
&  0.59306E+00_JPRB, 0.60865E+00_JPRB, 0.64663E+00_JPRB, 0.65147E+00_JPRB, 0.65491E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.68541E+00_JPRB, 0.70320E+00_JPRB, 0.69203E+00_JPRB, 0.69219E+00_JPRB, 0.65953E+00_JPRB, 0.64170E+00_JPRB, 0.57913E+00_JPRB ,&
&  0.60014E+00_JPRB, 0.57960E+00_JPRB, 0.55291E+00_JPRB, 0.54508E+00_JPRB, 0.55505E+00_JPRB, 0.48946E+00_JPRB, 0.43468E+00_JPRB ,&
&  0.40639E+00_JPRB, 0.40013E+00_JPRB, 0.41663E+00_JPRB, 0.42705E+00_JPRB, 0.43024E+00_JPRB, 0.43554E+00_JPRB, 0.43128E+00_JPRB ,&
&  0.44794E+00_JPRB, 0.45709E+00_JPRB, 0.46798E+00_JPRB, 0.46933E+00_JPRB, 0.46309E+00_JPRB, 0.46051E+00_JPRB, 0.46028E+00_JPRB ,&
&  0.44646E+00_JPRB, 0.44092E+00_JPRB, 0.43143E+00_JPRB, 0.43051E+00_JPRB, 0.47496E+00_JPRB, 0.46834E+00_JPRB, 0.46077E+00_JPRB ,&
&  0.46845E+00_JPRB, 0.48877E+00_JPRB, 0.51929E+00_JPRB, 0.55923E+00_JPRB, 0.60760E+00_JPRB, 0.65792E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.76304E+00_JPRB, 0.82768E+00_JPRB, 0.87492E+00_JPRB, 0.96348E+00_JPRB, 0.10262E+01_JPRB, 0.96939E+00_JPRB, 0.11371E+01_JPRB ,&
&  0.12326E+01_JPRB, 0.13666E+01_JPRB, 0.14961E+01_JPRB, 0.13810E+01_JPRB, 0.15583E+01_JPRB, 0.14726E+01_JPRB, 0.13023E+01_JPRB ,&
&  0.13011E+01_JPRB, 0.12047E+01_JPRB, 0.11773E+01_JPRB, 0.11592E+01_JPRB, 0.12121E+01_JPRB, 0.11729E+01_JPRB, 0.11548E+01_JPRB ,&
&  0.12207E+01_JPRB, 0.14164E+01_JPRB, 0.13491E+01_JPRB, 0.13747E+01_JPRB, 0.13403E+01_JPRB, 0.12946E+01_JPRB, 0.11926E+01_JPRB ,&
&  0.11071E+01_JPRB, 0.10087E+01_JPRB, 0.91612E+00_JPRB, 0.84190E+00_JPRB, 0.75729E+00_JPRB, 0.69441E+00_JPRB, 0.64600E+00_JPRB ,&
&  0.59446E+00_JPRB, 0.53037E+00_JPRB, 0.50257E+00_JPRB, 0.48387E+00_JPRB, 0.47384E+00_JPRB, 0.45301E+00_JPRB, 0.45325E+00_JPRB ,&
&  0.47258E+00_JPRB, 0.51205E+00_JPRB, 0.57354E+00_JPRB, 0.59936E+00_JPRB, 0.63741E+00_JPRB, 0.68013E+00_JPRB, 0.63917E+00_JPRB ,&
&  0.60616E+00_JPRB, 0.67300E+00_JPRB, 0.68363E+00_JPRB, 0.68788E+00_JPRB, 0.68306E+00_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.74968E+00_JPRB, 0.76095E+00_JPRB, 0.74505E+00_JPRB, 0.72447E+00_JPRB, 0.67638E+00_JPRB, 0.64811E+00_JPRB, 0.60725E+00_JPRB ,&
&  0.60845E+00_JPRB, 0.58979E+00_JPRB, 0.56560E+00_JPRB, 0.55671E+00_JPRB, 0.55906E+00_JPRB, 0.54559E+00_JPRB, 0.48288E+00_JPRB ,&
&  0.43479E+00_JPRB, 0.41959E+00_JPRB, 0.40664E+00_JPRB, 0.41448E+00_JPRB, 0.42665E+00_JPRB, 0.42793E+00_JPRB, 0.43746E+00_JPRB ,&
&  0.45693E+00_JPRB, 0.46383E+00_JPRB, 0.46483E+00_JPRB, 0.46419E+00_JPRB, 0.46394E+00_JPRB, 0.46981E+00_JPRB, 0.47098E+00_JPRB ,&
&  0.45514E+00_JPRB, 0.44234E+00_JPRB, 0.43071E+00_JPRB, 0.43364E+00_JPRB, 0.47700E+00_JPRB, 0.49375E+00_JPRB, 0.49397E+00_JPRB ,&
&  0.50420E+00_JPRB, 0.52081E+00_JPRB, 0.54194E+00_JPRB, 0.58121E+00_JPRB, 0.63503E+00_JPRB, 0.69862E+00_JPRB, 0.76504E+00_JPRB ,&
&  0.82984E+00_JPRB, 0.90062E+00_JPRB, 0.95736E+00_JPRB, 0.10268E+01_JPRB, 0.10044E+01_JPRB, 0.10588E+01_JPRB, 0.12183E+01_JPRB ,&
&  0.13263E+01_JPRB, 0.14600E+01_JPRB, 0.15395E+01_JPRB, 0.17138E+01_JPRB, 0.18457E+01_JPRB, 0.17189E+01_JPRB, 0.15678E+01_JPRB ,&
&  0.15024E+01_JPRB, 0.14406E+01_JPRB, 0.18135E+01_JPRB, 0.16329E+01_JPRB, 0.14918E+01_JPRB, 0.14673E+01_JPRB, 0.14221E+01_JPRB ,&
&  0.14792E+01_JPRB, 0.15978E+01_JPRB, 0.16587E+01_JPRB, 0.17115E+01_JPRB, 0.17191E+01_JPRB, 0.17206E+01_JPRB, 0.16157E+01_JPRB ,&
&  0.15420E+01_JPRB, 0.13986E+01_JPRB, 0.11032E+01_JPRB, 0.88758E+00_JPRB, 0.72938E+00_JPRB, 0.71692E+00_JPRB, 0.66591E+00_JPRB ,&
&  0.59991E+00_JPRB, 0.53123E+00_JPRB, 0.51741E+00_JPRB, 0.49903E+00_JPRB, 0.49500E+00_JPRB, 0.48426E+00_JPRB, 0.46762E+00_JPRB ,&
&  0.48144E+00_JPRB, 0.53826E+00_JPRB, 0.57142E+00_JPRB, 0.63150E+00_JPRB, 0.70534E+00_JPRB, 0.78707E+00_JPRB, 0.67719E+00_JPRB ,&
&  0.69608E+00_JPRB, 0.76837E+00_JPRB, 0.78410E+00_JPRB, 0.78398E+00_JPRB, 0.76277E+00_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.87810E+00_JPRB, 0.86156E+00_JPRB, 0.82871E+00_JPRB, 0.77429E+00_JPRB, 0.70892E+00_JPRB, 0.65893E+00_JPRB, 0.61337E+00_JPRB ,&
&  0.60719E+00_JPRB, 0.58900E+00_JPRB, 0.56845E+00_JPRB, 0.56393E+00_JPRB, 0.56282E+00_JPRB, 0.55844E+00_JPRB, 0.55406E+00_JPRB ,&
&  0.47892E+00_JPRB, 0.45856E+00_JPRB, 0.42847E+00_JPRB, 0.43087E+00_JPRB, 0.47112E+00_JPRB, 0.44576E+00_JPRB, 0.45231E+00_JPRB ,&
&  0.45937E+00_JPRB, 0.47132E+00_JPRB, 0.46623E+00_JPRB, 0.46601E+00_JPRB, 0.47489E+00_JPRB, 0.48531E+00_JPRB, 0.48596E+00_JPRB ,&
&  0.46949E+00_JPRB, 0.45166E+00_JPRB, 0.44727E+00_JPRB, 0.45367E+00_JPRB, 0.49575E+00_JPRB, 0.52927E+00_JPRB, 0.54898E+00_JPRB ,&
&  0.56710E+00_JPRB, 0.58219E+00_JPRB, 0.59479E+00_JPRB, 0.62461E+00_JPRB, 0.67230E+00_JPRB, 0.73040E+00_JPRB, 0.81421E+00_JPRB ,&
&  0.89897E+00_JPRB, 0.97701E+00_JPRB, 0.10597E+01_JPRB, 0.11376E+01_JPRB, 0.11366E+01_JPRB, 0.12538E+01_JPRB, 0.13737E+01_JPRB ,&
&  0.14927E+01_JPRB, 0.14443E+01_JPRB, 0.14527E+01_JPRB, 0.18312E+01_JPRB, 0.21139E+01_JPRB, 0.20939E+01_JPRB, 0.19299E+01_JPRB ,&
&  0.17725E+01_JPRB, 0.16588E+01_JPRB, 0.17404E+01_JPRB, 0.17056E+01_JPRB, 0.15764E+01_JPRB, 0.16353E+01_JPRB, 0.16465E+01_JPRB ,&
&  0.16977E+01_JPRB, 0.18069E+01_JPRB, 0.19418E+01_JPRB, 0.20391E+01_JPRB, 0.19877E+01_JPRB, 0.20175E+01_JPRB, 0.19510E+01_JPRB ,&
&  0.18631E+01_JPRB, 0.15696E+01_JPRB, 0.11241E+01_JPRB, 0.89123E+00_JPRB, 0.66744E+00_JPRB, 0.63474E+00_JPRB, 0.71995E+00_JPRB ,&
&  0.69579E+00_JPRB, 0.65489E+00_JPRB, 0.60587E+00_JPRB, 0.56863E+00_JPRB, 0.54772E+00_JPRB, 0.56069E+00_JPRB, 0.55105E+00_JPRB ,&
&  0.53803E+00_JPRB, 0.59614E+00_JPRB, 0.69808E+00_JPRB, 0.82226E+00_JPRB, 0.90586E+00_JPRB, 0.95224E+00_JPRB, 0.87896E+00_JPRB ,&
&  0.94409E+00_JPRB, 0.93542E+00_JPRB, 0.93450E+00_JPRB, 0.94086E+00_JPRB, 0.91908E+00_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.10115E+01_JPRB, 0.97416E+00_JPRB, 0.91022E+00_JPRB, 0.82169E+00_JPRB, 0.72734E+00_JPRB, 0.65669E+00_JPRB, 0.62286E+00_JPRB ,&
&  0.61472E+00_JPRB, 0.59811E+00_JPRB, 0.57559E+00_JPRB, 0.56868E+00_JPRB, 0.56616E+00_JPRB, 0.56013E+00_JPRB, 0.56287E+00_JPRB ,&
&  0.58453E+00_JPRB, 0.54037E+00_JPRB, 0.47692E+00_JPRB, 0.43625E+00_JPRB, 0.45517E+00_JPRB, 0.45306E+00_JPRB, 0.46094E+00_JPRB ,&
&  0.46384E+00_JPRB, 0.48308E+00_JPRB, 0.46875E+00_JPRB, 0.47448E+00_JPRB, 0.50055E+00_JPRB, 0.51910E+00_JPRB, 0.52791E+00_JPRB ,&
&  0.52432E+00_JPRB, 0.51501E+00_JPRB, 0.51937E+00_JPRB, 0.54549E+00_JPRB, 0.59574E+00_JPRB, 0.58771E+00_JPRB, 0.62334E+00_JPRB ,&
&  0.63883E+00_JPRB, 0.66007E+00_JPRB, 0.66519E+00_JPRB, 0.69862E+00_JPRB, 0.75916E+00_JPRB, 0.85881E+00_JPRB, 0.93543E+00_JPRB ,&
&  0.99868E+00_JPRB, 0.10515E+01_JPRB, 0.10810E+01_JPRB, 0.11648E+01_JPRB, 0.12847E+01_JPRB, 0.14376E+01_JPRB, 0.14078E+01_JPRB ,&
&  0.14225E+01_JPRB, 0.13580E+01_JPRB, 0.13028E+01_JPRB, 0.16350E+01_JPRB, 0.19977E+01_JPRB, 0.21047E+01_JPRB, 0.20978E+01_JPRB ,&
&  0.20428E+01_JPRB, 0.19719E+01_JPRB, 0.20468E+01_JPRB, 0.19710E+01_JPRB, 0.19122E+01_JPRB, 0.19497E+01_JPRB, 0.20221E+01_JPRB ,&
&  0.20201E+01_JPRB, 0.21204E+01_JPRB, 0.22693E+01_JPRB, 0.22790E+01_JPRB, 0.22510E+01_JPRB, 0.21643E+01_JPRB, 0.21671E+01_JPRB ,&
&  0.21023E+01_JPRB, 0.12789E+01_JPRB, 0.99709E+00_JPRB, 0.10233E+01_JPRB, 0.76514E+00_JPRB, 0.65809E+00_JPRB, 0.68764E+00_JPRB ,&
&  0.65831E+00_JPRB, 0.64948E+00_JPRB, 0.67052E+00_JPRB, 0.65618E+00_JPRB, 0.64739E+00_JPRB, 0.69210E+00_JPRB, 0.71754E+00_JPRB ,&
&  0.76106E+00_JPRB, 0.79723E+00_JPRB, 0.98777E+00_JPRB, 0.11121E+01_JPRB, 0.11523E+01_JPRB, 0.11545E+01_JPRB, 0.11375E+01_JPRB ,&
&  0.11090E+01_JPRB, 0.10833E+01_JPRB, 0.10657E+01_JPRB, 0.10595E+01_JPRB, 0.10494E+01_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.11555E+01_JPRB, 0.11055E+01_JPRB, 0.10174E+01_JPRB, 0.90454E+00_JPRB, 0.78145E+00_JPRB, 0.69489E+00_JPRB, 0.67088E+00_JPRB ,&
&  0.65557E+00_JPRB, 0.64198E+00_JPRB, 0.61814E+00_JPRB, 0.60232E+00_JPRB, 0.59478E+00_JPRB, 0.58584E+00_JPRB, 0.58640E+00_JPRB ,&
&  0.58798E+00_JPRB, 0.58090E+00_JPRB, 0.49520E+00_JPRB, 0.43550E+00_JPRB, 0.40373E+00_JPRB, 0.42131E+00_JPRB, 0.43868E+00_JPRB ,&
&  0.46148E+00_JPRB, 0.48478E+00_JPRB, 0.49074E+00_JPRB, 0.53508E+00_JPRB, 0.57887E+00_JPRB, 0.59996E+00_JPRB, 0.63273E+00_JPRB ,&
&  0.63826E+00_JPRB, 0.63078E+00_JPRB, 0.62750E+00_JPRB, 0.67679E+00_JPRB, 0.69140E+00_JPRB, 0.66002E+00_JPRB, 0.71931E+00_JPRB ,&
&  0.74873E+00_JPRB, 0.77637E+00_JPRB, 0.79925E+00_JPRB, 0.85158E+00_JPRB, 0.91452E+00_JPRB, 0.94492E+00_JPRB, 0.93459E+00_JPRB ,&
&  0.95980E+00_JPRB, 0.10116E+01_JPRB, 0.10871E+01_JPRB, 0.11541E+01_JPRB, 0.12413E+01_JPRB, 0.13302E+01_JPRB, 0.12983E+01_JPRB ,&
&  0.13193E+01_JPRB, 0.11583E+01_JPRB, 0.10209E+01_JPRB, 0.13006E+01_JPRB, 0.18597E+01_JPRB, 0.21246E+01_JPRB, 0.20406E+01_JPRB ,&
&  0.21976E+01_JPRB, 0.22526E+01_JPRB, 0.23563E+01_JPRB, 0.23191E+01_JPRB, 0.22382E+01_JPRB, 0.22596E+01_JPRB, 0.23920E+01_JPRB ,&
&  0.24424E+01_JPRB, 0.25738E+01_JPRB, 0.26486E+01_JPRB, 0.26287E+01_JPRB, 0.25751E+01_JPRB, 0.25897E+01_JPRB, 0.26101E+01_JPRB ,&
&  0.23769E+01_JPRB, 0.19810E+01_JPRB, 0.13779E+01_JPRB, 0.13028E+01_JPRB, 0.91029E+00_JPRB, 0.73937E+00_JPRB, 0.73190E+00_JPRB ,&
&  0.66631E+00_JPRB, 0.68255E+00_JPRB, 0.72309E+00_JPRB, 0.72890E+00_JPRB, 0.82757E+00_JPRB, 0.91374E+00_JPRB, 0.10363E+01_JPRB ,&
&  0.11399E+01_JPRB, 0.11366E+01_JPRB, 0.13403E+01_JPRB, 0.13795E+01_JPRB, 0.13915E+01_JPRB, 0.14050E+01_JPRB, 0.13997E+01_JPRB ,&
&  0.13570E+01_JPRB, 0.13049E+01_JPRB, 0.12617E+01_JPRB, 0.12194E+01_JPRB, 0.11882E+01_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.12142E+01_JPRB, 0.11437E+01_JPRB, 0.10312E+01_JPRB, 0.91895E+00_JPRB, 0.80261E+00_JPRB, 0.71165E+00_JPRB, 0.69707E+00_JPRB ,&
&  0.72248E+00_JPRB, 0.72103E+00_JPRB, 0.70703E+00_JPRB, 0.67617E+00_JPRB, 0.65761E+00_JPRB, 0.64456E+00_JPRB, 0.62959E+00_JPRB ,&
&  0.61474E+00_JPRB, 0.55587E+00_JPRB, 0.45770E+00_JPRB, 0.40348E+00_JPRB, 0.38714E+00_JPRB, 0.38588E+00_JPRB, 0.41504E+00_JPRB ,&
&  0.45163E+00_JPRB, 0.51161E+00_JPRB, 0.57032E+00_JPRB, 0.64034E+00_JPRB, 0.72945E+00_JPRB, 0.81724E+00_JPRB, 0.84639E+00_JPRB ,&
&  0.86495E+00_JPRB, 0.75710E+00_JPRB, 0.79122E+00_JPRB, 0.78824E+00_JPRB, 0.82713E+00_JPRB, 0.83001E+00_JPRB, 0.83918E+00_JPRB ,&
&  0.83059E+00_JPRB, 0.85671E+00_JPRB, 0.88497E+00_JPRB, 0.91190E+00_JPRB, 0.92655E+00_JPRB, 0.93461E+00_JPRB, 0.94372E+00_JPRB ,&
&  0.10028E+01_JPRB, 0.10727E+01_JPRB, 0.11209E+01_JPRB, 0.11582E+01_JPRB, 0.12291E+01_JPRB, 0.13229E+01_JPRB, 0.13439E+01_JPRB ,&
&  0.12737E+01_JPRB, 0.12154E+01_JPRB, 0.13757E+01_JPRB, 0.15662E+01_JPRB, 0.17924E+01_JPRB, 0.21063E+01_JPRB, 0.22633E+01_JPRB ,&
&  0.23617E+01_JPRB, 0.23006E+01_JPRB, 0.23430E+01_JPRB, 0.22145E+01_JPRB, 0.25083E+01_JPRB, 0.25668E+01_JPRB, 0.26964E+01_JPRB ,&
&  0.28188E+01_JPRB, 0.30212E+01_JPRB, 0.29444E+01_JPRB, 0.28622E+01_JPRB, 0.29295E+01_JPRB, 0.31147E+01_JPRB, 0.27988E+01_JPRB ,&
&  0.23475E+01_JPRB, 0.25166E+01_JPRB, 0.22079E+01_JPRB, 0.18072E+01_JPRB, 0.13640E+01_JPRB, 0.11581E+01_JPRB, 0.98272E+00_JPRB ,&
&  0.86170E+00_JPRB, 0.80615E+00_JPRB, 0.81947E+00_JPRB, 0.95488E+00_JPRB, 0.11761E+01_JPRB, 0.13733E+01_JPRB, 0.14839E+01_JPRB ,&
&  0.15240E+01_JPRB, 0.18382E+01_JPRB, 0.18136E+01_JPRB, 0.17346E+01_JPRB, 0.17166E+01_JPRB, 0.17082E+01_JPRB, 0.16598E+01_JPRB ,&
&  0.15490E+01_JPRB, 0.14635E+01_JPRB, 0.13943E+01_JPRB, 0.13315E+01_JPRB, 0.12674E+01_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.12450E+01_JPRB, 0.11793E+01_JPRB, 0.10653E+01_JPRB, 0.94357E+00_JPRB, 0.83297E+00_JPRB, 0.74858E+00_JPRB, 0.72886E+00_JPRB ,&
&  0.74415E+00_JPRB, 0.76015E+00_JPRB, 0.77166E+00_JPRB, 0.77416E+00_JPRB, 0.76627E+00_JPRB, 0.73923E+00_JPRB, 0.70551E+00_JPRB ,&
&  0.67861E+00_JPRB, 0.55250E+00_JPRB, 0.44611E+00_JPRB, 0.41240E+00_JPRB, 0.38962E+00_JPRB, 0.38016E+00_JPRB, 0.39880E+00_JPRB ,&
&  0.45716E+00_JPRB, 0.58349E+00_JPRB, 0.72179E+00_JPRB, 0.87756E+00_JPRB, 0.10102E+01_JPRB, 0.10960E+01_JPRB, 0.10689E+01_JPRB ,&
&  0.99367E+00_JPRB, 0.98739E+00_JPRB, 0.99221E+00_JPRB, 0.96002E+00_JPRB, 0.95374E+00_JPRB, 0.92547E+00_JPRB, 0.90861E+00_JPRB ,&
&  0.90151E+00_JPRB, 0.90199E+00_JPRB, 0.94587E+00_JPRB, 0.97872E+00_JPRB, 0.10049E+01_JPRB, 0.10286E+01_JPRB, 0.10540E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.11117E+01_JPRB, 0.11344E+01_JPRB, 0.12179E+01_JPRB, 0.10659E+01_JPRB, 0.11155E+01_JPRB, 0.12440E+01_JPRB ,&
&  0.15116E+01_JPRB, 0.15372E+01_JPRB, 0.14864E+01_JPRB, 0.16395E+01_JPRB, 0.15651E+01_JPRB, 0.17207E+01_JPRB, 0.21115E+01_JPRB ,&
&  0.22549E+01_JPRB, 0.22832E+01_JPRB, 0.24253E+01_JPRB, 0.21247E+01_JPRB, 0.21264E+01_JPRB, 0.30471E+01_JPRB, 0.33001E+01_JPRB ,&
&  0.33703E+01_JPRB, 0.34005E+01_JPRB, 0.32676E+01_JPRB, 0.32937E+01_JPRB, 0.28771E+01_JPRB, 0.25209E+01_JPRB, 0.19325E+01_JPRB ,&
&  0.20576E+01_JPRB, 0.19860E+01_JPRB, 0.21966E+01_JPRB, 0.19970E+01_JPRB, 0.18425E+01_JPRB, 0.16196E+01_JPRB, 0.13169E+01_JPRB ,&
&  0.11284E+01_JPRB, 0.10306E+01_JPRB, 0.10407E+01_JPRB, 0.13426E+01_JPRB, 0.17305E+01_JPRB, 0.18129E+01_JPRB, 0.21634E+01_JPRB ,&
&  0.23214E+01_JPRB, 0.23644E+01_JPRB, 0.20760E+01_JPRB, 0.20181E+01_JPRB, 0.19067E+01_JPRB, 0.18024E+01_JPRB, 0.16820E+01_JPRB ,&
&  0.15830E+01_JPRB, 0.15026E+01_JPRB, 0.14276E+01_JPRB, 0.13550E+01_JPRB, 0.12895E+01_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.12662E+01_JPRB, 0.12111E+01_JPRB, 0.10998E+01_JPRB, 0.97684E+00_JPRB, 0.87011E+00_JPRB, 0.80004E+00_JPRB, 0.78506E+00_JPRB ,&
&  0.80162E+00_JPRB, 0.81873E+00_JPRB, 0.82611E+00_JPRB, 0.82084E+00_JPRB, 0.80841E+00_JPRB, 0.80001E+00_JPRB, 0.78384E+00_JPRB ,&
&  0.74716E+00_JPRB, 0.62091E+00_JPRB, 0.47705E+00_JPRB, 0.42277E+00_JPRB, 0.42143E+00_JPRB, 0.40469E+00_JPRB, 0.41215E+00_JPRB ,&
&  0.52462E+00_JPRB, 0.72676E+00_JPRB, 0.98671E+00_JPRB, 0.11985E+01_JPRB, 0.13409E+01_JPRB, 0.13341E+01_JPRB, 0.12428E+01_JPRB ,&
&  0.12313E+01_JPRB, 0.11730E+01_JPRB, 0.10561E+01_JPRB, 0.96502E+00_JPRB, 0.92369E+00_JPRB, 0.90743E+00_JPRB, 0.88447E+00_JPRB ,&
&  0.86752E+00_JPRB, 0.87562E+00_JPRB, 0.90407E+00_JPRB, 0.92506E+00_JPRB, 0.95847E+00_JPRB, 0.97116E+00_JPRB, 0.97556E+00_JPRB ,&
&  0.10069E+01_JPRB, 0.10610E+01_JPRB, 0.11301E+01_JPRB, 0.12395E+01_JPRB, 0.11603E+01_JPRB, 0.12076E+01_JPRB, 0.13973E+01_JPRB ,&
&  0.15619E+01_JPRB, 0.15920E+01_JPRB, 0.15805E+01_JPRB, 0.16083E+01_JPRB, 0.17196E+01_JPRB, 0.17447E+01_JPRB, 0.20512E+01_JPRB ,&
&  0.19874E+01_JPRB, 0.18951E+01_JPRB, 0.19026E+01_JPRB, 0.18555E+01_JPRB, 0.23890E+01_JPRB, 0.33752E+01_JPRB, 0.38897E+01_JPRB ,&
&  0.38622E+01_JPRB, 0.37908E+01_JPRB, 0.36988E+01_JPRB, 0.32728E+01_JPRB, 0.25876E+01_JPRB, 0.19572E+01_JPRB, 0.23214E+01_JPRB ,&
&  0.25156E+01_JPRB, 0.25949E+01_JPRB, 0.24631E+01_JPRB, 0.20633E+01_JPRB, 0.17847E+01_JPRB, 0.19741E+01_JPRB, 0.17721E+01_JPRB ,&
&  0.14625E+01_JPRB, 0.13378E+01_JPRB, 0.16112E+01_JPRB, 0.19975E+01_JPRB, 0.22163E+01_JPRB, 0.23796E+01_JPRB, 0.26774E+01_JPRB ,&
&  0.25708E+01_JPRB, 0.24967E+01_JPRB, 0.23099E+01_JPRB, 0.20804E+01_JPRB, 0.18910E+01_JPRB, 0.17482E+01_JPRB, 0.16349E+01_JPRB ,&
&  0.15705E+01_JPRB, 0.15203E+01_JPRB, 0.14518E+01_JPRB, 0.13844E+01_JPRB, 0.13142E+01_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.12920E+01_JPRB, 0.12378E+01_JPRB, 0.11217E+01_JPRB, 0.99305E+00_JPRB, 0.90833E+00_JPRB, 0.85752E+00_JPRB, 0.83706E+00_JPRB ,&
&  0.84032E+00_JPRB, 0.84326E+00_JPRB, 0.84450E+00_JPRB, 0.83867E+00_JPRB, 0.82933E+00_JPRB, 0.81783E+00_JPRB, 0.81682E+00_JPRB ,&
&  0.79149E+00_JPRB, 0.75220E+00_JPRB, 0.56262E+00_JPRB, 0.48415E+00_JPRB, 0.46845E+00_JPRB, 0.46974E+00_JPRB, 0.47561E+00_JPRB ,&
&  0.65864E+00_JPRB, 0.10073E+01_JPRB, 0.13097E+01_JPRB, 0.15102E+01_JPRB, 0.15931E+01_JPRB, 0.13767E+01_JPRB, 0.13028E+01_JPRB ,&
&  0.13245E+01_JPRB, 0.12690E+01_JPRB, 0.11449E+01_JPRB, 0.10350E+01_JPRB, 0.94552E+00_JPRB, 0.90519E+00_JPRB, 0.88871E+00_JPRB ,&
&  0.87579E+00_JPRB, 0.87928E+00_JPRB, 0.89688E+00_JPRB, 0.93320E+00_JPRB, 0.95347E+00_JPRB, 0.96470E+00_JPRB, 0.98161E+00_JPRB ,&
&  0.10048E+01_JPRB, 0.10460E+01_JPRB, 0.11048E+01_JPRB, 0.12639E+01_JPRB, 0.12870E+01_JPRB, 0.14223E+01_JPRB, 0.17446E+01_JPRB ,&
&  0.17694E+01_JPRB, 0.18281E+01_JPRB, 0.17464E+01_JPRB, 0.17319E+01_JPRB, 0.17550E+01_JPRB, 0.17920E+01_JPRB, 0.20653E+01_JPRB ,&
&  0.20390E+01_JPRB, 0.20602E+01_JPRB, 0.22584E+01_JPRB, 0.24175E+01_JPRB, 0.26273E+01_JPRB, 0.31648E+01_JPRB, 0.44662E+01_JPRB ,&
&  0.39108E+01_JPRB, 0.38896E+01_JPRB, 0.38281E+01_JPRB, 0.34806E+01_JPRB, 0.18492E+01_JPRB, 0.13798E+01_JPRB, 0.20778E+01_JPRB ,&
&  0.21991E+01_JPRB, 0.13818E+01_JPRB, 0.10347E+01_JPRB, 0.14142E+01_JPRB, 0.11254E+01_JPRB, 0.13349E+01_JPRB, 0.18093E+01_JPRB ,&
&  0.18467E+01_JPRB, 0.16881E+01_JPRB, 0.20264E+01_JPRB, 0.22010E+01_JPRB, 0.22743E+01_JPRB, 0.28033E+01_JPRB, 0.28849E+01_JPRB ,&
&  0.27759E+01_JPRB, 0.23777E+01_JPRB, 0.23192E+01_JPRB, 0.20605E+01_JPRB, 0.18762E+01_JPRB, 0.17700E+01_JPRB, 0.16888E+01_JPRB ,&
&  0.15916E+01_JPRB, 0.14929E+01_JPRB, 0.14219E+01_JPRB, 0.13802E+01_JPRB, 0.13330E+01_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.12925E+01_JPRB, 0.12364E+01_JPRB, 0.11161E+01_JPRB, 0.99031E+00_JPRB, 0.92417E+00_JPRB, 0.87973E+00_JPRB, 0.85431E+00_JPRB ,&
&  0.85263E+00_JPRB, 0.85732E+00_JPRB, 0.85778E+00_JPRB, 0.85528E+00_JPRB, 0.84955E+00_JPRB, 0.84467E+00_JPRB, 0.84103E+00_JPRB ,&
&  0.81944E+00_JPRB, 0.76567E+00_JPRB, 0.66227E+00_JPRB, 0.58706E+00_JPRB, 0.56626E+00_JPRB, 0.55657E+00_JPRB, 0.60774E+00_JPRB ,&
&  0.81184E+00_JPRB, 0.12637E+01_JPRB, 0.16080E+01_JPRB, 0.17143E+01_JPRB, 0.15516E+01_JPRB, 0.13768E+01_JPRB, 0.14304E+01_JPRB ,&
&  0.14375E+01_JPRB, 0.13493E+01_JPRB, 0.11940E+01_JPRB, 0.10909E+01_JPRB, 0.10084E+01_JPRB, 0.95819E+00_JPRB, 0.93790E+00_JPRB ,&
&  0.91311E+00_JPRB, 0.90917E+00_JPRB, 0.93668E+00_JPRB, 0.95381E+00_JPRB, 0.96194E+00_JPRB, 0.96977E+00_JPRB, 0.98028E+00_JPRB ,&
&  0.10129E+01_JPRB, 0.10711E+01_JPRB, 0.11614E+01_JPRB, 0.13171E+01_JPRB, 0.16378E+01_JPRB, 0.18511E+01_JPRB, 0.17915E+01_JPRB ,&
&  0.16793E+01_JPRB, 0.17187E+01_JPRB, 0.19169E+01_JPRB, 0.17808E+01_JPRB, 0.17245E+01_JPRB, 0.17738E+01_JPRB, 0.19967E+01_JPRB ,&
&  0.23624E+01_JPRB, 0.25751E+01_JPRB, 0.24436E+01_JPRB, 0.25974E+01_JPRB, 0.28606E+01_JPRB, 0.27428E+01_JPRB, 0.37657E+01_JPRB ,&
&  0.41102E+01_JPRB, 0.35549E+01_JPRB, 0.27249E+01_JPRB, 0.20172E+01_JPRB, 0.19828E+01_JPRB, 0.15277E+01_JPRB, 0.94719E+00_JPRB ,&
&  0.84583E+00_JPRB, 0.79466E+00_JPRB, 0.73248E+00_JPRB, 0.81543E+00_JPRB, 0.82375E+00_JPRB, 0.11052E+01_JPRB, 0.16770E+01_JPRB ,&
&  0.21249E+01_JPRB, 0.21656E+01_JPRB, 0.22939E+01_JPRB, 0.22979E+01_JPRB, 0.22566E+01_JPRB, 0.27808E+01_JPRB, 0.29498E+01_JPRB ,&
&  0.24602E+01_JPRB, 0.21897E+01_JPRB, 0.21286E+01_JPRB, 0.19132E+01_JPRB, 0.18205E+01_JPRB, 0.17485E+01_JPRB, 0.16425E+01_JPRB ,&
&  0.15116E+01_JPRB, 0.13906E+01_JPRB, 0.13374E+01_JPRB, 0.13272E+01_JPRB, 0.13254E+01_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.12839E+01_JPRB, 0.12229E+01_JPRB, 0.11194E+01_JPRB, 0.10134E+01_JPRB, 0.96455E+00_JPRB, 0.92995E+00_JPRB, 0.90289E+00_JPRB ,&
&  0.89418E+00_JPRB, 0.89033E+00_JPRB, 0.89298E+00_JPRB, 0.89640E+00_JPRB, 0.89391E+00_JPRB, 0.88728E+00_JPRB, 0.87668E+00_JPRB ,&
&  0.85517E+00_JPRB, 0.79424E+00_JPRB, 0.73905E+00_JPRB, 0.68059E+00_JPRB, 0.67241E+00_JPRB, 0.65860E+00_JPRB, 0.74599E+00_JPRB ,&
&  0.10196E+01_JPRB, 0.15328E+01_JPRB, 0.18674E+01_JPRB, 0.18678E+01_JPRB, 0.17175E+01_JPRB, 0.16047E+01_JPRB, 0.15698E+01_JPRB ,&
&  0.14814E+01_JPRB, 0.13888E+01_JPRB, 0.12499E+01_JPRB, 0.11453E+01_JPRB, 0.10658E+01_JPRB, 0.10118E+01_JPRB, 0.97780E+00_JPRB ,&
&  0.95204E+00_JPRB, 0.97073E+00_JPRB, 0.97140E+00_JPRB, 0.97735E+00_JPRB, 0.97770E+00_JPRB, 0.98040E+00_JPRB, 0.99933E+00_JPRB ,&
&  0.10536E+01_JPRB, 0.11507E+01_JPRB, 0.13210E+01_JPRB, 0.15988E+01_JPRB, 0.17052E+01_JPRB, 0.15592E+01_JPRB, 0.16757E+01_JPRB ,&
&  0.17375E+01_JPRB, 0.18646E+01_JPRB, 0.18140E+01_JPRB, 0.18848E+01_JPRB, 0.17757E+01_JPRB, 0.17276E+01_JPRB, 0.19770E+01_JPRB ,&
&  0.24116E+01_JPRB, 0.29352E+01_JPRB, 0.25314E+01_JPRB, 0.24439E+01_JPRB, 0.31731E+01_JPRB, 0.30626E+01_JPRB, 0.34316E+01_JPRB ,&
&  0.38713E+01_JPRB, 0.32860E+01_JPRB, 0.26775E+01_JPRB, 0.21784E+01_JPRB, 0.27213E+01_JPRB, 0.26874E+01_JPRB, 0.13828E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.84087E+00_JPRB, 0.75863E+00_JPRB, 0.73493E+00_JPRB, 0.75100E+00_JPRB, 0.10670E+01_JPRB, 0.18861E+01_JPRB ,&
&  0.21092E+01_JPRB, 0.24577E+01_JPRB, 0.25500E+01_JPRB, 0.24988E+01_JPRB, 0.24294E+01_JPRB, 0.24688E+01_JPRB, 0.24047E+01_JPRB ,&
&  0.24566E+01_JPRB, 0.22588E+01_JPRB, 0.20021E+01_JPRB, 0.18091E+01_JPRB, 0.16762E+01_JPRB, 0.15247E+01_JPRB, 0.13880E+01_JPRB ,&
&  0.12913E+01_JPRB, 0.12585E+01_JPRB, 0.12544E+01_JPRB, 0.12790E+01_JPRB, 0.13143E+01_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.12750E+01_JPRB, 0.12451E+01_JPRB, 0.11568E+01_JPRB, 0.10613E+01_JPRB, 0.10129E+01_JPRB, 0.97916E+00_JPRB, 0.95641E+00_JPRB ,&
&  0.94275E+00_JPRB, 0.93482E+00_JPRB, 0.92590E+00_JPRB, 0.91671E+00_JPRB, 0.91604E+00_JPRB, 0.90369E+00_JPRB, 0.88961E+00_JPRB ,&
&  0.86741E+00_JPRB, 0.81455E+00_JPRB, 0.76136E+00_JPRB, 0.76506E+00_JPRB, 0.76006E+00_JPRB, 0.77509E+00_JPRB, 0.91296E+00_JPRB ,&
&  0.13138E+01_JPRB, 0.18965E+01_JPRB, 0.20884E+01_JPRB, 0.20499E+01_JPRB, 0.18976E+01_JPRB, 0.17533E+01_JPRB, 0.15327E+01_JPRB ,&
&  0.13471E+01_JPRB, 0.13096E+01_JPRB, 0.11994E+01_JPRB, 0.11525E+01_JPRB, 0.11126E+01_JPRB, 0.10691E+01_JPRB, 0.10353E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.99822E+00_JPRB, 0.99145E+00_JPRB, 0.99041E+00_JPRB, 0.99114E+00_JPRB, 0.10008E+01_JPRB, 0.10593E+01_JPRB ,&
&  0.11646E+01_JPRB, 0.12974E+01_JPRB, 0.15291E+01_JPRB, 0.17461E+01_JPRB, 0.14686E+01_JPRB, 0.16098E+01_JPRB, 0.16497E+01_JPRB ,&
&  0.16604E+01_JPRB, 0.16900E+01_JPRB, 0.16034E+01_JPRB, 0.16466E+01_JPRB, 0.17355E+01_JPRB, 0.17359E+01_JPRB, 0.19177E+01_JPRB ,&
&  0.26073E+01_JPRB, 0.24431E+01_JPRB, 0.22511E+01_JPRB, 0.23497E+01_JPRB, 0.29358E+01_JPRB, 0.44767E+01_JPRB, 0.31580E+01_JPRB ,&
&  0.28752E+01_JPRB, 0.32444E+01_JPRB, 0.27470E+01_JPRB, 0.24178E+01_JPRB, 0.28669E+01_JPRB, 0.34803E+01_JPRB, 0.31877E+01_JPRB ,&
&  0.13851E+01_JPRB, 0.94324E+00_JPRB, 0.92671E+00_JPRB, 0.92230E+00_JPRB, 0.87338E+00_JPRB, 0.10970E+01_JPRB, 0.27008E+01_JPRB ,&
&  0.23624E+01_JPRB, 0.26872E+01_JPRB, 0.26771E+01_JPRB, 0.24978E+01_JPRB, 0.24827E+01_JPRB, 0.24835E+01_JPRB, 0.24003E+01_JPRB ,&
&  0.21268E+01_JPRB, 0.18945E+01_JPRB, 0.16954E+01_JPRB, 0.14933E+01_JPRB, 0.13358E+01_JPRB, 0.12303E+01_JPRB, 0.11817E+01_JPRB ,&
&  0.11649E+01_JPRB, 0.11720E+01_JPRB, 0.11977E+01_JPRB, 0.12360E+01_JPRB, 0.12719E+01_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.12681E+01_JPRB, 0.12625E+01_JPRB, 0.11810E+01_JPRB, 0.10949E+01_JPRB, 0.10441E+01_JPRB, 0.10151E+01_JPRB, 0.98113E+00_JPRB ,&
&  0.10115E+01_JPRB, 0.98591E+00_JPRB, 0.94117E+00_JPRB, 0.90954E+00_JPRB, 0.90625E+00_JPRB, 0.89291E+00_JPRB, 0.88185E+00_JPRB ,&
&  0.85945E+00_JPRB, 0.82595E+00_JPRB, 0.78532E+00_JPRB, 0.77166E+00_JPRB, 0.81281E+00_JPRB, 0.78188E+00_JPRB, 0.92705E+00_JPRB ,&
&  0.15645E+01_JPRB, 0.23498E+01_JPRB, 0.22029E+01_JPRB, 0.21008E+01_JPRB, 0.19232E+01_JPRB, 0.17746E+01_JPRB, 0.14197E+01_JPRB ,&
&  0.12744E+01_JPRB, 0.12583E+01_JPRB, 0.11817E+01_JPRB, 0.11653E+01_JPRB, 0.11425E+01_JPRB, 0.11141E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.10305E+01_JPRB, 0.10070E+01_JPRB, 0.10193E+01_JPRB, 0.10277E+01_JPRB, 0.10293E+01_JPRB, 0.10782E+01_JPRB, 0.11723E+01_JPRB ,&
&  0.12779E+01_JPRB, 0.14387E+01_JPRB, 0.16803E+01_JPRB, 0.16749E+01_JPRB, 0.15922E+01_JPRB, 0.15809E+01_JPRB, 0.16178E+01_JPRB ,&
&  0.15919E+01_JPRB, 0.16081E+01_JPRB, 0.16030E+01_JPRB, 0.15663E+01_JPRB, 0.16446E+01_JPRB, 0.17738E+01_JPRB, 0.19337E+01_JPRB ,&
&  0.22780E+01_JPRB, 0.21417E+01_JPRB, 0.20116E+01_JPRB, 0.22689E+01_JPRB, 0.28596E+01_JPRB, 0.40303E+01_JPRB, 0.41375E+01_JPRB ,&
&  0.36408E+01_JPRB, 0.26796E+01_JPRB, 0.24060E+01_JPRB, 0.27831E+01_JPRB, 0.31062E+01_JPRB, 0.33954E+01_JPRB, 0.44992E+01_JPRB ,&
&  0.44120E+01_JPRB, 0.24918E+01_JPRB, 0.19959E+01_JPRB, 0.24694E+01_JPRB, 0.17163E+01_JPRB, 0.12699E+01_JPRB, 0.18655E+01_JPRB ,&
&  0.22564E+01_JPRB, 0.25635E+01_JPRB, 0.26269E+01_JPRB, 0.25287E+01_JPRB, 0.25464E+01_JPRB, 0.23777E+01_JPRB, 0.20311E+01_JPRB ,&
&  0.17457E+01_JPRB, 0.15053E+01_JPRB, 0.13362E+01_JPRB, 0.12400E+01_JPRB, 0.12019E+01_JPRB, 0.11784E+01_JPRB, 0.11546E+01_JPRB ,&
&  0.11446E+01_JPRB, 0.11495E+01_JPRB, 0.11797E+01_JPRB, 0.12308E+01_JPRB, 0.12655E+01_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.12617E+01_JPRB, 0.12417E+01_JPRB, 0.11704E+01_JPRB, 0.11012E+01_JPRB, 0.10557E+01_JPRB, 0.10263E+01_JPRB, 0.10015E+01_JPRB ,&
&  0.11634E+01_JPRB, 0.10318E+01_JPRB, 0.94033E+00_JPRB, 0.90080E+00_JPRB, 0.88314E+00_JPRB, 0.87536E+00_JPRB, 0.86445E+00_JPRB ,&
&  0.85169E+00_JPRB, 0.83111E+00_JPRB, 0.81523E+00_JPRB, 0.82114E+00_JPRB, 0.87837E+00_JPRB, 0.11053E+01_JPRB, 0.10816E+01_JPRB ,&
&  0.16645E+01_JPRB, 0.26977E+01_JPRB, 0.22977E+01_JPRB, 0.21393E+01_JPRB, 0.19566E+01_JPRB, 0.18580E+01_JPRB, 0.14301E+01_JPRB ,&
&  0.12830E+01_JPRB, 0.12442E+01_JPRB, 0.12004E+01_JPRB, 0.11756E+01_JPRB, 0.11703E+01_JPRB, 0.11504E+01_JPRB, 0.10701E+01_JPRB ,&
&  0.10246E+01_JPRB, 0.10239E+01_JPRB, 0.10528E+01_JPRB, 0.10734E+01_JPRB, 0.11092E+01_JPRB, 0.11952E+01_JPRB, 0.12667E+01_JPRB ,&
&  0.13847E+01_JPRB, 0.16417E+01_JPRB, 0.18004E+01_JPRB, 0.16252E+01_JPRB, 0.15852E+01_JPRB, 0.16032E+01_JPRB, 0.16271E+01_JPRB ,&
&  0.15247E+01_JPRB, 0.14164E+01_JPRB, 0.15108E+01_JPRB, 0.15976E+01_JPRB, 0.16667E+01_JPRB, 0.17193E+01_JPRB, 0.17959E+01_JPRB ,&
&  0.20920E+01_JPRB, 0.22387E+01_JPRB, 0.25248E+01_JPRB, 0.25654E+01_JPRB, 0.29277E+01_JPRB, 0.37666E+01_JPRB, 0.41166E+01_JPRB ,&
&  0.42933E+01_JPRB, 0.34940E+01_JPRB, 0.33183E+01_JPRB, 0.34079E+01_JPRB, 0.33561E+01_JPRB, 0.32657E+01_JPRB, 0.36321E+01_JPRB ,&
&  0.40407E+01_JPRB, 0.47028E+01_JPRB, 0.41568E+01_JPRB, 0.32310E+01_JPRB, 0.23977E+01_JPRB, 0.16903E+01_JPRB, 0.18709E+01_JPRB ,&
&  0.23082E+01_JPRB, 0.23961E+01_JPRB, 0.23989E+01_JPRB, 0.30900E+01_JPRB, 0.24400E+01_JPRB, 0.19429E+01_JPRB, 0.15427E+01_JPRB ,&
&  0.13200E+01_JPRB, 0.12519E+01_JPRB, 0.12372E+01_JPRB, 0.12082E+01_JPRB, 0.11875E+01_JPRB, 0.11681E+01_JPRB, 0.11570E+01_JPRB ,&
&  0.11600E+01_JPRB, 0.11781E+01_JPRB, 0.12138E+01_JPRB, 0.12519E+01_JPRB, 0.12733E+01_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.12151E+01_JPRB, 0.11737E+01_JPRB, 0.11236E+01_JPRB, 0.10767E+01_JPRB, 0.10362E+01_JPRB, 0.10157E+01_JPRB, 0.10361E+01_JPRB ,&
&  0.12880E+01_JPRB, 0.10150E+01_JPRB, 0.91466E+00_JPRB, 0.88489E+00_JPRB, 0.87349E+00_JPRB, 0.85928E+00_JPRB, 0.85818E+00_JPRB ,&
&  0.85931E+00_JPRB, 0.86030E+00_JPRB, 0.87957E+00_JPRB, 0.93604E+00_JPRB, 0.10880E+01_JPRB, 0.14638E+01_JPRB, 0.17332E+01_JPRB ,&
&  0.18804E+01_JPRB, 0.31356E+01_JPRB, 0.23931E+01_JPRB, 0.21720E+01_JPRB, 0.19700E+01_JPRB, 0.18637E+01_JPRB, 0.15593E+01_JPRB ,&
&  0.13398E+01_JPRB, 0.12601E+01_JPRB, 0.12238E+01_JPRB, 0.11823E+01_JPRB, 0.11532E+01_JPRB, 0.10965E+01_JPRB, 0.10653E+01_JPRB ,&
&  0.10621E+01_JPRB, 0.10878E+01_JPRB, 0.11182E+01_JPRB, 0.11535E+01_JPRB, 0.12039E+01_JPRB, 0.13030E+01_JPRB, 0.14253E+01_JPRB ,&
&  0.16383E+01_JPRB, 0.18716E+01_JPRB, 0.17621E+01_JPRB, 0.16379E+01_JPRB, 0.15735E+01_JPRB, 0.16110E+01_JPRB, 0.16328E+01_JPRB ,&
&  0.15531E+01_JPRB, 0.14609E+01_JPRB, 0.15336E+01_JPRB, 0.15625E+01_JPRB, 0.15702E+01_JPRB, 0.16961E+01_JPRB, 0.17474E+01_JPRB ,&
&  0.21176E+01_JPRB, 0.23513E+01_JPRB, 0.32658E+01_JPRB, 0.31974E+01_JPRB, 0.28921E+01_JPRB, 0.35668E+01_JPRB, 0.35850E+01_JPRB ,&
&  0.30924E+01_JPRB, 0.30585E+01_JPRB, 0.32214E+01_JPRB, 0.35665E+01_JPRB, 0.36010E+01_JPRB, 0.35494E+01_JPRB, 0.33574E+01_JPRB ,&
&  0.30061E+01_JPRB, 0.31154E+01_JPRB, 0.26313E+01_JPRB, 0.21820E+01_JPRB, 0.21312E+01_JPRB, 0.18298E+01_JPRB, 0.20695E+01_JPRB ,&
&  0.24791E+01_JPRB, 0.25375E+01_JPRB, 0.27128E+01_JPRB, 0.27710E+01_JPRB, 0.18961E+01_JPRB, 0.14556E+01_JPRB, 0.12440E+01_JPRB ,&
&  0.12056E+01_JPRB, 0.12248E+01_JPRB, 0.12254E+01_JPRB, 0.12023E+01_JPRB, 0.11920E+01_JPRB, 0.11885E+01_JPRB, 0.11924E+01_JPRB ,&
&  0.12100E+01_JPRB, 0.12326E+01_JPRB, 0.12475E+01_JPRB, 0.12505E+01_JPRB, 0.12477E+01_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.11869E+01_JPRB, 0.11628E+01_JPRB, 0.11324E+01_JPRB, 0.11121E+01_JPRB, 0.11211E+01_JPRB, 0.11215E+01_JPRB, 0.12158E+01_JPRB ,&
&  0.14145E+01_JPRB, 0.98838E+00_JPRB, 0.88816E+00_JPRB, 0.86870E+00_JPRB, 0.86578E+00_JPRB, 0.87134E+00_JPRB, 0.89904E+00_JPRB ,&
&  0.92459E+00_JPRB, 0.95232E+00_JPRB, 0.10142E+01_JPRB, 0.11783E+01_JPRB, 0.15131E+01_JPRB, 0.20681E+01_JPRB, 0.24117E+01_JPRB ,&
&  0.25955E+01_JPRB, 0.26152E+01_JPRB, 0.25168E+01_JPRB, 0.20908E+01_JPRB, 0.19434E+01_JPRB, 0.18494E+01_JPRB, 0.17863E+01_JPRB ,&
&  0.14930E+01_JPRB, 0.13537E+01_JPRB, 0.12836E+01_JPRB, 0.12092E+01_JPRB, 0.11328E+01_JPRB, 0.11158E+01_JPRB, 0.11363E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.12071E+01_JPRB, 0.12531E+01_JPRB, 0.13112E+01_JPRB, 0.14137E+01_JPRB, 0.15742E+01_JPRB, 0.17531E+01_JPRB ,&
&  0.19372E+01_JPRB, 0.19968E+01_JPRB, 0.18421E+01_JPRB, 0.16900E+01_JPRB, 0.16138E+01_JPRB, 0.16463E+01_JPRB, 0.16523E+01_JPRB ,&
&  0.16176E+01_JPRB, 0.16141E+01_JPRB, 0.16329E+01_JPRB, 0.16320E+01_JPRB, 0.15712E+01_JPRB, 0.17669E+01_JPRB, 0.18936E+01_JPRB ,&
&  0.22808E+01_JPRB, 0.27462E+01_JPRB, 0.37585E+01_JPRB, 0.30924E+01_JPRB, 0.32253E+01_JPRB, 0.33887E+01_JPRB, 0.33399E+01_JPRB ,&
&  0.28357E+01_JPRB, 0.29941E+01_JPRB, 0.33361E+01_JPRB, 0.35345E+01_JPRB, 0.37997E+01_JPRB, 0.35432E+01_JPRB, 0.35019E+01_JPRB ,&
&  0.26195E+01_JPRB, 0.24832E+01_JPRB, 0.18970E+01_JPRB, 0.19617E+01_JPRB, 0.19750E+01_JPRB, 0.19069E+01_JPRB, 0.20283E+01_JPRB ,&
&  0.23713E+01_JPRB, 0.24040E+01_JPRB, 0.22129E+01_JPRB, 0.18965E+01_JPRB, 0.13393E+01_JPRB, 0.11836E+01_JPRB, 0.11425E+01_JPRB ,&
&  0.11545E+01_JPRB, 0.12218E+01_JPRB, 0.12246E+01_JPRB, 0.12267E+01_JPRB, 0.12169E+01_JPRB, 0.12148E+01_JPRB, 0.12242E+01_JPRB ,&
&  0.12405E+01_JPRB, 0.12472E+01_JPRB, 0.12463E+01_JPRB, 0.12304E+01_JPRB, 0.12128E+01_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.11513E+01_JPRB, 0.11296E+01_JPRB, 0.11077E+01_JPRB, 0.10728E+01_JPRB, 0.10236E+01_JPRB, 0.10088E+01_JPRB, 0.99505E+00_JPRB ,&
&  0.95208E+00_JPRB, 0.90220E+00_JPRB, 0.90463E+00_JPRB, 0.90993E+00_JPRB, 0.92191E+00_JPRB, 0.94526E+00_JPRB, 0.98842E+00_JPRB ,&
&  0.10333E+01_JPRB, 0.11125E+01_JPRB, 0.12909E+01_JPRB, 0.15953E+01_JPRB, 0.19555E+01_JPRB, 0.23842E+01_JPRB, 0.29044E+01_JPRB ,&
&  0.28715E+01_JPRB, 0.19487E+01_JPRB, 0.22473E+01_JPRB, 0.21493E+01_JPRB, 0.19745E+01_JPRB, 0.17912E+01_JPRB, 0.16750E+01_JPRB ,&
&  0.15708E+01_JPRB, 0.14930E+01_JPRB, 0.14369E+01_JPRB, 0.13646E+01_JPRB, 0.12858E+01_JPRB, 0.13293E+01_JPRB, 0.13788E+01_JPRB ,&
&  0.14247E+01_JPRB, 0.14874E+01_JPRB, 0.15652E+01_JPRB, 0.16812E+01_JPRB, 0.18434E+01_JPRB, 0.19983E+01_JPRB, 0.21146E+01_JPRB ,&
&  0.21579E+01_JPRB, 0.21800E+01_JPRB, 0.21163E+01_JPRB, 0.19936E+01_JPRB, 0.18582E+01_JPRB, 0.18319E+01_JPRB, 0.17757E+01_JPRB ,&
&  0.17366E+01_JPRB, 0.17006E+01_JPRB, 0.17599E+01_JPRB, 0.18878E+01_JPRB, 0.19784E+01_JPRB, 0.19201E+01_JPRB, 0.22247E+01_JPRB ,&
&  0.25788E+01_JPRB, 0.30958E+01_JPRB, 0.33732E+01_JPRB, 0.42073E+01_JPRB, 0.28910E+01_JPRB, 0.30281E+01_JPRB, 0.26607E+01_JPRB ,&
&  0.28789E+01_JPRB, 0.29378E+01_JPRB, 0.31878E+01_JPRB, 0.34063E+01_JPRB, 0.37763E+01_JPRB, 0.34119E+01_JPRB, 0.30490E+01_JPRB ,&
&  0.24081E+01_JPRB, 0.19129E+01_JPRB, 0.16137E+01_JPRB, 0.17440E+01_JPRB, 0.18919E+01_JPRB, 0.20940E+01_JPRB, 0.19922E+01_JPRB ,&
&  0.20390E+01_JPRB, 0.18830E+01_JPRB, 0.16426E+01_JPRB, 0.15558E+01_JPRB, 0.12706E+01_JPRB, 0.11606E+01_JPRB, 0.11665E+01_JPRB ,&
&  0.11925E+01_JPRB, 0.12399E+01_JPRB, 0.12344E+01_JPRB, 0.12178E+01_JPRB, 0.12044E+01_JPRB, 0.11978E+01_JPRB, 0.12042E+01_JPRB ,&
&  0.12041E+01_JPRB, 0.12044E+01_JPRB, 0.12017E+01_JPRB, 0.11857E+01_JPRB, 0.11664E+01_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.10966E+01_JPRB, 0.10659E+01_JPRB, 0.10092E+01_JPRB, 0.95509E+00_JPRB, 0.93142E+00_JPRB, 0.91755E+00_JPRB, 0.89830E+00_JPRB ,&
&  0.90371E+00_JPRB, 0.92094E+00_JPRB, 0.94319E+00_JPRB, 0.97389E+00_JPRB, 0.10040E+01_JPRB, 0.10339E+01_JPRB, 0.11035E+01_JPRB ,&
&  0.11898E+01_JPRB, 0.13489E+01_JPRB, 0.15939E+01_JPRB, 0.18505E+01_JPRB, 0.20855E+01_JPRB, 0.23451E+01_JPRB, 0.25975E+01_JPRB ,&
&  0.28878E+01_JPRB, 0.29337E+01_JPRB, 0.26357E+01_JPRB, 0.18419E+01_JPRB, 0.16447E+01_JPRB, 0.17907E+01_JPRB, 0.17566E+01_JPRB ,&
&  0.17191E+01_JPRB, 0.16526E+01_JPRB, 0.16034E+01_JPRB, 0.15984E+01_JPRB, 0.16296E+01_JPRB, 0.16982E+01_JPRB, 0.17576E+01_JPRB ,&
&  0.18423E+01_JPRB, 0.19304E+01_JPRB, 0.20293E+01_JPRB, 0.21351E+01_JPRB, 0.22149E+01_JPRB, 0.22908E+01_JPRB, 0.23709E+01_JPRB ,&
&  0.24350E+01_JPRB, 0.25430E+01_JPRB, 0.26254E+01_JPRB, 0.26011E+01_JPRB, 0.24094E+01_JPRB, 0.23108E+01_JPRB, 0.21846E+01_JPRB ,&
&  0.21291E+01_JPRB, 0.20887E+01_JPRB, 0.21174E+01_JPRB, 0.22810E+01_JPRB, 0.24058E+01_JPRB, 0.22431E+01_JPRB, 0.23867E+01_JPRB ,&
&  0.29199E+01_JPRB, 0.34511E+01_JPRB, 0.31338E+01_JPRB, 0.39584E+01_JPRB, 0.23369E+01_JPRB, 0.26580E+01_JPRB, 0.27560E+01_JPRB ,&
&  0.24930E+01_JPRB, 0.26437E+01_JPRB, 0.29102E+01_JPRB, 0.31192E+01_JPRB, 0.31618E+01_JPRB, 0.28683E+01_JPRB, 0.25976E+01_JPRB ,&
&  0.21591E+01_JPRB, 0.16814E+01_JPRB, 0.14549E+01_JPRB, 0.15044E+01_JPRB, 0.17063E+01_JPRB, 0.19177E+01_JPRB, 0.15860E+01_JPRB ,&
&  0.12799E+01_JPRB, 0.15059E+01_JPRB, 0.15281E+01_JPRB, 0.16577E+01_JPRB, 0.14208E+01_JPRB, 0.11555E+01_JPRB, 0.11937E+01_JPRB ,&
&  0.12373E+01_JPRB, 0.12299E+01_JPRB, 0.12264E+01_JPRB, 0.12003E+01_JPRB, 0.11912E+01_JPRB, 0.11803E+01_JPRB, 0.11707E+01_JPRB ,&
&  0.11684E+01_JPRB, 0.11699E+01_JPRB, 0.11661E+01_JPRB, 0.11411E+01_JPRB, 0.11152E+01_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.10098E+01_JPRB, 0.94928E+00_JPRB, 0.89350E+00_JPRB, 0.86206E+00_JPRB, 0.85338E+00_JPRB, 0.85265E+00_JPRB, 0.86236E+00_JPRB ,&
&  0.88693E+00_JPRB, 0.91762E+00_JPRB, 0.95498E+00_JPRB, 0.10048E+01_JPRB, 0.10540E+01_JPRB, 0.11078E+01_JPRB, 0.11901E+01_JPRB ,&
&  0.12910E+01_JPRB, 0.14328E+01_JPRB, 0.16240E+01_JPRB, 0.18341E+01_JPRB, 0.20198E+01_JPRB, 0.22207E+01_JPRB, 0.24220E+01_JPRB ,&
&  0.26364E+01_JPRB, 0.27240E+01_JPRB, 0.26614E+01_JPRB, 0.25202E+01_JPRB, 0.20384E+01_JPRB, 0.21363E+01_JPRB, 0.22695E+01_JPRB ,&
&  0.23272E+01_JPRB, 0.20165E+01_JPRB, 0.19111E+01_JPRB, 0.19043E+01_JPRB, 0.19474E+01_JPRB, 0.20349E+01_JPRB, 0.21081E+01_JPRB ,&
&  0.21848E+01_JPRB, 0.22517E+01_JPRB, 0.23150E+01_JPRB, 0.23667E+01_JPRB, 0.24183E+01_JPRB, 0.25113E+01_JPRB, 0.26729E+01_JPRB ,&
&  0.27524E+01_JPRB, 0.28367E+01_JPRB, 0.29081E+01_JPRB, 0.29487E+01_JPRB, 0.31126E+01_JPRB, 0.32178E+01_JPRB, 0.31209E+01_JPRB ,&
&  0.30144E+01_JPRB, 0.28063E+01_JPRB, 0.28839E+01_JPRB, 0.31278E+01_JPRB, 0.31706E+01_JPRB, 0.29896E+01_JPRB, 0.30236E+01_JPRB ,&
&  0.34677E+01_JPRB, 0.39346E+01_JPRB, 0.27446E+01_JPRB, 0.30384E+01_JPRB, 0.34479E+01_JPRB, 0.27143E+01_JPRB, 0.22030E+01_JPRB ,&
&  0.19177E+01_JPRB, 0.20210E+01_JPRB, 0.23782E+01_JPRB, 0.25293E+01_JPRB, 0.25113E+01_JPRB, 0.28536E+01_JPRB, 0.22676E+01_JPRB ,&
&  0.18874E+01_JPRB, 0.14664E+01_JPRB, 0.12940E+01_JPRB, 0.12873E+01_JPRB, 0.13996E+01_JPRB, 0.14341E+01_JPRB, 0.11128E+01_JPRB ,&
&  0.10436E+01_JPRB, 0.13895E+01_JPRB, 0.15144E+01_JPRB, 0.17594E+01_JPRB, 0.17240E+01_JPRB, 0.11229E+01_JPRB, 0.11528E+01_JPRB ,&
&  0.11773E+01_JPRB, 0.11866E+01_JPRB, 0.11760E+01_JPRB, 0.11608E+01_JPRB, 0.11476E+01_JPRB, 0.11353E+01_JPRB, 0.11295E+01_JPRB ,&
&  0.11260E+01_JPRB, 0.11215E+01_JPRB, 0.11133E+01_JPRB, 0.10888E+01_JPRB, 0.10532E+01_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.87165E+00_JPRB, 0.82405E+00_JPRB, 0.79385E+00_JPRB, 0.79297E+00_JPRB, 0.80917E+00_JPRB, 0.82322E+00_JPRB, 0.82935E+00_JPRB ,&
&  0.84861E+00_JPRB, 0.87646E+00_JPRB, 0.91931E+00_JPRB, 0.95790E+00_JPRB, 0.10056E+01_JPRB, 0.10517E+01_JPRB, 0.11260E+01_JPRB ,&
&  0.12030E+01_JPRB, 0.12911E+01_JPRB, 0.14379E+01_JPRB, 0.16553E+01_JPRB, 0.18335E+01_JPRB, 0.19991E+01_JPRB, 0.22370E+01_JPRB ,&
&  0.24720E+01_JPRB, 0.26211E+01_JPRB, 0.27801E+01_JPRB, 0.28647E+01_JPRB, 0.27726E+01_JPRB, 0.26338E+01_JPRB, 0.26861E+01_JPRB ,&
&  0.22854E+01_JPRB, 0.19271E+01_JPRB, 0.19870E+01_JPRB, 0.20718E+01_JPRB, 0.21812E+01_JPRB, 0.22347E+01_JPRB, 0.22912E+01_JPRB ,&
&  0.23512E+01_JPRB, 0.23933E+01_JPRB, 0.24330E+01_JPRB, 0.24822E+01_JPRB, 0.25662E+01_JPRB, 0.27053E+01_JPRB, 0.28928E+01_JPRB ,&
&  0.29806E+01_JPRB, 0.30918E+01_JPRB, 0.32454E+01_JPRB, 0.30274E+01_JPRB, 0.33490E+01_JPRB, 0.37861E+01_JPRB, 0.40405E+01_JPRB ,&
&  0.40168E+01_JPRB, 0.37733E+01_JPRB, 0.38553E+01_JPRB, 0.39791E+01_JPRB, 0.40217E+01_JPRB, 0.39408E+01_JPRB, 0.38847E+01_JPRB ,&
&  0.41126E+01_JPRB, 0.39216E+01_JPRB, 0.27631E+01_JPRB, 0.24219E+01_JPRB, 0.24299E+01_JPRB, 0.18842E+01_JPRB, 0.17785E+01_JPRB ,&
&  0.15016E+01_JPRB, 0.15600E+01_JPRB, 0.18694E+01_JPRB, 0.21141E+01_JPRB, 0.22694E+01_JPRB, 0.26055E+01_JPRB, 0.21452E+01_JPRB ,&
&  0.15454E+01_JPRB, 0.11998E+01_JPRB, 0.10295E+01_JPRB, 0.98135E+00_JPRB, 0.10333E+01_JPRB, 0.10626E+01_JPRB, 0.10136E+01_JPRB ,&
&  0.10631E+01_JPRB, 0.11003E+01_JPRB, 0.11414E+01_JPRB, 0.11441E+01_JPRB, 0.10561E+01_JPRB, 0.10530E+01_JPRB, 0.10700E+01_JPRB ,&
&  0.10667E+01_JPRB, 0.10792E+01_JPRB, 0.10798E+01_JPRB, 0.10655E+01_JPRB, 0.10606E+01_JPRB, 0.10518E+01_JPRB, 0.10497E+01_JPRB ,&
&  0.10495E+01_JPRB, 0.10416E+01_JPRB, 0.10206E+01_JPRB, 0.99080E+00_JPRB, 0.93655E+00_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.75489E+00_JPRB, 0.72257E+00_JPRB, 0.72183E+00_JPRB, 0.73785E+00_JPRB, 0.74894E+00_JPRB, 0.76034E+00_JPRB, 0.77277E+00_JPRB ,&
&  0.77871E+00_JPRB, 0.78264E+00_JPRB, 0.79156E+00_JPRB, 0.79659E+00_JPRB, 0.83533E+00_JPRB, 0.88036E+00_JPRB, 0.93870E+00_JPRB ,&
&  0.10254E+01_JPRB, 0.11044E+01_JPRB, 0.11770E+01_JPRB, 0.13027E+01_JPRB, 0.13955E+01_JPRB, 0.14878E+01_JPRB, 0.16342E+01_JPRB ,&
&  0.18166E+01_JPRB, 0.20267E+01_JPRB, 0.22244E+01_JPRB, 0.24295E+01_JPRB, 0.27036E+01_JPRB, 0.29803E+01_JPRB, 0.30293E+01_JPRB ,&
&  0.23684E+01_JPRB, 0.18530E+01_JPRB, 0.20821E+01_JPRB, 0.20052E+01_JPRB, 0.21326E+01_JPRB, 0.22336E+01_JPRB, 0.22848E+01_JPRB ,&
&  0.23521E+01_JPRB, 0.24019E+01_JPRB, 0.24486E+01_JPRB, 0.25020E+01_JPRB, 0.26086E+01_JPRB, 0.27406E+01_JPRB, 0.29125E+01_JPRB ,&
&  0.30382E+01_JPRB, 0.31430E+01_JPRB, 0.31906E+01_JPRB, 0.32193E+01_JPRB, 0.35315E+01_JPRB, 0.39767E+01_JPRB, 0.43075E+01_JPRB ,&
&  0.45064E+01_JPRB, 0.46970E+01_JPRB, 0.43019E+01_JPRB, 0.43314E+01_JPRB, 0.42582E+01_JPRB, 0.38255E+01_JPRB, 0.38746E+01_JPRB ,&
&  0.41329E+01_JPRB, 0.38229E+01_JPRB, 0.23975E+01_JPRB, 0.21613E+01_JPRB, 0.20114E+01_JPRB, 0.17450E+01_JPRB, 0.15522E+01_JPRB ,&
&  0.12494E+01_JPRB, 0.12458E+01_JPRB, 0.13968E+01_JPRB, 0.17456E+01_JPRB, 0.19603E+01_JPRB, 0.20761E+01_JPRB, 0.18227E+01_JPRB ,&
&  0.13143E+01_JPRB, 0.10456E+01_JPRB, 0.90540E+00_JPRB, 0.91695E+00_JPRB, 0.10042E+01_JPRB, 0.11391E+01_JPRB, 0.99433E+00_JPRB ,&
&  0.10489E+01_JPRB, 0.11914E+01_JPRB, 0.14647E+01_JPRB, 0.12370E+01_JPRB, 0.94800E+00_JPRB, 0.96373E+00_JPRB, 0.95734E+00_JPRB ,&
&  0.95494E+00_JPRB, 0.96090E+00_JPRB, 0.95841E+00_JPRB, 0.95363E+00_JPRB, 0.95088E+00_JPRB, 0.95901E+00_JPRB, 0.95161E+00_JPRB ,&
&  0.93945E+00_JPRB, 0.91848E+00_JPRB, 0.90122E+00_JPRB, 0.84693E+00_JPRB, 0.79747E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.66815E+00_JPRB, 0.66819E+00_JPRB, 0.67512E+00_JPRB, 0.69190E+00_JPRB, 0.69030E+00_JPRB, 0.68916E+00_JPRB, 0.71760E+00_JPRB ,&
&  0.70265E+00_JPRB, 0.69111E+00_JPRB, 0.69201E+00_JPRB, 0.66844E+00_JPRB, 0.69165E+00_JPRB, 0.71510E+00_JPRB, 0.75971E+00_JPRB ,&
&  0.80600E+00_JPRB, 0.83627E+00_JPRB, 0.86283E+00_JPRB, 0.90116E+00_JPRB, 0.97248E+00_JPRB, 0.10279E+01_JPRB, 0.10763E+01_JPRB ,&
&  0.11604E+01_JPRB, 0.12885E+01_JPRB, 0.14285E+01_JPRB, 0.17452E+01_JPRB, 0.24755E+01_JPRB, 0.30696E+01_JPRB, 0.30757E+01_JPRB ,&
&  0.23602E+01_JPRB, 0.18326E+01_JPRB, 0.17664E+01_JPRB, 0.16352E+01_JPRB, 0.17478E+01_JPRB, 0.19657E+01_JPRB, 0.20598E+01_JPRB ,&
&  0.20711E+01_JPRB, 0.21164E+01_JPRB, 0.21428E+01_JPRB, 0.21768E+01_JPRB, 0.22208E+01_JPRB, 0.22524E+01_JPRB, 0.22326E+01_JPRB ,&
&  0.21560E+01_JPRB, 0.20608E+01_JPRB, 0.21502E+01_JPRB, 0.26177E+01_JPRB, 0.31148E+01_JPRB, 0.38016E+01_JPRB, 0.37432E+01_JPRB ,&
&  0.35918E+01_JPRB, 0.35682E+01_JPRB, 0.35791E+01_JPRB, 0.35162E+01_JPRB, 0.33745E+01_JPRB, 0.32152E+01_JPRB, 0.32762E+01_JPRB ,&
&  0.34832E+01_JPRB, 0.32396E+01_JPRB, 0.21224E+01_JPRB, 0.20507E+01_JPRB, 0.18491E+01_JPRB, 0.17471E+01_JPRB, 0.13964E+01_JPRB ,&
&  0.11098E+01_JPRB, 0.10499E+01_JPRB, 0.10948E+01_JPRB, 0.12327E+01_JPRB, 0.14955E+01_JPRB, 0.16128E+01_JPRB, 0.14394E+01_JPRB ,&
&  0.11647E+01_JPRB, 0.96822E+00_JPRB, 0.85652E+00_JPRB, 0.83977E+00_JPRB, 0.10096E+01_JPRB, 0.11231E+01_JPRB, 0.12687E+01_JPRB ,&
&  0.94003E+00_JPRB, 0.10492E+01_JPRB, 0.95391E+00_JPRB, 0.12096E+01_JPRB, 0.85197E+00_JPRB, 0.79417E+00_JPRB, 0.78426E+00_JPRB ,&
&  0.78631E+00_JPRB, 0.77285E+00_JPRB, 0.75918E+00_JPRB, 0.73880E+00_JPRB, 0.71896E+00_JPRB, 0.75845E+00_JPRB, 0.80157E+00_JPRB ,&
&  0.76358E+00_JPRB, 0.78370E+00_JPRB, 0.75666E+00_JPRB, 0.71164E+00_JPRB, 0.69586E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.64550E+00_JPRB, 0.65773E+00_JPRB, 0.66027E+00_JPRB, 0.66872E+00_JPRB, 0.68544E+00_JPRB, 0.69818E+00_JPRB, 0.71568E+00_JPRB ,&
&  0.73512E+00_JPRB, 0.74592E+00_JPRB, 0.75728E+00_JPRB, 0.77158E+00_JPRB, 0.78255E+00_JPRB, 0.80844E+00_JPRB, 0.83392E+00_JPRB ,&
&  0.86623E+00_JPRB, 0.89191E+00_JPRB, 0.91858E+00_JPRB, 0.95893E+00_JPRB, 0.10097E+01_JPRB, 0.10882E+01_JPRB, 0.11725E+01_JPRB ,&
&  0.12834E+01_JPRB, 0.14137E+01_JPRB, 0.15685E+01_JPRB, 0.16965E+01_JPRB, 0.17668E+01_JPRB, 0.18143E+01_JPRB, 0.20249E+01_JPRB ,&
&  0.18041E+01_JPRB, 0.19199E+01_JPRB, 0.18437E+01_JPRB, 0.16614E+01_JPRB, 0.16559E+01_JPRB, 0.15373E+01_JPRB, 0.15176E+01_JPRB ,&
&  0.15466E+01_JPRB, 0.14999E+01_JPRB, 0.14763E+01_JPRB, 0.14550E+01_JPRB, 0.14262E+01_JPRB, 0.13532E+01_JPRB, 0.12633E+01_JPRB ,&
&  0.12534E+01_JPRB, 0.13583E+01_JPRB, 0.15741E+01_JPRB, 0.20096E+01_JPRB, 0.24476E+01_JPRB, 0.26270E+01_JPRB, 0.24439E+01_JPRB ,&
&  0.23502E+01_JPRB, 0.23623E+01_JPRB, 0.22861E+01_JPRB, 0.21802E+01_JPRB, 0.22785E+01_JPRB, 0.23600E+01_JPRB, 0.24522E+01_JPRB ,&
&  0.23755E+01_JPRB, 0.21082E+01_JPRB, 0.19440E+01_JPRB, 0.17374E+01_JPRB, 0.16339E+01_JPRB, 0.15298E+01_JPRB, 0.13253E+01_JPRB ,&
&  0.11037E+01_JPRB, 0.97649E+00_JPRB, 0.92606E+00_JPRB, 0.96552E+00_JPRB, 0.11269E+01_JPRB, 0.11541E+01_JPRB, 0.10545E+01_JPRB ,&
&  0.89342E+00_JPRB, 0.80718E+00_JPRB, 0.77164E+00_JPRB, 0.78018E+00_JPRB, 0.88496E+00_JPRB, 0.17550E+01_JPRB, 0.23837E+01_JPRB ,&
&  0.94903E+00_JPRB, 0.79021E+00_JPRB, 0.57720E+00_JPRB, 0.80497E+00_JPRB, 0.67564E+00_JPRB, 0.66375E+00_JPRB, 0.66359E+00_JPRB ,&
&  0.65547E+00_JPRB, 0.65440E+00_JPRB, 0.65277E+00_JPRB, 0.63799E+00_JPRB, 0.62816E+00_JPRB, 0.63024E+00_JPRB, 0.66192E+00_JPRB ,&
&  0.65950E+00_JPRB, 0.66191E+00_JPRB, 0.65140E+00_JPRB, 0.65764E+00_JPRB, 0.65065E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.64754E+00_JPRB, 0.66361E+00_JPRB, 0.68332E+00_JPRB, 0.70775E+00_JPRB, 0.72525E+00_JPRB, 0.74828E+00_JPRB, 0.76786E+00_JPRB ,&
&  0.79389E+00_JPRB, 0.81583E+00_JPRB, 0.83532E+00_JPRB, 0.85582E+00_JPRB, 0.88062E+00_JPRB, 0.90950E+00_JPRB, 0.94571E+00_JPRB ,&
&  0.98487E+00_JPRB, 0.10243E+01_JPRB, 0.10675E+01_JPRB, 0.11171E+01_JPRB, 0.11753E+01_JPRB, 0.12467E+01_JPRB, 0.13153E+01_JPRB ,&
&  0.13802E+01_JPRB, 0.14468E+01_JPRB, 0.15162E+01_JPRB, 0.15706E+01_JPRB, 0.15932E+01_JPRB, 0.16320E+01_JPRB, 0.14817E+01_JPRB ,&
&  0.18393E+01_JPRB, 0.18015E+01_JPRB, 0.16956E+01_JPRB, 0.15507E+01_JPRB, 0.14107E+01_JPRB, 0.12831E+01_JPRB, 0.11820E+01_JPRB ,&
&  0.12073E+01_JPRB, 0.11666E+01_JPRB, 0.11551E+01_JPRB, 0.11604E+01_JPRB, 0.10850E+01_JPRB, 0.10866E+01_JPRB, 0.11671E+01_JPRB ,&
&  0.13033E+01_JPRB, 0.14945E+01_JPRB, 0.16181E+01_JPRB, 0.17646E+01_JPRB, 0.19466E+01_JPRB, 0.18826E+01_JPRB, 0.17794E+01_JPRB ,&
&  0.16567E+01_JPRB, 0.15826E+01_JPRB, 0.12800E+01_JPRB, 0.14562E+01_JPRB, 0.15161E+01_JPRB, 0.15870E+01_JPRB, 0.17047E+01_JPRB ,&
&  0.15969E+01_JPRB, 0.12996E+01_JPRB, 0.13348E+01_JPRB, 0.15493E+01_JPRB, 0.14649E+01_JPRB, 0.13375E+01_JPRB, 0.12257E+01_JPRB ,&
&  0.10995E+01_JPRB, 0.10017E+01_JPRB, 0.91765E+00_JPRB, 0.88825E+00_JPRB, 0.94136E+00_JPRB, 0.93267E+00_JPRB, 0.84627E+00_JPRB ,&
&  0.74861E+00_JPRB, 0.71899E+00_JPRB, 0.71882E+00_JPRB, 0.73903E+00_JPRB, 0.87346E+00_JPRB, 0.10317E+01_JPRB, 0.16078E+01_JPRB ,&
&  0.96706E+00_JPRB, 0.56311E+00_JPRB, 0.62586E+00_JPRB, 0.67624E+00_JPRB, 0.62368E+00_JPRB, 0.62245E+00_JPRB, 0.65214E+00_JPRB ,&
&  0.68271E+00_JPRB, 0.69444E+00_JPRB, 0.65220E+00_JPRB, 0.65101E+00_JPRB, 0.64467E+00_JPRB, 0.63849E+00_JPRB, 0.63378E+00_JPRB ,&
&  0.62690E+00_JPRB, 0.62878E+00_JPRB, 0.64095E+00_JPRB, 0.63922E+00_JPRB, 0.64015E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.57114E+00_JPRB, 0.58191E+00_JPRB, 0.59607E+00_JPRB, 0.60852E+00_JPRB, 0.61659E+00_JPRB, 0.63025E+00_JPRB, 0.64076E+00_JPRB ,&
&  0.65313E+00_JPRB, 0.65945E+00_JPRB, 0.66972E+00_JPRB, 0.67896E+00_JPRB, 0.69012E+00_JPRB, 0.70612E+00_JPRB, 0.72240E+00_JPRB ,&
&  0.73522E+00_JPRB, 0.75542E+00_JPRB, 0.77854E+00_JPRB, 0.79536E+00_JPRB, 0.83648E+00_JPRB, 0.91308E+00_JPRB, 0.10010E+01_JPRB ,&
&  0.10747E+01_JPRB, 0.11709E+01_JPRB, 0.12627E+01_JPRB, 0.13094E+01_JPRB, 0.13183E+01_JPRB, 0.13650E+01_JPRB, 0.12042E+01_JPRB ,&
&  0.16568E+01_JPRB, 0.16508E+01_JPRB, 0.15087E+01_JPRB, 0.14246E+01_JPRB, 0.13384E+01_JPRB, 0.12535E+01_JPRB, 0.12054E+01_JPRB ,&
&  0.12340E+01_JPRB, 0.14756E+01_JPRB, 0.15098E+01_JPRB, 0.15724E+01_JPRB, 0.14593E+01_JPRB, 0.14598E+01_JPRB, 0.16354E+01_JPRB ,&
&  0.18407E+01_JPRB, 0.19197E+01_JPRB, 0.19456E+01_JPRB, 0.20511E+01_JPRB, 0.21721E+01_JPRB, 0.22355E+01_JPRB, 0.22597E+01_JPRB ,&
&  0.21764E+01_JPRB, 0.20974E+01_JPRB, 0.19657E+01_JPRB, 0.23278E+01_JPRB, 0.20567E+01_JPRB, 0.17871E+01_JPRB, 0.16237E+01_JPRB ,&
&  0.12444E+01_JPRB, 0.11442E+01_JPRB, 0.12078E+01_JPRB, 0.12565E+01_JPRB, 0.12072E+01_JPRB, 0.11916E+01_JPRB, 0.11412E+01_JPRB ,&
&  0.10707E+01_JPRB, 0.10228E+01_JPRB, 0.96085E+00_JPRB, 0.93943E+00_JPRB, 0.94187E+00_JPRB, 0.89179E+00_JPRB, 0.78962E+00_JPRB ,&
&  0.71812E+00_JPRB, 0.71214E+00_JPRB, 0.70061E+00_JPRB, 0.70675E+00_JPRB, 0.77159E+00_JPRB, 0.77433E+00_JPRB, 0.11188E+01_JPRB ,&
&  0.84921E+00_JPRB, 0.72933E+00_JPRB, 0.94983E+00_JPRB, 0.46554E+00_JPRB, 0.56535E+00_JPRB, 0.67428E+00_JPRB, 0.86495E+00_JPRB ,&
&  0.91302E+00_JPRB, 0.11744E+01_JPRB, 0.15155E+01_JPRB, 0.12027E+01_JPRB, 0.92011E+00_JPRB, 0.63007E+00_JPRB, 0.59706E+00_JPRB ,&
&  0.58829E+00_JPRB, 0.57904E+00_JPRB, 0.58169E+00_JPRB, 0.57437E+00_JPRB, 0.57223E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.49482E+00_JPRB, 0.49860E+00_JPRB, 0.50851E+00_JPRB, 0.51529E+00_JPRB, 0.52201E+00_JPRB, 0.52732E+00_JPRB, 0.53230E+00_JPRB ,&
&  0.54732E+00_JPRB, 0.55656E+00_JPRB, 0.57990E+00_JPRB, 0.59923E+00_JPRB, 0.61659E+00_JPRB, 0.65236E+00_JPRB, 0.68258E+00_JPRB ,&
&  0.72985E+00_JPRB, 0.79134E+00_JPRB, 0.81875E+00_JPRB, 0.85673E+00_JPRB, 0.88068E+00_JPRB, 0.90845E+00_JPRB, 0.93091E+00_JPRB ,&
&  0.10174E+01_JPRB, 0.11862E+01_JPRB, 0.13076E+01_JPRB, 0.14349E+01_JPRB, 0.14503E+01_JPRB, 0.15366E+01_JPRB, 0.12413E+01_JPRB ,&
&  0.16426E+01_JPRB, 0.16861E+01_JPRB, 0.16057E+01_JPRB, 0.15814E+01_JPRB, 0.15778E+01_JPRB, 0.15868E+01_JPRB, 0.15910E+01_JPRB ,&
&  0.16827E+01_JPRB, 0.18275E+01_JPRB, 0.18412E+01_JPRB, 0.19645E+01_JPRB, 0.20760E+01_JPRB, 0.21190E+01_JPRB, 0.23729E+01_JPRB ,&
&  0.26042E+01_JPRB, 0.27913E+01_JPRB, 0.30254E+01_JPRB, 0.33118E+01_JPRB, 0.35145E+01_JPRB, 0.36419E+01_JPRB, 0.36953E+01_JPRB ,&
&  0.37167E+01_JPRB, 0.38375E+01_JPRB, 0.41680E+01_JPRB, 0.41462E+01_JPRB, 0.32642E+01_JPRB, 0.26407E+01_JPRB, 0.19961E+01_JPRB ,&
&  0.13108E+01_JPRB, 0.10419E+01_JPRB, 0.10196E+01_JPRB, 0.10498E+01_JPRB, 0.10180E+01_JPRB, 0.10036E+01_JPRB, 0.98259E+00_JPRB ,&
&  0.96796E+00_JPRB, 0.98292E+00_JPRB, 0.98331E+00_JPRB, 0.97660E+00_JPRB, 0.94092E+00_JPRB, 0.84734E+00_JPRB, 0.74118E+00_JPRB ,&
&  0.70842E+00_JPRB, 0.69254E+00_JPRB, 0.69495E+00_JPRB, 0.72335E+00_JPRB, 0.78145E+00_JPRB, 0.90663E+00_JPRB, 0.89401E+00_JPRB ,&
&  0.95722E+00_JPRB, 0.67893E+00_JPRB, 0.65504E+00_JPRB, 0.68607E+00_JPRB, 0.63801E+00_JPRB, 0.87766E+00_JPRB, 0.12796E+01_JPRB ,&
&  0.18836E+01_JPRB, 0.21357E+01_JPRB, 0.28112E+01_JPRB, 0.41149E+01_JPRB, 0.25951E+01_JPRB, 0.16680E+01_JPRB, 0.56303E+00_JPRB ,&
&  0.55317E+00_JPRB, 0.52915E+00_JPRB, 0.52204E+00_JPRB, 0.50652E+00_JPRB, 0.50067E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.40699E+00_JPRB, 0.40104E+00_JPRB, 0.41222E+00_JPRB, 0.43022E+00_JPRB, 0.44404E+00_JPRB, 0.47483E+00_JPRB, 0.51205E+00_JPRB ,&
&  0.55300E+00_JPRB, 0.60440E+00_JPRB, 0.64921E+00_JPRB, 0.68297E+00_JPRB, 0.73875E+00_JPRB, 0.81523E+00_JPRB, 0.85217E+00_JPRB ,&
&  0.92116E+00_JPRB, 0.96795E+00_JPRB, 0.99537E+00_JPRB, 0.10154E+01_JPRB, 0.10537E+01_JPRB, 0.11138E+01_JPRB, 0.11893E+01_JPRB ,&
&  0.12632E+01_JPRB, 0.13500E+01_JPRB, 0.14653E+01_JPRB, 0.15448E+01_JPRB, 0.15892E+01_JPRB, 0.16141E+01_JPRB, 0.12896E+01_JPRB ,&
&  0.16701E+01_JPRB, 0.17968E+01_JPRB, 0.18681E+01_JPRB, 0.18699E+01_JPRB, 0.18627E+01_JPRB, 0.18730E+01_JPRB, 0.19436E+01_JPRB ,&
&  0.20625E+01_JPRB, 0.21163E+01_JPRB, 0.21428E+01_JPRB, 0.21812E+01_JPRB, 0.28078E+01_JPRB, 0.29401E+01_JPRB, 0.32246E+01_JPRB ,&
&  0.34943E+01_JPRB, 0.37342E+01_JPRB, 0.40505E+01_JPRB, 0.43861E+01_JPRB, 0.47344E+01_JPRB, 0.50320E+01_JPRB, 0.53019E+01_JPRB ,&
&  0.55528E+01_JPRB, 0.58227E+01_JPRB, 0.60784E+01_JPRB, 0.52184E+01_JPRB, 0.41085E+01_JPRB, 0.33240E+01_JPRB, 0.20116E+01_JPRB ,&
&  0.12259E+01_JPRB, 0.10386E+01_JPRB, 0.10368E+01_JPRB, 0.99742E+00_JPRB, 0.89997E+00_JPRB, 0.86004E+00_JPRB, 0.82964E+00_JPRB ,&
&  0.84622E+00_JPRB, 0.89015E+00_JPRB, 0.92129E+00_JPRB, 0.92375E+00_JPRB, 0.87573E+00_JPRB, 0.78783E+00_JPRB, 0.70498E+00_JPRB ,&
&  0.71284E+00_JPRB, 0.71046E+00_JPRB, 0.71377E+00_JPRB, 0.77237E+00_JPRB, 0.84764E+00_JPRB, 0.10408E+01_JPRB, 0.12832E+01_JPRB ,&
&  0.12073E+01_JPRB, 0.70663E+00_JPRB, 0.65989E+00_JPRB, 0.65224E+00_JPRB, 0.65908E+00_JPRB, 0.72925E+00_JPRB, 0.88666E+00_JPRB ,&
&  0.12447E+01_JPRB, 0.20244E+01_JPRB, 0.22762E+01_JPRB, 0.15032E+01_JPRB, 0.20178E+01_JPRB, 0.13217E+01_JPRB, 0.54632E+00_JPRB ,&
&  0.51940E+00_JPRB, 0.50304E+00_JPRB, 0.47542E+00_JPRB, 0.44449E+00_JPRB, 0.42263E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.41084E+00_JPRB, 0.39287E+00_JPRB, 0.40403E+00_JPRB, 0.42897E+00_JPRB, 0.45848E+00_JPRB, 0.50372E+00_JPRB, 0.55965E+00_JPRB ,&
&  0.61000E+00_JPRB, 0.66035E+00_JPRB, 0.74605E+00_JPRB, 0.77025E+00_JPRB, 0.79403E+00_JPRB, 0.84442E+00_JPRB, 0.88275E+00_JPRB ,&
&  0.92859E+00_JPRB, 0.96939E+00_JPRB, 0.10133E+01_JPRB, 0.10556E+01_JPRB, 0.11148E+01_JPRB, 0.11818E+01_JPRB, 0.12389E+01_JPRB ,&
&  0.13130E+01_JPRB, 0.13930E+01_JPRB, 0.15067E+01_JPRB, 0.16279E+01_JPRB, 0.17435E+01_JPRB, 0.17788E+01_JPRB, 0.18454E+01_JPRB ,&
&  0.12950E+01_JPRB, 0.18053E+01_JPRB, 0.18824E+01_JPRB, 0.19289E+01_JPRB, 0.19307E+01_JPRB, 0.19299E+01_JPRB, 0.20127E+01_JPRB ,&
&  0.21362E+01_JPRB, 0.21294E+01_JPRB, 0.22842E+01_JPRB, 0.25426E+01_JPRB, 0.32310E+01_JPRB, 0.34092E+01_JPRB, 0.36337E+01_JPRB ,&
&  0.38751E+01_JPRB, 0.41092E+01_JPRB, 0.43835E+01_JPRB, 0.47147E+01_JPRB, 0.50503E+01_JPRB, 0.53671E+01_JPRB, 0.56912E+01_JPRB ,&
&  0.59884E+01_JPRB, 0.62627E+01_JPRB, 0.65499E+01_JPRB, 0.54202E+01_JPRB, 0.44651E+01_JPRB, 0.35754E+01_JPRB, 0.22712E+01_JPRB ,&
&  0.13762E+01_JPRB, 0.10200E+01_JPRB, 0.10226E+01_JPRB, 0.10362E+01_JPRB, 0.90229E+00_JPRB, 0.83157E+00_JPRB, 0.74495E+00_JPRB ,&
&  0.73919E+00_JPRB, 0.76412E+00_JPRB, 0.78779E+00_JPRB, 0.79236E+00_JPRB, 0.79331E+00_JPRB, 0.72566E+00_JPRB, 0.67907E+00_JPRB ,&
&  0.69346E+00_JPRB, 0.68898E+00_JPRB, 0.69767E+00_JPRB, 0.72807E+00_JPRB, 0.78107E+00_JPRB, 0.82922E+00_JPRB, 0.83094E+00_JPRB ,&
&  0.83025E+00_JPRB, 0.74152E+00_JPRB, 0.70688E+00_JPRB, 0.72022E+00_JPRB, 0.70626E+00_JPRB, 0.69095E+00_JPRB, 0.70884E+00_JPRB ,&
&  0.79179E+00_JPRB, 0.89727E+00_JPRB, 0.98613E+00_JPRB, 0.97068E+00_JPRB, 0.10067E+01_JPRB, 0.79803E+00_JPRB, 0.60168E+00_JPRB ,&
&  0.56501E+00_JPRB, 0.55467E+00_JPRB, 0.50990E+00_JPRB, 0.47814E+00_JPRB, 0.44207E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.50964E+00_JPRB, 0.44972E+00_JPRB, 0.44517E+00_JPRB, 0.48842E+00_JPRB, 0.53238E+00_JPRB, 0.57804E+00_JPRB, 0.62095E+00_JPRB ,&
&  0.66689E+00_JPRB, 0.69334E+00_JPRB, 0.74448E+00_JPRB, 0.77190E+00_JPRB, 0.79476E+00_JPRB, 0.80798E+00_JPRB, 0.82561E+00_JPRB ,&
&  0.86540E+00_JPRB, 0.91255E+00_JPRB, 0.95556E+00_JPRB, 0.10170E+01_JPRB, 0.10812E+01_JPRB, 0.11408E+01_JPRB, 0.12093E+01_JPRB ,&
&  0.12769E+01_JPRB, 0.13542E+01_JPRB, 0.14352E+01_JPRB, 0.15333E+01_JPRB, 0.16159E+01_JPRB, 0.16522E+01_JPRB, 0.16032E+01_JPRB ,&
&  0.84861E+00_JPRB, 0.13796E+01_JPRB, 0.18035E+01_JPRB, 0.18315E+01_JPRB, 0.18564E+01_JPRB, 0.18547E+01_JPRB, 0.19496E+01_JPRB ,&
&  0.19464E+01_JPRB, 0.18871E+01_JPRB, 0.20839E+01_JPRB, 0.29055E+01_JPRB, 0.33228E+01_JPRB, 0.35372E+01_JPRB, 0.36717E+01_JPRB ,&
&  0.38405E+01_JPRB, 0.39731E+01_JPRB, 0.41359E+01_JPRB, 0.43127E+01_JPRB, 0.45122E+01_JPRB, 0.47050E+01_JPRB, 0.48857E+01_JPRB ,&
&  0.50454E+01_JPRB, 0.51428E+01_JPRB, 0.51263E+01_JPRB, 0.40411E+01_JPRB, 0.39631E+01_JPRB, 0.33235E+01_JPRB, 0.27983E+01_JPRB ,&
&  0.16509E+01_JPRB, 0.10907E+01_JPRB, 0.10514E+01_JPRB, 0.10573E+01_JPRB, 0.91863E+00_JPRB, 0.83796E+00_JPRB, 0.73463E+00_JPRB ,&
&  0.72383E+00_JPRB, 0.72331E+00_JPRB, 0.73780E+00_JPRB, 0.73602E+00_JPRB, 0.73719E+00_JPRB, 0.67611E+00_JPRB, 0.68286E+00_JPRB ,&
&  0.70017E+00_JPRB, 0.69728E+00_JPRB, 0.70956E+00_JPRB, 0.74213E+00_JPRB, 0.78568E+00_JPRB, 0.82708E+00_JPRB, 0.84100E+00_JPRB ,&
&  0.83490E+00_JPRB, 0.81570E+00_JPRB, 0.81107E+00_JPRB, 0.80705E+00_JPRB, 0.80116E+00_JPRB, 0.79330E+00_JPRB, 0.77737E+00_JPRB ,&
&  0.75695E+00_JPRB, 0.76471E+00_JPRB, 0.79364E+00_JPRB, 0.73270E+00_JPRB, 0.77291E+00_JPRB, 0.70226E+00_JPRB, 0.68680E+00_JPRB ,&
&  0.66986E+00_JPRB, 0.65299E+00_JPRB, 0.60707E+00_JPRB, 0.58359E+00_JPRB, 0.55068E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.61852E+00_JPRB, 0.54661E+00_JPRB, 0.48893E+00_JPRB, 0.50406E+00_JPRB, 0.56941E+00_JPRB, 0.62477E+00_JPRB, 0.65483E+00_JPRB ,&
&  0.69078E+00_JPRB, 0.70187E+00_JPRB, 0.71049E+00_JPRB, 0.72246E+00_JPRB, 0.75053E+00_JPRB, 0.76484E+00_JPRB, 0.79736E+00_JPRB ,&
&  0.82072E+00_JPRB, 0.84192E+00_JPRB, 0.86699E+00_JPRB, 0.91848E+00_JPRB, 0.98022E+00_JPRB, 0.10420E+01_JPRB, 0.10995E+01_JPRB ,&
&  0.11627E+01_JPRB, 0.12364E+01_JPRB, 0.13400E+01_JPRB, 0.14672E+01_JPRB, 0.16403E+01_JPRB, 0.18316E+01_JPRB, 0.19779E+01_JPRB ,&
&  0.14093E+01_JPRB, 0.71842E+00_JPRB, 0.13977E+01_JPRB, 0.17464E+01_JPRB, 0.17197E+01_JPRB, 0.17215E+01_JPRB, 0.17116E+01_JPRB ,&
&  0.17047E+01_JPRB, 0.17521E+01_JPRB, 0.18486E+01_JPRB, 0.24719E+01_JPRB, 0.31339E+01_JPRB, 0.33744E+01_JPRB, 0.34606E+01_JPRB ,&
&  0.35475E+01_JPRB, 0.35818E+01_JPRB, 0.36382E+01_JPRB, 0.37253E+01_JPRB, 0.38356E+01_JPRB, 0.39048E+01_JPRB, 0.39588E+01_JPRB ,&
&  0.40031E+01_JPRB, 0.39934E+01_JPRB, 0.38415E+01_JPRB, 0.27580E+01_JPRB, 0.24569E+01_JPRB, 0.21087E+01_JPRB, 0.19080E+01_JPRB ,&
&  0.16507E+01_JPRB, 0.12416E+01_JPRB, 0.11209E+01_JPRB, 0.10799E+01_JPRB, 0.96065E+00_JPRB, 0.82329E+00_JPRB, 0.77198E+00_JPRB ,&
&  0.75942E+00_JPRB, 0.73527E+00_JPRB, 0.71484E+00_JPRB, 0.69549E+00_JPRB, 0.72520E+00_JPRB, 0.72100E+00_JPRB, 0.71174E+00_JPRB ,&
&  0.70543E+00_JPRB, 0.74119E+00_JPRB, 0.77917E+00_JPRB, 0.81152E+00_JPRB, 0.84479E+00_JPRB, 0.88335E+00_JPRB, 0.88867E+00_JPRB ,&
&  0.90146E+00_JPRB, 0.89319E+00_JPRB, 0.88876E+00_JPRB, 0.86834E+00_JPRB, 0.85150E+00_JPRB, 0.82100E+00_JPRB, 0.81849E+00_JPRB ,&
&  0.82034E+00_JPRB, 0.82582E+00_JPRB, 0.81500E+00_JPRB, 0.79802E+00_JPRB, 0.81645E+00_JPRB, 0.80483E+00_JPRB, 0.80916E+00_JPRB ,&
&  0.80382E+00_JPRB, 0.78280E+00_JPRB, 0.72659E+00_JPRB, 0.68033E+00_JPRB, 0.65977E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.67942E+00_JPRB, 0.63439E+00_JPRB, 0.57066E+00_JPRB, 0.52732E+00_JPRB, 0.54709E+00_JPRB, 0.61396E+00_JPRB, 0.66816E+00_JPRB ,&
&  0.68915E+00_JPRB, 0.68973E+00_JPRB, 0.68179E+00_JPRB, 0.68234E+00_JPRB, 0.69910E+00_JPRB, 0.70002E+00_JPRB, 0.71755E+00_JPRB ,&
&  0.74392E+00_JPRB, 0.76405E+00_JPRB, 0.78469E+00_JPRB, 0.81855E+00_JPRB, 0.85351E+00_JPRB, 0.91027E+00_JPRB, 0.96239E+00_JPRB ,&
&  0.10037E+01_JPRB, 0.10452E+01_JPRB, 0.11067E+01_JPRB, 0.11916E+01_JPRB, 0.13181E+01_JPRB, 0.15517E+01_JPRB, 0.19598E+01_JPRB ,&
&  0.27150E+01_JPRB, 0.18507E+01_JPRB, 0.91126E+00_JPRB, 0.15858E+01_JPRB, 0.17348E+01_JPRB, 0.16975E+01_JPRB, 0.15263E+01_JPRB ,&
&  0.15445E+01_JPRB, 0.15281E+01_JPRB, 0.15839E+01_JPRB, 0.18775E+01_JPRB, 0.22664E+01_JPRB, 0.27305E+01_JPRB, 0.30072E+01_JPRB ,&
&  0.30781E+01_JPRB, 0.31101E+01_JPRB, 0.31355E+01_JPRB, 0.31989E+01_JPRB, 0.32992E+01_JPRB, 0.33916E+01_JPRB, 0.34113E+01_JPRB ,&
&  0.34450E+01_JPRB, 0.35173E+01_JPRB, 0.34300E+01_JPRB, 0.25191E+01_JPRB, 0.23013E+01_JPRB, 0.22071E+01_JPRB, 0.20871E+01_JPRB ,&
&  0.18093E+01_JPRB, 0.15893E+01_JPRB, 0.14098E+01_JPRB, 0.11795E+01_JPRB, 0.10138E+01_JPRB, 0.91803E+00_JPRB, 0.90148E+00_JPRB ,&
&  0.86503E+00_JPRB, 0.79889E+00_JPRB, 0.77187E+00_JPRB, 0.74607E+00_JPRB, 0.76264E+00_JPRB, 0.79194E+00_JPRB, 0.79306E+00_JPRB ,&
&  0.81189E+00_JPRB, 0.84382E+00_JPRB, 0.86773E+00_JPRB, 0.89045E+00_JPRB, 0.91931E+00_JPRB, 0.94310E+00_JPRB, 0.95573E+00_JPRB ,&
&  0.97068E+00_JPRB, 0.96564E+00_JPRB, 0.95459E+00_JPRB, 0.93661E+00_JPRB, 0.89501E+00_JPRB, 0.85780E+00_JPRB, 0.86728E+00_JPRB ,&
&  0.86193E+00_JPRB, 0.86671E+00_JPRB, 0.85333E+00_JPRB, 0.86086E+00_JPRB, 0.85865E+00_JPRB, 0.87522E+00_JPRB, 0.90203E+00_JPRB ,&
&  0.92590E+00_JPRB, 0.90455E+00_JPRB, 0.81966E+00_JPRB, 0.72554E+00_JPRB, 0.70064E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.71546E+00_JPRB, 0.69956E+00_JPRB, 0.66245E+00_JPRB, 0.58774E+00_JPRB, 0.55938E+00_JPRB, 0.57289E+00_JPRB, 0.63614E+00_JPRB ,&
&  0.66445E+00_JPRB, 0.67386E+00_JPRB, 0.66072E+00_JPRB, 0.64746E+00_JPRB, 0.64255E+00_JPRB, 0.64871E+00_JPRB, 0.67029E+00_JPRB ,&
&  0.69263E+00_JPRB, 0.68967E+00_JPRB, 0.68679E+00_JPRB, 0.69619E+00_JPRB, 0.72543E+00_JPRB, 0.76712E+00_JPRB, 0.82684E+00_JPRB ,&
&  0.87457E+00_JPRB, 0.89907E+00_JPRB, 0.91895E+00_JPRB, 0.97270E+00_JPRB, 0.10401E+01_JPRB, 0.10696E+01_JPRB, 0.11771E+01_JPRB ,&
&  0.16838E+01_JPRB, 0.24472E+01_JPRB, 0.10668E+01_JPRB, 0.16308E+01_JPRB, 0.19170E+01_JPRB, 0.17415E+01_JPRB, 0.16001E+01_JPRB ,&
&  0.15303E+01_JPRB, 0.13363E+01_JPRB, 0.12386E+01_JPRB, 0.14188E+01_JPRB, 0.16313E+01_JPRB, 0.18076E+01_JPRB, 0.20881E+01_JPRB ,&
&  0.23394E+01_JPRB, 0.24966E+01_JPRB, 0.26191E+01_JPRB, 0.27166E+01_JPRB, 0.28564E+01_JPRB, 0.30508E+01_JPRB, 0.32270E+01_JPRB ,&
&  0.33600E+01_JPRB, 0.35523E+01_JPRB, 0.36817E+01_JPRB, 0.29123E+01_JPRB, 0.27677E+01_JPRB, 0.29633E+01_JPRB, 0.27871E+01_JPRB ,&
&  0.23787E+01_JPRB, 0.20591E+01_JPRB, 0.17093E+01_JPRB, 0.13593E+01_JPRB, 0.11321E+01_JPRB, 0.10408E+01_JPRB, 0.10056E+01_JPRB ,&
&  0.94551E+00_JPRB, 0.87854E+00_JPRB, 0.86524E+00_JPRB, 0.86089E+00_JPRB, 0.86190E+00_JPRB, 0.87111E+00_JPRB, 0.89097E+00_JPRB ,&
&  0.92696E+00_JPRB, 0.96076E+00_JPRB, 0.98657E+00_JPRB, 0.10005E+01_JPRB, 0.10116E+01_JPRB, 0.10195E+01_JPRB, 0.10087E+01_JPRB ,&
&  0.10099E+01_JPRB, 0.99960E+00_JPRB, 0.98541E+00_JPRB, 0.96413E+00_JPRB, 0.93743E+00_JPRB, 0.92385E+00_JPRB, 0.91677E+00_JPRB ,&
&  0.90651E+00_JPRB, 0.89120E+00_JPRB, 0.86224E+00_JPRB, 0.84454E+00_JPRB, 0.87393E+00_JPRB, 0.85389E+00_JPRB, 0.89731E+00_JPRB ,&
&  0.96285E+00_JPRB, 0.10024E+01_JPRB, 0.94823E+00_JPRB, 0.74993E+00_JPRB, 0.72432E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.74612E+00_JPRB, 0.73687E+00_JPRB, 0.72450E+00_JPRB, 0.67255E+00_JPRB, 0.62221E+00_JPRB, 0.59773E+00_JPRB, 0.61159E+00_JPRB ,&
&  0.64720E+00_JPRB, 0.67205E+00_JPRB, 0.67347E+00_JPRB, 0.66514E+00_JPRB, 0.66527E+00_JPRB, 0.67521E+00_JPRB, 0.68897E+00_JPRB ,&
&  0.69373E+00_JPRB, 0.68611E+00_JPRB, 0.67533E+00_JPRB, 0.67964E+00_JPRB, 0.68362E+00_JPRB, 0.69631E+00_JPRB, 0.73042E+00_JPRB ,&
&  0.77620E+00_JPRB, 0.80035E+00_JPRB, 0.80470E+00_JPRB, 0.81916E+00_JPRB, 0.87015E+00_JPRB, 0.88308E+00_JPRB, 0.90580E+00_JPRB ,&
&  0.10909E+01_JPRB, 0.19716E+01_JPRB, 0.21746E+01_JPRB, 0.25721E+01_JPRB, 0.23271E+01_JPRB, 0.19369E+01_JPRB, 0.18175E+01_JPRB ,&
&  0.17676E+01_JPRB, 0.15071E+01_JPRB, 0.13452E+01_JPRB, 0.12151E+01_JPRB, 0.13075E+01_JPRB, 0.14133E+01_JPRB, 0.14587E+01_JPRB ,&
&  0.15890E+01_JPRB, 0.18519E+01_JPRB, 0.20129E+01_JPRB, 0.21683E+01_JPRB, 0.23138E+01_JPRB, 0.24887E+01_JPRB, 0.27447E+01_JPRB ,&
&  0.30529E+01_JPRB, 0.34520E+01_JPRB, 0.39395E+01_JPRB, 0.37076E+01_JPRB, 0.32434E+01_JPRB, 0.37502E+01_JPRB, 0.38968E+01_JPRB ,&
&  0.36825E+01_JPRB, 0.27889E+01_JPRB, 0.21108E+01_JPRB, 0.16232E+01_JPRB, 0.13086E+01_JPRB, 0.13152E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.10144E+01_JPRB, 0.97455E+00_JPRB, 0.95278E+00_JPRB, 0.95930E+00_JPRB, 0.96429E+00_JPRB, 0.95479E+00_JPRB, 0.96840E+00_JPRB ,&
&  0.99904E+00_JPRB, 0.10257E+01_JPRB, 0.10440E+01_JPRB, 0.10561E+01_JPRB, 0.10715E+01_JPRB, 0.10803E+01_JPRB, 0.10656E+01_JPRB ,&
&  0.10381E+01_JPRB, 0.10176E+01_JPRB, 0.95721E+00_JPRB, 0.94970E+00_JPRB, 0.94735E+00_JPRB, 0.95101E+00_JPRB, 0.92342E+00_JPRB ,&
&  0.91602E+00_JPRB, 0.91874E+00_JPRB, 0.89261E+00_JPRB, 0.87237E+00_JPRB, 0.88012E+00_JPRB, 0.86842E+00_JPRB, 0.83631E+00_JPRB ,&
&  0.83855E+00_JPRB, 0.82250E+00_JPRB, 0.80334E+00_JPRB, 0.77678E+00_JPRB, 0.75858E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.79037E+00_JPRB, 0.78142E+00_JPRB, 0.77743E+00_JPRB, 0.75187E+00_JPRB, 0.71900E+00_JPRB, 0.69084E+00_JPRB, 0.67600E+00_JPRB ,&
&  0.67717E+00_JPRB, 0.68899E+00_JPRB, 0.70042E+00_JPRB, 0.69172E+00_JPRB, 0.68974E+00_JPRB, 0.70331E+00_JPRB, 0.71063E+00_JPRB ,&
&  0.71420E+00_JPRB, 0.71182E+00_JPRB, 0.70817E+00_JPRB, 0.70711E+00_JPRB, 0.70288E+00_JPRB, 0.69018E+00_JPRB, 0.68980E+00_JPRB ,&
&  0.70914E+00_JPRB, 0.72809E+00_JPRB, 0.73654E+00_JPRB, 0.73879E+00_JPRB, 0.76807E+00_JPRB, 0.78631E+00_JPRB, 0.81128E+00_JPRB ,&
&  0.90018E+00_JPRB, 0.17421E+01_JPRB, 0.16259E+01_JPRB, 0.29403E+01_JPRB, 0.25087E+01_JPRB, 0.21116E+01_JPRB, 0.17943E+01_JPRB ,&
&  0.17107E+01_JPRB, 0.17911E+01_JPRB, 0.14817E+01_JPRB, 0.12633E+01_JPRB, 0.11811E+01_JPRB, 0.12087E+01_JPRB, 0.12058E+01_JPRB ,&
&  0.11933E+01_JPRB, 0.12461E+01_JPRB, 0.13502E+01_JPRB, 0.15085E+01_JPRB, 0.16946E+01_JPRB, 0.18564E+01_JPRB, 0.20470E+01_JPRB ,&
&  0.22803E+01_JPRB, 0.26809E+01_JPRB, 0.33235E+01_JPRB, 0.34876E+01_JPRB, 0.35929E+01_JPRB, 0.42299E+01_JPRB, 0.49180E+01_JPRB ,&
&  0.48565E+01_JPRB, 0.38467E+01_JPRB, 0.26732E+01_JPRB, 0.19292E+01_JPRB, 0.15799E+01_JPRB, 0.14763E+01_JPRB, 0.12844E+01_JPRB ,&
&  0.11350E+01_JPRB, 0.10673E+01_JPRB, 0.10408E+01_JPRB, 0.10282E+01_JPRB, 0.10318E+01_JPRB, 0.10321E+01_JPRB, 0.10382E+01_JPRB ,&
&  0.10532E+01_JPRB, 0.10823E+01_JPRB, 0.10845E+01_JPRB, 0.10796E+01_JPRB, 0.10824E+01_JPRB, 0.10866E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.10722E+01_JPRB, 0.10550E+01_JPRB, 0.97855E+00_JPRB, 0.94175E+00_JPRB, 0.94204E+00_JPRB, 0.94031E+00_JPRB, 0.93396E+00_JPRB ,&
&  0.95226E+00_JPRB, 0.94858E+00_JPRB, 0.91709E+00_JPRB, 0.87325E+00_JPRB, 0.87915E+00_JPRB, 0.89416E+00_JPRB, 0.84324E+00_JPRB ,&
&  0.82926E+00_JPRB, 0.81939E+00_JPRB, 0.81527E+00_JPRB, 0.81006E+00_JPRB, 0.80016E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.79887E+00_JPRB, 0.79026E+00_JPRB, 0.78904E+00_JPRB, 0.77770E+00_JPRB, 0.75451E+00_JPRB, 0.73103E+00_JPRB, 0.71985E+00_JPRB ,&
&  0.71135E+00_JPRB, 0.71238E+00_JPRB, 0.72091E+00_JPRB, 0.71388E+00_JPRB, 0.71730E+00_JPRB, 0.72669E+00_JPRB, 0.73077E+00_JPRB ,&
&  0.72521E+00_JPRB, 0.72158E+00_JPRB, 0.71478E+00_JPRB, 0.71006E+00_JPRB, 0.70760E+00_JPRB, 0.69594E+00_JPRB, 0.68919E+00_JPRB ,&
&  0.68911E+00_JPRB, 0.69664E+00_JPRB, 0.70398E+00_JPRB, 0.70785E+00_JPRB, 0.72482E+00_JPRB, 0.73786E+00_JPRB, 0.75586E+00_JPRB ,&
&  0.84639E+00_JPRB, 0.18408E+01_JPRB, 0.15028E+01_JPRB, 0.26772E+01_JPRB, 0.23631E+01_JPRB, 0.19647E+01_JPRB, 0.16371E+01_JPRB ,&
&  0.16041E+01_JPRB, 0.16213E+01_JPRB, 0.14770E+01_JPRB, 0.13527E+01_JPRB, 0.12504E+01_JPRB, 0.11683E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.10687E+01_JPRB, 0.10422E+01_JPRB, 0.10462E+01_JPRB, 0.10867E+01_JPRB, 0.11956E+01_JPRB, 0.13344E+01_JPRB, 0.14665E+01_JPRB ,&
&  0.16101E+01_JPRB, 0.18451E+01_JPRB, 0.22942E+01_JPRB, 0.30318E+01_JPRB, 0.32736E+01_JPRB, 0.38240E+01_JPRB, 0.57990E+01_JPRB ,&
&  0.59978E+01_JPRB, 0.51126E+01_JPRB, 0.34415E+01_JPRB, 0.22744E+01_JPRB, 0.18131E+01_JPRB, 0.16279E+01_JPRB, 0.14525E+01_JPRB ,&
&  0.12871E+01_JPRB, 0.11685E+01_JPRB, 0.11305E+01_JPRB, 0.10814E+01_JPRB, 0.10541E+01_JPRB, 0.10748E+01_JPRB, 0.11116E+01_JPRB ,&
&  0.11332E+01_JPRB, 0.11233E+01_JPRB, 0.11248E+01_JPRB, 0.10953E+01_JPRB, 0.10716E+01_JPRB, 0.10514E+01_JPRB, 0.10513E+01_JPRB ,&
&  0.10600E+01_JPRB, 0.10640E+01_JPRB, 0.10142E+01_JPRB, 0.96541E+00_JPRB, 0.95420E+00_JPRB, 0.95533E+00_JPRB, 0.94450E+00_JPRB ,&
&  0.97393E+00_JPRB, 0.96526E+00_JPRB, 0.93089E+00_JPRB, 0.90308E+00_JPRB, 0.87640E+00_JPRB, 0.89302E+00_JPRB, 0.83578E+00_JPRB ,&
&  0.81280E+00_JPRB, 0.80447E+00_JPRB, 0.80511E+00_JPRB, 0.81238E+00_JPRB, 0.81322E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.80939E+00_JPRB, 0.79695E+00_JPRB, 0.79491E+00_JPRB, 0.79100E+00_JPRB, 0.77956E+00_JPRB, 0.76546E+00_JPRB, 0.76402E+00_JPRB ,&
&  0.76253E+00_JPRB, 0.76026E+00_JPRB, 0.74985E+00_JPRB, 0.73328E+00_JPRB, 0.73209E+00_JPRB, 0.73089E+00_JPRB, 0.72800E+00_JPRB ,&
&  0.72379E+00_JPRB, 0.72207E+00_JPRB, 0.71143E+00_JPRB, 0.70563E+00_JPRB, 0.70741E+00_JPRB, 0.70367E+00_JPRB, 0.69365E+00_JPRB ,&
&  0.68332E+00_JPRB, 0.67884E+00_JPRB, 0.68634E+00_JPRB, 0.69451E+00_JPRB, 0.71252E+00_JPRB, 0.71897E+00_JPRB, 0.73340E+00_JPRB ,&
&  0.84643E+00_JPRB, 0.80797E+00_JPRB, 0.17746E+01_JPRB, 0.23046E+01_JPRB, 0.22336E+01_JPRB, 0.19177E+01_JPRB, 0.16950E+01_JPRB ,&
&  0.16216E+01_JPRB, 0.15175E+01_JPRB, 0.14405E+01_JPRB, 0.13299E+01_JPRB, 0.12162E+01_JPRB, 0.10985E+01_JPRB, 0.10479E+01_JPRB ,&
&  0.10235E+01_JPRB, 0.99648E+00_JPRB, 0.98688E+00_JPRB, 0.99037E+00_JPRB, 0.10103E+01_JPRB, 0.10513E+01_JPRB, 0.11243E+01_JPRB ,&
&  0.12028E+01_JPRB, 0.12979E+01_JPRB, 0.14996E+01_JPRB, 0.20615E+01_JPRB, 0.29462E+01_JPRB, 0.36536E+01_JPRB, 0.51388E+01_JPRB ,&
&  0.64670E+01_JPRB, 0.59631E+01_JPRB, 0.41360E+01_JPRB, 0.26949E+01_JPRB, 0.20551E+01_JPRB, 0.18014E+01_JPRB, 0.16087E+01_JPRB ,&
&  0.14532E+01_JPRB, 0.12979E+01_JPRB, 0.11994E+01_JPRB, 0.11285E+01_JPRB, 0.10795E+01_JPRB, 0.10615E+01_JPRB, 0.11132E+01_JPRB ,&
&  0.11630E+01_JPRB, 0.11749E+01_JPRB, 0.11445E+01_JPRB, 0.11070E+01_JPRB, 0.10762E+01_JPRB, 0.10516E+01_JPRB, 0.10478E+01_JPRB ,&
&  0.10567E+01_JPRB, 0.10659E+01_JPRB, 0.10341E+01_JPRB, 0.99765E+00_JPRB, 0.98837E+00_JPRB, 0.99900E+00_JPRB, 0.10003E+01_JPRB ,&
&  0.98353E+00_JPRB, 0.96383E+00_JPRB, 0.92685E+00_JPRB, 0.90973E+00_JPRB, 0.85961E+00_JPRB, 0.88251E+00_JPRB, 0.82973E+00_JPRB ,&
&  0.80343E+00_JPRB, 0.79813E+00_JPRB, 0.80461E+00_JPRB, 0.81074E+00_JPRB, 0.82782E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.81866E+00_JPRB, 0.79711E+00_JPRB, 0.79137E+00_JPRB, 0.79681E+00_JPRB, 0.79540E+00_JPRB, 0.80019E+00_JPRB, 0.80369E+00_JPRB ,&
&  0.79767E+00_JPRB, 0.77856E+00_JPRB, 0.76188E+00_JPRB, 0.74411E+00_JPRB, 0.73815E+00_JPRB, 0.73074E+00_JPRB, 0.72936E+00_JPRB ,&
&  0.72178E+00_JPRB, 0.71683E+00_JPRB, 0.70847E+00_JPRB, 0.70612E+00_JPRB, 0.70612E+00_JPRB, 0.70054E+00_JPRB, 0.67949E+00_JPRB ,&
&  0.67305E+00_JPRB, 0.67259E+00_JPRB, 0.68283E+00_JPRB, 0.69596E+00_JPRB, 0.71256E+00_JPRB, 0.72038E+00_JPRB, 0.73023E+00_JPRB ,&
&  0.83150E+00_JPRB, 0.11286E+01_JPRB, 0.17557E+01_JPRB, 0.18787E+01_JPRB, 0.18805E+01_JPRB, 0.17187E+01_JPRB, 0.15844E+01_JPRB ,&
&  0.14675E+01_JPRB, 0.14167E+01_JPRB, 0.14075E+01_JPRB, 0.12942E+01_JPRB, 0.11310E+01_JPRB, 0.10042E+01_JPRB, 0.96073E+00_JPRB ,&
&  0.94070E+00_JPRB, 0.92617E+00_JPRB, 0.92026E+00_JPRB, 0.92426E+00_JPRB, 0.92568E+00_JPRB, 0.93110E+00_JPRB, 0.95387E+00_JPRB ,&
&  0.98514E+00_JPRB, 0.10291E+01_JPRB, 0.11079E+01_JPRB, 0.13937E+01_JPRB, 0.24187E+01_JPRB, 0.27991E+01_JPRB, 0.44098E+01_JPRB ,&
&  0.64994E+01_JPRB, 0.57710E+01_JPRB, 0.43748E+01_JPRB, 0.31166E+01_JPRB, 0.24111E+01_JPRB, 0.20705E+01_JPRB, 0.18138E+01_JPRB ,&
&  0.16125E+01_JPRB, 0.14442E+01_JPRB, 0.13032E+01_JPRB, 0.12186E+01_JPRB, 0.11468E+01_JPRB, 0.10944E+01_JPRB, 0.10940E+01_JPRB ,&
&  0.11319E+01_JPRB, 0.11569E+01_JPRB, 0.11407E+01_JPRB, 0.11074E+01_JPRB, 0.10796E+01_JPRB, 0.10742E+01_JPRB, 0.10705E+01_JPRB ,&
&  0.10737E+01_JPRB, 0.10726E+01_JPRB, 0.10477E+01_JPRB, 0.10053E+01_JPRB, 0.99987E+00_JPRB, 0.10199E+01_JPRB, 0.10170E+01_JPRB ,&
&  0.97351E+00_JPRB, 0.92927E+00_JPRB, 0.92953E+00_JPRB, 0.89772E+00_JPRB, 0.86776E+00_JPRB, 0.88812E+00_JPRB, 0.82842E+00_JPRB ,&
&  0.79477E+00_JPRB, 0.78802E+00_JPRB, 0.80546E+00_JPRB, 0.82589E+00_JPRB, 0.85815E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.84416E+00_JPRB, 0.80523E+00_JPRB, 0.78531E+00_JPRB, 0.78724E+00_JPRB, 0.79664E+00_JPRB, 0.79701E+00_JPRB, 0.80232E+00_JPRB ,&
&  0.79482E+00_JPRB, 0.77823E+00_JPRB, 0.75947E+00_JPRB, 0.74275E+00_JPRB, 0.73403E+00_JPRB, 0.72791E+00_JPRB, 0.72331E+00_JPRB ,&
&  0.71936E+00_JPRB, 0.71518E+00_JPRB, 0.70816E+00_JPRB, 0.71113E+00_JPRB, 0.71324E+00_JPRB, 0.71127E+00_JPRB, 0.68828E+00_JPRB ,&
&  0.67616E+00_JPRB, 0.67788E+00_JPRB, 0.68528E+00_JPRB, 0.69175E+00_JPRB, 0.70213E+00_JPRB, 0.71536E+00_JPRB, 0.73110E+00_JPRB ,&
&  0.78416E+00_JPRB, 0.10625E+01_JPRB, 0.14102E+01_JPRB, 0.16389E+01_JPRB, 0.17046E+01_JPRB, 0.16418E+01_JPRB, 0.15144E+01_JPRB ,&
&  0.14132E+01_JPRB, 0.13534E+01_JPRB, 0.12968E+01_JPRB, 0.11594E+01_JPRB, 0.10176E+01_JPRB, 0.94133E+00_JPRB, 0.90391E+00_JPRB ,&
&  0.88855E+00_JPRB, 0.88621E+00_JPRB, 0.88230E+00_JPRB, 0.87966E+00_JPRB, 0.87623E+00_JPRB, 0.87829E+00_JPRB, 0.88560E+00_JPRB ,&
&  0.89736E+00_JPRB, 0.90991E+00_JPRB, 0.93928E+00_JPRB, 0.99778E+00_JPRB, 0.14615E+01_JPRB, 0.21509E+01_JPRB, 0.32289E+01_JPRB ,&
&  0.37547E+01_JPRB, 0.38417E+01_JPRB, 0.34740E+01_JPRB, 0.29871E+01_JPRB, 0.25659E+01_JPRB, 0.22376E+01_JPRB, 0.19553E+01_JPRB ,&
&  0.17038E+01_JPRB, 0.14841E+01_JPRB, 0.13075E+01_JPRB, 0.11973E+01_JPRB, 0.11312E+01_JPRB, 0.10779E+01_JPRB, 0.10524E+01_JPRB ,&
&  0.10646E+01_JPRB, 0.10835E+01_JPRB, 0.10906E+01_JPRB, 0.10889E+01_JPRB, 0.10736E+01_JPRB, 0.10572E+01_JPRB, 0.10473E+01_JPRB ,&
&  0.10483E+01_JPRB, 0.10442E+01_JPRB, 0.10452E+01_JPRB, 0.10240E+01_JPRB, 0.10012E+01_JPRB, 0.98387E+00_JPRB, 0.97435E+00_JPRB ,&
&  0.96958E+00_JPRB, 0.93385E+00_JPRB, 0.91386E+00_JPRB, 0.89742E+00_JPRB, 0.86787E+00_JPRB, 0.88525E+00_JPRB, 0.82709E+00_JPRB ,&
&  0.79623E+00_JPRB, 0.78916E+00_JPRB, 0.79344E+00_JPRB, 0.80914E+00_JPRB, 0.84920E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.86492E+00_JPRB, 0.83589E+00_JPRB, 0.80702E+00_JPRB, 0.79056E+00_JPRB, 0.78556E+00_JPRB, 0.78520E+00_JPRB, 0.78586E+00_JPRB ,&
&  0.78309E+00_JPRB, 0.77644E+00_JPRB, 0.75697E+00_JPRB, 0.74167E+00_JPRB, 0.73072E+00_JPRB, 0.72685E+00_JPRB, 0.72342E+00_JPRB ,&
&  0.72216E+00_JPRB, 0.71984E+00_JPRB, 0.71408E+00_JPRB, 0.71265E+00_JPRB, 0.71404E+00_JPRB, 0.70863E+00_JPRB, 0.69075E+00_JPRB ,&
&  0.67208E+00_JPRB, 0.67301E+00_JPRB, 0.67982E+00_JPRB, 0.68548E+00_JPRB, 0.69514E+00_JPRB, 0.70637E+00_JPRB, 0.72535E+00_JPRB ,&
&  0.77571E+00_JPRB, 0.89901E+00_JPRB, 0.12153E+01_JPRB, 0.13997E+01_JPRB, 0.14823E+01_JPRB, 0.15020E+01_JPRB, 0.14248E+01_JPRB ,&
&  0.13262E+01_JPRB, 0.12134E+01_JPRB, 0.11218E+01_JPRB, 0.10247E+01_JPRB, 0.95806E+00_JPRB, 0.90938E+00_JPRB, 0.87711E+00_JPRB ,&
&  0.86102E+00_JPRB, 0.85572E+00_JPRB, 0.85133E+00_JPRB, 0.84540E+00_JPRB, 0.84188E+00_JPRB, 0.84093E+00_JPRB, 0.84210E+00_JPRB ,&
&  0.84664E+00_JPRB, 0.85666E+00_JPRB, 0.87052E+00_JPRB, 0.89051E+00_JPRB, 0.95877E+00_JPRB, 0.12199E+01_JPRB, 0.17978E+01_JPRB ,&
&  0.23197E+01_JPRB, 0.26218E+01_JPRB, 0.25789E+01_JPRB, 0.24749E+01_JPRB, 0.23796E+01_JPRB, 0.21735E+01_JPRB, 0.19541E+01_JPRB ,&
&  0.17366E+01_JPRB, 0.15349E+01_JPRB, 0.13311E+01_JPRB, 0.11704E+01_JPRB, 0.10696E+01_JPRB, 0.10132E+01_JPRB, 0.98406E+00_JPRB ,&
&  0.98564E+00_JPRB, 0.10084E+01_JPRB, 0.10232E+01_JPRB, 0.10391E+01_JPRB, 0.10370E+01_JPRB, 0.10217E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.97970E+00_JPRB, 0.95654E+00_JPRB, 0.94709E+00_JPRB, 0.93374E+00_JPRB, 0.92002E+00_JPRB, 0.91456E+00_JPRB, 0.91037E+00_JPRB ,&
&  0.90785E+00_JPRB, 0.91433E+00_JPRB, 0.87651E+00_JPRB, 0.86133E+00_JPRB, 0.91952E+00_JPRB, 0.88076E+00_JPRB, 0.83957E+00_JPRB ,&
&  0.80808E+00_JPRB, 0.79550E+00_JPRB, 0.79361E+00_JPRB, 0.80921E+00_JPRB, 0.85291E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.77520E+00_JPRB, 0.77503E+00_JPRB, 0.77305E+00_JPRB, 0.77500E+00_JPRB, 0.77551E+00_JPRB, 0.76769E+00_JPRB, 0.76165E+00_JPRB ,&
&  0.76075E+00_JPRB, 0.75711E+00_JPRB, 0.75206E+00_JPRB, 0.74289E+00_JPRB, 0.73027E+00_JPRB, 0.72081E+00_JPRB, 0.71847E+00_JPRB ,&
&  0.71386E+00_JPRB, 0.71177E+00_JPRB, 0.71080E+00_JPRB, 0.70603E+00_JPRB, 0.69993E+00_JPRB, 0.69064E+00_JPRB, 0.67513E+00_JPRB ,&
&  0.65537E+00_JPRB, 0.64878E+00_JPRB, 0.65810E+00_JPRB, 0.66693E+00_JPRB, 0.67356E+00_JPRB, 0.67744E+00_JPRB, 0.69181E+00_JPRB ,&
&  0.75217E+00_JPRB, 0.81269E+00_JPRB, 0.94414E+00_JPRB, 0.11217E+01_JPRB, 0.12252E+01_JPRB, 0.12050E+01_JPRB, 0.11877E+01_JPRB ,&
&  0.11410E+01_JPRB, 0.10892E+01_JPRB, 0.10344E+01_JPRB, 0.97157E+00_JPRB, 0.91727E+00_JPRB, 0.87828E+00_JPRB, 0.85799E+00_JPRB ,&
&  0.84479E+00_JPRB, 0.83322E+00_JPRB, 0.82651E+00_JPRB, 0.81535E+00_JPRB, 0.80638E+00_JPRB, 0.79996E+00_JPRB, 0.80168E+00_JPRB ,&
&  0.80567E+00_JPRB, 0.81154E+00_JPRB, 0.81781E+00_JPRB, 0.82689E+00_JPRB, 0.84670E+00_JPRB, 0.92057E+00_JPRB, 0.11750E+01_JPRB ,&
&  0.15154E+01_JPRB, 0.18350E+01_JPRB, 0.19856E+01_JPRB, 0.19559E+01_JPRB, 0.19121E+01_JPRB, 0.18466E+01_JPRB, 0.16893E+01_JPRB ,&
&  0.14938E+01_JPRB, 0.13642E+01_JPRB, 0.12656E+01_JPRB, 0.11796E+01_JPRB, 0.11155E+01_JPRB, 0.10626E+01_JPRB, 0.10219E+01_JPRB ,&
&  0.99785E+00_JPRB, 0.99206E+00_JPRB, 0.99054E+00_JPRB, 0.98956E+00_JPRB, 0.98153E+00_JPRB, 0.95726E+00_JPRB, 0.91994E+00_JPRB ,&
&  0.87599E+00_JPRB, 0.85390E+00_JPRB, 0.85042E+00_JPRB, 0.83981E+00_JPRB, 0.83076E+00_JPRB, 0.83383E+00_JPRB, 0.84483E+00_JPRB ,&
&  0.84864E+00_JPRB, 0.85753E+00_JPRB, 0.86495E+00_JPRB, 0.89018E+00_JPRB, 0.88362E+00_JPRB, 0.86055E+00_JPRB, 0.84593E+00_JPRB ,&
&  0.82152E+00_JPRB, 0.79810E+00_JPRB, 0.78170E+00_JPRB, 0.78171E+00_JPRB, 0.73743E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.76222E+00_JPRB, 0.75998E+00_JPRB, 0.75464E+00_JPRB, 0.75477E+00_JPRB, 0.75718E+00_JPRB, 0.74602E+00_JPRB, 0.73541E+00_JPRB ,&
&  0.72996E+00_JPRB, 0.73120E+00_JPRB, 0.73471E+00_JPRB, 0.73558E+00_JPRB, 0.72912E+00_JPRB, 0.71721E+00_JPRB, 0.70978E+00_JPRB ,&
&  0.69961E+00_JPRB, 0.69081E+00_JPRB, 0.68846E+00_JPRB, 0.68734E+00_JPRB, 0.68112E+00_JPRB, 0.67325E+00_JPRB, 0.66641E+00_JPRB ,&
&  0.65192E+00_JPRB, 0.64188E+00_JPRB, 0.63983E+00_JPRB, 0.64368E+00_JPRB, 0.64830E+00_JPRB, 0.64820E+00_JPRB, 0.66946E+00_JPRB ,&
&  0.74147E+00_JPRB, 0.71171E+00_JPRB, 0.71929E+00_JPRB, 0.85846E+00_JPRB, 0.96081E+00_JPRB, 0.97960E+00_JPRB, 0.10011E+01_JPRB ,&
&  0.10275E+01_JPRB, 0.10194E+01_JPRB, 0.99098E+00_JPRB, 0.93202E+00_JPRB, 0.88455E+00_JPRB, 0.84983E+00_JPRB, 0.81928E+00_JPRB ,&
&  0.79505E+00_JPRB, 0.77462E+00_JPRB, 0.76355E+00_JPRB, 0.75157E+00_JPRB, 0.74596E+00_JPRB, 0.73515E+00_JPRB, 0.73143E+00_JPRB ,&
&  0.73292E+00_JPRB, 0.74090E+00_JPRB, 0.75014E+00_JPRB, 0.75795E+00_JPRB, 0.77362E+00_JPRB, 0.80578E+00_JPRB, 0.90108E+00_JPRB ,&
&  0.10813E+01_JPRB, 0.13343E+01_JPRB, 0.15183E+01_JPRB, 0.15796E+01_JPRB, 0.15587E+01_JPRB, 0.14893E+01_JPRB, 0.13855E+01_JPRB ,&
&  0.12717E+01_JPRB, 0.11794E+01_JPRB, 0.11132E+01_JPRB, 0.10773E+01_JPRB, 0.10503E+01_JPRB, 0.10278E+01_JPRB, 0.10047E+01_JPRB ,&
&  0.98463E+00_JPRB, 0.97075E+00_JPRB, 0.96012E+00_JPRB, 0.94662E+00_JPRB, 0.92821E+00_JPRB, 0.89215E+00_JPRB, 0.84140E+00_JPRB ,&
&  0.80157E+00_JPRB, 0.77536E+00_JPRB, 0.76619E+00_JPRB, 0.75913E+00_JPRB, 0.75691E+00_JPRB, 0.76955E+00_JPRB, 0.78686E+00_JPRB ,&
&  0.79610E+00_JPRB, 0.80702E+00_JPRB, 0.82917E+00_JPRB, 0.80511E+00_JPRB, 0.84047E+00_JPRB, 0.83976E+00_JPRB, 0.84025E+00_JPRB ,&
&  0.82857E+00_JPRB, 0.81087E+00_JPRB, 0.79552E+00_JPRB, 0.68448E+00_JPRB, 0.74416E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.75016E+00_JPRB, 0.74683E+00_JPRB, 0.73474E+00_JPRB, 0.72698E+00_JPRB, 0.72168E+00_JPRB, 0.71528E+00_JPRB, 0.70457E+00_JPRB ,&
&  0.69843E+00_JPRB, 0.69760E+00_JPRB, 0.69713E+00_JPRB, 0.69788E+00_JPRB, 0.69806E+00_JPRB, 0.69211E+00_JPRB, 0.68281E+00_JPRB ,&
&  0.67347E+00_JPRB, 0.66511E+00_JPRB, 0.66641E+00_JPRB, 0.66531E+00_JPRB, 0.65952E+00_JPRB, 0.65476E+00_JPRB, 0.64871E+00_JPRB ,&
&  0.63392E+00_JPRB, 0.61862E+00_JPRB, 0.60680E+00_JPRB, 0.61000E+00_JPRB, 0.61689E+00_JPRB, 0.61251E+00_JPRB, 0.63111E+00_JPRB ,&
&  0.70361E+00_JPRB, 0.64117E+00_JPRB, 0.67042E+00_JPRB, 0.74886E+00_JPRB, 0.79168E+00_JPRB, 0.83284E+00_JPRB, 0.86290E+00_JPRB ,&
&  0.88548E+00_JPRB, 0.89939E+00_JPRB, 0.85847E+00_JPRB, 0.80928E+00_JPRB, 0.76388E+00_JPRB, 0.72636E+00_JPRB, 0.69963E+00_JPRB ,&
&  0.68473E+00_JPRB, 0.68123E+00_JPRB, 0.67936E+00_JPRB, 0.67289E+00_JPRB, 0.66766E+00_JPRB, 0.66130E+00_JPRB, 0.65766E+00_JPRB ,&
&  0.65634E+00_JPRB, 0.65909E+00_JPRB, 0.66515E+00_JPRB, 0.67869E+00_JPRB, 0.69358E+00_JPRB, 0.71101E+00_JPRB, 0.74322E+00_JPRB ,&
&  0.81188E+00_JPRB, 0.94301E+00_JPRB, 0.10918E+01_JPRB, 0.11765E+01_JPRB, 0.11848E+01_JPRB, 0.11448E+01_JPRB, 0.10779E+01_JPRB ,&
&  0.10116E+01_JPRB, 0.96275E+00_JPRB, 0.93319E+00_JPRB, 0.91438E+00_JPRB, 0.89611E+00_JPRB, 0.88052E+00_JPRB, 0.86812E+00_JPRB ,&
&  0.85768E+00_JPRB, 0.84450E+00_JPRB, 0.83626E+00_JPRB, 0.82372E+00_JPRB, 0.80320E+00_JPRB, 0.77380E+00_JPRB, 0.74456E+00_JPRB ,&
&  0.71684E+00_JPRB, 0.69515E+00_JPRB, 0.68303E+00_JPRB, 0.68241E+00_JPRB, 0.68629E+00_JPRB, 0.69916E+00_JPRB, 0.71852E+00_JPRB ,&
&  0.73359E+00_JPRB, 0.74502E+00_JPRB, 0.75729E+00_JPRB, 0.78129E+00_JPRB, 0.79721E+00_JPRB, 0.81289E+00_JPRB, 0.82124E+00_JPRB ,&
&  0.82165E+00_JPRB, 0.82124E+00_JPRB, 0.66000E+00_JPRB, 0.72512E+00_JPRB, 0.74184E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.74807E+00_JPRB, 0.73021E+00_JPRB, 0.70886E+00_JPRB, 0.69413E+00_JPRB, 0.68658E+00_JPRB, 0.68369E+00_JPRB, 0.67971E+00_JPRB ,&
&  0.67381E+00_JPRB, 0.67356E+00_JPRB, 0.67445E+00_JPRB, 0.67522E+00_JPRB, 0.67531E+00_JPRB, 0.67141E+00_JPRB, 0.66274E+00_JPRB ,&
&  0.65286E+00_JPRB, 0.64080E+00_JPRB, 0.63882E+00_JPRB, 0.63795E+00_JPRB, 0.63137E+00_JPRB, 0.62664E+00_JPRB, 0.62580E+00_JPRB ,&
&  0.61911E+00_JPRB, 0.60861E+00_JPRB, 0.58988E+00_JPRB, 0.58413E+00_JPRB, 0.58719E+00_JPRB, 0.58753E+00_JPRB, 0.59549E+00_JPRB ,&
&  0.58653E+00_JPRB, 0.57157E+00_JPRB, 0.62907E+00_JPRB, 0.66732E+00_JPRB, 0.68667E+00_JPRB, 0.70507E+00_JPRB, 0.72518E+00_JPRB ,&
&  0.72013E+00_JPRB, 0.72107E+00_JPRB, 0.69975E+00_JPRB, 0.66638E+00_JPRB, 0.64316E+00_JPRB, 0.63044E+00_JPRB, 0.62451E+00_JPRB ,&
&  0.62684E+00_JPRB, 0.62869E+00_JPRB, 0.62797E+00_JPRB, 0.62646E+00_JPRB, 0.62655E+00_JPRB, 0.62273E+00_JPRB, 0.62039E+00_JPRB ,&
&  0.62190E+00_JPRB, 0.62562E+00_JPRB, 0.62782E+00_JPRB, 0.63256E+00_JPRB, 0.64249E+00_JPRB, 0.65753E+00_JPRB, 0.67581E+00_JPRB ,&
&  0.69890E+00_JPRB, 0.73987E+00_JPRB, 0.81096E+00_JPRB, 0.87333E+00_JPRB, 0.89555E+00_JPRB, 0.87754E+00_JPRB, 0.84200E+00_JPRB ,&
&  0.81219E+00_JPRB, 0.79461E+00_JPRB, 0.78284E+00_JPRB, 0.77188E+00_JPRB, 0.75623E+00_JPRB, 0.74140E+00_JPRB, 0.72738E+00_JPRB ,&
&  0.72080E+00_JPRB, 0.71056E+00_JPRB, 0.69944E+00_JPRB, 0.68602E+00_JPRB, 0.67383E+00_JPRB, 0.66608E+00_JPRB, 0.66076E+00_JPRB ,&
&  0.65473E+00_JPRB, 0.64503E+00_JPRB, 0.63885E+00_JPRB, 0.64018E+00_JPRB, 0.64309E+00_JPRB, 0.65038E+00_JPRB, 0.66208E+00_JPRB ,&
&  0.67676E+00_JPRB, 0.69030E+00_JPRB, 0.70474E+00_JPRB, 0.73136E+00_JPRB, 0.75814E+00_JPRB, 0.78175E+00_JPRB, 0.79616E+00_JPRB ,&
&  0.80514E+00_JPRB, 0.79845E+00_JPRB, 0.79393E+00_JPRB, 0.77990E+00_JPRB, 0.76257E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.68936E+00_JPRB, 0.66791E+00_JPRB, 0.65099E+00_JPRB, 0.63983E+00_JPRB, 0.63352E+00_JPRB, 0.63232E+00_JPRB, 0.62943E+00_JPRB ,&
&  0.62726E+00_JPRB, 0.62361E+00_JPRB, 0.62134E+00_JPRB, 0.62392E+00_JPRB, 0.62523E+00_JPRB, 0.62566E+00_JPRB, 0.62645E+00_JPRB ,&
&  0.62033E+00_JPRB, 0.60697E+00_JPRB, 0.59641E+00_JPRB, 0.59337E+00_JPRB, 0.59406E+00_JPRB, 0.58976E+00_JPRB, 0.58815E+00_JPRB ,&
&  0.59118E+00_JPRB, 0.58315E+00_JPRB, 0.56959E+00_JPRB, 0.56082E+00_JPRB, 0.56148E+00_JPRB, 0.55913E+00_JPRB, 0.55712E+00_JPRB ,&
&  0.58299E+00_JPRB, 0.53613E+00_JPRB, 0.58703E+00_JPRB, 0.59597E+00_JPRB, 0.60339E+00_JPRB, 0.60812E+00_JPRB, 0.60862E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.59710E+00_JPRB, 0.58969E+00_JPRB, 0.58211E+00_JPRB, 0.57656E+00_JPRB, 0.57030E+00_JPRB, 0.56514E+00_JPRB ,&
&  0.56520E+00_JPRB, 0.56576E+00_JPRB, 0.56990E+00_JPRB, 0.57532E+00_JPRB, 0.58288E+00_JPRB, 0.58551E+00_JPRB, 0.58781E+00_JPRB ,&
&  0.59056E+00_JPRB, 0.59221E+00_JPRB, 0.59205E+00_JPRB, 0.59083E+00_JPRB, 0.58937E+00_JPRB, 0.59390E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60926E+00_JPRB, 0.62719E+00_JPRB, 0.65715E+00_JPRB, 0.68565E+00_JPRB, 0.69854E+00_JPRB, 0.69452E+00_JPRB, 0.68035E+00_JPRB ,&
&  0.67330E+00_JPRB, 0.67538E+00_JPRB, 0.67858E+00_JPRB, 0.67456E+00_JPRB, 0.66670E+00_JPRB, 0.65913E+00_JPRB, 0.64655E+00_JPRB ,&
&  0.62998E+00_JPRB, 0.61533E+00_JPRB, 0.60517E+00_JPRB, 0.59895E+00_JPRB, 0.59623E+00_JPRB, 0.59287E+00_JPRB, 0.58936E+00_JPRB ,&
&  0.58837E+00_JPRB, 0.58633E+00_JPRB, 0.58640E+00_JPRB, 0.59078E+00_JPRB, 0.59494E+00_JPRB, 0.60096E+00_JPRB, 0.60867E+00_JPRB ,&
&  0.61940E+00_JPRB, 0.63113E+00_JPRB, 0.64265E+00_JPRB, 0.66135E+00_JPRB, 0.68803E+00_JPRB, 0.70724E+00_JPRB, 0.71983E+00_JPRB ,&
&  0.73150E+00_JPRB, 0.73455E+00_JPRB, 0.72853E+00_JPRB, 0.71912E+00_JPRB, 0.70571E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.64098E+00_JPRB, 0.62772E+00_JPRB, 0.61740E+00_JPRB, 0.60952E+00_JPRB, 0.60439E+00_JPRB, 0.60323E+00_JPRB, 0.60097E+00_JPRB ,&
&  0.59873E+00_JPRB, 0.59670E+00_JPRB, 0.59003E+00_JPRB, 0.58349E+00_JPRB, 0.58328E+00_JPRB, 0.58923E+00_JPRB, 0.59375E+00_JPRB ,&
&  0.59060E+00_JPRB, 0.57972E+00_JPRB, 0.56956E+00_JPRB, 0.56728E+00_JPRB, 0.56997E+00_JPRB, 0.57224E+00_JPRB, 0.57177E+00_JPRB ,&
&  0.57344E+00_JPRB, 0.57456E+00_JPRB, 0.56631E+00_JPRB, 0.54925E+00_JPRB, 0.54148E+00_JPRB, 0.53704E+00_JPRB, 0.54207E+00_JPRB ,&
&  0.55190E+00_JPRB, 0.55101E+00_JPRB, 0.56232E+00_JPRB, 0.55939E+00_JPRB, 0.55833E+00_JPRB, 0.55470E+00_JPRB, 0.54792E+00_JPRB ,&
&  0.54233E+00_JPRB, 0.53647E+00_JPRB, 0.53260E+00_JPRB, 0.53051E+00_JPRB, 0.53031E+00_JPRB, 0.53107E+00_JPRB, 0.53491E+00_JPRB ,&
&  0.53532E+00_JPRB, 0.53302E+00_JPRB, 0.53537E+00_JPRB, 0.54234E+00_JPRB, 0.55117E+00_JPRB, 0.55632E+00_JPRB, 0.56103E+00_JPRB ,&
&  0.56488E+00_JPRB, 0.56703E+00_JPRB, 0.56984E+00_JPRB, 0.57046E+00_JPRB, 0.56656E+00_JPRB, 0.56560E+00_JPRB, 0.56530E+00_JPRB ,&
&  0.56787E+00_JPRB, 0.57442E+00_JPRB, 0.58875E+00_JPRB, 0.60662E+00_JPRB, 0.61400E+00_JPRB, 0.60840E+00_JPRB, 0.60062E+00_JPRB ,&
&  0.60051E+00_JPRB, 0.60766E+00_JPRB, 0.61578E+00_JPRB, 0.62099E+00_JPRB, 0.62133E+00_JPRB, 0.61366E+00_JPRB, 0.60325E+00_JPRB ,&
&  0.59171E+00_JPRB, 0.57982E+00_JPRB, 0.56899E+00_JPRB, 0.56178E+00_JPRB, 0.55591E+00_JPRB, 0.55122E+00_JPRB, 0.54822E+00_JPRB ,&
&  0.54683E+00_JPRB, 0.54657E+00_JPRB, 0.54956E+00_JPRB, 0.55609E+00_JPRB, 0.56025E+00_JPRB, 0.56909E+00_JPRB, 0.57989E+00_JPRB ,&
&  0.59078E+00_JPRB, 0.60078E+00_JPRB, 0.61252E+00_JPRB, 0.62941E+00_JPRB, 0.64859E+00_JPRB, 0.66259E+00_JPRB, 0.67366E+00_JPRB ,&
&  0.68077E+00_JPRB, 0.67981E+00_JPRB, 0.67193E+00_JPRB, 0.66297E+00_JPRB, 0.65333E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.59460E+00_JPRB, 0.58678E+00_JPRB, 0.58051E+00_JPRB, 0.57513E+00_JPRB, 0.57062E+00_JPRB, 0.56899E+00_JPRB, 0.56721E+00_JPRB ,&
&  0.56535E+00_JPRB, 0.56275E+00_JPRB, 0.56115E+00_JPRB, 0.55762E+00_JPRB, 0.55473E+00_JPRB, 0.55779E+00_JPRB, 0.56351E+00_JPRB ,&
&  0.56207E+00_JPRB, 0.55510E+00_JPRB, 0.54963E+00_JPRB, 0.54952E+00_JPRB, 0.55288E+00_JPRB, 0.55536E+00_JPRB, 0.55812E+00_JPRB ,&
&  0.55783E+00_JPRB, 0.55703E+00_JPRB, 0.55523E+00_JPRB, 0.53941E+00_JPRB, 0.52450E+00_JPRB, 0.51722E+00_JPRB, 0.51310E+00_JPRB ,&
&  0.51333E+00_JPRB, 0.52076E+00_JPRB, 0.50414E+00_JPRB, 0.51350E+00_JPRB, 0.50921E+00_JPRB, 0.50284E+00_JPRB, 0.49915E+00_JPRB ,&
&  0.49704E+00_JPRB, 0.49477E+00_JPRB, 0.49307E+00_JPRB, 0.49498E+00_JPRB, 0.49756E+00_JPRB, 0.49824E+00_JPRB, 0.49983E+00_JPRB ,&
&  0.49948E+00_JPRB, 0.50190E+00_JPRB, 0.50410E+00_JPRB, 0.50927E+00_JPRB, 0.51320E+00_JPRB, 0.51569E+00_JPRB, 0.51894E+00_JPRB ,&
&  0.52348E+00_JPRB, 0.52771E+00_JPRB, 0.53175E+00_JPRB, 0.53513E+00_JPRB, 0.53646E+00_JPRB, 0.53612E+00_JPRB, 0.53184E+00_JPRB ,&
&  0.52841E+00_JPRB, 0.52829E+00_JPRB, 0.53311E+00_JPRB, 0.54451E+00_JPRB, 0.55434E+00_JPRB, 0.55491E+00_JPRB, 0.55236E+00_JPRB ,&
&  0.55274E+00_JPRB, 0.55763E+00_JPRB, 0.56011E+00_JPRB, 0.56043E+00_JPRB, 0.55949E+00_JPRB, 0.55738E+00_JPRB, 0.55293E+00_JPRB ,&
&  0.54641E+00_JPRB, 0.54061E+00_JPRB, 0.53452E+00_JPRB, 0.52258E+00_JPRB, 0.51361E+00_JPRB, 0.51254E+00_JPRB, 0.51400E+00_JPRB ,&
&  0.51338E+00_JPRB, 0.51174E+00_JPRB, 0.51288E+00_JPRB, 0.51863E+00_JPRB, 0.51917E+00_JPRB, 0.52672E+00_JPRB, 0.54122E+00_JPRB ,&
&  0.55583E+00_JPRB, 0.56847E+00_JPRB, 0.57833E+00_JPRB, 0.58845E+00_JPRB, 0.59843E+00_JPRB, 0.60595E+00_JPRB, 0.61238E+00_JPRB ,&
&  0.61534E+00_JPRB, 0.61358E+00_JPRB, 0.60815E+00_JPRB, 0.60488E+00_JPRB, 0.60201E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.56751E+00_JPRB, 0.56578E+00_JPRB, 0.56389E+00_JPRB, 0.56145E+00_JPRB, 0.55806E+00_JPRB, 0.55506E+00_JPRB, 0.55033E+00_JPRB ,&
&  0.54549E+00_JPRB, 0.54166E+00_JPRB, 0.54006E+00_JPRB, 0.53953E+00_JPRB, 0.54093E+00_JPRB, 0.54522E+00_JPRB, 0.55224E+00_JPRB ,&
&  0.55235E+00_JPRB, 0.54543E+00_JPRB, 0.54213E+00_JPRB, 0.54217E+00_JPRB, 0.54456E+00_JPRB, 0.54626E+00_JPRB, 0.54886E+00_JPRB ,&
&  0.55046E+00_JPRB, 0.55088E+00_JPRB, 0.54981E+00_JPRB, 0.54238E+00_JPRB, 0.52542E+00_JPRB, 0.51527E+00_JPRB, 0.50645E+00_JPRB ,&
&  0.50133E+00_JPRB, 0.49744E+00_JPRB, 0.49424E+00_JPRB, 0.49314E+00_JPRB, 0.49194E+00_JPRB, 0.49006E+00_JPRB, 0.49001E+00_JPRB ,&
&  0.48816E+00_JPRB, 0.48575E+00_JPRB, 0.48451E+00_JPRB, 0.48516E+00_JPRB, 0.48697E+00_JPRB, 0.48857E+00_JPRB, 0.48840E+00_JPRB ,&
&  0.48929E+00_JPRB, 0.49291E+00_JPRB, 0.49381E+00_JPRB, 0.49382E+00_JPRB, 0.49348E+00_JPRB, 0.49289E+00_JPRB, 0.49288E+00_JPRB ,&
&  0.49522E+00_JPRB, 0.50041E+00_JPRB, 0.50493E+00_JPRB, 0.50954E+00_JPRB, 0.51161E+00_JPRB, 0.51259E+00_JPRB, 0.51252E+00_JPRB ,&
&  0.50923E+00_JPRB, 0.50504E+00_JPRB, 0.50353E+00_JPRB, 0.50667E+00_JPRB, 0.51315E+00_JPRB, 0.51923E+00_JPRB, 0.52187E+00_JPRB ,&
&  0.52391E+00_JPRB, 0.52514E+00_JPRB, 0.52352E+00_JPRB, 0.52306E+00_JPRB, 0.52109E+00_JPRB, 0.51711E+00_JPRB, 0.51487E+00_JPRB ,&
&  0.51257E+00_JPRB, 0.51029E+00_JPRB, 0.50798E+00_JPRB, 0.49885E+00_JPRB, 0.49135E+00_JPRB, 0.48704E+00_JPRB, 0.48429E+00_JPRB ,&
&  0.48344E+00_JPRB, 0.48267E+00_JPRB, 0.48250E+00_JPRB, 0.48372E+00_JPRB, 0.48590E+00_JPRB, 0.49315E+00_JPRB, 0.50724E+00_JPRB ,&
&  0.52760E+00_JPRB, 0.54632E+00_JPRB, 0.55636E+00_JPRB, 0.56333E+00_JPRB, 0.56761E+00_JPRB, 0.56966E+00_JPRB, 0.57173E+00_JPRB ,&
&  0.57511E+00_JPRB, 0.57618E+00_JPRB, 0.57397E+00_JPRB, 0.57127E+00_JPRB, 0.56973E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.53886E+00_JPRB, 0.53893E+00_JPRB, 0.53729E+00_JPRB, 0.53571E+00_JPRB, 0.53403E+00_JPRB, 0.53254E+00_JPRB, 0.53043E+00_JPRB ,&
&  0.52556E+00_JPRB, 0.51925E+00_JPRB, 0.51560E+00_JPRB, 0.51411E+00_JPRB, 0.51998E+00_JPRB, 0.52851E+00_JPRB, 0.53609E+00_JPRB ,&
&  0.53514E+00_JPRB, 0.52583E+00_JPRB, 0.52293E+00_JPRB, 0.52486E+00_JPRB, 0.52738E+00_JPRB, 0.53015E+00_JPRB, 0.53385E+00_JPRB ,&
&  0.53791E+00_JPRB, 0.54201E+00_JPRB, 0.54383E+00_JPRB, 0.54104E+00_JPRB, 0.53423E+00_JPRB, 0.52032E+00_JPRB, 0.50828E+00_JPRB ,&
&  0.50034E+00_JPRB, 0.49418E+00_JPRB, 0.49035E+00_JPRB, 0.48920E+00_JPRB, 0.48690E+00_JPRB, 0.48310E+00_JPRB, 0.47784E+00_JPRB ,&
&  0.47335E+00_JPRB, 0.47138E+00_JPRB, 0.47382E+00_JPRB, 0.47576E+00_JPRB, 0.47671E+00_JPRB, 0.47632E+00_JPRB, 0.47317E+00_JPRB ,&
&  0.47129E+00_JPRB, 0.47055E+00_JPRB, 0.46999E+00_JPRB, 0.47127E+00_JPRB, 0.46862E+00_JPRB, 0.46642E+00_JPRB, 0.46832E+00_JPRB ,&
&  0.47279E+00_JPRB, 0.47683E+00_JPRB, 0.48060E+00_JPRB, 0.48367E+00_JPRB, 0.48509E+00_JPRB, 0.48737E+00_JPRB, 0.48870E+00_JPRB ,&
&  0.48760E+00_JPRB, 0.48375E+00_JPRB, 0.48014E+00_JPRB, 0.48044E+00_JPRB, 0.48259E+00_JPRB, 0.48510E+00_JPRB, 0.48846E+00_JPRB ,&
&  0.48897E+00_JPRB, 0.48388E+00_JPRB, 0.47885E+00_JPRB, 0.47449E+00_JPRB, 0.47057E+00_JPRB, 0.46768E+00_JPRB, 0.46639E+00_JPRB ,&
&  0.46620E+00_JPRB, 0.46315E+00_JPRB, 0.45716E+00_JPRB, 0.45227E+00_JPRB, 0.45192E+00_JPRB, 0.45169E+00_JPRB, 0.44832E+00_JPRB ,&
&  0.44918E+00_JPRB, 0.45204E+00_JPRB, 0.45107E+00_JPRB, 0.44917E+00_JPRB, 0.45116E+00_JPRB, 0.45684E+00_JPRB, 0.46868E+00_JPRB ,&
&  0.48439E+00_JPRB, 0.50227E+00_JPRB, 0.51866E+00_JPRB, 0.53248E+00_JPRB, 0.54046E+00_JPRB, 0.54223E+00_JPRB, 0.54037E+00_JPRB ,&
&  0.53968E+00_JPRB, 0.54052E+00_JPRB, 0.53968E+00_JPRB, 0.53809E+00_JPRB, 0.53810E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.50985E+00_JPRB, 0.50959E+00_JPRB, 0.50963E+00_JPRB, 0.51064E+00_JPRB, 0.51132E+00_JPRB, 0.51021E+00_JPRB, 0.50986E+00_JPRB ,&
&  0.50664E+00_JPRB, 0.50202E+00_JPRB, 0.49930E+00_JPRB, 0.50110E+00_JPRB, 0.51059E+00_JPRB, 0.52246E+00_JPRB, 0.52948E+00_JPRB ,&
&  0.52479E+00_JPRB, 0.51206E+00_JPRB, 0.50897E+00_JPRB, 0.51128E+00_JPRB, 0.51353E+00_JPRB, 0.51765E+00_JPRB, 0.52373E+00_JPRB ,&
&  0.52927E+00_JPRB, 0.53305E+00_JPRB, 0.53722E+00_JPRB, 0.53893E+00_JPRB, 0.53904E+00_JPRB, 0.53178E+00_JPRB, 0.52047E+00_JPRB ,&
&  0.50961E+00_JPRB, 0.50168E+00_JPRB, 0.49649E+00_JPRB, 0.49279E+00_JPRB, 0.48764E+00_JPRB, 0.48256E+00_JPRB, 0.47601E+00_JPRB ,&
&  0.47294E+00_JPRB, 0.47283E+00_JPRB, 0.47392E+00_JPRB, 0.47210E+00_JPRB, 0.46945E+00_JPRB, 0.46401E+00_JPRB, 0.45934E+00_JPRB ,&
&  0.45732E+00_JPRB, 0.45821E+00_JPRB, 0.45911E+00_JPRB, 0.45782E+00_JPRB, 0.45399E+00_JPRB, 0.45119E+00_JPRB, 0.45230E+00_JPRB ,&
&  0.45803E+00_JPRB, 0.45777E+00_JPRB, 0.45678E+00_JPRB, 0.45778E+00_JPRB, 0.46019E+00_JPRB, 0.46226E+00_JPRB, 0.46162E+00_JPRB ,&
&  0.46081E+00_JPRB, 0.46051E+00_JPRB, 0.46050E+00_JPRB, 0.46212E+00_JPRB, 0.46205E+00_JPRB, 0.45974E+00_JPRB, 0.45403E+00_JPRB ,&
&  0.44545E+00_JPRB, 0.43477E+00_JPRB, 0.43041E+00_JPRB, 0.42731E+00_JPRB, 0.42438E+00_JPRB, 0.42393E+00_JPRB, 0.42556E+00_JPRB ,&
&  0.42432E+00_JPRB, 0.41819E+00_JPRB, 0.41549E+00_JPRB, 0.41629E+00_JPRB, 0.41590E+00_JPRB, 0.41873E+00_JPRB, 0.42318E+00_JPRB ,&
&  0.43413E+00_JPRB, 0.44038E+00_JPRB, 0.44225E+00_JPRB, 0.44010E+00_JPRB, 0.44035E+00_JPRB, 0.44214E+00_JPRB, 0.44669E+00_JPRB ,&
&  0.45351E+00_JPRB, 0.46094E+00_JPRB, 0.46866E+00_JPRB, 0.47722E+00_JPRB, 0.48768E+00_JPRB, 0.49771E+00_JPRB, 0.50160E+00_JPRB ,&
&  0.50196E+00_JPRB, 0.50251E+00_JPRB, 0.50397E+00_JPRB, 0.50567E+00_JPRB, 0.50864E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.47438E+00_JPRB, 0.47660E+00_JPRB, 0.47889E+00_JPRB, 0.48048E+00_JPRB, 0.48322E+00_JPRB, 0.48817E+00_JPRB, 0.49014E+00_JPRB ,&
&  0.48813E+00_JPRB, 0.48587E+00_JPRB, 0.48578E+00_JPRB, 0.48941E+00_JPRB, 0.49647E+00_JPRB, 0.50373E+00_JPRB, 0.51150E+00_JPRB ,&
&  0.50715E+00_JPRB, 0.49424E+00_JPRB, 0.48959E+00_JPRB, 0.49167E+00_JPRB, 0.49659E+00_JPRB, 0.50204E+00_JPRB, 0.50870E+00_JPRB ,&
&  0.51502E+00_JPRB, 0.51911E+00_JPRB, 0.52344E+00_JPRB, 0.52740E+00_JPRB, 0.52996E+00_JPRB, 0.52810E+00_JPRB, 0.52358E+00_JPRB ,&
&  0.51813E+00_JPRB, 0.51244E+00_JPRB, 0.50723E+00_JPRB, 0.49158E+00_JPRB, 0.48901E+00_JPRB, 0.48006E+00_JPRB, 0.47268E+00_JPRB ,&
&  0.46765E+00_JPRB, 0.46348E+00_JPRB, 0.46077E+00_JPRB, 0.45756E+00_JPRB, 0.45298E+00_JPRB, 0.44528E+00_JPRB, 0.44276E+00_JPRB ,&
&  0.44139E+00_JPRB, 0.44205E+00_JPRB, 0.43698E+00_JPRB, 0.43412E+00_JPRB, 0.43094E+00_JPRB, 0.42838E+00_JPRB, 0.43226E+00_JPRB ,&
&  0.43718E+00_JPRB, 0.43447E+00_JPRB, 0.43039E+00_JPRB, 0.43013E+00_JPRB, 0.43451E+00_JPRB, 0.43618E+00_JPRB, 0.43707E+00_JPRB ,&
&  0.43869E+00_JPRB, 0.43893E+00_JPRB, 0.43393E+00_JPRB, 0.42855E+00_JPRB, 0.41994E+00_JPRB, 0.40877E+00_JPRB, 0.40551E+00_JPRB ,&
&  0.40238E+00_JPRB, 0.39676E+00_JPRB, 0.39369E+00_JPRB, 0.39273E+00_JPRB, 0.39247E+00_JPRB, 0.39349E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39406E+00_JPRB, 0.39064E+00_JPRB, 0.38560E+00_JPRB, 0.38142E+00_JPRB, 0.37569E+00_JPRB, 0.37901E+00_JPRB, 0.38962E+00_JPRB ,&
&  0.40242E+00_JPRB, 0.41030E+00_JPRB, 0.41540E+00_JPRB, 0.41886E+00_JPRB, 0.42195E+00_JPRB, 0.42332E+00_JPRB, 0.42569E+00_JPRB ,&
&  0.42883E+00_JPRB, 0.43030E+00_JPRB, 0.42913E+00_JPRB, 0.42973E+00_JPRB, 0.43341E+00_JPRB, 0.43906E+00_JPRB, 0.44570E+00_JPRB ,&
&  0.45216E+00_JPRB, 0.45754E+00_JPRB, 0.46200E+00_JPRB, 0.46754E+00_JPRB, 0.47112E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.44525E+00_JPRB, 0.44968E+00_JPRB, 0.45393E+00_JPRB, 0.45857E+00_JPRB, 0.46511E+00_JPRB, 0.46861E+00_JPRB, 0.46975E+00_JPRB ,&
&  0.47104E+00_JPRB, 0.47147E+00_JPRB, 0.47285E+00_JPRB, 0.47716E+00_JPRB, 0.48352E+00_JPRB, 0.48825E+00_JPRB, 0.49426E+00_JPRB ,&
&  0.49431E+00_JPRB, 0.48527E+00_JPRB, 0.47720E+00_JPRB, 0.47695E+00_JPRB, 0.48259E+00_JPRB, 0.48842E+00_JPRB, 0.49569E+00_JPRB ,&
&  0.50283E+00_JPRB, 0.50795E+00_JPRB, 0.51081E+00_JPRB, 0.51224E+00_JPRB, 0.51412E+00_JPRB, 0.51358E+00_JPRB, 0.51042E+00_JPRB ,&
&  0.50595E+00_JPRB, 0.49645E+00_JPRB, 0.47690E+00_JPRB, 0.47425E+00_JPRB, 0.46550E+00_JPRB, 0.45792E+00_JPRB, 0.45268E+00_JPRB ,&
&  0.44850E+00_JPRB, 0.44496E+00_JPRB, 0.44082E+00_JPRB, 0.43708E+00_JPRB, 0.43275E+00_JPRB, 0.42523E+00_JPRB, 0.42117E+00_JPRB ,&
&  0.41746E+00_JPRB, 0.41444E+00_JPRB, 0.41137E+00_JPRB, 0.41118E+00_JPRB, 0.41117E+00_JPRB, 0.41360E+00_JPRB, 0.41468E+00_JPRB ,&
&  0.41122E+00_JPRB, 0.40722E+00_JPRB, 0.40420E+00_JPRB, 0.40579E+00_JPRB, 0.40814E+00_JPRB, 0.40976E+00_JPRB, 0.41033E+00_JPRB ,&
&  0.40758E+00_JPRB, 0.40253E+00_JPRB, 0.39723E+00_JPRB, 0.39126E+00_JPRB, 0.38480E+00_JPRB, 0.35912E+00_JPRB, 0.34814E+00_JPRB ,&
&  0.34547E+00_JPRB, 0.35481E+00_JPRB, 0.36675E+00_JPRB, 0.36601E+00_JPRB, 0.36898E+00_JPRB, 0.36934E+00_JPRB, 0.36680E+00_JPRB ,&
&  0.35736E+00_JPRB, 0.34406E+00_JPRB, 0.33561E+00_JPRB, 0.33197E+00_JPRB, 0.32943E+00_JPRB, 0.33164E+00_JPRB, 0.33845E+00_JPRB ,&
&  0.34921E+00_JPRB, 0.36109E+00_JPRB, 0.36999E+00_JPRB, 0.37401E+00_JPRB, 0.37275E+00_JPRB, 0.36965E+00_JPRB, 0.36554E+00_JPRB ,&
&  0.36139E+00_JPRB, 0.36774E+00_JPRB, 0.38274E+00_JPRB, 0.40263E+00_JPRB, 0.40536E+00_JPRB, 0.41085E+00_JPRB, 0.41461E+00_JPRB ,&
&  0.42013E+00_JPRB, 0.42707E+00_JPRB, 0.43288E+00_JPRB, 0.43882E+00_JPRB, 0.44238E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.42231E+00_JPRB, 0.42737E+00_JPRB, 0.43424E+00_JPRB, 0.44190E+00_JPRB, 0.44744E+00_JPRB, 0.45015E+00_JPRB, 0.45305E+00_JPRB ,&
&  0.45661E+00_JPRB, 0.45813E+00_JPRB, 0.45919E+00_JPRB, 0.46193E+00_JPRB, 0.46762E+00_JPRB, 0.47093E+00_JPRB, 0.47605E+00_JPRB ,&
&  0.47818E+00_JPRB, 0.47593E+00_JPRB, 0.46897E+00_JPRB, 0.46704E+00_JPRB, 0.47077E+00_JPRB, 0.47566E+00_JPRB, 0.48088E+00_JPRB ,&
&  0.48620E+00_JPRB, 0.49105E+00_JPRB, 0.49328E+00_JPRB, 0.49377E+00_JPRB, 0.49380E+00_JPRB, 0.49351E+00_JPRB, 0.49072E+00_JPRB ,&
&  0.48479E+00_JPRB, 0.44454E+00_JPRB, 0.41810E+00_JPRB, 0.42270E+00_JPRB, 0.43388E+00_JPRB, 0.42760E+00_JPRB, 0.42601E+00_JPRB ,&
&  0.42446E+00_JPRB, 0.42068E+00_JPRB, 0.41511E+00_JPRB, 0.41045E+00_JPRB, 0.40479E+00_JPRB, 0.39743E+00_JPRB, 0.39162E+00_JPRB ,&
&  0.39004E+00_JPRB, 0.39518E+00_JPRB, 0.39655E+00_JPRB, 0.39811E+00_JPRB, 0.39605E+00_JPRB, 0.39729E+00_JPRB, 0.39544E+00_JPRB ,&
&  0.38935E+00_JPRB, 0.38249E+00_JPRB, 0.38000E+00_JPRB, 0.37941E+00_JPRB, 0.38054E+00_JPRB, 0.38327E+00_JPRB, 0.38268E+00_JPRB ,&
&  0.36978E+00_JPRB, 0.35811E+00_JPRB, 0.33933E+00_JPRB, 0.32109E+00_JPRB, 0.30789E+00_JPRB, 0.30198E+00_JPRB, 0.29773E+00_JPRB ,&
&  0.29715E+00_JPRB, 0.30368E+00_JPRB, 0.32162E+00_JPRB, 0.34305E+00_JPRB, 0.35541E+00_JPRB, 0.33692E+00_JPRB, 0.32172E+00_JPRB ,&
&  0.31258E+00_JPRB, 0.30766E+00_JPRB, 0.30337E+00_JPRB, 0.30203E+00_JPRB, 0.30274E+00_JPRB, 0.30538E+00_JPRB, 0.30907E+00_JPRB ,&
&  0.31419E+00_JPRB, 0.31887E+00_JPRB, 0.32168E+00_JPRB, 0.32303E+00_JPRB, 0.32382E+00_JPRB, 0.32467E+00_JPRB, 0.32412E+00_JPRB ,&
&  0.32414E+00_JPRB, 0.32804E+00_JPRB, 0.33546E+00_JPRB, 0.34474E+00_JPRB, 0.35315E+00_JPRB, 0.36267E+00_JPRB, 0.37599E+00_JPRB ,&
&  0.39047E+00_JPRB, 0.40171E+00_JPRB, 0.40582E+00_JPRB, 0.41216E+00_JPRB, 0.41756E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.41216E+00_JPRB, 0.41926E+00_JPRB, 0.42414E+00_JPRB, 0.43387E+00_JPRB, 0.43686E+00_JPRB, 0.44009E+00_JPRB, 0.44294E+00_JPRB ,&
&  0.44957E+00_JPRB, 0.45330E+00_JPRB, 0.45123E+00_JPRB, 0.45078E+00_JPRB, 0.44998E+00_JPRB, 0.44860E+00_JPRB, 0.44983E+00_JPRB ,&
&  0.45017E+00_JPRB, 0.45246E+00_JPRB, 0.45000E+00_JPRB, 0.44731E+00_JPRB, 0.44934E+00_JPRB, 0.45427E+00_JPRB, 0.45755E+00_JPRB ,&
&  0.45214E+00_JPRB, 0.45040E+00_JPRB, 0.45880E+00_JPRB, 0.46060E+00_JPRB, 0.46152E+00_JPRB, 0.46326E+00_JPRB, 0.46282E+00_JPRB ,&
&  0.44854E+00_JPRB, 0.42241E+00_JPRB, 0.39483E+00_JPRB, 0.39742E+00_JPRB, 0.41221E+00_JPRB, 0.40545E+00_JPRB, 0.40365E+00_JPRB ,&
&  0.40075E+00_JPRB, 0.39606E+00_JPRB, 0.39166E+00_JPRB, 0.38938E+00_JPRB, 0.38659E+00_JPRB, 0.38166E+00_JPRB, 0.38012E+00_JPRB ,&
&  0.37880E+00_JPRB, 0.38802E+00_JPRB, 0.38233E+00_JPRB, 0.35890E+00_JPRB, 0.34199E+00_JPRB, 0.33264E+00_JPRB, 0.32059E+00_JPRB ,&
&  0.30642E+00_JPRB, 0.29857E+00_JPRB, 0.29758E+00_JPRB, 0.29542E+00_JPRB, 0.29667E+00_JPRB, 0.29782E+00_JPRB, 0.29894E+00_JPRB ,&
&  0.29959E+00_JPRB, 0.29920E+00_JPRB, 0.29441E+00_JPRB, 0.28823E+00_JPRB, 0.28489E+00_JPRB, 0.28496E+00_JPRB, 0.28799E+00_JPRB ,&
&  0.29441E+00_JPRB, 0.30094E+00_JPRB, 0.31594E+00_JPRB, 0.32378E+00_JPRB, 0.31457E+00_JPRB, 0.30977E+00_JPRB, 0.30431E+00_JPRB ,&
&  0.29317E+00_JPRB, 0.28903E+00_JPRB, 0.28557E+00_JPRB, 0.28367E+00_JPRB, 0.28463E+00_JPRB, 0.28827E+00_JPRB, 0.29006E+00_JPRB ,&
&  0.29285E+00_JPRB, 0.29590E+00_JPRB, 0.29716E+00_JPRB, 0.29814E+00_JPRB, 0.29896E+00_JPRB, 0.30106E+00_JPRB, 0.30321E+00_JPRB ,&
&  0.30491E+00_JPRB, 0.30805E+00_JPRB, 0.31182E+00_JPRB, 0.31680E+00_JPRB, 0.32118E+00_JPRB, 0.32370E+00_JPRB, 0.33159E+00_JPRB ,&
&  0.34060E+00_JPRB, 0.35900E+00_JPRB, 0.38873E+00_JPRB, 0.41284E+00_JPRB, 0.40708E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.41336E+00_JPRB, 0.41756E+00_JPRB, 0.41795E+00_JPRB, 0.42121E+00_JPRB, 0.42642E+00_JPRB, 0.43244E+00_JPRB, 0.43757E+00_JPRB ,&
&  0.44297E+00_JPRB, 0.44270E+00_JPRB, 0.43765E+00_JPRB, 0.43006E+00_JPRB, 0.42378E+00_JPRB, 0.40707E+00_JPRB, 0.39853E+00_JPRB ,&
&  0.39144E+00_JPRB, 0.39143E+00_JPRB, 0.39189E+00_JPRB, 0.39369E+00_JPRB, 0.39467E+00_JPRB, 0.39977E+00_JPRB, 0.40114E+00_JPRB ,&
&  0.39828E+00_JPRB, 0.39287E+00_JPRB, 0.38732E+00_JPRB, 0.38623E+00_JPRB, 0.38884E+00_JPRB, 0.39064E+00_JPRB, 0.38746E+00_JPRB ,&
&  0.38372E+00_JPRB, 0.38352E+00_JPRB, 0.38403E+00_JPRB, 0.38546E+00_JPRB, 0.39496E+00_JPRB, 0.38922E+00_JPRB, 0.38456E+00_JPRB ,&
&  0.37744E+00_JPRB, 0.37141E+00_JPRB, 0.36733E+00_JPRB, 0.36772E+00_JPRB, 0.37114E+00_JPRB, 0.36997E+00_JPRB, 0.37282E+00_JPRB ,&
&  0.36180E+00_JPRB, 0.35254E+00_JPRB, 0.33714E+00_JPRB, 0.32474E+00_JPRB, 0.31157E+00_JPRB, 0.30259E+00_JPRB, 0.29437E+00_JPRB ,&
&  0.28741E+00_JPRB, 0.28148E+00_JPRB, 0.27594E+00_JPRB, 0.27423E+00_JPRB, 0.27518E+00_JPRB, 0.27510E+00_JPRB, 0.27444E+00_JPRB ,&
&  0.27487E+00_JPRB, 0.27580E+00_JPRB, 0.27652E+00_JPRB, 0.27757E+00_JPRB, 0.27935E+00_JPRB, 0.28222E+00_JPRB, 0.28614E+00_JPRB ,&
&  0.29217E+00_JPRB, 0.29692E+00_JPRB, 0.29944E+00_JPRB, 0.30087E+00_JPRB, 0.29768E+00_JPRB, 0.29211E+00_JPRB, 0.28660E+00_JPRB ,&
&  0.28085E+00_JPRB, 0.27820E+00_JPRB, 0.27572E+00_JPRB, 0.27453E+00_JPRB, 0.27427E+00_JPRB, 0.27519E+00_JPRB, 0.27719E+00_JPRB ,&
&  0.27921E+00_JPRB, 0.28186E+00_JPRB, 0.28361E+00_JPRB, 0.28565E+00_JPRB, 0.28806E+00_JPRB, 0.29013E+00_JPRB, 0.29227E+00_JPRB ,&
&  0.29465E+00_JPRB, 0.29653E+00_JPRB, 0.29977E+00_JPRB, 0.30381E+00_JPRB, 0.31159E+00_JPRB, 0.32043E+00_JPRB, 0.33610E+00_JPRB ,&
&  0.35413E+00_JPRB, 0.38578E+00_JPRB, 0.39474E+00_JPRB, 0.40393E+00_JPRB, 0.41190E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.40874E+00_JPRB, 0.41126E+00_JPRB, 0.41507E+00_JPRB, 0.42070E+00_JPRB, 0.42779E+00_JPRB, 0.43441E+00_JPRB, 0.43609E+00_JPRB ,&
&  0.42641E+00_JPRB, 0.41356E+00_JPRB, 0.39516E+00_JPRB, 0.38079E+00_JPRB, 0.36663E+00_JPRB, 0.35619E+00_JPRB, 0.34781E+00_JPRB ,&
&  0.34221E+00_JPRB, 0.34193E+00_JPRB, 0.34217E+00_JPRB, 0.34340E+00_JPRB, 0.34213E+00_JPRB, 0.34305E+00_JPRB, 0.34316E+00_JPRB ,&
&  0.34169E+00_JPRB, 0.34150E+00_JPRB, 0.34141E+00_JPRB, 0.34482E+00_JPRB, 0.35168E+00_JPRB, 0.35791E+00_JPRB, 0.36408E+00_JPRB ,&
&  0.37137E+00_JPRB, 0.37314E+00_JPRB, 0.37720E+00_JPRB, 0.37567E+00_JPRB, 0.37217E+00_JPRB, 0.36769E+00_JPRB, 0.36138E+00_JPRB ,&
&  0.35720E+00_JPRB, 0.35342E+00_JPRB, 0.35237E+00_JPRB, 0.35280E+00_JPRB, 0.35289E+00_JPRB, 0.35601E+00_JPRB, 0.35058E+00_JPRB ,&
&  0.34342E+00_JPRB, 0.33258E+00_JPRB, 0.31961E+00_JPRB, 0.30990E+00_JPRB, 0.30034E+00_JPRB, 0.29350E+00_JPRB, 0.28742E+00_JPRB ,&
&  0.28182E+00_JPRB, 0.27808E+00_JPRB, 0.27416E+00_JPRB, 0.27127E+00_JPRB, 0.27033E+00_JPRB, 0.26830E+00_JPRB, 0.26712E+00_JPRB ,&
&  0.26662E+00_JPRB, 0.26704E+00_JPRB, 0.26889E+00_JPRB, 0.27054E+00_JPRB, 0.27283E+00_JPRB, 0.27536E+00_JPRB, 0.27749E+00_JPRB ,&
&  0.28034E+00_JPRB, 0.28266E+00_JPRB, 0.28295E+00_JPRB, 0.28236E+00_JPRB, 0.28013E+00_JPRB, 0.27715E+00_JPRB, 0.27461E+00_JPRB ,&
&  0.27193E+00_JPRB, 0.27007E+00_JPRB, 0.26942E+00_JPRB, 0.26846E+00_JPRB, 0.26896E+00_JPRB, 0.27021E+00_JPRB, 0.27112E+00_JPRB ,&
&  0.27303E+00_JPRB, 0.27459E+00_JPRB, 0.27645E+00_JPRB, 0.27965E+00_JPRB, 0.28254E+00_JPRB, 0.28508E+00_JPRB, 0.28760E+00_JPRB ,&
&  0.28763E+00_JPRB, 0.28941E+00_JPRB, 0.29191E+00_JPRB, 0.29661E+00_JPRB, 0.31046E+00_JPRB, 0.31972E+00_JPRB, 0.34103E+00_JPRB ,&
&  0.35831E+00_JPRB, 0.39226E+00_JPRB, 0.39094E+00_JPRB, 0.39798E+00_JPRB, 0.40495E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.40311E+00_JPRB, 0.41002E+00_JPRB, 0.41673E+00_JPRB, 0.42200E+00_JPRB, 0.42559E+00_JPRB, 0.42883E+00_JPRB, 0.42671E+00_JPRB ,&
&  0.42066E+00_JPRB, 0.40604E+00_JPRB, 0.39480E+00_JPRB, 0.38348E+00_JPRB, 0.37408E+00_JPRB, 0.36501E+00_JPRB, 0.35660E+00_JPRB ,&
&  0.34840E+00_JPRB, 0.34152E+00_JPRB, 0.33571E+00_JPRB, 0.33058E+00_JPRB, 0.32363E+00_JPRB, 0.31748E+00_JPRB, 0.31254E+00_JPRB ,&
&  0.30844E+00_JPRB, 0.30833E+00_JPRB, 0.31078E+00_JPRB, 0.31532E+00_JPRB, 0.32384E+00_JPRB, 0.33219E+00_JPRB, 0.34149E+00_JPRB ,&
&  0.35563E+00_JPRB, 0.35849E+00_JPRB, 0.35736E+00_JPRB, 0.35672E+00_JPRB, 0.35172E+00_JPRB, 0.34543E+00_JPRB, 0.33637E+00_JPRB ,&
&  0.33825E+00_JPRB, 0.33629E+00_JPRB, 0.34421E+00_JPRB, 0.34429E+00_JPRB, 0.34300E+00_JPRB, 0.34382E+00_JPRB, 0.33656E+00_JPRB ,&
&  0.33161E+00_JPRB, 0.32209E+00_JPRB, 0.31470E+00_JPRB, 0.30665E+00_JPRB, 0.29839E+00_JPRB, 0.29353E+00_JPRB, 0.28713E+00_JPRB ,&
&  0.28249E+00_JPRB, 0.27911E+00_JPRB, 0.27467E+00_JPRB, 0.27194E+00_JPRB, 0.26951E+00_JPRB, 0.26696E+00_JPRB, 0.26562E+00_JPRB ,&
&  0.26460E+00_JPRB, 0.26423E+00_JPRB, 0.26441E+00_JPRB, 0.26463E+00_JPRB, 0.26517E+00_JPRB, 0.26614E+00_JPRB, 0.26687E+00_JPRB ,&
&  0.26769E+00_JPRB, 0.26802E+00_JPRB, 0.26766E+00_JPRB, 0.26726E+00_JPRB, 0.26605E+00_JPRB, 0.26527E+00_JPRB, 0.26420E+00_JPRB ,&
&  0.26352E+00_JPRB, 0.26386E+00_JPRB, 0.26416E+00_JPRB, 0.26489E+00_JPRB, 0.26561E+00_JPRB, 0.26728E+00_JPRB, 0.26882E+00_JPRB ,&
&  0.27046E+00_JPRB, 0.27257E+00_JPRB, 0.27427E+00_JPRB, 0.27648E+00_JPRB, 0.27908E+00_JPRB, 0.28076E+00_JPRB, 0.28382E+00_JPRB ,&
&  0.28677E+00_JPRB, 0.29050E+00_JPRB, 0.29706E+00_JPRB, 0.30421E+00_JPRB, 0.31385E+00_JPRB, 0.32519E+00_JPRB, 0.33931E+00_JPRB ,&
&  0.35719E+00_JPRB, 0.38170E+00_JPRB, 0.38290E+00_JPRB, 0.39177E+00_JPRB, 0.39746E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.39153E+00_JPRB, 0.39680E+00_JPRB, 0.40128E+00_JPRB, 0.40530E+00_JPRB, 0.40893E+00_JPRB, 0.41006E+00_JPRB, 0.40917E+00_JPRB ,&
&  0.39991E+00_JPRB, 0.39321E+00_JPRB, 0.38494E+00_JPRB, 0.37741E+00_JPRB, 0.37042E+00_JPRB, 0.36168E+00_JPRB, 0.35449E+00_JPRB ,&
&  0.34712E+00_JPRB, 0.33970E+00_JPRB, 0.33276E+00_JPRB, 0.32456E+00_JPRB, 0.31606E+00_JPRB, 0.30789E+00_JPRB, 0.30303E+00_JPRB ,&
&  0.30029E+00_JPRB, 0.30026E+00_JPRB, 0.30349E+00_JPRB, 0.30743E+00_JPRB, 0.31250E+00_JPRB, 0.31898E+00_JPRB, 0.32597E+00_JPRB ,&
&  0.33033E+00_JPRB, 0.33266E+00_JPRB, 0.33206E+00_JPRB, 0.33023E+00_JPRB, 0.32800E+00_JPRB, 0.32626E+00_JPRB, 0.32353E+00_JPRB ,&
&  0.32004E+00_JPRB, 0.31791E+00_JPRB, 0.31772E+00_JPRB, 0.31949E+00_JPRB, 0.31979E+00_JPRB, 0.31825E+00_JPRB, 0.31431E+00_JPRB ,&
&  0.30970E+00_JPRB, 0.30499E+00_JPRB, 0.30002E+00_JPRB, 0.29533E+00_JPRB, 0.29148E+00_JPRB, 0.28737E+00_JPRB, 0.28290E+00_JPRB ,&
&  0.28010E+00_JPRB, 0.27717E+00_JPRB, 0.27396E+00_JPRB, 0.27095E+00_JPRB, 0.26843E+00_JPRB, 0.26648E+00_JPRB, 0.26484E+00_JPRB ,&
&  0.26361E+00_JPRB, 0.26267E+00_JPRB, 0.26194E+00_JPRB, 0.26139E+00_JPRB, 0.26087E+00_JPRB, 0.26001E+00_JPRB, 0.25917E+00_JPRB ,&
&  0.25833E+00_JPRB, 0.25803E+00_JPRB, 0.25752E+00_JPRB, 0.25756E+00_JPRB, 0.25785E+00_JPRB, 0.25802E+00_JPRB, 0.25857E+00_JPRB ,&
&  0.25976E+00_JPRB, 0.26101E+00_JPRB, 0.26159E+00_JPRB, 0.26276E+00_JPRB, 0.26388E+00_JPRB, 0.26536E+00_JPRB, 0.26733E+00_JPRB ,&
&  0.26853E+00_JPRB, 0.26987E+00_JPRB, 0.27147E+00_JPRB, 0.27319E+00_JPRB, 0.27613E+00_JPRB, 0.27931E+00_JPRB, 0.28284E+00_JPRB ,&
&  0.28737E+00_JPRB, 0.29172E+00_JPRB, 0.29669E+00_JPRB, 0.30261E+00_JPRB, 0.31343E+00_JPRB, 0.32018E+00_JPRB, 0.32861E+00_JPRB ,&
&  0.33673E+00_JPRB, 0.34005E+00_JPRB, 0.37542E+00_JPRB, 0.37832E+00_JPRB, 0.38411E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.31743E+00_JPRB, 0.32416E+00_JPRB, 0.32839E+00_JPRB, 0.33272E+00_JPRB, 0.32887E+00_JPRB, 0.36803E+00_JPRB, 0.36716E+00_JPRB ,&
&  0.36639E+00_JPRB, 0.35297E+00_JPRB, 0.35130E+00_JPRB, 0.34680E+00_JPRB, 0.34130E+00_JPRB, 0.33516E+00_JPRB, 0.32985E+00_JPRB ,&
&  0.32494E+00_JPRB, 0.31942E+00_JPRB, 0.31497E+00_JPRB, 0.31084E+00_JPRB, 0.30746E+00_JPRB, 0.30447E+00_JPRB, 0.30238E+00_JPRB ,&
&  0.30189E+00_JPRB, 0.30351E+00_JPRB, 0.30687E+00_JPRB, 0.30853E+00_JPRB, 0.30908E+00_JPRB, 0.30953E+00_JPRB, 0.31002E+00_JPRB ,&
&  0.30990E+00_JPRB, 0.30965E+00_JPRB, 0.30825E+00_JPRB, 0.30572E+00_JPRB, 0.30373E+00_JPRB, 0.30234E+00_JPRB, 0.30131E+00_JPRB ,&
&  0.30035E+00_JPRB, 0.29949E+00_JPRB, 0.29856E+00_JPRB, 0.29807E+00_JPRB, 0.29717E+00_JPRB, 0.29579E+00_JPRB, 0.29365E+00_JPRB ,&
&  0.29144E+00_JPRB, 0.28911E+00_JPRB, 0.28627E+00_JPRB, 0.28399E+00_JPRB, 0.28176E+00_JPRB, 0.27951E+00_JPRB, 0.27725E+00_JPRB ,&
&  0.27597E+00_JPRB, 0.27473E+00_JPRB, 0.27315E+00_JPRB, 0.27117E+00_JPRB, 0.26969E+00_JPRB, 0.26812E+00_JPRB, 0.26676E+00_JPRB ,&
&  0.26588E+00_JPRB, 0.26494E+00_JPRB, 0.26399E+00_JPRB, 0.26301E+00_JPRB, 0.26231E+00_JPRB, 0.26148E+00_JPRB, 0.26070E+00_JPRB ,&
&  0.26001E+00_JPRB, 0.25921E+00_JPRB, 0.25901E+00_JPRB, 0.25892E+00_JPRB, 0.25899E+00_JPRB, 0.25903E+00_JPRB, 0.25936E+00_JPRB ,&
&  0.25987E+00_JPRB, 0.26061E+00_JPRB, 0.26154E+00_JPRB, 0.26240E+00_JPRB, 0.26324E+00_JPRB, 0.26410E+00_JPRB, 0.26538E+00_JPRB ,&
&  0.26676E+00_JPRB, 0.26822E+00_JPRB, 0.26972E+00_JPRB, 0.27088E+00_JPRB, 0.27346E+00_JPRB, 0.27588E+00_JPRB, 0.27844E+00_JPRB ,&
&  0.28138E+00_JPRB, 0.28488E+00_JPRB, 0.28867E+00_JPRB, 0.29275E+00_JPRB, 0.29819E+00_JPRB, 0.30141E+00_JPRB, 0.30369E+00_JPRB ,&
&  0.30621E+00_JPRB, 0.31059E+00_JPRB, 0.31189E+00_JPRB, 0.31307E+00_JPRB, 0.31493E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.29039E+00_JPRB, 0.29065E+00_JPRB, 0.29067E+00_JPRB, 0.29050E+00_JPRB, 0.28799E+00_JPRB, 0.28809E+00_JPRB, 0.28797E+00_JPRB ,&
&  0.28808E+00_JPRB, 0.28829E+00_JPRB, 0.28899E+00_JPRB, 0.28981E+00_JPRB, 0.29064E+00_JPRB, 0.29142E+00_JPRB, 0.29214E+00_JPRB ,&
&  0.29259E+00_JPRB, 0.29315E+00_JPRB, 0.29170E+00_JPRB, 0.29081E+00_JPRB, 0.29068E+00_JPRB, 0.29076E+00_JPRB, 0.29072E+00_JPRB ,&
&  0.29061E+00_JPRB, 0.29045E+00_JPRB, 0.28991E+00_JPRB, 0.28954E+00_JPRB, 0.28937E+00_JPRB, 0.28915E+00_JPRB, 0.28880E+00_JPRB ,&
&  0.28839E+00_JPRB, 0.28806E+00_JPRB, 0.28768E+00_JPRB, 0.28722E+00_JPRB, 0.28712E+00_JPRB, 0.28701E+00_JPRB, 0.28644E+00_JPRB ,&
&  0.28577E+00_JPRB, 0.28474E+00_JPRB, 0.28380E+00_JPRB, 0.28294E+00_JPRB, 0.28225E+00_JPRB, 0.28192E+00_JPRB, 0.28156E+00_JPRB ,&
&  0.28106E+00_JPRB, 0.28038E+00_JPRB, 0.27973E+00_JPRB, 0.27891E+00_JPRB, 0.27817E+00_JPRB, 0.27750E+00_JPRB, 0.27672E+00_JPRB ,&
&  0.27586E+00_JPRB, 0.27498E+00_JPRB, 0.27406E+00_JPRB, 0.27304E+00_JPRB, 0.27204E+00_JPRB, 0.27102E+00_JPRB, 0.26991E+00_JPRB ,&
&  0.26881E+00_JPRB, 0.26835E+00_JPRB, 0.26782E+00_JPRB, 0.26737E+00_JPRB, 0.26703E+00_JPRB, 0.26677E+00_JPRB, 0.26651E+00_JPRB ,&
&  0.26630E+00_JPRB, 0.26608E+00_JPRB, 0.26598E+00_JPRB, 0.26588E+00_JPRB, 0.26588E+00_JPRB, 0.26595E+00_JPRB, 0.26614E+00_JPRB ,&
&  0.26637E+00_JPRB, 0.26665E+00_JPRB, 0.26703E+00_JPRB, 0.26767E+00_JPRB, 0.26831E+00_JPRB, 0.26904E+00_JPRB, 0.26977E+00_JPRB ,&
&  0.27057E+00_JPRB, 0.27135E+00_JPRB, 0.27216E+00_JPRB, 0.27292E+00_JPRB, 0.27396E+00_JPRB, 0.27496E+00_JPRB, 0.27606E+00_JPRB ,&
&  0.27715E+00_JPRB, 0.27832E+00_JPRB, 0.27949E+00_JPRB, 0.28073E+00_JPRB, 0.28216E+00_JPRB, 0.28256E+00_JPRB, 0.28298E+00_JPRB ,&
&  0.28350E+00_JPRB, 0.28427E+00_JPRB, 0.28497E+00_JPRB, 0.28592E+00_JPRB, 0.28753E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB, 0.27620E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB, 0.48165E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.48034E+00_JPRB, 0.48119E+00_JPRB, 0.48199E+00_JPRB, 0.48256E+00_JPRB, 0.48292E+00_JPRB, 0.48314E+00_JPRB, 0.48323E+00_JPRB ,&
&  0.48313E+00_JPRB, 0.48330E+00_JPRB, 0.48357E+00_JPRB, 0.48386E+00_JPRB, 0.48412E+00_JPRB, 0.48431E+00_JPRB, 0.48429E+00_JPRB ,&
&  0.48467E+00_JPRB, 0.48615E+00_JPRB, 0.48703E+00_JPRB, 0.48752E+00_JPRB, 0.48793E+00_JPRB, 0.48828E+00_JPRB, 0.48853E+00_JPRB ,&
&  0.48881E+00_JPRB, 0.48914E+00_JPRB, 0.48945E+00_JPRB, 0.48980E+00_JPRB, 0.49029E+00_JPRB, 0.49074E+00_JPRB, 0.49121E+00_JPRB ,&
&  0.49191E+00_JPRB, 0.49309E+00_JPRB, 0.49455E+00_JPRB, 0.49622E+00_JPRB, 0.49831E+00_JPRB, 0.50080E+00_JPRB, 0.50288E+00_JPRB ,&
&  0.50459E+00_JPRB, 0.50567E+00_JPRB, 0.50540E+00_JPRB, 0.50415E+00_JPRB, 0.50207E+00_JPRB, 0.49963E+00_JPRB, 0.49691E+00_JPRB ,&
&  0.49373E+00_JPRB, 0.49089E+00_JPRB, 0.48834E+00_JPRB, 0.48601E+00_JPRB, 0.48378E+00_JPRB, 0.48182E+00_JPRB, 0.47996E+00_JPRB ,&
&  0.47844E+00_JPRB, 0.47692E+00_JPRB, 0.47550E+00_JPRB, 0.47409E+00_JPRB, 0.47263E+00_JPRB, 0.47098E+00_JPRB, 0.46933E+00_JPRB ,&
&  0.46803E+00_JPRB, 0.46698E+00_JPRB, 0.46622E+00_JPRB, 0.46551E+00_JPRB, 0.46505E+00_JPRB, 0.46514E+00_JPRB, 0.46546E+00_JPRB ,&
&  0.46571E+00_JPRB, 0.46614E+00_JPRB, 0.46675E+00_JPRB, 0.46710E+00_JPRB, 0.46737E+00_JPRB, 0.46751E+00_JPRB, 0.46756E+00_JPRB ,&
&  0.46755E+00_JPRB, 0.46782E+00_JPRB, 0.46812E+00_JPRB, 0.46869E+00_JPRB, 0.46921E+00_JPRB, 0.46983E+00_JPRB, 0.47052E+00_JPRB ,&
&  0.47103E+00_JPRB, 0.47167E+00_JPRB, 0.47223E+00_JPRB, 0.47301E+00_JPRB, 0.47370E+00_JPRB, 0.47424E+00_JPRB, 0.47463E+00_JPRB ,&
&  0.47499E+00_JPRB, 0.47520E+00_JPRB, 0.47545E+00_JPRB, 0.47558E+00_JPRB, 0.47588E+00_JPRB, 0.47621E+00_JPRB, 0.47659E+00_JPRB ,&
&  0.47698E+00_JPRB, 0.47752E+00_JPRB, 0.47800E+00_JPRB, 0.47876E+00_JPRB, 0.47951E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.48324E+00_JPRB, 0.48445E+00_JPRB, 0.48520E+00_JPRB, 0.48553E+00_JPRB, 0.48563E+00_JPRB, 0.48516E+00_JPRB, 0.48405E+00_JPRB ,&
&  0.48234E+00_JPRB, 0.48088E+00_JPRB, 0.47980E+00_JPRB, 0.47852E+00_JPRB, 0.47669E+00_JPRB, 0.47657E+00_JPRB, 0.47702E+00_JPRB ,&
&  0.47582E+00_JPRB, 0.47429E+00_JPRB, 0.47264E+00_JPRB, 0.47138E+00_JPRB, 0.46998E+00_JPRB, 0.46840E+00_JPRB, 0.46734E+00_JPRB ,&
&  0.46685E+00_JPRB, 0.46607E+00_JPRB, 0.46482E+00_JPRB, 0.46301E+00_JPRB, 0.46256E+00_JPRB, 0.46207E+00_JPRB, 0.46145E+00_JPRB ,&
&  0.46056E+00_JPRB, 0.45856E+00_JPRB, 0.45682E+00_JPRB, 0.45594E+00_JPRB, 0.45882E+00_JPRB, 0.46237E+00_JPRB, 0.46690E+00_JPRB ,&
&  0.47246E+00_JPRB, 0.47717E+00_JPRB, 0.48269E+00_JPRB, 0.48765E+00_JPRB, 0.49094E+00_JPRB, 0.49550E+00_JPRB, 0.49859E+00_JPRB ,&
&  0.49952E+00_JPRB, 0.49815E+00_JPRB, 0.49693E+00_JPRB, 0.49674E+00_JPRB, 0.49676E+00_JPRB, 0.49635E+00_JPRB, 0.49548E+00_JPRB ,&
&  0.49456E+00_JPRB, 0.49343E+00_JPRB, 0.49190E+00_JPRB, 0.48965E+00_JPRB, 0.48669E+00_JPRB, 0.48362E+00_JPRB, 0.48029E+00_JPRB ,&
&  0.47741E+00_JPRB, 0.47407E+00_JPRB, 0.47089E+00_JPRB, 0.46774E+00_JPRB, 0.46498E+00_JPRB, 0.46185E+00_JPRB, 0.45885E+00_JPRB ,&
&  0.45698E+00_JPRB, 0.45694E+00_JPRB, 0.45755E+00_JPRB, 0.45718E+00_JPRB, 0.45713E+00_JPRB, 0.45711E+00_JPRB, 0.45798E+00_JPRB ,&
&  0.45904E+00_JPRB, 0.46112E+00_JPRB, 0.46339E+00_JPRB, 0.46645E+00_JPRB, 0.46924E+00_JPRB, 0.47188E+00_JPRB, 0.47387E+00_JPRB ,&
&  0.47566E+00_JPRB, 0.47694E+00_JPRB, 0.47782E+00_JPRB, 0.47849E+00_JPRB, 0.47918E+00_JPRB, 0.47961E+00_JPRB, 0.47960E+00_JPRB ,&
&  0.47943E+00_JPRB, 0.47915E+00_JPRB, 0.47872E+00_JPRB, 0.47802E+00_JPRB, 0.47764E+00_JPRB, 0.47687E+00_JPRB, 0.47667E+00_JPRB ,&
&  0.47665E+00_JPRB, 0.47736E+00_JPRB, 0.47828E+00_JPRB, 0.47981E+00_JPRB, 0.48153E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.48008E+00_JPRB, 0.48265E+00_JPRB, 0.48444E+00_JPRB, 0.48516E+00_JPRB, 0.48549E+00_JPRB, 0.48569E+00_JPRB, 0.48536E+00_JPRB ,&
&  0.48483E+00_JPRB, 0.48389E+00_JPRB, 0.48339E+00_JPRB, 0.48250E+00_JPRB, 0.48133E+00_JPRB, 0.48264E+00_JPRB, 0.48199E+00_JPRB ,&
&  0.48017E+00_JPRB, 0.47778E+00_JPRB, 0.47562E+00_JPRB, 0.47288E+00_JPRB, 0.46950E+00_JPRB, 0.46516E+00_JPRB, 0.45790E+00_JPRB ,&
&  0.45340E+00_JPRB, 0.44844E+00_JPRB, 0.44276E+00_JPRB, 0.44124E+00_JPRB, 0.43975E+00_JPRB, 0.42298E+00_JPRB, 0.42297E+00_JPRB ,&
&  0.43813E+00_JPRB, 0.44419E+00_JPRB, 0.44646E+00_JPRB, 0.44929E+00_JPRB, 0.43123E+00_JPRB, 0.41028E+00_JPRB, 0.39692E+00_JPRB ,&
&  0.37619E+00_JPRB, 0.39612E+00_JPRB, 0.40850E+00_JPRB, 0.41337E+00_JPRB, 0.41439E+00_JPRB, 0.41497E+00_JPRB, 0.43517E+00_JPRB ,&
&  0.47860E+00_JPRB, 0.48773E+00_JPRB, 0.49816E+00_JPRB, 0.50960E+00_JPRB, 0.52067E+00_JPRB, 0.52815E+00_JPRB, 0.53611E+00_JPRB ,&
&  0.53983E+00_JPRB, 0.54189E+00_JPRB, 0.53940E+00_JPRB, 0.53479E+00_JPRB, 0.52520E+00_JPRB, 0.51724E+00_JPRB, 0.50996E+00_JPRB ,&
&  0.50536E+00_JPRB, 0.50051E+00_JPRB, 0.49565E+00_JPRB, 0.48964E+00_JPRB, 0.48356E+00_JPRB, 0.47675E+00_JPRB, 0.46985E+00_JPRB ,&
&  0.46267E+00_JPRB, 0.45512E+00_JPRB, 0.44980E+00_JPRB, 0.44572E+00_JPRB, 0.44552E+00_JPRB, 0.44570E+00_JPRB, 0.44782E+00_JPRB ,&
&  0.44987E+00_JPRB, 0.45292E+00_JPRB, 0.45613E+00_JPRB, 0.46082E+00_JPRB, 0.46482E+00_JPRB, 0.46791E+00_JPRB, 0.47021E+00_JPRB ,&
&  0.47205E+00_JPRB, 0.47323E+00_JPRB, 0.47401E+00_JPRB, 0.47428E+00_JPRB, 0.47494E+00_JPRB, 0.47577E+00_JPRB, 0.47613E+00_JPRB ,&
&  0.47681E+00_JPRB, 0.47747E+00_JPRB, 0.47822E+00_JPRB, 0.47834E+00_JPRB, 0.47795E+00_JPRB, 0.47586E+00_JPRB, 0.47372E+00_JPRB ,&
&  0.47168E+00_JPRB, 0.47152E+00_JPRB, 0.47208E+00_JPRB, 0.47418E+00_JPRB, 0.47672E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.49022E+00_JPRB, 0.49177E+00_JPRB, 0.49381E+00_JPRB, 0.49508E+00_JPRB, 0.49540E+00_JPRB, 0.49515E+00_JPRB, 0.49454E+00_JPRB ,&
&  0.49388E+00_JPRB, 0.49357E+00_JPRB, 0.49267E+00_JPRB, 0.49088E+00_JPRB, 0.48979E+00_JPRB, 0.48870E+00_JPRB, 0.48711E+00_JPRB ,&
&  0.48389E+00_JPRB, 0.48040E+00_JPRB, 0.47743E+00_JPRB, 0.47367E+00_JPRB, 0.47013E+00_JPRB, 0.46715E+00_JPRB, 0.46155E+00_JPRB ,&
&  0.45770E+00_JPRB, 0.45067E+00_JPRB, 0.42517E+00_JPRB, 0.42613E+00_JPRB, 0.42107E+00_JPRB, 0.42024E+00_JPRB, 0.42319E+00_JPRB ,&
&  0.41660E+00_JPRB, 0.41579E+00_JPRB, 0.40702E+00_JPRB, 0.39380E+00_JPRB, 0.38669E+00_JPRB, 0.37484E+00_JPRB, 0.37491E+00_JPRB ,&
&  0.37276E+00_JPRB, 0.37681E+00_JPRB, 0.38619E+00_JPRB, 0.39047E+00_JPRB, 0.39103E+00_JPRB, 0.39209E+00_JPRB, 0.40160E+00_JPRB ,&
&  0.42776E+00_JPRB, 0.46706E+00_JPRB, 0.51880E+00_JPRB, 0.54328E+00_JPRB, 0.56437E+00_JPRB, 0.58463E+00_JPRB, 0.60315E+00_JPRB ,&
&  0.61605E+00_JPRB, 0.62644E+00_JPRB, 0.62539E+00_JPRB, 0.62208E+00_JPRB, 0.60926E+00_JPRB, 0.59133E+00_JPRB, 0.57011E+00_JPRB ,&
&  0.54996E+00_JPRB, 0.53528E+00_JPRB, 0.52412E+00_JPRB, 0.51740E+00_JPRB, 0.51339E+00_JPRB, 0.50989E+00_JPRB, 0.50779E+00_JPRB ,&
&  0.50140E+00_JPRB, 0.48788E+00_JPRB, 0.47582E+00_JPRB, 0.46484E+00_JPRB, 0.46099E+00_JPRB, 0.45941E+00_JPRB, 0.46195E+00_JPRB ,&
&  0.46510E+00_JPRB, 0.47126E+00_JPRB, 0.47879E+00_JPRB, 0.48178E+00_JPRB, 0.48638E+00_JPRB, 0.49206E+00_JPRB, 0.49202E+00_JPRB ,&
&  0.49184E+00_JPRB, 0.49096E+00_JPRB, 0.48885E+00_JPRB, 0.48602E+00_JPRB, 0.48269E+00_JPRB, 0.47945E+00_JPRB, 0.47689E+00_JPRB ,&
&  0.47602E+00_JPRB, 0.47687E+00_JPRB, 0.48066E+00_JPRB, 0.48503E+00_JPRB, 0.49038E+00_JPRB, 0.49302E+00_JPRB, 0.49212E+00_JPRB ,&
&  0.48767E+00_JPRB, 0.48554E+00_JPRB, 0.48401E+00_JPRB, 0.48605E+00_JPRB, 0.48811E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.50251E+00_JPRB, 0.50678E+00_JPRB, 0.51389E+00_JPRB, 0.52018E+00_JPRB, 0.52512E+00_JPRB, 0.52556E+00_JPRB, 0.51953E+00_JPRB ,&
&  0.51363E+00_JPRB, 0.50783E+00_JPRB, 0.50301E+00_JPRB, 0.49905E+00_JPRB, 0.49644E+00_JPRB, 0.49255E+00_JPRB, 0.48834E+00_JPRB ,&
&  0.48470E+00_JPRB, 0.48231E+00_JPRB, 0.48104E+00_JPRB, 0.48013E+00_JPRB, 0.48064E+00_JPRB, 0.47975E+00_JPRB, 0.47890E+00_JPRB ,&
&  0.47826E+00_JPRB, 0.46897E+00_JPRB, 0.44843E+00_JPRB, 0.43961E+00_JPRB, 0.43122E+00_JPRB, 0.43096E+00_JPRB, 0.43115E+00_JPRB ,&
&  0.42836E+00_JPRB, 0.38541E+00_JPRB, 0.39116E+00_JPRB, 0.38075E+00_JPRB, 0.37423E+00_JPRB, 0.37412E+00_JPRB, 0.36510E+00_JPRB ,&
&  0.35920E+00_JPRB, 0.35226E+00_JPRB, 0.35317E+00_JPRB, 0.35938E+00_JPRB, 0.36038E+00_JPRB, 0.36937E+00_JPRB, 0.38190E+00_JPRB ,&
&  0.41044E+00_JPRB, 0.51137E+00_JPRB, 0.55454E+00_JPRB, 0.58068E+00_JPRB, 0.60193E+00_JPRB, 0.62825E+00_JPRB, 0.65418E+00_JPRB ,&
&  0.67288E+00_JPRB, 0.69663E+00_JPRB, 0.70451E+00_JPRB, 0.69760E+00_JPRB, 0.69907E+00_JPRB, 0.68794E+00_JPRB, 0.66401E+00_JPRB ,&
&  0.63198E+00_JPRB, 0.59291E+00_JPRB, 0.56709E+00_JPRB, 0.55113E+00_JPRB, 0.53858E+00_JPRB, 0.52755E+00_JPRB, 0.51992E+00_JPRB ,&
&  0.52197E+00_JPRB, 0.51550E+00_JPRB, 0.50785E+00_JPRB, 0.49461E+00_JPRB, 0.48735E+00_JPRB, 0.48344E+00_JPRB, 0.48385E+00_JPRB ,&
&  0.48658E+00_JPRB, 0.49288E+00_JPRB, 0.50159E+00_JPRB, 0.50899E+00_JPRB, 0.51283E+00_JPRB, 0.51253E+00_JPRB, 0.51584E+00_JPRB ,&
&  0.51618E+00_JPRB, 0.51426E+00_JPRB, 0.51163E+00_JPRB, 0.50841E+00_JPRB, 0.50491E+00_JPRB, 0.50253E+00_JPRB, 0.50354E+00_JPRB ,&
&  0.50544E+00_JPRB, 0.50660E+00_JPRB, 0.50725E+00_JPRB, 0.50932E+00_JPRB, 0.51338E+00_JPRB, 0.51698E+00_JPRB, 0.51823E+00_JPRB ,&
&  0.51214E+00_JPRB, 0.50597E+00_JPRB, 0.50049E+00_JPRB, 0.49986E+00_JPRB, 0.50013E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.49404E+00_JPRB, 0.50655E+00_JPRB, 0.52581E+00_JPRB, 0.54127E+00_JPRB, 0.55292E+00_JPRB, 0.55178E+00_JPRB, 0.54225E+00_JPRB ,&
&  0.53240E+00_JPRB, 0.52463E+00_JPRB, 0.51743E+00_JPRB, 0.51048E+00_JPRB, 0.50224E+00_JPRB, 0.49578E+00_JPRB, 0.49064E+00_JPRB ,&
&  0.48770E+00_JPRB, 0.48647E+00_JPRB, 0.48575E+00_JPRB, 0.48500E+00_JPRB, 0.48507E+00_JPRB, 0.48629E+00_JPRB, 0.48657E+00_JPRB ,&
&  0.48730E+00_JPRB, 0.48073E+00_JPRB, 0.46365E+00_JPRB, 0.44812E+00_JPRB, 0.44128E+00_JPRB, 0.43252E+00_JPRB, 0.43501E+00_JPRB ,&
&  0.43439E+00_JPRB, 0.43321E+00_JPRB, 0.42777E+00_JPRB, 0.42563E+00_JPRB, 0.41175E+00_JPRB, 0.35747E+00_JPRB, 0.36041E+00_JPRB ,&
&  0.35016E+00_JPRB, 0.33836E+00_JPRB, 0.33668E+00_JPRB, 0.33730E+00_JPRB, 0.33981E+00_JPRB, 0.35815E+00_JPRB, 0.37973E+00_JPRB ,&
&  0.40834E+00_JPRB, 0.52193E+00_JPRB, 0.55242E+00_JPRB, 0.58514E+00_JPRB, 0.61439E+00_JPRB, 0.65190E+00_JPRB, 0.69265E+00_JPRB ,&
&  0.71671E+00_JPRB, 0.74274E+00_JPRB, 0.75976E+00_JPRB, 0.77024E+00_JPRB, 0.76869E+00_JPRB, 0.76524E+00_JPRB, 0.76052E+00_JPRB ,&
&  0.74362E+00_JPRB, 0.67789E+00_JPRB, 0.63327E+00_JPRB, 0.60720E+00_JPRB, 0.58390E+00_JPRB, 0.56716E+00_JPRB, 0.55365E+00_JPRB ,&
&  0.54766E+00_JPRB, 0.52965E+00_JPRB, 0.52477E+00_JPRB, 0.51601E+00_JPRB, 0.51128E+00_JPRB, 0.50785E+00_JPRB, 0.50357E+00_JPRB ,&
&  0.49724E+00_JPRB, 0.49833E+00_JPRB, 0.50821E+00_JPRB, 0.51317E+00_JPRB, 0.51606E+00_JPRB, 0.51297E+00_JPRB, 0.50781E+00_JPRB ,&
&  0.49905E+00_JPRB, 0.49904E+00_JPRB, 0.49533E+00_JPRB, 0.49409E+00_JPRB, 0.49445E+00_JPRB, 0.50107E+00_JPRB, 0.50975E+00_JPRB ,&
&  0.51619E+00_JPRB, 0.51835E+00_JPRB, 0.51796E+00_JPRB, 0.51828E+00_JPRB, 0.51723E+00_JPRB, 0.51507E+00_JPRB, 0.51044E+00_JPRB ,&
&  0.50604E+00_JPRB, 0.50313E+00_JPRB, 0.49732E+00_JPRB, 0.49227E+00_JPRB, 0.49010E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.48886E+00_JPRB, 0.51292E+00_JPRB, 0.54091E+00_JPRB, 0.56040E+00_JPRB, 0.56995E+00_JPRB, 0.55836E+00_JPRB, 0.54336E+00_JPRB ,&
&  0.53114E+00_JPRB, 0.52190E+00_JPRB, 0.51531E+00_JPRB, 0.51042E+00_JPRB, 0.50442E+00_JPRB, 0.49863E+00_JPRB, 0.49336E+00_JPRB ,&
&  0.48941E+00_JPRB, 0.48021E+00_JPRB, 0.47616E+00_JPRB, 0.47709E+00_JPRB, 0.47801E+00_JPRB, 0.48144E+00_JPRB, 0.48478E+00_JPRB ,&
&  0.48349E+00_JPRB, 0.47625E+00_JPRB, 0.46950E+00_JPRB, 0.45526E+00_JPRB, 0.44069E+00_JPRB, 0.43444E+00_JPRB, 0.43689E+00_JPRB ,&
&  0.42795E+00_JPRB, 0.41152E+00_JPRB, 0.42614E+00_JPRB, 0.43790E+00_JPRB, 0.41940E+00_JPRB, 0.40560E+00_JPRB, 0.36872E+00_JPRB ,&
&  0.35537E+00_JPRB, 0.33657E+00_JPRB, 0.33219E+00_JPRB, 0.33273E+00_JPRB, 0.34107E+00_JPRB, 0.35817E+00_JPRB, 0.39502E+00_JPRB ,&
&  0.44802E+00_JPRB, 0.52950E+00_JPRB, 0.55256E+00_JPRB, 0.59318E+00_JPRB, 0.62995E+00_JPRB, 0.66256E+00_JPRB, 0.68614E+00_JPRB ,&
&  0.72268E+00_JPRB, 0.77178E+00_JPRB, 0.79911E+00_JPRB, 0.80029E+00_JPRB, 0.80560E+00_JPRB, 0.81842E+00_JPRB, 0.84800E+00_JPRB ,&
&  0.89420E+00_JPRB, 0.78010E+00_JPRB, 0.70838E+00_JPRB, 0.66036E+00_JPRB, 0.62967E+00_JPRB, 0.61675E+00_JPRB, 0.57706E+00_JPRB ,&
&  0.56181E+00_JPRB, 0.55676E+00_JPRB, 0.54723E+00_JPRB, 0.54184E+00_JPRB, 0.53788E+00_JPRB, 0.53457E+00_JPRB, 0.52764E+00_JPRB ,&
&  0.51835E+00_JPRB, 0.52122E+00_JPRB, 0.56747E+00_JPRB, 0.57712E+00_JPRB, 0.56548E+00_JPRB, 0.53333E+00_JPRB, 0.50087E+00_JPRB ,&
&  0.47934E+00_JPRB, 0.47164E+00_JPRB, 0.46913E+00_JPRB, 0.46760E+00_JPRB, 0.47159E+00_JPRB, 0.47882E+00_JPRB, 0.49120E+00_JPRB ,&
&  0.49330E+00_JPRB, 0.50181E+00_JPRB, 0.50983E+00_JPRB, 0.51359E+00_JPRB, 0.51228E+00_JPRB, 0.50801E+00_JPRB, 0.50469E+00_JPRB ,&
&  0.50163E+00_JPRB, 0.49829E+00_JPRB, 0.49332E+00_JPRB, 0.48675E+00_JPRB, 0.48243E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.47620E+00_JPRB, 0.51031E+00_JPRB, 0.55472E+00_JPRB, 0.58494E+00_JPRB, 0.58628E+00_JPRB, 0.56010E+00_JPRB, 0.53489E+00_JPRB ,&
&  0.51943E+00_JPRB, 0.51235E+00_JPRB, 0.50970E+00_JPRB, 0.50521E+00_JPRB, 0.50040E+00_JPRB, 0.49527E+00_JPRB, 0.49018E+00_JPRB ,&
&  0.48590E+00_JPRB, 0.48197E+00_JPRB, 0.47654E+00_JPRB, 0.46924E+00_JPRB, 0.46470E+00_JPRB, 0.46448E+00_JPRB, 0.46672E+00_JPRB ,&
&  0.47021E+00_JPRB, 0.46852E+00_JPRB, 0.46625E+00_JPRB, 0.45832E+00_JPRB, 0.44738E+00_JPRB, 0.43530E+00_JPRB, 0.42598E+00_JPRB ,&
&  0.42153E+00_JPRB, 0.42274E+00_JPRB, 0.42078E+00_JPRB, 0.43697E+00_JPRB, 0.43272E+00_JPRB, 0.41957E+00_JPRB, 0.39235E+00_JPRB ,&
&  0.37047E+00_JPRB, 0.34620E+00_JPRB, 0.33715E+00_JPRB, 0.34109E+00_JPRB, 0.36381E+00_JPRB, 0.39638E+00_JPRB, 0.45079E+00_JPRB ,&
&  0.54795E+00_JPRB, 0.60045E+00_JPRB, 0.64070E+00_JPRB, 0.67785E+00_JPRB, 0.70706E+00_JPRB, 0.73806E+00_JPRB, 0.77415E+00_JPRB ,&
&  0.81961E+00_JPRB, 0.85721E+00_JPRB, 0.87475E+00_JPRB, 0.84693E+00_JPRB, 0.83131E+00_JPRB, 0.76849E+00_JPRB, 0.84313E+00_JPRB ,&
&  0.11010E+01_JPRB, 0.92869E+00_JPRB, 0.81117E+00_JPRB, 0.74973E+00_JPRB, 0.70774E+00_JPRB, 0.67218E+00_JPRB, 0.63956E+00_JPRB ,&
&  0.60819E+00_JPRB, 0.58980E+00_JPRB, 0.58018E+00_JPRB, 0.55320E+00_JPRB, 0.52728E+00_JPRB, 0.52596E+00_JPRB, 0.55550E+00_JPRB ,&
&  0.63024E+00_JPRB, 0.74669E+00_JPRB, 0.90809E+00_JPRB, 0.64116E+00_JPRB, 0.55729E+00_JPRB, 0.49979E+00_JPRB, 0.47502E+00_JPRB ,&
&  0.46887E+00_JPRB, 0.46865E+00_JPRB, 0.47003E+00_JPRB, 0.46665E+00_JPRB, 0.46295E+00_JPRB, 0.46402E+00_JPRB, 0.46545E+00_JPRB ,&
&  0.46563E+00_JPRB, 0.47386E+00_JPRB, 0.48587E+00_JPRB, 0.49552E+00_JPRB, 0.50226E+00_JPRB, 0.50442E+00_JPRB, 0.50631E+00_JPRB ,&
&  0.50527E+00_JPRB, 0.49963E+00_JPRB, 0.49225E+00_JPRB, 0.47842E+00_JPRB, 0.46922E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.47081E+00_JPRB, 0.51116E+00_JPRB, 0.56962E+00_JPRB, 0.60346E+00_JPRB, 0.60300E+00_JPRB, 0.56204E+00_JPRB, 0.51716E+00_JPRB ,&
&  0.49049E+00_JPRB, 0.47412E+00_JPRB, 0.47075E+00_JPRB, 0.46824E+00_JPRB, 0.46699E+00_JPRB, 0.46622E+00_JPRB, 0.46495E+00_JPRB ,&
&  0.46237E+00_JPRB, 0.45926E+00_JPRB, 0.45569E+00_JPRB, 0.45278E+00_JPRB, 0.44918E+00_JPRB, 0.44620E+00_JPRB, 0.44562E+00_JPRB ,&
&  0.44716E+00_JPRB, 0.44700E+00_JPRB, 0.44573E+00_JPRB, 0.44376E+00_JPRB, 0.44420E+00_JPRB, 0.44729E+00_JPRB, 0.44577E+00_JPRB ,&
&  0.43362E+00_JPRB, 0.42364E+00_JPRB, 0.42993E+00_JPRB, 0.44529E+00_JPRB, 0.44271E+00_JPRB, 0.43410E+00_JPRB, 0.40477E+00_JPRB ,&
&  0.37776E+00_JPRB, 0.35936E+00_JPRB, 0.37075E+00_JPRB, 0.41013E+00_JPRB, 0.47205E+00_JPRB, 0.59571E+00_JPRB, 0.67374E+00_JPRB ,&
&  0.74277E+00_JPRB, 0.78408E+00_JPRB, 0.80948E+00_JPRB, 0.83519E+00_JPRB, 0.88449E+00_JPRB, 0.93467E+00_JPRB, 0.96993E+00_JPRB ,&
&  0.99282E+00_JPRB, 0.10006E+01_JPRB, 0.99905E+00_JPRB, 0.76748E+00_JPRB, 0.73502E+00_JPRB, 0.79031E+00_JPRB, 0.86075E+00_JPRB ,&
&  0.90316E+00_JPRB, 0.84939E+00_JPRB, 0.82270E+00_JPRB, 0.83447E+00_JPRB, 0.78242E+00_JPRB, 0.74316E+00_JPRB, 0.71083E+00_JPRB ,&
&  0.67941E+00_JPRB, 0.66504E+00_JPRB, 0.62051E+00_JPRB, 0.57544E+00_JPRB, 0.57372E+00_JPRB, 0.57525E+00_JPRB, 0.58168E+00_JPRB ,&
&  0.60585E+00_JPRB, 0.62238E+00_JPRB, 0.57747E+00_JPRB, 0.47028E+00_JPRB, 0.44304E+00_JPRB, 0.43305E+00_JPRB, 0.43445E+00_JPRB ,&
&  0.43892E+00_JPRB, 0.44353E+00_JPRB, 0.44960E+00_JPRB, 0.45038E+00_JPRB, 0.44810E+00_JPRB, 0.44617E+00_JPRB, 0.44318E+00_JPRB ,&
&  0.43847E+00_JPRB, 0.43969E+00_JPRB, 0.44741E+00_JPRB, 0.45263E+00_JPRB, 0.46673E+00_JPRB, 0.48000E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.48959E+00_JPRB, 0.48523E+00_JPRB, 0.47933E+00_JPRB, 0.46566E+00_JPRB, 0.45642E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.51134E+00_JPRB, 0.55114E+00_JPRB, 0.58551E+00_JPRB, 0.60923E+00_JPRB, 0.60211E+00_JPRB, 0.57523E+00_JPRB, 0.51812E+00_JPRB ,&
&  0.50295E+00_JPRB, 0.47194E+00_JPRB, 0.46243E+00_JPRB, 0.44965E+00_JPRB, 0.43909E+00_JPRB, 0.43059E+00_JPRB, 0.42735E+00_JPRB ,&
&  0.42983E+00_JPRB, 0.43349E+00_JPRB, 0.43570E+00_JPRB, 0.43675E+00_JPRB, 0.43469E+00_JPRB, 0.43316E+00_JPRB, 0.43255E+00_JPRB ,&
&  0.43225E+00_JPRB, 0.43235E+00_JPRB, 0.43390E+00_JPRB, 0.43616E+00_JPRB, 0.43979E+00_JPRB, 0.44301E+00_JPRB, 0.44091E+00_JPRB ,&
&  0.43282E+00_JPRB, 0.42605E+00_JPRB, 0.43718E+00_JPRB, 0.45385E+00_JPRB, 0.45147E+00_JPRB, 0.44926E+00_JPRB, 0.41695E+00_JPRB ,&
&  0.38804E+00_JPRB, 0.39999E+00_JPRB, 0.47727E+00_JPRB, 0.63252E+00_JPRB, 0.69255E+00_JPRB, 0.75321E+00_JPRB, 0.79672E+00_JPRB ,&
&  0.75772E+00_JPRB, 0.76994E+00_JPRB, 0.83093E+00_JPRB, 0.10065E+01_JPRB, 0.10375E+01_JPRB, 0.10766E+01_JPRB, 0.11169E+01_JPRB ,&
&  0.11718E+01_JPRB, 0.11581E+01_JPRB, 0.11198E+01_JPRB, 0.92860E+00_JPRB, 0.10369E+01_JPRB, 0.12337E+01_JPRB, 0.10708E+01_JPRB ,&
&  0.10364E+01_JPRB, 0.10013E+01_JPRB, 0.10184E+01_JPRB, 0.95480E+00_JPRB, 0.86153E+00_JPRB, 0.80711E+00_JPRB, 0.78809E+00_JPRB ,&
&  0.75366E+00_JPRB, 0.72854E+00_JPRB, 0.65488E+00_JPRB, 0.63281E+00_JPRB, 0.63950E+00_JPRB, 0.61706E+00_JPRB, 0.59651E+00_JPRB ,&
&  0.57639E+00_JPRB, 0.53750E+00_JPRB, 0.50477E+00_JPRB, 0.46976E+00_JPRB, 0.44736E+00_JPRB, 0.44210E+00_JPRB, 0.44361E+00_JPRB ,&
&  0.44528E+00_JPRB, 0.44493E+00_JPRB, 0.44774E+00_JPRB, 0.45053E+00_JPRB, 0.44731E+00_JPRB, 0.44190E+00_JPRB, 0.44237E+00_JPRB ,&
&  0.43961E+00_JPRB, 0.43251E+00_JPRB, 0.42745E+00_JPRB, 0.43430E+00_JPRB, 0.44383E+00_JPRB, 0.45806E+00_JPRB, 0.47290E+00_JPRB ,&
&  0.48113E+00_JPRB, 0.48834E+00_JPRB, 0.48536E+00_JPRB, 0.48724E+00_JPRB, 0.48741E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.54694E+00_JPRB, 0.56787E+00_JPRB, 0.59361E+00_JPRB, 0.61294E+00_JPRB, 0.62621E+00_JPRB, 0.57335E+00_JPRB, 0.52533E+00_JPRB ,&
&  0.54857E+00_JPRB, 0.48556E+00_JPRB, 0.45853E+00_JPRB, 0.44494E+00_JPRB, 0.43995E+00_JPRB, 0.42736E+00_JPRB, 0.41654E+00_JPRB ,&
&  0.41257E+00_JPRB, 0.41608E+00_JPRB, 0.42335E+00_JPRB, 0.42686E+00_JPRB, 0.42739E+00_JPRB, 0.42728E+00_JPRB, 0.42656E+00_JPRB ,&
&  0.42678E+00_JPRB, 0.42828E+00_JPRB, 0.43032E+00_JPRB, 0.43282E+00_JPRB, 0.43379E+00_JPRB, 0.43538E+00_JPRB, 0.43458E+00_JPRB ,&
&  0.42934E+00_JPRB, 0.42889E+00_JPRB, 0.44148E+00_JPRB, 0.46233E+00_JPRB, 0.47025E+00_JPRB, 0.47838E+00_JPRB, 0.47600E+00_JPRB ,&
&  0.42056E+00_JPRB, 0.47524E+00_JPRB, 0.67582E+00_JPRB, 0.74280E+00_JPRB, 0.79083E+00_JPRB, 0.85534E+00_JPRB, 0.91182E+00_JPRB ,&
&  0.95525E+00_JPRB, 0.99172E+00_JPRB, 0.10222E+01_JPRB, 0.10633E+01_JPRB, 0.11173E+01_JPRB, 0.11632E+01_JPRB, 0.12212E+01_JPRB ,&
&  0.13663E+01_JPRB, 0.10565E+01_JPRB, 0.96315E+00_JPRB, 0.10121E+01_JPRB, 0.14196E+01_JPRB, 0.14537E+01_JPRB, 0.12542E+01_JPRB ,&
&  0.11936E+01_JPRB, 0.11284E+01_JPRB, 0.10788E+01_JPRB, 0.99583E+00_JPRB, 0.92749E+00_JPRB, 0.89861E+00_JPRB, 0.88158E+00_JPRB ,&
&  0.82462E+00_JPRB, 0.79743E+00_JPRB, 0.71415E+00_JPRB, 0.69896E+00_JPRB, 0.69817E+00_JPRB, 0.66042E+00_JPRB, 0.60692E+00_JPRB ,&
&  0.56491E+00_JPRB, 0.53137E+00_JPRB, 0.50678E+00_JPRB, 0.48992E+00_JPRB, 0.47745E+00_JPRB, 0.46991E+00_JPRB, 0.46457E+00_JPRB ,&
&  0.46174E+00_JPRB, 0.45918E+00_JPRB, 0.44954E+00_JPRB, 0.44923E+00_JPRB, 0.45055E+00_JPRB, 0.45245E+00_JPRB, 0.46133E+00_JPRB ,&
&  0.46195E+00_JPRB, 0.45677E+00_JPRB, 0.44678E+00_JPRB, 0.45288E+00_JPRB, 0.46732E+00_JPRB, 0.49172E+00_JPRB, 0.51629E+00_JPRB ,&
&  0.54427E+00_JPRB, 0.54461E+00_JPRB, 0.51947E+00_JPRB, 0.51370E+00_JPRB, 0.51926E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.57173E+00_JPRB, 0.59028E+00_JPRB, 0.60387E+00_JPRB, 0.62158E+00_JPRB, 0.63669E+00_JPRB, 0.59041E+00_JPRB, 0.55972E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.54272E+00_JPRB, 0.59022E+00_JPRB, 0.50161E+00_JPRB, 0.46079E+00_JPRB, 0.44254E+00_JPRB, 0.42858E+00_JPRB ,&
&  0.41572E+00_JPRB, 0.42049E+00_JPRB, 0.42658E+00_JPRB, 0.42882E+00_JPRB, 0.43449E+00_JPRB, 0.43083E+00_JPRB, 0.42766E+00_JPRB ,&
&  0.42571E+00_JPRB, 0.42626E+00_JPRB, 0.43193E+00_JPRB, 0.43203E+00_JPRB, 0.43094E+00_JPRB, 0.42962E+00_JPRB, 0.42447E+00_JPRB ,&
&  0.41364E+00_JPRB, 0.41745E+00_JPRB, 0.44007E+00_JPRB, 0.46149E+00_JPRB, 0.48166E+00_JPRB, 0.49850E+00_JPRB, 0.52719E+00_JPRB ,&
&  0.55444E+00_JPRB, 0.57764E+00_JPRB, 0.71830E+00_JPRB, 0.76073E+00_JPRB, 0.80998E+00_JPRB, 0.84581E+00_JPRB, 0.88934E+00_JPRB ,&
&  0.93373E+00_JPRB, 0.97134E+00_JPRB, 0.99455E+00_JPRB, 0.10389E+01_JPRB, 0.11248E+01_JPRB, 0.11808E+01_JPRB, 0.12730E+01_JPRB ,&
&  0.14548E+01_JPRB, 0.10744E+01_JPRB, 0.12614E+01_JPRB, 0.12440E+01_JPRB, 0.14627E+01_JPRB, 0.16162E+01_JPRB, 0.15756E+01_JPRB ,&
&  0.16745E+01_JPRB, 0.14400E+01_JPRB, 0.12764E+01_JPRB, 0.11190E+01_JPRB, 0.10011E+01_JPRB, 0.96871E+00_JPRB, 0.95157E+00_JPRB ,&
&  0.97231E+00_JPRB, 0.94947E+00_JPRB, 0.81689E+00_JPRB, 0.75946E+00_JPRB, 0.73990E+00_JPRB, 0.72365E+00_JPRB, 0.66524E+00_JPRB ,&
&  0.63246E+00_JPRB, 0.61183E+00_JPRB, 0.58433E+00_JPRB, 0.55605E+00_JPRB, 0.53427E+00_JPRB, 0.52149E+00_JPRB, 0.50357E+00_JPRB ,&
&  0.49903E+00_JPRB, 0.48658E+00_JPRB, 0.45408E+00_JPRB, 0.43981E+00_JPRB, 0.43600E+00_JPRB, 0.43053E+00_JPRB, 0.44205E+00_JPRB ,&
&  0.45635E+00_JPRB, 0.45949E+00_JPRB, 0.48979E+00_JPRB, 0.53099E+00_JPRB, 0.54441E+00_JPRB, 0.55423E+00_JPRB, 0.61388E+00_JPRB ,&
&  0.57686E+00_JPRB, 0.58672E+00_JPRB, 0.58874E+00_JPRB, 0.56477E+00_JPRB, 0.55831E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.60194E+00_JPRB, 0.60823E+00_JPRB, 0.61425E+00_JPRB, 0.63150E+00_JPRB, 0.64444E+00_JPRB, 0.64697E+00_JPRB, 0.61116E+00_JPRB ,&
&  0.65281E+00_JPRB, 0.63049E+00_JPRB, 0.61205E+00_JPRB, 0.60742E+00_JPRB, 0.59592E+00_JPRB, 0.51915E+00_JPRB, 0.44034E+00_JPRB ,&
&  0.41787E+00_JPRB, 0.41357E+00_JPRB, 0.42855E+00_JPRB, 0.43020E+00_JPRB, 0.43453E+00_JPRB, 0.43458E+00_JPRB, 0.42983E+00_JPRB ,&
&  0.42887E+00_JPRB, 0.42908E+00_JPRB, 0.43604E+00_JPRB, 0.43721E+00_JPRB, 0.43215E+00_JPRB, 0.43028E+00_JPRB, 0.42766E+00_JPRB ,&
&  0.41162E+00_JPRB, 0.40789E+00_JPRB, 0.42176E+00_JPRB, 0.42761E+00_JPRB, 0.48141E+00_JPRB, 0.50884E+00_JPRB, 0.54425E+00_JPRB ,&
&  0.58100E+00_JPRB, 0.62263E+00_JPRB, 0.69245E+00_JPRB, 0.74748E+00_JPRB, 0.78488E+00_JPRB, 0.81370E+00_JPRB, 0.85623E+00_JPRB ,&
&  0.89033E+00_JPRB, 0.93625E+00_JPRB, 0.98889E+00_JPRB, 0.10740E+01_JPRB, 0.12001E+01_JPRB, 0.10952E+01_JPRB, 0.12916E+01_JPRB ,&
&  0.13912E+01_JPRB, 0.15497E+01_JPRB, 0.16865E+01_JPRB, 0.15606E+01_JPRB, 0.17282E+01_JPRB, 0.18170E+01_JPRB, 0.17375E+01_JPRB ,&
&  0.16495E+01_JPRB, 0.14411E+01_JPRB, 0.13893E+01_JPRB, 0.13454E+01_JPRB, 0.13286E+01_JPRB, 0.12283E+01_JPRB, 0.11353E+01_JPRB ,&
&  0.11929E+01_JPRB, 0.13670E+01_JPRB, 0.10731E+01_JPRB, 0.98047E+00_JPRB, 0.90145E+00_JPRB, 0.87462E+00_JPRB, 0.82492E+00_JPRB ,&
&  0.80230E+00_JPRB, 0.80133E+00_JPRB, 0.77242E+00_JPRB, 0.73235E+00_JPRB, 0.65402E+00_JPRB, 0.61800E+00_JPRB, 0.57513E+00_JPRB ,&
&  0.55153E+00_JPRB, 0.51667E+00_JPRB, 0.46772E+00_JPRB, 0.44023E+00_JPRB, 0.42293E+00_JPRB, 0.41693E+00_JPRB, 0.43375E+00_JPRB ,&
&  0.47666E+00_JPRB, 0.52134E+00_JPRB, 0.61807E+00_JPRB, 0.64577E+00_JPRB, 0.66355E+00_JPRB, 0.69720E+00_JPRB, 0.61799E+00_JPRB ,&
&  0.57361E+00_JPRB, 0.64093E+00_JPRB, 0.62206E+00_JPRB, 0.60362E+00_JPRB, 0.59747E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.65295E+00_JPRB, 0.65190E+00_JPRB, 0.64965E+00_JPRB, 0.65555E+00_JPRB, 0.66413E+00_JPRB, 0.65690E+00_JPRB, 0.65633E+00_JPRB ,&
&  0.66388E+00_JPRB, 0.65806E+00_JPRB, 0.63874E+00_JPRB, 0.62610E+00_JPRB, 0.60982E+00_JPRB, 0.60581E+00_JPRB, 0.53181E+00_JPRB ,&
&  0.45667E+00_JPRB, 0.43581E+00_JPRB, 0.42098E+00_JPRB, 0.42426E+00_JPRB, 0.44090E+00_JPRB, 0.43991E+00_JPRB, 0.43803E+00_JPRB ,&
&  0.43785E+00_JPRB, 0.43790E+00_JPRB, 0.44009E+00_JPRB, 0.43989E+00_JPRB, 0.44130E+00_JPRB, 0.45124E+00_JPRB, 0.45410E+00_JPRB ,&
&  0.42847E+00_JPRB, 0.40266E+00_JPRB, 0.39903E+00_JPRB, 0.41408E+00_JPRB, 0.46341E+00_JPRB, 0.51497E+00_JPRB, 0.55488E+00_JPRB ,&
&  0.59151E+00_JPRB, 0.63083E+00_JPRB, 0.67797E+00_JPRB, 0.73479E+00_JPRB, 0.77092E+00_JPRB, 0.80604E+00_JPRB, 0.84487E+00_JPRB ,&
&  0.88877E+00_JPRB, 0.95087E+00_JPRB, 0.10320E+01_JPRB, 0.11523E+01_JPRB, 0.11512E+01_JPRB, 0.11792E+01_JPRB, 0.13180E+01_JPRB ,&
&  0.14235E+01_JPRB, 0.15477E+01_JPRB, 0.16544E+01_JPRB, 0.19096E+01_JPRB, 0.20645E+01_JPRB, 0.19835E+01_JPRB, 0.18340E+01_JPRB ,&
&  0.17205E+01_JPRB, 0.15208E+01_JPRB, 0.17157E+01_JPRB, 0.15647E+01_JPRB, 0.14481E+01_JPRB, 0.14376E+01_JPRB, 0.13627E+01_JPRB ,&
&  0.13444E+01_JPRB, 0.14041E+01_JPRB, 0.13559E+01_JPRB, 0.13364E+01_JPRB, 0.12382E+01_JPRB, 0.12169E+01_JPRB, 0.11315E+01_JPRB ,&
&  0.11348E+01_JPRB, 0.11248E+01_JPRB, 0.96818E+00_JPRB, 0.83200E+00_JPRB, 0.70685E+00_JPRB, 0.67908E+00_JPRB, 0.62416E+00_JPRB ,&
&  0.58259E+00_JPRB, 0.53276E+00_JPRB, 0.49530E+00_JPRB, 0.45529E+00_JPRB, 0.44585E+00_JPRB, 0.45610E+00_JPRB, 0.47304E+00_JPRB ,&
&  0.52895E+00_JPRB, 0.63530E+00_JPRB, 0.68444E+00_JPRB, 0.71712E+00_JPRB, 0.74033E+00_JPRB, 0.77172E+00_JPRB, 0.62170E+00_JPRB ,&
&  0.61131E+00_JPRB, 0.66929E+00_JPRB, 0.66233E+00_JPRB, 0.65113E+00_JPRB, 0.64713E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.73454E+00_JPRB, 0.72517E+00_JPRB, 0.71360E+00_JPRB, 0.70379E+00_JPRB, 0.69374E+00_JPRB, 0.68342E+00_JPRB, 0.68601E+00_JPRB ,&
&  0.69020E+00_JPRB, 0.67919E+00_JPRB, 0.66254E+00_JPRB, 0.64901E+00_JPRB, 0.63483E+00_JPRB, 0.63013E+00_JPRB, 0.64025E+00_JPRB ,&
&  0.52681E+00_JPRB, 0.47734E+00_JPRB, 0.42712E+00_JPRB, 0.42051E+00_JPRB, 0.47154E+00_JPRB, 0.45385E+00_JPRB, 0.44587E+00_JPRB ,&
&  0.44038E+00_JPRB, 0.44634E+00_JPRB, 0.44659E+00_JPRB, 0.46414E+00_JPRB, 0.48322E+00_JPRB, 0.50592E+00_JPRB, 0.50705E+00_JPRB ,&
&  0.47064E+00_JPRB, 0.42969E+00_JPRB, 0.41809E+00_JPRB, 0.42440E+00_JPRB, 0.46452E+00_JPRB, 0.50436E+00_JPRB, 0.57392E+00_JPRB ,&
&  0.62195E+00_JPRB, 0.66461E+00_JPRB, 0.70545E+00_JPRB, 0.75863E+00_JPRB, 0.79957E+00_JPRB, 0.84567E+00_JPRB, 0.90283E+00_JPRB ,&
&  0.96683E+00_JPRB, 0.10444E+01_JPRB, 0.11256E+01_JPRB, 0.12274E+01_JPRB, 0.12487E+01_JPRB, 0.12978E+01_JPRB, 0.13551E+01_JPRB ,&
&  0.14276E+01_JPRB, 0.14734E+01_JPRB, 0.15905E+01_JPRB, 0.21307E+01_JPRB, 0.24854E+01_JPRB, 0.25017E+01_JPRB, 0.22321E+01_JPRB ,&
&  0.19449E+01_JPRB, 0.16677E+01_JPRB, 0.16322E+01_JPRB, 0.15303E+01_JPRB, 0.14292E+01_JPRB, 0.14996E+01_JPRB, 0.14573E+01_JPRB ,&
&  0.14005E+01_JPRB, 0.14260E+01_JPRB, 0.14398E+01_JPRB, 0.14938E+01_JPRB, 0.14346E+01_JPRB, 0.14496E+01_JPRB, 0.14103E+01_JPRB ,&
&  0.14268E+01_JPRB, 0.13168E+01_JPRB, 0.10108E+01_JPRB, 0.86255E+00_JPRB, 0.72113E+00_JPRB, 0.67520E+00_JPRB, 0.76757E+00_JPRB ,&
&  0.68716E+00_JPRB, 0.63435E+00_JPRB, 0.58918E+00_JPRB, 0.55231E+00_JPRB, 0.56347E+00_JPRB, 0.62909E+00_JPRB, 0.59022E+00_JPRB ,&
&  0.58317E+00_JPRB, 0.67992E+00_JPRB, 0.81169E+00_JPRB, 0.90797E+00_JPRB, 0.94903E+00_JPRB, 0.93664E+00_JPRB, 0.78962E+00_JPRB ,&
&  0.78677E+00_JPRB, 0.76299E+00_JPRB, 0.75611E+00_JPRB, 0.73955E+00_JPRB, 0.73371E+00_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.85415E+00_JPRB, 0.83616E+00_JPRB, 0.81004E+00_JPRB, 0.77197E+00_JPRB, 0.74651E+00_JPRB, 0.73467E+00_JPRB, 0.73304E+00_JPRB ,&
&  0.72468E+00_JPRB, 0.70299E+00_JPRB, 0.68682E+00_JPRB, 0.67093E+00_JPRB, 0.66487E+00_JPRB, 0.66626E+00_JPRB, 0.66753E+00_JPRB ,&
&  0.68363E+00_JPRB, 0.58090E+00_JPRB, 0.48977E+00_JPRB, 0.43551E+00_JPRB, 0.45354E+00_JPRB, 0.44563E+00_JPRB, 0.44757E+00_JPRB ,&
&  0.44749E+00_JPRB, 0.47463E+00_JPRB, 0.48710E+00_JPRB, 0.52421E+00_JPRB, 0.56915E+00_JPRB, 0.61035E+00_JPRB, 0.60959E+00_JPRB ,&
&  0.56522E+00_JPRB, 0.50799E+00_JPRB, 0.48425E+00_JPRB, 0.49129E+00_JPRB, 0.53950E+00_JPRB, 0.53670E+00_JPRB, 0.60255E+00_JPRB ,&
&  0.64918E+00_JPRB, 0.69777E+00_JPRB, 0.75042E+00_JPRB, 0.79797E+00_JPRB, 0.85960E+00_JPRB, 0.93245E+00_JPRB, 0.10089E+01_JPRB ,&
&  0.10886E+01_JPRB, 0.11499E+01_JPRB, 0.11737E+01_JPRB, 0.12168E+01_JPRB, 0.13233E+01_JPRB, 0.14309E+01_JPRB, 0.13641E+01_JPRB ,&
&  0.13875E+01_JPRB, 0.14192E+01_JPRB, 0.15194E+01_JPRB, 0.20422E+01_JPRB, 0.25088E+01_JPRB, 0.26239E+01_JPRB, 0.25022E+01_JPRB ,&
&  0.22703E+01_JPRB, 0.20375E+01_JPRB, 0.19884E+01_JPRB, 0.17956E+01_JPRB, 0.16908E+01_JPRB, 0.16972E+01_JPRB, 0.16803E+01_JPRB ,&
&  0.16236E+01_JPRB, 0.16301E+01_JPRB, 0.16651E+01_JPRB, 0.16278E+01_JPRB, 0.16344E+01_JPRB, 0.16328E+01_JPRB, 0.16893E+01_JPRB ,&
&  0.18568E+01_JPRB, 0.12922E+01_JPRB, 0.10342E+01_JPRB, 0.10668E+01_JPRB, 0.88320E+00_JPRB, 0.77666E+00_JPRB, 0.78717E+00_JPRB ,&
&  0.69237E+00_JPRB, 0.68820E+00_JPRB, 0.73850E+00_JPRB, 0.71133E+00_JPRB, 0.79070E+00_JPRB, 0.91861E+00_JPRB, 0.83461E+00_JPRB ,&
&  0.82291E+00_JPRB, 0.84686E+00_JPRB, 0.10583E+01_JPRB, 0.11961E+01_JPRB, 0.12300E+01_JPRB, 0.11633E+01_JPRB, 0.10647E+01_JPRB ,&
&  0.96489E+00_JPRB, 0.91955E+00_JPRB, 0.90369E+00_JPRB, 0.88674E+00_JPRB, 0.87371E+00_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.99211E+00_JPRB, 0.96080E+00_JPRB, 0.92988E+00_JPRB, 0.89218E+00_JPRB, 0.86413E+00_JPRB, 0.83789E+00_JPRB, 0.80288E+00_JPRB ,&
&  0.76487E+00_JPRB, 0.73400E+00_JPRB, 0.71349E+00_JPRB, 0.70417E+00_JPRB, 0.70509E+00_JPRB, 0.71289E+00_JPRB, 0.71552E+00_JPRB ,&
&  0.72452E+00_JPRB, 0.68062E+00_JPRB, 0.55973E+00_JPRB, 0.47969E+00_JPRB, 0.42827E+00_JPRB, 0.43406E+00_JPRB, 0.44115E+00_JPRB ,&
&  0.46463E+00_JPRB, 0.50624E+00_JPRB, 0.58320E+00_JPRB, 0.68044E+00_JPRB, 0.73777E+00_JPRB, 0.75219E+00_JPRB, 0.74882E+00_JPRB ,&
&  0.69134E+00_JPRB, 0.63664E+00_JPRB, 0.58362E+00_JPRB, 0.60585E+00_JPRB, 0.61619E+00_JPRB, 0.61063E+00_JPRB, 0.67220E+00_JPRB ,&
&  0.72025E+00_JPRB, 0.77723E+00_JPRB, 0.82628E+00_JPRB, 0.89490E+00_JPRB, 0.95312E+00_JPRB, 0.99804E+00_JPRB, 0.10386E+01_JPRB ,&
&  0.10722E+01_JPRB, 0.11135E+01_JPRB, 0.11442E+01_JPRB, 0.11649E+01_JPRB, 0.12267E+01_JPRB, 0.13038E+01_JPRB, 0.12951E+01_JPRB ,&
&  0.13961E+01_JPRB, 0.13608E+01_JPRB, 0.13814E+01_JPRB, 0.20060E+01_JPRB, 0.29231E+01_JPRB, 0.29438E+01_JPRB, 0.26580E+01_JPRB ,&
&  0.25865E+01_JPRB, 0.25229E+01_JPRB, 0.25774E+01_JPRB, 0.25249E+01_JPRB, 0.23276E+01_JPRB, 0.22729E+01_JPRB, 0.21871E+01_JPRB ,&
&  0.21186E+01_JPRB, 0.20721E+01_JPRB, 0.20125E+01_JPRB, 0.20559E+01_JPRB, 0.20938E+01_JPRB, 0.21790E+01_JPRB, 0.22689E+01_JPRB ,&
&  0.22639E+01_JPRB, 0.20090E+01_JPRB, 0.14137E+01_JPRB, 0.13343E+01_JPRB, 0.10629E+01_JPRB, 0.94797E+00_JPRB, 0.93739E+00_JPRB ,&
&  0.82425E+00_JPRB, 0.85091E+00_JPRB, 0.90309E+00_JPRB, 0.88055E+00_JPRB, 0.11193E+01_JPRB, 0.12502E+01_JPRB, 0.12600E+01_JPRB ,&
&  0.12470E+01_JPRB, 0.12053E+01_JPRB, 0.14602E+01_JPRB, 0.15418E+01_JPRB, 0.15242E+01_JPRB, 0.14379E+01_JPRB, 0.13273E+01_JPRB ,&
&  0.11998E+01_JPRB, 0.11233E+01_JPRB, 0.10874E+01_JPRB, 0.10621E+01_JPRB, 0.10249E+01_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.11056E+01_JPRB, 0.10505E+01_JPRB, 0.99788E+00_JPRB, 0.95876E+00_JPRB, 0.92788E+00_JPRB, 0.88993E+00_JPRB, 0.83953E+00_JPRB ,&
&  0.79703E+00_JPRB, 0.77454E+00_JPRB, 0.76170E+00_JPRB, 0.74944E+00_JPRB, 0.74598E+00_JPRB, 0.75004E+00_JPRB, 0.75596E+00_JPRB ,&
&  0.75445E+00_JPRB, 0.68290E+00_JPRB, 0.55310E+00_JPRB, 0.46915E+00_JPRB, 0.41793E+00_JPRB, 0.40606E+00_JPRB, 0.42732E+00_JPRB ,&
&  0.47401E+00_JPRB, 0.59912E+00_JPRB, 0.77772E+00_JPRB, 0.88254E+00_JPRB, 0.94961E+00_JPRB, 0.97456E+00_JPRB, 0.94758E+00_JPRB ,&
&  0.95585E+00_JPRB, 0.80223E+00_JPRB, 0.78886E+00_JPRB, 0.73413E+00_JPRB, 0.74804E+00_JPRB, 0.78578E+00_JPRB, 0.79326E+00_JPRB ,&
&  0.78010E+00_JPRB, 0.81258E+00_JPRB, 0.84462E+00_JPRB, 0.89762E+00_JPRB, 0.93177E+00_JPRB, 0.97834E+00_JPRB, 0.10546E+01_JPRB ,&
&  0.11422E+01_JPRB, 0.11832E+01_JPRB, 0.11445E+01_JPRB, 0.11081E+01_JPRB, 0.12878E+01_JPRB, 0.15042E+01_JPRB, 0.16095E+01_JPRB ,&
&  0.16112E+01_JPRB, 0.17117E+01_JPRB, 0.24757E+01_JPRB, 0.29225E+01_JPRB, 0.31226E+01_JPRB, 0.33884E+01_JPRB, 0.33065E+01_JPRB ,&
&  0.30160E+01_JPRB, 0.27533E+01_JPRB, 0.26232E+01_JPRB, 0.23430E+01_JPRB, 0.26772E+01_JPRB, 0.27876E+01_JPRB, 0.28069E+01_JPRB ,&
&  0.28118E+01_JPRB, 0.26959E+01_JPRB, 0.26168E+01_JPRB, 0.26539E+01_JPRB, 0.27526E+01_JPRB, 0.28679E+01_JPRB, 0.27720E+01_JPRB ,&
&  0.24557E+01_JPRB, 0.25423E+01_JPRB, 0.21709E+01_JPRB, 0.17597E+01_JPRB, 0.14325E+01_JPRB, 0.13203E+01_JPRB, 0.12140E+01_JPRB ,&
&  0.10974E+01_JPRB, 0.10508E+01_JPRB, 0.11348E+01_JPRB, 0.12738E+01_JPRB, 0.15074E+01_JPRB, 0.17361E+01_JPRB, 0.17560E+01_JPRB ,&
&  0.16799E+01_JPRB, 0.18612E+01_JPRB, 0.18267E+01_JPRB, 0.17222E+01_JPRB, 0.16571E+01_JPRB, 0.15436E+01_JPRB, 0.14525E+01_JPRB ,&
&  0.13614E+01_JPRB, 0.13071E+01_JPRB, 0.12690E+01_JPRB, 0.12253E+01_JPRB, 0.11675E+01_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.11837E+01_JPRB, 0.11223E+01_JPRB, 0.10633E+01_JPRB, 0.10145E+01_JPRB, 0.96507E+00_JPRB, 0.91978E+00_JPRB, 0.86944E+00_JPRB ,&
&  0.83202E+00_JPRB, 0.81078E+00_JPRB, 0.79230E+00_JPRB, 0.78303E+00_JPRB, 0.78249E+00_JPRB, 0.77489E+00_JPRB, 0.77249E+00_JPRB ,&
&  0.75958E+00_JPRB, 0.64980E+00_JPRB, 0.53885E+00_JPRB, 0.49655E+00_JPRB, 0.45597E+00_JPRB, 0.42266E+00_JPRB, 0.43502E+00_JPRB ,&
&  0.52439E+00_JPRB, 0.76141E+00_JPRB, 0.98037E+00_JPRB, 0.11158E+01_JPRB, 0.11784E+01_JPRB, 0.11843E+01_JPRB, 0.11359E+01_JPRB ,&
&  0.10300E+01_JPRB, 0.10438E+01_JPRB, 0.10477E+01_JPRB, 0.10052E+01_JPRB, 0.94663E+00_JPRB, 0.92173E+00_JPRB, 0.90359E+00_JPRB ,&
&  0.88583E+00_JPRB, 0.90746E+00_JPRB, 0.94252E+00_JPRB, 0.98114E+00_JPRB, 0.10449E+01_JPRB, 0.11201E+01_JPRB, 0.11954E+01_JPRB ,&
&  0.12109E+01_JPRB, 0.12115E+01_JPRB, 0.11538E+01_JPRB, 0.11615E+01_JPRB, 0.12039E+01_JPRB, 0.16570E+01_JPRB, 0.19052E+01_JPRB ,&
&  0.23800E+01_JPRB, 0.27712E+01_JPRB, 0.28730E+01_JPRB, 0.33774E+01_JPRB, 0.28993E+01_JPRB, 0.32276E+01_JPRB, 0.35281E+01_JPRB ,&
&  0.31200E+01_JPRB, 0.27922E+01_JPRB, 0.26836E+01_JPRB, 0.22995E+01_JPRB, 0.22727E+01_JPRB, 0.33292E+01_JPRB, 0.35933E+01_JPRB ,&
&  0.36966E+01_JPRB, 0.38977E+01_JPRB, 0.36797E+01_JPRB, 0.34775E+01_JPRB, 0.32848E+01_JPRB, 0.30219E+01_JPRB, 0.23089E+01_JPRB ,&
&  0.23016E+01_JPRB, 0.20869E+01_JPRB, 0.21458E+01_JPRB, 0.19000E+01_JPRB, 0.17343E+01_JPRB, 0.15597E+01_JPRB, 0.13857E+01_JPRB ,&
&  0.12779E+01_JPRB, 0.12396E+01_JPRB, 0.15573E+01_JPRB, 0.20184E+01_JPRB, 0.23203E+01_JPRB, 0.21399E+01_JPRB, 0.24689E+01_JPRB ,&
&  0.23780E+01_JPRB, 0.20740E+01_JPRB, 0.17193E+01_JPRB, 0.16146E+01_JPRB, 0.15317E+01_JPRB, 0.14586E+01_JPRB, 0.14106E+01_JPRB ,&
&  0.13633E+01_JPRB, 0.13160E+01_JPRB, 0.12835E+01_JPRB, 0.12666E+01_JPRB, 0.12321E+01_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.12100E+01_JPRB, 0.11500E+01_JPRB, 0.10865E+01_JPRB, 0.10362E+01_JPRB, 0.98925E+00_JPRB, 0.94498E+00_JPRB, 0.90739E+00_JPRB ,&
&  0.88289E+00_JPRB, 0.85596E+00_JPRB, 0.81851E+00_JPRB, 0.79431E+00_JPRB, 0.78164E+00_JPRB, 0.79034E+00_JPRB, 0.77979E+00_JPRB ,&
&  0.75734E+00_JPRB, 0.67475E+00_JPRB, 0.56530E+00_JPRB, 0.52234E+00_JPRB, 0.51824E+00_JPRB, 0.48581E+00_JPRB, 0.48354E+00_JPRB ,&
&  0.66611E+00_JPRB, 0.10534E+01_JPRB, 0.12563E+01_JPRB, 0.13087E+01_JPRB, 0.13595E+01_JPRB, 0.13567E+01_JPRB, 0.12736E+01_JPRB ,&
&  0.12612E+01_JPRB, 0.11945E+01_JPRB, 0.11360E+01_JPRB, 0.10770E+01_JPRB, 0.10308E+01_JPRB, 0.98247E+00_JPRB, 0.94427E+00_JPRB ,&
&  0.94257E+00_JPRB, 0.94912E+00_JPRB, 0.98264E+00_JPRB, 0.99239E+00_JPRB, 0.10668E+01_JPRB, 0.11130E+01_JPRB, 0.11318E+01_JPRB ,&
&  0.11427E+01_JPRB, 0.11709E+01_JPRB, 0.11648E+01_JPRB, 0.11937E+01_JPRB, 0.14245E+01_JPRB, 0.19829E+01_JPRB, 0.24666E+01_JPRB ,&
&  0.27739E+01_JPRB, 0.30329E+01_JPRB, 0.33406E+01_JPRB, 0.35552E+01_JPRB, 0.40753E+01_JPRB, 0.39535E+01_JPRB, 0.38866E+01_JPRB ,&
&  0.29522E+01_JPRB, 0.23855E+01_JPRB, 0.19724E+01_JPRB, 0.16531E+01_JPRB, 0.26781E+01_JPRB, 0.44029E+01_JPRB, 0.50555E+01_JPRB ,&
&  0.50909E+01_JPRB, 0.52976E+01_JPRB, 0.50796E+01_JPRB, 0.42541E+01_JPRB, 0.35666E+01_JPRB, 0.28610E+01_JPRB, 0.32092E+01_JPRB ,&
&  0.30768E+01_JPRB, 0.29500E+01_JPRB, 0.25897E+01_JPRB, 0.20763E+01_JPRB, 0.16823E+01_JPRB, 0.17515E+01_JPRB, 0.15887E+01_JPRB ,&
&  0.14540E+01_JPRB, 0.15252E+01_JPRB, 0.24184E+01_JPRB, 0.28988E+01_JPRB, 0.28647E+01_JPRB, 0.27658E+01_JPRB, 0.30601E+01_JPRB ,&
&  0.24742E+01_JPRB, 0.20085E+01_JPRB, 0.17050E+01_JPRB, 0.15177E+01_JPRB, 0.14355E+01_JPRB, 0.13842E+01_JPRB, 0.13613E+01_JPRB ,&
&  0.13428E+01_JPRB, 0.13308E+01_JPRB, 0.13064E+01_JPRB, 0.12763E+01_JPRB, 0.12450E+01_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.12376E+01_JPRB, 0.11779E+01_JPRB, 0.11257E+01_JPRB, 0.10720E+01_JPRB, 0.10211E+01_JPRB, 0.98290E+00_JPRB, 0.95583E+00_JPRB ,&
&  0.92017E+00_JPRB, 0.88852E+00_JPRB, 0.85517E+00_JPRB, 0.82722E+00_JPRB, 0.81413E+00_JPRB, 0.81096E+00_JPRB, 0.79660E+00_JPRB ,&
&  0.77967E+00_JPRB, 0.74128E+00_JPRB, 0.62001E+00_JPRB, 0.57466E+00_JPRB, 0.56951E+00_JPRB, 0.56423E+00_JPRB, 0.60021E+00_JPRB ,&
&  0.96293E+00_JPRB, 0.14190E+01_JPRB, 0.14741E+01_JPRB, 0.14063E+01_JPRB, 0.14416E+01_JPRB, 0.12439E+01_JPRB, 0.11925E+01_JPRB ,&
&  0.12786E+01_JPRB, 0.12259E+01_JPRB, 0.11680E+01_JPRB, 0.11153E+01_JPRB, 0.10749E+01_JPRB, 0.10441E+01_JPRB, 0.10176E+01_JPRB ,&
&  0.10012E+01_JPRB, 0.99686E+00_JPRB, 0.10312E+01_JPRB, 0.11104E+01_JPRB, 0.11435E+01_JPRB, 0.11479E+01_JPRB, 0.11512E+01_JPRB ,&
&  0.11436E+01_JPRB, 0.11396E+01_JPRB, 0.11586E+01_JPRB, 0.13164E+01_JPRB, 0.16431E+01_JPRB, 0.23256E+01_JPRB, 0.34398E+01_JPRB ,&
&  0.34759E+01_JPRB, 0.38488E+01_JPRB, 0.37658E+01_JPRB, 0.45935E+01_JPRB, 0.50151E+01_JPRB, 0.46059E+01_JPRB, 0.41226E+01_JPRB ,&
&  0.29646E+01_JPRB, 0.23645E+01_JPRB, 0.21689E+01_JPRB, 0.24636E+01_JPRB, 0.33828E+01_JPRB, 0.44674E+01_JPRB, 0.56844E+01_JPRB ,&
&  0.50608E+01_JPRB, 0.49972E+01_JPRB, 0.49490E+01_JPRB, 0.45201E+01_JPRB, 0.27552E+01_JPRB, 0.21094E+01_JPRB, 0.30111E+01_JPRB ,&
&  0.30207E+01_JPRB, 0.18070E+01_JPRB, 0.12745E+01_JPRB, 0.14988E+01_JPRB, 0.11198E+01_JPRB, 0.12541E+01_JPRB, 0.16394E+01_JPRB ,&
&  0.17283E+01_JPRB, 0.20159E+01_JPRB, 0.29454E+01_JPRB, 0.28933E+01_JPRB, 0.29495E+01_JPRB, 0.34001E+01_JPRB, 0.28708E+01_JPRB ,&
&  0.22881E+01_JPRB, 0.18546E+01_JPRB, 0.17580E+01_JPRB, 0.15822E+01_JPRB, 0.15005E+01_JPRB, 0.14467E+01_JPRB, 0.14233E+01_JPRB ,&
&  0.14364E+01_JPRB, 0.14249E+01_JPRB, 0.13872E+01_JPRB, 0.13374E+01_JPRB, 0.12901E+01_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.12756E+01_JPRB, 0.12194E+01_JPRB, 0.11633E+01_JPRB, 0.11143E+01_JPRB, 0.10735E+01_JPRB, 0.10347E+01_JPRB, 0.98131E+00_JPRB ,&
&  0.91899E+00_JPRB, 0.88195E+00_JPRB, 0.86472E+00_JPRB, 0.84499E+00_JPRB, 0.83709E+00_JPRB, 0.84038E+00_JPRB, 0.82803E+00_JPRB ,&
&  0.79899E+00_JPRB, 0.76347E+00_JPRB, 0.70812E+00_JPRB, 0.65818E+00_JPRB, 0.63621E+00_JPRB, 0.61837E+00_JPRB, 0.78607E+00_JPRB ,&
&  0.13659E+01_JPRB, 0.18050E+01_JPRB, 0.16964E+01_JPRB, 0.14662E+01_JPRB, 0.12921E+01_JPRB, 0.11456E+01_JPRB, 0.12435E+01_JPRB ,&
&  0.12737E+01_JPRB, 0.12579E+01_JPRB, 0.11855E+01_JPRB, 0.11343E+01_JPRB, 0.10939E+01_JPRB, 0.10558E+01_JPRB, 0.10386E+01_JPRB ,&
&  0.10265E+01_JPRB, 0.10323E+01_JPRB, 0.10970E+01_JPRB, 0.11225E+01_JPRB, 0.11378E+01_JPRB, 0.11299E+01_JPRB, 0.11662E+01_JPRB ,&
&  0.11556E+01_JPRB, 0.11676E+01_JPRB, 0.12291E+01_JPRB, 0.14556E+01_JPRB, 0.20906E+01_JPRB, 0.30422E+01_JPRB, 0.33657E+01_JPRB ,&
&  0.34479E+01_JPRB, 0.37105E+01_JPRB, 0.44660E+01_JPRB, 0.47758E+01_JPRB, 0.50051E+01_JPRB, 0.47992E+01_JPRB, 0.39326E+01_JPRB ,&
&  0.33785E+01_JPRB, 0.29878E+01_JPRB, 0.25780E+01_JPRB, 0.29189E+01_JPRB, 0.41824E+01_JPRB, 0.44916E+01_JPRB, 0.53206E+01_JPRB ,&
&  0.52656E+01_JPRB, 0.47138E+01_JPRB, 0.36772E+01_JPRB, 0.28101E+01_JPRB, 0.28693E+01_JPRB, 0.21406E+01_JPRB, 0.13734E+01_JPRB ,&
&  0.12233E+01_JPRB, 0.10932E+01_JPRB, 0.96397E+00_JPRB, 0.96607E+00_JPRB, 0.91250E+00_JPRB, 0.11164E+01_JPRB, 0.16149E+01_JPRB ,&
&  0.21050E+01_JPRB, 0.24837E+01_JPRB, 0.28048E+01_JPRB, 0.26835E+01_JPRB, 0.26007E+01_JPRB, 0.31224E+01_JPRB, 0.30339E+01_JPRB ,&
&  0.19585E+01_JPRB, 0.16915E+01_JPRB, 0.16859E+01_JPRB, 0.15977E+01_JPRB, 0.15357E+01_JPRB, 0.15073E+01_JPRB, 0.15203E+01_JPRB ,&
&  0.15414E+01_JPRB, 0.15192E+01_JPRB, 0.14621E+01_JPRB, 0.13918E+01_JPRB, 0.13281E+01_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.12677E+01_JPRB, 0.11962E+01_JPRB, 0.11514E+01_JPRB, 0.11282E+01_JPRB, 0.10991E+01_JPRB, 0.10409E+01_JPRB, 0.95594E+00_JPRB ,&
&  0.89570E+00_JPRB, 0.86837E+00_JPRB, 0.85985E+00_JPRB, 0.85112E+00_JPRB, 0.84530E+00_JPRB, 0.84105E+00_JPRB, 0.82709E+00_JPRB ,&
&  0.79032E+00_JPRB, 0.76683E+00_JPRB, 0.76251E+00_JPRB, 0.73698E+00_JPRB, 0.73687E+00_JPRB, 0.73465E+00_JPRB, 0.10343E+01_JPRB ,&
&  0.18158E+01_JPRB, 0.21455E+01_JPRB, 0.18391E+01_JPRB, 0.14545E+01_JPRB, 0.13121E+01_JPRB, 0.12763E+01_JPRB, 0.13210E+01_JPRB ,&
&  0.12815E+01_JPRB, 0.12205E+01_JPRB, 0.11088E+01_JPRB, 0.10680E+01_JPRB, 0.10584E+01_JPRB, 0.10444E+01_JPRB, 0.10422E+01_JPRB ,&
&  0.10376E+01_JPRB, 0.10731E+01_JPRB, 0.10845E+01_JPRB, 0.10981E+01_JPRB, 0.11254E+01_JPRB, 0.11588E+01_JPRB, 0.12102E+01_JPRB ,&
&  0.12462E+01_JPRB, 0.12866E+01_JPRB, 0.14253E+01_JPRB, 0.18750E+01_JPRB, 0.25573E+01_JPRB, 0.29048E+01_JPRB, 0.34605E+01_JPRB ,&
&  0.40322E+01_JPRB, 0.44791E+01_JPRB, 0.42815E+01_JPRB, 0.51257E+01_JPRB, 0.50178E+01_JPRB, 0.43825E+01_JPRB, 0.38056E+01_JPRB ,&
&  0.34357E+01_JPRB, 0.37692E+01_JPRB, 0.30713E+01_JPRB, 0.28238E+01_JPRB, 0.45619E+01_JPRB, 0.50410E+01_JPRB, 0.53975E+01_JPRB ,&
&  0.57284E+01_JPRB, 0.48715E+01_JPRB, 0.38513E+01_JPRB, 0.32485E+01_JPRB, 0.41290E+01_JPRB, 0.38050E+01_JPRB, 0.16306E+01_JPRB ,&
&  0.12139E+01_JPRB, 0.10826E+01_JPRB, 0.92959E+00_JPRB, 0.85665E+00_JPRB, 0.84663E+00_JPRB, 0.10586E+01_JPRB, 0.18270E+01_JPRB ,&
&  0.20000E+01_JPRB, 0.26154E+01_JPRB, 0.26744E+01_JPRB, 0.26106E+01_JPRB, 0.24007E+01_JPRB, 0.23680E+01_JPRB, 0.24387E+01_JPRB ,&
&  0.20935E+01_JPRB, 0.18403E+01_JPRB, 0.15914E+01_JPRB, 0.14635E+01_JPRB, 0.14100E+01_JPRB, 0.14264E+01_JPRB, 0.15015E+01_JPRB ,&
&  0.15397E+01_JPRB, 0.15353E+01_JPRB, 0.14827E+01_JPRB, 0.14056E+01_JPRB, 0.13305E+01_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.12129E+01_JPRB, 0.11813E+01_JPRB, 0.11591E+01_JPRB, 0.11154E+01_JPRB, 0.10723E+01_JPRB, 0.10162E+01_JPRB, 0.94571E+00_JPRB ,&
&  0.90615E+00_JPRB, 0.88648E+00_JPRB, 0.87872E+00_JPRB, 0.86146E+00_JPRB, 0.84045E+00_JPRB, 0.82417E+00_JPRB, 0.81712E+00_JPRB ,&
&  0.79248E+00_JPRB, 0.76194E+00_JPRB, 0.75881E+00_JPRB, 0.80018E+00_JPRB, 0.85690E+00_JPRB, 0.98879E+00_JPRB, 0.13626E+01_JPRB ,&
&  0.21856E+01_JPRB, 0.23925E+01_JPRB, 0.19949E+01_JPRB, 0.16701E+01_JPRB, 0.15034E+01_JPRB, 0.14561E+01_JPRB, 0.13496E+01_JPRB ,&
&  0.12351E+01_JPRB, 0.11886E+01_JPRB, 0.10761E+01_JPRB, 0.10751E+01_JPRB, 0.10809E+01_JPRB, 0.10939E+01_JPRB, 0.11078E+01_JPRB ,&
&  0.11416E+01_JPRB, 0.11051E+01_JPRB, 0.11143E+01_JPRB, 0.11396E+01_JPRB, 0.11770E+01_JPRB, 0.12500E+01_JPRB, 0.13163E+01_JPRB ,&
&  0.13877E+01_JPRB, 0.14852E+01_JPRB, 0.17696E+01_JPRB, 0.22744E+01_JPRB, 0.24593E+01_JPRB, 0.31496E+01_JPRB, 0.35350E+01_JPRB ,&
&  0.38751E+01_JPRB, 0.39938E+01_JPRB, 0.38875E+01_JPRB, 0.44760E+01_JPRB, 0.47645E+01_JPRB, 0.42538E+01_JPRB, 0.37749E+01_JPRB ,&
&  0.39463E+01_JPRB, 0.34523E+01_JPRB, 0.30922E+01_JPRB, 0.28199E+01_JPRB, 0.37653E+01_JPRB, 0.63906E+01_JPRB, 0.50084E+01_JPRB ,&
&  0.48260E+01_JPRB, 0.54424E+01_JPRB, 0.45049E+01_JPRB, 0.39242E+01_JPRB, 0.42512E+01_JPRB, 0.46828E+01_JPRB, 0.35435E+01_JPRB ,&
&  0.14535E+01_JPRB, 0.10471E+01_JPRB, 0.10425E+01_JPRB, 0.10162E+01_JPRB, 0.89799E+00_JPRB, 0.10885E+01_JPRB, 0.26909E+01_JPRB ,&
&  0.22659E+01_JPRB, 0.28581E+01_JPRB, 0.27901E+01_JPRB, 0.25732E+01_JPRB, 0.24106E+01_JPRB, 0.21065E+01_JPRB, 0.20576E+01_JPRB ,&
&  0.17089E+01_JPRB, 0.15048E+01_JPRB, 0.14075E+01_JPRB, 0.13482E+01_JPRB, 0.13430E+01_JPRB, 0.13622E+01_JPRB, 0.14237E+01_JPRB ,&
&  0.14637E+01_JPRB, 0.14709E+01_JPRB, 0.14225E+01_JPRB, 0.13447E+01_JPRB, 0.12709E+01_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.11730E+01_JPRB, 0.11564E+01_JPRB, 0.11238E+01_JPRB, 0.10789E+01_JPRB, 0.10420E+01_JPRB, 0.98522E+00_JPRB, 0.93849E+00_JPRB ,&
&  0.92914E+00_JPRB, 0.90859E+00_JPRB, 0.88784E+00_JPRB, 0.86363E+00_JPRB, 0.83864E+00_JPRB, 0.81545E+00_JPRB, 0.80617E+00_JPRB ,&
&  0.79792E+00_JPRB, 0.76612E+00_JPRB, 0.75916E+00_JPRB, 0.82200E+00_JPRB, 0.10248E+01_JPRB, 0.12876E+01_JPRB, 0.15261E+01_JPRB ,&
&  0.24793E+01_JPRB, 0.26243E+01_JPRB, 0.19105E+01_JPRB, 0.17216E+01_JPRB, 0.16687E+01_JPRB, 0.15892E+01_JPRB, 0.14126E+01_JPRB ,&
&  0.12816E+01_JPRB, 0.12300E+01_JPRB, 0.11183E+01_JPRB, 0.10992E+01_JPRB, 0.11218E+01_JPRB, 0.11730E+01_JPRB, 0.12128E+01_JPRB ,&
&  0.11643E+01_JPRB, 0.11734E+01_JPRB, 0.12261E+01_JPRB, 0.12564E+01_JPRB, 0.12996E+01_JPRB, 0.13935E+01_JPRB, 0.14739E+01_JPRB ,&
&  0.15872E+01_JPRB, 0.17828E+01_JPRB, 0.21453E+01_JPRB, 0.24325E+01_JPRB, 0.28254E+01_JPRB, 0.31146E+01_JPRB, 0.35012E+01_JPRB ,&
&  0.35279E+01_JPRB, 0.36393E+01_JPRB, 0.38233E+01_JPRB, 0.40460E+01_JPRB, 0.41730E+01_JPRB, 0.40379E+01_JPRB, 0.38823E+01_JPRB ,&
&  0.39949E+01_JPRB, 0.34224E+01_JPRB, 0.29760E+01_JPRB, 0.28596E+01_JPRB, 0.35406E+01_JPRB, 0.56984E+01_JPRB, 0.63794E+01_JPRB ,&
&  0.64150E+01_JPRB, 0.45833E+01_JPRB, 0.39947E+01_JPRB, 0.44168E+01_JPRB, 0.41591E+01_JPRB, 0.41907E+01_JPRB, 0.51878E+01_JPRB ,&
&  0.45217E+01_JPRB, 0.24032E+01_JPRB, 0.19383E+01_JPRB, 0.23847E+01_JPRB, 0.14786E+01_JPRB, 0.11526E+01_JPRB, 0.17946E+01_JPRB ,&
&  0.21675E+01_JPRB, 0.25539E+01_JPRB, 0.25516E+01_JPRB, 0.24876E+01_JPRB, 0.23498E+01_JPRB, 0.19090E+01_JPRB, 0.16806E+01_JPRB ,&
&  0.13973E+01_JPRB, 0.13224E+01_JPRB, 0.12851E+01_JPRB, 0.12219E+01_JPRB, 0.12050E+01_JPRB, 0.12510E+01_JPRB, 0.13026E+01_JPRB ,&
&  0.13354E+01_JPRB, 0.13414E+01_JPRB, 0.13194E+01_JPRB, 0.12739E+01_JPRB, 0.12121E+01_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.11321E+01_JPRB, 0.11218E+01_JPRB, 0.11114E+01_JPRB, 0.10647E+01_JPRB, 0.10196E+01_JPRB, 0.97525E+00_JPRB, 0.96698E+00_JPRB ,&
&  0.93980E+00_JPRB, 0.91539E+00_JPRB, 0.89238E+00_JPRB, 0.85795E+00_JPRB, 0.84465E+00_JPRB, 0.82371E+00_JPRB, 0.83006E+00_JPRB ,&
&  0.84141E+00_JPRB, 0.81881E+00_JPRB, 0.84947E+00_JPRB, 0.10487E+01_JPRB, 0.14060E+01_JPRB, 0.21253E+01_JPRB, 0.17666E+01_JPRB ,&
&  0.24328E+01_JPRB, 0.28215E+01_JPRB, 0.19636E+01_JPRB, 0.18527E+01_JPRB, 0.18167E+01_JPRB, 0.17862E+01_JPRB, 0.15563E+01_JPRB ,&
&  0.13921E+01_JPRB, 0.13197E+01_JPRB, 0.12573E+01_JPRB, 0.12034E+01_JPRB, 0.12211E+01_JPRB, 0.12482E+01_JPRB, 0.12133E+01_JPRB ,&
&  0.12358E+01_JPRB, 0.13116E+01_JPRB, 0.13508E+01_JPRB, 0.13809E+01_JPRB, 0.14584E+01_JPRB, 0.15570E+01_JPRB, 0.16701E+01_JPRB ,&
&  0.18618E+01_JPRB, 0.21185E+01_JPRB, 0.24937E+01_JPRB, 0.26200E+01_JPRB, 0.28468E+01_JPRB, 0.31486E+01_JPRB, 0.33150E+01_JPRB ,&
&  0.31967E+01_JPRB, 0.30676E+01_JPRB, 0.34317E+01_JPRB, 0.37617E+01_JPRB, 0.38111E+01_JPRB, 0.37641E+01_JPRB, 0.38697E+01_JPRB ,&
&  0.41863E+01_JPRB, 0.40292E+01_JPRB, 0.41017E+01_JPRB, 0.35958E+01_JPRB, 0.42953E+01_JPRB, 0.57495E+01_JPRB, 0.58102E+01_JPRB ,&
&  0.69546E+01_JPRB, 0.58848E+01_JPRB, 0.53706E+01_JPRB, 0.47048E+01_JPRB, 0.40804E+01_JPRB, 0.39076E+01_JPRB, 0.40888E+01_JPRB ,&
&  0.41575E+01_JPRB, 0.44660E+01_JPRB, 0.41557E+01_JPRB, 0.25372E+01_JPRB, 0.16670E+01_JPRB, 0.13223E+01_JPRB, 0.15004E+01_JPRB ,&
&  0.19463E+01_JPRB, 0.20798E+01_JPRB, 0.20993E+01_JPRB, 0.30366E+01_JPRB, 0.20572E+01_JPRB, 0.16003E+01_JPRB, 0.13457E+01_JPRB ,&
&  0.12048E+01_JPRB, 0.11826E+01_JPRB, 0.11475E+01_JPRB, 0.10916E+01_JPRB, 0.10926E+01_JPRB, 0.11231E+01_JPRB, 0.11651E+01_JPRB ,&
&  0.12023E+01_JPRB, 0.12312E+01_JPRB, 0.12476E+01_JPRB, 0.12203E+01_JPRB, 0.11695E+01_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.10796E+01_JPRB, 0.10806E+01_JPRB, 0.10559E+01_JPRB, 0.10231E+01_JPRB, 0.99983E+00_JPRB, 0.98381E+00_JPRB, 0.99161E+00_JPRB ,&
&  0.92523E+00_JPRB, 0.89632E+00_JPRB, 0.87907E+00_JPRB, 0.86228E+00_JPRB, 0.87501E+00_JPRB, 0.89305E+00_JPRB, 0.92726E+00_JPRB ,&
&  0.97373E+00_JPRB, 0.10314E+01_JPRB, 0.12039E+01_JPRB, 0.15916E+01_JPRB, 0.20851E+01_JPRB, 0.27356E+01_JPRB, 0.26935E+01_JPRB ,&
&  0.23779E+01_JPRB, 0.30874E+01_JPRB, 0.20040E+01_JPRB, 0.20437E+01_JPRB, 0.20313E+01_JPRB, 0.22094E+01_JPRB, 0.19773E+01_JPRB ,&
&  0.16634E+01_JPRB, 0.15420E+01_JPRB, 0.14662E+01_JPRB, 0.13394E+01_JPRB, 0.13141E+01_JPRB, 0.12928E+01_JPRB, 0.13070E+01_JPRB ,&
&  0.13871E+01_JPRB, 0.14749E+01_JPRB, 0.15323E+01_JPRB, 0.15981E+01_JPRB, 0.16889E+01_JPRB, 0.17841E+01_JPRB, 0.19570E+01_JPRB ,&
&  0.22196E+01_JPRB, 0.25131E+01_JPRB, 0.26602E+01_JPRB, 0.27490E+01_JPRB, 0.28225E+01_JPRB, 0.30527E+01_JPRB, 0.31843E+01_JPRB ,&
&  0.31370E+01_JPRB, 0.30215E+01_JPRB, 0.32624E+01_JPRB, 0.33464E+01_JPRB, 0.33841E+01_JPRB, 0.37431E+01_JPRB, 0.38371E+01_JPRB ,&
&  0.43229E+01_JPRB, 0.40929E+01_JPRB, 0.47471E+01_JPRB, 0.44807E+01_JPRB, 0.47044E+01_JPRB, 0.52442E+01_JPRB, 0.45672E+01_JPRB ,&
&  0.44372E+01_JPRB, 0.47042E+01_JPRB, 0.46534E+01_JPRB, 0.44559E+01_JPRB, 0.41303E+01_JPRB, 0.38505E+01_JPRB, 0.35015E+01_JPRB ,&
&  0.30232E+01_JPRB, 0.29023E+01_JPRB, 0.25800E+01_JPRB, 0.17124E+01_JPRB, 0.13634E+01_JPRB, 0.12297E+01_JPRB, 0.13634E+01_JPRB ,&
&  0.17149E+01_JPRB, 0.18869E+01_JPRB, 0.22249E+01_JPRB, 0.22519E+01_JPRB, 0.15306E+01_JPRB, 0.12178E+01_JPRB, 0.11308E+01_JPRB ,&
&  0.10859E+01_JPRB, 0.10580E+01_JPRB, 0.99817E+00_JPRB, 0.96085E+00_JPRB, 0.97825E+00_JPRB, 0.10242E+01_JPRB, 0.10581E+01_JPRB ,&
&  0.11050E+01_JPRB, 0.11325E+01_JPRB, 0.11257E+01_JPRB, 0.10988E+01_JPRB, 0.10858E+01_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.11090E+01_JPRB, 0.11223E+01_JPRB, 0.11349E+01_JPRB, 0.11694E+01_JPRB, 0.12207E+01_JPRB, 0.12888E+01_JPRB, 0.12845E+01_JPRB ,&
&  0.11301E+01_JPRB, 0.88145E+00_JPRB, 0.89744E+00_JPRB, 0.93500E+00_JPRB, 0.10132E+01_JPRB, 0.10874E+01_JPRB, 0.11800E+01_JPRB ,&
&  0.13180E+01_JPRB, 0.15053E+01_JPRB, 0.18260E+01_JPRB, 0.22580E+01_JPRB, 0.27465E+01_JPRB, 0.33452E+01_JPRB, 0.34039E+01_JPRB ,&
&  0.30004E+01_JPRB, 0.23238E+01_JPRB, 0.22620E+01_JPRB, 0.22559E+01_JPRB, 0.21155E+01_JPRB, 0.23136E+01_JPRB, 0.22618E+01_JPRB ,&
&  0.19405E+01_JPRB, 0.17657E+01_JPRB, 0.16774E+01_JPRB, 0.15634E+01_JPRB, 0.14921E+01_JPRB, 0.15411E+01_JPRB, 0.16210E+01_JPRB ,&
&  0.17324E+01_JPRB, 0.18055E+01_JPRB, 0.18366E+01_JPRB, 0.18706E+01_JPRB, 0.19509E+01_JPRB, 0.20633E+01_JPRB, 0.23001E+01_JPRB ,&
&  0.25665E+01_JPRB, 0.27763E+01_JPRB, 0.28479E+01_JPRB, 0.27748E+01_JPRB, 0.28519E+01_JPRB, 0.29821E+01_JPRB, 0.30451E+01_JPRB ,&
&  0.30641E+01_JPRB, 0.31610E+01_JPRB, 0.32575E+01_JPRB, 0.33633E+01_JPRB, 0.32817E+01_JPRB, 0.37447E+01_JPRB, 0.38171E+01_JPRB ,&
&  0.40816E+01_JPRB, 0.39831E+01_JPRB, 0.46173E+01_JPRB, 0.41651E+01_JPRB, 0.46202E+01_JPRB, 0.45127E+01_JPRB, 0.40295E+01_JPRB ,&
&  0.38994E+01_JPRB, 0.42738E+01_JPRB, 0.43711E+01_JPRB, 0.42107E+01_JPRB, 0.40343E+01_JPRB, 0.34038E+01_JPRB, 0.33957E+01_JPRB ,&
&  0.26272E+01_JPRB, 0.26105E+01_JPRB, 0.18030E+01_JPRB, 0.13428E+01_JPRB, 0.10998E+01_JPRB, 0.12243E+01_JPRB, 0.14066E+01_JPRB ,&
&  0.17084E+01_JPRB, 0.17269E+01_JPRB, 0.16018E+01_JPRB, 0.15688E+01_JPRB, 0.13063E+01_JPRB, 0.10948E+01_JPRB, 0.10534E+01_JPRB ,&
&  0.10461E+01_JPRB, 0.99801E+00_JPRB, 0.93955E+00_JPRB, 0.93376E+00_JPRB, 0.95568E+00_JPRB, 0.10170E+01_JPRB, 0.10424E+01_JPRB ,&
&  0.10759E+01_JPRB, 0.10814E+01_JPRB, 0.10746E+01_JPRB, 0.10749E+01_JPRB, 0.10852E+01_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.11312E+01_JPRB, 0.11219E+01_JPRB, 0.11293E+01_JPRB, 0.11293E+01_JPRB, 0.11218E+01_JPRB, 0.10650E+01_JPRB, 0.10198E+01_JPRB ,&
&  0.96686E+00_JPRB, 0.92293E+00_JPRB, 0.10028E+01_JPRB, 0.11095E+01_JPRB, 0.12404E+01_JPRB, 0.13820E+01_JPRB, 0.15366E+01_JPRB ,&
&  0.17377E+01_JPRB, 0.19912E+01_JPRB, 0.23081E+01_JPRB, 0.25772E+01_JPRB, 0.28401E+01_JPRB, 0.32023E+01_JPRB, 0.34685E+01_JPRB ,&
&  0.29909E+01_JPRB, 0.19699E+01_JPRB, 0.23068E+01_JPRB, 0.24938E+01_JPRB, 0.23405E+01_JPRB, 0.23535E+01_JPRB, 0.22658E+01_JPRB ,&
&  0.20825E+01_JPRB, 0.19900E+01_JPRB, 0.19466E+01_JPRB, 0.19074E+01_JPRB, 0.18585E+01_JPRB, 0.19284E+01_JPRB, 0.19947E+01_JPRB ,&
&  0.20271E+01_JPRB, 0.20119E+01_JPRB, 0.20044E+01_JPRB, 0.20972E+01_JPRB, 0.22824E+01_JPRB, 0.24067E+01_JPRB, 0.25948E+01_JPRB ,&
&  0.27592E+01_JPRB, 0.29586E+01_JPRB, 0.30449E+01_JPRB, 0.30646E+01_JPRB, 0.31854E+01_JPRB, 0.32721E+01_JPRB, 0.32672E+01_JPRB ,&
&  0.32249E+01_JPRB, 0.32130E+01_JPRB, 0.33387E+01_JPRB, 0.37281E+01_JPRB, 0.39618E+01_JPRB, 0.36283E+01_JPRB, 0.38367E+01_JPRB ,&
&  0.39903E+01_JPRB, 0.41570E+01_JPRB, 0.40534E+01_JPRB, 0.52757E+01_JPRB, 0.39904E+01_JPRB, 0.39264E+01_JPRB, 0.33441E+01_JPRB ,&
&  0.37050E+01_JPRB, 0.38128E+01_JPRB, 0.40076E+01_JPRB, 0.40147E+01_JPRB, 0.37701E+01_JPRB, 0.32251E+01_JPRB, 0.32482E+01_JPRB ,&
&  0.27918E+01_JPRB, 0.23905E+01_JPRB, 0.17142E+01_JPRB, 0.12757E+01_JPRB, 0.12285E+01_JPRB, 0.18556E+01_JPRB, 0.18465E+01_JPRB ,&
&  0.16979E+01_JPRB, 0.14705E+01_JPRB, 0.15158E+01_JPRB, 0.16658E+01_JPRB, 0.13553E+01_JPRB, 0.10484E+01_JPRB, 0.98116E+00_JPRB ,&
&  0.98836E+00_JPRB, 0.97974E+00_JPRB, 0.94428E+00_JPRB, 0.94974E+00_JPRB, 0.98853E+00_JPRB, 0.10516E+01_JPRB, 0.10892E+01_JPRB ,&
&  0.11005E+01_JPRB, 0.10860E+01_JPRB, 0.10872E+01_JPRB, 0.10977E+01_JPRB, 0.11296E+01_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.10775E+01_JPRB, 0.10509E+01_JPRB, 0.10259E+01_JPRB, 0.99611E+00_JPRB, 0.95999E+00_JPRB, 0.93725E+00_JPRB, 0.94169E+00_JPRB ,&
&  0.95708E+00_JPRB, 0.10117E+01_JPRB, 0.11005E+01_JPRB, 0.12107E+01_JPRB, 0.13561E+01_JPRB, 0.15258E+01_JPRB, 0.17078E+01_JPRB ,&
&  0.19448E+01_JPRB, 0.21805E+01_JPRB, 0.23247E+01_JPRB, 0.24335E+01_JPRB, 0.25711E+01_JPRB, 0.27858E+01_JPRB, 0.30110E+01_JPRB ,&
&  0.31913E+01_JPRB, 0.31305E+01_JPRB, 0.28676E+01_JPRB, 0.21432E+01_JPRB, 0.19118E+01_JPRB, 0.22618E+01_JPRB, 0.22551E+01_JPRB ,&
&  0.22286E+01_JPRB, 0.21597E+01_JPRB, 0.21120E+01_JPRB, 0.20722E+01_JPRB, 0.20838E+01_JPRB, 0.21345E+01_JPRB, 0.21667E+01_JPRB ,&
&  0.21641E+01_JPRB, 0.21775E+01_JPRB, 0.22311E+01_JPRB, 0.23607E+01_JPRB, 0.25602E+01_JPRB, 0.26936E+01_JPRB, 0.28329E+01_JPRB ,&
&  0.29541E+01_JPRB, 0.31598E+01_JPRB, 0.32863E+01_JPRB, 0.34251E+01_JPRB, 0.34831E+01_JPRB, 0.35954E+01_JPRB, 0.37618E+01_JPRB ,&
&  0.38014E+01_JPRB, 0.38789E+01_JPRB, 0.40571E+01_JPRB, 0.43720E+01_JPRB, 0.45220E+01_JPRB, 0.40056E+01_JPRB, 0.39138E+01_JPRB ,&
&  0.42342E+01_JPRB, 0.46039E+01_JPRB, 0.42889E+01_JPRB, 0.54889E+01_JPRB, 0.34664E+01_JPRB, 0.36604E+01_JPRB, 0.34999E+01_JPRB ,&
&  0.33111E+01_JPRB, 0.32879E+01_JPRB, 0.34606E+01_JPRB, 0.35570E+01_JPRB, 0.33549E+01_JPRB, 0.28637E+01_JPRB, 0.31495E+01_JPRB ,&
&  0.29074E+01_JPRB, 0.21595E+01_JPRB, 0.16140E+01_JPRB, 0.11562E+01_JPRB, 0.13756E+01_JPRB, 0.21573E+01_JPRB, 0.17295E+01_JPRB ,&
&  0.12575E+01_JPRB, 0.14070E+01_JPRB, 0.15660E+01_JPRB, 0.17201E+01_JPRB, 0.14602E+01_JPRB, 0.95718E+00_JPRB, 0.94395E+00_JPRB ,&
&  0.97111E+00_JPRB, 0.98468E+00_JPRB, 0.10059E+01_JPRB, 0.10170E+01_JPRB, 0.10427E+01_JPRB, 0.10869E+01_JPRB, 0.11256E+01_JPRB ,&
&  0.11319E+01_JPRB, 0.11234E+01_JPRB, 0.11341E+01_JPRB, 0.11264E+01_JPRB, 0.11046E+01_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.95312E+00_JPRB, 0.91022E+00_JPRB, 0.87581E+00_JPRB, 0.86223E+00_JPRB, 0.86448E+00_JPRB, 0.87948E+00_JPRB, 0.90895E+00_JPRB ,&
&  0.96157E+00_JPRB, 0.10308E+01_JPRB, 0.11030E+01_JPRB, 0.11722E+01_JPRB, 0.12657E+01_JPRB, 0.13742E+01_JPRB, 0.15123E+01_JPRB ,&
&  0.16658E+01_JPRB, 0.18261E+01_JPRB, 0.19064E+01_JPRB, 0.19303E+01_JPRB, 0.19947E+01_JPRB, 0.20914E+01_JPRB, 0.23398E+01_JPRB ,&
&  0.27186E+01_JPRB, 0.30410E+01_JPRB, 0.32175E+01_JPRB, 0.30131E+01_JPRB, 0.23700E+01_JPRB, 0.23627E+01_JPRB, 0.25641E+01_JPRB ,&
&  0.26423E+01_JPRB, 0.24183E+01_JPRB, 0.22523E+01_JPRB, 0.22273E+01_JPRB, 0.22311E+01_JPRB, 0.22762E+01_JPRB, 0.23056E+01_JPRB ,&
&  0.23133E+01_JPRB, 0.23425E+01_JPRB, 0.24246E+01_JPRB, 0.25294E+01_JPRB, 0.26843E+01_JPRB, 0.28507E+01_JPRB, 0.30312E+01_JPRB ,&
&  0.31325E+01_JPRB, 0.32546E+01_JPRB, 0.32709E+01_JPRB, 0.32145E+01_JPRB, 0.33131E+01_JPRB, 0.36528E+01_JPRB, 0.41370E+01_JPRB ,&
&  0.44394E+01_JPRB, 0.44369E+01_JPRB, 0.47674E+01_JPRB, 0.51525E+01_JPRB, 0.51065E+01_JPRB, 0.47016E+01_JPRB, 0.46387E+01_JPRB ,&
&  0.48383E+01_JPRB, 0.53975E+01_JPRB, 0.39695E+01_JPRB, 0.45961E+01_JPRB, 0.46935E+01_JPRB, 0.39669E+01_JPRB, 0.32218E+01_JPRB ,&
&  0.28636E+01_JPRB, 0.25369E+01_JPRB, 0.21990E+01_JPRB, 0.22784E+01_JPRB, 0.23726E+01_JPRB, 0.24731E+01_JPRB, 0.26863E+01_JPRB ,&
&  0.25562E+01_JPRB, 0.19279E+01_JPRB, 0.13941E+01_JPRB, 0.11238E+01_JPRB, 0.12723E+01_JPRB, 0.16533E+01_JPRB, 0.13483E+01_JPRB ,&
&  0.11801E+01_JPRB, 0.13345E+01_JPRB, 0.14206E+01_JPRB, 0.16801E+01_JPRB, 0.16912E+01_JPRB, 0.91822E+00_JPRB, 0.92843E+00_JPRB ,&
&  0.95368E+00_JPRB, 0.99275E+00_JPRB, 0.10061E+01_JPRB, 0.10151E+01_JPRB, 0.10222E+01_JPRB, 0.10500E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.10873E+01_JPRB, 0.10855E+01_JPRB, 0.10733E+01_JPRB, 0.10419E+01_JPRB, 0.99369E+00_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.81535E+00_JPRB, 0.81051E+00_JPRB, 0.79911E+00_JPRB, 0.81410E+00_JPRB, 0.83185E+00_JPRB, 0.83358E+00_JPRB, 0.85326E+00_JPRB ,&
&  0.88029E+00_JPRB, 0.90847E+00_JPRB, 0.92995E+00_JPRB, 0.94676E+00_JPRB, 0.10146E+01_JPRB, 0.10937E+01_JPRB, 0.11885E+01_JPRB ,&
&  0.12409E+01_JPRB, 0.12865E+01_JPRB, 0.12903E+01_JPRB, 0.12640E+01_JPRB, 0.12425E+01_JPRB, 0.11805E+01_JPRB, 0.11130E+01_JPRB ,&
&  0.11828E+01_JPRB, 0.15169E+01_JPRB, 0.20554E+01_JPRB, 0.25660E+01_JPRB, 0.30314E+01_JPRB, 0.26462E+01_JPRB, 0.28477E+01_JPRB ,&
&  0.25258E+01_JPRB, 0.20749E+01_JPRB, 0.21282E+01_JPRB, 0.22474E+01_JPRB, 0.23604E+01_JPRB, 0.23583E+01_JPRB, 0.24035E+01_JPRB ,&
&  0.24368E+01_JPRB, 0.24762E+01_JPRB, 0.25407E+01_JPRB, 0.26699E+01_JPRB, 0.27851E+01_JPRB, 0.28790E+01_JPRB, 0.29939E+01_JPRB ,&
&  0.30897E+01_JPRB, 0.31431E+01_JPRB, 0.32203E+01_JPRB, 0.30258E+01_JPRB, 0.33041E+01_JPRB, 0.38506E+01_JPRB, 0.43306E+01_JPRB ,&
&  0.45678E+01_JPRB, 0.47090E+01_JPRB, 0.51708E+01_JPRB, 0.53895E+01_JPRB, 0.55382E+01_JPRB, 0.54790E+01_JPRB, 0.54702E+01_JPRB ,&
&  0.55033E+01_JPRB, 0.50452E+01_JPRB, 0.36882E+01_JPRB, 0.39153E+01_JPRB, 0.34584E+01_JPRB, 0.27468E+01_JPRB, 0.26960E+01_JPRB ,&
&  0.23788E+01_JPRB, 0.20727E+01_JPRB, 0.16892E+01_JPRB, 0.16315E+01_JPRB, 0.17478E+01_JPRB, 0.18905E+01_JPRB, 0.22106E+01_JPRB ,&
&  0.18985E+01_JPRB, 0.15223E+01_JPRB, 0.11843E+01_JPRB, 0.11182E+01_JPRB, 0.11347E+01_JPRB, 0.12219E+01_JPRB, 0.12253E+01_JPRB ,&
&  0.12436E+01_JPRB, 0.11598E+01_JPRB, 0.12089E+01_JPRB, 0.13463E+01_JPRB, 0.97376E+00_JPRB, 0.83366E+00_JPRB, 0.84987E+00_JPRB ,&
&  0.86677E+00_JPRB, 0.87595E+00_JPRB, 0.88106E+00_JPRB, 0.87846E+00_JPRB, 0.88965E+00_JPRB, 0.91088E+00_JPRB, 0.91829E+00_JPRB ,&
&  0.91103E+00_JPRB, 0.87996E+00_JPRB, 0.85171E+00_JPRB, 0.83253E+00_JPRB, 0.81766E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.66229E+00_JPRB, 0.66397E+00_JPRB, 0.66731E+00_JPRB, 0.67686E+00_JPRB, 0.67898E+00_JPRB, 0.67723E+00_JPRB, 0.66672E+00_JPRB ,&
&  0.66021E+00_JPRB, 0.64733E+00_JPRB, 0.64021E+00_JPRB, 0.63722E+00_JPRB, 0.65735E+00_JPRB, 0.69857E+00_JPRB, 0.75757E+00_JPRB ,&
&  0.77516E+00_JPRB, 0.77156E+00_JPRB, 0.73905E+00_JPRB, 0.71767E+00_JPRB, 0.73750E+00_JPRB, 0.73682E+00_JPRB, 0.70943E+00_JPRB ,&
&  0.68812E+00_JPRB, 0.75950E+00_JPRB, 0.95118E+00_JPRB, 0.12397E+01_JPRB, 0.17536E+01_JPRB, 0.20536E+01_JPRB, 0.26468E+01_JPRB ,&
&  0.25656E+01_JPRB, 0.18738E+01_JPRB, 0.20165E+01_JPRB, 0.19480E+01_JPRB, 0.20881E+01_JPRB, 0.21664E+01_JPRB, 0.21975E+01_JPRB ,&
&  0.22696E+01_JPRB, 0.23450E+01_JPRB, 0.24164E+01_JPRB, 0.24947E+01_JPRB, 0.25569E+01_JPRB, 0.25721E+01_JPRB, 0.25909E+01_JPRB ,&
&  0.26109E+01_JPRB, 0.26082E+01_JPRB, 0.26820E+01_JPRB, 0.28902E+01_JPRB, 0.34083E+01_JPRB, 0.39885E+01_JPRB, 0.40974E+01_JPRB ,&
&  0.41704E+01_JPRB, 0.45171E+01_JPRB, 0.44739E+01_JPRB, 0.47930E+01_JPRB, 0.49338E+01_JPRB, 0.47961E+01_JPRB, 0.51212E+01_JPRB ,&
&  0.53217E+01_JPRB, 0.44526E+01_JPRB, 0.29770E+01_JPRB, 0.30572E+01_JPRB, 0.27629E+01_JPRB, 0.24482E+01_JPRB, 0.24444E+01_JPRB ,&
&  0.22285E+01_JPRB, 0.20138E+01_JPRB, 0.17868E+01_JPRB, 0.16385E+01_JPRB, 0.15874E+01_JPRB, 0.14961E+01_JPRB, 0.15004E+01_JPRB ,&
&  0.13799E+01_JPRB, 0.13268E+01_JPRB, 0.12604E+01_JPRB, 0.12236E+01_JPRB, 0.12857E+01_JPRB, 0.14175E+01_JPRB, 0.12331E+01_JPRB ,&
&  0.12164E+01_JPRB, 0.11794E+01_JPRB, 0.13385E+01_JPRB, 0.14366E+01_JPRB, 0.86265E+00_JPRB, 0.71287E+00_JPRB, 0.67817E+00_JPRB ,&
&  0.68327E+00_JPRB, 0.67779E+00_JPRB, 0.67316E+00_JPRB, 0.66325E+00_JPRB, 0.68988E+00_JPRB, 0.73313E+00_JPRB, 0.72401E+00_JPRB ,&
&  0.72646E+00_JPRB, 0.69282E+00_JPRB, 0.68398E+00_JPRB, 0.68419E+00_JPRB, 0.68306E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.62517E+00_JPRB, 0.61538E+00_JPRB, 0.60590E+00_JPRB, 0.59615E+00_JPRB, 0.60400E+00_JPRB, 0.60740E+00_JPRB, 0.62028E+00_JPRB ,&
&  0.63468E+00_JPRB, 0.63752E+00_JPRB, 0.64543E+00_JPRB, 0.64919E+00_JPRB, 0.66816E+00_JPRB, 0.68448E+00_JPRB, 0.71905E+00_JPRB ,&
&  0.75259E+00_JPRB, 0.77104E+00_JPRB, 0.79418E+00_JPRB, 0.81710E+00_JPRB, 0.83223E+00_JPRB, 0.84842E+00_JPRB, 0.83427E+00_JPRB ,&
&  0.81519E+00_JPRB, 0.81479E+00_JPRB, 0.84933E+00_JPRB, 0.98754E+00_JPRB, 0.12528E+01_JPRB, 0.15980E+01_JPRB, 0.22902E+01_JPRB ,&
&  0.23184E+01_JPRB, 0.16851E+01_JPRB, 0.15083E+01_JPRB, 0.13408E+01_JPRB, 0.14168E+01_JPRB, 0.15291E+01_JPRB, 0.15449E+01_JPRB ,&
&  0.15672E+01_JPRB, 0.15817E+01_JPRB, 0.15780E+01_JPRB, 0.15543E+01_JPRB, 0.14804E+01_JPRB, 0.13852E+01_JPRB, 0.13281E+01_JPRB ,&
&  0.13457E+01_JPRB, 0.13541E+01_JPRB, 0.15652E+01_JPRB, 0.21011E+01_JPRB, 0.25559E+01_JPRB, 0.32960E+01_JPRB, 0.31385E+01_JPRB ,&
&  0.31903E+01_JPRB, 0.33612E+01_JPRB, 0.33147E+01_JPRB, 0.35480E+01_JPRB, 0.36459E+01_JPRB, 0.37927E+01_JPRB, 0.38013E+01_JPRB ,&
&  0.40477E+01_JPRB, 0.35268E+01_JPRB, 0.24774E+01_JPRB, 0.26244E+01_JPRB, 0.24248E+01_JPRB, 0.23814E+01_JPRB, 0.22146E+01_JPRB ,&
&  0.20669E+01_JPRB, 0.18956E+01_JPRB, 0.17506E+01_JPRB, 0.16319E+01_JPRB, 0.15917E+01_JPRB, 0.15339E+01_JPRB, 0.13979E+01_JPRB ,&
&  0.13650E+01_JPRB, 0.13500E+01_JPRB, 0.13197E+01_JPRB, 0.12565E+01_JPRB, 0.13906E+01_JPRB, 0.13553E+01_JPRB, 0.16903E+01_JPRB ,&
&  0.11757E+01_JPRB, 0.10663E+01_JPRB, 0.84462E+00_JPRB, 0.10621E+01_JPRB, 0.73825E+00_JPRB, 0.60686E+00_JPRB, 0.60619E+00_JPRB ,&
&  0.58876E+00_JPRB, 0.56811E+00_JPRB, 0.55197E+00_JPRB, 0.55134E+00_JPRB, 0.56499E+00_JPRB, 0.59334E+00_JPRB, 0.59317E+00_JPRB ,&
&  0.60009E+00_JPRB, 0.60979E+00_JPRB, 0.62659E+00_JPRB, 0.64721E+00_JPRB, 0.64291E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.65322E+00_JPRB, 0.65521E+00_JPRB, 0.66575E+00_JPRB, 0.68386E+00_JPRB, 0.69423E+00_JPRB, 0.70278E+00_JPRB, 0.71001E+00_JPRB ,&
&  0.73157E+00_JPRB, 0.75592E+00_JPRB, 0.77847E+00_JPRB, 0.80057E+00_JPRB, 0.82184E+00_JPRB, 0.83969E+00_JPRB, 0.88050E+00_JPRB ,&
&  0.90365E+00_JPRB, 0.93473E+00_JPRB, 0.95581E+00_JPRB, 0.97960E+00_JPRB, 0.10005E+01_JPRB, 0.99255E+00_JPRB, 0.10006E+01_JPRB ,&
&  0.10129E+01_JPRB, 0.10463E+01_JPRB, 0.10787E+01_JPRB, 0.11387E+01_JPRB, 0.12166E+01_JPRB, 0.12420E+01_JPRB, 0.14230E+01_JPRB ,&
&  0.14397E+01_JPRB, 0.16243E+01_JPRB, 0.14204E+01_JPRB, 0.12180E+01_JPRB, 0.11702E+01_JPRB, 0.10513E+01_JPRB, 0.99331E+00_JPRB ,&
&  0.10156E+01_JPRB, 0.10295E+01_JPRB, 0.10186E+01_JPRB, 0.99506E+00_JPRB, 0.95731E+00_JPRB, 0.10305E+01_JPRB, 0.11849E+01_JPRB ,&
&  0.14377E+01_JPRB, 0.15877E+01_JPRB, 0.17800E+01_JPRB, 0.19419E+01_JPRB, 0.21635E+01_JPRB, 0.23031E+01_JPRB, 0.22803E+01_JPRB ,&
&  0.22887E+01_JPRB, 0.22564E+01_JPRB, 0.21317E+01_JPRB, 0.24559E+01_JPRB, 0.27119E+01_JPRB, 0.28275E+01_JPRB, 0.28528E+01_JPRB ,&
&  0.28418E+01_JPRB, 0.25113E+01_JPRB, 0.24108E+01_JPRB, 0.22533E+01_JPRB, 0.20920E+01_JPRB, 0.20733E+01_JPRB, 0.19452E+01_JPRB ,&
&  0.17980E+01_JPRB, 0.16992E+01_JPRB, 0.16648E+01_JPRB, 0.16611E+01_JPRB, 0.16788E+01_JPRB, 0.16686E+01_JPRB, 0.15354E+01_JPRB ,&
&  0.14369E+01_JPRB, 0.13645E+01_JPRB, 0.13067E+01_JPRB, 0.12605E+01_JPRB, 0.12081E+01_JPRB, 0.21478E+01_JPRB, 0.28378E+01_JPRB ,&
&  0.11002E+01_JPRB, 0.79550E+00_JPRB, 0.52833E+00_JPRB, 0.66090E+00_JPRB, 0.61051E+00_JPRB, 0.61631E+00_JPRB, 0.68739E+00_JPRB ,&
&  0.70226E+00_JPRB, 0.68957E+00_JPRB, 0.62537E+00_JPRB, 0.56323E+00_JPRB, 0.58047E+00_JPRB, 0.58740E+00_JPRB, 0.58953E+00_JPRB ,&
&  0.60560E+00_JPRB, 0.61463E+00_JPRB, 0.62731E+00_JPRB, 0.64972E+00_JPRB, 0.64652E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.68967E+00_JPRB, 0.70627E+00_JPRB, 0.72777E+00_JPRB, 0.75600E+00_JPRB, 0.78333E+00_JPRB, 0.80539E+00_JPRB, 0.82090E+00_JPRB ,&
&  0.84477E+00_JPRB, 0.87577E+00_JPRB, 0.89511E+00_JPRB, 0.90500E+00_JPRB, 0.92716E+00_JPRB, 0.94680E+00_JPRB, 0.98123E+00_JPRB ,&
&  0.10038E+01_JPRB, 0.10527E+01_JPRB, 0.10925E+01_JPRB, 0.11309E+01_JPRB, 0.11621E+01_JPRB, 0.11974E+01_JPRB, 0.12308E+01_JPRB ,&
&  0.12583E+01_JPRB, 0.13125E+01_JPRB, 0.13565E+01_JPRB, 0.13857E+01_JPRB, 0.14415E+01_JPRB, 0.15105E+01_JPRB, 0.12794E+01_JPRB ,&
&  0.15983E+01_JPRB, 0.15109E+01_JPRB, 0.13469E+01_JPRB, 0.12193E+01_JPRB, 0.11285E+01_JPRB, 0.10780E+01_JPRB, 0.10615E+01_JPRB ,&
&  0.11017E+01_JPRB, 0.12351E+01_JPRB, 0.13440E+01_JPRB, 0.13763E+01_JPRB, 0.13610E+01_JPRB, 0.15242E+01_JPRB, 0.18584E+01_JPRB ,&
&  0.20414E+01_JPRB, 0.21471E+01_JPRB, 0.22227E+01_JPRB, 0.22437E+01_JPRB, 0.23458E+01_JPRB, 0.24986E+01_JPRB, 0.25823E+01_JPRB ,&
&  0.25747E+01_JPRB, 0.24884E+01_JPRB, 0.22018E+01_JPRB, 0.26749E+01_JPRB, 0.26421E+01_JPRB, 0.25010E+01_JPRB, 0.22397E+01_JPRB ,&
&  0.19812E+01_JPRB, 0.17352E+01_JPRB, 0.17978E+01_JPRB, 0.18907E+01_JPRB, 0.18010E+01_JPRB, 0.17302E+01_JPRB, 0.15971E+01_JPRB ,&
&  0.14743E+01_JPRB, 0.14439E+01_JPRB, 0.14753E+01_JPRB, 0.15432E+01_JPRB, 0.15979E+01_JPRB, 0.15870E+01_JPRB, 0.14728E+01_JPRB ,&
&  0.13584E+01_JPRB, 0.12380E+01_JPRB, 0.11590E+01_JPRB, 0.11044E+01_JPRB, 0.10771E+01_JPRB, 0.11633E+01_JPRB, 0.16625E+01_JPRB ,&
&  0.92972E+00_JPRB, 0.56952E+00_JPRB, 0.58976E+00_JPRB, 0.58460E+00_JPRB, 0.61608E+00_JPRB, 0.63511E+00_JPRB, 0.79433E+00_JPRB ,&
&  0.10042E+01_JPRB, 0.11521E+01_JPRB, 0.11279E+01_JPRB, 0.86494E+00_JPRB, 0.69342E+00_JPRB, 0.64199E+00_JPRB, 0.63994E+00_JPRB ,&
&  0.65122E+00_JPRB, 0.65232E+00_JPRB, 0.65803E+00_JPRB, 0.66642E+00_JPRB, 0.67867E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.67319E+00_JPRB, 0.68540E+00_JPRB, 0.70099E+00_JPRB, 0.71775E+00_JPRB, 0.73826E+00_JPRB, 0.75941E+00_JPRB, 0.77525E+00_JPRB ,&
&  0.79247E+00_JPRB, 0.81212E+00_JPRB, 0.83028E+00_JPRB, 0.84291E+00_JPRB, 0.85840E+00_JPRB, 0.88731E+00_JPRB, 0.92136E+00_JPRB ,&
&  0.93940E+00_JPRB, 0.99313E+00_JPRB, 0.10230E+01_JPRB, 0.10761E+01_JPRB, 0.11263E+01_JPRB, 0.11862E+01_JPRB, 0.12320E+01_JPRB ,&
&  0.12802E+01_JPRB, 0.13494E+01_JPRB, 0.13787E+01_JPRB, 0.13724E+01_JPRB, 0.13995E+01_JPRB, 0.15139E+01_JPRB, 0.12603E+01_JPRB ,&
&  0.15740E+01_JPRB, 0.15510E+01_JPRB, 0.14265E+01_JPRB, 0.13805E+01_JPRB, 0.13504E+01_JPRB, 0.13835E+01_JPRB, 0.14448E+01_JPRB ,&
&  0.15393E+01_JPRB, 0.17572E+01_JPRB, 0.17911E+01_JPRB, 0.18657E+01_JPRB, 0.19979E+01_JPRB, 0.22900E+01_JPRB, 0.25599E+01_JPRB ,&
&  0.26734E+01_JPRB, 0.28824E+01_JPRB, 0.29897E+01_JPRB, 0.30891E+01_JPRB, 0.32478E+01_JPRB, 0.34163E+01_JPRB, 0.35363E+01_JPRB ,&
&  0.34934E+01_JPRB, 0.34159E+01_JPRB, 0.31419E+01_JPRB, 0.33691E+01_JPRB, 0.31595E+01_JPRB, 0.25180E+01_JPRB, 0.19283E+01_JPRB ,&
&  0.13917E+01_JPRB, 0.14245E+01_JPRB, 0.15050E+01_JPRB, 0.15660E+01_JPRB, 0.14556E+01_JPRB, 0.13622E+01_JPRB, 0.11660E+01_JPRB ,&
&  0.10823E+01_JPRB, 0.10698E+01_JPRB, 0.11385E+01_JPRB, 0.12384E+01_JPRB, 0.12936E+01_JPRB, 0.12763E+01_JPRB, 0.11853E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.99977E+00_JPRB, 0.95599E+00_JPRB, 0.93927E+00_JPRB, 0.89101E+00_JPRB, 0.82200E+00_JPRB, 0.10631E+01_JPRB ,&
&  0.79225E+00_JPRB, 0.70127E+00_JPRB, 0.81640E+00_JPRB, 0.46217E+00_JPRB, 0.63896E+00_JPRB, 0.80812E+00_JPRB, 0.10741E+01_JPRB ,&
&  0.12785E+01_JPRB, 0.17932E+01_JPRB, 0.22359E+01_JPRB, 0.17012E+01_JPRB, 0.11852E+01_JPRB, 0.69978E+00_JPRB, 0.61072E+00_JPRB ,&
&  0.62253E+00_JPRB, 0.62988E+00_JPRB, 0.64062E+00_JPRB, 0.64323E+00_JPRB, 0.65813E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.61956E+00_JPRB, 0.61510E+00_JPRB, 0.61561E+00_JPRB, 0.60676E+00_JPRB, 0.60285E+00_JPRB, 0.60422E+00_JPRB, 0.59366E+00_JPRB ,&
&  0.60164E+00_JPRB, 0.61846E+00_JPRB, 0.62435E+00_JPRB, 0.64469E+00_JPRB, 0.65880E+00_JPRB, 0.70362E+00_JPRB, 0.76979E+00_JPRB ,&
&  0.82019E+00_JPRB, 0.92429E+00_JPRB, 0.98229E+00_JPRB, 0.10511E+01_JPRB, 0.11110E+01_JPRB, 0.11754E+01_JPRB, 0.12310E+01_JPRB ,&
&  0.13070E+01_JPRB, 0.14009E+01_JPRB, 0.14972E+01_JPRB, 0.15665E+01_JPRB, 0.15883E+01_JPRB, 0.17637E+01_JPRB, 0.14174E+01_JPRB ,&
&  0.16530E+01_JPRB, 0.16946E+01_JPRB, 0.16268E+01_JPRB, 0.16125E+01_JPRB, 0.16075E+01_JPRB, 0.16271E+01_JPRB, 0.16578E+01_JPRB ,&
&  0.17421E+01_JPRB, 0.18196E+01_JPRB, 0.18935E+01_JPRB, 0.21524E+01_JPRB, 0.24359E+01_JPRB, 0.26830E+01_JPRB, 0.29172E+01_JPRB ,&
&  0.31413E+01_JPRB, 0.33696E+01_JPRB, 0.36488E+01_JPRB, 0.39354E+01_JPRB, 0.42252E+01_JPRB, 0.44038E+01_JPRB, 0.46167E+01_JPRB ,&
&  0.47236E+01_JPRB, 0.47931E+01_JPRB, 0.44644E+01_JPRB, 0.38733E+01_JPRB, 0.32351E+01_JPRB, 0.25082E+01_JPRB, 0.17975E+01_JPRB ,&
&  0.13200E+01_JPRB, 0.12268E+01_JPRB, 0.12134E+01_JPRB, 0.11949E+01_JPRB, 0.10575E+01_JPRB, 0.96758E+00_JPRB, 0.90898E+00_JPRB ,&
&  0.85310E+00_JPRB, 0.78586E+00_JPRB, 0.82674E+00_JPRB, 0.89337E+00_JPRB, 0.91306E+00_JPRB, 0.90302E+00_JPRB, 0.87856E+00_JPRB ,&
&  0.84104E+00_JPRB, 0.78698E+00_JPRB, 0.78812E+00_JPRB, 0.79372E+00_JPRB, 0.81890E+00_JPRB, 0.88870E+00_JPRB, 0.78879E+00_JPRB ,&
&  0.84732E+00_JPRB, 0.64545E+00_JPRB, 0.63437E+00_JPRB, 0.64885E+00_JPRB, 0.67816E+00_JPRB, 0.86672E+00_JPRB, 0.12000E+01_JPRB ,&
&  0.17996E+01_JPRB, 0.20407E+01_JPRB, 0.22780E+01_JPRB, 0.28254E+01_JPRB, 0.24610E+01_JPRB, 0.18273E+01_JPRB, 0.56249E+00_JPRB ,&
&  0.56318E+00_JPRB, 0.56947E+00_JPRB, 0.59573E+00_JPRB, 0.60574E+00_JPRB, 0.61147E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.52748E+00_JPRB, 0.51199E+00_JPRB, 0.50441E+00_JPRB, 0.49987E+00_JPRB, 0.48465E+00_JPRB, 0.48621E+00_JPRB, 0.48455E+00_JPRB ,&
&  0.50285E+00_JPRB, 0.52710E+00_JPRB, 0.56911E+00_JPRB, 0.62066E+00_JPRB, 0.68928E+00_JPRB, 0.74322E+00_JPRB, 0.80975E+00_JPRB ,&
&  0.85550E+00_JPRB, 0.91301E+00_JPRB, 0.97407E+00_JPRB, 0.10323E+01_JPRB, 0.10995E+01_JPRB, 0.11555E+01_JPRB, 0.12348E+01_JPRB ,&
&  0.13089E+01_JPRB, 0.13949E+01_JPRB, 0.15041E+01_JPRB, 0.16019E+01_JPRB, 0.16816E+01_JPRB, 0.17345E+01_JPRB, 0.13816E+01_JPRB ,&
&  0.17207E+01_JPRB, 0.18034E+01_JPRB, 0.18268E+01_JPRB, 0.17906E+01_JPRB, 0.17393E+01_JPRB, 0.16871E+01_JPRB, 0.17112E+01_JPRB ,&
&  0.17548E+01_JPRB, 0.17874E+01_JPRB, 0.18171E+01_JPRB, 0.19187E+01_JPRB, 0.24543E+01_JPRB, 0.26938E+01_JPRB, 0.29411E+01_JPRB ,&
&  0.32046E+01_JPRB, 0.34997E+01_JPRB, 0.38079E+01_JPRB, 0.41190E+01_JPRB, 0.43285E+01_JPRB, 0.45543E+01_JPRB, 0.47906E+01_JPRB ,&
&  0.49736E+01_JPRB, 0.51277E+01_JPRB, 0.48988E+01_JPRB, 0.37849E+01_JPRB, 0.30714E+01_JPRB, 0.24515E+01_JPRB, 0.15385E+01_JPRB ,&
&  0.11442E+01_JPRB, 0.10412E+01_JPRB, 0.10292E+01_JPRB, 0.98376E+00_JPRB, 0.86421E+00_JPRB, 0.77817E+00_JPRB, 0.72098E+00_JPRB ,&
&  0.64231E+00_JPRB, 0.62361E+00_JPRB, 0.62322E+00_JPRB, 0.67428E+00_JPRB, 0.70576E+00_JPRB, 0.67566E+00_JPRB, 0.65579E+00_JPRB ,&
&  0.61691E+00_JPRB, 0.58641E+00_JPRB, 0.62940E+00_JPRB, 0.69548E+00_JPRB, 0.83915E+00_JPRB, 0.99073E+00_JPRB, 0.10892E+01_JPRB ,&
&  0.11048E+01_JPRB, 0.65813E+00_JPRB, 0.62968E+00_JPRB, 0.62503E+00_JPRB, 0.64670E+00_JPRB, 0.69649E+00_JPRB, 0.78987E+00_JPRB ,&
&  0.99298E+00_JPRB, 0.13530E+01_JPRB, 0.11934E+01_JPRB, 0.91305E+00_JPRB, 0.97464E+00_JPRB, 0.90012E+00_JPRB, 0.51011E+00_JPRB ,&
&  0.51334E+00_JPRB, 0.51281E+00_JPRB, 0.52946E+00_JPRB, 0.53478E+00_JPRB, 0.53820E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.46684E+00_JPRB, 0.44937E+00_JPRB, 0.43087E+00_JPRB, 0.42200E+00_JPRB, 0.41483E+00_JPRB, 0.40711E+00_JPRB, 0.41985E+00_JPRB ,&
&  0.45168E+00_JPRB, 0.48086E+00_JPRB, 0.52916E+00_JPRB, 0.60182E+00_JPRB, 0.67408E+00_JPRB, 0.72027E+00_JPRB, 0.77072E+00_JPRB ,&
&  0.81531E+00_JPRB, 0.86306E+00_JPRB, 0.90364E+00_JPRB, 0.94346E+00_JPRB, 0.99410E+00_JPRB, 0.10567E+01_JPRB, 0.11412E+01_JPRB ,&
&  0.12416E+01_JPRB, 0.13444E+01_JPRB, 0.14706E+01_JPRB, 0.16243E+01_JPRB, 0.17740E+01_JPRB, 0.18499E+01_JPRB, 0.19685E+01_JPRB ,&
&  0.14511E+01_JPRB, 0.18290E+01_JPRB, 0.18556E+01_JPRB, 0.18264E+01_JPRB, 0.17773E+01_JPRB, 0.17297E+01_JPRB, 0.17224E+01_JPRB ,&
&  0.17310E+01_JPRB, 0.16548E+01_JPRB, 0.16892E+01_JPRB, 0.18170E+01_JPRB, 0.21537E+01_JPRB, 0.24043E+01_JPRB, 0.26544E+01_JPRB ,&
&  0.29199E+01_JPRB, 0.32151E+01_JPRB, 0.35232E+01_JPRB, 0.38176E+01_JPRB, 0.40504E+01_JPRB, 0.43020E+01_JPRB, 0.44938E+01_JPRB ,&
&  0.47294E+01_JPRB, 0.49263E+01_JPRB, 0.49848E+01_JPRB, 0.40300E+01_JPRB, 0.31445E+01_JPRB, 0.26464E+01_JPRB, 0.18373E+01_JPRB ,&
&  0.11654E+01_JPRB, 0.90101E+00_JPRB, 0.90771E+00_JPRB, 0.92661E+00_JPRB, 0.84020E+00_JPRB, 0.76953E+00_JPRB, 0.69271E+00_JPRB ,&
&  0.63666E+00_JPRB, 0.56967E+00_JPRB, 0.54993E+00_JPRB, 0.57124E+00_JPRB, 0.61734E+00_JPRB, 0.58522E+00_JPRB, 0.53791E+00_JPRB ,&
&  0.50473E+00_JPRB, 0.49383E+00_JPRB, 0.54505E+00_JPRB, 0.60463E+00_JPRB, 0.69006E+00_JPRB, 0.73726E+00_JPRB, 0.75629E+00_JPRB ,&
&  0.78841E+00_JPRB, 0.67107E+00_JPRB, 0.65408E+00_JPRB, 0.65925E+00_JPRB, 0.65603E+00_JPRB, 0.66082E+00_JPRB, 0.67713E+00_JPRB ,&
&  0.71338E+00_JPRB, 0.77828E+00_JPRB, 0.79543E+00_JPRB, 0.73889E+00_JPRB, 0.67354E+00_JPRB, 0.62459E+00_JPRB, 0.57990E+00_JPRB ,&
&  0.58322E+00_JPRB, 0.57061E+00_JPRB, 0.55605E+00_JPRB, 0.53145E+00_JPRB, 0.50029E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.47943E+00_JPRB, 0.44970E+00_JPRB, 0.40815E+00_JPRB, 0.37286E+00_JPRB, 0.39761E+00_JPRB, 0.39981E+00_JPRB, 0.41782E+00_JPRB ,&
&  0.42615E+00_JPRB, 0.45471E+00_JPRB, 0.48463E+00_JPRB, 0.52878E+00_JPRB, 0.58725E+00_JPRB, 0.64710E+00_JPRB, 0.69871E+00_JPRB ,&
&  0.74554E+00_JPRB, 0.79164E+00_JPRB, 0.82735E+00_JPRB, 0.84327E+00_JPRB, 0.86752E+00_JPRB, 0.90717E+00_JPRB, 0.95634E+00_JPRB ,&
&  0.10215E+01_JPRB, 0.11061E+01_JPRB, 0.12216E+01_JPRB, 0.13695E+01_JPRB, 0.15429E+01_JPRB, 0.16487E+01_JPRB, 0.17289E+01_JPRB ,&
&  0.98598E+00_JPRB, 0.15455E+01_JPRB, 0.18796E+01_JPRB, 0.18122E+01_JPRB, 0.17708E+01_JPRB, 0.17285E+01_JPRB, 0.17070E+01_JPRB ,&
&  0.16029E+01_JPRB, 0.14947E+01_JPRB, 0.15130E+01_JPRB, 0.17750E+01_JPRB, 0.17947E+01_JPRB, 0.20171E+01_JPRB, 0.22054E+01_JPRB ,&
&  0.24086E+01_JPRB, 0.26246E+01_JPRB, 0.28736E+01_JPRB, 0.31805E+01_JPRB, 0.34915E+01_JPRB, 0.37488E+01_JPRB, 0.39731E+01_JPRB ,&
&  0.42747E+01_JPRB, 0.45548E+01_JPRB, 0.46523E+01_JPRB, 0.35532E+01_JPRB, 0.31799E+01_JPRB, 0.29440E+01_JPRB, 0.25735E+01_JPRB ,&
&  0.14303E+01_JPRB, 0.91781E+00_JPRB, 0.91580E+00_JPRB, 0.94868E+00_JPRB, 0.88251E+00_JPRB, 0.82011E+00_JPRB, 0.75681E+00_JPRB ,&
&  0.67631E+00_JPRB, 0.61569E+00_JPRB, 0.57495E+00_JPRB, 0.56051E+00_JPRB, 0.58587E+00_JPRB, 0.58992E+00_JPRB, 0.55942E+00_JPRB ,&
&  0.52512E+00_JPRB, 0.52692E+00_JPRB, 0.56319E+00_JPRB, 0.59458E+00_JPRB, 0.64368E+00_JPRB, 0.67619E+00_JPRB, 0.71050E+00_JPRB ,&
&  0.71723E+00_JPRB, 0.66848E+00_JPRB, 0.66839E+00_JPRB, 0.68144E+00_JPRB, 0.68630E+00_JPRB, 0.68970E+00_JPRB, 0.68947E+00_JPRB ,&
&  0.68846E+00_JPRB, 0.70747E+00_JPRB, 0.71641E+00_JPRB, 0.72320E+00_JPRB, 0.70552E+00_JPRB, 0.68453E+00_JPRB, 0.67322E+00_JPRB ,&
&  0.65403E+00_JPRB, 0.62775E+00_JPRB, 0.60164E+00_JPRB, 0.55852E+00_JPRB, 0.51591E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.50395E+00_JPRB, 0.46996E+00_JPRB, 0.42876E+00_JPRB, 0.41353E+00_JPRB, 0.43990E+00_JPRB, 0.46279E+00_JPRB, 0.45938E+00_JPRB ,&
&  0.45413E+00_JPRB, 0.44473E+00_JPRB, 0.44078E+00_JPRB, 0.45261E+00_JPRB, 0.49471E+00_JPRB, 0.55673E+00_JPRB, 0.60785E+00_JPRB ,&
&  0.65490E+00_JPRB, 0.69248E+00_JPRB, 0.72215E+00_JPRB, 0.73243E+00_JPRB, 0.72737E+00_JPRB, 0.74293E+00_JPRB, 0.76906E+00_JPRB ,&
&  0.81359E+00_JPRB, 0.87423E+00_JPRB, 0.95317E+00_JPRB, 0.10666E+01_JPRB, 0.12292E+01_JPRB, 0.14296E+01_JPRB, 0.16361E+01_JPRB ,&
&  0.11948E+01_JPRB, 0.57808E+00_JPRB, 0.15715E+01_JPRB, 0.19588E+01_JPRB, 0.18424E+01_JPRB, 0.17963E+01_JPRB, 0.16888E+01_JPRB ,&
&  0.15452E+01_JPRB, 0.15392E+01_JPRB, 0.14732E+01_JPRB, 0.16123E+01_JPRB, 0.15892E+01_JPRB, 0.16616E+01_JPRB, 0.17315E+01_JPRB ,&
&  0.18333E+01_JPRB, 0.19528E+01_JPRB, 0.21354E+01_JPRB, 0.23567E+01_JPRB, 0.26014E+01_JPRB, 0.29675E+01_JPRB, 0.32357E+01_JPRB ,&
&  0.34729E+01_JPRB, 0.37130E+01_JPRB, 0.37810E+01_JPRB, 0.27064E+01_JPRB, 0.21340E+01_JPRB, 0.18387E+01_JPRB, 0.16295E+01_JPRB ,&
&  0.13430E+01_JPRB, 0.10135E+01_JPRB, 0.97045E+00_JPRB, 0.99787E+00_JPRB, 0.95506E+00_JPRB, 0.85983E+00_JPRB, 0.84953E+00_JPRB ,&
&  0.74730E+00_JPRB, 0.68212E+00_JPRB, 0.63293E+00_JPRB, 0.59362E+00_JPRB, 0.59715E+00_JPRB, 0.61015E+00_JPRB, 0.61366E+00_JPRB ,&
&  0.59340E+00_JPRB, 0.59131E+00_JPRB, 0.60764E+00_JPRB, 0.60939E+00_JPRB, 0.61294E+00_JPRB, 0.62060E+00_JPRB, 0.62902E+00_JPRB ,&
&  0.66152E+00_JPRB, 0.65916E+00_JPRB, 0.66755E+00_JPRB, 0.68665E+00_JPRB, 0.69785E+00_JPRB, 0.68713E+00_JPRB, 0.69422E+00_JPRB ,&
&  0.71012E+00_JPRB, 0.72500E+00_JPRB, 0.71984E+00_JPRB, 0.73077E+00_JPRB, 0.72467E+00_JPRB, 0.71998E+00_JPRB, 0.72556E+00_JPRB ,&
&  0.73331E+00_JPRB, 0.72581E+00_JPRB, 0.68622E+00_JPRB, 0.59922E+00_JPRB, 0.54852E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.53928E+00_JPRB, 0.48767E+00_JPRB, 0.45036E+00_JPRB, 0.45307E+00_JPRB, 0.47714E+00_JPRB, 0.48558E+00_JPRB, 0.48644E+00_JPRB ,&
&  0.47722E+00_JPRB, 0.46535E+00_JPRB, 0.44646E+00_JPRB, 0.43774E+00_JPRB, 0.44365E+00_JPRB, 0.45963E+00_JPRB, 0.50285E+00_JPRB ,&
&  0.54584E+00_JPRB, 0.58705E+00_JPRB, 0.62046E+00_JPRB, 0.62309E+00_JPRB, 0.60222E+00_JPRB, 0.59958E+00_JPRB, 0.61270E+00_JPRB ,&
&  0.63040E+00_JPRB, 0.66712E+00_JPRB, 0.71638E+00_JPRB, 0.78409E+00_JPRB, 0.88872E+00_JPRB, 0.10646E+01_JPRB, 0.13369E+01_JPRB ,&
&  0.17863E+01_JPRB, 0.12288E+01_JPRB, 0.82589E+00_JPRB, 0.16781E+01_JPRB, 0.20490E+01_JPRB, 0.20911E+01_JPRB, 0.17482E+01_JPRB ,&
&  0.16871E+01_JPRB, 0.16014E+01_JPRB, 0.15195E+01_JPRB, 0.15745E+01_JPRB, 0.14883E+01_JPRB, 0.14730E+01_JPRB, 0.14589E+01_JPRB ,&
&  0.14417E+01_JPRB, 0.15304E+01_JPRB, 0.16508E+01_JPRB, 0.17674E+01_JPRB, 0.18860E+01_JPRB, 0.21376E+01_JPRB, 0.24594E+01_JPRB ,&
&  0.27728E+01_JPRB, 0.29734E+01_JPRB, 0.30442E+01_JPRB, 0.22508E+01_JPRB, 0.18311E+01_JPRB, 0.16307E+01_JPRB, 0.15287E+01_JPRB ,&
&  0.13253E+01_JPRB, 0.11935E+01_JPRB, 0.11511E+01_JPRB, 0.10841E+01_JPRB, 0.10171E+01_JPRB, 0.94659E+00_JPRB, 0.94791E+00_JPRB ,&
&  0.85311E+00_JPRB, 0.77288E+00_JPRB, 0.72222E+00_JPRB, 0.68909E+00_JPRB, 0.67454E+00_JPRB, 0.67397E+00_JPRB, 0.66830E+00_JPRB ,&
&  0.67027E+00_JPRB, 0.67241E+00_JPRB, 0.67564E+00_JPRB, 0.65724E+00_JPRB, 0.63837E+00_JPRB, 0.62573E+00_JPRB, 0.60915E+00_JPRB ,&
&  0.62610E+00_JPRB, 0.65080E+00_JPRB, 0.66453E+00_JPRB, 0.68647E+00_JPRB, 0.68732E+00_JPRB, 0.68217E+00_JPRB, 0.70363E+00_JPRB ,&
&  0.71320E+00_JPRB, 0.73157E+00_JPRB, 0.72719E+00_JPRB, 0.73335E+00_JPRB, 0.72304E+00_JPRB, 0.71613E+00_JPRB, 0.72301E+00_JPRB ,&
&  0.76262E+00_JPRB, 0.80748E+00_JPRB, 0.79136E+00_JPRB, 0.63111E+00_JPRB, 0.57252E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.52722E+00_JPRB, 0.50748E+00_JPRB, 0.46843E+00_JPRB, 0.47477E+00_JPRB, 0.48001E+00_JPRB, 0.48907E+00_JPRB, 0.49068E+00_JPRB ,&
&  0.49186E+00_JPRB, 0.49362E+00_JPRB, 0.49600E+00_JPRB, 0.48614E+00_JPRB, 0.45947E+00_JPRB, 0.44781E+00_JPRB, 0.44989E+00_JPRB ,&
&  0.46265E+00_JPRB, 0.49278E+00_JPRB, 0.52866E+00_JPRB, 0.53498E+00_JPRB, 0.53290E+00_JPRB, 0.52674E+00_JPRB, 0.52349E+00_JPRB ,&
&  0.52916E+00_JPRB, 0.54102E+00_JPRB, 0.55688E+00_JPRB, 0.59220E+00_JPRB, 0.65557E+00_JPRB, 0.75510E+00_JPRB, 0.92065E+00_JPRB ,&
&  0.13287E+01_JPRB, 0.14033E+01_JPRB, 0.67000E+00_JPRB, 0.13166E+01_JPRB, 0.20327E+01_JPRB, 0.21523E+01_JPRB, 0.20405E+01_JPRB ,&
&  0.19204E+01_JPRB, 0.17124E+01_JPRB, 0.15942E+01_JPRB, 0.15003E+01_JPRB, 0.13611E+01_JPRB, 0.12875E+01_JPRB, 0.12161E+01_JPRB ,&
&  0.11759E+01_JPRB, 0.12291E+01_JPRB, 0.13542E+01_JPRB, 0.14481E+01_JPRB, 0.14822E+01_JPRB, 0.15589E+01_JPRB, 0.17963E+01_JPRB ,&
&  0.21493E+01_JPRB, 0.25065E+01_JPRB, 0.26726E+01_JPRB, 0.21012E+01_JPRB, 0.18258E+01_JPRB, 0.18224E+01_JPRB, 0.17461E+01_JPRB ,&
&  0.15972E+01_JPRB, 0.13973E+01_JPRB, 0.12631E+01_JPRB, 0.11757E+01_JPRB, 0.10857E+01_JPRB, 0.10455E+01_JPRB, 0.10518E+01_JPRB ,&
&  0.97590E+00_JPRB, 0.87884E+00_JPRB, 0.81744E+00_JPRB, 0.77755E+00_JPRB, 0.74982E+00_JPRB, 0.73518E+00_JPRB, 0.72455E+00_JPRB ,&
&  0.71741E+00_JPRB, 0.72511E+00_JPRB, 0.73718E+00_JPRB, 0.72656E+00_JPRB, 0.70091E+00_JPRB, 0.67070E+00_JPRB, 0.64158E+00_JPRB ,&
&  0.63401E+00_JPRB, 0.64060E+00_JPRB, 0.66387E+00_JPRB, 0.67641E+00_JPRB, 0.68373E+00_JPRB, 0.69360E+00_JPRB, 0.70707E+00_JPRB ,&
&  0.72372E+00_JPRB, 0.73277E+00_JPRB, 0.72024E+00_JPRB, 0.70345E+00_JPRB, 0.71831E+00_JPRB, 0.70445E+00_JPRB, 0.69249E+00_JPRB ,&
&  0.72802E+00_JPRB, 0.79853E+00_JPRB, 0.90767E+00_JPRB, 0.63707E+00_JPRB, 0.57132E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.52404E+00_JPRB, 0.49458E+00_JPRB, 0.48857E+00_JPRB, 0.48860E+00_JPRB, 0.49401E+00_JPRB, 0.50442E+00_JPRB, 0.51534E+00_JPRB ,&
&  0.51833E+00_JPRB, 0.51896E+00_JPRB, 0.51901E+00_JPRB, 0.51429E+00_JPRB, 0.50023E+00_JPRB, 0.48541E+00_JPRB, 0.47105E+00_JPRB ,&
&  0.46632E+00_JPRB, 0.46826E+00_JPRB, 0.48294E+00_JPRB, 0.49112E+00_JPRB, 0.49198E+00_JPRB, 0.48918E+00_JPRB, 0.49040E+00_JPRB ,&
&  0.49516E+00_JPRB, 0.49578E+00_JPRB, 0.49472E+00_JPRB, 0.50893E+00_JPRB, 0.53969E+00_JPRB, 0.60037E+00_JPRB, 0.70508E+00_JPRB ,&
&  0.99497E+00_JPRB, 0.12312E+01_JPRB, 0.12470E+01_JPRB, 0.19583E+01_JPRB, 0.24248E+01_JPRB, 0.23613E+01_JPRB, 0.22755E+01_JPRB ,&
&  0.21107E+01_JPRB, 0.18807E+01_JPRB, 0.17898E+01_JPRB, 0.15128E+01_JPRB, 0.13111E+01_JPRB, 0.11558E+01_JPRB, 0.10370E+01_JPRB ,&
&  0.94998E+00_JPRB, 0.93587E+00_JPRB, 0.10157E+01_JPRB, 0.11532E+01_JPRB, 0.12443E+01_JPRB, 0.12788E+01_JPRB, 0.13715E+01_JPRB ,&
&  0.16163E+01_JPRB, 0.20638E+01_JPRB, 0.24317E+01_JPRB, 0.22554E+01_JPRB, 0.18253E+01_JPRB, 0.20197E+01_JPRB, 0.22360E+01_JPRB ,&
&  0.25334E+01_JPRB, 0.18143E+01_JPRB, 0.14206E+01_JPRB, 0.12681E+01_JPRB, 0.11528E+01_JPRB, 0.12319E+01_JPRB, 0.11640E+01_JPRB ,&
&  0.10781E+01_JPRB, 0.98940E+00_JPRB, 0.91172E+00_JPRB, 0.86150E+00_JPRB, 0.82378E+00_JPRB, 0.79562E+00_JPRB, 0.77751E+00_JPRB ,&
&  0.76677E+00_JPRB, 0.76166E+00_JPRB, 0.77076E+00_JPRB, 0.77271E+00_JPRB, 0.76111E+00_JPRB, 0.73415E+00_JPRB, 0.69682E+00_JPRB ,&
&  0.66585E+00_JPRB, 0.65472E+00_JPRB, 0.64103E+00_JPRB, 0.65490E+00_JPRB, 0.67461E+00_JPRB, 0.68616E+00_JPRB, 0.68930E+00_JPRB ,&
&  0.70874E+00_JPRB, 0.72556E+00_JPRB, 0.71889E+00_JPRB, 0.70360E+00_JPRB, 0.69929E+00_JPRB, 0.69845E+00_JPRB, 0.67035E+00_JPRB ,&
&  0.65659E+00_JPRB, 0.64417E+00_JPRB, 0.62478E+00_JPRB, 0.59939E+00_JPRB, 0.55686E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.52915E+00_JPRB, 0.51053E+00_JPRB, 0.50548E+00_JPRB, 0.51004E+00_JPRB, 0.51598E+00_JPRB, 0.52285E+00_JPRB, 0.53502E+00_JPRB ,&
&  0.54105E+00_JPRB, 0.54157E+00_JPRB, 0.53955E+00_JPRB, 0.53597E+00_JPRB, 0.52584E+00_JPRB, 0.51429E+00_JPRB, 0.50129E+00_JPRB ,&
&  0.49023E+00_JPRB, 0.46770E+00_JPRB, 0.45603E+00_JPRB, 0.46293E+00_JPRB, 0.46638E+00_JPRB, 0.46662E+00_JPRB, 0.46822E+00_JPRB ,&
&  0.46887E+00_JPRB, 0.47391E+00_JPRB, 0.47411E+00_JPRB, 0.48141E+00_JPRB, 0.49668E+00_JPRB, 0.53662E+00_JPRB, 0.60118E+00_JPRB ,&
&  0.80125E+00_JPRB, 0.11391E+01_JPRB, 0.11303E+01_JPRB, 0.22468E+01_JPRB, 0.25082E+01_JPRB, 0.24430E+01_JPRB, 0.21688E+01_JPRB ,&
&  0.19933E+01_JPRB, 0.20309E+01_JPRB, 0.16343E+01_JPRB, 0.13789E+01_JPRB, 0.12404E+01_JPRB, 0.11010E+01_JPRB, 0.94296E+00_JPRB ,&
&  0.82542E+00_JPRB, 0.77371E+00_JPRB, 0.76706E+00_JPRB, 0.83026E+00_JPRB, 0.94468E+00_JPRB, 0.10486E+01_JPRB, 0.11286E+01_JPRB ,&
&  0.12695E+01_JPRB, 0.15980E+01_JPRB, 0.20776E+01_JPRB, 0.20627E+01_JPRB, 0.19420E+01_JPRB, 0.22402E+01_JPRB, 0.28217E+01_JPRB ,&
&  0.34280E+01_JPRB, 0.26796E+01_JPRB, 0.17534E+01_JPRB, 0.14094E+01_JPRB, 0.13144E+01_JPRB, 0.13473E+01_JPRB, 0.13156E+01_JPRB ,&
&  0.12147E+01_JPRB, 0.11283E+01_JPRB, 0.10257E+01_JPRB, 0.95757E+00_JPRB, 0.90925E+00_JPRB, 0.85682E+00_JPRB, 0.82007E+00_JPRB ,&
&  0.80057E+00_JPRB, 0.78836E+00_JPRB, 0.78387E+00_JPRB, 0.77591E+00_JPRB, 0.77334E+00_JPRB, 0.76794E+00_JPRB, 0.75242E+00_JPRB ,&
&  0.72203E+00_JPRB, 0.68412E+00_JPRB, 0.64289E+00_JPRB, 0.63525E+00_JPRB, 0.65355E+00_JPRB, 0.66940E+00_JPRB, 0.67925E+00_JPRB ,&
&  0.71318E+00_JPRB, 0.72337E+00_JPRB, 0.70673E+00_JPRB, 0.67899E+00_JPRB, 0.67793E+00_JPRB, 0.70274E+00_JPRB, 0.67281E+00_JPRB ,&
&  0.66532E+00_JPRB, 0.65179E+00_JPRB, 0.63300E+00_JPRB, 0.61125E+00_JPRB, 0.57088E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.56044E+00_JPRB, 0.54524E+00_JPRB, 0.53597E+00_JPRB, 0.53463E+00_JPRB, 0.53126E+00_JPRB, 0.53374E+00_JPRB, 0.54421E+00_JPRB ,&
&  0.55925E+00_JPRB, 0.56309E+00_JPRB, 0.56011E+00_JPRB, 0.55616E+00_JPRB, 0.54869E+00_JPRB, 0.53777E+00_JPRB, 0.52476E+00_JPRB ,&
&  0.51321E+00_JPRB, 0.49598E+00_JPRB, 0.47314E+00_JPRB, 0.45618E+00_JPRB, 0.45868E+00_JPRB, 0.46362E+00_JPRB, 0.46480E+00_JPRB ,&
&  0.46441E+00_JPRB, 0.47104E+00_JPRB, 0.47380E+00_JPRB, 0.48407E+00_JPRB, 0.49222E+00_JPRB, 0.51544E+00_JPRB, 0.57356E+00_JPRB ,&
&  0.73218E+00_JPRB, 0.12779E+01_JPRB, 0.11872E+01_JPRB, 0.20503E+01_JPRB, 0.23251E+01_JPRB, 0.22466E+01_JPRB, 0.19681E+01_JPRB ,&
&  0.18196E+01_JPRB, 0.17113E+01_JPRB, 0.14450E+01_JPRB, 0.12294E+01_JPRB, 0.10943E+01_JPRB, 0.98721E+00_JPRB, 0.88683E+00_JPRB ,&
&  0.80933E+00_JPRB, 0.76475E+00_JPRB, 0.73823E+00_JPRB, 0.72636E+00_JPRB, 0.76466E+00_JPRB, 0.84269E+00_JPRB, 0.93180E+00_JPRB ,&
&  0.10323E+01_JPRB, 0.12096E+01_JPRB, 0.16188E+01_JPRB, 0.20326E+01_JPRB, 0.19204E+01_JPRB, 0.22502E+01_JPRB, 0.33871E+01_JPRB ,&
&  0.39867E+01_JPRB, 0.38678E+01_JPRB, 0.24712E+01_JPRB, 0.17167E+01_JPRB, 0.14808E+01_JPRB, 0.14926E+01_JPRB, 0.14791E+01_JPRB ,&
&  0.13854E+01_JPRB, 0.13138E+01_JPRB, 0.11985E+01_JPRB, 0.10721E+01_JPRB, 0.97933E+00_JPRB, 0.91467E+00_JPRB, 0.85517E+00_JPRB ,&
&  0.81195E+00_JPRB, 0.79250E+00_JPRB, 0.77652E+00_JPRB, 0.76021E+00_JPRB, 0.75224E+00_JPRB, 0.75443E+00_JPRB, 0.75790E+00_JPRB ,&
&  0.75380E+00_JPRB, 0.73001E+00_JPRB, 0.67747E+00_JPRB, 0.64426E+00_JPRB, 0.64484E+00_JPRB, 0.65856E+00_JPRB, 0.66821E+00_JPRB ,&
&  0.70981E+00_JPRB, 0.71825E+00_JPRB, 0.69266E+00_JPRB, 0.67709E+00_JPRB, 0.66718E+00_JPRB, 0.70735E+00_JPRB, 0.67661E+00_JPRB ,&
&  0.66423E+00_JPRB, 0.65273E+00_JPRB, 0.64711E+00_JPRB, 0.63326E+00_JPRB, 0.59779E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.58987E+00_JPRB, 0.57068E+00_JPRB, 0.56084E+00_JPRB, 0.54876E+00_JPRB, 0.54000E+00_JPRB, 0.53826E+00_JPRB, 0.54705E+00_JPRB ,&
&  0.56504E+00_JPRB, 0.57800E+00_JPRB, 0.57096E+00_JPRB, 0.56652E+00_JPRB, 0.56366E+00_JPRB, 0.55435E+00_JPRB, 0.54177E+00_JPRB ,&
&  0.52910E+00_JPRB, 0.51684E+00_JPRB, 0.50132E+00_JPRB, 0.48356E+00_JPRB, 0.47116E+00_JPRB, 0.47062E+00_JPRB, 0.46839E+00_JPRB ,&
&  0.46668E+00_JPRB, 0.47095E+00_JPRB, 0.47536E+00_JPRB, 0.48333E+00_JPRB, 0.49175E+00_JPRB, 0.50809E+00_JPRB, 0.55987E+00_JPRB ,&
&  0.74373E+00_JPRB, 0.59076E+00_JPRB, 0.14093E+01_JPRB, 0.18060E+01_JPRB, 0.18710E+01_JPRB, 0.17494E+01_JPRB, 0.16752E+01_JPRB ,&
&  0.16001E+01_JPRB, 0.14390E+01_JPRB, 0.12949E+01_JPRB, 0.11700E+01_JPRB, 0.10181E+01_JPRB, 0.91821E+00_JPRB, 0.85707E+00_JPRB ,&
&  0.80933E+00_JPRB, 0.76824E+00_JPRB, 0.73592E+00_JPRB, 0.70690E+00_JPRB, 0.69160E+00_JPRB, 0.71138E+00_JPRB, 0.76442E+00_JPRB ,&
&  0.82850E+00_JPRB, 0.92866E+00_JPRB, 0.12116E+01_JPRB, 0.16167E+01_JPRB, 0.18812E+01_JPRB, 0.21785E+01_JPRB, 0.27686E+01_JPRB ,&
&  0.36617E+01_JPRB, 0.40373E+01_JPRB, 0.30444E+01_JPRB, 0.22246E+01_JPRB, 0.17772E+01_JPRB, 0.16689E+01_JPRB, 0.16686E+01_JPRB ,&
&  0.15826E+01_JPRB, 0.14737E+01_JPRB, 0.13393E+01_JPRB, 0.11870E+01_JPRB, 0.10562E+01_JPRB, 0.96811E+00_JPRB, 0.90449E+00_JPRB ,&
&  0.85034E+00_JPRB, 0.80647E+00_JPRB, 0.77792E+00_JPRB, 0.75839E+00_JPRB, 0.74557E+00_JPRB, 0.74139E+00_JPRB, 0.74603E+00_JPRB ,&
&  0.75657E+00_JPRB, 0.75433E+00_JPRB, 0.72242E+00_JPRB, 0.69084E+00_JPRB, 0.68560E+00_JPRB, 0.69548E+00_JPRB, 0.70861E+00_JPRB ,&
&  0.72983E+00_JPRB, 0.72919E+00_JPRB, 0.70011E+00_JPRB, 0.68413E+00_JPRB, 0.65325E+00_JPRB, 0.68344E+00_JPRB, 0.66989E+00_JPRB ,&
&  0.65851E+00_JPRB, 0.65347E+00_JPRB, 0.65652E+00_JPRB, 0.64693E+00_JPRB, 0.62079E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.61247E+00_JPRB, 0.59303E+00_JPRB, 0.57868E+00_JPRB, 0.55877E+00_JPRB, 0.54885E+00_JPRB, 0.54727E+00_JPRB, 0.55579E+00_JPRB ,&
&  0.56486E+00_JPRB, 0.57159E+00_JPRB, 0.56601E+00_JPRB, 0.55997E+00_JPRB, 0.55214E+00_JPRB, 0.54643E+00_JPRB, 0.53786E+00_JPRB ,&
&  0.53283E+00_JPRB, 0.52818E+00_JPRB, 0.52001E+00_JPRB, 0.50920E+00_JPRB, 0.49477E+00_JPRB, 0.48747E+00_JPRB, 0.47961E+00_JPRB ,&
&  0.47257E+00_JPRB, 0.46969E+00_JPRB, 0.47097E+00_JPRB, 0.47976E+00_JPRB, 0.49173E+00_JPRB, 0.50739E+00_JPRB, 0.54320E+00_JPRB ,&
&  0.75476E+00_JPRB, 0.96175E+00_JPRB, 0.15304E+01_JPRB, 0.15639E+01_JPRB, 0.14154E+01_JPRB, 0.12531E+01_JPRB, 0.12410E+01_JPRB ,&
&  0.12089E+01_JPRB, 0.11455E+01_JPRB, 0.10717E+01_JPRB, 0.10205E+01_JPRB, 0.96652E+00_JPRB, 0.92904E+00_JPRB, 0.87379E+00_JPRB ,&
&  0.82222E+00_JPRB, 0.76787E+00_JPRB, 0.71051E+00_JPRB, 0.66863E+00_JPRB, 0.64261E+00_JPRB, 0.63701E+00_JPRB, 0.65581E+00_JPRB ,&
&  0.68642E+00_JPRB, 0.72974E+00_JPRB, 0.88854E+00_JPRB, 0.12286E+01_JPRB, 0.17632E+01_JPRB, 0.18906E+01_JPRB, 0.25966E+01_JPRB ,&
&  0.35352E+01_JPRB, 0.34995E+01_JPRB, 0.30986E+01_JPRB, 0.26831E+01_JPRB, 0.22356E+01_JPRB, 0.20604E+01_JPRB, 0.20177E+01_JPRB ,&
&  0.18707E+01_JPRB, 0.16362E+01_JPRB, 0.14007E+01_JPRB, 0.12240E+01_JPRB, 0.11026E+01_JPRB, 0.10268E+01_JPRB, 0.98234E+00_JPRB ,&
&  0.93903E+00_JPRB, 0.88997E+00_JPRB, 0.84335E+00_JPRB, 0.80659E+00_JPRB, 0.78325E+00_JPRB, 0.77250E+00_JPRB, 0.76900E+00_JPRB ,&
&  0.77111E+00_JPRB, 0.76928E+00_JPRB, 0.75701E+00_JPRB, 0.74014E+00_JPRB, 0.75543E+00_JPRB, 0.78725E+00_JPRB, 0.80041E+00_JPRB ,&
&  0.78450E+00_JPRB, 0.76061E+00_JPRB, 0.72665E+00_JPRB, 0.69248E+00_JPRB, 0.66202E+00_JPRB, 0.68187E+00_JPRB, 0.66502E+00_JPRB ,&
&  0.64451E+00_JPRB, 0.64268E+00_JPRB, 0.65757E+00_JPRB, 0.65626E+00_JPRB, 0.64565E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.63239E+00_JPRB, 0.61041E+00_JPRB, 0.59094E+00_JPRB, 0.57313E+00_JPRB, 0.55870E+00_JPRB, 0.55438E+00_JPRB, 0.55743E+00_JPRB ,&
&  0.56167E+00_JPRB, 0.56062E+00_JPRB, 0.55490E+00_JPRB, 0.54597E+00_JPRB, 0.53954E+00_JPRB, 0.53371E+00_JPRB, 0.52886E+00_JPRB ,&
&  0.52162E+00_JPRB, 0.52234E+00_JPRB, 0.51842E+00_JPRB, 0.51089E+00_JPRB, 0.50203E+00_JPRB, 0.49132E+00_JPRB, 0.48112E+00_JPRB ,&
&  0.47420E+00_JPRB, 0.47139E+00_JPRB, 0.47151E+00_JPRB, 0.47590E+00_JPRB, 0.48875E+00_JPRB, 0.50258E+00_JPRB, 0.52348E+00_JPRB ,&
&  0.68283E+00_JPRB, 0.97697E+00_JPRB, 0.11799E+01_JPRB, 0.13041E+01_JPRB, 0.11630E+01_JPRB, 0.10231E+01_JPRB, 0.95301E+00_JPRB ,&
&  0.92358E+00_JPRB, 0.91378E+00_JPRB, 0.87758E+00_JPRB, 0.83498E+00_JPRB, 0.78530E+00_JPRB, 0.76125E+00_JPRB, 0.73381E+00_JPRB ,&
&  0.70303E+00_JPRB, 0.67466E+00_JPRB, 0.64756E+00_JPRB, 0.63434E+00_JPRB, 0.62367E+00_JPRB, 0.61190E+00_JPRB, 0.60618E+00_JPRB ,&
&  0.61248E+00_JPRB, 0.62629E+00_JPRB, 0.67909E+00_JPRB, 0.88215E+00_JPRB, 0.13278E+01_JPRB, 0.16828E+01_JPRB, 0.21381E+01_JPRB ,&
&  0.22703E+01_JPRB, 0.23943E+01_JPRB, 0.24812E+01_JPRB, 0.24361E+01_JPRB, 0.22618E+01_JPRB, 0.21533E+01_JPRB, 0.21394E+01_JPRB ,&
&  0.20104E+01_JPRB, 0.16932E+01_JPRB, 0.13924E+01_JPRB, 0.12084E+01_JPRB, 0.11170E+01_JPRB, 0.10713E+01_JPRB, 0.10330E+01_JPRB ,&
&  0.10054E+01_JPRB, 0.97695E+00_JPRB, 0.93776E+00_JPRB, 0.89238E+00_JPRB, 0.85110E+00_JPRB, 0.81616E+00_JPRB, 0.79445E+00_JPRB ,&
&  0.77690E+00_JPRB, 0.76759E+00_JPRB, 0.77350E+00_JPRB, 0.79139E+00_JPRB, 0.81111E+00_JPRB, 0.82872E+00_JPRB, 0.83827E+00_JPRB ,&
&  0.84354E+00_JPRB, 0.81651E+00_JPRB, 0.75348E+00_JPRB, 0.70934E+00_JPRB, 0.67203E+00_JPRB, 0.67305E+00_JPRB, 0.64837E+00_JPRB ,&
&  0.63066E+00_JPRB, 0.62523E+00_JPRB, 0.63705E+00_JPRB, 0.64508E+00_JPRB, 0.65011E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.70818E+00_JPRB, 0.65918E+00_JPRB, 0.61809E+00_JPRB, 0.58657E+00_JPRB, 0.56949E+00_JPRB, 0.56103E+00_JPRB, 0.55472E+00_JPRB ,&
&  0.55079E+00_JPRB, 0.54515E+00_JPRB, 0.53456E+00_JPRB, 0.52619E+00_JPRB, 0.52017E+00_JPRB, 0.51691E+00_JPRB, 0.51386E+00_JPRB ,&
&  0.50391E+00_JPRB, 0.50207E+00_JPRB, 0.50204E+00_JPRB, 0.49733E+00_JPRB, 0.49083E+00_JPRB, 0.48365E+00_JPRB, 0.47429E+00_JPRB ,&
&  0.46763E+00_JPRB, 0.46531E+00_JPRB, 0.46709E+00_JPRB, 0.47118E+00_JPRB, 0.47579E+00_JPRB, 0.48797E+00_JPRB, 0.51474E+00_JPRB ,&
&  0.64895E+00_JPRB, 0.80022E+00_JPRB, 0.91888E+00_JPRB, 0.97413E+00_JPRB, 0.86279E+00_JPRB, 0.80815E+00_JPRB, 0.78442E+00_JPRB ,&
&  0.78613E+00_JPRB, 0.79316E+00_JPRB, 0.75843E+00_JPRB, 0.70228E+00_JPRB, 0.65170E+00_JPRB, 0.62419E+00_JPRB, 0.60647E+00_JPRB ,&
&  0.59040E+00_JPRB, 0.57808E+00_JPRB, 0.56681E+00_JPRB, 0.56676E+00_JPRB, 0.56716E+00_JPRB, 0.56358E+00_JPRB, 0.56077E+00_JPRB ,&
&  0.56107E+00_JPRB, 0.56606E+00_JPRB, 0.58030E+00_JPRB, 0.65479E+00_JPRB, 0.87236E+00_JPRB, 0.11183E+01_JPRB, 0.13664E+01_JPRB ,&
&  0.14865E+01_JPRB, 0.16059E+01_JPRB, 0.17503E+01_JPRB, 0.18559E+01_JPRB, 0.18693E+01_JPRB, 0.18675E+01_JPRB, 0.18639E+01_JPRB ,&
&  0.18120E+01_JPRB, 0.16235E+01_JPRB, 0.13751E+01_JPRB, 0.11938E+01_JPRB, 0.10954E+01_JPRB, 0.10448E+01_JPRB, 0.10124E+01_JPRB ,&
&  0.98507E+00_JPRB, 0.96072E+00_JPRB, 0.92779E+00_JPRB, 0.88228E+00_JPRB, 0.83255E+00_JPRB, 0.79691E+00_JPRB, 0.77052E+00_JPRB ,&
&  0.75368E+00_JPRB, 0.75596E+00_JPRB, 0.77518E+00_JPRB, 0.80833E+00_JPRB, 0.83106E+00_JPRB, 0.84123E+00_JPRB, 0.83739E+00_JPRB ,&
&  0.82890E+00_JPRB, 0.82442E+00_JPRB, 0.75650E+00_JPRB, 0.70906E+00_JPRB, 0.72354E+00_JPRB, 0.67449E+00_JPRB, 0.63320E+00_JPRB ,&
&  0.61750E+00_JPRB, 0.60520E+00_JPRB, 0.60376E+00_JPRB, 0.62021E+00_JPRB, 0.73229E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.65244E+00_JPRB, 0.63682E+00_JPRB, 0.61028E+00_JPRB, 0.58522E+00_JPRB, 0.56973E+00_JPRB, 0.56092E+00_JPRB, 0.55554E+00_JPRB ,&
&  0.54856E+00_JPRB, 0.53893E+00_JPRB, 0.52771E+00_JPRB, 0.51371E+00_JPRB, 0.50472E+00_JPRB, 0.50005E+00_JPRB, 0.49661E+00_JPRB ,&
&  0.49118E+00_JPRB, 0.48125E+00_JPRB, 0.47793E+00_JPRB, 0.47490E+00_JPRB, 0.47337E+00_JPRB, 0.47146E+00_JPRB, 0.46639E+00_JPRB ,&
&  0.46007E+00_JPRB, 0.45708E+00_JPRB, 0.46045E+00_JPRB, 0.46246E+00_JPRB, 0.46490E+00_JPRB, 0.47102E+00_JPRB, 0.48910E+00_JPRB ,&
&  0.57966E+00_JPRB, 0.64137E+00_JPRB, 0.73117E+00_JPRB, 0.80286E+00_JPRB, 0.76755E+00_JPRB, 0.72899E+00_JPRB, 0.70871E+00_JPRB ,&
&  0.68182E+00_JPRB, 0.66330E+00_JPRB, 0.62996E+00_JPRB, 0.58834E+00_JPRB, 0.55399E+00_JPRB, 0.54346E+00_JPRB, 0.53619E+00_JPRB ,&
&  0.53015E+00_JPRB, 0.52657E+00_JPRB, 0.52420E+00_JPRB, 0.52320E+00_JPRB, 0.52333E+00_JPRB, 0.52219E+00_JPRB, 0.52100E+00_JPRB ,&
&  0.52218E+00_JPRB, 0.52679E+00_JPRB, 0.53354E+00_JPRB, 0.55221E+00_JPRB, 0.63347E+00_JPRB, 0.78823E+00_JPRB, 0.95547E+00_JPRB ,&
&  0.10589E+01_JPRB, 0.11459E+01_JPRB, 0.12749E+01_JPRB, 0.13837E+01_JPRB, 0.14534E+01_JPRB, 0.14801E+01_JPRB, 0.14769E+01_JPRB ,&
&  0.14626E+01_JPRB, 0.13981E+01_JPRB, 0.12560E+01_JPRB, 0.11048E+01_JPRB, 0.10068E+01_JPRB, 0.94442E+00_JPRB, 0.90028E+00_JPRB ,&
&  0.86766E+00_JPRB, 0.82918E+00_JPRB, 0.79146E+00_JPRB, 0.75901E+00_JPRB, 0.73709E+00_JPRB, 0.71710E+00_JPRB, 0.70234E+00_JPRB ,&
&  0.70829E+00_JPRB, 0.73436E+00_JPRB, 0.77123E+00_JPRB, 0.79451E+00_JPRB, 0.79994E+00_JPRB, 0.79897E+00_JPRB, 0.78897E+00_JPRB ,&
&  0.78012E+00_JPRB, 0.77585E+00_JPRB, 0.75948E+00_JPRB, 0.74109E+00_JPRB, 0.70556E+00_JPRB, 0.67284E+00_JPRB, 0.62645E+00_JPRB ,&
&  0.60258E+00_JPRB, 0.58921E+00_JPRB, 0.57656E+00_JPRB, 0.58833E+00_JPRB, 0.63808E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.58890E+00_JPRB, 0.58900E+00_JPRB, 0.58492E+00_JPRB, 0.57629E+00_JPRB, 0.57141E+00_JPRB, 0.56593E+00_JPRB, 0.55658E+00_JPRB ,&
&  0.54569E+00_JPRB, 0.53584E+00_JPRB, 0.52610E+00_JPRB, 0.51353E+00_JPRB, 0.49990E+00_JPRB, 0.49431E+00_JPRB, 0.49230E+00_JPRB ,&
&  0.48786E+00_JPRB, 0.47961E+00_JPRB, 0.47185E+00_JPRB, 0.46982E+00_JPRB, 0.46885E+00_JPRB, 0.46812E+00_JPRB, 0.46475E+00_JPRB ,&
&  0.45934E+00_JPRB, 0.45850E+00_JPRB, 0.46184E+00_JPRB, 0.46271E+00_JPRB, 0.45884E+00_JPRB, 0.46077E+00_JPRB, 0.47486E+00_JPRB ,&
&  0.54559E+00_JPRB, 0.51244E+00_JPRB, 0.57088E+00_JPRB, 0.67180E+00_JPRB, 0.64707E+00_JPRB, 0.63453E+00_JPRB, 0.60411E+00_JPRB ,&
&  0.57306E+00_JPRB, 0.55682E+00_JPRB, 0.54412E+00_JPRB, 0.52630E+00_JPRB, 0.50815E+00_JPRB, 0.49821E+00_JPRB, 0.49763E+00_JPRB ,&
&  0.50218E+00_JPRB, 0.50449E+00_JPRB, 0.50508E+00_JPRB, 0.50000E+00_JPRB, 0.49684E+00_JPRB, 0.49452E+00_JPRB, 0.49335E+00_JPRB ,&
&  0.49279E+00_JPRB, 0.49583E+00_JPRB, 0.50106E+00_JPRB, 0.50761E+00_JPRB, 0.53104E+00_JPRB, 0.60602E+00_JPRB, 0.72879E+00_JPRB ,&
&  0.84024E+00_JPRB, 0.93239E+00_JPRB, 0.10187E+01_JPRB, 0.11070E+01_JPRB, 0.11729E+01_JPRB, 0.12310E+01_JPRB, 0.12604E+01_JPRB ,&
&  0.12406E+01_JPRB, 0.11810E+01_JPRB, 0.11004E+01_JPRB, 0.10062E+01_JPRB, 0.91912E+00_JPRB, 0.84946E+00_JPRB, 0.79611E+00_JPRB ,&
&  0.75585E+00_JPRB, 0.72940E+00_JPRB, 0.70864E+00_JPRB, 0.68677E+00_JPRB, 0.67448E+00_JPRB, 0.67338E+00_JPRB, 0.68284E+00_JPRB ,&
&  0.70208E+00_JPRB, 0.72335E+00_JPRB, 0.73668E+00_JPRB, 0.73897E+00_JPRB, 0.73085E+00_JPRB, 0.72224E+00_JPRB, 0.71389E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.72252E+00_JPRB, 0.72942E+00_JPRB, 0.70028E+00_JPRB, 0.70295E+00_JPRB, 0.68722E+00_JPRB, 0.65145E+00_JPRB ,&
&  0.61627E+00_JPRB, 0.59153E+00_JPRB, 0.57655E+00_JPRB, 0.51295E+00_JPRB, 0.58455E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.55724E+00_JPRB, 0.55971E+00_JPRB, 0.56059E+00_JPRB, 0.56042E+00_JPRB, 0.55874E+00_JPRB, 0.55298E+00_JPRB, 0.54277E+00_JPRB ,&
&  0.53204E+00_JPRB, 0.52151E+00_JPRB, 0.51351E+00_JPRB, 0.50515E+00_JPRB, 0.49266E+00_JPRB, 0.48422E+00_JPRB, 0.48214E+00_JPRB ,&
&  0.47984E+00_JPRB, 0.47424E+00_JPRB, 0.46791E+00_JPRB, 0.46676E+00_JPRB, 0.46460E+00_JPRB, 0.46148E+00_JPRB, 0.45719E+00_JPRB ,&
&  0.45282E+00_JPRB, 0.45143E+00_JPRB, 0.45189E+00_JPRB, 0.45300E+00_JPRB, 0.44991E+00_JPRB, 0.45152E+00_JPRB, 0.46565E+00_JPRB ,&
&  0.52093E+00_JPRB, 0.45957E+00_JPRB, 0.51238E+00_JPRB, 0.56932E+00_JPRB, 0.54199E+00_JPRB, 0.51991E+00_JPRB, 0.50110E+00_JPRB ,&
&  0.48734E+00_JPRB, 0.48128E+00_JPRB, 0.47814E+00_JPRB, 0.47351E+00_JPRB, 0.46904E+00_JPRB, 0.46378E+00_JPRB, 0.46237E+00_JPRB ,&
&  0.46353E+00_JPRB, 0.46939E+00_JPRB, 0.47273E+00_JPRB, 0.47287E+00_JPRB, 0.46935E+00_JPRB, 0.46558E+00_JPRB, 0.46233E+00_JPRB ,&
&  0.46086E+00_JPRB, 0.46207E+00_JPRB, 0.46481E+00_JPRB, 0.47006E+00_JPRB, 0.48209E+00_JPRB, 0.51422E+00_JPRB, 0.58222E+00_JPRB ,&
&  0.67110E+00_JPRB, 0.74473E+00_JPRB, 0.81411E+00_JPRB, 0.89903E+00_JPRB, 0.98183E+00_JPRB, 0.10408E+01_JPRB, 0.10596E+01_JPRB ,&
&  0.10391E+01_JPRB, 0.98603E+00_JPRB, 0.93694E+00_JPRB, 0.88542E+00_JPRB, 0.83466E+00_JPRB, 0.79087E+00_JPRB, 0.75966E+00_JPRB ,&
&  0.73880E+00_JPRB, 0.72521E+00_JPRB, 0.71793E+00_JPRB, 0.71375E+00_JPRB, 0.71502E+00_JPRB, 0.71827E+00_JPRB, 0.72333E+00_JPRB ,&
&  0.72055E+00_JPRB, 0.71318E+00_JPRB, 0.69273E+00_JPRB, 0.66370E+00_JPRB, 0.64313E+00_JPRB, 0.63238E+00_JPRB, 0.62775E+00_JPRB ,&
&  0.62984E+00_JPRB, 0.63909E+00_JPRB, 0.65200E+00_JPRB, 0.67207E+00_JPRB, 0.67483E+00_JPRB, 0.67608E+00_JPRB, 0.66189E+00_JPRB ,&
&  0.63197E+00_JPRB, 0.60081E+00_JPRB, 0.49528E+00_JPRB, 0.54491E+00_JPRB, 0.55866E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.56421E+00_JPRB, 0.55863E+00_JPRB, 0.55359E+00_JPRB, 0.54841E+00_JPRB, 0.54160E+00_JPRB, 0.53291E+00_JPRB, 0.52263E+00_JPRB ,&
&  0.51295E+00_JPRB, 0.50376E+00_JPRB, 0.49455E+00_JPRB, 0.48646E+00_JPRB, 0.47879E+00_JPRB, 0.47497E+00_JPRB, 0.47372E+00_JPRB ,&
&  0.47379E+00_JPRB, 0.47270E+00_JPRB, 0.47076E+00_JPRB, 0.47049E+00_JPRB, 0.46988E+00_JPRB, 0.46715E+00_JPRB, 0.46332E+00_JPRB ,&
&  0.45726E+00_JPRB, 0.45207E+00_JPRB, 0.45014E+00_JPRB, 0.45247E+00_JPRB, 0.45142E+00_JPRB, 0.45363E+00_JPRB, 0.46248E+00_JPRB ,&
&  0.44982E+00_JPRB, 0.42606E+00_JPRB, 0.46308E+00_JPRB, 0.50570E+00_JPRB, 0.48596E+00_JPRB, 0.46772E+00_JPRB, 0.46037E+00_JPRB ,&
&  0.45339E+00_JPRB, 0.44956E+00_JPRB, 0.44689E+00_JPRB, 0.44599E+00_JPRB, 0.44730E+00_JPRB, 0.44705E+00_JPRB, 0.44567E+00_JPRB ,&
&  0.44500E+00_JPRB, 0.44806E+00_JPRB, 0.45142E+00_JPRB, 0.45347E+00_JPRB, 0.45333E+00_JPRB, 0.45181E+00_JPRB, 0.44778E+00_JPRB ,&
&  0.44404E+00_JPRB, 0.44188E+00_JPRB, 0.44205E+00_JPRB, 0.44607E+00_JPRB, 0.45363E+00_JPRB, 0.46912E+00_JPRB, 0.50205E+00_JPRB ,&
&  0.55463E+00_JPRB, 0.61062E+00_JPRB, 0.66255E+00_JPRB, 0.72324E+00_JPRB, 0.79928E+00_JPRB, 0.87114E+00_JPRB, 0.91698E+00_JPRB ,&
&  0.91369E+00_JPRB, 0.87230E+00_JPRB, 0.82000E+00_JPRB, 0.77511E+00_JPRB, 0.74119E+00_JPRB, 0.71952E+00_JPRB, 0.70685E+00_JPRB ,&
&  0.70227E+00_JPRB, 0.70000E+00_JPRB, 0.69939E+00_JPRB, 0.69974E+00_JPRB, 0.69932E+00_JPRB, 0.69433E+00_JPRB, 0.68478E+00_JPRB ,&
&  0.66769E+00_JPRB, 0.64099E+00_JPRB, 0.61073E+00_JPRB, 0.58627E+00_JPRB, 0.56945E+00_JPRB, 0.56132E+00_JPRB, 0.56004E+00_JPRB ,&
&  0.56503E+00_JPRB, 0.57770E+00_JPRB, 0.59526E+00_JPRB, 0.62498E+00_JPRB, 0.64618E+00_JPRB, 0.66000E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.64898E+00_JPRB, 0.62187E+00_JPRB, 0.59530E+00_JPRB, 0.58030E+00_JPRB, 0.57360E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.56344E+00_JPRB, 0.54814E+00_JPRB, 0.53325E+00_JPRB, 0.52300E+00_JPRB, 0.51512E+00_JPRB, 0.50745E+00_JPRB, 0.49880E+00_JPRB ,&
&  0.48860E+00_JPRB, 0.47850E+00_JPRB, 0.46923E+00_JPRB, 0.46066E+00_JPRB, 0.45633E+00_JPRB, 0.45421E+00_JPRB, 0.45452E+00_JPRB ,&
&  0.45708E+00_JPRB, 0.46164E+00_JPRB, 0.46321E+00_JPRB, 0.46258E+00_JPRB, 0.46310E+00_JPRB, 0.46277E+00_JPRB, 0.46076E+00_JPRB ,&
&  0.45623E+00_JPRB, 0.45080E+00_JPRB, 0.44776E+00_JPRB, 0.44835E+00_JPRB, 0.44850E+00_JPRB, 0.44932E+00_JPRB, 0.44986E+00_JPRB ,&
&  0.45403E+00_JPRB, 0.41924E+00_JPRB, 0.44709E+00_JPRB, 0.47059E+00_JPRB, 0.46355E+00_JPRB, 0.45623E+00_JPRB, 0.44893E+00_JPRB ,&
&  0.44069E+00_JPRB, 0.43626E+00_JPRB, 0.43407E+00_JPRB, 0.43560E+00_JPRB, 0.43888E+00_JPRB, 0.44048E+00_JPRB, 0.44075E+00_JPRB ,&
&  0.43795E+00_JPRB, 0.43802E+00_JPRB, 0.43745E+00_JPRB, 0.43753E+00_JPRB, 0.43847E+00_JPRB, 0.43962E+00_JPRB, 0.43971E+00_JPRB ,&
&  0.43814E+00_JPRB, 0.43492E+00_JPRB, 0.43175E+00_JPRB, 0.43146E+00_JPRB, 0.43317E+00_JPRB, 0.43803E+00_JPRB, 0.44965E+00_JPRB ,&
&  0.47181E+00_JPRB, 0.50033E+00_JPRB, 0.53505E+00_JPRB, 0.58014E+00_JPRB, 0.64215E+00_JPRB, 0.70632E+00_JPRB, 0.75657E+00_JPRB ,&
&  0.77447E+00_JPRB, 0.76428E+00_JPRB, 0.73884E+00_JPRB, 0.70652E+00_JPRB, 0.67630E+00_JPRB, 0.64787E+00_JPRB, 0.62324E+00_JPRB ,&
&  0.60523E+00_JPRB, 0.59152E+00_JPRB, 0.58157E+00_JPRB, 0.57522E+00_JPRB, 0.56891E+00_JPRB, 0.55988E+00_JPRB, 0.55072E+00_JPRB ,&
&  0.54035E+00_JPRB, 0.53056E+00_JPRB, 0.52268E+00_JPRB, 0.51576E+00_JPRB, 0.50900E+00_JPRB, 0.50353E+00_JPRB, 0.50220E+00_JPRB ,&
&  0.50653E+00_JPRB, 0.51580E+00_JPRB, 0.53216E+00_JPRB, 0.55498E+00_JPRB, 0.58119E+00_JPRB, 0.60221E+00_JPRB, 0.61472E+00_JPRB ,&
&  0.61635E+00_JPRB, 0.61199E+00_JPRB, 0.60028E+00_JPRB, 0.58780E+00_JPRB, 0.57707E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.54098E+00_JPRB, 0.52840E+00_JPRB, 0.51578E+00_JPRB, 0.50560E+00_JPRB, 0.49740E+00_JPRB, 0.48849E+00_JPRB, 0.47847E+00_JPRB ,&
&  0.47015E+00_JPRB, 0.46139E+00_JPRB, 0.45498E+00_JPRB, 0.44960E+00_JPRB, 0.44571E+00_JPRB, 0.44446E+00_JPRB, 0.44519E+00_JPRB ,&
&  0.44779E+00_JPRB, 0.45338E+00_JPRB, 0.45601E+00_JPRB, 0.45427E+00_JPRB, 0.45590E+00_JPRB, 0.45815E+00_JPRB, 0.45852E+00_JPRB ,&
&  0.45588E+00_JPRB, 0.45243E+00_JPRB, 0.44869E+00_JPRB, 0.44641E+00_JPRB, 0.44621E+00_JPRB, 0.44721E+00_JPRB, 0.44417E+00_JPRB ,&
&  0.43875E+00_JPRB, 0.43314E+00_JPRB, 0.44519E+00_JPRB, 0.45369E+00_JPRB, 0.45361E+00_JPRB, 0.44870E+00_JPRB, 0.44260E+00_JPRB ,&
&  0.43488E+00_JPRB, 0.43076E+00_JPRB, 0.43142E+00_JPRB, 0.43385E+00_JPRB, 0.43660E+00_JPRB, 0.43799E+00_JPRB, 0.43938E+00_JPRB ,&
&  0.43626E+00_JPRB, 0.43460E+00_JPRB, 0.43263E+00_JPRB, 0.43136E+00_JPRB, 0.43221E+00_JPRB, 0.43349E+00_JPRB, 0.43437E+00_JPRB ,&
&  0.43358E+00_JPRB, 0.43434E+00_JPRB, 0.43518E+00_JPRB, 0.43437E+00_JPRB, 0.43580E+00_JPRB, 0.43813E+00_JPRB, 0.44196E+00_JPRB ,&
&  0.44934E+00_JPRB, 0.46130E+00_JPRB, 0.47660E+00_JPRB, 0.49981E+00_JPRB, 0.53804E+00_JPRB, 0.58223E+00_JPRB, 0.61796E+00_JPRB ,&
&  0.63667E+00_JPRB, 0.63951E+00_JPRB, 0.63184E+00_JPRB, 0.61706E+00_JPRB, 0.60189E+00_JPRB, 0.58674E+00_JPRB, 0.56945E+00_JPRB ,&
&  0.55090E+00_JPRB, 0.53647E+00_JPRB, 0.52450E+00_JPRB, 0.51521E+00_JPRB, 0.50702E+00_JPRB, 0.50001E+00_JPRB, 0.49538E+00_JPRB ,&
&  0.49302E+00_JPRB, 0.49124E+00_JPRB, 0.48799E+00_JPRB, 0.48433E+00_JPRB, 0.48204E+00_JPRB, 0.47843E+00_JPRB, 0.47643E+00_JPRB ,&
&  0.47816E+00_JPRB, 0.48382E+00_JPRB, 0.49377E+00_JPRB, 0.50963E+00_JPRB, 0.52673E+00_JPRB, 0.54191E+00_JPRB, 0.55422E+00_JPRB ,&
&  0.56327E+00_JPRB, 0.56756E+00_JPRB, 0.56620E+00_JPRB, 0.56131E+00_JPRB, 0.55293E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.49568E+00_JPRB, 0.48948E+00_JPRB, 0.48187E+00_JPRB, 0.47413E+00_JPRB, 0.46672E+00_JPRB, 0.45926E+00_JPRB, 0.45247E+00_JPRB ,&
&  0.44671E+00_JPRB, 0.44323E+00_JPRB, 0.44111E+00_JPRB, 0.43950E+00_JPRB, 0.43609E+00_JPRB, 0.43460E+00_JPRB, 0.43538E+00_JPRB ,&
&  0.43793E+00_JPRB, 0.44051E+00_JPRB, 0.44213E+00_JPRB, 0.44137E+00_JPRB, 0.44354E+00_JPRB, 0.44561E+00_JPRB, 0.44566E+00_JPRB ,&
&  0.44459E+00_JPRB, 0.44348E+00_JPRB, 0.44107E+00_JPRB, 0.43817E+00_JPRB, 0.43793E+00_JPRB, 0.43812E+00_JPRB, 0.43420E+00_JPRB ,&
&  0.42665E+00_JPRB, 0.42925E+00_JPRB, 0.42151E+00_JPRB, 0.43621E+00_JPRB, 0.43979E+00_JPRB, 0.43429E+00_JPRB, 0.42746E+00_JPRB ,&
&  0.42484E+00_JPRB, 0.42563E+00_JPRB, 0.42910E+00_JPRB, 0.43168E+00_JPRB, 0.43276E+00_JPRB, 0.43317E+00_JPRB, 0.43308E+00_JPRB ,&
&  0.42542E+00_JPRB, 0.42092E+00_JPRB, 0.42050E+00_JPRB, 0.42139E+00_JPRB, 0.42153E+00_JPRB, 0.42222E+00_JPRB, 0.42413E+00_JPRB ,&
&  0.42566E+00_JPRB, 0.42668E+00_JPRB, 0.42779E+00_JPRB, 0.43033E+00_JPRB, 0.43206E+00_JPRB, 0.43287E+00_JPRB, 0.43477E+00_JPRB ,&
&  0.43803E+00_JPRB, 0.44312E+00_JPRB, 0.44935E+00_JPRB, 0.45746E+00_JPRB, 0.47464E+00_JPRB, 0.50003E+00_JPRB, 0.52293E+00_JPRB ,&
&  0.53593E+00_JPRB, 0.54069E+00_JPRB, 0.54025E+00_JPRB, 0.53388E+00_JPRB, 0.52356E+00_JPRB, 0.51410E+00_JPRB, 0.50528E+00_JPRB ,&
&  0.49403E+00_JPRB, 0.48584E+00_JPRB, 0.47842E+00_JPRB, 0.46957E+00_JPRB, 0.46160E+00_JPRB, 0.45587E+00_JPRB, 0.45245E+00_JPRB ,&
&  0.45138E+00_JPRB, 0.45239E+00_JPRB, 0.45218E+00_JPRB, 0.45266E+00_JPRB, 0.45504E+00_JPRB, 0.45807E+00_JPRB, 0.45785E+00_JPRB ,&
&  0.45650E+00_JPRB, 0.45630E+00_JPRB, 0.45874E+00_JPRB, 0.46391E+00_JPRB, 0.46994E+00_JPRB, 0.47771E+00_JPRB, 0.48620E+00_JPRB ,&
&  0.49415E+00_JPRB, 0.49987E+00_JPRB, 0.50256E+00_JPRB, 0.50334E+00_JPRB, 0.50064E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.45029E+00_JPRB, 0.44838E+00_JPRB, 0.44585E+00_JPRB, 0.44195E+00_JPRB, 0.43768E+00_JPRB, 0.43391E+00_JPRB, 0.43089E+00_JPRB ,&
&  0.42898E+00_JPRB, 0.42846E+00_JPRB, 0.42897E+00_JPRB, 0.42917E+00_JPRB, 0.42908E+00_JPRB, 0.42999E+00_JPRB, 0.43044E+00_JPRB ,&
&  0.43211E+00_JPRB, 0.43246E+00_JPRB, 0.43002E+00_JPRB, 0.42936E+00_JPRB, 0.43178E+00_JPRB, 0.43645E+00_JPRB, 0.43807E+00_JPRB ,&
&  0.43556E+00_JPRB, 0.43505E+00_JPRB, 0.43455E+00_JPRB, 0.43216E+00_JPRB, 0.43033E+00_JPRB, 0.42912E+00_JPRB, 0.42505E+00_JPRB ,&
&  0.41951E+00_JPRB, 0.42007E+00_JPRB, 0.42166E+00_JPRB, 0.42567E+00_JPRB, 0.42609E+00_JPRB, 0.42477E+00_JPRB, 0.42533E+00_JPRB ,&
&  0.42368E+00_JPRB, 0.42357E+00_JPRB, 0.42349E+00_JPRB, 0.42411E+00_JPRB, 0.42241E+00_JPRB, 0.41966E+00_JPRB, 0.41444E+00_JPRB ,&
&  0.40473E+00_JPRB, 0.40150E+00_JPRB, 0.40356E+00_JPRB, 0.40799E+00_JPRB, 0.40744E+00_JPRB, 0.40805E+00_JPRB, 0.41114E+00_JPRB ,&
&  0.41508E+00_JPRB, 0.41795E+00_JPRB, 0.41973E+00_JPRB, 0.42088E+00_JPRB, 0.42251E+00_JPRB, 0.42405E+00_JPRB, 0.42489E+00_JPRB ,&
&  0.42599E+00_JPRB, 0.42801E+00_JPRB, 0.43110E+00_JPRB, 0.43545E+00_JPRB, 0.44190E+00_JPRB, 0.45286E+00_JPRB, 0.46448E+00_JPRB ,&
&  0.47200E+00_JPRB, 0.47409E+00_JPRB, 0.47363E+00_JPRB, 0.46985E+00_JPRB, 0.46309E+00_JPRB, 0.45644E+00_JPRB, 0.45017E+00_JPRB ,&
&  0.44155E+00_JPRB, 0.43396E+00_JPRB, 0.42783E+00_JPRB, 0.42422E+00_JPRB, 0.42216E+00_JPRB, 0.42153E+00_JPRB, 0.42226E+00_JPRB ,&
&  0.42310E+00_JPRB, 0.42559E+00_JPRB, 0.43156E+00_JPRB, 0.43504E+00_JPRB, 0.43651E+00_JPRB, 0.43954E+00_JPRB, 0.44119E+00_JPRB ,&
&  0.43965E+00_JPRB, 0.43834E+00_JPRB, 0.43765E+00_JPRB, 0.44009E+00_JPRB, 0.44188E+00_JPRB, 0.44191E+00_JPRB, 0.44258E+00_JPRB ,&
&  0.44582E+00_JPRB, 0.44945E+00_JPRB, 0.45061E+00_JPRB, 0.45107E+00_JPRB, 0.45155E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.41326E+00_JPRB, 0.41335E+00_JPRB, 0.41274E+00_JPRB, 0.41163E+00_JPRB, 0.41053E+00_JPRB, 0.40968E+00_JPRB, 0.40985E+00_JPRB ,&
&  0.41017E+00_JPRB, 0.41099E+00_JPRB, 0.41209E+00_JPRB, 0.41300E+00_JPRB, 0.41482E+00_JPRB, 0.41693E+00_JPRB, 0.41869E+00_JPRB ,&
&  0.42054E+00_JPRB, 0.41999E+00_JPRB, 0.41709E+00_JPRB, 0.41422E+00_JPRB, 0.41491E+00_JPRB, 0.42032E+00_JPRB, 0.42623E+00_JPRB ,&
&  0.42752E+00_JPRB, 0.42831E+00_JPRB, 0.42850E+00_JPRB, 0.42702E+00_JPRB, 0.42508E+00_JPRB, 0.42437E+00_JPRB, 0.42058E+00_JPRB ,&
&  0.41563E+00_JPRB, 0.41413E+00_JPRB, 0.41350E+00_JPRB, 0.41128E+00_JPRB, 0.41151E+00_JPRB, 0.41471E+00_JPRB, 0.41455E+00_JPRB ,&
&  0.41414E+00_JPRB, 0.41258E+00_JPRB, 0.40943E+00_JPRB, 0.40640E+00_JPRB, 0.40269E+00_JPRB, 0.39581E+00_JPRB, 0.38788E+00_JPRB ,&
&  0.38166E+00_JPRB, 0.38174E+00_JPRB, 0.38720E+00_JPRB, 0.39284E+00_JPRB, 0.39158E+00_JPRB, 0.39029E+00_JPRB, 0.39172E+00_JPRB ,&
&  0.39534E+00_JPRB, 0.39999E+00_JPRB, 0.40299E+00_JPRB, 0.40227E+00_JPRB, 0.40118E+00_JPRB, 0.40309E+00_JPRB, 0.40579E+00_JPRB ,&
&  0.40704E+00_JPRB, 0.40841E+00_JPRB, 0.40947E+00_JPRB, 0.41137E+00_JPRB, 0.41317E+00_JPRB, 0.41492E+00_JPRB, 0.41873E+00_JPRB ,&
&  0.42089E+00_JPRB, 0.42092E+00_JPRB, 0.41803E+00_JPRB, 0.41262E+00_JPRB, 0.40736E+00_JPRB, 0.40273E+00_JPRB, 0.40056E+00_JPRB ,&
&  0.39802E+00_JPRB, 0.39792E+00_JPRB, 0.39718E+00_JPRB, 0.39426E+00_JPRB, 0.39091E+00_JPRB, 0.39060E+00_JPRB, 0.39406E+00_JPRB ,&
&  0.39907E+00_JPRB, 0.40297E+00_JPRB, 0.40893E+00_JPRB, 0.41152E+00_JPRB, 0.41320E+00_JPRB, 0.41568E+00_JPRB, 0.41651E+00_JPRB ,&
&  0.41403E+00_JPRB, 0.41196E+00_JPRB, 0.41156E+00_JPRB, 0.41457E+00_JPRB, 0.41550E+00_JPRB, 0.41436E+00_JPRB, 0.41115E+00_JPRB ,&
&  0.40991E+00_JPRB, 0.41145E+00_JPRB, 0.41192E+00_JPRB, 0.41144E+00_JPRB, 0.41195E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.38827E+00_JPRB, 0.39042E+00_JPRB, 0.39272E+00_JPRB, 0.39480E+00_JPRB, 0.39614E+00_JPRB, 0.39714E+00_JPRB, 0.39636E+00_JPRB ,&
&  0.39629E+00_JPRB, 0.39805E+00_JPRB, 0.40026E+00_JPRB, 0.40209E+00_JPRB, 0.40321E+00_JPRB, 0.40532E+00_JPRB, 0.40752E+00_JPRB ,&
&  0.40901E+00_JPRB, 0.40819E+00_JPRB, 0.40456E+00_JPRB, 0.40269E+00_JPRB, 0.40151E+00_JPRB, 0.40664E+00_JPRB, 0.41528E+00_JPRB ,&
&  0.41901E+00_JPRB, 0.42121E+00_JPRB, 0.42390E+00_JPRB, 0.42423E+00_JPRB, 0.42414E+00_JPRB, 0.42364E+00_JPRB, 0.41900E+00_JPRB ,&
&  0.41378E+00_JPRB, 0.40894E+00_JPRB, 0.40574E+00_JPRB, 0.40242E+00_JPRB, 0.40260E+00_JPRB, 0.40076E+00_JPRB, 0.39961E+00_JPRB ,&
&  0.39872E+00_JPRB, 0.39621E+00_JPRB, 0.39272E+00_JPRB, 0.38695E+00_JPRB, 0.37987E+00_JPRB, 0.37068E+00_JPRB, 0.36430E+00_JPRB ,&
&  0.36181E+00_JPRB, 0.36561E+00_JPRB, 0.37427E+00_JPRB, 0.37797E+00_JPRB, 0.37433E+00_JPRB, 0.37321E+00_JPRB, 0.37629E+00_JPRB ,&
&  0.38118E+00_JPRB, 0.38670E+00_JPRB, 0.38884E+00_JPRB, 0.38810E+00_JPRB, 0.38610E+00_JPRB, 0.38769E+00_JPRB, 0.39201E+00_JPRB ,&
&  0.39575E+00_JPRB, 0.39765E+00_JPRB, 0.39680E+00_JPRB, 0.39594E+00_JPRB, 0.39379E+00_JPRB, 0.38909E+00_JPRB, 0.38282E+00_JPRB ,&
&  0.37711E+00_JPRB, 0.37348E+00_JPRB, 0.37064E+00_JPRB, 0.36754E+00_JPRB, 0.36617E+00_JPRB, 0.36625E+00_JPRB, 0.36822E+00_JPRB ,&
&  0.36887E+00_JPRB, 0.37054E+00_JPRB, 0.37055E+00_JPRB, 0.36632E+00_JPRB, 0.36302E+00_JPRB, 0.36470E+00_JPRB, 0.36671E+00_JPRB ,&
&  0.36968E+00_JPRB, 0.37256E+00_JPRB, 0.37539E+00_JPRB, 0.37641E+00_JPRB, 0.37793E+00_JPRB, 0.38069E+00_JPRB, 0.38239E+00_JPRB ,&
&  0.38325E+00_JPRB, 0.38383E+00_JPRB, 0.38568E+00_JPRB, 0.38889E+00_JPRB, 0.39196E+00_JPRB, 0.39444E+00_JPRB, 0.39379E+00_JPRB ,&
&  0.39223E+00_JPRB, 0.39026E+00_JPRB, 0.39024E+00_JPRB, 0.39014E+00_JPRB, 0.38837E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.38246E+00_JPRB, 0.38212E+00_JPRB, 0.38527E+00_JPRB, 0.38870E+00_JPRB, 0.38938E+00_JPRB, 0.38894E+00_JPRB, 0.38626E+00_JPRB ,&
&  0.38447E+00_JPRB, 0.38561E+00_JPRB, 0.38741E+00_JPRB, 0.38876E+00_JPRB, 0.39103E+00_JPRB, 0.39417E+00_JPRB, 0.39669E+00_JPRB ,&
&  0.39685E+00_JPRB, 0.39496E+00_JPRB, 0.39173E+00_JPRB, 0.39209E+00_JPRB, 0.39196E+00_JPRB, 0.39415E+00_JPRB, 0.40181E+00_JPRB ,&
&  0.40729E+00_JPRB, 0.41096E+00_JPRB, 0.41402E+00_JPRB, 0.41427E+00_JPRB, 0.41346E+00_JPRB, 0.41474E+00_JPRB, 0.41116E+00_JPRB ,&
&  0.40474E+00_JPRB, 0.39881E+00_JPRB, 0.39319E+00_JPRB, 0.38624E+00_JPRB, 0.38889E+00_JPRB, 0.38642E+00_JPRB, 0.38338E+00_JPRB ,&
&  0.37913E+00_JPRB, 0.37351E+00_JPRB, 0.36689E+00_JPRB, 0.36064E+00_JPRB, 0.35599E+00_JPRB, 0.35169E+00_JPRB, 0.34629E+00_JPRB ,&
&  0.34462E+00_JPRB, 0.34697E+00_JPRB, 0.35267E+00_JPRB, 0.35478E+00_JPRB, 0.35170E+00_JPRB, 0.35226E+00_JPRB, 0.35895E+00_JPRB ,&
&  0.36606E+00_JPRB, 0.36955E+00_JPRB, 0.36952E+00_JPRB, 0.37013E+00_JPRB, 0.37074E+00_JPRB, 0.37214E+00_JPRB, 0.37283E+00_JPRB ,&
&  0.37174E+00_JPRB, 0.37000E+00_JPRB, 0.36739E+00_JPRB, 0.36152E+00_JPRB, 0.35474E+00_JPRB, 0.34896E+00_JPRB, 0.34696E+00_JPRB ,&
&  0.34998E+00_JPRB, 0.35033E+00_JPRB, 0.35110E+00_JPRB, 0.35315E+00_JPRB, 0.35384E+00_JPRB, 0.35315E+00_JPRB, 0.35223E+00_JPRB ,&
&  0.35360E+00_JPRB, 0.35241E+00_JPRB, 0.35344E+00_JPRB, 0.35058E+00_JPRB, 0.34618E+00_JPRB, 0.34196E+00_JPRB, 0.33791E+00_JPRB ,&
&  0.33575E+00_JPRB, 0.33529E+00_JPRB, 0.33746E+00_JPRB, 0.33821E+00_JPRB, 0.34005E+00_JPRB, 0.34229E+00_JPRB, 0.34419E+00_JPRB ,&
&  0.34533E+00_JPRB, 0.34692E+00_JPRB, 0.34989E+00_JPRB, 0.35567E+00_JPRB, 0.36177E+00_JPRB, 0.36715E+00_JPRB, 0.37250E+00_JPRB ,&
&  0.37737E+00_JPRB, 0.38070E+00_JPRB, 0.38645E+00_JPRB, 0.38730E+00_JPRB, 0.38604E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.38103E+00_JPRB, 0.37822E+00_JPRB, 0.37801E+00_JPRB, 0.37859E+00_JPRB, 0.37771E+00_JPRB, 0.37652E+00_JPRB, 0.37360E+00_JPRB ,&
&  0.37106E+00_JPRB, 0.37137E+00_JPRB, 0.37276E+00_JPRB, 0.37540E+00_JPRB, 0.38005E+00_JPRB, 0.38500E+00_JPRB, 0.38460E+00_JPRB ,&
&  0.38162E+00_JPRB, 0.38042E+00_JPRB, 0.38274E+00_JPRB, 0.38474E+00_JPRB, 0.38495E+00_JPRB, 0.38559E+00_JPRB, 0.38934E+00_JPRB ,&
&  0.39400E+00_JPRB, 0.39610E+00_JPRB, 0.39902E+00_JPRB, 0.39851E+00_JPRB, 0.39453E+00_JPRB, 0.39454E+00_JPRB, 0.39260E+00_JPRB ,&
&  0.38760E+00_JPRB, 0.38134E+00_JPRB, 0.37206E+00_JPRB, 0.37281E+00_JPRB, 0.36911E+00_JPRB, 0.36603E+00_JPRB, 0.36138E+00_JPRB ,&
&  0.35620E+00_JPRB, 0.35173E+00_JPRB, 0.34676E+00_JPRB, 0.34277E+00_JPRB, 0.34084E+00_JPRB, 0.34076E+00_JPRB, 0.33864E+00_JPRB ,&
&  0.33745E+00_JPRB, 0.33953E+00_JPRB, 0.34077E+00_JPRB, 0.33715E+00_JPRB, 0.33637E+00_JPRB, 0.34250E+00_JPRB, 0.34777E+00_JPRB ,&
&  0.35000E+00_JPRB, 0.35010E+00_JPRB, 0.34743E+00_JPRB, 0.34674E+00_JPRB, 0.34569E+00_JPRB, 0.34370E+00_JPRB, 0.34024E+00_JPRB ,&
&  0.33497E+00_JPRB, 0.33240E+00_JPRB, 0.33070E+00_JPRB, 0.32724E+00_JPRB, 0.32111E+00_JPRB, 0.30472E+00_JPRB, 0.29571E+00_JPRB ,&
&  0.29638E+00_JPRB, 0.30556E+00_JPRB, 0.32019E+00_JPRB, 0.32752E+00_JPRB, 0.33419E+00_JPRB, 0.33951E+00_JPRB, 0.34010E+00_JPRB ,&
&  0.34052E+00_JPRB, 0.32053E+00_JPRB, 0.30949E+00_JPRB, 0.30037E+00_JPRB, 0.29236E+00_JPRB, 0.28756E+00_JPRB, 0.28610E+00_JPRB ,&
&  0.28915E+00_JPRB, 0.29510E+00_JPRB, 0.30073E+00_JPRB, 0.30359E+00_JPRB, 0.30344E+00_JPRB, 0.30060E+00_JPRB, 0.29736E+00_JPRB ,&
&  0.29749E+00_JPRB, 0.30481E+00_JPRB, 0.31855E+00_JPRB, 0.33140E+00_JPRB, 0.34079E+00_JPRB, 0.35088E+00_JPRB, 0.35930E+00_JPRB ,&
&  0.36521E+00_JPRB, 0.37078E+00_JPRB, 0.37749E+00_JPRB, 0.37995E+00_JPRB, 0.38095E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.36803E+00_JPRB, 0.36404E+00_JPRB, 0.36390E+00_JPRB, 0.36412E+00_JPRB, 0.36278E+00_JPRB, 0.36246E+00_JPRB, 0.36157E+00_JPRB ,&
&  0.36100E+00_JPRB, 0.35900E+00_JPRB, 0.35715E+00_JPRB, 0.35768E+00_JPRB, 0.35993E+00_JPRB, 0.36249E+00_JPRB, 0.36326E+00_JPRB ,&
&  0.36326E+00_JPRB, 0.36484E+00_JPRB, 0.36841E+00_JPRB, 0.37197E+00_JPRB, 0.37257E+00_JPRB, 0.37279E+00_JPRB, 0.37461E+00_JPRB ,&
&  0.37691E+00_JPRB, 0.37771E+00_JPRB, 0.38050E+00_JPRB, 0.37817E+00_JPRB, 0.37348E+00_JPRB, 0.37157E+00_JPRB, 0.36989E+00_JPRB ,&
&  0.36302E+00_JPRB, 0.33927E+00_JPRB, 0.32411E+00_JPRB, 0.32907E+00_JPRB, 0.34441E+00_JPRB, 0.34347E+00_JPRB, 0.34118E+00_JPRB ,&
&  0.33878E+00_JPRB, 0.33551E+00_JPRB, 0.33274E+00_JPRB, 0.33170E+00_JPRB, 0.33192E+00_JPRB, 0.33204E+00_JPRB, 0.33081E+00_JPRB ,&
&  0.33146E+00_JPRB, 0.33673E+00_JPRB, 0.34273E+00_JPRB, 0.33915E+00_JPRB, 0.33406E+00_JPRB, 0.32664E+00_JPRB, 0.32197E+00_JPRB ,&
&  0.31760E+00_JPRB, 0.31326E+00_JPRB, 0.31044E+00_JPRB, 0.30958E+00_JPRB, 0.30934E+00_JPRB, 0.30724E+00_JPRB, 0.30363E+00_JPRB ,&
&  0.29663E+00_JPRB, 0.28733E+00_JPRB, 0.27790E+00_JPRB, 0.26861E+00_JPRB, 0.26201E+00_JPRB, 0.25729E+00_JPRB, 0.25355E+00_JPRB ,&
&  0.25568E+00_JPRB, 0.26798E+00_JPRB, 0.28747E+00_JPRB, 0.30777E+00_JPRB, 0.32590E+00_JPRB, 0.30418E+00_JPRB, 0.29379E+00_JPRB ,&
&  0.28223E+00_JPRB, 0.27223E+00_JPRB, 0.26526E+00_JPRB, 0.26036E+00_JPRB, 0.25684E+00_JPRB, 0.25488E+00_JPRB, 0.25484E+00_JPRB ,&
&  0.25803E+00_JPRB, 0.26234E+00_JPRB, 0.26634E+00_JPRB, 0.26761E+00_JPRB, 0.26714E+00_JPRB, 0.26541E+00_JPRB, 0.26445E+00_JPRB ,&
&  0.26586E+00_JPRB, 0.27175E+00_JPRB, 0.28130E+00_JPRB, 0.29524E+00_JPRB, 0.30766E+00_JPRB, 0.32042E+00_JPRB, 0.33494E+00_JPRB ,&
&  0.35017E+00_JPRB, 0.36382E+00_JPRB, 0.36745E+00_JPRB, 0.37081E+00_JPRB, 0.37111E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.36102E+00_JPRB, 0.35804E+00_JPRB, 0.35607E+00_JPRB, 0.35639E+00_JPRB, 0.35656E+00_JPRB, 0.35903E+00_JPRB, 0.35787E+00_JPRB ,&
&  0.35324E+00_JPRB, 0.34859E+00_JPRB, 0.34727E+00_JPRB, 0.34732E+00_JPRB, 0.34627E+00_JPRB, 0.34495E+00_JPRB, 0.34582E+00_JPRB ,&
&  0.34436E+00_JPRB, 0.34652E+00_JPRB, 0.34964E+00_JPRB, 0.35130E+00_JPRB, 0.35198E+00_JPRB, 0.35208E+00_JPRB, 0.35232E+00_JPRB ,&
&  0.34605E+00_JPRB, 0.34412E+00_JPRB, 0.34981E+00_JPRB, 0.34687E+00_JPRB, 0.34275E+00_JPRB, 0.33917E+00_JPRB, 0.33782E+00_JPRB ,&
&  0.32888E+00_JPRB, 0.31665E+00_JPRB, 0.30413E+00_JPRB, 0.31062E+00_JPRB, 0.32744E+00_JPRB, 0.32421E+00_JPRB, 0.32188E+00_JPRB ,&
&  0.32161E+00_JPRB, 0.32087E+00_JPRB, 0.32176E+00_JPRB, 0.32235E+00_JPRB, 0.32360E+00_JPRB, 0.32400E+00_JPRB, 0.32554E+00_JPRB ,&
&  0.32726E+00_JPRB, 0.33383E+00_JPRB, 0.33709E+00_JPRB, 0.31655E+00_JPRB, 0.29923E+00_JPRB, 0.28602E+00_JPRB, 0.27330E+00_JPRB ,&
&  0.26200E+00_JPRB, 0.25995E+00_JPRB, 0.25622E+00_JPRB, 0.24925E+00_JPRB, 0.24972E+00_JPRB, 0.25046E+00_JPRB, 0.24959E+00_JPRB ,&
&  0.24738E+00_JPRB, 0.24604E+00_JPRB, 0.24463E+00_JPRB, 0.24223E+00_JPRB, 0.24139E+00_JPRB, 0.24282E+00_JPRB, 0.24676E+00_JPRB ,&
&  0.25284E+00_JPRB, 0.25949E+00_JPRB, 0.27453E+00_JPRB, 0.28722E+00_JPRB, 0.28302E+00_JPRB, 0.27600E+00_JPRB, 0.26879E+00_JPRB ,&
&  0.25741E+00_JPRB, 0.25196E+00_JPRB, 0.24652E+00_JPRB, 0.24209E+00_JPRB, 0.24009E+00_JPRB, 0.24035E+00_JPRB, 0.24120E+00_JPRB ,&
&  0.24248E+00_JPRB, 0.24465E+00_JPRB, 0.24554E+00_JPRB, 0.24523E+00_JPRB, 0.24449E+00_JPRB, 0.24518E+00_JPRB, 0.24705E+00_JPRB ,&
&  0.25036E+00_JPRB, 0.25596E+00_JPRB, 0.26267E+00_JPRB, 0.27101E+00_JPRB, 0.27841E+00_JPRB, 0.28535E+00_JPRB, 0.29620E+00_JPRB ,&
&  0.30443E+00_JPRB, 0.32375E+00_JPRB, 0.35004E+00_JPRB, 0.37362E+00_JPRB, 0.36676E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.34163E+00_JPRB, 0.33925E+00_JPRB, 0.33667E+00_JPRB, 0.33636E+00_JPRB, 0.33714E+00_JPRB, 0.33971E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33229E+00_JPRB, 0.33040E+00_JPRB, 0.33168E+00_JPRB, 0.33138E+00_JPRB, 0.33103E+00_JPRB, 0.32095E+00_JPRB, 0.31525E+00_JPRB ,&
&  0.30832E+00_JPRB, 0.30626E+00_JPRB, 0.30456E+00_JPRB, 0.30552E+00_JPRB, 0.30764E+00_JPRB, 0.30891E+00_JPRB, 0.30457E+00_JPRB ,&
&  0.30007E+00_JPRB, 0.29587E+00_JPRB, 0.29139E+00_JPRB, 0.28946E+00_JPRB, 0.29077E+00_JPRB, 0.29204E+00_JPRB, 0.29206E+00_JPRB ,&
&  0.29278E+00_JPRB, 0.29566E+00_JPRB, 0.29847E+00_JPRB, 0.30220E+00_JPRB, 0.31493E+00_JPRB, 0.31263E+00_JPRB, 0.31132E+00_JPRB ,&
&  0.31174E+00_JPRB, 0.31232E+00_JPRB, 0.31362E+00_JPRB, 0.31459E+00_JPRB, 0.31914E+00_JPRB, 0.32198E+00_JPRB, 0.32415E+00_JPRB ,&
&  0.31063E+00_JPRB, 0.30337E+00_JPRB, 0.29579E+00_JPRB, 0.28637E+00_JPRB, 0.27455E+00_JPRB, 0.26520E+00_JPRB, 0.25660E+00_JPRB ,&
&  0.24937E+00_JPRB, 0.24260E+00_JPRB, 0.23634E+00_JPRB, 0.23391E+00_JPRB, 0.23301E+00_JPRB, 0.23163E+00_JPRB, 0.23148E+00_JPRB ,&
&  0.23207E+00_JPRB, 0.23288E+00_JPRB, 0.23339E+00_JPRB, 0.23502E+00_JPRB, 0.23760E+00_JPRB, 0.24167E+00_JPRB, 0.24603E+00_JPRB ,&
&  0.25248E+00_JPRB, 0.25709E+00_JPRB, 0.26257E+00_JPRB, 0.26737E+00_JPRB, 0.26589E+00_JPRB, 0.26055E+00_JPRB, 0.25489E+00_JPRB ,&
&  0.24783E+00_JPRB, 0.24383E+00_JPRB, 0.23975E+00_JPRB, 0.23682E+00_JPRB, 0.23516E+00_JPRB, 0.23417E+00_JPRB, 0.23443E+00_JPRB ,&
&  0.23450E+00_JPRB, 0.23496E+00_JPRB, 0.23445E+00_JPRB, 0.23473E+00_JPRB, 0.23566E+00_JPRB, 0.23765E+00_JPRB, 0.24028E+00_JPRB ,&
&  0.24433E+00_JPRB, 0.24902E+00_JPRB, 0.25458E+00_JPRB, 0.26067E+00_JPRB, 0.26974E+00_JPRB, 0.28140E+00_JPRB, 0.29768E+00_JPRB ,&
&  0.31398E+00_JPRB, 0.34314E+00_JPRB, 0.35460E+00_JPRB, 0.34773E+00_JPRB, 0.34522E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.32568E+00_JPRB, 0.32709E+00_JPRB, 0.32828E+00_JPRB, 0.33056E+00_JPRB, 0.33327E+00_JPRB, 0.33411E+00_JPRB, 0.33421E+00_JPRB ,&
&  0.32477E+00_JPRB, 0.31731E+00_JPRB, 0.31045E+00_JPRB, 0.30020E+00_JPRB, 0.29204E+00_JPRB, 0.28474E+00_JPRB, 0.27949E+00_JPRB ,&
&  0.27429E+00_JPRB, 0.27180E+00_JPRB, 0.26885E+00_JPRB, 0.26802E+00_JPRB, 0.26761E+00_JPRB, 0.26811E+00_JPRB, 0.26648E+00_JPRB ,&
&  0.26472E+00_JPRB, 0.26598E+00_JPRB, 0.26674E+00_JPRB, 0.27123E+00_JPRB, 0.27791E+00_JPRB, 0.28543E+00_JPRB, 0.29278E+00_JPRB ,&
&  0.30219E+00_JPRB, 0.30607E+00_JPRB, 0.30925E+00_JPRB, 0.31001E+00_JPRB, 0.30885E+00_JPRB, 0.30845E+00_JPRB, 0.30652E+00_JPRB ,&
&  0.30642E+00_JPRB, 0.30616E+00_JPRB, 0.30780E+00_JPRB, 0.30902E+00_JPRB, 0.30748E+00_JPRB, 0.30540E+00_JPRB, 0.30013E+00_JPRB ,&
&  0.29206E+00_JPRB, 0.28559E+00_JPRB, 0.27910E+00_JPRB, 0.27398E+00_JPRB, 0.26654E+00_JPRB, 0.26004E+00_JPRB, 0.25368E+00_JPRB ,&
&  0.24823E+00_JPRB, 0.24418E+00_JPRB, 0.23936E+00_JPRB, 0.23561E+00_JPRB, 0.23382E+00_JPRB, 0.23165E+00_JPRB, 0.23072E+00_JPRB ,&
&  0.23069E+00_JPRB, 0.23114E+00_JPRB, 0.23345E+00_JPRB, 0.23532E+00_JPRB, 0.23824E+00_JPRB, 0.24164E+00_JPRB, 0.24365E+00_JPRB ,&
&  0.24723E+00_JPRB, 0.25032E+00_JPRB, 0.25200E+00_JPRB, 0.25327E+00_JPRB, 0.25229E+00_JPRB, 0.24912E+00_JPRB, 0.24594E+00_JPRB ,&
&  0.24175E+00_JPRB, 0.23885E+00_JPRB, 0.23701E+00_JPRB, 0.23500E+00_JPRB, 0.23398E+00_JPRB, 0.23309E+00_JPRB, 0.23208E+00_JPRB ,&
&  0.23133E+00_JPRB, 0.23099E+00_JPRB, 0.23067E+00_JPRB, 0.23183E+00_JPRB, 0.23307E+00_JPRB, 0.23530E+00_JPRB, 0.23800E+00_JPRB ,&
&  0.24048E+00_JPRB, 0.24431E+00_JPRB, 0.24850E+00_JPRB, 0.25421E+00_JPRB, 0.26469E+00_JPRB, 0.27301E+00_JPRB, 0.29111E+00_JPRB ,&
&  0.30300E+00_JPRB, 0.32666E+00_JPRB, 0.33666E+00_JPRB, 0.32640E+00_JPRB, 0.32563E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.31489E+00_JPRB, 0.31712E+00_JPRB, 0.31795E+00_JPRB, 0.31989E+00_JPRB, 0.32171E+00_JPRB, 0.32227E+00_JPRB, 0.32225E+00_JPRB ,&
&  0.31992E+00_JPRB, 0.31402E+00_JPRB, 0.30938E+00_JPRB, 0.30278E+00_JPRB, 0.29654E+00_JPRB, 0.28978E+00_JPRB, 0.28379E+00_JPRB ,&
&  0.27738E+00_JPRB, 0.27155E+00_JPRB, 0.26559E+00_JPRB, 0.26053E+00_JPRB, 0.25684E+00_JPRB, 0.25378E+00_JPRB, 0.25065E+00_JPRB ,&
&  0.24795E+00_JPRB, 0.24924E+00_JPRB, 0.25203E+00_JPRB, 0.25729E+00_JPRB, 0.26544E+00_JPRB, 0.27283E+00_JPRB, 0.28092E+00_JPRB ,&
&  0.29472E+00_JPRB, 0.29785E+00_JPRB, 0.29790E+00_JPRB, 0.30013E+00_JPRB, 0.29967E+00_JPRB, 0.30026E+00_JPRB, 0.29051E+00_JPRB ,&
&  0.29354E+00_JPRB, 0.29573E+00_JPRB, 0.30386E+00_JPRB, 0.30685E+00_JPRB, 0.30131E+00_JPRB, 0.29711E+00_JPRB, 0.28920E+00_JPRB ,&
&  0.28301E+00_JPRB, 0.27800E+00_JPRB, 0.27466E+00_JPRB, 0.27267E+00_JPRB, 0.26757E+00_JPRB, 0.26380E+00_JPRB, 0.25748E+00_JPRB ,&
&  0.25233E+00_JPRB, 0.24899E+00_JPRB, 0.24479E+00_JPRB, 0.24223E+00_JPRB, 0.24013E+00_JPRB, 0.23807E+00_JPRB, 0.23649E+00_JPRB ,&
&  0.23563E+00_JPRB, 0.23491E+00_JPRB, 0.23564E+00_JPRB, 0.23648E+00_JPRB, 0.23809E+00_JPRB, 0.23939E+00_JPRB, 0.23964E+00_JPRB ,&
&  0.23990E+00_JPRB, 0.24039E+00_JPRB, 0.24056E+00_JPRB, 0.24126E+00_JPRB, 0.24070E+00_JPRB, 0.23967E+00_JPRB, 0.23844E+00_JPRB ,&
&  0.23707E+00_JPRB, 0.23651E+00_JPRB, 0.23555E+00_JPRB, 0.23492E+00_JPRB, 0.23299E+00_JPRB, 0.23170E+00_JPRB, 0.23105E+00_JPRB ,&
&  0.23045E+00_JPRB, 0.23098E+00_JPRB, 0.23119E+00_JPRB, 0.23217E+00_JPRB, 0.23365E+00_JPRB, 0.23496E+00_JPRB, 0.23724E+00_JPRB ,&
&  0.24008E+00_JPRB, 0.24336E+00_JPRB, 0.24898E+00_JPRB, 0.25479E+00_JPRB, 0.26119E+00_JPRB, 0.26884E+00_JPRB, 0.27528E+00_JPRB ,&
&  0.28462E+00_JPRB, 0.29917E+00_JPRB, 0.30128E+00_JPRB, 0.30632E+00_JPRB, 0.31161E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.29898E+00_JPRB, 0.30307E+00_JPRB, 0.30573E+00_JPRB, 0.30748E+00_JPRB, 0.30924E+00_JPRB, 0.30849E+00_JPRB, 0.30683E+00_JPRB ,&
&  0.30247E+00_JPRB, 0.30098E+00_JPRB, 0.29814E+00_JPRB, 0.29500E+00_JPRB, 0.29138E+00_JPRB, 0.28700E+00_JPRB, 0.28294E+00_JPRB ,&
&  0.27835E+00_JPRB, 0.27331E+00_JPRB, 0.26740E+00_JPRB, 0.26081E+00_JPRB, 0.25479E+00_JPRB, 0.24913E+00_JPRB, 0.24605E+00_JPRB ,&
&  0.24467E+00_JPRB, 0.24563E+00_JPRB, 0.24877E+00_JPRB, 0.25249E+00_JPRB, 0.25699E+00_JPRB, 0.26206E+00_JPRB, 0.26836E+00_JPRB ,&
&  0.27368E+00_JPRB, 0.27998E+00_JPRB, 0.28378E+00_JPRB, 0.28534E+00_JPRB, 0.28393E+00_JPRB, 0.28392E+00_JPRB, 0.28301E+00_JPRB ,&
&  0.28180E+00_JPRB, 0.27998E+00_JPRB, 0.27941E+00_JPRB, 0.27975E+00_JPRB, 0.27912E+00_JPRB, 0.27739E+00_JPRB, 0.27479E+00_JPRB ,&
&  0.27228E+00_JPRB, 0.27013E+00_JPRB, 0.26730E+00_JPRB, 0.26488E+00_JPRB, 0.26308E+00_JPRB, 0.26076E+00_JPRB, 0.25762E+00_JPRB ,&
&  0.25483E+00_JPRB, 0.25207E+00_JPRB, 0.24883E+00_JPRB, 0.24614E+00_JPRB, 0.24348E+00_JPRB, 0.24156E+00_JPRB, 0.23984E+00_JPRB ,&
&  0.23914E+00_JPRB, 0.23838E+00_JPRB, 0.23818E+00_JPRB, 0.23808E+00_JPRB, 0.23857E+00_JPRB, 0.23827E+00_JPRB, 0.23779E+00_JPRB ,&
&  0.23723E+00_JPRB, 0.23707E+00_JPRB, 0.23647E+00_JPRB, 0.23625E+00_JPRB, 0.23602E+00_JPRB, 0.23523E+00_JPRB, 0.23482E+00_JPRB ,&
&  0.23481E+00_JPRB, 0.23464E+00_JPRB, 0.23396E+00_JPRB, 0.23384E+00_JPRB, 0.23341E+00_JPRB, 0.23280E+00_JPRB, 0.23310E+00_JPRB ,&
&  0.23293E+00_JPRB, 0.23319E+00_JPRB, 0.23359E+00_JPRB, 0.23418E+00_JPRB, 0.23542E+00_JPRB, 0.23695E+00_JPRB, 0.23897E+00_JPRB ,&
&  0.24206E+00_JPRB, 0.24484E+00_JPRB, 0.24760E+00_JPRB, 0.25079E+00_JPRB, 0.25695E+00_JPRB, 0.25951E+00_JPRB, 0.26421E+00_JPRB ,&
&  0.26841E+00_JPRB, 0.26580E+00_JPRB, 0.28924E+00_JPRB, 0.29047E+00_JPRB, 0.29313E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.25471E+00_JPRB, 0.25987E+00_JPRB, 0.26347E+00_JPRB, 0.26645E+00_JPRB, 0.26258E+00_JPRB, 0.28290E+00_JPRB, 0.28109E+00_JPRB ,&
&  0.28141E+00_JPRB, 0.27597E+00_JPRB, 0.27763E+00_JPRB, 0.27656E+00_JPRB, 0.27427E+00_JPRB, 0.27125E+00_JPRB, 0.26808E+00_JPRB ,&
&  0.26477E+00_JPRB, 0.26079E+00_JPRB, 0.25719E+00_JPRB, 0.25368E+00_JPRB, 0.25079E+00_JPRB, 0.24823E+00_JPRB, 0.24687E+00_JPRB ,&
&  0.24697E+00_JPRB, 0.24858E+00_JPRB, 0.25133E+00_JPRB, 0.25273E+00_JPRB, 0.25425E+00_JPRB, 0.25585E+00_JPRB, 0.25758E+00_JPRB ,&
&  0.25897E+00_JPRB, 0.26090E+00_JPRB, 0.26189E+00_JPRB, 0.26145E+00_JPRB, 0.26053E+00_JPRB, 0.26029E+00_JPRB, 0.26036E+00_JPRB ,&
&  0.26108E+00_JPRB, 0.26151E+00_JPRB, 0.26175E+00_JPRB, 0.26212E+00_JPRB, 0.26247E+00_JPRB, 0.26203E+00_JPRB, 0.26081E+00_JPRB ,&
&  0.25977E+00_JPRB, 0.25892E+00_JPRB, 0.25716E+00_JPRB, 0.25549E+00_JPRB, 0.25391E+00_JPRB, 0.25227E+00_JPRB, 0.25054E+00_JPRB ,&
&  0.24931E+00_JPRB, 0.24808E+00_JPRB, 0.24653E+00_JPRB, 0.24474E+00_JPRB, 0.24308E+00_JPRB, 0.24162E+00_JPRB, 0.24049E+00_JPRB ,&
&  0.24027E+00_JPRB, 0.23966E+00_JPRB, 0.23913E+00_JPRB, 0.23861E+00_JPRB, 0.23848E+00_JPRB, 0.23800E+00_JPRB, 0.23764E+00_JPRB ,&
&  0.23737E+00_JPRB, 0.23701E+00_JPRB, 0.23661E+00_JPRB, 0.23621E+00_JPRB, 0.23593E+00_JPRB, 0.23556E+00_JPRB, 0.23520E+00_JPRB ,&
&  0.23495E+00_JPRB, 0.23476E+00_JPRB, 0.23455E+00_JPRB, 0.23411E+00_JPRB, 0.23368E+00_JPRB, 0.23315E+00_JPRB, 0.23313E+00_JPRB ,&
&  0.23328E+00_JPRB, 0.23358E+00_JPRB, 0.23410E+00_JPRB, 0.23430E+00_JPRB, 0.23524E+00_JPRB, 0.23615E+00_JPRB, 0.23722E+00_JPRB ,&
&  0.23835E+00_JPRB, 0.24007E+00_JPRB, 0.24198E+00_JPRB, 0.24404E+00_JPRB, 0.24735E+00_JPRB, 0.24834E+00_JPRB, 0.24914E+00_JPRB ,&
&  0.25012E+00_JPRB, 0.25389E+00_JPRB, 0.25466E+00_JPRB, 0.25427E+00_JPRB, 0.25476E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.24372E+00_JPRB, 0.24399E+00_JPRB, 0.24396E+00_JPRB, 0.24378E+00_JPRB, 0.24167E+00_JPRB, 0.24079E+00_JPRB, 0.24000E+00_JPRB ,&
&  0.23942E+00_JPRB, 0.23913E+00_JPRB, 0.23957E+00_JPRB, 0.24015E+00_JPRB, 0.24074E+00_JPRB, 0.24136E+00_JPRB, 0.24189E+00_JPRB ,&
&  0.24215E+00_JPRB, 0.24245E+00_JPRB, 0.24129E+00_JPRB, 0.24074E+00_JPRB, 0.24057E+00_JPRB, 0.24064E+00_JPRB, 0.24062E+00_JPRB ,&
&  0.24074E+00_JPRB, 0.24082E+00_JPRB, 0.24079E+00_JPRB, 0.24116E+00_JPRB, 0.24181E+00_JPRB, 0.24233E+00_JPRB, 0.24288E+00_JPRB ,&
&  0.24342E+00_JPRB, 0.24420E+00_JPRB, 0.24491E+00_JPRB, 0.24559E+00_JPRB, 0.24650E+00_JPRB, 0.24739E+00_JPRB, 0.24791E+00_JPRB ,&
&  0.24833E+00_JPRB, 0.24846E+00_JPRB, 0.24865E+00_JPRB, 0.24880E+00_JPRB, 0.24907E+00_JPRB, 0.24941E+00_JPRB, 0.24948E+00_JPRB ,&
&  0.24941E+00_JPRB, 0.24925E+00_JPRB, 0.24900E+00_JPRB, 0.24841E+00_JPRB, 0.24779E+00_JPRB, 0.24719E+00_JPRB, 0.24660E+00_JPRB ,&
&  0.24609E+00_JPRB, 0.24550E+00_JPRB, 0.24485E+00_JPRB, 0.24417E+00_JPRB, 0.24344E+00_JPRB, 0.24274E+00_JPRB, 0.24203E+00_JPRB ,&
&  0.24131E+00_JPRB, 0.24085E+00_JPRB, 0.24034E+00_JPRB, 0.23990E+00_JPRB, 0.23954E+00_JPRB, 0.23918E+00_JPRB, 0.23884E+00_JPRB ,&
&  0.23849E+00_JPRB, 0.23808E+00_JPRB, 0.23770E+00_JPRB, 0.23728E+00_JPRB, 0.23691E+00_JPRB, 0.23650E+00_JPRB, 0.23617E+00_JPRB ,&
&  0.23584E+00_JPRB, 0.23555E+00_JPRB, 0.23532E+00_JPRB, 0.23522E+00_JPRB, 0.23516E+00_JPRB, 0.23520E+00_JPRB, 0.23524E+00_JPRB ,&
&  0.23531E+00_JPRB, 0.23535E+00_JPRB, 0.23539E+00_JPRB, 0.23526E+00_JPRB, 0.23549E+00_JPRB, 0.23574E+00_JPRB, 0.23609E+00_JPRB ,&
&  0.23642E+00_JPRB, 0.23695E+00_JPRB, 0.23749E+00_JPRB, 0.23811E+00_JPRB, 0.23906E+00_JPRB, 0.23941E+00_JPRB, 0.23970E+00_JPRB ,&
&  0.24000E+00_JPRB, 0.24064E+00_JPRB, 0.24117E+00_JPRB, 0.24146E+00_JPRB, 0.24209E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB, 0.23928E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB ,&
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB ,&
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB ,&
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB ,&
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB ,&
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB ,&
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB ,&
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB ,&
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB ,&
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB ,&
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB ,&
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB ,&
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB ,&
&  0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB, 0.62591E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.60577E+00_JPRB, 0.60266E+00_JPRB, 0.59970E+00_JPRB, 0.59655E+00_JPRB, 0.59324E+00_JPRB, 0.58939E+00_JPRB, 0.58522E+00_JPRB ,&
&  0.58144E+00_JPRB, 0.57801E+00_JPRB, 0.57493E+00_JPRB, 0.57213E+00_JPRB, 0.56978E+00_JPRB, 0.56790E+00_JPRB, 0.56708E+00_JPRB ,&
&  0.56682E+00_JPRB, 0.56662E+00_JPRB, 0.56663E+00_JPRB, 0.56668E+00_JPRB, 0.56646E+00_JPRB, 0.56605E+00_JPRB, 0.56554E+00_JPRB ,&
&  0.56526E+00_JPRB, 0.56508E+00_JPRB, 0.56497E+00_JPRB, 0.56527E+00_JPRB, 0.56576E+00_JPRB, 0.56625E+00_JPRB, 0.56670E+00_JPRB ,&
&  0.56761E+00_JPRB, 0.56874E+00_JPRB, 0.57030E+00_JPRB, 0.57225E+00_JPRB, 0.57616E+00_JPRB, 0.58033E+00_JPRB, 0.58461E+00_JPRB ,&
&  0.58862E+00_JPRB, 0.59263E+00_JPRB, 0.59591E+00_JPRB, 0.59868E+00_JPRB, 0.60122E+00_JPRB, 0.60445E+00_JPRB, 0.60805E+00_JPRB ,&
&  0.61195E+00_JPRB, 0.61631E+00_JPRB, 0.62146E+00_JPRB, 0.62652E+00_JPRB, 0.63116E+00_JPRB, 0.63593E+00_JPRB, 0.64110E+00_JPRB ,&
&  0.64613E+00_JPRB, 0.65121E+00_JPRB, 0.65634E+00_JPRB, 0.66173E+00_JPRB, 0.66688E+00_JPRB, 0.67301E+00_JPRB, 0.68031E+00_JPRB ,&
&  0.68877E+00_JPRB, 0.69531E+00_JPRB, 0.70293E+00_JPRB, 0.71115E+00_JPRB, 0.72376E+00_JPRB, 0.73007E+00_JPRB, 0.73612E+00_JPRB ,&
&  0.74138E+00_JPRB, 0.74997E+00_JPRB, 0.75536E+00_JPRB, 0.76063E+00_JPRB, 0.76546E+00_JPRB, 0.76979E+00_JPRB, 0.77266E+00_JPRB ,&
&  0.77515E+00_JPRB, 0.77734E+00_JPRB, 0.77781E+00_JPRB, 0.77750E+00_JPRB, 0.77614E+00_JPRB, 0.77432E+00_JPRB, 0.76946E+00_JPRB ,&
&  0.76506E+00_JPRB, 0.76084E+00_JPRB, 0.75692E+00_JPRB, 0.75115E+00_JPRB, 0.74520E+00_JPRB, 0.73906E+00_JPRB, 0.73273E+00_JPRB ,&
&  0.72452E+00_JPRB, 0.71607E+00_JPRB, 0.70698E+00_JPRB, 0.69726E+00_JPRB, 0.68786E+00_JPRB, 0.67803E+00_JPRB, 0.66730E+00_JPRB ,&
&  0.65526E+00_JPRB, 0.64397E+00_JPRB, 0.63123E+00_JPRB, 0.61863E+00_JPRB, 0.60952E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.59080E+00_JPRB, 0.57889E+00_JPRB, 0.57395E+00_JPRB, 0.57109E+00_JPRB, 0.56915E+00_JPRB, 0.56659E+00_JPRB, 0.56350E+00_JPRB ,&
&  0.56117E+00_JPRB, 0.55845E+00_JPRB, 0.55629E+00_JPRB, 0.55574E+00_JPRB, 0.55440E+00_JPRB, 0.55361E+00_JPRB, 0.55280E+00_JPRB ,&
&  0.55224E+00_JPRB, 0.55147E+00_JPRB, 0.55124E+00_JPRB, 0.55010E+00_JPRB, 0.54843E+00_JPRB, 0.54606E+00_JPRB, 0.54425E+00_JPRB ,&
&  0.54264E+00_JPRB, 0.54154E+00_JPRB, 0.53993E+00_JPRB, 0.53875E+00_JPRB, 0.53919E+00_JPRB, 0.53906E+00_JPRB, 0.53829E+00_JPRB ,&
&  0.54026E+00_JPRB, 0.54135E+00_JPRB, 0.54080E+00_JPRB, 0.53885E+00_JPRB, 0.53676E+00_JPRB, 0.53633E+00_JPRB, 0.53870E+00_JPRB ,&
&  0.54252E+00_JPRB, 0.55266E+00_JPRB, 0.56046E+00_JPRB, 0.56786E+00_JPRB, 0.57523E+00_JPRB, 0.58432E+00_JPRB, 0.59511E+00_JPRB ,&
&  0.60608E+00_JPRB, 0.61393E+00_JPRB, 0.62253E+00_JPRB, 0.63342E+00_JPRB, 0.64580E+00_JPRB, 0.65916E+00_JPRB, 0.67427E+00_JPRB ,&
&  0.68658E+00_JPRB, 0.69933E+00_JPRB, 0.71427E+00_JPRB, 0.73115E+00_JPRB, 0.74520E+00_JPRB, 0.76474E+00_JPRB, 0.78888E+00_JPRB ,&
&  0.82037E+00_JPRB, 0.83553E+00_JPRB, 0.85225E+00_JPRB, 0.86872E+00_JPRB, 0.89028E+00_JPRB, 0.90219E+00_JPRB, 0.91324E+00_JPRB ,&
&  0.92084E+00_JPRB, 0.92802E+00_JPRB, 0.92613E+00_JPRB, 0.92228E+00_JPRB, 0.91918E+00_JPRB, 0.91472E+00_JPRB, 0.91048E+00_JPRB ,&
&  0.90683E+00_JPRB, 0.90435E+00_JPRB, 0.89717E+00_JPRB, 0.89062E+00_JPRB, 0.88296E+00_JPRB, 0.87558E+00_JPRB, 0.86050E+00_JPRB ,&
&  0.84688E+00_JPRB, 0.83670E+00_JPRB, 0.83018E+00_JPRB, 0.82046E+00_JPRB, 0.81078E+00_JPRB, 0.80192E+00_JPRB, 0.79352E+00_JPRB ,&
&  0.78492E+00_JPRB, 0.77474E+00_JPRB, 0.76382E+00_JPRB, 0.75205E+00_JPRB, 0.74215E+00_JPRB, 0.72793E+00_JPRB, 0.71159E+00_JPRB ,&
&  0.69219E+00_JPRB, 0.67508E+00_JPRB, 0.65450E+00_JPRB, 0.63009E+00_JPRB, 0.60583E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.60943E+00_JPRB, 0.58115E+00_JPRB, 0.56656E+00_JPRB, 0.56299E+00_JPRB, 0.56154E+00_JPRB, 0.56009E+00_JPRB, 0.55870E+00_JPRB ,&
&  0.55643E+00_JPRB, 0.55528E+00_JPRB, 0.55647E+00_JPRB, 0.55769E+00_JPRB, 0.55778E+00_JPRB, 0.55769E+00_JPRB, 0.55646E+00_JPRB ,&
&  0.55468E+00_JPRB, 0.55127E+00_JPRB, 0.54794E+00_JPRB, 0.54249E+00_JPRB, 0.53684E+00_JPRB, 0.52977E+00_JPRB, 0.52048E+00_JPRB ,&
&  0.51230E+00_JPRB, 0.50485E+00_JPRB, 0.49500E+00_JPRB, 0.49181E+00_JPRB, 0.49023E+00_JPRB, 0.46880E+00_JPRB, 0.46403E+00_JPRB ,&
&  0.47024E+00_JPRB, 0.47166E+00_JPRB, 0.47263E+00_JPRB, 0.47368E+00_JPRB, 0.47288E+00_JPRB, 0.46480E+00_JPRB, 0.46286E+00_JPRB ,&
&  0.44737E+00_JPRB, 0.46027E+00_JPRB, 0.46975E+00_JPRB, 0.47848E+00_JPRB, 0.48778E+00_JPRB, 0.50144E+00_JPRB, 0.52683E+00_JPRB ,&
&  0.57350E+00_JPRB, 0.59450E+00_JPRB, 0.61810E+00_JPRB, 0.64019E+00_JPRB, 0.66981E+00_JPRB, 0.70164E+00_JPRB, 0.73115E+00_JPRB ,&
&  0.75594E+00_JPRB, 0.78260E+00_JPRB, 0.81344E+00_JPRB, 0.85474E+00_JPRB, 0.88691E+00_JPRB, 0.91673E+00_JPRB, 0.93762E+00_JPRB ,&
&  0.96358E+00_JPRB, 0.98259E+00_JPRB, 0.99588E+00_JPRB, 0.10043E+01_JPRB, 0.10131E+01_JPRB, 0.10164E+01_JPRB, 0.10205E+01_JPRB ,&
&  0.10216E+01_JPRB, 0.10209E+01_JPRB, 0.10117E+01_JPRB, 0.99575E+00_JPRB, 0.98376E+00_JPRB, 0.97761E+00_JPRB, 0.96858E+00_JPRB ,&
&  0.95896E+00_JPRB, 0.94844E+00_JPRB, 0.93602E+00_JPRB, 0.92120E+00_JPRB, 0.90418E+00_JPRB, 0.88950E+00_JPRB, 0.87186E+00_JPRB ,&
&  0.85716E+00_JPRB, 0.84555E+00_JPRB, 0.83618E+00_JPRB, 0.82427E+00_JPRB, 0.81193E+00_JPRB, 0.79981E+00_JPRB, 0.78621E+00_JPRB ,&
&  0.77354E+00_JPRB, 0.76248E+00_JPRB, 0.75347E+00_JPRB, 0.74345E+00_JPRB, 0.73666E+00_JPRB, 0.72685E+00_JPRB, 0.71792E+00_JPRB ,&
&  0.70752E+00_JPRB, 0.69835E+00_JPRB, 0.68343E+00_JPRB, 0.66228E+00_JPRB, 0.63401E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.64458E+00_JPRB, 0.61244E+00_JPRB, 0.57986E+00_JPRB, 0.56196E+00_JPRB, 0.55858E+00_JPRB, 0.55640E+00_JPRB, 0.55506E+00_JPRB ,&
&  0.55473E+00_JPRB, 0.55763E+00_JPRB, 0.56055E+00_JPRB, 0.56257E+00_JPRB, 0.56252E+00_JPRB, 0.56058E+00_JPRB, 0.55723E+00_JPRB ,&
&  0.55183E+00_JPRB, 0.54527E+00_JPRB, 0.53915E+00_JPRB, 0.53349E+00_JPRB, 0.52724E+00_JPRB, 0.52051E+00_JPRB, 0.51092E+00_JPRB ,&
&  0.50073E+00_JPRB, 0.49109E+00_JPRB, 0.45827E+00_JPRB, 0.45602E+00_JPRB, 0.44954E+00_JPRB, 0.44812E+00_JPRB, 0.45165E+00_JPRB ,&
&  0.45234E+00_JPRB, 0.44468E+00_JPRB, 0.43555E+00_JPRB, 0.42120E+00_JPRB, 0.40956E+00_JPRB, 0.39712E+00_JPRB, 0.39048E+00_JPRB ,&
&  0.38827E+00_JPRB, 0.39095E+00_JPRB, 0.39864E+00_JPRB, 0.40636E+00_JPRB, 0.41605E+00_JPRB, 0.42616E+00_JPRB, 0.44309E+00_JPRB ,&
&  0.48747E+00_JPRB, 0.54969E+00_JPRB, 0.62431E+00_JPRB, 0.66235E+00_JPRB, 0.70703E+00_JPRB, 0.75414E+00_JPRB, 0.78940E+00_JPRB ,&
&  0.82079E+00_JPRB, 0.85375E+00_JPRB, 0.88756E+00_JPRB, 0.92575E+00_JPRB, 0.96041E+00_JPRB, 0.98666E+00_JPRB, 0.10222E+01_JPRB ,&
&  0.10512E+01_JPRB, 0.10728E+01_JPRB, 0.10791E+01_JPRB, 0.10817E+01_JPRB, 0.10798E+01_JPRB, 0.10684E+01_JPRB, 0.10567E+01_JPRB ,&
&  0.10463E+01_JPRB, 0.10346E+01_JPRB, 0.10186E+01_JPRB, 0.98799E+00_JPRB, 0.96927E+00_JPRB, 0.96062E+00_JPRB, 0.94148E+00_JPRB ,&
&  0.92132E+00_JPRB, 0.89861E+00_JPRB, 0.87773E+00_JPRB, 0.84926E+00_JPRB, 0.82878E+00_JPRB, 0.82248E+00_JPRB, 0.81936E+00_JPRB ,&
&  0.81656E+00_JPRB, 0.81254E+00_JPRB, 0.80786E+00_JPRB, 0.79808E+00_JPRB, 0.78105E+00_JPRB, 0.76310E+00_JPRB, 0.74759E+00_JPRB ,&
&  0.73277E+00_JPRB, 0.71762E+00_JPRB, 0.70362E+00_JPRB, 0.68949E+00_JPRB, 0.68133E+00_JPRB, 0.67542E+00_JPRB, 0.67523E+00_JPRB ,&
&  0.67796E+00_JPRB, 0.68239E+00_JPRB, 0.68375E+00_JPRB, 0.68077E+00_JPRB, 0.66693E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.71870E+00_JPRB, 0.70597E+00_JPRB, 0.68010E+00_JPRB, 0.64463E+00_JPRB, 0.61783E+00_JPRB, 0.59978E+00_JPRB, 0.59115E+00_JPRB ,&
&  0.58611E+00_JPRB, 0.58365E+00_JPRB, 0.58145E+00_JPRB, 0.57852E+00_JPRB, 0.57393E+00_JPRB, 0.56756E+00_JPRB, 0.56143E+00_JPRB ,&
&  0.55058E+00_JPRB, 0.54072E+00_JPRB, 0.53011E+00_JPRB, 0.52403E+00_JPRB, 0.51894E+00_JPRB, 0.51486E+00_JPRB, 0.51149E+00_JPRB ,&
&  0.50733E+00_JPRB, 0.50190E+00_JPRB, 0.48397E+00_JPRB, 0.47676E+00_JPRB, 0.46963E+00_JPRB, 0.47023E+00_JPRB, 0.46858E+00_JPRB ,&
&  0.48283E+00_JPRB, 0.43533E+00_JPRB, 0.42678E+00_JPRB, 0.41213E+00_JPRB, 0.40112E+00_JPRB, 0.38837E+00_JPRB, 0.37636E+00_JPRB ,&
&  0.36932E+00_JPRB, 0.36279E+00_JPRB, 0.36172E+00_JPRB, 0.36960E+00_JPRB, 0.37727E+00_JPRB, 0.39125E+00_JPRB, 0.40683E+00_JPRB ,&
&  0.44094E+00_JPRB, 0.56461E+00_JPRB, 0.62357E+00_JPRB, 0.66876E+00_JPRB, 0.71337E+00_JPRB, 0.75802E+00_JPRB, 0.78396E+00_JPRB ,&
&  0.80956E+00_JPRB, 0.84338E+00_JPRB, 0.86794E+00_JPRB, 0.87450E+00_JPRB, 0.94068E+00_JPRB, 0.98889E+00_JPRB, 0.10362E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.11140E+01_JPRB, 0.11423E+01_JPRB, 0.11610E+01_JPRB, 0.11791E+01_JPRB, 0.11723E+01_JPRB, 0.11529E+01_JPRB ,&
&  0.11358E+01_JPRB, 0.11226E+01_JPRB, 0.11111E+01_JPRB, 0.10695E+01_JPRB, 0.10393E+01_JPRB, 0.10205E+01_JPRB, 0.98079E+00_JPRB ,&
&  0.93963E+00_JPRB, 0.90216E+00_JPRB, 0.86540E+00_JPRB, 0.82835E+00_JPRB, 0.79691E+00_JPRB, 0.77990E+00_JPRB, 0.77649E+00_JPRB ,&
&  0.77954E+00_JPRB, 0.77970E+00_JPRB, 0.77137E+00_JPRB, 0.75549E+00_JPRB, 0.73642E+00_JPRB, 0.72394E+00_JPRB, 0.70645E+00_JPRB ,&
&  0.68760E+00_JPRB, 0.66454E+00_JPRB, 0.64954E+00_JPRB, 0.63908E+00_JPRB, 0.63600E+00_JPRB, 0.63894E+00_JPRB, 0.64639E+00_JPRB ,&
&  0.65645E+00_JPRB, 0.67018E+00_JPRB, 0.68754E+00_JPRB, 0.70404E+00_JPRB, 0.71559E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.70420E+00_JPRB, 0.71356E+00_JPRB, 0.71602E+00_JPRB, 0.70954E+00_JPRB, 0.69617E+00_JPRB, 0.67827E+00_JPRB, 0.65970E+00_JPRB ,&
&  0.64288E+00_JPRB, 0.62974E+00_JPRB, 0.61760E+00_JPRB, 0.60851E+00_JPRB, 0.59782E+00_JPRB, 0.59091E+00_JPRB, 0.58293E+00_JPRB ,&
&  0.57857E+00_JPRB, 0.57702E+00_JPRB, 0.57050E+00_JPRB, 0.56260E+00_JPRB, 0.54734E+00_JPRB, 0.53350E+00_JPRB, 0.52291E+00_JPRB ,&
&  0.51343E+00_JPRB, 0.50582E+00_JPRB, 0.48839E+00_JPRB, 0.47562E+00_JPRB, 0.46943E+00_JPRB, 0.46610E+00_JPRB, 0.47381E+00_JPRB ,&
&  0.47497E+00_JPRB, 0.48199E+00_JPRB, 0.48784E+00_JPRB, 0.48181E+00_JPRB, 0.47495E+00_JPRB, 0.38383E+00_JPRB, 0.35636E+00_JPRB ,&
&  0.34810E+00_JPRB, 0.34183E+00_JPRB, 0.34081E+00_JPRB, 0.34677E+00_JPRB, 0.35636E+00_JPRB, 0.38084E+00_JPRB, 0.39959E+00_JPRB ,&
&  0.41809E+00_JPRB, 0.54862E+00_JPRB, 0.60177E+00_JPRB, 0.65060E+00_JPRB, 0.68568E+00_JPRB, 0.71643E+00_JPRB, 0.73795E+00_JPRB ,&
&  0.75716E+00_JPRB, 0.78428E+00_JPRB, 0.79871E+00_JPRB, 0.82916E+00_JPRB, 0.88883E+00_JPRB, 0.99286E+00_JPRB, 0.10790E+01_JPRB ,&
&  0.11766E+01_JPRB, 0.12088E+01_JPRB, 0.12319E+01_JPRB, 0.12247E+01_JPRB, 0.12406E+01_JPRB, 0.12475E+01_JPRB, 0.12415E+01_JPRB ,&
&  0.12417E+01_JPRB, 0.12162E+01_JPRB, 0.12156E+01_JPRB, 0.11917E+01_JPRB, 0.11714E+01_JPRB, 0.11501E+01_JPRB, 0.10806E+01_JPRB ,&
&  0.10003E+01_JPRB, 0.94664E+00_JPRB, 0.89780E+00_JPRB, 0.83790E+00_JPRB, 0.79360E+00_JPRB, 0.76216E+00_JPRB, 0.75241E+00_JPRB ,&
&  0.74884E+00_JPRB, 0.75478E+00_JPRB, 0.74108E+00_JPRB, 0.72114E+00_JPRB, 0.70640E+00_JPRB, 0.69631E+00_JPRB, 0.67552E+00_JPRB ,&
&  0.65721E+00_JPRB, 0.63936E+00_JPRB, 0.63007E+00_JPRB, 0.62084E+00_JPRB, 0.61519E+00_JPRB, 0.61450E+00_JPRB, 0.61807E+00_JPRB ,&
&  0.62693E+00_JPRB, 0.63936E+00_JPRB, 0.65693E+00_JPRB, 0.67547E+00_JPRB, 0.69518E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.68678E+00_JPRB, 0.70283E+00_JPRB, 0.71103E+00_JPRB, 0.70865E+00_JPRB, 0.69439E+00_JPRB, 0.67531E+00_JPRB, 0.65852E+00_JPRB ,&
&  0.64301E+00_JPRB, 0.62841E+00_JPRB, 0.61495E+00_JPRB, 0.60482E+00_JPRB, 0.59602E+00_JPRB, 0.59035E+00_JPRB, 0.58633E+00_JPRB ,&
&  0.58446E+00_JPRB, 0.57164E+00_JPRB, 0.56694E+00_JPRB, 0.56739E+00_JPRB, 0.56996E+00_JPRB, 0.56837E+00_JPRB, 0.56278E+00_JPRB ,&
&  0.54338E+00_JPRB, 0.51673E+00_JPRB, 0.49183E+00_JPRB, 0.47084E+00_JPRB, 0.45489E+00_JPRB, 0.44964E+00_JPRB, 0.45567E+00_JPRB ,&
&  0.46348E+00_JPRB, 0.46122E+00_JPRB, 0.46357E+00_JPRB, 0.48237E+00_JPRB, 0.49420E+00_JPRB, 0.43521E+00_JPRB, 0.35631E+00_JPRB ,&
&  0.34432E+00_JPRB, 0.33021E+00_JPRB, 0.32958E+00_JPRB, 0.33573E+00_JPRB, 0.34737E+00_JPRB, 0.36509E+00_JPRB, 0.39578E+00_JPRB ,&
&  0.43945E+00_JPRB, 0.53396E+00_JPRB, 0.58080E+00_JPRB, 0.61940E+00_JPRB, 0.65641E+00_JPRB, 0.68915E+00_JPRB, 0.70687E+00_JPRB ,&
&  0.71694E+00_JPRB, 0.73579E+00_JPRB, 0.77300E+00_JPRB, 0.81628E+00_JPRB, 0.89034E+00_JPRB, 0.10401E+01_JPRB, 0.11604E+01_JPRB ,&
&  0.13254E+01_JPRB, 0.13946E+01_JPRB, 0.13557E+01_JPRB, 0.12553E+01_JPRB, 0.12558E+01_JPRB, 0.12977E+01_JPRB, 0.12917E+01_JPRB ,&
&  0.13287E+01_JPRB, 0.13810E+01_JPRB, 0.13659E+01_JPRB, 0.13051E+01_JPRB, 0.12662E+01_JPRB, 0.12444E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.10452E+01_JPRB, 0.10069E+01_JPRB, 0.10142E+01_JPRB, 0.91329E+00_JPRB, 0.84460E+00_JPRB, 0.78008E+00_JPRB, 0.74531E+00_JPRB ,&
&  0.72422E+00_JPRB, 0.70658E+00_JPRB, 0.68232E+00_JPRB, 0.66252E+00_JPRB, 0.65219E+00_JPRB, 0.64555E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.62759E+00_JPRB, 0.61940E+00_JPRB, 0.61132E+00_JPRB, 0.60564E+00_JPRB, 0.60025E+00_JPRB, 0.59819E+00_JPRB, 0.59970E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.61813E+00_JPRB, 0.63444E+00_JPRB, 0.65042E+00_JPRB, 0.67012E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.68843E+00_JPRB, 0.70037E+00_JPRB, 0.69285E+00_JPRB, 0.66794E+00_JPRB, 0.63339E+00_JPRB, 0.59158E+00_JPRB, 0.56407E+00_JPRB ,&
&  0.54957E+00_JPRB, 0.54383E+00_JPRB, 0.53987E+00_JPRB, 0.53378E+00_JPRB, 0.53177E+00_JPRB, 0.53154E+00_JPRB, 0.53308E+00_JPRB ,&
&  0.53685E+00_JPRB, 0.54172E+00_JPRB, 0.54505E+00_JPRB, 0.54236E+00_JPRB, 0.54558E+00_JPRB, 0.54877E+00_JPRB, 0.55224E+00_JPRB ,&
&  0.54791E+00_JPRB, 0.52787E+00_JPRB, 0.50178E+00_JPRB, 0.48392E+00_JPRB, 0.46560E+00_JPRB, 0.46383E+00_JPRB, 0.45417E+00_JPRB ,&
&  0.43013E+00_JPRB, 0.42612E+00_JPRB, 0.45315E+00_JPRB, 0.48271E+00_JPRB, 0.48010E+00_JPRB, 0.50252E+00_JPRB, 0.38938E+00_JPRB ,&
&  0.35308E+00_JPRB, 0.33501E+00_JPRB, 0.33141E+00_JPRB, 0.34062E+00_JPRB, 0.36471E+00_JPRB, 0.39594E+00_JPRB, 0.42784E+00_JPRB ,&
&  0.48042E+00_JPRB, 0.52991E+00_JPRB, 0.55786E+00_JPRB, 0.58878E+00_JPRB, 0.62576E+00_JPRB, 0.66368E+00_JPRB, 0.69236E+00_JPRB ,&
&  0.71115E+00_JPRB, 0.74315E+00_JPRB, 0.79911E+00_JPRB, 0.88750E+00_JPRB, 0.10841E+01_JPRB, 0.10596E+01_JPRB, 0.10963E+01_JPRB ,&
&  0.14252E+01_JPRB, 0.16001E+01_JPRB, 0.14572E+01_JPRB, 0.13221E+01_JPRB, 0.12655E+01_JPRB, 0.12813E+01_JPRB, 0.13185E+01_JPRB ,&
&  0.14281E+01_JPRB, 0.15304E+01_JPRB, 0.15304E+01_JPRB, 0.13940E+01_JPRB, 0.13248E+01_JPRB, 0.12870E+01_JPRB, 0.11626E+01_JPRB ,&
&  0.10277E+01_JPRB, 0.10556E+01_JPRB, 0.11168E+01_JPRB, 0.94157E+00_JPRB, 0.82252E+00_JPRB, 0.74846E+00_JPRB, 0.71889E+00_JPRB ,&
&  0.70237E+00_JPRB, 0.68722E+00_JPRB, 0.66811E+00_JPRB, 0.64936E+00_JPRB, 0.62459E+00_JPRB, 0.60428E+00_JPRB, 0.58621E+00_JPRB ,&
&  0.57476E+00_JPRB, 0.56669E+00_JPRB, 0.56592E+00_JPRB, 0.56881E+00_JPRB, 0.57060E+00_JPRB, 0.57232E+00_JPRB, 0.58117E+00_JPRB ,&
&  0.59543E+00_JPRB, 0.60350E+00_JPRB, 0.61837E+00_JPRB, 0.63686E+00_JPRB, 0.66222E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.62954E+00_JPRB, 0.63183E+00_JPRB, 0.61812E+00_JPRB, 0.58891E+00_JPRB, 0.55891E+00_JPRB, 0.51897E+00_JPRB, 0.48976E+00_JPRB ,&
&  0.47355E+00_JPRB, 0.46608E+00_JPRB, 0.46398E+00_JPRB, 0.46427E+00_JPRB, 0.46725E+00_JPRB, 0.47304E+00_JPRB, 0.47399E+00_JPRB ,&
&  0.47589E+00_JPRB, 0.47646E+00_JPRB, 0.47974E+00_JPRB, 0.48653E+00_JPRB, 0.49365E+00_JPRB, 0.50131E+00_JPRB, 0.50939E+00_JPRB ,&
&  0.51727E+00_JPRB, 0.51381E+00_JPRB, 0.49537E+00_JPRB, 0.48122E+00_JPRB, 0.48563E+00_JPRB, 0.50459E+00_JPRB, 0.49181E+00_JPRB ,&
&  0.45182E+00_JPRB, 0.42769E+00_JPRB, 0.42859E+00_JPRB, 0.48812E+00_JPRB, 0.52531E+00_JPRB, 0.54382E+00_JPRB, 0.43123E+00_JPRB ,&
&  0.37221E+00_JPRB, 0.34961E+00_JPRB, 0.35109E+00_JPRB, 0.37627E+00_JPRB, 0.42311E+00_JPRB, 0.48736E+00_JPRB, 0.54439E+00_JPRB ,&
&  0.58986E+00_JPRB, 0.62874E+00_JPRB, 0.63998E+00_JPRB, 0.66277E+00_JPRB, 0.70065E+00_JPRB, 0.73531E+00_JPRB, 0.77321E+00_JPRB ,&
&  0.79694E+00_JPRB, 0.83619E+00_JPRB, 0.95493E+00_JPRB, 0.83143E+00_JPRB, 0.88274E+00_JPRB, 0.94580E+00_JPRB, 0.10684E+01_JPRB ,&
&  0.12590E+01_JPRB, 0.13201E+01_JPRB, 0.13234E+01_JPRB, 0.14085E+01_JPRB, 0.13089E+01_JPRB, 0.12321E+01_JPRB, 0.12395E+01_JPRB ,&
&  0.14267E+01_JPRB, 0.16033E+01_JPRB, 0.15760E+01_JPRB, 0.14449E+01_JPRB, 0.13998E+01_JPRB, 0.13749E+01_JPRB, 0.12263E+01_JPRB ,&
&  0.10403E+01_JPRB, 0.93603E+00_JPRB, 0.84726E+00_JPRB, 0.73732E+00_JPRB, 0.69339E+00_JPRB, 0.66833E+00_JPRB, 0.65414E+00_JPRB ,&
&  0.64558E+00_JPRB, 0.63255E+00_JPRB, 0.62258E+00_JPRB, 0.58368E+00_JPRB, 0.54505E+00_JPRB, 0.50792E+00_JPRB, 0.48360E+00_JPRB ,&
&  0.46977E+00_JPRB, 0.46876E+00_JPRB, 0.47626E+00_JPRB, 0.49604E+00_JPRB, 0.53163E+00_JPRB, 0.56399E+00_JPRB, 0.58571E+00_JPRB ,&
&  0.58955E+00_JPRB, 0.57577E+00_JPRB, 0.57361E+00_JPRB, 0.58079E+00_JPRB, 0.61104E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.60867E+00_JPRB, 0.57468E+00_JPRB, 0.55333E+00_JPRB, 0.55793E+00_JPRB, 0.51939E+00_JPRB, 0.48238E+00_JPRB, 0.46205E+00_JPRB ,&
&  0.46312E+00_JPRB, 0.46009E+00_JPRB, 0.46207E+00_JPRB, 0.44271E+00_JPRB, 0.42569E+00_JPRB, 0.41591E+00_JPRB, 0.41200E+00_JPRB ,&
&  0.42158E+00_JPRB, 0.43352E+00_JPRB, 0.44257E+00_JPRB, 0.44118E+00_JPRB, 0.43940E+00_JPRB, 0.44330E+00_JPRB, 0.45421E+00_JPRB ,&
&  0.46985E+00_JPRB, 0.48281E+00_JPRB, 0.48273E+00_JPRB, 0.48129E+00_JPRB, 0.50483E+00_JPRB, 0.53292E+00_JPRB, 0.50848E+00_JPRB ,&
&  0.47819E+00_JPRB, 0.45465E+00_JPRB, 0.45032E+00_JPRB, 0.49384E+00_JPRB, 0.56050E+00_JPRB, 0.59175E+00_JPRB, 0.47590E+00_JPRB ,&
&  0.39761E+00_JPRB, 0.38716E+00_JPRB, 0.42651E+00_JPRB, 0.54685E+00_JPRB, 0.58600E+00_JPRB, 0.63412E+00_JPRB, 0.70389E+00_JPRB ,&
&  0.65527E+00_JPRB, 0.65842E+00_JPRB, 0.68365E+00_JPRB, 0.82502E+00_JPRB, 0.86169E+00_JPRB, 0.87691E+00_JPRB, 0.88906E+00_JPRB ,&
&  0.89731E+00_JPRB, 0.93222E+00_JPRB, 0.10740E+01_JPRB, 0.90729E+00_JPRB, 0.95569E+00_JPRB, 0.11679E+01_JPRB, 0.11698E+01_JPRB ,&
&  0.12400E+01_JPRB, 0.12646E+01_JPRB, 0.14081E+01_JPRB, 0.13595E+01_JPRB, 0.12395E+01_JPRB, 0.11927E+01_JPRB, 0.12805E+01_JPRB ,&
&  0.15280E+01_JPRB, 0.17362E+01_JPRB, 0.16960E+01_JPRB, 0.16337E+01_JPRB, 0.15969E+01_JPRB, 0.15273E+01_JPRB, 0.13610E+01_JPRB ,&
&  0.11214E+01_JPRB, 0.94090E+00_JPRB, 0.80628E+00_JPRB, 0.68914E+00_JPRB, 0.63371E+00_JPRB, 0.61275E+00_JPRB, 0.59130E+00_JPRB ,&
&  0.58571E+00_JPRB, 0.57850E+00_JPRB, 0.56936E+00_JPRB, 0.52685E+00_JPRB, 0.48923E+00_JPRB, 0.46449E+00_JPRB, 0.44712E+00_JPRB ,&
&  0.43961E+00_JPRB, 0.42610E+00_JPRB, 0.41659E+00_JPRB, 0.43071E+00_JPRB, 0.47474E+00_JPRB, 0.52926E+00_JPRB, 0.55377E+00_JPRB ,&
&  0.54350E+00_JPRB, 0.53539E+00_JPRB, 0.56157E+00_JPRB, 0.60255E+00_JPRB, 0.62078E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.62496E+00_JPRB, 0.61350E+00_JPRB, 0.60687E+00_JPRB, 0.59610E+00_JPRB, 0.57906E+00_JPRB, 0.53143E+00_JPRB, 0.49765E+00_JPRB ,&
&  0.50611E+00_JPRB, 0.48196E+00_JPRB, 0.47546E+00_JPRB, 0.44834E+00_JPRB, 0.42887E+00_JPRB, 0.40740E+00_JPRB, 0.39426E+00_JPRB ,&
&  0.38436E+00_JPRB, 0.39299E+00_JPRB, 0.41388E+00_JPRB, 0.41960E+00_JPRB, 0.41980E+00_JPRB, 0.41958E+00_JPRB, 0.42908E+00_JPRB ,&
&  0.44287E+00_JPRB, 0.46287E+00_JPRB, 0.47692E+00_JPRB, 0.48855E+00_JPRB, 0.51051E+00_JPRB, 0.53934E+00_JPRB, 0.54042E+00_JPRB ,&
&  0.53796E+00_JPRB, 0.51745E+00_JPRB, 0.49733E+00_JPRB, 0.52886E+00_JPRB, 0.59362E+00_JPRB, 0.65528E+00_JPRB, 0.63260E+00_JPRB ,&
&  0.46732E+00_JPRB, 0.47138E+00_JPRB, 0.64014E+00_JPRB, 0.69179E+00_JPRB, 0.74703E+00_JPRB, 0.78329E+00_JPRB, 0.83022E+00_JPRB ,&
&  0.87246E+00_JPRB, 0.91546E+00_JPRB, 0.94653E+00_JPRB, 0.97183E+00_JPRB, 0.98950E+00_JPRB, 0.10100E+01_JPRB, 0.10038E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.87087E+00_JPRB, 0.82805E+00_JPRB, 0.88470E+00_JPRB, 0.11681E+01_JPRB, 0.12269E+01_JPRB, 0.11956E+01_JPRB ,&
&  0.12376E+01_JPRB, 0.12248E+01_JPRB, 0.12797E+01_JPRB, 0.13733E+01_JPRB, 0.13863E+01_JPRB, 0.14504E+01_JPRB, 0.15581E+01_JPRB ,&
&  0.17460E+01_JPRB, 0.19560E+01_JPRB, 0.19739E+01_JPRB, 0.19426E+01_JPRB, 0.18536E+01_JPRB, 0.16990E+01_JPRB, 0.14533E+01_JPRB ,&
&  0.11520E+01_JPRB, 0.94307E+00_JPRB, 0.77459E+00_JPRB, 0.65737E+00_JPRB, 0.59088E+00_JPRB, 0.56152E+00_JPRB, 0.53961E+00_JPRB ,&
&  0.53796E+00_JPRB, 0.53357E+00_JPRB, 0.52071E+00_JPRB, 0.48733E+00_JPRB, 0.46645E+00_JPRB, 0.47388E+00_JPRB, 0.48185E+00_JPRB ,&
&  0.48245E+00_JPRB, 0.46697E+00_JPRB, 0.43930E+00_JPRB, 0.45141E+00_JPRB, 0.49289E+00_JPRB, 0.53669E+00_JPRB, 0.55846E+00_JPRB ,&
&  0.55847E+00_JPRB, 0.55732E+00_JPRB, 0.57043E+00_JPRB, 0.59731E+00_JPRB, 0.61022E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.65159E+00_JPRB, 0.65458E+00_JPRB, 0.63984E+00_JPRB, 0.62630E+00_JPRB, 0.60858E+00_JPRB, 0.54953E+00_JPRB, 0.53570E+00_JPRB ,&
&  0.60067E+00_JPRB, 0.54557E+00_JPRB, 0.59986E+00_JPRB, 0.49669E+00_JPRB, 0.47342E+00_JPRB, 0.45040E+00_JPRB, 0.42851E+00_JPRB ,&
&  0.40266E+00_JPRB, 0.39467E+00_JPRB, 0.40308E+00_JPRB, 0.41134E+00_JPRB, 0.41207E+00_JPRB, 0.40912E+00_JPRB, 0.41224E+00_JPRB ,&
&  0.42038E+00_JPRB, 0.44348E+00_JPRB, 0.47505E+00_JPRB, 0.49210E+00_JPRB, 0.51085E+00_JPRB, 0.54211E+00_JPRB, 0.56141E+00_JPRB ,&
&  0.54488E+00_JPRB, 0.54604E+00_JPRB, 0.55414E+00_JPRB, 0.57813E+00_JPRB, 0.62113E+00_JPRB, 0.68385E+00_JPRB, 0.72469E+00_JPRB ,&
&  0.74223E+00_JPRB, 0.62598E+00_JPRB, 0.73513E+00_JPRB, 0.75995E+00_JPRB, 0.81850E+00_JPRB, 0.85159E+00_JPRB, 0.87508E+00_JPRB ,&
&  0.90833E+00_JPRB, 0.94648E+00_JPRB, 0.98934E+00_JPRB, 0.10283E+01_JPRB, 0.10679E+01_JPRB, 0.11038E+01_JPRB, 0.11030E+01_JPRB ,&
&  0.11831E+01_JPRB, 0.84757E+00_JPRB, 0.90652E+00_JPRB, 0.94576E+00_JPRB, 0.12730E+01_JPRB, 0.14210E+01_JPRB, 0.13629E+01_JPRB ,&
&  0.15548E+01_JPRB, 0.14718E+01_JPRB, 0.15709E+01_JPRB, 0.16633E+01_JPRB, 0.17770E+01_JPRB, 0.19330E+01_JPRB, 0.20240E+01_JPRB ,&
&  0.20622E+01_JPRB, 0.21576E+01_JPRB, 0.22742E+01_JPRB, 0.21318E+01_JPRB, 0.19033E+01_JPRB, 0.16363E+01_JPRB, 0.13272E+01_JPRB ,&
&  0.11149E+01_JPRB, 0.93276E+00_JPRB, 0.78003E+00_JPRB, 0.67617E+00_JPRB, 0.59599E+00_JPRB, 0.54389E+00_JPRB, 0.51490E+00_JPRB ,&
&  0.49994E+00_JPRB, 0.48180E+00_JPRB, 0.46929E+00_JPRB, 0.45039E+00_JPRB, 0.44695E+00_JPRB, 0.45741E+00_JPRB, 0.48970E+00_JPRB ,&
&  0.50170E+00_JPRB, 0.50065E+00_JPRB, 0.54613E+00_JPRB, 0.60131E+00_JPRB, 0.60875E+00_JPRB, 0.62671E+00_JPRB, 0.69804E+00_JPRB ,&
&  0.61229E+00_JPRB, 0.63216E+00_JPRB, 0.68173E+00_JPRB, 0.67734E+00_JPRB, 0.65507E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.70544E+00_JPRB, 0.70331E+00_JPRB, 0.67737E+00_JPRB, 0.65357E+00_JPRB, 0.64012E+00_JPRB, 0.61033E+00_JPRB, 0.58080E+00_JPRB ,&
&  0.63009E+00_JPRB, 0.61990E+00_JPRB, 0.62190E+00_JPRB, 0.62345E+00_JPRB, 0.61642E+00_JPRB, 0.55314E+00_JPRB, 0.49698E+00_JPRB ,&
&  0.45583E+00_JPRB, 0.43276E+00_JPRB, 0.43603E+00_JPRB, 0.43467E+00_JPRB, 0.43149E+00_JPRB, 0.42279E+00_JPRB, 0.41885E+00_JPRB ,&
&  0.41701E+00_JPRB, 0.43042E+00_JPRB, 0.46110E+00_JPRB, 0.48774E+00_JPRB, 0.49964E+00_JPRB, 0.51615E+00_JPRB, 0.54620E+00_JPRB ,&
&  0.53799E+00_JPRB, 0.54760E+00_JPRB, 0.56746E+00_JPRB, 0.53975E+00_JPRB, 0.63847E+00_JPRB, 0.68191E+00_JPRB, 0.70637E+00_JPRB ,&
&  0.75405E+00_JPRB, 0.79457E+00_JPRB, 0.81117E+00_JPRB, 0.81438E+00_JPRB, 0.83203E+00_JPRB, 0.84228E+00_JPRB, 0.86534E+00_JPRB ,&
&  0.88854E+00_JPRB, 0.92413E+00_JPRB, 0.97356E+00_JPRB, 0.10327E+01_JPRB, 0.11493E+01_JPRB, 0.10530E+01_JPRB, 0.11480E+01_JPRB ,&
&  0.12067E+01_JPRB, 0.12350E+01_JPRB, 0.13051E+01_JPRB, 0.11673E+01_JPRB, 0.14754E+01_JPRB, 0.16552E+01_JPRB, 0.15293E+01_JPRB ,&
&  0.15597E+01_JPRB, 0.15209E+01_JPRB, 0.18308E+01_JPRB, 0.20662E+01_JPRB, 0.22562E+01_JPRB, 0.24243E+01_JPRB, 0.24907E+01_JPRB ,&
&  0.24764E+01_JPRB, 0.26371E+01_JPRB, 0.26016E+01_JPRB, 0.23576E+01_JPRB, 0.20184E+01_JPRB, 0.17378E+01_JPRB, 0.14007E+01_JPRB ,&
&  0.11234E+01_JPRB, 0.90160E+00_JPRB, 0.76669E+00_JPRB, 0.70019E+00_JPRB, 0.59096E+00_JPRB, 0.55305E+00_JPRB, 0.51262E+00_JPRB ,&
&  0.47438E+00_JPRB, 0.44558E+00_JPRB, 0.43836E+00_JPRB, 0.43359E+00_JPRB, 0.43382E+00_JPRB, 0.45029E+00_JPRB, 0.47089E+00_JPRB ,&
&  0.52890E+00_JPRB, 0.63368E+00_JPRB, 0.84318E+00_JPRB, 0.83016E+00_JPRB, 0.81189E+00_JPRB, 0.79175E+00_JPRB, 0.69602E+00_JPRB ,&
&  0.65066E+00_JPRB, 0.74218E+00_JPRB, 0.72525E+00_JPRB, 0.70965E+00_JPRB, 0.69532E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.75202E+00_JPRB, 0.73780E+00_JPRB, 0.71920E+00_JPRB, 0.69569E+00_JPRB, 0.66927E+00_JPRB, 0.64699E+00_JPRB, 0.63692E+00_JPRB ,&
&  0.64026E+00_JPRB, 0.63919E+00_JPRB, 0.64066E+00_JPRB, 0.63671E+00_JPRB, 0.62442E+00_JPRB, 0.64011E+00_JPRB, 0.57056E+00_JPRB ,&
&  0.49583E+00_JPRB, 0.46741E+00_JPRB, 0.44801E+00_JPRB, 0.45044E+00_JPRB, 0.46137E+00_JPRB, 0.45517E+00_JPRB, 0.44539E+00_JPRB ,&
&  0.43601E+00_JPRB, 0.43304E+00_JPRB, 0.44439E+00_JPRB, 0.46464E+00_JPRB, 0.47986E+00_JPRB, 0.49591E+00_JPRB, 0.51726E+00_JPRB ,&
&  0.50766E+00_JPRB, 0.49138E+00_JPRB, 0.48936E+00_JPRB, 0.50256E+00_JPRB, 0.57499E+00_JPRB, 0.66876E+00_JPRB, 0.72080E+00_JPRB ,&
&  0.74148E+00_JPRB, 0.75701E+00_JPRB, 0.79184E+00_JPRB, 0.80484E+00_JPRB, 0.81129E+00_JPRB, 0.83112E+00_JPRB, 0.85615E+00_JPRB ,&
&  0.88331E+00_JPRB, 0.92085E+00_JPRB, 0.97220E+00_JPRB, 0.10560E+01_JPRB, 0.10755E+01_JPRB, 0.10617E+01_JPRB, 0.11658E+01_JPRB ,&
&  0.12529E+01_JPRB, 0.13856E+01_JPRB, 0.14211E+01_JPRB, 0.15100E+01_JPRB, 0.16982E+01_JPRB, 0.16829E+01_JPRB, 0.16468E+01_JPRB ,&
&  0.17521E+01_JPRB, 0.18012E+01_JPRB, 0.22722E+01_JPRB, 0.24516E+01_JPRB, 0.25188E+01_JPRB, 0.26847E+01_JPRB, 0.27008E+01_JPRB ,&
&  0.27540E+01_JPRB, 0.27775E+01_JPRB, 0.27400E+01_JPRB, 0.24925E+01_JPRB, 0.21049E+01_JPRB, 0.17934E+01_JPRB, 0.13903E+01_JPRB ,&
&  0.11275E+01_JPRB, 0.90716E+00_JPRB, 0.69900E+00_JPRB, 0.61406E+00_JPRB, 0.53797E+00_JPRB, 0.52048E+00_JPRB, 0.49320E+00_JPRB ,&
&  0.45898E+00_JPRB, 0.44878E+00_JPRB, 0.44170E+00_JPRB, 0.46455E+00_JPRB, 0.47672E+00_JPRB, 0.46969E+00_JPRB, 0.51936E+00_JPRB ,&
&  0.61754E+00_JPRB, 0.86792E+00_JPRB, 0.87954E+00_JPRB, 0.94479E+00_JPRB, 0.92763E+00_JPRB, 0.90068E+00_JPRB, 0.71168E+00_JPRB ,&
&  0.74664E+00_JPRB, 0.82053E+00_JPRB, 0.77448E+00_JPRB, 0.75923E+00_JPRB, 0.75668E+00_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.77845E+00_JPRB, 0.75800E+00_JPRB, 0.74429E+00_JPRB, 0.73152E+00_JPRB, 0.71039E+00_JPRB, 0.68383E+00_JPRB, 0.68367E+00_JPRB ,&
&  0.67885E+00_JPRB, 0.66948E+00_JPRB, 0.66857E+00_JPRB, 0.66605E+00_JPRB, 0.65693E+00_JPRB, 0.66005E+00_JPRB, 0.67821E+00_JPRB ,&
&  0.57531E+00_JPRB, 0.53218E+00_JPRB, 0.47634E+00_JPRB, 0.45987E+00_JPRB, 0.50381E+00_JPRB, 0.48746E+00_JPRB, 0.47931E+00_JPRB ,&
&  0.46058E+00_JPRB, 0.44971E+00_JPRB, 0.43677E+00_JPRB, 0.44408E+00_JPRB, 0.46128E+00_JPRB, 0.47875E+00_JPRB, 0.49191E+00_JPRB ,&
&  0.47413E+00_JPRB, 0.45307E+00_JPRB, 0.45259E+00_JPRB, 0.46994E+00_JPRB, 0.52324E+00_JPRB, 0.60320E+00_JPRB, 0.69385E+00_JPRB ,&
&  0.73393E+00_JPRB, 0.74664E+00_JPRB, 0.76843E+00_JPRB, 0.79310E+00_JPRB, 0.82515E+00_JPRB, 0.87603E+00_JPRB, 0.90939E+00_JPRB ,&
&  0.93323E+00_JPRB, 0.95446E+00_JPRB, 0.10014E+01_JPRB, 0.10711E+01_JPRB, 0.10338E+01_JPRB, 0.10340E+01_JPRB, 0.11661E+01_JPRB ,&
&  0.13290E+01_JPRB, 0.13189E+01_JPRB, 0.13827E+01_JPRB, 0.18612E+01_JPRB, 0.21617E+01_JPRB, 0.22156E+01_JPRB, 0.21376E+01_JPRB ,&
&  0.21917E+01_JPRB, 0.22972E+01_JPRB, 0.24371E+01_JPRB, 0.26133E+01_JPRB, 0.27216E+01_JPRB, 0.28511E+01_JPRB, 0.28577E+01_JPRB ,&
&  0.28629E+01_JPRB, 0.28727E+01_JPRB, 0.27746E+01_JPRB, 0.25367E+01_JPRB, 0.20690E+01_JPRB, 0.17060E+01_JPRB, 0.13142E+01_JPRB ,&
&  0.10925E+01_JPRB, 0.83664E+00_JPRB, 0.59298E+00_JPRB, 0.50150E+00_JPRB, 0.46824E+00_JPRB, 0.50320E+00_JPRB, 0.64415E+00_JPRB ,&
&  0.49832E+00_JPRB, 0.50651E+00_JPRB, 0.51685E+00_JPRB, 0.56319E+00_JPRB, 0.61364E+00_JPRB, 0.63556E+00_JPRB, 0.58083E+00_JPRB ,&
&  0.60261E+00_JPRB, 0.78050E+00_JPRB, 0.98921E+00_JPRB, 0.10683E+01_JPRB, 0.10380E+01_JPRB, 0.10158E+01_JPRB, 0.91923E+00_JPRB ,&
&  0.99673E+00_JPRB, 0.89083E+00_JPRB, 0.82577E+00_JPRB, 0.79767E+00_JPRB, 0.80054E+00_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.77812E+00_JPRB, 0.77347E+00_JPRB, 0.77223E+00_JPRB, 0.76554E+00_JPRB, 0.75809E+00_JPRB, 0.72771E+00_JPRB, 0.72424E+00_JPRB ,&
&  0.71626E+00_JPRB, 0.70663E+00_JPRB, 0.69600E+00_JPRB, 0.68994E+00_JPRB, 0.68337E+00_JPRB, 0.67684E+00_JPRB, 0.66926E+00_JPRB ,&
&  0.68871E+00_JPRB, 0.61591E+00_JPRB, 0.54474E+00_JPRB, 0.47218E+00_JPRB, 0.48547E+00_JPRB, 0.48015E+00_JPRB, 0.47980E+00_JPRB ,&
&  0.46818E+00_JPRB, 0.46713E+00_JPRB, 0.45240E+00_JPRB, 0.44318E+00_JPRB, 0.45622E+00_JPRB, 0.47634E+00_JPRB, 0.48721E+00_JPRB ,&
&  0.47727E+00_JPRB, 0.45755E+00_JPRB, 0.45599E+00_JPRB, 0.48504E+00_JPRB, 0.57067E+00_JPRB, 0.58132E+00_JPRB, 0.67921E+00_JPRB ,&
&  0.73254E+00_JPRB, 0.76551E+00_JPRB, 0.79454E+00_JPRB, 0.84159E+00_JPRB, 0.89654E+00_JPRB, 0.96872E+00_JPRB, 0.10119E+01_JPRB ,&
&  0.10377E+01_JPRB, 0.10333E+01_JPRB, 0.99838E+00_JPRB, 0.10639E+01_JPRB, 0.11255E+01_JPRB, 0.11617E+01_JPRB, 0.12052E+01_JPRB ,&
&  0.12756E+01_JPRB, 0.13170E+01_JPRB, 0.15039E+01_JPRB, 0.20553E+01_JPRB, 0.23816E+01_JPRB, 0.24004E+01_JPRB, 0.25290E+01_JPRB ,&
&  0.26977E+01_JPRB, 0.28640E+01_JPRB, 0.30277E+01_JPRB, 0.30547E+01_JPRB, 0.30870E+01_JPRB, 0.29933E+01_JPRB, 0.29611E+01_JPRB ,&
&  0.28957E+01_JPRB, 0.29167E+01_JPRB, 0.28751E+01_JPRB, 0.24974E+01_JPRB, 0.20107E+01_JPRB, 0.16067E+01_JPRB, 0.13783E+01_JPRB ,&
&  0.12994E+01_JPRB, 0.69656E+00_JPRB, 0.49769E+00_JPRB, 0.41318E+00_JPRB, 0.40066E+00_JPRB, 0.42240E+00_JPRB, 0.51708E+00_JPRB ,&
&  0.47216E+00_JPRB, 0.53639E+00_JPRB, 0.65809E+00_JPRB, 0.72621E+00_JPRB, 0.87050E+00_JPRB, 0.10362E+01_JPRB, 0.83360E+00_JPRB ,&
&  0.74346E+00_JPRB, 0.82618E+00_JPRB, 0.11477E+01_JPRB, 0.12118E+01_JPRB, 0.11496E+01_JPRB, 0.11178E+01_JPRB, 0.11212E+01_JPRB ,&
&  0.10569E+01_JPRB, 0.10045E+01_JPRB, 0.90334E+00_JPRB, 0.82310E+00_JPRB, 0.79481E+00_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.83087E+00_JPRB, 0.81666E+00_JPRB, 0.81929E+00_JPRB, 0.81769E+00_JPRB, 0.80794E+00_JPRB, 0.78623E+00_JPRB, 0.77929E+00_JPRB ,&
&  0.75632E+00_JPRB, 0.73462E+00_JPRB, 0.71504E+00_JPRB, 0.70346E+00_JPRB, 0.69632E+00_JPRB, 0.68830E+00_JPRB, 0.67075E+00_JPRB ,&
&  0.65067E+00_JPRB, 0.64540E+00_JPRB, 0.55956E+00_JPRB, 0.49672E+00_JPRB, 0.44229E+00_JPRB, 0.45186E+00_JPRB, 0.46688E+00_JPRB ,&
&  0.48656E+00_JPRB, 0.49511E+00_JPRB, 0.50558E+00_JPRB, 0.51690E+00_JPRB, 0.53052E+00_JPRB, 0.51779E+00_JPRB, 0.53487E+00_JPRB ,&
&  0.52626E+00_JPRB, 0.50961E+00_JPRB, 0.48991E+00_JPRB, 0.54720E+00_JPRB, 0.61342E+00_JPRB, 0.61987E+00_JPRB, 0.74984E+00_JPRB ,&
&  0.81886E+00_JPRB, 0.86734E+00_JPRB, 0.90562E+00_JPRB, 0.95320E+00_JPRB, 0.10041E+01_JPRB, 0.10063E+01_JPRB, 0.97540E+00_JPRB ,&
&  0.92059E+00_JPRB, 0.91971E+00_JPRB, 0.97396E+00_JPRB, 0.10060E+01_JPRB, 0.11397E+01_JPRB, 0.11994E+01_JPRB, 0.12796E+01_JPRB ,&
&  0.14051E+01_JPRB, 0.14221E+01_JPRB, 0.15138E+01_JPRB, 0.21295E+01_JPRB, 0.28612E+01_JPRB, 0.28999E+01_JPRB, 0.29845E+01_JPRB ,&
&  0.31927E+01_JPRB, 0.32511E+01_JPRB, 0.32085E+01_JPRB, 0.30645E+01_JPRB, 0.30569E+01_JPRB, 0.30852E+01_JPRB, 0.31281E+01_JPRB ,&
&  0.29962E+01_JPRB, 0.29274E+01_JPRB, 0.28153E+01_JPRB, 0.25220E+01_JPRB, 0.20880E+01_JPRB, 0.18354E+01_JPRB, 0.15922E+01_JPRB ,&
&  0.13638E+01_JPRB, 0.88238E+00_JPRB, 0.57426E+00_JPRB, 0.47609E+00_JPRB, 0.44858E+00_JPRB, 0.45146E+00_JPRB, 0.52527E+00_JPRB ,&
&  0.53725E+00_JPRB, 0.64841E+00_JPRB, 0.81732E+00_JPRB, 0.89937E+00_JPRB, 0.13135E+01_JPRB, 0.16044E+01_JPRB, 0.15034E+01_JPRB ,&
&  0.12409E+01_JPRB, 0.10989E+01_JPRB, 0.13816E+01_JPRB, 0.13909E+01_JPRB, 0.13213E+01_JPRB, 0.12110E+01_JPRB, 0.11828E+01_JPRB ,&
&  0.11214E+01_JPRB, 0.10787E+01_JPRB, 0.10145E+01_JPRB, 0.91894E+00_JPRB, 0.85803E+00_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.89426E+00_JPRB, 0.85401E+00_JPRB, 0.83133E+00_JPRB, 0.82053E+00_JPRB, 0.78993E+00_JPRB, 0.76891E+00_JPRB, 0.77984E+00_JPRB ,&
&  0.78244E+00_JPRB, 0.75666E+00_JPRB, 0.73528E+00_JPRB, 0.72794E+00_JPRB, 0.72628E+00_JPRB, 0.71926E+00_JPRB, 0.68264E+00_JPRB ,&
&  0.63801E+00_JPRB, 0.60815E+00_JPRB, 0.51789E+00_JPRB, 0.45919E+00_JPRB, 0.43322E+00_JPRB, 0.45260E+00_JPRB, 0.49402E+00_JPRB ,&
&  0.55009E+00_JPRB, 0.60238E+00_JPRB, 0.65571E+00_JPRB, 0.66771E+00_JPRB, 0.68046E+00_JPRB, 0.67210E+00_JPRB, 0.66906E+00_JPRB ,&
&  0.70247E+00_JPRB, 0.60472E+00_JPRB, 0.62236E+00_JPRB, 0.64434E+00_JPRB, 0.72059E+00_JPRB, 0.77528E+00_JPRB, 0.82763E+00_JPRB ,&
&  0.83898E+00_JPRB, 0.87314E+00_JPRB, 0.90171E+00_JPRB, 0.92950E+00_JPRB, 0.95121E+00_JPRB, 0.95249E+00_JPRB, 0.93599E+00_JPRB ,&
&  0.95792E+00_JPRB, 0.94051E+00_JPRB, 0.94000E+00_JPRB, 0.11444E+01_JPRB, 0.15936E+01_JPRB, 0.18026E+01_JPRB, 0.18390E+01_JPRB ,&
&  0.17559E+01_JPRB, 0.18456E+01_JPRB, 0.26856E+01_JPRB, 0.30540E+01_JPRB, 0.32164E+01_JPRB, 0.36261E+01_JPRB, 0.37611E+01_JPRB ,&
&  0.35941E+01_JPRB, 0.32613E+01_JPRB, 0.29687E+01_JPRB, 0.26906E+01_JPRB, 0.32432E+01_JPRB, 0.34515E+01_JPRB, 0.34195E+01_JPRB ,&
&  0.30417E+01_JPRB, 0.29310E+01_JPRB, 0.26379E+01_JPRB, 0.23663E+01_JPRB, 0.21738E+01_JPRB, 0.21777E+01_JPRB, 0.17287E+01_JPRB ,&
&  0.13143E+01_JPRB, 0.11908E+01_JPRB, 0.89552E+00_JPRB, 0.74978E+00_JPRB, 0.69210E+00_JPRB, 0.68526E+00_JPRB, 0.71419E+00_JPRB ,&
&  0.77164E+00_JPRB, 0.88329E+00_JPRB, 0.10534E+01_JPRB, 0.12459E+01_JPRB, 0.17205E+01_JPRB, 0.21061E+01_JPRB, 0.21441E+01_JPRB ,&
&  0.17587E+01_JPRB, 0.17719E+01_JPRB, 0.16514E+01_JPRB, 0.15741E+01_JPRB, 0.15032E+01_JPRB, 0.13495E+01_JPRB, 0.12700E+01_JPRB ,&
&  0.11666E+01_JPRB, 0.11105E+01_JPRB, 0.10735E+01_JPRB, 0.10048E+01_JPRB, 0.95435E+00_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.10021E+01_JPRB, 0.95091E+00_JPRB, 0.88068E+00_JPRB, 0.83139E+00_JPRB, 0.79001E+00_JPRB, 0.76719E+00_JPRB, 0.76176E+00_JPRB ,&
&  0.75838E+00_JPRB, 0.74985E+00_JPRB, 0.75772E+00_JPRB, 0.76943E+00_JPRB, 0.77156E+00_JPRB, 0.74930E+00_JPRB, 0.68740E+00_JPRB ,&
&  0.63384E+00_JPRB, 0.55693E+00_JPRB, 0.48911E+00_JPRB, 0.48298E+00_JPRB, 0.51198E+00_JPRB, 0.53682E+00_JPRB, 0.57821E+00_JPRB ,&
&  0.65982E+00_JPRB, 0.79512E+00_JPRB, 0.86314E+00_JPRB, 0.92378E+00_JPRB, 0.94254E+00_JPRB, 0.92260E+00_JPRB, 0.88083E+00_JPRB ,&
&  0.80900E+00_JPRB, 0.81630E+00_JPRB, 0.86784E+00_JPRB, 0.86431E+00_JPRB, 0.84637E+00_JPRB, 0.85052E+00_JPRB, 0.86171E+00_JPRB ,&
&  0.86318E+00_JPRB, 0.86662E+00_JPRB, 0.90894E+00_JPRB, 0.93369E+00_JPRB, 0.98825E+00_JPRB, 0.10214E+01_JPRB, 0.10430E+01_JPRB ,&
&  0.10053E+01_JPRB, 0.99646E+00_JPRB, 0.11424E+01_JPRB, 0.17041E+01_JPRB, 0.22259E+01_JPRB, 0.26254E+01_JPRB, 0.25595E+01_JPRB ,&
&  0.28410E+01_JPRB, 0.30666E+01_JPRB, 0.32124E+01_JPRB, 0.36082E+01_JPRB, 0.29472E+01_JPRB, 0.32587E+01_JPRB, 0.37114E+01_JPRB ,&
&  0.35630E+01_JPRB, 0.32273E+01_JPRB, 0.30561E+01_JPRB, 0.29578E+01_JPRB, 0.33312E+01_JPRB, 0.44599E+01_JPRB, 0.41072E+01_JPRB ,&
&  0.34177E+01_JPRB, 0.30424E+01_JPRB, 0.26281E+01_JPRB, 0.25301E+01_JPRB, 0.21829E+01_JPRB, 0.19026E+01_JPRB, 0.13456E+01_JPRB ,&
&  0.13081E+01_JPRB, 0.11535E+01_JPRB, 0.11427E+01_JPRB, 0.10493E+01_JPRB, 0.10496E+01_JPRB, 0.10012E+01_JPRB, 0.98791E+00_JPRB ,&
&  0.10841E+01_JPRB, 0.11945E+01_JPRB, 0.15637E+01_JPRB, 0.19050E+01_JPRB, 0.27228E+01_JPRB, 0.26665E+01_JPRB, 0.26874E+01_JPRB ,&
&  0.24263E+01_JPRB, 0.22830E+01_JPRB, 0.18875E+01_JPRB, 0.18457E+01_JPRB, 0.16572E+01_JPRB, 0.15071E+01_JPRB, 0.13828E+01_JPRB ,&
&  0.12340E+01_JPRB, 0.11304E+01_JPRB, 0.10659E+01_JPRB, 0.10234E+01_JPRB, 0.10207E+01_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.96747E+00_JPRB, 0.92312E+00_JPRB, 0.87343E+00_JPRB, 0.82403E+00_JPRB, 0.77324E+00_JPRB, 0.74947E+00_JPRB, 0.74168E+00_JPRB ,&
&  0.74222E+00_JPRB, 0.75607E+00_JPRB, 0.76323E+00_JPRB, 0.76444E+00_JPRB, 0.76356E+00_JPRB, 0.76471E+00_JPRB, 0.69904E+00_JPRB ,&
&  0.63543E+00_JPRB, 0.56898E+00_JPRB, 0.49560E+00_JPRB, 0.48703E+00_JPRB, 0.57465E+00_JPRB, 0.64150E+00_JPRB, 0.63236E+00_JPRB ,&
&  0.85484E+00_JPRB, 0.10634E+01_JPRB, 0.11191E+01_JPRB, 0.11828E+01_JPRB, 0.12474E+01_JPRB, 0.12603E+01_JPRB, 0.11559E+01_JPRB ,&
&  0.11045E+01_JPRB, 0.10685E+01_JPRB, 0.97822E+00_JPRB, 0.90722E+00_JPRB, 0.86242E+00_JPRB, 0.84049E+00_JPRB, 0.81589E+00_JPRB ,&
&  0.80760E+00_JPRB, 0.82132E+00_JPRB, 0.84175E+00_JPRB, 0.87549E+00_JPRB, 0.94234E+00_JPRB, 0.92409E+00_JPRB, 0.97533E+00_JPRB ,&
&  0.10371E+01_JPRB, 0.11260E+01_JPRB, 0.14668E+01_JPRB, 0.21726E+01_JPRB, 0.30616E+01_JPRB, 0.36373E+01_JPRB, 0.37341E+01_JPRB ,&
&  0.35145E+01_JPRB, 0.35038E+01_JPRB, 0.36487E+01_JPRB, 0.36191E+01_JPRB, 0.35475E+01_JPRB, 0.35571E+01_JPRB, 0.38049E+01_JPRB ,&
&  0.29769E+01_JPRB, 0.24601E+01_JPRB, 0.21801E+01_JPRB, 0.26009E+01_JPRB, 0.41862E+01_JPRB, 0.54177E+01_JPRB, 0.53490E+01_JPRB ,&
&  0.44803E+01_JPRB, 0.37277E+01_JPRB, 0.31317E+01_JPRB, 0.26726E+01_JPRB, 0.25814E+01_JPRB, 0.17700E+01_JPRB, 0.18560E+01_JPRB ,&
&  0.18475E+01_JPRB, 0.18400E+01_JPRB, 0.16675E+01_JPRB, 0.13898E+01_JPRB, 0.12424E+01_JPRB, 0.13606E+01_JPRB, 0.14328E+01_JPRB ,&
&  0.15836E+01_JPRB, 0.17499E+01_JPRB, 0.26971E+01_JPRB, 0.31975E+01_JPRB, 0.36601E+01_JPRB, 0.32903E+01_JPRB, 0.29610E+01_JPRB ,&
&  0.24517E+01_JPRB, 0.23402E+01_JPRB, 0.22539E+01_JPRB, 0.20696E+01_JPRB, 0.18549E+01_JPRB, 0.16812E+01_JPRB, 0.14966E+01_JPRB ,&
&  0.13394E+01_JPRB, 0.11991E+01_JPRB, 0.10857E+01_JPRB, 0.10125E+01_JPRB, 0.99045E+00_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.97492E+00_JPRB, 0.93650E+00_JPRB, 0.88083E+00_JPRB, 0.80114E+00_JPRB, 0.76119E+00_JPRB, 0.75643E+00_JPRB, 0.77329E+00_JPRB ,&
&  0.78487E+00_JPRB, 0.78837E+00_JPRB, 0.77556E+00_JPRB, 0.75816E+00_JPRB, 0.75920E+00_JPRB, 0.74680E+00_JPRB, 0.70197E+00_JPRB ,&
&  0.66356E+00_JPRB, 0.62341E+00_JPRB, 0.53204E+00_JPRB, 0.54342E+00_JPRB, 0.66444E+00_JPRB, 0.76574E+00_JPRB, 0.78498E+00_JPRB ,&
&  0.11736E+01_JPRB, 0.12724E+01_JPRB, 0.11829E+01_JPRB, 0.12315E+01_JPRB, 0.14774E+01_JPRB, 0.13683E+01_JPRB, 0.12415E+01_JPRB ,&
&  0.12127E+01_JPRB, 0.11324E+01_JPRB, 0.10090E+01_JPRB, 0.91523E+00_JPRB, 0.85719E+00_JPRB, 0.82488E+00_JPRB, 0.79998E+00_JPRB ,&
&  0.79650E+00_JPRB, 0.80881E+00_JPRB, 0.86200E+00_JPRB, 0.92277E+00_JPRB, 0.91072E+00_JPRB, 0.90369E+00_JPRB, 0.10019E+01_JPRB ,&
&  0.11030E+01_JPRB, 0.12273E+01_JPRB, 0.16009E+01_JPRB, 0.23048E+01_JPRB, 0.33318E+01_JPRB, 0.42361E+01_JPRB, 0.52570E+01_JPRB ,&
&  0.46381E+01_JPRB, 0.47012E+01_JPRB, 0.41964E+01_JPRB, 0.46653E+01_JPRB, 0.41849E+01_JPRB, 0.39172E+01_JPRB, 0.37945E+01_JPRB ,&
&  0.27407E+01_JPRB, 0.23249E+01_JPRB, 0.26651E+01_JPRB, 0.40305E+01_JPRB, 0.44932E+01_JPRB, 0.50813E+01_JPRB, 0.61897E+01_JPRB ,&
&  0.48905E+01_JPRB, 0.40360E+01_JPRB, 0.35807E+01_JPRB, 0.34769E+01_JPRB, 0.22202E+01_JPRB, 0.16822E+01_JPRB, 0.24552E+01_JPRB ,&
&  0.22898E+01_JPRB, 0.13600E+01_JPRB, 0.10344E+01_JPRB, 0.11837E+01_JPRB, 0.93627E+00_JPRB, 0.11107E+01_JPRB, 0.16314E+01_JPRB ,&
&  0.18966E+01_JPRB, 0.22236E+01_JPRB, 0.35017E+01_JPRB, 0.34185E+01_JPRB, 0.35941E+01_JPRB, 0.37395E+01_JPRB, 0.30140E+01_JPRB ,&
&  0.26826E+01_JPRB, 0.21894E+01_JPRB, 0.22148E+01_JPRB, 0.20120E+01_JPRB, 0.18641E+01_JPRB, 0.17542E+01_JPRB, 0.15771E+01_JPRB ,&
&  0.14081E+01_JPRB, 0.12481E+01_JPRB, 0.11169E+01_JPRB, 0.10326E+01_JPRB, 0.10001E+01_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.95529E+00_JPRB, 0.91363E+00_JPRB, 0.88957E+00_JPRB, 0.84343E+00_JPRB, 0.80798E+00_JPRB, 0.79950E+00_JPRB, 0.81887E+00_JPRB ,&
&  0.80555E+00_JPRB, 0.77033E+00_JPRB, 0.73382E+00_JPRB, 0.72282E+00_JPRB, 0.73344E+00_JPRB, 0.73243E+00_JPRB, 0.69564E+00_JPRB ,&
&  0.66336E+00_JPRB, 0.63355E+00_JPRB, 0.62612E+00_JPRB, 0.67300E+00_JPRB, 0.87404E+00_JPRB, 0.91068E+00_JPRB, 0.11333E+01_JPRB ,&
&  0.14672E+01_JPRB, 0.13688E+01_JPRB, 0.12532E+01_JPRB, 0.12984E+01_JPRB, 0.12661E+01_JPRB, 0.11740E+01_JPRB, 0.12056E+01_JPRB ,&
&  0.11615E+01_JPRB, 0.10963E+01_JPRB, 0.10014E+01_JPRB, 0.93114E+00_JPRB, 0.86128E+00_JPRB, 0.81181E+00_JPRB, 0.80022E+00_JPRB ,&
&  0.80825E+00_JPRB, 0.85661E+00_JPRB, 0.94965E+00_JPRB, 0.91165E+00_JPRB, 0.90841E+00_JPRB, 0.92917E+00_JPRB, 0.10416E+01_JPRB ,&
&  0.11695E+01_JPRB, 0.13296E+01_JPRB, 0.16731E+01_JPRB, 0.24648E+01_JPRB, 0.41520E+01_JPRB, 0.55025E+01_JPRB, 0.54495E+01_JPRB ,&
&  0.48907E+01_JPRB, 0.46867E+01_JPRB, 0.48627E+01_JPRB, 0.48966E+01_JPRB, 0.41863E+01_JPRB, 0.39985E+01_JPRB, 0.36805E+01_JPRB ,&
&  0.31808E+01_JPRB, 0.29528E+01_JPRB, 0.30539E+01_JPRB, 0.43105E+01_JPRB, 0.52639E+01_JPRB, 0.53746E+01_JPRB, 0.61097E+01_JPRB ,&
&  0.54304E+01_JPRB, 0.43790E+01_JPRB, 0.32775E+01_JPRB, 0.25943E+01_JPRB, 0.25709E+01_JPRB, 0.22501E+01_JPRB, 0.16468E+01_JPRB ,&
&  0.13893E+01_JPRB, 0.11478E+01_JPRB, 0.96301E+00_JPRB, 0.94844E+00_JPRB, 0.82586E+00_JPRB, 0.96685E+00_JPRB, 0.14567E+01_JPRB ,&
&  0.21235E+01_JPRB, 0.25616E+01_JPRB, 0.32680E+01_JPRB, 0.33431E+01_JPRB, 0.34414E+01_JPRB, 0.40137E+01_JPRB, 0.37696E+01_JPRB ,&
&  0.23848E+01_JPRB, 0.20837E+01_JPRB, 0.21131E+01_JPRB, 0.19882E+01_JPRB, 0.18558E+01_JPRB, 0.17223E+01_JPRB, 0.15397E+01_JPRB ,&
&  0.13521E+01_JPRB, 0.11855E+01_JPRB, 0.10907E+01_JPRB, 0.10571E+01_JPRB, 0.10181E+01_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.86125E+00_JPRB, 0.85226E+00_JPRB, 0.85570E+00_JPRB, 0.84276E+00_JPRB, 0.82318E+00_JPRB, 0.82793E+00_JPRB, 0.80769E+00_JPRB ,&
&  0.76214E+00_JPRB, 0.71708E+00_JPRB, 0.69552E+00_JPRB, 0.70300E+00_JPRB, 0.71418E+00_JPRB, 0.71930E+00_JPRB, 0.69475E+00_JPRB ,&
&  0.66097E+00_JPRB, 0.65162E+00_JPRB, 0.69098E+00_JPRB, 0.82167E+00_JPRB, 0.11850E+01_JPRB, 0.11601E+01_JPRB, 0.15474E+01_JPRB ,&
&  0.16870E+01_JPRB, 0.14030E+01_JPRB, 0.13020E+01_JPRB, 0.12438E+01_JPRB, 0.12167E+01_JPRB, 0.11167E+01_JPRB, 0.11101E+01_JPRB ,&
&  0.10177E+01_JPRB, 0.10251E+01_JPRB, 0.93033E+00_JPRB, 0.87374E+00_JPRB, 0.82546E+00_JPRB, 0.80969E+00_JPRB, 0.83512E+00_JPRB ,&
&  0.89151E+00_JPRB, 0.96615E+00_JPRB, 0.94738E+00_JPRB, 0.93265E+00_JPRB, 0.97073E+00_JPRB, 0.10137E+01_JPRB, 0.11250E+01_JPRB ,&
&  0.12691E+01_JPRB, 0.14606E+01_JPRB, 0.18572E+01_JPRB, 0.31495E+01_JPRB, 0.44171E+01_JPRB, 0.47459E+01_JPRB, 0.52905E+01_JPRB ,&
&  0.54459E+01_JPRB, 0.55153E+01_JPRB, 0.47499E+01_JPRB, 0.51757E+01_JPRB, 0.42394E+01_JPRB, 0.38209E+01_JPRB, 0.36024E+01_JPRB ,&
&  0.32474E+01_JPRB, 0.35773E+01_JPRB, 0.35228E+01_JPRB, 0.38097E+01_JPRB, 0.52898E+01_JPRB, 0.61506E+01_JPRB, 0.60571E+01_JPRB ,&
&  0.57553E+01_JPRB, 0.47366E+01_JPRB, 0.36958E+01_JPRB, 0.31859E+01_JPRB, 0.39782E+01_JPRB, 0.42465E+01_JPRB, 0.20063E+01_JPRB ,&
&  0.13810E+01_JPRB, 0.12613E+01_JPRB, 0.95826E+00_JPRB, 0.82055E+00_JPRB, 0.78446E+00_JPRB, 0.95281E+00_JPRB, 0.16396E+01_JPRB ,&
&  0.18505E+01_JPRB, 0.29345E+01_JPRB, 0.32161E+01_JPRB, 0.34421E+01_JPRB, 0.33058E+01_JPRB, 0.33087E+01_JPRB, 0.32203E+01_JPRB ,&
&  0.32335E+01_JPRB, 0.28105E+01_JPRB, 0.23169E+01_JPRB, 0.20423E+01_JPRB, 0.17871E+01_JPRB, 0.15601E+01_JPRB, 0.13954E+01_JPRB ,&
&  0.12830E+01_JPRB, 0.11577E+01_JPRB, 0.10819E+01_JPRB, 0.10290E+01_JPRB, 0.93121E+00_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.76272E+00_JPRB, 0.78329E+00_JPRB, 0.81208E+00_JPRB, 0.80743E+00_JPRB, 0.78851E+00_JPRB, 0.79292E+00_JPRB, 0.75034E+00_JPRB ,&
&  0.70857E+00_JPRB, 0.68963E+00_JPRB, 0.68603E+00_JPRB, 0.70188E+00_JPRB, 0.73325E+00_JPRB, 0.75565E+00_JPRB, 0.73333E+00_JPRB ,&
&  0.69524E+00_JPRB, 0.69214E+00_JPRB, 0.74738E+00_JPRB, 0.11055E+01_JPRB, 0.14437E+01_JPRB, 0.17185E+01_JPRB, 0.19333E+01_JPRB ,&
&  0.18259E+01_JPRB, 0.15113E+01_JPRB, 0.13470E+01_JPRB, 0.13241E+01_JPRB, 0.12870E+01_JPRB, 0.12377E+01_JPRB, 0.11227E+01_JPRB ,&
&  0.99943E+00_JPRB, 0.99492E+00_JPRB, 0.87942E+00_JPRB, 0.84346E+00_JPRB, 0.83079E+00_JPRB, 0.84878E+00_JPRB, 0.90991E+00_JPRB ,&
&  0.98468E+00_JPRB, 0.97387E+00_JPRB, 0.99817E+00_JPRB, 0.10439E+01_JPRB, 0.11122E+01_JPRB, 0.11573E+01_JPRB, 0.12505E+01_JPRB ,&
&  0.14248E+01_JPRB, 0.17129E+01_JPRB, 0.23213E+01_JPRB, 0.37654E+01_JPRB, 0.39954E+01_JPRB, 0.51456E+01_JPRB, 0.53144E+01_JPRB ,&
&  0.52862E+01_JPRB, 0.51483E+01_JPRB, 0.45662E+01_JPRB, 0.46763E+01_JPRB, 0.41776E+01_JPRB, 0.37946E+01_JPRB, 0.34835E+01_JPRB ,&
&  0.37153E+01_JPRB, 0.36211E+01_JPRB, 0.44625E+01_JPRB, 0.43169E+01_JPRB, 0.48862E+01_JPRB, 0.79294E+01_JPRB, 0.61627E+01_JPRB ,&
&  0.51151E+01_JPRB, 0.52398E+01_JPRB, 0.42150E+01_JPRB, 0.37611E+01_JPRB, 0.44806E+01_JPRB, 0.52631E+01_JPRB, 0.42466E+01_JPRB ,&
&  0.16883E+01_JPRB, 0.11517E+01_JPRB, 0.10013E+01_JPRB, 0.88413E+00_JPRB, 0.76419E+00_JPRB, 0.92541E+00_JPRB, 0.28414E+01_JPRB ,&
&  0.21126E+01_JPRB, 0.29295E+01_JPRB, 0.30200E+01_JPRB, 0.30619E+01_JPRB, 0.32385E+01_JPRB, 0.29432E+01_JPRB, 0.29391E+01_JPRB ,&
&  0.24800E+01_JPRB, 0.20157E+01_JPRB, 0.16772E+01_JPRB, 0.14427E+01_JPRB, 0.13109E+01_JPRB, 0.12429E+01_JPRB, 0.12154E+01_JPRB ,&
&  0.11759E+01_JPRB, 0.10807E+01_JPRB, 0.99105E+00_JPRB, 0.87376E+00_JPRB, 0.77338E+00_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.76697E+00_JPRB, 0.77067E+00_JPRB, 0.78068E+00_JPRB, 0.80547E+00_JPRB, 0.82388E+00_JPRB, 0.80429E+00_JPRB, 0.76433E+00_JPRB ,&
&  0.76931E+00_JPRB, 0.77007E+00_JPRB, 0.80250E+00_JPRB, 0.82645E+00_JPRB, 0.85769E+00_JPRB, 0.84860E+00_JPRB, 0.81559E+00_JPRB ,&
&  0.76830E+00_JPRB, 0.77884E+00_JPRB, 0.97809E+00_JPRB, 0.13960E+01_JPRB, 0.16459E+01_JPRB, 0.18260E+01_JPRB, 0.21194E+01_JPRB ,&
&  0.21308E+01_JPRB, 0.16352E+01_JPRB, 0.13428E+01_JPRB, 0.13887E+01_JPRB, 0.14014E+01_JPRB, 0.14183E+01_JPRB, 0.12134E+01_JPRB ,&
&  0.10632E+01_JPRB, 0.10100E+01_JPRB, 0.90098E+00_JPRB, 0.88938E+00_JPRB, 0.90195E+00_JPRB, 0.93276E+00_JPRB, 0.98839E+00_JPRB ,&
&  0.10293E+01_JPRB, 0.11027E+01_JPRB, 0.11970E+01_JPRB, 0.12756E+01_JPRB, 0.13331E+01_JPRB, 0.13734E+01_JPRB, 0.14733E+01_JPRB ,&
&  0.17005E+01_JPRB, 0.20857E+01_JPRB, 0.27659E+01_JPRB, 0.37079E+01_JPRB, 0.45009E+01_JPRB, 0.47490E+01_JPRB, 0.50579E+01_JPRB ,&
&  0.50324E+01_JPRB, 0.49676E+01_JPRB, 0.46825E+01_JPRB, 0.43054E+01_JPRB, 0.38734E+01_JPRB, 0.37812E+01_JPRB, 0.35610E+01_JPRB ,&
&  0.39800E+01_JPRB, 0.44559E+01_JPRB, 0.51872E+01_JPRB, 0.50191E+01_JPRB, 0.52592E+01_JPRB, 0.74674E+01_JPRB, 0.79183E+01_JPRB ,&
&  0.63647E+01_JPRB, 0.46022E+01_JPRB, 0.38974E+01_JPRB, 0.40754E+01_JPRB, 0.44167E+01_JPRB, 0.46948E+01_JPRB, 0.62163E+01_JPRB ,&
&  0.49055E+01_JPRB, 0.21851E+01_JPRB, 0.12541E+01_JPRB, 0.12341E+01_JPRB, 0.74190E+00_JPRB, 0.68103E+00_JPRB, 0.12769E+01_JPRB ,&
&  0.17592E+01_JPRB, 0.22505E+01_JPRB, 0.23303E+01_JPRB, 0.26364E+01_JPRB, 0.26988E+01_JPRB, 0.22623E+01_JPRB, 0.17608E+01_JPRB ,&
&  0.13503E+01_JPRB, 0.11336E+01_JPRB, 0.10249E+01_JPRB, 0.97238E+00_JPRB, 0.10035E+01_JPRB, 0.10566E+01_JPRB, 0.10924E+01_JPRB ,&
&  0.10700E+01_JPRB, 0.98323E+00_JPRB, 0.85345E+00_JPRB, 0.75846E+00_JPRB, 0.74162E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.85955E+00_JPRB, 0.84964E+00_JPRB, 0.85569E+00_JPRB, 0.88847E+00_JPRB, 0.90307E+00_JPRB, 0.91178E+00_JPRB, 0.97944E+00_JPRB ,&
&  0.10161E+01_JPRB, 0.10280E+01_JPRB, 0.10458E+01_JPRB, 0.10497E+01_JPRB, 0.10288E+01_JPRB, 0.96978E+00_JPRB, 0.92390E+00_JPRB ,&
&  0.92555E+00_JPRB, 0.10861E+01_JPRB, 0.14328E+01_JPRB, 0.18636E+01_JPRB, 0.20333E+01_JPRB, 0.25369E+01_JPRB, 0.20485E+01_JPRB ,&
&  0.20254E+01_JPRB, 0.19979E+01_JPRB, 0.15029E+01_JPRB, 0.15545E+01_JPRB, 0.15604E+01_JPRB, 0.16271E+01_JPRB, 0.13025E+01_JPRB ,&
&  0.11581E+01_JPRB, 0.10959E+01_JPRB, 0.99326E+00_JPRB, 0.97888E+00_JPRB, 0.10077E+01_JPRB, 0.10710E+01_JPRB, 0.11334E+01_JPRB ,&
&  0.12391E+01_JPRB, 0.13898E+01_JPRB, 0.15221E+01_JPRB, 0.15879E+01_JPRB, 0.16275E+01_JPRB, 0.16759E+01_JPRB, 0.18249E+01_JPRB ,&
&  0.20741E+01_JPRB, 0.24896E+01_JPRB, 0.31711E+01_JPRB, 0.39063E+01_JPRB, 0.43806E+01_JPRB, 0.43755E+01_JPRB, 0.46026E+01_JPRB ,&
&  0.44058E+01_JPRB, 0.41128E+01_JPRB, 0.41740E+01_JPRB, 0.40342E+01_JPRB, 0.37484E+01_JPRB, 0.35864E+01_JPRB, 0.34600E+01_JPRB ,&
&  0.43623E+01_JPRB, 0.57534E+01_JPRB, 0.73525E+01_JPRB, 0.63758E+01_JPRB, 0.62836E+01_JPRB, 0.76966E+01_JPRB, 0.79510E+01_JPRB ,&
&  0.68485E+01_JPRB, 0.50285E+01_JPRB, 0.44408E+01_JPRB, 0.39645E+01_JPRB, 0.38399E+01_JPRB, 0.38956E+01_JPRB, 0.46206E+01_JPRB ,&
&  0.47816E+01_JPRB, 0.47415E+01_JPRB, 0.24710E+01_JPRB, 0.10860E+01_JPRB, 0.76663E+00_JPRB, 0.83155E+00_JPRB, 0.10389E+01_JPRB ,&
&  0.14005E+01_JPRB, 0.16340E+01_JPRB, 0.17381E+01_JPRB, 0.22603E+01_JPRB, 0.20235E+01_JPRB, 0.16096E+01_JPRB, 0.12739E+01_JPRB ,&
&  0.10676E+01_JPRB, 0.91600E+00_JPRB, 0.84481E+00_JPRB, 0.84549E+00_JPRB, 0.91570E+00_JPRB, 0.99183E+00_JPRB, 0.10244E+01_JPRB ,&
&  0.96097E+00_JPRB, 0.85605E+00_JPRB, 0.80844E+00_JPRB, 0.80603E+00_JPRB, 0.83102E+00_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.99114E+00_JPRB, 0.10131E+01_JPRB, 0.10596E+01_JPRB, 0.11300E+01_JPRB, 0.11971E+01_JPRB, 0.12715E+01_JPRB, 0.13295E+01_JPRB ,&
&  0.13505E+01_JPRB, 0.13315E+01_JPRB, 0.13151E+01_JPRB, 0.12673E+01_JPRB, 0.12385E+01_JPRB, 0.12278E+01_JPRB, 0.12980E+01_JPRB ,&
&  0.15137E+01_JPRB, 0.18401E+01_JPRB, 0.21564E+01_JPRB, 0.23319E+01_JPRB, 0.23682E+01_JPRB, 0.26281E+01_JPRB, 0.25968E+01_JPRB ,&
&  0.19300E+01_JPRB, 0.25621E+01_JPRB, 0.17082E+01_JPRB, 0.17687E+01_JPRB, 0.18229E+01_JPRB, 0.19855E+01_JPRB, 0.16031E+01_JPRB ,&
&  0.13443E+01_JPRB, 0.12834E+01_JPRB, 0.12349E+01_JPRB, 0.12257E+01_JPRB, 0.12508E+01_JPRB, 0.13351E+01_JPRB, 0.14697E+01_JPRB ,&
&  0.16115E+01_JPRB, 0.17302E+01_JPRB, 0.18203E+01_JPRB, 0.18762E+01_JPRB, 0.19439E+01_JPRB, 0.20300E+01_JPRB, 0.22426E+01_JPRB ,&
&  0.25316E+01_JPRB, 0.29915E+01_JPRB, 0.35439E+01_JPRB, 0.41200E+01_JPRB, 0.41133E+01_JPRB, 0.40550E+01_JPRB, 0.41769E+01_JPRB ,&
&  0.40416E+01_JPRB, 0.37729E+01_JPRB, 0.38048E+01_JPRB, 0.35794E+01_JPRB, 0.33449E+01_JPRB, 0.35182E+01_JPRB, 0.36321E+01_JPRB ,&
&  0.50996E+01_JPRB, 0.61646E+01_JPRB, 0.84010E+01_JPRB, 0.77857E+01_JPRB, 0.67978E+01_JPRB, 0.71263E+01_JPRB, 0.61483E+01_JPRB ,&
&  0.47454E+01_JPRB, 0.42359E+01_JPRB, 0.38777E+01_JPRB, 0.35118E+01_JPRB, 0.33295E+01_JPRB, 0.32943E+01_JPRB, 0.36046E+01_JPRB ,&
&  0.36981E+01_JPRB, 0.34944E+01_JPRB, 0.24672E+01_JPRB, 0.11954E+01_JPRB, 0.88770E+00_JPRB, 0.84740E+00_JPRB, 0.10043E+01_JPRB ,&
&  0.13516E+01_JPRB, 0.15150E+01_JPRB, 0.15535E+01_JPRB, 0.14780E+01_JPRB, 0.12272E+01_JPRB, 0.10166E+01_JPRB, 0.86825E+00_JPRB ,&
&  0.82302E+00_JPRB, 0.80961E+00_JPRB, 0.81381E+00_JPRB, 0.87705E+00_JPRB, 0.94587E+00_JPRB, 0.98075E+00_JPRB, 0.90484E+00_JPRB ,&
&  0.85073E+00_JPRB, 0.84646E+00_JPRB, 0.88950E+00_JPRB, 0.93210E+00_JPRB, 0.96887E+00_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.11678E+01_JPRB, 0.12346E+01_JPRB, 0.13292E+01_JPRB, 0.14721E+01_JPRB, 0.16639E+01_JPRB, 0.18627E+01_JPRB, 0.18069E+01_JPRB ,&
&  0.17189E+01_JPRB, 0.15976E+01_JPRB, 0.16530E+01_JPRB, 0.17085E+01_JPRB, 0.17683E+01_JPRB, 0.18732E+01_JPRB, 0.20304E+01_JPRB ,&
&  0.22174E+01_JPRB, 0.23983E+01_JPRB, 0.25670E+01_JPRB, 0.27263E+01_JPRB, 0.28898E+01_JPRB, 0.33020E+01_JPRB, 0.34198E+01_JPRB ,&
&  0.29339E+01_JPRB, 0.20125E+01_JPRB, 0.21181E+01_JPRB, 0.18800E+01_JPRB, 0.19159E+01_JPRB, 0.20788E+01_JPRB, 0.19984E+01_JPRB ,&
&  0.17090E+01_JPRB, 0.16013E+01_JPRB, 0.15624E+01_JPRB, 0.15670E+01_JPRB, 0.15705E+01_JPRB, 0.16843E+01_JPRB, 0.18033E+01_JPRB ,&
&  0.19013E+01_JPRB, 0.19731E+01_JPRB, 0.20533E+01_JPRB, 0.21255E+01_JPRB, 0.22118E+01_JPRB, 0.23516E+01_JPRB, 0.26139E+01_JPRB ,&
&  0.29078E+01_JPRB, 0.32861E+01_JPRB, 0.36796E+01_JPRB, 0.38887E+01_JPRB, 0.38312E+01_JPRB, 0.37721E+01_JPRB, 0.37906E+01_JPRB ,&
&  0.37451E+01_JPRB, 0.37001E+01_JPRB, 0.36059E+01_JPRB, 0.34545E+01_JPRB, 0.32826E+01_JPRB, 0.38002E+01_JPRB, 0.44333E+01_JPRB ,&
&  0.57336E+01_JPRB, 0.66343E+01_JPRB, 0.85118E+01_JPRB, 0.75018E+01_JPRB, 0.64887E+01_JPRB, 0.58238E+01_JPRB, 0.49232E+01_JPRB ,&
&  0.38883E+01_JPRB, 0.37046E+01_JPRB, 0.34359E+01_JPRB, 0.28833E+01_JPRB, 0.25137E+01_JPRB, 0.22360E+01_JPRB, 0.26844E+01_JPRB ,&
&  0.28772E+01_JPRB, 0.28838E+01_JPRB, 0.18367E+01_JPRB, 0.12425E+01_JPRB, 0.96624E+00_JPRB, 0.90214E+00_JPRB, 0.10723E+01_JPRB ,&
&  0.13957E+01_JPRB, 0.14006E+01_JPRB, 0.12300E+01_JPRB, 0.11742E+01_JPRB, 0.10162E+01_JPRB, 0.84844E+00_JPRB, 0.72565E+00_JPRB ,&
&  0.74880E+00_JPRB, 0.76217E+00_JPRB, 0.79548E+00_JPRB, 0.85875E+00_JPRB, 0.88717E+00_JPRB, 0.92916E+00_JPRB, 0.92576E+00_JPRB ,&
&  0.96744E+00_JPRB, 0.10111E+01_JPRB, 0.10427E+01_JPRB, 0.10768E+01_JPRB, 0.11137E+01_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.12444E+01_JPRB, 0.13135E+01_JPRB, 0.14013E+01_JPRB, 0.15106E+01_JPRB, 0.16252E+01_JPRB, 0.16947E+01_JPRB, 0.17293E+01_JPRB ,&
&  0.18083E+01_JPRB, 0.19111E+01_JPRB, 0.20250E+01_JPRB, 0.21137E+01_JPRB, 0.21661E+01_JPRB, 0.22495E+01_JPRB, 0.23448E+01_JPRB ,&
&  0.24402E+01_JPRB, 0.25522E+01_JPRB, 0.26963E+01_JPRB, 0.28731E+01_JPRB, 0.31184E+01_JPRB, 0.34359E+01_JPRB, 0.36058E+01_JPRB ,&
&  0.28696E+01_JPRB, 0.18754E+01_JPRB, 0.22399E+01_JPRB, 0.21494E+01_JPRB, 0.20463E+01_JPRB, 0.20590E+01_JPRB, 0.19938E+01_JPRB ,&
&  0.18697E+01_JPRB, 0.18493E+01_JPRB, 0.18871E+01_JPRB, 0.19002E+01_JPRB, 0.18805E+01_JPRB, 0.19560E+01_JPRB, 0.20336E+01_JPRB ,&
&  0.21030E+01_JPRB, 0.21697E+01_JPRB, 0.22394E+01_JPRB, 0.22959E+01_JPRB, 0.23882E+01_JPRB, 0.25418E+01_JPRB, 0.27848E+01_JPRB ,&
&  0.30754E+01_JPRB, 0.33351E+01_JPRB, 0.35783E+01_JPRB, 0.37259E+01_JPRB, 0.36823E+01_JPRB, 0.36456E+01_JPRB, 0.36864E+01_JPRB ,&
&  0.37410E+01_JPRB, 0.36789E+01_JPRB, 0.37521E+01_JPRB, 0.40022E+01_JPRB, 0.43503E+01_JPRB, 0.44635E+01_JPRB, 0.54886E+01_JPRB ,&
&  0.64130E+01_JPRB, 0.68529E+01_JPRB, 0.73911E+01_JPRB, 0.87094E+01_JPRB, 0.55390E+01_JPRB, 0.48387E+01_JPRB, 0.39002E+01_JPRB ,&
&  0.39020E+01_JPRB, 0.36274E+01_JPRB, 0.31142E+01_JPRB, 0.24019E+01_JPRB, 0.19838E+01_JPRB, 0.15928E+01_JPRB, 0.18539E+01_JPRB ,&
&  0.22641E+01_JPRB, 0.22499E+01_JPRB, 0.16398E+01_JPRB, 0.13645E+01_JPRB, 0.11697E+01_JPRB, 0.12432E+01_JPRB, 0.13889E+01_JPRB ,&
&  0.14540E+01_JPRB, 0.12605E+01_JPRB, 0.11777E+01_JPRB, 0.12810E+01_JPRB, 0.10527E+01_JPRB, 0.73589E+00_JPRB, 0.61486E+00_JPRB ,&
&  0.62511E+00_JPRB, 0.66872E+00_JPRB, 0.73351E+00_JPRB, 0.80977E+00_JPRB, 0.85013E+00_JPRB, 0.89636E+00_JPRB, 0.93682E+00_JPRB ,&
&  0.10240E+01_JPRB, 0.10874E+01_JPRB, 0.11340E+01_JPRB, 0.11528E+01_JPRB, 0.11988E+01_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.12093E+01_JPRB, 0.12727E+01_JPRB, 0.13211E+01_JPRB, 0.13856E+01_JPRB, 0.14495E+01_JPRB, 0.15161E+01_JPRB, 0.16043E+01_JPRB ,&
&  0.17173E+01_JPRB, 0.18406E+01_JPRB, 0.19587E+01_JPRB, 0.20474E+01_JPRB, 0.21328E+01_JPRB, 0.21949E+01_JPRB, 0.22510E+01_JPRB ,&
&  0.22908E+01_JPRB, 0.23632E+01_JPRB, 0.24370E+01_JPRB, 0.25380E+01_JPRB, 0.26775E+01_JPRB, 0.28448E+01_JPRB, 0.30050E+01_JPRB ,&
&  0.30946E+01_JPRB, 0.29340E+01_JPRB, 0.25052E+01_JPRB, 0.18123E+01_JPRB, 0.16099E+01_JPRB, 0.18787E+01_JPRB, 0.19132E+01_JPRB ,&
&  0.19366E+01_JPRB, 0.19305E+01_JPRB, 0.19208E+01_JPRB, 0.19933E+01_JPRB, 0.20428E+01_JPRB, 0.20877E+01_JPRB, 0.21133E+01_JPRB ,&
&  0.21506E+01_JPRB, 0.22151E+01_JPRB, 0.22589E+01_JPRB, 0.23019E+01_JPRB, 0.23773E+01_JPRB, 0.25251E+01_JPRB, 0.27473E+01_JPRB ,&
&  0.30171E+01_JPRB, 0.32707E+01_JPRB, 0.34380E+01_JPRB, 0.35572E+01_JPRB, 0.35315E+01_JPRB, 0.36000E+01_JPRB, 0.37862E+01_JPRB ,&
&  0.40400E+01_JPRB, 0.41793E+01_JPRB, 0.44393E+01_JPRB, 0.49200E+01_JPRB, 0.52699E+01_JPRB, 0.51308E+01_JPRB, 0.57714E+01_JPRB ,&
&  0.64929E+01_JPRB, 0.67296E+01_JPRB, 0.66338E+01_JPRB, 0.84285E+01_JPRB, 0.52402E+01_JPRB, 0.50499E+01_JPRB, 0.45551E+01_JPRB ,&
&  0.40733E+01_JPRB, 0.39322E+01_JPRB, 0.29312E+01_JPRB, 0.20320E+01_JPRB, 0.16875E+01_JPRB, 0.13706E+01_JPRB, 0.16069E+01_JPRB ,&
&  0.19449E+01_JPRB, 0.17140E+01_JPRB, 0.13934E+01_JPRB, 0.12980E+01_JPRB, 0.12979E+01_JPRB, 0.15093E+01_JPRB, 0.16122E+01_JPRB ,&
&  0.12991E+01_JPRB, 0.11695E+01_JPRB, 0.11610E+01_JPRB, 0.14786E+01_JPRB, 0.13524E+01_JPRB, 0.68791E+00_JPRB, 0.56222E+00_JPRB ,&
&  0.57695E+00_JPRB, 0.62607E+00_JPRB, 0.68388E+00_JPRB, 0.72985E+00_JPRB, 0.77124E+00_JPRB, 0.82325E+00_JPRB, 0.88055E+00_JPRB ,&
&  0.94576E+00_JPRB, 0.10251E+01_JPRB, 0.10822E+01_JPRB, 0.11264E+01_JPRB, 0.11620E+01_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.97011E+00_JPRB, 0.10303E+01_JPRB, 0.10873E+01_JPRB, 0.11247E+01_JPRB, 0.11370E+01_JPRB, 0.11516E+01_JPRB, 0.11549E+01_JPRB ,&
&  0.11666E+01_JPRB, 0.11654E+01_JPRB, 0.12030E+01_JPRB, 0.12892E+01_JPRB, 0.14544E+01_JPRB, 0.15507E+01_JPRB, 0.16663E+01_JPRB ,&
&  0.16781E+01_JPRB, 0.16964E+01_JPRB, 0.17198E+01_JPRB, 0.17654E+01_JPRB, 0.18479E+01_JPRB, 0.19648E+01_JPRB, 0.22257E+01_JPRB ,&
&  0.27210E+01_JPRB, 0.30340E+01_JPRB, 0.29879E+01_JPRB, 0.26308E+01_JPRB, 0.20959E+01_JPRB, 0.19105E+01_JPRB, 0.21624E+01_JPRB ,&
&  0.22200E+01_JPRB, 0.20856E+01_JPRB, 0.20071E+01_JPRB, 0.20048E+01_JPRB, 0.20186E+01_JPRB, 0.20384E+01_JPRB, 0.20508E+01_JPRB ,&
&  0.20820E+01_JPRB, 0.21283E+01_JPRB, 0.21662E+01_JPRB, 0.22062E+01_JPRB, 0.22778E+01_JPRB, 0.23902E+01_JPRB, 0.25402E+01_JPRB ,&
&  0.27418E+01_JPRB, 0.29174E+01_JPRB, 0.30255E+01_JPRB, 0.30823E+01_JPRB, 0.32041E+01_JPRB, 0.34994E+01_JPRB, 0.39433E+01_JPRB ,&
&  0.43520E+01_JPRB, 0.45535E+01_JPRB, 0.50054E+01_JPRB, 0.54840E+01_JPRB, 0.55437E+01_JPRB, 0.55020E+01_JPRB, 0.58036E+01_JPRB ,&
&  0.60121E+01_JPRB, 0.60614E+01_JPRB, 0.52477E+01_JPRB, 0.72350E+01_JPRB, 0.73645E+01_JPRB, 0.61277E+01_JPRB, 0.47887E+01_JPRB ,&
&  0.42118E+01_JPRB, 0.38232E+01_JPRB, 0.30864E+01_JPRB, 0.22611E+01_JPRB, 0.18522E+01_JPRB, 0.18090E+01_JPRB, 0.17537E+01_JPRB ,&
&  0.17843E+01_JPRB, 0.14413E+01_JPRB, 0.13192E+01_JPRB, 0.13321E+01_JPRB, 0.12539E+01_JPRB, 0.14967E+01_JPRB, 0.13619E+01_JPRB ,&
&  0.10395E+01_JPRB, 0.10064E+01_JPRB, 0.10919E+01_JPRB, 0.15403E+01_JPRB, 0.22559E+01_JPRB, 0.65090E+00_JPRB, 0.50878E+00_JPRB ,&
&  0.52787E+00_JPRB, 0.56136E+00_JPRB, 0.60188E+00_JPRB, 0.62064E+00_JPRB, 0.62593E+00_JPRB, 0.66317E+00_JPRB, 0.71931E+00_JPRB ,&
&  0.75278E+00_JPRB, 0.77923E+00_JPRB, 0.81195E+00_JPRB, 0.87892E+00_JPRB, 0.92013E+00_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.65896E+00_JPRB, 0.69460E+00_JPRB, 0.71827E+00_JPRB, 0.74382E+00_JPRB, 0.75437E+00_JPRB, 0.74614E+00_JPRB, 0.71226E+00_JPRB ,&
&  0.68438E+00_JPRB, 0.65766E+00_JPRB, 0.64951E+00_JPRB, 0.63604E+00_JPRB, 0.66255E+00_JPRB, 0.71392E+00_JPRB, 0.75298E+00_JPRB ,&
&  0.84977E+00_JPRB, 0.79508E+00_JPRB, 0.79373E+00_JPRB, 0.82573E+00_JPRB, 0.83085E+00_JPRB, 0.87152E+00_JPRB, 0.10190E+01_JPRB ,&
&  0.12500E+01_JPRB, 0.15089E+01_JPRB, 0.18932E+01_JPRB, 0.25064E+01_JPRB, 0.26839E+01_JPRB, 0.21546E+01_JPRB, 0.22694E+01_JPRB ,&
&  0.20457E+01_JPRB, 0.17627E+01_JPRB, 0.17909E+01_JPRB, 0.17909E+01_JPRB, 0.18465E+01_JPRB, 0.18573E+01_JPRB, 0.19013E+01_JPRB ,&
&  0.19534E+01_JPRB, 0.20162E+01_JPRB, 0.20766E+01_JPRB, 0.21268E+01_JPRB, 0.21849E+01_JPRB, 0.22566E+01_JPRB, 0.23576E+01_JPRB ,&
&  0.24315E+01_JPRB, 0.24940E+01_JPRB, 0.26083E+01_JPRB, 0.26676E+01_JPRB, 0.30058E+01_JPRB, 0.34945E+01_JPRB, 0.39848E+01_JPRB ,&
&  0.43209E+01_JPRB, 0.45545E+01_JPRB, 0.49715E+01_JPRB, 0.51048E+01_JPRB, 0.51699E+01_JPRB, 0.52397E+01_JPRB, 0.51826E+01_JPRB ,&
&  0.49413E+01_JPRB, 0.40564E+01_JPRB, 0.36989E+01_JPRB, 0.57006E+01_JPRB, 0.54777E+01_JPRB, 0.42986E+01_JPRB, 0.42579E+01_JPRB ,&
&  0.39026E+01_JPRB, 0.36693E+01_JPRB, 0.31501E+01_JPRB, 0.25087E+01_JPRB, 0.20359E+01_JPRB, 0.17221E+01_JPRB, 0.16781E+01_JPRB ,&
&  0.14793E+01_JPRB, 0.12456E+01_JPRB, 0.12943E+01_JPRB, 0.13645E+01_JPRB, 0.12839E+01_JPRB, 0.13332E+01_JPRB, 0.11771E+01_JPRB ,&
&  0.10449E+01_JPRB, 0.87785E+00_JPRB, 0.99410E+00_JPRB, 0.13056E+01_JPRB, 0.97613E+00_JPRB, 0.53908E+00_JPRB, 0.46171E+00_JPRB ,&
&  0.47402E+00_JPRB, 0.48751E+00_JPRB, 0.50003E+00_JPRB, 0.50982E+00_JPRB, 0.51773E+00_JPRB, 0.53752E+00_JPRB, 0.55372E+00_JPRB ,&
&  0.57067E+00_JPRB, 0.60051E+00_JPRB, 0.61321E+00_JPRB, 0.62854E+00_JPRB, 0.64922E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.58471E+00_JPRB, 0.58343E+00_JPRB, 0.57689E+00_JPRB, 0.57862E+00_JPRB, 0.58476E+00_JPRB, 0.60044E+00_JPRB, 0.61042E+00_JPRB ,&
&  0.60995E+00_JPRB, 0.61597E+00_JPRB, 0.59937E+00_JPRB, 0.60199E+00_JPRB, 0.62129E+00_JPRB, 0.63524E+00_JPRB, 0.64933E+00_JPRB ,&
&  0.69280E+00_JPRB, 0.70709E+00_JPRB, 0.70475E+00_JPRB, 0.71025E+00_JPRB, 0.70389E+00_JPRB, 0.75399E+00_JPRB, 0.75831E+00_JPRB ,&
&  0.73849E+00_JPRB, 0.76430E+00_JPRB, 0.90252E+00_JPRB, 0.12574E+01_JPRB, 0.15929E+01_JPRB, 0.16138E+01_JPRB, 0.18788E+01_JPRB ,&
&  0.18020E+01_JPRB, 0.14111E+01_JPRB, 0.15057E+01_JPRB, 0.13370E+01_JPRB, 0.13724E+01_JPRB, 0.14412E+01_JPRB, 0.14598E+01_JPRB ,&
&  0.14944E+01_JPRB, 0.14963E+01_JPRB, 0.15240E+01_JPRB, 0.15653E+01_JPRB, 0.15904E+01_JPRB, 0.16254E+01_JPRB, 0.16695E+01_JPRB ,&
&  0.16133E+01_JPRB, 0.16828E+01_JPRB, 0.19041E+01_JPRB, 0.22798E+01_JPRB, 0.27102E+01_JPRB, 0.33124E+01_JPRB, 0.35884E+01_JPRB ,&
&  0.37862E+01_JPRB, 0.40114E+01_JPRB, 0.37932E+01_JPRB, 0.39145E+01_JPRB, 0.42485E+01_JPRB, 0.39865E+01_JPRB, 0.38173E+01_JPRB ,&
&  0.36359E+01_JPRB, 0.28573E+01_JPRB, 0.27295E+01_JPRB, 0.39580E+01_JPRB, 0.40970E+01_JPRB, 0.37818E+01_JPRB, 0.37711E+01_JPRB ,&
&  0.35759E+01_JPRB, 0.32638E+01_JPRB, 0.27446E+01_JPRB, 0.23268E+01_JPRB, 0.19588E+01_JPRB, 0.15975E+01_JPRB, 0.14887E+01_JPRB ,&
&  0.12171E+01_JPRB, 0.12029E+01_JPRB, 0.13898E+01_JPRB, 0.14667E+01_JPRB, 0.14577E+01_JPRB, 0.15757E+01_JPRB, 0.12140E+01_JPRB ,&
&  0.95070E+00_JPRB, 0.86951E+00_JPRB, 0.12179E+01_JPRB, 0.16461E+01_JPRB, 0.80337E+00_JPRB, 0.54234E+00_JPRB, 0.42883E+00_JPRB ,&
&  0.44570E+00_JPRB, 0.46434E+00_JPRB, 0.47048E+00_JPRB, 0.47849E+00_JPRB, 0.49500E+00_JPRB, 0.50917E+00_JPRB, 0.53214E+00_JPRB ,&
&  0.54612E+00_JPRB, 0.55733E+00_JPRB, 0.56894E+00_JPRB, 0.58390E+00_JPRB, 0.58071E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.63526E+00_JPRB, 0.64446E+00_JPRB, 0.65155E+00_JPRB, 0.67007E+00_JPRB, 0.69754E+00_JPRB, 0.70583E+00_JPRB, 0.72350E+00_JPRB ,&
&  0.73446E+00_JPRB, 0.75389E+00_JPRB, 0.77416E+00_JPRB, 0.79811E+00_JPRB, 0.81445E+00_JPRB, 0.82221E+00_JPRB, 0.84580E+00_JPRB ,&
&  0.86622E+00_JPRB, 0.86246E+00_JPRB, 0.87396E+00_JPRB, 0.89784E+00_JPRB, 0.90358E+00_JPRB, 0.93739E+00_JPRB, 0.93260E+00_JPRB ,&
&  0.96081E+00_JPRB, 0.95950E+00_JPRB, 0.98636E+00_JPRB, 0.10762E+01_JPRB, 0.11811E+01_JPRB, 0.13882E+01_JPRB, 0.17820E+01_JPRB ,&
&  0.15607E+01_JPRB, 0.12024E+01_JPRB, 0.10967E+01_JPRB, 0.94161E+00_JPRB, 0.94134E+00_JPRB, 0.97164E+00_JPRB, 0.97448E+00_JPRB ,&
&  0.96790E+00_JPRB, 0.93799E+00_JPRB, 0.94156E+00_JPRB, 0.96362E+00_JPRB, 0.10423E+01_JPRB, 0.11999E+01_JPRB, 0.13234E+01_JPRB ,&
&  0.13650E+01_JPRB, 0.15885E+01_JPRB, 0.18613E+01_JPRB, 0.22274E+01_JPRB, 0.24770E+01_JPRB, 0.31451E+01_JPRB, 0.32078E+01_JPRB ,&
&  0.31335E+01_JPRB, 0.30511E+01_JPRB, 0.26010E+01_JPRB, 0.27337E+01_JPRB, 0.30392E+01_JPRB, 0.29350E+01_JPRB, 0.26090E+01_JPRB ,&
&  0.25905E+01_JPRB, 0.24324E+01_JPRB, 0.24949E+01_JPRB, 0.32625E+01_JPRB, 0.34728E+01_JPRB, 0.34890E+01_JPRB, 0.32875E+01_JPRB ,&
&  0.30305E+01_JPRB, 0.26561E+01_JPRB, 0.23100E+01_JPRB, 0.21230E+01_JPRB, 0.20011E+01_JPRB, 0.17086E+01_JPRB, 0.14136E+01_JPRB ,&
&  0.12288E+01_JPRB, 0.13295E+01_JPRB, 0.14830E+01_JPRB, 0.15160E+01_JPRB, 0.17031E+01_JPRB, 0.14256E+01_JPRB, 0.15673E+01_JPRB ,&
&  0.88535E+00_JPRB, 0.84248E+00_JPRB, 0.69295E+00_JPRB, 0.91463E+00_JPRB, 0.63535E+00_JPRB, 0.49103E+00_JPRB, 0.48246E+00_JPRB ,&
&  0.49626E+00_JPRB, 0.50575E+00_JPRB, 0.52049E+00_JPRB, 0.53188E+00_JPRB, 0.54606E+00_JPRB, 0.57048E+00_JPRB, 0.59677E+00_JPRB ,&
&  0.61970E+00_JPRB, 0.60967E+00_JPRB, 0.61129E+00_JPRB, 0.61733E+00_JPRB, 0.63205E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.67861E+00_JPRB, 0.69936E+00_JPRB, 0.72037E+00_JPRB, 0.74171E+00_JPRB, 0.76047E+00_JPRB, 0.77837E+00_JPRB, 0.79693E+00_JPRB ,&
&  0.81528E+00_JPRB, 0.83648E+00_JPRB, 0.85881E+00_JPRB, 0.88302E+00_JPRB, 0.90374E+00_JPRB, 0.91426E+00_JPRB, 0.93000E+00_JPRB ,&
&  0.94295E+00_JPRB, 0.94942E+00_JPRB, 0.94453E+00_JPRB, 0.97210E+00_JPRB, 0.10009E+01_JPRB, 0.10197E+01_JPRB, 0.10486E+01_JPRB ,&
&  0.10799E+01_JPRB, 0.10628E+01_JPRB, 0.10580E+01_JPRB, 0.10870E+01_JPRB, 0.11083E+01_JPRB, 0.10627E+01_JPRB, 0.11840E+01_JPRB ,&
&  0.94482E+00_JPRB, 0.10601E+01_JPRB, 0.97642E+00_JPRB, 0.85954E+00_JPRB, 0.83429E+00_JPRB, 0.74645E+00_JPRB, 0.70740E+00_JPRB ,&
&  0.77676E+00_JPRB, 0.81200E+00_JPRB, 0.85545E+00_JPRB, 0.93791E+00_JPRB, 0.10976E+01_JPRB, 0.13486E+01_JPRB, 0.15499E+01_JPRB ,&
&  0.18019E+01_JPRB, 0.20498E+01_JPRB, 0.23345E+01_JPRB, 0.26511E+01_JPRB, 0.30061E+01_JPRB, 0.32074E+01_JPRB, 0.31757E+01_JPRB ,&
&  0.30305E+01_JPRB, 0.28441E+01_JPRB, 0.21430E+01_JPRB, 0.24234E+01_JPRB, 0.28736E+01_JPRB, 0.27876E+01_JPRB, 0.23507E+01_JPRB ,&
&  0.21072E+01_JPRB, 0.20698E+01_JPRB, 0.23211E+01_JPRB, 0.25902E+01_JPRB, 0.27587E+01_JPRB, 0.28075E+01_JPRB, 0.26476E+01_JPRB ,&
&  0.24582E+01_JPRB, 0.21154E+01_JPRB, 0.19508E+01_JPRB, 0.19775E+01_JPRB, 0.19239E+01_JPRB, 0.17510E+01_JPRB, 0.14321E+01_JPRB ,&
&  0.13330E+01_JPRB, 0.14027E+01_JPRB, 0.14892E+01_JPRB, 0.14989E+01_JPRB, 0.14941E+01_JPRB, 0.20355E+01_JPRB, 0.20864E+01_JPRB ,&
&  0.89430E+00_JPRB, 0.73102E+00_JPRB, 0.49000E+00_JPRB, 0.58685E+00_JPRB, 0.62024E+00_JPRB, 0.59897E+00_JPRB, 0.64794E+00_JPRB ,&
&  0.68140E+00_JPRB, 0.64561E+00_JPRB, 0.63714E+00_JPRB, 0.61465E+00_JPRB, 0.61497E+00_JPRB, 0.61149E+00_JPRB, 0.62563E+00_JPRB ,&
&  0.63437E+00_JPRB, 0.63559E+00_JPRB, 0.64421E+00_JPRB, 0.65588E+00_JPRB, 0.66439E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.70606E+00_JPRB, 0.72890E+00_JPRB, 0.75183E+00_JPRB, 0.77267E+00_JPRB, 0.79278E+00_JPRB, 0.81145E+00_JPRB, 0.83129E+00_JPRB ,&
&  0.85182E+00_JPRB, 0.87368E+00_JPRB, 0.89460E+00_JPRB, 0.92084E+00_JPRB, 0.94638E+00_JPRB, 0.96053E+00_JPRB, 0.98384E+00_JPRB ,&
&  0.99324E+00_JPRB, 0.10338E+01_JPRB, 0.10532E+01_JPRB, 0.10922E+01_JPRB, 0.11259E+01_JPRB, 0.11483E+01_JPRB, 0.11886E+01_JPRB ,&
&  0.12080E+01_JPRB, 0.12180E+01_JPRB, 0.12303E+01_JPRB, 0.12634E+01_JPRB, 0.12816E+01_JPRB, 0.13769E+01_JPRB, 0.11507E+01_JPRB ,&
&  0.11558E+01_JPRB, 0.10632E+01_JPRB, 0.98952E+00_JPRB, 0.93139E+00_JPRB, 0.88331E+00_JPRB, 0.83824E+00_JPRB, 0.82245E+00_JPRB ,&
&  0.86108E+00_JPRB, 0.90840E+00_JPRB, 0.98428E+00_JPRB, 0.10960E+01_JPRB, 0.12820E+01_JPRB, 0.15543E+01_JPRB, 0.17782E+01_JPRB ,&
&  0.20321E+01_JPRB, 0.23267E+01_JPRB, 0.26073E+01_JPRB, 0.29053E+01_JPRB, 0.32401E+01_JPRB, 0.35831E+01_JPRB, 0.38678E+01_JPRB ,&
&  0.39094E+01_JPRB, 0.38361E+01_JPRB, 0.32477E+01_JPRB, 0.36007E+01_JPRB, 0.38410E+01_JPRB, 0.34176E+01_JPRB, 0.25597E+01_JPRB ,&
&  0.19890E+01_JPRB, 0.16417E+01_JPRB, 0.17074E+01_JPRB, 0.19827E+01_JPRB, 0.20843E+01_JPRB, 0.20946E+01_JPRB, 0.19830E+01_JPRB ,&
&  0.18393E+01_JPRB, 0.16351E+01_JPRB, 0.15549E+01_JPRB, 0.15923E+01_JPRB, 0.16714E+01_JPRB, 0.16893E+01_JPRB, 0.15493E+01_JPRB ,&
&  0.14812E+01_JPRB, 0.14301E+01_JPRB, 0.13588E+01_JPRB, 0.12378E+01_JPRB, 0.11992E+01_JPRB, 0.11002E+01_JPRB, 0.11984E+01_JPRB ,&
&  0.74182E+00_JPRB, 0.60095E+00_JPRB, 0.61000E+00_JPRB, 0.60628E+00_JPRB, 0.67854E+00_JPRB, 0.72480E+00_JPRB, 0.89333E+00_JPRB ,&
&  0.10307E+01_JPRB, 0.10815E+01_JPRB, 0.10136E+01_JPRB, 0.83590E+00_JPRB, 0.69802E+00_JPRB, 0.64446E+00_JPRB, 0.64465E+00_JPRB ,&
&  0.64706E+00_JPRB, 0.64557E+00_JPRB, 0.65516E+00_JPRB, 0.67186E+00_JPRB, 0.68665E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.67645E+00_JPRB, 0.69705E+00_JPRB, 0.72013E+00_JPRB, 0.73891E+00_JPRB, 0.76325E+00_JPRB, 0.79831E+00_JPRB, 0.83293E+00_JPRB ,&
&  0.86240E+00_JPRB, 0.88402E+00_JPRB, 0.89990E+00_JPRB, 0.92351E+00_JPRB, 0.95430E+00_JPRB, 0.99695E+00_JPRB, 0.10516E+01_JPRB ,&
&  0.10769E+01_JPRB, 0.11242E+01_JPRB, 0.11525E+01_JPRB, 0.12049E+01_JPRB, 0.12313E+01_JPRB, 0.12934E+01_JPRB, 0.13395E+01_JPRB ,&
&  0.13644E+01_JPRB, 0.13769E+01_JPRB, 0.13917E+01_JPRB, 0.14229E+01_JPRB, 0.14122E+01_JPRB, 0.14669E+01_JPRB, 0.11751E+01_JPRB ,&
&  0.12585E+01_JPRB, 0.12114E+01_JPRB, 0.11239E+01_JPRB, 0.10886E+01_JPRB, 0.10592E+01_JPRB, 0.10413E+01_JPRB, 0.10409E+01_JPRB ,&
&  0.10513E+01_JPRB, 0.10806E+01_JPRB, 0.10766E+01_JPRB, 0.11533E+01_JPRB, 0.13239E+01_JPRB, 0.15963E+01_JPRB, 0.18322E+01_JPRB ,&
&  0.21040E+01_JPRB, 0.23826E+01_JPRB, 0.26766E+01_JPRB, 0.30542E+01_JPRB, 0.35453E+01_JPRB, 0.40655E+01_JPRB, 0.45832E+01_JPRB ,&
&  0.47863E+01_JPRB, 0.48812E+01_JPRB, 0.44595E+01_JPRB, 0.45114E+01_JPRB, 0.45954E+01_JPRB, 0.37136E+01_JPRB, 0.25369E+01_JPRB ,&
&  0.15405E+01_JPRB, 0.14007E+01_JPRB, 0.15310E+01_JPRB, 0.16279E+01_JPRB, 0.15052E+01_JPRB, 0.14118E+01_JPRB, 0.12865E+01_JPRB ,&
&  0.11596E+01_JPRB, 0.10574E+01_JPRB, 0.10679E+01_JPRB, 0.11699E+01_JPRB, 0.12937E+01_JPRB, 0.14336E+01_JPRB, 0.14683E+01_JPRB ,&
&  0.14263E+01_JPRB, 0.12999E+01_JPRB, 0.11702E+01_JPRB, 0.10586E+01_JPRB, 0.95264E+00_JPRB, 0.84725E+00_JPRB, 0.92613E+00_JPRB ,&
&  0.73468E+00_JPRB, 0.69838E+00_JPRB, 0.71373E+00_JPRB, 0.54949E+00_JPRB, 0.72691E+00_JPRB, 0.90096E+00_JPRB, 0.11529E+01_JPRB ,&
&  0.12793E+01_JPRB, 0.17221E+01_JPRB, 0.21320E+01_JPRB, 0.15263E+01_JPRB, 0.11364E+01_JPRB, 0.71356E+00_JPRB, 0.64308E+00_JPRB ,&
&  0.64412E+00_JPRB, 0.63253E+00_JPRB, 0.63449E+00_JPRB, 0.64644E+00_JPRB, 0.65937E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.63298E+00_JPRB, 0.64723E+00_JPRB, 0.66890E+00_JPRB, 0.67287E+00_JPRB, 0.68220E+00_JPRB, 0.70146E+00_JPRB, 0.72100E+00_JPRB ,&
&  0.74068E+00_JPRB, 0.75371E+00_JPRB, 0.76778E+00_JPRB, 0.78581E+00_JPRB, 0.80838E+00_JPRB, 0.86257E+00_JPRB, 0.94268E+00_JPRB ,&
&  0.10095E+01_JPRB, 0.10740E+01_JPRB, 0.11555E+01_JPRB, 0.11911E+01_JPRB, 0.12507E+01_JPRB, 0.13248E+01_JPRB, 0.13741E+01_JPRB ,&
&  0.14407E+01_JPRB, 0.14747E+01_JPRB, 0.15089E+01_JPRB, 0.15887E+01_JPRB, 0.15525E+01_JPRB, 0.15890E+01_JPRB, 0.12326E+01_JPRB ,&
&  0.13049E+01_JPRB, 0.12892E+01_JPRB, 0.12390E+01_JPRB, 0.12180E+01_JPRB, 0.11886E+01_JPRB, 0.11411E+01_JPRB, 0.11071E+01_JPRB ,&
&  0.10926E+01_JPRB, 0.10450E+01_JPRB, 0.99704E+00_JPRB, 0.10717E+01_JPRB, 0.11971E+01_JPRB, 0.14210E+01_JPRB, 0.17027E+01_JPRB ,&
&  0.19983E+01_JPRB, 0.22747E+01_JPRB, 0.25670E+01_JPRB, 0.29885E+01_JPRB, 0.35433E+01_JPRB, 0.41314E+01_JPRB, 0.47518E+01_JPRB ,&
&  0.51805E+01_JPRB, 0.55037E+01_JPRB, 0.55871E+01_JPRB, 0.47197E+01_JPRB, 0.44440E+01_JPRB, 0.33308E+01_JPRB, 0.21201E+01_JPRB ,&
&  0.15180E+01_JPRB, 0.13412E+01_JPRB, 0.13529E+01_JPRB, 0.13323E+01_JPRB, 0.11359E+01_JPRB, 0.10160E+01_JPRB, 0.88765E+00_JPRB ,&
&  0.79225E+00_JPRB, 0.77042E+00_JPRB, 0.80923E+00_JPRB, 0.91825E+00_JPRB, 0.10369E+01_JPRB, 0.11787E+01_JPRB, 0.12294E+01_JPRB ,&
&  0.11785E+01_JPRB, 0.10615E+01_JPRB, 0.96611E+00_JPRB, 0.88881E+00_JPRB, 0.86459E+00_JPRB, 0.89430E+00_JPRB, 0.80450E+00_JPRB ,&
&  0.74974E+00_JPRB, 0.64977E+00_JPRB, 0.63741E+00_JPRB, 0.69687E+00_JPRB, 0.76779E+00_JPRB, 0.99898E+00_JPRB, 0.14117E+01_JPRB ,&
&  0.20159E+01_JPRB, 0.21453E+01_JPRB, 0.26172E+01_JPRB, 0.34809E+01_JPRB, 0.25612E+01_JPRB, 0.18127E+01_JPRB, 0.58190E+00_JPRB ,&
&  0.59818E+00_JPRB, 0.59605E+00_JPRB, 0.59788E+00_JPRB, 0.59981E+00_JPRB, 0.61074E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.53211E+00_JPRB, 0.55593E+00_JPRB, 0.54703E+00_JPRB, 0.55043E+00_JPRB, 0.55727E+00_JPRB, 0.56890E+00_JPRB, 0.59324E+00_JPRB ,&
&  0.59981E+00_JPRB, 0.63445E+00_JPRB, 0.67211E+00_JPRB, 0.70297E+00_JPRB, 0.74469E+00_JPRB, 0.80313E+00_JPRB, 0.87757E+00_JPRB ,&
&  0.95687E+00_JPRB, 0.10053E+01_JPRB, 0.10499E+01_JPRB, 0.10841E+01_JPRB, 0.11427E+01_JPRB, 0.12198E+01_JPRB, 0.12671E+01_JPRB ,&
&  0.13559E+01_JPRB, 0.14195E+01_JPRB, 0.14949E+01_JPRB, 0.16023E+01_JPRB, 0.15365E+01_JPRB, 0.15261E+01_JPRB, 0.11049E+01_JPRB ,&
&  0.12649E+01_JPRB, 0.12855E+01_JPRB, 0.12751E+01_JPRB, 0.12552E+01_JPRB, 0.12075E+01_JPRB, 0.11280E+01_JPRB, 0.10800E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.99967E+00_JPRB, 0.94991E+00_JPRB, 0.93233E+00_JPRB, 0.10631E+01_JPRB, 0.11975E+01_JPRB, 0.14544E+01_JPRB ,&
&  0.17610E+01_JPRB, 0.21019E+01_JPRB, 0.24337E+01_JPRB, 0.28388E+01_JPRB, 0.33693E+01_JPRB, 0.39781E+01_JPRB, 0.45980E+01_JPRB ,&
&  0.50638E+01_JPRB, 0.54534E+01_JPRB, 0.53992E+01_JPRB, 0.44591E+01_JPRB, 0.39129E+01_JPRB, 0.28801E+01_JPRB, 0.18093E+01_JPRB ,&
&  0.14095E+01_JPRB, 0.12968E+01_JPRB, 0.13011E+01_JPRB, 0.12092E+01_JPRB, 0.98893E+00_JPRB, 0.84441E+00_JPRB, 0.72401E+00_JPRB ,&
&  0.63541E+00_JPRB, 0.63174E+00_JPRB, 0.70459E+00_JPRB, 0.80306E+00_JPRB, 0.89470E+00_JPRB, 0.98793E+00_JPRB, 0.98661E+00_JPRB ,&
&  0.92725E+00_JPRB, 0.83406E+00_JPRB, 0.77300E+00_JPRB, 0.77856E+00_JPRB, 0.88178E+00_JPRB, 0.10033E+01_JPRB, 0.10896E+01_JPRB ,&
&  0.80433E+00_JPRB, 0.64085E+00_JPRB, 0.63816E+00_JPRB, 0.66438E+00_JPRB, 0.71019E+00_JPRB, 0.78644E+00_JPRB, 0.90737E+00_JPRB ,&
&  0.10850E+01_JPRB, 0.14555E+01_JPRB, 0.14130E+01_JPRB, 0.11611E+01_JPRB, 0.16631E+01_JPRB, 0.13875E+01_JPRB, 0.53003E+00_JPRB ,&
&  0.51091E+00_JPRB, 0.50170E+00_JPRB, 0.50871E+00_JPRB, 0.51358E+00_JPRB, 0.52744E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.49827E+00_JPRB, 0.48675E+00_JPRB, 0.46878E+00_JPRB, 0.47780E+00_JPRB, 0.47934E+00_JPRB, 0.49741E+00_JPRB, 0.51809E+00_JPRB ,&
&  0.54669E+00_JPRB, 0.60155E+00_JPRB, 0.65592E+00_JPRB, 0.70132E+00_JPRB, 0.73102E+00_JPRB, 0.76673E+00_JPRB, 0.80083E+00_JPRB ,&
&  0.82597E+00_JPRB, 0.84973E+00_JPRB, 0.88319E+00_JPRB, 0.93019E+00_JPRB, 0.99103E+00_JPRB, 0.10609E+01_JPRB, 0.11456E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.13404E+01_JPRB, 0.14677E+01_JPRB, 0.15183E+01_JPRB, 0.15598E+01_JPRB, 0.16049E+01_JPRB, 0.15890E+01_JPRB ,&
&  0.11113E+01_JPRB, 0.13285E+01_JPRB, 0.12946E+01_JPRB, 0.12455E+01_JPRB, 0.12048E+01_JPRB, 0.11375E+01_JPRB, 0.10818E+01_JPRB ,&
&  0.10435E+01_JPRB, 0.97144E+00_JPRB, 0.94094E+00_JPRB, 0.88383E+00_JPRB, 0.93503E+00_JPRB, 0.10153E+01_JPRB, 0.11832E+01_JPRB ,&
&  0.14396E+01_JPRB, 0.18059E+01_JPRB, 0.22481E+01_JPRB, 0.26698E+01_JPRB, 0.30890E+01_JPRB, 0.35545E+01_JPRB, 0.42127E+01_JPRB ,&
&  0.49269E+01_JPRB, 0.54221E+01_JPRB, 0.57228E+01_JPRB, 0.49311E+01_JPRB, 0.38625E+01_JPRB, 0.32886E+01_JPRB, 0.23635E+01_JPRB ,&
&  0.14734E+01_JPRB, 0.11720E+01_JPRB, 0.11950E+01_JPRB, 0.11706E+01_JPRB, 0.97444E+00_JPRB, 0.84130E+00_JPRB, 0.71650E+00_JPRB ,&
&  0.61639E+00_JPRB, 0.56527E+00_JPRB, 0.59129E+00_JPRB, 0.69927E+00_JPRB, 0.76078E+00_JPRB, 0.79099E+00_JPRB, 0.76794E+00_JPRB ,&
&  0.73906E+00_JPRB, 0.67801E+00_JPRB, 0.61582E+00_JPRB, 0.61240E+00_JPRB, 0.65182E+00_JPRB, 0.69039E+00_JPRB, 0.68788E+00_JPRB ,&
&  0.66699E+00_JPRB, 0.65894E+00_JPRB, 0.66269E+00_JPRB, 0.67986E+00_JPRB, 0.69316E+00_JPRB, 0.71358E+00_JPRB, 0.73679E+00_JPRB ,&
&  0.76642E+00_JPRB, 0.80815E+00_JPRB, 0.79106E+00_JPRB, 0.77365E+00_JPRB, 0.84709E+00_JPRB, 0.89804E+00_JPRB, 0.58857E+00_JPRB ,&
&  0.50619E+00_JPRB, 0.47416E+00_JPRB, 0.47624E+00_JPRB, 0.47082E+00_JPRB, 0.50757E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.50629E+00_JPRB, 0.48707E+00_JPRB, 0.47611E+00_JPRB, 0.46775E+00_JPRB, 0.45431E+00_JPRB, 0.47833E+00_JPRB, 0.48420E+00_JPRB ,&
&  0.50937E+00_JPRB, 0.55153E+00_JPRB, 0.60360E+00_JPRB, 0.63560E+00_JPRB, 0.64779E+00_JPRB, 0.67758E+00_JPRB, 0.70271E+00_JPRB ,&
&  0.71862E+00_JPRB, 0.72988E+00_JPRB, 0.74655E+00_JPRB, 0.76857E+00_JPRB, 0.80771E+00_JPRB, 0.86763E+00_JPRB, 0.95739E+00_JPRB ,&
&  0.10604E+01_JPRB, 0.11709E+01_JPRB, 0.12896E+01_JPRB, 0.14012E+01_JPRB, 0.14784E+01_JPRB, 0.15397E+01_JPRB, 0.15049E+01_JPRB ,&
&  0.89258E+00_JPRB, 0.12126E+01_JPRB, 0.13903E+01_JPRB, 0.13008E+01_JPRB, 0.12231E+01_JPRB, 0.11528E+01_JPRB, 0.10822E+01_JPRB ,&
&  0.10131E+01_JPRB, 0.94168E+00_JPRB, 0.91649E+00_JPRB, 0.92700E+00_JPRB, 0.85439E+00_JPRB, 0.88383E+00_JPRB, 0.97168E+00_JPRB ,&
&  0.11353E+01_JPRB, 0.14074E+01_JPRB, 0.18988E+01_JPRB, 0.24276E+01_JPRB, 0.27231E+01_JPRB, 0.30164E+01_JPRB, 0.35586E+01_JPRB ,&
&  0.43912E+01_JPRB, 0.50379E+01_JPRB, 0.55054E+01_JPRB, 0.41216E+01_JPRB, 0.34637E+01_JPRB, 0.33385E+01_JPRB, 0.29838E+01_JPRB ,&
&  0.17417E+01_JPRB, 0.11893E+01_JPRB, 0.12029E+01_JPRB, 0.12020E+01_JPRB, 0.10178E+01_JPRB, 0.91286E+00_JPRB, 0.79778E+00_JPRB ,&
&  0.66984E+00_JPRB, 0.58862E+00_JPRB, 0.53643E+00_JPRB, 0.56965E+00_JPRB, 0.60703E+00_JPRB, 0.62641E+00_JPRB, 0.59502E+00_JPRB ,&
&  0.61956E+00_JPRB, 0.59466E+00_JPRB, 0.57777E+00_JPRB, 0.58235E+00_JPRB, 0.63888E+00_JPRB, 0.66403E+00_JPRB, 0.67473E+00_JPRB ,&
&  0.69063E+00_JPRB, 0.70324E+00_JPRB, 0.70553E+00_JPRB, 0.71481E+00_JPRB, 0.72200E+00_JPRB, 0.72113E+00_JPRB, 0.71776E+00_JPRB ,&
&  0.70758E+00_JPRB, 0.70552E+00_JPRB, 0.68512E+00_JPRB, 0.66129E+00_JPRB, 0.62539E+00_JPRB, 0.63636E+00_JPRB, 0.57276E+00_JPRB ,&
&  0.52971E+00_JPRB, 0.50012E+00_JPRB, 0.48009E+00_JPRB, 0.47606E+00_JPRB, 0.48515E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.52098E+00_JPRB, 0.51570E+00_JPRB, 0.50866E+00_JPRB, 0.49784E+00_JPRB, 0.48504E+00_JPRB, 0.48398E+00_JPRB, 0.47028E+00_JPRB ,&
&  0.46765E+00_JPRB, 0.49017E+00_JPRB, 0.52615E+00_JPRB, 0.55541E+00_JPRB, 0.58172E+00_JPRB, 0.59796E+00_JPRB, 0.62068E+00_JPRB ,&
&  0.62624E+00_JPRB, 0.62569E+00_JPRB, 0.62244E+00_JPRB, 0.62102E+00_JPRB, 0.63014E+00_JPRB, 0.66318E+00_JPRB, 0.72147E+00_JPRB ,&
&  0.80399E+00_JPRB, 0.91240E+00_JPRB, 0.10458E+01_JPRB, 0.11906E+01_JPRB, 0.13333E+01_JPRB, 0.14720E+01_JPRB, 0.15362E+01_JPRB ,&
&  0.10400E+01_JPRB, 0.58383E+00_JPRB, 0.12659E+01_JPRB, 0.14139E+01_JPRB, 0.13103E+01_JPRB, 0.12497E+01_JPRB, 0.11333E+01_JPRB ,&
&  0.10195E+01_JPRB, 0.98014E+00_JPRB, 0.92476E+00_JPRB, 0.89008E+00_JPRB, 0.82061E+00_JPRB, 0.81759E+00_JPRB, 0.84057E+00_JPRB ,&
&  0.92330E+00_JPRB, 0.10990E+01_JPRB, 0.14481E+01_JPRB, 0.20293E+01_JPRB, 0.24201E+01_JPRB, 0.25767E+01_JPRB, 0.29255E+01_JPRB ,&
&  0.36301E+01_JPRB, 0.42461E+01_JPRB, 0.44998E+01_JPRB, 0.29988E+01_JPRB, 0.20361E+01_JPRB, 0.17906E+01_JPRB, 0.16504E+01_JPRB ,&
&  0.15559E+01_JPRB, 0.12569E+01_JPRB, 0.12317E+01_JPRB, 0.12529E+01_JPRB, 0.11190E+01_JPRB, 0.96575E+00_JPRB, 0.93523E+00_JPRB ,&
&  0.81025E+00_JPRB, 0.70797E+00_JPRB, 0.62519E+00_JPRB, 0.58413E+00_JPRB, 0.56438E+00_JPRB, 0.57245E+00_JPRB, 0.56706E+00_JPRB ,&
&  0.56413E+00_JPRB, 0.57600E+00_JPRB, 0.57730E+00_JPRB, 0.58842E+00_JPRB, 0.62509E+00_JPRB, 0.66102E+00_JPRB, 0.68406E+00_JPRB ,&
&  0.70069E+00_JPRB, 0.72042E+00_JPRB, 0.73299E+00_JPRB, 0.73860E+00_JPRB, 0.74009E+00_JPRB, 0.72590E+00_JPRB, 0.73095E+00_JPRB ,&
&  0.73821E+00_JPRB, 0.74259E+00_JPRB, 0.72223E+00_JPRB, 0.70647E+00_JPRB, 0.66278E+00_JPRB, 0.63230E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.54721E+00_JPRB, 0.50950E+00_JPRB, 0.48325E+00_JPRB, 0.47886E+00_JPRB, 0.49129E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.52747E+00_JPRB, 0.52751E+00_JPRB, 0.52816E+00_JPRB, 0.53143E+00_JPRB, 0.52780E+00_JPRB, 0.52913E+00_JPRB, 0.51032E+00_JPRB ,&
&  0.48183E+00_JPRB, 0.47145E+00_JPRB, 0.48808E+00_JPRB, 0.49310E+00_JPRB, 0.49534E+00_JPRB, 0.51528E+00_JPRB, 0.53658E+00_JPRB ,&
&  0.54691E+00_JPRB, 0.55133E+00_JPRB, 0.55003E+00_JPRB, 0.54871E+00_JPRB, 0.54053E+00_JPRB, 0.54284E+00_JPRB, 0.56477E+00_JPRB ,&
&  0.60161E+00_JPRB, 0.66864E+00_JPRB, 0.76904E+00_JPRB, 0.89404E+00_JPRB, 0.10340E+01_JPRB, 0.12271E+01_JPRB, 0.14030E+01_JPRB ,&
&  0.17288E+01_JPRB, 0.11882E+01_JPRB, 0.74224E+00_JPRB, 0.12706E+01_JPRB, 0.14344E+01_JPRB, 0.14294E+01_JPRB, 0.12219E+01_JPRB ,&
&  0.11552E+01_JPRB, 0.11077E+01_JPRB, 0.10614E+01_JPRB, 0.98241E+00_JPRB, 0.86035E+00_JPRB, 0.81179E+00_JPRB, 0.77289E+00_JPRB ,&
&  0.77502E+00_JPRB, 0.85550E+00_JPRB, 0.10514E+01_JPRB, 0.14544E+01_JPRB, 0.19668E+01_JPRB, 0.22355E+01_JPRB, 0.23918E+01_JPRB ,&
&  0.28710E+01_JPRB, 0.34980E+01_JPRB, 0.36493E+01_JPRB, 0.23928E+01_JPRB, 0.17134E+01_JPRB, 0.15367E+01_JPRB, 0.14974E+01_JPRB ,&
&  0.14246E+01_JPRB, 0.14234E+01_JPRB, 0.14476E+01_JPRB, 0.13194E+01_JPRB, 0.11691E+01_JPRB, 0.10573E+01_JPRB, 0.10535E+01_JPRB ,&
&  0.93349E+00_JPRB, 0.83260E+00_JPRB, 0.74751E+00_JPRB, 0.66282E+00_JPRB, 0.59453E+00_JPRB, 0.58184E+00_JPRB, 0.57214E+00_JPRB ,&
&  0.57500E+00_JPRB, 0.59210E+00_JPRB, 0.59290E+00_JPRB, 0.61078E+00_JPRB, 0.62954E+00_JPRB, 0.64964E+00_JPRB, 0.66868E+00_JPRB ,&
&  0.68512E+00_JPRB, 0.70511E+00_JPRB, 0.72496E+00_JPRB, 0.72967E+00_JPRB, 0.72345E+00_JPRB, 0.71320E+00_JPRB, 0.72745E+00_JPRB ,&
&  0.73726E+00_JPRB, 0.75442E+00_JPRB, 0.75186E+00_JPRB, 0.75414E+00_JPRB, 0.72050E+00_JPRB, 0.68908E+00_JPRB, 0.64278E+00_JPRB ,&
&  0.63334E+00_JPRB, 0.59510E+00_JPRB, 0.56180E+00_JPRB, 0.51866E+00_JPRB, 0.52010E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.54709E+00_JPRB, 0.53916E+00_JPRB, 0.53861E+00_JPRB, 0.54148E+00_JPRB, 0.54166E+00_JPRB, 0.54460E+00_JPRB, 0.54585E+00_JPRB ,&
&  0.51858E+00_JPRB, 0.49211E+00_JPRB, 0.47474E+00_JPRB, 0.46531E+00_JPRB, 0.46163E+00_JPRB, 0.47472E+00_JPRB, 0.48062E+00_JPRB ,&
&  0.47824E+00_JPRB, 0.47903E+00_JPRB, 0.49878E+00_JPRB, 0.50388E+00_JPRB, 0.50247E+00_JPRB, 0.49768E+00_JPRB, 0.49728E+00_JPRB ,&
&  0.50899E+00_JPRB, 0.53568E+00_JPRB, 0.58889E+00_JPRB, 0.66368E+00_JPRB, 0.74333E+00_JPRB, 0.87623E+00_JPRB, 0.10388E+01_JPRB ,&
&  0.13165E+01_JPRB, 0.13427E+01_JPRB, 0.67026E+00_JPRB, 0.11136E+01_JPRB, 0.14864E+01_JPRB, 0.15300E+01_JPRB, 0.14274E+01_JPRB ,&
&  0.13327E+01_JPRB, 0.12512E+01_JPRB, 0.13006E+01_JPRB, 0.12129E+01_JPRB, 0.98914E+00_JPRB, 0.85142E+00_JPRB, 0.75524E+00_JPRB ,&
&  0.70584E+00_JPRB, 0.71078E+00_JPRB, 0.78619E+00_JPRB, 0.98606E+00_JPRB, 0.13858E+01_JPRB, 0.17588E+01_JPRB, 0.19460E+01_JPRB ,&
&  0.21739E+01_JPRB, 0.27599E+01_JPRB, 0.31692E+01_JPRB, 0.21677E+01_JPRB, 0.17148E+01_JPRB, 0.17418E+01_JPRB, 0.17171E+01_JPRB ,&
&  0.16266E+01_JPRB, 0.15777E+01_JPRB, 0.15334E+01_JPRB, 0.14020E+01_JPRB, 0.12392E+01_JPRB, 0.11649E+01_JPRB, 0.11635E+01_JPRB ,&
&  0.10512E+01_JPRB, 0.94944E+00_JPRB, 0.85778E+00_JPRB, 0.77176E+00_JPRB, 0.68112E+00_JPRB, 0.61655E+00_JPRB, 0.60998E+00_JPRB ,&
&  0.60979E+00_JPRB, 0.62611E+00_JPRB, 0.62613E+00_JPRB, 0.63664E+00_JPRB, 0.64762E+00_JPRB, 0.65864E+00_JPRB, 0.66599E+00_JPRB ,&
&  0.67538E+00_JPRB, 0.68913E+00_JPRB, 0.70602E+00_JPRB, 0.70773E+00_JPRB, 0.70667E+00_JPRB, 0.70407E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.71818E+00_JPRB, 0.73044E+00_JPRB, 0.73493E+00_JPRB, 0.73893E+00_JPRB, 0.75866E+00_JPRB, 0.75172E+00_JPRB, 0.73143E+00_JPRB ,&
&  0.77087E+00_JPRB, 0.80611E+00_JPRB, 0.74515E+00_JPRB, 0.59690E+00_JPRB, 0.56520E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.58179E+00_JPRB, 0.56883E+00_JPRB, 0.55974E+00_JPRB, 0.55859E+00_JPRB, 0.55623E+00_JPRB, 0.55621E+00_JPRB, 0.56491E+00_JPRB ,&
&  0.55589E+00_JPRB, 0.53813E+00_JPRB, 0.51460E+00_JPRB, 0.48995E+00_JPRB, 0.47093E+00_JPRB, 0.47173E+00_JPRB, 0.47711E+00_JPRB ,&
&  0.47772E+00_JPRB, 0.47470E+00_JPRB, 0.47528E+00_JPRB, 0.48078E+00_JPRB, 0.47954E+00_JPRB, 0.47584E+00_JPRB, 0.47173E+00_JPRB ,&
&  0.47059E+00_JPRB, 0.47661E+00_JPRB, 0.49816E+00_JPRB, 0.53582E+00_JPRB, 0.58072E+00_JPRB, 0.65062E+00_JPRB, 0.75026E+00_JPRB ,&
&  0.97188E+00_JPRB, 0.11087E+01_JPRB, 0.11940E+01_JPRB, 0.17827E+01_JPRB, 0.18751E+01_JPRB, 0.17492E+01_JPRB, 0.16971E+01_JPRB ,&
&  0.16295E+01_JPRB, 0.15440E+01_JPRB, 0.16545E+01_JPRB, 0.13654E+01_JPRB, 0.11191E+01_JPRB, 0.91478E+00_JPRB, 0.78093E+00_JPRB ,&
&  0.70633E+00_JPRB, 0.66160E+00_JPRB, 0.66299E+00_JPRB, 0.71775E+00_JPRB, 0.91751E+00_JPRB, 0.12673E+01_JPRB, 0.15455E+01_JPRB ,&
&  0.17487E+01_JPRB, 0.21380E+01_JPRB, 0.27173E+01_JPRB, 0.22815E+01_JPRB, 0.18326E+01_JPRB, 0.21071E+01_JPRB, 0.23836E+01_JPRB ,&
&  0.26973E+01_JPRB, 0.19732E+01_JPRB, 0.16942E+01_JPRB, 0.15198E+01_JPRB, 0.13131E+01_JPRB, 0.14192E+01_JPRB, 0.12594E+01_JPRB ,&
&  0.11463E+01_JPRB, 0.10598E+01_JPRB, 0.96894E+00_JPRB, 0.88236E+00_JPRB, 0.79983E+00_JPRB, 0.72338E+00_JPRB, 0.68204E+00_JPRB ,&
&  0.67454E+00_JPRB, 0.67753E+00_JPRB, 0.67334E+00_JPRB, 0.67849E+00_JPRB, 0.67776E+00_JPRB, 0.68449E+00_JPRB, 0.68948E+00_JPRB ,&
&  0.69462E+00_JPRB, 0.69487E+00_JPRB, 0.67992E+00_JPRB, 0.68378E+00_JPRB, 0.68714E+00_JPRB, 0.68492E+00_JPRB, 0.67948E+00_JPRB ,&
&  0.68687E+00_JPRB, 0.70227E+00_JPRB, 0.71215E+00_JPRB, 0.72088E+00_JPRB, 0.73467E+00_JPRB, 0.73392E+00_JPRB, 0.71455E+00_JPRB ,&
&  0.70463E+00_JPRB, 0.71037E+00_JPRB, 0.70425E+00_JPRB, 0.63696E+00_JPRB, 0.60509E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.59948E+00_JPRB, 0.58751E+00_JPRB, 0.57952E+00_JPRB, 0.57130E+00_JPRB, 0.56223E+00_JPRB, 0.56132E+00_JPRB, 0.56870E+00_JPRB ,&
&  0.56867E+00_JPRB, 0.55776E+00_JPRB, 0.54086E+00_JPRB, 0.52367E+00_JPRB, 0.50122E+00_JPRB, 0.48858E+00_JPRB, 0.48099E+00_JPRB ,&
&  0.47566E+00_JPRB, 0.47443E+00_JPRB, 0.47011E+00_JPRB, 0.46868E+00_JPRB, 0.46701E+00_JPRB, 0.45780E+00_JPRB, 0.45219E+00_JPRB ,&
&  0.44857E+00_JPRB, 0.45216E+00_JPRB, 0.46216E+00_JPRB, 0.48250E+00_JPRB, 0.51264E+00_JPRB, 0.55943E+00_JPRB, 0.61793E+00_JPRB ,&
&  0.76260E+00_JPRB, 0.98209E+00_JPRB, 0.95608E+00_JPRB, 0.19040E+01_JPRB, 0.18769E+01_JPRB, 0.18222E+01_JPRB, 0.17029E+01_JPRB ,&
&  0.16873E+01_JPRB, 0.18535E+01_JPRB, 0.16730E+01_JPRB, 0.14199E+01_JPRB, 0.11521E+01_JPRB, 0.95854E+00_JPRB, 0.81344E+00_JPRB ,&
&  0.72359E+00_JPRB, 0.66959E+00_JPRB, 0.64483E+00_JPRB, 0.63682E+00_JPRB, 0.66364E+00_JPRB, 0.86064E+00_JPRB, 0.11727E+01_JPRB ,&
&  0.14084E+01_JPRB, 0.16915E+01_JPRB, 0.22241E+01_JPRB, 0.21303E+01_JPRB, 0.19869E+01_JPRB, 0.23724E+01_JPRB, 0.31159E+01_JPRB ,&
&  0.37711E+01_JPRB, 0.28316E+01_JPRB, 0.20539E+01_JPRB, 0.17311E+01_JPRB, 0.15569E+01_JPRB, 0.15123E+01_JPRB, 0.13707E+01_JPRB ,&
&  0.12179E+01_JPRB, 0.11343E+01_JPRB, 0.10618E+01_JPRB, 0.98968E+00_JPRB, 0.91196E+00_JPRB, 0.84306E+00_JPRB, 0.77980E+00_JPRB ,&
&  0.74674E+00_JPRB, 0.73318E+00_JPRB, 0.72551E+00_JPRB, 0.73106E+00_JPRB, 0.73353E+00_JPRB, 0.74196E+00_JPRB, 0.74130E+00_JPRB ,&
&  0.74210E+00_JPRB, 0.72767E+00_JPRB, 0.68374E+00_JPRB, 0.67175E+00_JPRB, 0.67286E+00_JPRB, 0.66890E+00_JPRB, 0.66074E+00_JPRB ,&
&  0.67180E+00_JPRB, 0.68342E+00_JPRB, 0.68522E+00_JPRB, 0.68643E+00_JPRB, 0.70718E+00_JPRB, 0.71978E+00_JPRB, 0.71258E+00_JPRB ,&
&  0.69200E+00_JPRB, 0.67367E+00_JPRB, 0.65531E+00_JPRB, 0.62725E+00_JPRB, 0.61469E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.60544E+00_JPRB, 0.59668E+00_JPRB, 0.58828E+00_JPRB, 0.57742E+00_JPRB, 0.56295E+00_JPRB, 0.55656E+00_JPRB, 0.56168E+00_JPRB ,&
&  0.56842E+00_JPRB, 0.56229E+00_JPRB, 0.55042E+00_JPRB, 0.53332E+00_JPRB, 0.51113E+00_JPRB, 0.49931E+00_JPRB, 0.49123E+00_JPRB ,&
&  0.47904E+00_JPRB, 0.46948E+00_JPRB, 0.45685E+00_JPRB, 0.45172E+00_JPRB, 0.45121E+00_JPRB, 0.44501E+00_JPRB, 0.43287E+00_JPRB ,&
&  0.43455E+00_JPRB, 0.44275E+00_JPRB, 0.45304E+00_JPRB, 0.46425E+00_JPRB, 0.48694E+00_JPRB, 0.52384E+00_JPRB, 0.57854E+00_JPRB ,&
&  0.68610E+00_JPRB, 0.10987E+01_JPRB, 0.88942E+00_JPRB, 0.14909E+01_JPRB, 0.15249E+01_JPRB, 0.16238E+01_JPRB, 0.16476E+01_JPRB ,&
&  0.16820E+01_JPRB, 0.17178E+01_JPRB, 0.16115E+01_JPRB, 0.13446E+01_JPRB, 0.10858E+01_JPRB, 0.92268E+00_JPRB, 0.82015E+00_JPRB ,&
&  0.75260E+00_JPRB, 0.70083E+00_JPRB, 0.66376E+00_JPRB, 0.63863E+00_JPRB, 0.62165E+00_JPRB, 0.64806E+00_JPRB, 0.81126E+00_JPRB ,&
&  0.10840E+01_JPRB, 0.13333E+01_JPRB, 0.17354E+01_JPRB, 0.21654E+01_JPRB, 0.19957E+01_JPRB, 0.22666E+01_JPRB, 0.33751E+01_JPRB ,&
&  0.40813E+01_JPRB, 0.40697E+01_JPRB, 0.26949E+01_JPRB, 0.20897E+01_JPRB, 0.18381E+01_JPRB, 0.16883E+01_JPRB, 0.14655E+01_JPRB ,&
&  0.12626E+01_JPRB, 0.11640E+01_JPRB, 0.11106E+01_JPRB, 0.10628E+01_JPRB, 0.10022E+01_JPRB, 0.91987E+00_JPRB, 0.84168E+00_JPRB ,&
&  0.77775E+00_JPRB, 0.75809E+00_JPRB, 0.75462E+00_JPRB, 0.75988E+00_JPRB, 0.77274E+00_JPRB, 0.78637E+00_JPRB, 0.79481E+00_JPRB ,&
&  0.79010E+00_JPRB, 0.77618E+00_JPRB, 0.71650E+00_JPRB, 0.68701E+00_JPRB, 0.67910E+00_JPRB, 0.67189E+00_JPRB, 0.66227E+00_JPRB ,&
&  0.67004E+00_JPRB, 0.67477E+00_JPRB, 0.67178E+00_JPRB, 0.67725E+00_JPRB, 0.69184E+00_JPRB, 0.71410E+00_JPRB, 0.68807E+00_JPRB ,&
&  0.68724E+00_JPRB, 0.67135E+00_JPRB, 0.65243E+00_JPRB, 0.62922E+00_JPRB, 0.61799E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.60489E+00_JPRB, 0.59744E+00_JPRB, 0.58920E+00_JPRB, 0.57979E+00_JPRB, 0.56868E+00_JPRB, 0.56076E+00_JPRB, 0.55872E+00_JPRB ,&
&  0.55904E+00_JPRB, 0.56057E+00_JPRB, 0.55348E+00_JPRB, 0.53538E+00_JPRB, 0.51590E+00_JPRB, 0.49840E+00_JPRB, 0.48858E+00_JPRB ,&
&  0.48049E+00_JPRB, 0.47014E+00_JPRB, 0.45761E+00_JPRB, 0.44540E+00_JPRB, 0.44182E+00_JPRB, 0.44263E+00_JPRB, 0.43594E+00_JPRB ,&
&  0.44138E+00_JPRB, 0.44918E+00_JPRB, 0.45631E+00_JPRB, 0.46328E+00_JPRB, 0.48229E+00_JPRB, 0.51540E+00_JPRB, 0.56685E+00_JPRB ,&
&  0.66403E+00_JPRB, 0.57640E+00_JPRB, 0.98137E+00_JPRB, 0.11546E+01_JPRB, 0.12491E+01_JPRB, 0.13360E+01_JPRB, 0.15187E+01_JPRB ,&
&  0.16643E+01_JPRB, 0.16111E+01_JPRB, 0.14603E+01_JPRB, 0.12443E+01_JPRB, 0.10494E+01_JPRB, 0.90755E+00_JPRB, 0.81549E+00_JPRB ,&
&  0.75823E+00_JPRB, 0.71464E+00_JPRB, 0.67577E+00_JPRB, 0.64249E+00_JPRB, 0.61998E+00_JPRB, 0.60939E+00_JPRB, 0.63249E+00_JPRB ,&
&  0.74252E+00_JPRB, 0.94496E+00_JPRB, 0.12211E+01_JPRB, 0.17093E+01_JPRB, 0.19730E+01_JPRB, 0.21536E+01_JPRB, 0.25764E+01_JPRB ,&
&  0.34742E+01_JPRB, 0.40158E+01_JPRB, 0.31549E+01_JPRB, 0.24681E+01_JPRB, 0.21025E+01_JPRB, 0.17882E+01_JPRB, 0.15166E+01_JPRB ,&
&  0.13068E+01_JPRB, 0.11817E+01_JPRB, 0.11083E+01_JPRB, 0.10639E+01_JPRB, 0.10339E+01_JPRB, 0.98938E+00_JPRB, 0.92770E+00_JPRB ,&
&  0.87100E+00_JPRB, 0.83295E+00_JPRB, 0.81756E+00_JPRB, 0.81628E+00_JPRB, 0.81876E+00_JPRB, 0.82948E+00_JPRB, 0.83004E+00_JPRB ,&
&  0.81529E+00_JPRB, 0.79033E+00_JPRB, 0.74769E+00_JPRB, 0.71325E+00_JPRB, 0.70115E+00_JPRB, 0.68826E+00_JPRB, 0.67282E+00_JPRB ,&
&  0.66643E+00_JPRB, 0.66790E+00_JPRB, 0.66951E+00_JPRB, 0.67905E+00_JPRB, 0.67890E+00_JPRB, 0.70188E+00_JPRB, 0.67393E+00_JPRB ,&
&  0.66792E+00_JPRB, 0.65232E+00_JPRB, 0.63722E+00_JPRB, 0.62148E+00_JPRB, 0.61621E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.60797E+00_JPRB, 0.60030E+00_JPRB, 0.59035E+00_JPRB, 0.57977E+00_JPRB, 0.56789E+00_JPRB, 0.55830E+00_JPRB, 0.55362E+00_JPRB ,&
&  0.55091E+00_JPRB, 0.55057E+00_JPRB, 0.55057E+00_JPRB, 0.53765E+00_JPRB, 0.52071E+00_JPRB, 0.50522E+00_JPRB, 0.49591E+00_JPRB ,&
&  0.48951E+00_JPRB, 0.48238E+00_JPRB, 0.47310E+00_JPRB, 0.46123E+00_JPRB, 0.45516E+00_JPRB, 0.45135E+00_JPRB, 0.44871E+00_JPRB ,&
&  0.45363E+00_JPRB, 0.45785E+00_JPRB, 0.46341E+00_JPRB, 0.47185E+00_JPRB, 0.48905E+00_JPRB, 0.51911E+00_JPRB, 0.56757E+00_JPRB ,&
&  0.66496E+00_JPRB, 0.87716E+00_JPRB, 0.10916E+01_JPRB, 0.10173E+01_JPRB, 0.10255E+01_JPRB, 0.10629E+01_JPRB, 0.12324E+01_JPRB ,&
&  0.13604E+01_JPRB, 0.13993E+01_JPRB, 0.12851E+01_JPRB, 0.11320E+01_JPRB, 0.99658E+00_JPRB, 0.89070E+00_JPRB, 0.81483E+00_JPRB ,&
&  0.74591E+00_JPRB, 0.68547E+00_JPRB, 0.64227E+00_JPRB, 0.61205E+00_JPRB, 0.59234E+00_JPRB, 0.58045E+00_JPRB, 0.57719E+00_JPRB ,&
&  0.59494E+00_JPRB, 0.66286E+00_JPRB, 0.82200E+00_JPRB, 0.11509E+01_JPRB, 0.18700E+01_JPRB, 0.18898E+01_JPRB, 0.23760E+01_JPRB ,&
&  0.32746E+01_JPRB, 0.31975E+01_JPRB, 0.28644E+01_JPRB, 0.24471E+01_JPRB, 0.20964E+01_JPRB, 0.18029E+01_JPRB, 0.15605E+01_JPRB ,&
&  0.13867E+01_JPRB, 0.12615E+01_JPRB, 0.11545E+01_JPRB, 0.10745E+01_JPRB, 0.10220E+01_JPRB, 0.99741E+00_JPRB, 0.97479E+00_JPRB ,&
&  0.94621E+00_JPRB, 0.91331E+00_JPRB, 0.88950E+00_JPRB, 0.87784E+00_JPRB, 0.86660E+00_JPRB, 0.85782E+00_JPRB, 0.83195E+00_JPRB ,&
&  0.80133E+00_JPRB, 0.77054E+00_JPRB, 0.74210E+00_JPRB, 0.71284E+00_JPRB, 0.70553E+00_JPRB, 0.69595E+00_JPRB, 0.67666E+00_JPRB ,&
&  0.65757E+00_JPRB, 0.65312E+00_JPRB, 0.66555E+00_JPRB, 0.66976E+00_JPRB, 0.67819E+00_JPRB, 0.70009E+00_JPRB, 0.67084E+00_JPRB ,&
&  0.65862E+00_JPRB, 0.63676E+00_JPRB, 0.61821E+00_JPRB, 0.61124E+00_JPRB, 0.61737E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.60347E+00_JPRB, 0.59297E+00_JPRB, 0.58263E+00_JPRB, 0.57224E+00_JPRB, 0.55946E+00_JPRB, 0.54995E+00_JPRB, 0.54177E+00_JPRB ,&
&  0.53606E+00_JPRB, 0.53299E+00_JPRB, 0.53373E+00_JPRB, 0.53414E+00_JPRB, 0.53086E+00_JPRB, 0.52192E+00_JPRB, 0.50698E+00_JPRB ,&
&  0.49737E+00_JPRB, 0.48891E+00_JPRB, 0.47829E+00_JPRB, 0.46536E+00_JPRB, 0.45282E+00_JPRB, 0.44677E+00_JPRB, 0.44594E+00_JPRB ,&
&  0.45006E+00_JPRB, 0.45521E+00_JPRB, 0.45867E+00_JPRB, 0.47017E+00_JPRB, 0.48842E+00_JPRB, 0.51827E+00_JPRB, 0.56000E+00_JPRB ,&
&  0.63134E+00_JPRB, 0.85627E+00_JPRB, 0.91011E+00_JPRB, 0.89679E+00_JPRB, 0.90466E+00_JPRB, 0.92259E+00_JPRB, 0.10122E+01_JPRB ,&
&  0.10994E+01_JPRB, 0.11553E+01_JPRB, 0.11222E+01_JPRB, 0.10083E+01_JPRB, 0.91284E+00_JPRB, 0.84578E+00_JPRB, 0.78036E+00_JPRB ,&
&  0.71778E+00_JPRB, 0.66835E+00_JPRB, 0.62919E+00_JPRB, 0.60244E+00_JPRB, 0.58226E+00_JPRB, 0.56735E+00_JPRB, 0.55936E+00_JPRB ,&
&  0.55887E+00_JPRB, 0.56886E+00_JPRB, 0.61315E+00_JPRB, 0.72390E+00_JPRB, 0.11850E+01_JPRB, 0.16010E+01_JPRB, 0.19649E+01_JPRB ,&
&  0.21101E+01_JPRB, 0.21800E+01_JPRB, 0.22088E+01_JPRB, 0.20656E+01_JPRB, 0.18731E+01_JPRB, 0.16506E+01_JPRB, 0.14892E+01_JPRB ,&
&  0.13802E+01_JPRB, 0.12628E+01_JPRB, 0.11544E+01_JPRB, 0.10544E+01_JPRB, 0.98223E+00_JPRB, 0.94117E+00_JPRB, 0.91581E+00_JPRB ,&
&  0.88998E+00_JPRB, 0.87079E+00_JPRB, 0.85410E+00_JPRB, 0.83957E+00_JPRB, 0.82368E+00_JPRB, 0.80371E+00_JPRB, 0.77774E+00_JPRB ,&
&  0.74728E+00_JPRB, 0.71724E+00_JPRB, 0.69700E+00_JPRB, 0.68445E+00_JPRB, 0.67560E+00_JPRB, 0.66283E+00_JPRB, 0.64767E+00_JPRB ,&
&  0.64409E+00_JPRB, 0.64673E+00_JPRB, 0.65276E+00_JPRB, 0.65716E+00_JPRB, 0.65845E+00_JPRB, 0.67734E+00_JPRB, 0.65579E+00_JPRB ,&
&  0.63650E+00_JPRB, 0.61565E+00_JPRB, 0.59997E+00_JPRB, 0.59220E+00_JPRB, 0.60882E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.60255E+00_JPRB, 0.58791E+00_JPRB, 0.57163E+00_JPRB, 0.55265E+00_JPRB, 0.53695E+00_JPRB, 0.52757E+00_JPRB, 0.52122E+00_JPRB ,&
&  0.51798E+00_JPRB, 0.51684E+00_JPRB, 0.51578E+00_JPRB, 0.51391E+00_JPRB, 0.51089E+00_JPRB, 0.50696E+00_JPRB, 0.49912E+00_JPRB ,&
&  0.48952E+00_JPRB, 0.47795E+00_JPRB, 0.46831E+00_JPRB, 0.45992E+00_JPRB, 0.44543E+00_JPRB, 0.43639E+00_JPRB, 0.43732E+00_JPRB ,&
&  0.44180E+00_JPRB, 0.44704E+00_JPRB, 0.45115E+00_JPRB, 0.46171E+00_JPRB, 0.48351E+00_JPRB, 0.51310E+00_JPRB, 0.54812E+00_JPRB ,&
&  0.61631E+00_JPRB, 0.66060E+00_JPRB, 0.72558E+00_JPRB, 0.74173E+00_JPRB, 0.74817E+00_JPRB, 0.78598E+00_JPRB, 0.84115E+00_JPRB ,&
&  0.87604E+00_JPRB, 0.93937E+00_JPRB, 0.92997E+00_JPRB, 0.85252E+00_JPRB, 0.77414E+00_JPRB, 0.71860E+00_JPRB, 0.68370E+00_JPRB ,&
&  0.65715E+00_JPRB, 0.63377E+00_JPRB, 0.61382E+00_JPRB, 0.60038E+00_JPRB, 0.58748E+00_JPRB, 0.57462E+00_JPRB, 0.56203E+00_JPRB ,&
&  0.55445E+00_JPRB, 0.54986E+00_JPRB, 0.55267E+00_JPRB, 0.57757E+00_JPRB, 0.68168E+00_JPRB, 0.88937E+00_JPRB, 0.11775E+01_JPRB ,&
&  0.14144E+01_JPRB, 0.15579E+01_JPRB, 0.16772E+01_JPRB, 0.16840E+01_JPRB, 0.16140E+01_JPRB, 0.15195E+01_JPRB, 0.14183E+01_JPRB ,&
&  0.12857E+01_JPRB, 0.11324E+01_JPRB, 0.10166E+01_JPRB, 0.93250E+00_JPRB, 0.87325E+00_JPRB, 0.83487E+00_JPRB, 0.81127E+00_JPRB ,&
&  0.79328E+00_JPRB, 0.78829E+00_JPRB, 0.80602E+00_JPRB, 0.82412E+00_JPRB, 0.82277E+00_JPRB, 0.79248E+00_JPRB, 0.74882E+00_JPRB ,&
&  0.70947E+00_JPRB, 0.67900E+00_JPRB, 0.65485E+00_JPRB, 0.63675E+00_JPRB, 0.62281E+00_JPRB, 0.61250E+00_JPRB, 0.60172E+00_JPRB ,&
&  0.59656E+00_JPRB, 0.60643E+00_JPRB, 0.60447E+00_JPRB, 0.61528E+00_JPRB, 0.64570E+00_JPRB, 0.63190E+00_JPRB, 0.61021E+00_JPRB ,&
&  0.59376E+00_JPRB, 0.58417E+00_JPRB, 0.57441E+00_JPRB, 0.56715E+00_JPRB, 0.61712E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.52901E+00_JPRB, 0.52745E+00_JPRB, 0.51820E+00_JPRB, 0.50797E+00_JPRB, 0.49993E+00_JPRB, 0.49626E+00_JPRB, 0.49329E+00_JPRB ,&
&  0.49176E+00_JPRB, 0.49110E+00_JPRB, 0.49060E+00_JPRB, 0.48825E+00_JPRB, 0.48358E+00_JPRB, 0.47650E+00_JPRB, 0.46767E+00_JPRB ,&
&  0.45967E+00_JPRB, 0.45159E+00_JPRB, 0.44451E+00_JPRB, 0.43693E+00_JPRB, 0.42749E+00_JPRB, 0.42227E+00_JPRB, 0.42447E+00_JPRB ,&
&  0.43022E+00_JPRB, 0.43701E+00_JPRB, 0.44295E+00_JPRB, 0.45289E+00_JPRB, 0.47754E+00_JPRB, 0.50620E+00_JPRB, 0.53295E+00_JPRB ,&
&  0.59094E+00_JPRB, 0.57353E+00_JPRB, 0.60339E+00_JPRB, 0.64164E+00_JPRB, 0.67618E+00_JPRB, 0.69292E+00_JPRB, 0.71278E+00_JPRB ,&
&  0.73449E+00_JPRB, 0.75492E+00_JPRB, 0.75461E+00_JPRB, 0.72265E+00_JPRB, 0.66317E+00_JPRB, 0.60867E+00_JPRB, 0.57686E+00_JPRB ,&
&  0.56085E+00_JPRB, 0.55367E+00_JPRB, 0.54712E+00_JPRB, 0.54115E+00_JPRB, 0.53680E+00_JPRB, 0.53272E+00_JPRB, 0.52829E+00_JPRB ,&
&  0.52416E+00_JPRB, 0.52086E+00_JPRB, 0.51920E+00_JPRB, 0.52118E+00_JPRB, 0.54047E+00_JPRB, 0.60568E+00_JPRB, 0.75282E+00_JPRB ,&
&  0.96007E+00_JPRB, 0.11563E+01_JPRB, 0.12668E+01_JPRB, 0.13340E+01_JPRB, 0.13742E+01_JPRB, 0.13186E+01_JPRB, 0.11753E+01_JPRB ,&
&  0.10391E+01_JPRB, 0.92002E+00_JPRB, 0.83131E+00_JPRB, 0.76643E+00_JPRB, 0.72104E+00_JPRB, 0.69217E+00_JPRB, 0.67639E+00_JPRB ,&
&  0.67087E+00_JPRB, 0.67186E+00_JPRB, 0.67930E+00_JPRB, 0.68727E+00_JPRB, 0.69104E+00_JPRB, 0.68470E+00_JPRB, 0.66906E+00_JPRB ,&
&  0.64622E+00_JPRB, 0.62905E+00_JPRB, 0.61194E+00_JPRB, 0.59320E+00_JPRB, 0.57692E+00_JPRB, 0.56179E+00_JPRB, 0.55023E+00_JPRB ,&
&  0.54417E+00_JPRB, 0.54834E+00_JPRB, 0.55931E+00_JPRB, 0.58120E+00_JPRB, 0.57204E+00_JPRB, 0.55744E+00_JPRB, 0.54468E+00_JPRB ,&
&  0.53960E+00_JPRB, 0.53940E+00_JPRB, 0.53173E+00_JPRB, 0.52524E+00_JPRB, 0.52600E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.49052E+00_JPRB, 0.48820E+00_JPRB, 0.48080E+00_JPRB, 0.47342E+00_JPRB, 0.47053E+00_JPRB, 0.47028E+00_JPRB, 0.47111E+00_JPRB ,&
&  0.47010E+00_JPRB, 0.46712E+00_JPRB, 0.46291E+00_JPRB, 0.45986E+00_JPRB, 0.45555E+00_JPRB, 0.44939E+00_JPRB, 0.44180E+00_JPRB ,&
&  0.43457E+00_JPRB, 0.42786E+00_JPRB, 0.42266E+00_JPRB, 0.41710E+00_JPRB, 0.41284E+00_JPRB, 0.41265E+00_JPRB, 0.41424E+00_JPRB ,&
&  0.42026E+00_JPRB, 0.42810E+00_JPRB, 0.43441E+00_JPRB, 0.44873E+00_JPRB, 0.46580E+00_JPRB, 0.49140E+00_JPRB, 0.51745E+00_JPRB ,&
&  0.56746E+00_JPRB, 0.50224E+00_JPRB, 0.52644E+00_JPRB, 0.58322E+00_JPRB, 0.61692E+00_JPRB, 0.63527E+00_JPRB, 0.62825E+00_JPRB ,&
&  0.62305E+00_JPRB, 0.62437E+00_JPRB, 0.62312E+00_JPRB, 0.61561E+00_JPRB, 0.59413E+00_JPRB, 0.55432E+00_JPRB, 0.51572E+00_JPRB ,&
&  0.48933E+00_JPRB, 0.48016E+00_JPRB, 0.47615E+00_JPRB, 0.47034E+00_JPRB, 0.46448E+00_JPRB, 0.46178E+00_JPRB, 0.46224E+00_JPRB ,&
&  0.46238E+00_JPRB, 0.46192E+00_JPRB, 0.46058E+00_JPRB, 0.45918E+00_JPRB, 0.46143E+00_JPRB, 0.47836E+00_JPRB, 0.53358E+00_JPRB ,&
&  0.64502E+00_JPRB, 0.77582E+00_JPRB, 0.89899E+00_JPRB, 0.99570E+00_JPRB, 0.10193E+01_JPRB, 0.99310E+00_JPRB, 0.93785E+00_JPRB ,&
&  0.86867E+00_JPRB, 0.79860E+00_JPRB, 0.74222E+00_JPRB, 0.69699E+00_JPRB, 0.66078E+00_JPRB, 0.63246E+00_JPRB, 0.61170E+00_JPRB ,&
&  0.60067E+00_JPRB, 0.59323E+00_JPRB, 0.58808E+00_JPRB, 0.58102E+00_JPRB, 0.57379E+00_JPRB, 0.56887E+00_JPRB, 0.56672E+00_JPRB ,&
&  0.56189E+00_JPRB, 0.55572E+00_JPRB, 0.54998E+00_JPRB, 0.54358E+00_JPRB, 0.53668E+00_JPRB, 0.52986E+00_JPRB, 0.52404E+00_JPRB ,&
&  0.52135E+00_JPRB, 0.52052E+00_JPRB, 0.52304E+00_JPRB, 0.52137E+00_JPRB, 0.52935E+00_JPRB, 0.51849E+00_JPRB, 0.51027E+00_JPRB ,&
&  0.50696E+00_JPRB, 0.50296E+00_JPRB, 0.50071E+00_JPRB, 0.46278E+00_JPRB, 0.48709E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.45593E+00_JPRB, 0.45748E+00_JPRB, 0.45598E+00_JPRB, 0.45053E+00_JPRB, 0.44687E+00_JPRB, 0.44465E+00_JPRB, 0.44386E+00_JPRB ,&
&  0.44371E+00_JPRB, 0.44272E+00_JPRB, 0.44006E+00_JPRB, 0.43483E+00_JPRB, 0.43023E+00_JPRB, 0.42435E+00_JPRB, 0.41820E+00_JPRB ,&
&  0.41299E+00_JPRB, 0.40843E+00_JPRB, 0.40584E+00_JPRB, 0.40262E+00_JPRB, 0.40141E+00_JPRB, 0.40288E+00_JPRB, 0.40496E+00_JPRB ,&
&  0.40802E+00_JPRB, 0.41072E+00_JPRB, 0.41892E+00_JPRB, 0.42974E+00_JPRB, 0.44976E+00_JPRB, 0.46401E+00_JPRB, 0.48582E+00_JPRB ,&
&  0.53327E+00_JPRB, 0.45612E+00_JPRB, 0.48899E+00_JPRB, 0.53116E+00_JPRB, 0.55819E+00_JPRB, 0.57868E+00_JPRB, 0.55870E+00_JPRB ,&
&  0.54304E+00_JPRB, 0.53290E+00_JPRB, 0.52457E+00_JPRB, 0.51858E+00_JPRB, 0.51449E+00_JPRB, 0.50272E+00_JPRB, 0.48754E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.45801E+00_JPRB, 0.44781E+00_JPRB, 0.43781E+00_JPRB, 0.42909E+00_JPRB, 0.42359E+00_JPRB, 0.42102E+00_JPRB ,&
&  0.42067E+00_JPRB, 0.42086E+00_JPRB, 0.42067E+00_JPRB, 0.42118E+00_JPRB, 0.42319E+00_JPRB, 0.42777E+00_JPRB, 0.43806E+00_JPRB ,&
&  0.47035E+00_JPRB, 0.52980E+00_JPRB, 0.59894E+00_JPRB, 0.66664E+00_JPRB, 0.71606E+00_JPRB, 0.73984E+00_JPRB, 0.74971E+00_JPRB ,&
&  0.73914E+00_JPRB, 0.71307E+00_JPRB, 0.68023E+00_JPRB, 0.64726E+00_JPRB, 0.61800E+00_JPRB, 0.59435E+00_JPRB, 0.57313E+00_JPRB ,&
&  0.55395E+00_JPRB, 0.53500E+00_JPRB, 0.51921E+00_JPRB, 0.50735E+00_JPRB, 0.49656E+00_JPRB, 0.48712E+00_JPRB, 0.48187E+00_JPRB ,&
&  0.47979E+00_JPRB, 0.47939E+00_JPRB, 0.47887E+00_JPRB, 0.47981E+00_JPRB, 0.48119E+00_JPRB, 0.48229E+00_JPRB, 0.48202E+00_JPRB ,&
&  0.47969E+00_JPRB, 0.47897E+00_JPRB, 0.47820E+00_JPRB, 0.48510E+00_JPRB, 0.48342E+00_JPRB, 0.48093E+00_JPRB, 0.47713E+00_JPRB ,&
&  0.47449E+00_JPRB, 0.46918E+00_JPRB, 0.41928E+00_JPRB, 0.44196E+00_JPRB, 0.45052E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.43353E+00_JPRB, 0.43312E+00_JPRB, 0.43397E+00_JPRB, 0.43367E+00_JPRB, 0.43278E+00_JPRB, 0.43115E+00_JPRB, 0.42829E+00_JPRB ,&
&  0.42407E+00_JPRB, 0.42111E+00_JPRB, 0.41849E+00_JPRB, 0.41505E+00_JPRB, 0.41124E+00_JPRB, 0.40796E+00_JPRB, 0.40488E+00_JPRB ,&
&  0.40170E+00_JPRB, 0.39834E+00_JPRB, 0.39611E+00_JPRB, 0.39463E+00_JPRB, 0.39295E+00_JPRB, 0.39242E+00_JPRB, 0.39354E+00_JPRB ,&
&  0.39363E+00_JPRB, 0.39626E+00_JPRB, 0.40623E+00_JPRB, 0.41794E+00_JPRB, 0.43293E+00_JPRB, 0.44684E+00_JPRB, 0.46283E+00_JPRB ,&
&  0.45085E+00_JPRB, 0.40689E+00_JPRB, 0.43419E+00_JPRB, 0.46701E+00_JPRB, 0.50026E+00_JPRB, 0.52520E+00_JPRB, 0.51118E+00_JPRB ,&
&  0.49235E+00_JPRB, 0.47417E+00_JPRB, 0.46040E+00_JPRB, 0.45105E+00_JPRB, 0.44925E+00_JPRB, 0.44775E+00_JPRB, 0.44538E+00_JPRB ,&
&  0.44101E+00_JPRB, 0.43307E+00_JPRB, 0.42464E+00_JPRB, 0.41688E+00_JPRB, 0.41145E+00_JPRB, 0.40711E+00_JPRB, 0.40344E+00_JPRB ,&
&  0.40180E+00_JPRB, 0.40155E+00_JPRB, 0.40154E+00_JPRB, 0.40179E+00_JPRB, 0.40270E+00_JPRB, 0.40567E+00_JPRB, 0.40937E+00_JPRB ,&
&  0.41436E+00_JPRB, 0.42796E+00_JPRB, 0.45739E+00_JPRB, 0.49467E+00_JPRB, 0.53311E+00_JPRB, 0.56344E+00_JPRB, 0.58524E+00_JPRB ,&
&  0.60119E+00_JPRB, 0.60846E+00_JPRB, 0.60413E+00_JPRB, 0.58843E+00_JPRB, 0.56743E+00_JPRB, 0.54696E+00_JPRB, 0.52712E+00_JPRB ,&
&  0.50717E+00_JPRB, 0.48489E+00_JPRB, 0.46330E+00_JPRB, 0.44932E+00_JPRB, 0.44130E+00_JPRB, 0.43407E+00_JPRB, 0.42863E+00_JPRB ,&
&  0.42651E+00_JPRB, 0.42687E+00_JPRB, 0.42857E+00_JPRB, 0.43027E+00_JPRB, 0.43201E+00_JPRB, 0.43470E+00_JPRB, 0.43695E+00_JPRB ,&
&  0.43850E+00_JPRB, 0.44145E+00_JPRB, 0.44627E+00_JPRB, 0.45589E+00_JPRB, 0.46207E+00_JPRB, 0.46434E+00_JPRB, 0.46240E+00_JPRB ,&
&  0.45551E+00_JPRB, 0.44694E+00_JPRB, 0.43855E+00_JPRB, 0.43306E+00_JPRB, 0.43245E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.41685E+00_JPRB, 0.41608E+00_JPRB, 0.41570E+00_JPRB, 0.41430E+00_JPRB, 0.41298E+00_JPRB, 0.41155E+00_JPRB, 0.40787E+00_JPRB ,&
&  0.40631E+00_JPRB, 0.40253E+00_JPRB, 0.39894E+00_JPRB, 0.39724E+00_JPRB, 0.39513E+00_JPRB, 0.39248E+00_JPRB, 0.39050E+00_JPRB ,&
&  0.38832E+00_JPRB, 0.38580E+00_JPRB, 0.38403E+00_JPRB, 0.38166E+00_JPRB, 0.37812E+00_JPRB, 0.37560E+00_JPRB, 0.37615E+00_JPRB ,&
&  0.37695E+00_JPRB, 0.37954E+00_JPRB, 0.38802E+00_JPRB, 0.39641E+00_JPRB, 0.40592E+00_JPRB, 0.41526E+00_JPRB, 0.42363E+00_JPRB ,&
&  0.43674E+00_JPRB, 0.39143E+00_JPRB, 0.41617E+00_JPRB, 0.43454E+00_JPRB, 0.46240E+00_JPRB, 0.48391E+00_JPRB, 0.47593E+00_JPRB ,&
&  0.45664E+00_JPRB, 0.43517E+00_JPRB, 0.41986E+00_JPRB, 0.41091E+00_JPRB, 0.41114E+00_JPRB, 0.41302E+00_JPRB, 0.41181E+00_JPRB ,&
&  0.40871E+00_JPRB, 0.40171E+00_JPRB, 0.39382E+00_JPRB, 0.38873E+00_JPRB, 0.38604E+00_JPRB, 0.38402E+00_JPRB, 0.38187E+00_JPRB ,&
&  0.38058E+00_JPRB, 0.38034E+00_JPRB, 0.38026E+00_JPRB, 0.38019E+00_JPRB, 0.38083E+00_JPRB, 0.38253E+00_JPRB, 0.38478E+00_JPRB ,&
&  0.38649E+00_JPRB, 0.38726E+00_JPRB, 0.39159E+00_JPRB, 0.40384E+00_JPRB, 0.42154E+00_JPRB, 0.43958E+00_JPRB, 0.45452E+00_JPRB ,&
&  0.46685E+00_JPRB, 0.47787E+00_JPRB, 0.48509E+00_JPRB, 0.48546E+00_JPRB, 0.47879E+00_JPRB, 0.46781E+00_JPRB, 0.45544E+00_JPRB ,&
&  0.44218E+00_JPRB, 0.42829E+00_JPRB, 0.41284E+00_JPRB, 0.40314E+00_JPRB, 0.39836E+00_JPRB, 0.39559E+00_JPRB, 0.39377E+00_JPRB ,&
&  0.39279E+00_JPRB, 0.39245E+00_JPRB, 0.39350E+00_JPRB, 0.39510E+00_JPRB, 0.39605E+00_JPRB, 0.39696E+00_JPRB, 0.39877E+00_JPRB ,&
&  0.40221E+00_JPRB, 0.40780E+00_JPRB, 0.41506E+00_JPRB, 0.42504E+00_JPRB, 0.43361E+00_JPRB, 0.43892E+00_JPRB, 0.44144E+00_JPRB ,&
&  0.43891E+00_JPRB, 0.43312E+00_JPRB, 0.42560E+00_JPRB, 0.41910E+00_JPRB, 0.41770E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.40443E+00_JPRB, 0.40084E+00_JPRB, 0.39832E+00_JPRB, 0.39425E+00_JPRB, 0.39102E+00_JPRB, 0.38854E+00_JPRB, 0.38588E+00_JPRB ,&
&  0.38398E+00_JPRB, 0.38401E+00_JPRB, 0.38257E+00_JPRB, 0.38169E+00_JPRB, 0.38048E+00_JPRB, 0.37754E+00_JPRB, 0.37407E+00_JPRB ,&
&  0.37221E+00_JPRB, 0.37054E+00_JPRB, 0.36905E+00_JPRB, 0.36732E+00_JPRB, 0.36551E+00_JPRB, 0.36298E+00_JPRB, 0.36296E+00_JPRB ,&
&  0.36391E+00_JPRB, 0.36654E+00_JPRB, 0.36997E+00_JPRB, 0.37397E+00_JPRB, 0.38019E+00_JPRB, 0.38868E+00_JPRB, 0.39669E+00_JPRB ,&
&  0.40548E+00_JPRB, 0.39357E+00_JPRB, 0.40864E+00_JPRB, 0.41741E+00_JPRB, 0.43316E+00_JPRB, 0.44595E+00_JPRB, 0.44524E+00_JPRB ,&
&  0.43712E+00_JPRB, 0.42170E+00_JPRB, 0.40822E+00_JPRB, 0.39978E+00_JPRB, 0.39777E+00_JPRB, 0.39620E+00_JPRB, 0.39080E+00_JPRB ,&
&  0.38354E+00_JPRB, 0.37487E+00_JPRB, 0.36819E+00_JPRB, 0.36460E+00_JPRB, 0.36338E+00_JPRB, 0.36322E+00_JPRB, 0.36185E+00_JPRB ,&
&  0.36032E+00_JPRB, 0.35876E+00_JPRB, 0.35798E+00_JPRB, 0.35760E+00_JPRB, 0.35717E+00_JPRB, 0.35734E+00_JPRB, 0.35815E+00_JPRB ,&
&  0.35902E+00_JPRB, 0.35952E+00_JPRB, 0.35974E+00_JPRB, 0.36160E+00_JPRB, 0.36668E+00_JPRB, 0.37424E+00_JPRB, 0.38186E+00_JPRB ,&
&  0.38764E+00_JPRB, 0.39321E+00_JPRB, 0.39834E+00_JPRB, 0.40205E+00_JPRB, 0.40168E+00_JPRB, 0.39675E+00_JPRB, 0.38969E+00_JPRB ,&
&  0.38343E+00_JPRB, 0.37768E+00_JPRB, 0.37205E+00_JPRB, 0.36754E+00_JPRB, 0.36499E+00_JPRB, 0.36444E+00_JPRB, 0.36463E+00_JPRB ,&
&  0.36506E+00_JPRB, 0.36571E+00_JPRB, 0.36679E+00_JPRB, 0.36861E+00_JPRB, 0.36982E+00_JPRB, 0.37107E+00_JPRB, 0.37245E+00_JPRB ,&
&  0.37614E+00_JPRB, 0.38078E+00_JPRB, 0.38737E+00_JPRB, 0.39714E+00_JPRB, 0.40598E+00_JPRB, 0.41226E+00_JPRB, 0.41742E+00_JPRB ,&
&  0.41937E+00_JPRB, 0.41821E+00_JPRB, 0.41532E+00_JPRB, 0.41223E+00_JPRB, 0.40883E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.38038E+00_JPRB, 0.37537E+00_JPRB, 0.37162E+00_JPRB, 0.36866E+00_JPRB, 0.36810E+00_JPRB, 0.36640E+00_JPRB, 0.36292E+00_JPRB ,&
&  0.36017E+00_JPRB, 0.36114E+00_JPRB, 0.36364E+00_JPRB, 0.36458E+00_JPRB, 0.36369E+00_JPRB, 0.36204E+00_JPRB, 0.35904E+00_JPRB ,&
&  0.35501E+00_JPRB, 0.35307E+00_JPRB, 0.35295E+00_JPRB, 0.35390E+00_JPRB, 0.35375E+00_JPRB, 0.35186E+00_JPRB, 0.35029E+00_JPRB ,&
&  0.35098E+00_JPRB, 0.35243E+00_JPRB, 0.35282E+00_JPRB, 0.35482E+00_JPRB, 0.35921E+00_JPRB, 0.36736E+00_JPRB, 0.37480E+00_JPRB ,&
&  0.37801E+00_JPRB, 0.37946E+00_JPRB, 0.37339E+00_JPRB, 0.38601E+00_JPRB, 0.39449E+00_JPRB, 0.40575E+00_JPRB, 0.41253E+00_JPRB ,&
&  0.41368E+00_JPRB, 0.40847E+00_JPRB, 0.39672E+00_JPRB, 0.38749E+00_JPRB, 0.38161E+00_JPRB, 0.37527E+00_JPRB, 0.36566E+00_JPRB ,&
&  0.35581E+00_JPRB, 0.34749E+00_JPRB, 0.34260E+00_JPRB, 0.34041E+00_JPRB, 0.34062E+00_JPRB, 0.34128E+00_JPRB, 0.33961E+00_JPRB ,&
&  0.33757E+00_JPRB, 0.33558E+00_JPRB, 0.33477E+00_JPRB, 0.33394E+00_JPRB, 0.33273E+00_JPRB, 0.33175E+00_JPRB, 0.33145E+00_JPRB ,&
&  0.33157E+00_JPRB, 0.33168E+00_JPRB, 0.33207E+00_JPRB, 0.33283E+00_JPRB, 0.33345E+00_JPRB, 0.33473E+00_JPRB, 0.33745E+00_JPRB ,&
&  0.34034E+00_JPRB, 0.34276E+00_JPRB, 0.34489E+00_JPRB, 0.34648E+00_JPRB, 0.34711E+00_JPRB, 0.34651E+00_JPRB, 0.34407E+00_JPRB ,&
&  0.34110E+00_JPRB, 0.33973E+00_JPRB, 0.33898E+00_JPRB, 0.33737E+00_JPRB, 0.33565E+00_JPRB, 0.33535E+00_JPRB, 0.33637E+00_JPRB ,&
&  0.33764E+00_JPRB, 0.33948E+00_JPRB, 0.34112E+00_JPRB, 0.34266E+00_JPRB, 0.34343E+00_JPRB, 0.34473E+00_JPRB, 0.34623E+00_JPRB ,&
&  0.34839E+00_JPRB, 0.35159E+00_JPRB, 0.35610E+00_JPRB, 0.36199E+00_JPRB, 0.36812E+00_JPRB, 0.37295E+00_JPRB, 0.37816E+00_JPRB ,&
&  0.38288E+00_JPRB, 0.38540E+00_JPRB, 0.38598E+00_JPRB, 0.38638E+00_JPRB, 0.38479E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.35185E+00_JPRB, 0.34835E+00_JPRB, 0.34566E+00_JPRB, 0.34483E+00_JPRB, 0.34608E+00_JPRB, 0.34564E+00_JPRB, 0.34305E+00_JPRB ,&
&  0.34069E+00_JPRB, 0.34164E+00_JPRB, 0.34480E+00_JPRB, 0.34666E+00_JPRB, 0.34748E+00_JPRB, 0.34830E+00_JPRB, 0.34703E+00_JPRB ,&
&  0.34338E+00_JPRB, 0.33986E+00_JPRB, 0.34044E+00_JPRB, 0.34226E+00_JPRB, 0.34331E+00_JPRB, 0.34168E+00_JPRB, 0.33977E+00_JPRB ,&
&  0.34011E+00_JPRB, 0.34019E+00_JPRB, 0.34185E+00_JPRB, 0.34353E+00_JPRB, 0.34741E+00_JPRB, 0.35406E+00_JPRB, 0.35908E+00_JPRB ,&
&  0.36098E+00_JPRB, 0.35927E+00_JPRB, 0.35837E+00_JPRB, 0.36137E+00_JPRB, 0.36500E+00_JPRB, 0.37318E+00_JPRB, 0.38182E+00_JPRB ,&
&  0.38514E+00_JPRB, 0.38202E+00_JPRB, 0.37091E+00_JPRB, 0.36096E+00_JPRB, 0.35407E+00_JPRB, 0.34689E+00_JPRB, 0.33980E+00_JPRB ,&
&  0.33265E+00_JPRB, 0.32753E+00_JPRB, 0.32462E+00_JPRB, 0.32343E+00_JPRB, 0.32312E+00_JPRB, 0.32248E+00_JPRB, 0.32013E+00_JPRB ,&
&  0.31711E+00_JPRB, 0.31477E+00_JPRB, 0.31299E+00_JPRB, 0.31163E+00_JPRB, 0.31038E+00_JPRB, 0.30976E+00_JPRB, 0.30904E+00_JPRB ,&
&  0.30869E+00_JPRB, 0.30907E+00_JPRB, 0.30985E+00_JPRB, 0.30982E+00_JPRB, 0.30958E+00_JPRB, 0.30890E+00_JPRB, 0.30900E+00_JPRB ,&
&  0.31023E+00_JPRB, 0.31036E+00_JPRB, 0.31043E+00_JPRB, 0.31166E+00_JPRB, 0.31262E+00_JPRB, 0.31357E+00_JPRB, 0.31375E+00_JPRB ,&
&  0.31352E+00_JPRB, 0.31366E+00_JPRB, 0.31404E+00_JPRB, 0.31391E+00_JPRB, 0.31352E+00_JPRB, 0.31325E+00_JPRB, 0.31343E+00_JPRB ,&
&  0.31485E+00_JPRB, 0.31670E+00_JPRB, 0.31812E+00_JPRB, 0.31946E+00_JPRB, 0.32097E+00_JPRB, 0.32332E+00_JPRB, 0.32437E+00_JPRB ,&
&  0.32507E+00_JPRB, 0.32659E+00_JPRB, 0.32974E+00_JPRB, 0.33389E+00_JPRB, 0.33785E+00_JPRB, 0.34065E+00_JPRB, 0.34376E+00_JPRB ,&
&  0.34674E+00_JPRB, 0.34935E+00_JPRB, 0.35103E+00_JPRB, 0.35175E+00_JPRB, 0.35234E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.32485E+00_JPRB, 0.32359E+00_JPRB, 0.32308E+00_JPRB, 0.32331E+00_JPRB, 0.32485E+00_JPRB, 0.32453E+00_JPRB, 0.32372E+00_JPRB ,&
&  0.32300E+00_JPRB, 0.32478E+00_JPRB, 0.32849E+00_JPRB, 0.33088E+00_JPRB, 0.33144E+00_JPRB, 0.33155E+00_JPRB, 0.33118E+00_JPRB ,&
&  0.33082E+00_JPRB, 0.33054E+00_JPRB, 0.32967E+00_JPRB, 0.33029E+00_JPRB, 0.33023E+00_JPRB, 0.32906E+00_JPRB, 0.32979E+00_JPRB ,&
&  0.32982E+00_JPRB, 0.33031E+00_JPRB, 0.33255E+00_JPRB, 0.33437E+00_JPRB, 0.33845E+00_JPRB, 0.34516E+00_JPRB, 0.34891E+00_JPRB ,&
&  0.34892E+00_JPRB, 0.34442E+00_JPRB, 0.34044E+00_JPRB, 0.34090E+00_JPRB, 0.34290E+00_JPRB, 0.34713E+00_JPRB, 0.35185E+00_JPRB ,&
&  0.35325E+00_JPRB, 0.34709E+00_JPRB, 0.33710E+00_JPRB, 0.32959E+00_JPRB, 0.32449E+00_JPRB, 0.32068E+00_JPRB, 0.31726E+00_JPRB ,&
&  0.31373E+00_JPRB, 0.31138E+00_JPRB, 0.31056E+00_JPRB, 0.31066E+00_JPRB, 0.31035E+00_JPRB, 0.30740E+00_JPRB, 0.30389E+00_JPRB ,&
&  0.29947E+00_JPRB, 0.29637E+00_JPRB, 0.29383E+00_JPRB, 0.29293E+00_JPRB, 0.29301E+00_JPRB, 0.29268E+00_JPRB, 0.29154E+00_JPRB ,&
&  0.29011E+00_JPRB, 0.28958E+00_JPRB, 0.28913E+00_JPRB, 0.28788E+00_JPRB, 0.28631E+00_JPRB, 0.28573E+00_JPRB, 0.28488E+00_JPRB ,&
&  0.28363E+00_JPRB, 0.28379E+00_JPRB, 0.28502E+00_JPRB, 0.28698E+00_JPRB, 0.28754E+00_JPRB, 0.28901E+00_JPRB, 0.29113E+00_JPRB ,&
&  0.29081E+00_JPRB, 0.28848E+00_JPRB, 0.28688E+00_JPRB, 0.28691E+00_JPRB, 0.28840E+00_JPRB, 0.29069E+00_JPRB, 0.29169E+00_JPRB ,&
&  0.29259E+00_JPRB, 0.29251E+00_JPRB, 0.29344E+00_JPRB, 0.29492E+00_JPRB, 0.29707E+00_JPRB, 0.29907E+00_JPRB, 0.30036E+00_JPRB ,&
&  0.30120E+00_JPRB, 0.30189E+00_JPRB, 0.30415E+00_JPRB, 0.30754E+00_JPRB, 0.31046E+00_JPRB, 0.31373E+00_JPRB, 0.31624E+00_JPRB ,&
&  0.31801E+00_JPRB, 0.31990E+00_JPRB, 0.32120E+00_JPRB, 0.32218E+00_JPRB, 0.32355E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.30736E+00_JPRB, 0.30801E+00_JPRB, 0.30923E+00_JPRB, 0.30960E+00_JPRB, 0.31046E+00_JPRB, 0.31075E+00_JPRB, 0.31169E+00_JPRB ,&
&  0.31205E+00_JPRB, 0.31355E+00_JPRB, 0.31579E+00_JPRB, 0.31585E+00_JPRB, 0.31561E+00_JPRB, 0.31556E+00_JPRB, 0.31638E+00_JPRB ,&
&  0.31887E+00_JPRB, 0.32013E+00_JPRB, 0.31986E+00_JPRB, 0.32005E+00_JPRB, 0.31910E+00_JPRB, 0.31812E+00_JPRB, 0.31998E+00_JPRB ,&
&  0.32046E+00_JPRB, 0.32122E+00_JPRB, 0.32321E+00_JPRB, 0.32496E+00_JPRB, 0.32899E+00_JPRB, 0.33404E+00_JPRB, 0.33797E+00_JPRB ,&
&  0.33860E+00_JPRB, 0.33538E+00_JPRB, 0.32956E+00_JPRB, 0.32505E+00_JPRB, 0.32404E+00_JPRB, 0.32493E+00_JPRB, 0.32722E+00_JPRB ,&
&  0.32631E+00_JPRB, 0.32037E+00_JPRB, 0.31277E+00_JPRB, 0.30849E+00_JPRB, 0.30672E+00_JPRB, 0.30417E+00_JPRB, 0.30036E+00_JPRB ,&
&  0.29745E+00_JPRB, 0.29676E+00_JPRB, 0.29715E+00_JPRB, 0.29655E+00_JPRB, 0.29481E+00_JPRB, 0.29161E+00_JPRB, 0.28752E+00_JPRB ,&
&  0.28386E+00_JPRB, 0.28026E+00_JPRB, 0.27798E+00_JPRB, 0.27648E+00_JPRB, 0.27636E+00_JPRB, 0.27644E+00_JPRB, 0.27542E+00_JPRB ,&
&  0.27328E+00_JPRB, 0.27183E+00_JPRB, 0.27001E+00_JPRB, 0.26678E+00_JPRB, 0.26449E+00_JPRB, 0.26384E+00_JPRB, 0.26294E+00_JPRB ,&
&  0.26174E+00_JPRB, 0.26178E+00_JPRB, 0.26211E+00_JPRB, 0.26214E+00_JPRB, 0.26360E+00_JPRB, 0.26589E+00_JPRB, 0.26371E+00_JPRB ,&
&  0.26169E+00_JPRB, 0.26032E+00_JPRB, 0.26038E+00_JPRB, 0.26200E+00_JPRB, 0.26473E+00_JPRB, 0.26768E+00_JPRB, 0.27010E+00_JPRB ,&
&  0.27111E+00_JPRB, 0.27132E+00_JPRB, 0.27163E+00_JPRB, 0.27190E+00_JPRB, 0.27345E+00_JPRB, 0.27638E+00_JPRB, 0.27972E+00_JPRB ,&
&  0.28119E+00_JPRB, 0.28376E+00_JPRB, 0.28665E+00_JPRB, 0.29031E+00_JPRB, 0.29250E+00_JPRB, 0.29457E+00_JPRB, 0.29645E+00_JPRB ,&
&  0.29782E+00_JPRB, 0.29958E+00_JPRB, 0.30191E+00_JPRB, 0.30377E+00_JPRB, 0.30572E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.29447E+00_JPRB, 0.29597E+00_JPRB, 0.29681E+00_JPRB, 0.29824E+00_JPRB, 0.30076E+00_JPRB, 0.30213E+00_JPRB, 0.30198E+00_JPRB ,&
&  0.30199E+00_JPRB, 0.30335E+00_JPRB, 0.30445E+00_JPRB, 0.30498E+00_JPRB, 0.30274E+00_JPRB, 0.30249E+00_JPRB, 0.30437E+00_JPRB ,&
&  0.30684E+00_JPRB, 0.30910E+00_JPRB, 0.30946E+00_JPRB, 0.30966E+00_JPRB, 0.30894E+00_JPRB, 0.30762E+00_JPRB, 0.30908E+00_JPRB ,&
&  0.31039E+00_JPRB, 0.31098E+00_JPRB, 0.31236E+00_JPRB, 0.31383E+00_JPRB, 0.31699E+00_JPRB, 0.32073E+00_JPRB, 0.32329E+00_JPRB ,&
&  0.32413E+00_JPRB, 0.32258E+00_JPRB, 0.31548E+00_JPRB, 0.30829E+00_JPRB, 0.30826E+00_JPRB, 0.30910E+00_JPRB, 0.30886E+00_JPRB ,&
&  0.30663E+00_JPRB, 0.30368E+00_JPRB, 0.30170E+00_JPRB, 0.30014E+00_JPRB, 0.29815E+00_JPRB, 0.29442E+00_JPRB, 0.29033E+00_JPRB ,&
&  0.28691E+00_JPRB, 0.28582E+00_JPRB, 0.28461E+00_JPRB, 0.28226E+00_JPRB, 0.27861E+00_JPRB, 0.27557E+00_JPRB, 0.27239E+00_JPRB ,&
&  0.26693E+00_JPRB, 0.26166E+00_JPRB, 0.25879E+00_JPRB, 0.25805E+00_JPRB, 0.25834E+00_JPRB, 0.25780E+00_JPRB, 0.25584E+00_JPRB ,&
&  0.25357E+00_JPRB, 0.25084E+00_JPRB, 0.24922E+00_JPRB, 0.24884E+00_JPRB, 0.25013E+00_JPRB, 0.24882E+00_JPRB, 0.24436E+00_JPRB ,&
&  0.24202E+00_JPRB, 0.24090E+00_JPRB, 0.24158E+00_JPRB, 0.24094E+00_JPRB, 0.24042E+00_JPRB, 0.23944E+00_JPRB, 0.23990E+00_JPRB ,&
&  0.24109E+00_JPRB, 0.24127E+00_JPRB, 0.24395E+00_JPRB, 0.24872E+00_JPRB, 0.25295E+00_JPRB, 0.25543E+00_JPRB, 0.25698E+00_JPRB ,&
&  0.25763E+00_JPRB, 0.25726E+00_JPRB, 0.25683E+00_JPRB, 0.25753E+00_JPRB, 0.25992E+00_JPRB, 0.26322E+00_JPRB, 0.26613E+00_JPRB ,&
&  0.26661E+00_JPRB, 0.26773E+00_JPRB, 0.27067E+00_JPRB, 0.27511E+00_JPRB, 0.27771E+00_JPRB, 0.27978E+00_JPRB, 0.28142E+00_JPRB ,&
&  0.28209E+00_JPRB, 0.28352E+00_JPRB, 0.28705E+00_JPRB, 0.28926E+00_JPRB, 0.29161E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.28119E+00_JPRB, 0.28209E+00_JPRB, 0.28401E+00_JPRB, 0.28630E+00_JPRB, 0.28733E+00_JPRB, 0.28676E+00_JPRB, 0.28580E+00_JPRB ,&
&  0.28613E+00_JPRB, 0.28836E+00_JPRB, 0.29074E+00_JPRB, 0.29147E+00_JPRB, 0.29190E+00_JPRB, 0.29243E+00_JPRB, 0.29433E+00_JPRB ,&
&  0.29706E+00_JPRB, 0.30026E+00_JPRB, 0.30040E+00_JPRB, 0.30034E+00_JPRB, 0.29990E+00_JPRB, 0.29885E+00_JPRB, 0.30006E+00_JPRB ,&
&  0.30315E+00_JPRB, 0.30334E+00_JPRB, 0.30306E+00_JPRB, 0.30227E+00_JPRB, 0.30395E+00_JPRB, 0.30722E+00_JPRB, 0.30923E+00_JPRB ,&
&  0.30809E+00_JPRB, 0.30112E+00_JPRB, 0.29279E+00_JPRB, 0.29220E+00_JPRB, 0.29383E+00_JPRB, 0.29545E+00_JPRB, 0.29552E+00_JPRB ,&
&  0.29502E+00_JPRB, 0.29414E+00_JPRB, 0.29307E+00_JPRB, 0.29198E+00_JPRB, 0.29057E+00_JPRB, 0.28648E+00_JPRB, 0.28169E+00_JPRB ,&
&  0.27788E+00_JPRB, 0.27458E+00_JPRB, 0.27133E+00_JPRB, 0.26669E+00_JPRB, 0.26197E+00_JPRB, 0.25854E+00_JPRB, 0.25452E+00_JPRB ,&
&  0.24957E+00_JPRB, 0.24614E+00_JPRB, 0.24507E+00_JPRB, 0.24495E+00_JPRB, 0.24498E+00_JPRB, 0.24410E+00_JPRB, 0.24295E+00_JPRB ,&
&  0.24097E+00_JPRB, 0.23948E+00_JPRB, 0.23791E+00_JPRB, 0.23700E+00_JPRB, 0.23128E+00_JPRB, 0.22333E+00_JPRB, 0.21759E+00_JPRB ,&
&  0.21713E+00_JPRB, 0.21991E+00_JPRB, 0.22351E+00_JPRB, 0.22466E+00_JPRB, 0.22708E+00_JPRB, 0.22820E+00_JPRB, 0.22778E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22170E+00_JPRB, 0.22245E+00_JPRB, 0.22370E+00_JPRB, 0.22421E+00_JPRB, 0.22551E+00_JPRB, 0.22694E+00_JPRB ,&
&  0.22988E+00_JPRB, 0.23218E+00_JPRB, 0.23459E+00_JPRB, 0.23716E+00_JPRB, 0.23908E+00_JPRB, 0.23932E+00_JPRB, 0.23711E+00_JPRB ,&
&  0.23708E+00_JPRB, 0.24288E+00_JPRB, 0.25297E+00_JPRB, 0.26304E+00_JPRB, 0.26583E+00_JPRB, 0.26832E+00_JPRB, 0.27050E+00_JPRB ,&
&  0.27160E+00_JPRB, 0.27502E+00_JPRB, 0.27784E+00_JPRB, 0.27926E+00_JPRB, 0.28014E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.27235E+00_JPRB, 0.27175E+00_JPRB, 0.27028E+00_JPRB, 0.26902E+00_JPRB, 0.26738E+00_JPRB, 0.26587E+00_JPRB, 0.26652E+00_JPRB ,&
&  0.26917E+00_JPRB, 0.27267E+00_JPRB, 0.27606E+00_JPRB, 0.27865E+00_JPRB, 0.28066E+00_JPRB, 0.28170E+00_JPRB, 0.28328E+00_JPRB ,&
&  0.28510E+00_JPRB, 0.28834E+00_JPRB, 0.28972E+00_JPRB, 0.29091E+00_JPRB, 0.29027E+00_JPRB, 0.28957E+00_JPRB, 0.29070E+00_JPRB ,&
&  0.29396E+00_JPRB, 0.29540E+00_JPRB, 0.29443E+00_JPRB, 0.29218E+00_JPRB, 0.29265E+00_JPRB, 0.29466E+00_JPRB, 0.29416E+00_JPRB ,&
&  0.28720E+00_JPRB, 0.27340E+00_JPRB, 0.26420E+00_JPRB, 0.26741E+00_JPRB, 0.27572E+00_JPRB, 0.27952E+00_JPRB, 0.28238E+00_JPRB ,&
&  0.28421E+00_JPRB, 0.28443E+00_JPRB, 0.28359E+00_JPRB, 0.28202E+00_JPRB, 0.27929E+00_JPRB, 0.27450E+00_JPRB, 0.26881E+00_JPRB ,&
&  0.26377E+00_JPRB, 0.26135E+00_JPRB, 0.25824E+00_JPRB, 0.25259E+00_JPRB, 0.24712E+00_JPRB, 0.24375E+00_JPRB, 0.23948E+00_JPRB ,&
&  0.23507E+00_JPRB, 0.23191E+00_JPRB, 0.23122E+00_JPRB, 0.23075E+00_JPRB, 0.23053E+00_JPRB, 0.22976E+00_JPRB, 0.22716E+00_JPRB ,&
&  0.22395E+00_JPRB, 0.21952E+00_JPRB, 0.21384E+00_JPRB, 0.20864E+00_JPRB, 0.20506E+00_JPRB, 0.20139E+00_JPRB, 0.19687E+00_JPRB ,&
&  0.19465E+00_JPRB, 0.19733E+00_JPRB, 0.20658E+00_JPRB, 0.21659E+00_JPRB, 0.22054E+00_JPRB, 0.20898E+00_JPRB, 0.20529E+00_JPRB ,&
&  0.20484E+00_JPRB, 0.20474E+00_JPRB, 0.20515E+00_JPRB, 0.20663E+00_JPRB, 0.20733E+00_JPRB, 0.20698E+00_JPRB, 0.20759E+00_JPRB ,&
&  0.20984E+00_JPRB, 0.21214E+00_JPRB, 0.21334E+00_JPRB, 0.21474E+00_JPRB, 0.21560E+00_JPRB, 0.21542E+00_JPRB, 0.21500E+00_JPRB ,&
&  0.21572E+00_JPRB, 0.21902E+00_JPRB, 0.22434E+00_JPRB, 0.23180E+00_JPRB, 0.23700E+00_JPRB, 0.24174E+00_JPRB, 0.24767E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.26227E+00_JPRB, 0.26662E+00_JPRB, 0.27038E+00_JPRB, 0.27184E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.26404E+00_JPRB, 0.26459E+00_JPRB, 0.26046E+00_JPRB, 0.25688E+00_JPRB, 0.25508E+00_JPRB, 0.25461E+00_JPRB, 0.25605E+00_JPRB ,&
&  0.25935E+00_JPRB, 0.26199E+00_JPRB, 0.26456E+00_JPRB, 0.26681E+00_JPRB, 0.26777E+00_JPRB, 0.26758E+00_JPRB, 0.26865E+00_JPRB ,&
&  0.26975E+00_JPRB, 0.27260E+00_JPRB, 0.27452E+00_JPRB, 0.27755E+00_JPRB, 0.27787E+00_JPRB, 0.27867E+00_JPRB, 0.27996E+00_JPRB ,&
&  0.27948E+00_JPRB, 0.27985E+00_JPRB, 0.28121E+00_JPRB, 0.27876E+00_JPRB, 0.27715E+00_JPRB, 0.27478E+00_JPRB, 0.27290E+00_JPRB ,&
&  0.26610E+00_JPRB, 0.25910E+00_JPRB, 0.25265E+00_JPRB, 0.25657E+00_JPRB, 0.26423E+00_JPRB, 0.26557E+00_JPRB, 0.26967E+00_JPRB ,&
&  0.27095E+00_JPRB, 0.26928E+00_JPRB, 0.26766E+00_JPRB, 0.26440E+00_JPRB, 0.26129E+00_JPRB, 0.25774E+00_JPRB, 0.25488E+00_JPRB ,&
&  0.25162E+00_JPRB, 0.25236E+00_JPRB, 0.24799E+00_JPRB, 0.23401E+00_JPRB, 0.22705E+00_JPRB, 0.22267E+00_JPRB, 0.21538E+00_JPRB ,&
&  0.20795E+00_JPRB, 0.20606E+00_JPRB, 0.20570E+00_JPRB, 0.20311E+00_JPRB, 0.20291E+00_JPRB, 0.20324E+00_JPRB, 0.20313E+00_JPRB ,&
&  0.20168E+00_JPRB, 0.20139E+00_JPRB, 0.20001E+00_JPRB, 0.19792E+00_JPRB, 0.19587E+00_JPRB, 0.19500E+00_JPRB, 0.19538E+00_JPRB ,&
&  0.19693E+00_JPRB, 0.19852E+00_JPRB, 0.20430E+00_JPRB, 0.20783E+00_JPRB, 0.20258E+00_JPRB, 0.20021E+00_JPRB, 0.20102E+00_JPRB ,&
&  0.19683E+00_JPRB, 0.19608E+00_JPRB, 0.19608E+00_JPRB, 0.19608E+00_JPRB, 0.19654E+00_JPRB, 0.19751E+00_JPRB, 0.19738E+00_JPRB ,&
&  0.19810E+00_JPRB, 0.19959E+00_JPRB, 0.20000E+00_JPRB, 0.20003E+00_JPRB, 0.19970E+00_JPRB, 0.20015E+00_JPRB, 0.20143E+00_JPRB ,&
&  0.20292E+00_JPRB, 0.20580E+00_JPRB, 0.20879E+00_JPRB, 0.21320E+00_JPRB, 0.21676E+00_JPRB, 0.21961E+00_JPRB, 0.22482E+00_JPRB ,&
&  0.22913E+00_JPRB, 0.23892E+00_JPRB, 0.25518E+00_JPRB, 0.26449E+00_JPRB, 0.25968E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.25399E+00_JPRB, 0.25261E+00_JPRB, 0.25097E+00_JPRB, 0.24945E+00_JPRB, 0.25034E+00_JPRB, 0.25156E+00_JPRB, 0.25327E+00_JPRB ,&
&  0.25517E+00_JPRB, 0.25573E+00_JPRB, 0.25563E+00_JPRB, 0.25325E+00_JPRB, 0.25199E+00_JPRB, 0.24615E+00_JPRB, 0.24411E+00_JPRB ,&
&  0.24265E+00_JPRB, 0.24485E+00_JPRB, 0.24777E+00_JPRB, 0.25210E+00_JPRB, 0.25385E+00_JPRB, 0.25569E+00_JPRB, 0.25465E+00_JPRB ,&
&  0.25238E+00_JPRB, 0.24967E+00_JPRB, 0.24700E+00_JPRB, 0.24490E+00_JPRB, 0.24425E+00_JPRB, 0.24238E+00_JPRB, 0.24045E+00_JPRB ,&
&  0.24041E+00_JPRB, 0.24189E+00_JPRB, 0.24259E+00_JPRB, 0.24332E+00_JPRB, 0.24912E+00_JPRB, 0.24944E+00_JPRB, 0.25169E+00_JPRB ,&
&  0.25244E+00_JPRB, 0.25328E+00_JPRB, 0.25286E+00_JPRB, 0.25160E+00_JPRB, 0.25043E+00_JPRB, 0.24456E+00_JPRB, 0.24218E+00_JPRB ,&
&  0.23517E+00_JPRB, 0.23112E+00_JPRB, 0.22419E+00_JPRB, 0.21782E+00_JPRB, 0.21062E+00_JPRB, 0.20523E+00_JPRB, 0.20065E+00_JPRB ,&
&  0.19700E+00_JPRB, 0.19426E+00_JPRB, 0.19205E+00_JPRB, 0.19176E+00_JPRB, 0.19181E+00_JPRB, 0.19135E+00_JPRB, 0.19152E+00_JPRB ,&
&  0.19245E+00_JPRB, 0.19346E+00_JPRB, 0.19336E+00_JPRB, 0.19395E+00_JPRB, 0.19447E+00_JPRB, 0.19524E+00_JPRB, 0.19676E+00_JPRB ,&
&  0.19911E+00_JPRB, 0.20107E+00_JPRB, 0.20170E+00_JPRB, 0.20377E+00_JPRB, 0.20312E+00_JPRB, 0.20039E+00_JPRB, 0.19773E+00_JPRB ,&
&  0.19463E+00_JPRB, 0.19330E+00_JPRB, 0.19177E+00_JPRB, 0.19091E+00_JPRB, 0.19079E+00_JPRB, 0.19098E+00_JPRB, 0.19181E+00_JPRB ,&
&  0.19270E+00_JPRB, 0.19363E+00_JPRB, 0.19396E+00_JPRB, 0.19440E+00_JPRB, 0.19476E+00_JPRB, 0.19564E+00_JPRB, 0.19736E+00_JPRB ,&
&  0.19975E+00_JPRB, 0.20250E+00_JPRB, 0.20521E+00_JPRB, 0.20847E+00_JPRB, 0.21381E+00_JPRB, 0.22085E+00_JPRB, 0.23148E+00_JPRB ,&
&  0.23956E+00_JPRB, 0.25136E+00_JPRB, 0.25049E+00_JPRB, 0.25330E+00_JPRB, 0.25458E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.24647E+00_JPRB, 0.24632E+00_JPRB, 0.24540E+00_JPRB, 0.24483E+00_JPRB, 0.24529E+00_JPRB, 0.24611E+00_JPRB, 0.24728E+00_JPRB ,&
&  0.24388E+00_JPRB, 0.24150E+00_JPRB, 0.23715E+00_JPRB, 0.23238E+00_JPRB, 0.22813E+00_JPRB, 0.22495E+00_JPRB, 0.22315E+00_JPRB ,&
&  0.22300E+00_JPRB, 0.22533E+00_JPRB, 0.22702E+00_JPRB, 0.22920E+00_JPRB, 0.22931E+00_JPRB, 0.22950E+00_JPRB, 0.22795E+00_JPRB ,&
&  0.22630E+00_JPRB, 0.22565E+00_JPRB, 0.22590E+00_JPRB, 0.22745E+00_JPRB, 0.23009E+00_JPRB, 0.23172E+00_JPRB, 0.23375E+00_JPRB ,&
&  0.23650E+00_JPRB, 0.23558E+00_JPRB, 0.23628E+00_JPRB, 0.23562E+00_JPRB, 0.23612E+00_JPRB, 0.23728E+00_JPRB, 0.23792E+00_JPRB ,&
&  0.23917E+00_JPRB, 0.24035E+00_JPRB, 0.24048E+00_JPRB, 0.23976E+00_JPRB, 0.23541E+00_JPRB, 0.23309E+00_JPRB, 0.22848E+00_JPRB ,&
&  0.22288E+00_JPRB, 0.21779E+00_JPRB, 0.21173E+00_JPRB, 0.20745E+00_JPRB, 0.20231E+00_JPRB, 0.19928E+00_JPRB, 0.19693E+00_JPRB ,&
&  0.19473E+00_JPRB, 0.19391E+00_JPRB, 0.19272E+00_JPRB, 0.19141E+00_JPRB, 0.19108E+00_JPRB, 0.18976E+00_JPRB, 0.18967E+00_JPRB ,&
&  0.19001E+00_JPRB, 0.19033E+00_JPRB, 0.19140E+00_JPRB, 0.19186E+00_JPRB, 0.19268E+00_JPRB, 0.19342E+00_JPRB, 0.19393E+00_JPRB ,&
&  0.19508E+00_JPRB, 0.19665E+00_JPRB, 0.19665E+00_JPRB, 0.19706E+00_JPRB, 0.19623E+00_JPRB, 0.19430E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19000E+00_JPRB, 0.18827E+00_JPRB, 0.18775E+00_JPRB, 0.18697E+00_JPRB, 0.18711E+00_JPRB, 0.18751E+00_JPRB, 0.18765E+00_JPRB ,&
&  0.18833E+00_JPRB, 0.18893E+00_JPRB, 0.18983E+00_JPRB, 0.19186E+00_JPRB, 0.19324E+00_JPRB, 0.19505E+00_JPRB, 0.19697E+00_JPRB ,&
&  0.19849E+00_JPRB, 0.20129E+00_JPRB, 0.20376E+00_JPRB, 0.20712E+00_JPRB, 0.21494E+00_JPRB, 0.22152E+00_JPRB, 0.23330E+00_JPRB ,&
&  0.23784E+00_JPRB, 0.24778E+00_JPRB, 0.24499E+00_JPRB, 0.24582E+00_JPRB, 0.24694E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.24116E+00_JPRB, 0.24203E+00_JPRB, 0.24188E+00_JPRB, 0.24286E+00_JPRB, 0.24361E+00_JPRB, 0.24468E+00_JPRB, 0.24465E+00_JPRB ,&
&  0.24421E+00_JPRB, 0.24024E+00_JPRB, 0.23807E+00_JPRB, 0.23467E+00_JPRB, 0.23152E+00_JPRB, 0.22813E+00_JPRB, 0.22539E+00_JPRB ,&
&  0.22363E+00_JPRB, 0.22228E+00_JPRB, 0.22105E+00_JPRB, 0.21932E+00_JPRB, 0.21715E+00_JPRB, 0.21495E+00_JPRB, 0.21295E+00_JPRB ,&
&  0.21113E+00_JPRB, 0.21126E+00_JPRB, 0.21263E+00_JPRB, 0.21491E+00_JPRB, 0.21911E+00_JPRB, 0.22215E+00_JPRB, 0.22608E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.22818E+00_JPRB, 0.22841E+00_JPRB, 0.22886E+00_JPRB, 0.22874E+00_JPRB, 0.22871E+00_JPRB, 0.22627E+00_JPRB ,&
&  0.23070E+00_JPRB, 0.23099E+00_JPRB, 0.23481E+00_JPRB, 0.23370E+00_JPRB, 0.22880E+00_JPRB, 0.22525E+00_JPRB, 0.22015E+00_JPRB ,&
&  0.21664E+00_JPRB, 0.21193E+00_JPRB, 0.20875E+00_JPRB, 0.20577E+00_JPRB, 0.20211E+00_JPRB, 0.20062E+00_JPRB, 0.19808E+00_JPRB ,&
&  0.19657E+00_JPRB, 0.19625E+00_JPRB, 0.19520E+00_JPRB, 0.19517E+00_JPRB, 0.19425E+00_JPRB, 0.19295E+00_JPRB, 0.19212E+00_JPRB ,&
&  0.19140E+00_JPRB, 0.19064E+00_JPRB, 0.19040E+00_JPRB, 0.19011E+00_JPRB, 0.19025E+00_JPRB, 0.19025E+00_JPRB, 0.19028E+00_JPRB ,&
&  0.19047E+00_JPRB, 0.19064E+00_JPRB, 0.19019E+00_JPRB, 0.19008E+00_JPRB, 0.18912E+00_JPRB, 0.18826E+00_JPRB, 0.18693E+00_JPRB ,&
&  0.18608E+00_JPRB, 0.18556E+00_JPRB, 0.18472E+00_JPRB, 0.18416E+00_JPRB, 0.18395E+00_JPRB, 0.18444E+00_JPRB, 0.18508E+00_JPRB ,&
&  0.18582E+00_JPRB, 0.18725E+00_JPRB, 0.18842E+00_JPRB, 0.18999E+00_JPRB, 0.19172E+00_JPRB, 0.19307E+00_JPRB, 0.19503E+00_JPRB ,&
&  0.19696E+00_JPRB, 0.19896E+00_JPRB, 0.20239E+00_JPRB, 0.20615E+00_JPRB, 0.21111E+00_JPRB, 0.21627E+00_JPRB, 0.21867E+00_JPRB ,&
&  0.22128E+00_JPRB, 0.22760E+00_JPRB, 0.23027E+00_JPRB, 0.23891E+00_JPRB, 0.24028E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.23092E+00_JPRB, 0.23259E+00_JPRB, 0.23422E+00_JPRB, 0.23577E+00_JPRB, 0.23761E+00_JPRB, 0.23836E+00_JPRB, 0.23837E+00_JPRB ,&
&  0.23597E+00_JPRB, 0.23550E+00_JPRB, 0.23398E+00_JPRB, 0.23200E+00_JPRB, 0.23039E+00_JPRB, 0.22706E+00_JPRB, 0.22410E+00_JPRB ,&
&  0.22117E+00_JPRB, 0.21836E+00_JPRB, 0.21592E+00_JPRB, 0.21254E+00_JPRB, 0.20932E+00_JPRB, 0.20611E+00_JPRB, 0.20534E+00_JPRB ,&
&  0.20511E+00_JPRB, 0.20579E+00_JPRB, 0.20775E+00_JPRB, 0.20922E+00_JPRB, 0.21138E+00_JPRB, 0.21367E+00_JPRB, 0.21709E+00_JPRB ,&
&  0.22025E+00_JPRB, 0.22255E+00_JPRB, 0.22283E+00_JPRB, 0.22157E+00_JPRB, 0.22068E+00_JPRB, 0.22053E+00_JPRB, 0.22027E+00_JPRB ,&
&  0.21967E+00_JPRB, 0.21949E+00_JPRB, 0.21905E+00_JPRB, 0.21872E+00_JPRB, 0.21736E+00_JPRB, 0.21476E+00_JPRB, 0.21209E+00_JPRB ,&
&  0.20957E+00_JPRB, 0.20732E+00_JPRB, 0.20514E+00_JPRB, 0.20321E+00_JPRB, 0.20208E+00_JPRB, 0.20092E+00_JPRB, 0.19943E+00_JPRB ,&
&  0.19857E+00_JPRB, 0.19766E+00_JPRB, 0.19663E+00_JPRB, 0.19587E+00_JPRB, 0.19506E+00_JPRB, 0.19448E+00_JPRB, 0.19383E+00_JPRB ,&
&  0.19335E+00_JPRB, 0.19258E+00_JPRB, 0.19193E+00_JPRB, 0.19121E+00_JPRB, 0.19090E+00_JPRB, 0.19007E+00_JPRB, 0.18927E+00_JPRB ,&
&  0.18826E+00_JPRB, 0.18764E+00_JPRB, 0.18647E+00_JPRB, 0.18578E+00_JPRB, 0.18496E+00_JPRB, 0.18419E+00_JPRB, 0.18363E+00_JPRB ,&
&  0.18361E+00_JPRB, 0.18351E+00_JPRB, 0.18290E+00_JPRB, 0.18269E+00_JPRB, 0.18264E+00_JPRB, 0.18306E+00_JPRB, 0.18447E+00_JPRB ,&
&  0.18538E+00_JPRB, 0.18637E+00_JPRB, 0.18732E+00_JPRB, 0.18804E+00_JPRB, 0.18930E+00_JPRB, 0.19050E+00_JPRB, 0.19174E+00_JPRB ,&
&  0.19321E+00_JPRB, 0.19445E+00_JPRB, 0.19590E+00_JPRB, 0.19759E+00_JPRB, 0.20188E+00_JPRB, 0.20467E+00_JPRB, 0.20751E+00_JPRB ,&
&  0.20935E+00_JPRB, 0.20739E+00_JPRB, 0.22204E+00_JPRB, 0.22479E+00_JPRB, 0.22691E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.20571E+00_JPRB, 0.20865E+00_JPRB, 0.21036E+00_JPRB, 0.21164E+00_JPRB, 0.20434E+00_JPRB, 0.21591E+00_JPRB, 0.21257E+00_JPRB ,&
&  0.21210E+00_JPRB, 0.21133E+00_JPRB, 0.21336E+00_JPRB, 0.21364E+00_JPRB, 0.21316E+00_JPRB, 0.21236E+00_JPRB, 0.21127E+00_JPRB ,&
&  0.21020E+00_JPRB, 0.20880E+00_JPRB, 0.20797E+00_JPRB, 0.20704E+00_JPRB, 0.20631E+00_JPRB, 0.20557E+00_JPRB, 0.20507E+00_JPRB ,&
&  0.20514E+00_JPRB, 0.20607E+00_JPRB, 0.20768E+00_JPRB, 0.20699E+00_JPRB, 0.20695E+00_JPRB, 0.20733E+00_JPRB, 0.20794E+00_JPRB ,&
&  0.20819E+00_JPRB, 0.20826E+00_JPRB, 0.20793E+00_JPRB, 0.20676E+00_JPRB, 0.20689E+00_JPRB, 0.20707E+00_JPRB, 0.20733E+00_JPRB ,&
&  0.20739E+00_JPRB, 0.20720E+00_JPRB, 0.20635E+00_JPRB, 0.20556E+00_JPRB, 0.20474E+00_JPRB, 0.20373E+00_JPRB, 0.20249E+00_JPRB ,&
&  0.20146E+00_JPRB, 0.20067E+00_JPRB, 0.19970E+00_JPRB, 0.19872E+00_JPRB, 0.19787E+00_JPRB, 0.19704E+00_JPRB, 0.19624E+00_JPRB ,&
&  0.19573E+00_JPRB, 0.19535E+00_JPRB, 0.19494E+00_JPRB, 0.19433E+00_JPRB, 0.19366E+00_JPRB, 0.19302E+00_JPRB, 0.19244E+00_JPRB ,&
&  0.19207E+00_JPRB, 0.19144E+00_JPRB, 0.19082E+00_JPRB, 0.19014E+00_JPRB, 0.18962E+00_JPRB, 0.18876E+00_JPRB, 0.18802E+00_JPRB ,&
&  0.18729E+00_JPRB, 0.18662E+00_JPRB, 0.18611E+00_JPRB, 0.18572E+00_JPRB, 0.18536E+00_JPRB, 0.18508E+00_JPRB, 0.18487E+00_JPRB ,&
&  0.18470E+00_JPRB, 0.18455E+00_JPRB, 0.18449E+00_JPRB, 0.18437E+00_JPRB, 0.18421E+00_JPRB, 0.18415E+00_JPRB, 0.18442E+00_JPRB ,&
&  0.18466E+00_JPRB, 0.18482E+00_JPRB, 0.18490E+00_JPRB, 0.18454E+00_JPRB, 0.18496E+00_JPRB, 0.18531E+00_JPRB, 0.18576E+00_JPRB ,&
&  0.18615E+00_JPRB, 0.18700E+00_JPRB, 0.18802E+00_JPRB, 0.18926E+00_JPRB, 0.19168E+00_JPRB, 0.19379E+00_JPRB, 0.19603E+00_JPRB ,&
&  0.19854E+00_JPRB, 0.20221E+00_JPRB, 0.20365E+00_JPRB, 0.20454E+00_JPRB, 0.20518E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.19453E+00_JPRB, 0.19442E+00_JPRB, 0.19416E+00_JPRB, 0.19326E+00_JPRB, 0.19068E+00_JPRB, 0.19020E+00_JPRB, 0.19016E+00_JPRB ,&
&  0.19034E+00_JPRB, 0.19046E+00_JPRB, 0.19144E+00_JPRB, 0.19249E+00_JPRB, 0.19354E+00_JPRB, 0.19435E+00_JPRB, 0.19505E+00_JPRB ,&
&  0.19543E+00_JPRB, 0.19582E+00_JPRB, 0.19456E+00_JPRB, 0.19396E+00_JPRB, 0.19378E+00_JPRB, 0.19377E+00_JPRB, 0.19377E+00_JPRB ,&
&  0.19371E+00_JPRB, 0.19370E+00_JPRB, 0.19358E+00_JPRB, 0.19382E+00_JPRB, 0.19417E+00_JPRB, 0.19447E+00_JPRB, 0.19481E+00_JPRB ,&
&  0.19507E+00_JPRB, 0.19531E+00_JPRB, 0.19546E+00_JPRB, 0.19558E+00_JPRB, 0.19596E+00_JPRB, 0.19617E+00_JPRB, 0.19612E+00_JPRB ,&
&  0.19596E+00_JPRB, 0.19561E+00_JPRB, 0.19522E+00_JPRB, 0.19480E+00_JPRB, 0.19451E+00_JPRB, 0.19422E+00_JPRB, 0.19394E+00_JPRB ,&
&  0.19364E+00_JPRB, 0.19333E+00_JPRB, 0.19299E+00_JPRB, 0.19261E+00_JPRB, 0.19224E+00_JPRB, 0.19197E+00_JPRB, 0.19168E+00_JPRB ,&
&  0.19143E+00_JPRB, 0.19119E+00_JPRB, 0.19093E+00_JPRB, 0.19059E+00_JPRB, 0.19024E+00_JPRB, 0.18987E+00_JPRB, 0.18948E+00_JPRB ,&
&  0.18897E+00_JPRB, 0.18861E+00_JPRB, 0.18825E+00_JPRB, 0.18793E+00_JPRB, 0.18766E+00_JPRB, 0.18736E+00_JPRB, 0.18711E+00_JPRB ,&
&  0.18683E+00_JPRB, 0.18657E+00_JPRB, 0.18627E+00_JPRB, 0.18599E+00_JPRB, 0.18573E+00_JPRB, 0.18550E+00_JPRB, 0.18530E+00_JPRB ,&
&  0.18511E+00_JPRB, 0.18492E+00_JPRB, 0.18475E+00_JPRB, 0.18474E+00_JPRB, 0.18470E+00_JPRB, 0.18471E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18474E+00_JPRB, 0.18474E+00_JPRB, 0.18473E+00_JPRB, 0.18460E+00_JPRB, 0.18474E+00_JPRB, 0.18496E+00_JPRB, 0.18529E+00_JPRB ,&
&  0.18569E+00_JPRB, 0.18621E+00_JPRB, 0.18676E+00_JPRB, 0.18734E+00_JPRB, 0.18803E+00_JPRB, 0.18860E+00_JPRB, 0.18923E+00_JPRB ,&
&  0.18994E+00_JPRB, 0.19067E+00_JPRB, 0.19120E+00_JPRB, 0.19182E+00_JPRB, 0.19277E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB, 0.18815E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB, 0.59843E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.60840E+00_JPRB, 0.60754E+00_JPRB, 0.60684E+00_JPRB, 0.60649E+00_JPRB, 0.60638E+00_JPRB, 0.60624E+00_JPRB, 0.60527E+00_JPRB ,&
&  0.60409E+00_JPRB, 0.60257E+00_JPRB, 0.60094E+00_JPRB, 0.59898E+00_JPRB, 0.59658E+00_JPRB, 0.59373E+00_JPRB, 0.59112E+00_JPRB ,&
&  0.58869E+00_JPRB, 0.58668E+00_JPRB, 0.58457E+00_JPRB, 0.58308E+00_JPRB, 0.58136E+00_JPRB, 0.57957E+00_JPRB, 0.57771E+00_JPRB ,&
&  0.57622E+00_JPRB, 0.57483E+00_JPRB, 0.57351E+00_JPRB, 0.57233E+00_JPRB, 0.57177E+00_JPRB, 0.57108E+00_JPRB, 0.57036E+00_JPRB ,&
&  0.56972E+00_JPRB, 0.56932E+00_JPRB, 0.56898E+00_JPRB, 0.56862E+00_JPRB, 0.56835E+00_JPRB, 0.56845E+00_JPRB, 0.56856E+00_JPRB ,&
&  0.56883E+00_JPRB, 0.56965E+00_JPRB, 0.57092E+00_JPRB, 0.57209E+00_JPRB, 0.57338E+00_JPRB, 0.57521E+00_JPRB, 0.57742E+00_JPRB ,&
&  0.57935E+00_JPRB, 0.58116E+00_JPRB, 0.58327E+00_JPRB, 0.58568E+00_JPRB, 0.58794E+00_JPRB, 0.59037E+00_JPRB, 0.59268E+00_JPRB ,&
&  0.59500E+00_JPRB, 0.59689E+00_JPRB, 0.59880E+00_JPRB, 0.59968E+00_JPRB, 0.60097E+00_JPRB, 0.60204E+00_JPRB, 0.60330E+00_JPRB ,&
&  0.60360E+00_JPRB, 0.60377E+00_JPRB, 0.60404E+00_JPRB, 0.60455E+00_JPRB, 0.60485E+00_JPRB, 0.60527E+00_JPRB, 0.60578E+00_JPRB ,&
&  0.60654E+00_JPRB, 0.60694E+00_JPRB, 0.60758E+00_JPRB, 0.60804E+00_JPRB, 0.60865E+00_JPRB, 0.60879E+00_JPRB, 0.60918E+00_JPRB ,&
&  0.60961E+00_JPRB, 0.61029E+00_JPRB, 0.61042E+00_JPRB, 0.61062E+00_JPRB, 0.61074E+00_JPRB, 0.61099E+00_JPRB, 0.61123E+00_JPRB ,&
&  0.61156E+00_JPRB, 0.61179E+00_JPRB, 0.61202E+00_JPRB, 0.61188E+00_JPRB, 0.61143E+00_JPRB, 0.61103E+00_JPRB, 0.61073E+00_JPRB ,&
&  0.61064E+00_JPRB, 0.61052E+00_JPRB, 0.61073E+00_JPRB, 0.61111E+00_JPRB, 0.61183E+00_JPRB, 0.61194E+00_JPRB, 0.61202E+00_JPRB ,&
&  0.61196E+00_JPRB, 0.61184E+00_JPRB, 0.61117E+00_JPRB, 0.61030E+00_JPRB, 0.60932E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.61109E+00_JPRB, 0.60940E+00_JPRB, 0.60828E+00_JPRB, 0.60793E+00_JPRB, 0.60792E+00_JPRB, 0.60776E+00_JPRB, 0.60749E+00_JPRB ,&
&  0.60600E+00_JPRB, 0.60263E+00_JPRB, 0.59869E+00_JPRB, 0.59434E+00_JPRB, 0.58901E+00_JPRB, 0.58411E+00_JPRB, 0.58080E+00_JPRB ,&
&  0.57732E+00_JPRB, 0.57384E+00_JPRB, 0.57045E+00_JPRB, 0.56765E+00_JPRB, 0.56448E+00_JPRB, 0.56113E+00_JPRB, 0.55788E+00_JPRB ,&
&  0.55526E+00_JPRB, 0.55226E+00_JPRB, 0.54891E+00_JPRB, 0.54473E+00_JPRB, 0.54189E+00_JPRB, 0.53889E+00_JPRB, 0.53624E+00_JPRB ,&
&  0.53287E+00_JPRB, 0.53124E+00_JPRB, 0.53032E+00_JPRB, 0.53029E+00_JPRB, 0.53281E+00_JPRB, 0.53479E+00_JPRB, 0.53503E+00_JPRB ,&
&  0.53432E+00_JPRB, 0.53125E+00_JPRB, 0.53161E+00_JPRB, 0.53256E+00_JPRB, 0.53452E+00_JPRB, 0.53865E+00_JPRB, 0.54404E+00_JPRB ,&
&  0.55173E+00_JPRB, 0.56101E+00_JPRB, 0.57051E+00_JPRB, 0.57785E+00_JPRB, 0.58505E+00_JPRB, 0.59170E+00_JPRB, 0.59656E+00_JPRB ,&
&  0.60274E+00_JPRB, 0.60802E+00_JPRB, 0.61340E+00_JPRB, 0.61817E+00_JPRB, 0.62360E+00_JPRB, 0.62787E+00_JPRB, 0.63105E+00_JPRB ,&
&  0.63186E+00_JPRB, 0.63153E+00_JPRB, 0.63122E+00_JPRB, 0.63112E+00_JPRB, 0.62989E+00_JPRB, 0.62912E+00_JPRB, 0.62912E+00_JPRB ,&
&  0.62968E+00_JPRB, 0.62980E+00_JPRB, 0.63093E+00_JPRB, 0.63226E+00_JPRB, 0.63376E+00_JPRB, 0.63343E+00_JPRB, 0.63363E+00_JPRB ,&
&  0.63357E+00_JPRB, 0.63373E+00_JPRB, 0.63271E+00_JPRB, 0.63067E+00_JPRB, 0.62900E+00_JPRB, 0.62834E+00_JPRB, 0.62710E+00_JPRB ,&
&  0.62588E+00_JPRB, 0.62499E+00_JPRB, 0.62471E+00_JPRB, 0.62394E+00_JPRB, 0.62253E+00_JPRB, 0.62125E+00_JPRB, 0.62075E+00_JPRB ,&
&  0.62017E+00_JPRB, 0.61923E+00_JPRB, 0.61938E+00_JPRB, 0.61969E+00_JPRB, 0.62163E+00_JPRB, 0.62194E+00_JPRB, 0.62196E+00_JPRB ,&
&  0.62105E+00_JPRB, 0.62041E+00_JPRB, 0.61865E+00_JPRB, 0.61615E+00_JPRB, 0.61322E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.62228E+00_JPRB, 0.62149E+00_JPRB, 0.62233E+00_JPRB, 0.62133E+00_JPRB, 0.61959E+00_JPRB, 0.61652E+00_JPRB, 0.61471E+00_JPRB ,&
&  0.61115E+00_JPRB, 0.60533E+00_JPRB, 0.59639E+00_JPRB, 0.58865E+00_JPRB, 0.58097E+00_JPRB, 0.57598E+00_JPRB, 0.57124E+00_JPRB ,&
&  0.56650E+00_JPRB, 0.56257E+00_JPRB, 0.55836E+00_JPRB, 0.55322E+00_JPRB, 0.54686E+00_JPRB, 0.53933E+00_JPRB, 0.52950E+00_JPRB ,&
&  0.52209E+00_JPRB, 0.51499E+00_JPRB, 0.50891E+00_JPRB, 0.50480E+00_JPRB, 0.50379E+00_JPRB, 0.48640E+00_JPRB, 0.48334E+00_JPRB ,&
&  0.49127E+00_JPRB, 0.48187E+00_JPRB, 0.47303E+00_JPRB, 0.45871E+00_JPRB, 0.45532E+00_JPRB, 0.46328E+00_JPRB, 0.46917E+00_JPRB ,&
&  0.46263E+00_JPRB, 0.48615E+00_JPRB, 0.48813E+00_JPRB, 0.48906E+00_JPRB, 0.48732E+00_JPRB, 0.48340E+00_JPRB, 0.49173E+00_JPRB ,&
&  0.51157E+00_JPRB, 0.52013E+00_JPRB, 0.53866E+00_JPRB, 0.55874E+00_JPRB, 0.57213E+00_JPRB, 0.58648E+00_JPRB, 0.60116E+00_JPRB ,&
&  0.61555E+00_JPRB, 0.62645E+00_JPRB, 0.63511E+00_JPRB, 0.64324E+00_JPRB, 0.65489E+00_JPRB, 0.66641E+00_JPRB, 0.67645E+00_JPRB ,&
&  0.68350E+00_JPRB, 0.68780E+00_JPRB, 0.69295E+00_JPRB, 0.69793E+00_JPRB, 0.70494E+00_JPRB, 0.71103E+00_JPRB, 0.71724E+00_JPRB ,&
&  0.72210E+00_JPRB, 0.72601E+00_JPRB, 0.72837E+00_JPRB, 0.72915E+00_JPRB, 0.72909E+00_JPRB, 0.72620E+00_JPRB, 0.72525E+00_JPRB ,&
&  0.72042E+00_JPRB, 0.71508E+00_JPRB, 0.70817E+00_JPRB, 0.69931E+00_JPRB, 0.69275E+00_JPRB, 0.68769E+00_JPRB, 0.68191E+00_JPRB ,&
&  0.67592E+00_JPRB, 0.66994E+00_JPRB, 0.66464E+00_JPRB, 0.65803E+00_JPRB, 0.65186E+00_JPRB, 0.64552E+00_JPRB, 0.64129E+00_JPRB ,&
&  0.63840E+00_JPRB, 0.63619E+00_JPRB, 0.63577E+00_JPRB, 0.63502E+00_JPRB, 0.63602E+00_JPRB, 0.63596E+00_JPRB, 0.63705E+00_JPRB ,&
&  0.63686E+00_JPRB, 0.63395E+00_JPRB, 0.62936E+00_JPRB, 0.62573E+00_JPRB, 0.62275E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.62801E+00_JPRB, 0.62885E+00_JPRB, 0.63286E+00_JPRB, 0.63143E+00_JPRB, 0.62886E+00_JPRB, 0.62193E+00_JPRB, 0.61852E+00_JPRB ,&
&  0.61140E+00_JPRB, 0.60453E+00_JPRB, 0.59031E+00_JPRB, 0.58117E+00_JPRB, 0.57270E+00_JPRB, 0.56761E+00_JPRB, 0.56339E+00_JPRB ,&
&  0.56007E+00_JPRB, 0.55577E+00_JPRB, 0.55047E+00_JPRB, 0.54259E+00_JPRB, 0.53272E+00_JPRB, 0.52143E+00_JPRB, 0.50572E+00_JPRB ,&
&  0.49206E+00_JPRB, 0.48237E+00_JPRB, 0.46913E+00_JPRB, 0.47348E+00_JPRB, 0.46748E+00_JPRB, 0.46294E+00_JPRB, 0.46019E+00_JPRB ,&
&  0.45264E+00_JPRB, 0.44235E+00_JPRB, 0.42808E+00_JPRB, 0.41930E+00_JPRB, 0.41254E+00_JPRB, 0.40497E+00_JPRB, 0.41438E+00_JPRB ,&
&  0.42151E+00_JPRB, 0.44231E+00_JPRB, 0.45087E+00_JPRB, 0.45310E+00_JPRB, 0.45635E+00_JPRB, 0.45654E+00_JPRB, 0.46376E+00_JPRB ,&
&  0.47341E+00_JPRB, 0.48516E+00_JPRB, 0.52358E+00_JPRB, 0.54518E+00_JPRB, 0.55816E+00_JPRB, 0.57304E+00_JPRB, 0.58496E+00_JPRB ,&
&  0.59985E+00_JPRB, 0.61072E+00_JPRB, 0.62030E+00_JPRB, 0.63239E+00_JPRB, 0.64996E+00_JPRB, 0.67038E+00_JPRB, 0.68833E+00_JPRB ,&
&  0.70452E+00_JPRB, 0.71450E+00_JPRB, 0.72739E+00_JPRB, 0.74201E+00_JPRB, 0.76218E+00_JPRB, 0.78419E+00_JPRB, 0.80759E+00_JPRB ,&
&  0.83182E+00_JPRB, 0.85646E+00_JPRB, 0.87623E+00_JPRB, 0.89374E+00_JPRB, 0.90242E+00_JPRB, 0.90335E+00_JPRB, 0.89984E+00_JPRB ,&
&  0.88551E+00_JPRB, 0.86973E+00_JPRB, 0.84869E+00_JPRB, 0.81696E+00_JPRB, 0.79321E+00_JPRB, 0.78083E+00_JPRB, 0.75750E+00_JPRB ,&
&  0.73822E+00_JPRB, 0.72039E+00_JPRB, 0.70432E+00_JPRB, 0.68663E+00_JPRB, 0.67073E+00_JPRB, 0.65577E+00_JPRB, 0.64561E+00_JPRB ,&
&  0.63769E+00_JPRB, 0.63292E+00_JPRB, 0.63040E+00_JPRB, 0.62936E+00_JPRB, 0.63091E+00_JPRB, 0.63210E+00_JPRB, 0.63619E+00_JPRB ,&
&  0.63770E+00_JPRB, 0.63642E+00_JPRB, 0.63197E+00_JPRB, 0.62915E+00_JPRB, 0.62688E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.63630E+00_JPRB, 0.63599E+00_JPRB, 0.63433E+00_JPRB, 0.62639E+00_JPRB, 0.62103E+00_JPRB, 0.61442E+00_JPRB, 0.61190E+00_JPRB ,&
&  0.60424E+00_JPRB, 0.59757E+00_JPRB, 0.58322E+00_JPRB, 0.57449E+00_JPRB, 0.56465E+00_JPRB, 0.55926E+00_JPRB, 0.55404E+00_JPRB ,&
&  0.55060E+00_JPRB, 0.54602E+00_JPRB, 0.54293E+00_JPRB, 0.53541E+00_JPRB, 0.52816E+00_JPRB, 0.51655E+00_JPRB, 0.50428E+00_JPRB ,&
&  0.49098E+00_JPRB, 0.47771E+00_JPRB, 0.46644E+00_JPRB, 0.46817E+00_JPRB, 0.46248E+00_JPRB, 0.45841E+00_JPRB, 0.45522E+00_JPRB ,&
&  0.45703E+00_JPRB, 0.42316E+00_JPRB, 0.42741E+00_JPRB, 0.41576E+00_JPRB, 0.40270E+00_JPRB, 0.38694E+00_JPRB, 0.37472E+00_JPRB ,&
&  0.37727E+00_JPRB, 0.37926E+00_JPRB, 0.38812E+00_JPRB, 0.40196E+00_JPRB, 0.40814E+00_JPRB, 0.42041E+00_JPRB, 0.42968E+00_JPRB ,&
&  0.42853E+00_JPRB, 0.49039E+00_JPRB, 0.52081E+00_JPRB, 0.53791E+00_JPRB, 0.55574E+00_JPRB, 0.57215E+00_JPRB, 0.58597E+00_JPRB ,&
&  0.60603E+00_JPRB, 0.62898E+00_JPRB, 0.65097E+00_JPRB, 0.67229E+00_JPRB, 0.70427E+00_JPRB, 0.72673E+00_JPRB, 0.74670E+00_JPRB ,&
&  0.76379E+00_JPRB, 0.77074E+00_JPRB, 0.78386E+00_JPRB, 0.80326E+00_JPRB, 0.82935E+00_JPRB, 0.85850E+00_JPRB, 0.89154E+00_JPRB ,&
&  0.92836E+00_JPRB, 0.96385E+00_JPRB, 0.10008E+01_JPRB, 0.10323E+01_JPRB, 0.10560E+01_JPRB, 0.10703E+01_JPRB, 0.10849E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.10517E+01_JPRB, 0.10210E+01_JPRB, 0.97985E+00_JPRB, 0.93016E+00_JPRB, 0.87514E+00_JPRB, 0.81997E+00_JPRB ,&
&  0.79155E+00_JPRB, 0.76101E+00_JPRB, 0.73251E+00_JPRB, 0.70528E+00_JPRB, 0.68105E+00_JPRB, 0.66085E+00_JPRB, 0.64888E+00_JPRB ,&
&  0.63826E+00_JPRB, 0.63132E+00_JPRB, 0.62713E+00_JPRB, 0.62413E+00_JPRB, 0.62312E+00_JPRB, 0.62068E+00_JPRB, 0.62158E+00_JPRB ,&
&  0.62387E+00_JPRB, 0.62827E+00_JPRB, 0.62970E+00_JPRB, 0.63183E+00_JPRB, 0.63381E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.64602E+00_JPRB, 0.64895E+00_JPRB, 0.64679E+00_JPRB, 0.64181E+00_JPRB, 0.63760E+00_JPRB, 0.62740E+00_JPRB, 0.62024E+00_JPRB ,&
&  0.61448E+00_JPRB, 0.60886E+00_JPRB, 0.59915E+00_JPRB, 0.58902E+00_JPRB, 0.57606E+00_JPRB, 0.56667E+00_JPRB, 0.55648E+00_JPRB ,&
&  0.54844E+00_JPRB, 0.53889E+00_JPRB, 0.52846E+00_JPRB, 0.51878E+00_JPRB, 0.51131E+00_JPRB, 0.50208E+00_JPRB, 0.49560E+00_JPRB ,&
&  0.48807E+00_JPRB, 0.47807E+00_JPRB, 0.47084E+00_JPRB, 0.47083E+00_JPRB, 0.46453E+00_JPRB, 0.45992E+00_JPRB, 0.46166E+00_JPRB ,&
&  0.46731E+00_JPRB, 0.46300E+00_JPRB, 0.45765E+00_JPRB, 0.44352E+00_JPRB, 0.42046E+00_JPRB, 0.36796E+00_JPRB, 0.37527E+00_JPRB ,&
&  0.37100E+00_JPRB, 0.36365E+00_JPRB, 0.36637E+00_JPRB, 0.37344E+00_JPRB, 0.38107E+00_JPRB, 0.40230E+00_JPRB, 0.40510E+00_JPRB ,&
&  0.39956E+00_JPRB, 0.48639E+00_JPRB, 0.50579E+00_JPRB, 0.52905E+00_JPRB, 0.55258E+00_JPRB, 0.57583E+00_JPRB, 0.60018E+00_JPRB ,&
&  0.61551E+00_JPRB, 0.63970E+00_JPRB, 0.66159E+00_JPRB, 0.69635E+00_JPRB, 0.72743E+00_JPRB, 0.76382E+00_JPRB, 0.79522E+00_JPRB ,&
&  0.81162E+00_JPRB, 0.82100E+00_JPRB, 0.83464E+00_JPRB, 0.84437E+00_JPRB, 0.85878E+00_JPRB, 0.88656E+00_JPRB, 0.92605E+00_JPRB ,&
&  0.98048E+00_JPRB, 0.10096E+01_JPRB, 0.10796E+01_JPRB, 0.11303E+01_JPRB, 0.11548E+01_JPRB, 0.11921E+01_JPRB, 0.12361E+01_JPRB ,&
&  0.12251E+01_JPRB, 0.12191E+01_JPRB, 0.11961E+01_JPRB, 0.11093E+01_JPRB, 0.10172E+01_JPRB, 0.95501E+00_JPRB, 0.89744E+00_JPRB ,&
&  0.85381E+00_JPRB, 0.81022E+00_JPRB, 0.76341E+00_JPRB, 0.72762E+00_JPRB, 0.69986E+00_JPRB, 0.67952E+00_JPRB, 0.66850E+00_JPRB ,&
&  0.65599E+00_JPRB, 0.64528E+00_JPRB, 0.63564E+00_JPRB, 0.63206E+00_JPRB, 0.62344E+00_JPRB, 0.61619E+00_JPRB, 0.60880E+00_JPRB ,&
&  0.60840E+00_JPRB, 0.61075E+00_JPRB, 0.61964E+00_JPRB, 0.62916E+00_JPRB, 0.63572E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.65022E+00_JPRB, 0.66639E+00_JPRB, 0.67531E+00_JPRB, 0.67911E+00_JPRB, 0.68355E+00_JPRB, 0.66854E+00_JPRB, 0.65433E+00_JPRB ,&
&  0.64313E+00_JPRB, 0.63118E+00_JPRB, 0.62000E+00_JPRB, 0.60902E+00_JPRB, 0.59773E+00_JPRB, 0.58929E+00_JPRB, 0.58157E+00_JPRB ,&
&  0.57487E+00_JPRB, 0.54098E+00_JPRB, 0.52399E+00_JPRB, 0.51246E+00_JPRB, 0.50192E+00_JPRB, 0.49682E+00_JPRB, 0.49481E+00_JPRB ,&
&  0.48842E+00_JPRB, 0.48420E+00_JPRB, 0.48285E+00_JPRB, 0.47844E+00_JPRB, 0.47112E+00_JPRB, 0.46182E+00_JPRB, 0.46161E+00_JPRB ,&
&  0.46998E+00_JPRB, 0.45974E+00_JPRB, 0.44947E+00_JPRB, 0.43808E+00_JPRB, 0.41818E+00_JPRB, 0.39567E+00_JPRB, 0.38397E+00_JPRB ,&
&  0.37872E+00_JPRB, 0.36075E+00_JPRB, 0.36200E+00_JPRB, 0.36470E+00_JPRB, 0.36616E+00_JPRB, 0.37607E+00_JPRB, 0.39674E+00_JPRB ,&
&  0.42058E+00_JPRB, 0.49134E+00_JPRB, 0.51884E+00_JPRB, 0.55410E+00_JPRB, 0.58601E+00_JPRB, 0.61263E+00_JPRB, 0.63681E+00_JPRB ,&
&  0.65442E+00_JPRB, 0.68013E+00_JPRB, 0.71009E+00_JPRB, 0.74808E+00_JPRB, 0.78480E+00_JPRB, 0.81182E+00_JPRB, 0.84422E+00_JPRB ,&
&  0.88573E+00_JPRB, 0.87423E+00_JPRB, 0.88383E+00_JPRB, 0.87726E+00_JPRB, 0.88576E+00_JPRB, 0.92091E+00_JPRB, 0.92077E+00_JPRB ,&
&  0.95775E+00_JPRB, 0.10278E+01_JPRB, 0.11386E+01_JPRB, 0.12524E+01_JPRB, 0.13054E+01_JPRB, 0.13660E+01_JPRB, 0.13944E+01_JPRB ,&
&  0.13310E+01_JPRB, 0.12948E+01_JPRB, 0.13313E+01_JPRB, 0.12282E+01_JPRB, 0.11170E+01_JPRB, 0.10238E+01_JPRB, 0.95945E+00_JPRB ,&
&  0.89433E+00_JPRB, 0.82963E+00_JPRB, 0.78817E+00_JPRB, 0.75621E+00_JPRB, 0.72420E+00_JPRB, 0.69714E+00_JPRB, 0.69304E+00_JPRB ,&
&  0.68947E+00_JPRB, 0.68461E+00_JPRB, 0.67614E+00_JPRB, 0.66580E+00_JPRB, 0.64566E+00_JPRB, 0.62384E+00_JPRB, 0.60474E+00_JPRB ,&
&  0.59967E+00_JPRB, 0.60067E+00_JPRB, 0.61119E+00_JPRB, 0.62154E+00_JPRB, 0.63417E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.64463E+00_JPRB, 0.65797E+00_JPRB, 0.66795E+00_JPRB, 0.67871E+00_JPRB, 0.69822E+00_JPRB, 0.66073E+00_JPRB, 0.63460E+00_JPRB ,&
&  0.62032E+00_JPRB, 0.62116E+00_JPRB, 0.62109E+00_JPRB, 0.61510E+00_JPRB, 0.60462E+00_JPRB, 0.59419E+00_JPRB, 0.58605E+00_JPRB ,&
&  0.57919E+00_JPRB, 0.56284E+00_JPRB, 0.53574E+00_JPRB, 0.49642E+00_JPRB, 0.48590E+00_JPRB, 0.48359E+00_JPRB, 0.48652E+00_JPRB ,&
&  0.48762E+00_JPRB, 0.48339E+00_JPRB, 0.48327E+00_JPRB, 0.47566E+00_JPRB, 0.46954E+00_JPRB, 0.45400E+00_JPRB, 0.44090E+00_JPRB ,&
&  0.44253E+00_JPRB, 0.44215E+00_JPRB, 0.45564E+00_JPRB, 0.44544E+00_JPRB, 0.43926E+00_JPRB, 0.43763E+00_JPRB, 0.38016E+00_JPRB ,&
&  0.36965E+00_JPRB, 0.35727E+00_JPRB, 0.35362E+00_JPRB, 0.35339E+00_JPRB, 0.36708E+00_JPRB, 0.38060E+00_JPRB, 0.40071E+00_JPRB ,&
&  0.42745E+00_JPRB, 0.50050E+00_JPRB, 0.52522E+00_JPRB, 0.55376E+00_JPRB, 0.58248E+00_JPRB, 0.61462E+00_JPRB, 0.65626E+00_JPRB ,&
&  0.69464E+00_JPRB, 0.73178E+00_JPRB, 0.76128E+00_JPRB, 0.79481E+00_JPRB, 0.82954E+00_JPRB, 0.80279E+00_JPRB, 0.85576E+00_JPRB ,&
&  0.11071E+01_JPRB, 0.10509E+01_JPRB, 0.99105E+00_JPRB, 0.95799E+00_JPRB, 0.95571E+00_JPRB, 0.95977E+00_JPRB, 0.96916E+00_JPRB ,&
&  0.10372E+01_JPRB, 0.11631E+01_JPRB, 0.13625E+01_JPRB, 0.15095E+01_JPRB, 0.15183E+01_JPRB, 0.15449E+01_JPRB, 0.15310E+01_JPRB ,&
&  0.14287E+01_JPRB, 0.13649E+01_JPRB, 0.14407E+01_JPRB, 0.11904E+01_JPRB, 0.10180E+01_JPRB, 0.93088E+00_JPRB, 0.89471E+00_JPRB ,&
&  0.85294E+00_JPRB, 0.80683E+00_JPRB, 0.78044E+00_JPRB, 0.74873E+00_JPRB, 0.70773E+00_JPRB, 0.68414E+00_JPRB, 0.68058E+00_JPRB ,&
&  0.67796E+00_JPRB, 0.67209E+00_JPRB, 0.66783E+00_JPRB, 0.66037E+00_JPRB, 0.64527E+00_JPRB, 0.62757E+00_JPRB, 0.62185E+00_JPRB ,&
&  0.62211E+00_JPRB, 0.62159E+00_JPRB, 0.62168E+00_JPRB, 0.62329E+00_JPRB, 0.63062E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.63234E+00_JPRB, 0.64679E+00_JPRB, 0.66164E+00_JPRB, 0.65824E+00_JPRB, 0.65475E+00_JPRB, 0.59943E+00_JPRB, 0.54064E+00_JPRB ,&
&  0.50040E+00_JPRB, 0.50422E+00_JPRB, 0.52222E+00_JPRB, 0.53678E+00_JPRB, 0.53667E+00_JPRB, 0.52612E+00_JPRB, 0.52344E+00_JPRB ,&
&  0.52935E+00_JPRB, 0.52741E+00_JPRB, 0.50717E+00_JPRB, 0.49537E+00_JPRB, 0.47538E+00_JPRB, 0.47269E+00_JPRB, 0.47525E+00_JPRB ,&
&  0.47845E+00_JPRB, 0.47716E+00_JPRB, 0.47558E+00_JPRB, 0.47238E+00_JPRB, 0.47103E+00_JPRB, 0.46861E+00_JPRB, 0.46214E+00_JPRB ,&
&  0.45040E+00_JPRB, 0.44580E+00_JPRB, 0.45182E+00_JPRB, 0.46736E+00_JPRB, 0.46450E+00_JPRB, 0.46282E+00_JPRB, 0.39715E+00_JPRB ,&
&  0.37109E+00_JPRB, 0.35427E+00_JPRB, 0.35460E+00_JPRB, 0.36691E+00_JPRB, 0.39607E+00_JPRB, 0.41676E+00_JPRB, 0.47497E+00_JPRB ,&
&  0.52501E+00_JPRB, 0.55226E+00_JPRB, 0.56589E+00_JPRB, 0.57726E+00_JPRB, 0.60477E+00_JPRB, 0.66184E+00_JPRB, 0.73142E+00_JPRB ,&
&  0.77557E+00_JPRB, 0.80514E+00_JPRB, 0.80663E+00_JPRB, 0.67676E+00_JPRB, 0.66990E+00_JPRB, 0.72611E+00_JPRB, 0.80622E+00_JPRB ,&
&  0.91456E+00_JPRB, 0.93718E+00_JPRB, 0.94476E+00_JPRB, 0.98890E+00_JPRB, 0.97952E+00_JPRB, 0.97075E+00_JPRB, 0.95611E+00_JPRB ,&
&  0.10563E+01_JPRB, 0.12232E+01_JPRB, 0.14915E+01_JPRB, 0.16363E+01_JPRB, 0.16012E+01_JPRB, 0.16265E+01_JPRB, 0.15723E+01_JPRB ,&
&  0.15136E+01_JPRB, 0.13588E+01_JPRB, 0.12230E+01_JPRB, 0.10551E+01_JPRB, 0.92486E+00_JPRB, 0.85266E+00_JPRB, 0.81205E+00_JPRB ,&
&  0.77354E+00_JPRB, 0.73721E+00_JPRB, 0.73204E+00_JPRB, 0.71229E+00_JPRB, 0.67191E+00_JPRB, 0.63583E+00_JPRB, 0.60818E+00_JPRB ,&
&  0.58929E+00_JPRB, 0.57941E+00_JPRB, 0.57805E+00_JPRB, 0.56740E+00_JPRB, 0.56939E+00_JPRB, 0.57951E+00_JPRB, 0.58479E+00_JPRB ,&
&  0.58459E+00_JPRB, 0.58370E+00_JPRB, 0.59052E+00_JPRB, 0.59976E+00_JPRB, 0.61124E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.68936E+00_JPRB, 0.66910E+00_JPRB, 0.63916E+00_JPRB, 0.63646E+00_JPRB, 0.61302E+00_JPRB, 0.59732E+00_JPRB, 0.54481E+00_JPRB ,&
&  0.47917E+00_JPRB, 0.45866E+00_JPRB, 0.47807E+00_JPRB, 0.48265E+00_JPRB, 0.46738E+00_JPRB, 0.46201E+00_JPRB, 0.46698E+00_JPRB ,&
&  0.47443E+00_JPRB, 0.49601E+00_JPRB, 0.49467E+00_JPRB, 0.48059E+00_JPRB, 0.47229E+00_JPRB, 0.46936E+00_JPRB, 0.47344E+00_JPRB ,&
&  0.47575E+00_JPRB, 0.47667E+00_JPRB, 0.47130E+00_JPRB, 0.46185E+00_JPRB, 0.46095E+00_JPRB, 0.46214E+00_JPRB, 0.46241E+00_JPRB ,&
&  0.46462E+00_JPRB, 0.46391E+00_JPRB, 0.47527E+00_JPRB, 0.48602E+00_JPRB, 0.47993E+00_JPRB, 0.46600E+00_JPRB, 0.41012E+00_JPRB ,&
&  0.37513E+00_JPRB, 0.36472E+00_JPRB, 0.39159E+00_JPRB, 0.46552E+00_JPRB, 0.50323E+00_JPRB, 0.54408E+00_JPRB, 0.57653E+00_JPRB ,&
&  0.54881E+00_JPRB, 0.55335E+00_JPRB, 0.58825E+00_JPRB, 0.70396E+00_JPRB, 0.74832E+00_JPRB, 0.80842E+00_JPRB, 0.85259E+00_JPRB ,&
&  0.88736E+00_JPRB, 0.91425E+00_JPRB, 0.92065E+00_JPRB, 0.77162E+00_JPRB, 0.77568E+00_JPRB, 0.94875E+00_JPRB, 0.88898E+00_JPRB ,&
&  0.95099E+00_JPRB, 0.10150E+01_JPRB, 0.10720E+01_JPRB, 0.10467E+01_JPRB, 0.97584E+00_JPRB, 0.97488E+00_JPRB, 0.10357E+01_JPRB ,&
&  0.11484E+01_JPRB, 0.13361E+01_JPRB, 0.17006E+01_JPRB, 0.19065E+01_JPRB, 0.18781E+01_JPRB, 0.18638E+01_JPRB, 0.18278E+01_JPRB ,&
&  0.17936E+01_JPRB, 0.16251E+01_JPRB, 0.14053E+01_JPRB, 0.11994E+01_JPRB, 0.10234E+01_JPRB, 0.88703E+00_JPRB, 0.80747E+00_JPRB ,&
&  0.77510E+00_JPRB, 0.75113E+00_JPRB, 0.74628E+00_JPRB, 0.72388E+00_JPRB, 0.67548E+00_JPRB, 0.62335E+00_JPRB, 0.58168E+00_JPRB ,&
&  0.55504E+00_JPRB, 0.52731E+00_JPRB, 0.50039E+00_JPRB, 0.49248E+00_JPRB, 0.52150E+00_JPRB, 0.56329E+00_JPRB, 0.57842E+00_JPRB ,&
&  0.56664E+00_JPRB, 0.56805E+00_JPRB, 0.59998E+00_JPRB, 0.63538E+00_JPRB, 0.65239E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.69971E+00_JPRB, 0.67028E+00_JPRB, 0.64473E+00_JPRB, 0.61914E+00_JPRB, 0.60401E+00_JPRB, 0.58175E+00_JPRB, 0.52635E+00_JPRB ,&
&  0.46965E+00_JPRB, 0.43720E+00_JPRB, 0.44589E+00_JPRB, 0.43513E+00_JPRB, 0.42102E+00_JPRB, 0.41264E+00_JPRB, 0.41309E+00_JPRB ,&
&  0.42105E+00_JPRB, 0.44467E+00_JPRB, 0.46214E+00_JPRB, 0.46608E+00_JPRB, 0.46287E+00_JPRB, 0.45975E+00_JPRB, 0.46636E+00_JPRB ,&
&  0.47601E+00_JPRB, 0.48705E+00_JPRB, 0.49276E+00_JPRB, 0.49105E+00_JPRB, 0.48602E+00_JPRB, 0.48534E+00_JPRB, 0.48897E+00_JPRB ,&
&  0.49780E+00_JPRB, 0.50326E+00_JPRB, 0.49347E+00_JPRB, 0.49788E+00_JPRB, 0.47844E+00_JPRB, 0.47908E+00_JPRB, 0.46220E+00_JPRB ,&
&  0.40341E+00_JPRB, 0.41479E+00_JPRB, 0.53600E+00_JPRB, 0.57169E+00_JPRB, 0.58738E+00_JPRB, 0.62221E+00_JPRB, 0.66907E+00_JPRB ,&
&  0.71259E+00_JPRB, 0.75391E+00_JPRB, 0.79112E+00_JPRB, 0.83192E+00_JPRB, 0.86982E+00_JPRB, 0.92002E+00_JPRB, 0.96495E+00_JPRB ,&
&  0.10395E+01_JPRB, 0.86453E+00_JPRB, 0.81382E+00_JPRB, 0.86093E+00_JPRB, 0.11311E+01_JPRB, 0.11700E+01_JPRB, 0.11544E+01_JPRB ,&
&  0.11834E+01_JPRB, 0.11604E+01_JPRB, 0.11382E+01_JPRB, 0.11402E+01_JPRB, 0.11835E+01_JPRB, 0.12646E+01_JPRB, 0.14013E+01_JPRB ,&
&  0.15295E+01_JPRB, 0.17800E+01_JPRB, 0.21624E+01_JPRB, 0.22601E+01_JPRB, 0.22341E+01_JPRB, 0.22231E+01_JPRB, 0.21540E+01_JPRB ,&
&  0.20615E+01_JPRB, 0.18793E+01_JPRB, 0.15780E+01_JPRB, 0.13255E+01_JPRB, 0.11170E+01_JPRB, 0.95933E+00_JPRB, 0.83553E+00_JPRB ,&
&  0.77308E+00_JPRB, 0.75159E+00_JPRB, 0.73751E+00_JPRB, 0.71613E+00_JPRB, 0.68423E+00_JPRB, 0.64773E+00_JPRB, 0.60743E+00_JPRB ,&
&  0.55762E+00_JPRB, 0.52363E+00_JPRB, 0.49336E+00_JPRB, 0.50624E+00_JPRB, 0.54961E+00_JPRB, 0.59586E+00_JPRB, 0.61364E+00_JPRB ,&
&  0.62452E+00_JPRB, 0.62750E+00_JPRB, 0.62936E+00_JPRB, 0.64678E+00_JPRB, 0.67475E+00_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.69911E+00_JPRB, 0.66119E+00_JPRB, 0.62485E+00_JPRB, 0.59284E+00_JPRB, 0.57444E+00_JPRB, 0.55709E+00_JPRB, 0.53877E+00_JPRB ,&
&  0.58251E+00_JPRB, 0.50106E+00_JPRB, 0.58038E+00_JPRB, 0.47764E+00_JPRB, 0.44587E+00_JPRB, 0.43942E+00_JPRB, 0.42302E+00_JPRB ,&
&  0.41298E+00_JPRB, 0.41493E+00_JPRB, 0.42128E+00_JPRB, 0.42908E+00_JPRB, 0.42860E+00_JPRB, 0.42720E+00_JPRB, 0.43445E+00_JPRB ,&
&  0.44952E+00_JPRB, 0.47156E+00_JPRB, 0.49744E+00_JPRB, 0.51046E+00_JPRB, 0.51562E+00_JPRB, 0.51496E+00_JPRB, 0.52345E+00_JPRB ,&
&  0.49648E+00_JPRB, 0.49052E+00_JPRB, 0.49855E+00_JPRB, 0.49948E+00_JPRB, 0.49532E+00_JPRB, 0.48456E+00_JPRB, 0.50175E+00_JPRB ,&
&  0.49907E+00_JPRB, 0.47628E+00_JPRB, 0.58367E+00_JPRB, 0.59855E+00_JPRB, 0.62730E+00_JPRB, 0.65918E+00_JPRB, 0.69440E+00_JPRB ,&
&  0.73160E+00_JPRB, 0.77689E+00_JPRB, 0.82854E+00_JPRB, 0.86645E+00_JPRB, 0.92537E+00_JPRB, 0.99134E+00_JPRB, 0.10473E+01_JPRB ,&
&  0.11253E+01_JPRB, 0.87994E+00_JPRB, 0.96698E+00_JPRB, 0.10171E+01_JPRB, 0.12373E+01_JPRB, 0.14905E+01_JPRB, 0.14962E+01_JPRB ,&
&  0.16229E+01_JPRB, 0.15783E+01_JPRB, 0.15279E+01_JPRB, 0.14878E+01_JPRB, 0.15700E+01_JPRB, 0.16929E+01_JPRB, 0.18377E+01_JPRB ,&
&  0.20368E+01_JPRB, 0.23115E+01_JPRB, 0.27173E+01_JPRB, 0.27395E+01_JPRB, 0.26939E+01_JPRB, 0.26189E+01_JPRB, 0.24779E+01_JPRB ,&
&  0.22818E+01_JPRB, 0.20420E+01_JPRB, 0.17408E+01_JPRB, 0.14172E+01_JPRB, 0.11982E+01_JPRB, 0.10144E+01_JPRB, 0.86503E+00_JPRB ,&
&  0.75830E+00_JPRB, 0.69698E+00_JPRB, 0.68625E+00_JPRB, 0.66555E+00_JPRB, 0.65084E+00_JPRB, 0.62649E+00_JPRB, 0.58404E+00_JPRB ,&
&  0.54530E+00_JPRB, 0.55382E+00_JPRB, 0.59875E+00_JPRB, 0.66157E+00_JPRB, 0.68653E+00_JPRB, 0.69859E+00_JPRB, 0.75188E+00_JPRB ,&
&  0.68754E+00_JPRB, 0.72467E+00_JPRB, 0.77379E+00_JPRB, 0.75083E+00_JPRB, 0.73194E+00_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.71124E+00_JPRB, 0.65876E+00_JPRB, 0.59655E+00_JPRB, 0.56790E+00_JPRB, 0.57316E+00_JPRB, 0.58066E+00_JPRB, 0.56398E+00_JPRB ,&
&  0.59854E+00_JPRB, 0.58415E+00_JPRB, 0.61053E+00_JPRB, 0.64820E+00_JPRB, 0.68544E+00_JPRB, 0.60141E+00_JPRB, 0.47370E+00_JPRB ,&
&  0.43329E+00_JPRB, 0.43472E+00_JPRB, 0.44000E+00_JPRB, 0.43724E+00_JPRB, 0.43652E+00_JPRB, 0.43263E+00_JPRB, 0.43268E+00_JPRB ,&
&  0.43818E+00_JPRB, 0.45087E+00_JPRB, 0.48410E+00_JPRB, 0.49830E+00_JPRB, 0.50572E+00_JPRB, 0.50850E+00_JPRB, 0.52141E+00_JPRB ,&
&  0.49644E+00_JPRB, 0.48887E+00_JPRB, 0.48781E+00_JPRB, 0.46408E+00_JPRB, 0.49904E+00_JPRB, 0.51230E+00_JPRB, 0.52312E+00_JPRB ,&
&  0.53681E+00_JPRB, 0.56348E+00_JPRB, 0.59597E+00_JPRB, 0.62501E+00_JPRB, 0.65179E+00_JPRB, 0.67973E+00_JPRB, 0.71018E+00_JPRB ,&
&  0.74237E+00_JPRB, 0.78336E+00_JPRB, 0.83191E+00_JPRB, 0.88923E+00_JPRB, 0.10147E+01_JPRB, 0.98961E+00_JPRB, 0.10821E+01_JPRB ,&
&  0.11311E+01_JPRB, 0.12552E+01_JPRB, 0.13829E+01_JPRB, 0.13084E+01_JPRB, 0.14212E+01_JPRB, 0.16001E+01_JPRB, 0.15782E+01_JPRB ,&
&  0.15931E+01_JPRB, 0.15549E+01_JPRB, 0.16320E+01_JPRB, 0.17563E+01_JPRB, 0.19379E+01_JPRB, 0.20930E+01_JPRB, 0.22480E+01_JPRB ,&
&  0.25142E+01_JPRB, 0.30114E+01_JPRB, 0.32158E+01_JPRB, 0.31302E+01_JPRB, 0.30127E+01_JPRB, 0.28702E+01_JPRB, 0.26501E+01_JPRB ,&
&  0.24010E+01_JPRB, 0.21155E+01_JPRB, 0.17985E+01_JPRB, 0.14800E+01_JPRB, 0.12084E+01_JPRB, 0.10594E+01_JPRB, 0.89878E+00_JPRB ,&
&  0.74071E+00_JPRB, 0.63820E+00_JPRB, 0.63563E+00_JPRB, 0.63483E+00_JPRB, 0.62343E+00_JPRB, 0.58869E+00_JPRB, 0.55796E+00_JPRB ,&
&  0.58948E+00_JPRB, 0.67124E+00_JPRB, 0.88398E+00_JPRB, 0.89812E+00_JPRB, 0.90252E+00_JPRB, 0.87692E+00_JPRB, 0.72539E+00_JPRB ,&
&  0.66949E+00_JPRB, 0.80505E+00_JPRB, 0.77104E+00_JPRB, 0.76758E+00_JPRB, 0.75053E+00_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.73639E+00_JPRB, 0.66369E+00_JPRB, 0.58127E+00_JPRB, 0.55925E+00_JPRB, 0.57638E+00_JPRB, 0.58378E+00_JPRB, 0.58821E+00_JPRB ,&
&  0.60391E+00_JPRB, 0.61143E+00_JPRB, 0.62301E+00_JPRB, 0.63825E+00_JPRB, 0.64489E+00_JPRB, 0.67540E+00_JPRB, 0.60786E+00_JPRB ,&
&  0.51412E+00_JPRB, 0.48541E+00_JPRB, 0.46282E+00_JPRB, 0.45798E+00_JPRB, 0.47543E+00_JPRB, 0.47648E+00_JPRB, 0.47514E+00_JPRB ,&
&  0.48150E+00_JPRB, 0.48172E+00_JPRB, 0.49332E+00_JPRB, 0.49401E+00_JPRB, 0.49349E+00_JPRB, 0.50010E+00_JPRB, 0.51055E+00_JPRB ,&
&  0.48167E+00_JPRB, 0.45903E+00_JPRB, 0.44589E+00_JPRB, 0.43911E+00_JPRB, 0.46854E+00_JPRB, 0.51142E+00_JPRB, 0.53823E+00_JPRB ,&
&  0.55668E+00_JPRB, 0.57697E+00_JPRB, 0.60013E+00_JPRB, 0.62873E+00_JPRB, 0.66001E+00_JPRB, 0.69187E+00_JPRB, 0.71767E+00_JPRB ,&
&  0.74583E+00_JPRB, 0.77893E+00_JPRB, 0.82562E+00_JPRB, 0.90452E+00_JPRB, 0.94857E+00_JPRB, 0.10073E+01_JPRB, 0.11387E+01_JPRB ,&
&  0.12341E+01_JPRB, 0.13642E+01_JPRB, 0.14846E+01_JPRB, 0.16757E+01_JPRB, 0.18200E+01_JPRB, 0.17826E+01_JPRB, 0.17322E+01_JPRB ,&
&  0.17488E+01_JPRB, 0.17506E+01_JPRB, 0.20121E+01_JPRB, 0.21955E+01_JPRB, 0.23210E+01_JPRB, 0.25021E+01_JPRB, 0.26652E+01_JPRB ,&
&  0.29535E+01_JPRB, 0.32600E+01_JPRB, 0.33352E+01_JPRB, 0.32370E+01_JPRB, 0.30961E+01_JPRB, 0.29592E+01_JPRB, 0.27126E+01_JPRB ,&
&  0.24326E+01_JPRB, 0.21013E+01_JPRB, 0.17214E+01_JPRB, 0.14057E+01_JPRB, 0.11274E+01_JPRB, 0.10422E+01_JPRB, 0.92995E+00_JPRB ,&
&  0.73846E+00_JPRB, 0.64893E+00_JPRB, 0.64383E+00_JPRB, 0.67677E+00_JPRB, 0.66918E+00_JPRB, 0.62344E+00_JPRB, 0.60040E+00_JPRB ,&
&  0.65308E+00_JPRB, 0.82492E+00_JPRB, 0.90469E+00_JPRB, 0.96131E+00_JPRB, 0.97321E+00_JPRB, 0.94235E+00_JPRB, 0.68717E+00_JPRB ,&
&  0.70126E+00_JPRB, 0.81740E+00_JPRB, 0.80336E+00_JPRB, 0.79151E+00_JPRB, 0.77505E+00_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.76936E+00_JPRB, 0.68591E+00_JPRB, 0.61002E+00_JPRB, 0.57595E+00_JPRB, 0.58992E+00_JPRB, 0.60491E+00_JPRB, 0.61400E+00_JPRB ,&
&  0.62984E+00_JPRB, 0.64188E+00_JPRB, 0.65156E+00_JPRB, 0.66028E+00_JPRB, 0.66457E+00_JPRB, 0.65821E+00_JPRB, 0.70381E+00_JPRB ,&
&  0.56392E+00_JPRB, 0.51284E+00_JPRB, 0.47808E+00_JPRB, 0.47721E+00_JPRB, 0.54350E+00_JPRB, 0.55881E+00_JPRB, 0.57737E+00_JPRB ,&
&  0.57309E+00_JPRB, 0.57661E+00_JPRB, 0.56836E+00_JPRB, 0.55294E+00_JPRB, 0.53743E+00_JPRB, 0.53217E+00_JPRB, 0.52000E+00_JPRB ,&
&  0.48276E+00_JPRB, 0.45555E+00_JPRB, 0.45006E+00_JPRB, 0.45418E+00_JPRB, 0.48665E+00_JPRB, 0.51522E+00_JPRB, 0.56502E+00_JPRB ,&
&  0.59393E+00_JPRB, 0.61794E+00_JPRB, 0.64268E+00_JPRB, 0.66966E+00_JPRB, 0.69347E+00_JPRB, 0.72464E+00_JPRB, 0.76078E+00_JPRB ,&
&  0.79824E+00_JPRB, 0.83690E+00_JPRB, 0.90733E+00_JPRB, 0.99076E+00_JPRB, 0.99242E+00_JPRB, 0.10570E+01_JPRB, 0.12177E+01_JPRB ,&
&  0.13771E+01_JPRB, 0.14619E+01_JPRB, 0.16993E+01_JPRB, 0.23594E+01_JPRB, 0.25442E+01_JPRB, 0.24711E+01_JPRB, 0.23542E+01_JPRB ,&
&  0.22976E+01_JPRB, 0.22532E+01_JPRB, 0.23451E+01_JPRB, 0.24861E+01_JPRB, 0.26214E+01_JPRB, 0.28631E+01_JPRB, 0.30907E+01_JPRB ,&
&  0.32319E+01_JPRB, 0.32850E+01_JPRB, 0.32455E+01_JPRB, 0.31337E+01_JPRB, 0.29308E+01_JPRB, 0.27745E+01_JPRB, 0.25972E+01_JPRB ,&
&  0.23616E+01_JPRB, 0.20672E+01_JPRB, 0.16235E+01_JPRB, 0.13384E+01_JPRB, 0.10166E+01_JPRB, 0.92330E+00_JPRB, 0.10530E+01_JPRB ,&
&  0.79703E+00_JPRB, 0.69748E+00_JPRB, 0.71092E+00_JPRB, 0.76890E+00_JPRB, 0.78041E+00_JPRB, 0.76623E+00_JPRB, 0.68429E+00_JPRB ,&
&  0.61945E+00_JPRB, 0.74106E+00_JPRB, 0.10152E+01_JPRB, 0.10775E+01_JPRB, 0.10680E+01_JPRB, 0.10100E+01_JPRB, 0.81110E+00_JPRB ,&
&  0.88574E+00_JPRB, 0.84047E+00_JPRB, 0.83417E+00_JPRB, 0.82483E+00_JPRB, 0.81299E+00_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.79307E+00_JPRB, 0.71150E+00_JPRB, 0.64755E+00_JPRB, 0.61172E+00_JPRB, 0.61865E+00_JPRB, 0.63840E+00_JPRB, 0.65090E+00_JPRB ,&
&  0.66115E+00_JPRB, 0.66807E+00_JPRB, 0.67375E+00_JPRB, 0.67949E+00_JPRB, 0.68272E+00_JPRB, 0.66348E+00_JPRB, 0.64735E+00_JPRB ,&
&  0.63968E+00_JPRB, 0.54270E+00_JPRB, 0.51158E+00_JPRB, 0.50020E+00_JPRB, 0.55928E+00_JPRB, 0.60953E+00_JPRB, 0.66517E+00_JPRB ,&
&  0.71018E+00_JPRB, 0.74553E+00_JPRB, 0.73210E+00_JPRB, 0.69321E+00_JPRB, 0.65309E+00_JPRB, 0.62055E+00_JPRB, 0.57168E+00_JPRB ,&
&  0.51884E+00_JPRB, 0.49415E+00_JPRB, 0.49685E+00_JPRB, 0.52970E+00_JPRB, 0.58899E+00_JPRB, 0.55618E+00_JPRB, 0.61859E+00_JPRB ,&
&  0.65462E+00_JPRB, 0.68905E+00_JPRB, 0.70379E+00_JPRB, 0.72623E+00_JPRB, 0.78579E+00_JPRB, 0.86088E+00_JPRB, 0.92835E+00_JPRB ,&
&  0.98948E+00_JPRB, 0.10077E+01_JPRB, 0.99786E+00_JPRB, 0.10688E+01_JPRB, 0.11411E+01_JPRB, 0.11897E+01_JPRB, 0.12874E+01_JPRB ,&
&  0.14819E+01_JPRB, 0.16162E+01_JPRB, 0.19251E+01_JPRB, 0.27263E+01_JPRB, 0.29836E+01_JPRB, 0.29163E+01_JPRB, 0.29448E+01_JPRB ,&
&  0.29451E+01_JPRB, 0.28794E+01_JPRB, 0.28432E+01_JPRB, 0.27980E+01_JPRB, 0.29742E+01_JPRB, 0.32694E+01_JPRB, 0.35046E+01_JPRB ,&
&  0.33967E+01_JPRB, 0.31582E+01_JPRB, 0.30557E+01_JPRB, 0.28460E+01_JPRB, 0.26421E+01_JPRB, 0.24130E+01_JPRB, 0.24293E+01_JPRB ,&
&  0.24396E+01_JPRB, 0.17659E+01_JPRB, 0.14421E+01_JPRB, 0.13501E+01_JPRB, 0.10092E+01_JPRB, 0.86159E+00_JPRB, 0.89347E+00_JPRB ,&
&  0.73154E+00_JPRB, 0.67128E+00_JPRB, 0.74538E+00_JPRB, 0.82425E+00_JPRB, 0.91931E+00_JPRB, 0.97130E+00_JPRB, 0.92468E+00_JPRB ,&
&  0.85957E+00_JPRB, 0.91477E+00_JPRB, 0.12450E+01_JPRB, 0.12838E+01_JPRB, 0.12147E+01_JPRB, 0.10992E+01_JPRB, 0.99622E+00_JPRB ,&
&  0.91720E+00_JPRB, 0.91329E+00_JPRB, 0.90031E+00_JPRB, 0.88537E+00_JPRB, 0.86526E+00_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.86583E+00_JPRB, 0.78134E+00_JPRB, 0.70869E+00_JPRB, 0.67739E+00_JPRB, 0.68780E+00_JPRB, 0.69964E+00_JPRB, 0.70859E+00_JPRB ,&
&  0.69377E+00_JPRB, 0.68431E+00_JPRB, 0.69395E+00_JPRB, 0.69441E+00_JPRB, 0.69254E+00_JPRB, 0.67290E+00_JPRB, 0.63399E+00_JPRB ,&
&  0.61351E+00_JPRB, 0.58895E+00_JPRB, 0.54149E+00_JPRB, 0.54523E+00_JPRB, 0.57283E+00_JPRB, 0.66852E+00_JPRB, 0.75173E+00_JPRB ,&
&  0.83742E+00_JPRB, 0.90518E+00_JPRB, 0.92471E+00_JPRB, 0.91951E+00_JPRB, 0.87586E+00_JPRB, 0.80182E+00_JPRB, 0.73093E+00_JPRB ,&
&  0.66866E+00_JPRB, 0.64120E+00_JPRB, 0.62446E+00_JPRB, 0.69958E+00_JPRB, 0.73122E+00_JPRB, 0.69625E+00_JPRB, 0.78557E+00_JPRB ,&
&  0.82201E+00_JPRB, 0.85514E+00_JPRB, 0.87177E+00_JPRB, 0.93599E+00_JPRB, 0.10064E+01_JPRB, 0.10319E+01_JPRB, 0.10238E+01_JPRB ,&
&  0.10100E+01_JPRB, 0.10238E+01_JPRB, 0.10664E+01_JPRB, 0.10561E+01_JPRB, 0.11143E+01_JPRB, 0.12639E+01_JPRB, 0.15319E+01_JPRB ,&
&  0.18491E+01_JPRB, 0.18486E+01_JPRB, 0.18950E+01_JPRB, 0.27111E+01_JPRB, 0.34639E+01_JPRB, 0.34663E+01_JPRB, 0.33872E+01_JPRB ,&
&  0.34103E+01_JPRB, 0.32238E+01_JPRB, 0.30445E+01_JPRB, 0.30021E+01_JPRB, 0.34501E+01_JPRB, 0.37780E+01_JPRB, 0.38777E+01_JPRB ,&
&  0.36225E+01_JPRB, 0.32524E+01_JPRB, 0.30647E+01_JPRB, 0.28407E+01_JPRB, 0.25322E+01_JPRB, 0.24200E+01_JPRB, 0.24738E+01_JPRB ,&
&  0.24247E+01_JPRB, 0.21473E+01_JPRB, 0.16441E+01_JPRB, 0.14718E+01_JPRB, 0.11671E+01_JPRB, 0.10107E+01_JPRB, 0.99056E+00_JPRB ,&
&  0.84157E+00_JPRB, 0.80069E+00_JPRB, 0.82221E+00_JPRB, 0.87162E+00_JPRB, 0.12502E+01_JPRB, 0.13558E+01_JPRB, 0.14580E+01_JPRB ,&
&  0.14527E+01_JPRB, 0.13899E+01_JPRB, 0.17224E+01_JPRB, 0.16740E+01_JPRB, 0.14808E+01_JPRB, 0.12543E+01_JPRB, 0.10739E+01_JPRB ,&
&  0.10034E+01_JPRB, 0.10110E+01_JPRB, 0.98893E+00_JPRB, 0.96575E+00_JPRB, 0.94005E+00_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.89495E+00_JPRB, 0.80745E+00_JPRB, 0.73565E+00_JPRB, 0.70462E+00_JPRB, 0.70549E+00_JPRB, 0.71427E+00_JPRB, 0.71525E+00_JPRB ,&
&  0.70943E+00_JPRB, 0.71021E+00_JPRB, 0.71074E+00_JPRB, 0.69470E+00_JPRB, 0.67639E+00_JPRB, 0.65481E+00_JPRB, 0.62167E+00_JPRB ,&
&  0.59546E+00_JPRB, 0.58343E+00_JPRB, 0.54908E+00_JPRB, 0.56125E+00_JPRB, 0.61596E+00_JPRB, 0.75162E+00_JPRB, 0.90934E+00_JPRB ,&
&  0.10514E+01_JPRB, 0.11332E+01_JPRB, 0.11743E+01_JPRB, 0.11820E+01_JPRB, 0.11847E+01_JPRB, 0.11351E+01_JPRB, 0.10471E+01_JPRB ,&
&  0.10295E+01_JPRB, 0.88612E+00_JPRB, 0.96160E+00_JPRB, 0.96176E+00_JPRB, 0.10262E+01_JPRB, 0.10287E+01_JPRB, 0.10006E+01_JPRB ,&
&  0.97293E+00_JPRB, 0.98079E+00_JPRB, 0.98352E+00_JPRB, 0.10135E+01_JPRB, 0.10519E+01_JPRB, 0.10726E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.11236E+01_JPRB, 0.10853E+01_JPRB, 0.10154E+01_JPRB, 0.10166E+01_JPRB, 0.13713E+01_JPRB, 0.18929E+01_JPRB, 0.23834E+01_JPRB ,&
&  0.23560E+01_JPRB, 0.21498E+01_JPRB, 0.30547E+01_JPRB, 0.35324E+01_JPRB, 0.35153E+01_JPRB, 0.38640E+01_JPRB, 0.38093E+01_JPRB ,&
&  0.34671E+01_JPRB, 0.30550E+01_JPRB, 0.27477E+01_JPRB, 0.29162E+01_JPRB, 0.41460E+01_JPRB, 0.45030E+01_JPRB, 0.45108E+01_JPRB ,&
&  0.40503E+01_JPRB, 0.35533E+01_JPRB, 0.31089E+01_JPRB, 0.28073E+01_JPRB, 0.26417E+01_JPRB, 0.27520E+01_JPRB, 0.25901E+01_JPRB ,&
&  0.24010E+01_JPRB, 0.25174E+01_JPRB, 0.21068E+01_JPRB, 0.16619E+01_JPRB, 0.13873E+01_JPRB, 0.12592E+01_JPRB, 0.11784E+01_JPRB ,&
&  0.10509E+01_JPRB, 0.98952E+00_JPRB, 0.99698E+00_JPRB, 0.12095E+01_JPRB, 0.17721E+01_JPRB, 0.18301E+01_JPRB, 0.18744E+01_JPRB ,&
&  0.19644E+01_JPRB, 0.24268E+01_JPRB, 0.21938E+01_JPRB, 0.19299E+01_JPRB, 0.16690E+01_JPRB, 0.14207E+01_JPRB, 0.12350E+01_JPRB ,&
&  0.11210E+01_JPRB, 0.11113E+01_JPRB, 0.10920E+01_JPRB, 0.10421E+01_JPRB, 0.98644E+00_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.96176E+00_JPRB, 0.85989E+00_JPRB, 0.80847E+00_JPRB, 0.77097E+00_JPRB, 0.75817E+00_JPRB, 0.74549E+00_JPRB, 0.71134E+00_JPRB ,&
&  0.68919E+00_JPRB, 0.69821E+00_JPRB, 0.68553E+00_JPRB, 0.67809E+00_JPRB, 0.67036E+00_JPRB, 0.64109E+00_JPRB, 0.61020E+00_JPRB ,&
&  0.59812E+00_JPRB, 0.58123E+00_JPRB, 0.58135E+00_JPRB, 0.64617E+00_JPRB, 0.72694E+00_JPRB, 0.84002E+00_JPRB, 0.10299E+01_JPRB ,&
&  0.12087E+01_JPRB, 0.13419E+01_JPRB, 0.13981E+01_JPRB, 0.14931E+01_JPRB, 0.15536E+01_JPRB, 0.15671E+01_JPRB, 0.14751E+01_JPRB ,&
&  0.13120E+01_JPRB, 0.13148E+01_JPRB, 0.14009E+01_JPRB, 0.13526E+01_JPRB, 0.13179E+01_JPRB, 0.12605E+01_JPRB, 0.12007E+01_JPRB ,&
&  0.11378E+01_JPRB, 0.10903E+01_JPRB, 0.10869E+01_JPRB, 0.11092E+01_JPRB, 0.11440E+01_JPRB, 0.12088E+01_JPRB, 0.12023E+01_JPRB ,&
&  0.11248E+01_JPRB, 0.10775E+01_JPRB, 0.10326E+01_JPRB, 0.12918E+01_JPRB, 0.19348E+01_JPRB, 0.29900E+01_JPRB, 0.33534E+01_JPRB ,&
&  0.35227E+01_JPRB, 0.31412E+01_JPRB, 0.32002E+01_JPRB, 0.36937E+01_JPRB, 0.30757E+01_JPRB, 0.33510E+01_JPRB, 0.34648E+01_JPRB ,&
&  0.31132E+01_JPRB, 0.29637E+01_JPRB, 0.33603E+01_JPRB, 0.40974E+01_JPRB, 0.50900E+01_JPRB, 0.62125E+01_JPRB, 0.61029E+01_JPRB ,&
&  0.50319E+01_JPRB, 0.41061E+01_JPRB, 0.32514E+01_JPRB, 0.29828E+01_JPRB, 0.26500E+01_JPRB, 0.25844E+01_JPRB, 0.20796E+01_JPRB ,&
&  0.20961E+01_JPRB, 0.18026E+01_JPRB, 0.19190E+01_JPRB, 0.16877E+01_JPRB, 0.15490E+01_JPRB, 0.13949E+01_JPRB, 0.12762E+01_JPRB ,&
&  0.12197E+01_JPRB, 0.12034E+01_JPRB, 0.13335E+01_JPRB, 0.18332E+01_JPRB, 0.24910E+01_JPRB, 0.21655E+01_JPRB, 0.26562E+01_JPRB ,&
&  0.34354E+01_JPRB, 0.32692E+01_JPRB, 0.23786E+01_JPRB, 0.20919E+01_JPRB, 0.17595E+01_JPRB, 0.15241E+01_JPRB, 0.14038E+01_JPRB ,&
&  0.12796E+01_JPRB, 0.12163E+01_JPRB, 0.11737E+01_JPRB, 0.11215E+01_JPRB, 0.10591E+01_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.10207E+01_JPRB, 0.91446E+00_JPRB, 0.86791E+00_JPRB, 0.85389E+00_JPRB, 0.82776E+00_JPRB, 0.75446E+00_JPRB, 0.71974E+00_JPRB ,&
&  0.73425E+00_JPRB, 0.70918E+00_JPRB, 0.66601E+00_JPRB, 0.64237E+00_JPRB, 0.64231E+00_JPRB, 0.64658E+00_JPRB, 0.62135E+00_JPRB ,&
&  0.62948E+00_JPRB, 0.66990E+00_JPRB, 0.70741E+00_JPRB, 0.76397E+00_JPRB, 0.86106E+00_JPRB, 0.95422E+00_JPRB, 0.10417E+01_JPRB ,&
&  0.14362E+01_JPRB, 0.15842E+01_JPRB, 0.15767E+01_JPRB, 0.16892E+01_JPRB, 0.18203E+01_JPRB, 0.18944E+01_JPRB, 0.18247E+01_JPRB ,&
&  0.18298E+01_JPRB, 0.18471E+01_JPRB, 0.15904E+01_JPRB, 0.14111E+01_JPRB, 0.13060E+01_JPRB, 0.12496E+01_JPRB, 0.11859E+01_JPRB ,&
&  0.11031E+01_JPRB, 0.10472E+01_JPRB, 0.10434E+01_JPRB, 0.10450E+01_JPRB, 0.10876E+01_JPRB, 0.10305E+01_JPRB, 0.10130E+01_JPRB ,&
&  0.10214E+01_JPRB, 0.10716E+01_JPRB, 0.11794E+01_JPRB, 0.16607E+01_JPRB, 0.27025E+01_JPRB, 0.40527E+01_JPRB, 0.44119E+01_JPRB ,&
&  0.38642E+01_JPRB, 0.32720E+01_JPRB, 0.34076E+01_JPRB, 0.37799E+01_JPRB, 0.34547E+01_JPRB, 0.32150E+01_JPRB, 0.30122E+01_JPRB ,&
&  0.25444E+01_JPRB, 0.26580E+01_JPRB, 0.33800E+01_JPRB, 0.46280E+01_JPRB, 0.64242E+01_JPRB, 0.81723E+01_JPRB, 0.79407E+01_JPRB ,&
&  0.62714E+01_JPRB, 0.49032E+01_JPRB, 0.39034E+01_JPRB, 0.30849E+01_JPRB, 0.29676E+01_JPRB, 0.22567E+01_JPRB, 0.25012E+01_JPRB ,&
&  0.25052E+01_JPRB, 0.23885E+01_JPRB, 0.19756E+01_JPRB, 0.16157E+01_JPRB, 0.13147E+01_JPRB, 0.14190E+01_JPRB, 0.13676E+01_JPRB ,&
&  0.14206E+01_JPRB, 0.15079E+01_JPRB, 0.24501E+01_JPRB, 0.28798E+01_JPRB, 0.31132E+01_JPRB, 0.31248E+01_JPRB, 0.41425E+01_JPRB ,&
&  0.38936E+01_JPRB, 0.33949E+01_JPRB, 0.27170E+01_JPRB, 0.22641E+01_JPRB, 0.19110E+01_JPRB, 0.16970E+01_JPRB, 0.15700E+01_JPRB ,&
&  0.14256E+01_JPRB, 0.13490E+01_JPRB, 0.12885E+01_JPRB, 0.12189E+01_JPRB, 0.11350E+01_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.10325E+01_JPRB, 0.97738E+00_JPRB, 0.95637E+00_JPRB, 0.89394E+00_JPRB, 0.79740E+00_JPRB, 0.74011E+00_JPRB, 0.73712E+00_JPRB ,&
&  0.72430E+00_JPRB, 0.69468E+00_JPRB, 0.66220E+00_JPRB, 0.63899E+00_JPRB, 0.64309E+00_JPRB, 0.65445E+00_JPRB, 0.66368E+00_JPRB ,&
&  0.72929E+00_JPRB, 0.86919E+00_JPRB, 0.90147E+00_JPRB, 0.96062E+00_JPRB, 0.10146E+01_JPRB, 0.10727E+01_JPRB, 0.11838E+01_JPRB ,&
&  0.16640E+01_JPRB, 0.16605E+01_JPRB, 0.15086E+01_JPRB, 0.16297E+01_JPRB, 0.19347E+01_JPRB, 0.18818E+01_JPRB, 0.18433E+01_JPRB ,&
&  0.18723E+01_JPRB, 0.17838E+01_JPRB, 0.15444E+01_JPRB, 0.13458E+01_JPRB, 0.12153E+01_JPRB, 0.11421E+01_JPRB, 0.10633E+01_JPRB ,&
&  0.10174E+01_JPRB, 0.10247E+01_JPRB, 0.10505E+01_JPRB, 0.10865E+01_JPRB, 0.10591E+01_JPRB, 0.10287E+01_JPRB, 0.10356E+01_JPRB ,&
&  0.10545E+01_JPRB, 0.11295E+01_JPRB, 0.13327E+01_JPRB, 0.20132E+01_JPRB, 0.31779E+01_JPRB, 0.47057E+01_JPRB, 0.53739E+01_JPRB ,&
&  0.42538E+01_JPRB, 0.39100E+01_JPRB, 0.39363E+01_JPRB, 0.49235E+01_JPRB, 0.34124E+01_JPRB, 0.30918E+01_JPRB, 0.30100E+01_JPRB ,&
&  0.28612E+01_JPRB, 0.34234E+01_JPRB, 0.47385E+01_JPRB, 0.62884E+01_JPRB, 0.67682E+01_JPRB, 0.79503E+01_JPRB, 0.91466E+01_JPRB ,&
&  0.66827E+01_JPRB, 0.51033E+01_JPRB, 0.42701E+01_JPRB, 0.39195E+01_JPRB, 0.25498E+01_JPRB, 0.17314E+01_JPRB, 0.24601E+01_JPRB ,&
&  0.24105E+01_JPRB, 0.13790E+01_JPRB, 0.10158E+01_JPRB, 0.11432E+01_JPRB, 0.84495E+00_JPRB, 0.96097E+00_JPRB, 0.13503E+01_JPRB ,&
&  0.16339E+01_JPRB, 0.19511E+01_JPRB, 0.37491E+01_JPRB, 0.36058E+01_JPRB, 0.37785E+01_JPRB, 0.47339E+01_JPRB, 0.51331E+01_JPRB ,&
&  0.44752E+01_JPRB, 0.28434E+01_JPRB, 0.26597E+01_JPRB, 0.23763E+01_JPRB, 0.19892E+01_JPRB, 0.16977E+01_JPRB, 0.14900E+01_JPRB ,&
&  0.13188E+01_JPRB, 0.12057E+01_JPRB, 0.11384E+01_JPRB, 0.11031E+01_JPRB, 0.10915E+01_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.90929E+00_JPRB, 0.90181E+00_JPRB, 0.87488E+00_JPRB, 0.79321E+00_JPRB, 0.73797E+00_JPRB, 0.72720E+00_JPRB, 0.71548E+00_JPRB ,&
&  0.69709E+00_JPRB, 0.66179E+00_JPRB, 0.64314E+00_JPRB, 0.65046E+00_JPRB, 0.66328E+00_JPRB, 0.68097E+00_JPRB, 0.71708E+00_JPRB ,&
&  0.78088E+00_JPRB, 0.90034E+00_JPRB, 0.10917E+01_JPRB, 0.12222E+01_JPRB, 0.12873E+01_JPRB, 0.11361E+01_JPRB, 0.13883E+01_JPRB ,&
&  0.17255E+01_JPRB, 0.15638E+01_JPRB, 0.14492E+01_JPRB, 0.16430E+01_JPRB, 0.17565E+01_JPRB, 0.16220E+01_JPRB, 0.16191E+01_JPRB ,&
&  0.16439E+01_JPRB, 0.15712E+01_JPRB, 0.14296E+01_JPRB, 0.13389E+01_JPRB, 0.12158E+01_JPRB, 0.10926E+01_JPRB, 0.10730E+01_JPRB ,&
&  0.10874E+01_JPRB, 0.11482E+01_JPRB, 0.12124E+01_JPRB, 0.11731E+01_JPRB, 0.11584E+01_JPRB, 0.11348E+01_JPRB, 0.11441E+01_JPRB ,&
&  0.11811E+01_JPRB, 0.13030E+01_JPRB, 0.16434E+01_JPRB, 0.26211E+01_JPRB, 0.43264E+01_JPRB, 0.63386E+01_JPRB, 0.52110E+01_JPRB ,&
&  0.38447E+01_JPRB, 0.34393E+01_JPRB, 0.43739E+01_JPRB, 0.46442E+01_JPRB, 0.33008E+01_JPRB, 0.32011E+01_JPRB, 0.31482E+01_JPRB ,&
&  0.36518E+01_JPRB, 0.44143E+01_JPRB, 0.49129E+01_JPRB, 0.62383E+01_JPRB, 0.78848E+01_JPRB, 0.81689E+01_JPRB, 0.83123E+01_JPRB ,&
&  0.67057E+01_JPRB, 0.50469E+01_JPRB, 0.35876E+01_JPRB, 0.28027E+01_JPRB, 0.30757E+01_JPRB, 0.22281E+01_JPRB, 0.14043E+01_JPRB ,&
&  0.10874E+01_JPRB, 0.85100E+00_JPRB, 0.68237E+00_JPRB, 0.66422E+00_JPRB, 0.58358E+00_JPRB, 0.77529E+00_JPRB, 0.12414E+01_JPRB ,&
&  0.22612E+01_JPRB, 0.31733E+01_JPRB, 0.41246E+01_JPRB, 0.42581E+01_JPRB, 0.39269E+01_JPRB, 0.50200E+01_JPRB, 0.58465E+01_JPRB ,&
&  0.36003E+01_JPRB, 0.26852E+01_JPRB, 0.25895E+01_JPRB, 0.21409E+01_JPRB, 0.16533E+01_JPRB, 0.13556E+01_JPRB, 0.11554E+01_JPRB ,&
&  0.10348E+01_JPRB, 0.96454E+00_JPRB, 0.92716E+00_JPRB, 0.92111E+00_JPRB, 0.92901E+00_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.82069E+00_JPRB, 0.80705E+00_JPRB, 0.78798E+00_JPRB, 0.74357E+00_JPRB, 0.71261E+00_JPRB, 0.70926E+00_JPRB, 0.70417E+00_JPRB ,&
&  0.68046E+00_JPRB, 0.65419E+00_JPRB, 0.65571E+00_JPRB, 0.67704E+00_JPRB, 0.70666E+00_JPRB, 0.75243E+00_JPRB, 0.79970E+00_JPRB ,&
&  0.85700E+00_JPRB, 0.10108E+01_JPRB, 0.12831E+01_JPRB, 0.15265E+01_JPRB, 0.16839E+01_JPRB, 0.13660E+01_JPRB, 0.15921E+01_JPRB ,&
&  0.17016E+01_JPRB, 0.15539E+01_JPRB, 0.15357E+01_JPRB, 0.16790E+01_JPRB, 0.17042E+01_JPRB, 0.14828E+01_JPRB, 0.15314E+01_JPRB ,&
&  0.15204E+01_JPRB, 0.14738E+01_JPRB, 0.13859E+01_JPRB, 0.13304E+01_JPRB, 0.12334E+01_JPRB, 0.12540E+01_JPRB, 0.13420E+01_JPRB ,&
&  0.13915E+01_JPRB, 0.14592E+01_JPRB, 0.14186E+01_JPRB, 0.13730E+01_JPRB, 0.13484E+01_JPRB, 0.13337E+01_JPRB, 0.13616E+01_JPRB ,&
&  0.14544E+01_JPRB, 0.17073E+01_JPRB, 0.23833E+01_JPRB, 0.39203E+01_JPRB, 0.52778E+01_JPRB, 0.53265E+01_JPRB, 0.47340E+01_JPRB ,&
&  0.42209E+01_JPRB, 0.42441E+01_JPRB, 0.43072E+01_JPRB, 0.48282E+01_JPRB, 0.34814E+01_JPRB, 0.32541E+01_JPRB, 0.32891E+01_JPRB ,&
&  0.38680E+01_JPRB, 0.52943E+01_JPRB, 0.58319E+01_JPRB, 0.66675E+01_JPRB, 0.96856E+01_JPRB, 0.10072E+02_JPRB, 0.77748E+01_JPRB ,&
&  0.65755E+01_JPRB, 0.48848E+01_JPRB, 0.38105E+01_JPRB, 0.31892E+01_JPRB, 0.43276E+01_JPRB, 0.42899E+01_JPRB, 0.15437E+01_JPRB ,&
&  0.90690E+00_JPRB, 0.71004E+00_JPRB, 0.52151E+00_JPRB, 0.50251E+00_JPRB, 0.52695E+00_JPRB, 0.76166E+00_JPRB, 0.17934E+01_JPRB ,&
&  0.21050E+01_JPRB, 0.33688E+01_JPRB, 0.37765E+01_JPRB, 0.40969E+01_JPRB, 0.38778E+01_JPRB, 0.38924E+01_JPRB, 0.40129E+01_JPRB ,&
&  0.35246E+01_JPRB, 0.27604E+01_JPRB, 0.20666E+01_JPRB, 0.15977E+01_JPRB, 0.12590E+01_JPRB, 0.10833E+01_JPRB, 0.98228E+00_JPRB ,&
&  0.92930E+00_JPRB, 0.90048E+00_JPRB, 0.87485E+00_JPRB, 0.85014E+00_JPRB, 0.82203E+00_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.73981E+00_JPRB, 0.74571E+00_JPRB, 0.76119E+00_JPRB, 0.75415E+00_JPRB, 0.74411E+00_JPRB, 0.74780E+00_JPRB, 0.73594E+00_JPRB ,&
&  0.70998E+00_JPRB, 0.69095E+00_JPRB, 0.70521E+00_JPRB, 0.73652E+00_JPRB, 0.78645E+00_JPRB, 0.84667E+00_JPRB, 0.90025E+00_JPRB ,&
&  0.10002E+01_JPRB, 0.12231E+01_JPRB, 0.15317E+01_JPRB, 0.19315E+01_JPRB, 0.20429E+01_JPRB, 0.18002E+01_JPRB, 0.18118E+01_JPRB ,&
&  0.17528E+01_JPRB, 0.16714E+01_JPRB, 0.16140E+01_JPRB, 0.17378E+01_JPRB, 0.17426E+01_JPRB, 0.16353E+01_JPRB, 0.15976E+01_JPRB ,&
&  0.15921E+01_JPRB, 0.16044E+01_JPRB, 0.15625E+01_JPRB, 0.15925E+01_JPRB, 0.16711E+01_JPRB, 0.17648E+01_JPRB, 0.18480E+01_JPRB ,&
&  0.18843E+01_JPRB, 0.18248E+01_JPRB, 0.17198E+01_JPRB, 0.16495E+01_JPRB, 0.16281E+01_JPRB, 0.16560E+01_JPRB, 0.17492E+01_JPRB ,&
&  0.19544E+01_JPRB, 0.24606E+01_JPRB, 0.35572E+01_JPRB, 0.49588E+01_JPRB, 0.48376E+01_JPRB, 0.51836E+01_JPRB, 0.47032E+01_JPRB ,&
&  0.42521E+01_JPRB, 0.42201E+01_JPRB, 0.39920E+01_JPRB, 0.42780E+01_JPRB, 0.37527E+01_JPRB, 0.36075E+01_JPRB, 0.36692E+01_JPRB ,&
&  0.51483E+01_JPRB, 0.62204E+01_JPRB, 0.75832E+01_JPRB, 0.81460E+01_JPRB, 0.10033E+02_JPRB, 0.12929E+02_JPRB, 0.80133E+01_JPRB ,&
&  0.56614E+01_JPRB, 0.51193E+01_JPRB, 0.40291E+01_JPRB, 0.34357E+01_JPRB, 0.40228E+01_JPRB, 0.51109E+01_JPRB, 0.37036E+01_JPRB ,&
&  0.11868E+01_JPRB, 0.73730E+00_JPRB, 0.62672E+00_JPRB, 0.62179E+00_JPRB, 0.52036E+00_JPRB, 0.77462E+00_JPRB, 0.25005E+01_JPRB ,&
&  0.22446E+01_JPRB, 0.29698E+01_JPRB, 0.32945E+01_JPRB, 0.35862E+01_JPRB, 0.35913E+01_JPRB, 0.27447E+01_JPRB, 0.23873E+01_JPRB ,&
&  0.16588E+01_JPRB, 0.12575E+01_JPRB, 0.11031E+01_JPRB, 0.10124E+01_JPRB, 0.89732E+00_JPRB, 0.83727E+00_JPRB, 0.79309E+00_JPRB ,&
&  0.77725E+00_JPRB, 0.76585E+00_JPRB, 0.75407E+00_JPRB, 0.73870E+00_JPRB, 0.73985E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.71714E+00_JPRB, 0.74786E+00_JPRB, 0.78020E+00_JPRB, 0.81150E+00_JPRB, 0.82726E+00_JPRB, 0.81728E+00_JPRB, 0.79721E+00_JPRB ,&
&  0.78655E+00_JPRB, 0.77863E+00_JPRB, 0.79954E+00_JPRB, 0.83629E+00_JPRB, 0.90825E+00_JPRB, 0.99352E+00_JPRB, 0.11190E+01_JPRB ,&
&  0.13151E+01_JPRB, 0.15844E+01_JPRB, 0.19041E+01_JPRB, 0.22170E+01_JPRB, 0.22302E+01_JPRB, 0.18975E+01_JPRB, 0.18394E+01_JPRB ,&
&  0.18480E+01_JPRB, 0.18599E+01_JPRB, 0.17460E+01_JPRB, 0.19185E+01_JPRB, 0.20171E+01_JPRB, 0.20020E+01_JPRB, 0.19564E+01_JPRB ,&
&  0.19575E+01_JPRB, 0.19805E+01_JPRB, 0.20110E+01_JPRB, 0.21038E+01_JPRB, 0.21978E+01_JPRB, 0.22642E+01_JPRB, 0.22860E+01_JPRB ,&
&  0.22059E+01_JPRB, 0.20857E+01_JPRB, 0.20169E+01_JPRB, 0.20096E+01_JPRB, 0.20536E+01_JPRB, 0.21297E+01_JPRB, 0.23166E+01_JPRB ,&
&  0.27289E+01_JPRB, 0.34716E+01_JPRB, 0.44678E+01_JPRB, 0.48963E+01_JPRB, 0.47829E+01_JPRB, 0.46190E+01_JPRB, 0.43324E+01_JPRB ,&
&  0.40991E+01_JPRB, 0.41563E+01_JPRB, 0.40960E+01_JPRB, 0.40632E+01_JPRB, 0.39060E+01_JPRB, 0.40988E+01_JPRB, 0.45278E+01_JPRB ,&
&  0.63792E+01_JPRB, 0.78653E+01_JPRB, 0.82046E+01_JPRB, 0.82200E+01_JPRB, 0.86666E+01_JPRB, 0.10026E+02_JPRB, 0.91840E+01_JPRB ,&
&  0.66675E+01_JPRB, 0.46811E+01_JPRB, 0.37728E+01_JPRB, 0.37123E+01_JPRB, 0.36988E+01_JPRB, 0.39927E+01_JPRB, 0.46501E+01_JPRB ,&
&  0.39479E+01_JPRB, 0.15353E+01_JPRB, 0.96129E+00_JPRB, 0.13038E+01_JPRB, 0.79110E+00_JPRB, 0.80927E+00_JPRB, 0.13727E+01_JPRB ,&
&  0.17719E+01_JPRB, 0.22016E+01_JPRB, 0.24904E+01_JPRB, 0.29254E+01_JPRB, 0.25583E+01_JPRB, 0.19135E+01_JPRB, 0.13845E+01_JPRB ,&
&  0.10353E+01_JPRB, 0.91897E+00_JPRB, 0.90102E+00_JPRB, 0.82824E+00_JPRB, 0.75862E+00_JPRB, 0.73508E+00_JPRB, 0.70700E+00_JPRB ,&
&  0.68830E+00_JPRB, 0.70137E+00_JPRB, 0.71438E+00_JPRB, 0.70978E+00_JPRB, 0.69979E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.76919E+00_JPRB, 0.80906E+00_JPRB, 0.87279E+00_JPRB, 0.89964E+00_JPRB, 0.89444E+00_JPRB, 0.89350E+00_JPRB, 0.90877E+00_JPRB ,&
&  0.92736E+00_JPRB, 0.94914E+00_JPRB, 0.99631E+00_JPRB, 0.10763E+01_JPRB, 0.12091E+01_JPRB, 0.13570E+01_JPRB, 0.15424E+01_JPRB ,&
&  0.17578E+01_JPRB, 0.19878E+01_JPRB, 0.22279E+01_JPRB, 0.24523E+01_JPRB, 0.25820E+01_JPRB, 0.26772E+01_JPRB, 0.17871E+01_JPRB ,&
&  0.16085E+01_JPRB, 0.21047E+01_JPRB, 0.19091E+01_JPRB, 0.21235E+01_JPRB, 0.22624E+01_JPRB, 0.23369E+01_JPRB, 0.22517E+01_JPRB ,&
&  0.22643E+01_JPRB, 0.23188E+01_JPRB, 0.23892E+01_JPRB, 0.24244E+01_JPRB, 0.24416E+01_JPRB, 0.24545E+01_JPRB, 0.24234E+01_JPRB ,&
&  0.23719E+01_JPRB, 0.23461E+01_JPRB, 0.23730E+01_JPRB, 0.24365E+01_JPRB, 0.25443E+01_JPRB, 0.27250E+01_JPRB, 0.30360E+01_JPRB ,&
&  0.34925E+01_JPRB, 0.40482E+01_JPRB, 0.45563E+01_JPRB, 0.44135E+01_JPRB, 0.42794E+01_JPRB, 0.41320E+01_JPRB, 0.40999E+01_JPRB ,&
&  0.39272E+01_JPRB, 0.36814E+01_JPRB, 0.39541E+01_JPRB, 0.42230E+01_JPRB, 0.42427E+01_JPRB, 0.45010E+01_JPRB, 0.52772E+01_JPRB ,&
&  0.72189E+01_JPRB, 0.83505E+01_JPRB, 0.87726E+01_JPRB, 0.74625E+01_JPRB, 0.69678E+01_JPRB, 0.76821E+01_JPRB, 0.78883E+01_JPRB ,&
&  0.66397E+01_JPRB, 0.50065E+01_JPRB, 0.40845E+01_JPRB, 0.35095E+01_JPRB, 0.31507E+01_JPRB, 0.31403E+01_JPRB, 0.33478E+01_JPRB ,&
&  0.34340E+01_JPRB, 0.37665E+01_JPRB, 0.29111E+01_JPRB, 0.16430E+01_JPRB, 0.11417E+01_JPRB, 0.10944E+01_JPRB, 0.13346E+01_JPRB ,&
&  0.15656E+01_JPRB, 0.17095E+01_JPRB, 0.17142E+01_JPRB, 0.24902E+01_JPRB, 0.15208E+01_JPRB, 0.11263E+01_JPRB, 0.87572E+00_JPRB ,&
&  0.80770E+00_JPRB, 0.81265E+00_JPRB, 0.81526E+00_JPRB, 0.76711E+00_JPRB, 0.72080E+00_JPRB, 0.72602E+00_JPRB, 0.71106E+00_JPRB ,&
&  0.72578E+00_JPRB, 0.73777E+00_JPRB, 0.73454E+00_JPRB, 0.73889E+00_JPRB, 0.75647E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.88763E+00_JPRB, 0.93282E+00_JPRB, 0.97919E+00_JPRB, 0.99873E+00_JPRB, 0.10239E+01_JPRB, 0.10568E+01_JPRB, 0.11165E+01_JPRB ,&
&  0.11853E+01_JPRB, 0.12629E+01_JPRB, 0.13592E+01_JPRB, 0.14806E+01_JPRB, 0.16388E+01_JPRB, 0.17920E+01_JPRB, 0.19585E+01_JPRB ,&
&  0.21099E+01_JPRB, 0.22380E+01_JPRB, 0.23626E+01_JPRB, 0.24830E+01_JPRB, 0.25691E+01_JPRB, 0.26928E+01_JPRB, 0.23057E+01_JPRB ,&
&  0.15197E+01_JPRB, 0.24318E+01_JPRB, 0.20203E+01_JPRB, 0.22552E+01_JPRB, 0.23197E+01_JPRB, 0.24757E+01_JPRB, 0.24072E+01_JPRB ,&
&  0.24079E+01_JPRB, 0.24406E+01_JPRB, 0.24558E+01_JPRB, 0.24832E+01_JPRB, 0.24969E+01_JPRB, 0.25128E+01_JPRB, 0.25272E+01_JPRB ,&
&  0.25571E+01_JPRB, 0.26243E+01_JPRB, 0.27246E+01_JPRB, 0.28411E+01_JPRB, 0.30296E+01_JPRB, 0.32881E+01_JPRB, 0.35447E+01_JPRB ,&
&  0.38168E+01_JPRB, 0.40960E+01_JPRB, 0.41704E+01_JPRB, 0.40162E+01_JPRB, 0.39204E+01_JPRB, 0.39955E+01_JPRB, 0.41853E+01_JPRB ,&
&  0.40819E+01_JPRB, 0.38973E+01_JPRB, 0.41247E+01_JPRB, 0.42344E+01_JPRB, 0.43054E+01_JPRB, 0.50103E+01_JPRB, 0.57108E+01_JPRB ,&
&  0.71194E+01_JPRB, 0.73872E+01_JPRB, 0.85362E+01_JPRB, 0.70609E+01_JPRB, 0.57855E+01_JPRB, 0.60129E+01_JPRB, 0.54485E+01_JPRB ,&
&  0.44151E+01_JPRB, 0.40154E+01_JPRB, 0.34446E+01_JPRB, 0.27996E+01_JPRB, 0.26532E+01_JPRB, 0.25512E+01_JPRB, 0.27131E+01_JPRB ,&
&  0.28534E+01_JPRB, 0.28546E+01_JPRB, 0.21069E+01_JPRB, 0.12846E+01_JPRB, 0.11381E+01_JPRB, 0.11658E+01_JPRB, 0.13636E+01_JPRB ,&
&  0.14210E+01_JPRB, 0.14870E+01_JPRB, 0.15932E+01_JPRB, 0.14992E+01_JPRB, 0.96631E+00_JPRB, 0.77530E+00_JPRB, 0.71647E+00_JPRB ,&
&  0.74710E+00_JPRB, 0.78434E+00_JPRB, 0.78366E+00_JPRB, 0.76817E+00_JPRB, 0.73323E+00_JPRB, 0.76472E+00_JPRB, 0.77666E+00_JPRB ,&
&  0.81458E+00_JPRB, 0.81726E+00_JPRB, 0.83905E+00_JPRB, 0.85807E+00_JPRB, 0.86968E+00_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.10811E+01_JPRB, 0.11567E+01_JPRB, 0.12557E+01_JPRB, 0.13483E+01_JPRB, 0.14102E+01_JPRB, 0.14618E+01_JPRB, 0.14963E+01_JPRB ,&
&  0.14982E+01_JPRB, 0.15041E+01_JPRB, 0.16191E+01_JPRB, 0.17341E+01_JPRB, 0.18779E+01_JPRB, 0.20203E+01_JPRB, 0.21563E+01_JPRB ,&
&  0.22654E+01_JPRB, 0.23669E+01_JPRB, 0.24771E+01_JPRB, 0.26285E+01_JPRB, 0.28014E+01_JPRB, 0.30997E+01_JPRB, 0.29468E+01_JPRB ,&
&  0.21497E+01_JPRB, 0.18635E+01_JPRB, 0.21511E+01_JPRB, 0.21979E+01_JPRB, 0.21489E+01_JPRB, 0.23452E+01_JPRB, 0.24883E+01_JPRB ,&
&  0.24401E+01_JPRB, 0.24626E+01_JPRB, 0.24776E+01_JPRB, 0.25072E+01_JPRB, 0.24991E+01_JPRB, 0.25565E+01_JPRB, 0.26315E+01_JPRB ,&
&  0.27419E+01_JPRB, 0.28714E+01_JPRB, 0.30119E+01_JPRB, 0.31827E+01_JPRB, 0.33697E+01_JPRB, 0.35761E+01_JPRB, 0.37578E+01_JPRB ,&
&  0.39074E+01_JPRB, 0.40391E+01_JPRB, 0.40258E+01_JPRB, 0.38661E+01_JPRB, 0.39151E+01_JPRB, 0.40829E+01_JPRB, 0.41619E+01_JPRB ,&
&  0.41562E+01_JPRB, 0.42552E+01_JPRB, 0.44933E+01_JPRB, 0.46208E+01_JPRB, 0.46053E+01_JPRB, 0.53232E+01_JPRB, 0.58340E+01_JPRB ,&
&  0.64739E+01_JPRB, 0.66039E+01_JPRB, 0.77584E+01_JPRB, 0.65152E+01_JPRB, 0.54507E+01_JPRB, 0.50320E+01_JPRB, 0.43318E+01_JPRB ,&
&  0.36770E+01_JPRB, 0.34781E+01_JPRB, 0.31663E+01_JPRB, 0.23337E+01_JPRB, 0.22618E+01_JPRB, 0.19378E+01_JPRB, 0.24005E+01_JPRB ,&
&  0.26682E+01_JPRB, 0.29010E+01_JPRB, 0.17811E+01_JPRB, 0.13237E+01_JPRB, 0.11410E+01_JPRB, 0.12330E+01_JPRB, 0.12830E+01_JPRB ,&
&  0.12426E+01_JPRB, 0.11860E+01_JPRB, 0.11143E+01_JPRB, 0.10722E+01_JPRB, 0.76267E+00_JPRB, 0.68144E+00_JPRB, 0.66759E+00_JPRB ,&
&  0.71884E+00_JPRB, 0.76458E+00_JPRB, 0.77254E+00_JPRB, 0.77484E+00_JPRB, 0.79275E+00_JPRB, 0.84378E+00_JPRB, 0.87754E+00_JPRB ,&
&  0.92048E+00_JPRB, 0.94581E+00_JPRB, 0.97884E+00_JPRB, 0.99658E+00_JPRB, 0.10228E+01_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.11127E+01_JPRB, 0.11786E+01_JPRB, 0.12821E+01_JPRB, 0.13709E+01_JPRB, 0.14332E+01_JPRB, 0.14591E+01_JPRB, 0.14572E+01_JPRB ,&
&  0.14802E+01_JPRB, 0.15359E+01_JPRB, 0.16366E+01_JPRB, 0.17244E+01_JPRB, 0.18254E+01_JPRB, 0.19254E+01_JPRB, 0.20255E+01_JPRB ,&
&  0.21172E+01_JPRB, 0.22009E+01_JPRB, 0.23155E+01_JPRB, 0.24635E+01_JPRB, 0.26419E+01_JPRB, 0.29227E+01_JPRB, 0.29915E+01_JPRB ,&
&  0.23955E+01_JPRB, 0.16133E+01_JPRB, 0.18945E+01_JPRB, 0.19740E+01_JPRB, 0.19505E+01_JPRB, 0.21682E+01_JPRB, 0.23019E+01_JPRB ,&
&  0.22096E+01_JPRB, 0.22637E+01_JPRB, 0.23662E+01_JPRB, 0.24289E+01_JPRB, 0.24843E+01_JPRB, 0.26253E+01_JPRB, 0.27514E+01_JPRB ,&
&  0.28936E+01_JPRB, 0.30222E+01_JPRB, 0.31453E+01_JPRB, 0.32786E+01_JPRB, 0.34022E+01_JPRB, 0.35425E+01_JPRB, 0.37094E+01_JPRB ,&
&  0.38895E+01_JPRB, 0.40212E+01_JPRB, 0.39978E+01_JPRB, 0.39620E+01_JPRB, 0.40156E+01_JPRB, 0.42013E+01_JPRB, 0.43765E+01_JPRB ,&
&  0.44823E+01_JPRB, 0.44850E+01_JPRB, 0.48330E+01_JPRB, 0.53040E+01_JPRB, 0.55252E+01_JPRB, 0.53835E+01_JPRB, 0.57685E+01_JPRB ,&
&  0.60447E+01_JPRB, 0.59864E+01_JPRB, 0.63798E+01_JPRB, 0.70908E+01_JPRB, 0.48390E+01_JPRB, 0.45706E+01_JPRB, 0.36058E+01_JPRB ,&
&  0.37202E+01_JPRB, 0.34460E+01_JPRB, 0.30552E+01_JPRB, 0.25350E+01_JPRB, 0.19659E+01_JPRB, 0.16259E+01_JPRB, 0.19516E+01_JPRB ,&
&  0.23784E+01_JPRB, 0.21257E+01_JPRB, 0.15048E+01_JPRB, 0.12546E+01_JPRB, 0.11903E+01_JPRB, 0.14301E+01_JPRB, 0.14175E+01_JPRB ,&
&  0.12382E+01_JPRB, 0.11426E+01_JPRB, 0.11076E+01_JPRB, 0.13176E+01_JPRB, 0.93885E+00_JPRB, 0.67592E+00_JPRB, 0.63059E+00_JPRB ,&
&  0.68441E+00_JPRB, 0.73869E+00_JPRB, 0.78673E+00_JPRB, 0.82286E+00_JPRB, 0.85684E+00_JPRB, 0.89782E+00_JPRB, 0.92248E+00_JPRB ,&
&  0.97834E+00_JPRB, 0.97969E+00_JPRB, 0.10269E+01_JPRB, 0.10507E+01_JPRB, 0.10830E+01_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.11003E+01_JPRB, 0.11413E+01_JPRB, 0.11838E+01_JPRB, 0.12398E+01_JPRB, 0.12939E+01_JPRB, 0.13221E+01_JPRB, 0.13318E+01_JPRB ,&
&  0.13596E+01_JPRB, 0.14145E+01_JPRB, 0.14764E+01_JPRB, 0.15303E+01_JPRB, 0.16083E+01_JPRB, 0.16787E+01_JPRB, 0.17524E+01_JPRB ,&
&  0.18286E+01_JPRB, 0.19256E+01_JPRB, 0.20627E+01_JPRB, 0.22058E+01_JPRB, 0.23236E+01_JPRB, 0.24707E+01_JPRB, 0.26241E+01_JPRB ,&
&  0.27664E+01_JPRB, 0.24648E+01_JPRB, 0.22818E+01_JPRB, 0.18087E+01_JPRB, 0.13935E+01_JPRB, 0.17702E+01_JPRB, 0.18256E+01_JPRB ,&
&  0.18989E+01_JPRB, 0.19604E+01_JPRB, 0.20271E+01_JPRB, 0.21335E+01_JPRB, 0.23168E+01_JPRB, 0.24831E+01_JPRB, 0.26129E+01_JPRB ,&
&  0.27215E+01_JPRB, 0.28125E+01_JPRB, 0.28871E+01_JPRB, 0.29722E+01_JPRB, 0.30730E+01_JPRB, 0.31699E+01_JPRB, 0.32750E+01_JPRB ,&
&  0.34208E+01_JPRB, 0.36031E+01_JPRB, 0.37268E+01_JPRB, 0.38704E+01_JPRB, 0.39668E+01_JPRB, 0.42389E+01_JPRB, 0.45386E+01_JPRB ,&
&  0.47291E+01_JPRB, 0.48332E+01_JPRB, 0.51741E+01_JPRB, 0.55798E+01_JPRB, 0.56363E+01_JPRB, 0.50221E+01_JPRB, 0.52499E+01_JPRB ,&
&  0.55722E+01_JPRB, 0.54164E+01_JPRB, 0.52840E+01_JPRB, 0.68038E+01_JPRB, 0.41676E+01_JPRB, 0.43599E+01_JPRB, 0.42085E+01_JPRB ,&
&  0.36084E+01_JPRB, 0.33946E+01_JPRB, 0.29402E+01_JPRB, 0.22369E+01_JPRB, 0.17957E+01_JPRB, 0.14439E+01_JPRB, 0.17147E+01_JPRB ,&
&  0.19314E+01_JPRB, 0.15903E+01_JPRB, 0.11950E+01_JPRB, 0.10260E+01_JPRB, 0.11014E+01_JPRB, 0.14541E+01_JPRB, 0.14825E+01_JPRB ,&
&  0.11633E+01_JPRB, 0.11519E+01_JPRB, 0.10656E+01_JPRB, 0.16142E+01_JPRB, 0.14085E+01_JPRB, 0.66937E+00_JPRB, 0.55840E+00_JPRB ,&
&  0.59079E+00_JPRB, 0.65959E+00_JPRB, 0.74273E+00_JPRB, 0.79850E+00_JPRB, 0.81974E+00_JPRB, 0.84790E+00_JPRB, 0.87551E+00_JPRB ,&
&  0.90542E+00_JPRB, 0.95414E+00_JPRB, 0.10146E+01_JPRB, 0.10464E+01_JPRB, 0.10746E+01_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.91653E+00_JPRB, 0.94978E+00_JPRB, 0.97572E+00_JPRB, 0.98656E+00_JPRB, 0.98372E+00_JPRB, 0.97420E+00_JPRB, 0.94803E+00_JPRB ,&
&  0.91862E+00_JPRB, 0.93585E+00_JPRB, 0.99332E+00_JPRB, 0.11208E+01_JPRB, 0.12197E+01_JPRB, 0.12819E+01_JPRB, 0.13582E+01_JPRB ,&
&  0.14351E+01_JPRB, 0.14696E+01_JPRB, 0.14575E+01_JPRB, 0.14847E+01_JPRB, 0.14959E+01_JPRB, 0.15039E+01_JPRB, 0.16957E+01_JPRB ,&
&  0.19911E+01_JPRB, 0.23415E+01_JPRB, 0.26452E+01_JPRB, 0.28837E+01_JPRB, 0.23660E+01_JPRB, 0.17516E+01_JPRB, 0.18925E+01_JPRB ,&
&  0.21273E+01_JPRB, 0.20079E+01_JPRB, 0.17744E+01_JPRB, 0.17929E+01_JPRB, 0.18470E+01_JPRB, 0.19612E+01_JPRB, 0.21104E+01_JPRB ,&
&  0.22155E+01_JPRB, 0.22891E+01_JPRB, 0.23583E+01_JPRB, 0.24187E+01_JPRB, 0.24795E+01_JPRB, 0.25410E+01_JPRB, 0.26180E+01_JPRB ,&
&  0.26815E+01_JPRB, 0.27430E+01_JPRB, 0.29276E+01_JPRB, 0.31896E+01_JPRB, 0.34837E+01_JPRB, 0.38815E+01_JPRB, 0.42275E+01_JPRB ,&
&  0.44393E+01_JPRB, 0.46066E+01_JPRB, 0.51488E+01_JPRB, 0.55017E+01_JPRB, 0.52092E+01_JPRB, 0.47280E+01_JPRB, 0.49326E+01_JPRB ,&
&  0.49760E+01_JPRB, 0.46606E+01_JPRB, 0.36910E+01_JPRB, 0.53407E+01_JPRB, 0.53543E+01_JPRB, 0.45484E+01_JPRB, 0.36848E+01_JPRB ,&
&  0.32503E+01_JPRB, 0.30670E+01_JPRB, 0.27193E+01_JPRB, 0.22414E+01_JPRB, 0.17848E+01_JPRB, 0.18390E+01_JPRB, 0.17451E+01_JPRB ,&
&  0.17198E+01_JPRB, 0.13578E+01_JPRB, 0.10150E+01_JPRB, 0.88655E+00_JPRB, 0.89834E+00_JPRB, 0.12669E+01_JPRB, 0.12349E+01_JPRB ,&
&  0.97853E+00_JPRB, 0.96516E+00_JPRB, 0.92319E+00_JPRB, 0.14450E+01_JPRB, 0.25462E+01_JPRB, 0.70443E+00_JPRB, 0.50145E+00_JPRB ,&
&  0.52059E+00_JPRB, 0.56100E+00_JPRB, 0.60078E+00_JPRB, 0.61490E+00_JPRB, 0.61731E+00_JPRB, 0.64465E+00_JPRB, 0.69088E+00_JPRB ,&
&  0.74336E+00_JPRB, 0.78405E+00_JPRB, 0.80678E+00_JPRB, 0.85016E+00_JPRB, 0.88477E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.68938E+00_JPRB, 0.70767E+00_JPRB, 0.70760E+00_JPRB, 0.71405E+00_JPRB, 0.70755E+00_JPRB, 0.70773E+00_JPRB, 0.68837E+00_JPRB ,&
&  0.67374E+00_JPRB, 0.68552E+00_JPRB, 0.72618E+00_JPRB, 0.80369E+00_JPRB, 0.82621E+00_JPRB, 0.84805E+00_JPRB, 0.85712E+00_JPRB ,&
&  0.88306E+00_JPRB, 0.86180E+00_JPRB, 0.84928E+00_JPRB, 0.81734E+00_JPRB, 0.81731E+00_JPRB, 0.85400E+00_JPRB, 0.94666E+00_JPRB ,&
&  0.10826E+01_JPRB, 0.12546E+01_JPRB, 0.14849E+01_JPRB, 0.17508E+01_JPRB, 0.23365E+01_JPRB, 0.20365E+01_JPRB, 0.18330E+01_JPRB ,&
&  0.20547E+01_JPRB, 0.17108E+01_JPRB, 0.15579E+01_JPRB, 0.14607E+01_JPRB, 0.13932E+01_JPRB, 0.13550E+01_JPRB, 0.14101E+01_JPRB ,&
&  0.14447E+01_JPRB, 0.15070E+01_JPRB, 0.15805E+01_JPRB, 0.16128E+01_JPRB, 0.16484E+01_JPRB, 0.16328E+01_JPRB, 0.16063E+01_JPRB ,&
&  0.15672E+01_JPRB, 0.16701E+01_JPRB, 0.18932E+01_JPRB, 0.20503E+01_JPRB, 0.24631E+01_JPRB, 0.28283E+01_JPRB, 0.31406E+01_JPRB ,&
&  0.33179E+01_JPRB, 0.35577E+01_JPRB, 0.40468E+01_JPRB, 0.41823E+01_JPRB, 0.42691E+01_JPRB, 0.40754E+01_JPRB, 0.39628E+01_JPRB ,&
&  0.37393E+01_JPRB, 0.30089E+01_JPRB, 0.24807E+01_JPRB, 0.37450E+01_JPRB, 0.38417E+01_JPRB, 0.30224E+01_JPRB, 0.31233E+01_JPRB ,&
&  0.28681E+01_JPRB, 0.26912E+01_JPRB, 0.25665E+01_JPRB, 0.21521E+01_JPRB, 0.16784E+01_JPRB, 0.16458E+01_JPRB, 0.17187E+01_JPRB ,&
&  0.14925E+01_JPRB, 0.10971E+01_JPRB, 0.84452E+00_JPRB, 0.78224E+00_JPRB, 0.81528E+00_JPRB, 0.10604E+01_JPRB, 0.11043E+01_JPRB ,&
&  0.10607E+01_JPRB, 0.82394E+00_JPRB, 0.89779E+00_JPRB, 0.13604E+01_JPRB, 0.10880E+01_JPRB, 0.60033E+00_JPRB, 0.44885E+00_JPRB ,&
&  0.45401E+00_JPRB, 0.47418E+00_JPRB, 0.49511E+00_JPRB, 0.50573E+00_JPRB, 0.50998E+00_JPRB, 0.51854E+00_JPRB, 0.55208E+00_JPRB ,&
&  0.58041E+00_JPRB, 0.60750E+00_JPRB, 0.62990E+00_JPRB, 0.66097E+00_JPRB, 0.67905E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.68297E+00_JPRB, 0.69679E+00_JPRB, 0.71604E+00_JPRB, 0.72828E+00_JPRB, 0.74265E+00_JPRB, 0.75267E+00_JPRB, 0.78500E+00_JPRB ,&
&  0.81045E+00_JPRB, 0.83328E+00_JPRB, 0.86485E+00_JPRB, 0.89859E+00_JPRB, 0.91179E+00_JPRB, 0.92519E+00_JPRB, 0.93167E+00_JPRB ,&
&  0.94870E+00_JPRB, 0.96507E+00_JPRB, 0.95871E+00_JPRB, 0.96122E+00_JPRB, 0.96377E+00_JPRB, 0.10333E+01_JPRB, 0.10305E+01_JPRB ,&
&  0.10201E+01_JPRB, 0.10369E+01_JPRB, 0.10507E+01_JPRB, 0.11094E+01_JPRB, 0.12849E+01_JPRB, 0.14360E+01_JPRB, 0.14660E+01_JPRB ,&
&  0.16130E+01_JPRB, 0.13720E+01_JPRB, 0.13462E+01_JPRB, 0.10767E+01_JPRB, 0.10236E+01_JPRB, 0.10442E+01_JPRB, 0.10178E+01_JPRB ,&
&  0.10192E+01_JPRB, 0.10594E+01_JPRB, 0.11349E+01_JPRB, 0.11477E+01_JPRB, 0.12150E+01_JPRB, 0.12710E+01_JPRB, 0.13085E+01_JPRB ,&
&  0.13757E+01_JPRB, 0.14605E+01_JPRB, 0.15513E+01_JPRB, 0.16956E+01_JPRB, 0.18210E+01_JPRB, 0.21738E+01_JPRB, 0.23281E+01_JPRB ,&
&  0.23911E+01_JPRB, 0.25164E+01_JPRB, 0.23451E+01_JPRB, 0.23848E+01_JPRB, 0.27253E+01_JPRB, 0.27268E+01_JPRB, 0.26584E+01_JPRB ,&
&  0.27400E+01_JPRB, 0.24376E+01_JPRB, 0.17939E+01_JPRB, 0.26798E+01_JPRB, 0.29712E+01_JPRB, 0.27054E+01_JPRB, 0.26462E+01_JPRB ,&
&  0.24548E+01_JPRB, 0.21789E+01_JPRB, 0.19762E+01_JPRB, 0.18581E+01_JPRB, 0.15860E+01_JPRB, 0.14097E+01_JPRB, 0.13883E+01_JPRB ,&
&  0.11522E+01_JPRB, 0.92263E+00_JPRB, 0.79332E+00_JPRB, 0.82389E+00_JPRB, 0.10170E+01_JPRB, 0.15127E+01_JPRB, 0.13720E+01_JPRB ,&
&  0.10100E+01_JPRB, 0.80504E+00_JPRB, 0.11147E+01_JPRB, 0.16587E+01_JPRB, 0.91940E+00_JPRB, 0.61197E+00_JPRB, 0.44909E+00_JPRB ,&
&  0.44253E+00_JPRB, 0.46767E+00_JPRB, 0.49371E+00_JPRB, 0.51420E+00_JPRB, 0.53933E+00_JPRB, 0.56173E+00_JPRB, 0.57928E+00_JPRB ,&
&  0.59648E+00_JPRB, 0.61809E+00_JPRB, 0.63681E+00_JPRB, 0.65877E+00_JPRB, 0.67749E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.80831E+00_JPRB, 0.84289E+00_JPRB, 0.86723E+00_JPRB, 0.88723E+00_JPRB, 0.90090E+00_JPRB, 0.91467E+00_JPRB, 0.93656E+00_JPRB ,&
&  0.95601E+00_JPRB, 0.97566E+00_JPRB, 0.99581E+00_JPRB, 0.10230E+01_JPRB, 0.10574E+01_JPRB, 0.10864E+01_JPRB, 0.10992E+01_JPRB ,&
&  0.11058E+01_JPRB, 0.11430E+01_JPRB, 0.11657E+01_JPRB, 0.12125E+01_JPRB, 0.12309E+01_JPRB, 0.12637E+01_JPRB, 0.12709E+01_JPRB ,&
&  0.12668E+01_JPRB, 0.12695E+01_JPRB, 0.12771E+01_JPRB, 0.12957E+01_JPRB, 0.13104E+01_JPRB, 0.14285E+01_JPRB, 0.16041E+01_JPRB ,&
&  0.14345E+01_JPRB, 0.13694E+01_JPRB, 0.11010E+01_JPRB, 0.92053E+00_JPRB, 0.92414E+00_JPRB, 0.95730E+00_JPRB, 0.98288E+00_JPRB ,&
&  0.10157E+01_JPRB, 0.10618E+01_JPRB, 0.11190E+01_JPRB, 0.11492E+01_JPRB, 0.12459E+01_JPRB, 0.14123E+01_JPRB, 0.14826E+01_JPRB ,&
&  0.15931E+01_JPRB, 0.16891E+01_JPRB, 0.17441E+01_JPRB, 0.19123E+01_JPRB, 0.19253E+01_JPRB, 0.23410E+01_JPRB, 0.25465E+01_JPRB ,&
&  0.25990E+01_JPRB, 0.24652E+01_JPRB, 0.18906E+01_JPRB, 0.18905E+01_JPRB, 0.20855E+01_JPRB, 0.20294E+01_JPRB, 0.19609E+01_JPRB ,&
&  0.22879E+01_JPRB, 0.22184E+01_JPRB, 0.19250E+01_JPRB, 0.24226E+01_JPRB, 0.25561E+01_JPRB, 0.24794E+01_JPRB, 0.21754E+01_JPRB ,&
&  0.18934E+01_JPRB, 0.14684E+01_JPRB, 0.14385E+01_JPRB, 0.14449E+01_JPRB, 0.13306E+01_JPRB, 0.11998E+01_JPRB, 0.10552E+01_JPRB ,&
&  0.92239E+00_JPRB, 0.82695E+00_JPRB, 0.80327E+00_JPRB, 0.87876E+00_JPRB, 0.13011E+01_JPRB, 0.14403E+01_JPRB, 0.17674E+01_JPRB ,&
&  0.84718E+00_JPRB, 0.80646E+00_JPRB, 0.66313E+00_JPRB, 0.84240E+00_JPRB, 0.70049E+00_JPRB, 0.50290E+00_JPRB, 0.46854E+00_JPRB ,&
&  0.51382E+00_JPRB, 0.56026E+00_JPRB, 0.59551E+00_JPRB, 0.62138E+00_JPRB, 0.64445E+00_JPRB, 0.67045E+00_JPRB, 0.68344E+00_JPRB ,&
&  0.70140E+00_JPRB, 0.72132E+00_JPRB, 0.74065E+00_JPRB, 0.76875E+00_JPRB, 0.78461E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.86254E+00_JPRB, 0.89071E+00_JPRB, 0.91551E+00_JPRB, 0.93900E+00_JPRB, 0.96405E+00_JPRB, 0.98958E+00_JPRB, 0.10120E+01_JPRB ,&
&  0.10322E+01_JPRB, 0.10547E+01_JPRB, 0.10737E+01_JPRB, 0.10924E+01_JPRB, 0.11187E+01_JPRB, 0.11430E+01_JPRB, 0.11587E+01_JPRB ,&
&  0.11804E+01_JPRB, 0.12120E+01_JPRB, 0.12482E+01_JPRB, 0.12805E+01_JPRB, 0.12975E+01_JPRB, 0.13086E+01_JPRB, 0.13087E+01_JPRB ,&
&  0.12973E+01_JPRB, 0.12928E+01_JPRB, 0.13020E+01_JPRB, 0.13223E+01_JPRB, 0.13585E+01_JPRB, 0.13481E+01_JPRB, 0.14466E+01_JPRB ,&
&  0.12327E+01_JPRB, 0.14496E+01_JPRB, 0.12105E+01_JPRB, 0.10791E+01_JPRB, 0.10992E+01_JPRB, 0.10577E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.11481E+01_JPRB, 0.11838E+01_JPRB, 0.12332E+01_JPRB, 0.12752E+01_JPRB, 0.13670E+01_JPRB, 0.15019E+01_JPRB, 0.16020E+01_JPRB ,&
&  0.17598E+01_JPRB, 0.19372E+01_JPRB, 0.20772E+01_JPRB, 0.22406E+01_JPRB, 0.25896E+01_JPRB, 0.28892E+01_JPRB, 0.30388E+01_JPRB ,&
&  0.30068E+01_JPRB, 0.28607E+01_JPRB, 0.21535E+01_JPRB, 0.22253E+01_JPRB, 0.23618E+01_JPRB, 0.21831E+01_JPRB, 0.19844E+01_JPRB ,&
&  0.20642E+01_JPRB, 0.17213E+01_JPRB, 0.19728E+01_JPRB, 0.20080E+01_JPRB, 0.19153E+01_JPRB, 0.16909E+01_JPRB, 0.13905E+01_JPRB ,&
&  0.11238E+01_JPRB, 0.92208E+00_JPRB, 0.94088E+00_JPRB, 0.10814E+01_JPRB, 0.10336E+01_JPRB, 0.84863E+00_JPRB, 0.77943E+00_JPRB ,&
&  0.71321E+00_JPRB, 0.65624E+00_JPRB, 0.66179E+00_JPRB, 0.78463E+00_JPRB, 0.10501E+01_JPRB, 0.17629E+01_JPRB, 0.17448E+01_JPRB ,&
&  0.80144E+00_JPRB, 0.72289E+00_JPRB, 0.49086E+00_JPRB, 0.54757E+00_JPRB, 0.56271E+00_JPRB, 0.50288E+00_JPRB, 0.53633E+00_JPRB ,&
&  0.64738E+00_JPRB, 0.74707E+00_JPRB, 0.76031E+00_JPRB, 0.70274E+00_JPRB, 0.68512E+00_JPRB, 0.69652E+00_JPRB, 0.71908E+00_JPRB ,&
&  0.74239E+00_JPRB, 0.76421E+00_JPRB, 0.78639E+00_JPRB, 0.81034E+00_JPRB, 0.83509E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.89077E+00_JPRB, 0.92000E+00_JPRB, 0.94502E+00_JPRB, 0.96654E+00_JPRB, 0.99293E+00_JPRB, 0.10210E+01_JPRB, 0.10461E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.10898E+01_JPRB, 0.11127E+01_JPRB, 0.11342E+01_JPRB, 0.11575E+01_JPRB, 0.11840E+01_JPRB, 0.12132E+01_JPRB ,&
&  0.12412E+01_JPRB, 0.12609E+01_JPRB, 0.12991E+01_JPRB, 0.13452E+01_JPRB, 0.13767E+01_JPRB, 0.13915E+01_JPRB, 0.13977E+01_JPRB ,&
&  0.14044E+01_JPRB, 0.14345E+01_JPRB, 0.14555E+01_JPRB, 0.14774E+01_JPRB, 0.15128E+01_JPRB, 0.16126E+01_JPRB, 0.14100E+01_JPRB ,&
&  0.15623E+01_JPRB, 0.14475E+01_JPRB, 0.12680E+01_JPRB, 0.12111E+01_JPRB, 0.12046E+01_JPRB, 0.12083E+01_JPRB, 0.12262E+01_JPRB ,&
&  0.12742E+01_JPRB, 0.12796E+01_JPRB, 0.13095E+01_JPRB, 0.13482E+01_JPRB, 0.14362E+01_JPRB, 0.15569E+01_JPRB, 0.16859E+01_JPRB ,&
&  0.18918E+01_JPRB, 0.21526E+01_JPRB, 0.24562E+01_JPRB, 0.27869E+01_JPRB, 0.31793E+01_JPRB, 0.35118E+01_JPRB, 0.37229E+01_JPRB ,&
&  0.37675E+01_JPRB, 0.37236E+01_JPRB, 0.31490E+01_JPRB, 0.33291E+01_JPRB, 0.32821E+01_JPRB, 0.27025E+01_JPRB, 0.22021E+01_JPRB ,&
&  0.18070E+01_JPRB, 0.13881E+01_JPRB, 0.14540E+01_JPRB, 0.14787E+01_JPRB, 0.13404E+01_JPRB, 0.11256E+01_JPRB, 0.92560E+00_JPRB ,&
&  0.70382E+00_JPRB, 0.62846E+00_JPRB, 0.62708E+00_JPRB, 0.74559E+00_JPRB, 0.73979E+00_JPRB, 0.64073E+00_JPRB, 0.62420E+00_JPRB ,&
&  0.60486E+00_JPRB, 0.57589E+00_JPRB, 0.59744E+00_JPRB, 0.71590E+00_JPRB, 0.91486E+00_JPRB, 0.92067E+00_JPRB, 0.97843E+00_JPRB ,&
&  0.71562E+00_JPRB, 0.62117E+00_JPRB, 0.60911E+00_JPRB, 0.58444E+00_JPRB, 0.62422E+00_JPRB, 0.64880E+00_JPRB, 0.74927E+00_JPRB ,&
&  0.91091E+00_JPRB, 0.11224E+01_JPRB, 0.10939E+01_JPRB, 0.88695E+00_JPRB, 0.73186E+00_JPRB, 0.70450E+00_JPRB, 0.72496E+00_JPRB ,&
&  0.75050E+00_JPRB, 0.77252E+00_JPRB, 0.79891E+00_JPRB, 0.82767E+00_JPRB, 0.85821E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.89810E+00_JPRB, 0.92522E+00_JPRB, 0.94682E+00_JPRB, 0.96429E+00_JPRB, 0.98898E+00_JPRB, 0.10147E+01_JPRB, 0.10399E+01_JPRB ,&
&  0.10631E+01_JPRB, 0.10840E+01_JPRB, 0.11085E+01_JPRB, 0.11310E+01_JPRB, 0.11568E+01_JPRB, 0.11853E+01_JPRB, 0.12173E+01_JPRB ,&
&  0.12456E+01_JPRB, 0.12759E+01_JPRB, 0.12972E+01_JPRB, 0.13486E+01_JPRB, 0.14110E+01_JPRB, 0.14505E+01_JPRB, 0.14753E+01_JPRB ,&
&  0.14815E+01_JPRB, 0.15124E+01_JPRB, 0.15478E+01_JPRB, 0.15849E+01_JPRB, 0.15990E+01_JPRB, 0.16788E+01_JPRB, 0.14910E+01_JPRB ,&
&  0.15410E+01_JPRB, 0.14477E+01_JPRB, 0.13204E+01_JPRB, 0.13059E+01_JPRB, 0.12987E+01_JPRB, 0.13072E+01_JPRB, 0.13146E+01_JPRB ,&
&  0.13229E+01_JPRB, 0.13540E+01_JPRB, 0.13635E+01_JPRB, 0.13954E+01_JPRB, 0.14637E+01_JPRB, 0.15857E+01_JPRB, 0.17365E+01_JPRB ,&
&  0.19455E+01_JPRB, 0.22286E+01_JPRB, 0.25721E+01_JPRB, 0.29502E+01_JPRB, 0.33808E+01_JPRB, 0.37702E+01_JPRB, 0.41377E+01_JPRB ,&
&  0.44137E+01_JPRB, 0.45731E+01_JPRB, 0.41919E+01_JPRB, 0.41594E+01_JPRB, 0.39173E+01_JPRB, 0.26811E+01_JPRB, 0.19203E+01_JPRB ,&
&  0.13464E+01_JPRB, 0.12695E+01_JPRB, 0.13051E+01_JPRB, 0.12164E+01_JPRB, 0.96595E+00_JPRB, 0.82440E+00_JPRB, 0.66726E+00_JPRB ,&
&  0.54366E+00_JPRB, 0.49934E+00_JPRB, 0.50738E+00_JPRB, 0.56907E+00_JPRB, 0.55582E+00_JPRB, 0.51603E+00_JPRB, 0.53548E+00_JPRB ,&
&  0.53709E+00_JPRB, 0.56055E+00_JPRB, 0.60837E+00_JPRB, 0.69245E+00_JPRB, 0.77996E+00_JPRB, 0.78027E+00_JPRB, 0.82920E+00_JPRB ,&
&  0.74369E+00_JPRB, 0.72838E+00_JPRB, 0.68859E+00_JPRB, 0.55605E+00_JPRB, 0.72907E+00_JPRB, 0.83012E+00_JPRB, 0.96218E+00_JPRB ,&
&  0.10977E+01_JPRB, 0.14954E+01_JPRB, 0.17363E+01_JPRB, 0.13580E+01_JPRB, 0.10506E+01_JPRB, 0.76499E+00_JPRB, 0.73588E+00_JPRB ,&
&  0.76014E+00_JPRB, 0.77031E+00_JPRB, 0.79646E+00_JPRB, 0.82922E+00_JPRB, 0.86487E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.87600E+00_JPRB, 0.90657E+00_JPRB, 0.92332E+00_JPRB, 0.92697E+00_JPRB, 0.94761E+00_JPRB, 0.98191E+00_JPRB, 0.10004E+01_JPRB ,&
&  0.10209E+01_JPRB, 0.10307E+01_JPRB, 0.10535E+01_JPRB, 0.10756E+01_JPRB, 0.10985E+01_JPRB, 0.11230E+01_JPRB, 0.11501E+01_JPRB ,&
&  0.11718E+01_JPRB, 0.12200E+01_JPRB, 0.12485E+01_JPRB, 0.13033E+01_JPRB, 0.13620E+01_JPRB, 0.14401E+01_JPRB, 0.14853E+01_JPRB ,&
&  0.14957E+01_JPRB, 0.15382E+01_JPRB, 0.15866E+01_JPRB, 0.16450E+01_JPRB, 0.16432E+01_JPRB, 0.16575E+01_JPRB, 0.13167E+01_JPRB ,&
&  0.14410E+01_JPRB, 0.14064E+01_JPRB, 0.13534E+01_JPRB, 0.13417E+01_JPRB, 0.13255E+01_JPRB, 0.13053E+01_JPRB, 0.12838E+01_JPRB ,&
&  0.12952E+01_JPRB, 0.12996E+01_JPRB, 0.12781E+01_JPRB, 0.13752E+01_JPRB, 0.14499E+01_JPRB, 0.15554E+01_JPRB, 0.17075E+01_JPRB ,&
&  0.18788E+01_JPRB, 0.21271E+01_JPRB, 0.24505E+01_JPRB, 0.28413E+01_JPRB, 0.32711E+01_JPRB, 0.37020E+01_JPRB, 0.41297E+01_JPRB ,&
&  0.45270E+01_JPRB, 0.48419E+01_JPRB, 0.48966E+01_JPRB, 0.40083E+01_JPRB, 0.33327E+01_JPRB, 0.23181E+01_JPRB, 0.17274E+01_JPRB ,&
&  0.14031E+01_JPRB, 0.12605E+01_JPRB, 0.12041E+01_JPRB, 0.10876E+01_JPRB, 0.83174E+00_JPRB, 0.69095E+00_JPRB, 0.56260E+00_JPRB ,&
&  0.51201E+00_JPRB, 0.47525E+00_JPRB, 0.47470E+00_JPRB, 0.47470E+00_JPRB, 0.47515E+00_JPRB, 0.46961E+00_JPRB, 0.48927E+00_JPRB ,&
&  0.50582E+00_JPRB, 0.54591E+00_JPRB, 0.62765E+00_JPRB, 0.73576E+00_JPRB, 0.83523E+00_JPRB, 0.88492E+00_JPRB, 0.79749E+00_JPRB ,&
&  0.77728E+00_JPRB, 0.71195E+00_JPRB, 0.69191E+00_JPRB, 0.73416E+00_JPRB, 0.80605E+00_JPRB, 0.97919E+00_JPRB, 0.11869E+01_JPRB ,&
&  0.14425E+01_JPRB, 0.14868E+01_JPRB, 0.17825E+01_JPRB, 0.23367E+01_JPRB, 0.21078E+01_JPRB, 0.18015E+01_JPRB, 0.69149E+00_JPRB ,&
&  0.73204E+00_JPRB, 0.74508E+00_JPRB, 0.77963E+00_JPRB, 0.81374E+00_JPRB, 0.85622E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.71682E+00_JPRB, 0.74714E+00_JPRB, 0.77087E+00_JPRB, 0.77707E+00_JPRB, 0.81108E+00_JPRB, 0.84108E+00_JPRB, 0.86044E+00_JPRB ,&
&  0.87861E+00_JPRB, 0.90680E+00_JPRB, 0.92672E+00_JPRB, 0.95828E+00_JPRB, 0.98246E+00_JPRB, 0.10063E+01_JPRB, 0.10283E+01_JPRB ,&
&  0.10498E+01_JPRB, 0.10877E+01_JPRB, 0.11290E+01_JPRB, 0.11886E+01_JPRB, 0.12489E+01_JPRB, 0.13243E+01_JPRB, 0.13925E+01_JPRB ,&
&  0.14431E+01_JPRB, 0.15005E+01_JPRB, 0.15680E+01_JPRB, 0.16108E+01_JPRB, 0.15947E+01_JPRB, 0.15667E+01_JPRB, 0.11799E+01_JPRB ,&
&  0.13389E+01_JPRB, 0.13602E+01_JPRB, 0.13553E+01_JPRB, 0.13341E+01_JPRB, 0.12996E+01_JPRB, 0.12557E+01_JPRB, 0.12381E+01_JPRB ,&
&  0.12324E+01_JPRB, 0.12165E+01_JPRB, 0.12014E+01_JPRB, 0.12117E+01_JPRB, 0.13959E+01_JPRB, 0.14984E+01_JPRB, 0.16324E+01_JPRB ,&
&  0.18106E+01_JPRB, 0.20284E+01_JPRB, 0.22948E+01_JPRB, 0.26793E+01_JPRB, 0.31045E+01_JPRB, 0.35271E+01_JPRB, 0.39642E+01_JPRB ,&
&  0.43064E+01_JPRB, 0.44843E+01_JPRB, 0.45331E+01_JPRB, 0.35964E+01_JPRB, 0.29451E+01_JPRB, 0.22783E+01_JPRB, 0.16011E+01_JPRB ,&
&  0.13166E+01_JPRB, 0.12109E+01_JPRB, 0.11659E+01_JPRB, 0.10224E+01_JPRB, 0.80277E+00_JPRB, 0.64340E+00_JPRB, 0.53366E+00_JPRB ,&
&  0.49043E+00_JPRB, 0.45191E+00_JPRB, 0.44958E+00_JPRB, 0.45314E+00_JPRB, 0.46550E+00_JPRB, 0.47427E+00_JPRB, 0.49160E+00_JPRB ,&
&  0.51640E+00_JPRB, 0.55640E+00_JPRB, 0.62466E+00_JPRB, 0.73279E+00_JPRB, 0.88218E+00_JPRB, 0.10461E+01_JPRB, 0.10076E+01_JPRB ,&
&  0.80826E+00_JPRB, 0.69572E+00_JPRB, 0.66881E+00_JPRB, 0.66343E+00_JPRB, 0.68863E+00_JPRB, 0.74749E+00_JPRB, 0.82100E+00_JPRB ,&
&  0.94940E+00_JPRB, 0.10233E+01_JPRB, 0.92509E+00_JPRB, 0.77420E+00_JPRB, 0.10418E+01_JPRB, 0.10283E+01_JPRB, 0.56257E+00_JPRB ,&
&  0.61362E+00_JPRB, 0.60317E+00_JPRB, 0.62949E+00_JPRB, 0.66088E+00_JPRB, 0.69948E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.54992E+00_JPRB, 0.59931E+00_JPRB, 0.62121E+00_JPRB, 0.64890E+00_JPRB, 0.67051E+00_JPRB, 0.70100E+00_JPRB, 0.72907E+00_JPRB ,&
&  0.75607E+00_JPRB, 0.78801E+00_JPRB, 0.80836E+00_JPRB, 0.84003E+00_JPRB, 0.86532E+00_JPRB, 0.88671E+00_JPRB, 0.90633E+00_JPRB ,&
&  0.92557E+00_JPRB, 0.95999E+00_JPRB, 0.99541E+00_JPRB, 0.10375E+01_JPRB, 0.10809E+01_JPRB, 0.11389E+01_JPRB, 0.12207E+01_JPRB ,&
&  0.13199E+01_JPRB, 0.14151E+01_JPRB, 0.14970E+01_JPRB, 0.15630E+01_JPRB, 0.15958E+01_JPRB, 0.16554E+01_JPRB, 0.16471E+01_JPRB ,&
&  0.11759E+01_JPRB, 0.13977E+01_JPRB, 0.13543E+01_JPRB, 0.12977E+01_JPRB, 0.12562E+01_JPRB, 0.12201E+01_JPRB, 0.12170E+01_JPRB ,&
&  0.12104E+01_JPRB, 0.11452E+01_JPRB, 0.11483E+01_JPRB, 0.11700E+01_JPRB, 0.12794E+01_JPRB, 0.13839E+01_JPRB, 0.15221E+01_JPRB ,&
&  0.17045E+01_JPRB, 0.19332E+01_JPRB, 0.21813E+01_JPRB, 0.24494E+01_JPRB, 0.28111E+01_JPRB, 0.32585E+01_JPRB, 0.36671E+01_JPRB ,&
&  0.40968E+01_JPRB, 0.45341E+01_JPRB, 0.47986E+01_JPRB, 0.40255E+01_JPRB, 0.32752E+01_JPRB, 0.28913E+01_JPRB, 0.20745E+01_JPRB ,&
&  0.13524E+01_JPRB, 0.10837E+01_JPRB, 0.10701E+01_JPRB, 0.10076E+01_JPRB, 0.79409E+00_JPRB, 0.65377E+00_JPRB, 0.55857E+00_JPRB ,&
&  0.51607E+00_JPRB, 0.47498E+00_JPRB, 0.46180E+00_JPRB, 0.46539E+00_JPRB, 0.47615E+00_JPRB, 0.48960E+00_JPRB, 0.51998E+00_JPRB ,&
&  0.54177E+00_JPRB, 0.56238E+00_JPRB, 0.59078E+00_JPRB, 0.62054E+00_JPRB, 0.67589E+00_JPRB, 0.76377E+00_JPRB, 0.77306E+00_JPRB ,&
&  0.73648E+00_JPRB, 0.69854E+00_JPRB, 0.66469E+00_JPRB, 0.65853E+00_JPRB, 0.64643E+00_JPRB, 0.64243E+00_JPRB, 0.65708E+00_JPRB ,&
&  0.68099E+00_JPRB, 0.69749E+00_JPRB, 0.65999E+00_JPRB, 0.63189E+00_JPRB, 0.62749E+00_JPRB, 0.59730E+00_JPRB, 0.54341E+00_JPRB ,&
&  0.52319E+00_JPRB, 0.50555E+00_JPRB, 0.48769E+00_JPRB, 0.49538E+00_JPRB, 0.51928E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.46582E+00_JPRB, 0.48511E+00_JPRB, 0.50773E+00_JPRB, 0.53015E+00_JPRB, 0.55100E+00_JPRB, 0.57044E+00_JPRB, 0.60063E+00_JPRB ,&
&  0.63450E+00_JPRB, 0.65881E+00_JPRB, 0.67052E+00_JPRB, 0.69350E+00_JPRB, 0.72019E+00_JPRB, 0.75281E+00_JPRB, 0.77982E+00_JPRB ,&
&  0.80905E+00_JPRB, 0.84888E+00_JPRB, 0.88245E+00_JPRB, 0.91821E+00_JPRB, 0.95469E+00_JPRB, 0.99276E+00_JPRB, 0.10494E+01_JPRB ,&
&  0.11228E+01_JPRB, 0.12193E+01_JPRB, 0.13339E+01_JPRB, 0.14423E+01_JPRB, 0.15375E+01_JPRB, 0.16365E+01_JPRB, 0.16571E+01_JPRB ,&
&  0.10321E+01_JPRB, 0.13118E+01_JPRB, 0.14775E+01_JPRB, 0.13431E+01_JPRB, 0.12374E+01_JPRB, 0.11811E+01_JPRB, 0.11628E+01_JPRB ,&
&  0.11218E+01_JPRB, 0.10611E+01_JPRB, 0.10578E+01_JPRB, 0.11572E+01_JPRB, 0.11603E+01_JPRB, 0.12347E+01_JPRB, 0.13336E+01_JPRB ,&
&  0.14805E+01_JPRB, 0.16636E+01_JPRB, 0.18326E+01_JPRB, 0.20330E+01_JPRB, 0.23220E+01_JPRB, 0.26857E+01_JPRB, 0.31327E+01_JPRB ,&
&  0.36967E+01_JPRB, 0.43509E+01_JPRB, 0.49598E+01_JPRB, 0.39612E+01_JPRB, 0.35764E+01_JPRB, 0.33540E+01_JPRB, 0.28418E+01_JPRB ,&
&  0.16370E+01_JPRB, 0.10887E+01_JPRB, 0.10643E+01_JPRB, 0.10100E+01_JPRB, 0.84028E+00_JPRB, 0.72600E+00_JPRB, 0.64597E+00_JPRB ,&
&  0.59173E+00_JPRB, 0.54940E+00_JPRB, 0.52764E+00_JPRB, 0.52007E+00_JPRB, 0.52271E+00_JPRB, 0.54347E+00_JPRB, 0.56740E+00_JPRB ,&
&  0.58748E+00_JPRB, 0.59041E+00_JPRB, 0.61633E+00_JPRB, 0.62847E+00_JPRB, 0.66628E+00_JPRB, 0.69791E+00_JPRB, 0.71767E+00_JPRB ,&
&  0.72466E+00_JPRB, 0.71531E+00_JPRB, 0.70857E+00_JPRB, 0.70335E+00_JPRB, 0.70103E+00_JPRB, 0.69698E+00_JPRB, 0.69107E+00_JPRB ,&
&  0.68297E+00_JPRB, 0.69316E+00_JPRB, 0.68101E+00_JPRB, 0.66973E+00_JPRB, 0.67179E+00_JPRB, 0.64162E+00_JPRB, 0.62482E+00_JPRB ,&
&  0.60355E+00_JPRB, 0.55441E+00_JPRB, 0.51900E+00_JPRB, 0.49195E+00_JPRB, 0.47219E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.49225E+00_JPRB, 0.46479E+00_JPRB, 0.46715E+00_JPRB, 0.47172E+00_JPRB, 0.48853E+00_JPRB, 0.50543E+00_JPRB, 0.50100E+00_JPRB ,&
&  0.50892E+00_JPRB, 0.52030E+00_JPRB, 0.53629E+00_JPRB, 0.55533E+00_JPRB, 0.57905E+00_JPRB, 0.60461E+00_JPRB, 0.62739E+00_JPRB ,&
&  0.65838E+00_JPRB, 0.69522E+00_JPRB, 0.73597E+00_JPRB, 0.77517E+00_JPRB, 0.81679E+00_JPRB, 0.84989E+00_JPRB, 0.88155E+00_JPRB ,&
&  0.92105E+00_JPRB, 0.98623E+00_JPRB, 0.10799E+01_JPRB, 0.11941E+01_JPRB, 0.13395E+01_JPRB, 0.15107E+01_JPRB, 0.16185E+01_JPRB ,&
&  0.11850E+01_JPRB, 0.66782E+00_JPRB, 0.13524E+01_JPRB, 0.14992E+01_JPRB, 0.13130E+01_JPRB, 0.12302E+01_JPRB, 0.11393E+01_JPRB ,&
&  0.10783E+01_JPRB, 0.10790E+01_JPRB, 0.10370E+01_JPRB, 0.10704E+01_JPRB, 0.10623E+01_JPRB, 0.10947E+01_JPRB, 0.11433E+01_JPRB ,&
&  0.12030E+01_JPRB, 0.13203E+01_JPRB, 0.14924E+01_JPRB, 0.15988E+01_JPRB, 0.17564E+01_JPRB, 0.20304E+01_JPRB, 0.24381E+01_JPRB ,&
&  0.30104E+01_JPRB, 0.37127E+01_JPRB, 0.44221E+01_JPRB, 0.34203E+01_JPRB, 0.26333E+01_JPRB, 0.20683E+01_JPRB, 0.16790E+01_JPRB ,&
&  0.14403E+01_JPRB, 0.11194E+01_JPRB, 0.10521E+01_JPRB, 0.10066E+01_JPRB, 0.89577E+00_JPRB, 0.76481E+00_JPRB, 0.73860E+00_JPRB ,&
&  0.68353E+00_JPRB, 0.64338E+00_JPRB, 0.62049E+00_JPRB, 0.60447E+00_JPRB, 0.60099E+00_JPRB, 0.61470E+00_JPRB, 0.62808E+00_JPRB ,&
&  0.64481E+00_JPRB, 0.65466E+00_JPRB, 0.67449E+00_JPRB, 0.69818E+00_JPRB, 0.70507E+00_JPRB, 0.70737E+00_JPRB, 0.71847E+00_JPRB ,&
&  0.71751E+00_JPRB, 0.72665E+00_JPRB, 0.73738E+00_JPRB, 0.74006E+00_JPRB, 0.74268E+00_JPRB, 0.73371E+00_JPRB, 0.73941E+00_JPRB ,&
&  0.74007E+00_JPRB, 0.74078E+00_JPRB, 0.73076E+00_JPRB, 0.73522E+00_JPRB, 0.74062E+00_JPRB, 0.73714E+00_JPRB, 0.75185E+00_JPRB ,&
&  0.74213E+00_JPRB, 0.71053E+00_JPRB, 0.64594E+00_JPRB, 0.57367E+00_JPRB, 0.53541E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.59865E+00_JPRB, 0.55467E+00_JPRB, 0.51836E+00_JPRB, 0.49900E+00_JPRB, 0.48607E+00_JPRB, 0.48085E+00_JPRB, 0.47928E+00_JPRB ,&
&  0.48317E+00_JPRB, 0.48776E+00_JPRB, 0.47604E+00_JPRB, 0.48166E+00_JPRB, 0.49618E+00_JPRB, 0.51340E+00_JPRB, 0.53567E+00_JPRB ,&
&  0.55076E+00_JPRB, 0.56473E+00_JPRB, 0.58041E+00_JPRB, 0.60588E+00_JPRB, 0.63913E+00_JPRB, 0.67636E+00_JPRB, 0.71227E+00_JPRB ,&
&  0.74808E+00_JPRB, 0.78743E+00_JPRB, 0.84292E+00_JPRB, 0.92012E+00_JPRB, 0.10293E+01_JPRB, 0.11871E+01_JPRB, 0.13604E+01_JPRB ,&
&  0.15839E+01_JPRB, 0.11746E+01_JPRB, 0.84109E+00_JPRB, 0.13553E+01_JPRB, 0.14367E+01_JPRB, 0.13622E+01_JPRB, 0.11732E+01_JPRB ,&
&  0.11633E+01_JPRB, 0.11692E+01_JPRB, 0.10931E+01_JPRB, 0.10503E+01_JPRB, 0.97781E+00_JPRB, 0.98207E+00_JPRB, 0.99688E+00_JPRB ,&
&  0.99840E+00_JPRB, 0.10226E+01_JPRB, 0.11370E+01_JPRB, 0.12204E+01_JPRB, 0.12895E+01_JPRB, 0.14650E+01_JPRB, 0.18268E+01_JPRB ,&
&  0.23235E+01_JPRB, 0.29741E+01_JPRB, 0.37129E+01_JPRB, 0.31141E+01_JPRB, 0.22999E+01_JPRB, 0.17014E+01_JPRB, 0.14751E+01_JPRB ,&
&  0.13431E+01_JPRB, 0.12513E+01_JPRB, 0.11312E+01_JPRB, 0.10207E+01_JPRB, 0.91210E+00_JPRB, 0.81251E+00_JPRB, 0.79405E+00_JPRB ,&
&  0.76287E+00_JPRB, 0.71476E+00_JPRB, 0.68190E+00_JPRB, 0.65400E+00_JPRB, 0.64843E+00_JPRB, 0.65808E+00_JPRB, 0.66809E+00_JPRB ,&
&  0.67839E+00_JPRB, 0.69110E+00_JPRB, 0.71329E+00_JPRB, 0.74077E+00_JPRB, 0.74763E+00_JPRB, 0.74035E+00_JPRB, 0.73659E+00_JPRB ,&
&  0.73118E+00_JPRB, 0.72992E+00_JPRB, 0.74429E+00_JPRB, 0.75045E+00_JPRB, 0.74760E+00_JPRB, 0.73444E+00_JPRB, 0.74662E+00_JPRB ,&
&  0.74789E+00_JPRB, 0.75782E+00_JPRB, 0.75417E+00_JPRB, 0.76457E+00_JPRB, 0.76974E+00_JPRB, 0.79295E+00_JPRB, 0.81795E+00_JPRB ,&
&  0.83843E+00_JPRB, 0.83215E+00_JPRB, 0.79474E+00_JPRB, 0.68170E+00_JPRB, 0.63646E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.64664E+00_JPRB, 0.61789E+00_JPRB, 0.59580E+00_JPRB, 0.57529E+00_JPRB, 0.55400E+00_JPRB, 0.53279E+00_JPRB, 0.51594E+00_JPRB ,&
&  0.49648E+00_JPRB, 0.49366E+00_JPRB, 0.48973E+00_JPRB, 0.48424E+00_JPRB, 0.47755E+00_JPRB, 0.47931E+00_JPRB, 0.48798E+00_JPRB ,&
&  0.49483E+00_JPRB, 0.49917E+00_JPRB, 0.50506E+00_JPRB, 0.50789E+00_JPRB, 0.51334E+00_JPRB, 0.52685E+00_JPRB, 0.54907E+00_JPRB ,&
&  0.58099E+00_JPRB, 0.61483E+00_JPRB, 0.65783E+00_JPRB, 0.70864E+00_JPRB, 0.75996E+00_JPRB, 0.84040E+00_JPRB, 0.94252E+00_JPRB ,&
&  0.11712E+01_JPRB, 0.11885E+01_JPRB, 0.72882E+00_JPRB, 0.12143E+01_JPRB, 0.15213E+01_JPRB, 0.14447E+01_JPRB, 0.13540E+01_JPRB ,&
&  0.13444E+01_JPRB, 0.12658E+01_JPRB, 0.11819E+01_JPRB, 0.10546E+01_JPRB, 0.93351E+00_JPRB, 0.90759E+00_JPRB, 0.90925E+00_JPRB ,&
&  0.88015E+00_JPRB, 0.86138E+00_JPRB, 0.88702E+00_JPRB, 0.91751E+00_JPRB, 0.95111E+00_JPRB, 0.10276E+01_JPRB, 0.12756E+01_JPRB ,&
&  0.16821E+01_JPRB, 0.22974E+01_JPRB, 0.29947E+01_JPRB, 0.28400E+01_JPRB, 0.21140E+01_JPRB, 0.17738E+01_JPRB, 0.16116E+01_JPRB ,&
&  0.15554E+01_JPRB, 0.14370E+01_JPRB, 0.11844E+01_JPRB, 0.10290E+01_JPRB, 0.91529E+00_JPRB, 0.82877E+00_JPRB, 0.82612E+00_JPRB ,&
&  0.80286E+00_JPRB, 0.75969E+00_JPRB, 0.73460E+00_JPRB, 0.69982E+00_JPRB, 0.68233E+00_JPRB, 0.67642E+00_JPRB, 0.68244E+00_JPRB ,&
&  0.69148E+00_JPRB, 0.70009E+00_JPRB, 0.71391E+00_JPRB, 0.73270E+00_JPRB, 0.74575E+00_JPRB, 0.75033E+00_JPRB, 0.74618E+00_JPRB ,&
&  0.74271E+00_JPRB, 0.74425E+00_JPRB, 0.75089E+00_JPRB, 0.75312E+00_JPRB, 0.74939E+00_JPRB, 0.74378E+00_JPRB, 0.73520E+00_JPRB ,&
&  0.73137E+00_JPRB, 0.73172E+00_JPRB, 0.72642E+00_JPRB, 0.71970E+00_JPRB, 0.75123E+00_JPRB, 0.76195E+00_JPRB, 0.77038E+00_JPRB ,&
&  0.81066E+00_JPRB, 0.84297E+00_JPRB, 0.83876E+00_JPRB, 0.72587E+00_JPRB, 0.68047E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.66224E+00_JPRB, 0.63885E+00_JPRB, 0.62068E+00_JPRB, 0.60668E+00_JPRB, 0.58934E+00_JPRB, 0.57211E+00_JPRB, 0.55773E+00_JPRB ,&
&  0.53791E+00_JPRB, 0.52582E+00_JPRB, 0.51747E+00_JPRB, 0.50869E+00_JPRB, 0.50011E+00_JPRB, 0.49202E+00_JPRB, 0.48531E+00_JPRB ,&
&  0.47924E+00_JPRB, 0.47439E+00_JPRB, 0.47158E+00_JPRB, 0.46579E+00_JPRB, 0.46046E+00_JPRB, 0.45903E+00_JPRB, 0.46352E+00_JPRB ,&
&  0.48201E+00_JPRB, 0.50904E+00_JPRB, 0.54085E+00_JPRB, 0.57348E+00_JPRB, 0.60138E+00_JPRB, 0.63926E+00_JPRB, 0.68635E+00_JPRB ,&
&  0.87358E+00_JPRB, 0.97257E+00_JPRB, 0.11513E+01_JPRB, 0.18062E+01_JPRB, 0.18414E+01_JPRB, 0.16483E+01_JPRB, 0.16232E+01_JPRB ,&
&  0.16165E+01_JPRB, 0.14992E+01_JPRB, 0.14030E+01_JPRB, 0.11591E+01_JPRB, 0.97303E+00_JPRB, 0.88183E+00_JPRB, 0.83860E+00_JPRB ,&
&  0.79386E+00_JPRB, 0.75538E+00_JPRB, 0.75442E+00_JPRB, 0.74913E+00_JPRB, 0.76214E+00_JPRB, 0.85629E+00_JPRB, 0.98423E+00_JPRB ,&
&  0.12465E+01_JPRB, 0.16993E+01_JPRB, 0.23308E+01_JPRB, 0.27485E+01_JPRB, 0.21264E+01_JPRB, 0.19583E+01_JPRB, 0.19850E+01_JPRB ,&
&  0.23958E+01_JPRB, 0.18464E+01_JPRB, 0.13425E+01_JPRB, 0.10677E+01_JPRB, 0.91586E+00_JPRB, 0.85880E+00_JPRB, 0.81443E+00_JPRB ,&
&  0.79148E+00_JPRB, 0.76722E+00_JPRB, 0.74895E+00_JPRB, 0.72707E+00_JPRB, 0.71225E+00_JPRB, 0.69880E+00_JPRB, 0.69286E+00_JPRB ,&
&  0.69315E+00_JPRB, 0.69623E+00_JPRB, 0.70039E+00_JPRB, 0.70974E+00_JPRB, 0.72654E+00_JPRB, 0.73941E+00_JPRB, 0.74309E+00_JPRB ,&
&  0.74395E+00_JPRB, 0.74863E+00_JPRB, 0.73078E+00_JPRB, 0.72958E+00_JPRB, 0.73575E+00_JPRB, 0.73103E+00_JPRB, 0.70569E+00_JPRB ,&
&  0.69532E+00_JPRB, 0.70748E+00_JPRB, 0.70977E+00_JPRB, 0.70380E+00_JPRB, 0.71413E+00_JPRB, 0.72476E+00_JPRB, 0.70577E+00_JPRB ,&
&  0.71078E+00_JPRB, 0.71960E+00_JPRB, 0.72408E+00_JPRB, 0.70687E+00_JPRB, 0.68699E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.67061E+00_JPRB, 0.65165E+00_JPRB, 0.63227E+00_JPRB, 0.61334E+00_JPRB, 0.59264E+00_JPRB, 0.57654E+00_JPRB, 0.56759E+00_JPRB ,&
&  0.55360E+00_JPRB, 0.54148E+00_JPRB, 0.53267E+00_JPRB, 0.52851E+00_JPRB, 0.52526E+00_JPRB, 0.51591E+00_JPRB, 0.49821E+00_JPRB ,&
&  0.48016E+00_JPRB, 0.46804E+00_JPRB, 0.46132E+00_JPRB, 0.45439E+00_JPRB, 0.44682E+00_JPRB, 0.44247E+00_JPRB, 0.44365E+00_JPRB ,&
&  0.45077E+00_JPRB, 0.46269E+00_JPRB, 0.48637E+00_JPRB, 0.51259E+00_JPRB, 0.53352E+00_JPRB, 0.55216E+00_JPRB, 0.56629E+00_JPRB ,&
&  0.67554E+00_JPRB, 0.85526E+00_JPRB, 0.91631E+00_JPRB, 0.15615E+01_JPRB, 0.16565E+01_JPRB, 0.16153E+01_JPRB, 0.16136E+01_JPRB ,&
&  0.16042E+01_JPRB, 0.17156E+01_JPRB, 0.14971E+01_JPRB, 0.12672E+01_JPRB, 0.10644E+01_JPRB, 0.92820E+00_JPRB, 0.83377E+00_JPRB ,&
&  0.77359E+00_JPRB, 0.72321E+00_JPRB, 0.69607E+00_JPRB, 0.68359E+00_JPRB, 0.68178E+00_JPRB, 0.72859E+00_JPRB, 0.85252E+00_JPRB ,&
&  0.10377E+01_JPRB, 0.13573E+01_JPRB, 0.18165E+01_JPRB, 0.21408E+01_JPRB, 0.20533E+01_JPRB, 0.20815E+01_JPRB, 0.23334E+01_JPRB ,&
&  0.29708E+01_JPRB, 0.25326E+01_JPRB, 0.16360E+01_JPRB, 0.11611E+01_JPRB, 0.95356E+00_JPRB, 0.86733E+00_JPRB, 0.80724E+00_JPRB ,&
&  0.78009E+00_JPRB, 0.75701E+00_JPRB, 0.73949E+00_JPRB, 0.72630E+00_JPRB, 0.71672E+00_JPRB, 0.71395E+00_JPRB, 0.70929E+00_JPRB ,&
&  0.70359E+00_JPRB, 0.69974E+00_JPRB, 0.69678E+00_JPRB, 0.69564E+00_JPRB, 0.70127E+00_JPRB, 0.71332E+00_JPRB, 0.72583E+00_JPRB ,&
&  0.73405E+00_JPRB, 0.74107E+00_JPRB, 0.72424E+00_JPRB, 0.70980E+00_JPRB, 0.71103E+00_JPRB, 0.70524E+00_JPRB, 0.68566E+00_JPRB ,&
&  0.68905E+00_JPRB, 0.70026E+00_JPRB, 0.69889E+00_JPRB, 0.68303E+00_JPRB, 0.68967E+00_JPRB, 0.70993E+00_JPRB, 0.68670E+00_JPRB ,&
&  0.68354E+00_JPRB, 0.68878E+00_JPRB, 0.69646E+00_JPRB, 0.69011E+00_JPRB, 0.68137E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.65839E+00_JPRB, 0.64755E+00_JPRB, 0.63077E+00_JPRB, 0.61160E+00_JPRB, 0.59227E+00_JPRB, 0.57576E+00_JPRB, 0.56699E+00_JPRB ,&
&  0.55664E+00_JPRB, 0.54606E+00_JPRB, 0.53667E+00_JPRB, 0.53140E+00_JPRB, 0.52919E+00_JPRB, 0.52019E+00_JPRB, 0.50357E+00_JPRB ,&
&  0.48599E+00_JPRB, 0.47006E+00_JPRB, 0.45714E+00_JPRB, 0.44974E+00_JPRB, 0.44174E+00_JPRB, 0.43788E+00_JPRB, 0.43749E+00_JPRB ,&
&  0.43878E+00_JPRB, 0.44540E+00_JPRB, 0.46016E+00_JPRB, 0.48385E+00_JPRB, 0.50413E+00_JPRB, 0.51638E+00_JPRB, 0.52536E+00_JPRB ,&
&  0.58978E+00_JPRB, 0.94577E+00_JPRB, 0.85044E+00_JPRB, 0.12230E+01_JPRB, 0.13103E+01_JPRB, 0.13732E+01_JPRB, 0.14571E+01_JPRB ,&
&  0.14941E+01_JPRB, 0.15406E+01_JPRB, 0.14536E+01_JPRB, 0.13020E+01_JPRB, 0.11253E+01_JPRB, 0.99757E+00_JPRB, 0.89492E+00_JPRB ,&
&  0.82210E+00_JPRB, 0.76964E+00_JPRB, 0.72845E+00_JPRB, 0.69490E+00_JPRB, 0.67309E+00_JPRB, 0.68002E+00_JPRB, 0.77440E+00_JPRB ,&
&  0.98763E+00_JPRB, 0.12543E+01_JPRB, 0.15548E+01_JPRB, 0.18846E+01_JPRB, 0.18165E+01_JPRB, 0.20604E+01_JPRB, 0.27782E+01_JPRB ,&
&  0.32700E+01_JPRB, 0.32362E+01_JPRB, 0.21186E+01_JPRB, 0.13692E+01_JPRB, 0.10478E+01_JPRB, 0.89449E+00_JPRB, 0.80608E+00_JPRB ,&
&  0.75612E+00_JPRB, 0.72962E+00_JPRB, 0.71523E+00_JPRB, 0.70936E+00_JPRB, 0.71001E+00_JPRB, 0.71725E+00_JPRB, 0.71771E+00_JPRB ,&
&  0.71742E+00_JPRB, 0.71542E+00_JPRB, 0.71294E+00_JPRB, 0.70630E+00_JPRB, 0.69697E+00_JPRB, 0.69569E+00_JPRB, 0.70199E+00_JPRB ,&
&  0.71281E+00_JPRB, 0.72284E+00_JPRB, 0.71368E+00_JPRB, 0.70294E+00_JPRB, 0.69816E+00_JPRB, 0.69188E+00_JPRB, 0.68042E+00_JPRB ,&
&  0.68786E+00_JPRB, 0.69193E+00_JPRB, 0.68946E+00_JPRB, 0.68840E+00_JPRB, 0.68220E+00_JPRB, 0.70378E+00_JPRB, 0.67704E+00_JPRB ,&
&  0.67090E+00_JPRB, 0.67500E+00_JPRB, 0.67830E+00_JPRB, 0.66956E+00_JPRB, 0.66360E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.63308E+00_JPRB, 0.62375E+00_JPRB, 0.60801E+00_JPRB, 0.59108E+00_JPRB, 0.57916E+00_JPRB, 0.56623E+00_JPRB, 0.55619E+00_JPRB ,&
&  0.54798E+00_JPRB, 0.54087E+00_JPRB, 0.53541E+00_JPRB, 0.52912E+00_JPRB, 0.52068E+00_JPRB, 0.50891E+00_JPRB, 0.49712E+00_JPRB ,&
&  0.48485E+00_JPRB, 0.47303E+00_JPRB, 0.46113E+00_JPRB, 0.44886E+00_JPRB, 0.43917E+00_JPRB, 0.43262E+00_JPRB, 0.42811E+00_JPRB ,&
&  0.42738E+00_JPRB, 0.43023E+00_JPRB, 0.43916E+00_JPRB, 0.45781E+00_JPRB, 0.47999E+00_JPRB, 0.49632E+00_JPRB, 0.51281E+00_JPRB ,&
&  0.58605E+00_JPRB, 0.53037E+00_JPRB, 0.94147E+00_JPRB, 0.10608E+01_JPRB, 0.10329E+01_JPRB, 0.10508E+01_JPRB, 0.12222E+01_JPRB ,&
&  0.13387E+01_JPRB, 0.13786E+01_JPRB, 0.13763E+01_JPRB, 0.13041E+01_JPRB, 0.11681E+01_JPRB, 0.10658E+01_JPRB, 0.98717E+00_JPRB ,&
&  0.92887E+00_JPRB, 0.88157E+00_JPRB, 0.82445E+00_JPRB, 0.76654E+00_JPRB, 0.72286E+00_JPRB, 0.71412E+00_JPRB, 0.76505E+00_JPRB ,&
&  0.99474E+00_JPRB, 0.12094E+01_JPRB, 0.13816E+01_JPRB, 0.15927E+01_JPRB, 0.16595E+01_JPRB, 0.19727E+01_JPRB, 0.23622E+01_JPRB ,&
&  0.31954E+01_JPRB, 0.32925E+01_JPRB, 0.24149E+01_JPRB, 0.16960E+01_JPRB, 0.12595E+01_JPRB, 0.10283E+01_JPRB, 0.87172E+00_JPRB ,&
&  0.78293E+00_JPRB, 0.73101E+00_JPRB, 0.69890E+00_JPRB, 0.69036E+00_JPRB, 0.70093E+00_JPRB, 0.72100E+00_JPRB, 0.73831E+00_JPRB ,&
&  0.75911E+00_JPRB, 0.77475E+00_JPRB, 0.77704E+00_JPRB, 0.76935E+00_JPRB, 0.74157E+00_JPRB, 0.70839E+00_JPRB, 0.68865E+00_JPRB ,&
&  0.68662E+00_JPRB, 0.69399E+00_JPRB, 0.69294E+00_JPRB, 0.68653E+00_JPRB, 0.68470E+00_JPRB, 0.68204E+00_JPRB, 0.67872E+00_JPRB ,&
&  0.68130E+00_JPRB, 0.68641E+00_JPRB, 0.68577E+00_JPRB, 0.69124E+00_JPRB, 0.67307E+00_JPRB, 0.69773E+00_JPRB, 0.67804E+00_JPRB ,&
&  0.66717E+00_JPRB, 0.66220E+00_JPRB, 0.65800E+00_JPRB, 0.64680E+00_JPRB, 0.63652E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.60538E+00_JPRB, 0.59573E+00_JPRB, 0.57998E+00_JPRB, 0.56388E+00_JPRB, 0.55392E+00_JPRB, 0.54649E+00_JPRB, 0.54119E+00_JPRB ,&
&  0.53799E+00_JPRB, 0.53040E+00_JPRB, 0.52295E+00_JPRB, 0.51241E+00_JPRB, 0.49881E+00_JPRB, 0.48715E+00_JPRB, 0.47985E+00_JPRB ,&
&  0.47094E+00_JPRB, 0.46636E+00_JPRB, 0.46065E+00_JPRB, 0.44949E+00_JPRB, 0.43723E+00_JPRB, 0.42700E+00_JPRB, 0.42035E+00_JPRB ,&
&  0.41716E+00_JPRB, 0.41936E+00_JPRB, 0.42547E+00_JPRB, 0.43959E+00_JPRB, 0.45342E+00_JPRB, 0.46863E+00_JPRB, 0.49388E+00_JPRB ,&
&  0.58603E+00_JPRB, 0.87930E+00_JPRB, 0.10813E+01_JPRB, 0.96331E+00_JPRB, 0.88978E+00_JPRB, 0.84003E+00_JPRB, 0.95093E+00_JPRB ,&
&  0.10512E+01_JPRB, 0.11542E+01_JPRB, 0.12114E+01_JPRB, 0.12271E+01_JPRB, 0.11573E+01_JPRB, 0.11049E+01_JPRB, 0.10521E+01_JPRB ,&
&  0.99697E+00_JPRB, 0.93625E+00_JPRB, 0.85688E+00_JPRB, 0.77869E+00_JPRB, 0.73646E+00_JPRB, 0.74021E+00_JPRB, 0.80627E+00_JPRB ,&
&  0.97247E+00_JPRB, 0.11454E+01_JPRB, 0.12379E+01_JPRB, 0.14145E+01_JPRB, 0.16949E+01_JPRB, 0.17360E+01_JPRB, 0.23009E+01_JPRB ,&
&  0.29732E+01_JPRB, 0.26756E+01_JPRB, 0.22240E+01_JPRB, 0.17654E+01_JPRB, 0.14101E+01_JPRB, 0.11219E+01_JPRB, 0.95564E+00_JPRB ,&
&  0.84200E+00_JPRB, 0.76693E+00_JPRB, 0.71346E+00_JPRB, 0.68587E+00_JPRB, 0.69706E+00_JPRB, 0.73680E+00_JPRB, 0.77130E+00_JPRB ,&
&  0.81952E+00_JPRB, 0.87261E+00_JPRB, 0.90514E+00_JPRB, 0.90521E+00_JPRB, 0.86865E+00_JPRB, 0.80753E+00_JPRB, 0.74834E+00_JPRB ,&
&  0.70986E+00_JPRB, 0.68694E+00_JPRB, 0.67985E+00_JPRB, 0.67226E+00_JPRB, 0.67684E+00_JPRB, 0.68793E+00_JPRB, 0.68567E+00_JPRB ,&
&  0.68196E+00_JPRB, 0.68513E+00_JPRB, 0.69859E+00_JPRB, 0.68992E+00_JPRB, 0.67153E+00_JPRB, 0.70111E+00_JPRB, 0.68207E+00_JPRB ,&
&  0.66471E+00_JPRB, 0.64882E+00_JPRB, 0.63751E+00_JPRB, 0.61736E+00_JPRB, 0.60815E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.59172E+00_JPRB, 0.57769E+00_JPRB, 0.55520E+00_JPRB, 0.53556E+00_JPRB, 0.52409E+00_JPRB, 0.51684E+00_JPRB, 0.51410E+00_JPRB ,&
&  0.51319E+00_JPRB, 0.50619E+00_JPRB, 0.49770E+00_JPRB, 0.48481E+00_JPRB, 0.47337E+00_JPRB, 0.46311E+00_JPRB, 0.45420E+00_JPRB ,&
&  0.44693E+00_JPRB, 0.44297E+00_JPRB, 0.43914E+00_JPRB, 0.43477E+00_JPRB, 0.42805E+00_JPRB, 0.41827E+00_JPRB, 0.41187E+00_JPRB ,&
&  0.40829E+00_JPRB, 0.40825E+00_JPRB, 0.41203E+00_JPRB, 0.42126E+00_JPRB, 0.43209E+00_JPRB, 0.44575E+00_JPRB, 0.46698E+00_JPRB ,&
&  0.52798E+00_JPRB, 0.74799E+00_JPRB, 0.78774E+00_JPRB, 0.77422E+00_JPRB, 0.77220E+00_JPRB, 0.76066E+00_JPRB, 0.80128E+00_JPRB ,&
&  0.87914E+00_JPRB, 0.95022E+00_JPRB, 0.10089E+01_JPRB, 0.10267E+01_JPRB, 0.99101E+00_JPRB, 0.95435E+00_JPRB, 0.90777E+00_JPRB ,&
&  0.85927E+00_JPRB, 0.80744E+00_JPRB, 0.75690E+00_JPRB, 0.71174E+00_JPRB, 0.68180E+00_JPRB, 0.69799E+00_JPRB, 0.75039E+00_JPRB ,&
&  0.85103E+00_JPRB, 0.93933E+00_JPRB, 0.10098E+01_JPRB, 0.11520E+01_JPRB, 0.14455E+01_JPRB, 0.16308E+01_JPRB, 0.18768E+01_JPRB ,&
&  0.19389E+01_JPRB, 0.18801E+01_JPRB, 0.17696E+01_JPRB, 0.16126E+01_JPRB, 0.13931E+01_JPRB, 0.11777E+01_JPRB, 0.10025E+01_JPRB ,&
&  0.87289E+00_JPRB, 0.78554E+00_JPRB, 0.73087E+00_JPRB, 0.70641E+00_JPRB, 0.71541E+00_JPRB, 0.75368E+00_JPRB, 0.80530E+00_JPRB ,&
&  0.87113E+00_JPRB, 0.94002E+00_JPRB, 0.98684E+00_JPRB, 0.10079E+01_JPRB, 0.99475E+00_JPRB, 0.95496E+00_JPRB, 0.89100E+00_JPRB ,&
&  0.83055E+00_JPRB, 0.77695E+00_JPRB, 0.73770E+00_JPRB, 0.72189E+00_JPRB, 0.72015E+00_JPRB, 0.72053E+00_JPRB, 0.71941E+00_JPRB ,&
&  0.72310E+00_JPRB, 0.72319E+00_JPRB, 0.71919E+00_JPRB, 0.69943E+00_JPRB, 0.67377E+00_JPRB, 0.69060E+00_JPRB, 0.67164E+00_JPRB ,&
&  0.65270E+00_JPRB, 0.63129E+00_JPRB, 0.61353E+00_JPRB, 0.59046E+00_JPRB, 0.59447E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.62152E+00_JPRB, 0.59206E+00_JPRB, 0.55444E+00_JPRB, 0.52591E+00_JPRB, 0.50653E+00_JPRB, 0.49285E+00_JPRB, 0.48586E+00_JPRB ,&
&  0.48401E+00_JPRB, 0.47465E+00_JPRB, 0.45971E+00_JPRB, 0.44686E+00_JPRB, 0.44019E+00_JPRB, 0.43495E+00_JPRB, 0.42815E+00_JPRB ,&
&  0.42279E+00_JPRB, 0.41798E+00_JPRB, 0.41443E+00_JPRB, 0.41308E+00_JPRB, 0.41234E+00_JPRB, 0.41055E+00_JPRB, 0.40561E+00_JPRB ,&
&  0.39848E+00_JPRB, 0.39607E+00_JPRB, 0.39690E+00_JPRB, 0.40363E+00_JPRB, 0.41358E+00_JPRB, 0.42594E+00_JPRB, 0.44670E+00_JPRB ,&
&  0.51467E+00_JPRB, 0.53840E+00_JPRB, 0.58437E+00_JPRB, 0.59551E+00_JPRB, 0.61135E+00_JPRB, 0.65389E+00_JPRB, 0.69375E+00_JPRB ,&
&  0.73336E+00_JPRB, 0.78560E+00_JPRB, 0.84662E+00_JPRB, 0.87022E+00_JPRB, 0.84266E+00_JPRB, 0.79657E+00_JPRB, 0.75584E+00_JPRB ,&
&  0.71822E+00_JPRB, 0.69195E+00_JPRB, 0.66798E+00_JPRB, 0.64866E+00_JPRB, 0.64689E+00_JPRB, 0.66620E+00_JPRB, 0.69778E+00_JPRB ,&
&  0.72208E+00_JPRB, 0.72998E+00_JPRB, 0.81259E+00_JPRB, 0.92717E+00_JPRB, 0.10911E+01_JPRB, 0.12110E+01_JPRB, 0.13306E+01_JPRB ,&
&  0.14311E+01_JPRB, 0.14853E+01_JPRB, 0.15108E+01_JPRB, 0.14832E+01_JPRB, 0.13820E+01_JPRB, 0.12542E+01_JPRB, 0.11149E+01_JPRB ,&
&  0.98046E+00_JPRB, 0.88602E+00_JPRB, 0.83389E+00_JPRB, 0.80798E+00_JPRB, 0.80325E+00_JPRB, 0.81207E+00_JPRB, 0.83601E+00_JPRB ,&
&  0.87363E+00_JPRB, 0.91045E+00_JPRB, 0.94108E+00_JPRB, 0.95564E+00_JPRB, 0.95223E+00_JPRB, 0.94635E+00_JPRB, 0.93591E+00_JPRB ,&
&  0.92116E+00_JPRB, 0.89802E+00_JPRB, 0.86496E+00_JPRB, 0.82854E+00_JPRB, 0.79617E+00_JPRB, 0.77315E+00_JPRB, 0.75499E+00_JPRB ,&
&  0.73673E+00_JPRB, 0.72368E+00_JPRB, 0.69469E+00_JPRB, 0.67485E+00_JPRB, 0.68921E+00_JPRB, 0.67166E+00_JPRB, 0.65021E+00_JPRB ,&
&  0.62432E+00_JPRB, 0.60174E+00_JPRB, 0.58063E+00_JPRB, 0.55791E+00_JPRB, 0.60778E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.52926E+00_JPRB, 0.52666E+00_JPRB, 0.51264E+00_JPRB, 0.49354E+00_JPRB, 0.47959E+00_JPRB, 0.46690E+00_JPRB, 0.45636E+00_JPRB ,&
&  0.44950E+00_JPRB, 0.44078E+00_JPRB, 0.42946E+00_JPRB, 0.42073E+00_JPRB, 0.41926E+00_JPRB, 0.41659E+00_JPRB, 0.41222E+00_JPRB ,&
&  0.40447E+00_JPRB, 0.39700E+00_JPRB, 0.39191E+00_JPRB, 0.39031E+00_JPRB, 0.39116E+00_JPRB, 0.39323E+00_JPRB, 0.39390E+00_JPRB ,&
&  0.39111E+00_JPRB, 0.38615E+00_JPRB, 0.38524E+00_JPRB, 0.38718E+00_JPRB, 0.39255E+00_JPRB, 0.40155E+00_JPRB, 0.42186E+00_JPRB ,&
&  0.47475E+00_JPRB, 0.46089E+00_JPRB, 0.48367E+00_JPRB, 0.49853E+00_JPRB, 0.51918E+00_JPRB, 0.56017E+00_JPRB, 0.60479E+00_JPRB ,&
&  0.63185E+00_JPRB, 0.65315E+00_JPRB, 0.69340E+00_JPRB, 0.72619E+00_JPRB, 0.72349E+00_JPRB, 0.69764E+00_JPRB, 0.66276E+00_JPRB ,&
&  0.62808E+00_JPRB, 0.60238E+00_JPRB, 0.58165E+00_JPRB, 0.56782E+00_JPRB, 0.56833E+00_JPRB, 0.57996E+00_JPRB, 0.58871E+00_JPRB ,&
&  0.57857E+00_JPRB, 0.58645E+00_JPRB, 0.64128E+00_JPRB, 0.72456E+00_JPRB, 0.83358E+00_JPRB, 0.94420E+00_JPRB, 0.10565E+01_JPRB ,&
&  0.11531E+01_JPRB, 0.12156E+01_JPRB, 0.12643E+01_JPRB, 0.13003E+01_JPRB, 0.12911E+01_JPRB, 0.12675E+01_JPRB, 0.11898E+01_JPRB ,&
&  0.10922E+01_JPRB, 0.10084E+01_JPRB, 0.95061E+00_JPRB, 0.90460E+00_JPRB, 0.87027E+00_JPRB, 0.84762E+00_JPRB, 0.83559E+00_JPRB ,&
&  0.83475E+00_JPRB, 0.83936E+00_JPRB, 0.83939E+00_JPRB, 0.83410E+00_JPRB, 0.82766E+00_JPRB, 0.82395E+00_JPRB, 0.82080E+00_JPRB ,&
&  0.81874E+00_JPRB, 0.81670E+00_JPRB, 0.80720E+00_JPRB, 0.78687E+00_JPRB, 0.75772E+00_JPRB, 0.72855E+00_JPRB, 0.70039E+00_JPRB ,&
&  0.67746E+00_JPRB, 0.66645E+00_JPRB, 0.65272E+00_JPRB, 0.64694E+00_JPRB, 0.62887E+00_JPRB, 0.61215E+00_JPRB, 0.59570E+00_JPRB ,&
&  0.57800E+00_JPRB, 0.55983E+00_JPRB, 0.53688E+00_JPRB, 0.51446E+00_JPRB, 0.50964E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.47211E+00_JPRB, 0.47797E+00_JPRB, 0.47750E+00_JPRB, 0.46570E+00_JPRB, 0.45247E+00_JPRB, 0.44473E+00_JPRB, 0.43555E+00_JPRB ,&
&  0.42527E+00_JPRB, 0.41693E+00_JPRB, 0.40629E+00_JPRB, 0.40153E+00_JPRB, 0.39921E+00_JPRB, 0.39835E+00_JPRB, 0.39623E+00_JPRB ,&
&  0.39092E+00_JPRB, 0.38279E+00_JPRB, 0.37659E+00_JPRB, 0.37402E+00_JPRB, 0.37483E+00_JPRB, 0.37676E+00_JPRB, 0.37854E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37585E+00_JPRB, 0.37422E+00_JPRB, 0.37332E+00_JPRB, 0.37438E+00_JPRB, 0.38219E+00_JPRB, 0.39807E+00_JPRB ,&
&  0.44131E+00_JPRB, 0.40724E+00_JPRB, 0.41974E+00_JPRB, 0.44512E+00_JPRB, 0.47173E+00_JPRB, 0.50879E+00_JPRB, 0.54589E+00_JPRB ,&
&  0.56682E+00_JPRB, 0.56766E+00_JPRB, 0.58013E+00_JPRB, 0.60408E+00_JPRB, 0.60512E+00_JPRB, 0.59810E+00_JPRB, 0.58246E+00_JPRB ,&
&  0.55864E+00_JPRB, 0.53091E+00_JPRB, 0.50431E+00_JPRB, 0.48846E+00_JPRB, 0.48900E+00_JPRB, 0.49709E+00_JPRB, 0.49875E+00_JPRB ,&
&  0.48215E+00_JPRB, 0.47683E+00_JPRB, 0.50082E+00_JPRB, 0.53834E+00_JPRB, 0.58660E+00_JPRB, 0.66142E+00_JPRB, 0.75512E+00_JPRB ,&
&  0.83536E+00_JPRB, 0.91619E+00_JPRB, 0.99957E+00_JPRB, 0.10655E+01_JPRB, 0.11210E+01_JPRB, 0.11399E+01_JPRB, 0.11102E+01_JPRB ,&
&  0.10616E+01_JPRB, 0.10006E+01_JPRB, 0.94194E+00_JPRB, 0.88631E+00_JPRB, 0.83918E+00_JPRB, 0.80836E+00_JPRB, 0.78815E+00_JPRB ,&
&  0.77374E+00_JPRB, 0.76187E+00_JPRB, 0.74458E+00_JPRB, 0.72109E+00_JPRB, 0.69691E+00_JPRB, 0.67658E+00_JPRB, 0.65891E+00_JPRB ,&
&  0.64374E+00_JPRB, 0.62812E+00_JPRB, 0.61586E+00_JPRB, 0.60710E+00_JPRB, 0.59799E+00_JPRB, 0.59000E+00_JPRB, 0.58312E+00_JPRB ,&
&  0.57565E+00_JPRB, 0.57328E+00_JPRB, 0.56607E+00_JPRB, 0.55509E+00_JPRB, 0.55969E+00_JPRB, 0.55106E+00_JPRB, 0.54362E+00_JPRB ,&
&  0.53057E+00_JPRB, 0.51550E+00_JPRB, 0.49888E+00_JPRB, 0.45299E+00_JPRB, 0.46782E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.43293E+00_JPRB, 0.43806E+00_JPRB, 0.43821E+00_JPRB, 0.43299E+00_JPRB, 0.42446E+00_JPRB, 0.41850E+00_JPRB, 0.40937E+00_JPRB ,&
&  0.39931E+00_JPRB, 0.39390E+00_JPRB, 0.38756E+00_JPRB, 0.38484E+00_JPRB, 0.38513E+00_JPRB, 0.38150E+00_JPRB, 0.37818E+00_JPRB ,&
&  0.37511E+00_JPRB, 0.36962E+00_JPRB, 0.36604E+00_JPRB, 0.36297E+00_JPRB, 0.36270E+00_JPRB, 0.36297E+00_JPRB, 0.36287E+00_JPRB ,&
&  0.36101E+00_JPRB, 0.35784E+00_JPRB, 0.35512E+00_JPRB, 0.35331E+00_JPRB, 0.35502E+00_JPRB, 0.36170E+00_JPRB, 0.37493E+00_JPRB ,&
&  0.40617E+00_JPRB, 0.36777E+00_JPRB, 0.38306E+00_JPRB, 0.40141E+00_JPRB, 0.43040E+00_JPRB, 0.45355E+00_JPRB, 0.46455E+00_JPRB ,&
&  0.47730E+00_JPRB, 0.49101E+00_JPRB, 0.50191E+00_JPRB, 0.51220E+00_JPRB, 0.50912E+00_JPRB, 0.50105E+00_JPRB, 0.49382E+00_JPRB ,&
&  0.48335E+00_JPRB, 0.47019E+00_JPRB, 0.45701E+00_JPRB, 0.44448E+00_JPRB, 0.44049E+00_JPRB, 0.44359E+00_JPRB, 0.44061E+00_JPRB ,&
&  0.42855E+00_JPRB, 0.41860E+00_JPRB, 0.41447E+00_JPRB, 0.41974E+00_JPRB, 0.43500E+00_JPRB, 0.46168E+00_JPRB, 0.51427E+00_JPRB ,&
&  0.57650E+00_JPRB, 0.64896E+00_JPRB, 0.72936E+00_JPRB, 0.79864E+00_JPRB, 0.85317E+00_JPRB, 0.88313E+00_JPRB, 0.89506E+00_JPRB ,&
&  0.88585E+00_JPRB, 0.86644E+00_JPRB, 0.84391E+00_JPRB, 0.81391E+00_JPRB, 0.77379E+00_JPRB, 0.73485E+00_JPRB, 0.70292E+00_JPRB ,&
&  0.67787E+00_JPRB, 0.65237E+00_JPRB, 0.62515E+00_JPRB, 0.59561E+00_JPRB, 0.56441E+00_JPRB, 0.53562E+00_JPRB, 0.51174E+00_JPRB ,&
&  0.49180E+00_JPRB, 0.47488E+00_JPRB, 0.46436E+00_JPRB, 0.46008E+00_JPRB, 0.46316E+00_JPRB, 0.46871E+00_JPRB, 0.47258E+00_JPRB ,&
&  0.47642E+00_JPRB, 0.48235E+00_JPRB, 0.48608E+00_JPRB, 0.49332E+00_JPRB, 0.49220E+00_JPRB, 0.48922E+00_JPRB, 0.48678E+00_JPRB ,&
&  0.48038E+00_JPRB, 0.46827E+00_JPRB, 0.41143E+00_JPRB, 0.42886E+00_JPRB, 0.42995E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.40441E+00_JPRB, 0.40089E+00_JPRB, 0.39760E+00_JPRB, 0.39936E+00_JPRB, 0.39834E+00_JPRB, 0.39475E+00_JPRB, 0.38852E+00_JPRB ,&
&  0.37853E+00_JPRB, 0.37184E+00_JPRB, 0.36706E+00_JPRB, 0.36597E+00_JPRB, 0.36895E+00_JPRB, 0.36735E+00_JPRB, 0.36293E+00_JPRB ,&
&  0.35947E+00_JPRB, 0.35554E+00_JPRB, 0.35258E+00_JPRB, 0.34916E+00_JPRB, 0.34627E+00_JPRB, 0.34493E+00_JPRB, 0.34412E+00_JPRB ,&
&  0.34238E+00_JPRB, 0.33934E+00_JPRB, 0.33660E+00_JPRB, 0.33514E+00_JPRB, 0.33746E+00_JPRB, 0.34352E+00_JPRB, 0.35305E+00_JPRB ,&
&  0.35094E+00_JPRB, 0.33337E+00_JPRB, 0.35172E+00_JPRB, 0.37449E+00_JPRB, 0.38955E+00_JPRB, 0.40443E+00_JPRB, 0.41596E+00_JPRB ,&
&  0.42637E+00_JPRB, 0.42997E+00_JPRB, 0.43384E+00_JPRB, 0.43703E+00_JPRB, 0.43752E+00_JPRB, 0.43375E+00_JPRB, 0.42923E+00_JPRB ,&
&  0.42554E+00_JPRB, 0.42236E+00_JPRB, 0.41863E+00_JPRB, 0.41230E+00_JPRB, 0.40643E+00_JPRB, 0.40167E+00_JPRB, 0.39698E+00_JPRB ,&
&  0.39125E+00_JPRB, 0.38547E+00_JPRB, 0.38167E+00_JPRB, 0.38125E+00_JPRB, 0.38495E+00_JPRB, 0.39192E+00_JPRB, 0.40477E+00_JPRB ,&
&  0.42076E+00_JPRB, 0.44951E+00_JPRB, 0.50260E+00_JPRB, 0.55978E+00_JPRB, 0.60707E+00_JPRB, 0.64093E+00_JPRB, 0.66862E+00_JPRB ,&
&  0.67916E+00_JPRB, 0.67903E+00_JPRB, 0.67485E+00_JPRB, 0.66450E+00_JPRB, 0.64756E+00_JPRB, 0.62383E+00_JPRB, 0.59490E+00_JPRB ,&
&  0.56843E+00_JPRB, 0.54487E+00_JPRB, 0.52182E+00_JPRB, 0.49920E+00_JPRB, 0.47680E+00_JPRB, 0.45594E+00_JPRB, 0.43800E+00_JPRB ,&
&  0.42249E+00_JPRB, 0.40875E+00_JPRB, 0.39851E+00_JPRB, 0.39303E+00_JPRB, 0.39372E+00_JPRB, 0.39859E+00_JPRB, 0.40584E+00_JPRB ,&
&  0.41226E+00_JPRB, 0.42089E+00_JPRB, 0.42885E+00_JPRB, 0.43635E+00_JPRB, 0.44143E+00_JPRB, 0.44055E+00_JPRB, 0.43908E+00_JPRB ,&
&  0.43584E+00_JPRB, 0.42796E+00_JPRB, 0.42086E+00_JPRB, 0.41249E+00_JPRB, 0.40630E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.37331E+00_JPRB, 0.36813E+00_JPRB, 0.36774E+00_JPRB, 0.37136E+00_JPRB, 0.37532E+00_JPRB, 0.37500E+00_JPRB, 0.37100E+00_JPRB ,&
&  0.36166E+00_JPRB, 0.35350E+00_JPRB, 0.34707E+00_JPRB, 0.34441E+00_JPRB, 0.34656E+00_JPRB, 0.34716E+00_JPRB, 0.34465E+00_JPRB ,&
&  0.34170E+00_JPRB, 0.33626E+00_JPRB, 0.33156E+00_JPRB, 0.32692E+00_JPRB, 0.32355E+00_JPRB, 0.32198E+00_JPRB, 0.32130E+00_JPRB ,&
&  0.31980E+00_JPRB, 0.31804E+00_JPRB, 0.31761E+00_JPRB, 0.31753E+00_JPRB, 0.31918E+00_JPRB, 0.32329E+00_JPRB, 0.32858E+00_JPRB ,&
&  0.33801E+00_JPRB, 0.32337E+00_JPRB, 0.34153E+00_JPRB, 0.34670E+00_JPRB, 0.35207E+00_JPRB, 0.36056E+00_JPRB, 0.37078E+00_JPRB ,&
&  0.37997E+00_JPRB, 0.38489E+00_JPRB, 0.38740E+00_JPRB, 0.38814E+00_JPRB, 0.38680E+00_JPRB, 0.38600E+00_JPRB, 0.38279E+00_JPRB ,&
&  0.37795E+00_JPRB, 0.37593E+00_JPRB, 0.37498E+00_JPRB, 0.37287E+00_JPRB, 0.37081E+00_JPRB, 0.36785E+00_JPRB, 0.36492E+00_JPRB ,&
&  0.36249E+00_JPRB, 0.35973E+00_JPRB, 0.35729E+00_JPRB, 0.35465E+00_JPRB, 0.35301E+00_JPRB, 0.35405E+00_JPRB, 0.35804E+00_JPRB ,&
&  0.36273E+00_JPRB, 0.37002E+00_JPRB, 0.38566E+00_JPRB, 0.40946E+00_JPRB, 0.43801E+00_JPRB, 0.46565E+00_JPRB, 0.48813E+00_JPRB ,&
&  0.50219E+00_JPRB, 0.50965E+00_JPRB, 0.51101E+00_JPRB, 0.50877E+00_JPRB, 0.50207E+00_JPRB, 0.49281E+00_JPRB, 0.47971E+00_JPRB ,&
&  0.46639E+00_JPRB, 0.45351E+00_JPRB, 0.44025E+00_JPRB, 0.42701E+00_JPRB, 0.41272E+00_JPRB, 0.39777E+00_JPRB, 0.38413E+00_JPRB ,&
&  0.37311E+00_JPRB, 0.36506E+00_JPRB, 0.35928E+00_JPRB, 0.35579E+00_JPRB, 0.35393E+00_JPRB, 0.35506E+00_JPRB, 0.35792E+00_JPRB ,&
&  0.36247E+00_JPRB, 0.36851E+00_JPRB, 0.37557E+00_JPRB, 0.38195E+00_JPRB, 0.38798E+00_JPRB, 0.39039E+00_JPRB, 0.39203E+00_JPRB ,&
&  0.39114E+00_JPRB, 0.38713E+00_JPRB, 0.38398E+00_JPRB, 0.38057E+00_JPRB, 0.37759E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.34406E+00_JPRB, 0.34199E+00_JPRB, 0.34448E+00_JPRB, 0.34901E+00_JPRB, 0.35553E+00_JPRB, 0.35492E+00_JPRB, 0.35281E+00_JPRB ,&
&  0.34740E+00_JPRB, 0.34018E+00_JPRB, 0.33322E+00_JPRB, 0.32680E+00_JPRB, 0.32427E+00_JPRB, 0.32504E+00_JPRB, 0.32500E+00_JPRB ,&
&  0.32368E+00_JPRB, 0.31980E+00_JPRB, 0.31447E+00_JPRB, 0.30934E+00_JPRB, 0.30563E+00_JPRB, 0.30357E+00_JPRB, 0.30186E+00_JPRB ,&
&  0.30091E+00_JPRB, 0.29946E+00_JPRB, 0.30032E+00_JPRB, 0.30035E+00_JPRB, 0.30108E+00_JPRB, 0.30470E+00_JPRB, 0.30784E+00_JPRB ,&
&  0.30954E+00_JPRB, 0.31577E+00_JPRB, 0.31435E+00_JPRB, 0.31442E+00_JPRB, 0.31806E+00_JPRB, 0.32620E+00_JPRB, 0.33666E+00_JPRB ,&
&  0.34600E+00_JPRB, 0.35171E+00_JPRB, 0.35389E+00_JPRB, 0.35235E+00_JPRB, 0.34959E+00_JPRB, 0.34744E+00_JPRB, 0.34634E+00_JPRB ,&
&  0.34480E+00_JPRB, 0.34120E+00_JPRB, 0.33707E+00_JPRB, 0.33354E+00_JPRB, 0.33213E+00_JPRB, 0.33125E+00_JPRB, 0.33009E+00_JPRB ,&
&  0.32880E+00_JPRB, 0.32741E+00_JPRB, 0.32472E+00_JPRB, 0.32115E+00_JPRB, 0.31783E+00_JPRB, 0.31689E+00_JPRB, 0.31875E+00_JPRB ,&
&  0.32268E+00_JPRB, 0.32794E+00_JPRB, 0.33565E+00_JPRB, 0.34568E+00_JPRB, 0.35622E+00_JPRB, 0.36909E+00_JPRB, 0.38096E+00_JPRB ,&
&  0.38975E+00_JPRB, 0.39424E+00_JPRB, 0.39523E+00_JPRB, 0.39429E+00_JPRB, 0.39177E+00_JPRB, 0.38793E+00_JPRB, 0.38380E+00_JPRB ,&
&  0.37965E+00_JPRB, 0.37494E+00_JPRB, 0.36960E+00_JPRB, 0.36340E+00_JPRB, 0.35532E+00_JPRB, 0.34680E+00_JPRB, 0.33978E+00_JPRB ,&
&  0.33492E+00_JPRB, 0.33149E+00_JPRB, 0.32904E+00_JPRB, 0.32693E+00_JPRB, 0.32571E+00_JPRB, 0.32571E+00_JPRB, 0.32726E+00_JPRB ,&
&  0.33033E+00_JPRB, 0.33453E+00_JPRB, 0.33972E+00_JPRB, 0.34498E+00_JPRB, 0.34863E+00_JPRB, 0.35150E+00_JPRB, 0.35426E+00_JPRB ,&
&  0.35496E+00_JPRB, 0.35278E+00_JPRB, 0.35116E+00_JPRB, 0.35014E+00_JPRB, 0.34763E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.31910E+00_JPRB, 0.31939E+00_JPRB, 0.32400E+00_JPRB, 0.32897E+00_JPRB, 0.33426E+00_JPRB, 0.33597E+00_JPRB, 0.33376E+00_JPRB ,&
&  0.33101E+00_JPRB, 0.32721E+00_JPRB, 0.32150E+00_JPRB, 0.31434E+00_JPRB, 0.30785E+00_JPRB, 0.30619E+00_JPRB, 0.30644E+00_JPRB ,&
&  0.30673E+00_JPRB, 0.30482E+00_JPRB, 0.30111E+00_JPRB, 0.29702E+00_JPRB, 0.29236E+00_JPRB, 0.28828E+00_JPRB, 0.28520E+00_JPRB ,&
&  0.28430E+00_JPRB, 0.28384E+00_JPRB, 0.28449E+00_JPRB, 0.28506E+00_JPRB, 0.28603E+00_JPRB, 0.28874E+00_JPRB, 0.29419E+00_JPRB ,&
&  0.29212E+00_JPRB, 0.29431E+00_JPRB, 0.28701E+00_JPRB, 0.28978E+00_JPRB, 0.29260E+00_JPRB, 0.29758E+00_JPRB, 0.30342E+00_JPRB ,&
&  0.30947E+00_JPRB, 0.31445E+00_JPRB, 0.31754E+00_JPRB, 0.31770E+00_JPRB, 0.31619E+00_JPRB, 0.31616E+00_JPRB, 0.31627E+00_JPRB ,&
&  0.31574E+00_JPRB, 0.31486E+00_JPRB, 0.31293E+00_JPRB, 0.30892E+00_JPRB, 0.30340E+00_JPRB, 0.29872E+00_JPRB, 0.29557E+00_JPRB ,&
&  0.29526E+00_JPRB, 0.29541E+00_JPRB, 0.29441E+00_JPRB, 0.29295E+00_JPRB, 0.29128E+00_JPRB, 0.29070E+00_JPRB, 0.29216E+00_JPRB ,&
&  0.29487E+00_JPRB, 0.29762E+00_JPRB, 0.30012E+00_JPRB, 0.30446E+00_JPRB, 0.31047E+00_JPRB, 0.31615E+00_JPRB, 0.32134E+00_JPRB ,&
&  0.32561E+00_JPRB, 0.32686E+00_JPRB, 0.32615E+00_JPRB, 0.32437E+00_JPRB, 0.32312E+00_JPRB, 0.32215E+00_JPRB, 0.32197E+00_JPRB ,&
&  0.32147E+00_JPRB, 0.32035E+00_JPRB, 0.31810E+00_JPRB, 0.31484E+00_JPRB, 0.31149E+00_JPRB, 0.30820E+00_JPRB, 0.30577E+00_JPRB ,&
&  0.30436E+00_JPRB, 0.30339E+00_JPRB, 0.30221E+00_JPRB, 0.30103E+00_JPRB, 0.30094E+00_JPRB, 0.30120E+00_JPRB, 0.30219E+00_JPRB ,&
&  0.30385E+00_JPRB, 0.30637E+00_JPRB, 0.30986E+00_JPRB, 0.31362E+00_JPRB, 0.31422E+00_JPRB, 0.31466E+00_JPRB, 0.31680E+00_JPRB ,&
&  0.31892E+00_JPRB, 0.31963E+00_JPRB, 0.31949E+00_JPRB, 0.31941E+00_JPRB, 0.31989E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.29825E+00_JPRB, 0.30041E+00_JPRB, 0.30447E+00_JPRB, 0.31052E+00_JPRB, 0.31386E+00_JPRB, 0.31743E+00_JPRB, 0.31732E+00_JPRB ,&
&  0.31481E+00_JPRB, 0.31135E+00_JPRB, 0.30717E+00_JPRB, 0.30187E+00_JPRB, 0.29620E+00_JPRB, 0.29190E+00_JPRB, 0.29044E+00_JPRB ,&
&  0.29036E+00_JPRB, 0.28924E+00_JPRB, 0.28643E+00_JPRB, 0.28269E+00_JPRB, 0.27855E+00_JPRB, 0.27521E+00_JPRB, 0.27266E+00_JPRB ,&
&  0.27193E+00_JPRB, 0.27238E+00_JPRB, 0.27348E+00_JPRB, 0.27414E+00_JPRB, 0.27438E+00_JPRB, 0.27660E+00_JPRB, 0.28057E+00_JPRB ,&
&  0.27861E+00_JPRB, 0.27645E+00_JPRB, 0.27075E+00_JPRB, 0.27027E+00_JPRB, 0.27175E+00_JPRB, 0.27409E+00_JPRB, 0.27656E+00_JPRB ,&
&  0.27888E+00_JPRB, 0.28174E+00_JPRB, 0.28493E+00_JPRB, 0.28745E+00_JPRB, 0.28908E+00_JPRB, 0.29054E+00_JPRB, 0.29149E+00_JPRB ,&
&  0.29179E+00_JPRB, 0.29077E+00_JPRB, 0.28884E+00_JPRB, 0.28533E+00_JPRB, 0.28080E+00_JPRB, 0.27772E+00_JPRB, 0.27643E+00_JPRB ,&
&  0.27527E+00_JPRB, 0.27544E+00_JPRB, 0.27568E+00_JPRB, 0.27522E+00_JPRB, 0.27433E+00_JPRB, 0.27433E+00_JPRB, 0.27540E+00_JPRB ,&
&  0.27648E+00_JPRB, 0.27678E+00_JPRB, 0.27646E+00_JPRB, 0.27725E+00_JPRB, 0.27983E+00_JPRB, 0.28311E+00_JPRB, 0.28571E+00_JPRB ,&
&  0.28746E+00_JPRB, 0.28725E+00_JPRB, 0.28586E+00_JPRB, 0.28338E+00_JPRB, 0.28150E+00_JPRB, 0.28141E+00_JPRB, 0.28293E+00_JPRB ,&
&  0.28422E+00_JPRB, 0.28451E+00_JPRB, 0.28475E+00_JPRB, 0.28329E+00_JPRB, 0.28161E+00_JPRB, 0.28090E+00_JPRB, 0.28047E+00_JPRB ,&
&  0.27962E+00_JPRB, 0.27856E+00_JPRB, 0.27758E+00_JPRB, 0.27707E+00_JPRB, 0.27728E+00_JPRB, 0.27707E+00_JPRB, 0.27763E+00_JPRB ,&
&  0.27998E+00_JPRB, 0.28296E+00_JPRB, 0.28583E+00_JPRB, 0.28789E+00_JPRB, 0.28886E+00_JPRB, 0.28863E+00_JPRB, 0.29032E+00_JPRB ,&
&  0.29325E+00_JPRB, 0.29541E+00_JPRB, 0.29534E+00_JPRB, 0.29486E+00_JPRB, 0.29637E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.28027E+00_JPRB, 0.28279E+00_JPRB, 0.28707E+00_JPRB, 0.29203E+00_JPRB, 0.29527E+00_JPRB, 0.29956E+00_JPRB, 0.30030E+00_JPRB ,&
&  0.29960E+00_JPRB, 0.29782E+00_JPRB, 0.29370E+00_JPRB, 0.28867E+00_JPRB, 0.28384E+00_JPRB, 0.27978E+00_JPRB, 0.27699E+00_JPRB ,&
&  0.27549E+00_JPRB, 0.27419E+00_JPRB, 0.27261E+00_JPRB, 0.27010E+00_JPRB, 0.26708E+00_JPRB, 0.26438E+00_JPRB, 0.26229E+00_JPRB ,&
&  0.26159E+00_JPRB, 0.26154E+00_JPRB, 0.26190E+00_JPRB, 0.26181E+00_JPRB, 0.26152E+00_JPRB, 0.26259E+00_JPRB, 0.26422E+00_JPRB ,&
&  0.26329E+00_JPRB, 0.25911E+00_JPRB, 0.25632E+00_JPRB, 0.25608E+00_JPRB, 0.25612E+00_JPRB, 0.25658E+00_JPRB, 0.25727E+00_JPRB ,&
&  0.25746E+00_JPRB, 0.25805E+00_JPRB, 0.25957E+00_JPRB, 0.26134E+00_JPRB, 0.26274E+00_JPRB, 0.26532E+00_JPRB, 0.26663E+00_JPRB ,&
&  0.26676E+00_JPRB, 0.26604E+00_JPRB, 0.26470E+00_JPRB, 0.26218E+00_JPRB, 0.26049E+00_JPRB, 0.26013E+00_JPRB, 0.25963E+00_JPRB ,&
&  0.26047E+00_JPRB, 0.26138E+00_JPRB, 0.26224E+00_JPRB, 0.26150E+00_JPRB, 0.25988E+00_JPRB, 0.25979E+00_JPRB, 0.26044E+00_JPRB ,&
&  0.26070E+00_JPRB, 0.25990E+00_JPRB, 0.25835E+00_JPRB, 0.25764E+00_JPRB, 0.25747E+00_JPRB, 0.25821E+00_JPRB, 0.25859E+00_JPRB ,&
&  0.25828E+00_JPRB, 0.25685E+00_JPRB, 0.25580E+00_JPRB, 0.25525E+00_JPRB, 0.25496E+00_JPRB, 0.25534E+00_JPRB, 0.25530E+00_JPRB ,&
&  0.25629E+00_JPRB, 0.25728E+00_JPRB, 0.25656E+00_JPRB, 0.25597E+00_JPRB, 0.25545E+00_JPRB, 0.25537E+00_JPRB, 0.25505E+00_JPRB ,&
&  0.25427E+00_JPRB, 0.25338E+00_JPRB, 0.25279E+00_JPRB, 0.25269E+00_JPRB, 0.25177E+00_JPRB, 0.25070E+00_JPRB, 0.25133E+00_JPRB ,&
&  0.25449E+00_JPRB, 0.25797E+00_JPRB, 0.26099E+00_JPRB, 0.26301E+00_JPRB, 0.26370E+00_JPRB, 0.26510E+00_JPRB, 0.26776E+00_JPRB ,&
&  0.27000E+00_JPRB, 0.27088E+00_JPRB, 0.27297E+00_JPRB, 0.27490E+00_JPRB, 0.27707E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.26466E+00_JPRB, 0.26807E+00_JPRB, 0.27055E+00_JPRB, 0.27392E+00_JPRB, 0.27693E+00_JPRB, 0.28181E+00_JPRB, 0.28377E+00_JPRB ,&
&  0.28362E+00_JPRB, 0.28334E+00_JPRB, 0.28189E+00_JPRB, 0.27937E+00_JPRB, 0.27540E+00_JPRB, 0.27131E+00_JPRB, 0.26746E+00_JPRB ,&
&  0.26450E+00_JPRB, 0.26214E+00_JPRB, 0.26030E+00_JPRB, 0.25864E+00_JPRB, 0.25631E+00_JPRB, 0.25413E+00_JPRB, 0.25249E+00_JPRB ,&
&  0.25174E+00_JPRB, 0.25105E+00_JPRB, 0.25071E+00_JPRB, 0.25059E+00_JPRB, 0.25023E+00_JPRB, 0.24973E+00_JPRB, 0.24963E+00_JPRB ,&
&  0.24846E+00_JPRB, 0.24487E+00_JPRB, 0.24279E+00_JPRB, 0.24210E+00_JPRB, 0.24143E+00_JPRB, 0.24182E+00_JPRB, 0.24138E+00_JPRB ,&
&  0.24105E+00_JPRB, 0.24146E+00_JPRB, 0.24292E+00_JPRB, 0.24473E+00_JPRB, 0.24501E+00_JPRB, 0.24574E+00_JPRB, 0.24706E+00_JPRB ,&
&  0.24756E+00_JPRB, 0.24687E+00_JPRB, 0.24513E+00_JPRB, 0.24471E+00_JPRB, 0.24556E+00_JPRB, 0.24582E+00_JPRB, 0.24637E+00_JPRB ,&
&  0.24713E+00_JPRB, 0.24838E+00_JPRB, 0.24910E+00_JPRB, 0.24752E+00_JPRB, 0.24590E+00_JPRB, 0.24534E+00_JPRB, 0.24540E+00_JPRB ,&
&  0.24522E+00_JPRB, 0.24438E+00_JPRB, 0.24244E+00_JPRB, 0.24069E+00_JPRB, 0.23812E+00_JPRB, 0.23608E+00_JPRB, 0.23508E+00_JPRB ,&
&  0.23557E+00_JPRB, 0.23605E+00_JPRB, 0.23748E+00_JPRB, 0.23873E+00_JPRB, 0.23937E+00_JPRB, 0.23944E+00_JPRB, 0.23802E+00_JPRB ,&
&  0.23642E+00_JPRB, 0.23585E+00_JPRB, 0.23479E+00_JPRB, 0.23369E+00_JPRB, 0.23393E+00_JPRB, 0.23385E+00_JPRB, 0.23331E+00_JPRB ,&
&  0.23268E+00_JPRB, 0.23299E+00_JPRB, 0.23321E+00_JPRB, 0.23252E+00_JPRB, 0.23165E+00_JPRB, 0.23058E+00_JPRB, 0.23101E+00_JPRB ,&
&  0.23248E+00_JPRB, 0.23465E+00_JPRB, 0.23717E+00_JPRB, 0.23997E+00_JPRB, 0.24288E+00_JPRB, 0.24546E+00_JPRB, 0.24800E+00_JPRB ,&
&  0.25113E+00_JPRB, 0.25213E+00_JPRB, 0.25412E+00_JPRB, 0.25728E+00_JPRB, 0.26038E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.24616E+00_JPRB, 0.25109E+00_JPRB, 0.25372E+00_JPRB, 0.25588E+00_JPRB, 0.25840E+00_JPRB, 0.26303E+00_JPRB, 0.26634E+00_JPRB ,&
&  0.26708E+00_JPRB, 0.26720E+00_JPRB, 0.26692E+00_JPRB, 0.26595E+00_JPRB, 0.26429E+00_JPRB, 0.26270E+00_JPRB, 0.25996E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25337E+00_JPRB, 0.25054E+00_JPRB, 0.24788E+00_JPRB, 0.24539E+00_JPRB, 0.24310E+00_JPRB, 0.24134E+00_JPRB ,&
&  0.24089E+00_JPRB, 0.24077E+00_JPRB, 0.23994E+00_JPRB, 0.23973E+00_JPRB, 0.23966E+00_JPRB, 0.23874E+00_JPRB, 0.23718E+00_JPRB ,&
&  0.23525E+00_JPRB, 0.23203E+00_JPRB, 0.22877E+00_JPRB, 0.22780E+00_JPRB, 0.22772E+00_JPRB, 0.22683E+00_JPRB, 0.22555E+00_JPRB ,&
&  0.22669E+00_JPRB, 0.22873E+00_JPRB, 0.23100E+00_JPRB, 0.23215E+00_JPRB, 0.23296E+00_JPRB, 0.23426E+00_JPRB, 0.23585E+00_JPRB ,&
&  0.23607E+00_JPRB, 0.23354E+00_JPRB, 0.23132E+00_JPRB, 0.23098E+00_JPRB, 0.23105E+00_JPRB, 0.23086E+00_JPRB, 0.23139E+00_JPRB ,&
&  0.23213E+00_JPRB, 0.23372E+00_JPRB, 0.23478E+00_JPRB, 0.23375E+00_JPRB, 0.23181E+00_JPRB, 0.23038E+00_JPRB, 0.22950E+00_JPRB ,&
&  0.22913E+00_JPRB, 0.22872E+00_JPRB, 0.22789E+00_JPRB, 0.22647E+00_JPRB, 0.22485E+00_JPRB, 0.22332E+00_JPRB, 0.22173E+00_JPRB ,&
&  0.22037E+00_JPRB, 0.22071E+00_JPRB, 0.22325E+00_JPRB, 0.22475E+00_JPRB, 0.22430E+00_JPRB, 0.22292E+00_JPRB, 0.22135E+00_JPRB ,&
&  0.22045E+00_JPRB, 0.21809E+00_JPRB, 0.21601E+00_JPRB, 0.21447E+00_JPRB, 0.21340E+00_JPRB, 0.21191E+00_JPRB, 0.21138E+00_JPRB ,&
&  0.21178E+00_JPRB, 0.21342E+00_JPRB, 0.21375E+00_JPRB, 0.21357E+00_JPRB, 0.21303E+00_JPRB, 0.21260E+00_JPRB, 0.21350E+00_JPRB ,&
&  0.21477E+00_JPRB, 0.21689E+00_JPRB, 0.21925E+00_JPRB, 0.22236E+00_JPRB, 0.22525E+00_JPRB, 0.22880E+00_JPRB, 0.23168E+00_JPRB ,&
&  0.23353E+00_JPRB, 0.23410E+00_JPRB, 0.23533E+00_JPRB, 0.23731E+00_JPRB, 0.24070E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.22643E+00_JPRB, 0.23201E+00_JPRB, 0.23677E+00_JPRB, 0.23921E+00_JPRB, 0.24066E+00_JPRB, 0.24300E+00_JPRB, 0.24554E+00_JPRB ,&
&  0.24745E+00_JPRB, 0.24864E+00_JPRB, 0.24927E+00_JPRB, 0.24930E+00_JPRB, 0.24853E+00_JPRB, 0.24714E+00_JPRB, 0.24547E+00_JPRB ,&
&  0.24359E+00_JPRB, 0.24086E+00_JPRB, 0.23834E+00_JPRB, 0.23646E+00_JPRB, 0.23490E+00_JPRB, 0.23310E+00_JPRB, 0.23167E+00_JPRB ,&
&  0.23021E+00_JPRB, 0.22980E+00_JPRB, 0.22917E+00_JPRB, 0.22840E+00_JPRB, 0.22769E+00_JPRB, 0.22661E+00_JPRB, 0.22569E+00_JPRB ,&
&  0.22315E+00_JPRB, 0.21905E+00_JPRB, 0.21740E+00_JPRB, 0.21640E+00_JPRB, 0.21656E+00_JPRB, 0.21582E+00_JPRB, 0.21486E+00_JPRB ,&
&  0.21639E+00_JPRB, 0.21924E+00_JPRB, 0.22137E+00_JPRB, 0.22251E+00_JPRB, 0.22331E+00_JPRB, 0.22450E+00_JPRB, 0.22581E+00_JPRB ,&
&  0.22355E+00_JPRB, 0.22076E+00_JPRB, 0.21880E+00_JPRB, 0.21790E+00_JPRB, 0.21794E+00_JPRB, 0.21894E+00_JPRB, 0.22073E+00_JPRB ,&
&  0.22235E+00_JPRB, 0.22335E+00_JPRB, 0.22310E+00_JPRB, 0.22174E+00_JPRB, 0.21999E+00_JPRB, 0.21915E+00_JPRB, 0.21871E+00_JPRB ,&
&  0.21853E+00_JPRB, 0.21820E+00_JPRB, 0.21782E+00_JPRB, 0.21668E+00_JPRB, 0.21070E+00_JPRB, 0.20721E+00_JPRB, 0.20457E+00_JPRB ,&
&  0.20530E+00_JPRB, 0.20499E+00_JPRB, 0.20301E+00_JPRB, 0.20906E+00_JPRB, 0.21534E+00_JPRB, 0.21043E+00_JPRB, 0.20475E+00_JPRB ,&
&  0.19698E+00_JPRB, 0.19862E+00_JPRB, 0.19590E+00_JPRB, 0.19089E+00_JPRB, 0.18967E+00_JPRB, 0.18987E+00_JPRB, 0.18934E+00_JPRB ,&
&  0.19000E+00_JPRB, 0.18976E+00_JPRB, 0.19078E+00_JPRB, 0.19125E+00_JPRB, 0.19150E+00_JPRB, 0.19241E+00_JPRB, 0.19334E+00_JPRB ,&
&  0.19573E+00_JPRB, 0.19986E+00_JPRB, 0.20438E+00_JPRB, 0.20550E+00_JPRB, 0.20845E+00_JPRB, 0.21098E+00_JPRB, 0.21259E+00_JPRB ,&
&  0.21390E+00_JPRB, 0.21588E+00_JPRB, 0.21776E+00_JPRB, 0.21934E+00_JPRB, 0.22196E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.21284E+00_JPRB, 0.21530E+00_JPRB, 0.21830E+00_JPRB, 0.22127E+00_JPRB, 0.22337E+00_JPRB, 0.22544E+00_JPRB, 0.22728E+00_JPRB ,&
&  0.22891E+00_JPRB, 0.23006E+00_JPRB, 0.23084E+00_JPRB, 0.23132E+00_JPRB, 0.23166E+00_JPRB, 0.23146E+00_JPRB, 0.23081E+00_JPRB ,&
&  0.23013E+00_JPRB, 0.22889E+00_JPRB, 0.22721E+00_JPRB, 0.22588E+00_JPRB, 0.22505E+00_JPRB, 0.22395E+00_JPRB, 0.22232E+00_JPRB ,&
&  0.22054E+00_JPRB, 0.21890E+00_JPRB, 0.21755E+00_JPRB, 0.21619E+00_JPRB, 0.21604E+00_JPRB, 0.21662E+00_JPRB, 0.21711E+00_JPRB ,&
&  0.21234E+00_JPRB, 0.20706E+00_JPRB, 0.20844E+00_JPRB, 0.20947E+00_JPRB, 0.20682E+00_JPRB, 0.20808E+00_JPRB, 0.20875E+00_JPRB ,&
&  0.20970E+00_JPRB, 0.21102E+00_JPRB, 0.21239E+00_JPRB, 0.21303E+00_JPRB, 0.21343E+00_JPRB, 0.21411E+00_JPRB, 0.21348E+00_JPRB ,&
&  0.21142E+00_JPRB, 0.20973E+00_JPRB, 0.20828E+00_JPRB, 0.20681E+00_JPRB, 0.20655E+00_JPRB, 0.20837E+00_JPRB, 0.20952E+00_JPRB ,&
&  0.20866E+00_JPRB, 0.20725E+00_JPRB, 0.20592E+00_JPRB, 0.20636E+00_JPRB, 0.20724E+00_JPRB, 0.20772E+00_JPRB, 0.20552E+00_JPRB ,&
&  0.20443E+00_JPRB, 0.20156E+00_JPRB, 0.19896E+00_JPRB, 0.19638E+00_JPRB, 0.19518E+00_JPRB, 0.19149E+00_JPRB, 0.18753E+00_JPRB ,&
&  0.18675E+00_JPRB, 0.19101E+00_JPRB, 0.19740E+00_JPRB, 0.19930E+00_JPRB, 0.18729E+00_JPRB, 0.18891E+00_JPRB, 0.18950E+00_JPRB ,&
&  0.18492E+00_JPRB, 0.17950E+00_JPRB, 0.17550E+00_JPRB, 0.17381E+00_JPRB, 0.17314E+00_JPRB, 0.17247E+00_JPRB, 0.17286E+00_JPRB ,&
&  0.17370E+00_JPRB, 0.17409E+00_JPRB, 0.17465E+00_JPRB, 0.17589E+00_JPRB, 0.17714E+00_JPRB, 0.17841E+00_JPRB, 0.17957E+00_JPRB ,&
&  0.18216E+00_JPRB, 0.18531E+00_JPRB, 0.18767E+00_JPRB, 0.19072E+00_JPRB, 0.19397E+00_JPRB, 0.19714E+00_JPRB, 0.20014E+00_JPRB ,&
&  0.20345E+00_JPRB, 0.20518E+00_JPRB, 0.20686E+00_JPRB, 0.20989E+00_JPRB, 0.21138E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.20643E+00_JPRB, 0.20923E+00_JPRB, 0.21116E+00_JPRB, 0.21300E+00_JPRB, 0.21444E+00_JPRB, 0.21593E+00_JPRB, 0.21660E+00_JPRB ,&
&  0.21720E+00_JPRB, 0.21718E+00_JPRB, 0.21762E+00_JPRB, 0.21760E+00_JPRB, 0.21780E+00_JPRB, 0.21664E+00_JPRB, 0.21612E+00_JPRB ,&
&  0.21569E+00_JPRB, 0.21613E+00_JPRB, 0.21619E+00_JPRB, 0.21614E+00_JPRB, 0.21544E+00_JPRB, 0.21450E+00_JPRB, 0.21292E+00_JPRB ,&
&  0.20962E+00_JPRB, 0.20717E+00_JPRB, 0.20729E+00_JPRB, 0.20663E+00_JPRB, 0.20701E+00_JPRB, 0.20695E+00_JPRB, 0.20742E+00_JPRB ,&
&  0.20285E+00_JPRB, 0.20139E+00_JPRB, 0.20201E+00_JPRB, 0.20085E+00_JPRB, 0.20258E+00_JPRB, 0.20336E+00_JPRB, 0.20516E+00_JPRB ,&
&  0.20549E+00_JPRB, 0.20568E+00_JPRB, 0.20642E+00_JPRB, 0.20583E+00_JPRB, 0.20608E+00_JPRB, 0.20653E+00_JPRB, 0.20532E+00_JPRB ,&
&  0.20213E+00_JPRB, 0.19979E+00_JPRB, 0.19640E+00_JPRB, 0.18818E+00_JPRB, 0.18656E+00_JPRB, 0.18723E+00_JPRB, 0.18604E+00_JPRB ,&
&  0.18332E+00_JPRB, 0.18399E+00_JPRB, 0.18357E+00_JPRB, 0.18196E+00_JPRB, 0.18411E+00_JPRB, 0.18715E+00_JPRB, 0.18732E+00_JPRB ,&
&  0.18597E+00_JPRB, 0.18564E+00_JPRB, 0.18495E+00_JPRB, 0.18358E+00_JPRB, 0.18197E+00_JPRB, 0.18133E+00_JPRB, 0.18072E+00_JPRB ,&
&  0.18037E+00_JPRB, 0.17886E+00_JPRB, 0.18242E+00_JPRB, 0.18308E+00_JPRB, 0.17783E+00_JPRB, 0.18054E+00_JPRB, 0.17823E+00_JPRB ,&
&  0.17081E+00_JPRB, 0.16755E+00_JPRB, 0.16564E+00_JPRB, 0.16400E+00_JPRB, 0.16362E+00_JPRB, 0.16363E+00_JPRB, 0.16335E+00_JPRB ,&
&  0.16417E+00_JPRB, 0.16528E+00_JPRB, 0.16580E+00_JPRB, 0.16654E+00_JPRB, 0.16758E+00_JPRB, 0.16911E+00_JPRB, 0.17115E+00_JPRB ,&
&  0.17296E+00_JPRB, 0.17544E+00_JPRB, 0.17666E+00_JPRB, 0.17866E+00_JPRB, 0.18014E+00_JPRB, 0.18184E+00_JPRB, 0.18597E+00_JPRB ,&
&  0.18925E+00_JPRB, 0.19568E+00_JPRB, 0.20437E+00_JPRB, 0.20681E+00_JPRB, 0.20371E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.20125E+00_JPRB, 0.20338E+00_JPRB, 0.20546E+00_JPRB, 0.20713E+00_JPRB, 0.20885E+00_JPRB, 0.21027E+00_JPRB, 0.21101E+00_JPRB ,&
&  0.21113E+00_JPRB, 0.21041E+00_JPRB, 0.21040E+00_JPRB, 0.20889E+00_JPRB, 0.20745E+00_JPRB, 0.20316E+00_JPRB, 0.20276E+00_JPRB ,&
&  0.20179E+00_JPRB, 0.20253E+00_JPRB, 0.20208E+00_JPRB, 0.20240E+00_JPRB, 0.20162E+00_JPRB, 0.20121E+00_JPRB, 0.19936E+00_JPRB ,&
&  0.19666E+00_JPRB, 0.19467E+00_JPRB, 0.19280E+00_JPRB, 0.19231E+00_JPRB, 0.19316E+00_JPRB, 0.19374E+00_JPRB, 0.19336E+00_JPRB ,&
&  0.19339E+00_JPRB, 0.19364E+00_JPRB, 0.19406E+00_JPRB, 0.19398E+00_JPRB, 0.19936E+00_JPRB, 0.20164E+00_JPRB, 0.20277E+00_JPRB ,&
&  0.20246E+00_JPRB, 0.20263E+00_JPRB, 0.20261E+00_JPRB, 0.20153E+00_JPRB, 0.20046E+00_JPRB, 0.19585E+00_JPRB, 0.19298E+00_JPRB ,&
&  0.18510E+00_JPRB, 0.17942E+00_JPRB, 0.17512E+00_JPRB, 0.17249E+00_JPRB, 0.17148E+00_JPRB, 0.17161E+00_JPRB, 0.17125E+00_JPRB ,&
&  0.16968E+00_JPRB, 0.16821E+00_JPRB, 0.16730E+00_JPRB, 0.16813E+00_JPRB, 0.16944E+00_JPRB, 0.17086E+00_JPRB, 0.17246E+00_JPRB ,&
&  0.17385E+00_JPRB, 0.17423E+00_JPRB, 0.17422E+00_JPRB, 0.17474E+00_JPRB, 0.17515E+00_JPRB, 0.17575E+00_JPRB, 0.17618E+00_JPRB ,&
&  0.17729E+00_JPRB, 0.17633E+00_JPRB, 0.17702E+00_JPRB, 0.17830E+00_JPRB, 0.17574E+00_JPRB, 0.17169E+00_JPRB, 0.16769E+00_JPRB ,&
&  0.16385E+00_JPRB, 0.16169E+00_JPRB, 0.15990E+00_JPRB, 0.15872E+00_JPRB, 0.15840E+00_JPRB, 0.15841E+00_JPRB, 0.15889E+00_JPRB ,&
&  0.15956E+00_JPRB, 0.16056E+00_JPRB, 0.16120E+00_JPRB, 0.16211E+00_JPRB, 0.16322E+00_JPRB, 0.16447E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16780E+00_JPRB, 0.16953E+00_JPRB, 0.17077E+00_JPRB, 0.17250E+00_JPRB, 0.17571E+00_JPRB, 0.17994E+00_JPRB, 0.18629E+00_JPRB ,&
&  0.19318E+00_JPRB, 0.19841E+00_JPRB, 0.19498E+00_JPRB, 0.19791E+00_JPRB, 0.20018E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.20122E+00_JPRB, 0.20295E+00_JPRB, 0.20450E+00_JPRB, 0.20608E+00_JPRB, 0.20738E+00_JPRB, 0.20858E+00_JPRB, 0.20936E+00_JPRB ,&
&  0.20662E+00_JPRB, 0.20513E+00_JPRB, 0.20233E+00_JPRB, 0.19995E+00_JPRB, 0.19707E+00_JPRB, 0.19383E+00_JPRB, 0.19281E+00_JPRB ,&
&  0.19113E+00_JPRB, 0.19082E+00_JPRB, 0.18938E+00_JPRB, 0.18876E+00_JPRB, 0.18755E+00_JPRB, 0.18678E+00_JPRB, 0.18544E+00_JPRB ,&
&  0.18300E+00_JPRB, 0.18180E+00_JPRB, 0.18067E+00_JPRB, 0.18101E+00_JPRB, 0.18257E+00_JPRB, 0.18420E+00_JPRB, 0.18596E+00_JPRB ,&
&  0.18851E+00_JPRB, 0.18954E+00_JPRB, 0.19198E+00_JPRB, 0.19384E+00_JPRB, 0.19713E+00_JPRB, 0.19988E+00_JPRB, 0.20048E+00_JPRB ,&
&  0.20040E+00_JPRB, 0.19814E+00_JPRB, 0.19730E+00_JPRB, 0.19478E+00_JPRB, 0.18842E+00_JPRB, 0.18499E+00_JPRB, 0.18002E+00_JPRB ,&
&  0.17452E+00_JPRB, 0.17043E+00_JPRB, 0.16759E+00_JPRB, 0.16676E+00_JPRB, 0.16585E+00_JPRB, 0.16567E+00_JPRB, 0.16557E+00_JPRB ,&
&  0.16480E+00_JPRB, 0.16389E+00_JPRB, 0.16281E+00_JPRB, 0.16226E+00_JPRB, 0.16277E+00_JPRB, 0.16368E+00_JPRB, 0.16558E+00_JPRB ,&
&  0.16744E+00_JPRB, 0.16851E+00_JPRB, 0.16966E+00_JPRB, 0.17005E+00_JPRB, 0.17019E+00_JPRB, 0.17047E+00_JPRB, 0.17055E+00_JPRB ,&
&  0.17132E+00_JPRB, 0.17150E+00_JPRB, 0.17096E+00_JPRB, 0.17003E+00_JPRB, 0.16763E+00_JPRB, 0.16444E+00_JPRB, 0.16197E+00_JPRB ,&
&  0.15983E+00_JPRB, 0.15843E+00_JPRB, 0.15793E+00_JPRB, 0.15739E+00_JPRB, 0.15762E+00_JPRB, 0.15799E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15890E+00_JPRB, 0.15942E+00_JPRB, 0.15994E+00_JPRB, 0.16120E+00_JPRB, 0.16225E+00_JPRB, 0.16383E+00_JPRB, 0.16549E+00_JPRB ,&
&  0.16664E+00_JPRB, 0.16838E+00_JPRB, 0.16986E+00_JPRB, 0.17149E+00_JPRB, 0.17638E+00_JPRB, 0.18021E+00_JPRB, 0.18798E+00_JPRB ,&
&  0.19276E+00_JPRB, 0.19392E+00_JPRB, 0.19369E+00_JPRB, 0.19975E+00_JPRB, 0.20042E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.20158E+00_JPRB, 0.20226E+00_JPRB, 0.20227E+00_JPRB, 0.20329E+00_JPRB, 0.20402E+00_JPRB, 0.20481E+00_JPRB, 0.20393E+00_JPRB ,&
&  0.20319E+00_JPRB, 0.20178E+00_JPRB, 0.20065E+00_JPRB, 0.19748E+00_JPRB, 0.19498E+00_JPRB, 0.19192E+00_JPRB, 0.18927E+00_JPRB ,&
&  0.18691E+00_JPRB, 0.18465E+00_JPRB, 0.18253E+00_JPRB, 0.18070E+00_JPRB, 0.17898E+00_JPRB, 0.17744E+00_JPRB, 0.17592E+00_JPRB ,&
&  0.17368E+00_JPRB, 0.17307E+00_JPRB, 0.17308E+00_JPRB, 0.17443E+00_JPRB, 0.17721E+00_JPRB, 0.17895E+00_JPRB, 0.18105E+00_JPRB ,&
&  0.18326E+00_JPRB, 0.18238E+00_JPRB, 0.18563E+00_JPRB, 0.18873E+00_JPRB, 0.19130E+00_JPRB, 0.19345E+00_JPRB, 0.19497E+00_JPRB ,&
&  0.19553E+00_JPRB, 0.18989E+00_JPRB, 0.19089E+00_JPRB, 0.18611E+00_JPRB, 0.18117E+00_JPRB, 0.17914E+00_JPRB, 0.17507E+00_JPRB ,&
&  0.17344E+00_JPRB, 0.17025E+00_JPRB, 0.16821E+00_JPRB, 0.16644E+00_JPRB, 0.16428E+00_JPRB, 0.16382E+00_JPRB, 0.16301E+00_JPRB ,&
&  0.16260E+00_JPRB, 0.16277E+00_JPRB, 0.16201E+00_JPRB, 0.16181E+00_JPRB, 0.16140E+00_JPRB, 0.16105E+00_JPRB, 0.16159E+00_JPRB ,&
&  0.16236E+00_JPRB, 0.16323E+00_JPRB, 0.16404E+00_JPRB, 0.16416E+00_JPRB, 0.16408E+00_JPRB, 0.16406E+00_JPRB, 0.16398E+00_JPRB ,&
&  0.16430E+00_JPRB, 0.16434E+00_JPRB, 0.16393E+00_JPRB, 0.16345E+00_JPRB, 0.16230E+00_JPRB, 0.16114E+00_JPRB, 0.16000E+00_JPRB ,&
&  0.15897E+00_JPRB, 0.15841E+00_JPRB, 0.15793E+00_JPRB, 0.15777E+00_JPRB, 0.15777E+00_JPRB, 0.15809E+00_JPRB, 0.15843E+00_JPRB ,&
&  0.15874E+00_JPRB, 0.15953E+00_JPRB, 0.16010E+00_JPRB, 0.16105E+00_JPRB, 0.16221E+00_JPRB, 0.16324E+00_JPRB, 0.16481E+00_JPRB ,&
&  0.16653E+00_JPRB, 0.16839E+00_JPRB, 0.17107E+00_JPRB, 0.17337E+00_JPRB, 0.17631E+00_JPRB, 0.18005E+00_JPRB, 0.18277E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18834E+00_JPRB, 0.18830E+00_JPRB, 0.19592E+00_JPRB, 0.20017E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.19339E+00_JPRB, 0.19425E+00_JPRB, 0.19468E+00_JPRB, 0.19508E+00_JPRB, 0.19618E+00_JPRB, 0.19616E+00_JPRB, 0.19592E+00_JPRB ,&
&  0.19367E+00_JPRB, 0.19284E+00_JPRB, 0.19146E+00_JPRB, 0.18952E+00_JPRB, 0.18750E+00_JPRB, 0.18412E+00_JPRB, 0.18208E+00_JPRB ,&
&  0.18010E+00_JPRB, 0.17801E+00_JPRB, 0.17665E+00_JPRB, 0.17485E+00_JPRB, 0.17330E+00_JPRB, 0.17152E+00_JPRB, 0.17093E+00_JPRB ,&
&  0.17003E+00_JPRB, 0.16995E+00_JPRB, 0.17081E+00_JPRB, 0.17146E+00_JPRB, 0.17270E+00_JPRB, 0.17400E+00_JPRB, 0.17570E+00_JPRB ,&
&  0.17603E+00_JPRB, 0.17677E+00_JPRB, 0.17753E+00_JPRB, 0.17806E+00_JPRB, 0.17996E+00_JPRB, 0.18131E+00_JPRB, 0.18180E+00_JPRB ,&
&  0.18131E+00_JPRB, 0.18035E+00_JPRB, 0.17848E+00_JPRB, 0.17634E+00_JPRB, 0.17394E+00_JPRB, 0.17136E+00_JPRB, 0.16889E+00_JPRB ,&
&  0.16701E+00_JPRB, 0.16526E+00_JPRB, 0.16393E+00_JPRB, 0.16286E+00_JPRB, 0.16224E+00_JPRB, 0.16175E+00_JPRB, 0.16126E+00_JPRB ,&
&  0.16102E+00_JPRB, 0.16082E+00_JPRB, 0.16049E+00_JPRB, 0.16054E+00_JPRB, 0.16048E+00_JPRB, 0.16058E+00_JPRB, 0.16070E+00_JPRB ,&
&  0.16091E+00_JPRB, 0.16095E+00_JPRB, 0.16089E+00_JPRB, 0.16060E+00_JPRB, 0.16031E+00_JPRB, 0.15997E+00_JPRB, 0.15964E+00_JPRB ,&
&  0.15929E+00_JPRB, 0.15928E+00_JPRB, 0.15900E+00_JPRB, 0.15880E+00_JPRB, 0.15854E+00_JPRB, 0.15806E+00_JPRB, 0.15791E+00_JPRB ,&
&  0.15799E+00_JPRB, 0.15813E+00_JPRB, 0.15795E+00_JPRB, 0.15804E+00_JPRB, 0.15823E+00_JPRB, 0.15853E+00_JPRB, 0.15938E+00_JPRB ,&
&  0.15990E+00_JPRB, 0.16045E+00_JPRB, 0.16095E+00_JPRB, 0.16167E+00_JPRB, 0.16291E+00_JPRB, 0.16417E+00_JPRB, 0.16557E+00_JPRB ,&
&  0.16727E+00_JPRB, 0.16884E+00_JPRB, 0.17041E+00_JPRB, 0.17197E+00_JPRB, 0.17457E+00_JPRB, 0.17639E+00_JPRB, 0.17880E+00_JPRB ,&
&  0.18088E+00_JPRB, 0.17948E+00_JPRB, 0.18804E+00_JPRB, 0.18873E+00_JPRB, 0.18999E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.17696E+00_JPRB, 0.17699E+00_JPRB, 0.17607E+00_JPRB, 0.17505E+00_JPRB, 0.16793E+00_JPRB, 0.17438E+00_JPRB, 0.17373E+00_JPRB ,&
&  0.17433E+00_JPRB, 0.17463E+00_JPRB, 0.17617E+00_JPRB, 0.17645E+00_JPRB, 0.17620E+00_JPRB, 0.17612E+00_JPRB, 0.17589E+00_JPRB ,&
&  0.17559E+00_JPRB, 0.17480E+00_JPRB, 0.17443E+00_JPRB, 0.17385E+00_JPRB, 0.17326E+00_JPRB, 0.17224E+00_JPRB, 0.17116E+00_JPRB ,&
&  0.17066E+00_JPRB, 0.17072E+00_JPRB, 0.17110E+00_JPRB, 0.16930E+00_JPRB, 0.16848E+00_JPRB, 0.16793E+00_JPRB, 0.16765E+00_JPRB ,&
&  0.16719E+00_JPRB, 0.16704E+00_JPRB, 0.16703E+00_JPRB, 0.16696E+00_JPRB, 0.16836E+00_JPRB, 0.16879E+00_JPRB, 0.16912E+00_JPRB ,&
&  0.16919E+00_JPRB, 0.16905E+00_JPRB, 0.16835E+00_JPRB, 0.16771E+00_JPRB, 0.16705E+00_JPRB, 0.16590E+00_JPRB, 0.16470E+00_JPRB ,&
&  0.16367E+00_JPRB, 0.16290E+00_JPRB, 0.16201E+00_JPRB, 0.16133E+00_JPRB, 0.16078E+00_JPRB, 0.16026E+00_JPRB, 0.15993E+00_JPRB ,&
&  0.15985E+00_JPRB, 0.15988E+00_JPRB, 0.15989E+00_JPRB, 0.15989E+00_JPRB, 0.15986E+00_JPRB, 0.15985E+00_JPRB, 0.15986E+00_JPRB ,&
&  0.16024E+00_JPRB, 0.16024E+00_JPRB, 0.16009E+00_JPRB, 0.15980E+00_JPRB, 0.15970E+00_JPRB, 0.15938E+00_JPRB, 0.15917E+00_JPRB ,&
&  0.15900E+00_JPRB, 0.15878E+00_JPRB, 0.15868E+00_JPRB, 0.15867E+00_JPRB, 0.15868E+00_JPRB, 0.15868E+00_JPRB, 0.15878E+00_JPRB ,&
&  0.15895E+00_JPRB, 0.15915E+00_JPRB, 0.15954E+00_JPRB, 0.15984E+00_JPRB, 0.16011E+00_JPRB, 0.16035E+00_JPRB, 0.16086E+00_JPRB ,&
&  0.16141E+00_JPRB, 0.16194E+00_JPRB, 0.16247E+00_JPRB, 0.16282E+00_JPRB, 0.16359E+00_JPRB, 0.16437E+00_JPRB, 0.16523E+00_JPRB ,&
&  0.16615E+00_JPRB, 0.16731E+00_JPRB, 0.16853E+00_JPRB, 0.16975E+00_JPRB, 0.17137E+00_JPRB, 0.17267E+00_JPRB, 0.17395E+00_JPRB ,&
&  0.17537E+00_JPRB, 0.17801E+00_JPRB, 0.17867E+00_JPRB, 0.17874E+00_JPRB, 0.17811E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.17204E+00_JPRB, 0.17200E+00_JPRB, 0.17185E+00_JPRB, 0.17142E+00_JPRB, 0.17064E+00_JPRB, 0.17045E+00_JPRB, 0.17020E+00_JPRB ,&
&  0.16986E+00_JPRB, 0.16915E+00_JPRB, 0.16914E+00_JPRB, 0.16905E+00_JPRB, 0.16895E+00_JPRB, 0.16881E+00_JPRB, 0.16855E+00_JPRB ,&
&  0.16801E+00_JPRB, 0.16719E+00_JPRB, 0.16477E+00_JPRB, 0.16361E+00_JPRB, 0.16285E+00_JPRB, 0.16244E+00_JPRB, 0.16208E+00_JPRB ,&
&  0.16187E+00_JPRB, 0.16176E+00_JPRB, 0.16163E+00_JPRB, 0.16207E+00_JPRB, 0.16226E+00_JPRB, 0.16232E+00_JPRB, 0.16237E+00_JPRB ,&
&  0.16236E+00_JPRB, 0.16245E+00_JPRB, 0.16246E+00_JPRB, 0.16245E+00_JPRB, 0.16257E+00_JPRB, 0.16257E+00_JPRB, 0.16237E+00_JPRB ,&
&  0.16215E+00_JPRB, 0.16188E+00_JPRB, 0.16162E+00_JPRB, 0.16129E+00_JPRB, 0.16106E+00_JPRB, 0.16087E+00_JPRB, 0.16079E+00_JPRB ,&
&  0.16063E+00_JPRB, 0.16050E+00_JPRB, 0.16037E+00_JPRB, 0.16031E+00_JPRB, 0.16020E+00_JPRB, 0.16015E+00_JPRB, 0.16002E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16014E+00_JPRB, 0.16022E+00_JPRB, 0.16025E+00_JPRB, 0.16029E+00_JPRB, 0.16027E+00_JPRB, 0.16021E+00_JPRB ,&
&  0.16005E+00_JPRB, 0.15996E+00_JPRB, 0.15984E+00_JPRB, 0.15976E+00_JPRB, 0.15974E+00_JPRB, 0.15973E+00_JPRB, 0.15974E+00_JPRB ,&
&  0.15975E+00_JPRB, 0.15987E+00_JPRB, 0.15991E+00_JPRB, 0.15997E+00_JPRB, 0.16004E+00_JPRB, 0.16012E+00_JPRB, 0.16018E+00_JPRB ,&
&  0.16028E+00_JPRB, 0.16036E+00_JPRB, 0.16047E+00_JPRB, 0.16062E+00_JPRB, 0.16080E+00_JPRB, 0.16105E+00_JPRB, 0.16135E+00_JPRB ,&
&  0.16167E+00_JPRB, 0.16199E+00_JPRB, 0.16230E+00_JPRB, 0.16261E+00_JPRB, 0.16310E+00_JPRB, 0.16359E+00_JPRB, 0.16420E+00_JPRB ,&
&  0.16479E+00_JPRB, 0.16537E+00_JPRB, 0.16590E+00_JPRB, 0.16647E+00_JPRB, 0.16726E+00_JPRB, 0.16765E+00_JPRB, 0.16798E+00_JPRB ,&
&  0.16850E+00_JPRB, 0.16904E+00_JPRB, 0.16957E+00_JPRB, 0.17003E+00_JPRB, 0.17077E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB, 0.16191E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB, 0.56086E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.56369E+00_JPRB, 0.56230E+00_JPRB, 0.56113E+00_JPRB, 0.55978E+00_JPRB, 0.55819E+00_JPRB, 0.55688E+00_JPRB, 0.55565E+00_JPRB ,&
&  0.55460E+00_JPRB, 0.55400E+00_JPRB, 0.55370E+00_JPRB, 0.55354E+00_JPRB, 0.55360E+00_JPRB, 0.55360E+00_JPRB, 0.55367E+00_JPRB ,&
&  0.55364E+00_JPRB, 0.55372E+00_JPRB, 0.55376E+00_JPRB, 0.55351E+00_JPRB, 0.55306E+00_JPRB, 0.55249E+00_JPRB, 0.55171E+00_JPRB ,&
&  0.55056E+00_JPRB, 0.54941E+00_JPRB, 0.54837E+00_JPRB, 0.54726E+00_JPRB, 0.54627E+00_JPRB, 0.54517E+00_JPRB, 0.54400E+00_JPRB ,&
&  0.54298E+00_JPRB, 0.54247E+00_JPRB, 0.54211E+00_JPRB, 0.54174E+00_JPRB, 0.54169E+00_JPRB, 0.54256E+00_JPRB, 0.54335E+00_JPRB ,&
&  0.54423E+00_JPRB, 0.54530E+00_JPRB, 0.54720E+00_JPRB, 0.54908E+00_JPRB, 0.55084E+00_JPRB, 0.55247E+00_JPRB, 0.55380E+00_JPRB ,&
&  0.55421E+00_JPRB, 0.55415E+00_JPRB, 0.55377E+00_JPRB, 0.55334E+00_JPRB, 0.55242E+00_JPRB, 0.55168E+00_JPRB, 0.55107E+00_JPRB ,&
&  0.55031E+00_JPRB, 0.54965E+00_JPRB, 0.54896E+00_JPRB, 0.54781E+00_JPRB, 0.54538E+00_JPRB, 0.54359E+00_JPRB, 0.54266E+00_JPRB ,&
&  0.54244E+00_JPRB, 0.54232E+00_JPRB, 0.54258E+00_JPRB, 0.54313E+00_JPRB, 0.54426E+00_JPRB, 0.54555E+00_JPRB, 0.54713E+00_JPRB ,&
&  0.54888E+00_JPRB, 0.55107E+00_JPRB, 0.55397E+00_JPRB, 0.55702E+00_JPRB, 0.56035E+00_JPRB, 0.56420E+00_JPRB, 0.56771E+00_JPRB ,&
&  0.57124E+00_JPRB, 0.57495E+00_JPRB, 0.57814E+00_JPRB, 0.58131E+00_JPRB, 0.58450E+00_JPRB, 0.58786E+00_JPRB, 0.59048E+00_JPRB ,&
&  0.59188E+00_JPRB, 0.59296E+00_JPRB, 0.59379E+00_JPRB, 0.59353E+00_JPRB, 0.59253E+00_JPRB, 0.59173E+00_JPRB, 0.59086E+00_JPRB ,&
&  0.58971E+00_JPRB, 0.58816E+00_JPRB, 0.58669E+00_JPRB, 0.58502E+00_JPRB, 0.58266E+00_JPRB, 0.58008E+00_JPRB, 0.57762E+00_JPRB ,&
&  0.57518E+00_JPRB, 0.57252E+00_JPRB, 0.57011E+00_JPRB, 0.56783E+00_JPRB, 0.56572E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.58031E+00_JPRB, 0.57449E+00_JPRB, 0.56931E+00_JPRB, 0.56462E+00_JPRB, 0.56086E+00_JPRB, 0.55805E+00_JPRB, 0.55581E+00_JPRB ,&
&  0.55400E+00_JPRB, 0.55327E+00_JPRB, 0.55243E+00_JPRB, 0.55159E+00_JPRB, 0.55072E+00_JPRB, 0.54917E+00_JPRB, 0.54720E+00_JPRB ,&
&  0.54538E+00_JPRB, 0.54415E+00_JPRB, 0.54285E+00_JPRB, 0.54132E+00_JPRB, 0.53896E+00_JPRB, 0.53556E+00_JPRB, 0.53162E+00_JPRB ,&
&  0.52726E+00_JPRB, 0.52193E+00_JPRB, 0.51578E+00_JPRB, 0.50951E+00_JPRB, 0.50635E+00_JPRB, 0.50265E+00_JPRB, 0.49914E+00_JPRB ,&
&  0.49619E+00_JPRB, 0.49556E+00_JPRB, 0.49508E+00_JPRB, 0.49462E+00_JPRB, 0.49460E+00_JPRB, 0.49636E+00_JPRB, 0.49760E+00_JPRB ,&
&  0.49893E+00_JPRB, 0.50153E+00_JPRB, 0.50718E+00_JPRB, 0.51329E+00_JPRB, 0.52031E+00_JPRB, 0.52870E+00_JPRB, 0.53833E+00_JPRB ,&
&  0.54538E+00_JPRB, 0.54899E+00_JPRB, 0.55164E+00_JPRB, 0.55398E+00_JPRB, 0.55483E+00_JPRB, 0.55485E+00_JPRB, 0.55410E+00_JPRB ,&
&  0.55451E+00_JPRB, 0.55447E+00_JPRB, 0.55458E+00_JPRB, 0.55470E+00_JPRB, 0.55303E+00_JPRB, 0.55149E+00_JPRB, 0.54997E+00_JPRB ,&
&  0.54777E+00_JPRB, 0.54664E+00_JPRB, 0.54675E+00_JPRB, 0.54705E+00_JPRB, 0.54864E+00_JPRB, 0.55157E+00_JPRB, 0.55563E+00_JPRB ,&
&  0.56161E+00_JPRB, 0.56999E+00_JPRB, 0.58040E+00_JPRB, 0.59204E+00_JPRB, 0.60538E+00_JPRB, 0.62209E+00_JPRB, 0.63544E+00_JPRB ,&
&  0.64760E+00_JPRB, 0.65815E+00_JPRB, 0.66776E+00_JPRB, 0.67494E+00_JPRB, 0.68038E+00_JPRB, 0.68576E+00_JPRB, 0.68634E+00_JPRB ,&
&  0.68584E+00_JPRB, 0.68468E+00_JPRB, 0.68382E+00_JPRB, 0.68051E+00_JPRB, 0.67635E+00_JPRB, 0.67196E+00_JPRB, 0.66785E+00_JPRB ,&
&  0.66157E+00_JPRB, 0.65511E+00_JPRB, 0.64871E+00_JPRB, 0.64242E+00_JPRB, 0.63535E+00_JPRB, 0.62849E+00_JPRB, 0.62170E+00_JPRB ,&
&  0.61484E+00_JPRB, 0.60757E+00_JPRB, 0.60063E+00_JPRB, 0.59389E+00_JPRB, 0.58715E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.59893E+00_JPRB, 0.58890E+00_JPRB, 0.57984E+00_JPRB, 0.57195E+00_JPRB, 0.56522E+00_JPRB, 0.56015E+00_JPRB, 0.55634E+00_JPRB ,&
&  0.55352E+00_JPRB, 0.55233E+00_JPRB, 0.55106E+00_JPRB, 0.54931E+00_JPRB, 0.54722E+00_JPRB, 0.54677E+00_JPRB, 0.54486E+00_JPRB ,&
&  0.54183E+00_JPRB, 0.53777E+00_JPRB, 0.53360E+00_JPRB, 0.52834E+00_JPRB, 0.52138E+00_JPRB, 0.51352E+00_JPRB, 0.50399E+00_JPRB ,&
&  0.49752E+00_JPRB, 0.48874E+00_JPRB, 0.47908E+00_JPRB, 0.46861E+00_JPRB, 0.46128E+00_JPRB, 0.44480E+00_JPRB, 0.43850E+00_JPRB ,&
&  0.43415E+00_JPRB, 0.42568E+00_JPRB, 0.42204E+00_JPRB, 0.42184E+00_JPRB, 0.43647E+00_JPRB, 0.44896E+00_JPRB, 0.45504E+00_JPRB ,&
&  0.44346E+00_JPRB, 0.44565E+00_JPRB, 0.44310E+00_JPRB, 0.44145E+00_JPRB, 0.44363E+00_JPRB, 0.44974E+00_JPRB, 0.47236E+00_JPRB ,&
&  0.51244E+00_JPRB, 0.52774E+00_JPRB, 0.53569E+00_JPRB, 0.54226E+00_JPRB, 0.54504E+00_JPRB, 0.54775E+00_JPRB, 0.54818E+00_JPRB ,&
&  0.54923E+00_JPRB, 0.54952E+00_JPRB, 0.55198E+00_JPRB, 0.55351E+00_JPRB, 0.55610E+00_JPRB, 0.55867E+00_JPRB, 0.56108E+00_JPRB ,&
&  0.55835E+00_JPRB, 0.55670E+00_JPRB, 0.55605E+00_JPRB, 0.55763E+00_JPRB, 0.56112E+00_JPRB, 0.56915E+00_JPRB, 0.57932E+00_JPRB ,&
&  0.59076E+00_JPRB, 0.60003E+00_JPRB, 0.61526E+00_JPRB, 0.63576E+00_JPRB, 0.66027E+00_JPRB, 0.69049E+00_JPRB, 0.72108E+00_JPRB ,&
&  0.75162E+00_JPRB, 0.77717E+00_JPRB, 0.80223E+00_JPRB, 0.81569E+00_JPRB, 0.82383E+00_JPRB, 0.82972E+00_JPRB, 0.83062E+00_JPRB ,&
&  0.82987E+00_JPRB, 0.82603E+00_JPRB, 0.82160E+00_JPRB, 0.81499E+00_JPRB, 0.80593E+00_JPRB, 0.79353E+00_JPRB, 0.77941E+00_JPRB ,&
&  0.76271E+00_JPRB, 0.74630E+00_JPRB, 0.72999E+00_JPRB, 0.71392E+00_JPRB, 0.69796E+00_JPRB, 0.68326E+00_JPRB, 0.67013E+00_JPRB ,&
&  0.65763E+00_JPRB, 0.64501E+00_JPRB, 0.63329E+00_JPRB, 0.62193E+00_JPRB, 0.61050E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.61770E+00_JPRB, 0.60642E+00_JPRB, 0.59446E+00_JPRB, 0.58373E+00_JPRB, 0.57481E+00_JPRB, 0.56817E+00_JPRB, 0.56366E+00_JPRB ,&
&  0.56022E+00_JPRB, 0.55611E+00_JPRB, 0.55184E+00_JPRB, 0.54730E+00_JPRB, 0.54294E+00_JPRB, 0.54153E+00_JPRB, 0.54019E+00_JPRB ,&
&  0.53889E+00_JPRB, 0.53423E+00_JPRB, 0.52758E+00_JPRB, 0.51824E+00_JPRB, 0.50850E+00_JPRB, 0.50026E+00_JPRB, 0.49152E+00_JPRB ,&
&  0.48222E+00_JPRB, 0.47202E+00_JPRB, 0.45511E+00_JPRB, 0.44365E+00_JPRB, 0.42552E+00_JPRB, 0.41421E+00_JPRB, 0.40794E+00_JPRB ,&
&  0.40429E+00_JPRB, 0.40609E+00_JPRB, 0.41078E+00_JPRB, 0.41577E+00_JPRB, 0.41434E+00_JPRB, 0.40964E+00_JPRB, 0.40723E+00_JPRB ,&
&  0.40120E+00_JPRB, 0.39730E+00_JPRB, 0.39456E+00_JPRB, 0.39398E+00_JPRB, 0.39619E+00_JPRB, 0.40308E+00_JPRB, 0.41737E+00_JPRB ,&
&  0.45014E+00_JPRB, 0.49248E+00_JPRB, 0.53175E+00_JPRB, 0.54941E+00_JPRB, 0.54354E+00_JPRB, 0.54604E+00_JPRB, 0.54305E+00_JPRB ,&
&  0.54212E+00_JPRB, 0.53957E+00_JPRB, 0.54006E+00_JPRB, 0.54514E+00_JPRB, 0.55157E+00_JPRB, 0.55777E+00_JPRB, 0.56440E+00_JPRB ,&
&  0.56672E+00_JPRB, 0.56955E+00_JPRB, 0.56605E+00_JPRB, 0.56849E+00_JPRB, 0.57698E+00_JPRB, 0.58747E+00_JPRB, 0.59260E+00_JPRB ,&
&  0.60200E+00_JPRB, 0.61519E+00_JPRB, 0.64040E+00_JPRB, 0.67837E+00_JPRB, 0.71900E+00_JPRB, 0.75665E+00_JPRB, 0.79175E+00_JPRB ,&
&  0.82602E+00_JPRB, 0.86839E+00_JPRB, 0.90966E+00_JPRB, 0.92739E+00_JPRB, 0.92888E+00_JPRB, 0.93481E+00_JPRB, 0.92955E+00_JPRB ,&
&  0.92592E+00_JPRB, 0.91871E+00_JPRB, 0.91585E+00_JPRB, 0.91286E+00_JPRB, 0.90092E+00_JPRB, 0.88095E+00_JPRB, 0.85978E+00_JPRB ,&
&  0.83664E+00_JPRB, 0.81090E+00_JPRB, 0.78637E+00_JPRB, 0.76133E+00_JPRB, 0.73783E+00_JPRB, 0.71487E+00_JPRB, 0.69560E+00_JPRB ,&
&  0.67703E+00_JPRB, 0.66261E+00_JPRB, 0.64931E+00_JPRB, 0.63848E+00_JPRB, 0.62823E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.63630E+00_JPRB, 0.62892E+00_JPRB, 0.62137E+00_JPRB, 0.61146E+00_JPRB, 0.60133E+00_JPRB, 0.59209E+00_JPRB, 0.58495E+00_JPRB ,&
&  0.57916E+00_JPRB, 0.57312E+00_JPRB, 0.56737E+00_JPRB, 0.56034E+00_JPRB, 0.55374E+00_JPRB, 0.54794E+00_JPRB, 0.54310E+00_JPRB ,&
&  0.53962E+00_JPRB, 0.53245E+00_JPRB, 0.52505E+00_JPRB, 0.51587E+00_JPRB, 0.50731E+00_JPRB, 0.49786E+00_JPRB, 0.48783E+00_JPRB ,&
&  0.47960E+00_JPRB, 0.46562E+00_JPRB, 0.44994E+00_JPRB, 0.44009E+00_JPRB, 0.42306E+00_JPRB, 0.41572E+00_JPRB, 0.41390E+00_JPRB ,&
&  0.43038E+00_JPRB, 0.42450E+00_JPRB, 0.43045E+00_JPRB, 0.41700E+00_JPRB, 0.40557E+00_JPRB, 0.39491E+00_JPRB, 0.38654E+00_JPRB ,&
&  0.38027E+00_JPRB, 0.37485E+00_JPRB, 0.37454E+00_JPRB, 0.37707E+00_JPRB, 0.37908E+00_JPRB, 0.38770E+00_JPRB, 0.40125E+00_JPRB ,&
&  0.43682E+00_JPRB, 0.52722E+00_JPRB, 0.54062E+00_JPRB, 0.55651E+00_JPRB, 0.55476E+00_JPRB, 0.55556E+00_JPRB, 0.55640E+00_JPRB ,&
&  0.55628E+00_JPRB, 0.55478E+00_JPRB, 0.55101E+00_JPRB, 0.54815E+00_JPRB, 0.55763E+00_JPRB, 0.56515E+00_JPRB, 0.57526E+00_JPRB ,&
&  0.57941E+00_JPRB, 0.58459E+00_JPRB, 0.58523E+00_JPRB, 0.58892E+00_JPRB, 0.60117E+00_JPRB, 0.60746E+00_JPRB, 0.60749E+00_JPRB ,&
&  0.62163E+00_JPRB, 0.64294E+00_JPRB, 0.67617E+00_JPRB, 0.71744E+00_JPRB, 0.74824E+00_JPRB, 0.77455E+00_JPRB, 0.81434E+00_JPRB ,&
&  0.84261E+00_JPRB, 0.88937E+00_JPRB, 0.93976E+00_JPRB, 0.96338E+00_JPRB, 0.96876E+00_JPRB, 0.96456E+00_JPRB, 0.95752E+00_JPRB ,&
&  0.95707E+00_JPRB, 0.94382E+00_JPRB, 0.93729E+00_JPRB, 0.93412E+00_JPRB, 0.91919E+00_JPRB, 0.89919E+00_JPRB, 0.87801E+00_JPRB ,&
&  0.85204E+00_JPRB, 0.82869E+00_JPRB, 0.80883E+00_JPRB, 0.78602E+00_JPRB, 0.76230E+00_JPRB, 0.73917E+00_JPRB, 0.71675E+00_JPRB ,&
&  0.69515E+00_JPRB, 0.67767E+00_JPRB, 0.66148E+00_JPRB, 0.65115E+00_JPRB, 0.64198E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.65289E+00_JPRB, 0.64587E+00_JPRB, 0.63937E+00_JPRB, 0.63299E+00_JPRB, 0.62564E+00_JPRB, 0.61793E+00_JPRB, 0.61000E+00_JPRB ,&
&  0.60311E+00_JPRB, 0.59672E+00_JPRB, 0.59018E+00_JPRB, 0.58159E+00_JPRB, 0.57282E+00_JPRB, 0.56306E+00_JPRB, 0.55554E+00_JPRB ,&
&  0.54779E+00_JPRB, 0.53866E+00_JPRB, 0.52507E+00_JPRB, 0.51266E+00_JPRB, 0.50391E+00_JPRB, 0.49659E+00_JPRB, 0.48733E+00_JPRB ,&
&  0.47933E+00_JPRB, 0.46796E+00_JPRB, 0.44957E+00_JPRB, 0.43264E+00_JPRB, 0.41720E+00_JPRB, 0.41602E+00_JPRB, 0.44355E+00_JPRB ,&
&  0.45722E+00_JPRB, 0.45973E+00_JPRB, 0.46079E+00_JPRB, 0.45546E+00_JPRB, 0.44889E+00_JPRB, 0.40198E+00_JPRB, 0.40018E+00_JPRB ,&
&  0.38512E+00_JPRB, 0.37373E+00_JPRB, 0.37103E+00_JPRB, 0.37101E+00_JPRB, 0.37563E+00_JPRB, 0.38752E+00_JPRB, 0.40430E+00_JPRB ,&
&  0.43855E+00_JPRB, 0.53940E+00_JPRB, 0.55521E+00_JPRB, 0.57042E+00_JPRB, 0.57233E+00_JPRB, 0.57353E+00_JPRB, 0.57717E+00_JPRB ,&
&  0.57740E+00_JPRB, 0.57999E+00_JPRB, 0.58249E+00_JPRB, 0.58709E+00_JPRB, 0.59254E+00_JPRB, 0.59272E+00_JPRB, 0.60160E+00_JPRB ,&
&  0.60817E+00_JPRB, 0.61483E+00_JPRB, 0.61715E+00_JPRB, 0.61402E+00_JPRB, 0.62035E+00_JPRB, 0.61575E+00_JPRB, 0.61416E+00_JPRB ,&
&  0.64179E+00_JPRB, 0.68479E+00_JPRB, 0.72305E+00_JPRB, 0.75918E+00_JPRB, 0.79064E+00_JPRB, 0.82620E+00_JPRB, 0.87005E+00_JPRB ,&
&  0.90068E+00_JPRB, 0.94436E+00_JPRB, 0.99711E+00_JPRB, 0.10154E+01_JPRB, 0.10151E+01_JPRB, 0.10134E+01_JPRB, 0.10177E+01_JPRB ,&
&  0.10227E+01_JPRB, 0.10301E+01_JPRB, 0.10026E+01_JPRB, 0.98632E+00_JPRB, 0.95158E+00_JPRB, 0.90523E+00_JPRB, 0.86716E+00_JPRB ,&
&  0.82051E+00_JPRB, 0.78038E+00_JPRB, 0.75584E+00_JPRB, 0.73829E+00_JPRB, 0.72517E+00_JPRB, 0.71609E+00_JPRB, 0.70791E+00_JPRB ,&
&  0.70025E+00_JPRB, 0.69039E+00_JPRB, 0.67975E+00_JPRB, 0.66919E+00_JPRB, 0.66018E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.66042E+00_JPRB, 0.65610E+00_JPRB, 0.64968E+00_JPRB, 0.64380E+00_JPRB, 0.63560E+00_JPRB, 0.62930E+00_JPRB, 0.62319E+00_JPRB ,&
&  0.61898E+00_JPRB, 0.61632E+00_JPRB, 0.61344E+00_JPRB, 0.61013E+00_JPRB, 0.60505E+00_JPRB, 0.59960E+00_JPRB, 0.59185E+00_JPRB ,&
&  0.57978E+00_JPRB, 0.54774E+00_JPRB, 0.52413E+00_JPRB, 0.51409E+00_JPRB, 0.50499E+00_JPRB, 0.49791E+00_JPRB, 0.49199E+00_JPRB ,&
&  0.48133E+00_JPRB, 0.47059E+00_JPRB, 0.45640E+00_JPRB, 0.43562E+00_JPRB, 0.42608E+00_JPRB, 0.43180E+00_JPRB, 0.43964E+00_JPRB ,&
&  0.45265E+00_JPRB, 0.46365E+00_JPRB, 0.45886E+00_JPRB, 0.45670E+00_JPRB, 0.46959E+00_JPRB, 0.46535E+00_JPRB, 0.41830E+00_JPRB ,&
&  0.39277E+00_JPRB, 0.37572E+00_JPRB, 0.37065E+00_JPRB, 0.36961E+00_JPRB, 0.37480E+00_JPRB, 0.39013E+00_JPRB, 0.43033E+00_JPRB ,&
&  0.50593E+00_JPRB, 0.59072E+00_JPRB, 0.58305E+00_JPRB, 0.58134E+00_JPRB, 0.58726E+00_JPRB, 0.59167E+00_JPRB, 0.59660E+00_JPRB ,&
&  0.60130E+00_JPRB, 0.61052E+00_JPRB, 0.62294E+00_JPRB, 0.63472E+00_JPRB, 0.64943E+00_JPRB, 0.66147E+00_JPRB, 0.66749E+00_JPRB ,&
&  0.67873E+00_JPRB, 0.66364E+00_JPRB, 0.65777E+00_JPRB, 0.64739E+00_JPRB, 0.64277E+00_JPRB, 0.62961E+00_JPRB, 0.63929E+00_JPRB ,&
&  0.67418E+00_JPRB, 0.71442E+00_JPRB, 0.75742E+00_JPRB, 0.79215E+00_JPRB, 0.82911E+00_JPRB, 0.87945E+00_JPRB, 0.93470E+00_JPRB ,&
&  0.98390E+00_JPRB, 0.10503E+01_JPRB, 0.11463E+01_JPRB, 0.11323E+01_JPRB, 0.11137E+01_JPRB, 0.11043E+01_JPRB, 0.11086E+01_JPRB ,&
&  0.11093E+01_JPRB, 0.11234E+01_JPRB, 0.10750E+01_JPRB, 0.10017E+01_JPRB, 0.93697E+00_JPRB, 0.85965E+00_JPRB, 0.81287E+00_JPRB ,&
&  0.78113E+00_JPRB, 0.73316E+00_JPRB, 0.70935E+00_JPRB, 0.69635E+00_JPRB, 0.68845E+00_JPRB, 0.67891E+00_JPRB, 0.67069E+00_JPRB ,&
&  0.66743E+00_JPRB, 0.66623E+00_JPRB, 0.66701E+00_JPRB, 0.66532E+00_JPRB, 0.66336E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.64826E+00_JPRB, 0.65213E+00_JPRB, 0.65377E+00_JPRB, 0.65017E+00_JPRB, 0.63450E+00_JPRB, 0.60545E+00_JPRB, 0.58903E+00_JPRB ,&
&  0.57933E+00_JPRB, 0.57863E+00_JPRB, 0.57675E+00_JPRB, 0.57292E+00_JPRB, 0.57303E+00_JPRB, 0.57861E+00_JPRB, 0.58452E+00_JPRB ,&
&  0.58606E+00_JPRB, 0.57896E+00_JPRB, 0.56507E+00_JPRB, 0.54087E+00_JPRB, 0.52074E+00_JPRB, 0.50505E+00_JPRB, 0.49682E+00_JPRB ,&
&  0.49322E+00_JPRB, 0.48593E+00_JPRB, 0.47416E+00_JPRB, 0.45708E+00_JPRB, 0.44418E+00_JPRB, 0.43320E+00_JPRB, 0.43628E+00_JPRB ,&
&  0.44859E+00_JPRB, 0.45419E+00_JPRB, 0.45959E+00_JPRB, 0.46707E+00_JPRB, 0.47439E+00_JPRB, 0.49410E+00_JPRB, 0.44367E+00_JPRB ,&
&  0.40866E+00_JPRB, 0.39325E+00_JPRB, 0.38152E+00_JPRB, 0.37826E+00_JPRB, 0.38580E+00_JPRB, 0.40782E+00_JPRB, 0.45526E+00_JPRB ,&
&  0.56982E+00_JPRB, 0.58766E+00_JPRB, 0.59403E+00_JPRB, 0.60644E+00_JPRB, 0.61618E+00_JPRB, 0.61963E+00_JPRB, 0.62725E+00_JPRB ,&
&  0.64266E+00_JPRB, 0.66919E+00_JPRB, 0.69619E+00_JPRB, 0.69729E+00_JPRB, 0.69522E+00_JPRB, 0.68495E+00_JPRB, 0.75881E+00_JPRB ,&
&  0.97375E+00_JPRB, 0.89123E+00_JPRB, 0.78008E+00_JPRB, 0.70863E+00_JPRB, 0.67440E+00_JPRB, 0.66023E+00_JPRB, 0.67977E+00_JPRB ,&
&  0.71200E+00_JPRB, 0.75076E+00_JPRB, 0.80627E+00_JPRB, 0.84930E+00_JPRB, 0.89231E+00_JPRB, 0.95349E+00_JPRB, 0.10393E+01_JPRB ,&
&  0.11264E+01_JPRB, 0.12459E+01_JPRB, 0.15189E+01_JPRB, 0.13299E+01_JPRB, 0.12253E+01_JPRB, 0.11915E+01_JPRB, 0.11808E+01_JPRB ,&
&  0.11455E+01_JPRB, 0.11489E+01_JPRB, 0.10728E+01_JPRB, 0.96729E+00_JPRB, 0.91077E+00_JPRB, 0.84859E+00_JPRB, 0.80327E+00_JPRB ,&
&  0.76025E+00_JPRB, 0.72137E+00_JPRB, 0.70139E+00_JPRB, 0.68482E+00_JPRB, 0.67994E+00_JPRB, 0.67446E+00_JPRB, 0.66705E+00_JPRB ,&
&  0.65788E+00_JPRB, 0.65018E+00_JPRB, 0.64484E+00_JPRB, 0.64181E+00_JPRB, 0.64229E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.61777E+00_JPRB, 0.64282E+00_JPRB, 0.66673E+00_JPRB, 0.65122E+00_JPRB, 0.62775E+00_JPRB, 0.57420E+00_JPRB, 0.53488E+00_JPRB ,&
&  0.50924E+00_JPRB, 0.50415E+00_JPRB, 0.50292E+00_JPRB, 0.50536E+00_JPRB, 0.51176E+00_JPRB, 0.52251E+00_JPRB, 0.52955E+00_JPRB ,&
&  0.53644E+00_JPRB, 0.53446E+00_JPRB, 0.53349E+00_JPRB, 0.52650E+00_JPRB, 0.51374E+00_JPRB, 0.49788E+00_JPRB, 0.48668E+00_JPRB ,&
&  0.48098E+00_JPRB, 0.47861E+00_JPRB, 0.47361E+00_JPRB, 0.46529E+00_JPRB, 0.45905E+00_JPRB, 0.45016E+00_JPRB, 0.45412E+00_JPRB ,&
&  0.44782E+00_JPRB, 0.44792E+00_JPRB, 0.46115E+00_JPRB, 0.47411E+00_JPRB, 0.48573E+00_JPRB, 0.50545E+00_JPRB, 0.46926E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.40949E+00_JPRB, 0.40232E+00_JPRB, 0.40699E+00_JPRB, 0.44693E+00_JPRB, 0.55232E+00_JPRB, 0.58852E+00_JPRB ,&
&  0.60693E+00_JPRB, 0.62085E+00_JPRB, 0.63662E+00_JPRB, 0.65729E+00_JPRB, 0.66215E+00_JPRB, 0.66689E+00_JPRB, 0.68335E+00_JPRB ,&
&  0.71012E+00_JPRB, 0.75314E+00_JPRB, 0.79366E+00_JPRB, 0.69557E+00_JPRB, 0.65571E+00_JPRB, 0.68055E+00_JPRB, 0.72280E+00_JPRB ,&
&  0.78148E+00_JPRB, 0.77134E+00_JPRB, 0.75455E+00_JPRB, 0.77498E+00_JPRB, 0.77709E+00_JPRB, 0.78033E+00_JPRB, 0.77571E+00_JPRB ,&
&  0.78246E+00_JPRB, 0.81833E+00_JPRB, 0.90304E+00_JPRB, 0.94849E+00_JPRB, 0.10193E+01_JPRB, 0.11249E+01_JPRB, 0.12178E+01_JPRB ,&
&  0.13231E+01_JPRB, 0.14008E+01_JPRB, 0.14378E+01_JPRB, 0.13843E+01_JPRB, 0.13441E+01_JPRB, 0.13430E+01_JPRB, 0.13190E+01_JPRB ,&
&  0.12224E+01_JPRB, 0.11536E+01_JPRB, 0.10955E+01_JPRB, 0.10358E+01_JPRB, 0.94860E+00_JPRB, 0.87701E+00_JPRB, 0.80325E+00_JPRB ,&
&  0.73223E+00_JPRB, 0.68046E+00_JPRB, 0.65445E+00_JPRB, 0.63876E+00_JPRB, 0.63455E+00_JPRB, 0.63219E+00_JPRB, 0.62244E+00_JPRB ,&
&  0.60721E+00_JPRB, 0.59090E+00_JPRB, 0.58442E+00_JPRB, 0.59002E+00_JPRB, 0.60862E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.66971E+00_JPRB, 0.66598E+00_JPRB, 0.63378E+00_JPRB, 0.64375E+00_JPRB, 0.61681E+00_JPRB, 0.59512E+00_JPRB, 0.56138E+00_JPRB ,&
&  0.54294E+00_JPRB, 0.51331E+00_JPRB, 0.49393E+00_JPRB, 0.46658E+00_JPRB, 0.45033E+00_JPRB, 0.44107E+00_JPRB, 0.45430E+00_JPRB ,&
&  0.47327E+00_JPRB, 0.48666E+00_JPRB, 0.49394E+00_JPRB, 0.49413E+00_JPRB, 0.49145E+00_JPRB, 0.48423E+00_JPRB, 0.47671E+00_JPRB ,&
&  0.47057E+00_JPRB, 0.47173E+00_JPRB, 0.46922E+00_JPRB, 0.46626E+00_JPRB, 0.46228E+00_JPRB, 0.45996E+00_JPRB, 0.46083E+00_JPRB ,&
&  0.45314E+00_JPRB, 0.44713E+00_JPRB, 0.45202E+00_JPRB, 0.47211E+00_JPRB, 0.48963E+00_JPRB, 0.50788E+00_JPRB, 0.47949E+00_JPRB ,&
&  0.44296E+00_JPRB, 0.43509E+00_JPRB, 0.44836E+00_JPRB, 0.54652E+00_JPRB, 0.56871E+00_JPRB, 0.57708E+00_JPRB, 0.60288E+00_JPRB ,&
&  0.58288E+00_JPRB, 0.57963E+00_JPRB, 0.59674E+00_JPRB, 0.66696E+00_JPRB, 0.68999E+00_JPRB, 0.70288E+00_JPRB, 0.72619E+00_JPRB ,&
&  0.75926E+00_JPRB, 0.80892E+00_JPRB, 0.86502E+00_JPRB, 0.77642E+00_JPRB, 0.81549E+00_JPRB, 0.96089E+00_JPRB, 0.87156E+00_JPRB ,&
&  0.84286E+00_JPRB, 0.82142E+00_JPRB, 0.84812E+00_JPRB, 0.86528E+00_JPRB, 0.85836E+00_JPRB, 0.88323E+00_JPRB, 0.90407E+00_JPRB ,&
&  0.88918E+00_JPRB, 0.93307E+00_JPRB, 0.10378E+01_JPRB, 0.11440E+01_JPRB, 0.12407E+01_JPRB, 0.13677E+01_JPRB, 0.14996E+01_JPRB ,&
&  0.16619E+01_JPRB, 0.18080E+01_JPRB, 0.19137E+01_JPRB, 0.18996E+01_JPRB, 0.18526E+01_JPRB, 0.17625E+01_JPRB, 0.16323E+01_JPRB ,&
&  0.14506E+01_JPRB, 0.13190E+01_JPRB, 0.12291E+01_JPRB, 0.11576E+01_JPRB, 0.10683E+01_JPRB, 0.95200E+00_JPRB, 0.80387E+00_JPRB ,&
&  0.69297E+00_JPRB, 0.62154E+00_JPRB, 0.59252E+00_JPRB, 0.58483E+00_JPRB, 0.58878E+00_JPRB, 0.59245E+00_JPRB, 0.58661E+00_JPRB ,&
&  0.57675E+00_JPRB, 0.57056E+00_JPRB, 0.56938E+00_JPRB, 0.58560E+00_JPRB, 0.63709E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.68808E+00_JPRB, 0.67207E+00_JPRB, 0.65407E+00_JPRB, 0.64862E+00_JPRB, 0.65044E+00_JPRB, 0.61703E+00_JPRB, 0.55496E+00_JPRB ,&
&  0.56815E+00_JPRB, 0.52029E+00_JPRB, 0.50733E+00_JPRB, 0.49401E+00_JPRB, 0.48021E+00_JPRB, 0.45212E+00_JPRB, 0.42878E+00_JPRB ,&
&  0.42945E+00_JPRB, 0.44431E+00_JPRB, 0.46116E+00_JPRB, 0.47510E+00_JPRB, 0.47951E+00_JPRB, 0.47579E+00_JPRB, 0.47025E+00_JPRB ,&
&  0.46612E+00_JPRB, 0.46693E+00_JPRB, 0.47056E+00_JPRB, 0.47562E+00_JPRB, 0.47605E+00_JPRB, 0.47585E+00_JPRB, 0.47905E+00_JPRB ,&
&  0.48060E+00_JPRB, 0.48002E+00_JPRB, 0.47267E+00_JPRB, 0.48195E+00_JPRB, 0.48919E+00_JPRB, 0.50576E+00_JPRB, 0.53074E+00_JPRB ,&
&  0.46761E+00_JPRB, 0.47416E+00_JPRB, 0.56456E+00_JPRB, 0.57055E+00_JPRB, 0.58354E+00_JPRB, 0.59717E+00_JPRB, 0.60763E+00_JPRB ,&
&  0.62296E+00_JPRB, 0.64769E+00_JPRB, 0.67828E+00_JPRB, 0.70574E+00_JPRB, 0.72916E+00_JPRB, 0.75832E+00_JPRB, 0.78178E+00_JPRB ,&
&  0.84741E+00_JPRB, 0.76886E+00_JPRB, 0.73100E+00_JPRB, 0.78185E+00_JPRB, 0.10602E+01_JPRB, 0.10708E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10533E+01_JPRB, 0.10064E+01_JPRB, 0.99456E+00_JPRB, 0.10209E+01_JPRB, 0.10364E+01_JPRB, 0.10888E+01_JPRB, 0.11647E+01_JPRB ,&
&  0.11670E+01_JPRB, 0.12474E+01_JPRB, 0.13534E+01_JPRB, 0.14494E+01_JPRB, 0.15553E+01_JPRB, 0.17131E+01_JPRB, 0.18900E+01_JPRB ,&
&  0.20790E+01_JPRB, 0.22428E+01_JPRB, 0.23185E+01_JPRB, 0.22872E+01_JPRB, 0.21832E+01_JPRB, 0.20455E+01_JPRB, 0.18737E+01_JPRB ,&
&  0.16511E+01_JPRB, 0.14912E+01_JPRB, 0.13822E+01_JPRB, 0.12757E+01_JPRB, 0.11788E+01_JPRB, 0.10512E+01_JPRB, 0.86916E+00_JPRB ,&
&  0.68155E+00_JPRB, 0.60127E+00_JPRB, 0.58591E+00_JPRB, 0.58362E+00_JPRB, 0.58824E+00_JPRB, 0.59789E+00_JPRB, 0.60680E+00_JPRB ,&
&  0.61103E+00_JPRB, 0.60110E+00_JPRB, 0.60088E+00_JPRB, 0.62669E+00_JPRB, 0.65832E+00_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.65664E+00_JPRB, 0.64786E+00_JPRB, 0.64605E+00_JPRB, 0.66281E+00_JPRB, 0.66028E+00_JPRB, 0.62559E+00_JPRB, 0.58353E+00_JPRB ,&
&  0.69066E+00_JPRB, 0.64936E+00_JPRB, 0.70702E+00_JPRB, 0.56710E+00_JPRB, 0.51255E+00_JPRB, 0.49442E+00_JPRB, 0.47135E+00_JPRB ,&
&  0.45322E+00_JPRB, 0.44477E+00_JPRB, 0.44301E+00_JPRB, 0.45380E+00_JPRB, 0.46001E+00_JPRB, 0.45815E+00_JPRB, 0.45556E+00_JPRB ,&
&  0.45417E+00_JPRB, 0.45783E+00_JPRB, 0.46776E+00_JPRB, 0.47378E+00_JPRB, 0.47853E+00_JPRB, 0.48337E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.47902E+00_JPRB, 0.47627E+00_JPRB, 0.48240E+00_JPRB, 0.48580E+00_JPRB, 0.49612E+00_JPRB, 0.50247E+00_JPRB, 0.51972E+00_JPRB ,&
&  0.56446E+00_JPRB, 0.53477E+00_JPRB, 0.60983E+00_JPRB, 0.61413E+00_JPRB, 0.62262E+00_JPRB, 0.63170E+00_JPRB, 0.63975E+00_JPRB ,&
&  0.64773E+00_JPRB, 0.66487E+00_JPRB, 0.69185E+00_JPRB, 0.71599E+00_JPRB, 0.75224E+00_JPRB, 0.79488E+00_JPRB, 0.82489E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.76116E+00_JPRB, 0.82828E+00_JPRB, 0.86717E+00_JPRB, 0.10746E+01_JPRB, 0.13131E+01_JPRB, 0.12657E+01_JPRB ,&
&  0.14464E+01_JPRB, 0.13447E+01_JPRB, 0.12672E+01_JPRB, 0.12047E+01_JPRB, 0.12340E+01_JPRB, 0.12989E+01_JPRB, 0.13825E+01_JPRB ,&
&  0.14607E+01_JPRB, 0.15471E+01_JPRB, 0.16908E+01_JPRB, 0.18191E+01_JPRB, 0.19526E+01_JPRB, 0.21064E+01_JPRB, 0.22529E+01_JPRB ,&
&  0.23843E+01_JPRB, 0.24673E+01_JPRB, 0.25144E+01_JPRB, 0.23765E+01_JPRB, 0.22528E+01_JPRB, 0.21199E+01_JPRB, 0.18613E+01_JPRB ,&
&  0.16619E+01_JPRB, 0.15083E+01_JPRB, 0.13809E+01_JPRB, 0.12867E+01_JPRB, 0.11210E+01_JPRB, 0.98316E+00_JPRB, 0.85298E+00_JPRB ,&
&  0.68607E+00_JPRB, 0.62572E+00_JPRB, 0.63579E+00_JPRB, 0.64280E+00_JPRB, 0.63796E+00_JPRB, 0.64185E+00_JPRB, 0.65002E+00_JPRB ,&
&  0.63883E+00_JPRB, 0.63827E+00_JPRB, 0.66315E+00_JPRB, 0.66609E+00_JPRB, 0.66246E+00_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.68233E+00_JPRB, 0.68940E+00_JPRB, 0.69464E+00_JPRB, 0.69594E+00_JPRB, 0.67791E+00_JPRB, 0.69461E+00_JPRB, 0.64130E+00_JPRB ,&
&  0.70118E+00_JPRB, 0.70600E+00_JPRB, 0.68921E+00_JPRB, 0.70114E+00_JPRB, 0.73857E+00_JPRB, 0.69050E+00_JPRB, 0.55341E+00_JPRB ,&
&  0.49237E+00_JPRB, 0.48453E+00_JPRB, 0.48448E+00_JPRB, 0.48339E+00_JPRB, 0.48385E+00_JPRB, 0.48145E+00_JPRB, 0.48036E+00_JPRB ,&
&  0.47993E+00_JPRB, 0.47892E+00_JPRB, 0.48460E+00_JPRB, 0.49056E+00_JPRB, 0.49277E+00_JPRB, 0.49559E+00_JPRB, 0.50553E+00_JPRB ,&
&  0.48867E+00_JPRB, 0.47768E+00_JPRB, 0.47203E+00_JPRB, 0.46162E+00_JPRB, 0.48145E+00_JPRB, 0.49768E+00_JPRB, 0.52507E+00_JPRB ,&
&  0.56494E+00_JPRB, 0.60538E+00_JPRB, 0.62573E+00_JPRB, 0.64228E+00_JPRB, 0.65612E+00_JPRB, 0.67068E+00_JPRB, 0.68598E+00_JPRB ,&
&  0.70035E+00_JPRB, 0.71838E+00_JPRB, 0.74487E+00_JPRB, 0.76506E+00_JPRB, 0.82487E+00_JPRB, 0.81262E+00_JPRB, 0.90704E+00_JPRB ,&
&  0.97149E+00_JPRB, 0.10998E+01_JPRB, 0.11833E+01_JPRB, 0.10572E+01_JPRB, 0.12196E+01_JPRB, 0.14090E+01_JPRB, 0.14032E+01_JPRB ,&
&  0.14414E+01_JPRB, 0.14248E+01_JPRB, 0.14632E+01_JPRB, 0.15245E+01_JPRB, 0.16924E+01_JPRB, 0.18172E+01_JPRB, 0.18779E+01_JPRB ,&
&  0.19379E+01_JPRB, 0.21640E+01_JPRB, 0.23163E+01_JPRB, 0.24168E+01_JPRB, 0.24639E+01_JPRB, 0.25037E+01_JPRB, 0.25667E+01_JPRB ,&
&  0.26125E+01_JPRB, 0.26532E+01_JPRB, 0.26410E+01_JPRB, 0.25528E+01_JPRB, 0.24190E+01_JPRB, 0.22347E+01_JPRB, 0.19862E+01_JPRB ,&
&  0.17636E+01_JPRB, 0.14872E+01_JPRB, 0.12624E+01_JPRB, 0.11229E+01_JPRB, 0.99210E+00_JPRB, 0.86329E+00_JPRB, 0.77390E+00_JPRB ,&
&  0.68639E+00_JPRB, 0.67768E+00_JPRB, 0.73494E+00_JPRB, 0.73149E+00_JPRB, 0.72957E+00_JPRB, 0.72323E+00_JPRB, 0.67116E+00_JPRB ,&
&  0.65719E+00_JPRB, 0.68605E+00_JPRB, 0.69429E+00_JPRB, 0.68964E+00_JPRB, 0.68339E+00_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.75692E+00_JPRB, 0.75381E+00_JPRB, 0.75206E+00_JPRB, 0.73974E+00_JPRB, 0.72907E+00_JPRB, 0.72747E+00_JPRB, 0.72732E+00_JPRB ,&
&  0.72511E+00_JPRB, 0.73029E+00_JPRB, 0.72759E+00_JPRB, 0.71588E+00_JPRB, 0.70970E+00_JPRB, 0.72469E+00_JPRB, 0.65764E+00_JPRB ,&
&  0.55913E+00_JPRB, 0.53013E+00_JPRB, 0.52000E+00_JPRB, 0.52479E+00_JPRB, 0.53683E+00_JPRB, 0.54192E+00_JPRB, 0.54377E+00_JPRB ,&
&  0.54766E+00_JPRB, 0.54163E+00_JPRB, 0.54542E+00_JPRB, 0.54643E+00_JPRB, 0.54264E+00_JPRB, 0.54267E+00_JPRB, 0.54289E+00_JPRB ,&
&  0.51242E+00_JPRB, 0.49069E+00_JPRB, 0.47891E+00_JPRB, 0.47195E+00_JPRB, 0.48610E+00_JPRB, 0.51262E+00_JPRB, 0.55251E+00_JPRB ,&
&  0.60011E+00_JPRB, 0.63524E+00_JPRB, 0.66581E+00_JPRB, 0.69592E+00_JPRB, 0.71930E+00_JPRB, 0.74442E+00_JPRB, 0.76749E+00_JPRB ,&
&  0.78578E+00_JPRB, 0.80161E+00_JPRB, 0.81166E+00_JPRB, 0.82885E+00_JPRB, 0.82934E+00_JPRB, 0.91186E+00_JPRB, 0.10942E+01_JPRB ,&
&  0.11677E+01_JPRB, 0.12761E+01_JPRB, 0.13380E+01_JPRB, 0.15005E+01_JPRB, 0.16118E+01_JPRB, 0.15829E+01_JPRB, 0.16139E+01_JPRB ,&
&  0.17486E+01_JPRB, 0.19016E+01_JPRB, 0.22363E+01_JPRB, 0.22604E+01_JPRB, 0.23218E+01_JPRB, 0.24084E+01_JPRB, 0.23984E+01_JPRB ,&
&  0.24747E+01_JPRB, 0.25757E+01_JPRB, 0.27226E+01_JPRB, 0.27368E+01_JPRB, 0.26986E+01_JPRB, 0.27224E+01_JPRB, 0.27182E+01_JPRB ,&
&  0.27465E+01_JPRB, 0.27767E+01_JPRB, 0.26554E+01_JPRB, 0.25419E+01_JPRB, 0.23188E+01_JPRB, 0.23114E+01_JPRB, 0.21406E+01_JPRB ,&
&  0.17931E+01_JPRB, 0.13349E+01_JPRB, 0.12047E+01_JPRB, 0.10592E+01_JPRB, 0.95914E+00_JPRB, 0.84402E+00_JPRB, 0.75421E+00_JPRB ,&
&  0.71671E+00_JPRB, 0.76413E+00_JPRB, 0.77014E+00_JPRB, 0.77947E+00_JPRB, 0.78972E+00_JPRB, 0.79170E+00_JPRB, 0.70956E+00_JPRB ,&
&  0.70968E+00_JPRB, 0.75840E+00_JPRB, 0.76092E+00_JPRB, 0.76130E+00_JPRB, 0.75847E+00_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.85979E+00_JPRB, 0.84365E+00_JPRB, 0.82949E+00_JPRB, 0.81721E+00_JPRB, 0.80161E+00_JPRB, 0.78521E+00_JPRB, 0.76881E+00_JPRB ,&
&  0.76459E+00_JPRB, 0.77250E+00_JPRB, 0.77504E+00_JPRB, 0.75477E+00_JPRB, 0.73480E+00_JPRB, 0.72546E+00_JPRB, 0.75420E+00_JPRB ,&
&  0.62851E+00_JPRB, 0.60458E+00_JPRB, 0.60185E+00_JPRB, 0.62502E+00_JPRB, 0.68941E+00_JPRB, 0.70241E+00_JPRB, 0.72289E+00_JPRB ,&
&  0.71334E+00_JPRB, 0.69924E+00_JPRB, 0.69117E+00_JPRB, 0.67936E+00_JPRB, 0.65833E+00_JPRB, 0.64464E+00_JPRB, 0.62461E+00_JPRB ,&
&  0.57290E+00_JPRB, 0.53276E+00_JPRB, 0.51836E+00_JPRB, 0.51266E+00_JPRB, 0.53649E+00_JPRB, 0.56452E+00_JPRB, 0.61760E+00_JPRB ,&
&  0.66654E+00_JPRB, 0.71172E+00_JPRB, 0.75447E+00_JPRB, 0.80572E+00_JPRB, 0.84685E+00_JPRB, 0.88629E+00_JPRB, 0.91113E+00_JPRB ,&
&  0.92218E+00_JPRB, 0.92260E+00_JPRB, 0.93469E+00_JPRB, 0.91675E+00_JPRB, 0.91588E+00_JPRB, 0.11286E+01_JPRB, 0.14115E+01_JPRB ,&
&  0.15205E+01_JPRB, 0.16306E+01_JPRB, 0.20563E+01_JPRB, 0.26617E+01_JPRB, 0.25324E+01_JPRB, 0.23111E+01_JPRB, 0.22445E+01_JPRB ,&
&  0.23953E+01_JPRB, 0.26111E+01_JPRB, 0.28432E+01_JPRB, 0.29274E+01_JPRB, 0.29191E+01_JPRB, 0.30047E+01_JPRB, 0.30308E+01_JPRB ,&
&  0.30209E+01_JPRB, 0.30602E+01_JPRB, 0.30207E+01_JPRB, 0.29319E+01_JPRB, 0.28515E+01_JPRB, 0.29061E+01_JPRB, 0.29008E+01_JPRB ,&
&  0.28642E+01_JPRB, 0.27532E+01_JPRB, 0.25482E+01_JPRB, 0.24344E+01_JPRB, 0.20456E+01_JPRB, 0.19801E+01_JPRB, 0.22074E+01_JPRB ,&
&  0.18199E+01_JPRB, 0.14310E+01_JPRB, 0.12192E+01_JPRB, 0.10832E+01_JPRB, 0.10190E+01_JPRB, 0.98005E+00_JPRB, 0.90106E+00_JPRB ,&
&  0.80168E+00_JPRB, 0.82533E+00_JPRB, 0.89675E+00_JPRB, 0.93722E+00_JPRB, 0.94880E+00_JPRB, 0.95579E+00_JPRB, 0.87920E+00_JPRB ,&
&  0.90858E+00_JPRB, 0.88336E+00_JPRB, 0.87745E+00_JPRB, 0.87798E+00_JPRB, 0.87126E+00_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.96456E+00_JPRB, 0.92854E+00_JPRB, 0.90624E+00_JPRB, 0.87263E+00_JPRB, 0.83986E+00_JPRB, 0.81608E+00_JPRB, 0.80200E+00_JPRB ,&
&  0.80714E+00_JPRB, 0.80860E+00_JPRB, 0.80231E+00_JPRB, 0.77827E+00_JPRB, 0.76073E+00_JPRB, 0.74853E+00_JPRB, 0.76458E+00_JPRB ,&
&  0.76364E+00_JPRB, 0.69526E+00_JPRB, 0.68318E+00_JPRB, 0.68851E+00_JPRB, 0.80434E+00_JPRB, 0.90698E+00_JPRB, 0.99234E+00_JPRB ,&
&  0.10179E+01_JPRB, 0.10300E+01_JPRB, 0.10134E+01_JPRB, 0.95646E+00_JPRB, 0.89604E+00_JPRB, 0.84159E+00_JPRB, 0.78137E+00_JPRB ,&
&  0.71287E+00_JPRB, 0.64886E+00_JPRB, 0.61833E+00_JPRB, 0.62229E+00_JPRB, 0.67170E+00_JPRB, 0.65248E+00_JPRB, 0.71003E+00_JPRB ,&
&  0.76690E+00_JPRB, 0.83073E+00_JPRB, 0.88510E+00_JPRB, 0.95526E+00_JPRB, 0.10036E+01_JPRB, 0.10398E+01_JPRB, 0.10467E+01_JPRB ,&
&  0.10635E+01_JPRB, 0.10364E+01_JPRB, 0.93035E+00_JPRB, 0.96759E+00_JPRB, 0.10748E+01_JPRB, 0.15065E+01_JPRB, 0.19685E+01_JPRB ,&
&  0.22793E+01_JPRB, 0.27545E+01_JPRB, 0.33421E+01_JPRB, 0.37291E+01_JPRB, 0.33712E+01_JPRB, 0.31018E+01_JPRB, 0.31287E+01_JPRB ,&
&  0.33207E+01_JPRB, 0.35368E+01_JPRB, 0.36205E+01_JPRB, 0.34707E+01_JPRB, 0.35165E+01_JPRB, 0.36068E+01_JPRB, 0.36829E+01_JPRB ,&
&  0.36642E+01_JPRB, 0.35314E+01_JPRB, 0.33468E+01_JPRB, 0.31712E+01_JPRB, 0.30567E+01_JPRB, 0.28362E+01_JPRB, 0.27755E+01_JPRB ,&
&  0.28266E+01_JPRB, 0.24067E+01_JPRB, 0.21683E+01_JPRB, 0.22524E+01_JPRB, 0.18561E+01_JPRB, 0.18134E+01_JPRB, 0.19471E+01_JPRB ,&
&  0.16579E+01_JPRB, 0.13837E+01_JPRB, 0.12384E+01_JPRB, 0.11012E+01_JPRB, 0.10624E+01_JPRB, 0.11460E+01_JPRB, 0.11801E+01_JPRB ,&
&  0.11142E+01_JPRB, 0.10737E+01_JPRB, 0.12337E+01_JPRB, 0.13236E+01_JPRB, 0.13712E+01_JPRB, 0.13236E+01_JPRB, 0.12471E+01_JPRB ,&
&  0.11781E+01_JPRB, 0.11280E+01_JPRB, 0.10915E+01_JPRB, 0.10533E+01_JPRB, 0.10131E+01_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.10537E+01_JPRB, 0.10372E+01_JPRB, 0.99696E+00_JPRB, 0.93901E+00_JPRB, 0.89513E+00_JPRB, 0.87023E+00_JPRB, 0.84194E+00_JPRB ,&
&  0.81747E+00_JPRB, 0.79748E+00_JPRB, 0.78961E+00_JPRB, 0.77213E+00_JPRB, 0.76666E+00_JPRB, 0.76967E+00_JPRB, 0.77204E+00_JPRB ,&
&  0.76621E+00_JPRB, 0.76316E+00_JPRB, 0.72724E+00_JPRB, 0.73954E+00_JPRB, 0.85085E+00_JPRB, 0.11683E+01_JPRB, 0.13941E+01_JPRB ,&
&  0.14768E+01_JPRB, 0.14839E+01_JPRB, 0.14398E+01_JPRB, 0.13972E+01_JPRB, 0.13104E+01_JPRB, 0.11754E+01_JPRB, 0.10883E+01_JPRB ,&
&  0.10008E+01_JPRB, 0.90737E+00_JPRB, 0.81295E+00_JPRB, 0.82476E+00_JPRB, 0.84825E+00_JPRB, 0.81048E+00_JPRB, 0.89738E+00_JPRB ,&
&  0.96533E+00_JPRB, 0.10129E+01_JPRB, 0.10614E+01_JPRB, 0.11065E+01_JPRB, 0.11275E+01_JPRB, 0.10636E+01_JPRB, 0.98052E+00_JPRB ,&
&  0.94855E+00_JPRB, 0.98170E+00_JPRB, 0.10152E+01_JPRB, 0.11608E+01_JPRB, 0.15771E+01_JPRB, 0.23150E+01_JPRB, 0.29583E+01_JPRB ,&
&  0.33614E+01_JPRB, 0.35386E+01_JPRB, 0.33141E+01_JPRB, 0.36339E+01_JPRB, 0.41603E+01_JPRB, 0.40384E+01_JPRB, 0.38417E+01_JPRB ,&
&  0.38552E+01_JPRB, 0.38492E+01_JPRB, 0.37869E+01_JPRB, 0.38607E+01_JPRB, 0.41928E+01_JPRB, 0.43782E+01_JPRB, 0.43906E+01_JPRB ,&
&  0.38378E+01_JPRB, 0.36262E+01_JPRB, 0.35160E+01_JPRB, 0.33602E+01_JPRB, 0.29903E+01_JPRB, 0.28440E+01_JPRB, 0.29023E+01_JPRB ,&
&  0.28225E+01_JPRB, 0.27643E+01_JPRB, 0.21892E+01_JPRB, 0.20510E+01_JPRB, 0.15872E+01_JPRB, 0.15320E+01_JPRB, 0.17158E+01_JPRB ,&
&  0.15695E+01_JPRB, 0.14301E+01_JPRB, 0.12438E+01_JPRB, 0.11622E+01_JPRB, 0.13359E+01_JPRB, 0.15266E+01_JPRB, 0.18050E+01_JPRB ,&
&  0.17898E+01_JPRB, 0.15590E+01_JPRB, 0.17290E+01_JPRB, 0.17887E+01_JPRB, 0.18660E+01_JPRB, 0.17674E+01_JPRB, 0.15885E+01_JPRB ,&
&  0.14466E+01_JPRB, 0.13408E+01_JPRB, 0.12371E+01_JPRB, 0.11420E+01_JPRB, 0.10843E+01_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.10769E+01_JPRB, 0.10495E+01_JPRB, 0.99830E+00_JPRB, 0.93824E+00_JPRB, 0.89747E+00_JPRB, 0.86615E+00_JPRB, 0.84138E+00_JPRB ,&
&  0.81806E+00_JPRB, 0.79547E+00_JPRB, 0.78246E+00_JPRB, 0.77129E+00_JPRB, 0.76292E+00_JPRB, 0.75675E+00_JPRB, 0.75247E+00_JPRB ,&
&  0.75390E+00_JPRB, 0.78374E+00_JPRB, 0.76277E+00_JPRB, 0.78493E+00_JPRB, 0.96063E+00_JPRB, 0.13435E+01_JPRB, 0.17065E+01_JPRB ,&
&  0.18655E+01_JPRB, 0.18358E+01_JPRB, 0.17938E+01_JPRB, 0.17447E+01_JPRB, 0.17082E+01_JPRB, 0.16273E+01_JPRB, 0.15152E+01_JPRB ,&
&  0.13909E+01_JPRB, 0.11343E+01_JPRB, 0.10862E+01_JPRB, 0.10335E+01_JPRB, 0.10808E+01_JPRB, 0.11100E+01_JPRB, 0.10966E+01_JPRB ,&
&  0.10766E+01_JPRB, 0.10810E+01_JPRB, 0.10890E+01_JPRB, 0.10539E+01_JPRB, 0.10284E+01_JPRB, 0.97191E+00_JPRB, 0.97325E+00_JPRB ,&
&  0.10572E+01_JPRB, 0.11220E+01_JPRB, 0.13899E+01_JPRB, 0.18690E+01_JPRB, 0.27186E+01_JPRB, 0.38635E+01_JPRB, 0.40162E+01_JPRB ,&
&  0.32984E+01_JPRB, 0.31497E+01_JPRB, 0.43387E+01_JPRB, 0.41633E+01_JPRB, 0.43104E+01_JPRB, 0.48160E+01_JPRB, 0.45267E+01_JPRB ,&
&  0.41004E+01_JPRB, 0.38140E+01_JPRB, 0.37357E+01_JPRB, 0.40113E+01_JPRB, 0.53592E+01_JPRB, 0.56111E+01_JPRB, 0.52437E+01_JPRB ,&
&  0.43665E+01_JPRB, 0.41301E+01_JPRB, 0.37769E+01_JPRB, 0.33846E+01_JPRB, 0.31520E+01_JPRB, 0.32964E+01_JPRB, 0.31127E+01_JPRB ,&
&  0.28718E+01_JPRB, 0.30605E+01_JPRB, 0.24245E+01_JPRB, 0.18530E+01_JPRB, 0.15410E+01_JPRB, 0.15698E+01_JPRB, 0.16671E+01_JPRB ,&
&  0.16341E+01_JPRB, 0.15314E+01_JPRB, 0.14337E+01_JPRB, 0.16257E+01_JPRB, 0.21296E+01_JPRB, 0.24965E+01_JPRB, 0.24740E+01_JPRB ,&
&  0.24337E+01_JPRB, 0.26640E+01_JPRB, 0.25156E+01_JPRB, 0.24943E+01_JPRB, 0.25093E+01_JPRB, 0.22296E+01_JPRB, 0.18556E+01_JPRB ,&
&  0.15405E+01_JPRB, 0.13430E+01_JPRB, 0.12266E+01_JPRB, 0.11529E+01_JPRB, 0.11043E+01_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.11227E+01_JPRB, 0.10676E+01_JPRB, 0.10138E+01_JPRB, 0.96205E+00_JPRB, 0.91122E+00_JPRB, 0.87208E+00_JPRB, 0.83568E+00_JPRB ,&
&  0.79840E+00_JPRB, 0.77335E+00_JPRB, 0.77441E+00_JPRB, 0.78547E+00_JPRB, 0.78618E+00_JPRB, 0.75992E+00_JPRB, 0.75592E+00_JPRB ,&
&  0.78862E+00_JPRB, 0.81201E+00_JPRB, 0.80038E+00_JPRB, 0.87764E+00_JPRB, 0.10530E+01_JPRB, 0.13141E+01_JPRB, 0.16911E+01_JPRB ,&
&  0.19490E+01_JPRB, 0.18889E+01_JPRB, 0.18035E+01_JPRB, 0.18995E+01_JPRB, 0.20274E+01_JPRB, 0.20539E+01_JPRB, 0.19597E+01_JPRB ,&
&  0.16943E+01_JPRB, 0.15307E+01_JPRB, 0.14942E+01_JPRB, 0.13608E+01_JPRB, 0.13020E+01_JPRB, 0.12245E+01_JPRB, 0.11262E+01_JPRB ,&
&  0.10503E+01_JPRB, 0.10257E+01_JPRB, 0.10126E+01_JPRB, 0.10035E+01_JPRB, 0.10425E+01_JPRB, 0.10952E+01_JPRB, 0.11500E+01_JPRB ,&
&  0.12660E+01_JPRB, 0.15130E+01_JPRB, 0.19602E+01_JPRB, 0.26748E+01_JPRB, 0.32596E+01_JPRB, 0.48145E+01_JPRB, 0.44975E+01_JPRB ,&
&  0.41273E+01_JPRB, 0.40714E+01_JPRB, 0.40511E+01_JPRB, 0.44520E+01_JPRB, 0.38953E+01_JPRB, 0.42504E+01_JPRB, 0.44279E+01_JPRB ,&
&  0.42429E+01_JPRB, 0.41435E+01_JPRB, 0.44023E+01_JPRB, 0.45923E+01_JPRB, 0.56559E+01_JPRB, 0.72148E+01_JPRB, 0.65927E+01_JPRB ,&
&  0.53723E+01_JPRB, 0.47235E+01_JPRB, 0.40416E+01_JPRB, 0.36408E+01_JPRB, 0.30625E+01_JPRB, 0.30237E+01_JPRB, 0.26305E+01_JPRB ,&
&  0.23840E+01_JPRB, 0.19195E+01_JPRB, 0.20579E+01_JPRB, 0.17473E+01_JPRB, 0.16363E+01_JPRB, 0.16001E+01_JPRB, 0.15989E+01_JPRB ,&
&  0.16589E+01_JPRB, 0.17088E+01_JPRB, 0.19635E+01_JPRB, 0.25350E+01_JPRB, 0.33063E+01_JPRB, 0.29794E+01_JPRB, 0.32195E+01_JPRB ,&
&  0.34197E+01_JPRB, 0.33745E+01_JPRB, 0.28809E+01_JPRB, 0.27775E+01_JPRB, 0.25335E+01_JPRB, 0.22939E+01_JPRB, 0.20158E+01_JPRB ,&
&  0.17212E+01_JPRB, 0.14695E+01_JPRB, 0.13284E+01_JPRB, 0.12420E+01_JPRB, 0.11774E+01_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.10857E+01_JPRB, 0.10267E+01_JPRB, 0.98315E+00_JPRB, 0.94388E+00_JPRB, 0.90587E+00_JPRB, 0.85565E+00_JPRB, 0.79908E+00_JPRB ,&
&  0.75684E+00_JPRB, 0.74086E+00_JPRB, 0.74483E+00_JPRB, 0.75899E+00_JPRB, 0.76307E+00_JPRB, 0.77852E+00_JPRB, 0.79248E+00_JPRB ,&
&  0.84222E+00_JPRB, 0.88294E+00_JPRB, 0.87242E+00_JPRB, 0.91535E+00_JPRB, 0.10755E+01_JPRB, 0.12764E+01_JPRB, 0.14787E+01_JPRB ,&
&  0.19109E+01_JPRB, 0.17000E+01_JPRB, 0.15909E+01_JPRB, 0.17887E+01_JPRB, 0.22327E+01_JPRB, 0.24777E+01_JPRB, 0.23484E+01_JPRB ,&
&  0.22915E+01_JPRB, 0.20708E+01_JPRB, 0.16984E+01_JPRB, 0.14182E+01_JPRB, 0.12135E+01_JPRB, 0.10701E+01_JPRB, 0.95096E+00_JPRB ,&
&  0.89749E+00_JPRB, 0.88994E+00_JPRB, 0.90360E+00_JPRB, 0.92261E+00_JPRB, 0.99058E+00_JPRB, 0.10179E+01_JPRB, 0.12170E+01_JPRB ,&
&  0.14399E+01_JPRB, 0.17298E+01_JPRB, 0.21369E+01_JPRB, 0.25771E+01_JPRB, 0.32734E+01_JPRB, 0.52698E+01_JPRB, 0.59921E+01_JPRB ,&
&  0.45477E+01_JPRB, 0.42072E+01_JPRB, 0.45012E+01_JPRB, 0.47019E+01_JPRB, 0.46572E+01_JPRB, 0.42695E+01_JPRB, 0.41741E+01_JPRB ,&
&  0.33394E+01_JPRB, 0.30173E+01_JPRB, 0.32190E+01_JPRB, 0.41640E+01_JPRB, 0.68400E+01_JPRB, 0.90906E+01_JPRB, 0.83591E+01_JPRB ,&
&  0.68856E+01_JPRB, 0.57542E+01_JPRB, 0.48059E+01_JPRB, 0.36107E+01_JPRB, 0.31254E+01_JPRB, 0.26768E+01_JPRB, 0.30767E+01_JPRB ,&
&  0.28595E+01_JPRB, 0.25516E+01_JPRB, 0.20848E+01_JPRB, 0.15978E+01_JPRB, 0.12760E+01_JPRB, 0.14805E+01_JPRB, 0.15939E+01_JPRB ,&
&  0.17806E+01_JPRB, 0.19994E+01_JPRB, 0.31145E+01_JPRB, 0.35503E+01_JPRB, 0.37395E+01_JPRB, 0.33979E+01_JPRB, 0.41424E+01_JPRB ,&
&  0.38640E+01_JPRB, 0.37707E+01_JPRB, 0.33042E+01_JPRB, 0.26227E+01_JPRB, 0.21342E+01_JPRB, 0.18149E+01_JPRB, 0.16406E+01_JPRB ,&
&  0.15635E+01_JPRB, 0.14924E+01_JPRB, 0.13849E+01_JPRB, 0.12749E+01_JPRB, 0.11822E+01_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.94982E+00_JPRB, 0.89814E+00_JPRB, 0.86725E+00_JPRB, 0.84097E+00_JPRB, 0.80937E+00_JPRB, 0.77189E+00_JPRB, 0.73230E+00_JPRB ,&
&  0.71273E+00_JPRB, 0.70939E+00_JPRB, 0.71896E+00_JPRB, 0.74786E+00_JPRB, 0.77418E+00_JPRB, 0.80980E+00_JPRB, 0.83605E+00_JPRB ,&
&  0.89363E+00_JPRB, 0.98806E+00_JPRB, 0.98159E+00_JPRB, 0.10221E+01_JPRB, 0.11521E+01_JPRB, 0.12536E+01_JPRB, 0.14432E+01_JPRB ,&
&  0.18214E+01_JPRB, 0.15686E+01_JPRB, 0.14979E+01_JPRB, 0.17283E+01_JPRB, 0.23937E+01_JPRB, 0.24777E+01_JPRB, 0.22801E+01_JPRB ,&
&  0.23459E+01_JPRB, 0.20183E+01_JPRB, 0.15095E+01_JPRB, 0.12316E+01_JPRB, 0.10595E+01_JPRB, 0.92506E+00_JPRB, 0.85921E+00_JPRB ,&
&  0.86011E+00_JPRB, 0.89594E+00_JPRB, 0.94149E+00_JPRB, 0.97978E+00_JPRB, 0.10098E+01_JPRB, 0.11235E+01_JPRB, 0.13619E+01_JPRB ,&
&  0.15918E+01_JPRB, 0.18414E+01_JPRB, 0.20621E+01_JPRB, 0.23707E+01_JPRB, 0.31829E+01_JPRB, 0.52886E+01_JPRB, 0.75606E+01_JPRB ,&
&  0.61527E+01_JPRB, 0.57330E+01_JPRB, 0.55578E+01_JPRB, 0.56613E+01_JPRB, 0.46954E+01_JPRB, 0.42315E+01_JPRB, 0.40077E+01_JPRB ,&
&  0.32218E+01_JPRB, 0.32000E+01_JPRB, 0.41837E+01_JPRB, 0.59230E+01_JPRB, 0.73647E+01_JPRB, 0.89168E+01_JPRB, 0.10364E+02_JPRB ,&
&  0.76767E+01_JPRB, 0.62059E+01_JPRB, 0.52876E+01_JPRB, 0.44894E+01_JPRB, 0.25817E+01_JPRB, 0.18140E+01_JPRB, 0.24804E+01_JPRB ,&
&  0.22545E+01_JPRB, 0.12786E+01_JPRB, 0.99222E+00_JPRB, 0.12320E+01_JPRB, 0.89642E+00_JPRB, 0.10349E+01_JPRB, 0.15906E+01_JPRB ,&
&  0.19830E+01_JPRB, 0.24333E+01_JPRB, 0.42009E+01_JPRB, 0.39119E+01_JPRB, 0.40502E+01_JPRB, 0.51807E+01_JPRB, 0.52131E+01_JPRB ,&
&  0.47838E+01_JPRB, 0.31614E+01_JPRB, 0.23789E+01_JPRB, 0.18113E+01_JPRB, 0.14869E+01_JPRB, 0.13938E+01_JPRB, 0.13418E+01_JPRB ,&
&  0.12987E+01_JPRB, 0.12320E+01_JPRB, 0.11680E+01_JPRB, 0.10794E+01_JPRB, 0.10170E+01_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.79113E+00_JPRB, 0.75516E+00_JPRB, 0.74756E+00_JPRB, 0.73688E+00_JPRB, 0.70464E+00_JPRB, 0.68514E+00_JPRB, 0.67331E+00_JPRB ,&
&  0.67891E+00_JPRB, 0.69484E+00_JPRB, 0.72071E+00_JPRB, 0.76426E+00_JPRB, 0.81764E+00_JPRB, 0.87056E+00_JPRB, 0.89676E+00_JPRB ,&
&  0.93960E+00_JPRB, 0.10397E+01_JPRB, 0.11390E+01_JPRB, 0.12457E+01_JPRB, 0.14010E+01_JPRB, 0.13131E+01_JPRB, 0.16973E+01_JPRB ,&
&  0.17710E+01_JPRB, 0.15100E+01_JPRB, 0.15350E+01_JPRB, 0.18125E+01_JPRB, 0.20914E+01_JPRB, 0.19938E+01_JPRB, 0.18700E+01_JPRB ,&
&  0.17776E+01_JPRB, 0.15060E+01_JPRB, 0.12332E+01_JPRB, 0.10587E+01_JPRB, 0.94163E+00_JPRB, 0.88479E+00_JPRB, 0.90728E+00_JPRB ,&
&  0.94822E+00_JPRB, 0.10413E+01_JPRB, 0.11132E+01_JPRB, 0.10648E+01_JPRB, 0.11511E+01_JPRB, 0.12824E+01_JPRB, 0.14849E+01_JPRB ,&
&  0.16703E+01_JPRB, 0.18531E+01_JPRB, 0.20566E+01_JPRB, 0.25448E+01_JPRB, 0.40941E+01_JPRB, 0.63195E+01_JPRB, 0.68915E+01_JPRB ,&
&  0.61829E+01_JPRB, 0.56971E+01_JPRB, 0.63095E+01_JPRB, 0.58057E+01_JPRB, 0.49072E+01_JPRB, 0.47894E+01_JPRB, 0.43997E+01_JPRB ,&
&  0.44183E+01_JPRB, 0.48870E+01_JPRB, 0.55752E+01_JPRB, 0.70025E+01_JPRB, 0.90457E+01_JPRB, 0.94569E+01_JPRB, 0.96175E+01_JPRB ,&
&  0.83961E+01_JPRB, 0.63776E+01_JPRB, 0.43066E+01_JPRB, 0.29744E+01_JPRB, 0.27714E+01_JPRB, 0.22227E+01_JPRB, 0.14458E+01_JPRB ,&
&  0.11604E+01_JPRB, 0.96933E+00_JPRB, 0.70611E+00_JPRB, 0.77318E+00_JPRB, 0.69984E+00_JPRB, 0.84749E+00_JPRB, 0.14559E+01_JPRB ,&
&  0.25220E+01_JPRB, 0.32863E+01_JPRB, 0.41659E+01_JPRB, 0.39046E+01_JPRB, 0.39749E+01_JPRB, 0.53042E+01_JPRB, 0.60963E+01_JPRB ,&
&  0.30290E+01_JPRB, 0.20393E+01_JPRB, 0.15729E+01_JPRB, 0.13322E+01_JPRB, 0.12418E+01_JPRB, 0.11851E+01_JPRB, 0.11372E+01_JPRB ,&
&  0.10696E+01_JPRB, 0.10098E+01_JPRB, 0.98040E+00_JPRB, 0.93144E+00_JPRB, 0.85599E+00_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.71773E+00_JPRB, 0.71547E+00_JPRB, 0.71746E+00_JPRB, 0.70358E+00_JPRB, 0.66332E+00_JPRB, 0.66292E+00_JPRB, 0.67808E+00_JPRB ,&
&  0.70196E+00_JPRB, 0.73414E+00_JPRB, 0.78114E+00_JPRB, 0.84503E+00_JPRB, 0.93050E+00_JPRB, 0.98039E+00_JPRB, 0.99730E+00_JPRB ,&
&  0.10332E+01_JPRB, 0.11617E+01_JPRB, 0.13412E+01_JPRB, 0.15555E+01_JPRB, 0.17439E+01_JPRB, 0.15052E+01_JPRB, 0.19649E+01_JPRB ,&
&  0.17172E+01_JPRB, 0.15327E+01_JPRB, 0.15674E+01_JPRB, 0.17229E+01_JPRB, 0.17631E+01_JPRB, 0.16483E+01_JPRB, 0.15352E+01_JPRB ,&
&  0.13690E+01_JPRB, 0.12737E+01_JPRB, 0.10819E+01_JPRB, 0.98207E+00_JPRB, 0.91747E+00_JPRB, 0.94650E+00_JPRB, 0.10104E+01_JPRB ,&
&  0.10848E+01_JPRB, 0.12006E+01_JPRB, 0.11868E+01_JPRB, 0.12636E+01_JPRB, 0.14500E+01_JPRB, 0.16033E+01_JPRB, 0.17485E+01_JPRB ,&
&  0.18463E+01_JPRB, 0.20511E+01_JPRB, 0.24788E+01_JPRB, 0.36840E+01_JPRB, 0.49117E+01_JPRB, 0.53460E+01_JPRB, 0.61804E+01_JPRB ,&
&  0.66811E+01_JPRB, 0.67567E+01_JPRB, 0.59668E+01_JPRB, 0.64106E+01_JPRB, 0.55933E+01_JPRB, 0.49747E+01_JPRB, 0.49379E+01_JPRB ,&
&  0.53362E+01_JPRB, 0.67443E+01_JPRB, 0.73559E+01_JPRB, 0.79858E+01_JPRB, 0.11353E+02_JPRB, 0.12391E+02_JPRB, 0.98823E+01_JPRB ,&
&  0.86744E+01_JPRB, 0.61792E+01_JPRB, 0.42812E+01_JPRB, 0.30804E+01_JPRB, 0.37431E+01_JPRB, 0.39493E+01_JPRB, 0.19567E+01_JPRB ,&
&  0.12750E+01_JPRB, 0.95875E+00_JPRB, 0.68669E+00_JPRB, 0.66935E+00_JPRB, 0.67136E+00_JPRB, 0.85128E+00_JPRB, 0.18656E+01_JPRB ,&
&  0.23061E+01_JPRB, 0.36877E+01_JPRB, 0.42109E+01_JPRB, 0.41174E+01_JPRB, 0.40888E+01_JPRB, 0.38325E+01_JPRB, 0.32288E+01_JPRB ,&
&  0.21155E+01_JPRB, 0.13799E+01_JPRB, 0.11368E+01_JPRB, 0.11617E+01_JPRB, 0.11712E+01_JPRB, 0.11063E+01_JPRB, 0.10171E+01_JPRB ,&
&  0.95929E+00_JPRB, 0.94496E+00_JPRB, 0.90420E+00_JPRB, 0.85909E+00_JPRB, 0.78668E+00_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.72659E+00_JPRB, 0.74775E+00_JPRB, 0.76419E+00_JPRB, 0.72195E+00_JPRB, 0.70780E+00_JPRB, 0.71940E+00_JPRB, 0.71862E+00_JPRB ,&
&  0.76142E+00_JPRB, 0.81903E+00_JPRB, 0.88081E+00_JPRB, 0.98344E+00_JPRB, 0.10813E+01_JPRB, 0.11065E+01_JPRB, 0.11238E+01_JPRB ,&
&  0.11741E+01_JPRB, 0.13418E+01_JPRB, 0.16061E+01_JPRB, 0.19299E+01_JPRB, 0.19775E+01_JPRB, 0.18288E+01_JPRB, 0.20697E+01_JPRB ,&
&  0.17444E+01_JPRB, 0.16237E+01_JPRB, 0.15564E+01_JPRB, 0.16734E+01_JPRB, 0.16839E+01_JPRB, 0.16315E+01_JPRB, 0.14986E+01_JPRB ,&
&  0.13055E+01_JPRB, 0.12983E+01_JPRB, 0.10749E+01_JPRB, 0.11069E+01_JPRB, 0.10823E+01_JPRB, 0.11529E+01_JPRB, 0.12088E+01_JPRB ,&
&  0.13066E+01_JPRB, 0.13384E+01_JPRB, 0.13910E+01_JPRB, 0.15528E+01_JPRB, 0.17723E+01_JPRB, 0.19642E+01_JPRB, 0.21459E+01_JPRB ,&
&  0.22917E+01_JPRB, 0.26882E+01_JPRB, 0.34692E+01_JPRB, 0.46693E+01_JPRB, 0.45323E+01_JPRB, 0.59421E+01_JPRB, 0.61848E+01_JPRB ,&
&  0.64977E+01_JPRB, 0.64182E+01_JPRB, 0.56793E+01_JPRB, 0.58418E+01_JPRB, 0.58322E+01_JPRB, 0.55209E+01_JPRB, 0.56280E+01_JPRB ,&
&  0.71867E+01_JPRB, 0.80675E+01_JPRB, 0.90848E+01_JPRB, 0.97680E+01_JPRB, 0.12815E+02_JPRB, 0.17465E+02_JPRB, 0.10369E+02_JPRB ,&
&  0.70955E+01_JPRB, 0.62284E+01_JPRB, 0.43518E+01_JPRB, 0.33481E+01_JPRB, 0.41188E+01_JPRB, 0.48311E+01_JPRB, 0.42560E+01_JPRB ,&
&  0.17474E+01_JPRB, 0.97200E+00_JPRB, 0.83951E+00_JPRB, 0.84634E+00_JPRB, 0.71635E+00_JPRB, 0.92458E+00_JPRB, 0.29877E+01_JPRB ,&
&  0.24080E+01_JPRB, 0.37106E+01_JPRB, 0.39752E+01_JPRB, 0.38726E+01_JPRB, 0.40997E+01_JPRB, 0.30542E+01_JPRB, 0.21198E+01_JPRB ,&
&  0.10398E+01_JPRB, 0.96919E+00_JPRB, 0.10459E+01_JPRB, 0.10199E+01_JPRB, 0.10098E+01_JPRB, 0.95190E+00_JPRB, 0.86100E+00_JPRB ,&
&  0.85164E+00_JPRB, 0.85002E+00_JPRB, 0.79895E+00_JPRB, 0.75586E+00_JPRB, 0.73061E+00_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.76716E+00_JPRB, 0.79189E+00_JPRB, 0.79283E+00_JPRB, 0.78944E+00_JPRB, 0.79709E+00_JPRB, 0.80926E+00_JPRB, 0.80803E+00_JPRB ,&
&  0.85800E+00_JPRB, 0.94195E+00_JPRB, 0.10537E+01_JPRB, 0.11901E+01_JPRB, 0.12765E+01_JPRB, 0.12825E+01_JPRB, 0.13127E+01_JPRB ,&
&  0.14218E+01_JPRB, 0.16437E+01_JPRB, 0.19221E+01_JPRB, 0.21169E+01_JPRB, 0.20541E+01_JPRB, 0.18626E+01_JPRB, 0.19602E+01_JPRB ,&
&  0.18563E+01_JPRB, 0.17569E+01_JPRB, 0.15322E+01_JPRB, 0.16488E+01_JPRB, 0.17939E+01_JPRB, 0.18591E+01_JPRB, 0.17095E+01_JPRB ,&
&  0.15761E+01_JPRB, 0.15536E+01_JPRB, 0.14281E+01_JPRB, 0.14365E+01_JPRB, 0.14718E+01_JPRB, 0.15534E+01_JPRB, 0.16173E+01_JPRB ,&
&  0.15829E+01_JPRB, 0.16311E+01_JPRB, 0.17679E+01_JPRB, 0.19822E+01_JPRB, 0.22342E+01_JPRB, 0.24694E+01_JPRB, 0.27381E+01_JPRB ,&
&  0.30957E+01_JPRB, 0.36961E+01_JPRB, 0.44626E+01_JPRB, 0.47394E+01_JPRB, 0.50997E+01_JPRB, 0.55845E+01_JPRB, 0.59809E+01_JPRB ,&
&  0.62331E+01_JPRB, 0.63296E+01_JPRB, 0.60684E+01_JPRB, 0.58198E+01_JPRB, 0.59388E+01_JPRB, 0.62433E+01_JPRB, 0.70686E+01_JPRB ,&
&  0.88541E+01_JPRB, 0.99724E+01_JPRB, 0.10189E+02_JPRB, 0.10746E+02_JPRB, 0.11904E+02_JPRB, 0.12772E+02_JPRB, 0.10568E+02_JPRB ,&
&  0.76451E+01_JPRB, 0.52764E+01_JPRB, 0.40306E+01_JPRB, 0.40890E+01_JPRB, 0.47617E+01_JPRB, 0.49038E+01_JPRB, 0.60908E+01_JPRB ,&
&  0.56481E+01_JPRB, 0.21885E+01_JPRB, 0.14570E+01_JPRB, 0.18738E+01_JPRB, 0.97104E+00_JPRB, 0.87380E+00_JPRB, 0.15805E+01_JPRB ,&
&  0.25131E+01_JPRB, 0.34053E+01_JPRB, 0.34364E+01_JPRB, 0.34076E+01_JPRB, 0.36322E+01_JPRB, 0.22276E+01_JPRB, 0.11482E+01_JPRB ,&
&  0.87513E+00_JPRB, 0.92799E+00_JPRB, 0.92739E+00_JPRB, 0.85449E+00_JPRB, 0.82978E+00_JPRB, 0.85638E+00_JPRB, 0.77824E+00_JPRB ,&
&  0.78722E+00_JPRB, 0.79168E+00_JPRB, 0.79408E+00_JPRB, 0.78040E+00_JPRB, 0.76036E+00_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.86012E+00_JPRB, 0.88304E+00_JPRB, 0.92325E+00_JPRB, 0.95047E+00_JPRB, 0.95229E+00_JPRB, 0.96937E+00_JPRB, 0.99109E+00_JPRB ,&
&  0.10646E+01_JPRB, 0.11835E+01_JPRB, 0.13143E+01_JPRB, 0.14385E+01_JPRB, 0.14814E+01_JPRB, 0.14960E+01_JPRB, 0.15872E+01_JPRB ,&
&  0.17375E+01_JPRB, 0.19532E+01_JPRB, 0.21576E+01_JPRB, 0.23056E+01_JPRB, 0.23190E+01_JPRB, 0.23692E+01_JPRB, 0.19265E+01_JPRB ,&
&  0.17195E+01_JPRB, 0.19323E+01_JPRB, 0.16313E+01_JPRB, 0.17618E+01_JPRB, 0.19332E+01_JPRB, 0.20729E+01_JPRB, 0.20174E+01_JPRB ,&
&  0.19565E+01_JPRB, 0.19250E+01_JPRB, 0.18570E+01_JPRB, 0.18575E+01_JPRB, 0.19365E+01_JPRB, 0.20109E+01_JPRB, 0.20588E+01_JPRB ,&
&  0.21212E+01_JPRB, 0.21844E+01_JPRB, 0.23430E+01_JPRB, 0.25600E+01_JPRB, 0.27985E+01_JPRB, 0.30756E+01_JPRB, 0.34640E+01_JPRB ,&
&  0.38982E+01_JPRB, 0.42949E+01_JPRB, 0.46806E+01_JPRB, 0.46671E+01_JPRB, 0.49790E+01_JPRB, 0.53761E+01_JPRB, 0.56966E+01_JPRB ,&
&  0.55830E+01_JPRB, 0.54002E+01_JPRB, 0.58491E+01_JPRB, 0.64733E+01_JPRB, 0.69245E+01_JPRB, 0.72347E+01_JPRB, 0.80700E+01_JPRB ,&
&  0.97297E+01_JPRB, 0.10802E+02_JPRB, 0.11115E+02_JPRB, 0.96796E+01_JPRB, 0.88729E+01_JPRB, 0.83103E+01_JPRB, 0.81354E+01_JPRB ,&
&  0.68486E+01_JPRB, 0.53225E+01_JPRB, 0.45681E+01_JPRB, 0.45469E+01_JPRB, 0.47260E+01_JPRB, 0.48188E+01_JPRB, 0.53909E+01_JPRB ,&
&  0.57356E+01_JPRB, 0.50655E+01_JPRB, 0.38616E+01_JPRB, 0.23553E+01_JPRB, 0.14420E+01_JPRB, 0.12253E+01_JPRB, 0.15636E+01_JPRB ,&
&  0.24184E+01_JPRB, 0.26769E+01_JPRB, 0.26739E+01_JPRB, 0.34567E+01_JPRB, 0.26698E+01_JPRB, 0.15589E+01_JPRB, 0.91265E+00_JPRB ,&
&  0.77965E+00_JPRB, 0.81593E+00_JPRB, 0.82557E+00_JPRB, 0.80118E+00_JPRB, 0.82459E+00_JPRB, 0.85129E+00_JPRB, 0.82756E+00_JPRB ,&
&  0.84511E+00_JPRB, 0.86635E+00_JPRB, 0.85162E+00_JPRB, 0.83336E+00_JPRB, 0.84491E+00_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.10477E+01_JPRB, 0.10969E+01_JPRB, 0.11360E+01_JPRB, 0.11597E+01_JPRB, 0.11830E+01_JPRB, 0.11730E+01_JPRB, 0.12064E+01_JPRB ,&
&  0.13098E+01_JPRB, 0.14297E+01_JPRB, 0.15364E+01_JPRB, 0.16022E+01_JPRB, 0.16323E+01_JPRB, 0.16935E+01_JPRB, 0.18170E+01_JPRB ,&
&  0.19389E+01_JPRB, 0.20866E+01_JPRB, 0.22091E+01_JPRB, 0.23448E+01_JPRB, 0.24379E+01_JPRB, 0.26839E+01_JPRB, 0.23582E+01_JPRB ,&
&  0.16499E+01_JPRB, 0.23459E+01_JPRB, 0.17720E+01_JPRB, 0.19425E+01_JPRB, 0.20639E+01_JPRB, 0.22331E+01_JPRB, 0.21856E+01_JPRB ,&
&  0.21660E+01_JPRB, 0.22084E+01_JPRB, 0.22232E+01_JPRB, 0.22020E+01_JPRB, 0.22157E+01_JPRB, 0.23393E+01_JPRB, 0.24684E+01_JPRB ,&
&  0.25855E+01_JPRB, 0.26856E+01_JPRB, 0.28345E+01_JPRB, 0.30264E+01_JPRB, 0.32490E+01_JPRB, 0.35252E+01_JPRB, 0.38525E+01_JPRB ,&
&  0.41307E+01_JPRB, 0.43961E+01_JPRB, 0.44749E+01_JPRB, 0.45448E+01_JPRB, 0.46779E+01_JPRB, 0.49295E+01_JPRB, 0.52213E+01_JPRB ,&
&  0.53250E+01_JPRB, 0.53941E+01_JPRB, 0.59519E+01_JPRB, 0.63313E+01_JPRB, 0.65804E+01_JPRB, 0.72366E+01_JPRB, 0.76614E+01_JPRB ,&
&  0.90119E+01_JPRB, 0.90140E+01_JPRB, 0.91381E+01_JPRB, 0.75818E+01_JPRB, 0.59511E+01_JPRB, 0.51896E+01_JPRB, 0.49475E+01_JPRB ,&
&  0.43077E+01_JPRB, 0.41315E+01_JPRB, 0.38711E+01_JPRB, 0.39239E+01_JPRB, 0.42786E+01_JPRB, 0.46487E+01_JPRB, 0.44720E+01_JPRB ,&
&  0.41905E+01_JPRB, 0.29829E+01_JPRB, 0.22495E+01_JPRB, 0.18921E+01_JPRB, 0.13962E+01_JPRB, 0.12519E+01_JPRB, 0.16950E+01_JPRB ,&
&  0.23112E+01_JPRB, 0.24901E+01_JPRB, 0.27155E+01_JPRB, 0.26856E+01_JPRB, 0.16436E+01_JPRB, 0.10244E+01_JPRB, 0.76491E+00_JPRB ,&
&  0.72097E+00_JPRB, 0.76610E+00_JPRB, 0.78626E+00_JPRB, 0.80167E+00_JPRB, 0.84437E+00_JPRB, 0.83583E+00_JPRB, 0.88632E+00_JPRB ,&
&  0.89996E+00_JPRB, 0.92835E+00_JPRB, 0.95475E+00_JPRB, 0.96214E+00_JPRB, 0.99502E+00_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.12954E+01_JPRB, 0.14179E+01_JPRB, 0.15351E+01_JPRB, 0.16134E+01_JPRB, 0.15747E+01_JPRB, 0.15318E+01_JPRB, 0.15095E+01_JPRB ,&
&  0.14989E+01_JPRB, 0.15263E+01_JPRB, 0.16066E+01_JPRB, 0.16784E+01_JPRB, 0.17407E+01_JPRB, 0.18429E+01_JPRB, 0.19912E+01_JPRB ,&
&  0.21467E+01_JPRB, 0.23008E+01_JPRB, 0.24540E+01_JPRB, 0.26404E+01_JPRB, 0.28751E+01_JPRB, 0.32476E+01_JPRB, 0.31531E+01_JPRB ,&
&  0.22655E+01_JPRB, 0.18986E+01_JPRB, 0.20725E+01_JPRB, 0.20142E+01_JPRB, 0.20103E+01_JPRB, 0.21884E+01_JPRB, 0.22797E+01_JPRB ,&
&  0.22895E+01_JPRB, 0.23260E+01_JPRB, 0.23763E+01_JPRB, 0.23794E+01_JPRB, 0.23896E+01_JPRB, 0.25415E+01_JPRB, 0.26930E+01_JPRB ,&
&  0.28427E+01_JPRB, 0.29464E+01_JPRB, 0.30648E+01_JPRB, 0.32148E+01_JPRB, 0.33883E+01_JPRB, 0.35865E+01_JPRB, 0.38109E+01_JPRB ,&
&  0.39627E+01_JPRB, 0.41345E+01_JPRB, 0.42191E+01_JPRB, 0.42747E+01_JPRB, 0.43779E+01_JPRB, 0.44593E+01_JPRB, 0.45749E+01_JPRB ,&
&  0.48279E+01_JPRB, 0.52167E+01_JPRB, 0.56182E+01_JPRB, 0.58544E+01_JPRB, 0.58770E+01_JPRB, 0.64938E+01_JPRB, 0.68173E+01_JPRB ,&
&  0.71056E+01_JPRB, 0.66307E+01_JPRB, 0.66216E+01_JPRB, 0.55798E+01_JPRB, 0.46373E+01_JPRB, 0.37635E+01_JPRB, 0.34597E+01_JPRB ,&
&  0.32084E+01_JPRB, 0.30252E+01_JPRB, 0.27930E+01_JPRB, 0.28386E+01_JPRB, 0.33292E+01_JPRB, 0.35743E+01_JPRB, 0.36190E+01_JPRB ,&
&  0.29799E+01_JPRB, 0.22703E+01_JPRB, 0.16250E+01_JPRB, 0.16419E+01_JPRB, 0.13382E+01_JPRB, 0.14938E+01_JPRB, 0.17778E+01_JPRB ,&
&  0.20294E+01_JPRB, 0.18329E+01_JPRB, 0.18699E+01_JPRB, 0.17945E+01_JPRB, 0.11290E+01_JPRB, 0.79256E+00_JPRB, 0.65754E+00_JPRB ,&
&  0.62425E+00_JPRB, 0.66925E+00_JPRB, 0.70075E+00_JPRB, 0.71797E+00_JPRB, 0.77559E+00_JPRB, 0.83021E+00_JPRB, 0.90059E+00_JPRB ,&
&  0.93151E+00_JPRB, 0.97946E+00_JPRB, 0.10351E+01_JPRB, 0.10926E+01_JPRB, 0.11820E+01_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.12998E+01_JPRB, 0.13767E+01_JPRB, 0.14516E+01_JPRB, 0.14391E+01_JPRB, 0.14376E+01_JPRB, 0.14159E+01_JPRB, 0.13995E+01_JPRB ,&
&  0.14536E+01_JPRB, 0.15139E+01_JPRB, 0.15993E+01_JPRB, 0.16738E+01_JPRB, 0.17405E+01_JPRB, 0.18517E+01_JPRB, 0.19856E+01_JPRB ,&
&  0.20812E+01_JPRB, 0.21726E+01_JPRB, 0.22715E+01_JPRB, 0.24086E+01_JPRB, 0.25558E+01_JPRB, 0.27643E+01_JPRB, 0.28788E+01_JPRB ,&
&  0.22721E+01_JPRB, 0.14932E+01_JPRB, 0.16984E+01_JPRB, 0.18527E+01_JPRB, 0.18942E+01_JPRB, 0.20994E+01_JPRB, 0.22260E+01_JPRB ,&
&  0.21514E+01_JPRB, 0.21986E+01_JPRB, 0.22996E+01_JPRB, 0.23182E+01_JPRB, 0.23008E+01_JPRB, 0.24724E+01_JPRB, 0.26438E+01_JPRB ,&
&  0.28090E+01_JPRB, 0.29523E+01_JPRB, 0.30692E+01_JPRB, 0.31992E+01_JPRB, 0.33093E+01_JPRB, 0.34164E+01_JPRB, 0.35143E+01_JPRB ,&
&  0.36169E+01_JPRB, 0.36761E+01_JPRB, 0.37681E+01_JPRB, 0.39513E+01_JPRB, 0.39990E+01_JPRB, 0.40120E+01_JPRB, 0.40126E+01_JPRB ,&
&  0.41707E+01_JPRB, 0.44195E+01_JPRB, 0.48420E+01_JPRB, 0.52204E+01_JPRB, 0.55000E+01_JPRB, 0.54261E+01_JPRB, 0.54930E+01_JPRB ,&
&  0.51783E+01_JPRB, 0.45900E+01_JPRB, 0.46453E+01_JPRB, 0.48539E+01_JPRB, 0.34051E+01_JPRB, 0.28941E+01_JPRB, 0.24753E+01_JPRB ,&
&  0.25407E+01_JPRB, 0.22338E+01_JPRB, 0.19854E+01_JPRB, 0.20064E+01_JPRB, 0.24890E+01_JPRB, 0.25308E+01_JPRB, 0.25904E+01_JPRB ,&
&  0.21948E+01_JPRB, 0.15018E+01_JPRB, 0.13431E+01_JPRB, 0.15639E+01_JPRB, 0.15678E+01_JPRB, 0.21831E+01_JPRB, 0.18765E+01_JPRB ,&
&  0.14525E+01_JPRB, 0.13271E+01_JPRB, 0.14630E+01_JPRB, 0.14663E+01_JPRB, 0.10102E+01_JPRB, 0.73097E+00_JPRB, 0.63369E+00_JPRB ,&
&  0.59673E+00_JPRB, 0.60198E+00_JPRB, 0.63634E+00_JPRB, 0.64837E+00_JPRB, 0.69732E+00_JPRB, 0.74729E+00_JPRB, 0.84476E+00_JPRB ,&
&  0.92246E+00_JPRB, 0.99132E+00_JPRB, 0.10480E+01_JPRB, 0.11198E+01_JPRB, 0.12025E+01_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.96221E+00_JPRB, 0.10047E+01_JPRB, 0.10117E+01_JPRB, 0.10956E+01_JPRB, 0.11845E+01_JPRB, 0.12339E+01_JPRB, 0.12759E+01_JPRB ,&
&  0.13511E+01_JPRB, 0.14601E+01_JPRB, 0.15508E+01_JPRB, 0.16181E+01_JPRB, 0.16919E+01_JPRB, 0.17928E+01_JPRB, 0.18277E+01_JPRB ,&
&  0.18202E+01_JPRB, 0.18180E+01_JPRB, 0.18380E+01_JPRB, 0.18660E+01_JPRB, 0.19183E+01_JPRB, 0.19737E+01_JPRB, 0.21751E+01_JPRB ,&
&  0.22704E+01_JPRB, 0.21890E+01_JPRB, 0.20819E+01_JPRB, 0.16605E+01_JPRB, 0.12572E+01_JPRB, 0.16161E+01_JPRB, 0.16333E+01_JPRB ,&
&  0.18061E+01_JPRB, 0.19055E+01_JPRB, 0.19388E+01_JPRB, 0.20652E+01_JPRB, 0.21792E+01_JPRB, 0.23072E+01_JPRB, 0.24614E+01_JPRB ,&
&  0.25811E+01_JPRB, 0.27119E+01_JPRB, 0.28340E+01_JPRB, 0.29464E+01_JPRB, 0.30302E+01_JPRB, 0.30822E+01_JPRB, 0.30991E+01_JPRB ,&
&  0.31275E+01_JPRB, 0.31988E+01_JPRB, 0.32974E+01_JPRB, 0.35393E+01_JPRB, 0.36117E+01_JPRB, 0.36873E+01_JPRB, 0.36198E+01_JPRB ,&
&  0.36649E+01_JPRB, 0.37727E+01_JPRB, 0.40628E+01_JPRB, 0.42982E+01_JPRB, 0.43362E+01_JPRB, 0.40471E+01_JPRB, 0.38798E+01_JPRB ,&
&  0.36382E+01_JPRB, 0.33267E+01_JPRB, 0.33467E+01_JPRB, 0.36736E+01_JPRB, 0.24297E+01_JPRB, 0.24336E+01_JPRB, 0.24641E+01_JPRB ,&
&  0.20935E+01_JPRB, 0.18503E+01_JPRB, 0.16783E+01_JPRB, 0.15041E+01_JPRB, 0.16976E+01_JPRB, 0.18347E+01_JPRB, 0.21251E+01_JPRB ,&
&  0.17671E+01_JPRB, 0.12229E+01_JPRB, 0.11680E+01_JPRB, 0.14473E+01_JPRB, 0.16895E+01_JPRB, 0.22029E+01_JPRB, 0.16111E+01_JPRB ,&
&  0.11245E+01_JPRB, 0.13022E+01_JPRB, 0.14481E+01_JPRB, 0.16981E+01_JPRB, 0.13158E+01_JPRB, 0.72581E+00_JPRB, 0.59945E+00_JPRB ,&
&  0.57436E+00_JPRB, 0.58260E+00_JPRB, 0.59959E+00_JPRB, 0.60990E+00_JPRB, 0.63725E+00_JPRB, 0.67260E+00_JPRB, 0.69900E+00_JPRB ,&
&  0.72218E+00_JPRB, 0.73785E+00_JPRB, 0.77452E+00_JPRB, 0.82941E+00_JPRB, 0.90572E+00_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.67230E+00_JPRB, 0.67505E+00_JPRB, 0.66286E+00_JPRB, 0.68199E+00_JPRB, 0.71723E+00_JPRB, 0.79397E+00_JPRB, 0.86960E+00_JPRB ,&
&  0.95668E+00_JPRB, 0.10666E+01_JPRB, 0.12249E+01_JPRB, 0.13734E+01_JPRB, 0.14274E+01_JPRB, 0.14160E+01_JPRB, 0.12973E+01_JPRB ,&
&  0.11404E+01_JPRB, 0.97534E+00_JPRB, 0.88209E+00_JPRB, 0.85518E+00_JPRB, 0.90859E+00_JPRB, 0.93863E+00_JPRB, 0.10140E+01_JPRB ,&
&  0.11422E+01_JPRB, 0.13512E+01_JPRB, 0.15684E+01_JPRB, 0.20535E+01_JPRB, 0.23252E+01_JPRB, 0.16650E+01_JPRB, 0.15567E+01_JPRB ,&
&  0.18462E+01_JPRB, 0.19219E+01_JPRB, 0.15741E+01_JPRB, 0.15719E+01_JPRB, 0.17360E+01_JPRB, 0.18727E+01_JPRB, 0.20457E+01_JPRB ,&
&  0.21339E+01_JPRB, 0.22278E+01_JPRB, 0.23120E+01_JPRB, 0.23605E+01_JPRB, 0.23801E+01_JPRB, 0.23391E+01_JPRB, 0.23001E+01_JPRB ,&
&  0.22796E+01_JPRB, 0.22657E+01_JPRB, 0.24170E+01_JPRB, 0.26180E+01_JPRB, 0.28349E+01_JPRB, 0.30038E+01_JPRB, 0.30476E+01_JPRB ,&
&  0.30807E+01_JPRB, 0.31431E+01_JPRB, 0.34609E+01_JPRB, 0.35389E+01_JPRB, 0.33242E+01_JPRB, 0.30984E+01_JPRB, 0.28023E+01_JPRB ,&
&  0.25015E+01_JPRB, 0.21090E+01_JPRB, 0.16291E+01_JPRB, 0.24287E+01_JPRB, 0.27736E+01_JPRB, 0.23682E+01_JPRB, 0.19246E+01_JPRB ,&
&  0.17513E+01_JPRB, 0.16065E+01_JPRB, 0.14542E+01_JPRB, 0.13660E+01_JPRB, 0.14165E+01_JPRB, 0.18883E+01_JPRB, 0.18467E+01_JPRB ,&
&  0.15452E+01_JPRB, 0.10350E+01_JPRB, 0.90553E+00_JPRB, 0.10339E+01_JPRB, 0.12147E+01_JPRB, 0.15062E+01_JPRB, 0.11076E+01_JPRB ,&
&  0.94208E+00_JPRB, 0.12670E+01_JPRB, 0.14425E+01_JPRB, 0.18881E+01_JPRB, 0.21274E+01_JPRB, 0.61700E+00_JPRB, 0.57525E+00_JPRB ,&
&  0.56528E+00_JPRB, 0.56962E+00_JPRB, 0.59110E+00_JPRB, 0.58897E+00_JPRB, 0.59183E+00_JPRB, 0.59143E+00_JPRB, 0.58488E+00_JPRB ,&
&  0.58785E+00_JPRB, 0.60555E+00_JPRB, 0.62549E+00_JPRB, 0.63560E+00_JPRB, 0.65755E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.64352E+00_JPRB, 0.64386E+00_JPRB, 0.64818E+00_JPRB, 0.65583E+00_JPRB, 0.65747E+00_JPRB, 0.65726E+00_JPRB, 0.67296E+00_JPRB ,&
&  0.70116E+00_JPRB, 0.75217E+00_JPRB, 0.82778E+00_JPRB, 0.87581E+00_JPRB, 0.85959E+00_JPRB, 0.80378E+00_JPRB, 0.77926E+00_JPRB ,&
&  0.77425E+00_JPRB, 0.76984E+00_JPRB, 0.80069E+00_JPRB, 0.85261E+00_JPRB, 0.91948E+00_JPRB, 0.94004E+00_JPRB, 0.10038E+01_JPRB ,&
&  0.10029E+01_JPRB, 0.10338E+01_JPRB, 0.11296E+01_JPRB, 0.12176E+01_JPRB, 0.18414E+01_JPRB, 0.17523E+01_JPRB, 0.14363E+01_JPRB ,&
&  0.14591E+01_JPRB, 0.14816E+01_JPRB, 0.14377E+01_JPRB, 0.12784E+01_JPRB, 0.12582E+01_JPRB, 0.12357E+01_JPRB, 0.12708E+01_JPRB ,&
&  0.12680E+01_JPRB, 0.13683E+01_JPRB, 0.14552E+01_JPRB, 0.15533E+01_JPRB, 0.16242E+01_JPRB, 0.16492E+01_JPRB, 0.17151E+01_JPRB ,&
&  0.17732E+01_JPRB, 0.18353E+01_JPRB, 0.19161E+01_JPRB, 0.17422E+01_JPRB, 0.19140E+01_JPRB, 0.21217E+01_JPRB, 0.22725E+01_JPRB ,&
&  0.23419E+01_JPRB, 0.23807E+01_JPRB, 0.25937E+01_JPRB, 0.25296E+01_JPRB, 0.24163E+01_JPRB, 0.21977E+01_JPRB, 0.19966E+01_JPRB ,&
&  0.18407E+01_JPRB, 0.13553E+01_JPRB, 0.87830E+00_JPRB, 0.12809E+01_JPRB, 0.15521E+01_JPRB, 0.13477E+01_JPRB, 0.14405E+01_JPRB ,&
&  0.13758E+01_JPRB, 0.13021E+01_JPRB, 0.12272E+01_JPRB, 0.12078E+01_JPRB, 0.11611E+01_JPRB, 0.13851E+01_JPRB, 0.14843E+01_JPRB ,&
&  0.12298E+01_JPRB, 0.88627E+00_JPRB, 0.75174E+00_JPRB, 0.76914E+00_JPRB, 0.86154E+00_JPRB, 0.10099E+01_JPRB, 0.10937E+01_JPRB ,&
&  0.11273E+01_JPRB, 0.95813E+00_JPRB, 0.11201E+01_JPRB, 0.13798E+01_JPRB, 0.83185E+00_JPRB, 0.52327E+00_JPRB, 0.50628E+00_JPRB ,&
&  0.51049E+00_JPRB, 0.52445E+00_JPRB, 0.54784E+00_JPRB, 0.56270E+00_JPRB, 0.57375E+00_JPRB, 0.58335E+00_JPRB, 0.59821E+00_JPRB ,&
&  0.60818E+00_JPRB, 0.62064E+00_JPRB, 0.62647E+00_JPRB, 0.63314E+00_JPRB, 0.64005E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.82845E+00_JPRB, 0.84436E+00_JPRB, 0.86665E+00_JPRB, 0.88935E+00_JPRB, 0.90288E+00_JPRB, 0.92391E+00_JPRB, 0.94481E+00_JPRB ,&
&  0.97289E+00_JPRB, 0.10135E+01_JPRB, 0.10405E+01_JPRB, 0.10507E+01_JPRB, 0.10659E+01_JPRB, 0.10864E+01_JPRB, 0.10951E+01_JPRB ,&
&  0.11221E+01_JPRB, 0.11356E+01_JPRB, 0.11979E+01_JPRB, 0.12274E+01_JPRB, 0.12679E+01_JPRB, 0.13206E+01_JPRB, 0.13292E+01_JPRB ,&
&  0.13195E+01_JPRB, 0.13079E+01_JPRB, 0.13219E+01_JPRB, 0.13197E+01_JPRB, 0.14063E+01_JPRB, 0.14231E+01_JPRB, 0.13622E+01_JPRB ,&
&  0.12251E+01_JPRB, 0.11812E+01_JPRB, 0.13617E+01_JPRB, 0.11435E+01_JPRB, 0.11350E+01_JPRB, 0.11530E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.10765E+01_JPRB, 0.11433E+01_JPRB, 0.12231E+01_JPRB, 0.13033E+01_JPRB, 0.14121E+01_JPRB, 0.14712E+01_JPRB, 0.15163E+01_JPRB ,&
&  0.15714E+01_JPRB, 0.16328E+01_JPRB, 0.16681E+01_JPRB, 0.16370E+01_JPRB, 0.16385E+01_JPRB, 0.19261E+01_JPRB, 0.20527E+01_JPRB ,&
&  0.21067E+01_JPRB, 0.21693E+01_JPRB, 0.18249E+01_JPRB, 0.16993E+01_JPRB, 0.17920E+01_JPRB, 0.15931E+01_JPRB, 0.15147E+01_JPRB ,&
&  0.16548E+01_JPRB, 0.14215E+01_JPRB, 0.74859E+00_JPRB, 0.98326E+00_JPRB, 0.11473E+01_JPRB, 0.11969E+01_JPRB, 0.12129E+01_JPRB ,&
&  0.11394E+01_JPRB, 0.10029E+01_JPRB, 0.95449E+00_JPRB, 0.10195E+01_JPRB, 0.10656E+01_JPRB, 0.10189E+01_JPRB, 0.10274E+01_JPRB ,&
&  0.91127E+00_JPRB, 0.79196E+00_JPRB, 0.72915E+00_JPRB, 0.83473E+00_JPRB, 0.11117E+01_JPRB, 0.16297E+01_JPRB, 0.15123E+01_JPRB ,&
&  0.11493E+01_JPRB, 0.97684E+00_JPRB, 0.14836E+01_JPRB, 0.20399E+01_JPRB, 0.83463E+00_JPRB, 0.60542E+00_JPRB, 0.52503E+00_JPRB ,&
&  0.55706E+00_JPRB, 0.57327E+00_JPRB, 0.58521E+00_JPRB, 0.60940E+00_JPRB, 0.64564E+00_JPRB, 0.66163E+00_JPRB, 0.66752E+00_JPRB ,&
&  0.68859E+00_JPRB, 0.72131E+00_JPRB, 0.74594E+00_JPRB, 0.77610E+00_JPRB, 0.80295E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.97803E+00_JPRB, 0.10013E+01_JPRB, 0.10176E+01_JPRB, 0.10343E+01_JPRB, 0.10562E+01_JPRB, 0.10780E+01_JPRB, 0.10908E+01_JPRB ,&
&  0.11132E+01_JPRB, 0.11323E+01_JPRB, 0.11416E+01_JPRB, 0.11623E+01_JPRB, 0.11703E+01_JPRB, 0.12280E+01_JPRB, 0.12512E+01_JPRB ,&
&  0.13008E+01_JPRB, 0.13298E+01_JPRB, 0.13405E+01_JPRB, 0.13454E+01_JPRB, 0.13612E+01_JPRB, 0.13578E+01_JPRB, 0.13515E+01_JPRB ,&
&  0.13761E+01_JPRB, 0.14230E+01_JPRB, 0.14775E+01_JPRB, 0.15298E+01_JPRB, 0.15581E+01_JPRB, 0.16547E+01_JPRB, 0.17826E+01_JPRB ,&
&  0.13438E+01_JPRB, 0.13549E+01_JPRB, 0.13907E+01_JPRB, 0.12230E+01_JPRB, 0.12497E+01_JPRB, 0.12854E+01_JPRB, 0.12479E+01_JPRB ,&
&  0.12323E+01_JPRB, 0.12857E+01_JPRB, 0.13277E+01_JPRB, 0.13692E+01_JPRB, 0.14315E+01_JPRB, 0.15026E+01_JPRB, 0.15535E+01_JPRB ,&
&  0.16357E+01_JPRB, 0.17014E+01_JPRB, 0.17558E+01_JPRB, 0.18457E+01_JPRB, 0.18055E+01_JPRB, 0.21290E+01_JPRB, 0.22369E+01_JPRB ,&
&  0.22604E+01_JPRB, 0.21902E+01_JPRB, 0.16847E+01_JPRB, 0.16075E+01_JPRB, 0.16741E+01_JPRB, 0.15968E+01_JPRB, 0.15055E+01_JPRB ,&
&  0.16437E+01_JPRB, 0.15522E+01_JPRB, 0.10752E+01_JPRB, 0.11492E+01_JPRB, 0.11590E+01_JPRB, 0.11483E+01_JPRB, 0.10576E+01_JPRB ,&
&  0.95817E+00_JPRB, 0.82526E+00_JPRB, 0.78546E+00_JPRB, 0.81285E+00_JPRB, 0.82229E+00_JPRB, 0.79695E+00_JPRB, 0.81158E+00_JPRB ,&
&  0.80341E+00_JPRB, 0.77927E+00_JPRB, 0.80022E+00_JPRB, 0.81017E+00_JPRB, 0.12031E+01_JPRB, 0.14079E+01_JPRB, 0.20025E+01_JPRB ,&
&  0.96507E+00_JPRB, 0.92117E+00_JPRB, 0.83687E+00_JPRB, 0.11646E+01_JPRB, 0.78158E+00_JPRB, 0.55925E+00_JPRB, 0.57540E+00_JPRB ,&
&  0.64013E+00_JPRB, 0.66998E+00_JPRB, 0.68026E+00_JPRB, 0.69791E+00_JPRB, 0.74390E+00_JPRB, 0.77016E+00_JPRB, 0.79412E+00_JPRB ,&
&  0.80102E+00_JPRB, 0.83903E+00_JPRB, 0.89001E+00_JPRB, 0.92835E+00_JPRB, 0.94594E+00_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.10452E+01_JPRB, 0.10743E+01_JPRB, 0.10989E+01_JPRB, 0.11204E+01_JPRB, 0.11408E+01_JPRB, 0.11638E+01_JPRB, 0.11850E+01_JPRB ,&
&  0.12048E+01_JPRB, 0.12199E+01_JPRB, 0.12293E+01_JPRB, 0.12454E+01_JPRB, 0.12602E+01_JPRB, 0.13030E+01_JPRB, 0.13198E+01_JPRB ,&
&  0.13623E+01_JPRB, 0.13803E+01_JPRB, 0.13790E+01_JPRB, 0.14035E+01_JPRB, 0.13925E+01_JPRB, 0.13687E+01_JPRB, 0.13638E+01_JPRB ,&
&  0.13712E+01_JPRB, 0.13710E+01_JPRB, 0.13729E+01_JPRB, 0.13813E+01_JPRB, 0.13908E+01_JPRB, 0.13659E+01_JPRB, 0.14583E+01_JPRB ,&
&  0.11187E+01_JPRB, 0.15370E+01_JPRB, 0.16125E+01_JPRB, 0.14634E+01_JPRB, 0.14099E+01_JPRB, 0.13136E+01_JPRB, 0.13093E+01_JPRB ,&
&  0.13306E+01_JPRB, 0.13372E+01_JPRB, 0.13633E+01_JPRB, 0.14112E+01_JPRB, 0.14790E+01_JPRB, 0.15611E+01_JPRB, 0.16318E+01_JPRB ,&
&  0.17298E+01_JPRB, 0.18200E+01_JPRB, 0.18953E+01_JPRB, 0.19669E+01_JPRB, 0.21006E+01_JPRB, 0.21952E+01_JPRB, 0.22317E+01_JPRB ,&
&  0.22368E+01_JPRB, 0.22634E+01_JPRB, 0.17804E+01_JPRB, 0.17511E+01_JPRB, 0.18961E+01_JPRB, 0.18180E+01_JPRB, 0.16191E+01_JPRB ,&
&  0.16373E+01_JPRB, 0.15056E+01_JPRB, 0.13679E+01_JPRB, 0.12298E+01_JPRB, 0.11400E+01_JPRB, 0.10535E+01_JPRB, 0.95663E+00_JPRB ,&
&  0.78841E+00_JPRB, 0.66441E+00_JPRB, 0.63490E+00_JPRB, 0.67511E+00_JPRB, 0.68815E+00_JPRB, 0.68210E+00_JPRB, 0.65180E+00_JPRB ,&
&  0.65005E+00_JPRB, 0.66762E+00_JPRB, 0.71843E+00_JPRB, 0.82895E+00_JPRB, 0.96906E+00_JPRB, 0.17864E+01_JPRB, 0.20090E+01_JPRB ,&
&  0.95622E+00_JPRB, 0.85562E+00_JPRB, 0.61067E+00_JPRB, 0.75734E+00_JPRB, 0.76613E+00_JPRB, 0.67681E+00_JPRB, 0.70362E+00_JPRB ,&
&  0.80924E+00_JPRB, 0.86518E+00_JPRB, 0.88801E+00_JPRB, 0.82497E+00_JPRB, 0.84180E+00_JPRB, 0.82781E+00_JPRB, 0.83389E+00_JPRB ,&
&  0.85635E+00_JPRB, 0.89171E+00_JPRB, 0.93517E+00_JPRB, 0.97640E+00_JPRB, 0.10120E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.10730E+01_JPRB, 0.11020E+01_JPRB, 0.11314E+01_JPRB, 0.11635E+01_JPRB, 0.11957E+01_JPRB, 0.12199E+01_JPRB, 0.12462E+01_JPRB ,&
&  0.12733E+01_JPRB, 0.12940E+01_JPRB, 0.13028E+01_JPRB, 0.13116E+01_JPRB, 0.13251E+01_JPRB, 0.13647E+01_JPRB, 0.13972E+01_JPRB ,&
&  0.13868E+01_JPRB, 0.14056E+01_JPRB, 0.14176E+01_JPRB, 0.14283E+01_JPRB, 0.14178E+01_JPRB, 0.13848E+01_JPRB, 0.13960E+01_JPRB ,&
&  0.13913E+01_JPRB, 0.13962E+01_JPRB, 0.14028E+01_JPRB, 0.14358E+01_JPRB, 0.14848E+01_JPRB, 0.15969E+01_JPRB, 0.13679E+01_JPRB ,&
&  0.15149E+01_JPRB, 0.16585E+01_JPRB, 0.16768E+01_JPRB, 0.15658E+01_JPRB, 0.14353E+01_JPRB, 0.14018E+01_JPRB, 0.13987E+01_JPRB ,&
&  0.14164E+01_JPRB, 0.14002E+01_JPRB, 0.14171E+01_JPRB, 0.14417E+01_JPRB, 0.14879E+01_JPRB, 0.15694E+01_JPRB, 0.16327E+01_JPRB ,&
&  0.17284E+01_JPRB, 0.18768E+01_JPRB, 0.20229E+01_JPRB, 0.21568E+01_JPRB, 0.23264E+01_JPRB, 0.24978E+01_JPRB, 0.25995E+01_JPRB ,&
&  0.26495E+01_JPRB, 0.27092E+01_JPRB, 0.24743E+01_JPRB, 0.25103E+01_JPRB, 0.26371E+01_JPRB, 0.23810E+01_JPRB, 0.19885E+01_JPRB ,&
&  0.18090E+01_JPRB, 0.14263E+01_JPRB, 0.12129E+01_JPRB, 0.13039E+01_JPRB, 0.11585E+01_JPRB, 0.10097E+01_JPRB, 0.85482E+00_JPRB ,&
&  0.68516E+00_JPRB, 0.59818E+00_JPRB, 0.59543E+00_JPRB, 0.63830E+00_JPRB, 0.63360E+00_JPRB, 0.63862E+00_JPRB, 0.65848E+00_JPRB ,&
&  0.69519E+00_JPRB, 0.73522E+00_JPRB, 0.79937E+00_JPRB, 0.91933E+00_JPRB, 0.10733E+01_JPRB, 0.10737E+01_JPRB, 0.12218E+01_JPRB ,&
&  0.92852E+00_JPRB, 0.76688E+00_JPRB, 0.81488E+00_JPRB, 0.86300E+00_JPRB, 0.89272E+00_JPRB, 0.87718E+00_JPRB, 0.97762E+00_JPRB ,&
&  0.11223E+01_JPRB, 0.12599E+01_JPRB, 0.12701E+01_JPRB, 0.10542E+01_JPRB, 0.94386E+00_JPRB, 0.89754E+00_JPRB, 0.89401E+00_JPRB ,&
&  0.90660E+00_JPRB, 0.92906E+00_JPRB, 0.96501E+00_JPRB, 0.10062E+01_JPRB, 0.10423E+01_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.10746E+01_JPRB, 0.11055E+01_JPRB, 0.11369E+01_JPRB, 0.11671E+01_JPRB, 0.11903E+01_JPRB, 0.12088E+01_JPRB, 0.12230E+01_JPRB ,&
&  0.12465E+01_JPRB, 0.12698E+01_JPRB, 0.12874E+01_JPRB, 0.12971E+01_JPRB, 0.13067E+01_JPRB, 0.13298E+01_JPRB, 0.13598E+01_JPRB ,&
&  0.13964E+01_JPRB, 0.14187E+01_JPRB, 0.14626E+01_JPRB, 0.14531E+01_JPRB, 0.14427E+01_JPRB, 0.14537E+01_JPRB, 0.14917E+01_JPRB ,&
&  0.15074E+01_JPRB, 0.15046E+01_JPRB, 0.15071E+01_JPRB, 0.15502E+01_JPRB, 0.15971E+01_JPRB, 0.16704E+01_JPRB, 0.14729E+01_JPRB ,&
&  0.16467E+01_JPRB, 0.17706E+01_JPRB, 0.17357E+01_JPRB, 0.16671E+01_JPRB, 0.15394E+01_JPRB, 0.15020E+01_JPRB, 0.14936E+01_JPRB ,&
&  0.14861E+01_JPRB, 0.14852E+01_JPRB, 0.14807E+01_JPRB, 0.14934E+01_JPRB, 0.15241E+01_JPRB, 0.15729E+01_JPRB, 0.16471E+01_JPRB ,&
&  0.17274E+01_JPRB, 0.18435E+01_JPRB, 0.20234E+01_JPRB, 0.22054E+01_JPRB, 0.24489E+01_JPRB, 0.26880E+01_JPRB, 0.28676E+01_JPRB ,&
&  0.30443E+01_JPRB, 0.32223E+01_JPRB, 0.30490E+01_JPRB, 0.31537E+01_JPRB, 0.33446E+01_JPRB, 0.29519E+01_JPRB, 0.22644E+01_JPRB ,&
&  0.15691E+01_JPRB, 0.13301E+01_JPRB, 0.13196E+01_JPRB, 0.13485E+01_JPRB, 0.11555E+01_JPRB, 0.98846E+00_JPRB, 0.86943E+00_JPRB ,&
&  0.72812E+00_JPRB, 0.62592E+00_JPRB, 0.63321E+00_JPRB, 0.62747E+00_JPRB, 0.61936E+00_JPRB, 0.64960E+00_JPRB, 0.68887E+00_JPRB ,&
&  0.73793E+00_JPRB, 0.82152E+00_JPRB, 0.91135E+00_JPRB, 0.10108E+01_JPRB, 0.10792E+01_JPRB, 0.10427E+01_JPRB, 0.11146E+01_JPRB ,&
&  0.10337E+01_JPRB, 0.10048E+01_JPRB, 0.10161E+01_JPRB, 0.81841E+00_JPRB, 0.10888E+01_JPRB, 0.11595E+01_JPRB, 0.12779E+01_JPRB ,&
&  0.12448E+01_JPRB, 0.16609E+01_JPRB, 0.20315E+01_JPRB, 0.15904E+01_JPRB, 0.12666E+01_JPRB, 0.95133E+00_JPRB, 0.90227E+00_JPRB ,&
&  0.91277E+00_JPRB, 0.93162E+00_JPRB, 0.96358E+00_JPRB, 0.10034E+01_JPRB, 0.10401E+01_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.10604E+01_JPRB, 0.10965E+01_JPRB, 0.11351E+01_JPRB, 0.11622E+01_JPRB, 0.11790E+01_JPRB, 0.11694E+01_JPRB, 0.11603E+01_JPRB ,&
&  0.11643E+01_JPRB, 0.11775E+01_JPRB, 0.11973E+01_JPRB, 0.12186E+01_JPRB, 0.12331E+01_JPRB, 0.12471E+01_JPRB, 0.12714E+01_JPRB ,&
&  0.13014E+01_JPRB, 0.13393E+01_JPRB, 0.13893E+01_JPRB, 0.14312E+01_JPRB, 0.14778E+01_JPRB, 0.15180E+01_JPRB, 0.15695E+01_JPRB ,&
&  0.16150E+01_JPRB, 0.16773E+01_JPRB, 0.16974E+01_JPRB, 0.17401E+01_JPRB, 0.17665E+01_JPRB, 0.18284E+01_JPRB, 0.14849E+01_JPRB ,&
&  0.16940E+01_JPRB, 0.17740E+01_JPRB, 0.17623E+01_JPRB, 0.16858E+01_JPRB, 0.15642E+01_JPRB, 0.15068E+01_JPRB, 0.14693E+01_JPRB ,&
&  0.14601E+01_JPRB, 0.14322E+01_JPRB, 0.13741E+01_JPRB, 0.14532E+01_JPRB, 0.15266E+01_JPRB, 0.15360E+01_JPRB, 0.16183E+01_JPRB ,&
&  0.17051E+01_JPRB, 0.18283E+01_JPRB, 0.19814E+01_JPRB, 0.22058E+01_JPRB, 0.24869E+01_JPRB, 0.27533E+01_JPRB, 0.30218E+01_JPRB ,&
&  0.33001E+01_JPRB, 0.35958E+01_JPRB, 0.38293E+01_JPRB, 0.34979E+01_JPRB, 0.34959E+01_JPRB, 0.30820E+01_JPRB, 0.22671E+01_JPRB ,&
&  0.15004E+01_JPRB, 0.12904E+01_JPRB, 0.13169E+01_JPRB, 0.13574E+01_JPRB, 0.11769E+01_JPRB, 0.98666E+00_JPRB, 0.86156E+00_JPRB ,&
&  0.73887E+00_JPRB, 0.67406E+00_JPRB, 0.65816E+00_JPRB, 0.63175E+00_JPRB, 0.62415E+00_JPRB, 0.66056E+00_JPRB, 0.70624E+00_JPRB ,&
&  0.76427E+00_JPRB, 0.84521E+00_JPRB, 0.94314E+00_JPRB, 0.10475E+01_JPRB, 0.11537E+01_JPRB, 0.11907E+01_JPRB, 0.10933E+01_JPRB ,&
&  0.10921E+01_JPRB, 0.10614E+01_JPRB, 0.11059E+01_JPRB, 0.12055E+01_JPRB, 0.12523E+01_JPRB, 0.14773E+01_JPRB, 0.17141E+01_JPRB ,&
&  0.19361E+01_JPRB, 0.18005E+01_JPRB, 0.21182E+01_JPRB, 0.29289E+01_JPRB, 0.26948E+01_JPRB, 0.21975E+01_JPRB, 0.87390E+00_JPRB ,&
&  0.87261E+00_JPRB, 0.87581E+00_JPRB, 0.91510E+00_JPRB, 0.96534E+00_JPRB, 0.10235E+01_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.88125E+00_JPRB, 0.93765E+00_JPRB, 0.99479E+00_JPRB, 0.10683E+01_JPRB, 0.11305E+01_JPRB, 0.11302E+01_JPRB, 0.11080E+01_JPRB ,&
&  0.10930E+01_JPRB, 0.10894E+01_JPRB, 0.11013E+01_JPRB, 0.11217E+01_JPRB, 0.11386E+01_JPRB, 0.11502E+01_JPRB, 0.11674E+01_JPRB ,&
&  0.11925E+01_JPRB, 0.12245E+01_JPRB, 0.12611E+01_JPRB, 0.13027E+01_JPRB, 0.13417E+01_JPRB, 0.13983E+01_JPRB, 0.14646E+01_JPRB ,&
&  0.15363E+01_JPRB, 0.16155E+01_JPRB, 0.16827E+01_JPRB, 0.17508E+01_JPRB, 0.18131E+01_JPRB, 0.18120E+01_JPRB, 0.13399E+01_JPRB ,&
&  0.16539E+01_JPRB, 0.17004E+01_JPRB, 0.17337E+01_JPRB, 0.16609E+01_JPRB, 0.15285E+01_JPRB, 0.14476E+01_JPRB, 0.14236E+01_JPRB ,&
&  0.14004E+01_JPRB, 0.13626E+01_JPRB, 0.13159E+01_JPRB, 0.12934E+01_JPRB, 0.14310E+01_JPRB, 0.14492E+01_JPRB, 0.15502E+01_JPRB ,&
&  0.16560E+01_JPRB, 0.17762E+01_JPRB, 0.19174E+01_JPRB, 0.21386E+01_JPRB, 0.24306E+01_JPRB, 0.27305E+01_JPRB, 0.30588E+01_JPRB ,&
&  0.33881E+01_JPRB, 0.36837E+01_JPRB, 0.38568E+01_JPRB, 0.33850E+01_JPRB, 0.32128E+01_JPRB, 0.29550E+01_JPRB, 0.19459E+01_JPRB ,&
&  0.15008E+01_JPRB, 0.14108E+01_JPRB, 0.14723E+01_JPRB, 0.14111E+01_JPRB, 0.12331E+01_JPRB, 0.10740E+01_JPRB, 0.92649E+00_JPRB ,&
&  0.75660E+00_JPRB, 0.67504E+00_JPRB, 0.65840E+00_JPRB, 0.64430E+00_JPRB, 0.64359E+00_JPRB, 0.68165E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.78645E+00_JPRB, 0.85416E+00_JPRB, 0.91201E+00_JPRB, 0.96424E+00_JPRB, 0.10257E+01_JPRB, 0.11325E+01_JPRB, 0.11985E+01_JPRB ,&
&  0.10800E+01_JPRB, 0.10052E+01_JPRB, 0.10282E+01_JPRB, 0.10568E+01_JPRB, 0.10936E+01_JPRB, 0.11659E+01_JPRB, 0.12424E+01_JPRB ,&
&  0.13690E+01_JPRB, 0.14390E+01_JPRB, 0.12188E+01_JPRB, 0.10205E+01_JPRB, 0.13288E+01_JPRB, 0.14190E+01_JPRB, 0.75634E+00_JPRB ,&
&  0.72546E+00_JPRB, 0.74497E+00_JPRB, 0.76043E+00_JPRB, 0.79846E+00_JPRB, 0.83824E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.70657E+00_JPRB, 0.72533E+00_JPRB, 0.80315E+00_JPRB, 0.82840E+00_JPRB, 0.90625E+00_JPRB, 0.94067E+00_JPRB, 0.97837E+00_JPRB ,&
&  0.98815E+00_JPRB, 0.99357E+00_JPRB, 0.99737E+00_JPRB, 0.10051E+01_JPRB, 0.10147E+01_JPRB, 0.10298E+01_JPRB, 0.10432E+01_JPRB ,&
&  0.10622E+01_JPRB, 0.10868E+01_JPRB, 0.11157E+01_JPRB, 0.11466E+01_JPRB, 0.11876E+01_JPRB, 0.12371E+01_JPRB, 0.13031E+01_JPRB ,&
&  0.13871E+01_JPRB, 0.14860E+01_JPRB, 0.15938E+01_JPRB, 0.17099E+01_JPRB, 0.18335E+01_JPRB, 0.19276E+01_JPRB, 0.19517E+01_JPRB ,&
&  0.13807E+01_JPRB, 0.16776E+01_JPRB, 0.16569E+01_JPRB, 0.15860E+01_JPRB, 0.14911E+01_JPRB, 0.14291E+01_JPRB, 0.14125E+01_JPRB ,&
&  0.13936E+01_JPRB, 0.13139E+01_JPRB, 0.12988E+01_JPRB, 0.12933E+01_JPRB, 0.13172E+01_JPRB, 0.13308E+01_JPRB, 0.14289E+01_JPRB ,&
&  0.15294E+01_JPRB, 0.16635E+01_JPRB, 0.18322E+01_JPRB, 0.20069E+01_JPRB, 0.22829E+01_JPRB, 0.26475E+01_JPRB, 0.30279E+01_JPRB ,&
&  0.34281E+01_JPRB, 0.38444E+01_JPRB, 0.40967E+01_JPRB, 0.36392E+01_JPRB, 0.31727E+01_JPRB, 0.29778E+01_JPRB, 0.21413E+01_JPRB ,&
&  0.17035E+01_JPRB, 0.14681E+01_JPRB, 0.15691E+01_JPRB, 0.15603E+01_JPRB, 0.13479E+01_JPRB, 0.12075E+01_JPRB, 0.10710E+01_JPRB ,&
&  0.89468E+00_JPRB, 0.74406E+00_JPRB, 0.69266E+00_JPRB, 0.68492E+00_JPRB, 0.69198E+00_JPRB, 0.72074E+00_JPRB, 0.74714E+00_JPRB ,&
&  0.78040E+00_JPRB, 0.81780E+00_JPRB, 0.84940E+00_JPRB, 0.88304E+00_JPRB, 0.90122E+00_JPRB, 0.91978E+00_JPRB, 0.92494E+00_JPRB ,&
&  0.93485E+00_JPRB, 0.97119E+00_JPRB, 0.97224E+00_JPRB, 0.98866E+00_JPRB, 0.98241E+00_JPRB, 0.99550E+00_JPRB, 0.10217E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.10579E+01_JPRB, 0.10477E+01_JPRB, 0.10155E+01_JPRB, 0.94725E+00_JPRB, 0.86643E+00_JPRB, 0.70697E+00_JPRB ,&
&  0.68109E+00_JPRB, 0.66242E+00_JPRB, 0.67105E+00_JPRB, 0.67816E+00_JPRB, 0.69736E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.61199E+00_JPRB, 0.62967E+00_JPRB, 0.64838E+00_JPRB, 0.67154E+00_JPRB, 0.70731E+00_JPRB, 0.74980E+00_JPRB, 0.80510E+00_JPRB ,&
&  0.86109E+00_JPRB, 0.87911E+00_JPRB, 0.88374E+00_JPRB, 0.88637E+00_JPRB, 0.89194E+00_JPRB, 0.90685E+00_JPRB, 0.92618E+00_JPRB ,&
&  0.94516E+00_JPRB, 0.96287E+00_JPRB, 0.97959E+00_JPRB, 0.99871E+00_JPRB, 0.10232E+01_JPRB, 0.10604E+01_JPRB, 0.11088E+01_JPRB ,&
&  0.11666E+01_JPRB, 0.12661E+01_JPRB, 0.14008E+01_JPRB, 0.15549E+01_JPRB, 0.17207E+01_JPRB, 0.18548E+01_JPRB, 0.18992E+01_JPRB ,&
&  0.11210E+01_JPRB, 0.14246E+01_JPRB, 0.16899E+01_JPRB, 0.16051E+01_JPRB, 0.15013E+01_JPRB, 0.14497E+01_JPRB, 0.14379E+01_JPRB ,&
&  0.13774E+01_JPRB, 0.12876E+01_JPRB, 0.12611E+01_JPRB, 0.13183E+01_JPRB, 0.12327E+01_JPRB, 0.12252E+01_JPRB, 0.12771E+01_JPRB ,&
&  0.13530E+01_JPRB, 0.14381E+01_JPRB, 0.15737E+01_JPRB, 0.17552E+01_JPRB, 0.20115E+01_JPRB, 0.23927E+01_JPRB, 0.28267E+01_JPRB ,&
&  0.32997E+01_JPRB, 0.38435E+01_JPRB, 0.43377E+01_JPRB, 0.39427E+01_JPRB, 0.38911E+01_JPRB, 0.39987E+01_JPRB, 0.36048E+01_JPRB ,&
&  0.23469E+01_JPRB, 0.17475E+01_JPRB, 0.18035E+01_JPRB, 0.17926E+01_JPRB, 0.15145E+01_JPRB, 0.13745E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.11052E+01_JPRB, 0.95213E+00_JPRB, 0.79127E+00_JPRB, 0.74599E+00_JPRB, 0.74835E+00_JPRB, 0.76976E+00_JPRB, 0.79306E+00_JPRB ,&
&  0.80499E+00_JPRB, 0.80926E+00_JPRB, 0.82244E+00_JPRB, 0.83930E+00_JPRB, 0.87208E+00_JPRB, 0.88734E+00_JPRB, 0.89169E+00_JPRB ,&
&  0.91597E+00_JPRB, 0.92896E+00_JPRB, 0.94958E+00_JPRB, 0.97827E+00_JPRB, 0.10045E+01_JPRB, 0.10185E+01_JPRB, 0.10142E+01_JPRB ,&
&  0.99836E+00_JPRB, 0.10108E+01_JPRB, 0.10320E+01_JPRB, 0.10252E+01_JPRB, 0.98398E+00_JPRB, 0.88888E+00_JPRB, 0.80779E+00_JPRB ,&
&  0.72195E+00_JPRB, 0.70255E+00_JPRB, 0.66895E+00_JPRB, 0.64174E+00_JPRB, 0.62021E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.60145E+00_JPRB, 0.59613E+00_JPRB, 0.60062E+00_JPRB, 0.60596E+00_JPRB, 0.61878E+00_JPRB, 0.64678E+00_JPRB, 0.67967E+00_JPRB ,&
&  0.70678E+00_JPRB, 0.75347E+00_JPRB, 0.78588E+00_JPRB, 0.78736E+00_JPRB, 0.78223E+00_JPRB, 0.78919E+00_JPRB, 0.80536E+00_JPRB ,&
&  0.82830E+00_JPRB, 0.84730E+00_JPRB, 0.85967E+00_JPRB, 0.87080E+00_JPRB, 0.88677E+00_JPRB, 0.90380E+00_JPRB, 0.92665E+00_JPRB ,&
&  0.96262E+00_JPRB, 0.10251E+01_JPRB, 0.11221E+01_JPRB, 0.12711E+01_JPRB, 0.14733E+01_JPRB, 0.17017E+01_JPRB, 0.18399E+01_JPRB ,&
&  0.13515E+01_JPRB, 0.72648E+00_JPRB, 0.14535E+01_JPRB, 0.17241E+01_JPRB, 0.15773E+01_JPRB, 0.15328E+01_JPRB, 0.14667E+01_JPRB ,&
&  0.13897E+01_JPRB, 0.13733E+01_JPRB, 0.13175E+01_JPRB, 0.12996E+01_JPRB, 0.11744E+01_JPRB, 0.11376E+01_JPRB, 0.11361E+01_JPRB ,&
&  0.11646E+01_JPRB, 0.12131E+01_JPRB, 0.13072E+01_JPRB, 0.14466E+01_JPRB, 0.16458E+01_JPRB, 0.19617E+01_JPRB, 0.24396E+01_JPRB ,&
&  0.29566E+01_JPRB, 0.35755E+01_JPRB, 0.43861E+01_JPRB, 0.42998E+01_JPRB, 0.44993E+01_JPRB, 0.40994E+01_JPRB, 0.34118E+01_JPRB ,&
&  0.26375E+01_JPRB, 0.20928E+01_JPRB, 0.20922E+01_JPRB, 0.21034E+01_JPRB, 0.17664E+01_JPRB, 0.14823E+01_JPRB, 0.14660E+01_JPRB ,&
&  0.12631E+01_JPRB, 0.11704E+01_JPRB, 0.99527E+00_JPRB, 0.86787E+00_JPRB, 0.82400E+00_JPRB, 0.83452E+00_JPRB, 0.85053E+00_JPRB ,&
&  0.85086E+00_JPRB, 0.84675E+00_JPRB, 0.85115E+00_JPRB, 0.86170E+00_JPRB, 0.88062E+00_JPRB, 0.89329E+00_JPRB, 0.89766E+00_JPRB ,&
&  0.91991E+00_JPRB, 0.94464E+00_JPRB, 0.96882E+00_JPRB, 0.99342E+00_JPRB, 0.10122E+01_JPRB, 0.99816E+00_JPRB, 0.10026E+01_JPRB ,&
&  0.10097E+01_JPRB, 0.10123E+01_JPRB, 0.10102E+01_JPRB, 0.10200E+01_JPRB, 0.10061E+01_JPRB, 0.95245E+00_JPRB, 0.93100E+00_JPRB ,&
&  0.86367E+00_JPRB, 0.79086E+00_JPRB, 0.70328E+00_JPRB, 0.64932E+00_JPRB, 0.62830E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.69596E+00_JPRB, 0.66395E+00_JPRB, 0.62262E+00_JPRB, 0.61109E+00_JPRB, 0.60912E+00_JPRB, 0.60662E+00_JPRB, 0.62370E+00_JPRB ,&
&  0.63917E+00_JPRB, 0.66245E+00_JPRB, 0.66783E+00_JPRB, 0.69041E+00_JPRB, 0.70225E+00_JPRB, 0.70467E+00_JPRB, 0.71591E+00_JPRB ,&
&  0.72415E+00_JPRB, 0.73340E+00_JPRB, 0.74319E+00_JPRB, 0.75641E+00_JPRB, 0.77073E+00_JPRB, 0.77663E+00_JPRB, 0.78523E+00_JPRB ,&
&  0.80549E+00_JPRB, 0.83170E+00_JPRB, 0.87232E+00_JPRB, 0.96438E+00_JPRB, 0.11225E+01_JPRB, 0.13321E+01_JPRB, 0.15667E+01_JPRB ,&
&  0.17598E+01_JPRB, 0.12768E+01_JPRB, 0.92211E+00_JPRB, 0.15626E+01_JPRB, 0.17370E+01_JPRB, 0.17105E+01_JPRB, 0.14994E+01_JPRB ,&
&  0.15132E+01_JPRB, 0.14706E+01_JPRB, 0.14174E+01_JPRB, 0.13440E+01_JPRB, 0.11835E+01_JPRB, 0.11030E+01_JPRB, 0.10470E+01_JPRB ,&
&  0.10212E+01_JPRB, 0.10192E+01_JPRB, 0.10724E+01_JPRB, 0.11978E+01_JPRB, 0.13461E+01_JPRB, 0.15582E+01_JPRB, 0.19567E+01_JPRB ,&
&  0.24635E+01_JPRB, 0.30862E+01_JPRB, 0.40540E+01_JPRB, 0.45620E+01_JPRB, 0.52579E+01_JPRB, 0.44809E+01_JPRB, 0.36411E+01_JPRB ,&
&  0.28233E+01_JPRB, 0.26726E+01_JPRB, 0.27498E+01_JPRB, 0.24449E+01_JPRB, 0.19698E+01_JPRB, 0.17488E+01_JPRB, 0.17309E+01_JPRB ,&
&  0.14818E+01_JPRB, 0.13531E+01_JPRB, 0.12421E+01_JPRB, 0.10830E+01_JPRB, 0.97741E+00_JPRB, 0.93713E+00_JPRB, 0.92615E+00_JPRB ,&
&  0.91510E+00_JPRB, 0.89476E+00_JPRB, 0.88449E+00_JPRB, 0.88963E+00_JPRB, 0.89827E+00_JPRB, 0.90277E+00_JPRB, 0.91455E+00_JPRB ,&
&  0.92853E+00_JPRB, 0.94759E+00_JPRB, 0.96996E+00_JPRB, 0.98304E+00_JPRB, 0.97426E+00_JPRB, 0.95709E+00_JPRB, 0.97907E+00_JPRB ,&
&  0.98859E+00_JPRB, 0.10127E+01_JPRB, 0.10068E+01_JPRB, 0.10128E+01_JPRB, 0.10068E+01_JPRB, 0.10075E+01_JPRB, 0.10021E+01_JPRB ,&
&  0.99511E+00_JPRB, 0.96411E+00_JPRB, 0.87520E+00_JPRB, 0.76256E+00_JPRB, 0.73165E+00_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.78414E+00_JPRB, 0.75657E+00_JPRB, 0.72099E+00_JPRB, 0.68465E+00_JPRB, 0.66459E+00_JPRB, 0.64487E+00_JPRB, 0.62132E+00_JPRB ,&
&  0.60883E+00_JPRB, 0.60259E+00_JPRB, 0.59103E+00_JPRB, 0.60111E+00_JPRB, 0.62580E+00_JPRB, 0.63912E+00_JPRB, 0.64844E+00_JPRB ,&
&  0.65854E+00_JPRB, 0.66798E+00_JPRB, 0.66164E+00_JPRB, 0.66149E+00_JPRB, 0.67082E+00_JPRB, 0.67809E+00_JPRB, 0.67886E+00_JPRB ,&
&  0.68360E+00_JPRB, 0.69672E+00_JPRB, 0.71536E+00_JPRB, 0.75320E+00_JPRB, 0.83473E+00_JPRB, 0.96171E+00_JPRB, 0.11357E+01_JPRB ,&
&  0.13861E+01_JPRB, 0.13803E+01_JPRB, 0.78645E+00_JPRB, 0.13271E+01_JPRB, 0.18047E+01_JPRB, 0.18249E+01_JPRB, 0.17120E+01_JPRB ,&
&  0.16996E+01_JPRB, 0.16482E+01_JPRB, 0.16022E+01_JPRB, 0.14270E+01_JPRB, 0.12733E+01_JPRB, 0.11520E+01_JPRB, 0.10501E+01_JPRB ,&
&  0.96421E+00_JPRB, 0.91067E+00_JPRB, 0.90156E+00_JPRB, 0.96644E+00_JPRB, 0.10859E+01_JPRB, 0.12073E+01_JPRB, 0.14372E+01_JPRB ,&
&  0.18441E+01_JPRB, 0.23654E+01_JPRB, 0.32082E+01_JPRB, 0.40848E+01_JPRB, 0.54240E+01_JPRB, 0.54338E+01_JPRB, 0.43866E+01_JPRB ,&
&  0.34210E+01_JPRB, 0.33526E+01_JPRB, 0.33110E+01_JPRB, 0.28867E+01_JPRB, 0.22147E+01_JPRB, 0.20087E+01_JPRB, 0.19681E+01_JPRB ,&
&  0.17185E+01_JPRB, 0.15235E+01_JPRB, 0.14341E+01_JPRB, 0.13328E+01_JPRB, 0.11877E+01_JPRB, 0.10623E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.97896E+00_JPRB, 0.95687E+00_JPRB, 0.94209E+00_JPRB, 0.92866E+00_JPRB, 0.92173E+00_JPRB, 0.91972E+00_JPRB, 0.92367E+00_JPRB ,&
&  0.93544E+00_JPRB, 0.94925E+00_JPRB, 0.96547E+00_JPRB, 0.96349E+00_JPRB, 0.95967E+00_JPRB, 0.95771E+00_JPRB, 0.95593E+00_JPRB ,&
&  0.95370E+00_JPRB, 0.95947E+00_JPRB, 0.96115E+00_JPRB, 0.93531E+00_JPRB, 0.97858E+00_JPRB, 0.98244E+00_JPRB, 0.10064E+01_JPRB ,&
&  0.10360E+01_JPRB, 0.10363E+01_JPRB, 0.97407E+00_JPRB, 0.84091E+00_JPRB, 0.81441E+00_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.80996E+00_JPRB, 0.78818E+00_JPRB, 0.76564E+00_JPRB, 0.74673E+00_JPRB, 0.73090E+00_JPRB, 0.71421E+00_JPRB, 0.68840E+00_JPRB ,&
&  0.65648E+00_JPRB, 0.63608E+00_JPRB, 0.60473E+00_JPRB, 0.59449E+00_JPRB, 0.59792E+00_JPRB, 0.60189E+00_JPRB, 0.61239E+00_JPRB ,&
&  0.61927E+00_JPRB, 0.62603E+00_JPRB, 0.62914E+00_JPRB, 0.62943E+00_JPRB, 0.63067E+00_JPRB, 0.62989E+00_JPRB, 0.62844E+00_JPRB ,&
&  0.62311E+00_JPRB, 0.62447E+00_JPRB, 0.63302E+00_JPRB, 0.64696E+00_JPRB, 0.68224E+00_JPRB, 0.74658E+00_JPRB, 0.83821E+00_JPRB ,&
&  0.10508E+01_JPRB, 0.11586E+01_JPRB, 0.12589E+01_JPRB, 0.18210E+01_JPRB, 0.21503E+01_JPRB, 0.20618E+01_JPRB, 0.20378E+01_JPRB ,&
&  0.20717E+01_JPRB, 0.20398E+01_JPRB, 0.19144E+01_JPRB, 0.15422E+01_JPRB, 0.12857E+01_JPRB, 0.11516E+01_JPRB, 0.10509E+01_JPRB ,&
&  0.95780E+00_JPRB, 0.88789E+00_JPRB, 0.83624E+00_JPRB, 0.82352E+00_JPRB, 0.86793E+00_JPRB, 0.96335E+00_JPRB, 0.10799E+01_JPRB ,&
&  0.13113E+01_JPRB, 0.17072E+01_JPRB, 0.22842E+01_JPRB, 0.30278E+01_JPRB, 0.43654E+01_JPRB, 0.53435E+01_JPRB, 0.50925E+01_JPRB ,&
&  0.47507E+01_JPRB, 0.43179E+01_JPRB, 0.37913E+01_JPRB, 0.33571E+01_JPRB, 0.24986E+01_JPRB, 0.24121E+01_JPRB, 0.21234E+01_JPRB ,&
&  0.18441E+01_JPRB, 0.16425E+01_JPRB, 0.15292E+01_JPRB, 0.14611E+01_JPRB, 0.13611E+01_JPRB, 0.12198E+01_JPRB, 0.11024E+01_JPRB ,&
&  0.10360E+01_JPRB, 0.10092E+01_JPRB, 0.99740E+00_JPRB, 0.98157E+00_JPRB, 0.96785E+00_JPRB, 0.95994E+00_JPRB, 0.95611E+00_JPRB ,&
&  0.94947E+00_JPRB, 0.94742E+00_JPRB, 0.93130E+00_JPRB, 0.93863E+00_JPRB, 0.93298E+00_JPRB, 0.92262E+00_JPRB, 0.90901E+00_JPRB ,&
&  0.91490E+00_JPRB, 0.93454E+00_JPRB, 0.94208E+00_JPRB, 0.93799E+00_JPRB, 0.93405E+00_JPRB, 0.93242E+00_JPRB, 0.92178E+00_JPRB ,&
&  0.91882E+00_JPRB, 0.89241E+00_JPRB, 0.85247E+00_JPRB, 0.83677E+00_JPRB, 0.82476E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.81168E+00_JPRB, 0.79285E+00_JPRB, 0.77916E+00_JPRB, 0.76269E+00_JPRB, 0.74656E+00_JPRB, 0.73679E+00_JPRB, 0.72751E+00_JPRB ,&
&  0.71159E+00_JPRB, 0.67710E+00_JPRB, 0.64129E+00_JPRB, 0.62274E+00_JPRB, 0.61501E+00_JPRB, 0.60102E+00_JPRB, 0.60115E+00_JPRB ,&
&  0.60352E+00_JPRB, 0.61016E+00_JPRB, 0.60793E+00_JPRB, 0.60693E+00_JPRB, 0.60661E+00_JPRB, 0.60622E+00_JPRB, 0.60257E+00_JPRB ,&
&  0.59424E+00_JPRB, 0.58973E+00_JPRB, 0.59323E+00_JPRB, 0.59932E+00_JPRB, 0.61260E+00_JPRB, 0.64262E+00_JPRB, 0.68038E+00_JPRB ,&
&  0.81486E+00_JPRB, 0.96585E+00_JPRB, 0.10473E+01_JPRB, 0.18107E+01_JPRB, 0.21153E+01_JPRB, 0.22350E+01_JPRB, 0.21137E+01_JPRB ,&
&  0.21452E+01_JPRB, 0.23770E+01_JPRB, 0.20283E+01_JPRB, 0.15482E+01_JPRB, 0.12190E+01_JPRB, 0.10686E+01_JPRB, 0.97770E+00_JPRB ,&
&  0.90215E+00_JPRB, 0.83526E+00_JPRB, 0.79132E+00_JPRB, 0.76608E+00_JPRB, 0.77357E+00_JPRB, 0.80109E+00_JPRB, 0.85993E+00_JPRB ,&
&  0.97585E+00_JPRB, 0.12363E+01_JPRB, 0.16354E+01_JPRB, 0.20155E+01_JPRB, 0.29950E+01_JPRB, 0.41156E+01_JPRB, 0.47689E+01_JPRB ,&
&  0.50745E+01_JPRB, 0.50227E+01_JPRB, 0.41151E+01_JPRB, 0.35436E+01_JPRB, 0.28724E+01_JPRB, 0.25134E+01_JPRB, 0.21573E+01_JPRB ,&
&  0.19103E+01_JPRB, 0.17166E+01_JPRB, 0.15765E+01_JPRB, 0.14950E+01_JPRB, 0.14152E+01_JPRB, 0.13178E+01_JPRB, 0.11962E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.10395E+01_JPRB, 0.10164E+01_JPRB, 0.10015E+01_JPRB, 0.99591E+00_JPRB, 0.98767E+00_JPRB, 0.97668E+00_JPRB ,&
&  0.96470E+00_JPRB, 0.95197E+00_JPRB, 0.92353E+00_JPRB, 0.92229E+00_JPRB, 0.90407E+00_JPRB, 0.87150E+00_JPRB, 0.86495E+00_JPRB ,&
&  0.89842E+00_JPRB, 0.92425E+00_JPRB, 0.92644E+00_JPRB, 0.90508E+00_JPRB, 0.91701E+00_JPRB, 0.93268E+00_JPRB, 0.89445E+00_JPRB ,&
&  0.87975E+00_JPRB, 0.86519E+00_JPRB, 0.84312E+00_JPRB, 0.83040E+00_JPRB, 0.82341E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.80384E+00_JPRB, 0.77797E+00_JPRB, 0.75953E+00_JPRB, 0.74387E+00_JPRB, 0.72922E+00_JPRB, 0.71589E+00_JPRB, 0.71138E+00_JPRB ,&
&  0.70586E+00_JPRB, 0.69528E+00_JPRB, 0.67121E+00_JPRB, 0.64348E+00_JPRB, 0.62921E+00_JPRB, 0.62264E+00_JPRB, 0.60826E+00_JPRB ,&
&  0.60193E+00_JPRB, 0.60218E+00_JPRB, 0.60094E+00_JPRB, 0.59461E+00_JPRB, 0.58857E+00_JPRB, 0.58390E+00_JPRB, 0.57969E+00_JPRB ,&
&  0.56660E+00_JPRB, 0.55998E+00_JPRB, 0.56009E+00_JPRB, 0.56045E+00_JPRB, 0.56418E+00_JPRB, 0.58914E+00_JPRB, 0.62861E+00_JPRB ,&
&  0.70840E+00_JPRB, 0.10217E+01_JPRB, 0.10316E+01_JPRB, 0.14979E+01_JPRB, 0.16944E+01_JPRB, 0.19164E+01_JPRB, 0.20352E+01_JPRB ,&
&  0.21801E+01_JPRB, 0.23417E+01_JPRB, 0.19244E+01_JPRB, 0.14842E+01_JPRB, 0.11217E+01_JPRB, 0.92973E+00_JPRB, 0.84208E+00_JPRB ,&
&  0.79779E+00_JPRB, 0.75829E+00_JPRB, 0.72906E+00_JPRB, 0.71962E+00_JPRB, 0.72434E+00_JPRB, 0.73535E+00_JPRB, 0.75135E+00_JPRB ,&
&  0.81024E+00_JPRB, 0.96013E+00_JPRB, 0.13078E+01_JPRB, 0.17767E+01_JPRB, 0.20701E+01_JPRB, 0.28295E+01_JPRB, 0.39594E+01_JPRB ,&
&  0.47627E+01_JPRB, 0.52282E+01_JPRB, 0.43294E+01_JPRB, 0.35848E+01_JPRB, 0.30230E+01_JPRB, 0.25960E+01_JPRB, 0.22335E+01_JPRB ,&
&  0.19550E+01_JPRB, 0.17490E+01_JPRB, 0.16001E+01_JPRB, 0.14953E+01_JPRB, 0.14218E+01_JPRB, 0.13653E+01_JPRB, 0.12834E+01_JPRB ,&
&  0.11884E+01_JPRB, 0.11152E+01_JPRB, 0.10679E+01_JPRB, 0.10412E+01_JPRB, 0.10318E+01_JPRB, 0.10151E+01_JPRB, 0.98475E+00_JPRB ,&
&  0.95906E+00_JPRB, 0.94794E+00_JPRB, 0.92164E+00_JPRB, 0.91716E+00_JPRB, 0.90328E+00_JPRB, 0.86681E+00_JPRB, 0.85253E+00_JPRB ,&
&  0.88750E+00_JPRB, 0.90463E+00_JPRB, 0.90914E+00_JPRB, 0.91019E+00_JPRB, 0.91602E+00_JPRB, 0.95596E+00_JPRB, 0.90614E+00_JPRB ,&
&  0.86905E+00_JPRB, 0.85061E+00_JPRB, 0.83499E+00_JPRB, 0.82821E+00_JPRB, 0.81915E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.78843E+00_JPRB, 0.76131E+00_JPRB, 0.73903E+00_JPRB, 0.71695E+00_JPRB, 0.70024E+00_JPRB, 0.68260E+00_JPRB, 0.67083E+00_JPRB ,&
&  0.66513E+00_JPRB, 0.66435E+00_JPRB, 0.66103E+00_JPRB, 0.64951E+00_JPRB, 0.63060E+00_JPRB, 0.61581E+00_JPRB, 0.60553E+00_JPRB ,&
&  0.59533E+00_JPRB, 0.59261E+00_JPRB, 0.59119E+00_JPRB, 0.58244E+00_JPRB, 0.57281E+00_JPRB, 0.56396E+00_JPRB, 0.55434E+00_JPRB ,&
&  0.54523E+00_JPRB, 0.54013E+00_JPRB, 0.53643E+00_JPRB, 0.53304E+00_JPRB, 0.53107E+00_JPRB, 0.54394E+00_JPRB, 0.60703E+00_JPRB ,&
&  0.74041E+00_JPRB, 0.64384E+00_JPRB, 0.12331E+01_JPRB, 0.13800E+01_JPRB, 0.14333E+01_JPRB, 0.15288E+01_JPRB, 0.17731E+01_JPRB ,&
&  0.20304E+01_JPRB, 0.20022E+01_JPRB, 0.17395E+01_JPRB, 0.14010E+01_JPRB, 0.11032E+01_JPRB, 0.90692E+00_JPRB, 0.80503E+00_JPRB ,&
&  0.73458E+00_JPRB, 0.69109E+00_JPRB, 0.67168E+00_JPRB, 0.66079E+00_JPRB, 0.66369E+00_JPRB, 0.67708E+00_JPRB, 0.68928E+00_JPRB ,&
&  0.72186E+00_JPRB, 0.80900E+00_JPRB, 0.10474E+01_JPRB, 0.15115E+01_JPRB, 0.20041E+01_JPRB, 0.23787E+01_JPRB, 0.31044E+01_JPRB ,&
&  0.41692E+01_JPRB, 0.45029E+01_JPRB, 0.37349E+01_JPRB, 0.31581E+01_JPRB, 0.27870E+01_JPRB, 0.25167E+01_JPRB, 0.22481E+01_JPRB ,&
&  0.19886E+01_JPRB, 0.17383E+01_JPRB, 0.15571E+01_JPRB, 0.14309E+01_JPRB, 0.13591E+01_JPRB, 0.13316E+01_JPRB, 0.13070E+01_JPRB ,&
&  0.12730E+01_JPRB, 0.12227E+01_JPRB, 0.11717E+01_JPRB, 0.11243E+01_JPRB, 0.10877E+01_JPRB, 0.10524E+01_JPRB, 0.10018E+01_JPRB ,&
&  0.95886E+00_JPRB, 0.93689E+00_JPRB, 0.90937E+00_JPRB, 0.89011E+00_JPRB, 0.88419E+00_JPRB, 0.87372E+00_JPRB, 0.86948E+00_JPRB ,&
&  0.87887E+00_JPRB, 0.88190E+00_JPRB, 0.88242E+00_JPRB, 0.88998E+00_JPRB, 0.87781E+00_JPRB, 0.91909E+00_JPRB, 0.89668E+00_JPRB ,&
&  0.88402E+00_JPRB, 0.86021E+00_JPRB, 0.83287E+00_JPRB, 0.81786E+00_JPRB, 0.80723E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.77087E+00_JPRB, 0.74963E+00_JPRB, 0.71973E+00_JPRB, 0.69334E+00_JPRB, 0.67267E+00_JPRB, 0.65103E+00_JPRB, 0.63567E+00_JPRB ,&
&  0.62436E+00_JPRB, 0.62496E+00_JPRB, 0.62915E+00_JPRB, 0.63087E+00_JPRB, 0.62407E+00_JPRB, 0.60759E+00_JPRB, 0.59108E+00_JPRB ,&
&  0.57936E+00_JPRB, 0.57554E+00_JPRB, 0.57450E+00_JPRB, 0.56983E+00_JPRB, 0.56240E+00_JPRB, 0.55259E+00_JPRB, 0.54318E+00_JPRB ,&
&  0.53412E+00_JPRB, 0.52723E+00_JPRB, 0.52057E+00_JPRB, 0.51564E+00_JPRB, 0.50998E+00_JPRB, 0.51603E+00_JPRB, 0.57973E+00_JPRB ,&
&  0.76629E+00_JPRB, 0.96455E+00_JPRB, 0.11763E+01_JPRB, 0.11196E+01_JPRB, 0.11220E+01_JPRB, 0.11802E+01_JPRB, 0.13669E+01_JPRB ,&
&  0.14752E+01_JPRB, 0.15117E+01_JPRB, 0.14309E+01_JPRB, 0.12571E+01_JPRB, 0.10520E+01_JPRB, 0.90405E+00_JPRB, 0.79588E+00_JPRB ,&
&  0.70754E+00_JPRB, 0.65239E+00_JPRB, 0.62155E+00_JPRB, 0.61277E+00_JPRB, 0.61786E+00_JPRB, 0.63052E+00_JPRB, 0.63857E+00_JPRB ,&
&  0.65675E+00_JPRB, 0.70483E+00_JPRB, 0.88255E+00_JPRB, 0.13008E+01_JPRB, 0.20179E+01_JPRB, 0.21197E+01_JPRB, 0.29132E+01_JPRB ,&
&  0.37511E+01_JPRB, 0.33883E+01_JPRB, 0.30605E+01_JPRB, 0.27861E+01_JPRB, 0.25700E+01_JPRB, 0.23741E+01_JPRB, 0.20647E+01_JPRB ,&
&  0.18150E+01_JPRB, 0.15854E+01_JPRB, 0.14143E+01_JPRB, 0.13160E+01_JPRB, 0.12554E+01_JPRB, 0.12309E+01_JPRB, 0.12265E+01_JPRB ,&
&  0.12201E+01_JPRB, 0.12051E+01_JPRB, 0.11908E+01_JPRB, 0.11579E+01_JPRB, 0.11108E+01_JPRB, 0.10623E+01_JPRB, 0.10117E+01_JPRB ,&
&  0.96667E+00_JPRB, 0.92628E+00_JPRB, 0.88682E+00_JPRB, 0.85352E+00_JPRB, 0.84924E+00_JPRB, 0.85379E+00_JPRB, 0.85045E+00_JPRB ,&
&  0.83816E+00_JPRB, 0.83133E+00_JPRB, 0.84404E+00_JPRB, 0.84237E+00_JPRB, 0.83975E+00_JPRB, 0.88490E+00_JPRB, 0.87908E+00_JPRB ,&
&  0.87087E+00_JPRB, 0.84429E+00_JPRB, 0.81878E+00_JPRB, 0.79713E+00_JPRB, 0.78564E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.75768E+00_JPRB, 0.73085E+00_JPRB, 0.70047E+00_JPRB, 0.67232E+00_JPRB, 0.64687E+00_JPRB, 0.62725E+00_JPRB, 0.61496E+00_JPRB ,&
&  0.60440E+00_JPRB, 0.59810E+00_JPRB, 0.60021E+00_JPRB, 0.60553E+00_JPRB, 0.60480E+00_JPRB, 0.59495E+00_JPRB, 0.58040E+00_JPRB ,&
&  0.56559E+00_JPRB, 0.55438E+00_JPRB, 0.54650E+00_JPRB, 0.54339E+00_JPRB, 0.53852E+00_JPRB, 0.53310E+00_JPRB, 0.52412E+00_JPRB ,&
&  0.51696E+00_JPRB, 0.51085E+00_JPRB, 0.50203E+00_JPRB, 0.49290E+00_JPRB, 0.48634E+00_JPRB, 0.48832E+00_JPRB, 0.53903E+00_JPRB ,&
&  0.68737E+00_JPRB, 0.85613E+00_JPRB, 0.86250E+00_JPRB, 0.89188E+00_JPRB, 0.95357E+00_JPRB, 0.98059E+00_JPRB, 0.10573E+01_JPRB ,&
&  0.11002E+01_JPRB, 0.11224E+01_JPRB, 0.11290E+01_JPRB, 0.10691E+01_JPRB, 0.95587E+00_JPRB, 0.86721E+00_JPRB, 0.78228E+00_JPRB ,&
&  0.70560E+00_JPRB, 0.64572E+00_JPRB, 0.60768E+00_JPRB, 0.59246E+00_JPRB, 0.59468E+00_JPRB, 0.60318E+00_JPRB, 0.60680E+00_JPRB ,&
&  0.61326E+00_JPRB, 0.62419E+00_JPRB, 0.69102E+00_JPRB, 0.85692E+00_JPRB, 0.12807E+01_JPRB, 0.15812E+01_JPRB, 0.20449E+01_JPRB ,&
&  0.21525E+01_JPRB, 0.21273E+01_JPRB, 0.20721E+01_JPRB, 0.20100E+01_JPRB, 0.19274E+01_JPRB, 0.17947E+01_JPRB, 0.16761E+01_JPRB ,&
&  0.15296E+01_JPRB, 0.13775E+01_JPRB, 0.12578E+01_JPRB, 0.11768E+01_JPRB, 0.11251E+01_JPRB, 0.10940E+01_JPRB, 0.10822E+01_JPRB ,&
&  0.10646E+01_JPRB, 0.10594E+01_JPRB, 0.10436E+01_JPRB, 0.10303E+01_JPRB, 0.10208E+01_JPRB, 0.99191E+00_JPRB, 0.94612E+00_JPRB ,&
&  0.89773E+00_JPRB, 0.85628E+00_JPRB, 0.82474E+00_JPRB, 0.80662E+00_JPRB, 0.79698E+00_JPRB, 0.78896E+00_JPRB, 0.78126E+00_JPRB ,&
&  0.78516E+00_JPRB, 0.78691E+00_JPRB, 0.79006E+00_JPRB, 0.78579E+00_JPRB, 0.77643E+00_JPRB, 0.81017E+00_JPRB, 0.80706E+00_JPRB ,&
&  0.80463E+00_JPRB, 0.79994E+00_JPRB, 0.78653E+00_JPRB, 0.76707E+00_JPRB, 0.77200E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.75821E+00_JPRB, 0.73411E+00_JPRB, 0.70140E+00_JPRB, 0.66914E+00_JPRB, 0.63895E+00_JPRB, 0.61518E+00_JPRB, 0.60089E+00_JPRB ,&
&  0.59118E+00_JPRB, 0.58499E+00_JPRB, 0.58190E+00_JPRB, 0.58007E+00_JPRB, 0.57714E+00_JPRB, 0.57367E+00_JPRB, 0.56745E+00_JPRB ,&
&  0.55919E+00_JPRB, 0.54896E+00_JPRB, 0.53965E+00_JPRB, 0.53311E+00_JPRB, 0.52789E+00_JPRB, 0.52447E+00_JPRB, 0.51974E+00_JPRB ,&
&  0.51088E+00_JPRB, 0.50039E+00_JPRB, 0.48738E+00_JPRB, 0.47431E+00_JPRB, 0.46602E+00_JPRB, 0.46558E+00_JPRB, 0.49825E+00_JPRB ,&
&  0.60684E+00_JPRB, 0.64094E+00_JPRB, 0.68862E+00_JPRB, 0.73047E+00_JPRB, 0.78256E+00_JPRB, 0.82095E+00_JPRB, 0.84813E+00_JPRB ,&
&  0.86623E+00_JPRB, 0.88576E+00_JPRB, 0.91821E+00_JPRB, 0.91047E+00_JPRB, 0.84840E+00_JPRB, 0.79036E+00_JPRB, 0.73584E+00_JPRB ,&
&  0.68581E+00_JPRB, 0.64136E+00_JPRB, 0.60432E+00_JPRB, 0.58873E+00_JPRB, 0.58540E+00_JPRB, 0.58826E+00_JPRB, 0.58783E+00_JPRB ,&
&  0.58972E+00_JPRB, 0.58723E+00_JPRB, 0.59412E+00_JPRB, 0.62242E+00_JPRB, 0.69041E+00_JPRB, 0.82583E+00_JPRB, 0.10522E+01_JPRB ,&
&  0.11585E+01_JPRB, 0.11930E+01_JPRB, 0.12278E+01_JPRB, 0.12614E+01_JPRB, 0.12410E+01_JPRB, 0.12070E+01_JPRB, 0.11724E+01_JPRB ,&
&  0.11348E+01_JPRB, 0.10928E+01_JPRB, 0.10552E+01_JPRB, 0.10078E+01_JPRB, 0.96735E+00_JPRB, 0.93803E+00_JPRB, 0.91744E+00_JPRB ,&
&  0.90210E+00_JPRB, 0.89587E+00_JPRB, 0.88493E+00_JPRB, 0.86169E+00_JPRB, 0.84539E+00_JPRB, 0.82792E+00_JPRB, 0.80015E+00_JPRB ,&
&  0.76102E+00_JPRB, 0.73620E+00_JPRB, 0.72156E+00_JPRB, 0.71839E+00_JPRB, 0.72037E+00_JPRB, 0.72378E+00_JPRB, 0.72210E+00_JPRB ,&
&  0.72392E+00_JPRB, 0.73545E+00_JPRB, 0.72600E+00_JPRB, 0.71657E+00_JPRB, 0.74718E+00_JPRB, 0.74667E+00_JPRB, 0.74827E+00_JPRB ,&
&  0.74925E+00_JPRB, 0.74678E+00_JPRB, 0.73780E+00_JPRB, 0.71844E+00_JPRB, 0.75773E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.65295E+00_JPRB, 0.65483E+00_JPRB, 0.63907E+00_JPRB, 0.61984E+00_JPRB, 0.60184E+00_JPRB, 0.58808E+00_JPRB, 0.57682E+00_JPRB ,&
&  0.56894E+00_JPRB, 0.56309E+00_JPRB, 0.55950E+00_JPRB, 0.55732E+00_JPRB, 0.55393E+00_JPRB, 0.54848E+00_JPRB, 0.54243E+00_JPRB ,&
&  0.53713E+00_JPRB, 0.53061E+00_JPRB, 0.52062E+00_JPRB, 0.51355E+00_JPRB, 0.50939E+00_JPRB, 0.50390E+00_JPRB, 0.49806E+00_JPRB ,&
&  0.49086E+00_JPRB, 0.47726E+00_JPRB, 0.46369E+00_JPRB, 0.45454E+00_JPRB, 0.44738E+00_JPRB, 0.44466E+00_JPRB, 0.46519E+00_JPRB ,&
&  0.53533E+00_JPRB, 0.51934E+00_JPRB, 0.56373E+00_JPRB, 0.61153E+00_JPRB, 0.66687E+00_JPRB, 0.70021E+00_JPRB, 0.71892E+00_JPRB ,&
&  0.73075E+00_JPRB, 0.75129E+00_JPRB, 0.76782E+00_JPRB, 0.78952E+00_JPRB, 0.76167E+00_JPRB, 0.71735E+00_JPRB, 0.68037E+00_JPRB ,&
&  0.64629E+00_JPRB, 0.61603E+00_JPRB, 0.59103E+00_JPRB, 0.57422E+00_JPRB, 0.56883E+00_JPRB, 0.56676E+00_JPRB, 0.56138E+00_JPRB ,&
&  0.56098E+00_JPRB, 0.55736E+00_JPRB, 0.55578E+00_JPRB, 0.56237E+00_JPRB, 0.56068E+00_JPRB, 0.59391E+00_JPRB, 0.65002E+00_JPRB ,&
&  0.70247E+00_JPRB, 0.77333E+00_JPRB, 0.81231E+00_JPRB, 0.83922E+00_JPRB, 0.84023E+00_JPRB, 0.83884E+00_JPRB, 0.83563E+00_JPRB ,&
&  0.83212E+00_JPRB, 0.82628E+00_JPRB, 0.81642E+00_JPRB, 0.80009E+00_JPRB, 0.78034E+00_JPRB, 0.76271E+00_JPRB, 0.74736E+00_JPRB ,&
&  0.73470E+00_JPRB, 0.72477E+00_JPRB, 0.71831E+00_JPRB, 0.70158E+00_JPRB, 0.67506E+00_JPRB, 0.65225E+00_JPRB, 0.63466E+00_JPRB ,&
&  0.61951E+00_JPRB, 0.61322E+00_JPRB, 0.61282E+00_JPRB, 0.62025E+00_JPRB, 0.62776E+00_JPRB, 0.63561E+00_JPRB, 0.63986E+00_JPRB ,&
&  0.64333E+00_JPRB, 0.65351E+00_JPRB, 0.66984E+00_JPRB, 0.68877E+00_JPRB, 0.69630E+00_JPRB, 0.70022E+00_JPRB, 0.70145E+00_JPRB ,&
&  0.69994E+00_JPRB, 0.69685E+00_JPRB, 0.68521E+00_JPRB, 0.67030E+00_JPRB, 0.64206E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.59078E+00_JPRB, 0.59328E+00_JPRB, 0.58616E+00_JPRB, 0.57361E+00_JPRB, 0.56266E+00_JPRB, 0.55290E+00_JPRB, 0.54528E+00_JPRB ,&
&  0.53416E+00_JPRB, 0.52468E+00_JPRB, 0.52143E+00_JPRB, 0.52150E+00_JPRB, 0.52083E+00_JPRB, 0.51675E+00_JPRB, 0.51107E+00_JPRB ,&
&  0.50460E+00_JPRB, 0.49724E+00_JPRB, 0.48855E+00_JPRB, 0.47985E+00_JPRB, 0.47274E+00_JPRB, 0.46441E+00_JPRB, 0.45760E+00_JPRB ,&
&  0.45114E+00_JPRB, 0.44533E+00_JPRB, 0.43633E+00_JPRB, 0.42545E+00_JPRB, 0.42045E+00_JPRB, 0.41513E+00_JPRB, 0.43333E+00_JPRB ,&
&  0.49414E+00_JPRB, 0.43136E+00_JPRB, 0.46278E+00_JPRB, 0.51724E+00_JPRB, 0.57021E+00_JPRB, 0.62563E+00_JPRB, 0.64924E+00_JPRB ,&
&  0.66277E+00_JPRB, 0.67232E+00_JPRB, 0.67802E+00_JPRB, 0.68879E+00_JPRB, 0.68840E+00_JPRB, 0.66258E+00_JPRB, 0.62804E+00_JPRB ,&
&  0.60185E+00_JPRB, 0.57753E+00_JPRB, 0.56174E+00_JPRB, 0.54647E+00_JPRB, 0.53929E+00_JPRB, 0.53649E+00_JPRB, 0.52847E+00_JPRB ,&
&  0.52227E+00_JPRB, 0.52560E+00_JPRB, 0.53157E+00_JPRB, 0.54039E+00_JPRB, 0.54634E+00_JPRB, 0.55129E+00_JPRB, 0.54411E+00_JPRB ,&
&  0.53781E+00_JPRB, 0.56085E+00_JPRB, 0.60051E+00_JPRB, 0.62372E+00_JPRB, 0.63725E+00_JPRB, 0.63715E+00_JPRB, 0.63052E+00_JPRB ,&
&  0.62759E+00_JPRB, 0.62623E+00_JPRB, 0.62885E+00_JPRB, 0.62645E+00_JPRB, 0.62065E+00_JPRB, 0.60930E+00_JPRB, 0.59534E+00_JPRB ,&
&  0.58191E+00_JPRB, 0.57004E+00_JPRB, 0.56078E+00_JPRB, 0.55223E+00_JPRB, 0.54197E+00_JPRB, 0.52825E+00_JPRB, 0.52052E+00_JPRB ,&
&  0.51687E+00_JPRB, 0.51747E+00_JPRB, 0.52166E+00_JPRB, 0.52898E+00_JPRB, 0.53936E+00_JPRB, 0.54888E+00_JPRB, 0.56047E+00_JPRB ,&
&  0.57091E+00_JPRB, 0.58181E+00_JPRB, 0.59505E+00_JPRB, 0.59888E+00_JPRB, 0.62761E+00_JPRB, 0.63596E+00_JPRB, 0.64396E+00_JPRB ,&
&  0.64596E+00_JPRB, 0.64531E+00_JPRB, 0.64246E+00_JPRB, 0.56601E+00_JPRB, 0.58691E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.53902E+00_JPRB, 0.53851E+00_JPRB, 0.53302E+00_JPRB, 0.52614E+00_JPRB, 0.52033E+00_JPRB, 0.51416E+00_JPRB, 0.50905E+00_JPRB ,&
&  0.50139E+00_JPRB, 0.49180E+00_JPRB, 0.48500E+00_JPRB, 0.48237E+00_JPRB, 0.48126E+00_JPRB, 0.47839E+00_JPRB, 0.47292E+00_JPRB ,&
&  0.46587E+00_JPRB, 0.45820E+00_JPRB, 0.45088E+00_JPRB, 0.44204E+00_JPRB, 0.43389E+00_JPRB, 0.42631E+00_JPRB, 0.41993E+00_JPRB ,&
&  0.41350E+00_JPRB, 0.40658E+00_JPRB, 0.39974E+00_JPRB, 0.39487E+00_JPRB, 0.39106E+00_JPRB, 0.38952E+00_JPRB, 0.40431E+00_JPRB ,&
&  0.45094E+00_JPRB, 0.38851E+00_JPRB, 0.41176E+00_JPRB, 0.44521E+00_JPRB, 0.49492E+00_JPRB, 0.55661E+00_JPRB, 0.58900E+00_JPRB ,&
&  0.60443E+00_JPRB, 0.61272E+00_JPRB, 0.61321E+00_JPRB, 0.61309E+00_JPRB, 0.61683E+00_JPRB, 0.60979E+00_JPRB, 0.58991E+00_JPRB ,&
&  0.56837E+00_JPRB, 0.54768E+00_JPRB, 0.53222E+00_JPRB, 0.52070E+00_JPRB, 0.51075E+00_JPRB, 0.50325E+00_JPRB, 0.49288E+00_JPRB ,&
&  0.48124E+00_JPRB, 0.47986E+00_JPRB, 0.48257E+00_JPRB, 0.48370E+00_JPRB, 0.48442E+00_JPRB, 0.48120E+00_JPRB, 0.46656E+00_JPRB ,&
&  0.45388E+00_JPRB, 0.44714E+00_JPRB, 0.44806E+00_JPRB, 0.45669E+00_JPRB, 0.46656E+00_JPRB, 0.47256E+00_JPRB, 0.47148E+00_JPRB ,&
&  0.46748E+00_JPRB, 0.46097E+00_JPRB, 0.45850E+00_JPRB, 0.45864E+00_JPRB, 0.45884E+00_JPRB, 0.45953E+00_JPRB, 0.45771E+00_JPRB ,&
&  0.45399E+00_JPRB, 0.45249E+00_JPRB, 0.45411E+00_JPRB, 0.45609E+00_JPRB, 0.45630E+00_JPRB, 0.45453E+00_JPRB, 0.45211E+00_JPRB ,&
&  0.44857E+00_JPRB, 0.44625E+00_JPRB, 0.44765E+00_JPRB, 0.45363E+00_JPRB, 0.46209E+00_JPRB, 0.47147E+00_JPRB, 0.48181E+00_JPRB ,&
&  0.49317E+00_JPRB, 0.50492E+00_JPRB, 0.51769E+00_JPRB, 0.53909E+00_JPRB, 0.54981E+00_JPRB, 0.56206E+00_JPRB, 0.57388E+00_JPRB ,&
&  0.57560E+00_JPRB, 0.57254E+00_JPRB, 0.48939E+00_JPRB, 0.52371E+00_JPRB, 0.53253E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.50596E+00_JPRB, 0.49675E+00_JPRB, 0.48590E+00_JPRB, 0.47907E+00_JPRB, 0.47673E+00_JPRB, 0.47391E+00_JPRB, 0.47095E+00_JPRB ,&
&  0.46584E+00_JPRB, 0.45996E+00_JPRB, 0.45415E+00_JPRB, 0.44899E+00_JPRB, 0.44475E+00_JPRB, 0.44178E+00_JPRB, 0.43778E+00_JPRB ,&
&  0.43174E+00_JPRB, 0.42470E+00_JPRB, 0.41740E+00_JPRB, 0.40913E+00_JPRB, 0.39833E+00_JPRB, 0.38967E+00_JPRB, 0.38282E+00_JPRB ,&
&  0.37744E+00_JPRB, 0.37319E+00_JPRB, 0.37147E+00_JPRB, 0.36942E+00_JPRB, 0.36720E+00_JPRB, 0.36726E+00_JPRB, 0.37932E+00_JPRB ,&
&  0.38014E+00_JPRB, 0.35411E+00_JPRB, 0.37381E+00_JPRB, 0.40066E+00_JPRB, 0.44180E+00_JPRB, 0.50454E+00_JPRB, 0.54602E+00_JPRB ,&
&  0.56815E+00_JPRB, 0.57836E+00_JPRB, 0.58081E+00_JPRB, 0.57567E+00_JPRB, 0.56973E+00_JPRB, 0.56936E+00_JPRB, 0.56347E+00_JPRB ,&
&  0.54840E+00_JPRB, 0.52939E+00_JPRB, 0.51120E+00_JPRB, 0.49768E+00_JPRB, 0.48301E+00_JPRB, 0.47114E+00_JPRB, 0.45862E+00_JPRB ,&
&  0.44751E+00_JPRB, 0.44150E+00_JPRB, 0.44004E+00_JPRB, 0.43868E+00_JPRB, 0.43636E+00_JPRB, 0.42902E+00_JPRB, 0.41849E+00_JPRB ,&
&  0.41068E+00_JPRB, 0.40381E+00_JPRB, 0.39686E+00_JPRB, 0.39095E+00_JPRB, 0.38726E+00_JPRB, 0.38492E+00_JPRB, 0.38288E+00_JPRB ,&
&  0.38286E+00_JPRB, 0.38425E+00_JPRB, 0.38710E+00_JPRB, 0.39026E+00_JPRB, 0.39223E+00_JPRB, 0.39209E+00_JPRB, 0.39206E+00_JPRB ,&
&  0.39221E+00_JPRB, 0.38978E+00_JPRB, 0.38975E+00_JPRB, 0.39445E+00_JPRB, 0.40058E+00_JPRB, 0.40462E+00_JPRB, 0.40606E+00_JPRB ,&
&  0.40462E+00_JPRB, 0.40336E+00_JPRB, 0.40385E+00_JPRB, 0.40668E+00_JPRB, 0.41069E+00_JPRB, 0.41670E+00_JPRB, 0.42400E+00_JPRB ,&
&  0.43157E+00_JPRB, 0.44114E+00_JPRB, 0.45411E+00_JPRB, 0.47247E+00_JPRB, 0.48636E+00_JPRB, 0.49707E+00_JPRB, 0.50587E+00_JPRB ,&
&  0.50749E+00_JPRB, 0.50750E+00_JPRB, 0.51177E+00_JPRB, 0.51126E+00_JPRB, 0.51015E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.44989E+00_JPRB, 0.44434E+00_JPRB, 0.43760E+00_JPRB, 0.43205E+00_JPRB, 0.42949E+00_JPRB, 0.42761E+00_JPRB, 0.42399E+00_JPRB ,&
&  0.42080E+00_JPRB, 0.41794E+00_JPRB, 0.41489E+00_JPRB, 0.41231E+00_JPRB, 0.41008E+00_JPRB, 0.40834E+00_JPRB, 0.40501E+00_JPRB ,&
&  0.39934E+00_JPRB, 0.39208E+00_JPRB, 0.38390E+00_JPRB, 0.37699E+00_JPRB, 0.36851E+00_JPRB, 0.35976E+00_JPRB, 0.35324E+00_JPRB ,&
&  0.34787E+00_JPRB, 0.34422E+00_JPRB, 0.34287E+00_JPRB, 0.34103E+00_JPRB, 0.34181E+00_JPRB, 0.34493E+00_JPRB, 0.35399E+00_JPRB ,&
&  0.36838E+00_JPRB, 0.34961E+00_JPRB, 0.36830E+00_JPRB, 0.38538E+00_JPRB, 0.41949E+00_JPRB, 0.46829E+00_JPRB, 0.50136E+00_JPRB ,&
&  0.52226E+00_JPRB, 0.53889E+00_JPRB, 0.55063E+00_JPRB, 0.55253E+00_JPRB, 0.54438E+00_JPRB, 0.53951E+00_JPRB, 0.53624E+00_JPRB ,&
&  0.52686E+00_JPRB, 0.51231E+00_JPRB, 0.49261E+00_JPRB, 0.47190E+00_JPRB, 0.45316E+00_JPRB, 0.43921E+00_JPRB, 0.42828E+00_JPRB ,&
&  0.41967E+00_JPRB, 0.41260E+00_JPRB, 0.40728E+00_JPRB, 0.39885E+00_JPRB, 0.38913E+00_JPRB, 0.37935E+00_JPRB, 0.37003E+00_JPRB ,&
&  0.36084E+00_JPRB, 0.35186E+00_JPRB, 0.34400E+00_JPRB, 0.33749E+00_JPRB, 0.33383E+00_JPRB, 0.33199E+00_JPRB, 0.33120E+00_JPRB ,&
&  0.33147E+00_JPRB, 0.33335E+00_JPRB, 0.33610E+00_JPRB, 0.33932E+00_JPRB, 0.34222E+00_JPRB, 0.34315E+00_JPRB, 0.34134E+00_JPRB ,&
&  0.34076E+00_JPRB, 0.34049E+00_JPRB, 0.34119E+00_JPRB, 0.34384E+00_JPRB, 0.34578E+00_JPRB, 0.34809E+00_JPRB, 0.35176E+00_JPRB ,&
&  0.35481E+00_JPRB, 0.35632E+00_JPRB, 0.35743E+00_JPRB, 0.35882E+00_JPRB, 0.36067E+00_JPRB, 0.36388E+00_JPRB, 0.36882E+00_JPRB ,&
&  0.37472E+00_JPRB, 0.38139E+00_JPRB, 0.39117E+00_JPRB, 0.40411E+00_JPRB, 0.41640E+00_JPRB, 0.42677E+00_JPRB, 0.43533E+00_JPRB ,&
&  0.43885E+00_JPRB, 0.44170E+00_JPRB, 0.44697E+00_JPRB, 0.44990E+00_JPRB, 0.45216E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.40432E+00_JPRB, 0.40159E+00_JPRB, 0.39836E+00_JPRB, 0.39568E+00_JPRB, 0.39381E+00_JPRB, 0.39024E+00_JPRB, 0.38600E+00_JPRB ,&
&  0.38343E+00_JPRB, 0.38119E+00_JPRB, 0.37974E+00_JPRB, 0.37709E+00_JPRB, 0.37595E+00_JPRB, 0.37543E+00_JPRB, 0.37173E+00_JPRB ,&
&  0.36707E+00_JPRB, 0.36010E+00_JPRB, 0.35311E+00_JPRB, 0.34669E+00_JPRB, 0.34050E+00_JPRB, 0.33467E+00_JPRB, 0.32824E+00_JPRB ,&
&  0.32225E+00_JPRB, 0.31780E+00_JPRB, 0.31534E+00_JPRB, 0.31329E+00_JPRB, 0.31464E+00_JPRB, 0.32065E+00_JPRB, 0.33041E+00_JPRB ,&
&  0.33777E+00_JPRB, 0.34377E+00_JPRB, 0.35683E+00_JPRB, 0.37049E+00_JPRB, 0.39629E+00_JPRB, 0.43229E+00_JPRB, 0.45752E+00_JPRB ,&
&  0.47283E+00_JPRB, 0.48981E+00_JPRB, 0.50955E+00_JPRB, 0.52186E+00_JPRB, 0.52279E+00_JPRB, 0.51581E+00_JPRB, 0.51046E+00_JPRB ,&
&  0.50348E+00_JPRB, 0.49033E+00_JPRB, 0.47608E+00_JPRB, 0.45885E+00_JPRB, 0.44085E+00_JPRB, 0.42600E+00_JPRB, 0.41429E+00_JPRB ,&
&  0.40383E+00_JPRB, 0.39394E+00_JPRB, 0.38474E+00_JPRB, 0.37510E+00_JPRB, 0.36455E+00_JPRB, 0.35308E+00_JPRB, 0.34090E+00_JPRB ,&
&  0.32916E+00_JPRB, 0.31931E+00_JPRB, 0.31180E+00_JPRB, 0.30787E+00_JPRB, 0.30569E+00_JPRB, 0.30378E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30247E+00_JPRB, 0.30362E+00_JPRB, 0.30559E+00_JPRB, 0.30805E+00_JPRB, 0.31054E+00_JPRB, 0.31168E+00_JPRB, 0.30993E+00_JPRB ,&
&  0.30776E+00_JPRB, 0.30768E+00_JPRB, 0.30848E+00_JPRB, 0.30673E+00_JPRB, 0.30449E+00_JPRB, 0.30539E+00_JPRB, 0.30827E+00_JPRB ,&
&  0.31312E+00_JPRB, 0.31746E+00_JPRB, 0.32087E+00_JPRB, 0.32356E+00_JPRB, 0.32549E+00_JPRB, 0.32876E+00_JPRB, 0.33282E+00_JPRB ,&
&  0.33803E+00_JPRB, 0.34466E+00_JPRB, 0.35396E+00_JPRB, 0.36563E+00_JPRB, 0.37624E+00_JPRB, 0.38496E+00_JPRB, 0.39004E+00_JPRB ,&
&  0.39311E+00_JPRB, 0.39525E+00_JPRB, 0.39741E+00_JPRB, 0.40065E+00_JPRB, 0.40371E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.35965E+00_JPRB, 0.35965E+00_JPRB, 0.36031E+00_JPRB, 0.36041E+00_JPRB, 0.35745E+00_JPRB, 0.35444E+00_JPRB, 0.35103E+00_JPRB ,&
&  0.34956E+00_JPRB, 0.34861E+00_JPRB, 0.34849E+00_JPRB, 0.34589E+00_JPRB, 0.34408E+00_JPRB, 0.34327E+00_JPRB, 0.34005E+00_JPRB ,&
&  0.33663E+00_JPRB, 0.32876E+00_JPRB, 0.32028E+00_JPRB, 0.31400E+00_JPRB, 0.31003E+00_JPRB, 0.30720E+00_JPRB, 0.30165E+00_JPRB ,&
&  0.29754E+00_JPRB, 0.29429E+00_JPRB, 0.29269E+00_JPRB, 0.29274E+00_JPRB, 0.29494E+00_JPRB, 0.30100E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31837E+00_JPRB, 0.32941E+00_JPRB, 0.33063E+00_JPRB, 0.34640E+00_JPRB, 0.36292E+00_JPRB, 0.38666E+00_JPRB, 0.40767E+00_JPRB ,&
&  0.42119E+00_JPRB, 0.43119E+00_JPRB, 0.44540E+00_JPRB, 0.46067E+00_JPRB, 0.46782E+00_JPRB, 0.46984E+00_JPRB, 0.46788E+00_JPRB ,&
&  0.46358E+00_JPRB, 0.45740E+00_JPRB, 0.45014E+00_JPRB, 0.44097E+00_JPRB, 0.42843E+00_JPRB, 0.41422E+00_JPRB, 0.40027E+00_JPRB ,&
&  0.38763E+00_JPRB, 0.37526E+00_JPRB, 0.36301E+00_JPRB, 0.34968E+00_JPRB, 0.33621E+00_JPRB, 0.32360E+00_JPRB, 0.31111E+00_JPRB ,&
&  0.30082E+00_JPRB, 0.29299E+00_JPRB, 0.28727E+00_JPRB, 0.28315E+00_JPRB, 0.28003E+00_JPRB, 0.27700E+00_JPRB, 0.27552E+00_JPRB ,&
&  0.27548E+00_JPRB, 0.27700E+00_JPRB, 0.27953E+00_JPRB, 0.28167E+00_JPRB, 0.28431E+00_JPRB, 0.28485E+00_JPRB, 0.28191E+00_JPRB ,&
&  0.27885E+00_JPRB, 0.27804E+00_JPRB, 0.27441E+00_JPRB, 0.27235E+00_JPRB, 0.27209E+00_JPRB, 0.27276E+00_JPRB, 0.27491E+00_JPRB ,&
&  0.27902E+00_JPRB, 0.28458E+00_JPRB, 0.28977E+00_JPRB, 0.29437E+00_JPRB, 0.29846E+00_JPRB, 0.30248E+00_JPRB, 0.30706E+00_JPRB ,&
&  0.31189E+00_JPRB, 0.31654E+00_JPRB, 0.32371E+00_JPRB, 0.33179E+00_JPRB, 0.33870E+00_JPRB, 0.34613E+00_JPRB, 0.35059E+00_JPRB ,&
&  0.35227E+00_JPRB, 0.35281E+00_JPRB, 0.35470E+00_JPRB, 0.35767E+00_JPRB, 0.35951E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.32261E+00_JPRB, 0.32418E+00_JPRB, 0.32615E+00_JPRB, 0.32639E+00_JPRB, 0.32568E+00_JPRB, 0.32551E+00_JPRB, 0.32439E+00_JPRB ,&
&  0.32208E+00_JPRB, 0.32086E+00_JPRB, 0.32115E+00_JPRB, 0.31986E+00_JPRB, 0.31721E+00_JPRB, 0.31399E+00_JPRB, 0.31048E+00_JPRB ,&
&  0.30755E+00_JPRB, 0.30438E+00_JPRB, 0.29889E+00_JPRB, 0.29171E+00_JPRB, 0.28551E+00_JPRB, 0.28147E+00_JPRB, 0.27926E+00_JPRB ,&
&  0.27822E+00_JPRB, 0.27805E+00_JPRB, 0.27758E+00_JPRB, 0.27815E+00_JPRB, 0.28027E+00_JPRB, 0.28523E+00_JPRB, 0.29261E+00_JPRB ,&
&  0.29999E+00_JPRB, 0.30720E+00_JPRB, 0.31458E+00_JPRB, 0.32275E+00_JPRB, 0.33257E+00_JPRB, 0.34518E+00_JPRB, 0.35757E+00_JPRB ,&
&  0.36948E+00_JPRB, 0.37381E+00_JPRB, 0.37565E+00_JPRB, 0.38145E+00_JPRB, 0.39183E+00_JPRB, 0.39954E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40714E+00_JPRB, 0.40620E+00_JPRB, 0.40370E+00_JPRB, 0.39789E+00_JPRB, 0.38928E+00_JPRB, 0.37821E+00_JPRB, 0.36626E+00_JPRB ,&
&  0.35451E+00_JPRB, 0.34235E+00_JPRB, 0.32930E+00_JPRB, 0.31672E+00_JPRB, 0.30570E+00_JPRB, 0.29581E+00_JPRB, 0.28619E+00_JPRB ,&
&  0.27758E+00_JPRB, 0.27083E+00_JPRB, 0.26551E+00_JPRB, 0.26181E+00_JPRB, 0.25876E+00_JPRB, 0.25635E+00_JPRB, 0.25553E+00_JPRB ,&
&  0.25571E+00_JPRB, 0.25695E+00_JPRB, 0.25763E+00_JPRB, 0.25920E+00_JPRB, 0.26187E+00_JPRB, 0.26286E+00_JPRB, 0.26148E+00_JPRB ,&
&  0.25732E+00_JPRB, 0.25331E+00_JPRB, 0.24957E+00_JPRB, 0.24917E+00_JPRB, 0.24939E+00_JPRB, 0.25098E+00_JPRB, 0.25315E+00_JPRB ,&
&  0.25716E+00_JPRB, 0.26253E+00_JPRB, 0.26746E+00_JPRB, 0.27120E+00_JPRB, 0.27552E+00_JPRB, 0.27937E+00_JPRB, 0.28314E+00_JPRB ,&
&  0.28722E+00_JPRB, 0.29149E+00_JPRB, 0.29704E+00_JPRB, 0.30265E+00_JPRB, 0.30775E+00_JPRB, 0.31176E+00_JPRB, 0.31462E+00_JPRB ,&
&  0.31642E+00_JPRB, 0.31684E+00_JPRB, 0.31707E+00_JPRB, 0.31861E+00_JPRB, 0.32090E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.29018E+00_JPRB, 0.29233E+00_JPRB, 0.29513E+00_JPRB, 0.29734E+00_JPRB, 0.29807E+00_JPRB, 0.29748E+00_JPRB, 0.29663E+00_JPRB ,&
&  0.29636E+00_JPRB, 0.29611E+00_JPRB, 0.29458E+00_JPRB, 0.29206E+00_JPRB, 0.28954E+00_JPRB, 0.28692E+00_JPRB, 0.28515E+00_JPRB ,&
&  0.28343E+00_JPRB, 0.28137E+00_JPRB, 0.27753E+00_JPRB, 0.27176E+00_JPRB, 0.26577E+00_JPRB, 0.26232E+00_JPRB, 0.26075E+00_JPRB ,&
&  0.26071E+00_JPRB, 0.26128E+00_JPRB, 0.26378E+00_JPRB, 0.26550E+00_JPRB, 0.26790E+00_JPRB, 0.27198E+00_JPRB, 0.27768E+00_JPRB ,&
&  0.28305E+00_JPRB, 0.28823E+00_JPRB, 0.29363E+00_JPRB, 0.29837E+00_JPRB, 0.30308E+00_JPRB, 0.30911E+00_JPRB, 0.31596E+00_JPRB ,&
&  0.32427E+00_JPRB, 0.33010E+00_JPRB, 0.33073E+00_JPRB, 0.33156E+00_JPRB, 0.33484E+00_JPRB, 0.33905E+00_JPRB, 0.34208E+00_JPRB ,&
&  0.34438E+00_JPRB, 0.34333E+00_JPRB, 0.34099E+00_JPRB, 0.33705E+00_JPRB, 0.33247E+00_JPRB, 0.32712E+00_JPRB, 0.31991E+00_JPRB ,&
&  0.31124E+00_JPRB, 0.30180E+00_JPRB, 0.29272E+00_JPRB, 0.28403E+00_JPRB, 0.27567E+00_JPRB, 0.26799E+00_JPRB, 0.26113E+00_JPRB ,&
&  0.25474E+00_JPRB, 0.24982E+00_JPRB, 0.24506E+00_JPRB, 0.24207E+00_JPRB, 0.24014E+00_JPRB, 0.23793E+00_JPRB, 0.23734E+00_JPRB ,&
&  0.23625E+00_JPRB, 0.23563E+00_JPRB, 0.23551E+00_JPRB, 0.23708E+00_JPRB, 0.24020E+00_JPRB, 0.24208E+00_JPRB, 0.24081E+00_JPRB ,&
&  0.23747E+00_JPRB, 0.23546E+00_JPRB, 0.23436E+00_JPRB, 0.23357E+00_JPRB, 0.23134E+00_JPRB, 0.23123E+00_JPRB, 0.23342E+00_JPRB ,&
&  0.23814E+00_JPRB, 0.24310E+00_JPRB, 0.24714E+00_JPRB, 0.24966E+00_JPRB, 0.25243E+00_JPRB, 0.25487E+00_JPRB, 0.25702E+00_JPRB ,&
&  0.25928E+00_JPRB, 0.26092E+00_JPRB, 0.26211E+00_JPRB, 0.26367E+00_JPRB, 0.26614E+00_JPRB, 0.26852E+00_JPRB, 0.27085E+00_JPRB ,&
&  0.27342E+00_JPRB, 0.27748E+00_JPRB, 0.28159E+00_JPRB, 0.28462E+00_JPRB, 0.28809E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.26750E+00_JPRB, 0.27018E+00_JPRB, 0.27263E+00_JPRB, 0.27260E+00_JPRB, 0.27060E+00_JPRB, 0.26955E+00_JPRB, 0.26963E+00_JPRB ,&
&  0.26992E+00_JPRB, 0.26977E+00_JPRB, 0.26833E+00_JPRB, 0.26814E+00_JPRB, 0.26815E+00_JPRB, 0.26730E+00_JPRB, 0.26683E+00_JPRB ,&
&  0.26577E+00_JPRB, 0.26285E+00_JPRB, 0.25851E+00_JPRB, 0.25316E+00_JPRB, 0.24923E+00_JPRB, 0.24729E+00_JPRB, 0.24718E+00_JPRB ,&
&  0.24811E+00_JPRB, 0.24880E+00_JPRB, 0.25136E+00_JPRB, 0.25345E+00_JPRB, 0.25599E+00_JPRB, 0.25948E+00_JPRB, 0.26321E+00_JPRB ,&
&  0.26591E+00_JPRB, 0.26881E+00_JPRB, 0.27181E+00_JPRB, 0.27470E+00_JPRB, 0.27849E+00_JPRB, 0.28420E+00_JPRB, 0.28914E+00_JPRB ,&
&  0.29509E+00_JPRB, 0.29863E+00_JPRB, 0.29594E+00_JPRB, 0.29259E+00_JPRB, 0.29244E+00_JPRB, 0.29442E+00_JPRB, 0.29670E+00_JPRB ,&
&  0.29882E+00_JPRB, 0.30098E+00_JPRB, 0.30272E+00_JPRB, 0.30259E+00_JPRB, 0.29993E+00_JPRB, 0.29497E+00_JPRB, 0.28821E+00_JPRB ,&
&  0.28073E+00_JPRB, 0.27312E+00_JPRB, 0.26579E+00_JPRB, 0.25844E+00_JPRB, 0.25215E+00_JPRB, 0.24616E+00_JPRB, 0.24070E+00_JPRB ,&
&  0.23562E+00_JPRB, 0.23077E+00_JPRB, 0.22704E+00_JPRB, 0.22461E+00_JPRB, 0.22360E+00_JPRB, 0.22210E+00_JPRB, 0.22182E+00_JPRB ,&
&  0.22109E+00_JPRB, 0.22080E+00_JPRB, 0.22091E+00_JPRB, 0.22173E+00_JPRB, 0.22312E+00_JPRB, 0.22376E+00_JPRB, 0.22299E+00_JPRB ,&
&  0.22257E+00_JPRB, 0.22391E+00_JPRB, 0.22374E+00_JPRB, 0.22131E+00_JPRB, 0.21828E+00_JPRB, 0.21761E+00_JPRB, 0.21914E+00_JPRB ,&
&  0.22205E+00_JPRB, 0.22518E+00_JPRB, 0.22795E+00_JPRB, 0.23035E+00_JPRB, 0.23236E+00_JPRB, 0.23483E+00_JPRB, 0.23704E+00_JPRB ,&
&  0.23895E+00_JPRB, 0.24003E+00_JPRB, 0.24176E+00_JPRB, 0.24361E+00_JPRB, 0.24550E+00_JPRB, 0.24681E+00_JPRB, 0.24667E+00_JPRB ,&
&  0.24615E+00_JPRB, 0.24775E+00_JPRB, 0.25270E+00_JPRB, 0.26001E+00_JPRB, 0.26457E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.24818E+00_JPRB, 0.25045E+00_JPRB, 0.25161E+00_JPRB, 0.25125E+00_JPRB, 0.25045E+00_JPRB, 0.25101E+00_JPRB, 0.25148E+00_JPRB ,&
&  0.25211E+00_JPRB, 0.25212E+00_JPRB, 0.25068E+00_JPRB, 0.24938E+00_JPRB, 0.25091E+00_JPRB, 0.25053E+00_JPRB, 0.25014E+00_JPRB ,&
&  0.24871E+00_JPRB, 0.24690E+00_JPRB, 0.24409E+00_JPRB, 0.24087E+00_JPRB, 0.23698E+00_JPRB, 0.23478E+00_JPRB, 0.23538E+00_JPRB ,&
&  0.23717E+00_JPRB, 0.23786E+00_JPRB, 0.23866E+00_JPRB, 0.23959E+00_JPRB, 0.24162E+00_JPRB, 0.24426E+00_JPRB, 0.24620E+00_JPRB ,&
&  0.24770E+00_JPRB, 0.25071E+00_JPRB, 0.25261E+00_JPRB, 0.25185E+00_JPRB, 0.25609E+00_JPRB, 0.26038E+00_JPRB, 0.26437E+00_JPRB ,&
&  0.26818E+00_JPRB, 0.27054E+00_JPRB, 0.27069E+00_JPRB, 0.26959E+00_JPRB, 0.26869E+00_JPRB, 0.26965E+00_JPRB, 0.27024E+00_JPRB ,&
&  0.27107E+00_JPRB, 0.27392E+00_JPRB, 0.27719E+00_JPRB, 0.27849E+00_JPRB, 0.27704E+00_JPRB, 0.27268E+00_JPRB, 0.26568E+00_JPRB ,&
&  0.25654E+00_JPRB, 0.24687E+00_JPRB, 0.23893E+00_JPRB, 0.23261E+00_JPRB, 0.22778E+00_JPRB, 0.22394E+00_JPRB, 0.22081E+00_JPRB ,&
&  0.21731E+00_JPRB, 0.21477E+00_JPRB, 0.21457E+00_JPRB, 0.21503E+00_JPRB, 0.21487E+00_JPRB, 0.21287E+00_JPRB, 0.21147E+00_JPRB ,&
&  0.21025E+00_JPRB, 0.21067E+00_JPRB, 0.21168E+00_JPRB, 0.21207E+00_JPRB, 0.21225E+00_JPRB, 0.21210E+00_JPRB, 0.21198E+00_JPRB ,&
&  0.21296E+00_JPRB, 0.21299E+00_JPRB, 0.21186E+00_JPRB, 0.20992E+00_JPRB, 0.20827E+00_JPRB, 0.20805E+00_JPRB, 0.20853E+00_JPRB ,&
&  0.20941E+00_JPRB, 0.21106E+00_JPRB, 0.21322E+00_JPRB, 0.21543E+00_JPRB, 0.21715E+00_JPRB, 0.21796E+00_JPRB, 0.21834E+00_JPRB ,&
&  0.21863E+00_JPRB, 0.21868E+00_JPRB, 0.22000E+00_JPRB, 0.22359E+00_JPRB, 0.22714E+00_JPRB, 0.22985E+00_JPRB, 0.23128E+00_JPRB ,&
&  0.23077E+00_JPRB, 0.23137E+00_JPRB, 0.23448E+00_JPRB, 0.24030E+00_JPRB, 0.24467E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.23357E+00_JPRB, 0.23572E+00_JPRB, 0.23628E+00_JPRB, 0.23572E+00_JPRB, 0.23504E+00_JPRB, 0.23585E+00_JPRB, 0.23709E+00_JPRB ,&
&  0.23773E+00_JPRB, 0.23777E+00_JPRB, 0.23671E+00_JPRB, 0.23465E+00_JPRB, 0.23304E+00_JPRB, 0.23313E+00_JPRB, 0.23159E+00_JPRB ,&
&  0.23039E+00_JPRB, 0.23105E+00_JPRB, 0.23024E+00_JPRB, 0.22913E+00_JPRB, 0.22712E+00_JPRB, 0.22608E+00_JPRB, 0.22694E+00_JPRB ,&
&  0.22920E+00_JPRB, 0.22978E+00_JPRB, 0.23038E+00_JPRB, 0.23116E+00_JPRB, 0.23270E+00_JPRB, 0.23462E+00_JPRB, 0.23577E+00_JPRB ,&
&  0.23646E+00_JPRB, 0.23529E+00_JPRB, 0.23380E+00_JPRB, 0.23631E+00_JPRB, 0.23974E+00_JPRB, 0.24426E+00_JPRB, 0.24733E+00_JPRB ,&
&  0.25001E+00_JPRB, 0.25202E+00_JPRB, 0.25295E+00_JPRB, 0.25285E+00_JPRB, 0.25313E+00_JPRB, 0.25497E+00_JPRB, 0.25640E+00_JPRB ,&
&  0.25698E+00_JPRB, 0.25682E+00_JPRB, 0.25548E+00_JPRB, 0.25419E+00_JPRB, 0.25082E+00_JPRB, 0.24561E+00_JPRB, 0.23768E+00_JPRB ,&
&  0.22934E+00_JPRB, 0.22256E+00_JPRB, 0.21747E+00_JPRB, 0.21341E+00_JPRB, 0.21063E+00_JPRB, 0.21043E+00_JPRB, 0.21041E+00_JPRB ,&
&  0.20991E+00_JPRB, 0.20940E+00_JPRB, 0.21068E+00_JPRB, 0.21120E+00_JPRB, 0.20656E+00_JPRB, 0.20162E+00_JPRB, 0.19694E+00_JPRB ,&
&  0.19579E+00_JPRB, 0.19648E+00_JPRB, 0.19746E+00_JPRB, 0.19986E+00_JPRB, 0.20362E+00_JPRB, 0.20446E+00_JPRB, 0.20331E+00_JPRB ,&
&  0.19753E+00_JPRB, 0.19248E+00_JPRB, 0.18942E+00_JPRB, 0.18689E+00_JPRB, 0.18544E+00_JPRB, 0.18569E+00_JPRB, 0.18654E+00_JPRB ,&
&  0.18889E+00_JPRB, 0.19065E+00_JPRB, 0.19327E+00_JPRB, 0.19473E+00_JPRB, 0.19463E+00_JPRB, 0.19455E+00_JPRB, 0.19389E+00_JPRB ,&
&  0.19553E+00_JPRB, 0.20029E+00_JPRB, 0.20727E+00_JPRB, 0.21003E+00_JPRB, 0.21450E+00_JPRB, 0.21759E+00_JPRB, 0.21901E+00_JPRB ,&
&  0.21987E+00_JPRB, 0.22023E+00_JPRB, 0.22038E+00_JPRB, 0.22440E+00_JPRB, 0.23004E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.22138E+00_JPRB, 0.22302E+00_JPRB, 0.22351E+00_JPRB, 0.22339E+00_JPRB, 0.22306E+00_JPRB, 0.22338E+00_JPRB, 0.22381E+00_JPRB ,&
&  0.22370E+00_JPRB, 0.22290E+00_JPRB, 0.22163E+00_JPRB, 0.21944E+00_JPRB, 0.21673E+00_JPRB, 0.21511E+00_JPRB, 0.21610E+00_JPRB ,&
&  0.21754E+00_JPRB, 0.21838E+00_JPRB, 0.21935E+00_JPRB, 0.21979E+00_JPRB, 0.21972E+00_JPRB, 0.22022E+00_JPRB, 0.22157E+00_JPRB ,&
&  0.22288E+00_JPRB, 0.22356E+00_JPRB, 0.22480E+00_JPRB, 0.22577E+00_JPRB, 0.22774E+00_JPRB, 0.22952E+00_JPRB, 0.22998E+00_JPRB ,&
&  0.22679E+00_JPRB, 0.21893E+00_JPRB, 0.21381E+00_JPRB, 0.21639E+00_JPRB, 0.22401E+00_JPRB, 0.22943E+00_JPRB, 0.23291E+00_JPRB ,&
&  0.23525E+00_JPRB, 0.23636E+00_JPRB, 0.23656E+00_JPRB, 0.23786E+00_JPRB, 0.23990E+00_JPRB, 0.24192E+00_JPRB, 0.24156E+00_JPRB ,&
&  0.23958E+00_JPRB, 0.23790E+00_JPRB, 0.23545E+00_JPRB, 0.23109E+00_JPRB, 0.22508E+00_JPRB, 0.21915E+00_JPRB, 0.21400E+00_JPRB ,&
&  0.20934E+00_JPRB, 0.20487E+00_JPRB, 0.20121E+00_JPRB, 0.19888E+00_JPRB, 0.19825E+00_JPRB, 0.19972E+00_JPRB, 0.19937E+00_JPRB ,&
&  0.19965E+00_JPRB, 0.19780E+00_JPRB, 0.19558E+00_JPRB, 0.19287E+00_JPRB, 0.19057E+00_JPRB, 0.18627E+00_JPRB, 0.18085E+00_JPRB ,&
&  0.17813E+00_JPRB, 0.18081E+00_JPRB, 0.18792E+00_JPRB, 0.19336E+00_JPRB, 0.19176E+00_JPRB, 0.18517E+00_JPRB, 0.18160E+00_JPRB ,&
&  0.17758E+00_JPRB, 0.17333E+00_JPRB, 0.17056E+00_JPRB, 0.16992E+00_JPRB, 0.17014E+00_JPRB, 0.17042E+00_JPRB, 0.17154E+00_JPRB ,&
&  0.17330E+00_JPRB, 0.17481E+00_JPRB, 0.17566E+00_JPRB, 0.17668E+00_JPRB, 0.17713E+00_JPRB, 0.17724E+00_JPRB, 0.17738E+00_JPRB ,&
&  0.17918E+00_JPRB, 0.18286E+00_JPRB, 0.18766E+00_JPRB, 0.19296E+00_JPRB, 0.19697E+00_JPRB, 0.20066E+00_JPRB, 0.20378E+00_JPRB ,&
&  0.20681E+00_JPRB, 0.20810E+00_JPRB, 0.21112E+00_JPRB, 0.21532E+00_JPRB, 0.21868E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.20760E+00_JPRB, 0.21140E+00_JPRB, 0.21283E+00_JPRB, 0.21279E+00_JPRB, 0.21164E+00_JPRB, 0.21060E+00_JPRB, 0.21029E+00_JPRB ,&
&  0.21004E+00_JPRB, 0.20889E+00_JPRB, 0.20802E+00_JPRB, 0.20740E+00_JPRB, 0.20672E+00_JPRB, 0.20587E+00_JPRB, 0.20614E+00_JPRB ,&
&  0.20748E+00_JPRB, 0.20953E+00_JPRB, 0.21120E+00_JPRB, 0.21219E+00_JPRB, 0.21199E+00_JPRB, 0.21250E+00_JPRB, 0.21346E+00_JPRB ,&
&  0.21348E+00_JPRB, 0.21350E+00_JPRB, 0.21663E+00_JPRB, 0.21674E+00_JPRB, 0.21756E+00_JPRB, 0.21745E+00_JPRB, 0.21743E+00_JPRB ,&
&  0.21229E+00_JPRB, 0.20687E+00_JPRB, 0.20437E+00_JPRB, 0.20668E+00_JPRB, 0.21454E+00_JPRB, 0.21591E+00_JPRB, 0.21960E+00_JPRB ,&
&  0.22181E+00_JPRB, 0.22276E+00_JPRB, 0.22427E+00_JPRB, 0.22555E+00_JPRB, 0.22707E+00_JPRB, 0.22636E+00_JPRB, 0.22418E+00_JPRB ,&
&  0.22123E+00_JPRB, 0.22022E+00_JPRB, 0.21688E+00_JPRB, 0.20239E+00_JPRB, 0.19369E+00_JPRB, 0.18873E+00_JPRB, 0.18348E+00_JPRB ,&
&  0.17866E+00_JPRB, 0.17617E+00_JPRB, 0.17566E+00_JPRB, 0.17422E+00_JPRB, 0.17513E+00_JPRB, 0.17746E+00_JPRB, 0.17847E+00_JPRB ,&
&  0.17821E+00_JPRB, 0.17871E+00_JPRB, 0.17842E+00_JPRB, 0.17741E+00_JPRB, 0.17551E+00_JPRB, 0.17434E+00_JPRB, 0.17371E+00_JPRB ,&
&  0.17430E+00_JPRB, 0.17397E+00_JPRB, 0.17808E+00_JPRB, 0.17920E+00_JPRB, 0.17368E+00_JPRB, 0.17264E+00_JPRB, 0.17103E+00_JPRB ,&
&  0.16555E+00_JPRB, 0.16250E+00_JPRB, 0.16077E+00_JPRB, 0.16011E+00_JPRB, 0.16118E+00_JPRB, 0.16271E+00_JPRB, 0.16348E+00_JPRB ,&
&  0.16375E+00_JPRB, 0.16491E+00_JPRB, 0.16546E+00_JPRB, 0.16605E+00_JPRB, 0.16645E+00_JPRB, 0.16736E+00_JPRB, 0.16869E+00_JPRB ,&
&  0.17034E+00_JPRB, 0.17336E+00_JPRB, 0.17629E+00_JPRB, 0.17957E+00_JPRB, 0.18148E+00_JPRB, 0.18295E+00_JPRB, 0.18645E+00_JPRB ,&
&  0.18833E+00_JPRB, 0.19563E+00_JPRB, 0.20617E+00_JPRB, 0.20849E+00_JPRB, 0.20290E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.20007E+00_JPRB, 0.20188E+00_JPRB, 0.20333E+00_JPRB, 0.20235E+00_JPRB, 0.20252E+00_JPRB, 0.20241E+00_JPRB, 0.20244E+00_JPRB ,&
&  0.20232E+00_JPRB, 0.20167E+00_JPRB, 0.20196E+00_JPRB, 0.19999E+00_JPRB, 0.19890E+00_JPRB, 0.19564E+00_JPRB, 0.19481E+00_JPRB ,&
&  0.19312E+00_JPRB, 0.19300E+00_JPRB, 0.19340E+00_JPRB, 0.19482E+00_JPRB, 0.19527E+00_JPRB, 0.19670E+00_JPRB, 0.19588E+00_JPRB ,&
&  0.19622E+00_JPRB, 0.19638E+00_JPRB, 0.19504E+00_JPRB, 0.19460E+00_JPRB, 0.19491E+00_JPRB, 0.19518E+00_JPRB, 0.19545E+00_JPRB ,&
&  0.19754E+00_JPRB, 0.19985E+00_JPRB, 0.20204E+00_JPRB, 0.20324E+00_JPRB, 0.20798E+00_JPRB, 0.20931E+00_JPRB, 0.21150E+00_JPRB ,&
&  0.21260E+00_JPRB, 0.21392E+00_JPRB, 0.21498E+00_JPRB, 0.21429E+00_JPRB, 0.21302E+00_JPRB, 0.20904E+00_JPRB, 0.20761E+00_JPRB ,&
&  0.19760E+00_JPRB, 0.19069E+00_JPRB, 0.18385E+00_JPRB, 0.17766E+00_JPRB, 0.17300E+00_JPRB, 0.16979E+00_JPRB, 0.16722E+00_JPRB ,&
&  0.16512E+00_JPRB, 0.16305E+00_JPRB, 0.16077E+00_JPRB, 0.16091E+00_JPRB, 0.16176E+00_JPRB, 0.16309E+00_JPRB, 0.16445E+00_JPRB ,&
&  0.16601E+00_JPRB, 0.16691E+00_JPRB, 0.16732E+00_JPRB, 0.16815E+00_JPRB, 0.16834E+00_JPRB, 0.16872E+00_JPRB, 0.16900E+00_JPRB ,&
&  0.17043E+00_JPRB, 0.17091E+00_JPRB, 0.17094E+00_JPRB, 0.17255E+00_JPRB, 0.17065E+00_JPRB, 0.16774E+00_JPRB, 0.16468E+00_JPRB ,&
&  0.16151E+00_JPRB, 0.15954E+00_JPRB, 0.15786E+00_JPRB, 0.15722E+00_JPRB, 0.15766E+00_JPRB, 0.15847E+00_JPRB, 0.15951E+00_JPRB ,&
&  0.15973E+00_JPRB, 0.16040E+00_JPRB, 0.16089E+00_JPRB, 0.16180E+00_JPRB, 0.16277E+00_JPRB, 0.16390E+00_JPRB, 0.16514E+00_JPRB ,&
&  0.16703E+00_JPRB, 0.16922E+00_JPRB, 0.17139E+00_JPRB, 0.17361E+00_JPRB, 0.17717E+00_JPRB, 0.18136E+00_JPRB, 0.18797E+00_JPRB ,&
&  0.19350E+00_JPRB, 0.20002E+00_JPRB, 0.19790E+00_JPRB, 0.20119E+00_JPRB, 0.20154E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.19509E+00_JPRB, 0.19584E+00_JPRB, 0.19381E+00_JPRB, 0.19410E+00_JPRB, 0.19482E+00_JPRB, 0.19519E+00_JPRB, 0.19566E+00_JPRB ,&
&  0.19314E+00_JPRB, 0.19239E+00_JPRB, 0.19012E+00_JPRB, 0.18822E+00_JPRB, 0.18582E+00_JPRB, 0.18337E+00_JPRB, 0.18152E+00_JPRB ,&
&  0.17963E+00_JPRB, 0.17902E+00_JPRB, 0.17813E+00_JPRB, 0.17907E+00_JPRB, 0.17910E+00_JPRB, 0.18087E+00_JPRB, 0.18229E+00_JPRB ,&
&  0.18304E+00_JPRB, 0.18413E+00_JPRB, 0.18265E+00_JPRB, 0.18221E+00_JPRB, 0.18389E+00_JPRB, 0.18534E+00_JPRB, 0.18794E+00_JPRB ,&
&  0.19063E+00_JPRB, 0.19382E+00_JPRB, 0.19906E+00_JPRB, 0.20090E+00_JPRB, 0.20325E+00_JPRB, 0.20469E+00_JPRB, 0.20461E+00_JPRB ,&
&  0.20433E+00_JPRB, 0.20411E+00_JPRB, 0.20490E+00_JPRB, 0.20455E+00_JPRB, 0.19752E+00_JPRB, 0.19221E+00_JPRB, 0.18657E+00_JPRB ,&
&  0.18141E+00_JPRB, 0.17644E+00_JPRB, 0.17197E+00_JPRB, 0.16897E+00_JPRB, 0.16572E+00_JPRB, 0.16339E+00_JPRB, 0.16159E+00_JPRB ,&
&  0.15976E+00_JPRB, 0.15876E+00_JPRB, 0.15740E+00_JPRB, 0.15703E+00_JPRB, 0.15737E+00_JPRB, 0.15827E+00_JPRB, 0.15950E+00_JPRB ,&
&  0.16095E+00_JPRB, 0.16196E+00_JPRB, 0.16321E+00_JPRB, 0.16440E+00_JPRB, 0.16527E+00_JPRB, 0.16580E+00_JPRB, 0.16554E+00_JPRB ,&
&  0.16579E+00_JPRB, 0.16612E+00_JPRB, 0.16568E+00_JPRB, 0.16567E+00_JPRB, 0.16448E+00_JPRB, 0.16308E+00_JPRB, 0.16164E+00_JPRB ,&
&  0.15968E+00_JPRB, 0.15817E+00_JPRB, 0.15763E+00_JPRB, 0.15733E+00_JPRB, 0.15810E+00_JPRB, 0.15924E+00_JPRB, 0.15998E+00_JPRB ,&
&  0.16034E+00_JPRB, 0.16026E+00_JPRB, 0.16028E+00_JPRB, 0.16150E+00_JPRB, 0.16276E+00_JPRB, 0.16437E+00_JPRB, 0.16604E+00_JPRB ,&
&  0.16655E+00_JPRB, 0.16792E+00_JPRB, 0.16911E+00_JPRB, 0.17062E+00_JPRB, 0.17579E+00_JPRB, 0.17976E+00_JPRB, 0.18734E+00_JPRB ,&
&  0.19064E+00_JPRB, 0.19070E+00_JPRB, 0.18930E+00_JPRB, 0.19410E+00_JPRB, 0.19472E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.19331E+00_JPRB, 0.19581E+00_JPRB, 0.19437E+00_JPRB, 0.19414E+00_JPRB, 0.19344E+00_JPRB, 0.19321E+00_JPRB, 0.19199E+00_JPRB ,&
&  0.19177E+00_JPRB, 0.18991E+00_JPRB, 0.18958E+00_JPRB, 0.18790E+00_JPRB, 0.18617E+00_JPRB, 0.18375E+00_JPRB, 0.18135E+00_JPRB ,&
&  0.17899E+00_JPRB, 0.17691E+00_JPRB, 0.17552E+00_JPRB, 0.17495E+00_JPRB, 0.17380E+00_JPRB, 0.17390E+00_JPRB, 0.17608E+00_JPRB ,&
&  0.17655E+00_JPRB, 0.17771E+00_JPRB, 0.17781E+00_JPRB, 0.17905E+00_JPRB, 0.18068E+00_JPRB, 0.18122E+00_JPRB, 0.18348E+00_JPRB ,&
&  0.18442E+00_JPRB, 0.18594E+00_JPRB, 0.19216E+00_JPRB, 0.19549E+00_JPRB, 0.19727E+00_JPRB, 0.20008E+00_JPRB, 0.20042E+00_JPRB ,&
&  0.19889E+00_JPRB, 0.19424E+00_JPRB, 0.19729E+00_JPRB, 0.19454E+00_JPRB, 0.18788E+00_JPRB, 0.18506E+00_JPRB, 0.17968E+00_JPRB ,&
&  0.17670E+00_JPRB, 0.17299E+00_JPRB, 0.17022E+00_JPRB, 0.16807E+00_JPRB, 0.16492E+00_JPRB, 0.16291E+00_JPRB, 0.16065E+00_JPRB ,&
&  0.15855E+00_JPRB, 0.15771E+00_JPRB, 0.15630E+00_JPRB, 0.15619E+00_JPRB, 0.15639E+00_JPRB, 0.15688E+00_JPRB, 0.15767E+00_JPRB ,&
&  0.15871E+00_JPRB, 0.15967E+00_JPRB, 0.16057E+00_JPRB, 0.16135E+00_JPRB, 0.16201E+00_JPRB, 0.16239E+00_JPRB, 0.16226E+00_JPRB ,&
&  0.16239E+00_JPRB, 0.16231E+00_JPRB, 0.16184E+00_JPRB, 0.16163E+00_JPRB, 0.16067E+00_JPRB, 0.16024E+00_JPRB, 0.15981E+00_JPRB ,&
&  0.15961E+00_JPRB, 0.15963E+00_JPRB, 0.15955E+00_JPRB, 0.15976E+00_JPRB, 0.16005E+00_JPRB, 0.16078E+00_JPRB, 0.16136E+00_JPRB ,&
&  0.16183E+00_JPRB, 0.16276E+00_JPRB, 0.16333E+00_JPRB, 0.16411E+00_JPRB, 0.16521E+00_JPRB, 0.16567E+00_JPRB, 0.16691E+00_JPRB ,&
&  0.16795E+00_JPRB, 0.16920E+00_JPRB, 0.17156E+00_JPRB, 0.17392E+00_JPRB, 0.17734E+00_JPRB, 0.18155E+00_JPRB, 0.18306E+00_JPRB ,&
&  0.18430E+00_JPRB, 0.18496E+00_JPRB, 0.18316E+00_JPRB, 0.18838E+00_JPRB, 0.19082E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.18657E+00_JPRB, 0.18847E+00_JPRB, 0.18969E+00_JPRB, 0.19054E+00_JPRB, 0.19187E+00_JPRB, 0.19246E+00_JPRB, 0.19300E+00_JPRB ,&
&  0.19108E+00_JPRB, 0.19049E+00_JPRB, 0.18945E+00_JPRB, 0.18816E+00_JPRB, 0.18726E+00_JPRB, 0.18484E+00_JPRB, 0.18248E+00_JPRB ,&
&  0.17989E+00_JPRB, 0.17760E+00_JPRB, 0.17627E+00_JPRB, 0.17497E+00_JPRB, 0.17362E+00_JPRB, 0.17250E+00_JPRB, 0.17457E+00_JPRB ,&
&  0.17578E+00_JPRB, 0.17655E+00_JPRB, 0.17791E+00_JPRB, 0.17770E+00_JPRB, 0.17904E+00_JPRB, 0.18115E+00_JPRB, 0.18397E+00_JPRB ,&
&  0.18566E+00_JPRB, 0.18856E+00_JPRB, 0.19025E+00_JPRB, 0.19110E+00_JPRB, 0.19230E+00_JPRB, 0.19317E+00_JPRB, 0.19288E+00_JPRB ,&
&  0.19154E+00_JPRB, 0.18976E+00_JPRB, 0.18763E+00_JPRB, 0.18489E+00_JPRB, 0.18191E+00_JPRB, 0.17904E+00_JPRB, 0.17617E+00_JPRB ,&
&  0.17410E+00_JPRB, 0.17221E+00_JPRB, 0.17019E+00_JPRB, 0.16833E+00_JPRB, 0.16657E+00_JPRB, 0.16496E+00_JPRB, 0.16336E+00_JPRB ,&
&  0.16209E+00_JPRB, 0.16110E+00_JPRB, 0.15986E+00_JPRB, 0.15922E+00_JPRB, 0.15859E+00_JPRB, 0.15833E+00_JPRB, 0.15822E+00_JPRB ,&
&  0.15868E+00_JPRB, 0.15926E+00_JPRB, 0.15985E+00_JPRB, 0.16020E+00_JPRB, 0.16052E+00_JPRB, 0.16068E+00_JPRB, 0.16097E+00_JPRB ,&
&  0.16125E+00_JPRB, 0.16157E+00_JPRB, 0.16117E+00_JPRB, 0.16081E+00_JPRB, 0.16029E+00_JPRB, 0.15971E+00_JPRB, 0.15953E+00_JPRB ,&
&  0.15983E+00_JPRB, 0.16007E+00_JPRB, 0.15990E+00_JPRB, 0.16019E+00_JPRB, 0.16072E+00_JPRB, 0.16126E+00_JPRB, 0.16263E+00_JPRB ,&
&  0.16339E+00_JPRB, 0.16410E+00_JPRB, 0.16477E+00_JPRB, 0.16529E+00_JPRB, 0.16609E+00_JPRB, 0.16674E+00_JPRB, 0.16755E+00_JPRB ,&
&  0.16852E+00_JPRB, 0.16947E+00_JPRB, 0.17078E+00_JPRB, 0.17246E+00_JPRB, 0.17649E+00_JPRB, 0.17910E+00_JPRB, 0.18128E+00_JPRB ,&
&  0.18218E+00_JPRB, 0.17828E+00_JPRB, 0.18546E+00_JPRB, 0.18439E+00_JPRB, 0.18405E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.17855E+00_JPRB, 0.18033E+00_JPRB, 0.18169E+00_JPRB, 0.18355E+00_JPRB, 0.17884E+00_JPRB, 0.18574E+00_JPRB, 0.18275E+00_JPRB ,&
&  0.17965E+00_JPRB, 0.17480E+00_JPRB, 0.17503E+00_JPRB, 0.17429E+00_JPRB, 0.17319E+00_JPRB, 0.17270E+00_JPRB, 0.17268E+00_JPRB ,&
&  0.17283E+00_JPRB, 0.17272E+00_JPRB, 0.17329E+00_JPRB, 0.17370E+00_JPRB, 0.17403E+00_JPRB, 0.17428E+00_JPRB, 0.17464E+00_JPRB ,&
&  0.17523E+00_JPRB, 0.17621E+00_JPRB, 0.17723E+00_JPRB, 0.17527E+00_JPRB, 0.17504E+00_JPRB, 0.17528E+00_JPRB, 0.17587E+00_JPRB ,&
&  0.17625E+00_JPRB, 0.17737E+00_JPRB, 0.17822E+00_JPRB, 0.17885E+00_JPRB, 0.18062E+00_JPRB, 0.18073E+00_JPRB, 0.18059E+00_JPRB ,&
&  0.18019E+00_JPRB, 0.17934E+00_JPRB, 0.17814E+00_JPRB, 0.17702E+00_JPRB, 0.17587E+00_JPRB, 0.17435E+00_JPRB, 0.17289E+00_JPRB ,&
&  0.17169E+00_JPRB, 0.17071E+00_JPRB, 0.16944E+00_JPRB, 0.16820E+00_JPRB, 0.16708E+00_JPRB, 0.16601E+00_JPRB, 0.16509E+00_JPRB ,&
&  0.16442E+00_JPRB, 0.16386E+00_JPRB, 0.16327E+00_JPRB, 0.16283E+00_JPRB, 0.16236E+00_JPRB, 0.16193E+00_JPRB, 0.16153E+00_JPRB ,&
&  0.16172E+00_JPRB, 0.16189E+00_JPRB, 0.16206E+00_JPRB, 0.16222E+00_JPRB, 0.16269E+00_JPRB, 0.16280E+00_JPRB, 0.16294E+00_JPRB ,&
&  0.16303E+00_JPRB, 0.16296E+00_JPRB, 0.16285E+00_JPRB, 0.16264E+00_JPRB, 0.16239E+00_JPRB, 0.16207E+00_JPRB, 0.16192E+00_JPRB ,&
&  0.16191E+00_JPRB, 0.16202E+00_JPRB, 0.16236E+00_JPRB, 0.16260E+00_JPRB, 0.16285E+00_JPRB, 0.16305E+00_JPRB, 0.16353E+00_JPRB ,&
&  0.16402E+00_JPRB, 0.16452E+00_JPRB, 0.16499E+00_JPRB, 0.16488E+00_JPRB, 0.16513E+00_JPRB, 0.16537E+00_JPRB, 0.16559E+00_JPRB ,&
&  0.16573E+00_JPRB, 0.16622E+00_JPRB, 0.16701E+00_JPRB, 0.16807E+00_JPRB, 0.17003E+00_JPRB, 0.17168E+00_JPRB, 0.17322E+00_JPRB ,&
&  0.17485E+00_JPRB, 0.17815E+00_JPRB, 0.17938E+00_JPRB, 0.17971E+00_JPRB, 0.17960E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.16907E+00_JPRB, 0.16991E+00_JPRB, 0.17050E+00_JPRB, 0.17095E+00_JPRB, 0.17073E+00_JPRB, 0.17078E+00_JPRB, 0.17081E+00_JPRB ,&
&  0.17075E+00_JPRB, 0.17069E+00_JPRB, 0.17096E+00_JPRB, 0.17120E+00_JPRB, 0.17148E+00_JPRB, 0.17177E+00_JPRB, 0.17214E+00_JPRB ,&
&  0.17228E+00_JPRB, 0.17260E+00_JPRB, 0.17133E+00_JPRB, 0.17028E+00_JPRB, 0.16958E+00_JPRB, 0.16915E+00_JPRB, 0.16891E+00_JPRB ,&
&  0.16870E+00_JPRB, 0.16856E+00_JPRB, 0.16840E+00_JPRB, 0.16929E+00_JPRB, 0.16986E+00_JPRB, 0.17037E+00_JPRB, 0.17076E+00_JPRB ,&
&  0.17090E+00_JPRB, 0.17113E+00_JPRB, 0.17132E+00_JPRB, 0.17153E+00_JPRB, 0.17177E+00_JPRB, 0.17174E+00_JPRB, 0.17162E+00_JPRB ,&
&  0.17142E+00_JPRB, 0.17109E+00_JPRB, 0.17071E+00_JPRB, 0.17035E+00_JPRB, 0.17006E+00_JPRB, 0.16971E+00_JPRB, 0.16933E+00_JPRB ,&
&  0.16896E+00_JPRB, 0.16854E+00_JPRB, 0.16814E+00_JPRB, 0.16776E+00_JPRB, 0.16739E+00_JPRB, 0.16708E+00_JPRB, 0.16675E+00_JPRB ,&
&  0.16652E+00_JPRB, 0.16631E+00_JPRB, 0.16610E+00_JPRB, 0.16596E+00_JPRB, 0.16572E+00_JPRB, 0.16564E+00_JPRB, 0.16544E+00_JPRB ,&
&  0.16539E+00_JPRB, 0.16532E+00_JPRB, 0.16528E+00_JPRB, 0.16522E+00_JPRB, 0.16524E+00_JPRB, 0.16523E+00_JPRB, 0.16524E+00_JPRB ,&
&  0.16523E+00_JPRB, 0.16538E+00_JPRB, 0.16526E+00_JPRB, 0.16511E+00_JPRB, 0.16494E+00_JPRB, 0.16477E+00_JPRB, 0.16461E+00_JPRB ,&
&  0.16446E+00_JPRB, 0.16431E+00_JPRB, 0.16421E+00_JPRB, 0.16424E+00_JPRB, 0.16428E+00_JPRB, 0.16435E+00_JPRB, 0.16443E+00_JPRB ,&
&  0.16452E+00_JPRB, 0.16464E+00_JPRB, 0.16476E+00_JPRB, 0.16473E+00_JPRB, 0.16476E+00_JPRB, 0.16478E+00_JPRB, 0.16481E+00_JPRB ,&
&  0.16483E+00_JPRB, 0.16487E+00_JPRB, 0.16497E+00_JPRB, 0.16509E+00_JPRB, 0.16538E+00_JPRB, 0.16549E+00_JPRB, 0.16566E+00_JPRB ,&
&  0.16589E+00_JPRB, 0.16631E+00_JPRB, 0.16667E+00_JPRB, 0.16707E+00_JPRB, 0.16752E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB ,&
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB ,&
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB ,&
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB ,&
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB ,&
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB ,&
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB ,&
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB ,&
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB ,&
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB ,&
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB ,&
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB ,&
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB ,&
&  0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB, 0.16634E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB, 0.52863E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.52483E+00_JPRB, 0.52403E+00_JPRB, 0.52331E+00_JPRB, 0.52260E+00_JPRB, 0.52177E+00_JPRB, 0.52119E+00_JPRB, 0.52071E+00_JPRB ,&
&  0.52026E+00_JPRB, 0.51981E+00_JPRB, 0.51932E+00_JPRB, 0.51895E+00_JPRB, 0.51873E+00_JPRB, 0.51842E+00_JPRB, 0.51808E+00_JPRB ,&
&  0.51769E+00_JPRB, 0.51739E+00_JPRB, 0.51724E+00_JPRB, 0.51680E+00_JPRB, 0.51640E+00_JPRB, 0.51603E+00_JPRB, 0.51539E+00_JPRB ,&
&  0.51451E+00_JPRB, 0.51369E+00_JPRB, 0.51281E+00_JPRB, 0.51209E+00_JPRB, 0.51157E+00_JPRB, 0.51073E+00_JPRB, 0.50971E+00_JPRB ,&
&  0.50884E+00_JPRB, 0.50830E+00_JPRB, 0.50804E+00_JPRB, 0.50801E+00_JPRB, 0.50815E+00_JPRB, 0.50827E+00_JPRB, 0.50834E+00_JPRB ,&
&  0.50848E+00_JPRB, 0.50883E+00_JPRB, 0.50940E+00_JPRB, 0.51024E+00_JPRB, 0.51131E+00_JPRB, 0.51261E+00_JPRB, 0.51444E+00_JPRB ,&
&  0.51651E+00_JPRB, 0.51863E+00_JPRB, 0.52078E+00_JPRB, 0.52270E+00_JPRB, 0.52441E+00_JPRB, 0.52588E+00_JPRB, 0.52689E+00_JPRB ,&
&  0.52800E+00_JPRB, 0.52922E+00_JPRB, 0.53050E+00_JPRB, 0.53138E+00_JPRB, 0.53215E+00_JPRB, 0.53287E+00_JPRB, 0.53368E+00_JPRB ,&
&  0.53403E+00_JPRB, 0.53431E+00_JPRB, 0.53468E+00_JPRB, 0.53520E+00_JPRB, 0.53547E+00_JPRB, 0.53562E+00_JPRB, 0.53580E+00_JPRB ,&
&  0.53616E+00_JPRB, 0.53643E+00_JPRB, 0.53650E+00_JPRB, 0.53661E+00_JPRB, 0.53692E+00_JPRB, 0.53739E+00_JPRB, 0.53786E+00_JPRB ,&
&  0.53828E+00_JPRB, 0.53865E+00_JPRB, 0.53886E+00_JPRB, 0.53891E+00_JPRB, 0.53881E+00_JPRB, 0.53846E+00_JPRB, 0.53770E+00_JPRB ,&
&  0.53677E+00_JPRB, 0.53575E+00_JPRB, 0.53489E+00_JPRB, 0.53431E+00_JPRB, 0.53367E+00_JPRB, 0.53314E+00_JPRB, 0.53275E+00_JPRB ,&
&  0.53258E+00_JPRB, 0.53247E+00_JPRB, 0.53231E+00_JPRB, 0.53219E+00_JPRB, 0.53198E+00_JPRB, 0.53165E+00_JPRB, 0.53115E+00_JPRB ,&
&  0.53047E+00_JPRB, 0.52952E+00_JPRB, 0.52845E+00_JPRB, 0.52725E+00_JPRB, 0.52600E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.51512E+00_JPRB, 0.51342E+00_JPRB, 0.51221E+00_JPRB, 0.51145E+00_JPRB, 0.51071E+00_JPRB, 0.51005E+00_JPRB, 0.50944E+00_JPRB ,&
&  0.50880E+00_JPRB, 0.50880E+00_JPRB, 0.50846E+00_JPRB, 0.50821E+00_JPRB, 0.50787E+00_JPRB, 0.50798E+00_JPRB, 0.50779E+00_JPRB ,&
&  0.50739E+00_JPRB, 0.50632E+00_JPRB, 0.50563E+00_JPRB, 0.50449E+00_JPRB, 0.50451E+00_JPRB, 0.50465E+00_JPRB, 0.50598E+00_JPRB ,&
&  0.50589E+00_JPRB, 0.50562E+00_JPRB, 0.50461E+00_JPRB, 0.50113E+00_JPRB, 0.49805E+00_JPRB, 0.49507E+00_JPRB, 0.49261E+00_JPRB ,&
&  0.49044E+00_JPRB, 0.48893E+00_JPRB, 0.48857E+00_JPRB, 0.48882E+00_JPRB, 0.49134E+00_JPRB, 0.49201E+00_JPRB, 0.49215E+00_JPRB ,&
&  0.49147E+00_JPRB, 0.48874E+00_JPRB, 0.48933E+00_JPRB, 0.49021E+00_JPRB, 0.49166E+00_JPRB, 0.49510E+00_JPRB, 0.49817E+00_JPRB ,&
&  0.50195E+00_JPRB, 0.50635E+00_JPRB, 0.51319E+00_JPRB, 0.51813E+00_JPRB, 0.52321E+00_JPRB, 0.52757E+00_JPRB, 0.52991E+00_JPRB ,&
&  0.53200E+00_JPRB, 0.53388E+00_JPRB, 0.53576E+00_JPRB, 0.53721E+00_JPRB, 0.53840E+00_JPRB, 0.53966E+00_JPRB, 0.54115E+00_JPRB ,&
&  0.54215E+00_JPRB, 0.54326E+00_JPRB, 0.54441E+00_JPRB, 0.54552E+00_JPRB, 0.54620E+00_JPRB, 0.54671E+00_JPRB, 0.54741E+00_JPRB ,&
&  0.54816E+00_JPRB, 0.54798E+00_JPRB, 0.54777E+00_JPRB, 0.54830E+00_JPRB, 0.54891E+00_JPRB, 0.54952E+00_JPRB, 0.54926E+00_JPRB ,&
&  0.54922E+00_JPRB, 0.54889E+00_JPRB, 0.54876E+00_JPRB, 0.54841E+00_JPRB, 0.54785E+00_JPRB, 0.54735E+00_JPRB, 0.54641E+00_JPRB ,&
&  0.54502E+00_JPRB, 0.54333E+00_JPRB, 0.54194E+00_JPRB, 0.54114E+00_JPRB, 0.54008E+00_JPRB, 0.53966E+00_JPRB, 0.53945E+00_JPRB ,&
&  0.53943E+00_JPRB, 0.53804E+00_JPRB, 0.53643E+00_JPRB, 0.53449E+00_JPRB, 0.53258E+00_JPRB, 0.53062E+00_JPRB, 0.52878E+00_JPRB ,&
&  0.52690E+00_JPRB, 0.52475E+00_JPRB, 0.52238E+00_JPRB, 0.51987E+00_JPRB, 0.51744E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.51637E+00_JPRB, 0.51306E+00_JPRB, 0.51082E+00_JPRB, 0.50937E+00_JPRB, 0.50787E+00_JPRB, 0.50644E+00_JPRB, 0.50575E+00_JPRB ,&
&  0.50523E+00_JPRB, 0.50399E+00_JPRB, 0.50327E+00_JPRB, 0.50270E+00_JPRB, 0.50175E+00_JPRB, 0.50204E+00_JPRB, 0.50179E+00_JPRB ,&
&  0.50198E+00_JPRB, 0.50125E+00_JPRB, 0.50227E+00_JPRB, 0.50109E+00_JPRB, 0.50034E+00_JPRB, 0.49959E+00_JPRB, 0.49773E+00_JPRB ,&
&  0.49789E+00_JPRB, 0.49744E+00_JPRB, 0.49414E+00_JPRB, 0.48992E+00_JPRB, 0.48716E+00_JPRB, 0.47336E+00_JPRB, 0.47424E+00_JPRB ,&
&  0.47972E+00_JPRB, 0.47232E+00_JPRB, 0.46934E+00_JPRB, 0.46609E+00_JPRB, 0.46271E+00_JPRB, 0.46288E+00_JPRB, 0.46628E+00_JPRB ,&
&  0.46109E+00_JPRB, 0.47171E+00_JPRB, 0.47291E+00_JPRB, 0.47363E+00_JPRB, 0.47488E+00_JPRB, 0.47346E+00_JPRB, 0.47770E+00_JPRB ,&
&  0.49462E+00_JPRB, 0.49794E+00_JPRB, 0.50521E+00_JPRB, 0.51179E+00_JPRB, 0.51990E+00_JPRB, 0.52732E+00_JPRB, 0.53504E+00_JPRB ,&
&  0.54082E+00_JPRB, 0.54468E+00_JPRB, 0.54722E+00_JPRB, 0.54770E+00_JPRB, 0.54824E+00_JPRB, 0.54818E+00_JPRB, 0.54894E+00_JPRB ,&
&  0.54867E+00_JPRB, 0.54965E+00_JPRB, 0.55149E+00_JPRB, 0.55390E+00_JPRB, 0.55707E+00_JPRB, 0.55908E+00_JPRB, 0.56094E+00_JPRB ,&
&  0.56211E+00_JPRB, 0.56262E+00_JPRB, 0.56217E+00_JPRB, 0.56212E+00_JPRB, 0.56113E+00_JPRB, 0.56122E+00_JPRB, 0.56069E+00_JPRB ,&
&  0.56114E+00_JPRB, 0.56068E+00_JPRB, 0.56095E+00_JPRB, 0.56058E+00_JPRB, 0.56068E+00_JPRB, 0.56151E+00_JPRB, 0.56203E+00_JPRB ,&
&  0.56289E+00_JPRB, 0.56302E+00_JPRB, 0.56276E+00_JPRB, 0.56230E+00_JPRB, 0.56133E+00_JPRB, 0.56106E+00_JPRB, 0.56061E+00_JPRB ,&
&  0.56057E+00_JPRB, 0.55746E+00_JPRB, 0.55395E+00_JPRB, 0.55022E+00_JPRB, 0.54689E+00_JPRB, 0.54388E+00_JPRB, 0.54071E+00_JPRB ,&
&  0.53783E+00_JPRB, 0.53396E+00_JPRB, 0.52972E+00_JPRB, 0.52496E+00_JPRB, 0.52036E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.52798E+00_JPRB, 0.52316E+00_JPRB, 0.51893E+00_JPRB, 0.51593E+00_JPRB, 0.51323E+00_JPRB, 0.51097E+00_JPRB, 0.50923E+00_JPRB ,&
&  0.50771E+00_JPRB, 0.50658E+00_JPRB, 0.50592E+00_JPRB, 0.50533E+00_JPRB, 0.50478E+00_JPRB, 0.50398E+00_JPRB, 0.50312E+00_JPRB ,&
&  0.50405E+00_JPRB, 0.50332E+00_JPRB, 0.50429E+00_JPRB, 0.50236E+00_JPRB, 0.50087E+00_JPRB, 0.49798E+00_JPRB, 0.49193E+00_JPRB ,&
&  0.48945E+00_JPRB, 0.48666E+00_JPRB, 0.47397E+00_JPRB, 0.48071E+00_JPRB, 0.47643E+00_JPRB, 0.47434E+00_JPRB, 0.47151E+00_JPRB ,&
&  0.46379E+00_JPRB, 0.45986E+00_JPRB, 0.45460E+00_JPRB, 0.45403E+00_JPRB, 0.45640E+00_JPRB, 0.45389E+00_JPRB, 0.45565E+00_JPRB ,&
&  0.45199E+00_JPRB, 0.45074E+00_JPRB, 0.45121E+00_JPRB, 0.45135E+00_JPRB, 0.45472E+00_JPRB, 0.45548E+00_JPRB, 0.45888E+00_JPRB ,&
&  0.46709E+00_JPRB, 0.47845E+00_JPRB, 0.49651E+00_JPRB, 0.50521E+00_JPRB, 0.51713E+00_JPRB, 0.52659E+00_JPRB, 0.53571E+00_JPRB ,&
&  0.54151E+00_JPRB, 0.54539E+00_JPRB, 0.54810E+00_JPRB, 0.54946E+00_JPRB, 0.54929E+00_JPRB, 0.54636E+00_JPRB, 0.54575E+00_JPRB ,&
&  0.54555E+00_JPRB, 0.54640E+00_JPRB, 0.54785E+00_JPRB, 0.54935E+00_JPRB, 0.55226E+00_JPRB, 0.55508E+00_JPRB, 0.55890E+00_JPRB ,&
&  0.56261E+00_JPRB, 0.56669E+00_JPRB, 0.56752E+00_JPRB, 0.56955E+00_JPRB, 0.57014E+00_JPRB, 0.57092E+00_JPRB, 0.57137E+00_JPRB ,&
&  0.57202E+00_JPRB, 0.57286E+00_JPRB, 0.57463E+00_JPRB, 0.57090E+00_JPRB, 0.57102E+00_JPRB, 0.57867E+00_JPRB, 0.57986E+00_JPRB ,&
&  0.58171E+00_JPRB, 0.58113E+00_JPRB, 0.57914E+00_JPRB, 0.57717E+00_JPRB, 0.57539E+00_JPRB, 0.57454E+00_JPRB, 0.57398E+00_JPRB ,&
&  0.57322E+00_JPRB, 0.56904E+00_JPRB, 0.56477E+00_JPRB, 0.56092E+00_JPRB, 0.55675E+00_JPRB, 0.55360E+00_JPRB, 0.55048E+00_JPRB ,&
&  0.54828E+00_JPRB, 0.54643E+00_JPRB, 0.54417E+00_JPRB, 0.53994E+00_JPRB, 0.53434E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.54240E+00_JPRB, 0.53750E+00_JPRB, 0.53363E+00_JPRB, 0.53024E+00_JPRB, 0.52799E+00_JPRB, 0.52594E+00_JPRB, 0.52401E+00_JPRB ,&
&  0.52186E+00_JPRB, 0.51964E+00_JPRB, 0.51707E+00_JPRB, 0.51529E+00_JPRB, 0.51358E+00_JPRB, 0.51210E+00_JPRB, 0.51015E+00_JPRB ,&
&  0.50912E+00_JPRB, 0.50754E+00_JPRB, 0.50756E+00_JPRB, 0.50509E+00_JPRB, 0.50523E+00_JPRB, 0.50191E+00_JPRB, 0.49447E+00_JPRB ,&
&  0.48770E+00_JPRB, 0.48042E+00_JPRB, 0.47023E+00_JPRB, 0.47570E+00_JPRB, 0.47463E+00_JPRB, 0.47539E+00_JPRB, 0.46695E+00_JPRB ,&
&  0.46082E+00_JPRB, 0.44928E+00_JPRB, 0.46173E+00_JPRB, 0.45584E+00_JPRB, 0.45184E+00_JPRB, 0.44526E+00_JPRB, 0.43438E+00_JPRB ,&
&  0.43185E+00_JPRB, 0.42554E+00_JPRB, 0.42515E+00_JPRB, 0.42824E+00_JPRB, 0.43238E+00_JPRB, 0.43836E+00_JPRB, 0.44470E+00_JPRB ,&
&  0.44621E+00_JPRB, 0.47499E+00_JPRB, 0.49285E+00_JPRB, 0.50606E+00_JPRB, 0.51775E+00_JPRB, 0.52711E+00_JPRB, 0.53499E+00_JPRB ,&
&  0.54203E+00_JPRB, 0.54793E+00_JPRB, 0.55355E+00_JPRB, 0.55489E+00_JPRB, 0.56198E+00_JPRB, 0.56374E+00_JPRB, 0.56601E+00_JPRB ,&
&  0.56850E+00_JPRB, 0.56799E+00_JPRB, 0.56698E+00_JPRB, 0.56434E+00_JPRB, 0.56328E+00_JPRB, 0.56108E+00_JPRB, 0.56238E+00_JPRB ,&
&  0.56230E+00_JPRB, 0.56348E+00_JPRB, 0.56663E+00_JPRB, 0.56961E+00_JPRB, 0.57206E+00_JPRB, 0.57451E+00_JPRB, 0.57712E+00_JPRB ,&
&  0.57959E+00_JPRB, 0.58449E+00_JPRB, 0.58908E+00_JPRB, 0.59727E+00_JPRB, 0.60392E+00_JPRB, 0.60616E+00_JPRB, 0.60778E+00_JPRB ,&
&  0.61036E+00_JPRB, 0.60559E+00_JPRB, 0.60112E+00_JPRB, 0.59790E+00_JPRB, 0.59592E+00_JPRB, 0.59480E+00_JPRB, 0.59485E+00_JPRB ,&
&  0.59401E+00_JPRB, 0.59008E+00_JPRB, 0.58543E+00_JPRB, 0.57997E+00_JPRB, 0.57451E+00_JPRB, 0.57056E+00_JPRB, 0.56712E+00_JPRB ,&
&  0.56444E+00_JPRB, 0.56195E+00_JPRB, 0.55791E+00_JPRB, 0.55265E+00_JPRB, 0.54735E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.56277E+00_JPRB, 0.55841E+00_JPRB, 0.55512E+00_JPRB, 0.55171E+00_JPRB, 0.54964E+00_JPRB, 0.54753E+00_JPRB, 0.54568E+00_JPRB ,&
&  0.54417E+00_JPRB, 0.54092E+00_JPRB, 0.53701E+00_JPRB, 0.53224E+00_JPRB, 0.52876E+00_JPRB, 0.52552E+00_JPRB, 0.52189E+00_JPRB ,&
&  0.51644E+00_JPRB, 0.51043E+00_JPRB, 0.50402E+00_JPRB, 0.49881E+00_JPRB, 0.49893E+00_JPRB, 0.49649E+00_JPRB, 0.49315E+00_JPRB ,&
&  0.48640E+00_JPRB, 0.47859E+00_JPRB, 0.47075E+00_JPRB, 0.47105E+00_JPRB, 0.47208E+00_JPRB, 0.46583E+00_JPRB, 0.46610E+00_JPRB ,&
&  0.47496E+00_JPRB, 0.47672E+00_JPRB, 0.47482E+00_JPRB, 0.47366E+00_JPRB, 0.45417E+00_JPRB, 0.42212E+00_JPRB, 0.43533E+00_JPRB ,&
&  0.43383E+00_JPRB, 0.42240E+00_JPRB, 0.41901E+00_JPRB, 0.41697E+00_JPRB, 0.42151E+00_JPRB, 0.43230E+00_JPRB, 0.44329E+00_JPRB ,&
&  0.44813E+00_JPRB, 0.49651E+00_JPRB, 0.51636E+00_JPRB, 0.52987E+00_JPRB, 0.53813E+00_JPRB, 0.54550E+00_JPRB, 0.55277E+00_JPRB ,&
&  0.55903E+00_JPRB, 0.56849E+00_JPRB, 0.58066E+00_JPRB, 0.59213E+00_JPRB, 0.60267E+00_JPRB, 0.61062E+00_JPRB, 0.61866E+00_JPRB ,&
&  0.62939E+00_JPRB, 0.63017E+00_JPRB, 0.62105E+00_JPRB, 0.61053E+00_JPRB, 0.60118E+00_JPRB, 0.59191E+00_JPRB, 0.58398E+00_JPRB ,&
&  0.57996E+00_JPRB, 0.57636E+00_JPRB, 0.57977E+00_JPRB, 0.58015E+00_JPRB, 0.58177E+00_JPRB, 0.58452E+00_JPRB, 0.59280E+00_JPRB ,&
&  0.59828E+00_JPRB, 0.60623E+00_JPRB, 0.61472E+00_JPRB, 0.62617E+00_JPRB, 0.63430E+00_JPRB, 0.63642E+00_JPRB, 0.63754E+00_JPRB ,&
&  0.63657E+00_JPRB, 0.64099E+00_JPRB, 0.64150E+00_JPRB, 0.64410E+00_JPRB, 0.64956E+00_JPRB, 0.65196E+00_JPRB, 0.65506E+00_JPRB ,&
&  0.65645E+00_JPRB, 0.65614E+00_JPRB, 0.65211E+00_JPRB, 0.64519E+00_JPRB, 0.63572E+00_JPRB, 0.62505E+00_JPRB, 0.61253E+00_JPRB ,&
&  0.60042E+00_JPRB, 0.58794E+00_JPRB, 0.57758E+00_JPRB, 0.57133E+00_JPRB, 0.56640E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.59673E+00_JPRB, 0.58796E+00_JPRB, 0.58143E+00_JPRB, 0.57769E+00_JPRB, 0.57462E+00_JPRB, 0.57117E+00_JPRB, 0.56809E+00_JPRB ,&
&  0.56515E+00_JPRB, 0.56204E+00_JPRB, 0.55768E+00_JPRB, 0.55288E+00_JPRB, 0.54869E+00_JPRB, 0.54498E+00_JPRB, 0.54194E+00_JPRB ,&
&  0.53543E+00_JPRB, 0.51877E+00_JPRB, 0.51321E+00_JPRB, 0.50533E+00_JPRB, 0.49400E+00_JPRB, 0.48774E+00_JPRB, 0.48325E+00_JPRB ,&
&  0.47757E+00_JPRB, 0.47486E+00_JPRB, 0.47508E+00_JPRB, 0.47031E+00_JPRB, 0.46782E+00_JPRB, 0.47383E+00_JPRB, 0.47644E+00_JPRB ,&
&  0.46196E+00_JPRB, 0.46260E+00_JPRB, 0.48385E+00_JPRB, 0.48469E+00_JPRB, 0.46268E+00_JPRB, 0.45623E+00_JPRB, 0.45094E+00_JPRB ,&
&  0.44663E+00_JPRB, 0.42894E+00_JPRB, 0.42430E+00_JPRB, 0.42177E+00_JPRB, 0.42931E+00_JPRB, 0.43909E+00_JPRB, 0.46327E+00_JPRB ,&
&  0.48664E+00_JPRB, 0.53160E+00_JPRB, 0.54920E+00_JPRB, 0.56108E+00_JPRB, 0.57200E+00_JPRB, 0.58376E+00_JPRB, 0.59634E+00_JPRB ,&
&  0.61325E+00_JPRB, 0.63230E+00_JPRB, 0.65675E+00_JPRB, 0.67827E+00_JPRB, 0.69928E+00_JPRB, 0.70949E+00_JPRB, 0.71079E+00_JPRB ,&
&  0.72711E+00_JPRB, 0.72804E+00_JPRB, 0.70489E+00_JPRB, 0.68538E+00_JPRB, 0.66465E+00_JPRB, 0.64447E+00_JPRB, 0.62141E+00_JPRB ,&
&  0.61459E+00_JPRB, 0.62119E+00_JPRB, 0.61483E+00_JPRB, 0.60952E+00_JPRB, 0.60692E+00_JPRB, 0.60665E+00_JPRB, 0.60983E+00_JPRB ,&
&  0.61083E+00_JPRB, 0.63259E+00_JPRB, 0.70931E+00_JPRB, 0.72611E+00_JPRB, 0.72812E+00_JPRB, 0.72581E+00_JPRB, 0.72266E+00_JPRB ,&
&  0.72491E+00_JPRB, 0.73252E+00_JPRB, 0.74262E+00_JPRB, 0.74737E+00_JPRB, 0.75270E+00_JPRB, 0.74768E+00_JPRB, 0.75273E+00_JPRB ,&
&  0.74784E+00_JPRB, 0.74175E+00_JPRB, 0.73270E+00_JPRB, 0.72600E+00_JPRB, 0.71791E+00_JPRB, 0.70979E+00_JPRB, 0.69803E+00_JPRB ,&
&  0.68340E+00_JPRB, 0.66566E+00_JPRB, 0.64740E+00_JPRB, 0.62860E+00_JPRB, 0.61090E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.64405E+00_JPRB, 0.62130E+00_JPRB, 0.60627E+00_JPRB, 0.60028E+00_JPRB, 0.59720E+00_JPRB, 0.58209E+00_JPRB, 0.57093E+00_JPRB ,&
&  0.56394E+00_JPRB, 0.56219E+00_JPRB, 0.55953E+00_JPRB, 0.55452E+00_JPRB, 0.55188E+00_JPRB, 0.54951E+00_JPRB, 0.54844E+00_JPRB ,&
&  0.54603E+00_JPRB, 0.53961E+00_JPRB, 0.52607E+00_JPRB, 0.50725E+00_JPRB, 0.49326E+00_JPRB, 0.48320E+00_JPRB, 0.47655E+00_JPRB ,&
&  0.47175E+00_JPRB, 0.47088E+00_JPRB, 0.47678E+00_JPRB, 0.47735E+00_JPRB, 0.47640E+00_JPRB, 0.47254E+00_JPRB, 0.46816E+00_JPRB ,&
&  0.46907E+00_JPRB, 0.47829E+00_JPRB, 0.46926E+00_JPRB, 0.47186E+00_JPRB, 0.48243E+00_JPRB, 0.47654E+00_JPRB, 0.46296E+00_JPRB ,&
&  0.45207E+00_JPRB, 0.44165E+00_JPRB, 0.43626E+00_JPRB, 0.43547E+00_JPRB, 0.44806E+00_JPRB, 0.46651E+00_JPRB, 0.49860E+00_JPRB ,&
&  0.54467E+00_JPRB, 0.56800E+00_JPRB, 0.58038E+00_JPRB, 0.59300E+00_JPRB, 0.60657E+00_JPRB, 0.62156E+00_JPRB, 0.64485E+00_JPRB ,&
&  0.68141E+00_JPRB, 0.71996E+00_JPRB, 0.75802E+00_JPRB, 0.78337E+00_JPRB, 0.81144E+00_JPRB, 0.78800E+00_JPRB, 0.78683E+00_JPRB ,&
&  0.87634E+00_JPRB, 0.90936E+00_JPRB, 0.86132E+00_JPRB, 0.81882E+00_JPRB, 0.77558E+00_JPRB, 0.73705E+00_JPRB, 0.70489E+00_JPRB ,&
&  0.68358E+00_JPRB, 0.67112E+00_JPRB, 0.66126E+00_JPRB, 0.64748E+00_JPRB, 0.63789E+00_JPRB, 0.64087E+00_JPRB, 0.65943E+00_JPRB ,&
&  0.70029E+00_JPRB, 0.81518E+00_JPRB, 0.10601E+01_JPRB, 0.90436E+00_JPRB, 0.84497E+00_JPRB, 0.83301E+00_JPRB, 0.83478E+00_JPRB ,&
&  0.85311E+00_JPRB, 0.86822E+00_JPRB, 0.87584E+00_JPRB, 0.86089E+00_JPRB, 0.83230E+00_JPRB, 0.79905E+00_JPRB, 0.77656E+00_JPRB ,&
&  0.75796E+00_JPRB, 0.73952E+00_JPRB, 0.72651E+00_JPRB, 0.72215E+00_JPRB, 0.71355E+00_JPRB, 0.70423E+00_JPRB, 0.69651E+00_JPRB ,&
&  0.68946E+00_JPRB, 0.68577E+00_JPRB, 0.68520E+00_JPRB, 0.67928E+00_JPRB, 0.66855E+00_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.64724E+00_JPRB, 0.65401E+00_JPRB, 0.65219E+00_JPRB, 0.64235E+00_JPRB, 0.63149E+00_JPRB, 0.59571E+00_JPRB, 0.56814E+00_JPRB ,&
&  0.55489E+00_JPRB, 0.55040E+00_JPRB, 0.55156E+00_JPRB, 0.55170E+00_JPRB, 0.55254E+00_JPRB, 0.55246E+00_JPRB, 0.54893E+00_JPRB ,&
&  0.54103E+00_JPRB, 0.53395E+00_JPRB, 0.52716E+00_JPRB, 0.51560E+00_JPRB, 0.50068E+00_JPRB, 0.48508E+00_JPRB, 0.47303E+00_JPRB ,&
&  0.46515E+00_JPRB, 0.46265E+00_JPRB, 0.46852E+00_JPRB, 0.47510E+00_JPRB, 0.48188E+00_JPRB, 0.48447E+00_JPRB, 0.48333E+00_JPRB ,&
&  0.47424E+00_JPRB, 0.46951E+00_JPRB, 0.47668E+00_JPRB, 0.47922E+00_JPRB, 0.48337E+00_JPRB, 0.48892E+00_JPRB, 0.47755E+00_JPRB ,&
&  0.46591E+00_JPRB, 0.45623E+00_JPRB, 0.45690E+00_JPRB, 0.46837E+00_JPRB, 0.50373E+00_JPRB, 0.55632E+00_JPRB, 0.57358E+00_JPRB ,&
&  0.58845E+00_JPRB, 0.59886E+00_JPRB, 0.61007E+00_JPRB, 0.62923E+00_JPRB, 0.65292E+00_JPRB, 0.68014E+00_JPRB, 0.72876E+00_JPRB ,&
&  0.79602E+00_JPRB, 0.84759E+00_JPRB, 0.89853E+00_JPRB, 0.83302E+00_JPRB, 0.80737E+00_JPRB, 0.83209E+00_JPRB, 0.86789E+00_JPRB ,&
&  0.91241E+00_JPRB, 0.93578E+00_JPRB, 0.93031E+00_JPRB, 0.93847E+00_JPRB, 0.90050E+00_JPRB, 0.85110E+00_JPRB, 0.78250E+00_JPRB ,&
&  0.73169E+00_JPRB, 0.69833E+00_JPRB, 0.68438E+00_JPRB, 0.67501E+00_JPRB, 0.68613E+00_JPRB, 0.71018E+00_JPRB, 0.75110E+00_JPRB ,&
&  0.81614E+00_JPRB, 0.89451E+00_JPRB, 0.93727E+00_JPRB, 0.90528E+00_JPRB, 0.92326E+00_JPRB, 0.96475E+00_JPRB, 0.99595E+00_JPRB ,&
&  0.99569E+00_JPRB, 0.99150E+00_JPRB, 0.99513E+00_JPRB, 0.98459E+00_JPRB, 0.91862E+00_JPRB, 0.85958E+00_JPRB, 0.79288E+00_JPRB ,&
&  0.72925E+00_JPRB, 0.67242E+00_JPRB, 0.64197E+00_JPRB, 0.63327E+00_JPRB, 0.62634E+00_JPRB, 0.61829E+00_JPRB, 0.60512E+00_JPRB ,&
&  0.59332E+00_JPRB, 0.59230E+00_JPRB, 0.60285E+00_JPRB, 0.61641E+00_JPRB, 0.63516E+00_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.68260E+00_JPRB, 0.69632E+00_JPRB, 0.69924E+00_JPRB, 0.70228E+00_JPRB, 0.67177E+00_JPRB, 0.64305E+00_JPRB, 0.61496E+00_JPRB ,&
&  0.60282E+00_JPRB, 0.58432E+00_JPRB, 0.57288E+00_JPRB, 0.55677E+00_JPRB, 0.54235E+00_JPRB, 0.53449E+00_JPRB, 0.53524E+00_JPRB ,&
&  0.53786E+00_JPRB, 0.53415E+00_JPRB, 0.52674E+00_JPRB, 0.51289E+00_JPRB, 0.50030E+00_JPRB, 0.48903E+00_JPRB, 0.47936E+00_JPRB ,&
&  0.47213E+00_JPRB, 0.47135E+00_JPRB, 0.47476E+00_JPRB, 0.48233E+00_JPRB, 0.48810E+00_JPRB, 0.49220E+00_JPRB, 0.49317E+00_JPRB ,&
&  0.48857E+00_JPRB, 0.48096E+00_JPRB, 0.48334E+00_JPRB, 0.49223E+00_JPRB, 0.49663E+00_JPRB, 0.50592E+00_JPRB, 0.49142E+00_JPRB ,&
&  0.47934E+00_JPRB, 0.48072E+00_JPRB, 0.49872E+00_JPRB, 0.55854E+00_JPRB, 0.58129E+00_JPRB, 0.59225E+00_JPRB, 0.60413E+00_JPRB ,&
&  0.59528E+00_JPRB, 0.59503E+00_JPRB, 0.61341E+00_JPRB, 0.65857E+00_JPRB, 0.70325E+00_JPRB, 0.77515E+00_JPRB, 0.86234E+00_JPRB ,&
&  0.93145E+00_JPRB, 0.96142E+00_JPRB, 0.98242E+00_JPRB, 0.91425E+00_JPRB, 0.91365E+00_JPRB, 0.10460E+01_JPRB, 0.10758E+01_JPRB ,&
&  0.11032E+01_JPRB, 0.11100E+01_JPRB, 0.11392E+01_JPRB, 0.11012E+01_JPRB, 0.99601E+00_JPRB, 0.91835E+00_JPRB, 0.86045E+00_JPRB ,&
&  0.80111E+00_JPRB, 0.76737E+00_JPRB, 0.76413E+00_JPRB, 0.78879E+00_JPRB, 0.83987E+00_JPRB, 0.93810E+00_JPRB, 0.10613E+01_JPRB ,&
&  0.11788E+01_JPRB, 0.12492E+01_JPRB, 0.12920E+01_JPRB, 0.12819E+01_JPRB, 0.12635E+01_JPRB, 0.12371E+01_JPRB, 0.12178E+01_JPRB ,&
&  0.11940E+01_JPRB, 0.11864E+01_JPRB, 0.11750E+01_JPRB, 0.11611E+01_JPRB, 0.10792E+01_JPRB, 0.94251E+00_JPRB, 0.80544E+00_JPRB ,&
&  0.70889E+00_JPRB, 0.65193E+00_JPRB, 0.61279E+00_JPRB, 0.60350E+00_JPRB, 0.60561E+00_JPRB, 0.60719E+00_JPRB, 0.59168E+00_JPRB ,&
&  0.58554E+00_JPRB, 0.59579E+00_JPRB, 0.61210E+00_JPRB, 0.62972E+00_JPRB, 0.65485E+00_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.73263E+00_JPRB, 0.74359E+00_JPRB, 0.74927E+00_JPRB, 0.74394E+00_JPRB, 0.72653E+00_JPRB, 0.68861E+00_JPRB, 0.63912E+00_JPRB ,&
&  0.62596E+00_JPRB, 0.59523E+00_JPRB, 0.58153E+00_JPRB, 0.56706E+00_JPRB, 0.55555E+00_JPRB, 0.53913E+00_JPRB, 0.52151E+00_JPRB ,&
&  0.51854E+00_JPRB, 0.52381E+00_JPRB, 0.52711E+00_JPRB, 0.52285E+00_JPRB, 0.51387E+00_JPRB, 0.50279E+00_JPRB, 0.49289E+00_JPRB ,&
&  0.48659E+00_JPRB, 0.48697E+00_JPRB, 0.49432E+00_JPRB, 0.50405E+00_JPRB, 0.50809E+00_JPRB, 0.51068E+00_JPRB, 0.51546E+00_JPRB ,&
&  0.51561E+00_JPRB, 0.51036E+00_JPRB, 0.50748E+00_JPRB, 0.51138E+00_JPRB, 0.51359E+00_JPRB, 0.52845E+00_JPRB, 0.53388E+00_JPRB ,&
&  0.50519E+00_JPRB, 0.51828E+00_JPRB, 0.56953E+00_JPRB, 0.58903E+00_JPRB, 0.60352E+00_JPRB, 0.61547E+00_JPRB, 0.62697E+00_JPRB ,&
&  0.63888E+00_JPRB, 0.65612E+00_JPRB, 0.67531E+00_JPRB, 0.69401E+00_JPRB, 0.76462E+00_JPRB, 0.92357E+00_JPRB, 0.99301E+00_JPRB ,&
&  0.10627E+01_JPRB, 0.95417E+00_JPRB, 0.89720E+00_JPRB, 0.90929E+00_JPRB, 0.10625E+01_JPRB, 0.11494E+01_JPRB, 0.11984E+01_JPRB ,&
&  0.12664E+01_JPRB, 0.12648E+01_JPRB, 0.12434E+01_JPRB, 0.11874E+01_JPRB, 0.11000E+01_JPRB, 0.10350E+01_JPRB, 0.10129E+01_JPRB ,&
&  0.97877E+00_JPRB, 0.97991E+00_JPRB, 0.10373E+01_JPRB, 0.11601E+01_JPRB, 0.13009E+01_JPRB, 0.14195E+01_JPRB, 0.15040E+01_JPRB ,&
&  0.15842E+01_JPRB, 0.16219E+01_JPRB, 0.16029E+01_JPRB, 0.15661E+01_JPRB, 0.15008E+01_JPRB, 0.14626E+01_JPRB, 0.14481E+01_JPRB ,&
&  0.14287E+01_JPRB, 0.14116E+01_JPRB, 0.13758E+01_JPRB, 0.13232E+01_JPRB, 0.12362E+01_JPRB, 0.11497E+01_JPRB, 0.10290E+01_JPRB ,&
&  0.83890E+00_JPRB, 0.72878E+00_JPRB, 0.65055E+00_JPRB, 0.62840E+00_JPRB, 0.63072E+00_JPRB, 0.63881E+00_JPRB, 0.63835E+00_JPRB ,&
&  0.65358E+00_JPRB, 0.66357E+00_JPRB, 0.66626E+00_JPRB, 0.68386E+00_JPRB, 0.70771E+00_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.78201E+00_JPRB, 0.78567E+00_JPRB, 0.78277E+00_JPRB, 0.77961E+00_JPRB, 0.76965E+00_JPRB, 0.73417E+00_JPRB, 0.70279E+00_JPRB ,&
&  0.75326E+00_JPRB, 0.71127E+00_JPRB, 0.72442E+00_JPRB, 0.62766E+00_JPRB, 0.58866E+00_JPRB, 0.57703E+00_JPRB, 0.55469E+00_JPRB ,&
&  0.54328E+00_JPRB, 0.54209E+00_JPRB, 0.53745E+00_JPRB, 0.53589E+00_JPRB, 0.53212E+00_JPRB, 0.52086E+00_JPRB, 0.51184E+00_JPRB ,&
&  0.50834E+00_JPRB, 0.51238E+00_JPRB, 0.52210E+00_JPRB, 0.52944E+00_JPRB, 0.53577E+00_JPRB, 0.54124E+00_JPRB, 0.54591E+00_JPRB ,&
&  0.53547E+00_JPRB, 0.53103E+00_JPRB, 0.53127E+00_JPRB, 0.52818E+00_JPRB, 0.53210E+00_JPRB, 0.54284E+00_JPRB, 0.56217E+00_JPRB ,&
&  0.57968E+00_JPRB, 0.56813E+00_JPRB, 0.60516E+00_JPRB, 0.62270E+00_JPRB, 0.63648E+00_JPRB, 0.64793E+00_JPRB, 0.65727E+00_JPRB ,&
&  0.66739E+00_JPRB, 0.68348E+00_JPRB, 0.70275E+00_JPRB, 0.72253E+00_JPRB, 0.84868E+00_JPRB, 0.10980E+01_JPRB, 0.11259E+01_JPRB ,&
&  0.11291E+01_JPRB, 0.96014E+00_JPRB, 0.10662E+01_JPRB, 0.11196E+01_JPRB, 0.12441E+01_JPRB, 0.13643E+01_JPRB, 0.13651E+01_JPRB ,&
&  0.15214E+01_JPRB, 0.15892E+01_JPRB, 0.15301E+01_JPRB, 0.14096E+01_JPRB, 0.13010E+01_JPRB, 0.12478E+01_JPRB, 0.12279E+01_JPRB ,&
&  0.12499E+01_JPRB, 0.12383E+01_JPRB, 0.13972E+01_JPRB, 0.15269E+01_JPRB, 0.15925E+01_JPRB, 0.16563E+01_JPRB, 0.17340E+01_JPRB ,&
&  0.17875E+01_JPRB, 0.17953E+01_JPRB, 0.17906E+01_JPRB, 0.16967E+01_JPRB, 0.16492E+01_JPRB, 0.16128E+01_JPRB, 0.15456E+01_JPRB ,&
&  0.15020E+01_JPRB, 0.14659E+01_JPRB, 0.14136E+01_JPRB, 0.14200E+01_JPRB, 0.12935E+01_JPRB, 0.12032E+01_JPRB, 0.11446E+01_JPRB ,&
&  0.99346E+00_JPRB, 0.82464E+00_JPRB, 0.76567E+00_JPRB, 0.73854E+00_JPRB, 0.71947E+00_JPRB, 0.71727E+00_JPRB, 0.72711E+00_JPRB ,&
&  0.70838E+00_JPRB, 0.71850E+00_JPRB, 0.75709E+00_JPRB, 0.76760E+00_JPRB, 0.77191E+00_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.83157E+00_JPRB, 0.83792E+00_JPRB, 0.83626E+00_JPRB, 0.83304E+00_JPRB, 0.82682E+00_JPRB, 0.81328E+00_JPRB, 0.77281E+00_JPRB ,&
&  0.79146E+00_JPRB, 0.76957E+00_JPRB, 0.73704E+00_JPRB, 0.71878E+00_JPRB, 0.72091E+00_JPRB, 0.67371E+00_JPRB, 0.58646E+00_JPRB ,&
&  0.55581E+00_JPRB, 0.55719E+00_JPRB, 0.56565E+00_JPRB, 0.56667E+00_JPRB, 0.56452E+00_JPRB, 0.55869E+00_JPRB, 0.55382E+00_JPRB ,&
&  0.55275E+00_JPRB, 0.55518E+00_JPRB, 0.56463E+00_JPRB, 0.57696E+00_JPRB, 0.57980E+00_JPRB, 0.58322E+00_JPRB, 0.58899E+00_JPRB ,&
&  0.57179E+00_JPRB, 0.55582E+00_JPRB, 0.54611E+00_JPRB, 0.53225E+00_JPRB, 0.55448E+00_JPRB, 0.57132E+00_JPRB, 0.59431E+00_JPRB ,&
&  0.61748E+00_JPRB, 0.63960E+00_JPRB, 0.66092E+00_JPRB, 0.67727E+00_JPRB, 0.68665E+00_JPRB, 0.69571E+00_JPRB, 0.70416E+00_JPRB ,&
&  0.71449E+00_JPRB, 0.73053E+00_JPRB, 0.75183E+00_JPRB, 0.77216E+00_JPRB, 0.10167E+01_JPRB, 0.12704E+01_JPRB, 0.13595E+01_JPRB ,&
&  0.13322E+01_JPRB, 0.14587E+01_JPRB, 0.15408E+01_JPRB, 0.14769E+01_JPRB, 0.15843E+01_JPRB, 0.16520E+01_JPRB, 0.16154E+01_JPRB ,&
&  0.17021E+01_JPRB, 0.17039E+01_JPRB, 0.16745E+01_JPRB, 0.16390E+01_JPRB, 0.16196E+01_JPRB, 0.15777E+01_JPRB, 0.15479E+01_JPRB ,&
&  0.15861E+01_JPRB, 0.17409E+01_JPRB, 0.17646E+01_JPRB, 0.18056E+01_JPRB, 0.18139E+01_JPRB, 0.18419E+01_JPRB, 0.18724E+01_JPRB ,&
&  0.18967E+01_JPRB, 0.19292E+01_JPRB, 0.19364E+01_JPRB, 0.18870E+01_JPRB, 0.18174E+01_JPRB, 0.17432E+01_JPRB, 0.16516E+01_JPRB ,&
&  0.15790E+01_JPRB, 0.14230E+01_JPRB, 0.12989E+01_JPRB, 0.12567E+01_JPRB, 0.12148E+01_JPRB, 0.11163E+01_JPRB, 0.10820E+01_JPRB ,&
&  0.10415E+01_JPRB, 0.96086E+00_JPRB, 0.97593E+00_JPRB, 0.91450E+00_JPRB, 0.87260E+00_JPRB, 0.82532E+00_JPRB, 0.74908E+00_JPRB ,&
&  0.72133E+00_JPRB, 0.77275E+00_JPRB, 0.80439E+00_JPRB, 0.82175E+00_JPRB, 0.82629E+00_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.89012E+00_JPRB, 0.90069E+00_JPRB, 0.90831E+00_JPRB, 0.90454E+00_JPRB, 0.88397E+00_JPRB, 0.85596E+00_JPRB, 0.82905E+00_JPRB ,&
&  0.80977E+00_JPRB, 0.78525E+00_JPRB, 0.76245E+00_JPRB, 0.74385E+00_JPRB, 0.73574E+00_JPRB, 0.73239E+00_JPRB, 0.68336E+00_JPRB ,&
&  0.61479E+00_JPRB, 0.59789E+00_JPRB, 0.58339E+00_JPRB, 0.58315E+00_JPRB, 0.59307E+00_JPRB, 0.59796E+00_JPRB, 0.60281E+00_JPRB ,&
&  0.61774E+00_JPRB, 0.62723E+00_JPRB, 0.63119E+00_JPRB, 0.63546E+00_JPRB, 0.63811E+00_JPRB, 0.64597E+00_JPRB, 0.65336E+00_JPRB ,&
&  0.62704E+00_JPRB, 0.59408E+00_JPRB, 0.57645E+00_JPRB, 0.57009E+00_JPRB, 0.59302E+00_JPRB, 0.62375E+00_JPRB, 0.65805E+00_JPRB ,&
&  0.68799E+00_JPRB, 0.71228E+00_JPRB, 0.73463E+00_JPRB, 0.75121E+00_JPRB, 0.75763E+00_JPRB, 0.76423E+00_JPRB, 0.77543E+00_JPRB ,&
&  0.79573E+00_JPRB, 0.80710E+00_JPRB, 0.81808E+00_JPRB, 0.84836E+00_JPRB, 0.12280E+01_JPRB, 0.16831E+01_JPRB, 0.19073E+01_JPRB ,&
&  0.19088E+01_JPRB, 0.19493E+01_JPRB, 0.19948E+01_JPRB, 0.21252E+01_JPRB, 0.21306E+01_JPRB, 0.20131E+01_JPRB, 0.19041E+01_JPRB ,&
&  0.19301E+01_JPRB, 0.18461E+01_JPRB, 0.19791E+01_JPRB, 0.19701E+01_JPRB, 0.19864E+01_JPRB, 0.20225E+01_JPRB, 0.20038E+01_JPRB ,&
&  0.20513E+01_JPRB, 0.20850E+01_JPRB, 0.20607E+01_JPRB, 0.19928E+01_JPRB, 0.19597E+01_JPRB, 0.19826E+01_JPRB, 0.20041E+01_JPRB ,&
&  0.20734E+01_JPRB, 0.21280E+01_JPRB, 0.20398E+01_JPRB, 0.19681E+01_JPRB, 0.18750E+01_JPRB, 0.19635E+01_JPRB, 0.18097E+01_JPRB ,&
&  0.15559E+01_JPRB, 0.12507E+01_JPRB, 0.12335E+01_JPRB, 0.12175E+01_JPRB, 0.12232E+01_JPRB, 0.11845E+01_JPRB, 0.11046E+01_JPRB ,&
&  0.11011E+01_JPRB, 0.11739E+01_JPRB, 0.11535E+01_JPRB, 0.10461E+01_JPRB, 0.99272E+00_JPRB, 0.92019E+00_JPRB, 0.79738E+00_JPRB ,&
&  0.78882E+00_JPRB, 0.85244E+00_JPRB, 0.86603E+00_JPRB, 0.86840E+00_JPRB, 0.87329E+00_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.97574E+00_JPRB, 0.96031E+00_JPRB, 0.95845E+00_JPRB, 0.94968E+00_JPRB, 0.92646E+00_JPRB, 0.89829E+00_JPRB, 0.86800E+00_JPRB ,&
&  0.84541E+00_JPRB, 0.82755E+00_JPRB, 0.81105E+00_JPRB, 0.79700E+00_JPRB, 0.78827E+00_JPRB, 0.77342E+00_JPRB, 0.77044E+00_JPRB ,&
&  0.67872E+00_JPRB, 0.64403E+00_JPRB, 0.62285E+00_JPRB, 0.62153E+00_JPRB, 0.65979E+00_JPRB, 0.65867E+00_JPRB, 0.66528E+00_JPRB ,&
&  0.67476E+00_JPRB, 0.68321E+00_JPRB, 0.67701E+00_JPRB, 0.67678E+00_JPRB, 0.68673E+00_JPRB, 0.71530E+00_JPRB, 0.74834E+00_JPRB ,&
&  0.72542E+00_JPRB, 0.67757E+00_JPRB, 0.65907E+00_JPRB, 0.65450E+00_JPRB, 0.68974E+00_JPRB, 0.71747E+00_JPRB, 0.77544E+00_JPRB ,&
&  0.80477E+00_JPRB, 0.82143E+00_JPRB, 0.83045E+00_JPRB, 0.83646E+00_JPRB, 0.84670E+00_JPRB, 0.86725E+00_JPRB, 0.89543E+00_JPRB ,&
&  0.92044E+00_JPRB, 0.91195E+00_JPRB, 0.91760E+00_JPRB, 0.99161E+00_JPRB, 0.15690E+01_JPRB, 0.22084E+01_JPRB, 0.25918E+01_JPRB ,&
&  0.27776E+01_JPRB, 0.27003E+01_JPRB, 0.27214E+01_JPRB, 0.30950E+01_JPRB, 0.30113E+01_JPRB, 0.27113E+01_JPRB, 0.24336E+01_JPRB ,&
&  0.23105E+01_JPRB, 0.22002E+01_JPRB, 0.22877E+01_JPRB, 0.23633E+01_JPRB, 0.22992E+01_JPRB, 0.22991E+01_JPRB, 0.22924E+01_JPRB ,&
&  0.23004E+01_JPRB, 0.23131E+01_JPRB, 0.22217E+01_JPRB, 0.21275E+01_JPRB, 0.20531E+01_JPRB, 0.21180E+01_JPRB, 0.21655E+01_JPRB ,&
&  0.22948E+01_JPRB, 0.22675E+01_JPRB, 0.20287E+01_JPRB, 0.20174E+01_JPRB, 0.16428E+01_JPRB, 0.15585E+01_JPRB, 0.17597E+01_JPRB ,&
&  0.14993E+01_JPRB, 0.12876E+01_JPRB, 0.12210E+01_JPRB, 0.12420E+01_JPRB, 0.12704E+01_JPRB, 0.13335E+01_JPRB, 0.12804E+01_JPRB ,&
&  0.11438E+01_JPRB, 0.11975E+01_JPRB, 0.13224E+01_JPRB, 0.12062E+01_JPRB, 0.11341E+01_JPRB, 0.10671E+01_JPRB, 0.96527E+00_JPRB ,&
&  0.98678E+00_JPRB, 0.97029E+00_JPRB, 0.96128E+00_JPRB, 0.95874E+00_JPRB, 0.96886E+00_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.10251E+01_JPRB, 0.97261E+00_JPRB, 0.98159E+00_JPRB, 0.94286E+00_JPRB, 0.92148E+00_JPRB, 0.90141E+00_JPRB, 0.88789E+00_JPRB ,&
&  0.87984E+00_JPRB, 0.86967E+00_JPRB, 0.85734E+00_JPRB, 0.84371E+00_JPRB, 0.83339E+00_JPRB, 0.81773E+00_JPRB, 0.80260E+00_JPRB ,&
&  0.79261E+00_JPRB, 0.72998E+00_JPRB, 0.71153E+00_JPRB, 0.68146E+00_JPRB, 0.71357E+00_JPRB, 0.71491E+00_JPRB, 0.72633E+00_JPRB ,&
&  0.74243E+00_JPRB, 0.76313E+00_JPRB, 0.76064E+00_JPRB, 0.76178E+00_JPRB, 0.77827E+00_JPRB, 0.82770E+00_JPRB, 0.88827E+00_JPRB ,&
&  0.89963E+00_JPRB, 0.85599E+00_JPRB, 0.83342E+00_JPRB, 0.85247E+00_JPRB, 0.92931E+00_JPRB, 0.87848E+00_JPRB, 0.91055E+00_JPRB ,&
&  0.92012E+00_JPRB, 0.91687E+00_JPRB, 0.91471E+00_JPRB, 0.92634E+00_JPRB, 0.96594E+00_JPRB, 0.10194E+01_JPRB, 0.10434E+01_JPRB ,&
&  0.10517E+01_JPRB, 0.99461E+00_JPRB, 0.96544E+00_JPRB, 0.12649E+01_JPRB, 0.20127E+01_JPRB, 0.29045E+01_JPRB, 0.32889E+01_JPRB ,&
&  0.33891E+01_JPRB, 0.33393E+01_JPRB, 0.33099E+01_JPRB, 0.36333E+01_JPRB, 0.34586E+01_JPRB, 0.31145E+01_JPRB, 0.29206E+01_JPRB ,&
&  0.27860E+01_JPRB, 0.27083E+01_JPRB, 0.27436E+01_JPRB, 0.25916E+01_JPRB, 0.24267E+01_JPRB, 0.23303E+01_JPRB, 0.22613E+01_JPRB ,&
&  0.21650E+01_JPRB, 0.21445E+01_JPRB, 0.21234E+01_JPRB, 0.20024E+01_JPRB, 0.19499E+01_JPRB, 0.19223E+01_JPRB, 0.21191E+01_JPRB ,&
&  0.23820E+01_JPRB, 0.18594E+01_JPRB, 0.15644E+01_JPRB, 0.17076E+01_JPRB, 0.13234E+01_JPRB, 0.12735E+01_JPRB, 0.14063E+01_JPRB ,&
&  0.13073E+01_JPRB, 0.12630E+01_JPRB, 0.13086E+01_JPRB, 0.12692E+01_JPRB, 0.13182E+01_JPRB, 0.14383E+01_JPRB, 0.14205E+01_JPRB ,&
&  0.13436E+01_JPRB, 0.12943E+01_JPRB, 0.15197E+01_JPRB, 0.14002E+01_JPRB, 0.13088E+01_JPRB, 0.12421E+01_JPRB, 0.11820E+01_JPRB ,&
&  0.11473E+01_JPRB, 0.11138E+01_JPRB, 0.10772E+01_JPRB, 0.10564E+01_JPRB, 0.10535E+01_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.10400E+01_JPRB, 0.98434E+00_JPRB, 0.10155E+01_JPRB, 0.99292E+00_JPRB, 0.98192E+00_JPRB, 0.97106E+00_JPRB, 0.96168E+00_JPRB ,&
&  0.93770E+00_JPRB, 0.92665E+00_JPRB, 0.91243E+00_JPRB, 0.88406E+00_JPRB, 0.86263E+00_JPRB, 0.84467E+00_JPRB, 0.83404E+00_JPRB ,&
&  0.81882E+00_JPRB, 0.82136E+00_JPRB, 0.77598E+00_JPRB, 0.76072E+00_JPRB, 0.73741E+00_JPRB, 0.76219E+00_JPRB, 0.78688E+00_JPRB ,&
&  0.82897E+00_JPRB, 0.87343E+00_JPRB, 0.91791E+00_JPRB, 0.97119E+00_JPRB, 0.10222E+01_JPRB, 0.10655E+01_JPRB, 0.11451E+01_JPRB ,&
&  0.11681E+01_JPRB, 0.11111E+01_JPRB, 0.10554E+01_JPRB, 0.10829E+01_JPRB, 0.11121E+01_JPRB, 0.10486E+01_JPRB, 0.10632E+01_JPRB ,&
&  0.10505E+01_JPRB, 0.10424E+01_JPRB, 0.10451E+01_JPRB, 0.10842E+01_JPRB, 0.10927E+01_JPRB, 0.10319E+01_JPRB, 0.97032E+00_JPRB ,&
&  0.94653E+00_JPRB, 0.97811E+00_JPRB, 0.12070E+01_JPRB, 0.17486E+01_JPRB, 0.26653E+01_JPRB, 0.37446E+01_JPRB, 0.40238E+01_JPRB ,&
&  0.40352E+01_JPRB, 0.36295E+01_JPRB, 0.28405E+01_JPRB, 0.31276E+01_JPRB, 0.37950E+01_JPRB, 0.35535E+01_JPRB, 0.31974E+01_JPRB ,&
&  0.31648E+01_JPRB, 0.29464E+01_JPRB, 0.28496E+01_JPRB, 0.25973E+01_JPRB, 0.24153E+01_JPRB, 0.22359E+01_JPRB, 0.20759E+01_JPRB ,&
&  0.19449E+01_JPRB, 0.18974E+01_JPRB, 0.18887E+01_JPRB, 0.19109E+01_JPRB, 0.19132E+01_JPRB, 0.20684E+01_JPRB, 0.23471E+01_JPRB ,&
&  0.23552E+01_JPRB, 0.21740E+01_JPRB, 0.16902E+01_JPRB, 0.16232E+01_JPRB, 0.12385E+01_JPRB, 0.11872E+01_JPRB, 0.12961E+01_JPRB ,&
&  0.12614E+01_JPRB, 0.13784E+01_JPRB, 0.15424E+01_JPRB, 0.15425E+01_JPRB, 0.17397E+01_JPRB, 0.17958E+01_JPRB, 0.17699E+01_JPRB ,&
&  0.16927E+01_JPRB, 0.15414E+01_JPRB, 0.17923E+01_JPRB, 0.16188E+01_JPRB, 0.14782E+01_JPRB, 0.13864E+01_JPRB, 0.13118E+01_JPRB ,&
&  0.12698E+01_JPRB, 0.12145E+01_JPRB, 0.11568E+01_JPRB, 0.11251E+01_JPRB, 0.10959E+01_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.10340E+01_JPRB, 0.99978E+00_JPRB, 0.10357E+01_JPRB, 0.10063E+01_JPRB, 0.10114E+01_JPRB, 0.97994E+00_JPRB, 0.96940E+00_JPRB ,&
&  0.96042E+00_JPRB, 0.94072E+00_JPRB, 0.91236E+00_JPRB, 0.88394E+00_JPRB, 0.86315E+00_JPRB, 0.84613E+00_JPRB, 0.84207E+00_JPRB ,&
&  0.83283E+00_JPRB, 0.81449E+00_JPRB, 0.74642E+00_JPRB, 0.71193E+00_JPRB, 0.69877E+00_JPRB, 0.73766E+00_JPRB, 0.81147E+00_JPRB ,&
&  0.90487E+00_JPRB, 0.99596E+00_JPRB, 0.11227E+01_JPRB, 0.12128E+01_JPRB, 0.12900E+01_JPRB, 0.13551E+01_JPRB, 0.14357E+01_JPRB ,&
&  0.15036E+01_JPRB, 0.13165E+01_JPRB, 0.12824E+01_JPRB, 0.11975E+01_JPRB, 0.11824E+01_JPRB, 0.11656E+01_JPRB, 0.11263E+01_JPRB ,&
&  0.10943E+01_JPRB, 0.10905E+01_JPRB, 0.10859E+01_JPRB, 0.10445E+01_JPRB, 0.99146E+00_JPRB, 0.95694E+00_JPRB, 0.95723E+00_JPRB ,&
&  0.10091E+01_JPRB, 0.11713E+01_JPRB, 0.17076E+01_JPRB, 0.25070E+01_JPRB, 0.37750E+01_JPRB, 0.48783E+01_JPRB, 0.48361E+01_JPRB ,&
&  0.37308E+01_JPRB, 0.32485E+01_JPRB, 0.41868E+01_JPRB, 0.36704E+01_JPRB, 0.37028E+01_JPRB, 0.39000E+01_JPRB, 0.36031E+01_JPRB ,&
&  0.31578E+01_JPRB, 0.26613E+01_JPRB, 0.25909E+01_JPRB, 0.24594E+01_JPRB, 0.31142E+01_JPRB, 0.26607E+01_JPRB, 0.23318E+01_JPRB ,&
&  0.21296E+01_JPRB, 0.20926E+01_JPRB, 0.20488E+01_JPRB, 0.21101E+01_JPRB, 0.23394E+01_JPRB, 0.27697E+01_JPRB, 0.28784E+01_JPRB ,&
&  0.25794E+01_JPRB, 0.25617E+01_JPRB, 0.21291E+01_JPRB, 0.17259E+01_JPRB, 0.14520E+01_JPRB, 0.14152E+01_JPRB, 0.14221E+01_JPRB ,&
&  0.14658E+01_JPRB, 0.15397E+01_JPRB, 0.18270E+01_JPRB, 0.21571E+01_JPRB, 0.24659E+01_JPRB, 0.24555E+01_JPRB, 0.21092E+01_JPRB ,&
&  0.20447E+01_JPRB, 0.22927E+01_JPRB, 0.22714E+01_JPRB, 0.18510E+01_JPRB, 0.16233E+01_JPRB, 0.15076E+01_JPRB, 0.14192E+01_JPRB ,&
&  0.13240E+01_JPRB, 0.12372E+01_JPRB, 0.11750E+01_JPRB, 0.11327E+01_JPRB, 0.11013E+01_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.10749E+01_JPRB, 0.10476E+01_JPRB, 0.10123E+01_JPRB, 0.99422E+00_JPRB, 0.99296E+00_JPRB, 0.94827E+00_JPRB, 0.91918E+00_JPRB ,&
&  0.89230E+00_JPRB, 0.86622E+00_JPRB, 0.87114E+00_JPRB, 0.87736E+00_JPRB, 0.87035E+00_JPRB, 0.85294E+00_JPRB, 0.84752E+00_JPRB ,&
&  0.83644E+00_JPRB, 0.77604E+00_JPRB, 0.71682E+00_JPRB, 0.72180E+00_JPRB, 0.72230E+00_JPRB, 0.72924E+00_JPRB, 0.82616E+00_JPRB ,&
&  0.97798E+00_JPRB, 0.11671E+01_JPRB, 0.13367E+01_JPRB, 0.14383E+01_JPRB, 0.14794E+01_JPRB, 0.15223E+01_JPRB, 0.15749E+01_JPRB ,&
&  0.14849E+01_JPRB, 0.14790E+01_JPRB, 0.14661E+01_JPRB, 0.13525E+01_JPRB, 0.12679E+01_JPRB, 0.11849E+01_JPRB, 0.11416E+01_JPRB ,&
&  0.11196E+01_JPRB, 0.11271E+01_JPRB, 0.10915E+01_JPRB, 0.10379E+01_JPRB, 0.10537E+01_JPRB, 0.10691E+01_JPRB, 0.10735E+01_JPRB ,&
&  0.11049E+01_JPRB, 0.14690E+01_JPRB, 0.21635E+01_JPRB, 0.30957E+01_JPRB, 0.34182E+01_JPRB, 0.44650E+01_JPRB, 0.44395E+01_JPRB ,&
&  0.49485E+01_JPRB, 0.46287E+01_JPRB, 0.43115E+01_JPRB, 0.46694E+01_JPRB, 0.38181E+01_JPRB, 0.37657E+01_JPRB, 0.36231E+01_JPRB ,&
&  0.32110E+01_JPRB, 0.30247E+01_JPRB, 0.33081E+01_JPRB, 0.30540E+01_JPRB, 0.33191E+01_JPRB, 0.36923E+01_JPRB, 0.32851E+01_JPRB ,&
&  0.30974E+01_JPRB, 0.29306E+01_JPRB, 0.27395E+01_JPRB, 0.28736E+01_JPRB, 0.28905E+01_JPRB, 0.29716E+01_JPRB, 0.25275E+01_JPRB ,&
&  0.23781E+01_JPRB, 0.19974E+01_JPRB, 0.21179E+01_JPRB, 0.18820E+01_JPRB, 0.17944E+01_JPRB, 0.17007E+01_JPRB, 0.16371E+01_JPRB ,&
&  0.17431E+01_JPRB, 0.18171E+01_JPRB, 0.23023E+01_JPRB, 0.28153E+01_JPRB, 0.33138E+01_JPRB, 0.29812E+01_JPRB, 0.26372E+01_JPRB ,&
&  0.28234E+01_JPRB, 0.29087E+01_JPRB, 0.24435E+01_JPRB, 0.19882E+01_JPRB, 0.17143E+01_JPRB, 0.15959E+01_JPRB, 0.15090E+01_JPRB ,&
&  0.14113E+01_JPRB, 0.13159E+01_JPRB, 0.12476E+01_JPRB, 0.11954E+01_JPRB, 0.11485E+01_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.10056E+01_JPRB, 0.98861E+00_JPRB, 0.94862E+00_JPRB, 0.94476E+00_JPRB, 0.95665E+00_JPRB, 0.92102E+00_JPRB, 0.89982E+00_JPRB ,&
&  0.86630E+00_JPRB, 0.84888E+00_JPRB, 0.85710E+00_JPRB, 0.87223E+00_JPRB, 0.86277E+00_JPRB, 0.85744E+00_JPRB, 0.84445E+00_JPRB ,&
&  0.83039E+00_JPRB, 0.78814E+00_JPRB, 0.73591E+00_JPRB, 0.73097E+00_JPRB, 0.76025E+00_JPRB, 0.77934E+00_JPRB, 0.85377E+00_JPRB ,&
&  0.11378E+01_JPRB, 0.13937E+01_JPRB, 0.15156E+01_JPRB, 0.15611E+01_JPRB, 0.15487E+01_JPRB, 0.15375E+01_JPRB, 0.15028E+01_JPRB ,&
&  0.14782E+01_JPRB, 0.13990E+01_JPRB, 0.13307E+01_JPRB, 0.12749E+01_JPRB, 0.12226E+01_JPRB, 0.11736E+01_JPRB, 0.11313E+01_JPRB ,&
&  0.10984E+01_JPRB, 0.10266E+01_JPRB, 0.95883E+00_JPRB, 0.91445E+00_JPRB, 0.94549E+00_JPRB, 0.93276E+00_JPRB, 0.10007E+01_JPRB ,&
&  0.12326E+01_JPRB, 0.16494E+01_JPRB, 0.22032E+01_JPRB, 0.29211E+01_JPRB, 0.34605E+01_JPRB, 0.45325E+01_JPRB, 0.51145E+01_JPRB ,&
&  0.48611E+01_JPRB, 0.43324E+01_JPRB, 0.47833E+01_JPRB, 0.53042E+01_JPRB, 0.49468E+01_JPRB, 0.41993E+01_JPRB, 0.38973E+01_JPRB ,&
&  0.29593E+01_JPRB, 0.24101E+01_JPRB, 0.22986E+01_JPRB, 0.26951E+01_JPRB, 0.41314E+01_JPRB, 0.53226E+01_JPRB, 0.50595E+01_JPRB ,&
&  0.49230E+01_JPRB, 0.45972E+01_JPRB, 0.42125E+01_JPRB, 0.36409E+01_JPRB, 0.32982E+01_JPRB, 0.30078E+01_JPRB, 0.36927E+01_JPRB ,&
&  0.35555E+01_JPRB, 0.34630E+01_JPRB, 0.28874E+01_JPRB, 0.22916E+01_JPRB, 0.18597E+01_JPRB, 0.19671E+01_JPRB, 0.19503E+01_JPRB ,&
&  0.20577E+01_JPRB, 0.22454E+01_JPRB, 0.36489E+01_JPRB, 0.42194E+01_JPRB, 0.41442E+01_JPRB, 0.39168E+01_JPRB, 0.37947E+01_JPRB ,&
&  0.34682E+01_JPRB, 0.33812E+01_JPRB, 0.27484E+01_JPRB, 0.20887E+01_JPRB, 0.17077E+01_JPRB, 0.14841E+01_JPRB, 0.13568E+01_JPRB ,&
&  0.13164E+01_JPRB, 0.12769E+01_JPRB, 0.12145E+01_JPRB, 0.11354E+01_JPRB, 0.10741E+01_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.93822E+00_JPRB, 0.92011E+00_JPRB, 0.87565E+00_JPRB, 0.87181E+00_JPRB, 0.88889E+00_JPRB, 0.90001E+00_JPRB, 0.88373E+00_JPRB ,&
&  0.86123E+00_JPRB, 0.87351E+00_JPRB, 0.88212E+00_JPRB, 0.87494E+00_JPRB, 0.85782E+00_JPRB, 0.86047E+00_JPRB, 0.85361E+00_JPRB ,&
&  0.86005E+00_JPRB, 0.88011E+00_JPRB, 0.80819E+00_JPRB, 0.81589E+00_JPRB, 0.86033E+00_JPRB, 0.91839E+00_JPRB, 0.10115E+01_JPRB ,&
&  0.13885E+01_JPRB, 0.15397E+01_JPRB, 0.15052E+01_JPRB, 0.15307E+01_JPRB, 0.15676E+01_JPRB, 0.14006E+01_JPRB, 0.13659E+01_JPRB ,&
&  0.13851E+01_JPRB, 0.13085E+01_JPRB, 0.12503E+01_JPRB, 0.12035E+01_JPRB, 0.11820E+01_JPRB, 0.11677E+01_JPRB, 0.11177E+01_JPRB ,&
&  0.10368E+01_JPRB, 0.95482E+00_JPRB, 0.92195E+00_JPRB, 0.91705E+00_JPRB, 0.86319E+00_JPRB, 0.87352E+00_JPRB, 0.96379E+00_JPRB ,&
&  0.12724E+01_JPRB, 0.16940E+01_JPRB, 0.20209E+01_JPRB, 0.25122E+01_JPRB, 0.31312E+01_JPRB, 0.43139E+01_JPRB, 0.60292E+01_JPRB ,&
&  0.55173E+01_JPRB, 0.53872E+01_JPRB, 0.52492E+01_JPRB, 0.58855E+01_JPRB, 0.49959E+01_JPRB, 0.42739E+01_JPRB, 0.38374E+01_JPRB ,&
&  0.28470E+01_JPRB, 0.24296E+01_JPRB, 0.28081E+01_JPRB, 0.37075E+01_JPRB, 0.42179E+01_JPRB, 0.49280E+01_JPRB, 0.61276E+01_JPRB ,&
&  0.55085E+01_JPRB, 0.52873E+01_JPRB, 0.50454E+01_JPRB, 0.47270E+01_JPRB, 0.28320E+01_JPRB, 0.22980E+01_JPRB, 0.32971E+01_JPRB ,&
&  0.33986E+01_JPRB, 0.21924E+01_JPRB, 0.14308E+01_JPRB, 0.17608E+01_JPRB, 0.12737E+01_JPRB, 0.13868E+01_JPRB, 0.18814E+01_JPRB ,&
&  0.23496E+01_JPRB, 0.30677E+01_JPRB, 0.51569E+01_JPRB, 0.48856E+01_JPRB, 0.46599E+01_JPRB, 0.55351E+01_JPRB, 0.47497E+01_JPRB ,&
&  0.44580E+01_JPRB, 0.32707E+01_JPRB, 0.26159E+01_JPRB, 0.18810E+01_JPRB, 0.14000E+01_JPRB, 0.11679E+01_JPRB, 0.10861E+01_JPRB ,&
&  0.10597E+01_JPRB, 0.10666E+01_JPRB, 0.10592E+01_JPRB, 0.10216E+01_JPRB, 0.97731E+00_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.96135E+00_JPRB, 0.89373E+00_JPRB, 0.81146E+00_JPRB, 0.80957E+00_JPRB, 0.82845E+00_JPRB, 0.85671E+00_JPRB, 0.84736E+00_JPRB ,&
&  0.85793E+00_JPRB, 0.88620E+00_JPRB, 0.92397E+00_JPRB, 0.92094E+00_JPRB, 0.88836E+00_JPRB, 0.88992E+00_JPRB, 0.87175E+00_JPRB ,&
&  0.87274E+00_JPRB, 0.92028E+00_JPRB, 0.96745E+00_JPRB, 0.10302E+01_JPRB, 0.11118E+01_JPRB, 0.11558E+01_JPRB, 0.13806E+01_JPRB ,&
&  0.16017E+01_JPRB, 0.15452E+01_JPRB, 0.14875E+01_JPRB, 0.15029E+01_JPRB, 0.14252E+01_JPRB, 0.12408E+01_JPRB, 0.12935E+01_JPRB ,&
&  0.13049E+01_JPRB, 0.12621E+01_JPRB, 0.11992E+01_JPRB, 0.11535E+01_JPRB, 0.11440E+01_JPRB, 0.11372E+01_JPRB, 0.10944E+01_JPRB ,&
&  0.10104E+01_JPRB, 0.98579E+00_JPRB, 0.95691E+00_JPRB, 0.86511E+00_JPRB, 0.82987E+00_JPRB, 0.86320E+00_JPRB, 0.98197E+00_JPRB ,&
&  0.12709E+01_JPRB, 0.16107E+01_JPRB, 0.18276E+01_JPRB, 0.21812E+01_JPRB, 0.34786E+01_JPRB, 0.44858E+01_JPRB, 0.46365E+01_JPRB ,&
&  0.45082E+01_JPRB, 0.45462E+01_JPRB, 0.57579E+01_JPRB, 0.57055E+01_JPRB, 0.52260E+01_JPRB, 0.49706E+01_JPRB, 0.41687E+01_JPRB ,&
&  0.36245E+01_JPRB, 0.33127E+01_JPRB, 0.36368E+01_JPRB, 0.43310E+01_JPRB, 0.51409E+01_JPRB, 0.51948E+01_JPRB, 0.63639E+01_JPRB ,&
&  0.64344E+01_JPRB, 0.60394E+01_JPRB, 0.47087E+01_JPRB, 0.35383E+01_JPRB, 0.31899E+01_JPRB, 0.23531E+01_JPRB, 0.16153E+01_JPRB ,&
&  0.14079E+01_JPRB, 0.13352E+01_JPRB, 0.11801E+01_JPRB, 0.11867E+01_JPRB, 0.10552E+01_JPRB, 0.11820E+01_JPRB, 0.16650E+01_JPRB ,&
&  0.29400E+01_JPRB, 0.43493E+01_JPRB, 0.51233E+01_JPRB, 0.45104E+01_JPRB, 0.42808E+01_JPRB, 0.54248E+01_JPRB, 0.58987E+01_JPRB ,&
&  0.34179E+01_JPRB, 0.26509E+01_JPRB, 0.18347E+01_JPRB, 0.13153E+01_JPRB, 0.10914E+01_JPRB, 0.96984E+00_JPRB, 0.93211E+00_JPRB ,&
&  0.92796E+00_JPRB, 0.92271E+00_JPRB, 0.93404E+00_JPRB, 0.95296E+00_JPRB, 0.94912E+00_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.94170E+00_JPRB, 0.84091E+00_JPRB, 0.75906E+00_JPRB, 0.74301E+00_JPRB, 0.77822E+00_JPRB, 0.79771E+00_JPRB, 0.81537E+00_JPRB ,&
&  0.85675E+00_JPRB, 0.91812E+00_JPRB, 0.95608E+00_JPRB, 0.94220E+00_JPRB, 0.92193E+00_JPRB, 0.91152E+00_JPRB, 0.89692E+00_JPRB ,&
&  0.92199E+00_JPRB, 0.10196E+01_JPRB, 0.11940E+01_JPRB, 0.13617E+01_JPRB, 0.15345E+01_JPRB, 0.14929E+01_JPRB, 0.17026E+01_JPRB ,&
&  0.16211E+01_JPRB, 0.14989E+01_JPRB, 0.15423E+01_JPRB, 0.15027E+01_JPRB, 0.14139E+01_JPRB, 0.12948E+01_JPRB, 0.13071E+01_JPRB ,&
&  0.12514E+01_JPRB, 0.12374E+01_JPRB, 0.11628E+01_JPRB, 0.11628E+01_JPRB, 0.11565E+01_JPRB, 0.11283E+01_JPRB, 0.10687E+01_JPRB ,&
&  0.10164E+01_JPRB, 0.10279E+01_JPRB, 0.90614E+00_JPRB, 0.88676E+00_JPRB, 0.89519E+00_JPRB, 0.96440E+00_JPRB, 0.10977E+01_JPRB ,&
&  0.13442E+01_JPRB, 0.16209E+01_JPRB, 0.18862E+01_JPRB, 0.25975E+01_JPRB, 0.32847E+01_JPRB, 0.32221E+01_JPRB, 0.41026E+01_JPRB ,&
&  0.48338E+01_JPRB, 0.53261E+01_JPRB, 0.53089E+01_JPRB, 0.59732E+01_JPRB, 0.57138E+01_JPRB, 0.51062E+01_JPRB, 0.45943E+01_JPRB ,&
&  0.39744E+01_JPRB, 0.42807E+01_JPRB, 0.54160E+01_JPRB, 0.59189E+01_JPRB, 0.72946E+01_JPRB, 0.69836E+01_JPRB, 0.73845E+01_JPRB ,&
&  0.76080E+01_JPRB, 0.69094E+01_JPRB, 0.55188E+01_JPRB, 0.43462E+01_JPRB, 0.46796E+01_JPRB, 0.41786E+01_JPRB, 0.22246E+01_JPRB ,&
&  0.18230E+01_JPRB, 0.15762E+01_JPRB, 0.12636E+01_JPRB, 0.10890E+01_JPRB, 0.10250E+01_JPRB, 0.12117E+01_JPRB, 0.20267E+01_JPRB ,&
&  0.26097E+01_JPRB, 0.45675E+01_JPRB, 0.53006E+01_JPRB, 0.47052E+01_JPRB, 0.41680E+01_JPRB, 0.50866E+01_JPRB, 0.41762E+01_JPRB ,&
&  0.31923E+01_JPRB, 0.22937E+01_JPRB, 0.13345E+01_JPRB, 0.10418E+01_JPRB, 0.97713E+00_JPRB, 0.93057E+00_JPRB, 0.88093E+00_JPRB ,&
&  0.83147E+00_JPRB, 0.82969E+00_JPRB, 0.85747E+00_JPRB, 0.88626E+00_JPRB, 0.89762E+00_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.82215E+00_JPRB, 0.74720E+00_JPRB, 0.71087E+00_JPRB, 0.68881E+00_JPRB, 0.73875E+00_JPRB, 0.80109E+00_JPRB, 0.81775E+00_JPRB ,&
&  0.88405E+00_JPRB, 0.95403E+00_JPRB, 0.98995E+00_JPRB, 0.98129E+00_JPRB, 0.94711E+00_JPRB, 0.94321E+00_JPRB, 0.97879E+00_JPRB ,&
&  0.10722E+01_JPRB, 0.12464E+01_JPRB, 0.14984E+01_JPRB, 0.17680E+01_JPRB, 0.18748E+01_JPRB, 0.18908E+01_JPRB, 0.18238E+01_JPRB ,&
&  0.15520E+01_JPRB, 0.14847E+01_JPRB, 0.15751E+01_JPRB, 0.15889E+01_JPRB, 0.14522E+01_JPRB, 0.14028E+01_JPRB, 0.13149E+01_JPRB ,&
&  0.12476E+01_JPRB, 0.12436E+01_JPRB, 0.12192E+01_JPRB, 0.12294E+01_JPRB, 0.11999E+01_JPRB, 0.11557E+01_JPRB, 0.11211E+01_JPRB ,&
&  0.10880E+01_JPRB, 0.97354E+00_JPRB, 0.91531E+00_JPRB, 0.92114E+00_JPRB, 0.10314E+01_JPRB, 0.12055E+01_JPRB, 0.13994E+01_JPRB ,&
&  0.16398E+01_JPRB, 0.19236E+01_JPRB, 0.23333E+01_JPRB, 0.30761E+01_JPRB, 0.28021E+01_JPRB, 0.35907E+01_JPRB, 0.41782E+01_JPRB ,&
&  0.44620E+01_JPRB, 0.46957E+01_JPRB, 0.45915E+01_JPRB, 0.51882E+01_JPRB, 0.55243E+01_JPRB, 0.50899E+01_JPRB, 0.45090E+01_JPRB ,&
&  0.46680E+01_JPRB, 0.50050E+01_JPRB, 0.71566E+01_JPRB, 0.82211E+01_JPRB, 0.91061E+01_JPRB, 0.11762E+02_JPRB, 0.86744E+01_JPRB ,&
&  0.72852E+01_JPRB, 0.75142E+01_JPRB, 0.63143E+01_JPRB, 0.50190E+01_JPRB, 0.50134E+01_JPRB, 0.55196E+01_JPRB, 0.54185E+01_JPRB ,&
&  0.24994E+01_JPRB, 0.15563E+01_JPRB, 0.14682E+01_JPRB, 0.13137E+01_JPRB, 0.11029E+01_JPRB, 0.12535E+01_JPRB, 0.30337E+01_JPRB ,&
&  0.25761E+01_JPRB, 0.42353E+01_JPRB, 0.48393E+01_JPRB, 0.42382E+01_JPRB, 0.40236E+01_JPRB, 0.40899E+01_JPRB, 0.38020E+01_JPRB ,&
&  0.23767E+01_JPRB, 0.16409E+01_JPRB, 0.11487E+01_JPRB, 0.95896E+00_JPRB, 0.92580E+00_JPRB, 0.90269E+00_JPRB, 0.85737E+00_JPRB ,&
&  0.81310E+00_JPRB, 0.78773E+00_JPRB, 0.80602E+00_JPRB, 0.79731E+00_JPRB, 0.83377E+00_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.75099E+00_JPRB, 0.72592E+00_JPRB, 0.75315E+00_JPRB, 0.77340E+00_JPRB, 0.80508E+00_JPRB, 0.83076E+00_JPRB, 0.87319E+00_JPRB ,&
&  0.95359E+00_JPRB, 0.10272E+01_JPRB, 0.10786E+01_JPRB, 0.10736E+01_JPRB, 0.10500E+01_JPRB, 0.11010E+01_JPRB, 0.12126E+01_JPRB ,&
&  0.13668E+01_JPRB, 0.15705E+01_JPRB, 0.18064E+01_JPRB, 0.19664E+01_JPRB, 0.20404E+01_JPRB, 0.18661E+01_JPRB, 0.16786E+01_JPRB ,&
&  0.14804E+01_JPRB, 0.14483E+01_JPRB, 0.14536E+01_JPRB, 0.15115E+01_JPRB, 0.15091E+01_JPRB, 0.14071E+01_JPRB, 0.13085E+01_JPRB ,&
&  0.12697E+01_JPRB, 0.12630E+01_JPRB, 0.12419E+01_JPRB, 0.12587E+01_JPRB, 0.12754E+01_JPRB, 0.12750E+01_JPRB, 0.12290E+01_JPRB ,&
&  0.11132E+01_JPRB, 0.10560E+01_JPRB, 0.10329E+01_JPRB, 0.11254E+01_JPRB, 0.13672E+01_JPRB, 0.16411E+01_JPRB, 0.19067E+01_JPRB ,&
&  0.21847E+01_JPRB, 0.24680E+01_JPRB, 0.28316E+01_JPRB, 0.30750E+01_JPRB, 0.34521E+01_JPRB, 0.37896E+01_JPRB, 0.40441E+01_JPRB ,&
&  0.40952E+01_JPRB, 0.43942E+01_JPRB, 0.46826E+01_JPRB, 0.49123E+01_JPRB, 0.51052E+01_JPRB, 0.49249E+01_JPRB, 0.46726E+01_JPRB ,&
&  0.53306E+01_JPRB, 0.66994E+01_JPRB, 0.84599E+01_JPRB, 0.92440E+01_JPRB, 0.97237E+01_JPRB, 0.11847E+02_JPRB, 0.11672E+02_JPRB ,&
&  0.90841E+01_JPRB, 0.66274E+01_JPRB, 0.56511E+01_JPRB, 0.55028E+01_JPRB, 0.56586E+01_JPRB, 0.61500E+01_JPRB, 0.82619E+01_JPRB ,&
&  0.78926E+01_JPRB, 0.36464E+01_JPRB, 0.23109E+01_JPRB, 0.24638E+01_JPRB, 0.13800E+01_JPRB, 0.11684E+01_JPRB, 0.18819E+01_JPRB ,&
&  0.25633E+01_JPRB, 0.37881E+01_JPRB, 0.44455E+01_JPRB, 0.36132E+01_JPRB, 0.35005E+01_JPRB, 0.29575E+01_JPRB, 0.24250E+01_JPRB ,&
&  0.16791E+01_JPRB, 0.12409E+01_JPRB, 0.97241E+00_JPRB, 0.89544E+00_JPRB, 0.85372E+00_JPRB, 0.86145E+00_JPRB, 0.87097E+00_JPRB ,&
&  0.81894E+00_JPRB, 0.75258E+00_JPRB, 0.75584E+00_JPRB, 0.76324E+00_JPRB, 0.75218E+00_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.78617E+00_JPRB, 0.80327E+00_JPRB, 0.86116E+00_JPRB, 0.91579E+00_JPRB, 0.95116E+00_JPRB, 0.97675E+00_JPRB, 0.99667E+00_JPRB ,&
&  0.10638E+01_JPRB, 0.11549E+01_JPRB, 0.12103E+01_JPRB, 0.12458E+01_JPRB, 0.12974E+01_JPRB, 0.13974E+01_JPRB, 0.15213E+01_JPRB ,&
&  0.16657E+01_JPRB, 0.18215E+01_JPRB, 0.19794E+01_JPRB, 0.21471E+01_JPRB, 0.23697E+01_JPRB, 0.24764E+01_JPRB, 0.15325E+01_JPRB ,&
&  0.13566E+01_JPRB, 0.15407E+01_JPRB, 0.14237E+01_JPRB, 0.15115E+01_JPRB, 0.15577E+01_JPRB, 0.15635E+01_JPRB, 0.13688E+01_JPRB ,&
&  0.12684E+01_JPRB, 0.12468E+01_JPRB, 0.12427E+01_JPRB, 0.12915E+01_JPRB, 0.13716E+01_JPRB, 0.14653E+01_JPRB, 0.14609E+01_JPRB ,&
&  0.14483E+01_JPRB, 0.14836E+01_JPRB, 0.15505E+01_JPRB, 0.17838E+01_JPRB, 0.21165E+01_JPRB, 0.23873E+01_JPRB, 0.25809E+01_JPRB ,&
&  0.27417E+01_JPRB, 0.29315E+01_JPRB, 0.32003E+01_JPRB, 0.33897E+01_JPRB, 0.37641E+01_JPRB, 0.39487E+01_JPRB, 0.40969E+01_JPRB ,&
&  0.39975E+01_JPRB, 0.38472E+01_JPRB, 0.42841E+01_JPRB, 0.46232E+01_JPRB, 0.47128E+01_JPRB, 0.46546E+01_JPRB, 0.49387E+01_JPRB ,&
&  0.65616E+01_JPRB, 0.85459E+01_JPRB, 0.10723E+02_JPRB, 0.96112E+01_JPRB, 0.92059E+01_JPRB, 0.10238E+02_JPRB, 0.11094E+02_JPRB ,&
&  0.91324E+01_JPRB, 0.70553E+01_JPRB, 0.64507E+01_JPRB, 0.59814E+01_JPRB, 0.60832E+01_JPRB, 0.61923E+01_JPRB, 0.72419E+01_JPRB ,&
&  0.78162E+01_JPRB, 0.59608E+01_JPRB, 0.34077E+01_JPRB, 0.18513E+01_JPRB, 0.12915E+01_JPRB, 0.13255E+01_JPRB, 0.16860E+01_JPRB ,&
&  0.26855E+01_JPRB, 0.33982E+01_JPRB, 0.40181E+01_JPRB, 0.41886E+01_JPRB, 0.25332E+01_JPRB, 0.18687E+01_JPRB, 0.14804E+01_JPRB ,&
&  0.10296E+01_JPRB, 0.85483E+00_JPRB, 0.84135E+00_JPRB, 0.82540E+00_JPRB, 0.79337E+00_JPRB, 0.82725E+00_JPRB, 0.88500E+00_JPRB ,&
&  0.83611E+00_JPRB, 0.74547E+00_JPRB, 0.74892E+00_JPRB, 0.75098E+00_JPRB, 0.75979E+00_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.84747E+00_JPRB, 0.88996E+00_JPRB, 0.95395E+00_JPRB, 0.10506E+01_JPRB, 0.11547E+01_JPRB, 0.12389E+01_JPRB, 0.12089E+01_JPRB ,&
&  0.11859E+01_JPRB, 0.12508E+01_JPRB, 0.13253E+01_JPRB, 0.14039E+01_JPRB, 0.14883E+01_JPRB, 0.15861E+01_JPRB, 0.16860E+01_JPRB ,&
&  0.17935E+01_JPRB, 0.19042E+01_JPRB, 0.20186E+01_JPRB, 0.21715E+01_JPRB, 0.23570E+01_JPRB, 0.24577E+01_JPRB, 0.18279E+01_JPRB ,&
&  0.13333E+01_JPRB, 0.19826E+01_JPRB, 0.14976E+01_JPRB, 0.16055E+01_JPRB, 0.16773E+01_JPRB, 0.18169E+01_JPRB, 0.15369E+01_JPRB ,&
&  0.12842E+01_JPRB, 0.12732E+01_JPRB, 0.13021E+01_JPRB, 0.13149E+01_JPRB, 0.13840E+01_JPRB, 0.15694E+01_JPRB, 0.17647E+01_JPRB ,&
&  0.18555E+01_JPRB, 0.19863E+01_JPRB, 0.21832E+01_JPRB, 0.24660E+01_JPRB, 0.26914E+01_JPRB, 0.28782E+01_JPRB, 0.30258E+01_JPRB ,&
&  0.31666E+01_JPRB, 0.33616E+01_JPRB, 0.35187E+01_JPRB, 0.37579E+01_JPRB, 0.38749E+01_JPRB, 0.40786E+01_JPRB, 0.42154E+01_JPRB ,&
&  0.41078E+01_JPRB, 0.39525E+01_JPRB, 0.41571E+01_JPRB, 0.41807E+01_JPRB, 0.44145E+01_JPRB, 0.53034E+01_JPRB, 0.59787E+01_JPRB ,&
&  0.78114E+01_JPRB, 0.88157E+01_JPRB, 0.10676E+02_JPRB, 0.85616E+01_JPRB, 0.77262E+01_JPRB, 0.80703E+01_JPRB, 0.74574E+01_JPRB ,&
&  0.62595E+01_JPRB, 0.60914E+01_JPRB, 0.58968E+01_JPRB, 0.59272E+01_JPRB, 0.60337E+01_JPRB, 0.61057E+01_JPRB, 0.62666E+01_JPRB ,&
&  0.50812E+01_JPRB, 0.27045E+01_JPRB, 0.18105E+01_JPRB, 0.13686E+01_JPRB, 0.11318E+01_JPRB, 0.12415E+01_JPRB, 0.19214E+01_JPRB ,&
&  0.30508E+01_JPRB, 0.35049E+01_JPRB, 0.39147E+01_JPRB, 0.28879E+01_JPRB, 0.15506E+01_JPRB, 0.11235E+01_JPRB, 0.87603E+00_JPRB ,&
&  0.68763E+00_JPRB, 0.66226E+00_JPRB, 0.72019E+00_JPRB, 0.73975E+00_JPRB, 0.72326E+00_JPRB, 0.75097E+00_JPRB, 0.77926E+00_JPRB ,&
&  0.76348E+00_JPRB, 0.75451E+00_JPRB, 0.79487E+00_JPRB, 0.79130E+00_JPRB, 0.81212E+00_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.90201E+00_JPRB, 0.96012E+00_JPRB, 0.10700E+01_JPRB, 0.12198E+01_JPRB, 0.14265E+01_JPRB, 0.16326E+01_JPRB, 0.16987E+01_JPRB ,&
&  0.14787E+01_JPRB, 0.12671E+01_JPRB, 0.13894E+01_JPRB, 0.15057E+01_JPRB, 0.16024E+01_JPRB, 0.17131E+01_JPRB, 0.18187E+01_JPRB ,&
&  0.19525E+01_JPRB, 0.20921E+01_JPRB, 0.22454E+01_JPRB, 0.24362E+01_JPRB, 0.26495E+01_JPRB, 0.28488E+01_JPRB, 0.27393E+01_JPRB ,&
&  0.19989E+01_JPRB, 0.16038E+01_JPRB, 0.17920E+01_JPRB, 0.16574E+01_JPRB, 0.15972E+01_JPRB, 0.17309E+01_JPRB, 0.17403E+01_JPRB ,&
&  0.15190E+01_JPRB, 0.13673E+01_JPRB, 0.14109E+01_JPRB, 0.13986E+01_JPRB, 0.12319E+01_JPRB, 0.14458E+01_JPRB, 0.17741E+01_JPRB ,&
&  0.19768E+01_JPRB, 0.21614E+01_JPRB, 0.24020E+01_JPRB, 0.26716E+01_JPRB, 0.28733E+01_JPRB, 0.31163E+01_JPRB, 0.33328E+01_JPRB ,&
&  0.35032E+01_JPRB, 0.36981E+01_JPRB, 0.37933E+01_JPRB, 0.38226E+01_JPRB, 0.39798E+01_JPRB, 0.41725E+01_JPRB, 0.42440E+01_JPRB ,&
&  0.42571E+01_JPRB, 0.44476E+01_JPRB, 0.46654E+01_JPRB, 0.49028E+01_JPRB, 0.50891E+01_JPRB, 0.61241E+01_JPRB, 0.69009E+01_JPRB ,&
&  0.77843E+01_JPRB, 0.79701E+01_JPRB, 0.87414E+01_JPRB, 0.68248E+01_JPRB, 0.61428E+01_JPRB, 0.57521E+01_JPRB, 0.53812E+01_JPRB ,&
&  0.50428E+01_JPRB, 0.52059E+01_JPRB, 0.51670E+01_JPRB, 0.53881E+01_JPRB, 0.57229E+01_JPRB, 0.54324E+01_JPRB, 0.52790E+01_JPRB ,&
&  0.31397E+01_JPRB, 0.19628E+01_JPRB, 0.13022E+01_JPRB, 0.12294E+01_JPRB, 0.12042E+01_JPRB, 0.13963E+01_JPRB, 0.18285E+01_JPRB ,&
&  0.25118E+01_JPRB, 0.22491E+01_JPRB, 0.21028E+01_JPRB, 0.18852E+01_JPRB, 0.10840E+01_JPRB, 0.77770E+00_JPRB, 0.65003E+00_JPRB ,&
&  0.59884E+00_JPRB, 0.61454E+00_JPRB, 0.65338E+00_JPRB, 0.66119E+00_JPRB, 0.64492E+00_JPRB, 0.67037E+00_JPRB, 0.71636E+00_JPRB ,&
&  0.73957E+00_JPRB, 0.76322E+00_JPRB, 0.81112E+00_JPRB, 0.82377E+00_JPRB, 0.86904E+00_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.86380E+00_JPRB, 0.95264E+00_JPRB, 0.10230E+01_JPRB, 0.10950E+01_JPRB, 0.11429E+01_JPRB, 0.12215E+01_JPRB, 0.12296E+01_JPRB ,&
&  0.11704E+01_JPRB, 0.11319E+01_JPRB, 0.12243E+01_JPRB, 0.13406E+01_JPRB, 0.14434E+01_JPRB, 0.15076E+01_JPRB, 0.15768E+01_JPRB ,&
&  0.16565E+01_JPRB, 0.17359E+01_JPRB, 0.18266E+01_JPRB, 0.19124E+01_JPRB, 0.20408E+01_JPRB, 0.21959E+01_JPRB, 0.23060E+01_JPRB ,&
&  0.19943E+01_JPRB, 0.13860E+01_JPRB, 0.16290E+01_JPRB, 0.17020E+01_JPRB, 0.15560E+01_JPRB, 0.16198E+01_JPRB, 0.17241E+01_JPRB ,&
&  0.14921E+01_JPRB, 0.13892E+01_JPRB, 0.14227E+01_JPRB, 0.14479E+01_JPRB, 0.11599E+01_JPRB, 0.12997E+01_JPRB, 0.15833E+01_JPRB ,&
&  0.18449E+01_JPRB, 0.21025E+01_JPRB, 0.23622E+01_JPRB, 0.25905E+01_JPRB, 0.28246E+01_JPRB, 0.31168E+01_JPRB, 0.33839E+01_JPRB ,&
&  0.36039E+01_JPRB, 0.37697E+01_JPRB, 0.38967E+01_JPRB, 0.40232E+01_JPRB, 0.41563E+01_JPRB, 0.43634E+01_JPRB, 0.44317E+01_JPRB ,&
&  0.45733E+01_JPRB, 0.46738E+01_JPRB, 0.51728E+01_JPRB, 0.57870E+01_JPRB, 0.62608E+01_JPRB, 0.61861E+01_JPRB, 0.66015E+01_JPRB ,&
&  0.66843E+01_JPRB, 0.64220E+01_JPRB, 0.64819E+01_JPRB, 0.64121E+01_JPRB, 0.43203E+01_JPRB, 0.39981E+01_JPRB, 0.40054E+01_JPRB ,&
&  0.44958E+01_JPRB, 0.44072E+01_JPRB, 0.44147E+01_JPRB, 0.45408E+01_JPRB, 0.45230E+01_JPRB, 0.43164E+01_JPRB, 0.37582E+01_JPRB ,&
&  0.25432E+01_JPRB, 0.16022E+01_JPRB, 0.12380E+01_JPRB, 0.12680E+01_JPRB, 0.14088E+01_JPRB, 0.19119E+01_JPRB, 0.16249E+01_JPRB ,&
&  0.18793E+01_JPRB, 0.14914E+01_JPRB, 0.14240E+01_JPRB, 0.16236E+01_JPRB, 0.10884E+01_JPRB, 0.76618E+00_JPRB, 0.63704E+00_JPRB ,&
&  0.59292E+00_JPRB, 0.60711E+00_JPRB, 0.61097E+00_JPRB, 0.60274E+00_JPRB, 0.59683E+00_JPRB, 0.62662E+00_JPRB, 0.66416E+00_JPRB ,&
&  0.68706E+00_JPRB, 0.71740E+00_JPRB, 0.75211E+00_JPRB, 0.78991E+00_JPRB, 0.84051E+00_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.70724E+00_JPRB, 0.72992E+00_JPRB, 0.76817E+00_JPRB, 0.82481E+00_JPRB, 0.86773E+00_JPRB, 0.90432E+00_JPRB, 0.91640E+00_JPRB ,&
&  0.95645E+00_JPRB, 0.99612E+00_JPRB, 0.10513E+01_JPRB, 0.10967E+01_JPRB, 0.11268E+01_JPRB, 0.11976E+01_JPRB, 0.12482E+01_JPRB ,&
&  0.12622E+01_JPRB, 0.12552E+01_JPRB, 0.13028E+01_JPRB, 0.13865E+01_JPRB, 0.15523E+01_JPRB, 0.18205E+01_JPRB, 0.21043E+01_JPRB ,&
&  0.23711E+01_JPRB, 0.23237E+01_JPRB, 0.21895E+01_JPRB, 0.14388E+01_JPRB, 0.10753E+01_JPRB, 0.12451E+01_JPRB, 0.13158E+01_JPRB ,&
&  0.13895E+01_JPRB, 0.14182E+01_JPRB, 0.12969E+01_JPRB, 0.12255E+01_JPRB, 0.11864E+01_JPRB, 0.12251E+01_JPRB, 0.13728E+01_JPRB ,&
&  0.15693E+01_JPRB, 0.18011E+01_JPRB, 0.20678E+01_JPRB, 0.22853E+01_JPRB, 0.25191E+01_JPRB, 0.28164E+01_JPRB, 0.31306E+01_JPRB ,&
&  0.33941E+01_JPRB, 0.36234E+01_JPRB, 0.38163E+01_JPRB, 0.41177E+01_JPRB, 0.43202E+01_JPRB, 0.46021E+01_JPRB, 0.48055E+01_JPRB ,&
&  0.50376E+01_JPRB, 0.51438E+01_JPRB, 0.55658E+01_JPRB, 0.60369E+01_JPRB, 0.61133E+01_JPRB, 0.55552E+01_JPRB, 0.53842E+01_JPRB ,&
&  0.53541E+01_JPRB, 0.49822E+01_JPRB, 0.48501E+01_JPRB, 0.50375E+01_JPRB, 0.33314E+01_JPRB, 0.33872E+01_JPRB, 0.38128E+01_JPRB ,&
&  0.37345E+01_JPRB, 0.37213E+01_JPRB, 0.34736E+01_JPRB, 0.29920E+01_JPRB, 0.28103E+01_JPRB, 0.27584E+01_JPRB, 0.25837E+01_JPRB ,&
&  0.21251E+01_JPRB, 0.15190E+01_JPRB, 0.12006E+01_JPRB, 0.10423E+01_JPRB, 0.12729E+01_JPRB, 0.19738E+01_JPRB, 0.14774E+01_JPRB ,&
&  0.12342E+01_JPRB, 0.13078E+01_JPRB, 0.12146E+01_JPRB, 0.17444E+01_JPRB, 0.14780E+01_JPRB, 0.94494E+00_JPRB, 0.70409E+00_JPRB ,&
&  0.59455E+00_JPRB, 0.58025E+00_JPRB, 0.58837E+00_JPRB, 0.57820E+00_JPRB, 0.58584E+00_JPRB, 0.61432E+00_JPRB, 0.61970E+00_JPRB ,&
&  0.65988E+00_JPRB, 0.67378E+00_JPRB, 0.69512E+00_JPRB, 0.71337E+00_JPRB, 0.69845E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.54454E+00_JPRB, 0.54328E+00_JPRB, 0.55630E+00_JPRB, 0.60736E+00_JPRB, 0.63765E+00_JPRB, 0.65734E+00_JPRB, 0.69716E+00_JPRB ,&
&  0.74617E+00_JPRB, 0.82497E+00_JPRB, 0.88367E+00_JPRB, 0.90480E+00_JPRB, 0.87456E+00_JPRB, 0.84540E+00_JPRB, 0.83664E+00_JPRB ,&
&  0.83638E+00_JPRB, 0.77629E+00_JPRB, 0.73624E+00_JPRB, 0.75536E+00_JPRB, 0.81914E+00_JPRB, 0.94362E+00_JPRB, 0.10989E+01_JPRB ,&
&  0.13175E+01_JPRB, 0.15316E+01_JPRB, 0.17718E+01_JPRB, 0.23585E+01_JPRB, 0.24751E+01_JPRB, 0.13029E+01_JPRB, 0.13889E+01_JPRB ,&
&  0.18171E+01_JPRB, 0.18682E+01_JPRB, 0.14275E+01_JPRB, 0.12723E+01_JPRB, 0.11980E+01_JPRB, 0.11615E+01_JPRB, 0.12046E+01_JPRB ,&
&  0.13353E+01_JPRB, 0.14770E+01_JPRB, 0.16729E+01_JPRB, 0.19226E+01_JPRB, 0.21858E+01_JPRB, 0.24238E+01_JPRB, 0.27122E+01_JPRB ,&
&  0.29633E+01_JPRB, 0.31128E+01_JPRB, 0.33089E+01_JPRB, 0.35909E+01_JPRB, 0.39706E+01_JPRB, 0.44409E+01_JPRB, 0.48697E+01_JPRB ,&
&  0.52152E+01_JPRB, 0.53883E+01_JPRB, 0.57868E+01_JPRB, 0.60269E+01_JPRB, 0.55288E+01_JPRB, 0.50019E+01_JPRB, 0.45423E+01_JPRB ,&
&  0.41174E+01_JPRB, 0.35243E+01_JPRB, 0.30846E+01_JPRB, 0.37109E+01_JPRB, 0.38156E+01_JPRB, 0.33113E+01_JPRB, 0.31581E+01_JPRB ,&
&  0.31717E+01_JPRB, 0.30859E+01_JPRB, 0.27471E+01_JPRB, 0.21615E+01_JPRB, 0.17612E+01_JPRB, 0.19255E+01_JPRB, 0.19286E+01_JPRB ,&
&  0.16975E+01_JPRB, 0.12824E+01_JPRB, 0.10588E+01_JPRB, 0.96493E+00_JPRB, 0.97169E+00_JPRB, 0.14217E+01_JPRB, 0.11095E+01_JPRB ,&
&  0.87275E+00_JPRB, 0.10643E+01_JPRB, 0.10231E+01_JPRB, 0.15331E+01_JPRB, 0.23859E+01_JPRB, 0.12463E+01_JPRB, 0.76337E+00_JPRB ,&
&  0.58339E+00_JPRB, 0.57378E+00_JPRB, 0.58558E+00_JPRB, 0.58823E+00_JPRB, 0.58687E+00_JPRB, 0.60033E+00_JPRB, 0.60189E+00_JPRB ,&
&  0.61095E+00_JPRB, 0.60208E+00_JPRB, 0.60421E+00_JPRB, 0.58124E+00_JPRB, 0.54441E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.60497E+00_JPRB, 0.60443E+00_JPRB, 0.59221E+00_JPRB, 0.58508E+00_JPRB, 0.58811E+00_JPRB, 0.59561E+00_JPRB, 0.60831E+00_JPRB ,&
&  0.62923E+00_JPRB, 0.65795E+00_JPRB, 0.68586E+00_JPRB, 0.72280E+00_JPRB, 0.73153E+00_JPRB, 0.69747E+00_JPRB, 0.70234E+00_JPRB ,&
&  0.69936E+00_JPRB, 0.71412E+00_JPRB, 0.70799E+00_JPRB, 0.79060E+00_JPRB, 0.81529E+00_JPRB, 0.82541E+00_JPRB, 0.92460E+00_JPRB ,&
&  0.91816E+00_JPRB, 0.10209E+01_JPRB, 0.10986E+01_JPRB, 0.12268E+01_JPRB, 0.20025E+01_JPRB, 0.18533E+01_JPRB, 0.13416E+01_JPRB ,&
&  0.15876E+01_JPRB, 0.16330E+01_JPRB, 0.13772E+01_JPRB, 0.11830E+01_JPRB, 0.11257E+01_JPRB, 0.10472E+01_JPRB, 0.10259E+01_JPRB ,&
&  0.10942E+01_JPRB, 0.11978E+01_JPRB, 0.13955E+01_JPRB, 0.15969E+01_JPRB, 0.17662E+01_JPRB, 0.19251E+01_JPRB, 0.21041E+01_JPRB ,&
&  0.22374E+01_JPRB, 0.23754E+01_JPRB, 0.24494E+01_JPRB, 0.25692E+01_JPRB, 0.30309E+01_JPRB, 0.34893E+01_JPRB, 0.38537E+01_JPRB ,&
&  0.41431E+01_JPRB, 0.44192E+01_JPRB, 0.48623E+01_JPRB, 0.46940E+01_JPRB, 0.42578E+01_JPRB, 0.39934E+01_JPRB, 0.33467E+01_JPRB ,&
&  0.27089E+01_JPRB, 0.20969E+01_JPRB, 0.18575E+01_JPRB, 0.22910E+01_JPRB, 0.23780E+01_JPRB, 0.20780E+01_JPRB, 0.23883E+01_JPRB ,&
&  0.24672E+01_JPRB, 0.24946E+01_JPRB, 0.21267E+01_JPRB, 0.18602E+01_JPRB, 0.15558E+01_JPRB, 0.15236E+01_JPRB, 0.15174E+01_JPRB ,&
&  0.12856E+01_JPRB, 0.10167E+01_JPRB, 0.88752E+00_JPRB, 0.82275E+00_JPRB, 0.80676E+00_JPRB, 0.10138E+01_JPRB, 0.11107E+01_JPRB ,&
&  0.10991E+01_JPRB, 0.97048E+00_JPRB, 0.98246E+00_JPRB, 0.14737E+01_JPRB, 0.13266E+01_JPRB, 0.81058E+00_JPRB, 0.58921E+00_JPRB ,&
&  0.54829E+00_JPRB, 0.55629E+00_JPRB, 0.55814E+00_JPRB, 0.55827E+00_JPRB, 0.57009E+00_JPRB, 0.58456E+00_JPRB, 0.57373E+00_JPRB ,&
&  0.57395E+00_JPRB, 0.56599E+00_JPRB, 0.56036E+00_JPRB, 0.56252E+00_JPRB, 0.57623E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.74892E+00_JPRB, 0.75396E+00_JPRB, 0.76021E+00_JPRB, 0.76953E+00_JPRB, 0.78832E+00_JPRB, 0.80890E+00_JPRB, 0.82404E+00_JPRB ,&
&  0.85592E+00_JPRB, 0.88172E+00_JPRB, 0.90856E+00_JPRB, 0.92153E+00_JPRB, 0.93974E+00_JPRB, 0.95497E+00_JPRB, 0.96124E+00_JPRB ,&
&  0.98357E+00_JPRB, 0.10097E+01_JPRB, 0.10103E+01_JPRB, 0.10205E+01_JPRB, 0.10484E+01_JPRB, 0.10668E+01_JPRB, 0.10486E+01_JPRB ,&
&  0.10260E+01_JPRB, 0.99964E+00_JPRB, 0.10212E+01_JPRB, 0.10613E+01_JPRB, 0.12609E+01_JPRB, 0.13921E+01_JPRB, 0.13800E+01_JPRB ,&
&  0.13466E+01_JPRB, 0.14099E+01_JPRB, 0.15141E+01_JPRB, 0.12282E+01_JPRB, 0.11335E+01_JPRB, 0.10896E+01_JPRB, 0.10425E+01_JPRB ,&
&  0.10610E+01_JPRB, 0.11114E+01_JPRB, 0.11733E+01_JPRB, 0.12415E+01_JPRB, 0.13287E+01_JPRB, 0.13655E+01_JPRB, 0.14304E+01_JPRB ,&
&  0.14507E+01_JPRB, 0.15029E+01_JPRB, 0.15831E+01_JPRB, 0.17621E+01_JPRB, 0.20987E+01_JPRB, 0.25338E+01_JPRB, 0.26028E+01_JPRB ,&
&  0.26493E+01_JPRB, 0.29033E+01_JPRB, 0.27098E+01_JPRB, 0.26029E+01_JPRB, 0.25107E+01_JPRB, 0.21520E+01_JPRB, 0.17936E+01_JPRB ,&
&  0.17334E+01_JPRB, 0.15747E+01_JPRB, 0.11923E+01_JPRB, 0.14745E+01_JPRB, 0.16607E+01_JPRB, 0.17039E+01_JPRB, 0.19074E+01_JPRB ,&
&  0.20123E+01_JPRB, 0.17796E+01_JPRB, 0.14724E+01_JPRB, 0.14452E+01_JPRB, 0.14218E+01_JPRB, 0.12048E+01_JPRB, 0.11546E+01_JPRB ,&
&  0.99875E+00_JPRB, 0.88847E+00_JPRB, 0.83682E+00_JPRB, 0.83925E+00_JPRB, 0.95281E+00_JPRB, 0.14060E+01_JPRB, 0.15028E+01_JPRB ,&
&  0.12209E+01_JPRB, 0.10241E+01_JPRB, 0.11812E+01_JPRB, 0.18528E+01_JPRB, 0.10335E+01_JPRB, 0.74457E+00_JPRB, 0.60345E+00_JPRB ,&
&  0.60892E+00_JPRB, 0.61359E+00_JPRB, 0.61711E+00_JPRB, 0.62031E+00_JPRB, 0.62565E+00_JPRB, 0.64327E+00_JPRB, 0.63752E+00_JPRB ,&
&  0.63313E+00_JPRB, 0.63160E+00_JPRB, 0.64893E+00_JPRB, 0.67291E+00_JPRB, 0.71610E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.85062E+00_JPRB, 0.84552E+00_JPRB, 0.85830E+00_JPRB, 0.87639E+00_JPRB, 0.89329E+00_JPRB, 0.91265E+00_JPRB, 0.94460E+00_JPRB ,&
&  0.98290E+00_JPRB, 0.10142E+01_JPRB, 0.10452E+01_JPRB, 0.10678E+01_JPRB, 0.10760E+01_JPRB, 0.10901E+01_JPRB, 0.11042E+01_JPRB ,&
&  0.11195E+01_JPRB, 0.11140E+01_JPRB, 0.11334E+01_JPRB, 0.11661E+01_JPRB, 0.12042E+01_JPRB, 0.12202E+01_JPRB, 0.12349E+01_JPRB ,&
&  0.12522E+01_JPRB, 0.12952E+01_JPRB, 0.13450E+01_JPRB, 0.14052E+01_JPRB, 0.15061E+01_JPRB, 0.16820E+01_JPRB, 0.18551E+01_JPRB ,&
&  0.14259E+01_JPRB, 0.15654E+01_JPRB, 0.15048E+01_JPRB, 0.12687E+01_JPRB, 0.11999E+01_JPRB, 0.12497E+01_JPRB, 0.12345E+01_JPRB ,&
&  0.12186E+01_JPRB, 0.12565E+01_JPRB, 0.12917E+01_JPRB, 0.13271E+01_JPRB, 0.13639E+01_JPRB, 0.14147E+01_JPRB, 0.14196E+01_JPRB ,&
&  0.14586E+01_JPRB, 0.14884E+01_JPRB, 0.15223E+01_JPRB, 0.16080E+01_JPRB, 0.16129E+01_JPRB, 0.20251E+01_JPRB, 0.19684E+01_JPRB ,&
&  0.18481E+01_JPRB, 0.18758E+01_JPRB, 0.14566E+01_JPRB, 0.14876E+01_JPRB, 0.14975E+01_JPRB, 0.12926E+01_JPRB, 0.11603E+01_JPRB ,&
&  0.14938E+01_JPRB, 0.16165E+01_JPRB, 0.12319E+01_JPRB, 0.14130E+01_JPRB, 0.15068E+01_JPRB, 0.15518E+01_JPRB, 0.15726E+01_JPRB ,&
&  0.15376E+01_JPRB, 0.13414E+01_JPRB, 0.11757E+01_JPRB, 0.10892E+01_JPRB, 0.10508E+01_JPRB, 0.99595E+00_JPRB, 0.93228E+00_JPRB ,&
&  0.86407E+00_JPRB, 0.82807E+00_JPRB, 0.79387E+00_JPRB, 0.78568E+00_JPRB, 0.11019E+01_JPRB, 0.13141E+01_JPRB, 0.19753E+01_JPRB ,&
&  0.11410E+01_JPRB, 0.98584E+00_JPRB, 0.81033E+00_JPRB, 0.11899E+01_JPRB, 0.90582E+00_JPRB, 0.66392E+00_JPRB, 0.69921E+00_JPRB ,&
&  0.75389E+00_JPRB, 0.79001E+00_JPRB, 0.77880E+00_JPRB, 0.75238E+00_JPRB, 0.74287E+00_JPRB, 0.74465E+00_JPRB, 0.72285E+00_JPRB ,&
&  0.71412E+00_JPRB, 0.73297E+00_JPRB, 0.76807E+00_JPRB, 0.78216E+00_JPRB, 0.81759E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.90027E+00_JPRB, 0.92031E+00_JPRB, 0.94238E+00_JPRB, 0.96779E+00_JPRB, 0.99237E+00_JPRB, 0.10152E+01_JPRB, 0.10415E+01_JPRB ,&
&  0.10747E+01_JPRB, 0.10998E+01_JPRB, 0.11247E+01_JPRB, 0.11398E+01_JPRB, 0.11474E+01_JPRB, 0.11629E+01_JPRB, 0.11683E+01_JPRB ,&
&  0.11717E+01_JPRB, 0.11749E+01_JPRB, 0.11942E+01_JPRB, 0.12070E+01_JPRB, 0.12113E+01_JPRB, 0.12113E+01_JPRB, 0.12310E+01_JPRB ,&
&  0.12350E+01_JPRB, 0.12732E+01_JPRB, 0.13046E+01_JPRB, 0.13447E+01_JPRB, 0.13789E+01_JPRB, 0.13943E+01_JPRB, 0.14891E+01_JPRB ,&
&  0.12154E+01_JPRB, 0.18151E+01_JPRB, 0.17976E+01_JPRB, 0.14736E+01_JPRB, 0.14218E+01_JPRB, 0.13575E+01_JPRB, 0.13609E+01_JPRB ,&
&  0.13843E+01_JPRB, 0.14154E+01_JPRB, 0.14642E+01_JPRB, 0.15371E+01_JPRB, 0.16034E+01_JPRB, 0.16637E+01_JPRB, 0.16936E+01_JPRB ,&
&  0.17422E+01_JPRB, 0.17921E+01_JPRB, 0.18457E+01_JPRB, 0.18810E+01_JPRB, 0.19427E+01_JPRB, 0.19902E+01_JPRB, 0.19467E+01_JPRB ,&
&  0.18064E+01_JPRB, 0.16756E+01_JPRB, 0.12215E+01_JPRB, 0.11927E+01_JPRB, 0.12744E+01_JPRB, 0.12106E+01_JPRB, 0.10837E+01_JPRB ,&
&  0.14091E+01_JPRB, 0.15308E+01_JPRB, 0.14262E+01_JPRB, 0.13951E+01_JPRB, 0.13657E+01_JPRB, 0.12792E+01_JPRB, 0.12271E+01_JPRB ,&
&  0.11076E+01_JPRB, 0.96322E+00_JPRB, 0.87500E+00_JPRB, 0.82082E+00_JPRB, 0.82396E+00_JPRB, 0.80929E+00_JPRB, 0.77545E+00_JPRB ,&
&  0.72699E+00_JPRB, 0.72574E+00_JPRB, 0.75828E+00_JPRB, 0.80418E+00_JPRB, 0.91265E+00_JPRB, 0.17285E+01_JPRB, 0.20041E+01_JPRB ,&
&  0.96656E+00_JPRB, 0.85791E+00_JPRB, 0.65144E+00_JPRB, 0.81787E+00_JPRB, 0.84318E+00_JPRB, 0.83358E+00_JPRB, 0.94536E+00_JPRB ,&
&  0.10421E+01_JPRB, 0.10566E+01_JPRB, 0.96904E+00_JPRB, 0.86427E+00_JPRB, 0.82344E+00_JPRB, 0.80922E+00_JPRB, 0.78606E+00_JPRB ,&
&  0.78626E+00_JPRB, 0.81283E+00_JPRB, 0.83788E+00_JPRB, 0.85780E+00_JPRB, 0.87904E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.95166E+00_JPRB, 0.97001E+00_JPRB, 0.98842E+00_JPRB, 0.10185E+01_JPRB, 0.10512E+01_JPRB, 0.10819E+01_JPRB, 0.11070E+01_JPRB ,&
&  0.11331E+01_JPRB, 0.11528E+01_JPRB, 0.11759E+01_JPRB, 0.11953E+01_JPRB, 0.12094E+01_JPRB, 0.12188E+01_JPRB, 0.12213E+01_JPRB ,&
&  0.12249E+01_JPRB, 0.12316E+01_JPRB, 0.12479E+01_JPRB, 0.12623E+01_JPRB, 0.12662E+01_JPRB, 0.12779E+01_JPRB, 0.13004E+01_JPRB ,&
&  0.13151E+01_JPRB, 0.13306E+01_JPRB, 0.13667E+01_JPRB, 0.14185E+01_JPRB, 0.14900E+01_JPRB, 0.16426E+01_JPRB, 0.15207E+01_JPRB ,&
&  0.16953E+01_JPRB, 0.19285E+01_JPRB, 0.19513E+01_JPRB, 0.16180E+01_JPRB, 0.14922E+01_JPRB, 0.14638E+01_JPRB, 0.14790E+01_JPRB ,&
&  0.15070E+01_JPRB, 0.15292E+01_JPRB, 0.15692E+01_JPRB, 0.16388E+01_JPRB, 0.17425E+01_JPRB, 0.18473E+01_JPRB, 0.19043E+01_JPRB ,&
&  0.19817E+01_JPRB, 0.20734E+01_JPRB, 0.21715E+01_JPRB, 0.22639E+01_JPRB, 0.24060E+01_JPRB, 0.24904E+01_JPRB, 0.24613E+01_JPRB ,&
&  0.23425E+01_JPRB, 0.22535E+01_JPRB, 0.19216E+01_JPRB, 0.18429E+01_JPRB, 0.19106E+01_JPRB, 0.16817E+01_JPRB, 0.14913E+01_JPRB ,&
&  0.16312E+01_JPRB, 0.13923E+01_JPRB, 0.12138E+01_JPRB, 0.13189E+01_JPRB, 0.12425E+01_JPRB, 0.10969E+01_JPRB, 0.10335E+01_JPRB ,&
&  0.88363E+00_JPRB, 0.75703E+00_JPRB, 0.67504E+00_JPRB, 0.65932E+00_JPRB, 0.67195E+00_JPRB, 0.70447E+00_JPRB, 0.70511E+00_JPRB ,&
&  0.72325E+00_JPRB, 0.76693E+00_JPRB, 0.81206E+00_JPRB, 0.88067E+00_JPRB, 0.10740E+01_JPRB, 0.10624E+01_JPRB, 0.12691E+01_JPRB ,&
&  0.96269E+00_JPRB, 0.80304E+00_JPRB, 0.87704E+00_JPRB, 0.94396E+00_JPRB, 0.10025E+01_JPRB, 0.10541E+01_JPRB, 0.12057E+01_JPRB ,&
&  0.13842E+01_JPRB, 0.15277E+01_JPRB, 0.13849E+01_JPRB, 0.11824E+01_JPRB, 0.97440E+00_JPRB, 0.88666E+00_JPRB, 0.85112E+00_JPRB ,&
&  0.84883E+00_JPRB, 0.87814E+00_JPRB, 0.90105E+00_JPRB, 0.91421E+00_JPRB, 0.93238E+00_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.96190E+00_JPRB, 0.98153E+00_JPRB, 0.99671E+00_JPRB, 0.10186E+01_JPRB, 0.10455E+01_JPRB, 0.10755E+01_JPRB, 0.10986E+01_JPRB ,&
&  0.11237E+01_JPRB, 0.11437E+01_JPRB, 0.11628E+01_JPRB, 0.11853E+01_JPRB, 0.12025E+01_JPRB, 0.12260E+01_JPRB, 0.12356E+01_JPRB ,&
&  0.12584E+01_JPRB, 0.12586E+01_JPRB, 0.12721E+01_JPRB, 0.12881E+01_JPRB, 0.13039E+01_JPRB, 0.13198E+01_JPRB, 0.13436E+01_JPRB ,&
&  0.13591E+01_JPRB, 0.13601E+01_JPRB, 0.13724E+01_JPRB, 0.13913E+01_JPRB, 0.13984E+01_JPRB, 0.14967E+01_JPRB, 0.13879E+01_JPRB ,&
&  0.18822E+01_JPRB, 0.21227E+01_JPRB, 0.20593E+01_JPRB, 0.17384E+01_JPRB, 0.16138E+01_JPRB, 0.15885E+01_JPRB, 0.16002E+01_JPRB ,&
&  0.16110E+01_JPRB, 0.16277E+01_JPRB, 0.16401E+01_JPRB, 0.16887E+01_JPRB, 0.17959E+01_JPRB, 0.19539E+01_JPRB, 0.20895E+01_JPRB ,&
&  0.21960E+01_JPRB, 0.23322E+01_JPRB, 0.24995E+01_JPRB, 0.26884E+01_JPRB, 0.29223E+01_JPRB, 0.30653E+01_JPRB, 0.31213E+01_JPRB ,&
&  0.31138E+01_JPRB, 0.30780E+01_JPRB, 0.28050E+01_JPRB, 0.29359E+01_JPRB, 0.30207E+01_JPRB, 0.24260E+01_JPRB, 0.19440E+01_JPRB ,&
&  0.15822E+01_JPRB, 0.13305E+01_JPRB, 0.13037E+01_JPRB, 0.12756E+01_JPRB, 0.11307E+01_JPRB, 0.10331E+01_JPRB, 0.94291E+00_JPRB ,&
&  0.77900E+00_JPRB, 0.69503E+00_JPRB, 0.64863E+00_JPRB, 0.60903E+00_JPRB, 0.61183E+00_JPRB, 0.63820E+00_JPRB, 0.70291E+00_JPRB ,&
&  0.75136E+00_JPRB, 0.80561E+00_JPRB, 0.87682E+00_JPRB, 0.94276E+00_JPRB, 0.10514E+01_JPRB, 0.10453E+01_JPRB, 0.11985E+01_JPRB ,&
&  0.10851E+01_JPRB, 0.10438E+01_JPRB, 0.10673E+01_JPRB, 0.85392E+00_JPRB, 0.10917E+01_JPRB, 0.11782E+01_JPRB, 0.13267E+01_JPRB ,&
&  0.13718E+01_JPRB, 0.18369E+01_JPRB, 0.21993E+01_JPRB, 0.18489E+01_JPRB, 0.13545E+01_JPRB, 0.95660E+00_JPRB, 0.86578E+00_JPRB ,&
&  0.86682E+00_JPRB, 0.89154E+00_JPRB, 0.91616E+00_JPRB, 0.92952E+00_JPRB, 0.94521E+00_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.92707E+00_JPRB, 0.94512E+00_JPRB, 0.96174E+00_JPRB, 0.97429E+00_JPRB, 0.97926E+00_JPRB, 0.99435E+00_JPRB, 0.10201E+01_JPRB ,&
&  0.10436E+01_JPRB, 0.10669E+01_JPRB, 0.10898E+01_JPRB, 0.11107E+01_JPRB, 0.11328E+01_JPRB, 0.11506E+01_JPRB, 0.11822E+01_JPRB ,&
&  0.12263E+01_JPRB, 0.12689E+01_JPRB, 0.12961E+01_JPRB, 0.13440E+01_JPRB, 0.13795E+01_JPRB, 0.14034E+01_JPRB, 0.14654E+01_JPRB ,&
&  0.15261E+01_JPRB, 0.15986E+01_JPRB, 0.16526E+01_JPRB, 0.16843E+01_JPRB, 0.16785E+01_JPRB, 0.17813E+01_JPRB, 0.16178E+01_JPRB ,&
&  0.20233E+01_JPRB, 0.22634E+01_JPRB, 0.21445E+01_JPRB, 0.18254E+01_JPRB, 0.17139E+01_JPRB, 0.16618E+01_JPRB, 0.16236E+01_JPRB ,&
&  0.16265E+01_JPRB, 0.16096E+01_JPRB, 0.15527E+01_JPRB, 0.16562E+01_JPRB, 0.17728E+01_JPRB, 0.19506E+01_JPRB, 0.21556E+01_JPRB ,&
&  0.23274E+01_JPRB, 0.25156E+01_JPRB, 0.27171E+01_JPRB, 0.29347E+01_JPRB, 0.32111E+01_JPRB, 0.34432E+01_JPRB, 0.36290E+01_JPRB ,&
&  0.37748E+01_JPRB, 0.38747E+01_JPRB, 0.39578E+01_JPRB, 0.37413E+01_JPRB, 0.37221E+01_JPRB, 0.32613E+01_JPRB, 0.24055E+01_JPRB ,&
&  0.15203E+01_JPRB, 0.12629E+01_JPRB, 0.12680E+01_JPRB, 0.12308E+01_JPRB, 0.11195E+01_JPRB, 0.95631E+00_JPRB, 0.88650E+00_JPRB ,&
&  0.74444E+00_JPRB, 0.66049E+00_JPRB, 0.63335E+00_JPRB, 0.61149E+00_JPRB, 0.60447E+00_JPRB, 0.64721E+00_JPRB, 0.68721E+00_JPRB ,&
&  0.75302E+00_JPRB, 0.84340E+00_JPRB, 0.92500E+00_JPRB, 0.10041E+01_JPRB, 0.11514E+01_JPRB, 0.12864E+01_JPRB, 0.11777E+01_JPRB ,&
&  0.11752E+01_JPRB, 0.11077E+01_JPRB, 0.11158E+01_JPRB, 0.12286E+01_JPRB, 0.11799E+01_JPRB, 0.13821E+01_JPRB, 0.16005E+01_JPRB ,&
&  0.19105E+01_JPRB, 0.18360E+01_JPRB, 0.21263E+01_JPRB, 0.31520E+01_JPRB, 0.26905E+01_JPRB, 0.22797E+01_JPRB, 0.87816E+00_JPRB ,&
&  0.87275E+00_JPRB, 0.89146E+00_JPRB, 0.91939E+00_JPRB, 0.92434E+00_JPRB, 0.92943E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.82896E+00_JPRB, 0.86605E+00_JPRB, 0.87429E+00_JPRB, 0.90539E+00_JPRB, 0.90674E+00_JPRB, 0.90013E+00_JPRB, 0.91606E+00_JPRB ,&
&  0.93531E+00_JPRB, 0.95207E+00_JPRB, 0.98056E+00_JPRB, 0.10090E+01_JPRB, 0.10315E+01_JPRB, 0.10421E+01_JPRB, 0.10633E+01_JPRB ,&
&  0.10979E+01_JPRB, 0.11434E+01_JPRB, 0.12003E+01_JPRB, 0.12766E+01_JPRB, 0.13336E+01_JPRB, 0.13835E+01_JPRB, 0.14549E+01_JPRB ,&
&  0.15164E+01_JPRB, 0.16241E+01_JPRB, 0.17381E+01_JPRB, 0.18482E+01_JPRB, 0.19187E+01_JPRB, 0.19485E+01_JPRB, 0.15574E+01_JPRB ,&
&  0.21263E+01_JPRB, 0.21853E+01_JPRB, 0.21132E+01_JPRB, 0.18633E+01_JPRB, 0.17386E+01_JPRB, 0.16559E+01_JPRB, 0.16069E+01_JPRB ,&
&  0.15820E+01_JPRB, 0.15459E+01_JPRB, 0.14981E+01_JPRB, 0.14806E+01_JPRB, 0.17188E+01_JPRB, 0.18437E+01_JPRB, 0.20387E+01_JPRB ,&
&  0.22452E+01_JPRB, 0.24381E+01_JPRB, 0.26373E+01_JPRB, 0.28559E+01_JPRB, 0.31465E+01_JPRB, 0.34766E+01_JPRB, 0.37947E+01_JPRB ,&
&  0.40812E+01_JPRB, 0.42480E+01_JPRB, 0.44829E+01_JPRB, 0.40239E+01_JPRB, 0.39103E+01_JPRB, 0.36691E+01_JPRB, 0.22923E+01_JPRB ,&
&  0.13847E+01_JPRB, 0.12700E+01_JPRB, 0.13235E+01_JPRB, 0.12658E+01_JPRB, 0.11180E+01_JPRB, 0.95124E+00_JPRB, 0.83171E+00_JPRB ,&
&  0.71717E+00_JPRB, 0.68053E+00_JPRB, 0.65133E+00_JPRB, 0.63104E+00_JPRB, 0.62473E+00_JPRB, 0.65878E+00_JPRB, 0.69294E+00_JPRB ,&
&  0.76590E+00_JPRB, 0.82611E+00_JPRB, 0.89831E+00_JPRB, 0.97078E+00_JPRB, 0.10733E+01_JPRB, 0.11988E+01_JPRB, 0.13200E+01_JPRB ,&
&  0.11301E+01_JPRB, 0.10168E+01_JPRB, 0.10263E+01_JPRB, 0.10760E+01_JPRB, 0.11070E+01_JPRB, 0.11916E+01_JPRB, 0.12950E+01_JPRB ,&
&  0.14266E+01_JPRB, 0.15209E+01_JPRB, 0.13829E+01_JPRB, 0.11547E+01_JPRB, 0.14934E+01_JPRB, 0.15341E+01_JPRB, 0.73819E+00_JPRB ,&
&  0.73118E+00_JPRB, 0.76667E+00_JPRB, 0.78913E+00_JPRB, 0.81437E+00_JPRB, 0.83783E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.68031E+00_JPRB, 0.68962E+00_JPRB, 0.69165E+00_JPRB, 0.73276E+00_JPRB, 0.74533E+00_JPRB, 0.76694E+00_JPRB, 0.78445E+00_JPRB ,&
&  0.80298E+00_JPRB, 0.83167E+00_JPRB, 0.86438E+00_JPRB, 0.89775E+00_JPRB, 0.92555E+00_JPRB, 0.94672E+00_JPRB, 0.96506E+00_JPRB ,&
&  0.98419E+00_JPRB, 0.10073E+01_JPRB, 0.10378E+01_JPRB, 0.10802E+01_JPRB, 0.11482E+01_JPRB, 0.12256E+01_JPRB, 0.13088E+01_JPRB ,&
&  0.13911E+01_JPRB, 0.14988E+01_JPRB, 0.16265E+01_JPRB, 0.17755E+01_JPRB, 0.19284E+01_JPRB, 0.20390E+01_JPRB, 0.21075E+01_JPRB ,&
&  0.16652E+01_JPRB, 0.20258E+01_JPRB, 0.19983E+01_JPRB, 0.18488E+01_JPRB, 0.17228E+01_JPRB, 0.16547E+01_JPRB, 0.16029E+01_JPRB ,&
&  0.15668E+01_JPRB, 0.14826E+01_JPRB, 0.14753E+01_JPRB, 0.15050E+01_JPRB, 0.16226E+01_JPRB, 0.17076E+01_JPRB, 0.18465E+01_JPRB ,&
&  0.20385E+01_JPRB, 0.21933E+01_JPRB, 0.23819E+01_JPRB, 0.25778E+01_JPRB, 0.28401E+01_JPRB, 0.32417E+01_JPRB, 0.37094E+01_JPRB ,&
&  0.42124E+01_JPRB, 0.46137E+01_JPRB, 0.49111E+01_JPRB, 0.43493E+01_JPRB, 0.39928E+01_JPRB, 0.36249E+01_JPRB, 0.22338E+01_JPRB ,&
&  0.14694E+01_JPRB, 0.12276E+01_JPRB, 0.12970E+01_JPRB, 0.13339E+01_JPRB, 0.11537E+01_JPRB, 0.10045E+01_JPRB, 0.86111E+00_JPRB ,&
&  0.75278E+00_JPRB, 0.69192E+00_JPRB, 0.67018E+00_JPRB, 0.64577E+00_JPRB, 0.64433E+00_JPRB, 0.66727E+00_JPRB, 0.69199E+00_JPRB ,&
&  0.74072E+00_JPRB, 0.76418E+00_JPRB, 0.82120E+00_JPRB, 0.87123E+00_JPRB, 0.92281E+00_JPRB, 0.96238E+00_JPRB, 0.99599E+00_JPRB ,&
&  0.10160E+01_JPRB, 0.10242E+01_JPRB, 0.10264E+01_JPRB, 0.10502E+01_JPRB, 0.10568E+01_JPRB, 0.10712E+01_JPRB, 0.10971E+01_JPRB ,&
&  0.11252E+01_JPRB, 0.11394E+01_JPRB, 0.11075E+01_JPRB, 0.10709E+01_JPRB, 0.10058E+01_JPRB, 0.94526E+00_JPRB, 0.72909E+00_JPRB ,&
&  0.68968E+00_JPRB, 0.64924E+00_JPRB, 0.66788E+00_JPRB, 0.69158E+00_JPRB, 0.68438E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.60240E+00_JPRB, 0.59059E+00_JPRB, 0.57773E+00_JPRB, 0.58404E+00_JPRB, 0.59173E+00_JPRB, 0.61768E+00_JPRB, 0.62048E+00_JPRB ,&
&  0.63801E+00_JPRB, 0.68947E+00_JPRB, 0.75264E+00_JPRB, 0.79009E+00_JPRB, 0.82033E+00_JPRB, 0.84987E+00_JPRB, 0.87571E+00_JPRB ,&
&  0.89050E+00_JPRB, 0.90153E+00_JPRB, 0.91231E+00_JPRB, 0.92696E+00_JPRB, 0.95589E+00_JPRB, 0.10023E+01_JPRB, 0.10759E+01_JPRB ,&
&  0.11525E+01_JPRB, 0.12376E+01_JPRB, 0.13438E+01_JPRB, 0.14777E+01_JPRB, 0.16214E+01_JPRB, 0.17064E+01_JPRB, 0.17673E+01_JPRB ,&
&  0.10830E+01_JPRB, 0.16670E+01_JPRB, 0.19948E+01_JPRB, 0.18858E+01_JPRB, 0.17376E+01_JPRB, 0.16573E+01_JPRB, 0.15883E+01_JPRB ,&
&  0.14809E+01_JPRB, 0.13712E+01_JPRB, 0.13580E+01_JPRB, 0.15160E+01_JPRB, 0.15057E+01_JPRB, 0.15524E+01_JPRB, 0.16046E+01_JPRB ,&
&  0.17470E+01_JPRB, 0.18531E+01_JPRB, 0.19164E+01_JPRB, 0.20067E+01_JPRB, 0.21584E+01_JPRB, 0.24958E+01_JPRB, 0.30722E+01_JPRB ,&
&  0.38504E+01_JPRB, 0.46767E+01_JPRB, 0.51485E+01_JPRB, 0.44315E+01_JPRB, 0.45635E+01_JPRB, 0.41585E+01_JPRB, 0.33566E+01_JPRB ,&
&  0.18952E+01_JPRB, 0.13232E+01_JPRB, 0.13521E+01_JPRB, 0.13769E+01_JPRB, 0.12154E+01_JPRB, 0.10988E+01_JPRB, 0.97125E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.76901E+00_JPRB, 0.72617E+00_JPRB, 0.69734E+00_JPRB, 0.68471E+00_JPRB, 0.68750E+00_JPRB, 0.70468E+00_JPRB ,&
&  0.71535E+00_JPRB, 0.73363E+00_JPRB, 0.76557E+00_JPRB, 0.80309E+00_JPRB, 0.85006E+00_JPRB, 0.89541E+00_JPRB, 0.94276E+00_JPRB ,&
&  0.97713E+00_JPRB, 0.10055E+01_JPRB, 0.10305E+01_JPRB, 0.10493E+01_JPRB, 0.10578E+01_JPRB, 0.10662E+01_JPRB, 0.10547E+01_JPRB ,&
&  0.10439E+01_JPRB, 0.10588E+01_JPRB, 0.10624E+01_JPRB, 0.10468E+01_JPRB, 0.99843E+00_JPRB, 0.95060E+00_JPRB, 0.90607E+00_JPRB ,&
&  0.82921E+00_JPRB, 0.72651E+00_JPRB, 0.64037E+00_JPRB, 0.61844E+00_JPRB, 0.60636E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.56723E+00_JPRB, 0.54248E+00_JPRB, 0.53527E+00_JPRB, 0.53839E+00_JPRB, 0.55190E+00_JPRB, 0.56039E+00_JPRB, 0.56038E+00_JPRB ,&
&  0.56424E+00_JPRB, 0.57719E+00_JPRB, 0.59336E+00_JPRB, 0.64516E+00_JPRB, 0.70116E+00_JPRB, 0.74418E+00_JPRB, 0.77198E+00_JPRB ,&
&  0.79400E+00_JPRB, 0.79973E+00_JPRB, 0.79556E+00_JPRB, 0.79681E+00_JPRB, 0.80009E+00_JPRB, 0.81579E+00_JPRB, 0.85190E+00_JPRB ,&
&  0.91471E+00_JPRB, 0.98476E+00_JPRB, 0.10735E+01_JPRB, 0.11928E+01_JPRB, 0.13472E+01_JPRB, 0.15023E+01_JPRB, 0.16208E+01_JPRB ,&
&  0.12283E+01_JPRB, 0.67274E+00_JPRB, 0.17283E+01_JPRB, 0.20578E+01_JPRB, 0.18286E+01_JPRB, 0.17390E+01_JPRB, 0.15901E+01_JPRB ,&
&  0.13941E+01_JPRB, 0.13266E+01_JPRB, 0.12718E+01_JPRB, 0.13457E+01_JPRB, 0.13655E+01_JPRB, 0.14133E+01_JPRB, 0.14580E+01_JPRB ,&
&  0.15399E+01_JPRB, 0.16563E+01_JPRB, 0.16799E+01_JPRB, 0.16584E+01_JPRB, 0.16687E+01_JPRB, 0.17932E+01_JPRB, 0.21719E+01_JPRB ,&
&  0.29386E+01_JPRB, 0.40805E+01_JPRB, 0.51530E+01_JPRB, 0.46429E+01_JPRB, 0.48106E+01_JPRB, 0.37516E+01_JPRB, 0.26565E+01_JPRB ,&
&  0.19195E+01_JPRB, 0.14941E+01_JPRB, 0.14293E+01_JPRB, 0.14398E+01_JPRB, 0.13094E+01_JPRB, 0.11527E+01_JPRB, 0.11147E+01_JPRB ,&
&  0.95819E+00_JPRB, 0.86178E+00_JPRB, 0.79295E+00_JPRB, 0.75407E+00_JPRB, 0.73450E+00_JPRB, 0.72988E+00_JPRB, 0.73299E+00_JPRB ,&
&  0.73219E+00_JPRB, 0.73880E+00_JPRB, 0.74992E+00_JPRB, 0.77887E+00_JPRB, 0.82002E+00_JPRB, 0.87600E+00_JPRB, 0.92135E+00_JPRB ,&
&  0.94745E+00_JPRB, 0.96690E+00_JPRB, 0.98656E+00_JPRB, 0.99764E+00_JPRB, 0.10057E+01_JPRB, 0.98383E+00_JPRB, 0.97951E+00_JPRB ,&
&  0.99072E+00_JPRB, 0.10094E+01_JPRB, 0.10305E+01_JPRB, 0.10595E+01_JPRB, 0.10234E+01_JPRB, 0.99003E+00_JPRB, 0.10021E+01_JPRB ,&
&  0.97984E+00_JPRB, 0.89852E+00_JPRB, 0.76395E+00_JPRB, 0.64070E+00_JPRB, 0.59434E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.62058E+00_JPRB, 0.59384E+00_JPRB, 0.56496E+00_JPRB, 0.55872E+00_JPRB, 0.57021E+00_JPRB, 0.56530E+00_JPRB, 0.56277E+00_JPRB ,&
&  0.55832E+00_JPRB, 0.55342E+00_JPRB, 0.53782E+00_JPRB, 0.54640E+00_JPRB, 0.57823E+00_JPRB, 0.62694E+00_JPRB, 0.66625E+00_JPRB ,&
&  0.69701E+00_JPRB, 0.70745E+00_JPRB, 0.70672E+00_JPRB, 0.70508E+00_JPRB, 0.70355E+00_JPRB, 0.70400E+00_JPRB, 0.71006E+00_JPRB ,&
&  0.73366E+00_JPRB, 0.77332E+00_JPRB, 0.82657E+00_JPRB, 0.88919E+00_JPRB, 0.97766E+00_JPRB, 0.11272E+01_JPRB, 0.13651E+01_JPRB ,&
&  0.17411E+01_JPRB, 0.12886E+01_JPRB, 0.84864E+00_JPRB, 0.17674E+01_JPRB, 0.19698E+01_JPRB, 0.19376E+01_JPRB, 0.16429E+01_JPRB ,&
&  0.15123E+01_JPRB, 0.13514E+01_JPRB, 0.12455E+01_JPRB, 0.12822E+01_JPRB, 0.12190E+01_JPRB, 0.12524E+01_JPRB, 0.13251E+01_JPRB ,&
&  0.13887E+01_JPRB, 0.14571E+01_JPRB, 0.15160E+01_JPRB, 0.15034E+01_JPRB, 0.14455E+01_JPRB, 0.14258E+01_JPRB, 0.15582E+01_JPRB ,&
&  0.20295E+01_JPRB, 0.30030E+01_JPRB, 0.45758E+01_JPRB, 0.47368E+01_JPRB, 0.49043E+01_JPRB, 0.34132E+01_JPRB, 0.24434E+01_JPRB ,&
&  0.19678E+01_JPRB, 0.18631E+01_JPRB, 0.17441E+01_JPRB, 0.15546E+01_JPRB, 0.13564E+01_JPRB, 0.12366E+01_JPRB, 0.12407E+01_JPRB ,&
&  0.10980E+01_JPRB, 0.99339E+00_JPRB, 0.90480E+00_JPRB, 0.84921E+00_JPRB, 0.81181E+00_JPRB, 0.78717E+00_JPRB, 0.77069E+00_JPRB ,&
&  0.77176E+00_JPRB, 0.77088E+00_JPRB, 0.77975E+00_JPRB, 0.79242E+00_JPRB, 0.81216E+00_JPRB, 0.84492E+00_JPRB, 0.88734E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.93322E+00_JPRB, 0.93851E+00_JPRB, 0.93582E+00_JPRB, 0.91168E+00_JPRB, 0.89612E+00_JPRB, 0.91891E+00_JPRB ,&
&  0.91949E+00_JPRB, 0.93690E+00_JPRB, 0.94322E+00_JPRB, 0.97447E+00_JPRB, 0.99574E+00_JPRB, 0.10056E+01_JPRB, 0.10288E+01_JPRB ,&
&  0.10589E+01_JPRB, 0.10308E+01_JPRB, 0.91660E+00_JPRB, 0.70914E+00_JPRB, 0.64994E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.70058E+00_JPRB, 0.67987E+00_JPRB, 0.66531E+00_JPRB, 0.66505E+00_JPRB, 0.66425E+00_JPRB, 0.64793E+00_JPRB, 0.62614E+00_JPRB ,&
&  0.59767E+00_JPRB, 0.57595E+00_JPRB, 0.54536E+00_JPRB, 0.51596E+00_JPRB, 0.50984E+00_JPRB, 0.52372E+00_JPRB, 0.55385E+00_JPRB ,&
&  0.58300E+00_JPRB, 0.59739E+00_JPRB, 0.60786E+00_JPRB, 0.61774E+00_JPRB, 0.62654E+00_JPRB, 0.63018E+00_JPRB, 0.62905E+00_JPRB ,&
&  0.63446E+00_JPRB, 0.64879E+00_JPRB, 0.67109E+00_JPRB, 0.69763E+00_JPRB, 0.73526E+00_JPRB, 0.80139E+00_JPRB, 0.93014E+00_JPRB ,&
&  0.12008E+01_JPRB, 0.13481E+01_JPRB, 0.71232E+00_JPRB, 0.13916E+01_JPRB, 0.19633E+01_JPRB, 0.19538E+01_JPRB, 0.18403E+01_JPRB ,&
&  0.16791E+01_JPRB, 0.14267E+01_JPRB, 0.13516E+01_JPRB, 0.12621E+01_JPRB, 0.11469E+01_JPRB, 0.11222E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.11579E+01_JPRB, 0.11819E+01_JPRB, 0.12422E+01_JPRB, 0.12379E+01_JPRB, 0.11942E+01_JPRB, 0.12184E+01_JPRB ,&
&  0.14478E+01_JPRB, 0.21153E+01_JPRB, 0.35849E+01_JPRB, 0.45088E+01_JPRB, 0.48260E+01_JPRB, 0.37304E+01_JPRB, 0.27461E+01_JPRB ,&
&  0.23548E+01_JPRB, 0.23836E+01_JPRB, 0.21748E+01_JPRB, 0.17887E+01_JPRB, 0.14740E+01_JPRB, 0.13729E+01_JPRB, 0.13425E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.11080E+01_JPRB, 0.10355E+01_JPRB, 0.97633E+00_JPRB, 0.92972E+00_JPRB, 0.88133E+00_JPRB, 0.84241E+00_JPRB ,&
&  0.82826E+00_JPRB, 0.81938E+00_JPRB, 0.81729E+00_JPRB, 0.81670E+00_JPRB, 0.81615E+00_JPRB, 0.82370E+00_JPRB, 0.84639E+00_JPRB ,&
&  0.87674E+00_JPRB, 0.89219E+00_JPRB, 0.89813E+00_JPRB, 0.87852E+00_JPRB, 0.86250E+00_JPRB, 0.86318E+00_JPRB, 0.88118E+00_JPRB ,&
&  0.88582E+00_JPRB, 0.89718E+00_JPRB, 0.89844E+00_JPRB, 0.90053E+00_JPRB, 0.95093E+00_JPRB, 0.96900E+00_JPRB, 0.10043E+01_JPRB ,&
&  0.10499E+01_JPRB, 0.11063E+01_JPRB, 0.10892E+01_JPRB, 0.79970E+00_JPRB, 0.74281E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.76555E+00_JPRB, 0.73975E+00_JPRB, 0.73611E+00_JPRB, 0.73729E+00_JPRB, 0.73547E+00_JPRB, 0.72634E+00_JPRB, 0.70603E+00_JPRB ,&
&  0.68019E+00_JPRB, 0.65959E+00_JPRB, 0.63520E+00_JPRB, 0.60346E+00_JPRB, 0.55799E+00_JPRB, 0.51708E+00_JPRB, 0.51100E+00_JPRB ,&
&  0.52984E+00_JPRB, 0.53752E+00_JPRB, 0.54453E+00_JPRB, 0.55397E+00_JPRB, 0.56462E+00_JPRB, 0.57702E+00_JPRB, 0.58552E+00_JPRB ,&
&  0.58918E+00_JPRB, 0.59544E+00_JPRB, 0.60604E+00_JPRB, 0.61712E+00_JPRB, 0.63389E+00_JPRB, 0.66720E+00_JPRB, 0.72997E+00_JPRB ,&
&  0.90281E+00_JPRB, 0.11488E+01_JPRB, 0.12005E+01_JPRB, 0.19758E+01_JPRB, 0.22908E+01_JPRB, 0.21601E+01_JPRB, 0.21306E+01_JPRB ,&
&  0.20524E+01_JPRB, 0.18120E+01_JPRB, 0.16022E+01_JPRB, 0.13184E+01_JPRB, 0.11581E+01_JPRB, 0.10842E+01_JPRB, 0.10511E+01_JPRB ,&
&  0.10272E+01_JPRB, 0.99360E+00_JPRB, 0.95804E+00_JPRB, 0.96771E+00_JPRB, 0.98431E+00_JPRB, 0.97933E+00_JPRB, 0.99535E+00_JPRB ,&
&  0.11051E+01_JPRB, 0.15064E+01_JPRB, 0.25515E+01_JPRB, 0.42245E+01_JPRB, 0.47794E+01_JPRB, 0.42754E+01_JPRB, 0.34868E+01_JPRB ,&
&  0.34590E+01_JPRB, 0.32304E+01_JPRB, 0.28620E+01_JPRB, 0.22335E+01_JPRB, 0.17485E+01_JPRB, 0.17002E+01_JPRB, 0.14531E+01_JPRB ,&
&  0.12998E+01_JPRB, 0.11952E+01_JPRB, 0.11313E+01_JPRB, 0.10839E+01_JPRB, 0.10442E+01_JPRB, 0.99211E+00_JPRB, 0.93116E+00_JPRB ,&
&  0.88386E+00_JPRB, 0.85973E+00_JPRB, 0.84411E+00_JPRB, 0.83700E+00_JPRB, 0.82789E+00_JPRB, 0.82021E+00_JPRB, 0.82181E+00_JPRB ,&
&  0.84058E+00_JPRB, 0.85493E+00_JPRB, 0.82960E+00_JPRB, 0.82367E+00_JPRB, 0.82897E+00_JPRB, 0.83145E+00_JPRB, 0.83387E+00_JPRB ,&
&  0.83482E+00_JPRB, 0.85763E+00_JPRB, 0.88199E+00_JPRB, 0.88653E+00_JPRB, 0.90766E+00_JPRB, 0.93423E+00_JPRB, 0.94689E+00_JPRB ,&
&  0.95692E+00_JPRB, 0.94247E+00_JPRB, 0.91447E+00_JPRB, 0.83870E+00_JPRB, 0.80262E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.79759E+00_JPRB, 0.77416E+00_JPRB, 0.76082E+00_JPRB, 0.75315E+00_JPRB, 0.74857E+00_JPRB, 0.74432E+00_JPRB, 0.72826E+00_JPRB ,&
&  0.70981E+00_JPRB, 0.69814E+00_JPRB, 0.67890E+00_JPRB, 0.65879E+00_JPRB, 0.61939E+00_JPRB, 0.56893E+00_JPRB, 0.54482E+00_JPRB ,&
&  0.53135E+00_JPRB, 0.52850E+00_JPRB, 0.53681E+00_JPRB, 0.54140E+00_JPRB, 0.54441E+00_JPRB, 0.55198E+00_JPRB, 0.55901E+00_JPRB ,&
&  0.56653E+00_JPRB, 0.56943E+00_JPRB, 0.57626E+00_JPRB, 0.58275E+00_JPRB, 0.58910E+00_JPRB, 0.61105E+00_JPRB, 0.64917E+00_JPRB ,&
&  0.76331E+00_JPRB, 0.10817E+01_JPRB, 0.95304E+00_JPRB, 0.19166E+01_JPRB, 0.21892E+01_JPRB, 0.21907E+01_JPRB, 0.20373E+01_JPRB ,&
&  0.20228E+01_JPRB, 0.20090E+01_JPRB, 0.16043E+01_JPRB, 0.13438E+01_JPRB, 0.11660E+01_JPRB, 0.10742E+01_JPRB, 0.10186E+01_JPRB ,&
&  0.97860E+00_JPRB, 0.93755E+00_JPRB, 0.88647E+00_JPRB, 0.84924E+00_JPRB, 0.83870E+00_JPRB, 0.83435E+00_JPRB, 0.84756E+00_JPRB ,&
&  0.89473E+00_JPRB, 0.11090E+01_JPRB, 0.17880E+01_JPRB, 0.32516E+01_JPRB, 0.45684E+01_JPRB, 0.47353E+01_JPRB, 0.42308E+01_JPRB ,&
&  0.42553E+01_JPRB, 0.42942E+01_JPRB, 0.38485E+01_JPRB, 0.29357E+01_JPRB, 0.22968E+01_JPRB, 0.19785E+01_JPRB, 0.16568E+01_JPRB ,&
&  0.14494E+01_JPRB, 0.13197E+01_JPRB, 0.12400E+01_JPRB, 0.11729E+01_JPRB, 0.11190E+01_JPRB, 0.10627E+01_JPRB, 0.99245E+00_JPRB ,&
&  0.92928E+00_JPRB, 0.87991E+00_JPRB, 0.85527E+00_JPRB, 0.84202E+00_JPRB, 0.83786E+00_JPRB, 0.83198E+00_JPRB, 0.82651E+00_JPRB ,&
&  0.82857E+00_JPRB, 0.84068E+00_JPRB, 0.80158E+00_JPRB, 0.78933E+00_JPRB, 0.79559E+00_JPRB, 0.79971E+00_JPRB, 0.80944E+00_JPRB ,&
&  0.83155E+00_JPRB, 0.84179E+00_JPRB, 0.85174E+00_JPRB, 0.84681E+00_JPRB, 0.86706E+00_JPRB, 0.91471E+00_JPRB, 0.92037E+00_JPRB ,&
&  0.92172E+00_JPRB, 0.90293E+00_JPRB, 0.88023E+00_JPRB, 0.84677E+00_JPRB, 0.82117E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.80585E+00_JPRB, 0.78828E+00_JPRB, 0.76655E+00_JPRB, 0.74670E+00_JPRB, 0.74254E+00_JPRB, 0.74114E+00_JPRB, 0.72819E+00_JPRB ,&
&  0.71491E+00_JPRB, 0.69885E+00_JPRB, 0.67538E+00_JPRB, 0.64858E+00_JPRB, 0.61892E+00_JPRB, 0.58290E+00_JPRB, 0.54986E+00_JPRB ,&
&  0.53451E+00_JPRB, 0.52247E+00_JPRB, 0.52407E+00_JPRB, 0.52990E+00_JPRB, 0.53828E+00_JPRB, 0.54376E+00_JPRB, 0.55060E+00_JPRB ,&
&  0.55713E+00_JPRB, 0.56084E+00_JPRB, 0.56517E+00_JPRB, 0.56887E+00_JPRB, 0.57055E+00_JPRB, 0.58110E+00_JPRB, 0.62223E+00_JPRB ,&
&  0.71243E+00_JPRB, 0.11897E+01_JPRB, 0.95741E+00_JPRB, 0.15885E+01_JPRB, 0.18278E+01_JPRB, 0.19435E+01_JPRB, 0.18291E+01_JPRB ,&
&  0.18309E+01_JPRB, 0.17379E+01_JPRB, 0.14862E+01_JPRB, 0.12955E+01_JPRB, 0.11547E+01_JPRB, 0.10632E+01_JPRB, 0.99865E+00_JPRB ,&
&  0.95357E+00_JPRB, 0.90099E+00_JPRB, 0.84331E+00_JPRB, 0.79340E+00_JPRB, 0.77045E+00_JPRB, 0.76000E+00_JPRB, 0.76235E+00_JPRB ,&
&  0.77833E+00_JPRB, 0.87724E+00_JPRB, 0.12515E+01_JPRB, 0.24294E+01_JPRB, 0.38415E+01_JPRB, 0.46828E+01_JPRB, 0.48600E+01_JPRB ,&
&  0.49750E+01_JPRB, 0.53068E+01_JPRB, 0.48111E+01_JPRB, 0.37825E+01_JPRB, 0.29785E+01_JPRB, 0.23989E+01_JPRB, 0.19568E+01_JPRB ,&
&  0.16866E+01_JPRB, 0.15238E+01_JPRB, 0.14107E+01_JPRB, 0.13164E+01_JPRB, 0.12265E+01_JPRB, 0.11267E+01_JPRB, 0.10232E+01_JPRB ,&
&  0.94393E+00_JPRB, 0.89031E+00_JPRB, 0.85748E+00_JPRB, 0.84822E+00_JPRB, 0.84711E+00_JPRB, 0.84580E+00_JPRB, 0.84510E+00_JPRB ,&
&  0.84494E+00_JPRB, 0.84821E+00_JPRB, 0.81070E+00_JPRB, 0.79419E+00_JPRB, 0.79143E+00_JPRB, 0.79775E+00_JPRB, 0.80245E+00_JPRB ,&
&  0.83285E+00_JPRB, 0.83890E+00_JPRB, 0.82961E+00_JPRB, 0.82837E+00_JPRB, 0.84639E+00_JPRB, 0.90655E+00_JPRB, 0.91474E+00_JPRB ,&
&  0.90764E+00_JPRB, 0.88284E+00_JPRB, 0.86072E+00_JPRB, 0.84488E+00_JPRB, 0.82704E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.80910E+00_JPRB, 0.78862E+00_JPRB, 0.77513E+00_JPRB, 0.76335E+00_JPRB, 0.76043E+00_JPRB, 0.75467E+00_JPRB, 0.73744E+00_JPRB ,&
&  0.71219E+00_JPRB, 0.68222E+00_JPRB, 0.65192E+00_JPRB, 0.62661E+00_JPRB, 0.60711E+00_JPRB, 0.58606E+00_JPRB, 0.56853E+00_JPRB ,&
&  0.55341E+00_JPRB, 0.53651E+00_JPRB, 0.52564E+00_JPRB, 0.52099E+00_JPRB, 0.52336E+00_JPRB, 0.52814E+00_JPRB, 0.53527E+00_JPRB ,&
&  0.54420E+00_JPRB, 0.55014E+00_JPRB, 0.55114E+00_JPRB, 0.55327E+00_JPRB, 0.55133E+00_JPRB, 0.55997E+00_JPRB, 0.60361E+00_JPRB ,&
&  0.70938E+00_JPRB, 0.67116E+00_JPRB, 0.10750E+01_JPRB, 0.12822E+01_JPRB, 0.14446E+01_JPRB, 0.16035E+01_JPRB, 0.16915E+01_JPRB ,&
&  0.17338E+01_JPRB, 0.15602E+01_JPRB, 0.13488E+01_JPRB, 0.12076E+01_JPRB, 0.10963E+01_JPRB, 0.10138E+01_JPRB, 0.94893E+00_JPRB ,&
&  0.88568E+00_JPRB, 0.83767E+00_JPRB, 0.79931E+00_JPRB, 0.75171E+00_JPRB, 0.72389E+00_JPRB, 0.71438E+00_JPRB, 0.71289E+00_JPRB ,&
&  0.70917E+00_JPRB, 0.74389E+00_JPRB, 0.92510E+00_JPRB, 0.15484E+01_JPRB, 0.30977E+01_JPRB, 0.43123E+01_JPRB, 0.51267E+01_JPRB ,&
&  0.60837E+01_JPRB, 0.64570E+01_JPRB, 0.57391E+01_JPRB, 0.46049E+01_JPRB, 0.35115E+01_JPRB, 0.27756E+01_JPRB, 0.21765E+01_JPRB ,&
&  0.18301E+01_JPRB, 0.16366E+01_JPRB, 0.15236E+01_JPRB, 0.14335E+01_JPRB, 0.13393E+01_JPRB, 0.12106E+01_JPRB, 0.10660E+01_JPRB ,&
&  0.95234E+00_JPRB, 0.88769E+00_JPRB, 0.84815E+00_JPRB, 0.82613E+00_JPRB, 0.81602E+00_JPRB, 0.81765E+00_JPRB, 0.82706E+00_JPRB ,&
&  0.84005E+00_JPRB, 0.85314E+00_JPRB, 0.82033E+00_JPRB, 0.79830E+00_JPRB, 0.79576E+00_JPRB, 0.80816E+00_JPRB, 0.82009E+00_JPRB ,&
&  0.82112E+00_JPRB, 0.80972E+00_JPRB, 0.79534E+00_JPRB, 0.79769E+00_JPRB, 0.81414E+00_JPRB, 0.89610E+00_JPRB, 0.90857E+00_JPRB ,&
&  0.90192E+00_JPRB, 0.87987E+00_JPRB, 0.84947E+00_JPRB, 0.83939E+00_JPRB, 0.83284E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.80747E+00_JPRB, 0.78975E+00_JPRB, 0.78767E+00_JPRB, 0.79905E+00_JPRB, 0.80940E+00_JPRB, 0.79583E+00_JPRB, 0.75192E+00_JPRB ,&
&  0.71069E+00_JPRB, 0.67512E+00_JPRB, 0.64216E+00_JPRB, 0.61615E+00_JPRB, 0.59885E+00_JPRB, 0.58635E+00_JPRB, 0.57376E+00_JPRB ,&
&  0.56437E+00_JPRB, 0.55604E+00_JPRB, 0.53941E+00_JPRB, 0.52402E+00_JPRB, 0.51693E+00_JPRB, 0.51559E+00_JPRB, 0.51946E+00_JPRB ,&
&  0.52416E+00_JPRB, 0.52967E+00_JPRB, 0.53347E+00_JPRB, 0.53605E+00_JPRB, 0.53563E+00_JPRB, 0.54511E+00_JPRB, 0.58358E+00_JPRB ,&
&  0.70452E+00_JPRB, 0.93465E+00_JPRB, 0.10805E+01_JPRB, 0.10929E+01_JPRB, 0.11855E+01_JPRB, 0.13543E+01_JPRB, 0.15384E+01_JPRB ,&
&  0.15290E+01_JPRB, 0.13969E+01_JPRB, 0.12145E+01_JPRB, 0.10882E+01_JPRB, 0.99308E+00_JPRB, 0.92577E+00_JPRB, 0.85864E+00_JPRB ,&
&  0.80314E+00_JPRB, 0.76549E+00_JPRB, 0.72804E+00_JPRB, 0.69496E+00_JPRB, 0.68360E+00_JPRB, 0.68565E+00_JPRB, 0.68789E+00_JPRB ,&
&  0.68568E+00_JPRB, 0.68691E+00_JPRB, 0.76382E+00_JPRB, 0.11729E+01_JPRB, 0.24496E+01_JPRB, 0.33098E+01_JPRB, 0.48118E+01_JPRB ,&
&  0.63119E+01_JPRB, 0.62793E+01_JPRB, 0.56529E+01_JPRB, 0.47041E+01_JPRB, 0.36651E+01_JPRB, 0.27929E+01_JPRB, 0.22420E+01_JPRB ,&
&  0.18966E+01_JPRB, 0.16269E+01_JPRB, 0.14649E+01_JPRB, 0.13623E+01_JPRB, 0.12785E+01_JPRB, 0.11988E+01_JPRB, 0.11020E+01_JPRB ,&
&  0.10015E+01_JPRB, 0.91807E+00_JPRB, 0.85980E+00_JPRB, 0.82016E+00_JPRB, 0.79967E+00_JPRB, 0.79502E+00_JPRB, 0.79581E+00_JPRB ,&
&  0.79309E+00_JPRB, 0.80329E+00_JPRB, 0.80758E+00_JPRB, 0.79652E+00_JPRB, 0.79993E+00_JPRB, 0.81327E+00_JPRB, 0.82078E+00_JPRB ,&
&  0.79797E+00_JPRB, 0.77869E+00_JPRB, 0.77533E+00_JPRB, 0.77284E+00_JPRB, 0.80545E+00_JPRB, 0.89373E+00_JPRB, 0.89415E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.86998E+00_JPRB, 0.85204E+00_JPRB, 0.83186E+00_JPRB, 0.83187E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.79787E+00_JPRB, 0.77840E+00_JPRB, 0.77081E+00_JPRB, 0.78921E+00_JPRB, 0.81152E+00_JPRB, 0.80478E+00_JPRB, 0.76373E+00_JPRB ,&
&  0.70844E+00_JPRB, 0.66391E+00_JPRB, 0.63284E+00_JPRB, 0.61094E+00_JPRB, 0.59422E+00_JPRB, 0.57983E+00_JPRB, 0.56426E+00_JPRB ,&
&  0.55151E+00_JPRB, 0.54427E+00_JPRB, 0.53903E+00_JPRB, 0.53220E+00_JPRB, 0.52018E+00_JPRB, 0.51078E+00_JPRB, 0.50749E+00_JPRB ,&
&  0.50573E+00_JPRB, 0.50581E+00_JPRB, 0.51100E+00_JPRB, 0.51702E+00_JPRB, 0.51929E+00_JPRB, 0.52475E+00_JPRB, 0.55028E+00_JPRB ,&
&  0.63966E+00_JPRB, 0.83670E+00_JPRB, 0.83752E+00_JPRB, 0.90850E+00_JPRB, 0.10407E+01_JPRB, 0.11875E+01_JPRB, 0.13163E+01_JPRB ,&
&  0.13207E+01_JPRB, 0.12514E+01_JPRB, 0.11233E+01_JPRB, 0.99485E+00_JPRB, 0.89161E+00_JPRB, 0.83042E+00_JPRB, 0.78694E+00_JPRB ,&
&  0.74733E+00_JPRB, 0.70749E+00_JPRB, 0.67280E+00_JPRB, 0.64680E+00_JPRB, 0.62957E+00_JPRB, 0.62402E+00_JPRB, 0.62682E+00_JPRB ,&
&  0.62976E+00_JPRB, 0.63046E+00_JPRB, 0.64652E+00_JPRB, 0.79183E+00_JPRB, 0.14566E+01_JPRB, 0.23875E+01_JPRB, 0.34238E+01_JPRB ,&
&  0.37768E+01_JPRB, 0.39614E+01_JPRB, 0.39262E+01_JPRB, 0.36791E+01_JPRB, 0.32245E+01_JPRB, 0.27724E+01_JPRB, 0.22843E+01_JPRB ,&
&  0.19242E+01_JPRB, 0.16817E+01_JPRB, 0.15064E+01_JPRB, 0.13582E+01_JPRB, 0.12253E+01_JPRB, 0.11188E+01_JPRB, 0.10443E+01_JPRB ,&
&  0.99024E+00_JPRB, 0.94357E+00_JPRB, 0.89955E+00_JPRB, 0.85941E+00_JPRB, 0.82927E+00_JPRB, 0.81179E+00_JPRB, 0.80878E+00_JPRB ,&
&  0.80232E+00_JPRB, 0.78556E+00_JPRB, 0.78079E+00_JPRB, 0.78175E+00_JPRB, 0.78690E+00_JPRB, 0.78661E+00_JPRB, 0.78278E+00_JPRB ,&
&  0.78487E+00_JPRB, 0.78500E+00_JPRB, 0.78127E+00_JPRB, 0.77749E+00_JPRB, 0.80141E+00_JPRB, 0.86546E+00_JPRB, 0.88247E+00_JPRB ,&
&  0.88072E+00_JPRB, 0.86317E+00_JPRB, 0.84885E+00_JPRB, 0.83761E+00_JPRB, 0.82954E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.88386E+00_JPRB, 0.80382E+00_JPRB, 0.75792E+00_JPRB, 0.74472E+00_JPRB, 0.74896E+00_JPRB, 0.73753E+00_JPRB, 0.71270E+00_JPRB ,&
&  0.68539E+00_JPRB, 0.66131E+00_JPRB, 0.64029E+00_JPRB, 0.61682E+00_JPRB, 0.59367E+00_JPRB, 0.57371E+00_JPRB, 0.55496E+00_JPRB ,&
&  0.53728E+00_JPRB, 0.52681E+00_JPRB, 0.51950E+00_JPRB, 0.51457E+00_JPRB, 0.51094E+00_JPRB, 0.50414E+00_JPRB, 0.49683E+00_JPRB ,&
&  0.49123E+00_JPRB, 0.48335E+00_JPRB, 0.48093E+00_JPRB, 0.48741E+00_JPRB, 0.49386E+00_JPRB, 0.49957E+00_JPRB, 0.52085E+00_JPRB ,&
&  0.59373E+00_JPRB, 0.66174E+00_JPRB, 0.73270E+00_JPRB, 0.79618E+00_JPRB, 0.85991E+00_JPRB, 0.94164E+00_JPRB, 0.97400E+00_JPRB ,&
&  0.94826E+00_JPRB, 0.95515E+00_JPRB, 0.93498E+00_JPRB, 0.86724E+00_JPRB, 0.79172E+00_JPRB, 0.74152E+00_JPRB, 0.70647E+00_JPRB ,&
&  0.67708E+00_JPRB, 0.64517E+00_JPRB, 0.61255E+00_JPRB, 0.58669E+00_JPRB, 0.57162E+00_JPRB, 0.56645E+00_JPRB, 0.56718E+00_JPRB ,&
&  0.57027E+00_JPRB, 0.57566E+00_JPRB, 0.58152E+00_JPRB, 0.60427E+00_JPRB, 0.75788E+00_JPRB, 0.11676E+01_JPRB, 0.18071E+01_JPRB ,&
&  0.21347E+01_JPRB, 0.22577E+01_JPRB, 0.23540E+01_JPRB, 0.23753E+01_JPRB, 0.22220E+01_JPRB, 0.20487E+01_JPRB, 0.18560E+01_JPRB ,&
&  0.16631E+01_JPRB, 0.14614E+01_JPRB, 0.12898E+01_JPRB, 0.11552E+01_JPRB, 0.10507E+01_JPRB, 0.97854E+00_JPRB, 0.93832E+00_JPRB ,&
&  0.92048E+00_JPRB, 0.91092E+00_JPRB, 0.90210E+00_JPRB, 0.88322E+00_JPRB, 0.85870E+00_JPRB, 0.84053E+00_JPRB, 0.82881E+00_JPRB ,&
&  0.81372E+00_JPRB, 0.79879E+00_JPRB, 0.78615E+00_JPRB, 0.77701E+00_JPRB, 0.77221E+00_JPRB, 0.76682E+00_JPRB, 0.76469E+00_JPRB ,&
&  0.76621E+00_JPRB, 0.77403E+00_JPRB, 0.76184E+00_JPRB, 0.75951E+00_JPRB, 0.81147E+00_JPRB, 0.83104E+00_JPRB, 0.85193E+00_JPRB ,&
&  0.86947E+00_JPRB, 0.86111E+00_JPRB, 0.83834E+00_JPRB, 0.83229E+00_JPRB, 0.93989E+00_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.79270E+00_JPRB, 0.75972E+00_JPRB, 0.71528E+00_JPRB, 0.68920E+00_JPRB, 0.67187E+00_JPRB, 0.64908E+00_JPRB, 0.63175E+00_JPRB ,&
&  0.61771E+00_JPRB, 0.60476E+00_JPRB, 0.58874E+00_JPRB, 0.56965E+00_JPRB, 0.55126E+00_JPRB, 0.53814E+00_JPRB, 0.52706E+00_JPRB ,&
&  0.51667E+00_JPRB, 0.50676E+00_JPRB, 0.49754E+00_JPRB, 0.49053E+00_JPRB, 0.48283E+00_JPRB, 0.47684E+00_JPRB, 0.47168E+00_JPRB ,&
&  0.46562E+00_JPRB, 0.45901E+00_JPRB, 0.45067E+00_JPRB, 0.45418E+00_JPRB, 0.46363E+00_JPRB, 0.47351E+00_JPRB, 0.48652E+00_JPRB ,&
&  0.53363E+00_JPRB, 0.55604E+00_JPRB, 0.61780E+00_JPRB, 0.68598E+00_JPRB, 0.73366E+00_JPRB, 0.77183E+00_JPRB, 0.77692E+00_JPRB ,&
&  0.75334E+00_JPRB, 0.76271E+00_JPRB, 0.75019E+00_JPRB, 0.72007E+00_JPRB, 0.69682E+00_JPRB, 0.65112E+00_JPRB, 0.61053E+00_JPRB ,&
&  0.58224E+00_JPRB, 0.56419E+00_JPRB, 0.54835E+00_JPRB, 0.53141E+00_JPRB, 0.52128E+00_JPRB, 0.51941E+00_JPRB, 0.52343E+00_JPRB ,&
&  0.52765E+00_JPRB, 0.53388E+00_JPRB, 0.54021E+00_JPRB, 0.54897E+00_JPRB, 0.57352E+00_JPRB, 0.69497E+00_JPRB, 0.10114E+01_JPRB ,&
&  0.12556E+01_JPRB, 0.14080E+01_JPRB, 0.15124E+01_JPRB, 0.15586E+01_JPRB, 0.15162E+01_JPRB, 0.14265E+01_JPRB, 0.13147E+01_JPRB ,&
&  0.11958E+01_JPRB, 0.10708E+01_JPRB, 0.95757E+00_JPRB, 0.87000E+00_JPRB, 0.80300E+00_JPRB, 0.75947E+00_JPRB, 0.73818E+00_JPRB ,&
&  0.73809E+00_JPRB, 0.75283E+00_JPRB, 0.77354E+00_JPRB, 0.79281E+00_JPRB, 0.80280E+00_JPRB, 0.80187E+00_JPRB, 0.79539E+00_JPRB ,&
&  0.78368E+00_JPRB, 0.76832E+00_JPRB, 0.75140E+00_JPRB, 0.73719E+00_JPRB, 0.72609E+00_JPRB, 0.71818E+00_JPRB, 0.71351E+00_JPRB ,&
&  0.71140E+00_JPRB, 0.71262E+00_JPRB, 0.71917E+00_JPRB, 0.73671E+00_JPRB, 0.74473E+00_JPRB, 0.76046E+00_JPRB, 0.77087E+00_JPRB ,&
&  0.78276E+00_JPRB, 0.78505E+00_JPRB, 0.77393E+00_JPRB, 0.74733E+00_JPRB, 0.75740E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.68722E+00_JPRB, 0.67835E+00_JPRB, 0.65568E+00_JPRB, 0.63097E+00_JPRB, 0.61005E+00_JPRB, 0.59063E+00_JPRB, 0.57379E+00_JPRB ,&
&  0.55981E+00_JPRB, 0.54700E+00_JPRB, 0.53305E+00_JPRB, 0.51992E+00_JPRB, 0.50700E+00_JPRB, 0.49659E+00_JPRB, 0.48676E+00_JPRB ,&
&  0.48002E+00_JPRB, 0.47512E+00_JPRB, 0.47019E+00_JPRB, 0.46424E+00_JPRB, 0.45806E+00_JPRB, 0.45194E+00_JPRB, 0.44523E+00_JPRB ,&
&  0.43986E+00_JPRB, 0.43689E+00_JPRB, 0.42984E+00_JPRB, 0.42757E+00_JPRB, 0.43443E+00_JPRB, 0.44421E+00_JPRB, 0.45492E+00_JPRB ,&
&  0.48928E+00_JPRB, 0.46481E+00_JPRB, 0.47962E+00_JPRB, 0.54918E+00_JPRB, 0.60035E+00_JPRB, 0.65918E+00_JPRB, 0.68859E+00_JPRB ,&
&  0.69640E+00_JPRB, 0.70429E+00_JPRB, 0.68763E+00_JPRB, 0.66305E+00_JPRB, 0.62415E+00_JPRB, 0.59049E+00_JPRB, 0.56501E+00_JPRB ,&
&  0.54288E+00_JPRB, 0.52348E+00_JPRB, 0.50818E+00_JPRB, 0.49577E+00_JPRB, 0.48546E+00_JPRB, 0.48126E+00_JPRB, 0.48264E+00_JPRB ,&
&  0.48533E+00_JPRB, 0.48863E+00_JPRB, 0.49335E+00_JPRB, 0.50196E+00_JPRB, 0.51386E+00_JPRB, 0.54147E+00_JPRB, 0.64579E+00_JPRB ,&
&  0.81361E+00_JPRB, 0.94238E+00_JPRB, 0.10560E+01_JPRB, 0.11225E+01_JPRB, 0.11227E+01_JPRB, 0.10695E+01_JPRB, 0.96805E+00_JPRB ,&
&  0.85930E+00_JPRB, 0.77523E+00_JPRB, 0.72388E+00_JPRB, 0.69328E+00_JPRB, 0.67739E+00_JPRB, 0.66664E+00_JPRB, 0.65816E+00_JPRB ,&
&  0.65071E+00_JPRB, 0.64731E+00_JPRB, 0.65120E+00_JPRB, 0.66179E+00_JPRB, 0.67677E+00_JPRB, 0.68862E+00_JPRB, 0.69279E+00_JPRB ,&
&  0.69078E+00_JPRB, 0.68487E+00_JPRB, 0.67866E+00_JPRB, 0.67115E+00_JPRB, 0.66249E+00_JPRB, 0.65272E+00_JPRB, 0.64672E+00_JPRB ,&
&  0.64586E+00_JPRB, 0.64944E+00_JPRB, 0.65728E+00_JPRB, 0.65461E+00_JPRB, 0.67853E+00_JPRB, 0.68723E+00_JPRB, 0.69238E+00_JPRB ,&
&  0.69599E+00_JPRB, 0.69304E+00_JPRB, 0.68758E+00_JPRB, 0.61017E+00_JPRB, 0.67034E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.58876E+00_JPRB, 0.58703E+00_JPRB, 0.57250E+00_JPRB, 0.55603E+00_JPRB, 0.53939E+00_JPRB, 0.52340E+00_JPRB, 0.51131E+00_JPRB ,&
&  0.50216E+00_JPRB, 0.49065E+00_JPRB, 0.48064E+00_JPRB, 0.47381E+00_JPRB, 0.46658E+00_JPRB, 0.46007E+00_JPRB, 0.45270E+00_JPRB ,&
&  0.44683E+00_JPRB, 0.44306E+00_JPRB, 0.44026E+00_JPRB, 0.43684E+00_JPRB, 0.43293E+00_JPRB, 0.42820E+00_JPRB, 0.42351E+00_JPRB ,&
&  0.41915E+00_JPRB, 0.41615E+00_JPRB, 0.41164E+00_JPRB, 0.40873E+00_JPRB, 0.41135E+00_JPRB, 0.41837E+00_JPRB, 0.42506E+00_JPRB ,&
&  0.45287E+00_JPRB, 0.40324E+00_JPRB, 0.41732E+00_JPRB, 0.45120E+00_JPRB, 0.48937E+00_JPRB, 0.53952E+00_JPRB, 0.58061E+00_JPRB ,&
&  0.60010E+00_JPRB, 0.61876E+00_JPRB, 0.61078E+00_JPRB, 0.59468E+00_JPRB, 0.56684E+00_JPRB, 0.53889E+00_JPRB, 0.51667E+00_JPRB ,&
&  0.50281E+00_JPRB, 0.49254E+00_JPRB, 0.48045E+00_JPRB, 0.46708E+00_JPRB, 0.45462E+00_JPRB, 0.44604E+00_JPRB, 0.44257E+00_JPRB ,&
&  0.44333E+00_JPRB, 0.44477E+00_JPRB, 0.44515E+00_JPRB, 0.44837E+00_JPRB, 0.45621E+00_JPRB, 0.46865E+00_JPRB, 0.49688E+00_JPRB ,&
&  0.55986E+00_JPRB, 0.64435E+00_JPRB, 0.71844E+00_JPRB, 0.78868E+00_JPRB, 0.83378E+00_JPRB, 0.84764E+00_JPRB, 0.80596E+00_JPRB ,&
&  0.74699E+00_JPRB, 0.69229E+00_JPRB, 0.65669E+00_JPRB, 0.63810E+00_JPRB, 0.62717E+00_JPRB, 0.61952E+00_JPRB, 0.60824E+00_JPRB ,&
&  0.59345E+00_JPRB, 0.57603E+00_JPRB, 0.56051E+00_JPRB, 0.54523E+00_JPRB, 0.53457E+00_JPRB, 0.53160E+00_JPRB, 0.53472E+00_JPRB ,&
&  0.53949E+00_JPRB, 0.54285E+00_JPRB, 0.54558E+00_JPRB, 0.54869E+00_JPRB, 0.55098E+00_JPRB, 0.55321E+00_JPRB, 0.55494E+00_JPRB ,&
&  0.55926E+00_JPRB, 0.56546E+00_JPRB, 0.57227E+00_JPRB, 0.58494E+00_JPRB, 0.59428E+00_JPRB, 0.60032E+00_JPRB, 0.60433E+00_JPRB ,&
&  0.60489E+00_JPRB, 0.60924E+00_JPRB, 0.52633E+00_JPRB, 0.56249E+00_JPRB, 0.57948E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.51573E+00_JPRB, 0.51473E+00_JPRB, 0.50978E+00_JPRB, 0.49778E+00_JPRB, 0.48394E+00_JPRB, 0.47192E+00_JPRB, 0.46419E+00_JPRB ,&
&  0.45923E+00_JPRB, 0.45248E+00_JPRB, 0.44674E+00_JPRB, 0.44291E+00_JPRB, 0.43820E+00_JPRB, 0.43273E+00_JPRB, 0.42490E+00_JPRB ,&
&  0.41826E+00_JPRB, 0.41304E+00_JPRB, 0.40923E+00_JPRB, 0.40697E+00_JPRB, 0.40565E+00_JPRB, 0.40418E+00_JPRB, 0.40182E+00_JPRB ,&
&  0.39946E+00_JPRB, 0.39810E+00_JPRB, 0.39675E+00_JPRB, 0.39595E+00_JPRB, 0.39625E+00_JPRB, 0.40040E+00_JPRB, 0.40668E+00_JPRB ,&
&  0.39593E+00_JPRB, 0.37056E+00_JPRB, 0.38293E+00_JPRB, 0.40102E+00_JPRB, 0.42312E+00_JPRB, 0.45949E+00_JPRB, 0.49767E+00_JPRB ,&
&  0.51996E+00_JPRB, 0.53314E+00_JPRB, 0.52436E+00_JPRB, 0.51107E+00_JPRB, 0.49680E+00_JPRB, 0.48165E+00_JPRB, 0.46945E+00_JPRB ,&
&  0.45847E+00_JPRB, 0.45082E+00_JPRB, 0.44418E+00_JPRB, 0.43477E+00_JPRB, 0.42352E+00_JPRB, 0.41433E+00_JPRB, 0.40882E+00_JPRB ,&
&  0.40697E+00_JPRB, 0.40792E+00_JPRB, 0.40942E+00_JPRB, 0.41121E+00_JPRB, 0.41477E+00_JPRB, 0.42189E+00_JPRB, 0.43303E+00_JPRB ,&
&  0.44800E+00_JPRB, 0.47698E+00_JPRB, 0.51795E+00_JPRB, 0.55825E+00_JPRB, 0.59690E+00_JPRB, 0.62506E+00_JPRB, 0.64210E+00_JPRB ,&
&  0.64348E+00_JPRB, 0.62917E+00_JPRB, 0.60863E+00_JPRB, 0.58692E+00_JPRB, 0.56101E+00_JPRB, 0.52970E+00_JPRB, 0.49619E+00_JPRB ,&
&  0.46814E+00_JPRB, 0.44698E+00_JPRB, 0.43338E+00_JPRB, 0.42721E+00_JPRB, 0.42507E+00_JPRB, 0.42718E+00_JPRB, 0.43194E+00_JPRB ,&
&  0.43831E+00_JPRB, 0.44536E+00_JPRB, 0.45079E+00_JPRB, 0.45677E+00_JPRB, 0.46220E+00_JPRB, 0.46810E+00_JPRB, 0.47439E+00_JPRB ,&
&  0.48148E+00_JPRB, 0.48823E+00_JPRB, 0.49521E+00_JPRB, 0.50592E+00_JPRB, 0.51429E+00_JPRB, 0.52205E+00_JPRB, 0.52609E+00_JPRB ,&
&  0.52370E+00_JPRB, 0.52197E+00_JPRB, 0.51953E+00_JPRB, 0.51568E+00_JPRB, 0.51724E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.46373E+00_JPRB, 0.46171E+00_JPRB, 0.45963E+00_JPRB, 0.45640E+00_JPRB, 0.45092E+00_JPRB, 0.44309E+00_JPRB, 0.43684E+00_JPRB ,&
&  0.43276E+00_JPRB, 0.42841E+00_JPRB, 0.42476E+00_JPRB, 0.42035E+00_JPRB, 0.41503E+00_JPRB, 0.40843E+00_JPRB, 0.39969E+00_JPRB ,&
&  0.39159E+00_JPRB, 0.38415E+00_JPRB, 0.37928E+00_JPRB, 0.37701E+00_JPRB, 0.37702E+00_JPRB, 0.37711E+00_JPRB, 0.37544E+00_JPRB ,&
&  0.37438E+00_JPRB, 0.37432E+00_JPRB, 0.37431E+00_JPRB, 0.37414E+00_JPRB, 0.37432E+00_JPRB, 0.37773E+00_JPRB, 0.38158E+00_JPRB ,&
&  0.38854E+00_JPRB, 0.36524E+00_JPRB, 0.37643E+00_JPRB, 0.38073E+00_JPRB, 0.39189E+00_JPRB, 0.41275E+00_JPRB, 0.43839E+00_JPRB ,&
&  0.45832E+00_JPRB, 0.45938E+00_JPRB, 0.45116E+00_JPRB, 0.44031E+00_JPRB, 0.43342E+00_JPRB, 0.42655E+00_JPRB, 0.41892E+00_JPRB ,&
&  0.41162E+00_JPRB, 0.40658E+00_JPRB, 0.40218E+00_JPRB, 0.39779E+00_JPRB, 0.39176E+00_JPRB, 0.38416E+00_JPRB, 0.37688E+00_JPRB ,&
&  0.37175E+00_JPRB, 0.36995E+00_JPRB, 0.37093E+00_JPRB, 0.37318E+00_JPRB, 0.37600E+00_JPRB, 0.38092E+00_JPRB, 0.38734E+00_JPRB ,&
&  0.39480E+00_JPRB, 0.39994E+00_JPRB, 0.40676E+00_JPRB, 0.42028E+00_JPRB, 0.43624E+00_JPRB, 0.44934E+00_JPRB, 0.45790E+00_JPRB ,&
&  0.46213E+00_JPRB, 0.45953E+00_JPRB, 0.45221E+00_JPRB, 0.44137E+00_JPRB, 0.42863E+00_JPRB, 0.41545E+00_JPRB, 0.40390E+00_JPRB ,&
&  0.39461E+00_JPRB, 0.38676E+00_JPRB, 0.38010E+00_JPRB, 0.37549E+00_JPRB, 0.37338E+00_JPRB, 0.37307E+00_JPRB, 0.37444E+00_JPRB ,&
&  0.37847E+00_JPRB, 0.38380E+00_JPRB, 0.38860E+00_JPRB, 0.39336E+00_JPRB, 0.39770E+00_JPRB, 0.40148E+00_JPRB, 0.40636E+00_JPRB ,&
&  0.41283E+00_JPRB, 0.41783E+00_JPRB, 0.42211E+00_JPRB, 0.42791E+00_JPRB, 0.43471E+00_JPRB, 0.44209E+00_JPRB, 0.44902E+00_JPRB ,&
&  0.45385E+00_JPRB, 0.45635E+00_JPRB, 0.45950E+00_JPRB, 0.46105E+00_JPRB, 0.46426E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.42898E+00_JPRB, 0.42779E+00_JPRB, 0.42549E+00_JPRB, 0.42396E+00_JPRB, 0.42148E+00_JPRB, 0.41764E+00_JPRB, 0.41323E+00_JPRB ,&
&  0.40961E+00_JPRB, 0.40629E+00_JPRB, 0.40281E+00_JPRB, 0.39807E+00_JPRB, 0.39156E+00_JPRB, 0.38514E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37044E+00_JPRB, 0.36416E+00_JPRB, 0.35980E+00_JPRB, 0.35724E+00_JPRB, 0.35514E+00_JPRB, 0.35336E+00_JPRB, 0.35128E+00_JPRB ,&
&  0.35005E+00_JPRB, 0.34965E+00_JPRB, 0.34979E+00_JPRB, 0.35039E+00_JPRB, 0.35109E+00_JPRB, 0.35389E+00_JPRB, 0.35574E+00_JPRB ,&
&  0.35853E+00_JPRB, 0.36028E+00_JPRB, 0.36777E+00_JPRB, 0.37209E+00_JPRB, 0.37907E+00_JPRB, 0.38635E+00_JPRB, 0.39390E+00_JPRB ,&
&  0.40068E+00_JPRB, 0.40375E+00_JPRB, 0.39908E+00_JPRB, 0.39183E+00_JPRB, 0.38682E+00_JPRB, 0.38425E+00_JPRB, 0.37879E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.36900E+00_JPRB, 0.36568E+00_JPRB, 0.36327E+00_JPRB, 0.36081E+00_JPRB, 0.35719E+00_JPRB, 0.35319E+00_JPRB ,&
&  0.34817E+00_JPRB, 0.34439E+00_JPRB, 0.34149E+00_JPRB, 0.34043E+00_JPRB, 0.34072E+00_JPRB, 0.34216E+00_JPRB, 0.34446E+00_JPRB ,&
&  0.34833E+00_JPRB, 0.35242E+00_JPRB, 0.35464E+00_JPRB, 0.35599E+00_JPRB, 0.35799E+00_JPRB, 0.36142E+00_JPRB, 0.36437E+00_JPRB ,&
&  0.36538E+00_JPRB, 0.36472E+00_JPRB, 0.36247E+00_JPRB, 0.35885E+00_JPRB, 0.35436E+00_JPRB, 0.35047E+00_JPRB, 0.34721E+00_JPRB ,&
&  0.34417E+00_JPRB, 0.34099E+00_JPRB, 0.33784E+00_JPRB, 0.33609E+00_JPRB, 0.33513E+00_JPRB, 0.33488E+00_JPRB, 0.33609E+00_JPRB ,&
&  0.33878E+00_JPRB, 0.34212E+00_JPRB, 0.34597E+00_JPRB, 0.34971E+00_JPRB, 0.35328E+00_JPRB, 0.35624E+00_JPRB, 0.35981E+00_JPRB ,&
&  0.36538E+00_JPRB, 0.37153E+00_JPRB, 0.37762E+00_JPRB, 0.38330E+00_JPRB, 0.38953E+00_JPRB, 0.39688E+00_JPRB, 0.40472E+00_JPRB ,&
&  0.41011E+00_JPRB, 0.41378E+00_JPRB, 0.41824E+00_JPRB, 0.42316E+00_JPRB, 0.42750E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.38749E+00_JPRB, 0.38942E+00_JPRB, 0.39015E+00_JPRB, 0.39118E+00_JPRB, 0.39121E+00_JPRB, 0.38978E+00_JPRB, 0.38659E+00_JPRB ,&
&  0.38296E+00_JPRB, 0.37880E+00_JPRB, 0.37462E+00_JPRB, 0.36921E+00_JPRB, 0.36348E+00_JPRB, 0.35867E+00_JPRB, 0.35335E+00_JPRB ,&
&  0.34787E+00_JPRB, 0.34276E+00_JPRB, 0.33910E+00_JPRB, 0.33617E+00_JPRB, 0.33333E+00_JPRB, 0.33056E+00_JPRB, 0.32703E+00_JPRB ,&
&  0.32397E+00_JPRB, 0.32116E+00_JPRB, 0.32149E+00_JPRB, 0.32411E+00_JPRB, 0.32630E+00_JPRB, 0.32782E+00_JPRB, 0.32916E+00_JPRB ,&
&  0.32952E+00_JPRB, 0.33859E+00_JPRB, 0.33289E+00_JPRB, 0.34322E+00_JPRB, 0.34618E+00_JPRB, 0.34681E+00_JPRB, 0.34735E+00_JPRB ,&
&  0.34993E+00_JPRB, 0.35298E+00_JPRB, 0.35506E+00_JPRB, 0.35156E+00_JPRB, 0.34650E+00_JPRB, 0.34257E+00_JPRB, 0.33831E+00_JPRB ,&
&  0.33261E+00_JPRB, 0.33024E+00_JPRB, 0.33016E+00_JPRB, 0.33046E+00_JPRB, 0.33058E+00_JPRB, 0.33013E+00_JPRB, 0.32894E+00_JPRB ,&
&  0.32718E+00_JPRB, 0.32458E+00_JPRB, 0.32153E+00_JPRB, 0.31853E+00_JPRB, 0.31664E+00_JPRB, 0.31455E+00_JPRB, 0.31275E+00_JPRB ,&
&  0.31222E+00_JPRB, 0.31316E+00_JPRB, 0.31423E+00_JPRB, 0.31465E+00_JPRB, 0.31415E+00_JPRB, 0.31382E+00_JPRB, 0.31395E+00_JPRB ,&
&  0.31374E+00_JPRB, 0.31319E+00_JPRB, 0.31225E+00_JPRB, 0.31072E+00_JPRB, 0.30892E+00_JPRB, 0.30780E+00_JPRB, 0.30735E+00_JPRB ,&
&  0.30658E+00_JPRB, 0.30472E+00_JPRB, 0.30214E+00_JPRB, 0.30163E+00_JPRB, 0.30168E+00_JPRB, 0.30207E+00_JPRB, 0.30272E+00_JPRB ,&
&  0.30414E+00_JPRB, 0.30670E+00_JPRB, 0.30965E+00_JPRB, 0.31263E+00_JPRB, 0.31564E+00_JPRB, 0.31970E+00_JPRB, 0.32434E+00_JPRB ,&
&  0.32941E+00_JPRB, 0.33486E+00_JPRB, 0.34026E+00_JPRB, 0.34551E+00_JPRB, 0.35000E+00_JPRB, 0.35501E+00_JPRB, 0.36124E+00_JPRB ,&
&  0.36644E+00_JPRB, 0.37051E+00_JPRB, 0.37408E+00_JPRB, 0.37865E+00_JPRB, 0.38365E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.35309E+00_JPRB, 0.35810E+00_JPRB, 0.36106E+00_JPRB, 0.36264E+00_JPRB, 0.36323E+00_JPRB, 0.36189E+00_JPRB, 0.35931E+00_JPRB ,&
&  0.35521E+00_JPRB, 0.35103E+00_JPRB, 0.34650E+00_JPRB, 0.34146E+00_JPRB, 0.33690E+00_JPRB, 0.33297E+00_JPRB, 0.32947E+00_JPRB ,&
&  0.32589E+00_JPRB, 0.32255E+00_JPRB, 0.31982E+00_JPRB, 0.31705E+00_JPRB, 0.31383E+00_JPRB, 0.30975E+00_JPRB, 0.30457E+00_JPRB ,&
&  0.30141E+00_JPRB, 0.29892E+00_JPRB, 0.29974E+00_JPRB, 0.30271E+00_JPRB, 0.30505E+00_JPRB, 0.30643E+00_JPRB, 0.30757E+00_JPRB ,&
&  0.30913E+00_JPRB, 0.31135E+00_JPRB, 0.31318E+00_JPRB, 0.31626E+00_JPRB, 0.31888E+00_JPRB, 0.31862E+00_JPRB, 0.31679E+00_JPRB ,&
&  0.31539E+00_JPRB, 0.31594E+00_JPRB, 0.31707E+00_JPRB, 0.31758E+00_JPRB, 0.31664E+00_JPRB, 0.31399E+00_JPRB, 0.30972E+00_JPRB ,&
&  0.30267E+00_JPRB, 0.29748E+00_JPRB, 0.29635E+00_JPRB, 0.29755E+00_JPRB, 0.29957E+00_JPRB, 0.30111E+00_JPRB, 0.30205E+00_JPRB ,&
&  0.30233E+00_JPRB, 0.30170E+00_JPRB, 0.30049E+00_JPRB, 0.29902E+00_JPRB, 0.29710E+00_JPRB, 0.29498E+00_JPRB, 0.29373E+00_JPRB ,&
&  0.29240E+00_JPRB, 0.29131E+00_JPRB, 0.29075E+00_JPRB, 0.29046E+00_JPRB, 0.28939E+00_JPRB, 0.28769E+00_JPRB, 0.28667E+00_JPRB ,&
&  0.28531E+00_JPRB, 0.28454E+00_JPRB, 0.28341E+00_JPRB, 0.28227E+00_JPRB, 0.28081E+00_JPRB, 0.27939E+00_JPRB, 0.27884E+00_JPRB ,&
&  0.27883E+00_JPRB, 0.27762E+00_JPRB, 0.27622E+00_JPRB, 0.27621E+00_JPRB, 0.27695E+00_JPRB, 0.27800E+00_JPRB, 0.27882E+00_JPRB ,&
&  0.27962E+00_JPRB, 0.28066E+00_JPRB, 0.28294E+00_JPRB, 0.28558E+00_JPRB, 0.28865E+00_JPRB, 0.29231E+00_JPRB, 0.29645E+00_JPRB ,&
&  0.30124E+00_JPRB, 0.30670E+00_JPRB, 0.31275E+00_JPRB, 0.31844E+00_JPRB, 0.32334E+00_JPRB, 0.32670E+00_JPRB, 0.33083E+00_JPRB ,&
&  0.33567E+00_JPRB, 0.33890E+00_JPRB, 0.34154E+00_JPRB, 0.34469E+00_JPRB, 0.34821E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.31967E+00_JPRB, 0.32483E+00_JPRB, 0.32881E+00_JPRB, 0.33117E+00_JPRB, 0.33148E+00_JPRB, 0.33035E+00_JPRB, 0.32771E+00_JPRB ,&
&  0.32372E+00_JPRB, 0.31964E+00_JPRB, 0.31559E+00_JPRB, 0.31231E+00_JPRB, 0.31023E+00_JPRB, 0.30851E+00_JPRB, 0.30702E+00_JPRB ,&
&  0.30496E+00_JPRB, 0.30249E+00_JPRB, 0.29953E+00_JPRB, 0.29692E+00_JPRB, 0.29373E+00_JPRB, 0.28934E+00_JPRB, 0.28603E+00_JPRB ,&
&  0.28370E+00_JPRB, 0.28151E+00_JPRB, 0.28080E+00_JPRB, 0.28204E+00_JPRB, 0.28446E+00_JPRB, 0.28683E+00_JPRB, 0.28944E+00_JPRB ,&
&  0.29126E+00_JPRB, 0.29243E+00_JPRB, 0.29335E+00_JPRB, 0.29325E+00_JPRB, 0.29318E+00_JPRB, 0.29284E+00_JPRB, 0.29213E+00_JPRB ,&
&  0.29135E+00_JPRB, 0.29059E+00_JPRB, 0.28927E+00_JPRB, 0.28895E+00_JPRB, 0.28863E+00_JPRB, 0.28724E+00_JPRB, 0.28486E+00_JPRB ,&
&  0.28112E+00_JPRB, 0.27607E+00_JPRB, 0.27214E+00_JPRB, 0.27024E+00_JPRB, 0.26972E+00_JPRB, 0.27010E+00_JPRB, 0.27105E+00_JPRB ,&
&  0.27225E+00_JPRB, 0.27393E+00_JPRB, 0.27550E+00_JPRB, 0.27627E+00_JPRB, 0.27613E+00_JPRB, 0.27597E+00_JPRB, 0.27473E+00_JPRB ,&
&  0.27353E+00_JPRB, 0.27275E+00_JPRB, 0.27222E+00_JPRB, 0.27161E+00_JPRB, 0.26947E+00_JPRB, 0.26680E+00_JPRB, 0.26614E+00_JPRB ,&
&  0.26458E+00_JPRB, 0.26357E+00_JPRB, 0.26209E+00_JPRB, 0.26127E+00_JPRB, 0.26029E+00_JPRB, 0.25890E+00_JPRB, 0.25745E+00_JPRB ,&
&  0.25633E+00_JPRB, 0.25575E+00_JPRB, 0.25545E+00_JPRB, 0.25473E+00_JPRB, 0.25531E+00_JPRB, 0.25629E+00_JPRB, 0.25784E+00_JPRB ,&
&  0.25907E+00_JPRB, 0.26022E+00_JPRB, 0.26189E+00_JPRB, 0.26370E+00_JPRB, 0.26645E+00_JPRB, 0.26967E+00_JPRB, 0.27335E+00_JPRB ,&
&  0.27651E+00_JPRB, 0.28067E+00_JPRB, 0.28537E+00_JPRB, 0.28978E+00_JPRB, 0.29330E+00_JPRB, 0.29584E+00_JPRB, 0.29925E+00_JPRB ,&
&  0.30460E+00_JPRB, 0.30878E+00_JPRB, 0.31071E+00_JPRB, 0.31309E+00_JPRB, 0.31548E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.29470E+00_JPRB, 0.29648E+00_JPRB, 0.29833E+00_JPRB, 0.30037E+00_JPRB, 0.30181E+00_JPRB, 0.30239E+00_JPRB, 0.30106E+00_JPRB ,&
&  0.29842E+00_JPRB, 0.29534E+00_JPRB, 0.29223E+00_JPRB, 0.28959E+00_JPRB, 0.28802E+00_JPRB, 0.28707E+00_JPRB, 0.28644E+00_JPRB ,&
&  0.28588E+00_JPRB, 0.28527E+00_JPRB, 0.28323E+00_JPRB, 0.28024E+00_JPRB, 0.27630E+00_JPRB, 0.27289E+00_JPRB, 0.27179E+00_JPRB ,&
&  0.27008E+00_JPRB, 0.26887E+00_JPRB, 0.26793E+00_JPRB, 0.26699E+00_JPRB, 0.26658E+00_JPRB, 0.26751E+00_JPRB, 0.26934E+00_JPRB ,&
&  0.27126E+00_JPRB, 0.27288E+00_JPRB, 0.27417E+00_JPRB, 0.27499E+00_JPRB, 0.27440E+00_JPRB, 0.27334E+00_JPRB, 0.27205E+00_JPRB ,&
&  0.27043E+00_JPRB, 0.26883E+00_JPRB, 0.26738E+00_JPRB, 0.26547E+00_JPRB, 0.26274E+00_JPRB, 0.25944E+00_JPRB, 0.25706E+00_JPRB ,&
&  0.25523E+00_JPRB, 0.25354E+00_JPRB, 0.25182E+00_JPRB, 0.25092E+00_JPRB, 0.25026E+00_JPRB, 0.24998E+00_JPRB, 0.25005E+00_JPRB ,&
&  0.25050E+00_JPRB, 0.25181E+00_JPRB, 0.25363E+00_JPRB, 0.25563E+00_JPRB, 0.25628E+00_JPRB, 0.25651E+00_JPRB, 0.25674E+00_JPRB ,&
&  0.25604E+00_JPRB, 0.25468E+00_JPRB, 0.25282E+00_JPRB, 0.25072E+00_JPRB, 0.24931E+00_JPRB, 0.24826E+00_JPRB, 0.24710E+00_JPRB ,&
&  0.24510E+00_JPRB, 0.24383E+00_JPRB, 0.24316E+00_JPRB, 0.24301E+00_JPRB, 0.24318E+00_JPRB, 0.24316E+00_JPRB, 0.24239E+00_JPRB ,&
&  0.24146E+00_JPRB, 0.24068E+00_JPRB, 0.23977E+00_JPRB, 0.23890E+00_JPRB, 0.23735E+00_JPRB, 0.23800E+00_JPRB, 0.23913E+00_JPRB ,&
&  0.23955E+00_JPRB, 0.23962E+00_JPRB, 0.23986E+00_JPRB, 0.24127E+00_JPRB, 0.24288E+00_JPRB, 0.24495E+00_JPRB, 0.24769E+00_JPRB ,&
&  0.25004E+00_JPRB, 0.25313E+00_JPRB, 0.25753E+00_JPRB, 0.26164E+00_JPRB, 0.26466E+00_JPRB, 0.26617E+00_JPRB, 0.26919E+00_JPRB ,&
&  0.27548E+00_JPRB, 0.28194E+00_JPRB, 0.28584E+00_JPRB, 0.28965E+00_JPRB, 0.29316E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.27108E+00_JPRB, 0.27193E+00_JPRB, 0.27274E+00_JPRB, 0.27386E+00_JPRB, 0.27546E+00_JPRB, 0.27646E+00_JPRB, 0.27726E+00_JPRB ,&
&  0.27667E+00_JPRB, 0.27490E+00_JPRB, 0.27281E+00_JPRB, 0.27058E+00_JPRB, 0.26960E+00_JPRB, 0.26950E+00_JPRB, 0.26964E+00_JPRB ,&
&  0.26919E+00_JPRB, 0.26902E+00_JPRB, 0.26718E+00_JPRB, 0.26406E+00_JPRB, 0.26033E+00_JPRB, 0.25773E+00_JPRB, 0.25696E+00_JPRB ,&
&  0.25636E+00_JPRB, 0.25537E+00_JPRB, 0.25482E+00_JPRB, 0.25410E+00_JPRB, 0.25166E+00_JPRB, 0.25040E+00_JPRB, 0.25074E+00_JPRB ,&
&  0.25151E+00_JPRB, 0.25295E+00_JPRB, 0.25308E+00_JPRB, 0.25074E+00_JPRB, 0.25294E+00_JPRB, 0.25270E+00_JPRB, 0.25079E+00_JPRB ,&
&  0.24832E+00_JPRB, 0.24577E+00_JPRB, 0.24344E+00_JPRB, 0.24111E+00_JPRB, 0.23872E+00_JPRB, 0.23688E+00_JPRB, 0.23566E+00_JPRB ,&
&  0.23531E+00_JPRB, 0.23550E+00_JPRB, 0.23601E+00_JPRB, 0.23696E+00_JPRB, 0.23761E+00_JPRB, 0.23796E+00_JPRB, 0.23802E+00_JPRB ,&
&  0.23815E+00_JPRB, 0.23837E+00_JPRB, 0.23878E+00_JPRB, 0.23902E+00_JPRB, 0.23929E+00_JPRB, 0.23950E+00_JPRB, 0.23930E+00_JPRB ,&
&  0.23812E+00_JPRB, 0.23619E+00_JPRB, 0.23364E+00_JPRB, 0.23261E+00_JPRB, 0.23249E+00_JPRB, 0.23082E+00_JPRB, 0.22827E+00_JPRB ,&
&  0.22593E+00_JPRB, 0.22553E+00_JPRB, 0.22684E+00_JPRB, 0.22817E+00_JPRB, 0.22922E+00_JPRB, 0.22957E+00_JPRB, 0.22928E+00_JPRB ,&
&  0.22826E+00_JPRB, 0.22594E+00_JPRB, 0.22333E+00_JPRB, 0.22128E+00_JPRB, 0.22010E+00_JPRB, 0.22014E+00_JPRB, 0.22113E+00_JPRB ,&
&  0.22131E+00_JPRB, 0.22148E+00_JPRB, 0.22269E+00_JPRB, 0.22381E+00_JPRB, 0.22469E+00_JPRB, 0.22569E+00_JPRB, 0.22728E+00_JPRB ,&
&  0.22941E+00_JPRB, 0.23292E+00_JPRB, 0.23722E+00_JPRB, 0.24201E+00_JPRB, 0.24536E+00_JPRB, 0.24473E+00_JPRB, 0.24579E+00_JPRB ,&
&  0.25093E+00_JPRB, 0.25686E+00_JPRB, 0.26154E+00_JPRB, 0.26516E+00_JPRB, 0.26853E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.25130E+00_JPRB, 0.25247E+00_JPRB, 0.25343E+00_JPRB, 0.25423E+00_JPRB, 0.25576E+00_JPRB, 0.25699E+00_JPRB, 0.25858E+00_JPRB ,&
&  0.25971E+00_JPRB, 0.25843E+00_JPRB, 0.25674E+00_JPRB, 0.25534E+00_JPRB, 0.25478E+00_JPRB, 0.25406E+00_JPRB, 0.25346E+00_JPRB ,&
&  0.25412E+00_JPRB, 0.25343E+00_JPRB, 0.25071E+00_JPRB, 0.24860E+00_JPRB, 0.24514E+00_JPRB, 0.24288E+00_JPRB, 0.24247E+00_JPRB ,&
&  0.24408E+00_JPRB, 0.24367E+00_JPRB, 0.24278E+00_JPRB, 0.24151E+00_JPRB, 0.23990E+00_JPRB, 0.23900E+00_JPRB, 0.23779E+00_JPRB ,&
&  0.23705E+00_JPRB, 0.23584E+00_JPRB, 0.23118E+00_JPRB, 0.23166E+00_JPRB, 0.23129E+00_JPRB, 0.23132E+00_JPRB, 0.23079E+00_JPRB ,&
&  0.23015E+00_JPRB, 0.22911E+00_JPRB, 0.22729E+00_JPRB, 0.22526E+00_JPRB, 0.22354E+00_JPRB, 0.22259E+00_JPRB, 0.22250E+00_JPRB ,&
&  0.22295E+00_JPRB, 0.22409E+00_JPRB, 0.22590E+00_JPRB, 0.22806E+00_JPRB, 0.22900E+00_JPRB, 0.22969E+00_JPRB, 0.23077E+00_JPRB ,&
&  0.23121E+00_JPRB, 0.23127E+00_JPRB, 0.23119E+00_JPRB, 0.23097E+00_JPRB, 0.23046E+00_JPRB, 0.22941E+00_JPRB, 0.22857E+00_JPRB ,&
&  0.22670E+00_JPRB, 0.22409E+00_JPRB, 0.22227E+00_JPRB, 0.22006E+00_JPRB, 0.21407E+00_JPRB, 0.20387E+00_JPRB, 0.19821E+00_JPRB ,&
&  0.19858E+00_JPRB, 0.20336E+00_JPRB, 0.20876E+00_JPRB, 0.21220E+00_JPRB, 0.21789E+00_JPRB, 0.21804E+00_JPRB, 0.21407E+00_JPRB ,&
&  0.20508E+00_JPRB, 0.20044E+00_JPRB, 0.19699E+00_JPRB, 0.19260E+00_JPRB, 0.18999E+00_JPRB, 0.18967E+00_JPRB, 0.19006E+00_JPRB ,&
&  0.19218E+00_JPRB, 0.19500E+00_JPRB, 0.19801E+00_JPRB, 0.19977E+00_JPRB, 0.20002E+00_JPRB, 0.19888E+00_JPRB, 0.19740E+00_JPRB ,&
&  0.19737E+00_JPRB, 0.20169E+00_JPRB, 0.21024E+00_JPRB, 0.21897E+00_JPRB, 0.22363E+00_JPRB, 0.22562E+00_JPRB, 0.22766E+00_JPRB ,&
&  0.23176E+00_JPRB, 0.23710E+00_JPRB, 0.24234E+00_JPRB, 0.24598E+00_JPRB, 0.24896E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.23138E+00_JPRB, 0.23258E+00_JPRB, 0.23378E+00_JPRB, 0.23494E+00_JPRB, 0.23619E+00_JPRB, 0.23763E+00_JPRB, 0.23890E+00_JPRB ,&
&  0.24038E+00_JPRB, 0.24133E+00_JPRB, 0.24154E+00_JPRB, 0.24045E+00_JPRB, 0.23965E+00_JPRB, 0.23950E+00_JPRB, 0.24006E+00_JPRB ,&
&  0.23802E+00_JPRB, 0.23504E+00_JPRB, 0.23374E+00_JPRB, 0.23346E+00_JPRB, 0.23277E+00_JPRB, 0.23246E+00_JPRB, 0.23325E+00_JPRB ,&
&  0.23418E+00_JPRB, 0.23358E+00_JPRB, 0.23304E+00_JPRB, 0.23162E+00_JPRB, 0.23048E+00_JPRB, 0.22958E+00_JPRB, 0.22838E+00_JPRB ,&
&  0.22456E+00_JPRB, 0.21447E+00_JPRB, 0.20819E+00_JPRB, 0.21201E+00_JPRB, 0.21851E+00_JPRB, 0.21770E+00_JPRB, 0.21782E+00_JPRB ,&
&  0.21745E+00_JPRB, 0.21615E+00_JPRB, 0.21467E+00_JPRB, 0.21330E+00_JPRB, 0.21277E+00_JPRB, 0.21317E+00_JPRB, 0.21419E+00_JPRB ,&
&  0.21546E+00_JPRB, 0.21801E+00_JPRB, 0.22071E+00_JPRB, 0.22183E+00_JPRB, 0.22300E+00_JPRB, 0.22360E+00_JPRB, 0.22376E+00_JPRB ,&
&  0.22314E+00_JPRB, 0.22183E+00_JPRB, 0.22073E+00_JPRB, 0.21964E+00_JPRB, 0.21859E+00_JPRB, 0.21712E+00_JPRB, 0.21289E+00_JPRB ,&
&  0.20763E+00_JPRB, 0.20221E+00_JPRB, 0.19566E+00_JPRB, 0.18910E+00_JPRB, 0.18336E+00_JPRB, 0.17920E+00_JPRB, 0.17567E+00_JPRB ,&
&  0.17533E+00_JPRB, 0.17955E+00_JPRB, 0.18784E+00_JPRB, 0.19756E+00_JPRB, 0.20207E+00_JPRB, 0.19278E+00_JPRB, 0.18685E+00_JPRB ,&
&  0.18093E+00_JPRB, 0.17616E+00_JPRB, 0.17331E+00_JPRB, 0.17183E+00_JPRB, 0.17080E+00_JPRB, 0.17081E+00_JPRB, 0.17149E+00_JPRB ,&
&  0.17295E+00_JPRB, 0.17450E+00_JPRB, 0.17594E+00_JPRB, 0.17689E+00_JPRB, 0.17727E+00_JPRB, 0.17714E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17739E+00_JPRB, 0.17998E+00_JPRB, 0.18428E+00_JPRB, 0.18928E+00_JPRB, 0.19369E+00_JPRB, 0.19800E+00_JPRB, 0.20378E+00_JPRB ,&
&  0.21101E+00_JPRB, 0.21647E+00_JPRB, 0.22137E+00_JPRB, 0.22653E+00_JPRB, 0.22955E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.21604E+00_JPRB, 0.21824E+00_JPRB, 0.21894E+00_JPRB, 0.22023E+00_JPRB, 0.22055E+00_JPRB, 0.22176E+00_JPRB, 0.22263E+00_JPRB ,&
&  0.22495E+00_JPRB, 0.22709E+00_JPRB, 0.22755E+00_JPRB, 0.22564E+00_JPRB, 0.22431E+00_JPRB, 0.22109E+00_JPRB, 0.21976E+00_JPRB ,&
&  0.21929E+00_JPRB, 0.22024E+00_JPRB, 0.22039E+00_JPRB, 0.22043E+00_JPRB, 0.22035E+00_JPRB, 0.22027E+00_JPRB, 0.22080E+00_JPRB ,&
&  0.21866E+00_JPRB, 0.21749E+00_JPRB, 0.21948E+00_JPRB, 0.21841E+00_JPRB, 0.21753E+00_JPRB, 0.21599E+00_JPRB, 0.21511E+00_JPRB ,&
&  0.21082E+00_JPRB, 0.20484E+00_JPRB, 0.19846E+00_JPRB, 0.20188E+00_JPRB, 0.20900E+00_JPRB, 0.20632E+00_JPRB, 0.20569E+00_JPRB ,&
&  0.20529E+00_JPRB, 0.20426E+00_JPRB, 0.20399E+00_JPRB, 0.20366E+00_JPRB, 0.20453E+00_JPRB, 0.20540E+00_JPRB, 0.20747E+00_JPRB ,&
&  0.21028E+00_JPRB, 0.21414E+00_JPRB, 0.21407E+00_JPRB, 0.20744E+00_JPRB, 0.20446E+00_JPRB, 0.20345E+00_JPRB, 0.19996E+00_JPRB ,&
&  0.19577E+00_JPRB, 0.19441E+00_JPRB, 0.19317E+00_JPRB, 0.19010E+00_JPRB, 0.18900E+00_JPRB, 0.18846E+00_JPRB, 0.18569E+00_JPRB ,&
&  0.18217E+00_JPRB, 0.17956E+00_JPRB, 0.17600E+00_JPRB, 0.17240E+00_JPRB, 0.16994E+00_JPRB, 0.16883E+00_JPRB, 0.16901E+00_JPRB ,&
&  0.17103E+00_JPRB, 0.17300E+00_JPRB, 0.17904E+00_JPRB, 0.18278E+00_JPRB, 0.17940E+00_JPRB, 0.17648E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.16731E+00_JPRB, 0.16475E+00_JPRB, 0.16272E+00_JPRB, 0.16122E+00_JPRB, 0.16113E+00_JPRB, 0.16192E+00_JPRB, 0.16244E+00_JPRB ,&
&  0.16321E+00_JPRB, 0.16436E+00_JPRB, 0.16490E+00_JPRB, 0.16520E+00_JPRB, 0.16528E+00_JPRB, 0.16562E+00_JPRB, 0.16635E+00_JPRB ,&
&  0.16724E+00_JPRB, 0.16931E+00_JPRB, 0.17196E+00_JPRB, 0.17471E+00_JPRB, 0.17684E+00_JPRB, 0.17860E+00_JPRB, 0.18229E+00_JPRB ,&
&  0.18636E+00_JPRB, 0.19581E+00_JPRB, 0.20902E+00_JPRB, 0.21808E+00_JPRB, 0.21430E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.21305E+00_JPRB, 0.21447E+00_JPRB, 0.21417E+00_JPRB, 0.21433E+00_JPRB, 0.21382E+00_JPRB, 0.21471E+00_JPRB, 0.21372E+00_JPRB ,&
&  0.21339E+00_JPRB, 0.21194E+00_JPRB, 0.21060E+00_JPRB, 0.20750E+00_JPRB, 0.20581E+00_JPRB, 0.19964E+00_JPRB, 0.19787E+00_JPRB ,&
&  0.19625E+00_JPRB, 0.19698E+00_JPRB, 0.19712E+00_JPRB, 0.19758E+00_JPRB, 0.19833E+00_JPRB, 0.19987E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19838E+00_JPRB, 0.19632E+00_JPRB, 0.19388E+00_JPRB, 0.19165E+00_JPRB, 0.19199E+00_JPRB, 0.19154E+00_JPRB, 0.19045E+00_JPRB ,&
&  0.18946E+00_JPRB, 0.19006E+00_JPRB, 0.19051E+00_JPRB, 0.19188E+00_JPRB, 0.19677E+00_JPRB, 0.19492E+00_JPRB, 0.19384E+00_JPRB ,&
&  0.19388E+00_JPRB, 0.19406E+00_JPRB, 0.19489E+00_JPRB, 0.19559E+00_JPRB, 0.19810E+00_JPRB, 0.19767E+00_JPRB, 0.20073E+00_JPRB ,&
&  0.19875E+00_JPRB, 0.19751E+00_JPRB, 0.19555E+00_JPRB, 0.19256E+00_JPRB, 0.18832E+00_JPRB, 0.18562E+00_JPRB, 0.18309E+00_JPRB ,&
&  0.18024E+00_JPRB, 0.17788E+00_JPRB, 0.17481E+00_JPRB, 0.17372E+00_JPRB, 0.17209E+00_JPRB, 0.17065E+00_JPRB, 0.16936E+00_JPRB ,&
&  0.16844E+00_JPRB, 0.16712E+00_JPRB, 0.16602E+00_JPRB, 0.16530E+00_JPRB, 0.16513E+00_JPRB, 0.16530E+00_JPRB, 0.16645E+00_JPRB ,&
&  0.16857E+00_JPRB, 0.17026E+00_JPRB, 0.17160E+00_JPRB, 0.17289E+00_JPRB, 0.17140E+00_JPRB, 0.16868E+00_JPRB, 0.16601E+00_JPRB ,&
&  0.16224E+00_JPRB, 0.16023E+00_JPRB, 0.15830E+00_JPRB, 0.15714E+00_JPRB, 0.15683E+00_JPRB, 0.15674E+00_JPRB, 0.15697E+00_JPRB ,&
&  0.15722E+00_JPRB, 0.15771E+00_JPRB, 0.15808E+00_JPRB, 0.15859E+00_JPRB, 0.15928E+00_JPRB, 0.15995E+00_JPRB, 0.16098E+00_JPRB ,&
&  0.16250E+00_JPRB, 0.16443E+00_JPRB, 0.16684E+00_JPRB, 0.16952E+00_JPRB, 0.17301E+00_JPRB, 0.17754E+00_JPRB, 0.18409E+00_JPRB ,&
&  0.19058E+00_JPRB, 0.20073E+00_JPRB, 0.20167E+00_JPRB, 0.20840E+00_JPRB, 0.21205E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.21051E+00_JPRB, 0.21251E+00_JPRB, 0.21317E+00_JPRB, 0.21400E+00_JPRB, 0.21339E+00_JPRB, 0.21077E+00_JPRB, 0.20803E+00_JPRB ,&
&  0.20218E+00_JPRB, 0.19752E+00_JPRB, 0.19190E+00_JPRB, 0.18671E+00_JPRB, 0.18176E+00_JPRB, 0.17865E+00_JPRB, 0.17677E+00_JPRB ,&
&  0.17659E+00_JPRB, 0.17792E+00_JPRB, 0.17887E+00_JPRB, 0.17943E+00_JPRB, 0.17968E+00_JPRB, 0.18059E+00_JPRB, 0.18050E+00_JPRB ,&
&  0.17995E+00_JPRB, 0.17886E+00_JPRB, 0.17807E+00_JPRB, 0.17884E+00_JPRB, 0.18085E+00_JPRB, 0.18292E+00_JPRB, 0.18464E+00_JPRB ,&
&  0.18609E+00_JPRB, 0.18524E+00_JPRB, 0.18520E+00_JPRB, 0.18437E+00_JPRB, 0.18399E+00_JPRB, 0.18352E+00_JPRB, 0.18358E+00_JPRB ,&
&  0.18470E+00_JPRB, 0.18559E+00_JPRB, 0.18775E+00_JPRB, 0.18886E+00_JPRB, 0.18779E+00_JPRB, 0.19030E+00_JPRB, 0.19098E+00_JPRB ,&
&  0.19047E+00_JPRB, 0.18729E+00_JPRB, 0.18378E+00_JPRB, 0.18097E+00_JPRB, 0.17797E+00_JPRB, 0.17546E+00_JPRB, 0.17340E+00_JPRB ,&
&  0.17153E+00_JPRB, 0.17055E+00_JPRB, 0.16850E+00_JPRB, 0.16678E+00_JPRB, 0.16568E+00_JPRB, 0.16453E+00_JPRB, 0.16382E+00_JPRB ,&
&  0.16340E+00_JPRB, 0.16262E+00_JPRB, 0.16250E+00_JPRB, 0.16208E+00_JPRB, 0.16219E+00_JPRB, 0.16233E+00_JPRB, 0.16268E+00_JPRB ,&
&  0.16350E+00_JPRB, 0.16454E+00_JPRB, 0.16445E+00_JPRB, 0.16434E+00_JPRB, 0.16276E+00_JPRB, 0.16102E+00_JPRB, 0.15955E+00_JPRB ,&
&  0.15759E+00_JPRB, 0.15617E+00_JPRB, 0.15530E+00_JPRB, 0.15443E+00_JPRB, 0.15428E+00_JPRB, 0.15430E+00_JPRB, 0.15425E+00_JPRB ,&
&  0.15446E+00_JPRB, 0.15482E+00_JPRB, 0.15537E+00_JPRB, 0.15647E+00_JPRB, 0.15746E+00_JPRB, 0.15886E+00_JPRB, 0.16039E+00_JPRB ,&
&  0.16160E+00_JPRB, 0.16377E+00_JPRB, 0.16599E+00_JPRB, 0.16849E+00_JPRB, 0.17379E+00_JPRB, 0.17835E+00_JPRB, 0.18648E+00_JPRB ,&
&  0.19248E+00_JPRB, 0.20141E+00_JPRB, 0.19917E+00_JPRB, 0.20410E+00_JPRB, 0.20825E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.20937E+00_JPRB, 0.21059E+00_JPRB, 0.21027E+00_JPRB, 0.21045E+00_JPRB, 0.20913E+00_JPRB, 0.20716E+00_JPRB, 0.20425E+00_JPRB ,&
&  0.20133E+00_JPRB, 0.19545E+00_JPRB, 0.19122E+00_JPRB, 0.18651E+00_JPRB, 0.18256E+00_JPRB, 0.17905E+00_JPRB, 0.17657E+00_JPRB ,&
&  0.17541E+00_JPRB, 0.17495E+00_JPRB, 0.17467E+00_JPRB, 0.17373E+00_JPRB, 0.17201E+00_JPRB, 0.17087E+00_JPRB, 0.16943E+00_JPRB ,&
&  0.16838E+00_JPRB, 0.16833E+00_JPRB, 0.16921E+00_JPRB, 0.17107E+00_JPRB, 0.17442E+00_JPRB, 0.17672E+00_JPRB, 0.17883E+00_JPRB ,&
&  0.18079E+00_JPRB, 0.17695E+00_JPRB, 0.17683E+00_JPRB, 0.17726E+00_JPRB, 0.17728E+00_JPRB, 0.17671E+00_JPRB, 0.17577E+00_JPRB ,&
&  0.17920E+00_JPRB, 0.18009E+00_JPRB, 0.18548E+00_JPRB, 0.18520E+00_JPRB, 0.18424E+00_JPRB, 0.18575E+00_JPRB, 0.18457E+00_JPRB ,&
&  0.18449E+00_JPRB, 0.18131E+00_JPRB, 0.17896E+00_JPRB, 0.17547E+00_JPRB, 0.17195E+00_JPRB, 0.17017E+00_JPRB, 0.16812E+00_JPRB ,&
&  0.16745E+00_JPRB, 0.16729E+00_JPRB, 0.16581E+00_JPRB, 0.16488E+00_JPRB, 0.16315E+00_JPRB, 0.16121E+00_JPRB, 0.16002E+00_JPRB ,&
&  0.15880E+00_JPRB, 0.15785E+00_JPRB, 0.15734E+00_JPRB, 0.15684E+00_JPRB, 0.15663E+00_JPRB, 0.15651E+00_JPRB, 0.15639E+00_JPRB ,&
&  0.15642E+00_JPRB, 0.15634E+00_JPRB, 0.15579E+00_JPRB, 0.15550E+00_JPRB, 0.15445E+00_JPRB, 0.15373E+00_JPRB, 0.15314E+00_JPRB ,&
&  0.15261E+00_JPRB, 0.15243E+00_JPRB, 0.15233E+00_JPRB, 0.15240E+00_JPRB, 0.15255E+00_JPRB, 0.15307E+00_JPRB, 0.15366E+00_JPRB ,&
&  0.15426E+00_JPRB, 0.15526E+00_JPRB, 0.15604E+00_JPRB, 0.15715E+00_JPRB, 0.15848E+00_JPRB, 0.15961E+00_JPRB, 0.16141E+00_JPRB ,&
&  0.16337E+00_JPRB, 0.16563E+00_JPRB, 0.16891E+00_JPRB, 0.17175E+00_JPRB, 0.17529E+00_JPRB, 0.18008E+00_JPRB, 0.18519E+00_JPRB ,&
&  0.19077E+00_JPRB, 0.19693E+00_JPRB, 0.19743E+00_JPRB, 0.20350E+00_JPRB, 0.20731E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.20093E+00_JPRB, 0.20164E+00_JPRB, 0.20232E+00_JPRB, 0.20256E+00_JPRB, 0.20263E+00_JPRB, 0.20179E+00_JPRB, 0.20069E+00_JPRB ,&
&  0.19667E+00_JPRB, 0.19377E+00_JPRB, 0.19050E+00_JPRB, 0.18724E+00_JPRB, 0.18437E+00_JPRB, 0.18133E+00_JPRB, 0.17867E+00_JPRB ,&
&  0.17624E+00_JPRB, 0.17441E+00_JPRB, 0.17276E+00_JPRB, 0.17057E+00_JPRB, 0.16781E+00_JPRB, 0.16528E+00_JPRB, 0.16373E+00_JPRB ,&
&  0.16331E+00_JPRB, 0.16366E+00_JPRB, 0.16515E+00_JPRB, 0.16665E+00_JPRB, 0.16824E+00_JPRB, 0.16943E+00_JPRB, 0.17064E+00_JPRB ,&
&  0.17007E+00_JPRB, 0.16971E+00_JPRB, 0.16937E+00_JPRB, 0.16874E+00_JPRB, 0.16942E+00_JPRB, 0.17036E+00_JPRB, 0.17096E+00_JPRB ,&
&  0.17168E+00_JPRB, 0.17341E+00_JPRB, 0.17468E+00_JPRB, 0.17585E+00_JPRB, 0.17587E+00_JPRB, 0.17524E+00_JPRB, 0.17384E+00_JPRB ,&
&  0.17203E+00_JPRB, 0.17004E+00_JPRB, 0.16824E+00_JPRB, 0.16670E+00_JPRB, 0.16551E+00_JPRB, 0.16415E+00_JPRB, 0.16297E+00_JPRB ,&
&  0.16239E+00_JPRB, 0.16186E+00_JPRB, 0.16111E+00_JPRB, 0.16037E+00_JPRB, 0.15923E+00_JPRB, 0.15823E+00_JPRB, 0.15720E+00_JPRB ,&
&  0.15626E+00_JPRB, 0.15538E+00_JPRB, 0.15457E+00_JPRB, 0.15374E+00_JPRB, 0.15316E+00_JPRB, 0.15265E+00_JPRB, 0.15215E+00_JPRB ,&
&  0.15162E+00_JPRB, 0.15145E+00_JPRB, 0.15107E+00_JPRB, 0.15095E+00_JPRB, 0.15071E+00_JPRB, 0.15055E+00_JPRB, 0.15074E+00_JPRB ,&
&  0.15128E+00_JPRB, 0.15184E+00_JPRB, 0.15211E+00_JPRB, 0.15248E+00_JPRB, 0.15286E+00_JPRB, 0.15336E+00_JPRB, 0.15415E+00_JPRB ,&
&  0.15478E+00_JPRB, 0.15551E+00_JPRB, 0.15609E+00_JPRB, 0.15680E+00_JPRB, 0.15813E+00_JPRB, 0.15951E+00_JPRB, 0.16108E+00_JPRB ,&
&  0.16304E+00_JPRB, 0.16508E+00_JPRB, 0.16743E+00_JPRB, 0.16969E+00_JPRB, 0.17276E+00_JPRB, 0.17449E+00_JPRB, 0.17785E+00_JPRB ,&
&  0.18156E+00_JPRB, 0.18261E+00_JPRB, 0.19521E+00_JPRB, 0.19643E+00_JPRB, 0.19815E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.17395E+00_JPRB, 0.17594E+00_JPRB, 0.17726E+00_JPRB, 0.17879E+00_JPRB, 0.17716E+00_JPRB, 0.18914E+00_JPRB, 0.18704E+00_JPRB ,&
&  0.18584E+00_JPRB, 0.18130E+00_JPRB, 0.18107E+00_JPRB, 0.17943E+00_JPRB, 0.17737E+00_JPRB, 0.17506E+00_JPRB, 0.17293E+00_JPRB ,&
&  0.17094E+00_JPRB, 0.16873E+00_JPRB, 0.16717E+00_JPRB, 0.16593E+00_JPRB, 0.16479E+00_JPRB, 0.16375E+00_JPRB, 0.16296E+00_JPRB ,&
&  0.16295E+00_JPRB, 0.16354E+00_JPRB, 0.16466E+00_JPRB, 0.16451E+00_JPRB, 0.16417E+00_JPRB, 0.16372E+00_JPRB, 0.16347E+00_JPRB ,&
&  0.16287E+00_JPRB, 0.16243E+00_JPRB, 0.16202E+00_JPRB, 0.16144E+00_JPRB, 0.16217E+00_JPRB, 0.16294E+00_JPRB, 0.16371E+00_JPRB ,&
&  0.16443E+00_JPRB, 0.16513E+00_JPRB, 0.16517E+00_JPRB, 0.16506E+00_JPRB, 0.16468E+00_JPRB, 0.16419E+00_JPRB, 0.16339E+00_JPRB ,&
&  0.16263E+00_JPRB, 0.16180E+00_JPRB, 0.16082E+00_JPRB, 0.16017E+00_JPRB, 0.15958E+00_JPRB, 0.15904E+00_JPRB, 0.15857E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15816E+00_JPRB, 0.15796E+00_JPRB, 0.15765E+00_JPRB, 0.15726E+00_JPRB, 0.15687E+00_JPRB, 0.15652E+00_JPRB ,&
&  0.15625E+00_JPRB, 0.15574E+00_JPRB, 0.15528E+00_JPRB, 0.15480E+00_JPRB, 0.15443E+00_JPRB, 0.15394E+00_JPRB, 0.15353E+00_JPRB ,&
&  0.15314E+00_JPRB, 0.15285E+00_JPRB, 0.15275E+00_JPRB, 0.15267E+00_JPRB, 0.15261E+00_JPRB, 0.15265E+00_JPRB, 0.15277E+00_JPRB ,&
&  0.15295E+00_JPRB, 0.15315E+00_JPRB, 0.15344E+00_JPRB, 0.15367E+00_JPRB, 0.15388E+00_JPRB, 0.15406E+00_JPRB, 0.15449E+00_JPRB ,&
&  0.15500E+00_JPRB, 0.15554E+00_JPRB, 0.15603E+00_JPRB, 0.15621E+00_JPRB, 0.15702E+00_JPRB, 0.15788E+00_JPRB, 0.15882E+00_JPRB ,&
&  0.15981E+00_JPRB, 0.16108E+00_JPRB, 0.16248E+00_JPRB, 0.16401E+00_JPRB, 0.16582E+00_JPRB, 0.16694E+00_JPRB, 0.16823E+00_JPRB ,&
&  0.16999E+00_JPRB, 0.17290E+00_JPRB, 0.17384E+00_JPRB, 0.17414E+00_JPRB, 0.17422E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.16286E+00_JPRB, 0.16263E+00_JPRB, 0.16237E+00_JPRB, 0.16207E+00_JPRB, 0.16116E+00_JPRB, 0.16070E+00_JPRB, 0.16032E+00_JPRB ,&
&  0.16009E+00_JPRB, 0.15994E+00_JPRB, 0.16001E+00_JPRB, 0.16015E+00_JPRB, 0.16030E+00_JPRB, 0.16044E+00_JPRB, 0.16055E+00_JPRB ,&
&  0.16052E+00_JPRB, 0.16052E+00_JPRB, 0.15980E+00_JPRB, 0.15922E+00_JPRB, 0.15886E+00_JPRB, 0.15859E+00_JPRB, 0.15823E+00_JPRB ,&
&  0.15794E+00_JPRB, 0.15759E+00_JPRB, 0.15718E+00_JPRB, 0.15691E+00_JPRB, 0.15693E+00_JPRB, 0.15678E+00_JPRB, 0.15679E+00_JPRB ,&
&  0.15679E+00_JPRB, 0.15700E+00_JPRB, 0.15709E+00_JPRB, 0.15719E+00_JPRB, 0.15760E+00_JPRB, 0.15797E+00_JPRB, 0.15807E+00_JPRB ,&
&  0.15812E+00_JPRB, 0.15796E+00_JPRB, 0.15779E+00_JPRB, 0.15766E+00_JPRB, 0.15761E+00_JPRB, 0.15761E+00_JPRB, 0.15758E+00_JPRB ,&
&  0.15747E+00_JPRB, 0.15730E+00_JPRB, 0.15707E+00_JPRB, 0.15680E+00_JPRB, 0.15656E+00_JPRB, 0.15646E+00_JPRB, 0.15646E+00_JPRB ,&
&  0.15647E+00_JPRB, 0.15636E+00_JPRB, 0.15616E+00_JPRB, 0.15584E+00_JPRB, 0.15553E+00_JPRB, 0.15522E+00_JPRB, 0.15494E+00_JPRB ,&
&  0.15466E+00_JPRB, 0.15453E+00_JPRB, 0.15438E+00_JPRB, 0.15428E+00_JPRB, 0.15421E+00_JPRB, 0.15409E+00_JPRB, 0.15399E+00_JPRB ,&
&  0.15389E+00_JPRB, 0.15381E+00_JPRB, 0.15371E+00_JPRB, 0.15363E+00_JPRB, 0.15358E+00_JPRB, 0.15354E+00_JPRB, 0.15353E+00_JPRB ,&
&  0.15354E+00_JPRB, 0.15356E+00_JPRB, 0.15359E+00_JPRB, 0.15374E+00_JPRB, 0.15390E+00_JPRB, 0.15412E+00_JPRB, 0.15436E+00_JPRB ,&
&  0.15463E+00_JPRB, 0.15493E+00_JPRB, 0.15524E+00_JPRB, 0.15546E+00_JPRB, 0.15581E+00_JPRB, 0.15617E+00_JPRB, 0.15656E+00_JPRB ,&
&  0.15693E+00_JPRB, 0.15738E+00_JPRB, 0.15790E+00_JPRB, 0.15848E+00_JPRB, 0.15916E+00_JPRB, 0.15951E+00_JPRB, 0.15997E+00_JPRB ,&
&  0.16051E+00_JPRB, 0.16125E+00_JPRB, 0.16165E+00_JPRB, 0.16193E+00_JPRB, 0.16224E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB, 0.15593E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB, 0.51354E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.51691E+00_JPRB, 0.51673E+00_JPRB, 0.51654E+00_JPRB, 0.51641E+00_JPRB, 0.51628E+00_JPRB, 0.51625E+00_JPRB, 0.51617E+00_JPRB ,&
&  0.51626E+00_JPRB, 0.51634E+00_JPRB, 0.51638E+00_JPRB, 0.51632E+00_JPRB, 0.51639E+00_JPRB, 0.51639E+00_JPRB, 0.51636E+00_JPRB ,&
&  0.51623E+00_JPRB, 0.51619E+00_JPRB, 0.51592E+00_JPRB, 0.51561E+00_JPRB, 0.51520E+00_JPRB, 0.51492E+00_JPRB, 0.51447E+00_JPRB ,&
&  0.51413E+00_JPRB, 0.51357E+00_JPRB, 0.51304E+00_JPRB, 0.51216E+00_JPRB, 0.51151E+00_JPRB, 0.51080E+00_JPRB, 0.51018E+00_JPRB ,&
&  0.50964E+00_JPRB, 0.50929E+00_JPRB, 0.50895E+00_JPRB, 0.50848E+00_JPRB, 0.50811E+00_JPRB, 0.50820E+00_JPRB, 0.50839E+00_JPRB ,&
&  0.50857E+00_JPRB, 0.50883E+00_JPRB, 0.50919E+00_JPRB, 0.50932E+00_JPRB, 0.50923E+00_JPRB, 0.50924E+00_JPRB, 0.50918E+00_JPRB ,&
&  0.50903E+00_JPRB, 0.50877E+00_JPRB, 0.50851E+00_JPRB, 0.50844E+00_JPRB, 0.50838E+00_JPRB, 0.50816E+00_JPRB, 0.50799E+00_JPRB ,&
&  0.50785E+00_JPRB, 0.50769E+00_JPRB, 0.50746E+00_JPRB, 0.50709E+00_JPRB, 0.50683E+00_JPRB, 0.50656E+00_JPRB, 0.50626E+00_JPRB ,&
&  0.50593E+00_JPRB, 0.50555E+00_JPRB, 0.50516E+00_JPRB, 0.50473E+00_JPRB, 0.50425E+00_JPRB, 0.50387E+00_JPRB, 0.50359E+00_JPRB ,&
&  0.50340E+00_JPRB, 0.50335E+00_JPRB, 0.50338E+00_JPRB, 0.50345E+00_JPRB, 0.50360E+00_JPRB, 0.50387E+00_JPRB, 0.50425E+00_JPRB ,&
&  0.50472E+00_JPRB, 0.50527E+00_JPRB, 0.50600E+00_JPRB, 0.50689E+00_JPRB, 0.50775E+00_JPRB, 0.50863E+00_JPRB, 0.50960E+00_JPRB ,&
&  0.51047E+00_JPRB, 0.51114E+00_JPRB, 0.51164E+00_JPRB, 0.51213E+00_JPRB, 0.51294E+00_JPRB, 0.51355E+00_JPRB, 0.51402E+00_JPRB ,&
&  0.51443E+00_JPRB, 0.51492E+00_JPRB, 0.51537E+00_JPRB, 0.51587E+00_JPRB, 0.51607E+00_JPRB, 0.51630E+00_JPRB, 0.51652E+00_JPRB ,&
&  0.51674E+00_JPRB, 0.51686E+00_JPRB, 0.51694E+00_JPRB, 0.51699E+00_JPRB, 0.51697E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.51412E+00_JPRB, 0.51374E+00_JPRB, 0.51360E+00_JPRB, 0.51367E+00_JPRB, 0.51385E+00_JPRB, 0.51411E+00_JPRB, 0.51445E+00_JPRB ,&
&  0.51493E+00_JPRB, 0.51518E+00_JPRB, 0.51505E+00_JPRB, 0.51526E+00_JPRB, 0.51588E+00_JPRB, 0.51661E+00_JPRB, 0.51651E+00_JPRB ,&
&  0.51648E+00_JPRB, 0.51638E+00_JPRB, 0.51647E+00_JPRB, 0.51607E+00_JPRB, 0.51543E+00_JPRB, 0.51449E+00_JPRB, 0.51372E+00_JPRB ,&
&  0.51307E+00_JPRB, 0.51211E+00_JPRB, 0.51096E+00_JPRB, 0.50877E+00_JPRB, 0.50689E+00_JPRB, 0.50498E+00_JPRB, 0.50310E+00_JPRB ,&
&  0.50059E+00_JPRB, 0.49842E+00_JPRB, 0.49671E+00_JPRB, 0.49528E+00_JPRB, 0.49734E+00_JPRB, 0.49868E+00_JPRB, 0.49943E+00_JPRB ,&
&  0.49948E+00_JPRB, 0.49725E+00_JPRB, 0.49653E+00_JPRB, 0.49611E+00_JPRB, 0.49584E+00_JPRB, 0.49765E+00_JPRB, 0.49886E+00_JPRB ,&
&  0.49985E+00_JPRB, 0.50069E+00_JPRB, 0.50278E+00_JPRB, 0.50379E+00_JPRB, 0.50450E+00_JPRB, 0.50491E+00_JPRB, 0.50482E+00_JPRB ,&
&  0.50525E+00_JPRB, 0.50569E+00_JPRB, 0.50608E+00_JPRB, 0.50596E+00_JPRB, 0.50581E+00_JPRB, 0.50562E+00_JPRB, 0.50510E+00_JPRB ,&
&  0.50420E+00_JPRB, 0.50342E+00_JPRB, 0.50254E+00_JPRB, 0.50156E+00_JPRB, 0.50044E+00_JPRB, 0.49968E+00_JPRB, 0.49903E+00_JPRB ,&
&  0.49840E+00_JPRB, 0.49795E+00_JPRB, 0.49766E+00_JPRB, 0.49743E+00_JPRB, 0.49732E+00_JPRB, 0.49758E+00_JPRB, 0.49829E+00_JPRB ,&
&  0.49942E+00_JPRB, 0.50081E+00_JPRB, 0.50243E+00_JPRB, 0.50420E+00_JPRB, 0.50575E+00_JPRB, 0.50715E+00_JPRB, 0.50898E+00_JPRB ,&
&  0.51096E+00_JPRB, 0.51251E+00_JPRB, 0.51374E+00_JPRB, 0.51480E+00_JPRB, 0.51620E+00_JPRB, 0.51698E+00_JPRB, 0.51733E+00_JPRB ,&
&  0.51750E+00_JPRB, 0.51811E+00_JPRB, 0.51876E+00_JPRB, 0.51947E+00_JPRB, 0.51968E+00_JPRB, 0.51943E+00_JPRB, 0.51888E+00_JPRB ,&
&  0.51822E+00_JPRB, 0.51758E+00_JPRB, 0.51671E+00_JPRB, 0.51573E+00_JPRB, 0.51484E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.51093E+00_JPRB, 0.51214E+00_JPRB, 0.51387E+00_JPRB, 0.51586E+00_JPRB, 0.51817E+00_JPRB, 0.51977E+00_JPRB, 0.52155E+00_JPRB ,&
&  0.52273E+00_JPRB, 0.52353E+00_JPRB, 0.52347E+00_JPRB, 0.52416E+00_JPRB, 0.52440E+00_JPRB, 0.52489E+00_JPRB, 0.52439E+00_JPRB ,&
&  0.52433E+00_JPRB, 0.52354E+00_JPRB, 0.52423E+00_JPRB, 0.52446E+00_JPRB, 0.52462E+00_JPRB, 0.52471E+00_JPRB, 0.52103E+00_JPRB ,&
&  0.51751E+00_JPRB, 0.51510E+00_JPRB, 0.51085E+00_JPRB, 0.50680E+00_JPRB, 0.50405E+00_JPRB, 0.48494E+00_JPRB, 0.47911E+00_JPRB ,&
&  0.48623E+00_JPRB, 0.49094E+00_JPRB, 0.49519E+00_JPRB, 0.49828E+00_JPRB, 0.49009E+00_JPRB, 0.47953E+00_JPRB, 0.47330E+00_JPRB ,&
&  0.45635E+00_JPRB, 0.46044E+00_JPRB, 0.46334E+00_JPRB, 0.46571E+00_JPRB, 0.47008E+00_JPRB, 0.47160E+00_JPRB, 0.47608E+00_JPRB ,&
&  0.48728E+00_JPRB, 0.49022E+00_JPRB, 0.49426E+00_JPRB, 0.49758E+00_JPRB, 0.49955E+00_JPRB, 0.50194E+00_JPRB, 0.50703E+00_JPRB ,&
&  0.50937E+00_JPRB, 0.51148E+00_JPRB, 0.51295E+00_JPRB, 0.51292E+00_JPRB, 0.51316E+00_JPRB, 0.51258E+00_JPRB, 0.51176E+00_JPRB ,&
&  0.50968E+00_JPRB, 0.50864E+00_JPRB, 0.50787E+00_JPRB, 0.50697E+00_JPRB, 0.50642E+00_JPRB, 0.50445E+00_JPRB, 0.50298E+00_JPRB ,&
&  0.50208E+00_JPRB, 0.50167E+00_JPRB, 0.50167E+00_JPRB, 0.50233E+00_JPRB, 0.50375E+00_JPRB, 0.50559E+00_JPRB, 0.50729E+00_JPRB ,&
&  0.50946E+00_JPRB, 0.51134E+00_JPRB, 0.51361E+00_JPRB, 0.51586E+00_JPRB, 0.51759E+00_JPRB, 0.51898E+00_JPRB, 0.52032E+00_JPRB ,&
&  0.52218E+00_JPRB, 0.52423E+00_JPRB, 0.52526E+00_JPRB, 0.52504E+00_JPRB, 0.52623E+00_JPRB, 0.52708E+00_JPRB, 0.52728E+00_JPRB ,&
&  0.52715E+00_JPRB, 0.52728E+00_JPRB, 0.52630E+00_JPRB, 0.52472E+00_JPRB, 0.52274E+00_JPRB, 0.52091E+00_JPRB, 0.51896E+00_JPRB ,&
&  0.51693E+00_JPRB, 0.51499E+00_JPRB, 0.51326E+00_JPRB, 0.51170E+00_JPRB, 0.51079E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.51542E+00_JPRB, 0.51891E+00_JPRB, 0.52339E+00_JPRB, 0.52603E+00_JPRB, 0.52941E+00_JPRB, 0.53070E+00_JPRB, 0.53273E+00_JPRB ,&
&  0.53331E+00_JPRB, 0.53429E+00_JPRB, 0.53492E+00_JPRB, 0.53516E+00_JPRB, 0.53504E+00_JPRB, 0.53443E+00_JPRB, 0.53395E+00_JPRB ,&
&  0.53389E+00_JPRB, 0.53341E+00_JPRB, 0.53407E+00_JPRB, 0.53431E+00_JPRB, 0.53440E+00_JPRB, 0.53420E+00_JPRB, 0.53003E+00_JPRB ,&
&  0.52505E+00_JPRB, 0.52049E+00_JPRB, 0.49639E+00_JPRB, 0.49425E+00_JPRB, 0.49021E+00_JPRB, 0.48810E+00_JPRB, 0.48927E+00_JPRB ,&
&  0.48664E+00_JPRB, 0.48330E+00_JPRB, 0.47153E+00_JPRB, 0.45964E+00_JPRB, 0.45044E+00_JPRB, 0.44034E+00_JPRB, 0.43936E+00_JPRB ,&
&  0.43807E+00_JPRB, 0.44114E+00_JPRB, 0.44427E+00_JPRB, 0.44644E+00_JPRB, 0.45293E+00_JPRB, 0.45695E+00_JPRB, 0.46452E+00_JPRB ,&
&  0.47400E+00_JPRB, 0.48057E+00_JPRB, 0.48076E+00_JPRB, 0.47936E+00_JPRB, 0.49216E+00_JPRB, 0.50071E+00_JPRB, 0.50846E+00_JPRB ,&
&  0.51328E+00_JPRB, 0.51441E+00_JPRB, 0.51707E+00_JPRB, 0.51738E+00_JPRB, 0.51826E+00_JPRB, 0.52057E+00_JPRB, 0.52120E+00_JPRB ,&
&  0.52266E+00_JPRB, 0.52227E+00_JPRB, 0.52081E+00_JPRB, 0.51867E+00_JPRB, 0.51489E+00_JPRB, 0.51248E+00_JPRB, 0.51101E+00_JPRB ,&
&  0.51116E+00_JPRB, 0.51283E+00_JPRB, 0.51511E+00_JPRB, 0.51843E+00_JPRB, 0.52190E+00_JPRB, 0.52444E+00_JPRB, 0.52582E+00_JPRB ,&
&  0.52665E+00_JPRB, 0.52685E+00_JPRB, 0.52968E+00_JPRB, 0.52947E+00_JPRB, 0.53071E+00_JPRB, 0.53540E+00_JPRB, 0.53463E+00_JPRB ,&
&  0.53549E+00_JPRB, 0.53817E+00_JPRB, 0.53985E+00_JPRB, 0.54126E+00_JPRB, 0.54224E+00_JPRB, 0.54254E+00_JPRB, 0.54306E+00_JPRB ,&
&  0.54289E+00_JPRB, 0.54174E+00_JPRB, 0.53923E+00_JPRB, 0.53590E+00_JPRB, 0.53230E+00_JPRB, 0.52863E+00_JPRB, 0.52394E+00_JPRB ,&
&  0.51954E+00_JPRB, 0.51540E+00_JPRB, 0.51283E+00_JPRB, 0.51141E+00_JPRB, 0.51275E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.52747E+00_JPRB, 0.53115E+00_JPRB, 0.53645E+00_JPRB, 0.53833E+00_JPRB, 0.54132E+00_JPRB, 0.54241E+00_JPRB, 0.54354E+00_JPRB ,&
&  0.54404E+00_JPRB, 0.54462E+00_JPRB, 0.54447E+00_JPRB, 0.54449E+00_JPRB, 0.54400E+00_JPRB, 0.54383E+00_JPRB, 0.54332E+00_JPRB ,&
&  0.54287E+00_JPRB, 0.54226E+00_JPRB, 0.54198E+00_JPRB, 0.54159E+00_JPRB, 0.54199E+00_JPRB, 0.54209E+00_JPRB, 0.54003E+00_JPRB ,&
&  0.53607E+00_JPRB, 0.52736E+00_JPRB, 0.50904E+00_JPRB, 0.50522E+00_JPRB, 0.50159E+00_JPRB, 0.50185E+00_JPRB, 0.50057E+00_JPRB ,&
&  0.49591E+00_JPRB, 0.46152E+00_JPRB, 0.46350E+00_JPRB, 0.45550E+00_JPRB, 0.45188E+00_JPRB, 0.44820E+00_JPRB, 0.43942E+00_JPRB ,&
&  0.43632E+00_JPRB, 0.43295E+00_JPRB, 0.43241E+00_JPRB, 0.43713E+00_JPRB, 0.44179E+00_JPRB, 0.44902E+00_JPRB, 0.45776E+00_JPRB ,&
&  0.45788E+00_JPRB, 0.47137E+00_JPRB, 0.47912E+00_JPRB, 0.48396E+00_JPRB, 0.49675E+00_JPRB, 0.50716E+00_JPRB, 0.51004E+00_JPRB ,&
&  0.51347E+00_JPRB, 0.51296E+00_JPRB, 0.51599E+00_JPRB, 0.52145E+00_JPRB, 0.52755E+00_JPRB, 0.52794E+00_JPRB, 0.52879E+00_JPRB ,&
&  0.52882E+00_JPRB, 0.52881E+00_JPRB, 0.52865E+00_JPRB, 0.52778E+00_JPRB, 0.52730E+00_JPRB, 0.52871E+00_JPRB, 0.53083E+00_JPRB ,&
&  0.53516E+00_JPRB, 0.53951E+00_JPRB, 0.54431E+00_JPRB, 0.54834E+00_JPRB, 0.55144E+00_JPRB, 0.55407E+00_JPRB, 0.55671E+00_JPRB ,&
&  0.55925E+00_JPRB, 0.56092E+00_JPRB, 0.56049E+00_JPRB, 0.56017E+00_JPRB, 0.55841E+00_JPRB, 0.55553E+00_JPRB, 0.55500E+00_JPRB ,&
&  0.55697E+00_JPRB, 0.55794E+00_JPRB, 0.55829E+00_JPRB, 0.55878E+00_JPRB, 0.56003E+00_JPRB, 0.56040E+00_JPRB, 0.56050E+00_JPRB ,&
&  0.56001E+00_JPRB, 0.55848E+00_JPRB, 0.55585E+00_JPRB, 0.55030E+00_JPRB, 0.54424E+00_JPRB, 0.54019E+00_JPRB, 0.53363E+00_JPRB ,&
&  0.52756E+00_JPRB, 0.52137E+00_JPRB, 0.51919E+00_JPRB, 0.51837E+00_JPRB, 0.52229E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.54305E+00_JPRB, 0.54857E+00_JPRB, 0.55448E+00_JPRB, 0.55744E+00_JPRB, 0.56041E+00_JPRB, 0.56092E+00_JPRB, 0.56205E+00_JPRB ,&
&  0.56255E+00_JPRB, 0.56348E+00_JPRB, 0.56367E+00_JPRB, 0.56368E+00_JPRB, 0.56299E+00_JPRB, 0.56211E+00_JPRB, 0.56059E+00_JPRB ,&
&  0.55820E+00_JPRB, 0.55588E+00_JPRB, 0.55261E+00_JPRB, 0.55007E+00_JPRB, 0.54869E+00_JPRB, 0.54945E+00_JPRB, 0.54832E+00_JPRB ,&
&  0.54452E+00_JPRB, 0.53652E+00_JPRB, 0.52242E+00_JPRB, 0.51592E+00_JPRB, 0.51475E+00_JPRB, 0.50693E+00_JPRB, 0.50562E+00_JPRB ,&
&  0.50303E+00_JPRB, 0.50218E+00_JPRB, 0.49653E+00_JPRB, 0.49323E+00_JPRB, 0.48301E+00_JPRB, 0.43734E+00_JPRB, 0.43333E+00_JPRB ,&
&  0.43027E+00_JPRB, 0.42451E+00_JPRB, 0.42389E+00_JPRB, 0.42542E+00_JPRB, 0.43031E+00_JPRB, 0.44612E+00_JPRB, 0.46005E+00_JPRB ,&
&  0.45952E+00_JPRB, 0.47547E+00_JPRB, 0.48169E+00_JPRB, 0.48787E+00_JPRB, 0.49910E+00_JPRB, 0.50926E+00_JPRB, 0.51172E+00_JPRB ,&
&  0.51560E+00_JPRB, 0.51954E+00_JPRB, 0.52653E+00_JPRB, 0.53373E+00_JPRB, 0.53897E+00_JPRB, 0.54180E+00_JPRB, 0.54545E+00_JPRB ,&
&  0.54922E+00_JPRB, 0.55285E+00_JPRB, 0.55734E+00_JPRB, 0.55962E+00_JPRB, 0.55998E+00_JPRB, 0.55963E+00_JPRB, 0.56088E+00_JPRB ,&
&  0.56546E+00_JPRB, 0.56917E+00_JPRB, 0.57646E+00_JPRB, 0.57552E+00_JPRB, 0.57707E+00_JPRB, 0.57788E+00_JPRB, 0.58201E+00_JPRB ,&
&  0.58433E+00_JPRB, 0.59311E+00_JPRB, 0.59927E+00_JPRB, 0.60273E+00_JPRB, 0.59581E+00_JPRB, 0.58909E+00_JPRB, 0.58431E+00_JPRB ,&
&  0.58215E+00_JPRB, 0.58226E+00_JPRB, 0.57736E+00_JPRB, 0.57585E+00_JPRB, 0.57568E+00_JPRB, 0.57605E+00_JPRB, 0.57661E+00_JPRB ,&
&  0.57613E+00_JPRB, 0.57317E+00_JPRB, 0.56823E+00_JPRB, 0.56093E+00_JPRB, 0.55434E+00_JPRB, 0.54831E+00_JPRB, 0.53759E+00_JPRB ,&
&  0.52978E+00_JPRB, 0.52369E+00_JPRB, 0.52376E+00_JPRB, 0.52651E+00_JPRB, 0.53405E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.55623E+00_JPRB, 0.56465E+00_JPRB, 0.57301E+00_JPRB, 0.57992E+00_JPRB, 0.58287E+00_JPRB, 0.58287E+00_JPRB, 0.58085E+00_JPRB ,&
&  0.57863E+00_JPRB, 0.57773E+00_JPRB, 0.57669E+00_JPRB, 0.57597E+00_JPRB, 0.57521E+00_JPRB, 0.57360E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.56962E+00_JPRB, 0.55819E+00_JPRB, 0.55106E+00_JPRB, 0.54966E+00_JPRB, 0.54843E+00_JPRB, 0.54879E+00_JPRB, 0.54984E+00_JPRB ,&
&  0.54592E+00_JPRB, 0.53835E+00_JPRB, 0.53216E+00_JPRB, 0.52361E+00_JPRB, 0.51504E+00_JPRB, 0.51659E+00_JPRB, 0.51536E+00_JPRB ,&
&  0.50297E+00_JPRB, 0.49323E+00_JPRB, 0.49772E+00_JPRB, 0.49906E+00_JPRB, 0.48071E+00_JPRB, 0.45959E+00_JPRB, 0.44035E+00_JPRB ,&
&  0.43999E+00_JPRB, 0.42998E+00_JPRB, 0.42817E+00_JPRB, 0.42959E+00_JPRB, 0.43667E+00_JPRB, 0.44464E+00_JPRB, 0.45927E+00_JPRB ,&
&  0.46434E+00_JPRB, 0.46891E+00_JPRB, 0.48114E+00_JPRB, 0.49417E+00_JPRB, 0.50341E+00_JPRB, 0.51302E+00_JPRB, 0.51932E+00_JPRB ,&
&  0.52893E+00_JPRB, 0.53887E+00_JPRB, 0.54770E+00_JPRB, 0.55526E+00_JPRB, 0.56328E+00_JPRB, 0.56609E+00_JPRB, 0.57051E+00_JPRB ,&
&  0.59747E+00_JPRB, 0.61955E+00_JPRB, 0.63012E+00_JPRB, 0.63749E+00_JPRB, 0.63835E+00_JPRB, 0.63511E+00_JPRB, 0.62368E+00_JPRB ,&
&  0.61844E+00_JPRB, 0.62068E+00_JPRB, 0.62089E+00_JPRB, 0.62253E+00_JPRB, 0.62248E+00_JPRB, 0.62014E+00_JPRB, 0.61370E+00_JPRB ,&
&  0.60394E+00_JPRB, 0.60788E+00_JPRB, 0.66965E+00_JPRB, 0.68071E+00_JPRB, 0.66974E+00_JPRB, 0.64946E+00_JPRB, 0.62771E+00_JPRB ,&
&  0.61199E+00_JPRB, 0.60230E+00_JPRB, 0.59667E+00_JPRB, 0.59233E+00_JPRB, 0.58922E+00_JPRB, 0.58449E+00_JPRB, 0.58474E+00_JPRB ,&
&  0.58227E+00_JPRB, 0.57690E+00_JPRB, 0.57094E+00_JPRB, 0.56384E+00_JPRB, 0.55690E+00_JPRB, 0.55087E+00_JPRB, 0.54230E+00_JPRB ,&
&  0.53540E+00_JPRB, 0.53042E+00_JPRB, 0.53226E+00_JPRB, 0.53805E+00_JPRB, 0.54750E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.56902E+00_JPRB, 0.57902E+00_JPRB, 0.59038E+00_JPRB, 0.59854E+00_JPRB, 0.59954E+00_JPRB, 0.58766E+00_JPRB, 0.57709E+00_JPRB ,&
&  0.57030E+00_JPRB, 0.56767E+00_JPRB, 0.56660E+00_JPRB, 0.56480E+00_JPRB, 0.56393E+00_JPRB, 0.56308E+00_JPRB, 0.56242E+00_JPRB ,&
&  0.56212E+00_JPRB, 0.56001E+00_JPRB, 0.55484E+00_JPRB, 0.54708E+00_JPRB, 0.54249E+00_JPRB, 0.54179E+00_JPRB, 0.54293E+00_JPRB ,&
&  0.54220E+00_JPRB, 0.53964E+00_JPRB, 0.53841E+00_JPRB, 0.52932E+00_JPRB, 0.52017E+00_JPRB, 0.50846E+00_JPRB, 0.50113E+00_JPRB ,&
&  0.50150E+00_JPRB, 0.50289E+00_JPRB, 0.49481E+00_JPRB, 0.49513E+00_JPRB, 0.49690E+00_JPRB, 0.47675E+00_JPRB, 0.45614E+00_JPRB ,&
&  0.44473E+00_JPRB, 0.43693E+00_JPRB, 0.43345E+00_JPRB, 0.43476E+00_JPRB, 0.45110E+00_JPRB, 0.47476E+00_JPRB, 0.48224E+00_JPRB ,&
&  0.46231E+00_JPRB, 0.47569E+00_JPRB, 0.49320E+00_JPRB, 0.50394E+00_JPRB, 0.51400E+00_JPRB, 0.52761E+00_JPRB, 0.54137E+00_JPRB ,&
&  0.55337E+00_JPRB, 0.56338E+00_JPRB, 0.56927E+00_JPRB, 0.58187E+00_JPRB, 0.58765E+00_JPRB, 0.58204E+00_JPRB, 0.59191E+00_JPRB ,&
&  0.66403E+00_JPRB, 0.72558E+00_JPRB, 0.74545E+00_JPRB, 0.74624E+00_JPRB, 0.73853E+00_JPRB, 0.72316E+00_JPRB, 0.71195E+00_JPRB ,&
&  0.70172E+00_JPRB, 0.69133E+00_JPRB, 0.68231E+00_JPRB, 0.67433E+00_JPRB, 0.66705E+00_JPRB, 0.65820E+00_JPRB, 0.65259E+00_JPRB ,&
&  0.64710E+00_JPRB, 0.66198E+00_JPRB, 0.84137E+00_JPRB, 0.81200E+00_JPRB, 0.76229E+00_JPRB, 0.72821E+00_JPRB, 0.69425E+00_JPRB ,&
&  0.66893E+00_JPRB, 0.65190E+00_JPRB, 0.64158E+00_JPRB, 0.63001E+00_JPRB, 0.60587E+00_JPRB, 0.58777E+00_JPRB, 0.57919E+00_JPRB ,&
&  0.57471E+00_JPRB, 0.56711E+00_JPRB, 0.56046E+00_JPRB, 0.55437E+00_JPRB, 0.54943E+00_JPRB, 0.54686E+00_JPRB, 0.54259E+00_JPRB ,&
&  0.53937E+00_JPRB, 0.54153E+00_JPRB, 0.54660E+00_JPRB, 0.55530E+00_JPRB, 0.56242E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.57845E+00_JPRB, 0.59943E+00_JPRB, 0.61496E+00_JPRB, 0.62037E+00_JPRB, 0.61673E+00_JPRB, 0.58635E+00_JPRB, 0.55901E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.53841E+00_JPRB, 0.53909E+00_JPRB, 0.54003E+00_JPRB, 0.54207E+00_JPRB, 0.54491E+00_JPRB, 0.54669E+00_JPRB ,&
&  0.54622E+00_JPRB, 0.54574E+00_JPRB, 0.54541E+00_JPRB, 0.54324E+00_JPRB, 0.53951E+00_JPRB, 0.53669E+00_JPRB, 0.53627E+00_JPRB ,&
&  0.53578E+00_JPRB, 0.53340E+00_JPRB, 0.53108E+00_JPRB, 0.52580E+00_JPRB, 0.52269E+00_JPRB, 0.51880E+00_JPRB, 0.51189E+00_JPRB ,&
&  0.50187E+00_JPRB, 0.50163E+00_JPRB, 0.50824E+00_JPRB, 0.50939E+00_JPRB, 0.51044E+00_JPRB, 0.49706E+00_JPRB, 0.47313E+00_JPRB ,&
&  0.45721E+00_JPRB, 0.45066E+00_JPRB, 0.45860E+00_JPRB, 0.48486E+00_JPRB, 0.50250E+00_JPRB, 0.48301E+00_JPRB, 0.49321E+00_JPRB ,&
&  0.49673E+00_JPRB, 0.49949E+00_JPRB, 0.50525E+00_JPRB, 0.51944E+00_JPRB, 0.53636E+00_JPRB, 0.55241E+00_JPRB, 0.56509E+00_JPRB ,&
&  0.57699E+00_JPRB, 0.58662E+00_JPRB, 0.59618E+00_JPRB, 0.58458E+00_JPRB, 0.58448E+00_JPRB, 0.61747E+00_JPRB, 0.65911E+00_JPRB ,&
&  0.70674E+00_JPRB, 0.73907E+00_JPRB, 0.75009E+00_JPRB, 0.77964E+00_JPRB, 0.79580E+00_JPRB, 0.79643E+00_JPRB, 0.78763E+00_JPRB ,&
&  0.77624E+00_JPRB, 0.75193E+00_JPRB, 0.73053E+00_JPRB, 0.71784E+00_JPRB, 0.71627E+00_JPRB, 0.72213E+00_JPRB, 0.72223E+00_JPRB ,&
&  0.71880E+00_JPRB, 0.69984E+00_JPRB, 0.68286E+00_JPRB, 0.64804E+00_JPRB, 0.64717E+00_JPRB, 0.66290E+00_JPRB, 0.67669E+00_JPRB ,&
&  0.67311E+00_JPRB, 0.66472E+00_JPRB, 0.65740E+00_JPRB, 0.64529E+00_JPRB, 0.60517E+00_JPRB, 0.57791E+00_JPRB, 0.56141E+00_JPRB ,&
&  0.55184E+00_JPRB, 0.53950E+00_JPRB, 0.53830E+00_JPRB, 0.53539E+00_JPRB, 0.53793E+00_JPRB, 0.54161E+00_JPRB, 0.53875E+00_JPRB ,&
&  0.53748E+00_JPRB, 0.53876E+00_JPRB, 0.54521E+00_JPRB, 0.55267E+00_JPRB, 0.56414E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.63637E+00_JPRB, 0.64461E+00_JPRB, 0.64152E+00_JPRB, 0.64195E+00_JPRB, 0.62761E+00_JPRB, 0.60978E+00_JPRB, 0.59051E+00_JPRB ,&
&  0.58165E+00_JPRB, 0.56446E+00_JPRB, 0.54674E+00_JPRB, 0.52971E+00_JPRB, 0.52023E+00_JPRB, 0.51641E+00_JPRB, 0.51933E+00_JPRB ,&
&  0.52573E+00_JPRB, 0.53338E+00_JPRB, 0.53846E+00_JPRB, 0.53698E+00_JPRB, 0.53312E+00_JPRB, 0.53140E+00_JPRB, 0.53210E+00_JPRB ,&
&  0.53127E+00_JPRB, 0.52954E+00_JPRB, 0.52696E+00_JPRB, 0.52372E+00_JPRB, 0.52067E+00_JPRB, 0.51789E+00_JPRB, 0.51730E+00_JPRB ,&
&  0.51755E+00_JPRB, 0.51735E+00_JPRB, 0.52093E+00_JPRB, 0.52201E+00_JPRB, 0.52465E+00_JPRB, 0.51328E+00_JPRB, 0.49252E+00_JPRB ,&
&  0.47528E+00_JPRB, 0.47452E+00_JPRB, 0.50658E+00_JPRB, 0.49647E+00_JPRB, 0.51061E+00_JPRB, 0.52157E+00_JPRB, 0.52089E+00_JPRB ,&
&  0.52011E+00_JPRB, 0.52321E+00_JPRB, 0.53680E+00_JPRB, 0.55404E+00_JPRB, 0.56824E+00_JPRB, 0.58386E+00_JPRB, 0.59734E+00_JPRB ,&
&  0.61366E+00_JPRB, 0.62969E+00_JPRB, 0.63781E+00_JPRB, 0.62479E+00_JPRB, 0.67322E+00_JPRB, 0.83494E+00_JPRB, 0.86323E+00_JPRB ,&
&  0.88317E+00_JPRB, 0.89819E+00_JPRB, 0.92801E+00_JPRB, 0.95069E+00_JPRB, 0.92724E+00_JPRB, 0.90749E+00_JPRB, 0.89366E+00_JPRB ,&
&  0.86316E+00_JPRB, 0.84250E+00_JPRB, 0.82805E+00_JPRB, 0.83831E+00_JPRB, 0.84079E+00_JPRB, 0.84542E+00_JPRB, 0.85380E+00_JPRB ,&
&  0.85906E+00_JPRB, 0.84227E+00_JPRB, 0.82147E+00_JPRB, 0.78569E+00_JPRB, 0.76680E+00_JPRB, 0.75378E+00_JPRB, 0.74656E+00_JPRB ,&
&  0.73345E+00_JPRB, 0.72455E+00_JPRB, 0.71291E+00_JPRB, 0.69206E+00_JPRB, 0.64277E+00_JPRB, 0.59568E+00_JPRB, 0.56685E+00_JPRB ,&
&  0.54640E+00_JPRB, 0.52945E+00_JPRB, 0.52718E+00_JPRB, 0.53029E+00_JPRB, 0.53786E+00_JPRB, 0.54650E+00_JPRB, 0.54451E+00_JPRB ,&
&  0.54849E+00_JPRB, 0.55637E+00_JPRB, 0.57312E+00_JPRB, 0.59539E+00_JPRB, 0.61635E+00_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.68739E+00_JPRB, 0.68873E+00_JPRB, 0.67940E+00_JPRB, 0.67055E+00_JPRB, 0.65911E+00_JPRB, 0.63560E+00_JPRB, 0.60087E+00_JPRB ,&
&  0.59149E+00_JPRB, 0.55903E+00_JPRB, 0.53918E+00_JPRB, 0.53234E+00_JPRB, 0.52809E+00_JPRB, 0.52005E+00_JPRB, 0.50997E+00_JPRB ,&
&  0.51060E+00_JPRB, 0.52029E+00_JPRB, 0.53107E+00_JPRB, 0.53534E+00_JPRB, 0.53724E+00_JPRB, 0.53534E+00_JPRB, 0.53498E+00_JPRB ,&
&  0.53467E+00_JPRB, 0.53460E+00_JPRB, 0.53519E+00_JPRB, 0.53047E+00_JPRB, 0.52683E+00_JPRB, 0.52559E+00_JPRB, 0.52419E+00_JPRB ,&
&  0.52826E+00_JPRB, 0.52982E+00_JPRB, 0.53298E+00_JPRB, 0.53194E+00_JPRB, 0.52932E+00_JPRB, 0.52966E+00_JPRB, 0.50443E+00_JPRB ,&
&  0.49107E+00_JPRB, 0.50657E+00_JPRB, 0.50459E+00_JPRB, 0.51333E+00_JPRB, 0.52627E+00_JPRB, 0.53547E+00_JPRB, 0.54377E+00_JPRB ,&
&  0.54998E+00_JPRB, 0.55825E+00_JPRB, 0.56897E+00_JPRB, 0.58262E+00_JPRB, 0.60020E+00_JPRB, 0.61786E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.66191E+00_JPRB, 0.64021E+00_JPRB, 0.64234E+00_JPRB, 0.69801E+00_JPRB, 0.89090E+00_JPRB, 0.95439E+00_JPRB, 0.10060E+01_JPRB ,&
&  0.10664E+01_JPRB, 0.10948E+01_JPRB, 0.11006E+01_JPRB, 0.11106E+01_JPRB, 0.10929E+01_JPRB, 0.10740E+01_JPRB, 0.10757E+01_JPRB ,&
&  0.10567E+01_JPRB, 0.10589E+01_JPRB, 0.10567E+01_JPRB, 0.10732E+01_JPRB, 0.10745E+01_JPRB, 0.10898E+01_JPRB, 0.11019E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.10660E+01_JPRB, 0.10102E+01_JPRB, 0.95234E+00_JPRB, 0.89865E+00_JPRB, 0.86775E+00_JPRB, 0.84213E+00_JPRB ,&
&  0.81393E+00_JPRB, 0.79317E+00_JPRB, 0.76792E+00_JPRB, 0.73188E+00_JPRB, 0.68522E+00_JPRB, 0.64723E+00_JPRB, 0.61688E+00_JPRB ,&
&  0.57500E+00_JPRB, 0.55715E+00_JPRB, 0.55518E+00_JPRB, 0.55666E+00_JPRB, 0.56451E+00_JPRB, 0.58136E+00_JPRB, 0.58067E+00_JPRB ,&
&  0.59863E+00_JPRB, 0.61390E+00_JPRB, 0.62202E+00_JPRB, 0.63773E+00_JPRB, 0.66000E+00_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.73567E+00_JPRB, 0.73342E+00_JPRB, 0.72201E+00_JPRB, 0.71017E+00_JPRB, 0.69606E+00_JPRB, 0.66157E+00_JPRB, 0.63624E+00_JPRB ,&
&  0.66164E+00_JPRB, 0.62399E+00_JPRB, 0.61398E+00_JPRB, 0.56820E+00_JPRB, 0.54505E+00_JPRB, 0.53180E+00_JPRB, 0.51983E+00_JPRB ,&
&  0.51785E+00_JPRB, 0.52477E+00_JPRB, 0.53198E+00_JPRB, 0.53893E+00_JPRB, 0.54214E+00_JPRB, 0.54097E+00_JPRB, 0.53848E+00_JPRB ,&
&  0.53803E+00_JPRB, 0.54157E+00_JPRB, 0.54471E+00_JPRB, 0.54557E+00_JPRB, 0.54606E+00_JPRB, 0.54570E+00_JPRB, 0.54117E+00_JPRB ,&
&  0.53709E+00_JPRB, 0.53772E+00_JPRB, 0.54387E+00_JPRB, 0.54294E+00_JPRB, 0.54308E+00_JPRB, 0.54184E+00_JPRB, 0.53991E+00_JPRB ,&
&  0.52664E+00_JPRB, 0.52159E+00_JPRB, 0.52819E+00_JPRB, 0.53849E+00_JPRB, 0.55068E+00_JPRB, 0.56118E+00_JPRB, 0.57105E+00_JPRB ,&
&  0.58313E+00_JPRB, 0.59750E+00_JPRB, 0.61402E+00_JPRB, 0.63071E+00_JPRB, 0.64980E+00_JPRB, 0.67217E+00_JPRB, 0.68897E+00_JPRB ,&
&  0.74200E+00_JPRB, 0.70153E+00_JPRB, 0.81600E+00_JPRB, 0.91658E+00_JPRB, 0.10898E+01_JPRB, 0.12253E+01_JPRB, 0.12504E+01_JPRB ,&
&  0.13625E+01_JPRB, 0.13834E+01_JPRB, 0.13936E+01_JPRB, 0.13681E+01_JPRB, 0.13383E+01_JPRB, 0.13242E+01_JPRB, 0.13313E+01_JPRB ,&
&  0.13503E+01_JPRB, 0.13341E+01_JPRB, 0.14221E+01_JPRB, 0.14197E+01_JPRB, 0.14089E+01_JPRB, 0.14164E+01_JPRB, 0.14003E+01_JPRB ,&
&  0.13612E+01_JPRB, 0.12961E+01_JPRB, 0.12092E+01_JPRB, 0.10725E+01_JPRB, 0.10114E+01_JPRB, 0.96037E+00_JPRB, 0.88548E+00_JPRB ,&
&  0.84374E+00_JPRB, 0.79916E+00_JPRB, 0.75884E+00_JPRB, 0.75005E+00_JPRB, 0.69343E+00_JPRB, 0.65077E+00_JPRB, 0.62393E+00_JPRB ,&
&  0.59156E+00_JPRB, 0.58151E+00_JPRB, 0.59316E+00_JPRB, 0.60839E+00_JPRB, 0.61838E+00_JPRB, 0.62278E+00_JPRB, 0.64570E+00_JPRB ,&
&  0.63648E+00_JPRB, 0.65920E+00_JPRB, 0.70226E+00_JPRB, 0.71684E+00_JPRB, 0.72741E+00_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.80503E+00_JPRB, 0.79706E+00_JPRB, 0.78250E+00_JPRB, 0.76587E+00_JPRB, 0.73620E+00_JPRB, 0.69875E+00_JPRB, 0.66333E+00_JPRB ,&
&  0.67247E+00_JPRB, 0.66043E+00_JPRB, 0.64534E+00_JPRB, 0.62422E+00_JPRB, 0.60119E+00_JPRB, 0.57005E+00_JPRB, 0.53278E+00_JPRB ,&
&  0.52440E+00_JPRB, 0.53579E+00_JPRB, 0.54774E+00_JPRB, 0.55633E+00_JPRB, 0.55633E+00_JPRB, 0.55492E+00_JPRB, 0.55498E+00_JPRB ,&
&  0.55677E+00_JPRB, 0.56003E+00_JPRB, 0.56463E+00_JPRB, 0.56811E+00_JPRB, 0.56828E+00_JPRB, 0.56864E+00_JPRB, 0.56812E+00_JPRB ,&
&  0.55368E+00_JPRB, 0.54493E+00_JPRB, 0.54242E+00_JPRB, 0.54109E+00_JPRB, 0.56392E+00_JPRB, 0.57685E+00_JPRB, 0.58787E+00_JPRB ,&
&  0.59054E+00_JPRB, 0.58845E+00_JPRB, 0.58995E+00_JPRB, 0.60217E+00_JPRB, 0.61435E+00_JPRB, 0.62041E+00_JPRB, 0.63007E+00_JPRB ,&
&  0.64480E+00_JPRB, 0.66264E+00_JPRB, 0.68270E+00_JPRB, 0.70893E+00_JPRB, 0.74267E+00_JPRB, 0.74344E+00_JPRB, 0.78839E+00_JPRB ,&
&  0.86407E+00_JPRB, 0.99883E+00_JPRB, 0.11475E+01_JPRB, 0.11801E+01_JPRB, 0.13463E+01_JPRB, 0.14697E+01_JPRB, 0.14928E+01_JPRB ,&
&  0.15734E+01_JPRB, 0.15909E+01_JPRB, 0.16733E+01_JPRB, 0.16900E+01_JPRB, 0.17133E+01_JPRB, 0.17303E+01_JPRB, 0.17324E+01_JPRB ,&
&  0.17480E+01_JPRB, 0.18033E+01_JPRB, 0.18536E+01_JPRB, 0.18631E+01_JPRB, 0.18190E+01_JPRB, 0.17495E+01_JPRB, 0.16527E+01_JPRB ,&
&  0.15583E+01_JPRB, 0.14522E+01_JPRB, 0.13464E+01_JPRB, 0.12618E+01_JPRB, 0.11631E+01_JPRB, 0.10596E+01_JPRB, 0.96055E+00_JPRB ,&
&  0.86589E+00_JPRB, 0.76050E+00_JPRB, 0.71473E+00_JPRB, 0.71182E+00_JPRB, 0.69635E+00_JPRB, 0.64489E+00_JPRB, 0.61741E+00_JPRB ,&
&  0.60972E+00_JPRB, 0.62602E+00_JPRB, 0.65132E+00_JPRB, 0.66541E+00_JPRB, 0.69160E+00_JPRB, 0.71581E+00_JPRB, 0.68374E+00_JPRB ,&
&  0.66488E+00_JPRB, 0.72820E+00_JPRB, 0.76685E+00_JPRB, 0.78912E+00_JPRB, 0.80206E+00_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.86864E+00_JPRB, 0.85315E+00_JPRB, 0.83717E+00_JPRB, 0.82052E+00_JPRB, 0.78941E+00_JPRB, 0.74669E+00_JPRB, 0.71630E+00_JPRB ,&
&  0.70486E+00_JPRB, 0.68512E+00_JPRB, 0.66466E+00_JPRB, 0.64494E+00_JPRB, 0.62848E+00_JPRB, 0.61017E+00_JPRB, 0.58818E+00_JPRB ,&
&  0.57030E+00_JPRB, 0.57535E+00_JPRB, 0.57106E+00_JPRB, 0.57205E+00_JPRB, 0.57864E+00_JPRB, 0.57531E+00_JPRB, 0.57521E+00_JPRB ,&
&  0.58303E+00_JPRB, 0.58652E+00_JPRB, 0.58661E+00_JPRB, 0.58448E+00_JPRB, 0.58372E+00_JPRB, 0.58705E+00_JPRB, 0.58728E+00_JPRB ,&
&  0.56825E+00_JPRB, 0.55293E+00_JPRB, 0.54250E+00_JPRB, 0.54396E+00_JPRB, 0.56968E+00_JPRB, 0.60794E+00_JPRB, 0.63897E+00_JPRB ,&
&  0.66377E+00_JPRB, 0.68245E+00_JPRB, 0.69006E+00_JPRB, 0.69654E+00_JPRB, 0.69785E+00_JPRB, 0.69646E+00_JPRB, 0.69749E+00_JPRB ,&
&  0.70322E+00_JPRB, 0.71673E+00_JPRB, 0.73731E+00_JPRB, 0.77493E+00_JPRB, 0.80183E+00_JPRB, 0.83196E+00_JPRB, 0.92168E+00_JPRB ,&
&  0.10695E+01_JPRB, 0.12534E+01_JPRB, 0.13608E+01_JPRB, 0.15467E+01_JPRB, 0.17024E+01_JPRB, 0.17598E+01_JPRB, 0.17747E+01_JPRB ,&
&  0.18395E+01_JPRB, 0.18220E+01_JPRB, 0.19655E+01_JPRB, 0.19562E+01_JPRB, 0.19709E+01_JPRB, 0.20573E+01_JPRB, 0.20556E+01_JPRB ,&
&  0.20354E+01_JPRB, 0.20144E+01_JPRB, 0.21001E+01_JPRB, 0.21019E+01_JPRB, 0.20651E+01_JPRB, 0.19988E+01_JPRB, 0.18760E+01_JPRB ,&
&  0.17544E+01_JPRB, 0.16221E+01_JPRB, 0.13799E+01_JPRB, 0.11976E+01_JPRB, 0.10175E+01_JPRB, 0.99873E+00_JPRB, 0.91361E+00_JPRB ,&
&  0.82549E+00_JPRB, 0.71793E+00_JPRB, 0.71020E+00_JPRB, 0.71146E+00_JPRB, 0.72542E+00_JPRB, 0.70212E+00_JPRB, 0.65948E+00_JPRB ,&
&  0.65520E+00_JPRB, 0.68475E+00_JPRB, 0.69502E+00_JPRB, 0.71507E+00_JPRB, 0.75564E+00_JPRB, 0.81024E+00_JPRB, 0.72639E+00_JPRB ,&
&  0.74354E+00_JPRB, 0.82282E+00_JPRB, 0.85773E+00_JPRB, 0.87379E+00_JPRB, 0.87650E+00_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.89506E+00_JPRB, 0.87213E+00_JPRB, 0.85087E+00_JPRB, 0.83374E+00_JPRB, 0.80926E+00_JPRB, 0.78562E+00_JPRB, 0.75658E+00_JPRB ,&
&  0.73004E+00_JPRB, 0.70292E+00_JPRB, 0.68332E+00_JPRB, 0.67098E+00_JPRB, 0.66109E+00_JPRB, 0.64257E+00_JPRB, 0.64308E+00_JPRB ,&
&  0.61323E+00_JPRB, 0.60674E+00_JPRB, 0.57836E+00_JPRB, 0.58229E+00_JPRB, 0.61552E+00_JPRB, 0.59798E+00_JPRB, 0.59764E+00_JPRB ,&
&  0.60008E+00_JPRB, 0.60677E+00_JPRB, 0.60112E+00_JPRB, 0.59421E+00_JPRB, 0.59117E+00_JPRB, 0.59294E+00_JPRB, 0.59813E+00_JPRB ,&
&  0.58450E+00_JPRB, 0.55625E+00_JPRB, 0.54246E+00_JPRB, 0.54779E+00_JPRB, 0.58309E+00_JPRB, 0.62852E+00_JPRB, 0.67999E+00_JPRB ,&
&  0.71079E+00_JPRB, 0.73712E+00_JPRB, 0.75578E+00_JPRB, 0.76712E+00_JPRB, 0.76901E+00_JPRB, 0.76990E+00_JPRB, 0.77359E+00_JPRB ,&
&  0.77787E+00_JPRB, 0.79327E+00_JPRB, 0.82486E+00_JPRB, 0.86449E+00_JPRB, 0.87445E+00_JPRB, 0.94645E+00_JPRB, 0.11422E+01_JPRB ,&
&  0.14399E+01_JPRB, 0.15417E+01_JPRB, 0.16510E+01_JPRB, 0.20142E+01_JPRB, 0.22204E+01_JPRB, 0.22786E+01_JPRB, 0.22845E+01_JPRB ,&
&  0.22947E+01_JPRB, 0.22695E+01_JPRB, 0.22727E+01_JPRB, 0.23536E+01_JPRB, 0.23275E+01_JPRB, 0.23190E+01_JPRB, 0.22502E+01_JPRB ,&
&  0.21609E+01_JPRB, 0.21705E+01_JPRB, 0.22017E+01_JPRB, 0.21504E+01_JPRB, 0.20599E+01_JPRB, 0.20240E+01_JPRB, 0.19633E+01_JPRB ,&
&  0.18518E+01_JPRB, 0.16225E+01_JPRB, 0.12021E+01_JPRB, 0.10576E+01_JPRB, 0.82707E+00_JPRB, 0.82590E+00_JPRB, 0.94665E+00_JPRB ,&
&  0.90936E+00_JPRB, 0.84081E+00_JPRB, 0.79860E+00_JPRB, 0.78442E+00_JPRB, 0.78745E+00_JPRB, 0.81042E+00_JPRB, 0.77231E+00_JPRB ,&
&  0.72340E+00_JPRB, 0.73292E+00_JPRB, 0.75591E+00_JPRB, 0.80394E+00_JPRB, 0.87842E+00_JPRB, 0.92959E+00_JPRB, 0.86706E+00_JPRB ,&
&  0.91928E+00_JPRB, 0.94225E+00_JPRB, 0.94851E+00_JPRB, 0.94453E+00_JPRB, 0.92405E+00_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.87835E+00_JPRB, 0.85499E+00_JPRB, 0.83099E+00_JPRB, 0.81904E+00_JPRB, 0.81271E+00_JPRB, 0.80724E+00_JPRB, 0.78508E+00_JPRB ,&
&  0.74921E+00_JPRB, 0.72068E+00_JPRB, 0.70306E+00_JPRB, 0.69958E+00_JPRB, 0.69473E+00_JPRB, 0.67571E+00_JPRB, 0.66881E+00_JPRB ,&
&  0.68927E+00_JPRB, 0.66776E+00_JPRB, 0.61749E+00_JPRB, 0.58331E+00_JPRB, 0.60473E+00_JPRB, 0.60065E+00_JPRB, 0.60621E+00_JPRB ,&
&  0.60073E+00_JPRB, 0.61915E+00_JPRB, 0.61447E+00_JPRB, 0.61199E+00_JPRB, 0.61327E+00_JPRB, 0.62340E+00_JPRB, 0.65066E+00_JPRB ,&
&  0.65119E+00_JPRB, 0.61037E+00_JPRB, 0.59021E+00_JPRB, 0.60733E+00_JPRB, 0.67078E+00_JPRB, 0.66776E+00_JPRB, 0.71306E+00_JPRB ,&
&  0.74819E+00_JPRB, 0.78240E+00_JPRB, 0.81113E+00_JPRB, 0.83662E+00_JPRB, 0.85442E+00_JPRB, 0.86711E+00_JPRB, 0.88158E+00_JPRB ,&
&  0.90183E+00_JPRB, 0.91980E+00_JPRB, 0.91940E+00_JPRB, 0.95094E+00_JPRB, 0.10433E+01_JPRB, 0.12772E+01_JPRB, 0.15542E+01_JPRB ,&
&  0.17449E+01_JPRB, 0.17714E+01_JPRB, 0.18076E+01_JPRB, 0.21549E+01_JPRB, 0.24511E+01_JPRB, 0.25741E+01_JPRB, 0.27028E+01_JPRB ,&
&  0.27325E+01_JPRB, 0.27275E+01_JPRB, 0.26914E+01_JPRB, 0.24856E+01_JPRB, 0.21947E+01_JPRB, 0.20147E+01_JPRB, 0.19512E+01_JPRB ,&
&  0.18996E+01_JPRB, 0.19269E+01_JPRB, 0.20169E+01_JPRB, 0.19817E+01_JPRB, 0.19660E+01_JPRB, 0.18825E+01_JPRB, 0.19744E+01_JPRB ,&
&  0.19174E+01_JPRB, 0.11777E+01_JPRB, 0.91183E+00_JPRB, 0.95512E+00_JPRB, 0.80102E+00_JPRB, 0.80649E+00_JPRB, 0.89089E+00_JPRB ,&
&  0.86250E+00_JPRB, 0.85433E+00_JPRB, 0.88523E+00_JPRB, 0.89115E+00_JPRB, 0.89291E+00_JPRB, 0.94585E+00_JPRB, 0.95115E+00_JPRB ,&
&  0.91030E+00_JPRB, 0.86013E+00_JPRB, 0.89594E+00_JPRB, 0.96892E+00_JPRB, 0.10518E+01_JPRB, 0.10733E+01_JPRB, 0.10579E+01_JPRB ,&
&  0.10475E+01_JPRB, 0.10335E+01_JPRB, 0.10101E+01_JPRB, 0.96514E+00_JPRB, 0.91229E+00_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.89951E+00_JPRB, 0.87735E+00_JPRB, 0.85068E+00_JPRB, 0.84686E+00_JPRB, 0.85059E+00_JPRB, 0.84291E+00_JPRB, 0.81390E+00_JPRB ,&
&  0.75897E+00_JPRB, 0.73145E+00_JPRB, 0.72027E+00_JPRB, 0.71863E+00_JPRB, 0.72043E+00_JPRB, 0.71519E+00_JPRB, 0.70773E+00_JPRB ,&
&  0.69148E+00_JPRB, 0.70614E+00_JPRB, 0.64953E+00_JPRB, 0.61665E+00_JPRB, 0.57767E+00_JPRB, 0.59135E+00_JPRB, 0.60787E+00_JPRB ,&
&  0.62234E+00_JPRB, 0.63333E+00_JPRB, 0.65874E+00_JPRB, 0.69465E+00_JPRB, 0.72927E+00_JPRB, 0.76141E+00_JPRB, 0.83156E+00_JPRB ,&
&  0.82028E+00_JPRB, 0.75779E+00_JPRB, 0.70257E+00_JPRB, 0.72752E+00_JPRB, 0.76779E+00_JPRB, 0.73473E+00_JPRB, 0.79900E+00_JPRB ,&
&  0.84407E+00_JPRB, 0.88351E+00_JPRB, 0.92056E+00_JPRB, 0.94165E+00_JPRB, 0.94713E+00_JPRB, 0.92157E+00_JPRB, 0.91176E+00_JPRB ,&
&  0.91532E+00_JPRB, 0.94006E+00_JPRB, 0.97941E+00_JPRB, 0.10207E+01_JPRB, 0.12686E+01_JPRB, 0.16049E+01_JPRB, 0.18571E+01_JPRB ,&
&  0.20430E+01_JPRB, 0.17570E+01_JPRB, 0.15762E+01_JPRB, 0.20766E+01_JPRB, 0.29431E+01_JPRB, 0.32086E+01_JPRB, 0.30031E+01_JPRB ,&
&  0.31626E+01_JPRB, 0.28710E+01_JPRB, 0.24940E+01_JPRB, 0.22430E+01_JPRB, 0.20624E+01_JPRB, 0.19369E+01_JPRB, 0.18839E+01_JPRB ,&
&  0.17771E+01_JPRB, 0.17807E+01_JPRB, 0.18823E+01_JPRB, 0.19425E+01_JPRB, 0.19346E+01_JPRB, 0.19201E+01_JPRB, 0.19744E+01_JPRB ,&
&  0.18439E+01_JPRB, 0.14891E+01_JPRB, 0.10515E+01_JPRB, 0.96500E+00_JPRB, 0.78999E+00_JPRB, 0.77709E+00_JPRB, 0.86047E+00_JPRB ,&
&  0.85131E+00_JPRB, 0.89878E+00_JPRB, 0.92405E+00_JPRB, 0.91959E+00_JPRB, 0.10064E+01_JPRB, 0.10596E+01_JPRB, 0.11569E+01_JPRB ,&
&  0.12191E+01_JPRB, 0.10645E+01_JPRB, 0.11375E+01_JPRB, 0.12197E+01_JPRB, 0.12579E+01_JPRB, 0.12196E+01_JPRB, 0.11610E+01_JPRB ,&
&  0.11215E+01_JPRB, 0.10868E+01_JPRB, 0.10386E+01_JPRB, 0.97237E+00_JPRB, 0.92593E+00_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.91317E+00_JPRB, 0.89716E+00_JPRB, 0.86677E+00_JPRB, 0.86284E+00_JPRB, 0.85741E+00_JPRB, 0.84124E+00_JPRB, 0.82087E+00_JPRB ,&
&  0.79920E+00_JPRB, 0.79076E+00_JPRB, 0.77735E+00_JPRB, 0.76656E+00_JPRB, 0.76150E+00_JPRB, 0.74948E+00_JPRB, 0.73367E+00_JPRB ,&
&  0.70964E+00_JPRB, 0.69799E+00_JPRB, 0.63125E+00_JPRB, 0.59078E+00_JPRB, 0.56820E+00_JPRB, 0.57723E+00_JPRB, 0.60464E+00_JPRB ,&
&  0.64706E+00_JPRB, 0.70387E+00_JPRB, 0.79949E+00_JPRB, 0.89053E+00_JPRB, 0.10191E+01_JPRB, 0.11261E+01_JPRB, 0.12021E+01_JPRB ,&
&  0.12095E+01_JPRB, 0.96813E+00_JPRB, 0.94763E+00_JPRB, 0.89670E+00_JPRB, 0.91182E+00_JPRB, 0.89577E+00_JPRB, 0.90560E+00_JPRB ,&
&  0.91375E+00_JPRB, 0.94205E+00_JPRB, 0.96388E+00_JPRB, 0.95211E+00_JPRB, 0.93342E+00_JPRB, 0.92357E+00_JPRB, 0.96017E+00_JPRB ,&
&  0.10073E+01_JPRB, 0.10175E+01_JPRB, 0.10600E+01_JPRB, 0.12591E+01_JPRB, 0.16841E+01_JPRB, 0.20531E+01_JPRB, 0.22934E+01_JPRB ,&
&  0.21747E+01_JPRB, 0.20651E+01_JPRB, 0.28286E+01_JPRB, 0.31524E+01_JPRB, 0.29579E+01_JPRB, 0.33421E+01_JPRB, 0.33509E+01_JPRB ,&
&  0.30120E+01_JPRB, 0.24299E+01_JPRB, 0.21591E+01_JPRB, 0.19750E+01_JPRB, 0.23163E+01_JPRB, 0.21869E+01_JPRB, 0.19441E+01_JPRB ,&
&  0.17350E+01_JPRB, 0.18419E+01_JPRB, 0.18943E+01_JPRB, 0.19866E+01_JPRB, 0.21137E+01_JPRB, 0.21707E+01_JPRB, 0.19760E+01_JPRB ,&
&  0.17241E+01_JPRB, 0.17971E+01_JPRB, 0.14298E+01_JPRB, 0.10789E+01_JPRB, 0.91649E+00_JPRB, 0.90910E+00_JPRB, 0.93210E+00_JPRB ,&
&  0.95570E+00_JPRB, 0.97117E+00_JPRB, 0.97568E+00_JPRB, 0.10429E+01_JPRB, 0.11883E+01_JPRB, 0.12823E+01_JPRB, 0.13458E+01_JPRB ,&
&  0.13337E+01_JPRB, 0.14487E+01_JPRB, 0.14655E+01_JPRB, 0.14499E+01_JPRB, 0.13688E+01_JPRB, 0.12471E+01_JPRB, 0.11688E+01_JPRB ,&
&  0.11312E+01_JPRB, 0.11048E+01_JPRB, 0.10403E+01_JPRB, 0.97101E+00_JPRB, 0.92245E+00_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.92765E+00_JPRB, 0.92237E+00_JPRB, 0.90993E+00_JPRB, 0.89216E+00_JPRB, 0.87494E+00_JPRB, 0.86230E+00_JPRB, 0.85959E+00_JPRB ,&
&  0.84924E+00_JPRB, 0.82664E+00_JPRB, 0.82562E+00_JPRB, 0.83352E+00_JPRB, 0.82854E+00_JPRB, 0.80203E+00_JPRB, 0.77214E+00_JPRB ,&
&  0.74986E+00_JPRB, 0.68789E+00_JPRB, 0.61188E+00_JPRB, 0.61009E+00_JPRB, 0.60737E+00_JPRB, 0.59404E+00_JPRB, 0.62787E+00_JPRB ,&
&  0.69358E+00_JPRB, 0.83175E+00_JPRB, 0.99696E+00_JPRB, 0.11887E+01_JPRB, 0.13178E+01_JPRB, 0.13722E+01_JPRB, 0.13925E+01_JPRB ,&
&  0.12869E+01_JPRB, 0.12286E+01_JPRB, 0.11847E+01_JPRB, 0.11172E+01_JPRB, 0.10881E+01_JPRB, 0.10691E+01_JPRB, 0.10326E+01_JPRB ,&
&  0.99533E+00_JPRB, 0.98630E+00_JPRB, 0.98807E+00_JPRB, 0.96215E+00_JPRB, 0.96609E+00_JPRB, 0.99808E+00_JPRB, 0.10492E+01_JPRB ,&
&  0.10590E+01_JPRB, 0.10761E+01_JPRB, 0.11991E+01_JPRB, 0.15358E+01_JPRB, 0.16885E+01_JPRB, 0.19862E+01_JPRB, 0.22822E+01_JPRB ,&
&  0.30855E+01_JPRB, 0.33644E+01_JPRB, 0.31309E+01_JPRB, 0.37926E+01_JPRB, 0.24253E+01_JPRB, 0.27367E+01_JPRB, 0.30843E+01_JPRB ,&
&  0.28336E+01_JPRB, 0.26807E+01_JPRB, 0.28333E+01_JPRB, 0.25408E+01_JPRB, 0.21916E+01_JPRB, 0.29556E+01_JPRB, 0.24240E+01_JPRB ,&
&  0.20526E+01_JPRB, 0.20556E+01_JPRB, 0.21814E+01_JPRB, 0.24452E+01_JPRB, 0.21330E+01_JPRB, 0.18923E+01_JPRB, 0.15270E+01_JPRB ,&
&  0.14986E+01_JPRB, 0.13434E+01_JPRB, 0.14295E+01_JPRB, 0.12452E+01_JPRB, 0.12189E+01_JPRB, 0.11200E+01_JPRB, 0.10343E+01_JPRB ,&
&  0.10454E+01_JPRB, 0.10424E+01_JPRB, 0.10755E+01_JPRB, 0.12546E+01_JPRB, 0.15470E+01_JPRB, 0.14631E+01_JPRB, 0.15726E+01_JPRB ,&
&  0.16702E+01_JPRB, 0.17047E+01_JPRB, 0.15820E+01_JPRB, 0.14900E+01_JPRB, 0.13178E+01_JPRB, 0.12140E+01_JPRB, 0.11663E+01_JPRB ,&
&  0.11299E+01_JPRB, 0.10632E+01_JPRB, 0.98821E+00_JPRB, 0.95804E+00_JPRB, 0.92089E+00_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.93469E+00_JPRB, 0.91922E+00_JPRB, 0.91560E+00_JPRB, 0.90256E+00_JPRB, 0.90236E+00_JPRB, 0.89579E+00_JPRB, 0.89174E+00_JPRB ,&
&  0.87619E+00_JPRB, 0.85900E+00_JPRB, 0.86270E+00_JPRB, 0.86379E+00_JPRB, 0.85262E+00_JPRB, 0.83462E+00_JPRB, 0.79617E+00_JPRB ,&
&  0.77441E+00_JPRB, 0.72766E+00_JPRB, 0.63579E+00_JPRB, 0.61156E+00_JPRB, 0.62675E+00_JPRB, 0.62588E+00_JPRB, 0.65582E+00_JPRB ,&
&  0.82900E+00_JPRB, 0.10731E+01_JPRB, 0.12427E+01_JPRB, 0.13834E+01_JPRB, 0.14630E+01_JPRB, 0.14227E+01_JPRB, 0.13765E+01_JPRB ,&
&  0.13716E+01_JPRB, 0.12309E+01_JPRB, 0.11402E+01_JPRB, 0.11274E+01_JPRB, 0.10935E+01_JPRB, 0.10524E+01_JPRB, 0.10169E+01_JPRB ,&
&  0.98012E+00_JPRB, 0.94583E+00_JPRB, 0.92367E+00_JPRB, 0.90883E+00_JPRB, 0.93587E+00_JPRB, 0.98034E+00_JPRB, 0.10362E+01_JPRB ,&
&  0.10768E+01_JPRB, 0.11620E+01_JPRB, 0.14149E+01_JPRB, 0.17532E+01_JPRB, 0.20111E+01_JPRB, 0.23028E+01_JPRB, 0.29770E+01_JPRB ,&
&  0.33435E+01_JPRB, 0.32890E+01_JPRB, 0.34511E+01_JPRB, 0.36905E+01_JPRB, 0.33218E+01_JPRB, 0.28135E+01_JPRB, 0.29875E+01_JPRB ,&
&  0.23337E+01_JPRB, 0.20981E+01_JPRB, 0.20649E+01_JPRB, 0.21348E+01_JPRB, 0.27738E+01_JPRB, 0.41235E+01_JPRB, 0.36397E+01_JPRB ,&
&  0.32345E+01_JPRB, 0.28008E+01_JPRB, 0.28484E+01_JPRB, 0.26289E+01_JPRB, 0.22797E+01_JPRB, 0.16755E+01_JPRB, 0.20153E+01_JPRB ,&
&  0.20596E+01_JPRB, 0.20918E+01_JPRB, 0.18795E+01_JPRB, 0.15276E+01_JPRB, 0.12627E+01_JPRB, 0.13314E+01_JPRB, 0.12775E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.12224E+01_JPRB, 0.15273E+01_JPRB, 0.17184E+01_JPRB, 0.18397E+01_JPRB, 0.18829E+01_JPRB, 0.20619E+01_JPRB ,&
&  0.19350E+01_JPRB, 0.18758E+01_JPRB, 0.16869E+01_JPRB, 0.14357E+01_JPRB, 0.12787E+01_JPRB, 0.12007E+01_JPRB, 0.11423E+01_JPRB ,&
&  0.10817E+01_JPRB, 0.99606E+00_JPRB, 0.95636E+00_JPRB, 0.92770E+00_JPRB, 0.91352E+00_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.93169E+00_JPRB, 0.92773E+00_JPRB, 0.91708E+00_JPRB, 0.91029E+00_JPRB, 0.89660E+00_JPRB, 0.87810E+00_JPRB, 0.88099E+00_JPRB ,&
&  0.88226E+00_JPRB, 0.86816E+00_JPRB, 0.84764E+00_JPRB, 0.83369E+00_JPRB, 0.81950E+00_JPRB, 0.82003E+00_JPRB, 0.81152E+00_JPRB ,&
&  0.80504E+00_JPRB, 0.80096E+00_JPRB, 0.68979E+00_JPRB, 0.66273E+00_JPRB, 0.67099E+00_JPRB, 0.69330E+00_JPRB, 0.73296E+00_JPRB ,&
&  0.10067E+01_JPRB, 0.12808E+01_JPRB, 0.13608E+01_JPRB, 0.14275E+01_JPRB, 0.14631E+01_JPRB, 0.14057E+01_JPRB, 0.13883E+01_JPRB ,&
&  0.13884E+01_JPRB, 0.13008E+01_JPRB, 0.11954E+01_JPRB, 0.11284E+01_JPRB, 0.10614E+01_JPRB, 0.10194E+01_JPRB, 0.98739E+00_JPRB ,&
&  0.95142E+00_JPRB, 0.92239E+00_JPRB, 0.90998E+00_JPRB, 0.92270E+00_JPRB, 0.92684E+00_JPRB, 0.10099E+01_JPRB, 0.11045E+01_JPRB ,&
&  0.11958E+01_JPRB, 0.13916E+01_JPRB, 0.16409E+01_JPRB, 0.19916E+01_JPRB, 0.23582E+01_JPRB, 0.27354E+01_JPRB, 0.38223E+01_JPRB ,&
&  0.37230E+01_JPRB, 0.37457E+01_JPRB, 0.37525E+01_JPRB, 0.44915E+01_JPRB, 0.32675E+01_JPRB, 0.27269E+01_JPRB, 0.28257E+01_JPRB ,&
&  0.21935E+01_JPRB, 0.20388E+01_JPRB, 0.24262E+01_JPRB, 0.31891E+01_JPRB, 0.33392E+01_JPRB, 0.37112E+01_JPRB, 0.48395E+01_JPRB ,&
&  0.42819E+01_JPRB, 0.37085E+01_JPRB, 0.34670E+01_JPRB, 0.33689E+01_JPRB, 0.17381E+01_JPRB, 0.13562E+01_JPRB, 0.20308E+01_JPRB ,&
&  0.21022E+01_JPRB, 0.14252E+01_JPRB, 0.10943E+01_JPRB, 0.13059E+01_JPRB, 0.10211E+01_JPRB, 0.11364E+01_JPRB, 0.14400E+01_JPRB ,&
&  0.14604E+01_JPRB, 0.15771E+01_JPRB, 0.22168E+01_JPRB, 0.21643E+01_JPRB, 0.21233E+01_JPRB, 0.27532E+01_JPRB, 0.25124E+01_JPRB ,&
&  0.22243E+01_JPRB, 0.17755E+01_JPRB, 0.15943E+01_JPRB, 0.13575E+01_JPRB, 0.12287E+01_JPRB, 0.11337E+01_JPRB, 0.10606E+01_JPRB ,&
&  0.98156E+00_JPRB, 0.94096E+00_JPRB, 0.94339E+00_JPRB, 0.94295E+00_JPRB, 0.91904E+00_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.93163E+00_JPRB, 0.91333E+00_JPRB, 0.89049E+00_JPRB, 0.89257E+00_JPRB, 0.87379E+00_JPRB, 0.85003E+00_JPRB, 0.88576E+00_JPRB ,&
&  0.89435E+00_JPRB, 0.85302E+00_JPRB, 0.81241E+00_JPRB, 0.79923E+00_JPRB, 0.79780E+00_JPRB, 0.83552E+00_JPRB, 0.85254E+00_JPRB ,&
&  0.86430E+00_JPRB, 0.85597E+00_JPRB, 0.82365E+00_JPRB, 0.80957E+00_JPRB, 0.82054E+00_JPRB, 0.84332E+00_JPRB, 0.95314E+00_JPRB ,&
&  0.12746E+01_JPRB, 0.14081E+01_JPRB, 0.13785E+01_JPRB, 0.13829E+01_JPRB, 0.14010E+01_JPRB, 0.13672E+01_JPRB, 0.14308E+01_JPRB ,&
&  0.13875E+01_JPRB, 0.12766E+01_JPRB, 0.11860E+01_JPRB, 0.11091E+01_JPRB, 0.10680E+01_JPRB, 0.10400E+01_JPRB, 0.10131E+01_JPRB ,&
&  0.96691E+00_JPRB, 0.93315E+00_JPRB, 0.94329E+00_JPRB, 0.93145E+00_JPRB, 0.97749E+00_JPRB, 0.10884E+01_JPRB, 0.12147E+01_JPRB ,&
&  0.13759E+01_JPRB, 0.16117E+01_JPRB, 0.18608E+01_JPRB, 0.22825E+01_JPRB, 0.29558E+01_JPRB, 0.33082E+01_JPRB, 0.30891E+01_JPRB ,&
&  0.28516E+01_JPRB, 0.28280E+01_JPRB, 0.37219E+01_JPRB, 0.39323E+01_JPRB, 0.33885E+01_JPRB, 0.30565E+01_JPRB, 0.28367E+01_JPRB ,&
&  0.25265E+01_JPRB, 0.24554E+01_JPRB, 0.28260E+01_JPRB, 0.37004E+01_JPRB, 0.41902E+01_JPRB, 0.39591E+01_JPRB, 0.50482E+01_JPRB ,&
&  0.50796E+01_JPRB, 0.44115E+01_JPRB, 0.30350E+01_JPRB, 0.22376E+01_JPRB, 0.19691E+01_JPRB, 0.16320E+01_JPRB, 0.11762E+01_JPRB ,&
&  0.10667E+01_JPRB, 0.10562E+01_JPRB, 0.98833E+00_JPRB, 0.99462E+00_JPRB, 0.95789E+00_JPRB, 0.11522E+01_JPRB, 0.14296E+01_JPRB ,&
&  0.18371E+01_JPRB, 0.22558E+01_JPRB, 0.25073E+01_JPRB, 0.24590E+01_JPRB, 0.23524E+01_JPRB, 0.31412E+01_JPRB, 0.26896E+01_JPRB ,&
&  0.18639E+01_JPRB, 0.15688E+01_JPRB, 0.14083E+01_JPRB, 0.12527E+01_JPRB, 0.11426E+01_JPRB, 0.10426E+01_JPRB, 0.96801E+00_JPRB ,&
&  0.93018E+00_JPRB, 0.92959E+00_JPRB, 0.93448E+00_JPRB, 0.93214E+00_JPRB, 0.91012E+00_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.92931E+00_JPRB, 0.90745E+00_JPRB, 0.88118E+00_JPRB, 0.86123E+00_JPRB, 0.82560E+00_JPRB, 0.83409E+00_JPRB, 0.89395E+00_JPRB ,&
&  0.90352E+00_JPRB, 0.83291E+00_JPRB, 0.78876E+00_JPRB, 0.78854E+00_JPRB, 0.83155E+00_JPRB, 0.87838E+00_JPRB, 0.89041E+00_JPRB ,&
&  0.91697E+00_JPRB, 0.92215E+00_JPRB, 0.97454E+00_JPRB, 0.10176E+01_JPRB, 0.11097E+01_JPRB, 0.11229E+01_JPRB, 0.13341E+01_JPRB ,&
&  0.15363E+01_JPRB, 0.14482E+01_JPRB, 0.13742E+01_JPRB, 0.13815E+01_JPRB, 0.14139E+01_JPRB, 0.14274E+01_JPRB, 0.14404E+01_JPRB ,&
&  0.13308E+01_JPRB, 0.12308E+01_JPRB, 0.11365E+01_JPRB, 0.10879E+01_JPRB, 0.10722E+01_JPRB, 0.10289E+01_JPRB, 0.10002E+01_JPRB ,&
&  0.96950E+00_JPRB, 0.96523E+00_JPRB, 0.94984E+00_JPRB, 0.98063E+00_JPRB, 0.10959E+01_JPRB, 0.12465E+01_JPRB, 0.14137E+01_JPRB ,&
&  0.16035E+01_JPRB, 0.18798E+01_JPRB, 0.21894E+01_JPRB, 0.26879E+01_JPRB, 0.26895E+01_JPRB, 0.23328E+01_JPRB, 0.26732E+01_JPRB ,&
&  0.29496E+01_JPRB, 0.32203E+01_JPRB, 0.31553E+01_JPRB, 0.38807E+01_JPRB, 0.36011E+01_JPRB, 0.30040E+01_JPRB, 0.27597E+01_JPRB ,&
&  0.25035E+01_JPRB, 0.30361E+01_JPRB, 0.35805E+01_JPRB, 0.41150E+01_JPRB, 0.54656E+01_JPRB, 0.52172E+01_JPRB, 0.56426E+01_JPRB ,&
&  0.58446E+01_JPRB, 0.48084E+01_JPRB, 0.34504E+01_JPRB, 0.25722E+01_JPRB, 0.29227E+01_JPRB, 0.30353E+01_JPRB, 0.14108E+01_JPRB ,&
&  0.11631E+01_JPRB, 0.10848E+01_JPRB, 0.10522E+01_JPRB, 0.98281E+00_JPRB, 0.96689E+00_JPRB, 0.11866E+01_JPRB, 0.19059E+01_JPRB ,&
&  0.18409E+01_JPRB, 0.28810E+01_JPRB, 0.27521E+01_JPRB, 0.26691E+01_JPRB, 0.24807E+01_JPRB, 0.30955E+01_JPRB, 0.23983E+01_JPRB ,&
&  0.19031E+01_JPRB, 0.16240E+01_JPRB, 0.13724E+01_JPRB, 0.12346E+01_JPRB, 0.11008E+01_JPRB, 0.99175E+00_JPRB, 0.95276E+00_JPRB ,&
&  0.92371E+00_JPRB, 0.89412E+00_JPRB, 0.88781E+00_JPRB, 0.89471E+00_JPRB, 0.90694E+00_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.91521E+00_JPRB, 0.89171E+00_JPRB, 0.87787E+00_JPRB, 0.86844E+00_JPRB, 0.84312E+00_JPRB, 0.88254E+00_JPRB, 0.91026E+00_JPRB ,&
&  0.89591E+00_JPRB, 0.84617E+00_JPRB, 0.82074E+00_JPRB, 0.83787E+00_JPRB, 0.88416E+00_JPRB, 0.93505E+00_JPRB, 0.97469E+00_JPRB ,&
&  0.10203E+01_JPRB, 0.10721E+01_JPRB, 0.11602E+01_JPRB, 0.13589E+01_JPRB, 0.14879E+01_JPRB, 0.15416E+01_JPRB, 0.17314E+01_JPRB ,&
&  0.16252E+01_JPRB, 0.13885E+01_JPRB, 0.13782E+01_JPRB, 0.14340E+01_JPRB, 0.14045E+01_JPRB, 0.13745E+01_JPRB, 0.13378E+01_JPRB ,&
&  0.12519E+01_JPRB, 0.11929E+01_JPRB, 0.11308E+01_JPRB, 0.11361E+01_JPRB, 0.11415E+01_JPRB, 0.10947E+01_JPRB, 0.10728E+01_JPRB ,&
&  0.10546E+01_JPRB, 0.10349E+01_JPRB, 0.10497E+01_JPRB, 0.10971E+01_JPRB, 0.12578E+01_JPRB, 0.14132E+01_JPRB, 0.16115E+01_JPRB ,&
&  0.18649E+01_JPRB, 0.21218E+01_JPRB, 0.24139E+01_JPRB, 0.27689E+01_JPRB, 0.23486E+01_JPRB, 0.28162E+01_JPRB, 0.29776E+01_JPRB ,&
&  0.31295E+01_JPRB, 0.30297E+01_JPRB, 0.27394E+01_JPRB, 0.32206E+01_JPRB, 0.33305E+01_JPRB, 0.28423E+01_JPRB, 0.25776E+01_JPRB ,&
&  0.29709E+01_JPRB, 0.34467E+01_JPRB, 0.43645E+01_JPRB, 0.51024E+01_JPRB, 0.58639E+01_JPRB, 0.81427E+01_JPRB, 0.57442E+01_JPRB ,&
&  0.47918E+01_JPRB, 0.46834E+01_JPRB, 0.38473E+01_JPRB, 0.31727E+01_JPRB, 0.34429E+01_JPRB, 0.45246E+01_JPRB, 0.32760E+01_JPRB ,&
&  0.13183E+01_JPRB, 0.10248E+01_JPRB, 0.11301E+01_JPRB, 0.11395E+01_JPRB, 0.10138E+01_JPRB, 0.12030E+01_JPRB, 0.29608E+01_JPRB ,&
&  0.22800E+01_JPRB, 0.31694E+01_JPRB, 0.27743E+01_JPRB, 0.26044E+01_JPRB, 0.26077E+01_JPRB, 0.28916E+01_JPRB, 0.26024E+01_JPRB ,&
&  0.18502E+01_JPRB, 0.15513E+01_JPRB, 0.13257E+01_JPRB, 0.11768E+01_JPRB, 0.10455E+01_JPRB, 0.96641E+00_JPRB, 0.92400E+00_JPRB ,&
&  0.89450E+00_JPRB, 0.88240E+00_JPRB, 0.87971E+00_JPRB, 0.88514E+00_JPRB, 0.90124E+00_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.91965E+00_JPRB, 0.87836E+00_JPRB, 0.86335E+00_JPRB, 0.84892E+00_JPRB, 0.89361E+00_JPRB, 0.94333E+00_JPRB, 0.91703E+00_JPRB ,&
&  0.93264E+00_JPRB, 0.93248E+00_JPRB, 0.94058E+00_JPRB, 0.93458E+00_JPRB, 0.96863E+00_JPRB, 0.10582E+01_JPRB, 0.11359E+01_JPRB ,&
&  0.12283E+01_JPRB, 0.13292E+01_JPRB, 0.14632E+01_JPRB, 0.16730E+01_JPRB, 0.18661E+01_JPRB, 0.18290E+01_JPRB, 0.17723E+01_JPRB ,&
&  0.15863E+01_JPRB, 0.13991E+01_JPRB, 0.13394E+01_JPRB, 0.14497E+01_JPRB, 0.14389E+01_JPRB, 0.13944E+01_JPRB, 0.13872E+01_JPRB ,&
&  0.13339E+01_JPRB, 0.13010E+01_JPRB, 0.12580E+01_JPRB, 0.12759E+01_JPRB, 0.12445E+01_JPRB, 0.12543E+01_JPRB, 0.13324E+01_JPRB ,&
&  0.12979E+01_JPRB, 0.12831E+01_JPRB, 0.13022E+01_JPRB, 0.13585E+01_JPRB, 0.15238E+01_JPRB, 0.16814E+01_JPRB, 0.18878E+01_JPRB ,&
&  0.21210E+01_JPRB, 0.23169E+01_JPRB, 0.25708E+01_JPRB, 0.26331E+01_JPRB, 0.26674E+01_JPRB, 0.27935E+01_JPRB, 0.30584E+01_JPRB ,&
&  0.31293E+01_JPRB, 0.30109E+01_JPRB, 0.28505E+01_JPRB, 0.28865E+01_JPRB, 0.28676E+01_JPRB, 0.26970E+01_JPRB, 0.27293E+01_JPRB ,&
&  0.35112E+01_JPRB, 0.45886E+01_JPRB, 0.53726E+01_JPRB, 0.60529E+01_JPRB, 0.64645E+01_JPRB, 0.81678E+01_JPRB, 0.74395E+01_JPRB ,&
&  0.56261E+01_JPRB, 0.42936E+01_JPRB, 0.38509E+01_JPRB, 0.40566E+01_JPRB, 0.42420E+01_JPRB, 0.44054E+01_JPRB, 0.48869E+01_JPRB ,&
&  0.47619E+01_JPRB, 0.26127E+01_JPRB, 0.19629E+01_JPRB, 0.26556E+01_JPRB, 0.17914E+01_JPRB, 0.12901E+01_JPRB, 0.17574E+01_JPRB ,&
&  0.24489E+01_JPRB, 0.29367E+01_JPRB, 0.28343E+01_JPRB, 0.25663E+01_JPRB, 0.26936E+01_JPRB, 0.26168E+01_JPRB, 0.22034E+01_JPRB ,&
&  0.16381E+01_JPRB, 0.13501E+01_JPRB, 0.11777E+01_JPRB, 0.10619E+01_JPRB, 0.96295E+00_JPRB, 0.92501E+00_JPRB, 0.91902E+00_JPRB ,&
&  0.89740E+00_JPRB, 0.90469E+00_JPRB, 0.90537E+00_JPRB, 0.90083E+00_JPRB, 0.91735E+00_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.89748E+00_JPRB, 0.86441E+00_JPRB, 0.84344E+00_JPRB, 0.85048E+00_JPRB, 0.92777E+00_JPRB, 0.98307E+00_JPRB, 0.98679E+00_JPRB ,&
&  0.99388E+00_JPRB, 0.10055E+01_JPRB, 0.10457E+01_JPRB, 0.10393E+01_JPRB, 0.11228E+01_JPRB, 0.12919E+01_JPRB, 0.14199E+01_JPRB ,&
&  0.15224E+01_JPRB, 0.16311E+01_JPRB, 0.17666E+01_JPRB, 0.19852E+01_JPRB, 0.22846E+01_JPRB, 0.26622E+01_JPRB, 0.16351E+01_JPRB ,&
&  0.14226E+01_JPRB, 0.14606E+01_JPRB, 0.14103E+01_JPRB, 0.15707E+01_JPRB, 0.15412E+01_JPRB, 0.15317E+01_JPRB, 0.14772E+01_JPRB ,&
&  0.14949E+01_JPRB, 0.14921E+01_JPRB, 0.15070E+01_JPRB, 0.15609E+01_JPRB, 0.15807E+01_JPRB, 0.17366E+01_JPRB, 0.18371E+01_JPRB ,&
&  0.17970E+01_JPRB, 0.17826E+01_JPRB, 0.18529E+01_JPRB, 0.19271E+01_JPRB, 0.20277E+01_JPRB, 0.21255E+01_JPRB, 0.22539E+01_JPRB ,&
&  0.23948E+01_JPRB, 0.25637E+01_JPRB, 0.27951E+01_JPRB, 0.26615E+01_JPRB, 0.26433E+01_JPRB, 0.27133E+01_JPRB, 0.29101E+01_JPRB ,&
&  0.27817E+01_JPRB, 0.24923E+01_JPRB, 0.25687E+01_JPRB, 0.27173E+01_JPRB, 0.26970E+01_JPRB, 0.26817E+01_JPRB, 0.32641E+01_JPRB ,&
&  0.45883E+01_JPRB, 0.59096E+01_JPRB, 0.73538E+01_JPRB, 0.66333E+01_JPRB, 0.66776E+01_JPRB, 0.72190E+01_JPRB, 0.68347E+01_JPRB ,&
&  0.61518E+01_JPRB, 0.50465E+01_JPRB, 0.45784E+01_JPRB, 0.44863E+01_JPRB, 0.41937E+01_JPRB, 0.37649E+01_JPRB, 0.37333E+01_JPRB ,&
&  0.44201E+01_JPRB, 0.54518E+01_JPRB, 0.45543E+01_JPRB, 0.28919E+01_JPRB, 0.20495E+01_JPRB, 0.14756E+01_JPRB, 0.16766E+01_JPRB ,&
&  0.25975E+01_JPRB, 0.26131E+01_JPRB, 0.25330E+01_JPRB, 0.28696E+01_JPRB, 0.24111E+01_JPRB, 0.21086E+01_JPRB, 0.16603E+01_JPRB ,&
&  0.12857E+01_JPRB, 0.10496E+01_JPRB, 0.96288E+00_JPRB, 0.89037E+00_JPRB, 0.88099E+00_JPRB, 0.89011E+00_JPRB, 0.91481E+00_JPRB ,&
&  0.93862E+00_JPRB, 0.94091E+00_JPRB, 0.92682E+00_JPRB, 0.91348E+00_JPRB, 0.91725E+00_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.90351E+00_JPRB, 0.87479E+00_JPRB, 0.85032E+00_JPRB, 0.87875E+00_JPRB, 0.95848E+00_JPRB, 0.10040E+01_JPRB, 0.10228E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.10448E+01_JPRB, 0.11148E+01_JPRB, 0.11689E+01_JPRB, 0.13260E+01_JPRB, 0.14955E+01_JPRB, 0.16386E+01_JPRB ,&
&  0.17538E+01_JPRB, 0.18488E+01_JPRB, 0.19858E+01_JPRB, 0.21851E+01_JPRB, 0.23913E+01_JPRB, 0.25618E+01_JPRB, 0.19648E+01_JPRB ,&
&  0.12997E+01_JPRB, 0.18021E+01_JPRB, 0.15760E+01_JPRB, 0.17554E+01_JPRB, 0.18024E+01_JPRB, 0.19357E+01_JPRB, 0.18445E+01_JPRB ,&
&  0.18465E+01_JPRB, 0.18294E+01_JPRB, 0.19232E+01_JPRB, 0.19934E+01_JPRB, 0.20358E+01_JPRB, 0.21839E+01_JPRB, 0.23052E+01_JPRB ,&
&  0.23752E+01_JPRB, 0.24552E+01_JPRB, 0.24818E+01_JPRB, 0.24726E+01_JPRB, 0.24855E+01_JPRB, 0.25163E+01_JPRB, 0.25793E+01_JPRB ,&
&  0.27221E+01_JPRB, 0.29067E+01_JPRB, 0.29516E+01_JPRB, 0.29085E+01_JPRB, 0.27352E+01_JPRB, 0.27880E+01_JPRB, 0.28763E+01_JPRB ,&
&  0.26567E+01_JPRB, 0.23749E+01_JPRB, 0.24886E+01_JPRB, 0.25932E+01_JPRB, 0.27829E+01_JPRB, 0.35658E+01_JPRB, 0.44313E+01_JPRB ,&
&  0.57963E+01_JPRB, 0.64255E+01_JPRB, 0.81742E+01_JPRB, 0.60245E+01_JPRB, 0.57235E+01_JPRB, 0.60394E+01_JPRB, 0.52714E+01_JPRB ,&
&  0.45495E+01_JPRB, 0.44993E+01_JPRB, 0.42573E+01_JPRB, 0.42937E+01_JPRB, 0.40569E+01_JPRB, 0.35461E+01_JPRB, 0.31117E+01_JPRB ,&
&  0.31708E+01_JPRB, 0.33306E+01_JPRB, 0.28644E+01_JPRB, 0.19478E+01_JPRB, 0.16651E+01_JPRB, 0.15999E+01_JPRB, 0.21626E+01_JPRB ,&
&  0.25963E+01_JPRB, 0.26167E+01_JPRB, 0.26229E+01_JPRB, 0.25059E+01_JPRB, 0.17849E+01_JPRB, 0.14274E+01_JPRB, 0.10752E+01_JPRB ,&
&  0.84774E+00_JPRB, 0.76317E+00_JPRB, 0.83426E+00_JPRB, 0.82362E+00_JPRB, 0.84717E+00_JPRB, 0.88052E+00_JPRB, 0.89675E+00_JPRB ,&
&  0.91526E+00_JPRB, 0.91917E+00_JPRB, 0.92339E+00_JPRB, 0.91820E+00_JPRB, 0.91881E+00_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.84107E+00_JPRB, 0.83103E+00_JPRB, 0.83245E+00_JPRB, 0.91749E+00_JPRB, 0.10525E+01_JPRB, 0.11637E+01_JPRB, 0.11841E+01_JPRB ,&
&  0.11976E+01_JPRB, 0.10516E+01_JPRB, 0.11437E+01_JPRB, 0.12246E+01_JPRB, 0.13764E+01_JPRB, 0.15475E+01_JPRB, 0.17385E+01_JPRB ,&
&  0.19220E+01_JPRB, 0.20977E+01_JPRB, 0.22532E+01_JPRB, 0.24275E+01_JPRB, 0.26527E+01_JPRB, 0.28768E+01_JPRB, 0.25725E+01_JPRB ,&
&  0.18841E+01_JPRB, 0.15421E+01_JPRB, 0.17610E+01_JPRB, 0.18224E+01_JPRB, 0.18674E+01_JPRB, 0.20201E+01_JPRB, 0.21002E+01_JPRB ,&
&  0.21653E+01_JPRB, 0.20786E+01_JPRB, 0.21421E+01_JPRB, 0.22366E+01_JPRB, 0.22285E+01_JPRB, 0.23807E+01_JPRB, 0.25600E+01_JPRB ,&
&  0.26765E+01_JPRB, 0.27679E+01_JPRB, 0.27903E+01_JPRB, 0.27768E+01_JPRB, 0.27890E+01_JPRB, 0.28524E+01_JPRB, 0.29570E+01_JPRB ,&
&  0.31422E+01_JPRB, 0.33643E+01_JPRB, 0.34803E+01_JPRB, 0.32949E+01_JPRB, 0.32071E+01_JPRB, 0.32674E+01_JPRB, 0.32175E+01_JPRB ,&
&  0.30382E+01_JPRB, 0.30161E+01_JPRB, 0.32305E+01_JPRB, 0.36399E+01_JPRB, 0.38937E+01_JPRB, 0.48791E+01_JPRB, 0.54719E+01_JPRB ,&
&  0.61532E+01_JPRB, 0.65180E+01_JPRB, 0.72307E+01_JPRB, 0.50782E+01_JPRB, 0.50167E+01_JPRB, 0.49247E+01_JPRB, 0.44346E+01_JPRB ,&
&  0.41085E+01_JPRB, 0.41265E+01_JPRB, 0.40117E+01_JPRB, 0.38657E+01_JPRB, 0.37094E+01_JPRB, 0.30679E+01_JPRB, 0.28783E+01_JPRB ,&
&  0.26216E+01_JPRB, 0.27888E+01_JPRB, 0.22082E+01_JPRB, 0.19146E+01_JPRB, 0.16728E+01_JPRB, 0.17776E+01_JPRB, 0.20074E+01_JPRB ,&
&  0.23382E+01_JPRB, 0.23328E+01_JPRB, 0.21020E+01_JPRB, 0.16537E+01_JPRB, 0.10597E+01_JPRB, 0.82418E+00_JPRB, 0.71479E+00_JPRB ,&
&  0.71064E+00_JPRB, 0.73703E+00_JPRB, 0.77181E+00_JPRB, 0.79167E+00_JPRB, 0.81132E+00_JPRB, 0.83543E+00_JPRB, 0.83819E+00_JPRB ,&
&  0.83533E+00_JPRB, 0.82384E+00_JPRB, 0.81613E+00_JPRB, 0.84881E+00_JPRB, 0.86170E+00_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.73095E+00_JPRB, 0.71324E+00_JPRB, 0.72018E+00_JPRB, 0.81175E+00_JPRB, 0.91248E+00_JPRB, 0.97473E+00_JPRB, 0.95696E+00_JPRB ,&
&  0.92632E+00_JPRB, 0.90372E+00_JPRB, 0.98100E+00_JPRB, 0.10752E+01_JPRB, 0.11995E+01_JPRB, 0.13679E+01_JPRB, 0.15448E+01_JPRB ,&
&  0.17381E+01_JPRB, 0.18884E+01_JPRB, 0.20006E+01_JPRB, 0.21401E+01_JPRB, 0.23213E+01_JPRB, 0.25097E+01_JPRB, 0.25948E+01_JPRB ,&
&  0.20745E+01_JPRB, 0.13826E+01_JPRB, 0.16039E+01_JPRB, 0.17957E+01_JPRB, 0.19663E+01_JPRB, 0.21056E+01_JPRB, 0.21529E+01_JPRB ,&
&  0.20010E+01_JPRB, 0.20018E+01_JPRB, 0.21095E+01_JPRB, 0.22075E+01_JPRB, 0.21955E+01_JPRB, 0.23668E+01_JPRB, 0.25467E+01_JPRB ,&
&  0.27332E+01_JPRB, 0.28703E+01_JPRB, 0.29495E+01_JPRB, 0.30251E+01_JPRB, 0.31062E+01_JPRB, 0.32322E+01_JPRB, 0.34370E+01_JPRB ,&
&  0.37019E+01_JPRB, 0.39427E+01_JPRB, 0.41288E+01_JPRB, 0.41235E+01_JPRB, 0.40929E+01_JPRB, 0.41993E+01_JPRB, 0.41712E+01_JPRB ,&
&  0.41696E+01_JPRB, 0.41672E+01_JPRB, 0.46363E+01_JPRB, 0.53008E+01_JPRB, 0.56955E+01_JPRB, 0.52178E+01_JPRB, 0.57236E+01_JPRB ,&
&  0.58583E+01_JPRB, 0.57162E+01_JPRB, 0.52953E+01_JPRB, 0.54233E+01_JPRB, 0.37435E+01_JPRB, 0.38350E+01_JPRB, 0.36777E+01_JPRB ,&
&  0.39330E+01_JPRB, 0.38119E+01_JPRB, 0.36611E+01_JPRB, 0.34067E+01_JPRB, 0.33966E+01_JPRB, 0.27782E+01_JPRB, 0.24813E+01_JPRB ,&
&  0.23783E+01_JPRB, 0.23698E+01_JPRB, 0.21157E+01_JPRB, 0.20252E+01_JPRB, 0.18489E+01_JPRB, 0.19503E+01_JPRB, 0.18697E+01_JPRB ,&
&  0.19921E+01_JPRB, 0.17030E+01_JPRB, 0.14336E+01_JPRB, 0.11172E+01_JPRB, 0.71696E+00_JPRB, 0.62117E+00_JPRB, 0.63879E+00_JPRB ,&
&  0.66702E+00_JPRB, 0.69656E+00_JPRB, 0.70178E+00_JPRB, 0.72934E+00_JPRB, 0.74761E+00_JPRB, 0.73822E+00_JPRB, 0.71815E+00_JPRB ,&
&  0.71631E+00_JPRB, 0.73766E+00_JPRB, 0.73738E+00_JPRB, 0.74069E+00_JPRB, 0.74116E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.61578E+00_JPRB, 0.61970E+00_JPRB, 0.64043E+00_JPRB, 0.68618E+00_JPRB, 0.76010E+00_JPRB, 0.80345E+00_JPRB, 0.81420E+00_JPRB ,&
&  0.79963E+00_JPRB, 0.81501E+00_JPRB, 0.85846E+00_JPRB, 0.87844E+00_JPRB, 0.94822E+00_JPRB, 0.10720E+01_JPRB, 0.11938E+01_JPRB ,&
&  0.13551E+01_JPRB, 0.15012E+01_JPRB, 0.16291E+01_JPRB, 0.18210E+01_JPRB, 0.20120E+01_JPRB, 0.22475E+01_JPRB, 0.23912E+01_JPRB ,&
&  0.23628E+01_JPRB, 0.20174E+01_JPRB, 0.19185E+01_JPRB, 0.16047E+01_JPRB, 0.14865E+01_JPRB, 0.17997E+01_JPRB, 0.17599E+01_JPRB ,&
&  0.17796E+01_JPRB, 0.18452E+01_JPRB, 0.18758E+01_JPRB, 0.19791E+01_JPRB, 0.21291E+01_JPRB, 0.22943E+01_JPRB, 0.24960E+01_JPRB ,&
&  0.27216E+01_JPRB, 0.29311E+01_JPRB, 0.31200E+01_JPRB, 0.32981E+01_JPRB, 0.34356E+01_JPRB, 0.36150E+01_JPRB, 0.38700E+01_JPRB ,&
&  0.41613E+01_JPRB, 0.44023E+01_JPRB, 0.46782E+01_JPRB, 0.50313E+01_JPRB, 0.50160E+01_JPRB, 0.51941E+01_JPRB, 0.52783E+01_JPRB ,&
&  0.53312E+01_JPRB, 0.52752E+01_JPRB, 0.53300E+01_JPRB, 0.55547E+01_JPRB, 0.55132E+01_JPRB, 0.48842E+01_JPRB, 0.48409E+01_JPRB ,&
&  0.49752E+01_JPRB, 0.45722E+01_JPRB, 0.39099E+01_JPRB, 0.43479E+01_JPRB, 0.28958E+01_JPRB, 0.32534E+01_JPRB, 0.35490E+01_JPRB ,&
&  0.34114E+01_JPRB, 0.32543E+01_JPRB, 0.29966E+01_JPRB, 0.28936E+01_JPRB, 0.28072E+01_JPRB, 0.24854E+01_JPRB, 0.23490E+01_JPRB ,&
&  0.22384E+01_JPRB, 0.18317E+01_JPRB, 0.16843E+01_JPRB, 0.17901E+01_JPRB, 0.19235E+01_JPRB, 0.17667E+01_JPRB, 0.14443E+01_JPRB ,&
&  0.12494E+01_JPRB, 0.14041E+01_JPRB, 0.12820E+01_JPRB, 0.13994E+01_JPRB, 0.10412E+01_JPRB, 0.60340E+00_JPRB, 0.59975E+00_JPRB ,&
&  0.61524E+00_JPRB, 0.63192E+00_JPRB, 0.63119E+00_JPRB, 0.65968E+00_JPRB, 0.66485E+00_JPRB, 0.65284E+00_JPRB, 0.65484E+00_JPRB ,&
&  0.66492E+00_JPRB, 0.66213E+00_JPRB, 0.66156E+00_JPRB, 0.65130E+00_JPRB, 0.64079E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.62111E+00_JPRB, 0.61426E+00_JPRB, 0.62006E+00_JPRB, 0.61765E+00_JPRB, 0.62857E+00_JPRB, 0.62915E+00_JPRB, 0.63744E+00_JPRB ,&
&  0.65580E+00_JPRB, 0.68073E+00_JPRB, 0.67532E+00_JPRB, 0.65400E+00_JPRB, 0.65188E+00_JPRB, 0.66656E+00_JPRB, 0.71498E+00_JPRB ,&
&  0.82251E+00_JPRB, 0.93646E+00_JPRB, 0.10270E+01_JPRB, 0.11473E+01_JPRB, 0.13748E+01_JPRB, 0.17718E+01_JPRB, 0.21564E+01_JPRB ,&
&  0.24442E+01_JPRB, 0.26425E+01_JPRB, 0.26344E+01_JPRB, 0.27475E+01_JPRB, 0.21977E+01_JPRB, 0.16973E+01_JPRB, 0.19021E+01_JPRB ,&
&  0.20812E+01_JPRB, 0.20718E+01_JPRB, 0.18592E+01_JPRB, 0.18585E+01_JPRB, 0.19112E+01_JPRB, 0.20538E+01_JPRB, 0.23340E+01_JPRB ,&
&  0.26134E+01_JPRB, 0.29080E+01_JPRB, 0.31570E+01_JPRB, 0.33782E+01_JPRB, 0.35683E+01_JPRB, 0.37873E+01_JPRB, 0.39995E+01_JPRB ,&
&  0.42291E+01_JPRB, 0.44092E+01_JPRB, 0.45978E+01_JPRB, 0.49556E+01_JPRB, 0.53030E+01_JPRB, 0.56120E+01_JPRB, 0.57747E+01_JPRB ,&
&  0.57979E+01_JPRB, 0.54569E+01_JPRB, 0.54700E+01_JPRB, 0.55004E+01_JPRB, 0.51358E+01_JPRB, 0.46322E+01_JPRB, 0.44082E+01_JPRB ,&
&  0.41572E+01_JPRB, 0.37638E+01_JPRB, 0.28286E+01_JPRB, 0.33522E+01_JPRB, 0.37287E+01_JPRB, 0.32697E+01_JPRB, 0.30661E+01_JPRB ,&
&  0.28238E+01_JPRB, 0.26621E+01_JPRB, 0.24585E+01_JPRB, 0.24420E+01_JPRB, 0.25551E+01_JPRB, 0.30974E+01_JPRB, 0.23873E+01_JPRB ,&
&  0.20514E+01_JPRB, 0.13900E+01_JPRB, 0.12081E+01_JPRB, 0.13315E+01_JPRB, 0.14667E+01_JPRB, 0.13649E+01_JPRB, 0.10334E+01_JPRB ,&
&  0.98586E+00_JPRB, 0.12139E+01_JPRB, 0.12969E+01_JPRB, 0.17628E+01_JPRB, 0.20242E+01_JPRB, 0.63481E+00_JPRB, 0.56413E+00_JPRB ,&
&  0.56866E+00_JPRB, 0.57175E+00_JPRB, 0.57234E+00_JPRB, 0.58529E+00_JPRB, 0.58117E+00_JPRB, 0.58755E+00_JPRB, 0.60553E+00_JPRB ,&
&  0.61793E+00_JPRB, 0.62277E+00_JPRB, 0.62614E+00_JPRB, 0.61447E+00_JPRB, 0.62154E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.69845E+00_JPRB, 0.69818E+00_JPRB, 0.70200E+00_JPRB, 0.69604E+00_JPRB, 0.69012E+00_JPRB, 0.68798E+00_JPRB, 0.69002E+00_JPRB ,&
&  0.69068E+00_JPRB, 0.68046E+00_JPRB, 0.66911E+00_JPRB, 0.66028E+00_JPRB, 0.64853E+00_JPRB, 0.61895E+00_JPRB, 0.57742E+00_JPRB ,&
&  0.58643E+00_JPRB, 0.60097E+00_JPRB, 0.61821E+00_JPRB, 0.65916E+00_JPRB, 0.75100E+00_JPRB, 0.89069E+00_JPRB, 0.10831E+01_JPRB ,&
&  0.14536E+01_JPRB, 0.18624E+01_JPRB, 0.21561E+01_JPRB, 0.22503E+01_JPRB, 0.25043E+01_JPRB, 0.19084E+01_JPRB, 0.18489E+01_JPRB ,&
&  0.20852E+01_JPRB, 0.18225E+01_JPRB, 0.17270E+01_JPRB, 0.16859E+01_JPRB, 0.17335E+01_JPRB, 0.18520E+01_JPRB, 0.20498E+01_JPRB ,&
&  0.23390E+01_JPRB, 0.26393E+01_JPRB, 0.29055E+01_JPRB, 0.31751E+01_JPRB, 0.34327E+01_JPRB, 0.36981E+01_JPRB, 0.39220E+01_JPRB ,&
&  0.40706E+01_JPRB, 0.42443E+01_JPRB, 0.44870E+01_JPRB, 0.46277E+01_JPRB, 0.50808E+01_JPRB, 0.53730E+01_JPRB, 0.55075E+01_JPRB ,&
&  0.53682E+01_JPRB, 0.50896E+01_JPRB, 0.50710E+01_JPRB, 0.47688E+01_JPRB, 0.44376E+01_JPRB, 0.41330E+01_JPRB, 0.37908E+01_JPRB ,&
&  0.34585E+01_JPRB, 0.29972E+01_JPRB, 0.23525E+01_JPRB, 0.25798E+01_JPRB, 0.26633E+01_JPRB, 0.23629E+01_JPRB, 0.26869E+01_JPRB ,&
&  0.24678E+01_JPRB, 0.23704E+01_JPRB, 0.22493E+01_JPRB, 0.21249E+01_JPRB, 0.20251E+01_JPRB, 0.22908E+01_JPRB, 0.21705E+01_JPRB ,&
&  0.15586E+01_JPRB, 0.11156E+01_JPRB, 0.91727E+00_JPRB, 0.91752E+00_JPRB, 0.10121E+01_JPRB, 0.10794E+01_JPRB, 0.10232E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.10140E+01_JPRB, 0.10990E+01_JPRB, 0.13808E+01_JPRB, 0.96065E+00_JPRB, 0.61081E+00_JPRB, 0.53951E+00_JPRB ,&
&  0.52572E+00_JPRB, 0.52726E+00_JPRB, 0.53492E+00_JPRB, 0.55917E+00_JPRB, 0.58177E+00_JPRB, 0.60271E+00_JPRB, 0.61985E+00_JPRB ,&
&  0.62969E+00_JPRB, 0.65028E+00_JPRB, 0.66930E+00_JPRB, 0.68670E+00_JPRB, 0.69904E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.79679E+00_JPRB, 0.81144E+00_JPRB, 0.80554E+00_JPRB, 0.81347E+00_JPRB, 0.81837E+00_JPRB, 0.83277E+00_JPRB, 0.83114E+00_JPRB ,&
&  0.82566E+00_JPRB, 0.83009E+00_JPRB, 0.80928E+00_JPRB, 0.81423E+00_JPRB, 0.81138E+00_JPRB, 0.81438E+00_JPRB, 0.80329E+00_JPRB ,&
&  0.80793E+00_JPRB, 0.82592E+00_JPRB, 0.83424E+00_JPRB, 0.83911E+00_JPRB, 0.83037E+00_JPRB, 0.80784E+00_JPRB, 0.78703E+00_JPRB ,&
&  0.83346E+00_JPRB, 0.94813E+00_JPRB, 0.10857E+01_JPRB, 0.11881E+01_JPRB, 0.14239E+01_JPRB, 0.14451E+01_JPRB, 0.15884E+01_JPRB ,&
&  0.17624E+01_JPRB, 0.16074E+01_JPRB, 0.17112E+01_JPRB, 0.15178E+01_JPRB, 0.15220E+01_JPRB, 0.15084E+01_JPRB, 0.15276E+01_JPRB ,&
&  0.16056E+01_JPRB, 0.17436E+01_JPRB, 0.19001E+01_JPRB, 0.20744E+01_JPRB, 0.22229E+01_JPRB, 0.22881E+01_JPRB, 0.23300E+01_JPRB ,&
&  0.23649E+01_JPRB, 0.25975E+01_JPRB, 0.32158E+01_JPRB, 0.37211E+01_JPRB, 0.42939E+01_JPRB, 0.45652E+01_JPRB, 0.45965E+01_JPRB ,&
&  0.45281E+01_JPRB, 0.45158E+01_JPRB, 0.38572E+01_JPRB, 0.33410E+01_JPRB, 0.31952E+01_JPRB, 0.28265E+01_JPRB, 0.26872E+01_JPRB ,&
&  0.27494E+01_JPRB, 0.25299E+01_JPRB, 0.19374E+01_JPRB, 0.20439E+01_JPRB, 0.21436E+01_JPRB, 0.20549E+01_JPRB, 0.22895E+01_JPRB ,&
&  0.19906E+01_JPRB, 0.18033E+01_JPRB, 0.17209E+01_JPRB, 0.16722E+01_JPRB, 0.16063E+01_JPRB, 0.15049E+01_JPRB, 0.14773E+01_JPRB ,&
&  0.11932E+01_JPRB, 0.10182E+01_JPRB, 0.93954E+00_JPRB, 0.92665E+00_JPRB, 0.10694E+01_JPRB, 0.13597E+01_JPRB, 0.11087E+01_JPRB ,&
&  0.10919E+01_JPRB, 0.10231E+01_JPRB, 0.13209E+01_JPRB, 0.18682E+01_JPRB, 0.90096E+00_JPRB, 0.65081E+00_JPRB, 0.54069E+00_JPRB ,&
&  0.55089E+00_JPRB, 0.58292E+00_JPRB, 0.60981E+00_JPRB, 0.64428E+00_JPRB, 0.66293E+00_JPRB, 0.66741E+00_JPRB, 0.67953E+00_JPRB ,&
&  0.70966E+00_JPRB, 0.73884E+00_JPRB, 0.77923E+00_JPRB, 0.78456E+00_JPRB, 0.78000E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.85816E+00_JPRB, 0.87689E+00_JPRB, 0.87921E+00_JPRB, 0.89660E+00_JPRB, 0.91861E+00_JPRB, 0.92559E+00_JPRB, 0.92786E+00_JPRB ,&
&  0.91788E+00_JPRB, 0.91962E+00_JPRB, 0.91608E+00_JPRB, 0.93410E+00_JPRB, 0.93734E+00_JPRB, 0.93952E+00_JPRB, 0.94236E+00_JPRB ,&
&  0.95930E+00_JPRB, 0.97304E+00_JPRB, 0.10074E+01_JPRB, 0.10120E+01_JPRB, 0.10078E+01_JPRB, 0.10145E+01_JPRB, 0.10375E+01_JPRB ,&
&  0.10636E+01_JPRB, 0.10926E+01_JPRB, 0.11319E+01_JPRB, 0.11669E+01_JPRB, 0.12862E+01_JPRB, 0.14526E+01_JPRB, 0.17451E+01_JPRB ,&
&  0.16370E+01_JPRB, 0.16658E+01_JPRB, 0.15600E+01_JPRB, 0.13496E+01_JPRB, 0.13414E+01_JPRB, 0.13719E+01_JPRB, 0.13178E+01_JPRB ,&
&  0.12625E+01_JPRB, 0.12454E+01_JPRB, 0.12275E+01_JPRB, 0.12291E+01_JPRB, 0.12458E+01_JPRB, 0.12220E+01_JPRB, 0.12107E+01_JPRB ,&
&  0.11641E+01_JPRB, 0.11977E+01_JPRB, 0.15425E+01_JPRB, 0.20079E+01_JPRB, 0.24638E+01_JPRB, 0.28649E+01_JPRB, 0.25860E+01_JPRB ,&
&  0.24751E+01_JPRB, 0.26380E+01_JPRB, 0.24037E+01_JPRB, 0.22019E+01_JPRB, 0.19808E+01_JPRB, 0.17658E+01_JPRB, 0.16527E+01_JPRB ,&
&  0.18054E+01_JPRB, 0.19143E+01_JPRB, 0.17155E+01_JPRB, 0.19265E+01_JPRB, 0.19385E+01_JPRB, 0.19271E+01_JPRB, 0.17006E+01_JPRB ,&
&  0.14693E+01_JPRB, 0.13528E+01_JPRB, 0.13281E+01_JPRB, 0.13147E+01_JPRB, 0.13215E+01_JPRB, 0.12452E+01_JPRB, 0.11234E+01_JPRB ,&
&  0.10469E+01_JPRB, 0.95780E+00_JPRB, 0.92949E+00_JPRB, 0.88030E+00_JPRB, 0.10978E+01_JPRB, 0.12043E+01_JPRB, 0.15541E+01_JPRB ,&
&  0.11261E+01_JPRB, 0.10243E+01_JPRB, 0.89642E+00_JPRB, 0.13545E+01_JPRB, 0.81496E+00_JPRB, 0.57164E+00_JPRB, 0.57721E+00_JPRB ,&
&  0.64979E+00_JPRB, 0.69145E+00_JPRB, 0.71025E+00_JPRB, 0.73999E+00_JPRB, 0.75047E+00_JPRB, 0.74651E+00_JPRB, 0.75651E+00_JPRB ,&
&  0.79565E+00_JPRB, 0.80584E+00_JPRB, 0.81547E+00_JPRB, 0.81846E+00_JPRB, 0.83741E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.88721E+00_JPRB, 0.91552E+00_JPRB, 0.93257E+00_JPRB, 0.94942E+00_JPRB, 0.97141E+00_JPRB, 0.98341E+00_JPRB, 0.99247E+00_JPRB ,&
&  0.99575E+00_JPRB, 0.99294E+00_JPRB, 0.10014E+01_JPRB, 0.10162E+01_JPRB, 0.10246E+01_JPRB, 0.10223E+01_JPRB, 0.10292E+01_JPRB ,&
&  0.10464E+01_JPRB, 0.10738E+01_JPRB, 0.11035E+01_JPRB, 0.10982E+01_JPRB, 0.10954E+01_JPRB, 0.11135E+01_JPRB, 0.11381E+01_JPRB ,&
&  0.11410E+01_JPRB, 0.11364E+01_JPRB, 0.11460E+01_JPRB, 0.11597E+01_JPRB, 0.11853E+01_JPRB, 0.11947E+01_JPRB, 0.13016E+01_JPRB ,&
&  0.12705E+01_JPRB, 0.17887E+01_JPRB, 0.17084E+01_JPRB, 0.14874E+01_JPRB, 0.14748E+01_JPRB, 0.13767E+01_JPRB, 0.13700E+01_JPRB ,&
&  0.13777E+01_JPRB, 0.13439E+01_JPRB, 0.13203E+01_JPRB, 0.12954E+01_JPRB, 0.13062E+01_JPRB, 0.13279E+01_JPRB, 0.12986E+01_JPRB ,&
&  0.13170E+01_JPRB, 0.13524E+01_JPRB, 0.14679E+01_JPRB, 0.16215E+01_JPRB, 0.17702E+01_JPRB, 0.18212E+01_JPRB, 0.16886E+01_JPRB ,&
&  0.15956E+01_JPRB, 0.14456E+01_JPRB, 0.11924E+01_JPRB, 0.12031E+01_JPRB, 0.11733E+01_JPRB, 0.96710E+00_JPRB, 0.88248E+00_JPRB ,&
&  0.12000E+01_JPRB, 0.13683E+01_JPRB, 0.17126E+01_JPRB, 0.17166E+01_JPRB, 0.16405E+01_JPRB, 0.16387E+01_JPRB, 0.14072E+01_JPRB ,&
&  0.12444E+01_JPRB, 0.11375E+01_JPRB, 0.11249E+01_JPRB, 0.10865E+01_JPRB, 0.10955E+01_JPRB, 0.10725E+01_JPRB, 0.98143E+00_JPRB ,&
&  0.90687E+00_JPRB, 0.87620E+00_JPRB, 0.87532E+00_JPRB, 0.91274E+00_JPRB, 0.94516E+00_JPRB, 0.16100E+01_JPRB, 0.25359E+01_JPRB ,&
&  0.12831E+01_JPRB, 0.92530E+00_JPRB, 0.61458E+00_JPRB, 0.77699E+00_JPRB, 0.66201E+00_JPRB, 0.59614E+00_JPRB, 0.64853E+00_JPRB ,&
&  0.76482E+00_JPRB, 0.81479E+00_JPRB, 0.82459E+00_JPRB, 0.81814E+00_JPRB, 0.79677E+00_JPRB, 0.77736E+00_JPRB, 0.76983E+00_JPRB ,&
&  0.78797E+00_JPRB, 0.80626E+00_JPRB, 0.82699E+00_JPRB, 0.83994E+00_JPRB, 0.85873E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.90266E+00_JPRB, 0.93011E+00_JPRB, 0.95153E+00_JPRB, 0.97593E+00_JPRB, 0.10017E+01_JPRB, 0.10254E+01_JPRB, 0.10470E+01_JPRB ,&
&  0.10649E+01_JPRB, 0.10796E+01_JPRB, 0.10934E+01_JPRB, 0.11105E+01_JPRB, 0.11318E+01_JPRB, 0.11488E+01_JPRB, 0.11707E+01_JPRB ,&
&  0.12042E+01_JPRB, 0.12322E+01_JPRB, 0.12637E+01_JPRB, 0.12638E+01_JPRB, 0.12665E+01_JPRB, 0.12942E+01_JPRB, 0.13200E+01_JPRB ,&
&  0.13293E+01_JPRB, 0.13468E+01_JPRB, 0.13747E+01_JPRB, 0.14089E+01_JPRB, 0.14549E+01_JPRB, 0.15472E+01_JPRB, 0.14995E+01_JPRB ,&
&  0.16639E+01_JPRB, 0.18672E+01_JPRB, 0.18125E+01_JPRB, 0.16453E+01_JPRB, 0.15753E+01_JPRB, 0.15644E+01_JPRB, 0.16012E+01_JPRB ,&
&  0.16670E+01_JPRB, 0.16668E+01_JPRB, 0.16766E+01_JPRB, 0.16725E+01_JPRB, 0.16862E+01_JPRB, 0.17284E+01_JPRB, 0.17686E+01_JPRB ,&
&  0.18225E+01_JPRB, 0.18926E+01_JPRB, 0.19923E+01_JPRB, 0.21227E+01_JPRB, 0.22671E+01_JPRB, 0.22692E+01_JPRB, 0.22053E+01_JPRB ,&
&  0.20465E+01_JPRB, 0.18765E+01_JPRB, 0.14517E+01_JPRB, 0.13506E+01_JPRB, 0.12560E+01_JPRB, 0.10819E+01_JPRB, 0.94259E+00_JPRB ,&
&  0.12061E+01_JPRB, 0.12654E+01_JPRB, 0.14039E+01_JPRB, 0.15974E+01_JPRB, 0.14850E+01_JPRB, 0.13684E+01_JPRB, 0.12131E+01_JPRB ,&
&  0.10907E+01_JPRB, 0.10073E+01_JPRB, 0.97539E+00_JPRB, 0.93489E+00_JPRB, 0.92724E+00_JPRB, 0.92954E+00_JPRB, 0.85652E+00_JPRB ,&
&  0.82731E+00_JPRB, 0.82804E+00_JPRB, 0.84244E+00_JPRB, 0.88043E+00_JPRB, 0.99624E+00_JPRB, 0.10671E+01_JPRB, 0.15821E+01_JPRB ,&
&  0.11642E+01_JPRB, 0.73274E+00_JPRB, 0.74042E+00_JPRB, 0.73066E+00_JPRB, 0.68538E+00_JPRB, 0.65506E+00_JPRB, 0.72741E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.10003E+01_JPRB, 0.10294E+01_JPRB, 0.97608E+00_JPRB, 0.88638E+00_JPRB, 0.83521E+00_JPRB, 0.80917E+00_JPRB ,&
&  0.82207E+00_JPRB, 0.83448E+00_JPRB, 0.84999E+00_JPRB, 0.85735E+00_JPRB, 0.87379E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.89503E+00_JPRB, 0.92578E+00_JPRB, 0.95147E+00_JPRB, 0.97671E+00_JPRB, 0.10007E+01_JPRB, 0.10240E+01_JPRB, 0.10492E+01_JPRB ,&
&  0.10674E+01_JPRB, 0.10919E+01_JPRB, 0.11208E+01_JPRB, 0.11499E+01_JPRB, 0.11760E+01_JPRB, 0.12087E+01_JPRB, 0.12372E+01_JPRB ,&
&  0.12755E+01_JPRB, 0.13028E+01_JPRB, 0.13318E+01_JPRB, 0.13382E+01_JPRB, 0.13444E+01_JPRB, 0.13650E+01_JPRB, 0.13821E+01_JPRB ,&
&  0.13830E+01_JPRB, 0.13923E+01_JPRB, 0.14097E+01_JPRB, 0.14398E+01_JPRB, 0.14550E+01_JPRB, 0.15308E+01_JPRB, 0.13859E+01_JPRB ,&
&  0.18040E+01_JPRB, 0.20333E+01_JPRB, 0.19658E+01_JPRB, 0.18184E+01_JPRB, 0.17420E+01_JPRB, 0.17875E+01_JPRB, 0.18505E+01_JPRB ,&
&  0.19341E+01_JPRB, 0.20523E+01_JPRB, 0.20256E+01_JPRB, 0.20229E+01_JPRB, 0.20561E+01_JPRB, 0.21328E+01_JPRB, 0.22454E+01_JPRB ,&
&  0.23619E+01_JPRB, 0.24899E+01_JPRB, 0.26284E+01_JPRB, 0.27646E+01_JPRB, 0.28923E+01_JPRB, 0.29776E+01_JPRB, 0.29928E+01_JPRB ,&
&  0.29083E+01_JPRB, 0.27782E+01_JPRB, 0.24792E+01_JPRB, 0.24277E+01_JPRB, 0.22802E+01_JPRB, 0.18647E+01_JPRB, 0.14674E+01_JPRB ,&
&  0.13869E+01_JPRB, 0.13839E+01_JPRB, 0.13424E+01_JPRB, 0.14594E+01_JPRB, 0.13735E+01_JPRB, 0.12549E+01_JPRB, 0.11233E+01_JPRB ,&
&  0.10089E+01_JPRB, 0.91869E+00_JPRB, 0.89540E+00_JPRB, 0.84305E+00_JPRB, 0.80188E+00_JPRB, 0.77454E+00_JPRB, 0.75395E+00_JPRB ,&
&  0.76645E+00_JPRB, 0.79006E+00_JPRB, 0.82555E+00_JPRB, 0.86279E+00_JPRB, 0.89555E+00_JPRB, 0.87496E+00_JPRB, 0.12985E+01_JPRB ,&
&  0.10451E+01_JPRB, 0.85790E+00_JPRB, 0.10024E+01_JPRB, 0.61888E+00_JPRB, 0.70136E+00_JPRB, 0.76377E+00_JPRB, 0.89228E+00_JPRB ,&
&  0.97824E+00_JPRB, 0.13556E+01_JPRB, 0.18374E+01_JPRB, 0.14699E+01_JPRB, 0.11924E+01_JPRB, 0.91802E+00_JPRB, 0.82902E+00_JPRB ,&
&  0.82978E+00_JPRB, 0.83496E+00_JPRB, 0.84618E+00_JPRB, 0.85191E+00_JPRB, 0.86734E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.87879E+00_JPRB, 0.90279E+00_JPRB, 0.93053E+00_JPRB, 0.95585E+00_JPRB, 0.97373E+00_JPRB, 0.98989E+00_JPRB, 0.10096E+01_JPRB ,&
&  0.10216E+01_JPRB, 0.10269E+01_JPRB, 0.10428E+01_JPRB, 0.10674E+01_JPRB, 0.10970E+01_JPRB, 0.11305E+01_JPRB, 0.11689E+01_JPRB ,&
&  0.12207E+01_JPRB, 0.12686E+01_JPRB, 0.13300E+01_JPRB, 0.13775E+01_JPRB, 0.14119E+01_JPRB, 0.14474E+01_JPRB, 0.14825E+01_JPRB ,&
&  0.15337E+01_JPRB, 0.15675E+01_JPRB, 0.15992E+01_JPRB, 0.16616E+01_JPRB, 0.17131E+01_JPRB, 0.17791E+01_JPRB, 0.15051E+01_JPRB ,&
&  0.19584E+01_JPRB, 0.22928E+01_JPRB, 0.21662E+01_JPRB, 0.19721E+01_JPRB, 0.18750E+01_JPRB, 0.18977E+01_JPRB, 0.19257E+01_JPRB ,&
&  0.19836E+01_JPRB, 0.20278E+01_JPRB, 0.19982E+01_JPRB, 0.21842E+01_JPRB, 0.23287E+01_JPRB, 0.24113E+01_JPRB, 0.25888E+01_JPRB ,&
&  0.27702E+01_JPRB, 0.29495E+01_JPRB, 0.31168E+01_JPRB, 0.32732E+01_JPRB, 0.34099E+01_JPRB, 0.35565E+01_JPRB, 0.36120E+01_JPRB ,&
&  0.36219E+01_JPRB, 0.36236E+01_JPRB, 0.35793E+01_JPRB, 0.32448E+01_JPRB, 0.30877E+01_JPRB, 0.28824E+01_JPRB, 0.22986E+01_JPRB ,&
&  0.18004E+01_JPRB, 0.14053E+01_JPRB, 0.12724E+01_JPRB, 0.12869E+01_JPRB, 0.11939E+01_JPRB, 0.10698E+01_JPRB, 0.96680E+00_JPRB ,&
&  0.87478E+00_JPRB, 0.81778E+00_JPRB, 0.81770E+00_JPRB, 0.77387E+00_JPRB, 0.67964E+00_JPRB, 0.69475E+00_JPRB, 0.72086E+00_JPRB ,&
&  0.71672E+00_JPRB, 0.74059E+00_JPRB, 0.77437E+00_JPRB, 0.78971E+00_JPRB, 0.84513E+00_JPRB, 0.97272E+00_JPRB, 0.10461E+01_JPRB ,&
&  0.11006E+01_JPRB, 0.86541E+00_JPRB, 0.83420E+00_JPRB, 0.86111E+00_JPRB, 0.79645E+00_JPRB, 0.10175E+01_JPRB, 0.13697E+01_JPRB ,&
&  0.19702E+01_JPRB, 0.21092E+01_JPRB, 0.27116E+01_JPRB, 0.42030E+01_JPRB, 0.31978E+01_JPRB, 0.21421E+01_JPRB, 0.85377E+00_JPRB ,&
&  0.85761E+00_JPRB, 0.85291E+00_JPRB, 0.87056E+00_JPRB, 0.87004E+00_JPRB, 0.87665E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.75891E+00_JPRB, 0.75956E+00_JPRB, 0.78884E+00_JPRB, 0.82051E+00_JPRB, 0.84983E+00_JPRB, 0.87780E+00_JPRB, 0.89955E+00_JPRB ,&
&  0.93070E+00_JPRB, 0.95643E+00_JPRB, 0.96553E+00_JPRB, 0.96608E+00_JPRB, 0.98124E+00_JPRB, 0.99043E+00_JPRB, 0.10193E+01_JPRB ,&
&  0.10630E+01_JPRB, 0.11129E+01_JPRB, 0.11809E+01_JPRB, 0.12284E+01_JPRB, 0.12770E+01_JPRB, 0.13480E+01_JPRB, 0.14166E+01_JPRB ,&
&  0.14918E+01_JPRB, 0.15854E+01_JPRB, 0.16669E+01_JPRB, 0.17530E+01_JPRB, 0.18386E+01_JPRB, 0.18344E+01_JPRB, 0.14188E+01_JPRB ,&
&  0.20539E+01_JPRB, 0.23687E+01_JPRB, 0.23101E+01_JPRB, 0.20916E+01_JPRB, 0.19531E+01_JPRB, 0.19097E+01_JPRB, 0.19247E+01_JPRB ,&
&  0.19416E+01_JPRB, 0.19198E+01_JPRB, 0.19039E+01_JPRB, 0.19646E+01_JPRB, 0.23544E+01_JPRB, 0.24193E+01_JPRB, 0.26153E+01_JPRB ,&
&  0.28136E+01_JPRB, 0.30006E+01_JPRB, 0.31608E+01_JPRB, 0.32988E+01_JPRB, 0.34043E+01_JPRB, 0.35332E+01_JPRB, 0.36724E+01_JPRB ,&
&  0.37989E+01_JPRB, 0.38730E+01_JPRB, 0.39966E+01_JPRB, 0.36370E+01_JPRB, 0.34638E+01_JPRB, 0.37234E+01_JPRB, 0.32919E+01_JPRB ,&
&  0.20951E+01_JPRB, 0.14244E+01_JPRB, 0.13904E+01_JPRB, 0.12815E+01_JPRB, 0.11189E+01_JPRB, 0.97920E+00_JPRB, 0.88531E+00_JPRB ,&
&  0.82111E+00_JPRB, 0.77283E+00_JPRB, 0.74751E+00_JPRB, 0.70479E+00_JPRB, 0.69232E+00_JPRB, 0.70819E+00_JPRB, 0.70972E+00_JPRB ,&
&  0.73554E+00_JPRB, 0.70825E+00_JPRB, 0.70242E+00_JPRB, 0.71942E+00_JPRB, 0.79226E+00_JPRB, 0.94896E+00_JPRB, 0.12470E+01_JPRB ,&
&  0.13857E+01_JPRB, 0.83971E+00_JPRB, 0.83161E+00_JPRB, 0.84255E+00_JPRB, 0.89526E+00_JPRB, 0.10188E+01_JPRB, 0.12174E+01_JPRB ,&
&  0.15655E+01_JPRB, 0.20490E+01_JPRB, 0.19647E+01_JPRB, 0.15554E+01_JPRB, 0.19401E+01_JPRB, 0.16381E+01_JPRB, 0.75785E+00_JPRB ,&
&  0.76286E+00_JPRB, 0.74089E+00_JPRB, 0.74851E+00_JPRB, 0.75739E+00_JPRB, 0.76145E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.62184E+00_JPRB, 0.61070E+00_JPRB, 0.62044E+00_JPRB, 0.64184E+00_JPRB, 0.66166E+00_JPRB, 0.69876E+00_JPRB, 0.74410E+00_JPRB ,&
&  0.80030E+00_JPRB, 0.83891E+00_JPRB, 0.85407E+00_JPRB, 0.86012E+00_JPRB, 0.87220E+00_JPRB, 0.88210E+00_JPRB, 0.90157E+00_JPRB ,&
&  0.93443E+00_JPRB, 0.96785E+00_JPRB, 0.10126E+01_JPRB, 0.10512E+01_JPRB, 0.10874E+01_JPRB, 0.11604E+01_JPRB, 0.12444E+01_JPRB ,&
&  0.13468E+01_JPRB, 0.14565E+01_JPRB, 0.15861E+01_JPRB, 0.17244E+01_JPRB, 0.18671E+01_JPRB, 0.19288E+01_JPRB, 0.19504E+01_JPRB ,&
&  0.15099E+01_JPRB, 0.21899E+01_JPRB, 0.22768E+01_JPRB, 0.21333E+01_JPRB, 0.19879E+01_JPRB, 0.19537E+01_JPRB, 0.19142E+01_JPRB ,&
&  0.18713E+01_JPRB, 0.17451E+01_JPRB, 0.17732E+01_JPRB, 0.18848E+01_JPRB, 0.21344E+01_JPRB, 0.22650E+01_JPRB, 0.24415E+01_JPRB ,&
&  0.26108E+01_JPRB, 0.27536E+01_JPRB, 0.28804E+01_JPRB, 0.30067E+01_JPRB, 0.31655E+01_JPRB, 0.33779E+01_JPRB, 0.36501E+01_JPRB ,&
&  0.38925E+01_JPRB, 0.40786E+01_JPRB, 0.42052E+01_JPRB, 0.39435E+01_JPRB, 0.36675E+01_JPRB, 0.40781E+01_JPRB, 0.38780E+01_JPRB ,&
&  0.23804E+01_JPRB, 0.15166E+01_JPRB, 0.15330E+01_JPRB, 0.15000E+01_JPRB, 0.12456E+01_JPRB, 0.10913E+01_JPRB, 0.96495E+00_JPRB ,&
&  0.81964E+00_JPRB, 0.73070E+00_JPRB, 0.73550E+00_JPRB, 0.72857E+00_JPRB, 0.73493E+00_JPRB, 0.73613E+00_JPRB, 0.74912E+00_JPRB ,&
&  0.77375E+00_JPRB, 0.74168E+00_JPRB, 0.72341E+00_JPRB, 0.71758E+00_JPRB, 0.74111E+00_JPRB, 0.77664E+00_JPRB, 0.81349E+00_JPRB ,&
&  0.87697E+00_JPRB, 0.82124E+00_JPRB, 0.81963E+00_JPRB, 0.85416E+00_JPRB, 0.88685E+00_JPRB, 0.93186E+00_JPRB, 0.98497E+00_JPRB ,&
&  0.10273E+01_JPRB, 0.10704E+01_JPRB, 0.11027E+01_JPRB, 0.10966E+01_JPRB, 0.10049E+01_JPRB, 0.92251E+00_JPRB, 0.68156E+00_JPRB ,&
&  0.66553E+00_JPRB, 0.64729E+00_JPRB, 0.64671E+00_JPRB, 0.64368E+00_JPRB, 0.64184E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.54618E+00_JPRB, 0.53982E+00_JPRB, 0.54741E+00_JPRB, 0.55873E+00_JPRB, 0.57962E+00_JPRB, 0.61450E+00_JPRB, 0.66436E+00_JPRB ,&
&  0.71964E+00_JPRB, 0.75176E+00_JPRB, 0.77819E+00_JPRB, 0.77148E+00_JPRB, 0.77536E+00_JPRB, 0.79549E+00_JPRB, 0.81003E+00_JPRB ,&
&  0.84469E+00_JPRB, 0.88083E+00_JPRB, 0.90257E+00_JPRB, 0.91009E+00_JPRB, 0.92494E+00_JPRB, 0.96423E+00_JPRB, 0.10196E+01_JPRB ,&
&  0.10895E+01_JPRB, 0.11874E+01_JPRB, 0.13221E+01_JPRB, 0.15043E+01_JPRB, 0.17051E+01_JPRB, 0.17863E+01_JPRB, 0.17889E+01_JPRB ,&
&  0.90509E+00_JPRB, 0.15864E+01_JPRB, 0.21598E+01_JPRB, 0.21595E+01_JPRB, 0.20898E+01_JPRB, 0.20221E+01_JPRB, 0.19773E+01_JPRB ,&
&  0.17799E+01_JPRB, 0.15570E+01_JPRB, 0.15156E+01_JPRB, 0.18009E+01_JPRB, 0.18555E+01_JPRB, 0.20269E+01_JPRB, 0.21777E+01_JPRB ,&
&  0.23399E+01_JPRB, 0.24787E+01_JPRB, 0.25705E+01_JPRB, 0.26539E+01_JPRB, 0.27908E+01_JPRB, 0.29950E+01_JPRB, 0.32493E+01_JPRB ,&
&  0.35755E+01_JPRB, 0.39743E+01_JPRB, 0.42586E+01_JPRB, 0.40957E+01_JPRB, 0.39974E+01_JPRB, 0.42850E+01_JPRB, 0.53374E+01_JPRB ,&
&  0.32608E+01_JPRB, 0.18257E+01_JPRB, 0.17982E+01_JPRB, 0.17964E+01_JPRB, 0.14524E+01_JPRB, 0.12796E+01_JPRB, 0.11373E+01_JPRB ,&
&  0.93356E+00_JPRB, 0.84389E+00_JPRB, 0.80221E+00_JPRB, 0.77587E+00_JPRB, 0.78789E+00_JPRB, 0.81595E+00_JPRB, 0.86715E+00_JPRB ,&
&  0.89008E+00_JPRB, 0.87326E+00_JPRB, 0.84681E+00_JPRB, 0.83790E+00_JPRB, 0.82940E+00_JPRB, 0.83415E+00_JPRB, 0.82907E+00_JPRB ,&
&  0.84490E+00_JPRB, 0.83337E+00_JPRB, 0.83730E+00_JPRB, 0.86950E+00_JPRB, 0.91449E+00_JPRB, 0.92787E+00_JPRB, 0.92948E+00_JPRB ,&
&  0.89628E+00_JPRB, 0.91035E+00_JPRB, 0.91495E+00_JPRB, 0.88965E+00_JPRB, 0.85175E+00_JPRB, 0.79438E+00_JPRB, 0.69604E+00_JPRB ,&
&  0.64756E+00_JPRB, 0.59387E+00_JPRB, 0.56255E+00_JPRB, 0.54865E+00_JPRB, 0.55751E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.52692E+00_JPRB, 0.52107E+00_JPRB, 0.52193E+00_JPRB, 0.54483E+00_JPRB, 0.54304E+00_JPRB, 0.56089E+00_JPRB, 0.58553E+00_JPRB ,&
&  0.64056E+00_JPRB, 0.68961E+00_JPRB, 0.73402E+00_JPRB, 0.73196E+00_JPRB, 0.73052E+00_JPRB, 0.71516E+00_JPRB, 0.71943E+00_JPRB ,&
&  0.73438E+00_JPRB, 0.75984E+00_JPRB, 0.78835E+00_JPRB, 0.80226E+00_JPRB, 0.81290E+00_JPRB, 0.82578E+00_JPRB, 0.84481E+00_JPRB ,&
&  0.87429E+00_JPRB, 0.91875E+00_JPRB, 0.10038E+01_JPRB, 0.11633E+01_JPRB, 0.14198E+01_JPRB, 0.16910E+01_JPRB, 0.18920E+01_JPRB ,&
&  0.13071E+01_JPRB, 0.57758E+00_JPRB, 0.16065E+01_JPRB, 0.21406E+01_JPRB, 0.21291E+01_JPRB, 0.20426E+01_JPRB, 0.19554E+01_JPRB ,&
&  0.16974E+01_JPRB, 0.15458E+01_JPRB, 0.13603E+01_JPRB, 0.15028E+01_JPRB, 0.14673E+01_JPRB, 0.15897E+01_JPRB, 0.17696E+01_JPRB ,&
&  0.19597E+01_JPRB, 0.21351E+01_JPRB, 0.22974E+01_JPRB, 0.23682E+01_JPRB, 0.24045E+01_JPRB, 0.24920E+01_JPRB, 0.26662E+01_JPRB ,&
&  0.29735E+01_JPRB, 0.34836E+01_JPRB, 0.41489E+01_JPRB, 0.44522E+01_JPRB, 0.51936E+01_JPRB, 0.49045E+01_JPRB, 0.53250E+01_JPRB ,&
&  0.34284E+01_JPRB, 0.22576E+01_JPRB, 0.20961E+01_JPRB, 0.21014E+01_JPRB, 0.17145E+01_JPRB, 0.14009E+01_JPRB, 0.13742E+01_JPRB ,&
&  0.10967E+01_JPRB, 0.96949E+00_JPRB, 0.92505E+00_JPRB, 0.88532E+00_JPRB, 0.85692E+00_JPRB, 0.88976E+00_JPRB, 0.94748E+00_JPRB ,&
&  0.99006E+00_JPRB, 0.99651E+00_JPRB, 0.97953E+00_JPRB, 0.96466E+00_JPRB, 0.95003E+00_JPRB, 0.92275E+00_JPRB, 0.89818E+00_JPRB ,&
&  0.90384E+00_JPRB, 0.91383E+00_JPRB, 0.92740E+00_JPRB, 0.94156E+00_JPRB, 0.95752E+00_JPRB, 0.93910E+00_JPRB, 0.92783E+00_JPRB ,&
&  0.92332E+00_JPRB, 0.93566E+00_JPRB, 0.91480E+00_JPRB, 0.91238E+00_JPRB, 0.88521E+00_JPRB, 0.83858E+00_JPRB, 0.80157E+00_JPRB ,&
&  0.73798E+00_JPRB, 0.64385E+00_JPRB, 0.57632E+00_JPRB, 0.55278E+00_JPRB, 0.53328E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.56387E+00_JPRB, 0.55008E+00_JPRB, 0.55033E+00_JPRB, 0.56296E+00_JPRB, 0.57468E+00_JPRB, 0.58555E+00_JPRB, 0.57141E+00_JPRB ,&
&  0.58819E+00_JPRB, 0.62863E+00_JPRB, 0.65246E+00_JPRB, 0.67744E+00_JPRB, 0.67304E+00_JPRB, 0.66013E+00_JPRB, 0.65134E+00_JPRB ,&
&  0.64787E+00_JPRB, 0.65172E+00_JPRB, 0.66948E+00_JPRB, 0.67904E+00_JPRB, 0.68735E+00_JPRB, 0.69912E+00_JPRB, 0.70604E+00_JPRB ,&
&  0.71363E+00_JPRB, 0.72969E+00_JPRB, 0.76454E+00_JPRB, 0.83728E+00_JPRB, 0.97229E+00_JPRB, 0.12182E+01_JPRB, 0.15775E+01_JPRB ,&
&  0.22136E+01_JPRB, 0.15996E+01_JPRB, 0.75412E+00_JPRB, 0.17047E+01_JPRB, 0.22034E+01_JPRB, 0.22193E+01_JPRB, 0.18939E+01_JPRB ,&
&  0.17735E+01_JPRB, 0.15347E+01_JPRB, 0.13638E+01_JPRB, 0.14067E+01_JPRB, 0.12165E+01_JPRB, 0.11697E+01_JPRB, 0.12330E+01_JPRB ,&
&  0.13757E+01_JPRB, 0.15939E+01_JPRB, 0.18314E+01_JPRB, 0.19972E+01_JPRB, 0.20887E+01_JPRB, 0.21333E+01_JPRB, 0.22032E+01_JPRB ,&
&  0.24060E+01_JPRB, 0.27293E+01_JPRB, 0.33106E+01_JPRB, 0.43325E+01_JPRB, 0.63712E+01_JPRB, 0.59644E+01_JPRB, 0.59369E+01_JPRB ,&
&  0.36596E+01_JPRB, 0.30241E+01_JPRB, 0.27630E+01_JPRB, 0.24903E+01_JPRB, 0.19620E+01_JPRB, 0.16960E+01_JPRB, 0.16949E+01_JPRB ,&
&  0.13376E+01_JPRB, 0.11143E+01_JPRB, 0.10339E+01_JPRB, 0.97151E+00_JPRB, 0.93923E+00_JPRB, 0.95104E+00_JPRB, 0.99358E+00_JPRB ,&
&  0.10413E+01_JPRB, 0.10412E+01_JPRB, 0.10217E+01_JPRB, 0.10047E+01_JPRB, 0.99329E+00_JPRB, 0.97949E+00_JPRB, 0.96088E+00_JPRB ,&
&  0.95169E+00_JPRB, 0.96143E+00_JPRB, 0.98294E+00_JPRB, 0.97507E+00_JPRB, 0.94468E+00_JPRB, 0.92331E+00_JPRB, 0.94841E+00_JPRB ,&
&  0.93650E+00_JPRB, 0.95681E+00_JPRB, 0.94067E+00_JPRB, 0.94949E+00_JPRB, 0.92161E+00_JPRB, 0.89550E+00_JPRB, 0.87079E+00_JPRB ,&
&  0.83052E+00_JPRB, 0.74770E+00_JPRB, 0.65872E+00_JPRB, 0.57353E+00_JPRB, 0.56286E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.63445E+00_JPRB, 0.61763E+00_JPRB, 0.62170E+00_JPRB, 0.61578E+00_JPRB, 0.60606E+00_JPRB, 0.60632E+00_JPRB, 0.60129E+00_JPRB ,&
&  0.59065E+00_JPRB, 0.58466E+00_JPRB, 0.57621E+00_JPRB, 0.60613E+00_JPRB, 0.62233E+00_JPRB, 0.61164E+00_JPRB, 0.60111E+00_JPRB ,&
&  0.59247E+00_JPRB, 0.59297E+00_JPRB, 0.59942E+00_JPRB, 0.60477E+00_JPRB, 0.60820E+00_JPRB, 0.61623E+00_JPRB, 0.62314E+00_JPRB ,&
&  0.62011E+00_JPRB, 0.62137E+00_JPRB, 0.63232E+00_JPRB, 0.65946E+00_JPRB, 0.71530E+00_JPRB, 0.81819E+00_JPRB, 0.10102E+01_JPRB ,&
&  0.14758E+01_JPRB, 0.15190E+01_JPRB, 0.71309E+00_JPRB, 0.13474E+01_JPRB, 0.22123E+01_JPRB, 0.23199E+01_JPRB, 0.21915E+01_JPRB ,&
&  0.19919E+01_JPRB, 0.15774E+01_JPRB, 0.14387E+01_JPRB, 0.14262E+01_JPRB, 0.12251E+01_JPRB, 0.10777E+01_JPRB, 0.99032E+00_JPRB ,&
&  0.96756E+00_JPRB, 0.10385E+01_JPRB, 0.12113E+01_JPRB, 0.14192E+01_JPRB, 0.16049E+01_JPRB, 0.17201E+01_JPRB, 0.17827E+01_JPRB ,&
&  0.18713E+01_JPRB, 0.20512E+01_JPRB, 0.24290E+01_JPRB, 0.29528E+01_JPRB, 0.51454E+01_JPRB, 0.63419E+01_JPRB, 0.67036E+01_JPRB ,&
&  0.48961E+01_JPRB, 0.41915E+01_JPRB, 0.36004E+01_JPRB, 0.30201E+01_JPRB, 0.23309E+01_JPRB, 0.21247E+01_JPRB, 0.20883E+01_JPRB ,&
&  0.16900E+01_JPRB, 0.13555E+01_JPRB, 0.12038E+01_JPRB, 0.11020E+01_JPRB, 0.10320E+01_JPRB, 0.10039E+01_JPRB, 0.99685E+00_JPRB ,&
&  0.10168E+01_JPRB, 0.10284E+01_JPRB, 0.10105E+01_JPRB, 0.99529E+00_JPRB, 0.99502E+00_JPRB, 0.99445E+00_JPRB, 0.99103E+00_JPRB ,&
&  0.97860E+00_JPRB, 0.98464E+00_JPRB, 0.10009E+01_JPRB, 0.99205E+00_JPRB, 0.99448E+00_JPRB, 0.10047E+01_JPRB, 0.97021E+00_JPRB ,&
&  0.93543E+00_JPRB, 0.93808E+00_JPRB, 0.93150E+00_JPRB, 0.90631E+00_JPRB, 0.95077E+00_JPRB, 0.94361E+00_JPRB, 0.92721E+00_JPRB ,&
&  0.96249E+00_JPRB, 0.10632E+01_JPRB, 0.10122E+01_JPRB, 0.70025E+00_JPRB, 0.64768E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.69028E+00_JPRB, 0.66442E+00_JPRB, 0.66008E+00_JPRB, 0.65388E+00_JPRB, 0.64030E+00_JPRB, 0.63057E+00_JPRB, 0.62802E+00_JPRB ,&
&  0.61265E+00_JPRB, 0.58733E+00_JPRB, 0.56319E+00_JPRB, 0.56149E+00_JPRB, 0.57314E+00_JPRB, 0.58377E+00_JPRB, 0.57429E+00_JPRB ,&
&  0.56066E+00_JPRB, 0.55759E+00_JPRB, 0.56960E+00_JPRB, 0.57589E+00_JPRB, 0.57536E+00_JPRB, 0.57829E+00_JPRB, 0.57664E+00_JPRB ,&
&  0.57455E+00_JPRB, 0.56963E+00_JPRB, 0.57041E+00_JPRB, 0.58254E+00_JPRB, 0.60709E+00_JPRB, 0.65546E+00_JPRB, 0.73331E+00_JPRB ,&
&  0.10003E+01_JPRB, 0.12227E+01_JPRB, 0.15170E+01_JPRB, 0.23244E+01_JPRB, 0.28827E+01_JPRB, 0.27412E+01_JPRB, 0.25667E+01_JPRB ,&
&  0.23927E+01_JPRB, 0.20170E+01_JPRB, 0.17517E+01_JPRB, 0.15233E+01_JPRB, 0.12827E+01_JPRB, 0.11057E+01_JPRB, 0.98602E+00_JPRB ,&
&  0.91844E+00_JPRB, 0.88284E+00_JPRB, 0.89772E+00_JPRB, 0.97168E+00_JPRB, 0.10886E+01_JPRB, 0.12167E+01_JPRB, 0.13099E+01_JPRB ,&
&  0.13897E+01_JPRB, 0.15181E+01_JPRB, 0.17695E+01_JPRB, 0.21019E+01_JPRB, 0.33457E+01_JPRB, 0.53882E+01_JPRB, 0.64630E+01_JPRB ,&
&  0.64299E+01_JPRB, 0.56909E+01_JPRB, 0.46540E+01_JPRB, 0.37188E+01_JPRB, 0.27101E+01_JPRB, 0.27446E+01_JPRB, 0.24271E+01_JPRB ,&
&  0.20319E+01_JPRB, 0.17085E+01_JPRB, 0.14666E+01_JPRB, 0.12976E+01_JPRB, 0.11716E+01_JPRB, 0.10764E+01_JPRB, 0.10258E+01_JPRB ,&
&  0.10060E+01_JPRB, 0.10062E+01_JPRB, 0.99135E+00_JPRB, 0.97100E+00_JPRB, 0.97640E+00_JPRB, 0.98710E+00_JPRB, 0.99315E+00_JPRB ,&
&  0.98500E+00_JPRB, 0.98550E+00_JPRB, 0.96034E+00_JPRB, 0.97817E+00_JPRB, 0.10038E+01_JPRB, 0.99211E+00_JPRB, 0.95675E+00_JPRB ,&
&  0.91396E+00_JPRB, 0.91953E+00_JPRB, 0.92345E+00_JPRB, 0.91688E+00_JPRB, 0.93304E+00_JPRB, 0.94882E+00_JPRB, 0.94039E+00_JPRB ,&
&  0.94922E+00_JPRB, 0.91886E+00_JPRB, 0.85645E+00_JPRB, 0.76044E+00_JPRB, 0.72061E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.74945E+00_JPRB, 0.72165E+00_JPRB, 0.70284E+00_JPRB, 0.68705E+00_JPRB, 0.67961E+00_JPRB, 0.67971E+00_JPRB, 0.67466E+00_JPRB ,&
&  0.66195E+00_JPRB, 0.63630E+00_JPRB, 0.60261E+00_JPRB, 0.57266E+00_JPRB, 0.56935E+00_JPRB, 0.56577E+00_JPRB, 0.56486E+00_JPRB ,&
&  0.55322E+00_JPRB, 0.54907E+00_JPRB, 0.55423E+00_JPRB, 0.55506E+00_JPRB, 0.55355E+00_JPRB, 0.55026E+00_JPRB, 0.54793E+00_JPRB ,&
&  0.55134E+00_JPRB, 0.55242E+00_JPRB, 0.55133E+00_JPRB, 0.55365E+00_JPRB, 0.56428E+00_JPRB, 0.59403E+00_JPRB, 0.64655E+00_JPRB ,&
&  0.79999E+00_JPRB, 0.11277E+01_JPRB, 0.13235E+01_JPRB, 0.26605E+01_JPRB, 0.30931E+01_JPRB, 0.30097E+01_JPRB, 0.26604E+01_JPRB ,&
&  0.23336E+01_JPRB, 0.22877E+01_JPRB, 0.19012E+01_JPRB, 0.15503E+01_JPRB, 0.12575E+01_JPRB, 0.10937E+01_JPRB, 0.97367E+00_JPRB ,&
&  0.88985E+00_JPRB, 0.84492E+00_JPRB, 0.83072E+00_JPRB, 0.83306E+00_JPRB, 0.84650E+00_JPRB, 0.89540E+00_JPRB, 0.95000E+00_JPRB ,&
&  0.10125E+01_JPRB, 0.11509E+01_JPRB, 0.14037E+01_JPRB, 0.16463E+01_JPRB, 0.24002E+01_JPRB, 0.39964E+01_JPRB, 0.53919E+01_JPRB ,&
&  0.60546E+01_JPRB, 0.57151E+01_JPRB, 0.50197E+01_JPRB, 0.40800E+01_JPRB, 0.32613E+01_JPRB, 0.29471E+01_JPRB, 0.26076E+01_JPRB ,&
&  0.22660E+01_JPRB, 0.20348E+01_JPRB, 0.17277E+01_JPRB, 0.15036E+01_JPRB, 0.13805E+01_JPRB, 0.12396E+01_JPRB, 0.11185E+01_JPRB ,&
&  0.10429E+01_JPRB, 0.10081E+01_JPRB, 0.98648E+00_JPRB, 0.96284E+00_JPRB, 0.95620E+00_JPRB, 0.96954E+00_JPRB, 0.97976E+00_JPRB ,&
&  0.97185E+00_JPRB, 0.97191E+00_JPRB, 0.92762E+00_JPRB, 0.92355E+00_JPRB, 0.95215E+00_JPRB, 0.94968E+00_JPRB, 0.92062E+00_JPRB ,&
&  0.89850E+00_JPRB, 0.89621E+00_JPRB, 0.89744E+00_JPRB, 0.88967E+00_JPRB, 0.88095E+00_JPRB, 0.95484E+00_JPRB, 0.95319E+00_JPRB ,&
&  0.95110E+00_JPRB, 0.88213E+00_JPRB, 0.84245E+00_JPRB, 0.80006E+00_JPRB, 0.77637E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.78064E+00_JPRB, 0.76155E+00_JPRB, 0.74535E+00_JPRB, 0.72618E+00_JPRB, 0.71506E+00_JPRB, 0.71021E+00_JPRB, 0.70260E+00_JPRB ,&
&  0.68580E+00_JPRB, 0.66121E+00_JPRB, 0.63305E+00_JPRB, 0.59563E+00_JPRB, 0.57250E+00_JPRB, 0.56638E+00_JPRB, 0.57178E+00_JPRB ,&
&  0.56217E+00_JPRB, 0.54602E+00_JPRB, 0.54649E+00_JPRB, 0.55001E+00_JPRB, 0.54891E+00_JPRB, 0.54119E+00_JPRB, 0.53222E+00_JPRB ,&
&  0.53512E+00_JPRB, 0.53994E+00_JPRB, 0.54344E+00_JPRB, 0.54576E+00_JPRB, 0.54988E+00_JPRB, 0.56339E+00_JPRB, 0.60454E+00_JPRB ,&
&  0.73516E+00_JPRB, 0.13947E+01_JPRB, 0.13889E+01_JPRB, 0.24724E+01_JPRB, 0.26707E+01_JPRB, 0.26257E+01_JPRB, 0.23944E+01_JPRB ,&
&  0.21456E+01_JPRB, 0.19814E+01_JPRB, 0.16653E+01_JPRB, 0.14280E+01_JPRB, 0.12026E+01_JPRB, 0.10353E+01_JPRB, 0.92823E+00_JPRB ,&
&  0.85719E+00_JPRB, 0.80406E+00_JPRB, 0.77273E+00_JPRB, 0.76348E+00_JPRB, 0.76066E+00_JPRB, 0.76843E+00_JPRB, 0.78807E+00_JPRB ,&
&  0.81930E+00_JPRB, 0.91229E+00_JPRB, 0.11539E+01_JPRB, 0.16928E+01_JPRB, 0.21558E+01_JPRB, 0.32772E+01_JPRB, 0.49457E+01_JPRB ,&
&  0.58057E+01_JPRB, 0.56730E+01_JPRB, 0.47395E+01_JPRB, 0.39045E+01_JPRB, 0.32730E+01_JPRB, 0.28456E+01_JPRB, 0.24900E+01_JPRB ,&
&  0.22488E+01_JPRB, 0.20728E+01_JPRB, 0.18901E+01_JPRB, 0.16667E+01_JPRB, 0.15028E+01_JPRB, 0.13816E+01_JPRB, 0.12610E+01_JPRB ,&
&  0.11410E+01_JPRB, 0.10456E+01_JPRB, 0.99177E+00_JPRB, 0.95128E+00_JPRB, 0.92753E+00_JPRB, 0.93351E+00_JPRB, 0.94511E+00_JPRB ,&
&  0.93782E+00_JPRB, 0.93597E+00_JPRB, 0.89442E+00_JPRB, 0.88397E+00_JPRB, 0.90870E+00_JPRB, 0.91145E+00_JPRB, 0.88466E+00_JPRB ,&
&  0.88076E+00_JPRB, 0.88377E+00_JPRB, 0.88270E+00_JPRB, 0.89461E+00_JPRB, 0.87059E+00_JPRB, 0.95260E+00_JPRB, 0.94303E+00_JPRB ,&
&  0.94028E+00_JPRB, 0.88789E+00_JPRB, 0.84636E+00_JPRB, 0.83069E+00_JPRB, 0.80936E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.79361E+00_JPRB, 0.77500E+00_JPRB, 0.76118E+00_JPRB, 0.75156E+00_JPRB, 0.74441E+00_JPRB, 0.73102E+00_JPRB, 0.71562E+00_JPRB ,&
&  0.70096E+00_JPRB, 0.68366E+00_JPRB, 0.65532E+00_JPRB, 0.61486E+00_JPRB, 0.58215E+00_JPRB, 0.56623E+00_JPRB, 0.56779E+00_JPRB ,&
&  0.56796E+00_JPRB, 0.55797E+00_JPRB, 0.55049E+00_JPRB, 0.54929E+00_JPRB, 0.54779E+00_JPRB, 0.53945E+00_JPRB, 0.52884E+00_JPRB ,&
&  0.52127E+00_JPRB, 0.52237E+00_JPRB, 0.52649E+00_JPRB, 0.53241E+00_JPRB, 0.53766E+00_JPRB, 0.54535E+00_JPRB, 0.57384E+00_JPRB ,&
&  0.70065E+00_JPRB, 0.72345E+00_JPRB, 0.16116E+01_JPRB, 0.21204E+01_JPRB, 0.21517E+01_JPRB, 0.20794E+01_JPRB, 0.20589E+01_JPRB ,&
&  0.19051E+01_JPRB, 0.16799E+01_JPRB, 0.14194E+01_JPRB, 0.12057E+01_JPRB, 0.10459E+01_JPRB, 0.94057E+00_JPRB, 0.86289E+00_JPRB ,&
&  0.80278E+00_JPRB, 0.75713E+00_JPRB, 0.72123E+00_JPRB, 0.70270E+00_JPRB, 0.70579E+00_JPRB, 0.71471E+00_JPRB, 0.72670E+00_JPRB ,&
&  0.74495E+00_JPRB, 0.78526E+00_JPRB, 0.91710E+00_JPRB, 0.13400E+01_JPRB, 0.22974E+01_JPRB, 0.31292E+01_JPRB, 0.44558E+01_JPRB ,&
&  0.55236E+01_JPRB, 0.50567E+01_JPRB, 0.40828E+01_JPRB, 0.33908E+01_JPRB, 0.28842E+01_JPRB, 0.25274E+01_JPRB, 0.22579E+01_JPRB ,&
&  0.20534E+01_JPRB, 0.19080E+01_JPRB, 0.17868E+01_JPRB, 0.16536E+01_JPRB, 0.15382E+01_JPRB, 0.14474E+01_JPRB, 0.13577E+01_JPRB ,&
&  0.12542E+01_JPRB, 0.11524E+01_JPRB, 0.10606E+01_JPRB, 0.98110E+00_JPRB, 0.91669E+00_JPRB, 0.89898E+00_JPRB, 0.89840E+00_JPRB ,&
&  0.88842E+00_JPRB, 0.88473E+00_JPRB, 0.86664E+00_JPRB, 0.86815E+00_JPRB, 0.89740E+00_JPRB, 0.89894E+00_JPRB, 0.88545E+00_JPRB ,&
&  0.86016E+00_JPRB, 0.86968E+00_JPRB, 0.86456E+00_JPRB, 0.85526E+00_JPRB, 0.81583E+00_JPRB, 0.90893E+00_JPRB, 0.90521E+00_JPRB ,&
&  0.90195E+00_JPRB, 0.88871E+00_JPRB, 0.85014E+00_JPRB, 0.83716E+00_JPRB, 0.81996E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.79320E+00_JPRB, 0.77156E+00_JPRB, 0.75875E+00_JPRB, 0.75034E+00_JPRB, 0.74387E+00_JPRB, 0.73751E+00_JPRB, 0.72145E+00_JPRB ,&
&  0.70230E+00_JPRB, 0.69090E+00_JPRB, 0.67473E+00_JPRB, 0.65262E+00_JPRB, 0.61998E+00_JPRB, 0.59440E+00_JPRB, 0.58677E+00_JPRB ,&
&  0.58198E+00_JPRB, 0.57425E+00_JPRB, 0.56046E+00_JPRB, 0.55212E+00_JPRB, 0.54342E+00_JPRB, 0.53291E+00_JPRB, 0.52626E+00_JPRB ,&
&  0.51505E+00_JPRB, 0.51041E+00_JPRB, 0.51312E+00_JPRB, 0.52020E+00_JPRB, 0.52688E+00_JPRB, 0.53521E+00_JPRB, 0.54753E+00_JPRB ,&
&  0.64941E+00_JPRB, 0.10811E+01_JPRB, 0.14934E+01_JPRB, 0.17229E+01_JPRB, 0.17441E+01_JPRB, 0.16356E+01_JPRB, 0.16309E+01_JPRB ,&
&  0.14711E+01_JPRB, 0.13162E+01_JPRB, 0.11566E+01_JPRB, 0.10079E+01_JPRB, 0.89190E+00_JPRB, 0.83030E+00_JPRB, 0.79589E+00_JPRB ,&
&  0.75583E+00_JPRB, 0.72022E+00_JPRB, 0.68862E+00_JPRB, 0.66178E+00_JPRB, 0.65238E+00_JPRB, 0.66431E+00_JPRB, 0.68373E+00_JPRB ,&
&  0.69431E+00_JPRB, 0.71066E+00_JPRB, 0.77612E+00_JPRB, 0.10339E+01_JPRB, 0.18200E+01_JPRB, 0.22513E+01_JPRB, 0.34621E+01_JPRB ,&
&  0.41661E+01_JPRB, 0.37399E+01_JPRB, 0.31121E+01_JPRB, 0.26480E+01_JPRB, 0.22682E+01_JPRB, 0.19786E+01_JPRB, 0.17688E+01_JPRB ,&
&  0.16479E+01_JPRB, 0.15647E+01_JPRB, 0.14915E+01_JPRB, 0.14211E+01_JPRB, 0.13559E+01_JPRB, 0.13045E+01_JPRB, 0.12629E+01_JPRB ,&
&  0.12178E+01_JPRB, 0.11598E+01_JPRB, 0.10945E+01_JPRB, 0.10209E+01_JPRB, 0.94735E+00_JPRB, 0.90013E+00_JPRB, 0.87394E+00_JPRB ,&
&  0.85165E+00_JPRB, 0.84560E+00_JPRB, 0.84075E+00_JPRB, 0.85029E+00_JPRB, 0.87753E+00_JPRB, 0.88876E+00_JPRB, 0.88414E+00_JPRB ,&
&  0.84745E+00_JPRB, 0.83375E+00_JPRB, 0.82239E+00_JPRB, 0.78394E+00_JPRB, 0.77494E+00_JPRB, 0.85207E+00_JPRB, 0.84100E+00_JPRB ,&
&  0.82953E+00_JPRB, 0.82455E+00_JPRB, 0.81435E+00_JPRB, 0.80823E+00_JPRB, 0.81888E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.80440E+00_JPRB, 0.77439E+00_JPRB, 0.75463E+00_JPRB, 0.74210E+00_JPRB, 0.73541E+00_JPRB, 0.72819E+00_JPRB, 0.71419E+00_JPRB ,&
&  0.69739E+00_JPRB, 0.68100E+00_JPRB, 0.66689E+00_JPRB, 0.65646E+00_JPRB, 0.63480E+00_JPRB, 0.59841E+00_JPRB, 0.56995E+00_JPRB ,&
&  0.56063E+00_JPRB, 0.55316E+00_JPRB, 0.54665E+00_JPRB, 0.53575E+00_JPRB, 0.52668E+00_JPRB, 0.52264E+00_JPRB, 0.51841E+00_JPRB ,&
&  0.51372E+00_JPRB, 0.50910E+00_JPRB, 0.51056E+00_JPRB, 0.51546E+00_JPRB, 0.51992E+00_JPRB, 0.52396E+00_JPRB, 0.53046E+00_JPRB ,&
&  0.58095E+00_JPRB, 0.84114E+00_JPRB, 0.10243E+01_JPRB, 0.12871E+01_JPRB, 0.13643E+01_JPRB, 0.12899E+01_JPRB, 0.12393E+01_JPRB ,&
&  0.11589E+01_JPRB, 0.10661E+01_JPRB, 0.97933E+00_JPRB, 0.89651E+00_JPRB, 0.80515E+00_JPRB, 0.75209E+00_JPRB, 0.72706E+00_JPRB ,&
&  0.70885E+00_JPRB, 0.68434E+00_JPRB, 0.65826E+00_JPRB, 0.63159E+00_JPRB, 0.61385E+00_JPRB, 0.61800E+00_JPRB, 0.63049E+00_JPRB ,&
&  0.63021E+00_JPRB, 0.63263E+00_JPRB, 0.65004E+00_JPRB, 0.75113E+00_JPRB, 0.11422E+01_JPRB, 0.15357E+01_JPRB, 0.20851E+01_JPRB ,&
&  0.21383E+01_JPRB, 0.22631E+01_JPRB, 0.20730E+01_JPRB, 0.18504E+01_JPRB, 0.16557E+01_JPRB, 0.15046E+01_JPRB, 0.13903E+01_JPRB ,&
&  0.13114E+01_JPRB, 0.12465E+01_JPRB, 0.11977E+01_JPRB, 0.11609E+01_JPRB, 0.11316E+01_JPRB, 0.10966E+01_JPRB, 0.10768E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.10421E+01_JPRB, 0.10115E+01_JPRB, 0.97594E+00_JPRB, 0.93330E+00_JPRB, 0.89694E+00_JPRB, 0.86972E+00_JPRB ,&
&  0.84536E+00_JPRB, 0.81908E+00_JPRB, 0.80755E+00_JPRB, 0.81489E+00_JPRB, 0.82496E+00_JPRB, 0.83222E+00_JPRB, 0.82590E+00_JPRB ,&
&  0.81417E+00_JPRB, 0.80433E+00_JPRB, 0.78892E+00_JPRB, 0.75482E+00_JPRB, 0.74363E+00_JPRB, 0.79450E+00_JPRB, 0.78749E+00_JPRB ,&
&  0.77229E+00_JPRB, 0.76125E+00_JPRB, 0.76407E+00_JPRB, 0.77403E+00_JPRB, 0.82994E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.82737E+00_JPRB, 0.79359E+00_JPRB, 0.75790E+00_JPRB, 0.73484E+00_JPRB, 0.71894E+00_JPRB, 0.70522E+00_JPRB, 0.68774E+00_JPRB ,&
&  0.67104E+00_JPRB, 0.65453E+00_JPRB, 0.64055E+00_JPRB, 0.63003E+00_JPRB, 0.61873E+00_JPRB, 0.59377E+00_JPRB, 0.55981E+00_JPRB ,&
&  0.53654E+00_JPRB, 0.52889E+00_JPRB, 0.52382E+00_JPRB, 0.51504E+00_JPRB, 0.50708E+00_JPRB, 0.50425E+00_JPRB, 0.50153E+00_JPRB ,&
&  0.50194E+00_JPRB, 0.50324E+00_JPRB, 0.50728E+00_JPRB, 0.50935E+00_JPRB, 0.50715E+00_JPRB, 0.51336E+00_JPRB, 0.51351E+00_JPRB ,&
&  0.55189E+00_JPRB, 0.63275E+00_JPRB, 0.76164E+00_JPRB, 0.90765E+00_JPRB, 0.10093E+01_JPRB, 0.10427E+01_JPRB, 0.10235E+01_JPRB ,&
&  0.98966E+00_JPRB, 0.95258E+00_JPRB, 0.89855E+00_JPRB, 0.84410E+00_JPRB, 0.78248E+00_JPRB, 0.71655E+00_JPRB, 0.67767E+00_JPRB ,&
&  0.66492E+00_JPRB, 0.65020E+00_JPRB, 0.63501E+00_JPRB, 0.61687E+00_JPRB, 0.59981E+00_JPRB, 0.59418E+00_JPRB, 0.59532E+00_JPRB ,&
&  0.59088E+00_JPRB, 0.58574E+00_JPRB, 0.58149E+00_JPRB, 0.58957E+00_JPRB, 0.65391E+00_JPRB, 0.84185E+00_JPRB, 0.10922E+01_JPRB ,&
&  0.11972E+01_JPRB, 0.12752E+01_JPRB, 0.13223E+01_JPRB, 0.13165E+01_JPRB, 0.12593E+01_JPRB, 0.11741E+01_JPRB, 0.10843E+01_JPRB ,&
&  0.10179E+01_JPRB, 0.97195E+00_JPRB, 0.94740E+00_JPRB, 0.93932E+00_JPRB, 0.93668E+00_JPRB, 0.92758E+00_JPRB, 0.90936E+00_JPRB ,&
&  0.89391E+00_JPRB, 0.88659E+00_JPRB, 0.88619E+00_JPRB, 0.87985E+00_JPRB, 0.87022E+00_JPRB, 0.85837E+00_JPRB, 0.85049E+00_JPRB ,&
&  0.84982E+00_JPRB, 0.84105E+00_JPRB, 0.81570E+00_JPRB, 0.79140E+00_JPRB, 0.78887E+00_JPRB, 0.78788E+00_JPRB, 0.78141E+00_JPRB ,&
&  0.77435E+00_JPRB, 0.77500E+00_JPRB, 0.76013E+00_JPRB, 0.75839E+00_JPRB, 0.77966E+00_JPRB, 0.76087E+00_JPRB, 0.75065E+00_JPRB ,&
&  0.72977E+00_JPRB, 0.71768E+00_JPRB, 0.72264E+00_JPRB, 0.74189E+00_JPRB, 0.81337E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.69971E+00_JPRB, 0.69362E+00_JPRB, 0.68481E+00_JPRB, 0.67678E+00_JPRB, 0.66440E+00_JPRB, 0.65209E+00_JPRB, 0.63636E+00_JPRB ,&
&  0.62149E+00_JPRB, 0.60709E+00_JPRB, 0.59488E+00_JPRB, 0.59080E+00_JPRB, 0.58308E+00_JPRB, 0.57155E+00_JPRB, 0.54768E+00_JPRB ,&
&  0.52762E+00_JPRB, 0.51230E+00_JPRB, 0.50284E+00_JPRB, 0.49830E+00_JPRB, 0.49508E+00_JPRB, 0.49129E+00_JPRB, 0.48978E+00_JPRB ,&
&  0.49075E+00_JPRB, 0.49070E+00_JPRB, 0.49335E+00_JPRB, 0.49293E+00_JPRB, 0.49554E+00_JPRB, 0.49124E+00_JPRB, 0.48827E+00_JPRB ,&
&  0.51322E+00_JPRB, 0.53591E+00_JPRB, 0.61110E+00_JPRB, 0.69430E+00_JPRB, 0.78001E+00_JPRB, 0.83891E+00_JPRB, 0.88481E+00_JPRB ,&
&  0.91021E+00_JPRB, 0.88602E+00_JPRB, 0.84336E+00_JPRB, 0.78691E+00_JPRB, 0.74273E+00_JPRB, 0.69587E+00_JPRB, 0.64911E+00_JPRB ,&
&  0.62563E+00_JPRB, 0.61334E+00_JPRB, 0.60331E+00_JPRB, 0.59469E+00_JPRB, 0.58623E+00_JPRB, 0.58119E+00_JPRB, 0.57833E+00_JPRB ,&
&  0.57049E+00_JPRB, 0.56093E+00_JPRB, 0.54970E+00_JPRB, 0.54568E+00_JPRB, 0.54892E+00_JPRB, 0.59195E+00_JPRB, 0.70952E+00_JPRB ,&
&  0.77461E+00_JPRB, 0.80971E+00_JPRB, 0.84008E+00_JPRB, 0.88627E+00_JPRB, 0.91964E+00_JPRB, 0.92074E+00_JPRB, 0.87408E+00_JPRB ,&
&  0.82810E+00_JPRB, 0.79307E+00_JPRB, 0.77336E+00_JPRB, 0.76572E+00_JPRB, 0.76723E+00_JPRB, 0.76984E+00_JPRB, 0.77184E+00_JPRB ,&
&  0.76362E+00_JPRB, 0.75491E+00_JPRB, 0.75266E+00_JPRB, 0.75455E+00_JPRB, 0.76078E+00_JPRB, 0.76643E+00_JPRB, 0.77529E+00_JPRB ,&
&  0.79214E+00_JPRB, 0.81367E+00_JPRB, 0.83003E+00_JPRB, 0.81430E+00_JPRB, 0.78514E+00_JPRB, 0.76370E+00_JPRB, 0.74918E+00_JPRB ,&
&  0.73937E+00_JPRB, 0.73918E+00_JPRB, 0.74345E+00_JPRB, 0.75600E+00_JPRB, 0.75477E+00_JPRB, 0.73852E+00_JPRB, 0.71706E+00_JPRB ,&
&  0.69760E+00_JPRB, 0.68462E+00_JPRB, 0.68375E+00_JPRB, 0.69051E+00_JPRB, 0.67547E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.62244E+00_JPRB, 0.61892E+00_JPRB, 0.61366E+00_JPRB, 0.61001E+00_JPRB, 0.60251E+00_JPRB, 0.59398E+00_JPRB, 0.58493E+00_JPRB ,&
&  0.57336E+00_JPRB, 0.56200E+00_JPRB, 0.55238E+00_JPRB, 0.54948E+00_JPRB, 0.54793E+00_JPRB, 0.54647E+00_JPRB, 0.53911E+00_JPRB ,&
&  0.52728E+00_JPRB, 0.51575E+00_JPRB, 0.50541E+00_JPRB, 0.49860E+00_JPRB, 0.49580E+00_JPRB, 0.49249E+00_JPRB, 0.48835E+00_JPRB ,&
&  0.48546E+00_JPRB, 0.48548E+00_JPRB, 0.48423E+00_JPRB, 0.48265E+00_JPRB, 0.48028E+00_JPRB, 0.47432E+00_JPRB, 0.47363E+00_JPRB ,&
&  0.48608E+00_JPRB, 0.45851E+00_JPRB, 0.49971E+00_JPRB, 0.54103E+00_JPRB, 0.57983E+00_JPRB, 0.64176E+00_JPRB, 0.71868E+00_JPRB ,&
&  0.78417E+00_JPRB, 0.79507E+00_JPRB, 0.77757E+00_JPRB, 0.73696E+00_JPRB, 0.70624E+00_JPRB, 0.68334E+00_JPRB, 0.65048E+00_JPRB ,&
&  0.61793E+00_JPRB, 0.59615E+00_JPRB, 0.58055E+00_JPRB, 0.56810E+00_JPRB, 0.55914E+00_JPRB, 0.55150E+00_JPRB, 0.54456E+00_JPRB ,&
&  0.53504E+00_JPRB, 0.52547E+00_JPRB, 0.51720E+00_JPRB, 0.51318E+00_JPRB, 0.51489E+00_JPRB, 0.52245E+00_JPRB, 0.55659E+00_JPRB ,&
&  0.59362E+00_JPRB, 0.61650E+00_JPRB, 0.62274E+00_JPRB, 0.64370E+00_JPRB, 0.67589E+00_JPRB, 0.70324E+00_JPRB, 0.71115E+00_JPRB ,&
&  0.69423E+00_JPRB, 0.66690E+00_JPRB, 0.64308E+00_JPRB, 0.62565E+00_JPRB, 0.61717E+00_JPRB, 0.61950E+00_JPRB, 0.63017E+00_JPRB ,&
&  0.63555E+00_JPRB, 0.63533E+00_JPRB, 0.63648E+00_JPRB, 0.64141E+00_JPRB, 0.64836E+00_JPRB, 0.66028E+00_JPRB, 0.67687E+00_JPRB ,&
&  0.69778E+00_JPRB, 0.72207E+00_JPRB, 0.74926E+00_JPRB, 0.76977E+00_JPRB, 0.77363E+00_JPRB, 0.76836E+00_JPRB, 0.75701E+00_JPRB ,&
&  0.74401E+00_JPRB, 0.73311E+00_JPRB, 0.72258E+00_JPRB, 0.69815E+00_JPRB, 0.71750E+00_JPRB, 0.71150E+00_JPRB, 0.69064E+00_JPRB ,&
&  0.67186E+00_JPRB, 0.65251E+00_JPRB, 0.64069E+00_JPRB, 0.58559E+00_JPRB, 0.61775E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.55980E+00_JPRB, 0.55655E+00_JPRB, 0.55088E+00_JPRB, 0.54585E+00_JPRB, 0.54251E+00_JPRB, 0.53763E+00_JPRB, 0.53385E+00_JPRB ,&
&  0.52846E+00_JPRB, 0.52440E+00_JPRB, 0.51737E+00_JPRB, 0.51293E+00_JPRB, 0.51103E+00_JPRB, 0.51128E+00_JPRB, 0.51125E+00_JPRB ,&
&  0.50758E+00_JPRB, 0.50061E+00_JPRB, 0.49316E+00_JPRB, 0.48712E+00_JPRB, 0.48227E+00_JPRB, 0.47688E+00_JPRB, 0.47161E+00_JPRB ,&
&  0.46688E+00_JPRB, 0.46276E+00_JPRB, 0.45907E+00_JPRB, 0.45838E+00_JPRB, 0.45783E+00_JPRB, 0.45628E+00_JPRB, 0.45345E+00_JPRB ,&
&  0.45937E+00_JPRB, 0.40849E+00_JPRB, 0.42431E+00_JPRB, 0.44222E+00_JPRB, 0.46253E+00_JPRB, 0.51035E+00_JPRB, 0.57784E+00_JPRB ,&
&  0.63845E+00_JPRB, 0.68071E+00_JPRB, 0.69198E+00_JPRB, 0.67417E+00_JPRB, 0.65219E+00_JPRB, 0.63786E+00_JPRB, 0.62579E+00_JPRB ,&
&  0.60314E+00_JPRB, 0.58303E+00_JPRB, 0.56384E+00_JPRB, 0.54497E+00_JPRB, 0.52724E+00_JPRB, 0.51188E+00_JPRB, 0.49885E+00_JPRB ,&
&  0.49047E+00_JPRB, 0.48674E+00_JPRB, 0.48439E+00_JPRB, 0.48290E+00_JPRB, 0.48372E+00_JPRB, 0.48631E+00_JPRB, 0.48945E+00_JPRB ,&
&  0.49755E+00_JPRB, 0.51332E+00_JPRB, 0.52084E+00_JPRB, 0.52741E+00_JPRB, 0.54113E+00_JPRB, 0.56090E+00_JPRB, 0.57169E+00_JPRB ,&
&  0.57280E+00_JPRB, 0.56639E+00_JPRB, 0.55638E+00_JPRB, 0.54541E+00_JPRB, 0.53560E+00_JPRB, 0.52743E+00_JPRB, 0.52621E+00_JPRB ,&
&  0.52675E+00_JPRB, 0.52975E+00_JPRB, 0.53387E+00_JPRB, 0.54175E+00_JPRB, 0.55134E+00_JPRB, 0.56117E+00_JPRB, 0.57106E+00_JPRB ,&
&  0.58126E+00_JPRB, 0.59290E+00_JPRB, 0.60952E+00_JPRB, 0.62846E+00_JPRB, 0.64380E+00_JPRB, 0.65133E+00_JPRB, 0.65304E+00_JPRB ,&
&  0.65061E+00_JPRB, 0.64684E+00_JPRB, 0.63876E+00_JPRB, 0.63802E+00_JPRB, 0.63468E+00_JPRB, 0.63340E+00_JPRB, 0.62459E+00_JPRB ,&
&  0.61363E+00_JPRB, 0.60237E+00_JPRB, 0.52483E+00_JPRB, 0.55431E+00_JPRB, 0.55822E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.51896E+00_JPRB, 0.51237E+00_JPRB, 0.50610E+00_JPRB, 0.49959E+00_JPRB, 0.49453E+00_JPRB, 0.49115E+00_JPRB, 0.48992E+00_JPRB ,&
&  0.49116E+00_JPRB, 0.49290E+00_JPRB, 0.49177E+00_JPRB, 0.48640E+00_JPRB, 0.48126E+00_JPRB, 0.47831E+00_JPRB, 0.47743E+00_JPRB ,&
&  0.47592E+00_JPRB, 0.47246E+00_JPRB, 0.46752E+00_JPRB, 0.46269E+00_JPRB, 0.45690E+00_JPRB, 0.45184E+00_JPRB, 0.44737E+00_JPRB ,&
&  0.44355E+00_JPRB, 0.44149E+00_JPRB, 0.43979E+00_JPRB, 0.43850E+00_JPRB, 0.43837E+00_JPRB, 0.43732E+00_JPRB, 0.44065E+00_JPRB ,&
&  0.41314E+00_JPRB, 0.38067E+00_JPRB, 0.39427E+00_JPRB, 0.40725E+00_JPRB, 0.42002E+00_JPRB, 0.44919E+00_JPRB, 0.49099E+00_JPRB ,&
&  0.54015E+00_JPRB, 0.58217E+00_JPRB, 0.60681E+00_JPRB, 0.60399E+00_JPRB, 0.58954E+00_JPRB, 0.58006E+00_JPRB, 0.57511E+00_JPRB ,&
&  0.56338E+00_JPRB, 0.54920E+00_JPRB, 0.53712E+00_JPRB, 0.52287E+00_JPRB, 0.50757E+00_JPRB, 0.49366E+00_JPRB, 0.48256E+00_JPRB ,&
&  0.47404E+00_JPRB, 0.46892E+00_JPRB, 0.46730E+00_JPRB, 0.46550E+00_JPRB, 0.46388E+00_JPRB, 0.46384E+00_JPRB, 0.46276E+00_JPRB ,&
&  0.46287E+00_JPRB, 0.46395E+00_JPRB, 0.46850E+00_JPRB, 0.47246E+00_JPRB, 0.47785E+00_JPRB, 0.48268E+00_JPRB, 0.48487E+00_JPRB ,&
&  0.48344E+00_JPRB, 0.48223E+00_JPRB, 0.48297E+00_JPRB, 0.48093E+00_JPRB, 0.47362E+00_JPRB, 0.46546E+00_JPRB, 0.46179E+00_JPRB ,&
&  0.46186E+00_JPRB, 0.46303E+00_JPRB, 0.46489E+00_JPRB, 0.46885E+00_JPRB, 0.47442E+00_JPRB, 0.48075E+00_JPRB, 0.48731E+00_JPRB ,&
&  0.49410E+00_JPRB, 0.50041E+00_JPRB, 0.50666E+00_JPRB, 0.51465E+00_JPRB, 0.52595E+00_JPRB, 0.53627E+00_JPRB, 0.54493E+00_JPRB ,&
&  0.55173E+00_JPRB, 0.55687E+00_JPRB, 0.56085E+00_JPRB, 0.56630E+00_JPRB, 0.57085E+00_JPRB, 0.57302E+00_JPRB, 0.56941E+00_JPRB ,&
&  0.56101E+00_JPRB, 0.54911E+00_JPRB, 0.54257E+00_JPRB, 0.53241E+00_JPRB, 0.52425E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.46581E+00_JPRB, 0.46179E+00_JPRB, 0.45685E+00_JPRB, 0.45169E+00_JPRB, 0.44659E+00_JPRB, 0.44424E+00_JPRB, 0.44388E+00_JPRB ,&
&  0.44644E+00_JPRB, 0.44787E+00_JPRB, 0.45044E+00_JPRB, 0.44909E+00_JPRB, 0.44657E+00_JPRB, 0.44449E+00_JPRB, 0.44234E+00_JPRB ,&
&  0.43986E+00_JPRB, 0.43648E+00_JPRB, 0.43309E+00_JPRB, 0.43045E+00_JPRB, 0.42734E+00_JPRB, 0.42295E+00_JPRB, 0.41877E+00_JPRB ,&
&  0.41543E+00_JPRB, 0.41366E+00_JPRB, 0.41240E+00_JPRB, 0.41127E+00_JPRB, 0.41012E+00_JPRB, 0.41037E+00_JPRB, 0.41011E+00_JPRB ,&
&  0.41190E+00_JPRB, 0.38145E+00_JPRB, 0.39267E+00_JPRB, 0.39693E+00_JPRB, 0.40497E+00_JPRB, 0.42225E+00_JPRB, 0.44369E+00_JPRB ,&
&  0.46877E+00_JPRB, 0.49472E+00_JPRB, 0.51442E+00_JPRB, 0.52016E+00_JPRB, 0.51464E+00_JPRB, 0.51109E+00_JPRB, 0.51247E+00_JPRB ,&
&  0.51072E+00_JPRB, 0.50572E+00_JPRB, 0.50133E+00_JPRB, 0.49586E+00_JPRB, 0.48767E+00_JPRB, 0.47624E+00_JPRB, 0.46531E+00_JPRB ,&
&  0.45891E+00_JPRB, 0.45522E+00_JPRB, 0.45185E+00_JPRB, 0.44814E+00_JPRB, 0.44224E+00_JPRB, 0.43651E+00_JPRB, 0.43144E+00_JPRB ,&
&  0.42677E+00_JPRB, 0.42087E+00_JPRB, 0.41772E+00_JPRB, 0.41709E+00_JPRB, 0.41638E+00_JPRB, 0.41742E+00_JPRB, 0.41824E+00_JPRB ,&
&  0.41698E+00_JPRB, 0.41410E+00_JPRB, 0.41278E+00_JPRB, 0.41331E+00_JPRB, 0.41281E+00_JPRB, 0.41186E+00_JPRB, 0.41048E+00_JPRB ,&
&  0.41100E+00_JPRB, 0.41182E+00_JPRB, 0.41179E+00_JPRB, 0.41240E+00_JPRB, 0.41438E+00_JPRB, 0.41821E+00_JPRB, 0.42263E+00_JPRB ,&
&  0.42745E+00_JPRB, 0.43271E+00_JPRB, 0.43752E+00_JPRB, 0.44207E+00_JPRB, 0.44861E+00_JPRB, 0.45521E+00_JPRB, 0.46337E+00_JPRB ,&
&  0.47248E+00_JPRB, 0.48078E+00_JPRB, 0.48809E+00_JPRB, 0.49458E+00_JPRB, 0.49867E+00_JPRB, 0.49953E+00_JPRB, 0.49846E+00_JPRB ,&
&  0.49588E+00_JPRB, 0.48825E+00_JPRB, 0.48176E+00_JPRB, 0.47475E+00_JPRB, 0.46874E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.42567E+00_JPRB, 0.42129E+00_JPRB, 0.41637E+00_JPRB, 0.41255E+00_JPRB, 0.40915E+00_JPRB, 0.40580E+00_JPRB, 0.40461E+00_JPRB ,&
&  0.40537E+00_JPRB, 0.40756E+00_JPRB, 0.41079E+00_JPRB, 0.41273E+00_JPRB, 0.41368E+00_JPRB, 0.41475E+00_JPRB, 0.41459E+00_JPRB ,&
&  0.41371E+00_JPRB, 0.41208E+00_JPRB, 0.40979E+00_JPRB, 0.40664E+00_JPRB, 0.40295E+00_JPRB, 0.39946E+00_JPRB, 0.39492E+00_JPRB ,&
&  0.39160E+00_JPRB, 0.38946E+00_JPRB, 0.38807E+00_JPRB, 0.38736E+00_JPRB, 0.38795E+00_JPRB, 0.38886E+00_JPRB, 0.38961E+00_JPRB ,&
&  0.38830E+00_JPRB, 0.38016E+00_JPRB, 0.38594E+00_JPRB, 0.38832E+00_JPRB, 0.39602E+00_JPRB, 0.40980E+00_JPRB, 0.42238E+00_JPRB ,&
&  0.43116E+00_JPRB, 0.44287E+00_JPRB, 0.45534E+00_JPRB, 0.46347E+00_JPRB, 0.46317E+00_JPRB, 0.46156E+00_JPRB, 0.46425E+00_JPRB ,&
&  0.46602E+00_JPRB, 0.46510E+00_JPRB, 0.46450E+00_JPRB, 0.46294E+00_JPRB, 0.45981E+00_JPRB, 0.45599E+00_JPRB, 0.44987E+00_JPRB ,&
&  0.44362E+00_JPRB, 0.43872E+00_JPRB, 0.43361E+00_JPRB, 0.42834E+00_JPRB, 0.42093E+00_JPRB, 0.41239E+00_JPRB, 0.40267E+00_JPRB ,&
&  0.39251E+00_JPRB, 0.38520E+00_JPRB, 0.38055E+00_JPRB, 0.37675E+00_JPRB, 0.37362E+00_JPRB, 0.37243E+00_JPRB, 0.37100E+00_JPRB ,&
&  0.37049E+00_JPRB, 0.37096E+00_JPRB, 0.36994E+00_JPRB, 0.36947E+00_JPRB, 0.37060E+00_JPRB, 0.37140E+00_JPRB, 0.37343E+00_JPRB ,&
&  0.37408E+00_JPRB, 0.37449E+00_JPRB, 0.37492E+00_JPRB, 0.37479E+00_JPRB, 0.37413E+00_JPRB, 0.37552E+00_JPRB, 0.37829E+00_JPRB ,&
&  0.38129E+00_JPRB, 0.38576E+00_JPRB, 0.39058E+00_JPRB, 0.39463E+00_JPRB, 0.39844E+00_JPRB, 0.40450E+00_JPRB, 0.41227E+00_JPRB ,&
&  0.42109E+00_JPRB, 0.43059E+00_JPRB, 0.43919E+00_JPRB, 0.44582E+00_JPRB, 0.44798E+00_JPRB, 0.44825E+00_JPRB, 0.44801E+00_JPRB ,&
&  0.44584E+00_JPRB, 0.43974E+00_JPRB, 0.43611E+00_JPRB, 0.43319E+00_JPRB, 0.42954E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.37676E+00_JPRB, 0.37513E+00_JPRB, 0.37379E+00_JPRB, 0.37325E+00_JPRB, 0.37296E+00_JPRB, 0.37298E+00_JPRB, 0.37377E+00_JPRB ,&
&  0.37557E+00_JPRB, 0.37723E+00_JPRB, 0.38155E+00_JPRB, 0.38404E+00_JPRB, 0.38526E+00_JPRB, 0.38691E+00_JPRB, 0.38800E+00_JPRB ,&
&  0.38704E+00_JPRB, 0.38611E+00_JPRB, 0.38483E+00_JPRB, 0.38228E+00_JPRB, 0.37819E+00_JPRB, 0.37573E+00_JPRB, 0.37339E+00_JPRB ,&
&  0.36993E+00_JPRB, 0.36585E+00_JPRB, 0.36219E+00_JPRB, 0.35999E+00_JPRB, 0.35956E+00_JPRB, 0.35868E+00_JPRB, 0.35880E+00_JPRB ,&
&  0.35830E+00_JPRB, 0.36195E+00_JPRB, 0.35017E+00_JPRB, 0.35975E+00_JPRB, 0.36859E+00_JPRB, 0.37738E+00_JPRB, 0.38675E+00_JPRB ,&
&  0.39338E+00_JPRB, 0.40016E+00_JPRB, 0.40909E+00_JPRB, 0.41384E+00_JPRB, 0.41441E+00_JPRB, 0.41170E+00_JPRB, 0.41138E+00_JPRB ,&
&  0.41126E+00_JPRB, 0.40855E+00_JPRB, 0.40732E+00_JPRB, 0.40838E+00_JPRB, 0.40961E+00_JPRB, 0.40895E+00_JPRB, 0.40733E+00_JPRB ,&
&  0.40592E+00_JPRB, 0.40279E+00_JPRB, 0.39732E+00_JPRB, 0.39116E+00_JPRB, 0.38484E+00_JPRB, 0.37745E+00_JPRB, 0.36788E+00_JPRB ,&
&  0.35718E+00_JPRB, 0.34866E+00_JPRB, 0.34160E+00_JPRB, 0.33713E+00_JPRB, 0.33506E+00_JPRB, 0.33350E+00_JPRB, 0.33225E+00_JPRB ,&
&  0.33229E+00_JPRB, 0.33280E+00_JPRB, 0.33275E+00_JPRB, 0.33236E+00_JPRB, 0.33315E+00_JPRB, 0.33412E+00_JPRB, 0.33739E+00_JPRB ,&
&  0.34121E+00_JPRB, 0.34354E+00_JPRB, 0.34500E+00_JPRB, 0.34614E+00_JPRB, 0.34510E+00_JPRB, 0.34594E+00_JPRB, 0.34798E+00_JPRB ,&
&  0.35017E+00_JPRB, 0.35288E+00_JPRB, 0.35476E+00_JPRB, 0.35670E+00_JPRB, 0.35940E+00_JPRB, 0.36339E+00_JPRB, 0.36948E+00_JPRB ,&
&  0.37550E+00_JPRB, 0.38199E+00_JPRB, 0.38889E+00_JPRB, 0.39432E+00_JPRB, 0.39748E+00_JPRB, 0.39831E+00_JPRB, 0.39831E+00_JPRB ,&
&  0.39614E+00_JPRB, 0.39123E+00_JPRB, 0.38622E+00_JPRB, 0.38243E+00_JPRB, 0.37926E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.34159E+00_JPRB, 0.34198E+00_JPRB, 0.34330E+00_JPRB, 0.34381E+00_JPRB, 0.34384E+00_JPRB, 0.34481E+00_JPRB, 0.34803E+00_JPRB ,&
&  0.35107E+00_JPRB, 0.35334E+00_JPRB, 0.35562E+00_JPRB, 0.35722E+00_JPRB, 0.35830E+00_JPRB, 0.36063E+00_JPRB, 0.36245E+00_JPRB ,&
&  0.36302E+00_JPRB, 0.36211E+00_JPRB, 0.36084E+00_JPRB, 0.35906E+00_JPRB, 0.35653E+00_JPRB, 0.35369E+00_JPRB, 0.35122E+00_JPRB ,&
&  0.34803E+00_JPRB, 0.34443E+00_JPRB, 0.34082E+00_JPRB, 0.33891E+00_JPRB, 0.33812E+00_JPRB, 0.33763E+00_JPRB, 0.33800E+00_JPRB ,&
&  0.33838E+00_JPRB, 0.34066E+00_JPRB, 0.34342E+00_JPRB, 0.34685E+00_JPRB, 0.35026E+00_JPRB, 0.35480E+00_JPRB, 0.35982E+00_JPRB ,&
&  0.36458E+00_JPRB, 0.36718E+00_JPRB, 0.36964E+00_JPRB, 0.36878E+00_JPRB, 0.37021E+00_JPRB, 0.37116E+00_JPRB, 0.37159E+00_JPRB ,&
&  0.37054E+00_JPRB, 0.36505E+00_JPRB, 0.36086E+00_JPRB, 0.36088E+00_JPRB, 0.36238E+00_JPRB, 0.36369E+00_JPRB, 0.36465E+00_JPRB ,&
&  0.36425E+00_JPRB, 0.36284E+00_JPRB, 0.35936E+00_JPRB, 0.35449E+00_JPRB, 0.34924E+00_JPRB, 0.34314E+00_JPRB, 0.33477E+00_JPRB ,&
&  0.32704E+00_JPRB, 0.32076E+00_JPRB, 0.31577E+00_JPRB, 0.31137E+00_JPRB, 0.30841E+00_JPRB, 0.30538E+00_JPRB, 0.30482E+00_JPRB ,&
&  0.30567E+00_JPRB, 0.30795E+00_JPRB, 0.30910E+00_JPRB, 0.30823E+00_JPRB, 0.30697E+00_JPRB, 0.30709E+00_JPRB, 0.30959E+00_JPRB ,&
&  0.31417E+00_JPRB, 0.31790E+00_JPRB, 0.31948E+00_JPRB, 0.31975E+00_JPRB, 0.31942E+00_JPRB, 0.32086E+00_JPRB, 0.32341E+00_JPRB ,&
&  0.32483E+00_JPRB, 0.32557E+00_JPRB, 0.32664E+00_JPRB, 0.32642E+00_JPRB, 0.32830E+00_JPRB, 0.33159E+00_JPRB, 0.33534E+00_JPRB ,&
&  0.33899E+00_JPRB, 0.34356E+00_JPRB, 0.34807E+00_JPRB, 0.35140E+00_JPRB, 0.35415E+00_JPRB, 0.35392E+00_JPRB, 0.35370E+00_JPRB ,&
&  0.35316E+00_JPRB, 0.35119E+00_JPRB, 0.34919E+00_JPRB, 0.34656E+00_JPRB, 0.34346E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.31210E+00_JPRB, 0.31526E+00_JPRB, 0.31915E+00_JPRB, 0.31998E+00_JPRB, 0.32032E+00_JPRB, 0.32186E+00_JPRB, 0.32519E+00_JPRB ,&
&  0.32779E+00_JPRB, 0.32897E+00_JPRB, 0.32962E+00_JPRB, 0.33014E+00_JPRB, 0.33125E+00_JPRB, 0.33355E+00_JPRB, 0.33481E+00_JPRB ,&
&  0.33450E+00_JPRB, 0.33398E+00_JPRB, 0.33310E+00_JPRB, 0.33226E+00_JPRB, 0.33044E+00_JPRB, 0.32777E+00_JPRB, 0.32538E+00_JPRB ,&
&  0.32216E+00_JPRB, 0.31913E+00_JPRB, 0.31673E+00_JPRB, 0.31475E+00_JPRB, 0.31342E+00_JPRB, 0.31298E+00_JPRB, 0.31415E+00_JPRB ,&
&  0.31578E+00_JPRB, 0.31802E+00_JPRB, 0.32089E+00_JPRB, 0.32428E+00_JPRB, 0.32765E+00_JPRB, 0.33151E+00_JPRB, 0.33495E+00_JPRB ,&
&  0.33770E+00_JPRB, 0.34017E+00_JPRB, 0.34197E+00_JPRB, 0.33953E+00_JPRB, 0.33671E+00_JPRB, 0.33712E+00_JPRB, 0.33803E+00_JPRB ,&
&  0.33598E+00_JPRB, 0.33205E+00_JPRB, 0.32691E+00_JPRB, 0.32415E+00_JPRB, 0.32349E+00_JPRB, 0.32253E+00_JPRB, 0.32125E+00_JPRB ,&
&  0.32038E+00_JPRB, 0.31957E+00_JPRB, 0.31807E+00_JPRB, 0.31555E+00_JPRB, 0.31170E+00_JPRB, 0.30666E+00_JPRB, 0.30175E+00_JPRB ,&
&  0.29868E+00_JPRB, 0.29650E+00_JPRB, 0.29634E+00_JPRB, 0.29407E+00_JPRB, 0.28961E+00_JPRB, 0.28537E+00_JPRB, 0.28484E+00_JPRB ,&
&  0.28552E+00_JPRB, 0.28684E+00_JPRB, 0.28723E+00_JPRB, 0.28611E+00_JPRB, 0.28611E+00_JPRB, 0.28655E+00_JPRB, 0.28793E+00_JPRB ,&
&  0.29244E+00_JPRB, 0.29585E+00_JPRB, 0.29603E+00_JPRB, 0.29513E+00_JPRB, 0.29327E+00_JPRB, 0.29298E+00_JPRB, 0.29406E+00_JPRB ,&
&  0.29528E+00_JPRB, 0.29597E+00_JPRB, 0.29640E+00_JPRB, 0.29588E+00_JPRB, 0.29592E+00_JPRB, 0.29735E+00_JPRB, 0.29921E+00_JPRB ,&
&  0.30153E+00_JPRB, 0.30440E+00_JPRB, 0.30688E+00_JPRB, 0.30933E+00_JPRB, 0.31154E+00_JPRB, 0.31246E+00_JPRB, 0.31332E+00_JPRB ,&
&  0.31427E+00_JPRB, 0.31425E+00_JPRB, 0.31372E+00_JPRB, 0.31330E+00_JPRB, 0.31276E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.28949E+00_JPRB, 0.29398E+00_JPRB, 0.29964E+00_JPRB, 0.30086E+00_JPRB, 0.30301E+00_JPRB, 0.30538E+00_JPRB, 0.30889E+00_JPRB ,&
&  0.31149E+00_JPRB, 0.31220E+00_JPRB, 0.31117E+00_JPRB, 0.31003E+00_JPRB, 0.30960E+00_JPRB, 0.30976E+00_JPRB, 0.30958E+00_JPRB ,&
&  0.30959E+00_JPRB, 0.30982E+00_JPRB, 0.30973E+00_JPRB, 0.30938E+00_JPRB, 0.30787E+00_JPRB, 0.30565E+00_JPRB, 0.30314E+00_JPRB ,&
&  0.29925E+00_JPRB, 0.29653E+00_JPRB, 0.29540E+00_JPRB, 0.29476E+00_JPRB, 0.29366E+00_JPRB, 0.29391E+00_JPRB, 0.29553E+00_JPRB ,&
&  0.29921E+00_JPRB, 0.30210E+00_JPRB, 0.30581E+00_JPRB, 0.30954E+00_JPRB, 0.31167E+00_JPRB, 0.31324E+00_JPRB, 0.31509E+00_JPRB ,&
&  0.31625E+00_JPRB, 0.31653E+00_JPRB, 0.31664E+00_JPRB, 0.31668E+00_JPRB, 0.31418E+00_JPRB, 0.31409E+00_JPRB, 0.31486E+00_JPRB ,&
&  0.31454E+00_JPRB, 0.31195E+00_JPRB, 0.30809E+00_JPRB, 0.30482E+00_JPRB, 0.30154E+00_JPRB, 0.29908E+00_JPRB, 0.29701E+00_JPRB ,&
&  0.29542E+00_JPRB, 0.29454E+00_JPRB, 0.29427E+00_JPRB, 0.29249E+00_JPRB, 0.28701E+00_JPRB, 0.28171E+00_JPRB, 0.27880E+00_JPRB ,&
&  0.27747E+00_JPRB, 0.27714E+00_JPRB, 0.27853E+00_JPRB, 0.27718E+00_JPRB, 0.27300E+00_JPRB, 0.26958E+00_JPRB, 0.26839E+00_JPRB ,&
&  0.26718E+00_JPRB, 0.26685E+00_JPRB, 0.26791E+00_JPRB, 0.26849E+00_JPRB, 0.26881E+00_JPRB, 0.26864E+00_JPRB, 0.26977E+00_JPRB ,&
&  0.27132E+00_JPRB, 0.27328E+00_JPRB, 0.27332E+00_JPRB, 0.27056E+00_JPRB, 0.26670E+00_JPRB, 0.26405E+00_JPRB, 0.26401E+00_JPRB ,&
&  0.26616E+00_JPRB, 0.26819E+00_JPRB, 0.26903E+00_JPRB, 0.26947E+00_JPRB, 0.27066E+00_JPRB, 0.27318E+00_JPRB, 0.27639E+00_JPRB ,&
&  0.27781E+00_JPRB, 0.27916E+00_JPRB, 0.28106E+00_JPRB, 0.28300E+00_JPRB, 0.28420E+00_JPRB, 0.28478E+00_JPRB, 0.28482E+00_JPRB ,&
&  0.28412E+00_JPRB, 0.28474E+00_JPRB, 0.28622E+00_JPRB, 0.28801E+00_JPRB, 0.28913E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.27383E+00_JPRB, 0.27704E+00_JPRB, 0.28017E+00_JPRB, 0.28183E+00_JPRB, 0.28354E+00_JPRB, 0.28547E+00_JPRB, 0.28895E+00_JPRB ,&
&  0.29316E+00_JPRB, 0.29479E+00_JPRB, 0.29330E+00_JPRB, 0.29060E+00_JPRB, 0.28896E+00_JPRB, 0.28840E+00_JPRB, 0.28843E+00_JPRB ,&
&  0.28843E+00_JPRB, 0.28836E+00_JPRB, 0.28751E+00_JPRB, 0.28549E+00_JPRB, 0.28260E+00_JPRB, 0.27963E+00_JPRB, 0.27726E+00_JPRB ,&
&  0.27655E+00_JPRB, 0.27765E+00_JPRB, 0.27963E+00_JPRB, 0.27958E+00_JPRB, 0.27962E+00_JPRB, 0.28089E+00_JPRB, 0.28306E+00_JPRB ,&
&  0.28704E+00_JPRB, 0.28883E+00_JPRB, 0.29045E+00_JPRB, 0.28655E+00_JPRB, 0.29057E+00_JPRB, 0.29201E+00_JPRB, 0.29231E+00_JPRB ,&
&  0.29340E+00_JPRB, 0.29433E+00_JPRB, 0.29544E+00_JPRB, 0.29486E+00_JPRB, 0.29073E+00_JPRB, 0.28944E+00_JPRB, 0.29102E+00_JPRB ,&
&  0.29235E+00_JPRB, 0.29085E+00_JPRB, 0.28789E+00_JPRB, 0.28506E+00_JPRB, 0.28262E+00_JPRB, 0.28082E+00_JPRB, 0.27931E+00_JPRB ,&
&  0.27861E+00_JPRB, 0.27776E+00_JPRB, 0.27563E+00_JPRB, 0.27020E+00_JPRB, 0.26409E+00_JPRB, 0.26002E+00_JPRB, 0.25710E+00_JPRB ,&
&  0.25696E+00_JPRB, 0.25888E+00_JPRB, 0.26070E+00_JPRB, 0.26086E+00_JPRB, 0.25939E+00_JPRB, 0.25663E+00_JPRB, 0.25291E+00_JPRB ,&
&  0.24801E+00_JPRB, 0.24628E+00_JPRB, 0.24609E+00_JPRB, 0.24536E+00_JPRB, 0.24577E+00_JPRB, 0.24757E+00_JPRB, 0.24951E+00_JPRB ,&
&  0.25196E+00_JPRB, 0.25182E+00_JPRB, 0.25027E+00_JPRB, 0.24770E+00_JPRB, 0.24508E+00_JPRB, 0.24494E+00_JPRB, 0.24588E+00_JPRB ,&
&  0.24689E+00_JPRB, 0.24787E+00_JPRB, 0.24918E+00_JPRB, 0.25080E+00_JPRB, 0.25273E+00_JPRB, 0.25527E+00_JPRB, 0.25727E+00_JPRB ,&
&  0.25717E+00_JPRB, 0.25596E+00_JPRB, 0.25708E+00_JPRB, 0.25908E+00_JPRB, 0.26013E+00_JPRB, 0.26070E+00_JPRB, 0.26106E+00_JPRB ,&
&  0.26058E+00_JPRB, 0.26045E+00_JPRB, 0.26230E+00_JPRB, 0.26685E+00_JPRB, 0.27102E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.25974E+00_JPRB, 0.26115E+00_JPRB, 0.26136E+00_JPRB, 0.26185E+00_JPRB, 0.26253E+00_JPRB, 0.26445E+00_JPRB, 0.26872E+00_JPRB ,&
&  0.27298E+00_JPRB, 0.27499E+00_JPRB, 0.27341E+00_JPRB, 0.27107E+00_JPRB, 0.26987E+00_JPRB, 0.26935E+00_JPRB, 0.26928E+00_JPRB ,&
&  0.26920E+00_JPRB, 0.26820E+00_JPRB, 0.26522E+00_JPRB, 0.26106E+00_JPRB, 0.25781E+00_JPRB, 0.25668E+00_JPRB, 0.25675E+00_JPRB ,&
&  0.26028E+00_JPRB, 0.26267E+00_JPRB, 0.26490E+00_JPRB, 0.26708E+00_JPRB, 0.26780E+00_JPRB, 0.26765E+00_JPRB, 0.26926E+00_JPRB ,&
&  0.27013E+00_JPRB, 0.26722E+00_JPRB, 0.26168E+00_JPRB, 0.26531E+00_JPRB, 0.26755E+00_JPRB, 0.26888E+00_JPRB, 0.26811E+00_JPRB ,&
&  0.26870E+00_JPRB, 0.27084E+00_JPRB, 0.27308E+00_JPRB, 0.27480E+00_JPRB, 0.27409E+00_JPRB, 0.27131E+00_JPRB, 0.27075E+00_JPRB ,&
&  0.27134E+00_JPRB, 0.27033E+00_JPRB, 0.26747E+00_JPRB, 0.26481E+00_JPRB, 0.26184E+00_JPRB, 0.26046E+00_JPRB, 0.26030E+00_JPRB ,&
&  0.25956E+00_JPRB, 0.25744E+00_JPRB, 0.25315E+00_JPRB, 0.24886E+00_JPRB, 0.24566E+00_JPRB, 0.24238E+00_JPRB, 0.24072E+00_JPRB ,&
&  0.24095E+00_JPRB, 0.24205E+00_JPRB, 0.24205E+00_JPRB, 0.24035E+00_JPRB, 0.23467E+00_JPRB, 0.22533E+00_JPRB, 0.21809E+00_JPRB ,&
&  0.21679E+00_JPRB, 0.22115E+00_JPRB, 0.22876E+00_JPRB, 0.23108E+00_JPRB, 0.23375E+00_JPRB, 0.23537E+00_JPRB, 0.23650E+00_JPRB ,&
&  0.23416E+00_JPRB, 0.22162E+00_JPRB, 0.21607E+00_JPRB, 0.21264E+00_JPRB, 0.20953E+00_JPRB, 0.20946E+00_JPRB, 0.21133E+00_JPRB ,&
&  0.21505E+00_JPRB, 0.21932E+00_JPRB, 0.22253E+00_JPRB, 0.22452E+00_JPRB, 0.22522E+00_JPRB, 0.22523E+00_JPRB, 0.22456E+00_JPRB ,&
&  0.22500E+00_JPRB, 0.22918E+00_JPRB, 0.23726E+00_JPRB, 0.24456E+00_JPRB, 0.24563E+00_JPRB, 0.24717E+00_JPRB, 0.24730E+00_JPRB ,&
&  0.24749E+00_JPRB, 0.24947E+00_JPRB, 0.25063E+00_JPRB, 0.25335E+00_JPRB, 0.25686E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.24793E+00_JPRB, 0.24921E+00_JPRB, 0.24942E+00_JPRB, 0.24846E+00_JPRB, 0.24714E+00_JPRB, 0.24722E+00_JPRB, 0.24857E+00_JPRB ,&
&  0.25149E+00_JPRB, 0.25357E+00_JPRB, 0.25397E+00_JPRB, 0.25392E+00_JPRB, 0.25298E+00_JPRB, 0.25097E+00_JPRB, 0.24936E+00_JPRB ,&
&  0.24816E+00_JPRB, 0.24752E+00_JPRB, 0.24699E+00_JPRB, 0.24683E+00_JPRB, 0.24672E+00_JPRB, 0.24743E+00_JPRB, 0.24806E+00_JPRB ,&
&  0.25147E+00_JPRB, 0.25127E+00_JPRB, 0.25053E+00_JPRB, 0.25029E+00_JPRB, 0.25010E+00_JPRB, 0.25015E+00_JPRB, 0.24997E+00_JPRB ,&
&  0.24671E+00_JPRB, 0.23908E+00_JPRB, 0.23273E+00_JPRB, 0.23885E+00_JPRB, 0.24977E+00_JPRB, 0.25004E+00_JPRB, 0.25031E+00_JPRB ,&
&  0.25174E+00_JPRB, 0.25458E+00_JPRB, 0.25675E+00_JPRB, 0.25773E+00_JPRB, 0.25653E+00_JPRB, 0.25485E+00_JPRB, 0.25437E+00_JPRB ,&
&  0.25468E+00_JPRB, 0.25299E+00_JPRB, 0.24758E+00_JPRB, 0.24123E+00_JPRB, 0.23487E+00_JPRB, 0.23248E+00_JPRB, 0.23102E+00_JPRB ,&
&  0.22848E+00_JPRB, 0.22697E+00_JPRB, 0.22649E+00_JPRB, 0.22606E+00_JPRB, 0.22653E+00_JPRB, 0.22421E+00_JPRB, 0.22059E+00_JPRB ,&
&  0.21758E+00_JPRB, 0.21334E+00_JPRB, 0.20848E+00_JPRB, 0.20337E+00_JPRB, 0.19906E+00_JPRB, 0.19414E+00_JPRB, 0.19075E+00_JPRB ,&
&  0.19029E+00_JPRB, 0.19477E+00_JPRB, 0.20501E+00_JPRB, 0.21491E+00_JPRB, 0.22019E+00_JPRB, 0.21087E+00_JPRB, 0.20464E+00_JPRB ,&
&  0.19717E+00_JPRB, 0.19068E+00_JPRB, 0.18695E+00_JPRB, 0.18619E+00_JPRB, 0.18637E+00_JPRB, 0.18726E+00_JPRB, 0.18912E+00_JPRB ,&
&  0.19158E+00_JPRB, 0.19462E+00_JPRB, 0.19690E+00_JPRB, 0.19866E+00_JPRB, 0.19993E+00_JPRB, 0.20028E+00_JPRB, 0.20003E+00_JPRB ,&
&  0.19978E+00_JPRB, 0.20115E+00_JPRB, 0.20360E+00_JPRB, 0.20806E+00_JPRB, 0.21088E+00_JPRB, 0.21466E+00_JPRB, 0.21963E+00_JPRB ,&
&  0.22708E+00_JPRB, 0.23563E+00_JPRB, 0.23952E+00_JPRB, 0.24220E+00_JPRB, 0.24537E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.23544E+00_JPRB, 0.24017E+00_JPRB, 0.24250E+00_JPRB, 0.24185E+00_JPRB, 0.24132E+00_JPRB, 0.24172E+00_JPRB, 0.24226E+00_JPRB ,&
&  0.24315E+00_JPRB, 0.24252E+00_JPRB, 0.24092E+00_JPRB, 0.23944E+00_JPRB, 0.23789E+00_JPRB, 0.23594E+00_JPRB, 0.23475E+00_JPRB ,&
&  0.23388E+00_JPRB, 0.23415E+00_JPRB, 0.23423E+00_JPRB, 0.23546E+00_JPRB, 0.23616E+00_JPRB, 0.23820E+00_JPRB, 0.23637E+00_JPRB ,&
&  0.23245E+00_JPRB, 0.23011E+00_JPRB, 0.23241E+00_JPRB, 0.23266E+00_JPRB, 0.23316E+00_JPRB, 0.23382E+00_JPRB, 0.23523E+00_JPRB ,&
&  0.23247E+00_JPRB, 0.22965E+00_JPRB, 0.22674E+00_JPRB, 0.23305E+00_JPRB, 0.24061E+00_JPRB, 0.23931E+00_JPRB, 0.23897E+00_JPRB ,&
&  0.24067E+00_JPRB, 0.24332E+00_JPRB, 0.24459E+00_JPRB, 0.24469E+00_JPRB, 0.24320E+00_JPRB, 0.24255E+00_JPRB, 0.24130E+00_JPRB ,&
&  0.23823E+00_JPRB, 0.23552E+00_JPRB, 0.22603E+00_JPRB, 0.20922E+00_JPRB, 0.20328E+00_JPRB, 0.20073E+00_JPRB, 0.19687E+00_JPRB ,&
&  0.19163E+00_JPRB, 0.18967E+00_JPRB, 0.18965E+00_JPRB, 0.18763E+00_JPRB, 0.18775E+00_JPRB, 0.18741E+00_JPRB, 0.18618E+00_JPRB ,&
&  0.18465E+00_JPRB, 0.18365E+00_JPRB, 0.18214E+00_JPRB, 0.17963E+00_JPRB, 0.17773E+00_JPRB, 0.17743E+00_JPRB, 0.17840E+00_JPRB ,&
&  0.18052E+00_JPRB, 0.18353E+00_JPRB, 0.19236E+00_JPRB, 0.19746E+00_JPRB, 0.19555E+00_JPRB, 0.19204E+00_JPRB, 0.18546E+00_JPRB ,&
&  0.17918E+00_JPRB, 0.17728E+00_JPRB, 0.17593E+00_JPRB, 0.17478E+00_JPRB, 0.17516E+00_JPRB, 0.17628E+00_JPRB, 0.17741E+00_JPRB ,&
&  0.17883E+00_JPRB, 0.18056E+00_JPRB, 0.18162E+00_JPRB, 0.18223E+00_JPRB, 0.18279E+00_JPRB, 0.18307E+00_JPRB, 0.18359E+00_JPRB ,&
&  0.18363E+00_JPRB, 0.18505E+00_JPRB, 0.18617E+00_JPRB, 0.18862E+00_JPRB, 0.19025E+00_JPRB, 0.19248E+00_JPRB, 0.19735E+00_JPRB ,&
&  0.20115E+00_JPRB, 0.21264E+00_JPRB, 0.22790E+00_JPRB, 0.23426E+00_JPRB, 0.23093E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.22676E+00_JPRB, 0.22912E+00_JPRB, 0.23098E+00_JPRB, 0.22943E+00_JPRB, 0.22935E+00_JPRB, 0.22954E+00_JPRB, 0.23061E+00_JPRB ,&
&  0.23106E+00_JPRB, 0.23045E+00_JPRB, 0.23004E+00_JPRB, 0.22824E+00_JPRB, 0.22666E+00_JPRB, 0.22005E+00_JPRB, 0.21742E+00_JPRB ,&
&  0.21529E+00_JPRB, 0.21605E+00_JPRB, 0.21659E+00_JPRB, 0.21861E+00_JPRB, 0.21853E+00_JPRB, 0.21731E+00_JPRB, 0.21390E+00_JPRB ,&
&  0.21145E+00_JPRB, 0.21020E+00_JPRB, 0.20870E+00_JPRB, 0.20917E+00_JPRB, 0.21145E+00_JPRB, 0.21374E+00_JPRB, 0.21525E+00_JPRB ,&
&  0.21625E+00_JPRB, 0.21860E+00_JPRB, 0.22075E+00_JPRB, 0.22372E+00_JPRB, 0.23008E+00_JPRB, 0.23041E+00_JPRB, 0.23145E+00_JPRB ,&
&  0.23242E+00_JPRB, 0.23348E+00_JPRB, 0.23399E+00_JPRB, 0.23343E+00_JPRB, 0.23151E+00_JPRB, 0.22587E+00_JPRB, 0.22431E+00_JPRB ,&
&  0.21217E+00_JPRB, 0.20499E+00_JPRB, 0.19845E+00_JPRB, 0.19372E+00_JPRB, 0.18785E+00_JPRB, 0.18474E+00_JPRB, 0.18121E+00_JPRB ,&
&  0.17821E+00_JPRB, 0.17493E+00_JPRB, 0.17254E+00_JPRB, 0.17154E+00_JPRB, 0.17122E+00_JPRB, 0.17094E+00_JPRB, 0.17020E+00_JPRB ,&
&  0.17035E+00_JPRB, 0.16989E+00_JPRB, 0.16958E+00_JPRB, 0.16925E+00_JPRB, 0.16904E+00_JPRB, 0.16918E+00_JPRB, 0.17004E+00_JPRB ,&
&  0.17219E+00_JPRB, 0.17408E+00_JPRB, 0.17611E+00_JPRB, 0.17817E+00_JPRB, 0.17778E+00_JPRB, 0.17592E+00_JPRB, 0.17314E+00_JPRB ,&
&  0.17067E+00_JPRB, 0.16908E+00_JPRB, 0.16737E+00_JPRB, 0.16691E+00_JPRB, 0.16758E+00_JPRB, 0.16840E+00_JPRB, 0.16985E+00_JPRB ,&
&  0.17113E+00_JPRB, 0.17218E+00_JPRB, 0.17288E+00_JPRB, 0.17333E+00_JPRB, 0.17406E+00_JPRB, 0.17471E+00_JPRB, 0.17584E+00_JPRB ,&
&  0.17710E+00_JPRB, 0.17865E+00_JPRB, 0.18003E+00_JPRB, 0.18196E+00_JPRB, 0.18493E+00_JPRB, 0.18928E+00_JPRB, 0.19680E+00_JPRB ,&
&  0.20381E+00_JPRB, 0.22204E+00_JPRB, 0.22437E+00_JPRB, 0.22516E+00_JPRB, 0.22691E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.21708E+00_JPRB, 0.21900E+00_JPRB, 0.22007E+00_JPRB, 0.22117E+00_JPRB, 0.22115E+00_JPRB, 0.22126E+00_JPRB, 0.22189E+00_JPRB ,&
&  0.21786E+00_JPRB, 0.21512E+00_JPRB, 0.21045E+00_JPRB, 0.20623E+00_JPRB, 0.20219E+00_JPRB, 0.19888E+00_JPRB, 0.19687E+00_JPRB ,&
&  0.19577E+00_JPRB, 0.19588E+00_JPRB, 0.19586E+00_JPRB, 0.19631E+00_JPRB, 0.19521E+00_JPRB, 0.19412E+00_JPRB, 0.19315E+00_JPRB ,&
&  0.19211E+00_JPRB, 0.19282E+00_JPRB, 0.19360E+00_JPRB, 0.19714E+00_JPRB, 0.20211E+00_JPRB, 0.20760E+00_JPRB, 0.21167E+00_JPRB ,&
&  0.21445E+00_JPRB, 0.21661E+00_JPRB, 0.21944E+00_JPRB, 0.22066E+00_JPRB, 0.22201E+00_JPRB, 0.22338E+00_JPRB, 0.22379E+00_JPRB ,&
&  0.22507E+00_JPRB, 0.22490E+00_JPRB, 0.22461E+00_JPRB, 0.22458E+00_JPRB, 0.21578E+00_JPRB, 0.20946E+00_JPRB, 0.20295E+00_JPRB ,&
&  0.19681E+00_JPRB, 0.19054E+00_JPRB, 0.18487E+00_JPRB, 0.18179E+00_JPRB, 0.17827E+00_JPRB, 0.17624E+00_JPRB, 0.17447E+00_JPRB ,&
&  0.17313E+00_JPRB, 0.17174E+00_JPRB, 0.16976E+00_JPRB, 0.16780E+00_JPRB, 0.16665E+00_JPRB, 0.16569E+00_JPRB, 0.16447E+00_JPRB ,&
&  0.16439E+00_JPRB, 0.16373E+00_JPRB, 0.16374E+00_JPRB, 0.16340E+00_JPRB, 0.16316E+00_JPRB, 0.16296E+00_JPRB, 0.16299E+00_JPRB ,&
&  0.16344E+00_JPRB, 0.16434E+00_JPRB, 0.16473E+00_JPRB, 0.16524E+00_JPRB, 0.16519E+00_JPRB, 0.16452E+00_JPRB, 0.16398E+00_JPRB ,&
&  0.16336E+00_JPRB, 0.16288E+00_JPRB, 0.16292E+00_JPRB, 0.16282E+00_JPRB, 0.16337E+00_JPRB, 0.16410E+00_JPRB, 0.16479E+00_JPRB ,&
&  0.16579E+00_JPRB, 0.16650E+00_JPRB, 0.16724E+00_JPRB, 0.16821E+00_JPRB, 0.16901E+00_JPRB, 0.17005E+00_JPRB, 0.17124E+00_JPRB ,&
&  0.17211E+00_JPRB, 0.17392E+00_JPRB, 0.17543E+00_JPRB, 0.17735E+00_JPRB, 0.18110E+00_JPRB, 0.18481E+00_JPRB, 0.19374E+00_JPRB ,&
&  0.20167E+00_JPRB, 0.21782E+00_JPRB, 0.21898E+00_JPRB, 0.21489E+00_JPRB, 0.21579E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.21125E+00_JPRB, 0.21264E+00_JPRB, 0.21313E+00_JPRB, 0.21399E+00_JPRB, 0.21423E+00_JPRB, 0.21562E+00_JPRB, 0.21557E+00_JPRB ,&
&  0.21566E+00_JPRB, 0.21191E+00_JPRB, 0.20964E+00_JPRB, 0.20692E+00_JPRB, 0.20454E+00_JPRB, 0.20168E+00_JPRB, 0.19891E+00_JPRB ,&
&  0.19676E+00_JPRB, 0.19483E+00_JPRB, 0.19274E+00_JPRB, 0.18990E+00_JPRB, 0.18704E+00_JPRB, 0.18504E+00_JPRB, 0.18371E+00_JPRB ,&
&  0.18245E+00_JPRB, 0.18350E+00_JPRB, 0.18538E+00_JPRB, 0.18895E+00_JPRB, 0.19364E+00_JPRB, 0.19741E+00_JPRB, 0.20246E+00_JPRB ,&
&  0.20840E+00_JPRB, 0.21021E+00_JPRB, 0.21296E+00_JPRB, 0.21413E+00_JPRB, 0.21380E+00_JPRB, 0.21445E+00_JPRB, 0.21239E+00_JPRB ,&
&  0.21251E+00_JPRB, 0.20932E+00_JPRB, 0.21379E+00_JPRB, 0.21024E+00_JPRB, 0.20317E+00_JPRB, 0.20016E+00_JPRB, 0.19390E+00_JPRB ,&
&  0.18961E+00_JPRB, 0.18482E+00_JPRB, 0.18171E+00_JPRB, 0.17997E+00_JPRB, 0.17662E+00_JPRB, 0.17484E+00_JPRB, 0.17204E+00_JPRB ,&
&  0.16946E+00_JPRB, 0.16866E+00_JPRB, 0.16710E+00_JPRB, 0.16651E+00_JPRB, 0.16568E+00_JPRB, 0.16449E+00_JPRB, 0.16319E+00_JPRB ,&
&  0.16234E+00_JPRB, 0.16163E+00_JPRB, 0.16129E+00_JPRB, 0.16077E+00_JPRB, 0.16003E+00_JPRB, 0.15927E+00_JPRB, 0.15839E+00_JPRB ,&
&  0.15783E+00_JPRB, 0.15759E+00_JPRB, 0.15742E+00_JPRB, 0.15746E+00_JPRB, 0.15748E+00_JPRB, 0.15730E+00_JPRB, 0.15749E+00_JPRB ,&
&  0.15780E+00_JPRB, 0.15847E+00_JPRB, 0.15900E+00_JPRB, 0.15972E+00_JPRB, 0.16034E+00_JPRB, 0.16117E+00_JPRB, 0.16174E+00_JPRB ,&
&  0.16241E+00_JPRB, 0.16339E+00_JPRB, 0.16425E+00_JPRB, 0.16527E+00_JPRB, 0.16636E+00_JPRB, 0.16727E+00_JPRB, 0.16875E+00_JPRB ,&
&  0.17027E+00_JPRB, 0.17176E+00_JPRB, 0.17390E+00_JPRB, 0.17605E+00_JPRB, 0.17880E+00_JPRB, 0.18335E+00_JPRB, 0.18849E+00_JPRB ,&
&  0.19493E+00_JPRB, 0.20434E+00_JPRB, 0.20465E+00_JPRB, 0.20805E+00_JPRB, 0.20984E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.21038E+00_JPRB, 0.21166E+00_JPRB, 0.21273E+00_JPRB, 0.21325E+00_JPRB, 0.21390E+00_JPRB, 0.21391E+00_JPRB, 0.21376E+00_JPRB ,&
&  0.21077E+00_JPRB, 0.20953E+00_JPRB, 0.20766E+00_JPRB, 0.20554E+00_JPRB, 0.20368E+00_JPRB, 0.20100E+00_JPRB, 0.19856E+00_JPRB ,&
&  0.19630E+00_JPRB, 0.19370E+00_JPRB, 0.19116E+00_JPRB, 0.18814E+00_JPRB, 0.18532E+00_JPRB, 0.18268E+00_JPRB, 0.18141E+00_JPRB ,&
&  0.18087E+00_JPRB, 0.18142E+00_JPRB, 0.18329E+00_JPRB, 0.18471E+00_JPRB, 0.18671E+00_JPRB, 0.18922E+00_JPRB, 0.19308E+00_JPRB ,&
&  0.19588E+00_JPRB, 0.19943E+00_JPRB, 0.20171E+00_JPRB, 0.20234E+00_JPRB, 0.20166E+00_JPRB, 0.20131E+00_JPRB, 0.20039E+00_JPRB ,&
&  0.19839E+00_JPRB, 0.19633E+00_JPRB, 0.19459E+00_JPRB, 0.19269E+00_JPRB, 0.19066E+00_JPRB, 0.18827E+00_JPRB, 0.18525E+00_JPRB ,&
&  0.18255E+00_JPRB, 0.18018E+00_JPRB, 0.17790E+00_JPRB, 0.17610E+00_JPRB, 0.17432E+00_JPRB, 0.17270E+00_JPRB, 0.17089E+00_JPRB ,&
&  0.16949E+00_JPRB, 0.16829E+00_JPRB, 0.16677E+00_JPRB, 0.16557E+00_JPRB, 0.16419E+00_JPRB, 0.16304E+00_JPRB, 0.16180E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16008E+00_JPRB, 0.15944E+00_JPRB, 0.15883E+00_JPRB, 0.15849E+00_JPRB, 0.15817E+00_JPRB, 0.15778E+00_JPRB ,&
&  0.15747E+00_JPRB, 0.15753E+00_JPRB, 0.15762E+00_JPRB, 0.15772E+00_JPRB, 0.15787E+00_JPRB, 0.15757E+00_JPRB, 0.15765E+00_JPRB ,&
&  0.15797E+00_JPRB, 0.15830E+00_JPRB, 0.15833E+00_JPRB, 0.15874E+00_JPRB, 0.15943E+00_JPRB, 0.16026E+00_JPRB, 0.16152E+00_JPRB ,&
&  0.16251E+00_JPRB, 0.16352E+00_JPRB, 0.16448E+00_JPRB, 0.16547E+00_JPRB, 0.16676E+00_JPRB, 0.16815E+00_JPRB, 0.16960E+00_JPRB ,&
&  0.17131E+00_JPRB, 0.17290E+00_JPRB, 0.17456E+00_JPRB, 0.17627E+00_JPRB, 0.17901E+00_JPRB, 0.18158E+00_JPRB, 0.18554E+00_JPRB ,&
&  0.18953E+00_JPRB, 0.19114E+00_JPRB, 0.20485E+00_JPRB, 0.20589E+00_JPRB, 0.20750E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.18892E+00_JPRB, 0.19114E+00_JPRB, 0.19254E+00_JPRB, 0.19400E+00_JPRB, 0.19126E+00_JPRB, 0.20300E+00_JPRB, 0.20083E+00_JPRB ,&
&  0.20044E+00_JPRB, 0.19561E+00_JPRB, 0.19620E+00_JPRB, 0.19557E+00_JPRB, 0.19418E+00_JPRB, 0.19259E+00_JPRB, 0.19079E+00_JPRB ,&
&  0.18901E+00_JPRB, 0.18674E+00_JPRB, 0.18493E+00_JPRB, 0.18342E+00_JPRB, 0.18203E+00_JPRB, 0.18084E+00_JPRB, 0.17964E+00_JPRB ,&
&  0.17957E+00_JPRB, 0.18032E+00_JPRB, 0.18149E+00_JPRB, 0.18071E+00_JPRB, 0.18108E+00_JPRB, 0.18195E+00_JPRB, 0.18315E+00_JPRB ,&
&  0.18413E+00_JPRB, 0.18555E+00_JPRB, 0.18648E+00_JPRB, 0.18676E+00_JPRB, 0.18732E+00_JPRB, 0.18661E+00_JPRB, 0.18611E+00_JPRB ,&
&  0.18536E+00_JPRB, 0.18431E+00_JPRB, 0.18299E+00_JPRB, 0.18185E+00_JPRB, 0.18059E+00_JPRB, 0.17900E+00_JPRB, 0.17736E+00_JPRB ,&
&  0.17589E+00_JPRB, 0.17459E+00_JPRB, 0.17317E+00_JPRB, 0.17201E+00_JPRB, 0.17089E+00_JPRB, 0.16978E+00_JPRB, 0.16874E+00_JPRB ,&
&  0.16791E+00_JPRB, 0.16717E+00_JPRB, 0.16637E+00_JPRB, 0.16555E+00_JPRB, 0.16466E+00_JPRB, 0.16381E+00_JPRB, 0.16299E+00_JPRB ,&
&  0.16264E+00_JPRB, 0.16233E+00_JPRB, 0.16201E+00_JPRB, 0.16167E+00_JPRB, 0.16161E+00_JPRB, 0.16152E+00_JPRB, 0.16138E+00_JPRB ,&
&  0.16123E+00_JPRB, 0.16098E+00_JPRB, 0.16090E+00_JPRB, 0.16082E+00_JPRB, 0.16074E+00_JPRB, 0.16068E+00_JPRB, 0.16079E+00_JPRB ,&
&  0.16098E+00_JPRB, 0.16124E+00_JPRB, 0.16176E+00_JPRB, 0.16232E+00_JPRB, 0.16291E+00_JPRB, 0.16347E+00_JPRB, 0.16428E+00_JPRB ,&
&  0.16517E+00_JPRB, 0.16602E+00_JPRB, 0.16674E+00_JPRB, 0.16734E+00_JPRB, 0.16835E+00_JPRB, 0.16929E+00_JPRB, 0.17023E+00_JPRB ,&
&  0.17125E+00_JPRB, 0.17255E+00_JPRB, 0.17390E+00_JPRB, 0.17530E+00_JPRB, 0.17693E+00_JPRB, 0.17827E+00_JPRB, 0.17963E+00_JPRB ,&
&  0.18124E+00_JPRB, 0.18424E+00_JPRB, 0.18618E+00_JPRB, 0.18742E+00_JPRB, 0.18871E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.17404E+00_JPRB, 0.17419E+00_JPRB, 0.17423E+00_JPRB, 0.17420E+00_JPRB, 0.17395E+00_JPRB, 0.17389E+00_JPRB, 0.17383E+00_JPRB ,&
&  0.17378E+00_JPRB, 0.17357E+00_JPRB, 0.17377E+00_JPRB, 0.17386E+00_JPRB, 0.17389E+00_JPRB, 0.17388E+00_JPRB, 0.17373E+00_JPRB ,&
&  0.17340E+00_JPRB, 0.17299E+00_JPRB, 0.17194E+00_JPRB, 0.17175E+00_JPRB, 0.17175E+00_JPRB, 0.17196E+00_JPRB, 0.17210E+00_JPRB ,&
&  0.17236E+00_JPRB, 0.17261E+00_JPRB, 0.17283E+00_JPRB, 0.17349E+00_JPRB, 0.17377E+00_JPRB, 0.17409E+00_JPRB, 0.17440E+00_JPRB ,&
&  0.17470E+00_JPRB, 0.17505E+00_JPRB, 0.17532E+00_JPRB, 0.17560E+00_JPRB, 0.17587E+00_JPRB, 0.17581E+00_JPRB, 0.17559E+00_JPRB ,&
&  0.17524E+00_JPRB, 0.17470E+00_JPRB, 0.17413E+00_JPRB, 0.17361E+00_JPRB, 0.17314E+00_JPRB, 0.17265E+00_JPRB, 0.17218E+00_JPRB ,&
&  0.17168E+00_JPRB, 0.17112E+00_JPRB, 0.17053E+00_JPRB, 0.17000E+00_JPRB, 0.16946E+00_JPRB, 0.16898E+00_JPRB, 0.16844E+00_JPRB ,&
&  0.16802E+00_JPRB, 0.16760E+00_JPRB, 0.16719E+00_JPRB, 0.16677E+00_JPRB, 0.16641E+00_JPRB, 0.16605E+00_JPRB, 0.16568E+00_JPRB ,&
&  0.16531E+00_JPRB, 0.16512E+00_JPRB, 0.16486E+00_JPRB, 0.16463E+00_JPRB, 0.16442E+00_JPRB, 0.16427E+00_JPRB, 0.16406E+00_JPRB ,&
&  0.16386E+00_JPRB, 0.16367E+00_JPRB, 0.16360E+00_JPRB, 0.16347E+00_JPRB, 0.16343E+00_JPRB, 0.16341E+00_JPRB, 0.16348E+00_JPRB ,&
&  0.16354E+00_JPRB, 0.16364E+00_JPRB, 0.16382E+00_JPRB, 0.16411E+00_JPRB, 0.16437E+00_JPRB, 0.16466E+00_JPRB, 0.16495E+00_JPRB ,&
&  0.16527E+00_JPRB, 0.16560E+00_JPRB, 0.16595E+00_JPRB, 0.16628E+00_JPRB, 0.16670E+00_JPRB, 0.16712E+00_JPRB, 0.16760E+00_JPRB ,&
&  0.16804E+00_JPRB, 0.16852E+00_JPRB, 0.16894E+00_JPRB, 0.16943E+00_JPRB, 0.16984E+00_JPRB, 0.17010E+00_JPRB, 0.17034E+00_JPRB ,&
&  0.17076E+00_JPRB, 0.17116E+00_JPRB, 0.17165E+00_JPRB, 0.17208E+00_JPRB, 0.17285E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB, 0.16706E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB, 0.51019E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.50356E+00_JPRB, 0.50367E+00_JPRB, 0.50374E+00_JPRB, 0.50381E+00_JPRB, 0.50387E+00_JPRB, 0.50393E+00_JPRB, 0.50401E+00_JPRB ,&
&  0.50403E+00_JPRB, 0.50421E+00_JPRB, 0.50446E+00_JPRB, 0.50477E+00_JPRB, 0.50505E+00_JPRB, 0.50532E+00_JPRB, 0.50558E+00_JPRB ,&
&  0.50581E+00_JPRB, 0.50594E+00_JPRB, 0.50613E+00_JPRB, 0.50650E+00_JPRB, 0.50701E+00_JPRB, 0.50763E+00_JPRB, 0.50848E+00_JPRB ,&
&  0.50882E+00_JPRB, 0.50875E+00_JPRB, 0.50871E+00_JPRB, 0.50917E+00_JPRB, 0.50909E+00_JPRB, 0.50908E+00_JPRB, 0.50913E+00_JPRB ,&
&  0.50915E+00_JPRB, 0.50910E+00_JPRB, 0.50894E+00_JPRB, 0.50876E+00_JPRB, 0.50871E+00_JPRB, 0.50884E+00_JPRB, 0.50903E+00_JPRB ,&
&  0.50930E+00_JPRB, 0.50954E+00_JPRB, 0.51012E+00_JPRB, 0.51076E+00_JPRB, 0.51150E+00_JPRB, 0.51202E+00_JPRB, 0.51259E+00_JPRB ,&
&  0.51296E+00_JPRB, 0.51337E+00_JPRB, 0.51399E+00_JPRB, 0.51486E+00_JPRB, 0.51578E+00_JPRB, 0.51666E+00_JPRB, 0.51735E+00_JPRB ,&
&  0.51837E+00_JPRB, 0.51969E+00_JPRB, 0.52110E+00_JPRB, 0.52263E+00_JPRB, 0.52388E+00_JPRB, 0.52515E+00_JPRB, 0.52651E+00_JPRB ,&
&  0.52768E+00_JPRB, 0.52837E+00_JPRB, 0.52919E+00_JPRB, 0.53005E+00_JPRB, 0.53067E+00_JPRB, 0.53027E+00_JPRB, 0.52999E+00_JPRB ,&
&  0.52974E+00_JPRB, 0.52922E+00_JPRB, 0.52771E+00_JPRB, 0.52643E+00_JPRB, 0.52525E+00_JPRB, 0.52400E+00_JPRB, 0.52261E+00_JPRB ,&
&  0.52136E+00_JPRB, 0.52024E+00_JPRB, 0.51901E+00_JPRB, 0.51771E+00_JPRB, 0.51654E+00_JPRB, 0.51541E+00_JPRB, 0.51432E+00_JPRB ,&
&  0.51332E+00_JPRB, 0.51245E+00_JPRB, 0.51163E+00_JPRB, 0.51086E+00_JPRB, 0.51000E+00_JPRB, 0.50917E+00_JPRB, 0.50834E+00_JPRB ,&
&  0.50760E+00_JPRB, 0.50683E+00_JPRB, 0.50612E+00_JPRB, 0.50543E+00_JPRB, 0.50494E+00_JPRB, 0.50459E+00_JPRB, 0.50428E+00_JPRB ,&
&  0.50401E+00_JPRB, 0.50384E+00_JPRB, 0.50376E+00_JPRB, 0.50365E+00_JPRB, 0.50355E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.50083E+00_JPRB, 0.50123E+00_JPRB, 0.50159E+00_JPRB, 0.50188E+00_JPRB, 0.50228E+00_JPRB, 0.50275E+00_JPRB, 0.50314E+00_JPRB ,&
&  0.50343E+00_JPRB, 0.50396E+00_JPRB, 0.50457E+00_JPRB, 0.50519E+00_JPRB, 0.50577E+00_JPRB, 0.50641E+00_JPRB, 0.50700E+00_JPRB ,&
&  0.50761E+00_JPRB, 0.50819E+00_JPRB, 0.50880E+00_JPRB, 0.50936E+00_JPRB, 0.50994E+00_JPRB, 0.51035E+00_JPRB, 0.51194E+00_JPRB ,&
&  0.51330E+00_JPRB, 0.51395E+00_JPRB, 0.51346E+00_JPRB, 0.50936E+00_JPRB, 0.50639E+00_JPRB, 0.50373E+00_JPRB, 0.50154E+00_JPRB ,&
&  0.49927E+00_JPRB, 0.49936E+00_JPRB, 0.49981E+00_JPRB, 0.50029E+00_JPRB, 0.50239E+00_JPRB, 0.50207E+00_JPRB, 0.50157E+00_JPRB ,&
&  0.50022E+00_JPRB, 0.49672E+00_JPRB, 0.49721E+00_JPRB, 0.49783E+00_JPRB, 0.49850E+00_JPRB, 0.50039E+00_JPRB, 0.50181E+00_JPRB ,&
&  0.50354E+00_JPRB, 0.50547E+00_JPRB, 0.50904E+00_JPRB, 0.51305E+00_JPRB, 0.51668E+00_JPRB, 0.51971E+00_JPRB, 0.52263E+00_JPRB ,&
&  0.52519E+00_JPRB, 0.52780E+00_JPRB, 0.53063E+00_JPRB, 0.53393E+00_JPRB, 0.53649E+00_JPRB, 0.53931E+00_JPRB, 0.54163E+00_JPRB ,&
&  0.54300E+00_JPRB, 0.54361E+00_JPRB, 0.54532E+00_JPRB, 0.54780E+00_JPRB, 0.55037E+00_JPRB, 0.55075E+00_JPRB, 0.55020E+00_JPRB ,&
&  0.54883E+00_JPRB, 0.54736E+00_JPRB, 0.54476E+00_JPRB, 0.54234E+00_JPRB, 0.53972E+00_JPRB, 0.53704E+00_JPRB, 0.53385E+00_JPRB ,&
&  0.53065E+00_JPRB, 0.52757E+00_JPRB, 0.52519E+00_JPRB, 0.52300E+00_JPRB, 0.52083E+00_JPRB, 0.51880E+00_JPRB, 0.51747E+00_JPRB ,&
&  0.51610E+00_JPRB, 0.51491E+00_JPRB, 0.51366E+00_JPRB, 0.51242E+00_JPRB, 0.51043E+00_JPRB, 0.50849E+00_JPRB, 0.50644E+00_JPRB ,&
&  0.50477E+00_JPRB, 0.50312E+00_JPRB, 0.50177E+00_JPRB, 0.50063E+00_JPRB, 0.50010E+00_JPRB, 0.49981E+00_JPRB, 0.49966E+00_JPRB ,&
&  0.49956E+00_JPRB, 0.49976E+00_JPRB, 0.50005E+00_JPRB, 0.50027E+00_JPRB, 0.50051E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.50462E+00_JPRB, 0.50536E+00_JPRB, 0.50612E+00_JPRB, 0.50656E+00_JPRB, 0.50746E+00_JPRB, 0.50825E+00_JPRB, 0.50902E+00_JPRB ,&
&  0.50975E+00_JPRB, 0.51024E+00_JPRB, 0.51064E+00_JPRB, 0.51108E+00_JPRB, 0.51141E+00_JPRB, 0.51227E+00_JPRB, 0.51305E+00_JPRB ,&
&  0.51374E+00_JPRB, 0.51415E+00_JPRB, 0.51541E+00_JPRB, 0.51660E+00_JPRB, 0.51724E+00_JPRB, 0.51782E+00_JPRB, 0.51450E+00_JPRB ,&
&  0.51243E+00_JPRB, 0.51114E+00_JPRB, 0.50947E+00_JPRB, 0.50552E+00_JPRB, 0.50359E+00_JPRB, 0.49130E+00_JPRB, 0.49507E+00_JPRB ,&
&  0.51008E+00_JPRB, 0.50917E+00_JPRB, 0.50599E+00_JPRB, 0.50053E+00_JPRB, 0.48107E+00_JPRB, 0.47249E+00_JPRB, 0.47099E+00_JPRB ,&
&  0.45873E+00_JPRB, 0.46959E+00_JPRB, 0.47263E+00_JPRB, 0.47394E+00_JPRB, 0.47668E+00_JPRB, 0.47488E+00_JPRB, 0.47534E+00_JPRB ,&
&  0.48358E+00_JPRB, 0.48400E+00_JPRB, 0.48847E+00_JPRB, 0.49396E+00_JPRB, 0.49992E+00_JPRB, 0.50552E+00_JPRB, 0.51190E+00_JPRB ,&
&  0.51706E+00_JPRB, 0.52083E+00_JPRB, 0.52338E+00_JPRB, 0.52432E+00_JPRB, 0.52556E+00_JPRB, 0.52703E+00_JPRB, 0.53016E+00_JPRB ,&
&  0.53525E+00_JPRB, 0.54077E+00_JPRB, 0.54373E+00_JPRB, 0.54306E+00_JPRB, 0.54406E+00_JPRB, 0.54477E+00_JPRB, 0.54544E+00_JPRB ,&
&  0.54534E+00_JPRB, 0.54464E+00_JPRB, 0.54212E+00_JPRB, 0.54150E+00_JPRB, 0.54107E+00_JPRB, 0.53966E+00_JPRB, 0.53745E+00_JPRB ,&
&  0.53447E+00_JPRB, 0.53033E+00_JPRB, 0.52691E+00_JPRB, 0.52469E+00_JPRB, 0.52256E+00_JPRB, 0.52078E+00_JPRB, 0.52006E+00_JPRB ,&
&  0.51862E+00_JPRB, 0.51749E+00_JPRB, 0.51543E+00_JPRB, 0.51259E+00_JPRB, 0.50960E+00_JPRB, 0.50718E+00_JPRB, 0.50462E+00_JPRB ,&
&  0.50319E+00_JPRB, 0.50234E+00_JPRB, 0.50186E+00_JPRB, 0.50102E+00_JPRB, 0.50091E+00_JPRB, 0.50097E+00_JPRB, 0.50137E+00_JPRB ,&
&  0.50181E+00_JPRB, 0.50246E+00_JPRB, 0.50289E+00_JPRB, 0.50347E+00_JPRB, 0.50375E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.50605E+00_JPRB, 0.50620E+00_JPRB, 0.50660E+00_JPRB, 0.50691E+00_JPRB, 0.50773E+00_JPRB, 0.50840E+00_JPRB, 0.51011E+00_JPRB ,&
&  0.51175E+00_JPRB, 0.51333E+00_JPRB, 0.51484E+00_JPRB, 0.51607E+00_JPRB, 0.51729E+00_JPRB, 0.51778E+00_JPRB, 0.51796E+00_JPRB ,&
&  0.51804E+00_JPRB, 0.51789E+00_JPRB, 0.51854E+00_JPRB, 0.51946E+00_JPRB, 0.51967E+00_JPRB, 0.52041E+00_JPRB, 0.51686E+00_JPRB ,&
&  0.51422E+00_JPRB, 0.51256E+00_JPRB, 0.49989E+00_JPRB, 0.50603E+00_JPRB, 0.50470E+00_JPRB, 0.50543E+00_JPRB, 0.50334E+00_JPRB ,&
&  0.48974E+00_JPRB, 0.48387E+00_JPRB, 0.46897E+00_JPRB, 0.45642E+00_JPRB, 0.45719E+00_JPRB, 0.44829E+00_JPRB, 0.44906E+00_JPRB ,&
&  0.44715E+00_JPRB, 0.44369E+00_JPRB, 0.44222E+00_JPRB, 0.44129E+00_JPRB, 0.44361E+00_JPRB, 0.44549E+00_JPRB, 0.44767E+00_JPRB ,&
&  0.45308E+00_JPRB, 0.45783E+00_JPRB, 0.46496E+00_JPRB, 0.46606E+00_JPRB, 0.47438E+00_JPRB, 0.48367E+00_JPRB, 0.49329E+00_JPRB ,&
&  0.49919E+00_JPRB, 0.49785E+00_JPRB, 0.50076E+00_JPRB, 0.50565E+00_JPRB, 0.50968E+00_JPRB, 0.51387E+00_JPRB, 0.52019E+00_JPRB ,&
&  0.52827E+00_JPRB, 0.53598E+00_JPRB, 0.53876E+00_JPRB, 0.53753E+00_JPRB, 0.53515E+00_JPRB, 0.53310E+00_JPRB, 0.53294E+00_JPRB ,&
&  0.53274E+00_JPRB, 0.53470E+00_JPRB, 0.53524E+00_JPRB, 0.53730E+00_JPRB, 0.53864E+00_JPRB, 0.54164E+00_JPRB, 0.54371E+00_JPRB ,&
&  0.54467E+00_JPRB, 0.54428E+00_JPRB, 0.54527E+00_JPRB, 0.54089E+00_JPRB, 0.53976E+00_JPRB, 0.53824E+00_JPRB, 0.53356E+00_JPRB ,&
&  0.52909E+00_JPRB, 0.52550E+00_JPRB, 0.52182E+00_JPRB, 0.51848E+00_JPRB, 0.51568E+00_JPRB, 0.51369E+00_JPRB, 0.51236E+00_JPRB ,&
&  0.51170E+00_JPRB, 0.51070E+00_JPRB, 0.50969E+00_JPRB, 0.50725E+00_JPRB, 0.50669E+00_JPRB, 0.50506E+00_JPRB, 0.50415E+00_JPRB ,&
&  0.50353E+00_JPRB, 0.50406E+00_JPRB, 0.50406E+00_JPRB, 0.50488E+00_JPRB, 0.50576E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.50242E+00_JPRB, 0.50473E+00_JPRB, 0.50879E+00_JPRB, 0.51230E+00_JPRB, 0.51555E+00_JPRB, 0.51818E+00_JPRB, 0.51958E+00_JPRB ,&
&  0.52061E+00_JPRB, 0.52093E+00_JPRB, 0.52131E+00_JPRB, 0.52189E+00_JPRB, 0.52290E+00_JPRB, 0.52367E+00_JPRB, 0.52471E+00_JPRB ,&
&  0.52501E+00_JPRB, 0.52507E+00_JPRB, 0.52485E+00_JPRB, 0.52465E+00_JPRB, 0.52467E+00_JPRB, 0.52560E+00_JPRB, 0.52477E+00_JPRB ,&
&  0.52500E+00_JPRB, 0.52171E+00_JPRB, 0.51232E+00_JPRB, 0.51517E+00_JPRB, 0.51588E+00_JPRB, 0.51680E+00_JPRB, 0.51142E+00_JPRB ,&
&  0.49728E+00_JPRB, 0.47044E+00_JPRB, 0.48402E+00_JPRB, 0.47413E+00_JPRB, 0.46382E+00_JPRB, 0.45329E+00_JPRB, 0.43822E+00_JPRB ,&
&  0.42906E+00_JPRB, 0.41732E+00_JPRB, 0.40863E+00_JPRB, 0.40567E+00_JPRB, 0.40545E+00_JPRB, 0.41009E+00_JPRB, 0.41650E+00_JPRB ,&
&  0.41975E+00_JPRB, 0.42902E+00_JPRB, 0.43442E+00_JPRB, 0.44139E+00_JPRB, 0.44983E+00_JPRB, 0.46064E+00_JPRB, 0.46487E+00_JPRB ,&
&  0.47245E+00_JPRB, 0.47572E+00_JPRB, 0.48358E+00_JPRB, 0.49370E+00_JPRB, 0.50754E+00_JPRB, 0.51612E+00_JPRB, 0.52159E+00_JPRB ,&
&  0.52780E+00_JPRB, 0.53083E+00_JPRB, 0.53239E+00_JPRB, 0.53307E+00_JPRB, 0.53495E+00_JPRB, 0.53713E+00_JPRB, 0.53972E+00_JPRB ,&
&  0.54228E+00_JPRB, 0.54634E+00_JPRB, 0.55073E+00_JPRB, 0.55574E+00_JPRB, 0.55893E+00_JPRB, 0.56486E+00_JPRB, 0.57175E+00_JPRB ,&
&  0.57776E+00_JPRB, 0.58085E+00_JPRB, 0.58716E+00_JPRB, 0.58092E+00_JPRB, 0.57845E+00_JPRB, 0.56767E+00_JPRB, 0.55978E+00_JPRB ,&
&  0.55387E+00_JPRB, 0.54650E+00_JPRB, 0.54035E+00_JPRB, 0.53517E+00_JPRB, 0.53126E+00_JPRB, 0.52762E+00_JPRB, 0.52428E+00_JPRB ,&
&  0.52083E+00_JPRB, 0.51657E+00_JPRB, 0.51378E+00_JPRB, 0.51023E+00_JPRB, 0.50807E+00_JPRB, 0.50543E+00_JPRB, 0.50396E+00_JPRB ,&
&  0.50278E+00_JPRB, 0.50238E+00_JPRB, 0.50184E+00_JPRB, 0.50237E+00_JPRB, 0.50259E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.50603E+00_JPRB, 0.51013E+00_JPRB, 0.51517E+00_JPRB, 0.51944E+00_JPRB, 0.52150E+00_JPRB, 0.52320E+00_JPRB, 0.52208E+00_JPRB ,&
&  0.52137E+00_JPRB, 0.52011E+00_JPRB, 0.51920E+00_JPRB, 0.51917E+00_JPRB, 0.52000E+00_JPRB, 0.52148E+00_JPRB, 0.52339E+00_JPRB ,&
&  0.52509E+00_JPRB, 0.52722E+00_JPRB, 0.52801E+00_JPRB, 0.52906E+00_JPRB, 0.53030E+00_JPRB, 0.53328E+00_JPRB, 0.53443E+00_JPRB ,&
&  0.53593E+00_JPRB, 0.53178E+00_JPRB, 0.52236E+00_JPRB, 0.51924E+00_JPRB, 0.52340E+00_JPRB, 0.51451E+00_JPRB, 0.51567E+00_JPRB ,&
&  0.52167E+00_JPRB, 0.52261E+00_JPRB, 0.51346E+00_JPRB, 0.50653E+00_JPRB, 0.48940E+00_JPRB, 0.43572E+00_JPRB, 0.43398E+00_JPRB ,&
&  0.42387E+00_JPRB, 0.40663E+00_JPRB, 0.39769E+00_JPRB, 0.38997E+00_JPRB, 0.38657E+00_JPRB, 0.39320E+00_JPRB, 0.40376E+00_JPRB ,&
&  0.40681E+00_JPRB, 0.42009E+00_JPRB, 0.42523E+00_JPRB, 0.43331E+00_JPRB, 0.44198E+00_JPRB, 0.45411E+00_JPRB, 0.46099E+00_JPRB ,&
&  0.46854E+00_JPRB, 0.47629E+00_JPRB, 0.48781E+00_JPRB, 0.50281E+00_JPRB, 0.51513E+00_JPRB, 0.52132E+00_JPRB, 0.52010E+00_JPRB ,&
&  0.52642E+00_JPRB, 0.52183E+00_JPRB, 0.52480E+00_JPRB, 0.52872E+00_JPRB, 0.53123E+00_JPRB, 0.53594E+00_JPRB, 0.53774E+00_JPRB ,&
&  0.54493E+00_JPRB, 0.55688E+00_JPRB, 0.57005E+00_JPRB, 0.57732E+00_JPRB, 0.58358E+00_JPRB, 0.59061E+00_JPRB, 0.60191E+00_JPRB ,&
&  0.61301E+00_JPRB, 0.62652E+00_JPRB, 0.63844E+00_JPRB, 0.62504E+00_JPRB, 0.61936E+00_JPRB, 0.60506E+00_JPRB, 0.59456E+00_JPRB ,&
&  0.58186E+00_JPRB, 0.57343E+00_JPRB, 0.55934E+00_JPRB, 0.55140E+00_JPRB, 0.54343E+00_JPRB, 0.53685E+00_JPRB, 0.53067E+00_JPRB ,&
&  0.52542E+00_JPRB, 0.52055E+00_JPRB, 0.51735E+00_JPRB, 0.51532E+00_JPRB, 0.51298E+00_JPRB, 0.50999E+00_JPRB, 0.50781E+00_JPRB ,&
&  0.50414E+00_JPRB, 0.50184E+00_JPRB, 0.50021E+00_JPRB, 0.50051E+00_JPRB, 0.50199E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.50278E+00_JPRB, 0.50837E+00_JPRB, 0.51323E+00_JPRB, 0.51962E+00_JPRB, 0.52013E+00_JPRB, 0.52210E+00_JPRB, 0.51775E+00_JPRB ,&
&  0.51475E+00_JPRB, 0.51316E+00_JPRB, 0.51244E+00_JPRB, 0.51369E+00_JPRB, 0.51567E+00_JPRB, 0.51808E+00_JPRB, 0.52154E+00_JPRB ,&
&  0.52486E+00_JPRB, 0.52356E+00_JPRB, 0.52563E+00_JPRB, 0.53053E+00_JPRB, 0.53255E+00_JPRB, 0.53610E+00_JPRB, 0.53819E+00_JPRB ,&
&  0.53844E+00_JPRB, 0.53602E+00_JPRB, 0.53351E+00_JPRB, 0.52668E+00_JPRB, 0.52407E+00_JPRB, 0.52671E+00_JPRB, 0.52963E+00_JPRB ,&
&  0.51803E+00_JPRB, 0.51258E+00_JPRB, 0.52493E+00_JPRB, 0.52760E+00_JPRB, 0.49767E+00_JPRB, 0.47006E+00_JPRB, 0.45298E+00_JPRB ,&
&  0.43361E+00_JPRB, 0.40702E+00_JPRB, 0.39742E+00_JPRB, 0.39096E+00_JPRB, 0.39306E+00_JPRB, 0.39652E+00_JPRB, 0.40517E+00_JPRB ,&
&  0.41153E+00_JPRB, 0.41983E+00_JPRB, 0.42995E+00_JPRB, 0.44293E+00_JPRB, 0.45191E+00_JPRB, 0.46182E+00_JPRB, 0.46998E+00_JPRB ,&
&  0.47882E+00_JPRB, 0.49005E+00_JPRB, 0.50495E+00_JPRB, 0.52357E+00_JPRB, 0.53666E+00_JPRB, 0.54583E+00_JPRB, 0.54881E+00_JPRB ,&
&  0.56858E+00_JPRB, 0.55558E+00_JPRB, 0.55831E+00_JPRB, 0.55987E+00_JPRB, 0.56046E+00_JPRB, 0.56366E+00_JPRB, 0.56576E+00_JPRB ,&
&  0.57702E+00_JPRB, 0.59485E+00_JPRB, 0.60064E+00_JPRB, 0.60434E+00_JPRB, 0.60767E+00_JPRB, 0.61291E+00_JPRB, 0.62752E+00_JPRB ,&
&  0.64135E+00_JPRB, 0.66854E+00_JPRB, 0.73499E+00_JPRB, 0.70443E+00_JPRB, 0.68194E+00_JPRB, 0.65867E+00_JPRB, 0.63515E+00_JPRB ,&
&  0.61371E+00_JPRB, 0.59444E+00_JPRB, 0.57535E+00_JPRB, 0.55819E+00_JPRB, 0.54590E+00_JPRB, 0.53471E+00_JPRB, 0.53181E+00_JPRB ,&
&  0.52890E+00_JPRB, 0.52512E+00_JPRB, 0.52214E+00_JPRB, 0.51955E+00_JPRB, 0.51482E+00_JPRB, 0.51066E+00_JPRB, 0.50794E+00_JPRB ,&
&  0.50286E+00_JPRB, 0.50073E+00_JPRB, 0.49840E+00_JPRB, 0.49867E+00_JPRB, 0.49939E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.50043E+00_JPRB, 0.51204E+00_JPRB, 0.52059E+00_JPRB, 0.52625E+00_JPRB, 0.52430E+00_JPRB, 0.52500E+00_JPRB, 0.51886E+00_JPRB ,&
&  0.51563E+00_JPRB, 0.51611E+00_JPRB, 0.51689E+00_JPRB, 0.51753E+00_JPRB, 0.51822E+00_JPRB, 0.51865E+00_JPRB, 0.51986E+00_JPRB ,&
&  0.52209E+00_JPRB, 0.52500E+00_JPRB, 0.52697E+00_JPRB, 0.52973E+00_JPRB, 0.53343E+00_JPRB, 0.53804E+00_JPRB, 0.54222E+00_JPRB ,&
&  0.54344E+00_JPRB, 0.54123E+00_JPRB, 0.54259E+00_JPRB, 0.53888E+00_JPRB, 0.53491E+00_JPRB, 0.53031E+00_JPRB, 0.52787E+00_JPRB ,&
&  0.53323E+00_JPRB, 0.54052E+00_JPRB, 0.53385E+00_JPRB, 0.52889E+00_JPRB, 0.52580E+00_JPRB, 0.50067E+00_JPRB, 0.47259E+00_JPRB ,&
&  0.44422E+00_JPRB, 0.41989E+00_JPRB, 0.40850E+00_JPRB, 0.40305E+00_JPRB, 0.40768E+00_JPRB, 0.41841E+00_JPRB, 0.42911E+00_JPRB ,&
&  0.43032E+00_JPRB, 0.44521E+00_JPRB, 0.46039E+00_JPRB, 0.47006E+00_JPRB, 0.47798E+00_JPRB, 0.48578E+00_JPRB, 0.49335E+00_JPRB ,&
&  0.50541E+00_JPRB, 0.52309E+00_JPRB, 0.55350E+00_JPRB, 0.58609E+00_JPRB, 0.61160E+00_JPRB, 0.62005E+00_JPRB, 0.65561E+00_JPRB ,&
&  0.74892E+00_JPRB, 0.68918E+00_JPRB, 0.65710E+00_JPRB, 0.64439E+00_JPRB, 0.63180E+00_JPRB, 0.62753E+00_JPRB, 0.63220E+00_JPRB ,&
&  0.64316E+00_JPRB, 0.65389E+00_JPRB, 0.65568E+00_JPRB, 0.65367E+00_JPRB, 0.65536E+00_JPRB, 0.66156E+00_JPRB, 0.67650E+00_JPRB ,&
&  0.69400E+00_JPRB, 0.74333E+00_JPRB, 0.86314E+00_JPRB, 0.76160E+00_JPRB, 0.70387E+00_JPRB, 0.67576E+00_JPRB, 0.64975E+00_JPRB ,&
&  0.62755E+00_JPRB, 0.60833E+00_JPRB, 0.58861E+00_JPRB, 0.56498E+00_JPRB, 0.54613E+00_JPRB, 0.52999E+00_JPRB, 0.52386E+00_JPRB ,&
&  0.51927E+00_JPRB, 0.51484E+00_JPRB, 0.51412E+00_JPRB, 0.51137E+00_JPRB, 0.50766E+00_JPRB, 0.50787E+00_JPRB, 0.50700E+00_JPRB ,&
&  0.50180E+00_JPRB, 0.49732E+00_JPRB, 0.49388E+00_JPRB, 0.49286E+00_JPRB, 0.49383E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.51314E+00_JPRB, 0.52132E+00_JPRB, 0.53190E+00_JPRB, 0.55079E+00_JPRB, 0.55912E+00_JPRB, 0.55342E+00_JPRB, 0.53733E+00_JPRB ,&
&  0.52761E+00_JPRB, 0.52355E+00_JPRB, 0.52680E+00_JPRB, 0.53101E+00_JPRB, 0.53292E+00_JPRB, 0.53272E+00_JPRB, 0.53084E+00_JPRB ,&
&  0.52710E+00_JPRB, 0.52569E+00_JPRB, 0.52657E+00_JPRB, 0.52910E+00_JPRB, 0.53276E+00_JPRB, 0.53685E+00_JPRB, 0.54213E+00_JPRB ,&
&  0.54628E+00_JPRB, 0.54655E+00_JPRB, 0.54609E+00_JPRB, 0.54360E+00_JPRB, 0.54369E+00_JPRB, 0.54649E+00_JPRB, 0.54527E+00_JPRB ,&
&  0.53844E+00_JPRB, 0.54285E+00_JPRB, 0.54866E+00_JPRB, 0.53964E+00_JPRB, 0.53547E+00_JPRB, 0.51727E+00_JPRB, 0.48983E+00_JPRB ,&
&  0.46578E+00_JPRB, 0.44283E+00_JPRB, 0.43629E+00_JPRB, 0.43848E+00_JPRB, 0.44638E+00_JPRB, 0.45165E+00_JPRB, 0.46613E+00_JPRB ,&
&  0.47852E+00_JPRB, 0.48818E+00_JPRB, 0.49560E+00_JPRB, 0.50024E+00_JPRB, 0.50565E+00_JPRB, 0.51475E+00_JPRB, 0.53300E+00_JPRB ,&
&  0.56642E+00_JPRB, 0.60966E+00_JPRB, 0.65550E+00_JPRB, 0.63725E+00_JPRB, 0.64277E+00_JPRB, 0.66630E+00_JPRB, 0.70214E+00_JPRB ,&
&  0.73254E+00_JPRB, 0.72742E+00_JPRB, 0.72082E+00_JPRB, 0.74768E+00_JPRB, 0.76175E+00_JPRB, 0.77871E+00_JPRB, 0.77526E+00_JPRB ,&
&  0.76289E+00_JPRB, 0.74661E+00_JPRB, 0.73593E+00_JPRB, 0.73645E+00_JPRB, 0.75055E+00_JPRB, 0.76323E+00_JPRB, 0.76332E+00_JPRB ,&
&  0.74457E+00_JPRB, 0.71284E+00_JPRB, 0.67624E+00_JPRB, 0.63975E+00_JPRB, 0.63170E+00_JPRB, 0.63485E+00_JPRB, 0.63576E+00_JPRB ,&
&  0.62363E+00_JPRB, 0.60721E+00_JPRB, 0.58996E+00_JPRB, 0.56758E+00_JPRB, 0.54488E+00_JPRB, 0.52230E+00_JPRB, 0.51034E+00_JPRB ,&
&  0.50325E+00_JPRB, 0.49973E+00_JPRB, 0.50233E+00_JPRB, 0.49541E+00_JPRB, 0.49674E+00_JPRB, 0.50031E+00_JPRB, 0.49522E+00_JPRB ,&
&  0.49083E+00_JPRB, 0.49014E+00_JPRB, 0.48610E+00_JPRB, 0.48899E+00_JPRB, 0.49615E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.53873E+00_JPRB, 0.56350E+00_JPRB, 0.58599E+00_JPRB, 0.60066E+00_JPRB, 0.60489E+00_JPRB, 0.60397E+00_JPRB, 0.58919E+00_JPRB ,&
&  0.58734E+00_JPRB, 0.56357E+00_JPRB, 0.54792E+00_JPRB, 0.53085E+00_JPRB, 0.51781E+00_JPRB, 0.50998E+00_JPRB, 0.51151E+00_JPRB ,&
&  0.51620E+00_JPRB, 0.52125E+00_JPRB, 0.52440E+00_JPRB, 0.52559E+00_JPRB, 0.52849E+00_JPRB, 0.53320E+00_JPRB, 0.53984E+00_JPRB ,&
&  0.54696E+00_JPRB, 0.55132E+00_JPRB, 0.55449E+00_JPRB, 0.55630E+00_JPRB, 0.55727E+00_JPRB, 0.55701E+00_JPRB, 0.55239E+00_JPRB ,&
&  0.55246E+00_JPRB, 0.55075E+00_JPRB, 0.55168E+00_JPRB, 0.54666E+00_JPRB, 0.54331E+00_JPRB, 0.52958E+00_JPRB, 0.49990E+00_JPRB ,&
&  0.48328E+00_JPRB, 0.47484E+00_JPRB, 0.47890E+00_JPRB, 0.48552E+00_JPRB, 0.49443E+00_JPRB, 0.50528E+00_JPRB, 0.51026E+00_JPRB ,&
&  0.50874E+00_JPRB, 0.50948E+00_JPRB, 0.51904E+00_JPRB, 0.52720E+00_JPRB, 0.53525E+00_JPRB, 0.54820E+00_JPRB, 0.56306E+00_JPRB ,&
&  0.59109E+00_JPRB, 0.63498E+00_JPRB, 0.70597E+00_JPRB, 0.72682E+00_JPRB, 0.78969E+00_JPRB, 0.93270E+00_JPRB, 0.85276E+00_JPRB ,&
&  0.86857E+00_JPRB, 0.88402E+00_JPRB, 0.91604E+00_JPRB, 0.97322E+00_JPRB, 0.95169E+00_JPRB, 0.93841E+00_JPRB, 0.92847E+00_JPRB ,&
&  0.89061E+00_JPRB, 0.87628E+00_JPRB, 0.86315E+00_JPRB, 0.88217E+00_JPRB, 0.87731E+00_JPRB, 0.87070E+00_JPRB, 0.86007E+00_JPRB ,&
&  0.82087E+00_JPRB, 0.77169E+00_JPRB, 0.72249E+00_JPRB, 0.67560E+00_JPRB, 0.65187E+00_JPRB, 0.63812E+00_JPRB, 0.62916E+00_JPRB ,&
&  0.61696E+00_JPRB, 0.60707E+00_JPRB, 0.59777E+00_JPRB, 0.58355E+00_JPRB, 0.55910E+00_JPRB, 0.53046E+00_JPRB, 0.51825E+00_JPRB ,&
&  0.50826E+00_JPRB, 0.50118E+00_JPRB, 0.49695E+00_JPRB, 0.49356E+00_JPRB, 0.49689E+00_JPRB, 0.49745E+00_JPRB, 0.49461E+00_JPRB ,&
&  0.49656E+00_JPRB, 0.49261E+00_JPRB, 0.48869E+00_JPRB, 0.49350E+00_JPRB, 0.51464E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.58188E+00_JPRB, 0.61441E+00_JPRB, 0.62837E+00_JPRB, 0.63732E+00_JPRB, 0.64537E+00_JPRB, 0.64409E+00_JPRB, 0.61521E+00_JPRB ,&
&  0.61034E+00_JPRB, 0.55967E+00_JPRB, 0.54011E+00_JPRB, 0.52455E+00_JPRB, 0.51720E+00_JPRB, 0.50920E+00_JPRB, 0.50007E+00_JPRB ,&
&  0.50227E+00_JPRB, 0.51222E+00_JPRB, 0.52300E+00_JPRB, 0.52765E+00_JPRB, 0.53171E+00_JPRB, 0.53489E+00_JPRB, 0.53862E+00_JPRB ,&
&  0.54494E+00_JPRB, 0.55265E+00_JPRB, 0.56075E+00_JPRB, 0.56374E+00_JPRB, 0.56398E+00_JPRB, 0.56346E+00_JPRB, 0.56175E+00_JPRB ,&
&  0.56042E+00_JPRB, 0.55300E+00_JPRB, 0.55323E+00_JPRB, 0.55155E+00_JPRB, 0.54556E+00_JPRB, 0.54340E+00_JPRB, 0.52632E+00_JPRB ,&
&  0.50377E+00_JPRB, 0.51465E+00_JPRB, 0.52202E+00_JPRB, 0.53283E+00_JPRB, 0.54131E+00_JPRB, 0.54740E+00_JPRB, 0.55021E+00_JPRB ,&
&  0.54987E+00_JPRB, 0.54870E+00_JPRB, 0.55067E+00_JPRB, 0.55491E+00_JPRB, 0.56048E+00_JPRB, 0.56824E+00_JPRB, 0.57546E+00_JPRB ,&
&  0.58475E+00_JPRB, 0.57256E+00_JPRB, 0.62094E+00_JPRB, 0.75494E+00_JPRB, 0.10388E+01_JPRB, 0.10263E+01_JPRB, 0.10517E+01_JPRB ,&
&  0.11249E+01_JPRB, 0.11088E+01_JPRB, 0.11230E+01_JPRB, 0.11367E+01_JPRB, 0.11293E+01_JPRB, 0.11115E+01_JPRB, 0.11135E+01_JPRB ,&
&  0.10773E+01_JPRB, 0.10927E+01_JPRB, 0.10718E+01_JPRB, 0.10611E+01_JPRB, 0.10301E+01_JPRB, 0.10141E+01_JPRB, 0.98675E+00_JPRB ,&
&  0.94369E+00_JPRB, 0.90546E+00_JPRB, 0.84270E+00_JPRB, 0.77001E+00_JPRB, 0.71163E+00_JPRB, 0.68067E+00_JPRB, 0.65361E+00_JPRB ,&
&  0.63412E+00_JPRB, 0.62318E+00_JPRB, 0.61047E+00_JPRB, 0.60283E+00_JPRB, 0.58743E+00_JPRB, 0.56966E+00_JPRB, 0.56355E+00_JPRB ,&
&  0.54197E+00_JPRB, 0.52974E+00_JPRB, 0.51858E+00_JPRB, 0.51330E+00_JPRB, 0.52003E+00_JPRB, 0.52756E+00_JPRB, 0.53180E+00_JPRB ,&
&  0.54021E+00_JPRB, 0.53497E+00_JPRB, 0.53409E+00_JPRB, 0.54119E+00_JPRB, 0.55584E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.62003E+00_JPRB, 0.64405E+00_JPRB, 0.65506E+00_JPRB, 0.66717E+00_JPRB, 0.68195E+00_JPRB, 0.68034E+00_JPRB, 0.67663E+00_JPRB ,&
&  0.69871E+00_JPRB, 0.66350E+00_JPRB, 0.69909E+00_JPRB, 0.58530E+00_JPRB, 0.54109E+00_JPRB, 0.53075E+00_JPRB, 0.51780E+00_JPRB ,&
&  0.51586E+00_JPRB, 0.52123E+00_JPRB, 0.52673E+00_JPRB, 0.53213E+00_JPRB, 0.53835E+00_JPRB, 0.53878E+00_JPRB, 0.53965E+00_JPRB ,&
&  0.54338E+00_JPRB, 0.55168E+00_JPRB, 0.56464E+00_JPRB, 0.56954E+00_JPRB, 0.57346E+00_JPRB, 0.57790E+00_JPRB, 0.57403E+00_JPRB ,&
&  0.56568E+00_JPRB, 0.56592E+00_JPRB, 0.56540E+00_JPRB, 0.55762E+00_JPRB, 0.54934E+00_JPRB, 0.54883E+00_JPRB, 0.56016E+00_JPRB ,&
&  0.56027E+00_JPRB, 0.55083E+00_JPRB, 0.56514E+00_JPRB, 0.57194E+00_JPRB, 0.57536E+00_JPRB, 0.57626E+00_JPRB, 0.57606E+00_JPRB ,&
&  0.57684E+00_JPRB, 0.57638E+00_JPRB, 0.57969E+00_JPRB, 0.58141E+00_JPRB, 0.58373E+00_JPRB, 0.58798E+00_JPRB, 0.58878E+00_JPRB ,&
&  0.60154E+00_JPRB, 0.59329E+00_JPRB, 0.70658E+00_JPRB, 0.83703E+00_JPRB, 0.10971E+01_JPRB, 0.12380E+01_JPRB, 0.12551E+01_JPRB ,&
&  0.14256E+01_JPRB, 0.13581E+01_JPRB, 0.14208E+01_JPRB, 0.13986E+01_JPRB, 0.13672E+01_JPRB, 0.13212E+01_JPRB, 0.12653E+01_JPRB ,&
&  0.12549E+01_JPRB, 0.12498E+01_JPRB, 0.12982E+01_JPRB, 0.12639E+01_JPRB, 0.11916E+01_JPRB, 0.11640E+01_JPRB, 0.11443E+01_JPRB ,&
&  0.11337E+01_JPRB, 0.11159E+01_JPRB, 0.10613E+01_JPRB, 0.91171E+00_JPRB, 0.83718E+00_JPRB, 0.77668E+00_JPRB, 0.70276E+00_JPRB ,&
&  0.66676E+00_JPRB, 0.63142E+00_JPRB, 0.60501E+00_JPRB, 0.60061E+00_JPRB, 0.58883E+00_JPRB, 0.57809E+00_JPRB, 0.57553E+00_JPRB ,&
&  0.55853E+00_JPRB, 0.55111E+00_JPRB, 0.54916E+00_JPRB, 0.55295E+00_JPRB, 0.55810E+00_JPRB, 0.56493E+00_JPRB, 0.57415E+00_JPRB ,&
&  0.57058E+00_JPRB, 0.56540E+00_JPRB, 0.57535E+00_JPRB, 0.58080E+00_JPRB, 0.59662E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.64124E+00_JPRB, 0.66324E+00_JPRB, 0.68101E+00_JPRB, 0.69769E+00_JPRB, 0.72034E+00_JPRB, 0.74394E+00_JPRB, 0.74231E+00_JPRB ,&
&  0.75387E+00_JPRB, 0.75709E+00_JPRB, 0.75440E+00_JPRB, 0.74051E+00_JPRB, 0.71266E+00_JPRB, 0.63581E+00_JPRB, 0.54196E+00_JPRB ,&
&  0.51470E+00_JPRB, 0.51901E+00_JPRB, 0.52890E+00_JPRB, 0.53316E+00_JPRB, 0.53571E+00_JPRB, 0.53900E+00_JPRB, 0.53966E+00_JPRB ,&
&  0.54399E+00_JPRB, 0.55074E+00_JPRB, 0.56241E+00_JPRB, 0.57671E+00_JPRB, 0.58954E+00_JPRB, 0.59555E+00_JPRB, 0.59293E+00_JPRB ,&
&  0.58152E+00_JPRB, 0.57689E+00_JPRB, 0.56778E+00_JPRB, 0.55662E+00_JPRB, 0.56059E+00_JPRB, 0.56818E+00_JPRB, 0.58531E+00_JPRB ,&
&  0.60056E+00_JPRB, 0.61125E+00_JPRB, 0.61574E+00_JPRB, 0.62076E+00_JPRB, 0.62329E+00_JPRB, 0.62008E+00_JPRB, 0.61348E+00_JPRB ,&
&  0.60771E+00_JPRB, 0.60364E+00_JPRB, 0.60125E+00_JPRB, 0.60332E+00_JPRB, 0.60280E+00_JPRB, 0.61284E+00_JPRB, 0.62983E+00_JPRB ,&
&  0.65186E+00_JPRB, 0.74208E+00_JPRB, 0.88136E+00_JPRB, 0.92394E+00_JPRB, 0.11783E+01_JPRB, 0.13110E+01_JPRB, 0.14074E+01_JPRB ,&
&  0.14743E+01_JPRB, 0.14183E+01_JPRB, 0.15801E+01_JPRB, 0.16233E+01_JPRB, 0.16359E+01_JPRB, 0.15861E+01_JPRB, 0.14760E+01_JPRB ,&
&  0.14214E+01_JPRB, 0.15000E+01_JPRB, 0.15357E+01_JPRB, 0.15149E+01_JPRB, 0.14467E+01_JPRB, 0.14276E+01_JPRB, 0.14157E+01_JPRB ,&
&  0.14386E+01_JPRB, 0.14544E+01_JPRB, 0.13821E+01_JPRB, 0.12352E+01_JPRB, 0.10652E+01_JPRB, 0.91985E+00_JPRB, 0.80576E+00_JPRB ,&
&  0.71665E+00_JPRB, 0.63309E+00_JPRB, 0.60247E+00_JPRB, 0.59447E+00_JPRB, 0.59464E+00_JPRB, 0.58028E+00_JPRB, 0.57664E+00_JPRB ,&
&  0.57722E+00_JPRB, 0.57750E+00_JPRB, 0.58337E+00_JPRB, 0.58651E+00_JPRB, 0.60053E+00_JPRB, 0.60862E+00_JPRB, 0.59013E+00_JPRB ,&
&  0.57718E+00_JPRB, 0.58241E+00_JPRB, 0.58872E+00_JPRB, 0.59838E+00_JPRB, 0.61809E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.67033E+00_JPRB, 0.69532E+00_JPRB, 0.71394E+00_JPRB, 0.72977E+00_JPRB, 0.75576E+00_JPRB, 0.79411E+00_JPRB, 0.80912E+00_JPRB ,&
&  0.80770E+00_JPRB, 0.80388E+00_JPRB, 0.79935E+00_JPRB, 0.78854E+00_JPRB, 0.76589E+00_JPRB, 0.72845E+00_JPRB, 0.64236E+00_JPRB ,&
&  0.55102E+00_JPRB, 0.54169E+00_JPRB, 0.52909E+00_JPRB, 0.53362E+00_JPRB, 0.53911E+00_JPRB, 0.53839E+00_JPRB, 0.53587E+00_JPRB ,&
&  0.54122E+00_JPRB, 0.54798E+00_JPRB, 0.56027E+00_JPRB, 0.57539E+00_JPRB, 0.58119E+00_JPRB, 0.58786E+00_JPRB, 0.59554E+00_JPRB ,&
&  0.58498E+00_JPRB, 0.57165E+00_JPRB, 0.56207E+00_JPRB, 0.55764E+00_JPRB, 0.57215E+00_JPRB, 0.59549E+00_JPRB, 0.61687E+00_JPRB ,&
&  0.63206E+00_JPRB, 0.64190E+00_JPRB, 0.64990E+00_JPRB, 0.65643E+00_JPRB, 0.66000E+00_JPRB, 0.65681E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.63442E+00_JPRB, 0.62676E+00_JPRB, 0.62261E+00_JPRB, 0.62676E+00_JPRB, 0.63264E+00_JPRB, 0.66511E+00_JPRB, 0.70707E+00_JPRB ,&
&  0.76973E+00_JPRB, 0.87713E+00_JPRB, 0.95972E+00_JPRB, 0.11064E+01_JPRB, 0.13226E+01_JPRB, 0.14839E+01_JPRB, 0.15752E+01_JPRB ,&
&  0.16661E+01_JPRB, 0.16393E+01_JPRB, 0.18883E+01_JPRB, 0.19027E+01_JPRB, 0.18154E+01_JPRB, 0.17876E+01_JPRB, 0.16614E+01_JPRB ,&
&  0.15618E+01_JPRB, 0.14883E+01_JPRB, 0.15785E+01_JPRB, 0.16358E+01_JPRB, 0.16729E+01_JPRB, 0.17244E+01_JPRB, 0.17321E+01_JPRB ,&
&  0.17490E+01_JPRB, 0.17361E+01_JPRB, 0.14981E+01_JPRB, 0.12533E+01_JPRB, 0.10094E+01_JPRB, 0.96765E+00_JPRB, 0.85616E+00_JPRB ,&
&  0.73982E+00_JPRB, 0.63178E+00_JPRB, 0.61817E+00_JPRB, 0.60944E+00_JPRB, 0.62043E+00_JPRB, 0.61583E+00_JPRB, 0.60766E+00_JPRB ,&
&  0.61526E+00_JPRB, 0.62910E+00_JPRB, 0.62659E+00_JPRB, 0.63275E+00_JPRB, 0.64933E+00_JPRB, 0.65184E+00_JPRB, 0.62501E+00_JPRB ,&
&  0.61376E+00_JPRB, 0.61390E+00_JPRB, 0.60536E+00_JPRB, 0.61439E+00_JPRB, 0.63826E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.70622E+00_JPRB, 0.72741E+00_JPRB, 0.74395E+00_JPRB, 0.76680E+00_JPRB, 0.80222E+00_JPRB, 0.84135E+00_JPRB, 0.85274E+00_JPRB ,&
&  0.84810E+00_JPRB, 0.84112E+00_JPRB, 0.83166E+00_JPRB, 0.81559E+00_JPRB, 0.79649E+00_JPRB, 0.75902E+00_JPRB, 0.71596E+00_JPRB ,&
&  0.59899E+00_JPRB, 0.56309E+00_JPRB, 0.52188E+00_JPRB, 0.52197E+00_JPRB, 0.56091E+00_JPRB, 0.55369E+00_JPRB, 0.55318E+00_JPRB ,&
&  0.54938E+00_JPRB, 0.55656E+00_JPRB, 0.56273E+00_JPRB, 0.57221E+00_JPRB, 0.57008E+00_JPRB, 0.57937E+00_JPRB, 0.59130E+00_JPRB ,&
&  0.58383E+00_JPRB, 0.57472E+00_JPRB, 0.57441E+00_JPRB, 0.58069E+00_JPRB, 0.60043E+00_JPRB, 0.62861E+00_JPRB, 0.66291E+00_JPRB ,&
&  0.68042E+00_JPRB, 0.69609E+00_JPRB, 0.70472E+00_JPRB, 0.71057E+00_JPRB, 0.71195E+00_JPRB, 0.70679E+00_JPRB, 0.69581E+00_JPRB ,&
&  0.68084E+00_JPRB, 0.66868E+00_JPRB, 0.67030E+00_JPRB, 0.68221E+00_JPRB, 0.68270E+00_JPRB, 0.73290E+00_JPRB, 0.83682E+00_JPRB ,&
&  0.97004E+00_JPRB, 0.10287E+01_JPRB, 0.11211E+01_JPRB, 0.14336E+01_JPRB, 0.16841E+01_JPRB, 0.18297E+01_JPRB, 0.18629E+01_JPRB ,&
&  0.18787E+01_JPRB, 0.18703E+01_JPRB, 0.18767E+01_JPRB, 0.19165E+01_JPRB, 0.18556E+01_JPRB, 0.18795E+01_JPRB, 0.17866E+01_JPRB ,&
&  0.16360E+01_JPRB, 0.16130E+01_JPRB, 0.17058E+01_JPRB, 0.17776E+01_JPRB, 0.18117E+01_JPRB, 0.18787E+01_JPRB, 0.19353E+01_JPRB ,&
&  0.19086E+01_JPRB, 0.16886E+01_JPRB, 0.12687E+01_JPRB, 0.11289E+01_JPRB, 0.85402E+00_JPRB, 0.82806E+00_JPRB, 0.90738E+00_JPRB ,&
&  0.87534E+00_JPRB, 0.77001E+00_JPRB, 0.69522E+00_JPRB, 0.66174E+00_JPRB, 0.66155E+00_JPRB, 0.72006E+00_JPRB, 0.70948E+00_JPRB ,&
&  0.67194E+00_JPRB, 0.68467E+00_JPRB, 0.69986E+00_JPRB, 0.71696E+00_JPRB, 0.73734E+00_JPRB, 0.73974E+00_JPRB, 0.70977E+00_JPRB ,&
&  0.70713E+00_JPRB, 0.68461E+00_JPRB, 0.66540E+00_JPRB, 0.66484E+00_JPRB, 0.68102E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.76714E+00_JPRB, 0.77619E+00_JPRB, 0.79087E+00_JPRB, 0.81454E+00_JPRB, 0.85511E+00_JPRB, 0.88603E+00_JPRB, 0.88691E+00_JPRB ,&
&  0.87751E+00_JPRB, 0.86870E+00_JPRB, 0.85442E+00_JPRB, 0.83684E+00_JPRB, 0.80671E+00_JPRB, 0.75908E+00_JPRB, 0.71497E+00_JPRB ,&
&  0.70023E+00_JPRB, 0.62310E+00_JPRB, 0.56565E+00_JPRB, 0.52801E+00_JPRB, 0.55186E+00_JPRB, 0.55409E+00_JPRB, 0.56683E+00_JPRB ,&
&  0.56562E+00_JPRB, 0.58131E+00_JPRB, 0.58298E+00_JPRB, 0.57997E+00_JPRB, 0.57602E+00_JPRB, 0.59386E+00_JPRB, 0.61555E+00_JPRB ,&
&  0.62874E+00_JPRB, 0.63355E+00_JPRB, 0.63839E+00_JPRB, 0.65939E+00_JPRB, 0.69839E+00_JPRB, 0.68065E+00_JPRB, 0.71782E+00_JPRB ,&
&  0.73746E+00_JPRB, 0.75141E+00_JPRB, 0.76170E+00_JPRB, 0.77531E+00_JPRB, 0.77973E+00_JPRB, 0.78500E+00_JPRB, 0.79094E+00_JPRB ,&
&  0.79861E+00_JPRB, 0.79791E+00_JPRB, 0.77093E+00_JPRB, 0.76436E+00_JPRB, 0.77596E+00_JPRB, 0.87807E+00_JPRB, 0.93658E+00_JPRB ,&
&  0.10034E+01_JPRB, 0.10046E+01_JPRB, 0.10562E+01_JPRB, 0.14121E+01_JPRB, 0.17159E+01_JPRB, 0.18303E+01_JPRB, 0.19279E+01_JPRB ,&
&  0.19779E+01_JPRB, 0.19991E+01_JPRB, 0.20653E+01_JPRB, 0.20903E+01_JPRB, 0.20989E+01_JPRB, 0.20820E+01_JPRB, 0.20369E+01_JPRB ,&
&  0.19178E+01_JPRB, 0.18828E+01_JPRB, 0.19529E+01_JPRB, 0.19236E+01_JPRB, 0.19041E+01_JPRB, 0.18275E+01_JPRB, 0.19648E+01_JPRB ,&
&  0.19676E+01_JPRB, 0.12001E+01_JPRB, 0.96105E+00_JPRB, 0.10828E+01_JPRB, 0.89963E+00_JPRB, 0.86109E+00_JPRB, 0.90413E+00_JPRB ,&
&  0.85351E+00_JPRB, 0.80013E+00_JPRB, 0.77569E+00_JPRB, 0.73614E+00_JPRB, 0.74196E+00_JPRB, 0.87587E+00_JPRB, 0.86923E+00_JPRB ,&
&  0.82887E+00_JPRB, 0.81643E+00_JPRB, 0.85130E+00_JPRB, 0.85723E+00_JPRB, 0.85665E+00_JPRB, 0.86626E+00_JPRB, 0.84724E+00_JPRB ,&
&  0.80565E+00_JPRB, 0.77337E+00_JPRB, 0.75073E+00_JPRB, 0.74692E+00_JPRB, 0.75689E+00_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.86419E+00_JPRB, 0.86016E+00_JPRB, 0.86167E+00_JPRB, 0.87942E+00_JPRB, 0.91387E+00_JPRB, 0.93451E+00_JPRB, 0.92435E+00_JPRB ,&
&  0.91266E+00_JPRB, 0.89559E+00_JPRB, 0.87790E+00_JPRB, 0.85154E+00_JPRB, 0.80945E+00_JPRB, 0.76066E+00_JPRB, 0.71578E+00_JPRB ,&
&  0.68617E+00_JPRB, 0.65300E+00_JPRB, 0.57920E+00_JPRB, 0.54807E+00_JPRB, 0.52565E+00_JPRB, 0.54088E+00_JPRB, 0.56012E+00_JPRB ,&
&  0.58043E+00_JPRB, 0.59058E+00_JPRB, 0.59492E+00_JPRB, 0.60067E+00_JPRB, 0.60887E+00_JPRB, 0.64034E+00_JPRB, 0.68702E+00_JPRB ,&
&  0.71498E+00_JPRB, 0.72963E+00_JPRB, 0.71729E+00_JPRB, 0.76526E+00_JPRB, 0.77468E+00_JPRB, 0.75177E+00_JPRB, 0.80404E+00_JPRB ,&
&  0.83251E+00_JPRB, 0.85291E+00_JPRB, 0.85714E+00_JPRB, 0.87361E+00_JPRB, 0.88294E+00_JPRB, 0.86395E+00_JPRB, 0.83977E+00_JPRB ,&
&  0.80934E+00_JPRB, 0.79234E+00_JPRB, 0.78989E+00_JPRB, 0.78412E+00_JPRB, 0.79524E+00_JPRB, 0.85636E+00_JPRB, 0.86214E+00_JPRB ,&
&  0.85328E+00_JPRB, 0.73906E+00_JPRB, 0.70896E+00_JPRB, 0.10116E+01_JPRB, 0.14965E+01_JPRB, 0.17821E+01_JPRB, 0.17820E+01_JPRB ,&
&  0.19584E+01_JPRB, 0.20830E+01_JPRB, 0.22489E+01_JPRB, 0.23473E+01_JPRB, 0.23620E+01_JPRB, 0.22924E+01_JPRB, 0.22167E+01_JPRB ,&
&  0.21477E+01_JPRB, 0.21263E+01_JPRB, 0.21214E+01_JPRB, 0.21140E+01_JPRB, 0.20811E+01_JPRB, 0.21338E+01_JPRB, 0.22720E+01_JPRB ,&
&  0.21246E+01_JPRB, 0.18261E+01_JPRB, 0.12300E+01_JPRB, 0.12076E+01_JPRB, 0.93382E+00_JPRB, 0.88528E+00_JPRB, 0.93615E+00_JPRB ,&
&  0.88240E+00_JPRB, 0.88374E+00_JPRB, 0.85989E+00_JPRB, 0.82169E+00_JPRB, 0.89341E+00_JPRB, 0.10458E+01_JPRB, 0.10882E+01_JPRB ,&
&  0.11185E+01_JPRB, 0.10414E+01_JPRB, 0.11099E+01_JPRB, 0.11147E+01_JPRB, 0.11025E+01_JPRB, 0.10839E+01_JPRB, 0.10477E+01_JPRB ,&
&  0.98682E+00_JPRB, 0.92530E+00_JPRB, 0.88492E+00_JPRB, 0.86800E+00_JPRB, 0.86272E+00_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.93448E+00_JPRB, 0.91217E+00_JPRB, 0.90184E+00_JPRB, 0.91235E+00_JPRB, 0.93766E+00_JPRB, 0.94887E+00_JPRB, 0.93278E+00_JPRB ,&
&  0.91963E+00_JPRB, 0.90196E+00_JPRB, 0.88813E+00_JPRB, 0.84750E+00_JPRB, 0.80781E+00_JPRB, 0.75675E+00_JPRB, 0.70941E+00_JPRB ,&
&  0.67876E+00_JPRB, 0.62769E+00_JPRB, 0.56230E+00_JPRB, 0.53439E+00_JPRB, 0.51314E+00_JPRB, 0.51710E+00_JPRB, 0.54166E+00_JPRB ,&
&  0.57260E+00_JPRB, 0.59622E+00_JPRB, 0.63197E+00_JPRB, 0.64928E+00_JPRB, 0.67962E+00_JPRB, 0.73484E+00_JPRB, 0.79805E+00_JPRB ,&
&  0.88727E+00_JPRB, 0.79861E+00_JPRB, 0.82894E+00_JPRB, 0.83012E+00_JPRB, 0.85734E+00_JPRB, 0.86480E+00_JPRB, 0.86574E+00_JPRB ,&
&  0.87828E+00_JPRB, 0.88900E+00_JPRB, 0.90875E+00_JPRB, 0.90703E+00_JPRB, 0.89076E+00_JPRB, 0.87034E+00_JPRB, 0.85452E+00_JPRB ,&
&  0.85023E+00_JPRB, 0.83884E+00_JPRB, 0.81277E+00_JPRB, 0.78337E+00_JPRB, 0.84769E+00_JPRB, 0.89469E+00_JPRB, 0.84684E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.73344E+00_JPRB, 0.96421E+00_JPRB, 0.12770E+01_JPRB, 0.16117E+01_JPRB, 0.19295E+01_JPRB, 0.21219E+01_JPRB ,&
&  0.22934E+01_JPRB, 0.23954E+01_JPRB, 0.24889E+01_JPRB, 0.22355E+01_JPRB, 0.24266E+01_JPRB, 0.23799E+01_JPRB, 0.22867E+01_JPRB ,&
&  0.21931E+01_JPRB, 0.22016E+01_JPRB, 0.21521E+01_JPRB, 0.22369E+01_JPRB, 0.24042E+01_JPRB, 0.26485E+01_JPRB, 0.23584E+01_JPRB ,&
&  0.19905E+01_JPRB, 0.21038E+01_JPRB, 0.17977E+01_JPRB, 0.14183E+01_JPRB, 0.11132E+01_JPRB, 0.10688E+01_JPRB, 0.10396E+01_JPRB ,&
&  0.99823E+00_JPRB, 0.96643E+00_JPRB, 0.92463E+00_JPRB, 0.97563E+00_JPRB, 0.11385E+01_JPRB, 0.12814E+01_JPRB, 0.12857E+01_JPRB ,&
&  0.12622E+01_JPRB, 0.13629E+01_JPRB, 0.13896E+01_JPRB, 0.13822E+01_JPRB, 0.13858E+01_JPRB, 0.13502E+01_JPRB, 0.12763E+01_JPRB ,&
&  0.11833E+01_JPRB, 0.11055E+01_JPRB, 0.10498E+01_JPRB, 0.10028E+01_JPRB, 0.96396E+00_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.98427E+00_JPRB, 0.95567E+00_JPRB, 0.94535E+00_JPRB, 0.95298E+00_JPRB, 0.97187E+00_JPRB, 0.96827E+00_JPRB, 0.94931E+00_JPRB ,&
&  0.92289E+00_JPRB, 0.89360E+00_JPRB, 0.85842E+00_JPRB, 0.82442E+00_JPRB, 0.79644E+00_JPRB, 0.75265E+00_JPRB, 0.71264E+00_JPRB ,&
&  0.67563E+00_JPRB, 0.61538E+00_JPRB, 0.56633E+00_JPRB, 0.56078E+00_JPRB, 0.54685E+00_JPRB, 0.52300E+00_JPRB, 0.53739E+00_JPRB ,&
&  0.57172E+00_JPRB, 0.61977E+00_JPRB, 0.67487E+00_JPRB, 0.74341E+00_JPRB, 0.80702E+00_JPRB, 0.88784E+00_JPRB, 0.94668E+00_JPRB ,&
&  0.91556E+00_JPRB, 0.90819E+00_JPRB, 0.92887E+00_JPRB, 0.93061E+00_JPRB, 0.90091E+00_JPRB, 0.89569E+00_JPRB, 0.90811E+00_JPRB ,&
&  0.92616E+00_JPRB, 0.94300E+00_JPRB, 0.95041E+00_JPRB, 0.94115E+00_JPRB, 0.93783E+00_JPRB, 0.93291E+00_JPRB, 0.93176E+00_JPRB ,&
&  0.88729E+00_JPRB, 0.84735E+00_JPRB, 0.80262E+00_JPRB, 0.82158E+00_JPRB, 0.75627E+00_JPRB, 0.72830E+00_JPRB, 0.72231E+00_JPRB ,&
&  0.81020E+00_JPRB, 0.98653E+00_JPRB, 0.11205E+01_JPRB, 0.16217E+01_JPRB, 0.16569E+01_JPRB, 0.17967E+01_JPRB, 0.21831E+01_JPRB ,&
&  0.23991E+01_JPRB, 0.24722E+01_JPRB, 0.24740E+01_JPRB, 0.20159E+01_JPRB, 0.17958E+01_JPRB, 0.23639E+01_JPRB, 0.24021E+01_JPRB ,&
&  0.23132E+01_JPRB, 0.22945E+01_JPRB, 0.23723E+01_JPRB, 0.27707E+01_JPRB, 0.23035E+01_JPRB, 0.20452E+01_JPRB, 0.16369E+01_JPRB ,&
&  0.17089E+01_JPRB, 0.14915E+01_JPRB, 0.16767E+01_JPRB, 0.14668E+01_JPRB, 0.13854E+01_JPRB, 0.12700E+01_JPRB, 0.11234E+01_JPRB ,&
&  0.10544E+01_JPRB, 0.10168E+01_JPRB, 0.10269E+01_JPRB, 0.12215E+01_JPRB, 0.14602E+01_JPRB, 0.15132E+01_JPRB, 0.15856E+01_JPRB ,&
&  0.16415E+01_JPRB, 0.16762E+01_JPRB, 0.16082E+01_JPRB, 0.16115E+01_JPRB, 0.15612E+01_JPRB, 0.14963E+01_JPRB, 0.14193E+01_JPRB ,&
&  0.13384E+01_JPRB, 0.12587E+01_JPRB, 0.11738E+01_JPRB, 0.10944E+01_JPRB, 0.10296E+01_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.10320E+01_JPRB, 0.10070E+01_JPRB, 0.10017E+01_JPRB, 0.10054E+01_JPRB, 0.10008E+01_JPRB, 0.97949E+00_JPRB, 0.95038E+00_JPRB ,&
&  0.91843E+00_JPRB, 0.87022E+00_JPRB, 0.82427E+00_JPRB, 0.78932E+00_JPRB, 0.76957E+00_JPRB, 0.74952E+00_JPRB, 0.72030E+00_JPRB ,&
&  0.68703E+00_JPRB, 0.65626E+00_JPRB, 0.60556E+00_JPRB, 0.58159E+00_JPRB, 0.60404E+00_JPRB, 0.56931E+00_JPRB, 0.54355E+00_JPRB ,&
&  0.59287E+00_JPRB, 0.65675E+00_JPRB, 0.73996E+00_JPRB, 0.84839E+00_JPRB, 0.94616E+00_JPRB, 0.10376E+01_JPRB, 0.10874E+01_JPRB ,&
&  0.10475E+01_JPRB, 0.98733E+00_JPRB, 0.94026E+00_JPRB, 0.89616E+00_JPRB, 0.85929E+00_JPRB, 0.86101E+00_JPRB, 0.87660E+00_JPRB ,&
&  0.91285E+00_JPRB, 0.94525E+00_JPRB, 0.93511E+00_JPRB, 0.89458E+00_JPRB, 0.88311E+00_JPRB, 0.83775E+00_JPRB, 0.80714E+00_JPRB ,&
&  0.81202E+00_JPRB, 0.80765E+00_JPRB, 0.79895E+00_JPRB, 0.85523E+00_JPRB, 0.77526E+00_JPRB, 0.72458E+00_JPRB, 0.74697E+00_JPRB ,&
&  0.84858E+00_JPRB, 0.10375E+01_JPRB, 0.12241E+01_JPRB, 0.16238E+01_JPRB, 0.21565E+01_JPRB, 0.22391E+01_JPRB, 0.24690E+01_JPRB ,&
&  0.21188E+01_JPRB, 0.17830E+01_JPRB, 0.15579E+01_JPRB, 0.13774E+01_JPRB, 0.17629E+01_JPRB, 0.25853E+01_JPRB, 0.28695E+01_JPRB ,&
&  0.28313E+01_JPRB, 0.28391E+01_JPRB, 0.31136E+01_JPRB, 0.29168E+01_JPRB, 0.21625E+01_JPRB, 0.15583E+01_JPRB, 0.20115E+01_JPRB ,&
&  0.21547E+01_JPRB, 0.21386E+01_JPRB, 0.19245E+01_JPRB, 0.15688E+01_JPRB, 0.12600E+01_JPRB, 0.14022E+01_JPRB, 0.12936E+01_JPRB ,&
&  0.11563E+01_JPRB, 0.10969E+01_JPRB, 0.13026E+01_JPRB, 0.16476E+01_JPRB, 0.17896E+01_JPRB, 0.20131E+01_JPRB, 0.20817E+01_JPRB ,&
&  0.19914E+01_JPRB, 0.19203E+01_JPRB, 0.18182E+01_JPRB, 0.16962E+01_JPRB, 0.15892E+01_JPRB, 0.14786E+01_JPRB, 0.13949E+01_JPRB ,&
&  0.13217E+01_JPRB, 0.12579E+01_JPRB, 0.11893E+01_JPRB, 0.11245E+01_JPRB, 0.10707E+01_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.10659E+01_JPRB, 0.10520E+01_JPRB, 0.10399E+01_JPRB, 0.10253E+01_JPRB, 0.10038E+01_JPRB, 0.97506E+00_JPRB, 0.93954E+00_JPRB ,&
&  0.88908E+00_JPRB, 0.84354E+00_JPRB, 0.79755E+00_JPRB, 0.77312E+00_JPRB, 0.74889E+00_JPRB, 0.72402E+00_JPRB, 0.71003E+00_JPRB ,&
&  0.70981E+00_JPRB, 0.72303E+00_JPRB, 0.66108E+00_JPRB, 0.63831E+00_JPRB, 0.64977E+00_JPRB, 0.64411E+00_JPRB, 0.57267E+00_JPRB ,&
&  0.64091E+00_JPRB, 0.71150E+00_JPRB, 0.78415E+00_JPRB, 0.90909E+00_JPRB, 0.11069E+01_JPRB, 0.10852E+01_JPRB, 0.10621E+01_JPRB ,&
&  0.10488E+01_JPRB, 0.10067E+01_JPRB, 0.96511E+00_JPRB, 0.90732E+00_JPRB, 0.86901E+00_JPRB, 0.87945E+00_JPRB, 0.90016E+00_JPRB ,&
&  0.93772E+00_JPRB, 0.95679E+00_JPRB, 0.92626E+00_JPRB, 0.87629E+00_JPRB, 0.83358E+00_JPRB, 0.80698E+00_JPRB, 0.78817E+00_JPRB ,&
&  0.78939E+00_JPRB, 0.78775E+00_JPRB, 0.79099E+00_JPRB, 0.86569E+00_JPRB, 0.80437E+00_JPRB, 0.78150E+00_JPRB, 0.92537E+00_JPRB ,&
&  0.10725E+01_JPRB, 0.13963E+01_JPRB, 0.15328E+01_JPRB, 0.18308E+01_JPRB, 0.21583E+01_JPRB, 0.22932E+01_JPRB, 0.24065E+01_JPRB ,&
&  0.19035E+01_JPRB, 0.16778E+01_JPRB, 0.19021E+01_JPRB, 0.22121E+01_JPRB, 0.22185E+01_JPRB, 0.24603E+01_JPRB, 0.40706E+01_JPRB ,&
&  0.29523E+01_JPRB, 0.31389E+01_JPRB, 0.35244E+01_JPRB, 0.35083E+01_JPRB, 0.16375E+01_JPRB, 0.11337E+01_JPRB, 0.17598E+01_JPRB ,&
&  0.17972E+01_JPRB, 0.11004E+01_JPRB, 0.85164E+00_JPRB, 0.10658E+01_JPRB, 0.85005E+00_JPRB, 0.97304E+00_JPRB, 0.12078E+01_JPRB ,&
&  0.12736E+01_JPRB, 0.12994E+01_JPRB, 0.15992E+01_JPRB, 0.17521E+01_JPRB, 0.19100E+01_JPRB, 0.27136E+01_JPRB, 0.25243E+01_JPRB ,&
&  0.23548E+01_JPRB, 0.20064E+01_JPRB, 0.18994E+01_JPRB, 0.16870E+01_JPRB, 0.15349E+01_JPRB, 0.14020E+01_JPRB, 0.12981E+01_JPRB ,&
&  0.12233E+01_JPRB, 0.11675E+01_JPRB, 0.11285E+01_JPRB, 0.10972E+01_JPRB, 0.10829E+01_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.10885E+01_JPRB, 0.10724E+01_JPRB, 0.10550E+01_JPRB, 0.10325E+01_JPRB, 0.10035E+01_JPRB, 0.96463E+00_JPRB, 0.91481E+00_JPRB ,&
&  0.85995E+00_JPRB, 0.81227E+00_JPRB, 0.77500E+00_JPRB, 0.75097E+00_JPRB, 0.72520E+00_JPRB, 0.70407E+00_JPRB, 0.70451E+00_JPRB ,&
&  0.71638E+00_JPRB, 0.73906E+00_JPRB, 0.73461E+00_JPRB, 0.72109E+00_JPRB, 0.72866E+00_JPRB, 0.72537E+00_JPRB, 0.68703E+00_JPRB ,&
&  0.75943E+00_JPRB, 0.80330E+00_JPRB, 0.85024E+00_JPRB, 0.94032E+00_JPRB, 0.10286E+01_JPRB, 0.10114E+01_JPRB, 0.10417E+01_JPRB ,&
&  0.10293E+01_JPRB, 0.10204E+01_JPRB, 0.97361E+00_JPRB, 0.92825E+00_JPRB, 0.91320E+00_JPRB, 0.91898E+00_JPRB, 0.95306E+00_JPRB ,&
&  0.96737E+00_JPRB, 0.97396E+00_JPRB, 0.93012E+00_JPRB, 0.86123E+00_JPRB, 0.82735E+00_JPRB, 0.79978E+00_JPRB, 0.78413E+00_JPRB ,&
&  0.78137E+00_JPRB, 0.80106E+00_JPRB, 0.83281E+00_JPRB, 0.94329E+00_JPRB, 0.11515E+01_JPRB, 0.12743E+01_JPRB, 0.11738E+01_JPRB ,&
&  0.11176E+01_JPRB, 0.13201E+01_JPRB, 0.17219E+01_JPRB, 0.18450E+01_JPRB, 0.20393E+01_JPRB, 0.22181E+01_JPRB, 0.22023E+01_JPRB ,&
&  0.21677E+01_JPRB, 0.24397E+01_JPRB, 0.25090E+01_JPRB, 0.28882E+01_JPRB, 0.31618E+01_JPRB, 0.25798E+01_JPRB, 0.32986E+01_JPRB ,&
&  0.33655E+01_JPRB, 0.32464E+01_JPRB, 0.25885E+01_JPRB, 0.18566E+01_JPRB, 0.20089E+01_JPRB, 0.14567E+01_JPRB, 0.90764E+00_JPRB ,&
&  0.79030E+00_JPRB, 0.73419E+00_JPRB, 0.69498E+00_JPRB, 0.72317E+00_JPRB, 0.72842E+00_JPRB, 0.90205E+00_JPRB, 0.11510E+01_JPRB ,&
&  0.15071E+01_JPRB, 0.16668E+01_JPRB, 0.17155E+01_JPRB, 0.17605E+01_JPRB, 0.18107E+01_JPRB, 0.24813E+01_JPRB, 0.24418E+01_JPRB ,&
&  0.20871E+01_JPRB, 0.19268E+01_JPRB, 0.17501E+01_JPRB, 0.15793E+01_JPRB, 0.14586E+01_JPRB, 0.13601E+01_JPRB, 0.12821E+01_JPRB ,&
&  0.12287E+01_JPRB, 0.11884E+01_JPRB, 0.11582E+01_JPRB, 0.11331E+01_JPRB, 0.11062E+01_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.10749E+01_JPRB, 0.10551E+01_JPRB, 0.10359E+01_JPRB, 0.10058E+01_JPRB, 0.97020E+00_JPRB, 0.92619E+00_JPRB, 0.86962E+00_JPRB ,&
&  0.80887E+00_JPRB, 0.76355E+00_JPRB, 0.73694E+00_JPRB, 0.71268E+00_JPRB, 0.69922E+00_JPRB, 0.69261E+00_JPRB, 0.70517E+00_JPRB ,&
&  0.72515E+00_JPRB, 0.75299E+00_JPRB, 0.79618E+00_JPRB, 0.81900E+00_JPRB, 0.90768E+00_JPRB, 0.91093E+00_JPRB, 0.91660E+00_JPRB ,&
&  0.95866E+00_JPRB, 0.96175E+00_JPRB, 0.98428E+00_JPRB, 0.10169E+01_JPRB, 0.10651E+01_JPRB, 0.10506E+01_JPRB, 0.10413E+01_JPRB ,&
&  0.10285E+01_JPRB, 0.10463E+01_JPRB, 0.10086E+01_JPRB, 0.97464E+00_JPRB, 0.95380E+00_JPRB, 0.97018E+00_JPRB, 0.97550E+00_JPRB ,&
&  0.98808E+00_JPRB, 0.98366E+00_JPRB, 0.94475E+00_JPRB, 0.88034E+00_JPRB, 0.84539E+00_JPRB, 0.82722E+00_JPRB, 0.83291E+00_JPRB ,&
&  0.84362E+00_JPRB, 0.89954E+00_JPRB, 0.97030E+00_JPRB, 0.11303E+01_JPRB, 0.11128E+01_JPRB, 0.94754E+00_JPRB, 0.10743E+01_JPRB ,&
&  0.11894E+01_JPRB, 0.15098E+01_JPRB, 0.16444E+01_JPRB, 0.18978E+01_JPRB, 0.19863E+01_JPRB, 0.19544E+01_JPRB, 0.21400E+01_JPRB ,&
&  0.24299E+01_JPRB, 0.34763E+01_JPRB, 0.27288E+01_JPRB, 0.27979E+01_JPRB, 0.44194E+01_JPRB, 0.35799E+01_JPRB, 0.33462E+01_JPRB ,&
&  0.38240E+01_JPRB, 0.32719E+01_JPRB, 0.30316E+01_JPRB, 0.25283E+01_JPRB, 0.39128E+01_JPRB, 0.40679E+01_JPRB, 0.12846E+01_JPRB ,&
&  0.89628E+00_JPRB, 0.78791E+00_JPRB, 0.72045E+00_JPRB, 0.66935E+00_JPRB, 0.71298E+00_JPRB, 0.93648E+00_JPRB, 0.13718E+01_JPRB ,&
&  0.15933E+01_JPRB, 0.18359E+01_JPRB, 0.18187E+01_JPRB, 0.18739E+01_JPRB, 0.18024E+01_JPRB, 0.20833E+01_JPRB, 0.19417E+01_JPRB ,&
&  0.18253E+01_JPRB, 0.17435E+01_JPRB, 0.15612E+01_JPRB, 0.14283E+01_JPRB, 0.13511E+01_JPRB, 0.12934E+01_JPRB, 0.12376E+01_JPRB ,&
&  0.11930E+01_JPRB, 0.11666E+01_JPRB, 0.11434E+01_JPRB, 0.11185E+01_JPRB, 0.10959E+01_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.10308E+01_JPRB, 0.10132E+01_JPRB, 0.98829E+00_JPRB, 0.94939E+00_JPRB, 0.92245E+00_JPRB, 0.88660E+00_JPRB, 0.82948E+00_JPRB ,&
&  0.77994E+00_JPRB, 0.75118E+00_JPRB, 0.72134E+00_JPRB, 0.69514E+00_JPRB, 0.69154E+00_JPRB, 0.69701E+00_JPRB, 0.72056E+00_JPRB ,&
&  0.75233E+00_JPRB, 0.79271E+00_JPRB, 0.84037E+00_JPRB, 0.96938E+00_JPRB, 0.11200E+01_JPRB, 0.11463E+01_JPRB, 0.13138E+01_JPRB ,&
&  0.12721E+01_JPRB, 0.12033E+01_JPRB, 0.11853E+01_JPRB, 0.11656E+01_JPRB, 0.11542E+01_JPRB, 0.11025E+01_JPRB, 0.10807E+01_JPRB ,&
&  0.10499E+01_JPRB, 0.10845E+01_JPRB, 0.10492E+01_JPRB, 0.10285E+01_JPRB, 0.10282E+01_JPRB, 0.10248E+01_JPRB, 0.10201E+01_JPRB ,&
&  0.10168E+01_JPRB, 0.10071E+01_JPRB, 0.97769E+00_JPRB, 0.93019E+00_JPRB, 0.89657E+00_JPRB, 0.88353E+00_JPRB, 0.90014E+00_JPRB ,&
&  0.95341E+00_JPRB, 0.10443E+01_JPRB, 0.11367E+01_JPRB, 0.12445E+01_JPRB, 0.10225E+01_JPRB, 0.11767E+01_JPRB, 0.12480E+01_JPRB ,&
&  0.14187E+01_JPRB, 0.15709E+01_JPRB, 0.15813E+01_JPRB, 0.17095E+01_JPRB, 0.19204E+01_JPRB, 0.19653E+01_JPRB, 0.22103E+01_JPRB ,&
&  0.29237E+01_JPRB, 0.29356E+01_JPRB, 0.26230E+01_JPRB, 0.29265E+01_JPRB, 0.39659E+01_JPRB, 0.58363E+01_JPRB, 0.30660E+01_JPRB ,&
&  0.24687E+01_JPRB, 0.32182E+01_JPRB, 0.31994E+01_JPRB, 0.30316E+01_JPRB, 0.42973E+01_JPRB, 0.55047E+01_JPRB, 0.35825E+01_JPRB ,&
&  0.10853E+01_JPRB, 0.79481E+00_JPRB, 0.86982E+00_JPRB, 0.79555E+00_JPRB, 0.76375E+00_JPRB, 0.98873E+00_JPRB, 0.21283E+01_JPRB ,&
&  0.17854E+01_JPRB, 0.19499E+01_JPRB, 0.19359E+01_JPRB, 0.18974E+01_JPRB, 0.18302E+01_JPRB, 0.18917E+01_JPRB, 0.18222E+01_JPRB ,&
&  0.15163E+01_JPRB, 0.14224E+01_JPRB, 0.12913E+01_JPRB, 0.12262E+01_JPRB, 0.11993E+01_JPRB, 0.11795E+01_JPRB, 0.11391E+01_JPRB ,&
&  0.11098E+01_JPRB, 0.10924E+01_JPRB, 0.10764E+01_JPRB, 0.10634E+01_JPRB, 0.10500E+01_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.97665E+00_JPRB, 0.95704E+00_JPRB, 0.92376E+00_JPRB, 0.90360E+00_JPRB, 0.90616E+00_JPRB, 0.87232E+00_JPRB, 0.83513E+00_JPRB ,&
&  0.78862E+00_JPRB, 0.76495E+00_JPRB, 0.73740E+00_JPRB, 0.72747E+00_JPRB, 0.71974E+00_JPRB, 0.73392E+00_JPRB, 0.76370E+00_JPRB ,&
&  0.80525E+00_JPRB, 0.85115E+00_JPRB, 0.91110E+00_JPRB, 0.10887E+01_JPRB, 0.13351E+01_JPRB, 0.13457E+01_JPRB, 0.15422E+01_JPRB ,&
&  0.17334E+01_JPRB, 0.15910E+01_JPRB, 0.14171E+01_JPRB, 0.13319E+01_JPRB, 0.12794E+01_JPRB, 0.12227E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.11133E+01_JPRB, 0.11625E+01_JPRB, 0.11225E+01_JPRB, 0.10915E+01_JPRB, 0.10584E+01_JPRB, 0.10585E+01_JPRB, 0.10788E+01_JPRB ,&
&  0.10650E+01_JPRB, 0.10723E+01_JPRB, 0.10455E+01_JPRB, 0.10197E+01_JPRB, 0.99549E+00_JPRB, 0.99249E+00_JPRB, 0.10524E+01_JPRB ,&
&  0.11600E+01_JPRB, 0.12714E+01_JPRB, 0.13378E+01_JPRB, 0.13191E+01_JPRB, 0.13326E+01_JPRB, 0.13576E+01_JPRB, 0.15125E+01_JPRB ,&
&  0.16460E+01_JPRB, 0.17267E+01_JPRB, 0.17675E+01_JPRB, 0.17902E+01_JPRB, 0.19703E+01_JPRB, 0.22069E+01_JPRB, 0.25433E+01_JPRB ,&
&  0.30796E+01_JPRB, 0.32390E+01_JPRB, 0.28773E+01_JPRB, 0.30611E+01_JPRB, 0.37136E+01_JPRB, 0.45315E+01_JPRB, 0.39790E+01_JPRB ,&
&  0.31313E+01_JPRB, 0.26065E+01_JPRB, 0.28577E+01_JPRB, 0.35606E+01_JPRB, 0.39961E+01_JPRB, 0.39274E+01_JPRB, 0.46545E+01_JPRB ,&
&  0.39235E+01_JPRB, 0.19021E+01_JPRB, 0.14995E+01_JPRB, 0.17353E+01_JPRB, 0.12776E+01_JPRB, 0.12890E+01_JPRB, 0.17012E+01_JPRB ,&
&  0.18093E+01_JPRB, 0.19819E+01_JPRB, 0.20019E+01_JPRB, 0.17576E+01_JPRB, 0.17635E+01_JPRB, 0.17487E+01_JPRB, 0.15606E+01_JPRB ,&
&  0.13368E+01_JPRB, 0.12229E+01_JPRB, 0.11424E+01_JPRB, 0.10923E+01_JPRB, 0.10634E+01_JPRB, 0.10330E+01_JPRB, 0.10142E+01_JPRB ,&
&  0.99307E+00_JPRB, 0.98190E+00_JPRB, 0.97901E+00_JPRB, 0.97815E+00_JPRB, 0.98303E+00_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.93251E+00_JPRB, 0.90892E+00_JPRB, 0.91286E+00_JPRB, 0.93405E+00_JPRB, 0.93222E+00_JPRB, 0.92164E+00_JPRB, 0.89321E+00_JPRB ,&
&  0.82930E+00_JPRB, 0.81799E+00_JPRB, 0.82530E+00_JPRB, 0.81773E+00_JPRB, 0.79980E+00_JPRB, 0.81073E+00_JPRB, 0.83878E+00_JPRB ,&
&  0.88277E+00_JPRB, 0.96570E+00_JPRB, 0.11076E+01_JPRB, 0.14120E+01_JPRB, 0.17050E+01_JPRB, 0.21038E+01_JPRB, 0.15819E+01_JPRB ,&
&  0.17495E+01_JPRB, 0.18139E+01_JPRB, 0.15525E+01_JPRB, 0.14658E+01_JPRB, 0.14178E+01_JPRB, 0.13879E+01_JPRB, 0.12802E+01_JPRB ,&
&  0.12557E+01_JPRB, 0.12642E+01_JPRB, 0.12101E+01_JPRB, 0.11285E+01_JPRB, 0.10912E+01_JPRB, 0.11562E+01_JPRB, 0.11586E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.11927E+01_JPRB, 0.11588E+01_JPRB, 0.11599E+01_JPRB, 0.11490E+01_JPRB, 0.11594E+01_JPRB, 0.12484E+01_JPRB ,&
&  0.13687E+01_JPRB, 0.14678E+01_JPRB, 0.15307E+01_JPRB, 0.14694E+01_JPRB, 0.15433E+01_JPRB, 0.16489E+01_JPRB, 0.18495E+01_JPRB ,&
&  0.18903E+01_JPRB, 0.17259E+01_JPRB, 0.17927E+01_JPRB, 0.19992E+01_JPRB, 0.22494E+01_JPRB, 0.23687E+01_JPRB, 0.26796E+01_JPRB ,&
&  0.33933E+01_JPRB, 0.39361E+01_JPRB, 0.40840E+01_JPRB, 0.34304E+01_JPRB, 0.34627E+01_JPRB, 0.40437E+01_JPRB, 0.42914E+01_JPRB ,&
&  0.39550E+01_JPRB, 0.34451E+01_JPRB, 0.35231E+01_JPRB, 0.34121E+01_JPRB, 0.32543E+01_JPRB, 0.30067E+01_JPRB, 0.31409E+01_JPRB ,&
&  0.35033E+01_JPRB, 0.37558E+01_JPRB, 0.27564E+01_JPRB, 0.21606E+01_JPRB, 0.20290E+01_JPRB, 0.17893E+01_JPRB, 0.17167E+01_JPRB ,&
&  0.19181E+01_JPRB, 0.19280E+01_JPRB, 0.19133E+01_JPRB, 0.21087E+01_JPRB, 0.18083E+01_JPRB, 0.15712E+01_JPRB, 0.12630E+01_JPRB ,&
&  0.10507E+01_JPRB, 0.98176E+00_JPRB, 0.94413E+00_JPRB, 0.92571E+00_JPRB, 0.94132E+00_JPRB, 0.94141E+00_JPRB, 0.94178E+00_JPRB ,&
&  0.92917E+00_JPRB, 0.93499E+00_JPRB, 0.94757E+00_JPRB, 0.95776E+00_JPRB, 0.95355E+00_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.91432E+00_JPRB, 0.92204E+00_JPRB, 0.94853E+00_JPRB, 0.97609E+00_JPRB, 0.98985E+00_JPRB, 0.10165E+01_JPRB, 0.10018E+01_JPRB ,&
&  0.92104E+00_JPRB, 0.87050E+00_JPRB, 0.88645E+00_JPRB, 0.87094E+00_JPRB, 0.85882E+00_JPRB, 0.87327E+00_JPRB, 0.91604E+00_JPRB ,&
&  0.99225E+00_JPRB, 0.11585E+01_JPRB, 0.14195E+01_JPRB, 0.17547E+01_JPRB, 0.19274E+01_JPRB, 0.20733E+01_JPRB, 0.20833E+01_JPRB ,&
&  0.15868E+01_JPRB, 0.19559E+01_JPRB, 0.15611E+01_JPRB, 0.15483E+01_JPRB, 0.15385E+01_JPRB, 0.16670E+01_JPRB, 0.15721E+01_JPRB ,&
&  0.14030E+01_JPRB, 0.13018E+01_JPRB, 0.13071E+01_JPRB, 0.12511E+01_JPRB, 0.12352E+01_JPRB, 0.12454E+01_JPRB, 0.12629E+01_JPRB ,&
&  0.12485E+01_JPRB, 0.12530E+01_JPRB, 0.12851E+01_JPRB, 0.13181E+01_JPRB, 0.13170E+01_JPRB, 0.13370E+01_JPRB, 0.13904E+01_JPRB ,&
&  0.15290E+01_JPRB, 0.16485E+01_JPRB, 0.16872E+01_JPRB, 0.17407E+01_JPRB, 0.17881E+01_JPRB, 0.20203E+01_JPRB, 0.22428E+01_JPRB ,&
&  0.22016E+01_JPRB, 0.19739E+01_JPRB, 0.20120E+01_JPRB, 0.21585E+01_JPRB, 0.22916E+01_JPRB, 0.26617E+01_JPRB, 0.29130E+01_JPRB ,&
&  0.36717E+01_JPRB, 0.36554E+01_JPRB, 0.43333E+01_JPRB, 0.33960E+01_JPRB, 0.32508E+01_JPRB, 0.37310E+01_JPRB, 0.34093E+01_JPRB ,&
&  0.28994E+01_JPRB, 0.31632E+01_JPRB, 0.31611E+01_JPRB, 0.31749E+01_JPRB, 0.30340E+01_JPRB, 0.27122E+01_JPRB, 0.25332E+01_JPRB ,&
&  0.25767E+01_JPRB, 0.27079E+01_JPRB, 0.25497E+01_JPRB, 0.21232E+01_JPRB, 0.19687E+01_JPRB, 0.17887E+01_JPRB, 0.18748E+01_JPRB ,&
&  0.21138E+01_JPRB, 0.22709E+01_JPRB, 0.23387E+01_JPRB, 0.22011E+01_JPRB, 0.14289E+01_JPRB, 0.11450E+01_JPRB, 0.97440E+00_JPRB ,&
&  0.89925E+00_JPRB, 0.86486E+00_JPRB, 0.87077E+00_JPRB, 0.89704E+00_JPRB, 0.92083E+00_JPRB, 0.93200E+00_JPRB, 0.92395E+00_JPRB ,&
&  0.92270E+00_JPRB, 0.92912E+00_JPRB, 0.93246E+00_JPRB, 0.92919E+00_JPRB, 0.92247E+00_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.94952E+00_JPRB, 0.97692E+00_JPRB, 0.10149E+01_JPRB, 0.10735E+01_JPRB, 0.11262E+01_JPRB, 0.12170E+01_JPRB, 0.12971E+01_JPRB ,&
&  0.11739E+01_JPRB, 0.84092E+00_JPRB, 0.86163E+00_JPRB, 0.88454E+00_JPRB, 0.91696E+00_JPRB, 0.96734E+00_JPRB, 0.10477E+01_JPRB ,&
&  0.11912E+01_JPRB, 0.13795E+01_JPRB, 0.15928E+01_JPRB, 0.17915E+01_JPRB, 0.19865E+01_JPRB, 0.21360E+01_JPRB, 0.21039E+01_JPRB ,&
&  0.17042E+01_JPRB, 0.15405E+01_JPRB, 0.17325E+01_JPRB, 0.17091E+01_JPRB, 0.17221E+01_JPRB, 0.17475E+01_JPRB, 0.17615E+01_JPRB ,&
&  0.15501E+01_JPRB, 0.14462E+01_JPRB, 0.14626E+01_JPRB, 0.14801E+01_JPRB, 0.13714E+01_JPRB, 0.14169E+01_JPRB, 0.14536E+01_JPRB ,&
&  0.14582E+01_JPRB, 0.14630E+01_JPRB, 0.15161E+01_JPRB, 0.15728E+01_JPRB, 0.15589E+01_JPRB, 0.15746E+01_JPRB, 0.16514E+01_JPRB ,&
&  0.18327E+01_JPRB, 0.19704E+01_JPRB, 0.20514E+01_JPRB, 0.20652E+01_JPRB, 0.22055E+01_JPRB, 0.24496E+01_JPRB, 0.25332E+01_JPRB ,&
&  0.24130E+01_JPRB, 0.23741E+01_JPRB, 0.24193E+01_JPRB, 0.25275E+01_JPRB, 0.24824E+01_JPRB, 0.29818E+01_JPRB, 0.31756E+01_JPRB ,&
&  0.34570E+01_JPRB, 0.34950E+01_JPRB, 0.40898E+01_JPRB, 0.30055E+01_JPRB, 0.31966E+01_JPRB, 0.33194E+01_JPRB, 0.29291E+01_JPRB ,&
&  0.25939E+01_JPRB, 0.27118E+01_JPRB, 0.26491E+01_JPRB, 0.26756E+01_JPRB, 0.26399E+01_JPRB, 0.22983E+01_JPRB, 0.23763E+01_JPRB ,&
&  0.22539E+01_JPRB, 0.24638E+01_JPRB, 0.23192E+01_JPRB, 0.22175E+01_JPRB, 0.19086E+01_JPRB, 0.18174E+01_JPRB, 0.18938E+01_JPRB ,&
&  0.21918E+01_JPRB, 0.23580E+01_JPRB, 0.21032E+01_JPRB, 0.16556E+01_JPRB, 0.10345E+01_JPRB, 0.86657E+00_JPRB, 0.84368E+00_JPRB ,&
&  0.84082E+00_JPRB, 0.86132E+00_JPRB, 0.88779E+00_JPRB, 0.91535E+00_JPRB, 0.93048E+00_JPRB, 0.93065E+00_JPRB, 0.92737E+00_JPRB ,&
&  0.93316E+00_JPRB, 0.93613E+00_JPRB, 0.93257E+00_JPRB, 0.93373E+00_JPRB, 0.93903E+00_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.96493E+00_JPRB, 0.97435E+00_JPRB, 0.98723E+00_JPRB, 0.98102E+00_JPRB, 0.96443E+00_JPRB, 0.93232E+00_JPRB, 0.86721E+00_JPRB ,&
&  0.80961E+00_JPRB, 0.78756E+00_JPRB, 0.82921E+00_JPRB, 0.88416E+00_JPRB, 0.94254E+00_JPRB, 0.10146E+01_JPRB, 0.11250E+01_JPRB ,&
&  0.12604E+01_JPRB, 0.13727E+01_JPRB, 0.14917E+01_JPRB, 0.16879E+01_JPRB, 0.19034E+01_JPRB, 0.19356E+01_JPRB, 0.19496E+01_JPRB ,&
&  0.17162E+01_JPRB, 0.13466E+01_JPRB, 0.16773E+01_JPRB, 0.19150E+01_JPRB, 0.19268E+01_JPRB, 0.19290E+01_JPRB, 0.18656E+01_JPRB ,&
&  0.16239E+01_JPRB, 0.16606E+01_JPRB, 0.17516E+01_JPRB, 0.17921E+01_JPRB, 0.16354E+01_JPRB, 0.16803E+01_JPRB, 0.17297E+01_JPRB ,&
&  0.17810E+01_JPRB, 0.18157E+01_JPRB, 0.19104E+01_JPRB, 0.19691E+01_JPRB, 0.19846E+01_JPRB, 0.20530E+01_JPRB, 0.21710E+01_JPRB ,&
&  0.22955E+01_JPRB, 0.23926E+01_JPRB, 0.25300E+01_JPRB, 0.26206E+01_JPRB, 0.27196E+01_JPRB, 0.28548E+01_JPRB, 0.28213E+01_JPRB ,&
&  0.27024E+01_JPRB, 0.25903E+01_JPRB, 0.27350E+01_JPRB, 0.30777E+01_JPRB, 0.33419E+01_JPRB, 0.30164E+01_JPRB, 0.33089E+01_JPRB ,&
&  0.32873E+01_JPRB, 0.32878E+01_JPRB, 0.32067E+01_JPRB, 0.36359E+01_JPRB, 0.25004E+01_JPRB, 0.27137E+01_JPRB, 0.23258E+01_JPRB ,&
&  0.24095E+01_JPRB, 0.22996E+01_JPRB, 0.22033E+01_JPRB, 0.21894E+01_JPRB, 0.22007E+01_JPRB, 0.19750E+01_JPRB, 0.20582E+01_JPRB ,&
&  0.21581E+01_JPRB, 0.22249E+01_JPRB, 0.22305E+01_JPRB, 0.22640E+01_JPRB, 0.21279E+01_JPRB, 0.21189E+01_JPRB, 0.21361E+01_JPRB ,&
&  0.23369E+01_JPRB, 0.19837E+01_JPRB, 0.15796E+01_JPRB, 0.11989E+01_JPRB, 0.90403E+00_JPRB, 0.78576E+00_JPRB, 0.82509E+00_JPRB ,&
&  0.85296E+00_JPRB, 0.88007E+00_JPRB, 0.89504E+00_JPRB, 0.91125E+00_JPRB, 0.92130E+00_JPRB, 0.92518E+00_JPRB, 0.94034E+00_JPRB ,&
&  0.94710E+00_JPRB, 0.95071E+00_JPRB, 0.94971E+00_JPRB, 0.95106E+00_JPRB, 0.95616E+00_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.95349E+00_JPRB, 0.94086E+00_JPRB, 0.91030E+00_JPRB, 0.85715E+00_JPRB, 0.79761E+00_JPRB, 0.73734E+00_JPRB, 0.70040E+00_JPRB ,&
&  0.66862E+00_JPRB, 0.69013E+00_JPRB, 0.74524E+00_JPRB, 0.82135E+00_JPRB, 0.91203E+00_JPRB, 0.10076E+01_JPRB, 0.11022E+01_JPRB ,&
&  0.11620E+01_JPRB, 0.12116E+01_JPRB, 0.13019E+01_JPRB, 0.15111E+01_JPRB, 0.17394E+01_JPRB, 0.18907E+01_JPRB, 0.19629E+01_JPRB ,&
&  0.20037E+01_JPRB, 0.20080E+01_JPRB, 0.19192E+01_JPRB, 0.16111E+01_JPRB, 0.14798E+01_JPRB, 0.18709E+01_JPRB, 0.17662E+01_JPRB ,&
&  0.17596E+01_JPRB, 0.17618E+01_JPRB, 0.18131E+01_JPRB, 0.18829E+01_JPRB, 0.18967E+01_JPRB, 0.19313E+01_JPRB, 0.20232E+01_JPRB ,&
&  0.21051E+01_JPRB, 0.22021E+01_JPRB, 0.22965E+01_JPRB, 0.23630E+01_JPRB, 0.24485E+01_JPRB, 0.25582E+01_JPRB, 0.26636E+01_JPRB ,&
&  0.27604E+01_JPRB, 0.29073E+01_JPRB, 0.30800E+01_JPRB, 0.32134E+01_JPRB, 0.31796E+01_JPRB, 0.32456E+01_JPRB, 0.31812E+01_JPRB ,&
&  0.30779E+01_JPRB, 0.29984E+01_JPRB, 0.30090E+01_JPRB, 0.32094E+01_JPRB, 0.32751E+01_JPRB, 0.28580E+01_JPRB, 0.28650E+01_JPRB ,&
&  0.30298E+01_JPRB, 0.30417E+01_JPRB, 0.26411E+01_JPRB, 0.33642E+01_JPRB, 0.19617E+01_JPRB, 0.23069E+01_JPRB, 0.23902E+01_JPRB ,&
&  0.22160E+01_JPRB, 0.20087E+01_JPRB, 0.19393E+01_JPRB, 0.19465E+01_JPRB, 0.19017E+01_JPRB, 0.18283E+01_JPRB, 0.19792E+01_JPRB ,&
&  0.20461E+01_JPRB, 0.20656E+01_JPRB, 0.21711E+01_JPRB, 0.21903E+01_JPRB, 0.21870E+01_JPRB, 0.20554E+01_JPRB, 0.18289E+01_JPRB ,&
&  0.15927E+01_JPRB, 0.16697E+01_JPRB, 0.14664E+01_JPRB, 0.13338E+01_JPRB, 0.10304E+01_JPRB, 0.78855E+00_JPRB, 0.83252E+00_JPRB ,&
&  0.85786E+00_JPRB, 0.86960E+00_JPRB, 0.89396E+00_JPRB, 0.91001E+00_JPRB, 0.90789E+00_JPRB, 0.91434E+00_JPRB, 0.93237E+00_JPRB ,&
&  0.93320E+00_JPRB, 0.93343E+00_JPRB, 0.93752E+00_JPRB, 0.94027E+00_JPRB, 0.94922E+00_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.80717E+00_JPRB, 0.78646E+00_JPRB, 0.75402E+00_JPRB, 0.72100E+00_JPRB, 0.68866E+00_JPRB, 0.64905E+00_JPRB, 0.64446E+00_JPRB ,&
&  0.62660E+00_JPRB, 0.63294E+00_JPRB, 0.65850E+00_JPRB, 0.68639E+00_JPRB, 0.73505E+00_JPRB, 0.81003E+00_JPRB, 0.88309E+00_JPRB ,&
&  0.92849E+00_JPRB, 0.94623E+00_JPRB, 0.10119E+01_JPRB, 0.11139E+01_JPRB, 0.12910E+01_JPRB, 0.15869E+01_JPRB, 0.20692E+01_JPRB ,&
&  0.23084E+01_JPRB, 0.24558E+01_JPRB, 0.24286E+01_JPRB, 0.22834E+01_JPRB, 0.19946E+01_JPRB, 0.20354E+01_JPRB, 0.22519E+01_JPRB ,&
&  0.23915E+01_JPRB, 0.22460E+01_JPRB, 0.20085E+01_JPRB, 0.20236E+01_JPRB, 0.20765E+01_JPRB, 0.21493E+01_JPRB, 0.22496E+01_JPRB ,&
&  0.23590E+01_JPRB, 0.24429E+01_JPRB, 0.25271E+01_JPRB, 0.26342E+01_JPRB, 0.27277E+01_JPRB, 0.28207E+01_JPRB, 0.29395E+01_JPRB ,&
&  0.30707E+01_JPRB, 0.31912E+01_JPRB, 0.32891E+01_JPRB, 0.33958E+01_JPRB, 0.35665E+01_JPRB, 0.37275E+01_JPRB, 0.37355E+01_JPRB ,&
&  0.36574E+01_JPRB, 0.34090E+01_JPRB, 0.34559E+01_JPRB, 0.36032E+01_JPRB, 0.34356E+01_JPRB, 0.30857E+01_JPRB, 0.29655E+01_JPRB ,&
&  0.30589E+01_JPRB, 0.31102E+01_JPRB, 0.21132E+01_JPRB, 0.26393E+01_JPRB, 0.28934E+01_JPRB, 0.24843E+01_JPRB, 0.21348E+01_JPRB ,&
&  0.19466E+01_JPRB, 0.17938E+01_JPRB, 0.18050E+01_JPRB, 0.17961E+01_JPRB, 0.19228E+01_JPRB, 0.18968E+01_JPRB, 0.16466E+01_JPRB ,&
&  0.16602E+01_JPRB, 0.16760E+01_JPRB, 0.18407E+01_JPRB, 0.19270E+01_JPRB, 0.18570E+01_JPRB, 0.16754E+01_JPRB, 0.13893E+01_JPRB ,&
&  0.12123E+01_JPRB, 0.14625E+01_JPRB, 0.13498E+01_JPRB, 0.14363E+01_JPRB, 0.12828E+01_JPRB, 0.77958E+00_JPRB, 0.82070E+00_JPRB ,&
&  0.83293E+00_JPRB, 0.84641E+00_JPRB, 0.86337E+00_JPRB, 0.86887E+00_JPRB, 0.87413E+00_JPRB, 0.87939E+00_JPRB, 0.88970E+00_JPRB ,&
&  0.88341E+00_JPRB, 0.86969E+00_JPRB, 0.85735E+00_JPRB, 0.84890E+00_JPRB, 0.83266E+00_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.73256E+00_JPRB, 0.72681E+00_JPRB, 0.72040E+00_JPRB, 0.72087E+00_JPRB, 0.72875E+00_JPRB, 0.72628E+00_JPRB, 0.71270E+00_JPRB ,&
&  0.68935E+00_JPRB, 0.67797E+00_JPRB, 0.65350E+00_JPRB, 0.62768E+00_JPRB, 0.63053E+00_JPRB, 0.68275E+00_JPRB, 0.73799E+00_JPRB ,&
&  0.77205E+00_JPRB, 0.78785E+00_JPRB, 0.78831E+00_JPRB, 0.79060E+00_JPRB, 0.82996E+00_JPRB, 0.86396E+00_JPRB, 0.10889E+01_JPRB ,&
&  0.15034E+01_JPRB, 0.20938E+01_JPRB, 0.25360E+01_JPRB, 0.27698E+01_JPRB, 0.26491E+01_JPRB, 0.22804E+01_JPRB, 0.26400E+01_JPRB ,&
&  0.24376E+01_JPRB, 0.19787E+01_JPRB, 0.18689E+01_JPRB, 0.18780E+01_JPRB, 0.19400E+01_JPRB, 0.20302E+01_JPRB, 0.21497E+01_JPRB ,&
&  0.22636E+01_JPRB, 0.23804E+01_JPRB, 0.25028E+01_JPRB, 0.26445E+01_JPRB, 0.27947E+01_JPRB, 0.29578E+01_JPRB, 0.31238E+01_JPRB ,&
&  0.33204E+01_JPRB, 0.34457E+01_JPRB, 0.36073E+01_JPRB, 0.34904E+01_JPRB, 0.38288E+01_JPRB, 0.41118E+01_JPRB, 0.42487E+01_JPRB ,&
&  0.41884E+01_JPRB, 0.40247E+01_JPRB, 0.41088E+01_JPRB, 0.40847E+01_JPRB, 0.40348E+01_JPRB, 0.38067E+01_JPRB, 0.34984E+01_JPRB ,&
&  0.35642E+01_JPRB, 0.33056E+01_JPRB, 0.21818E+01_JPRB, 0.21600E+01_JPRB, 0.23216E+01_JPRB, 0.18546E+01_JPRB, 0.19884E+01_JPRB ,&
&  0.17611E+01_JPRB, 0.16056E+01_JPRB, 0.14715E+01_JPRB, 0.14303E+01_JPRB, 0.15827E+01_JPRB, 0.17440E+01_JPRB, 0.14775E+01_JPRB ,&
&  0.12841E+01_JPRB, 0.12888E+01_JPRB, 0.13841E+01_JPRB, 0.14600E+01_JPRB, 0.14222E+01_JPRB, 0.13184E+01_JPRB, 0.12028E+01_JPRB ,&
&  0.12130E+01_JPRB, 0.11746E+01_JPRB, 0.10617E+01_JPRB, 0.10345E+01_JPRB, 0.80812E+00_JPRB, 0.73548E+00_JPRB, 0.75904E+00_JPRB ,&
&  0.76707E+00_JPRB, 0.75695E+00_JPRB, 0.76153E+00_JPRB, 0.76460E+00_JPRB, 0.77511E+00_JPRB, 0.76875E+00_JPRB, 0.77992E+00_JPRB ,&
&  0.78324E+00_JPRB, 0.77811E+00_JPRB, 0.77104E+00_JPRB, 0.77093E+00_JPRB, 0.75216E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.82482E+00_JPRB, 0.83365E+00_JPRB, 0.84987E+00_JPRB, 0.86410E+00_JPRB, 0.88608E+00_JPRB, 0.87445E+00_JPRB, 0.85967E+00_JPRB ,&
&  0.82546E+00_JPRB, 0.81824E+00_JPRB, 0.81019E+00_JPRB, 0.80593E+00_JPRB, 0.81360E+00_JPRB, 0.84074E+00_JPRB, 0.89156E+00_JPRB ,&
&  0.90741E+00_JPRB, 0.91243E+00_JPRB, 0.90297E+00_JPRB, 0.88920E+00_JPRB, 0.85025E+00_JPRB, 0.81429E+00_JPRB, 0.78750E+00_JPRB ,&
&  0.81436E+00_JPRB, 0.10327E+01_JPRB, 0.13219E+01_JPRB, 0.16016E+01_JPRB, 0.19356E+01_JPRB, 0.18491E+01_JPRB, 0.21662E+01_JPRB ,&
&  0.23989E+01_JPRB, 0.18306E+01_JPRB, 0.18858E+01_JPRB, 0.16854E+01_JPRB, 0.17125E+01_JPRB, 0.17385E+01_JPRB, 0.18504E+01_JPRB ,&
&  0.19906E+01_JPRB, 0.20980E+01_JPRB, 0.21887E+01_JPRB, 0.22874E+01_JPRB, 0.24089E+01_JPRB, 0.25209E+01_JPRB, 0.25938E+01_JPRB ,&
&  0.26541E+01_JPRB, 0.25881E+01_JPRB, 0.26701E+01_JPRB, 0.31964E+01_JPRB, 0.36734E+01_JPRB, 0.39296E+01_JPRB, 0.41222E+01_JPRB ,&
&  0.43786E+01_JPRB, 0.45386E+01_JPRB, 0.41421E+01_JPRB, 0.40796E+01_JPRB, 0.42932E+01_JPRB, 0.38511E+01_JPRB, 0.38353E+01_JPRB ,&
&  0.39674E+01_JPRB, 0.34170E+01_JPRB, 0.20211E+01_JPRB, 0.21320E+01_JPRB, 0.21678E+01_JPRB, 0.19886E+01_JPRB, 0.19661E+01_JPRB ,&
&  0.16817E+01_JPRB, 0.15025E+01_JPRB, 0.12891E+01_JPRB, 0.11996E+01_JPRB, 0.12104E+01_JPRB, 0.13241E+01_JPRB, 0.12965E+01_JPRB ,&
&  0.11200E+01_JPRB, 0.10815E+01_JPRB, 0.12034E+01_JPRB, 0.13437E+01_JPRB, 0.13707E+01_JPRB, 0.12711E+01_JPRB, 0.11211E+01_JPRB ,&
&  0.11815E+01_JPRB, 0.11645E+01_JPRB, 0.13686E+01_JPRB, 0.13459E+01_JPRB, 0.82261E+00_JPRB, 0.75558E+00_JPRB, 0.74315E+00_JPRB ,&
&  0.75493E+00_JPRB, 0.74982E+00_JPRB, 0.73963E+00_JPRB, 0.74582E+00_JPRB, 0.74450E+00_JPRB, 0.74072E+00_JPRB, 0.76445E+00_JPRB ,&
&  0.79528E+00_JPRB, 0.79662E+00_JPRB, 0.79904E+00_JPRB, 0.80250E+00_JPRB, 0.81203E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.93614E+00_JPRB, 0.95675E+00_JPRB, 0.98225E+00_JPRB, 0.98760E+00_JPRB, 0.99492E+00_JPRB, 0.98809E+00_JPRB, 0.98286E+00_JPRB ,&
&  0.96832E+00_JPRB, 0.96459E+00_JPRB, 0.96771E+00_JPRB, 0.97307E+00_JPRB, 0.98536E+00_JPRB, 0.10005E+01_JPRB, 0.10344E+01_JPRB ,&
&  0.10358E+01_JPRB, 0.10420E+01_JPRB, 0.10504E+01_JPRB, 0.10674E+01_JPRB, 0.10829E+01_JPRB, 0.10778E+01_JPRB, 0.10476E+01_JPRB ,&
&  0.10275E+01_JPRB, 0.10245E+01_JPRB, 0.10389E+01_JPRB, 0.10709E+01_JPRB, 0.12500E+01_JPRB, 0.15803E+01_JPRB, 0.21185E+01_JPRB ,&
&  0.21121E+01_JPRB, 0.18585E+01_JPRB, 0.16719E+01_JPRB, 0.14509E+01_JPRB, 0.14331E+01_JPRB, 0.14665E+01_JPRB, 0.14075E+01_JPRB ,&
&  0.14020E+01_JPRB, 0.13848E+01_JPRB, 0.14159E+01_JPRB, 0.14838E+01_JPRB, 0.15406E+01_JPRB, 0.15270E+01_JPRB, 0.14996E+01_JPRB ,&
&  0.13773E+01_JPRB, 0.13405E+01_JPRB, 0.14568E+01_JPRB, 0.17657E+01_JPRB, 0.24119E+01_JPRB, 0.28601E+01_JPRB, 0.27313E+01_JPRB ,&
&  0.30390E+01_JPRB, 0.35936E+01_JPRB, 0.37532E+01_JPRB, 0.37604E+01_JPRB, 0.36003E+01_JPRB, 0.33676E+01_JPRB, 0.32197E+01_JPRB ,&
&  0.31545E+01_JPRB, 0.27986E+01_JPRB, 0.20760E+01_JPRB, 0.21602E+01_JPRB, 0.20450E+01_JPRB, 0.20574E+01_JPRB, 0.18351E+01_JPRB ,&
&  0.15899E+01_JPRB, 0.14294E+01_JPRB, 0.12985E+01_JPRB, 0.11737E+01_JPRB, 0.11010E+01_JPRB, 0.11346E+01_JPRB, 0.11035E+01_JPRB ,&
&  0.10344E+01_JPRB, 0.10592E+01_JPRB, 0.10847E+01_JPRB, 0.11386E+01_JPRB, 0.13756E+01_JPRB, 0.12363E+01_JPRB, 0.13308E+01_JPRB ,&
&  0.11838E+01_JPRB, 0.11773E+01_JPRB, 0.96201E+00_JPRB, 0.13907E+01_JPRB, 0.84087E+00_JPRB, 0.70857E+00_JPRB, 0.73120E+00_JPRB ,&
&  0.77186E+00_JPRB, 0.79132E+00_JPRB, 0.81874E+00_JPRB, 0.84293E+00_JPRB, 0.84315E+00_JPRB, 0.84209E+00_JPRB, 0.86062E+00_JPRB ,&
&  0.88022E+00_JPRB, 0.89010E+00_JPRB, 0.90048E+00_JPRB, 0.90518E+00_JPRB, 0.92023E+00_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.10091E+01_JPRB, 0.10285E+01_JPRB, 0.10449E+01_JPRB, 0.10572E+01_JPRB, 0.10631E+01_JPRB, 0.10674E+01_JPRB, 0.10719E+01_JPRB ,&
&  0.10727E+01_JPRB, 0.10709E+01_JPRB, 0.10701E+01_JPRB, 0.10863E+01_JPRB, 0.11101E+01_JPRB, 0.11215E+01_JPRB, 0.11487E+01_JPRB ,&
&  0.11446E+01_JPRB, 0.11607E+01_JPRB, 0.11845E+01_JPRB, 0.12035E+01_JPRB, 0.12334E+01_JPRB, 0.12344E+01_JPRB, 0.12184E+01_JPRB ,&
&  0.12141E+01_JPRB, 0.12042E+01_JPRB, 0.12075E+01_JPRB, 0.12131E+01_JPRB, 0.12297E+01_JPRB, 0.12554E+01_JPRB, 0.14494E+01_JPRB ,&
&  0.15895E+01_JPRB, 0.19788E+01_JPRB, 0.18518E+01_JPRB, 0.16721E+01_JPRB, 0.16764E+01_JPRB, 0.15250E+01_JPRB, 0.14438E+01_JPRB ,&
&  0.13224E+01_JPRB, 0.12075E+01_JPRB, 0.11712E+01_JPRB, 0.11680E+01_JPRB, 0.12313E+01_JPRB, 0.13059E+01_JPRB, 0.13600E+01_JPRB ,&
&  0.14663E+01_JPRB, 0.15537E+01_JPRB, 0.16655E+01_JPRB, 0.17487E+01_JPRB, 0.18208E+01_JPRB, 0.20522E+01_JPRB, 0.19538E+01_JPRB ,&
&  0.20281E+01_JPRB, 0.22858E+01_JPRB, 0.24296E+01_JPRB, 0.25346E+01_JPRB, 0.23739E+01_JPRB, 0.21933E+01_JPRB, 0.19590E+01_JPRB ,&
&  0.17308E+01_JPRB, 0.16718E+01_JPRB, 0.18396E+01_JPRB, 0.18259E+01_JPRB, 0.17948E+01_JPRB, 0.18162E+01_JPRB, 0.16430E+01_JPRB ,&
&  0.14793E+01_JPRB, 0.13604E+01_JPRB, 0.12410E+01_JPRB, 0.11319E+01_JPRB, 0.10730E+01_JPRB, 0.10586E+01_JPRB, 0.10237E+01_JPRB ,&
&  0.98265E+00_JPRB, 0.97505E+00_JPRB, 0.99537E+00_JPRB, 0.10442E+01_JPRB, 0.10993E+01_JPRB, 0.17211E+01_JPRB, 0.21242E+01_JPRB ,&
&  0.13395E+01_JPRB, 0.10717E+01_JPRB, 0.70089E+00_JPRB, 0.10083E+01_JPRB, 0.73558E+00_JPRB, 0.68974E+00_JPRB, 0.75631E+00_JPRB ,&
&  0.85521E+00_JPRB, 0.93093E+00_JPRB, 0.96321E+00_JPRB, 0.99947E+00_JPRB, 0.90712E+00_JPRB, 0.89367E+00_JPRB, 0.89437E+00_JPRB ,&
&  0.90764E+00_JPRB, 0.92777E+00_JPRB, 0.94716E+00_JPRB, 0.96585E+00_JPRB, 0.98810E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.10475E+01_JPRB, 0.10643E+01_JPRB, 0.10799E+01_JPRB, 0.10990E+01_JPRB, 0.11160E+01_JPRB, 0.11369E+01_JPRB, 0.11504E+01_JPRB ,&
&  0.11623E+01_JPRB, 0.11706E+01_JPRB, 0.11723E+01_JPRB, 0.11868E+01_JPRB, 0.12019E+01_JPRB, 0.12296E+01_JPRB, 0.12621E+01_JPRB ,&
&  0.12995E+01_JPRB, 0.13396E+01_JPRB, 0.13852E+01_JPRB, 0.14138E+01_JPRB, 0.14378E+01_JPRB, 0.14632E+01_JPRB, 0.14867E+01_JPRB ,&
&  0.15022E+01_JPRB, 0.15061E+01_JPRB, 0.15290E+01_JPRB, 0.15724E+01_JPRB, 0.16234E+01_JPRB, 0.17009E+01_JPRB, 0.15157E+01_JPRB ,&
&  0.18545E+01_JPRB, 0.20788E+01_JPRB, 0.20948E+01_JPRB, 0.19742E+01_JPRB, 0.19615E+01_JPRB, 0.19605E+01_JPRB, 0.19982E+01_JPRB ,&
&  0.20737E+01_JPRB, 0.20075E+01_JPRB, 0.19154E+01_JPRB, 0.17645E+01_JPRB, 0.16862E+01_JPRB, 0.17273E+01_JPRB, 0.18299E+01_JPRB ,&
&  0.18949E+01_JPRB, 0.19686E+01_JPRB, 0.19893E+01_JPRB, 0.19930E+01_JPRB, 0.20068E+01_JPRB, 0.20046E+01_JPRB, 0.18918E+01_JPRB ,&
&  0.18000E+01_JPRB, 0.17737E+01_JPRB, 0.15423E+01_JPRB, 0.15582E+01_JPRB, 0.15543E+01_JPRB, 0.14194E+01_JPRB, 0.12136E+01_JPRB ,&
&  0.11619E+01_JPRB, 0.10957E+01_JPRB, 0.11675E+01_JPRB, 0.15155E+01_JPRB, 0.16063E+01_JPRB, 0.15735E+01_JPRB, 0.14781E+01_JPRB ,&
&  0.13368E+01_JPRB, 0.12013E+01_JPRB, 0.10541E+01_JPRB, 0.96829E+00_JPRB, 0.94499E+00_JPRB, 0.97597E+00_JPRB, 0.98647E+00_JPRB ,&
&  0.97518E+00_JPRB, 0.95631E+00_JPRB, 0.95243E+00_JPRB, 0.96104E+00_JPRB, 0.10855E+01_JPRB, 0.12998E+01_JPRB, 0.19254E+01_JPRB ,&
&  0.13865E+01_JPRB, 0.83235E+00_JPRB, 0.79581E+00_JPRB, 0.85631E+00_JPRB, 0.68877E+00_JPRB, 0.64859E+00_JPRB, 0.74996E+00_JPRB ,&
&  0.96013E+00_JPRB, 0.10863E+01_JPRB, 0.10787E+01_JPRB, 0.10574E+01_JPRB, 0.10076E+01_JPRB, 0.10116E+01_JPRB, 0.99580E+00_JPRB ,&
&  0.98679E+00_JPRB, 0.99256E+00_JPRB, 0.10022E+01_JPRB, 0.10128E+01_JPRB, 0.10291E+01_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.10673E+01_JPRB, 0.10876E+01_JPRB, 0.11060E+01_JPRB, 0.11239E+01_JPRB, 0.11380E+01_JPRB, 0.11609E+01_JPRB, 0.11710E+01_JPRB ,&
&  0.11843E+01_JPRB, 0.11927E+01_JPRB, 0.12108E+01_JPRB, 0.12335E+01_JPRB, 0.12570E+01_JPRB, 0.12879E+01_JPRB, 0.13255E+01_JPRB ,&
&  0.13723E+01_JPRB, 0.14095E+01_JPRB, 0.14520E+01_JPRB, 0.14794E+01_JPRB, 0.14829E+01_JPRB, 0.15003E+01_JPRB, 0.15162E+01_JPRB ,&
&  0.15140E+01_JPRB, 0.15099E+01_JPRB, 0.15343E+01_JPRB, 0.15720E+01_JPRB, 0.15853E+01_JPRB, 0.16291E+01_JPRB, 0.13214E+01_JPRB ,&
&  0.19676E+01_JPRB, 0.22667E+01_JPRB, 0.23861E+01_JPRB, 0.22770E+01_JPRB, 0.22347E+01_JPRB, 0.23178E+01_JPRB, 0.24207E+01_JPRB ,&
&  0.25531E+01_JPRB, 0.26749E+01_JPRB, 0.26271E+01_JPRB, 0.24900E+01_JPRB, 0.23487E+01_JPRB, 0.23741E+01_JPRB, 0.24249E+01_JPRB ,&
&  0.24728E+01_JPRB, 0.25097E+01_JPRB, 0.25031E+01_JPRB, 0.24868E+01_JPRB, 0.24762E+01_JPRB, 0.24445E+01_JPRB, 0.23662E+01_JPRB ,&
&  0.22855E+01_JPRB, 0.21905E+01_JPRB, 0.19258E+01_JPRB, 0.20962E+01_JPRB, 0.19562E+01_JPRB, 0.16289E+01_JPRB, 0.13067E+01_JPRB ,&
&  0.11398E+01_JPRB, 0.10919E+01_JPRB, 0.12867E+01_JPRB, 0.13835E+01_JPRB, 0.13471E+01_JPRB, 0.13073E+01_JPRB, 0.12378E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.96832E+00_JPRB, 0.88776E+00_JPRB, 0.86490E+00_JPRB, 0.87855E+00_JPRB, 0.91100E+00_JPRB, 0.92708E+00_JPRB ,&
&  0.92748E+00_JPRB, 0.92705E+00_JPRB, 0.91580E+00_JPRB, 0.90218E+00_JPRB, 0.94268E+00_JPRB, 0.10170E+01_JPRB, 0.15685E+01_JPRB ,&
&  0.12626E+01_JPRB, 0.92189E+00_JPRB, 0.11732E+01_JPRB, 0.54653E+00_JPRB, 0.61693E+00_JPRB, 0.59120E+00_JPRB, 0.68197E+00_JPRB ,&
&  0.81850E+00_JPRB, 0.11514E+01_JPRB, 0.15681E+01_JPRB, 0.13776E+01_JPRB, 0.12562E+01_JPRB, 0.11523E+01_JPRB, 0.10648E+01_JPRB ,&
&  0.10335E+01_JPRB, 0.10270E+01_JPRB, 0.10297E+01_JPRB, 0.10316E+01_JPRB, 0.10461E+01_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.10953E+01_JPRB, 0.11188E+01_JPRB, 0.11249E+01_JPRB, 0.11453E+01_JPRB, 0.11284E+01_JPRB, 0.11486E+01_JPRB, 0.11527E+01_JPRB ,&
&  0.11744E+01_JPRB, 0.11898E+01_JPRB, 0.12135E+01_JPRB, 0.12371E+01_JPRB, 0.12708E+01_JPRB, 0.13030E+01_JPRB, 0.13331E+01_JPRB ,&
&  0.13710E+01_JPRB, 0.14135E+01_JPRB, 0.14489E+01_JPRB, 0.14894E+01_JPRB, 0.14920E+01_JPRB, 0.15182E+01_JPRB, 0.15377E+01_JPRB ,&
&  0.15867E+01_JPRB, 0.15907E+01_JPRB, 0.16111E+01_JPRB, 0.16323E+01_JPRB, 0.16322E+01_JPRB, 0.16854E+01_JPRB, 0.12788E+01_JPRB ,&
&  0.20818E+01_JPRB, 0.23953E+01_JPRB, 0.25939E+01_JPRB, 0.25275E+01_JPRB, 0.24458E+01_JPRB, 0.24548E+01_JPRB, 0.25057E+01_JPRB ,&
&  0.26278E+01_JPRB, 0.27316E+01_JPRB, 0.27345E+01_JPRB, 0.29309E+01_JPRB, 0.30259E+01_JPRB, 0.29770E+01_JPRB, 0.30758E+01_JPRB ,&
&  0.31572E+01_JPRB, 0.32301E+01_JPRB, 0.32851E+01_JPRB, 0.33423E+01_JPRB, 0.33924E+01_JPRB, 0.34907E+01_JPRB, 0.35532E+01_JPRB ,&
&  0.35340E+01_JPRB, 0.35344E+01_JPRB, 0.35128E+01_JPRB, 0.33289E+01_JPRB, 0.31292E+01_JPRB, 0.26141E+01_JPRB, 0.21150E+01_JPRB ,&
&  0.14455E+01_JPRB, 0.12001E+01_JPRB, 0.12403E+01_JPRB, 0.12874E+01_JPRB, 0.12125E+01_JPRB, 0.11444E+01_JPRB, 0.10321E+01_JPRB ,&
&  0.90135E+00_JPRB, 0.86065E+00_JPRB, 0.83666E+00_JPRB, 0.84406E+00_JPRB, 0.86813E+00_JPRB, 0.88942E+00_JPRB, 0.90528E+00_JPRB ,&
&  0.91079E+00_JPRB, 0.91294E+00_JPRB, 0.88095E+00_JPRB, 0.84916E+00_JPRB, 0.87889E+00_JPRB, 0.10096E+01_JPRB, 0.10502E+01_JPRB ,&
&  0.12195E+01_JPRB, 0.95481E+00_JPRB, 0.85323E+00_JPRB, 0.78319E+00_JPRB, 0.63884E+00_JPRB, 0.63258E+00_JPRB, 0.79416E+00_JPRB ,&
&  0.12936E+01_JPRB, 0.16624E+01_JPRB, 0.23668E+01_JPRB, 0.40014E+01_JPRB, 0.28355E+01_JPRB, 0.21171E+01_JPRB, 0.11238E+01_JPRB ,&
&  0.10638E+01_JPRB, 0.10475E+01_JPRB, 0.10416E+01_JPRB, 0.10382E+01_JPRB, 0.10644E+01_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.10056E+01_JPRB, 0.10364E+01_JPRB, 0.10695E+01_JPRB, 0.11004E+01_JPRB, 0.11162E+01_JPRB, 0.11232E+01_JPRB, 0.11261E+01_JPRB ,&
&  0.11534E+01_JPRB, 0.11385E+01_JPRB, 0.11656E+01_JPRB, 0.11496E+01_JPRB, 0.11741E+01_JPRB, 0.11708E+01_JPRB, 0.11901E+01_JPRB ,&
&  0.12201E+01_JPRB, 0.12632E+01_JPRB, 0.13142E+01_JPRB, 0.13486E+01_JPRB, 0.13741E+01_JPRB, 0.14350E+01_JPRB, 0.15136E+01_JPRB ,&
&  0.16080E+01_JPRB, 0.16653E+01_JPRB, 0.17260E+01_JPRB, 0.17677E+01_JPRB, 0.17543E+01_JPRB, 0.17386E+01_JPRB, 0.12942E+01_JPRB ,&
&  0.19972E+01_JPRB, 0.24328E+01_JPRB, 0.27547E+01_JPRB, 0.27772E+01_JPRB, 0.26455E+01_JPRB, 0.25175E+01_JPRB, 0.25549E+01_JPRB ,&
&  0.26227E+01_JPRB, 0.26438E+01_JPRB, 0.26577E+01_JPRB, 0.27654E+01_JPRB, 0.31622E+01_JPRB, 0.31195E+01_JPRB, 0.32243E+01_JPRB ,&
&  0.33393E+01_JPRB, 0.34723E+01_JPRB, 0.36414E+01_JPRB, 0.38413E+01_JPRB, 0.40379E+01_JPRB, 0.42279E+01_JPRB, 0.44306E+01_JPRB ,&
&  0.45904E+01_JPRB, 0.46544E+01_JPRB, 0.47018E+01_JPRB, 0.41790E+01_JPRB, 0.38395E+01_JPRB, 0.38615E+01_JPRB, 0.28436E+01_JPRB ,&
&  0.16298E+01_JPRB, 0.13038E+01_JPRB, 0.13104E+01_JPRB, 0.12949E+01_JPRB, 0.12202E+01_JPRB, 0.10809E+01_JPRB, 0.97035E+00_JPRB ,&
&  0.89017E+00_JPRB, 0.86206E+00_JPRB, 0.84888E+00_JPRB, 0.87601E+00_JPRB, 0.89397E+00_JPRB, 0.91429E+00_JPRB, 0.92316E+00_JPRB ,&
&  0.93730E+00_JPRB, 0.91602E+00_JPRB, 0.87712E+00_JPRB, 0.85909E+00_JPRB, 0.90488E+00_JPRB, 0.10653E+01_JPRB, 0.13249E+01_JPRB ,&
&  0.14350E+01_JPRB, 0.86940E+00_JPRB, 0.84910E+00_JPRB, 0.76912E+00_JPRB, 0.73623E+00_JPRB, 0.77886E+00_JPRB, 0.95090E+00_JPRB ,&
&  0.14120E+01_JPRB, 0.21579E+01_JPRB, 0.24074E+01_JPRB, 0.20633E+01_JPRB, 0.25252E+01_JPRB, 0.18279E+01_JPRB, 0.10434E+01_JPRB ,&
&  0.10286E+01_JPRB, 0.10028E+01_JPRB, 0.96849E+00_JPRB, 0.96052E+00_JPRB, 0.97311E+00_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.83822E+00_JPRB, 0.85918E+00_JPRB, 0.88538E+00_JPRB, 0.92476E+00_JPRB, 0.96873E+00_JPRB, 0.99977E+00_JPRB, 0.10317E+01_JPRB ,&
&  0.10444E+01_JPRB, 0.10657E+01_JPRB, 0.10863E+01_JPRB, 0.10568E+01_JPRB, 0.10750E+01_JPRB, 0.10698E+01_JPRB, 0.10740E+01_JPRB ,&
&  0.10835E+01_JPRB, 0.11107E+01_JPRB, 0.11450E+01_JPRB, 0.11841E+01_JPRB, 0.12419E+01_JPRB, 0.13121E+01_JPRB, 0.13989E+01_JPRB ,&
&  0.15057E+01_JPRB, 0.16041E+01_JPRB, 0.17128E+01_JPRB, 0.17868E+01_JPRB, 0.18539E+01_JPRB, 0.18940E+01_JPRB, 0.18916E+01_JPRB ,&
&  0.12196E+01_JPRB, 0.21768E+01_JPRB, 0.24758E+01_JPRB, 0.26531E+01_JPRB, 0.27849E+01_JPRB, 0.26609E+01_JPRB, 0.26845E+01_JPRB ,&
&  0.26029E+01_JPRB, 0.24542E+01_JPRB, 0.25197E+01_JPRB, 0.26736E+01_JPRB, 0.28766E+01_JPRB, 0.28559E+01_JPRB, 0.28991E+01_JPRB ,&
&  0.29709E+01_JPRB, 0.31101E+01_JPRB, 0.32992E+01_JPRB, 0.35198E+01_JPRB, 0.37846E+01_JPRB, 0.40954E+01_JPRB, 0.44678E+01_JPRB ,&
&  0.48585E+01_JPRB, 0.52118E+01_JPRB, 0.55244E+01_JPRB, 0.49022E+01_JPRB, 0.43310E+01_JPRB, 0.44495E+01_JPRB, 0.36856E+01_JPRB ,&
&  0.20437E+01_JPRB, 0.13469E+01_JPRB, 0.13662E+01_JPRB, 0.14565E+01_JPRB, 0.13310E+01_JPRB, 0.12096E+01_JPRB, 0.10713E+01_JPRB ,&
&  0.98154E+00_JPRB, 0.93184E+00_JPRB, 0.95915E+00_JPRB, 0.93791E+00_JPRB, 0.97426E+00_JPRB, 0.97793E+00_JPRB, 0.99309E+00_JPRB ,&
&  0.10106E+01_JPRB, 0.99076E+00_JPRB, 0.97072E+00_JPRB, 0.95839E+00_JPRB, 0.93898E+00_JPRB, 0.92334E+00_JPRB, 0.91686E+00_JPRB ,&
&  0.92594E+00_JPRB, 0.87934E+00_JPRB, 0.84763E+00_JPRB, 0.85536E+00_JPRB, 0.86526E+00_JPRB, 0.88493E+00_JPRB, 0.97446E+00_JPRB ,&
&  0.11257E+01_JPRB, 0.12954E+01_JPRB, 0.15798E+01_JPRB, 0.16634E+01_JPRB, 0.15934E+01_JPRB, 0.12140E+01_JPRB, 0.90711E+00_JPRB ,&
&  0.90531E+00_JPRB, 0.86453E+00_JPRB, 0.83182E+00_JPRB, 0.81405E+00_JPRB, 0.82016E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.75819E+00_JPRB, 0.77456E+00_JPRB, 0.76916E+00_JPRB, 0.79610E+00_JPRB, 0.81719E+00_JPRB, 0.84818E+00_JPRB, 0.88186E+00_JPRB ,&
&  0.91006E+00_JPRB, 0.93577E+00_JPRB, 0.97251E+00_JPRB, 0.96185E+00_JPRB, 0.98239E+00_JPRB, 0.98459E+00_JPRB, 0.97269E+00_JPRB ,&
&  0.97834E+00_JPRB, 0.99739E+00_JPRB, 0.10169E+01_JPRB, 0.10387E+01_JPRB, 0.10697E+01_JPRB, 0.11248E+01_JPRB, 0.11891E+01_JPRB ,&
&  0.12696E+01_JPRB, 0.13709E+01_JPRB, 0.15056E+01_JPRB, 0.16312E+01_JPRB, 0.17484E+01_JPRB, 0.17930E+01_JPRB, 0.17353E+01_JPRB ,&
&  0.86188E+00_JPRB, 0.12022E+01_JPRB, 0.20320E+01_JPRB, 0.23439E+01_JPRB, 0.25112E+01_JPRB, 0.25979E+01_JPRB, 0.26053E+01_JPRB ,&
&  0.24502E+01_JPRB, 0.22153E+01_JPRB, 0.22507E+01_JPRB, 0.25905E+01_JPRB, 0.25531E+01_JPRB, 0.25548E+01_JPRB, 0.25605E+01_JPRB ,&
&  0.25932E+01_JPRB, 0.26729E+01_JPRB, 0.28023E+01_JPRB, 0.29737E+01_JPRB, 0.32429E+01_JPRB, 0.35925E+01_JPRB, 0.40423E+01_JPRB ,&
&  0.45905E+01_JPRB, 0.51599E+01_JPRB, 0.56710E+01_JPRB, 0.50167E+01_JPRB, 0.49061E+01_JPRB, 0.48794E+01_JPRB, 0.50637E+01_JPRB ,&
&  0.26458E+01_JPRB, 0.15140E+01_JPRB, 0.15503E+01_JPRB, 0.16500E+01_JPRB, 0.14644E+01_JPRB, 0.13757E+01_JPRB, 0.12955E+01_JPRB ,&
&  0.11898E+01_JPRB, 0.11093E+01_JPRB, 0.10883E+01_JPRB, 0.10581E+01_JPRB, 0.10486E+01_JPRB, 0.10663E+01_JPRB, 0.10989E+01_JPRB ,&
&  0.11045E+01_JPRB, 0.11091E+01_JPRB, 0.11070E+01_JPRB, 0.11042E+01_JPRB, 0.10658E+01_JPRB, 0.10269E+01_JPRB, 0.10088E+01_JPRB ,&
&  0.10052E+01_JPRB, 0.98031E+00_JPRB, 0.98242E+00_JPRB, 0.10142E+01_JPRB, 0.10441E+01_JPRB, 0.10563E+01_JPRB, 0.10651E+01_JPRB ,&
&  0.10458E+01_JPRB, 0.10533E+01_JPRB, 0.11681E+01_JPRB, 0.12067E+01_JPRB, 0.11866E+01_JPRB, 0.10035E+01_JPRB, 0.83656E+00_JPRB ,&
&  0.81854E+00_JPRB, 0.79191E+00_JPRB, 0.76724E+00_JPRB, 0.76019E+00_JPRB, 0.75378E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.69754E+00_JPRB, 0.68116E+00_JPRB, 0.66582E+00_JPRB, 0.67344E+00_JPRB, 0.68405E+00_JPRB, 0.70313E+00_JPRB, 0.75650E+00_JPRB ,&
&  0.80106E+00_JPRB, 0.81817E+00_JPRB, 0.84208E+00_JPRB, 0.84857E+00_JPRB, 0.86581E+00_JPRB, 0.86271E+00_JPRB, 0.87369E+00_JPRB ,&
&  0.87236E+00_JPRB, 0.88710E+00_JPRB, 0.91105E+00_JPRB, 0.93337E+00_JPRB, 0.95033E+00_JPRB, 0.97073E+00_JPRB, 0.10013E+01_JPRB ,&
&  0.10379E+01_JPRB, 0.10900E+01_JPRB, 0.11770E+01_JPRB, 0.13163E+01_JPRB, 0.15300E+01_JPRB, 0.17818E+01_JPRB, 0.19565E+01_JPRB ,&
&  0.14934E+01_JPRB, 0.67179E+00_JPRB, 0.13476E+01_JPRB, 0.20485E+01_JPRB, 0.21629E+01_JPRB, 0.22755E+01_JPRB, 0.22305E+01_JPRB ,&
&  0.20766E+01_JPRB, 0.19629E+01_JPRB, 0.19247E+01_JPRB, 0.22501E+01_JPRB, 0.23000E+01_JPRB, 0.23139E+01_JPRB, 0.23115E+01_JPRB ,&
&  0.22993E+01_JPRB, 0.23233E+01_JPRB, 0.23728E+01_JPRB, 0.24761E+01_JPRB, 0.26794E+01_JPRB, 0.30036E+01_JPRB, 0.34294E+01_JPRB ,&
&  0.40037E+01_JPRB, 0.47890E+01_JPRB, 0.55786E+01_JPRB, 0.51866E+01_JPRB, 0.53520E+01_JPRB, 0.47430E+01_JPRB, 0.40665E+01_JPRB ,&
&  0.24212E+01_JPRB, 0.16779E+01_JPRB, 0.16807E+01_JPRB, 0.18313E+01_JPRB, 0.16374E+01_JPRB, 0.14163E+01_JPRB, 0.14767E+01_JPRB ,&
&  0.13576E+01_JPRB, 0.12791E+01_JPRB, 0.12280E+01_JPRB, 0.11835E+01_JPRB, 0.11657E+01_JPRB, 0.11596E+01_JPRB, 0.11584E+01_JPRB ,&
&  0.11540E+01_JPRB, 0.11666E+01_JPRB, 0.11849E+01_JPRB, 0.11810E+01_JPRB, 0.11522E+01_JPRB, 0.11338E+01_JPRB, 0.11363E+01_JPRB ,&
&  0.11437E+01_JPRB, 0.11325E+01_JPRB, 0.11445E+01_JPRB, 0.11601E+01_JPRB, 0.11684E+01_JPRB, 0.11358E+01_JPRB, 0.11310E+01_JPRB ,&
&  0.11122E+01_JPRB, 0.11025E+01_JPRB, 0.10801E+01_JPRB, 0.10417E+01_JPRB, 0.10143E+01_JPRB, 0.93629E+00_JPRB, 0.84438E+00_JPRB ,&
&  0.81378E+00_JPRB, 0.77999E+00_JPRB, 0.75630E+00_JPRB, 0.73841E+00_JPRB, 0.71952E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.75175E+00_JPRB, 0.73438E+00_JPRB, 0.70153E+00_JPRB, 0.65641E+00_JPRB, 0.64950E+00_JPRB, 0.64591E+00_JPRB, 0.65749E+00_JPRB ,&
&  0.70204E+00_JPRB, 0.75702E+00_JPRB, 0.76881E+00_JPRB, 0.76741E+00_JPRB, 0.74670E+00_JPRB, 0.74909E+00_JPRB, 0.75693E+00_JPRB ,&
&  0.77280E+00_JPRB, 0.78666E+00_JPRB, 0.81074E+00_JPRB, 0.84124E+00_JPRB, 0.87008E+00_JPRB, 0.89262E+00_JPRB, 0.90180E+00_JPRB ,&
&  0.91233E+00_JPRB, 0.91639E+00_JPRB, 0.93900E+00_JPRB, 0.97926E+00_JPRB, 0.10658E+01_JPRB, 0.12451E+01_JPRB, 0.15435E+01_JPRB ,&
&  0.22439E+01_JPRB, 0.18475E+01_JPRB, 0.89913E+00_JPRB, 0.16912E+01_JPRB, 0.19908E+01_JPRB, 0.21750E+01_JPRB, 0.18893E+01_JPRB ,&
&  0.18269E+01_JPRB, 0.16508E+01_JPRB, 0.15665E+01_JPRB, 0.17694E+01_JPRB, 0.18432E+01_JPRB, 0.19120E+01_JPRB, 0.19286E+01_JPRB ,&
&  0.18751E+01_JPRB, 0.18963E+01_JPRB, 0.19349E+01_JPRB, 0.19931E+01_JPRB, 0.20982E+01_JPRB, 0.23362E+01_JPRB, 0.27635E+01_JPRB ,&
&  0.33065E+01_JPRB, 0.40436E+01_JPRB, 0.51574E+01_JPRB, 0.56763E+01_JPRB, 0.62155E+01_JPRB, 0.46247E+01_JPRB, 0.35441E+01_JPRB ,&
&  0.22742E+01_JPRB, 0.20204E+01_JPRB, 0.20865E+01_JPRB, 0.20605E+01_JPRB, 0.18545E+01_JPRB, 0.16660E+01_JPRB, 0.17194E+01_JPRB ,&
&  0.15594E+01_JPRB, 0.14508E+01_JPRB, 0.13927E+01_JPRB, 0.13369E+01_JPRB, 0.12956E+01_JPRB, 0.12716E+01_JPRB, 0.12522E+01_JPRB ,&
&  0.12325E+01_JPRB, 0.12376E+01_JPRB, 0.12481E+01_JPRB, 0.12576E+01_JPRB, 0.12567E+01_JPRB, 0.12407E+01_JPRB, 0.12429E+01_JPRB ,&
&  0.12431E+01_JPRB, 0.12447E+01_JPRB, 0.12482E+01_JPRB, 0.12616E+01_JPRB, 0.12215E+01_JPRB, 0.11640E+01_JPRB, 0.11689E+01_JPRB ,&
&  0.11571E+01_JPRB, 0.11632E+01_JPRB, 0.11348E+01_JPRB, 0.11315E+01_JPRB, 0.10601E+01_JPRB, 0.97039E+00_JPRB, 0.88641E+00_JPRB ,&
&  0.87227E+00_JPRB, 0.85932E+00_JPRB, 0.81608E+00_JPRB, 0.76505E+00_JPRB, 0.75004E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.82661E+00_JPRB, 0.83413E+00_JPRB, 0.82634E+00_JPRB, 0.77903E+00_JPRB, 0.73451E+00_JPRB, 0.69125E+00_JPRB, 0.66145E+00_JPRB ,&
&  0.65465E+00_JPRB, 0.65947E+00_JPRB, 0.68868E+00_JPRB, 0.71330E+00_JPRB, 0.70588E+00_JPRB, 0.68323E+00_JPRB, 0.67587E+00_JPRB ,&
&  0.68332E+00_JPRB, 0.70614E+00_JPRB, 0.72519E+00_JPRB, 0.75598E+00_JPRB, 0.78515E+00_JPRB, 0.80735E+00_JPRB, 0.83053E+00_JPRB ,&
&  0.82123E+00_JPRB, 0.82233E+00_JPRB, 0.83274E+00_JPRB, 0.84382E+00_JPRB, 0.86771E+00_JPRB, 0.92639E+00_JPRB, 0.99925E+00_JPRB ,&
&  0.12518E+01_JPRB, 0.20433E+01_JPRB, 0.12080E+01_JPRB, 0.16997E+01_JPRB, 0.20373E+01_JPRB, 0.20890E+01_JPRB, 0.18798E+01_JPRB ,&
&  0.17071E+01_JPRB, 0.14802E+01_JPRB, 0.14337E+01_JPRB, 0.14661E+01_JPRB, 0.13716E+01_JPRB, 0.14020E+01_JPRB, 0.14350E+01_JPRB ,&
&  0.14530E+01_JPRB, 0.14993E+01_JPRB, 0.15405E+01_JPRB, 0.15788E+01_JPRB, 0.16050E+01_JPRB, 0.16880E+01_JPRB, 0.19403E+01_JPRB ,&
&  0.24223E+01_JPRB, 0.31128E+01_JPRB, 0.42948E+01_JPRB, 0.53869E+01_JPRB, 0.66216E+01_JPRB, 0.51251E+01_JPRB, 0.37483E+01_JPRB ,&
&  0.27015E+01_JPRB, 0.25321E+01_JPRB, 0.25198E+01_JPRB, 0.24657E+01_JPRB, 0.21948E+01_JPRB, 0.21523E+01_JPRB, 0.21794E+01_JPRB ,&
&  0.19197E+01_JPRB, 0.17097E+01_JPRB, 0.16029E+01_JPRB, 0.15303E+01_JPRB, 0.14549E+01_JPRB, 0.13993E+01_JPRB, 0.13624E+01_JPRB ,&
&  0.13326E+01_JPRB, 0.13202E+01_JPRB, 0.13328E+01_JPRB, 0.13443E+01_JPRB, 0.13558E+01_JPRB, 0.13489E+01_JPRB, 0.13355E+01_JPRB ,&
&  0.13053E+01_JPRB, 0.12872E+01_JPRB, 0.12792E+01_JPRB, 0.12694E+01_JPRB, 0.12344E+01_JPRB, 0.11901E+01_JPRB, 0.11805E+01_JPRB ,&
&  0.11800E+01_JPRB, 0.12015E+01_JPRB, 0.11918E+01_JPRB, 0.11514E+01_JPRB, 0.11651E+01_JPRB, 0.10883E+01_JPRB, 0.10028E+01_JPRB ,&
&  0.10181E+01_JPRB, 0.11536E+01_JPRB, 0.11972E+01_JPRB, 0.84807E+00_JPRB, 0.82750E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.87369E+00_JPRB, 0.87123E+00_JPRB, 0.85535E+00_JPRB, 0.84984E+00_JPRB, 0.81421E+00_JPRB, 0.77132E+00_JPRB, 0.73098E+00_JPRB ,&
&  0.70284E+00_JPRB, 0.68538E+00_JPRB, 0.68435E+00_JPRB, 0.68172E+00_JPRB, 0.68195E+00_JPRB, 0.67392E+00_JPRB, 0.66936E+00_JPRB ,&
&  0.65724E+00_JPRB, 0.66322E+00_JPRB, 0.67973E+00_JPRB, 0.70796E+00_JPRB, 0.74130E+00_JPRB, 0.75247E+00_JPRB, 0.76264E+00_JPRB ,&
&  0.76577E+00_JPRB, 0.76249E+00_JPRB, 0.76266E+00_JPRB, 0.77109E+00_JPRB, 0.78994E+00_JPRB, 0.81411E+00_JPRB, 0.85250E+00_JPRB ,&
&  0.94443E+00_JPRB, 0.17524E+01_JPRB, 0.26301E+01_JPRB, 0.29884E+01_JPRB, 0.26280E+01_JPRB, 0.22237E+01_JPRB, 0.19955E+01_JPRB ,&
&  0.18860E+01_JPRB, 0.17136E+01_JPRB, 0.16667E+01_JPRB, 0.14406E+01_JPRB, 0.12256E+01_JPRB, 0.11521E+01_JPRB, 0.11693E+01_JPRB ,&
&  0.12450E+01_JPRB, 0.12792E+01_JPRB, 0.12964E+01_JPRB, 0.12933E+01_JPRB, 0.12653E+01_JPRB, 0.12654E+01_JPRB, 0.13377E+01_JPRB ,&
&  0.15612E+01_JPRB, 0.20408E+01_JPRB, 0.29198E+01_JPRB, 0.44492E+01_JPRB, 0.60976E+01_JPRB, 0.54511E+01_JPRB, 0.43694E+01_JPRB ,&
&  0.39944E+01_JPRB, 0.36018E+01_JPRB, 0.32498E+01_JPRB, 0.30550E+01_JPRB, 0.24401E+01_JPRB, 0.29279E+01_JPRB, 0.27380E+01_JPRB ,&
&  0.25047E+01_JPRB, 0.23427E+01_JPRB, 0.21184E+01_JPRB, 0.18826E+01_JPRB, 0.17096E+01_JPRB, 0.15545E+01_JPRB, 0.14399E+01_JPRB ,&
&  0.14026E+01_JPRB, 0.14086E+01_JPRB, 0.14278E+01_JPRB, 0.14622E+01_JPRB, 0.14989E+01_JPRB, 0.15067E+01_JPRB, 0.14709E+01_JPRB ,&
&  0.13896E+01_JPRB, 0.13190E+01_JPRB, 0.12227E+01_JPRB, 0.12050E+01_JPRB, 0.11882E+01_JPRB, 0.11460E+01_JPRB, 0.11204E+01_JPRB ,&
&  0.11395E+01_JPRB, 0.11842E+01_JPRB, 0.11982E+01_JPRB, 0.11807E+01_JPRB, 0.11581E+01_JPRB, 0.11444E+01_JPRB, 0.10568E+01_JPRB ,&
&  0.10138E+01_JPRB, 0.10558E+01_JPRB, 0.10032E+01_JPRB, 0.92785E+00_JPRB, 0.89892E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.91422E+00_JPRB, 0.88200E+00_JPRB, 0.86140E+00_JPRB, 0.85968E+00_JPRB, 0.83811E+00_JPRB, 0.82133E+00_JPRB, 0.81204E+00_JPRB ,&
&  0.76670E+00_JPRB, 0.73176E+00_JPRB, 0.70196E+00_JPRB, 0.68977E+00_JPRB, 0.68804E+00_JPRB, 0.68139E+00_JPRB, 0.68556E+00_JPRB ,&
&  0.67812E+00_JPRB, 0.66570E+00_JPRB, 0.66556E+00_JPRB, 0.68507E+00_JPRB, 0.71437E+00_JPRB, 0.72304E+00_JPRB, 0.73814E+00_JPRB ,&
&  0.75358E+00_JPRB, 0.75030E+00_JPRB, 0.73932E+00_JPRB, 0.73736E+00_JPRB, 0.74136E+00_JPRB, 0.73962E+00_JPRB, 0.76119E+00_JPRB ,&
&  0.85251E+00_JPRB, 0.18033E+01_JPRB, 0.19148E+01_JPRB, 0.33405E+01_JPRB, 0.29165E+01_JPRB, 0.24936E+01_JPRB, 0.19536E+01_JPRB ,&
&  0.19280E+01_JPRB, 0.21690E+01_JPRB, 0.18302E+01_JPRB, 0.15700E+01_JPRB, 0.13186E+01_JPRB, 0.11309E+01_JPRB, 0.10850E+01_JPRB ,&
&  0.11268E+01_JPRB, 0.11724E+01_JPRB, 0.11911E+01_JPRB, 0.11455E+01_JPRB, 0.10913E+01_JPRB, 0.10715E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.11485E+01_JPRB, 0.13101E+01_JPRB, 0.17612E+01_JPRB, 0.26893E+01_JPRB, 0.47204E+01_JPRB, 0.56554E+01_JPRB, 0.51352E+01_JPRB ,&
&  0.49010E+01_JPRB, 0.50258E+01_JPRB, 0.45376E+01_JPRB, 0.39944E+01_JPRB, 0.32820E+01_JPRB, 0.32673E+01_JPRB, 0.30750E+01_JPRB ,&
&  0.28678E+01_JPRB, 0.28028E+01_JPRB, 0.26953E+01_JPRB, 0.24744E+01_JPRB, 0.22069E+01_JPRB, 0.19329E+01_JPRB, 0.16837E+01_JPRB ,&
&  0.15119E+01_JPRB, 0.14360E+01_JPRB, 0.14087E+01_JPRB, 0.14547E+01_JPRB, 0.15397E+01_JPRB, 0.15966E+01_JPRB, 0.16037E+01_JPRB ,&
&  0.15356E+01_JPRB, 0.14293E+01_JPRB, 0.12502E+01_JPRB, 0.11605E+01_JPRB, 0.11418E+01_JPRB, 0.11002E+01_JPRB, 0.10806E+01_JPRB ,&
&  0.11241E+01_JPRB, 0.11636E+01_JPRB, 0.11692E+01_JPRB, 0.11498E+01_JPRB, 0.11433E+01_JPRB, 0.11759E+01_JPRB, 0.11072E+01_JPRB ,&
&  0.10612E+01_JPRB, 0.10711E+01_JPRB, 0.10717E+01_JPRB, 0.10064E+01_JPRB, 0.96336E+00_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.97126E+00_JPRB, 0.92051E+00_JPRB, 0.88195E+00_JPRB, 0.86426E+00_JPRB, 0.85558E+00_JPRB, 0.84436E+00_JPRB, 0.83313E+00_JPRB ,&
&  0.80586E+00_JPRB, 0.77199E+00_JPRB, 0.74137E+00_JPRB, 0.71586E+00_JPRB, 0.71119E+00_JPRB, 0.70390E+00_JPRB, 0.69809E+00_JPRB ,&
&  0.68805E+00_JPRB, 0.67761E+00_JPRB, 0.67140E+00_JPRB, 0.67779E+00_JPRB, 0.69213E+00_JPRB, 0.70352E+00_JPRB, 0.71947E+00_JPRB ,&
&  0.72616E+00_JPRB, 0.73488E+00_JPRB, 0.73595E+00_JPRB, 0.73089E+00_JPRB, 0.72189E+00_JPRB, 0.70944E+00_JPRB, 0.69901E+00_JPRB ,&
&  0.82883E+00_JPRB, 0.20971E+01_JPRB, 0.16863E+01_JPRB, 0.30248E+01_JPRB, 0.26514E+01_JPRB, 0.23993E+01_JPRB, 0.19086E+01_JPRB ,&
&  0.17997E+01_JPRB, 0.19812E+01_JPRB, 0.18096E+01_JPRB, 0.16261E+01_JPRB, 0.14601E+01_JPRB, 0.12919E+01_JPRB, 0.11788E+01_JPRB ,&
&  0.11401E+01_JPRB, 0.11335E+01_JPRB, 0.11170E+01_JPRB, 0.10769E+01_JPRB, 0.10454E+01_JPRB, 0.10196E+01_JPRB, 0.10182E+01_JPRB ,&
&  0.10322E+01_JPRB, 0.10783E+01_JPRB, 0.12497E+01_JPRB, 0.18288E+01_JPRB, 0.31110E+01_JPRB, 0.47743E+01_JPRB, 0.58881E+01_JPRB ,&
&  0.60477E+01_JPRB, 0.65636E+01_JPRB, 0.58552E+01_JPRB, 0.49833E+01_JPRB, 0.42892E+01_JPRB, 0.38781E+01_JPRB, 0.35119E+01_JPRB ,&
&  0.31383E+01_JPRB, 0.29132E+01_JPRB, 0.28416E+01_JPRB, 0.27903E+01_JPRB, 0.26786E+01_JPRB, 0.24778E+01_JPRB, 0.22163E+01_JPRB ,&
&  0.19415E+01_JPRB, 0.17460E+01_JPRB, 0.16144E+01_JPRB, 0.15597E+01_JPRB, 0.15663E+01_JPRB, 0.16058E+01_JPRB, 0.16232E+01_JPRB ,&
&  0.15614E+01_JPRB, 0.14782E+01_JPRB, 0.13190E+01_JPRB, 0.11966E+01_JPRB, 0.11599E+01_JPRB, 0.11150E+01_JPRB, 0.10844E+01_JPRB ,&
&  0.11280E+01_JPRB, 0.11591E+01_JPRB, 0.11498E+01_JPRB, 0.11367E+01_JPRB, 0.11125E+01_JPRB, 0.11819E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.10795E+01_JPRB, 0.10688E+01_JPRB, 0.10634E+01_JPRB, 0.10395E+01_JPRB, 0.10169E+01_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.10110E+01_JPRB, 0.97335E+00_JPRB, 0.93491E+00_JPRB, 0.90577E+00_JPRB, 0.88176E+00_JPRB, 0.86794E+00_JPRB, 0.85485E+00_JPRB ,&
&  0.82754E+00_JPRB, 0.79868E+00_JPRB, 0.77764E+00_JPRB, 0.76104E+00_JPRB, 0.74755E+00_JPRB, 0.73835E+00_JPRB, 0.72236E+00_JPRB ,&
&  0.70420E+00_JPRB, 0.68001E+00_JPRB, 0.68406E+00_JPRB, 0.69465E+00_JPRB, 0.70071E+00_JPRB, 0.70675E+00_JPRB, 0.72715E+00_JPRB ,&
&  0.72216E+00_JPRB, 0.72106E+00_JPRB, 0.72334E+00_JPRB, 0.72438E+00_JPRB, 0.71346E+00_JPRB, 0.71186E+00_JPRB, 0.68934E+00_JPRB ,&
&  0.79885E+00_JPRB, 0.93455E+00_JPRB, 0.18833E+01_JPRB, 0.25913E+01_JPRB, 0.23862E+01_JPRB, 0.21478E+01_JPRB, 0.19339E+01_JPRB ,&
&  0.18965E+01_JPRB, 0.18170E+01_JPRB, 0.16466E+01_JPRB, 0.15343E+01_JPRB, 0.14245E+01_JPRB, 0.13347E+01_JPRB, 0.12618E+01_JPRB ,&
&  0.12079E+01_JPRB, 0.11476E+01_JPRB, 0.11007E+01_JPRB, 0.10506E+01_JPRB, 0.10163E+01_JPRB, 0.99463E+00_JPRB, 0.98868E+00_JPRB ,&
&  0.98659E+00_JPRB, 0.98961E+00_JPRB, 0.10404E+01_JPRB, 0.13388E+01_JPRB, 0.24038E+01_JPRB, 0.38172E+01_JPRB, 0.54126E+01_JPRB ,&
&  0.66076E+01_JPRB, 0.70790E+01_JPRB, 0.64515E+01_JPRB, 0.56699E+01_JPRB, 0.49020E+01_JPRB, 0.43148E+01_JPRB, 0.38613E+01_JPRB ,&
&  0.34505E+01_JPRB, 0.30957E+01_JPRB, 0.28211E+01_JPRB, 0.26756E+01_JPRB, 0.25901E+01_JPRB, 0.25235E+01_JPRB, 0.24376E+01_JPRB ,&
&  0.23086E+01_JPRB, 0.21244E+01_JPRB, 0.19330E+01_JPRB, 0.17794E+01_JPRB, 0.16865E+01_JPRB, 0.16516E+01_JPRB, 0.16232E+01_JPRB ,&
&  0.15418E+01_JPRB, 0.14583E+01_JPRB, 0.13361E+01_JPRB, 0.12526E+01_JPRB, 0.12368E+01_JPRB, 0.12051E+01_JPRB, 0.11741E+01_JPRB ,&
&  0.11667E+01_JPRB, 0.11729E+01_JPRB, 0.11568E+01_JPRB, 0.11493E+01_JPRB, 0.11065E+01_JPRB, 0.11785E+01_JPRB, 0.11351E+01_JPRB ,&
&  0.10922E+01_JPRB, 0.10770E+01_JPRB, 0.10540E+01_JPRB, 0.10387E+01_JPRB, 0.10317E+01_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.10418E+01_JPRB, 0.10224E+01_JPRB, 0.97933E+00_JPRB, 0.93752E+00_JPRB, 0.90760E+00_JPRB, 0.88640E+00_JPRB, 0.86798E+00_JPRB ,&
&  0.84568E+00_JPRB, 0.81722E+00_JPRB, 0.79439E+00_JPRB, 0.78314E+00_JPRB, 0.78008E+00_JPRB, 0.77472E+00_JPRB, 0.75623E+00_JPRB ,&
&  0.73215E+00_JPRB, 0.71223E+00_JPRB, 0.71148E+00_JPRB, 0.71699E+00_JPRB, 0.72344E+00_JPRB, 0.71362E+00_JPRB, 0.72313E+00_JPRB ,&
&  0.72830E+00_JPRB, 0.72504E+00_JPRB, 0.72188E+00_JPRB, 0.71842E+00_JPRB, 0.71262E+00_JPRB, 0.70761E+00_JPRB, 0.69020E+00_JPRB ,&
&  0.75637E+00_JPRB, 0.11444E+01_JPRB, 0.17805E+01_JPRB, 0.22192E+01_JPRB, 0.22034E+01_JPRB, 0.19507E+01_JPRB, 0.17981E+01_JPRB ,&
&  0.16719E+01_JPRB, 0.15983E+01_JPRB, 0.14649E+01_JPRB, 0.13743E+01_JPRB, 0.12933E+01_JPRB, 0.12305E+01_JPRB, 0.11789E+01_JPRB ,&
&  0.11428E+01_JPRB, 0.11086E+01_JPRB, 0.10671E+01_JPRB, 0.10278E+01_JPRB, 0.99792E+00_JPRB, 0.97346E+00_JPRB, 0.96152E+00_JPRB ,&
&  0.95753E+00_JPRB, 0.95100E+00_JPRB, 0.96083E+00_JPRB, 0.10832E+01_JPRB, 0.18311E+01_JPRB, 0.23951E+01_JPRB, 0.42144E+01_JPRB ,&
&  0.56852E+01_JPRB, 0.54018E+01_JPRB, 0.51571E+01_JPRB, 0.47327E+01_JPRB, 0.41952E+01_JPRB, 0.37463E+01_JPRB, 0.32812E+01_JPRB ,&
&  0.28819E+01_JPRB, 0.25681E+01_JPRB, 0.23123E+01_JPRB, 0.21382E+01_JPRB, 0.20314E+01_JPRB, 0.19786E+01_JPRB, 0.19608E+01_JPRB ,&
&  0.19506E+01_JPRB, 0.19345E+01_JPRB, 0.18986E+01_JPRB, 0.18353E+01_JPRB, 0.17438E+01_JPRB, 0.16477E+01_JPRB, 0.15524E+01_JPRB ,&
&  0.14533E+01_JPRB, 0.13883E+01_JPRB, 0.13074E+01_JPRB, 0.12567E+01_JPRB, 0.12748E+01_JPRB, 0.13058E+01_JPRB, 0.12785E+01_JPRB ,&
&  0.12164E+01_JPRB, 0.11894E+01_JPRB, 0.11821E+01_JPRB, 0.11398E+01_JPRB, 0.10926E+01_JPRB, 0.11800E+01_JPRB, 0.11566E+01_JPRB ,&
&  0.11171E+01_JPRB, 0.10907E+01_JPRB, 0.10678E+01_JPRB, 0.10427E+01_JPRB, 0.10493E+01_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.10671E+01_JPRB, 0.10289E+01_JPRB, 0.99139E+00_JPRB, 0.95175E+00_JPRB, 0.92325E+00_JPRB, 0.89289E+00_JPRB, 0.86361E+00_JPRB ,&
&  0.83904E+00_JPRB, 0.82459E+00_JPRB, 0.81166E+00_JPRB, 0.79588E+00_JPRB, 0.79438E+00_JPRB, 0.78768E+00_JPRB, 0.77127E+00_JPRB ,&
&  0.74642E+00_JPRB, 0.72784E+00_JPRB, 0.72005E+00_JPRB, 0.71881E+00_JPRB, 0.72619E+00_JPRB, 0.72837E+00_JPRB, 0.73433E+00_JPRB ,&
&  0.73940E+00_JPRB, 0.72168E+00_JPRB, 0.71025E+00_JPRB, 0.70943E+00_JPRB, 0.70800E+00_JPRB, 0.70910E+00_JPRB, 0.69733E+00_JPRB ,&
&  0.73548E+00_JPRB, 0.94823E+00_JPRB, 0.13177E+01_JPRB, 0.17996E+01_JPRB, 0.18638E+01_JPRB, 0.17369E+01_JPRB, 0.16120E+01_JPRB ,&
&  0.15050E+01_JPRB, 0.14185E+01_JPRB, 0.13093E+01_JPRB, 0.11864E+01_JPRB, 0.10915E+01_JPRB, 0.10330E+01_JPRB, 0.99851E+00_JPRB ,&
&  0.98540E+00_JPRB, 0.97450E+00_JPRB, 0.95782E+00_JPRB, 0.94239E+00_JPRB, 0.92613E+00_JPRB, 0.91180E+00_JPRB, 0.90474E+00_JPRB ,&
&  0.89865E+00_JPRB, 0.89202E+00_JPRB, 0.89138E+00_JPRB, 0.92031E+00_JPRB, 0.12403E+01_JPRB, 0.15804E+01_JPRB, 0.23935E+01_JPRB ,&
&  0.27549E+01_JPRB, 0.29217E+01_JPRB, 0.30745E+01_JPRB, 0.30415E+01_JPRB, 0.28553E+01_JPRB, 0.26635E+01_JPRB, 0.24621E+01_JPRB ,&
&  0.22445E+01_JPRB, 0.20261E+01_JPRB, 0.18750E+01_JPRB, 0.17645E+01_JPRB, 0.16619E+01_JPRB, 0.15861E+01_JPRB, 0.15412E+01_JPRB ,&
&  0.15337E+01_JPRB, 0.15627E+01_JPRB, 0.15969E+01_JPRB, 0.16199E+01_JPRB, 0.16023E+01_JPRB, 0.15578E+01_JPRB, 0.14856E+01_JPRB ,&
&  0.13877E+01_JPRB, 0.13088E+01_JPRB, 0.12683E+01_JPRB, 0.12586E+01_JPRB, 0.12678E+01_JPRB, 0.12736E+01_JPRB, 0.12789E+01_JPRB ,&
&  0.12750E+01_JPRB, 0.12459E+01_JPRB, 0.12162E+01_JPRB, 0.11748E+01_JPRB, 0.11313E+01_JPRB, 0.11955E+01_JPRB, 0.11779E+01_JPRB ,&
&  0.11467E+01_JPRB, 0.11136E+01_JPRB, 0.10898E+01_JPRB, 0.10627E+01_JPRB, 0.10933E+01_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.11325E+01_JPRB, 0.10659E+01_JPRB, 0.10129E+01_JPRB, 0.97398E+00_JPRB, 0.94837E+00_JPRB, 0.91564E+00_JPRB, 0.87662E+00_JPRB ,&
&  0.84735E+00_JPRB, 0.83607E+00_JPRB, 0.82405E+00_JPRB, 0.80341E+00_JPRB, 0.78784E+00_JPRB, 0.77248E+00_JPRB, 0.75872E+00_JPRB ,&
&  0.73839E+00_JPRB, 0.72884E+00_JPRB, 0.72068E+00_JPRB, 0.71960E+00_JPRB, 0.71603E+00_JPRB, 0.71694E+00_JPRB, 0.72580E+00_JPRB ,&
&  0.72867E+00_JPRB, 0.71056E+00_JPRB, 0.70770E+00_JPRB, 0.70632E+00_JPRB, 0.70344E+00_JPRB, 0.70140E+00_JPRB, 0.69238E+00_JPRB ,&
&  0.73667E+00_JPRB, 0.80759E+00_JPRB, 0.10331E+01_JPRB, 0.12614E+01_JPRB, 0.13698E+01_JPRB, 0.14250E+01_JPRB, 0.13916E+01_JPRB ,&
&  0.13318E+01_JPRB, 0.12495E+01_JPRB, 0.11584E+01_JPRB, 0.10676E+01_JPRB, 0.98350E+00_JPRB, 0.92627E+00_JPRB, 0.89298E+00_JPRB ,&
&  0.87407E+00_JPRB, 0.86573E+00_JPRB, 0.85728E+00_JPRB, 0.85142E+00_JPRB, 0.84527E+00_JPRB, 0.83610E+00_JPRB, 0.83421E+00_JPRB ,&
&  0.83856E+00_JPRB, 0.82876E+00_JPRB, 0.82605E+00_JPRB, 0.84292E+00_JPRB, 0.90008E+00_JPRB, 0.10358E+01_JPRB, 0.14015E+01_JPRB ,&
&  0.16375E+01_JPRB, 0.18182E+01_JPRB, 0.20106E+01_JPRB, 0.20871E+01_JPRB, 0.20372E+01_JPRB, 0.19386E+01_JPRB, 0.17997E+01_JPRB ,&
&  0.16896E+01_JPRB, 0.16106E+01_JPRB, 0.15382E+01_JPRB, 0.14655E+01_JPRB, 0.14026E+01_JPRB, 0.13553E+01_JPRB, 0.13230E+01_JPRB ,&
&  0.13007E+01_JPRB, 0.13058E+01_JPRB, 0.13264E+01_JPRB, 0.13488E+01_JPRB, 0.13619E+01_JPRB, 0.13606E+01_JPRB, 0.13436E+01_JPRB ,&
&  0.12944E+01_JPRB, 0.12280E+01_JPRB, 0.11877E+01_JPRB, 0.11897E+01_JPRB, 0.12187E+01_JPRB, 0.12349E+01_JPRB, 0.12420E+01_JPRB ,&
&  0.12503E+01_JPRB, 0.12515E+01_JPRB, 0.12112E+01_JPRB, 0.11776E+01_JPRB, 0.12337E+01_JPRB, 0.12097E+01_JPRB, 0.11783E+01_JPRB ,&
&  0.11431E+01_JPRB, 0.11194E+01_JPRB, 0.10923E+01_JPRB, 0.10605E+01_JPRB, 0.11230E+01_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.97631E+00_JPRB, 0.97079E+00_JPRB, 0.94670E+00_JPRB, 0.92551E+00_JPRB, 0.91426E+00_JPRB, 0.90305E+00_JPRB, 0.88250E+00_JPRB ,&
&  0.85897E+00_JPRB, 0.83668E+00_JPRB, 0.82220E+00_JPRB, 0.80190E+00_JPRB, 0.78106E+00_JPRB, 0.76735E+00_JPRB, 0.75366E+00_JPRB ,&
&  0.73518E+00_JPRB, 0.72701E+00_JPRB, 0.72383E+00_JPRB, 0.71914E+00_JPRB, 0.71192E+00_JPRB, 0.70661E+00_JPRB, 0.71085E+00_JPRB ,&
&  0.71067E+00_JPRB, 0.71014E+00_JPRB, 0.71146E+00_JPRB, 0.70601E+00_JPRB, 0.69788E+00_JPRB, 0.69411E+00_JPRB, 0.68144E+00_JPRB ,&
&  0.69602E+00_JPRB, 0.68656E+00_JPRB, 0.79671E+00_JPRB, 0.93189E+00_JPRB, 0.10463E+01_JPRB, 0.11274E+01_JPRB, 0.11833E+01_JPRB ,&
&  0.11726E+01_JPRB, 0.11077E+01_JPRB, 0.10379E+01_JPRB, 0.98985E+00_JPRB, 0.94016E+00_JPRB, 0.89995E+00_JPRB, 0.86628E+00_JPRB ,&
&  0.84380E+00_JPRB, 0.83105E+00_JPRB, 0.82104E+00_JPRB, 0.81167E+00_JPRB, 0.80183E+00_JPRB, 0.79356E+00_JPRB, 0.78918E+00_JPRB ,&
&  0.78414E+00_JPRB, 0.78411E+00_JPRB, 0.78679E+00_JPRB, 0.78623E+00_JPRB, 0.80092E+00_JPRB, 0.87438E+00_JPRB, 0.10334E+01_JPRB ,&
&  0.11460E+01_JPRB, 0.12708E+01_JPRB, 0.14351E+01_JPRB, 0.15302E+01_JPRB, 0.15174E+01_JPRB, 0.14699E+01_JPRB, 0.14028E+01_JPRB ,&
&  0.13457E+01_JPRB, 0.12828E+01_JPRB, 0.12425E+01_JPRB, 0.12015E+01_JPRB, 0.11577E+01_JPRB, 0.11216E+01_JPRB, 0.11010E+01_JPRB ,&
&  0.10935E+01_JPRB, 0.10918E+01_JPRB, 0.11028E+01_JPRB, 0.11197E+01_JPRB, 0.11306E+01_JPRB, 0.11392E+01_JPRB, 0.11494E+01_JPRB ,&
&  0.11556E+01_JPRB, 0.11376E+01_JPRB, 0.11067E+01_JPRB, 0.11080E+01_JPRB, 0.11366E+01_JPRB, 0.11605E+01_JPRB, 0.11756E+01_JPRB ,&
&  0.11893E+01_JPRB, 0.12026E+01_JPRB, 0.12081E+01_JPRB, 0.12168E+01_JPRB, 0.12002E+01_JPRB, 0.11609E+01_JPRB, 0.11234E+01_JPRB ,&
&  0.10907E+01_JPRB, 0.10609E+01_JPRB, 0.10398E+01_JPRB, 0.10119E+01_JPRB, 0.92950E+00_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.91472E+00_JPRB, 0.90319E+00_JPRB, 0.88343E+00_JPRB, 0.86446E+00_JPRB, 0.85540E+00_JPRB, 0.85232E+00_JPRB, 0.85149E+00_JPRB ,&
&  0.84518E+00_JPRB, 0.83505E+00_JPRB, 0.82293E+00_JPRB, 0.80760E+00_JPRB, 0.79058E+00_JPRB, 0.77554E+00_JPRB, 0.76177E+00_JPRB ,&
&  0.74116E+00_JPRB, 0.72836E+00_JPRB, 0.72637E+00_JPRB, 0.72311E+00_JPRB, 0.71973E+00_JPRB, 0.71387E+00_JPRB, 0.71209E+00_JPRB ,&
&  0.71411E+00_JPRB, 0.71603E+00_JPRB, 0.71720E+00_JPRB, 0.70998E+00_JPRB, 0.70290E+00_JPRB, 0.70055E+00_JPRB, 0.69426E+00_JPRB ,&
&  0.69501E+00_JPRB, 0.60173E+00_JPRB, 0.68324E+00_JPRB, 0.78632E+00_JPRB, 0.84306E+00_JPRB, 0.92110E+00_JPRB, 0.10123E+01_JPRB ,&
&  0.10479E+01_JPRB, 0.10235E+01_JPRB, 0.97877E+00_JPRB, 0.93931E+00_JPRB, 0.89901E+00_JPRB, 0.86488E+00_JPRB, 0.83679E+00_JPRB ,&
&  0.81843E+00_JPRB, 0.80672E+00_JPRB, 0.79618E+00_JPRB, 0.78322E+00_JPRB, 0.76966E+00_JPRB, 0.76223E+00_JPRB, 0.75733E+00_JPRB ,&
&  0.75576E+00_JPRB, 0.75851E+00_JPRB, 0.76015E+00_JPRB, 0.76741E+00_JPRB, 0.77477E+00_JPRB, 0.80176E+00_JPRB, 0.87685E+00_JPRB ,&
&  0.93311E+00_JPRB, 0.10020E+01_JPRB, 0.11006E+01_JPRB, 0.11476E+01_JPRB, 0.11471E+01_JPRB, 0.11336E+01_JPRB, 0.11076E+01_JPRB ,&
&  0.10741E+01_JPRB, 0.10273E+01_JPRB, 0.99779E+00_JPRB, 0.96925E+00_JPRB, 0.94780E+00_JPRB, 0.93638E+00_JPRB, 0.93986E+00_JPRB ,&
&  0.94491E+00_JPRB, 0.94454E+00_JPRB, 0.94723E+00_JPRB, 0.95437E+00_JPRB, 0.95893E+00_JPRB, 0.96149E+00_JPRB, 0.97606E+00_JPRB ,&
&  0.10037E+01_JPRB, 0.10326E+01_JPRB, 0.10397E+01_JPRB, 0.10317E+01_JPRB, 0.10334E+01_JPRB, 0.10517E+01_JPRB, 0.10703E+01_JPRB ,&
&  0.10855E+01_JPRB, 0.10968E+01_JPRB, 0.11113E+01_JPRB, 0.10698E+01_JPRB, 0.11118E+01_JPRB, 0.10910E+01_JPRB, 0.10614E+01_JPRB ,&
&  0.10315E+01_JPRB, 0.10047E+01_JPRB, 0.98558E+00_JPRB, 0.84945E+00_JPRB, 0.89339E+00_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.84041E+00_JPRB, 0.82928E+00_JPRB, 0.81234E+00_JPRB, 0.79638E+00_JPRB, 0.78536E+00_JPRB, 0.78091E+00_JPRB, 0.77921E+00_JPRB ,&
&  0.78291E+00_JPRB, 0.78647E+00_JPRB, 0.78692E+00_JPRB, 0.78361E+00_JPRB, 0.77762E+00_JPRB, 0.76777E+00_JPRB, 0.75900E+00_JPRB ,&
&  0.74490E+00_JPRB, 0.73001E+00_JPRB, 0.72453E+00_JPRB, 0.72469E+00_JPRB, 0.72283E+00_JPRB, 0.71547E+00_JPRB, 0.71028E+00_JPRB ,&
&  0.70782E+00_JPRB, 0.70973E+00_JPRB, 0.70321E+00_JPRB, 0.69825E+00_JPRB, 0.69185E+00_JPRB, 0.69242E+00_JPRB, 0.69185E+00_JPRB ,&
&  0.69011E+00_JPRB, 0.56737E+00_JPRB, 0.64822E+00_JPRB, 0.71467E+00_JPRB, 0.74522E+00_JPRB, 0.80487E+00_JPRB, 0.86582E+00_JPRB ,&
&  0.90726E+00_JPRB, 0.92460E+00_JPRB, 0.91541E+00_JPRB, 0.89480E+00_JPRB, 0.86432E+00_JPRB, 0.83623E+00_JPRB, 0.81182E+00_JPRB ,&
&  0.78872E+00_JPRB, 0.77069E+00_JPRB, 0.75455E+00_JPRB, 0.73952E+00_JPRB, 0.72995E+00_JPRB, 0.72042E+00_JPRB, 0.71218E+00_JPRB ,&
&  0.71091E+00_JPRB, 0.71639E+00_JPRB, 0.71640E+00_JPRB, 0.71924E+00_JPRB, 0.72404E+00_JPRB, 0.72926E+00_JPRB, 0.75850E+00_JPRB ,&
&  0.78871E+00_JPRB, 0.81693E+00_JPRB, 0.84818E+00_JPRB, 0.86999E+00_JPRB, 0.87597E+00_JPRB, 0.88401E+00_JPRB, 0.88225E+00_JPRB ,&
&  0.87753E+00_JPRB, 0.86302E+00_JPRB, 0.84592E+00_JPRB, 0.82917E+00_JPRB, 0.81512E+00_JPRB, 0.80527E+00_JPRB, 0.80461E+00_JPRB ,&
&  0.80879E+00_JPRB, 0.80228E+00_JPRB, 0.79501E+00_JPRB, 0.79354E+00_JPRB, 0.79547E+00_JPRB, 0.80242E+00_JPRB, 0.81866E+00_JPRB ,&
&  0.84552E+00_JPRB, 0.87991E+00_JPRB, 0.90751E+00_JPRB, 0.91867E+00_JPRB, 0.92047E+00_JPRB, 0.92801E+00_JPRB, 0.94157E+00_JPRB ,&
&  0.95698E+00_JPRB, 0.96719E+00_JPRB, 0.97702E+00_JPRB, 0.10024E+01_JPRB, 0.10035E+01_JPRB, 0.98977E+00_JPRB, 0.96830E+00_JPRB ,&
&  0.93774E+00_JPRB, 0.92074E+00_JPRB, 0.75502E+00_JPRB, 0.81789E+00_JPRB, 0.83047E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.78053E+00_JPRB, 0.76752E+00_JPRB, 0.74963E+00_JPRB, 0.73538E+00_JPRB, 0.72417E+00_JPRB, 0.71978E+00_JPRB, 0.71717E+00_JPRB ,&
&  0.72118E+00_JPRB, 0.72905E+00_JPRB, 0.73723E+00_JPRB, 0.74247E+00_JPRB, 0.74651E+00_JPRB, 0.74686E+00_JPRB, 0.74296E+00_JPRB ,&
&  0.73966E+00_JPRB, 0.72845E+00_JPRB, 0.71947E+00_JPRB, 0.71939E+00_JPRB, 0.71667E+00_JPRB, 0.71307E+00_JPRB, 0.70781E+00_JPRB ,&
&  0.70223E+00_JPRB, 0.69696E+00_JPRB, 0.68581E+00_JPRB, 0.67986E+00_JPRB, 0.67780E+00_JPRB, 0.67983E+00_JPRB, 0.68883E+00_JPRB ,&
&  0.61257E+00_JPRB, 0.53343E+00_JPRB, 0.59955E+00_JPRB, 0.66971E+00_JPRB, 0.70386E+00_JPRB, 0.74492E+00_JPRB, 0.78142E+00_JPRB ,&
&  0.82977E+00_JPRB, 0.86044E+00_JPRB, 0.87754E+00_JPRB, 0.87289E+00_JPRB, 0.85489E+00_JPRB, 0.83296E+00_JPRB, 0.80947E+00_JPRB ,&
&  0.78027E+00_JPRB, 0.74674E+00_JPRB, 0.72420E+00_JPRB, 0.71240E+00_JPRB, 0.70265E+00_JPRB, 0.69239E+00_JPRB, 0.68543E+00_JPRB ,&
&  0.68201E+00_JPRB, 0.67971E+00_JPRB, 0.67724E+00_JPRB, 0.67581E+00_JPRB, 0.67784E+00_JPRB, 0.68093E+00_JPRB, 0.68797E+00_JPRB ,&
&  0.70040E+00_JPRB, 0.71316E+00_JPRB, 0.72419E+00_JPRB, 0.73008E+00_JPRB, 0.73564E+00_JPRB, 0.74731E+00_JPRB, 0.75565E+00_JPRB ,&
&  0.76299E+00_JPRB, 0.76314E+00_JPRB, 0.75911E+00_JPRB, 0.75223E+00_JPRB, 0.74179E+00_JPRB, 0.73141E+00_JPRB, 0.72375E+00_JPRB ,&
&  0.71957E+00_JPRB, 0.71221E+00_JPRB, 0.70942E+00_JPRB, 0.71163E+00_JPRB, 0.71633E+00_JPRB, 0.72231E+00_JPRB, 0.72926E+00_JPRB ,&
&  0.74263E+00_JPRB, 0.76585E+00_JPRB, 0.78554E+00_JPRB, 0.79841E+00_JPRB, 0.80672E+00_JPRB, 0.81314E+00_JPRB, 0.82325E+00_JPRB ,&
&  0.83386E+00_JPRB, 0.84519E+00_JPRB, 0.85964E+00_JPRB, 0.87558E+00_JPRB, 0.88040E+00_JPRB, 0.87575E+00_JPRB, 0.86398E+00_JPRB ,&
&  0.84568E+00_JPRB, 0.82180E+00_JPRB, 0.81161E+00_JPRB, 0.79854E+00_JPRB, 0.78842E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.68881E+00_JPRB, 0.68028E+00_JPRB, 0.66790E+00_JPRB, 0.66041E+00_JPRB, 0.65713E+00_JPRB, 0.65967E+00_JPRB, 0.66363E+00_JPRB ,&
&  0.66794E+00_JPRB, 0.66947E+00_JPRB, 0.67711E+00_JPRB, 0.68558E+00_JPRB, 0.69128E+00_JPRB, 0.69550E+00_JPRB, 0.69522E+00_JPRB ,&
&  0.69540E+00_JPRB, 0.69576E+00_JPRB, 0.69172E+00_JPRB, 0.68804E+00_JPRB, 0.68274E+00_JPRB, 0.68353E+00_JPRB, 0.68316E+00_JPRB ,&
&  0.68079E+00_JPRB, 0.67590E+00_JPRB, 0.66751E+00_JPRB, 0.65713E+00_JPRB, 0.65202E+00_JPRB, 0.65051E+00_JPRB, 0.64767E+00_JPRB ,&
&  0.65329E+00_JPRB, 0.57208E+00_JPRB, 0.61174E+00_JPRB, 0.63316E+00_JPRB, 0.66152E+00_JPRB, 0.69209E+00_JPRB, 0.72411E+00_JPRB ,&
&  0.75685E+00_JPRB, 0.78358E+00_JPRB, 0.79398E+00_JPRB, 0.79487E+00_JPRB, 0.78077E+00_JPRB, 0.75666E+00_JPRB, 0.72934E+00_JPRB ,&
&  0.70623E+00_JPRB, 0.68877E+00_JPRB, 0.66915E+00_JPRB, 0.65742E+00_JPRB, 0.64788E+00_JPRB, 0.63579E+00_JPRB, 0.62514E+00_JPRB ,&
&  0.61971E+00_JPRB, 0.61612E+00_JPRB, 0.61516E+00_JPRB, 0.61661E+00_JPRB, 0.61945E+00_JPRB, 0.62170E+00_JPRB, 0.62192E+00_JPRB ,&
&  0.62362E+00_JPRB, 0.62755E+00_JPRB, 0.63136E+00_JPRB, 0.63664E+00_JPRB, 0.64340E+00_JPRB, 0.65256E+00_JPRB, 0.65545E+00_JPRB ,&
&  0.65179E+00_JPRB, 0.65372E+00_JPRB, 0.65579E+00_JPRB, 0.65466E+00_JPRB, 0.65293E+00_JPRB, 0.65053E+00_JPRB, 0.64602E+00_JPRB ,&
&  0.64222E+00_JPRB, 0.64280E+00_JPRB, 0.64436E+00_JPRB, 0.64968E+00_JPRB, 0.65363E+00_JPRB, 0.66285E+00_JPRB, 0.67156E+00_JPRB ,&
&  0.68128E+00_JPRB, 0.69706E+00_JPRB, 0.71438E+00_JPRB, 0.72824E+00_JPRB, 0.73692E+00_JPRB, 0.74286E+00_JPRB, 0.74859E+00_JPRB ,&
&  0.75363E+00_JPRB, 0.75891E+00_JPRB, 0.76511E+00_JPRB, 0.76832E+00_JPRB, 0.76583E+00_JPRB, 0.76058E+00_JPRB, 0.75441E+00_JPRB ,&
&  0.74065E+00_JPRB, 0.72235E+00_JPRB, 0.71150E+00_JPRB, 0.70191E+00_JPRB, 0.69377E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.62842E+00_JPRB, 0.62232E+00_JPRB, 0.61645E+00_JPRB, 0.61524E+00_JPRB, 0.61403E+00_JPRB, 0.61574E+00_JPRB, 0.61821E+00_JPRB ,&
&  0.62479E+00_JPRB, 0.62911E+00_JPRB, 0.63859E+00_JPRB, 0.64655E+00_JPRB, 0.65075E+00_JPRB, 0.65565E+00_JPRB, 0.65885E+00_JPRB ,&
&  0.66038E+00_JPRB, 0.66239E+00_JPRB, 0.66370E+00_JPRB, 0.66843E+00_JPRB, 0.66939E+00_JPRB, 0.66895E+00_JPRB, 0.66795E+00_JPRB ,&
&  0.66552E+00_JPRB, 0.66231E+00_JPRB, 0.65575E+00_JPRB, 0.64715E+00_JPRB, 0.63673E+00_JPRB, 0.63164E+00_JPRB, 0.62475E+00_JPRB ,&
&  0.62521E+00_JPRB, 0.59650E+00_JPRB, 0.60869E+00_JPRB, 0.60698E+00_JPRB, 0.62428E+00_JPRB, 0.65150E+00_JPRB, 0.68606E+00_JPRB ,&
&  0.70907E+00_JPRB, 0.72025E+00_JPRB, 0.72042E+00_JPRB, 0.71616E+00_JPRB, 0.70019E+00_JPRB, 0.68412E+00_JPRB, 0.66610E+00_JPRB ,&
&  0.64581E+00_JPRB, 0.63427E+00_JPRB, 0.62016E+00_JPRB, 0.60978E+00_JPRB, 0.60536E+00_JPRB, 0.60211E+00_JPRB, 0.59661E+00_JPRB ,&
&  0.58828E+00_JPRB, 0.58097E+00_JPRB, 0.58101E+00_JPRB, 0.58422E+00_JPRB, 0.58780E+00_JPRB, 0.59009E+00_JPRB, 0.58894E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.57875E+00_JPRB, 0.57947E+00_JPRB, 0.58222E+00_JPRB, 0.58153E+00_JPRB, 0.57729E+00_JPRB, 0.57248E+00_JPRB ,&
&  0.57230E+00_JPRB, 0.57537E+00_JPRB, 0.57867E+00_JPRB, 0.58299E+00_JPRB, 0.58568E+00_JPRB, 0.58882E+00_JPRB, 0.58994E+00_JPRB ,&
&  0.59409E+00_JPRB, 0.59811E+00_JPRB, 0.60670E+00_JPRB, 0.61497E+00_JPRB, 0.61656E+00_JPRB, 0.62500E+00_JPRB, 0.63423E+00_JPRB ,&
&  0.64430E+00_JPRB, 0.65656E+00_JPRB, 0.66742E+00_JPRB, 0.67644E+00_JPRB, 0.68549E+00_JPRB, 0.69077E+00_JPRB, 0.69726E+00_JPRB ,&
&  0.70541E+00_JPRB, 0.70750E+00_JPRB, 0.71356E+00_JPRB, 0.71398E+00_JPRB, 0.70916E+00_JPRB, 0.70090E+00_JPRB, 0.68868E+00_JPRB ,&
&  0.67402E+00_JPRB, 0.65524E+00_JPRB, 0.64388E+00_JPRB, 0.63635E+00_JPRB, 0.63164E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.55815E+00_JPRB, 0.55530E+00_JPRB, 0.55504E+00_JPRB, 0.55791E+00_JPRB, 0.56130E+00_JPRB, 0.56385E+00_JPRB, 0.56930E+00_JPRB ,&
&  0.57912E+00_JPRB, 0.58697E+00_JPRB, 0.59671E+00_JPRB, 0.60674E+00_JPRB, 0.61452E+00_JPRB, 0.61983E+00_JPRB, 0.62276E+00_JPRB ,&
&  0.62135E+00_JPRB, 0.61910E+00_JPRB, 0.62253E+00_JPRB, 0.62756E+00_JPRB, 0.62917E+00_JPRB, 0.62888E+00_JPRB, 0.62895E+00_JPRB ,&
&  0.62827E+00_JPRB, 0.62555E+00_JPRB, 0.62182E+00_JPRB, 0.61635E+00_JPRB, 0.61001E+00_JPRB, 0.60642E+00_JPRB, 0.60326E+00_JPRB ,&
&  0.59584E+00_JPRB, 0.59609E+00_JPRB, 0.54525E+00_JPRB, 0.55991E+00_JPRB, 0.57225E+00_JPRB, 0.59343E+00_JPRB, 0.61449E+00_JPRB ,&
&  0.62918E+00_JPRB, 0.63776E+00_JPRB, 0.64164E+00_JPRB, 0.63919E+00_JPRB, 0.62614E+00_JPRB, 0.61822E+00_JPRB, 0.60949E+00_JPRB ,&
&  0.59829E+00_JPRB, 0.59029E+00_JPRB, 0.57683E+00_JPRB, 0.56238E+00_JPRB, 0.55747E+00_JPRB, 0.55796E+00_JPRB, 0.55782E+00_JPRB ,&
&  0.55364E+00_JPRB, 0.54959E+00_JPRB, 0.55055E+00_JPRB, 0.55128E+00_JPRB, 0.55091E+00_JPRB, 0.54559E+00_JPRB, 0.53894E+00_JPRB ,&
&  0.52862E+00_JPRB, 0.51874E+00_JPRB, 0.51206E+00_JPRB, 0.50775E+00_JPRB, 0.50445E+00_JPRB, 0.50080E+00_JPRB, 0.49701E+00_JPRB ,&
&  0.50204E+00_JPRB, 0.51481E+00_JPRB, 0.52656E+00_JPRB, 0.53587E+00_JPRB, 0.54196E+00_JPRB, 0.54581E+00_JPRB, 0.55092E+00_JPRB ,&
&  0.55685E+00_JPRB, 0.56720E+00_JPRB, 0.57468E+00_JPRB, 0.58126E+00_JPRB, 0.58588E+00_JPRB, 0.59028E+00_JPRB, 0.59548E+00_JPRB ,&
&  0.60024E+00_JPRB, 0.60834E+00_JPRB, 0.61515E+00_JPRB, 0.62322E+00_JPRB, 0.62860E+00_JPRB, 0.63498E+00_JPRB, 0.64144E+00_JPRB ,&
&  0.64829E+00_JPRB, 0.65410E+00_JPRB, 0.65634E+00_JPRB, 0.65435E+00_JPRB, 0.64684E+00_JPRB, 0.63709E+00_JPRB, 0.62663E+00_JPRB ,&
&  0.61308E+00_JPRB, 0.59708E+00_JPRB, 0.58106E+00_JPRB, 0.57057E+00_JPRB, 0.56168E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.50522E+00_JPRB, 0.50532E+00_JPRB, 0.50951E+00_JPRB, 0.51369E+00_JPRB, 0.51684E+00_JPRB, 0.52158E+00_JPRB, 0.53112E+00_JPRB ,&
&  0.54080E+00_JPRB, 0.55018E+00_JPRB, 0.55666E+00_JPRB, 0.56449E+00_JPRB, 0.57561E+00_JPRB, 0.58324E+00_JPRB, 0.58934E+00_JPRB ,&
&  0.59200E+00_JPRB, 0.59262E+00_JPRB, 0.59399E+00_JPRB, 0.59739E+00_JPRB, 0.59712E+00_JPRB, 0.59360E+00_JPRB, 0.59239E+00_JPRB ,&
&  0.59227E+00_JPRB, 0.59042E+00_JPRB, 0.58989E+00_JPRB, 0.59044E+00_JPRB, 0.58740E+00_JPRB, 0.58279E+00_JPRB, 0.58150E+00_JPRB ,&
&  0.58084E+00_JPRB, 0.56667E+00_JPRB, 0.55851E+00_JPRB, 0.55109E+00_JPRB, 0.55434E+00_JPRB, 0.56547E+00_JPRB, 0.57787E+00_JPRB ,&
&  0.58532E+00_JPRB, 0.59031E+00_JPRB, 0.59005E+00_JPRB, 0.58912E+00_JPRB, 0.58430E+00_JPRB, 0.57285E+00_JPRB, 0.56334E+00_JPRB ,&
&  0.55695E+00_JPRB, 0.54754E+00_JPRB, 0.52474E+00_JPRB, 0.51133E+00_JPRB, 0.50770E+00_JPRB, 0.51045E+00_JPRB, 0.51475E+00_JPRB ,&
&  0.51642E+00_JPRB, 0.51365E+00_JPRB, 0.50724E+00_JPRB, 0.49987E+00_JPRB, 0.48998E+00_JPRB, 0.47868E+00_JPRB, 0.46882E+00_JPRB ,&
&  0.46338E+00_JPRB, 0.45954E+00_JPRB, 0.45652E+00_JPRB, 0.45951E+00_JPRB, 0.45905E+00_JPRB, 0.45562E+00_JPRB, 0.45844E+00_JPRB ,&
&  0.47116E+00_JPRB, 0.48719E+00_JPRB, 0.50202E+00_JPRB, 0.51014E+00_JPRB, 0.51359E+00_JPRB, 0.51748E+00_JPRB, 0.52179E+00_JPRB ,&
&  0.52756E+00_JPRB, 0.53301E+00_JPRB, 0.53827E+00_JPRB, 0.54537E+00_JPRB, 0.55407E+00_JPRB, 0.56173E+00_JPRB, 0.56494E+00_JPRB ,&
&  0.56771E+00_JPRB, 0.56981E+00_JPRB, 0.57198E+00_JPRB, 0.57834E+00_JPRB, 0.58711E+00_JPRB, 0.59330E+00_JPRB, 0.59798E+00_JPRB ,&
&  0.60255E+00_JPRB, 0.60458E+00_JPRB, 0.60295E+00_JPRB, 0.59843E+00_JPRB, 0.59261E+00_JPRB, 0.58148E+00_JPRB, 0.56746E+00_JPRB ,&
&  0.55124E+00_JPRB, 0.53609E+00_JPRB, 0.52464E+00_JPRB, 0.51474E+00_JPRB, 0.50767E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.46519E+00_JPRB, 0.46437E+00_JPRB, 0.46677E+00_JPRB, 0.47270E+00_JPRB, 0.47677E+00_JPRB, 0.48054E+00_JPRB, 0.48830E+00_JPRB ,&
&  0.49548E+00_JPRB, 0.50195E+00_JPRB, 0.50952E+00_JPRB, 0.51765E+00_JPRB, 0.52389E+00_JPRB, 0.53298E+00_JPRB, 0.54171E+00_JPRB ,&
&  0.54741E+00_JPRB, 0.54981E+00_JPRB, 0.55131E+00_JPRB, 0.55263E+00_JPRB, 0.55265E+00_JPRB, 0.55027E+00_JPRB, 0.54992E+00_JPRB ,&
&  0.54811E+00_JPRB, 0.54602E+00_JPRB, 0.54807E+00_JPRB, 0.54829E+00_JPRB, 0.54965E+00_JPRB, 0.55194E+00_JPRB, 0.55145E+00_JPRB ,&
&  0.54861E+00_JPRB, 0.53490E+00_JPRB, 0.52960E+00_JPRB, 0.52628E+00_JPRB, 0.53023E+00_JPRB, 0.53106E+00_JPRB, 0.52942E+00_JPRB ,&
&  0.52281E+00_JPRB, 0.51870E+00_JPRB, 0.52044E+00_JPRB, 0.52267E+00_JPRB, 0.51677E+00_JPRB, 0.50363E+00_JPRB, 0.49345E+00_JPRB ,&
&  0.48558E+00_JPRB, 0.47337E+00_JPRB, 0.45532E+00_JPRB, 0.44438E+00_JPRB, 0.43963E+00_JPRB, 0.43474E+00_JPRB, 0.43042E+00_JPRB ,&
&  0.42821E+00_JPRB, 0.42746E+00_JPRB, 0.42881E+00_JPRB, 0.42803E+00_JPRB, 0.42448E+00_JPRB, 0.42018E+00_JPRB, 0.41864E+00_JPRB ,&
&  0.42162E+00_JPRB, 0.42447E+00_JPRB, 0.42839E+00_JPRB, 0.43123E+00_JPRB, 0.43091E+00_JPRB, 0.43158E+00_JPRB, 0.43584E+00_JPRB ,&
&  0.44290E+00_JPRB, 0.44986E+00_JPRB, 0.45614E+00_JPRB, 0.45869E+00_JPRB, 0.46235E+00_JPRB, 0.46972E+00_JPRB, 0.47056E+00_JPRB ,&
&  0.46893E+00_JPRB, 0.47045E+00_JPRB, 0.47443E+00_JPRB, 0.48261E+00_JPRB, 0.49522E+00_JPRB, 0.50355E+00_JPRB, 0.50489E+00_JPRB ,&
&  0.50504E+00_JPRB, 0.50746E+00_JPRB, 0.51185E+00_JPRB, 0.52218E+00_JPRB, 0.53312E+00_JPRB, 0.54086E+00_JPRB, 0.54393E+00_JPRB ,&
&  0.54054E+00_JPRB, 0.53343E+00_JPRB, 0.52092E+00_JPRB, 0.50641E+00_JPRB, 0.49371E+00_JPRB, 0.48386E+00_JPRB, 0.47576E+00_JPRB ,&
&  0.46779E+00_JPRB, 0.46414E+00_JPRB, 0.46356E+00_JPRB, 0.46515E+00_JPRB, 0.46566E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.43389E+00_JPRB, 0.43541E+00_JPRB, 0.43748E+00_JPRB, 0.44028E+00_JPRB, 0.44328E+00_JPRB, 0.44507E+00_JPRB, 0.44874E+00_JPRB ,&
&  0.45092E+00_JPRB, 0.45757E+00_JPRB, 0.46843E+00_JPRB, 0.47926E+00_JPRB, 0.48707E+00_JPRB, 0.49365E+00_JPRB, 0.50021E+00_JPRB ,&
&  0.50562E+00_JPRB, 0.51275E+00_JPRB, 0.51375E+00_JPRB, 0.51144E+00_JPRB, 0.50889E+00_JPRB, 0.50698E+00_JPRB, 0.50523E+00_JPRB ,&
&  0.50274E+00_JPRB, 0.50259E+00_JPRB, 0.50629E+00_JPRB, 0.51043E+00_JPRB, 0.51461E+00_JPRB, 0.51557E+00_JPRB, 0.51389E+00_JPRB ,&
&  0.51703E+00_JPRB, 0.50697E+00_JPRB, 0.50026E+00_JPRB, 0.49255E+00_JPRB, 0.48582E+00_JPRB, 0.47413E+00_JPRB, 0.46546E+00_JPRB ,&
&  0.45514E+00_JPRB, 0.44601E+00_JPRB, 0.44206E+00_JPRB, 0.43786E+00_JPRB, 0.42801E+00_JPRB, 0.41666E+00_JPRB, 0.40840E+00_JPRB ,&
&  0.40295E+00_JPRB, 0.39739E+00_JPRB, 0.39063E+00_JPRB, 0.38501E+00_JPRB, 0.38043E+00_JPRB, 0.37882E+00_JPRB, 0.38051E+00_JPRB ,&
&  0.38479E+00_JPRB, 0.38896E+00_JPRB, 0.39156E+00_JPRB, 0.39235E+00_JPRB, 0.39085E+00_JPRB, 0.38837E+00_JPRB, 0.38703E+00_JPRB ,&
&  0.38799E+00_JPRB, 0.38991E+00_JPRB, 0.39213E+00_JPRB, 0.39733E+00_JPRB, 0.39839E+00_JPRB, 0.39708E+00_JPRB, 0.39190E+00_JPRB ,&
&  0.39013E+00_JPRB, 0.38956E+00_JPRB, 0.39175E+00_JPRB, 0.39719E+00_JPRB, 0.40588E+00_JPRB, 0.41426E+00_JPRB, 0.42012E+00_JPRB ,&
&  0.42284E+00_JPRB, 0.42428E+00_JPRB, 0.42715E+00_JPRB, 0.43082E+00_JPRB, 0.43247E+00_JPRB, 0.43202E+00_JPRB, 0.43071E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.44136E+00_JPRB, 0.44970E+00_JPRB, 0.45550E+00_JPRB, 0.45859E+00_JPRB, 0.45788E+00_JPRB, 0.45108E+00_JPRB ,&
&  0.43844E+00_JPRB, 0.42384E+00_JPRB, 0.40969E+00_JPRB, 0.39782E+00_JPRB, 0.38968E+00_JPRB, 0.38578E+00_JPRB, 0.38864E+00_JPRB ,&
&  0.39640E+00_JPRB, 0.40633E+00_JPRB, 0.41539E+00_JPRB, 0.42200E+00_JPRB, 0.42913E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.40376E+00_JPRB, 0.40365E+00_JPRB, 0.40520E+00_JPRB, 0.40415E+00_JPRB, 0.40140E+00_JPRB, 0.39913E+00_JPRB, 0.40230E+00_JPRB ,&
&  0.40897E+00_JPRB, 0.42066E+00_JPRB, 0.43632E+00_JPRB, 0.45012E+00_JPRB, 0.45839E+00_JPRB, 0.46337E+00_JPRB, 0.47100E+00_JPRB ,&
&  0.47817E+00_JPRB, 0.48617E+00_JPRB, 0.48643E+00_JPRB, 0.47768E+00_JPRB, 0.47029E+00_JPRB, 0.46640E+00_JPRB, 0.46594E+00_JPRB ,&
&  0.46526E+00_JPRB, 0.46696E+00_JPRB, 0.47002E+00_JPRB, 0.47322E+00_JPRB, 0.47237E+00_JPRB, 0.47031E+00_JPRB, 0.47361E+00_JPRB ,&
&  0.47120E+00_JPRB, 0.46097E+00_JPRB, 0.45044E+00_JPRB, 0.42525E+00_JPRB, 0.42414E+00_JPRB, 0.41343E+00_JPRB, 0.40532E+00_JPRB ,&
&  0.39444E+00_JPRB, 0.38094E+00_JPRB, 0.37112E+00_JPRB, 0.36482E+00_JPRB, 0.36197E+00_JPRB, 0.35945E+00_JPRB, 0.35743E+00_JPRB ,&
&  0.35602E+00_JPRB, 0.35572E+00_JPRB, 0.35711E+00_JPRB, 0.35472E+00_JPRB, 0.35355E+00_JPRB, 0.35433E+00_JPRB, 0.35615E+00_JPRB ,&
&  0.35629E+00_JPRB, 0.35653E+00_JPRB, 0.35680E+00_JPRB, 0.35618E+00_JPRB, 0.35467E+00_JPRB, 0.35229E+00_JPRB, 0.35009E+00_JPRB ,&
&  0.34938E+00_JPRB, 0.35087E+00_JPRB, 0.35166E+00_JPRB, 0.34813E+00_JPRB, 0.34071E+00_JPRB, 0.33128E+00_JPRB, 0.32529E+00_JPRB ,&
&  0.32315E+00_JPRB, 0.32584E+00_JPRB, 0.33426E+00_JPRB, 0.34584E+00_JPRB, 0.35638E+00_JPRB, 0.36266E+00_JPRB, 0.36639E+00_JPRB ,&
&  0.37054E+00_JPRB, 0.36818E+00_JPRB, 0.36405E+00_JPRB, 0.36053E+00_JPRB, 0.35741E+00_JPRB, 0.35326E+00_JPRB, 0.35320E+00_JPRB ,&
&  0.35603E+00_JPRB, 0.35896E+00_JPRB, 0.36003E+00_JPRB, 0.35918E+00_JPRB, 0.35728E+00_JPRB, 0.35311E+00_JPRB, 0.34721E+00_JPRB ,&
&  0.33980E+00_JPRB, 0.33113E+00_JPRB, 0.32938E+00_JPRB, 0.33021E+00_JPRB, 0.33404E+00_JPRB, 0.34180E+00_JPRB, 0.35249E+00_JPRB ,&
&  0.36367E+00_JPRB, 0.37276E+00_JPRB, 0.38267E+00_JPRB, 0.39188E+00_JPRB, 0.40017E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.38461E+00_JPRB, 0.38605E+00_JPRB, 0.38347E+00_JPRB, 0.38070E+00_JPRB, 0.38044E+00_JPRB, 0.38191E+00_JPRB, 0.38737E+00_JPRB ,&
&  0.39704E+00_JPRB, 0.40987E+00_JPRB, 0.42219E+00_JPRB, 0.43236E+00_JPRB, 0.44144E+00_JPRB, 0.44539E+00_JPRB, 0.45127E+00_JPRB ,&
&  0.45767E+00_JPRB, 0.46353E+00_JPRB, 0.46428E+00_JPRB, 0.45396E+00_JPRB, 0.44396E+00_JPRB, 0.43865E+00_JPRB, 0.43861E+00_JPRB ,&
&  0.44030E+00_JPRB, 0.44278E+00_JPRB, 0.44877E+00_JPRB, 0.45083E+00_JPRB, 0.44472E+00_JPRB, 0.44302E+00_JPRB, 0.44062E+00_JPRB ,&
&  0.43468E+00_JPRB, 0.42248E+00_JPRB, 0.38971E+00_JPRB, 0.38348E+00_JPRB, 0.37489E+00_JPRB, 0.36566E+00_JPRB, 0.35666E+00_JPRB ,&
&  0.34633E+00_JPRB, 0.33587E+00_JPRB, 0.32874E+00_JPRB, 0.32451E+00_JPRB, 0.32509E+00_JPRB, 0.32632E+00_JPRB, 0.32581E+00_JPRB ,&
&  0.32783E+00_JPRB, 0.33024E+00_JPRB, 0.33238E+00_JPRB, 0.33269E+00_JPRB, 0.33235E+00_JPRB, 0.33145E+00_JPRB, 0.32727E+00_JPRB ,&
&  0.32350E+00_JPRB, 0.32166E+00_JPRB, 0.32031E+00_JPRB, 0.31918E+00_JPRB, 0.31923E+00_JPRB, 0.31985E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.31792E+00_JPRB, 0.31451E+00_JPRB, 0.31067E+00_JPRB, 0.30371E+00_JPRB, 0.29112E+00_JPRB, 0.27655E+00_JPRB, 0.26727E+00_JPRB ,&
&  0.26607E+00_JPRB, 0.27685E+00_JPRB, 0.29793E+00_JPRB, 0.30835E+00_JPRB, 0.31584E+00_JPRB, 0.31960E+00_JPRB, 0.32361E+00_JPRB ,&
&  0.31767E+00_JPRB, 0.29443E+00_JPRB, 0.28127E+00_JPRB, 0.27385E+00_JPRB, 0.26849E+00_JPRB, 0.26466E+00_JPRB, 0.26434E+00_JPRB ,&
&  0.26762E+00_JPRB, 0.27368E+00_JPRB, 0.28011E+00_JPRB, 0.28325E+00_JPRB, 0.28247E+00_JPRB, 0.27938E+00_JPRB, 0.27604E+00_JPRB ,&
&  0.27368E+00_JPRB, 0.27865E+00_JPRB, 0.29345E+00_JPRB, 0.31429E+00_JPRB, 0.32264E+00_JPRB, 0.32916E+00_JPRB, 0.33522E+00_JPRB ,&
&  0.34179E+00_JPRB, 0.35166E+00_JPRB, 0.36343E+00_JPRB, 0.37268E+00_JPRB, 0.38080E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.35686E+00_JPRB, 0.36086E+00_JPRB, 0.36337E+00_JPRB, 0.36675E+00_JPRB, 0.37437E+00_JPRB, 0.38488E+00_JPRB, 0.39691E+00_JPRB ,&
&  0.40831E+00_JPRB, 0.41781E+00_JPRB, 0.42260E+00_JPRB, 0.42782E+00_JPRB, 0.43274E+00_JPRB, 0.43503E+00_JPRB, 0.44004E+00_JPRB ,&
&  0.44068E+00_JPRB, 0.43715E+00_JPRB, 0.43096E+00_JPRB, 0.42285E+00_JPRB, 0.41852E+00_JPRB, 0.41966E+00_JPRB, 0.42238E+00_JPRB ,&
&  0.42406E+00_JPRB, 0.42534E+00_JPRB, 0.42524E+00_JPRB, 0.42162E+00_JPRB, 0.41821E+00_JPRB, 0.41616E+00_JPRB, 0.41181E+00_JPRB ,&
&  0.40153E+00_JPRB, 0.35154E+00_JPRB, 0.31412E+00_JPRB, 0.31036E+00_JPRB, 0.32596E+00_JPRB, 0.32796E+00_JPRB, 0.32310E+00_JPRB ,&
&  0.31752E+00_JPRB, 0.31265E+00_JPRB, 0.30981E+00_JPRB, 0.30761E+00_JPRB, 0.30485E+00_JPRB, 0.30225E+00_JPRB, 0.30267E+00_JPRB ,&
&  0.30379E+00_JPRB, 0.30153E+00_JPRB, 0.29845E+00_JPRB, 0.29208E+00_JPRB, 0.28754E+00_JPRB, 0.28597E+00_JPRB, 0.28472E+00_JPRB ,&
&  0.28057E+00_JPRB, 0.27643E+00_JPRB, 0.27651E+00_JPRB, 0.27791E+00_JPRB, 0.28002E+00_JPRB, 0.28144E+00_JPRB, 0.27868E+00_JPRB ,&
&  0.27350E+00_JPRB, 0.26688E+00_JPRB, 0.25886E+00_JPRB, 0.24914E+00_JPRB, 0.23996E+00_JPRB, 0.23318E+00_JPRB, 0.22774E+00_JPRB ,&
&  0.22720E+00_JPRB, 0.23362E+00_JPRB, 0.24752E+00_JPRB, 0.26310E+00_JPRB, 0.27942E+00_JPRB, 0.27069E+00_JPRB, 0.26042E+00_JPRB ,&
&  0.24946E+00_JPRB, 0.24024E+00_JPRB, 0.23352E+00_JPRB, 0.23049E+00_JPRB, 0.22928E+00_JPRB, 0.22920E+00_JPRB, 0.23039E+00_JPRB ,&
&  0.23259E+00_JPRB, 0.23533E+00_JPRB, 0.23762E+00_JPRB, 0.23868E+00_JPRB, 0.23917E+00_JPRB, 0.23952E+00_JPRB, 0.23920E+00_JPRB ,&
&  0.23855E+00_JPRB, 0.24111E+00_JPRB, 0.24810E+00_JPRB, 0.25680E+00_JPRB, 0.26504E+00_JPRB, 0.27299E+00_JPRB, 0.28603E+00_JPRB ,&
&  0.30204E+00_JPRB, 0.32162E+00_JPRB, 0.33466E+00_JPRB, 0.34472E+00_JPRB, 0.35317E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.34867E+00_JPRB, 0.35769E+00_JPRB, 0.36949E+00_JPRB, 0.37983E+00_JPRB, 0.38905E+00_JPRB, 0.39770E+00_JPRB, 0.40548E+00_JPRB ,&
&  0.40986E+00_JPRB, 0.41239E+00_JPRB, 0.41340E+00_JPRB, 0.41701E+00_JPRB, 0.41388E+00_JPRB, 0.40668E+00_JPRB, 0.40101E+00_JPRB ,&
&  0.39625E+00_JPRB, 0.39259E+00_JPRB, 0.38725E+00_JPRB, 0.38442E+00_JPRB, 0.38408E+00_JPRB, 0.38404E+00_JPRB, 0.37827E+00_JPRB ,&
&  0.36750E+00_JPRB, 0.36612E+00_JPRB, 0.37060E+00_JPRB, 0.36490E+00_JPRB, 0.35901E+00_JPRB, 0.35327E+00_JPRB, 0.34720E+00_JPRB ,&
&  0.33076E+00_JPRB, 0.31036E+00_JPRB, 0.28375E+00_JPRB, 0.28054E+00_JPRB, 0.29628E+00_JPRB, 0.28938E+00_JPRB, 0.29291E+00_JPRB ,&
&  0.29373E+00_JPRB, 0.29387E+00_JPRB, 0.29238E+00_JPRB, 0.29072E+00_JPRB, 0.28716E+00_JPRB, 0.28599E+00_JPRB, 0.28421E+00_JPRB ,&
&  0.27802E+00_JPRB, 0.27289E+00_JPRB, 0.26463E+00_JPRB, 0.24656E+00_JPRB, 0.24200E+00_JPRB, 0.24089E+00_JPRB, 0.23638E+00_JPRB ,&
&  0.22881E+00_JPRB, 0.22368E+00_JPRB, 0.22189E+00_JPRB, 0.21773E+00_JPRB, 0.21994E+00_JPRB, 0.22210E+00_JPRB, 0.22203E+00_JPRB ,&
&  0.22197E+00_JPRB, 0.22152E+00_JPRB, 0.21921E+00_JPRB, 0.21529E+00_JPRB, 0.21219E+00_JPRB, 0.21127E+00_JPRB, 0.21239E+00_JPRB ,&
&  0.21496E+00_JPRB, 0.21952E+00_JPRB, 0.23081E+00_JPRB, 0.23954E+00_JPRB, 0.23833E+00_JPRB, 0.23423E+00_JPRB, 0.22507E+00_JPRB ,&
&  0.21656E+00_JPRB, 0.21343E+00_JPRB, 0.21067E+00_JPRB, 0.20913E+00_JPRB, 0.20936E+00_JPRB, 0.21080E+00_JPRB, 0.21149E+00_JPRB ,&
&  0.21261E+00_JPRB, 0.21378E+00_JPRB, 0.21464E+00_JPRB, 0.21509E+00_JPRB, 0.21587E+00_JPRB, 0.21678E+00_JPRB, 0.21854E+00_JPRB ,&
&  0.22114E+00_JPRB, 0.22564E+00_JPRB, 0.23148E+00_JPRB, 0.23779E+00_JPRB, 0.24243E+00_JPRB, 0.24477E+00_JPRB, 0.24927E+00_JPRB ,&
&  0.25530E+00_JPRB, 0.27097E+00_JPRB, 0.29987E+00_JPRB, 0.34424E+00_JPRB, 0.34760E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.34765E+00_JPRB, 0.35986E+00_JPRB, 0.37510E+00_JPRB, 0.38469E+00_JPRB, 0.39036E+00_JPRB, 0.39326E+00_JPRB, 0.39242E+00_JPRB ,&
&  0.38908E+00_JPRB, 0.38846E+00_JPRB, 0.39944E+00_JPRB, 0.38104E+00_JPRB, 0.36360E+00_JPRB, 0.33075E+00_JPRB, 0.31419E+00_JPRB ,&
&  0.30151E+00_JPRB, 0.29924E+00_JPRB, 0.29876E+00_JPRB, 0.30335E+00_JPRB, 0.30698E+00_JPRB, 0.30981E+00_JPRB, 0.30578E+00_JPRB ,&
&  0.30019E+00_JPRB, 0.29086E+00_JPRB, 0.28302E+00_JPRB, 0.27576E+00_JPRB, 0.27596E+00_JPRB, 0.27479E+00_JPRB, 0.27214E+00_JPRB ,&
&  0.26923E+00_JPRB, 0.26749E+00_JPRB, 0.26755E+00_JPRB, 0.26818E+00_JPRB, 0.27974E+00_JPRB, 0.27862E+00_JPRB, 0.27908E+00_JPRB ,&
&  0.27888E+00_JPRB, 0.27885E+00_JPRB, 0.27781E+00_JPRB, 0.27602E+00_JPRB, 0.27111E+00_JPRB, 0.25657E+00_JPRB, 0.25535E+00_JPRB ,&
&  0.24665E+00_JPRB, 0.23617E+00_JPRB, 0.22592E+00_JPRB, 0.21879E+00_JPRB, 0.21446E+00_JPRB, 0.21279E+00_JPRB, 0.20813E+00_JPRB ,&
&  0.20318E+00_JPRB, 0.19730E+00_JPRB, 0.19467E+00_JPRB, 0.19465E+00_JPRB, 0.19536E+00_JPRB, 0.19598E+00_JPRB, 0.19648E+00_JPRB ,&
&  0.19785E+00_JPRB, 0.19850E+00_JPRB, 0.19892E+00_JPRB, 0.19972E+00_JPRB, 0.20052E+00_JPRB, 0.20167E+00_JPRB, 0.20357E+00_JPRB ,&
&  0.20692E+00_JPRB, 0.21046E+00_JPRB, 0.21380E+00_JPRB, 0.21686E+00_JPRB, 0.21588E+00_JPRB, 0.21216E+00_JPRB, 0.20733E+00_JPRB ,&
&  0.20373E+00_JPRB, 0.20246E+00_JPRB, 0.20125E+00_JPRB, 0.20105E+00_JPRB, 0.20112E+00_JPRB, 0.20148E+00_JPRB, 0.20207E+00_JPRB ,&
&  0.20292E+00_JPRB, 0.20365E+00_JPRB, 0.20453E+00_JPRB, 0.20567E+00_JPRB, 0.20726E+00_JPRB, 0.20869E+00_JPRB, 0.21070E+00_JPRB ,&
&  0.21368E+00_JPRB, 0.21700E+00_JPRB, 0.22169E+00_JPRB, 0.22599E+00_JPRB, 0.23204E+00_JPRB, 0.24041E+00_JPRB, 0.25410E+00_JPRB ,&
&  0.26860E+00_JPRB, 0.30225E+00_JPRB, 0.31739E+00_JPRB, 0.32953E+00_JPRB, 0.33741E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.34767E+00_JPRB, 0.35749E+00_JPRB, 0.37154E+00_JPRB, 0.38077E+00_JPRB, 0.38796E+00_JPRB, 0.38385E+00_JPRB, 0.37990E+00_JPRB ,&
&  0.37571E+00_JPRB, 0.36190E+00_JPRB, 0.34466E+00_JPRB, 0.31553E+00_JPRB, 0.29102E+00_JPRB, 0.27233E+00_JPRB, 0.26166E+00_JPRB ,&
&  0.25502E+00_JPRB, 0.25267E+00_JPRB, 0.25024E+00_JPRB, 0.24945E+00_JPRB, 0.24784E+00_JPRB, 0.24914E+00_JPRB, 0.25019E+00_JPRB ,&
&  0.24957E+00_JPRB, 0.24859E+00_JPRB, 0.24815E+00_JPRB, 0.25133E+00_JPRB, 0.25755E+00_JPRB, 0.26238E+00_JPRB, 0.26708E+00_JPRB ,&
&  0.27033E+00_JPRB, 0.26754E+00_JPRB, 0.26903E+00_JPRB, 0.26722E+00_JPRB, 0.26575E+00_JPRB, 0.26651E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26836E+00_JPRB, 0.26847E+00_JPRB, 0.26749E+00_JPRB, 0.26402E+00_JPRB, 0.25174E+00_JPRB, 0.24548E+00_JPRB, 0.23733E+00_JPRB ,&
&  0.22926E+00_JPRB, 0.21983E+00_JPRB, 0.21204E+00_JPRB, 0.20841E+00_JPRB, 0.20375E+00_JPRB, 0.20206E+00_JPRB, 0.19879E+00_JPRB ,&
&  0.19510E+00_JPRB, 0.19164E+00_JPRB, 0.18951E+00_JPRB, 0.18847E+00_JPRB, 0.18813E+00_JPRB, 0.18686E+00_JPRB, 0.18646E+00_JPRB ,&
&  0.18685E+00_JPRB, 0.18723E+00_JPRB, 0.18824E+00_JPRB, 0.18916E+00_JPRB, 0.19012E+00_JPRB, 0.19095E+00_JPRB, 0.19193E+00_JPRB ,&
&  0.19360E+00_JPRB, 0.19570E+00_JPRB, 0.19664E+00_JPRB, 0.19748E+00_JPRB, 0.19716E+00_JPRB, 0.19653E+00_JPRB, 0.19561E+00_JPRB ,&
&  0.19486E+00_JPRB, 0.19461E+00_JPRB, 0.19459E+00_JPRB, 0.19448E+00_JPRB, 0.19494E+00_JPRB, 0.19582E+00_JPRB, 0.19671E+00_JPRB ,&
&  0.19828E+00_JPRB, 0.19969E+00_JPRB, 0.20131E+00_JPRB, 0.20330E+00_JPRB, 0.20551E+00_JPRB, 0.20818E+00_JPRB, 0.21114E+00_JPRB ,&
&  0.21371E+00_JPRB, 0.21708E+00_JPRB, 0.21996E+00_JPRB, 0.22230E+00_JPRB, 0.22820E+00_JPRB, 0.23563E+00_JPRB, 0.25061E+00_JPRB ,&
&  0.26719E+00_JPRB, 0.30616E+00_JPRB, 0.34758E+00_JPRB, 0.33398E+00_JPRB, 0.33944E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.32043E+00_JPRB, 0.33154E+00_JPRB, 0.34026E+00_JPRB, 0.34574E+00_JPRB, 0.35117E+00_JPRB, 0.35316E+00_JPRB, 0.36090E+00_JPRB ,&
&  0.35546E+00_JPRB, 0.33759E+00_JPRB, 0.32645E+00_JPRB, 0.30992E+00_JPRB, 0.28963E+00_JPRB, 0.27291E+00_JPRB, 0.26432E+00_JPRB ,&
&  0.25914E+00_JPRB, 0.25491E+00_JPRB, 0.24989E+00_JPRB, 0.24443E+00_JPRB, 0.23897E+00_JPRB, 0.23489E+00_JPRB, 0.23236E+00_JPRB ,&
&  0.23004E+00_JPRB, 0.23091E+00_JPRB, 0.23382E+00_JPRB, 0.23843E+00_JPRB, 0.24504E+00_JPRB, 0.24928E+00_JPRB, 0.25269E+00_JPRB ,&
&  0.25429E+00_JPRB, 0.24468E+00_JPRB, 0.24711E+00_JPRB, 0.24964E+00_JPRB, 0.24858E+00_JPRB, 0.25243E+00_JPRB, 0.25867E+00_JPRB ,&
&  0.25563E+00_JPRB, 0.24729E+00_JPRB, 0.24954E+00_JPRB, 0.24362E+00_JPRB, 0.23501E+00_JPRB, 0.23355E+00_JPRB, 0.22766E+00_JPRB ,&
&  0.22535E+00_JPRB, 0.21821E+00_JPRB, 0.21306E+00_JPRB, 0.20967E+00_JPRB, 0.20351E+00_JPRB, 0.20104E+00_JPRB, 0.19737E+00_JPRB ,&
&  0.19372E+00_JPRB, 0.19129E+00_JPRB, 0.18837E+00_JPRB, 0.18707E+00_JPRB, 0.18592E+00_JPRB, 0.18458E+00_JPRB, 0.18353E+00_JPRB ,&
&  0.18292E+00_JPRB, 0.18235E+00_JPRB, 0.18236E+00_JPRB, 0.18235E+00_JPRB, 0.18227E+00_JPRB, 0.18256E+00_JPRB, 0.18268E+00_JPRB ,&
&  0.18330E+00_JPRB, 0.18407E+00_JPRB, 0.18468E+00_JPRB, 0.18573E+00_JPRB, 0.18614E+00_JPRB, 0.18672E+00_JPRB, 0.18713E+00_JPRB ,&
&  0.18755E+00_JPRB, 0.18821E+00_JPRB, 0.18848E+00_JPRB, 0.18913E+00_JPRB, 0.18984E+00_JPRB, 0.19135E+00_JPRB, 0.19283E+00_JPRB ,&
&  0.19465E+00_JPRB, 0.19683E+00_JPRB, 0.19913E+00_JPRB, 0.20156E+00_JPRB, 0.20460E+00_JPRB, 0.20734E+00_JPRB, 0.21100E+00_JPRB ,&
&  0.21496E+00_JPRB, 0.21876E+00_JPRB, 0.22338E+00_JPRB, 0.22911E+00_JPRB, 0.23677E+00_JPRB, 0.24663E+00_JPRB, 0.25540E+00_JPRB ,&
&  0.26537E+00_JPRB, 0.29086E+00_JPRB, 0.29729E+00_JPRB, 0.30261E+00_JPRB, 0.31157E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.31127E+00_JPRB, 0.31636E+00_JPRB, 0.32028E+00_JPRB, 0.32282E+00_JPRB, 0.32531E+00_JPRB, 0.32699E+00_JPRB, 0.32762E+00_JPRB ,&
&  0.32110E+00_JPRB, 0.31618E+00_JPRB, 0.30913E+00_JPRB, 0.30057E+00_JPRB, 0.29273E+00_JPRB, 0.28503E+00_JPRB, 0.27667E+00_JPRB ,&
&  0.26980E+00_JPRB, 0.26332E+00_JPRB, 0.25672E+00_JPRB, 0.24974E+00_JPRB, 0.24246E+00_JPRB, 0.23513E+00_JPRB, 0.23198E+00_JPRB ,&
&  0.23129E+00_JPRB, 0.23279E+00_JPRB, 0.23523E+00_JPRB, 0.23678E+00_JPRB, 0.23767E+00_JPRB, 0.23816E+00_JPRB, 0.23696E+00_JPRB ,&
&  0.23311E+00_JPRB, 0.23397E+00_JPRB, 0.23519E+00_JPRB, 0.23601E+00_JPRB, 0.23778E+00_JPRB, 0.23927E+00_JPRB, 0.23913E+00_JPRB ,&
&  0.23716E+00_JPRB, 0.23458E+00_JPRB, 0.23105E+00_JPRB, 0.22775E+00_JPRB, 0.22663E+00_JPRB, 0.22452E+00_JPRB, 0.22211E+00_JPRB ,&
&  0.22042E+00_JPRB, 0.21654E+00_JPRB, 0.21237E+00_JPRB, 0.20941E+00_JPRB, 0.20538E+00_JPRB, 0.20266E+00_JPRB, 0.19908E+00_JPRB ,&
&  0.19624E+00_JPRB, 0.19372E+00_JPRB, 0.19095E+00_JPRB, 0.18884E+00_JPRB, 0.18689E+00_JPRB, 0.18544E+00_JPRB, 0.18398E+00_JPRB ,&
&  0.18314E+00_JPRB, 0.18231E+00_JPRB, 0.18177E+00_JPRB, 0.18116E+00_JPRB, 0.18076E+00_JPRB, 0.18038E+00_JPRB, 0.18007E+00_JPRB ,&
&  0.17989E+00_JPRB, 0.18022E+00_JPRB, 0.18044E+00_JPRB, 0.18085E+00_JPRB, 0.18132E+00_JPRB, 0.18158E+00_JPRB, 0.18211E+00_JPRB ,&
&  0.18317E+00_JPRB, 0.18432E+00_JPRB, 0.18503E+00_JPRB, 0.18598E+00_JPRB, 0.18706E+00_JPRB, 0.18837E+00_JPRB, 0.19019E+00_JPRB ,&
&  0.19210E+00_JPRB, 0.19425E+00_JPRB, 0.19668E+00_JPRB, 0.19906E+00_JPRB, 0.20218E+00_JPRB, 0.20538E+00_JPRB, 0.20876E+00_JPRB ,&
&  0.21284E+00_JPRB, 0.21717E+00_JPRB, 0.22166E+00_JPRB, 0.22661E+00_JPRB, 0.23375E+00_JPRB, 0.23971E+00_JPRB, 0.24708E+00_JPRB ,&
&  0.25410E+00_JPRB, 0.25880E+00_JPRB, 0.29148E+00_JPRB, 0.29630E+00_JPRB, 0.30315E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.23671E+00_JPRB, 0.24327E+00_JPRB, 0.24967E+00_JPRB, 0.25880E+00_JPRB, 0.26408E+00_JPRB, 0.30000E+00_JPRB, 0.29497E+00_JPRB ,&
&  0.29194E+00_JPRB, 0.27911E+00_JPRB, 0.27795E+00_JPRB, 0.27449E+00_JPRB, 0.27004E+00_JPRB, 0.26522E+00_JPRB, 0.26030E+00_JPRB ,&
&  0.25571E+00_JPRB, 0.25027E+00_JPRB, 0.24574E+00_JPRB, 0.24213E+00_JPRB, 0.23891E+00_JPRB, 0.23574E+00_JPRB, 0.23300E+00_JPRB ,&
&  0.23188E+00_JPRB, 0.23182E+00_JPRB, 0.23107E+00_JPRB, 0.22661E+00_JPRB, 0.22303E+00_JPRB, 0.22087E+00_JPRB, 0.22028E+00_JPRB ,&
&  0.22028E+00_JPRB, 0.22222E+00_JPRB, 0.22355E+00_JPRB, 0.22468E+00_JPRB, 0.22630E+00_JPRB, 0.22541E+00_JPRB, 0.22453E+00_JPRB ,&
&  0.22324E+00_JPRB, 0.22185E+00_JPRB, 0.22042E+00_JPRB, 0.21915E+00_JPRB, 0.21789E+00_JPRB, 0.21602E+00_JPRB, 0.21445E+00_JPRB ,&
&  0.21295E+00_JPRB, 0.21093E+00_JPRB, 0.20808E+00_JPRB, 0.20619E+00_JPRB, 0.20394E+00_JPRB, 0.20205E+00_JPRB, 0.20011E+00_JPRB ,&
&  0.19867E+00_JPRB, 0.19736E+00_JPRB, 0.19586E+00_JPRB, 0.19404E+00_JPRB, 0.19226E+00_JPRB, 0.19068E+00_JPRB, 0.18919E+00_JPRB ,&
&  0.18840E+00_JPRB, 0.18756E+00_JPRB, 0.18674E+00_JPRB, 0.18585E+00_JPRB, 0.18542E+00_JPRB, 0.18496E+00_JPRB, 0.18453E+00_JPRB ,&
&  0.18417E+00_JPRB, 0.18394E+00_JPRB, 0.18394E+00_JPRB, 0.18400E+00_JPRB, 0.18409E+00_JPRB, 0.18445E+00_JPRB, 0.18490E+00_JPRB ,&
&  0.18552E+00_JPRB, 0.18628E+00_JPRB, 0.18736E+00_JPRB, 0.18847E+00_JPRB, 0.18947E+00_JPRB, 0.19026E+00_JPRB, 0.19127E+00_JPRB ,&
&  0.19289E+00_JPRB, 0.19475E+00_JPRB, 0.19678E+00_JPRB, 0.19830E+00_JPRB, 0.20055E+00_JPRB, 0.20258E+00_JPRB, 0.20462E+00_JPRB ,&
&  0.20663E+00_JPRB, 0.20911E+00_JPRB, 0.21191E+00_JPRB, 0.21495E+00_JPRB, 0.21852E+00_JPRB, 0.22139E+00_JPRB, 0.22409E+00_JPRB ,&
&  0.22684E+00_JPRB, 0.23087E+00_JPRB, 0.23281E+00_JPRB, 0.23380E+00_JPRB, 0.23503E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.21580E+00_JPRB, 0.21586E+00_JPRB, 0.21622E+00_JPRB, 0.21661E+00_JPRB, 0.21696E+00_JPRB, 0.21759E+00_JPRB, 0.21771E+00_JPRB ,&
&  0.21764E+00_JPRB, 0.21749E+00_JPRB, 0.21783E+00_JPRB, 0.21823E+00_JPRB, 0.21847E+00_JPRB, 0.21848E+00_JPRB, 0.21808E+00_JPRB ,&
&  0.21737E+00_JPRB, 0.21650E+00_JPRB, 0.21510E+00_JPRB, 0.21448E+00_JPRB, 0.21404E+00_JPRB, 0.21363E+00_JPRB, 0.21285E+00_JPRB ,&
&  0.21215E+00_JPRB, 0.21094E+00_JPRB, 0.20929E+00_JPRB, 0.20810E+00_JPRB, 0.20796E+00_JPRB, 0.20835E+00_JPRB, 0.20907E+00_JPRB ,&
&  0.20963E+00_JPRB, 0.21001E+00_JPRB, 0.21017E+00_JPRB, 0.21040E+00_JPRB, 0.21072E+00_JPRB, 0.21058E+00_JPRB, 0.21033E+00_JPRB ,&
&  0.20999E+00_JPRB, 0.20943E+00_JPRB, 0.20890E+00_JPRB, 0.20841E+00_JPRB, 0.20797E+00_JPRB, 0.20746E+00_JPRB, 0.20683E+00_JPRB ,&
&  0.20611E+00_JPRB, 0.20523E+00_JPRB, 0.20416E+00_JPRB, 0.20330E+00_JPRB, 0.20240E+00_JPRB, 0.20169E+00_JPRB, 0.20079E+00_JPRB ,&
&  0.20005E+00_JPRB, 0.19937E+00_JPRB, 0.19868E+00_JPRB, 0.19804E+00_JPRB, 0.19744E+00_JPRB, 0.19686E+00_JPRB, 0.19630E+00_JPRB ,&
&  0.19577E+00_JPRB, 0.19533E+00_JPRB, 0.19484E+00_JPRB, 0.19437E+00_JPRB, 0.19406E+00_JPRB, 0.19379E+00_JPRB, 0.19357E+00_JPRB ,&
&  0.19338E+00_JPRB, 0.19332E+00_JPRB, 0.19331E+00_JPRB, 0.19328E+00_JPRB, 0.19328E+00_JPRB, 0.19340E+00_JPRB, 0.19352E+00_JPRB ,&
&  0.19369E+00_JPRB, 0.19380E+00_JPRB, 0.19393E+00_JPRB, 0.19412E+00_JPRB, 0.19432E+00_JPRB, 0.19456E+00_JPRB, 0.19489E+00_JPRB ,&
&  0.19528E+00_JPRB, 0.19575E+00_JPRB, 0.19625E+00_JPRB, 0.19654E+00_JPRB, 0.19722E+00_JPRB, 0.19800E+00_JPRB, 0.19891E+00_JPRB ,&
&  0.19991E+00_JPRB, 0.20112E+00_JPRB, 0.20246E+00_JPRB, 0.20406E+00_JPRB, 0.20572E+00_JPRB, 0.20675E+00_JPRB, 0.20779E+00_JPRB ,&
&  0.20887E+00_JPRB, 0.21015E+00_JPRB, 0.21136E+00_JPRB, 0.21259E+00_JPRB, 0.21406E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB, 0.20012E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB, 0.45192E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.46096E+00_JPRB, 0.46054E+00_JPRB, 0.46014E+00_JPRB, 0.45968E+00_JPRB, 0.45909E+00_JPRB, 0.45850E+00_JPRB, 0.45800E+00_JPRB ,&
&  0.45749E+00_JPRB, 0.45713E+00_JPRB, 0.45651E+00_JPRB, 0.45588E+00_JPRB, 0.45524E+00_JPRB, 0.45468E+00_JPRB, 0.45407E+00_JPRB ,&
&  0.45344E+00_JPRB, 0.45282E+00_JPRB, 0.45234E+00_JPRB, 0.45176E+00_JPRB, 0.45113E+00_JPRB, 0.45055E+00_JPRB, 0.45004E+00_JPRB ,&
&  0.44955E+00_JPRB, 0.44900E+00_JPRB, 0.44842E+00_JPRB, 0.44773E+00_JPRB, 0.44728E+00_JPRB, 0.44665E+00_JPRB, 0.44612E+00_JPRB ,&
&  0.44552E+00_JPRB, 0.44509E+00_JPRB, 0.44463E+00_JPRB, 0.44418E+00_JPRB, 0.44366E+00_JPRB, 0.44358E+00_JPRB, 0.44326E+00_JPRB ,&
&  0.44311E+00_JPRB, 0.44294E+00_JPRB, 0.44295E+00_JPRB, 0.44290E+00_JPRB, 0.44290E+00_JPRB, 0.44296E+00_JPRB, 0.44332E+00_JPRB ,&
&  0.44361E+00_JPRB, 0.44396E+00_JPRB, 0.44449E+00_JPRB, 0.44498E+00_JPRB, 0.44543E+00_JPRB, 0.44590E+00_JPRB, 0.44643E+00_JPRB ,&
&  0.44715E+00_JPRB, 0.44785E+00_JPRB, 0.44850E+00_JPRB, 0.44918E+00_JPRB, 0.44984E+00_JPRB, 0.45037E+00_JPRB, 0.45082E+00_JPRB ,&
&  0.45109E+00_JPRB, 0.45143E+00_JPRB, 0.45175E+00_JPRB, 0.45211E+00_JPRB, 0.45220E+00_JPRB, 0.45244E+00_JPRB, 0.45256E+00_JPRB ,&
&  0.45266E+00_JPRB, 0.45246E+00_JPRB, 0.45252E+00_JPRB, 0.45245E+00_JPRB, 0.45252E+00_JPRB, 0.45250E+00_JPRB, 0.45277E+00_JPRB ,&
&  0.45297E+00_JPRB, 0.45341E+00_JPRB, 0.45388E+00_JPRB, 0.45452E+00_JPRB, 0.45501E+00_JPRB, 0.45556E+00_JPRB, 0.45605E+00_JPRB ,&
&  0.45661E+00_JPRB, 0.45719E+00_JPRB, 0.45783E+00_JPRB, 0.45828E+00_JPRB, 0.45869E+00_JPRB, 0.45916E+00_JPRB, 0.45966E+00_JPRB ,&
&  0.46031E+00_JPRB, 0.46070E+00_JPRB, 0.46113E+00_JPRB, 0.46148E+00_JPRB, 0.46163E+00_JPRB, 0.46176E+00_JPRB, 0.46176E+00_JPRB ,&
&  0.46170E+00_JPRB, 0.46161E+00_JPRB, 0.46147E+00_JPRB, 0.46136E+00_JPRB, 0.46124E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.46834E+00_JPRB, 0.46810E+00_JPRB, 0.46767E+00_JPRB, 0.46699E+00_JPRB, 0.46617E+00_JPRB, 0.46518E+00_JPRB, 0.46421E+00_JPRB ,&
&  0.46311E+00_JPRB, 0.46204E+00_JPRB, 0.46073E+00_JPRB, 0.45945E+00_JPRB, 0.45813E+00_JPRB, 0.45689E+00_JPRB, 0.45559E+00_JPRB ,&
&  0.45436E+00_JPRB, 0.45313E+00_JPRB, 0.45228E+00_JPRB, 0.45137E+00_JPRB, 0.45037E+00_JPRB, 0.44916E+00_JPRB, 0.44814E+00_JPRB ,&
&  0.44706E+00_JPRB, 0.44578E+00_JPRB, 0.44437E+00_JPRB, 0.44191E+00_JPRB, 0.43980E+00_JPRB, 0.43786E+00_JPRB, 0.43619E+00_JPRB ,&
&  0.43468E+00_JPRB, 0.43325E+00_JPRB, 0.43215E+00_JPRB, 0.43143E+00_JPRB, 0.43216E+00_JPRB, 0.43229E+00_JPRB, 0.43204E+00_JPRB ,&
&  0.43166E+00_JPRB, 0.43017E+00_JPRB, 0.42969E+00_JPRB, 0.42955E+00_JPRB, 0.42958E+00_JPRB, 0.43061E+00_JPRB, 0.43146E+00_JPRB ,&
&  0.43238E+00_JPRB, 0.43312E+00_JPRB, 0.43461E+00_JPRB, 0.43602E+00_JPRB, 0.43734E+00_JPRB, 0.43876E+00_JPRB, 0.43960E+00_JPRB ,&
&  0.44054E+00_JPRB, 0.44192E+00_JPRB, 0.44325E+00_JPRB, 0.44465E+00_JPRB, 0.44550E+00_JPRB, 0.44628E+00_JPRB, 0.44686E+00_JPRB ,&
&  0.44736E+00_JPRB, 0.44804E+00_JPRB, 0.44878E+00_JPRB, 0.44948E+00_JPRB, 0.44996E+00_JPRB, 0.45038E+00_JPRB, 0.45065E+00_JPRB ,&
&  0.45074E+00_JPRB, 0.45039E+00_JPRB, 0.45024E+00_JPRB, 0.45003E+00_JPRB, 0.44964E+00_JPRB, 0.44917E+00_JPRB, 0.44904E+00_JPRB ,&
&  0.44888E+00_JPRB, 0.44856E+00_JPRB, 0.44808E+00_JPRB, 0.44833E+00_JPRB, 0.44866E+00_JPRB, 0.44884E+00_JPRB, 0.44887E+00_JPRB ,&
&  0.45042E+00_JPRB, 0.45259E+00_JPRB, 0.45512E+00_JPRB, 0.45851E+00_JPRB, 0.46102E+00_JPRB, 0.46263E+00_JPRB, 0.46374E+00_JPRB ,&
&  0.46421E+00_JPRB, 0.46468E+00_JPRB, 0.46511E+00_JPRB, 0.46579E+00_JPRB, 0.46627E+00_JPRB, 0.46711E+00_JPRB, 0.46822E+00_JPRB ,&
&  0.46899E+00_JPRB, 0.46921E+00_JPRB, 0.46917E+00_JPRB, 0.46893E+00_JPRB, 0.46864E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.47100E+00_JPRB, 0.47009E+00_JPRB, 0.46918E+00_JPRB, 0.46778E+00_JPRB, 0.46692E+00_JPRB, 0.46570E+00_JPRB, 0.46459E+00_JPRB ,&
&  0.46326E+00_JPRB, 0.46174E+00_JPRB, 0.46058E+00_JPRB, 0.45962E+00_JPRB, 0.45861E+00_JPRB, 0.45762E+00_JPRB, 0.45653E+00_JPRB ,&
&  0.45540E+00_JPRB, 0.45412E+00_JPRB, 0.45356E+00_JPRB, 0.45269E+00_JPRB, 0.45171E+00_JPRB, 0.45060E+00_JPRB, 0.44784E+00_JPRB ,&
&  0.44514E+00_JPRB, 0.44181E+00_JPRB, 0.43834E+00_JPRB, 0.43534E+00_JPRB, 0.43345E+00_JPRB, 0.42064E+00_JPRB, 0.41852E+00_JPRB ,&
&  0.42363E+00_JPRB, 0.42648E+00_JPRB, 0.42972E+00_JPRB, 0.43308E+00_JPRB, 0.42623E+00_JPRB, 0.41780E+00_JPRB, 0.41285E+00_JPRB ,&
&  0.40094E+00_JPRB, 0.40562E+00_JPRB, 0.40802E+00_JPRB, 0.41029E+00_JPRB, 0.41369E+00_JPRB, 0.41381E+00_JPRB, 0.41598E+00_JPRB ,&
&  0.42384E+00_JPRB, 0.42391E+00_JPRB, 0.42496E+00_JPRB, 0.42614E+00_JPRB, 0.42728E+00_JPRB, 0.42843E+00_JPRB, 0.43059E+00_JPRB ,&
&  0.43233E+00_JPRB, 0.43434E+00_JPRB, 0.43656E+00_JPRB, 0.43762E+00_JPRB, 0.43872E+00_JPRB, 0.44022E+00_JPRB, 0.44162E+00_JPRB ,&
&  0.44232E+00_JPRB, 0.44323E+00_JPRB, 0.44424E+00_JPRB, 0.44517E+00_JPRB, 0.44644E+00_JPRB, 0.44726E+00_JPRB, 0.44793E+00_JPRB ,&
&  0.44843E+00_JPRB, 0.44829E+00_JPRB, 0.44822E+00_JPRB, 0.44834E+00_JPRB, 0.44824E+00_JPRB, 0.44800E+00_JPRB, 0.44812E+00_JPRB ,&
&  0.44857E+00_JPRB, 0.44890E+00_JPRB, 0.44957E+00_JPRB, 0.45061E+00_JPRB, 0.45319E+00_JPRB, 0.45675E+00_JPRB, 0.46379E+00_JPRB ,&
&  0.46956E+00_JPRB, 0.47454E+00_JPRB, 0.47633E+00_JPRB, 0.47820E+00_JPRB, 0.47959E+00_JPRB, 0.47985E+00_JPRB, 0.47970E+00_JPRB ,&
&  0.47876E+00_JPRB, 0.47713E+00_JPRB, 0.47553E+00_JPRB, 0.47399E+00_JPRB, 0.47311E+00_JPRB, 0.47250E+00_JPRB, 0.47253E+00_JPRB ,&
&  0.47306E+00_JPRB, 0.47390E+00_JPRB, 0.47409E+00_JPRB, 0.47312E+00_JPRB, 0.47192E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.47728E+00_JPRB, 0.47514E+00_JPRB, 0.47432E+00_JPRB, 0.47250E+00_JPRB, 0.47067E+00_JPRB, 0.46835E+00_JPRB, 0.46644E+00_JPRB ,&
&  0.46430E+00_JPRB, 0.46240E+00_JPRB, 0.46079E+00_JPRB, 0.45931E+00_JPRB, 0.45857E+00_JPRB, 0.45772E+00_JPRB, 0.45718E+00_JPRB ,&
&  0.45647E+00_JPRB, 0.45549E+00_JPRB, 0.45496E+00_JPRB, 0.45415E+00_JPRB, 0.45333E+00_JPRB, 0.45237E+00_JPRB, 0.45010E+00_JPRB ,&
&  0.44602E+00_JPRB, 0.44037E+00_JPRB, 0.42838E+00_JPRB, 0.43033E+00_JPRB, 0.42889E+00_JPRB, 0.42737E+00_JPRB, 0.42795E+00_JPRB ,&
&  0.42203E+00_JPRB, 0.42039E+00_JPRB, 0.41370E+00_JPRB, 0.40769E+00_JPRB, 0.40549E+00_JPRB, 0.39956E+00_JPRB, 0.39949E+00_JPRB ,&
&  0.39738E+00_JPRB, 0.39632E+00_JPRB, 0.39706E+00_JPRB, 0.39672E+00_JPRB, 0.40025E+00_JPRB, 0.40199E+00_JPRB, 0.40468E+00_JPRB ,&
&  0.40951E+00_JPRB, 0.41343E+00_JPRB, 0.41608E+00_JPRB, 0.41204E+00_JPRB, 0.41412E+00_JPRB, 0.41622E+00_JPRB, 0.41887E+00_JPRB ,&
&  0.42136E+00_JPRB, 0.42309E+00_JPRB, 0.42488E+00_JPRB, 0.42616E+00_JPRB, 0.42774E+00_JPRB, 0.43009E+00_JPRB, 0.43256E+00_JPRB ,&
&  0.43553E+00_JPRB, 0.43760E+00_JPRB, 0.43894E+00_JPRB, 0.44030E+00_JPRB, 0.44176E+00_JPRB, 0.44314E+00_JPRB, 0.44487E+00_JPRB ,&
&  0.44696E+00_JPRB, 0.44961E+00_JPRB, 0.45243E+00_JPRB, 0.45539E+00_JPRB, 0.45811E+00_JPRB, 0.46171E+00_JPRB, 0.46590E+00_JPRB ,&
&  0.47214E+00_JPRB, 0.47786E+00_JPRB, 0.48673E+00_JPRB, 0.49221E+00_JPRB, 0.50355E+00_JPRB, 0.51439E+00_JPRB, 0.52118E+00_JPRB ,&
&  0.51997E+00_JPRB, 0.51674E+00_JPRB, 0.51338E+00_JPRB, 0.50962E+00_JPRB, 0.50637E+00_JPRB, 0.50244E+00_JPRB, 0.49969E+00_JPRB ,&
&  0.49703E+00_JPRB, 0.49441E+00_JPRB, 0.49179E+00_JPRB, 0.48913E+00_JPRB, 0.48690E+00_JPRB, 0.48442E+00_JPRB, 0.48280E+00_JPRB ,&
&  0.48216E+00_JPRB, 0.48247E+00_JPRB, 0.48237E+00_JPRB, 0.48145E+00_JPRB, 0.47956E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.48455E+00_JPRB, 0.48409E+00_JPRB, 0.48494E+00_JPRB, 0.48403E+00_JPRB, 0.48181E+00_JPRB, 0.47963E+00_JPRB, 0.47539E+00_JPRB ,&
&  0.47122E+00_JPRB, 0.46713E+00_JPRB, 0.46354E+00_JPRB, 0.46088E+00_JPRB, 0.45872E+00_JPRB, 0.45696E+00_JPRB, 0.45626E+00_JPRB ,&
&  0.45519E+00_JPRB, 0.45507E+00_JPRB, 0.45502E+00_JPRB, 0.45513E+00_JPRB, 0.45577E+00_JPRB, 0.45598E+00_JPRB, 0.45601E+00_JPRB ,&
&  0.45342E+00_JPRB, 0.44669E+00_JPRB, 0.43763E+00_JPRB, 0.43913E+00_JPRB, 0.44007E+00_JPRB, 0.44183E+00_JPRB, 0.44383E+00_JPRB ,&
&  0.44206E+00_JPRB, 0.42463E+00_JPRB, 0.43009E+00_JPRB, 0.42207E+00_JPRB, 0.41781E+00_JPRB, 0.41296E+00_JPRB, 0.40031E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.38830E+00_JPRB, 0.38405E+00_JPRB, 0.38409E+00_JPRB, 0.38586E+00_JPRB, 0.39066E+00_JPRB, 0.39670E+00_JPRB ,&
&  0.39764E+00_JPRB, 0.40663E+00_JPRB, 0.40780E+00_JPRB, 0.40752E+00_JPRB, 0.40975E+00_JPRB, 0.41236E+00_JPRB, 0.41125E+00_JPRB ,&
&  0.41217E+00_JPRB, 0.41187E+00_JPRB, 0.41138E+00_JPRB, 0.41376E+00_JPRB, 0.41878E+00_JPRB, 0.42135E+00_JPRB, 0.42400E+00_JPRB ,&
&  0.42767E+00_JPRB, 0.42982E+00_JPRB, 0.43117E+00_JPRB, 0.43302E+00_JPRB, 0.43549E+00_JPRB, 0.43798E+00_JPRB, 0.44178E+00_JPRB ,&
&  0.44605E+00_JPRB, 0.45344E+00_JPRB, 0.46036E+00_JPRB, 0.47071E+00_JPRB, 0.48043E+00_JPRB, 0.49346E+00_JPRB, 0.50495E+00_JPRB ,&
&  0.51702E+00_JPRB, 0.52679E+00_JPRB, 0.53812E+00_JPRB, 0.54926E+00_JPRB, 0.55816E+00_JPRB, 0.55566E+00_JPRB, 0.54556E+00_JPRB ,&
&  0.53747E+00_JPRB, 0.52600E+00_JPRB, 0.51935E+00_JPRB, 0.51405E+00_JPRB, 0.51038E+00_JPRB, 0.50871E+00_JPRB, 0.50692E+00_JPRB ,&
&  0.50610E+00_JPRB, 0.50441E+00_JPRB, 0.50218E+00_JPRB, 0.49906E+00_JPRB, 0.49642E+00_JPRB, 0.49357E+00_JPRB, 0.49163E+00_JPRB ,&
&  0.48900E+00_JPRB, 0.48802E+00_JPRB, 0.48659E+00_JPRB, 0.48574E+00_JPRB, 0.48510E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.49190E+00_JPRB, 0.49442E+00_JPRB, 0.49370E+00_JPRB, 0.49227E+00_JPRB, 0.48760E+00_JPRB, 0.48368E+00_JPRB, 0.47754E+00_JPRB ,&
&  0.47166E+00_JPRB, 0.46577E+00_JPRB, 0.46050E+00_JPRB, 0.45692E+00_JPRB, 0.45425E+00_JPRB, 0.45239E+00_JPRB, 0.45161E+00_JPRB ,&
&  0.45145E+00_JPRB, 0.45205E+00_JPRB, 0.45262E+00_JPRB, 0.45329E+00_JPRB, 0.45493E+00_JPRB, 0.45686E+00_JPRB, 0.45895E+00_JPRB ,&
&  0.45934E+00_JPRB, 0.45636E+00_JPRB, 0.44791E+00_JPRB, 0.44711E+00_JPRB, 0.45135E+00_JPRB, 0.44857E+00_JPRB, 0.45733E+00_JPRB ,&
&  0.46354E+00_JPRB, 0.46623E+00_JPRB, 0.45971E+00_JPRB, 0.45524E+00_JPRB, 0.44643E+00_JPRB, 0.41000E+00_JPRB, 0.40729E+00_JPRB ,&
&  0.40061E+00_JPRB, 0.39078E+00_JPRB, 0.38644E+00_JPRB, 0.38289E+00_JPRB, 0.38231E+00_JPRB, 0.38829E+00_JPRB, 0.39685E+00_JPRB ,&
&  0.39904E+00_JPRB, 0.41285E+00_JPRB, 0.41296E+00_JPRB, 0.41250E+00_JPRB, 0.41364E+00_JPRB, 0.41680E+00_JPRB, 0.41589E+00_JPRB ,&
&  0.41694E+00_JPRB, 0.41734E+00_JPRB, 0.41735E+00_JPRB, 0.42069E+00_JPRB, 0.42405E+00_JPRB, 0.42675E+00_JPRB, 0.43102E+00_JPRB ,&
&  0.43997E+00_JPRB, 0.44581E+00_JPRB, 0.44924E+00_JPRB, 0.45230E+00_JPRB, 0.45730E+00_JPRB, 0.46280E+00_JPRB, 0.47042E+00_JPRB ,&
&  0.47617E+00_JPRB, 0.48295E+00_JPRB, 0.49098E+00_JPRB, 0.49908E+00_JPRB, 0.50744E+00_JPRB, 0.51650E+00_JPRB, 0.52662E+00_JPRB ,&
&  0.53200E+00_JPRB, 0.54004E+00_JPRB, 0.54909E+00_JPRB, 0.56581E+00_JPRB, 0.56697E+00_JPRB, 0.55547E+00_JPRB, 0.54642E+00_JPRB ,&
&  0.54008E+00_JPRB, 0.54033E+00_JPRB, 0.53467E+00_JPRB, 0.53120E+00_JPRB, 0.52728E+00_JPRB, 0.52599E+00_JPRB, 0.52330E+00_JPRB ,&
&  0.52129E+00_JPRB, 0.51808E+00_JPRB, 0.51365E+00_JPRB, 0.50849E+00_JPRB, 0.50457E+00_JPRB, 0.50034E+00_JPRB, 0.49725E+00_JPRB ,&
&  0.49340E+00_JPRB, 0.49156E+00_JPRB, 0.49029E+00_JPRB, 0.49007E+00_JPRB, 0.48963E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.49829E+00_JPRB, 0.50007E+00_JPRB, 0.49568E+00_JPRB, 0.49348E+00_JPRB, 0.48624E+00_JPRB, 0.47917E+00_JPRB, 0.47033E+00_JPRB ,&
&  0.46360E+00_JPRB, 0.45858E+00_JPRB, 0.45373E+00_JPRB, 0.45171E+00_JPRB, 0.45104E+00_JPRB, 0.45099E+00_JPRB, 0.45241E+00_JPRB ,&
&  0.45404E+00_JPRB, 0.45247E+00_JPRB, 0.45417E+00_JPRB, 0.45730E+00_JPRB, 0.45969E+00_JPRB, 0.46178E+00_JPRB, 0.46388E+00_JPRB ,&
&  0.46298E+00_JPRB, 0.46175E+00_JPRB, 0.46001E+00_JPRB, 0.45504E+00_JPRB, 0.45310E+00_JPRB, 0.45974E+00_JPRB, 0.46930E+00_JPRB ,&
&  0.46502E+00_JPRB, 0.46589E+00_JPRB, 0.47049E+00_JPRB, 0.47062E+00_JPRB, 0.45788E+00_JPRB, 0.43918E+00_JPRB, 0.41998E+00_JPRB ,&
&  0.40632E+00_JPRB, 0.38931E+00_JPRB, 0.38204E+00_JPRB, 0.37835E+00_JPRB, 0.38170E+00_JPRB, 0.38852E+00_JPRB, 0.40121E+00_JPRB ,&
&  0.40932E+00_JPRB, 0.41319E+00_JPRB, 0.41762E+00_JPRB, 0.42196E+00_JPRB, 0.42358E+00_JPRB, 0.42688E+00_JPRB, 0.42827E+00_JPRB ,&
&  0.43258E+00_JPRB, 0.43519E+00_JPRB, 0.43751E+00_JPRB, 0.44279E+00_JPRB, 0.45110E+00_JPRB, 0.45977E+00_JPRB, 0.47325E+00_JPRB ,&
&  0.50650E+00_JPRB, 0.51793E+00_JPRB, 0.52517E+00_JPRB, 0.52854E+00_JPRB, 0.53338E+00_JPRB, 0.53500E+00_JPRB, 0.52879E+00_JPRB ,&
&  0.52819E+00_JPRB, 0.53569E+00_JPRB, 0.53662E+00_JPRB, 0.53844E+00_JPRB, 0.54158E+00_JPRB, 0.54462E+00_JPRB, 0.54873E+00_JPRB ,&
&  0.54448E+00_JPRB, 0.55472E+00_JPRB, 0.60628E+00_JPRB, 0.62184E+00_JPRB, 0.61248E+00_JPRB, 0.59668E+00_JPRB, 0.58224E+00_JPRB ,&
&  0.57125E+00_JPRB, 0.56165E+00_JPRB, 0.55363E+00_JPRB, 0.54703E+00_JPRB, 0.54216E+00_JPRB, 0.53945E+00_JPRB, 0.53879E+00_JPRB ,&
&  0.53376E+00_JPRB, 0.52778E+00_JPRB, 0.52114E+00_JPRB, 0.51611E+00_JPRB, 0.51175E+00_JPRB, 0.50675E+00_JPRB, 0.50352E+00_JPRB ,&
&  0.50027E+00_JPRB, 0.49958E+00_JPRB, 0.49924E+00_JPRB, 0.49953E+00_JPRB, 0.49920E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.49951E+00_JPRB, 0.50247E+00_JPRB, 0.50304E+00_JPRB, 0.49610E+00_JPRB, 0.48148E+00_JPRB, 0.46790E+00_JPRB, 0.45813E+00_JPRB ,&
&  0.45446E+00_JPRB, 0.45283E+00_JPRB, 0.45155E+00_JPRB, 0.45095E+00_JPRB, 0.45130E+00_JPRB, 0.45258E+00_JPRB, 0.45471E+00_JPRB ,&
&  0.45772E+00_JPRB, 0.46045E+00_JPRB, 0.46163E+00_JPRB, 0.46180E+00_JPRB, 0.46357E+00_JPRB, 0.46727E+00_JPRB, 0.47138E+00_JPRB ,&
&  0.47323E+00_JPRB, 0.47224E+00_JPRB, 0.47269E+00_JPRB, 0.47070E+00_JPRB, 0.46923E+00_JPRB, 0.46689E+00_JPRB, 0.46711E+00_JPRB ,&
&  0.47332E+00_JPRB, 0.47831E+00_JPRB, 0.47200E+00_JPRB, 0.47294E+00_JPRB, 0.47759E+00_JPRB, 0.46101E+00_JPRB, 0.43644E+00_JPRB ,&
&  0.41000E+00_JPRB, 0.39009E+00_JPRB, 0.38159E+00_JPRB, 0.37964E+00_JPRB, 0.38572E+00_JPRB, 0.39948E+00_JPRB, 0.41628E+00_JPRB ,&
&  0.41926E+00_JPRB, 0.42412E+00_JPRB, 0.43184E+00_JPRB, 0.43693E+00_JPRB, 0.43911E+00_JPRB, 0.43999E+00_JPRB, 0.44129E+00_JPRB ,&
&  0.44484E+00_JPRB, 0.45307E+00_JPRB, 0.46722E+00_JPRB, 0.48721E+00_JPRB, 0.51757E+00_JPRB, 0.53433E+00_JPRB, 0.56602E+00_JPRB ,&
&  0.61604E+00_JPRB, 0.61455E+00_JPRB, 0.61702E+00_JPRB, 0.61315E+00_JPRB, 0.60914E+00_JPRB, 0.60279E+00_JPRB, 0.59917E+00_JPRB ,&
&  0.59647E+00_JPRB, 0.59240E+00_JPRB, 0.58500E+00_JPRB, 0.57781E+00_JPRB, 0.57813E+00_JPRB, 0.58399E+00_JPRB, 0.59858E+00_JPRB ,&
&  0.62279E+00_JPRB, 0.66444E+00_JPRB, 0.76839E+00_JPRB, 0.70125E+00_JPRB, 0.66515E+00_JPRB, 0.64926E+00_JPRB, 0.62865E+00_JPRB ,&
&  0.61195E+00_JPRB, 0.60098E+00_JPRB, 0.59416E+00_JPRB, 0.58559E+00_JPRB, 0.56927E+00_JPRB, 0.55723E+00_JPRB, 0.54831E+00_JPRB ,&
&  0.54083E+00_JPRB, 0.53338E+00_JPRB, 0.52785E+00_JPRB, 0.52389E+00_JPRB, 0.51927E+00_JPRB, 0.51510E+00_JPRB, 0.51240E+00_JPRB ,&
&  0.50838E+00_JPRB, 0.50570E+00_JPRB, 0.50430E+00_JPRB, 0.50179E+00_JPRB, 0.50021E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.50252E+00_JPRB, 0.50585E+00_JPRB, 0.50145E+00_JPRB, 0.49382E+00_JPRB, 0.47754E+00_JPRB, 0.46464E+00_JPRB, 0.45606E+00_JPRB ,&
&  0.45416E+00_JPRB, 0.45111E+00_JPRB, 0.44976E+00_JPRB, 0.44984E+00_JPRB, 0.45122E+00_JPRB, 0.45411E+00_JPRB, 0.45624E+00_JPRB ,&
&  0.45784E+00_JPRB, 0.46066E+00_JPRB, 0.46407E+00_JPRB, 0.46658E+00_JPRB, 0.46878E+00_JPRB, 0.47165E+00_JPRB, 0.47528E+00_JPRB ,&
&  0.47923E+00_JPRB, 0.48096E+00_JPRB, 0.48170E+00_JPRB, 0.48226E+00_JPRB, 0.48539E+00_JPRB, 0.48993E+00_JPRB, 0.49085E+00_JPRB ,&
&  0.48428E+00_JPRB, 0.48133E+00_JPRB, 0.48570E+00_JPRB, 0.48622E+00_JPRB, 0.48959E+00_JPRB, 0.47809E+00_JPRB, 0.44979E+00_JPRB ,&
&  0.41977E+00_JPRB, 0.39958E+00_JPRB, 0.39668E+00_JPRB, 0.40486E+00_JPRB, 0.42006E+00_JPRB, 0.42820E+00_JPRB, 0.43685E+00_JPRB ,&
&  0.44376E+00_JPRB, 0.44637E+00_JPRB, 0.44878E+00_JPRB, 0.45436E+00_JPRB, 0.45805E+00_JPRB, 0.45902E+00_JPRB, 0.46156E+00_JPRB ,&
&  0.46866E+00_JPRB, 0.48045E+00_JPRB, 0.48641E+00_JPRB, 0.48472E+00_JPRB, 0.50841E+00_JPRB, 0.57649E+00_JPRB, 0.63385E+00_JPRB ,&
&  0.64745E+00_JPRB, 0.65249E+00_JPRB, 0.65701E+00_JPRB, 0.67755E+00_JPRB, 0.68391E+00_JPRB, 0.68883E+00_JPRB, 0.68493E+00_JPRB ,&
&  0.67234E+00_JPRB, 0.65521E+00_JPRB, 0.64139E+00_JPRB, 0.64002E+00_JPRB, 0.65722E+00_JPRB, 0.69159E+00_JPRB, 0.72671E+00_JPRB ,&
&  0.74867E+00_JPRB, 0.73350E+00_JPRB, 0.67621E+00_JPRB, 0.62434E+00_JPRB, 0.60888E+00_JPRB, 0.61341E+00_JPRB, 0.62143E+00_JPRB ,&
&  0.61807E+00_JPRB, 0.61253E+00_JPRB, 0.60634E+00_JPRB, 0.59972E+00_JPRB, 0.57277E+00_JPRB, 0.55537E+00_JPRB, 0.54130E+00_JPRB ,&
&  0.53162E+00_JPRB, 0.52491E+00_JPRB, 0.52560E+00_JPRB, 0.52303E+00_JPRB, 0.51960E+00_JPRB, 0.51512E+00_JPRB, 0.50826E+00_JPRB ,&
&  0.50351E+00_JPRB, 0.50031E+00_JPRB, 0.49745E+00_JPRB, 0.49612E+00_JPRB, 0.49713E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.51228E+00_JPRB, 0.51001E+00_JPRB, 0.50708E+00_JPRB, 0.49857E+00_JPRB, 0.49144E+00_JPRB, 0.48799E+00_JPRB, 0.48562E+00_JPRB ,&
&  0.49315E+00_JPRB, 0.47771E+00_JPRB, 0.46334E+00_JPRB, 0.45300E+00_JPRB, 0.44731E+00_JPRB, 0.44581E+00_JPRB, 0.44981E+00_JPRB ,&
&  0.45592E+00_JPRB, 0.46174E+00_JPRB, 0.46664E+00_JPRB, 0.47165E+00_JPRB, 0.47636E+00_JPRB, 0.47938E+00_JPRB, 0.48250E+00_JPRB ,&
&  0.48658E+00_JPRB, 0.49070E+00_JPRB, 0.49458E+00_JPRB, 0.50055E+00_JPRB, 0.50494E+00_JPRB, 0.50789E+00_JPRB, 0.50630E+00_JPRB ,&
&  0.50135E+00_JPRB, 0.49679E+00_JPRB, 0.49705E+00_JPRB, 0.49876E+00_JPRB, 0.49948E+00_JPRB, 0.48943E+00_JPRB, 0.46086E+00_JPRB ,&
&  0.43386E+00_JPRB, 0.42297E+00_JPRB, 0.43402E+00_JPRB, 0.45102E+00_JPRB, 0.45816E+00_JPRB, 0.46409E+00_JPRB, 0.46522E+00_JPRB ,&
&  0.46288E+00_JPRB, 0.46253E+00_JPRB, 0.46915E+00_JPRB, 0.47771E+00_JPRB, 0.48035E+00_JPRB, 0.48259E+00_JPRB, 0.48688E+00_JPRB ,&
&  0.49773E+00_JPRB, 0.50997E+00_JPRB, 0.51667E+00_JPRB, 0.51266E+00_JPRB, 0.57241E+00_JPRB, 0.78079E+00_JPRB, 0.78024E+00_JPRB ,&
&  0.78793E+00_JPRB, 0.79148E+00_JPRB, 0.80620E+00_JPRB, 0.81999E+00_JPRB, 0.80222E+00_JPRB, 0.79185E+00_JPRB, 0.78658E+00_JPRB ,&
&  0.76274E+00_JPRB, 0.75233E+00_JPRB, 0.75388E+00_JPRB, 0.78629E+00_JPRB, 0.81297E+00_JPRB, 0.83062E+00_JPRB, 0.84485E+00_JPRB ,&
&  0.83789E+00_JPRB, 0.79403E+00_JPRB, 0.73982E+00_JPRB, 0.69038E+00_JPRB, 0.66545E+00_JPRB, 0.64961E+00_JPRB, 0.64000E+00_JPRB ,&
&  0.62915E+00_JPRB, 0.62496E+00_JPRB, 0.62418E+00_JPRB, 0.62075E+00_JPRB, 0.59631E+00_JPRB, 0.56630E+00_JPRB, 0.54808E+00_JPRB ,&
&  0.53693E+00_JPRB, 0.52750E+00_JPRB, 0.51931E+00_JPRB, 0.51380E+00_JPRB, 0.51485E+00_JPRB, 0.51557E+00_JPRB, 0.51018E+00_JPRB ,&
&  0.50955E+00_JPRB, 0.50483E+00_JPRB, 0.50410E+00_JPRB, 0.50572E+00_JPRB, 0.51133E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.51660E+00_JPRB, 0.51721E+00_JPRB, 0.51561E+00_JPRB, 0.51175E+00_JPRB, 0.51164E+00_JPRB, 0.51044E+00_JPRB, 0.50234E+00_JPRB ,&
&  0.51538E+00_JPRB, 0.48294E+00_JPRB, 0.46790E+00_JPRB, 0.46095E+00_JPRB, 0.45991E+00_JPRB, 0.45599E+00_JPRB, 0.45062E+00_JPRB ,&
&  0.45262E+00_JPRB, 0.46106E+00_JPRB, 0.47077E+00_JPRB, 0.47777E+00_JPRB, 0.48388E+00_JPRB, 0.48746E+00_JPRB, 0.49058E+00_JPRB ,&
&  0.49484E+00_JPRB, 0.50109E+00_JPRB, 0.50995E+00_JPRB, 0.51630E+00_JPRB, 0.51949E+00_JPRB, 0.52215E+00_JPRB, 0.52080E+00_JPRB ,&
&  0.51797E+00_JPRB, 0.51189E+00_JPRB, 0.50776E+00_JPRB, 0.50590E+00_JPRB, 0.50374E+00_JPRB, 0.50387E+00_JPRB, 0.49213E+00_JPRB ,&
&  0.46481E+00_JPRB, 0.46474E+00_JPRB, 0.47998E+00_JPRB, 0.48803E+00_JPRB, 0.48851E+00_JPRB, 0.48586E+00_JPRB, 0.48613E+00_JPRB ,&
&  0.48817E+00_JPRB, 0.49028E+00_JPRB, 0.49525E+00_JPRB, 0.49746E+00_JPRB, 0.50086E+00_JPRB, 0.50421E+00_JPRB, 0.51554E+00_JPRB ,&
&  0.54054E+00_JPRB, 0.52441E+00_JPRB, 0.53066E+00_JPRB, 0.57057E+00_JPRB, 0.74353E+00_JPRB, 0.82006E+00_JPRB, 0.87525E+00_JPRB ,&
&  0.96379E+00_JPRB, 0.97209E+00_JPRB, 0.95988E+00_JPRB, 0.95325E+00_JPRB, 0.95305E+00_JPRB, 0.94884E+00_JPRB, 0.95256E+00_JPRB ,&
&  0.92795E+00_JPRB, 0.93540E+00_JPRB, 0.95433E+00_JPRB, 0.98524E+00_JPRB, 0.99201E+00_JPRB, 0.98960E+00_JPRB, 0.97538E+00_JPRB ,&
&  0.94600E+00_JPRB, 0.90340E+00_JPRB, 0.84193E+00_JPRB, 0.78229E+00_JPRB, 0.73249E+00_JPRB, 0.70733E+00_JPRB, 0.68987E+00_JPRB ,&
&  0.67477E+00_JPRB, 0.66621E+00_JPRB, 0.65732E+00_JPRB, 0.64594E+00_JPRB, 0.63189E+00_JPRB, 0.61738E+00_JPRB, 0.61262E+00_JPRB ,&
&  0.58449E+00_JPRB, 0.56663E+00_JPRB, 0.54649E+00_JPRB, 0.53481E+00_JPRB, 0.53310E+00_JPRB, 0.53379E+00_JPRB, 0.53326E+00_JPRB ,&
&  0.53636E+00_JPRB, 0.52645E+00_JPRB, 0.52370E+00_JPRB, 0.52221E+00_JPRB, 0.52118E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.52170E+00_JPRB, 0.52652E+00_JPRB, 0.52635E+00_JPRB, 0.52610E+00_JPRB, 0.53106E+00_JPRB, 0.53264E+00_JPRB, 0.53359E+00_JPRB ,&
&  0.57441E+00_JPRB, 0.55058E+00_JPRB, 0.55985E+00_JPRB, 0.50582E+00_JPRB, 0.48320E+00_JPRB, 0.47274E+00_JPRB, 0.45940E+00_JPRB ,&
&  0.45754E+00_JPRB, 0.46213E+00_JPRB, 0.46832E+00_JPRB, 0.47634E+00_JPRB, 0.48509E+00_JPRB, 0.48983E+00_JPRB, 0.49413E+00_JPRB ,&
&  0.49963E+00_JPRB, 0.50710E+00_JPRB, 0.51472E+00_JPRB, 0.51997E+00_JPRB, 0.52351E+00_JPRB, 0.52745E+00_JPRB, 0.52484E+00_JPRB ,&
&  0.51499E+00_JPRB, 0.51082E+00_JPRB, 0.50960E+00_JPRB, 0.50719E+00_JPRB, 0.50919E+00_JPRB, 0.51060E+00_JPRB, 0.51911E+00_JPRB ,&
&  0.52001E+00_JPRB, 0.51237E+00_JPRB, 0.51971E+00_JPRB, 0.51681E+00_JPRB, 0.51446E+00_JPRB, 0.51093E+00_JPRB, 0.50731E+00_JPRB ,&
&  0.50605E+00_JPRB, 0.50829E+00_JPRB, 0.51430E+00_JPRB, 0.51916E+00_JPRB, 0.52352E+00_JPRB, 0.53301E+00_JPRB, 0.55665E+00_JPRB ,&
&  0.59612E+00_JPRB, 0.56360E+00_JPRB, 0.63317E+00_JPRB, 0.67020E+00_JPRB, 0.85815E+00_JPRB, 0.10540E+01_JPRB, 0.10817E+01_JPRB ,&
&  0.12365E+01_JPRB, 0.12264E+01_JPRB, 0.11903E+01_JPRB, 0.11641E+01_JPRB, 0.11624E+01_JPRB, 0.11290E+01_JPRB, 0.10809E+01_JPRB ,&
&  0.10759E+01_JPRB, 0.10682E+01_JPRB, 0.11343E+01_JPRB, 0.11357E+01_JPRB, 0.11019E+01_JPRB, 0.10787E+01_JPRB, 0.10341E+01_JPRB ,&
&  0.99814E+00_JPRB, 0.96023E+00_JPRB, 0.92187E+00_JPRB, 0.84431E+00_JPRB, 0.81415E+00_JPRB, 0.78510E+00_JPRB, 0.73288E+00_JPRB ,&
&  0.70941E+00_JPRB, 0.68081E+00_JPRB, 0.65356E+00_JPRB, 0.65927E+00_JPRB, 0.63925E+00_JPRB, 0.62791E+00_JPRB, 0.62221E+00_JPRB ,&
&  0.60364E+00_JPRB, 0.58365E+00_JPRB, 0.58604E+00_JPRB, 0.58076E+00_JPRB, 0.56783E+00_JPRB, 0.55673E+00_JPRB, 0.55472E+00_JPRB ,&
&  0.54621E+00_JPRB, 0.54218E+00_JPRB, 0.54663E+00_JPRB, 0.53554E+00_JPRB, 0.52683E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.53755E+00_JPRB, 0.53996E+00_JPRB, 0.53747E+00_JPRB, 0.54100E+00_JPRB, 0.55075E+00_JPRB, 0.56007E+00_JPRB, 0.56830E+00_JPRB ,&
&  0.58897E+00_JPRB, 0.59664E+00_JPRB, 0.59584E+00_JPRB, 0.58663E+00_JPRB, 0.57001E+00_JPRB, 0.52589E+00_JPRB, 0.47032E+00_JPRB ,&
&  0.45745E+00_JPRB, 0.46532E+00_JPRB, 0.47527E+00_JPRB, 0.48051E+00_JPRB, 0.48730E+00_JPRB, 0.49430E+00_JPRB, 0.50046E+00_JPRB ,&
&  0.50732E+00_JPRB, 0.51256E+00_JPRB, 0.51857E+00_JPRB, 0.52372E+00_JPRB, 0.52659E+00_JPRB, 0.53111E+00_JPRB, 0.53282E+00_JPRB ,&
&  0.51797E+00_JPRB, 0.51014E+00_JPRB, 0.50596E+00_JPRB, 0.50837E+00_JPRB, 0.52183E+00_JPRB, 0.53028E+00_JPRB, 0.54069E+00_JPRB ,&
&  0.55001E+00_JPRB, 0.55503E+00_JPRB, 0.55383E+00_JPRB, 0.55161E+00_JPRB, 0.54774E+00_JPRB, 0.54553E+00_JPRB, 0.54176E+00_JPRB ,&
&  0.53960E+00_JPRB, 0.54035E+00_JPRB, 0.54631E+00_JPRB, 0.55103E+00_JPRB, 0.55835E+00_JPRB, 0.57601E+00_JPRB, 0.63250E+00_JPRB ,&
&  0.69414E+00_JPRB, 0.76987E+00_JPRB, 0.81149E+00_JPRB, 0.81676E+00_JPRB, 0.98074E+00_JPRB, 0.11196E+01_JPRB, 0.11707E+01_JPRB ,&
&  0.12518E+01_JPRB, 0.12122E+01_JPRB, 0.12791E+01_JPRB, 0.13500E+01_JPRB, 0.13989E+01_JPRB, 0.13629E+01_JPRB, 0.12757E+01_JPRB ,&
&  0.12630E+01_JPRB, 0.13382E+01_JPRB, 0.13394E+01_JPRB, 0.13324E+01_JPRB, 0.12731E+01_JPRB, 0.12289E+01_JPRB, 0.11716E+01_JPRB ,&
&  0.11563E+01_JPRB, 0.11416E+01_JPRB, 0.10890E+01_JPRB, 0.10288E+01_JPRB, 0.96093E+00_JPRB, 0.89116E+00_JPRB, 0.82119E+00_JPRB ,&
&  0.75538E+00_JPRB, 0.67851E+00_JPRB, 0.64921E+00_JPRB, 0.64612E+00_JPRB, 0.65042E+00_JPRB, 0.63032E+00_JPRB, 0.62337E+00_JPRB ,&
&  0.61954E+00_JPRB, 0.61884E+00_JPRB, 0.63224E+00_JPRB, 0.61591E+00_JPRB, 0.59206E+00_JPRB, 0.56593E+00_JPRB, 0.55058E+00_JPRB ,&
&  0.53797E+00_JPRB, 0.54460E+00_JPRB, 0.54887E+00_JPRB, 0.54395E+00_JPRB, 0.53786E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.55114E+00_JPRB, 0.54646E+00_JPRB, 0.54546E+00_JPRB, 0.55423E+00_JPRB, 0.56812E+00_JPRB, 0.58382E+00_JPRB, 0.60400E+00_JPRB ,&
&  0.62298E+00_JPRB, 0.63201E+00_JPRB, 0.62315E+00_JPRB, 0.60736E+00_JPRB, 0.59411E+00_JPRB, 0.58588E+00_JPRB, 0.55262E+00_JPRB ,&
&  0.50319E+00_JPRB, 0.49773E+00_JPRB, 0.48901E+00_JPRB, 0.49132E+00_JPRB, 0.50144E+00_JPRB, 0.50855E+00_JPRB, 0.51562E+00_JPRB ,&
&  0.52557E+00_JPRB, 0.52976E+00_JPRB, 0.53213E+00_JPRB, 0.53388E+00_JPRB, 0.53760E+00_JPRB, 0.54927E+00_JPRB, 0.55708E+00_JPRB ,&
&  0.53656E+00_JPRB, 0.52323E+00_JPRB, 0.52222E+00_JPRB, 0.52956E+00_JPRB, 0.54478E+00_JPRB, 0.56231E+00_JPRB, 0.57502E+00_JPRB ,&
&  0.58583E+00_JPRB, 0.59244E+00_JPRB, 0.59393E+00_JPRB, 0.59249E+00_JPRB, 0.58926E+00_JPRB, 0.58477E+00_JPRB, 0.58264E+00_JPRB ,&
&  0.58143E+00_JPRB, 0.58359E+00_JPRB, 0.59065E+00_JPRB, 0.59779E+00_JPRB, 0.60508E+00_JPRB, 0.64804E+00_JPRB, 0.74969E+00_JPRB ,&
&  0.87216E+00_JPRB, 0.10014E+01_JPRB, 0.10366E+01_JPRB, 0.11314E+01_JPRB, 0.12242E+01_JPRB, 0.13178E+01_JPRB, 0.13700E+01_JPRB ,&
&  0.14111E+01_JPRB, 0.13432E+01_JPRB, 0.15038E+01_JPRB, 0.15802E+01_JPRB, 0.15611E+01_JPRB, 0.15831E+01_JPRB, 0.14999E+01_JPRB ,&
&  0.14467E+01_JPRB, 0.14011E+01_JPRB, 0.14535E+01_JPRB, 0.14566E+01_JPRB, 0.14222E+01_JPRB, 0.14330E+01_JPRB, 0.13788E+01_JPRB ,&
&  0.13463E+01_JPRB, 0.12952E+01_JPRB, 0.11287E+01_JPRB, 0.10000E+01_JPRB, 0.87357E+00_JPRB, 0.90157E+00_JPRB, 0.84838E+00_JPRB ,&
&  0.77650E+00_JPRB, 0.68530E+00_JPRB, 0.68425E+00_JPRB, 0.68012E+00_JPRB, 0.69298E+00_JPRB, 0.69704E+00_JPRB, 0.66946E+00_JPRB ,&
&  0.66878E+00_JPRB, 0.68309E+00_JPRB, 0.66476E+00_JPRB, 0.64546E+00_JPRB, 0.62212E+00_JPRB, 0.59177E+00_JPRB, 0.57409E+00_JPRB ,&
&  0.57352E+00_JPRB, 0.57752E+00_JPRB, 0.57206E+00_JPRB, 0.56159E+00_JPRB, 0.55522E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.58199E+00_JPRB, 0.57757E+00_JPRB, 0.57792E+00_JPRB, 0.58606E+00_JPRB, 0.59900E+00_JPRB, 0.61807E+00_JPRB, 0.63878E+00_JPRB ,&
&  0.65428E+00_JPRB, 0.65621E+00_JPRB, 0.64045E+00_JPRB, 0.62782E+00_JPRB, 0.61973E+00_JPRB, 0.61120E+00_JPRB, 0.61133E+00_JPRB ,&
&  0.55095E+00_JPRB, 0.52464E+00_JPRB, 0.49761E+00_JPRB, 0.49668E+00_JPRB, 0.51642E+00_JPRB, 0.52339E+00_JPRB, 0.53089E+00_JPRB ,&
&  0.53773E+00_JPRB, 0.54705E+00_JPRB, 0.54932E+00_JPRB, 0.55242E+00_JPRB, 0.55872E+00_JPRB, 0.57609E+00_JPRB, 0.59462E+00_JPRB ,&
&  0.57804E+00_JPRB, 0.56327E+00_JPRB, 0.56691E+00_JPRB, 0.57926E+00_JPRB, 0.59106E+00_JPRB, 0.61086E+00_JPRB, 0.63425E+00_JPRB ,&
&  0.64310E+00_JPRB, 0.64300E+00_JPRB, 0.63726E+00_JPRB, 0.63406E+00_JPRB, 0.63282E+00_JPRB, 0.63120E+00_JPRB, 0.63534E+00_JPRB ,&
&  0.64162E+00_JPRB, 0.64719E+00_JPRB, 0.65522E+00_JPRB, 0.65949E+00_JPRB, 0.66014E+00_JPRB, 0.73013E+00_JPRB, 0.86182E+00_JPRB ,&
&  0.10329E+01_JPRB, 0.10683E+01_JPRB, 0.11388E+01_JPRB, 0.14613E+01_JPRB, 0.16374E+01_JPRB, 0.17095E+01_JPRB, 0.16756E+01_JPRB ,&
&  0.16280E+01_JPRB, 0.15336E+01_JPRB, 0.15317E+01_JPRB, 0.15669E+01_JPRB, 0.15553E+01_JPRB, 0.16402E+01_JPRB, 0.16011E+01_JPRB ,&
&  0.15061E+01_JPRB, 0.14788E+01_JPRB, 0.14768E+01_JPRB, 0.14638E+01_JPRB, 0.14212E+01_JPRB, 0.14294E+01_JPRB, 0.13953E+01_JPRB ,&
&  0.13312E+01_JPRB, 0.11630E+01_JPRB, 0.92124E+00_JPRB, 0.87108E+00_JPRB, 0.74213E+00_JPRB, 0.77394E+00_JPRB, 0.89610E+00_JPRB ,&
&  0.90496E+00_JPRB, 0.85190E+00_JPRB, 0.80045E+00_JPRB, 0.77481E+00_JPRB, 0.76900E+00_JPRB, 0.81224E+00_JPRB, 0.77372E+00_JPRB ,&
&  0.72879E+00_JPRB, 0.73899E+00_JPRB, 0.73457E+00_JPRB, 0.71259E+00_JPRB, 0.68170E+00_JPRB, 0.65516E+00_JPRB, 0.63861E+00_JPRB ,&
&  0.63452E+00_JPRB, 0.63157E+00_JPRB, 0.62282E+00_JPRB, 0.60066E+00_JPRB, 0.58642E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.63376E+00_JPRB, 0.63097E+00_JPRB, 0.62749E+00_JPRB, 0.63380E+00_JPRB, 0.64525E+00_JPRB, 0.66133E+00_JPRB, 0.67752E+00_JPRB ,&
&  0.68421E+00_JPRB, 0.67542E+00_JPRB, 0.65680E+00_JPRB, 0.65044E+00_JPRB, 0.64339E+00_JPRB, 0.63173E+00_JPRB, 0.62854E+00_JPRB ,&
&  0.63824E+00_JPRB, 0.57919E+00_JPRB, 0.53812E+00_JPRB, 0.50829E+00_JPRB, 0.51871E+00_JPRB, 0.52475E+00_JPRB, 0.53712E+00_JPRB ,&
&  0.54597E+00_JPRB, 0.56425E+00_JPRB, 0.57161E+00_JPRB, 0.57675E+00_JPRB, 0.58867E+00_JPRB, 0.61577E+00_JPRB, 0.64180E+00_JPRB ,&
&  0.64402E+00_JPRB, 0.64597E+00_JPRB, 0.65766E+00_JPRB, 0.68048E+00_JPRB, 0.70113E+00_JPRB, 0.68035E+00_JPRB, 0.70334E+00_JPRB ,&
&  0.70534E+00_JPRB, 0.70011E+00_JPRB, 0.68844E+00_JPRB, 0.69086E+00_JPRB, 0.69810E+00_JPRB, 0.71022E+00_JPRB, 0.72497E+00_JPRB ,&
&  0.74238E+00_JPRB, 0.74495E+00_JPRB, 0.72417E+00_JPRB, 0.71956E+00_JPRB, 0.73496E+00_JPRB, 0.85304E+00_JPRB, 0.92759E+00_JPRB ,&
&  0.10536E+01_JPRB, 0.10511E+01_JPRB, 0.10590E+01_JPRB, 0.13615E+01_JPRB, 0.16626E+01_JPRB, 0.17902E+01_JPRB, 0.18231E+01_JPRB ,&
&  0.17904E+01_JPRB, 0.17311E+01_JPRB, 0.17009E+01_JPRB, 0.16325E+01_JPRB, 0.16395E+01_JPRB, 0.16846E+01_JPRB, 0.16461E+01_JPRB ,&
&  0.15446E+01_JPRB, 0.14615E+01_JPRB, 0.14355E+01_JPRB, 0.13826E+01_JPRB, 0.13381E+01_JPRB, 0.12539E+01_JPRB, 0.12825E+01_JPRB ,&
&  0.12986E+01_JPRB, 0.84824E+00_JPRB, 0.73093E+00_JPRB, 0.81634E+00_JPRB, 0.73043E+00_JPRB, 0.75459E+00_JPRB, 0.84438E+00_JPRB ,&
&  0.85166E+00_JPRB, 0.84998E+00_JPRB, 0.87858E+00_JPRB, 0.85524E+00_JPRB, 0.87126E+00_JPRB, 0.90846E+00_JPRB, 0.88702E+00_JPRB ,&
&  0.85864E+00_JPRB, 0.82067E+00_JPRB, 0.82249E+00_JPRB, 0.79625E+00_JPRB, 0.76017E+00_JPRB, 0.73461E+00_JPRB, 0.71459E+00_JPRB ,&
&  0.69572E+00_JPRB, 0.68355E+00_JPRB, 0.67665E+00_JPRB, 0.65519E+00_JPRB, 0.63784E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.70902E+00_JPRB, 0.70511E+00_JPRB, 0.70231E+00_JPRB, 0.70468E+00_JPRB, 0.70860E+00_JPRB, 0.71826E+00_JPRB, 0.72297E+00_JPRB ,&
&  0.71672E+00_JPRB, 0.69397E+00_JPRB, 0.67493E+00_JPRB, 0.67505E+00_JPRB, 0.66324E+00_JPRB, 0.64911E+00_JPRB, 0.64923E+00_JPRB ,&
&  0.66371E+00_JPRB, 0.64878E+00_JPRB, 0.57217E+00_JPRB, 0.53484E+00_JPRB, 0.50717E+00_JPRB, 0.51732E+00_JPRB, 0.52922E+00_JPRB ,&
&  0.55159E+00_JPRB, 0.56731E+00_JPRB, 0.57749E+00_JPRB, 0.59254E+00_JPRB, 0.62275E+00_JPRB, 0.66214E+00_JPRB, 0.70712E+00_JPRB ,&
&  0.72793E+00_JPRB, 0.74992E+00_JPRB, 0.74138E+00_JPRB, 0.78328E+00_JPRB, 0.79721E+00_JPRB, 0.76613E+00_JPRB, 0.79132E+00_JPRB ,&
&  0.78896E+00_JPRB, 0.78573E+00_JPRB, 0.77588E+00_JPRB, 0.78224E+00_JPRB, 0.78831E+00_JPRB, 0.77851E+00_JPRB, 0.76289E+00_JPRB ,&
&  0.75171E+00_JPRB, 0.74770E+00_JPRB, 0.74149E+00_JPRB, 0.73336E+00_JPRB, 0.76455E+00_JPRB, 0.84465E+00_JPRB, 0.87831E+00_JPRB ,&
&  0.95570E+00_JPRB, 0.84555E+00_JPRB, 0.77208E+00_JPRB, 0.10550E+01_JPRB, 0.14221E+01_JPRB, 0.15632E+01_JPRB, 0.15431E+01_JPRB ,&
&  0.16737E+01_JPRB, 0.17118E+01_JPRB, 0.17551E+01_JPRB, 0.18066E+01_JPRB, 0.18105E+01_JPRB, 0.17959E+01_JPRB, 0.17281E+01_JPRB ,&
&  0.15850E+01_JPRB, 0.14850E+01_JPRB, 0.14367E+01_JPRB, 0.14654E+01_JPRB, 0.14102E+01_JPRB, 0.13701E+01_JPRB, 0.13411E+01_JPRB ,&
&  0.11919E+01_JPRB, 0.10617E+01_JPRB, 0.82619E+00_JPRB, 0.83714E+00_JPRB, 0.70877E+00_JPRB, 0.71594E+00_JPRB, 0.79863E+00_JPRB ,&
&  0.80515E+00_JPRB, 0.85725E+00_JPRB, 0.88012E+00_JPRB, 0.86602E+00_JPRB, 0.97925E+00_JPRB, 0.10195E+01_JPRB, 0.10059E+01_JPRB ,&
&  0.98419E+00_JPRB, 0.89711E+00_JPRB, 0.91539E+00_JPRB, 0.91862E+00_JPRB, 0.89593E+00_JPRB, 0.87276E+00_JPRB, 0.84659E+00_JPRB ,&
&  0.81406E+00_JPRB, 0.78381E+00_JPRB, 0.76454E+00_JPRB, 0.73723E+00_JPRB, 0.71738E+00_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.78610E+00_JPRB, 0.76708E+00_JPRB, 0.75563E+00_JPRB, 0.75275E+00_JPRB, 0.75383E+00_JPRB, 0.75512E+00_JPRB, 0.75242E+00_JPRB ,&
&  0.74093E+00_JPRB, 0.71302E+00_JPRB, 0.70274E+00_JPRB, 0.69830E+00_JPRB, 0.67800E+00_JPRB, 0.66230E+00_JPRB, 0.66703E+00_JPRB ,&
&  0.68577E+00_JPRB, 0.64708E+00_JPRB, 0.55800E+00_JPRB, 0.51204E+00_JPRB, 0.48798E+00_JPRB, 0.49274E+00_JPRB, 0.51122E+00_JPRB ,&
&  0.53542E+00_JPRB, 0.55438E+00_JPRB, 0.57627E+00_JPRB, 0.60595E+00_JPRB, 0.65206E+00_JPRB, 0.70244E+00_JPRB, 0.78973E+00_JPRB ,&
&  0.89738E+00_JPRB, 0.82681E+00_JPRB, 0.85012E+00_JPRB, 0.84912E+00_JPRB, 0.88270E+00_JPRB, 0.87402E+00_JPRB, 0.85558E+00_JPRB ,&
&  0.82509E+00_JPRB, 0.80636E+00_JPRB, 0.79073E+00_JPRB, 0.77099E+00_JPRB, 0.75253E+00_JPRB, 0.74320E+00_JPRB, 0.74946E+00_JPRB ,&
&  0.77609E+00_JPRB, 0.78458E+00_JPRB, 0.77039E+00_JPRB, 0.77999E+00_JPRB, 0.85884E+00_JPRB, 0.87720E+00_JPRB, 0.87082E+00_JPRB ,&
&  0.82845E+00_JPRB, 0.87247E+00_JPRB, 0.11253E+01_JPRB, 0.13199E+01_JPRB, 0.14582E+01_JPRB, 0.15787E+01_JPRB, 0.16186E+01_JPRB ,&
&  0.17126E+01_JPRB, 0.16748E+01_JPRB, 0.17127E+01_JPRB, 0.15545E+01_JPRB, 0.17420E+01_JPRB, 0.17448E+01_JPRB, 0.17060E+01_JPRB ,&
&  0.16298E+01_JPRB, 0.16046E+01_JPRB, 0.15505E+01_JPRB, 0.15954E+01_JPRB, 0.15870E+01_JPRB, 0.15845E+01_JPRB, 0.12699E+01_JPRB ,&
&  0.10531E+01_JPRB, 0.11057E+01_JPRB, 0.10048E+01_JPRB, 0.89489E+00_JPRB, 0.79483E+00_JPRB, 0.80426E+00_JPRB, 0.81976E+00_JPRB ,&
&  0.83774E+00_JPRB, 0.85230E+00_JPRB, 0.84760E+00_JPRB, 0.89156E+00_JPRB, 0.10246E+01_JPRB, 0.11567E+01_JPRB, 0.11649E+01_JPRB ,&
&  0.10808E+01_JPRB, 0.11042E+01_JPRB, 0.11123E+01_JPRB, 0.10893E+01_JPRB, 0.11102E+01_JPRB, 0.10832E+01_JPRB, 0.10333E+01_JPRB ,&
&  0.97835E+00_JPRB, 0.93370E+00_JPRB, 0.89643E+00_JPRB, 0.85667E+00_JPRB, 0.81494E+00_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.86660E+00_JPRB, 0.84342E+00_JPRB, 0.82329E+00_JPRB, 0.80980E+00_JPRB, 0.79784E+00_JPRB, 0.79003E+00_JPRB, 0.78016E+00_JPRB ,&
&  0.75902E+00_JPRB, 0.72379E+00_JPRB, 0.72335E+00_JPRB, 0.71369E+00_JPRB, 0.69643E+00_JPRB, 0.68240E+00_JPRB, 0.68959E+00_JPRB ,&
&  0.70755E+00_JPRB, 0.64029E+00_JPRB, 0.55222E+00_JPRB, 0.54019E+00_JPRB, 0.52079E+00_JPRB, 0.49553E+00_JPRB, 0.50194E+00_JPRB ,&
&  0.52262E+00_JPRB, 0.54634E+00_JPRB, 0.56978E+00_JPRB, 0.62307E+00_JPRB, 0.69517E+00_JPRB, 0.78185E+00_JPRB, 0.89913E+00_JPRB ,&
&  0.91739E+00_JPRB, 0.94477E+00_JPRB, 0.10025E+01_JPRB, 0.97042E+00_JPRB, 0.93255E+00_JPRB, 0.89170E+00_JPRB, 0.85326E+00_JPRB ,&
&  0.82555E+00_JPRB, 0.80462E+00_JPRB, 0.78769E+00_JPRB, 0.77458E+00_JPRB, 0.77622E+00_JPRB, 0.77892E+00_JPRB, 0.78514E+00_JPRB ,&
&  0.79492E+00_JPRB, 0.78569E+00_JPRB, 0.76264E+00_JPRB, 0.83695E+00_JPRB, 0.76671E+00_JPRB, 0.74299E+00_JPRB, 0.75213E+00_JPRB ,&
&  0.95044E+00_JPRB, 0.11214E+01_JPRB, 0.12383E+01_JPRB, 0.15084E+01_JPRB, 0.13370E+01_JPRB, 0.14532E+01_JPRB, 0.17502E+01_JPRB ,&
&  0.18164E+01_JPRB, 0.18149E+01_JPRB, 0.19302E+01_JPRB, 0.15930E+01_JPRB, 0.14409E+01_JPRB, 0.19069E+01_JPRB, 0.19248E+01_JPRB ,&
&  0.18545E+01_JPRB, 0.18231E+01_JPRB, 0.18032E+01_JPRB, 0.19879E+01_JPRB, 0.14018E+01_JPRB, 0.11759E+01_JPRB, 0.94658E+00_JPRB ,&
&  0.93815E+00_JPRB, 0.89858E+00_JPRB, 0.10003E+01_JPRB, 0.95178E+00_JPRB, 0.94093E+00_JPRB, 0.90859E+00_JPRB, 0.85429E+00_JPRB ,&
&  0.84418E+00_JPRB, 0.84104E+00_JPRB, 0.85134E+00_JPRB, 0.97335E+00_JPRB, 0.11893E+01_JPRB, 0.12197E+01_JPRB, 0.13293E+01_JPRB ,&
&  0.13577E+01_JPRB, 0.14301E+01_JPRB, 0.13617E+01_JPRB, 0.13603E+01_JPRB, 0.13306E+01_JPRB, 0.12729E+01_JPRB, 0.11890E+01_JPRB ,&
&  0.11230E+01_JPRB, 0.10629E+01_JPRB, 0.10019E+01_JPRB, 0.94777E+00_JPRB, 0.90210E+00_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.92895E+00_JPRB, 0.89478E+00_JPRB, 0.86422E+00_JPRB, 0.83863E+00_JPRB, 0.82471E+00_JPRB, 0.81243E+00_JPRB, 0.79468E+00_JPRB ,&
&  0.76007E+00_JPRB, 0.72863E+00_JPRB, 0.73053E+00_JPRB, 0.70425E+00_JPRB, 0.68631E+00_JPRB, 0.68695E+00_JPRB, 0.70974E+00_JPRB ,&
&  0.72356E+00_JPRB, 0.69756E+00_JPRB, 0.60020E+00_JPRB, 0.55577E+00_JPRB, 0.55732E+00_JPRB, 0.52364E+00_JPRB, 0.51003E+00_JPRB ,&
&  0.53434E+00_JPRB, 0.56047E+00_JPRB, 0.60698E+00_JPRB, 0.70274E+00_JPRB, 0.80974E+00_JPRB, 0.92469E+00_JPRB, 0.10521E+01_JPRB ,&
&  0.10715E+01_JPRB, 0.10585E+01_JPRB, 0.10135E+01_JPRB, 0.95432E+00_JPRB, 0.89057E+00_JPRB, 0.84718E+00_JPRB, 0.81504E+00_JPRB ,&
&  0.79248E+00_JPRB, 0.78230E+00_JPRB, 0.78442E+00_JPRB, 0.77747E+00_JPRB, 0.78525E+00_JPRB, 0.75644E+00_JPRB, 0.74839E+00_JPRB ,&
&  0.75794E+00_JPRB, 0.76766E+00_JPRB, 0.78214E+00_JPRB, 0.90014E+00_JPRB, 0.85525E+00_JPRB, 0.79064E+00_JPRB, 0.82201E+00_JPRB ,&
&  0.94582E+00_JPRB, 0.10554E+01_JPRB, 0.12269E+01_JPRB, 0.14621E+01_JPRB, 0.17926E+01_JPRB, 0.19224E+01_JPRB, 0.20784E+01_JPRB ,&
&  0.16991E+01_JPRB, 0.15157E+01_JPRB, 0.15589E+01_JPRB, 0.15030E+01_JPRB, 0.19277E+01_JPRB, 0.26440E+01_JPRB, 0.26828E+01_JPRB ,&
&  0.25578E+01_JPRB, 0.24176E+01_JPRB, 0.23999E+01_JPRB, 0.19473E+01_JPRB, 0.13870E+01_JPRB, 0.10608E+01_JPRB, 0.12128E+01_JPRB ,&
&  0.12611E+01_JPRB, 0.13017E+01_JPRB, 0.11856E+01_JPRB, 0.10221E+01_JPRB, 0.89992E+00_JPRB, 0.10235E+01_JPRB, 0.99712E+00_JPRB ,&
&  0.93257E+00_JPRB, 0.90268E+00_JPRB, 0.10781E+01_JPRB, 0.13285E+01_JPRB, 0.14663E+01_JPRB, 0.15338E+01_JPRB, 0.17703E+01_JPRB ,&
&  0.18628E+01_JPRB, 0.19418E+01_JPRB, 0.18595E+01_JPRB, 0.17351E+01_JPRB, 0.16000E+01_JPRB, 0.14494E+01_JPRB, 0.13264E+01_JPRB ,&
&  0.12285E+01_JPRB, 0.11436E+01_JPRB, 0.10707E+01_JPRB, 0.10102E+01_JPRB, 0.96545E+00_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.97801E+00_JPRB, 0.93189E+00_JPRB, 0.89150E+00_JPRB, 0.86680E+00_JPRB, 0.84662E+00_JPRB, 0.82556E+00_JPRB, 0.79466E+00_JPRB ,&
&  0.74851E+00_JPRB, 0.74067E+00_JPRB, 0.73346E+00_JPRB, 0.70292E+00_JPRB, 0.69172E+00_JPRB, 0.69676E+00_JPRB, 0.71406E+00_JPRB ,&
&  0.73127E+00_JPRB, 0.76531E+00_JPRB, 0.66060E+00_JPRB, 0.61473E+00_JPRB, 0.60405E+00_JPRB, 0.58504E+00_JPRB, 0.54130E+00_JPRB ,&
&  0.56627E+00_JPRB, 0.60192E+00_JPRB, 0.66851E+00_JPRB, 0.79336E+00_JPRB, 0.96688E+00_JPRB, 0.10330E+01_JPRB, 0.10539E+01_JPRB ,&
&  0.10884E+01_JPRB, 0.10513E+01_JPRB, 0.99319E+00_JPRB, 0.93603E+00_JPRB, 0.87312E+00_JPRB, 0.84216E+00_JPRB, 0.82560E+00_JPRB ,&
&  0.81864E+00_JPRB, 0.83207E+00_JPRB, 0.83590E+00_JPRB, 0.85648E+00_JPRB, 0.82778E+00_JPRB, 0.78706E+00_JPRB, 0.77149E+00_JPRB ,&
&  0.76260E+00_JPRB, 0.77428E+00_JPRB, 0.80633E+00_JPRB, 0.93697E+00_JPRB, 0.99457E+00_JPRB, 0.94327E+00_JPRB, 0.10500E+01_JPRB ,&
&  0.11337E+01_JPRB, 0.12924E+01_JPRB, 0.13761E+01_JPRB, 0.16119E+01_JPRB, 0.18309E+01_JPRB, 0.19509E+01_JPRB, 0.21130E+01_JPRB ,&
&  0.19975E+01_JPRB, 0.21313E+01_JPRB, 0.26368E+01_JPRB, 0.32457E+01_JPRB, 0.28597E+01_JPRB, 0.28669E+01_JPRB, 0.39058E+01_JPRB ,&
&  0.28509E+01_JPRB, 0.27903E+01_JPRB, 0.28885E+01_JPRB, 0.27885E+01_JPRB, 0.11740E+01_JPRB, 0.92478E+00_JPRB, 0.12724E+01_JPRB ,&
&  0.13134E+01_JPRB, 0.90515E+00_JPRB, 0.72912E+00_JPRB, 0.83217E+00_JPRB, 0.70540E+00_JPRB, 0.81286E+00_JPRB, 0.10881E+01_JPRB ,&
&  0.11303E+01_JPRB, 0.11520E+01_JPRB, 0.15023E+01_JPRB, 0.16877E+01_JPRB, 0.16834E+01_JPRB, 0.21501E+01_JPRB, 0.23719E+01_JPRB ,&
&  0.24541E+01_JPRB, 0.22930E+01_JPRB, 0.22903E+01_JPRB, 0.20779E+01_JPRB, 0.18289E+01_JPRB, 0.16095E+01_JPRB, 0.14368E+01_JPRB ,&
&  0.12899E+01_JPRB, 0.11871E+01_JPRB, 0.11192E+01_JPRB, 0.10591E+01_JPRB, 0.10137E+01_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.96638E+00_JPRB, 0.92309E+00_JPRB, 0.88143E+00_JPRB, 0.85482E+00_JPRB, 0.83859E+00_JPRB, 0.81768E+00_JPRB, 0.78479E+00_JPRB ,&
&  0.75093E+00_JPRB, 0.76196E+00_JPRB, 0.73400E+00_JPRB, 0.70986E+00_JPRB, 0.69320E+00_JPRB, 0.69346E+00_JPRB, 0.71193E+00_JPRB ,&
&  0.72738E+00_JPRB, 0.74968E+00_JPRB, 0.73234E+00_JPRB, 0.68747E+00_JPRB, 0.67042E+00_JPRB, 0.64191E+00_JPRB, 0.61341E+00_JPRB ,&
&  0.63838E+00_JPRB, 0.69209E+00_JPRB, 0.77335E+00_JPRB, 0.89745E+00_JPRB, 0.98280E+00_JPRB, 0.10174E+01_JPRB, 0.10841E+01_JPRB ,&
&  0.10734E+01_JPRB, 0.10242E+01_JPRB, 0.97287E+00_JPRB, 0.92342E+00_JPRB, 0.88375E+00_JPRB, 0.86415E+00_JPRB, 0.84707E+00_JPRB ,&
&  0.84284E+00_JPRB, 0.84868E+00_JPRB, 0.86197E+00_JPRB, 0.83087E+00_JPRB, 0.79872E+00_JPRB, 0.77532E+00_JPRB, 0.75713E+00_JPRB ,&
&  0.75566E+00_JPRB, 0.80074E+00_JPRB, 0.86924E+00_JPRB, 0.97335E+00_JPRB, 0.11624E+01_JPRB, 0.12343E+01_JPRB, 0.11404E+01_JPRB ,&
&  0.10603E+01_JPRB, 0.11704E+01_JPRB, 0.15270E+01_JPRB, 0.15498E+01_JPRB, 0.15252E+01_JPRB, 0.16354E+01_JPRB, 0.16897E+01_JPRB ,&
&  0.19483E+01_JPRB, 0.25017E+01_JPRB, 0.26660E+01_JPRB, 0.33379E+01_JPRB, 0.35957E+01_JPRB, 0.27001E+01_JPRB, 0.30807E+01_JPRB ,&
&  0.30391E+01_JPRB, 0.26404E+01_JPRB, 0.20166E+01_JPRB, 0.14827E+01_JPRB, 0.15580E+01_JPRB, 0.13331E+01_JPRB, 0.85589E+00_JPRB ,&
&  0.76325E+00_JPRB, 0.72246E+00_JPRB, 0.68857E+00_JPRB, 0.69026E+00_JPRB, 0.68242E+00_JPRB, 0.82556E+00_JPRB, 0.10764E+01_JPRB ,&
&  0.14117E+01_JPRB, 0.16304E+01_JPRB, 0.18472E+01_JPRB, 0.19070E+01_JPRB, 0.17859E+01_JPRB, 0.22492E+01_JPRB, 0.27275E+01_JPRB ,&
&  0.23140E+01_JPRB, 0.21248E+01_JPRB, 0.20745E+01_JPRB, 0.18858E+01_JPRB, 0.16903E+01_JPRB, 0.15193E+01_JPRB, 0.13705E+01_JPRB ,&
&  0.12534E+01_JPRB, 0.11740E+01_JPRB, 0.11161E+01_JPRB, 0.10625E+01_JPRB, 0.10134E+01_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.93515E+00_JPRB, 0.90391E+00_JPRB, 0.88186E+00_JPRB, 0.86208E+00_JPRB, 0.84004E+00_JPRB, 0.82366E+00_JPRB, 0.78283E+00_JPRB ,&
&  0.77433E+00_JPRB, 0.76611E+00_JPRB, 0.72246E+00_JPRB, 0.69080E+00_JPRB, 0.67424E+00_JPRB, 0.67824E+00_JPRB, 0.69805E+00_JPRB ,&
&  0.71140E+00_JPRB, 0.73728E+00_JPRB, 0.79276E+00_JPRB, 0.78501E+00_JPRB, 0.82879E+00_JPRB, 0.76732E+00_JPRB, 0.76896E+00_JPRB ,&
&  0.77836E+00_JPRB, 0.84209E+00_JPRB, 0.93473E+00_JPRB, 0.99958E+00_JPRB, 0.10303E+01_JPRB, 0.10867E+01_JPRB, 0.10793E+01_JPRB ,&
&  0.10485E+01_JPRB, 0.10334E+01_JPRB, 0.96519E+00_JPRB, 0.91842E+00_JPRB, 0.89787E+00_JPRB, 0.87006E+00_JPRB, 0.85248E+00_JPRB ,&
&  0.85373E+00_JPRB, 0.86790E+00_JPRB, 0.83741E+00_JPRB, 0.82456E+00_JPRB, 0.81938E+00_JPRB, 0.80471E+00_JPRB, 0.79228E+00_JPRB ,&
&  0.81675E+00_JPRB, 0.88029E+00_JPRB, 0.96400E+00_JPRB, 0.10562E+01_JPRB, 0.10641E+01_JPRB, 0.92455E+00_JPRB, 0.10506E+01_JPRB ,&
&  0.11635E+01_JPRB, 0.13882E+01_JPRB, 0.14495E+01_JPRB, 0.16573E+01_JPRB, 0.15491E+01_JPRB, 0.15017E+01_JPRB, 0.16826E+01_JPRB ,&
&  0.21001E+01_JPRB, 0.29829E+01_JPRB, 0.26629E+01_JPRB, 0.30955E+01_JPRB, 0.41676E+01_JPRB, 0.31472E+01_JPRB, 0.26570E+01_JPRB ,&
&  0.28668E+01_JPRB, 0.23960E+01_JPRB, 0.19753E+01_JPRB, 0.17181E+01_JPRB, 0.23677E+01_JPRB, 0.26112E+01_JPRB, 0.11810E+01_JPRB ,&
&  0.90118E+00_JPRB, 0.84936E+00_JPRB, 0.80701E+00_JPRB, 0.75230E+00_JPRB, 0.74979E+00_JPRB, 0.94554E+00_JPRB, 0.13805E+01_JPRB ,&
&  0.15836E+01_JPRB, 0.20022E+01_JPRB, 0.21566E+01_JPRB, 0.21022E+01_JPRB, 0.18080E+01_JPRB, 0.21174E+01_JPRB, 0.21073E+01_JPRB ,&
&  0.22926E+01_JPRB, 0.21894E+01_JPRB, 0.19891E+01_JPRB, 0.17933E+01_JPRB, 0.16010E+01_JPRB, 0.14340E+01_JPRB, 0.13002E+01_JPRB ,&
&  0.11897E+01_JPRB, 0.11143E+01_JPRB, 0.10598E+01_JPRB, 0.10172E+01_JPRB, 0.97246E+00_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.93682E+00_JPRB, 0.90875E+00_JPRB, 0.88269E+00_JPRB, 0.85529E+00_JPRB, 0.84198E+00_JPRB, 0.82133E+00_JPRB, 0.78492E+00_JPRB ,&
&  0.80672E+00_JPRB, 0.75828E+00_JPRB, 0.70876E+00_JPRB, 0.67704E+00_JPRB, 0.66788E+00_JPRB, 0.67177E+00_JPRB, 0.68567E+00_JPRB ,&
&  0.70961E+00_JPRB, 0.73152E+00_JPRB, 0.79724E+00_JPRB, 0.95382E+00_JPRB, 0.10284E+01_JPRB, 0.96510E+00_JPRB, 0.10214E+01_JPRB ,&
&  0.99474E+00_JPRB, 0.10349E+01_JPRB, 0.11162E+01_JPRB, 0.11134E+01_JPRB, 0.11289E+01_JPRB, 0.11593E+01_JPRB, 0.10952E+01_JPRB ,&
&  0.10411E+01_JPRB, 0.10391E+01_JPRB, 0.97664E+00_JPRB, 0.94300E+00_JPRB, 0.91516E+00_JPRB, 0.90631E+00_JPRB, 0.89272E+00_JPRB ,&
&  0.89871E+00_JPRB, 0.87482E+00_JPRB, 0.86175E+00_JPRB, 0.86284E+00_JPRB, 0.86791E+00_JPRB, 0.86593E+00_JPRB, 0.88214E+00_JPRB ,&
&  0.94161E+00_JPRB, 0.10028E+01_JPRB, 0.10871E+01_JPRB, 0.11504E+01_JPRB, 0.95019E+00_JPRB, 0.10935E+01_JPRB, 0.11341E+01_JPRB ,&
&  0.11822E+01_JPRB, 0.12547E+01_JPRB, 0.12439E+01_JPRB, 0.13918E+01_JPRB, 0.15267E+01_JPRB, 0.15158E+01_JPRB, 0.16795E+01_JPRB ,&
&  0.23929E+01_JPRB, 0.24560E+01_JPRB, 0.25525E+01_JPRB, 0.30886E+01_JPRB, 0.35047E+01_JPRB, 0.47132E+01_JPRB, 0.24114E+01_JPRB ,&
&  0.18358E+01_JPRB, 0.21911E+01_JPRB, 0.20787E+01_JPRB, 0.20234E+01_JPRB, 0.25336E+01_JPRB, 0.34100E+01_JPRB, 0.28838E+01_JPRB ,&
&  0.11301E+01_JPRB, 0.88015E+00_JPRB, 0.99769E+00_JPRB, 0.10028E+01_JPRB, 0.91061E+00_JPRB, 0.11576E+01_JPRB, 0.22242E+01_JPRB ,&
&  0.21078E+01_JPRB, 0.23613E+01_JPRB, 0.23099E+01_JPRB, 0.20027E+01_JPRB, 0.18034E+01_JPRB, 0.20101E+01_JPRB, 0.23072E+01_JPRB ,&
&  0.21806E+01_JPRB, 0.19484E+01_JPRB, 0.17720E+01_JPRB, 0.16262E+01_JPRB, 0.14544E+01_JPRB, 0.12957E+01_JPRB, 0.11688E+01_JPRB ,&
&  0.10822E+01_JPRB, 0.10347E+01_JPRB, 0.10094E+01_JPRB, 0.98793E+00_JPRB, 0.96334E+00_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.89993E+00_JPRB, 0.87914E+00_JPRB, 0.86337E+00_JPRB, 0.85320E+00_JPRB, 0.85585E+00_JPRB, 0.83523E+00_JPRB, 0.81070E+00_JPRB ,&
&  0.84076E+00_JPRB, 0.75672E+00_JPRB, 0.70881E+00_JPRB, 0.67355E+00_JPRB, 0.67038E+00_JPRB, 0.68221E+00_JPRB, 0.69235E+00_JPRB ,&
&  0.70336E+00_JPRB, 0.73630E+00_JPRB, 0.83000E+00_JPRB, 0.10407E+01_JPRB, 0.11792E+01_JPRB, 0.11301E+01_JPRB, 0.11758E+01_JPRB ,&
&  0.12649E+01_JPRB, 0.12432E+01_JPRB, 0.11808E+01_JPRB, 0.11549E+01_JPRB, 0.12241E+01_JPRB, 0.12146E+01_JPRB, 0.11361E+01_JPRB ,&
&  0.10758E+01_JPRB, 0.10726E+01_JPRB, 0.10265E+01_JPRB, 0.99546E+00_JPRB, 0.98359E+00_JPRB, 0.97638E+00_JPRB, 0.97710E+00_JPRB ,&
&  0.94637E+00_JPRB, 0.93412E+00_JPRB, 0.93211E+00_JPRB, 0.93361E+00_JPRB, 0.95884E+00_JPRB, 0.96026E+00_JPRB, 0.10115E+01_JPRB ,&
&  0.10893E+01_JPRB, 0.11438E+01_JPRB, 0.11979E+01_JPRB, 0.11374E+01_JPRB, 0.11454E+01_JPRB, 0.11521E+01_JPRB, 0.12030E+01_JPRB ,&
&  0.12085E+01_JPRB, 0.12534E+01_JPRB, 0.13288E+01_JPRB, 0.14078E+01_JPRB, 0.15676E+01_JPRB, 0.17047E+01_JPRB, 0.19520E+01_JPRB ,&
&  0.24330E+01_JPRB, 0.26553E+01_JPRB, 0.26194E+01_JPRB, 0.28329E+01_JPRB, 0.32331E+01_JPRB, 0.39933E+01_JPRB, 0.30786E+01_JPRB ,&
&  0.22775E+01_JPRB, 0.18825E+01_JPRB, 0.20351E+01_JPRB, 0.25757E+01_JPRB, 0.29846E+01_JPRB, 0.35968E+01_JPRB, 0.46308E+01_JPRB ,&
&  0.50671E+01_JPRB, 0.28166E+01_JPRB, 0.22458E+01_JPRB, 0.26097E+01_JPRB, 0.19880E+01_JPRB, 0.16857E+01_JPRB, 0.20808E+01_JPRB ,&
&  0.23208E+01_JPRB, 0.24107E+01_JPRB, 0.22885E+01_JPRB, 0.18857E+01_JPRB, 0.18534E+01_JPRB, 0.21100E+01_JPRB, 0.21646E+01_JPRB ,&
&  0.18701E+01_JPRB, 0.16934E+01_JPRB, 0.15155E+01_JPRB, 0.13341E+01_JPRB, 0.11898E+01_JPRB, 0.10897E+01_JPRB, 0.10304E+01_JPRB ,&
&  0.10041E+01_JPRB, 0.98921E+00_JPRB, 0.97463E+00_JPRB, 0.95099E+00_JPRB, 0.92482E+00_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.87170E+00_JPRB, 0.87141E+00_JPRB, 0.88181E+00_JPRB, 0.88624E+00_JPRB, 0.87753E+00_JPRB, 0.84789E+00_JPRB, 0.85745E+00_JPRB ,&
&  0.88173E+00_JPRB, 0.77641E+00_JPRB, 0.71475E+00_JPRB, 0.68810E+00_JPRB, 0.69303E+00_JPRB, 0.71169E+00_JPRB, 0.72113E+00_JPRB ,&
&  0.73916E+00_JPRB, 0.79893E+00_JPRB, 0.97704E+00_JPRB, 0.11677E+01_JPRB, 0.13680E+01_JPRB, 0.16857E+01_JPRB, 0.12426E+01_JPRB ,&
&  0.13108E+01_JPRB, 0.12917E+01_JPRB, 0.11928E+01_JPRB, 0.12240E+01_JPRB, 0.12766E+01_JPRB, 0.12733E+01_JPRB, 0.11979E+01_JPRB ,&
&  0.11401E+01_JPRB, 0.11433E+01_JPRB, 0.11178E+01_JPRB, 0.10922E+01_JPRB, 0.10867E+01_JPRB, 0.11007E+01_JPRB, 0.10699E+01_JPRB ,&
&  0.10745E+01_JPRB, 0.10679E+01_JPRB, 0.10424E+01_JPRB, 0.10647E+01_JPRB, 0.10920E+01_JPRB, 0.11071E+01_JPRB, 0.11748E+01_JPRB ,&
&  0.12321E+01_JPRB, 0.12607E+01_JPRB, 0.12693E+01_JPRB, 0.12040E+01_JPRB, 0.12413E+01_JPRB, 0.12706E+01_JPRB, 0.13215E+01_JPRB ,&
&  0.12652E+01_JPRB, 0.11795E+01_JPRB, 0.13506E+01_JPRB, 0.15852E+01_JPRB, 0.17785E+01_JPRB, 0.18487E+01_JPRB, 0.21016E+01_JPRB ,&
&  0.26535E+01_JPRB, 0.30836E+01_JPRB, 0.32570E+01_JPRB, 0.26981E+01_JPRB, 0.29489E+01_JPRB, 0.34893E+01_JPRB, 0.35739E+01_JPRB ,&
&  0.32963E+01_JPRB, 0.27083E+01_JPRB, 0.27933E+01_JPRB, 0.29404E+01_JPRB, 0.30885E+01_JPRB, 0.35841E+01_JPRB, 0.36705E+01_JPRB ,&
&  0.43132E+01_JPRB, 0.53304E+01_JPRB, 0.44556E+01_JPRB, 0.34470E+01_JPRB, 0.28420E+01_JPRB, 0.23964E+01_JPRB, 0.23291E+01_JPRB ,&
&  0.25388E+01_JPRB, 0.23688E+01_JPRB, 0.21260E+01_JPRB, 0.21694E+01_JPRB, 0.19274E+01_JPRB, 0.19615E+01_JPRB, 0.18109E+01_JPRB ,&
&  0.15063E+01_JPRB, 0.13038E+01_JPRB, 0.11137E+01_JPRB, 0.10058E+01_JPRB, 0.94810E+00_JPRB, 0.93469E+00_JPRB, 0.92918E+00_JPRB ,&
&  0.92987E+00_JPRB, 0.92452E+00_JPRB, 0.91409E+00_JPRB, 0.90430E+00_JPRB, 0.88656E+00_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.88561E+00_JPRB, 0.89431E+00_JPRB, 0.91614E+00_JPRB, 0.92317E+00_JPRB, 0.90792E+00_JPRB, 0.90478E+00_JPRB, 0.92587E+00_JPRB ,&
&  0.92689E+00_JPRB, 0.77920E+00_JPRB, 0.75233E+00_JPRB, 0.75287E+00_JPRB, 0.76848E+00_JPRB, 0.77460E+00_JPRB, 0.77923E+00_JPRB ,&
&  0.82293E+00_JPRB, 0.94420E+00_JPRB, 0.11387E+01_JPRB, 0.12882E+01_JPRB, 0.14348E+01_JPRB, 0.16279E+01_JPRB, 0.15711E+01_JPRB ,&
&  0.12438E+01_JPRB, 0.14211E+01_JPRB, 0.12558E+01_JPRB, 0.13659E+01_JPRB, 0.13946E+01_JPRB, 0.14684E+01_JPRB, 0.13743E+01_JPRB ,&
&  0.12840E+01_JPRB, 0.12797E+01_JPRB, 0.13265E+01_JPRB, 0.12814E+01_JPRB, 0.12405E+01_JPRB, 0.11955E+01_JPRB, 0.12216E+01_JPRB ,&
&  0.12353E+01_JPRB, 0.12219E+01_JPRB, 0.12177E+01_JPRB, 0.12527E+01_JPRB, 0.12772E+01_JPRB, 0.12849E+01_JPRB, 0.13052E+01_JPRB ,&
&  0.13197E+01_JPRB, 0.13274E+01_JPRB, 0.13034E+01_JPRB, 0.13250E+01_JPRB, 0.13490E+01_JPRB, 0.14232E+01_JPRB, 0.14875E+01_JPRB ,&
&  0.14593E+01_JPRB, 0.13942E+01_JPRB, 0.15211E+01_JPRB, 0.16779E+01_JPRB, 0.18213E+01_JPRB, 0.20882E+01_JPRB, 0.22831E+01_JPRB ,&
&  0.28171E+01_JPRB, 0.29032E+01_JPRB, 0.34360E+01_JPRB, 0.25066E+01_JPRB, 0.26427E+01_JPRB, 0.30782E+01_JPRB, 0.29620E+01_JPRB ,&
&  0.24243E+01_JPRB, 0.25206E+01_JPRB, 0.26493E+01_JPRB, 0.29024E+01_JPRB, 0.30876E+01_JPRB, 0.32735E+01_JPRB, 0.30534E+01_JPRB ,&
&  0.31224E+01_JPRB, 0.33831E+01_JPRB, 0.32384E+01_JPRB, 0.26720E+01_JPRB, 0.24465E+01_JPRB, 0.21684E+01_JPRB, 0.23528E+01_JPRB ,&
&  0.25275E+01_JPRB, 0.24235E+01_JPRB, 0.24054E+01_JPRB, 0.23052E+01_JPRB, 0.17346E+01_JPRB, 0.15702E+01_JPRB, 0.13364E+01_JPRB ,&
&  0.11147E+01_JPRB, 0.98553E+00_JPRB, 0.87198E+00_JPRB, 0.85775E+00_JPRB, 0.86420E+00_JPRB, 0.86826E+00_JPRB, 0.88879E+00_JPRB ,&
&  0.88952E+00_JPRB, 0.88732E+00_JPRB, 0.89293E+00_JPRB, 0.89235E+00_JPRB, 0.88406E+00_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.91650E+00_JPRB, 0.92547E+00_JPRB, 0.94984E+00_JPRB, 0.97927E+00_JPRB, 0.10076E+01_JPRB, 0.10510E+01_JPRB, 0.10146E+01_JPRB ,&
&  0.97902E+00_JPRB, 0.81378E+00_JPRB, 0.82792E+00_JPRB, 0.83773E+00_JPRB, 0.85041E+00_JPRB, 0.84352E+00_JPRB, 0.86735E+00_JPRB ,&
&  0.95930E+00_JPRB, 0.11014E+01_JPRB, 0.12598E+01_JPRB, 0.14236E+01_JPRB, 0.15858E+01_JPRB, 0.17857E+01_JPRB, 0.17998E+01_JPRB ,&
&  0.14334E+01_JPRB, 0.13613E+01_JPRB, 0.14073E+01_JPRB, 0.14883E+01_JPRB, 0.14896E+01_JPRB, 0.15921E+01_JPRB, 0.16384E+01_JPRB ,&
&  0.15915E+01_JPRB, 0.15483E+01_JPRB, 0.16171E+01_JPRB, 0.14771E+01_JPRB, 0.12929E+01_JPRB, 0.13195E+01_JPRB, 0.13681E+01_JPRB ,&
&  0.13955E+01_JPRB, 0.14078E+01_JPRB, 0.14286E+01_JPRB, 0.14686E+01_JPRB, 0.14881E+01_JPRB, 0.14668E+01_JPRB, 0.14398E+01_JPRB ,&
&  0.14369E+01_JPRB, 0.14566E+01_JPRB, 0.14581E+01_JPRB, 0.14418E+01_JPRB, 0.15068E+01_JPRB, 0.15869E+01_JPRB, 0.16465E+01_JPRB ,&
&  0.16908E+01_JPRB, 0.17700E+01_JPRB, 0.18554E+01_JPRB, 0.19760E+01_JPRB, 0.19934E+01_JPRB, 0.23394E+01_JPRB, 0.25362E+01_JPRB ,&
&  0.27985E+01_JPRB, 0.28667E+01_JPRB, 0.31502E+01_JPRB, 0.22557E+01_JPRB, 0.25861E+01_JPRB, 0.27373E+01_JPRB, 0.26569E+01_JPRB ,&
&  0.23834E+01_JPRB, 0.25019E+01_JPRB, 0.25527E+01_JPRB, 0.26499E+01_JPRB, 0.27851E+01_JPRB, 0.26276E+01_JPRB, 0.26576E+01_JPRB ,&
&  0.24960E+01_JPRB, 0.27595E+01_JPRB, 0.26281E+01_JPRB, 0.24538E+01_JPRB, 0.21566E+01_JPRB, 0.20318E+01_JPRB, 0.21327E+01_JPRB ,&
&  0.23227E+01_JPRB, 0.22595E+01_JPRB, 0.21246E+01_JPRB, 0.18216E+01_JPRB, 0.13417E+01_JPRB, 0.11655E+01_JPRB, 0.99526E+00_JPRB ,&
&  0.92940E+00_JPRB, 0.85886E+00_JPRB, 0.83463E+00_JPRB, 0.85434E+00_JPRB, 0.86048E+00_JPRB, 0.89827E+00_JPRB, 0.90007E+00_JPRB ,&
&  0.88802E+00_JPRB, 0.89939E+00_JPRB, 0.91233E+00_JPRB, 0.91667E+00_JPRB, 0.91638E+00_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.90045E+00_JPRB, 0.89453E+00_JPRB, 0.89139E+00_JPRB, 0.89962E+00_JPRB, 0.91990E+00_JPRB, 0.93155E+00_JPRB, 0.88796E+00_JPRB ,&
&  0.87366E+00_JPRB, 0.87469E+00_JPRB, 0.90865E+00_JPRB, 0.92274E+00_JPRB, 0.92762E+00_JPRB, 0.93521E+00_JPRB, 0.98663E+00_JPRB ,&
&  0.10789E+01_JPRB, 0.12185E+01_JPRB, 0.13666E+01_JPRB, 0.14682E+01_JPRB, 0.15594E+01_JPRB, 0.16816E+01_JPRB, 0.18073E+01_JPRB ,&
&  0.15929E+01_JPRB, 0.12906E+01_JPRB, 0.14932E+01_JPRB, 0.16044E+01_JPRB, 0.15860E+01_JPRB, 0.17602E+01_JPRB, 0.20175E+01_JPRB ,&
&  0.16993E+01_JPRB, 0.16215E+01_JPRB, 0.16445E+01_JPRB, 0.15618E+01_JPRB, 0.14074E+01_JPRB, 0.14564E+01_JPRB, 0.15203E+01_JPRB ,&
&  0.15672E+01_JPRB, 0.15953E+01_JPRB, 0.16515E+01_JPRB, 0.17034E+01_JPRB, 0.17197E+01_JPRB, 0.16912E+01_JPRB, 0.16512E+01_JPRB ,&
&  0.16710E+01_JPRB, 0.17098E+01_JPRB, 0.17570E+01_JPRB, 0.17584E+01_JPRB, 0.18017E+01_JPRB, 0.18877E+01_JPRB, 0.19683E+01_JPRB ,&
&  0.20425E+01_JPRB, 0.20638E+01_JPRB, 0.21888E+01_JPRB, 0.24286E+01_JPRB, 0.26424E+01_JPRB, 0.24060E+01_JPRB, 0.26380E+01_JPRB ,&
&  0.26022E+01_JPRB, 0.25154E+01_JPRB, 0.23943E+01_JPRB, 0.28084E+01_JPRB, 0.19769E+01_JPRB, 0.22684E+01_JPRB, 0.21235E+01_JPRB ,&
&  0.24307E+01_JPRB, 0.23790E+01_JPRB, 0.23853E+01_JPRB, 0.24593E+01_JPRB, 0.25516E+01_JPRB, 0.23257E+01_JPRB, 0.22886E+01_JPRB ,&
&  0.23330E+01_JPRB, 0.23665E+01_JPRB, 0.23406E+01_JPRB, 0.23260E+01_JPRB, 0.21535E+01_JPRB, 0.21090E+01_JPRB, 0.20589E+01_JPRB ,&
&  0.21138E+01_JPRB, 0.18849E+01_JPRB, 0.16240E+01_JPRB, 0.13191E+01_JPRB, 0.10219E+01_JPRB, 0.93448E+00_JPRB, 0.91025E+00_JPRB ,&
&  0.91386E+00_JPRB, 0.93126E+00_JPRB, 0.89672E+00_JPRB, 0.88195E+00_JPRB, 0.89152E+00_JPRB, 0.92435E+00_JPRB, 0.91459E+00_JPRB ,&
&  0.92124E+00_JPRB, 0.92109E+00_JPRB, 0.93018E+00_JPRB, 0.92874E+00_JPRB, 0.91970E+00_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.89535E+00_JPRB, 0.88529E+00_JPRB, 0.87161E+00_JPRB, 0.88182E+00_JPRB, 0.88692E+00_JPRB, 0.88342E+00_JPRB, 0.87280E+00_JPRB ,&
&  0.88154E+00_JPRB, 0.91541E+00_JPRB, 0.95416E+00_JPRB, 0.97609E+00_JPRB, 0.98165E+00_JPRB, 0.99472E+00_JPRB, 0.10473E+01_JPRB ,&
&  0.11287E+01_JPRB, 0.12517E+01_JPRB, 0.14027E+01_JPRB, 0.15108E+01_JPRB, 0.16009E+01_JPRB, 0.16484E+01_JPRB, 0.17470E+01_JPRB ,&
&  0.17312E+01_JPRB, 0.17497E+01_JPRB, 0.16917E+01_JPRB, 0.14266E+01_JPRB, 0.13630E+01_JPRB, 0.17039E+01_JPRB, 0.17071E+01_JPRB ,&
&  0.16164E+01_JPRB, 0.15768E+01_JPRB, 0.15576E+01_JPRB, 0.15464E+01_JPRB, 0.15485E+01_JPRB, 0.16154E+01_JPRB, 0.16896E+01_JPRB ,&
&  0.17378E+01_JPRB, 0.18074E+01_JPRB, 0.18752E+01_JPRB, 0.19230E+01_JPRB, 0.19486E+01_JPRB, 0.19873E+01_JPRB, 0.20002E+01_JPRB ,&
&  0.20787E+01_JPRB, 0.21699E+01_JPRB, 0.22636E+01_JPRB, 0.22985E+01_JPRB, 0.22708E+01_JPRB, 0.23438E+01_JPRB, 0.24073E+01_JPRB ,&
&  0.24724E+01_JPRB, 0.24670E+01_JPRB, 0.24484E+01_JPRB, 0.25539E+01_JPRB, 0.25962E+01_JPRB, 0.22459E+01_JPRB, 0.22342E+01_JPRB ,&
&  0.23597E+01_JPRB, 0.22801E+01_JPRB, 0.20101E+01_JPRB, 0.25208E+01_JPRB, 0.15893E+01_JPRB, 0.20107E+01_JPRB, 0.23013E+01_JPRB ,&
&  0.22361E+01_JPRB, 0.22030E+01_JPRB, 0.22266E+01_JPRB, 0.22618E+01_JPRB, 0.22497E+01_JPRB, 0.20768E+01_JPRB, 0.21651E+01_JPRB ,&
&  0.22401E+01_JPRB, 0.21968E+01_JPRB, 0.21617E+01_JPRB, 0.21310E+01_JPRB, 0.20607E+01_JPRB, 0.18880E+01_JPRB, 0.16435E+01_JPRB ,&
&  0.13983E+01_JPRB, 0.15624E+01_JPRB, 0.13655E+01_JPRB, 0.12210E+01_JPRB, 0.99847E+00_JPRB, 0.86863E+00_JPRB, 0.89792E+00_JPRB ,&
&  0.92596E+00_JPRB, 0.92233E+00_JPRB, 0.90547E+00_JPRB, 0.91481E+00_JPRB, 0.92024E+00_JPRB, 0.92912E+00_JPRB, 0.91983E+00_JPRB ,&
&  0.92523E+00_JPRB, 0.93016E+00_JPRB, 0.93225E+00_JPRB, 0.92278E+00_JPRB, 0.90985E+00_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.86098E+00_JPRB, 0.85109E+00_JPRB, 0.84165E+00_JPRB, 0.87295E+00_JPRB, 0.86714E+00_JPRB, 0.85465E+00_JPRB, 0.86077E+00_JPRB ,&
&  0.87296E+00_JPRB, 0.89287E+00_JPRB, 0.92043E+00_JPRB, 0.94992E+00_JPRB, 0.98957E+00_JPRB, 0.10488E+01_JPRB, 0.11077E+01_JPRB ,&
&  0.11763E+01_JPRB, 0.12514E+01_JPRB, 0.13768E+01_JPRB, 0.15351E+01_JPRB, 0.17367E+01_JPRB, 0.18398E+01_JPRB, 0.18391E+01_JPRB ,&
&  0.17263E+01_JPRB, 0.17798E+01_JPRB, 0.18661E+01_JPRB, 0.19359E+01_JPRB, 0.17017E+01_JPRB, 0.18520E+01_JPRB, 0.20132E+01_JPRB ,&
&  0.19998E+01_JPRB, 0.18005E+01_JPRB, 0.16547E+01_JPRB, 0.16519E+01_JPRB, 0.16664E+01_JPRB, 0.17522E+01_JPRB, 0.18433E+01_JPRB ,&
&  0.19184E+01_JPRB, 0.20041E+01_JPRB, 0.20650E+01_JPRB, 0.21284E+01_JPRB, 0.22180E+01_JPRB, 0.23531E+01_JPRB, 0.24220E+01_JPRB ,&
&  0.24918E+01_JPRB, 0.26288E+01_JPRB, 0.26977E+01_JPRB, 0.26613E+01_JPRB, 0.27373E+01_JPRB, 0.27868E+01_JPRB, 0.28239E+01_JPRB ,&
&  0.28937E+01_JPRB, 0.27399E+01_JPRB, 0.27686E+01_JPRB, 0.28298E+01_JPRB, 0.26869E+01_JPRB, 0.23791E+01_JPRB, 0.22885E+01_JPRB ,&
&  0.23249E+01_JPRB, 0.22898E+01_JPRB, 0.16466E+01_JPRB, 0.22326E+01_JPRB, 0.26811E+01_JPRB, 0.23005E+01_JPRB, 0.22062E+01_JPRB ,&
&  0.20908E+01_JPRB, 0.21048E+01_JPRB, 0.21649E+01_JPRB, 0.22241E+01_JPRB, 0.22500E+01_JPRB, 0.22459E+01_JPRB, 0.20944E+01_JPRB ,&
&  0.22486E+01_JPRB, 0.21603E+01_JPRB, 0.20319E+01_JPRB, 0.19152E+01_JPRB, 0.17612E+01_JPRB, 0.15171E+01_JPRB, 0.12787E+01_JPRB ,&
&  0.11410E+01_JPRB, 0.14015E+01_JPRB, 0.13096E+01_JPRB, 0.13361E+01_JPRB, 0.12665E+01_JPRB, 0.87064E+00_JPRB, 0.91634E+00_JPRB ,&
&  0.92494E+00_JPRB, 0.92608E+00_JPRB, 0.91583E+00_JPRB, 0.89512E+00_JPRB, 0.89725E+00_JPRB, 0.90427E+00_JPRB, 0.90964E+00_JPRB ,&
&  0.90852E+00_JPRB, 0.89954E+00_JPRB, 0.89626E+00_JPRB, 0.88086E+00_JPRB, 0.87201E+00_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.86123E+00_JPRB, 0.86129E+00_JPRB, 0.85454E+00_JPRB, 0.86778E+00_JPRB, 0.85971E+00_JPRB, 0.87498E+00_JPRB, 0.85665E+00_JPRB ,&
&  0.83909E+00_JPRB, 0.83524E+00_JPRB, 0.84081E+00_JPRB, 0.84761E+00_JPRB, 0.87327E+00_JPRB, 0.92664E+00_JPRB, 0.99291E+00_JPRB ,&
&  0.10579E+01_JPRB, 0.10717E+01_JPRB, 0.11116E+01_JPRB, 0.12221E+01_JPRB, 0.14634E+01_JPRB, 0.17033E+01_JPRB, 0.17676E+01_JPRB ,&
&  0.18724E+01_JPRB, 0.20336E+01_JPRB, 0.21757E+01_JPRB, 0.23304E+01_JPRB, 0.23525E+01_JPRB, 0.23714E+01_JPRB, 0.24833E+01_JPRB ,&
&  0.21490E+01_JPRB, 0.17511E+01_JPRB, 0.16773E+01_JPRB, 0.16954E+01_JPRB, 0.17584E+01_JPRB, 0.18411E+01_JPRB, 0.19172E+01_JPRB ,&
&  0.19956E+01_JPRB, 0.20892E+01_JPRB, 0.21762E+01_JPRB, 0.22944E+01_JPRB, 0.24596E+01_JPRB, 0.26488E+01_JPRB, 0.27715E+01_JPRB ,&
&  0.28484E+01_JPRB, 0.29543E+01_JPRB, 0.30881E+01_JPRB, 0.29836E+01_JPRB, 0.33465E+01_JPRB, 0.34649E+01_JPRB, 0.35110E+01_JPRB ,&
&  0.34460E+01_JPRB, 0.32387E+01_JPRB, 0.32331E+01_JPRB, 0.32091E+01_JPRB, 0.31631E+01_JPRB, 0.28435E+01_JPRB, 0.26138E+01_JPRB ,&
&  0.25612E+01_JPRB, 0.21632E+01_JPRB, 0.15512E+01_JPRB, 0.16874E+01_JPRB, 0.18899E+01_JPRB, 0.17009E+01_JPRB, 0.21214E+01_JPRB ,&
&  0.20386E+01_JPRB, 0.19938E+01_JPRB, 0.20087E+01_JPRB, 0.20528E+01_JPRB, 0.21581E+01_JPRB, 0.21198E+01_JPRB, 0.20471E+01_JPRB ,&
&  0.20517E+01_JPRB, 0.19148E+01_JPRB, 0.17465E+01_JPRB, 0.15602E+01_JPRB, 0.13714E+01_JPRB, 0.12141E+01_JPRB, 0.11315E+01_JPRB ,&
&  0.11202E+01_JPRB, 0.11393E+01_JPRB, 0.10919E+01_JPRB, 0.10137E+01_JPRB, 0.87390E+00_JPRB, 0.83573E+00_JPRB, 0.85067E+00_JPRB ,&
&  0.88821E+00_JPRB, 0.88915E+00_JPRB, 0.88259E+00_JPRB, 0.86577E+00_JPRB, 0.85808E+00_JPRB, 0.85259E+00_JPRB, 0.85293E+00_JPRB ,&
&  0.86539E+00_JPRB, 0.88197E+00_JPRB, 0.87981E+00_JPRB, 0.87501E+00_JPRB, 0.86887E+00_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.85267E+00_JPRB, 0.84909E+00_JPRB, 0.84821E+00_JPRB, 0.86357E+00_JPRB, 0.87607E+00_JPRB, 0.86341E+00_JPRB, 0.87024E+00_JPRB ,&
&  0.84670E+00_JPRB, 0.82788E+00_JPRB, 0.83790E+00_JPRB, 0.84316E+00_JPRB, 0.86033E+00_JPRB, 0.87707E+00_JPRB, 0.88675E+00_JPRB ,&
&  0.88437E+00_JPRB, 0.87543E+00_JPRB, 0.83759E+00_JPRB, 0.84477E+00_JPRB, 0.93252E+00_JPRB, 0.10840E+01_JPRB, 0.13739E+01_JPRB ,&
&  0.16985E+01_JPRB, 0.18663E+01_JPRB, 0.19941E+01_JPRB, 0.20352E+01_JPRB, 0.19689E+01_JPRB, 0.18609E+01_JPRB, 0.24502E+01_JPRB ,&
&  0.24482E+01_JPRB, 0.17342E+01_JPRB, 0.17781E+01_JPRB, 0.16731E+01_JPRB, 0.17375E+01_JPRB, 0.17744E+01_JPRB, 0.18423E+01_JPRB ,&
&  0.19030E+01_JPRB, 0.20046E+01_JPRB, 0.21478E+01_JPRB, 0.23365E+01_JPRB, 0.25275E+01_JPRB, 0.27286E+01_JPRB, 0.28879E+01_JPRB ,&
&  0.29955E+01_JPRB, 0.30953E+01_JPRB, 0.32773E+01_JPRB, 0.33426E+01_JPRB, 0.33833E+01_JPRB, 0.35655E+01_JPRB, 0.37572E+01_JPRB ,&
&  0.38925E+01_JPRB, 0.41308E+01_JPRB, 0.36207E+01_JPRB, 0.35626E+01_JPRB, 0.36670E+01_JPRB, 0.30943E+01_JPRB, 0.29059E+01_JPRB ,&
&  0.27941E+01_JPRB, 0.21413E+01_JPRB, 0.14094E+01_JPRB, 0.14994E+01_JPRB, 0.16294E+01_JPRB, 0.17183E+01_JPRB, 0.18536E+01_JPRB ,&
&  0.17788E+01_JPRB, 0.16960E+01_JPRB, 0.17088E+01_JPRB, 0.18124E+01_JPRB, 0.18345E+01_JPRB, 0.17695E+01_JPRB, 0.16796E+01_JPRB ,&
&  0.16157E+01_JPRB, 0.14779E+01_JPRB, 0.14001E+01_JPRB, 0.12772E+01_JPRB, 0.11905E+01_JPRB, 0.11132E+01_JPRB, 0.10207E+01_JPRB ,&
&  0.10904E+01_JPRB, 0.12002E+01_JPRB, 0.14280E+01_JPRB, 0.12637E+01_JPRB, 0.87479E+00_JPRB, 0.84235E+00_JPRB, 0.83057E+00_JPRB ,&
&  0.84162E+00_JPRB, 0.84831E+00_JPRB, 0.86700E+00_JPRB, 0.86251E+00_JPRB, 0.84692E+00_JPRB, 0.83149E+00_JPRB, 0.81855E+00_JPRB ,&
&  0.83615E+00_JPRB, 0.85919E+00_JPRB, 0.87038E+00_JPRB, 0.86017E+00_JPRB, 0.85167E+00_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.91529E+00_JPRB, 0.91897E+00_JPRB, 0.93845E+00_JPRB, 0.94324E+00_JPRB, 0.94259E+00_JPRB, 0.94969E+00_JPRB, 0.95028E+00_JPRB ,&
&  0.94293E+00_JPRB, 0.94935E+00_JPRB, 0.95874E+00_JPRB, 0.97009E+00_JPRB, 0.98542E+00_JPRB, 0.99966E+00_JPRB, 0.10092E+01_JPRB ,&
&  0.10011E+01_JPRB, 0.99324E+00_JPRB, 0.10196E+01_JPRB, 0.10397E+01_JPRB, 0.10359E+01_JPRB, 0.10298E+01_JPRB, 0.10513E+01_JPRB ,&
&  0.11077E+01_JPRB, 0.11426E+01_JPRB, 0.11548E+01_JPRB, 0.11438E+01_JPRB, 0.11857E+01_JPRB, 0.14067E+01_JPRB, 0.21787E+01_JPRB ,&
&  0.22936E+01_JPRB, 0.17568E+01_JPRB, 0.16025E+01_JPRB, 0.14617E+01_JPRB, 0.15049E+01_JPRB, 0.16142E+01_JPRB, 0.16314E+01_JPRB ,&
&  0.16300E+01_JPRB, 0.16864E+01_JPRB, 0.17481E+01_JPRB, 0.18074E+01_JPRB, 0.18949E+01_JPRB, 0.19613E+01_JPRB, 0.19983E+01_JPRB ,&
&  0.19673E+01_JPRB, 0.19634E+01_JPRB, 0.20355E+01_JPRB, 0.21642E+01_JPRB, 0.25072E+01_JPRB, 0.30276E+01_JPRB, 0.29262E+01_JPRB ,&
&  0.29337E+01_JPRB, 0.32096E+01_JPRB, 0.34185E+01_JPRB, 0.35153E+01_JPRB, 0.33401E+01_JPRB, 0.29672E+01_JPRB, 0.26524E+01_JPRB ,&
&  0.23315E+01_JPRB, 0.18662E+01_JPRB, 0.15027E+01_JPRB, 0.16850E+01_JPRB, 0.17126E+01_JPRB, 0.17522E+01_JPRB, 0.16095E+01_JPRB ,&
&  0.15203E+01_JPRB, 0.13653E+01_JPRB, 0.13502E+01_JPRB, 0.14214E+01_JPRB, 0.14786E+01_JPRB, 0.14384E+01_JPRB, 0.13581E+01_JPRB ,&
&  0.12543E+01_JPRB, 0.12203E+01_JPRB, 0.12376E+01_JPRB, 0.12034E+01_JPRB, 0.11970E+01_JPRB, 0.10145E+01_JPRB, 0.99560E+00_JPRB ,&
&  0.10512E+01_JPRB, 0.11621E+01_JPRB, 0.98617E+00_JPRB, 0.12568E+01_JPRB, 0.84574E+00_JPRB, 0.77116E+00_JPRB, 0.78092E+00_JPRB ,&
&  0.80324E+00_JPRB, 0.83382E+00_JPRB, 0.84202E+00_JPRB, 0.92730E+00_JPRB, 0.91471E+00_JPRB, 0.90698E+00_JPRB, 0.91596E+00_JPRB ,&
&  0.89032E+00_JPRB, 0.88770E+00_JPRB, 0.88987E+00_JPRB, 0.90325E+00_JPRB, 0.90993E+00_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.98139E+00_JPRB, 0.98591E+00_JPRB, 0.99415E+00_JPRB, 0.10072E+01_JPRB, 0.10183E+01_JPRB, 0.10284E+01_JPRB, 0.10321E+01_JPRB ,&
&  0.10258E+01_JPRB, 0.10281E+01_JPRB, 0.10390E+01_JPRB, 0.10559E+01_JPRB, 0.10723E+01_JPRB, 0.10863E+01_JPRB, 0.11038E+01_JPRB ,&
&  0.11045E+01_JPRB, 0.11100E+01_JPRB, 0.11330E+01_JPRB, 0.11584E+01_JPRB, 0.11717E+01_JPRB, 0.11715E+01_JPRB, 0.11756E+01_JPRB ,&
&  0.11704E+01_JPRB, 0.11820E+01_JPRB, 0.12157E+01_JPRB, 0.12408E+01_JPRB, 0.12738E+01_JPRB, 0.13569E+01_JPRB, 0.16065E+01_JPRB ,&
&  0.17028E+01_JPRB, 0.19039E+01_JPRB, 0.17104E+01_JPRB, 0.15430E+01_JPRB, 0.15434E+01_JPRB, 0.14780E+01_JPRB, 0.14311E+01_JPRB ,&
&  0.13808E+01_JPRB, 0.13603E+01_JPRB, 0.13104E+01_JPRB, 0.13185E+01_JPRB, 0.13094E+01_JPRB, 0.13468E+01_JPRB, 0.13488E+01_JPRB ,&
&  0.14149E+01_JPRB, 0.14914E+01_JPRB, 0.16441E+01_JPRB, 0.17816E+01_JPRB, 0.19647E+01_JPRB, 0.20906E+01_JPRB, 0.18945E+01_JPRB ,&
&  0.18581E+01_JPRB, 0.20776E+01_JPRB, 0.24101E+01_JPRB, 0.26412E+01_JPRB, 0.24795E+01_JPRB, 0.22017E+01_JPRB, 0.19635E+01_JPRB ,&
&  0.17236E+01_JPRB, 0.15665E+01_JPRB, 0.17583E+01_JPRB, 0.17135E+01_JPRB, 0.17000E+01_JPRB, 0.15910E+01_JPRB, 0.14106E+01_JPRB ,&
&  0.12406E+01_JPRB, 0.10468E+01_JPRB, 0.10245E+01_JPRB, 0.10222E+01_JPRB, 0.10438E+01_JPRB, 0.10200E+01_JPRB, 0.10048E+01_JPRB ,&
&  0.95349E+00_JPRB, 0.92102E+00_JPRB, 0.10265E+01_JPRB, 0.11546E+01_JPRB, 0.12602E+01_JPRB, 0.15649E+01_JPRB, 0.14540E+01_JPRB ,&
&  0.10177E+01_JPRB, 0.94013E+00_JPRB, 0.63867E+00_JPRB, 0.87914E+00_JPRB, 0.70912E+00_JPRB, 0.72212E+00_JPRB, 0.80063E+00_JPRB ,&
&  0.91107E+00_JPRB, 0.10075E+01_JPRB, 0.10749E+01_JPRB, 0.10395E+01_JPRB, 0.96826E+00_JPRB, 0.95665E+00_JPRB, 0.95106E+00_JPRB ,&
&  0.95983E+00_JPRB, 0.96538E+00_JPRB, 0.96835E+00_JPRB, 0.97077E+00_JPRB, 0.96950E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.10156E+01_JPRB, 0.10279E+01_JPRB, 0.10444E+01_JPRB, 0.10649E+01_JPRB, 0.10843E+01_JPRB, 0.11024E+01_JPRB, 0.11171E+01_JPRB ,&
&  0.11275E+01_JPRB, 0.11442E+01_JPRB, 0.11704E+01_JPRB, 0.12005E+01_JPRB, 0.12149E+01_JPRB, 0.12332E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.12675E+01_JPRB, 0.13002E+01_JPRB, 0.13175E+01_JPRB, 0.13379E+01_JPRB, 0.13524E+01_JPRB, 0.13708E+01_JPRB, 0.13947E+01_JPRB ,&
&  0.14156E+01_JPRB, 0.14628E+01_JPRB, 0.15333E+01_JPRB, 0.15907E+01_JPRB, 0.16728E+01_JPRB, 0.18342E+01_JPRB, 0.16282E+01_JPRB ,&
&  0.18773E+01_JPRB, 0.19180E+01_JPRB, 0.18317E+01_JPRB, 0.16439E+01_JPRB, 0.15808E+01_JPRB, 0.15225E+01_JPRB, 0.14800E+01_JPRB ,&
&  0.14549E+01_JPRB, 0.14052E+01_JPRB, 0.14280E+01_JPRB, 0.14281E+01_JPRB, 0.14315E+01_JPRB, 0.15326E+01_JPRB, 0.16127E+01_JPRB ,&
&  0.16586E+01_JPRB, 0.17083E+01_JPRB, 0.17799E+01_JPRB, 0.18249E+01_JPRB, 0.19003E+01_JPRB, 0.19377E+01_JPRB, 0.18439E+01_JPRB ,&
&  0.17116E+01_JPRB, 0.16523E+01_JPRB, 0.15992E+01_JPRB, 0.16437E+01_JPRB, 0.16049E+01_JPRB, 0.14022E+01_JPRB, 0.11851E+01_JPRB ,&
&  0.13258E+01_JPRB, 0.12780E+01_JPRB, 0.13232E+01_JPRB, 0.17858E+01_JPRB, 0.16695E+01_JPRB, 0.15149E+01_JPRB, 0.12514E+01_JPRB ,&
&  0.10836E+01_JPRB, 0.87664E+00_JPRB, 0.80423E+00_JPRB, 0.81069E+00_JPRB, 0.79464E+00_JPRB, 0.78912E+00_JPRB, 0.76539E+00_JPRB ,&
&  0.72607E+00_JPRB, 0.72883E+00_JPRB, 0.83545E+00_JPRB, 0.10081E+01_JPRB, 0.12397E+01_JPRB, 0.13334E+01_JPRB, 0.17450E+01_JPRB ,&
&  0.11458E+01_JPRB, 0.68633E+00_JPRB, 0.76189E+00_JPRB, 0.75040E+00_JPRB, 0.69747E+00_JPRB, 0.67398E+00_JPRB, 0.79925E+00_JPRB ,&
&  0.10082E+01_JPRB, 0.10721E+01_JPRB, 0.10934E+01_JPRB, 0.10448E+01_JPRB, 0.10383E+01_JPRB, 0.10395E+01_JPRB, 0.10261E+01_JPRB ,&
&  0.10217E+01_JPRB, 0.10204E+01_JPRB, 0.10203E+01_JPRB, 0.10131E+01_JPRB, 0.10098E+01_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.10587E+01_JPRB, 0.10759E+01_JPRB, 0.11008E+01_JPRB, 0.11330E+01_JPRB, 0.11519E+01_JPRB, 0.11800E+01_JPRB, 0.11935E+01_JPRB ,&
&  0.12096E+01_JPRB, 0.12438E+01_JPRB, 0.12759E+01_JPRB, 0.13109E+01_JPRB, 0.13448E+01_JPRB, 0.13767E+01_JPRB, 0.14081E+01_JPRB ,&
&  0.14342E+01_JPRB, 0.14612E+01_JPRB, 0.14754E+01_JPRB, 0.14794E+01_JPRB, 0.14688E+01_JPRB, 0.14649E+01_JPRB, 0.14502E+01_JPRB ,&
&  0.14360E+01_JPRB, 0.14425E+01_JPRB, 0.14605E+01_JPRB, 0.14752E+01_JPRB, 0.15024E+01_JPRB, 0.16224E+01_JPRB, 0.12483E+01_JPRB ,&
&  0.18032E+01_JPRB, 0.18963E+01_JPRB, 0.19042E+01_JPRB, 0.17740E+01_JPRB, 0.17112E+01_JPRB, 0.17359E+01_JPRB, 0.18017E+01_JPRB ,&
&  0.19030E+01_JPRB, 0.20402E+01_JPRB, 0.21731E+01_JPRB, 0.20956E+01_JPRB, 0.19800E+01_JPRB, 0.20349E+01_JPRB, 0.21140E+01_JPRB ,&
&  0.21608E+01_JPRB, 0.21789E+01_JPRB, 0.22018E+01_JPRB, 0.21835E+01_JPRB, 0.21696E+01_JPRB, 0.21454E+01_JPRB, 0.20799E+01_JPRB ,&
&  0.19751E+01_JPRB, 0.18337E+01_JPRB, 0.16508E+01_JPRB, 0.17866E+01_JPRB, 0.17956E+01_JPRB, 0.14989E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.97308E+00_JPRB, 0.11380E+01_JPRB, 0.13632E+01_JPRB, 0.18041E+01_JPRB, 0.15819E+01_JPRB, 0.13784E+01_JPRB, 0.12256E+01_JPRB ,&
&  0.99754E+00_JPRB, 0.83577E+00_JPRB, 0.74431E+00_JPRB, 0.73845E+00_JPRB, 0.73803E+00_JPRB, 0.74958E+00_JPRB, 0.73370E+00_JPRB ,&
&  0.71370E+00_JPRB, 0.70849E+00_JPRB, 0.77399E+00_JPRB, 0.87793E+00_JPRB, 0.10433E+01_JPRB, 0.10721E+01_JPRB, 0.16429E+01_JPRB ,&
&  0.11229E+01_JPRB, 0.90497E+00_JPRB, 0.12026E+01_JPRB, 0.61941E+00_JPRB, 0.74413E+00_JPRB, 0.82480E+00_JPRB, 0.93752E+00_JPRB ,&
&  0.96533E+00_JPRB, 0.12656E+01_JPRB, 0.15592E+01_JPRB, 0.13842E+01_JPRB, 0.12515E+01_JPRB, 0.11412E+01_JPRB, 0.10976E+01_JPRB ,&
&  0.10872E+01_JPRB, 0.10779E+01_JPRB, 0.10757E+01_JPRB, 0.10686E+01_JPRB, 0.10579E+01_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.11344E+01_JPRB, 0.11413E+01_JPRB, 0.11391E+01_JPRB, 0.11751E+01_JPRB, 0.11775E+01_JPRB, 0.11978E+01_JPRB, 0.12030E+01_JPRB ,&
&  0.12298E+01_JPRB, 0.12535E+01_JPRB, 0.12822E+01_JPRB, 0.13264E+01_JPRB, 0.13479E+01_JPRB, 0.13716E+01_JPRB, 0.14090E+01_JPRB ,&
&  0.14453E+01_JPRB, 0.14901E+01_JPRB, 0.14951E+01_JPRB, 0.14882E+01_JPRB, 0.14627E+01_JPRB, 0.14503E+01_JPRB, 0.14326E+01_JPRB ,&
&  0.14194E+01_JPRB, 0.14195E+01_JPRB, 0.14355E+01_JPRB, 0.14622E+01_JPRB, 0.14924E+01_JPRB, 0.15666E+01_JPRB, 0.11067E+01_JPRB ,&
&  0.17508E+01_JPRB, 0.19630E+01_JPRB, 0.19413E+01_JPRB, 0.19337E+01_JPRB, 0.18596E+01_JPRB, 0.19229E+01_JPRB, 0.20371E+01_JPRB ,&
&  0.21982E+01_JPRB, 0.23605E+01_JPRB, 0.24449E+01_JPRB, 0.24883E+01_JPRB, 0.24983E+01_JPRB, 0.26011E+01_JPRB, 0.27015E+01_JPRB ,&
&  0.27705E+01_JPRB, 0.28156E+01_JPRB, 0.28674E+01_JPRB, 0.28972E+01_JPRB, 0.29392E+01_JPRB, 0.29926E+01_JPRB, 0.29914E+01_JPRB ,&
&  0.29352E+01_JPRB, 0.28826E+01_JPRB, 0.29164E+01_JPRB, 0.30966E+01_JPRB, 0.30693E+01_JPRB, 0.26226E+01_JPRB, 0.19835E+01_JPRB ,&
&  0.12390E+01_JPRB, 0.97568E+00_JPRB, 0.13061E+01_JPRB, 0.17251E+01_JPRB, 0.14871E+01_JPRB, 0.12865E+01_JPRB, 0.11171E+01_JPRB ,&
&  0.10008E+01_JPRB, 0.86084E+00_JPRB, 0.75561E+00_JPRB, 0.74900E+00_JPRB, 0.77514E+00_JPRB, 0.83302E+00_JPRB, 0.81675E+00_JPRB ,&
&  0.83540E+00_JPRB, 0.85431E+00_JPRB, 0.86350E+00_JPRB, 0.91736E+00_JPRB, 0.10706E+01_JPRB, 0.11810E+01_JPRB, 0.11067E+01_JPRB ,&
&  0.12608E+01_JPRB, 0.10122E+01_JPRB, 0.97609E+00_JPRB, 0.10602E+01_JPRB, 0.10343E+01_JPRB, 0.12173E+01_JPRB, 0.15086E+01_JPRB ,&
&  0.19614E+01_JPRB, 0.18983E+01_JPRB, 0.24997E+01_JPRB, 0.37991E+01_JPRB, 0.29872E+01_JPRB, 0.21474E+01_JPRB, 0.11836E+01_JPRB ,&
&  0.11732E+01_JPRB, 0.11497E+01_JPRB, 0.11623E+01_JPRB, 0.11491E+01_JPRB, 0.11387E+01_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.10841E+01_JPRB, 0.10717E+01_JPRB, 0.10756E+01_JPRB, 0.10784E+01_JPRB, 0.10772E+01_JPRB, 0.10893E+01_JPRB, 0.11144E+01_JPRB ,&
&  0.11292E+01_JPRB, 0.11694E+01_JPRB, 0.12322E+01_JPRB, 0.12768E+01_JPRB, 0.13356E+01_JPRB, 0.13516E+01_JPRB, 0.13298E+01_JPRB ,&
&  0.13315E+01_JPRB, 0.13749E+01_JPRB, 0.14153E+01_JPRB, 0.14357E+01_JPRB, 0.14067E+01_JPRB, 0.13975E+01_JPRB, 0.14292E+01_JPRB ,&
&  0.14859E+01_JPRB, 0.15333E+01_JPRB, 0.15795E+01_JPRB, 0.16454E+01_JPRB, 0.16657E+01_JPRB, 0.15966E+01_JPRB, 0.11329E+01_JPRB ,&
&  0.16376E+01_JPRB, 0.19357E+01_JPRB, 0.20487E+01_JPRB, 0.20410E+01_JPRB, 0.20644E+01_JPRB, 0.20100E+01_JPRB, 0.21141E+01_JPRB ,&
&  0.22816E+01_JPRB, 0.24564E+01_JPRB, 0.24901E+01_JPRB, 0.24306E+01_JPRB, 0.28103E+01_JPRB, 0.29254E+01_JPRB, 0.30901E+01_JPRB ,&
&  0.32293E+01_JPRB, 0.33539E+01_JPRB, 0.34746E+01_JPRB, 0.36154E+01_JPRB, 0.37400E+01_JPRB, 0.38549E+01_JPRB, 0.39462E+01_JPRB ,&
&  0.39771E+01_JPRB, 0.39859E+01_JPRB, 0.40870E+01_JPRB, 0.39303E+01_JPRB, 0.38353E+01_JPRB, 0.39751E+01_JPRB, 0.33539E+01_JPRB ,&
&  0.18723E+01_JPRB, 0.11776E+01_JPRB, 0.14499E+01_JPRB, 0.16147E+01_JPRB, 0.13683E+01_JPRB, 0.12060E+01_JPRB, 0.11353E+01_JPRB ,&
&  0.99403E+00_JPRB, 0.86767E+00_JPRB, 0.83019E+00_JPRB, 0.87668E+00_JPRB, 0.96317E+00_JPRB, 0.10443E+01_JPRB, 0.10628E+01_JPRB ,&
&  0.10292E+01_JPRB, 0.10156E+01_JPRB, 0.10280E+01_JPRB, 0.10705E+01_JPRB, 0.11713E+01_JPRB, 0.13749E+01_JPRB, 0.16587E+01_JPRB ,&
&  0.15807E+01_JPRB, 0.11175E+01_JPRB, 0.11693E+01_JPRB, 0.12074E+01_JPRB, 0.12941E+01_JPRB, 0.14390E+01_JPRB, 0.17352E+01_JPRB ,&
&  0.22027E+01_JPRB, 0.26963E+01_JPRB, 0.26175E+01_JPRB, 0.21890E+01_JPRB, 0.27430E+01_JPRB, 0.18554E+01_JPRB, 0.10144E+01_JPRB ,&
&  0.10484E+01_JPRB, 0.10687E+01_JPRB, 0.10920E+01_JPRB, 0.10931E+01_JPRB, 0.10917E+01_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.93612E+00_JPRB, 0.92039E+00_JPRB, 0.91062E+00_JPRB, 0.90194E+00_JPRB, 0.91445E+00_JPRB, 0.95180E+00_JPRB, 0.99555E+00_JPRB ,&
&  0.10429E+01_JPRB, 0.10968E+01_JPRB, 0.11666E+01_JPRB, 0.11997E+01_JPRB, 0.12472E+01_JPRB, 0.12550E+01_JPRB, 0.12534E+01_JPRB ,&
&  0.12415E+01_JPRB, 0.12630E+01_JPRB, 0.12989E+01_JPRB, 0.13355E+01_JPRB, 0.13581E+01_JPRB, 0.14020E+01_JPRB, 0.14594E+01_JPRB ,&
&  0.15328E+01_JPRB, 0.16031E+01_JPRB, 0.16895E+01_JPRB, 0.18229E+01_JPRB, 0.18797E+01_JPRB, 0.19044E+01_JPRB, 0.19473E+01_JPRB ,&
&  0.10894E+01_JPRB, 0.17414E+01_JPRB, 0.19275E+01_JPRB, 0.19510E+01_JPRB, 0.21391E+01_JPRB, 0.22044E+01_JPRB, 0.22740E+01_JPRB ,&
&  0.24219E+01_JPRB, 0.24211E+01_JPRB, 0.24531E+01_JPRB, 0.25788E+01_JPRB, 0.28415E+01_JPRB, 0.29731E+01_JPRB, 0.31192E+01_JPRB ,&
&  0.31986E+01_JPRB, 0.32694E+01_JPRB, 0.34164E+01_JPRB, 0.35935E+01_JPRB, 0.37811E+01_JPRB, 0.39687E+01_JPRB, 0.42025E+01_JPRB ,&
&  0.43788E+01_JPRB, 0.45280E+01_JPRB, 0.47896E+01_JPRB, 0.43874E+01_JPRB, 0.41039E+01_JPRB, 0.43885E+01_JPRB, 0.40254E+01_JPRB ,&
&  0.27223E+01_JPRB, 0.14956E+01_JPRB, 0.14686E+01_JPRB, 0.16499E+01_JPRB, 0.13575E+01_JPRB, 0.12360E+01_JPRB, 0.11399E+01_JPRB ,&
&  0.99384E+00_JPRB, 0.90418E+00_JPRB, 0.96971E+00_JPRB, 0.10535E+01_JPRB, 0.11521E+01_JPRB, 0.12367E+01_JPRB, 0.12546E+01_JPRB ,&
&  0.11802E+01_JPRB, 0.11432E+01_JPRB, 0.10880E+01_JPRB, 0.10746E+01_JPRB, 0.10929E+01_JPRB, 0.11194E+01_JPRB, 0.11205E+01_JPRB ,&
&  0.11206E+01_JPRB, 0.11004E+01_JPRB, 0.11318E+01_JPRB, 0.11628E+01_JPRB, 0.11906E+01_JPRB, 0.12079E+01_JPRB, 0.13396E+01_JPRB ,&
&  0.15673E+01_JPRB, 0.17747E+01_JPRB, 0.20203E+01_JPRB, 0.19624E+01_JPRB, 0.17685E+01_JPRB, 0.12836E+01_JPRB, 0.88149E+00_JPRB ,&
&  0.90516E+00_JPRB, 0.92926E+00_JPRB, 0.93610E+00_JPRB, 0.95149E+00_JPRB, 0.94437E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.84240E+00_JPRB, 0.83533E+00_JPRB, 0.82875E+00_JPRB, 0.83511E+00_JPRB, 0.84562E+00_JPRB, 0.89760E+00_JPRB, 0.98197E+00_JPRB ,&
&  0.10661E+01_JPRB, 0.11200E+01_JPRB, 0.11680E+01_JPRB, 0.11973E+01_JPRB, 0.12070E+01_JPRB, 0.11776E+01_JPRB, 0.11807E+01_JPRB ,&
&  0.11849E+01_JPRB, 0.11910E+01_JPRB, 0.12227E+01_JPRB, 0.12695E+01_JPRB, 0.13140E+01_JPRB, 0.13630E+01_JPRB, 0.14200E+01_JPRB ,&
&  0.14911E+01_JPRB, 0.15930E+01_JPRB, 0.17021E+01_JPRB, 0.18600E+01_JPRB, 0.19918E+01_JPRB, 0.20873E+01_JPRB, 0.20777E+01_JPRB ,&
&  0.97839E+00_JPRB, 0.11384E+01_JPRB, 0.17470E+01_JPRB, 0.17561E+01_JPRB, 0.19231E+01_JPRB, 0.21467E+01_JPRB, 0.24384E+01_JPRB ,&
&  0.24037E+01_JPRB, 0.21758E+01_JPRB, 0.22503E+01_JPRB, 0.26438E+01_JPRB, 0.26781E+01_JPRB, 0.28025E+01_JPRB, 0.29132E+01_JPRB ,&
&  0.29541E+01_JPRB, 0.29593E+01_JPRB, 0.30093E+01_JPRB, 0.31542E+01_JPRB, 0.33583E+01_JPRB, 0.35794E+01_JPRB, 0.38417E+01_JPRB ,&
&  0.41147E+01_JPRB, 0.44193E+01_JPRB, 0.47217E+01_JPRB, 0.40231E+01_JPRB, 0.43182E+01_JPRB, 0.47323E+01_JPRB, 0.51077E+01_JPRB ,&
&  0.38614E+01_JPRB, 0.22291E+01_JPRB, 0.17868E+01_JPRB, 0.18664E+01_JPRB, 0.15223E+01_JPRB, 0.14596E+01_JPRB, 0.12424E+01_JPRB ,&
&  0.11873E+01_JPRB, 0.11013E+01_JPRB, 0.11460E+01_JPRB, 0.12279E+01_JPRB, 0.13171E+01_JPRB, 0.13853E+01_JPRB, 0.13922E+01_JPRB ,&
&  0.13139E+01_JPRB, 0.12608E+01_JPRB, 0.12305E+01_JPRB, 0.12243E+01_JPRB, 0.12323E+01_JPRB, 0.12383E+01_JPRB, 0.12267E+01_JPRB ,&
&  0.12102E+01_JPRB, 0.12047E+01_JPRB, 0.12425E+01_JPRB, 0.12663E+01_JPRB, 0.12866E+01_JPRB, 0.13045E+01_JPRB, 0.13197E+01_JPRB ,&
&  0.12621E+01_JPRB, 0.12211E+01_JPRB, 0.12952E+01_JPRB, 0.11877E+01_JPRB, 0.10498E+01_JPRB, 0.90326E+00_JPRB, 0.77069E+00_JPRB ,&
&  0.79355E+00_JPRB, 0.80573E+00_JPRB, 0.80628E+00_JPRB, 0.81748E+00_JPRB, 0.83682E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.75290E+00_JPRB, 0.75983E+00_JPRB, 0.75367E+00_JPRB, 0.77374E+00_JPRB, 0.77701E+00_JPRB, 0.85045E+00_JPRB, 0.92260E+00_JPRB ,&
&  0.10376E+01_JPRB, 0.11245E+01_JPRB, 0.11827E+01_JPRB, 0.12043E+01_JPRB, 0.11900E+01_JPRB, 0.11447E+01_JPRB, 0.11019E+01_JPRB ,&
&  0.11084E+01_JPRB, 0.11163E+01_JPRB, 0.11327E+01_JPRB, 0.11600E+01_JPRB, 0.11906E+01_JPRB, 0.12351E+01_JPRB, 0.12817E+01_JPRB ,&
&  0.13460E+01_JPRB, 0.14434E+01_JPRB, 0.15681E+01_JPRB, 0.17489E+01_JPRB, 0.19859E+01_JPRB, 0.23086E+01_JPRB, 0.26409E+01_JPRB ,&
&  0.20916E+01_JPRB, 0.10557E+01_JPRB, 0.15579E+01_JPRB, 0.15793E+01_JPRB, 0.15014E+01_JPRB, 0.18014E+01_JPRB, 0.21886E+01_JPRB ,&
&  0.21361E+01_JPRB, 0.20491E+01_JPRB, 0.20521E+01_JPRB, 0.24291E+01_JPRB, 0.24884E+01_JPRB, 0.25876E+01_JPRB, 0.26682E+01_JPRB ,&
&  0.27273E+01_JPRB, 0.27394E+01_JPRB, 0.27474E+01_JPRB, 0.28679E+01_JPRB, 0.30672E+01_JPRB, 0.33184E+01_JPRB, 0.35826E+01_JPRB ,&
&  0.38846E+01_JPRB, 0.42216E+01_JPRB, 0.46301E+01_JPRB, 0.40067E+01_JPRB, 0.45104E+01_JPRB, 0.45606E+01_JPRB, 0.40142E+01_JPRB ,&
&  0.37522E+01_JPRB, 0.25363E+01_JPRB, 0.19400E+01_JPRB, 0.20281E+01_JPRB, 0.17585E+01_JPRB, 0.13643E+01_JPRB, 0.13530E+01_JPRB ,&
&  0.12822E+01_JPRB, 0.12800E+01_JPRB, 0.12702E+01_JPRB, 0.13616E+01_JPRB, 0.14166E+01_JPRB, 0.14550E+01_JPRB, 0.14370E+01_JPRB ,&
&  0.14184E+01_JPRB, 0.13665E+01_JPRB, 0.13428E+01_JPRB, 0.13408E+01_JPRB, 0.13399E+01_JPRB, 0.13622E+01_JPRB, 0.13590E+01_JPRB ,&
&  0.13469E+01_JPRB, 0.13581E+01_JPRB, 0.14028E+01_JPRB, 0.14524E+01_JPRB, 0.14676E+01_JPRB, 0.13833E+01_JPRB, 0.13088E+01_JPRB ,&
&  0.12584E+01_JPRB, 0.12080E+01_JPRB, 0.10879E+01_JPRB, 0.95551E+00_JPRB, 0.90406E+00_JPRB, 0.82161E+00_JPRB, 0.75490E+00_JPRB ,&
&  0.76420E+00_JPRB, 0.76211E+00_JPRB, 0.74996E+00_JPRB, 0.75119E+00_JPRB, 0.75482E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.80608E+00_JPRB, 0.80024E+00_JPRB, 0.76890E+00_JPRB, 0.75204E+00_JPRB, 0.74948E+00_JPRB, 0.78083E+00_JPRB, 0.82423E+00_JPRB ,&
&  0.89191E+00_JPRB, 0.98577E+00_JPRB, 0.10657E+01_JPRB, 0.11092E+01_JPRB, 0.11060E+01_JPRB, 0.11156E+01_JPRB, 0.10807E+01_JPRB ,&
&  0.10431E+01_JPRB, 0.10410E+01_JPRB, 0.10548E+01_JPRB, 0.10631E+01_JPRB, 0.10800E+01_JPRB, 0.11044E+01_JPRB, 0.11317E+01_JPRB ,&
&  0.11466E+01_JPRB, 0.11631E+01_JPRB, 0.11867E+01_JPRB, 0.12891E+01_JPRB, 0.14642E+01_JPRB, 0.17380E+01_JPRB, 0.22369E+01_JPRB ,&
&  0.33052E+01_JPRB, 0.33257E+01_JPRB, 0.17085E+01_JPRB, 0.18286E+01_JPRB, 0.16162E+01_JPRB, 0.17487E+01_JPRB, 0.17477E+01_JPRB ,&
&  0.19127E+01_JPRB, 0.18217E+01_JPRB, 0.18191E+01_JPRB, 0.21195E+01_JPRB, 0.21999E+01_JPRB, 0.23009E+01_JPRB, 0.23797E+01_JPRB ,&
&  0.24887E+01_JPRB, 0.25416E+01_JPRB, 0.25140E+01_JPRB, 0.25495E+01_JPRB, 0.27244E+01_JPRB, 0.29633E+01_JPRB, 0.32399E+01_JPRB ,&
&  0.35718E+01_JPRB, 0.38829E+01_JPRB, 0.43006E+01_JPRB, 0.40026E+01_JPRB, 0.44677E+01_JPRB, 0.41281E+01_JPRB, 0.33108E+01_JPRB ,&
&  0.27846E+01_JPRB, 0.24606E+01_JPRB, 0.22699E+01_JPRB, 0.22079E+01_JPRB, 0.19750E+01_JPRB, 0.15185E+01_JPRB, 0.14824E+01_JPRB ,&
&  0.14080E+01_JPRB, 0.13711E+01_JPRB, 0.13573E+01_JPRB, 0.13843E+01_JPRB, 0.14415E+01_JPRB, 0.14629E+01_JPRB, 0.14732E+01_JPRB ,&
&  0.14565E+01_JPRB, 0.14196E+01_JPRB, 0.14006E+01_JPRB, 0.13946E+01_JPRB, 0.13950E+01_JPRB, 0.14193E+01_JPRB, 0.14289E+01_JPRB ,&
&  0.14386E+01_JPRB, 0.14646E+01_JPRB, 0.15098E+01_JPRB, 0.15558E+01_JPRB, 0.15211E+01_JPRB, 0.14233E+01_JPRB, 0.13181E+01_JPRB ,&
&  0.12810E+01_JPRB, 0.12793E+01_JPRB, 0.11887E+01_JPRB, 0.11166E+01_JPRB, 0.10410E+01_JPRB, 0.10089E+01_JPRB, 0.96942E+00_JPRB ,&
&  0.95786E+00_JPRB, 0.95193E+00_JPRB, 0.91289E+00_JPRB, 0.80502E+00_JPRB, 0.80482E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.91635E+00_JPRB, 0.87261E+00_JPRB, 0.86811E+00_JPRB, 0.86461E+00_JPRB, 0.85348E+00_JPRB, 0.85539E+00_JPRB, 0.81210E+00_JPRB ,&
&  0.80295E+00_JPRB, 0.84906E+00_JPRB, 0.91667E+00_JPRB, 0.95833E+00_JPRB, 0.98474E+00_JPRB, 0.98906E+00_JPRB, 0.97201E+00_JPRB ,&
&  0.95399E+00_JPRB, 0.97052E+00_JPRB, 0.98876E+00_JPRB, 0.99955E+00_JPRB, 0.10184E+01_JPRB, 0.10349E+01_JPRB, 0.10586E+01_JPRB ,&
&  0.10777E+01_JPRB, 0.10535E+01_JPRB, 0.10211E+01_JPRB, 0.10141E+01_JPRB, 0.10825E+01_JPRB, 0.11950E+01_JPRB, 0.13600E+01_JPRB ,&
&  0.20738E+01_JPRB, 0.43204E+01_JPRB, 0.21958E+01_JPRB, 0.20429E+01_JPRB, 0.19404E+01_JPRB, 0.18288E+01_JPRB, 0.17320E+01_JPRB ,&
&  0.17968E+01_JPRB, 0.15885E+01_JPRB, 0.15369E+01_JPRB, 0.17411E+01_JPRB, 0.18242E+01_JPRB, 0.19000E+01_JPRB, 0.19964E+01_JPRB ,&
&  0.20835E+01_JPRB, 0.21475E+01_JPRB, 0.21812E+01_JPRB, 0.21544E+01_JPRB, 0.22777E+01_JPRB, 0.24600E+01_JPRB, 0.27120E+01_JPRB ,&
&  0.30005E+01_JPRB, 0.33704E+01_JPRB, 0.38631E+01_JPRB, 0.38428E+01_JPRB, 0.42090E+01_JPRB, 0.39474E+01_JPRB, 0.32187E+01_JPRB ,&
&  0.27973E+01_JPRB, 0.26875E+01_JPRB, 0.24337E+01_JPRB, 0.22877E+01_JPRB, 0.20831E+01_JPRB, 0.17572E+01_JPRB, 0.17271E+01_JPRB ,&
&  0.15405E+01_JPRB, 0.14618E+01_JPRB, 0.14488E+01_JPRB, 0.14404E+01_JPRB, 0.14631E+01_JPRB, 0.14700E+01_JPRB, 0.14759E+01_JPRB ,&
&  0.14629E+01_JPRB, 0.14322E+01_JPRB, 0.14148E+01_JPRB, 0.14215E+01_JPRB, 0.14252E+01_JPRB, 0.14230E+01_JPRB, 0.14383E+01_JPRB ,&
&  0.14525E+01_JPRB, 0.14897E+01_JPRB, 0.15501E+01_JPRB, 0.15512E+01_JPRB, 0.14992E+01_JPRB, 0.13971E+01_JPRB, 0.12946E+01_JPRB ,&
&  0.12563E+01_JPRB, 0.12590E+01_JPRB, 0.12190E+01_JPRB, 0.11637E+01_JPRB, 0.11554E+01_JPRB, 0.11205E+01_JPRB, 0.11423E+01_JPRB ,&
&  0.12181E+01_JPRB, 0.13208E+01_JPRB, 0.13992E+01_JPRB, 0.94731E+00_JPRB, 0.94426E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.97916E+00_JPRB, 0.93261E+00_JPRB, 0.93157E+00_JPRB, 0.96044E+00_JPRB, 0.96222E+00_JPRB, 0.94222E+00_JPRB, 0.89584E+00_JPRB ,&
&  0.83454E+00_JPRB, 0.82933E+00_JPRB, 0.84900E+00_JPRB, 0.87254E+00_JPRB, 0.89076E+00_JPRB, 0.90706E+00_JPRB, 0.90273E+00_JPRB ,&
&  0.88504E+00_JPRB, 0.89628E+00_JPRB, 0.91295E+00_JPRB, 0.94104E+00_JPRB, 0.96327E+00_JPRB, 0.98504E+00_JPRB, 0.10141E+01_JPRB ,&
&  0.10205E+01_JPRB, 0.10110E+01_JPRB, 0.99238E+00_JPRB, 0.97883E+00_JPRB, 0.98012E+00_JPRB, 0.10119E+01_JPRB, 0.10849E+01_JPRB ,&
&  0.14396E+01_JPRB, 0.37459E+01_JPRB, 0.35906E+01_JPRB, 0.25878E+01_JPRB, 0.20344E+01_JPRB, 0.17608E+01_JPRB, 0.16531E+01_JPRB ,&
&  0.18036E+01_JPRB, 0.18279E+01_JPRB, 0.18186E+01_JPRB, 0.16381E+01_JPRB, 0.15819E+01_JPRB, 0.16216E+01_JPRB, 0.16906E+01_JPRB ,&
&  0.17073E+01_JPRB, 0.17267E+01_JPRB, 0.17475E+01_JPRB, 0.17399E+01_JPRB, 0.17699E+01_JPRB, 0.19227E+01_JPRB, 0.21164E+01_JPRB ,&
&  0.23124E+01_JPRB, 0.26558E+01_JPRB, 0.32596E+01_JPRB, 0.38699E+01_JPRB, 0.39272E+01_JPRB, 0.40317E+01_JPRB, 0.38168E+01_JPRB ,&
&  0.39743E+01_JPRB, 0.34752E+01_JPRB, 0.27599E+01_JPRB, 0.22623E+01_JPRB, 0.19587E+01_JPRB, 0.20168E+01_JPRB, 0.19028E+01_JPRB ,&
&  0.16437E+01_JPRB, 0.15164E+01_JPRB, 0.14678E+01_JPRB, 0.14619E+01_JPRB, 0.14686E+01_JPRB, 0.14715E+01_JPRB, 0.14428E+01_JPRB ,&
&  0.14203E+01_JPRB, 0.14089E+01_JPRB, 0.14078E+01_JPRB, 0.14202E+01_JPRB, 0.14206E+01_JPRB, 0.14194E+01_JPRB, 0.14316E+01_JPRB ,&
&  0.14289E+01_JPRB, 0.14476E+01_JPRB, 0.14574E+01_JPRB, 0.14983E+01_JPRB, 0.14519E+01_JPRB, 0.13438E+01_JPRB, 0.12472E+01_JPRB ,&
&  0.12086E+01_JPRB, 0.12257E+01_JPRB, 0.12280E+01_JPRB, 0.12083E+01_JPRB, 0.11883E+01_JPRB, 0.11631E+01_JPRB, 0.11304E+01_JPRB ,&
&  0.11360E+01_JPRB, 0.11411E+01_JPRB, 0.10858E+01_JPRB, 0.10160E+01_JPRB, 0.10218E+01_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.10482E+01_JPRB, 0.10163E+01_JPRB, 0.99313E+00_JPRB, 0.10007E+01_JPRB, 0.10084E+01_JPRB, 0.99462E+00_JPRB, 0.96725E+00_JPRB ,&
&  0.93406E+00_JPRB, 0.89979E+00_JPRB, 0.90292E+00_JPRB, 0.89231E+00_JPRB, 0.88455E+00_JPRB, 0.88057E+00_JPRB, 0.88109E+00_JPRB ,&
&  0.88757E+00_JPRB, 0.89963E+00_JPRB, 0.89049E+00_JPRB, 0.89294E+00_JPRB, 0.91538E+00_JPRB, 0.94034E+00_JPRB, 0.95052E+00_JPRB ,&
&  0.95854E+00_JPRB, 0.96675E+00_JPRB, 0.96260E+00_JPRB, 0.95850E+00_JPRB, 0.94482E+00_JPRB, 0.95311E+00_JPRB, 0.98712E+00_JPRB ,&
&  0.12004E+01_JPRB, 0.29738E+01_JPRB, 0.21200E+01_JPRB, 0.26041E+01_JPRB, 0.19971E+01_JPRB, 0.17292E+01_JPRB, 0.14513E+01_JPRB ,&
&  0.15817E+01_JPRB, 0.19459E+01_JPRB, 0.17101E+01_JPRB, 0.15622E+01_JPRB, 0.14682E+01_JPRB, 0.14550E+01_JPRB, 0.14621E+01_JPRB ,&
&  0.14698E+01_JPRB, 0.14922E+01_JPRB, 0.14847E+01_JPRB, 0.15006E+01_JPRB, 0.14868E+01_JPRB, 0.15595E+01_JPRB, 0.16822E+01_JPRB ,&
&  0.17931E+01_JPRB, 0.19745E+01_JPRB, 0.23907E+01_JPRB, 0.29181E+01_JPRB, 0.39285E+01_JPRB, 0.44726E+01_JPRB, 0.47383E+01_JPRB ,&
&  0.49445E+01_JPRB, 0.42337E+01_JPRB, 0.33369E+01_JPRB, 0.25818E+01_JPRB, 0.21318E+01_JPRB, 0.20562E+01_JPRB, 0.20253E+01_JPRB ,&
&  0.18020E+01_JPRB, 0.16385E+01_JPRB, 0.15440E+01_JPRB, 0.14995E+01_JPRB, 0.15036E+01_JPRB, 0.15037E+01_JPRB, 0.14484E+01_JPRB ,&
&  0.14049E+01_JPRB, 0.14001E+01_JPRB, 0.14110E+01_JPRB, 0.14245E+01_JPRB, 0.14149E+01_JPRB, 0.13885E+01_JPRB, 0.13876E+01_JPRB ,&
&  0.13950E+01_JPRB, 0.14016E+01_JPRB, 0.13997E+01_JPRB, 0.14731E+01_JPRB, 0.14234E+01_JPRB, 0.13372E+01_JPRB, 0.12502E+01_JPRB ,&
&  0.12252E+01_JPRB, 0.12188E+01_JPRB, 0.12093E+01_JPRB, 0.11856E+01_JPRB, 0.12063E+01_JPRB, 0.12297E+01_JPRB, 0.11560E+01_JPRB ,&
&  0.11402E+01_JPRB, 0.11280E+01_JPRB, 0.11076E+01_JPRB, 0.10669E+01_JPRB, 0.10718E+01_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.10654E+01_JPRB, 0.10424E+01_JPRB, 0.10439E+01_JPRB, 0.10436E+01_JPRB, 0.10350E+01_JPRB, 0.10103E+01_JPRB, 0.99393E+00_JPRB ,&
&  0.99742E+00_JPRB, 0.98232E+00_JPRB, 0.96610E+00_JPRB, 0.93710E+00_JPRB, 0.92072E+00_JPRB, 0.90310E+00_JPRB, 0.89511E+00_JPRB ,&
&  0.90089E+00_JPRB, 0.90527E+00_JPRB, 0.89750E+00_JPRB, 0.89611E+00_JPRB, 0.88956E+00_JPRB, 0.90086E+00_JPRB, 0.91405E+00_JPRB ,&
&  0.92606E+00_JPRB, 0.93346E+00_JPRB, 0.94575E+00_JPRB, 0.94864E+00_JPRB, 0.93880E+00_JPRB, 0.92722E+00_JPRB, 0.94321E+00_JPRB ,&
&  0.11167E+01_JPRB, 0.29811E+01_JPRB, 0.18057E+01_JPRB, 0.25912E+01_JPRB, 0.19662E+01_JPRB, 0.17269E+01_JPRB, 0.14353E+01_JPRB ,&
&  0.15429E+01_JPRB, 0.18432E+01_JPRB, 0.16693E+01_JPRB, 0.14753E+01_JPRB, 0.14149E+01_JPRB, 0.13773E+01_JPRB, 0.13372E+01_JPRB ,&
&  0.13311E+01_JPRB, 0.13608E+01_JPRB, 0.13672E+01_JPRB, 0.13847E+01_JPRB, 0.13939E+01_JPRB, 0.13802E+01_JPRB, 0.14197E+01_JPRB ,&
&  0.14626E+01_JPRB, 0.15350E+01_JPRB, 0.17917E+01_JPRB, 0.26596E+01_JPRB, 0.34653E+01_JPRB, 0.47383E+01_JPRB, 0.60972E+01_JPRB ,&
&  0.59279E+01_JPRB, 0.50664E+01_JPRB, 0.36108E+01_JPRB, 0.29266E+01_JPRB, 0.24390E+01_JPRB, 0.22256E+01_JPRB, 0.21061E+01_JPRB ,&
&  0.19788E+01_JPRB, 0.18628E+01_JPRB, 0.17018E+01_JPRB, 0.16212E+01_JPRB, 0.15981E+01_JPRB, 0.15769E+01_JPRB, 0.15348E+01_JPRB ,&
&  0.14859E+01_JPRB, 0.14772E+01_JPRB, 0.14822E+01_JPRB, 0.14777E+01_JPRB, 0.14608E+01_JPRB, 0.14183E+01_JPRB, 0.13951E+01_JPRB ,&
&  0.14041E+01_JPRB, 0.13994E+01_JPRB, 0.14080E+01_JPRB, 0.15135E+01_JPRB, 0.14649E+01_JPRB, 0.13584E+01_JPRB, 0.12819E+01_JPRB ,&
&  0.12716E+01_JPRB, 0.12352E+01_JPRB, 0.12198E+01_JPRB, 0.12151E+01_JPRB, 0.12281E+01_JPRB, 0.13013E+01_JPRB, 0.12227E+01_JPRB ,&
&  0.11709E+01_JPRB, 0.11278E+01_JPRB, 0.11006E+01_JPRB, 0.10817E+01_JPRB, 0.10769E+01_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.11069E+01_JPRB, 0.10895E+01_JPRB, 0.10874E+01_JPRB, 0.11002E+01_JPRB, 0.10816E+01_JPRB, 0.10516E+01_JPRB, 0.10240E+01_JPRB ,&
&  0.10333E+01_JPRB, 0.10251E+01_JPRB, 0.10098E+01_JPRB, 0.97144E+00_JPRB, 0.95204E+00_JPRB, 0.93748E+00_JPRB, 0.93365E+00_JPRB ,&
&  0.93853E+00_JPRB, 0.92914E+00_JPRB, 0.91158E+00_JPRB, 0.90173E+00_JPRB, 0.89874E+00_JPRB, 0.89577E+00_JPRB, 0.90980E+00_JPRB ,&
&  0.90226E+00_JPRB, 0.90801E+00_JPRB, 0.92898E+00_JPRB, 0.93536E+00_JPRB, 0.93945E+00_JPRB, 0.92045E+00_JPRB, 0.92381E+00_JPRB ,&
&  0.10231E+01_JPRB, 0.12241E+01_JPRB, 0.20183E+01_JPRB, 0.23725E+01_JPRB, 0.19690E+01_JPRB, 0.17606E+01_JPRB, 0.16328E+01_JPRB ,&
&  0.16692E+01_JPRB, 0.17046E+01_JPRB, 0.15914E+01_JPRB, 0.14661E+01_JPRB, 0.13979E+01_JPRB, 0.13285E+01_JPRB, 0.12743E+01_JPRB ,&
&  0.12312E+01_JPRB, 0.12168E+01_JPRB, 0.12369E+01_JPRB, 0.12783E+01_JPRB, 0.13047E+01_JPRB, 0.13024E+01_JPRB, 0.12724E+01_JPRB ,&
&  0.12593E+01_JPRB, 0.12642E+01_JPRB, 0.13741E+01_JPRB, 0.19610E+01_JPRB, 0.33097E+01_JPRB, 0.48958E+01_JPRB, 0.59958E+01_JPRB ,&
&  0.61371E+01_JPRB, 0.50676E+01_JPRB, 0.36731E+01_JPRB, 0.30959E+01_JPRB, 0.26901E+01_JPRB, 0.23823E+01_JPRB, 0.21301E+01_JPRB ,&
&  0.19855E+01_JPRB, 0.19351E+01_JPRB, 0.18609E+01_JPRB, 0.17579E+01_JPRB, 0.16623E+01_JPRB, 0.16068E+01_JPRB, 0.15652E+01_JPRB ,&
&  0.15328E+01_JPRB, 0.15409E+01_JPRB, 0.15600E+01_JPRB, 0.15265E+01_JPRB, 0.14845E+01_JPRB, 0.14374E+01_JPRB, 0.14131E+01_JPRB ,&
&  0.14219E+01_JPRB, 0.14354E+01_JPRB, 0.14519E+01_JPRB, 0.15155E+01_JPRB, 0.14806E+01_JPRB, 0.14216E+01_JPRB, 0.13512E+01_JPRB ,&
&  0.12888E+01_JPRB, 0.12385E+01_JPRB, 0.12133E+01_JPRB, 0.12467E+01_JPRB, 0.12379E+01_JPRB, 0.13260E+01_JPRB, 0.12907E+01_JPRB ,&
&  0.12198E+01_JPRB, 0.11697E+01_JPRB, 0.11411E+01_JPRB, 0.11263E+01_JPRB, 0.11234E+01_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.11526E+01_JPRB, 0.11437E+01_JPRB, 0.11516E+01_JPRB, 0.11617E+01_JPRB, 0.11494E+01_JPRB, 0.11160E+01_JPRB, 0.10824E+01_JPRB ,&
&  0.10593E+01_JPRB, 0.10362E+01_JPRB, 0.10089E+01_JPRB, 0.97299E+00_JPRB, 0.95680E+00_JPRB, 0.95933E+00_JPRB, 0.96652E+00_JPRB ,&
&  0.97296E+00_JPRB, 0.96122E+00_JPRB, 0.94080E+00_JPRB, 0.91870E+00_JPRB, 0.91287E+00_JPRB, 0.91226E+00_JPRB, 0.92135E+00_JPRB ,&
&  0.91467E+00_JPRB, 0.91205E+00_JPRB, 0.91718E+00_JPRB, 0.92324E+00_JPRB, 0.93001E+00_JPRB, 0.91915E+00_JPRB, 0.90996E+00_JPRB ,&
&  0.95961E+00_JPRB, 0.14750E+01_JPRB, 0.18211E+01_JPRB, 0.19569E+01_JPRB, 0.18906E+01_JPRB, 0.17407E+01_JPRB, 0.16738E+01_JPRB ,&
&  0.15730E+01_JPRB, 0.15615E+01_JPRB, 0.15386E+01_JPRB, 0.14747E+01_JPRB, 0.13973E+01_JPRB, 0.13280E+01_JPRB, 0.12734E+01_JPRB ,&
&  0.12230E+01_JPRB, 0.11880E+01_JPRB, 0.11688E+01_JPRB, 0.11817E+01_JPRB, 0.12076E+01_JPRB, 0.12331E+01_JPRB, 0.12171E+01_JPRB ,&
&  0.11835E+01_JPRB, 0.11529E+01_JPRB, 0.11653E+01_JPRB, 0.14598E+01_JPRB, 0.25318E+01_JPRB, 0.32222E+01_JPRB, 0.47774E+01_JPRB ,&
&  0.53010E+01_JPRB, 0.39577E+01_JPRB, 0.31375E+01_JPRB, 0.27542E+01_JPRB, 0.24793E+01_JPRB, 0.22040E+01_JPRB, 0.19962E+01_JPRB ,&
&  0.18979E+01_JPRB, 0.18417E+01_JPRB, 0.18290E+01_JPRB, 0.17699E+01_JPRB, 0.16482E+01_JPRB, 0.15340E+01_JPRB, 0.14775E+01_JPRB ,&
&  0.14469E+01_JPRB, 0.14239E+01_JPRB, 0.14251E+01_JPRB, 0.14341E+01_JPRB, 0.14231E+01_JPRB, 0.13772E+01_JPRB, 0.13536E+01_JPRB ,&
&  0.13554E+01_JPRB, 0.13864E+01_JPRB, 0.14321E+01_JPRB, 0.14506E+01_JPRB, 0.14611E+01_JPRB, 0.14697E+01_JPRB, 0.14006E+01_JPRB ,&
&  0.12984E+01_JPRB, 0.12331E+01_JPRB, 0.12162E+01_JPRB, 0.12257E+01_JPRB, 0.12557E+01_JPRB, 0.13570E+01_JPRB, 0.13169E+01_JPRB ,&
&  0.12534E+01_JPRB, 0.11995E+01_JPRB, 0.11729E+01_JPRB, 0.11547E+01_JPRB, 0.11689E+01_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.11794E+01_JPRB, 0.11681E+01_JPRB, 0.11754E+01_JPRB, 0.11801E+01_JPRB, 0.11690E+01_JPRB, 0.11350E+01_JPRB, 0.11058E+01_JPRB ,&
&  0.10881E+01_JPRB, 0.10463E+01_JPRB, 0.10040E+01_JPRB, 0.97278E+00_JPRB, 0.97056E+00_JPRB, 0.97717E+00_JPRB, 0.98124E+00_JPRB ,&
&  0.97354E+00_JPRB, 0.97079E+00_JPRB, 0.95422E+00_JPRB, 0.93800E+00_JPRB, 0.92931E+00_JPRB, 0.92146E+00_JPRB, 0.90655E+00_JPRB ,&
&  0.90634E+00_JPRB, 0.90490E+00_JPRB, 0.90907E+00_JPRB, 0.91758E+00_JPRB, 0.91530E+00_JPRB, 0.91825E+00_JPRB, 0.91375E+00_JPRB ,&
&  0.91642E+00_JPRB, 0.11157E+01_JPRB, 0.12907E+01_JPRB, 0.15743E+01_JPRB, 0.16790E+01_JPRB, 0.16410E+01_JPRB, 0.15764E+01_JPRB ,&
&  0.14955E+01_JPRB, 0.14520E+01_JPRB, 0.14395E+01_JPRB, 0.14022E+01_JPRB, 0.13440E+01_JPRB, 0.12910E+01_JPRB, 0.12431E+01_JPRB ,&
&  0.12070E+01_JPRB, 0.11796E+01_JPRB, 0.11720E+01_JPRB, 0.11614E+01_JPRB, 0.11539E+01_JPRB, 0.11665E+01_JPRB, 0.11817E+01_JPRB ,&
&  0.11424E+01_JPRB, 0.11073E+01_JPRB, 0.10818E+01_JPRB, 0.11409E+01_JPRB, 0.16091E+01_JPRB, 0.22261E+01_JPRB, 0.30588E+01_JPRB ,&
&  0.28842E+01_JPRB, 0.25303E+01_JPRB, 0.23052E+01_JPRB, 0.21350E+01_JPRB, 0.20139E+01_JPRB, 0.18513E+01_JPRB, 0.17082E+01_JPRB ,&
&  0.15809E+01_JPRB, 0.15224E+01_JPRB, 0.14758E+01_JPRB, 0.14507E+01_JPRB, 0.14154E+01_JPRB, 0.13899E+01_JPRB, 0.13304E+01_JPRB ,&
&  0.12928E+01_JPRB, 0.12678E+01_JPRB, 0.12625E+01_JPRB, 0.12815E+01_JPRB, 0.13135E+01_JPRB, 0.13202E+01_JPRB, 0.13213E+01_JPRB ,&
&  0.13361E+01_JPRB, 0.13493E+01_JPRB, 0.13915E+01_JPRB, 0.14417E+01_JPRB, 0.14206E+01_JPRB, 0.13799E+01_JPRB, 0.13481E+01_JPRB ,&
&  0.13195E+01_JPRB, 0.12682E+01_JPRB, 0.12245E+01_JPRB, 0.12347E+01_JPRB, 0.12433E+01_JPRB, 0.13323E+01_JPRB, 0.13173E+01_JPRB ,&
&  0.12676E+01_JPRB, 0.12040E+01_JPRB, 0.11765E+01_JPRB, 0.11604E+01_JPRB, 0.11795E+01_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.11991E+01_JPRB, 0.12042E+01_JPRB, 0.11990E+01_JPRB, 0.11931E+01_JPRB, 0.11798E+01_JPRB, 0.11488E+01_JPRB, 0.11148E+01_JPRB ,&
&  0.10890E+01_JPRB, 0.10585E+01_JPRB, 0.10145E+01_JPRB, 0.98857E+00_JPRB, 0.97954E+00_JPRB, 0.97766E+00_JPRB, 0.97263E+00_JPRB ,&
&  0.96734E+00_JPRB, 0.97002E+00_JPRB, 0.96154E+00_JPRB, 0.95919E+00_JPRB, 0.94939E+00_JPRB, 0.94187E+00_JPRB, 0.91914E+00_JPRB ,&
&  0.90334E+00_JPRB, 0.89258E+00_JPRB, 0.88998E+00_JPRB, 0.89761E+00_JPRB, 0.90490E+00_JPRB, 0.91005E+00_JPRB, 0.91737E+00_JPRB ,&
&  0.92844E+00_JPRB, 0.88778E+00_JPRB, 0.99931E+00_JPRB, 0.11884E+01_JPRB, 0.13245E+01_JPRB, 0.14183E+01_JPRB, 0.13965E+01_JPRB ,&
&  0.13607E+01_JPRB, 0.13249E+01_JPRB, 0.13145E+01_JPRB, 0.12821E+01_JPRB, 0.12337E+01_JPRB, 0.11872E+01_JPRB, 0.11555E+01_JPRB ,&
&  0.11460E+01_JPRB, 0.11344E+01_JPRB, 0.11254E+01_JPRB, 0.11169E+01_JPRB, 0.11117E+01_JPRB, 0.11126E+01_JPRB, 0.11273E+01_JPRB ,&
&  0.11233E+01_JPRB, 0.10910E+01_JPRB, 0.10543E+01_JPRB, 0.10322E+01_JPRB, 0.11045E+01_JPRB, 0.13164E+01_JPRB, 0.16909E+01_JPRB ,&
&  0.18921E+01_JPRB, 0.17834E+01_JPRB, 0.16968E+01_JPRB, 0.16206E+01_JPRB, 0.15571E+01_JPRB, 0.15141E+01_JPRB, 0.14689E+01_JPRB ,&
&  0.14342E+01_JPRB, 0.14084E+01_JPRB, 0.13928E+01_JPRB, 0.13738E+01_JPRB, 0.13523E+01_JPRB, 0.13296E+01_JPRB, 0.12996E+01_JPRB ,&
&  0.12739E+01_JPRB, 0.12601E+01_JPRB, 0.12464E+01_JPRB, 0.12557E+01_JPRB, 0.12677E+01_JPRB, 0.12774E+01_JPRB, 0.12873E+01_JPRB ,&
&  0.13055E+01_JPRB, 0.13423E+01_JPRB, 0.13680E+01_JPRB, 0.13912E+01_JPRB, 0.13769E+01_JPRB, 0.13439E+01_JPRB, 0.13225E+01_JPRB ,&
&  0.12951E+01_JPRB, 0.12699E+01_JPRB, 0.12060E+01_JPRB, 0.12019E+01_JPRB, 0.13021E+01_JPRB, 0.12910E+01_JPRB, 0.12754E+01_JPRB ,&
&  0.12639E+01_JPRB, 0.12313E+01_JPRB, 0.11812E+01_JPRB, 0.11317E+01_JPRB, 0.11608E+01_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.10834E+01_JPRB, 0.11181E+01_JPRB, 0.11198E+01_JPRB, 0.11197E+01_JPRB, 0.11176E+01_JPRB, 0.11032E+01_JPRB, 0.10825E+01_JPRB ,&
&  0.10600E+01_JPRB, 0.10465E+01_JPRB, 0.10272E+01_JPRB, 0.10017E+01_JPRB, 0.98426E+00_JPRB, 0.97252E+00_JPRB, 0.96157E+00_JPRB ,&
&  0.95519E+00_JPRB, 0.94871E+00_JPRB, 0.94328E+00_JPRB, 0.93699E+00_JPRB, 0.92553E+00_JPRB, 0.91586E+00_JPRB, 0.90061E+00_JPRB ,&
&  0.88998E+00_JPRB, 0.88792E+00_JPRB, 0.88797E+00_JPRB, 0.89235E+00_JPRB, 0.90082E+00_JPRB, 0.91059E+00_JPRB, 0.91744E+00_JPRB ,&
&  0.92645E+00_JPRB, 0.83185E+00_JPRB, 0.86982E+00_JPRB, 0.97388E+00_JPRB, 0.11002E+01_JPRB, 0.12129E+01_JPRB, 0.12495E+01_JPRB ,&
&  0.12475E+01_JPRB, 0.12336E+01_JPRB, 0.12088E+01_JPRB, 0.11799E+01_JPRB, 0.11509E+01_JPRB, 0.11304E+01_JPRB, 0.10947E+01_JPRB ,&
&  0.10721E+01_JPRB, 0.10629E+01_JPRB, 0.10648E+01_JPRB, 0.10627E+01_JPRB, 0.10563E+01_JPRB, 0.10508E+01_JPRB, 0.10551E+01_JPRB ,&
&  0.10569E+01_JPRB, 0.10401E+01_JPRB, 0.10169E+01_JPRB, 0.10013E+01_JPRB, 0.10138E+01_JPRB, 0.11023E+01_JPRB, 0.12578E+01_JPRB ,&
&  0.14051E+01_JPRB, 0.13959E+01_JPRB, 0.13542E+01_JPRB, 0.13083E+01_JPRB, 0.12856E+01_JPRB, 0.12772E+01_JPRB, 0.12755E+01_JPRB ,&
&  0.12664E+01_JPRB, 0.12586E+01_JPRB, 0.12710E+01_JPRB, 0.12759E+01_JPRB, 0.12818E+01_JPRB, 0.12802E+01_JPRB, 0.12766E+01_JPRB ,&
&  0.12651E+01_JPRB, 0.12496E+01_JPRB, 0.12423E+01_JPRB, 0.12419E+01_JPRB, 0.12515E+01_JPRB, 0.12555E+01_JPRB, 0.12494E+01_JPRB ,&
&  0.12407E+01_JPRB, 0.12533E+01_JPRB, 0.12943E+01_JPRB, 0.13284E+01_JPRB, 0.13170E+01_JPRB, 0.12947E+01_JPRB, 0.12736E+01_JPRB ,&
&  0.12640E+01_JPRB, 0.12452E+01_JPRB, 0.12312E+01_JPRB, 0.12580E+01_JPRB, 0.12532E+01_JPRB, 0.12372E+01_JPRB, 0.12213E+01_JPRB ,&
&  0.12089E+01_JPRB, 0.11857E+01_JPRB, 0.11618E+01_JPRB, 0.11201E+01_JPRB, 0.10245E+01_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.98883E+00_JPRB, 0.10239E+01_JPRB, 0.10440E+01_JPRB, 0.10500E+01_JPRB, 0.10587E+01_JPRB, 0.10638E+01_JPRB, 0.10621E+01_JPRB ,&
&  0.10601E+01_JPRB, 0.10547E+01_JPRB, 0.10410E+01_JPRB, 0.10193E+01_JPRB, 0.99684E+00_JPRB, 0.97194E+00_JPRB, 0.95167E+00_JPRB ,&
&  0.94365E+00_JPRB, 0.93775E+00_JPRB, 0.92225E+00_JPRB, 0.90796E+00_JPRB, 0.89469E+00_JPRB, 0.88413E+00_JPRB, 0.87938E+00_JPRB ,&
&  0.87375E+00_JPRB, 0.87119E+00_JPRB, 0.87633E+00_JPRB, 0.88348E+00_JPRB, 0.89336E+00_JPRB, 0.90447E+00_JPRB, 0.91910E+00_JPRB ,&
&  0.92510E+00_JPRB, 0.74477E+00_JPRB, 0.77590E+00_JPRB, 0.86738E+00_JPRB, 0.94180E+00_JPRB, 0.10307E+01_JPRB, 0.11106E+01_JPRB ,&
&  0.11376E+01_JPRB, 0.11591E+01_JPRB, 0.11462E+01_JPRB, 0.11258E+01_JPRB, 0.11175E+01_JPRB, 0.10942E+01_JPRB, 0.10501E+01_JPRB ,&
&  0.10163E+01_JPRB, 0.10114E+01_JPRB, 0.10214E+01_JPRB, 0.10271E+01_JPRB, 0.10226E+01_JPRB, 0.10141E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10102E+01_JPRB, 0.10103E+01_JPRB, 0.10093E+01_JPRB, 0.10053E+01_JPRB, 0.10207E+01_JPRB, 0.10644E+01_JPRB, 0.11327E+01_JPRB ,&
&  0.11717E+01_JPRB, 0.11815E+01_JPRB, 0.11713E+01_JPRB, 0.11489E+01_JPRB, 0.11379E+01_JPRB, 0.11472E+01_JPRB, 0.11541E+01_JPRB ,&
&  0.11540E+01_JPRB, 0.11532E+01_JPRB, 0.11623E+01_JPRB, 0.11703E+01_JPRB, 0.11843E+01_JPRB, 0.11984E+01_JPRB, 0.12213E+01_JPRB ,&
&  0.12388E+01_JPRB, 0.12408E+01_JPRB, 0.12388E+01_JPRB, 0.12264E+01_JPRB, 0.12042E+01_JPRB, 0.11885E+01_JPRB, 0.11820E+01_JPRB ,&
&  0.11780E+01_JPRB, 0.11860E+01_JPRB, 0.12086E+01_JPRB, 0.12326E+01_JPRB, 0.12350E+01_JPRB, 0.12316E+01_JPRB, 0.12286E+01_JPRB ,&
&  0.12296E+01_JPRB, 0.12272E+01_JPRB, 0.12288E+01_JPRB, 0.11614E+01_JPRB, 0.11972E+01_JPRB, 0.11876E+01_JPRB, 0.11600E+01_JPRB ,&
&  0.11283E+01_JPRB, 0.10926E+01_JPRB, 0.10677E+01_JPRB, 0.89686E+00_JPRB, 0.96241E+00_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.92970E+00_JPRB, 0.95022E+00_JPRB, 0.95612E+00_JPRB, 0.97111E+00_JPRB, 0.98199E+00_JPRB, 0.99014E+00_JPRB, 0.10012E+01_JPRB ,&
&  0.10083E+01_JPRB, 0.10049E+01_JPRB, 0.99809E+00_JPRB, 0.98363E+00_JPRB, 0.95938E+00_JPRB, 0.93342E+00_JPRB, 0.91195E+00_JPRB ,&
&  0.89605E+00_JPRB, 0.89223E+00_JPRB, 0.88295E+00_JPRB, 0.87131E+00_JPRB, 0.86403E+00_JPRB, 0.86000E+00_JPRB, 0.86149E+00_JPRB ,&
&  0.86890E+00_JPRB, 0.87266E+00_JPRB, 0.86739E+00_JPRB, 0.86591E+00_JPRB, 0.87458E+00_JPRB, 0.89040E+00_JPRB, 0.89761E+00_JPRB ,&
&  0.89953E+00_JPRB, 0.70357E+00_JPRB, 0.74933E+00_JPRB, 0.80399E+00_JPRB, 0.85383E+00_JPRB, 0.92492E+00_JPRB, 0.99970E+00_JPRB ,&
&  0.10478E+01_JPRB, 0.10759E+01_JPRB, 0.10751E+01_JPRB, 0.10627E+01_JPRB, 0.10421E+01_JPRB, 0.10229E+01_JPRB, 0.99986E+00_JPRB ,&
&  0.97924E+00_JPRB, 0.97158E+00_JPRB, 0.97465E+00_JPRB, 0.97868E+00_JPRB, 0.98019E+00_JPRB, 0.98401E+00_JPRB, 0.97845E+00_JPRB ,&
&  0.97712E+00_JPRB, 0.97546E+00_JPRB, 0.97595E+00_JPRB, 0.97872E+00_JPRB, 0.98192E+00_JPRB, 0.99103E+00_JPRB, 0.10092E+01_JPRB ,&
&  0.10180E+01_JPRB, 0.10280E+01_JPRB, 0.10305E+01_JPRB, 0.10239E+01_JPRB, 0.10245E+01_JPRB, 0.10335E+01_JPRB, 0.10487E+01_JPRB ,&
&  0.10690E+01_JPRB, 0.10738E+01_JPRB, 0.10816E+01_JPRB, 0.10918E+01_JPRB, 0.11073E+01_JPRB, 0.11254E+01_JPRB, 0.11469E+01_JPRB ,&
&  0.11603E+01_JPRB, 0.11615E+01_JPRB, 0.11622E+01_JPRB, 0.11536E+01_JPRB, 0.11506E+01_JPRB, 0.11305E+01_JPRB, 0.11077E+01_JPRB ,&
&  0.11029E+01_JPRB, 0.11036E+01_JPRB, 0.11076E+01_JPRB, 0.11213E+01_JPRB, 0.11275E+01_JPRB, 0.11240E+01_JPRB, 0.11275E+01_JPRB ,&
&  0.11340E+01_JPRB, 0.11360E+01_JPRB, 0.11255E+01_JPRB, 0.11167E+01_JPRB, 0.10967E+01_JPRB, 0.10795E+01_JPRB, 0.10520E+01_JPRB ,&
&  0.10197E+01_JPRB, 0.99437E+00_JPRB, 0.79228E+00_JPRB, 0.88057E+00_JPRB, 0.90243E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.91404E+00_JPRB, 0.91983E+00_JPRB, 0.92321E+00_JPRB, 0.92835E+00_JPRB, 0.93051E+00_JPRB, 0.93076E+00_JPRB, 0.93690E+00_JPRB ,&
&  0.94352E+00_JPRB, 0.94015E+00_JPRB, 0.93259E+00_JPRB, 0.92978E+00_JPRB, 0.92005E+00_JPRB, 0.90571E+00_JPRB, 0.89375E+00_JPRB ,&
&  0.87878E+00_JPRB, 0.87242E+00_JPRB, 0.86836E+00_JPRB, 0.86408E+00_JPRB, 0.85682E+00_JPRB, 0.84809E+00_JPRB, 0.85148E+00_JPRB ,&
&  0.85831E+00_JPRB, 0.86565E+00_JPRB, 0.86755E+00_JPRB, 0.87135E+00_JPRB, 0.87720E+00_JPRB, 0.88376E+00_JPRB, 0.89544E+00_JPRB ,&
&  0.78319E+00_JPRB, 0.65689E+00_JPRB, 0.71095E+00_JPRB, 0.76634E+00_JPRB, 0.81594E+00_JPRB, 0.89177E+00_JPRB, 0.94844E+00_JPRB ,&
&  0.99147E+00_JPRB, 0.10250E+01_JPRB, 0.10320E+01_JPRB, 0.10087E+01_JPRB, 0.98463E+00_JPRB, 0.97708E+00_JPRB, 0.97644E+00_JPRB ,&
&  0.97240E+00_JPRB, 0.97024E+00_JPRB, 0.97345E+00_JPRB, 0.97748E+00_JPRB, 0.98194E+00_JPRB, 0.98828E+00_JPRB, 0.99070E+00_JPRB ,&
&  0.98876E+00_JPRB, 0.97607E+00_JPRB, 0.96270E+00_JPRB, 0.94240E+00_JPRB, 0.92512E+00_JPRB, 0.92168E+00_JPRB, 0.92119E+00_JPRB ,&
&  0.91551E+00_JPRB, 0.92071E+00_JPRB, 0.93097E+00_JPRB, 0.93966E+00_JPRB, 0.95234E+00_JPRB, 0.96725E+00_JPRB, 0.98793E+00_JPRB ,&
&  0.10125E+01_JPRB, 0.10331E+01_JPRB, 0.10492E+01_JPRB, 0.10659E+01_JPRB, 0.10736E+01_JPRB, 0.10824E+01_JPRB, 0.10985E+01_JPRB ,&
&  0.11115E+01_JPRB, 0.10993E+01_JPRB, 0.10873E+01_JPRB, 0.10765E+01_JPRB, 0.10736E+01_JPRB, 0.10727E+01_JPRB, 0.10637E+01_JPRB ,&
&  0.10543E+01_JPRB, 0.10479E+01_JPRB, 0.10451E+01_JPRB, 0.10500E+01_JPRB, 0.10475E+01_JPRB, 0.10391E+01_JPRB, 0.10327E+01_JPRB ,&
&  0.10262E+01_JPRB, 0.10122E+01_JPRB, 0.99684E+00_JPRB, 0.98657E+00_JPRB, 0.97209E+00_JPRB, 0.96072E+00_JPRB, 0.94937E+00_JPRB ,&
&  0.93993E+00_JPRB, 0.92187E+00_JPRB, 0.92335E+00_JPRB, 0.91490E+00_JPRB, 0.91039E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.88372E+00_JPRB, 0.88706E+00_JPRB, 0.89126E+00_JPRB, 0.89233E+00_JPRB, 0.88953E+00_JPRB, 0.88843E+00_JPRB, 0.89169E+00_JPRB ,&
&  0.89464E+00_JPRB, 0.88827E+00_JPRB, 0.88146E+00_JPRB, 0.87614E+00_JPRB, 0.87394E+00_JPRB, 0.86494E+00_JPRB, 0.85106E+00_JPRB ,&
&  0.84105E+00_JPRB, 0.83957E+00_JPRB, 0.84448E+00_JPRB, 0.84705E+00_JPRB, 0.83819E+00_JPRB, 0.82890E+00_JPRB, 0.82582E+00_JPRB ,&
&  0.82702E+00_JPRB, 0.83319E+00_JPRB, 0.84344E+00_JPRB, 0.84928E+00_JPRB, 0.85789E+00_JPRB, 0.87195E+00_JPRB, 0.87276E+00_JPRB ,&
&  0.87054E+00_JPRB, 0.73252E+00_JPRB, 0.77176E+00_JPRB, 0.78767E+00_JPRB, 0.81909E+00_JPRB, 0.86548E+00_JPRB, 0.90477E+00_JPRB ,&
&  0.94262E+00_JPRB, 0.96395E+00_JPRB, 0.97337E+00_JPRB, 0.96660E+00_JPRB, 0.95067E+00_JPRB, 0.93584E+00_JPRB, 0.93245E+00_JPRB ,&
&  0.93263E+00_JPRB, 0.94058E+00_JPRB, 0.95481E+00_JPRB, 0.96464E+00_JPRB, 0.96826E+00_JPRB, 0.96943E+00_JPRB, 0.97216E+00_JPRB ,&
&  0.96905E+00_JPRB, 0.95251E+00_JPRB, 0.92851E+00_JPRB, 0.89886E+00_JPRB, 0.87498E+00_JPRB, 0.86340E+00_JPRB, 0.85286E+00_JPRB ,&
&  0.83872E+00_JPRB, 0.83457E+00_JPRB, 0.84094E+00_JPRB, 0.85039E+00_JPRB, 0.86478E+00_JPRB, 0.88070E+00_JPRB, 0.89420E+00_JPRB ,&
&  0.90831E+00_JPRB, 0.92858E+00_JPRB, 0.95030E+00_JPRB, 0.97031E+00_JPRB, 0.98765E+00_JPRB, 0.10011E+01_JPRB, 0.10089E+01_JPRB ,&
&  0.10122E+01_JPRB, 0.10064E+01_JPRB, 0.99234E+00_JPRB, 0.98229E+00_JPRB, 0.97901E+00_JPRB, 0.98450E+00_JPRB, 0.98641E+00_JPRB ,&
&  0.97847E+00_JPRB, 0.97131E+00_JPRB, 0.96681E+00_JPRB, 0.95458E+00_JPRB, 0.94065E+00_JPRB, 0.92712E+00_JPRB, 0.91320E+00_JPRB ,&
&  0.90247E+00_JPRB, 0.89731E+00_JPRB, 0.89254E+00_JPRB, 0.88495E+00_JPRB, 0.87325E+00_JPRB, 0.86621E+00_JPRB, 0.86028E+00_JPRB ,&
&  0.85489E+00_JPRB, 0.84904E+00_JPRB, 0.84949E+00_JPRB, 0.85921E+00_JPRB, 0.87376E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.84077E+00_JPRB, 0.84647E+00_JPRB, 0.85567E+00_JPRB, 0.86652E+00_JPRB, 0.87070E+00_JPRB, 0.87114E+00_JPRB, 0.86985E+00_JPRB ,&
&  0.87165E+00_JPRB, 0.86447E+00_JPRB, 0.85925E+00_JPRB, 0.85609E+00_JPRB, 0.85547E+00_JPRB, 0.85180E+00_JPRB, 0.84204E+00_JPRB ,&
&  0.83375E+00_JPRB, 0.83020E+00_JPRB, 0.83422E+00_JPRB, 0.83526E+00_JPRB, 0.81964E+00_JPRB, 0.81448E+00_JPRB, 0.80649E+00_JPRB ,&
&  0.80521E+00_JPRB, 0.80505E+00_JPRB, 0.80764E+00_JPRB, 0.81341E+00_JPRB, 0.82950E+00_JPRB, 0.84088E+00_JPRB, 0.84685E+00_JPRB ,&
&  0.84461E+00_JPRB, 0.76701E+00_JPRB, 0.79508E+00_JPRB, 0.80706E+00_JPRB, 0.82957E+00_JPRB, 0.85258E+00_JPRB, 0.87360E+00_JPRB ,&
&  0.90105E+00_JPRB, 0.91659E+00_JPRB, 0.92336E+00_JPRB, 0.92500E+00_JPRB, 0.92672E+00_JPRB, 0.92635E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91503E+00_JPRB, 0.92293E+00_JPRB, 0.94075E+00_JPRB, 0.95038E+00_JPRB, 0.95540E+00_JPRB, 0.95819E+00_JPRB, 0.95520E+00_JPRB ,&
&  0.94805E+00_JPRB, 0.93305E+00_JPRB, 0.90766E+00_JPRB, 0.87686E+00_JPRB, 0.84203E+00_JPRB, 0.81736E+00_JPRB, 0.80381E+00_JPRB ,&
&  0.79669E+00_JPRB, 0.79945E+00_JPRB, 0.80497E+00_JPRB, 0.81092E+00_JPRB, 0.81823E+00_JPRB, 0.83073E+00_JPRB, 0.83860E+00_JPRB ,&
&  0.84901E+00_JPRB, 0.85828E+00_JPRB, 0.87116E+00_JPRB, 0.89094E+00_JPRB, 0.91008E+00_JPRB, 0.92006E+00_JPRB, 0.91972E+00_JPRB ,&
&  0.92473E+00_JPRB, 0.92746E+00_JPRB, 0.92343E+00_JPRB, 0.92247E+00_JPRB, 0.91771E+00_JPRB, 0.91853E+00_JPRB, 0.91599E+00_JPRB ,&
&  0.90695E+00_JPRB, 0.89984E+00_JPRB, 0.89116E+00_JPRB, 0.87484E+00_JPRB, 0.86010E+00_JPRB, 0.84901E+00_JPRB, 0.83972E+00_JPRB ,&
&  0.83775E+00_JPRB, 0.84097E+00_JPRB, 0.84592E+00_JPRB, 0.84030E+00_JPRB, 0.82517E+00_JPRB, 0.81505E+00_JPRB, 0.80354E+00_JPRB ,&
&  0.79570E+00_JPRB, 0.79736E+00_JPRB, 0.79709E+00_JPRB, 0.80440E+00_JPRB, 0.82363E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.79357E+00_JPRB, 0.80621E+00_JPRB, 0.82216E+00_JPRB, 0.83875E+00_JPRB, 0.84957E+00_JPRB, 0.85649E+00_JPRB, 0.85508E+00_JPRB ,&
&  0.84498E+00_JPRB, 0.83493E+00_JPRB, 0.83728E+00_JPRB, 0.83765E+00_JPRB, 0.83679E+00_JPRB, 0.83487E+00_JPRB, 0.82791E+00_JPRB ,&
&  0.81784E+00_JPRB, 0.80912E+00_JPRB, 0.80528E+00_JPRB, 0.80204E+00_JPRB, 0.79190E+00_JPRB, 0.78405E+00_JPRB, 0.77552E+00_JPRB ,&
&  0.77503E+00_JPRB, 0.77729E+00_JPRB, 0.77975E+00_JPRB, 0.77952E+00_JPRB, 0.78706E+00_JPRB, 0.79686E+00_JPRB, 0.80083E+00_JPRB ,&
&  0.79485E+00_JPRB, 0.80221E+00_JPRB, 0.71887E+00_JPRB, 0.78864E+00_JPRB, 0.81361E+00_JPRB, 0.82855E+00_JPRB, 0.84455E+00_JPRB ,&
&  0.85566E+00_JPRB, 0.86777E+00_JPRB, 0.87651E+00_JPRB, 0.88174E+00_JPRB, 0.89969E+00_JPRB, 0.91389E+00_JPRB, 0.91721E+00_JPRB ,&
&  0.91881E+00_JPRB, 0.91784E+00_JPRB, 0.91934E+00_JPRB, 0.92020E+00_JPRB, 0.92554E+00_JPRB, 0.92636E+00_JPRB, 0.92092E+00_JPRB ,&
&  0.91306E+00_JPRB, 0.90105E+00_JPRB, 0.88295E+00_JPRB, 0.86162E+00_JPRB, 0.82827E+00_JPRB, 0.79913E+00_JPRB, 0.78046E+00_JPRB ,&
&  0.76580E+00_JPRB, 0.76224E+00_JPRB, 0.75831E+00_JPRB, 0.75465E+00_JPRB, 0.75471E+00_JPRB, 0.75882E+00_JPRB, 0.76001E+00_JPRB ,&
&  0.76811E+00_JPRB, 0.77746E+00_JPRB, 0.78183E+00_JPRB, 0.78950E+00_JPRB, 0.79977E+00_JPRB, 0.81451E+00_JPRB, 0.82855E+00_JPRB ,&
&  0.84163E+00_JPRB, 0.85085E+00_JPRB, 0.85261E+00_JPRB, 0.85272E+00_JPRB, 0.84979E+00_JPRB, 0.84794E+00_JPRB, 0.84042E+00_JPRB ,&
&  0.83447E+00_JPRB, 0.83120E+00_JPRB, 0.82443E+00_JPRB, 0.81183E+00_JPRB, 0.80367E+00_JPRB, 0.79671E+00_JPRB, 0.79465E+00_JPRB ,&
&  0.79511E+00_JPRB, 0.79627E+00_JPRB, 0.79378E+00_JPRB, 0.79440E+00_JPRB, 0.78774E+00_JPRB, 0.76662E+00_JPRB, 0.74992E+00_JPRB ,&
&  0.74850E+00_JPRB, 0.75340E+00_JPRB, 0.75793E+00_JPRB, 0.76661E+00_JPRB, 0.78140E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.80992E+00_JPRB, 0.81601E+00_JPRB, 0.82530E+00_JPRB, 0.83901E+00_JPRB, 0.85353E+00_JPRB, 0.86322E+00_JPRB, 0.86335E+00_JPRB ,&
&  0.85282E+00_JPRB, 0.83993E+00_JPRB, 0.83595E+00_JPRB, 0.83570E+00_JPRB, 0.83299E+00_JPRB, 0.83166E+00_JPRB, 0.82701E+00_JPRB ,&
&  0.81553E+00_JPRB, 0.79813E+00_JPRB, 0.78975E+00_JPRB, 0.77560E+00_JPRB, 0.76223E+00_JPRB, 0.75123E+00_JPRB, 0.75097E+00_JPRB ,&
&  0.75941E+00_JPRB, 0.76926E+00_JPRB, 0.78144E+00_JPRB, 0.79517E+00_JPRB, 0.80221E+00_JPRB, 0.80030E+00_JPRB, 0.79667E+00_JPRB ,&
&  0.79366E+00_JPRB, 0.79064E+00_JPRB, 0.80105E+00_JPRB, 0.80977E+00_JPRB, 0.81562E+00_JPRB, 0.82676E+00_JPRB, 0.83622E+00_JPRB ,&
&  0.84445E+00_JPRB, 0.85618E+00_JPRB, 0.87512E+00_JPRB, 0.88913E+00_JPRB, 0.89954E+00_JPRB, 0.91051E+00_JPRB, 0.91391E+00_JPRB ,&
&  0.91994E+00_JPRB, 0.93119E+00_JPRB, 0.93346E+00_JPRB, 0.92450E+00_JPRB, 0.91040E+00_JPRB, 0.90165E+00_JPRB, 0.89698E+00_JPRB ,&
&  0.88807E+00_JPRB, 0.86872E+00_JPRB, 0.85390E+00_JPRB, 0.84178E+00_JPRB, 0.82218E+00_JPRB, 0.80129E+00_JPRB, 0.77696E+00_JPRB ,&
&  0.74954E+00_JPRB, 0.72913E+00_JPRB, 0.71343E+00_JPRB, 0.69930E+00_JPRB, 0.69597E+00_JPRB, 0.68765E+00_JPRB, 0.68182E+00_JPRB ,&
&  0.68728E+00_JPRB, 0.70671E+00_JPRB, 0.72156E+00_JPRB, 0.72744E+00_JPRB, 0.73079E+00_JPRB, 0.74738E+00_JPRB, 0.76736E+00_JPRB ,&
&  0.78306E+00_JPRB, 0.78997E+00_JPRB, 0.78647E+00_JPRB, 0.78909E+00_JPRB, 0.79430E+00_JPRB, 0.79843E+00_JPRB, 0.79190E+00_JPRB ,&
&  0.78275E+00_JPRB, 0.77680E+00_JPRB, 0.77523E+00_JPRB, 0.77122E+00_JPRB, 0.77245E+00_JPRB, 0.77217E+00_JPRB, 0.76934E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.75460E+00_JPRB, 0.74271E+00_JPRB, 0.73499E+00_JPRB, 0.72369E+00_JPRB, 0.70690E+00_JPRB, 0.70697E+00_JPRB ,&
&  0.71803E+00_JPRB, 0.73614E+00_JPRB, 0.75365E+00_JPRB, 0.77446E+00_JPRB, 0.79876E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.83536E+00_JPRB, 0.84322E+00_JPRB, 0.84868E+00_JPRB, 0.85271E+00_JPRB, 0.86164E+00_JPRB, 0.87296E+00_JPRB, 0.87312E+00_JPRB ,&
&  0.85865E+00_JPRB, 0.84503E+00_JPRB, 0.83561E+00_JPRB, 0.83320E+00_JPRB, 0.83513E+00_JPRB, 0.83756E+00_JPRB, 0.83171E+00_JPRB ,&
&  0.81004E+00_JPRB, 0.78690E+00_JPRB, 0.77267E+00_JPRB, 0.75859E+00_JPRB, 0.74977E+00_JPRB, 0.74408E+00_JPRB, 0.75068E+00_JPRB ,&
&  0.76054E+00_JPRB, 0.77618E+00_JPRB, 0.78882E+00_JPRB, 0.79789E+00_JPRB, 0.80139E+00_JPRB, 0.78967E+00_JPRB, 0.79072E+00_JPRB ,&
&  0.79752E+00_JPRB, 0.79950E+00_JPRB, 0.81431E+00_JPRB, 0.81353E+00_JPRB, 0.80003E+00_JPRB, 0.78901E+00_JPRB, 0.77827E+00_JPRB ,&
&  0.77557E+00_JPRB, 0.78686E+00_JPRB, 0.81219E+00_JPRB, 0.84895E+00_JPRB, 0.87056E+00_JPRB, 0.87893E+00_JPRB, 0.87572E+00_JPRB ,&
&  0.87815E+00_JPRB, 0.89000E+00_JPRB, 0.89668E+00_JPRB, 0.88843E+00_JPRB, 0.87023E+00_JPRB, 0.85549E+00_JPRB, 0.84258E+00_JPRB ,&
&  0.83317E+00_JPRB, 0.82172E+00_JPRB, 0.81384E+00_JPRB, 0.80386E+00_JPRB, 0.78177E+00_JPRB, 0.75558E+00_JPRB, 0.73267E+00_JPRB ,&
&  0.71548E+00_JPRB, 0.69849E+00_JPRB, 0.67792E+00_JPRB, 0.67124E+00_JPRB, 0.67511E+00_JPRB, 0.65230E+00_JPRB, 0.62802E+00_JPRB ,&
&  0.63234E+00_JPRB, 0.66464E+00_JPRB, 0.69014E+00_JPRB, 0.70103E+00_JPRB, 0.70271E+00_JPRB, 0.70758E+00_JPRB, 0.71148E+00_JPRB ,&
&  0.71850E+00_JPRB, 0.72760E+00_JPRB, 0.73458E+00_JPRB, 0.74601E+00_JPRB, 0.75775E+00_JPRB, 0.75599E+00_JPRB, 0.74117E+00_JPRB ,&
&  0.72504E+00_JPRB, 0.71581E+00_JPRB, 0.71201E+00_JPRB, 0.71422E+00_JPRB, 0.71761E+00_JPRB, 0.71572E+00_JPRB, 0.70995E+00_JPRB ,&
&  0.69804E+00_JPRB, 0.67956E+00_JPRB, 0.66384E+00_JPRB, 0.65467E+00_JPRB, 0.65371E+00_JPRB, 0.65987E+00_JPRB, 0.67598E+00_JPRB ,&
&  0.70732E+00_JPRB, 0.75511E+00_JPRB, 0.79099E+00_JPRB, 0.81170E+00_JPRB, 0.81784E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.86261E+00_JPRB, 0.87070E+00_JPRB, 0.88036E+00_JPRB, 0.89355E+00_JPRB, 0.90583E+00_JPRB, 0.90277E+00_JPRB, 0.89018E+00_JPRB ,&
&  0.86649E+00_JPRB, 0.85742E+00_JPRB, 0.86203E+00_JPRB, 0.86446E+00_JPRB, 0.86625E+00_JPRB, 0.86408E+00_JPRB, 0.84782E+00_JPRB ,&
&  0.80964E+00_JPRB, 0.77917E+00_JPRB, 0.76456E+00_JPRB, 0.75688E+00_JPRB, 0.75434E+00_JPRB, 0.76105E+00_JPRB, 0.76895E+00_JPRB ,&
&  0.78508E+00_JPRB, 0.79447E+00_JPRB, 0.80620E+00_JPRB, 0.81558E+00_JPRB, 0.81835E+00_JPRB, 0.82224E+00_JPRB, 0.82313E+00_JPRB ,&
&  0.80866E+00_JPRB, 0.80871E+00_JPRB, 0.80976E+00_JPRB, 0.79977E+00_JPRB, 0.77782E+00_JPRB, 0.75578E+00_JPRB, 0.72891E+00_JPRB ,&
&  0.71098E+00_JPRB, 0.70187E+00_JPRB, 0.72738E+00_JPRB, 0.78946E+00_JPRB, 0.83332E+00_JPRB, 0.84147E+00_JPRB, 0.83325E+00_JPRB ,&
&  0.83485E+00_JPRB, 0.84211E+00_JPRB, 0.84568E+00_JPRB, 0.83430E+00_JPRB, 0.81465E+00_JPRB, 0.79740E+00_JPRB, 0.78100E+00_JPRB ,&
&  0.77567E+00_JPRB, 0.77517E+00_JPRB, 0.75808E+00_JPRB, 0.72601E+00_JPRB, 0.69829E+00_JPRB, 0.67866E+00_JPRB, 0.66825E+00_JPRB ,&
&  0.66175E+00_JPRB, 0.65180E+00_JPRB, 0.64538E+00_JPRB, 0.64987E+00_JPRB, 0.62984E+00_JPRB, 0.61216E+00_JPRB, 0.61640E+00_JPRB ,&
&  0.63336E+00_JPRB, 0.63962E+00_JPRB, 0.64952E+00_JPRB, 0.65338E+00_JPRB, 0.65111E+00_JPRB, 0.64805E+00_JPRB, 0.64972E+00_JPRB ,&
&  0.65889E+00_JPRB, 0.66939E+00_JPRB, 0.67710E+00_JPRB, 0.68209E+00_JPRB, 0.68701E+00_JPRB, 0.67464E+00_JPRB, 0.65301E+00_JPRB ,&
&  0.63528E+00_JPRB, 0.63187E+00_JPRB, 0.63622E+00_JPRB, 0.63547E+00_JPRB, 0.63384E+00_JPRB, 0.63411E+00_JPRB, 0.63301E+00_JPRB ,&
&  0.63044E+00_JPRB, 0.62337E+00_JPRB, 0.61478E+00_JPRB, 0.61470E+00_JPRB, 0.62568E+00_JPRB, 0.64847E+00_JPRB, 0.68396E+00_JPRB ,&
&  0.73619E+00_JPRB, 0.77554E+00_JPRB, 0.81410E+00_JPRB, 0.83041E+00_JPRB, 0.83471E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.89670E+00_JPRB, 0.91877E+00_JPRB, 0.93757E+00_JPRB, 0.95031E+00_JPRB, 0.95215E+00_JPRB, 0.94521E+00_JPRB, 0.92908E+00_JPRB ,&
&  0.89877E+00_JPRB, 0.88149E+00_JPRB, 0.89038E+00_JPRB, 0.90510E+00_JPRB, 0.91401E+00_JPRB, 0.91397E+00_JPRB, 0.88213E+00_JPRB ,&
&  0.82697E+00_JPRB, 0.80602E+00_JPRB, 0.80287E+00_JPRB, 0.81228E+00_JPRB, 0.83011E+00_JPRB, 0.83646E+00_JPRB, 0.82363E+00_JPRB ,&
&  0.81073E+00_JPRB, 0.81189E+00_JPRB, 0.81444E+00_JPRB, 0.82135E+00_JPRB, 0.81090E+00_JPRB, 0.78986E+00_JPRB, 0.78047E+00_JPRB ,&
&  0.77743E+00_JPRB, 0.76915E+00_JPRB, 0.76053E+00_JPRB, 0.71170E+00_JPRB, 0.73411E+00_JPRB, 0.72238E+00_JPRB, 0.70365E+00_JPRB ,&
&  0.68611E+00_JPRB, 0.68833E+00_JPRB, 0.73038E+00_JPRB, 0.77735E+00_JPRB, 0.78901E+00_JPRB, 0.79155E+00_JPRB, 0.78613E+00_JPRB ,&
&  0.78238E+00_JPRB, 0.78277E+00_JPRB, 0.78417E+00_JPRB, 0.77601E+00_JPRB, 0.75565E+00_JPRB, 0.72934E+00_JPRB, 0.70058E+00_JPRB ,&
&  0.68331E+00_JPRB, 0.65858E+00_JPRB, 0.61523E+00_JPRB, 0.59502E+00_JPRB, 0.59331E+00_JPRB, 0.60145E+00_JPRB, 0.60530E+00_JPRB ,&
&  0.60231E+00_JPRB, 0.59482E+00_JPRB, 0.59824E+00_JPRB, 0.59495E+00_JPRB, 0.57315E+00_JPRB, 0.55970E+00_JPRB, 0.55371E+00_JPRB ,&
&  0.54279E+00_JPRB, 0.54259E+00_JPRB, 0.54821E+00_JPRB, 0.56052E+00_JPRB, 0.57137E+00_JPRB, 0.58626E+00_JPRB, 0.59275E+00_JPRB ,&
&  0.59522E+00_JPRB, 0.59135E+00_JPRB, 0.58494E+00_JPRB, 0.57241E+00_JPRB, 0.55343E+00_JPRB, 0.54609E+00_JPRB, 0.54757E+00_JPRB ,&
&  0.55662E+00_JPRB, 0.56280E+00_JPRB, 0.56249E+00_JPRB, 0.56148E+00_JPRB, 0.54993E+00_JPRB, 0.53728E+00_JPRB, 0.52782E+00_JPRB ,&
&  0.52364E+00_JPRB, 0.52705E+00_JPRB, 0.54320E+00_JPRB, 0.56871E+00_JPRB, 0.60124E+00_JPRB, 0.64072E+00_JPRB, 0.68580E+00_JPRB ,&
&  0.71740E+00_JPRB, 0.73567E+00_JPRB, 0.76006E+00_JPRB, 0.80824E+00_JPRB, 0.85513E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.89800E+00_JPRB, 0.92514E+00_JPRB, 0.95452E+00_JPRB, 0.97137E+00_JPRB, 0.97216E+00_JPRB, 0.96892E+00_JPRB, 0.95375E+00_JPRB ,&
&  0.92277E+00_JPRB, 0.89752E+00_JPRB, 0.89341E+00_JPRB, 0.89741E+00_JPRB, 0.90916E+00_JPRB, 0.92776E+00_JPRB, 0.92880E+00_JPRB ,&
&  0.90140E+00_JPRB, 0.88155E+00_JPRB, 0.87156E+00_JPRB, 0.86404E+00_JPRB, 0.84649E+00_JPRB, 0.82135E+00_JPRB, 0.80489E+00_JPRB ,&
&  0.80004E+00_JPRB, 0.79971E+00_JPRB, 0.79069E+00_JPRB, 0.77682E+00_JPRB, 0.74526E+00_JPRB, 0.70624E+00_JPRB, 0.67527E+00_JPRB ,&
&  0.66400E+00_JPRB, 0.65368E+00_JPRB, 0.61350E+00_JPRB, 0.62840E+00_JPRB, 0.63245E+00_JPRB, 0.63287E+00_JPRB, 0.63001E+00_JPRB ,&
&  0.64151E+00_JPRB, 0.67478E+00_JPRB, 0.70809E+00_JPRB, 0.71783E+00_JPRB, 0.72711E+00_JPRB, 0.72876E+00_JPRB, 0.73009E+00_JPRB ,&
&  0.73452E+00_JPRB, 0.72652E+00_JPRB, 0.69687E+00_JPRB, 0.65284E+00_JPRB, 0.60944E+00_JPRB, 0.57566E+00_JPRB, 0.54695E+00_JPRB ,&
&  0.52733E+00_JPRB, 0.51501E+00_JPRB, 0.50533E+00_JPRB, 0.50204E+00_JPRB, 0.50998E+00_JPRB, 0.52532E+00_JPRB, 0.52034E+00_JPRB ,&
&  0.50195E+00_JPRB, 0.48473E+00_JPRB, 0.46222E+00_JPRB, 0.43166E+00_JPRB, 0.40007E+00_JPRB, 0.36227E+00_JPRB, 0.34249E+00_JPRB ,&
&  0.34742E+00_JPRB, 0.38720E+00_JPRB, 0.43933E+00_JPRB, 0.45996E+00_JPRB, 0.48773E+00_JPRB, 0.51507E+00_JPRB, 0.50454E+00_JPRB ,&
&  0.45562E+00_JPRB, 0.38860E+00_JPRB, 0.36562E+00_JPRB, 0.35226E+00_JPRB, 0.34521E+00_JPRB, 0.34043E+00_JPRB, 0.33907E+00_JPRB ,&
&  0.34343E+00_JPRB, 0.35452E+00_JPRB, 0.36438E+00_JPRB, 0.36159E+00_JPRB, 0.35145E+00_JPRB, 0.34584E+00_JPRB, 0.34703E+00_JPRB ,&
&  0.35779E+00_JPRB, 0.38691E+00_JPRB, 0.44998E+00_JPRB, 0.56382E+00_JPRB, 0.60186E+00_JPRB, 0.63603E+00_JPRB, 0.67508E+00_JPRB ,&
&  0.71258E+00_JPRB, 0.74655E+00_JPRB, 0.77528E+00_JPRB, 0.81047E+00_JPRB, 0.85786E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.83571E+00_JPRB, 0.85460E+00_JPRB, 0.87636E+00_JPRB, 0.88783E+00_JPRB, 0.88906E+00_JPRB, 0.88618E+00_JPRB, 0.87752E+00_JPRB ,&
&  0.86260E+00_JPRB, 0.84470E+00_JPRB, 0.83144E+00_JPRB, 0.82230E+00_JPRB, 0.81796E+00_JPRB, 0.82045E+00_JPRB, 0.82519E+00_JPRB ,&
&  0.82369E+00_JPRB, 0.81912E+00_JPRB, 0.80648E+00_JPRB, 0.79010E+00_JPRB, 0.77280E+00_JPRB, 0.75949E+00_JPRB, 0.74940E+00_JPRB ,&
&  0.73961E+00_JPRB, 0.72494E+00_JPRB, 0.70212E+00_JPRB, 0.66671E+00_JPRB, 0.62475E+00_JPRB, 0.58641E+00_JPRB, 0.55597E+00_JPRB ,&
&  0.52864E+00_JPRB, 0.46257E+00_JPRB, 0.42620E+00_JPRB, 0.43646E+00_JPRB, 0.52554E+00_JPRB, 0.55841E+00_JPRB, 0.58340E+00_JPRB ,&
&  0.61966E+00_JPRB, 0.65830E+00_JPRB, 0.67693E+00_JPRB, 0.67728E+00_JPRB, 0.67162E+00_JPRB, 0.65886E+00_JPRB, 0.63904E+00_JPRB ,&
&  0.61388E+00_JPRB, 0.58319E+00_JPRB, 0.55694E+00_JPRB, 0.52739E+00_JPRB, 0.49725E+00_JPRB, 0.47515E+00_JPRB, 0.45983E+00_JPRB ,&
&  0.44926E+00_JPRB, 0.43937E+00_JPRB, 0.43141E+00_JPRB, 0.42157E+00_JPRB, 0.41250E+00_JPRB, 0.39146E+00_JPRB, 0.36892E+00_JPRB ,&
&  0.34666E+00_JPRB, 0.32861E+00_JPRB, 0.31006E+00_JPRB, 0.29171E+00_JPRB, 0.27670E+00_JPRB, 0.26680E+00_JPRB, 0.25938E+00_JPRB ,&
&  0.25742E+00_JPRB, 0.26439E+00_JPRB, 0.28424E+00_JPRB, 0.31717E+00_JPRB, 0.35283E+00_JPRB, 0.32762E+00_JPRB, 0.30807E+00_JPRB ,&
&  0.28628E+00_JPRB, 0.27375E+00_JPRB, 0.26887E+00_JPRB, 0.26812E+00_JPRB, 0.26922E+00_JPRB, 0.27134E+00_JPRB, 0.27370E+00_JPRB ,&
&  0.27721E+00_JPRB, 0.28056E+00_JPRB, 0.28395E+00_JPRB, 0.28648E+00_JPRB, 0.28804E+00_JPRB, 0.29079E+00_JPRB, 0.29557E+00_JPRB ,&
&  0.30343E+00_JPRB, 0.31619E+00_JPRB, 0.33648E+00_JPRB, 0.36289E+00_JPRB, 0.39093E+00_JPRB, 0.41611E+00_JPRB, 0.46934E+00_JPRB ,&
&  0.55684E+00_JPRB, 0.66801E+00_JPRB, 0.72688E+00_JPRB, 0.78438E+00_JPRB, 0.81327E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.77027E+00_JPRB, 0.78336E+00_JPRB, 0.79407E+00_JPRB, 0.79661E+00_JPRB, 0.79599E+00_JPRB, 0.79097E+00_JPRB, 0.77879E+00_JPRB ,&
&  0.75902E+00_JPRB, 0.73267E+00_JPRB, 0.70695E+00_JPRB, 0.68647E+00_JPRB, 0.67355E+00_JPRB, 0.67618E+00_JPRB, 0.68437E+00_JPRB ,&
&  0.70128E+00_JPRB, 0.72633E+00_JPRB, 0.73114E+00_JPRB, 0.72401E+00_JPRB, 0.68802E+00_JPRB, 0.65053E+00_JPRB, 0.61619E+00_JPRB ,&
&  0.58545E+00_JPRB, 0.55601E+00_JPRB, 0.54504E+00_JPRB, 0.52587E+00_JPRB, 0.51237E+00_JPRB, 0.49540E+00_JPRB, 0.47099E+00_JPRB ,&
&  0.43126E+00_JPRB, 0.40113E+00_JPRB, 0.38560E+00_JPRB, 0.39634E+00_JPRB, 0.48220E+00_JPRB, 0.54330E+00_JPRB, 0.55476E+00_JPRB ,&
&  0.58877E+00_JPRB, 0.61278E+00_JPRB, 0.60762E+00_JPRB, 0.58742E+00_JPRB, 0.56562E+00_JPRB, 0.54209E+00_JPRB, 0.52230E+00_JPRB ,&
&  0.50260E+00_JPRB, 0.47615E+00_JPRB, 0.42612E+00_JPRB, 0.35525E+00_JPRB, 0.32009E+00_JPRB, 0.30679E+00_JPRB, 0.29243E+00_JPRB ,&
&  0.28166E+00_JPRB, 0.27345E+00_JPRB, 0.27009E+00_JPRB, 0.26621E+00_JPRB, 0.26564E+00_JPRB, 0.26186E+00_JPRB, 0.25837E+00_JPRB ,&
&  0.25343E+00_JPRB, 0.24976E+00_JPRB, 0.24475E+00_JPRB, 0.23989E+00_JPRB, 0.23604E+00_JPRB, 0.23547E+00_JPRB, 0.23536E+00_JPRB ,&
&  0.23658E+00_JPRB, 0.24086E+00_JPRB, 0.25331E+00_JPRB, 0.26007E+00_JPRB, 0.26063E+00_JPRB, 0.25666E+00_JPRB, 0.24938E+00_JPRB ,&
&  0.24227E+00_JPRB, 0.24070E+00_JPRB, 0.23926E+00_JPRB, 0.23987E+00_JPRB, 0.24285E+00_JPRB, 0.24667E+00_JPRB, 0.24961E+00_JPRB ,&
&  0.25194E+00_JPRB, 0.25451E+00_JPRB, 0.25682E+00_JPRB, 0.25936E+00_JPRB, 0.26254E+00_JPRB, 0.26630E+00_JPRB, 0.27150E+00_JPRB ,&
&  0.27693E+00_JPRB, 0.28465E+00_JPRB, 0.29530E+00_JPRB, 0.31083E+00_JPRB, 0.32507E+00_JPRB, 0.33445E+00_JPRB, 0.34756E+00_JPRB ,&
&  0.36406E+00_JPRB, 0.41224E+00_JPRB, 0.54752E+00_JPRB, 0.77061E+00_JPRB, 0.80309E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.69147E+00_JPRB, 0.70387E+00_JPRB, 0.71841E+00_JPRB, 0.72197E+00_JPRB, 0.73221E+00_JPRB, 0.73100E+00_JPRB, 0.71346E+00_JPRB ,&
&  0.68527E+00_JPRB, 0.65839E+00_JPRB, 0.64376E+00_JPRB, 0.62469E+00_JPRB, 0.59648E+00_JPRB, 0.53673E+00_JPRB, 0.51398E+00_JPRB ,&
&  0.49519E+00_JPRB, 0.49460E+00_JPRB, 0.48503E+00_JPRB, 0.48491E+00_JPRB, 0.47361E+00_JPRB, 0.46211E+00_JPRB, 0.44317E+00_JPRB ,&
&  0.42580E+00_JPRB, 0.40904E+00_JPRB, 0.39726E+00_JPRB, 0.39195E+00_JPRB, 0.38951E+00_JPRB, 0.38438E+00_JPRB, 0.37134E+00_JPRB ,&
&  0.36229E+00_JPRB, 0.36045E+00_JPRB, 0.36281E+00_JPRB, 0.37067E+00_JPRB, 0.41258E+00_JPRB, 0.46937E+00_JPRB, 0.47051E+00_JPRB ,&
&  0.49249E+00_JPRB, 0.50545E+00_JPRB, 0.51819E+00_JPRB, 0.50687E+00_JPRB, 0.47376E+00_JPRB, 0.43371E+00_JPRB, 0.38047E+00_JPRB ,&
&  0.32987E+00_JPRB, 0.30068E+00_JPRB, 0.27696E+00_JPRB, 0.26005E+00_JPRB, 0.24823E+00_JPRB, 0.24355E+00_JPRB, 0.23750E+00_JPRB ,&
&  0.23362E+00_JPRB, 0.23062E+00_JPRB, 0.22942E+00_JPRB, 0.22892E+00_JPRB, 0.22817E+00_JPRB, 0.22605E+00_JPRB, 0.22548E+00_JPRB ,&
&  0.22614E+00_JPRB, 0.22628E+00_JPRB, 0.22601E+00_JPRB, 0.22694E+00_JPRB, 0.22797E+00_JPRB, 0.22953E+00_JPRB, 0.23104E+00_JPRB ,&
&  0.23333E+00_JPRB, 0.23460E+00_JPRB, 0.23651E+00_JPRB, 0.23863E+00_JPRB, 0.23759E+00_JPRB, 0.23649E+00_JPRB, 0.23372E+00_JPRB ,&
&  0.23052E+00_JPRB, 0.23042E+00_JPRB, 0.22994E+00_JPRB, 0.23062E+00_JPRB, 0.23242E+00_JPRB, 0.23477E+00_JPRB, 0.23781E+00_JPRB ,&
&  0.24067E+00_JPRB, 0.24364E+00_JPRB, 0.24590E+00_JPRB, 0.24820E+00_JPRB, 0.25210E+00_JPRB, 0.25619E+00_JPRB, 0.26140E+00_JPRB ,&
&  0.26678E+00_JPRB, 0.27294E+00_JPRB, 0.28137E+00_JPRB, 0.29233E+00_JPRB, 0.30841E+00_JPRB, 0.32616E+00_JPRB, 0.35375E+00_JPRB ,&
&  0.37778E+00_JPRB, 0.47583E+00_JPRB, 0.71059E+00_JPRB, 0.64179E+00_JPRB, 0.66282E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.65438E+00_JPRB, 0.67158E+00_JPRB, 0.69621E+00_JPRB, 0.68913E+00_JPRB, 0.70900E+00_JPRB, 0.69561E+00_JPRB, 0.67235E+00_JPRB ,&
&  0.62498E+00_JPRB, 0.56883E+00_JPRB, 0.52196E+00_JPRB, 0.46988E+00_JPRB, 0.42884E+00_JPRB, 0.39932E+00_JPRB, 0.38464E+00_JPRB ,&
&  0.37698E+00_JPRB, 0.37449E+00_JPRB, 0.37245E+00_JPRB, 0.37129E+00_JPRB, 0.36587E+00_JPRB, 0.36215E+00_JPRB, 0.35608E+00_JPRB ,&
&  0.35019E+00_JPRB, 0.34649E+00_JPRB, 0.34332E+00_JPRB, 0.34828E+00_JPRB, 0.35422E+00_JPRB, 0.36031E+00_JPRB, 0.36321E+00_JPRB ,&
&  0.36557E+00_JPRB, 0.36701E+00_JPRB, 0.36996E+00_JPRB, 0.36777E+00_JPRB, 0.37908E+00_JPRB, 0.40000E+00_JPRB, 0.41734E+00_JPRB ,&
&  0.43419E+00_JPRB, 0.44809E+00_JPRB, 0.44368E+00_JPRB, 0.41951E+00_JPRB, 0.35343E+00_JPRB, 0.31936E+00_JPRB, 0.28895E+00_JPRB ,&
&  0.26958E+00_JPRB, 0.25515E+00_JPRB, 0.24532E+00_JPRB, 0.23975E+00_JPRB, 0.23506E+00_JPRB, 0.23263E+00_JPRB, 0.23002E+00_JPRB ,&
&  0.22757E+00_JPRB, 0.22619E+00_JPRB, 0.22450E+00_JPRB, 0.22366E+00_JPRB, 0.22265E+00_JPRB, 0.22128E+00_JPRB, 0.22038E+00_JPRB ,&
&  0.22071E+00_JPRB, 0.22061E+00_JPRB, 0.22190E+00_JPRB, 0.22296E+00_JPRB, 0.22499E+00_JPRB, 0.22664E+00_JPRB, 0.22899E+00_JPRB ,&
&  0.23108E+00_JPRB, 0.23283E+00_JPRB, 0.23272E+00_JPRB, 0.23233E+00_JPRB, 0.23146E+00_JPRB, 0.23101E+00_JPRB, 0.23022E+00_JPRB ,&
&  0.22909E+00_JPRB, 0.22939E+00_JPRB, 0.22983E+00_JPRB, 0.23030E+00_JPRB, 0.23162E+00_JPRB, 0.23337E+00_JPRB, 0.23593E+00_JPRB ,&
&  0.23934E+00_JPRB, 0.24285E+00_JPRB, 0.24585E+00_JPRB, 0.24909E+00_JPRB, 0.25322E+00_JPRB, 0.25785E+00_JPRB, 0.26355E+00_JPRB ,&
&  0.26833E+00_JPRB, 0.27694E+00_JPRB, 0.28907E+00_JPRB, 0.30186E+00_JPRB, 0.32241E+00_JPRB, 0.34085E+00_JPRB, 0.37157E+00_JPRB ,&
&  0.41904E+00_JPRB, 0.58681E+00_JPRB, 0.86920E+00_JPRB, 0.64749E+00_JPRB, 0.63879E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.57377E+00_JPRB, 0.58325E+00_JPRB, 0.59736E+00_JPRB, 0.60888E+00_JPRB, 0.63263E+00_JPRB, 0.64602E+00_JPRB, 0.69094E+00_JPRB ,&
&  0.64537E+00_JPRB, 0.56700E+00_JPRB, 0.53490E+00_JPRB, 0.49968E+00_JPRB, 0.45985E+00_JPRB, 0.43105E+00_JPRB, 0.41081E+00_JPRB ,&
&  0.39848E+00_JPRB, 0.38791E+00_JPRB, 0.37760E+00_JPRB, 0.36490E+00_JPRB, 0.35195E+00_JPRB, 0.33780E+00_JPRB, 0.32745E+00_JPRB ,&
&  0.32139E+00_JPRB, 0.32009E+00_JPRB, 0.31962E+00_JPRB, 0.32354E+00_JPRB, 0.32888E+00_JPRB, 0.33587E+00_JPRB, 0.34419E+00_JPRB ,&
&  0.36392E+00_JPRB, 0.37130E+00_JPRB, 0.37295E+00_JPRB, 0.37257E+00_JPRB, 0.37343E+00_JPRB, 0.37606E+00_JPRB, 0.36661E+00_JPRB ,&
&  0.37326E+00_JPRB, 0.39282E+00_JPRB, 0.39837E+00_JPRB, 0.37837E+00_JPRB, 0.32382E+00_JPRB, 0.29158E+00_JPRB, 0.26888E+00_JPRB ,&
&  0.26076E+00_JPRB, 0.25058E+00_JPRB, 0.24423E+00_JPRB, 0.23806E+00_JPRB, 0.23378E+00_JPRB, 0.23187E+00_JPRB, 0.22991E+00_JPRB ,&
&  0.22860E+00_JPRB, 0.22785E+00_JPRB, 0.22587E+00_JPRB, 0.22505E+00_JPRB, 0.22386E+00_JPRB, 0.22320E+00_JPRB, 0.22288E+00_JPRB ,&
&  0.22272E+00_JPRB, 0.22243E+00_JPRB, 0.22269E+00_JPRB, 0.22325E+00_JPRB, 0.22420E+00_JPRB, 0.22496E+00_JPRB, 0.22566E+00_JPRB ,&
&  0.22665E+00_JPRB, 0.22758E+00_JPRB, 0.22787E+00_JPRB, 0.22849E+00_JPRB, 0.22901E+00_JPRB, 0.23033E+00_JPRB, 0.23097E+00_JPRB ,&
&  0.23162E+00_JPRB, 0.23220E+00_JPRB, 0.23226E+00_JPRB, 0.23300E+00_JPRB, 0.23404E+00_JPRB, 0.23606E+00_JPRB, 0.23829E+00_JPRB ,&
&  0.24114E+00_JPRB, 0.24452E+00_JPRB, 0.24820E+00_JPRB, 0.25191E+00_JPRB, 0.25673E+00_JPRB, 0.26183E+00_JPRB, 0.26859E+00_JPRB ,&
&  0.27588E+00_JPRB, 0.28552E+00_JPRB, 0.29883E+00_JPRB, 0.31478E+00_JPRB, 0.33442E+00_JPRB, 0.35282E+00_JPRB, 0.37270E+00_JPRB ,&
&  0.40657E+00_JPRB, 0.50408E+00_JPRB, 0.56430E+00_JPRB, 0.55505E+00_JPRB, 0.56694E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.55452E+00_JPRB, 0.55731E+00_JPRB, 0.56027E+00_JPRB, 0.56468E+00_JPRB, 0.56945E+00_JPRB, 0.57395E+00_JPRB, 0.57764E+00_JPRB ,&
&  0.55916E+00_JPRB, 0.53605E+00_JPRB, 0.51694E+00_JPRB, 0.49415E+00_JPRB, 0.47333E+00_JPRB, 0.45567E+00_JPRB, 0.43542E+00_JPRB ,&
&  0.41510E+00_JPRB, 0.39489E+00_JPRB, 0.37215E+00_JPRB, 0.35346E+00_JPRB, 0.33510E+00_JPRB, 0.32026E+00_JPRB, 0.31155E+00_JPRB ,&
&  0.30662E+00_JPRB, 0.30519E+00_JPRB, 0.30649E+00_JPRB, 0.30892E+00_JPRB, 0.31136E+00_JPRB, 0.31581E+00_JPRB, 0.32215E+00_JPRB ,&
&  0.33095E+00_JPRB, 0.34604E+00_JPRB, 0.35953E+00_JPRB, 0.36423E+00_JPRB, 0.36032E+00_JPRB, 0.35155E+00_JPRB, 0.34393E+00_JPRB ,&
&  0.33484E+00_JPRB, 0.32274E+00_JPRB, 0.30775E+00_JPRB, 0.29287E+00_JPRB, 0.27901E+00_JPRB, 0.26895E+00_JPRB, 0.26042E+00_JPRB ,&
&  0.25311E+00_JPRB, 0.24663E+00_JPRB, 0.24157E+00_JPRB, 0.23819E+00_JPRB, 0.23529E+00_JPRB, 0.23300E+00_JPRB, 0.23076E+00_JPRB ,&
&  0.22974E+00_JPRB, 0.22868E+00_JPRB, 0.22804E+00_JPRB, 0.22771E+00_JPRB, 0.22719E+00_JPRB, 0.22702E+00_JPRB, 0.22692E+00_JPRB ,&
&  0.22657E+00_JPRB, 0.22594E+00_JPRB, 0.22559E+00_JPRB, 0.22536E+00_JPRB, 0.22585E+00_JPRB, 0.22626E+00_JPRB, 0.22690E+00_JPRB ,&
&  0.22789E+00_JPRB, 0.22984E+00_JPRB, 0.23105E+00_JPRB, 0.23218E+00_JPRB, 0.23278E+00_JPRB, 0.23333E+00_JPRB, 0.23404E+00_JPRB ,&
&  0.23519E+00_JPRB, 0.23657E+00_JPRB, 0.23777E+00_JPRB, 0.23939E+00_JPRB, 0.24105E+00_JPRB, 0.24332E+00_JPRB, 0.24578E+00_JPRB ,&
&  0.24853E+00_JPRB, 0.25192E+00_JPRB, 0.25610E+00_JPRB, 0.25985E+00_JPRB, 0.26583E+00_JPRB, 0.27186E+00_JPRB, 0.27867E+00_JPRB ,&
&  0.28576E+00_JPRB, 0.29344E+00_JPRB, 0.30364E+00_JPRB, 0.31577E+00_JPRB, 0.33104E+00_JPRB, 0.33894E+00_JPRB, 0.34952E+00_JPRB ,&
&  0.36591E+00_JPRB, 0.40038E+00_JPRB, 0.53684E+00_JPRB, 0.55048E+00_JPRB, 0.55546E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.33471E+00_JPRB, 0.34049E+00_JPRB, 0.34441E+00_JPRB, 0.35333E+00_JPRB, 0.36494E+00_JPRB, 0.46308E+00_JPRB, 0.45809E+00_JPRB ,&
&  0.45733E+00_JPRB, 0.41418E+00_JPRB, 0.40636E+00_JPRB, 0.39427E+00_JPRB, 0.37895E+00_JPRB, 0.36004E+00_JPRB, 0.34835E+00_JPRB ,&
&  0.33864E+00_JPRB, 0.32725E+00_JPRB, 0.31667E+00_JPRB, 0.31040E+00_JPRB, 0.30544E+00_JPRB, 0.30099E+00_JPRB, 0.29830E+00_JPRB ,&
&  0.29707E+00_JPRB, 0.29726E+00_JPRB, 0.29777E+00_JPRB, 0.29837E+00_JPRB, 0.30154E+00_JPRB, 0.30508E+00_JPRB, 0.30868E+00_JPRB ,&
&  0.31025E+00_JPRB, 0.31464E+00_JPRB, 0.31562E+00_JPRB, 0.31252E+00_JPRB, 0.30754E+00_JPRB, 0.30023E+00_JPRB, 0.29286E+00_JPRB ,&
&  0.28527E+00_JPRB, 0.27828E+00_JPRB, 0.27248E+00_JPRB, 0.26726E+00_JPRB, 0.26127E+00_JPRB, 0.25552E+00_JPRB, 0.25176E+00_JPRB ,&
&  0.24833E+00_JPRB, 0.24481E+00_JPRB, 0.24200E+00_JPRB, 0.24061E+00_JPRB, 0.23904E+00_JPRB, 0.23798E+00_JPRB, 0.23719E+00_JPRB ,&
&  0.23697E+00_JPRB, 0.23685E+00_JPRB, 0.23693E+00_JPRB, 0.23721E+00_JPRB, 0.23719E+00_JPRB, 0.23712E+00_JPRB, 0.23708E+00_JPRB ,&
&  0.23767E+00_JPRB, 0.23772E+00_JPRB, 0.23767E+00_JPRB, 0.23746E+00_JPRB, 0.23821E+00_JPRB, 0.23851E+00_JPRB, 0.23897E+00_JPRB ,&
&  0.23964E+00_JPRB, 0.24055E+00_JPRB, 0.24123E+00_JPRB, 0.24198E+00_JPRB, 0.24271E+00_JPRB, 0.24360E+00_JPRB, 0.24464E+00_JPRB ,&
&  0.24571E+00_JPRB, 0.24727E+00_JPRB, 0.24887E+00_JPRB, 0.25027E+00_JPRB, 0.25152E+00_JPRB, 0.25323E+00_JPRB, 0.25515E+00_JPRB ,&
&  0.25800E+00_JPRB, 0.26075E+00_JPRB, 0.26357E+00_JPRB, 0.26516E+00_JPRB, 0.26785E+00_JPRB, 0.27012E+00_JPRB, 0.27257E+00_JPRB ,&
&  0.27546E+00_JPRB, 0.28020E+00_JPRB, 0.28722E+00_JPRB, 0.29498E+00_JPRB, 0.30374E+00_JPRB, 0.31066E+00_JPRB, 0.31516E+00_JPRB ,&
&  0.31897E+00_JPRB, 0.32320E+00_JPRB, 0.32847E+00_JPRB, 0.33126E+00_JPRB, 0.33344E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.29997E+00_JPRB, 0.29994E+00_JPRB, 0.30000E+00_JPRB, 0.29963E+00_JPRB, 0.29798E+00_JPRB, 0.29666E+00_JPRB, 0.29545E+00_JPRB ,&
&  0.29475E+00_JPRB, 0.29377E+00_JPRB, 0.29329E+00_JPRB, 0.29304E+00_JPRB, 0.29258E+00_JPRB, 0.29193E+00_JPRB, 0.29108E+00_JPRB ,&
&  0.28997E+00_JPRB, 0.28853E+00_JPRB, 0.28696E+00_JPRB, 0.28653E+00_JPRB, 0.28647E+00_JPRB, 0.28641E+00_JPRB, 0.28628E+00_JPRB ,&
&  0.28604E+00_JPRB, 0.28581E+00_JPRB, 0.28517E+00_JPRB, 0.28524E+00_JPRB, 0.28520E+00_JPRB, 0.28517E+00_JPRB, 0.28525E+00_JPRB ,&
&  0.28542E+00_JPRB, 0.28564E+00_JPRB, 0.28544E+00_JPRB, 0.28498E+00_JPRB, 0.28411E+00_JPRB, 0.28243E+00_JPRB, 0.27976E+00_JPRB ,&
&  0.27671E+00_JPRB, 0.27368E+00_JPRB, 0.27125E+00_JPRB, 0.26902E+00_JPRB, 0.26698E+00_JPRB, 0.26512E+00_JPRB, 0.26359E+00_JPRB ,&
&  0.26175E+00_JPRB, 0.25993E+00_JPRB, 0.25825E+00_JPRB, 0.25704E+00_JPRB, 0.25570E+00_JPRB, 0.25453E+00_JPRB, 0.25343E+00_JPRB ,&
&  0.25266E+00_JPRB, 0.25200E+00_JPRB, 0.25143E+00_JPRB, 0.25114E+00_JPRB, 0.25091E+00_JPRB, 0.25070E+00_JPRB, 0.25046E+00_JPRB ,&
&  0.25026E+00_JPRB, 0.25013E+00_JPRB, 0.25003E+00_JPRB, 0.25004E+00_JPRB, 0.25018E+00_JPRB, 0.25011E+00_JPRB, 0.25001E+00_JPRB ,&
&  0.25000E+00_JPRB, 0.25032E+00_JPRB, 0.25053E+00_JPRB, 0.25086E+00_JPRB, 0.25135E+00_JPRB, 0.25203E+00_JPRB, 0.25277E+00_JPRB ,&
&  0.25352E+00_JPRB, 0.25438E+00_JPRB, 0.25531E+00_JPRB, 0.25645E+00_JPRB, 0.25769E+00_JPRB, 0.25892E+00_JPRB, 0.26010E+00_JPRB ,&
&  0.26137E+00_JPRB, 0.26254E+00_JPRB, 0.26364E+00_JPRB, 0.26440E+00_JPRB, 0.26551E+00_JPRB, 0.26665E+00_JPRB, 0.26800E+00_JPRB ,&
&  0.26954E+00_JPRB, 0.27160E+00_JPRB, 0.27451E+00_JPRB, 0.27798E+00_JPRB, 0.28242E+00_JPRB, 0.28563E+00_JPRB, 0.28886E+00_JPRB ,&
&  0.29114E+00_JPRB, 0.29295E+00_JPRB, 0.29432E+00_JPRB, 0.29605E+00_JPRB, 0.29794E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB, 0.26795E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2070:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_A1B2070
