!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_A1B2080 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS2080* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB, 0.29530E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.30569E+00_JPRB, 0.30528E+00_JPRB, 0.30491E+00_JPRB, 0.30458E+00_JPRB, 0.30428E+00_JPRB, 0.30406E+00_JPRB, 0.30389E+00_JPRB ,&
&  0.30364E+00_JPRB, 0.30341E+00_JPRB, 0.30310E+00_JPRB, 0.30277E+00_JPRB, 0.30249E+00_JPRB, 0.30216E+00_JPRB, 0.30175E+00_JPRB ,&
&  0.30125E+00_JPRB, 0.30081E+00_JPRB, 0.30027E+00_JPRB, 0.29958E+00_JPRB, 0.29888E+00_JPRB, 0.29812E+00_JPRB, 0.29721E+00_JPRB ,&
&  0.29617E+00_JPRB, 0.29515E+00_JPRB, 0.29407E+00_JPRB, 0.29292E+00_JPRB, 0.29170E+00_JPRB, 0.29038E+00_JPRB, 0.28921E+00_JPRB ,&
&  0.28835E+00_JPRB, 0.28741E+00_JPRB, 0.28650E+00_JPRB, 0.28558E+00_JPRB, 0.28485E+00_JPRB, 0.28434E+00_JPRB, 0.28379E+00_JPRB ,&
&  0.28330E+00_JPRB, 0.28291E+00_JPRB, 0.28295E+00_JPRB, 0.28314E+00_JPRB, 0.28344E+00_JPRB, 0.28409E+00_JPRB, 0.28512E+00_JPRB ,&
&  0.28599E+00_JPRB, 0.28677E+00_JPRB, 0.28770E+00_JPRB, 0.28873E+00_JPRB, 0.28982E+00_JPRB, 0.29093E+00_JPRB, 0.29192E+00_JPRB ,&
&  0.29267E+00_JPRB, 0.29329E+00_JPRB, 0.29397E+00_JPRB, 0.29451E+00_JPRB, 0.29482E+00_JPRB, 0.29511E+00_JPRB, 0.29544E+00_JPRB ,&
&  0.29567E+00_JPRB, 0.29581E+00_JPRB, 0.29599E+00_JPRB, 0.29631E+00_JPRB, 0.29659E+00_JPRB, 0.29690E+00_JPRB, 0.29739E+00_JPRB ,&
&  0.29807E+00_JPRB, 0.29856E+00_JPRB, 0.29915E+00_JPRB, 0.29993E+00_JPRB, 0.30096E+00_JPRB, 0.30220E+00_JPRB, 0.30318E+00_JPRB ,&
&  0.30421E+00_JPRB, 0.30530E+00_JPRB, 0.30647E+00_JPRB, 0.30728E+00_JPRB, 0.30817E+00_JPRB, 0.30897E+00_JPRB, 0.30974E+00_JPRB ,&
&  0.30993E+00_JPRB, 0.31026E+00_JPRB, 0.31068E+00_JPRB, 0.31106E+00_JPRB, 0.31073E+00_JPRB, 0.31053E+00_JPRB, 0.31039E+00_JPRB ,&
&  0.31062E+00_JPRB, 0.30999E+00_JPRB, 0.30970E+00_JPRB, 0.30930E+00_JPRB, 0.30911E+00_JPRB, 0.30851E+00_JPRB, 0.30809E+00_JPRB ,&
&  0.30764E+00_JPRB, 0.30730E+00_JPRB, 0.30688E+00_JPRB, 0.30647E+00_JPRB, 0.30604E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.31579E+00_JPRB, 0.31508E+00_JPRB, 0.31428E+00_JPRB, 0.31343E+00_JPRB, 0.31258E+00_JPRB, 0.31196E+00_JPRB, 0.31131E+00_JPRB ,&
&  0.31060E+00_JPRB, 0.31004E+00_JPRB, 0.30929E+00_JPRB, 0.30868E+00_JPRB, 0.30815E+00_JPRB, 0.30798E+00_JPRB, 0.30759E+00_JPRB ,&
&  0.30710E+00_JPRB, 0.30633E+00_JPRB, 0.30565E+00_JPRB, 0.30442E+00_JPRB, 0.30302E+00_JPRB, 0.30113E+00_JPRB, 0.29934E+00_JPRB ,&
&  0.29719E+00_JPRB, 0.29469E+00_JPRB, 0.29249E+00_JPRB, 0.29030E+00_JPRB, 0.28810E+00_JPRB, 0.28585E+00_JPRB, 0.28424E+00_JPRB ,&
&  0.28324E+00_JPRB, 0.28185E+00_JPRB, 0.28065E+00_JPRB, 0.27953E+00_JPRB, 0.27821E+00_JPRB, 0.27732E+00_JPRB, 0.27651E+00_JPRB ,&
&  0.27500E+00_JPRB, 0.27350E+00_JPRB, 0.27354E+00_JPRB, 0.27382E+00_JPRB, 0.27459E+00_JPRB, 0.27621E+00_JPRB, 0.27927E+00_JPRB ,&
&  0.28328E+00_JPRB, 0.28712E+00_JPRB, 0.29119E+00_JPRB, 0.29484E+00_JPRB, 0.29782E+00_JPRB, 0.30013E+00_JPRB, 0.30201E+00_JPRB ,&
&  0.30318E+00_JPRB, 0.30403E+00_JPRB, 0.30450E+00_JPRB, 0.30453E+00_JPRB, 0.30406E+00_JPRB, 0.30390E+00_JPRB, 0.30397E+00_JPRB ,&
&  0.30416E+00_JPRB, 0.30319E+00_JPRB, 0.30289E+00_JPRB, 0.30311E+00_JPRB, 0.30378E+00_JPRB, 0.30439E+00_JPRB, 0.30571E+00_JPRB ,&
&  0.30767E+00_JPRB, 0.31011E+00_JPRB, 0.31235E+00_JPRB, 0.31534E+00_JPRB, 0.31886E+00_JPRB, 0.32293E+00_JPRB, 0.32537E+00_JPRB ,&
&  0.32838E+00_JPRB, 0.33157E+00_JPRB, 0.33494E+00_JPRB, 0.33572E+00_JPRB, 0.33660E+00_JPRB, 0.33725E+00_JPRB, 0.33810E+00_JPRB ,&
&  0.33676E+00_JPRB, 0.33624E+00_JPRB, 0.33589E+00_JPRB, 0.33636E+00_JPRB, 0.33466E+00_JPRB, 0.33384E+00_JPRB, 0.33299E+00_JPRB ,&
&  0.33337E+00_JPRB, 0.33084E+00_JPRB, 0.32943E+00_JPRB, 0.32769E+00_JPRB, 0.32670E+00_JPRB, 0.32426E+00_JPRB, 0.32235E+00_JPRB ,&
&  0.32076E+00_JPRB, 0.31980E+00_JPRB, 0.31857E+00_JPRB, 0.31751E+00_JPRB, 0.31668E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.32508E+00_JPRB, 0.32309E+00_JPRB, 0.32015E+00_JPRB, 0.31642E+00_JPRB, 0.31308E+00_JPRB, 0.31062E+00_JPRB, 0.30805E+00_JPRB ,&
&  0.30601E+00_JPRB, 0.30429E+00_JPRB, 0.30313E+00_JPRB, 0.30232E+00_JPRB, 0.30152E+00_JPRB, 0.30213E+00_JPRB, 0.30285E+00_JPRB ,&
&  0.30345E+00_JPRB, 0.30345E+00_JPRB, 0.30457E+00_JPRB, 0.30439E+00_JPRB, 0.30368E+00_JPRB, 0.30179E+00_JPRB, 0.29899E+00_JPRB ,&
&  0.29565E+00_JPRB, 0.29176E+00_JPRB, 0.28717E+00_JPRB, 0.28191E+00_JPRB, 0.27693E+00_JPRB, 0.25969E+00_JPRB, 0.25554E+00_JPRB ,&
&  0.25629E+00_JPRB, 0.25355E+00_JPRB, 0.25152E+00_JPRB, 0.25057E+00_JPRB, 0.24988E+00_JPRB, 0.24829E+00_JPRB, 0.24646E+00_JPRB ,&
&  0.23326E+00_JPRB, 0.23201E+00_JPRB, 0.23012E+00_JPRB, 0.22901E+00_JPRB, 0.23099E+00_JPRB, 0.23829E+00_JPRB, 0.24996E+00_JPRB ,&
&  0.27095E+00_JPRB, 0.27979E+00_JPRB, 0.28730E+00_JPRB, 0.29620E+00_JPRB, 0.30315E+00_JPRB, 0.30827E+00_JPRB, 0.31321E+00_JPRB ,&
&  0.31727E+00_JPRB, 0.32118E+00_JPRB, 0.32400E+00_JPRB, 0.32546E+00_JPRB, 0.32549E+00_JPRB, 0.32535E+00_JPRB, 0.32511E+00_JPRB ,&
&  0.32495E+00_JPRB, 0.32475E+00_JPRB, 0.32491E+00_JPRB, 0.32481E+00_JPRB, 0.32439E+00_JPRB, 0.32452E+00_JPRB, 0.32660E+00_JPRB ,&
&  0.32984E+00_JPRB, 0.33493E+00_JPRB, 0.33936E+00_JPRB, 0.34575E+00_JPRB, 0.35156E+00_JPRB, 0.35903E+00_JPRB, 0.36194E+00_JPRB ,&
&  0.36559E+00_JPRB, 0.36791E+00_JPRB, 0.37106E+00_JPRB, 0.37099E+00_JPRB, 0.37081E+00_JPRB, 0.37001E+00_JPRB, 0.36996E+00_JPRB ,&
&  0.36787E+00_JPRB, 0.36635E+00_JPRB, 0.36433E+00_JPRB, 0.36390E+00_JPRB, 0.36138E+00_JPRB, 0.36058E+00_JPRB, 0.35886E+00_JPRB ,&
&  0.35935E+00_JPRB, 0.35641E+00_JPRB, 0.35522E+00_JPRB, 0.35174E+00_JPRB, 0.34884E+00_JPRB, 0.34367E+00_JPRB, 0.33925E+00_JPRB ,&
&  0.33553E+00_JPRB, 0.33274E+00_JPRB, 0.33043E+00_JPRB, 0.32848E+00_JPRB, 0.32664E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.33032E+00_JPRB, 0.32481E+00_JPRB, 0.31877E+00_JPRB, 0.31374E+00_JPRB, 0.30917E+00_JPRB, 0.30589E+00_JPRB, 0.30280E+00_JPRB ,&
&  0.30051E+00_JPRB, 0.29837E+00_JPRB, 0.29687E+00_JPRB, 0.29558E+00_JPRB, 0.29470E+00_JPRB, 0.29441E+00_JPRB, 0.29419E+00_JPRB ,&
&  0.29453E+00_JPRB, 0.29468E+00_JPRB, 0.29538E+00_JPRB, 0.29563E+00_JPRB, 0.29620E+00_JPRB, 0.29518E+00_JPRB, 0.29252E+00_JPRB ,&
&  0.28922E+00_JPRB, 0.28539E+00_JPRB, 0.27138E+00_JPRB, 0.26618E+00_JPRB, 0.25455E+00_JPRB, 0.24625E+00_JPRB, 0.24150E+00_JPRB ,&
&  0.23332E+00_JPRB, 0.22909E+00_JPRB, 0.22095E+00_JPRB, 0.21595E+00_JPRB, 0.21420E+00_JPRB, 0.20787E+00_JPRB, 0.20503E+00_JPRB ,&
&  0.20071E+00_JPRB, 0.19341E+00_JPRB, 0.19053E+00_JPRB, 0.18936E+00_JPRB, 0.19188E+00_JPRB, 0.19848E+00_JPRB, 0.21022E+00_JPRB ,&
&  0.23233E+00_JPRB, 0.25690E+00_JPRB, 0.29045E+00_JPRB, 0.30002E+00_JPRB, 0.30169E+00_JPRB, 0.30929E+00_JPRB, 0.31528E+00_JPRB ,&
&  0.31909E+00_JPRB, 0.32751E+00_JPRB, 0.33699E+00_JPRB, 0.34052E+00_JPRB, 0.34312E+00_JPRB, 0.34402E+00_JPRB, 0.34454E+00_JPRB ,&
&  0.34517E+00_JPRB, 0.34855E+00_JPRB, 0.35065E+00_JPRB, 0.35284E+00_JPRB, 0.35500E+00_JPRB, 0.35707E+00_JPRB, 0.36174E+00_JPRB ,&
&  0.36568E+00_JPRB, 0.37481E+00_JPRB, 0.38227E+00_JPRB, 0.39423E+00_JPRB, 0.39907E+00_JPRB, 0.40507E+00_JPRB, 0.40642E+00_JPRB ,&
&  0.40902E+00_JPRB, 0.41035E+00_JPRB, 0.41136E+00_JPRB, 0.40769E+00_JPRB, 0.40683E+00_JPRB, 0.40916E+00_JPRB, 0.40796E+00_JPRB ,&
&  0.40617E+00_JPRB, 0.40463E+00_JPRB, 0.40260E+00_JPRB, 0.40077E+00_JPRB, 0.39934E+00_JPRB, 0.39901E+00_JPRB, 0.39782E+00_JPRB ,&
&  0.39865E+00_JPRB, 0.39646E+00_JPRB, 0.39415E+00_JPRB, 0.38973E+00_JPRB, 0.38406E+00_JPRB, 0.37630E+00_JPRB, 0.36675E+00_JPRB ,&
&  0.35861E+00_JPRB, 0.35241E+00_JPRB, 0.34573E+00_JPRB, 0.34065E+00_JPRB, 0.33534E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.31728E+00_JPRB, 0.31034E+00_JPRB, 0.30471E+00_JPRB, 0.30081E+00_JPRB, 0.29751E+00_JPRB, 0.29550E+00_JPRB, 0.29345E+00_JPRB ,&
&  0.29246E+00_JPRB, 0.29111E+00_JPRB, 0.29054E+00_JPRB, 0.28980E+00_JPRB, 0.28973E+00_JPRB, 0.28998E+00_JPRB, 0.29074E+00_JPRB ,&
&  0.29175E+00_JPRB, 0.29240E+00_JPRB, 0.29328E+00_JPRB, 0.29250E+00_JPRB, 0.29189E+00_JPRB, 0.29051E+00_JPRB, 0.28925E+00_JPRB ,&
&  0.28547E+00_JPRB, 0.28069E+00_JPRB, 0.27068E+00_JPRB, 0.26486E+00_JPRB, 0.25447E+00_JPRB, 0.24710E+00_JPRB, 0.24006E+00_JPRB ,&
&  0.23503E+00_JPRB, 0.22163E+00_JPRB, 0.22464E+00_JPRB, 0.21589E+00_JPRB, 0.20734E+00_JPRB, 0.19889E+00_JPRB, 0.18939E+00_JPRB ,&
&  0.18558E+00_JPRB, 0.17623E+00_JPRB, 0.17283E+00_JPRB, 0.17288E+00_JPRB, 0.17450E+00_JPRB, 0.18162E+00_JPRB, 0.19264E+00_JPRB ,&
&  0.21435E+00_JPRB, 0.26906E+00_JPRB, 0.28744E+00_JPRB, 0.29616E+00_JPRB, 0.29457E+00_JPRB, 0.29577E+00_JPRB, 0.29389E+00_JPRB ,&
&  0.30305E+00_JPRB, 0.31477E+00_JPRB, 0.32731E+00_JPRB, 0.33510E+00_JPRB, 0.34546E+00_JPRB, 0.35117E+00_JPRB, 0.35567E+00_JPRB ,&
&  0.36091E+00_JPRB, 0.36955E+00_JPRB, 0.37719E+00_JPRB, 0.38532E+00_JPRB, 0.39597E+00_JPRB, 0.40696E+00_JPRB, 0.41962E+00_JPRB ,&
&  0.42684E+00_JPRB, 0.44191E+00_JPRB, 0.44677E+00_JPRB, 0.45363E+00_JPRB, 0.45414E+00_JPRB, 0.45270E+00_JPRB, 0.45251E+00_JPRB ,&
&  0.45414E+00_JPRB, 0.45611E+00_JPRB, 0.45750E+00_JPRB, 0.45626E+00_JPRB, 0.45561E+00_JPRB, 0.44911E+00_JPRB, 0.44619E+00_JPRB ,&
&  0.44506E+00_JPRB, 0.44279E+00_JPRB, 0.43990E+00_JPRB, 0.43581E+00_JPRB, 0.43081E+00_JPRB, 0.42508E+00_JPRB, 0.42149E+00_JPRB ,&
&  0.41700E+00_JPRB, 0.41192E+00_JPRB, 0.40603E+00_JPRB, 0.40033E+00_JPRB, 0.39423E+00_JPRB, 0.38791E+00_JPRB, 0.37738E+00_JPRB ,&
&  0.36407E+00_JPRB, 0.35276E+00_JPRB, 0.34202E+00_JPRB, 0.33365E+00_JPRB, 0.32503E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.30685E+00_JPRB, 0.30080E+00_JPRB, 0.29596E+00_JPRB, 0.29363E+00_JPRB, 0.29117E+00_JPRB, 0.29031E+00_JPRB, 0.28865E+00_JPRB ,&
&  0.28779E+00_JPRB, 0.28671E+00_JPRB, 0.28619E+00_JPRB, 0.28553E+00_JPRB, 0.28542E+00_JPRB, 0.28539E+00_JPRB, 0.28605E+00_JPRB ,&
&  0.28688E+00_JPRB, 0.28845E+00_JPRB, 0.29013E+00_JPRB, 0.29037E+00_JPRB, 0.29064E+00_JPRB, 0.29003E+00_JPRB, 0.28898E+00_JPRB ,&
&  0.28586E+00_JPRB, 0.28130E+00_JPRB, 0.27160E+00_JPRB, 0.26433E+00_JPRB, 0.25799E+00_JPRB, 0.24935E+00_JPRB, 0.24777E+00_JPRB ,&
&  0.24534E+00_JPRB, 0.23924E+00_JPRB, 0.22993E+00_JPRB, 0.22632E+00_JPRB, 0.22052E+00_JPRB, 0.19868E+00_JPRB, 0.19822E+00_JPRB ,&
&  0.19157E+00_JPRB, 0.17752E+00_JPRB, 0.17329E+00_JPRB, 0.17038E+00_JPRB, 0.17156E+00_JPRB, 0.17778E+00_JPRB, 0.18772E+00_JPRB ,&
&  0.20890E+00_JPRB, 0.27337E+00_JPRB, 0.28907E+00_JPRB, 0.29560E+00_JPRB, 0.29078E+00_JPRB, 0.29434E+00_JPRB, 0.29540E+00_JPRB ,&
&  0.30244E+00_JPRB, 0.30840E+00_JPRB, 0.31609E+00_JPRB, 0.32222E+00_JPRB, 0.32901E+00_JPRB, 0.33173E+00_JPRB, 0.33851E+00_JPRB ,&
&  0.35126E+00_JPRB, 0.36643E+00_JPRB, 0.37974E+00_JPRB, 0.39750E+00_JPRB, 0.42157E+00_JPRB, 0.46582E+00_JPRB, 0.51312E+00_JPRB ,&
&  0.51709E+00_JPRB, 0.51191E+00_JPRB, 0.50910E+00_JPRB, 0.50380E+00_JPRB, 0.50220E+00_JPRB, 0.50125E+00_JPRB, 0.50369E+00_JPRB ,&
&  0.51006E+00_JPRB, 0.51602E+00_JPRB, 0.50732E+00_JPRB, 0.48868E+00_JPRB, 0.48057E+00_JPRB, 0.46708E+00_JPRB, 0.46393E+00_JPRB ,&
&  0.45968E+00_JPRB, 0.46505E+00_JPRB, 0.46018E+00_JPRB, 0.45235E+00_JPRB, 0.44496E+00_JPRB, 0.43681E+00_JPRB, 0.43109E+00_JPRB ,&
&  0.42435E+00_JPRB, 0.41645E+00_JPRB, 0.40832E+00_JPRB, 0.40192E+00_JPRB, 0.39637E+00_JPRB, 0.39191E+00_JPRB, 0.38484E+00_JPRB ,&
&  0.37057E+00_JPRB, 0.35240E+00_JPRB, 0.33847E+00_JPRB, 0.32529E+00_JPRB, 0.31471E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.30231E+00_JPRB, 0.29898E+00_JPRB, 0.29721E+00_JPRB, 0.29768E+00_JPRB, 0.29753E+00_JPRB, 0.29786E+00_JPRB, 0.29596E+00_JPRB ,&
&  0.29408E+00_JPRB, 0.29202E+00_JPRB, 0.29016E+00_JPRB, 0.28830E+00_JPRB, 0.28703E+00_JPRB, 0.28567E+00_JPRB, 0.28469E+00_JPRB ,&
&  0.28380E+00_JPRB, 0.27997E+00_JPRB, 0.27866E+00_JPRB, 0.28024E+00_JPRB, 0.28250E+00_JPRB, 0.28437E+00_JPRB, 0.28668E+00_JPRB ,&
&  0.28499E+00_JPRB, 0.28086E+00_JPRB, 0.27493E+00_JPRB, 0.26693E+00_JPRB, 0.26057E+00_JPRB, 0.25782E+00_JPRB, 0.25366E+00_JPRB ,&
&  0.24592E+00_JPRB, 0.23944E+00_JPRB, 0.24275E+00_JPRB, 0.24086E+00_JPRB, 0.22772E+00_JPRB, 0.22166E+00_JPRB, 0.21097E+00_JPRB ,&
&  0.19741E+00_JPRB, 0.18003E+00_JPRB, 0.17589E+00_JPRB, 0.17412E+00_JPRB, 0.17812E+00_JPRB, 0.18514E+00_JPRB, 0.20058E+00_JPRB ,&
&  0.22739E+00_JPRB, 0.27852E+00_JPRB, 0.29260E+00_JPRB, 0.30069E+00_JPRB, 0.30516E+00_JPRB, 0.30651E+00_JPRB, 0.30765E+00_JPRB ,&
&  0.31298E+00_JPRB, 0.31742E+00_JPRB, 0.32428E+00_JPRB, 0.33194E+00_JPRB, 0.34197E+00_JPRB, 0.34999E+00_JPRB, 0.35497E+00_JPRB ,&
&  0.37493E+00_JPRB, 0.38285E+00_JPRB, 0.38908E+00_JPRB, 0.41931E+00_JPRB, 0.48053E+00_JPRB, 0.54763E+00_JPRB, 0.57893E+00_JPRB ,&
&  0.57627E+00_JPRB, 0.56991E+00_JPRB, 0.54844E+00_JPRB, 0.53550E+00_JPRB, 0.52948E+00_JPRB, 0.52818E+00_JPRB, 0.52986E+00_JPRB ,&
&  0.53518E+00_JPRB, 0.54683E+00_JPRB, 0.58807E+00_JPRB, 0.55645E+00_JPRB, 0.53712E+00_JPRB, 0.51812E+00_JPRB, 0.49665E+00_JPRB ,&
&  0.47991E+00_JPRB, 0.47039E+00_JPRB, 0.46400E+00_JPRB, 0.45573E+00_JPRB, 0.44769E+00_JPRB, 0.44085E+00_JPRB, 0.43827E+00_JPRB ,&
&  0.43162E+00_JPRB, 0.42265E+00_JPRB, 0.41074E+00_JPRB, 0.40452E+00_JPRB, 0.39877E+00_JPRB, 0.39315E+00_JPRB, 0.38705E+00_JPRB ,&
&  0.37230E+00_JPRB, 0.35198E+00_JPRB, 0.33580E+00_JPRB, 0.32147E+00_JPRB, 0.31008E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.30560E+00_JPRB, 0.30460E+00_JPRB, 0.30659E+00_JPRB, 0.31140E+00_JPRB, 0.31538E+00_JPRB, 0.31107E+00_JPRB, 0.30574E+00_JPRB ,&
&  0.30211E+00_JPRB, 0.30083E+00_JPRB, 0.30034E+00_JPRB, 0.29809E+00_JPRB, 0.29623E+00_JPRB, 0.29421E+00_JPRB, 0.29207E+00_JPRB ,&
&  0.28972E+00_JPRB, 0.28708E+00_JPRB, 0.28397E+00_JPRB, 0.27956E+00_JPRB, 0.27728E+00_JPRB, 0.27827E+00_JPRB, 0.28135E+00_JPRB ,&
&  0.28424E+00_JPRB, 0.28380E+00_JPRB, 0.28124E+00_JPRB, 0.27446E+00_JPRB, 0.26669E+00_JPRB, 0.26050E+00_JPRB, 0.25984E+00_JPRB ,&
&  0.25605E+00_JPRB, 0.24926E+00_JPRB, 0.24224E+00_JPRB, 0.24581E+00_JPRB, 0.24718E+00_JPRB, 0.23382E+00_JPRB, 0.21976E+00_JPRB ,&
&  0.20533E+00_JPRB, 0.19236E+00_JPRB, 0.18641E+00_JPRB, 0.18615E+00_JPRB, 0.19318E+00_JPRB, 0.20666E+00_JPRB, 0.22576E+00_JPRB ,&
&  0.25573E+00_JPRB, 0.28157E+00_JPRB, 0.29424E+00_JPRB, 0.30456E+00_JPRB, 0.30698E+00_JPRB, 0.30658E+00_JPRB, 0.30614E+00_JPRB ,&
&  0.30819E+00_JPRB, 0.31128E+00_JPRB, 0.31652E+00_JPRB, 0.32100E+00_JPRB, 0.33158E+00_JPRB, 0.34947E+00_JPRB, 0.38549E+00_JPRB ,&
&  0.48229E+00_JPRB, 0.48251E+00_JPRB, 0.43172E+00_JPRB, 0.46449E+00_JPRB, 0.52895E+00_JPRB, 0.58907E+00_JPRB, 0.63490E+00_JPRB ,&
&  0.64245E+00_JPRB, 0.61018E+00_JPRB, 0.58154E+00_JPRB, 0.56573E+00_JPRB, 0.55500E+00_JPRB, 0.54739E+00_JPRB, 0.54538E+00_JPRB ,&
&  0.55026E+00_JPRB, 0.56487E+00_JPRB, 0.64069E+00_JPRB, 0.56022E+00_JPRB, 0.51439E+00_JPRB, 0.49610E+00_JPRB, 0.47661E+00_JPRB ,&
&  0.46035E+00_JPRB, 0.45216E+00_JPRB, 0.45244E+00_JPRB, 0.45522E+00_JPRB, 0.43714E+00_JPRB, 0.42084E+00_JPRB, 0.40873E+00_JPRB ,&
&  0.40392E+00_JPRB, 0.39698E+00_JPRB, 0.39240E+00_JPRB, 0.39387E+00_JPRB, 0.39143E+00_JPRB, 0.38201E+00_JPRB, 0.37639E+00_JPRB ,&
&  0.36657E+00_JPRB, 0.34887E+00_JPRB, 0.33675E+00_JPRB, 0.32499E+00_JPRB, 0.31285E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.30387E+00_JPRB, 0.31205E+00_JPRB, 0.32359E+00_JPRB, 0.33182E+00_JPRB, 0.33453E+00_JPRB, 0.32000E+00_JPRB, 0.30517E+00_JPRB ,&
&  0.29717E+00_JPRB, 0.29401E+00_JPRB, 0.29495E+00_JPRB, 0.29549E+00_JPRB, 0.29548E+00_JPRB, 0.29561E+00_JPRB, 0.29526E+00_JPRB ,&
&  0.29237E+00_JPRB, 0.28816E+00_JPRB, 0.28416E+00_JPRB, 0.28029E+00_JPRB, 0.27728E+00_JPRB, 0.27653E+00_JPRB, 0.27765E+00_JPRB ,&
&  0.28002E+00_JPRB, 0.28112E+00_JPRB, 0.28047E+00_JPRB, 0.27636E+00_JPRB, 0.27292E+00_JPRB, 0.27008E+00_JPRB, 0.26507E+00_JPRB ,&
&  0.25665E+00_JPRB, 0.25580E+00_JPRB, 0.25376E+00_JPRB, 0.25148E+00_JPRB, 0.25336E+00_JPRB, 0.24485E+00_JPRB, 0.23034E+00_JPRB ,&
&  0.21536E+00_JPRB, 0.20434E+00_JPRB, 0.20229E+00_JPRB, 0.20934E+00_JPRB, 0.22600E+00_JPRB, 0.25103E+00_JPRB, 0.26375E+00_JPRB ,&
&  0.27513E+00_JPRB, 0.28528E+00_JPRB, 0.29198E+00_JPRB, 0.29844E+00_JPRB, 0.30334E+00_JPRB, 0.30570E+00_JPRB, 0.30638E+00_JPRB ,&
&  0.30590E+00_JPRB, 0.30618E+00_JPRB, 0.30804E+00_JPRB, 0.29472E+00_JPRB, 0.30449E+00_JPRB, 0.36314E+00_JPRB, 0.41075E+00_JPRB ,&
&  0.46916E+00_JPRB, 0.47881E+00_JPRB, 0.48797E+00_JPRB, 0.53036E+00_JPRB, 0.57474E+00_JPRB, 0.61353E+00_JPRB, 0.64699E+00_JPRB ,&
&  0.65821E+00_JPRB, 0.62291E+00_JPRB, 0.58696E+00_JPRB, 0.57445E+00_JPRB, 0.56971E+00_JPRB, 0.56393E+00_JPRB, 0.54373E+00_JPRB ,&
&  0.52862E+00_JPRB, 0.50239E+00_JPRB, 0.46981E+00_JPRB, 0.44127E+00_JPRB, 0.43697E+00_JPRB, 0.44885E+00_JPRB, 0.46615E+00_JPRB ,&
&  0.46630E+00_JPRB, 0.46386E+00_JPRB, 0.46281E+00_JPRB, 0.46614E+00_JPRB, 0.42746E+00_JPRB, 0.40205E+00_JPRB, 0.38418E+00_JPRB ,&
&  0.37558E+00_JPRB, 0.37039E+00_JPRB, 0.36694E+00_JPRB, 0.37319E+00_JPRB, 0.37309E+00_JPRB, 0.36877E+00_JPRB, 0.35522E+00_JPRB ,&
&  0.33723E+00_JPRB, 0.32286E+00_JPRB, 0.31263E+00_JPRB, 0.30579E+00_JPRB, 0.30338E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.33813E+00_JPRB, 0.34505E+00_JPRB, 0.34892E+00_JPRB, 0.35210E+00_JPRB, 0.34868E+00_JPRB, 0.34202E+00_JPRB, 0.33457E+00_JPRB ,&
&  0.33511E+00_JPRB, 0.32692E+00_JPRB, 0.31941E+00_JPRB, 0.30813E+00_JPRB, 0.29851E+00_JPRB, 0.29021E+00_JPRB, 0.28872E+00_JPRB ,&
&  0.29056E+00_JPRB, 0.29264E+00_JPRB, 0.29118E+00_JPRB, 0.28628E+00_JPRB, 0.28122E+00_JPRB, 0.27831E+00_JPRB, 0.27818E+00_JPRB ,&
&  0.27951E+00_JPRB, 0.28148E+00_JPRB, 0.28198E+00_JPRB, 0.28054E+00_JPRB, 0.27876E+00_JPRB, 0.27642E+00_JPRB, 0.27321E+00_JPRB ,&
&  0.26741E+00_JPRB, 0.26395E+00_JPRB, 0.26216E+00_JPRB, 0.26162E+00_JPRB, 0.26388E+00_JPRB, 0.25585E+00_JPRB, 0.24173E+00_JPRB ,&
&  0.22698E+00_JPRB, 0.22230E+00_JPRB, 0.23111E+00_JPRB, 0.25896E+00_JPRB, 0.27535E+00_JPRB, 0.28866E+00_JPRB, 0.29968E+00_JPRB ,&
&  0.29878E+00_JPRB, 0.29653E+00_JPRB, 0.30106E+00_JPRB, 0.31462E+00_JPRB, 0.31726E+00_JPRB, 0.31798E+00_JPRB, 0.31605E+00_JPRB ,&
&  0.31305E+00_JPRB, 0.31277E+00_JPRB, 0.31344E+00_JPRB, 0.30576E+00_JPRB, 0.36016E+00_JPRB, 0.45848E+00_JPRB, 0.46999E+00_JPRB ,&
&  0.51501E+00_JPRB, 0.54429E+00_JPRB, 0.58800E+00_JPRB, 0.59827E+00_JPRB, 0.58486E+00_JPRB, 0.60203E+00_JPRB, 0.65204E+00_JPRB ,&
&  0.66416E+00_JPRB, 0.64501E+00_JPRB, 0.61793E+00_JPRB, 0.60782E+00_JPRB, 0.58208E+00_JPRB, 0.55589E+00_JPRB, 0.53226E+00_JPRB ,&
&  0.52627E+00_JPRB, 0.51289E+00_JPRB, 0.48807E+00_JPRB, 0.45962E+00_JPRB, 0.44936E+00_JPRB, 0.44867E+00_JPRB, 0.45714E+00_JPRB ,&
&  0.46035E+00_JPRB, 0.46717E+00_JPRB, 0.47144E+00_JPRB, 0.47539E+00_JPRB, 0.44055E+00_JPRB, 0.39998E+00_JPRB, 0.37378E+00_JPRB ,&
&  0.35635E+00_JPRB, 0.34403E+00_JPRB, 0.33309E+00_JPRB, 0.33766E+00_JPRB, 0.34602E+00_JPRB, 0.34663E+00_JPRB, 0.33116E+00_JPRB ,&
&  0.31899E+00_JPRB, 0.31326E+00_JPRB, 0.31143E+00_JPRB, 0.32036E+00_JPRB, 0.33039E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.35999E+00_JPRB, 0.36845E+00_JPRB, 0.36907E+00_JPRB, 0.36771E+00_JPRB, 0.36951E+00_JPRB, 0.36389E+00_JPRB, 0.35079E+00_JPRB ,&
&  0.34995E+00_JPRB, 0.33093E+00_JPRB, 0.32234E+00_JPRB, 0.31385E+00_JPRB, 0.30769E+00_JPRB, 0.30016E+00_JPRB, 0.28933E+00_JPRB ,&
&  0.28622E+00_JPRB, 0.28923E+00_JPRB, 0.29454E+00_JPRB, 0.29427E+00_JPRB, 0.29062E+00_JPRB, 0.28621E+00_JPRB, 0.28395E+00_JPRB ,&
&  0.28381E+00_JPRB, 0.28551E+00_JPRB, 0.28813E+00_JPRB, 0.28662E+00_JPRB, 0.28315E+00_JPRB, 0.28021E+00_JPRB, 0.28004E+00_JPRB ,&
&  0.27691E+00_JPRB, 0.27210E+00_JPRB, 0.27241E+00_JPRB, 0.27244E+00_JPRB, 0.27514E+00_JPRB, 0.27691E+00_JPRB, 0.26707E+00_JPRB ,&
&  0.25006E+00_JPRB, 0.25552E+00_JPRB, 0.28423E+00_JPRB, 0.30246E+00_JPRB, 0.31913E+00_JPRB, 0.32661E+00_JPRB, 0.33193E+00_JPRB ,&
&  0.33626E+00_JPRB, 0.34137E+00_JPRB, 0.34398E+00_JPRB, 0.34648E+00_JPRB, 0.34675E+00_JPRB, 0.34497E+00_JPRB, 0.34140E+00_JPRB ,&
&  0.33921E+00_JPRB, 0.31424E+00_JPRB, 0.30810E+00_JPRB, 0.32882E+00_JPRB, 0.47047E+00_JPRB, 0.48606E+00_JPRB, 0.49733E+00_JPRB ,&
&  0.54597E+00_JPRB, 0.57310E+00_JPRB, 0.61613E+00_JPRB, 0.64351E+00_JPRB, 0.64145E+00_JPRB, 0.64362E+00_JPRB, 0.68272E+00_JPRB ,&
&  0.71143E+00_JPRB, 0.70571E+00_JPRB, 0.68159E+00_JPRB, 0.64987E+00_JPRB, 0.60488E+00_JPRB, 0.58552E+00_JPRB, 0.57349E+00_JPRB ,&
&  0.57565E+00_JPRB, 0.57132E+00_JPRB, 0.54469E+00_JPRB, 0.51229E+00_JPRB, 0.48977E+00_JPRB, 0.49037E+00_JPRB, 0.49415E+00_JPRB ,&
&  0.49518E+00_JPRB, 0.50178E+00_JPRB, 0.49807E+00_JPRB, 0.48942E+00_JPRB, 0.46617E+00_JPRB, 0.44250E+00_JPRB, 0.42848E+00_JPRB ,&
&  0.39448E+00_JPRB, 0.36101E+00_JPRB, 0.33554E+00_JPRB, 0.33440E+00_JPRB, 0.34269E+00_JPRB, 0.34279E+00_JPRB, 0.33305E+00_JPRB ,&
&  0.33102E+00_JPRB, 0.32817E+00_JPRB, 0.32710E+00_JPRB, 0.33756E+00_JPRB, 0.34708E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.36681E+00_JPRB, 0.37413E+00_JPRB, 0.37514E+00_JPRB, 0.37410E+00_JPRB, 0.38046E+00_JPRB, 0.37579E+00_JPRB, 0.37629E+00_JPRB ,&
&  0.39437E+00_JPRB, 0.38434E+00_JPRB, 0.40027E+00_JPRB, 0.34920E+00_JPRB, 0.32117E+00_JPRB, 0.31108E+00_JPRB, 0.29952E+00_JPRB ,&
&  0.29617E+00_JPRB, 0.29996E+00_JPRB, 0.30180E+00_JPRB, 0.30133E+00_JPRB, 0.30033E+00_JPRB, 0.29412E+00_JPRB, 0.29086E+00_JPRB ,&
&  0.29086E+00_JPRB, 0.29328E+00_JPRB, 0.29665E+00_JPRB, 0.29480E+00_JPRB, 0.29172E+00_JPRB, 0.28928E+00_JPRB, 0.28691E+00_JPRB ,&
&  0.28296E+00_JPRB, 0.28326E+00_JPRB, 0.28483E+00_JPRB, 0.28390E+00_JPRB, 0.28555E+00_JPRB, 0.28985E+00_JPRB, 0.29093E+00_JPRB ,&
&  0.29855E+00_JPRB, 0.30010E+00_JPRB, 0.32714E+00_JPRB, 0.34628E+00_JPRB, 0.35733E+00_JPRB, 0.36078E+00_JPRB, 0.36382E+00_JPRB ,&
&  0.37036E+00_JPRB, 0.37917E+00_JPRB, 0.38871E+00_JPRB, 0.39618E+00_JPRB, 0.39694E+00_JPRB, 0.39514E+00_JPRB, 0.39004E+00_JPRB ,&
&  0.38664E+00_JPRB, 0.33932E+00_JPRB, 0.38378E+00_JPRB, 0.40932E+00_JPRB, 0.51358E+00_JPRB, 0.62562E+00_JPRB, 0.61093E+00_JPRB ,&
&  0.72326E+00_JPRB, 0.68629E+00_JPRB, 0.70288E+00_JPRB, 0.69803E+00_JPRB, 0.69355E+00_JPRB, 0.70995E+00_JPRB, 0.74651E+00_JPRB ,&
&  0.81482E+00_JPRB, 0.78449E+00_JPRB, 0.78015E+00_JPRB, 0.71401E+00_JPRB, 0.67054E+00_JPRB, 0.66501E+00_JPRB, 0.65523E+00_JPRB ,&
&  0.65523E+00_JPRB, 0.65396E+00_JPRB, 0.64080E+00_JPRB, 0.57519E+00_JPRB, 0.55817E+00_JPRB, 0.55183E+00_JPRB, 0.52131E+00_JPRB ,&
&  0.52980E+00_JPRB, 0.52486E+00_JPRB, 0.50010E+00_JPRB, 0.51757E+00_JPRB, 0.48057E+00_JPRB, 0.45745E+00_JPRB, 0.45242E+00_JPRB ,&
&  0.43650E+00_JPRB, 0.38774E+00_JPRB, 0.36794E+00_JPRB, 0.36274E+00_JPRB, 0.35394E+00_JPRB, 0.34624E+00_JPRB, 0.34723E+00_JPRB ,&
&  0.33814E+00_JPRB, 0.33993E+00_JPRB, 0.35217E+00_JPRB, 0.35478E+00_JPRB, 0.35830E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.37543E+00_JPRB, 0.37687E+00_JPRB, 0.37845E+00_JPRB, 0.38087E+00_JPRB, 0.39479E+00_JPRB, 0.40144E+00_JPRB, 0.40154E+00_JPRB ,&
&  0.41653E+00_JPRB, 0.41474E+00_JPRB, 0.41439E+00_JPRB, 0.40966E+00_JPRB, 0.40199E+00_JPRB, 0.36695E+00_JPRB, 0.31637E+00_JPRB ,&
&  0.29948E+00_JPRB, 0.30123E+00_JPRB, 0.30745E+00_JPRB, 0.30884E+00_JPRB, 0.30525E+00_JPRB, 0.29988E+00_JPRB, 0.29674E+00_JPRB ,&
&  0.29843E+00_JPRB, 0.30093E+00_JPRB, 0.30444E+00_JPRB, 0.30519E+00_JPRB, 0.30293E+00_JPRB, 0.30168E+00_JPRB, 0.30020E+00_JPRB ,&
&  0.29489E+00_JPRB, 0.29510E+00_JPRB, 0.29639E+00_JPRB, 0.29294E+00_JPRB, 0.30469E+00_JPRB, 0.31394E+00_JPRB, 0.31762E+00_JPRB ,&
&  0.33528E+00_JPRB, 0.35900E+00_JPRB, 0.37620E+00_JPRB, 0.39250E+00_JPRB, 0.39676E+00_JPRB, 0.39959E+00_JPRB, 0.40867E+00_JPRB ,&
&  0.42413E+00_JPRB, 0.43886E+00_JPRB, 0.45276E+00_JPRB, 0.46374E+00_JPRB, 0.46863E+00_JPRB, 0.45753E+00_JPRB, 0.47191E+00_JPRB ,&
&  0.47420E+00_JPRB, 0.49928E+00_JPRB, 0.55544E+00_JPRB, 0.55249E+00_JPRB, 0.64330E+00_JPRB, 0.70800E+00_JPRB, 0.69088E+00_JPRB ,&
&  0.73306E+00_JPRB, 0.70237E+00_JPRB, 0.77936E+00_JPRB, 0.83159E+00_JPRB, 0.87910E+00_JPRB, 0.89632E+00_JPRB, 0.92799E+00_JPRB ,&
&  0.10123E+01_JPRB, 0.10770E+01_JPRB, 0.92784E+00_JPRB, 0.85692E+00_JPRB, 0.79883E+00_JPRB, 0.79434E+00_JPRB, 0.75723E+00_JPRB ,&
&  0.75930E+00_JPRB, 0.77581E+00_JPRB, 0.76012E+00_JPRB, 0.73571E+00_JPRB, 0.68865E+00_JPRB, 0.64130E+00_JPRB, 0.59674E+00_JPRB ,&
&  0.56935E+00_JPRB, 0.51598E+00_JPRB, 0.48907E+00_JPRB, 0.48906E+00_JPRB, 0.48676E+00_JPRB, 0.44768E+00_JPRB, 0.45492E+00_JPRB ,&
&  0.46346E+00_JPRB, 0.43014E+00_JPRB, 0.40375E+00_JPRB, 0.38224E+00_JPRB, 0.37497E+00_JPRB, 0.36457E+00_JPRB, 0.34556E+00_JPRB ,&
&  0.33285E+00_JPRB, 0.35049E+00_JPRB, 0.36242E+00_JPRB, 0.36685E+00_JPRB, 0.37184E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.38863E+00_JPRB, 0.38566E+00_JPRB, 0.38267E+00_JPRB, 0.38894E+00_JPRB, 0.40483E+00_JPRB, 0.41120E+00_JPRB, 0.42229E+00_JPRB ,&
&  0.42925E+00_JPRB, 0.42974E+00_JPRB, 0.43115E+00_JPRB, 0.42240E+00_JPRB, 0.41688E+00_JPRB, 0.41199E+00_JPRB, 0.38187E+00_JPRB ,&
&  0.34196E+00_JPRB, 0.33003E+00_JPRB, 0.31903E+00_JPRB, 0.31696E+00_JPRB, 0.31603E+00_JPRB, 0.31016E+00_JPRB, 0.30639E+00_JPRB ,&
&  0.30935E+00_JPRB, 0.30975E+00_JPRB, 0.31076E+00_JPRB, 0.31164E+00_JPRB, 0.31182E+00_JPRB, 0.31362E+00_JPRB, 0.31351E+00_JPRB ,&
&  0.30761E+00_JPRB, 0.30895E+00_JPRB, 0.31299E+00_JPRB, 0.31149E+00_JPRB, 0.32828E+00_JPRB, 0.34831E+00_JPRB, 0.36431E+00_JPRB ,&
&  0.38911E+00_JPRB, 0.41111E+00_JPRB, 0.42506E+00_JPRB, 0.43612E+00_JPRB, 0.43966E+00_JPRB, 0.45228E+00_JPRB, 0.47314E+00_JPRB ,&
&  0.49894E+00_JPRB, 0.51774E+00_JPRB, 0.53311E+00_JPRB, 0.54988E+00_JPRB, 0.54575E+00_JPRB, 0.55523E+00_JPRB, 0.59688E+00_JPRB ,&
&  0.65115E+00_JPRB, 0.72353E+00_JPRB, 0.75135E+00_JPRB, 0.78734E+00_JPRB, 0.86282E+00_JPRB, 0.87110E+00_JPRB, 0.81259E+00_JPRB ,&
&  0.82610E+00_JPRB, 0.78480E+00_JPRB, 0.10421E+01_JPRB, 0.10579E+01_JPRB, 0.10344E+01_JPRB, 0.10789E+01_JPRB, 0.10574E+01_JPRB ,&
&  0.10638E+01_JPRB, 0.10196E+01_JPRB, 0.95716E+00_JPRB, 0.92681E+00_JPRB, 0.90454E+00_JPRB, 0.95233E+00_JPRB, 0.90302E+00_JPRB ,&
&  0.91444E+00_JPRB, 0.90779E+00_JPRB, 0.77994E+00_JPRB, 0.68197E+00_JPRB, 0.58937E+00_JPRB, 0.61388E+00_JPRB, 0.58999E+00_JPRB ,&
&  0.56081E+00_JPRB, 0.48667E+00_JPRB, 0.50594E+00_JPRB, 0.50989E+00_JPRB, 0.51612E+00_JPRB, 0.52974E+00_JPRB, 0.52571E+00_JPRB ,&
&  0.52117E+00_JPRB, 0.51919E+00_JPRB, 0.46160E+00_JPRB, 0.42267E+00_JPRB, 0.40558E+00_JPRB, 0.38621E+00_JPRB, 0.35726E+00_JPRB ,&
&  0.35038E+00_JPRB, 0.37026E+00_JPRB, 0.38400E+00_JPRB, 0.38986E+00_JPRB, 0.39140E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.40756E+00_JPRB, 0.40201E+00_JPRB, 0.39552E+00_JPRB, 0.40411E+00_JPRB, 0.41675E+00_JPRB, 0.42142E+00_JPRB, 0.43242E+00_JPRB ,&
&  0.44370E+00_JPRB, 0.44537E+00_JPRB, 0.43848E+00_JPRB, 0.43115E+00_JPRB, 0.42669E+00_JPRB, 0.42321E+00_JPRB, 0.42271E+00_JPRB ,&
&  0.37817E+00_JPRB, 0.35296E+00_JPRB, 0.32748E+00_JPRB, 0.32430E+00_JPRB, 0.34205E+00_JPRB, 0.33137E+00_JPRB, 0.32873E+00_JPRB ,&
&  0.32479E+00_JPRB, 0.32558E+00_JPRB, 0.32130E+00_JPRB, 0.31897E+00_JPRB, 0.31891E+00_JPRB, 0.32212E+00_JPRB, 0.32695E+00_JPRB ,&
&  0.32540E+00_JPRB, 0.33449E+00_JPRB, 0.34665E+00_JPRB, 0.34860E+00_JPRB, 0.36962E+00_JPRB, 0.39123E+00_JPRB, 0.42104E+00_JPRB ,&
&  0.45042E+00_JPRB, 0.46243E+00_JPRB, 0.46975E+00_JPRB, 0.48393E+00_JPRB, 0.50041E+00_JPRB, 0.53051E+00_JPRB, 0.56637E+00_JPRB ,&
&  0.60420E+00_JPRB, 0.63341E+00_JPRB, 0.65269E+00_JPRB, 0.66330E+00_JPRB, 0.65270E+00_JPRB, 0.68484E+00_JPRB, 0.75411E+00_JPRB ,&
&  0.87034E+00_JPRB, 0.91342E+00_JPRB, 0.92179E+00_JPRB, 0.11152E+01_JPRB, 0.12330E+01_JPRB, 0.11988E+01_JPRB, 0.10541E+01_JPRB ,&
&  0.97044E+00_JPRB, 0.89391E+00_JPRB, 0.98597E+00_JPRB, 0.10468E+01_JPRB, 0.10193E+01_JPRB, 0.11104E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.97794E+00_JPRB, 0.90633E+00_JPRB, 0.87133E+00_JPRB, 0.91804E+00_JPRB, 0.91481E+00_JPRB, 0.97928E+00_JPRB, 0.97406E+00_JPRB ,&
&  0.97679E+00_JPRB, 0.83867E+00_JPRB, 0.63019E+00_JPRB, 0.57948E+00_JPRB, 0.46785E+00_JPRB, 0.47582E+00_JPRB, 0.57483E+00_JPRB ,&
&  0.57408E+00_JPRB, 0.57412E+00_JPRB, 0.56655E+00_JPRB, 0.56833E+00_JPRB, 0.57171E+00_JPRB, 0.65731E+00_JPRB, 0.67869E+00_JPRB ,&
&  0.59748E+00_JPRB, 0.59177E+00_JPRB, 0.56034E+00_JPRB, 0.52131E+00_JPRB, 0.48558E+00_JPRB, 0.44246E+00_JPRB, 0.41075E+00_JPRB ,&
&  0.40481E+00_JPRB, 0.40553E+00_JPRB, 0.41248E+00_JPRB, 0.41230E+00_JPRB, 0.40894E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.43531E+00_JPRB, 0.43034E+00_JPRB, 0.42071E+00_JPRB, 0.42411E+00_JPRB, 0.43080E+00_JPRB, 0.43723E+00_JPRB, 0.44868E+00_JPRB ,&
&  0.46048E+00_JPRB, 0.45737E+00_JPRB, 0.44640E+00_JPRB, 0.44211E+00_JPRB, 0.43841E+00_JPRB, 0.43185E+00_JPRB, 0.43063E+00_JPRB ,&
&  0.43614E+00_JPRB, 0.40975E+00_JPRB, 0.36992E+00_JPRB, 0.33880E+00_JPRB, 0.34797E+00_JPRB, 0.34652E+00_JPRB, 0.35427E+00_JPRB ,&
&  0.34940E+00_JPRB, 0.35845E+00_JPRB, 0.34923E+00_JPRB, 0.34402E+00_JPRB, 0.34104E+00_JPRB, 0.34570E+00_JPRB, 0.35814E+00_JPRB ,&
&  0.37514E+00_JPRB, 0.40438E+00_JPRB, 0.43301E+00_JPRB, 0.43963E+00_JPRB, 0.46464E+00_JPRB, 0.45700E+00_JPRB, 0.48894E+00_JPRB ,&
&  0.51490E+00_JPRB, 0.52132E+00_JPRB, 0.52866E+00_JPRB, 0.55265E+00_JPRB, 0.59035E+00_JPRB, 0.63590E+00_JPRB, 0.69339E+00_JPRB ,&
&  0.74791E+00_JPRB, 0.76977E+00_JPRB, 0.74962E+00_JPRB, 0.74080E+00_JPRB, 0.77373E+00_JPRB, 0.84664E+00_JPRB, 0.85550E+00_JPRB ,&
&  0.91580E+00_JPRB, 0.93371E+00_JPRB, 0.89434E+00_JPRB, 0.11590E+01_JPRB, 0.13827E+01_JPRB, 0.13934E+01_JPRB, 0.12837E+01_JPRB ,&
&  0.11890E+01_JPRB, 0.11023E+01_JPRB, 0.11593E+01_JPRB, 0.11683E+01_JPRB, 0.11495E+01_JPRB, 0.11327E+01_JPRB, 0.10564E+01_JPRB ,&
&  0.93660E+00_JPRB, 0.87819E+00_JPRB, 0.89521E+00_JPRB, 0.91206E+00_JPRB, 0.93557E+00_JPRB, 0.90816E+00_JPRB, 0.93515E+00_JPRB ,&
&  0.97862E+00_JPRB, 0.56941E+00_JPRB, 0.46546E+00_JPRB, 0.52783E+00_JPRB, 0.45743E+00_JPRB, 0.46667E+00_JPRB, 0.52226E+00_JPRB ,&
&  0.51784E+00_JPRB, 0.52198E+00_JPRB, 0.57839E+00_JPRB, 0.61686E+00_JPRB, 0.68319E+00_JPRB, 0.79916E+00_JPRB, 0.79444E+00_JPRB ,&
&  0.75314E+00_JPRB, 0.69240E+00_JPRB, 0.69544E+00_JPRB, 0.66368E+00_JPRB, 0.60687E+00_JPRB, 0.53418E+00_JPRB, 0.48111E+00_JPRB ,&
&  0.45720E+00_JPRB, 0.45202E+00_JPRB, 0.44995E+00_JPRB, 0.44396E+00_JPRB, 0.43563E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.47959E+00_JPRB, 0.47263E+00_JPRB, 0.45232E+00_JPRB, 0.44681E+00_JPRB, 0.45557E+00_JPRB, 0.46921E+00_JPRB, 0.47968E+00_JPRB ,&
&  0.48063E+00_JPRB, 0.47247E+00_JPRB, 0.46398E+00_JPRB, 0.46057E+00_JPRB, 0.45276E+00_JPRB, 0.43946E+00_JPRB, 0.43290E+00_JPRB ,&
&  0.43046E+00_JPRB, 0.42799E+00_JPRB, 0.38867E+00_JPRB, 0.37064E+00_JPRB, 0.35010E+00_JPRB, 0.35711E+00_JPRB, 0.36765E+00_JPRB ,&
&  0.38095E+00_JPRB, 0.37816E+00_JPRB, 0.37160E+00_JPRB, 0.37263E+00_JPRB, 0.37391E+00_JPRB, 0.38466E+00_JPRB, 0.42900E+00_JPRB ,&
&  0.48126E+00_JPRB, 0.53587E+00_JPRB, 0.54376E+00_JPRB, 0.56615E+00_JPRB, 0.55719E+00_JPRB, 0.53523E+00_JPRB, 0.58188E+00_JPRB ,&
&  0.59824E+00_JPRB, 0.60088E+00_JPRB, 0.60759E+00_JPRB, 0.63937E+00_JPRB, 0.68416E+00_JPRB, 0.71329E+00_JPRB, 0.75716E+00_JPRB ,&
&  0.78323E+00_JPRB, 0.78967E+00_JPRB, 0.77856E+00_JPRB, 0.75548E+00_JPRB, 0.80334E+00_JPRB, 0.83149E+00_JPRB, 0.78228E+00_JPRB ,&
&  0.80115E+00_JPRB, 0.71809E+00_JPRB, 0.59680E+00_JPRB, 0.87322E+00_JPRB, 0.13742E+01_JPRB, 0.14457E+01_JPRB, 0.12106E+01_JPRB ,&
&  0.12334E+01_JPRB, 0.12178E+01_JPRB, 0.13112E+01_JPRB, 0.13804E+01_JPRB, 0.13242E+01_JPRB, 0.12009E+01_JPRB, 0.10876E+01_JPRB ,&
&  0.97829E+00_JPRB, 0.94842E+00_JPRB, 0.98803E+00_JPRB, 0.11618E+01_JPRB, 0.11889E+01_JPRB, 0.11735E+01_JPRB, 0.11190E+01_JPRB ,&
&  0.92785E+00_JPRB, 0.77539E+00_JPRB, 0.55698E+00_JPRB, 0.56752E+00_JPRB, 0.45905E+00_JPRB, 0.46105E+00_JPRB, 0.51259E+00_JPRB ,&
&  0.50387E+00_JPRB, 0.53839E+00_JPRB, 0.60002E+00_JPRB, 0.65423E+00_JPRB, 0.85751E+00_JPRB, 0.97745E+00_JPRB, 0.10112E+01_JPRB ,&
&  0.96727E+00_JPRB, 0.84631E+00_JPRB, 0.85099E+00_JPRB, 0.78369E+00_JPRB, 0.73247E+00_JPRB, 0.64867E+00_JPRB, 0.57745E+00_JPRB ,&
&  0.53493E+00_JPRB, 0.51934E+00_JPRB, 0.50783E+00_JPRB, 0.49548E+00_JPRB, 0.48220E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.50295E+00_JPRB, 0.48319E+00_JPRB, 0.44980E+00_JPRB, 0.44700E+00_JPRB, 0.46010E+00_JPRB, 0.48546E+00_JPRB, 0.50192E+00_JPRB ,&
&  0.50127E+00_JPRB, 0.49254E+00_JPRB, 0.48139E+00_JPRB, 0.47451E+00_JPRB, 0.46076E+00_JPRB, 0.44136E+00_JPRB, 0.43361E+00_JPRB ,&
&  0.42696E+00_JPRB, 0.41360E+00_JPRB, 0.38070E+00_JPRB, 0.36211E+00_JPRB, 0.34664E+00_JPRB, 0.34881E+00_JPRB, 0.36244E+00_JPRB ,&
&  0.37571E+00_JPRB, 0.38605E+00_JPRB, 0.39986E+00_JPRB, 0.41031E+00_JPRB, 0.41686E+00_JPRB, 0.45657E+00_JPRB, 0.54070E+00_JPRB ,&
&  0.69263E+00_JPRB, 0.65059E+00_JPRB, 0.66637E+00_JPRB, 0.63733E+00_JPRB, 0.63203E+00_JPRB, 0.63691E+00_JPRB, 0.63212E+00_JPRB ,&
&  0.60811E+00_JPRB, 0.59740E+00_JPRB, 0.61183E+00_JPRB, 0.64155E+00_JPRB, 0.70425E+00_JPRB, 0.75793E+00_JPRB, 0.83550E+00_JPRB ,&
&  0.86505E+00_JPRB, 0.83973E+00_JPRB, 0.78605E+00_JPRB, 0.77961E+00_JPRB, 0.83626E+00_JPRB, 0.83313E+00_JPRB, 0.80180E+00_JPRB ,&
&  0.68396E+00_JPRB, 0.60701E+00_JPRB, 0.69660E+00_JPRB, 0.88228E+00_JPRB, 0.12100E+01_JPRB, 0.14425E+01_JPRB, 0.13528E+01_JPRB ,&
&  0.13490E+01_JPRB, 0.13241E+01_JPRB, 0.13375E+01_JPRB, 0.10843E+01_JPRB, 0.11845E+01_JPRB, 0.11205E+01_JPRB, 0.10537E+01_JPRB ,&
&  0.10052E+01_JPRB, 0.10280E+01_JPRB, 0.11449E+01_JPRB, 0.13549E+01_JPRB, 0.14943E+01_JPRB, 0.15489E+01_JPRB, 0.11534E+01_JPRB ,&
&  0.87357E+00_JPRB, 0.92577E+00_JPRB, 0.83078E+00_JPRB, 0.69246E+00_JPRB, 0.58673E+00_JPRB, 0.59598E+00_JPRB, 0.59072E+00_JPRB ,&
&  0.59083E+00_JPRB, 0.60225E+00_JPRB, 0.67250E+00_JPRB, 0.83037E+00_JPRB, 0.11053E+01_JPRB, 0.12911E+01_JPRB, 0.12637E+01_JPRB ,&
&  0.11148E+01_JPRB, 0.11135E+01_JPRB, 0.10131E+01_JPRB, 0.88604E+00_JPRB, 0.82533E+00_JPRB, 0.75481E+00_JPRB, 0.68302E+00_JPRB ,&
&  0.62683E+00_JPRB, 0.60092E+00_JPRB, 0.58093E+00_JPRB, 0.55602E+00_JPRB, 0.52289E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.52742E+00_JPRB, 0.50570E+00_JPRB, 0.47413E+00_JPRB, 0.47264E+00_JPRB, 0.49368E+00_JPRB, 0.51869E+00_JPRB, 0.52618E+00_JPRB ,&
&  0.51151E+00_JPRB, 0.49948E+00_JPRB, 0.48676E+00_JPRB, 0.48570E+00_JPRB, 0.47741E+00_JPRB, 0.45364E+00_JPRB, 0.43841E+00_JPRB ,&
&  0.43495E+00_JPRB, 0.40642E+00_JPRB, 0.37395E+00_JPRB, 0.37261E+00_JPRB, 0.36837E+00_JPRB, 0.34887E+00_JPRB, 0.35605E+00_JPRB ,&
&  0.37191E+00_JPRB, 0.39193E+00_JPRB, 0.41353E+00_JPRB, 0.44883E+00_JPRB, 0.50369E+00_JPRB, 0.59563E+00_JPRB, 0.71224E+00_JPRB ,&
&  0.73316E+00_JPRB, 0.73486E+00_JPRB, 0.75953E+00_JPRB, 0.73053E+00_JPRB, 0.66926E+00_JPRB, 0.63800E+00_JPRB, 0.62622E+00_JPRB ,&
&  0.60674E+00_JPRB, 0.59973E+00_JPRB, 0.63658E+00_JPRB, 0.70632E+00_JPRB, 0.80161E+00_JPRB, 0.86077E+00_JPRB, 0.90859E+00_JPRB ,&
&  0.87149E+00_JPRB, 0.86188E+00_JPRB, 0.85081E+00_JPRB, 0.92650E+00_JPRB, 0.71297E+00_JPRB, 0.68269E+00_JPRB, 0.66020E+00_JPRB ,&
&  0.75175E+00_JPRB, 0.71932E+00_JPRB, 0.72246E+00_JPRB, 0.91233E+00_JPRB, 0.91142E+00_JPRB, 0.11038E+01_JPRB, 0.13235E+01_JPRB ,&
&  0.14383E+01_JPRB, 0.14485E+01_JPRB, 0.14259E+01_JPRB, 0.88288E+00_JPRB, 0.81749E+00_JPRB, 0.10599E+01_JPRB, 0.11244E+01_JPRB ,&
&  0.11315E+01_JPRB, 0.12624E+01_JPRB, 0.14423E+01_JPRB, 0.17271E+01_JPRB, 0.13427E+01_JPRB, 0.11278E+01_JPRB, 0.85927E+00_JPRB ,&
&  0.83516E+00_JPRB, 0.77214E+00_JPRB, 0.89814E+00_JPRB, 0.81889E+00_JPRB, 0.81333E+00_JPRB, 0.77948E+00_JPRB, 0.71917E+00_JPRB ,&
&  0.72210E+00_JPRB, 0.75231E+00_JPRB, 0.86919E+00_JPRB, 0.11575E+01_JPRB, 0.14872E+01_JPRB, 0.14892E+01_JPRB, 0.15784E+01_JPRB ,&
&  0.14964E+01_JPRB, 0.13046E+01_JPRB, 0.10877E+01_JPRB, 0.98187E+00_JPRB, 0.91212E+00_JPRB, 0.85658E+00_JPRB, 0.79550E+00_JPRB ,&
&  0.72738E+00_JPRB, 0.67624E+00_JPRB, 0.63599E+00_JPRB, 0.59464E+00_JPRB, 0.55380E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.56871E+00_JPRB, 0.54569E+00_JPRB, 0.50971E+00_JPRB, 0.51294E+00_JPRB, 0.54080E+00_JPRB, 0.55218E+00_JPRB, 0.53826E+00_JPRB ,&
&  0.52578E+00_JPRB, 0.50683E+00_JPRB, 0.49756E+00_JPRB, 0.48890E+00_JPRB, 0.47791E+00_JPRB, 0.47465E+00_JPRB, 0.45759E+00_JPRB ,&
&  0.45608E+00_JPRB, 0.44066E+00_JPRB, 0.39865E+00_JPRB, 0.37731E+00_JPRB, 0.38321E+00_JPRB, 0.36267E+00_JPRB, 0.35132E+00_JPRB ,&
&  0.38007E+00_JPRB, 0.41347E+00_JPRB, 0.45801E+00_JPRB, 0.52530E+00_JPRB, 0.62128E+00_JPRB, 0.78848E+00_JPRB, 0.87738E+00_JPRB ,&
&  0.83258E+00_JPRB, 0.79256E+00_JPRB, 0.71678E+00_JPRB, 0.64981E+00_JPRB, 0.59987E+00_JPRB, 0.57893E+00_JPRB, 0.57268E+00_JPRB ,&
&  0.57527E+00_JPRB, 0.59612E+00_JPRB, 0.66074E+00_JPRB, 0.75289E+00_JPRB, 0.84202E+00_JPRB, 0.87043E+00_JPRB, 0.87027E+00_JPRB ,&
&  0.89942E+00_JPRB, 0.97025E+00_JPRB, 0.98541E+00_JPRB, 0.92254E+00_JPRB, 0.82874E+00_JPRB, 0.70566E+00_JPRB, 0.70855E+00_JPRB ,&
&  0.74978E+00_JPRB, 0.69424E+00_JPRB, 0.73202E+00_JPRB, 0.92957E+00_JPRB, 0.10587E+01_JPRB, 0.11215E+01_JPRB, 0.13580E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.88801E+00_JPRB, 0.75444E+00_JPRB, 0.63505E+00_JPRB, 0.92423E+00_JPRB, 0.12947E+01_JPRB, 0.14328E+01_JPRB ,&
&  0.15003E+01_JPRB, 0.16434E+01_JPRB, 0.18385E+01_JPRB, 0.16884E+01_JPRB, 0.12494E+01_JPRB, 0.99969E+00_JPRB, 0.12088E+01_JPRB ,&
&  0.12625E+01_JPRB, 0.12957E+01_JPRB, 0.12420E+01_JPRB, 0.10649E+01_JPRB, 0.86996E+00_JPRB, 0.97792E+00_JPRB, 0.96298E+00_JPRB ,&
&  0.97164E+00_JPRB, 0.99424E+00_JPRB, 0.12630E+01_JPRB, 0.16301E+01_JPRB, 0.18195E+01_JPRB, 0.20136E+01_JPRB, 0.20435E+01_JPRB ,&
&  0.17173E+01_JPRB, 0.15262E+01_JPRB, 0.12815E+01_JPRB, 0.10969E+01_JPRB, 0.99291E+00_JPRB, 0.92674E+00_JPRB, 0.87403E+00_JPRB ,&
&  0.80593E+00_JPRB, 0.75596E+00_JPRB, 0.70633E+00_JPRB, 0.64494E+00_JPRB, 0.59659E+00_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.62486E+00_JPRB, 0.59193E+00_JPRB, 0.54986E+00_JPRB, 0.55356E+00_JPRB, 0.58588E+00_JPRB, 0.57568E+00_JPRB, 0.54988E+00_JPRB ,&
&  0.54388E+00_JPRB, 0.52295E+00_JPRB, 0.51046E+00_JPRB, 0.49459E+00_JPRB, 0.47952E+00_JPRB, 0.47147E+00_JPRB, 0.47029E+00_JPRB ,&
&  0.48558E+00_JPRB, 0.50712E+00_JPRB, 0.45500E+00_JPRB, 0.42012E+00_JPRB, 0.40628E+00_JPRB, 0.39058E+00_JPRB, 0.36124E+00_JPRB ,&
&  0.40299E+00_JPRB, 0.46245E+00_JPRB, 0.52874E+00_JPRB, 0.62985E+00_JPRB, 0.78537E+00_JPRB, 0.83775E+00_JPRB, 0.83488E+00_JPRB ,&
&  0.80642E+00_JPRB, 0.74848E+00_JPRB, 0.67490E+00_JPRB, 0.61130E+00_JPRB, 0.57959E+00_JPRB, 0.57196E+00_JPRB, 0.57220E+00_JPRB ,&
&  0.59268E+00_JPRB, 0.65411E+00_JPRB, 0.72753E+00_JPRB, 0.81133E+00_JPRB, 0.89187E+00_JPRB, 0.93761E+00_JPRB, 0.96367E+00_JPRB ,&
&  0.10171E+01_JPRB, 0.10779E+01_JPRB, 0.10927E+01_JPRB, 0.10436E+01_JPRB, 0.10028E+01_JPRB, 0.86681E+00_JPRB, 0.99797E+00_JPRB ,&
&  0.94738E+00_JPRB, 0.98666E+00_JPRB, 0.94999E+00_JPRB, 0.12207E+01_JPRB, 0.11340E+01_JPRB, 0.11076E+01_JPRB, 0.12337E+01_JPRB ,&
&  0.95438E+00_JPRB, 0.88304E+00_JPRB, 0.10138E+01_JPRB, 0.11688E+01_JPRB, 0.12840E+01_JPRB, 0.14241E+01_JPRB, 0.20455E+01_JPRB ,&
&  0.17238E+01_JPRB, 0.19651E+01_JPRB, 0.21403E+01_JPRB, 0.20692E+01_JPRB, 0.94799E+00_JPRB, 0.71811E+00_JPRB, 0.11211E+01_JPRB ,&
&  0.12343E+01_JPRB, 0.84754E+00_JPRB, 0.64334E+00_JPRB, 0.79889E+00_JPRB, 0.62594E+00_JPRB, 0.73805E+00_JPRB, 0.10314E+01_JPRB ,&
&  0.11716E+01_JPRB, 0.13480E+01_JPRB, 0.18614E+01_JPRB, 0.20257E+01_JPRB, 0.19699E+01_JPRB, 0.24586E+01_JPRB, 0.21327E+01_JPRB ,&
&  0.18624E+01_JPRB, 0.15286E+01_JPRB, 0.13984E+01_JPRB, 0.12184E+01_JPRB, 0.10928E+01_JPRB, 0.10199E+01_JPRB, 0.95807E+00_JPRB ,&
&  0.88836E+00_JPRB, 0.82864E+00_JPRB, 0.77290E+00_JPRB, 0.71272E+00_JPRB, 0.66362E+00_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.66617E+00_JPRB, 0.63417E+00_JPRB, 0.60001E+00_JPRB, 0.60016E+00_JPRB, 0.62172E+00_JPRB, 0.58688E+00_JPRB, 0.56848E+00_JPRB ,&
&  0.54690E+00_JPRB, 0.53303E+00_JPRB, 0.51399E+00_JPRB, 0.49261E+00_JPRB, 0.47557E+00_JPRB, 0.47602E+00_JPRB, 0.48343E+00_JPRB ,&
&  0.49685E+00_JPRB, 0.53864E+00_JPRB, 0.56260E+00_JPRB, 0.52833E+00_JPRB, 0.49919E+00_JPRB, 0.45839E+00_JPRB, 0.42999E+00_JPRB ,&
&  0.47137E+00_JPRB, 0.55793E+00_JPRB, 0.62688E+00_JPRB, 0.68769E+00_JPRB, 0.74664E+00_JPRB, 0.73853E+00_JPRB, 0.76133E+00_JPRB ,&
&  0.75307E+00_JPRB, 0.69749E+00_JPRB, 0.64334E+00_JPRB, 0.60135E+00_JPRB, 0.58550E+00_JPRB, 0.58485E+00_JPRB, 0.61359E+00_JPRB ,&
&  0.66109E+00_JPRB, 0.71434E+00_JPRB, 0.76662E+00_JPRB, 0.86552E+00_JPRB, 0.97372E+00_JPRB, 0.10263E+01_JPRB, 0.10722E+01_JPRB ,&
&  0.11141E+01_JPRB, 0.11344E+01_JPRB, 0.11418E+01_JPRB, 0.11555E+01_JPRB, 0.12398E+01_JPRB, 0.11202E+01_JPRB, 0.95211E+00_JPRB ,&
&  0.86953E+00_JPRB, 0.99772E+00_JPRB, 0.14202E+01_JPRB, 0.14052E+01_JPRB, 0.11588E+01_JPRB, 0.11725E+01_JPRB, 0.11462E+01_JPRB ,&
&  0.11163E+01_JPRB, 0.13006E+01_JPRB, 0.14698E+01_JPRB, 0.16390E+01_JPRB, 0.17522E+01_JPRB, 0.14435E+01_JPRB, 0.20042E+01_JPRB ,&
&  0.21375E+01_JPRB, 0.20859E+01_JPRB, 0.15582E+01_JPRB, 0.10769E+01_JPRB, 0.10785E+01_JPRB, 0.80700E+00_JPRB, 0.57393E+00_JPRB ,&
&  0.54156E+00_JPRB, 0.51899E+00_JPRB, 0.47766E+00_JPRB, 0.50276E+00_JPRB, 0.51014E+00_JPRB, 0.68449E+00_JPRB, 0.10347E+01_JPRB ,&
&  0.14955E+01_JPRB, 0.18779E+01_JPRB, 0.21374E+01_JPRB, 0.21845E+01_JPRB, 0.20816E+01_JPRB, 0.24100E+01_JPRB, 0.22285E+01_JPRB ,&
&  0.16020E+01_JPRB, 0.14199E+01_JPRB, 0.12849E+01_JPRB, 0.12249E+01_JPRB, 0.11648E+01_JPRB, 0.11073E+01_JPRB, 0.10307E+01_JPRB ,&
&  0.94786E+00_JPRB, 0.88786E+00_JPRB, 0.83484E+00_JPRB, 0.76593E+00_JPRB, 0.70982E+00_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.70363E+00_JPRB, 0.67631E+00_JPRB, 0.66860E+00_JPRB, 0.65805E+00_JPRB, 0.65112E+00_JPRB, 0.60143E+00_JPRB, 0.58546E+00_JPRB ,&
&  0.54782E+00_JPRB, 0.53206E+00_JPRB, 0.50248E+00_JPRB, 0.48532E+00_JPRB, 0.47459E+00_JPRB, 0.48479E+00_JPRB, 0.50537E+00_JPRB ,&
&  0.52854E+00_JPRB, 0.59487E+00_JPRB, 0.69088E+00_JPRB, 0.72233E+00_JPRB, 0.73486E+00_JPRB, 0.68019E+00_JPRB, 0.63076E+00_JPRB ,&
&  0.62325E+00_JPRB, 0.71612E+00_JPRB, 0.73039E+00_JPRB, 0.73579E+00_JPRB, 0.75128E+00_JPRB, 0.73832E+00_JPRB, 0.71937E+00_JPRB ,&
&  0.71121E+00_JPRB, 0.67056E+00_JPRB, 0.61888E+00_JPRB, 0.59751E+00_JPRB, 0.59744E+00_JPRB, 0.61829E+00_JPRB, 0.65727E+00_JPRB ,&
&  0.72332E+00_JPRB, 0.75421E+00_JPRB, 0.81882E+00_JPRB, 0.94133E+00_JPRB, 0.10413E+01_JPRB, 0.10825E+01_JPRB, 0.11416E+01_JPRB ,&
&  0.11941E+01_JPRB, 0.11937E+01_JPRB, 0.11807E+01_JPRB, 0.11497E+01_JPRB, 0.99386E+00_JPRB, 0.83678E+00_JPRB, 0.10067E+01_JPRB ,&
&  0.11594E+01_JPRB, 0.14459E+01_JPRB, 0.16249E+01_JPRB, 0.17496E+01_JPRB, 0.13113E+01_JPRB, 0.12028E+01_JPRB, 0.12013E+01_JPRB ,&
&  0.13347E+01_JPRB, 0.20750E+01_JPRB, 0.18757E+01_JPRB, 0.16936E+01_JPRB, 0.20505E+01_JPRB, 0.17064E+01_JPRB, 0.18065E+01_JPRB ,&
&  0.21773E+01_JPRB, 0.20408E+01_JPRB, 0.15668E+01_JPRB, 0.12094E+01_JPRB, 0.14785E+01_JPRB, 0.14041E+01_JPRB, 0.72635E+00_JPRB ,&
&  0.59217E+00_JPRB, 0.55070E+00_JPRB, 0.51764E+00_JPRB, 0.49467E+00_JPRB, 0.50631E+00_JPRB, 0.72449E+00_JPRB, 0.13196E+01_JPRB ,&
&  0.17472E+01_JPRB, 0.22853E+01_JPRB, 0.23607E+01_JPRB, 0.22938E+01_JPRB, 0.21053E+01_JPRB, 0.22070E+01_JPRB, 0.20231E+01_JPRB ,&
&  0.16573E+01_JPRB, 0.14754E+01_JPRB, 0.13294E+01_JPRB, 0.12582E+01_JPRB, 0.11988E+01_JPRB, 0.11422E+01_JPRB, 0.10743E+01_JPRB ,&
&  0.99705E+00_JPRB, 0.93282E+00_JPRB, 0.87410E+00_JPRB, 0.80178E+00_JPRB, 0.74293E+00_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.73283E+00_JPRB, 0.71707E+00_JPRB, 0.72369E+00_JPRB, 0.71960E+00_JPRB, 0.68798E+00_JPRB, 0.64241E+00_JPRB, 0.59382E+00_JPRB ,&
&  0.55534E+00_JPRB, 0.52491E+00_JPRB, 0.50186E+00_JPRB, 0.49040E+00_JPRB, 0.48835E+00_JPRB, 0.50400E+00_JPRB, 0.52731E+00_JPRB ,&
&  0.57761E+00_JPRB, 0.66775E+00_JPRB, 0.77378E+00_JPRB, 0.95140E+00_JPRB, 0.97339E+00_JPRB, 0.89427E+00_JPRB, 0.92958E+00_JPRB ,&
&  0.92019E+00_JPRB, 0.93944E+00_JPRB, 0.90328E+00_JPRB, 0.83808E+00_JPRB, 0.79150E+00_JPRB, 0.76892E+00_JPRB, 0.74155E+00_JPRB ,&
&  0.69130E+00_JPRB, 0.66663E+00_JPRB, 0.62607E+00_JPRB, 0.63040E+00_JPRB, 0.64595E+00_JPRB, 0.67369E+00_JPRB, 0.72350E+00_JPRB ,&
&  0.74801E+00_JPRB, 0.76486E+00_JPRB, 0.86426E+00_JPRB, 0.98518E+00_JPRB, 0.10691E+01_JPRB, 0.11387E+01_JPRB, 0.12188E+01_JPRB ,&
&  0.12414E+01_JPRB, 0.12370E+01_JPRB, 0.12047E+01_JPRB, 0.11577E+01_JPRB, 0.95175E+00_JPRB, 0.10863E+01_JPRB, 0.11347E+01_JPRB ,&
&  0.12621E+01_JPRB, 0.14306E+01_JPRB, 0.14987E+01_JPRB, 0.16248E+01_JPRB, 0.13927E+01_JPRB, 0.12460E+01_JPRB, 0.12115E+01_JPRB ,&
&  0.16051E+01_JPRB, 0.17285E+01_JPRB, 0.15502E+01_JPRB, 0.16581E+01_JPRB, 0.20170E+01_JPRB, 0.25296E+01_JPRB, 0.16467E+01_JPRB ,&
&  0.14656E+01_JPRB, 0.18693E+01_JPRB, 0.15612E+01_JPRB, 0.13614E+01_JPRB, 0.15779E+01_JPRB, 0.17820E+01_JPRB, 0.13385E+01_JPRB ,&
&  0.71934E+00_JPRB, 0.59073E+00_JPRB, 0.60681E+00_JPRB, 0.64649E+00_JPRB, 0.64380E+00_JPRB, 0.92641E+00_JPRB, 0.22721E+01_JPRB ,&
&  0.24077E+01_JPRB, 0.27131E+01_JPRB, 0.25415E+01_JPRB, 0.22726E+01_JPRB, 0.21012E+01_JPRB, 0.20789E+01_JPRB, 0.19276E+01_JPRB ,&
&  0.16418E+01_JPRB, 0.14782E+01_JPRB, 0.13474E+01_JPRB, 0.12572E+01_JPRB, 0.11785E+01_JPRB, 0.11037E+01_JPRB, 0.10331E+01_JPRB ,&
&  0.97972E+00_JPRB, 0.92371E+00_JPRB, 0.87419E+00_JPRB, 0.81434E+00_JPRB, 0.76276E+00_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.74529E+00_JPRB, 0.73698E+00_JPRB, 0.74489E+00_JPRB, 0.74104E+00_JPRB, 0.73180E+00_JPRB, 0.71608E+00_JPRB, 0.65007E+00_JPRB ,&
&  0.57933E+00_JPRB, 0.53138E+00_JPRB, 0.51017E+00_JPRB, 0.50543E+00_JPRB, 0.51369E+00_JPRB, 0.53483E+00_JPRB, 0.55869E+00_JPRB ,&
&  0.64233E+00_JPRB, 0.72697E+00_JPRB, 0.86909E+00_JPRB, 0.10585E+01_JPRB, 0.11457E+01_JPRB, 0.10329E+01_JPRB, 0.10699E+01_JPRB ,&
&  0.12277E+01_JPRB, 0.11594E+01_JPRB, 0.10457E+01_JPRB, 0.98759E+00_JPRB, 0.93682E+00_JPRB, 0.88954E+00_JPRB, 0.82189E+00_JPRB ,&
&  0.75517E+00_JPRB, 0.75339E+00_JPRB, 0.71162E+00_JPRB, 0.71281E+00_JPRB, 0.71909E+00_JPRB, 0.76058E+00_JPRB, 0.75415E+00_JPRB ,&
&  0.74743E+00_JPRB, 0.81181E+00_JPRB, 0.91246E+00_JPRB, 0.10073E+01_JPRB, 0.10965E+01_JPRB, 0.11780E+01_JPRB, 0.12727E+01_JPRB ,&
&  0.13179E+01_JPRB, 0.12962E+01_JPRB, 0.12388E+01_JPRB, 0.11347E+01_JPRB, 0.11165E+01_JPRB, 0.11373E+01_JPRB, 0.12257E+01_JPRB ,&
&  0.12759E+01_JPRB, 0.13813E+01_JPRB, 0.14325E+01_JPRB, 0.14352E+01_JPRB, 0.13260E+01_JPRB, 0.12525E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.15142E+01_JPRB, 0.15545E+01_JPRB, 0.13972E+01_JPRB, 0.17132E+01_JPRB, 0.21335E+01_JPRB, 0.24537E+01_JPRB, 0.22717E+01_JPRB ,&
&  0.19253E+01_JPRB, 0.15908E+01_JPRB, 0.14480E+01_JPRB, 0.15814E+01_JPRB, 0.17033E+01_JPRB, 0.17858E+01_JPRB, 0.21289E+01_JPRB ,&
&  0.21514E+01_JPRB, 0.13841E+01_JPRB, 0.11950E+01_JPRB, 0.14977E+01_JPRB, 0.13009E+01_JPRB, 0.13629E+01_JPRB, 0.19182E+01_JPRB ,&
&  0.24061E+01_JPRB, 0.27178E+01_JPRB, 0.25696E+01_JPRB, 0.22544E+01_JPRB, 0.21418E+01_JPRB, 0.20061E+01_JPRB, 0.18556E+01_JPRB ,&
&  0.15859E+01_JPRB, 0.13868E+01_JPRB, 0.12541E+01_JPRB, 0.11604E+01_JPRB, 0.10728E+01_JPRB, 0.10007E+01_JPRB, 0.95844E+00_JPRB ,&
&  0.92348E+00_JPRB, 0.88357E+00_JPRB, 0.84815E+00_JPRB, 0.80296E+00_JPRB, 0.76323E+00_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.74906E+00_JPRB, 0.74933E+00_JPRB, 0.75480E+00_JPRB, 0.76080E+00_JPRB, 0.80459E+00_JPRB, 0.81198E+00_JPRB, 0.74655E+00_JPRB ,&
&  0.67322E+00_JPRB, 0.55853E+00_JPRB, 0.53811E+00_JPRB, 0.54136E+00_JPRB, 0.55556E+00_JPRB, 0.58095E+00_JPRB, 0.63656E+00_JPRB ,&
&  0.72102E+00_JPRB, 0.80793E+00_JPRB, 0.98764E+00_JPRB, 0.11976E+01_JPRB, 0.14092E+01_JPRB, 0.16311E+01_JPRB, 0.11444E+01_JPRB ,&
&  0.11235E+01_JPRB, 0.11591E+01_JPRB, 0.10936E+01_JPRB, 0.10922E+01_JPRB, 0.10645E+01_JPRB, 0.10424E+01_JPRB, 0.92657E+00_JPRB ,&
&  0.85404E+00_JPRB, 0.84879E+00_JPRB, 0.80629E+00_JPRB, 0.77558E+00_JPRB, 0.79830E+00_JPRB, 0.80558E+00_JPRB, 0.76135E+00_JPRB ,&
&  0.79440E+00_JPRB, 0.86213E+00_JPRB, 0.94893E+00_JPRB, 0.10336E+01_JPRB, 0.11248E+01_JPRB, 0.12245E+01_JPRB, 0.12915E+01_JPRB ,&
&  0.13622E+01_JPRB, 0.13607E+01_JPRB, 0.13019E+01_JPRB, 0.11783E+01_JPRB, 0.11732E+01_JPRB, 0.11828E+01_JPRB, 0.11989E+01_JPRB ,&
&  0.11351E+01_JPRB, 0.10946E+01_JPRB, 0.12425E+01_JPRB, 0.13167E+01_JPRB, 0.12610E+01_JPRB, 0.11693E+01_JPRB, 0.11793E+01_JPRB ,&
&  0.14599E+01_JPRB, 0.16619E+01_JPRB, 0.18656E+01_JPRB, 0.19852E+01_JPRB, 0.22764E+01_JPRB, 0.24657E+01_JPRB, 0.25324E+01_JPRB ,&
&  0.25354E+01_JPRB, 0.21582E+01_JPRB, 0.19309E+01_JPRB, 0.19090E+01_JPRB, 0.18490E+01_JPRB, 0.18794E+01_JPRB, 0.20675E+01_JPRB ,&
&  0.23454E+01_JPRB, 0.27542E+01_JPRB, 0.28207E+01_JPRB, 0.23604E+01_JPRB, 0.21191E+01_JPRB, 0.18481E+01_JPRB, 0.19870E+01_JPRB ,&
&  0.23417E+01_JPRB, 0.23636E+01_JPRB, 0.22947E+01_JPRB, 0.24014E+01_JPRB, 0.19602E+01_JPRB, 0.17737E+01_JPRB, 0.16105E+01_JPRB ,&
&  0.14499E+01_JPRB, 0.12837E+01_JPRB, 0.11669E+01_JPRB, 0.10747E+01_JPRB, 0.97577E+00_JPRB, 0.91490E+00_JPRB, 0.89142E+00_JPRB ,&
&  0.86388E+00_JPRB, 0.84318E+00_JPRB, 0.81742E+00_JPRB, 0.78629E+00_JPRB, 0.76271E+00_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.75417E+00_JPRB, 0.75074E+00_JPRB, 0.75755E+00_JPRB, 0.77241E+00_JPRB, 0.86032E+00_JPRB, 0.89133E+00_JPRB, 0.88517E+00_JPRB ,&
&  0.86192E+00_JPRB, 0.61277E+00_JPRB, 0.57867E+00_JPRB, 0.58336E+00_JPRB, 0.60747E+00_JPRB, 0.64794E+00_JPRB, 0.72061E+00_JPRB ,&
&  0.79364E+00_JPRB, 0.90481E+00_JPRB, 0.10938E+01_JPRB, 0.13353E+01_JPRB, 0.15246E+01_JPRB, 0.16077E+01_JPRB, 0.14912E+01_JPRB ,&
&  0.10032E+01_JPRB, 0.12267E+01_JPRB, 0.11543E+01_JPRB, 0.12128E+01_JPRB, 0.11995E+01_JPRB, 0.11512E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.94413E+00_JPRB, 0.93296E+00_JPRB, 0.91512E+00_JPRB, 0.91938E+00_JPRB, 0.90966E+00_JPRB, 0.84500E+00_JPRB, 0.83102E+00_JPRB ,&
&  0.87371E+00_JPRB, 0.95406E+00_JPRB, 0.10444E+01_JPRB, 0.11187E+01_JPRB, 0.12143E+01_JPRB, 0.13006E+01_JPRB, 0.13559E+01_JPRB ,&
&  0.14197E+01_JPRB, 0.14234E+01_JPRB, 0.13214E+01_JPRB, 0.12651E+01_JPRB, 0.12359E+01_JPRB, 0.12427E+01_JPRB, 0.12306E+01_JPRB ,&
&  0.11498E+01_JPRB, 0.10871E+01_JPRB, 0.11679E+01_JPRB, 0.11605E+01_JPRB, 0.11103E+01_JPRB, 0.11674E+01_JPRB, 0.12442E+01_JPRB ,&
&  0.15879E+01_JPRB, 0.17648E+01_JPRB, 0.23613E+01_JPRB, 0.21722E+01_JPRB, 0.22085E+01_JPRB, 0.24433E+01_JPRB, 0.23624E+01_JPRB ,&
&  0.21110E+01_JPRB, 0.21308E+01_JPRB, 0.20108E+01_JPRB, 0.20593E+01_JPRB, 0.20430E+01_JPRB, 0.21122E+01_JPRB, 0.21537E+01_JPRB ,&
&  0.22828E+01_JPRB, 0.24932E+01_JPRB, 0.27590E+01_JPRB, 0.23071E+01_JPRB, 0.20915E+01_JPRB, 0.19422E+01_JPRB, 0.20609E+01_JPRB ,&
&  0.22399E+01_JPRB, 0.22387E+01_JPRB, 0.23399E+01_JPRB, 0.22113E+01_JPRB, 0.17053E+01_JPRB, 0.15449E+01_JPRB, 0.14151E+01_JPRB ,&
&  0.13042E+01_JPRB, 0.11826E+01_JPRB, 0.10871E+01_JPRB, 0.99297E+00_JPRB, 0.91707E+00_JPRB, 0.88176E+00_JPRB, 0.86144E+00_JPRB ,&
&  0.84062E+00_JPRB, 0.82554E+00_JPRB, 0.80955E+00_JPRB, 0.78481E+00_JPRB, 0.76544E+00_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.77145E+00_JPRB, 0.77446E+00_JPRB, 0.78757E+00_JPRB, 0.79502E+00_JPRB, 0.88567E+00_JPRB, 0.10016E+01_JPRB, 0.10701E+01_JPRB ,&
&  0.10169E+01_JPRB, 0.64441E+00_JPRB, 0.62254E+00_JPRB, 0.63197E+00_JPRB, 0.65872E+00_JPRB, 0.72065E+00_JPRB, 0.79776E+00_JPRB ,&
&  0.90213E+00_JPRB, 0.10336E+01_JPRB, 0.12274E+01_JPRB, 0.14652E+01_JPRB, 0.16213E+01_JPRB, 0.17820E+01_JPRB, 0.17404E+01_JPRB ,&
&  0.12383E+01_JPRB, 0.10735E+01_JPRB, 0.11997E+01_JPRB, 0.12516E+01_JPRB, 0.12103E+01_JPRB, 0.12188E+01_JPRB, 0.12039E+01_JPRB ,&
&  0.11376E+01_JPRB, 0.10753E+01_JPRB, 0.10779E+01_JPRB, 0.10647E+01_JPRB, 0.95286E+00_JPRB, 0.93153E+00_JPRB, 0.94626E+00_JPRB ,&
&  0.10009E+01_JPRB, 0.10770E+01_JPRB, 0.11589E+01_JPRB, 0.12268E+01_JPRB, 0.13211E+01_JPRB, 0.13871E+01_JPRB, 0.14460E+01_JPRB ,&
&  0.14687E+01_JPRB, 0.14821E+01_JPRB, 0.14034E+01_JPRB, 0.13124E+01_JPRB, 0.12865E+01_JPRB, 0.12657E+01_JPRB, 0.12215E+01_JPRB ,&
&  0.11809E+01_JPRB, 0.12026E+01_JPRB, 0.12129E+01_JPRB, 0.11723E+01_JPRB, 0.11457E+01_JPRB, 0.13287E+01_JPRB, 0.15104E+01_JPRB ,&
&  0.17573E+01_JPRB, 0.20571E+01_JPRB, 0.27147E+01_JPRB, 0.21235E+01_JPRB, 0.21527E+01_JPRB, 0.22939E+01_JPRB, 0.22613E+01_JPRB ,&
&  0.21036E+01_JPRB, 0.21460E+01_JPRB, 0.20986E+01_JPRB, 0.20954E+01_JPRB, 0.21311E+01_JPRB, 0.20746E+01_JPRB, 0.23921E+01_JPRB ,&
&  0.22796E+01_JPRB, 0.25969E+01_JPRB, 0.24376E+01_JPRB, 0.23079E+01_JPRB, 0.19831E+01_JPRB, 0.19294E+01_JPRB, 0.19118E+01_JPRB ,&
&  0.20357E+01_JPRB, 0.20194E+01_JPRB, 0.19127E+01_JPRB, 0.17836E+01_JPRB, 0.14850E+01_JPRB, 0.13714E+01_JPRB, 0.12613E+01_JPRB ,&
&  0.11489E+01_JPRB, 0.10572E+01_JPRB, 0.10053E+01_JPRB, 0.95625E+00_JPRB, 0.92665E+00_JPRB, 0.89890E+00_JPRB, 0.86923E+00_JPRB ,&
&  0.84392E+00_JPRB, 0.82681E+00_JPRB, 0.81159E+00_JPRB, 0.79019E+00_JPRB, 0.77603E+00_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.77781E+00_JPRB, 0.76973E+00_JPRB, 0.76847E+00_JPRB, 0.74468E+00_JPRB, 0.73627E+00_JPRB, 0.73943E+00_JPRB, 0.72049E+00_JPRB ,&
&  0.69122E+00_JPRB, 0.65879E+00_JPRB, 0.66256E+00_JPRB, 0.69036E+00_JPRB, 0.73349E+00_JPRB, 0.80573E+00_JPRB, 0.90169E+00_JPRB ,&
&  0.10253E+01_JPRB, 0.11370E+01_JPRB, 0.13099E+01_JPRB, 0.14697E+01_JPRB, 0.15871E+01_JPRB, 0.16654E+01_JPRB, 0.16625E+01_JPRB ,&
&  0.13305E+01_JPRB, 0.10281E+01_JPRB, 0.12570E+01_JPRB, 0.13687E+01_JPRB, 0.13245E+01_JPRB, 0.13523E+01_JPRB, 0.13296E+01_JPRB ,&
&  0.12167E+01_JPRB, 0.11918E+01_JPRB, 0.12027E+01_JPRB, 0.11358E+01_JPRB, 0.10264E+01_JPRB, 0.10412E+01_JPRB, 0.10864E+01_JPRB ,&
&  0.11532E+01_JPRB, 0.12097E+01_JPRB, 0.12776E+01_JPRB, 0.13470E+01_JPRB, 0.14178E+01_JPRB, 0.15075E+01_JPRB, 0.15619E+01_JPRB ,&
&  0.15674E+01_JPRB, 0.15557E+01_JPRB, 0.15217E+01_JPRB, 0.14146E+01_JPRB, 0.13433E+01_JPRB, 0.13051E+01_JPRB, 0.12599E+01_JPRB ,&
&  0.12506E+01_JPRB, 0.12412E+01_JPRB, 0.12945E+01_JPRB, 0.14165E+01_JPRB, 0.16181E+01_JPRB, 0.16156E+01_JPRB, 0.18348E+01_JPRB ,&
&  0.19957E+01_JPRB, 0.23478E+01_JPRB, 0.24510E+01_JPRB, 0.27525E+01_JPRB, 0.17394E+01_JPRB, 0.20026E+01_JPRB, 0.18748E+01_JPRB ,&
&  0.21740E+01_JPRB, 0.20607E+01_JPRB, 0.20583E+01_JPRB, 0.21208E+01_JPRB, 0.22936E+01_JPRB, 0.22413E+01_JPRB, 0.23400E+01_JPRB ,&
&  0.23251E+01_JPRB, 0.22890E+01_JPRB, 0.21104E+01_JPRB, 0.20702E+01_JPRB, 0.19468E+01_JPRB, 0.20148E+01_JPRB, 0.18054E+01_JPRB ,&
&  0.18250E+01_JPRB, 0.17665E+01_JPRB, 0.16948E+01_JPRB, 0.16297E+01_JPRB, 0.13798E+01_JPRB, 0.11768E+01_JPRB, 0.10736E+01_JPRB ,&
&  0.10257E+01_JPRB, 0.99046E+00_JPRB, 0.95051E+00_JPRB, 0.92186E+00_JPRB, 0.90657E+00_JPRB, 0.89140E+00_JPRB, 0.87352E+00_JPRB ,&
&  0.85374E+00_JPRB, 0.83594E+00_JPRB, 0.82080E+00_JPRB, 0.80291E+00_JPRB, 0.78930E+00_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.76274E+00_JPRB, 0.74865E+00_JPRB, 0.74205E+00_JPRB, 0.73057E+00_JPRB, 0.72098E+00_JPRB, 0.70303E+00_JPRB, 0.69651E+00_JPRB ,&
&  0.69127E+00_JPRB, 0.70012E+00_JPRB, 0.70691E+00_JPRB, 0.73292E+00_JPRB, 0.76930E+00_JPRB, 0.83319E+00_JPRB, 0.89606E+00_JPRB ,&
&  0.96965E+00_JPRB, 0.10677E+01_JPRB, 0.12537E+01_JPRB, 0.14793E+01_JPRB, 0.16410E+01_JPRB, 0.17165E+01_JPRB, 0.16024E+01_JPRB ,&
&  0.14805E+01_JPRB, 0.14464E+01_JPRB, 0.13907E+01_JPRB, 0.11673E+01_JPRB, 0.11111E+01_JPRB, 0.13534E+01_JPRB, 0.13015E+01_JPRB ,&
&  0.12321E+01_JPRB, 0.12133E+01_JPRB, 0.11838E+01_JPRB, 0.11528E+01_JPRB, 0.11295E+01_JPRB, 0.11535E+01_JPRB, 0.11997E+01_JPRB ,&
&  0.12572E+01_JPRB, 0.13126E+01_JPRB, 0.13893E+01_JPRB, 0.14660E+01_JPRB, 0.15462E+01_JPRB, 0.16630E+01_JPRB, 0.17188E+01_JPRB ,&
&  0.17054E+01_JPRB, 0.16897E+01_JPRB, 0.16459E+01_JPRB, 0.15437E+01_JPRB, 0.14271E+01_JPRB, 0.14074E+01_JPRB, 0.14223E+01_JPRB ,&
&  0.14518E+01_JPRB, 0.15003E+01_JPRB, 0.15710E+01_JPRB, 0.17648E+01_JPRB, 0.19417E+01_JPRB, 0.18397E+01_JPRB, 0.18642E+01_JPRB ,&
&  0.21310E+01_JPRB, 0.24413E+01_JPRB, 0.21677E+01_JPRB, 0.25219E+01_JPRB, 0.14808E+01_JPRB, 0.17973E+01_JPRB, 0.20594E+01_JPRB ,&
&  0.19434E+01_JPRB, 0.19347E+01_JPRB, 0.20359E+01_JPRB, 0.22122E+01_JPRB, 0.23955E+01_JPRB, 0.22222E+01_JPRB, 0.22146E+01_JPRB ,&
&  0.21043E+01_JPRB, 0.18869E+01_JPRB, 0.18232E+01_JPRB, 0.17899E+01_JPRB, 0.18395E+01_JPRB, 0.18393E+01_JPRB, 0.15457E+01_JPRB ,&
&  0.13815E+01_JPRB, 0.15647E+01_JPRB, 0.16139E+01_JPRB, 0.15899E+01_JPRB, 0.13053E+01_JPRB, 0.10120E+01_JPRB, 0.97803E+00_JPRB ,&
&  0.97001E+00_JPRB, 0.93296E+00_JPRB, 0.91235E+00_JPRB, 0.90071E+00_JPRB, 0.89614E+00_JPRB, 0.88602E+00_JPRB, 0.87572E+00_JPRB ,&
&  0.86445E+00_JPRB, 0.84459E+00_JPRB, 0.82612E+00_JPRB, 0.80528E+00_JPRB, 0.78286E+00_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.73552E+00_JPRB, 0.72852E+00_JPRB, 0.72526E+00_JPRB, 0.71653E+00_JPRB, 0.68268E+00_JPRB, 0.66178E+00_JPRB, 0.65616E+00_JPRB ,&
&  0.66649E+00_JPRB, 0.68384E+00_JPRB, 0.69690E+00_JPRB, 0.70507E+00_JPRB, 0.73260E+00_JPRB, 0.74976E+00_JPRB, 0.77989E+00_JPRB ,&
&  0.83646E+00_JPRB, 0.92839E+00_JPRB, 0.11011E+01_JPRB, 0.13743E+01_JPRB, 0.16257E+01_JPRB, 0.18501E+01_JPRB, 0.17121E+01_JPRB ,&
&  0.15745E+01_JPRB, 0.15003E+01_JPRB, 0.14857E+01_JPRB, 0.14853E+01_JPRB, 0.13215E+01_JPRB, 0.14201E+01_JPRB, 0.14880E+01_JPRB ,&
&  0.14998E+01_JPRB, 0.14067E+01_JPRB, 0.13009E+01_JPRB, 0.12657E+01_JPRB, 0.12422E+01_JPRB, 0.12572E+01_JPRB, 0.12990E+01_JPRB ,&
&  0.13521E+01_JPRB, 0.14083E+01_JPRB, 0.14980E+01_JPRB, 0.16099E+01_JPRB, 0.17440E+01_JPRB, 0.18671E+01_JPRB, 0.19135E+01_JPRB ,&
&  0.19026E+01_JPRB, 0.18887E+01_JPRB, 0.18542E+01_JPRB, 0.17393E+01_JPRB, 0.16892E+01_JPRB, 0.17152E+01_JPRB, 0.17794E+01_JPRB ,&
&  0.18656E+01_JPRB, 0.18436E+01_JPRB, 0.19413E+01_JPRB, 0.21572E+01_JPRB, 0.22085E+01_JPRB, 0.21425E+01_JPRB, 0.21276E+01_JPRB ,&
&  0.22901E+01_JPRB, 0.25151E+01_JPRB, 0.17130E+01_JPRB, 0.20285E+01_JPRB, 0.24015E+01_JPRB, 0.20281E+01_JPRB, 0.18863E+01_JPRB ,&
&  0.18660E+01_JPRB, 0.19110E+01_JPRB, 0.20444E+01_JPRB, 0.22169E+01_JPRB, 0.23996E+01_JPRB, 0.25232E+01_JPRB, 0.19245E+01_JPRB ,&
&  0.18069E+01_JPRB, 0.16070E+01_JPRB, 0.15689E+01_JPRB, 0.15833E+01_JPRB, 0.16238E+01_JPRB, 0.15727E+01_JPRB, 0.13360E+01_JPRB ,&
&  0.12650E+01_JPRB, 0.14150E+01_JPRB, 0.14347E+01_JPRB, 0.15533E+01_JPRB, 0.15146E+01_JPRB, 0.93543E+00_JPRB, 0.95100E+00_JPRB ,&
&  0.93500E+00_JPRB, 0.90659E+00_JPRB, 0.89466E+00_JPRB, 0.88608E+00_JPRB, 0.88228E+00_JPRB, 0.87631E+00_JPRB, 0.87055E+00_JPRB ,&
&  0.85529E+00_JPRB, 0.83108E+00_JPRB, 0.80534E+00_JPRB, 0.77951E+00_JPRB, 0.75515E+00_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.71964E+00_JPRB, 0.71569E+00_JPRB, 0.71180E+00_JPRB, 0.68723E+00_JPRB, 0.65379E+00_JPRB, 0.63835E+00_JPRB, 0.62966E+00_JPRB ,&
&  0.63191E+00_JPRB, 0.64934E+00_JPRB, 0.65995E+00_JPRB, 0.65752E+00_JPRB, 0.65463E+00_JPRB, 0.66945E+00_JPRB, 0.71300E+00_JPRB ,&
&  0.76923E+00_JPRB, 0.84047E+00_JPRB, 0.97270E+00_JPRB, 0.12453E+01_JPRB, 0.15852E+01_JPRB, 0.18489E+01_JPRB, 0.17892E+01_JPRB ,&
&  0.16541E+01_JPRB, 0.16525E+01_JPRB, 0.16635E+01_JPRB, 0.16972E+01_JPRB, 0.17163E+01_JPRB, 0.17742E+01_JPRB, 0.18192E+01_JPRB ,&
&  0.16032E+01_JPRB, 0.13268E+01_JPRB, 0.12894E+01_JPRB, 0.12899E+01_JPRB, 0.13146E+01_JPRB, 0.13408E+01_JPRB, 0.13840E+01_JPRB ,&
&  0.14388E+01_JPRB, 0.15209E+01_JPRB, 0.16373E+01_JPRB, 0.18170E+01_JPRB, 0.20066E+01_JPRB, 0.21183E+01_JPRB, 0.21742E+01_JPRB ,&
&  0.22059E+01_JPRB, 0.22248E+01_JPRB, 0.22815E+01_JPRB, 0.20596E+01_JPRB, 0.21324E+01_JPRB, 0.21962E+01_JPRB, 0.22596E+01_JPRB ,&
&  0.22434E+01_JPRB, 0.21785E+01_JPRB, 0.23442E+01_JPRB, 0.24668E+01_JPRB, 0.25853E+01_JPRB, 0.25690E+01_JPRB, 0.24942E+01_JPRB ,&
&  0.25857E+01_JPRB, 0.23503E+01_JPRB, 0.15406E+01_JPRB, 0.15077E+01_JPRB, 0.16241E+01_JPRB, 0.14936E+01_JPRB, 0.17928E+01_JPRB ,&
&  0.17591E+01_JPRB, 0.18039E+01_JPRB, 0.18628E+01_JPRB, 0.19418E+01_JPRB, 0.20260E+01_JPRB, 0.20633E+01_JPRB, 0.17078E+01_JPRB ,&
&  0.15083E+01_JPRB, 0.13857E+01_JPRB, 0.13582E+01_JPRB, 0.13309E+01_JPRB, 0.13105E+01_JPRB, 0.12558E+01_JPRB, 0.12216E+01_JPRB ,&
&  0.12180E+01_JPRB, 0.12015E+01_JPRB, 0.11471E+01_JPRB, 0.11116E+01_JPRB, 0.10096E+01_JPRB, 0.92656E+00_JPRB, 0.92677E+00_JPRB ,&
&  0.91862E+00_JPRB, 0.88697E+00_JPRB, 0.87926E+00_JPRB, 0.86841E+00_JPRB, 0.86196E+00_JPRB, 0.85346E+00_JPRB, 0.84101E+00_JPRB ,&
&  0.81952E+00_JPRB, 0.79763E+00_JPRB, 0.77529E+00_JPRB, 0.75013E+00_JPRB, 0.72900E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.71469E+00_JPRB, 0.71161E+00_JPRB, 0.70061E+00_JPRB, 0.67873E+00_JPRB, 0.67933E+00_JPRB, 0.68436E+00_JPRB, 0.66999E+00_JPRB ,&
&  0.66075E+00_JPRB, 0.65435E+00_JPRB, 0.66913E+00_JPRB, 0.67110E+00_JPRB, 0.67531E+00_JPRB, 0.69041E+00_JPRB, 0.70857E+00_JPRB ,&
&  0.72709E+00_JPRB, 0.74217E+00_JPRB, 0.80934E+00_JPRB, 0.94853E+00_JPRB, 0.12095E+01_JPRB, 0.15515E+01_JPRB, 0.17922E+01_JPRB ,&
&  0.19266E+01_JPRB, 0.20485E+01_JPRB, 0.21577E+01_JPRB, 0.22457E+01_JPRB, 0.20975E+01_JPRB, 0.21313E+01_JPRB, 0.22874E+01_JPRB ,&
&  0.16841E+01_JPRB, 0.12504E+01_JPRB, 0.13602E+01_JPRB, 0.12793E+01_JPRB, 0.13167E+01_JPRB, 0.13930E+01_JPRB, 0.14575E+01_JPRB ,&
&  0.15461E+01_JPRB, 0.16529E+01_JPRB, 0.17868E+01_JPRB, 0.19720E+01_JPRB, 0.21645E+01_JPRB, 0.23000E+01_JPRB, 0.24050E+01_JPRB ,&
&  0.24929E+01_JPRB, 0.25865E+01_JPRB, 0.27266E+01_JPRB, 0.27084E+01_JPRB, 0.26495E+01_JPRB, 0.27472E+01_JPRB, 0.27945E+01_JPRB ,&
&  0.28595E+01_JPRB, 0.28578E+01_JPRB, 0.25055E+01_JPRB, 0.26014E+01_JPRB, 0.29262E+01_JPRB, 0.27702E+01_JPRB, 0.27257E+01_JPRB ,&
&  0.29096E+01_JPRB, 0.24172E+01_JPRB, 0.14123E+01_JPRB, 0.13433E+01_JPRB, 0.14527E+01_JPRB, 0.15708E+01_JPRB, 0.16874E+01_JPRB ,&
&  0.17159E+01_JPRB, 0.17238E+01_JPRB, 0.16916E+01_JPRB, 0.16833E+01_JPRB, 0.16135E+01_JPRB, 0.15360E+01_JPRB, 0.14527E+01_JPRB ,&
&  0.12939E+01_JPRB, 0.12480E+01_JPRB, 0.12032E+01_JPRB, 0.11601E+01_JPRB, 0.11472E+01_JPRB, 0.11101E+01_JPRB, 0.10958E+01_JPRB ,&
&  0.11704E+01_JPRB, 0.11757E+01_JPRB, 0.12584E+01_JPRB, 0.11081E+01_JPRB, 0.94742E+00_JPRB, 0.94283E+00_JPRB, 0.92745E+00_JPRB ,&
&  0.94655E+00_JPRB, 0.92513E+00_JPRB, 0.90538E+00_JPRB, 0.88550E+00_JPRB, 0.87015E+00_JPRB, 0.84724E+00_JPRB, 0.81633E+00_JPRB ,&
&  0.78507E+00_JPRB, 0.76152E+00_JPRB, 0.74689E+00_JPRB, 0.73184E+00_JPRB, 0.71550E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.70418E+00_JPRB, 0.68497E+00_JPRB, 0.68807E+00_JPRB, 0.72712E+00_JPRB, 0.70466E+00_JPRB, 0.72632E+00_JPRB, 0.72074E+00_JPRB ,&
&  0.72062E+00_JPRB, 0.71771E+00_JPRB, 0.73252E+00_JPRB, 0.75119E+00_JPRB, 0.77674E+00_JPRB, 0.79859E+00_JPRB, 0.80608E+00_JPRB ,&
&  0.81724E+00_JPRB, 0.82866E+00_JPRB, 0.84764E+00_JPRB, 0.88192E+00_JPRB, 0.94465E+00_JPRB, 0.10493E+01_JPRB, 0.12545E+01_JPRB ,&
&  0.14632E+01_JPRB, 0.15695E+01_JPRB, 0.16065E+01_JPRB, 0.16028E+01_JPRB, 0.17756E+01_JPRB, 0.22153E+01_JPRB, 0.25362E+01_JPRB ,&
&  0.15337E+01_JPRB, 0.13420E+01_JPRB, 0.12414E+01_JPRB, 0.11409E+01_JPRB, 0.11973E+01_JPRB, 0.13639E+01_JPRB, 0.14593E+01_JPRB ,&
&  0.15189E+01_JPRB, 0.16214E+01_JPRB, 0.17500E+01_JPRB, 0.19097E+01_JPRB, 0.20501E+01_JPRB, 0.21895E+01_JPRB, 0.23346E+01_JPRB ,&
&  0.24910E+01_JPRB, 0.26923E+01_JPRB, 0.29037E+01_JPRB, 0.31306E+01_JPRB, 0.31112E+01_JPRB, 0.34810E+01_JPRB, 0.35622E+01_JPRB ,&
&  0.36163E+01_JPRB, 0.34475E+01_JPRB, 0.29838E+01_JPRB, 0.30414E+01_JPRB, 0.32630E+01_JPRB, 0.32281E+01_JPRB, 0.29810E+01_JPRB ,&
&  0.29659E+01_JPRB, 0.22368E+01_JPRB, 0.15063E+01_JPRB, 0.14763E+01_JPRB, 0.14821E+01_JPRB, 0.15359E+01_JPRB, 0.14835E+01_JPRB ,&
&  0.15257E+01_JPRB, 0.14897E+01_JPRB, 0.14424E+01_JPRB, 0.13843E+01_JPRB, 0.13292E+01_JPRB, 0.13282E+01_JPRB, 0.13228E+01_JPRB ,&
&  0.12121E+01_JPRB, 0.11771E+01_JPRB, 0.11149E+01_JPRB, 0.10861E+01_JPRB, 0.11065E+01_JPRB, 0.95615E+00_JPRB, 0.10343E+01_JPRB ,&
&  0.10740E+01_JPRB, 0.11057E+01_JPRB, 0.99505E+00_JPRB, 0.11772E+01_JPRB, 0.96355E+00_JPRB, 0.91227E+00_JPRB, 0.92346E+00_JPRB ,&
&  0.95152E+00_JPRB, 0.91388E+00_JPRB, 0.89620E+00_JPRB, 0.87970E+00_JPRB, 0.87172E+00_JPRB, 0.84970E+00_JPRB, 0.80069E+00_JPRB ,&
&  0.76347E+00_JPRB, 0.74007E+00_JPRB, 0.72962E+00_JPRB, 0.71981E+00_JPRB, 0.71627E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.68280E+00_JPRB, 0.68394E+00_JPRB, 0.74563E+00_JPRB, 0.70876E+00_JPRB, 0.70888E+00_JPRB, 0.72606E+00_JPRB, 0.74508E+00_JPRB ,&
&  0.74969E+00_JPRB, 0.76311E+00_JPRB, 0.78833E+00_JPRB, 0.82835E+00_JPRB, 0.86681E+00_JPRB, 0.89644E+00_JPRB, 0.92910E+00_JPRB ,&
&  0.95421E+00_JPRB, 0.96920E+00_JPRB, 0.97759E+00_JPRB, 0.98668E+00_JPRB, 0.10155E+01_JPRB, 0.10487E+01_JPRB, 0.10987E+01_JPRB ,&
&  0.11632E+01_JPRB, 0.11807E+01_JPRB, 0.11824E+01_JPRB, 0.12160E+01_JPRB, 0.12527E+01_JPRB, 0.13720E+01_JPRB, 0.15992E+01_JPRB ,&
&  0.12513E+01_JPRB, 0.13227E+01_JPRB, 0.12683E+01_JPRB, 0.12048E+01_JPRB, 0.12536E+01_JPRB, 0.12434E+01_JPRB, 0.13085E+01_JPRB ,&
&  0.14221E+01_JPRB, 0.14540E+01_JPRB, 0.15304E+01_JPRB, 0.16021E+01_JPRB, 0.16898E+01_JPRB, 0.18070E+01_JPRB, 0.19251E+01_JPRB ,&
&  0.20744E+01_JPRB, 0.22585E+01_JPRB, 0.24034E+01_JPRB, 0.26337E+01_JPRB, 0.28550E+01_JPRB, 0.30896E+01_JPRB, 0.32173E+01_JPRB ,&
&  0.32986E+01_JPRB, 0.33555E+01_JPRB, 0.30981E+01_JPRB, 0.31162E+01_JPRB, 0.31248E+01_JPRB, 0.29733E+01_JPRB, 0.26069E+01_JPRB ,&
&  0.22275E+01_JPRB, 0.18795E+01_JPRB, 0.16411E+01_JPRB, 0.15113E+01_JPRB, 0.14609E+01_JPRB, 0.13944E+01_JPRB, 0.13455E+01_JPRB ,&
&  0.13285E+01_JPRB, 0.12605E+01_JPRB, 0.11929E+01_JPRB, 0.11243E+01_JPRB, 0.10994E+01_JPRB, 0.11137E+01_JPRB, 0.11160E+01_JPRB ,&
&  0.10693E+01_JPRB, 0.10293E+01_JPRB, 0.10023E+01_JPRB, 0.10186E+01_JPRB, 0.10722E+01_JPRB, 0.13599E+01_JPRB, 0.11568E+01_JPRB ,&
&  0.92056E+00_JPRB, 0.83648E+00_JPRB, 0.68925E+00_JPRB, 0.92112E+00_JPRB, 0.86800E+00_JPRB, 0.85501E+00_JPRB, 0.88941E+00_JPRB ,&
&  0.90272E+00_JPRB, 0.89594E+00_JPRB, 0.92529E+00_JPRB, 0.88544E+00_JPRB, 0.85098E+00_JPRB, 0.83177E+00_JPRB, 0.82369E+00_JPRB ,&
&  0.77153E+00_JPRB, 0.71950E+00_JPRB, 0.66929E+00_JPRB, 0.67437E+00_JPRB, 0.64014E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.61331E+00_JPRB, 0.66217E+00_JPRB, 0.75680E+00_JPRB, 0.69120E+00_JPRB, 0.69130E+00_JPRB, 0.70900E+00_JPRB, 0.73548E+00_JPRB ,&
&  0.76331E+00_JPRB, 0.79848E+00_JPRB, 0.83598E+00_JPRB, 0.88730E+00_JPRB, 0.93236E+00_JPRB, 0.97155E+00_JPRB, 0.10085E+01_JPRB ,&
&  0.10420E+01_JPRB, 0.10708E+01_JPRB, 0.10984E+01_JPRB, 0.11122E+01_JPRB, 0.11245E+01_JPRB, 0.11496E+01_JPRB, 0.11815E+01_JPRB ,&
&  0.12416E+01_JPRB, 0.12719E+01_JPRB, 0.13097E+01_JPRB, 0.13697E+01_JPRB, 0.14132E+01_JPRB, 0.14585E+01_JPRB, 0.12721E+01_JPRB ,&
&  0.14163E+01_JPRB, 0.13049E+01_JPRB, 0.12598E+01_JPRB, 0.12473E+01_JPRB, 0.12267E+01_JPRB, 0.12169E+01_JPRB, 0.12335E+01_JPRB ,&
&  0.12953E+01_JPRB, 0.12729E+01_JPRB, 0.13007E+01_JPRB, 0.12996E+01_JPRB, 0.13535E+01_JPRB, 0.14585E+01_JPRB, 0.15272E+01_JPRB ,&
&  0.16572E+01_JPRB, 0.18161E+01_JPRB, 0.19849E+01_JPRB, 0.21157E+01_JPRB, 0.22334E+01_JPRB, 0.22406E+01_JPRB, 0.22454E+01_JPRB ,&
&  0.23116E+01_JPRB, 0.24471E+01_JPRB, 0.24041E+01_JPRB, 0.24638E+01_JPRB, 0.24528E+01_JPRB, 0.22527E+01_JPRB, 0.17801E+01_JPRB ,&
&  0.15441E+01_JPRB, 0.15902E+01_JPRB, 0.14045E+01_JPRB, 0.15880E+01_JPRB, 0.14239E+01_JPRB, 0.12708E+01_JPRB, 0.12270E+01_JPRB ,&
&  0.11465E+01_JPRB, 0.10644E+01_JPRB, 0.10010E+01_JPRB, 0.90970E+00_JPRB, 0.90786E+00_JPRB, 0.91528E+00_JPRB, 0.92221E+00_JPRB ,&
&  0.87693E+00_JPRB, 0.83351E+00_JPRB, 0.80403E+00_JPRB, 0.84986E+00_JPRB, 0.10837E+01_JPRB, 0.11331E+01_JPRB, 0.12887E+01_JPRB ,&
&  0.79824E+00_JPRB, 0.51993E+00_JPRB, 0.62042E+00_JPRB, 0.71070E+00_JPRB, 0.71729E+00_JPRB, 0.73463E+00_JPRB, 0.79690E+00_JPRB ,&
&  0.86886E+00_JPRB, 0.86806E+00_JPRB, 0.84088E+00_JPRB, 0.83965E+00_JPRB, 0.83079E+00_JPRB, 0.82846E+00_JPRB, 0.74773E+00_JPRB ,&
&  0.72070E+00_JPRB, 0.66507E+00_JPRB, 0.63360E+00_JPRB, 0.61934E+00_JPRB, 0.58984E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.59508E+00_JPRB, 0.67502E+00_JPRB, 0.74628E+00_JPRB, 0.68045E+00_JPRB, 0.69159E+00_JPRB, 0.71487E+00_JPRB, 0.74876E+00_JPRB ,&
&  0.78733E+00_JPRB, 0.83230E+00_JPRB, 0.87906E+00_JPRB, 0.93395E+00_JPRB, 0.97745E+00_JPRB, 0.10175E+01_JPRB, 0.10604E+01_JPRB ,&
&  0.11045E+01_JPRB, 0.11218E+01_JPRB, 0.11580E+01_JPRB, 0.11669E+01_JPRB, 0.11930E+01_JPRB, 0.11989E+01_JPRB, 0.12210E+01_JPRB ,&
&  0.12742E+01_JPRB, 0.13090E+01_JPRB, 0.13479E+01_JPRB, 0.13796E+01_JPRB, 0.13812E+01_JPRB, 0.14416E+01_JPRB, 0.11494E+01_JPRB ,&
&  0.13854E+01_JPRB, 0.12948E+01_JPRB, 0.12306E+01_JPRB, 0.12474E+01_JPRB, 0.12322E+01_JPRB, 0.12410E+01_JPRB, 0.12429E+01_JPRB ,&
&  0.12427E+01_JPRB, 0.12569E+01_JPRB, 0.12843E+01_JPRB, 0.12758E+01_JPRB, 0.12257E+01_JPRB, 0.12884E+01_JPRB, 0.13552E+01_JPRB ,&
&  0.14426E+01_JPRB, 0.15496E+01_JPRB, 0.16335E+01_JPRB, 0.16863E+01_JPRB, 0.17136E+01_JPRB, 0.16818E+01_JPRB, 0.16470E+01_JPRB ,&
&  0.16070E+01_JPRB, 0.16273E+01_JPRB, 0.15417E+01_JPRB, 0.16541E+01_JPRB, 0.18248E+01_JPRB, 0.15511E+01_JPRB, 0.11869E+01_JPRB ,&
&  0.10219E+01_JPRB, 0.12555E+01_JPRB, 0.14433E+01_JPRB, 0.15832E+01_JPRB, 0.13423E+01_JPRB, 0.12164E+01_JPRB, 0.11182E+01_JPRB ,&
&  0.10045E+01_JPRB, 0.90887E+00_JPRB, 0.84588E+00_JPRB, 0.76422E+00_JPRB, 0.73085E+00_JPRB, 0.72592E+00_JPRB, 0.72674E+00_JPRB ,&
&  0.70633E+00_JPRB, 0.68781E+00_JPRB, 0.68542E+00_JPRB, 0.70947E+00_JPRB, 0.78902E+00_JPRB, 0.84543E+00_JPRB, 0.14294E+01_JPRB ,&
&  0.83182E+00_JPRB, 0.59542E+00_JPRB, 0.89440E+00_JPRB, 0.45767E+00_JPRB, 0.56763E+00_JPRB, 0.61738E+00_JPRB, 0.66134E+00_JPRB ,&
&  0.62336E+00_JPRB, 0.79224E+00_JPRB, 0.11402E+01_JPRB, 0.11702E+01_JPRB, 0.12288E+01_JPRB, 0.13036E+01_JPRB, 0.78327E+00_JPRB ,&
&  0.63563E+00_JPRB, 0.62579E+00_JPRB, 0.62927E+00_JPRB, 0.58090E+00_JPRB, 0.57572E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.57143E+00_JPRB, 0.63376E+00_JPRB, 0.70580E+00_JPRB, 0.75510E+00_JPRB, 0.73722E+00_JPRB, 0.73800E+00_JPRB, 0.75706E+00_JPRB ,&
&  0.78338E+00_JPRB, 0.81737E+00_JPRB, 0.85250E+00_JPRB, 0.89853E+00_JPRB, 0.93012E+00_JPRB, 0.97009E+00_JPRB, 0.10257E+01_JPRB ,&
&  0.10933E+01_JPRB, 0.11117E+01_JPRB, 0.11370E+01_JPRB, 0.11659E+01_JPRB, 0.11920E+01_JPRB, 0.12204E+01_JPRB, 0.12468E+01_JPRB ,&
&  0.12599E+01_JPRB, 0.12846E+01_JPRB, 0.13006E+01_JPRB, 0.13515E+01_JPRB, 0.13832E+01_JPRB, 0.15269E+01_JPRB, 0.11260E+01_JPRB ,&
&  0.13305E+01_JPRB, 0.12654E+01_JPRB, 0.11947E+01_JPRB, 0.12208E+01_JPRB, 0.12211E+01_JPRB, 0.12102E+01_JPRB, 0.12011E+01_JPRB ,&
&  0.12429E+01_JPRB, 0.12648E+01_JPRB, 0.12849E+01_JPRB, 0.14075E+01_JPRB, 0.14089E+01_JPRB, 0.14140E+01_JPRB, 0.15479E+01_JPRB ,&
&  0.16759E+01_JPRB, 0.17856E+01_JPRB, 0.18269E+01_JPRB, 0.18902E+01_JPRB, 0.18939E+01_JPRB, 0.18596E+01_JPRB, 0.18101E+01_JPRB ,&
&  0.17489E+01_JPRB, 0.17379E+01_JPRB, 0.17940E+01_JPRB, 0.19594E+01_JPRB, 0.20183E+01_JPRB, 0.17851E+01_JPRB, 0.14547E+01_JPRB ,&
&  0.10661E+01_JPRB, 0.94351E+00_JPRB, 0.12918E+01_JPRB, 0.14662E+01_JPRB, 0.12650E+01_JPRB, 0.11692E+01_JPRB, 0.10047E+01_JPRB ,&
&  0.91844E+00_JPRB, 0.81931E+00_JPRB, 0.76590E+00_JPRB, 0.69681E+00_JPRB, 0.66087E+00_JPRB, 0.65693E+00_JPRB, 0.68888E+00_JPRB ,&
&  0.70315E+00_JPRB, 0.68535E+00_JPRB, 0.67996E+00_JPRB, 0.68183E+00_JPRB, 0.77502E+00_JPRB, 0.95399E+00_JPRB, 0.96746E+00_JPRB ,&
&  0.10117E+01_JPRB, 0.74057E+00_JPRB, 0.65772E+00_JPRB, 0.68428E+00_JPRB, 0.55312E+00_JPRB, 0.58871E+00_JPRB, 0.64519E+00_JPRB ,&
&  0.82666E+00_JPRB, 0.90867E+00_JPRB, 0.12443E+01_JPRB, 0.34479E+01_JPRB, 0.39653E+01_JPRB, 0.49644E+01_JPRB, 0.19889E+01_JPRB ,&
&  0.93396E+00_JPRB, 0.62713E+00_JPRB, 0.56060E+00_JPRB, 0.56060E+00_JPRB, 0.55301E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.54208E+00_JPRB, 0.59086E+00_JPRB, 0.63139E+00_JPRB, 0.68895E+00_JPRB, 0.71056E+00_JPRB, 0.71787E+00_JPRB, 0.71914E+00_JPRB ,&
&  0.70990E+00_JPRB, 0.71711E+00_JPRB, 0.74036E+00_JPRB, 0.79676E+00_JPRB, 0.85646E+00_JPRB, 0.91533E+00_JPRB, 0.97963E+00_JPRB ,&
&  0.10615E+01_JPRB, 0.11187E+01_JPRB, 0.11112E+01_JPRB, 0.11234E+01_JPRB, 0.11604E+01_JPRB, 0.11983E+01_JPRB, 0.12141E+01_JPRB ,&
&  0.12190E+01_JPRB, 0.12470E+01_JPRB, 0.12814E+01_JPRB, 0.13388E+01_JPRB, 0.14105E+01_JPRB, 0.14700E+01_JPRB, 0.10687E+01_JPRB ,&
&  0.12891E+01_JPRB, 0.12274E+01_JPRB, 0.12083E+01_JPRB, 0.12278E+01_JPRB, 0.11842E+01_JPRB, 0.11183E+01_JPRB, 0.11193E+01_JPRB ,&
&  0.11887E+01_JPRB, 0.12895E+01_JPRB, 0.13987E+01_JPRB, 0.14944E+01_JPRB, 0.16655E+01_JPRB, 0.17056E+01_JPRB, 0.19497E+01_JPRB ,&
&  0.21588E+01_JPRB, 0.22873E+01_JPRB, 0.23505E+01_JPRB, 0.24100E+01_JPRB, 0.24838E+01_JPRB, 0.25906E+01_JPRB, 0.27140E+01_JPRB ,&
&  0.27883E+01_JPRB, 0.28288E+01_JPRB, 0.28965E+01_JPRB, 0.29653E+01_JPRB, 0.26553E+01_JPRB, 0.28185E+01_JPRB, 0.22547E+01_JPRB ,&
&  0.14894E+01_JPRB, 0.11866E+01_JPRB, 0.13824E+01_JPRB, 0.13944E+01_JPRB, 0.12499E+01_JPRB, 0.11329E+01_JPRB, 0.10032E+01_JPRB ,&
&  0.86927E+00_JPRB, 0.78260E+00_JPRB, 0.72717E+00_JPRB, 0.67611E+00_JPRB, 0.63317E+00_JPRB, 0.62916E+00_JPRB, 0.64132E+00_JPRB ,&
&  0.64933E+00_JPRB, 0.64710E+00_JPRB, 0.64575E+00_JPRB, 0.67703E+00_JPRB, 0.76540E+00_JPRB, 0.99604E+00_JPRB, 0.13668E+01_JPRB ,&
&  0.16196E+01_JPRB, 0.79143E+00_JPRB, 0.66584E+00_JPRB, 0.64652E+00_JPRB, 0.65611E+00_JPRB, 0.68980E+00_JPRB, 0.74391E+00_JPRB ,&
&  0.89707E+00_JPRB, 0.11464E+01_JPRB, 0.10639E+01_JPRB, 0.12589E+01_JPRB, 0.25340E+01_JPRB, 0.27610E+01_JPRB, 0.15969E+01_JPRB ,&
&  0.10109E+01_JPRB, 0.71872E+00_JPRB, 0.58491E+00_JPRB, 0.54292E+00_JPRB, 0.53233E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.54728E+00_JPRB, 0.56242E+00_JPRB, 0.58391E+00_JPRB, 0.60962E+00_JPRB, 0.60774E+00_JPRB, 0.60990E+00_JPRB, 0.59835E+00_JPRB ,&
&  0.59670E+00_JPRB, 0.61592E+00_JPRB, 0.66396E+00_JPRB, 0.74539E+00_JPRB, 0.80555E+00_JPRB, 0.88817E+00_JPRB, 0.10011E+01_JPRB ,&
&  0.10429E+01_JPRB, 0.10310E+01_JPRB, 0.10263E+01_JPRB, 0.10356E+01_JPRB, 0.10591E+01_JPRB, 0.10922E+01_JPRB, 0.11224E+01_JPRB ,&
&  0.11544E+01_JPRB, 0.12243E+01_JPRB, 0.12853E+01_JPRB, 0.13700E+01_JPRB, 0.14510E+01_JPRB, 0.14918E+01_JPRB, 0.15642E+01_JPRB ,&
&  0.92672E+00_JPRB, 0.12742E+01_JPRB, 0.12489E+01_JPRB, 0.12209E+01_JPRB, 0.10663E+01_JPRB, 0.97009E+00_JPRB, 0.10086E+01_JPRB ,&
&  0.11886E+01_JPRB, 0.13842E+01_JPRB, 0.16507E+01_JPRB, 0.16872E+01_JPRB, 0.19022E+01_JPRB, 0.20550E+01_JPRB, 0.23425E+01_JPRB ,&
&  0.24650E+01_JPRB, 0.25831E+01_JPRB, 0.27302E+01_JPRB, 0.28943E+01_JPRB, 0.30686E+01_JPRB, 0.32817E+01_JPRB, 0.34797E+01_JPRB ,&
&  0.36360E+01_JPRB, 0.37107E+01_JPRB, 0.38656E+01_JPRB, 0.34801E+01_JPRB, 0.30336E+01_JPRB, 0.31969E+01_JPRB, 0.30909E+01_JPRB ,&
&  0.25417E+01_JPRB, 0.17258E+01_JPRB, 0.16047E+01_JPRB, 0.15630E+01_JPRB, 0.13561E+01_JPRB, 0.12235E+01_JPRB, 0.10831E+01_JPRB ,&
&  0.91751E+00_JPRB, 0.76612E+00_JPRB, 0.70133E+00_JPRB, 0.65029E+00_JPRB, 0.62671E+00_JPRB, 0.64631E+00_JPRB, 0.68210E+00_JPRB ,&
&  0.68600E+00_JPRB, 0.67989E+00_JPRB, 0.67038E+00_JPRB, 0.68710E+00_JPRB, 0.73891E+00_JPRB, 0.78776E+00_JPRB, 0.82758E+00_JPRB ,&
&  0.79350E+00_JPRB, 0.75115E+00_JPRB, 0.73838E+00_JPRB, 0.74589E+00_JPRB, 0.76177E+00_JPRB, 0.77405E+00_JPRB, 0.80468E+00_JPRB ,&
&  0.81922E+00_JPRB, 0.81455E+00_JPRB, 0.90179E+00_JPRB, 0.10996E+01_JPRB, 0.11971E+01_JPRB, 0.12761E+01_JPRB, 0.10556E+01_JPRB ,&
&  0.84680E+00_JPRB, 0.71617E+00_JPRB, 0.63370E+00_JPRB, 0.57956E+00_JPRB, 0.53646E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.53915E+00_JPRB, 0.53941E+00_JPRB, 0.53725E+00_JPRB, 0.54882E+00_JPRB, 0.54858E+00_JPRB, 0.55014E+00_JPRB, 0.54706E+00_JPRB ,&
&  0.57243E+00_JPRB, 0.62221E+00_JPRB, 0.69844E+00_JPRB, 0.78524E+00_JPRB, 0.84911E+00_JPRB, 0.92722E+00_JPRB, 0.96682E+00_JPRB ,&
&  0.96636E+00_JPRB, 0.95095E+00_JPRB, 0.94993E+00_JPRB, 0.97492E+00_JPRB, 0.99394E+00_JPRB, 0.10272E+01_JPRB, 0.10645E+01_JPRB ,&
&  0.11047E+01_JPRB, 0.11637E+01_JPRB, 0.12394E+01_JPRB, 0.13297E+01_JPRB, 0.14362E+01_JPRB, 0.15080E+01_JPRB, 0.15473E+01_JPRB ,&
&  0.60560E+00_JPRB, 0.83946E+00_JPRB, 0.11816E+01_JPRB, 0.12379E+01_JPRB, 0.96378E+00_JPRB, 0.86222E+00_JPRB, 0.80692E+00_JPRB ,&
&  0.98483E+00_JPRB, 0.12990E+01_JPRB, 0.15744E+01_JPRB, 0.19308E+01_JPRB, 0.20291E+01_JPRB, 0.22713E+01_JPRB, 0.24555E+01_JPRB ,&
&  0.25244E+01_JPRB, 0.26182E+01_JPRB, 0.27857E+01_JPRB, 0.29927E+01_JPRB, 0.31940E+01_JPRB, 0.33832E+01_JPRB, 0.35852E+01_JPRB ,&
&  0.38040E+01_JPRB, 0.39525E+01_JPRB, 0.41237E+01_JPRB, 0.35053E+01_JPRB, 0.36510E+01_JPRB, 0.38806E+01_JPRB, 0.44014E+01_JPRB ,&
&  0.36091E+01_JPRB, 0.21856E+01_JPRB, 0.16610E+01_JPRB, 0.16721E+01_JPRB, 0.14620E+01_JPRB, 0.13343E+01_JPRB, 0.11464E+01_JPRB ,&
&  0.95360E+00_JPRB, 0.81496E+00_JPRB, 0.74291E+00_JPRB, 0.72052E+00_JPRB, 0.72011E+00_JPRB, 0.74744E+00_JPRB, 0.77057E+00_JPRB ,&
&  0.76423E+00_JPRB, 0.76036E+00_JPRB, 0.75147E+00_JPRB, 0.74804E+00_JPRB, 0.79186E+00_JPRB, 0.83486E+00_JPRB, 0.85460E+00_JPRB ,&
&  0.85357E+00_JPRB, 0.85022E+00_JPRB, 0.85901E+00_JPRB, 0.87658E+00_JPRB, 0.90068E+00_JPRB, 0.91048E+00_JPRB, 0.91191E+00_JPRB ,&
&  0.82017E+00_JPRB, 0.77314E+00_JPRB, 0.76807E+00_JPRB, 0.75816E+00_JPRB, 0.76786E+00_JPRB, 0.79953E+00_JPRB, 0.75273E+00_JPRB ,&
&  0.72353E+00_JPRB, 0.71246E+00_JPRB, 0.68402E+00_JPRB, 0.61925E+00_JPRB, 0.55330E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.55791E+00_JPRB, 0.55624E+00_JPRB, 0.54641E+00_JPRB, 0.54672E+00_JPRB, 0.53915E+00_JPRB, 0.54620E+00_JPRB, 0.56114E+00_JPRB ,&
&  0.59263E+00_JPRB, 0.64860E+00_JPRB, 0.71845E+00_JPRB, 0.78083E+00_JPRB, 0.85297E+00_JPRB, 0.90897E+00_JPRB, 0.91449E+00_JPRB ,&
&  0.90058E+00_JPRB, 0.90002E+00_JPRB, 0.89286E+00_JPRB, 0.91972E+00_JPRB, 0.94462E+00_JPRB, 0.98153E+00_JPRB, 0.10264E+01_JPRB ,&
&  0.10811E+01_JPRB, 0.11513E+01_JPRB, 0.12395E+01_JPRB, 0.13437E+01_JPRB, 0.14892E+01_JPRB, 0.16553E+01_JPRB, 0.18172E+01_JPRB ,&
&  0.12447E+01_JPRB, 0.65671E+00_JPRB, 0.11357E+01_JPRB, 0.12939E+01_JPRB, 0.10080E+01_JPRB, 0.89256E+00_JPRB, 0.76308E+00_JPRB ,&
&  0.76706E+00_JPRB, 0.11537E+01_JPRB, 0.14922E+01_JPRB, 0.18296E+01_JPRB, 0.19775E+01_JPRB, 0.22022E+01_JPRB, 0.23672E+01_JPRB ,&
&  0.24546E+01_JPRB, 0.25506E+01_JPRB, 0.26646E+01_JPRB, 0.27961E+01_JPRB, 0.29621E+01_JPRB, 0.31342E+01_JPRB, 0.33489E+01_JPRB ,&
&  0.36092E+01_JPRB, 0.38096E+01_JPRB, 0.39578E+01_JPRB, 0.34226E+01_JPRB, 0.36793E+01_JPRB, 0.35177E+01_JPRB, 0.33536E+01_JPRB ,&
&  0.29745E+01_JPRB, 0.18243E+01_JPRB, 0.13708E+01_JPRB, 0.16289E+01_JPRB, 0.14886E+01_JPRB, 0.11343E+01_JPRB, 0.11883E+01_JPRB ,&
&  0.10293E+01_JPRB, 0.88879E+00_JPRB, 0.85536E+00_JPRB, 0.85640E+00_JPRB, 0.82335E+00_JPRB, 0.82423E+00_JPRB, 0.81964E+00_JPRB ,&
&  0.84695E+00_JPRB, 0.83764E+00_JPRB, 0.81834E+00_JPRB, 0.82909E+00_JPRB, 0.85681E+00_JPRB, 0.89502E+00_JPRB, 0.92069E+00_JPRB ,&
&  0.94467E+00_JPRB, 0.96042E+00_JPRB, 0.97125E+00_JPRB, 0.99280E+00_JPRB, 0.10097E+01_JPRB, 0.94946E+00_JPRB, 0.92361E+00_JPRB ,&
&  0.85390E+00_JPRB, 0.83186E+00_JPRB, 0.81437E+00_JPRB, 0.80040E+00_JPRB, 0.77968E+00_JPRB, 0.78686E+00_JPRB, 0.81573E+00_JPRB ,&
&  0.84631E+00_JPRB, 0.85138E+00_JPRB, 0.78168E+00_JPRB, 0.65623E+00_JPRB, 0.58148E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.55863E+00_JPRB, 0.54695E+00_JPRB, 0.53695E+00_JPRB, 0.55157E+00_JPRB, 0.54984E+00_JPRB, 0.55751E+00_JPRB, 0.57685E+00_JPRB ,&
&  0.60832E+00_JPRB, 0.64968E+00_JPRB, 0.68268E+00_JPRB, 0.72987E+00_JPRB, 0.79246E+00_JPRB, 0.84959E+00_JPRB, 0.85684E+00_JPRB ,&
&  0.84592E+00_JPRB, 0.84187E+00_JPRB, 0.84856E+00_JPRB, 0.86025E+00_JPRB, 0.89083E+00_JPRB, 0.92865E+00_JPRB, 0.96761E+00_JPRB ,&
&  0.10228E+01_JPRB, 0.10998E+01_JPRB, 0.11817E+01_JPRB, 0.12787E+01_JPRB, 0.14021E+01_JPRB, 0.16002E+01_JPRB, 0.18767E+01_JPRB ,&
&  0.26354E+01_JPRB, 0.25903E+01_JPRB, 0.10115E+01_JPRB, 0.13064E+01_JPRB, 0.11669E+01_JPRB, 0.99699E+00_JPRB, 0.80490E+00_JPRB ,&
&  0.75449E+00_JPRB, 0.97141E+00_JPRB, 0.13496E+01_JPRB, 0.17165E+01_JPRB, 0.17565E+01_JPRB, 0.19381E+01_JPRB, 0.20940E+01_JPRB ,&
&  0.22692E+01_JPRB, 0.23880E+01_JPRB, 0.24474E+01_JPRB, 0.24763E+01_JPRB, 0.25517E+01_JPRB, 0.26743E+01_JPRB, 0.28947E+01_JPRB ,&
&  0.32292E+01_JPRB, 0.35982E+01_JPRB, 0.38833E+01_JPRB, 0.36084E+01_JPRB, 0.37807E+01_JPRB, 0.33000E+01_JPRB, 0.29103E+01_JPRB ,&
&  0.22238E+01_JPRB, 0.18469E+01_JPRB, 0.17097E+01_JPRB, 0.16862E+01_JPRB, 0.14420E+01_JPRB, 0.12693E+01_JPRB, 0.13382E+01_JPRB ,&
&  0.12267E+01_JPRB, 0.10775E+01_JPRB, 0.98555E+00_JPRB, 0.92825E+00_JPRB, 0.90579E+00_JPRB, 0.89126E+00_JPRB, 0.89212E+00_JPRB ,&
&  0.91566E+00_JPRB, 0.91919E+00_JPRB, 0.89457E+00_JPRB, 0.91819E+00_JPRB, 0.94321E+00_JPRB, 0.97155E+00_JPRB, 0.99251E+00_JPRB ,&
&  0.10165E+01_JPRB, 0.10413E+01_JPRB, 0.10533E+01_JPRB, 0.10540E+01_JPRB, 0.99079E+00_JPRB, 0.92623E+00_JPRB, 0.93068E+00_JPRB ,&
&  0.88661E+00_JPRB, 0.89037E+00_JPRB, 0.90103E+00_JPRB, 0.90833E+00_JPRB, 0.88937E+00_JPRB, 0.90717E+00_JPRB, 0.91453E+00_JPRB ,&
&  0.94995E+00_JPRB, 0.98036E+00_JPRB, 0.89066E+00_JPRB, 0.68196E+00_JPRB, 0.59601E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.60013E+00_JPRB, 0.58611E+00_JPRB, 0.58032E+00_JPRB, 0.58692E+00_JPRB, 0.59049E+00_JPRB, 0.57998E+00_JPRB, 0.58179E+00_JPRB ,&
&  0.59323E+00_JPRB, 0.60980E+00_JPRB, 0.60942E+00_JPRB, 0.64222E+00_JPRB, 0.71685E+00_JPRB, 0.79284E+00_JPRB, 0.79872E+00_JPRB ,&
&  0.78829E+00_JPRB, 0.77600E+00_JPRB, 0.77042E+00_JPRB, 0.78581E+00_JPRB, 0.80251E+00_JPRB, 0.84891E+00_JPRB, 0.89670E+00_JPRB ,&
&  0.94491E+00_JPRB, 0.99639E+00_JPRB, 0.10422E+01_JPRB, 0.10862E+01_JPRB, 0.11672E+01_JPRB, 0.12968E+01_JPRB, 0.16416E+01_JPRB ,&
&  0.22722E+01_JPRB, 0.41707E+01_JPRB, 0.12686E+01_JPRB, 0.13014E+01_JPRB, 0.12532E+01_JPRB, 0.11124E+01_JPRB, 0.98114E+00_JPRB ,&
&  0.94209E+00_JPRB, 0.90007E+00_JPRB, 0.12137E+01_JPRB, 0.15723E+01_JPRB, 0.15349E+01_JPRB, 0.15924E+01_JPRB, 0.16899E+01_JPRB ,&
&  0.19149E+01_JPRB, 0.20793E+01_JPRB, 0.21836E+01_JPRB, 0.21737E+01_JPRB, 0.21492E+01_JPRB, 0.21314E+01_JPRB, 0.21939E+01_JPRB ,&
&  0.24651E+01_JPRB, 0.29823E+01_JPRB, 0.35371E+01_JPRB, 0.36532E+01_JPRB, 0.39459E+01_JPRB, 0.34596E+01_JPRB, 0.31280E+01_JPRB ,&
&  0.25517E+01_JPRB, 0.22876E+01_JPRB, 0.22334E+01_JPRB, 0.20206E+01_JPRB, 0.16452E+01_JPRB, 0.14721E+01_JPRB, 0.14729E+01_JPRB ,&
&  0.13902E+01_JPRB, 0.12446E+01_JPRB, 0.10986E+01_JPRB, 0.98796E+00_JPRB, 0.96664E+00_JPRB, 0.99105E+00_JPRB, 0.96686E+00_JPRB ,&
&  0.99352E+00_JPRB, 0.10331E+01_JPRB, 0.10072E+01_JPRB, 0.10448E+01_JPRB, 0.10618E+01_JPRB, 0.10654E+01_JPRB, 0.10642E+01_JPRB ,&
&  0.10641E+01_JPRB, 0.10873E+01_JPRB, 0.10919E+01_JPRB, 0.10456E+01_JPRB, 0.95299E+00_JPRB, 0.89513E+00_JPRB, 0.89689E+00_JPRB ,&
&  0.90148E+00_JPRB, 0.93040E+00_JPRB, 0.94119E+00_JPRB, 0.91518E+00_JPRB, 0.95181E+00_JPRB, 0.94634E+00_JPRB, 0.94919E+00_JPRB ,&
&  0.97421E+00_JPRB, 0.10383E+01_JPRB, 0.10755E+01_JPRB, 0.77448E+00_JPRB, 0.65895E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.67764E+00_JPRB, 0.65300E+00_JPRB, 0.63099E+00_JPRB, 0.61222E+00_JPRB, 0.60051E+00_JPRB, 0.60471E+00_JPRB, 0.60908E+00_JPRB ,&
&  0.60517E+00_JPRB, 0.60729E+00_JPRB, 0.60263E+00_JPRB, 0.60724E+00_JPRB, 0.62977E+00_JPRB, 0.67579E+00_JPRB, 0.72463E+00_JPRB ,&
&  0.72588E+00_JPRB, 0.69531E+00_JPRB, 0.69030E+00_JPRB, 0.68633E+00_JPRB, 0.70631E+00_JPRB, 0.73338E+00_JPRB, 0.78309E+00_JPRB ,&
&  0.84951E+00_JPRB, 0.90208E+00_JPRB, 0.92745E+00_JPRB, 0.93842E+00_JPRB, 0.96629E+00_JPRB, 0.10142E+01_JPRB, 0.13263E+01_JPRB ,&
&  0.19926E+01_JPRB, 0.40975E+01_JPRB, 0.30477E+01_JPRB, 0.20086E+01_JPRB, 0.13646E+01_JPRB, 0.11482E+01_JPRB, 0.11147E+01_JPRB ,&
&  0.12164E+01_JPRB, 0.11769E+01_JPRB, 0.13524E+01_JPRB, 0.14161E+01_JPRB, 0.13492E+01_JPRB, 0.12822E+01_JPRB, 0.13422E+01_JPRB ,&
&  0.15203E+01_JPRB, 0.16946E+01_JPRB, 0.18200E+01_JPRB, 0.18505E+01_JPRB, 0.18097E+01_JPRB, 0.17721E+01_JPRB, 0.17468E+01_JPRB ,&
&  0.17452E+01_JPRB, 0.19560E+01_JPRB, 0.27124E+01_JPRB, 0.35459E+01_JPRB, 0.40383E+01_JPRB, 0.37365E+01_JPRB, 0.35252E+01_JPRB ,&
&  0.34154E+01_JPRB, 0.30103E+01_JPRB, 0.27461E+01_JPRB, 0.23304E+01_JPRB, 0.18378E+01_JPRB, 0.18605E+01_JPRB, 0.15795E+01_JPRB ,&
&  0.14601E+01_JPRB, 0.13493E+01_JPRB, 0.12190E+01_JPRB, 0.10775E+01_JPRB, 0.10649E+01_JPRB, 0.10702E+01_JPRB, 0.10377E+01_JPRB ,&
&  0.10668E+01_JPRB, 0.11047E+01_JPRB, 0.10832E+01_JPRB, 0.10896E+01_JPRB, 0.10918E+01_JPRB, 0.11004E+01_JPRB, 0.11015E+01_JPRB ,&
&  0.10991E+01_JPRB, 0.11205E+01_JPRB, 0.10507E+01_JPRB, 0.97798E+00_JPRB, 0.93556E+00_JPRB, 0.88508E+00_JPRB, 0.86871E+00_JPRB ,&
&  0.90075E+00_JPRB, 0.96685E+00_JPRB, 0.97280E+00_JPRB, 0.97851E+00_JPRB, 0.96293E+00_JPRB, 0.96444E+00_JPRB, 0.92909E+00_JPRB ,&
&  0.91779E+00_JPRB, 0.89932E+00_JPRB, 0.91863E+00_JPRB, 0.84205E+00_JPRB, 0.74191E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.73088E+00_JPRB, 0.70468E+00_JPRB, 0.65053E+00_JPRB, 0.60571E+00_JPRB, 0.59773E+00_JPRB, 0.60342E+00_JPRB, 0.61492E+00_JPRB ,&
&  0.61995E+00_JPRB, 0.62155E+00_JPRB, 0.61866E+00_JPRB, 0.61138E+00_JPRB, 0.61534E+00_JPRB, 0.60082E+00_JPRB, 0.62332E+00_JPRB ,&
&  0.65020E+00_JPRB, 0.65182E+00_JPRB, 0.66311E+00_JPRB, 0.64197E+00_JPRB, 0.64117E+00_JPRB, 0.63736E+00_JPRB, 0.66436E+00_JPRB ,&
&  0.72656E+00_JPRB, 0.80171E+00_JPRB, 0.84634E+00_JPRB, 0.85989E+00_JPRB, 0.87614E+00_JPRB, 0.92675E+00_JPRB, 0.11309E+01_JPRB ,&
&  0.16824E+01_JPRB, 0.35263E+01_JPRB, 0.17267E+01_JPRB, 0.24302E+01_JPRB, 0.16287E+01_JPRB, 0.12427E+01_JPRB, 0.10377E+01_JPRB ,&
&  0.11144E+01_JPRB, 0.13520E+01_JPRB, 0.12407E+01_JPRB, 0.12327E+01_JPRB, 0.11832E+01_JPRB, 0.11463E+01_JPRB, 0.11402E+01_JPRB ,&
&  0.11960E+01_JPRB, 0.13451E+01_JPRB, 0.14932E+01_JPRB, 0.15602E+01_JPRB, 0.15225E+01_JPRB, 0.14380E+01_JPRB, 0.14152E+01_JPRB ,&
&  0.14055E+01_JPRB, 0.14282E+01_JPRB, 0.17500E+01_JPRB, 0.22673E+01_JPRB, 0.35065E+01_JPRB, 0.39554E+01_JPRB, 0.40821E+01_JPRB ,&
&  0.41876E+01_JPRB, 0.38487E+01_JPRB, 0.32310E+01_JPRB, 0.26657E+01_JPRB, 0.21820E+01_JPRB, 0.20680E+01_JPRB, 0.17558E+01_JPRB ,&
&  0.15576E+01_JPRB, 0.14230E+01_JPRB, 0.13003E+01_JPRB, 0.11873E+01_JPRB, 0.11286E+01_JPRB, 0.11121E+01_JPRB, 0.10842E+01_JPRB ,&
&  0.11233E+01_JPRB, 0.11732E+01_JPRB, 0.11577E+01_JPRB, 0.11212E+01_JPRB, 0.11038E+01_JPRB, 0.11035E+01_JPRB, 0.10985E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.11331E+01_JPRB, 0.10614E+01_JPRB, 0.95215E+00_JPRB, 0.92387E+00_JPRB, 0.91577E+00_JPRB, 0.90346E+00_JPRB ,&
&  0.95803E+00_JPRB, 0.10128E+01_JPRB, 0.10100E+01_JPRB, 0.97422E+00_JPRB, 0.97480E+00_JPRB, 0.10115E+01_JPRB, 0.95158E+00_JPRB ,&
&  0.90969E+00_JPRB, 0.88173E+00_JPRB, 0.86189E+00_JPRB, 0.83681E+00_JPRB, 0.79152E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.73711E+00_JPRB, 0.68876E+00_JPRB, 0.63290E+00_JPRB, 0.60591E+00_JPRB, 0.60937E+00_JPRB, 0.59656E+00_JPRB, 0.60527E+00_JPRB ,&
&  0.62718E+00_JPRB, 0.62947E+00_JPRB, 0.62948E+00_JPRB, 0.61902E+00_JPRB, 0.60965E+00_JPRB, 0.59109E+00_JPRB, 0.57709E+00_JPRB ,&
&  0.59569E+00_JPRB, 0.64500E+00_JPRB, 0.66483E+00_JPRB, 0.65351E+00_JPRB, 0.64100E+00_JPRB, 0.62237E+00_JPRB, 0.61766E+00_JPRB ,&
&  0.64342E+00_JPRB, 0.69621E+00_JPRB, 0.75196E+00_JPRB, 0.78076E+00_JPRB, 0.79969E+00_JPRB, 0.82510E+00_JPRB, 0.95611E+00_JPRB ,&
&  0.13387E+01_JPRB, 0.33894E+01_JPRB, 0.14685E+01_JPRB, 0.25237E+01_JPRB, 0.18398E+01_JPRB, 0.14279E+01_JPRB, 0.11037E+01_JPRB ,&
&  0.10782E+01_JPRB, 0.11908E+01_JPRB, 0.11355E+01_JPRB, 0.11312E+01_JPRB, 0.10953E+01_JPRB, 0.10719E+01_JPRB, 0.10773E+01_JPRB ,&
&  0.10849E+01_JPRB, 0.11024E+01_JPRB, 0.11675E+01_JPRB, 0.12252E+01_JPRB, 0.12327E+01_JPRB, 0.11867E+01_JPRB, 0.11408E+01_JPRB ,&
&  0.11203E+01_JPRB, 0.11279E+01_JPRB, 0.12876E+01_JPRB, 0.16832E+01_JPRB, 0.25335E+01_JPRB, 0.36610E+01_JPRB, 0.48661E+01_JPRB ,&
&  0.53183E+01_JPRB, 0.48793E+01_JPRB, 0.39286E+01_JPRB, 0.32246E+01_JPRB, 0.27138E+01_JPRB, 0.24888E+01_JPRB, 0.21198E+01_JPRB ,&
&  0.18224E+01_JPRB, 0.16211E+01_JPRB, 0.14607E+01_JPRB, 0.13578E+01_JPRB, 0.12638E+01_JPRB, 0.12334E+01_JPRB, 0.12015E+01_JPRB ,&
&  0.12114E+01_JPRB, 0.12812E+01_JPRB, 0.12787E+01_JPRB, 0.12218E+01_JPRB, 0.11450E+01_JPRB, 0.11120E+01_JPRB, 0.10803E+01_JPRB ,&
&  0.10725E+01_JPRB, 0.11027E+01_JPRB, 0.10760E+01_JPRB, 0.99371E+00_JPRB, 0.95114E+00_JPRB, 0.94621E+00_JPRB, 0.94772E+00_JPRB ,&
&  0.10091E+01_JPRB, 0.10552E+01_JPRB, 0.10471E+01_JPRB, 0.10285E+01_JPRB, 0.10029E+01_JPRB, 0.10473E+01_JPRB, 0.98257E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.88152E+00_JPRB, 0.85217E+00_JPRB, 0.80906E+00_JPRB, 0.79166E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.74696E+00_JPRB, 0.68627E+00_JPRB, 0.64474E+00_JPRB, 0.61589E+00_JPRB, 0.61620E+00_JPRB, 0.60983E+00_JPRB, 0.60681E+00_JPRB ,&
&  0.62504E+00_JPRB, 0.63617E+00_JPRB, 0.64883E+00_JPRB, 0.63802E+00_JPRB, 0.61627E+00_JPRB, 0.61053E+00_JPRB, 0.59863E+00_JPRB ,&
&  0.59282E+00_JPRB, 0.62844E+00_JPRB, 0.66325E+00_JPRB, 0.67292E+00_JPRB, 0.65832E+00_JPRB, 0.62598E+00_JPRB, 0.60981E+00_JPRB ,&
&  0.61594E+00_JPRB, 0.63460E+00_JPRB, 0.66519E+00_JPRB, 0.69775E+00_JPRB, 0.72443E+00_JPRB, 0.74821E+00_JPRB, 0.79056E+00_JPRB ,&
&  0.94973E+00_JPRB, 0.12578E+01_JPRB, 0.19365E+01_JPRB, 0.24050E+01_JPRB, 0.19418E+01_JPRB, 0.15162E+01_JPRB, 0.12337E+01_JPRB ,&
&  0.11623E+01_JPRB, 0.11084E+01_JPRB, 0.10675E+01_JPRB, 0.10333E+01_JPRB, 0.99145E+00_JPRB, 0.96529E+00_JPRB, 0.97098E+00_JPRB ,&
&  0.96008E+00_JPRB, 0.94769E+00_JPRB, 0.93601E+00_JPRB, 0.94002E+00_JPRB, 0.94670E+00_JPRB, 0.94113E+00_JPRB, 0.92675E+00_JPRB ,&
&  0.92379E+00_JPRB, 0.93276E+00_JPRB, 0.99860E+00_JPRB, 0.12833E+01_JPRB, 0.21400E+01_JPRB, 0.32581E+01_JPRB, 0.49272E+01_JPRB ,&
&  0.60222E+01_JPRB, 0.54272E+01_JPRB, 0.45056E+01_JPRB, 0.37521E+01_JPRB, 0.32097E+01_JPRB, 0.29070E+01_JPRB, 0.25917E+01_JPRB ,&
&  0.22978E+01_JPRB, 0.20031E+01_JPRB, 0.17602E+01_JPRB, 0.16089E+01_JPRB, 0.14476E+01_JPRB, 0.13728E+01_JPRB, 0.13731E+01_JPRB ,&
&  0.13392E+01_JPRB, 0.13373E+01_JPRB, 0.13452E+01_JPRB, 0.13875E+01_JPRB, 0.13078E+01_JPRB, 0.12162E+01_JPRB, 0.11325E+01_JPRB ,&
&  0.10720E+01_JPRB, 0.10763E+01_JPRB, 0.10577E+01_JPRB, 0.10346E+01_JPRB, 0.10247E+01_JPRB, 0.10187E+01_JPRB, 0.10167E+01_JPRB ,&
&  0.10313E+01_JPRB, 0.10596E+01_JPRB, 0.10677E+01_JPRB, 0.10646E+01_JPRB, 0.99501E+00_JPRB, 0.10597E+01_JPRB, 0.99623E+00_JPRB ,&
&  0.93314E+00_JPRB, 0.89244E+00_JPRB, 0.86411E+00_JPRB, 0.82911E+00_JPRB, 0.81052E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.81482E+00_JPRB, 0.75454E+00_JPRB, 0.69700E+00_JPRB, 0.66085E+00_JPRB, 0.63300E+00_JPRB, 0.62162E+00_JPRB, 0.61478E+00_JPRB ,&
&  0.62258E+00_JPRB, 0.63053E+00_JPRB, 0.64556E+00_JPRB, 0.64327E+00_JPRB, 0.62436E+00_JPRB, 0.62556E+00_JPRB, 0.64182E+00_JPRB ,&
&  0.63898E+00_JPRB, 0.64680E+00_JPRB, 0.66118E+00_JPRB, 0.65227E+00_JPRB, 0.63498E+00_JPRB, 0.61624E+00_JPRB, 0.59986E+00_JPRB ,&
&  0.60261E+00_JPRB, 0.60856E+00_JPRB, 0.62197E+00_JPRB, 0.64393E+00_JPRB, 0.67347E+00_JPRB, 0.70242E+00_JPRB, 0.72489E+00_JPRB ,&
&  0.76393E+00_JPRB, 0.12292E+01_JPRB, 0.17452E+01_JPRB, 0.20733E+01_JPRB, 0.19067E+01_JPRB, 0.15228E+01_JPRB, 0.13221E+01_JPRB ,&
&  0.11515E+01_JPRB, 0.10642E+01_JPRB, 0.10097E+01_JPRB, 0.97155E+00_JPRB, 0.91933E+00_JPRB, 0.89200E+00_JPRB, 0.88712E+00_JPRB ,&
&  0.87021E+00_JPRB, 0.83651E+00_JPRB, 0.79936E+00_JPRB, 0.77118E+00_JPRB, 0.76613E+00_JPRB, 0.77328E+00_JPRB, 0.78160E+00_JPRB ,&
&  0.78991E+00_JPRB, 0.80311E+00_JPRB, 0.84513E+00_JPRB, 0.96366E+00_JPRB, 0.16101E+01_JPRB, 0.20131E+01_JPRB, 0.37277E+01_JPRB ,&
&  0.46249E+01_JPRB, 0.39985E+01_JPRB, 0.35922E+01_JPRB, 0.33019E+01_JPRB, 0.29909E+01_JPRB, 0.27116E+01_JPRB, 0.25342E+01_JPRB ,&
&  0.23529E+01_JPRB, 0.21463E+01_JPRB, 0.19725E+01_JPRB, 0.18471E+01_JPRB, 0.17260E+01_JPRB, 0.16358E+01_JPRB, 0.15228E+01_JPRB ,&
&  0.14781E+01_JPRB, 0.14185E+01_JPRB, 0.13951E+01_JPRB, 0.14272E+01_JPRB, 0.14470E+01_JPRB, 0.13681E+01_JPRB, 0.12700E+01_JPRB ,&
&  0.11667E+01_JPRB, 0.11111E+01_JPRB, 0.10756E+01_JPRB, 0.10569E+01_JPRB, 0.10696E+01_JPRB, 0.10846E+01_JPRB, 0.10830E+01_JPRB ,&
&  0.10543E+01_JPRB, 0.10489E+01_JPRB, 0.10738E+01_JPRB, 0.10554E+01_JPRB, 0.99578E+00_JPRB, 0.10672E+01_JPRB, 0.10132E+01_JPRB ,&
&  0.94895E+00_JPRB, 0.89226E+00_JPRB, 0.85664E+00_JPRB, 0.82238E+00_JPRB, 0.84259E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.85617E+00_JPRB, 0.81471E+00_JPRB, 0.76571E+00_JPRB, 0.72851E+00_JPRB, 0.69629E+00_JPRB, 0.67363E+00_JPRB, 0.66853E+00_JPRB ,&
&  0.66279E+00_JPRB, 0.65345E+00_JPRB, 0.65978E+00_JPRB, 0.65904E+00_JPRB, 0.65633E+00_JPRB, 0.65206E+00_JPRB, 0.66615E+00_JPRB ,&
&  0.67781E+00_JPRB, 0.67724E+00_JPRB, 0.67124E+00_JPRB, 0.65841E+00_JPRB, 0.64354E+00_JPRB, 0.62584E+00_JPRB, 0.61455E+00_JPRB ,&
&  0.60780E+00_JPRB, 0.60856E+00_JPRB, 0.61556E+00_JPRB, 0.62904E+00_JPRB, 0.65338E+00_JPRB, 0.68404E+00_JPRB, 0.71348E+00_JPRB ,&
&  0.71189E+00_JPRB, 0.87194E+00_JPRB, 0.11669E+01_JPRB, 0.16540E+01_JPRB, 0.17752E+01_JPRB, 0.15312E+01_JPRB, 0.13239E+01_JPRB ,&
&  0.11709E+01_JPRB, 0.10640E+01_JPRB, 0.99009E+00_JPRB, 0.94106E+00_JPRB, 0.89067E+00_JPRB, 0.86487E+00_JPRB, 0.84151E+00_JPRB ,&
&  0.81468E+00_JPRB, 0.78224E+00_JPRB, 0.74338E+00_JPRB, 0.71853E+00_JPRB, 0.71403E+00_JPRB, 0.72153E+00_JPRB, 0.72323E+00_JPRB ,&
&  0.72158E+00_JPRB, 0.73247E+00_JPRB, 0.74688E+00_JPRB, 0.78400E+00_JPRB, 0.10681E+01_JPRB, 0.13814E+01_JPRB, 0.20966E+01_JPRB ,&
&  0.21388E+01_JPRB, 0.20283E+01_JPRB, 0.20138E+01_JPRB, 0.19941E+01_JPRB, 0.20230E+01_JPRB, 0.20394E+01_JPRB, 0.20761E+01_JPRB ,&
&  0.20999E+01_JPRB, 0.20730E+01_JPRB, 0.20061E+01_JPRB, 0.19534E+01_JPRB, 0.19018E+01_JPRB, 0.18466E+01_JPRB, 0.17387E+01_JPRB ,&
&  0.16516E+01_JPRB, 0.15794E+01_JPRB, 0.15301E+01_JPRB, 0.14791E+01_JPRB, 0.14688E+01_JPRB, 0.14294E+01_JPRB, 0.13550E+01_JPRB ,&
&  0.12554E+01_JPRB, 0.11621E+01_JPRB, 0.11278E+01_JPRB, 0.11128E+01_JPRB, 0.10989E+01_JPRB, 0.10804E+01_JPRB, 0.10801E+01_JPRB ,&
&  0.10884E+01_JPRB, 0.10872E+01_JPRB, 0.10756E+01_JPRB, 0.10538E+01_JPRB, 0.10006E+01_JPRB, 0.10593E+01_JPRB, 0.10220E+01_JPRB ,&
&  0.97002E+00_JPRB, 0.90859E+00_JPRB, 0.85319E+00_JPRB, 0.82317E+00_JPRB, 0.86892E+00_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.94288E+00_JPRB, 0.89788E+00_JPRB, 0.85093E+00_JPRB, 0.79238E+00_JPRB, 0.74610E+00_JPRB, 0.72582E+00_JPRB, 0.71369E+00_JPRB ,&
&  0.69615E+00_JPRB, 0.67944E+00_JPRB, 0.67865E+00_JPRB, 0.68488E+00_JPRB, 0.68978E+00_JPRB, 0.69164E+00_JPRB, 0.68575E+00_JPRB ,&
&  0.68426E+00_JPRB, 0.68792E+00_JPRB, 0.68049E+00_JPRB, 0.66774E+00_JPRB, 0.65385E+00_JPRB, 0.64217E+00_JPRB, 0.63321E+00_JPRB ,&
&  0.62830E+00_JPRB, 0.63124E+00_JPRB, 0.63346E+00_JPRB, 0.64437E+00_JPRB, 0.66422E+00_JPRB, 0.68751E+00_JPRB, 0.71701E+00_JPRB ,&
&  0.72115E+00_JPRB, 0.67186E+00_JPRB, 0.86704E+00_JPRB, 0.11337E+01_JPRB, 0.13461E+01_JPRB, 0.13697E+01_JPRB, 0.12697E+01_JPRB ,&
&  0.11862E+01_JPRB, 0.10876E+01_JPRB, 0.10098E+01_JPRB, 0.94696E+00_JPRB, 0.89227E+00_JPRB, 0.84868E+00_JPRB, 0.81710E+00_JPRB ,&
&  0.78333E+00_JPRB, 0.75553E+00_JPRB, 0.72889E+00_JPRB, 0.70525E+00_JPRB, 0.70030E+00_JPRB, 0.69726E+00_JPRB, 0.69358E+00_JPRB ,&
&  0.69167E+00_JPRB, 0.69386E+00_JPRB, 0.69834E+00_JPRB, 0.70895E+00_JPRB, 0.75082E+00_JPRB, 0.84694E+00_JPRB, 0.10508E+01_JPRB ,&
&  0.11601E+01_JPRB, 0.12236E+01_JPRB, 0.12430E+01_JPRB, 0.12438E+01_JPRB, 0.12719E+01_JPRB, 0.13384E+01_JPRB, 0.14431E+01_JPRB ,&
&  0.15703E+01_JPRB, 0.16814E+01_JPRB, 0.17649E+01_JPRB, 0.18163E+01_JPRB, 0.18220E+01_JPRB, 0.17954E+01_JPRB, 0.17445E+01_JPRB ,&
&  0.16890E+01_JPRB, 0.16040E+01_JPRB, 0.15201E+01_JPRB, 0.14519E+01_JPRB, 0.14117E+01_JPRB, 0.13837E+01_JPRB, 0.13372E+01_JPRB ,&
&  0.12696E+01_JPRB, 0.11924E+01_JPRB, 0.11397E+01_JPRB, 0.11127E+01_JPRB, 0.10983E+01_JPRB, 0.10800E+01_JPRB, 0.10766E+01_JPRB ,&
&  0.10735E+01_JPRB, 0.10750E+01_JPRB, 0.10327E+01_JPRB, 0.10267E+01_JPRB, 0.10657E+01_JPRB, 0.10306E+01_JPRB, 0.99809E+00_JPRB ,&
&  0.95625E+00_JPRB, 0.91257E+00_JPRB, 0.87110E+00_JPRB, 0.83713E+00_JPRB, 0.93213E+00_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.87622E+00_JPRB, 0.87246E+00_JPRB, 0.84101E+00_JPRB, 0.79631E+00_JPRB, 0.75389E+00_JPRB, 0.71960E+00_JPRB, 0.70216E+00_JPRB ,&
&  0.68906E+00_JPRB, 0.68038E+00_JPRB, 0.68604E+00_JPRB, 0.69953E+00_JPRB, 0.71543E+00_JPRB, 0.72065E+00_JPRB, 0.71634E+00_JPRB ,&
&  0.70474E+00_JPRB, 0.68619E+00_JPRB, 0.67405E+00_JPRB, 0.66663E+00_JPRB, 0.65650E+00_JPRB, 0.65168E+00_JPRB, 0.64941E+00_JPRB ,&
&  0.65230E+00_JPRB, 0.65740E+00_JPRB, 0.66039E+00_JPRB, 0.66961E+00_JPRB, 0.68504E+00_JPRB, 0.70850E+00_JPRB, 0.73289E+00_JPRB ,&
&  0.73638E+00_JPRB, 0.63685E+00_JPRB, 0.71288E+00_JPRB, 0.88150E+00_JPRB, 0.10567E+01_JPRB, 0.11498E+01_JPRB, 0.11593E+01_JPRB ,&
&  0.11226E+01_JPRB, 0.10754E+01_JPRB, 0.10182E+01_JPRB, 0.95384E+00_JPRB, 0.89200E+00_JPRB, 0.83872E+00_JPRB, 0.78988E+00_JPRB ,&
&  0.75284E+00_JPRB, 0.72658E+00_JPRB, 0.71160E+00_JPRB, 0.70120E+00_JPRB, 0.69303E+00_JPRB, 0.67717E+00_JPRB, 0.66862E+00_JPRB ,&
&  0.67001E+00_JPRB, 0.67316E+00_JPRB, 0.67783E+00_JPRB, 0.68450E+00_JPRB, 0.70724E+00_JPRB, 0.74141E+00_JPRB, 0.78094E+00_JPRB ,&
&  0.82198E+00_JPRB, 0.86196E+00_JPRB, 0.88492E+00_JPRB, 0.89457E+00_JPRB, 0.90128E+00_JPRB, 0.95340E+00_JPRB, 0.10050E+01_JPRB ,&
&  0.10791E+01_JPRB, 0.11969E+01_JPRB, 0.13356E+01_JPRB, 0.14232E+01_JPRB, 0.14401E+01_JPRB, 0.14059E+01_JPRB, 0.13472E+01_JPRB ,&
&  0.13051E+01_JPRB, 0.12585E+01_JPRB, 0.12112E+01_JPRB, 0.11718E+01_JPRB, 0.11504E+01_JPRB, 0.11521E+01_JPRB, 0.11551E+01_JPRB ,&
&  0.11513E+01_JPRB, 0.11314E+01_JPRB, 0.11083E+01_JPRB, 0.11029E+01_JPRB, 0.11030E+01_JPRB, 0.10921E+01_JPRB, 0.10728E+01_JPRB ,&
&  0.10502E+01_JPRB, 0.10300E+01_JPRB, 0.10121E+01_JPRB, 0.10211E+01_JPRB, 0.99111E+00_JPRB, 0.97008E+00_JPRB, 0.94186E+00_JPRB ,&
&  0.91615E+00_JPRB, 0.88673E+00_JPRB, 0.84898E+00_JPRB, 0.81587E+00_JPRB, 0.80918E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.78031E+00_JPRB, 0.81660E+00_JPRB, 0.82224E+00_JPRB, 0.79559E+00_JPRB, 0.76336E+00_JPRB, 0.74130E+00_JPRB, 0.72647E+00_JPRB ,&
&  0.72224E+00_JPRB, 0.71715E+00_JPRB, 0.72090E+00_JPRB, 0.73357E+00_JPRB, 0.74730E+00_JPRB, 0.75093E+00_JPRB, 0.73461E+00_JPRB ,&
&  0.71238E+00_JPRB, 0.69398E+00_JPRB, 0.68418E+00_JPRB, 0.67957E+00_JPRB, 0.67417E+00_JPRB, 0.67148E+00_JPRB, 0.67961E+00_JPRB ,&
&  0.68954E+00_JPRB, 0.70174E+00_JPRB, 0.71513E+00_JPRB, 0.72646E+00_JPRB, 0.73531E+00_JPRB, 0.74399E+00_JPRB, 0.76433E+00_JPRB ,&
&  0.77570E+00_JPRB, 0.60508E+00_JPRB, 0.63489E+00_JPRB, 0.75553E+00_JPRB, 0.86740E+00_JPRB, 0.99867E+00_JPRB, 0.10624E+01_JPRB ,&
&  0.10587E+01_JPRB, 0.10288E+01_JPRB, 0.98475E+00_JPRB, 0.93465E+00_JPRB, 0.87991E+00_JPRB, 0.83028E+00_JPRB, 0.78344E+00_JPRB ,&
&  0.75132E+00_JPRB, 0.72715E+00_JPRB, 0.71249E+00_JPRB, 0.69583E+00_JPRB, 0.68553E+00_JPRB, 0.67864E+00_JPRB, 0.67286E+00_JPRB ,&
&  0.66898E+00_JPRB, 0.66397E+00_JPRB, 0.66629E+00_JPRB, 0.67323E+00_JPRB, 0.67629E+00_JPRB, 0.68604E+00_JPRB, 0.68702E+00_JPRB ,&
&  0.70750E+00_JPRB, 0.73403E+00_JPRB, 0.74698E+00_JPRB, 0.75156E+00_JPRB, 0.76062E+00_JPRB, 0.77952E+00_JPRB, 0.80559E+00_JPRB ,&
&  0.83633E+00_JPRB, 0.89755E+00_JPRB, 0.96392E+00_JPRB, 0.10264E+01_JPRB, 0.10631E+01_JPRB, 0.10684E+01_JPRB, 0.10556E+01_JPRB ,&
&  0.10566E+01_JPRB, 0.10492E+01_JPRB, 0.10349E+01_JPRB, 0.10145E+01_JPRB, 0.98839E+00_JPRB, 0.97537E+00_JPRB, 0.98229E+00_JPRB ,&
&  0.99976E+00_JPRB, 0.10234E+01_JPRB, 0.10371E+01_JPRB, 0.10544E+01_JPRB, 0.10536E+01_JPRB, 0.10351E+01_JPRB, 0.10158E+01_JPRB ,&
&  0.99405E+00_JPRB, 0.96604E+00_JPRB, 0.94135E+00_JPRB, 0.87308E+00_JPRB, 0.91758E+00_JPRB, 0.91414E+00_JPRB, 0.89532E+00_JPRB ,&
&  0.87802E+00_JPRB, 0.84395E+00_JPRB, 0.82242E+00_JPRB, 0.66709E+00_JPRB, 0.73999E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.74464E+00_JPRB, 0.77687E+00_JPRB, 0.78831E+00_JPRB, 0.78157E+00_JPRB, 0.76326E+00_JPRB, 0.75271E+00_JPRB, 0.74551E+00_JPRB ,&
&  0.74089E+00_JPRB, 0.73705E+00_JPRB, 0.74641E+00_JPRB, 0.75383E+00_JPRB, 0.75672E+00_JPRB, 0.74904E+00_JPRB, 0.74014E+00_JPRB ,&
&  0.72583E+00_JPRB, 0.70814E+00_JPRB, 0.69529E+00_JPRB, 0.68427E+00_JPRB, 0.68283E+00_JPRB, 0.69068E+00_JPRB, 0.70673E+00_JPRB ,&
&  0.72758E+00_JPRB, 0.74233E+00_JPRB, 0.75456E+00_JPRB, 0.76762E+00_JPRB, 0.77081E+00_JPRB, 0.77191E+00_JPRB, 0.77608E+00_JPRB ,&
&  0.78352E+00_JPRB, 0.59879E+00_JPRB, 0.64146E+00_JPRB, 0.70056E+00_JPRB, 0.75720E+00_JPRB, 0.84169E+00_JPRB, 0.91277E+00_JPRB ,&
&  0.93826E+00_JPRB, 0.90999E+00_JPRB, 0.88630E+00_JPRB, 0.85721E+00_JPRB, 0.82376E+00_JPRB, 0.79385E+00_JPRB, 0.76048E+00_JPRB ,&
&  0.73537E+00_JPRB, 0.71770E+00_JPRB, 0.70978E+00_JPRB, 0.70281E+00_JPRB, 0.69497E+00_JPRB, 0.68458E+00_JPRB, 0.67229E+00_JPRB ,&
&  0.65985E+00_JPRB, 0.65371E+00_JPRB, 0.65028E+00_JPRB, 0.64758E+00_JPRB, 0.65033E+00_JPRB, 0.65558E+00_JPRB, 0.65895E+00_JPRB ,&
&  0.66893E+00_JPRB, 0.67880E+00_JPRB, 0.68415E+00_JPRB, 0.68497E+00_JPRB, 0.68602E+00_JPRB, 0.69057E+00_JPRB, 0.70953E+00_JPRB ,&
&  0.72520E+00_JPRB, 0.75105E+00_JPRB, 0.78435E+00_JPRB, 0.83165E+00_JPRB, 0.87040E+00_JPRB, 0.88735E+00_JPRB, 0.89413E+00_JPRB ,&
&  0.89594E+00_JPRB, 0.88707E+00_JPRB, 0.86852E+00_JPRB, 0.84987E+00_JPRB, 0.83571E+00_JPRB, 0.83127E+00_JPRB, 0.82929E+00_JPRB ,&
&  0.83724E+00_JPRB, 0.85192E+00_JPRB, 0.86463E+00_JPRB, 0.88257E+00_JPRB, 0.90120E+00_JPRB, 0.90296E+00_JPRB, 0.87982E+00_JPRB ,&
&  0.85929E+00_JPRB, 0.84821E+00_JPRB, 0.83924E+00_JPRB, 0.85103E+00_JPRB, 0.85012E+00_JPRB, 0.84162E+00_JPRB, 0.82091E+00_JPRB ,&
&  0.80542E+00_JPRB, 0.79266E+00_JPRB, 0.60473E+00_JPRB, 0.68964E+00_JPRB, 0.71135E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.74959E+00_JPRB, 0.74624E+00_JPRB, 0.74065E+00_JPRB, 0.74453E+00_JPRB, 0.74916E+00_JPRB, 0.74929E+00_JPRB, 0.74960E+00_JPRB ,&
&  0.74800E+00_JPRB, 0.74511E+00_JPRB, 0.75370E+00_JPRB, 0.76138E+00_JPRB, 0.76254E+00_JPRB, 0.75641E+00_JPRB, 0.75011E+00_JPRB ,&
&  0.74544E+00_JPRB, 0.73747E+00_JPRB, 0.72938E+00_JPRB, 0.72646E+00_JPRB, 0.72940E+00_JPRB, 0.74355E+00_JPRB, 0.76486E+00_JPRB ,&
&  0.78077E+00_JPRB, 0.78017E+00_JPRB, 0.78393E+00_JPRB, 0.79521E+00_JPRB, 0.79804E+00_JPRB, 0.80001E+00_JPRB, 0.80732E+00_JPRB ,&
&  0.68935E+00_JPRB, 0.56432E+00_JPRB, 0.62833E+00_JPRB, 0.68576E+00_JPRB, 0.72967E+00_JPRB, 0.78033E+00_JPRB, 0.83882E+00_JPRB ,&
&  0.85730E+00_JPRB, 0.84424E+00_JPRB, 0.83052E+00_JPRB, 0.81249E+00_JPRB, 0.78980E+00_JPRB, 0.77455E+00_JPRB, 0.76256E+00_JPRB ,&
&  0.76012E+00_JPRB, 0.74968E+00_JPRB, 0.73339E+00_JPRB, 0.72017E+00_JPRB, 0.70926E+00_JPRB, 0.69512E+00_JPRB, 0.68128E+00_JPRB ,&
&  0.66685E+00_JPRB, 0.65445E+00_JPRB, 0.65098E+00_JPRB, 0.64957E+00_JPRB, 0.65238E+00_JPRB, 0.65274E+00_JPRB, 0.64807E+00_JPRB ,&
&  0.64499E+00_JPRB, 0.64729E+00_JPRB, 0.65693E+00_JPRB, 0.66588E+00_JPRB, 0.66948E+00_JPRB, 0.66926E+00_JPRB, 0.67921E+00_JPRB ,&
&  0.68770E+00_JPRB, 0.70262E+00_JPRB, 0.72143E+00_JPRB, 0.75085E+00_JPRB, 0.77402E+00_JPRB, 0.78339E+00_JPRB, 0.78897E+00_JPRB ,&
&  0.79707E+00_JPRB, 0.79214E+00_JPRB, 0.77581E+00_JPRB, 0.76370E+00_JPRB, 0.75792E+00_JPRB, 0.75022E+00_JPRB, 0.74695E+00_JPRB ,&
&  0.75311E+00_JPRB, 0.75927E+00_JPRB, 0.75941E+00_JPRB, 0.76641E+00_JPRB, 0.77785E+00_JPRB, 0.78905E+00_JPRB, 0.79641E+00_JPRB ,&
&  0.78135E+00_JPRB, 0.78365E+00_JPRB, 0.78250E+00_JPRB, 0.79000E+00_JPRB, 0.79834E+00_JPRB, 0.78583E+00_JPRB, 0.77362E+00_JPRB ,&
&  0.76245E+00_JPRB, 0.73972E+00_JPRB, 0.73878E+00_JPRB, 0.73870E+00_JPRB, 0.74068E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.71517E+00_JPRB, 0.70662E+00_JPRB, 0.70092E+00_JPRB, 0.70955E+00_JPRB, 0.72508E+00_JPRB, 0.73746E+00_JPRB, 0.73895E+00_JPRB ,&
&  0.74008E+00_JPRB, 0.74555E+00_JPRB, 0.75435E+00_JPRB, 0.75683E+00_JPRB, 0.75004E+00_JPRB, 0.73360E+00_JPRB, 0.72532E+00_JPRB ,&
&  0.72357E+00_JPRB, 0.72503E+00_JPRB, 0.72536E+00_JPRB, 0.73008E+00_JPRB, 0.74999E+00_JPRB, 0.78087E+00_JPRB, 0.81010E+00_JPRB ,&
&  0.82456E+00_JPRB, 0.81924E+00_JPRB, 0.81310E+00_JPRB, 0.80806E+00_JPRB, 0.80685E+00_JPRB, 0.81174E+00_JPRB, 0.81283E+00_JPRB ,&
&  0.80475E+00_JPRB, 0.65850E+00_JPRB, 0.71404E+00_JPRB, 0.73463E+00_JPRB, 0.75829E+00_JPRB, 0.78648E+00_JPRB, 0.82161E+00_JPRB ,&
&  0.82677E+00_JPRB, 0.81701E+00_JPRB, 0.79772E+00_JPRB, 0.78413E+00_JPRB, 0.76875E+00_JPRB, 0.76426E+00_JPRB, 0.76163E+00_JPRB ,&
&  0.75992E+00_JPRB, 0.75699E+00_JPRB, 0.73511E+00_JPRB, 0.69989E+00_JPRB, 0.68063E+00_JPRB, 0.66918E+00_JPRB, 0.65973E+00_JPRB ,&
&  0.65097E+00_JPRB, 0.63901E+00_JPRB, 0.62731E+00_JPRB, 0.61756E+00_JPRB, 0.61921E+00_JPRB, 0.61762E+00_JPRB, 0.60527E+00_JPRB ,&
&  0.60036E+00_JPRB, 0.59864E+00_JPRB, 0.60665E+00_JPRB, 0.61761E+00_JPRB, 0.62239E+00_JPRB, 0.61835E+00_JPRB, 0.62805E+00_JPRB ,&
&  0.65019E+00_JPRB, 0.66499E+00_JPRB, 0.67774E+00_JPRB, 0.69361E+00_JPRB, 0.70208E+00_JPRB, 0.70265E+00_JPRB, 0.70508E+00_JPRB ,&
&  0.71307E+00_JPRB, 0.71257E+00_JPRB, 0.70482E+00_JPRB, 0.69891E+00_JPRB, 0.70051E+00_JPRB, 0.70179E+00_JPRB, 0.71159E+00_JPRB ,&
&  0.72081E+00_JPRB, 0.72432E+00_JPRB, 0.72094E+00_JPRB, 0.72392E+00_JPRB, 0.73024E+00_JPRB, 0.74051E+00_JPRB, 0.74692E+00_JPRB ,&
&  0.75342E+00_JPRB, 0.74922E+00_JPRB, 0.73594E+00_JPRB, 0.73534E+00_JPRB, 0.73938E+00_JPRB, 0.73597E+00_JPRB, 0.72292E+00_JPRB ,&
&  0.71166E+00_JPRB, 0.70280E+00_JPRB, 0.70495E+00_JPRB, 0.70652E+00_JPRB, 0.71150E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.70978E+00_JPRB, 0.70587E+00_JPRB, 0.70449E+00_JPRB, 0.71524E+00_JPRB, 0.73270E+00_JPRB, 0.73730E+00_JPRB, 0.73930E+00_JPRB ,&
&  0.75212E+00_JPRB, 0.76181E+00_JPRB, 0.76162E+00_JPRB, 0.76021E+00_JPRB, 0.75427E+00_JPRB, 0.74050E+00_JPRB, 0.72997E+00_JPRB ,&
&  0.73537E+00_JPRB, 0.74116E+00_JPRB, 0.74362E+00_JPRB, 0.76785E+00_JPRB, 0.79982E+00_JPRB, 0.83773E+00_JPRB, 0.86348E+00_JPRB ,&
&  0.87271E+00_JPRB, 0.86546E+00_JPRB, 0.86174E+00_JPRB, 0.85638E+00_JPRB, 0.85096E+00_JPRB, 0.85263E+00_JPRB, 0.84712E+00_JPRB ,&
&  0.83721E+00_JPRB, 0.74394E+00_JPRB, 0.77334E+00_JPRB, 0.78272E+00_JPRB, 0.79823E+00_JPRB, 0.82198E+00_JPRB, 0.84915E+00_JPRB ,&
&  0.84871E+00_JPRB, 0.84386E+00_JPRB, 0.83396E+00_JPRB, 0.81372E+00_JPRB, 0.79631E+00_JPRB, 0.78631E+00_JPRB, 0.76960E+00_JPRB ,&
&  0.74238E+00_JPRB, 0.70545E+00_JPRB, 0.67274E+00_JPRB, 0.65049E+00_JPRB, 0.64046E+00_JPRB, 0.63807E+00_JPRB, 0.63400E+00_JPRB ,&
&  0.63249E+00_JPRB, 0.63139E+00_JPRB, 0.62122E+00_JPRB, 0.61202E+00_JPRB, 0.60258E+00_JPRB, 0.59640E+00_JPRB, 0.58374E+00_JPRB ,&
&  0.57577E+00_JPRB, 0.56510E+00_JPRB, 0.56461E+00_JPRB, 0.58475E+00_JPRB, 0.60592E+00_JPRB, 0.61084E+00_JPRB, 0.62397E+00_JPRB ,&
&  0.64172E+00_JPRB, 0.64931E+00_JPRB, 0.65089E+00_JPRB, 0.64806E+00_JPRB, 0.64059E+00_JPRB, 0.64002E+00_JPRB, 0.64816E+00_JPRB ,&
&  0.65569E+00_JPRB, 0.66601E+00_JPRB, 0.66989E+00_JPRB, 0.67474E+00_JPRB, 0.68165E+00_JPRB, 0.68697E+00_JPRB, 0.69790E+00_JPRB ,&
&  0.70350E+00_JPRB, 0.70748E+00_JPRB, 0.71865E+00_JPRB, 0.73009E+00_JPRB, 0.73510E+00_JPRB, 0.73755E+00_JPRB, 0.74679E+00_JPRB ,&
&  0.75043E+00_JPRB, 0.74844E+00_JPRB, 0.73960E+00_JPRB, 0.72689E+00_JPRB, 0.71770E+00_JPRB, 0.70431E+00_JPRB, 0.68431E+00_JPRB ,&
&  0.66688E+00_JPRB, 0.66474E+00_JPRB, 0.67399E+00_JPRB, 0.68397E+00_JPRB, 0.70083E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.70043E+00_JPRB, 0.71176E+00_JPRB, 0.72473E+00_JPRB, 0.73741E+00_JPRB, 0.75067E+00_JPRB, 0.75051E+00_JPRB, 0.75216E+00_JPRB ,&
&  0.77182E+00_JPRB, 0.77523E+00_JPRB, 0.77484E+00_JPRB, 0.76642E+00_JPRB, 0.76022E+00_JPRB, 0.76016E+00_JPRB, 0.75124E+00_JPRB ,&
&  0.74679E+00_JPRB, 0.75314E+00_JPRB, 0.77157E+00_JPRB, 0.79977E+00_JPRB, 0.82732E+00_JPRB, 0.86349E+00_JPRB, 0.89027E+00_JPRB ,&
&  0.89459E+00_JPRB, 0.88384E+00_JPRB, 0.88395E+00_JPRB, 0.88711E+00_JPRB, 0.88719E+00_JPRB, 0.87017E+00_JPRB, 0.86568E+00_JPRB ,&
&  0.84556E+00_JPRB, 0.85170E+00_JPRB, 0.72163E+00_JPRB, 0.79126E+00_JPRB, 0.83512E+00_JPRB, 0.86918E+00_JPRB, 0.87878E+00_JPRB ,&
&  0.87075E+00_JPRB, 0.85807E+00_JPRB, 0.83507E+00_JPRB, 0.80894E+00_JPRB, 0.79950E+00_JPRB, 0.77919E+00_JPRB, 0.75606E+00_JPRB ,&
&  0.70746E+00_JPRB, 0.64540E+00_JPRB, 0.61376E+00_JPRB, 0.60230E+00_JPRB, 0.59674E+00_JPRB, 0.59821E+00_JPRB, 0.60378E+00_JPRB ,&
&  0.60769E+00_JPRB, 0.60849E+00_JPRB, 0.59777E+00_JPRB, 0.58162E+00_JPRB, 0.56240E+00_JPRB, 0.55226E+00_JPRB, 0.54987E+00_JPRB ,&
&  0.54657E+00_JPRB, 0.53368E+00_JPRB, 0.53705E+00_JPRB, 0.56723E+00_JPRB, 0.59251E+00_JPRB, 0.61046E+00_JPRB, 0.61834E+00_JPRB ,&
&  0.62392E+00_JPRB, 0.61774E+00_JPRB, 0.61401E+00_JPRB, 0.60491E+00_JPRB, 0.58152E+00_JPRB, 0.57517E+00_JPRB, 0.59624E+00_JPRB ,&
&  0.62018E+00_JPRB, 0.63388E+00_JPRB, 0.64207E+00_JPRB, 0.65444E+00_JPRB, 0.66760E+00_JPRB, 0.67497E+00_JPRB, 0.68113E+00_JPRB ,&
&  0.68559E+00_JPRB, 0.69001E+00_JPRB, 0.70344E+00_JPRB, 0.71850E+00_JPRB, 0.72546E+00_JPRB, 0.72666E+00_JPRB, 0.72668E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.71624E+00_JPRB, 0.70739E+00_JPRB, 0.69898E+00_JPRB, 0.67888E+00_JPRB, 0.65480E+00_JPRB, 0.63658E+00_JPRB ,&
&  0.62716E+00_JPRB, 0.63707E+00_JPRB, 0.64468E+00_JPRB, 0.65718E+00_JPRB, 0.68373E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.74095E+00_JPRB, 0.75474E+00_JPRB, 0.76613E+00_JPRB, 0.78709E+00_JPRB, 0.80621E+00_JPRB, 0.80363E+00_JPRB, 0.79682E+00_JPRB ,&
&  0.79214E+00_JPRB, 0.78786E+00_JPRB, 0.78094E+00_JPRB, 0.77153E+00_JPRB, 0.76518E+00_JPRB, 0.76470E+00_JPRB, 0.76535E+00_JPRB ,&
&  0.76685E+00_JPRB, 0.78690E+00_JPRB, 0.81990E+00_JPRB, 0.84584E+00_JPRB, 0.87421E+00_JPRB, 0.91762E+00_JPRB, 0.93820E+00_JPRB ,&
&  0.92172E+00_JPRB, 0.92082E+00_JPRB, 0.90995E+00_JPRB, 0.89534E+00_JPRB, 0.89711E+00_JPRB, 0.88776E+00_JPRB, 0.85949E+00_JPRB ,&
&  0.85287E+00_JPRB, 0.84548E+00_JPRB, 0.84130E+00_JPRB, 0.86946E+00_JPRB, 0.89150E+00_JPRB, 0.89125E+00_JPRB, 0.88743E+00_JPRB ,&
&  0.86546E+00_JPRB, 0.82855E+00_JPRB, 0.79396E+00_JPRB, 0.77734E+00_JPRB, 0.77551E+00_JPRB, 0.75538E+00_JPRB, 0.72725E+00_JPRB ,&
&  0.66970E+00_JPRB, 0.62252E+00_JPRB, 0.59647E+00_JPRB, 0.58104E+00_JPRB, 0.57611E+00_JPRB, 0.57355E+00_JPRB, 0.56931E+00_JPRB ,&
&  0.57485E+00_JPRB, 0.56996E+00_JPRB, 0.54254E+00_JPRB, 0.51378E+00_JPRB, 0.50904E+00_JPRB, 0.51811E+00_JPRB, 0.53831E+00_JPRB ,&
&  0.55350E+00_JPRB, 0.54896E+00_JPRB, 0.55597E+00_JPRB, 0.56404E+00_JPRB, 0.58159E+00_JPRB, 0.59893E+00_JPRB, 0.60000E+00_JPRB ,&
&  0.59991E+00_JPRB, 0.59463E+00_JPRB, 0.58145E+00_JPRB, 0.57273E+00_JPRB, 0.56138E+00_JPRB, 0.55680E+00_JPRB, 0.56815E+00_JPRB ,&
&  0.58561E+00_JPRB, 0.59542E+00_JPRB, 0.60626E+00_JPRB, 0.61991E+00_JPRB, 0.63108E+00_JPRB, 0.63946E+00_JPRB, 0.64599E+00_JPRB ,&
&  0.65613E+00_JPRB, 0.67078E+00_JPRB, 0.68580E+00_JPRB, 0.69898E+00_JPRB, 0.69901E+00_JPRB, 0.68471E+00_JPRB, 0.65918E+00_JPRB ,&
&  0.64216E+00_JPRB, 0.62887E+00_JPRB, 0.62579E+00_JPRB, 0.63155E+00_JPRB, 0.62676E+00_JPRB, 0.60723E+00_JPRB, 0.59734E+00_JPRB ,&
&  0.60799E+00_JPRB, 0.63185E+00_JPRB, 0.65459E+00_JPRB, 0.68541E+00_JPRB, 0.71772E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.77888E+00_JPRB, 0.81029E+00_JPRB, 0.83443E+00_JPRB, 0.84310E+00_JPRB, 0.84097E+00_JPRB, 0.83237E+00_JPRB, 0.81003E+00_JPRB ,&
&  0.78573E+00_JPRB, 0.76863E+00_JPRB, 0.75041E+00_JPRB, 0.72401E+00_JPRB, 0.70215E+00_JPRB, 0.69768E+00_JPRB, 0.71033E+00_JPRB ,&
&  0.75660E+00_JPRB, 0.79770E+00_JPRB, 0.81443E+00_JPRB, 0.82828E+00_JPRB, 0.87829E+00_JPRB, 0.92029E+00_JPRB, 0.92952E+00_JPRB ,&
&  0.92453E+00_JPRB, 0.92618E+00_JPRB, 0.88969E+00_JPRB, 0.85649E+00_JPRB, 0.85651E+00_JPRB, 0.86170E+00_JPRB, 0.84314E+00_JPRB ,&
&  0.86857E+00_JPRB, 0.88562E+00_JPRB, 0.87951E+00_JPRB, 0.85469E+00_JPRB, 0.84726E+00_JPRB, 0.83677E+00_JPRB, 0.82475E+00_JPRB ,&
&  0.80284E+00_JPRB, 0.75473E+00_JPRB, 0.72480E+00_JPRB, 0.72322E+00_JPRB, 0.71612E+00_JPRB, 0.70882E+00_JPRB, 0.69526E+00_JPRB ,&
&  0.65515E+00_JPRB, 0.61927E+00_JPRB, 0.59231E+00_JPRB, 0.57380E+00_JPRB, 0.56221E+00_JPRB, 0.55320E+00_JPRB, 0.54377E+00_JPRB ,&
&  0.53564E+00_JPRB, 0.50849E+00_JPRB, 0.48049E+00_JPRB, 0.47452E+00_JPRB, 0.48048E+00_JPRB, 0.48800E+00_JPRB, 0.51577E+00_JPRB ,&
&  0.54083E+00_JPRB, 0.53926E+00_JPRB, 0.53575E+00_JPRB, 0.53841E+00_JPRB, 0.54743E+00_JPRB, 0.54790E+00_JPRB, 0.53768E+00_JPRB ,&
&  0.53252E+00_JPRB, 0.53002E+00_JPRB, 0.52044E+00_JPRB, 0.51327E+00_JPRB, 0.51258E+00_JPRB, 0.51648E+00_JPRB, 0.53080E+00_JPRB ,&
&  0.53963E+00_JPRB, 0.54092E+00_JPRB, 0.55446E+00_JPRB, 0.57157E+00_JPRB, 0.58191E+00_JPRB, 0.59009E+00_JPRB, 0.59603E+00_JPRB ,&
&  0.60964E+00_JPRB, 0.62920E+00_JPRB, 0.64430E+00_JPRB, 0.64645E+00_JPRB, 0.62350E+00_JPRB, 0.59919E+00_JPRB, 0.58012E+00_JPRB ,&
&  0.57180E+00_JPRB, 0.57627E+00_JPRB, 0.58399E+00_JPRB, 0.59145E+00_JPRB, 0.59821E+00_JPRB, 0.60275E+00_JPRB, 0.60211E+00_JPRB ,&
&  0.62171E+00_JPRB, 0.65278E+00_JPRB, 0.67784E+00_JPRB, 0.70769E+00_JPRB, 0.73884E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.84560E+00_JPRB, 0.86311E+00_JPRB, 0.87482E+00_JPRB, 0.87548E+00_JPRB, 0.85992E+00_JPRB, 0.83320E+00_JPRB, 0.81036E+00_JPRB ,&
&  0.78436E+00_JPRB, 0.75910E+00_JPRB, 0.73511E+00_JPRB, 0.70237E+00_JPRB, 0.68478E+00_JPRB, 0.68828E+00_JPRB, 0.73738E+00_JPRB ,&
&  0.80280E+00_JPRB, 0.80782E+00_JPRB, 0.80200E+00_JPRB, 0.82748E+00_JPRB, 0.88009E+00_JPRB, 0.91200E+00_JPRB, 0.93292E+00_JPRB ,&
&  0.93555E+00_JPRB, 0.90245E+00_JPRB, 0.86939E+00_JPRB, 0.85610E+00_JPRB, 0.87950E+00_JPRB, 0.89699E+00_JPRB, 0.90172E+00_JPRB ,&
&  0.89707E+00_JPRB, 0.87590E+00_JPRB, 0.85697E+00_JPRB, 0.84084E+00_JPRB, 0.84486E+00_JPRB, 0.81977E+00_JPRB, 0.76742E+00_JPRB ,&
&  0.70099E+00_JPRB, 0.67454E+00_JPRB, 0.66317E+00_JPRB, 0.64887E+00_JPRB, 0.63807E+00_JPRB, 0.63836E+00_JPRB, 0.62933E+00_JPRB ,&
&  0.61582E+00_JPRB, 0.60011E+00_JPRB, 0.58507E+00_JPRB, 0.56737E+00_JPRB, 0.55345E+00_JPRB, 0.53370E+00_JPRB, 0.51519E+00_JPRB ,&
&  0.49803E+00_JPRB, 0.47678E+00_JPRB, 0.45727E+00_JPRB, 0.44368E+00_JPRB, 0.43775E+00_JPRB, 0.44847E+00_JPRB, 0.48894E+00_JPRB ,&
&  0.52045E+00_JPRB, 0.51745E+00_JPRB, 0.51511E+00_JPRB, 0.51920E+00_JPRB, 0.51216E+00_JPRB, 0.50171E+00_JPRB, 0.49592E+00_JPRB ,&
&  0.49065E+00_JPRB, 0.48654E+00_JPRB, 0.48471E+00_JPRB, 0.48652E+00_JPRB, 0.48869E+00_JPRB, 0.48686E+00_JPRB, 0.48914E+00_JPRB ,&
&  0.49363E+00_JPRB, 0.49883E+00_JPRB, 0.51587E+00_JPRB, 0.54031E+00_JPRB, 0.56106E+00_JPRB, 0.57590E+00_JPRB, 0.58963E+00_JPRB ,&
&  0.61075E+00_JPRB, 0.62368E+00_JPRB, 0.61112E+00_JPRB, 0.59923E+00_JPRB, 0.59312E+00_JPRB, 0.59835E+00_JPRB, 0.60892E+00_JPRB ,&
&  0.62956E+00_JPRB, 0.64236E+00_JPRB, 0.64852E+00_JPRB, 0.64999E+00_JPRB, 0.65316E+00_JPRB, 0.65402E+00_JPRB, 0.66657E+00_JPRB ,&
&  0.70035E+00_JPRB, 0.72947E+00_JPRB, 0.73670E+00_JPRB, 0.76917E+00_JPRB, 0.81077E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.88485E+00_JPRB, 0.88898E+00_JPRB, 0.87909E+00_JPRB, 0.87468E+00_JPRB, 0.86409E+00_JPRB, 0.85048E+00_JPRB, 0.82819E+00_JPRB ,&
&  0.80202E+00_JPRB, 0.77395E+00_JPRB, 0.74736E+00_JPRB, 0.72559E+00_JPRB, 0.72954E+00_JPRB, 0.76221E+00_JPRB, 0.78980E+00_JPRB ,&
&  0.77900E+00_JPRB, 0.77528E+00_JPRB, 0.79252E+00_JPRB, 0.82327E+00_JPRB, 0.85663E+00_JPRB, 0.88737E+00_JPRB, 0.90931E+00_JPRB ,&
&  0.92708E+00_JPRB, 0.90887E+00_JPRB, 0.91274E+00_JPRB, 0.91490E+00_JPRB, 0.90872E+00_JPRB, 0.89048E+00_JPRB, 0.86147E+00_JPRB ,&
&  0.83283E+00_JPRB, 0.80562E+00_JPRB, 0.77199E+00_JPRB, 0.71336E+00_JPRB, 0.78642E+00_JPRB, 0.71024E+00_JPRB, 0.68179E+00_JPRB ,&
&  0.65441E+00_JPRB, 0.63618E+00_JPRB, 0.61578E+00_JPRB, 0.59369E+00_JPRB, 0.58615E+00_JPRB, 0.58164E+00_JPRB, 0.57376E+00_JPRB ,&
&  0.56725E+00_JPRB, 0.56014E+00_JPRB, 0.54930E+00_JPRB, 0.53556E+00_JPRB, 0.52266E+00_JPRB, 0.50885E+00_JPRB, 0.49615E+00_JPRB ,&
&  0.47771E+00_JPRB, 0.45065E+00_JPRB, 0.42220E+00_JPRB, 0.40128E+00_JPRB, 0.39043E+00_JPRB, 0.39985E+00_JPRB, 0.43295E+00_JPRB ,&
&  0.46687E+00_JPRB, 0.47256E+00_JPRB, 0.48224E+00_JPRB, 0.48499E+00_JPRB, 0.47626E+00_JPRB, 0.46585E+00_JPRB, 0.44796E+00_JPRB ,&
&  0.43446E+00_JPRB, 0.43220E+00_JPRB, 0.43312E+00_JPRB, 0.43700E+00_JPRB, 0.44215E+00_JPRB, 0.44932E+00_JPRB, 0.45936E+00_JPRB ,&
&  0.46856E+00_JPRB, 0.47185E+00_JPRB, 0.48419E+00_JPRB, 0.51096E+00_JPRB, 0.54193E+00_JPRB, 0.57333E+00_JPRB, 0.60023E+00_JPRB ,&
&  0.61855E+00_JPRB, 0.63140E+00_JPRB, 0.63973E+00_JPRB, 0.65490E+00_JPRB, 0.67059E+00_JPRB, 0.68636E+00_JPRB, 0.69451E+00_JPRB ,&
&  0.69212E+00_JPRB, 0.69583E+00_JPRB, 0.71310E+00_JPRB, 0.74139E+00_JPRB, 0.76116E+00_JPRB, 0.79493E+00_JPRB, 0.81599E+00_JPRB ,&
&  0.83829E+00_JPRB, 0.84393E+00_JPRB, 0.83842E+00_JPRB, 0.85084E+00_JPRB, 0.86750E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.87027E+00_JPRB, 0.88073E+00_JPRB, 0.87849E+00_JPRB, 0.87736E+00_JPRB, 0.87908E+00_JPRB, 0.87210E+00_JPRB, 0.85820E+00_JPRB ,&
&  0.84416E+00_JPRB, 0.81769E+00_JPRB, 0.78726E+00_JPRB, 0.77358E+00_JPRB, 0.78100E+00_JPRB, 0.78083E+00_JPRB, 0.76227E+00_JPRB ,&
&  0.76072E+00_JPRB, 0.77462E+00_JPRB, 0.79772E+00_JPRB, 0.82337E+00_JPRB, 0.85781E+00_JPRB, 0.89350E+00_JPRB, 0.92012E+00_JPRB ,&
&  0.93066E+00_JPRB, 0.92880E+00_JPRB, 0.92114E+00_JPRB, 0.89441E+00_JPRB, 0.85860E+00_JPRB, 0.83105E+00_JPRB, 0.81015E+00_JPRB ,&
&  0.78434E+00_JPRB, 0.73622E+00_JPRB, 0.66703E+00_JPRB, 0.66136E+00_JPRB, 0.59794E+00_JPRB, 0.57826E+00_JPRB, 0.60062E+00_JPRB ,&
&  0.61599E+00_JPRB, 0.59808E+00_JPRB, 0.57422E+00_JPRB, 0.55443E+00_JPRB, 0.55064E+00_JPRB, 0.54929E+00_JPRB, 0.54420E+00_JPRB ,&
&  0.54035E+00_JPRB, 0.53364E+00_JPRB, 0.51956E+00_JPRB, 0.50666E+00_JPRB, 0.49449E+00_JPRB, 0.48231E+00_JPRB, 0.47393E+00_JPRB ,&
&  0.44562E+00_JPRB, 0.40237E+00_JPRB, 0.37014E+00_JPRB, 0.35608E+00_JPRB, 0.34829E+00_JPRB, 0.35200E+00_JPRB, 0.36488E+00_JPRB ,&
&  0.37537E+00_JPRB, 0.37606E+00_JPRB, 0.37048E+00_JPRB, 0.35769E+00_JPRB, 0.33769E+00_JPRB, 0.29269E+00_JPRB, 0.26671E+00_JPRB ,&
&  0.28264E+00_JPRB, 0.31576E+00_JPRB, 0.35037E+00_JPRB, 0.36027E+00_JPRB, 0.36959E+00_JPRB, 0.37176E+00_JPRB, 0.35694E+00_JPRB ,&
&  0.32850E+00_JPRB, 0.29888E+00_JPRB, 0.29323E+00_JPRB, 0.29540E+00_JPRB, 0.30021E+00_JPRB, 0.30893E+00_JPRB, 0.32482E+00_JPRB ,&
&  0.34884E+00_JPRB, 0.38709E+00_JPRB, 0.40932E+00_JPRB, 0.41203E+00_JPRB, 0.40477E+00_JPRB, 0.39340E+00_JPRB, 0.39124E+00_JPRB ,&
&  0.40524E+00_JPRB, 0.46572E+00_JPRB, 0.58732E+00_JPRB, 0.73414E+00_JPRB, 0.73424E+00_JPRB, 0.77843E+00_JPRB, 0.81311E+00_JPRB ,&
&  0.84283E+00_JPRB, 0.85616E+00_JPRB, 0.84286E+00_JPRB, 0.84396E+00_JPRB, 0.85038E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.83021E+00_JPRB, 0.85017E+00_JPRB, 0.86112E+00_JPRB, 0.87515E+00_JPRB, 0.87544E+00_JPRB, 0.84538E+00_JPRB, 0.83094E+00_JPRB ,&
&  0.83566E+00_JPRB, 0.83078E+00_JPRB, 0.81984E+00_JPRB, 0.81298E+00_JPRB, 0.80403E+00_JPRB, 0.79421E+00_JPRB, 0.78600E+00_JPRB ,&
&  0.77779E+00_JPRB, 0.77284E+00_JPRB, 0.77985E+00_JPRB, 0.79377E+00_JPRB, 0.80520E+00_JPRB, 0.81146E+00_JPRB, 0.81646E+00_JPRB ,&
&  0.81602E+00_JPRB, 0.78301E+00_JPRB, 0.73788E+00_JPRB, 0.70501E+00_JPRB, 0.68855E+00_JPRB, 0.67468E+00_JPRB, 0.64143E+00_JPRB ,&
&  0.57339E+00_JPRB, 0.45216E+00_JPRB, 0.39627E+00_JPRB, 0.43757E+00_JPRB, 0.55482E+00_JPRB, 0.52021E+00_JPRB, 0.54108E+00_JPRB ,&
&  0.57177E+00_JPRB, 0.58768E+00_JPRB, 0.58354E+00_JPRB, 0.57313E+00_JPRB, 0.56975E+00_JPRB, 0.57346E+00_JPRB, 0.57027E+00_JPRB ,&
&  0.56702E+00_JPRB, 0.55119E+00_JPRB, 0.54175E+00_JPRB, 0.53081E+00_JPRB, 0.51313E+00_JPRB, 0.48739E+00_JPRB, 0.47169E+00_JPRB ,&
&  0.43323E+00_JPRB, 0.39534E+00_JPRB, 0.38419E+00_JPRB, 0.38464E+00_JPRB, 0.35372E+00_JPRB, 0.34107E+00_JPRB, 0.32597E+00_JPRB ,&
&  0.28624E+00_JPRB, 0.26483E+00_JPRB, 0.23284E+00_JPRB, 0.21278E+00_JPRB, 0.19798E+00_JPRB, 0.18369E+00_JPRB, 0.18158E+00_JPRB ,&
&  0.18639E+00_JPRB, 0.19889E+00_JPRB, 0.22007E+00_JPRB, 0.25372E+00_JPRB, 0.28424E+00_JPRB, 0.24201E+00_JPRB, 0.22421E+00_JPRB ,&
&  0.21004E+00_JPRB, 0.20696E+00_JPRB, 0.21267E+00_JPRB, 0.21935E+00_JPRB, 0.22614E+00_JPRB, 0.23411E+00_JPRB, 0.24180E+00_JPRB ,&
&  0.24732E+00_JPRB, 0.25033E+00_JPRB, 0.25115E+00_JPRB, 0.25044E+00_JPRB, 0.24935E+00_JPRB, 0.25012E+00_JPRB, 0.25358E+00_JPRB ,&
&  0.25984E+00_JPRB, 0.27217E+00_JPRB, 0.29604E+00_JPRB, 0.33134E+00_JPRB, 0.37740E+00_JPRB, 0.43775E+00_JPRB, 0.52984E+00_JPRB ,&
&  0.65459E+00_JPRB, 0.81876E+00_JPRB, 0.86365E+00_JPRB, 0.87268E+00_JPRB, 0.84176E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.83590E+00_JPRB, 0.83349E+00_JPRB, 0.84122E+00_JPRB, 0.83098E+00_JPRB, 0.81203E+00_JPRB, 0.80312E+00_JPRB, 0.82020E+00_JPRB ,&
&  0.84003E+00_JPRB, 0.81059E+00_JPRB, 0.78892E+00_JPRB, 0.76969E+00_JPRB, 0.75597E+00_JPRB, 0.74213E+00_JPRB, 0.72011E+00_JPRB ,&
&  0.70011E+00_JPRB, 0.69282E+00_JPRB, 0.69095E+00_JPRB, 0.68594E+00_JPRB, 0.67573E+00_JPRB, 0.66571E+00_JPRB, 0.64806E+00_JPRB ,&
&  0.61526E+00_JPRB, 0.58083E+00_JPRB, 0.56291E+00_JPRB, 0.53935E+00_JPRB, 0.52260E+00_JPRB, 0.50319E+00_JPRB, 0.47558E+00_JPRB ,&
&  0.43048E+00_JPRB, 0.37258E+00_JPRB, 0.33715E+00_JPRB, 0.37384E+00_JPRB, 0.46814E+00_JPRB, 0.46168E+00_JPRB, 0.48118E+00_JPRB ,&
&  0.51595E+00_JPRB, 0.53723E+00_JPRB, 0.54785E+00_JPRB, 0.55139E+00_JPRB, 0.56078E+00_JPRB, 0.56666E+00_JPRB, 0.56330E+00_JPRB ,&
&  0.54864E+00_JPRB, 0.49648E+00_JPRB, 0.45377E+00_JPRB, 0.35054E+00_JPRB, 0.27941E+00_JPRB, 0.24188E+00_JPRB, 0.21245E+00_JPRB ,&
&  0.19323E+00_JPRB, 0.18191E+00_JPRB, 0.17936E+00_JPRB, 0.17874E+00_JPRB, 0.18405E+00_JPRB, 0.18982E+00_JPRB, 0.18968E+00_JPRB ,&
&  0.18564E+00_JPRB, 0.17761E+00_JPRB, 0.16625E+00_JPRB, 0.15942E+00_JPRB, 0.15582E+00_JPRB, 0.15590E+00_JPRB, 0.15780E+00_JPRB ,&
&  0.15919E+00_JPRB, 0.16312E+00_JPRB, 0.17559E+00_JPRB, 0.18646E+00_JPRB, 0.19992E+00_JPRB, 0.18211E+00_JPRB, 0.16751E+00_JPRB ,&
&  0.16229E+00_JPRB, 0.16677E+00_JPRB, 0.17361E+00_JPRB, 0.17876E+00_JPRB, 0.18419E+00_JPRB, 0.18875E+00_JPRB, 0.19332E+00_JPRB ,&
&  0.19782E+00_JPRB, 0.20097E+00_JPRB, 0.20253E+00_JPRB, 0.20424E+00_JPRB, 0.20579E+00_JPRB, 0.20811E+00_JPRB, 0.21100E+00_JPRB ,&
&  0.21597E+00_JPRB, 0.22536E+00_JPRB, 0.23760E+00_JPRB, 0.25784E+00_JPRB, 0.27683E+00_JPRB, 0.29574E+00_JPRB, 0.31756E+00_JPRB ,&
&  0.33837E+00_JPRB, 0.41880E+00_JPRB, 0.61237E+00_JPRB, 0.93305E+00_JPRB, 0.92663E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.78476E+00_JPRB, 0.79719E+00_JPRB, 0.81418E+00_JPRB, 0.79076E+00_JPRB, 0.77555E+00_JPRB, 0.77895E+00_JPRB, 0.77979E+00_JPRB ,&
&  0.75664E+00_JPRB, 0.72135E+00_JPRB, 0.67213E+00_JPRB, 0.63148E+00_JPRB, 0.56644E+00_JPRB, 0.48271E+00_JPRB, 0.44507E+00_JPRB ,&
&  0.41073E+00_JPRB, 0.40714E+00_JPRB, 0.40937E+00_JPRB, 0.42546E+00_JPRB, 0.43428E+00_JPRB, 0.43090E+00_JPRB, 0.40918E+00_JPRB ,&
&  0.38220E+00_JPRB, 0.35524E+00_JPRB, 0.33774E+00_JPRB, 0.32398E+00_JPRB, 0.32384E+00_JPRB, 0.32202E+00_JPRB, 0.31552E+00_JPRB ,&
&  0.30714E+00_JPRB, 0.31166E+00_JPRB, 0.32916E+00_JPRB, 0.35716E+00_JPRB, 0.40614E+00_JPRB, 0.38606E+00_JPRB, 0.39291E+00_JPRB ,&
&  0.42307E+00_JPRB, 0.44482E+00_JPRB, 0.45355E+00_JPRB, 0.45677E+00_JPRB, 0.44995E+00_JPRB, 0.47436E+00_JPRB, 0.41797E+00_JPRB ,&
&  0.31107E+00_JPRB, 0.25206E+00_JPRB, 0.20989E+00_JPRB, 0.18267E+00_JPRB, 0.16435E+00_JPRB, 0.15363E+00_JPRB, 0.14174E+00_JPRB ,&
&  0.13621E+00_JPRB, 0.13207E+00_JPRB, 0.13109E+00_JPRB, 0.13267E+00_JPRB, 0.13520E+00_JPRB, 0.13838E+00_JPRB, 0.13943E+00_JPRB ,&
&  0.14123E+00_JPRB, 0.14123E+00_JPRB, 0.14035E+00_JPRB, 0.14009E+00_JPRB, 0.13898E+00_JPRB, 0.13951E+00_JPRB, 0.13930E+00_JPRB ,&
&  0.13910E+00_JPRB, 0.13895E+00_JPRB, 0.14051E+00_JPRB, 0.14110E+00_JPRB, 0.14185E+00_JPRB, 0.13882E+00_JPRB, 0.13715E+00_JPRB ,&
&  0.13736E+00_JPRB, 0.13942E+00_JPRB, 0.14279E+00_JPRB, 0.14667E+00_JPRB, 0.15201E+00_JPRB, 0.15627E+00_JPRB, 0.16113E+00_JPRB ,&
&  0.16535E+00_JPRB, 0.16945E+00_JPRB, 0.17347E+00_JPRB, 0.17758E+00_JPRB, 0.18199E+00_JPRB, 0.18643E+00_JPRB, 0.19107E+00_JPRB ,&
&  0.19589E+00_JPRB, 0.20418E+00_JPRB, 0.21442E+00_JPRB, 0.23219E+00_JPRB, 0.25607E+00_JPRB, 0.28275E+00_JPRB, 0.32963E+00_JPRB ,&
&  0.38920E+00_JPRB, 0.61831E+00_JPRB, 0.91899E+00_JPRB, 0.74865E+00_JPRB, 0.76641E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.77579E+00_JPRB, 0.79416E+00_JPRB, 0.81036E+00_JPRB, 0.80776E+00_JPRB, 0.79054E+00_JPRB, 0.73298E+00_JPRB, 0.66966E+00_JPRB ,&
&  0.59111E+00_JPRB, 0.52518E+00_JPRB, 0.45588E+00_JPRB, 0.39596E+00_JPRB, 0.34560E+00_JPRB, 0.30690E+00_JPRB, 0.28355E+00_JPRB ,&
&  0.26706E+00_JPRB, 0.25984E+00_JPRB, 0.25707E+00_JPRB, 0.26167E+00_JPRB, 0.26406E+00_JPRB, 0.26956E+00_JPRB, 0.26399E+00_JPRB ,&
&  0.25710E+00_JPRB, 0.24919E+00_JPRB, 0.24717E+00_JPRB, 0.25588E+00_JPRB, 0.26915E+00_JPRB, 0.28204E+00_JPRB, 0.29881E+00_JPRB ,&
&  0.31712E+00_JPRB, 0.32302E+00_JPRB, 0.33727E+00_JPRB, 0.34095E+00_JPRB, 0.32636E+00_JPRB, 0.32815E+00_JPRB, 0.33405E+00_JPRB ,&
&  0.34731E+00_JPRB, 0.36592E+00_JPRB, 0.37660E+00_JPRB, 0.39666E+00_JPRB, 0.34363E+00_JPRB, 0.28359E+00_JPRB, 0.23177E+00_JPRB ,&
&  0.19317E+00_JPRB, 0.17118E+00_JPRB, 0.15567E+00_JPRB, 0.14779E+00_JPRB, 0.14013E+00_JPRB, 0.13595E+00_JPRB, 0.13151E+00_JPRB ,&
&  0.12950E+00_JPRB, 0.12939E+00_JPRB, 0.12885E+00_JPRB, 0.12848E+00_JPRB, 0.12882E+00_JPRB, 0.12942E+00_JPRB, 0.12880E+00_JPRB ,&
&  0.12800E+00_JPRB, 0.12767E+00_JPRB, 0.12714E+00_JPRB, 0.12670E+00_JPRB, 0.12581E+00_JPRB, 0.12555E+00_JPRB, 0.12384E+00_JPRB ,&
&  0.12288E+00_JPRB, 0.12141E+00_JPRB, 0.12054E+00_JPRB, 0.12041E+00_JPRB, 0.12072E+00_JPRB, 0.12104E+00_JPRB, 0.12162E+00_JPRB ,&
&  0.12281E+00_JPRB, 0.12387E+00_JPRB, 0.12568E+00_JPRB, 0.12744E+00_JPRB, 0.13029E+00_JPRB, 0.13286E+00_JPRB, 0.13648E+00_JPRB ,&
&  0.14032E+00_JPRB, 0.14496E+00_JPRB, 0.15027E+00_JPRB, 0.15624E+00_JPRB, 0.16138E+00_JPRB, 0.16643E+00_JPRB, 0.17192E+00_JPRB ,&
&  0.17628E+00_JPRB, 0.18349E+00_JPRB, 0.19165E+00_JPRB, 0.20511E+00_JPRB, 0.23322E+00_JPRB, 0.25684E+00_JPRB, 0.30542E+00_JPRB ,&
&  0.38733E+00_JPRB, 0.64126E+00_JPRB, 0.87337E+00_JPRB, 0.72601E+00_JPRB, 0.75734E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.65472E+00_JPRB, 0.68586E+00_JPRB, 0.71013E+00_JPRB, 0.71225E+00_JPRB, 0.71107E+00_JPRB, 0.67380E+00_JPRB, 0.61588E+00_JPRB ,&
&  0.50936E+00_JPRB, 0.43978E+00_JPRB, 0.39581E+00_JPRB, 0.35436E+00_JPRB, 0.31679E+00_JPRB, 0.29146E+00_JPRB, 0.27421E+00_JPRB ,&
&  0.26114E+00_JPRB, 0.25394E+00_JPRB, 0.24849E+00_JPRB, 0.24390E+00_JPRB, 0.23895E+00_JPRB, 0.23418E+00_JPRB, 0.22552E+00_JPRB ,&
&  0.22028E+00_JPRB, 0.21686E+00_JPRB, 0.21542E+00_JPRB, 0.21787E+00_JPRB, 0.22479E+00_JPRB, 0.23179E+00_JPRB, 0.24246E+00_JPRB ,&
&  0.25552E+00_JPRB, 0.26955E+00_JPRB, 0.27732E+00_JPRB, 0.28074E+00_JPRB, 0.28376E+00_JPRB, 0.28204E+00_JPRB, 0.25962E+00_JPRB ,&
&  0.26457E+00_JPRB, 0.28717E+00_JPRB, 0.30550E+00_JPRB, 0.31543E+00_JPRB, 0.27092E+00_JPRB, 0.22969E+00_JPRB, 0.18906E+00_JPRB ,&
&  0.16968E+00_JPRB, 0.15431E+00_JPRB, 0.14628E+00_JPRB, 0.14020E+00_JPRB, 0.13568E+00_JPRB, 0.13309E+00_JPRB, 0.13077E+00_JPRB ,&
&  0.12870E+00_JPRB, 0.12784E+00_JPRB, 0.12641E+00_JPRB, 0.12578E+00_JPRB, 0.12478E+00_JPRB, 0.12458E+00_JPRB, 0.12333E+00_JPRB ,&
&  0.12203E+00_JPRB, 0.12097E+00_JPRB, 0.12027E+00_JPRB, 0.11988E+00_JPRB, 0.11903E+00_JPRB, 0.11813E+00_JPRB, 0.11621E+00_JPRB ,&
&  0.11480E+00_JPRB, 0.11344E+00_JPRB, 0.11239E+00_JPRB, 0.11239E+00_JPRB, 0.11285E+00_JPRB, 0.11459E+00_JPRB, 0.11552E+00_JPRB ,&
&  0.11566E+00_JPRB, 0.11567E+00_JPRB, 0.11563E+00_JPRB, 0.11630E+00_JPRB, 0.11733E+00_JPRB, 0.11917E+00_JPRB, 0.12176E+00_JPRB ,&
&  0.12489E+00_JPRB, 0.12971E+00_JPRB, 0.13456E+00_JPRB, 0.14001E+00_JPRB, 0.14511E+00_JPRB, 0.15018E+00_JPRB, 0.15681E+00_JPRB ,&
&  0.16361E+00_JPRB, 0.17154E+00_JPRB, 0.18326E+00_JPRB, 0.19577E+00_JPRB, 0.21829E+00_JPRB, 0.23783E+00_JPRB, 0.26918E+00_JPRB ,&
&  0.33939E+00_JPRB, 0.48609E+00_JPRB, 0.57187E+00_JPRB, 0.58365E+00_JPRB, 0.61689E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.47500E+00_JPRB, 0.48656E+00_JPRB, 0.50170E+00_JPRB, 0.51089E+00_JPRB, 0.52100E+00_JPRB, 0.48843E+00_JPRB, 0.46276E+00_JPRB ,&
&  0.41017E+00_JPRB, 0.38009E+00_JPRB, 0.35508E+00_JPRB, 0.33210E+00_JPRB, 0.31165E+00_JPRB, 0.29358E+00_JPRB, 0.27638E+00_JPRB ,&
&  0.26535E+00_JPRB, 0.25871E+00_JPRB, 0.24967E+00_JPRB, 0.23958E+00_JPRB, 0.22881E+00_JPRB, 0.21940E+00_JPRB, 0.21447E+00_JPRB ,&
&  0.20988E+00_JPRB, 0.20658E+00_JPRB, 0.20526E+00_JPRB, 0.20546E+00_JPRB, 0.20738E+00_JPRB, 0.20767E+00_JPRB, 0.20841E+00_JPRB ,&
&  0.20545E+00_JPRB, 0.21262E+00_JPRB, 0.22207E+00_JPRB, 0.22638E+00_JPRB, 0.23432E+00_JPRB, 0.23296E+00_JPRB, 0.22823E+00_JPRB ,&
&  0.22118E+00_JPRB, 0.21283E+00_JPRB, 0.20342E+00_JPRB, 0.19178E+00_JPRB, 0.18135E+00_JPRB, 0.17362E+00_JPRB, 0.16601E+00_JPRB ,&
&  0.16142E+00_JPRB, 0.15589E+00_JPRB, 0.14943E+00_JPRB, 0.14385E+00_JPRB, 0.13902E+00_JPRB, 0.13502E+00_JPRB, 0.13138E+00_JPRB ,&
&  0.12869E+00_JPRB, 0.12708E+00_JPRB, 0.12552E+00_JPRB, 0.12503E+00_JPRB, 0.12387E+00_JPRB, 0.12271E+00_JPRB, 0.12157E+00_JPRB ,&
&  0.12091E+00_JPRB, 0.11963E+00_JPRB, 0.11862E+00_JPRB, 0.11769E+00_JPRB, 0.11633E+00_JPRB, 0.11516E+00_JPRB, 0.11384E+00_JPRB ,&
&  0.11269E+00_JPRB, 0.11206E+00_JPRB, 0.11168E+00_JPRB, 0.11155E+00_JPRB, 0.11177E+00_JPRB, 0.11168E+00_JPRB, 0.11188E+00_JPRB ,&
&  0.11214E+00_JPRB, 0.11253E+00_JPRB, 0.11248E+00_JPRB, 0.11275E+00_JPRB, 0.11355E+00_JPRB, 0.11469E+00_JPRB, 0.11750E+00_JPRB ,&
&  0.12075E+00_JPRB, 0.12493E+00_JPRB, 0.12950E+00_JPRB, 0.13477E+00_JPRB, 0.14008E+00_JPRB, 0.14539E+00_JPRB, 0.15120E+00_JPRB ,&
&  0.15881E+00_JPRB, 0.16564E+00_JPRB, 0.17357E+00_JPRB, 0.18559E+00_JPRB, 0.21138E+00_JPRB, 0.22561E+00_JPRB, 0.24055E+00_JPRB ,&
&  0.26704E+00_JPRB, 0.30339E+00_JPRB, 0.40680E+00_JPRB, 0.43952E+00_JPRB, 0.45691E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.21871E+00_JPRB, 0.21886E+00_JPRB, 0.21970E+00_JPRB, 0.22625E+00_JPRB, 0.23836E+00_JPRB, 0.31566E+00_JPRB, 0.30685E+00_JPRB ,&
&  0.29969E+00_JPRB, 0.27147E+00_JPRB, 0.26789E+00_JPRB, 0.26078E+00_JPRB, 0.25124E+00_JPRB, 0.24346E+00_JPRB, 0.23884E+00_JPRB ,&
&  0.23381E+00_JPRB, 0.22831E+00_JPRB, 0.22515E+00_JPRB, 0.22194E+00_JPRB, 0.21880E+00_JPRB, 0.21524E+00_JPRB, 0.21252E+00_JPRB ,&
&  0.21006E+00_JPRB, 0.20836E+00_JPRB, 0.20648E+00_JPRB, 0.20277E+00_JPRB, 0.20114E+00_JPRB, 0.19792E+00_JPRB, 0.19310E+00_JPRB ,&
&  0.18682E+00_JPRB, 0.18342E+00_JPRB, 0.17966E+00_JPRB, 0.17628E+00_JPRB, 0.17904E+00_JPRB, 0.17803E+00_JPRB, 0.17583E+00_JPRB ,&
&  0.17230E+00_JPRB, 0.16865E+00_JPRB, 0.16392E+00_JPRB, 0.15939E+00_JPRB, 0.15560E+00_JPRB, 0.15250E+00_JPRB, 0.14981E+00_JPRB ,&
&  0.14767E+00_JPRB, 0.14587E+00_JPRB, 0.14342E+00_JPRB, 0.14118E+00_JPRB, 0.13881E+00_JPRB, 0.13672E+00_JPRB, 0.13459E+00_JPRB ,&
&  0.13289E+00_JPRB, 0.13129E+00_JPRB, 0.12963E+00_JPRB, 0.12792E+00_JPRB, 0.12656E+00_JPRB, 0.12480E+00_JPRB, 0.12347E+00_JPRB ,&
&  0.12230E+00_JPRB, 0.12084E+00_JPRB, 0.11975E+00_JPRB, 0.11842E+00_JPRB, 0.11743E+00_JPRB, 0.11661E+00_JPRB, 0.11582E+00_JPRB ,&
&  0.11477E+00_JPRB, 0.11407E+00_JPRB, 0.11378E+00_JPRB, 0.11355E+00_JPRB, 0.11321E+00_JPRB, 0.11322E+00_JPRB, 0.11351E+00_JPRB ,&
&  0.11393E+00_JPRB, 0.11442E+00_JPRB, 0.11527E+00_JPRB, 0.11628E+00_JPRB, 0.11779E+00_JPRB, 0.11981E+00_JPRB, 0.12303E+00_JPRB ,&
&  0.12653E+00_JPRB, 0.13042E+00_JPRB, 0.13443E+00_JPRB, 0.13896E+00_JPRB, 0.14354E+00_JPRB, 0.14821E+00_JPRB, 0.15355E+00_JPRB ,&
&  0.15937E+00_JPRB, 0.16386E+00_JPRB, 0.16801E+00_JPRB, 0.17240E+00_JPRB, 0.18148E+00_JPRB, 0.18826E+00_JPRB, 0.19346E+00_JPRB ,&
&  0.19931E+00_JPRB, 0.20723E+00_JPRB, 0.21201E+00_JPRB, 0.21529E+00_JPRB, 0.21725E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.19165E+00_JPRB, 0.19426E+00_JPRB, 0.19600E+00_JPRB, 0.19711E+00_JPRB, 0.19687E+00_JPRB, 0.19779E+00_JPRB, 0.19875E+00_JPRB ,&
&  0.20002E+00_JPRB, 0.20161E+00_JPRB, 0.20273E+00_JPRB, 0.20377E+00_JPRB, 0.20427E+00_JPRB, 0.20428E+00_JPRB, 0.20429E+00_JPRB ,&
&  0.20411E+00_JPRB, 0.20387E+00_JPRB, 0.20056E+00_JPRB, 0.19759E+00_JPRB, 0.19547E+00_JPRB, 0.19429E+00_JPRB, 0.19322E+00_JPRB ,&
&  0.19234E+00_JPRB, 0.19113E+00_JPRB, 0.18969E+00_JPRB, 0.18754E+00_JPRB, 0.18429E+00_JPRB, 0.18112E+00_JPRB, 0.17786E+00_JPRB ,&
&  0.17491E+00_JPRB, 0.17250E+00_JPRB, 0.17028E+00_JPRB, 0.16804E+00_JPRB, 0.16667E+00_JPRB, 0.16512E+00_JPRB, 0.16351E+00_JPRB ,&
&  0.16150E+00_JPRB, 0.15933E+00_JPRB, 0.15726E+00_JPRB, 0.15515E+00_JPRB, 0.15312E+00_JPRB, 0.15111E+00_JPRB, 0.14965E+00_JPRB ,&
&  0.14811E+00_JPRB, 0.14695E+00_JPRB, 0.14566E+00_JPRB, 0.14448E+00_JPRB, 0.14325E+00_JPRB, 0.14208E+00_JPRB, 0.14074E+00_JPRB ,&
&  0.13983E+00_JPRB, 0.13887E+00_JPRB, 0.13790E+00_JPRB, 0.13700E+00_JPRB, 0.13610E+00_JPRB, 0.13524E+00_JPRB, 0.13425E+00_JPRB ,&
&  0.13336E+00_JPRB, 0.13277E+00_JPRB, 0.13216E+00_JPRB, 0.13148E+00_JPRB, 0.13085E+00_JPRB, 0.13041E+00_JPRB, 0.13003E+00_JPRB ,&
&  0.12967E+00_JPRB, 0.12963E+00_JPRB, 0.12979E+00_JPRB, 0.12994E+00_JPRB, 0.13010E+00_JPRB, 0.13048E+00_JPRB, 0.13105E+00_JPRB ,&
&  0.13172E+00_JPRB, 0.13246E+00_JPRB, 0.13340E+00_JPRB, 0.13474E+00_JPRB, 0.13623E+00_JPRB, 0.13794E+00_JPRB, 0.13982E+00_JPRB ,&
&  0.14184E+00_JPRB, 0.14402E+00_JPRB, 0.14632E+00_JPRB, 0.14859E+00_JPRB, 0.15104E+00_JPRB, 0.15347E+00_JPRB, 0.15582E+00_JPRB ,&
&  0.15827E+00_JPRB, 0.16070E+00_JPRB, 0.16318E+00_JPRB, 0.16545E+00_JPRB, 0.16890E+00_JPRB, 0.17115E+00_JPRB, 0.17351E+00_JPRB ,&
&  0.17570E+00_JPRB, 0.17824E+00_JPRB, 0.18080E+00_JPRB, 0.18397E+00_JPRB, 0.18712E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB, 0.15719E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB, 0.33710E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.35148E+00_JPRB, 0.35049E+00_JPRB, 0.34921E+00_JPRB, 0.34812E+00_JPRB, 0.34672E+00_JPRB, 0.34566E+00_JPRB, 0.34456E+00_JPRB ,&
&  0.34363E+00_JPRB, 0.34262E+00_JPRB, 0.34185E+00_JPRB, 0.34077E+00_JPRB, 0.33981E+00_JPRB, 0.33890E+00_JPRB, 0.33811E+00_JPRB ,&
&  0.33729E+00_JPRB, 0.33659E+00_JPRB, 0.33588E+00_JPRB, 0.33515E+00_JPRB, 0.33453E+00_JPRB, 0.33372E+00_JPRB, 0.33254E+00_JPRB ,&
&  0.33048E+00_JPRB, 0.32815E+00_JPRB, 0.32612E+00_JPRB, 0.32481E+00_JPRB, 0.32394E+00_JPRB, 0.32324E+00_JPRB, 0.32265E+00_JPRB ,&
&  0.32236E+00_JPRB, 0.32242E+00_JPRB, 0.32262E+00_JPRB, 0.32273E+00_JPRB, 0.32304E+00_JPRB, 0.32399E+00_JPRB, 0.32462E+00_JPRB ,&
&  0.32500E+00_JPRB, 0.32538E+00_JPRB, 0.32625E+00_JPRB, 0.32698E+00_JPRB, 0.32775E+00_JPRB, 0.32860E+00_JPRB, 0.32996E+00_JPRB ,&
&  0.33132E+00_JPRB, 0.33261E+00_JPRB, 0.33398E+00_JPRB, 0.33537E+00_JPRB, 0.33671E+00_JPRB, 0.33809E+00_JPRB, 0.33979E+00_JPRB ,&
&  0.34157E+00_JPRB, 0.34313E+00_JPRB, 0.34451E+00_JPRB, 0.34553E+00_JPRB, 0.34636E+00_JPRB, 0.34666E+00_JPRB, 0.34681E+00_JPRB ,&
&  0.34649E+00_JPRB, 0.34666E+00_JPRB, 0.34643E+00_JPRB, 0.34624E+00_JPRB, 0.34567E+00_JPRB, 0.34572E+00_JPRB, 0.34555E+00_JPRB ,&
&  0.34587E+00_JPRB, 0.34591E+00_JPRB, 0.34632E+00_JPRB, 0.34667E+00_JPRB, 0.34747E+00_JPRB, 0.34850E+00_JPRB, 0.34966E+00_JPRB ,&
&  0.35098E+00_JPRB, 0.35241E+00_JPRB, 0.35411E+00_JPRB, 0.35489E+00_JPRB, 0.35566E+00_JPRB, 0.35645E+00_JPRB, 0.35745E+00_JPRB ,&
&  0.35815E+00_JPRB, 0.35881E+00_JPRB, 0.35958E+00_JPRB, 0.36014E+00_JPRB, 0.36036E+00_JPRB, 0.36043E+00_JPRB, 0.36052E+00_JPRB ,&
&  0.36035E+00_JPRB, 0.36019E+00_JPRB, 0.35996E+00_JPRB, 0.35981E+00_JPRB, 0.35919E+00_JPRB, 0.35842E+00_JPRB, 0.35745E+00_JPRB ,&
&  0.35669E+00_JPRB, 0.35566E+00_JPRB, 0.35472E+00_JPRB, 0.35368E+00_JPRB, 0.35269E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.38615E+00_JPRB, 0.38423E+00_JPRB, 0.38220E+00_JPRB, 0.38031E+00_JPRB, 0.37768E+00_JPRB, 0.37500E+00_JPRB, 0.37225E+00_JPRB ,&
&  0.36946E+00_JPRB, 0.36630E+00_JPRB, 0.36301E+00_JPRB, 0.35952E+00_JPRB, 0.35591E+00_JPRB, 0.35253E+00_JPRB, 0.34978E+00_JPRB ,&
&  0.34735E+00_JPRB, 0.34469E+00_JPRB, 0.34253E+00_JPRB, 0.33966E+00_JPRB, 0.33670E+00_JPRB, 0.33418E+00_JPRB, 0.33270E+00_JPRB ,&
&  0.32993E+00_JPRB, 0.32648E+00_JPRB, 0.32283E+00_JPRB, 0.31885E+00_JPRB, 0.31513E+00_JPRB, 0.31139E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30375E+00_JPRB, 0.30102E+00_JPRB, 0.29870E+00_JPRB, 0.29637E+00_JPRB, 0.29405E+00_JPRB, 0.29384E+00_JPRB, 0.29345E+00_JPRB ,&
&  0.29321E+00_JPRB, 0.29364E+00_JPRB, 0.29620E+00_JPRB, 0.29856E+00_JPRB, 0.30098E+00_JPRB, 0.30344E+00_JPRB, 0.30740E+00_JPRB ,&
&  0.31107E+00_JPRB, 0.31546E+00_JPRB, 0.32074E+00_JPRB, 0.32580E+00_JPRB, 0.32977E+00_JPRB, 0.33248E+00_JPRB, 0.33592E+00_JPRB ,&
&  0.33816E+00_JPRB, 0.34117E+00_JPRB, 0.34317E+00_JPRB, 0.34394E+00_JPRB, 0.34347E+00_JPRB, 0.34301E+00_JPRB, 0.34308E+00_JPRB ,&
&  0.34266E+00_JPRB, 0.34385E+00_JPRB, 0.34496E+00_JPRB, 0.34607E+00_JPRB, 0.34603E+00_JPRB, 0.34759E+00_JPRB, 0.34879E+00_JPRB ,&
&  0.35147E+00_JPRB, 0.35550E+00_JPRB, 0.35957E+00_JPRB, 0.36411E+00_JPRB, 0.36981E+00_JPRB, 0.37669E+00_JPRB, 0.38204E+00_JPRB ,&
&  0.38730E+00_JPRB, 0.39132E+00_JPRB, 0.39428E+00_JPRB, 0.39537E+00_JPRB, 0.39642E+00_JPRB, 0.39751E+00_JPRB, 0.39937E+00_JPRB ,&
&  0.40001E+00_JPRB, 0.40087E+00_JPRB, 0.40177E+00_JPRB, 0.40237E+00_JPRB, 0.40163E+00_JPRB, 0.40082E+00_JPRB, 0.40042E+00_JPRB ,&
&  0.39980E+00_JPRB, 0.39902E+00_JPRB, 0.39832E+00_JPRB, 0.39790E+00_JPRB, 0.39712E+00_JPRB, 0.39616E+00_JPRB, 0.39521E+00_JPRB ,&
&  0.39430E+00_JPRB, 0.39289E+00_JPRB, 0.39126E+00_JPRB, 0.38963E+00_JPRB, 0.38814E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.39943E+00_JPRB, 0.39911E+00_JPRB, 0.39868E+00_JPRB, 0.39807E+00_JPRB, 0.39659E+00_JPRB, 0.39477E+00_JPRB, 0.39244E+00_JPRB ,&
&  0.38958E+00_JPRB, 0.38554E+00_JPRB, 0.38133E+00_JPRB, 0.37689E+00_JPRB, 0.37276E+00_JPRB, 0.36939E+00_JPRB, 0.36622E+00_JPRB ,&
&  0.36272E+00_JPRB, 0.35801E+00_JPRB, 0.35256E+00_JPRB, 0.34541E+00_JPRB, 0.33883E+00_JPRB, 0.33317E+00_JPRB, 0.32700E+00_JPRB ,&
&  0.31795E+00_JPRB, 0.30804E+00_JPRB, 0.29923E+00_JPRB, 0.29026E+00_JPRB, 0.28359E+00_JPRB, 0.26435E+00_JPRB, 0.25839E+00_JPRB ,&
&  0.25435E+00_JPRB, 0.25050E+00_JPRB, 0.24908E+00_JPRB, 0.24828E+00_JPRB, 0.25008E+00_JPRB, 0.24737E+00_JPRB, 0.24589E+00_JPRB ,&
&  0.23656E+00_JPRB, 0.23758E+00_JPRB, 0.23984E+00_JPRB, 0.24279E+00_JPRB, 0.24892E+00_JPRB, 0.25613E+00_JPRB, 0.26764E+00_JPRB ,&
&  0.28842E+00_JPRB, 0.29761E+00_JPRB, 0.31124E+00_JPRB, 0.32173E+00_JPRB, 0.32768E+00_JPRB, 0.33072E+00_JPRB, 0.33387E+00_JPRB ,&
&  0.33734E+00_JPRB, 0.34233E+00_JPRB, 0.34516E+00_JPRB, 0.34703E+00_JPRB, 0.34666E+00_JPRB, 0.34543E+00_JPRB, 0.34479E+00_JPRB ,&
&  0.34357E+00_JPRB, 0.34448E+00_JPRB, 0.34655E+00_JPRB, 0.35154E+00_JPRB, 0.35857E+00_JPRB, 0.36770E+00_JPRB, 0.37763E+00_JPRB ,&
&  0.38799E+00_JPRB, 0.39976E+00_JPRB, 0.40932E+00_JPRB, 0.41859E+00_JPRB, 0.42836E+00_JPRB, 0.43660E+00_JPRB, 0.44323E+00_JPRB ,&
&  0.44781E+00_JPRB, 0.44831E+00_JPRB, 0.44489E+00_JPRB, 0.44094E+00_JPRB, 0.43643E+00_JPRB, 0.43314E+00_JPRB, 0.43210E+00_JPRB ,&
&  0.43104E+00_JPRB, 0.43062E+00_JPRB, 0.43061E+00_JPRB, 0.42868E+00_JPRB, 0.42655E+00_JPRB, 0.42397E+00_JPRB, 0.42142E+00_JPRB ,&
&  0.41845E+00_JPRB, 0.41588E+00_JPRB, 0.41359E+00_JPRB, 0.41144E+00_JPRB, 0.40968E+00_JPRB, 0.40722E+00_JPRB, 0.40525E+00_JPRB ,&
&  0.40318E+00_JPRB, 0.40260E+00_JPRB, 0.40145E+00_JPRB, 0.40063E+00_JPRB, 0.39977E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.37586E+00_JPRB, 0.37617E+00_JPRB, 0.37701E+00_JPRB, 0.37750E+00_JPRB, 0.37864E+00_JPRB, 0.37940E+00_JPRB, 0.38046E+00_JPRB ,&
&  0.38101E+00_JPRB, 0.38159E+00_JPRB, 0.38132E+00_JPRB, 0.38058E+00_JPRB, 0.37904E+00_JPRB, 0.37763E+00_JPRB, 0.37595E+00_JPRB ,&
&  0.37333E+00_JPRB, 0.36764E+00_JPRB, 0.35958E+00_JPRB, 0.35044E+00_JPRB, 0.34002E+00_JPRB, 0.32862E+00_JPRB, 0.31558E+00_JPRB ,&
&  0.30183E+00_JPRB, 0.28892E+00_JPRB, 0.27195E+00_JPRB, 0.26522E+00_JPRB, 0.25433E+00_JPRB, 0.24604E+00_JPRB, 0.24028E+00_JPRB ,&
&  0.23336E+00_JPRB, 0.22853E+00_JPRB, 0.22364E+00_JPRB, 0.21846E+00_JPRB, 0.21474E+00_JPRB, 0.20847E+00_JPRB, 0.20523E+00_JPRB ,&
&  0.20297E+00_JPRB, 0.20197E+00_JPRB, 0.20215E+00_JPRB, 0.20384E+00_JPRB, 0.20771E+00_JPRB, 0.21273E+00_JPRB, 0.22254E+00_JPRB ,&
&  0.24409E+00_JPRB, 0.27522E+00_JPRB, 0.32228E+00_JPRB, 0.33126E+00_JPRB, 0.33025E+00_JPRB, 0.33652E+00_JPRB, 0.34310E+00_JPRB ,&
&  0.34933E+00_JPRB, 0.35399E+00_JPRB, 0.35755E+00_JPRB, 0.35955E+00_JPRB, 0.36220E+00_JPRB, 0.36443E+00_JPRB, 0.36677E+00_JPRB ,&
&  0.36912E+00_JPRB, 0.37497E+00_JPRB, 0.38315E+00_JPRB, 0.39314E+00_JPRB, 0.40347E+00_JPRB, 0.41148E+00_JPRB, 0.41920E+00_JPRB ,&
&  0.42542E+00_JPRB, 0.43271E+00_JPRB, 0.44282E+00_JPRB, 0.45428E+00_JPRB, 0.46614E+00_JPRB, 0.47297E+00_JPRB, 0.47829E+00_JPRB ,&
&  0.48121E+00_JPRB, 0.47873E+00_JPRB, 0.47253E+00_JPRB, 0.46529E+00_JPRB, 0.45992E+00_JPRB, 0.46134E+00_JPRB, 0.45927E+00_JPRB ,&
&  0.45776E+00_JPRB, 0.45288E+00_JPRB, 0.44556E+00_JPRB, 0.43667E+00_JPRB, 0.42998E+00_JPRB, 0.42386E+00_JPRB, 0.41981E+00_JPRB ,&
&  0.41621E+00_JPRB, 0.41265E+00_JPRB, 0.40818E+00_JPRB, 0.40241E+00_JPRB, 0.39665E+00_JPRB, 0.39064E+00_JPRB, 0.38594E+00_JPRB ,&
&  0.38173E+00_JPRB, 0.37928E+00_JPRB, 0.37723E+00_JPRB, 0.37635E+00_JPRB, 0.37555E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.36139E+00_JPRB, 0.36485E+00_JPRB, 0.36833E+00_JPRB, 0.37089E+00_JPRB, 0.37301E+00_JPRB, 0.37465E+00_JPRB, 0.37524E+00_JPRB ,&
&  0.37549E+00_JPRB, 0.37540E+00_JPRB, 0.37563E+00_JPRB, 0.37572E+00_JPRB, 0.37613E+00_JPRB, 0.37698E+00_JPRB, 0.37761E+00_JPRB ,&
&  0.37813E+00_JPRB, 0.37393E+00_JPRB, 0.36507E+00_JPRB, 0.35494E+00_JPRB, 0.34417E+00_JPRB, 0.33064E+00_JPRB, 0.31456E+00_JPRB ,&
&  0.30001E+00_JPRB, 0.28921E+00_JPRB, 0.27696E+00_JPRB, 0.26883E+00_JPRB, 0.25985E+00_JPRB, 0.25488E+00_JPRB, 0.24785E+00_JPRB ,&
&  0.24607E+00_JPRB, 0.22817E+00_JPRB, 0.22555E+00_JPRB, 0.21425E+00_JPRB, 0.20546E+00_JPRB, 0.19831E+00_JPRB, 0.19309E+00_JPRB ,&
&  0.19154E+00_JPRB, 0.18966E+00_JPRB, 0.18976E+00_JPRB, 0.19225E+00_JPRB, 0.19559E+00_JPRB, 0.20266E+00_JPRB, 0.21091E+00_JPRB ,&
&  0.23049E+00_JPRB, 0.29499E+00_JPRB, 0.31786E+00_JPRB, 0.32508E+00_JPRB, 0.32980E+00_JPRB, 0.34138E+00_JPRB, 0.35203E+00_JPRB ,&
&  0.36499E+00_JPRB, 0.37227E+00_JPRB, 0.37810E+00_JPRB, 0.38041E+00_JPRB, 0.39160E+00_JPRB, 0.40160E+00_JPRB, 0.40989E+00_JPRB ,&
&  0.41858E+00_JPRB, 0.42686E+00_JPRB, 0.43646E+00_JPRB, 0.44168E+00_JPRB, 0.44128E+00_JPRB, 0.43877E+00_JPRB, 0.43619E+00_JPRB ,&
&  0.43687E+00_JPRB, 0.43885E+00_JPRB, 0.45084E+00_JPRB, 0.46769E+00_JPRB, 0.48747E+00_JPRB, 0.50141E+00_JPRB, 0.50774E+00_JPRB ,&
&  0.51066E+00_JPRB, 0.51041E+00_JPRB, 0.51335E+00_JPRB, 0.51909E+00_JPRB, 0.51303E+00_JPRB, 0.50142E+00_JPRB, 0.49430E+00_JPRB ,&
&  0.48724E+00_JPRB, 0.47219E+00_JPRB, 0.45984E+00_JPRB, 0.44897E+00_JPRB, 0.44046E+00_JPRB, 0.43270E+00_JPRB, 0.42554E+00_JPRB ,&
&  0.41701E+00_JPRB, 0.40827E+00_JPRB, 0.39865E+00_JPRB, 0.38646E+00_JPRB, 0.37614E+00_JPRB, 0.36559E+00_JPRB, 0.35938E+00_JPRB ,&
&  0.35422E+00_JPRB, 0.35330E+00_JPRB, 0.35258E+00_JPRB, 0.35504E+00_JPRB, 0.35701E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.35296E+00_JPRB, 0.36121E+00_JPRB, 0.36807E+00_JPRB, 0.37285E+00_JPRB, 0.37584E+00_JPRB, 0.37765E+00_JPRB, 0.37731E+00_JPRB ,&
&  0.37662E+00_JPRB, 0.37515E+00_JPRB, 0.37454E+00_JPRB, 0.37351E+00_JPRB, 0.37362E+00_JPRB, 0.37299E+00_JPRB, 0.37284E+00_JPRB ,&
&  0.37312E+00_JPRB, 0.37240E+00_JPRB, 0.36788E+00_JPRB, 0.35887E+00_JPRB, 0.34782E+00_JPRB, 0.33563E+00_JPRB, 0.31882E+00_JPRB ,&
&  0.30255E+00_JPRB, 0.29237E+00_JPRB, 0.28250E+00_JPRB, 0.27403E+00_JPRB, 0.26911E+00_JPRB, 0.26284E+00_JPRB, 0.26197E+00_JPRB ,&
&  0.26004E+00_JPRB, 0.25397E+00_JPRB, 0.24638E+00_JPRB, 0.23864E+00_JPRB, 0.23014E+00_JPRB, 0.20512E+00_JPRB, 0.20117E+00_JPRB ,&
&  0.19447E+00_JPRB, 0.18881E+00_JPRB, 0.18742E+00_JPRB, 0.18830E+00_JPRB, 0.19093E+00_JPRB, 0.19868E+00_JPRB, 0.20904E+00_JPRB ,&
&  0.22495E+00_JPRB, 0.29928E+00_JPRB, 0.31485E+00_JPRB, 0.32536E+00_JPRB, 0.34167E+00_JPRB, 0.35769E+00_JPRB, 0.37136E+00_JPRB ,&
&  0.38338E+00_JPRB, 0.39254E+00_JPRB, 0.40419E+00_JPRB, 0.41435E+00_JPRB, 0.42597E+00_JPRB, 0.43620E+00_JPRB, 0.44266E+00_JPRB ,&
&  0.44767E+00_JPRB, 0.44680E+00_JPRB, 0.46051E+00_JPRB, 0.46536E+00_JPRB, 0.46173E+00_JPRB, 0.47148E+00_JPRB, 0.47562E+00_JPRB ,&
&  0.47197E+00_JPRB, 0.46468E+00_JPRB, 0.47282E+00_JPRB, 0.49616E+00_JPRB, 0.53021E+00_JPRB, 0.55654E+00_JPRB, 0.56304E+00_JPRB ,&
&  0.56456E+00_JPRB, 0.55984E+00_JPRB, 0.55470E+00_JPRB, 0.55306E+00_JPRB, 0.53751E+00_JPRB, 0.51465E+00_JPRB, 0.49859E+00_JPRB ,&
&  0.48530E+00_JPRB, 0.48285E+00_JPRB, 0.47509E+00_JPRB, 0.46602E+00_JPRB, 0.45576E+00_JPRB, 0.44504E+00_JPRB, 0.43063E+00_JPRB ,&
&  0.41124E+00_JPRB, 0.38981E+00_JPRB, 0.37303E+00_JPRB, 0.35816E+00_JPRB, 0.34997E+00_JPRB, 0.34075E+00_JPRB, 0.33652E+00_JPRB ,&
&  0.33249E+00_JPRB, 0.33303E+00_JPRB, 0.33421E+00_JPRB, 0.33903E+00_JPRB, 0.34452E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.35582E+00_JPRB, 0.36912E+00_JPRB, 0.37994E+00_JPRB, 0.38637E+00_JPRB, 0.38776E+00_JPRB, 0.38821E+00_JPRB, 0.38691E+00_JPRB ,&
&  0.38607E+00_JPRB, 0.38485E+00_JPRB, 0.38315E+00_JPRB, 0.37995E+00_JPRB, 0.37680E+00_JPRB, 0.37284E+00_JPRB, 0.37030E+00_JPRB ,&
&  0.36741E+00_JPRB, 0.36143E+00_JPRB, 0.35731E+00_JPRB, 0.35300E+00_JPRB, 0.34549E+00_JPRB, 0.33550E+00_JPRB, 0.32169E+00_JPRB ,&
&  0.30639E+00_JPRB, 0.29511E+00_JPRB, 0.28761E+00_JPRB, 0.27954E+00_JPRB, 0.27268E+00_JPRB, 0.26805E+00_JPRB, 0.26588E+00_JPRB ,&
&  0.26578E+00_JPRB, 0.26220E+00_JPRB, 0.25979E+00_JPRB, 0.24973E+00_JPRB, 0.23693E+00_JPRB, 0.23201E+00_JPRB, 0.21599E+00_JPRB ,&
&  0.20216E+00_JPRB, 0.19231E+00_JPRB, 0.18990E+00_JPRB, 0.19017E+00_JPRB, 0.19444E+00_JPRB, 0.20371E+00_JPRB, 0.22427E+00_JPRB ,&
&  0.25974E+00_JPRB, 0.31790E+00_JPRB, 0.33004E+00_JPRB, 0.34640E+00_JPRB, 0.36463E+00_JPRB, 0.38211E+00_JPRB, 0.39471E+00_JPRB ,&
&  0.40347E+00_JPRB, 0.41116E+00_JPRB, 0.42429E+00_JPRB, 0.43703E+00_JPRB, 0.45885E+00_JPRB, 0.47946E+00_JPRB, 0.49639E+00_JPRB ,&
&  0.51192E+00_JPRB, 0.48922E+00_JPRB, 0.49144E+00_JPRB, 0.49467E+00_JPRB, 0.51071E+00_JPRB, 0.52318E+00_JPRB, 0.51565E+00_JPRB ,&
&  0.51220E+00_JPRB, 0.53096E+00_JPRB, 0.53262E+00_JPRB, 0.54558E+00_JPRB, 0.57480E+00_JPRB, 0.60649E+00_JPRB, 0.61774E+00_JPRB ,&
&  0.60510E+00_JPRB, 0.60651E+00_JPRB, 0.64104E+00_JPRB, 0.61006E+00_JPRB, 0.59134E+00_JPRB, 0.56045E+00_JPRB, 0.52495E+00_JPRB ,&
&  0.50593E+00_JPRB, 0.49525E+00_JPRB, 0.48973E+00_JPRB, 0.48225E+00_JPRB, 0.47053E+00_JPRB, 0.45408E+00_JPRB, 0.42709E+00_JPRB ,&
&  0.39332E+00_JPRB, 0.36879E+00_JPRB, 0.35155E+00_JPRB, 0.34046E+00_JPRB, 0.33313E+00_JPRB, 0.32503E+00_JPRB, 0.31928E+00_JPRB ,&
&  0.31589E+00_JPRB, 0.31803E+00_JPRB, 0.32419E+00_JPRB, 0.33479E+00_JPRB, 0.34575E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.37236E+00_JPRB, 0.38611E+00_JPRB, 0.39444E+00_JPRB, 0.39535E+00_JPRB, 0.39018E+00_JPRB, 0.37989E+00_JPRB, 0.37167E+00_JPRB ,&
&  0.36914E+00_JPRB, 0.36948E+00_JPRB, 0.37087E+00_JPRB, 0.36854E+00_JPRB, 0.36629E+00_JPRB, 0.36416E+00_JPRB, 0.36300E+00_JPRB ,&
&  0.36181E+00_JPRB, 0.36084E+00_JPRB, 0.35953E+00_JPRB, 0.35348E+00_JPRB, 0.34503E+00_JPRB, 0.33458E+00_JPRB, 0.32294E+00_JPRB ,&
&  0.31115E+00_JPRB, 0.30195E+00_JPRB, 0.29456E+00_JPRB, 0.28873E+00_JPRB, 0.28156E+00_JPRB, 0.27740E+00_JPRB, 0.27438E+00_JPRB ,&
&  0.27200E+00_JPRB, 0.27215E+00_JPRB, 0.27046E+00_JPRB, 0.26539E+00_JPRB, 0.25492E+00_JPRB, 0.24215E+00_JPRB, 0.22641E+00_JPRB ,&
&  0.21357E+00_JPRB, 0.20546E+00_JPRB, 0.20082E+00_JPRB, 0.20045E+00_JPRB, 0.20719E+00_JPRB, 0.22237E+00_JPRB, 0.25396E+00_JPRB ,&
&  0.31490E+00_JPRB, 0.33610E+00_JPRB, 0.34819E+00_JPRB, 0.36448E+00_JPRB, 0.37962E+00_JPRB, 0.39216E+00_JPRB, 0.40351E+00_JPRB ,&
&  0.41230E+00_JPRB, 0.42085E+00_JPRB, 0.43184E+00_JPRB, 0.44061E+00_JPRB, 0.45695E+00_JPRB, 0.46326E+00_JPRB, 0.50283E+00_JPRB ,&
&  0.59501E+00_JPRB, 0.59468E+00_JPRB, 0.54839E+00_JPRB, 0.55176E+00_JPRB, 0.56792E+00_JPRB, 0.58358E+00_JPRB, 0.60110E+00_JPRB ,&
&  0.61435E+00_JPRB, 0.61832E+00_JPRB, 0.62028E+00_JPRB, 0.61909E+00_JPRB, 0.62825E+00_JPRB, 0.64038E+00_JPRB, 0.64203E+00_JPRB ,&
&  0.63616E+00_JPRB, 0.65625E+00_JPRB, 0.71080E+00_JPRB, 0.59508E+00_JPRB, 0.54335E+00_JPRB, 0.52784E+00_JPRB, 0.50648E+00_JPRB ,&
&  0.49333E+00_JPRB, 0.49014E+00_JPRB, 0.49551E+00_JPRB, 0.49453E+00_JPRB, 0.46525E+00_JPRB, 0.42626E+00_JPRB, 0.38628E+00_JPRB ,&
&  0.36463E+00_JPRB, 0.35303E+00_JPRB, 0.34015E+00_JPRB, 0.33141E+00_JPRB, 0.32462E+00_JPRB, 0.31709E+00_JPRB, 0.30980E+00_JPRB ,&
&  0.31120E+00_JPRB, 0.31788E+00_JPRB, 0.32864E+00_JPRB, 0.34687E+00_JPRB, 0.36100E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.37113E+00_JPRB, 0.39026E+00_JPRB, 0.39745E+00_JPRB, 0.39001E+00_JPRB, 0.38100E+00_JPRB, 0.35916E+00_JPRB, 0.33894E+00_JPRB ,&
&  0.33134E+00_JPRB, 0.32678E+00_JPRB, 0.32619E+00_JPRB, 0.32611E+00_JPRB, 0.32766E+00_JPRB, 0.33108E+00_JPRB, 0.33435E+00_JPRB ,&
&  0.33476E+00_JPRB, 0.33478E+00_JPRB, 0.33591E+00_JPRB, 0.33743E+00_JPRB, 0.33472E+00_JPRB, 0.32688E+00_JPRB, 0.31803E+00_JPRB ,&
&  0.30686E+00_JPRB, 0.29917E+00_JPRB, 0.29383E+00_JPRB, 0.28990E+00_JPRB, 0.28751E+00_JPRB, 0.28738E+00_JPRB, 0.28824E+00_JPRB ,&
&  0.28672E+00_JPRB, 0.28515E+00_JPRB, 0.28526E+00_JPRB, 0.28319E+00_JPRB, 0.27369E+00_JPRB, 0.25901E+00_JPRB, 0.23996E+00_JPRB ,&
&  0.22633E+00_JPRB, 0.22072E+00_JPRB, 0.22214E+00_JPRB, 0.23212E+00_JPRB, 0.25950E+00_JPRB, 0.31215E+00_JPRB, 0.33248E+00_JPRB ,&
&  0.34984E+00_JPRB, 0.36113E+00_JPRB, 0.37252E+00_JPRB, 0.38780E+00_JPRB, 0.40062E+00_JPRB, 0.41088E+00_JPRB, 0.42062E+00_JPRB ,&
&  0.43436E+00_JPRB, 0.44811E+00_JPRB, 0.46660E+00_JPRB, 0.43204E+00_JPRB, 0.44173E+00_JPRB, 0.51406E+00_JPRB, 0.54522E+00_JPRB ,&
&  0.59653E+00_JPRB, 0.60862E+00_JPRB, 0.60976E+00_JPRB, 0.65262E+00_JPRB, 0.66761E+00_JPRB, 0.69042E+00_JPRB, 0.69380E+00_JPRB ,&
&  0.68705E+00_JPRB, 0.67214E+00_JPRB, 0.65709E+00_JPRB, 0.64365E+00_JPRB, 0.64903E+00_JPRB, 0.66680E+00_JPRB, 0.65719E+00_JPRB ,&
&  0.64598E+00_JPRB, 0.63893E+00_JPRB, 0.59305E+00_JPRB, 0.52067E+00_JPRB, 0.49481E+00_JPRB, 0.49070E+00_JPRB, 0.49571E+00_JPRB ,&
&  0.49263E+00_JPRB, 0.49427E+00_JPRB, 0.49576E+00_JPRB, 0.48976E+00_JPRB, 0.43942E+00_JPRB, 0.39290E+00_JPRB, 0.35561E+00_JPRB ,&
&  0.33596E+00_JPRB, 0.33146E+00_JPRB, 0.32413E+00_JPRB, 0.32359E+00_JPRB, 0.31906E+00_JPRB, 0.31621E+00_JPRB, 0.31089E+00_JPRB ,&
&  0.31099E+00_JPRB, 0.31629E+00_JPRB, 0.33043E+00_JPRB, 0.34737E+00_JPRB, 0.35938E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.39685E+00_JPRB, 0.40218E+00_JPRB, 0.39443E+00_JPRB, 0.38947E+00_JPRB, 0.38026E+00_JPRB, 0.36911E+00_JPRB, 0.35514E+00_JPRB ,&
&  0.35599E+00_JPRB, 0.33701E+00_JPRB, 0.32299E+00_JPRB, 0.31048E+00_JPRB, 0.30305E+00_JPRB, 0.29881E+00_JPRB, 0.30205E+00_JPRB ,&
&  0.31070E+00_JPRB, 0.32187E+00_JPRB, 0.32838E+00_JPRB, 0.32781E+00_JPRB, 0.32348E+00_JPRB, 0.31456E+00_JPRB, 0.30819E+00_JPRB ,&
&  0.29964E+00_JPRB, 0.29534E+00_JPRB, 0.29594E+00_JPRB, 0.29892E+00_JPRB, 0.30209E+00_JPRB, 0.30220E+00_JPRB, 0.29890E+00_JPRB ,&
&  0.29389E+00_JPRB, 0.29119E+00_JPRB, 0.29314E+00_JPRB, 0.29594E+00_JPRB, 0.29130E+00_JPRB, 0.27297E+00_JPRB, 0.25378E+00_JPRB ,&
&  0.23962E+00_JPRB, 0.23912E+00_JPRB, 0.26115E+00_JPRB, 0.31611E+00_JPRB, 0.33661E+00_JPRB, 0.35164E+00_JPRB, 0.36324E+00_JPRB ,&
&  0.35940E+00_JPRB, 0.36403E+00_JPRB, 0.38580E+00_JPRB, 0.42716E+00_JPRB, 0.43582E+00_JPRB, 0.44469E+00_JPRB, 0.45377E+00_JPRB ,&
&  0.47263E+00_JPRB, 0.47284E+00_JPRB, 0.48697E+00_JPRB, 0.47741E+00_JPRB, 0.57143E+00_JPRB, 0.75732E+00_JPRB, 0.69508E+00_JPRB ,&
&  0.71199E+00_JPRB, 0.71596E+00_JPRB, 0.75989E+00_JPRB, 0.77378E+00_JPRB, 0.76360E+00_JPRB, 0.76002E+00_JPRB, 0.75133E+00_JPRB ,&
&  0.71894E+00_JPRB, 0.70027E+00_JPRB, 0.67430E+00_JPRB, 0.66999E+00_JPRB, 0.66644E+00_JPRB, 0.67117E+00_JPRB, 0.66822E+00_JPRB ,&
&  0.65993E+00_JPRB, 0.64895E+00_JPRB, 0.60623E+00_JPRB, 0.54882E+00_JPRB, 0.51921E+00_JPRB, 0.49418E+00_JPRB, 0.47869E+00_JPRB ,&
&  0.47006E+00_JPRB, 0.47868E+00_JPRB, 0.49095E+00_JPRB, 0.48909E+00_JPRB, 0.44488E+00_JPRB, 0.39302E+00_JPRB, 0.34928E+00_JPRB ,&
&  0.32247E+00_JPRB, 0.31451E+00_JPRB, 0.30127E+00_JPRB, 0.29644E+00_JPRB, 0.30251E+00_JPRB, 0.31099E+00_JPRB, 0.31426E+00_JPRB ,&
&  0.31802E+00_JPRB, 0.32664E+00_JPRB, 0.34804E+00_JPRB, 0.37150E+00_JPRB, 0.38675E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.39774E+00_JPRB, 0.40459E+00_JPRB, 0.40659E+00_JPRB, 0.40979E+00_JPRB, 0.41051E+00_JPRB, 0.39840E+00_JPRB, 0.37578E+00_JPRB ,&
&  0.38123E+00_JPRB, 0.33695E+00_JPRB, 0.32252E+00_JPRB, 0.31403E+00_JPRB, 0.31262E+00_JPRB, 0.30765E+00_JPRB, 0.29918E+00_JPRB ,&
&  0.29985E+00_JPRB, 0.30873E+00_JPRB, 0.32328E+00_JPRB, 0.32950E+00_JPRB, 0.32739E+00_JPRB, 0.31875E+00_JPRB, 0.30704E+00_JPRB ,&
&  0.29649E+00_JPRB, 0.29385E+00_JPRB, 0.29897E+00_JPRB, 0.30695E+00_JPRB, 0.31172E+00_JPRB, 0.31074E+00_JPRB, 0.30773E+00_JPRB ,&
&  0.30405E+00_JPRB, 0.30057E+00_JPRB, 0.30013E+00_JPRB, 0.30178E+00_JPRB, 0.30229E+00_JPRB, 0.29684E+00_JPRB, 0.28168E+00_JPRB ,&
&  0.26590E+00_JPRB, 0.27420E+00_JPRB, 0.32980E+00_JPRB, 0.35635E+00_JPRB, 0.37644E+00_JPRB, 0.38688E+00_JPRB, 0.39735E+00_JPRB ,&
&  0.41127E+00_JPRB, 0.42756E+00_JPRB, 0.44702E+00_JPRB, 0.47109E+00_JPRB, 0.48616E+00_JPRB, 0.49930E+00_JPRB, 0.51212E+00_JPRB ,&
&  0.55246E+00_JPRB, 0.47545E+00_JPRB, 0.45566E+00_JPRB, 0.49059E+00_JPRB, 0.73049E+00_JPRB, 0.80605E+00_JPRB, 0.81608E+00_JPRB ,&
&  0.85069E+00_JPRB, 0.81137E+00_JPRB, 0.81908E+00_JPRB, 0.84379E+00_JPRB, 0.85308E+00_JPRB, 0.85090E+00_JPRB, 0.84284E+00_JPRB ,&
&  0.82276E+00_JPRB, 0.79496E+00_JPRB, 0.74867E+00_JPRB, 0.71667E+00_JPRB, 0.69523E+00_JPRB, 0.70091E+00_JPRB, 0.68962E+00_JPRB ,&
&  0.66889E+00_JPRB, 0.66010E+00_JPRB, 0.61904E+00_JPRB, 0.57286E+00_JPRB, 0.53755E+00_JPRB, 0.51431E+00_JPRB, 0.49669E+00_JPRB ,&
&  0.48509E+00_JPRB, 0.48767E+00_JPRB, 0.49532E+00_JPRB, 0.49888E+00_JPRB, 0.47233E+00_JPRB, 0.43075E+00_JPRB, 0.39271E+00_JPRB ,&
&  0.35117E+00_JPRB, 0.33099E+00_JPRB, 0.30831E+00_JPRB, 0.30398E+00_JPRB, 0.31455E+00_JPRB, 0.32714E+00_JPRB, 0.33807E+00_JPRB ,&
&  0.35133E+00_JPRB, 0.36131E+00_JPRB, 0.37168E+00_JPRB, 0.38597E+00_JPRB, 0.39171E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.39841E+00_JPRB, 0.41047E+00_JPRB, 0.42036E+00_JPRB, 0.43098E+00_JPRB, 0.43469E+00_JPRB, 0.41501E+00_JPRB, 0.40696E+00_JPRB ,&
&  0.44508E+00_JPRB, 0.39601E+00_JPRB, 0.41577E+00_JPRB, 0.35530E+00_JPRB, 0.33178E+00_JPRB, 0.32514E+00_JPRB, 0.31350E+00_JPRB ,&
&  0.31104E+00_JPRB, 0.31758E+00_JPRB, 0.32499E+00_JPRB, 0.33118E+00_JPRB, 0.33298E+00_JPRB, 0.32175E+00_JPRB, 0.31064E+00_JPRB ,&
&  0.29884E+00_JPRB, 0.29570E+00_JPRB, 0.30180E+00_JPRB, 0.30845E+00_JPRB, 0.31440E+00_JPRB, 0.31583E+00_JPRB, 0.31340E+00_JPRB ,&
&  0.30705E+00_JPRB, 0.30486E+00_JPRB, 0.30735E+00_JPRB, 0.30803E+00_JPRB, 0.31048E+00_JPRB, 0.30865E+00_JPRB, 0.30799E+00_JPRB ,&
&  0.31211E+00_JPRB, 0.32028E+00_JPRB, 0.35977E+00_JPRB, 0.38727E+00_JPRB, 0.40592E+00_JPRB, 0.41533E+00_JPRB, 0.42934E+00_JPRB ,&
&  0.44344E+00_JPRB, 0.46107E+00_JPRB, 0.48787E+00_JPRB, 0.52060E+00_JPRB, 0.54808E+00_JPRB, 0.56785E+00_JPRB, 0.59136E+00_JPRB ,&
&  0.63943E+00_JPRB, 0.52251E+00_JPRB, 0.60402E+00_JPRB, 0.61511E+00_JPRB, 0.76221E+00_JPRB, 0.93364E+00_JPRB, 0.90069E+00_JPRB ,&
&  0.99876E+00_JPRB, 0.94482E+00_JPRB, 0.94769E+00_JPRB, 0.93662E+00_JPRB, 0.94120E+00_JPRB, 0.94417E+00_JPRB, 0.94207E+00_JPRB ,&
&  0.96962E+00_JPRB, 0.87237E+00_JPRB, 0.85611E+00_JPRB, 0.80257E+00_JPRB, 0.76469E+00_JPRB, 0.75853E+00_JPRB, 0.72500E+00_JPRB ,&
&  0.69911E+00_JPRB, 0.68539E+00_JPRB, 0.65817E+00_JPRB, 0.58561E+00_JPRB, 0.56336E+00_JPRB, 0.54418E+00_JPRB, 0.50368E+00_JPRB ,&
&  0.48876E+00_JPRB, 0.47266E+00_JPRB, 0.45717E+00_JPRB, 0.48324E+00_JPRB, 0.46846E+00_JPRB, 0.43184E+00_JPRB, 0.39366E+00_JPRB ,&
&  0.36141E+00_JPRB, 0.33858E+00_JPRB, 0.34585E+00_JPRB, 0.34906E+00_JPRB, 0.34636E+00_JPRB, 0.34924E+00_JPRB, 0.37344E+00_JPRB ,&
&  0.37300E+00_JPRB, 0.38173E+00_JPRB, 0.40028E+00_JPRB, 0.40298E+00_JPRB, 0.39861E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.40656E+00_JPRB, 0.42215E+00_JPRB, 0.43123E+00_JPRB, 0.44586E+00_JPRB, 0.45350E+00_JPRB, 0.44111E+00_JPRB, 0.42004E+00_JPRB ,&
&  0.42835E+00_JPRB, 0.42670E+00_JPRB, 0.42566E+00_JPRB, 0.42700E+00_JPRB, 0.42910E+00_JPRB, 0.38978E+00_JPRB, 0.32429E+00_JPRB ,&
&  0.30383E+00_JPRB, 0.30899E+00_JPRB, 0.32257E+00_JPRB, 0.33166E+00_JPRB, 0.33133E+00_JPRB, 0.32388E+00_JPRB, 0.31715E+00_JPRB ,&
&  0.31144E+00_JPRB, 0.30755E+00_JPRB, 0.31062E+00_JPRB, 0.31327E+00_JPRB, 0.31508E+00_JPRB, 0.31887E+00_JPRB, 0.32060E+00_JPRB ,&
&  0.31511E+00_JPRB, 0.31460E+00_JPRB, 0.31436E+00_JPRB, 0.30944E+00_JPRB, 0.32140E+00_JPRB, 0.32097E+00_JPRB, 0.32135E+00_JPRB ,&
&  0.33664E+00_JPRB, 0.36052E+00_JPRB, 0.38397E+00_JPRB, 0.41262E+00_JPRB, 0.43627E+00_JPRB, 0.45136E+00_JPRB, 0.46770E+00_JPRB ,&
&  0.48881E+00_JPRB, 0.51844E+00_JPRB, 0.55377E+00_JPRB, 0.59432E+00_JPRB, 0.63895E+00_JPRB, 0.63032E+00_JPRB, 0.69657E+00_JPRB ,&
&  0.70717E+00_JPRB, 0.74955E+00_JPRB, 0.76896E+00_JPRB, 0.71199E+00_JPRB, 0.86211E+00_JPRB, 0.99725E+00_JPRB, 0.96094E+00_JPRB ,&
&  0.98290E+00_JPRB, 0.91345E+00_JPRB, 0.98584E+00_JPRB, 0.10600E+01_JPRB, 0.11179E+01_JPRB, 0.10902E+01_JPRB, 0.10631E+01_JPRB ,&
&  0.11131E+01_JPRB, 0.11215E+01_JPRB, 0.97306E+00_JPRB, 0.91759E+00_JPRB, 0.86134E+00_JPRB, 0.84308E+00_JPRB, 0.78644E+00_JPRB ,&
&  0.75894E+00_JPRB, 0.74777E+00_JPRB, 0.71708E+00_JPRB, 0.67866E+00_JPRB, 0.63454E+00_JPRB, 0.59518E+00_JPRB, 0.55845E+00_JPRB ,&
&  0.51552E+00_JPRB, 0.46126E+00_JPRB, 0.43733E+00_JPRB, 0.44220E+00_JPRB, 0.45573E+00_JPRB, 0.41104E+00_JPRB, 0.37151E+00_JPRB ,&
&  0.35821E+00_JPRB, 0.36001E+00_JPRB, 0.38679E+00_JPRB, 0.38674E+00_JPRB, 0.38174E+00_JPRB, 0.37658E+00_JPRB, 0.36594E+00_JPRB ,&
&  0.36051E+00_JPRB, 0.39128E+00_JPRB, 0.40294E+00_JPRB, 0.40291E+00_JPRB, 0.40089E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.42384E+00_JPRB, 0.44330E+00_JPRB, 0.45005E+00_JPRB, 0.46546E+00_JPRB, 0.47715E+00_JPRB, 0.46053E+00_JPRB, 0.45326E+00_JPRB ,&
&  0.45077E+00_JPRB, 0.44674E+00_JPRB, 0.44025E+00_JPRB, 0.43733E+00_JPRB, 0.43631E+00_JPRB, 0.42877E+00_JPRB, 0.38388E+00_JPRB ,&
&  0.33234E+00_JPRB, 0.32889E+00_JPRB, 0.32493E+00_JPRB, 0.33300E+00_JPRB, 0.34113E+00_JPRB, 0.33393E+00_JPRB, 0.32794E+00_JPRB ,&
&  0.32777E+00_JPRB, 0.32344E+00_JPRB, 0.32081E+00_JPRB, 0.32150E+00_JPRB, 0.32518E+00_JPRB, 0.33236E+00_JPRB, 0.33564E+00_JPRB ,&
&  0.32733E+00_JPRB, 0.32252E+00_JPRB, 0.31965E+00_JPRB, 0.31979E+00_JPRB, 0.33126E+00_JPRB, 0.33686E+00_JPRB, 0.34188E+00_JPRB ,&
&  0.35801E+00_JPRB, 0.38137E+00_JPRB, 0.40563E+00_JPRB, 0.44089E+00_JPRB, 0.47457E+00_JPRB, 0.50274E+00_JPRB, 0.53448E+00_JPRB ,&
&  0.58718E+00_JPRB, 0.63853E+00_JPRB, 0.66983E+00_JPRB, 0.71279E+00_JPRB, 0.74446E+00_JPRB, 0.78527E+00_JPRB, 0.82359E+00_JPRB ,&
&  0.85045E+00_JPRB, 0.86701E+00_JPRB, 0.83657E+00_JPRB, 0.86015E+00_JPRB, 0.10023E+01_JPRB, 0.10733E+01_JPRB, 0.10664E+01_JPRB ,&
&  0.11069E+01_JPRB, 0.10315E+01_JPRB, 0.12715E+01_JPRB, 0.13228E+01_JPRB, 0.12532E+01_JPRB, 0.12536E+01_JPRB, 0.11736E+01_JPRB ,&
&  0.11636E+01_JPRB, 0.11081E+01_JPRB, 0.10364E+01_JPRB, 0.98314E+00_JPRB, 0.93473E+00_JPRB, 0.94292E+00_JPRB, 0.88738E+00_JPRB ,&
&  0.87772E+00_JPRB, 0.85080E+00_JPRB, 0.73388E+00_JPRB, 0.65329E+00_JPRB, 0.57308E+00_JPRB, 0.60064E+00_JPRB, 0.57795E+00_JPRB ,&
&  0.52730E+00_JPRB, 0.45297E+00_JPRB, 0.46383E+00_JPRB, 0.47264E+00_JPRB, 0.49659E+00_JPRB, 0.48075E+00_JPRB, 0.41451E+00_JPRB ,&
&  0.38963E+00_JPRB, 0.40542E+00_JPRB, 0.40677E+00_JPRB, 0.40817E+00_JPRB, 0.40827E+00_JPRB, 0.40347E+00_JPRB, 0.37433E+00_JPRB ,&
&  0.36779E+00_JPRB, 0.39514E+00_JPRB, 0.40778E+00_JPRB, 0.40858E+00_JPRB, 0.41386E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.44650E+00_JPRB, 0.47111E+00_JPRB, 0.48198E+00_JPRB, 0.49105E+00_JPRB, 0.50084E+00_JPRB, 0.48333E+00_JPRB, 0.47648E+00_JPRB ,&
&  0.47763E+00_JPRB, 0.47095E+00_JPRB, 0.45565E+00_JPRB, 0.45156E+00_JPRB, 0.44931E+00_JPRB, 0.44150E+00_JPRB, 0.42920E+00_JPRB ,&
&  0.36064E+00_JPRB, 0.34377E+00_JPRB, 0.32407E+00_JPRB, 0.32765E+00_JPRB, 0.36091E+00_JPRB, 0.35274E+00_JPRB, 0.34858E+00_JPRB ,&
&  0.34499E+00_JPRB, 0.34291E+00_JPRB, 0.33574E+00_JPRB, 0.33409E+00_JPRB, 0.33791E+00_JPRB, 0.34957E+00_JPRB, 0.36173E+00_JPRB ,&
&  0.35492E+00_JPRB, 0.34393E+00_JPRB, 0.33765E+00_JPRB, 0.33750E+00_JPRB, 0.34735E+00_JPRB, 0.35898E+00_JPRB, 0.37337E+00_JPRB ,&
&  0.39279E+00_JPRB, 0.41352E+00_JPRB, 0.44766E+00_JPRB, 0.49771E+00_JPRB, 0.54087E+00_JPRB, 0.57956E+00_JPRB, 0.64359E+00_JPRB ,&
&  0.73007E+00_JPRB, 0.79737E+00_JPRB, 0.83705E+00_JPRB, 0.87454E+00_JPRB, 0.88262E+00_JPRB, 0.92453E+00_JPRB, 0.93524E+00_JPRB ,&
&  0.99770E+00_JPRB, 0.96005E+00_JPRB, 0.93875E+00_JPRB, 0.10867E+01_JPRB, 0.12506E+01_JPRB, 0.12940E+01_JPRB, 0.12492E+01_JPRB ,&
&  0.12405E+01_JPRB, 0.11763E+01_JPRB, 0.12543E+01_JPRB, 0.13271E+01_JPRB, 0.12721E+01_JPRB, 0.13141E+01_JPRB, 0.12135E+01_JPRB ,&
&  0.10878E+01_JPRB, 0.10127E+01_JPRB, 0.98183E+00_JPRB, 0.98536E+00_JPRB, 0.94760E+00_JPRB, 0.99300E+00_JPRB, 0.97305E+00_JPRB ,&
&  0.98558E+00_JPRB, 0.86795E+00_JPRB, 0.66148E+00_JPRB, 0.61389E+00_JPRB, 0.50262E+00_JPRB, 0.51538E+00_JPRB, 0.62158E+00_JPRB ,&
&  0.60238E+00_JPRB, 0.55000E+00_JPRB, 0.54062E+00_JPRB, 0.55068E+00_JPRB, 0.58089E+00_JPRB, 0.63406E+00_JPRB, 0.54956E+00_JPRB ,&
&  0.46269E+00_JPRB, 0.45575E+00_JPRB, 0.45916E+00_JPRB, 0.45283E+00_JPRB, 0.44718E+00_JPRB, 0.44715E+00_JPRB, 0.43805E+00_JPRB ,&
&  0.42948E+00_JPRB, 0.42647E+00_JPRB, 0.42907E+00_JPRB, 0.42963E+00_JPRB, 0.43839E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.47312E+00_JPRB, 0.49469E+00_JPRB, 0.50955E+00_JPRB, 0.51416E+00_JPRB, 0.52545E+00_JPRB, 0.51461E+00_JPRB, 0.50345E+00_JPRB ,&
&  0.50133E+00_JPRB, 0.49131E+00_JPRB, 0.47239E+00_JPRB, 0.46775E+00_JPRB, 0.46216E+00_JPRB, 0.45115E+00_JPRB, 0.43916E+00_JPRB ,&
&  0.43703E+00_JPRB, 0.39118E+00_JPRB, 0.35408E+00_JPRB, 0.33266E+00_JPRB, 0.35651E+00_JPRB, 0.35479E+00_JPRB, 0.36137E+00_JPRB ,&
&  0.35730E+00_JPRB, 0.36622E+00_JPRB, 0.36351E+00_JPRB, 0.36074E+00_JPRB, 0.36876E+00_JPRB, 0.38437E+00_JPRB, 0.39824E+00_JPRB ,&
&  0.39820E+00_JPRB, 0.39480E+00_JPRB, 0.38658E+00_JPRB, 0.38794E+00_JPRB, 0.40827E+00_JPRB, 0.40696E+00_JPRB, 0.43028E+00_JPRB ,&
&  0.45090E+00_JPRB, 0.47312E+00_JPRB, 0.50815E+00_JPRB, 0.56474E+00_JPRB, 0.61884E+00_JPRB, 0.67346E+00_JPRB, 0.75874E+00_JPRB ,&
&  0.84043E+00_JPRB, 0.87491E+00_JPRB, 0.87718E+00_JPRB, 0.90115E+00_JPRB, 0.97297E+00_JPRB, 0.10606E+01_JPRB, 0.10424E+01_JPRB ,&
&  0.10248E+01_JPRB, 0.94550E+00_JPRB, 0.87753E+00_JPRB, 0.11145E+01_JPRB, 0.13340E+01_JPRB, 0.13490E+01_JPRB, 0.13148E+01_JPRB ,&
&  0.13302E+01_JPRB, 0.13315E+01_JPRB, 0.14235E+01_JPRB, 0.14408E+01_JPRB, 0.14193E+01_JPRB, 0.13683E+01_JPRB, 0.12270E+01_JPRB ,&
&  0.10727E+01_JPRB, 0.10013E+01_JPRB, 0.10306E+01_JPRB, 0.10457E+01_JPRB, 0.10513E+01_JPRB, 0.10071E+01_JPRB, 0.10647E+01_JPRB ,&
&  0.11441E+01_JPRB, 0.65176E+00_JPRB, 0.52838E+00_JPRB, 0.60705E+00_JPRB, 0.53002E+00_JPRB, 0.53793E+00_JPRB, 0.59702E+00_JPRB ,&
&  0.57353E+00_JPRB, 0.55798E+00_JPRB, 0.61028E+00_JPRB, 0.62724E+00_JPRB, 0.68391E+00_JPRB, 0.76244E+00_JPRB, 0.67614E+00_JPRB ,&
&  0.59580E+00_JPRB, 0.55496E+00_JPRB, 0.55449E+00_JPRB, 0.53632E+00_JPRB, 0.53529E+00_JPRB, 0.52321E+00_JPRB, 0.51327E+00_JPRB ,&
&  0.49572E+00_JPRB, 0.48441E+00_JPRB, 0.47342E+00_JPRB, 0.47015E+00_JPRB, 0.47143E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.52218E+00_JPRB, 0.53866E+00_JPRB, 0.54695E+00_JPRB, 0.55188E+00_JPRB, 0.56021E+00_JPRB, 0.55608E+00_JPRB, 0.53929E+00_JPRB ,&
&  0.52397E+00_JPRB, 0.50992E+00_JPRB, 0.49013E+00_JPRB, 0.48388E+00_JPRB, 0.47370E+00_JPRB, 0.46079E+00_JPRB, 0.44944E+00_JPRB ,&
&  0.44632E+00_JPRB, 0.42957E+00_JPRB, 0.37214E+00_JPRB, 0.34949E+00_JPRB, 0.33248E+00_JPRB, 0.34455E+00_JPRB, 0.35590E+00_JPRB ,&
&  0.37292E+00_JPRB, 0.38080E+00_JPRB, 0.38657E+00_JPRB, 0.39830E+00_JPRB, 0.41952E+00_JPRB, 0.43433E+00_JPRB, 0.45916E+00_JPRB ,&
&  0.46523E+00_JPRB, 0.47236E+00_JPRB, 0.45085E+00_JPRB, 0.46143E+00_JPRB, 0.48492E+00_JPRB, 0.48436E+00_JPRB, 0.52874E+00_JPRB ,&
&  0.55939E+00_JPRB, 0.59466E+00_JPRB, 0.63047E+00_JPRB, 0.65992E+00_JPRB, 0.69682E+00_JPRB, 0.72305E+00_JPRB, 0.78555E+00_JPRB ,&
&  0.83039E+00_JPRB, 0.84942E+00_JPRB, 0.83964E+00_JPRB, 0.84564E+00_JPRB, 0.88994E+00_JPRB, 0.94148E+00_JPRB, 0.93931E+00_JPRB ,&
&  0.95846E+00_JPRB, 0.80334E+00_JPRB, 0.60847E+00_JPRB, 0.91885E+00_JPRB, 0.13225E+01_JPRB, 0.12982E+01_JPRB, 0.11324E+01_JPRB ,&
&  0.12218E+01_JPRB, 0.13070E+01_JPRB, 0.14711E+01_JPRB, 0.15148E+01_JPRB, 0.14858E+01_JPRB, 0.14052E+01_JPRB, 0.13108E+01_JPRB ,&
&  0.12116E+01_JPRB, 0.12255E+01_JPRB, 0.13214E+01_JPRB, 0.14818E+01_JPRB, 0.15028E+01_JPRB, 0.14808E+01_JPRB, 0.14178E+01_JPRB ,&
&  0.12001E+01_JPRB, 0.99399E+00_JPRB, 0.68680E+00_JPRB, 0.70633E+00_JPRB, 0.56326E+00_JPRB, 0.55961E+00_JPRB, 0.61090E+00_JPRB ,&
&  0.58829E+00_JPRB, 0.61857E+00_JPRB, 0.68229E+00_JPRB, 0.72057E+00_JPRB, 0.88454E+00_JPRB, 0.88622E+00_JPRB, 0.83194E+00_JPRB ,&
&  0.73940E+00_JPRB, 0.64835E+00_JPRB, 0.66055E+00_JPRB, 0.64859E+00_JPRB, 0.64950E+00_JPRB, 0.62788E+00_JPRB, 0.60524E+00_JPRB ,&
&  0.57772E+00_JPRB, 0.55602E+00_JPRB, 0.54097E+00_JPRB, 0.52680E+00_JPRB, 0.51786E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.57032E+00_JPRB, 0.56956E+00_JPRB, 0.56269E+00_JPRB, 0.55728E+00_JPRB, 0.55800E+00_JPRB, 0.55419E+00_JPRB, 0.54378E+00_JPRB ,&
&  0.53811E+00_JPRB, 0.52411E+00_JPRB, 0.50822E+00_JPRB, 0.49452E+00_JPRB, 0.48628E+00_JPRB, 0.47372E+00_JPRB, 0.46394E+00_JPRB ,&
&  0.45922E+00_JPRB, 0.41954E+00_JPRB, 0.35583E+00_JPRB, 0.32865E+00_JPRB, 0.31492E+00_JPRB, 0.32495E+00_JPRB, 0.34452E+00_JPRB ,&
&  0.36837E+00_JPRB, 0.38485E+00_JPRB, 0.41196E+00_JPRB, 0.44873E+00_JPRB, 0.49194E+00_JPRB, 0.52839E+00_JPRB, 0.55515E+00_JPRB ,&
&  0.63235E+00_JPRB, 0.57344E+00_JPRB, 0.58495E+00_JPRB, 0.57177E+00_JPRB, 0.60739E+00_JPRB, 0.62717E+00_JPRB, 0.63349E+00_JPRB ,&
&  0.63662E+00_JPRB, 0.64196E+00_JPRB, 0.65079E+00_JPRB, 0.65283E+00_JPRB, 0.66720E+00_JPRB, 0.70591E+00_JPRB, 0.78698E+00_JPRB ,&
&  0.85662E+00_JPRB, 0.84649E+00_JPRB, 0.81113E+00_JPRB, 0.83929E+00_JPRB, 0.89843E+00_JPRB, 0.93217E+00_JPRB, 0.93010E+00_JPRB ,&
&  0.88795E+00_JPRB, 0.81868E+00_JPRB, 0.97187E+00_JPRB, 0.11270E+01_JPRB, 0.13297E+01_JPRB, 0.13684E+01_JPRB, 0.12784E+01_JPRB ,&
&  0.13435E+01_JPRB, 0.13774E+01_JPRB, 0.14639E+01_JPRB, 0.11878E+01_JPRB, 0.13350E+01_JPRB, 0.13352E+01_JPRB, 0.13258E+01_JPRB ,&
&  0.13286E+01_JPRB, 0.14289E+01_JPRB, 0.15521E+01_JPRB, 0.17805E+01_JPRB, 0.19487E+01_JPRB, 0.20581E+01_JPRB, 0.15536E+01_JPRB ,&
&  0.11808E+01_JPRB, 0.12327E+01_JPRB, 0.11019E+01_JPRB, 0.88971E+00_JPRB, 0.72801E+00_JPRB, 0.72487E+00_JPRB, 0.71053E+00_JPRB ,&
&  0.70200E+00_JPRB, 0.73205E+00_JPRB, 0.78421E+00_JPRB, 0.95965E+00_JPRB, 0.11419E+01_JPRB, 0.11489E+01_JPRB, 0.10268E+01_JPRB ,&
&  0.85563E+00_JPRB, 0.85942E+00_JPRB, 0.82034E+00_JPRB, 0.78384E+00_JPRB, 0.79046E+00_JPRB, 0.75746E+00_JPRB, 0.70563E+00_JPRB ,&
&  0.65515E+00_JPRB, 0.62649E+00_JPRB, 0.60637E+00_JPRB, 0.58753E+00_JPRB, 0.57339E+00_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.60683E+00_JPRB, 0.60123E+00_JPRB, 0.58963E+00_JPRB, 0.58020E+00_JPRB, 0.57101E+00_JPRB, 0.56117E+00_JPRB, 0.55114E+00_JPRB ,&
&  0.55118E+00_JPRB, 0.52386E+00_JPRB, 0.51352E+00_JPRB, 0.50465E+00_JPRB, 0.49842E+00_JPRB, 0.48987E+00_JPRB, 0.48426E+00_JPRB ,&
&  0.47875E+00_JPRB, 0.40659E+00_JPRB, 0.34238E+00_JPRB, 0.33700E+00_JPRB, 0.33615E+00_JPRB, 0.32952E+00_JPRB, 0.34469E+00_JPRB ,&
&  0.36791E+00_JPRB, 0.39527E+00_JPRB, 0.43455E+00_JPRB, 0.51412E+00_JPRB, 0.59596E+00_JPRB, 0.66379E+00_JPRB, 0.72334E+00_JPRB ,&
&  0.71567E+00_JPRB, 0.72801E+00_JPRB, 0.75805E+00_JPRB, 0.72386E+00_JPRB, 0.69311E+00_JPRB, 0.67751E+00_JPRB, 0.67643E+00_JPRB ,&
&  0.67666E+00_JPRB, 0.66755E+00_JPRB, 0.67281E+00_JPRB, 0.68453E+00_JPRB, 0.70767E+00_JPRB, 0.75998E+00_JPRB, 0.83238E+00_JPRB ,&
&  0.86556E+00_JPRB, 0.84264E+00_JPRB, 0.82696E+00_JPRB, 0.89524E+00_JPRB, 0.82699E+00_JPRB, 0.79146E+00_JPRB, 0.76669E+00_JPRB ,&
&  0.96783E+00_JPRB, 0.10948E+01_JPRB, 0.10935E+01_JPRB, 0.12112E+01_JPRB, 0.11265E+01_JPRB, 0.11979E+01_JPRB, 0.13749E+01_JPRB ,&
&  0.14841E+01_JPRB, 0.14961E+01_JPRB, 0.15424E+01_JPRB, 0.10624E+01_JPRB, 0.97046E+00_JPRB, 0.13232E+01_JPRB, 0.14137E+01_JPRB ,&
&  0.14484E+01_JPRB, 0.16611E+01_JPRB, 0.19581E+01_JPRB, 0.23130E+01_JPRB, 0.17667E+01_JPRB, 0.15156E+01_JPRB, 0.11332E+01_JPRB ,&
&  0.10834E+01_JPRB, 0.97706E+00_JPRB, 0.10893E+01_JPRB, 0.99812E+00_JPRB, 0.96090E+00_JPRB, 0.91963E+00_JPRB, 0.85331E+00_JPRB ,&
&  0.84586E+00_JPRB, 0.84685E+00_JPRB, 0.97639E+00_JPRB, 0.12625E+01_JPRB, 0.14962E+01_JPRB, 0.13607E+01_JPRB, 0.13007E+01_JPRB ,&
&  0.12291E+01_JPRB, 0.11408E+01_JPRB, 0.10127E+01_JPRB, 0.97227E+00_JPRB, 0.91610E+00_JPRB, 0.84779E+00_JPRB, 0.77119E+00_JPRB ,&
&  0.70988E+00_JPRB, 0.67607E+00_JPRB, 0.64636E+00_JPRB, 0.62505E+00_JPRB, 0.61295E+00_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.64380E+00_JPRB, 0.63216E+00_JPRB, 0.61981E+00_JPRB, 0.60667E+00_JPRB, 0.59371E+00_JPRB, 0.58006E+00_JPRB, 0.57070E+00_JPRB ,&
&  0.57467E+00_JPRB, 0.54360E+00_JPRB, 0.51862E+00_JPRB, 0.50972E+00_JPRB, 0.50754E+00_JPRB, 0.50721E+00_JPRB, 0.50772E+00_JPRB ,&
&  0.50330E+00_JPRB, 0.45205E+00_JPRB, 0.36619E+00_JPRB, 0.34337E+00_JPRB, 0.35780E+00_JPRB, 0.34902E+00_JPRB, 0.34876E+00_JPRB ,&
&  0.38147E+00_JPRB, 0.42595E+00_JPRB, 0.49751E+00_JPRB, 0.62341E+00_JPRB, 0.73662E+00_JPRB, 0.83358E+00_JPRB, 0.89359E+00_JPRB ,&
&  0.89541E+00_JPRB, 0.83022E+00_JPRB, 0.75089E+00_JPRB, 0.67804E+00_JPRB, 0.64399E+00_JPRB, 0.63266E+00_JPRB, 0.64215E+00_JPRB ,&
&  0.65790E+00_JPRB, 0.66755E+00_JPRB, 0.67889E+00_JPRB, 0.68522E+00_JPRB, 0.72630E+00_JPRB, 0.75935E+00_JPRB, 0.81471E+00_JPRB ,&
&  0.84076E+00_JPRB, 0.83535E+00_JPRB, 0.86456E+00_JPRB, 0.92346E+00_JPRB, 0.90880E+00_JPRB, 0.84677E+00_JPRB, 0.83156E+00_JPRB ,&
&  0.92776E+00_JPRB, 0.10331E+01_JPRB, 0.11198E+01_JPRB, 0.12248E+01_JPRB, 0.13359E+01_JPRB, 0.13940E+01_JPRB, 0.15697E+01_JPRB ,&
&  0.13345E+01_JPRB, 0.10761E+01_JPRB, 0.90293E+00_JPRB, 0.78326E+00_JPRB, 0.11003E+01_JPRB, 0.15816E+01_JPRB, 0.17996E+01_JPRB ,&
&  0.20102E+01_JPRB, 0.22397E+01_JPRB, 0.24891E+01_JPRB, 0.22277E+01_JPRB, 0.17032E+01_JPRB, 0.12843E+01_JPRB, 0.15040E+01_JPRB ,&
&  0.15345E+01_JPRB, 0.14853E+01_JPRB, 0.13582E+01_JPRB, 0.11866E+01_JPRB, 0.94257E+00_JPRB, 0.10646E+01_JPRB, 0.10233E+01_JPRB ,&
&  0.98241E+00_JPRB, 0.97437E+00_JPRB, 0.13380E+01_JPRB, 0.17123E+01_JPRB, 0.17751E+01_JPRB, 0.17488E+01_JPRB, 0.16872E+01_JPRB ,&
&  0.15258E+01_JPRB, 0.14246E+01_JPRB, 0.12585E+01_JPRB, 0.11085E+01_JPRB, 0.98456E+00_JPRB, 0.87535E+00_JPRB, 0.79028E+00_JPRB ,&
&  0.73949E+00_JPRB, 0.70664E+00_JPRB, 0.67365E+00_JPRB, 0.65999E+00_JPRB, 0.65091E+00_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.70032E+00_JPRB, 0.68218E+00_JPRB, 0.66581E+00_JPRB, 0.65085E+00_JPRB, 0.63452E+00_JPRB, 0.60980E+00_JPRB, 0.60167E+00_JPRB ,&
&  0.59823E+00_JPRB, 0.56814E+00_JPRB, 0.53962E+00_JPRB, 0.52956E+00_JPRB, 0.53589E+00_JPRB, 0.53909E+00_JPRB, 0.53600E+00_JPRB ,&
&  0.53138E+00_JPRB, 0.53252E+00_JPRB, 0.42277E+00_JPRB, 0.37931E+00_JPRB, 0.37807E+00_JPRB, 0.37287E+00_JPRB, 0.35933E+00_JPRB ,&
&  0.41181E+00_JPRB, 0.48882E+00_JPRB, 0.58344E+00_JPRB, 0.72611E+00_JPRB, 0.91169E+00_JPRB, 0.90956E+00_JPRB, 0.90473E+00_JPRB ,&
&  0.91058E+00_JPRB, 0.85424E+00_JPRB, 0.77356E+00_JPRB, 0.71339E+00_JPRB, 0.68235E+00_JPRB, 0.67344E+00_JPRB, 0.68200E+00_JPRB ,&
&  0.69950E+00_JPRB, 0.71197E+00_JPRB, 0.71959E+00_JPRB, 0.74301E+00_JPRB, 0.75055E+00_JPRB, 0.77451E+00_JPRB, 0.82835E+00_JPRB ,&
&  0.84556E+00_JPRB, 0.83197E+00_JPRB, 0.86347E+00_JPRB, 0.95906E+00_JPRB, 0.10199E+01_JPRB, 0.94033E+00_JPRB, 0.11082E+01_JPRB ,&
&  0.10369E+01_JPRB, 0.11710E+01_JPRB, 0.12518E+01_JPRB, 0.15390E+01_JPRB, 0.15995E+01_JPRB, 0.14994E+01_JPRB, 0.15521E+01_JPRB ,&
&  0.12766E+01_JPRB, 0.11460E+01_JPRB, 0.12726E+01_JPRB, 0.14306E+01_JPRB, 0.15585E+01_JPRB, 0.16813E+01_JPRB, 0.25593E+01_JPRB ,&
&  0.22030E+01_JPRB, 0.24666E+01_JPRB, 0.26497E+01_JPRB, 0.25602E+01_JPRB, 0.12274E+01_JPRB, 0.88655E+00_JPRB, 0.12475E+01_JPRB ,&
&  0.13397E+01_JPRB, 0.93166E+00_JPRB, 0.68010E+00_JPRB, 0.82201E+00_JPRB, 0.63214E+00_JPRB, 0.75042E+00_JPRB, 0.98310E+00_JPRB ,&
&  0.10502E+01_JPRB, 0.11554E+01_JPRB, 0.16540E+01_JPRB, 0.19309E+01_JPRB, 0.19948E+01_JPRB, 0.22877E+01_JPRB, 0.19505E+01_JPRB ,&
&  0.17787E+01_JPRB, 0.15393E+01_JPRB, 0.14492E+01_JPRB, 0.12450E+01_JPRB, 0.10810E+01_JPRB, 0.97090E+00_JPRB, 0.88842E+00_JPRB ,&
&  0.82902E+00_JPRB, 0.77894E+00_JPRB, 0.74532E+00_JPRB, 0.72868E+00_JPRB, 0.71623E+00_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.77348E+00_JPRB, 0.74499E+00_JPRB, 0.72007E+00_JPRB, 0.70036E+00_JPRB, 0.67414E+00_JPRB, 0.64807E+00_JPRB, 0.64107E+00_JPRB ,&
&  0.62430E+00_JPRB, 0.59554E+00_JPRB, 0.57535E+00_JPRB, 0.55676E+00_JPRB, 0.55721E+00_JPRB, 0.55988E+00_JPRB, 0.54662E+00_JPRB ,&
&  0.53011E+00_JPRB, 0.52088E+00_JPRB, 0.48544E+00_JPRB, 0.44554E+00_JPRB, 0.42412E+00_JPRB, 0.40900E+00_JPRB, 0.41654E+00_JPRB ,&
&  0.49557E+00_JPRB, 0.62570E+00_JPRB, 0.72958E+00_JPRB, 0.83852E+00_JPRB, 0.92826E+00_JPRB, 0.90679E+00_JPRB, 0.96253E+00_JPRB ,&
&  0.98254E+00_JPRB, 0.94360E+00_JPRB, 0.86679E+00_JPRB, 0.79744E+00_JPRB, 0.76044E+00_JPRB, 0.74042E+00_JPRB, 0.72815E+00_JPRB ,&
&  0.73337E+00_JPRB, 0.75028E+00_JPRB, 0.76383E+00_JPRB, 0.75938E+00_JPRB, 0.75488E+00_JPRB, 0.78360E+00_JPRB, 0.83778E+00_JPRB ,&
&  0.85049E+00_JPRB, 0.83504E+00_JPRB, 0.88097E+00_JPRB, 0.95120E+00_JPRB, 0.11226E+01_JPRB, 0.11657E+01_JPRB, 0.97903E+00_JPRB ,&
&  0.86407E+00_JPRB, 0.95700E+00_JPRB, 0.13296E+01_JPRB, 0.15105E+01_JPRB, 0.15019E+01_JPRB, 0.14693E+01_JPRB, 0.13739E+01_JPRB ,&
&  0.13572E+01_JPRB, 0.15999E+01_JPRB, 0.17615E+01_JPRB, 0.19951E+01_JPRB, 0.21035E+01_JPRB, 0.16918E+01_JPRB, 0.22942E+01_JPRB ,&
&  0.24785E+01_JPRB, 0.24642E+01_JPRB, 0.19798E+01_JPRB, 0.13629E+01_JPRB, 0.13426E+01_JPRB, 0.95896E+00_JPRB, 0.65082E+00_JPRB ,&
&  0.60923E+00_JPRB, 0.58195E+00_JPRB, 0.52869E+00_JPRB, 0.53592E+00_JPRB, 0.52309E+00_JPRB, 0.67436E+00_JPRB, 0.93672E+00_JPRB ,&
&  0.12316E+01_JPRB, 0.15359E+01_JPRB, 0.19073E+01_JPRB, 0.21467E+01_JPRB, 0.20511E+01_JPRB, 0.22534E+01_JPRB, 0.21277E+01_JPRB ,&
&  0.17097E+01_JPRB, 0.15431E+01_JPRB, 0.14048E+01_JPRB, 0.12379E+01_JPRB, 0.11142E+01_JPRB, 0.10344E+01_JPRB, 0.97440E+00_JPRB ,&
&  0.91529E+00_JPRB, 0.86432E+00_JPRB, 0.83166E+00_JPRB, 0.81333E+00_JPRB, 0.79696E+00_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.83346E+00_JPRB, 0.80265E+00_JPRB, 0.77656E+00_JPRB, 0.75545E+00_JPRB, 0.72673E+00_JPRB, 0.70065E+00_JPRB, 0.68669E+00_JPRB ,&
&  0.65849E+00_JPRB, 0.62616E+00_JPRB, 0.60099E+00_JPRB, 0.58234E+00_JPRB, 0.57582E+00_JPRB, 0.57339E+00_JPRB, 0.56152E+00_JPRB ,&
&  0.54463E+00_JPRB, 0.53048E+00_JPRB, 0.52406E+00_JPRB, 0.51064E+00_JPRB, 0.50997E+00_JPRB, 0.51257E+00_JPRB, 0.55585E+00_JPRB ,&
&  0.67684E+00_JPRB, 0.85253E+00_JPRB, 0.94249E+00_JPRB, 0.10214E+01_JPRB, 0.10961E+01_JPRB, 0.11081E+01_JPRB, 0.11116E+01_JPRB ,&
&  0.10943E+01_JPRB, 0.10504E+01_JPRB, 0.96151E+00_JPRB, 0.87334E+00_JPRB, 0.81382E+00_JPRB, 0.78364E+00_JPRB, 0.77161E+00_JPRB ,&
&  0.77034E+00_JPRB, 0.79595E+00_JPRB, 0.79265E+00_JPRB, 0.77104E+00_JPRB, 0.76221E+00_JPRB, 0.80597E+00_JPRB, 0.85384E+00_JPRB ,&
&  0.86576E+00_JPRB, 0.89148E+00_JPRB, 0.95398E+00_JPRB, 0.98190E+00_JPRB, 0.93670E+00_JPRB, 0.79351E+00_JPRB, 0.87157E+00_JPRB ,&
&  0.97582E+00_JPRB, 0.12048E+01_JPRB, 0.12662E+01_JPRB, 0.16165E+01_JPRB, 0.16249E+01_JPRB, 0.14763E+01_JPRB, 0.14687E+01_JPRB ,&
&  0.15807E+01_JPRB, 0.21393E+01_JPRB, 0.18978E+01_JPRB, 0.19027E+01_JPRB, 0.23258E+01_JPRB, 0.19008E+01_JPRB, 0.20503E+01_JPRB ,&
&  0.24121E+01_JPRB, 0.22462E+01_JPRB, 0.18593E+01_JPRB, 0.13796E+01_JPRB, 0.16488E+01_JPRB, 0.14840E+01_JPRB, 0.76476E+00_JPRB ,&
&  0.63012E+00_JPRB, 0.58580E+00_JPRB, 0.55200E+00_JPRB, 0.53080E+00_JPRB, 0.52839E+00_JPRB, 0.74042E+00_JPRB, 0.12099E+01_JPRB ,&
&  0.13795E+01_JPRB, 0.18899E+01_JPRB, 0.22154E+01_JPRB, 0.23297E+01_JPRB, 0.20572E+01_JPRB, 0.19256E+01_JPRB, 0.18573E+01_JPRB ,&
&  0.16403E+01_JPRB, 0.15882E+01_JPRB, 0.14234E+01_JPRB, 0.13010E+01_JPRB, 0.11993E+01_JPRB, 0.11189E+01_JPRB, 0.10480E+01_JPRB ,&
&  0.98114E+00_JPRB, 0.93621E+00_JPRB, 0.90248E+00_JPRB, 0.88813E+00_JPRB, 0.86238E+00_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.87533E+00_JPRB, 0.85252E+00_JPRB, 0.82067E+00_JPRB, 0.79619E+00_JPRB, 0.76768E+00_JPRB, 0.73722E+00_JPRB, 0.72969E+00_JPRB ,&
&  0.70458E+00_JPRB, 0.66740E+00_JPRB, 0.63291E+00_JPRB, 0.61691E+00_JPRB, 0.60362E+00_JPRB, 0.58687E+00_JPRB, 0.57189E+00_JPRB ,&
&  0.55527E+00_JPRB, 0.54580E+00_JPRB, 0.55251E+00_JPRB, 0.61070E+00_JPRB, 0.64342E+00_JPRB, 0.66616E+00_JPRB, 0.72489E+00_JPRB ,&
&  0.94230E+00_JPRB, 0.10683E+01_JPRB, 0.10996E+01_JPRB, 0.11382E+01_JPRB, 0.11358E+01_JPRB, 0.11212E+01_JPRB, 0.11084E+01_JPRB ,&
&  0.10728E+01_JPRB, 0.10382E+01_JPRB, 0.96999E+00_JPRB, 0.90194E+00_JPRB, 0.84771E+00_JPRB, 0.82552E+00_JPRB, 0.81967E+00_JPRB ,&
&  0.83904E+00_JPRB, 0.82801E+00_JPRB, 0.80615E+00_JPRB, 0.77380E+00_JPRB, 0.77386E+00_JPRB, 0.82425E+00_JPRB, 0.87412E+00_JPRB ,&
&  0.89830E+00_JPRB, 0.96730E+00_JPRB, 0.10127E+01_JPRB, 0.98970E+00_JPRB, 0.81687E+00_JPRB, 0.91175E+00_JPRB, 0.94984E+00_JPRB ,&
&  0.10030E+01_JPRB, 0.10699E+01_JPRB, 0.10736E+01_JPRB, 0.13493E+01_JPRB, 0.15512E+01_JPRB, 0.14607E+01_JPRB, 0.14107E+01_JPRB ,&
&  0.17905E+01_JPRB, 0.18389E+01_JPRB, 0.17048E+01_JPRB, 0.17985E+01_JPRB, 0.20156E+01_JPRB, 0.25337E+01_JPRB, 0.17079E+01_JPRB ,&
&  0.15625E+01_JPRB, 0.18739E+01_JPRB, 0.16313E+01_JPRB, 0.12667E+01_JPRB, 0.14687E+01_JPRB, 0.16547E+01_JPRB, 0.13470E+01_JPRB ,&
&  0.72227E+00_JPRB, 0.59594E+00_JPRB, 0.62819E+00_JPRB, 0.67982E+00_JPRB, 0.63163E+00_JPRB, 0.88853E+00_JPRB, 0.20034E+01_JPRB ,&
&  0.18998E+01_JPRB, 0.23015E+01_JPRB, 0.23796E+01_JPRB, 0.22395E+01_JPRB, 0.19376E+01_JPRB, 0.17281E+01_JPRB, 0.17637E+01_JPRB ,&
&  0.15873E+01_JPRB, 0.14976E+01_JPRB, 0.14266E+01_JPRB, 0.13565E+01_JPRB, 0.12691E+01_JPRB, 0.11681E+01_JPRB, 0.10834E+01_JPRB ,&
&  0.10206E+01_JPRB, 0.98135E+00_JPRB, 0.94651E+00_JPRB, 0.92143E+00_JPRB, 0.89356E+00_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.92388E+00_JPRB, 0.88731E+00_JPRB, 0.85207E+00_JPRB, 0.82943E+00_JPRB, 0.79668E+00_JPRB, 0.76793E+00_JPRB, 0.78381E+00_JPRB ,&
&  0.76773E+00_JPRB, 0.71964E+00_JPRB, 0.66432E+00_JPRB, 0.64103E+00_JPRB, 0.62164E+00_JPRB, 0.60056E+00_JPRB, 0.58709E+00_JPRB ,&
&  0.57509E+00_JPRB, 0.57238E+00_JPRB, 0.57593E+00_JPRB, 0.63343E+00_JPRB, 0.68955E+00_JPRB, 0.67553E+00_JPRB, 0.77540E+00_JPRB ,&
&  0.10862E+01_JPRB, 0.11964E+01_JPRB, 0.11902E+01_JPRB, 0.12063E+01_JPRB, 0.11942E+01_JPRB, 0.11700E+01_JPRB, 0.11348E+01_JPRB ,&
&  0.10475E+01_JPRB, 0.10046E+01_JPRB, 0.93151E+00_JPRB, 0.90407E+00_JPRB, 0.88295E+00_JPRB, 0.86493E+00_JPRB, 0.86404E+00_JPRB ,&
&  0.83610E+00_JPRB, 0.81327E+00_JPRB, 0.78654E+00_JPRB, 0.75846E+00_JPRB, 0.78805E+00_JPRB, 0.85564E+00_JPRB, 0.91709E+00_JPRB ,&
&  0.96906E+00_JPRB, 0.10380E+01_JPRB, 0.10452E+01_JPRB, 0.95558E+00_JPRB, 0.92525E+00_JPRB, 0.92394E+00_JPRB, 0.96640E+00_JPRB ,&
&  0.96677E+00_JPRB, 0.10351E+01_JPRB, 0.11352E+01_JPRB, 0.12623E+01_JPRB, 0.14042E+01_JPRB, 0.14163E+01_JPRB, 0.14151E+01_JPRB ,&
&  0.16305E+01_JPRB, 0.16421E+01_JPRB, 0.15211E+01_JPRB, 0.16850E+01_JPRB, 0.18776E+01_JPRB, 0.23271E+01_JPRB, 0.22969E+01_JPRB ,&
&  0.18786E+01_JPRB, 0.14757E+01_JPRB, 0.13291E+01_JPRB, 0.13021E+01_JPRB, 0.13989E+01_JPRB, 0.15470E+01_JPRB, 0.19962E+01_JPRB ,&
&  0.20056E+01_JPRB, 0.14020E+01_JPRB, 0.11978E+01_JPRB, 0.15277E+01_JPRB, 0.13400E+01_JPRB, 0.12778E+01_JPRB, 0.16905E+01_JPRB ,&
&  0.20921E+01_JPRB, 0.23874E+01_JPRB, 0.23331E+01_JPRB, 0.20895E+01_JPRB, 0.19201E+01_JPRB, 0.17252E+01_JPRB, 0.17076E+01_JPRB ,&
&  0.15902E+01_JPRB, 0.14892E+01_JPRB, 0.14538E+01_JPRB, 0.14033E+01_JPRB, 0.13034E+01_JPRB, 0.11855E+01_JPRB, 0.10864E+01_JPRB ,&
&  0.10314E+01_JPRB, 0.99973E+00_JPRB, 0.97630E+00_JPRB, 0.96078E+00_JPRB, 0.94442E+00_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.95103E+00_JPRB, 0.90727E+00_JPRB, 0.87786E+00_JPRB, 0.85153E+00_JPRB, 0.81562E+00_JPRB, 0.80531E+00_JPRB, 0.85480E+00_JPRB ,&
&  0.85440E+00_JPRB, 0.76041E+00_JPRB, 0.70784E+00_JPRB, 0.66516E+00_JPRB, 0.63536E+00_JPRB, 0.61662E+00_JPRB, 0.60737E+00_JPRB ,&
&  0.59956E+00_JPRB, 0.60202E+00_JPRB, 0.60398E+00_JPRB, 0.66748E+00_JPRB, 0.78729E+00_JPRB, 0.98321E+00_JPRB, 0.81052E+00_JPRB ,&
&  0.11414E+01_JPRB, 0.13756E+01_JPRB, 0.13450E+01_JPRB, 0.12712E+01_JPRB, 0.11856E+01_JPRB, 0.11924E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.10090E+01_JPRB, 0.95502E+00_JPRB, 0.89943E+00_JPRB, 0.87034E+00_JPRB, 0.86529E+00_JPRB, 0.85015E+00_JPRB, 0.81881E+00_JPRB ,&
&  0.79691E+00_JPRB, 0.77949E+00_JPRB, 0.76546E+00_JPRB, 0.77502E+00_JPRB, 0.81921E+00_JPRB, 0.88840E+00_JPRB, 0.96957E+00_JPRB ,&
&  0.10614E+01_JPRB, 0.11004E+01_JPRB, 0.10775E+01_JPRB, 0.97666E+00_JPRB, 0.96690E+00_JPRB, 0.95917E+00_JPRB, 0.96672E+00_JPRB ,&
&  0.93349E+00_JPRB, 0.90806E+00_JPRB, 0.10673E+01_JPRB, 0.12429E+01_JPRB, 0.13330E+01_JPRB, 0.12891E+01_JPRB, 0.12807E+01_JPRB ,&
&  0.15355E+01_JPRB, 0.17575E+01_JPRB, 0.19212E+01_JPRB, 0.18560E+01_JPRB, 0.20634E+01_JPRB, 0.23174E+01_JPRB, 0.25088E+01_JPRB ,&
&  0.23942E+01_JPRB, 0.18166E+01_JPRB, 0.15273E+01_JPRB, 0.13815E+01_JPRB, 0.13424E+01_JPRB, 0.14855E+01_JPRB, 0.18597E+01_JPRB ,&
&  0.22992E+01_JPRB, 0.29171E+01_JPRB, 0.29317E+01_JPRB, 0.23611E+01_JPRB, 0.20890E+01_JPRB, 0.18903E+01_JPRB, 0.17932E+01_JPRB ,&
&  0.22882E+01_JPRB, 0.23719E+01_JPRB, 0.22608E+01_JPRB, 0.24826E+01_JPRB, 0.20207E+01_JPRB, 0.17772E+01_JPRB, 0.16696E+01_JPRB ,&
&  0.16039E+01_JPRB, 0.15242E+01_JPRB, 0.14495E+01_JPRB, 0.13718E+01_JPRB, 0.12495E+01_JPRB, 0.11399E+01_JPRB, 0.10739E+01_JPRB ,&
&  0.10370E+01_JPRB, 0.10178E+01_JPRB, 0.10002E+01_JPRB, 0.99108E+00_JPRB, 0.97998E+00_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.95022E+00_JPRB, 0.91102E+00_JPRB, 0.86747E+00_JPRB, 0.85302E+00_JPRB, 0.83954E+00_JPRB, 0.84733E+00_JPRB, 0.87731E+00_JPRB ,&
&  0.96253E+00_JPRB, 0.81177E+00_JPRB, 0.72807E+00_JPRB, 0.66937E+00_JPRB, 0.64303E+00_JPRB, 0.63653E+00_JPRB, 0.63081E+00_JPRB ,&
&  0.63312E+00_JPRB, 0.64378E+00_JPRB, 0.67202E+00_JPRB, 0.71561E+00_JPRB, 0.85254E+00_JPRB, 0.10614E+01_JPRB, 0.11065E+01_JPRB ,&
&  0.12807E+01_JPRB, 0.17631E+01_JPRB, 0.14801E+01_JPRB, 0.13042E+01_JPRB, 0.12225E+01_JPRB, 0.13117E+01_JPRB, 0.11674E+01_JPRB ,&
&  0.10128E+01_JPRB, 0.97523E+00_JPRB, 0.95510E+00_JPRB, 0.87815E+00_JPRB, 0.84724E+00_JPRB, 0.81255E+00_JPRB, 0.80352E+00_JPRB ,&
&  0.79323E+00_JPRB, 0.78363E+00_JPRB, 0.77446E+00_JPRB, 0.80430E+00_JPRB, 0.86428E+00_JPRB, 0.94353E+00_JPRB, 0.10610E+01_JPRB ,&
&  0.11689E+01_JPRB, 0.11405E+01_JPRB, 0.10539E+01_JPRB, 0.10100E+01_JPRB, 0.99495E+00_JPRB, 0.10025E+01_JPRB, 0.10100E+01_JPRB ,&
&  0.98765E+00_JPRB, 0.95665E+00_JPRB, 0.10511E+01_JPRB, 0.11011E+01_JPRB, 0.11252E+01_JPRB, 0.12162E+01_JPRB, 0.12335E+01_JPRB ,&
&  0.15292E+01_JPRB, 0.16357E+01_JPRB, 0.20883E+01_JPRB, 0.19175E+01_JPRB, 0.21536E+01_JPRB, 0.23547E+01_JPRB, 0.22196E+01_JPRB ,&
&  0.18504E+01_JPRB, 0.16509E+01_JPRB, 0.14127E+01_JPRB, 0.13308E+01_JPRB, 0.14121E+01_JPRB, 0.16628E+01_JPRB, 0.19543E+01_JPRB ,&
&  0.22858E+01_JPRB, 0.26408E+01_JPRB, 0.29367E+01_JPRB, 0.23976E+01_JPRB, 0.21737E+01_JPRB, 0.20366E+01_JPRB, 0.20632E+01_JPRB ,&
&  0.23930E+01_JPRB, 0.24776E+01_JPRB, 0.25414E+01_JPRB, 0.24812E+01_JPRB, 0.19209E+01_JPRB, 0.16915E+01_JPRB, 0.15828E+01_JPRB ,&
&  0.15139E+01_JPRB, 0.14419E+01_JPRB, 0.13245E+01_JPRB, 0.12265E+01_JPRB, 0.11539E+01_JPRB, 0.11041E+01_JPRB, 0.10541E+01_JPRB ,&
&  0.10253E+01_JPRB, 0.99819E+00_JPRB, 0.98451E+00_JPRB, 0.98487E+00_JPRB, 0.97498E+00_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.92117E+00_JPRB, 0.89449E+00_JPRB, 0.88321E+00_JPRB, 0.88761E+00_JPRB, 0.88945E+00_JPRB, 0.93088E+00_JPRB, 0.98103E+00_JPRB ,&
&  0.11107E+01_JPRB, 0.80693E+00_JPRB, 0.71059E+00_JPRB, 0.67623E+00_JPRB, 0.66548E+00_JPRB, 0.65943E+00_JPRB, 0.66374E+00_JPRB ,&
&  0.67275E+00_JPRB, 0.70269E+00_JPRB, 0.75193E+00_JPRB, 0.83284E+00_JPRB, 0.10233E+01_JPRB, 0.12928E+01_JPRB, 0.14879E+01_JPRB ,&
&  0.15934E+01_JPRB, 0.15957E+01_JPRB, 0.15171E+01_JPRB, 0.13102E+01_JPRB, 0.12243E+01_JPRB, 0.12775E+01_JPRB, 0.12455E+01_JPRB ,&
&  0.11237E+01_JPRB, 0.10219E+01_JPRB, 0.95743E+00_JPRB, 0.87689E+00_JPRB, 0.83610E+00_JPRB, 0.81990E+00_JPRB, 0.80773E+00_JPRB ,&
&  0.78646E+00_JPRB, 0.77311E+00_JPRB, 0.78069E+00_JPRB, 0.83884E+00_JPRB, 0.91621E+00_JPRB, 0.10322E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12422E+01_JPRB, 0.11636E+01_JPRB, 0.10933E+01_JPRB, 0.10203E+01_JPRB, 0.10195E+01_JPRB, 0.10280E+01_JPRB, 0.10321E+01_JPRB ,&
&  0.10258E+01_JPRB, 0.10573E+01_JPRB, 0.10904E+01_JPRB, 0.10967E+01_JPRB, 0.10723E+01_JPRB, 0.12241E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.15222E+01_JPRB, 0.17525E+01_JPRB, 0.22563E+01_JPRB, 0.18413E+01_JPRB, 0.22008E+01_JPRB, 0.22576E+01_JPRB, 0.20436E+01_JPRB ,&
&  0.17172E+01_JPRB, 0.15276E+01_JPRB, 0.13878E+01_JPRB, 0.13788E+01_JPRB, 0.15922E+01_JPRB, 0.18521E+01_JPRB, 0.24332E+01_JPRB ,&
&  0.24448E+01_JPRB, 0.27311E+01_JPRB, 0.25864E+01_JPRB, 0.25199E+01_JPRB, 0.22313E+01_JPRB, 0.21245E+01_JPRB, 0.21154E+01_JPRB ,&
&  0.23195E+01_JPRB, 0.23298E+01_JPRB, 0.21923E+01_JPRB, 0.20279E+01_JPRB, 0.16738E+01_JPRB, 0.14860E+01_JPRB, 0.13964E+01_JPRB ,&
&  0.13080E+01_JPRB, 0.12177E+01_JPRB, 0.11514E+01_JPRB, 0.11265E+01_JPRB, 0.10924E+01_JPRB, 0.10560E+01_JPRB, 0.10219E+01_JPRB ,&
&  0.10027E+01_JPRB, 0.98591E+00_JPRB, 0.97553E+00_JPRB, 0.96811E+00_JPRB, 0.95295E+00_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.87858E+00_JPRB, 0.86165E+00_JPRB, 0.85917E+00_JPRB, 0.84606E+00_JPRB, 0.83734E+00_JPRB, 0.83940E+00_JPRB, 0.82681E+00_JPRB ,&
&  0.77064E+00_JPRB, 0.72261E+00_JPRB, 0.70585E+00_JPRB, 0.70325E+00_JPRB, 0.69616E+00_JPRB, 0.68618E+00_JPRB, 0.68996E+00_JPRB ,&
&  0.71265E+00_JPRB, 0.76267E+00_JPRB, 0.84335E+00_JPRB, 0.99739E+00_JPRB, 0.12011E+01_JPRB, 0.14435E+01_JPRB, 0.16605E+01_JPRB ,&
&  0.16529E+01_JPRB, 0.12073E+01_JPRB, 0.13698E+01_JPRB, 0.13709E+01_JPRB, 0.12211E+01_JPRB, 0.12169E+01_JPRB, 0.12020E+01_JPRB ,&
&  0.10553E+01_JPRB, 0.10022E+01_JPRB, 0.97422E+00_JPRB, 0.92680E+00_JPRB, 0.84769E+00_JPRB, 0.82566E+00_JPRB, 0.80158E+00_JPRB ,&
&  0.78846E+00_JPRB, 0.78901E+00_JPRB, 0.82112E+00_JPRB, 0.91807E+00_JPRB, 0.10419E+01_JPRB, 0.12241E+01_JPRB, 0.13256E+01_JPRB ,&
&  0.12787E+01_JPRB, 0.12295E+01_JPRB, 0.12014E+01_JPRB, 0.11208E+01_JPRB, 0.11072E+01_JPRB, 0.11197E+01_JPRB, 0.11226E+01_JPRB ,&
&  0.11203E+01_JPRB, 0.11171E+01_JPRB, 0.11637E+01_JPRB, 0.12489E+01_JPRB, 0.13276E+01_JPRB, 0.12826E+01_JPRB, 0.14528E+01_JPRB ,&
&  0.16319E+01_JPRB, 0.19650E+01_JPRB, 0.21846E+01_JPRB, 0.25084E+01_JPRB, 0.18285E+01_JPRB, 0.20016E+01_JPRB, 0.16718E+01_JPRB ,&
&  0.16488E+01_JPRB, 0.15032E+01_JPRB, 0.15385E+01_JPRB, 0.16523E+01_JPRB, 0.19619E+01_JPRB, 0.22190E+01_JPRB, 0.27412E+01_JPRB ,&
&  0.25084E+01_JPRB, 0.24273E+01_JPRB, 0.23326E+01_JPRB, 0.24504E+01_JPRB, 0.22865E+01_JPRB, 0.22913E+01_JPRB, 0.21245E+01_JPRB ,&
&  0.21371E+01_JPRB, 0.19739E+01_JPRB, 0.17905E+01_JPRB, 0.16801E+01_JPRB, 0.14600E+01_JPRB, 0.12306E+01_JPRB, 0.11287E+01_JPRB ,&
&  0.10659E+01_JPRB, 0.10580E+01_JPRB, 0.10518E+01_JPRB, 0.10453E+01_JPRB, 0.10296E+01_JPRB, 0.10102E+01_JPRB, 0.99299E+00_JPRB ,&
&  0.97955E+00_JPRB, 0.96388E+00_JPRB, 0.95124E+00_JPRB, 0.93716E+00_JPRB, 0.91206E+00_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.83839E+00_JPRB, 0.82162E+00_JPRB, 0.81730E+00_JPRB, 0.80931E+00_JPRB, 0.80366E+00_JPRB, 0.79550E+00_JPRB, 0.78172E+00_JPRB ,&
&  0.75609E+00_JPRB, 0.74329E+00_JPRB, 0.73953E+00_JPRB, 0.73407E+00_JPRB, 0.71455E+00_JPRB, 0.70662E+00_JPRB, 0.72781E+00_JPRB ,&
&  0.78435E+00_JPRB, 0.88030E+00_JPRB, 0.10062E+01_JPRB, 0.11513E+01_JPRB, 0.13446E+01_JPRB, 0.14901E+01_JPRB, 0.15352E+01_JPRB ,&
&  0.16448E+01_JPRB, 0.16856E+01_JPRB, 0.15002E+01_JPRB, 0.11091E+01_JPRB, 0.99572E+00_JPRB, 0.11564E+01_JPRB, 0.10840E+01_JPRB ,&
&  0.10029E+01_JPRB, 0.96715E+00_JPRB, 0.93731E+00_JPRB, 0.90540E+00_JPRB, 0.86742E+00_JPRB, 0.84499E+00_JPRB, 0.83159E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.86696E+00_JPRB, 0.95817E+00_JPRB, 0.11297E+01_JPRB, 0.12973E+01_JPRB, 0.14143E+01_JPRB, 0.14159E+01_JPRB ,&
&  0.14061E+01_JPRB, 0.14234E+01_JPRB, 0.14149E+01_JPRB, 0.13372E+01_JPRB, 0.12788E+01_JPRB, 0.12933E+01_JPRB, 0.13076E+01_JPRB ,&
&  0.13168E+01_JPRB, 0.13222E+01_JPRB, 0.13338E+01_JPRB, 0.14097E+01_JPRB, 0.14696E+01_JPRB, 0.13948E+01_JPRB, 0.14907E+01_JPRB ,&
&  0.17960E+01_JPRB, 0.22369E+01_JPRB, 0.22285E+01_JPRB, 0.26547E+01_JPRB, 0.15689E+01_JPRB, 0.17553E+01_JPRB, 0.17719E+01_JPRB ,&
&  0.15554E+01_JPRB, 0.16219E+01_JPRB, 0.17354E+01_JPRB, 0.19351E+01_JPRB, 0.22435E+01_JPRB, 0.23648E+01_JPRB, 0.25832E+01_JPRB ,&
&  0.22973E+01_JPRB, 0.21033E+01_JPRB, 0.20835E+01_JPRB, 0.21668E+01_JPRB, 0.21730E+01_JPRB, 0.21184E+01_JPRB, 0.18437E+01_JPRB ,&
&  0.16062E+01_JPRB, 0.16716E+01_JPRB, 0.15289E+01_JPRB, 0.15047E+01_JPRB, 0.12884E+01_JPRB, 0.10370E+01_JPRB, 0.10302E+01_JPRB ,&
&  0.10205E+01_JPRB, 0.10034E+01_JPRB, 0.10001E+01_JPRB, 0.98337E+00_JPRB, 0.96981E+00_JPRB, 0.95794E+00_JPRB, 0.94534E+00_JPRB ,&
&  0.93660E+00_JPRB, 0.92679E+00_JPRB, 0.91252E+00_JPRB, 0.89226E+00_JPRB, 0.86583E+00_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.80696E+00_JPRB, 0.79453E+00_JPRB, 0.79654E+00_JPRB, 0.78142E+00_JPRB, 0.76457E+00_JPRB, 0.76235E+00_JPRB, 0.76522E+00_JPRB ,&
&  0.75755E+00_JPRB, 0.75218E+00_JPRB, 0.74226E+00_JPRB, 0.73429E+00_JPRB, 0.73638E+00_JPRB, 0.76423E+00_JPRB, 0.81910E+00_JPRB ,&
&  0.89428E+00_JPRB, 0.10081E+01_JPRB, 0.11564E+01_JPRB, 0.12955E+01_JPRB, 0.14599E+01_JPRB, 0.15350E+01_JPRB, 0.15091E+01_JPRB ,&
&  0.15609E+01_JPRB, 0.16265E+01_JPRB, 0.16137E+01_JPRB, 0.15902E+01_JPRB, 0.12942E+01_JPRB, 0.13426E+01_JPRB, 0.13818E+01_JPRB ,&
&  0.13117E+01_JPRB, 0.11026E+01_JPRB, 0.98031E+00_JPRB, 0.94720E+00_JPRB, 0.91181E+00_JPRB, 0.90828E+00_JPRB, 0.93377E+00_JPRB ,&
&  0.99052E+00_JPRB, 0.10890E+01_JPRB, 0.12433E+01_JPRB, 0.14139E+01_JPRB, 0.15260E+01_JPRB, 0.15763E+01_JPRB, 0.15959E+01_JPRB ,&
&  0.16275E+01_JPRB, 0.16649E+01_JPRB, 0.17043E+01_JPRB, 0.16515E+01_JPRB, 0.16352E+01_JPRB, 0.16340E+01_JPRB, 0.16833E+01_JPRB ,&
&  0.17334E+01_JPRB, 0.16649E+01_JPRB, 0.16730E+01_JPRB, 0.17780E+01_JPRB, 0.17934E+01_JPRB, 0.17732E+01_JPRB, 0.18433E+01_JPRB ,&
&  0.21383E+01_JPRB, 0.26302E+01_JPRB, 0.20817E+01_JPRB, 0.21611E+01_JPRB, 0.24268E+01_JPRB, 0.19631E+01_JPRB, 0.16473E+01_JPRB ,&
&  0.16263E+01_JPRB, 0.17137E+01_JPRB, 0.18250E+01_JPRB, 0.20130E+01_JPRB, 0.23065E+01_JPRB, 0.26442E+01_JPRB, 0.22492E+01_JPRB ,&
&  0.20110E+01_JPRB, 0.18289E+01_JPRB, 0.18310E+01_JPRB, 0.18491E+01_JPRB, 0.18312E+01_JPRB, 0.17822E+01_JPRB, 0.15833E+01_JPRB ,&
&  0.14385E+01_JPRB, 0.14969E+01_JPRB, 0.14524E+01_JPRB, 0.14468E+01_JPRB, 0.15032E+01_JPRB, 0.10518E+01_JPRB, 0.10226E+01_JPRB ,&
&  0.96722E+00_JPRB, 0.95935E+00_JPRB, 0.95438E+00_JPRB, 0.94164E+00_JPRB, 0.92770E+00_JPRB, 0.91180E+00_JPRB, 0.90162E+00_JPRB ,&
&  0.89211E+00_JPRB, 0.87679E+00_JPRB, 0.86397E+00_JPRB, 0.85288E+00_JPRB, 0.82658E+00_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.78716E+00_JPRB, 0.78248E+00_JPRB, 0.78235E+00_JPRB, 0.77222E+00_JPRB, 0.75677E+00_JPRB, 0.75688E+00_JPRB, 0.75111E+00_JPRB ,&
&  0.73398E+00_JPRB, 0.71715E+00_JPRB, 0.69749E+00_JPRB, 0.70700E+00_JPRB, 0.72942E+00_JPRB, 0.77589E+00_JPRB, 0.83413E+00_JPRB ,&
&  0.91212E+00_JPRB, 0.10289E+01_JPRB, 0.11865E+01_JPRB, 0.13095E+01_JPRB, 0.14773E+01_JPRB, 0.15504E+01_JPRB, 0.15863E+01_JPRB ,&
&  0.16485E+01_JPRB, 0.17213E+01_JPRB, 0.17749E+01_JPRB, 0.17939E+01_JPRB, 0.17468E+01_JPRB, 0.17265E+01_JPRB, 0.17581E+01_JPRB ,&
&  0.14549E+01_JPRB, 0.11215E+01_JPRB, 0.10473E+01_JPRB, 0.10210E+01_JPRB, 0.10262E+01_JPRB, 0.10510E+01_JPRB, 0.11217E+01_JPRB ,&
&  0.12318E+01_JPRB, 0.13557E+01_JPRB, 0.15125E+01_JPRB, 0.16335E+01_JPRB, 0.17325E+01_JPRB, 0.18062E+01_JPRB, 0.18582E+01_JPRB ,&
&  0.19431E+01_JPRB, 0.20352E+01_JPRB, 0.21738E+01_JPRB, 0.20857E+01_JPRB, 0.22757E+01_JPRB, 0.22731E+01_JPRB, 0.22897E+01_JPRB ,&
&  0.22318E+01_JPRB, 0.20833E+01_JPRB, 0.21518E+01_JPRB, 0.22552E+01_JPRB, 0.23604E+01_JPRB, 0.23780E+01_JPRB, 0.23490E+01_JPRB ,&
&  0.26311E+01_JPRB, 0.27070E+01_JPRB, 0.20076E+01_JPRB, 0.17455E+01_JPRB, 0.17583E+01_JPRB, 0.14486E+01_JPRB, 0.16042E+01_JPRB ,&
&  0.15973E+01_JPRB, 0.16726E+01_JPRB, 0.17631E+01_JPRB, 0.19373E+01_JPRB, 0.21243E+01_JPRB, 0.22416E+01_JPRB, 0.19803E+01_JPRB ,&
&  0.16683E+01_JPRB, 0.15334E+01_JPRB, 0.15068E+01_JPRB, 0.14819E+01_JPRB, 0.14864E+01_JPRB, 0.14789E+01_JPRB, 0.14261E+01_JPRB ,&
&  0.13634E+01_JPRB, 0.12726E+01_JPRB, 0.12177E+01_JPRB, 0.10800E+01_JPRB, 0.98801E+00_JPRB, 0.97073E+00_JPRB, 0.91935E+00_JPRB ,&
&  0.91052E+00_JPRB, 0.92299E+00_JPRB, 0.92174E+00_JPRB, 0.91077E+00_JPRB, 0.90331E+00_JPRB, 0.88338E+00_JPRB, 0.87306E+00_JPRB ,&
&  0.86742E+00_JPRB, 0.85210E+00_JPRB, 0.83681E+00_JPRB, 0.82725E+00_JPRB, 0.80347E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.75689E+00_JPRB, 0.74511E+00_JPRB, 0.74186E+00_JPRB, 0.74071E+00_JPRB, 0.72673E+00_JPRB, 0.71531E+00_JPRB, 0.70677E+00_JPRB ,&
&  0.67441E+00_JPRB, 0.67090E+00_JPRB, 0.65726E+00_JPRB, 0.66405E+00_JPRB, 0.69077E+00_JPRB, 0.71688E+00_JPRB, 0.75964E+00_JPRB ,&
&  0.80173E+00_JPRB, 0.85894E+00_JPRB, 0.95083E+00_JPRB, 0.11421E+01_JPRB, 0.13556E+01_JPRB, 0.15114E+01_JPRB, 0.16953E+01_JPRB ,&
&  0.17639E+01_JPRB, 0.18226E+01_JPRB, 0.19470E+01_JPRB, 0.21095E+01_JPRB, 0.21844E+01_JPRB, 0.22224E+01_JPRB, 0.21448E+01_JPRB ,&
&  0.15774E+01_JPRB, 0.11030E+01_JPRB, 0.11303E+01_JPRB, 0.10746E+01_JPRB, 0.11260E+01_JPRB, 0.11932E+01_JPRB, 0.12724E+01_JPRB ,&
&  0.13951E+01_JPRB, 0.15215E+01_JPRB, 0.16523E+01_JPRB, 0.17590E+01_JPRB, 0.18851E+01_JPRB, 0.20208E+01_JPRB, 0.20999E+01_JPRB ,&
&  0.21743E+01_JPRB, 0.22551E+01_JPRB, 0.23248E+01_JPRB, 0.22844E+01_JPRB, 0.24095E+01_JPRB, 0.25784E+01_JPRB, 0.27180E+01_JPRB ,&
&  0.28505E+01_JPRB, 0.28540E+01_JPRB, 0.24020E+01_JPRB, 0.24910E+01_JPRB, 0.28373E+01_JPRB, 0.26526E+01_JPRB, 0.27166E+01_JPRB ,&
&  0.31460E+01_JPRB, 0.32217E+01_JPRB, 0.18325E+01_JPRB, 0.15577E+01_JPRB, 0.15528E+01_JPRB, 0.14847E+01_JPRB, 0.14981E+01_JPRB ,&
&  0.15016E+01_JPRB, 0.15854E+01_JPRB, 0.16990E+01_JPRB, 0.17708E+01_JPRB, 0.17947E+01_JPRB, 0.16893E+01_JPRB, 0.15812E+01_JPRB ,&
&  0.13629E+01_JPRB, 0.12579E+01_JPRB, 0.12588E+01_JPRB, 0.12745E+01_JPRB, 0.12906E+01_JPRB, 0.13175E+01_JPRB, 0.12298E+01_JPRB ,&
&  0.11962E+01_JPRB, 0.11633E+01_JPRB, 0.12569E+01_JPRB, 0.11065E+01_JPRB, 0.90927E+00_JPRB, 0.89216E+00_JPRB, 0.86304E+00_JPRB ,&
&  0.89819E+00_JPRB, 0.91501E+00_JPRB, 0.92068E+00_JPRB, 0.92144E+00_JPRB, 0.91749E+00_JPRB, 0.90396E+00_JPRB, 0.88813E+00_JPRB ,&
&  0.86851E+00_JPRB, 0.84658E+00_JPRB, 0.82215E+00_JPRB, 0.79846E+00_JPRB, 0.77641E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.68238E+00_JPRB, 0.67015E+00_JPRB, 0.67409E+00_JPRB, 0.64913E+00_JPRB, 0.66741E+00_JPRB, 0.64935E+00_JPRB, 0.67850E+00_JPRB ,&
&  0.65406E+00_JPRB, 0.64888E+00_JPRB, 0.64829E+00_JPRB, 0.65963E+00_JPRB, 0.67164E+00_JPRB, 0.68929E+00_JPRB, 0.70340E+00_JPRB ,&
&  0.72663E+00_JPRB, 0.77147E+00_JPRB, 0.83431E+00_JPRB, 0.90665E+00_JPRB, 0.10008E+01_JPRB, 0.10939E+01_JPRB, 0.11748E+01_JPRB ,&
&  0.12838E+01_JPRB, 0.13771E+01_JPRB, 0.14289E+01_JPRB, 0.15699E+01_JPRB, 0.19759E+01_JPRB, 0.24728E+01_JPRB, 0.25150E+01_JPRB ,&
&  0.16039E+01_JPRB, 0.11677E+01_JPRB, 0.10729E+01_JPRB, 0.10110E+01_JPRB, 0.10824E+01_JPRB, 0.12405E+01_JPRB, 0.13189E+01_JPRB ,&
&  0.13974E+01_JPRB, 0.14931E+01_JPRB, 0.16043E+01_JPRB, 0.17252E+01_JPRB, 0.18724E+01_JPRB, 0.20214E+01_JPRB, 0.21056E+01_JPRB ,&
&  0.21239E+01_JPRB, 0.21778E+01_JPRB, 0.21998E+01_JPRB, 0.23691E+01_JPRB, 0.26605E+01_JPRB, 0.30047E+01_JPRB, 0.32655E+01_JPRB ,&
&  0.34051E+01_JPRB, 0.33435E+01_JPRB, 0.28360E+01_JPRB, 0.28751E+01_JPRB, 0.30347E+01_JPRB, 0.29738E+01_JPRB, 0.28595E+01_JPRB ,&
&  0.31401E+01_JPRB, 0.29401E+01_JPRB, 0.18757E+01_JPRB, 0.16700E+01_JPRB, 0.15535E+01_JPRB, 0.14637E+01_JPRB, 0.13686E+01_JPRB ,&
&  0.14177E+01_JPRB, 0.14964E+01_JPRB, 0.15488E+01_JPRB, 0.15381E+01_JPRB, 0.14821E+01_JPRB, 0.13980E+01_JPRB, 0.13718E+01_JPRB ,&
&  0.12527E+01_JPRB, 0.12228E+01_JPRB, 0.12170E+01_JPRB, 0.11858E+01_JPRB, 0.13000E+01_JPRB, 0.11572E+01_JPRB, 0.11654E+01_JPRB ,&
&  0.10331E+01_JPRB, 0.10101E+01_JPRB, 0.87446E+00_JPRB, 0.10520E+01_JPRB, 0.88943E+00_JPRB, 0.79626E+00_JPRB, 0.81778E+00_JPRB ,&
&  0.83472E+00_JPRB, 0.86066E+00_JPRB, 0.85423E+00_JPRB, 0.85879E+00_JPRB, 0.86367E+00_JPRB, 0.85271E+00_JPRB, 0.84331E+00_JPRB ,&
&  0.82771E+00_JPRB, 0.80868E+00_JPRB, 0.77436E+00_JPRB, 0.74311E+00_JPRB, 0.71253E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.66704E+00_JPRB, 0.67117E+00_JPRB, 0.65680E+00_JPRB, 0.63072E+00_JPRB, 0.61234E+00_JPRB, 0.60211E+00_JPRB, 0.61029E+00_JPRB ,&
&  0.61999E+00_JPRB, 0.64089E+00_JPRB, 0.65773E+00_JPRB, 0.67607E+00_JPRB, 0.68524E+00_JPRB, 0.69230E+00_JPRB, 0.70826E+00_JPRB ,&
&  0.72939E+00_JPRB, 0.75061E+00_JPRB, 0.77741E+00_JPRB, 0.81405E+00_JPRB, 0.84791E+00_JPRB, 0.89031E+00_JPRB, 0.90803E+00_JPRB ,&
&  0.90681E+00_JPRB, 0.93330E+00_JPRB, 0.95406E+00_JPRB, 0.10024E+01_JPRB, 0.12098E+01_JPRB, 0.14244E+01_JPRB, 0.16596E+01_JPRB ,&
&  0.12740E+01_JPRB, 0.12832E+01_JPRB, 0.11614E+01_JPRB, 0.10822E+01_JPRB, 0.11203E+01_JPRB, 0.11181E+01_JPRB, 0.11587E+01_JPRB ,&
&  0.12178E+01_JPRB, 0.12548E+01_JPRB, 0.12893E+01_JPRB, 0.13343E+01_JPRB, 0.13825E+01_JPRB, 0.14768E+01_JPRB, 0.14498E+01_JPRB ,&
&  0.14451E+01_JPRB, 0.15181E+01_JPRB, 0.16087E+01_JPRB, 0.18600E+01_JPRB, 0.22242E+01_JPRB, 0.25619E+01_JPRB, 0.26014E+01_JPRB ,&
&  0.26389E+01_JPRB, 0.28328E+01_JPRB, 0.27683E+01_JPRB, 0.27913E+01_JPRB, 0.27472E+01_JPRB, 0.26440E+01_JPRB, 0.26422E+01_JPRB ,&
&  0.25046E+01_JPRB, 0.22319E+01_JPRB, 0.19529E+01_JPRB, 0.16723E+01_JPRB, 0.14835E+01_JPRB, 0.13326E+01_JPRB, 0.12866E+01_JPRB ,&
&  0.12797E+01_JPRB, 0.13269E+01_JPRB, 0.13493E+01_JPRB, 0.12944E+01_JPRB, 0.12533E+01_JPRB, 0.11985E+01_JPRB, 0.11960E+01_JPRB ,&
&  0.11772E+01_JPRB, 0.10861E+01_JPRB, 0.10422E+01_JPRB, 0.10470E+01_JPRB, 0.10741E+01_JPRB, 0.16042E+01_JPRB, 0.14789E+01_JPRB ,&
&  0.92596E+00_JPRB, 0.76340E+00_JPRB, 0.61147E+00_JPRB, 0.87706E+00_JPRB, 0.73642E+00_JPRB, 0.72037E+00_JPRB, 0.74230E+00_JPRB ,&
&  0.76330E+00_JPRB, 0.77845E+00_JPRB, 0.78791E+00_JPRB, 0.78036E+00_JPRB, 0.77792E+00_JPRB, 0.77588E+00_JPRB, 0.75948E+00_JPRB ,&
&  0.74593E+00_JPRB, 0.72806E+00_JPRB, 0.72290E+00_JPRB, 0.68973E+00_JPRB, 0.67285E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.64397E+00_JPRB, 0.64351E+00_JPRB, 0.62057E+00_JPRB, 0.58777E+00_JPRB, 0.57279E+00_JPRB, 0.58092E+00_JPRB, 0.59708E+00_JPRB ,&
&  0.61250E+00_JPRB, 0.63307E+00_JPRB, 0.65692E+00_JPRB, 0.68021E+00_JPRB, 0.70102E+00_JPRB, 0.72350E+00_JPRB, 0.75015E+00_JPRB ,&
&  0.77542E+00_JPRB, 0.80649E+00_JPRB, 0.83800E+00_JPRB, 0.86965E+00_JPRB, 0.89704E+00_JPRB, 0.94509E+00_JPRB, 0.97889E+00_JPRB ,&
&  0.10322E+01_JPRB, 0.10857E+01_JPRB, 0.11510E+01_JPRB, 0.12147E+01_JPRB, 0.12946E+01_JPRB, 0.13523E+01_JPRB, 0.11769E+01_JPRB ,&
&  0.13600E+01_JPRB, 0.13047E+01_JPRB, 0.12000E+01_JPRB, 0.11180E+01_JPRB, 0.10594E+01_JPRB, 0.10176E+01_JPRB, 0.97467E+00_JPRB ,&
&  0.95796E+00_JPRB, 0.94317E+00_JPRB, 0.91204E+00_JPRB, 0.89105E+00_JPRB, 0.89058E+00_JPRB, 0.96371E+00_JPRB, 0.97346E+00_JPRB ,&
&  0.10674E+01_JPRB, 0.12267E+01_JPRB, 0.13669E+01_JPRB, 0.15945E+01_JPRB, 0.17912E+01_JPRB, 0.19310E+01_JPRB, 0.18603E+01_JPRB ,&
&  0.18883E+01_JPRB, 0.20736E+01_JPRB, 0.20508E+01_JPRB, 0.21104E+01_JPRB, 0.20880E+01_JPRB, 0.19837E+01_JPRB, 0.19142E+01_JPRB ,&
&  0.17029E+01_JPRB, 0.16232E+01_JPRB, 0.16276E+01_JPRB, 0.17307E+01_JPRB, 0.14656E+01_JPRB, 0.12880E+01_JPRB, 0.12629E+01_JPRB ,&
&  0.11861E+01_JPRB, 0.12022E+01_JPRB, 0.12053E+01_JPRB, 0.11374E+01_JPRB, 0.10859E+01_JPRB, 0.10441E+01_JPRB, 0.10441E+01_JPRB ,&
&  0.10463E+01_JPRB, 0.94246E+00_JPRB, 0.91902E+00_JPRB, 0.97167E+00_JPRB, 0.10901E+01_JPRB, 0.11512E+01_JPRB, 0.13746E+01_JPRB ,&
&  0.86087E+00_JPRB, 0.54277E+00_JPRB, 0.58938E+00_JPRB, 0.67743E+00_JPRB, 0.63076E+00_JPRB, 0.62359E+00_JPRB, 0.66920E+00_JPRB ,&
&  0.71194E+00_JPRB, 0.73679E+00_JPRB, 0.72736E+00_JPRB, 0.75034E+00_JPRB, 0.78673E+00_JPRB, 0.77544E+00_JPRB, 0.71955E+00_JPRB ,&
&  0.72619E+00_JPRB, 0.69006E+00_JPRB, 0.67098E+00_JPRB, 0.66171E+00_JPRB, 0.65486E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.61522E+00_JPRB, 0.60597E+00_JPRB, 0.58273E+00_JPRB, 0.55014E+00_JPRB, 0.53822E+00_JPRB, 0.55596E+00_JPRB, 0.58155E+00_JPRB ,&
&  0.60152E+00_JPRB, 0.62568E+00_JPRB, 0.65057E+00_JPRB, 0.67790E+00_JPRB, 0.70789E+00_JPRB, 0.73590E+00_JPRB, 0.75749E+00_JPRB ,&
&  0.77275E+00_JPRB, 0.79340E+00_JPRB, 0.81182E+00_JPRB, 0.82712E+00_JPRB, 0.83292E+00_JPRB, 0.83596E+00_JPRB, 0.83805E+00_JPRB ,&
&  0.87350E+00_JPRB, 0.91010E+00_JPRB, 0.95827E+00_JPRB, 0.10069E+01_JPRB, 0.10398E+01_JPRB, 0.10389E+01_JPRB, 0.86010E+00_JPRB ,&
&  0.12403E+01_JPRB, 0.12608E+01_JPRB, 0.11810E+01_JPRB, 0.10965E+01_JPRB, 0.10128E+01_JPRB, 0.95123E+00_JPRB, 0.88982E+00_JPRB ,&
&  0.83668E+00_JPRB, 0.87279E+00_JPRB, 0.85602E+00_JPRB, 0.86664E+00_JPRB, 0.83574E+00_JPRB, 0.89892E+00_JPRB, 0.97422E+00_JPRB ,&
&  0.11198E+01_JPRB, 0.12805E+01_JPRB, 0.14241E+01_JPRB, 0.15516E+01_JPRB, 0.16141E+01_JPRB, 0.16647E+01_JPRB, 0.15887E+01_JPRB ,&
&  0.15977E+01_JPRB, 0.16042E+01_JPRB, 0.15058E+01_JPRB, 0.15622E+01_JPRB, 0.16203E+01_JPRB, 0.14001E+01_JPRB, 0.12423E+01_JPRB ,&
&  0.11249E+01_JPRB, 0.12513E+01_JPRB, 0.16048E+01_JPRB, 0.17391E+01_JPRB, 0.14749E+01_JPRB, 0.13604E+01_JPRB, 0.12558E+01_JPRB ,&
&  0.11432E+01_JPRB, 0.11236E+01_JPRB, 0.10951E+01_JPRB, 0.10387E+01_JPRB, 0.95097E+00_JPRB, 0.90216E+00_JPRB, 0.90379E+00_JPRB ,&
&  0.91105E+00_JPRB, 0.83006E+00_JPRB, 0.78258E+00_JPRB, 0.84130E+00_JPRB, 0.88460E+00_JPRB, 0.82581E+00_JPRB, 0.12880E+01_JPRB ,&
&  0.86172E+00_JPRB, 0.63535E+00_JPRB, 0.84085E+00_JPRB, 0.45030E+00_JPRB, 0.53795E+00_JPRB, 0.52272E+00_JPRB, 0.54466E+00_JPRB ,&
&  0.47032E+00_JPRB, 0.54984E+00_JPRB, 0.90955E+00_JPRB, 0.12351E+01_JPRB, 0.14207E+01_JPRB, 0.14105E+01_JPRB, 0.80761E+00_JPRB ,&
&  0.71058E+00_JPRB, 0.68454E+00_JPRB, 0.66550E+00_JPRB, 0.64298E+00_JPRB, 0.61886E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.58487E+00_JPRB, 0.55248E+00_JPRB, 0.54086E+00_JPRB, 0.53422E+00_JPRB, 0.52518E+00_JPRB, 0.54460E+00_JPRB, 0.56037E+00_JPRB ,&
&  0.58322E+00_JPRB, 0.61122E+00_JPRB, 0.64516E+00_JPRB, 0.67498E+00_JPRB, 0.71724E+00_JPRB, 0.75822E+00_JPRB, 0.79008E+00_JPRB ,&
&  0.81534E+00_JPRB, 0.83447E+00_JPRB, 0.84880E+00_JPRB, 0.85486E+00_JPRB, 0.85598E+00_JPRB, 0.85587E+00_JPRB, 0.85324E+00_JPRB ,&
&  0.85020E+00_JPRB, 0.86868E+00_JPRB, 0.90852E+00_JPRB, 0.95458E+00_JPRB, 0.98026E+00_JPRB, 0.10131E+01_JPRB, 0.66795E+00_JPRB ,&
&  0.10832E+01_JPRB, 0.11314E+01_JPRB, 0.11173E+01_JPRB, 0.10482E+01_JPRB, 0.97545E+00_JPRB, 0.91920E+00_JPRB, 0.87582E+00_JPRB ,&
&  0.85041E+00_JPRB, 0.87682E+00_JPRB, 0.87711E+00_JPRB, 0.99436E+00_JPRB, 0.10726E+01_JPRB, 0.10838E+01_JPRB, 0.12108E+01_JPRB ,&
&  0.13161E+01_JPRB, 0.14755E+01_JPRB, 0.15819E+01_JPRB, 0.16646E+01_JPRB, 0.17293E+01_JPRB, 0.17882E+01_JPRB, 0.18147E+01_JPRB ,&
&  0.18292E+01_JPRB, 0.17882E+01_JPRB, 0.17343E+01_JPRB, 0.18227E+01_JPRB, 0.18598E+01_JPRB, 0.15953E+01_JPRB, 0.14539E+01_JPRB ,&
&  0.10737E+01_JPRB, 0.97390E+00_JPRB, 0.14115E+01_JPRB, 0.16822E+01_JPRB, 0.15171E+01_JPRB, 0.14302E+01_JPRB, 0.12701E+01_JPRB ,&
&  0.11142E+01_JPRB, 0.10324E+01_JPRB, 0.98169E+00_JPRB, 0.92512E+00_JPRB, 0.85506E+00_JPRB, 0.80714E+00_JPRB, 0.78398E+00_JPRB ,&
&  0.76228E+00_JPRB, 0.68707E+00_JPRB, 0.65141E+00_JPRB, 0.67608E+00_JPRB, 0.69438E+00_JPRB, 0.79118E+00_JPRB, 0.80551E+00_JPRB ,&
&  0.95640E+00_JPRB, 0.86906E+00_JPRB, 0.69506E+00_JPRB, 0.65034E+00_JPRB, 0.51558E+00_JPRB, 0.48061E+00_JPRB, 0.47539E+00_JPRB ,&
&  0.49668E+00_JPRB, 0.45379E+00_JPRB, 0.94960E+00_JPRB, 0.37360E+01_JPRB, 0.36935E+01_JPRB, 0.46849E+01_JPRB, 0.24447E+01_JPRB ,&
&  0.11590E+01_JPRB, 0.71939E+00_JPRB, 0.62520E+00_JPRB, 0.61630E+00_JPRB, 0.59816E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.52794E+00_JPRB, 0.50869E+00_JPRB, 0.49070E+00_JPRB, 0.50181E+00_JPRB, 0.51072E+00_JPRB, 0.51769E+00_JPRB, 0.54514E+00_JPRB ,&
&  0.60172E+00_JPRB, 0.61612E+00_JPRB, 0.64333E+00_JPRB, 0.68721E+00_JPRB, 0.75124E+00_JPRB, 0.80115E+00_JPRB, 0.84696E+00_JPRB ,&
&  0.89305E+00_JPRB, 0.92068E+00_JPRB, 0.91652E+00_JPRB, 0.90362E+00_JPRB, 0.90265E+00_JPRB, 0.90812E+00_JPRB, 0.92206E+00_JPRB ,&
&  0.92444E+00_JPRB, 0.94336E+00_JPRB, 0.98091E+00_JPRB, 0.10251E+01_JPRB, 0.10626E+01_JPRB, 0.10506E+01_JPRB, 0.67367E+00_JPRB ,&
&  0.88063E+00_JPRB, 0.95162E+00_JPRB, 0.98971E+00_JPRB, 0.97288E+00_JPRB, 0.91287E+00_JPRB, 0.84842E+00_JPRB, 0.84624E+00_JPRB ,&
&  0.90764E+00_JPRB, 0.99673E+00_JPRB, 0.10795E+01_JPRB, 0.11487E+01_JPRB, 0.13472E+01_JPRB, 0.13778E+01_JPRB, 0.16076E+01_JPRB ,&
&  0.17337E+01_JPRB, 0.18319E+01_JPRB, 0.19629E+01_JPRB, 0.20441E+01_JPRB, 0.21492E+01_JPRB, 0.22749E+01_JPRB, 0.23931E+01_JPRB ,&
&  0.24511E+01_JPRB, 0.24520E+01_JPRB, 0.25383E+01_JPRB, 0.28022E+01_JPRB, 0.25865E+01_JPRB, 0.28925E+01_JPRB, 0.23480E+01_JPRB ,&
&  0.15703E+01_JPRB, 0.13083E+01_JPRB, 0.15462E+01_JPRB, 0.16538E+01_JPRB, 0.15397E+01_JPRB, 0.14819E+01_JPRB, 0.13344E+01_JPRB ,&
&  0.11260E+01_JPRB, 0.98788E+00_JPRB, 0.92634E+00_JPRB, 0.87608E+00_JPRB, 0.81950E+00_JPRB, 0.75987E+00_JPRB, 0.71134E+00_JPRB ,&
&  0.65566E+00_JPRB, 0.61618E+00_JPRB, 0.63010E+00_JPRB, 0.63000E+00_JPRB, 0.66075E+00_JPRB, 0.79330E+00_JPRB, 0.10983E+01_JPRB ,&
&  0.17109E+01_JPRB, 0.92634E+00_JPRB, 0.74133E+00_JPRB, 0.65514E+00_JPRB, 0.56818E+00_JPRB, 0.52004E+00_JPRB, 0.49178E+00_JPRB ,&
&  0.49218E+00_JPRB, 0.55865E+00_JPRB, 0.68926E+00_JPRB, 0.14059E+01_JPRB, 0.25556E+01_JPRB, 0.26166E+01_JPRB, 0.17638E+01_JPRB ,&
&  0.10720E+01_JPRB, 0.74133E+00_JPRB, 0.59373E+00_JPRB, 0.55835E+00_JPRB, 0.53336E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.49973E+00_JPRB, 0.48705E+00_JPRB, 0.48718E+00_JPRB, 0.48669E+00_JPRB, 0.52304E+00_JPRB, 0.54725E+00_JPRB, 0.56929E+00_JPRB ,&
&  0.60272E+00_JPRB, 0.62381E+00_JPRB, 0.65638E+00_JPRB, 0.71615E+00_JPRB, 0.78041E+00_JPRB, 0.83281E+00_JPRB, 0.85795E+00_JPRB ,&
&  0.87176E+00_JPRB, 0.86884E+00_JPRB, 0.85676E+00_JPRB, 0.84141E+00_JPRB, 0.85514E+00_JPRB, 0.86041E+00_JPRB, 0.89019E+00_JPRB ,&
&  0.93449E+00_JPRB, 0.98128E+00_JPRB, 0.10386E+01_JPRB, 0.10918E+01_JPRB, 0.11464E+01_JPRB, 0.11820E+01_JPRB, 0.11691E+01_JPRB ,&
&  0.56517E+00_JPRB, 0.79773E+00_JPRB, 0.84854E+00_JPRB, 0.84410E+00_JPRB, 0.79967E+00_JPRB, 0.78675E+00_JPRB, 0.87277E+00_JPRB ,&
&  0.99957E+00_JPRB, 0.10727E+01_JPRB, 0.12315E+01_JPRB, 0.13079E+01_JPRB, 0.16101E+01_JPRB, 0.17204E+01_JPRB, 0.19244E+01_JPRB ,&
&  0.20734E+01_JPRB, 0.21850E+01_JPRB, 0.23342E+01_JPRB, 0.24939E+01_JPRB, 0.26724E+01_JPRB, 0.28695E+01_JPRB, 0.31127E+01_JPRB ,&
&  0.33454E+01_JPRB, 0.36381E+01_JPRB, 0.40228E+01_JPRB, 0.40284E+01_JPRB, 0.37436E+01_JPRB, 0.40270E+01_JPRB, 0.38169E+01_JPRB ,&
&  0.31954E+01_JPRB, 0.22540E+01_JPRB, 0.21294E+01_JPRB, 0.18792E+01_JPRB, 0.16603E+01_JPRB, 0.15611E+01_JPRB, 0.13963E+01_JPRB ,&
&  0.11763E+01_JPRB, 0.10498E+01_JPRB, 0.95048E+00_JPRB, 0.88811E+00_JPRB, 0.81177E+00_JPRB, 0.75535E+00_JPRB, 0.71356E+00_JPRB ,&
&  0.68434E+00_JPRB, 0.69734E+00_JPRB, 0.71321E+00_JPRB, 0.69015E+00_JPRB, 0.66331E+00_JPRB, 0.69708E+00_JPRB, 0.78665E+00_JPRB ,&
&  0.90531E+00_JPRB, 0.80655E+00_JPRB, 0.72788E+00_JPRB, 0.72418E+00_JPRB, 0.69952E+00_JPRB, 0.67319E+00_JPRB, 0.65879E+00_JPRB ,&
&  0.66281E+00_JPRB, 0.64592E+00_JPRB, 0.73847E+00_JPRB, 0.11289E+01_JPRB, 0.14991E+01_JPRB, 0.17396E+01_JPRB, 0.14377E+01_JPRB ,&
&  0.10229E+01_JPRB, 0.75980E+00_JPRB, 0.60751E+00_JPRB, 0.56606E+00_JPRB, 0.54334E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.54625E+00_JPRB, 0.53517E+00_JPRB, 0.53521E+00_JPRB, 0.53194E+00_JPRB, 0.56513E+00_JPRB, 0.60241E+00_JPRB, 0.61307E+00_JPRB ,&
&  0.60837E+00_JPRB, 0.62631E+00_JPRB, 0.64357E+00_JPRB, 0.67518E+00_JPRB, 0.74102E+00_JPRB, 0.77316E+00_JPRB, 0.81086E+00_JPRB ,&
&  0.84736E+00_JPRB, 0.84419E+00_JPRB, 0.81466E+00_JPRB, 0.80126E+00_JPRB, 0.82504E+00_JPRB, 0.84516E+00_JPRB, 0.88340E+00_JPRB ,&
&  0.93838E+00_JPRB, 0.10038E+01_JPRB, 0.10839E+01_JPRB, 0.11819E+01_JPRB, 0.12932E+01_JPRB, 0.13858E+01_JPRB, 0.13872E+01_JPRB ,&
&  0.60199E+00_JPRB, 0.58876E+00_JPRB, 0.73433E+00_JPRB, 0.74348E+00_JPRB, 0.70981E+00_JPRB, 0.75124E+00_JPRB, 0.84301E+00_JPRB ,&
&  0.93272E+00_JPRB, 0.95625E+00_JPRB, 0.11128E+01_JPRB, 0.15541E+01_JPRB, 0.17728E+01_JPRB, 0.19491E+01_JPRB, 0.21255E+01_JPRB ,&
&  0.22688E+01_JPRB, 0.24229E+01_JPRB, 0.25606E+01_JPRB, 0.26834E+01_JPRB, 0.28381E+01_JPRB, 0.30380E+01_JPRB, 0.33058E+01_JPRB ,&
&  0.36493E+01_JPRB, 0.41532E+01_JPRB, 0.46461E+01_JPRB, 0.43002E+01_JPRB, 0.44297E+01_JPRB, 0.47082E+01_JPRB, 0.53739E+01_JPRB ,&
&  0.47936E+01_JPRB, 0.32735E+01_JPRB, 0.24615E+01_JPRB, 0.20392E+01_JPRB, 0.15995E+01_JPRB, 0.14491E+01_JPRB, 0.12908E+01_JPRB ,&
&  0.11644E+01_JPRB, 0.10787E+01_JPRB, 0.10011E+01_JPRB, 0.92134E+00_JPRB, 0.82200E+00_JPRB, 0.78157E+00_JPRB, 0.75887E+00_JPRB ,&
&  0.78491E+00_JPRB, 0.80639E+00_JPRB, 0.80056E+00_JPRB, 0.79101E+00_JPRB, 0.75306E+00_JPRB, 0.76903E+00_JPRB, 0.81916E+00_JPRB ,&
&  0.85950E+00_JPRB, 0.88099E+00_JPRB, 0.89220E+00_JPRB, 0.90395E+00_JPRB, 0.91852E+00_JPRB, 0.92418E+00_JPRB, 0.89527E+00_JPRB ,&
&  0.85339E+00_JPRB, 0.81505E+00_JPRB, 0.81356E+00_JPRB, 0.80418E+00_JPRB, 0.95571E+00_JPRB, 0.12534E+01_JPRB, 0.12911E+01_JPRB ,&
&  0.11007E+01_JPRB, 0.84660E+00_JPRB, 0.68501E+00_JPRB, 0.63429E+00_JPRB, 0.60942E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.65525E+00_JPRB, 0.62023E+00_JPRB, 0.61085E+00_JPRB, 0.59412E+00_JPRB, 0.60781E+00_JPRB, 0.63436E+00_JPRB, 0.65405E+00_JPRB ,&
&  0.65883E+00_JPRB, 0.65756E+00_JPRB, 0.65813E+00_JPRB, 0.67742E+00_JPRB, 0.70477E+00_JPRB, 0.73467E+00_JPRB, 0.77184E+00_JPRB ,&
&  0.79626E+00_JPRB, 0.80743E+00_JPRB, 0.78033E+00_JPRB, 0.74375E+00_JPRB, 0.75330E+00_JPRB, 0.77610E+00_JPRB, 0.81218E+00_JPRB ,&
&  0.86176E+00_JPRB, 0.91841E+00_JPRB, 0.99901E+00_JPRB, 0.11374E+01_JPRB, 0.13487E+01_JPRB, 0.16031E+01_JPRB, 0.18648E+01_JPRB ,&
&  0.14683E+01_JPRB, 0.70218E+00_JPRB, 0.72683E+00_JPRB, 0.80666E+00_JPRB, 0.74313E+00_JPRB, 0.78683E+00_JPRB, 0.82238E+00_JPRB ,&
&  0.85445E+00_JPRB, 0.93264E+00_JPRB, 0.10039E+01_JPRB, 0.12606E+01_JPRB, 0.16509E+01_JPRB, 0.18805E+01_JPRB, 0.20775E+01_JPRB ,&
&  0.22338E+01_JPRB, 0.23459E+01_JPRB, 0.24167E+01_JPRB, 0.24892E+01_JPRB, 0.26206E+01_JPRB, 0.28208E+01_JPRB, 0.30890E+01_JPRB ,&
&  0.34574E+01_JPRB, 0.39300E+01_JPRB, 0.44987E+01_JPRB, 0.41488E+01_JPRB, 0.47009E+01_JPRB, 0.47081E+01_JPRB, 0.47727E+01_JPRB ,&
&  0.43650E+01_JPRB, 0.29151E+01_JPRB, 0.21728E+01_JPRB, 0.19678E+01_JPRB, 0.15123E+01_JPRB, 0.11170E+01_JPRB, 0.11624E+01_JPRB ,&
&  0.11166E+01_JPRB, 0.10766E+01_JPRB, 0.10507E+01_JPRB, 0.95015E+00_JPRB, 0.86008E+00_JPRB, 0.82736E+00_JPRB, 0.87345E+00_JPRB ,&
&  0.91725E+00_JPRB, 0.94682E+00_JPRB, 0.92729E+00_JPRB, 0.91969E+00_JPRB, 0.88955E+00_JPRB, 0.90732E+00_JPRB, 0.94430E+00_JPRB ,&
&  0.97775E+00_JPRB, 0.10114E+01_JPRB, 0.10297E+01_JPRB, 0.10589E+01_JPRB, 0.10962E+01_JPRB, 0.10417E+01_JPRB, 0.97029E+00_JPRB ,&
&  0.95751E+00_JPRB, 0.94764E+00_JPRB, 0.89896E+00_JPRB, 0.87526E+00_JPRB, 0.85268E+00_JPRB, 0.88394E+00_JPRB, 0.98599E+00_JPRB ,&
&  0.96837E+00_JPRB, 0.85411E+00_JPRB, 0.73914E+00_JPRB, 0.70811E+00_JPRB, 0.71276E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.74768E+00_JPRB, 0.67227E+00_JPRB, 0.62803E+00_JPRB, 0.62325E+00_JPRB, 0.61045E+00_JPRB, 0.61673E+00_JPRB, 0.62627E+00_JPRB ,&
&  0.62428E+00_JPRB, 0.64378E+00_JPRB, 0.63799E+00_JPRB, 0.65194E+00_JPRB, 0.67224E+00_JPRB, 0.68013E+00_JPRB, 0.70562E+00_JPRB ,&
&  0.73087E+00_JPRB, 0.75785E+00_JPRB, 0.77947E+00_JPRB, 0.74311E+00_JPRB, 0.73545E+00_JPRB, 0.73830E+00_JPRB, 0.75555E+00_JPRB ,&
&  0.79238E+00_JPRB, 0.84332E+00_JPRB, 0.90199E+00_JPRB, 0.97683E+00_JPRB, 0.10884E+01_JPRB, 0.12919E+01_JPRB, 0.16759E+01_JPRB ,&
&  0.26345E+01_JPRB, 0.21369E+01_JPRB, 0.72579E+00_JPRB, 0.91095E+00_JPRB, 0.89812E+00_JPRB, 0.89674E+00_JPRB, 0.79204E+00_JPRB ,&
&  0.83889E+00_JPRB, 0.93315E+00_JPRB, 0.94540E+00_JPRB, 0.10203E+01_JPRB, 0.12400E+01_JPRB, 0.15522E+01_JPRB, 0.17987E+01_JPRB ,&
&  0.19615E+01_JPRB, 0.20791E+01_JPRB, 0.21286E+01_JPRB, 0.21775E+01_JPRB, 0.22499E+01_JPRB, 0.23896E+01_JPRB, 0.26121E+01_JPRB ,&
&  0.29408E+01_JPRB, 0.34114E+01_JPRB, 0.40371E+01_JPRB, 0.40101E+01_JPRB, 0.49523E+01_JPRB, 0.47454E+01_JPRB, 0.44079E+01_JPRB ,&
&  0.36032E+01_JPRB, 0.28622E+01_JPRB, 0.23751E+01_JPRB, 0.18860E+01_JPRB, 0.13414E+01_JPRB, 0.10728E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.10697E+01_JPRB, 0.96051E+00_JPRB, 0.93688E+00_JPRB, 0.92120E+00_JPRB, 0.87028E+00_JPRB, 0.92505E+00_JPRB, 0.10116E+01_JPRB ,&
&  0.10466E+01_JPRB, 0.10458E+01_JPRB, 0.10450E+01_JPRB, 0.10280E+01_JPRB, 0.10143E+01_JPRB, 0.10192E+01_JPRB, 0.10494E+01_JPRB ,&
&  0.10650E+01_JPRB, 0.10823E+01_JPRB, 0.10840E+01_JPRB, 0.11107E+01_JPRB, 0.11374E+01_JPRB, 0.10736E+01_JPRB, 0.99148E+00_JPRB ,&
&  0.97906E+00_JPRB, 0.99427E+00_JPRB, 0.97076E+00_JPRB, 0.95526E+00_JPRB, 0.91084E+00_JPRB, 0.87343E+00_JPRB, 0.86697E+00_JPRB ,&
&  0.94641E+00_JPRB, 0.88300E+00_JPRB, 0.82611E+00_JPRB, 0.75786E+00_JPRB, 0.77742E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.78549E+00_JPRB, 0.73978E+00_JPRB, 0.66265E+00_JPRB, 0.63428E+00_JPRB, 0.62931E+00_JPRB, 0.61826E+00_JPRB, 0.60232E+00_JPRB ,&
&  0.58038E+00_JPRB, 0.57319E+00_JPRB, 0.59949E+00_JPRB, 0.63093E+00_JPRB, 0.65828E+00_JPRB, 0.66237E+00_JPRB, 0.67264E+00_JPRB ,&
&  0.67933E+00_JPRB, 0.70170E+00_JPRB, 0.74399E+00_JPRB, 0.75355E+00_JPRB, 0.74054E+00_JPRB, 0.73328E+00_JPRB, 0.73495E+00_JPRB ,&
&  0.74493E+00_JPRB, 0.76064E+00_JPRB, 0.79574E+00_JPRB, 0.83638E+00_JPRB, 0.90185E+00_JPRB, 0.10146E+01_JPRB, 0.12597E+01_JPRB ,&
&  0.18029E+01_JPRB, 0.31208E+01_JPRB, 0.86524E+00_JPRB, 0.11237E+01_JPRB, 0.10789E+01_JPRB, 0.98541E+00_JPRB, 0.91479E+00_JPRB ,&
&  0.89931E+00_JPRB, 0.91496E+00_JPRB, 0.97244E+00_JPRB, 0.97395E+00_JPRB, 0.90504E+00_JPRB, 0.10851E+01_JPRB, 0.13548E+01_JPRB ,&
&  0.16031E+01_JPRB, 0.17249E+01_JPRB, 0.17720E+01_JPRB, 0.17650E+01_JPRB, 0.17915E+01_JPRB, 0.18488E+01_JPRB, 0.19766E+01_JPRB ,&
&  0.22076E+01_JPRB, 0.25827E+01_JPRB, 0.31395E+01_JPRB, 0.33904E+01_JPRB, 0.45345E+01_JPRB, 0.48236E+01_JPRB, 0.46017E+01_JPRB ,&
&  0.36851E+01_JPRB, 0.32791E+01_JPRB, 0.28575E+01_JPRB, 0.20703E+01_JPRB, 0.14672E+01_JPRB, 0.11654E+01_JPRB, 0.11156E+01_JPRB ,&
&  0.10741E+01_JPRB, 0.97360E+00_JPRB, 0.95927E+00_JPRB, 0.93644E+00_JPRB, 0.94530E+00_JPRB, 0.10041E+01_JPRB, 0.10706E+01_JPRB ,&
&  0.10831E+01_JPRB, 0.10914E+01_JPRB, 0.10783E+01_JPRB, 0.10644E+01_JPRB, 0.10646E+01_JPRB, 0.10704E+01_JPRB, 0.11027E+01_JPRB ,&
&  0.11148E+01_JPRB, 0.11226E+01_JPRB, 0.11176E+01_JPRB, 0.10870E+01_JPRB, 0.10902E+01_JPRB, 0.10514E+01_JPRB, 0.99879E+00_JPRB ,&
&  0.10026E+01_JPRB, 0.10219E+01_JPRB, 0.10101E+01_JPRB, 0.97078E+00_JPRB, 0.99314E+00_JPRB, 0.96200E+00_JPRB, 0.92757E+00_JPRB ,&
&  0.96015E+00_JPRB, 0.97201E+00_JPRB, 0.94771E+00_JPRB, 0.82205E+00_JPRB, 0.82917E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.81621E+00_JPRB, 0.77408E+00_JPRB, 0.72239E+00_JPRB, 0.66272E+00_JPRB, 0.66483E+00_JPRB, 0.67371E+00_JPRB, 0.64680E+00_JPRB ,&
&  0.59845E+00_JPRB, 0.59407E+00_JPRB, 0.60373E+00_JPRB, 0.62693E+00_JPRB, 0.64637E+00_JPRB, 0.65104E+00_JPRB, 0.67552E+00_JPRB ,&
&  0.66829E+00_JPRB, 0.66386E+00_JPRB, 0.68605E+00_JPRB, 0.72036E+00_JPRB, 0.72370E+00_JPRB, 0.72199E+00_JPRB, 0.73517E+00_JPRB ,&
&  0.74127E+00_JPRB, 0.72998E+00_JPRB, 0.72689E+00_JPRB, 0.74016E+00_JPRB, 0.78347E+00_JPRB, 0.86009E+00_JPRB, 0.10006E+01_JPRB ,&
&  0.15193E+01_JPRB, 0.34630E+01_JPRB, 0.24604E+01_JPRB, 0.19781E+01_JPRB, 0.14117E+01_JPRB, 0.11255E+01_JPRB, 0.10121E+01_JPRB ,&
&  0.10700E+01_JPRB, 0.10496E+01_JPRB, 0.11110E+01_JPRB, 0.96597E+00_JPRB, 0.85541E+00_JPRB, 0.83598E+00_JPRB, 0.97937E+00_JPRB ,&
&  0.11387E+01_JPRB, 0.12737E+01_JPRB, 0.13732E+01_JPRB, 0.13846E+01_JPRB, 0.13983E+01_JPRB, 0.14391E+01_JPRB, 0.14964E+01_JPRB ,&
&  0.16219E+01_JPRB, 0.18713E+01_JPRB, 0.23007E+01_JPRB, 0.28603E+01_JPRB, 0.35393E+01_JPRB, 0.42357E+01_JPRB, 0.49246E+01_JPRB ,&
&  0.46857E+01_JPRB, 0.40812E+01_JPRB, 0.33046E+01_JPRB, 0.22715E+01_JPRB, 0.15439E+01_JPRB, 0.14220E+01_JPRB, 0.11567E+01_JPRB ,&
&  0.10865E+01_JPRB, 0.10138E+01_JPRB, 0.10122E+01_JPRB, 0.10325E+01_JPRB, 0.10627E+01_JPRB, 0.11039E+01_JPRB, 0.11330E+01_JPRB ,&
&  0.11393E+01_JPRB, 0.11256E+01_JPRB, 0.11164E+01_JPRB, 0.10983E+01_JPRB, 0.10898E+01_JPRB, 0.10887E+01_JPRB, 0.10989E+01_JPRB ,&
&  0.11045E+01_JPRB, 0.11197E+01_JPRB, 0.10699E+01_JPRB, 0.10565E+01_JPRB, 0.10517E+01_JPRB, 0.10300E+01_JPRB, 0.97886E+00_JPRB ,&
&  0.99235E+00_JPRB, 0.10355E+01_JPRB, 0.10500E+01_JPRB, 0.10353E+01_JPRB, 0.10338E+01_JPRB, 0.10423E+01_JPRB, 0.10056E+01_JPRB ,&
&  0.10061E+01_JPRB, 0.98634E+00_JPRB, 0.92101E+00_JPRB, 0.87823E+00_JPRB, 0.87081E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.86155E+00_JPRB, 0.82763E+00_JPRB, 0.78587E+00_JPRB, 0.73560E+00_JPRB, 0.70851E+00_JPRB, 0.71273E+00_JPRB, 0.72795E+00_JPRB ,&
&  0.70327E+00_JPRB, 0.68074E+00_JPRB, 0.67280E+00_JPRB, 0.68762E+00_JPRB, 0.71082E+00_JPRB, 0.71733E+00_JPRB, 0.71947E+00_JPRB ,&
&  0.70670E+00_JPRB, 0.70008E+00_JPRB, 0.68640E+00_JPRB, 0.69202E+00_JPRB, 0.74053E+00_JPRB, 0.73861E+00_JPRB, 0.72753E+00_JPRB ,&
&  0.74536E+00_JPRB, 0.74089E+00_JPRB, 0.71999E+00_JPRB, 0.71837E+00_JPRB, 0.72989E+00_JPRB, 0.77702E+00_JPRB, 0.87893E+00_JPRB ,&
&  0.14166E+01_JPRB, 0.32152E+01_JPRB, 0.16956E+01_JPRB, 0.22331E+01_JPRB, 0.15483E+01_JPRB, 0.12531E+01_JPRB, 0.10148E+01_JPRB ,&
&  0.10333E+01_JPRB, 0.12325E+01_JPRB, 0.10820E+01_JPRB, 0.98073E+00_JPRB, 0.86786E+00_JPRB, 0.81856E+00_JPRB, 0.88087E+00_JPRB ,&
&  0.96448E+00_JPRB, 0.10031E+01_JPRB, 0.10553E+01_JPRB, 0.10914E+01_JPRB, 0.10793E+01_JPRB, 0.10847E+01_JPRB, 0.11275E+01_JPRB ,&
&  0.11879E+01_JPRB, 0.13223E+01_JPRB, 0.15999E+01_JPRB, 0.20255E+01_JPRB, 0.29945E+01_JPRB, 0.40599E+01_JPRB, 0.52276E+01_JPRB ,&
&  0.52782E+01_JPRB, 0.51712E+01_JPRB, 0.38427E+01_JPRB, 0.25261E+01_JPRB, 0.17421E+01_JPRB, 0.14823E+01_JPRB, 0.12609E+01_JPRB ,&
&  0.11509E+01_JPRB, 0.11067E+01_JPRB, 0.10703E+01_JPRB, 0.10816E+01_JPRB, 0.11587E+01_JPRB, 0.12061E+01_JPRB, 0.11884E+01_JPRB ,&
&  0.11800E+01_JPRB, 0.11524E+01_JPRB, 0.11364E+01_JPRB, 0.11105E+01_JPRB, 0.11091E+01_JPRB, 0.11029E+01_JPRB, 0.10900E+01_JPRB ,&
&  0.10829E+01_JPRB, 0.10912E+01_JPRB, 0.10546E+01_JPRB, 0.10545E+01_JPRB, 0.10519E+01_JPRB, 0.10364E+01_JPRB, 0.98953E+00_JPRB ,&
&  0.10083E+01_JPRB, 0.10317E+01_JPRB, 0.10383E+01_JPRB, 0.10226E+01_JPRB, 0.10251E+01_JPRB, 0.10747E+01_JPRB, 0.10454E+01_JPRB ,&
&  0.10401E+01_JPRB, 0.10145E+01_JPRB, 0.95315E+00_JPRB, 0.92441E+00_JPRB, 0.91251E+00_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.90893E+00_JPRB, 0.86764E+00_JPRB, 0.83597E+00_JPRB, 0.79694E+00_JPRB, 0.76775E+00_JPRB, 0.75113E+00_JPRB, 0.75598E+00_JPRB ,&
&  0.74967E+00_JPRB, 0.72428E+00_JPRB, 0.70780E+00_JPRB, 0.71031E+00_JPRB, 0.71030E+00_JPRB, 0.71730E+00_JPRB, 0.72283E+00_JPRB ,&
&  0.71740E+00_JPRB, 0.70533E+00_JPRB, 0.70038E+00_JPRB, 0.70307E+00_JPRB, 0.72917E+00_JPRB, 0.76618E+00_JPRB, 0.77377E+00_JPRB ,&
&  0.76312E+00_JPRB, 0.75551E+00_JPRB, 0.73514E+00_JPRB, 0.71640E+00_JPRB, 0.71457E+00_JPRB, 0.73876E+00_JPRB, 0.81812E+00_JPRB ,&
&  0.13249E+01_JPRB, 0.33698E+01_JPRB, 0.15579E+01_JPRB, 0.23182E+01_JPRB, 0.16338E+01_JPRB, 0.13190E+01_JPRB, 0.10663E+01_JPRB ,&
&  0.10746E+01_JPRB, 0.11885E+01_JPRB, 0.11034E+01_JPRB, 0.10150E+01_JPRB, 0.92185E+00_JPRB, 0.86775E+00_JPRB, 0.85715E+00_JPRB ,&
&  0.90983E+00_JPRB, 0.92314E+00_JPRB, 0.92302E+00_JPRB, 0.93711E+00_JPRB, 0.92075E+00_JPRB, 0.91162E+00_JPRB, 0.92463E+00_JPRB ,&
&  0.93024E+00_JPRB, 0.95938E+00_JPRB, 0.10965E+01_JPRB, 0.16234E+01_JPRB, 0.23756E+01_JPRB, 0.38435E+01_JPRB, 0.60055E+01_JPRB ,&
&  0.59030E+01_JPRB, 0.60828E+01_JPRB, 0.43108E+01_JPRB, 0.28286E+01_JPRB, 0.19016E+01_JPRB, 0.15438E+01_JPRB, 0.14001E+01_JPRB ,&
&  0.12755E+01_JPRB, 0.11885E+01_JPRB, 0.11367E+01_JPRB, 0.11377E+01_JPRB, 0.11880E+01_JPRB, 0.11998E+01_JPRB, 0.12118E+01_JPRB ,&
&  0.12050E+01_JPRB, 0.11461E+01_JPRB, 0.11178E+01_JPRB, 0.11024E+01_JPRB, 0.11067E+01_JPRB, 0.11020E+01_JPRB, 0.10870E+01_JPRB ,&
&  0.10746E+01_JPRB, 0.10674E+01_JPRB, 0.10462E+01_JPRB, 0.10582E+01_JPRB, 0.10685E+01_JPRB, 0.10572E+01_JPRB, 0.10080E+01_JPRB ,&
&  0.10155E+01_JPRB, 0.10275E+01_JPRB, 0.10197E+01_JPRB, 0.10300E+01_JPRB, 0.10334E+01_JPRB, 0.10902E+01_JPRB, 0.10467E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.10145E+01_JPRB, 0.96778E+00_JPRB, 0.94831E+00_JPRB, 0.94515E+00_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.94523E+00_JPRB, 0.91771E+00_JPRB, 0.89589E+00_JPRB, 0.86476E+00_JPRB, 0.82152E+00_JPRB, 0.79884E+00_JPRB, 0.78451E+00_JPRB ,&
&  0.77743E+00_JPRB, 0.75165E+00_JPRB, 0.73278E+00_JPRB, 0.73238E+00_JPRB, 0.73793E+00_JPRB, 0.73136E+00_JPRB, 0.72998E+00_JPRB ,&
&  0.73645E+00_JPRB, 0.73918E+00_JPRB, 0.72962E+00_JPRB, 0.71937E+00_JPRB, 0.71850E+00_JPRB, 0.75144E+00_JPRB, 0.76616E+00_JPRB ,&
&  0.77173E+00_JPRB, 0.76115E+00_JPRB, 0.73826E+00_JPRB, 0.70952E+00_JPRB, 0.70235E+00_JPRB, 0.71369E+00_JPRB, 0.77160E+00_JPRB ,&
&  0.11569E+01_JPRB, 0.14162E+01_JPRB, 0.23701E+01_JPRB, 0.23702E+01_JPRB, 0.17015E+01_JPRB, 0.13887E+01_JPRB, 0.12210E+01_JPRB ,&
&  0.12264E+01_JPRB, 0.11863E+01_JPRB, 0.11058E+01_JPRB, 0.10346E+01_JPRB, 0.96896E+00_JPRB, 0.91464E+00_JPRB, 0.87783E+00_JPRB ,&
&  0.87199E+00_JPRB, 0.87345E+00_JPRB, 0.87335E+00_JPRB, 0.87631E+00_JPRB, 0.87290E+00_JPRB, 0.87242E+00_JPRB, 0.87267E+00_JPRB ,&
&  0.87274E+00_JPRB, 0.87068E+00_JPRB, 0.90006E+00_JPRB, 0.11242E+01_JPRB, 0.20163E+01_JPRB, 0.33801E+01_JPRB, 0.53577E+01_JPRB ,&
&  0.62873E+01_JPRB, 0.58415E+01_JPRB, 0.40724E+01_JPRB, 0.28941E+01_JPRB, 0.20107E+01_JPRB, 0.15510E+01_JPRB, 0.14143E+01_JPRB ,&
&  0.13545E+01_JPRB, 0.12986E+01_JPRB, 0.12150E+01_JPRB, 0.11930E+01_JPRB, 0.11914E+01_JPRB, 0.11721E+01_JPRB, 0.11697E+01_JPRB ,&
&  0.11574E+01_JPRB, 0.11481E+01_JPRB, 0.11300E+01_JPRB, 0.11103E+01_JPRB, 0.10886E+01_JPRB, 0.10674E+01_JPRB, 0.10544E+01_JPRB ,&
&  0.10453E+01_JPRB, 0.10301E+01_JPRB, 0.10219E+01_JPRB, 0.10497E+01_JPRB, 0.11033E+01_JPRB, 0.11094E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.10391E+01_JPRB, 0.10171E+01_JPRB, 0.10081E+01_JPRB, 0.10298E+01_JPRB, 0.10152E+01_JPRB, 0.11031E+01_JPRB, 0.10665E+01_JPRB ,&
&  0.10488E+01_JPRB, 0.10298E+01_JPRB, 0.99239E+00_JPRB, 0.96175E+00_JPRB, 0.94968E+00_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.95520E+00_JPRB, 0.92178E+00_JPRB, 0.90043E+00_JPRB, 0.88599E+00_JPRB, 0.85884E+00_JPRB, 0.83004E+00_JPRB, 0.82301E+00_JPRB ,&
&  0.82045E+00_JPRB, 0.80180E+00_JPRB, 0.77922E+00_JPRB, 0.75027E+00_JPRB, 0.74301E+00_JPRB, 0.74774E+00_JPRB, 0.75332E+00_JPRB ,&
&  0.75292E+00_JPRB, 0.75386E+00_JPRB, 0.75251E+00_JPRB, 0.75003E+00_JPRB, 0.74746E+00_JPRB, 0.76814E+00_JPRB, 0.77628E+00_JPRB ,&
&  0.76781E+00_JPRB, 0.76807E+00_JPRB, 0.74622E+00_JPRB, 0.72344E+00_JPRB, 0.71283E+00_JPRB, 0.70119E+00_JPRB, 0.70330E+00_JPRB ,&
&  0.95291E+00_JPRB, 0.15476E+01_JPRB, 0.22276E+01_JPRB, 0.20844E+01_JPRB, 0.16750E+01_JPRB, 0.13663E+01_JPRB, 0.12860E+01_JPRB ,&
&  0.12382E+01_JPRB, 0.11842E+01_JPRB, 0.11077E+01_JPRB, 0.10450E+01_JPRB, 0.98982E+00_JPRB, 0.94931E+00_JPRB, 0.90951E+00_JPRB ,&
&  0.88902E+00_JPRB, 0.88388E+00_JPRB, 0.86576E+00_JPRB, 0.85575E+00_JPRB, 0.85357E+00_JPRB, 0.85943E+00_JPRB, 0.86550E+00_JPRB ,&
&  0.87030E+00_JPRB, 0.87029E+00_JPRB, 0.87635E+00_JPRB, 0.95516E+00_JPRB, 0.14803E+01_JPRB, 0.18639E+01_JPRB, 0.35112E+01_JPRB ,&
&  0.50209E+01_JPRB, 0.40617E+01_JPRB, 0.30798E+01_JPRB, 0.25039E+01_JPRB, 0.20079E+01_JPRB, 0.15792E+01_JPRB, 0.13793E+01_JPRB ,&
&  0.13043E+01_JPRB, 0.12625E+01_JPRB, 0.12010E+01_JPRB, 0.11668E+01_JPRB, 0.11393E+01_JPRB, 0.11225E+01_JPRB, 0.11117E+01_JPRB ,&
&  0.10887E+01_JPRB, 0.10861E+01_JPRB, 0.10816E+01_JPRB, 0.10769E+01_JPRB, 0.10571E+01_JPRB, 0.10417E+01_JPRB, 0.10279E+01_JPRB ,&
&  0.10174E+01_JPRB, 0.99796E+00_JPRB, 0.99009E+00_JPRB, 0.10167E+01_JPRB, 0.10972E+01_JPRB, 0.11540E+01_JPRB, 0.11445E+01_JPRB ,&
&  0.10848E+01_JPRB, 0.10349E+01_JPRB, 0.10336E+01_JPRB, 0.10285E+01_JPRB, 0.10352E+01_JPRB, 0.11449E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.10648E+01_JPRB, 0.10507E+01_JPRB, 0.10245E+01_JPRB, 0.99092E+00_JPRB, 0.97648E+00_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.99306E+00_JPRB, 0.95517E+00_JPRB, 0.92446E+00_JPRB, 0.89393E+00_JPRB, 0.88425E+00_JPRB, 0.86541E+00_JPRB, 0.84162E+00_JPRB ,&
&  0.82884E+00_JPRB, 0.82838E+00_JPRB, 0.82187E+00_JPRB, 0.80546E+00_JPRB, 0.79082E+00_JPRB, 0.78520E+00_JPRB, 0.78510E+00_JPRB ,&
&  0.78373E+00_JPRB, 0.78239E+00_JPRB, 0.77957E+00_JPRB, 0.77142E+00_JPRB, 0.77410E+00_JPRB, 0.78281E+00_JPRB, 0.79125E+00_JPRB ,&
&  0.77312E+00_JPRB, 0.75839E+00_JPRB, 0.74608E+00_JPRB, 0.73367E+00_JPRB, 0.73007E+00_JPRB, 0.71234E+00_JPRB, 0.69094E+00_JPRB ,&
&  0.79013E+00_JPRB, 0.11191E+01_JPRB, 0.15416E+01_JPRB, 0.18848E+01_JPRB, 0.16683E+01_JPRB, 0.14321E+01_JPRB, 0.13278E+01_JPRB ,&
&  0.12527E+01_JPRB, 0.11606E+01_JPRB, 0.10898E+01_JPRB, 0.10392E+01_JPRB, 0.10129E+01_JPRB, 0.97945E+00_JPRB, 0.92413E+00_JPRB ,&
&  0.88479E+00_JPRB, 0.86362E+00_JPRB, 0.85754E+00_JPRB, 0.85819E+00_JPRB, 0.85337E+00_JPRB, 0.86732E+00_JPRB, 0.87916E+00_JPRB ,&
&  0.88602E+00_JPRB, 0.88110E+00_JPRB, 0.87120E+00_JPRB, 0.87935E+00_JPRB, 0.10798E+01_JPRB, 0.12620E+01_JPRB, 0.18206E+01_JPRB ,&
&  0.21085E+01_JPRB, 0.22474E+01_JPRB, 0.21482E+01_JPRB, 0.20480E+01_JPRB, 0.18767E+01_JPRB, 0.15840E+01_JPRB, 0.13574E+01_JPRB ,&
&  0.12342E+01_JPRB, 0.11850E+01_JPRB, 0.11381E+01_JPRB, 0.11171E+01_JPRB, 0.10957E+01_JPRB, 0.10741E+01_JPRB, 0.10639E+01_JPRB ,&
&  0.10529E+01_JPRB, 0.10419E+01_JPRB, 0.10323E+01_JPRB, 0.10375E+01_JPRB, 0.10254E+01_JPRB, 0.99849E+00_JPRB, 0.97757E+00_JPRB ,&
&  0.96924E+00_JPRB, 0.95682E+00_JPRB, 0.96816E+00_JPRB, 0.10063E+01_JPRB, 0.10444E+01_JPRB, 0.10691E+01_JPRB, 0.10936E+01_JPRB ,&
&  0.11115E+01_JPRB, 0.10961E+01_JPRB, 0.10686E+01_JPRB, 0.10667E+01_JPRB, 0.10687E+01_JPRB, 0.11580E+01_JPRB, 0.11154E+01_JPRB ,&
&  0.10878E+01_JPRB, 0.10683E+01_JPRB, 0.10459E+01_JPRB, 0.99992E+00_JPRB, 0.10235E+01_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.99855E+00_JPRB, 0.98255E+00_JPRB, 0.96113E+00_JPRB, 0.92623E+00_JPRB, 0.91074E+00_JPRB, 0.89525E+00_JPRB, 0.87803E+00_JPRB ,&
&  0.86945E+00_JPRB, 0.86153E+00_JPRB, 0.85575E+00_JPRB, 0.84210E+00_JPRB, 0.82816E+00_JPRB, 0.81964E+00_JPRB, 0.80883E+00_JPRB ,&
&  0.80929E+00_JPRB, 0.80945E+00_JPRB, 0.80127E+00_JPRB, 0.79274E+00_JPRB, 0.78904E+00_JPRB, 0.78616E+00_JPRB, 0.78993E+00_JPRB ,&
&  0.78087E+00_JPRB, 0.76976E+00_JPRB, 0.76849E+00_JPRB, 0.76025E+00_JPRB, 0.74711E+00_JPRB, 0.73621E+00_JPRB, 0.70622E+00_JPRB ,&
&  0.71287E+00_JPRB, 0.78413E+00_JPRB, 0.10919E+01_JPRB, 0.14399E+01_JPRB, 0.14293E+01_JPRB, 0.13959E+01_JPRB, 0.13051E+01_JPRB ,&
&  0.12226E+01_JPRB, 0.11425E+01_JPRB, 0.10534E+01_JPRB, 0.99541E+00_JPRB, 0.98005E+00_JPRB, 0.96079E+00_JPRB, 0.93575E+00_JPRB ,&
&  0.90593E+00_JPRB, 0.87923E+00_JPRB, 0.86319E+00_JPRB, 0.86920E+00_JPRB, 0.86585E+00_JPRB, 0.87012E+00_JPRB, 0.87983E+00_JPRB ,&
&  0.88159E+00_JPRB, 0.87926E+00_JPRB, 0.86341E+00_JPRB, 0.85218E+00_JPRB, 0.84920E+00_JPRB, 0.87329E+00_JPRB, 0.10201E+01_JPRB ,&
&  0.12340E+01_JPRB, 0.13419E+01_JPRB, 0.14014E+01_JPRB, 0.14674E+01_JPRB, 0.15025E+01_JPRB, 0.14601E+01_JPRB, 0.13383E+01_JPRB ,&
&  0.12371E+01_JPRB, 0.11742E+01_JPRB, 0.11115E+01_JPRB, 0.11091E+01_JPRB, 0.10890E+01_JPRB, 0.10614E+01_JPRB, 0.10344E+01_JPRB ,&
&  0.10059E+01_JPRB, 0.99845E+00_JPRB, 0.98067E+00_JPRB, 0.96981E+00_JPRB, 0.95891E+00_JPRB, 0.94177E+00_JPRB, 0.92266E+00_JPRB ,&
&  0.91629E+00_JPRB, 0.91029E+00_JPRB, 0.91944E+00_JPRB, 0.94931E+00_JPRB, 0.98680E+00_JPRB, 0.10120E+01_JPRB, 0.10384E+01_JPRB ,&
&  0.10602E+01_JPRB, 0.10851E+01_JPRB, 0.10533E+01_JPRB, 0.10433E+01_JPRB, 0.11452E+01_JPRB, 0.11429E+01_JPRB, 0.11038E+01_JPRB ,&
&  0.10788E+01_JPRB, 0.10547E+01_JPRB, 0.10391E+01_JPRB, 0.98433E+00_JPRB, 0.97584E+00_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.89234E+00_JPRB, 0.91921E+00_JPRB, 0.91410E+00_JPRB, 0.88896E+00_JPRB, 0.87505E+00_JPRB, 0.86993E+00_JPRB, 0.86937E+00_JPRB ,&
&  0.86625E+00_JPRB, 0.85661E+00_JPRB, 0.86190E+00_JPRB, 0.86865E+00_JPRB, 0.86635E+00_JPRB, 0.85465E+00_JPRB, 0.84601E+00_JPRB ,&
&  0.84342E+00_JPRB, 0.84675E+00_JPRB, 0.85129E+00_JPRB, 0.84085E+00_JPRB, 0.81952E+00_JPRB, 0.80532E+00_JPRB, 0.79279E+00_JPRB ,&
&  0.79088E+00_JPRB, 0.79034E+00_JPRB, 0.78774E+00_JPRB, 0.78882E+00_JPRB, 0.76772E+00_JPRB, 0.74419E+00_JPRB, 0.72498E+00_JPRB ,&
&  0.71048E+00_JPRB, 0.64478E+00_JPRB, 0.79472E+00_JPRB, 0.10324E+01_JPRB, 0.11514E+01_JPRB, 0.11547E+01_JPRB, 0.10943E+01_JPRB ,&
&  0.10553E+01_JPRB, 0.10168E+01_JPRB, 0.97558E+00_JPRB, 0.93323E+00_JPRB, 0.90959E+00_JPRB, 0.89593E+00_JPRB, 0.88562E+00_JPRB ,&
&  0.87548E+00_JPRB, 0.86396E+00_JPRB, 0.85667E+00_JPRB, 0.83997E+00_JPRB, 0.82625E+00_JPRB, 0.82458E+00_JPRB, 0.82733E+00_JPRB ,&
&  0.83393E+00_JPRB, 0.83186E+00_JPRB, 0.83255E+00_JPRB, 0.83270E+00_JPRB, 0.81335E+00_JPRB, 0.81262E+00_JPRB, 0.83261E+00_JPRB ,&
&  0.89901E+00_JPRB, 0.95531E+00_JPRB, 0.99185E+00_JPRB, 0.10743E+01_JPRB, 0.11455E+01_JPRB, 0.12047E+01_JPRB, 0.12015E+01_JPRB ,&
&  0.11654E+01_JPRB, 0.11342E+01_JPRB, 0.11000E+01_JPRB, 0.10843E+01_JPRB, 0.10640E+01_JPRB, 0.10249E+01_JPRB, 0.98479E+00_JPRB ,&
&  0.96432E+00_JPRB, 0.95043E+00_JPRB, 0.92824E+00_JPRB, 0.91227E+00_JPRB, 0.89529E+00_JPRB, 0.87186E+00_JPRB, 0.85197E+00_JPRB ,&
&  0.83832E+00_JPRB, 0.83558E+00_JPRB, 0.84010E+00_JPRB, 0.85873E+00_JPRB, 0.88708E+00_JPRB, 0.92054E+00_JPRB, 0.95415E+00_JPRB ,&
&  0.98501E+00_JPRB, 0.10076E+01_JPRB, 0.10362E+01_JPRB, 0.10834E+01_JPRB, 0.10827E+01_JPRB, 0.10735E+01_JPRB, 0.10759E+01_JPRB ,&
&  0.10598E+01_JPRB, 0.10249E+01_JPRB, 0.98745E+00_JPRB, 0.92545E+00_JPRB, 0.84177E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.85549E+00_JPRB, 0.87211E+00_JPRB, 0.87139E+00_JPRB, 0.86220E+00_JPRB, 0.85490E+00_JPRB, 0.85106E+00_JPRB, 0.85554E+00_JPRB ,&
&  0.85483E+00_JPRB, 0.84970E+00_JPRB, 0.85301E+00_JPRB, 0.85553E+00_JPRB, 0.85754E+00_JPRB, 0.85488E+00_JPRB, 0.85956E+00_JPRB ,&
&  0.86017E+00_JPRB, 0.86170E+00_JPRB, 0.86479E+00_JPRB, 0.85840E+00_JPRB, 0.83843E+00_JPRB, 0.82418E+00_JPRB, 0.80998E+00_JPRB ,&
&  0.80862E+00_JPRB, 0.80796E+00_JPRB, 0.80822E+00_JPRB, 0.80174E+00_JPRB, 0.78168E+00_JPRB, 0.76438E+00_JPRB, 0.75347E+00_JPRB ,&
&  0.75304E+00_JPRB, 0.61213E+00_JPRB, 0.68137E+00_JPRB, 0.88283E+00_JPRB, 0.97685E+00_JPRB, 0.95263E+00_JPRB, 0.94380E+00_JPRB ,&
&  0.95084E+00_JPRB, 0.94845E+00_JPRB, 0.92878E+00_JPRB, 0.89703E+00_JPRB, 0.85479E+00_JPRB, 0.84183E+00_JPRB, 0.82260E+00_JPRB ,&
&  0.80418E+00_JPRB, 0.79698E+00_JPRB, 0.79078E+00_JPRB, 0.79037E+00_JPRB, 0.78070E+00_JPRB, 0.76950E+00_JPRB, 0.77083E+00_JPRB ,&
&  0.78001E+00_JPRB, 0.78202E+00_JPRB, 0.78888E+00_JPRB, 0.79548E+00_JPRB, 0.79124E+00_JPRB, 0.80682E+00_JPRB, 0.81716E+00_JPRB ,&
&  0.83454E+00_JPRB, 0.84999E+00_JPRB, 0.86266E+00_JPRB, 0.88638E+00_JPRB, 0.92840E+00_JPRB, 0.10029E+01_JPRB, 0.10486E+01_JPRB ,&
&  0.10466E+01_JPRB, 0.10307E+01_JPRB, 0.10280E+01_JPRB, 0.10225E+01_JPRB, 0.10076E+01_JPRB, 0.97916E+00_JPRB, 0.94806E+00_JPRB ,&
&  0.92700E+00_JPRB, 0.90675E+00_JPRB, 0.88791E+00_JPRB, 0.86662E+00_JPRB, 0.84449E+00_JPRB, 0.82427E+00_JPRB, 0.80794E+00_JPRB ,&
&  0.79726E+00_JPRB, 0.79295E+00_JPRB, 0.80104E+00_JPRB, 0.82102E+00_JPRB, 0.84739E+00_JPRB, 0.87750E+00_JPRB, 0.90883E+00_JPRB ,&
&  0.94433E+00_JPRB, 0.98145E+00_JPRB, 0.10183E+01_JPRB, 0.97592E+00_JPRB, 0.10361E+01_JPRB, 0.10459E+01_JPRB, 0.10492E+01_JPRB ,&
&  0.10364E+01_JPRB, 0.99535E+00_JPRB, 0.93626E+00_JPRB, 0.74101E+00_JPRB, 0.81863E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.84457E+00_JPRB, 0.85018E+00_JPRB, 0.85338E+00_JPRB, 0.84188E+00_JPRB, 0.82740E+00_JPRB, 0.82250E+00_JPRB, 0.83003E+00_JPRB ,&
&  0.82651E+00_JPRB, 0.81365E+00_JPRB, 0.81856E+00_JPRB, 0.82214E+00_JPRB, 0.82902E+00_JPRB, 0.83222E+00_JPRB, 0.83584E+00_JPRB ,&
&  0.83964E+00_JPRB, 0.84341E+00_JPRB, 0.84162E+00_JPRB, 0.84994E+00_JPRB, 0.84245E+00_JPRB, 0.83673E+00_JPRB, 0.83369E+00_JPRB ,&
&  0.82514E+00_JPRB, 0.81628E+00_JPRB, 0.80887E+00_JPRB, 0.79861E+00_JPRB, 0.79083E+00_JPRB, 0.78163E+00_JPRB, 0.76617E+00_JPRB ,&
&  0.77741E+00_JPRB, 0.62526E+00_JPRB, 0.70477E+00_JPRB, 0.84310E+00_JPRB, 0.84671E+00_JPRB, 0.84095E+00_JPRB, 0.85452E+00_JPRB ,&
&  0.87496E+00_JPRB, 0.88551E+00_JPRB, 0.87714E+00_JPRB, 0.85250E+00_JPRB, 0.80566E+00_JPRB, 0.78232E+00_JPRB, 0.76852E+00_JPRB ,&
&  0.75319E+00_JPRB, 0.74201E+00_JPRB, 0.73530E+00_JPRB, 0.73698E+00_JPRB, 0.73723E+00_JPRB, 0.73423E+00_JPRB, 0.73111E+00_JPRB ,&
&  0.73486E+00_JPRB, 0.74504E+00_JPRB, 0.75222E+00_JPRB, 0.75976E+00_JPRB, 0.76672E+00_JPRB, 0.76379E+00_JPRB, 0.76378E+00_JPRB ,&
&  0.76521E+00_JPRB, 0.77014E+00_JPRB, 0.77365E+00_JPRB, 0.78431E+00_JPRB, 0.80404E+00_JPRB, 0.85143E+00_JPRB, 0.89450E+00_JPRB ,&
&  0.91026E+00_JPRB, 0.91318E+00_JPRB, 0.91502E+00_JPRB, 0.91627E+00_JPRB, 0.91856E+00_JPRB, 0.91505E+00_JPRB, 0.90017E+00_JPRB ,&
&  0.88232E+00_JPRB, 0.86016E+00_JPRB, 0.84447E+00_JPRB, 0.83218E+00_JPRB, 0.81803E+00_JPRB, 0.80832E+00_JPRB, 0.80564E+00_JPRB ,&
&  0.79753E+00_JPRB, 0.78553E+00_JPRB, 0.78862E+00_JPRB, 0.80412E+00_JPRB, 0.81926E+00_JPRB, 0.83108E+00_JPRB, 0.85223E+00_JPRB ,&
&  0.88859E+00_JPRB, 0.92655E+00_JPRB, 0.95765E+00_JPRB, 0.98901E+00_JPRB, 0.10035E+01_JPRB, 0.10080E+01_JPRB, 0.10012E+01_JPRB ,&
&  0.98182E+00_JPRB, 0.95918E+00_JPRB, 0.70648E+00_JPRB, 0.80779E+00_JPRB, 0.83299E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.85257E+00_JPRB, 0.84297E+00_JPRB, 0.83120E+00_JPRB, 0.81468E+00_JPRB, 0.79862E+00_JPRB, 0.78979E+00_JPRB, 0.79598E+00_JPRB ,&
&  0.79512E+00_JPRB, 0.78347E+00_JPRB, 0.79347E+00_JPRB, 0.80654E+00_JPRB, 0.81703E+00_JPRB, 0.82663E+00_JPRB, 0.82987E+00_JPRB ,&
&  0.83360E+00_JPRB, 0.83082E+00_JPRB, 0.83424E+00_JPRB, 0.84652E+00_JPRB, 0.86512E+00_JPRB, 0.86669E+00_JPRB, 0.86358E+00_JPRB ,&
&  0.84110E+00_JPRB, 0.82980E+00_JPRB, 0.81948E+00_JPRB, 0.80696E+00_JPRB, 0.79562E+00_JPRB, 0.79157E+00_JPRB, 0.79402E+00_JPRB ,&
&  0.68007E+00_JPRB, 0.58467E+00_JPRB, 0.69560E+00_JPRB, 0.78816E+00_JPRB, 0.79086E+00_JPRB, 0.79100E+00_JPRB, 0.80821E+00_JPRB ,&
&  0.81420E+00_JPRB, 0.81499E+00_JPRB, 0.81272E+00_JPRB, 0.80097E+00_JPRB, 0.76955E+00_JPRB, 0.73554E+00_JPRB, 0.71716E+00_JPRB ,&
&  0.70612E+00_JPRB, 0.71088E+00_JPRB, 0.71281E+00_JPRB, 0.72016E+00_JPRB, 0.72043E+00_JPRB, 0.71353E+00_JPRB, 0.71213E+00_JPRB ,&
&  0.71300E+00_JPRB, 0.72310E+00_JPRB, 0.73274E+00_JPRB, 0.74004E+00_JPRB, 0.74305E+00_JPRB, 0.74398E+00_JPRB, 0.74116E+00_JPRB ,&
&  0.73956E+00_JPRB, 0.73908E+00_JPRB, 0.73776E+00_JPRB, 0.74296E+00_JPRB, 0.75446E+00_JPRB, 0.77683E+00_JPRB, 0.80962E+00_JPRB ,&
&  0.83032E+00_JPRB, 0.83795E+00_JPRB, 0.84194E+00_JPRB, 0.84084E+00_JPRB, 0.84890E+00_JPRB, 0.85275E+00_JPRB, 0.84225E+00_JPRB ,&
&  0.82804E+00_JPRB, 0.81683E+00_JPRB, 0.81551E+00_JPRB, 0.81323E+00_JPRB, 0.81302E+00_JPRB, 0.81061E+00_JPRB, 0.80327E+00_JPRB ,&
&  0.79392E+00_JPRB, 0.78044E+00_JPRB, 0.77071E+00_JPRB, 0.77140E+00_JPRB, 0.78298E+00_JPRB, 0.79286E+00_JPRB, 0.81280E+00_JPRB ,&
&  0.84712E+00_JPRB, 0.88772E+00_JPRB, 0.92146E+00_JPRB, 0.95695E+00_JPRB, 0.97700E+00_JPRB, 0.98527E+00_JPRB, 0.97672E+00_JPRB ,&
&  0.95633E+00_JPRB, 0.91653E+00_JPRB, 0.90123E+00_JPRB, 0.87642E+00_JPRB, 0.86351E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.81480E+00_JPRB, 0.80765E+00_JPRB, 0.79337E+00_JPRB, 0.76789E+00_JPRB, 0.73486E+00_JPRB, 0.74341E+00_JPRB, 0.75519E+00_JPRB ,&
&  0.74890E+00_JPRB, 0.73955E+00_JPRB, 0.75018E+00_JPRB, 0.77540E+00_JPRB, 0.79999E+00_JPRB, 0.81403E+00_JPRB, 0.81628E+00_JPRB ,&
&  0.81123E+00_JPRB, 0.81319E+00_JPRB, 0.82154E+00_JPRB, 0.83876E+00_JPRB, 0.83794E+00_JPRB, 0.81999E+00_JPRB, 0.80890E+00_JPRB ,&
&  0.80578E+00_JPRB, 0.80393E+00_JPRB, 0.80138E+00_JPRB, 0.78468E+00_JPRB, 0.77376E+00_JPRB, 0.77095E+00_JPRB, 0.77156E+00_JPRB ,&
&  0.76729E+00_JPRB, 0.62798E+00_JPRB, 0.73258E+00_JPRB, 0.74221E+00_JPRB, 0.75193E+00_JPRB, 0.74581E+00_JPRB, 0.74622E+00_JPRB ,&
&  0.75079E+00_JPRB, 0.76622E+00_JPRB, 0.75876E+00_JPRB, 0.73407E+00_JPRB, 0.70805E+00_JPRB, 0.68039E+00_JPRB, 0.66165E+00_JPRB ,&
&  0.65229E+00_JPRB, 0.65525E+00_JPRB, 0.66538E+00_JPRB, 0.67602E+00_JPRB, 0.68737E+00_JPRB, 0.68923E+00_JPRB, 0.69327E+00_JPRB ,&
&  0.69602E+00_JPRB, 0.70115E+00_JPRB, 0.70366E+00_JPRB, 0.70935E+00_JPRB, 0.71162E+00_JPRB, 0.70852E+00_JPRB, 0.70115E+00_JPRB ,&
&  0.69065E+00_JPRB, 0.68499E+00_JPRB, 0.68556E+00_JPRB, 0.68932E+00_JPRB, 0.70034E+00_JPRB, 0.72051E+00_JPRB, 0.74373E+00_JPRB ,&
&  0.76790E+00_JPRB, 0.78357E+00_JPRB, 0.79224E+00_JPRB, 0.79444E+00_JPRB, 0.80150E+00_JPRB, 0.80869E+00_JPRB, 0.80505E+00_JPRB ,&
&  0.79390E+00_JPRB, 0.78181E+00_JPRB, 0.77523E+00_JPRB, 0.76793E+00_JPRB, 0.76500E+00_JPRB, 0.76288E+00_JPRB, 0.75712E+00_JPRB ,&
&  0.74892E+00_JPRB, 0.74777E+00_JPRB, 0.74988E+00_JPRB, 0.74003E+00_JPRB, 0.74271E+00_JPRB, 0.75419E+00_JPRB, 0.77451E+00_JPRB ,&
&  0.79968E+00_JPRB, 0.83241E+00_JPRB, 0.86447E+00_JPRB, 0.89041E+00_JPRB, 0.90512E+00_JPRB, 0.90871E+00_JPRB, 0.90165E+00_JPRB ,&
&  0.88327E+00_JPRB, 0.85832E+00_JPRB, 0.83967E+00_JPRB, 0.82949E+00_JPRB, 0.83002E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.77391E+00_JPRB, 0.77427E+00_JPRB, 0.76399E+00_JPRB, 0.72624E+00_JPRB, 0.69766E+00_JPRB, 0.70724E+00_JPRB, 0.71592E+00_JPRB ,&
&  0.71535E+00_JPRB, 0.73031E+00_JPRB, 0.75544E+00_JPRB, 0.78091E+00_JPRB, 0.79791E+00_JPRB, 0.79772E+00_JPRB, 0.80164E+00_JPRB ,&
&  0.79905E+00_JPRB, 0.79689E+00_JPRB, 0.80679E+00_JPRB, 0.81168E+00_JPRB, 0.80099E+00_JPRB, 0.77998E+00_JPRB, 0.77760E+00_JPRB ,&
&  0.78367E+00_JPRB, 0.77939E+00_JPRB, 0.76124E+00_JPRB, 0.74899E+00_JPRB, 0.73914E+00_JPRB, 0.74455E+00_JPRB, 0.75148E+00_JPRB ,&
&  0.74577E+00_JPRB, 0.69584E+00_JPRB, 0.73373E+00_JPRB, 0.74407E+00_JPRB, 0.75005E+00_JPRB, 0.73735E+00_JPRB, 0.74142E+00_JPRB ,&
&  0.75212E+00_JPRB, 0.75851E+00_JPRB, 0.73831E+00_JPRB, 0.71514E+00_JPRB, 0.69454E+00_JPRB, 0.67743E+00_JPRB, 0.64989E+00_JPRB ,&
&  0.63951E+00_JPRB, 0.63643E+00_JPRB, 0.64986E+00_JPRB, 0.66418E+00_JPRB, 0.68073E+00_JPRB, 0.68754E+00_JPRB, 0.69807E+00_JPRB ,&
&  0.70023E+00_JPRB, 0.69461E+00_JPRB, 0.68942E+00_JPRB, 0.68541E+00_JPRB, 0.68463E+00_JPRB, 0.67876E+00_JPRB, 0.67003E+00_JPRB ,&
&  0.66298E+00_JPRB, 0.66181E+00_JPRB, 0.66713E+00_JPRB, 0.67713E+00_JPRB, 0.69057E+00_JPRB, 0.71338E+00_JPRB, 0.73264E+00_JPRB ,&
&  0.75534E+00_JPRB, 0.76429E+00_JPRB, 0.77128E+00_JPRB, 0.78444E+00_JPRB, 0.79161E+00_JPRB, 0.78887E+00_JPRB, 0.78471E+00_JPRB ,&
&  0.77906E+00_JPRB, 0.77048E+00_JPRB, 0.77260E+00_JPRB, 0.76932E+00_JPRB, 0.76780E+00_JPRB, 0.75781E+00_JPRB, 0.74162E+00_JPRB ,&
&  0.73548E+00_JPRB, 0.73125E+00_JPRB, 0.72328E+00_JPRB, 0.72094E+00_JPRB, 0.71890E+00_JPRB, 0.73737E+00_JPRB, 0.76967E+00_JPRB ,&
&  0.78747E+00_JPRB, 0.81112E+00_JPRB, 0.84471E+00_JPRB, 0.86614E+00_JPRB, 0.86502E+00_JPRB, 0.86258E+00_JPRB, 0.84956E+00_JPRB ,&
&  0.82963E+00_JPRB, 0.81562E+00_JPRB, 0.80298E+00_JPRB, 0.79838E+00_JPRB, 0.79438E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.73104E+00_JPRB, 0.75265E+00_JPRB, 0.72023E+00_JPRB, 0.70279E+00_JPRB, 0.68249E+00_JPRB, 0.67709E+00_JPRB, 0.66779E+00_JPRB ,&
&  0.67851E+00_JPRB, 0.69571E+00_JPRB, 0.71952E+00_JPRB, 0.72746E+00_JPRB, 0.73423E+00_JPRB, 0.73696E+00_JPRB, 0.75375E+00_JPRB ,&
&  0.74899E+00_JPRB, 0.74069E+00_JPRB, 0.74376E+00_JPRB, 0.73340E+00_JPRB, 0.72463E+00_JPRB, 0.72956E+00_JPRB, 0.73857E+00_JPRB ,&
&  0.74869E+00_JPRB, 0.73994E+00_JPRB, 0.72260E+00_JPRB, 0.69615E+00_JPRB, 0.68370E+00_JPRB, 0.68107E+00_JPRB, 0.68863E+00_JPRB ,&
&  0.69396E+00_JPRB, 0.71839E+00_JPRB, 0.65495E+00_JPRB, 0.71075E+00_JPRB, 0.71820E+00_JPRB, 0.71569E+00_JPRB, 0.71367E+00_JPRB ,&
&  0.71788E+00_JPRB, 0.70720E+00_JPRB, 0.69826E+00_JPRB, 0.68961E+00_JPRB, 0.66699E+00_JPRB, 0.64882E+00_JPRB, 0.62961E+00_JPRB ,&
&  0.61255E+00_JPRB, 0.60212E+00_JPRB, 0.60740E+00_JPRB, 0.62515E+00_JPRB, 0.64332E+00_JPRB, 0.65340E+00_JPRB, 0.65950E+00_JPRB ,&
&  0.66267E+00_JPRB, 0.65527E+00_JPRB, 0.64528E+00_JPRB, 0.64533E+00_JPRB, 0.65172E+00_JPRB, 0.65651E+00_JPRB, 0.65954E+00_JPRB ,&
&  0.65985E+00_JPRB, 0.63762E+00_JPRB, 0.62556E+00_JPRB, 0.64250E+00_JPRB, 0.66142E+00_JPRB, 0.68323E+00_JPRB, 0.71156E+00_JPRB ,&
&  0.73349E+00_JPRB, 0.74242E+00_JPRB, 0.74263E+00_JPRB, 0.74913E+00_JPRB, 0.75484E+00_JPRB, 0.76258E+00_JPRB, 0.76858E+00_JPRB ,&
&  0.75648E+00_JPRB, 0.75285E+00_JPRB, 0.75306E+00_JPRB, 0.75242E+00_JPRB, 0.74252E+00_JPRB, 0.71856E+00_JPRB, 0.68567E+00_JPRB ,&
&  0.67055E+00_JPRB, 0.65559E+00_JPRB, 0.64471E+00_JPRB, 0.64149E+00_JPRB, 0.64302E+00_JPRB, 0.66941E+00_JPRB, 0.70762E+00_JPRB ,&
&  0.74563E+00_JPRB, 0.77614E+00_JPRB, 0.80235E+00_JPRB, 0.82626E+00_JPRB, 0.82446E+00_JPRB, 0.80198E+00_JPRB, 0.79154E+00_JPRB ,&
&  0.77573E+00_JPRB, 0.75893E+00_JPRB, 0.74753E+00_JPRB, 0.73861E+00_JPRB, 0.72923E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.73306E+00_JPRB, 0.73466E+00_JPRB, 0.72023E+00_JPRB, 0.71029E+00_JPRB, 0.69344E+00_JPRB, 0.67119E+00_JPRB, 0.65185E+00_JPRB ,&
&  0.65352E+00_JPRB, 0.66897E+00_JPRB, 0.68383E+00_JPRB, 0.69454E+00_JPRB, 0.70991E+00_JPRB, 0.71295E+00_JPRB, 0.71925E+00_JPRB ,&
&  0.71482E+00_JPRB, 0.67767E+00_JPRB, 0.67180E+00_JPRB, 0.67942E+00_JPRB, 0.68128E+00_JPRB, 0.68739E+00_JPRB, 0.69827E+00_JPRB ,&
&  0.70827E+00_JPRB, 0.70357E+00_JPRB, 0.69539E+00_JPRB, 0.66582E+00_JPRB, 0.65912E+00_JPRB, 0.66398E+00_JPRB, 0.66883E+00_JPRB ,&
&  0.67882E+00_JPRB, 0.69524E+00_JPRB, 0.69826E+00_JPRB, 0.70619E+00_JPRB, 0.71455E+00_JPRB, 0.71838E+00_JPRB, 0.71475E+00_JPRB ,&
&  0.68854E+00_JPRB, 0.66232E+00_JPRB, 0.65145E+00_JPRB, 0.63405E+00_JPRB, 0.61214E+00_JPRB, 0.59678E+00_JPRB, 0.58913E+00_JPRB ,&
&  0.58499E+00_JPRB, 0.59010E+00_JPRB, 0.59632E+00_JPRB, 0.61030E+00_JPRB, 0.61555E+00_JPRB, 0.61964E+00_JPRB, 0.62491E+00_JPRB ,&
&  0.62449E+00_JPRB, 0.62046E+00_JPRB, 0.62069E+00_JPRB, 0.63063E+00_JPRB, 0.63657E+00_JPRB, 0.63424E+00_JPRB, 0.62971E+00_JPRB ,&
&  0.61713E+00_JPRB, 0.60445E+00_JPRB, 0.61257E+00_JPRB, 0.62830E+00_JPRB, 0.63616E+00_JPRB, 0.66309E+00_JPRB, 0.69777E+00_JPRB ,&
&  0.72733E+00_JPRB, 0.74625E+00_JPRB, 0.74619E+00_JPRB, 0.74163E+00_JPRB, 0.73885E+00_JPRB, 0.74554E+00_JPRB, 0.75727E+00_JPRB ,&
&  0.74084E+00_JPRB, 0.73151E+00_JPRB, 0.72942E+00_JPRB, 0.70997E+00_JPRB, 0.67314E+00_JPRB, 0.64092E+00_JPRB, 0.61734E+00_JPRB ,&
&  0.59867E+00_JPRB, 0.58813E+00_JPRB, 0.59181E+00_JPRB, 0.59506E+00_JPRB, 0.60751E+00_JPRB, 0.64328E+00_JPRB, 0.68091E+00_JPRB ,&
&  0.72185E+00_JPRB, 0.74294E+00_JPRB, 0.74977E+00_JPRB, 0.75793E+00_JPRB, 0.76356E+00_JPRB, 0.75210E+00_JPRB, 0.74241E+00_JPRB ,&
&  0.73929E+00_JPRB, 0.73066E+00_JPRB, 0.72128E+00_JPRB, 0.71181E+00_JPRB, 0.71950E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.72729E+00_JPRB, 0.72018E+00_JPRB, 0.70711E+00_JPRB, 0.68098E+00_JPRB, 0.66327E+00_JPRB, 0.63605E+00_JPRB, 0.62069E+00_JPRB ,&
&  0.62302E+00_JPRB, 0.62446E+00_JPRB, 0.62954E+00_JPRB, 0.63866E+00_JPRB, 0.63635E+00_JPRB, 0.63150E+00_JPRB, 0.62671E+00_JPRB ,&
&  0.62965E+00_JPRB, 0.63009E+00_JPRB, 0.63858E+00_JPRB, 0.65401E+00_JPRB, 0.67032E+00_JPRB, 0.67965E+00_JPRB, 0.68961E+00_JPRB ,&
&  0.69724E+00_JPRB, 0.69112E+00_JPRB, 0.68669E+00_JPRB, 0.65615E+00_JPRB, 0.63840E+00_JPRB, 0.64348E+00_JPRB, 0.65082E+00_JPRB ,&
&  0.66074E+00_JPRB, 0.67103E+00_JPRB, 0.68058E+00_JPRB, 0.68615E+00_JPRB, 0.67213E+00_JPRB, 0.66811E+00_JPRB, 0.65715E+00_JPRB ,&
&  0.61393E+00_JPRB, 0.57173E+00_JPRB, 0.54887E+00_JPRB, 0.54117E+00_JPRB, 0.53596E+00_JPRB, 0.54194E+00_JPRB, 0.54960E+00_JPRB ,&
&  0.56288E+00_JPRB, 0.57499E+00_JPRB, 0.57800E+00_JPRB, 0.58104E+00_JPRB, 0.58246E+00_JPRB, 0.58158E+00_JPRB, 0.58207E+00_JPRB ,&
&  0.57949E+00_JPRB, 0.57807E+00_JPRB, 0.58552E+00_JPRB, 0.58626E+00_JPRB, 0.57754E+00_JPRB, 0.57498E+00_JPRB, 0.57313E+00_JPRB ,&
&  0.57126E+00_JPRB, 0.56955E+00_JPRB, 0.56681E+00_JPRB, 0.57722E+00_JPRB, 0.60309E+00_JPRB, 0.63285E+00_JPRB, 0.65865E+00_JPRB ,&
&  0.67373E+00_JPRB, 0.68607E+00_JPRB, 0.68918E+00_JPRB, 0.69085E+00_JPRB, 0.69017E+00_JPRB, 0.68119E+00_JPRB, 0.67771E+00_JPRB ,&
&  0.67000E+00_JPRB, 0.65093E+00_JPRB, 0.62525E+00_JPRB, 0.58533E+00_JPRB, 0.54631E+00_JPRB, 0.52369E+00_JPRB, 0.52283E+00_JPRB ,&
&  0.53475E+00_JPRB, 0.54713E+00_JPRB, 0.55345E+00_JPRB, 0.56137E+00_JPRB, 0.57587E+00_JPRB, 0.59338E+00_JPRB, 0.61202E+00_JPRB ,&
&  0.62368E+00_JPRB, 0.62377E+00_JPRB, 0.62142E+00_JPRB, 0.62935E+00_JPRB, 0.64008E+00_JPRB, 0.65776E+00_JPRB, 0.67723E+00_JPRB ,&
&  0.69429E+00_JPRB, 0.69402E+00_JPRB, 0.69457E+00_JPRB, 0.69605E+00_JPRB, 0.71618E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.72086E+00_JPRB, 0.71999E+00_JPRB, 0.69192E+00_JPRB, 0.66048E+00_JPRB, 0.64187E+00_JPRB, 0.62115E+00_JPRB, 0.60444E+00_JPRB ,&
&  0.60281E+00_JPRB, 0.59988E+00_JPRB, 0.59736E+00_JPRB, 0.59427E+00_JPRB, 0.59211E+00_JPRB, 0.58881E+00_JPRB, 0.57664E+00_JPRB ,&
&  0.57220E+00_JPRB, 0.57868E+00_JPRB, 0.60096E+00_JPRB, 0.62603E+00_JPRB, 0.64621E+00_JPRB, 0.67254E+00_JPRB, 0.68386E+00_JPRB ,&
&  0.69581E+00_JPRB, 0.70844E+00_JPRB, 0.71066E+00_JPRB, 0.67768E+00_JPRB, 0.65055E+00_JPRB, 0.65165E+00_JPRB, 0.66280E+00_JPRB ,&
&  0.66315E+00_JPRB, 0.66613E+00_JPRB, 0.66921E+00_JPRB, 0.66543E+00_JPRB, 0.64443E+00_JPRB, 0.60607E+00_JPRB, 0.57572E+00_JPRB ,&
&  0.55039E+00_JPRB, 0.53069E+00_JPRB, 0.52440E+00_JPRB, 0.51270E+00_JPRB, 0.51077E+00_JPRB, 0.52086E+00_JPRB, 0.53446E+00_JPRB ,&
&  0.54861E+00_JPRB, 0.55508E+00_JPRB, 0.56206E+00_JPRB, 0.55843E+00_JPRB, 0.55407E+00_JPRB, 0.55686E+00_JPRB, 0.55671E+00_JPRB ,&
&  0.55614E+00_JPRB, 0.54701E+00_JPRB, 0.54433E+00_JPRB, 0.53750E+00_JPRB, 0.53367E+00_JPRB, 0.53423E+00_JPRB, 0.53740E+00_JPRB ,&
&  0.53900E+00_JPRB, 0.54492E+00_JPRB, 0.54688E+00_JPRB, 0.54925E+00_JPRB, 0.55876E+00_JPRB, 0.57538E+00_JPRB, 0.59199E+00_JPRB ,&
&  0.60446E+00_JPRB, 0.61434E+00_JPRB, 0.62198E+00_JPRB, 0.62786E+00_JPRB, 0.62778E+00_JPRB, 0.62320E+00_JPRB, 0.60941E+00_JPRB ,&
&  0.57911E+00_JPRB, 0.54651E+00_JPRB, 0.52160E+00_JPRB, 0.50166E+00_JPRB, 0.49792E+00_JPRB, 0.50866E+00_JPRB, 0.52607E+00_JPRB ,&
&  0.53927E+00_JPRB, 0.54298E+00_JPRB, 0.53668E+00_JPRB, 0.53465E+00_JPRB, 0.53920E+00_JPRB, 0.54846E+00_JPRB, 0.55570E+00_JPRB ,&
&  0.55482E+00_JPRB, 0.55169E+00_JPRB, 0.55567E+00_JPRB, 0.56288E+00_JPRB, 0.57711E+00_JPRB, 0.59497E+00_JPRB, 0.61421E+00_JPRB ,&
&  0.63331E+00_JPRB, 0.64815E+00_JPRB, 0.66736E+00_JPRB, 0.68688E+00_JPRB, 0.71242E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.70234E+00_JPRB, 0.67530E+00_JPRB, 0.64662E+00_JPRB, 0.62197E+00_JPRB, 0.61023E+00_JPRB, 0.59277E+00_JPRB, 0.57805E+00_JPRB ,&
&  0.57710E+00_JPRB, 0.58474E+00_JPRB, 0.57966E+00_JPRB, 0.57325E+00_JPRB, 0.56363E+00_JPRB, 0.55581E+00_JPRB, 0.54673E+00_JPRB ,&
&  0.54270E+00_JPRB, 0.55410E+00_JPRB, 0.57966E+00_JPRB, 0.59233E+00_JPRB, 0.59849E+00_JPRB, 0.62901E+00_JPRB, 0.66048E+00_JPRB ,&
&  0.66988E+00_JPRB, 0.68402E+00_JPRB, 0.68823E+00_JPRB, 0.66282E+00_JPRB, 0.64625E+00_JPRB, 0.63984E+00_JPRB, 0.64363E+00_JPRB ,&
&  0.63886E+00_JPRB, 0.64293E+00_JPRB, 0.63460E+00_JPRB, 0.56484E+00_JPRB, 0.56365E+00_JPRB, 0.53300E+00_JPRB, 0.52224E+00_JPRB ,&
&  0.51659E+00_JPRB, 0.52149E+00_JPRB, 0.51735E+00_JPRB, 0.50802E+00_JPRB, 0.50436E+00_JPRB, 0.50772E+00_JPRB, 0.50943E+00_JPRB ,&
&  0.50532E+00_JPRB, 0.49490E+00_JPRB, 0.49947E+00_JPRB, 0.50588E+00_JPRB, 0.50959E+00_JPRB, 0.51672E+00_JPRB, 0.52727E+00_JPRB ,&
&  0.51702E+00_JPRB, 0.50767E+00_JPRB, 0.49913E+00_JPRB, 0.49965E+00_JPRB, 0.50648E+00_JPRB, 0.52103E+00_JPRB, 0.52606E+00_JPRB ,&
&  0.52418E+00_JPRB, 0.52320E+00_JPRB, 0.52450E+00_JPRB, 0.53095E+00_JPRB, 0.54569E+00_JPRB, 0.56040E+00_JPRB, 0.56162E+00_JPRB ,&
&  0.56282E+00_JPRB, 0.56752E+00_JPRB, 0.57265E+00_JPRB, 0.57434E+00_JPRB, 0.57027E+00_JPRB, 0.55508E+00_JPRB, 0.53749E+00_JPRB ,&
&  0.52086E+00_JPRB, 0.50370E+00_JPRB, 0.48926E+00_JPRB, 0.48481E+00_JPRB, 0.49820E+00_JPRB, 0.52764E+00_JPRB, 0.54410E+00_JPRB ,&
&  0.54809E+00_JPRB, 0.55011E+00_JPRB, 0.55331E+00_JPRB, 0.55408E+00_JPRB, 0.55580E+00_JPRB, 0.55363E+00_JPRB, 0.54682E+00_JPRB ,&
&  0.53542E+00_JPRB, 0.52439E+00_JPRB, 0.52198E+00_JPRB, 0.52660E+00_JPRB, 0.53204E+00_JPRB, 0.54445E+00_JPRB, 0.56123E+00_JPRB ,&
&  0.58325E+00_JPRB, 0.62138E+00_JPRB, 0.65348E+00_JPRB, 0.67841E+00_JPRB, 0.69656E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.67904E+00_JPRB, 0.65340E+00_JPRB, 0.63630E+00_JPRB, 0.62290E+00_JPRB, 0.60473E+00_JPRB, 0.59085E+00_JPRB, 0.58523E+00_JPRB ,&
&  0.59336E+00_JPRB, 0.59180E+00_JPRB, 0.58174E+00_JPRB, 0.57327E+00_JPRB, 0.55715E+00_JPRB, 0.54280E+00_JPRB, 0.53694E+00_JPRB ,&
&  0.54536E+00_JPRB, 0.56149E+00_JPRB, 0.58133E+00_JPRB, 0.58933E+00_JPRB, 0.58859E+00_JPRB, 0.59707E+00_JPRB, 0.61641E+00_JPRB ,&
&  0.64513E+00_JPRB, 0.66518E+00_JPRB, 0.67211E+00_JPRB, 0.65873E+00_JPRB, 0.64104E+00_JPRB, 0.63281E+00_JPRB, 0.61848E+00_JPRB ,&
&  0.60172E+00_JPRB, 0.60497E+00_JPRB, 0.53766E+00_JPRB, 0.53006E+00_JPRB, 0.50684E+00_JPRB, 0.49225E+00_JPRB, 0.48832E+00_JPRB ,&
&  0.50115E+00_JPRB, 0.50488E+00_JPRB, 0.49631E+00_JPRB, 0.48404E+00_JPRB, 0.47578E+00_JPRB, 0.46479E+00_JPRB, 0.45470E+00_JPRB ,&
&  0.44779E+00_JPRB, 0.45045E+00_JPRB, 0.45950E+00_JPRB, 0.46882E+00_JPRB, 0.47980E+00_JPRB, 0.49032E+00_JPRB, 0.48685E+00_JPRB ,&
&  0.47518E+00_JPRB, 0.47320E+00_JPRB, 0.48034E+00_JPRB, 0.49521E+00_JPRB, 0.51147E+00_JPRB, 0.51908E+00_JPRB, 0.52268E+00_JPRB ,&
&  0.51460E+00_JPRB, 0.49769E+00_JPRB, 0.48318E+00_JPRB, 0.47125E+00_JPRB, 0.44511E+00_JPRB, 0.38208E+00_JPRB, 0.35938E+00_JPRB ,&
&  0.37037E+00_JPRB, 0.42897E+00_JPRB, 0.49897E+00_JPRB, 0.49092E+00_JPRB, 0.48687E+00_JPRB, 0.47254E+00_JPRB, 0.43594E+00_JPRB ,&
&  0.39347E+00_JPRB, 0.33209E+00_JPRB, 0.31094E+00_JPRB, 0.30752E+00_JPRB, 0.30683E+00_JPRB, 0.31285E+00_JPRB, 0.32520E+00_JPRB ,&
&  0.34145E+00_JPRB, 0.36796E+00_JPRB, 0.38518E+00_JPRB, 0.39162E+00_JPRB, 0.38985E+00_JPRB, 0.37366E+00_JPRB, 0.35967E+00_JPRB ,&
&  0.35051E+00_JPRB, 0.37556E+00_JPRB, 0.43449E+00_JPRB, 0.51709E+00_JPRB, 0.51674E+00_JPRB, 0.54184E+00_JPRB, 0.56694E+00_JPRB ,&
&  0.59078E+00_JPRB, 0.63061E+00_JPRB, 0.65667E+00_JPRB, 0.66238E+00_JPRB, 0.68080E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.63876E+00_JPRB, 0.62696E+00_JPRB, 0.61896E+00_JPRB, 0.61332E+00_JPRB, 0.60658E+00_JPRB, 0.60538E+00_JPRB, 0.60469E+00_JPRB ,&
&  0.59882E+00_JPRB, 0.59024E+00_JPRB, 0.59094E+00_JPRB, 0.57356E+00_JPRB, 0.54891E+00_JPRB, 0.53240E+00_JPRB, 0.53010E+00_JPRB ,&
&  0.53393E+00_JPRB, 0.54103E+00_JPRB, 0.55009E+00_JPRB, 0.55868E+00_JPRB, 0.56362E+00_JPRB, 0.57540E+00_JPRB, 0.59638E+00_JPRB ,&
&  0.62076E+00_JPRB, 0.63445E+00_JPRB, 0.63856E+00_JPRB, 0.63045E+00_JPRB, 0.61807E+00_JPRB, 0.60527E+00_JPRB, 0.59428E+00_JPRB ,&
&  0.58526E+00_JPRB, 0.48353E+00_JPRB, 0.41587E+00_JPRB, 0.42255E+00_JPRB, 0.49131E+00_JPRB, 0.47512E+00_JPRB, 0.47328E+00_JPRB ,&
&  0.48564E+00_JPRB, 0.48556E+00_JPRB, 0.47221E+00_JPRB, 0.45512E+00_JPRB, 0.43918E+00_JPRB, 0.42775E+00_JPRB, 0.41950E+00_JPRB ,&
&  0.41576E+00_JPRB, 0.42248E+00_JPRB, 0.43050E+00_JPRB, 0.44295E+00_JPRB, 0.44598E+00_JPRB, 0.43316E+00_JPRB, 0.41774E+00_JPRB ,&
&  0.40292E+00_JPRB, 0.39764E+00_JPRB, 0.40007E+00_JPRB, 0.40671E+00_JPRB, 0.41544E+00_JPRB, 0.42228E+00_JPRB, 0.42026E+00_JPRB ,&
&  0.39159E+00_JPRB, 0.36854E+00_JPRB, 0.34036E+00_JPRB, 0.31380E+00_JPRB, 0.29069E+00_JPRB, 0.27519E+00_JPRB, 0.26724E+00_JPRB ,&
&  0.26541E+00_JPRB, 0.27302E+00_JPRB, 0.29460E+00_JPRB, 0.34370E+00_JPRB, 0.41026E+00_JPRB, 0.33117E+00_JPRB, 0.29211E+00_JPRB ,&
&  0.27064E+00_JPRB, 0.25750E+00_JPRB, 0.25249E+00_JPRB, 0.25324E+00_JPRB, 0.25742E+00_JPRB, 0.26360E+00_JPRB, 0.26847E+00_JPRB ,&
&  0.27235E+00_JPRB, 0.27574E+00_JPRB, 0.27753E+00_JPRB, 0.27835E+00_JPRB, 0.27979E+00_JPRB, 0.28088E+00_JPRB, 0.27978E+00_JPRB ,&
&  0.27833E+00_JPRB, 0.28464E+00_JPRB, 0.30235E+00_JPRB, 0.33027E+00_JPRB, 0.36299E+00_JPRB, 0.39848E+00_JPRB, 0.45061E+00_JPRB ,&
&  0.51607E+00_JPRB, 0.59055E+00_JPRB, 0.60874E+00_JPRB, 0.62175E+00_JPRB, 0.63233E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.58095E+00_JPRB, 0.58471E+00_JPRB, 0.58893E+00_JPRB, 0.59130E+00_JPRB, 0.58590E+00_JPRB, 0.57860E+00_JPRB, 0.57393E+00_JPRB ,&
&  0.57903E+00_JPRB, 0.57893E+00_JPRB, 0.55546E+00_JPRB, 0.53713E+00_JPRB, 0.52661E+00_JPRB, 0.52294E+00_JPRB, 0.51987E+00_JPRB ,&
&  0.51365E+00_JPRB, 0.51145E+00_JPRB, 0.51261E+00_JPRB, 0.52379E+00_JPRB, 0.53310E+00_JPRB, 0.54124E+00_JPRB, 0.54863E+00_JPRB ,&
&  0.54393E+00_JPRB, 0.55648E+00_JPRB, 0.57804E+00_JPRB, 0.57720E+00_JPRB, 0.57444E+00_JPRB, 0.56724E+00_JPRB, 0.55278E+00_JPRB ,&
&  0.50457E+00_JPRB, 0.44057E+00_JPRB, 0.37967E+00_JPRB, 0.39133E+00_JPRB, 0.46804E+00_JPRB, 0.47681E+00_JPRB, 0.46169E+00_JPRB ,&
&  0.47216E+00_JPRB, 0.46781E+00_JPRB, 0.44991E+00_JPRB, 0.43666E+00_JPRB, 0.42117E+00_JPRB, 0.41107E+00_JPRB, 0.40392E+00_JPRB ,&
&  0.40098E+00_JPRB, 0.39990E+00_JPRB, 0.38482E+00_JPRB, 0.33608E+00_JPRB, 0.30781E+00_JPRB, 0.29044E+00_JPRB, 0.27262E+00_JPRB ,&
&  0.25854E+00_JPRB, 0.24966E+00_JPRB, 0.24815E+00_JPRB, 0.24839E+00_JPRB, 0.25323E+00_JPRB, 0.25544E+00_JPRB, 0.25750E+00_JPRB ,&
&  0.25742E+00_JPRB, 0.25634E+00_JPRB, 0.25229E+00_JPRB, 0.24775E+00_JPRB, 0.24458E+00_JPRB, 0.24413E+00_JPRB, 0.24383E+00_JPRB ,&
&  0.24651E+00_JPRB, 0.25050E+00_JPRB, 0.26718E+00_JPRB, 0.28498E+00_JPRB, 0.28362E+00_JPRB, 0.26843E+00_JPRB, 0.25377E+00_JPRB ,&
&  0.24382E+00_JPRB, 0.23882E+00_JPRB, 0.23557E+00_JPRB, 0.23501E+00_JPRB, 0.23793E+00_JPRB, 0.24213E+00_JPRB, 0.24323E+00_JPRB ,&
&  0.24306E+00_JPRB, 0.24372E+00_JPRB, 0.24378E+00_JPRB, 0.24426E+00_JPRB, 0.24492E+00_JPRB, 0.24749E+00_JPRB, 0.25237E+00_JPRB ,&
&  0.25811E+00_JPRB, 0.26844E+00_JPRB, 0.28016E+00_JPRB, 0.29590E+00_JPRB, 0.30745E+00_JPRB, 0.31602E+00_JPRB, 0.32730E+00_JPRB ,&
&  0.34506E+00_JPRB, 0.38365E+00_JPRB, 0.45257E+00_JPRB, 0.57833E+00_JPRB, 0.57299E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.54912E+00_JPRB, 0.56063E+00_JPRB, 0.57305E+00_JPRB, 0.56498E+00_JPRB, 0.55143E+00_JPRB, 0.54380E+00_JPRB, 0.54355E+00_JPRB ,&
&  0.54321E+00_JPRB, 0.54025E+00_JPRB, 0.52805E+00_JPRB, 0.51512E+00_JPRB, 0.49704E+00_JPRB, 0.44895E+00_JPRB, 0.43087E+00_JPRB ,&
&  0.41332E+00_JPRB, 0.41588E+00_JPRB, 0.42036E+00_JPRB, 0.42691E+00_JPRB, 0.44081E+00_JPRB, 0.45166E+00_JPRB, 0.45361E+00_JPRB ,&
&  0.44360E+00_JPRB, 0.42528E+00_JPRB, 0.40766E+00_JPRB, 0.39402E+00_JPRB, 0.39366E+00_JPRB, 0.39371E+00_JPRB, 0.38675E+00_JPRB ,&
&  0.37326E+00_JPRB, 0.36727E+00_JPRB, 0.36706E+00_JPRB, 0.38697E+00_JPRB, 0.43900E+00_JPRB, 0.45823E+00_JPRB, 0.43327E+00_JPRB ,&
&  0.43772E+00_JPRB, 0.43286E+00_JPRB, 0.42886E+00_JPRB, 0.43020E+00_JPRB, 0.42007E+00_JPRB, 0.40058E+00_JPRB, 0.36675E+00_JPRB ,&
&  0.32315E+00_JPRB, 0.30053E+00_JPRB, 0.27935E+00_JPRB, 0.25936E+00_JPRB, 0.24394E+00_JPRB, 0.23494E+00_JPRB, 0.22722E+00_JPRB ,&
&  0.22219E+00_JPRB, 0.21794E+00_JPRB, 0.21514E+00_JPRB, 0.21498E+00_JPRB, 0.21646E+00_JPRB, 0.21719E+00_JPRB, 0.21813E+00_JPRB ,&
&  0.21955E+00_JPRB, 0.22032E+00_JPRB, 0.22074E+00_JPRB, 0.22200E+00_JPRB, 0.22344E+00_JPRB, 0.22609E+00_JPRB, 0.22974E+00_JPRB ,&
&  0.23428E+00_JPRB, 0.23804E+00_JPRB, 0.24168E+00_JPRB, 0.24788E+00_JPRB, 0.24845E+00_JPRB, 0.24724E+00_JPRB, 0.23938E+00_JPRB ,&
&  0.23367E+00_JPRB, 0.23114E+00_JPRB, 0.22852E+00_JPRB, 0.22781E+00_JPRB, 0.22894E+00_JPRB, 0.23090E+00_JPRB, 0.23286E+00_JPRB ,&
&  0.23423E+00_JPRB, 0.23428E+00_JPRB, 0.23344E+00_JPRB, 0.23334E+00_JPRB, 0.23390E+00_JPRB, 0.23406E+00_JPRB, 0.23769E+00_JPRB ,&
&  0.24557E+00_JPRB, 0.25715E+00_JPRB, 0.27003E+00_JPRB, 0.28769E+00_JPRB, 0.30424E+00_JPRB, 0.32074E+00_JPRB, 0.34833E+00_JPRB ,&
&  0.38764E+00_JPRB, 0.49545E+00_JPRB, 0.56481E+00_JPRB, 0.50420E+00_JPRB, 0.52854E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.52122E+00_JPRB, 0.52886E+00_JPRB, 0.53988E+00_JPRB, 0.54462E+00_JPRB, 0.55273E+00_JPRB, 0.54114E+00_JPRB, 0.53405E+00_JPRB ,&
&  0.50380E+00_JPRB, 0.48178E+00_JPRB, 0.45077E+00_JPRB, 0.41216E+00_JPRB, 0.38615E+00_JPRB, 0.36605E+00_JPRB, 0.35883E+00_JPRB ,&
&  0.35050E+00_JPRB, 0.34534E+00_JPRB, 0.34014E+00_JPRB, 0.33805E+00_JPRB, 0.33661E+00_JPRB, 0.33758E+00_JPRB, 0.33723E+00_JPRB ,&
&  0.33353E+00_JPRB, 0.32790E+00_JPRB, 0.32123E+00_JPRB, 0.32409E+00_JPRB, 0.33700E+00_JPRB, 0.35191E+00_JPRB, 0.36950E+00_JPRB ,&
&  0.38980E+00_JPRB, 0.40050E+00_JPRB, 0.41075E+00_JPRB, 0.41481E+00_JPRB, 0.39938E+00_JPRB, 0.40195E+00_JPRB, 0.40498E+00_JPRB ,&
&  0.41301E+00_JPRB, 0.41971E+00_JPRB, 0.41667E+00_JPRB, 0.41217E+00_JPRB, 0.36122E+00_JPRB, 0.32825E+00_JPRB, 0.29960E+00_JPRB ,&
&  0.27814E+00_JPRB, 0.26435E+00_JPRB, 0.25265E+00_JPRB, 0.24268E+00_JPRB, 0.23252E+00_JPRB, 0.22598E+00_JPRB, 0.22106E+00_JPRB ,&
&  0.21709E+00_JPRB, 0.21533E+00_JPRB, 0.21367E+00_JPRB, 0.21316E+00_JPRB, 0.21355E+00_JPRB, 0.21295E+00_JPRB, 0.21289E+00_JPRB ,&
&  0.21267E+00_JPRB, 0.21197E+00_JPRB, 0.21202E+00_JPRB, 0.21323E+00_JPRB, 0.21518E+00_JPRB, 0.21842E+00_JPRB, 0.21949E+00_JPRB ,&
&  0.22046E+00_JPRB, 0.22111E+00_JPRB, 0.22117E+00_JPRB, 0.22309E+00_JPRB, 0.22412E+00_JPRB, 0.22428E+00_JPRB, 0.22269E+00_JPRB ,&
&  0.22103E+00_JPRB, 0.21978E+00_JPRB, 0.21967E+00_JPRB, 0.22019E+00_JPRB, 0.22166E+00_JPRB, 0.22396E+00_JPRB, 0.22448E+00_JPRB ,&
&  0.22537E+00_JPRB, 0.22498E+00_JPRB, 0.22459E+00_JPRB, 0.22648E+00_JPRB, 0.22870E+00_JPRB, 0.23113E+00_JPRB, 0.23441E+00_JPRB ,&
&  0.23617E+00_JPRB, 0.24330E+00_JPRB, 0.25642E+00_JPRB, 0.27662E+00_JPRB, 0.30814E+00_JPRB, 0.32469E+00_JPRB, 0.36396E+00_JPRB ,&
&  0.42504E+00_JPRB, 0.56732E+00_JPRB, 0.63174E+00_JPRB, 0.49027E+00_JPRB, 0.50576E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.46313E+00_JPRB, 0.47542E+00_JPRB, 0.49253E+00_JPRB, 0.50319E+00_JPRB, 0.51721E+00_JPRB, 0.51983E+00_JPRB, 0.51407E+00_JPRB ,&
&  0.49563E+00_JPRB, 0.46640E+00_JPRB, 0.44982E+00_JPRB, 0.42907E+00_JPRB, 0.40779E+00_JPRB, 0.38628E+00_JPRB, 0.37667E+00_JPRB ,&
&  0.36506E+00_JPRB, 0.35189E+00_JPRB, 0.33849E+00_JPRB, 0.32689E+00_JPRB, 0.31578E+00_JPRB, 0.30549E+00_JPRB, 0.29957E+00_JPRB ,&
&  0.29346E+00_JPRB, 0.29295E+00_JPRB, 0.29235E+00_JPRB, 0.29649E+00_JPRB, 0.30783E+00_JPRB, 0.32670E+00_JPRB, 0.35262E+00_JPRB ,&
&  0.39144E+00_JPRB, 0.39674E+00_JPRB, 0.38200E+00_JPRB, 0.37570E+00_JPRB, 0.36835E+00_JPRB, 0.36207E+00_JPRB, 0.34921E+00_JPRB ,&
&  0.35660E+00_JPRB, 0.36817E+00_JPRB, 0.38366E+00_JPRB, 0.37220E+00_JPRB, 0.32659E+00_JPRB, 0.30655E+00_JPRB, 0.28292E+00_JPRB ,&
&  0.27080E+00_JPRB, 0.26062E+00_JPRB, 0.25252E+00_JPRB, 0.24466E+00_JPRB, 0.23636E+00_JPRB, 0.23129E+00_JPRB, 0.22575E+00_JPRB ,&
&  0.22187E+00_JPRB, 0.22032E+00_JPRB, 0.21792E+00_JPRB, 0.21797E+00_JPRB, 0.21787E+00_JPRB, 0.21715E+00_JPRB, 0.21648E+00_JPRB ,&
&  0.21528E+00_JPRB, 0.21399E+00_JPRB, 0.21340E+00_JPRB, 0.21396E+00_JPRB, 0.21478E+00_JPRB, 0.21560E+00_JPRB, 0.21357E+00_JPRB ,&
&  0.21231E+00_JPRB, 0.21106E+00_JPRB, 0.21092E+00_JPRB, 0.21221E+00_JPRB, 0.21262E+00_JPRB, 0.21349E+00_JPRB, 0.21366E+00_JPRB ,&
&  0.21379E+00_JPRB, 0.21440E+00_JPRB, 0.21409E+00_JPRB, 0.21486E+00_JPRB, 0.21517E+00_JPRB, 0.21635E+00_JPRB, 0.21714E+00_JPRB ,&
&  0.21800E+00_JPRB, 0.22005E+00_JPRB, 0.22221E+00_JPRB, 0.22534E+00_JPRB, 0.22960E+00_JPRB, 0.23242E+00_JPRB, 0.23695E+00_JPRB ,&
&  0.24230E+00_JPRB, 0.25122E+00_JPRB, 0.26831E+00_JPRB, 0.28747E+00_JPRB, 0.31133E+00_JPRB, 0.32269E+00_JPRB, 0.33078E+00_JPRB ,&
&  0.34353E+00_JPRB, 0.37658E+00_JPRB, 0.42731E+00_JPRB, 0.44970E+00_JPRB, 0.45558E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.45953E+00_JPRB, 0.46122E+00_JPRB, 0.46236E+00_JPRB, 0.45931E+00_JPRB, 0.45846E+00_JPRB, 0.45853E+00_JPRB, 0.45579E+00_JPRB ,&
&  0.44780E+00_JPRB, 0.44549E+00_JPRB, 0.44048E+00_JPRB, 0.43313E+00_JPRB, 0.42585E+00_JPRB, 0.41402E+00_JPRB, 0.39761E+00_JPRB ,&
&  0.37564E+00_JPRB, 0.35622E+00_JPRB, 0.34168E+00_JPRB, 0.32810E+00_JPRB, 0.31483E+00_JPRB, 0.30188E+00_JPRB, 0.29686E+00_JPRB ,&
&  0.29238E+00_JPRB, 0.29056E+00_JPRB, 0.29010E+00_JPRB, 0.29353E+00_JPRB, 0.30109E+00_JPRB, 0.31053E+00_JPRB, 0.32089E+00_JPRB ,&
&  0.33252E+00_JPRB, 0.34340E+00_JPRB, 0.34466E+00_JPRB, 0.33730E+00_JPRB, 0.33040E+00_JPRB, 0.32610E+00_JPRB, 0.32278E+00_JPRB ,&
&  0.31886E+00_JPRB, 0.31060E+00_JPRB, 0.30111E+00_JPRB, 0.29109E+00_JPRB, 0.28098E+00_JPRB, 0.27361E+00_JPRB, 0.26496E+00_JPRB ,&
&  0.25861E+00_JPRB, 0.25276E+00_JPRB, 0.24649E+00_JPRB, 0.24109E+00_JPRB, 0.23742E+00_JPRB, 0.23438E+00_JPRB, 0.23130E+00_JPRB ,&
&  0.22980E+00_JPRB, 0.22831E+00_JPRB, 0.22667E+00_JPRB, 0.22508E+00_JPRB, 0.22308E+00_JPRB, 0.22157E+00_JPRB, 0.22020E+00_JPRB ,&
&  0.21916E+00_JPRB, 0.21823E+00_JPRB, 0.21768E+00_JPRB, 0.21709E+00_JPRB, 0.21659E+00_JPRB, 0.21556E+00_JPRB, 0.21435E+00_JPRB ,&
&  0.21360E+00_JPRB, 0.21335E+00_JPRB, 0.21257E+00_JPRB, 0.21215E+00_JPRB, 0.21182E+00_JPRB, 0.21106E+00_JPRB, 0.21155E+00_JPRB ,&
&  0.21276E+00_JPRB, 0.21411E+00_JPRB, 0.21437E+00_JPRB, 0.21545E+00_JPRB, 0.21674E+00_JPRB, 0.21837E+00_JPRB, 0.22096E+00_JPRB ,&
&  0.22292E+00_JPRB, 0.22497E+00_JPRB, 0.22755E+00_JPRB, 0.22979E+00_JPRB, 0.23329E+00_JPRB, 0.23762E+00_JPRB, 0.24295E+00_JPRB ,&
&  0.25023E+00_JPRB, 0.25902E+00_JPRB, 0.27145E+00_JPRB, 0.28666E+00_JPRB, 0.30855E+00_JPRB, 0.31668E+00_JPRB, 0.32446E+00_JPRB ,&
&  0.33074E+00_JPRB, 0.34311E+00_JPRB, 0.42567E+00_JPRB, 0.44041E+00_JPRB, 0.45537E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.33421E+00_JPRB, 0.34440E+00_JPRB, 0.35286E+00_JPRB, 0.36086E+00_JPRB, 0.36151E+00_JPRB, 0.42634E+00_JPRB, 0.40627E+00_JPRB ,&
&  0.39661E+00_JPRB, 0.36905E+00_JPRB, 0.37044E+00_JPRB, 0.36635E+00_JPRB, 0.36003E+00_JPRB, 0.35303E+00_JPRB, 0.34705E+00_JPRB ,&
&  0.34106E+00_JPRB, 0.33341E+00_JPRB, 0.32873E+00_JPRB, 0.32286E+00_JPRB, 0.31722E+00_JPRB, 0.31154E+00_JPRB, 0.30814E+00_JPRB ,&
&  0.30559E+00_JPRB, 0.30415E+00_JPRB, 0.30305E+00_JPRB, 0.30172E+00_JPRB, 0.30400E+00_JPRB, 0.30510E+00_JPRB, 0.30312E+00_JPRB ,&
&  0.29925E+00_JPRB, 0.29563E+00_JPRB, 0.29120E+00_JPRB, 0.28670E+00_JPRB, 0.28389E+00_JPRB, 0.28131E+00_JPRB, 0.27862E+00_JPRB ,&
&  0.27499E+00_JPRB, 0.27095E+00_JPRB, 0.26677E+00_JPRB, 0.26343E+00_JPRB, 0.26021E+00_JPRB, 0.25688E+00_JPRB, 0.25373E+00_JPRB ,&
&  0.25043E+00_JPRB, 0.24716E+00_JPRB, 0.24388E+00_JPRB, 0.24175E+00_JPRB, 0.23997E+00_JPRB, 0.23832E+00_JPRB, 0.23683E+00_JPRB ,&
&  0.23622E+00_JPRB, 0.23548E+00_JPRB, 0.23456E+00_JPRB, 0.23341E+00_JPRB, 0.23234E+00_JPRB, 0.23118E+00_JPRB, 0.23024E+00_JPRB ,&
&  0.23011E+00_JPRB, 0.22952E+00_JPRB, 0.22890E+00_JPRB, 0.22810E+00_JPRB, 0.22773E+00_JPRB, 0.22690E+00_JPRB, 0.22586E+00_JPRB ,&
&  0.22492E+00_JPRB, 0.22407E+00_JPRB, 0.22387E+00_JPRB, 0.22362E+00_JPRB, 0.22335E+00_JPRB, 0.22314E+00_JPRB, 0.22354E+00_JPRB ,&
&  0.22404E+00_JPRB, 0.22467E+00_JPRB, 0.22570E+00_JPRB, 0.22692E+00_JPRB, 0.22801E+00_JPRB, 0.22908E+00_JPRB, 0.23058E+00_JPRB ,&
&  0.23257E+00_JPRB, 0.23456E+00_JPRB, 0.23680E+00_JPRB, 0.23882E+00_JPRB, 0.24230E+00_JPRB, 0.24606E+00_JPRB, 0.25021E+00_JPRB ,&
&  0.25527E+00_JPRB, 0.26223E+00_JPRB, 0.27029E+00_JPRB, 0.27759E+00_JPRB, 0.28500E+00_JPRB, 0.29278E+00_JPRB, 0.29919E+00_JPRB ,&
&  0.30476E+00_JPRB, 0.31026E+00_JPRB, 0.31562E+00_JPRB, 0.32156E+00_JPRB, 0.32802E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.29204E+00_JPRB, 0.29484E+00_JPRB, 0.29717E+00_JPRB, 0.29880E+00_JPRB, 0.29889E+00_JPRB, 0.30048E+00_JPRB, 0.30075E+00_JPRB ,&
&  0.30081E+00_JPRB, 0.30060E+00_JPRB, 0.30136E+00_JPRB, 0.30216E+00_JPRB, 0.30268E+00_JPRB, 0.30298E+00_JPRB, 0.30303E+00_JPRB ,&
&  0.30252E+00_JPRB, 0.30152E+00_JPRB, 0.29749E+00_JPRB, 0.29525E+00_JPRB, 0.29392E+00_JPRB, 0.29327E+00_JPRB, 0.29292E+00_JPRB ,&
&  0.29209E+00_JPRB, 0.29111E+00_JPRB, 0.28954E+00_JPRB, 0.28932E+00_JPRB, 0.28850E+00_JPRB, 0.28742E+00_JPRB, 0.28590E+00_JPRB ,&
&  0.28441E+00_JPRB, 0.28322E+00_JPRB, 0.28178E+00_JPRB, 0.28030E+00_JPRB, 0.27850E+00_JPRB, 0.27630E+00_JPRB, 0.27351E+00_JPRB ,&
&  0.27065E+00_JPRB, 0.26773E+00_JPRB, 0.26531E+00_JPRB, 0.26306E+00_JPRB, 0.26111E+00_JPRB, 0.25953E+00_JPRB, 0.25838E+00_JPRB ,&
&  0.25711E+00_JPRB, 0.25567E+00_JPRB, 0.25419E+00_JPRB, 0.25296E+00_JPRB, 0.25174E+00_JPRB, 0.25069E+00_JPRB, 0.25015E+00_JPRB ,&
&  0.24983E+00_JPRB, 0.24945E+00_JPRB, 0.24898E+00_JPRB, 0.24856E+00_JPRB, 0.24802E+00_JPRB, 0.24753E+00_JPRB, 0.24703E+00_JPRB ,&
&  0.24661E+00_JPRB, 0.24637E+00_JPRB, 0.24599E+00_JPRB, 0.24560E+00_JPRB, 0.24536E+00_JPRB, 0.24495E+00_JPRB, 0.24449E+00_JPRB ,&
&  0.24396E+00_JPRB, 0.24367E+00_JPRB, 0.24334E+00_JPRB, 0.24293E+00_JPRB, 0.24255E+00_JPRB, 0.24229E+00_JPRB, 0.24229E+00_JPRB ,&
&  0.24226E+00_JPRB, 0.24215E+00_JPRB, 0.24220E+00_JPRB, 0.24296E+00_JPRB, 0.24376E+00_JPRB, 0.24472E+00_JPRB, 0.24580E+00_JPRB ,&
&  0.24722E+00_JPRB, 0.24869E+00_JPRB, 0.25035E+00_JPRB, 0.25195E+00_JPRB, 0.25407E+00_JPRB, 0.25619E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.26034E+00_JPRB, 0.26244E+00_JPRB, 0.26457E+00_JPRB, 0.26675E+00_JPRB, 0.26962E+00_JPRB, 0.27201E+00_JPRB, 0.27472E+00_JPRB ,&
&  0.27729E+00_JPRB, 0.28017E+00_JPRB, 0.28308E+00_JPRB, 0.28564E+00_JPRB, 0.28825E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB, 0.26553E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB, 0.34660E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.35322E+00_JPRB, 0.35329E+00_JPRB, 0.35338E+00_JPRB, 0.35356E+00_JPRB, 0.35353E+00_JPRB, 0.35335E+00_JPRB, 0.35302E+00_JPRB ,&
&  0.35263E+00_JPRB, 0.35218E+00_JPRB, 0.35149E+00_JPRB, 0.35081E+00_JPRB, 0.35016E+00_JPRB, 0.34950E+00_JPRB, 0.34854E+00_JPRB ,&
&  0.34747E+00_JPRB, 0.34623E+00_JPRB, 0.34498E+00_JPRB, 0.34389E+00_JPRB, 0.34344E+00_JPRB, 0.34313E+00_JPRB, 0.34290E+00_JPRB ,&
&  0.34263E+00_JPRB, 0.34246E+00_JPRB, 0.34232E+00_JPRB, 0.34221E+00_JPRB, 0.34219E+00_JPRB, 0.34205E+00_JPRB, 0.34192E+00_JPRB ,&
&  0.34177E+00_JPRB, 0.34170E+00_JPRB, 0.34149E+00_JPRB, 0.34132E+00_JPRB, 0.34108E+00_JPRB, 0.34112E+00_JPRB, 0.34106E+00_JPRB ,&
&  0.34112E+00_JPRB, 0.34109E+00_JPRB, 0.34110E+00_JPRB, 0.34107E+00_JPRB, 0.34111E+00_JPRB, 0.34098E+00_JPRB, 0.34083E+00_JPRB ,&
&  0.34068E+00_JPRB, 0.34068E+00_JPRB, 0.34107E+00_JPRB, 0.34172E+00_JPRB, 0.34236E+00_JPRB, 0.34308E+00_JPRB, 0.34374E+00_JPRB ,&
&  0.34442E+00_JPRB, 0.34496E+00_JPRB, 0.34572E+00_JPRB, 0.34644E+00_JPRB, 0.34726E+00_JPRB, 0.34798E+00_JPRB, 0.34880E+00_JPRB ,&
&  0.34955E+00_JPRB, 0.35055E+00_JPRB, 0.35178E+00_JPRB, 0.35312E+00_JPRB, 0.35441E+00_JPRB, 0.35557E+00_JPRB, 0.35681E+00_JPRB ,&
&  0.35812E+00_JPRB, 0.35937E+00_JPRB, 0.36030E+00_JPRB, 0.36133E+00_JPRB, 0.36239E+00_JPRB, 0.36345E+00_JPRB, 0.36420E+00_JPRB ,&
&  0.36498E+00_JPRB, 0.36586E+00_JPRB, 0.36677E+00_JPRB, 0.36703E+00_JPRB, 0.36720E+00_JPRB, 0.36727E+00_JPRB, 0.36711E+00_JPRB ,&
&  0.36646E+00_JPRB, 0.36578E+00_JPRB, 0.36510E+00_JPRB, 0.36438E+00_JPRB, 0.36339E+00_JPRB, 0.36228E+00_JPRB, 0.36100E+00_JPRB ,&
&  0.35967E+00_JPRB, 0.35807E+00_JPRB, 0.35657E+00_JPRB, 0.35552E+00_JPRB, 0.35491E+00_JPRB, 0.35425E+00_JPRB, 0.35363E+00_JPRB ,&
&  0.35312E+00_JPRB, 0.35306E+00_JPRB, 0.35310E+00_JPRB, 0.35315E+00_JPRB, 0.35325E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.35774E+00_JPRB, 0.35799E+00_JPRB, 0.35843E+00_JPRB, 0.35868E+00_JPRB, 0.35821E+00_JPRB, 0.35781E+00_JPRB, 0.35738E+00_JPRB ,&
&  0.35725E+00_JPRB, 0.35717E+00_JPRB, 0.35659E+00_JPRB, 0.35570E+00_JPRB, 0.35480E+00_JPRB, 0.35390E+00_JPRB, 0.35286E+00_JPRB ,&
&  0.35195E+00_JPRB, 0.35098E+00_JPRB, 0.34973E+00_JPRB, 0.34736E+00_JPRB, 0.34549E+00_JPRB, 0.34443E+00_JPRB, 0.34445E+00_JPRB ,&
&  0.34414E+00_JPRB, 0.34347E+00_JPRB, 0.34215E+00_JPRB, 0.34064E+00_JPRB, 0.33895E+00_JPRB, 0.33729E+00_JPRB, 0.33552E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33257E+00_JPRB, 0.33152E+00_JPRB, 0.33049E+00_JPRB, 0.33066E+00_JPRB, 0.33084E+00_JPRB, 0.33102E+00_JPRB ,&
&  0.33119E+00_JPRB, 0.33115E+00_JPRB, 0.33178E+00_JPRB, 0.33244E+00_JPRB, 0.33312E+00_JPRB, 0.33406E+00_JPRB, 0.33438E+00_JPRB ,&
&  0.33408E+00_JPRB, 0.33396E+00_JPRB, 0.33533E+00_JPRB, 0.33621E+00_JPRB, 0.33681E+00_JPRB, 0.33723E+00_JPRB, 0.33867E+00_JPRB ,&
&  0.34003E+00_JPRB, 0.34131E+00_JPRB, 0.34276E+00_JPRB, 0.34495E+00_JPRB, 0.34676E+00_JPRB, 0.34868E+00_JPRB, 0.35090E+00_JPRB ,&
&  0.35314E+00_JPRB, 0.35559E+00_JPRB, 0.35815E+00_JPRB, 0.36071E+00_JPRB, 0.36304E+00_JPRB, 0.36504E+00_JPRB, 0.36740E+00_JPRB ,&
&  0.36990E+00_JPRB, 0.37279E+00_JPRB, 0.37455E+00_JPRB, 0.37643E+00_JPRB, 0.37854E+00_JPRB, 0.38159E+00_JPRB, 0.38271E+00_JPRB ,&
&  0.38372E+00_JPRB, 0.38465E+00_JPRB, 0.38571E+00_JPRB, 0.38613E+00_JPRB, 0.38654E+00_JPRB, 0.38696E+00_JPRB, 0.38724E+00_JPRB ,&
&  0.38709E+00_JPRB, 0.38637E+00_JPRB, 0.38512E+00_JPRB, 0.38342E+00_JPRB, 0.38139E+00_JPRB, 0.37941E+00_JPRB, 0.37704E+00_JPRB ,&
&  0.37499E+00_JPRB, 0.37232E+00_JPRB, 0.36979E+00_JPRB, 0.36819E+00_JPRB, 0.36736E+00_JPRB, 0.36540E+00_JPRB, 0.36320E+00_JPRB ,&
&  0.36127E+00_JPRB, 0.35983E+00_JPRB, 0.35908E+00_JPRB, 0.35839E+00_JPRB, 0.35805E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.36228E+00_JPRB, 0.36254E+00_JPRB, 0.36282E+00_JPRB, 0.36290E+00_JPRB, 0.36218E+00_JPRB, 0.36168E+00_JPRB, 0.36109E+00_JPRB ,&
&  0.36055E+00_JPRB, 0.35965E+00_JPRB, 0.35894E+00_JPRB, 0.35837E+00_JPRB, 0.35785E+00_JPRB, 0.35758E+00_JPRB, 0.35725E+00_JPRB ,&
&  0.35714E+00_JPRB, 0.35672E+00_JPRB, 0.35624E+00_JPRB, 0.35482E+00_JPRB, 0.35245E+00_JPRB, 0.34921E+00_JPRB, 0.34651E+00_JPRB ,&
&  0.34368E+00_JPRB, 0.34080E+00_JPRB, 0.33743E+00_JPRB, 0.33385E+00_JPRB, 0.33048E+00_JPRB, 0.31007E+00_JPRB, 0.30367E+00_JPRB ,&
&  0.30334E+00_JPRB, 0.30360E+00_JPRB, 0.30602E+00_JPRB, 0.30943E+00_JPRB, 0.31299E+00_JPRB, 0.30911E+00_JPRB, 0.30785E+00_JPRB ,&
&  0.29518E+00_JPRB, 0.29624E+00_JPRB, 0.29742E+00_JPRB, 0.29889E+00_JPRB, 0.30244E+00_JPRB, 0.30576E+00_JPRB, 0.31155E+00_JPRB ,&
&  0.32460E+00_JPRB, 0.32647E+00_JPRB, 0.32816E+00_JPRB, 0.33023E+00_JPRB, 0.33207E+00_JPRB, 0.33377E+00_JPRB, 0.33574E+00_JPRB ,&
&  0.33705E+00_JPRB, 0.33846E+00_JPRB, 0.34020E+00_JPRB, 0.34246E+00_JPRB, 0.34527E+00_JPRB, 0.34848E+00_JPRB, 0.35194E+00_JPRB ,&
&  0.35532E+00_JPRB, 0.35931E+00_JPRB, 0.36348E+00_JPRB, 0.36778E+00_JPRB, 0.37211E+00_JPRB, 0.37634E+00_JPRB, 0.38086E+00_JPRB ,&
&  0.38495E+00_JPRB, 0.38926E+00_JPRB, 0.39214E+00_JPRB, 0.39536E+00_JPRB, 0.39743E+00_JPRB, 0.40010E+00_JPRB, 0.40069E+00_JPRB ,&
&  0.40153E+00_JPRB, 0.40046E+00_JPRB, 0.40023E+00_JPRB, 0.39945E+00_JPRB, 0.39867E+00_JPRB, 0.39830E+00_JPRB, 0.39762E+00_JPRB ,&
&  0.39725E+00_JPRB, 0.39641E+00_JPRB, 0.39554E+00_JPRB, 0.39396E+00_JPRB, 0.39135E+00_JPRB, 0.38827E+00_JPRB, 0.38517E+00_JPRB ,&
&  0.38327E+00_JPRB, 0.38223E+00_JPRB, 0.38160E+00_JPRB, 0.38145E+00_JPRB, 0.38030E+00_JPRB, 0.37808E+00_JPRB, 0.37516E+00_JPRB ,&
&  0.37245E+00_JPRB, 0.36945E+00_JPRB, 0.36667E+00_JPRB, 0.36439E+00_JPRB, 0.36300E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.36808E+00_JPRB, 0.36881E+00_JPRB, 0.36933E+00_JPRB, 0.36970E+00_JPRB, 0.36925E+00_JPRB, 0.36888E+00_JPRB, 0.36825E+00_JPRB ,&
&  0.36752E+00_JPRB, 0.36675E+00_JPRB, 0.36593E+00_JPRB, 0.36531E+00_JPRB, 0.36450E+00_JPRB, 0.36401E+00_JPRB, 0.36302E+00_JPRB ,&
&  0.36264E+00_JPRB, 0.36144E+00_JPRB, 0.35991E+00_JPRB, 0.35667E+00_JPRB, 0.35352E+00_JPRB, 0.34980E+00_JPRB, 0.34437E+00_JPRB ,&
&  0.33861E+00_JPRB, 0.33343E+00_JPRB, 0.31766E+00_JPRB, 0.31266E+00_JPRB, 0.30399E+00_JPRB, 0.29908E+00_JPRB, 0.29598E+00_JPRB ,&
&  0.29285E+00_JPRB, 0.29102E+00_JPRB, 0.28552E+00_JPRB, 0.28039E+00_JPRB, 0.27578E+00_JPRB, 0.26838E+00_JPRB, 0.26396E+00_JPRB ,&
&  0.25909E+00_JPRB, 0.25681E+00_JPRB, 0.25755E+00_JPRB, 0.26040E+00_JPRB, 0.26737E+00_JPRB, 0.27593E+00_JPRB, 0.28727E+00_JPRB ,&
&  0.30240E+00_JPRB, 0.31564E+00_JPRB, 0.32846E+00_JPRB, 0.32866E+00_JPRB, 0.33157E+00_JPRB, 0.33417E+00_JPRB, 0.33568E+00_JPRB ,&
&  0.33250E+00_JPRB, 0.33152E+00_JPRB, 0.33721E+00_JPRB, 0.33944E+00_JPRB, 0.34252E+00_JPRB, 0.34634E+00_JPRB, 0.35044E+00_JPRB ,&
&  0.35499E+00_JPRB, 0.36021E+00_JPRB, 0.36590E+00_JPRB, 0.37242E+00_JPRB, 0.38044E+00_JPRB, 0.38784E+00_JPRB, 0.39378E+00_JPRB ,&
&  0.39863E+00_JPRB, 0.40481E+00_JPRB, 0.40884E+00_JPRB, 0.41264E+00_JPRB, 0.41622E+00_JPRB, 0.42190E+00_JPRB, 0.42636E+00_JPRB ,&
&  0.42927E+00_JPRB, 0.42791E+00_JPRB, 0.42434E+00_JPRB, 0.41650E+00_JPRB, 0.41268E+00_JPRB, 0.41604E+00_JPRB, 0.41441E+00_JPRB ,&
&  0.41359E+00_JPRB, 0.41141E+00_JPRB, 0.40952E+00_JPRB, 0.40677E+00_JPRB, 0.40392E+00_JPRB, 0.39979E+00_JPRB, 0.39550E+00_JPRB ,&
&  0.39236E+00_JPRB, 0.39173E+00_JPRB, 0.39141E+00_JPRB, 0.39122E+00_JPRB, 0.38915E+00_JPRB, 0.38679E+00_JPRB, 0.38400E+00_JPRB ,&
&  0.38086E+00_JPRB, 0.37722E+00_JPRB, 0.37344E+00_JPRB, 0.37040E+00_JPRB, 0.36853E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.37951E+00_JPRB, 0.38003E+00_JPRB, 0.37963E+00_JPRB, 0.37959E+00_JPRB, 0.37884E+00_JPRB, 0.37836E+00_JPRB, 0.37723E+00_JPRB ,&
&  0.37596E+00_JPRB, 0.37472E+00_JPRB, 0.37332E+00_JPRB, 0.37199E+00_JPRB, 0.37051E+00_JPRB, 0.36914E+00_JPRB, 0.36826E+00_JPRB ,&
&  0.36739E+00_JPRB, 0.36567E+00_JPRB, 0.36278E+00_JPRB, 0.35910E+00_JPRB, 0.35589E+00_JPRB, 0.35210E+00_JPRB, 0.34795E+00_JPRB ,&
&  0.34329E+00_JPRB, 0.33729E+00_JPRB, 0.32726E+00_JPRB, 0.32227E+00_JPRB, 0.31519E+00_JPRB, 0.31160E+00_JPRB, 0.30703E+00_JPRB ,&
&  0.30668E+00_JPRB, 0.28543E+00_JPRB, 0.28360E+00_JPRB, 0.27467E+00_JPRB, 0.26793E+00_JPRB, 0.26279E+00_JPRB, 0.25358E+00_JPRB ,&
&  0.24951E+00_JPRB, 0.24456E+00_JPRB, 0.24277E+00_JPRB, 0.24523E+00_JPRB, 0.25004E+00_JPRB, 0.26000E+00_JPRB, 0.27320E+00_JPRB ,&
&  0.29013E+00_JPRB, 0.32332E+00_JPRB, 0.33094E+00_JPRB, 0.33572E+00_JPRB, 0.34043E+00_JPRB, 0.34368E+00_JPRB, 0.33881E+00_JPRB ,&
&  0.33683E+00_JPRB, 0.33457E+00_JPRB, 0.33449E+00_JPRB, 0.33657E+00_JPRB, 0.34205E+00_JPRB, 0.34447E+00_JPRB, 0.34870E+00_JPRB ,&
&  0.35330E+00_JPRB, 0.36036E+00_JPRB, 0.36815E+00_JPRB, 0.37818E+00_JPRB, 0.39195E+00_JPRB, 0.40489E+00_JPRB, 0.41616E+00_JPRB ,&
&  0.42413E+00_JPRB, 0.42833E+00_JPRB, 0.42832E+00_JPRB, 0.43188E+00_JPRB, 0.43724E+00_JPRB, 0.44668E+00_JPRB, 0.45240E+00_JPRB ,&
&  0.45623E+00_JPRB, 0.45990E+00_JPRB, 0.45876E+00_JPRB, 0.45527E+00_JPRB, 0.44750E+00_JPRB, 0.43977E+00_JPRB, 0.43599E+00_JPRB ,&
&  0.43725E+00_JPRB, 0.43385E+00_JPRB, 0.42879E+00_JPRB, 0.42184E+00_JPRB, 0.41746E+00_JPRB, 0.41182E+00_JPRB, 0.40685E+00_JPRB ,&
&  0.40306E+00_JPRB, 0.40152E+00_JPRB, 0.39903E+00_JPRB, 0.39772E+00_JPRB, 0.39476E+00_JPRB, 0.39290E+00_JPRB, 0.39106E+00_JPRB ,&
&  0.38797E+00_JPRB, 0.38539E+00_JPRB, 0.38165E+00_JPRB, 0.37955E+00_JPRB, 0.37962E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.39305E+00_JPRB, 0.39305E+00_JPRB, 0.39290E+00_JPRB, 0.39244E+00_JPRB, 0.39160E+00_JPRB, 0.39131E+00_JPRB, 0.39151E+00_JPRB ,&
&  0.39046E+00_JPRB, 0.38983E+00_JPRB, 0.38871E+00_JPRB, 0.38762E+00_JPRB, 0.38624E+00_JPRB, 0.38540E+00_JPRB, 0.38445E+00_JPRB ,&
&  0.38258E+00_JPRB, 0.37942E+00_JPRB, 0.37501E+00_JPRB, 0.37058E+00_JPRB, 0.36624E+00_JPRB, 0.36328E+00_JPRB, 0.35901E+00_JPRB ,&
&  0.35468E+00_JPRB, 0.34896E+00_JPRB, 0.34032E+00_JPRB, 0.33350E+00_JPRB, 0.33086E+00_JPRB, 0.32567E+00_JPRB, 0.32586E+00_JPRB ,&
&  0.32346E+00_JPRB, 0.31992E+00_JPRB, 0.31093E+00_JPRB, 0.30783E+00_JPRB, 0.30225E+00_JPRB, 0.27145E+00_JPRB, 0.26371E+00_JPRB ,&
&  0.25342E+00_JPRB, 0.24290E+00_JPRB, 0.23895E+00_JPRB, 0.23788E+00_JPRB, 0.24052E+00_JPRB, 0.25058E+00_JPRB, 0.26472E+00_JPRB ,&
&  0.28200E+00_JPRB, 0.32607E+00_JPRB, 0.33800E+00_JPRB, 0.34805E+00_JPRB, 0.35601E+00_JPRB, 0.36029E+00_JPRB, 0.35949E+00_JPRB ,&
&  0.35812E+00_JPRB, 0.35749E+00_JPRB, 0.35965E+00_JPRB, 0.36315E+00_JPRB, 0.36420E+00_JPRB, 0.36095E+00_JPRB, 0.36152E+00_JPRB ,&
&  0.35843E+00_JPRB, 0.36268E+00_JPRB, 0.37215E+00_JPRB, 0.38524E+00_JPRB, 0.40126E+00_JPRB, 0.41195E+00_JPRB, 0.42664E+00_JPRB ,&
&  0.43974E+00_JPRB, 0.44749E+00_JPRB, 0.45997E+00_JPRB, 0.47158E+00_JPRB, 0.47784E+00_JPRB, 0.48150E+00_JPRB, 0.48106E+00_JPRB ,&
&  0.47852E+00_JPRB, 0.48903E+00_JPRB, 0.49444E+00_JPRB, 0.49052E+00_JPRB, 0.48324E+00_JPRB, 0.47150E+00_JPRB, 0.46087E+00_JPRB ,&
&  0.45591E+00_JPRB, 0.45752E+00_JPRB, 0.45272E+00_JPRB, 0.44066E+00_JPRB, 0.43149E+00_JPRB, 0.42382E+00_JPRB, 0.41761E+00_JPRB ,&
&  0.41305E+00_JPRB, 0.41062E+00_JPRB, 0.40696E+00_JPRB, 0.40449E+00_JPRB, 0.40237E+00_JPRB, 0.40078E+00_JPRB, 0.39985E+00_JPRB ,&
&  0.39866E+00_JPRB, 0.39843E+00_JPRB, 0.39718E+00_JPRB, 0.39506E+00_JPRB, 0.39455E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.40895E+00_JPRB, 0.41079E+00_JPRB, 0.41251E+00_JPRB, 0.41275E+00_JPRB, 0.41318E+00_JPRB, 0.41336E+00_JPRB, 0.41123E+00_JPRB ,&
&  0.40692E+00_JPRB, 0.40351E+00_JPRB, 0.39972E+00_JPRB, 0.39860E+00_JPRB, 0.39763E+00_JPRB, 0.39704E+00_JPRB, 0.39654E+00_JPRB ,&
&  0.39446E+00_JPRB, 0.38620E+00_JPRB, 0.38059E+00_JPRB, 0.37787E+00_JPRB, 0.37513E+00_JPRB, 0.37374E+00_JPRB, 0.37213E+00_JPRB ,&
&  0.36781E+00_JPRB, 0.36166E+00_JPRB, 0.35564E+00_JPRB, 0.34510E+00_JPRB, 0.33715E+00_JPRB, 0.33647E+00_JPRB, 0.33956E+00_JPRB ,&
&  0.33636E+00_JPRB, 0.32626E+00_JPRB, 0.32628E+00_JPRB, 0.32395E+00_JPRB, 0.31156E+00_JPRB, 0.30814E+00_JPRB, 0.28218E+00_JPRB ,&
&  0.26331E+00_JPRB, 0.24821E+00_JPRB, 0.24268E+00_JPRB, 0.23934E+00_JPRB, 0.24323E+00_JPRB, 0.25320E+00_JPRB, 0.27623E+00_JPRB ,&
&  0.30171E+00_JPRB, 0.32976E+00_JPRB, 0.33987E+00_JPRB, 0.35281E+00_JPRB, 0.36385E+00_JPRB, 0.37774E+00_JPRB, 0.39058E+00_JPRB ,&
&  0.39747E+00_JPRB, 0.40200E+00_JPRB, 0.40739E+00_JPRB, 0.40995E+00_JPRB, 0.41604E+00_JPRB, 0.41992E+00_JPRB, 0.41962E+00_JPRB ,&
&  0.42337E+00_JPRB, 0.42341E+00_JPRB, 0.42900E+00_JPRB, 0.43605E+00_JPRB, 0.44493E+00_JPRB, 0.44826E+00_JPRB, 0.45075E+00_JPRB ,&
&  0.45666E+00_JPRB, 0.47398E+00_JPRB, 0.49743E+00_JPRB, 0.52539E+00_JPRB, 0.53112E+00_JPRB, 0.52621E+00_JPRB, 0.51099E+00_JPRB ,&
&  0.49361E+00_JPRB, 0.50707E+00_JPRB, 0.57687E+00_JPRB, 0.56206E+00_JPRB, 0.55056E+00_JPRB, 0.53688E+00_JPRB, 0.51025E+00_JPRB ,&
&  0.48085E+00_JPRB, 0.46932E+00_JPRB, 0.46012E+00_JPRB, 0.44453E+00_JPRB, 0.43247E+00_JPRB, 0.42252E+00_JPRB, 0.41646E+00_JPRB ,&
&  0.41289E+00_JPRB, 0.41284E+00_JPRB, 0.41069E+00_JPRB, 0.40996E+00_JPRB, 0.40798E+00_JPRB, 0.40622E+00_JPRB, 0.40646E+00_JPRB ,&
&  0.40646E+00_JPRB, 0.40789E+00_JPRB, 0.40932E+00_JPRB, 0.40993E+00_JPRB, 0.40971E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.42518E+00_JPRB, 0.43176E+00_JPRB, 0.43681E+00_JPRB, 0.44510E+00_JPRB, 0.45234E+00_JPRB, 0.45137E+00_JPRB, 0.44584E+00_JPRB ,&
&  0.43439E+00_JPRB, 0.42050E+00_JPRB, 0.41165E+00_JPRB, 0.40480E+00_JPRB, 0.40121E+00_JPRB, 0.39951E+00_JPRB, 0.39786E+00_JPRB ,&
&  0.39643E+00_JPRB, 0.39459E+00_JPRB, 0.38979E+00_JPRB, 0.38261E+00_JPRB, 0.37778E+00_JPRB, 0.37650E+00_JPRB, 0.37573E+00_JPRB ,&
&  0.37380E+00_JPRB, 0.36979E+00_JPRB, 0.36426E+00_JPRB, 0.35425E+00_JPRB, 0.34575E+00_JPRB, 0.34693E+00_JPRB, 0.34271E+00_JPRB ,&
&  0.33822E+00_JPRB, 0.33564E+00_JPRB, 0.33395E+00_JPRB, 0.33088E+00_JPRB, 0.32721E+00_JPRB, 0.31821E+00_JPRB, 0.29842E+00_JPRB ,&
&  0.27687E+00_JPRB, 0.25932E+00_JPRB, 0.24868E+00_JPRB, 0.24569E+00_JPRB, 0.25178E+00_JPRB, 0.26823E+00_JPRB, 0.29327E+00_JPRB ,&
&  0.32669E+00_JPRB, 0.34153E+00_JPRB, 0.35705E+00_JPRB, 0.37594E+00_JPRB, 0.39258E+00_JPRB, 0.41153E+00_JPRB, 0.42789E+00_JPRB ,&
&  0.44292E+00_JPRB, 0.46195E+00_JPRB, 0.48450E+00_JPRB, 0.51094E+00_JPRB, 0.54375E+00_JPRB, 0.52496E+00_JPRB, 0.54209E+00_JPRB ,&
&  0.61125E+00_JPRB, 0.61376E+00_JPRB, 0.58928E+00_JPRB, 0.55975E+00_JPRB, 0.53909E+00_JPRB, 0.52980E+00_JPRB, 0.52769E+00_JPRB ,&
&  0.52959E+00_JPRB, 0.53780E+00_JPRB, 0.55363E+00_JPRB, 0.57215E+00_JPRB, 0.57553E+00_JPRB, 0.56848E+00_JPRB, 0.54620E+00_JPRB ,&
&  0.52631E+00_JPRB, 0.54197E+00_JPRB, 0.69779E+00_JPRB, 0.62743E+00_JPRB, 0.56960E+00_JPRB, 0.54101E+00_JPRB, 0.50862E+00_JPRB ,&
&  0.48482E+00_JPRB, 0.46763E+00_JPRB, 0.45180E+00_JPRB, 0.44071E+00_JPRB, 0.42519E+00_JPRB, 0.40909E+00_JPRB, 0.39530E+00_JPRB ,&
&  0.39196E+00_JPRB, 0.39267E+00_JPRB, 0.39467E+00_JPRB, 0.39694E+00_JPRB, 0.39736E+00_JPRB, 0.39941E+00_JPRB, 0.40244E+00_JPRB ,&
&  0.40223E+00_JPRB, 0.40382E+00_JPRB, 0.40882E+00_JPRB, 0.41407E+00_JPRB, 0.41931E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.41461E+00_JPRB, 0.43301E+00_JPRB, 0.45844E+00_JPRB, 0.47387E+00_JPRB, 0.48120E+00_JPRB, 0.46749E+00_JPRB, 0.45310E+00_JPRB ,&
&  0.43782E+00_JPRB, 0.41566E+00_JPRB, 0.40432E+00_JPRB, 0.39704E+00_JPRB, 0.39263E+00_JPRB, 0.38939E+00_JPRB, 0.38678E+00_JPRB ,&
&  0.38451E+00_JPRB, 0.38177E+00_JPRB, 0.38020E+00_JPRB, 0.38117E+00_JPRB, 0.38235E+00_JPRB, 0.38036E+00_JPRB, 0.37682E+00_JPRB ,&
&  0.37372E+00_JPRB, 0.37013E+00_JPRB, 0.36249E+00_JPRB, 0.35186E+00_JPRB, 0.34816E+00_JPRB, 0.35337E+00_JPRB, 0.35180E+00_JPRB ,&
&  0.34731E+00_JPRB, 0.34139E+00_JPRB, 0.34014E+00_JPRB, 0.34079E+00_JPRB, 0.33764E+00_JPRB, 0.33040E+00_JPRB, 0.31257E+00_JPRB ,&
&  0.28881E+00_JPRB, 0.26996E+00_JPRB, 0.26360E+00_JPRB, 0.27402E+00_JPRB, 0.29582E+00_JPRB, 0.33032E+00_JPRB, 0.34314E+00_JPRB ,&
&  0.35464E+00_JPRB, 0.36847E+00_JPRB, 0.38289E+00_JPRB, 0.40367E+00_JPRB, 0.42722E+00_JPRB, 0.44779E+00_JPRB, 0.46592E+00_JPRB ,&
&  0.49107E+00_JPRB, 0.51617E+00_JPRB, 0.54782E+00_JPRB, 0.51627E+00_JPRB, 0.52844E+00_JPRB, 0.57176E+00_JPRB, 0.59304E+00_JPRB ,&
&  0.60460E+00_JPRB, 0.62469E+00_JPRB, 0.63975E+00_JPRB, 0.65977E+00_JPRB, 0.64731E+00_JPRB, 0.62877E+00_JPRB, 0.60936E+00_JPRB ,&
&  0.60153E+00_JPRB, 0.59242E+00_JPRB, 0.59040E+00_JPRB, 0.60787E+00_JPRB, 0.62472E+00_JPRB, 0.62199E+00_JPRB, 0.60220E+00_JPRB ,&
&  0.57254E+00_JPRB, 0.52983E+00_JPRB, 0.49120E+00_JPRB, 0.46634E+00_JPRB, 0.46562E+00_JPRB, 0.47472E+00_JPRB, 0.47724E+00_JPRB ,&
&  0.47348E+00_JPRB, 0.46908E+00_JPRB, 0.46043E+00_JPRB, 0.45359E+00_JPRB, 0.42275E+00_JPRB, 0.40044E+00_JPRB, 0.38319E+00_JPRB ,&
&  0.37802E+00_JPRB, 0.37625E+00_JPRB, 0.37706E+00_JPRB, 0.38423E+00_JPRB, 0.38508E+00_JPRB, 0.38782E+00_JPRB, 0.39001E+00_JPRB ,&
&  0.38936E+00_JPRB, 0.38839E+00_JPRB, 0.39215E+00_JPRB, 0.39701E+00_JPRB, 0.40405E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.46675E+00_JPRB, 0.48087E+00_JPRB, 0.49265E+00_JPRB, 0.49994E+00_JPRB, 0.49120E+00_JPRB, 0.48652E+00_JPRB, 0.48272E+00_JPRB ,&
&  0.48490E+00_JPRB, 0.44193E+00_JPRB, 0.41678E+00_JPRB, 0.39655E+00_JPRB, 0.38245E+00_JPRB, 0.36895E+00_JPRB, 0.36434E+00_JPRB ,&
&  0.36492E+00_JPRB, 0.36723E+00_JPRB, 0.36895E+00_JPRB, 0.37020E+00_JPRB, 0.37155E+00_JPRB, 0.37163E+00_JPRB, 0.37262E+00_JPRB ,&
&  0.37479E+00_JPRB, 0.37460E+00_JPRB, 0.36705E+00_JPRB, 0.36067E+00_JPRB, 0.35900E+00_JPRB, 0.36046E+00_JPRB, 0.35746E+00_JPRB ,&
&  0.35360E+00_JPRB, 0.34934E+00_JPRB, 0.34752E+00_JPRB, 0.34872E+00_JPRB, 0.34698E+00_JPRB, 0.34055E+00_JPRB, 0.32031E+00_JPRB ,&
&  0.29377E+00_JPRB, 0.28508E+00_JPRB, 0.29915E+00_JPRB, 0.34346E+00_JPRB, 0.35764E+00_JPRB, 0.36696E+00_JPRB, 0.37976E+00_JPRB ,&
&  0.37672E+00_JPRB, 0.37932E+00_JPRB, 0.39762E+00_JPRB, 0.44094E+00_JPRB, 0.46193E+00_JPRB, 0.48022E+00_JPRB, 0.49995E+00_JPRB ,&
&  0.53201E+00_JPRB, 0.55501E+00_JPRB, 0.58015E+00_JPRB, 0.54679E+00_JPRB, 0.59734E+00_JPRB, 0.72544E+00_JPRB, 0.69887E+00_JPRB ,&
&  0.69565E+00_JPRB, 0.69744E+00_JPRB, 0.72415E+00_JPRB, 0.73432E+00_JPRB, 0.72183E+00_JPRB, 0.70124E+00_JPRB, 0.68786E+00_JPRB ,&
&  0.66862E+00_JPRB, 0.65723E+00_JPRB, 0.65853E+00_JPRB, 0.68548E+00_JPRB, 0.67972E+00_JPRB, 0.66501E+00_JPRB, 0.64488E+00_JPRB ,&
&  0.61258E+00_JPRB, 0.56847E+00_JPRB, 0.52767E+00_JPRB, 0.49742E+00_JPRB, 0.48911E+00_JPRB, 0.48983E+00_JPRB, 0.48655E+00_JPRB ,&
&  0.47691E+00_JPRB, 0.47595E+00_JPRB, 0.47868E+00_JPRB, 0.47891E+00_JPRB, 0.44717E+00_JPRB, 0.41103E+00_JPRB, 0.38439E+00_JPRB ,&
&  0.36610E+00_JPRB, 0.35592E+00_JPRB, 0.34837E+00_JPRB, 0.35343E+00_JPRB, 0.36308E+00_JPRB, 0.37393E+00_JPRB, 0.37545E+00_JPRB ,&
&  0.37876E+00_JPRB, 0.38237E+00_JPRB, 0.39442E+00_JPRB, 0.41856E+00_JPRB, 0.44411E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.49148E+00_JPRB, 0.51016E+00_JPRB, 0.52530E+00_JPRB, 0.52162E+00_JPRB, 0.51108E+00_JPRB, 0.50089E+00_JPRB, 0.49396E+00_JPRB ,&
&  0.49764E+00_JPRB, 0.43098E+00_JPRB, 0.40592E+00_JPRB, 0.39006E+00_JPRB, 0.38096E+00_JPRB, 0.36473E+00_JPRB, 0.35018E+00_JPRB ,&
&  0.34754E+00_JPRB, 0.35346E+00_JPRB, 0.36314E+00_JPRB, 0.36925E+00_JPRB, 0.37431E+00_JPRB, 0.37757E+00_JPRB, 0.37881E+00_JPRB ,&
&  0.38054E+00_JPRB, 0.38233E+00_JPRB, 0.37854E+00_JPRB, 0.37318E+00_JPRB, 0.36994E+00_JPRB, 0.36932E+00_JPRB, 0.36903E+00_JPRB ,&
&  0.36789E+00_JPRB, 0.36559E+00_JPRB, 0.35898E+00_JPRB, 0.35660E+00_JPRB, 0.35486E+00_JPRB, 0.35628E+00_JPRB, 0.35549E+00_JPRB ,&
&  0.31037E+00_JPRB, 0.31352E+00_JPRB, 0.35482E+00_JPRB, 0.36244E+00_JPRB, 0.37364E+00_JPRB, 0.38020E+00_JPRB, 0.38640E+00_JPRB ,&
&  0.39732E+00_JPRB, 0.41760E+00_JPRB, 0.44559E+00_JPRB, 0.47857E+00_JPRB, 0.50532E+00_JPRB, 0.52525E+00_JPRB, 0.54204E+00_JPRB ,&
&  0.58277E+00_JPRB, 0.53513E+00_JPRB, 0.51585E+00_JPRB, 0.55997E+00_JPRB, 0.67553E+00_JPRB, 0.73619E+00_JPRB, 0.78587E+00_JPRB ,&
&  0.82384E+00_JPRB, 0.81100E+00_JPRB, 0.79937E+00_JPRB, 0.79915E+00_JPRB, 0.79036E+00_JPRB, 0.77633E+00_JPRB, 0.78014E+00_JPRB ,&
&  0.76740E+00_JPRB, 0.77018E+00_JPRB, 0.78064E+00_JPRB, 0.78622E+00_JPRB, 0.75210E+00_JPRB, 0.72690E+00_JPRB, 0.69902E+00_JPRB ,&
&  0.66556E+00_JPRB, 0.62717E+00_JPRB, 0.58381E+00_JPRB, 0.54925E+00_JPRB, 0.52506E+00_JPRB, 0.52255E+00_JPRB, 0.51872E+00_JPRB ,&
&  0.50841E+00_JPRB, 0.50311E+00_JPRB, 0.49668E+00_JPRB, 0.49036E+00_JPRB, 0.46960E+00_JPRB, 0.44878E+00_JPRB, 0.43742E+00_JPRB ,&
&  0.40462E+00_JPRB, 0.38395E+00_JPRB, 0.35822E+00_JPRB, 0.35749E+00_JPRB, 0.37242E+00_JPRB, 0.38692E+00_JPRB, 0.39634E+00_JPRB ,&
&  0.41260E+00_JPRB, 0.42276E+00_JPRB, 0.42825E+00_JPRB, 0.44070E+00_JPRB, 0.46300E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.50528E+00_JPRB, 0.51741E+00_JPRB, 0.52456E+00_JPRB, 0.52261E+00_JPRB, 0.52390E+00_JPRB, 0.51779E+00_JPRB, 0.52330E+00_JPRB ,&
&  0.55794E+00_JPRB, 0.51274E+00_JPRB, 0.54710E+00_JPRB, 0.44579E+00_JPRB, 0.39957E+00_JPRB, 0.37728E+00_JPRB, 0.35914E+00_JPRB ,&
&  0.35463E+00_JPRB, 0.36011E+00_JPRB, 0.36523E+00_JPRB, 0.37125E+00_JPRB, 0.38171E+00_JPRB, 0.38188E+00_JPRB, 0.38164E+00_JPRB ,&
&  0.38534E+00_JPRB, 0.38866E+00_JPRB, 0.39407E+00_JPRB, 0.38514E+00_JPRB, 0.38159E+00_JPRB, 0.38056E+00_JPRB, 0.38570E+00_JPRB ,&
&  0.37331E+00_JPRB, 0.37070E+00_JPRB, 0.37743E+00_JPRB, 0.37525E+00_JPRB, 0.37416E+00_JPRB, 0.37291E+00_JPRB, 0.37508E+00_JPRB ,&
&  0.38247E+00_JPRB, 0.36308E+00_JPRB, 0.38439E+00_JPRB, 0.39263E+00_JPRB, 0.39889E+00_JPRB, 0.40629E+00_JPRB, 0.41255E+00_JPRB ,&
&  0.42347E+00_JPRB, 0.44213E+00_JPRB, 0.46984E+00_JPRB, 0.49881E+00_JPRB, 0.53195E+00_JPRB, 0.56098E+00_JPRB, 0.58267E+00_JPRB ,&
&  0.63276E+00_JPRB, 0.55481E+00_JPRB, 0.64878E+00_JPRB, 0.69467E+00_JPRB, 0.80268E+00_JPRB, 0.94382E+00_JPRB, 0.98183E+00_JPRB ,&
&  0.10817E+01_JPRB, 0.10311E+01_JPRB, 0.99785E+00_JPRB, 0.95772E+00_JPRB, 0.91571E+00_JPRB, 0.88903E+00_JPRB, 0.87330E+00_JPRB ,&
&  0.89207E+00_JPRB, 0.90371E+00_JPRB, 0.96115E+00_JPRB, 0.92615E+00_JPRB, 0.85129E+00_JPRB, 0.80632E+00_JPRB, 0.76095E+00_JPRB ,&
&  0.72325E+00_JPRB, 0.69813E+00_JPRB, 0.67198E+00_JPRB, 0.61241E+00_JPRB, 0.59712E+00_JPRB, 0.58757E+00_JPRB, 0.55244E+00_JPRB ,&
&  0.53546E+00_JPRB, 0.50933E+00_JPRB, 0.48050E+00_JPRB, 0.48655E+00_JPRB, 0.46425E+00_JPRB, 0.44765E+00_JPRB, 0.44375E+00_JPRB ,&
&  0.42693E+00_JPRB, 0.39824E+00_JPRB, 0.40006E+00_JPRB, 0.40727E+00_JPRB, 0.40566E+00_JPRB, 0.40847E+00_JPRB, 0.43212E+00_JPRB ,&
&  0.43800E+00_JPRB, 0.44966E+00_JPRB, 0.47523E+00_JPRB, 0.48337E+00_JPRB, 0.49614E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.51295E+00_JPRB, 0.51094E+00_JPRB, 0.51202E+00_JPRB, 0.51551E+00_JPRB, 0.52910E+00_JPRB, 0.55336E+00_JPRB, 0.54271E+00_JPRB ,&
&  0.56806E+00_JPRB, 0.56964E+00_JPRB, 0.55947E+00_JPRB, 0.54887E+00_JPRB, 0.52596E+00_JPRB, 0.45629E+00_JPRB, 0.37517E+00_JPRB ,&
&  0.35163E+00_JPRB, 0.35628E+00_JPRB, 0.36717E+00_JPRB, 0.37240E+00_JPRB, 0.37956E+00_JPRB, 0.37904E+00_JPRB, 0.38026E+00_JPRB ,&
&  0.38551E+00_JPRB, 0.39121E+00_JPRB, 0.39650E+00_JPRB, 0.39750E+00_JPRB, 0.39126E+00_JPRB, 0.39588E+00_JPRB, 0.40853E+00_JPRB ,&
&  0.38867E+00_JPRB, 0.37746E+00_JPRB, 0.37458E+00_JPRB, 0.36671E+00_JPRB, 0.38167E+00_JPRB, 0.38823E+00_JPRB, 0.39532E+00_JPRB ,&
&  0.40409E+00_JPRB, 0.41281E+00_JPRB, 0.41537E+00_JPRB, 0.42159E+00_JPRB, 0.43155E+00_JPRB, 0.44885E+00_JPRB, 0.46636E+00_JPRB ,&
&  0.48675E+00_JPRB, 0.50805E+00_JPRB, 0.53551E+00_JPRB, 0.57216E+00_JPRB, 0.61846E+00_JPRB, 0.62238E+00_JPRB, 0.67237E+00_JPRB ,&
&  0.71777E+00_JPRB, 0.80526E+00_JPRB, 0.90947E+00_JPRB, 0.87955E+00_JPRB, 0.99707E+00_JPRB, 0.10992E+01_JPRB, 0.10958E+01_JPRB ,&
&  0.11355E+01_JPRB, 0.10786E+01_JPRB, 0.10997E+01_JPRB, 0.11182E+01_JPRB, 0.11287E+01_JPRB, 0.11046E+01_JPRB, 0.10466E+01_JPRB ,&
&  0.10301E+01_JPRB, 0.11574E+01_JPRB, 0.11625E+01_JPRB, 0.10876E+01_JPRB, 0.97783E+00_JPRB, 0.91152E+00_JPRB, 0.84078E+00_JPRB ,&
&  0.81660E+00_JPRB, 0.81794E+00_JPRB, 0.78837E+00_JPRB, 0.75672E+00_JPRB, 0.70458E+00_JPRB, 0.65007E+00_JPRB, 0.60668E+00_JPRB ,&
&  0.55782E+00_JPRB, 0.49396E+00_JPRB, 0.46468E+00_JPRB, 0.45961E+00_JPRB, 0.45634E+00_JPRB, 0.43390E+00_JPRB, 0.42867E+00_JPRB ,&
&  0.42973E+00_JPRB, 0.42585E+00_JPRB, 0.44333E+00_JPRB, 0.44269E+00_JPRB, 0.44195E+00_JPRB, 0.43934E+00_JPRB, 0.42881E+00_JPRB ,&
&  0.42418E+00_JPRB, 0.45664E+00_JPRB, 0.48069E+00_JPRB, 0.49749E+00_JPRB, 0.50677E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.51248E+00_JPRB, 0.50845E+00_JPRB, 0.50012E+00_JPRB, 0.50826E+00_JPRB, 0.53462E+00_JPRB, 0.55926E+00_JPRB, 0.57406E+00_JPRB ,&
&  0.57319E+00_JPRB, 0.57180E+00_JPRB, 0.56731E+00_JPRB, 0.55905E+00_JPRB, 0.54172E+00_JPRB, 0.52199E+00_JPRB, 0.46529E+00_JPRB ,&
&  0.39228E+00_JPRB, 0.38296E+00_JPRB, 0.36539E+00_JPRB, 0.36804E+00_JPRB, 0.38219E+00_JPRB, 0.38158E+00_JPRB, 0.38288E+00_JPRB ,&
&  0.38914E+00_JPRB, 0.39466E+00_JPRB, 0.39972E+00_JPRB, 0.39880E+00_JPRB, 0.39755E+00_JPRB, 0.42128E+00_JPRB, 0.41888E+00_JPRB ,&
&  0.39034E+00_JPRB, 0.37323E+00_JPRB, 0.36978E+00_JPRB, 0.37374E+00_JPRB, 0.38615E+00_JPRB, 0.40181E+00_JPRB, 0.41048E+00_JPRB ,&
&  0.41867E+00_JPRB, 0.42776E+00_JPRB, 0.43395E+00_JPRB, 0.44788E+00_JPRB, 0.46898E+00_JPRB, 0.49549E+00_JPRB, 0.51936E+00_JPRB ,&
&  0.53877E+00_JPRB, 0.56394E+00_JPRB, 0.59916E+00_JPRB, 0.64736E+00_JPRB, 0.68339E+00_JPRB, 0.72403E+00_JPRB, 0.79163E+00_JPRB ,&
&  0.85723E+00_JPRB, 0.95479E+00_JPRB, 0.10147E+01_JPRB, 0.10958E+01_JPRB, 0.12251E+01_JPRB, 0.12478E+01_JPRB, 0.12221E+01_JPRB ,&
&  0.12237E+01_JPRB, 0.11733E+01_JPRB, 0.13544E+01_JPRB, 0.13733E+01_JPRB, 0.13258E+01_JPRB, 0.13192E+01_JPRB, 0.12310E+01_JPRB ,&
&  0.11718E+01_JPRB, 0.11547E+01_JPRB, 0.11266E+01_JPRB, 0.10900E+01_JPRB, 0.10456E+01_JPRB, 0.10457E+01_JPRB, 0.97622E+00_JPRB ,&
&  0.97397E+00_JPRB, 0.94596E+00_JPRB, 0.80385E+00_JPRB, 0.70303E+00_JPRB, 0.61223E+00_JPRB, 0.62074E+00_JPRB, 0.60536E+00_JPRB ,&
&  0.55504E+00_JPRB, 0.48100E+00_JPRB, 0.47673E+00_JPRB, 0.46914E+00_JPRB, 0.47251E+00_JPRB, 0.47472E+00_JPRB, 0.45849E+00_JPRB ,&
&  0.46369E+00_JPRB, 0.48385E+00_JPRB, 0.48407E+00_JPRB, 0.49261E+00_JPRB, 0.48882E+00_JPRB, 0.47578E+00_JPRB, 0.43035E+00_JPRB ,&
&  0.42646E+00_JPRB, 0.45240E+00_JPRB, 0.47168E+00_JPRB, 0.49215E+00_JPRB, 0.50675E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.52457E+00_JPRB, 0.52387E+00_JPRB, 0.51698E+00_JPRB, 0.52352E+00_JPRB, 0.54768E+00_JPRB, 0.57228E+00_JPRB, 0.58643E+00_JPRB ,&
&  0.58612E+00_JPRB, 0.58779E+00_JPRB, 0.57927E+00_JPRB, 0.56926E+00_JPRB, 0.55715E+00_JPRB, 0.53620E+00_JPRB, 0.53510E+00_JPRB ,&
&  0.43518E+00_JPRB, 0.40177E+00_JPRB, 0.35902E+00_JPRB, 0.35367E+00_JPRB, 0.38872E+00_JPRB, 0.38868E+00_JPRB, 0.39312E+00_JPRB ,&
&  0.39378E+00_JPRB, 0.40207E+00_JPRB, 0.40134E+00_JPRB, 0.40090E+00_JPRB, 0.41498E+00_JPRB, 0.43833E+00_JPRB, 0.42599E+00_JPRB ,&
&  0.40376E+00_JPRB, 0.38793E+00_JPRB, 0.38682E+00_JPRB, 0.38887E+00_JPRB, 0.39741E+00_JPRB, 0.41334E+00_JPRB, 0.42611E+00_JPRB ,&
&  0.43800E+00_JPRB, 0.46221E+00_JPRB, 0.48382E+00_JPRB, 0.51049E+00_JPRB, 0.54151E+00_JPRB, 0.58240E+00_JPRB, 0.61394E+00_JPRB ,&
&  0.64648E+00_JPRB, 0.68674E+00_JPRB, 0.73231E+00_JPRB, 0.78842E+00_JPRB, 0.81843E+00_JPRB, 0.89016E+00_JPRB, 0.96993E+00_JPRB ,&
&  0.11176E+01_JPRB, 0.11717E+01_JPRB, 0.12049E+01_JPRB, 0.14343E+01_JPRB, 0.15351E+01_JPRB, 0.15153E+01_JPRB, 0.13992E+01_JPRB ,&
&  0.13385E+01_JPRB, 0.12888E+01_JPRB, 0.13149E+01_JPRB, 0.13612E+01_JPRB, 0.13479E+01_JPRB, 0.13805E+01_JPRB, 0.13083E+01_JPRB ,&
&  0.11775E+01_JPRB, 0.11182E+01_JPRB, 0.10872E+01_JPRB, 0.10975E+01_JPRB, 0.10488E+01_JPRB, 0.10619E+01_JPRB, 0.10221E+01_JPRB ,&
&  0.10314E+01_JPRB, 0.89114E+00_JPRB, 0.67743E+00_JPRB, 0.62162E+00_JPRB, 0.51689E+00_JPRB, 0.52844E+00_JPRB, 0.63623E+00_JPRB ,&
&  0.61596E+00_JPRB, 0.55746E+00_JPRB, 0.51805E+00_JPRB, 0.49650E+00_JPRB, 0.49364E+00_JPRB, 0.55133E+00_JPRB, 0.54395E+00_JPRB ,&
&  0.50427E+00_JPRB, 0.52860E+00_JPRB, 0.55613E+00_JPRB, 0.57332E+00_JPRB, 0.56708E+00_JPRB, 0.55230E+00_JPRB, 0.50951E+00_JPRB ,&
&  0.50498E+00_JPRB, 0.50189E+00_JPRB, 0.50642E+00_JPRB, 0.51601E+00_JPRB, 0.52185E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.55983E+00_JPRB, 0.55769E+00_JPRB, 0.55335E+00_JPRB, 0.55914E+00_JPRB, 0.57559E+00_JPRB, 0.59611E+00_JPRB, 0.60921E+00_JPRB ,&
&  0.60564E+00_JPRB, 0.60374E+00_JPRB, 0.59334E+00_JPRB, 0.58165E+00_JPRB, 0.56949E+00_JPRB, 0.55173E+00_JPRB, 0.54089E+00_JPRB ,&
&  0.54361E+00_JPRB, 0.45560E+00_JPRB, 0.38809E+00_JPRB, 0.35184E+00_JPRB, 0.37126E+00_JPRB, 0.37899E+00_JPRB, 0.39294E+00_JPRB ,&
&  0.39262E+00_JPRB, 0.41040E+00_JPRB, 0.41080E+00_JPRB, 0.41356E+00_JPRB, 0.44636E+00_JPRB, 0.45488E+00_JPRB, 0.44576E+00_JPRB ,&
&  0.44250E+00_JPRB, 0.42760E+00_JPRB, 0.42344E+00_JPRB, 0.43139E+00_JPRB, 0.44821E+00_JPRB, 0.43877E+00_JPRB, 0.46459E+00_JPRB ,&
&  0.49564E+00_JPRB, 0.52898E+00_JPRB, 0.55869E+00_JPRB, 0.60321E+00_JPRB, 0.65849E+00_JPRB, 0.70173E+00_JPRB, 0.74637E+00_JPRB ,&
&  0.78901E+00_JPRB, 0.81768E+00_JPRB, 0.83092E+00_JPRB, 0.87529E+00_JPRB, 0.95144E+00_JPRB, 0.10861E+01_JPRB, 0.10918E+01_JPRB ,&
&  0.11497E+01_JPRB, 0.11937E+01_JPRB, 0.11619E+01_JPRB, 0.14422E+01_JPRB, 0.16195E+01_JPRB, 0.16085E+01_JPRB, 0.15383E+01_JPRB ,&
&  0.14934E+01_JPRB, 0.14547E+01_JPRB, 0.15201E+01_JPRB, 0.15261E+01_JPRB, 0.15155E+01_JPRB, 0.14751E+01_JPRB, 0.13556E+01_JPRB ,&
&  0.12210E+01_JPRB, 0.11352E+01_JPRB, 0.10929E+01_JPRB, 0.10497E+01_JPRB, 0.10100E+01_JPRB, 0.97575E+00_JPRB, 0.10070E+01_JPRB ,&
&  0.10852E+01_JPRB, 0.64471E+00_JPRB, 0.52764E+00_JPRB, 0.59257E+00_JPRB, 0.51262E+00_JPRB, 0.52390E+00_JPRB, 0.57862E+00_JPRB ,&
&  0.56762E+00_JPRB, 0.55660E+00_JPRB, 0.56728E+00_JPRB, 0.55175E+00_JPRB, 0.56508E+00_JPRB, 0.67283E+00_JPRB, 0.67861E+00_JPRB ,&
&  0.63991E+00_JPRB, 0.61353E+00_JPRB, 0.65378E+00_JPRB, 0.65785E+00_JPRB, 0.65669E+00_JPRB, 0.64251E+00_JPRB, 0.61385E+00_JPRB ,&
&  0.58978E+00_JPRB, 0.57903E+00_JPRB, 0.56944E+00_JPRB, 0.56452E+00_JPRB, 0.56160E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.62803E+00_JPRB, 0.62697E+00_JPRB, 0.62572E+00_JPRB, 0.62363E+00_JPRB, 0.62812E+00_JPRB, 0.64271E+00_JPRB, 0.64971E+00_JPRB ,&
&  0.64195E+00_JPRB, 0.63381E+00_JPRB, 0.61755E+00_JPRB, 0.60219E+00_JPRB, 0.58739E+00_JPRB, 0.57341E+00_JPRB, 0.56150E+00_JPRB ,&
&  0.54943E+00_JPRB, 0.50844E+00_JPRB, 0.41517E+00_JPRB, 0.37202E+00_JPRB, 0.34759E+00_JPRB, 0.36309E+00_JPRB, 0.38099E+00_JPRB ,&
&  0.39270E+00_JPRB, 0.40771E+00_JPRB, 0.41832E+00_JPRB, 0.45832E+00_JPRB, 0.51240E+00_JPRB, 0.49684E+00_JPRB, 0.51260E+00_JPRB ,&
&  0.51491E+00_JPRB, 0.50028E+00_JPRB, 0.47499E+00_JPRB, 0.49700E+00_JPRB, 0.49556E+00_JPRB, 0.49339E+00_JPRB, 0.56466E+00_JPRB ,&
&  0.61539E+00_JPRB, 0.65316E+00_JPRB, 0.69274E+00_JPRB, 0.73414E+00_JPRB, 0.77216E+00_JPRB, 0.77807E+00_JPRB, 0.80204E+00_JPRB ,&
&  0.83672E+00_JPRB, 0.86741E+00_JPRB, 0.90642E+00_JPRB, 0.92885E+00_JPRB, 0.98285E+00_JPRB, 0.10507E+01_JPRB, 0.10436E+01_JPRB ,&
&  0.10839E+01_JPRB, 0.98161E+00_JPRB, 0.78962E+00_JPRB, 0.10556E+01_JPRB, 0.15503E+01_JPRB, 0.16511E+01_JPRB, 0.15301E+01_JPRB ,&
&  0.16277E+01_JPRB, 0.16487E+01_JPRB, 0.17729E+01_JPRB, 0.18622E+01_JPRB, 0.17802E+01_JPRB, 0.16550E+01_JPRB, 0.15107E+01_JPRB ,&
&  0.13460E+01_JPRB, 0.12371E+01_JPRB, 0.11897E+01_JPRB, 0.12522E+01_JPRB, 0.12161E+01_JPRB, 0.12190E+01_JPRB, 0.12168E+01_JPRB ,&
&  0.10688E+01_JPRB, 0.89333E+00_JPRB, 0.65102E+00_JPRB, 0.66598E+00_JPRB, 0.53877E+00_JPRB, 0.54152E+00_JPRB, 0.59081E+00_JPRB ,&
&  0.57235E+00_JPRB, 0.60103E+00_JPRB, 0.61851E+00_JPRB, 0.61489E+00_JPRB, 0.72548E+00_JPRB, 0.83388E+00_JPRB, 0.91479E+00_JPRB ,&
&  0.89030E+00_JPRB, 0.76856E+00_JPRB, 0.80535E+00_JPRB, 0.78498E+00_JPRB, 0.78973E+00_JPRB, 0.77281E+00_JPRB, 0.74907E+00_JPRB ,&
&  0.71852E+00_JPRB, 0.68621E+00_JPRB, 0.65994E+00_JPRB, 0.64142E+00_JPRB, 0.63195E+00_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.66910E+00_JPRB, 0.65705E+00_JPRB, 0.64973E+00_JPRB, 0.64708E+00_JPRB, 0.65175E+00_JPRB, 0.66340E+00_JPRB, 0.66355E+00_JPRB ,&
&  0.65786E+00_JPRB, 0.64535E+00_JPRB, 0.62859E+00_JPRB, 0.61121E+00_JPRB, 0.59590E+00_JPRB, 0.58458E+00_JPRB, 0.57497E+00_JPRB ,&
&  0.55826E+00_JPRB, 0.48858E+00_JPRB, 0.39371E+00_JPRB, 0.35015E+00_JPRB, 0.33229E+00_JPRB, 0.33892E+00_JPRB, 0.36704E+00_JPRB ,&
&  0.39439E+00_JPRB, 0.41776E+00_JPRB, 0.45625E+00_JPRB, 0.52052E+00_JPRB, 0.57579E+00_JPRB, 0.58829E+00_JPRB, 0.60948E+00_JPRB ,&
&  0.68138E+00_JPRB, 0.56858E+00_JPRB, 0.57070E+00_JPRB, 0.57536E+00_JPRB, 0.63742E+00_JPRB, 0.68615E+00_JPRB, 0.68006E+00_JPRB ,&
&  0.66981E+00_JPRB, 0.69691E+00_JPRB, 0.72736E+00_JPRB, 0.76228E+00_JPRB, 0.78847E+00_JPRB, 0.83027E+00_JPRB, 0.92074E+00_JPRB ,&
&  0.99872E+00_JPRB, 0.10181E+01_JPRB, 0.97222E+00_JPRB, 0.94830E+00_JPRB, 0.10172E+01_JPRB, 0.10640E+01_JPRB, 0.10448E+01_JPRB ,&
&  0.94233E+00_JPRB, 0.88673E+00_JPRB, 0.10196E+01_JPRB, 0.12477E+01_JPRB, 0.15285E+01_JPRB, 0.17446E+01_JPRB, 0.17691E+01_JPRB ,&
&  0.18389E+01_JPRB, 0.18294E+01_JPRB, 0.18867E+01_JPRB, 0.17199E+01_JPRB, 0.18356E+01_JPRB, 0.17678E+01_JPRB, 0.16820E+01_JPRB ,&
&  0.16346E+01_JPRB, 0.16130E+01_JPRB, 0.15545E+01_JPRB, 0.15959E+01_JPRB, 0.17050E+01_JPRB, 0.17404E+01_JPRB, 0.13804E+01_JPRB ,&
&  0.10853E+01_JPRB, 0.10856E+01_JPRB, 0.93749E+00_JPRB, 0.81111E+00_JPRB, 0.70272E+00_JPRB, 0.70336E+00_JPRB, 0.69598E+00_JPRB ,&
&  0.68690E+00_JPRB, 0.69100E+00_JPRB, 0.68217E+00_JPRB, 0.74226E+00_JPRB, 0.85941E+00_JPRB, 0.99328E+00_JPRB, 0.10839E+01_JPRB ,&
&  0.99317E+00_JPRB, 0.10437E+01_JPRB, 0.10094E+01_JPRB, 0.96299E+00_JPRB, 0.98235E+00_JPRB, 0.96702E+00_JPRB, 0.92028E+00_JPRB ,&
&  0.85719E+00_JPRB, 0.80596E+00_JPRB, 0.76176E+00_JPRB, 0.72208E+00_JPRB, 0.69059E+00_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.76590E+00_JPRB, 0.74024E+00_JPRB, 0.72162E+00_JPRB, 0.71202E+00_JPRB, 0.71347E+00_JPRB, 0.72096E+00_JPRB, 0.71213E+00_JPRB ,&
&  0.68739E+00_JPRB, 0.66212E+00_JPRB, 0.64403E+00_JPRB, 0.62961E+00_JPRB, 0.61272E+00_JPRB, 0.59780E+00_JPRB, 0.58725E+00_JPRB ,&
&  0.57915E+00_JPRB, 0.47350E+00_JPRB, 0.37411E+00_JPRB, 0.35442E+00_JPRB, 0.34511E+00_JPRB, 0.33279E+00_JPRB, 0.35284E+00_JPRB ,&
&  0.38910E+00_JPRB, 0.43911E+00_JPRB, 0.49735E+00_JPRB, 0.60366E+00_JPRB, 0.66561E+00_JPRB, 0.70904E+00_JPRB, 0.73479E+00_JPRB ,&
&  0.70481E+00_JPRB, 0.69694E+00_JPRB, 0.73713E+00_JPRB, 0.73920E+00_JPRB, 0.72778E+00_JPRB, 0.73367E+00_JPRB, 0.71742E+00_JPRB ,&
&  0.71611E+00_JPRB, 0.73544E+00_JPRB, 0.77156E+00_JPRB, 0.82385E+00_JPRB, 0.89158E+00_JPRB, 0.97441E+00_JPRB, 0.10823E+01_JPRB ,&
&  0.10953E+01_JPRB, 0.10538E+01_JPRB, 0.94531E+00_JPRB, 0.94287E+00_JPRB, 0.87820E+00_JPRB, 0.88638E+00_JPRB, 0.87927E+00_JPRB ,&
&  0.10799E+01_JPRB, 0.11348E+01_JPRB, 0.11041E+01_JPRB, 0.13442E+01_JPRB, 0.13425E+01_JPRB, 0.15035E+01_JPRB, 0.18819E+01_JPRB ,&
&  0.19772E+01_JPRB, 0.19870E+01_JPRB, 0.20271E+01_JPRB, 0.16204E+01_JPRB, 0.14239E+01_JPRB, 0.19616E+01_JPRB, 0.20789E+01_JPRB ,&
&  0.21640E+01_JPRB, 0.22721E+01_JPRB, 0.22768E+01_JPRB, 0.23865E+01_JPRB, 0.19968E+01_JPRB, 0.16060E+01_JPRB, 0.11809E+01_JPRB ,&
&  0.10900E+01_JPRB, 0.99352E+00_JPRB, 0.10572E+01_JPRB, 0.94027E+00_JPRB, 0.90741E+00_JPRB, 0.84286E+00_JPRB, 0.79310E+00_JPRB ,&
&  0.79501E+00_JPRB, 0.78853E+00_JPRB, 0.83480E+00_JPRB, 0.92781E+00_JPRB, 0.10653E+01_JPRB, 0.10789E+01_JPRB, 0.12255E+01_JPRB ,&
&  0.12722E+01_JPRB, 0.12316E+01_JPRB, 0.11054E+01_JPRB, 0.11050E+01_JPRB, 0.10663E+01_JPRB, 0.10366E+01_JPRB, 0.10000E+01_JPRB ,&
&  0.95218E+00_JPRB, 0.90500E+00_JPRB, 0.86706E+00_JPRB, 0.82983E+00_JPRB, 0.79623E+00_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.84960E+00_JPRB, 0.82296E+00_JPRB, 0.80128E+00_JPRB, 0.78942E+00_JPRB, 0.78271E+00_JPRB, 0.77052E+00_JPRB, 0.74089E+00_JPRB ,&
&  0.70263E+00_JPRB, 0.67107E+00_JPRB, 0.64282E+00_JPRB, 0.62478E+00_JPRB, 0.60404E+00_JPRB, 0.59881E+00_JPRB, 0.60162E+00_JPRB ,&
&  0.59463E+00_JPRB, 0.52196E+00_JPRB, 0.40180E+00_JPRB, 0.36594E+00_JPRB, 0.36650E+00_JPRB, 0.35012E+00_JPRB, 0.35131E+00_JPRB ,&
&  0.40637E+00_JPRB, 0.47959E+00_JPRB, 0.56290E+00_JPRB, 0.70409E+00_JPRB, 0.77612E+00_JPRB, 0.84646E+00_JPRB, 0.87274E+00_JPRB ,&
&  0.88363E+00_JPRB, 0.80001E+00_JPRB, 0.75199E+00_JPRB, 0.71185E+00_JPRB, 0.68108E+00_JPRB, 0.66417E+00_JPRB, 0.68098E+00_JPRB ,&
&  0.70489E+00_JPRB, 0.76048E+00_JPRB, 0.82502E+00_JPRB, 0.85648E+00_JPRB, 0.92664E+00_JPRB, 0.98499E+00_JPRB, 0.10446E+01_JPRB ,&
&  0.10523E+01_JPRB, 0.10209E+01_JPRB, 0.99108E+00_JPRB, 0.10416E+01_JPRB, 0.10295E+01_JPRB, 0.96201E+00_JPRB, 0.98148E+00_JPRB ,&
&  0.10984E+01_JPRB, 0.11083E+01_JPRB, 0.11547E+01_JPRB, 0.13232E+01_JPRB, 0.15658E+01_JPRB, 0.16418E+01_JPRB, 0.18916E+01_JPRB ,&
&  0.15816E+01_JPRB, 0.13961E+01_JPRB, 0.12148E+01_JPRB, 0.10730E+01_JPRB, 0.14390E+01_JPRB, 0.24287E+01_JPRB, 0.25927E+01_JPRB ,&
&  0.26173E+01_JPRB, 0.27901E+01_JPRB, 0.28546E+01_JPRB, 0.24699E+01_JPRB, 0.19693E+01_JPRB, 0.15349E+01_JPRB, 0.17891E+01_JPRB ,&
&  0.17445E+01_JPRB, 0.18314E+01_JPRB, 0.16098E+01_JPRB, 0.12482E+01_JPRB, 0.99325E+00_JPRB, 0.10846E+01_JPRB, 0.10429E+01_JPRB ,&
&  0.10098E+01_JPRB, 0.10096E+01_JPRB, 0.12518E+01_JPRB, 0.12665E+01_JPRB, 0.12731E+01_JPRB, 0.13754E+01_JPRB, 0.14763E+01_JPRB ,&
&  0.13537E+01_JPRB, 0.13075E+01_JPRB, 0.12556E+01_JPRB, 0.11811E+01_JPRB, 0.11298E+01_JPRB, 0.10764E+01_JPRB, 0.10272E+01_JPRB ,&
&  0.99180E+00_JPRB, 0.96160E+00_JPRB, 0.92769E+00_JPRB, 0.90268E+00_JPRB, 0.87573E+00_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.91044E+00_JPRB, 0.88442E+00_JPRB, 0.86398E+00_JPRB, 0.84385E+00_JPRB, 0.82233E+00_JPRB, 0.79166E+00_JPRB, 0.75199E+00_JPRB ,&
&  0.71097E+00_JPRB, 0.67373E+00_JPRB, 0.64622E+00_JPRB, 0.62673E+00_JPRB, 0.60584E+00_JPRB, 0.59822E+00_JPRB, 0.60104E+00_JPRB ,&
&  0.60053E+00_JPRB, 0.60350E+00_JPRB, 0.46006E+00_JPRB, 0.39786E+00_JPRB, 0.39117E+00_JPRB, 0.38474E+00_JPRB, 0.37491E+00_JPRB ,&
&  0.44759E+00_JPRB, 0.54176E+00_JPRB, 0.61318E+00_JPRB, 0.75927E+00_JPRB, 0.92965E+00_JPRB, 0.89126E+00_JPRB, 0.86078E+00_JPRB ,&
&  0.86965E+00_JPRB, 0.81348E+00_JPRB, 0.75270E+00_JPRB, 0.71814E+00_JPRB, 0.69687E+00_JPRB, 0.70218E+00_JPRB, 0.72841E+00_JPRB ,&
&  0.78428E+00_JPRB, 0.85916E+00_JPRB, 0.87639E+00_JPRB, 0.91114E+00_JPRB, 0.94569E+00_JPRB, 0.10206E+01_JPRB, 0.10866E+01_JPRB ,&
&  0.10542E+01_JPRB, 0.10233E+01_JPRB, 0.10521E+01_JPRB, 0.11654E+01_JPRB, 0.11650E+01_JPRB, 0.11043E+01_JPRB, 0.12332E+01_JPRB ,&
&  0.12285E+01_JPRB, 0.13378E+01_JPRB, 0.13128E+01_JPRB, 0.14824E+01_JPRB, 0.16043E+01_JPRB, 0.15942E+01_JPRB, 0.16907E+01_JPRB ,&
&  0.13381E+01_JPRB, 0.11946E+01_JPRB, 0.12548E+01_JPRB, 0.14242E+01_JPRB, 0.16679E+01_JPRB, 0.21190E+01_JPRB, 0.32025E+01_JPRB ,&
&  0.26912E+01_JPRB, 0.28647E+01_JPRB, 0.28644E+01_JPRB, 0.26885E+01_JPRB, 0.14541E+01_JPRB, 0.11504E+01_JPRB, 0.18195E+01_JPRB ,&
&  0.19878E+01_JPRB, 0.12427E+01_JPRB, 0.85574E+00_JPRB, 0.99887E+00_JPRB, 0.77179E+00_JPRB, 0.90702E+00_JPRB, 0.11835E+01_JPRB ,&
&  0.12675E+01_JPRB, 0.13397E+01_JPRB, 0.16603E+01_JPRB, 0.14928E+01_JPRB, 0.14399E+01_JPRB, 0.18725E+01_JPRB, 0.16736E+01_JPRB ,&
&  0.15431E+01_JPRB, 0.14005E+01_JPRB, 0.13770E+01_JPRB, 0.12265E+01_JPRB, 0.11373E+01_JPRB, 0.10849E+01_JPRB, 0.10572E+01_JPRB ,&
&  0.10440E+01_JPRB, 0.10275E+01_JPRB, 0.10063E+01_JPRB, 0.97583E+00_JPRB, 0.93989E+00_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.93667E+00_JPRB, 0.91740E+00_JPRB, 0.90184E+00_JPRB, 0.88115E+00_JPRB, 0.85037E+00_JPRB, 0.80865E+00_JPRB, 0.76402E+00_JPRB ,&
&  0.72793E+00_JPRB, 0.68619E+00_JPRB, 0.65567E+00_JPRB, 0.63352E+00_JPRB, 0.61427E+00_JPRB, 0.60310E+00_JPRB, 0.59961E+00_JPRB ,&
&  0.59063E+00_JPRB, 0.58490E+00_JPRB, 0.53917E+00_JPRB, 0.47965E+00_JPRB, 0.46632E+00_JPRB, 0.46238E+00_JPRB, 0.46673E+00_JPRB ,&
&  0.54487E+00_JPRB, 0.63614E+00_JPRB, 0.72063E+00_JPRB, 0.82567E+00_JPRB, 0.89816E+00_JPRB, 0.85063E+00_JPRB, 0.87530E+00_JPRB ,&
&  0.86763E+00_JPRB, 0.83392E+00_JPRB, 0.78213E+00_JPRB, 0.76645E+00_JPRB, 0.77118E+00_JPRB, 0.78901E+00_JPRB, 0.81886E+00_JPRB ,&
&  0.86397E+00_JPRB, 0.90617E+00_JPRB, 0.90524E+00_JPRB, 0.89947E+00_JPRB, 0.96327E+00_JPRB, 0.10633E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.10844E+01_JPRB, 0.10905E+01_JPRB, 0.11259E+01_JPRB, 0.12314E+01_JPRB, 0.13813E+01_JPRB, 0.13809E+01_JPRB, 0.11848E+01_JPRB ,&
&  0.10613E+01_JPRB, 0.11505E+01_JPRB, 0.14515E+01_JPRB, 0.14377E+01_JPRB, 0.14677E+01_JPRB, 0.15168E+01_JPRB, 0.15520E+01_JPRB ,&
&  0.15136E+01_JPRB, 0.15383E+01_JPRB, 0.15893E+01_JPRB, 0.18611E+01_JPRB, 0.21692E+01_JPRB, 0.22249E+01_JPRB, 0.31515E+01_JPRB ,&
&  0.31515E+01_JPRB, 0.28645E+01_JPRB, 0.21695E+01_JPRB, 0.16039E+01_JPRB, 0.15840E+01_JPRB, 0.12555E+01_JPRB, 0.84652E+00_JPRB ,&
&  0.80433E+00_JPRB, 0.75952E+00_JPRB, 0.66784E+00_JPRB, 0.65817E+00_JPRB, 0.62154E+00_JPRB, 0.80361E+00_JPRB, 0.11737E+01_JPRB ,&
&  0.16042E+01_JPRB, 0.18721E+01_JPRB, 0.19154E+01_JPRB, 0.17531E+01_JPRB, 0.16008E+01_JPRB, 0.20417E+01_JPRB, 0.18851E+01_JPRB ,&
&  0.14812E+01_JPRB, 0.13920E+01_JPRB, 0.13197E+01_JPRB, 0.12141E+01_JPRB, 0.11590E+01_JPRB, 0.11142E+01_JPRB, 0.10858E+01_JPRB ,&
&  0.10614E+01_JPRB, 0.10414E+01_JPRB, 0.10194E+01_JPRB, 0.98899E+00_JPRB, 0.96177E+00_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.95632E+00_JPRB, 0.94174E+00_JPRB, 0.92290E+00_JPRB, 0.89825E+00_JPRB, 0.86285E+00_JPRB, 0.82331E+00_JPRB, 0.78563E+00_JPRB ,&
&  0.75754E+00_JPRB, 0.71816E+00_JPRB, 0.68186E+00_JPRB, 0.65322E+00_JPRB, 0.63002E+00_JPRB, 0.61391E+00_JPRB, 0.60745E+00_JPRB ,&
&  0.60332E+00_JPRB, 0.59178E+00_JPRB, 0.59987E+00_JPRB, 0.57447E+00_JPRB, 0.61148E+00_JPRB, 0.63534E+00_JPRB, 0.66971E+00_JPRB ,&
&  0.73972E+00_JPRB, 0.82176E+00_JPRB, 0.93471E+00_JPRB, 0.98158E+00_JPRB, 0.10170E+01_JPRB, 0.99914E+00_JPRB, 0.96498E+00_JPRB ,&
&  0.93190E+00_JPRB, 0.92608E+00_JPRB, 0.89500E+00_JPRB, 0.88017E+00_JPRB, 0.86794E+00_JPRB, 0.86651E+00_JPRB, 0.88050E+00_JPRB ,&
&  0.90575E+00_JPRB, 0.93627E+00_JPRB, 0.90329E+00_JPRB, 0.92173E+00_JPRB, 0.10013E+01_JPRB, 0.10888E+01_JPRB, 0.11473E+01_JPRB ,&
&  0.11642E+01_JPRB, 0.11747E+01_JPRB, 0.12253E+01_JPRB, 0.13215E+01_JPRB, 0.12422E+01_JPRB, 0.10543E+01_JPRB, 0.11544E+01_JPRB ,&
&  0.12416E+01_JPRB, 0.14253E+01_JPRB, 0.14034E+01_JPRB, 0.16340E+01_JPRB, 0.15901E+01_JPRB, 0.14970E+01_JPRB, 0.16195E+01_JPRB ,&
&  0.17616E+01_JPRB, 0.22138E+01_JPRB, 0.19298E+01_JPRB, 0.19456E+01_JPRB, 0.28080E+01_JPRB, 0.26572E+01_JPRB, 0.28607E+01_JPRB ,&
&  0.30950E+01_JPRB, 0.25393E+01_JPRB, 0.20255E+01_JPRB, 0.17537E+01_JPRB, 0.22954E+01_JPRB, 0.21708E+01_JPRB, 0.10813E+01_JPRB ,&
&  0.87169E+00_JPRB, 0.76455E+00_JPRB, 0.69258E+00_JPRB, 0.63397E+00_JPRB, 0.60811E+00_JPRB, 0.84586E+00_JPRB, 0.15304E+01_JPRB ,&
&  0.16142E+01_JPRB, 0.23497E+01_JPRB, 0.23521E+01_JPRB, 0.21207E+01_JPRB, 0.18595E+01_JPRB, 0.20336E+01_JPRB, 0.18216E+01_JPRB ,&
&  0.15313E+01_JPRB, 0.14479E+01_JPRB, 0.13029E+01_JPRB, 0.11996E+01_JPRB, 0.11174E+01_JPRB, 0.10717E+01_JPRB, 0.10543E+01_JPRB ,&
&  0.10381E+01_JPRB, 0.10221E+01_JPRB, 0.10080E+01_JPRB, 0.99059E+00_JPRB, 0.97382E+00_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.95258E+00_JPRB, 0.92887E+00_JPRB, 0.91133E+00_JPRB, 0.88936E+00_JPRB, 0.86631E+00_JPRB, 0.84011E+00_JPRB, 0.80877E+00_JPRB ,&
&  0.78272E+00_JPRB, 0.74404E+00_JPRB, 0.70670E+00_JPRB, 0.67302E+00_JPRB, 0.64502E+00_JPRB, 0.62489E+00_JPRB, 0.61742E+00_JPRB ,&
&  0.61189E+00_JPRB, 0.61120E+00_JPRB, 0.61992E+00_JPRB, 0.67262E+00_JPRB, 0.75069E+00_JPRB, 0.78713E+00_JPRB, 0.89388E+00_JPRB ,&
&  0.10309E+01_JPRB, 0.11468E+01_JPRB, 0.12799E+01_JPRB, 0.12928E+01_JPRB, 0.12692E+01_JPRB, 0.12060E+01_JPRB, 0.11425E+01_JPRB ,&
&  0.10908E+01_JPRB, 0.10870E+01_JPRB, 0.10343E+01_JPRB, 0.98984E+00_JPRB, 0.95466E+00_JPRB, 0.94054E+00_JPRB, 0.94267E+00_JPRB ,&
&  0.97520E+00_JPRB, 0.94486E+00_JPRB, 0.91998E+00_JPRB, 0.95261E+00_JPRB, 0.10262E+01_JPRB, 0.11264E+01_JPRB, 0.11969E+01_JPRB ,&
&  0.12185E+01_JPRB, 0.12152E+01_JPRB, 0.12597E+01_JPRB, 0.13356E+01_JPRB, 0.11055E+01_JPRB, 0.12599E+01_JPRB, 0.12628E+01_JPRB ,&
&  0.12822E+01_JPRB, 0.13480E+01_JPRB, 0.13346E+01_JPRB, 0.15458E+01_JPRB, 0.16062E+01_JPRB, 0.15162E+01_JPRB, 0.15988E+01_JPRB ,&
&  0.20238E+01_JPRB, 0.19524E+01_JPRB, 0.18786E+01_JPRB, 0.22113E+01_JPRB, 0.27829E+01_JPRB, 0.39180E+01_JPRB, 0.25305E+01_JPRB ,&
&  0.21358E+01_JPRB, 0.23519E+01_JPRB, 0.20129E+01_JPRB, 0.17952E+01_JPRB, 0.21745E+01_JPRB, 0.24223E+01_JPRB, 0.19969E+01_JPRB ,&
&  0.98126E+00_JPRB, 0.74449E+00_JPRB, 0.76931E+00_JPRB, 0.78798E+00_JPRB, 0.75428E+00_JPRB, 0.10084E+01_JPRB, 0.22819E+01_JPRB ,&
&  0.19756E+01_JPRB, 0.25655E+01_JPRB, 0.24651E+01_JPRB, 0.21547E+01_JPRB, 0.19121E+01_JPRB, 0.19061E+01_JPRB, 0.17873E+01_JPRB ,&
&  0.14652E+01_JPRB, 0.13132E+01_JPRB, 0.11854E+01_JPRB, 0.11027E+01_JPRB, 0.10630E+01_JPRB, 0.10368E+01_JPRB, 0.10243E+01_JPRB ,&
&  0.10114E+01_JPRB, 0.99891E+00_JPRB, 0.99252E+00_JPRB, 0.98730E+00_JPRB, 0.97500E+00_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.92125E+00_JPRB, 0.90438E+00_JPRB, 0.88987E+00_JPRB, 0.88230E+00_JPRB, 0.87023E+00_JPRB, 0.83894E+00_JPRB, 0.80776E+00_JPRB ,&
&  0.78012E+00_JPRB, 0.74216E+00_JPRB, 0.71057E+00_JPRB, 0.67681E+00_JPRB, 0.65035E+00_JPRB, 0.63221E+00_JPRB, 0.62386E+00_JPRB ,&
&  0.62725E+00_JPRB, 0.62892E+00_JPRB, 0.64034E+00_JPRB, 0.72456E+00_JPRB, 0.85869E+00_JPRB, 0.89863E+00_JPRB, 0.10217E+01_JPRB ,&
&  0.14729E+01_JPRB, 0.16773E+01_JPRB, 0.16056E+01_JPRB, 0.15677E+01_JPRB, 0.15459E+01_JPRB, 0.14982E+01_JPRB, 0.14099E+01_JPRB ,&
&  0.12867E+01_JPRB, 0.12115E+01_JPRB, 0.11060E+01_JPRB, 0.10391E+01_JPRB, 0.10005E+01_JPRB, 0.98521E+00_JPRB, 0.99368E+00_JPRB ,&
&  0.98150E+00_JPRB, 0.95818E+00_JPRB, 0.94257E+00_JPRB, 0.97066E+00_JPRB, 0.10555E+01_JPRB, 0.11501E+01_JPRB, 0.12266E+01_JPRB ,&
&  0.12410E+01_JPRB, 0.12354E+01_JPRB, 0.12477E+01_JPRB, 0.12277E+01_JPRB, 0.12499E+01_JPRB, 0.12768E+01_JPRB, 0.13192E+01_JPRB ,&
&  0.13185E+01_JPRB, 0.14042E+01_JPRB, 0.14986E+01_JPRB, 0.15291E+01_JPRB, 0.15498E+01_JPRB, 0.15713E+01_JPRB, 0.16821E+01_JPRB ,&
&  0.19632E+01_JPRB, 0.19922E+01_JPRB, 0.20919E+01_JPRB, 0.26414E+01_JPRB, 0.29940E+01_JPRB, 0.33811E+01_JPRB, 0.30626E+01_JPRB ,&
&  0.24728E+01_JPRB, 0.18714E+01_JPRB, 0.17026E+01_JPRB, 0.18253E+01_JPRB, 0.20371E+01_JPRB, 0.21009E+01_JPRB, 0.27569E+01_JPRB ,&
&  0.27109E+01_JPRB, 0.18614E+01_JPRB, 0.15777E+01_JPRB, 0.19658E+01_JPRB, 0.16811E+01_JPRB, 0.15146E+01_JPRB, 0.18715E+01_JPRB ,&
&  0.21350E+01_JPRB, 0.25278E+01_JPRB, 0.23393E+01_JPRB, 0.19581E+01_JPRB, 0.18947E+01_JPRB, 0.17706E+01_JPRB, 0.15840E+01_JPRB ,&
&  0.13379E+01_JPRB, 0.12319E+01_JPRB, 0.11570E+01_JPRB, 0.10872E+01_JPRB, 0.10360E+01_JPRB, 0.10111E+01_JPRB, 0.99749E+00_JPRB ,&
&  0.97866E+00_JPRB, 0.96991E+00_JPRB, 0.96652E+00_JPRB, 0.95883E+00_JPRB, 0.93947E+00_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.88628E+00_JPRB, 0.88073E+00_JPRB, 0.88167E+00_JPRB, 0.85501E+00_JPRB, 0.82076E+00_JPRB, 0.78988E+00_JPRB, 0.77489E+00_JPRB ,&
&  0.75645E+00_JPRB, 0.72545E+00_JPRB, 0.69496E+00_JPRB, 0.66926E+00_JPRB, 0.64458E+00_JPRB, 0.63173E+00_JPRB, 0.62789E+00_JPRB ,&
&  0.63558E+00_JPRB, 0.64930E+00_JPRB, 0.70263E+00_JPRB, 0.82607E+00_JPRB, 0.98429E+00_JPRB, 0.12360E+01_JPRB, 0.11276E+01_JPRB ,&
&  0.15532E+01_JPRB, 0.20312E+01_JPRB, 0.19646E+01_JPRB, 0.18799E+01_JPRB, 0.17932E+01_JPRB, 0.17180E+01_JPRB, 0.15386E+01_JPRB ,&
&  0.13666E+01_JPRB, 0.12683E+01_JPRB, 0.11535E+01_JPRB, 0.10793E+01_JPRB, 0.10501E+01_JPRB, 0.10371E+01_JPRB, 0.10064E+01_JPRB ,&
&  0.98444E+00_JPRB, 0.95981E+00_JPRB, 0.94840E+00_JPRB, 0.97709E+00_JPRB, 0.10403E+01_JPRB, 0.11440E+01_JPRB, 0.12131E+01_JPRB ,&
&  0.12445E+01_JPRB, 0.12442E+01_JPRB, 0.12369E+01_JPRB, 0.11732E+01_JPRB, 0.12172E+01_JPRB, 0.12615E+01_JPRB, 0.13148E+01_JPRB ,&
&  0.12885E+01_JPRB, 0.12413E+01_JPRB, 0.13975E+01_JPRB, 0.15395E+01_JPRB, 0.15840E+01_JPRB, 0.15505E+01_JPRB, 0.16446E+01_JPRB ,&
&  0.20093E+01_JPRB, 0.23402E+01_JPRB, 0.29016E+01_JPRB, 0.30721E+01_JPRB, 0.31561E+01_JPRB, 0.31843E+01_JPRB, 0.30665E+01_JPRB ,&
&  0.29598E+01_JPRB, 0.23414E+01_JPRB, 0.20347E+01_JPRB, 0.19179E+01_JPRB, 0.18922E+01_JPRB, 0.19738E+01_JPRB, 0.24830E+01_JPRB ,&
&  0.29504E+01_JPRB, 0.31985E+01_JPRB, 0.32483E+01_JPRB, 0.27796E+01_JPRB, 0.24085E+01_JPRB, 0.20600E+01_JPRB, 0.19510E+01_JPRB ,&
&  0.23134E+01_JPRB, 0.23875E+01_JPRB, 0.21687E+01_JPRB, 0.23245E+01_JPRB, 0.18677E+01_JPRB, 0.17408E+01_JPRB, 0.15385E+01_JPRB ,&
&  0.13660E+01_JPRB, 0.12439E+01_JPRB, 0.11471E+01_JPRB, 0.10805E+01_JPRB, 0.10443E+01_JPRB, 0.99290E+00_JPRB, 0.95527E+00_JPRB ,&
&  0.93151E+00_JPRB, 0.92566E+00_JPRB, 0.91943E+00_JPRB, 0.91147E+00_JPRB, 0.89548E+00_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.88582E+00_JPRB, 0.88326E+00_JPRB, 0.87128E+00_JPRB, 0.83474E+00_JPRB, 0.80701E+00_JPRB, 0.78804E+00_JPRB, 0.75310E+00_JPRB ,&
&  0.72956E+00_JPRB, 0.70760E+00_JPRB, 0.68122E+00_JPRB, 0.65782E+00_JPRB, 0.64389E+00_JPRB, 0.63022E+00_JPRB, 0.63512E+00_JPRB ,&
&  0.67435E+00_JPRB, 0.74228E+00_JPRB, 0.86114E+00_JPRB, 0.97918E+00_JPRB, 0.11489E+01_JPRB, 0.14428E+01_JPRB, 0.15975E+01_JPRB ,&
&  0.16741E+01_JPRB, 0.24364E+01_JPRB, 0.21392E+01_JPRB, 0.19926E+01_JPRB, 0.18747E+01_JPRB, 0.17344E+01_JPRB, 0.15263E+01_JPRB ,&
&  0.13196E+01_JPRB, 0.12436E+01_JPRB, 0.11896E+01_JPRB, 0.11305E+01_JPRB, 0.11045E+01_JPRB, 0.10368E+01_JPRB, 0.10267E+01_JPRB ,&
&  0.10089E+01_JPRB, 0.99441E+00_JPRB, 0.97651E+00_JPRB, 0.10083E+01_JPRB, 0.10800E+01_JPRB, 0.11589E+01_JPRB, 0.12308E+01_JPRB ,&
&  0.12592E+01_JPRB, 0.12501E+01_JPRB, 0.12005E+01_JPRB, 0.11787E+01_JPRB, 0.11814E+01_JPRB, 0.12359E+01_JPRB, 0.13025E+01_JPRB ,&
&  0.13191E+01_JPRB, 0.12852E+01_JPRB, 0.13754E+01_JPRB, 0.14135E+01_JPRB, 0.14398E+01_JPRB, 0.15828E+01_JPRB, 0.16773E+01_JPRB ,&
&  0.20717E+01_JPRB, 0.21497E+01_JPRB, 0.30797E+01_JPRB, 0.28652E+01_JPRB, 0.28106E+01_JPRB, 0.29006E+01_JPRB, 0.26740E+01_JPRB ,&
&  0.22547E+01_JPRB, 0.20704E+01_JPRB, 0.18641E+01_JPRB, 0.18153E+01_JPRB, 0.18868E+01_JPRB, 0.20912E+01_JPRB, 0.24798E+01_JPRB ,&
&  0.25709E+01_JPRB, 0.24900E+01_JPRB, 0.25211E+01_JPRB, 0.21505E+01_JPRB, 0.20673E+01_JPRB, 0.19243E+01_JPRB, 0.18860E+01_JPRB ,&
&  0.22627E+01_JPRB, 0.23097E+01_JPRB, 0.22801E+01_JPRB, 0.21595E+01_JPRB, 0.17191E+01_JPRB, 0.15661E+01_JPRB, 0.13852E+01_JPRB ,&
&  0.12579E+01_JPRB, 0.11556E+01_JPRB, 0.10975E+01_JPRB, 0.10460E+01_JPRB, 0.99553E+00_JPRB, 0.94515E+00_JPRB, 0.91551E+00_JPRB ,&
&  0.89516E+00_JPRB, 0.88737E+00_JPRB, 0.88944E+00_JPRB, 0.88609E+00_JPRB, 0.88338E+00_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.92624E+00_JPRB, 0.93613E+00_JPRB, 0.94250E+00_JPRB, 0.97423E+00_JPRB, 0.10199E+01_JPRB, 0.10870E+01_JPRB, 0.11198E+01_JPRB ,&
&  0.97810E+00_JPRB, 0.69156E+00_JPRB, 0.67344E+00_JPRB, 0.65903E+00_JPRB, 0.64637E+00_JPRB, 0.65070E+00_JPRB, 0.69549E+00_JPRB ,&
&  0.76485E+00_JPRB, 0.85378E+00_JPRB, 0.95643E+00_JPRB, 0.11054E+01_JPRB, 0.13216E+01_JPRB, 0.17276E+01_JPRB, 0.20706E+01_JPRB ,&
&  0.19702E+01_JPRB, 0.21044E+01_JPRB, 0.21924E+01_JPRB, 0.19283E+01_JPRB, 0.17674E+01_JPRB, 0.16505E+01_JPRB, 0.15508E+01_JPRB ,&
&  0.13748E+01_JPRB, 0.12839E+01_JPRB, 0.12595E+01_JPRB, 0.12428E+01_JPRB, 0.10795E+01_JPRB, 0.10622E+01_JPRB, 0.10602E+01_JPRB ,&
&  0.10597E+01_JPRB, 0.10657E+01_JPRB, 0.10716E+01_JPRB, 0.11145E+01_JPRB, 0.11753E+01_JPRB, 0.12441E+01_JPRB, 0.12968E+01_JPRB ,&
&  0.13139E+01_JPRB, 0.13035E+01_JPRB, 0.12689E+01_JPRB, 0.11903E+01_JPRB, 0.11888E+01_JPRB, 0.12318E+01_JPRB, 0.12898E+01_JPRB ,&
&  0.13276E+01_JPRB, 0.13814E+01_JPRB, 0.14165E+01_JPRB, 0.14324E+01_JPRB, 0.14245E+01_JPRB, 0.16682E+01_JPRB, 0.18194E+01_JPRB ,&
&  0.20745E+01_JPRB, 0.23894E+01_JPRB, 0.33140E+01_JPRB, 0.24409E+01_JPRB, 0.25152E+01_JPRB, 0.25619E+01_JPRB, 0.23699E+01_JPRB ,&
&  0.20187E+01_JPRB, 0.18774E+01_JPRB, 0.17183E+01_JPRB, 0.17024E+01_JPRB, 0.18840E+01_JPRB, 0.20925E+01_JPRB, 0.25498E+01_JPRB ,&
&  0.21721E+01_JPRB, 0.22062E+01_JPRB, 0.19542E+01_JPRB, 0.19407E+01_JPRB, 0.18761E+01_JPRB, 0.17700E+01_JPRB, 0.16860E+01_JPRB ,&
&  0.19958E+01_JPRB, 0.19368E+01_JPRB, 0.17935E+01_JPRB, 0.16371E+01_JPRB, 0.13468E+01_JPRB, 0.12243E+01_JPRB, 0.11052E+01_JPRB ,&
&  0.10524E+01_JPRB, 0.10308E+01_JPRB, 0.10254E+01_JPRB, 0.99092E+00_JPRB, 0.96602E+00_JPRB, 0.93574E+00_JPRB, 0.91731E+00_JPRB ,&
&  0.90435E+00_JPRB, 0.89980E+00_JPRB, 0.89972E+00_JPRB, 0.90596E+00_JPRB, 0.91408E+00_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.99468E+00_JPRB, 0.98411E+00_JPRB, 0.97701E+00_JPRB, 0.97154E+00_JPRB, 0.96132E+00_JPRB, 0.93037E+00_JPRB, 0.86632E+00_JPRB ,&
&  0.77674E+00_JPRB, 0.70715E+00_JPRB, 0.69236E+00_JPRB, 0.68555E+00_JPRB, 0.69055E+00_JPRB, 0.72075E+00_JPRB, 0.77604E+00_JPRB ,&
&  0.83574E+00_JPRB, 0.93657E+00_JPRB, 0.10769E+01_JPRB, 0.12603E+01_JPRB, 0.15088E+01_JPRB, 0.18983E+01_JPRB, 0.22905E+01_JPRB ,&
&  0.22477E+01_JPRB, 0.16468E+01_JPRB, 0.18679E+01_JPRB, 0.18327E+01_JPRB, 0.17099E+01_JPRB, 0.15784E+01_JPRB, 0.14367E+01_JPRB ,&
&  0.12935E+01_JPRB, 0.12487E+01_JPRB, 0.12259E+01_JPRB, 0.11815E+01_JPRB, 0.11104E+01_JPRB, 0.11354E+01_JPRB, 0.11645E+01_JPRB ,&
&  0.11937E+01_JPRB, 0.12159E+01_JPRB, 0.12200E+01_JPRB, 0.12632E+01_JPRB, 0.13027E+01_JPRB, 0.13666E+01_JPRB, 0.14380E+01_JPRB ,&
&  0.14629E+01_JPRB, 0.14644E+01_JPRB, 0.14467E+01_JPRB, 0.13538E+01_JPRB, 0.13296E+01_JPRB, 0.13826E+01_JPRB, 0.14331E+01_JPRB ,&
&  0.14660E+01_JPRB, 0.14556E+01_JPRB, 0.15225E+01_JPRB, 0.16860E+01_JPRB, 0.18891E+01_JPRB, 0.18053E+01_JPRB, 0.20914E+01_JPRB ,&
&  0.22943E+01_JPRB, 0.26872E+01_JPRB, 0.26188E+01_JPRB, 0.29843E+01_JPRB, 0.19635E+01_JPRB, 0.21107E+01_JPRB, 0.18707E+01_JPRB ,&
&  0.18728E+01_JPRB, 0.16853E+01_JPRB, 0.16222E+01_JPRB, 0.17282E+01_JPRB, 0.19848E+01_JPRB, 0.22935E+01_JPRB, 0.22605E+01_JPRB ,&
&  0.17650E+01_JPRB, 0.16653E+01_JPRB, 0.17096E+01_JPRB, 0.18544E+01_JPRB, 0.18290E+01_JPRB, 0.17545E+01_JPRB, 0.15496E+01_JPRB ,&
&  0.16047E+01_JPRB, 0.15335E+01_JPRB, 0.14506E+01_JPRB, 0.13231E+01_JPRB, 0.10802E+01_JPRB, 0.97166E+00_JPRB, 0.97052E+00_JPRB ,&
&  0.99084E+00_JPRB, 0.10102E+01_JPRB, 0.10142E+01_JPRB, 0.10076E+01_JPRB, 0.98424E+00_JPRB, 0.96285E+00_JPRB, 0.94957E+00_JPRB ,&
&  0.94219E+00_JPRB, 0.94978E+00_JPRB, 0.95901E+00_JPRB, 0.96848E+00_JPRB, 0.98372E+00_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.97904E+00_JPRB, 0.95446E+00_JPRB, 0.92932E+00_JPRB, 0.90604E+00_JPRB, 0.88556E+00_JPRB, 0.83878E+00_JPRB, 0.78819E+00_JPRB ,&
&  0.75538E+00_JPRB, 0.73807E+00_JPRB, 0.73760E+00_JPRB, 0.74452E+00_JPRB, 0.76897E+00_JPRB, 0.79792E+00_JPRB, 0.85057E+00_JPRB ,&
&  0.93340E+00_JPRB, 0.10514E+01_JPRB, 0.12139E+01_JPRB, 0.14140E+01_JPRB, 0.16679E+01_JPRB, 0.19601E+01_JPRB, 0.21344E+01_JPRB ,&
&  0.22909E+01_JPRB, 0.22914E+01_JPRB, 0.19882E+01_JPRB, 0.14636E+01_JPRB, 0.13526E+01_JPRB, 0.14061E+01_JPRB, 0.13196E+01_JPRB ,&
&  0.13054E+01_JPRB, 0.12707E+01_JPRB, 0.12317E+01_JPRB, 0.12333E+01_JPRB, 0.12419E+01_JPRB, 0.12866E+01_JPRB, 0.13233E+01_JPRB ,&
&  0.13506E+01_JPRB, 0.13734E+01_JPRB, 0.14058E+01_JPRB, 0.14736E+01_JPRB, 0.15311E+01_JPRB, 0.16015E+01_JPRB, 0.16572E+01_JPRB ,&
&  0.16909E+01_JPRB, 0.17400E+01_JPRB, 0.17288E+01_JPRB, 0.16740E+01_JPRB, 0.16190E+01_JPRB, 0.16694E+01_JPRB, 0.17353E+01_JPRB ,&
&  0.17904E+01_JPRB, 0.18082E+01_JPRB, 0.18551E+01_JPRB, 0.20657E+01_JPRB, 0.22526E+01_JPRB, 0.20918E+01_JPRB, 0.21824E+01_JPRB ,&
&  0.25397E+01_JPRB, 0.27970E+01_JPRB, 0.21381E+01_JPRB, 0.25179E+01_JPRB, 0.14940E+01_JPRB, 0.18045E+01_JPRB, 0.18744E+01_JPRB ,&
&  0.16899E+01_JPRB, 0.16404E+01_JPRB, 0.16517E+01_JPRB, 0.18227E+01_JPRB, 0.20741E+01_JPRB, 0.21840E+01_JPRB, 0.18945E+01_JPRB ,&
&  0.15148E+01_JPRB, 0.14631E+01_JPRB, 0.15280E+01_JPRB, 0.16095E+01_JPRB, 0.16111E+01_JPRB, 0.14966E+01_JPRB, 0.12010E+01_JPRB ,&
&  0.10262E+01_JPRB, 0.12643E+01_JPRB, 0.12899E+01_JPRB, 0.13189E+01_JPRB, 0.10796E+01_JPRB, 0.92873E+00_JPRB, 0.98356E+00_JPRB ,&
&  0.10312E+01_JPRB, 0.10572E+01_JPRB, 0.10593E+01_JPRB, 0.10441E+01_JPRB, 0.10199E+01_JPRB, 0.99987E+00_JPRB, 0.99202E+00_JPRB ,&
&  0.99145E+00_JPRB, 0.98698E+00_JPRB, 0.99135E+00_JPRB, 0.99787E+00_JPRB, 0.99661E+00_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.90041E+00_JPRB, 0.87089E+00_JPRB, 0.84920E+00_JPRB, 0.83171E+00_JPRB, 0.82086E+00_JPRB, 0.79743E+00_JPRB, 0.76685E+00_JPRB ,&
&  0.75015E+00_JPRB, 0.74385E+00_JPRB, 0.75429E+00_JPRB, 0.78278E+00_JPRB, 0.82419E+00_JPRB, 0.86535E+00_JPRB, 0.94360E+00_JPRB ,&
&  0.10380E+01_JPRB, 0.11778E+01_JPRB, 0.13580E+01_JPRB, 0.16169E+01_JPRB, 0.18305E+01_JPRB, 0.20274E+01_JPRB, 0.20654E+01_JPRB ,&
&  0.21495E+01_JPRB, 0.21552E+01_JPRB, 0.20376E+01_JPRB, 0.18948E+01_JPRB, 0.15087E+01_JPRB, 0.15153E+01_JPRB, 0.16687E+01_JPRB ,&
&  0.17333E+01_JPRB, 0.15158E+01_JPRB, 0.13684E+01_JPRB, 0.13865E+01_JPRB, 0.13965E+01_JPRB, 0.14498E+01_JPRB, 0.14978E+01_JPRB ,&
&  0.15399E+01_JPRB, 0.15857E+01_JPRB, 0.16319E+01_JPRB, 0.17022E+01_JPRB, 0.17772E+01_JPRB, 0.18206E+01_JPRB, 0.18676E+01_JPRB ,&
&  0.18922E+01_JPRB, 0.19274E+01_JPRB, 0.19548E+01_JPRB, 0.19489E+01_JPRB, 0.19979E+01_JPRB, 0.20771E+01_JPRB, 0.22159E+01_JPRB ,&
&  0.23558E+01_JPRB, 0.23411E+01_JPRB, 0.25003E+01_JPRB, 0.27638E+01_JPRB, 0.28015E+01_JPRB, 0.26990E+01_JPRB, 0.26851E+01_JPRB ,&
&  0.29450E+01_JPRB, 0.31699E+01_JPRB, 0.17606E+01_JPRB, 0.19206E+01_JPRB, 0.21849E+01_JPRB, 0.18572E+01_JPRB, 0.16598E+01_JPRB ,&
&  0.16266E+01_JPRB, 0.16281E+01_JPRB, 0.17078E+01_JPRB, 0.18271E+01_JPRB, 0.20389E+01_JPRB, 0.24184E+01_JPRB, 0.17069E+01_JPRB ,&
&  0.12969E+01_JPRB, 0.12239E+01_JPRB, 0.12546E+01_JPRB, 0.12953E+01_JPRB, 0.12954E+01_JPRB, 0.11621E+01_JPRB, 0.95075E+00_JPRB ,&
&  0.91464E+00_JPRB, 0.12596E+01_JPRB, 0.13939E+01_JPRB, 0.15629E+01_JPRB, 0.14268E+01_JPRB, 0.93219E+00_JPRB, 0.10188E+01_JPRB ,&
&  0.10499E+01_JPRB, 0.10594E+01_JPRB, 0.10301E+01_JPRB, 0.10190E+01_JPRB, 0.10105E+01_JPRB, 0.99970E+00_JPRB, 0.99806E+00_JPRB ,&
&  0.99293E+00_JPRB, 0.99436E+00_JPRB, 0.99727E+00_JPRB, 0.97186E+00_JPRB, 0.93199E+00_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.82449E+00_JPRB, 0.80041E+00_JPRB, 0.78707E+00_JPRB, 0.76963E+00_JPRB, 0.76057E+00_JPRB, 0.75014E+00_JPRB, 0.74575E+00_JPRB ,&
&  0.74430E+00_JPRB, 0.74789E+00_JPRB, 0.76625E+00_JPRB, 0.79767E+00_JPRB, 0.84285E+00_JPRB, 0.90415E+00_JPRB, 0.10124E+01_JPRB ,&
&  0.11209E+01_JPRB, 0.12866E+01_JPRB, 0.14881E+01_JPRB, 0.17213E+01_JPRB, 0.18879E+01_JPRB, 0.19400E+01_JPRB, 0.20442E+01_JPRB ,&
&  0.21139E+01_JPRB, 0.21349E+01_JPRB, 0.21040E+01_JPRB, 0.20457E+01_JPRB, 0.19548E+01_JPRB, 0.19684E+01_JPRB, 0.20589E+01_JPRB ,&
&  0.18756E+01_JPRB, 0.15097E+01_JPRB, 0.14617E+01_JPRB, 0.15021E+01_JPRB, 0.15536E+01_JPRB, 0.15891E+01_JPRB, 0.16530E+01_JPRB ,&
&  0.17078E+01_JPRB, 0.17519E+01_JPRB, 0.18015E+01_JPRB, 0.18586E+01_JPRB, 0.19515E+01_JPRB, 0.20108E+01_JPRB, 0.20887E+01_JPRB ,&
&  0.21537E+01_JPRB, 0.22242E+01_JPRB, 0.23498E+01_JPRB, 0.22451E+01_JPRB, 0.24669E+01_JPRB, 0.26341E+01_JPRB, 0.27912E+01_JPRB ,&
&  0.28519E+01_JPRB, 0.28366E+01_JPRB, 0.30665E+01_JPRB, 0.32463E+01_JPRB, 0.33771E+01_JPRB, 0.34129E+01_JPRB, 0.33171E+01_JPRB ,&
&  0.34416E+01_JPRB, 0.31913E+01_JPRB, 0.17470E+01_JPRB, 0.15131E+01_JPRB, 0.15936E+01_JPRB, 0.14039E+01_JPRB, 0.16019E+01_JPRB ,&
&  0.15860E+01_JPRB, 0.16200E+01_JPRB, 0.16782E+01_JPRB, 0.17917E+01_JPRB, 0.19126E+01_JPRB, 0.22233E+01_JPRB, 0.15666E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.97226E+00_JPRB, 0.97105E+00_JPRB, 0.99501E+00_JPRB, 0.10323E+01_JPRB, 0.99665E+00_JPRB, 0.95404E+00_JPRB ,&
&  0.10010E+01_JPRB, 0.10501E+01_JPRB, 0.10964E+01_JPRB, 0.10552E+01_JPRB, 0.91101E+00_JPRB, 0.89327E+00_JPRB, 0.93124E+00_JPRB ,&
&  0.95816E+00_JPRB, 0.97020E+00_JPRB, 0.96180E+00_JPRB, 0.96302E+00_JPRB, 0.95222E+00_JPRB, 0.94595E+00_JPRB, 0.93961E+00_JPRB ,&
&  0.92822E+00_JPRB, 0.92346E+00_JPRB, 0.90832E+00_JPRB, 0.87060E+00_JPRB, 0.84351E+00_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.73574E+00_JPRB, 0.73730E+00_JPRB, 0.73030E+00_JPRB, 0.72566E+00_JPRB, 0.72435E+00_JPRB, 0.70790E+00_JPRB, 0.72844E+00_JPRB ,&
&  0.72310E+00_JPRB, 0.74359E+00_JPRB, 0.76557E+00_JPRB, 0.78861E+00_JPRB, 0.83400E+00_JPRB, 0.88107E+00_JPRB, 0.94736E+00_JPRB ,&
&  0.10615E+01_JPRB, 0.12471E+01_JPRB, 0.14825E+01_JPRB, 0.17742E+01_JPRB, 0.18687E+01_JPRB, 0.19277E+01_JPRB, 0.20355E+01_JPRB ,&
&  0.21712E+01_JPRB, 0.22798E+01_JPRB, 0.23400E+01_JPRB, 0.24145E+01_JPRB, 0.24816E+01_JPRB, 0.24174E+01_JPRB, 0.22990E+01_JPRB ,&
&  0.18999E+01_JPRB, 0.14807E+01_JPRB, 0.16184E+01_JPRB, 0.15388E+01_JPRB, 0.16326E+01_JPRB, 0.17159E+01_JPRB, 0.17821E+01_JPRB ,&
&  0.18520E+01_JPRB, 0.19025E+01_JPRB, 0.19721E+01_JPRB, 0.20540E+01_JPRB, 0.21654E+01_JPRB, 0.22742E+01_JPRB, 0.23955E+01_JPRB ,&
&  0.24861E+01_JPRB, 0.25781E+01_JPRB, 0.26521E+01_JPRB, 0.26212E+01_JPRB, 0.27945E+01_JPRB, 0.30310E+01_JPRB, 0.32497E+01_JPRB ,&
&  0.34247E+01_JPRB, 0.34082E+01_JPRB, 0.30666E+01_JPRB, 0.32138E+01_JPRB, 0.34519E+01_JPRB, 0.33083E+01_JPRB, 0.34319E+01_JPRB ,&
&  0.37786E+01_JPRB, 0.33155E+01_JPRB, 0.16271E+01_JPRB, 0.13906E+01_JPRB, 0.14468E+01_JPRB, 0.14731E+01_JPRB, 0.14955E+01_JPRB ,&
&  0.14944E+01_JPRB, 0.15136E+01_JPRB, 0.15595E+01_JPRB, 0.16092E+01_JPRB, 0.15937E+01_JPRB, 0.16022E+01_JPRB, 0.13425E+01_JPRB ,&
&  0.90741E+00_JPRB, 0.88209E+00_JPRB, 0.92270E+00_JPRB, 0.96293E+00_JPRB, 0.10380E+01_JPRB, 0.10127E+01_JPRB, 0.90911E+00_JPRB ,&
&  0.98162E+00_JPRB, 0.10894E+01_JPRB, 0.12564E+01_JPRB, 0.11175E+01_JPRB, 0.83332E+00_JPRB, 0.82101E+00_JPRB, 0.82359E+00_JPRB ,&
&  0.84935E+00_JPRB, 0.85144E+00_JPRB, 0.86455E+00_JPRB, 0.87169E+00_JPRB, 0.88276E+00_JPRB, 0.86789E+00_JPRB, 0.85045E+00_JPRB ,&
&  0.82950E+00_JPRB, 0.81100E+00_JPRB, 0.79348E+00_JPRB, 0.76630E+00_JPRB, 0.74633E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.65840E+00_JPRB, 0.62843E+00_JPRB, 0.60306E+00_JPRB, 0.60448E+00_JPRB, 0.60950E+00_JPRB, 0.59569E+00_JPRB, 0.62526E+00_JPRB ,&
&  0.59975E+00_JPRB, 0.62212E+00_JPRB, 0.63425E+00_JPRB, 0.63328E+00_JPRB, 0.67274E+00_JPRB, 0.71368E+00_JPRB, 0.76630E+00_JPRB ,&
&  0.83917E+00_JPRB, 0.95456E+00_JPRB, 0.10990E+01_JPRB, 0.12786E+01_JPRB, 0.14954E+01_JPRB, 0.15972E+01_JPRB, 0.17016E+01_JPRB ,&
&  0.18509E+01_JPRB, 0.20490E+01_JPRB, 0.23322E+01_JPRB, 0.26527E+01_JPRB, 0.30332E+01_JPRB, 0.28712E+01_JPRB, 0.24402E+01_JPRB ,&
&  0.18357E+01_JPRB, 0.15071E+01_JPRB, 0.14515E+01_JPRB, 0.13697E+01_JPRB, 0.14827E+01_JPRB, 0.17057E+01_JPRB, 0.18244E+01_JPRB ,&
&  0.19049E+01_JPRB, 0.19957E+01_JPRB, 0.20965E+01_JPRB, 0.22084E+01_JPRB, 0.23319E+01_JPRB, 0.24723E+01_JPRB, 0.25570E+01_JPRB ,&
&  0.26102E+01_JPRB, 0.26678E+01_JPRB, 0.27235E+01_JPRB, 0.28936E+01_JPRB, 0.31119E+01_JPRB, 0.33929E+01_JPRB, 0.34745E+01_JPRB ,&
&  0.34982E+01_JPRB, 0.34181E+01_JPRB, 0.29696E+01_JPRB, 0.29446E+01_JPRB, 0.30322E+01_JPRB, 0.30617E+01_JPRB, 0.31077E+01_JPRB ,&
&  0.33820E+01_JPRB, 0.26801E+01_JPRB, 0.16252E+01_JPRB, 0.15528E+01_JPRB, 0.15273E+01_JPRB, 0.15368E+01_JPRB, 0.13616E+01_JPRB ,&
&  0.13509E+01_JPRB, 0.13452E+01_JPRB, 0.13515E+01_JPRB, 0.13162E+01_JPRB, 0.13123E+01_JPRB, 0.12943E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.10118E+01_JPRB, 0.96135E+00_JPRB, 0.90900E+00_JPRB, 0.87574E+00_JPRB, 0.10129E+01_JPRB, 0.96225E+00_JPRB, 0.96140E+00_JPRB ,&
&  0.90432E+00_JPRB, 0.97919E+00_JPRB, 0.87569E+00_JPRB, 0.10650E+01_JPRB, 0.79406E+00_JPRB, 0.73562E+00_JPRB, 0.72413E+00_JPRB ,&
&  0.74500E+00_JPRB, 0.75439E+00_JPRB, 0.76654E+00_JPRB, 0.75525E+00_JPRB, 0.76989E+00_JPRB, 0.79768E+00_JPRB, 0.79816E+00_JPRB ,&
&  0.75381E+00_JPRB, 0.73765E+00_JPRB, 0.70990E+00_JPRB, 0.67656E+00_JPRB, 0.66970E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.56717E+00_JPRB, 0.56037E+00_JPRB, 0.55852E+00_JPRB, 0.55536E+00_JPRB, 0.56408E+00_JPRB, 0.56542E+00_JPRB, 0.56612E+00_JPRB ,&
&  0.56545E+00_JPRB, 0.57635E+00_JPRB, 0.58451E+00_JPRB, 0.59719E+00_JPRB, 0.61171E+00_JPRB, 0.64408E+00_JPRB, 0.68071E+00_JPRB ,&
&  0.72396E+00_JPRB, 0.77354E+00_JPRB, 0.82836E+00_JPRB, 0.90764E+00_JPRB, 0.10190E+01_JPRB, 0.10969E+01_JPRB, 0.11829E+01_JPRB ,&
&  0.12963E+01_JPRB, 0.14683E+01_JPRB, 0.16584E+01_JPRB, 0.19144E+01_JPRB, 0.22078E+01_JPRB, 0.22668E+01_JPRB, 0.21044E+01_JPRB ,&
&  0.16087E+01_JPRB, 0.16195E+01_JPRB, 0.15375E+01_JPRB, 0.14129E+01_JPRB, 0.14561E+01_JPRB, 0.14378E+01_JPRB, 0.14971E+01_JPRB ,&
&  0.15791E+01_JPRB, 0.16098E+01_JPRB, 0.16269E+01_JPRB, 0.16713E+01_JPRB, 0.17099E+01_JPRB, 0.17462E+01_JPRB, 0.17409E+01_JPRB ,&
&  0.17480E+01_JPRB, 0.18341E+01_JPRB, 0.21454E+01_JPRB, 0.25369E+01_JPRB, 0.28214E+01_JPRB, 0.29513E+01_JPRB, 0.28145E+01_JPRB ,&
&  0.26092E+01_JPRB, 0.24827E+01_JPRB, 0.23318E+01_JPRB, 0.23004E+01_JPRB, 0.23671E+01_JPRB, 0.23977E+01_JPRB, 0.25493E+01_JPRB ,&
&  0.24032E+01_JPRB, 0.19736E+01_JPRB, 0.17581E+01_JPRB, 0.16154E+01_JPRB, 0.15337E+01_JPRB, 0.14405E+01_JPRB, 0.13334E+01_JPRB ,&
&  0.13326E+01_JPRB, 0.12760E+01_JPRB, 0.12274E+01_JPRB, 0.11393E+01_JPRB, 0.10633E+01_JPRB, 0.10190E+01_JPRB, 0.98787E+00_JPRB ,&
&  0.89896E+00_JPRB, 0.83043E+00_JPRB, 0.80447E+00_JPRB, 0.84274E+00_JPRB, 0.93536E+00_JPRB, 0.13629E+01_JPRB, 0.12803E+01_JPRB ,&
&  0.83024E+00_JPRB, 0.71883E+00_JPRB, 0.57962E+00_JPRB, 0.76774E+00_JPRB, 0.67553E+00_JPRB, 0.65737E+00_JPRB, 0.65944E+00_JPRB ,&
&  0.66457E+00_JPRB, 0.66170E+00_JPRB, 0.64183E+00_JPRB, 0.63813E+00_JPRB, 0.64712E+00_JPRB, 0.65476E+00_JPRB, 0.64413E+00_JPRB ,&
&  0.60752E+00_JPRB, 0.59034E+00_JPRB, 0.61561E+00_JPRB, 0.57723E+00_JPRB, 0.56994E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.52036E+00_JPRB, 0.52027E+00_JPRB, 0.53408E+00_JPRB, 0.54733E+00_JPRB, 0.55474E+00_JPRB, 0.56056E+00_JPRB, 0.57021E+00_JPRB ,&
&  0.57573E+00_JPRB, 0.58391E+00_JPRB, 0.59354E+00_JPRB, 0.61112E+00_JPRB, 0.63676E+00_JPRB, 0.67546E+00_JPRB, 0.71957E+00_JPRB ,&
&  0.77358E+00_JPRB, 0.83185E+00_JPRB, 0.89733E+00_JPRB, 0.96888E+00_JPRB, 0.10579E+01_JPRB, 0.11822E+01_JPRB, 0.13519E+01_JPRB ,&
&  0.14471E+01_JPRB, 0.15677E+01_JPRB, 0.17146E+01_JPRB, 0.18402E+01_JPRB, 0.18589E+01_JPRB, 0.18046E+01_JPRB, 0.14667E+01_JPRB ,&
&  0.17019E+01_JPRB, 0.16277E+01_JPRB, 0.15008E+01_JPRB, 0.13705E+01_JPRB, 0.12794E+01_JPRB, 0.12074E+01_JPRB, 0.11514E+01_JPRB ,&
&  0.11391E+01_JPRB, 0.11027E+01_JPRB, 0.10473E+01_JPRB, 0.10076E+01_JPRB, 0.99205E+00_JPRB, 0.10163E+01_JPRB, 0.10095E+01_JPRB ,&
&  0.11201E+01_JPRB, 0.13660E+01_JPRB, 0.16827E+01_JPRB, 0.20679E+01_JPRB, 0.22870E+01_JPRB, 0.22623E+01_JPRB, 0.18962E+01_JPRB ,&
&  0.16810E+01_JPRB, 0.15970E+01_JPRB, 0.15730E+01_JPRB, 0.16610E+01_JPRB, 0.16699E+01_JPRB, 0.17468E+01_JPRB, 0.17872E+01_JPRB ,&
&  0.17605E+01_JPRB, 0.15445E+01_JPRB, 0.15023E+01_JPRB, 0.17452E+01_JPRB, 0.15772E+01_JPRB, 0.14053E+01_JPRB, 0.13830E+01_JPRB ,&
&  0.12911E+01_JPRB, 0.12296E+01_JPRB, 0.11931E+01_JPRB, 0.11120E+01_JPRB, 0.10213E+01_JPRB, 0.94994E+00_JPRB, 0.91300E+00_JPRB ,&
&  0.83746E+00_JPRB, 0.75661E+00_JPRB, 0.70794E+00_JPRB, 0.72694E+00_JPRB, 0.84607E+00_JPRB, 0.96763E+00_JPRB, 0.13363E+01_JPRB ,&
&  0.81290E+00_JPRB, 0.53501E+00_JPRB, 0.56905E+00_JPRB, 0.62282E+00_JPRB, 0.59147E+00_JPRB, 0.59232E+00_JPRB, 0.65151E+00_JPRB ,&
&  0.72010E+00_JPRB, 0.78797E+00_JPRB, 0.78453E+00_JPRB, 0.67610E+00_JPRB, 0.63309E+00_JPRB, 0.61138E+00_JPRB, 0.58622E+00_JPRB ,&
&  0.56628E+00_JPRB, 0.52979E+00_JPRB, 0.54278E+00_JPRB, 0.52684E+00_JPRB, 0.52428E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.49074E+00_JPRB, 0.48926E+00_JPRB, 0.49731E+00_JPRB, 0.50521E+00_JPRB, 0.50683E+00_JPRB, 0.50843E+00_JPRB, 0.50973E+00_JPRB ,&
&  0.51235E+00_JPRB, 0.51201E+00_JPRB, 0.51410E+00_JPRB, 0.52117E+00_JPRB, 0.53665E+00_JPRB, 0.57037E+00_JPRB, 0.60939E+00_JPRB ,&
&  0.64360E+00_JPRB, 0.67543E+00_JPRB, 0.70645E+00_JPRB, 0.73338E+00_JPRB, 0.76162E+00_JPRB, 0.78156E+00_JPRB, 0.83810E+00_JPRB ,&
&  0.91033E+00_JPRB, 0.99694E+00_JPRB, 0.10856E+01_JPRB, 0.11519E+01_JPRB, 0.12096E+01_JPRB, 0.11672E+01_JPRB, 0.95399E+00_JPRB ,&
&  0.15008E+01_JPRB, 0.14888E+01_JPRB, 0.13682E+01_JPRB, 0.12368E+01_JPRB, 0.11421E+01_JPRB, 0.10573E+01_JPRB, 0.96933E+00_JPRB ,&
&  0.88655E+00_JPRB, 0.92252E+00_JPRB, 0.89245E+00_JPRB, 0.94343E+00_JPRB, 0.88213E+00_JPRB, 0.94352E+00_JPRB, 0.11272E+01_JPRB ,&
&  0.12809E+01_JPRB, 0.14403E+01_JPRB, 0.16351E+01_JPRB, 0.18317E+01_JPRB, 0.19435E+01_JPRB, 0.19075E+01_JPRB, 0.17086E+01_JPRB ,&
&  0.15949E+01_JPRB, 0.15591E+01_JPRB, 0.14984E+01_JPRB, 0.16276E+01_JPRB, 0.15646E+01_JPRB, 0.14615E+01_JPRB, 0.14754E+01_JPRB ,&
&  0.13024E+01_JPRB, 0.13007E+01_JPRB, 0.14595E+01_JPRB, 0.17546E+01_JPRB, 0.15950E+01_JPRB, 0.14770E+01_JPRB, 0.14236E+01_JPRB ,&
&  0.13217E+01_JPRB, 0.12358E+01_JPRB, 0.11742E+01_JPRB, 0.10923E+01_JPRB, 0.10058E+01_JPRB, 0.92738E+00_JPRB, 0.85643E+00_JPRB ,&
&  0.79511E+00_JPRB, 0.72414E+00_JPRB, 0.67127E+00_JPRB, 0.65223E+00_JPRB, 0.67956E+00_JPRB, 0.70783E+00_JPRB, 0.11251E+01_JPRB ,&
&  0.82089E+00_JPRB, 0.66621E+00_JPRB, 0.71741E+00_JPRB, 0.40658E+00_JPRB, 0.49122E+00_JPRB, 0.53132E+00_JPRB, 0.64535E+00_JPRB ,&
&  0.80944E+00_JPRB, 0.12963E+01_JPRB, 0.20190E+01_JPRB, 0.18198E+01_JPRB, 0.13875E+01_JPRB, 0.89020E+00_JPRB, 0.53156E+00_JPRB ,&
&  0.51125E+00_JPRB, 0.50675E+00_JPRB, 0.51149E+00_JPRB, 0.50249E+00_JPRB, 0.49868E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.44757E+00_JPRB, 0.43849E+00_JPRB, 0.44669E+00_JPRB, 0.45624E+00_JPRB, 0.45211E+00_JPRB, 0.45136E+00_JPRB, 0.44612E+00_JPRB ,&
&  0.43659E+00_JPRB, 0.43027E+00_JPRB, 0.45210E+00_JPRB, 0.49328E+00_JPRB, 0.55020E+00_JPRB, 0.62379E+00_JPRB, 0.69590E+00_JPRB ,&
&  0.74163E+00_JPRB, 0.79362E+00_JPRB, 0.83755E+00_JPRB, 0.87258E+00_JPRB, 0.87231E+00_JPRB, 0.86984E+00_JPRB, 0.85224E+00_JPRB ,&
&  0.84057E+00_JPRB, 0.85299E+00_JPRB, 0.89748E+00_JPRB, 0.94044E+00_JPRB, 0.10215E+01_JPRB, 0.11220E+01_JPRB, 0.75678E+00_JPRB ,&
&  0.12333E+01_JPRB, 0.12468E+01_JPRB, 0.11956E+01_JPRB, 0.10909E+01_JPRB, 0.10348E+01_JPRB, 0.94490E+00_JPRB, 0.87207E+00_JPRB ,&
&  0.87551E+00_JPRB, 0.93003E+00_JPRB, 0.96785E+00_JPRB, 0.11168E+01_JPRB, 0.12805E+01_JPRB, 0.13818E+01_JPRB, 0.15939E+01_JPRB ,&
&  0.17754E+01_JPRB, 0.20044E+01_JPRB, 0.22498E+01_JPRB, 0.24180E+01_JPRB, 0.25578E+01_JPRB, 0.26207E+01_JPRB, 0.26148E+01_JPRB ,&
&  0.24499E+01_JPRB, 0.23391E+01_JPRB, 0.24197E+01_JPRB, 0.26071E+01_JPRB, 0.22769E+01_JPRB, 0.21292E+01_JPRB, 0.17807E+01_JPRB ,&
&  0.13202E+01_JPRB, 0.11976E+01_JPRB, 0.13559E+01_JPRB, 0.16256E+01_JPRB, 0.15989E+01_JPRB, 0.15036E+01_JPRB, 0.13687E+01_JPRB ,&
&  0.12609E+01_JPRB, 0.11501E+01_JPRB, 0.10690E+01_JPRB, 0.99559E+00_JPRB, 0.92785E+00_JPRB, 0.85960E+00_JPRB, 0.78896E+00_JPRB ,&
&  0.75016E+00_JPRB, 0.70298E+00_JPRB, 0.65053E+00_JPRB, 0.64316E+00_JPRB, 0.66742E+00_JPRB, 0.76298E+00_JPRB, 0.81380E+00_JPRB ,&
&  0.92757E+00_JPRB, 0.70124E+00_JPRB, 0.61203E+00_JPRB, 0.58675E+00_JPRB, 0.49339E+00_JPRB, 0.52937E+00_JPRB, 0.69747E+00_JPRB ,&
&  0.11067E+01_JPRB, 0.16449E+01_JPRB, 0.27894E+01_JPRB, 0.50181E+01_JPRB, 0.27749E+01_JPRB, 0.21395E+01_JPRB, 0.51557E+00_JPRB ,&
&  0.49388E+00_JPRB, 0.49581E+00_JPRB, 0.49304E+00_JPRB, 0.47642E+00_JPRB, 0.47128E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.43620E+00_JPRB, 0.42791E+00_JPRB, 0.43651E+00_JPRB, 0.43456E+00_JPRB, 0.43780E+00_JPRB, 0.44444E+00_JPRB, 0.44844E+00_JPRB ,&
&  0.45209E+00_JPRB, 0.46854E+00_JPRB, 0.55782E+00_JPRB, 0.68046E+00_JPRB, 0.79600E+00_JPRB, 0.88336E+00_JPRB, 0.92121E+00_JPRB ,&
&  0.95711E+00_JPRB, 0.97071E+00_JPRB, 0.95712E+00_JPRB, 0.93758E+00_JPRB, 0.94498E+00_JPRB, 0.98703E+00_JPRB, 0.10202E+01_JPRB ,&
&  0.10501E+01_JPRB, 0.10787E+01_JPRB, 0.11095E+01_JPRB, 0.11593E+01_JPRB, 0.12090E+01_JPRB, 0.11537E+01_JPRB, 0.73163E+00_JPRB ,&
&  0.10431E+01_JPRB, 0.11094E+01_JPRB, 0.11215E+01_JPRB, 0.10423E+01_JPRB, 0.96832E+00_JPRB, 0.92678E+00_JPRB, 0.99704E+00_JPRB ,&
&  0.11043E+01_JPRB, 0.11790E+01_JPRB, 0.12250E+01_JPRB, 0.13114E+01_JPRB, 0.18418E+01_JPRB, 0.20484E+01_JPRB, 0.24147E+01_JPRB ,&
&  0.26845E+01_JPRB, 0.29719E+01_JPRB, 0.31801E+01_JPRB, 0.34106E+01_JPRB, 0.36040E+01_JPRB, 0.37411E+01_JPRB, 0.39334E+01_JPRB ,&
&  0.40042E+01_JPRB, 0.40991E+01_JPRB, 0.41143E+01_JPRB, 0.40076E+01_JPRB, 0.32615E+01_JPRB, 0.34637E+01_JPRB, 0.23451E+01_JPRB ,&
&  0.16548E+01_JPRB, 0.14184E+01_JPRB, 0.15638E+01_JPRB, 0.15983E+01_JPRB, 0.14808E+01_JPRB, 0.13402E+01_JPRB, 0.11706E+01_JPRB ,&
&  0.10827E+01_JPRB, 0.10185E+01_JPRB, 0.96443E+00_JPRB, 0.90372E+00_JPRB, 0.87344E+00_JPRB, 0.82505E+00_JPRB, 0.78527E+00_JPRB ,&
&  0.76950E+00_JPRB, 0.72918E+00_JPRB, 0.68016E+00_JPRB, 0.68482E+00_JPRB, 0.73195E+00_JPRB, 0.89632E+00_JPRB, 0.12207E+01_JPRB ,&
&  0.13521E+01_JPRB, 0.77345E+00_JPRB, 0.69370E+00_JPRB, 0.63317E+00_JPRB, 0.60820E+00_JPRB, 0.63404E+00_JPRB, 0.71661E+00_JPRB ,&
&  0.93365E+00_JPRB, 0.16554E+01_JPRB, 0.25430E+01_JPRB, 0.19317E+01_JPRB, 0.22029E+01_JPRB, 0.15144E+01_JPRB, 0.61575E+00_JPRB ,&
&  0.47980E+00_JPRB, 0.46875E+00_JPRB, 0.47411E+00_JPRB, 0.46450E+00_JPRB, 0.45163E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.48727E+00_JPRB, 0.47623E+00_JPRB, 0.47511E+00_JPRB, 0.48507E+00_JPRB, 0.49128E+00_JPRB, 0.51556E+00_JPRB, 0.54592E+00_JPRB ,&
&  0.56158E+00_JPRB, 0.63150E+00_JPRB, 0.78803E+00_JPRB, 0.92153E+00_JPRB, 0.97684E+00_JPRB, 0.10165E+01_JPRB, 0.97084E+00_JPRB ,&
&  0.95946E+00_JPRB, 0.93000E+00_JPRB, 0.88301E+00_JPRB, 0.87817E+00_JPRB, 0.89657E+00_JPRB, 0.93410E+00_JPRB, 0.98351E+00_JPRB ,&
&  0.10398E+01_JPRB, 0.10974E+01_JPRB, 0.11456E+01_JPRB, 0.11907E+01_JPRB, 0.12526E+01_JPRB, 0.12394E+01_JPRB, 0.11759E+01_JPRB ,&
&  0.62189E+00_JPRB, 0.10374E+01_JPRB, 0.10567E+01_JPRB, 0.10030E+01_JPRB, 0.95580E+00_JPRB, 0.97849E+00_JPRB, 0.11496E+01_JPRB ,&
&  0.13342E+01_JPRB, 0.14090E+01_JPRB, 0.16291E+01_JPRB, 0.17630E+01_JPRB, 0.22964E+01_JPRB, 0.26219E+01_JPRB, 0.29229E+01_JPRB ,&
&  0.32050E+01_JPRB, 0.34179E+01_JPRB, 0.36480E+01_JPRB, 0.38761E+01_JPRB, 0.41008E+01_JPRB, 0.42941E+01_JPRB, 0.45201E+01_JPRB ,&
&  0.47842E+01_JPRB, 0.51465E+01_JPRB, 0.55248E+01_JPRB, 0.53323E+01_JPRB, 0.45163E+01_JPRB, 0.42207E+01_JPRB, 0.33398E+01_JPRB ,&
&  0.22313E+01_JPRB, 0.14084E+01_JPRB, 0.14179E+01_JPRB, 0.14936E+01_JPRB, 0.13651E+01_JPRB, 0.12376E+01_JPRB, 0.11197E+01_JPRB ,&
&  0.10420E+01_JPRB, 0.97057E+00_JPRB, 0.88048E+00_JPRB, 0.87125E+00_JPRB, 0.88309E+00_JPRB, 0.85965E+00_JPRB, 0.83331E+00_JPRB ,&
&  0.83003E+00_JPRB, 0.80457E+00_JPRB, 0.77377E+00_JPRB, 0.75167E+00_JPRB, 0.76243E+00_JPRB, 0.81493E+00_JPRB, 0.86018E+00_JPRB ,&
&  0.88885E+00_JPRB, 0.80737E+00_JPRB, 0.74729E+00_JPRB, 0.74425E+00_JPRB, 0.74024E+00_JPRB, 0.74214E+00_JPRB, 0.73661E+00_JPRB ,&
&  0.78765E+00_JPRB, 0.94667E+00_JPRB, 0.13385E+01_JPRB, 0.17343E+01_JPRB, 0.14021E+01_JPRB, 0.11066E+01_JPRB, 0.63254E+00_JPRB ,&
&  0.50581E+00_JPRB, 0.48892E+00_JPRB, 0.50686E+00_JPRB, 0.51059E+00_JPRB, 0.50981E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.59545E+00_JPRB, 0.58296E+00_JPRB, 0.56560E+00_JPRB, 0.58139E+00_JPRB, 0.61445E+00_JPRB, 0.65671E+00_JPRB, 0.67490E+00_JPRB ,&
&  0.69867E+00_JPRB, 0.79948E+00_JPRB, 0.93293E+00_JPRB, 0.10510E+01_JPRB, 0.10719E+01_JPRB, 0.10407E+01_JPRB, 0.94286E+00_JPRB ,&
&  0.91736E+00_JPRB, 0.89169E+00_JPRB, 0.87681E+00_JPRB, 0.86167E+00_JPRB, 0.86713E+00_JPRB, 0.90157E+00_JPRB, 0.95953E+00_JPRB ,&
&  0.10333E+01_JPRB, 0.11131E+01_JPRB, 0.12024E+01_JPRB, 0.12982E+01_JPRB, 0.13819E+01_JPRB, 0.14050E+01_JPRB, 0.12889E+01_JPRB ,&
&  0.55314E+00_JPRB, 0.68535E+00_JPRB, 0.10168E+01_JPRB, 0.10280E+01_JPRB, 0.10168E+01_JPRB, 0.10651E+01_JPRB, 0.12202E+01_JPRB ,&
&  0.13793E+01_JPRB, 0.14673E+01_JPRB, 0.17068E+01_JPRB, 0.21569E+01_JPRB, 0.24115E+01_JPRB, 0.27608E+01_JPRB, 0.29710E+01_JPRB ,&
&  0.31850E+01_JPRB, 0.34006E+01_JPRB, 0.36371E+01_JPRB, 0.38793E+01_JPRB, 0.41217E+01_JPRB, 0.43467E+01_JPRB, 0.46332E+01_JPRB ,&
&  0.50179E+01_JPRB, 0.54401E+01_JPRB, 0.57985E+01_JPRB, 0.52452E+01_JPRB, 0.55730E+01_JPRB, 0.48677E+01_JPRB, 0.41799E+01_JPRB ,&
&  0.24424E+01_JPRB, 0.14562E+01_JPRB, 0.14537E+01_JPRB, 0.15377E+01_JPRB, 0.13431E+01_JPRB, 0.11779E+01_JPRB, 0.10138E+01_JPRB ,&
&  0.94116E+00_JPRB, 0.86459E+00_JPRB, 0.87475E+00_JPRB, 0.89332E+00_JPRB, 0.87823E+00_JPRB, 0.91205E+00_JPRB, 0.86441E+00_JPRB ,&
&  0.87266E+00_JPRB, 0.88119E+00_JPRB, 0.86979E+00_JPRB, 0.87312E+00_JPRB, 0.87262E+00_JPRB, 0.87578E+00_JPRB, 0.86278E+00_JPRB ,&
&  0.86677E+00_JPRB, 0.85735E+00_JPRB, 0.85774E+00_JPRB, 0.86546E+00_JPRB, 0.86084E+00_JPRB, 0.84701E+00_JPRB, 0.80313E+00_JPRB ,&
&  0.75907E+00_JPRB, 0.77152E+00_JPRB, 0.87350E+00_JPRB, 0.10064E+01_JPRB, 0.96289E+00_JPRB, 0.83137E+00_JPRB, 0.66016E+00_JPRB ,&
&  0.64580E+00_JPRB, 0.66676E+00_JPRB, 0.66850E+00_JPRB, 0.64883E+00_JPRB, 0.62476E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.70948E+00_JPRB, 0.68330E+00_JPRB, 0.64491E+00_JPRB, 0.63723E+00_JPRB, 0.69135E+00_JPRB, 0.73938E+00_JPRB, 0.74157E+00_JPRB ,&
&  0.74977E+00_JPRB, 0.81462E+00_JPRB, 0.93224E+00_JPRB, 0.10256E+01_JPRB, 0.10560E+01_JPRB, 0.10484E+01_JPRB, 0.95636E+00_JPRB ,&
&  0.92131E+00_JPRB, 0.89892E+00_JPRB, 0.87253E+00_JPRB, 0.86068E+00_JPRB, 0.85829E+00_JPRB, 0.86720E+00_JPRB, 0.89298E+00_JPRB ,&
&  0.93199E+00_JPRB, 0.10014E+01_JPRB, 0.11148E+01_JPRB, 0.12741E+01_JPRB, 0.15086E+01_JPRB, 0.17797E+01_JPRB, 0.20134E+01_JPRB ,&
&  0.15074E+01_JPRB, 0.65740E+00_JPRB, 0.76665E+00_JPRB, 0.10854E+01_JPRB, 0.10594E+01_JPRB, 0.11340E+01_JPRB, 0.11618E+01_JPRB ,&
&  0.12633E+01_JPRB, 0.14702E+01_JPRB, 0.16926E+01_JPRB, 0.21007E+01_JPRB, 0.23771E+01_JPRB, 0.26641E+01_JPRB, 0.28042E+01_JPRB ,&
&  0.29453E+01_JPRB, 0.31128E+01_JPRB, 0.32833E+01_JPRB, 0.34607E+01_JPRB, 0.36576E+01_JPRB, 0.38781E+01_JPRB, 0.42041E+01_JPRB ,&
&  0.46164E+01_JPRB, 0.50179E+01_JPRB, 0.53014E+01_JPRB, 0.45685E+01_JPRB, 0.48276E+01_JPRB, 0.38116E+01_JPRB, 0.28513E+01_JPRB ,&
&  0.21964E+01_JPRB, 0.17428E+01_JPRB, 0.15871E+01_JPRB, 0.16058E+01_JPRB, 0.13643E+01_JPRB, 0.10484E+01_JPRB, 0.10073E+01_JPRB ,&
&  0.90696E+00_JPRB, 0.87628E+00_JPRB, 0.88655E+00_JPRB, 0.83733E+00_JPRB, 0.89428E+00_JPRB, 0.91469E+00_JPRB, 0.92669E+00_JPRB ,&
&  0.90800E+00_JPRB, 0.92692E+00_JPRB, 0.95119E+00_JPRB, 0.99070E+00_JPRB, 0.97716E+00_JPRB, 0.97234E+00_JPRB, 0.94942E+00_JPRB ,&
&  0.95123E+00_JPRB, 0.95630E+00_JPRB, 0.96630E+00_JPRB, 0.98286E+00_JPRB, 0.98400E+00_JPRB, 0.90862E+00_JPRB, 0.85790E+00_JPRB ,&
&  0.83195E+00_JPRB, 0.82154E+00_JPRB, 0.79130E+00_JPRB, 0.83336E+00_JPRB, 0.84131E+00_JPRB, 0.85005E+00_JPRB, 0.88279E+00_JPRB ,&
&  0.89554E+00_JPRB, 0.89619E+00_JPRB, 0.84506E+00_JPRB, 0.78788E+00_JPRB, 0.75147E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.77328E+00_JPRB, 0.72680E+00_JPRB, 0.69402E+00_JPRB, 0.70376E+00_JPRB, 0.75334E+00_JPRB, 0.80442E+00_JPRB, 0.80231E+00_JPRB ,&
&  0.78779E+00_JPRB, 0.80867E+00_JPRB, 0.87901E+00_JPRB, 0.94032E+00_JPRB, 0.94956E+00_JPRB, 0.95864E+00_JPRB, 0.97962E+00_JPRB ,&
&  0.93092E+00_JPRB, 0.89922E+00_JPRB, 0.88384E+00_JPRB, 0.86389E+00_JPRB, 0.86303E+00_JPRB, 0.85756E+00_JPRB, 0.86113E+00_JPRB ,&
&  0.88182E+00_JPRB, 0.91100E+00_JPRB, 0.97629E+00_JPRB, 0.10780E+01_JPRB, 0.12439E+01_JPRB, 0.15854E+01_JPRB, 0.21093E+01_JPRB ,&
&  0.29659E+01_JPRB, 0.19604E+01_JPRB, 0.74785E+00_JPRB, 0.10111E+01_JPRB, 0.11244E+01_JPRB, 0.11968E+01_JPRB, 0.10932E+01_JPRB ,&
&  0.11517E+01_JPRB, 0.12685E+01_JPRB, 0.14633E+01_JPRB, 0.19017E+01_JPRB, 0.21786E+01_JPRB, 0.23865E+01_JPRB, 0.25205E+01_JPRB ,&
&  0.25769E+01_JPRB, 0.26749E+01_JPRB, 0.27626E+01_JPRB, 0.28416E+01_JPRB, 0.29366E+01_JPRB, 0.30804E+01_JPRB, 0.33201E+01_JPRB ,&
&  0.37700E+01_JPRB, 0.43261E+01_JPRB, 0.47968E+01_JPRB, 0.41827E+01_JPRB, 0.41974E+01_JPRB, 0.32242E+01_JPRB, 0.26545E+01_JPRB ,&
&  0.22812E+01_JPRB, 0.22186E+01_JPRB, 0.20911E+01_JPRB, 0.17329E+01_JPRB, 0.13930E+01_JPRB, 0.11823E+01_JPRB, 0.11836E+01_JPRB ,&
&  0.10059E+01_JPRB, 0.91671E+00_JPRB, 0.89807E+00_JPRB, 0.92769E+00_JPRB, 0.94583E+00_JPRB, 0.95402E+00_JPRB, 0.95251E+00_JPRB ,&
&  0.96871E+00_JPRB, 0.96887E+00_JPRB, 0.10120E+01_JPRB, 0.10408E+01_JPRB, 0.10609E+01_JPRB, 0.10982E+01_JPRB, 0.10929E+01_JPRB ,&
&  0.10753E+01_JPRB, 0.10646E+01_JPRB, 0.10667E+01_JPRB, 0.10769E+01_JPRB, 0.10225E+01_JPRB, 0.94192E+00_JPRB, 0.93464E+00_JPRB ,&
&  0.92661E+00_JPRB, 0.92172E+00_JPRB, 0.88701E+00_JPRB, 0.89624E+00_JPRB, 0.90412E+00_JPRB, 0.96398E+00_JPRB, 0.10364E+01_JPRB ,&
&  0.10677E+01_JPRB, 0.10385E+01_JPRB, 0.96770E+00_JPRB, 0.83807E+00_JPRB, 0.82270E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.83236E+00_JPRB, 0.81824E+00_JPRB, 0.78997E+00_JPRB, 0.77055E+00_JPRB, 0.79337E+00_JPRB, 0.82831E+00_JPRB, 0.82743E+00_JPRB ,&
&  0.80255E+00_JPRB, 0.78830E+00_JPRB, 0.82422E+00_JPRB, 0.86263E+00_JPRB, 0.86065E+00_JPRB, 0.87887E+00_JPRB, 0.88897E+00_JPRB ,&
&  0.92270E+00_JPRB, 0.91691E+00_JPRB, 0.89703E+00_JPRB, 0.87032E+00_JPRB, 0.86745E+00_JPRB, 0.85428E+00_JPRB, 0.85368E+00_JPRB ,&
&  0.86471E+00_JPRB, 0.88171E+00_JPRB, 0.91624E+00_JPRB, 0.97197E+00_JPRB, 0.10372E+01_JPRB, 0.11804E+01_JPRB, 0.14768E+01_JPRB ,&
&  0.18534E+01_JPRB, 0.24018E+01_JPRB, 0.87898E+00_JPRB, 0.11445E+01_JPRB, 0.13893E+01_JPRB, 0.13441E+01_JPRB, 0.12291E+01_JPRB ,&
&  0.11718E+01_JPRB, 0.11145E+01_JPRB, 0.12048E+01_JPRB, 0.15090E+01_JPRB, 0.17856E+01_JPRB, 0.19697E+01_JPRB, 0.19758E+01_JPRB ,&
&  0.20307E+01_JPRB, 0.21392E+01_JPRB, 0.23190E+01_JPRB, 0.23680E+01_JPRB, 0.23707E+01_JPRB, 0.23582E+01_JPRB, 0.24524E+01_JPRB ,&
&  0.27649E+01_JPRB, 0.33088E+01_JPRB, 0.41123E+01_JPRB, 0.38483E+01_JPRB, 0.38500E+01_JPRB, 0.33538E+01_JPRB, 0.29472E+01_JPRB ,&
&  0.26252E+01_JPRB, 0.25968E+01_JPRB, 0.24328E+01_JPRB, 0.19789E+01_JPRB, 0.15457E+01_JPRB, 0.13518E+01_JPRB, 0.13182E+01_JPRB ,&
&  0.11441E+01_JPRB, 0.10294E+01_JPRB, 0.10135E+01_JPRB, 0.10135E+01_JPRB, 0.10511E+01_JPRB, 0.10346E+01_JPRB, 0.10077E+01_JPRB ,&
&  0.10229E+01_JPRB, 0.10007E+01_JPRB, 0.10205E+01_JPRB, 0.10570E+01_JPRB, 0.10937E+01_JPRB, 0.11496E+01_JPRB, 0.11743E+01_JPRB ,&
&  0.11649E+01_JPRB, 0.11631E+01_JPRB, 0.11460E+01_JPRB, 0.10919E+01_JPRB, 0.10192E+01_JPRB, 0.97936E+00_JPRB, 0.97215E+00_JPRB ,&
&  0.98246E+00_JPRB, 0.98983E+00_JPRB, 0.95761E+00_JPRB, 0.91305E+00_JPRB, 0.94752E+00_JPRB, 0.96187E+00_JPRB, 0.10510E+01_JPRB ,&
&  0.11585E+01_JPRB, 0.12269E+01_JPRB, 0.11792E+01_JPRB, 0.90380E+00_JPRB, 0.88903E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.92512E+00_JPRB, 0.88787E+00_JPRB, 0.86604E+00_JPRB, 0.84375E+00_JPRB, 0.82419E+00_JPRB, 0.85631E+00_JPRB, 0.84872E+00_JPRB ,&
&  0.82540E+00_JPRB, 0.79559E+00_JPRB, 0.81308E+00_JPRB, 0.86107E+00_JPRB, 0.86446E+00_JPRB, 0.84525E+00_JPRB, 0.83171E+00_JPRB ,&
&  0.83087E+00_JPRB, 0.84715E+00_JPRB, 0.85033E+00_JPRB, 0.86346E+00_JPRB, 0.86578E+00_JPRB, 0.85655E+00_JPRB, 0.85113E+00_JPRB ,&
&  0.86740E+00_JPRB, 0.87965E+00_JPRB, 0.89515E+00_JPRB, 0.90913E+00_JPRB, 0.93431E+00_JPRB, 0.95697E+00_JPRB, 0.10949E+01_JPRB ,&
&  0.12877E+01_JPRB, 0.20232E+01_JPRB, 0.16612E+01_JPRB, 0.19682E+01_JPRB, 0.18531E+01_JPRB, 0.16837E+01_JPRB, 0.14527E+01_JPRB ,&
&  0.14485E+01_JPRB, 0.13407E+01_JPRB, 0.13825E+01_JPRB, 0.12554E+01_JPRB, 0.13248E+01_JPRB, 0.14450E+01_JPRB, 0.14407E+01_JPRB ,&
&  0.14005E+01_JPRB, 0.13884E+01_JPRB, 0.16206E+01_JPRB, 0.17360E+01_JPRB, 0.17897E+01_JPRB, 0.18060E+01_JPRB, 0.18329E+01_JPRB ,&
&  0.19947E+01_JPRB, 0.22867E+01_JPRB, 0.30485E+01_JPRB, 0.36295E+01_JPRB, 0.35456E+01_JPRB, 0.34748E+01_JPRB, 0.33969E+01_JPRB ,&
&  0.34304E+01_JPRB, 0.30791E+01_JPRB, 0.27206E+01_JPRB, 0.22086E+01_JPRB, 0.17252E+01_JPRB, 0.16535E+01_JPRB, 0.14205E+01_JPRB ,&
&  0.12445E+01_JPRB, 0.11316E+01_JPRB, 0.11157E+01_JPRB, 0.11183E+01_JPRB, 0.11369E+01_JPRB, 0.11072E+01_JPRB, 0.10603E+01_JPRB ,&
&  0.10426E+01_JPRB, 0.10463E+01_JPRB, 0.10284E+01_JPRB, 0.10496E+01_JPRB, 0.10688E+01_JPRB, 0.11044E+01_JPRB, 0.11350E+01_JPRB ,&
&  0.11692E+01_JPRB, 0.11904E+01_JPRB, 0.11423E+01_JPRB, 0.10952E+01_JPRB, 0.10391E+01_JPRB, 0.10048E+01_JPRB, 0.97581E+00_JPRB ,&
&  0.98862E+00_JPRB, 0.10132E+01_JPRB, 0.10093E+01_JPRB, 0.96925E+00_JPRB, 0.94592E+00_JPRB, 0.93793E+00_JPRB, 0.94514E+00_JPRB ,&
&  0.96777E+00_JPRB, 0.10233E+01_JPRB, 0.10199E+01_JPRB, 0.96569E+00_JPRB, 0.95133E+00_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.98153E+00_JPRB, 0.96366E+00_JPRB, 0.92056E+00_JPRB, 0.91118E+00_JPRB, 0.87933E+00_JPRB, 0.90429E+00_JPRB, 0.90960E+00_JPRB ,&
&  0.87653E+00_JPRB, 0.84465E+00_JPRB, 0.82303E+00_JPRB, 0.84362E+00_JPRB, 0.86623E+00_JPRB, 0.85415E+00_JPRB, 0.81539E+00_JPRB ,&
&  0.80229E+00_JPRB, 0.79923E+00_JPRB, 0.79805E+00_JPRB, 0.80255E+00_JPRB, 0.81987E+00_JPRB, 0.84497E+00_JPRB, 0.84400E+00_JPRB ,&
&  0.84128E+00_JPRB, 0.87350E+00_JPRB, 0.90437E+00_JPRB, 0.90658E+00_JPRB, 0.90153E+00_JPRB, 0.89710E+00_JPRB, 0.95722E+00_JPRB ,&
&  0.10482E+01_JPRB, 0.21079E+01_JPRB, 0.15428E+01_JPRB, 0.25331E+01_JPRB, 0.20221E+01_JPRB, 0.17886E+01_JPRB, 0.14567E+01_JPRB ,&
&  0.13391E+01_JPRB, 0.14339E+01_JPRB, 0.12599E+01_JPRB, 0.11290E+01_JPRB, 0.10699E+01_JPRB, 0.10792E+01_JPRB, 0.10567E+01_JPRB ,&
&  0.97478E+00_JPRB, 0.98074E+00_JPRB, 0.10582E+01_JPRB, 0.11765E+01_JPRB, 0.12628E+01_JPRB, 0.13090E+01_JPRB, 0.13748E+01_JPRB ,&
&  0.14807E+01_JPRB, 0.16191E+01_JPRB, 0.21211E+01_JPRB, 0.27834E+01_JPRB, 0.33995E+01_JPRB, 0.36186E+01_JPRB, 0.39242E+01_JPRB ,&
&  0.40385E+01_JPRB, 0.37043E+01_JPRB, 0.30529E+01_JPRB, 0.24308E+01_JPRB, 0.19935E+01_JPRB, 0.18340E+01_JPRB, 0.16508E+01_JPRB ,&
&  0.14686E+01_JPRB, 0.13006E+01_JPRB, 0.12603E+01_JPRB, 0.12282E+01_JPRB, 0.12435E+01_JPRB, 0.12120E+01_JPRB, 0.11316E+01_JPRB ,&
&  0.10672E+01_JPRB, 0.10515E+01_JPRB, 0.10596E+01_JPRB, 0.10748E+01_JPRB, 0.10960E+01_JPRB, 0.10967E+01_JPRB, 0.10991E+01_JPRB ,&
&  0.11296E+01_JPRB, 0.11698E+01_JPRB, 0.11434E+01_JPRB, 0.11317E+01_JPRB, 0.10882E+01_JPRB, 0.10524E+01_JPRB, 0.10346E+01_JPRB ,&
&  0.10632E+01_JPRB, 0.10558E+01_JPRB, 0.10162E+01_JPRB, 0.96834E+00_JPRB, 0.94546E+00_JPRB, 0.95366E+00_JPRB, 0.92267E+00_JPRB ,&
&  0.92463E+00_JPRB, 0.96797E+00_JPRB, 0.10175E+01_JPRB, 0.98204E+00_JPRB, 0.97841E+00_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.10007E+01_JPRB, 0.99879E+00_JPRB, 0.94494E+00_JPRB, 0.92842E+00_JPRB, 0.90190E+00_JPRB, 0.89485E+00_JPRB, 0.90633E+00_JPRB ,&
&  0.89846E+00_JPRB, 0.87442E+00_JPRB, 0.84378E+00_JPRB, 0.83092E+00_JPRB, 0.84345E+00_JPRB, 0.85638E+00_JPRB, 0.82391E+00_JPRB ,&
&  0.78647E+00_JPRB, 0.77292E+00_JPRB, 0.76503E+00_JPRB, 0.76777E+00_JPRB, 0.77986E+00_JPRB, 0.79752E+00_JPRB, 0.80814E+00_JPRB ,&
&  0.80467E+00_JPRB, 0.84434E+00_JPRB, 0.88979E+00_JPRB, 0.90781E+00_JPRB, 0.89455E+00_JPRB, 0.89079E+00_JPRB, 0.92700E+00_JPRB ,&
&  0.10652E+01_JPRB, 0.26001E+01_JPRB, 0.16550E+01_JPRB, 0.26024E+01_JPRB, 0.20697E+01_JPRB, 0.17709E+01_JPRB, 0.14274E+01_JPRB ,&
&  0.13278E+01_JPRB, 0.13056E+01_JPRB, 0.11977E+01_JPRB, 0.10953E+01_JPRB, 0.10271E+01_JPRB, 0.98387E+00_JPRB, 0.94942E+00_JPRB ,&
&  0.91224E+00_JPRB, 0.89067E+00_JPRB, 0.89534E+00_JPRB, 0.96086E+00_JPRB, 0.10158E+01_JPRB, 0.10502E+01_JPRB, 0.10852E+01_JPRB ,&
&  0.11490E+01_JPRB, 0.12412E+01_JPRB, 0.15409E+01_JPRB, 0.23764E+01_JPRB, 0.30256E+01_JPRB, 0.36327E+01_JPRB, 0.53078E+01_JPRB ,&
&  0.51079E+01_JPRB, 0.46627E+01_JPRB, 0.34205E+01_JPRB, 0.26962E+01_JPRB, 0.22255E+01_JPRB, 0.19556E+01_JPRB, 0.18376E+01_JPRB ,&
&  0.17127E+01_JPRB, 0.15394E+01_JPRB, 0.14628E+01_JPRB, 0.14088E+01_JPRB, 0.13554E+01_JPRB, 0.12952E+01_JPRB, 0.12173E+01_JPRB ,&
&  0.11530E+01_JPRB, 0.11065E+01_JPRB, 0.11112E+01_JPRB, 0.11206E+01_JPRB, 0.11294E+01_JPRB, 0.11128E+01_JPRB, 0.11017E+01_JPRB ,&
&  0.11119E+01_JPRB, 0.11455E+01_JPRB, 0.11337E+01_JPRB, 0.11379E+01_JPRB, 0.11240E+01_JPRB, 0.10950E+01_JPRB, 0.10752E+01_JPRB ,&
&  0.11163E+01_JPRB, 0.11118E+01_JPRB, 0.10465E+01_JPRB, 0.10100E+01_JPRB, 0.96820E+00_JPRB, 0.98658E+00_JPRB, 0.93745E+00_JPRB ,&
&  0.91712E+00_JPRB, 0.91453E+00_JPRB, 0.98221E+00_JPRB, 0.98917E+00_JPRB, 0.10039E+01_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.10098E+01_JPRB, 0.99429E+00_JPRB, 0.96792E+00_JPRB, 0.92526E+00_JPRB, 0.90937E+00_JPRB, 0.89113E+00_JPRB, 0.88169E+00_JPRB ,&
&  0.87919E+00_JPRB, 0.86679E+00_JPRB, 0.85357E+00_JPRB, 0.83379E+00_JPRB, 0.82612E+00_JPRB, 0.83778E+00_JPRB, 0.83235E+00_JPRB ,&
&  0.81417E+00_JPRB, 0.79026E+00_JPRB, 0.78180E+00_JPRB, 0.77223E+00_JPRB, 0.76226E+00_JPRB, 0.76327E+00_JPRB, 0.77419E+00_JPRB ,&
&  0.78037E+00_JPRB, 0.79927E+00_JPRB, 0.83028E+00_JPRB, 0.85855E+00_JPRB, 0.85784E+00_JPRB, 0.85507E+00_JPRB, 0.90404E+00_JPRB ,&
&  0.10896E+01_JPRB, 0.10972E+01_JPRB, 0.21955E+01_JPRB, 0.24900E+01_JPRB, 0.20856E+01_JPRB, 0.18089E+01_JPRB, 0.15379E+01_JPRB ,&
&  0.14135E+01_JPRB, 0.12511E+01_JPRB, 0.11226E+01_JPRB, 0.10292E+01_JPRB, 0.98025E+00_JPRB, 0.92292E+00_JPRB, 0.90550E+00_JPRB ,&
&  0.89681E+00_JPRB, 0.87679E+00_JPRB, 0.87195E+00_JPRB, 0.89156E+00_JPRB, 0.90396E+00_JPRB, 0.91902E+00_JPRB, 0.93791E+00_JPRB ,&
&  0.98072E+00_JPRB, 0.10377E+01_JPRB, 0.11928E+01_JPRB, 0.17323E+01_JPRB, 0.27589E+01_JPRB, 0.37750E+01_JPRB, 0.52914E+01_JPRB ,&
&  0.59967E+01_JPRB, 0.54365E+01_JPRB, 0.38090E+01_JPRB, 0.29182E+01_JPRB, 0.24390E+01_JPRB, 0.21223E+01_JPRB, 0.19728E+01_JPRB ,&
&  0.18674E+01_JPRB, 0.17338E+01_JPRB, 0.16298E+01_JPRB, 0.16145E+01_JPRB, 0.15075E+01_JPRB, 0.13960E+01_JPRB, 0.12962E+01_JPRB ,&
&  0.12383E+01_JPRB, 0.11902E+01_JPRB, 0.11431E+01_JPRB, 0.11128E+01_JPRB, 0.11189E+01_JPRB, 0.11142E+01_JPRB, 0.10992E+01_JPRB ,&
&  0.10973E+01_JPRB, 0.11131E+01_JPRB, 0.11115E+01_JPRB, 0.11106E+01_JPRB, 0.11334E+01_JPRB, 0.11370E+01_JPRB, 0.11368E+01_JPRB ,&
&  0.11260E+01_JPRB, 0.11286E+01_JPRB, 0.10840E+01_JPRB, 0.10545E+01_JPRB, 0.99004E+00_JPRB, 0.10264E+01_JPRB, 0.97335E+00_JPRB ,&
&  0.93488E+00_JPRB, 0.90688E+00_JPRB, 0.92190E+00_JPRB, 0.97384E+00_JPRB, 0.10098E+01_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.98621E+00_JPRB, 0.97335E+00_JPRB, 0.95647E+00_JPRB, 0.93340E+00_JPRB, 0.91260E+00_JPRB, 0.90207E+00_JPRB, 0.89093E+00_JPRB ,&
&  0.89243E+00_JPRB, 0.89431E+00_JPRB, 0.88563E+00_JPRB, 0.86037E+00_JPRB, 0.83954E+00_JPRB, 0.83579E+00_JPRB, 0.84100E+00_JPRB ,&
&  0.83518E+00_JPRB, 0.82439E+00_JPRB, 0.82746E+00_JPRB, 0.81027E+00_JPRB, 0.78848E+00_JPRB, 0.77479E+00_JPRB, 0.77011E+00_JPRB ,&
&  0.77556E+00_JPRB, 0.78669E+00_JPRB, 0.81307E+00_JPRB, 0.83846E+00_JPRB, 0.84134E+00_JPRB, 0.83427E+00_JPRB, 0.82708E+00_JPRB ,&
&  0.99220E+00_JPRB, 0.14582E+01_JPRB, 0.23377E+01_JPRB, 0.22411E+01_JPRB, 0.19613E+01_JPRB, 0.16764E+01_JPRB, 0.15404E+01_JPRB ,&
&  0.13502E+01_JPRB, 0.12358E+01_JPRB, 0.11213E+01_JPRB, 0.10191E+01_JPRB, 0.94780E+00_JPRB, 0.90757E+00_JPRB, 0.88823E+00_JPRB ,&
&  0.86391E+00_JPRB, 0.84795E+00_JPRB, 0.84745E+00_JPRB, 0.85209E+00_JPRB, 0.85671E+00_JPRB, 0.85479E+00_JPRB, 0.85711E+00_JPRB ,&
&  0.88894E+00_JPRB, 0.93683E+00_JPRB, 0.10200E+01_JPRB, 0.13046E+01_JPRB, 0.23185E+01_JPRB, 0.28903E+01_JPRB, 0.45242E+01_JPRB ,&
&  0.61726E+01_JPRB, 0.51202E+01_JPRB, 0.39089E+01_JPRB, 0.31011E+01_JPRB, 0.26035E+01_JPRB, 0.22861E+01_JPRB, 0.20988E+01_JPRB ,&
&  0.19154E+01_JPRB, 0.18048E+01_JPRB, 0.17076E+01_JPRB, 0.16051E+01_JPRB, 0.15329E+01_JPRB, 0.14281E+01_JPRB, 0.13356E+01_JPRB ,&
&  0.13023E+01_JPRB, 0.12498E+01_JPRB, 0.11919E+01_JPRB, 0.11292E+01_JPRB, 0.11102E+01_JPRB, 0.11031E+01_JPRB, 0.10929E+01_JPRB ,&
&  0.10919E+01_JPRB, 0.10899E+01_JPRB, 0.10865E+01_JPRB, 0.10909E+01_JPRB, 0.11286E+01_JPRB, 0.11612E+01_JPRB, 0.11549E+01_JPRB ,&
&  0.11078E+01_JPRB, 0.10946E+01_JPRB, 0.11075E+01_JPRB, 0.10639E+01_JPRB, 0.10269E+01_JPRB, 0.10925E+01_JPRB, 0.10252E+01_JPRB ,&
&  0.96251E+00_JPRB, 0.92838E+00_JPRB, 0.90767E+00_JPRB, 0.91949E+00_JPRB, 0.98171E+00_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.98077E+00_JPRB, 0.97404E+00_JPRB, 0.95782E+00_JPRB, 0.94484E+00_JPRB, 0.94149E+00_JPRB, 0.92898E+00_JPRB, 0.92014E+00_JPRB ,&
&  0.91621E+00_JPRB, 0.90593E+00_JPRB, 0.89787E+00_JPRB, 0.88395E+00_JPRB, 0.86625E+00_JPRB, 0.85137E+00_JPRB, 0.84658E+00_JPRB ,&
&  0.84712E+00_JPRB, 0.83837E+00_JPRB, 0.84277E+00_JPRB, 0.84455E+00_JPRB, 0.84141E+00_JPRB, 0.82917E+00_JPRB, 0.81688E+00_JPRB ,&
&  0.80973E+00_JPRB, 0.79626E+00_JPRB, 0.79299E+00_JPRB, 0.81371E+00_JPRB, 0.83210E+00_JPRB, 0.83458E+00_JPRB, 0.82235E+00_JPRB ,&
&  0.88700E+00_JPRB, 0.12301E+01_JPRB, 0.17644E+01_JPRB, 0.19052E+01_JPRB, 0.17919E+01_JPRB, 0.16189E+01_JPRB, 0.14398E+01_JPRB ,&
&  0.12875E+01_JPRB, 0.11647E+01_JPRB, 0.10925E+01_JPRB, 0.10337E+01_JPRB, 0.97605E+00_JPRB, 0.92881E+00_JPRB, 0.89167E+00_JPRB ,&
&  0.85360E+00_JPRB, 0.83740E+00_JPRB, 0.82833E+00_JPRB, 0.82587E+00_JPRB, 0.82399E+00_JPRB, 0.82195E+00_JPRB, 0.82881E+00_JPRB ,&
&  0.84947E+00_JPRB, 0.88151E+00_JPRB, 0.92803E+00_JPRB, 0.10257E+01_JPRB, 0.14954E+01_JPRB, 0.22806E+01_JPRB, 0.33351E+01_JPRB ,&
&  0.36314E+01_JPRB, 0.37022E+01_JPRB, 0.33529E+01_JPRB, 0.28530E+01_JPRB, 0.24478E+01_JPRB, 0.21615E+01_JPRB, 0.19753E+01_JPRB ,&
&  0.17994E+01_JPRB, 0.16764E+01_JPRB, 0.15888E+01_JPRB, 0.14794E+01_JPRB, 0.13677E+01_JPRB, 0.13023E+01_JPRB, 0.13026E+01_JPRB ,&
&  0.13214E+01_JPRB, 0.12954E+01_JPRB, 0.12325E+01_JPRB, 0.11573E+01_JPRB, 0.10835E+01_JPRB, 0.10520E+01_JPRB, 0.10408E+01_JPRB ,&
&  0.10321E+01_JPRB, 0.10408E+01_JPRB, 0.10661E+01_JPRB, 0.10952E+01_JPRB, 0.11135E+01_JPRB, 0.11195E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.11257E+01_JPRB, 0.11125E+01_JPRB, 0.10954E+01_JPRB, 0.10858E+01_JPRB, 0.10542E+01_JPRB, 0.11113E+01_JPRB, 0.10645E+01_JPRB ,&
&  0.99846E+00_JPRB, 0.95166E+00_JPRB, 0.92765E+00_JPRB, 0.91469E+00_JPRB, 0.96239E+00_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.99368E+00_JPRB, 0.98745E+00_JPRB, 0.97068E+00_JPRB, 0.95791E+00_JPRB, 0.94135E+00_JPRB, 0.92628E+00_JPRB, 0.91340E+00_JPRB ,&
&  0.90709E+00_JPRB, 0.89190E+00_JPRB, 0.88436E+00_JPRB, 0.87664E+00_JPRB, 0.87337E+00_JPRB, 0.86472E+00_JPRB, 0.85142E+00_JPRB ,&
&  0.83728E+00_JPRB, 0.82953E+00_JPRB, 0.82741E+00_JPRB, 0.83184E+00_JPRB, 0.84192E+00_JPRB, 0.84840E+00_JPRB, 0.84334E+00_JPRB ,&
&  0.83402E+00_JPRB, 0.81937E+00_JPRB, 0.80154E+00_JPRB, 0.81557E+00_JPRB, 0.83141E+00_JPRB, 0.83594E+00_JPRB, 0.82211E+00_JPRB ,&
&  0.84913E+00_JPRB, 0.96397E+00_JPRB, 0.12868E+01_JPRB, 0.14024E+01_JPRB, 0.14103E+01_JPRB, 0.14360E+01_JPRB, 0.13448E+01_JPRB ,&
&  0.12515E+01_JPRB, 0.11369E+01_JPRB, 0.10432E+01_JPRB, 0.99608E+00_JPRB, 0.96652E+00_JPRB, 0.92433E+00_JPRB, 0.88999E+00_JPRB ,&
&  0.84503E+00_JPRB, 0.82320E+00_JPRB, 0.80907E+00_JPRB, 0.80356E+00_JPRB, 0.80075E+00_JPRB, 0.80161E+00_JPRB, 0.80807E+00_JPRB ,&
&  0.81888E+00_JPRB, 0.83493E+00_JPRB, 0.86149E+00_JPRB, 0.90867E+00_JPRB, 0.10129E+01_JPRB, 0.13429E+01_JPRB, 0.18548E+01_JPRB ,&
&  0.22040E+01_JPRB, 0.24623E+01_JPRB, 0.26409E+01_JPRB, 0.26175E+01_JPRB, 0.23834E+01_JPRB, 0.21095E+01_JPRB, 0.19060E+01_JPRB ,&
&  0.17367E+01_JPRB, 0.15746E+01_JPRB, 0.14383E+01_JPRB, 0.13252E+01_JPRB, 0.12251E+01_JPRB, 0.11828E+01_JPRB, 0.12220E+01_JPRB ,&
&  0.12467E+01_JPRB, 0.12353E+01_JPRB, 0.11853E+01_JPRB, 0.11242E+01_JPRB, 0.10521E+01_JPRB, 0.10091E+01_JPRB, 0.99723E+00_JPRB ,&
&  0.99875E+00_JPRB, 0.10127E+01_JPRB, 0.10332E+01_JPRB, 0.10575E+01_JPRB, 0.10785E+01_JPRB, 0.10891E+01_JPRB, 0.10907E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.10997E+01_JPRB, 0.10424E+01_JPRB, 0.10226E+01_JPRB, 0.11034E+01_JPRB, 0.10878E+01_JPRB, 0.10670E+01_JPRB ,&
&  0.10253E+01_JPRB, 0.96586E+00_JPRB, 0.92502E+00_JPRB, 0.89644E+00_JPRB, 0.95126E+00_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.86283E+00_JPRB, 0.89022E+00_JPRB, 0.90040E+00_JPRB, 0.90279E+00_JPRB, 0.89345E+00_JPRB, 0.87815E+00_JPRB, 0.86531E+00_JPRB ,&
&  0.86024E+00_JPRB, 0.84672E+00_JPRB, 0.84118E+00_JPRB, 0.83919E+00_JPRB, 0.84281E+00_JPRB, 0.85030E+00_JPRB, 0.85050E+00_JPRB ,&
&  0.84147E+00_JPRB, 0.82778E+00_JPRB, 0.81871E+00_JPRB, 0.81851E+00_JPRB, 0.82852E+00_JPRB, 0.83930E+00_JPRB, 0.84630E+00_JPRB ,&
&  0.84895E+00_JPRB, 0.84224E+00_JPRB, 0.83861E+00_JPRB, 0.84777E+00_JPRB, 0.85571E+00_JPRB, 0.84760E+00_JPRB, 0.82781E+00_JPRB ,&
&  0.82969E+00_JPRB, 0.79925E+00_JPRB, 0.92284E+00_JPRB, 0.10502E+01_JPRB, 0.11529E+01_JPRB, 0.11940E+01_JPRB, 0.11882E+01_JPRB ,&
&  0.11392E+01_JPRB, 0.10820E+01_JPRB, 0.10138E+01_JPRB, 0.95833E+00_JPRB, 0.91731E+00_JPRB, 0.88872E+00_JPRB, 0.87014E+00_JPRB ,&
&  0.84301E+00_JPRB, 0.81730E+00_JPRB, 0.79488E+00_JPRB, 0.78247E+00_JPRB, 0.77800E+00_JPRB, 0.77415E+00_JPRB, 0.77084E+00_JPRB ,&
&  0.76785E+00_JPRB, 0.76942E+00_JPRB, 0.78583E+00_JPRB, 0.81834E+00_JPRB, 0.86926E+00_JPRB, 0.99610E+00_JPRB, 0.12259E+01_JPRB ,&
&  0.14389E+01_JPRB, 0.16639E+01_JPRB, 0.18847E+01_JPRB, 0.20568E+01_JPRB, 0.20570E+01_JPRB, 0.19524E+01_JPRB, 0.18056E+01_JPRB ,&
&  0.16344E+01_JPRB, 0.14295E+01_JPRB, 0.12486E+01_JPRB, 0.11425E+01_JPRB, 0.11064E+01_JPRB, 0.11126E+01_JPRB, 0.11508E+01_JPRB ,&
&  0.11679E+01_JPRB, 0.11537E+01_JPRB, 0.11003E+01_JPRB, 0.10460E+01_JPRB, 0.10072E+01_JPRB, 0.97136E+00_JPRB, 0.95953E+00_JPRB ,&
&  0.95558E+00_JPRB, 0.96241E+00_JPRB, 0.97924E+00_JPRB, 0.99602E+00_JPRB, 0.10191E+01_JPRB, 0.10341E+01_JPRB, 0.10360E+01_JPRB ,&
&  0.10337E+01_JPRB, 0.10277E+01_JPRB, 0.10221E+01_JPRB, 0.10394E+01_JPRB, 0.10239E+01_JPRB, 0.10143E+01_JPRB, 0.10024E+01_JPRB ,&
&  0.97940E+00_JPRB, 0.93921E+00_JPRB, 0.89504E+00_JPRB, 0.86927E+00_JPRB, 0.80796E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.80671E+00_JPRB, 0.83265E+00_JPRB, 0.84494E+00_JPRB, 0.84828E+00_JPRB, 0.84734E+00_JPRB, 0.83610E+00_JPRB, 0.82222E+00_JPRB ,&
&  0.80540E+00_JPRB, 0.79896E+00_JPRB, 0.79700E+00_JPRB, 0.80196E+00_JPRB, 0.80811E+00_JPRB, 0.81696E+00_JPRB, 0.82931E+00_JPRB ,&
&  0.84110E+00_JPRB, 0.84681E+00_JPRB, 0.85127E+00_JPRB, 0.84955E+00_JPRB, 0.84789E+00_JPRB, 0.85018E+00_JPRB, 0.85434E+00_JPRB ,&
&  0.86305E+00_JPRB, 0.86575E+00_JPRB, 0.86210E+00_JPRB, 0.87032E+00_JPRB, 0.88339E+00_JPRB, 0.87293E+00_JPRB, 0.85391E+00_JPRB ,&
&  0.84219E+00_JPRB, 0.72128E+00_JPRB, 0.74986E+00_JPRB, 0.86901E+00_JPRB, 0.96510E+00_JPRB, 0.10600E+01_JPRB, 0.10857E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.10154E+01_JPRB, 0.96738E+00_JPRB, 0.92244E+00_JPRB, 0.89116E+00_JPRB, 0.85801E+00_JPRB, 0.83518E+00_JPRB ,&
&  0.82001E+00_JPRB, 0.80364E+00_JPRB, 0.77991E+00_JPRB, 0.76050E+00_JPRB, 0.74477E+00_JPRB, 0.72918E+00_JPRB, 0.72069E+00_JPRB ,&
&  0.72194E+00_JPRB, 0.73394E+00_JPRB, 0.75082E+00_JPRB, 0.77609E+00_JPRB, 0.82076E+00_JPRB, 0.88358E+00_JPRB, 0.98604E+00_JPRB ,&
&  0.11020E+01_JPRB, 0.12298E+01_JPRB, 0.13551E+01_JPRB, 0.14667E+01_JPRB, 0.15604E+01_JPRB, 0.15741E+01_JPRB, 0.14977E+01_JPRB ,&
&  0.13588E+01_JPRB, 0.12414E+01_JPRB, 0.11519E+01_JPRB, 0.10925E+01_JPRB, 0.10809E+01_JPRB, 0.10962E+01_JPRB, 0.11064E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.10519E+01_JPRB, 0.10002E+01_JPRB, 0.95327E+00_JPRB, 0.92685E+00_JPRB, 0.90945E+00_JPRB, 0.89180E+00_JPRB ,&
&  0.87718E+00_JPRB, 0.87205E+00_JPRB, 0.87750E+00_JPRB, 0.89576E+00_JPRB, 0.92028E+00_JPRB, 0.94662E+00_JPRB, 0.95959E+00_JPRB ,&
&  0.96664E+00_JPRB, 0.97295E+00_JPRB, 0.98744E+00_JPRB, 0.93117E+00_JPRB, 0.96330E+00_JPRB, 0.96810E+00_JPRB, 0.96477E+00_JPRB ,&
&  0.94563E+00_JPRB, 0.91723E+00_JPRB, 0.88013E+00_JPRB, 0.73077E+00_JPRB, 0.78015E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.78555E+00_JPRB, 0.79348E+00_JPRB, 0.80006E+00_JPRB, 0.79803E+00_JPRB, 0.79532E+00_JPRB, 0.78866E+00_JPRB, 0.77428E+00_JPRB ,&
&  0.76056E+00_JPRB, 0.75568E+00_JPRB, 0.75655E+00_JPRB, 0.75859E+00_JPRB, 0.76211E+00_JPRB, 0.76861E+00_JPRB, 0.78005E+00_JPRB ,&
&  0.79369E+00_JPRB, 0.80793E+00_JPRB, 0.82461E+00_JPRB, 0.83991E+00_JPRB, 0.84307E+00_JPRB, 0.83940E+00_JPRB, 0.83906E+00_JPRB ,&
&  0.84476E+00_JPRB, 0.84342E+00_JPRB, 0.83652E+00_JPRB, 0.83853E+00_JPRB, 0.84685E+00_JPRB, 0.83824E+00_JPRB, 0.82036E+00_JPRB ,&
&  0.80888E+00_JPRB, 0.66727E+00_JPRB, 0.70514E+00_JPRB, 0.77355E+00_JPRB, 0.85080E+00_JPRB, 0.92432E+00_JPRB, 0.95494E+00_JPRB ,&
&  0.94664E+00_JPRB, 0.91104E+00_JPRB, 0.87961E+00_JPRB, 0.84155E+00_JPRB, 0.82378E+00_JPRB, 0.81067E+00_JPRB, 0.79453E+00_JPRB ,&
&  0.77119E+00_JPRB, 0.74683E+00_JPRB, 0.72001E+00_JPRB, 0.69651E+00_JPRB, 0.67793E+00_JPRB, 0.66924E+00_JPRB, 0.66660E+00_JPRB ,&
&  0.66702E+00_JPRB, 0.67811E+00_JPRB, 0.68500E+00_JPRB, 0.69776E+00_JPRB, 0.72026E+00_JPRB, 0.75676E+00_JPRB, 0.80868E+00_JPRB ,&
&  0.87994E+00_JPRB, 0.96519E+00_JPRB, 0.10493E+01_JPRB, 0.11091E+01_JPRB, 0.11474E+01_JPRB, 0.11770E+01_JPRB, 0.11734E+01_JPRB ,&
&  0.11279E+01_JPRB, 0.10785E+01_JPRB, 0.10667E+01_JPRB, 0.10620E+01_JPRB, 0.10456E+01_JPRB, 0.10274E+01_JPRB, 0.99943E+00_JPRB ,&
&  0.95945E+00_JPRB, 0.91414E+00_JPRB, 0.87475E+00_JPRB, 0.83914E+00_JPRB, 0.81647E+00_JPRB, 0.80409E+00_JPRB, 0.78277E+00_JPRB ,&
&  0.76862E+00_JPRB, 0.76580E+00_JPRB, 0.77393E+00_JPRB, 0.79496E+00_JPRB, 0.82348E+00_JPRB, 0.84980E+00_JPRB, 0.86483E+00_JPRB ,&
&  0.87033E+00_JPRB, 0.87660E+00_JPRB, 0.88136E+00_JPRB, 0.90360E+00_JPRB, 0.90785E+00_JPRB, 0.90808E+00_JPRB, 0.90421E+00_JPRB ,&
&  0.89711E+00_JPRB, 0.88923E+00_JPRB, 0.68975E+00_JPRB, 0.75276E+00_JPRB, 0.76688E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.80238E+00_JPRB, 0.78524E+00_JPRB, 0.77658E+00_JPRB, 0.77161E+00_JPRB, 0.76781E+00_JPRB, 0.76147E+00_JPRB, 0.74867E+00_JPRB ,&
&  0.73572E+00_JPRB, 0.72933E+00_JPRB, 0.72356E+00_JPRB, 0.71589E+00_JPRB, 0.71781E+00_JPRB, 0.72421E+00_JPRB, 0.73548E+00_JPRB ,&
&  0.74773E+00_JPRB, 0.76327E+00_JPRB, 0.77810E+00_JPRB, 0.79668E+00_JPRB, 0.80874E+00_JPRB, 0.81698E+00_JPRB, 0.82111E+00_JPRB ,&
&  0.82380E+00_JPRB, 0.82394E+00_JPRB, 0.81619E+00_JPRB, 0.80619E+00_JPRB, 0.80343E+00_JPRB, 0.80481E+00_JPRB, 0.79996E+00_JPRB ,&
&  0.70157E+00_JPRB, 0.61578E+00_JPRB, 0.67087E+00_JPRB, 0.73458E+00_JPRB, 0.79659E+00_JPRB, 0.84879E+00_JPRB, 0.87201E+00_JPRB ,&
&  0.86042E+00_JPRB, 0.83109E+00_JPRB, 0.80329E+00_JPRB, 0.77048E+00_JPRB, 0.75708E+00_JPRB, 0.75164E+00_JPRB, 0.74275E+00_JPRB ,&
&  0.73281E+00_JPRB, 0.72091E+00_JPRB, 0.70582E+00_JPRB, 0.68237E+00_JPRB, 0.66581E+00_JPRB, 0.65380E+00_JPRB, 0.65163E+00_JPRB ,&
&  0.65378E+00_JPRB, 0.65766E+00_JPRB, 0.66129E+00_JPRB, 0.66453E+00_JPRB, 0.67291E+00_JPRB, 0.68757E+00_JPRB, 0.70751E+00_JPRB ,&
&  0.73925E+00_JPRB, 0.77841E+00_JPRB, 0.82889E+00_JPRB, 0.86438E+00_JPRB, 0.88556E+00_JPRB, 0.90513E+00_JPRB, 0.93112E+00_JPRB ,&
&  0.94751E+00_JPRB, 0.94351E+00_JPRB, 0.91754E+00_JPRB, 0.90350E+00_JPRB, 0.89876E+00_JPRB, 0.89082E+00_JPRB, 0.87209E+00_JPRB ,&
&  0.84072E+00_JPRB, 0.80295E+00_JPRB, 0.76449E+00_JPRB, 0.74103E+00_JPRB, 0.73492E+00_JPRB, 0.73202E+00_JPRB, 0.72489E+00_JPRB ,&
&  0.71855E+00_JPRB, 0.71718E+00_JPRB, 0.72271E+00_JPRB, 0.73441E+00_JPRB, 0.75008E+00_JPRB, 0.76886E+00_JPRB, 0.78359E+00_JPRB ,&
&  0.79744E+00_JPRB, 0.81081E+00_JPRB, 0.82078E+00_JPRB, 0.83699E+00_JPRB, 0.85345E+00_JPRB, 0.86082E+00_JPRB, 0.85546E+00_JPRB ,&
&  0.85198E+00_JPRB, 0.83925E+00_JPRB, 0.82931E+00_JPRB, 0.82457E+00_JPRB, 0.81657E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.77027E+00_JPRB, 0.75268E+00_JPRB, 0.74268E+00_JPRB, 0.73870E+00_JPRB, 0.73024E+00_JPRB, 0.71136E+00_JPRB, 0.69573E+00_JPRB ,&
&  0.68146E+00_JPRB, 0.67386E+00_JPRB, 0.67086E+00_JPRB, 0.67036E+00_JPRB, 0.67644E+00_JPRB, 0.68224E+00_JPRB, 0.68927E+00_JPRB ,&
&  0.70078E+00_JPRB, 0.71915E+00_JPRB, 0.73481E+00_JPRB, 0.74909E+00_JPRB, 0.76178E+00_JPRB, 0.77429E+00_JPRB, 0.77743E+00_JPRB ,&
&  0.77987E+00_JPRB, 0.78717E+00_JPRB, 0.78843E+00_JPRB, 0.77569E+00_JPRB, 0.76235E+00_JPRB, 0.76500E+00_JPRB, 0.76210E+00_JPRB ,&
&  0.75257E+00_JPRB, 0.63863E+00_JPRB, 0.69231E+00_JPRB, 0.71743E+00_JPRB, 0.74605E+00_JPRB, 0.77284E+00_JPRB, 0.78714E+00_JPRB ,&
&  0.78334E+00_JPRB, 0.76607E+00_JPRB, 0.74574E+00_JPRB, 0.71831E+00_JPRB, 0.71011E+00_JPRB, 0.70328E+00_JPRB, 0.69460E+00_JPRB ,&
&  0.68893E+00_JPRB, 0.68328E+00_JPRB, 0.67522E+00_JPRB, 0.66397E+00_JPRB, 0.64942E+00_JPRB, 0.63864E+00_JPRB, 0.62911E+00_JPRB ,&
&  0.62659E+00_JPRB, 0.61918E+00_JPRB, 0.61240E+00_JPRB, 0.61309E+00_JPRB, 0.62508E+00_JPRB, 0.63206E+00_JPRB, 0.63525E+00_JPRB ,&
&  0.64072E+00_JPRB, 0.65069E+00_JPRB, 0.66855E+00_JPRB, 0.69173E+00_JPRB, 0.71066E+00_JPRB, 0.72493E+00_JPRB, 0.74462E+00_JPRB ,&
&  0.77489E+00_JPRB, 0.79638E+00_JPRB, 0.79799E+00_JPRB, 0.78236E+00_JPRB, 0.77031E+00_JPRB, 0.76550E+00_JPRB, 0.74924E+00_JPRB ,&
&  0.72334E+00_JPRB, 0.70382E+00_JPRB, 0.69491E+00_JPRB, 0.69009E+00_JPRB, 0.68393E+00_JPRB, 0.67705E+00_JPRB, 0.67598E+00_JPRB ,&
&  0.67699E+00_JPRB, 0.67675E+00_JPRB, 0.67551E+00_JPRB, 0.67820E+00_JPRB, 0.68424E+00_JPRB, 0.69204E+00_JPRB, 0.70275E+00_JPRB ,&
&  0.71840E+00_JPRB, 0.73272E+00_JPRB, 0.74052E+00_JPRB, 0.75075E+00_JPRB, 0.76354E+00_JPRB, 0.76916E+00_JPRB, 0.78267E+00_JPRB ,&
&  0.78212E+00_JPRB, 0.77701E+00_JPRB, 0.77479E+00_JPRB, 0.77971E+00_JPRB, 0.78154E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.73645E+00_JPRB, 0.71680E+00_JPRB, 0.70513E+00_JPRB, 0.69377E+00_JPRB, 0.68576E+00_JPRB, 0.66809E+00_JPRB, 0.65810E+00_JPRB ,&
&  0.65168E+00_JPRB, 0.64767E+00_JPRB, 0.64710E+00_JPRB, 0.64757E+00_JPRB, 0.65787E+00_JPRB, 0.66452E+00_JPRB, 0.66712E+00_JPRB ,&
&  0.67486E+00_JPRB, 0.69254E+00_JPRB, 0.70890E+00_JPRB, 0.72255E+00_JPRB, 0.73325E+00_JPRB, 0.74091E+00_JPRB, 0.74065E+00_JPRB ,&
&  0.74425E+00_JPRB, 0.75213E+00_JPRB, 0.75592E+00_JPRB, 0.74566E+00_JPRB, 0.73270E+00_JPRB, 0.72585E+00_JPRB, 0.72621E+00_JPRB ,&
&  0.71569E+00_JPRB, 0.67139E+00_JPRB, 0.68613E+00_JPRB, 0.69796E+00_JPRB, 0.71175E+00_JPRB, 0.71917E+00_JPRB, 0.72695E+00_JPRB ,&
&  0.72880E+00_JPRB, 0.71203E+00_JPRB, 0.69056E+00_JPRB, 0.67481E+00_JPRB, 0.67815E+00_JPRB, 0.67564E+00_JPRB, 0.66651E+00_JPRB ,&
&  0.66263E+00_JPRB, 0.65941E+00_JPRB, 0.65218E+00_JPRB, 0.63937E+00_JPRB, 0.62778E+00_JPRB, 0.61991E+00_JPRB, 0.61462E+00_JPRB ,&
&  0.60333E+00_JPRB, 0.59403E+00_JPRB, 0.58902E+00_JPRB, 0.58342E+00_JPRB, 0.57884E+00_JPRB, 0.58357E+00_JPRB, 0.58817E+00_JPRB ,&
&  0.58987E+00_JPRB, 0.59401E+00_JPRB, 0.60515E+00_JPRB, 0.62280E+00_JPRB, 0.63866E+00_JPRB, 0.65572E+00_JPRB, 0.67472E+00_JPRB ,&
&  0.69449E+00_JPRB, 0.71317E+00_JPRB, 0.72107E+00_JPRB, 0.72634E+00_JPRB, 0.72041E+00_JPRB, 0.70977E+00_JPRB, 0.68860E+00_JPRB ,&
&  0.66499E+00_JPRB, 0.65786E+00_JPRB, 0.65640E+00_JPRB, 0.66011E+00_JPRB, 0.65688E+00_JPRB, 0.65342E+00_JPRB, 0.65194E+00_JPRB ,&
&  0.64813E+00_JPRB, 0.64459E+00_JPRB, 0.63942E+00_JPRB, 0.63465E+00_JPRB, 0.63776E+00_JPRB, 0.64331E+00_JPRB, 0.65637E+00_JPRB ,&
&  0.67412E+00_JPRB, 0.69310E+00_JPRB, 0.71256E+00_JPRB, 0.72533E+00_JPRB, 0.72924E+00_JPRB, 0.73165E+00_JPRB, 0.73452E+00_JPRB ,&
&  0.73519E+00_JPRB, 0.73292E+00_JPRB, 0.73824E+00_JPRB, 0.74217E+00_JPRB, 0.74646E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.68276E+00_JPRB, 0.65778E+00_JPRB, 0.63976E+00_JPRB, 0.63316E+00_JPRB, 0.62973E+00_JPRB, 0.61779E+00_JPRB, 0.61263E+00_JPRB ,&
&  0.61863E+00_JPRB, 0.62040E+00_JPRB, 0.62746E+00_JPRB, 0.62714E+00_JPRB, 0.62646E+00_JPRB, 0.62911E+00_JPRB, 0.62155E+00_JPRB ,&
&  0.62165E+00_JPRB, 0.63079E+00_JPRB, 0.64158E+00_JPRB, 0.65614E+00_JPRB, 0.66841E+00_JPRB, 0.67765E+00_JPRB, 0.68545E+00_JPRB ,&
&  0.68853E+00_JPRB, 0.69503E+00_JPRB, 0.69857E+00_JPRB, 0.69426E+00_JPRB, 0.68308E+00_JPRB, 0.67427E+00_JPRB, 0.67260E+00_JPRB ,&
&  0.67108E+00_JPRB, 0.66553E+00_JPRB, 0.62118E+00_JPRB, 0.64519E+00_JPRB, 0.65216E+00_JPRB, 0.65574E+00_JPRB, 0.65819E+00_JPRB ,&
&  0.65316E+00_JPRB, 0.63838E+00_JPRB, 0.63116E+00_JPRB, 0.61862E+00_JPRB, 0.61713E+00_JPRB, 0.62308E+00_JPRB, 0.62160E+00_JPRB ,&
&  0.61731E+00_JPRB, 0.60696E+00_JPRB, 0.59312E+00_JPRB, 0.58237E+00_JPRB, 0.57966E+00_JPRB, 0.57755E+00_JPRB, 0.56977E+00_JPRB ,&
&  0.56450E+00_JPRB, 0.56374E+00_JPRB, 0.56210E+00_JPRB, 0.55620E+00_JPRB, 0.54138E+00_JPRB, 0.52799E+00_JPRB, 0.52614E+00_JPRB ,&
&  0.53097E+00_JPRB, 0.53989E+00_JPRB, 0.55616E+00_JPRB, 0.57633E+00_JPRB, 0.59734E+00_JPRB, 0.61684E+00_JPRB, 0.63400E+00_JPRB ,&
&  0.64074E+00_JPRB, 0.64387E+00_JPRB, 0.64776E+00_JPRB, 0.65416E+00_JPRB, 0.65637E+00_JPRB, 0.64874E+00_JPRB, 0.62815E+00_JPRB ,&
&  0.60577E+00_JPRB, 0.59891E+00_JPRB, 0.60166E+00_JPRB, 0.60956E+00_JPRB, 0.61319E+00_JPRB, 0.61229E+00_JPRB, 0.60589E+00_JPRB ,&
&  0.60216E+00_JPRB, 0.59401E+00_JPRB, 0.58505E+00_JPRB, 0.58058E+00_JPRB, 0.58012E+00_JPRB, 0.58600E+00_JPRB, 0.60330E+00_JPRB ,&
&  0.62426E+00_JPRB, 0.64765E+00_JPRB, 0.66838E+00_JPRB, 0.67561E+00_JPRB, 0.67914E+00_JPRB, 0.67773E+00_JPRB, 0.66831E+00_JPRB ,&
&  0.67220E+00_JPRB, 0.69081E+00_JPRB, 0.70004E+00_JPRB, 0.70117E+00_JPRB, 0.69840E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.63173E+00_JPRB, 0.60317E+00_JPRB, 0.59195E+00_JPRB, 0.59384E+00_JPRB, 0.59728E+00_JPRB, 0.59309E+00_JPRB, 0.60040E+00_JPRB ,&
&  0.61155E+00_JPRB, 0.62002E+00_JPRB, 0.62470E+00_JPRB, 0.62095E+00_JPRB, 0.61847E+00_JPRB, 0.61549E+00_JPRB, 0.60069E+00_JPRB ,&
&  0.59577E+00_JPRB, 0.59404E+00_JPRB, 0.59827E+00_JPRB, 0.60554E+00_JPRB, 0.61351E+00_JPRB, 0.62590E+00_JPRB, 0.63509E+00_JPRB ,&
&  0.64110E+00_JPRB, 0.64485E+00_JPRB, 0.64901E+00_JPRB, 0.64917E+00_JPRB, 0.64550E+00_JPRB, 0.63619E+00_JPRB, 0.62900E+00_JPRB ,&
&  0.62466E+00_JPRB, 0.61310E+00_JPRB, 0.60259E+00_JPRB, 0.60112E+00_JPRB, 0.60402E+00_JPRB, 0.60685E+00_JPRB, 0.60625E+00_JPRB ,&
&  0.59364E+00_JPRB, 0.57720E+00_JPRB, 0.57566E+00_JPRB, 0.57525E+00_JPRB, 0.58021E+00_JPRB, 0.59219E+00_JPRB, 0.59551E+00_JPRB ,&
&  0.59218E+00_JPRB, 0.57798E+00_JPRB, 0.55881E+00_JPRB, 0.55517E+00_JPRB, 0.55731E+00_JPRB, 0.55043E+00_JPRB, 0.54375E+00_JPRB ,&
&  0.54266E+00_JPRB, 0.53981E+00_JPRB, 0.53388E+00_JPRB, 0.52569E+00_JPRB, 0.51277E+00_JPRB, 0.49617E+00_JPRB, 0.48800E+00_JPRB ,&
&  0.48853E+00_JPRB, 0.50570E+00_JPRB, 0.53511E+00_JPRB, 0.56734E+00_JPRB, 0.59729E+00_JPRB, 0.61159E+00_JPRB, 0.61029E+00_JPRB ,&
&  0.59651E+00_JPRB, 0.59195E+00_JPRB, 0.59629E+00_JPRB, 0.60112E+00_JPRB, 0.60313E+00_JPRB, 0.59323E+00_JPRB, 0.57590E+00_JPRB ,&
&  0.56295E+00_JPRB, 0.55351E+00_JPRB, 0.55225E+00_JPRB, 0.55675E+00_JPRB, 0.56119E+00_JPRB, 0.56216E+00_JPRB, 0.55638E+00_JPRB ,&
&  0.55065E+00_JPRB, 0.54438E+00_JPRB, 0.54036E+00_JPRB, 0.54273E+00_JPRB, 0.54755E+00_JPRB, 0.55254E+00_JPRB, 0.56570E+00_JPRB ,&
&  0.58481E+00_JPRB, 0.60380E+00_JPRB, 0.60809E+00_JPRB, 0.60357E+00_JPRB, 0.60274E+00_JPRB, 0.59965E+00_JPRB, 0.59670E+00_JPRB ,&
&  0.60655E+00_JPRB, 0.62862E+00_JPRB, 0.64437E+00_JPRB, 0.65286E+00_JPRB, 0.65271E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.55634E+00_JPRB, 0.53958E+00_JPRB, 0.54019E+00_JPRB, 0.54756E+00_JPRB, 0.55173E+00_JPRB, 0.55957E+00_JPRB, 0.57906E+00_JPRB ,&
&  0.59036E+00_JPRB, 0.59750E+00_JPRB, 0.60030E+00_JPRB, 0.59514E+00_JPRB, 0.59425E+00_JPRB, 0.58704E+00_JPRB, 0.57425E+00_JPRB ,&
&  0.56889E+00_JPRB, 0.56783E+00_JPRB, 0.56948E+00_JPRB, 0.57337E+00_JPRB, 0.57574E+00_JPRB, 0.57874E+00_JPRB, 0.58217E+00_JPRB ,&
&  0.58404E+00_JPRB, 0.58405E+00_JPRB, 0.58317E+00_JPRB, 0.58336E+00_JPRB, 0.58232E+00_JPRB, 0.57458E+00_JPRB, 0.56580E+00_JPRB ,&
&  0.55629E+00_JPRB, 0.54699E+00_JPRB, 0.54418E+00_JPRB, 0.54835E+00_JPRB, 0.55313E+00_JPRB, 0.55598E+00_JPRB, 0.55354E+00_JPRB ,&
&  0.54893E+00_JPRB, 0.54658E+00_JPRB, 0.54921E+00_JPRB, 0.55610E+00_JPRB, 0.56454E+00_JPRB, 0.57399E+00_JPRB, 0.57896E+00_JPRB ,&
&  0.57327E+00_JPRB, 0.55556E+00_JPRB, 0.54478E+00_JPRB, 0.54414E+00_JPRB, 0.54014E+00_JPRB, 0.52525E+00_JPRB, 0.51494E+00_JPRB ,&
&  0.51018E+00_JPRB, 0.50136E+00_JPRB, 0.49203E+00_JPRB, 0.47936E+00_JPRB, 0.46774E+00_JPRB, 0.46334E+00_JPRB, 0.46237E+00_JPRB ,&
&  0.46660E+00_JPRB, 0.48685E+00_JPRB, 0.52216E+00_JPRB, 0.55563E+00_JPRB, 0.56965E+00_JPRB, 0.56259E+00_JPRB, 0.54467E+00_JPRB ,&
&  0.53556E+00_JPRB, 0.53108E+00_JPRB, 0.53183E+00_JPRB, 0.53174E+00_JPRB, 0.52896E+00_JPRB, 0.52061E+00_JPRB, 0.51337E+00_JPRB ,&
&  0.50306E+00_JPRB, 0.49181E+00_JPRB, 0.48732E+00_JPRB, 0.48448E+00_JPRB, 0.48142E+00_JPRB, 0.48152E+00_JPRB, 0.48083E+00_JPRB ,&
&  0.48167E+00_JPRB, 0.48572E+00_JPRB, 0.49715E+00_JPRB, 0.50320E+00_JPRB, 0.50362E+00_JPRB, 0.49340E+00_JPRB, 0.49483E+00_JPRB ,&
&  0.49809E+00_JPRB, 0.49598E+00_JPRB, 0.49131E+00_JPRB, 0.49100E+00_JPRB, 0.49082E+00_JPRB, 0.49259E+00_JPRB, 0.50422E+00_JPRB ,&
&  0.53097E+00_JPRB, 0.55428E+00_JPRB, 0.57325E+00_JPRB, 0.57844E+00_JPRB, 0.57399E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.50063E+00_JPRB, 0.50586E+00_JPRB, 0.51898E+00_JPRB, 0.52979E+00_JPRB, 0.53335E+00_JPRB, 0.54302E+00_JPRB, 0.55425E+00_JPRB ,&
&  0.56320E+00_JPRB, 0.56480E+00_JPRB, 0.56104E+00_JPRB, 0.55094E+00_JPRB, 0.53900E+00_JPRB, 0.52910E+00_JPRB, 0.52521E+00_JPRB ,&
&  0.52742E+00_JPRB, 0.53044E+00_JPRB, 0.53595E+00_JPRB, 0.53904E+00_JPRB, 0.54282E+00_JPRB, 0.54178E+00_JPRB, 0.54239E+00_JPRB ,&
&  0.54438E+00_JPRB, 0.54552E+00_JPRB, 0.54605E+00_JPRB, 0.54233E+00_JPRB, 0.53930E+00_JPRB, 0.53663E+00_JPRB, 0.52781E+00_JPRB ,&
&  0.51806E+00_JPRB, 0.51155E+00_JPRB, 0.51537E+00_JPRB, 0.52663E+00_JPRB, 0.53374E+00_JPRB, 0.53072E+00_JPRB, 0.52919E+00_JPRB ,&
&  0.52941E+00_JPRB, 0.53444E+00_JPRB, 0.54256E+00_JPRB, 0.55446E+00_JPRB, 0.56266E+00_JPRB, 0.57352E+00_JPRB, 0.57405E+00_JPRB ,&
&  0.56758E+00_JPRB, 0.55832E+00_JPRB, 0.55531E+00_JPRB, 0.55390E+00_JPRB, 0.52451E+00_JPRB, 0.49238E+00_JPRB, 0.47670E+00_JPRB ,&
&  0.46827E+00_JPRB, 0.45780E+00_JPRB, 0.44200E+00_JPRB, 0.43098E+00_JPRB, 0.42992E+00_JPRB, 0.43487E+00_JPRB, 0.44656E+00_JPRB ,&
&  0.45613E+00_JPRB, 0.46993E+00_JPRB, 0.48573E+00_JPRB, 0.49588E+00_JPRB, 0.49584E+00_JPRB, 0.48312E+00_JPRB, 0.47112E+00_JPRB ,&
&  0.46372E+00_JPRB, 0.46132E+00_JPRB, 0.45956E+00_JPRB, 0.45749E+00_JPRB, 0.45575E+00_JPRB, 0.45528E+00_JPRB, 0.45429E+00_JPRB ,&
&  0.45011E+00_JPRB, 0.44303E+00_JPRB, 0.43439E+00_JPRB, 0.42955E+00_JPRB, 0.42873E+00_JPRB, 0.42888E+00_JPRB, 0.43069E+00_JPRB ,&
&  0.43968E+00_JPRB, 0.45108E+00_JPRB, 0.46008E+00_JPRB, 0.46374E+00_JPRB, 0.46035E+00_JPRB, 0.45223E+00_JPRB, 0.44743E+00_JPRB ,&
&  0.44396E+00_JPRB, 0.44150E+00_JPRB, 0.43816E+00_JPRB, 0.43664E+00_JPRB, 0.43650E+00_JPRB, 0.44394E+00_JPRB, 0.45982E+00_JPRB ,&
&  0.47937E+00_JPRB, 0.50475E+00_JPRB, 0.51605E+00_JPRB, 0.51433E+00_JPRB, 0.50696E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.47071E+00_JPRB, 0.48354E+00_JPRB, 0.49971E+00_JPRB, 0.51110E+00_JPRB, 0.51598E+00_JPRB, 0.52135E+00_JPRB, 0.52378E+00_JPRB ,&
&  0.51840E+00_JPRB, 0.51609E+00_JPRB, 0.51054E+00_JPRB, 0.50267E+00_JPRB, 0.49583E+00_JPRB, 0.49240E+00_JPRB, 0.49281E+00_JPRB ,&
&  0.49494E+00_JPRB, 0.49677E+00_JPRB, 0.49333E+00_JPRB, 0.49166E+00_JPRB, 0.49574E+00_JPRB, 0.50126E+00_JPRB, 0.50487E+00_JPRB ,&
&  0.50642E+00_JPRB, 0.50801E+00_JPRB, 0.51284E+00_JPRB, 0.51588E+00_JPRB, 0.51595E+00_JPRB, 0.50926E+00_JPRB, 0.50419E+00_JPRB ,&
&  0.49905E+00_JPRB, 0.49717E+00_JPRB, 0.50344E+00_JPRB, 0.49560E+00_JPRB, 0.51022E+00_JPRB, 0.51201E+00_JPRB, 0.51736E+00_JPRB ,&
&  0.52544E+00_JPRB, 0.53906E+00_JPRB, 0.54974E+00_JPRB, 0.55785E+00_JPRB, 0.56019E+00_JPRB, 0.56323E+00_JPRB, 0.55328E+00_JPRB ,&
&  0.53951E+00_JPRB, 0.52092E+00_JPRB, 0.50213E+00_JPRB, 0.48156E+00_JPRB, 0.46161E+00_JPRB, 0.44623E+00_JPRB, 0.43888E+00_JPRB ,&
&  0.42988E+00_JPRB, 0.41595E+00_JPRB, 0.40257E+00_JPRB, 0.39740E+00_JPRB, 0.40209E+00_JPRB, 0.41227E+00_JPRB, 0.42804E+00_JPRB ,&
&  0.43828E+00_JPRB, 0.44166E+00_JPRB, 0.43874E+00_JPRB, 0.43463E+00_JPRB, 0.43029E+00_JPRB, 0.42271E+00_JPRB, 0.41817E+00_JPRB ,&
&  0.41450E+00_JPRB, 0.40873E+00_JPRB, 0.40571E+00_JPRB, 0.40727E+00_JPRB, 0.41031E+00_JPRB, 0.41239E+00_JPRB, 0.41107E+00_JPRB ,&
&  0.40835E+00_JPRB, 0.40379E+00_JPRB, 0.39573E+00_JPRB, 0.39161E+00_JPRB, 0.39219E+00_JPRB, 0.39669E+00_JPRB, 0.40417E+00_JPRB ,&
&  0.40659E+00_JPRB, 0.41273E+00_JPRB, 0.41866E+00_JPRB, 0.42075E+00_JPRB, 0.42325E+00_JPRB, 0.42518E+00_JPRB, 0.42520E+00_JPRB ,&
&  0.42340E+00_JPRB, 0.41838E+00_JPRB, 0.41155E+00_JPRB, 0.40884E+00_JPRB, 0.41022E+00_JPRB, 0.41725E+00_JPRB, 0.42658E+00_JPRB ,&
&  0.43569E+00_JPRB, 0.44969E+00_JPRB, 0.46019E+00_JPRB, 0.46244E+00_JPRB, 0.46455E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.44981E+00_JPRB, 0.46146E+00_JPRB, 0.47509E+00_JPRB, 0.48448E+00_JPRB, 0.48342E+00_JPRB, 0.48330E+00_JPRB, 0.48932E+00_JPRB ,&
&  0.48291E+00_JPRB, 0.47550E+00_JPRB, 0.46956E+00_JPRB, 0.46360E+00_JPRB, 0.46161E+00_JPRB, 0.46041E+00_JPRB, 0.46346E+00_JPRB ,&
&  0.46570E+00_JPRB, 0.46472E+00_JPRB, 0.46324E+00_JPRB, 0.46419E+00_JPRB, 0.47100E+00_JPRB, 0.47943E+00_JPRB, 0.48645E+00_JPRB ,&
&  0.48787E+00_JPRB, 0.49152E+00_JPRB, 0.49736E+00_JPRB, 0.50231E+00_JPRB, 0.50476E+00_JPRB, 0.50618E+00_JPRB, 0.50483E+00_JPRB ,&
&  0.50210E+00_JPRB, 0.49944E+00_JPRB, 0.48239E+00_JPRB, 0.49560E+00_JPRB, 0.50008E+00_JPRB, 0.50082E+00_JPRB, 0.50602E+00_JPRB ,&
&  0.51372E+00_JPRB, 0.52270E+00_JPRB, 0.52700E+00_JPRB, 0.52834E+00_JPRB, 0.52350E+00_JPRB, 0.51397E+00_JPRB, 0.48951E+00_JPRB ,&
&  0.46901E+00_JPRB, 0.44776E+00_JPRB, 0.43445E+00_JPRB, 0.42628E+00_JPRB, 0.41976E+00_JPRB, 0.41422E+00_JPRB, 0.40912E+00_JPRB ,&
&  0.40207E+00_JPRB, 0.39579E+00_JPRB, 0.39191E+00_JPRB, 0.39185E+00_JPRB, 0.39590E+00_JPRB, 0.40675E+00_JPRB, 0.42019E+00_JPRB ,&
&  0.42627E+00_JPRB, 0.41882E+00_JPRB, 0.41086E+00_JPRB, 0.40195E+00_JPRB, 0.38728E+00_JPRB, 0.36027E+00_JPRB, 0.34343E+00_JPRB ,&
&  0.33948E+00_JPRB, 0.35207E+00_JPRB, 0.37316E+00_JPRB, 0.37928E+00_JPRB, 0.38466E+00_JPRB, 0.38216E+00_JPRB, 0.37666E+00_JPRB ,&
&  0.36742E+00_JPRB, 0.34420E+00_JPRB, 0.33186E+00_JPRB, 0.32833E+00_JPRB, 0.32620E+00_JPRB, 0.32674E+00_JPRB, 0.32942E+00_JPRB ,&
&  0.33384E+00_JPRB, 0.34099E+00_JPRB, 0.34600E+00_JPRB, 0.34803E+00_JPRB, 0.35105E+00_JPRB, 0.35129E+00_JPRB, 0.35023E+00_JPRB ,&
&  0.34677E+00_JPRB, 0.35375E+00_JPRB, 0.37245E+00_JPRB, 0.40054E+00_JPRB, 0.40706E+00_JPRB, 0.41489E+00_JPRB, 0.42375E+00_JPRB ,&
&  0.42921E+00_JPRB, 0.43450E+00_JPRB, 0.43506E+00_JPRB, 0.43582E+00_JPRB, 0.44089E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.42537E+00_JPRB, 0.43247E+00_JPRB, 0.44109E+00_JPRB, 0.45227E+00_JPRB, 0.45473E+00_JPRB, 0.45634E+00_JPRB, 0.46534E+00_JPRB ,&
&  0.46550E+00_JPRB, 0.46239E+00_JPRB, 0.45917E+00_JPRB, 0.45955E+00_JPRB, 0.46185E+00_JPRB, 0.46441E+00_JPRB, 0.46415E+00_JPRB ,&
&  0.45518E+00_JPRB, 0.45008E+00_JPRB, 0.45298E+00_JPRB, 0.45616E+00_JPRB, 0.46083E+00_JPRB, 0.46732E+00_JPRB, 0.47301E+00_JPRB ,&
&  0.47613E+00_JPRB, 0.47982E+00_JPRB, 0.48244E+00_JPRB, 0.48876E+00_JPRB, 0.49838E+00_JPRB, 0.50499E+00_JPRB, 0.51105E+00_JPRB ,&
&  0.49613E+00_JPRB, 0.44825E+00_JPRB, 0.41795E+00_JPRB, 0.42408E+00_JPRB, 0.47256E+00_JPRB, 0.48306E+00_JPRB, 0.48392E+00_JPRB ,&
&  0.48689E+00_JPRB, 0.49447E+00_JPRB, 0.49404E+00_JPRB, 0.48826E+00_JPRB, 0.47632E+00_JPRB, 0.45717E+00_JPRB, 0.43387E+00_JPRB ,&
&  0.41835E+00_JPRB, 0.40815E+00_JPRB, 0.40133E+00_JPRB, 0.39452E+00_JPRB, 0.39099E+00_JPRB, 0.38597E+00_JPRB, 0.37820E+00_JPRB ,&
&  0.37511E+00_JPRB, 0.37389E+00_JPRB, 0.37507E+00_JPRB, 0.37652E+00_JPRB, 0.37828E+00_JPRB, 0.37937E+00_JPRB, 0.37569E+00_JPRB ,&
&  0.36364E+00_JPRB, 0.34784E+00_JPRB, 0.33289E+00_JPRB, 0.31660E+00_JPRB, 0.30275E+00_JPRB, 0.29426E+00_JPRB, 0.28739E+00_JPRB ,&
&  0.28482E+00_JPRB, 0.29099E+00_JPRB, 0.30862E+00_JPRB, 0.32863E+00_JPRB, 0.35955E+00_JPRB, 0.33444E+00_JPRB, 0.31080E+00_JPRB ,&
&  0.29946E+00_JPRB, 0.29383E+00_JPRB, 0.29065E+00_JPRB, 0.29058E+00_JPRB, 0.29215E+00_JPRB, 0.29259E+00_JPRB, 0.29269E+00_JPRB ,&
&  0.29410E+00_JPRB, 0.29666E+00_JPRB, 0.29859E+00_JPRB, 0.30030E+00_JPRB, 0.30291E+00_JPRB, 0.30589E+00_JPRB, 0.30747E+00_JPRB ,&
&  0.30833E+00_JPRB, 0.31333E+00_JPRB, 0.32321E+00_JPRB, 0.33467E+00_JPRB, 0.34596E+00_JPRB, 0.35768E+00_JPRB, 0.37504E+00_JPRB ,&
&  0.39657E+00_JPRB, 0.42216E+00_JPRB, 0.42200E+00_JPRB, 0.42246E+00_JPRB, 0.42255E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.41250E+00_JPRB, 0.42313E+00_JPRB, 0.42861E+00_JPRB, 0.43408E+00_JPRB, 0.43644E+00_JPRB, 0.43796E+00_JPRB, 0.44597E+00_JPRB ,&
&  0.45080E+00_JPRB, 0.45465E+00_JPRB, 0.45709E+00_JPRB, 0.45834E+00_JPRB, 0.45446E+00_JPRB, 0.44807E+00_JPRB, 0.44624E+00_JPRB ,&
&  0.44513E+00_JPRB, 0.44564E+00_JPRB, 0.44467E+00_JPRB, 0.44469E+00_JPRB, 0.44523E+00_JPRB, 0.44999E+00_JPRB, 0.45686E+00_JPRB ,&
&  0.45845E+00_JPRB, 0.46282E+00_JPRB, 0.47570E+00_JPRB, 0.48341E+00_JPRB, 0.48825E+00_JPRB, 0.48763E+00_JPRB, 0.48164E+00_JPRB ,&
&  0.45351E+00_JPRB, 0.42537E+00_JPRB, 0.39840E+00_JPRB, 0.39974E+00_JPRB, 0.45224E+00_JPRB, 0.45834E+00_JPRB, 0.45785E+00_JPRB ,&
&  0.45634E+00_JPRB, 0.46056E+00_JPRB, 0.46077E+00_JPRB, 0.44613E+00_JPRB, 0.43548E+00_JPRB, 0.42089E+00_JPRB, 0.40823E+00_JPRB ,&
&  0.39717E+00_JPRB, 0.38958E+00_JPRB, 0.37617E+00_JPRB, 0.34857E+00_JPRB, 0.33129E+00_JPRB, 0.32238E+00_JPRB, 0.30853E+00_JPRB ,&
&  0.29801E+00_JPRB, 0.29424E+00_JPRB, 0.29332E+00_JPRB, 0.29252E+00_JPRB, 0.29655E+00_JPRB, 0.29854E+00_JPRB, 0.29696E+00_JPRB ,&
&  0.29335E+00_JPRB, 0.29036E+00_JPRB, 0.28654E+00_JPRB, 0.28132E+00_JPRB, 0.27860E+00_JPRB, 0.27787E+00_JPRB, 0.27850E+00_JPRB ,&
&  0.28270E+00_JPRB, 0.28900E+00_JPRB, 0.30278E+00_JPRB, 0.31245E+00_JPRB, 0.30821E+00_JPRB, 0.29651E+00_JPRB, 0.28613E+00_JPRB ,&
&  0.27919E+00_JPRB, 0.27636E+00_JPRB, 0.27419E+00_JPRB, 0.27193E+00_JPRB, 0.27252E+00_JPRB, 0.27460E+00_JPRB, 0.27604E+00_JPRB ,&
&  0.27630E+00_JPRB, 0.27813E+00_JPRB, 0.27924E+00_JPRB, 0.28055E+00_JPRB, 0.28169E+00_JPRB, 0.28353E+00_JPRB, 0.28602E+00_JPRB ,&
&  0.28835E+00_JPRB, 0.29371E+00_JPRB, 0.29961E+00_JPRB, 0.30628E+00_JPRB, 0.31050E+00_JPRB, 0.31369E+00_JPRB, 0.32015E+00_JPRB ,&
&  0.32638E+00_JPRB, 0.34205E+00_JPRB, 0.37331E+00_JPRB, 0.40952E+00_JPRB, 0.40292E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.39989E+00_JPRB, 0.40876E+00_JPRB, 0.41652E+00_JPRB, 0.42259E+00_JPRB, 0.42649E+00_JPRB, 0.42736E+00_JPRB, 0.43020E+00_JPRB ,&
&  0.43783E+00_JPRB, 0.44234E+00_JPRB, 0.44578E+00_JPRB, 0.44500E+00_JPRB, 0.44450E+00_JPRB, 0.42607E+00_JPRB, 0.42164E+00_JPRB ,&
&  0.41550E+00_JPRB, 0.41122E+00_JPRB, 0.40527E+00_JPRB, 0.40516E+00_JPRB, 0.40490E+00_JPRB, 0.40695E+00_JPRB, 0.40774E+00_JPRB ,&
&  0.40810E+00_JPRB, 0.40647E+00_JPRB, 0.40342E+00_JPRB, 0.40156E+00_JPRB, 0.40277E+00_JPRB, 0.40406E+00_JPRB, 0.40101E+00_JPRB ,&
&  0.39660E+00_JPRB, 0.39556E+00_JPRB, 0.39873E+00_JPRB, 0.40569E+00_JPRB, 0.44574E+00_JPRB, 0.44029E+00_JPRB, 0.43810E+00_JPRB ,&
&  0.43478E+00_JPRB, 0.43055E+00_JPRB, 0.42991E+00_JPRB, 0.42320E+00_JPRB, 0.41354E+00_JPRB, 0.39169E+00_JPRB, 0.37958E+00_JPRB ,&
&  0.34527E+00_JPRB, 0.32645E+00_JPRB, 0.30989E+00_JPRB, 0.29912E+00_JPRB, 0.28861E+00_JPRB, 0.28251E+00_JPRB, 0.27706E+00_JPRB ,&
&  0.27293E+00_JPRB, 0.26966E+00_JPRB, 0.26674E+00_JPRB, 0.26730E+00_JPRB, 0.26908E+00_JPRB, 0.27001E+00_JPRB, 0.27095E+00_JPRB ,&
&  0.27151E+00_JPRB, 0.27203E+00_JPRB, 0.27269E+00_JPRB, 0.27411E+00_JPRB, 0.27598E+00_JPRB, 0.27844E+00_JPRB, 0.28157E+00_JPRB ,&
&  0.28665E+00_JPRB, 0.29061E+00_JPRB, 0.29209E+00_JPRB, 0.29482E+00_JPRB, 0.29448E+00_JPRB, 0.28853E+00_JPRB, 0.28183E+00_JPRB ,&
&  0.27479E+00_JPRB, 0.27112E+00_JPRB, 0.26750E+00_JPRB, 0.26494E+00_JPRB, 0.26432E+00_JPRB, 0.26427E+00_JPRB, 0.26624E+00_JPRB ,&
&  0.26842E+00_JPRB, 0.27037E+00_JPRB, 0.27175E+00_JPRB, 0.27325E+00_JPRB, 0.27518E+00_JPRB, 0.27664E+00_JPRB, 0.27854E+00_JPRB ,&
&  0.28096E+00_JPRB, 0.28424E+00_JPRB, 0.28914E+00_JPRB, 0.29482E+00_JPRB, 0.30215E+00_JPRB, 0.31291E+00_JPRB, 0.33169E+00_JPRB ,&
&  0.35624E+00_JPRB, 0.41147E+00_JPRB, 0.41623E+00_JPRB, 0.38668E+00_JPRB, 0.39288E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.37654E+00_JPRB, 0.38353E+00_JPRB, 0.39062E+00_JPRB, 0.39825E+00_JPRB, 0.40779E+00_JPRB, 0.41198E+00_JPRB, 0.41398E+00_JPRB ,&
&  0.40985E+00_JPRB, 0.40539E+00_JPRB, 0.39463E+00_JPRB, 0.38500E+00_JPRB, 0.37559E+00_JPRB, 0.36810E+00_JPRB, 0.36221E+00_JPRB ,&
&  0.35869E+00_JPRB, 0.35601E+00_JPRB, 0.35177E+00_JPRB, 0.34840E+00_JPRB, 0.34536E+00_JPRB, 0.34546E+00_JPRB, 0.34582E+00_JPRB ,&
&  0.34528E+00_JPRB, 0.34559E+00_JPRB, 0.34420E+00_JPRB, 0.34941E+00_JPRB, 0.35863E+00_JPRB, 0.36981E+00_JPRB, 0.38440E+00_JPRB ,&
&  0.40484E+00_JPRB, 0.41590E+00_JPRB, 0.43147E+00_JPRB, 0.43177E+00_JPRB, 0.42999E+00_JPRB, 0.42625E+00_JPRB, 0.42315E+00_JPRB ,&
&  0.41847E+00_JPRB, 0.41169E+00_JPRB, 0.40667E+00_JPRB, 0.40031E+00_JPRB, 0.37067E+00_JPRB, 0.34759E+00_JPRB, 0.32983E+00_JPRB ,&
&  0.31464E+00_JPRB, 0.30031E+00_JPRB, 0.28762E+00_JPRB, 0.28125E+00_JPRB, 0.27598E+00_JPRB, 0.27286E+00_JPRB, 0.26958E+00_JPRB ,&
&  0.26609E+00_JPRB, 0.26489E+00_JPRB, 0.26295E+00_JPRB, 0.26221E+00_JPRB, 0.26326E+00_JPRB, 0.26315E+00_JPRB, 0.26391E+00_JPRB ,&
&  0.26464E+00_JPRB, 0.26583E+00_JPRB, 0.26853E+00_JPRB, 0.27055E+00_JPRB, 0.27263E+00_JPRB, 0.27455E+00_JPRB, 0.27551E+00_JPRB ,&
&  0.27742E+00_JPRB, 0.27906E+00_JPRB, 0.27901E+00_JPRB, 0.27824E+00_JPRB, 0.27639E+00_JPRB, 0.27221E+00_JPRB, 0.26806E+00_JPRB ,&
&  0.26362E+00_JPRB, 0.26041E+00_JPRB, 0.25920E+00_JPRB, 0.25816E+00_JPRB, 0.25899E+00_JPRB, 0.25974E+00_JPRB, 0.26061E+00_JPRB ,&
&  0.26252E+00_JPRB, 0.26434E+00_JPRB, 0.26649E+00_JPRB, 0.26955E+00_JPRB, 0.27187E+00_JPRB, 0.27383E+00_JPRB, 0.27630E+00_JPRB ,&
&  0.27649E+00_JPRB, 0.28087E+00_JPRB, 0.28643E+00_JPRB, 0.29378E+00_JPRB, 0.30671E+00_JPRB, 0.31681E+00_JPRB, 0.33966E+00_JPRB ,&
&  0.37446E+00_JPRB, 0.44033E+00_JPRB, 0.43376E+00_JPRB, 0.36650E+00_JPRB, 0.37028E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.36488E+00_JPRB, 0.36792E+00_JPRB, 0.37126E+00_JPRB, 0.37536E+00_JPRB, 0.38042E+00_JPRB, 0.38498E+00_JPRB, 0.38961E+00_JPRB ,&
&  0.39249E+00_JPRB, 0.38953E+00_JPRB, 0.39028E+00_JPRB, 0.38572E+00_JPRB, 0.38112E+00_JPRB, 0.37521E+00_JPRB, 0.36910E+00_JPRB ,&
&  0.36275E+00_JPRB, 0.35372E+00_JPRB, 0.34407E+00_JPRB, 0.33557E+00_JPRB, 0.32836E+00_JPRB, 0.32277E+00_JPRB, 0.31836E+00_JPRB ,&
&  0.31530E+00_JPRB, 0.31576E+00_JPRB, 0.31787E+00_JPRB, 0.32393E+00_JPRB, 0.33410E+00_JPRB, 0.34662E+00_JPRB, 0.36566E+00_JPRB ,&
&  0.39249E+00_JPRB, 0.39466E+00_JPRB, 0.40004E+00_JPRB, 0.39917E+00_JPRB, 0.39506E+00_JPRB, 0.39319E+00_JPRB, 0.38948E+00_JPRB ,&
&  0.39005E+00_JPRB, 0.38207E+00_JPRB, 0.39182E+00_JPRB, 0.37491E+00_JPRB, 0.34762E+00_JPRB, 0.33222E+00_JPRB, 0.31383E+00_JPRB ,&
&  0.30053E+00_JPRB, 0.28920E+00_JPRB, 0.28073E+00_JPRB, 0.27727E+00_JPRB, 0.27336E+00_JPRB, 0.27155E+00_JPRB, 0.26771E+00_JPRB ,&
&  0.26377E+00_JPRB, 0.26186E+00_JPRB, 0.25867E+00_JPRB, 0.25845E+00_JPRB, 0.25857E+00_JPRB, 0.25797E+00_JPRB, 0.25806E+00_JPRB ,&
&  0.25849E+00_JPRB, 0.25960E+00_JPRB, 0.26162E+00_JPRB, 0.26344E+00_JPRB, 0.26453E+00_JPRB, 0.26517E+00_JPRB, 0.26503E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26523E+00_JPRB, 0.26436E+00_JPRB, 0.26300E+00_JPRB, 0.26092E+00_JPRB, 0.25873E+00_JPRB, 0.25663E+00_JPRB ,&
&  0.25492E+00_JPRB, 0.25425E+00_JPRB, 0.25367E+00_JPRB, 0.25385E+00_JPRB, 0.25441E+00_JPRB, 0.25620E+00_JPRB, 0.25788E+00_JPRB ,&
&  0.25982E+00_JPRB, 0.26255E+00_JPRB, 0.26494E+00_JPRB, 0.26788E+00_JPRB, 0.27054E+00_JPRB, 0.27204E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.28157E+00_JPRB, 0.28890E+00_JPRB, 0.29744E+00_JPRB, 0.30560E+00_JPRB, 0.31591E+00_JPRB, 0.33135E+00_JPRB, 0.35225E+00_JPRB ,&
&  0.37338E+00_JPRB, 0.39584E+00_JPRB, 0.38961E+00_JPRB, 0.36251E+00_JPRB, 0.36155E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.36540E+00_JPRB, 0.36373E+00_JPRB, 0.36442E+00_JPRB, 0.36561E+00_JPRB, 0.36951E+00_JPRB, 0.37500E+00_JPRB, 0.38095E+00_JPRB ,&
&  0.38058E+00_JPRB, 0.38381E+00_JPRB, 0.38287E+00_JPRB, 0.38006E+00_JPRB, 0.37541E+00_JPRB, 0.37087E+00_JPRB, 0.36612E+00_JPRB ,&
&  0.35937E+00_JPRB, 0.35069E+00_JPRB, 0.34222E+00_JPRB, 0.33357E+00_JPRB, 0.32535E+00_JPRB, 0.31680E+00_JPRB, 0.31186E+00_JPRB ,&
&  0.30853E+00_JPRB, 0.30767E+00_JPRB, 0.30969E+00_JPRB, 0.31238E+00_JPRB, 0.31774E+00_JPRB, 0.32506E+00_JPRB, 0.33479E+00_JPRB ,&
&  0.34426E+00_JPRB, 0.35504E+00_JPRB, 0.36067E+00_JPRB, 0.36055E+00_JPRB, 0.36038E+00_JPRB, 0.35834E+00_JPRB, 0.35490E+00_JPRB ,&
&  0.34830E+00_JPRB, 0.34085E+00_JPRB, 0.33217E+00_JPRB, 0.32299E+00_JPRB, 0.31363E+00_JPRB, 0.30490E+00_JPRB, 0.29678E+00_JPRB ,&
&  0.29019E+00_JPRB, 0.28460E+00_JPRB, 0.27946E+00_JPRB, 0.27514E+00_JPRB, 0.27180E+00_JPRB, 0.26933E+00_JPRB, 0.26626E+00_JPRB ,&
&  0.26371E+00_JPRB, 0.26145E+00_JPRB, 0.25876E+00_JPRB, 0.25691E+00_JPRB, 0.25526E+00_JPRB, 0.25416E+00_JPRB, 0.25326E+00_JPRB ,&
&  0.25319E+00_JPRB, 0.25314E+00_JPRB, 0.25382E+00_JPRB, 0.25451E+00_JPRB, 0.25532E+00_JPRB, 0.25532E+00_JPRB, 0.25512E+00_JPRB ,&
&  0.25471E+00_JPRB, 0.25430E+00_JPRB, 0.25287E+00_JPRB, 0.25175E+00_JPRB, 0.25097E+00_JPRB, 0.25011E+00_JPRB, 0.25005E+00_JPRB ,&
&  0.25088E+00_JPRB, 0.25174E+00_JPRB, 0.25244E+00_JPRB, 0.25408E+00_JPRB, 0.25571E+00_JPRB, 0.25809E+00_JPRB, 0.26096E+00_JPRB ,&
&  0.26307E+00_JPRB, 0.26555E+00_JPRB, 0.26831E+00_JPRB, 0.27151E+00_JPRB, 0.27534E+00_JPRB, 0.27929E+00_JPRB, 0.28395E+00_JPRB ,&
&  0.28921E+00_JPRB, 0.29457E+00_JPRB, 0.30060E+00_JPRB, 0.30725E+00_JPRB, 0.31527E+00_JPRB, 0.32166E+00_JPRB, 0.33140E+00_JPRB ,&
&  0.33897E+00_JPRB, 0.33725E+00_JPRB, 0.37766E+00_JPRB, 0.37132E+00_JPRB, 0.36724E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.30798E+00_JPRB, 0.31447E+00_JPRB, 0.31903E+00_JPRB, 0.32362E+00_JPRB, 0.31827E+00_JPRB, 0.35552E+00_JPRB, 0.35389E+00_JPRB ,&
&  0.35471E+00_JPRB, 0.34198E+00_JPRB, 0.34293E+00_JPRB, 0.34180E+00_JPRB, 0.33902E+00_JPRB, 0.33486E+00_JPRB, 0.33101E+00_JPRB ,&
&  0.32761E+00_JPRB, 0.32295E+00_JPRB, 0.31751E+00_JPRB, 0.31284E+00_JPRB, 0.30913E+00_JPRB, 0.30595E+00_JPRB, 0.30324E+00_JPRB ,&
&  0.30262E+00_JPRB, 0.30389E+00_JPRB, 0.30634E+00_JPRB, 0.30503E+00_JPRB, 0.30593E+00_JPRB, 0.30777E+00_JPRB, 0.30999E+00_JPRB ,&
&  0.31206E+00_JPRB, 0.31387E+00_JPRB, 0.31391E+00_JPRB, 0.31220E+00_JPRB, 0.31113E+00_JPRB, 0.30885E+00_JPRB, 0.30671E+00_JPRB ,&
&  0.30410E+00_JPRB, 0.30121E+00_JPRB, 0.29780E+00_JPRB, 0.29508E+00_JPRB, 0.29208E+00_JPRB, 0.28844E+00_JPRB, 0.28493E+00_JPRB ,&
&  0.28193E+00_JPRB, 0.27923E+00_JPRB, 0.27625E+00_JPRB, 0.27348E+00_JPRB, 0.27084E+00_JPRB, 0.26816E+00_JPRB, 0.26576E+00_JPRB ,&
&  0.26393E+00_JPRB, 0.26238E+00_JPRB, 0.26084E+00_JPRB, 0.25923E+00_JPRB, 0.25766E+00_JPRB, 0.25614E+00_JPRB, 0.25491E+00_JPRB ,&
&  0.25475E+00_JPRB, 0.25406E+00_JPRB, 0.25338E+00_JPRB, 0.25302E+00_JPRB, 0.25310E+00_JPRB, 0.25262E+00_JPRB, 0.25244E+00_JPRB ,&
&  0.25241E+00_JPRB, 0.25246E+00_JPRB, 0.25270E+00_JPRB, 0.25301E+00_JPRB, 0.25344E+00_JPRB, 0.25400E+00_JPRB, 0.25491E+00_JPRB ,&
&  0.25598E+00_JPRB, 0.25729E+00_JPRB, 0.25869E+00_JPRB, 0.26013E+00_JPRB, 0.26160E+00_JPRB, 0.26316E+00_JPRB, 0.26491E+00_JPRB ,&
&  0.26667E+00_JPRB, 0.26838E+00_JPRB, 0.27005E+00_JPRB, 0.27169E+00_JPRB, 0.27439E+00_JPRB, 0.27697E+00_JPRB, 0.27973E+00_JPRB ,&
&  0.28272E+00_JPRB, 0.28621E+00_JPRB, 0.28989E+00_JPRB, 0.29387E+00_JPRB, 0.29833E+00_JPRB, 0.29973E+00_JPRB, 0.30083E+00_JPRB ,&
&  0.30148E+00_JPRB, 0.30440E+00_JPRB, 0.30625E+00_JPRB, 0.30585E+00_JPRB, 0.30543E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.28037E+00_JPRB, 0.28049E+00_JPRB, 0.28057E+00_JPRB, 0.28061E+00_JPRB, 0.27932E+00_JPRB, 0.27980E+00_JPRB, 0.28024E+00_JPRB ,&
&  0.28099E+00_JPRB, 0.28247E+00_JPRB, 0.28415E+00_JPRB, 0.28541E+00_JPRB, 0.28630E+00_JPRB, 0.28684E+00_JPRB, 0.28722E+00_JPRB ,&
&  0.28727E+00_JPRB, 0.28750E+00_JPRB, 0.28415E+00_JPRB, 0.28296E+00_JPRB, 0.28258E+00_JPRB, 0.28260E+00_JPRB, 0.28281E+00_JPRB ,&
&  0.28295E+00_JPRB, 0.28295E+00_JPRB, 0.28244E+00_JPRB, 0.28274E+00_JPRB, 0.28307E+00_JPRB, 0.28330E+00_JPRB, 0.28355E+00_JPRB ,&
&  0.28379E+00_JPRB, 0.28398E+00_JPRB, 0.28392E+00_JPRB, 0.28369E+00_JPRB, 0.28355E+00_JPRB, 0.28305E+00_JPRB, 0.28218E+00_JPRB ,&
&  0.28112E+00_JPRB, 0.27980E+00_JPRB, 0.27849E+00_JPRB, 0.27728E+00_JPRB, 0.27624E+00_JPRB, 0.27535E+00_JPRB, 0.27449E+00_JPRB ,&
&  0.27346E+00_JPRB, 0.27211E+00_JPRB, 0.27068E+00_JPRB, 0.26929E+00_JPRB, 0.26796E+00_JPRB, 0.26675E+00_JPRB, 0.26547E+00_JPRB ,&
&  0.26449E+00_JPRB, 0.26366E+00_JPRB, 0.26284E+00_JPRB, 0.26214E+00_JPRB, 0.26145E+00_JPRB, 0.26079E+00_JPRB, 0.26017E+00_JPRB ,&
&  0.25932E+00_JPRB, 0.25886E+00_JPRB, 0.25834E+00_JPRB, 0.25810E+00_JPRB, 0.25817E+00_JPRB, 0.25820E+00_JPRB, 0.25833E+00_JPRB ,&
&  0.25851E+00_JPRB, 0.25876E+00_JPRB, 0.25890E+00_JPRB, 0.25912E+00_JPRB, 0.25945E+00_JPRB, 0.25987E+00_JPRB, 0.26039E+00_JPRB ,&
&  0.26094E+00_JPRB, 0.26155E+00_JPRB, 0.26222E+00_JPRB, 0.26296E+00_JPRB, 0.26373E+00_JPRB, 0.26459E+00_JPRB, 0.26540E+00_JPRB ,&
&  0.26622E+00_JPRB, 0.26698E+00_JPRB, 0.26774E+00_JPRB, 0.26832E+00_JPRB, 0.26917E+00_JPRB, 0.26998E+00_JPRB, 0.27096E+00_JPRB ,&
&  0.27194E+00_JPRB, 0.27298E+00_JPRB, 0.27389E+00_JPRB, 0.27488E+00_JPRB, 0.27597E+00_JPRB, 0.27577E+00_JPRB, 0.27565E+00_JPRB ,&
&  0.27560E+00_JPRB, 0.27584E+00_JPRB, 0.27580E+00_JPRB, 0.27611E+00_JPRB, 0.27697E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB, 0.26519E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB, 0.45286E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.45603E+00_JPRB, 0.45628E+00_JPRB, 0.45642E+00_JPRB, 0.45647E+00_JPRB, 0.45653E+00_JPRB, 0.45678E+00_JPRB, 0.45696E+00_JPRB ,&
&  0.45720E+00_JPRB, 0.45752E+00_JPRB, 0.45813E+00_JPRB, 0.45844E+00_JPRB, 0.45875E+00_JPRB, 0.45891E+00_JPRB, 0.45912E+00_JPRB ,&
&  0.45924E+00_JPRB, 0.45935E+00_JPRB, 0.45934E+00_JPRB, 0.45943E+00_JPRB, 0.45958E+00_JPRB, 0.45986E+00_JPRB, 0.46010E+00_JPRB ,&
&  0.45999E+00_JPRB, 0.45957E+00_JPRB, 0.45925E+00_JPRB, 0.45883E+00_JPRB, 0.45828E+00_JPRB, 0.45735E+00_JPRB, 0.45619E+00_JPRB ,&
&  0.45490E+00_JPRB, 0.45365E+00_JPRB, 0.45262E+00_JPRB, 0.45163E+00_JPRB, 0.45091E+00_JPRB, 0.45030E+00_JPRB, 0.44991E+00_JPRB ,&
&  0.44937E+00_JPRB, 0.44912E+00_JPRB, 0.44887E+00_JPRB, 0.44873E+00_JPRB, 0.44825E+00_JPRB, 0.44788E+00_JPRB, 0.44737E+00_JPRB ,&
&  0.44683E+00_JPRB, 0.44607E+00_JPRB, 0.44548E+00_JPRB, 0.44475E+00_JPRB, 0.44395E+00_JPRB, 0.44273E+00_JPRB, 0.44162E+00_JPRB ,&
&  0.44054E+00_JPRB, 0.43941E+00_JPRB, 0.43816E+00_JPRB, 0.43711E+00_JPRB, 0.43634E+00_JPRB, 0.43560E+00_JPRB, 0.43471E+00_JPRB ,&
&  0.43443E+00_JPRB, 0.43468E+00_JPRB, 0.43490E+00_JPRB, 0.43489E+00_JPRB, 0.43526E+00_JPRB, 0.43574E+00_JPRB, 0.43630E+00_JPRB ,&
&  0.43680E+00_JPRB, 0.43757E+00_JPRB, 0.43871E+00_JPRB, 0.43977E+00_JPRB, 0.44084E+00_JPRB, 0.44194E+00_JPRB, 0.44320E+00_JPRB ,&
&  0.44436E+00_JPRB, 0.44550E+00_JPRB, 0.44655E+00_JPRB, 0.44766E+00_JPRB, 0.44837E+00_JPRB, 0.44895E+00_JPRB, 0.44920E+00_JPRB ,&
&  0.44951E+00_JPRB, 0.44969E+00_JPRB, 0.44992E+00_JPRB, 0.45010E+00_JPRB, 0.45050E+00_JPRB, 0.45082E+00_JPRB, 0.45119E+00_JPRB ,&
&  0.45158E+00_JPRB, 0.45212E+00_JPRB, 0.45255E+00_JPRB, 0.45293E+00_JPRB, 0.45331E+00_JPRB, 0.45388E+00_JPRB, 0.45432E+00_JPRB ,&
&  0.45467E+00_JPRB, 0.45507E+00_JPRB, 0.45544E+00_JPRB, 0.45575E+00_JPRB, 0.45590E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.45094E+00_JPRB, 0.45263E+00_JPRB, 0.45363E+00_JPRB, 0.45457E+00_JPRB, 0.45548E+00_JPRB, 0.45691E+00_JPRB, 0.45806E+00_JPRB ,&
&  0.45926E+00_JPRB, 0.46091E+00_JPRB, 0.46310E+00_JPRB, 0.46453E+00_JPRB, 0.46563E+00_JPRB, 0.46634E+00_JPRB, 0.46698E+00_JPRB ,&
&  0.46714E+00_JPRB, 0.46692E+00_JPRB, 0.46631E+00_JPRB, 0.46588E+00_JPRB, 0.46545E+00_JPRB, 0.46505E+00_JPRB, 0.46444E+00_JPRB ,&
&  0.46317E+00_JPRB, 0.46151E+00_JPRB, 0.45967E+00_JPRB, 0.45698E+00_JPRB, 0.45391E+00_JPRB, 0.45055E+00_JPRB, 0.44691E+00_JPRB ,&
&  0.44324E+00_JPRB, 0.44028E+00_JPRB, 0.43741E+00_JPRB, 0.43466E+00_JPRB, 0.43342E+00_JPRB, 0.43237E+00_JPRB, 0.43131E+00_JPRB ,&
&  0.43025E+00_JPRB, 0.42876E+00_JPRB, 0.42848E+00_JPRB, 0.42847E+00_JPRB, 0.42881E+00_JPRB, 0.42990E+00_JPRB, 0.43090E+00_JPRB ,&
&  0.43153E+00_JPRB, 0.43151E+00_JPRB, 0.43109E+00_JPRB, 0.43097E+00_JPRB, 0.43088E+00_JPRB, 0.43049E+00_JPRB, 0.42954E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42765E+00_JPRB, 0.42619E+00_JPRB, 0.42453E+00_JPRB, 0.42349E+00_JPRB, 0.42225E+00_JPRB, 0.42092E+00_JPRB ,&
&  0.42003E+00_JPRB, 0.41969E+00_JPRB, 0.41953E+00_JPRB, 0.41947E+00_JPRB, 0.42000E+00_JPRB, 0.42080E+00_JPRB, 0.42161E+00_JPRB ,&
&  0.42247E+00_JPRB, 0.42412E+00_JPRB, 0.42614E+00_JPRB, 0.42783E+00_JPRB, 0.42940E+00_JPRB, 0.43090E+00_JPRB, 0.43276E+00_JPRB ,&
&  0.43407E+00_JPRB, 0.43498E+00_JPRB, 0.43507E+00_JPRB, 0.43584E+00_JPRB, 0.43635E+00_JPRB, 0.43728E+00_JPRB, 0.43762E+00_JPRB ,&
&  0.43866E+00_JPRB, 0.43908E+00_JPRB, 0.43925E+00_JPRB, 0.43957E+00_JPRB, 0.44004E+00_JPRB, 0.44040E+00_JPRB, 0.44042E+00_JPRB ,&
&  0.44045E+00_JPRB, 0.44063E+00_JPRB, 0.44076E+00_JPRB, 0.44085E+00_JPRB, 0.44139E+00_JPRB, 0.44211E+00_JPRB, 0.44294E+00_JPRB ,&
&  0.44391E+00_JPRB, 0.44539E+00_JPRB, 0.44705E+00_JPRB, 0.44832E+00_JPRB, 0.44949E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.44539E+00_JPRB, 0.44675E+00_JPRB, 0.44775E+00_JPRB, 0.44933E+00_JPRB, 0.45223E+00_JPRB, 0.45566E+00_JPRB, 0.45785E+00_JPRB ,&
&  0.46075E+00_JPRB, 0.46420E+00_JPRB, 0.46794E+00_JPRB, 0.47027E+00_JPRB, 0.47202E+00_JPRB, 0.47290E+00_JPRB, 0.47290E+00_JPRB ,&
&  0.47184E+00_JPRB, 0.47038E+00_JPRB, 0.46903E+00_JPRB, 0.46760E+00_JPRB, 0.46613E+00_JPRB, 0.46449E+00_JPRB, 0.46135E+00_JPRB ,&
&  0.45847E+00_JPRB, 0.45695E+00_JPRB, 0.45277E+00_JPRB, 0.44765E+00_JPRB, 0.44330E+00_JPRB, 0.41171E+00_JPRB, 0.40073E+00_JPRB ,&
&  0.39859E+00_JPRB, 0.39604E+00_JPRB, 0.39687E+00_JPRB, 0.39818E+00_JPRB, 0.40388E+00_JPRB, 0.40019E+00_JPRB, 0.39789E+00_JPRB ,&
&  0.37968E+00_JPRB, 0.37891E+00_JPRB, 0.37567E+00_JPRB, 0.37401E+00_JPRB, 0.37597E+00_JPRB, 0.37821E+00_JPRB, 0.38531E+00_JPRB ,&
&  0.40305E+00_JPRB, 0.40853E+00_JPRB, 0.41111E+00_JPRB, 0.41276E+00_JPRB, 0.41419E+00_JPRB, 0.41493E+00_JPRB, 0.41562E+00_JPRB ,&
&  0.41600E+00_JPRB, 0.41584E+00_JPRB, 0.41569E+00_JPRB, 0.41511E+00_JPRB, 0.41440E+00_JPRB, 0.41327E+00_JPRB, 0.41240E+00_JPRB ,&
&  0.41180E+00_JPRB, 0.41197E+00_JPRB, 0.41266E+00_JPRB, 0.41323E+00_JPRB, 0.41426E+00_JPRB, 0.41483E+00_JPRB, 0.41537E+00_JPRB ,&
&  0.41572E+00_JPRB, 0.41613E+00_JPRB, 0.41709E+00_JPRB, 0.41827E+00_JPRB, 0.41977E+00_JPRB, 0.42032E+00_JPRB, 0.42157E+00_JPRB ,&
&  0.42250E+00_JPRB, 0.42393E+00_JPRB, 0.42486E+00_JPRB, 0.42702E+00_JPRB, 0.42824E+00_JPRB, 0.43061E+00_JPRB, 0.43249E+00_JPRB ,&
&  0.43539E+00_JPRB, 0.43712E+00_JPRB, 0.43879E+00_JPRB, 0.43984E+00_JPRB, 0.44089E+00_JPRB, 0.44161E+00_JPRB, 0.44166E+00_JPRB ,&
&  0.44160E+00_JPRB, 0.44086E+00_JPRB, 0.44012E+00_JPRB, 0.43910E+00_JPRB, 0.43849E+00_JPRB, 0.43770E+00_JPRB, 0.43732E+00_JPRB ,&
&  0.43669E+00_JPRB, 0.43744E+00_JPRB, 0.43926E+00_JPRB, 0.44160E+00_JPRB, 0.44358E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.45639E+00_JPRB, 0.45640E+00_JPRB, 0.45581E+00_JPRB, 0.45737E+00_JPRB, 0.45947E+00_JPRB, 0.46367E+00_JPRB, 0.46442E+00_JPRB ,&
&  0.46830E+00_JPRB, 0.47212E+00_JPRB, 0.47617E+00_JPRB, 0.47778E+00_JPRB, 0.47892E+00_JPRB, 0.47861E+00_JPRB, 0.47688E+00_JPRB ,&
&  0.47476E+00_JPRB, 0.47133E+00_JPRB, 0.46844E+00_JPRB, 0.46483E+00_JPRB, 0.46104E+00_JPRB, 0.45891E+00_JPRB, 0.45664E+00_JPRB ,&
&  0.45555E+00_JPRB, 0.45342E+00_JPRB, 0.42397E+00_JPRB, 0.41358E+00_JPRB, 0.39806E+00_JPRB, 0.38922E+00_JPRB, 0.38444E+00_JPRB ,&
&  0.37944E+00_JPRB, 0.37450E+00_JPRB, 0.36802E+00_JPRB, 0.35874E+00_JPRB, 0.34750E+00_JPRB, 0.33212E+00_JPRB, 0.32467E+00_JPRB ,&
&  0.31814E+00_JPRB, 0.31652E+00_JPRB, 0.31763E+00_JPRB, 0.32163E+00_JPRB, 0.32983E+00_JPRB, 0.33924E+00_JPRB, 0.35118E+00_JPRB ,&
&  0.36796E+00_JPRB, 0.38469E+00_JPRB, 0.39414E+00_JPRB, 0.39263E+00_JPRB, 0.40252E+00_JPRB, 0.40740E+00_JPRB, 0.40998E+00_JPRB ,&
&  0.41104E+00_JPRB, 0.40948E+00_JPRB, 0.41255E+00_JPRB, 0.41229E+00_JPRB, 0.41227E+00_JPRB, 0.41271E+00_JPRB, 0.41361E+00_JPRB ,&
&  0.41562E+00_JPRB, 0.41761E+00_JPRB, 0.41893E+00_JPRB, 0.41992E+00_JPRB, 0.42013E+00_JPRB, 0.41971E+00_JPRB, 0.41914E+00_JPRB ,&
&  0.41868E+00_JPRB, 0.41833E+00_JPRB, 0.41818E+00_JPRB, 0.41841E+00_JPRB, 0.41947E+00_JPRB, 0.42006E+00_JPRB, 0.42010E+00_JPRB ,&
&  0.42096E+00_JPRB, 0.42242E+00_JPRB, 0.42406E+00_JPRB, 0.42340E+00_JPRB, 0.42553E+00_JPRB, 0.43184E+00_JPRB, 0.43506E+00_JPRB ,&
&  0.44077E+00_JPRB, 0.44520E+00_JPRB, 0.45050E+00_JPRB, 0.45449E+00_JPRB, 0.45777E+00_JPRB, 0.45908E+00_JPRB, 0.45853E+00_JPRB ,&
&  0.45763E+00_JPRB, 0.45614E+00_JPRB, 0.45518E+00_JPRB, 0.45317E+00_JPRB, 0.45172E+00_JPRB, 0.44961E+00_JPRB, 0.44875E+00_JPRB ,&
&  0.44793E+00_JPRB, 0.44853E+00_JPRB, 0.44806E+00_JPRB, 0.45101E+00_JPRB, 0.45362E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.47197E+00_JPRB, 0.47160E+00_JPRB, 0.47207E+00_JPRB, 0.47600E+00_JPRB, 0.48337E+00_JPRB, 0.49179E+00_JPRB, 0.49057E+00_JPRB ,&
&  0.49011E+00_JPRB, 0.48704E+00_JPRB, 0.48789E+00_JPRB, 0.48669E+00_JPRB, 0.48468E+00_JPRB, 0.48160E+00_JPRB, 0.47889E+00_JPRB ,&
&  0.47646E+00_JPRB, 0.47051E+00_JPRB, 0.46326E+00_JPRB, 0.46143E+00_JPRB, 0.46260E+00_JPRB, 0.46311E+00_JPRB, 0.46073E+00_JPRB ,&
&  0.45614E+00_JPRB, 0.44587E+00_JPRB, 0.42270E+00_JPRB, 0.40556E+00_JPRB, 0.38937E+00_JPRB, 0.38349E+00_JPRB, 0.37373E+00_JPRB ,&
&  0.37819E+00_JPRB, 0.34678E+00_JPRB, 0.34652E+00_JPRB, 0.33287E+00_JPRB, 0.32217E+00_JPRB, 0.31057E+00_JPRB, 0.29957E+00_JPRB ,&
&  0.29609E+00_JPRB, 0.29233E+00_JPRB, 0.29190E+00_JPRB, 0.29750E+00_JPRB, 0.30481E+00_JPRB, 0.31769E+00_JPRB, 0.33255E+00_JPRB ,&
&  0.34563E+00_JPRB, 0.38344E+00_JPRB, 0.39408E+00_JPRB, 0.39805E+00_JPRB, 0.40757E+00_JPRB, 0.41381E+00_JPRB, 0.41460E+00_JPRB ,&
&  0.41463E+00_JPRB, 0.41325E+00_JPRB, 0.41632E+00_JPRB, 0.41808E+00_JPRB, 0.42217E+00_JPRB, 0.42211E+00_JPRB, 0.42221E+00_JPRB ,&
&  0.42429E+00_JPRB, 0.42665E+00_JPRB, 0.42943E+00_JPRB, 0.43174E+00_JPRB, 0.43391E+00_JPRB, 0.43527E+00_JPRB, 0.43599E+00_JPRB ,&
&  0.43599E+00_JPRB, 0.43550E+00_JPRB, 0.43484E+00_JPRB, 0.43525E+00_JPRB, 0.43559E+00_JPRB, 0.43784E+00_JPRB, 0.43871E+00_JPRB ,&
&  0.43955E+00_JPRB, 0.43967E+00_JPRB, 0.43912E+00_JPRB, 0.43989E+00_JPRB, 0.44257E+00_JPRB, 0.44511E+00_JPRB, 0.44978E+00_JPRB ,&
&  0.45740E+00_JPRB, 0.46251E+00_JPRB, 0.46856E+00_JPRB, 0.47374E+00_JPRB, 0.47792E+00_JPRB, 0.47821E+00_JPRB, 0.47614E+00_JPRB ,&
&  0.47462E+00_JPRB, 0.47290E+00_JPRB, 0.47117E+00_JPRB, 0.46833E+00_JPRB, 0.46557E+00_JPRB, 0.46227E+00_JPRB, 0.46024E+00_JPRB ,&
&  0.45927E+00_JPRB, 0.46091E+00_JPRB, 0.46051E+00_JPRB, 0.46389E+00_JPRB, 0.46809E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.48418E+00_JPRB, 0.48600E+00_JPRB, 0.49100E+00_JPRB, 0.49715E+00_JPRB, 0.50365E+00_JPRB, 0.51053E+00_JPRB, 0.51312E+00_JPRB ,&
&  0.51471E+00_JPRB, 0.51138E+00_JPRB, 0.50656E+00_JPRB, 0.49436E+00_JPRB, 0.48612E+00_JPRB, 0.48204E+00_JPRB, 0.47721E+00_JPRB ,&
&  0.47231E+00_JPRB, 0.46979E+00_JPRB, 0.46822E+00_JPRB, 0.46801E+00_JPRB, 0.46564E+00_JPRB, 0.46424E+00_JPRB, 0.45735E+00_JPRB ,&
&  0.44337E+00_JPRB, 0.43092E+00_JPRB, 0.41157E+00_JPRB, 0.39427E+00_JPRB, 0.38254E+00_JPRB, 0.37289E+00_JPRB, 0.37249E+00_JPRB ,&
&  0.37051E+00_JPRB, 0.36940E+00_JPRB, 0.36814E+00_JPRB, 0.36666E+00_JPRB, 0.36889E+00_JPRB, 0.31280E+00_JPRB, 0.29862E+00_JPRB ,&
&  0.28928E+00_JPRB, 0.28280E+00_JPRB, 0.28130E+00_JPRB, 0.28197E+00_JPRB, 0.28732E+00_JPRB, 0.30326E+00_JPRB, 0.32534E+00_JPRB ,&
&  0.34327E+00_JPRB, 0.39472E+00_JPRB, 0.40402E+00_JPRB, 0.41283E+00_JPRB, 0.42611E+00_JPRB, 0.43475E+00_JPRB, 0.43860E+00_JPRB ,&
&  0.44470E+00_JPRB, 0.45434E+00_JPRB, 0.46462E+00_JPRB, 0.47351E+00_JPRB, 0.47276E+00_JPRB, 0.47370E+00_JPRB, 0.47138E+00_JPRB ,&
&  0.46521E+00_JPRB, 0.45804E+00_JPRB, 0.45588E+00_JPRB, 0.45501E+00_JPRB, 0.45473E+00_JPRB, 0.45503E+00_JPRB, 0.45433E+00_JPRB ,&
&  0.45614E+00_JPRB, 0.45639E+00_JPRB, 0.46059E+00_JPRB, 0.46194E+00_JPRB, 0.46484E+00_JPRB, 0.46868E+00_JPRB, 0.47093E+00_JPRB ,&
&  0.47092E+00_JPRB, 0.47111E+00_JPRB, 0.46783E+00_JPRB, 0.46310E+00_JPRB, 0.46138E+00_JPRB, 0.45939E+00_JPRB, 0.46132E+00_JPRB ,&
&  0.46495E+00_JPRB, 0.47265E+00_JPRB, 0.47656E+00_JPRB, 0.48095E+00_JPRB, 0.48485E+00_JPRB, 0.48505E+00_JPRB, 0.48541E+00_JPRB ,&
&  0.48512E+00_JPRB, 0.48546E+00_JPRB, 0.48407E+00_JPRB, 0.48153E+00_JPRB, 0.47712E+00_JPRB, 0.47244E+00_JPRB, 0.46833E+00_JPRB ,&
&  0.46797E+00_JPRB, 0.47047E+00_JPRB, 0.47087E+00_JPRB, 0.47388E+00_JPRB, 0.47911E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.49570E+00_JPRB, 0.49911E+00_JPRB, 0.50655E+00_JPRB, 0.51251E+00_JPRB, 0.52073E+00_JPRB, 0.52706E+00_JPRB, 0.52948E+00_JPRB ,&
&  0.53709E+00_JPRB, 0.53264E+00_JPRB, 0.51893E+00_JPRB, 0.50153E+00_JPRB, 0.48415E+00_JPRB, 0.47307E+00_JPRB, 0.46793E+00_JPRB ,&
&  0.46764E+00_JPRB, 0.46289E+00_JPRB, 0.45970E+00_JPRB, 0.45849E+00_JPRB, 0.45659E+00_JPRB, 0.45460E+00_JPRB, 0.44845E+00_JPRB ,&
&  0.43245E+00_JPRB, 0.42083E+00_JPRB, 0.40846E+00_JPRB, 0.39263E+00_JPRB, 0.37895E+00_JPRB, 0.37165E+00_JPRB, 0.37472E+00_JPRB ,&
&  0.37745E+00_JPRB, 0.37110E+00_JPRB, 0.36969E+00_JPRB, 0.36767E+00_JPRB, 0.36484E+00_JPRB, 0.36159E+00_JPRB, 0.31404E+00_JPRB ,&
&  0.29800E+00_JPRB, 0.28490E+00_JPRB, 0.28018E+00_JPRB, 0.28070E+00_JPRB, 0.29188E+00_JPRB, 0.31043E+00_JPRB, 0.34386E+00_JPRB ,&
&  0.37741E+00_JPRB, 0.40769E+00_JPRB, 0.42581E+00_JPRB, 0.44020E+00_JPRB, 0.45576E+00_JPRB, 0.47036E+00_JPRB, 0.48030E+00_JPRB ,&
&  0.49509E+00_JPRB, 0.51363E+00_JPRB, 0.52910E+00_JPRB, 0.54239E+00_JPRB, 0.55040E+00_JPRB, 0.55073E+00_JPRB, 0.55092E+00_JPRB ,&
&  0.54451E+00_JPRB, 0.50045E+00_JPRB, 0.48170E+00_JPRB, 0.46980E+00_JPRB, 0.46218E+00_JPRB, 0.46209E+00_JPRB, 0.46047E+00_JPRB ,&
&  0.46458E+00_JPRB, 0.47044E+00_JPRB, 0.47287E+00_JPRB, 0.47527E+00_JPRB, 0.47732E+00_JPRB, 0.47781E+00_JPRB, 0.48101E+00_JPRB ,&
&  0.48724E+00_JPRB, 0.49709E+00_JPRB, 0.51583E+00_JPRB, 0.50362E+00_JPRB, 0.48788E+00_JPRB, 0.47281E+00_JPRB, 0.46491E+00_JPRB ,&
&  0.46319E+00_JPRB, 0.46606E+00_JPRB, 0.47189E+00_JPRB, 0.47444E+00_JPRB, 0.47730E+00_JPRB, 0.48067E+00_JPRB, 0.48899E+00_JPRB ,&
&  0.49110E+00_JPRB, 0.49345E+00_JPRB, 0.49398E+00_JPRB, 0.49327E+00_JPRB, 0.48869E+00_JPRB, 0.48399E+00_JPRB, 0.48063E+00_JPRB ,&
&  0.48094E+00_JPRB, 0.48313E+00_JPRB, 0.48481E+00_JPRB, 0.48761E+00_JPRB, 0.49197E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.49936E+00_JPRB, 0.50805E+00_JPRB, 0.52189E+00_JPRB, 0.53186E+00_JPRB, 0.54178E+00_JPRB, 0.53406E+00_JPRB, 0.52535E+00_JPRB ,&
&  0.52304E+00_JPRB, 0.51982E+00_JPRB, 0.50908E+00_JPRB, 0.49559E+00_JPRB, 0.47801E+00_JPRB, 0.46460E+00_JPRB, 0.45969E+00_JPRB ,&
&  0.45929E+00_JPRB, 0.45654E+00_JPRB, 0.45297E+00_JPRB, 0.44609E+00_JPRB, 0.43920E+00_JPRB, 0.43797E+00_JPRB, 0.43340E+00_JPRB ,&
&  0.42308E+00_JPRB, 0.41519E+00_JPRB, 0.40667E+00_JPRB, 0.39635E+00_JPRB, 0.38587E+00_JPRB, 0.38222E+00_JPRB, 0.38093E+00_JPRB ,&
&  0.37948E+00_JPRB, 0.37989E+00_JPRB, 0.38230E+00_JPRB, 0.37641E+00_JPRB, 0.37667E+00_JPRB, 0.37970E+00_JPRB, 0.34532E+00_JPRB ,&
&  0.31439E+00_JPRB, 0.29425E+00_JPRB, 0.28353E+00_JPRB, 0.28409E+00_JPRB, 0.29947E+00_JPRB, 0.33113E+00_JPRB, 0.36988E+00_JPRB ,&
&  0.40960E+00_JPRB, 0.44065E+00_JPRB, 0.46484E+00_JPRB, 0.48103E+00_JPRB, 0.49367E+00_JPRB, 0.50867E+00_JPRB, 0.52492E+00_JPRB ,&
&  0.54526E+00_JPRB, 0.57170E+00_JPRB, 0.59297E+00_JPRB, 0.61444E+00_JPRB, 0.62703E+00_JPRB, 0.59563E+00_JPRB, 0.62854E+00_JPRB ,&
&  0.75563E+00_JPRB, 0.62685E+00_JPRB, 0.52914E+00_JPRB, 0.49184E+00_JPRB, 0.48578E+00_JPRB, 0.49106E+00_JPRB, 0.49861E+00_JPRB ,&
&  0.50751E+00_JPRB, 0.50961E+00_JPRB, 0.50620E+00_JPRB, 0.50223E+00_JPRB, 0.50267E+00_JPRB, 0.50634E+00_JPRB, 0.51951E+00_JPRB ,&
&  0.54135E+00_JPRB, 0.64320E+00_JPRB, 0.82384E+00_JPRB, 0.62151E+00_JPRB, 0.54619E+00_JPRB, 0.49790E+00_JPRB, 0.46211E+00_JPRB ,&
&  0.44813E+00_JPRB, 0.44545E+00_JPRB, 0.45024E+00_JPRB, 0.45340E+00_JPRB, 0.45252E+00_JPRB, 0.45483E+00_JPRB, 0.45968E+00_JPRB ,&
&  0.47034E+00_JPRB, 0.48152E+00_JPRB, 0.48876E+00_JPRB, 0.49455E+00_JPRB, 0.49407E+00_JPRB, 0.48939E+00_JPRB, 0.48446E+00_JPRB ,&
&  0.48763E+00_JPRB, 0.49264E+00_JPRB, 0.49635E+00_JPRB, 0.49783E+00_JPRB, 0.49756E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.49909E+00_JPRB, 0.52473E+00_JPRB, 0.55385E+00_JPRB, 0.55509E+00_JPRB, 0.55457E+00_JPRB, 0.52463E+00_JPRB, 0.49256E+00_JPRB ,&
&  0.47156E+00_JPRB, 0.46035E+00_JPRB, 0.45645E+00_JPRB, 0.44908E+00_JPRB, 0.44299E+00_JPRB, 0.44025E+00_JPRB, 0.43811E+00_JPRB ,&
&  0.43671E+00_JPRB, 0.43462E+00_JPRB, 0.43190E+00_JPRB, 0.42675E+00_JPRB, 0.42463E+00_JPRB, 0.42747E+00_JPRB, 0.42506E+00_JPRB ,&
&  0.41802E+00_JPRB, 0.41419E+00_JPRB, 0.40346E+00_JPRB, 0.38989E+00_JPRB, 0.38369E+00_JPRB, 0.38574E+00_JPRB, 0.39179E+00_JPRB ,&
&  0.38869E+00_JPRB, 0.38363E+00_JPRB, 0.39002E+00_JPRB, 0.39968E+00_JPRB, 0.39178E+00_JPRB, 0.39168E+00_JPRB, 0.35551E+00_JPRB ,&
&  0.32088E+00_JPRB, 0.30024E+00_JPRB, 0.29478E+00_JPRB, 0.31698E+00_JPRB, 0.36019E+00_JPRB, 0.41878E+00_JPRB, 0.45060E+00_JPRB ,&
&  0.47356E+00_JPRB, 0.49734E+00_JPRB, 0.51858E+00_JPRB, 0.54027E+00_JPRB, 0.56906E+00_JPRB, 0.59771E+00_JPRB, 0.63070E+00_JPRB ,&
&  0.65500E+00_JPRB, 0.66686E+00_JPRB, 0.66279E+00_JPRB, 0.59786E+00_JPRB, 0.58085E+00_JPRB, 0.57016E+00_JPRB, 0.61180E+00_JPRB ,&
&  0.65717E+00_JPRB, 0.60651E+00_JPRB, 0.57194E+00_JPRB, 0.57236E+00_JPRB, 0.57004E+00_JPRB, 0.57911E+00_JPRB, 0.58166E+00_JPRB ,&
&  0.57079E+00_JPRB, 0.55117E+00_JPRB, 0.53174E+00_JPRB, 0.51747E+00_JPRB, 0.51906E+00_JPRB, 0.53301E+00_JPRB, 0.54885E+00_JPRB ,&
&  0.57839E+00_JPRB, 0.64555E+00_JPRB, 0.63978E+00_JPRB, 0.50081E+00_JPRB, 0.47948E+00_JPRB, 0.47042E+00_JPRB, 0.46130E+00_JPRB ,&
&  0.44600E+00_JPRB, 0.43713E+00_JPRB, 0.43707E+00_JPRB, 0.43804E+00_JPRB, 0.43237E+00_JPRB, 0.42688E+00_JPRB, 0.42477E+00_JPRB ,&
&  0.42810E+00_JPRB, 0.43641E+00_JPRB, 0.44787E+00_JPRB, 0.47054E+00_JPRB, 0.48336E+00_JPRB, 0.48741E+00_JPRB, 0.48217E+00_JPRB ,&
&  0.47883E+00_JPRB, 0.48113E+00_JPRB, 0.48152E+00_JPRB, 0.47919E+00_JPRB, 0.48432E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.58121E+00_JPRB, 0.58371E+00_JPRB, 0.57257E+00_JPRB, 0.56947E+00_JPRB, 0.54969E+00_JPRB, 0.53658E+00_JPRB, 0.50952E+00_JPRB ,&
&  0.50187E+00_JPRB, 0.47576E+00_JPRB, 0.44979E+00_JPRB, 0.42238E+00_JPRB, 0.40917E+00_JPRB, 0.40435E+00_JPRB, 0.40649E+00_JPRB ,&
&  0.41192E+00_JPRB, 0.41824E+00_JPRB, 0.42226E+00_JPRB, 0.41653E+00_JPRB, 0.41943E+00_JPRB, 0.42217E+00_JPRB, 0.42069E+00_JPRB ,&
&  0.41760E+00_JPRB, 0.41683E+00_JPRB, 0.41060E+00_JPRB, 0.40060E+00_JPRB, 0.39496E+00_JPRB, 0.39334E+00_JPRB, 0.38944E+00_JPRB ,&
&  0.38215E+00_JPRB, 0.38212E+00_JPRB, 0.39471E+00_JPRB, 0.41942E+00_JPRB, 0.40169E+00_JPRB, 0.39776E+00_JPRB, 0.35450E+00_JPRB ,&
&  0.32211E+00_JPRB, 0.31815E+00_JPRB, 0.34348E+00_JPRB, 0.41002E+00_JPRB, 0.43445E+00_JPRB, 0.46624E+00_JPRB, 0.48552E+00_JPRB ,&
&  0.48477E+00_JPRB, 0.49950E+00_JPRB, 0.54078E+00_JPRB, 0.61855E+00_JPRB, 0.64759E+00_JPRB, 0.68564E+00_JPRB, 0.72283E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.76461E+00_JPRB, 0.76767E+00_JPRB, 0.72608E+00_JPRB, 0.73841E+00_JPRB, 0.78489E+00_JPRB, 0.67947E+00_JPRB ,&
&  0.66850E+00_JPRB, 0.64010E+00_JPRB, 0.64200E+00_JPRB, 0.67497E+00_JPRB, 0.64579E+00_JPRB, 0.65192E+00_JPRB, 0.63533E+00_JPRB ,&
&  0.59202E+00_JPRB, 0.56764E+00_JPRB, 0.55068E+00_JPRB, 0.54652E+00_JPRB, 0.54892E+00_JPRB, 0.56270E+00_JPRB, 0.58494E+00_JPRB ,&
&  0.61426E+00_JPRB, 0.62630E+00_JPRB, 0.59023E+00_JPRB, 0.51019E+00_JPRB, 0.48064E+00_JPRB, 0.46740E+00_JPRB, 0.45735E+00_JPRB ,&
&  0.44638E+00_JPRB, 0.43997E+00_JPRB, 0.44090E+00_JPRB, 0.44057E+00_JPRB, 0.43382E+00_JPRB, 0.42031E+00_JPRB, 0.40872E+00_JPRB ,&
&  0.40392E+00_JPRB, 0.40481E+00_JPRB, 0.40523E+00_JPRB, 0.42546E+00_JPRB, 0.45017E+00_JPRB, 0.47119E+00_JPRB, 0.47419E+00_JPRB ,&
&  0.47599E+00_JPRB, 0.48611E+00_JPRB, 0.49849E+00_JPRB, 0.52657E+00_JPRB, 0.55122E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.59458E+00_JPRB, 0.59902E+00_JPRB, 0.59531E+00_JPRB, 0.58395E+00_JPRB, 0.57611E+00_JPRB, 0.54783E+00_JPRB, 0.48269E+00_JPRB ,&
&  0.48332E+00_JPRB, 0.43770E+00_JPRB, 0.41795E+00_JPRB, 0.40658E+00_JPRB, 0.40563E+00_JPRB, 0.39981E+00_JPRB, 0.39585E+00_JPRB ,&
&  0.39609E+00_JPRB, 0.40636E+00_JPRB, 0.41552E+00_JPRB, 0.41348E+00_JPRB, 0.41716E+00_JPRB, 0.42060E+00_JPRB, 0.41936E+00_JPRB ,&
&  0.42219E+00_JPRB, 0.42520E+00_JPRB, 0.42528E+00_JPRB, 0.41988E+00_JPRB, 0.40811E+00_JPRB, 0.39939E+00_JPRB, 0.39016E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38528E+00_JPRB, 0.39580E+00_JPRB, 0.41928E+00_JPRB, 0.41037E+00_JPRB, 0.39959E+00_JPRB, 0.38582E+00_JPRB ,&
&  0.33979E+00_JPRB, 0.35709E+00_JPRB, 0.42300E+00_JPRB, 0.44854E+00_JPRB, 0.47463E+00_JPRB, 0.50655E+00_JPRB, 0.55080E+00_JPRB ,&
&  0.58967E+00_JPRB, 0.63036E+00_JPRB, 0.67565E+00_JPRB, 0.72709E+00_JPRB, 0.78251E+00_JPRB, 0.83063E+00_JPRB, 0.86040E+00_JPRB ,&
&  0.89527E+00_JPRB, 0.73674E+00_JPRB, 0.71559E+00_JPRB, 0.83187E+00_JPRB, 0.10155E+01_JPRB, 0.83497E+00_JPRB, 0.77247E+00_JPRB ,&
&  0.74330E+00_JPRB, 0.70416E+00_JPRB, 0.68897E+00_JPRB, 0.72299E+00_JPRB, 0.72153E+00_JPRB, 0.73845E+00_JPRB, 0.75124E+00_JPRB ,&
&  0.68828E+00_JPRB, 0.65313E+00_JPRB, 0.62466E+00_JPRB, 0.62400E+00_JPRB, 0.63394E+00_JPRB, 0.64770E+00_JPRB, 0.65238E+00_JPRB ,&
&  0.64581E+00_JPRB, 0.63136E+00_JPRB, 0.59921E+00_JPRB, 0.55278E+00_JPRB, 0.51987E+00_JPRB, 0.50086E+00_JPRB, 0.48613E+00_JPRB ,&
&  0.47306E+00_JPRB, 0.45927E+00_JPRB, 0.44760E+00_JPRB, 0.44088E+00_JPRB, 0.43834E+00_JPRB, 0.43930E+00_JPRB, 0.44847E+00_JPRB ,&
&  0.43430E+00_JPRB, 0.42578E+00_JPRB, 0.41547E+00_JPRB, 0.42763E+00_JPRB, 0.45451E+00_JPRB, 0.47941E+00_JPRB, 0.49516E+00_JPRB ,&
&  0.51760E+00_JPRB, 0.52976E+00_JPRB, 0.52484E+00_JPRB, 0.54058E+00_JPRB, 0.56036E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.60443E+00_JPRB, 0.61315E+00_JPRB, 0.60890E+00_JPRB, 0.60875E+00_JPRB, 0.59902E+00_JPRB, 0.54957E+00_JPRB, 0.50499E+00_JPRB ,&
&  0.56435E+00_JPRB, 0.49386E+00_JPRB, 0.52000E+00_JPRB, 0.44095E+00_JPRB, 0.41596E+00_JPRB, 0.40955E+00_JPRB, 0.39320E+00_JPRB ,&
&  0.37569E+00_JPRB, 0.38500E+00_JPRB, 0.39681E+00_JPRB, 0.39993E+00_JPRB, 0.40568E+00_JPRB, 0.41242E+00_JPRB, 0.41217E+00_JPRB ,&
&  0.41633E+00_JPRB, 0.42506E+00_JPRB, 0.43617E+00_JPRB, 0.42775E+00_JPRB, 0.41531E+00_JPRB, 0.40846E+00_JPRB, 0.40044E+00_JPRB ,&
&  0.38710E+00_JPRB, 0.38151E+00_JPRB, 0.39253E+00_JPRB, 0.41272E+00_JPRB, 0.42336E+00_JPRB, 0.40739E+00_JPRB, 0.39930E+00_JPRB ,&
&  0.40006E+00_JPRB, 0.40383E+00_JPRB, 0.45465E+00_JPRB, 0.48222E+00_JPRB, 0.51912E+00_JPRB, 0.56696E+00_JPRB, 0.61415E+00_JPRB ,&
&  0.66868E+00_JPRB, 0.72551E+00_JPRB, 0.78677E+00_JPRB, 0.85682E+00_JPRB, 0.91977E+00_JPRB, 0.98063E+00_JPRB, 0.10301E+01_JPRB ,&
&  0.10954E+01_JPRB, 0.83001E+00_JPRB, 0.10072E+01_JPRB, 0.10798E+01_JPRB, 0.12380E+01_JPRB, 0.11875E+01_JPRB, 0.10545E+01_JPRB ,&
&  0.10731E+01_JPRB, 0.94326E+00_JPRB, 0.86238E+00_JPRB, 0.79459E+00_JPRB, 0.77793E+00_JPRB, 0.81293E+00_JPRB, 0.84877E+00_JPRB ,&
&  0.90248E+00_JPRB, 0.87893E+00_JPRB, 0.83057E+00_JPRB, 0.82133E+00_JPRB, 0.82057E+00_JPRB, 0.81628E+00_JPRB, 0.79404E+00_JPRB ,&
&  0.75650E+00_JPRB, 0.71241E+00_JPRB, 0.66663E+00_JPRB, 0.60926E+00_JPRB, 0.57995E+00_JPRB, 0.55493E+00_JPRB, 0.52784E+00_JPRB ,&
&  0.51225E+00_JPRB, 0.48304E+00_JPRB, 0.45195E+00_JPRB, 0.43399E+00_JPRB, 0.42043E+00_JPRB, 0.41682E+00_JPRB, 0.42620E+00_JPRB ,&
&  0.42997E+00_JPRB, 0.42841E+00_JPRB, 0.45977E+00_JPRB, 0.48362E+00_JPRB, 0.49763E+00_JPRB, 0.51395E+00_JPRB, 0.56168E+00_JPRB ,&
&  0.53915E+00_JPRB, 0.55572E+00_JPRB, 0.59176E+00_JPRB, 0.59617E+00_JPRB, 0.59898E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.62376E+00_JPRB, 0.63996E+00_JPRB, 0.62846E+00_JPRB, 0.62956E+00_JPRB, 0.60210E+00_JPRB, 0.58647E+00_JPRB, 0.52664E+00_JPRB ,&
&  0.54856E+00_JPRB, 0.53003E+00_JPRB, 0.50880E+00_JPRB, 0.50305E+00_JPRB, 0.51369E+00_JPRB, 0.44962E+00_JPRB, 0.39541E+00_JPRB ,&
&  0.36862E+00_JPRB, 0.36327E+00_JPRB, 0.38009E+00_JPRB, 0.39006E+00_JPRB, 0.39288E+00_JPRB, 0.39783E+00_JPRB, 0.39390E+00_JPRB ,&
&  0.41039E+00_JPRB, 0.41913E+00_JPRB, 0.42994E+00_JPRB, 0.43153E+00_JPRB, 0.42539E+00_JPRB, 0.42230E+00_JPRB, 0.42169E+00_JPRB ,&
&  0.40770E+00_JPRB, 0.40202E+00_JPRB, 0.39352E+00_JPRB, 0.39290E+00_JPRB, 0.43827E+00_JPRB, 0.43201E+00_JPRB, 0.42530E+00_JPRB ,&
&  0.43366E+00_JPRB, 0.45399E+00_JPRB, 0.48448E+00_JPRB, 0.52347E+00_JPRB, 0.57098E+00_JPRB, 0.62096E+00_JPRB, 0.67537E+00_JPRB ,&
&  0.72649E+00_JPRB, 0.79166E+00_JPRB, 0.83916E+00_JPRB, 0.92749E+00_JPRB, 0.99013E+00_JPRB, 0.93398E+00_JPRB, 0.10988E+01_JPRB ,&
&  0.11921E+01_JPRB, 0.13223E+01_JPRB, 0.14455E+01_JPRB, 0.13203E+01_JPRB, 0.14820E+01_JPRB, 0.13782E+01_JPRB, 0.11942E+01_JPRB ,&
&  0.11761E+01_JPRB, 0.10842E+01_JPRB, 0.10552E+01_JPRB, 0.10320E+01_JPRB, 0.10739E+01_JPRB, 0.10369E+01_JPRB, 0.10199E+01_JPRB ,&
&  0.10772E+01_JPRB, 0.12506E+01_JPRB, 0.11871E+01_JPRB, 0.12054E+01_JPRB, 0.11732E+01_JPRB, 0.11324E+01_JPRB, 0.10436E+01_JPRB ,&
&  0.96943E+00_JPRB, 0.88464E+00_JPRB, 0.80483E+00_JPRB, 0.74051E+00_JPRB, 0.66723E+00_JPRB, 0.61302E+00_JPRB, 0.57168E+00_JPRB ,&
&  0.52736E+00_JPRB, 0.47162E+00_JPRB, 0.44745E+00_JPRB, 0.43175E+00_JPRB, 0.42381E+00_JPRB, 0.40615E+00_JPRB, 0.40673E+00_JPRB ,&
&  0.42411E+00_JPRB, 0.46013E+00_JPRB, 0.51632E+00_JPRB, 0.53797E+00_JPRB, 0.57079E+00_JPRB, 0.60917E+00_JPRB, 0.57452E+00_JPRB ,&
&  0.54731E+00_JPRB, 0.61284E+00_JPRB, 0.62254E+00_JPRB, 0.62690E+00_JPRB, 0.62171E+00_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.67711E+00_JPRB, 0.68829E+00_JPRB, 0.67353E+00_JPRB, 0.65612E+00_JPRB, 0.61450E+00_JPRB, 0.58880E+00_JPRB, 0.55155E+00_JPRB ,&
&  0.55370E+00_JPRB, 0.53835E+00_JPRB, 0.51988E+00_JPRB, 0.51312E+00_JPRB, 0.51605E+00_JPRB, 0.50361E+00_JPRB, 0.44302E+00_JPRB ,&
&  0.39633E+00_JPRB, 0.38278E+00_JPRB, 0.37037E+00_JPRB, 0.37889E+00_JPRB, 0.39106E+00_JPRB, 0.39167E+00_JPRB, 0.40087E+00_JPRB ,&
&  0.42000E+00_JPRB, 0.42651E+00_JPRB, 0.42756E+00_JPRB, 0.42691E+00_JPRB, 0.42637E+00_JPRB, 0.43131E+00_JPRB, 0.43171E+00_JPRB ,&
&  0.41586E+00_JPRB, 0.40346E+00_JPRB, 0.39239E+00_JPRB, 0.39597E+00_JPRB, 0.43941E+00_JPRB, 0.45613E+00_JPRB, 0.45684E+00_JPRB ,&
&  0.46803E+00_JPRB, 0.48542E+00_JPRB, 0.50662E+00_JPRB, 0.54531E+00_JPRB, 0.59838E+00_JPRB, 0.66186E+00_JPRB, 0.72834E+00_JPRB ,&
&  0.79299E+00_JPRB, 0.86325E+00_JPRB, 0.91960E+00_JPRB, 0.98947E+00_JPRB, 0.96820E+00_JPRB, 0.10241E+01_JPRB, 0.11836E+01_JPRB ,&
&  0.12893E+01_JPRB, 0.14194E+01_JPRB, 0.14913E+01_JPRB, 0.16440E+01_JPRB, 0.17521E+01_JPRB, 0.16081E+01_JPRB, 0.14410E+01_JPRB ,&
&  0.13575E+01_JPRB, 0.12913E+01_JPRB, 0.16173E+01_JPRB, 0.14430E+01_JPRB, 0.13163E+01_JPRB, 0.12963E+01_JPRB, 0.12573E+01_JPRB ,&
&  0.13065E+01_JPRB, 0.14098E+01_JPRB, 0.14566E+01_JPRB, 0.14985E+01_JPRB, 0.15025E+01_JPRB, 0.15025E+01_JPRB, 0.14111E+01_JPRB ,&
&  0.13461E+01_JPRB, 0.12199E+01_JPRB, 0.96311E+00_JPRB, 0.77710E+00_JPRB, 0.64140E+00_JPRB, 0.63079E+00_JPRB, 0.58726E+00_JPRB ,&
&  0.53058E+00_JPRB, 0.47105E+00_JPRB, 0.45897E+00_JPRB, 0.44293E+00_JPRB, 0.44004E+00_JPRB, 0.43177E+00_JPRB, 0.41783E+00_JPRB ,&
&  0.43051E+00_JPRB, 0.48172E+00_JPRB, 0.51047E+00_JPRB, 0.56228E+00_JPRB, 0.62547E+00_JPRB, 0.69970E+00_JPRB, 0.60229E+00_JPRB ,&
&  0.62377E+00_JPRB, 0.69223E+00_JPRB, 0.70731E+00_JPRB, 0.70821E+00_JPRB, 0.68828E+00_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.78614E+00_JPRB, 0.77440E+00_JPRB, 0.74675E+00_JPRB, 0.69979E+00_JPRB, 0.64350E+00_JPRB, 0.59792E+00_JPRB, 0.55632E+00_JPRB ,&
&  0.55167E+00_JPRB, 0.53651E+00_JPRB, 0.52074E+00_JPRB, 0.51846E+00_JPRB, 0.51839E+00_JPRB, 0.51476E+00_JPRB, 0.51167E+00_JPRB ,&
&  0.43802E+00_JPRB, 0.42002E+00_JPRB, 0.39093E+00_JPRB, 0.39491E+00_JPRB, 0.43663E+00_JPRB, 0.40993E+00_JPRB, 0.41623E+00_JPRB ,&
&  0.42262E+00_JPRB, 0.43417E+00_JPRB, 0.42902E+00_JPRB, 0.42848E+00_JPRB, 0.43671E+00_JPRB, 0.44608E+00_JPRB, 0.44589E+00_JPRB ,&
&  0.42963E+00_JPRB, 0.41227E+00_JPRB, 0.40802E+00_JPRB, 0.41466E+00_JPRB, 0.45661E+00_JPRB, 0.49019E+00_JPRB, 0.51003E+00_JPRB ,&
&  0.52903E+00_JPRB, 0.54531E+00_JPRB, 0.55838E+00_JPRB, 0.58826E+00_JPRB, 0.63599E+00_JPRB, 0.69399E+00_JPRB, 0.77745E+00_JPRB ,&
&  0.86063E+00_JPRB, 0.93696E+00_JPRB, 0.10183E+01_JPRB, 0.10953E+01_JPRB, 0.10937E+01_JPRB, 0.12110E+01_JPRB, 0.13325E+01_JPRB ,&
&  0.14507E+01_JPRB, 0.14019E+01_JPRB, 0.14112E+01_JPRB, 0.17453E+01_JPRB, 0.19837E+01_JPRB, 0.19433E+01_JPRB, 0.17688E+01_JPRB ,&
&  0.16065E+01_JPRB, 0.14914E+01_JPRB, 0.15534E+01_JPRB, 0.15129E+01_JPRB, 0.13943E+01_JPRB, 0.14460E+01_JPRB, 0.14580E+01_JPRB ,&
&  0.15004E+01_JPRB, 0.15894E+01_JPRB, 0.17017E+01_JPRB, 0.17833E+01_JPRB, 0.17372E+01_JPRB, 0.17627E+01_JPRB, 0.17041E+01_JPRB ,&
&  0.16257E+01_JPRB, 0.13676E+01_JPRB, 0.98113E+00_JPRB, 0.77986E+00_JPRB, 0.58770E+00_JPRB, 0.55962E+00_JPRB, 0.63360E+00_JPRB ,&
&  0.61307E+00_JPRB, 0.57785E+00_JPRB, 0.53542E+00_JPRB, 0.50266E+00_JPRB, 0.48530E+00_JPRB, 0.49739E+00_JPRB, 0.48792E+00_JPRB ,&
&  0.47548E+00_JPRB, 0.52592E+00_JPRB, 0.61277E+00_JPRB, 0.71833E+00_JPRB, 0.79189E+00_JPRB, 0.83767E+00_JPRB, 0.77523E+00_JPRB ,&
&  0.83785E+00_JPRB, 0.83227E+00_JPRB, 0.83246E+00_JPRB, 0.84011E+00_JPRB, 0.82168E+00_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.89874E+00_JPRB, 0.86890E+00_JPRB, 0.81671E+00_JPRB, 0.74091E+00_JPRB, 0.65921E+00_JPRB, 0.59497E+00_JPRB, 0.56399E+00_JPRB ,&
&  0.55728E+00_JPRB, 0.54342E+00_JPRB, 0.52547E+00_JPRB, 0.52125E+00_JPRB, 0.52022E+00_JPRB, 0.51527E+00_JPRB, 0.51848E+00_JPRB ,&
&  0.54142E+00_JPRB, 0.50068E+00_JPRB, 0.43899E+00_JPRB, 0.39900E+00_JPRB, 0.41825E+00_JPRB, 0.41605E+00_JPRB, 0.42379E+00_JPRB ,&
&  0.42619E+00_JPRB, 0.44507E+00_JPRB, 0.43071E+00_JPRB, 0.43554E+00_JPRB, 0.46047E+00_JPRB, 0.47776E+00_JPRB, 0.48564E+00_JPRB ,&
&  0.48234E+00_JPRB, 0.47384E+00_JPRB, 0.47793E+00_JPRB, 0.50366E+00_JPRB, 0.55388E+00_JPRB, 0.54630E+00_JPRB, 0.58231E+00_JPRB ,&
&  0.59870E+00_JPRB, 0.62141E+00_JPRB, 0.62751E+00_JPRB, 0.66114E+00_JPRB, 0.72146E+00_JPRB, 0.82105E+00_JPRB, 0.89769E+00_JPRB ,&
&  0.95958E+00_JPRB, 0.10104E+01_JPRB, 0.10382E+01_JPRB, 0.11198E+01_JPRB, 0.12368E+01_JPRB, 0.13873E+01_JPRB, 0.13556E+01_JPRB ,&
&  0.13659E+01_JPRB, 0.12995E+01_JPRB, 0.12475E+01_JPRB, 0.15404E+01_JPRB, 0.18516E+01_JPRB, 0.19358E+01_JPRB, 0.19118E+01_JPRB ,&
&  0.18447E+01_JPRB, 0.17687E+01_JPRB, 0.18289E+01_JPRB, 0.17560E+01_JPRB, 0.17012E+01_JPRB, 0.17347E+01_JPRB, 0.17973E+01_JPRB ,&
&  0.17875E+01_JPRB, 0.18666E+01_JPRB, 0.19924E+01_JPRB, 0.19982E+01_JPRB, 0.19727E+01_JPRB, 0.18953E+01_JPRB, 0.18954E+01_JPRB ,&
&  0.18361E+01_JPRB, 0.11174E+01_JPRB, 0.87263E+00_JPRB, 0.89460E+00_JPRB, 0.67179E+00_JPRB, 0.57932E+00_JPRB, 0.60494E+00_JPRB ,&
&  0.58008E+00_JPRB, 0.57344E+00_JPRB, 0.59194E+00_JPRB, 0.57683E+00_JPRB, 0.56850E+00_JPRB, 0.60543E+00_JPRB, 0.62349E+00_JPRB ,&
&  0.65594E+00_JPRB, 0.68760E+00_JPRB, 0.85119E+00_JPRB, 0.95748E+00_JPRB, 0.99784E+00_JPRB, 0.10054E+01_JPRB, 0.99610E+00_JPRB ,&
&  0.97434E+00_JPRB, 0.95514E+00_JPRB, 0.94142E+00_JPRB, 0.93749E+00_JPRB, 0.93102E+00_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.10239E+01_JPRB, 0.98343E+00_JPRB, 0.91077E+00_JPRB, 0.81513E+00_JPRB, 0.70879E+00_JPRB, 0.63088E+00_JPRB, 0.60772E+00_JPRB ,&
&  0.59265E+00_JPRB, 0.58135E+00_JPRB, 0.56190E+00_JPRB, 0.54969E+00_JPRB, 0.54409E+00_JPRB, 0.53690E+00_JPRB, 0.53820E+00_JPRB ,&
&  0.54104E+00_JPRB, 0.53797E+00_JPRB, 0.45471E+00_JPRB, 0.39717E+00_JPRB, 0.36680E+00_JPRB, 0.38449E+00_JPRB, 0.40142E+00_JPRB ,&
&  0.42300E+00_JPRB, 0.44471E+00_JPRB, 0.44993E+00_JPRB, 0.49208E+00_JPRB, 0.53354E+00_JPRB, 0.55313E+00_JPRB, 0.58480E+00_JPRB ,&
&  0.59111E+00_JPRB, 0.58525E+00_JPRB, 0.58195E+00_JPRB, 0.63079E+00_JPRB, 0.64556E+00_JPRB, 0.61552E+00_JPRB, 0.67569E+00_JPRB ,&
&  0.70613E+00_JPRB, 0.73541E+00_JPRB, 0.75954E+00_JPRB, 0.81249E+00_JPRB, 0.87534E+00_JPRB, 0.90557E+00_JPRB, 0.89512E+00_JPRB ,&
&  0.91931E+00_JPRB, 0.96934E+00_JPRB, 0.10420E+01_JPRB, 0.11047E+01_JPRB, 0.11876E+01_JPRB, 0.12729E+01_JPRB, 0.12365E+01_JPRB ,&
&  0.12459E+01_JPRB, 0.10824E+01_JPRB, 0.94607E+00_JPRB, 0.12039E+01_JPRB, 0.17080E+01_JPRB, 0.19370E+01_JPRB, 0.18453E+01_JPRB ,&
&  0.19762E+01_JPRB, 0.20165E+01_JPRB, 0.21060E+01_JPRB, 0.20704E+01_JPRB, 0.19952E+01_JPRB, 0.20179E+01_JPRB, 0.21299E+01_JPRB ,&
&  0.21632E+01_JPRB, 0.22683E+01_JPRB, 0.23305E+01_JPRB, 0.23081E+01_JPRB, 0.22581E+01_JPRB, 0.22661E+01_JPRB, 0.22782E+01_JPRB ,&
&  0.20710E+01_JPRB, 0.17247E+01_JPRB, 0.12017E+01_JPRB, 0.11362E+01_JPRB, 0.79740E+00_JPRB, 0.64962E+00_JPRB, 0.64270E+00_JPRB ,&
&  0.58635E+00_JPRB, 0.60125E+00_JPRB, 0.63552E+00_JPRB, 0.63722E+00_JPRB, 0.71693E+00_JPRB, 0.78367E+00_JPRB, 0.87812E+00_JPRB ,&
&  0.95981E+00_JPRB, 0.96391E+00_JPRB, 0.11438E+01_JPRB, 0.11811E+01_JPRB, 0.11999E+01_JPRB, 0.12168E+01_JPRB, 0.12149E+01_JPRB ,&
&  0.11809E+01_JPRB, 0.11393E+01_JPRB, 0.11046E+01_JPRB, 0.10710E+01_JPRB, 0.10480E+01_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.10669E+01_JPRB, 0.10081E+01_JPRB, 0.91371E+00_JPRB, 0.82111E+00_JPRB, 0.72263E+00_JPRB, 0.64234E+00_JPRB, 0.62889E+00_JPRB ,&
&  0.65223E+00_JPRB, 0.65104E+00_JPRB, 0.63956E+00_JPRB, 0.61273E+00_JPRB, 0.59744E+00_JPRB, 0.58721E+00_JPRB, 0.57490E+00_JPRB ,&
&  0.56234E+00_JPRB, 0.50863E+00_JPRB, 0.41543E+00_JPRB, 0.36365E+00_JPRB, 0.34845E+00_JPRB, 0.34866E+00_JPRB, 0.37735E+00_JPRB ,&
&  0.41168E+00_JPRB, 0.46709E+00_JPRB, 0.52208E+00_JPRB, 0.58823E+00_JPRB, 0.67242E+00_JPRB, 0.75787E+00_JPRB, 0.78920E+00_JPRB ,&
&  0.81210E+00_JPRB, 0.70781E+00_JPRB, 0.74225E+00_JPRB, 0.73922E+00_JPRB, 0.77756E+00_JPRB, 0.78124E+00_JPRB, 0.79191E+00_JPRB ,&
&  0.78486E+00_JPRB, 0.81264E+00_JPRB, 0.84241E+00_JPRB, 0.87027E+00_JPRB, 0.88518E+00_JPRB, 0.89319E+00_JPRB, 0.90212E+00_JPRB ,&
&  0.96026E+00_JPRB, 0.10273E+01_JPRB, 0.10707E+01_JPRB, 0.11023E+01_JPRB, 0.11685E+01_JPRB, 0.12562E+01_JPRB, 0.12670E+01_JPRB ,&
&  0.11835E+01_JPRB, 0.11195E+01_JPRB, 0.12672E+01_JPRB, 0.14386E+01_JPRB, 0.16410E+01_JPRB, 0.19153E+01_JPRB, 0.20447E+01_JPRB ,&
&  0.21267E+01_JPRB, 0.20692E+01_JPRB, 0.21072E+01_JPRB, 0.19866E+01_JPRB, 0.22402E+01_JPRB, 0.22921E+01_JPRB, 0.23961E+01_JPRB ,&
&  0.24886E+01_JPRB, 0.26564E+01_JPRB, 0.25849E+01_JPRB, 0.25075E+01_JPRB, 0.25581E+01_JPRB, 0.27083E+01_JPRB, 0.24313E+01_JPRB ,&
&  0.20379E+01_JPRB, 0.21804E+01_JPRB, 0.19158E+01_JPRB, 0.15702E+01_JPRB, 0.11880E+01_JPRB, 0.10106E+01_JPRB, 0.85897E+00_JPRB ,&
&  0.75449E+00_JPRB, 0.70654E+00_JPRB, 0.71586E+00_JPRB, 0.82701E+00_JPRB, 0.10072E+01_JPRB, 0.11637E+01_JPRB, 0.12484E+01_JPRB ,&
&  0.12826E+01_JPRB, 0.15508E+01_JPRB, 0.15354E+01_JPRB, 0.14758E+01_JPRB, 0.14720E+01_JPRB, 0.14702E+01_JPRB, 0.14312E+01_JPRB ,&
&  0.13400E+01_JPRB, 0.12707E+01_JPRB, 0.12152E+01_JPRB, 0.11642E+01_JPRB, 0.11110E+01_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.10914E+01_JPRB, 0.10358E+01_JPRB, 0.93945E+00_JPRB, 0.83899E+00_JPRB, 0.74580E+00_JPRB, 0.67163E+00_JPRB, 0.65329E+00_JPRB ,&
&  0.66610E+00_JPRB, 0.68043E+00_JPRB, 0.69173E+00_JPRB, 0.69567E+00_JPRB, 0.69097E+00_JPRB, 0.66820E+00_JPRB, 0.63960E+00_JPRB ,&
&  0.61691E+00_JPRB, 0.49964E+00_JPRB, 0.39997E+00_JPRB, 0.36933E+00_JPRB, 0.35038E+00_JPRB, 0.34289E+00_JPRB, 0.36142E+00_JPRB ,&
&  0.41410E+00_JPRB, 0.52765E+00_JPRB, 0.65311E+00_JPRB, 0.80147E+00_JPRB, 0.93061E+00_JPRB, 0.10184E+01_JPRB, 0.10010E+01_JPRB ,&
&  0.93401E+00_JPRB, 0.93090E+00_JPRB, 0.93672E+00_JPRB, 0.90585E+00_JPRB, 0.90017E+00_JPRB, 0.87235E+00_JPRB, 0.85676E+00_JPRB ,&
&  0.85107E+00_JPRB, 0.85321E+00_JPRB, 0.89874E+00_JPRB, 0.93282E+00_JPRB, 0.95977E+00_JPRB, 0.98401E+00_JPRB, 0.10093E+01_JPRB ,&
&  0.10321E+01_JPRB, 0.10605E+01_JPRB, 0.10771E+01_JPRB, 0.11544E+01_JPRB, 0.99978E+00_JPRB, 0.10378E+01_JPRB, 0.11496E+01_JPRB ,&
&  0.13882E+01_JPRB, 0.14025E+01_JPRB, 0.13550E+01_JPRB, 0.14988E+01_JPRB, 0.14307E+01_JPRB, 0.15626E+01_JPRB, 0.19043E+01_JPRB ,&
&  0.20342E+01_JPRB, 0.20618E+01_JPRB, 0.21891E+01_JPRB, 0.19164E+01_JPRB, 0.19064E+01_JPRB, 0.27117E+01_JPRB, 0.29133E+01_JPRB ,&
&  0.29610E+01_JPRB, 0.29795E+01_JPRB, 0.28561E+01_JPRB, 0.28649E+01_JPRB, 0.24928E+01_JPRB, 0.21794E+01_JPRB, 0.16733E+01_JPRB ,&
&  0.17779E+01_JPRB, 0.17155E+01_JPRB, 0.18977E+01_JPRB, 0.17293E+01_JPRB, 0.15966E+01_JPRB, 0.14061E+01_JPRB, 0.11466E+01_JPRB ,&
&  0.98393E+00_JPRB, 0.89776E+00_JPRB, 0.90137E+00_JPRB, 0.11491E+01_JPRB, 0.14682E+01_JPRB, 0.15369E+01_JPRB, 0.18233E+01_JPRB ,&
&  0.19514E+01_JPRB, 0.19935E+01_JPRB, 0.17589E+01_JPRB, 0.17168E+01_JPRB, 0.16304E+01_JPRB, 0.15490E+01_JPRB, 0.14521E+01_JPRB ,&
&  0.13714E+01_JPRB, 0.13044E+01_JPRB, 0.12419E+01_JPRB, 0.11817E+01_JPRB, 0.11275E+01_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.11072E+01_JPRB, 0.10610E+01_JPRB, 0.96690E+00_JPRB, 0.86573E+00_JPRB, 0.77594E+00_JPRB, 0.71554E+00_JPRB, 0.70150E+00_JPRB ,&
&  0.71562E+00_JPRB, 0.73065E+00_JPRB, 0.73737E+00_JPRB, 0.73322E+00_JPRB, 0.72302E+00_JPRB, 0.71839E+00_JPRB, 0.70658E+00_JPRB ,&
&  0.67533E+00_JPRB, 0.55953E+00_JPRB, 0.42673E+00_JPRB, 0.37712E+00_JPRB, 0.37856E+00_JPRB, 0.36429E+00_JPRB, 0.37156E+00_JPRB ,&
&  0.47035E+00_JPRB, 0.64698E+00_JPRB, 0.87733E+00_JPRB, 0.10753E+01_JPRB, 0.12219E+01_JPRB, 0.12332E+01_JPRB, 0.11602E+01_JPRB ,&
&  0.11557E+01_JPRB, 0.11001E+01_JPRB, 0.98634E+00_JPRB, 0.89945E+00_JPRB, 0.86220E+00_JPRB, 0.84883E+00_JPRB, 0.82767E+00_JPRB ,&
&  0.81236E+00_JPRB, 0.82225E+00_JPRB, 0.85219E+00_JPRB, 0.87451E+00_JPRB, 0.90892E+00_JPRB, 0.92184E+00_JPRB, 0.92545E+00_JPRB ,&
&  0.95371E+00_JPRB, 0.10020E+01_JPRB, 0.10639E+01_JPRB, 0.11659E+01_JPRB, 0.10805E+01_JPRB, 0.11079E+01_JPRB, 0.12743E+01_JPRB ,&
&  0.14171E+01_JPRB, 0.14371E+01_JPRB, 0.14304E+01_JPRB, 0.14640E+01_JPRB, 0.15721E+01_JPRB, 0.15873E+01_JPRB, 0.18567E+01_JPRB ,&
&  0.17993E+01_JPRB, 0.17172E+01_JPRB, 0.17236E+01_JPRB, 0.16796E+01_JPRB, 0.21410E+01_JPRB, 0.29914E+01_JPRB, 0.34161E+01_JPRB ,&
&  0.33792E+01_JPRB, 0.33068E+01_JPRB, 0.32123E+01_JPRB, 0.28259E+01_JPRB, 0.22246E+01_JPRB, 0.16757E+01_JPRB, 0.19800E+01_JPRB ,&
&  0.21446E+01_JPRB, 0.22014E+01_JPRB, 0.20895E+01_JPRB, 0.17627E+01_JPRB, 0.15317E+01_JPRB, 0.16961E+01_JPRB, 0.15271E+01_JPRB ,&
&  0.12650E+01_JPRB, 0.11551E+01_JPRB, 0.13767E+01_JPRB, 0.16895E+01_JPRB, 0.18701E+01_JPRB, 0.20131E+01_JPRB, 0.22489E+01_JPRB ,&
&  0.21556E+01_JPRB, 0.21006E+01_JPRB, 0.19524E+01_JPRB, 0.17687E+01_JPRB, 0.16181E+01_JPRB, 0.15050E+01_JPRB, 0.14131E+01_JPRB ,&
&  0.13602E+01_JPRB, 0.13181E+01_JPRB, 0.12605E+01_JPRB, 0.12048E+01_JPRB, 0.11464E+01_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.11277E+01_JPRB, 0.10830E+01_JPRB, 0.98533E+00_JPRB, 0.87809E+00_JPRB, 0.80733E+00_JPRB, 0.76477E+00_JPRB, 0.74689E+00_JPRB ,&
&  0.74971E+00_JPRB, 0.75229E+00_JPRB, 0.75406E+00_JPRB, 0.74995E+00_JPRB, 0.74215E+00_JPRB, 0.73229E+00_JPRB, 0.73341E+00_JPRB ,&
&  0.71190E+00_JPRB, 0.68016E+00_JPRB, 0.50398E+00_JPRB, 0.43167E+00_JPRB, 0.41897E+00_JPRB, 0.41982E+00_JPRB, 0.42341E+00_JPRB ,&
&  0.58042E+00_JPRB, 0.87993E+00_JPRB, 0.11424E+01_JPRB, 0.13291E+01_JPRB, 0.14344E+01_JPRB, 0.12538E+01_JPRB, 0.11931E+01_JPRB ,&
&  0.12196E+01_JPRB, 0.11717E+01_JPRB, 0.10564E+01_JPRB, 0.95540E+00_JPRB, 0.87326E+00_JPRB, 0.83786E+00_JPRB, 0.82455E+00_JPRB ,&
&  0.81460E+00_JPRB, 0.82057E+00_JPRB, 0.84004E+00_JPRB, 0.87762E+00_JPRB, 0.89858E+00_JPRB, 0.90947E+00_JPRB, 0.92439E+00_JPRB ,&
&  0.94278E+00_JPRB, 0.97690E+00_JPRB, 0.10276E+01_JPRB, 0.11767E+01_JPRB, 0.11827E+01_JPRB, 0.12846E+01_JPRB, 0.15691E+01_JPRB ,&
&  0.15845E+01_JPRB, 0.16316E+01_JPRB, 0.15647E+01_JPRB, 0.15728E+01_JPRB, 0.16021E+01_JPRB, 0.16358E+01_JPRB, 0.18706E+01_JPRB ,&
&  0.18431E+01_JPRB, 0.18635E+01_JPRB, 0.20385E+01_JPRB, 0.21759E+01_JPRB, 0.23415E+01_JPRB, 0.27916E+01_JPRB, 0.38925E+01_JPRB ,&
&  0.33969E+01_JPRB, 0.33678E+01_JPRB, 0.33004E+01_JPRB, 0.29716E+01_JPRB, 0.15746E+01_JPRB, 0.11651E+01_JPRB, 0.17291E+01_JPRB ,&
&  0.18306E+01_JPRB, 0.11612E+01_JPRB, 0.88239E+00_JPRB, 0.12052E+01_JPRB, 0.96716E+00_JPRB, 0.11469E+01_JPRB, 0.15476E+01_JPRB ,&
&  0.15786E+01_JPRB, 0.14449E+01_JPRB, 0.17204E+01_JPRB, 0.18589E+01_JPRB, 0.19220E+01_JPRB, 0.23222E+01_JPRB, 0.24069E+01_JPRB ,&
&  0.23218E+01_JPRB, 0.20088E+01_JPRB, 0.19728E+01_JPRB, 0.17622E+01_JPRB, 0.16128E+01_JPRB, 0.15278E+01_JPRB, 0.14626E+01_JPRB ,&
&  0.13830E+01_JPRB, 0.12999E+01_JPRB, 0.12392E+01_JPRB, 0.12024E+01_JPRB, 0.11617E+01_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.11268E+01_JPRB, 0.10801E+01_JPRB, 0.97915E+00_JPRB, 0.87407E+00_JPRB, 0.81949E+00_JPRB, 0.78254E+00_JPRB, 0.76119E+00_JPRB ,&
&  0.76072E+00_JPRB, 0.76618E+00_JPRB, 0.76863E+00_JPRB, 0.76855E+00_JPRB, 0.76409E+00_JPRB, 0.75898E+00_JPRB, 0.75522E+00_JPRB ,&
&  0.73534E+00_JPRB, 0.68851E+00_JPRB, 0.59506E+00_JPRB, 0.52483E+00_JPRB, 0.50477E+00_JPRB, 0.49298E+00_JPRB, 0.53393E+00_JPRB ,&
&  0.70584E+00_JPRB, 0.10917E+01_JPRB, 0.13954E+01_JPRB, 0.14998E+01_JPRB, 0.13787E+01_JPRB, 0.12364E+01_JPRB, 0.12975E+01_JPRB ,&
&  0.13097E+01_JPRB, 0.12335E+01_JPRB, 0.10901E+01_JPRB, 0.99682E+00_JPRB, 0.92290E+00_JPRB, 0.87882E+00_JPRB, 0.86300E+00_JPRB ,&
&  0.84270E+00_JPRB, 0.84242E+00_JPRB, 0.87233E+00_JPRB, 0.89093E+00_JPRB, 0.89919E+00_JPRB, 0.90573E+00_JPRB, 0.91326E+00_JPRB ,&
&  0.94031E+00_JPRB, 0.99071E+00_JPRB, 0.10699E+01_JPRB, 0.12070E+01_JPRB, 0.14787E+01_JPRB, 0.16467E+01_JPRB, 0.15880E+01_JPRB ,&
&  0.14905E+01_JPRB, 0.15260E+01_JPRB, 0.17075E+01_JPRB, 0.16047E+01_JPRB, 0.15641E+01_JPRB, 0.16120E+01_JPRB, 0.18025E+01_JPRB ,&
&  0.21191E+01_JPRB, 0.23096E+01_JPRB, 0.21885E+01_JPRB, 0.23223E+01_JPRB, 0.25346E+01_JPRB, 0.24102E+01_JPRB, 0.32573E+01_JPRB ,&
&  0.35402E+01_JPRB, 0.30580E+01_JPRB, 0.23359E+01_JPRB, 0.17073E+01_JPRB, 0.16435E+01_JPRB, 0.12557E+01_JPRB, 0.78979E+00_JPRB ,&
&  0.71489E+00_JPRB, 0.67689E+00_JPRB, 0.62816E+00_JPRB, 0.70112E+00_JPRB, 0.71066E+00_JPRB, 0.94690E+00_JPRB, 0.14279E+01_JPRB ,&
&  0.18002E+01_JPRB, 0.18357E+01_JPRB, 0.19380E+01_JPRB, 0.19368E+01_JPRB, 0.18996E+01_JPRB, 0.22820E+01_JPRB, 0.24644E+01_JPRB ,&
&  0.20821E+01_JPRB, 0.18776E+01_JPRB, 0.18255E+01_JPRB, 0.16446E+01_JPRB, 0.15695E+01_JPRB, 0.15141E+01_JPRB, 0.14276E+01_JPRB ,&
&  0.13184E+01_JPRB, 0.12162E+01_JPRB, 0.11705E+01_JPRB, 0.11588E+01_JPRB, 0.11544E+01_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.11200E+01_JPRB, 0.10685E+01_JPRB, 0.98266E+00_JPRB, 0.89438E+00_JPRB, 0.85503E+00_JPRB, 0.82686E+00_JPRB, 0.80451E+00_JPRB ,&
&  0.80007E+00_JPRB, 0.80046E+00_JPRB, 0.80592E+00_JPRB, 0.81033E+00_JPRB, 0.80768E+00_JPRB, 0.79950E+00_JPRB, 0.78816E+00_JPRB ,&
&  0.76744E+00_JPRB, 0.71262E+00_JPRB, 0.66538E+00_JPRB, 0.60977E+00_JPRB, 0.59737E+00_JPRB, 0.57920E+00_JPRB, 0.64929E+00_JPRB ,&
&  0.87585E+00_JPRB, 0.13183E+01_JPRB, 0.16240E+01_JPRB, 0.16412E+01_JPRB, 0.15275E+01_JPRB, 0.14426E+01_JPRB, 0.14234E+01_JPRB ,&
&  0.13483E+01_JPRB, 0.12652E+01_JPRB, 0.11336E+01_JPRB, 0.10377E+01_JPRB, 0.96590E+00_JPRB, 0.91900E+00_JPRB, 0.89138E+00_JPRB ,&
&  0.87147E+00_JPRB, 0.89432E+00_JPRB, 0.89771E+00_JPRB, 0.90457E+00_JPRB, 0.90497E+00_JPRB, 0.90712E+00_JPRB, 0.92300E+00_JPRB ,&
&  0.97082E+00_JPRB, 0.10567E+01_JPRB, 0.12045E+01_JPRB, 0.14424E+01_JPRB, 0.15136E+01_JPRB, 0.13828E+01_JPRB, 0.14853E+01_JPRB ,&
&  0.15412E+01_JPRB, 0.16563E+01_JPRB, 0.16155E+01_JPRB, 0.16920E+01_JPRB, 0.16057E+01_JPRB, 0.15652E+01_JPRB, 0.17821E+01_JPRB ,&
&  0.21552E+01_JPRB, 0.26062E+01_JPRB, 0.22446E+01_JPRB, 0.21730E+01_JPRB, 0.27908E+01_JPRB, 0.26638E+01_JPRB, 0.29507E+01_JPRB ,&
&  0.33177E+01_JPRB, 0.28170E+01_JPRB, 0.22796E+01_JPRB, 0.18125E+01_JPRB, 0.21874E+01_JPRB, 0.21199E+01_JPRB, 0.11169E+01_JPRB ,&
&  0.82412E+00_JPRB, 0.70580E+00_JPRB, 0.64417E+00_JPRB, 0.62925E+00_JPRB, 0.64470E+00_JPRB, 0.90882E+00_JPRB, 0.15913E+01_JPRB ,&
&  0.17777E+01_JPRB, 0.20652E+01_JPRB, 0.21374E+01_JPRB, 0.20926E+01_JPRB, 0.20310E+01_JPRB, 0.20464E+01_JPRB, 0.20664E+01_JPRB ,&
&  0.21323E+01_JPRB, 0.19632E+01_JPRB, 0.17352E+01_JPRB, 0.15701E+01_JPRB, 0.14620E+01_JPRB, 0.13369E+01_JPRB, 0.12213E+01_JPRB ,&
&  0.11370E+01_JPRB, 0.11067E+01_JPRB, 0.11016E+01_JPRB, 0.11199E+01_JPRB, 0.11464E+01_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.11141E+01_JPRB, 0.10901E+01_JPRB, 0.10177E+01_JPRB, 0.93823E+00_JPRB, 0.89863E+00_JPRB, 0.87100E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.84759E+00_JPRB, 0.84673E+00_JPRB, 0.84155E+00_JPRB, 0.83248E+00_JPRB, 0.83021E+00_JPRB, 0.81605E+00_JPRB, 0.80107E+00_JPRB ,&
&  0.77922E+00_JPRB, 0.73067E+00_JPRB, 0.68444E+00_JPRB, 0.68057E+00_JPRB, 0.66807E+00_JPRB, 0.66857E+00_JPRB, 0.78083E+00_JPRB ,&
&  0.11173E+01_JPRB, 0.16260E+01_JPRB, 0.18212E+01_JPRB, 0.18099E+01_JPRB, 0.16907E+01_JPRB, 0.15766E+01_JPRB, 0.13949E+01_JPRB ,&
&  0.12228E+01_JPRB, 0.11885E+01_JPRB, 0.10827E+01_JPRB, 0.10376E+01_JPRB, 0.10025E+01_JPRB, 0.96724E+00_JPRB, 0.94205E+00_JPRB ,&
&  0.93342E+00_JPRB, 0.91548E+00_JPRB, 0.90967E+00_JPRB, 0.90877E+00_JPRB, 0.91021E+00_JPRB, 0.91995E+00_JPRB, 0.97272E+00_JPRB ,&
&  0.10681E+01_JPRB, 0.11833E+01_JPRB, 0.13821E+01_JPRB, 0.15665E+01_JPRB, 0.13091E+01_JPRB, 0.14320E+01_JPRB, 0.14655E+01_JPRB ,&
&  0.14762E+01_JPRB, 0.15065E+01_JPRB, 0.14348E+01_JPRB, 0.14824E+01_JPRB, 0.15676E+01_JPRB, 0.15707E+01_JPRB, 0.17248E+01_JPRB ,&
&  0.23052E+01_JPRB, 0.21640E+01_JPRB, 0.19937E+01_JPRB, 0.20783E+01_JPRB, 0.25711E+01_JPRB, 0.38463E+01_JPRB, 0.27109E+01_JPRB ,&
&  0.24631E+01_JPRB, 0.27682E+01_JPRB, 0.23099E+01_JPRB, 0.19650E+01_JPRB, 0.22624E+01_JPRB, 0.26800E+01_JPRB, 0.24282E+01_JPRB ,&
&  0.11030E+01_JPRB, 0.77652E+00_JPRB, 0.76731E+00_JPRB, 0.76881E+00_JPRB, 0.73400E+00_JPRB, 0.92454E+00_JPRB, 0.22410E+01_JPRB ,&
&  0.19675E+01_JPRB, 0.22353E+01_JPRB, 0.22258E+01_JPRB, 0.20769E+01_JPRB, 0.20638E+01_JPRB, 0.20709E+01_JPRB, 0.20607E+01_JPRB ,&
&  0.18232E+01_JPRB, 0.16295E+01_JPRB, 0.14694E+01_JPRB, 0.13050E+01_JPRB, 0.11782E+01_JPRB, 0.10926E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.10343E+01_JPRB, 0.10365E+01_JPRB, 0.10561E+01_JPRB, 0.10858E+01_JPRB, 0.11123E+01_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.11104E+01_JPRB, 0.11084E+01_JPRB, 0.10415E+01_JPRB, 0.96855E+00_JPRB, 0.92595E+00_JPRB, 0.90234E+00_JPRB, 0.87724E+00_JPRB ,&
&  0.91985E+00_JPRB, 0.90246E+00_JPRB, 0.86188E+00_JPRB, 0.83040E+00_JPRB, 0.82472E+00_JPRB, 0.80894E+00_JPRB, 0.79627E+00_JPRB ,&
&  0.77349E+00_JPRB, 0.74214E+00_JPRB, 0.70582E+00_JPRB, 0.68835E+00_JPRB, 0.71345E+00_JPRB, 0.68358E+00_JPRB, 0.80262E+00_JPRB ,&
&  0.13309E+01_JPRB, 0.20072E+01_JPRB, 0.19139E+01_JPRB, 0.18530E+01_JPRB, 0.17186E+01_JPRB, 0.15998E+01_JPRB, 0.12973E+01_JPRB ,&
&  0.11606E+01_JPRB, 0.11458E+01_JPRB, 0.10673E+01_JPRB, 0.10502E+01_JPRB, 0.10318E+01_JPRB, 0.10107E+01_JPRB, 0.98887E+00_JPRB ,&
&  0.94316E+00_JPRB, 0.92177E+00_JPRB, 0.93225E+00_JPRB, 0.93991E+00_JPRB, 0.94193E+00_JPRB, 0.98640E+00_JPRB, 0.10722E+01_JPRB ,&
&  0.11674E+01_JPRB, 0.13047E+01_JPRB, 0.15112E+01_JPRB, 0.15013E+01_JPRB, 0.14238E+01_JPRB, 0.14129E+01_JPRB, 0.14459E+01_JPRB ,&
&  0.14244E+01_JPRB, 0.14415E+01_JPRB, 0.14408E+01_JPRB, 0.14137E+01_JPRB, 0.14877E+01_JPRB, 0.16024E+01_JPRB, 0.17342E+01_JPRB ,&
&  0.20210E+01_JPRB, 0.19018E+01_JPRB, 0.17810E+01_JPRB, 0.19936E+01_JPRB, 0.24887E+01_JPRB, 0.34598E+01_JPRB, 0.35338E+01_JPRB ,&
&  0.30980E+01_JPRB, 0.22761E+01_JPRB, 0.19900E+01_JPRB, 0.22138E+01_JPRB, 0.24263E+01_JPRB, 0.26069E+01_JPRB, 0.33701E+01_JPRB ,&
&  0.32882E+01_JPRB, 0.19026E+01_JPRB, 0.15456E+01_JPRB, 0.19003E+01_JPRB, 0.13566E+01_JPRB, 0.10414E+01_JPRB, 0.15405E+01_JPRB ,&
&  0.18611E+01_JPRB, 0.21152E+01_JPRB, 0.21672E+01_JPRB, 0.20748E+01_JPRB, 0.20905E+01_JPRB, 0.19750E+01_JPRB, 0.17233E+01_JPRB ,&
&  0.14973E+01_JPRB, 0.13124E+01_JPRB, 0.11808E+01_JPRB, 0.11049E+01_JPRB, 0.10748E+01_JPRB, 0.10551E+01_JPRB, 0.10305E+01_JPRB ,&
&  0.10189E+01_JPRB, 0.10194E+01_JPRB, 0.10419E+01_JPRB, 0.10824E+01_JPRB, 0.11086E+01_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.11080E+01_JPRB, 0.10934E+01_JPRB, 0.10342E+01_JPRB, 0.97581E+00_JPRB, 0.93750E+00_JPRB, 0.91638E+00_JPRB, 0.90464E+00_JPRB ,&
&  0.10783E+01_JPRB, 0.95357E+00_JPRB, 0.86536E+00_JPRB, 0.82591E+00_JPRB, 0.80678E+00_JPRB, 0.79597E+00_JPRB, 0.78234E+00_JPRB ,&
&  0.76874E+00_JPRB, 0.74868E+00_JPRB, 0.73396E+00_JPRB, 0.73580E+00_JPRB, 0.78036E+00_JPRB, 0.96931E+00_JPRB, 0.94764E+00_JPRB ,&
&  0.14172E+01_JPRB, 0.22903E+01_JPRB, 0.19983E+01_JPRB, 0.18891E+01_JPRB, 0.17373E+01_JPRB, 0.16589E+01_JPRB, 0.13032E+01_JPRB ,&
&  0.11730E+01_JPRB, 0.11363E+01_JPRB, 0.10928E+01_JPRB, 0.10692E+01_JPRB, 0.10663E+01_JPRB, 0.10524E+01_JPRB, 0.97950E+00_JPRB ,&
&  0.93902E+00_JPRB, 0.93785E+00_JPRB, 0.96308E+00_JPRB, 0.98094E+00_JPRB, 0.10133E+01_JPRB, 0.10905E+01_JPRB, 0.11550E+01_JPRB ,&
&  0.12581E+01_JPRB, 0.14777E+01_JPRB, 0.16131E+01_JPRB, 0.14559E+01_JPRB, 0.14216E+01_JPRB, 0.14387E+01_JPRB, 0.14598E+01_JPRB ,&
&  0.13706E+01_JPRB, 0.12765E+01_JPRB, 0.13634E+01_JPRB, 0.14439E+01_JPRB, 0.15057E+01_JPRB, 0.15487E+01_JPRB, 0.16080E+01_JPRB ,&
&  0.18563E+01_JPRB, 0.19767E+01_JPRB, 0.22106E+01_JPRB, 0.22310E+01_JPRB, 0.25289E+01_JPRB, 0.32248E+01_JPRB, 0.35100E+01_JPRB ,&
&  0.36315E+01_JPRB, 0.29143E+01_JPRB, 0.26878E+01_JPRB, 0.26885E+01_JPRB, 0.26090E+01_JPRB, 0.25074E+01_JPRB, 0.27522E+01_JPRB ,&
&  0.30275E+01_JPRB, 0.34811E+01_JPRB, 0.30974E+01_JPRB, 0.24390E+01_JPRB, 0.18576E+01_JPRB, 0.13565E+01_JPRB, 0.15233E+01_JPRB ,&
&  0.18828E+01_JPRB, 0.19618E+01_JPRB, 0.19603E+01_JPRB, 0.24852E+01_JPRB, 0.19902E+01_JPRB, 0.16287E+01_JPRB, 0.13282E+01_JPRB ,&
&  0.11607E+01_JPRB, 0.11166E+01_JPRB, 0.11092E+01_JPRB, 0.10854E+01_JPRB, 0.10647E+01_JPRB, 0.10454E+01_JPRB, 0.10312E+01_JPRB ,&
&  0.10282E+01_JPRB, 0.10400E+01_JPRB, 0.10688E+01_JPRB, 0.10996E+01_JPRB, 0.11173E+01_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.10701E+01_JPRB, 0.10362E+01_JPRB, 0.99535E+00_JPRB, 0.95723E+00_JPRB, 0.92606E+00_JPRB, 0.91735E+00_JPRB, 0.95031E+00_JPRB ,&
&  0.12123E+01_JPRB, 0.94357E+00_JPRB, 0.84491E+00_JPRB, 0.81450E+00_JPRB, 0.80122E+00_JPRB, 0.78367E+00_JPRB, 0.77946E+00_JPRB ,&
&  0.77958E+00_JPRB, 0.77922E+00_JPRB, 0.79539E+00_JPRB, 0.84319E+00_JPRB, 0.97559E+00_JPRB, 0.13042E+01_JPRB, 0.15223E+01_JPRB ,&
&  0.16044E+01_JPRB, 0.26468E+01_JPRB, 0.20877E+01_JPRB, 0.19228E+01_JPRB, 0.17414E+01_JPRB, 0.16472E+01_JPRB, 0.14091E+01_JPRB ,&
&  0.12289E+01_JPRB, 0.11570E+01_JPRB, 0.11244E+01_JPRB, 0.10858E+01_JPRB, 0.10585E+01_JPRB, 0.10050E+01_JPRB, 0.97710E+00_JPRB ,&
&  0.97436E+00_JPRB, 0.99651E+00_JPRB, 0.10222E+01_JPRB, 0.10527E+01_JPRB, 0.10975E+01_JPRB, 0.11867E+01_JPRB, 0.12935E+01_JPRB ,&
&  0.14756E+01_JPRB, 0.16743E+01_JPRB, 0.15758E+01_JPRB, 0.14652E+01_JPRB, 0.14131E+01_JPRB, 0.14473E+01_JPRB, 0.14679E+01_JPRB ,&
&  0.13996E+01_JPRB, 0.13197E+01_JPRB, 0.13851E+01_JPRB, 0.14104E+01_JPRB, 0.14148E+01_JPRB, 0.15225E+01_JPRB, 0.15601E+01_JPRB ,&
&  0.18736E+01_JPRB, 0.20622E+01_JPRB, 0.28266E+01_JPRB, 0.27482E+01_JPRB, 0.24881E+01_JPRB, 0.30485E+01_JPRB, 0.30498E+01_JPRB ,&
&  0.26075E+01_JPRB, 0.25111E+01_JPRB, 0.25698E+01_JPRB, 0.27938E+01_JPRB, 0.27878E+01_JPRB, 0.27121E+01_JPRB, 0.25441E+01_JPRB ,&
&  0.22809E+01_JPRB, 0.23633E+01_JPRB, 0.20368E+01_JPRB, 0.17071E+01_JPRB, 0.16734E+01_JPRB, 0.14599E+01_JPRB, 0.16675E+01_JPRB ,&
&  0.20068E+01_JPRB, 0.20691E+01_JPRB, 0.22032E+01_JPRB, 0.22247E+01_JPRB, 0.15826E+01_JPRB, 0.12558E+01_JPRB, 0.11044E+01_JPRB ,&
&  0.10850E+01_JPRB, 0.11053E+01_JPRB, 0.11018E+01_JPRB, 0.10773E+01_JPRB, 0.10665E+01_JPRB, 0.10612E+01_JPRB, 0.10596E+01_JPRB ,&
&  0.10694E+01_JPRB, 0.10850E+01_JPRB, 0.10965E+01_JPRB, 0.10988E+01_JPRB, 0.10974E+01_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.10536E+01_JPRB, 0.10368E+01_JPRB, 0.10142E+01_JPRB, 0.10030E+01_JPRB, 0.10215E+01_JPRB, 0.10364E+01_JPRB, 0.11438E+01_JPRB ,&
&  0.13491E+01_JPRB, 0.92371E+00_JPRB, 0.82326E+00_JPRB, 0.80289E+00_JPRB, 0.79717E+00_JPRB, 0.79915E+00_JPRB, 0.82272E+00_JPRB ,&
&  0.84441E+00_JPRB, 0.86759E+00_JPRB, 0.92089E+00_JPRB, 0.10661E+01_JPRB, 0.13642E+01_JPRB, 0.18513E+01_JPRB, 0.21061E+01_JPRB ,&
&  0.21578E+01_JPRB, 0.21953E+01_JPRB, 0.22204E+01_JPRB, 0.18701E+01_JPRB, 0.17346E+01_JPRB, 0.16437E+01_JPRB, 0.15949E+01_JPRB ,&
&  0.13565E+01_JPRB, 0.12384E+01_JPRB, 0.11759E+01_JPRB, 0.11081E+01_JPRB, 0.10405E+01_JPRB, 0.10224E+01_JPRB, 0.10406E+01_JPRB ,&
&  0.10692E+01_JPRB, 0.11006E+01_JPRB, 0.11391E+01_JPRB, 0.11901E+01_JPRB, 0.12802E+01_JPRB, 0.14213E+01_JPRB, 0.15758E+01_JPRB ,&
&  0.17313E+01_JPRB, 0.17783E+01_JPRB, 0.16390E+01_JPRB, 0.15075E+01_JPRB, 0.14462E+01_JPRB, 0.14769E+01_JPRB, 0.14839E+01_JPRB ,&
&  0.14552E+01_JPRB, 0.14527E+01_JPRB, 0.14691E+01_JPRB, 0.14660E+01_JPRB, 0.14088E+01_JPRB, 0.15756E+01_JPRB, 0.16781E+01_JPRB ,&
&  0.20015E+01_JPRB, 0.23819E+01_JPRB, 0.32253E+01_JPRB, 0.26497E+01_JPRB, 0.27537E+01_JPRB, 0.28889E+01_JPRB, 0.28265E+01_JPRB ,&
&  0.23492E+01_JPRB, 0.24087E+01_JPRB, 0.26268E+01_JPRB, 0.27533E+01_JPRB, 0.29279E+01_JPRB, 0.27097E+01_JPRB, 0.26442E+01_JPRB ,&
&  0.20051E+01_JPRB, 0.19290E+01_JPRB, 0.15178E+01_JPRB, 0.15635E+01_JPRB, 0.15620E+01_JPRB, 0.15098E+01_JPRB, 0.16251E+01_JPRB ,&
&  0.19177E+01_JPRB, 0.19625E+01_JPRB, 0.18115E+01_JPRB, 0.15759E+01_JPRB, 0.11722E+01_JPRB, 0.10647E+01_JPRB, 0.10403E+01_JPRB ,&
&  0.10515E+01_JPRB, 0.11076E+01_JPRB, 0.11022E+01_JPRB, 0.10994E+01_JPRB, 0.10887E+01_JPRB, 0.10830E+01_JPRB, 0.10879E+01_JPRB ,&
&  0.10980E+01_JPRB, 0.11017E+01_JPRB, 0.11003E+01_JPRB, 0.10878E+01_JPRB, 0.10743E+01_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.10293E+01_JPRB, 0.10137E+01_JPRB, 0.99835E+00_JPRB, 0.97245E+00_JPRB, 0.93846E+00_JPRB, 0.93920E+00_JPRB, 0.93403E+00_JPRB ,&
&  0.89185E+00_JPRB, 0.83973E+00_JPRB, 0.84029E+00_JPRB, 0.84339E+00_JPRB, 0.85227E+00_JPRB, 0.87083E+00_JPRB, 0.90799E+00_JPRB ,&
&  0.94507E+00_JPRB, 0.10127E+01_JPRB, 0.11712E+01_JPRB, 0.14483E+01_JPRB, 0.17717E+01_JPRB, 0.21395E+01_JPRB, 0.25648E+01_JPRB ,&
&  0.24905E+01_JPRB, 0.17123E+01_JPRB, 0.20217E+01_JPRB, 0.19526E+01_JPRB, 0.17674E+01_JPRB, 0.16013E+01_JPRB, 0.15050E+01_JPRB ,&
&  0.14210E+01_JPRB, 0.13548E+01_JPRB, 0.13018E+01_JPRB, 0.12385E+01_JPRB, 0.11765E+01_JPRB, 0.12115E+01_JPRB, 0.12536E+01_JPRB ,&
&  0.12914E+01_JPRB, 0.13431E+01_JPRB, 0.14084E+01_JPRB, 0.15072E+01_JPRB, 0.16492E+01_JPRB, 0.17865E+01_JPRB, 0.18870E+01_JPRB ,&
&  0.19219E+01_JPRB, 0.19380E+01_JPRB, 0.18709E+01_JPRB, 0.17620E+01_JPRB, 0.16511E+01_JPRB, 0.16313E+01_JPRB, 0.15853E+01_JPRB ,&
&  0.15534E+01_JPRB, 0.15218E+01_JPRB, 0.15718E+01_JPRB, 0.16786E+01_JPRB, 0.17520E+01_JPRB, 0.16954E+01_JPRB, 0.19488E+01_JPRB ,&
&  0.22400E+01_JPRB, 0.26637E+01_JPRB, 0.28825E+01_JPRB, 0.35745E+01_JPRB, 0.24585E+01_JPRB, 0.25682E+01_JPRB, 0.22212E+01_JPRB ,&
&  0.23136E+01_JPRB, 0.23099E+01_JPRB, 0.24836E+01_JPRB, 0.26440E+01_JPRB, 0.28870E+01_JPRB, 0.25968E+01_JPRB, 0.23174E+01_JPRB ,&
&  0.18568E+01_JPRB, 0.15187E+01_JPRB, 0.13068E+01_JPRB, 0.14028E+01_JPRB, 0.14978E+01_JPRB, 0.16352E+01_JPRB, 0.15936E+01_JPRB ,&
&  0.16603E+01_JPRB, 0.15680E+01_JPRB, 0.13956E+01_JPRB, 0.13459E+01_JPRB, 0.11598E+01_JPRB, 0.10631E+01_JPRB, 0.10683E+01_JPRB ,&
&  0.10874E+01_JPRB, 0.11227E+01_JPRB, 0.11094E+01_JPRB, 0.10902E+01_JPRB, 0.10747E+01_JPRB, 0.10658E+01_JPRB, 0.10711E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.10679E+01_JPRB, 0.10657E+01_JPRB, 0.10535E+01_JPRB, 0.10394E+01_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.98280E+00_JPRB, 0.95919E+00_JPRB, 0.91448E+00_JPRB, 0.87600E+00_JPRB, 0.86686E+00_JPRB, 0.86218E+00_JPRB, 0.84387E+00_JPRB ,&
&  0.84696E+00_JPRB, 0.86109E+00_JPRB, 0.87982E+00_JPRB, 0.90613E+00_JPRB, 0.93254E+00_JPRB, 0.95705E+00_JPRB, 0.10181E+01_JPRB ,&
&  0.10946E+01_JPRB, 0.12382E+01_JPRB, 0.14621E+01_JPRB, 0.16935E+01_JPRB, 0.19055E+01_JPRB, 0.21418E+01_JPRB, 0.23688E+01_JPRB ,&
&  0.26345E+01_JPRB, 0.26863E+01_JPRB, 0.24349E+01_JPRB, 0.16935E+01_JPRB, 0.14920E+01_JPRB, 0.16191E+01_JPRB, 0.15908E+01_JPRB ,&
&  0.15580E+01_JPRB, 0.14998E+01_JPRB, 0.14565E+01_JPRB, 0.14484E+01_JPRB, 0.14745E+01_JPRB, 0.15317E+01_JPRB, 0.15791E+01_JPRB ,&
&  0.16488E+01_JPRB, 0.17231E+01_JPRB, 0.18067E+01_JPRB, 0.18972E+01_JPRB, 0.19674E+01_JPRB, 0.20355E+01_JPRB, 0.21065E+01_JPRB ,&
&  0.21603E+01_JPRB, 0.22513E+01_JPRB, 0.23125E+01_JPRB, 0.22749E+01_JPRB, 0.21127E+01_JPRB, 0.20306E+01_JPRB, 0.19244E+01_JPRB ,&
&  0.18782E+01_JPRB, 0.18429E+01_JPRB, 0.18652E+01_JPRB, 0.20000E+01_JPRB, 0.21011E+01_JPRB, 0.19557E+01_JPRB, 0.20713E+01_JPRB ,&
&  0.25104E+01_JPRB, 0.29398E+01_JPRB, 0.26574E+01_JPRB, 0.33461E+01_JPRB, 0.19751E+01_JPRB, 0.22263E+01_JPRB, 0.22448E+01_JPRB ,&
&  0.19704E+01_JPRB, 0.20633E+01_JPRB, 0.22506E+01_JPRB, 0.24015E+01_JPRB, 0.24289E+01_JPRB, 0.21867E+01_JPRB, 0.19817E+01_JPRB ,&
&  0.16842E+01_JPRB, 0.13488E+01_JPRB, 0.11811E+01_JPRB, 0.12230E+01_JPRB, 0.13693E+01_JPRB, 0.15118E+01_JPRB, 0.12940E+01_JPRB ,&
&  0.10769E+01_JPRB, 0.13044E+01_JPRB, 0.13480E+01_JPRB, 0.14887E+01_JPRB, 0.13282E+01_JPRB, 0.10656E+01_JPRB, 0.10950E+01_JPRB ,&
&  0.11263E+01_JPRB, 0.11134E+01_JPRB, 0.11043E+01_JPRB, 0.10768E+01_JPRB, 0.10664E+01_JPRB, 0.10547E+01_JPRB, 0.10444E+01_JPRB ,&
&  0.10409E+01_JPRB, 0.10409E+01_JPRB, 0.10374E+01_JPRB, 0.10171E+01_JPRB, 0.99637E+00_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.91230E+00_JPRB, 0.86618E+00_JPRB, 0.82641E+00_JPRB, 0.80799E+00_JPRB, 0.80724E+00_JPRB, 0.80838E+00_JPRB, 0.81671E+00_JPRB ,&
&  0.83884E+00_JPRB, 0.86599E+00_JPRB, 0.89798E+00_JPRB, 0.94180E+00_JPRB, 0.98586E+00_JPRB, 0.10339E+01_JPRB, 0.11083E+01_JPRB ,&
&  0.11997E+01_JPRB, 0.13285E+01_JPRB, 0.15031E+01_JPRB, 0.16934E+01_JPRB, 0.18633E+01_JPRB, 0.20511E+01_JPRB, 0.22391E+01_JPRB ,&
&  0.24442E+01_JPRB, 0.25265E+01_JPRB, 0.24637E+01_JPRB, 0.23198E+01_JPRB, 0.18601E+01_JPRB, 0.19313E+01_JPRB, 0.20549E+01_JPRB ,&
&  0.21017E+01_JPRB, 0.18171E+01_JPRB, 0.17249E+01_JPRB, 0.17138E+01_JPRB, 0.17475E+01_JPRB, 0.18201E+01_JPRB, 0.18794E+01_JPRB ,&
&  0.19437E+01_JPRB, 0.20004E+01_JPRB, 0.20541E+01_JPRB, 0.20980E+01_JPRB, 0.21425E+01_JPRB, 0.22242E+01_JPRB, 0.23697E+01_JPRB ,&
&  0.24350E+01_JPRB, 0.25050E+01_JPRB, 0.25669E+01_JPRB, 0.25867E+01_JPRB, 0.27166E+01_JPRB, 0.28004E+01_JPRB, 0.27115E+01_JPRB ,&
&  0.26166E+01_JPRB, 0.24326E+01_JPRB, 0.24961E+01_JPRB, 0.26981E+01_JPRB, 0.27281E+01_JPRB, 0.25695E+01_JPRB, 0.25934E+01_JPRB ,&
&  0.29544E+01_JPRB, 0.33283E+01_JPRB, 0.23180E+01_JPRB, 0.25552E+01_JPRB, 0.28855E+01_JPRB, 0.22479E+01_JPRB, 0.17854E+01_JPRB ,&
&  0.15428E+01_JPRB, 0.16067E+01_JPRB, 0.18538E+01_JPRB, 0.19546E+01_JPRB, 0.19453E+01_JPRB, 0.21941E+01_JPRB, 0.17300E+01_JPRB ,&
&  0.14842E+01_JPRB, 0.11896E+01_JPRB, 0.10616E+01_JPRB, 0.10651E+01_JPRB, 0.11508E+01_JPRB, 0.11804E+01_JPRB, 0.96815E+00_JPRB ,&
&  0.92494E+00_JPRB, 0.12520E+01_JPRB, 0.13944E+01_JPRB, 0.16534E+01_JPRB, 0.16398E+01_JPRB, 0.10431E+01_JPRB, 0.10624E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.10763E+01_JPRB, 0.10614E+01_JPRB, 0.10443E+01_JPRB, 0.10300E+01_JPRB, 0.10168E+01_JPRB, 0.10095E+01_JPRB ,&
&  0.10047E+01_JPRB, 0.99978E+00_JPRB, 0.99349E+00_JPRB, 0.97389E+00_JPRB, 0.94601E+00_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.80503E+00_JPRB, 0.77160E+00_JPRB, 0.75214E+00_JPRB, 0.75693E+00_JPRB, 0.77492E+00_JPRB, 0.78864E+00_JPRB, 0.79390E+00_JPRB ,&
&  0.81115E+00_JPRB, 0.83576E+00_JPRB, 0.87417E+00_JPRB, 0.90870E+00_JPRB, 0.95188E+00_JPRB, 0.99318E+00_JPRB, 0.10604E+01_JPRB ,&
&  0.11290E+01_JPRB, 0.12087E+01_JPRB, 0.13438E+01_JPRB, 0.15451E+01_JPRB, 0.17114E+01_JPRB, 0.18642E+01_JPRB, 0.20874E+01_JPRB ,&
&  0.23061E+01_JPRB, 0.24383E+01_JPRB, 0.25812E+01_JPRB, 0.26460E+01_JPRB, 0.25389E+01_JPRB, 0.23911E+01_JPRB, 0.24288E+01_JPRB ,&
&  0.20500E+01_JPRB, 0.17300E+01_JPRB, 0.17850E+01_JPRB, 0.18613E+01_JPRB, 0.19530E+01_JPRB, 0.19937E+01_JPRB, 0.20380E+01_JPRB ,&
&  0.20882E+01_JPRB, 0.21232E+01_JPRB, 0.21562E+01_JPRB, 0.21977E+01_JPRB, 0.22698E+01_JPRB, 0.23923E+01_JPRB, 0.25598E+01_JPRB ,&
&  0.26304E+01_JPRB, 0.27234E+01_JPRB, 0.28623E+01_JPRB, 0.26658E+01_JPRB, 0.29304E+01_JPRB, 0.32906E+01_JPRB, 0.34987E+01_JPRB ,&
&  0.34701E+01_JPRB, 0.32445E+01_JPRB, 0.33030E+01_JPRB, 0.34006E+01_JPRB, 0.34323E+01_JPRB, 0.33584E+01_JPRB, 0.33052E+01_JPRB ,&
&  0.34879E+01_JPRB, 0.33161E+01_JPRB, 0.23340E+01_JPRB, 0.20405E+01_JPRB, 0.20378E+01_JPRB, 0.15666E+01_JPRB, 0.14738E+01_JPRB ,&
&  0.12521E+01_JPRB, 0.12840E+01_JPRB, 0.14982E+01_JPRB, 0.16650E+01_JPRB, 0.17816E+01_JPRB, 0.20448E+01_JPRB, 0.16680E+01_JPRB ,&
&  0.12505E+01_JPRB, 0.99884E+00_JPRB, 0.86937E+00_JPRB, 0.84013E+00_JPRB, 0.87822E+00_JPRB, 0.90888E+00_JPRB, 0.89207E+00_JPRB ,&
&  0.94172E+00_JPRB, 0.98018E+00_JPRB, 0.10172E+01_JPRB, 0.10440E+01_JPRB, 0.97877E+00_JPRB, 0.99228E+00_JPRB, 0.99551E+00_JPRB ,&
&  0.98279E+00_JPRB, 0.98951E+00_JPRB, 0.98557E+00_JPRB, 0.96807E+00_JPRB, 0.96060E+00_JPRB, 0.94991E+00_JPRB, 0.94665E+00_JPRB ,&
&  0.94535E+00_JPRB, 0.93875E+00_JPRB, 0.92178E+00_JPRB, 0.89861E+00_JPRB, 0.85516E+00_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.71690E+00_JPRB, 0.69208E+00_JPRB, 0.69568E+00_JPRB, 0.71343E+00_JPRB, 0.72428E+00_JPRB, 0.73518E+00_JPRB, 0.74682E+00_JPRB ,&
&  0.75116E+00_JPRB, 0.75350E+00_JPRB, 0.76011E+00_JPRB, 0.76312E+00_JPRB, 0.79847E+00_JPRB, 0.83920E+00_JPRB, 0.89205E+00_JPRB ,&
&  0.97101E+00_JPRB, 0.10427E+01_JPRB, 0.11085E+01_JPRB, 0.12257E+01_JPRB, 0.13114E+01_JPRB, 0.13949E+01_JPRB, 0.15295E+01_JPRB ,&
&  0.16983E+01_JPRB, 0.18878E+01_JPRB, 0.20667E+01_JPRB, 0.22474E+01_JPRB, 0.24822E+01_JPRB, 0.27137E+01_JPRB, 0.27459E+01_JPRB ,&
&  0.21281E+01_JPRB, 0.16722E+01_JPRB, 0.18765E+01_JPRB, 0.18059E+01_JPRB, 0.19117E+01_JPRB, 0.19934E+01_JPRB, 0.20326E+01_JPRB ,&
&  0.20887E+01_JPRB, 0.21297E+01_JPRB, 0.21678E+01_JPRB, 0.22119E+01_JPRB, 0.23036E+01_JPRB, 0.24189E+01_JPRB, 0.25716E+01_JPRB ,&
&  0.26756E+01_JPRB, 0.27646E+01_JPRB, 0.28062E+01_JPRB, 0.28277E+01_JPRB, 0.30791E+01_JPRB, 0.34515E+01_JPRB, 0.37301E+01_JPRB ,&
&  0.38922E+01_JPRB, 0.40376E+01_JPRB, 0.36790E+01_JPRB, 0.36918E+01_JPRB, 0.36278E+01_JPRB, 0.32594E+01_JPRB, 0.32958E+01_JPRB ,&
&  0.35073E+01_JPRB, 0.32386E+01_JPRB, 0.20332E+01_JPRB, 0.18357E+01_JPRB, 0.17052E+01_JPRB, 0.14829E+01_JPRB, 0.13149E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.10544E+01_JPRB, 0.11656E+01_JPRB, 0.14209E+01_JPRB, 0.15835E+01_JPRB, 0.16740E+01_JPRB, 0.14767E+01_JPRB ,&
&  0.11012E+01_JPRB, 0.89417E+00_JPRB, 0.78354E+00_JPRB, 0.79285E+00_JPRB, 0.83779E+00_JPRB, 0.92759E+00_JPRB, 0.84496E+00_JPRB ,&
&  0.90070E+00_JPRB, 0.10003E+01_JPRB, 0.12122E+01_JPRB, 0.10743E+01_JPRB, 0.87317E+00_JPRB, 0.92040E+00_JPRB, 0.90834E+00_JPRB ,&
&  0.90025E+00_JPRB, 0.90198E+00_JPRB, 0.89571E+00_JPRB, 0.88845E+00_JPRB, 0.88305E+00_JPRB, 0.88794E+00_JPRB, 0.87713E+00_JPRB ,&
&  0.86625E+00_JPRB, 0.84760E+00_JPRB, 0.83381E+00_JPRB, 0.78836E+00_JPRB, 0.75011E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.64680E+00_JPRB, 0.64872E+00_JPRB, 0.65651E+00_JPRB, 0.67326E+00_JPRB, 0.67098E+00_JPRB, 0.66914E+00_JPRB, 0.69600E+00_JPRB ,&
&  0.67990E+00_JPRB, 0.66756E+00_JPRB, 0.66658E+00_JPRB, 0.64178E+00_JPRB, 0.66198E+00_JPRB, 0.68202E+00_JPRB, 0.72259E+00_JPRB ,&
&  0.76485E+00_JPRB, 0.79195E+00_JPRB, 0.81607E+00_JPRB, 0.85150E+00_JPRB, 0.91728E+00_JPRB, 0.96811E+00_JPRB, 0.10121E+01_JPRB ,&
&  0.10884E+01_JPRB, 0.12057E+01_JPRB, 0.13346E+01_JPRB, 0.16291E+01_JPRB, 0.23084E+01_JPRB, 0.28557E+01_JPRB, 0.28535E+01_JPRB ,&
&  0.21736E+01_JPRB, 0.16613E+01_JPRB, 0.16019E+01_JPRB, 0.14808E+01_JPRB, 0.15754E+01_JPRB, 0.17642E+01_JPRB, 0.18415E+01_JPRB ,&
&  0.18456E+01_JPRB, 0.18808E+01_JPRB, 0.19012E+01_JPRB, 0.19288E+01_JPRB, 0.19661E+01_JPRB, 0.19929E+01_JPRB, 0.19779E+01_JPRB ,&
&  0.19099E+01_JPRB, 0.18265E+01_JPRB, 0.19014E+01_JPRB, 0.23090E+01_JPRB, 0.27190E+01_JPRB, 0.33038E+01_JPRB, 0.32545E+01_JPRB ,&
&  0.31200E+01_JPRB, 0.30742E+01_JPRB, 0.30643E+01_JPRB, 0.30045E+01_JPRB, 0.28838E+01_JPRB, 0.27480E+01_JPRB, 0.27951E+01_JPRB ,&
&  0.29628E+01_JPRB, 0.27530E+01_JPRB, 0.18082E+01_JPRB, 0.17657E+01_JPRB, 0.15983E+01_JPRB, 0.15152E+01_JPRB, 0.12061E+01_JPRB ,&
&  0.96481E+00_JPRB, 0.91011E+00_JPRB, 0.93803E+00_JPRB, 0.10370E+01_JPRB, 0.12337E+01_JPRB, 0.13363E+01_JPRB, 0.12082E+01_JPRB ,&
&  0.10057E+01_JPRB, 0.84943E+00_JPRB, 0.75633E+00_JPRB, 0.72595E+00_JPRB, 0.83314E+00_JPRB, 0.88586E+00_JPRB, 0.10229E+01_JPRB ,&
&  0.79194E+00_JPRB, 0.87715E+00_JPRB, 0.80604E+00_JPRB, 0.10307E+01_JPRB, 0.80844E+00_JPRB, 0.76959E+00_JPRB, 0.75960E+00_JPRB ,&
&  0.75871E+00_JPRB, 0.74380E+00_JPRB, 0.72895E+00_JPRB, 0.70821E+00_JPRB, 0.68771E+00_JPRB, 0.72103E+00_JPRB, 0.75986E+00_JPRB ,&
&  0.72525E+00_JPRB, 0.74346E+00_JPRB, 0.72009E+00_JPRB, 0.68177E+00_JPRB, 0.67098E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.62739E+00_JPRB, 0.63967E+00_JPRB, 0.64151E+00_JPRB, 0.64903E+00_JPRB, 0.66413E+00_JPRB, 0.67507E+00_JPRB, 0.69057E+00_JPRB ,&
&  0.70756E+00_JPRB, 0.71618E+00_JPRB, 0.72506E+00_JPRB, 0.73650E+00_JPRB, 0.74468E+00_JPRB, 0.76742E+00_JPRB, 0.78998E+00_JPRB ,&
&  0.81904E+00_JPRB, 0.84202E+00_JPRB, 0.86608E+00_JPRB, 0.90274E+00_JPRB, 0.94903E+00_JPRB, 0.10213E+01_JPRB, 0.10983E+01_JPRB ,&
&  0.11990E+01_JPRB, 0.13169E+01_JPRB, 0.14569E+01_JPRB, 0.15714E+01_JPRB, 0.16326E+01_JPRB, 0.16694E+01_JPRB, 0.18460E+01_JPRB ,&
&  0.16364E+01_JPRB, 0.17485E+01_JPRB, 0.16761E+01_JPRB, 0.15081E+01_JPRB, 0.15006E+01_JPRB, 0.13949E+01_JPRB, 0.13748E+01_JPRB ,&
&  0.13977E+01_JPRB, 0.13511E+01_JPRB, 0.13280E+01_JPRB, 0.13077E+01_JPRB, 0.12822E+01_JPRB, 0.12184E+01_JPRB, 0.11401E+01_JPRB ,&
&  0.11317E+01_JPRB, 0.12201E+01_JPRB, 0.14043E+01_JPRB, 0.17776E+01_JPRB, 0.21496E+01_JPRB, 0.22959E+01_JPRB, 0.21398E+01_JPRB ,&
&  0.20530E+01_JPRB, 0.20542E+01_JPRB, 0.19735E+01_JPRB, 0.18796E+01_JPRB, 0.19629E+01_JPRB, 0.20313E+01_JPRB, 0.21065E+01_JPRB ,&
&  0.20342E+01_JPRB, 0.18080E+01_JPRB, 0.16785E+01_JPRB, 0.15218E+01_JPRB, 0.14334E+01_JPRB, 0.13413E+01_JPRB, 0.11606E+01_JPRB ,&
&  0.96930E+00_JPRB, 0.85931E+00_JPRB, 0.81271E+00_JPRB, 0.84066E+00_JPRB, 0.96514E+00_JPRB, 0.98805E+00_JPRB, 0.91064E+00_JPRB ,&
&  0.78267E+00_JPRB, 0.71267E+00_JPRB, 0.67782E+00_JPRB, 0.67047E+00_JPRB, 0.71542E+00_JPRB, 0.13129E+01_JPRB, 0.17839E+01_JPRB ,&
&  0.76721E+00_JPRB, 0.65235E+00_JPRB, 0.51904E+00_JPRB, 0.71998E+00_JPRB, 0.65008E+00_JPRB, 0.64890E+00_JPRB, 0.64897E+00_JPRB ,&
&  0.64072E+00_JPRB, 0.63971E+00_JPRB, 0.63758E+00_JPRB, 0.62215E+00_JPRB, 0.61131E+00_JPRB, 0.61091E+00_JPRB, 0.63915E+00_JPRB ,&
&  0.63608E+00_JPRB, 0.63738E+00_JPRB, 0.62928E+00_JPRB, 0.63757E+00_JPRB, 0.63213E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.62727E+00_JPRB, 0.64218E+00_JPRB, 0.66024E+00_JPRB, 0.68260E+00_JPRB, 0.69795E+00_JPRB, 0.71872E+00_JPRB, 0.73598E+00_JPRB ,&
&  0.75950E+00_JPRB, 0.77885E+00_JPRB, 0.79561E+00_JPRB, 0.81316E+00_JPRB, 0.83499E+00_JPRB, 0.86117E+00_JPRB, 0.89431E+00_JPRB ,&
&  0.93027E+00_JPRB, 0.96632E+00_JPRB, 0.10059E+01_JPRB, 0.10513E+01_JPRB, 0.11046E+01_JPRB, 0.11700E+01_JPRB, 0.12324E+01_JPRB ,&
&  0.12908E+01_JPRB, 0.13500E+01_JPRB, 0.14102E+01_JPRB, 0.14571E+01_JPRB, 0.14749E+01_JPRB, 0.15017E+01_JPRB, 0.13487E+01_JPRB ,&
&  0.16678E+01_JPRB, 0.16433E+01_JPRB, 0.15434E+01_JPRB, 0.14111E+01_JPRB, 0.12858E+01_JPRB, 0.11723E+01_JPRB, 0.10797E+01_JPRB ,&
&  0.10989E+01_JPRB, 0.10603E+01_JPRB, 0.10491E+01_JPRB, 0.10524E+01_JPRB, 0.98756E+00_JPRB, 0.99149E+00_JPRB, 0.10591E+01_JPRB ,&
&  0.11749E+01_JPRB, 0.13394E+01_JPRB, 0.14398E+01_JPRB, 0.15597E+01_JPRB, 0.17131E+01_JPRB, 0.16538E+01_JPRB, 0.15716E+01_JPRB ,&
&  0.14592E+01_JPRB, 0.13869E+01_JPRB, 0.11193E+01_JPRB, 0.12680E+01_JPRB, 0.13220E+01_JPRB, 0.13834E+01_JPRB, 0.14812E+01_JPRB ,&
&  0.13853E+01_JPRB, 0.11375E+01_JPRB, 0.11743E+01_JPRB, 0.13791E+01_JPRB, 0.13027E+01_JPRB, 0.11859E+01_JPRB, 0.10833E+01_JPRB ,&
&  0.97240E+00_JPRB, 0.88677E+00_JPRB, 0.81327E+00_JPRB, 0.78768E+00_JPRB, 0.83141E+00_JPRB, 0.82451E+00_JPRB, 0.75165E+00_JPRB ,&
&  0.66603E+00_JPRB, 0.63610E+00_JPRB, 0.62972E+00_JPRB, 0.63518E+00_JPRB, 0.71936E+00_JPRB, 0.80856E+00_JPRB, 0.12212E+01_JPRB ,&
&  0.77099E+00_JPRB, 0.47316E+00_JPRB, 0.54242E+00_JPRB, 0.61857E+00_JPRB, 0.58613E+00_JPRB, 0.60596E+00_JPRB, 0.63808E+00_JPRB ,&
&  0.66972E+00_JPRB, 0.68160E+00_JPRB, 0.63945E+00_JPRB, 0.63787E+00_JPRB, 0.62992E+00_JPRB, 0.62184E+00_JPRB, 0.61425E+00_JPRB ,&
&  0.60559E+00_JPRB, 0.60727E+00_JPRB, 0.62020E+00_JPRB, 0.61898E+00_JPRB, 0.62019E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.55322E+00_JPRB, 0.56327E+00_JPRB, 0.57595E+00_JPRB, 0.58697E+00_JPRB, 0.59391E+00_JPRB, 0.60624E+00_JPRB, 0.61548E+00_JPRB ,&
&  0.62646E+00_JPRB, 0.63148E+00_JPRB, 0.64039E+00_JPRB, 0.64815E+00_JPRB, 0.65814E+00_JPRB, 0.67282E+00_JPRB, 0.68763E+00_JPRB ,&
&  0.69876E+00_JPRB, 0.71639E+00_JPRB, 0.73711E+00_JPRB, 0.75140E+00_JPRB, 0.78799E+00_JPRB, 0.85683E+00_JPRB, 0.93516E+00_JPRB ,&
&  0.99905E+00_JPRB, 0.10815E+01_JPRB, 0.11591E+01_JPRB, 0.11942E+01_JPRB, 0.11955E+01_JPRB, 0.12249E+01_JPRB, 0.10739E+01_JPRB ,&
&  0.14975E+01_JPRB, 0.14944E+01_JPRB, 0.13634E+01_JPRB, 0.12856E+01_JPRB, 0.12075E+01_JPRB, 0.11303E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.11010E+01_JPRB, 0.13051E+01_JPRB, 0.13394E+01_JPRB, 0.13960E+01_JPRB, 0.13070E+01_JPRB, 0.13113E+01_JPRB, 0.14634E+01_JPRB ,&
&  0.16416E+01_JPRB, 0.17030E+01_JPRB, 0.17117E+01_JPRB, 0.17983E+01_JPRB, 0.19004E+01_JPRB, 0.19530E+01_JPRB, 0.19709E+01_JPRB ,&
&  0.18980E+01_JPRB, 0.18274E+01_JPRB, 0.17077E+01_JPRB, 0.20131E+01_JPRB, 0.17900E+01_JPRB, 0.15657E+01_JPRB, 0.14265E+01_JPRB ,&
&  0.11013E+01_JPRB, 0.10225E+01_JPRB, 0.10842E+01_JPRB, 0.11399E+01_JPRB, 0.10890E+01_JPRB, 0.10711E+01_JPRB, 0.10187E+01_JPRB ,&
&  0.95403E+00_JPRB, 0.91145E+00_JPRB, 0.85824E+00_JPRB, 0.83951E+00_JPRB, 0.84068E+00_JPRB, 0.79757E+00_JPRB, 0.71008E+00_JPRB ,&
&  0.64555E+00_JPRB, 0.63476E+00_JPRB, 0.62011E+00_JPRB, 0.61861E+00_JPRB, 0.65558E+00_JPRB, 0.63862E+00_JPRB, 0.89123E+00_JPRB ,&
&  0.70752E+00_JPRB, 0.60761E+00_JPRB, 0.78425E+00_JPRB, 0.43499E+00_JPRB, 0.53924E+00_JPRB, 0.65211E+00_JPRB, 0.84205E+00_JPRB ,&
&  0.89344E+00_JPRB, 0.11528E+01_JPRB, 0.14981E+01_JPRB, 0.11880E+01_JPRB, 0.90472E+00_JPRB, 0.61314E+00_JPRB, 0.57930E+00_JPRB ,&
&  0.56989E+00_JPRB, 0.56045E+00_JPRB, 0.56328E+00_JPRB, 0.55612E+00_JPRB, 0.55433E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.48085E+00_JPRB, 0.48388E+00_JPRB, 0.49282E+00_JPRB, 0.49882E+00_JPRB, 0.50483E+00_JPRB, 0.50930E+00_JPRB, 0.51372E+00_JPRB ,&
&  0.52796E+00_JPRB, 0.53648E+00_JPRB, 0.55822E+00_JPRB, 0.57536E+00_JPRB, 0.59061E+00_JPRB, 0.62323E+00_JPRB, 0.64994E+00_JPRB ,&
&  0.69245E+00_JPRB, 0.74772E+00_JPRB, 0.77026E+00_JPRB, 0.80322E+00_JPRB, 0.82312E+00_JPRB, 0.84518E+00_JPRB, 0.86189E+00_JPRB ,&
&  0.93577E+00_JPRB, 0.10803E+01_JPRB, 0.11821E+01_JPRB, 0.12880E+01_JPRB, 0.12961E+01_JPRB, 0.13586E+01_JPRB, 0.11009E+01_JPRB ,&
&  0.14688E+01_JPRB, 0.15037E+01_JPRB, 0.14291E+01_JPRB, 0.14044E+01_JPRB, 0.13982E+01_JPRB, 0.14019E+01_JPRB, 0.14012E+01_JPRB ,&
&  0.14746E+01_JPRB, 0.15978E+01_JPRB, 0.16147E+01_JPRB, 0.17292E+01_JPRB, 0.18355E+01_JPRB, 0.18718E+01_JPRB, 0.20867E+01_JPRB ,&
&  0.22773E+01_JPRB, 0.24308E+01_JPRB, 0.26258E+01_JPRB, 0.28666E+01_JPRB, 0.30382E+01_JPRB, 0.31463E+01_JPRB, 0.31899E+01_JPRB ,&
&  0.32040E+01_JPRB, 0.33040E+01_JPRB, 0.35831E+01_JPRB, 0.35630E+01_JPRB, 0.28232E+01_JPRB, 0.22998E+01_JPRB, 0.17568E+01_JPRB ,&
&  0.11742E+01_JPRB, 0.94572E+00_JPRB, 0.93261E+00_JPRB, 0.97188E+00_JPRB, 0.93869E+00_JPRB, 0.91959E+00_JPRB, 0.89549E+00_JPRB ,&
&  0.87656E+00_JPRB, 0.88847E+00_JPRB, 0.89019E+00_JPRB, 0.88399E+00_JPRB, 0.85205E+00_JPRB, 0.77159E+00_JPRB, 0.68000E+00_JPRB ,&
&  0.65039E+00_JPRB, 0.63307E+00_JPRB, 0.63094E+00_JPRB, 0.64980E+00_JPRB, 0.68644E+00_JPRB, 0.77352E+00_JPRB, 0.75440E+00_JPRB ,&
&  0.81546E+00_JPRB, 0.60609E+00_JPRB, 0.59593E+00_JPRB, 0.62819E+00_JPRB, 0.62028E+00_JPRB, 0.85583E+00_JPRB, 0.12543E+01_JPRB ,&
&  0.18520E+01_JPRB, 0.21133E+01_JPRB, 0.27946E+01_JPRB, 0.40980E+01_JPRB, 0.25825E+01_JPRB, 0.16539E+01_JPRB, 0.54866E+00_JPRB ,&
&  0.53846E+00_JPRB, 0.51413E+00_JPRB, 0.50701E+00_JPRB, 0.49179E+00_JPRB, 0.48644E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.39603E+00_JPRB, 0.39006E+00_JPRB, 0.40059E+00_JPRB, 0.41804E+00_JPRB, 0.43128E+00_JPRB, 0.46099E+00_JPRB, 0.49682E+00_JPRB ,&
&  0.53617E+00_JPRB, 0.58505E+00_JPRB, 0.62623E+00_JPRB, 0.65548E+00_JPRB, 0.70513E+00_JPRB, 0.77438E+00_JPRB, 0.80460E+00_JPRB ,&
&  0.86411E+00_JPRB, 0.90176E+00_JPRB, 0.92041E+00_JPRB, 0.93307E+00_JPRB, 0.96366E+00_JPRB, 0.10147E+01_JPRB, 0.10768E+01_JPRB ,&
&  0.11356E+01_JPRB, 0.12071E+01_JPRB, 0.13041E+01_JPRB, 0.13696E+01_JPRB, 0.14038E+01_JPRB, 0.14189E+01_JPRB, 0.11344E+01_JPRB ,&
&  0.14768E+01_JPRB, 0.15790E+01_JPRB, 0.16386E+01_JPRB, 0.16386E+01_JPRB, 0.16323E+01_JPRB, 0.16395E+01_JPRB, 0.16979E+01_JPRB ,&
&  0.17987E+01_JPRB, 0.18457E+01_JPRB, 0.18731E+01_JPRB, 0.19093E+01_JPRB, 0.24471E+01_JPRB, 0.25594E+01_JPRB, 0.27999E+01_JPRB ,&
&  0.30250E+01_JPRB, 0.32254E+01_JPRB, 0.34925E+01_JPRB, 0.37763E+01_JPRB, 0.40713E+01_JPRB, 0.43241E+01_JPRB, 0.45528E+01_JPRB ,&
&  0.47638E+01_JPRB, 0.49923E+01_JPRB, 0.52121E+01_JPRB, 0.44814E+01_JPRB, 0.35417E+01_JPRB, 0.28784E+01_JPRB, 0.17737E+01_JPRB ,&
&  0.11049E+01_JPRB, 0.94760E+00_JPRB, 0.95587E+00_JPRB, 0.93424E+00_JPRB, 0.84673E+00_JPRB, 0.80582E+00_JPRB, 0.77365E+00_JPRB ,&
&  0.78552E+00_JPRB, 0.82168E+00_JPRB, 0.84919E+00_JPRB, 0.85164E+00_JPRB, 0.80897E+00_JPRB, 0.73416E+00_JPRB, 0.66348E+00_JPRB ,&
&  0.67088E+00_JPRB, 0.66612E+00_JPRB, 0.66494E+00_JPRB, 0.70981E+00_JPRB, 0.75897E+00_JPRB, 0.90537E+00_JPRB, 0.10984E+01_JPRB ,&
&  0.10541E+01_JPRB, 0.63793E+00_JPRB, 0.61478E+00_JPRB, 0.61860E+00_JPRB, 0.63613E+00_JPRB, 0.70808E+00_JPRB, 0.86463E+00_JPRB ,&
&  0.12170E+01_JPRB, 0.19864E+01_JPRB, 0.22489E+01_JPRB, 0.14866E+01_JPRB, 0.20064E+01_JPRB, 0.13087E+01_JPRB, 0.53286E+00_JPRB ,&
&  0.50552E+00_JPRB, 0.49018E+00_JPRB, 0.46270E+00_JPRB, 0.43231E+00_JPRB, 0.41123E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.40065E+00_JPRB, 0.38298E+00_JPRB, 0.39355E+00_JPRB, 0.41798E+00_JPRB, 0.44704E+00_JPRB, 0.49122E+00_JPRB, 0.54520E+00_JPRB ,&
&  0.59286E+00_JPRB, 0.63959E+00_JPRB, 0.71942E+00_JPRB, 0.73909E+00_JPRB, 0.75682E+00_JPRB, 0.80011E+00_JPRB, 0.83017E+00_JPRB ,&
&  0.86531E+00_JPRB, 0.89371E+00_JPRB, 0.92581E+00_JPRB, 0.95723E+00_JPRB, 0.10047E+01_JPRB, 0.10589E+01_JPRB, 0.11045E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.12291E+01_JPRB, 0.13251E+01_JPRB, 0.14273E+01_JPRB, 0.15240E+01_JPRB, 0.15506E+01_JPRB, 0.15970E+01_JPRB ,&
&  0.11369E+01_JPRB, 0.15734E+01_JPRB, 0.16395E+01_JPRB, 0.16804E+01_JPRB, 0.16840E+01_JPRB, 0.16849E+01_JPRB, 0.17560E+01_JPRB ,&
&  0.18620E+01_JPRB, 0.18577E+01_JPRB, 0.19921E+01_JPRB, 0.22219E+01_JPRB, 0.28036E+01_JPRB, 0.29520E+01_JPRB, 0.31395E+01_JPRB ,&
&  0.33439E+01_JPRB, 0.35417E+01_JPRB, 0.37730E+01_JPRB, 0.40523E+01_JPRB, 0.43366E+01_JPRB, 0.46056E+01_JPRB, 0.48809E+01_JPRB ,&
&  0.51304E+01_JPRB, 0.53606E+01_JPRB, 0.56085E+01_JPRB, 0.46543E+01_JPRB, 0.38452E+01_JPRB, 0.30794E+01_JPRB, 0.19790E+01_JPRB ,&
&  0.12284E+01_JPRB, 0.93153E+00_JPRB, 0.94379E+00_JPRB, 0.96822E+00_JPRB, 0.85184E+00_JPRB, 0.78684E+00_JPRB, 0.70730E+00_JPRB ,&
&  0.70064E+00_JPRB, 0.72138E+00_JPRB, 0.74177E+00_JPRB, 0.74387E+00_JPRB, 0.74532E+00_JPRB, 0.68758E+00_JPRB, 0.64935E+00_JPRB ,&
&  0.66200E+00_JPRB, 0.65528E+00_JPRB, 0.65870E+00_JPRB, 0.68091E+00_JPRB, 0.72381E+00_JPRB, 0.76650E+00_JPRB, 0.76932E+00_JPRB ,&
&  0.77110E+00_JPRB, 0.69310E+00_JPRB, 0.66442E+00_JPRB, 0.67981E+00_JPRB, 0.67420E+00_JPRB, 0.66672E+00_JPRB, 0.68712E+00_JPRB ,&
&  0.77104E+00_JPRB, 0.87563E+00_JPRB, 0.96491E+00_JPRB, 0.95263E+00_JPRB, 0.99471E+00_JPRB, 0.78537E+00_JPRB, 0.58805E+00_JPRB ,&
&  0.55204E+00_JPRB, 0.54125E+00_JPRB, 0.49724E+00_JPRB, 0.46635E+00_JPRB, 0.43117E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.49636E+00_JPRB, 0.43874E+00_JPRB, 0.43398E+00_JPRB, 0.47635E+00_JPRB, 0.51964E+00_JPRB, 0.56430E+00_JPRB, 0.60556E+00_JPRB ,&
&  0.64863E+00_JPRB, 0.67268E+00_JPRB, 0.71983E+00_JPRB, 0.74312E+00_JPRB, 0.76130E+00_JPRB, 0.76844E+00_JPRB, 0.77818E+00_JPRB ,&
&  0.80863E+00_JPRB, 0.84392E+00_JPRB, 0.87500E+00_JPRB, 0.92290E+00_JPRB, 0.97291E+00_JPRB, 0.10200E+01_JPRB, 0.10754E+01_JPRB ,&
&  0.11297E+01_JPRB, 0.11923E+01_JPRB, 0.12578E+01_JPRB, 0.13381E+01_JPRB, 0.14019E+01_JPRB, 0.14275E+01_JPRB, 0.13753E+01_JPRB ,&
&  0.73967E+00_JPRB, 0.12001E+01_JPRB, 0.15664E+01_JPRB, 0.15954E+01_JPRB, 0.16180E+01_JPRB, 0.16212E+01_JPRB, 0.17083E+01_JPRB ,&
&  0.17092E+01_JPRB, 0.16614E+01_JPRB, 0.18299E+01_JPRB, 0.25377E+01_JPRB, 0.28774E+01_JPRB, 0.30558E+01_JPRB, 0.31685E+01_JPRB ,&
&  0.33107E+01_JPRB, 0.34223E+01_JPRB, 0.35594E+01_JPRB, 0.37076E+01_JPRB, 0.38775E+01_JPRB, 0.40417E+01_JPRB, 0.41944E+01_JPRB ,&
&  0.43266E+01_JPRB, 0.44051E+01_JPRB, 0.43914E+01_JPRB, 0.34645E+01_JPRB, 0.33951E+01_JPRB, 0.28529E+01_JPRB, 0.24127E+01_JPRB ,&
&  0.14550E+01_JPRB, 0.99159E+00_JPRB, 0.96406E+00_JPRB, 0.98129E+00_JPRB, 0.86166E+00_JPRB, 0.79067E+00_JPRB, 0.69889E+00_JPRB ,&
&  0.69151E+00_JPRB, 0.69180E+00_JPRB, 0.70352E+00_JPRB, 0.69896E+00_JPRB, 0.69957E+00_JPRB, 0.64552E+00_JPRB, 0.65714E+00_JPRB ,&
&  0.67287E+00_JPRB, 0.66689E+00_JPRB, 0.67488E+00_JPRB, 0.70358E+00_JPRB, 0.74245E+00_JPRB, 0.78053E+00_JPRB, 0.79343E+00_JPRB ,&
&  0.78634E+00_JPRB, 0.76907E+00_JPRB, 0.76685E+00_JPRB, 0.76499E+00_JPRB, 0.76177E+00_JPRB, 0.75715E+00_JPRB, 0.74393E+00_JPRB ,&
&  0.72750E+00_JPRB, 0.73789E+00_JPRB, 0.76983E+00_JPRB, 0.71356E+00_JPRB, 0.75445E+00_JPRB, 0.68499E+00_JPRB, 0.66937E+00_JPRB ,&
&  0.65279E+00_JPRB, 0.63544E+00_JPRB, 0.58968E+00_JPRB, 0.56659E+00_JPRB, 0.53520E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.59904E+00_JPRB, 0.53153E+00_JPRB, 0.47632E+00_JPRB, 0.49146E+00_JPRB, 0.55552E+00_JPRB, 0.60994E+00_JPRB, 0.63888E+00_JPRB ,&
&  0.67294E+00_JPRB, 0.68228E+00_JPRB, 0.68932E+00_JPRB, 0.69956E+00_JPRB, 0.72394E+00_JPRB, 0.73343E+00_JPRB, 0.75889E+00_JPRB ,&
&  0.77443E+00_JPRB, 0.78665E+00_JPRB, 0.80364E+00_JPRB, 0.84428E+00_JPRB, 0.89306E+00_JPRB, 0.94212E+00_JPRB, 0.98804E+00_JPRB ,&
&  0.10378E+01_JPRB, 0.10945E+01_JPRB, 0.11755E+01_JPRB, 0.12757E+01_JPRB, 0.14105E+01_JPRB, 0.15590E+01_JPRB, 0.16712E+01_JPRB ,&
&  0.12003E+01_JPRB, 0.63971E+00_JPRB, 0.12114E+01_JPRB, 0.15217E+01_JPRB, 0.15026E+01_JPRB, 0.15061E+01_JPRB, 0.15031E+01_JPRB ,&
&  0.15012E+01_JPRB, 0.15494E+01_JPRB, 0.16349E+01_JPRB, 0.21725E+01_JPRB, 0.27188E+01_JPRB, 0.29188E+01_JPRB, 0.29906E+01_JPRB ,&
&  0.30625E+01_JPRB, 0.30910E+01_JPRB, 0.31382E+01_JPRB, 0.32121E+01_JPRB, 0.33056E+01_JPRB, 0.33654E+01_JPRB, 0.34104E+01_JPRB ,&
&  0.34478E+01_JPRB, 0.34363E+01_JPRB, 0.33054E+01_JPRB, 0.23814E+01_JPRB, 0.21289E+01_JPRB, 0.18393E+01_JPRB, 0.16709E+01_JPRB ,&
&  0.14574E+01_JPRB, 0.11172E+01_JPRB, 0.10195E+01_JPRB, 0.99330E+00_JPRB, 0.89241E+00_JPRB, 0.76950E+00_JPRB, 0.73195E+00_JPRB ,&
&  0.72548E+00_JPRB, 0.70375E+00_JPRB, 0.68235E+00_JPRB, 0.66410E+00_JPRB, 0.69255E+00_JPRB, 0.68895E+00_JPRB, 0.68268E+00_JPRB ,&
&  0.67637E+00_JPRB, 0.70874E+00_JPRB, 0.74269E+00_JPRB, 0.77124E+00_JPRB, 0.80088E+00_JPRB, 0.83550E+00_JPRB, 0.83927E+00_JPRB ,&
&  0.84986E+00_JPRB, 0.84256E+00_JPRB, 0.84046E+00_JPRB, 0.82236E+00_JPRB, 0.80733E+00_JPRB, 0.77875E+00_JPRB, 0.77690E+00_JPRB ,&
&  0.78054E+00_JPRB, 0.78726E+00_JPRB, 0.78003E+00_JPRB, 0.76625E+00_JPRB, 0.78659E+00_JPRB, 0.77757E+00_JPRB, 0.78364E+00_JPRB ,&
&  0.77956E+00_JPRB, 0.75901E+00_JPRB, 0.70270E+00_JPRB, 0.65716E+00_JPRB, 0.63760E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.65260E+00_JPRB, 0.61259E+00_JPRB, 0.55276E+00_JPRB, 0.51301E+00_JPRB, 0.53389E+00_JPRB, 0.59985E+00_JPRB, 0.65277E+00_JPRB ,&
&  0.67288E+00_JPRB, 0.67218E+00_JPRB, 0.66328E+00_JPRB, 0.66247E+00_JPRB, 0.67689E+00_JPRB, 0.67489E+00_JPRB, 0.68821E+00_JPRB ,&
&  0.70857E+00_JPRB, 0.72234E+00_JPRB, 0.73865E+00_JPRB, 0.76487E+00_JPRB, 0.79230E+00_JPRB, 0.83700E+00_JPRB, 0.87892E+00_JPRB ,&
&  0.91158E+00_JPRB, 0.94309E+00_JPRB, 0.99019E+00_JPRB, 0.10542E+01_JPRB, 0.11513E+01_JPRB, 0.13327E+01_JPRB, 0.16505E+01_JPRB ,&
&  0.22433E+01_JPRB, 0.15405E+01_JPRB, 0.80018E+00_JPRB, 0.13835E+01_JPRB, 0.15163E+01_JPRB, 0.14828E+01_JPRB, 0.13405E+01_JPRB ,&
&  0.13560E+01_JPRB, 0.13530E+01_JPRB, 0.14129E+01_JPRB, 0.16681E+01_JPRB, 0.19904E+01_JPRB, 0.23781E+01_JPRB, 0.26097E+01_JPRB ,&
&  0.26676E+01_JPRB, 0.26927E+01_JPRB, 0.27129E+01_JPRB, 0.27646E+01_JPRB, 0.28485E+01_JPRB, 0.29283E+01_JPRB, 0.29453E+01_JPRB ,&
&  0.29741E+01_JPRB, 0.30339E+01_JPRB, 0.29554E+01_JPRB, 0.21809E+01_JPRB, 0.19990E+01_JPRB, 0.19216E+01_JPRB, 0.18219E+01_JPRB ,&
&  0.15891E+01_JPRB, 0.14119E+01_JPRB, 0.12677E+01_JPRB, 0.10744E+01_JPRB, 0.93252E+00_JPRB, 0.84933E+00_JPRB, 0.84492E+00_JPRB ,&
&  0.81765E+00_JPRB, 0.75863E+00_JPRB, 0.73229E+00_JPRB, 0.70888E+00_JPRB, 0.72638E+00_JPRB, 0.75266E+00_JPRB, 0.75194E+00_JPRB ,&
&  0.76868E+00_JPRB, 0.79709E+00_JPRB, 0.81850E+00_JPRB, 0.83905E+00_JPRB, 0.86554E+00_JPRB, 0.88787E+00_JPRB, 0.89937E+00_JPRB ,&
&  0.91232E+00_JPRB, 0.90806E+00_JPRB, 0.89962E+00_JPRB, 0.88421E+00_JPRB, 0.84519E+00_JPRB, 0.80978E+00_JPRB, 0.81864E+00_JPRB ,&
&  0.81485E+00_JPRB, 0.82078E+00_JPRB, 0.80978E+00_JPRB, 0.81893E+00_JPRB, 0.82000E+00_JPRB, 0.83827E+00_JPRB, 0.86641E+00_JPRB ,&
&  0.89179E+00_JPRB, 0.87304E+00_JPRB, 0.78983E+00_JPRB, 0.69594E+00_JPRB, 0.67214E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.68442E+00_JPRB, 0.67069E+00_JPRB, 0.63710E+00_JPRB, 0.56812E+00_JPRB, 0.54438E+00_JPRB, 0.55905E+00_JPRB, 0.62125E+00_JPRB ,&
&  0.64856E+00_JPRB, 0.65661E+00_JPRB, 0.64262E+00_JPRB, 0.62809E+00_JPRB, 0.62146E+00_JPRB, 0.62529E+00_JPRB, 0.64402E+00_JPRB ,&
&  0.66239E+00_JPRB, 0.65735E+00_JPRB, 0.65496E+00_JPRB, 0.66370E+00_JPRB, 0.68800E+00_JPRB, 0.72343E+00_JPRB, 0.77148E+00_JPRB ,&
&  0.80981E+00_JPRB, 0.82868E+00_JPRB, 0.84251E+00_JPRB, 0.88310E+00_JPRB, 0.93436E+00_JPRB, 0.95488E+00_JPRB, 0.10357E+01_JPRB ,&
&  0.14345E+01_JPRB, 0.20468E+01_JPRB, 0.93270E+00_JPRB, 0.14265E+01_JPRB, 0.16733E+01_JPRB, 0.15175E+01_JPRB, 0.13945E+01_JPRB ,&
&  0.13282E+01_JPRB, 0.11711E+01_JPRB, 0.11127E+01_JPRB, 0.12799E+01_JPRB, 0.14544E+01_JPRB, 0.16024E+01_JPRB, 0.18360E+01_JPRB ,&
&  0.20462E+01_JPRB, 0.21767E+01_JPRB, 0.22787E+01_JPRB, 0.23582E+01_JPRB, 0.24736E+01_JPRB, 0.26374E+01_JPRB, 0.27865E+01_JPRB ,&
&  0.28981E+01_JPRB, 0.30609E+01_JPRB, 0.31632E+01_JPRB, 0.25093E+01_JPRB, 0.23864E+01_JPRB, 0.25524E+01_JPRB, 0.24060E+01_JPRB ,&
&  0.20656E+01_JPRB, 0.18073E+01_JPRB, 0.15193E+01_JPRB, 0.12242E+01_JPRB, 0.10302E+01_JPRB, 0.95239E+00_JPRB, 0.93044E+00_JPRB ,&
&  0.88140E+00_JPRB, 0.82630E+00_JPRB, 0.81519E+00_JPRB, 0.81267E+00_JPRB, 0.81358E+00_JPRB, 0.82091E+00_JPRB, 0.83690E+00_JPRB ,&
&  0.86787E+00_JPRB, 0.89696E+00_JPRB, 0.91982E+00_JPRB, 0.93250E+00_JPRB, 0.94357E+00_JPRB, 0.95196E+00_JPRB, 0.94255E+00_JPRB ,&
&  0.94420E+00_JPRB, 0.93563E+00_JPRB, 0.92460E+00_JPRB, 0.90578E+00_JPRB, 0.88060E+00_JPRB, 0.86784E+00_JPRB, 0.86092E+00_JPRB ,&
&  0.85180E+00_JPRB, 0.83877E+00_JPRB, 0.81246E+00_JPRB, 0.79723E+00_JPRB, 0.82843E+00_JPRB, 0.81115E+00_JPRB, 0.85409E+00_JPRB ,&
&  0.91973E+00_JPRB, 0.96346E+00_JPRB, 0.91338E+00_JPRB, 0.71687E+00_JPRB, 0.69255E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.71120E+00_JPRB, 0.70323E+00_JPRB, 0.69297E+00_JPRB, 0.64547E+00_JPRB, 0.60110E+00_JPRB, 0.58060E+00_JPRB, 0.59524E+00_JPRB ,&
&  0.63008E+00_JPRB, 0.65411E+00_JPRB, 0.65409E+00_JPRB, 0.64438E+00_JPRB, 0.64279E+00_JPRB, 0.65058E+00_JPRB, 0.66210E+00_JPRB ,&
&  0.66472E+00_JPRB, 0.65608E+00_JPRB, 0.64644E+00_JPRB, 0.65149E+00_JPRB, 0.65494E+00_JPRB, 0.66630E+00_JPRB, 0.69451E+00_JPRB ,&
&  0.73243E+00_JPRB, 0.75170E+00_JPRB, 0.75459E+00_JPRB, 0.76412E+00_JPRB, 0.80243E+00_JPRB, 0.80906E+00_JPRB, 0.82420E+00_JPRB ,&
&  0.96630E+00_JPRB, 0.16779E+01_JPRB, 0.18752E+01_JPRB, 0.22414E+01_JPRB, 0.20288E+01_JPRB, 0.16833E+01_JPRB, 0.15693E+01_JPRB ,&
&  0.15113E+01_JPRB, 0.13038E+01_JPRB, 0.12037E+01_JPRB, 0.11001E+01_JPRB, 0.11815E+01_JPRB, 0.12668E+01_JPRB, 0.13055E+01_JPRB ,&
&  0.14163E+01_JPRB, 0.16368E+01_JPRB, 0.17710E+01_JPRB, 0.18997E+01_JPRB, 0.20200E+01_JPRB, 0.21648E+01_JPRB, 0.23796E+01_JPRB ,&
&  0.26376E+01_JPRB, 0.29728E+01_JPRB, 0.33779E+01_JPRB, 0.31749E+01_JPRB, 0.27815E+01_JPRB, 0.32081E+01_JPRB, 0.33331E+01_JPRB ,&
&  0.31591E+01_JPRB, 0.24205E+01_JPRB, 0.18549E+01_JPRB, 0.14458E+01_JPRB, 0.11785E+01_JPRB, 0.11900E+01_JPRB, 0.10406E+01_JPRB ,&
&  0.93779E+00_JPRB, 0.90816E+00_JPRB, 0.89050E+00_JPRB, 0.89726E+00_JPRB, 0.90080E+00_JPRB, 0.89204E+00_JPRB, 0.90227E+00_JPRB ,&
&  0.92838E+00_JPRB, 0.95064E+00_JPRB, 0.96701E+00_JPRB, 0.97808E+00_JPRB, 0.99277E+00_JPRB, 0.10011E+01_JPRB, 0.98833E+00_JPRB ,&
&  0.96401E+00_JPRB, 0.94734E+00_JPRB, 0.89354E+00_JPRB, 0.88814E+00_JPRB, 0.88619E+00_JPRB, 0.88862E+00_JPRB, 0.86255E+00_JPRB ,&
&  0.85670E+00_JPRB, 0.86102E+00_JPRB, 0.83817E+00_JPRB, 0.82074E+00_JPRB, 0.82953E+00_JPRB, 0.81993E+00_JPRB, 0.78820E+00_JPRB ,&
&  0.79003E+00_JPRB, 0.77630E+00_JPRB, 0.76290E+00_JPRB, 0.73873E+00_JPRB, 0.72243E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.75116E+00_JPRB, 0.74354E+00_JPRB, 0.74091E+00_JPRB, 0.71813E+00_JPRB, 0.68906E+00_JPRB, 0.66547E+00_JPRB, 0.65408E+00_JPRB ,&
&  0.65587E+00_JPRB, 0.66747E+00_JPRB, 0.67798E+00_JPRB, 0.66831E+00_JPRB, 0.66495E+00_JPRB, 0.67652E+00_JPRB, 0.68181E+00_JPRB ,&
&  0.68351E+00_JPRB, 0.68043E+00_JPRB, 0.67735E+00_JPRB, 0.67650E+00_JPRB, 0.67275E+00_JPRB, 0.66091E+00_JPRB, 0.66053E+00_JPRB ,&
&  0.67695E+00_JPRB, 0.69280E+00_JPRB, 0.69966E+00_JPRB, 0.70105E+00_JPRB, 0.72277E+00_JPRB, 0.73362E+00_JPRB, 0.74987E+00_JPRB ,&
&  0.81580E+00_JPRB, 0.14969E+01_JPRB, 0.14314E+01_JPRB, 0.25493E+01_JPRB, 0.21764E+01_JPRB, 0.18320E+01_JPRB, 0.15562E+01_JPRB ,&
&  0.14815E+01_JPRB, 0.15616E+01_JPRB, 0.13122E+01_JPRB, 0.11321E+01_JPRB, 0.10676E+01_JPRB, 0.10936E+01_JPRB, 0.10921E+01_JPRB ,&
&  0.10814E+01_JPRB, 0.11273E+01_JPRB, 0.12156E+01_JPRB, 0.13483E+01_JPRB, 0.15028E+01_JPRB, 0.16360E+01_JPRB, 0.17947E+01_JPRB ,&
&  0.19898E+01_JPRB, 0.23255E+01_JPRB, 0.28620E+01_JPRB, 0.29914E+01_JPRB, 0.30729E+01_JPRB, 0.36062E+01_JPRB, 0.41816E+01_JPRB ,&
&  0.41353E+01_JPRB, 0.33072E+01_JPRB, 0.23255E+01_JPRB, 0.17028E+01_JPRB, 0.14093E+01_JPRB, 0.13227E+01_JPRB, 0.11642E+01_JPRB ,&
&  0.10403E+01_JPRB, 0.98712E+00_JPRB, 0.96653E+00_JPRB, 0.95472E+00_JPRB, 0.95544E+00_JPRB, 0.95452E+00_JPRB, 0.95765E+00_JPRB ,&
&  0.97046E+00_JPRB, 0.99761E+00_JPRB, 0.10004E+01_JPRB, 0.99744E+00_JPRB, 0.10012E+01_JPRB, 0.10048E+01_JPRB, 0.99599E+00_JPRB ,&
&  0.98996E+00_JPRB, 0.97577E+00_JPRB, 0.90831E+00_JPRB, 0.87609E+00_JPRB, 0.87740E+00_JPRB, 0.87531E+00_JPRB, 0.87022E+00_JPRB ,&
&  0.88961E+00_JPRB, 0.88806E+00_JPRB, 0.85988E+00_JPRB, 0.81968E+00_JPRB, 0.82732E+00_JPRB, 0.84370E+00_JPRB, 0.79473E+00_JPRB ,&
&  0.78266E+00_JPRB, 0.77451E+00_JPRB, 0.77183E+00_JPRB, 0.76772E+00_JPRB, 0.75941E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.75818E+00_JPRB, 0.75076E+00_JPRB, 0.75034E+00_JPRB, 0.74050E+00_JPRB, 0.71965E+00_JPRB, 0.69920E+00_JPRB, 0.69105E+00_JPRB ,&
&  0.68462E+00_JPRB, 0.68637E+00_JPRB, 0.69444E+00_JPRB, 0.68718E+00_JPRB, 0.68963E+00_JPRB, 0.69724E+00_JPRB, 0.69975E+00_JPRB ,&
&  0.69328E+00_JPRB, 0.68941E+00_JPRB, 0.68332E+00_JPRB, 0.67899E+00_JPRB, 0.67696E+00_JPRB, 0.66622E+00_JPRB, 0.66020E+00_JPRB ,&
&  0.65977E+00_JPRB, 0.66612E+00_JPRB, 0.67239E+00_JPRB, 0.67508E+00_JPRB, 0.68813E+00_JPRB, 0.69608E+00_JPRB, 0.70632E+00_JPRB ,&
&  0.77322E+00_JPRB, 0.15522E+01_JPRB, 0.13153E+01_JPRB, 0.23029E+01_JPRB, 0.20399E+01_JPRB, 0.17012E+01_JPRB, 0.14208E+01_JPRB ,&
&  0.13969E+01_JPRB, 0.14239E+01_JPRB, 0.13080E+01_JPRB, 0.12040E+01_JPRB, 0.11202E+01_JPRB, 0.10545E+01_JPRB, 0.10130E+01_JPRB ,&
&  0.97493E+00_JPRB, 0.95427E+00_JPRB, 0.95902E+00_JPRB, 0.99346E+00_JPRB, 0.10848E+01_JPRB, 0.12005E+01_JPRB, 0.13092E+01_JPRB ,&
&  0.14285E+01_JPRB, 0.16253E+01_JPRB, 0.20011E+01_JPRB, 0.26160E+01_JPRB, 0.28057E+01_JPRB, 0.32636E+01_JPRB, 0.49107E+01_JPRB ,&
&  0.50874E+01_JPRB, 0.43693E+01_JPRB, 0.29687E+01_JPRB, 0.19910E+01_JPRB, 0.16056E+01_JPRB, 0.14503E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.11666E+01_JPRB, 0.10718E+01_JPRB, 0.10452E+01_JPRB, 0.10027E+01_JPRB, 0.97520E+00_JPRB, 0.99005E+00_JPRB, 0.10201E+01_JPRB ,&
&  0.10385E+01_JPRB, 0.10288E+01_JPRB, 0.10331E+01_JPRB, 0.10103E+01_JPRB, 0.99088E+00_JPRB, 0.97219E+00_JPRB, 0.96974E+00_JPRB ,&
&  0.97554E+00_JPRB, 0.98043E+00_JPRB, 0.93803E+00_JPRB, 0.89558E+00_JPRB, 0.88682E+00_JPRB, 0.88786E+00_JPRB, 0.87856E+00_JPRB ,&
&  0.90936E+00_JPRB, 0.90392E+00_JPRB, 0.87352E+00_JPRB, 0.84937E+00_JPRB, 0.82538E+00_JPRB, 0.84328E+00_JPRB, 0.78848E+00_JPRB ,&
&  0.76803E+00_JPRB, 0.76097E+00_JPRB, 0.76222E+00_JPRB, 0.76970E+00_JPRB, 0.77137E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.76930E+00_JPRB, 0.75798E+00_JPRB, 0.75632E+00_JPRB, 0.75281E+00_JPRB, 0.74237E+00_JPRB, 0.72948E+00_JPRB, 0.72958E+00_JPRB ,&
&  0.73004E+00_JPRB, 0.72928E+00_JPRB, 0.71942E+00_JPRB, 0.70319E+00_JPRB, 0.70166E+00_JPRB, 0.69973E+00_JPRB, 0.69623E+00_JPRB ,&
&  0.69186E+00_JPRB, 0.69044E+00_JPRB, 0.68068E+00_JPRB, 0.67535E+00_JPRB, 0.67737E+00_JPRB, 0.67421E+00_JPRB, 0.66498E+00_JPRB ,&
&  0.65529E+00_JPRB, 0.65041E+00_JPRB, 0.65704E+00_JPRB, 0.66406E+00_JPRB, 0.67968E+00_JPRB, 0.68404E+00_JPRB, 0.69184E+00_JPRB ,&
&  0.77575E+00_JPRB, 0.72688E+00_JPRB, 0.15385E+01_JPRB, 0.19800E+01_JPRB, 0.19233E+01_JPRB, 0.16576E+01_JPRB, 0.14685E+01_JPRB ,&
&  0.14187E+01_JPRB, 0.13345E+01_JPRB, 0.12737E+01_JPRB, 0.11825E+01_JPRB, 0.10897E+01_JPRB, 0.99420E+00_JPRB, 0.95558E+00_JPRB ,&
&  0.93740E+00_JPRB, 0.91605E+00_JPRB, 0.90932E+00_JPRB, 0.91165E+00_JPRB, 0.92772E+00_JPRB, 0.96099E+00_JPRB, 0.10220E+01_JPRB ,&
&  0.10869E+01_JPRB, 0.11664E+01_JPRB, 0.13348E+01_JPRB, 0.18042E+01_JPRB, 0.25339E+01_JPRB, 0.31204E+01_JPRB, 0.43578E+01_JPRB ,&
&  0.54850E+01_JPRB, 0.50862E+01_JPRB, 0.35549E+01_JPRB, 0.23447E+01_JPRB, 0.18070E+01_JPRB, 0.15951E+01_JPRB, 0.14340E+01_JPRB ,&
&  0.13034E+01_JPRB, 0.11759E+01_JPRB, 0.10992E+01_JPRB, 0.10425E+01_JPRB, 0.10002E+01_JPRB, 0.98149E+00_JPRB, 0.10228E+01_JPRB ,&
&  0.10649E+01_JPRB, 0.10742E+01_JPRB, 0.10471E+01_JPRB, 0.10162E+01_JPRB, 0.99072E+00_JPRB, 0.96873E+00_JPRB, 0.96442E+00_JPRB ,&
&  0.97020E+00_JPRB, 0.97957E+00_JPRB, 0.95379E+00_JPRB, 0.92386E+00_JPRB, 0.91796E+00_JPRB, 0.92930E+00_JPRB, 0.93144E+00_JPRB ,&
&  0.91810E+00_JPRB, 0.90255E+00_JPRB, 0.87076E+00_JPRB, 0.85760E+00_JPRB, 0.81097E+00_JPRB, 0.83516E+00_JPRB, 0.78465E+00_JPRB ,&
&  0.76082E+00_JPRB, 0.75656E+00_JPRB, 0.76310E+00_JPRB, 0.76907E+00_JPRB, 0.78677E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.77938E+00_JPRB, 0.75901E+00_JPRB, 0.75360E+00_JPRB, 0.75892E+00_JPRB, 0.75747E+00_JPRB, 0.76213E+00_JPRB, 0.76623E+00_JPRB ,&
&  0.76157E+00_JPRB, 0.74414E+00_JPRB, 0.72863E+00_JPRB, 0.71174E+00_JPRB, 0.70605E+00_JPRB, 0.69867E+00_JPRB, 0.69727E+00_JPRB ,&
&  0.69010E+00_JPRB, 0.68573E+00_JPRB, 0.67823E+00_JPRB, 0.67627E+00_JPRB, 0.67648E+00_JPRB, 0.67134E+00_JPRB, 0.65117E+00_JPRB ,&
&  0.64560E+00_JPRB, 0.64502E+00_JPRB, 0.65430E+00_JPRB, 0.66659E+00_JPRB, 0.68180E+00_JPRB, 0.68910E+00_JPRB, 0.69527E+00_JPRB ,&
&  0.76831E+00_JPRB, 0.96966E+00_JPRB, 0.14998E+01_JPRB, 0.16147E+01_JPRB, 0.16242E+01_JPRB, 0.14921E+01_JPRB, 0.13838E+01_JPRB ,&
&  0.12926E+01_JPRB, 0.12524E+01_JPRB, 0.12448E+01_JPRB, 0.11514E+01_JPRB, 0.10181E+01_JPRB, 0.91525E+00_JPRB, 0.88146E+00_JPRB ,&
&  0.86635E+00_JPRB, 0.85506E+00_JPRB, 0.85081E+00_JPRB, 0.85417E+00_JPRB, 0.85418E+00_JPRB, 0.85786E+00_JPRB, 0.87724E+00_JPRB ,&
&  0.90352E+00_JPRB, 0.94095E+00_JPRB, 0.10072E+01_JPRB, 0.12481E+01_JPRB, 0.21043E+01_JPRB, 0.24143E+01_JPRB, 0.37653E+01_JPRB ,&
&  0.55366E+01_JPRB, 0.49315E+01_JPRB, 0.37597E+01_JPRB, 0.27010E+01_JPRB, 0.21060E+01_JPRB, 0.18204E+01_JPRB, 0.16054E+01_JPRB ,&
&  0.14372E+01_JPRB, 0.12965E+01_JPRB, 0.11800E+01_JPRB, 0.11136E+01_JPRB, 0.10561E+01_JPRB, 0.10118E+01_JPRB, 0.10093E+01_JPRB ,&
&  0.10395E+01_JPRB, 0.10604E+01_JPRB, 0.10464E+01_JPRB, 0.10160E+01_JPRB, 0.99132E+00_JPRB, 0.98698E+00_JPRB, 0.98382E+00_JPRB ,&
&  0.98577E+00_JPRB, 0.98523E+00_JPRB, 0.96591E+00_JPRB, 0.92989E+00_JPRB, 0.92896E+00_JPRB, 0.95056E+00_JPRB, 0.94900E+00_JPRB ,&
&  0.90983E+00_JPRB, 0.87074E+00_JPRB, 0.87462E+00_JPRB, 0.84718E+00_JPRB, 0.82162E+00_JPRB, 0.84332E+00_JPRB, 0.78529E+00_JPRB ,&
&  0.75333E+00_JPRB, 0.74752E+00_JPRB, 0.76489E+00_JPRB, 0.78510E+00_JPRB, 0.81771E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.80597E+00_JPRB, 0.76796E+00_JPRB, 0.74854E+00_JPRB, 0.75004E+00_JPRB, 0.75908E+00_JPRB, 0.75925E+00_JPRB, 0.76443E+00_JPRB ,&
&  0.75768E+00_JPRB, 0.74250E+00_JPRB, 0.72524E+00_JPRB, 0.70959E+00_JPRB, 0.70156E+00_JPRB, 0.69594E+00_JPRB, 0.69177E+00_JPRB ,&
&  0.68847E+00_JPRB, 0.68506E+00_JPRB, 0.67883E+00_JPRB, 0.68211E+00_JPRB, 0.68432E+00_JPRB, 0.68265E+00_JPRB, 0.66045E+00_JPRB ,&
&  0.64925E+00_JPRB, 0.65103E+00_JPRB, 0.65753E+00_JPRB, 0.66333E+00_JPRB, 0.67273E+00_JPRB, 0.68562E+00_JPRB, 0.69935E+00_JPRB ,&
&  0.73611E+00_JPRB, 0.92825E+00_JPRB, 0.12206E+01_JPRB, 0.14175E+01_JPRB, 0.14752E+01_JPRB, 0.14291E+01_JPRB, 0.13293E+01_JPRB ,&
&  0.12488E+01_JPRB, 0.12000E+01_JPRB, 0.11531E+01_JPRB, 0.10396E+01_JPRB, 0.92385E+00_JPRB, 0.86236E+00_JPRB, 0.83278E+00_JPRB ,&
&  0.82111E+00_JPRB, 0.81999E+00_JPRB, 0.81721E+00_JPRB, 0.81504E+00_JPRB, 0.81214E+00_JPRB, 0.81347E+00_JPRB, 0.81967E+00_JPRB ,&
&  0.82992E+00_JPRB, 0.84003E+00_JPRB, 0.86545E+00_JPRB, 0.91474E+00_JPRB, 0.13187E+01_JPRB, 0.18959E+01_JPRB, 0.28007E+01_JPRB ,&
&  0.32354E+01_JPRB, 0.33145E+01_JPRB, 0.30082E+01_JPRB, 0.25980E+01_JPRB, 0.22408E+01_JPRB, 0.19619E+01_JPRB, 0.17232E+01_JPRB ,&
&  0.15127E+01_JPRB, 0.13285E+01_JPRB, 0.11806E+01_JPRB, 0.10899E+01_JPRB, 0.10371E+01_JPRB, 0.99275E+00_JPRB, 0.97157E+00_JPRB ,&
&  0.98078E+00_JPRB, 0.99623E+00_JPRB, 0.10020E+01_JPRB, 0.99959E+00_JPRB, 0.98544E+00_JPRB, 0.97130E+00_JPRB, 0.96311E+00_JPRB ,&
&  0.96508E+00_JPRB, 0.96185E+00_JPRB, 0.96596E+00_JPRB, 0.94990E+00_JPRB, 0.93252E+00_JPRB, 0.91852E+00_JPRB, 0.91159E+00_JPRB ,&
&  0.91029E+00_JPRB, 0.87921E+00_JPRB, 0.86227E+00_JPRB, 0.84933E+00_JPRB, 0.82312E+00_JPRB, 0.84212E+00_JPRB, 0.78616E+00_JPRB ,&
&  0.75648E+00_JPRB, 0.74993E+00_JPRB, 0.75454E+00_JPRB, 0.77023E+00_JPRB, 0.81078E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.82908E+00_JPRB, 0.80031E+00_JPRB, 0.77157E+00_JPRB, 0.75477E+00_JPRB, 0.74942E+00_JPRB, 0.74886E+00_JPRB, 0.74947E+00_JPRB ,&
&  0.74692E+00_JPRB, 0.74103E+00_JPRB, 0.72284E+00_JPRB, 0.70882E+00_JPRB, 0.69896E+00_JPRB, 0.69579E+00_JPRB, 0.69298E+00_JPRB ,&
&  0.69245E+00_JPRB, 0.69082E+00_JPRB, 0.68567E+00_JPRB, 0.68449E+00_JPRB, 0.68600E+00_JPRB, 0.68086E+00_JPRB, 0.66377E+00_JPRB ,&
&  0.64611E+00_JPRB, 0.64743E+00_JPRB, 0.65363E+00_JPRB, 0.65838E+00_JPRB, 0.66693E+00_JPRB, 0.67762E+00_JPRB, 0.69532E+00_JPRB ,&
&  0.73214E+00_JPRB, 0.80257E+00_JPRB, 0.10615E+01_JPRB, 0.12186E+01_JPRB, 0.12904E+01_JPRB, 0.13133E+01_JPRB, 0.12534E+01_JPRB ,&
&  0.11765E+01_JPRB, 0.10835E+01_JPRB, 0.10079E+01_JPRB, 0.92771E+00_JPRB, 0.87364E+00_JPRB, 0.83491E+00_JPRB, 0.80958E+00_JPRB ,&
&  0.79731E+00_JPRB, 0.79333E+00_JPRB, 0.79010E+00_JPRB, 0.78552E+00_JPRB, 0.78303E+00_JPRB, 0.78257E+00_JPRB, 0.78386E+00_JPRB ,&
&  0.78791E+00_JPRB, 0.79675E+00_JPRB, 0.80838E+00_JPRB, 0.82582E+00_JPRB, 0.88507E+00_JPRB, 0.11080E+01_JPRB, 0.15949E+01_JPRB ,&
&  0.20358E+01_JPRB, 0.22913E+01_JPRB, 0.22529E+01_JPRB, 0.21643E+01_JPRB, 0.20836E+01_JPRB, 0.19099E+01_JPRB, 0.17255E+01_JPRB ,&
&  0.15425E+01_JPRB, 0.13721E+01_JPRB, 0.11996E+01_JPRB, 0.10640E+01_JPRB, 0.97932E+00_JPRB, 0.93245E+00_JPRB, 0.90846E+00_JPRB ,&
&  0.91007E+00_JPRB, 0.92939E+00_JPRB, 0.94146E+00_JPRB, 0.95497E+00_JPRB, 0.95306E+00_JPRB, 0.94027E+00_JPRB, 0.92428E+00_JPRB ,&
&  0.90584E+00_JPRB, 0.88646E+00_JPRB, 0.88031E+00_JPRB, 0.87062E+00_JPRB, 0.86090E+00_JPRB, 0.85833E+00_JPRB, 0.85615E+00_JPRB ,&
&  0.85557E+00_JPRB, 0.86400E+00_JPRB, 0.82898E+00_JPRB, 0.81644E+00_JPRB, 0.87532E+00_JPRB, 0.83885E+00_JPRB, 0.79944E+00_JPRB ,&
&  0.76907E+00_JPRB, 0.75749E+00_JPRB, 0.75644E+00_JPRB, 0.77295E+00_JPRB, 0.81746E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.74231E+00_JPRB, 0.74221E+00_JPRB, 0.73993E+00_JPRB, 0.74129E+00_JPRB, 0.74121E+00_JPRB, 0.73322E+00_JPRB, 0.72716E+00_JPRB ,&
&  0.72644E+00_JPRB, 0.72328E+00_JPRB, 0.71915E+00_JPRB, 0.71105E+00_JPRB, 0.69956E+00_JPRB, 0.69115E+00_JPRB, 0.68966E+00_JPRB ,&
&  0.68584E+00_JPRB, 0.68426E+00_JPRB, 0.68357E+00_JPRB, 0.67908E+00_JPRB, 0.67317E+00_JPRB, 0.66412E+00_JPRB, 0.64918E+00_JPRB ,&
&  0.63042E+00_JPRB, 0.62445E+00_JPRB, 0.63361E+00_JPRB, 0.64187E+00_JPRB, 0.64779E+00_JPRB, 0.65114E+00_JPRB, 0.66396E+00_JPRB ,&
&  0.71160E+00_JPRB, 0.73556E+00_JPRB, 0.84210E+00_JPRB, 0.99333E+00_JPRB, 0.10808E+01_JPRB, 0.10675E+01_JPRB, 0.10567E+01_JPRB ,&
&  0.10207E+01_JPRB, 0.97954E+00_JPRB, 0.93449E+00_JPRB, 0.88206E+00_JPRB, 0.83685E+00_JPRB, 0.80574E+00_JPRB, 0.79104E+00_JPRB ,&
&  0.78196E+00_JPRB, 0.77384E+00_JPRB, 0.77011E+00_JPRB, 0.76143E+00_JPRB, 0.75415E+00_JPRB, 0.74875E+00_JPRB, 0.75067E+00_JPRB ,&
&  0.75431E+00_JPRB, 0.75935E+00_JPRB, 0.76425E+00_JPRB, 0.77180E+00_JPRB, 0.78932E+00_JPRB, 0.85382E+00_JPRB, 0.10701E+01_JPRB ,&
&  0.13588E+01_JPRB, 0.16296E+01_JPRB, 0.17562E+01_JPRB, 0.17291E+01_JPRB, 0.16914E+01_JPRB, 0.16360E+01_JPRB, 0.15037E+01_JPRB ,&
&  0.13373E+01_JPRB, 0.12267E+01_JPRB, 0.11427E+01_JPRB, 0.10693E+01_JPRB, 0.10152E+01_JPRB, 0.97135E+00_JPRB, 0.93776E+00_JPRB ,&
&  0.91781E+00_JPRB, 0.91272E+00_JPRB, 0.91105E+00_JPRB, 0.91011E+00_JPRB, 0.90356E+00_JPRB, 0.88290E+00_JPRB, 0.85125E+00_JPRB ,&
&  0.81404E+00_JPRB, 0.79637E+00_JPRB, 0.79531E+00_JPRB, 0.78764E+00_JPRB, 0.78134E+00_JPRB, 0.78613E+00_JPRB, 0.79787E+00_JPRB ,&
&  0.80248E+00_JPRB, 0.81217E+00_JPRB, 0.82065E+00_JPRB, 0.84740E+00_JPRB, 0.84194E+00_JPRB, 0.82036E+00_JPRB, 0.80679E+00_JPRB ,&
&  0.78354E+00_JPRB, 0.76162E+00_JPRB, 0.74662E+00_JPRB, 0.74815E+00_JPRB, 0.70475E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.72988E+00_JPRB, 0.72793E+00_JPRB, 0.72274E+00_JPRB, 0.72265E+00_JPRB, 0.72480E+00_JPRB, 0.71396E+00_JPRB, 0.70396E+00_JPRB ,&
&  0.69894E+00_JPRB, 0.70051E+00_JPRB, 0.70439E+00_JPRB, 0.70578E+00_JPRB, 0.70009E+00_JPRB, 0.68918E+00_JPRB, 0.68257E+00_JPRB ,&
&  0.67311E+00_JPRB, 0.66477E+00_JPRB, 0.66272E+00_JPRB, 0.66188E+00_JPRB, 0.65598E+00_JPRB, 0.64825E+00_JPRB, 0.64162E+00_JPRB ,&
&  0.62766E+00_JPRB, 0.61829E+00_JPRB, 0.61643E+00_JPRB, 0.62009E+00_JPRB, 0.62434E+00_JPRB, 0.62380E+00_JPRB, 0.64302E+00_JPRB ,&
&  0.70114E+00_JPRB, 0.65404E+00_JPRB, 0.66048E+00_JPRB, 0.78175E+00_JPRB, 0.86596E+00_JPRB, 0.88313E+00_JPRB, 0.90325E+00_JPRB ,&
&  0.92675E+00_JPRB, 0.91900E+00_JPRB, 0.89537E+00_JPRB, 0.84590E+00_JPRB, 0.80691E+00_JPRB, 0.77972E+00_JPRB, 0.75630E+00_JPRB ,&
&  0.73862E+00_JPRB, 0.72406E+00_JPRB, 0.71715E+00_JPRB, 0.70782E+00_JPRB, 0.70375E+00_JPRB, 0.69418E+00_JPRB, 0.69109E+00_JPRB ,&
&  0.69246E+00_JPRB, 0.69956E+00_JPRB, 0.70761E+00_JPRB, 0.71416E+00_JPRB, 0.72821E+00_JPRB, 0.75733E+00_JPRB, 0.84021E+00_JPRB ,&
&  0.99397E+00_JPRB, 0.12093E+01_JPRB, 0.13642E+01_JPRB, 0.14153E+01_JPRB, 0.13966E+01_JPRB, 0.13377E+01_JPRB, 0.12487E+01_JPRB ,&
&  0.11503E+01_JPRB, 0.10696E+01_JPRB, 0.10117E+01_JPRB, 0.98041E+00_JPRB, 0.95732E+00_JPRB, 0.93826E+00_JPRB, 0.91906E+00_JPRB ,&
&  0.90232E+00_JPRB, 0.89114E+00_JPRB, 0.88284E+00_JPRB, 0.87197E+00_JPRB, 0.85698E+00_JPRB, 0.82641E+00_JPRB, 0.78307E+00_JPRB ,&
&  0.74921E+00_JPRB, 0.72768E+00_JPRB, 0.72150E+00_JPRB, 0.71676E+00_JPRB, 0.71642E+00_JPRB, 0.72964E+00_JPRB, 0.74709E+00_JPRB ,&
&  0.75643E+00_JPRB, 0.76758E+00_JPRB, 0.79008E+00_JPRB, 0.76771E+00_JPRB, 0.80262E+00_JPRB, 0.80228E+00_JPRB, 0.80294E+00_JPRB ,&
&  0.79165E+00_JPRB, 0.77483E+00_JPRB, 0.76150E+00_JPRB, 0.65266E+00_JPRB, 0.71174E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.71829E+00_JPRB, 0.71534E+00_JPRB, 0.70373E+00_JPRB, 0.69642E+00_JPRB, 0.69146E+00_JPRB, 0.68565E+00_JPRB, 0.67579E+00_JPRB ,&
&  0.67018E+00_JPRB, 0.66966E+00_JPRB, 0.66932E+00_JPRB, 0.67026E+00_JPRB, 0.67086E+00_JPRB, 0.66557E+00_JPRB, 0.65702E+00_JPRB ,&
&  0.64828E+00_JPRB, 0.64027E+00_JPRB, 0.64165E+00_JPRB, 0.64075E+00_JPRB, 0.63525E+00_JPRB, 0.63075E+00_JPRB, 0.62492E+00_JPRB ,&
&  0.61038E+00_JPRB, 0.59558E+00_JPRB, 0.58409E+00_JPRB, 0.58720E+00_JPRB, 0.59400E+00_JPRB, 0.58948E+00_JPRB, 0.60616E+00_JPRB ,&
&  0.66651E+00_JPRB, 0.59737E+00_JPRB, 0.62513E+00_JPRB, 0.69319E+00_JPRB, 0.72811E+00_JPRB, 0.76219E+00_JPRB, 0.78687E+00_JPRB ,&
&  0.80462E+00_JPRB, 0.81512E+00_JPRB, 0.78080E+00_JPRB, 0.74009E+00_JPRB, 0.70372E+00_JPRB, 0.67423E+00_JPRB, 0.65332E+00_JPRB ,&
&  0.64289E+00_JPRB, 0.64267E+00_JPRB, 0.64314E+00_JPRB, 0.63859E+00_JPRB, 0.63476E+00_JPRB, 0.62940E+00_JPRB, 0.62625E+00_JPRB ,&
&  0.62517E+00_JPRB, 0.62770E+00_JPRB, 0.63304E+00_JPRB, 0.64516E+00_JPRB, 0.65834E+00_JPRB, 0.67398E+00_JPRB, 0.70259E+00_JPRB ,&
&  0.76187E+00_JPRB, 0.87281E+00_JPRB, 0.99821E+00_JPRB, 0.10696E+01_JPRB, 0.10761E+01_JPRB, 0.10423E+01_JPRB, 0.98525E+00_JPRB ,&
&  0.92777E+00_JPRB, 0.88453E+00_JPRB, 0.85818E+00_JPRB, 0.84175E+00_JPRB, 0.82566E+00_JPRB, 0.81182E+00_JPRB, 0.80075E+00_JPRB ,&
&  0.79150E+00_JPRB, 0.77986E+00_JPRB, 0.77321E+00_JPRB, 0.76316E+00_JPRB, 0.74661E+00_JPRB, 0.72207E+00_JPRB, 0.69759E+00_JPRB ,&
&  0.67453E+00_JPRB, 0.65656E+00_JPRB, 0.64696E+00_JPRB, 0.64735E+00_JPRB, 0.65206E+00_JPRB, 0.66502E+00_JPRB, 0.68391E+00_JPRB ,&
&  0.69848E+00_JPRB, 0.70973E+00_JPRB, 0.72213E+00_JPRB, 0.74665E+00_JPRB, 0.76298E+00_JPRB, 0.77825E+00_JPRB, 0.78608E+00_JPRB ,&
&  0.78604E+00_JPRB, 0.78601E+00_JPRB, 0.62863E+00_JPRB, 0.69290E+00_JPRB, 0.70983E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.71717E+00_JPRB, 0.70009E+00_JPRB, 0.67954E+00_JPRB, 0.66552E+00_JPRB, 0.65851E+00_JPRB, 0.65610E+00_JPRB, 0.65269E+00_JPRB ,&
&  0.64732E+00_JPRB, 0.64736E+00_JPRB, 0.64834E+00_JPRB, 0.64921E+00_JPRB, 0.64955E+00_JPRB, 0.64612E+00_JPRB, 0.63803E+00_JPRB ,&
&  0.62874E+00_JPRB, 0.61706E+00_JPRB, 0.61524E+00_JPRB, 0.61451E+00_JPRB, 0.60808E+00_JPRB, 0.60350E+00_JPRB, 0.60282E+00_JPRB ,&
&  0.59625E+00_JPRB, 0.58601E+00_JPRB, 0.56767E+00_JPRB, 0.56196E+00_JPRB, 0.56491E+00_JPRB, 0.56518E+00_JPRB, 0.57243E+00_JPRB ,&
&  0.55766E+00_JPRB, 0.53724E+00_JPRB, 0.59134E+00_JPRB, 0.62629E+00_JPRB, 0.64210E+00_JPRB, 0.65701E+00_JPRB, 0.67358E+00_JPRB ,&
&  0.66879E+00_JPRB, 0.67010E+00_JPRB, 0.65202E+00_JPRB, 0.62338E+00_JPRB, 0.60374E+00_JPRB, 0.59423E+00_JPRB, 0.59087E+00_JPRB ,&
&  0.59504E+00_JPRB, 0.59834E+00_JPRB, 0.59877E+00_JPRB, 0.59819E+00_JPRB, 0.59897E+00_JPRB, 0.59571E+00_JPRB, 0.59364E+00_JPRB ,&
&  0.59522E+00_JPRB, 0.59885E+00_JPRB, 0.60092E+00_JPRB, 0.60539E+00_JPRB, 0.61471E+00_JPRB, 0.62876E+00_JPRB, 0.64522E+00_JPRB ,&
&  0.66532E+00_JPRB, 0.69966E+00_JPRB, 0.75925E+00_JPRB, 0.81131E+00_JPRB, 0.82984E+00_JPRB, 0.81392E+00_JPRB, 0.78334E+00_JPRB ,&
&  0.75729E+00_JPRB, 0.74158E+00_JPRB, 0.73089E+00_JPRB, 0.72114E+00_JPRB, 0.70708E+00_JPRB, 0.69370E+00_JPRB, 0.68077E+00_JPRB ,&
&  0.67408E+00_JPRB, 0.66416E+00_JPRB, 0.65436E+00_JPRB, 0.64315E+00_JPRB, 0.63361E+00_JPRB, 0.62809E+00_JPRB, 0.62473E+00_JPRB ,&
&  0.62049E+00_JPRB, 0.61233E+00_JPRB, 0.60722E+00_JPRB, 0.60902E+00_JPRB, 0.61230E+00_JPRB, 0.61978E+00_JPRB, 0.63132E+00_JPRB ,&
&  0.64565E+00_JPRB, 0.65892E+00_JPRB, 0.67318E+00_JPRB, 0.69982E+00_JPRB, 0.72657E+00_JPRB, 0.74986E+00_JPRB, 0.76372E+00_JPRB ,&
&  0.77210E+00_JPRB, 0.76510E+00_JPRB, 0.76107E+00_JPRB, 0.74767E+00_JPRB, 0.73099E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.66104E+00_JPRB, 0.64028E+00_JPRB, 0.62397E+00_JPRB, 0.61328E+00_JPRB, 0.60739E+00_JPRB, 0.60654E+00_JPRB, 0.60398E+00_JPRB ,&
&  0.60218E+00_JPRB, 0.59888E+00_JPRB, 0.59681E+00_JPRB, 0.59942E+00_JPRB, 0.60079E+00_JPRB, 0.60137E+00_JPRB, 0.60245E+00_JPRB ,&
&  0.59672E+00_JPRB, 0.58378E+00_JPRB, 0.57351E+00_JPRB, 0.57067E+00_JPRB, 0.57154E+00_JPRB, 0.56743E+00_JPRB, 0.56594E+00_JPRB ,&
&  0.56898E+00_JPRB, 0.56102E+00_JPRB, 0.54762E+00_JPRB, 0.53897E+00_JPRB, 0.53956E+00_JPRB, 0.53720E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.55717E+00_JPRB, 0.50907E+00_JPRB, 0.55759E+00_JPRB, 0.56556E+00_JPRB, 0.57181E+00_JPRB, 0.57545E+00_JPRB, 0.57521E+00_JPRB ,&
&  0.57080E+00_JPRB, 0.56438E+00_JPRB, 0.55774E+00_JPRB, 0.55123E+00_JPRB, 0.54683E+00_JPRB, 0.54165E+00_JPRB, 0.53731E+00_JPRB ,&
&  0.53808E+00_JPRB, 0.53946E+00_JPRB, 0.54431E+00_JPRB, 0.55016E+00_JPRB, 0.55789E+00_JPRB, 0.56066E+00_JPRB, 0.56310E+00_JPRB ,&
&  0.56590E+00_JPRB, 0.56758E+00_JPRB, 0.56748E+00_JPRB, 0.56633E+00_JPRB, 0.56490E+00_JPRB, 0.56944E+00_JPRB, 0.57617E+00_JPRB ,&
&  0.58352E+00_JPRB, 0.59868E+00_JPRB, 0.62386E+00_JPRB, 0.64740E+00_JPRB, 0.65817E+00_JPRB, 0.65472E+00_JPRB, 0.64251E+00_JPRB ,&
&  0.63628E+00_JPRB, 0.63803E+00_JPRB, 0.64082E+00_JPRB, 0.63683E+00_JPRB, 0.62927E+00_JPRB, 0.62201E+00_JPRB, 0.61033E+00_JPRB ,&
&  0.59499E+00_JPRB, 0.58158E+00_JPRB, 0.57260E+00_JPRB, 0.56737E+00_JPRB, 0.56544E+00_JPRB, 0.56288E+00_JPRB, 0.56023E+00_JPRB ,&
&  0.55991E+00_JPRB, 0.55836E+00_JPRB, 0.55866E+00_JPRB, 0.56290E+00_JPRB, 0.56686E+00_JPRB, 0.57279E+00_JPRB, 0.58044E+00_JPRB ,&
&  0.59104E+00_JPRB, 0.60263E+00_JPRB, 0.61397E+00_JPRB, 0.63247E+00_JPRB, 0.65881E+00_JPRB, 0.67791E+00_JPRB, 0.69027E+00_JPRB ,&
&  0.70169E+00_JPRB, 0.70457E+00_JPRB, 0.69860E+00_JPRB, 0.68963E+00_JPRB, 0.67676E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.61504E+00_JPRB, 0.60203E+00_JPRB, 0.59195E+00_JPRB, 0.58439E+00_JPRB, 0.57962E+00_JPRB, 0.57872E+00_JPRB, 0.57665E+00_JPRB ,&
&  0.57462E+00_JPRB, 0.57288E+00_JPRB, 0.56657E+00_JPRB, 0.56022E+00_JPRB, 0.56004E+00_JPRB, 0.56592E+00_JPRB, 0.57050E+00_JPRB ,&
&  0.56758E+00_JPRB, 0.55704E+00_JPRB, 0.54716E+00_JPRB, 0.54507E+00_JPRB, 0.54791E+00_JPRB, 0.55029E+00_JPRB, 0.54991E+00_JPRB ,&
&  0.55160E+00_JPRB, 0.55268E+00_JPRB, 0.54446E+00_JPRB, 0.52762E+00_JPRB, 0.51987E+00_JPRB, 0.51541E+00_JPRB, 0.52006E+00_JPRB ,&
&  0.52815E+00_JPRB, 0.52408E+00_JPRB, 0.53578E+00_JPRB, 0.53322E+00_JPRB, 0.53215E+00_JPRB, 0.52830E+00_JPRB, 0.52149E+00_JPRB ,&
&  0.51600E+00_JPRB, 0.51037E+00_JPRB, 0.50680E+00_JPRB, 0.50505E+00_JPRB, 0.50520E+00_JPRB, 0.50632E+00_JPRB, 0.51059E+00_JPRB ,&
&  0.51145E+00_JPRB, 0.50953E+00_JPRB, 0.51206E+00_JPRB, 0.51912E+00_JPRB, 0.52795E+00_JPRB, 0.53307E+00_JPRB, 0.53773E+00_JPRB ,&
&  0.54155E+00_JPRB, 0.54368E+00_JPRB, 0.54653E+00_JPRB, 0.54725E+00_JPRB, 0.54349E+00_JPRB, 0.54261E+00_JPRB, 0.54236E+00_JPRB ,&
&  0.54480E+00_JPRB, 0.55060E+00_JPRB, 0.56277E+00_JPRB, 0.57787E+00_JPRB, 0.58415E+00_JPRB, 0.57927E+00_JPRB, 0.57270E+00_JPRB ,&
&  0.57289E+00_JPRB, 0.57938E+00_JPRB, 0.58673E+00_JPRB, 0.59139E+00_JPRB, 0.59154E+00_JPRB, 0.58409E+00_JPRB, 0.57423E+00_JPRB ,&
&  0.56339E+00_JPRB, 0.55218E+00_JPRB, 0.54203E+00_JPRB, 0.53535E+00_JPRB, 0.52998E+00_JPRB, 0.52579E+00_JPRB, 0.52310E+00_JPRB ,&
&  0.52189E+00_JPRB, 0.52166E+00_JPRB, 0.52458E+00_JPRB, 0.53086E+00_JPRB, 0.53459E+00_JPRB, 0.54307E+00_JPRB, 0.55364E+00_JPRB ,&
&  0.56435E+00_JPRB, 0.57421E+00_JPRB, 0.58583E+00_JPRB, 0.60260E+00_JPRB, 0.62161E+00_JPRB, 0.63555E+00_JPRB, 0.64658E+00_JPRB ,&
&  0.65369E+00_JPRB, 0.65288E+00_JPRB, 0.64523E+00_JPRB, 0.63653E+00_JPRB, 0.62715E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.57019E+00_JPRB, 0.56242E+00_JPRB, 0.55628E+00_JPRB, 0.55111E+00_JPRB, 0.54685E+00_JPRB, 0.54541E+00_JPRB, 0.54376E+00_JPRB ,&
&  0.54199E+00_JPRB, 0.53952E+00_JPRB, 0.53814E+00_JPRB, 0.53489E+00_JPRB, 0.53214E+00_JPRB, 0.53519E+00_JPRB, 0.54084E+00_JPRB ,&
&  0.53948E+00_JPRB, 0.53271E+00_JPRB, 0.52741E+00_JPRB, 0.52739E+00_JPRB, 0.53081E+00_JPRB, 0.53336E+00_JPRB, 0.53624E+00_JPRB ,&
&  0.53603E+00_JPRB, 0.53530E+00_JPRB, 0.53367E+00_JPRB, 0.51810E+00_JPRB, 0.50328E+00_JPRB, 0.49604E+00_JPRB, 0.49186E+00_JPRB ,&
&  0.49153E+00_JPRB, 0.49708E+00_JPRB, 0.48103E+00_JPRB, 0.49031E+00_JPRB, 0.48612E+00_JPRB, 0.47988E+00_JPRB, 0.47636E+00_JPRB ,&
&  0.47431E+00_JPRB, 0.47200E+00_JPRB, 0.47036E+00_JPRB, 0.47239E+00_JPRB, 0.47508E+00_JPRB, 0.47592E+00_JPRB, 0.47768E+00_JPRB ,&
&  0.47744E+00_JPRB, 0.47996E+00_JPRB, 0.48218E+00_JPRB, 0.48735E+00_JPRB, 0.49131E+00_JPRB, 0.49383E+00_JPRB, 0.49706E+00_JPRB ,&
&  0.50156E+00_JPRB, 0.50575E+00_JPRB, 0.50975E+00_JPRB, 0.51310E+00_JPRB, 0.51442E+00_JPRB, 0.51411E+00_JPRB, 0.50989E+00_JPRB ,&
&  0.50649E+00_JPRB, 0.50633E+00_JPRB, 0.51065E+00_JPRB, 0.52078E+00_JPRB, 0.52965E+00_JPRB, 0.53056E+00_JPRB, 0.52886E+00_JPRB ,&
&  0.52962E+00_JPRB, 0.53432E+00_JPRB, 0.53649E+00_JPRB, 0.53650E+00_JPRB, 0.53534E+00_JPRB, 0.53314E+00_JPRB, 0.52872E+00_JPRB ,&
&  0.52241E+00_JPRB, 0.51691E+00_JPRB, 0.51117E+00_JPRB, 0.49954E+00_JPRB, 0.49068E+00_JPRB, 0.48964E+00_JPRB, 0.49105E+00_JPRB ,&
&  0.49042E+00_JPRB, 0.48876E+00_JPRB, 0.48985E+00_JPRB, 0.49548E+00_JPRB, 0.49571E+00_JPRB, 0.50277E+00_JPRB, 0.51682E+00_JPRB ,&
&  0.53113E+00_JPRB, 0.54361E+00_JPRB, 0.55336E+00_JPRB, 0.56341E+00_JPRB, 0.57335E+00_JPRB, 0.58088E+00_JPRB, 0.58740E+00_JPRB ,&
&  0.59055E+00_JPRB, 0.58899E+00_JPRB, 0.58373E+00_JPRB, 0.58051E+00_JPRB, 0.57763E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.54415E+00_JPRB, 0.54243E+00_JPRB, 0.54061E+00_JPRB, 0.53831E+00_JPRB, 0.53513E+00_JPRB, 0.53232E+00_JPRB, 0.52776E+00_JPRB ,&
&  0.52298E+00_JPRB, 0.51917E+00_JPRB, 0.51759E+00_JPRB, 0.51714E+00_JPRB, 0.51861E+00_JPRB, 0.52287E+00_JPRB, 0.52983E+00_JPRB ,&
&  0.52997E+00_JPRB, 0.52323E+00_JPRB, 0.52009E+00_JPRB, 0.52020E+00_JPRB, 0.52264E+00_JPRB, 0.52438E+00_JPRB, 0.52706E+00_JPRB ,&
&  0.52877E+00_JPRB, 0.52927E+00_JPRB, 0.52836E+00_JPRB, 0.52121E+00_JPRB, 0.50448E+00_JPRB, 0.49442E+00_JPRB, 0.48559E+00_JPRB ,&
&  0.48036E+00_JPRB, 0.47606E+00_JPRB, 0.47307E+00_JPRB, 0.47197E+00_JPRB, 0.47083E+00_JPRB, 0.46897E+00_JPRB, 0.46888E+00_JPRB ,&
&  0.46700E+00_JPRB, 0.46453E+00_JPRB, 0.46320E+00_JPRB, 0.46381E+00_JPRB, 0.46557E+00_JPRB, 0.46718E+00_JPRB, 0.46707E+00_JPRB ,&
&  0.46806E+00_JPRB, 0.47176E+00_JPRB, 0.47280E+00_JPRB, 0.47295E+00_JPRB, 0.47274E+00_JPRB, 0.47225E+00_JPRB, 0.47232E+00_JPRB ,&
&  0.47466E+00_JPRB, 0.47979E+00_JPRB, 0.48420E+00_JPRB, 0.48865E+00_JPRB, 0.49059E+00_JPRB, 0.49151E+00_JPRB, 0.49145E+00_JPRB ,&
&  0.48827E+00_JPRB, 0.48418E+00_JPRB, 0.48273E+00_JPRB, 0.48575E+00_JPRB, 0.49192E+00_JPRB, 0.49803E+00_JPRB, 0.50097E+00_JPRB ,&
&  0.50328E+00_JPRB, 0.50455E+00_JPRB, 0.50285E+00_JPRB, 0.50225E+00_JPRB, 0.50012E+00_JPRB, 0.49600E+00_JPRB, 0.49368E+00_JPRB ,&
&  0.49133E+00_JPRB, 0.48911E+00_JPRB, 0.48694E+00_JPRB, 0.47792E+00_JPRB, 0.47037E+00_JPRB, 0.46590E+00_JPRB, 0.46300E+00_JPRB ,&
&  0.46197E+00_JPRB, 0.46112E+00_JPRB, 0.46088E+00_JPRB, 0.46211E+00_JPRB, 0.46421E+00_JPRB, 0.47111E+00_JPRB, 0.48468E+00_JPRB ,&
&  0.50453E+00_JPRB, 0.52288E+00_JPRB, 0.53280E+00_JPRB, 0.53977E+00_JPRB, 0.54413E+00_JPRB, 0.54626E+00_JPRB, 0.54843E+00_JPRB ,&
&  0.55187E+00_JPRB, 0.55299E+00_JPRB, 0.55076E+00_JPRB, 0.54802E+00_JPRB, 0.54641E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.51631E+00_JPRB, 0.51641E+00_JPRB, 0.51486E+00_JPRB, 0.51341E+00_JPRB, 0.51185E+00_JPRB, 0.51047E+00_JPRB, 0.50842E+00_JPRB ,&
&  0.50357E+00_JPRB, 0.49730E+00_JPRB, 0.49368E+00_JPRB, 0.49215E+00_JPRB, 0.49795E+00_JPRB, 0.50644E+00_JPRB, 0.51400E+00_JPRB ,&
&  0.51314E+00_JPRB, 0.50403E+00_JPRB, 0.50122E+00_JPRB, 0.50321E+00_JPRB, 0.50576E+00_JPRB, 0.50856E+00_JPRB, 0.51232E+00_JPRB ,&
&  0.51643E+00_JPRB, 0.52060E+00_JPRB, 0.52245E+00_JPRB, 0.51981E+00_JPRB, 0.51320E+00_JPRB, 0.49947E+00_JPRB, 0.48751E+00_JPRB ,&
&  0.47957E+00_JPRB, 0.47340E+00_JPRB, 0.46973E+00_JPRB, 0.46865E+00_JPRB, 0.46639E+00_JPRB, 0.46260E+00_JPRB, 0.45728E+00_JPRB ,&
&  0.45270E+00_JPRB, 0.45062E+00_JPRB, 0.45291E+00_JPRB, 0.45472E+00_JPRB, 0.45563E+00_JPRB, 0.45531E+00_JPRB, 0.45230E+00_JPRB ,&
&  0.45060E+00_JPRB, 0.45006E+00_JPRB, 0.44970E+00_JPRB, 0.45121E+00_JPRB, 0.44880E+00_JPRB, 0.44680E+00_JPRB, 0.44883E+00_JPRB ,&
&  0.45336E+00_JPRB, 0.45736E+00_JPRB, 0.46104E+00_JPRB, 0.46394E+00_JPRB, 0.46520E+00_JPRB, 0.46738E+00_JPRB, 0.46872E+00_JPRB ,&
&  0.46773E+00_JPRB, 0.46404E+00_JPRB, 0.46059E+00_JPRB, 0.46100E+00_JPRB, 0.46325E+00_JPRB, 0.46583E+00_JPRB, 0.46929E+00_JPRB ,&
&  0.46990E+00_JPRB, 0.46486E+00_JPRB, 0.45985E+00_JPRB, 0.45548E+00_JPRB, 0.45152E+00_JPRB, 0.44852E+00_JPRB, 0.44713E+00_JPRB ,&
&  0.44683E+00_JPRB, 0.44379E+00_JPRB, 0.43784E+00_JPRB, 0.43295E+00_JPRB, 0.43247E+00_JPRB, 0.43195E+00_JPRB, 0.42835E+00_JPRB ,&
&  0.42897E+00_JPRB, 0.43165E+00_JPRB, 0.43061E+00_JPRB, 0.42873E+00_JPRB, 0.43072E+00_JPRB, 0.43629E+00_JPRB, 0.44778E+00_JPRB ,&
&  0.46306E+00_JPRB, 0.48051E+00_JPRB, 0.49656E+00_JPRB, 0.51023E+00_JPRB, 0.51819E+00_JPRB, 0.52000E+00_JPRB, 0.51817E+00_JPRB ,&
&  0.51745E+00_JPRB, 0.51820E+00_JPRB, 0.51728E+00_JPRB, 0.51561E+00_JPRB, 0.51556E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.48811E+00_JPRB, 0.48791E+00_JPRB, 0.48801E+00_JPRB, 0.48907E+00_JPRB, 0.48977E+00_JPRB, 0.48870E+00_JPRB, 0.48839E+00_JPRB ,&
&  0.48521E+00_JPRB, 0.48061E+00_JPRB, 0.47786E+00_JPRB, 0.47951E+00_JPRB, 0.48883E+00_JPRB, 0.50060E+00_JPRB, 0.50765E+00_JPRB ,&
&  0.50317E+00_JPRB, 0.49066E+00_JPRB, 0.48765E+00_JPRB, 0.48997E+00_JPRB, 0.49221E+00_JPRB, 0.49634E+00_JPRB, 0.50244E+00_JPRB ,&
&  0.50801E+00_JPRB, 0.51181E+00_JPRB, 0.51600E+00_JPRB, 0.51773E+00_JPRB, 0.51793E+00_JPRB, 0.51079E+00_JPRB, 0.49957E+00_JPRB ,&
&  0.48877E+00_JPRB, 0.48093E+00_JPRB, 0.47598E+00_JPRB, 0.47236E+00_JPRB, 0.46725E+00_JPRB, 0.46213E+00_JPRB, 0.45552E+00_JPRB ,&
&  0.45234E+00_JPRB, 0.45212E+00_JPRB, 0.45310E+00_JPRB, 0.45125E+00_JPRB, 0.44866E+00_JPRB, 0.44340E+00_JPRB, 0.43897E+00_JPRB ,&
&  0.43719E+00_JPRB, 0.43830E+00_JPRB, 0.43946E+00_JPRB, 0.43845E+00_JPRB, 0.43491E+00_JPRB, 0.43238E+00_JPRB, 0.43367E+00_JPRB ,&
&  0.43949E+00_JPRB, 0.43928E+00_JPRB, 0.43830E+00_JPRB, 0.43918E+00_JPRB, 0.44146E+00_JPRB, 0.44347E+00_JPRB, 0.44287E+00_JPRB ,&
&  0.44214E+00_JPRB, 0.44199E+00_JPRB, 0.44215E+00_JPRB, 0.44392E+00_JPRB, 0.44395E+00_JPRB, 0.44174E+00_JPRB, 0.43613E+00_JPRB ,&
&  0.42766E+00_JPRB, 0.41705E+00_JPRB, 0.41269E+00_JPRB, 0.40961E+00_JPRB, 0.40667E+00_JPRB, 0.40614E+00_JPRB, 0.40765E+00_JPRB ,&
&  0.40635E+00_JPRB, 0.40020E+00_JPRB, 0.39745E+00_JPRB, 0.39810E+00_JPRB, 0.39742E+00_JPRB, 0.39993E+00_JPRB, 0.40404E+00_JPRB ,&
&  0.41460E+00_JPRB, 0.42056E+00_JPRB, 0.42233E+00_JPRB, 0.42021E+00_JPRB, 0.42051E+00_JPRB, 0.42230E+00_JPRB, 0.42676E+00_JPRB ,&
&  0.43344E+00_JPRB, 0.44068E+00_JPRB, 0.44817E+00_JPRB, 0.45647E+00_JPRB, 0.46673E+00_JPRB, 0.47661E+00_JPRB, 0.48040E+00_JPRB ,&
&  0.48067E+00_JPRB, 0.48109E+00_JPRB, 0.48243E+00_JPRB, 0.48401E+00_JPRB, 0.48693E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.45348E+00_JPRB, 0.45571E+00_JPRB, 0.45797E+00_JPRB, 0.45953E+00_JPRB, 0.46222E+00_JPRB, 0.46710E+00_JPRB, 0.46905E+00_JPRB ,&
&  0.46709E+00_JPRB, 0.46490E+00_JPRB, 0.46479E+00_JPRB, 0.46832E+00_JPRB, 0.47518E+00_JPRB, 0.48230E+00_JPRB, 0.49004E+00_JPRB ,&
&  0.48587E+00_JPRB, 0.47322E+00_JPRB, 0.46864E+00_JPRB, 0.47070E+00_JPRB, 0.47559E+00_JPRB, 0.48101E+00_JPRB, 0.48768E+00_JPRB ,&
&  0.49402E+00_JPRB, 0.49816E+00_JPRB, 0.50252E+00_JPRB, 0.50648E+00_JPRB, 0.50905E+00_JPRB, 0.50727E+00_JPRB, 0.50281E+00_JPRB ,&
&  0.49744E+00_JPRB, 0.49192E+00_JPRB, 0.48698E+00_JPRB, 0.47130E+00_JPRB, 0.46871E+00_JPRB, 0.45972E+00_JPRB, 0.45228E+00_JPRB ,&
&  0.44715E+00_JPRB, 0.44291E+00_JPRB, 0.44020E+00_JPRB, 0.43708E+00_JPRB, 0.43267E+00_JPRB, 0.42524E+00_JPRB, 0.42299E+00_JPRB ,&
&  0.42190E+00_JPRB, 0.42282E+00_JPRB, 0.41804E+00_JPRB, 0.41545E+00_JPRB, 0.41258E+00_JPRB, 0.41030E+00_JPRB, 0.41440E+00_JPRB ,&
&  0.41946E+00_JPRB, 0.41688E+00_JPRB, 0.41288E+00_JPRB, 0.41258E+00_JPRB, 0.41690E+00_JPRB, 0.41854E+00_JPRB, 0.41948E+00_JPRB ,&
&  0.42119E+00_JPRB, 0.42159E+00_JPRB, 0.41680E+00_JPRB, 0.41157E+00_JPRB, 0.40295E+00_JPRB, 0.39199E+00_JPRB, 0.38893E+00_JPRB ,&
&  0.38589E+00_JPRB, 0.38019E+00_JPRB, 0.37690E+00_JPRB, 0.37587E+00_JPRB, 0.37555E+00_JPRB, 0.37646E+00_JPRB, 0.37666E+00_JPRB ,&
&  0.37675E+00_JPRB, 0.37328E+00_JPRB, 0.36818E+00_JPRB, 0.36384E+00_JPRB, 0.35793E+00_JPRB, 0.36092E+00_JPRB, 0.37114E+00_JPRB ,&
&  0.38358E+00_JPRB, 0.39119E+00_JPRB, 0.39613E+00_JPRB, 0.39959E+00_JPRB, 0.40270E+00_JPRB, 0.40412E+00_JPRB, 0.40650E+00_JPRB ,&
&  0.40966E+00_JPRB, 0.41112E+00_JPRB, 0.40987E+00_JPRB, 0.41015E+00_JPRB, 0.41350E+00_JPRB, 0.41887E+00_JPRB, 0.42529E+00_JPRB ,&
&  0.43158E+00_JPRB, 0.43684E+00_JPRB, 0.44121E+00_JPRB, 0.44667E+00_JPRB, 0.45021E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.42522E+00_JPRB, 0.42959E+00_JPRB, 0.43375E+00_JPRB, 0.43828E+00_JPRB, 0.44472E+00_JPRB, 0.44809E+00_JPRB, 0.44916E+00_JPRB ,&
&  0.45044E+00_JPRB, 0.45092E+00_JPRB, 0.45237E+00_JPRB, 0.45670E+00_JPRB, 0.46292E+00_JPRB, 0.46746E+00_JPRB, 0.47333E+00_JPRB ,&
&  0.47340E+00_JPRB, 0.46456E+00_JPRB, 0.45659E+00_JPRB, 0.45632E+00_JPRB, 0.46190E+00_JPRB, 0.46768E+00_JPRB, 0.47493E+00_JPRB ,&
&  0.48209E+00_JPRB, 0.48728E+00_JPRB, 0.49020E+00_JPRB, 0.49167E+00_JPRB, 0.49360E+00_JPRB, 0.49315E+00_JPRB, 0.49005E+00_JPRB ,&
&  0.48573E+00_JPRB, 0.47651E+00_JPRB, 0.45699E+00_JPRB, 0.45428E+00_JPRB, 0.44553E+00_JPRB, 0.43793E+00_JPRB, 0.43266E+00_JPRB ,&
&  0.42843E+00_JPRB, 0.42492E+00_JPRB, 0.42087E+00_JPRB, 0.41731E+00_JPRB, 0.41320E+00_JPRB, 0.40597E+00_JPRB, 0.40219E+00_JPRB ,&
&  0.39876E+00_JPRB, 0.39593E+00_JPRB, 0.39312E+00_JPRB, 0.39317E+00_JPRB, 0.39340E+00_JPRB, 0.39606E+00_JPRB, 0.39731E+00_JPRB ,&
&  0.39403E+00_JPRB, 0.39021E+00_JPRB, 0.38730E+00_JPRB, 0.38892E+00_JPRB, 0.39121E+00_JPRB, 0.39280E+00_JPRB, 0.39337E+00_JPRB ,&
&  0.39076E+00_JPRB, 0.38596E+00_JPRB, 0.38095E+00_JPRB, 0.37523E+00_JPRB, 0.36934E+00_JPRB, 0.34365E+00_JPRB, 0.33254E+00_JPRB ,&
&  0.32954E+00_JPRB, 0.33870E+00_JPRB, 0.35089E+00_JPRB, 0.34992E+00_JPRB, 0.35246E+00_JPRB, 0.35269E+00_JPRB, 0.35015E+00_JPRB ,&
&  0.34099E+00_JPRB, 0.32757E+00_JPRB, 0.31879E+00_JPRB, 0.31494E+00_JPRB, 0.31222E+00_JPRB, 0.31403E+00_JPRB, 0.32049E+00_JPRB ,&
&  0.33091E+00_JPRB, 0.34270E+00_JPRB, 0.35158E+00_JPRB, 0.35561E+00_JPRB, 0.35445E+00_JPRB, 0.35135E+00_JPRB, 0.34723E+00_JPRB ,&
&  0.34291E+00_JPRB, 0.34894E+00_JPRB, 0.36354E+00_JPRB, 0.38349E+00_JPRB, 0.38618E+00_JPRB, 0.39144E+00_JPRB, 0.39501E+00_JPRB ,&
&  0.40036E+00_JPRB, 0.40709E+00_JPRB, 0.41281E+00_JPRB, 0.41875E+00_JPRB, 0.42235E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.40299E+00_JPRB, 0.40801E+00_JPRB, 0.41481E+00_JPRB, 0.42240E+00_JPRB, 0.42786E+00_JPRB, 0.43045E+00_JPRB, 0.43319E+00_JPRB ,&
&  0.43664E+00_JPRB, 0.43810E+00_JPRB, 0.43916E+00_JPRB, 0.44192E+00_JPRB, 0.44758E+00_JPRB, 0.45077E+00_JPRB, 0.45574E+00_JPRB ,&
&  0.45776E+00_JPRB, 0.45557E+00_JPRB, 0.44870E+00_JPRB, 0.44674E+00_JPRB, 0.45039E+00_JPRB, 0.45521E+00_JPRB, 0.46045E+00_JPRB ,&
&  0.46585E+00_JPRB, 0.47078E+00_JPRB, 0.47309E+00_JPRB, 0.47368E+00_JPRB, 0.47379E+00_JPRB, 0.47357E+00_JPRB, 0.47085E+00_JPRB ,&
&  0.46532E+00_JPRB, 0.42548E+00_JPRB, 0.39897E+00_JPRB, 0.40287E+00_JPRB, 0.41425E+00_JPRB, 0.40832E+00_JPRB, 0.40675E+00_JPRB ,&
&  0.40518E+00_JPRB, 0.40153E+00_JPRB, 0.39612E+00_JPRB, 0.39165E+00_JPRB, 0.38616E+00_JPRB, 0.37907E+00_JPRB, 0.37342E+00_JPRB ,&
&  0.37199E+00_JPRB, 0.37698E+00_JPRB, 0.37844E+00_JPRB, 0.38033E+00_JPRB, 0.37869E+00_JPRB, 0.38029E+00_JPRB, 0.37877E+00_JPRB ,&
&  0.37304E+00_JPRB, 0.36654E+00_JPRB, 0.36428E+00_JPRB, 0.36381E+00_JPRB, 0.36492E+00_JPRB, 0.36761E+00_JPRB, 0.36727E+00_JPRB ,&
&  0.35426E+00_JPRB, 0.34266E+00_JPRB, 0.32402E+00_JPRB, 0.30593E+00_JPRB, 0.29278E+00_JPRB, 0.28684E+00_JPRB, 0.28255E+00_JPRB ,&
&  0.28183E+00_JPRB, 0.28800E+00_JPRB, 0.30526E+00_JPRB, 0.32608E+00_JPRB, 0.33926E+00_JPRB, 0.32090E+00_JPRB, 0.30546E+00_JPRB ,&
&  0.29609E+00_JPRB, 0.29109E+00_JPRB, 0.28680E+00_JPRB, 0.28535E+00_JPRB, 0.28591E+00_JPRB, 0.28839E+00_JPRB, 0.29190E+00_JPRB ,&
&  0.29681E+00_JPRB, 0.30134E+00_JPRB, 0.30414E+00_JPRB, 0.30549E+00_JPRB, 0.30626E+00_JPRB, 0.30706E+00_JPRB, 0.30648E+00_JPRB ,&
&  0.30640E+00_JPRB, 0.31002E+00_JPRB, 0.31713E+00_JPRB, 0.32616E+00_JPRB, 0.33447E+00_JPRB, 0.34378E+00_JPRB, 0.35685E+00_JPRB ,&
&  0.37119E+00_JPRB, 0.38270E+00_JPRB, 0.38684E+00_JPRB, 0.39300E+00_JPRB, 0.39830E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.39361E+00_JPRB, 0.40051E+00_JPRB, 0.40542E+00_JPRB, 0.41522E+00_JPRB, 0.41813E+00_JPRB, 0.42118E+00_JPRB, 0.42380E+00_JPRB ,&
&  0.43021E+00_JPRB, 0.43388E+00_JPRB, 0.43176E+00_JPRB, 0.43125E+00_JPRB, 0.43044E+00_JPRB, 0.42917E+00_JPRB, 0.43031E+00_JPRB ,&
&  0.43054E+00_JPRB, 0.43275E+00_JPRB, 0.43035E+00_JPRB, 0.42763E+00_JPRB, 0.42962E+00_JPRB, 0.43448E+00_JPRB, 0.43774E+00_JPRB ,&
&  0.43246E+00_JPRB, 0.43091E+00_JPRB, 0.43937E+00_JPRB, 0.44132E+00_JPRB, 0.44240E+00_JPRB, 0.44426E+00_JPRB, 0.44391E+00_JPRB ,&
&  0.42986E+00_JPRB, 0.40412E+00_JPRB, 0.37654E+00_JPRB, 0.37858E+00_JPRB, 0.39329E+00_JPRB, 0.38714E+00_JPRB, 0.38541E+00_JPRB ,&
&  0.38250E+00_JPRB, 0.37790E+00_JPRB, 0.37353E+00_JPRB, 0.37137E+00_JPRB, 0.36858E+00_JPRB, 0.36380E+00_JPRB, 0.36223E+00_JPRB ,&
&  0.36103E+00_JPRB, 0.36992E+00_JPRB, 0.36471E+00_JPRB, 0.34204E+00_JPRB, 0.32531E+00_JPRB, 0.31613E+00_JPRB, 0.30434E+00_JPRB ,&
&  0.29057E+00_JPRB, 0.28273E+00_JPRB, 0.28161E+00_JPRB, 0.27964E+00_JPRB, 0.28105E+00_JPRB, 0.28218E+00_JPRB, 0.28331E+00_JPRB ,&
&  0.28407E+00_JPRB, 0.28379E+00_JPRB, 0.27910E+00_JPRB, 0.27305E+00_JPRB, 0.26976E+00_JPRB, 0.26979E+00_JPRB, 0.27264E+00_JPRB ,&
&  0.27877E+00_JPRB, 0.28516E+00_JPRB, 0.29981E+00_JPRB, 0.30761E+00_JPRB, 0.29871E+00_JPRB, 0.29356E+00_JPRB, 0.28769E+00_JPRB ,&
&  0.27693E+00_JPRB, 0.27286E+00_JPRB, 0.26948E+00_JPRB, 0.26760E+00_JPRB, 0.26851E+00_JPRB, 0.27199E+00_JPRB, 0.27379E+00_JPRB ,&
&  0.27647E+00_JPRB, 0.27935E+00_JPRB, 0.28053E+00_JPRB, 0.28146E+00_JPRB, 0.28224E+00_JPRB, 0.28423E+00_JPRB, 0.28626E+00_JPRB ,&
&  0.28792E+00_JPRB, 0.29094E+00_JPRB, 0.29459E+00_JPRB, 0.29943E+00_JPRB, 0.30369E+00_JPRB, 0.30618E+00_JPRB, 0.31369E+00_JPRB ,&
&  0.32240E+00_JPRB, 0.34027E+00_JPRB, 0.36929E+00_JPRB, 0.39368E+00_JPRB, 0.38874E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.39518E+00_JPRB, 0.39942E+00_JPRB, 0.39996E+00_JPRB, 0.40321E+00_JPRB, 0.40822E+00_JPRB, 0.41402E+00_JPRB, 0.41892E+00_JPRB ,&
&  0.42422E+00_JPRB, 0.42395E+00_JPRB, 0.41887E+00_JPRB, 0.41152E+00_JPRB, 0.40540E+00_JPRB, 0.38868E+00_JPRB, 0.38000E+00_JPRB ,&
&  0.37286E+00_JPRB, 0.37269E+00_JPRB, 0.37319E+00_JPRB, 0.37495E+00_JPRB, 0.37598E+00_JPRB, 0.38105E+00_JPRB, 0.38248E+00_JPRB ,&
&  0.37977E+00_JPRB, 0.37447E+00_JPRB, 0.36906E+00_JPRB, 0.36803E+00_JPRB, 0.37063E+00_JPRB, 0.37249E+00_JPRB, 0.36944E+00_JPRB ,&
&  0.36574E+00_JPRB, 0.36550E+00_JPRB, 0.36605E+00_JPRB, 0.36748E+00_JPRB, 0.37704E+00_JPRB, 0.37172E+00_JPRB, 0.36714E+00_JPRB ,&
&  0.36001E+00_JPRB, 0.35392E+00_JPRB, 0.34975E+00_JPRB, 0.35010E+00_JPRB, 0.35331E+00_JPRB, 0.35253E+00_JPRB, 0.35527E+00_JPRB ,&
&  0.34394E+00_JPRB, 0.33444E+00_JPRB, 0.31920E+00_JPRB, 0.30705E+00_JPRB, 0.29454E+00_JPRB, 0.28600E+00_JPRB, 0.27813E+00_JPRB ,&
&  0.27144E+00_JPRB, 0.26582E+00_JPRB, 0.26054E+00_JPRB, 0.25886E+00_JPRB, 0.25978E+00_JPRB, 0.25977E+00_JPRB, 0.25924E+00_JPRB ,&
&  0.25972E+00_JPRB, 0.26067E+00_JPRB, 0.26140E+00_JPRB, 0.26243E+00_JPRB, 0.26412E+00_JPRB, 0.26688E+00_JPRB, 0.27062E+00_JPRB ,&
&  0.27641E+00_JPRB, 0.28101E+00_JPRB, 0.28346E+00_JPRB, 0.28476E+00_JPRB, 0.28159E+00_JPRB, 0.27614E+00_JPRB, 0.27067E+00_JPRB ,&
&  0.26507E+00_JPRB, 0.26247E+00_JPRB, 0.26009E+00_JPRB, 0.25895E+00_JPRB, 0.25872E+00_JPRB, 0.25963E+00_JPRB, 0.26156E+00_JPRB ,&
&  0.26352E+00_JPRB, 0.26603E+00_JPRB, 0.26769E+00_JPRB, 0.26963E+00_JPRB, 0.27195E+00_JPRB, 0.27395E+00_JPRB, 0.27602E+00_JPRB ,&
&  0.27833E+00_JPRB, 0.28014E+00_JPRB, 0.28326E+00_JPRB, 0.28713E+00_JPRB, 0.29454E+00_JPRB, 0.30310E+00_JPRB, 0.31834E+00_JPRB ,&
&  0.33606E+00_JPRB, 0.36813E+00_JPRB, 0.37780E+00_JPRB, 0.38599E+00_JPRB, 0.39354E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.39096E+00_JPRB, 0.39347E+00_JPRB, 0.39736E+00_JPRB, 0.40293E+00_JPRB, 0.40990E+00_JPRB, 0.41633E+00_JPRB, 0.41791E+00_JPRB ,&
&  0.40838E+00_JPRB, 0.39551E+00_JPRB, 0.37721E+00_JPRB, 0.36283E+00_JPRB, 0.34876E+00_JPRB, 0.33835E+00_JPRB, 0.32994E+00_JPRB ,&
&  0.32444E+00_JPRB, 0.32410E+00_JPRB, 0.32443E+00_JPRB, 0.32567E+00_JPRB, 0.32453E+00_JPRB, 0.32552E+00_JPRB, 0.32574E+00_JPRB ,&
&  0.32439E+00_JPRB, 0.32422E+00_JPRB, 0.32413E+00_JPRB, 0.32758E+00_JPRB, 0.33442E+00_JPRB, 0.34069E+00_JPRB, 0.34689E+00_JPRB ,&
&  0.35420E+00_JPRB, 0.35613E+00_JPRB, 0.36027E+00_JPRB, 0.35885E+00_JPRB, 0.35535E+00_JPRB, 0.35085E+00_JPRB, 0.34454E+00_JPRB ,&
&  0.34020E+00_JPRB, 0.33633E+00_JPRB, 0.33514E+00_JPRB, 0.33559E+00_JPRB, 0.33569E+00_JPRB, 0.33827E+00_JPRB, 0.33254E+00_JPRB ,&
&  0.32529E+00_JPRB, 0.31459E+00_JPRB, 0.30203E+00_JPRB, 0.29269E+00_JPRB, 0.28364E+00_JPRB, 0.27715E+00_JPRB, 0.27137E+00_JPRB ,&
&  0.26605E+00_JPRB, 0.26248E+00_JPRB, 0.25876E+00_JPRB, 0.25603E+00_JPRB, 0.25515E+00_JPRB, 0.25324E+00_JPRB, 0.25215E+00_JPRB ,&
&  0.25170E+00_JPRB, 0.25212E+00_JPRB, 0.25386E+00_JPRB, 0.25543E+00_JPRB, 0.25760E+00_JPRB, 0.26001E+00_JPRB, 0.26207E+00_JPRB ,&
&  0.26480E+00_JPRB, 0.26702E+00_JPRB, 0.26729E+00_JPRB, 0.26669E+00_JPRB, 0.26451E+00_JPRB, 0.26163E+00_JPRB, 0.25913E+00_JPRB ,&
&  0.25655E+00_JPRB, 0.25475E+00_JPRB, 0.25408E+00_JPRB, 0.25318E+00_JPRB, 0.25365E+00_JPRB, 0.25487E+00_JPRB, 0.25579E+00_JPRB ,&
&  0.25764E+00_JPRB, 0.25916E+00_JPRB, 0.26094E+00_JPRB, 0.26395E+00_JPRB, 0.26672E+00_JPRB, 0.26915E+00_JPRB, 0.27160E+00_JPRB ,&
&  0.27170E+00_JPRB, 0.27345E+00_JPRB, 0.27587E+00_JPRB, 0.28034E+00_JPRB, 0.29338E+00_JPRB, 0.30234E+00_JPRB, 0.32297E+00_JPRB ,&
&  0.34027E+00_JPRB, 0.37489E+00_JPRB, 0.37430E+00_JPRB, 0.38065E+00_JPRB, 0.38726E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.38548E+00_JPRB, 0.39228E+00_JPRB, 0.39905E+00_JPRB, 0.40427E+00_JPRB, 0.40789E+00_JPRB, 0.41107E+00_JPRB, 0.40923E+00_JPRB ,&
&  0.40318E+00_JPRB, 0.38836E+00_JPRB, 0.37706E+00_JPRB, 0.36583E+00_JPRB, 0.35645E+00_JPRB, 0.34745E+00_JPRB, 0.33910E+00_JPRB ,&
&  0.33098E+00_JPRB, 0.32421E+00_JPRB, 0.31854E+00_JPRB, 0.31356E+00_JPRB, 0.30680E+00_JPRB, 0.30082E+00_JPRB, 0.29602E+00_JPRB ,&
&  0.29205E+00_JPRB, 0.29192E+00_JPRB, 0.29430E+00_JPRB, 0.29878E+00_JPRB, 0.30715E+00_JPRB, 0.31548E+00_JPRB, 0.32478E+00_JPRB ,&
&  0.33897E+00_JPRB, 0.34223E+00_JPRB, 0.34114E+00_JPRB, 0.34048E+00_JPRB, 0.33549E+00_JPRB, 0.32927E+00_JPRB, 0.32012E+00_JPRB ,&
&  0.32153E+00_JPRB, 0.31958E+00_JPRB, 0.32719E+00_JPRB, 0.32733E+00_JPRB, 0.32607E+00_JPRB, 0.32654E+00_JPRB, 0.31919E+00_JPRB ,&
&  0.31396E+00_JPRB, 0.30458E+00_JPRB, 0.29730E+00_JPRB, 0.28956E+00_JPRB, 0.28177E+00_JPRB, 0.27716E+00_JPRB, 0.27113E+00_JPRB ,&
&  0.26671E+00_JPRB, 0.26344E+00_JPRB, 0.25919E+00_JPRB, 0.25653E+00_JPRB, 0.25420E+00_JPRB, 0.25177E+00_JPRB, 0.25050E+00_JPRB ,&
&  0.24953E+00_JPRB, 0.24918E+00_JPRB, 0.24932E+00_JPRB, 0.24953E+00_JPRB, 0.25004E+00_JPRB, 0.25097E+00_JPRB, 0.25168E+00_JPRB ,&
&  0.25248E+00_JPRB, 0.25277E+00_JPRB, 0.25241E+00_JPRB, 0.25200E+00_JPRB, 0.25083E+00_JPRB, 0.25006E+00_JPRB, 0.24903E+00_JPRB ,&
&  0.24840E+00_JPRB, 0.24873E+00_JPRB, 0.24904E+00_JPRB, 0.24976E+00_JPRB, 0.25049E+00_JPRB, 0.25212E+00_JPRB, 0.25363E+00_JPRB ,&
&  0.25524E+00_JPRB, 0.25728E+00_JPRB, 0.25894E+00_JPRB, 0.26107E+00_JPRB, 0.26359E+00_JPRB, 0.26527E+00_JPRB, 0.26824E+00_JPRB ,&
&  0.27112E+00_JPRB, 0.27475E+00_JPRB, 0.28103E+00_JPRB, 0.28793E+00_JPRB, 0.29722E+00_JPRB, 0.30840E+00_JPRB, 0.32242E+00_JPRB ,&
&  0.34024E+00_JPRB, 0.36496E+00_JPRB, 0.36641E+00_JPRB, 0.37458E+00_JPRB, 0.38000E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.37469E+00_JPRB, 0.37976E+00_JPRB, 0.38408E+00_JPRB, 0.38795E+00_JPRB, 0.39140E+00_JPRB, 0.39242E+00_JPRB, 0.39151E+00_JPRB ,&
&  0.38233E+00_JPRB, 0.37563E+00_JPRB, 0.36740E+00_JPRB, 0.35998E+00_JPRB, 0.35311E+00_JPRB, 0.34470E+00_JPRB, 0.33766E+00_JPRB ,&
&  0.33044E+00_JPRB, 0.32319E+00_JPRB, 0.31637E+00_JPRB, 0.30833E+00_JPRB, 0.29998E+00_JPRB, 0.29197E+00_JPRB, 0.28715E+00_JPRB ,&
&  0.28441E+00_JPRB, 0.28434E+00_JPRB, 0.28745E+00_JPRB, 0.29136E+00_JPRB, 0.29639E+00_JPRB, 0.30283E+00_JPRB, 0.30977E+00_JPRB ,&
&  0.31412E+00_JPRB, 0.31646E+00_JPRB, 0.31593E+00_JPRB, 0.31423E+00_JPRB, 0.31208E+00_JPRB, 0.31035E+00_JPRB, 0.30764E+00_JPRB ,&
&  0.30418E+00_JPRB, 0.30195E+00_JPRB, 0.30152E+00_JPRB, 0.30297E+00_JPRB, 0.30303E+00_JPRB, 0.30134E+00_JPRB, 0.29740E+00_JPRB ,&
&  0.29288E+00_JPRB, 0.28829E+00_JPRB, 0.28349E+00_JPRB, 0.27899E+00_JPRB, 0.27529E+00_JPRB, 0.27137E+00_JPRB, 0.26713E+00_JPRB ,&
&  0.26446E+00_JPRB, 0.26165E+00_JPRB, 0.25857E+00_JPRB, 0.25568E+00_JPRB, 0.25326E+00_JPRB, 0.25139E+00_JPRB, 0.24981E+00_JPRB ,&
&  0.24861E+00_JPRB, 0.24769E+00_JPRB, 0.24696E+00_JPRB, 0.24642E+00_JPRB, 0.24590E+00_JPRB, 0.24507E+00_JPRB, 0.24425E+00_JPRB ,&
&  0.24344E+00_JPRB, 0.24314E+00_JPRB, 0.24265E+00_JPRB, 0.24269E+00_JPRB, 0.24297E+00_JPRB, 0.24314E+00_JPRB, 0.24368E+00_JPRB ,&
&  0.24484E+00_JPRB, 0.24606E+00_JPRB, 0.24667E+00_JPRB, 0.24782E+00_JPRB, 0.24893E+00_JPRB, 0.25038E+00_JPRB, 0.25229E+00_JPRB ,&
&  0.25348E+00_JPRB, 0.25480E+00_JPRB, 0.25637E+00_JPRB, 0.25807E+00_JPRB, 0.26091E+00_JPRB, 0.26399E+00_JPRB, 0.26740E+00_JPRB ,&
&  0.27178E+00_JPRB, 0.27601E+00_JPRB, 0.28082E+00_JPRB, 0.28655E+00_JPRB, 0.29687E+00_JPRB, 0.30358E+00_JPRB, 0.31209E+00_JPRB ,&
&  0.32041E+00_JPRB, 0.32446E+00_JPRB, 0.35934E+00_JPRB, 0.36211E+00_JPRB, 0.36772E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.30136E+00_JPRB, 0.30802E+00_JPRB, 0.31232E+00_JPRB, 0.31681E+00_JPRB, 0.31399E+00_JPRB, 0.35284E+00_JPRB, 0.35195E+00_JPRB ,&
&  0.35104E+00_JPRB, 0.33726E+00_JPRB, 0.33530E+00_JPRB, 0.33070E+00_JPRB, 0.32516E+00_JPRB, 0.31902E+00_JPRB, 0.31372E+00_JPRB ,&
&  0.30884E+00_JPRB, 0.30338E+00_JPRB, 0.29895E+00_JPRB, 0.29485E+00_JPRB, 0.29150E+00_JPRB, 0.28852E+00_JPRB, 0.28643E+00_JPRB ,&
&  0.28593E+00_JPRB, 0.28749E+00_JPRB, 0.29075E+00_JPRB, 0.29245E+00_JPRB, 0.29308E+00_JPRB, 0.29361E+00_JPRB, 0.29415E+00_JPRB ,&
&  0.29409E+00_JPRB, 0.29394E+00_JPRB, 0.29265E+00_JPRB, 0.29028E+00_JPRB, 0.28832E+00_JPRB, 0.28688E+00_JPRB, 0.28577E+00_JPRB ,&
&  0.28474E+00_JPRB, 0.28376E+00_JPRB, 0.28272E+00_JPRB, 0.28211E+00_JPRB, 0.28114E+00_JPRB, 0.27971E+00_JPRB, 0.27759E+00_JPRB ,&
&  0.27542E+00_JPRB, 0.27317E+00_JPRB, 0.27045E+00_JPRB, 0.26827E+00_JPRB, 0.26614E+00_JPRB, 0.26399E+00_JPRB, 0.26184E+00_JPRB ,&
&  0.26062E+00_JPRB, 0.25943E+00_JPRB, 0.25794E+00_JPRB, 0.25606E+00_JPRB, 0.25464E+00_JPRB, 0.25314E+00_JPRB, 0.25182E+00_JPRB ,&
&  0.25097E+00_JPRB, 0.25007E+00_JPRB, 0.24916E+00_JPRB, 0.24821E+00_JPRB, 0.24753E+00_JPRB, 0.24673E+00_JPRB, 0.24599E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24458E+00_JPRB, 0.24440E+00_JPRB, 0.24432E+00_JPRB, 0.24439E+00_JPRB, 0.24444E+00_JPRB, 0.24476E+00_JPRB ,&
&  0.24526E+00_JPRB, 0.24599E+00_JPRB, 0.24688E+00_JPRB, 0.24772E+00_JPRB, 0.24853E+00_JPRB, 0.24937E+00_JPRB, 0.25061E+00_JPRB ,&
&  0.25195E+00_JPRB, 0.25338E+00_JPRB, 0.25485E+00_JPRB, 0.25600E+00_JPRB, 0.25847E+00_JPRB, 0.26079E+00_JPRB, 0.26325E+00_JPRB ,&
&  0.26607E+00_JPRB, 0.26942E+00_JPRB, 0.27303E+00_JPRB, 0.27690E+00_JPRB, 0.28204E+00_JPRB, 0.28516E+00_JPRB, 0.28740E+00_JPRB ,&
&  0.28988E+00_JPRB, 0.29410E+00_JPRB, 0.29553E+00_JPRB, 0.29679E+00_JPRB, 0.29874E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.27441E+00_JPRB, 0.27468E+00_JPRB, 0.27474E+00_JPRB, 0.27464E+00_JPRB, 0.27242E+00_JPRB, 0.27265E+00_JPRB, 0.27262E+00_JPRB ,&
&  0.27280E+00_JPRB, 0.27304E+00_JPRB, 0.27370E+00_JPRB, 0.27449E+00_JPRB, 0.27527E+00_JPRB, 0.27599E+00_JPRB, 0.27664E+00_JPRB ,&
&  0.27705E+00_JPRB, 0.27756E+00_JPRB, 0.27618E+00_JPRB, 0.27534E+00_JPRB, 0.27523E+00_JPRB, 0.27533E+00_JPRB, 0.27530E+00_JPRB ,&
&  0.27522E+00_JPRB, 0.27508E+00_JPRB, 0.27458E+00_JPRB, 0.27423E+00_JPRB, 0.27407E+00_JPRB, 0.27386E+00_JPRB, 0.27353E+00_JPRB ,&
&  0.27312E+00_JPRB, 0.27281E+00_JPRB, 0.27243E+00_JPRB, 0.27199E+00_JPRB, 0.27187E+00_JPRB, 0.27172E+00_JPRB, 0.27114E+00_JPRB ,&
&  0.27044E+00_JPRB, 0.26942E+00_JPRB, 0.26848E+00_JPRB, 0.26762E+00_JPRB, 0.26692E+00_JPRB, 0.26658E+00_JPRB, 0.26622E+00_JPRB ,&
&  0.26572E+00_JPRB, 0.26505E+00_JPRB, 0.26440E+00_JPRB, 0.26360E+00_JPRB, 0.26289E+00_JPRB, 0.26224E+00_JPRB, 0.26149E+00_JPRB ,&
&  0.26066E+00_JPRB, 0.25981E+00_JPRB, 0.25893E+00_JPRB, 0.25796E+00_JPRB, 0.25701E+00_JPRB, 0.25605E+00_JPRB, 0.25499E+00_JPRB ,&
&  0.25396E+00_JPRB, 0.25352E+00_JPRB, 0.25301E+00_JPRB, 0.25257E+00_JPRB, 0.25225E+00_JPRB, 0.25200E+00_JPRB, 0.25175E+00_JPRB ,&
&  0.25155E+00_JPRB, 0.25134E+00_JPRB, 0.25124E+00_JPRB, 0.25114E+00_JPRB, 0.25114E+00_JPRB, 0.25121E+00_JPRB, 0.25140E+00_JPRB ,&
&  0.25161E+00_JPRB, 0.25188E+00_JPRB, 0.25225E+00_JPRB, 0.25286E+00_JPRB, 0.25347E+00_JPRB, 0.25417E+00_JPRB, 0.25486E+00_JPRB ,&
&  0.25562E+00_JPRB, 0.25637E+00_JPRB, 0.25714E+00_JPRB, 0.25787E+00_JPRB, 0.25885E+00_JPRB, 0.25979E+00_JPRB, 0.26083E+00_JPRB ,&
&  0.26186E+00_JPRB, 0.26297E+00_JPRB, 0.26407E+00_JPRB, 0.26525E+00_JPRB, 0.26660E+00_JPRB, 0.26698E+00_JPRB, 0.26738E+00_JPRB ,&
&  0.26789E+00_JPRB, 0.26862E+00_JPRB, 0.26930E+00_JPRB, 0.27020E+00_JPRB, 0.27173E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB, 0.26102E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB, 0.43452E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.43346E+00_JPRB, 0.43434E+00_JPRB, 0.43516E+00_JPRB, 0.43576E+00_JPRB, 0.43616E+00_JPRB, 0.43641E+00_JPRB, 0.43652E+00_JPRB ,&
&  0.43643E+00_JPRB, 0.43657E+00_JPRB, 0.43683E+00_JPRB, 0.43708E+00_JPRB, 0.43730E+00_JPRB, 0.43745E+00_JPRB, 0.43743E+00_JPRB ,&
&  0.43776E+00_JPRB, 0.43901E+00_JPRB, 0.43974E+00_JPRB, 0.44016E+00_JPRB, 0.44053E+00_JPRB, 0.44084E+00_JPRB, 0.44107E+00_JPRB ,&
&  0.44133E+00_JPRB, 0.44163E+00_JPRB, 0.44193E+00_JPRB, 0.44227E+00_JPRB, 0.44275E+00_JPRB, 0.44320E+00_JPRB, 0.44369E+00_JPRB ,&
&  0.44440E+00_JPRB, 0.44557E+00_JPRB, 0.44698E+00_JPRB, 0.44861E+00_JPRB, 0.45065E+00_JPRB, 0.45305E+00_JPRB, 0.45506E+00_JPRB ,&
&  0.45675E+00_JPRB, 0.45787E+00_JPRB, 0.45775E+00_JPRB, 0.45675E+00_JPRB, 0.45491E+00_JPRB, 0.45267E+00_JPRB, 0.45009E+00_JPRB ,&
&  0.44708E+00_JPRB, 0.44441E+00_JPRB, 0.44199E+00_JPRB, 0.43978E+00_JPRB, 0.43765E+00_JPRB, 0.43574E+00_JPRB, 0.43393E+00_JPRB ,&
&  0.43244E+00_JPRB, 0.43096E+00_JPRB, 0.42957E+00_JPRB, 0.42815E+00_JPRB, 0.42671E+00_JPRB, 0.42515E+00_JPRB, 0.42358E+00_JPRB ,&
&  0.42235E+00_JPRB, 0.42132E+00_JPRB, 0.42056E+00_JPRB, 0.41985E+00_JPRB, 0.41937E+00_JPRB, 0.41942E+00_JPRB, 0.41967E+00_JPRB ,&
&  0.41985E+00_JPRB, 0.42020E+00_JPRB, 0.42069E+00_JPRB, 0.42095E+00_JPRB, 0.42117E+00_JPRB, 0.42128E+00_JPRB, 0.42132E+00_JPRB ,&
&  0.42132E+00_JPRB, 0.42157E+00_JPRB, 0.42182E+00_JPRB, 0.42229E+00_JPRB, 0.42272E+00_JPRB, 0.42323E+00_JPRB, 0.42381E+00_JPRB ,&
&  0.42423E+00_JPRB, 0.42480E+00_JPRB, 0.42529E+00_JPRB, 0.42600E+00_JPRB, 0.42663E+00_JPRB, 0.42714E+00_JPRB, 0.42752E+00_JPRB ,&
&  0.42788E+00_JPRB, 0.42809E+00_JPRB, 0.42836E+00_JPRB, 0.42852E+00_JPRB, 0.42884E+00_JPRB, 0.42919E+00_JPRB, 0.42959E+00_JPRB ,&
&  0.43000E+00_JPRB, 0.43057E+00_JPRB, 0.43107E+00_JPRB, 0.43184E+00_JPRB, 0.43261E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.43614E+00_JPRB, 0.43738E+00_JPRB, 0.43814E+00_JPRB, 0.43847E+00_JPRB, 0.43864E+00_JPRB, 0.43825E+00_JPRB, 0.43723E+00_JPRB ,&
&  0.43557E+00_JPRB, 0.43411E+00_JPRB, 0.43297E+00_JPRB, 0.43163E+00_JPRB, 0.42978E+00_JPRB, 0.42942E+00_JPRB, 0.42954E+00_JPRB ,&
&  0.42827E+00_JPRB, 0.42670E+00_JPRB, 0.42508E+00_JPRB, 0.42387E+00_JPRB, 0.42253E+00_JPRB, 0.42105E+00_JPRB, 0.42006E+00_JPRB ,&
&  0.41956E+00_JPRB, 0.41883E+00_JPRB, 0.41768E+00_JPRB, 0.41603E+00_JPRB, 0.41560E+00_JPRB, 0.41513E+00_JPRB, 0.41452E+00_JPRB ,&
&  0.41366E+00_JPRB, 0.41190E+00_JPRB, 0.41039E+00_JPRB, 0.40970E+00_JPRB, 0.41235E+00_JPRB, 0.41572E+00_JPRB, 0.42015E+00_JPRB ,&
&  0.42571E+00_JPRB, 0.43070E+00_JPRB, 0.43643E+00_JPRB, 0.44172E+00_JPRB, 0.44556E+00_JPRB, 0.45039E+00_JPRB, 0.45372E+00_JPRB ,&
&  0.45493E+00_JPRB, 0.45385E+00_JPRB, 0.45278E+00_JPRB, 0.45263E+00_JPRB, 0.45268E+00_JPRB, 0.45231E+00_JPRB, 0.45138E+00_JPRB ,&
&  0.45031E+00_JPRB, 0.44909E+00_JPRB, 0.44753E+00_JPRB, 0.44520E+00_JPRB, 0.44221E+00_JPRB, 0.43918E+00_JPRB, 0.43592E+00_JPRB ,&
&  0.43303E+00_JPRB, 0.42967E+00_JPRB, 0.42651E+00_JPRB, 0.42341E+00_JPRB, 0.42066E+00_JPRB, 0.41758E+00_JPRB, 0.41465E+00_JPRB ,&
&  0.41277E+00_JPRB, 0.41259E+00_JPRB, 0.41295E+00_JPRB, 0.41252E+00_JPRB, 0.41243E+00_JPRB, 0.41242E+00_JPRB, 0.41319E+00_JPRB ,&
&  0.41414E+00_JPRB, 0.41600E+00_JPRB, 0.41803E+00_JPRB, 0.42068E+00_JPRB, 0.42303E+00_JPRB, 0.42525E+00_JPRB, 0.42693E+00_JPRB ,&
&  0.42847E+00_JPRB, 0.42960E+00_JPRB, 0.43044E+00_JPRB, 0.43115E+00_JPRB, 0.43182E+00_JPRB, 0.43227E+00_JPRB, 0.43234E+00_JPRB ,&
&  0.43225E+00_JPRB, 0.43197E+00_JPRB, 0.43156E+00_JPRB, 0.43090E+00_JPRB, 0.43054E+00_JPRB, 0.42975E+00_JPRB, 0.42956E+00_JPRB ,&
&  0.42957E+00_JPRB, 0.43030E+00_JPRB, 0.43123E+00_JPRB, 0.43276E+00_JPRB, 0.43446E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.43419E+00_JPRB, 0.43670E+00_JPRB, 0.43842E+00_JPRB, 0.43917E+00_JPRB, 0.43948E+00_JPRB, 0.43972E+00_JPRB, 0.43941E+00_JPRB ,&
&  0.43888E+00_JPRB, 0.43790E+00_JPRB, 0.43725E+00_JPRB, 0.43623E+00_JPRB, 0.43490E+00_JPRB, 0.43562E+00_JPRB, 0.43467E+00_JPRB ,&
&  0.43274E+00_JPRB, 0.43035E+00_JPRB, 0.42824E+00_JPRB, 0.42564E+00_JPRB, 0.42250E+00_JPRB, 0.41851E+00_JPRB, 0.41200E+00_JPRB ,&
&  0.40796E+00_JPRB, 0.40340E+00_JPRB, 0.39813E+00_JPRB, 0.39667E+00_JPRB, 0.39523E+00_JPRB, 0.37992E+00_JPRB, 0.37987E+00_JPRB ,&
&  0.39312E+00_JPRB, 0.39841E+00_JPRB, 0.40056E+00_JPRB, 0.40312E+00_JPRB, 0.38732E+00_JPRB, 0.36871E+00_JPRB, 0.35686E+00_JPRB ,&
&  0.33847E+00_JPRB, 0.35648E+00_JPRB, 0.36777E+00_JPRB, 0.37255E+00_JPRB, 0.37406E+00_JPRB, 0.37566E+00_JPRB, 0.39566E+00_JPRB ,&
&  0.43760E+00_JPRB, 0.44667E+00_JPRB, 0.45678E+00_JPRB, 0.46780E+00_JPRB, 0.47842E+00_JPRB, 0.48560E+00_JPRB, 0.49305E+00_JPRB ,&
&  0.49641E+00_JPRB, 0.49818E+00_JPRB, 0.49554E+00_JPRB, 0.49080E+00_JPRB, 0.48129E+00_JPRB, 0.47337E+00_JPRB, 0.46622E+00_JPRB ,&
&  0.46149E+00_JPRB, 0.45652E+00_JPRB, 0.45140E+00_JPRB, 0.44529E+00_JPRB, 0.43927E+00_JPRB, 0.43258E+00_JPRB, 0.42583E+00_JPRB ,&
&  0.41878E+00_JPRB, 0.41146E+00_JPRB, 0.40619E+00_JPRB, 0.40215E+00_JPRB, 0.40177E+00_JPRB, 0.40205E+00_JPRB, 0.40397E+00_JPRB ,&
&  0.40587E+00_JPRB, 0.40849E+00_JPRB, 0.41118E+00_JPRB, 0.41522E+00_JPRB, 0.41870E+00_JPRB, 0.42139E+00_JPRB, 0.42349E+00_JPRB ,&
&  0.42520E+00_JPRB, 0.42639E+00_JPRB, 0.42726E+00_JPRB, 0.42770E+00_JPRB, 0.42851E+00_JPRB, 0.42948E+00_JPRB, 0.42999E+00_JPRB ,&
&  0.43080E+00_JPRB, 0.43160E+00_JPRB, 0.43243E+00_JPRB, 0.43260E+00_JPRB, 0.43220E+00_JPRB, 0.43012E+00_JPRB, 0.42800E+00_JPRB ,&
&  0.42602E+00_JPRB, 0.42588E+00_JPRB, 0.42642E+00_JPRB, 0.42846E+00_JPRB, 0.43098E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.44469E+00_JPRB, 0.44647E+00_JPRB, 0.44862E+00_JPRB, 0.45003E+00_JPRB, 0.45037E+00_JPRB, 0.45014E+00_JPRB, 0.44950E+00_JPRB ,&
&  0.44866E+00_JPRB, 0.44816E+00_JPRB, 0.44708E+00_JPRB, 0.44512E+00_JPRB, 0.44372E+00_JPRB, 0.44219E+00_JPRB, 0.44031E+00_JPRB ,&
&  0.43710E+00_JPRB, 0.43378E+00_JPRB, 0.43097E+00_JPRB, 0.42740E+00_JPRB, 0.42411E+00_JPRB, 0.42135E+00_JPRB, 0.41647E+00_JPRB ,&
&  0.41310E+00_JPRB, 0.40658E+00_JPRB, 0.38283E+00_JPRB, 0.38314E+00_JPRB, 0.37821E+00_JPRB, 0.37733E+00_JPRB, 0.38008E+00_JPRB ,&
&  0.37428E+00_JPRB, 0.37365E+00_JPRB, 0.36593E+00_JPRB, 0.35417E+00_JPRB, 0.34764E+00_JPRB, 0.33692E+00_JPRB, 0.33679E+00_JPRB ,&
&  0.33489E+00_JPRB, 0.33856E+00_JPRB, 0.34711E+00_JPRB, 0.35132E+00_JPRB, 0.35264E+00_JPRB, 0.35435E+00_JPRB, 0.36370E+00_JPRB ,&
&  0.38938E+00_JPRB, 0.42806E+00_JPRB, 0.47888E+00_JPRB, 0.50234E+00_JPRB, 0.52199E+00_JPRB, 0.54152E+00_JPRB, 0.55937E+00_JPRB ,&
&  0.57160E+00_JPRB, 0.58148E+00_JPRB, 0.57999E+00_JPRB, 0.57632E+00_JPRB, 0.56334E+00_JPRB, 0.54548E+00_JPRB, 0.52468E+00_JPRB ,&
&  0.50511E+00_JPRB, 0.49088E+00_JPRB, 0.47990E+00_JPRB, 0.47310E+00_JPRB, 0.46873E+00_JPRB, 0.46486E+00_JPRB, 0.46242E+00_JPRB ,&
&  0.45578E+00_JPRB, 0.44269E+00_JPRB, 0.43087E+00_JPRB, 0.42043E+00_JPRB, 0.41659E+00_JPRB, 0.41507E+00_JPRB, 0.41742E+00_JPRB ,&
&  0.42031E+00_JPRB, 0.42566E+00_JPRB, 0.43216E+00_JPRB, 0.43472E+00_JPRB, 0.43884E+00_JPRB, 0.44403E+00_JPRB, 0.44407E+00_JPRB ,&
&  0.44395E+00_JPRB, 0.44333E+00_JPRB, 0.44161E+00_JPRB, 0.43924E+00_JPRB, 0.43643E+00_JPRB, 0.43359E+00_JPRB, 0.43140E+00_JPRB ,&
&  0.43072E+00_JPRB, 0.43162E+00_JPRB, 0.43526E+00_JPRB, 0.43948E+00_JPRB, 0.44463E+00_JPRB, 0.44715E+00_JPRB, 0.44618E+00_JPRB ,&
&  0.44172E+00_JPRB, 0.43964E+00_JPRB, 0.43819E+00_JPRB, 0.44024E+00_JPRB, 0.44241E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.45631E+00_JPRB, 0.46106E+00_JPRB, 0.46854E+00_JPRB, 0.47485E+00_JPRB, 0.47973E+00_JPRB, 0.47997E+00_JPRB, 0.47420E+00_JPRB ,&
&  0.46840E+00_JPRB, 0.46272E+00_JPRB, 0.45791E+00_JPRB, 0.45380E+00_JPRB, 0.45086E+00_JPRB, 0.44685E+00_JPRB, 0.44260E+00_JPRB ,&
&  0.43898E+00_JPRB, 0.43655E+00_JPRB, 0.43517E+00_JPRB, 0.43422E+00_JPRB, 0.43453E+00_JPRB, 0.43364E+00_JPRB, 0.43317E+00_JPRB ,&
&  0.43286E+00_JPRB, 0.42448E+00_JPRB, 0.40526E+00_JPRB, 0.39633E+00_JPRB, 0.38812E+00_JPRB, 0.38760E+00_JPRB, 0.38778E+00_JPRB ,&
&  0.38586E+00_JPRB, 0.34680E+00_JPRB, 0.35171E+00_JPRB, 0.34209E+00_JPRB, 0.33587E+00_JPRB, 0.33549E+00_JPRB, 0.32762E+00_JPRB ,&
&  0.32245E+00_JPRB, 0.31651E+00_JPRB, 0.31756E+00_JPRB, 0.32346E+00_JPRB, 0.32489E+00_JPRB, 0.33364E+00_JPRB, 0.34574E+00_JPRB ,&
&  0.37472E+00_JPRB, 0.47287E+00_JPRB, 0.51393E+00_JPRB, 0.53895E+00_JPRB, 0.55918E+00_JPRB, 0.58475E+00_JPRB, 0.60998E+00_JPRB ,&
&  0.62779E+00_JPRB, 0.65070E+00_JPRB, 0.65756E+00_JPRB, 0.65018E+00_JPRB, 0.65006E+00_JPRB, 0.63836E+00_JPRB, 0.61403E+00_JPRB ,&
&  0.58258E+00_JPRB, 0.54546E+00_JPRB, 0.52114E+00_JPRB, 0.50609E+00_JPRB, 0.49394E+00_JPRB, 0.48275E+00_JPRB, 0.47482E+00_JPRB ,&
&  0.47629E+00_JPRB, 0.46956E+00_JPRB, 0.46199E+00_JPRB, 0.44920E+00_JPRB, 0.44211E+00_JPRB, 0.43826E+00_JPRB, 0.43842E+00_JPRB ,&
&  0.44073E+00_JPRB, 0.44604E+00_JPRB, 0.45337E+00_JPRB, 0.45977E+00_JPRB, 0.46314E+00_JPRB, 0.46287E+00_JPRB, 0.46594E+00_JPRB ,&
&  0.46666E+00_JPRB, 0.46533E+00_JPRB, 0.46332E+00_JPRB, 0.46062E+00_JPRB, 0.45757E+00_JPRB, 0.45548E+00_JPRB, 0.45648E+00_JPRB ,&
&  0.45824E+00_JPRB, 0.45941E+00_JPRB, 0.46019E+00_JPRB, 0.46231E+00_JPRB, 0.46639E+00_JPRB, 0.47006E+00_JPRB, 0.47132E+00_JPRB ,&
&  0.46526E+00_JPRB, 0.45922E+00_JPRB, 0.45386E+00_JPRB, 0.45314E+00_JPRB, 0.45352E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.44779E+00_JPRB, 0.46087E+00_JPRB, 0.48005E+00_JPRB, 0.49531E+00_JPRB, 0.50710E+00_JPRB, 0.50583E+00_JPRB, 0.49679E+00_JPRB ,&
&  0.48726E+00_JPRB, 0.47964E+00_JPRB, 0.47262E+00_JPRB, 0.46572E+00_JPRB, 0.45753E+00_JPRB, 0.45114E+00_JPRB, 0.44596E+00_JPRB ,&
&  0.44302E+00_JPRB, 0.44159E+00_JPRB, 0.44074E+00_JPRB, 0.43986E+00_JPRB, 0.43970E+00_JPRB, 0.44072E+00_JPRB, 0.44108E+00_JPRB ,&
&  0.44205E+00_JPRB, 0.43627E+00_JPRB, 0.42048E+00_JPRB, 0.40540E+00_JPRB, 0.39839E+00_JPRB, 0.39012E+00_JPRB, 0.39234E+00_JPRB ,&
&  0.39166E+00_JPRB, 0.39063E+00_JPRB, 0.38590E+00_JPRB, 0.38418E+00_JPRB, 0.37194E+00_JPRB, 0.32225E+00_JPRB, 0.32481E+00_JPRB ,&
&  0.31505E+00_JPRB, 0.30444E+00_JPRB, 0.30303E+00_JPRB, 0.30378E+00_JPRB, 0.30639E+00_JPRB, 0.32335E+00_JPRB, 0.34372E+00_JPRB ,&
&  0.37265E+00_JPRB, 0.48313E+00_JPRB, 0.51256E+00_JPRB, 0.54432E+00_JPRB, 0.57269E+00_JPRB, 0.60929E+00_JPRB, 0.64959E+00_JPRB ,&
&  0.67300E+00_JPRB, 0.69829E+00_JPRB, 0.71378E+00_JPRB, 0.72262E+00_JPRB, 0.71882E+00_JPRB, 0.71310E+00_JPRB, 0.70648E+00_JPRB ,&
&  0.68816E+00_JPRB, 0.62631E+00_JPRB, 0.58364E+00_JPRB, 0.55869E+00_JPRB, 0.53678E+00_JPRB, 0.52061E+00_JPRB, 0.50748E+00_JPRB ,&
&  0.50106E+00_JPRB, 0.48271E+00_JPRB, 0.47718E+00_JPRB, 0.46857E+00_JPRB, 0.46389E+00_JPRB, 0.46043E+00_JPRB, 0.45617E+00_JPRB ,&
&  0.45010E+00_JPRB, 0.45044E+00_JPRB, 0.45812E+00_JPRB, 0.46202E+00_JPRB, 0.46387E+00_JPRB, 0.46103E+00_JPRB, 0.45680E+00_JPRB ,&
&  0.44966E+00_JPRB, 0.45033E+00_JPRB, 0.44748E+00_JPRB, 0.44659E+00_JPRB, 0.44719E+00_JPRB, 0.45339E+00_JPRB, 0.46138E+00_JPRB ,&
&  0.46751E+00_JPRB, 0.46972E+00_JPRB, 0.46952E+00_JPRB, 0.46992E+00_JPRB, 0.46906E+00_JPRB, 0.46719E+00_JPRB, 0.46297E+00_JPRB ,&
&  0.45904E+00_JPRB, 0.45637E+00_JPRB, 0.45081E+00_JPRB, 0.44584E+00_JPRB, 0.44357E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.44318E+00_JPRB, 0.46730E+00_JPRB, 0.49501E+00_JPRB, 0.51439E+00_JPRB, 0.52406E+00_JPRB, 0.51257E+00_JPRB, 0.49824E+00_JPRB ,&
&  0.48655E+00_JPRB, 0.47750E+00_JPRB, 0.47119E+00_JPRB, 0.46637E+00_JPRB, 0.46047E+00_JPRB, 0.45481E+00_JPRB, 0.44956E+00_JPRB ,&
&  0.44562E+00_JPRB, 0.43682E+00_JPRB, 0.43262E+00_JPRB, 0.43305E+00_JPRB, 0.43377E+00_JPRB, 0.43693E+00_JPRB, 0.44017E+00_JPRB ,&
&  0.43938E+00_JPRB, 0.43264E+00_JPRB, 0.42623E+00_JPRB, 0.41285E+00_JPRB, 0.39892E+00_JPRB, 0.39227E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.38638E+00_JPRB, 0.37156E+00_JPRB, 0.38462E+00_JPRB, 0.39502E+00_JPRB, 0.37955E+00_JPRB, 0.36944E+00_JPRB, 0.33366E+00_JPRB ,&
&  0.32028E+00_JPRB, 0.30328E+00_JPRB, 0.29928E+00_JPRB, 0.29996E+00_JPRB, 0.30797E+00_JPRB, 0.32429E+00_JPRB, 0.35984E+00_JPRB ,&
&  0.41288E+00_JPRB, 0.49228E+00_JPRB, 0.51395E+00_JPRB, 0.55359E+00_JPRB, 0.58999E+00_JPRB, 0.62223E+00_JPRB, 0.64566E+00_JPRB ,&
&  0.68163E+00_JPRB, 0.72901E+00_JPRB, 0.75402E+00_JPRB, 0.75285E+00_JPRB, 0.75480E+00_JPRB, 0.76333E+00_JPRB, 0.78620E+00_JPRB ,&
&  0.82383E+00_JPRB, 0.72044E+00_JPRB, 0.65327E+00_JPRB, 0.60723E+00_JPRB, 0.57728E+00_JPRB, 0.56437E+00_JPRB, 0.52607E+00_JPRB ,&
&  0.51043E+00_JPRB, 0.50504E+00_JPRB, 0.49565E+00_JPRB, 0.49004E+00_JPRB, 0.48561E+00_JPRB, 0.48192E+00_JPRB, 0.47492E+00_JPRB ,&
&  0.46580E+00_JPRB, 0.46725E+00_JPRB, 0.50581E+00_JPRB, 0.51326E+00_JPRB, 0.50247E+00_JPRB, 0.47448E+00_JPRB, 0.44682E+00_JPRB ,&
&  0.42866E+00_JPRB, 0.42247E+00_JPRB, 0.42078E+00_JPRB, 0.41975E+00_JPRB, 0.42360E+00_JPRB, 0.43050E+00_JPRB, 0.44212E+00_JPRB ,&
&  0.44486E+00_JPRB, 0.45331E+00_JPRB, 0.46120E+00_JPRB, 0.46493E+00_JPRB, 0.46375E+00_JPRB, 0.45995E+00_JPRB, 0.45690E+00_JPRB ,&
&  0.45411E+00_JPRB, 0.45108E+00_JPRB, 0.44669E+00_JPRB, 0.44058E+00_JPRB, 0.43641E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.43171E+00_JPRB, 0.46506E+00_JPRB, 0.50874E+00_JPRB, 0.53876E+00_JPRB, 0.54035E+00_JPRB, 0.51431E+00_JPRB, 0.49003E+00_JPRB ,&
&  0.47530E+00_JPRB, 0.46867E+00_JPRB, 0.46645E+00_JPRB, 0.46212E+00_JPRB, 0.45737E+00_JPRB, 0.45225E+00_JPRB, 0.44718E+00_JPRB ,&
&  0.44284E+00_JPRB, 0.43885E+00_JPRB, 0.43363E+00_JPRB, 0.42657E+00_JPRB, 0.42202E+00_JPRB, 0.42157E+00_JPRB, 0.42350E+00_JPRB ,&
&  0.42700E+00_JPRB, 0.42562E+00_JPRB, 0.42319E+00_JPRB, 0.41583E+00_JPRB, 0.40534E+00_JPRB, 0.39393E+00_JPRB, 0.38475E+00_JPRB ,&
&  0.38003E+00_JPRB, 0.38061E+00_JPRB, 0.38012E+00_JPRB, 0.39494E+00_JPRB, 0.39053E+00_JPRB, 0.38211E+00_JPRB, 0.35724E+00_JPRB ,&
&  0.33615E+00_JPRB, 0.31323E+00_JPRB, 0.30460E+00_JPRB, 0.30818E+00_JPRB, 0.32910E+00_JPRB, 0.36071E+00_JPRB, 0.41532E+00_JPRB ,&
&  0.51346E+00_JPRB, 0.56375E+00_JPRB, 0.60369E+00_JPRB, 0.64111E+00_JPRB, 0.67042E+00_JPRB, 0.70127E+00_JPRB, 0.73727E+00_JPRB ,&
&  0.78209E+00_JPRB, 0.81808E+00_JPRB, 0.83440E+00_JPRB, 0.80490E+00_JPRB, 0.78699E+00_JPRB, 0.71973E+00_JPRB, 0.77754E+00_JPRB ,&
&  0.10020E+01_JPRB, 0.85092E+00_JPRB, 0.74493E+00_JPRB, 0.68855E+00_JPRB, 0.64804E+00_JPRB, 0.61268E+00_JPRB, 0.57992E+00_JPRB ,&
&  0.54948E+00_JPRB, 0.53171E+00_JPRB, 0.52173E+00_JPRB, 0.49637E+00_JPRB, 0.47210E+00_JPRB, 0.46983E+00_JPRB, 0.49443E+00_JPRB ,&
&  0.55764E+00_JPRB, 0.65814E+00_JPRB, 0.80382E+00_JPRB, 0.56703E+00_JPRB, 0.49326E+00_JPRB, 0.44301E+00_JPRB, 0.42185E+00_JPRB ,&
&  0.41696E+00_JPRB, 0.41703E+00_JPRB, 0.41853E+00_JPRB, 0.41574E+00_JPRB, 0.41251E+00_JPRB, 0.41385E+00_JPRB, 0.41570E+00_JPRB ,&
&  0.41670E+00_JPRB, 0.42495E+00_JPRB, 0.43683E+00_JPRB, 0.44639E+00_JPRB, 0.45278E+00_JPRB, 0.45521E+00_JPRB, 0.45761E+00_JPRB ,&
&  0.45704E+00_JPRB, 0.45230E+00_JPRB, 0.44592E+00_JPRB, 0.43335E+00_JPRB, 0.42479E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.42598E+00_JPRB, 0.46584E+00_JPRB, 0.52366E+00_JPRB, 0.55682E+00_JPRB, 0.55686E+00_JPRB, 0.51602E+00_JPRB, 0.47256E+00_JPRB ,&
&  0.44762E+00_JPRB, 0.43223E+00_JPRB, 0.42945E+00_JPRB, 0.42703E+00_JPRB, 0.42559E+00_JPRB, 0.42465E+00_JPRB, 0.42341E+00_JPRB ,&
&  0.42081E+00_JPRB, 0.41748E+00_JPRB, 0.41369E+00_JPRB, 0.41074E+00_JPRB, 0.40733E+00_JPRB, 0.40448E+00_JPRB, 0.40382E+00_JPRB ,&
&  0.40526E+00_JPRB, 0.40520E+00_JPRB, 0.40385E+00_JPRB, 0.40178E+00_JPRB, 0.40187E+00_JPRB, 0.40462E+00_JPRB, 0.40329E+00_JPRB ,&
&  0.39220E+00_JPRB, 0.38247E+00_JPRB, 0.38837E+00_JPRB, 0.40341E+00_JPRB, 0.40026E+00_JPRB, 0.39553E+00_JPRB, 0.36925E+00_JPRB ,&
&  0.34342E+00_JPRB, 0.32574E+00_JPRB, 0.33583E+00_JPRB, 0.37390E+00_JPRB, 0.43730E+00_JPRB, 0.56252E+00_JPRB, 0.63851E+00_JPRB ,&
&  0.70672E+00_JPRB, 0.74853E+00_JPRB, 0.77423E+00_JPRB, 0.80015E+00_JPRB, 0.84933E+00_JPRB, 0.89930E+00_JPRB, 0.93445E+00_JPRB ,&
&  0.95716E+00_JPRB, 0.96426E+00_JPRB, 0.96250E+00_JPRB, 0.73073E+00_JPRB, 0.69463E+00_JPRB, 0.73852E+00_JPRB, 0.79496E+00_JPRB ,&
&  0.82810E+00_JPRB, 0.77738E+00_JPRB, 0.75366E+00_JPRB, 0.76364E+00_JPRB, 0.71496E+00_JPRB, 0.67586E+00_JPRB, 0.64222E+00_JPRB ,&
&  0.61061E+00_JPRB, 0.59357E+00_JPRB, 0.55205E+00_JPRB, 0.51197E+00_JPRB, 0.51060E+00_JPRB, 0.51220E+00_JPRB, 0.51728E+00_JPRB ,&
&  0.53713E+00_JPRB, 0.55017E+00_JPRB, 0.51140E+00_JPRB, 0.41818E+00_JPRB, 0.39413E+00_JPRB, 0.38527E+00_JPRB, 0.38627E+00_JPRB ,&
&  0.38995E+00_JPRB, 0.39387E+00_JPRB, 0.39924E+00_JPRB, 0.40001E+00_JPRB, 0.39785E+00_JPRB, 0.39645E+00_JPRB, 0.39442E+00_JPRB ,&
&  0.39065E+00_JPRB, 0.39219E+00_JPRB, 0.39966E+00_JPRB, 0.40539E+00_JPRB, 0.41907E+00_JPRB, 0.43197E+00_JPRB, 0.44347E+00_JPRB ,&
&  0.44128E+00_JPRB, 0.43775E+00_JPRB, 0.43257E+00_JPRB, 0.42000E+00_JPRB, 0.41194E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.46587E+00_JPRB, 0.50523E+00_JPRB, 0.53868E+00_JPRB, 0.56261E+00_JPRB, 0.55546E+00_JPRB, 0.52922E+00_JPRB, 0.47407E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.43102E+00_JPRB, 0.42223E+00_JPRB, 0.40924E+00_JPRB, 0.39860E+00_JPRB, 0.38997E+00_JPRB, 0.38650E+00_JPRB ,&
&  0.38888E+00_JPRB, 0.39264E+00_JPRB, 0.39499E+00_JPRB, 0.39589E+00_JPRB, 0.39379E+00_JPRB, 0.39224E+00_JPRB, 0.39162E+00_JPRB ,&
&  0.39135E+00_JPRB, 0.39140E+00_JPRB, 0.39276E+00_JPRB, 0.39467E+00_JPRB, 0.39785E+00_JPRB, 0.40078E+00_JPRB, 0.39872E+00_JPRB ,&
&  0.39111E+00_JPRB, 0.38470E+00_JPRB, 0.39569E+00_JPRB, 0.41220E+00_JPRB, 0.40931E+00_JPRB, 0.41006E+00_JPRB, 0.38080E+00_JPRB ,&
&  0.35287E+00_JPRB, 0.36482E+00_JPRB, 0.43999E+00_JPRB, 0.59821E+00_JPRB, 0.65691E+00_JPRB, 0.71675E+00_JPRB, 0.76130E+00_JPRB ,&
&  0.72277E+00_JPRB, 0.73523E+00_JPRB, 0.79613E+00_JPRB, 0.97171E+00_JPRB, 0.10021E+01_JPRB, 0.10409E+01_JPRB, 0.10811E+01_JPRB ,&
&  0.11368E+01_JPRB, 0.11219E+01_JPRB, 0.10806E+01_JPRB, 0.88586E+00_JPRB, 0.98625E+00_JPRB, 0.11613E+01_JPRB, 0.99442E+00_JPRB ,&
&  0.95307E+00_JPRB, 0.91531E+00_JPRB, 0.92667E+00_JPRB, 0.86838E+00_JPRB, 0.78305E+00_JPRB, 0.73067E+00_JPRB, 0.70792E+00_JPRB ,&
&  0.67095E+00_JPRB, 0.64489E+00_JPRB, 0.58043E+00_JPRB, 0.56144E+00_JPRB, 0.56784E+00_JPRB, 0.54892E+00_JPRB, 0.53051E+00_JPRB ,&
&  0.51211E+00_JPRB, 0.47755E+00_JPRB, 0.44807E+00_JPRB, 0.41685E+00_JPRB, 0.39714E+00_JPRB, 0.39246E+00_JPRB, 0.39366E+00_JPRB ,&
&  0.39504E+00_JPRB, 0.39453E+00_JPRB, 0.39699E+00_JPRB, 0.39946E+00_JPRB, 0.39628E+00_JPRB, 0.39193E+00_JPRB, 0.39289E+00_JPRB ,&
&  0.39074E+00_JPRB, 0.38470E+00_JPRB, 0.38051E+00_JPRB, 0.38727E+00_JPRB, 0.39681E+00_JPRB, 0.41057E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.43201E+00_JPRB, 0.43954E+00_JPRB, 0.43704E+00_JPRB, 0.43870E+00_JPRB, 0.44025E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.49861E+00_JPRB, 0.52066E+00_JPRB, 0.54643E+00_JPRB, 0.56550E+00_JPRB, 0.57914E+00_JPRB, 0.52705E+00_JPRB, 0.48033E+00_JPRB ,&
&  0.50508E+00_JPRB, 0.44326E+00_JPRB, 0.41746E+00_JPRB, 0.40359E+00_JPRB, 0.39851E+00_JPRB, 0.38689E+00_JPRB, 0.37631E+00_JPRB ,&
&  0.37239E+00_JPRB, 0.37566E+00_JPRB, 0.38315E+00_JPRB, 0.38678E+00_JPRB, 0.38737E+00_JPRB, 0.38724E+00_JPRB, 0.38644E+00_JPRB ,&
&  0.38655E+00_JPRB, 0.38790E+00_JPRB, 0.38968E+00_JPRB, 0.39199E+00_JPRB, 0.39274E+00_JPRB, 0.39397E+00_JPRB, 0.39290E+00_JPRB ,&
&  0.38788E+00_JPRB, 0.38776E+00_JPRB, 0.40012E+00_JPRB, 0.42079E+00_JPRB, 0.42825E+00_JPRB, 0.43760E+00_JPRB, 0.43914E+00_JPRB ,&
&  0.38378E+00_JPRB, 0.43833E+00_JPRB, 0.63975E+00_JPRB, 0.70579E+00_JPRB, 0.75376E+00_JPRB, 0.81866E+00_JPRB, 0.87572E+00_JPRB ,&
&  0.91940E+00_JPRB, 0.95609E+00_JPRB, 0.98656E+00_JPRB, 0.10274E+01_JPRB, 0.10814E+01_JPRB, 0.11276E+01_JPRB, 0.11858E+01_JPRB ,&
&  0.13321E+01_JPRB, 0.10199E+01_JPRB, 0.92160E+00_JPRB, 0.96421E+00_JPRB, 0.13611E+01_JPRB, 0.13872E+01_JPRB, 0.11812E+01_JPRB ,&
&  0.10967E+01_JPRB, 0.10230E+01_JPRB, 0.97362E+00_JPRB, 0.89831E+00_JPRB, 0.83384E+00_JPRB, 0.80333E+00_JPRB, 0.78291E+00_JPRB ,&
&  0.72868E+00_JPRB, 0.70283E+00_JPRB, 0.63066E+00_JPRB, 0.61753E+00_JPRB, 0.61718E+00_JPRB, 0.58482E+00_JPRB, 0.53833E+00_JPRB ,&
&  0.50117E+00_JPRB, 0.47126E+00_JPRB, 0.44894E+00_JPRB, 0.43379E+00_JPRB, 0.42299E+00_JPRB, 0.41636E+00_JPRB, 0.41164E+00_JPRB ,&
&  0.40907E+00_JPRB, 0.40652E+00_JPRB, 0.39796E+00_JPRB, 0.39770E+00_JPRB, 0.39879E+00_JPRB, 0.40100E+00_JPRB, 0.40928E+00_JPRB ,&
&  0.41021E+00_JPRB, 0.40526E+00_JPRB, 0.39678E+00_JPRB, 0.40269E+00_JPRB, 0.41606E+00_JPRB, 0.43823E+00_JPRB, 0.46151E+00_JPRB ,&
&  0.48693E+00_JPRB, 0.48901E+00_JPRB, 0.46607E+00_JPRB, 0.46088E+00_JPRB, 0.46819E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.52194E+00_JPRB, 0.54141E+00_JPRB, 0.55568E+00_JPRB, 0.57340E+00_JPRB, 0.58859E+00_JPRB, 0.54253E+00_JPRB, 0.51305E+00_JPRB ,&
&  0.57554E+00_JPRB, 0.49806E+00_JPRB, 0.54618E+00_JPRB, 0.45737E+00_JPRB, 0.41721E+00_JPRB, 0.40029E+00_JPRB, 0.38724E+00_JPRB ,&
&  0.37512E+00_JPRB, 0.38026E+00_JPRB, 0.38642E+00_JPRB, 0.38841E+00_JPRB, 0.39432E+00_JPRB, 0.39065E+00_JPRB, 0.38739E+00_JPRB ,&
&  0.38541E+00_JPRB, 0.38567E+00_JPRB, 0.39109E+00_JPRB, 0.39099E+00_JPRB, 0.38989E+00_JPRB, 0.38848E+00_JPRB, 0.38371E+00_JPRB ,&
&  0.37330E+00_JPRB, 0.37709E+00_JPRB, 0.39928E+00_JPRB, 0.42026E+00_JPRB, 0.43951E+00_JPRB, 0.45698E+00_JPRB, 0.48804E+00_JPRB ,&
&  0.51688E+00_JPRB, 0.53934E+00_JPRB, 0.68100E+00_JPRB, 0.72295E+00_JPRB, 0.77219E+00_JPRB, 0.80800E+00_JPRB, 0.85174E+00_JPRB ,&
&  0.89637E+00_JPRB, 0.93453E+00_JPRB, 0.95818E+00_JPRB, 0.10028E+01_JPRB, 0.10889E+01_JPRB, 0.11456E+01_JPRB, 0.12383E+01_JPRB ,&
&  0.14190E+01_JPRB, 0.10365E+01_JPRB, 0.12186E+01_JPRB, 0.11944E+01_JPRB, 0.14059E+01_JPRB, 0.15492E+01_JPRB, 0.14969E+01_JPRB ,&
&  0.15468E+01_JPRB, 0.13091E+01_JPRB, 0.11476E+01_JPRB, 0.10016E+01_JPRB, 0.89413E+00_JPRB, 0.86162E+00_JPRB, 0.84221E+00_JPRB ,&
&  0.85604E+00_JPRB, 0.83434E+00_JPRB, 0.71770E+00_JPRB, 0.66818E+00_JPRB, 0.65171E+00_JPRB, 0.63811E+00_JPRB, 0.58782E+00_JPRB ,&
&  0.55871E+00_JPRB, 0.54007E+00_JPRB, 0.51569E+00_JPRB, 0.49123E+00_JPRB, 0.47230E+00_JPRB, 0.46080E+00_JPRB, 0.44522E+00_JPRB ,&
&  0.44098E+00_JPRB, 0.42950E+00_JPRB, 0.40091E+00_JPRB, 0.38803E+00_JPRB, 0.38485E+00_JPRB, 0.38112E+00_JPRB, 0.39227E+00_JPRB ,&
&  0.40523E+00_JPRB, 0.40771E+00_JPRB, 0.43521E+00_JPRB, 0.47270E+00_JPRB, 0.48475E+00_JPRB, 0.49451E+00_JPRB, 0.55010E+00_JPRB ,&
&  0.51705E+00_JPRB, 0.52855E+00_JPRB, 0.53153E+00_JPRB, 0.51039E+00_JPRB, 0.50644E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.54943E+00_JPRB, 0.55758E+00_JPRB, 0.56497E+00_JPRB, 0.58273E+00_JPRB, 0.59529E+00_JPRB, 0.59732E+00_JPRB, 0.56106E+00_JPRB ,&
&  0.60170E+00_JPRB, 0.58062E+00_JPRB, 0.56202E+00_JPRB, 0.55806E+00_JPRB, 0.54786E+00_JPRB, 0.47391E+00_JPRB, 0.39796E+00_JPRB ,&
&  0.37616E+00_JPRB, 0.37242E+00_JPRB, 0.38763E+00_JPRB, 0.38983E+00_JPRB, 0.39437E+00_JPRB, 0.39413E+00_JPRB, 0.38965E+00_JPRB ,&
&  0.38853E+00_JPRB, 0.38810E+00_JPRB, 0.39413E+00_JPRB, 0.39504E+00_JPRB, 0.39030E+00_JPRB, 0.38858E+00_JPRB, 0.38656E+00_JPRB ,&
&  0.37156E+00_JPRB, 0.36833E+00_JPRB, 0.38252E+00_JPRB, 0.38735E+00_JPRB, 0.43984E+00_JPRB, 0.46688E+00_JPRB, 0.50386E+00_JPRB ,&
&  0.54204E+00_JPRB, 0.58391E+00_JPRB, 0.65427E+00_JPRB, 0.70934E+00_JPRB, 0.74669E+00_JPRB, 0.77555E+00_JPRB, 0.81828E+00_JPRB ,&
&  0.85283E+00_JPRB, 0.89924E+00_JPRB, 0.95231E+00_JPRB, 0.10376E+01_JPRB, 0.11639E+01_JPRB, 0.10603E+01_JPRB, 0.12563E+01_JPRB ,&
&  0.13529E+01_JPRB, 0.15088E+01_JPRB, 0.16429E+01_JPRB, 0.15056E+01_JPRB, 0.16612E+01_JPRB, 0.17342E+01_JPRB, 0.16422E+01_JPRB ,&
&  0.15324E+01_JPRB, 0.13205E+01_JPRB, 0.12575E+01_JPRB, 0.12055E+01_JPRB, 0.11814E+01_JPRB, 0.10881E+01_JPRB, 0.10030E+01_JPRB ,&
&  0.10477E+01_JPRB, 0.11964E+01_JPRB, 0.93781E+00_JPRB, 0.85782E+00_JPRB, 0.79021E+00_JPRB, 0.76754E+00_JPRB, 0.72506E+00_JPRB ,&
&  0.70489E+00_JPRB, 0.70297E+00_JPRB, 0.67704E+00_JPRB, 0.64196E+00_JPRB, 0.57436E+00_JPRB, 0.54327E+00_JPRB, 0.50648E+00_JPRB ,&
&  0.48604E+00_JPRB, 0.45522E+00_JPRB, 0.41201E+00_JPRB, 0.38741E+00_JPRB, 0.37239E+00_JPRB, 0.36802E+00_JPRB, 0.38401E+00_JPRB ,&
&  0.42256E+00_JPRB, 0.46172E+00_JPRB, 0.54819E+00_JPRB, 0.57119E+00_JPRB, 0.58678E+00_JPRB, 0.61903E+00_JPRB, 0.55018E+00_JPRB ,&
&  0.51219E+00_JPRB, 0.57799E+00_JPRB, 0.56112E+00_JPRB, 0.54582E+00_JPRB, 0.54273E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.59503E+00_JPRB, 0.59694E+00_JPRB, 0.59756E+00_JPRB, 0.60452E+00_JPRB, 0.61266E+00_JPRB, 0.60448E+00_JPRB, 0.60298E+00_JPRB ,&
&  0.60911E+00_JPRB, 0.60378E+00_JPRB, 0.58572E+00_JPRB, 0.57360E+00_JPRB, 0.55800E+00_JPRB, 0.55437E+00_JPRB, 0.48477E+00_JPRB ,&
&  0.41254E+00_JPRB, 0.39430E+00_JPRB, 0.38052E+00_JPRB, 0.38469E+00_JPRB, 0.40141E+00_JPRB, 0.40000E+00_JPRB, 0.39808E+00_JPRB ,&
&  0.39735E+00_JPRB, 0.39605E+00_JPRB, 0.39726E+00_JPRB, 0.39706E+00_JPRB, 0.39847E+00_JPRB, 0.40790E+00_JPRB, 0.41101E+00_JPRB ,&
&  0.38708E+00_JPRB, 0.36309E+00_JPRB, 0.36038E+00_JPRB, 0.37494E+00_JPRB, 0.42301E+00_JPRB, 0.47352E+00_JPRB, 0.51382E+00_JPRB ,&
&  0.55132E+00_JPRB, 0.59148E+00_JPRB, 0.63911E+00_JPRB, 0.69603E+00_JPRB, 0.73246E+00_JPRB, 0.76783E+00_JPRB, 0.80683E+00_JPRB ,&
&  0.85105E+00_JPRB, 0.91324E+00_JPRB, 0.99439E+00_JPRB, 0.11145E+01_JPRB, 0.11132E+01_JPRB, 0.11416E+01_JPRB, 0.12795E+01_JPRB ,&
&  0.13830E+01_JPRB, 0.15041E+01_JPRB, 0.16081E+01_JPRB, 0.18379E+01_JPRB, 0.19690E+01_JPRB, 0.18718E+01_JPRB, 0.17124E+01_JPRB ,&
&  0.15872E+01_JPRB, 0.13897E+01_JPRB, 0.15468E+01_JPRB, 0.14005E+01_JPRB, 0.12869E+01_JPRB, 0.12701E+01_JPRB, 0.11995E+01_JPRB ,&
&  0.11792E+01_JPRB, 0.12296E+01_JPRB, 0.11853E+01_JPRB, 0.11670E+01_JPRB, 0.10819E+01_JPRB, 0.10629E+01_JPRB, 0.98944E+00_JPRB ,&
&  0.99106E+00_JPRB, 0.98038E+00_JPRB, 0.84431E+00_JPRB, 0.72676E+00_JPRB, 0.61946E+00_JPRB, 0.59524E+00_JPRB, 0.54813E+00_JPRB ,&
&  0.51246E+00_JPRB, 0.46911E+00_JPRB, 0.43520E+00_JPRB, 0.39964E+00_JPRB, 0.39142E+00_JPRB, 0.40031E+00_JPRB, 0.41665E+00_JPRB ,&
&  0.46722E+00_JPRB, 0.56055E+00_JPRB, 0.59989E+00_JPRB, 0.62771E+00_JPRB, 0.64749E+00_JPRB, 0.67725E+00_JPRB, 0.54873E+00_JPRB ,&
&  0.54430E+00_JPRB, 0.60156E+00_JPRB, 0.59621E+00_JPRB, 0.58784E+00_JPRB, 0.58649E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.66884E+00_JPRB, 0.66428E+00_JPRB, 0.65685E+00_JPRB, 0.64905E+00_JPRB, 0.63977E+00_JPRB, 0.62892E+00_JPRB, 0.62978E+00_JPRB ,&
&  0.63237E+00_JPRB, 0.62183E+00_JPRB, 0.60606E+00_JPRB, 0.59329E+00_JPRB, 0.58021E+00_JPRB, 0.57514E+00_JPRB, 0.58553E+00_JPRB ,&
&  0.47661E+00_JPRB, 0.43243E+00_JPRB, 0.38526E+00_JPRB, 0.38073E+00_JPRB, 0.43244E+00_JPRB, 0.41395E+00_JPRB, 0.40609E+00_JPRB ,&
&  0.39961E+00_JPRB, 0.40427E+00_JPRB, 0.40330E+00_JPRB, 0.41924E+00_JPRB, 0.43585E+00_JPRB, 0.45634E+00_JPRB, 0.45755E+00_JPRB ,&
&  0.42421E+00_JPRB, 0.38655E+00_JPRB, 0.37698E+00_JPRB, 0.38417E+00_JPRB, 0.42377E+00_JPRB, 0.46286E+00_JPRB, 0.53194E+00_JPRB ,&
&  0.58039E+00_JPRB, 0.62375E+00_JPRB, 0.66509E+00_JPRB, 0.71864E+00_JPRB, 0.76001E+00_JPRB, 0.80648E+00_JPRB, 0.86380E+00_JPRB ,&
&  0.92772E+00_JPRB, 0.10051E+01_JPRB, 0.10861E+01_JPRB, 0.11874E+01_JPRB, 0.12080E+01_JPRB, 0.12557E+01_JPRB, 0.13123E+01_JPRB ,&
&  0.13823E+01_JPRB, 0.14262E+01_JPRB, 0.15434E+01_JPRB, 0.20218E+01_JPRB, 0.23167E+01_JPRB, 0.23097E+01_JPRB, 0.20481E+01_JPRB ,&
&  0.17767E+01_JPRB, 0.15162E+01_JPRB, 0.14698E+01_JPRB, 0.13682E+01_JPRB, 0.12700E+01_JPRB, 0.13244E+01_JPRB, 0.12833E+01_JPRB ,&
&  0.12309E+01_JPRB, 0.12514E+01_JPRB, 0.12629E+01_JPRB, 0.13077E+01_JPRB, 0.12551E+01_JPRB, 0.12660E+01_JPRB, 0.12309E+01_JPRB ,&
&  0.12426E+01_JPRB, 0.11447E+01_JPRB, 0.88029E+00_JPRB, 0.75254E+00_JPRB, 0.63126E+00_JPRB, 0.59171E+00_JPRB, 0.67027E+00_JPRB ,&
&  0.60133E+00_JPRB, 0.55472E+00_JPRB, 0.51381E+00_JPRB, 0.48045E+00_JPRB, 0.48857E+00_JPRB, 0.54302E+00_JPRB, 0.51304E+00_JPRB ,&
&  0.50940E+00_JPRB, 0.59512E+00_JPRB, 0.70544E+00_JPRB, 0.78899E+00_JPRB, 0.82620E+00_JPRB, 0.81958E+00_JPRB, 0.69570E+00_JPRB ,&
&  0.70080E+00_JPRB, 0.68169E+00_JPRB, 0.67866E+00_JPRB, 0.66667E+00_JPRB, 0.66455E+00_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.77535E+00_JPRB, 0.76276E+00_JPRB, 0.74184E+00_JPRB, 0.70926E+00_JPRB, 0.68607E+00_JPRB, 0.67392E+00_JPRB, 0.67058E+00_JPRB ,&
&  0.66170E+00_JPRB, 0.64178E+00_JPRB, 0.62684E+00_JPRB, 0.61166E+00_JPRB, 0.60599E+00_JPRB, 0.60663E+00_JPRB, 0.60673E+00_JPRB ,&
&  0.62424E+00_JPRB, 0.52975E+00_JPRB, 0.44398E+00_JPRB, 0.39315E+00_JPRB, 0.41179E+00_JPRB, 0.40387E+00_JPRB, 0.40652E+00_JPRB ,&
&  0.40588E+00_JPRB, 0.43162E+00_JPRB, 0.44037E+00_JPRB, 0.47211E+00_JPRB, 0.51135E+00_JPRB, 0.55006E+00_JPRB, 0.55072E+00_JPRB ,&
&  0.51091E+00_JPRB, 0.45911E+00_JPRB, 0.43887E+00_JPRB, 0.44810E+00_JPRB, 0.49729E+00_JPRB, 0.49494E+00_JPRB, 0.56040E+00_JPRB ,&
&  0.60679E+00_JPRB, 0.65537E+00_JPRB, 0.70819E+00_JPRB, 0.75628E+00_JPRB, 0.81837E+00_JPRB, 0.89152E+00_JPRB, 0.96812E+00_JPRB ,&
&  0.10479E+01_JPRB, 0.11089E+01_JPRB, 0.11321E+01_JPRB, 0.11739E+01_JPRB, 0.12786E+01_JPRB, 0.13817E+01_JPRB, 0.13101E+01_JPRB ,&
&  0.13298E+01_JPRB, 0.13580E+01_JPRB, 0.14545E+01_JPRB, 0.19124E+01_JPRB, 0.23033E+01_JPRB, 0.23889E+01_JPRB, 0.22656E+01_JPRB ,&
&  0.20468E+01_JPRB, 0.18303E+01_JPRB, 0.17742E+01_JPRB, 0.15965E+01_JPRB, 0.14986E+01_JPRB, 0.14991E+01_JPRB, 0.14809E+01_JPRB ,&
&  0.14294E+01_JPRB, 0.14340E+01_JPRB, 0.14634E+01_JPRB, 0.14282E+01_JPRB, 0.14308E+01_JPRB, 0.14258E+01_JPRB, 0.14720E+01_JPRB ,&
&  0.16109E+01_JPRB, 0.11222E+01_JPRB, 0.90013E+00_JPRB, 0.92753E+00_JPRB, 0.76966E+00_JPRB, 0.67807E+00_JPRB, 0.68672E+00_JPRB ,&
&  0.60516E+00_JPRB, 0.59943E+00_JPRB, 0.63853E+00_JPRB, 0.61290E+00_JPRB, 0.67488E+00_JPRB, 0.77907E+00_JPRB, 0.71251E+00_JPRB ,&
&  0.70389E+00_JPRB, 0.72538E+00_JPRB, 0.90700E+00_JPRB, 0.10239E+01_JPRB, 0.10581E+01_JPRB, 0.10109E+01_JPRB, 0.93724E+00_JPRB ,&
&  0.85534E+00_JPRB, 0.81977E+00_JPRB, 0.80974E+00_JPRB, 0.79809E+00_JPRB, 0.78940E+00_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.89847E+00_JPRB, 0.87342E+00_JPRB, 0.84731E+00_JPRB, 0.81391E+00_JPRB, 0.78868E+00_JPRB, 0.76402E+00_JPRB, 0.73061E+00_JPRB ,&
&  0.69581E+00_JPRB, 0.66785E+00_JPRB, 0.64851E+00_JPRB, 0.63926E+00_JPRB, 0.63971E+00_JPRB, 0.64652E+00_JPRB, 0.64833E+00_JPRB ,&
&  0.65594E+00_JPRB, 0.61805E+00_JPRB, 0.50473E+00_JPRB, 0.43096E+00_JPRB, 0.38414E+00_JPRB, 0.39064E+00_JPRB, 0.39867E+00_JPRB ,&
&  0.42176E+00_JPRB, 0.45763E+00_JPRB, 0.52363E+00_JPRB, 0.61063E+00_JPRB, 0.66370E+00_JPRB, 0.68196E+00_JPRB, 0.68380E+00_JPRB ,&
&  0.63346E+00_JPRB, 0.58490E+00_JPRB, 0.53556E+00_JPRB, 0.56007E+00_JPRB, 0.57231E+00_JPRB, 0.56791E+00_JPRB, 0.62953E+00_JPRB ,&
&  0.67718E+00_JPRB, 0.73394E+00_JPRB, 0.78300E+00_JPRB, 0.85183E+00_JPRB, 0.91038E+00_JPRB, 0.95567E+00_JPRB, 0.99643E+00_JPRB ,&
&  0.10297E+01_JPRB, 0.10704E+01_JPRB, 0.10996E+01_JPRB, 0.11181E+01_JPRB, 0.11764E+01_JPRB, 0.12447E+01_JPRB, 0.12256E+01_JPRB ,&
&  0.13126E+01_JPRB, 0.12675E+01_JPRB, 0.12754E+01_JPRB, 0.18410E+01_JPRB, 0.26515E+01_JPRB, 0.26572E+01_JPRB, 0.23888E+01_JPRB ,&
&  0.23199E+01_JPRB, 0.22525E+01_JPRB, 0.22857E+01_JPRB, 0.22285E+01_JPRB, 0.20503E+01_JPRB, 0.20001E+01_JPRB, 0.19250E+01_JPRB ,&
&  0.18641E+01_JPRB, 0.18196E+01_JPRB, 0.17643E+01_JPRB, 0.17957E+01_JPRB, 0.18239E+01_JPRB, 0.18929E+01_JPRB, 0.19663E+01_JPRB ,&
&  0.19576E+01_JPRB, 0.17358E+01_JPRB, 0.12238E+01_JPRB, 0.11556E+01_JPRB, 0.92249E+00_JPRB, 0.82383E+00_JPRB, 0.81435E+00_JPRB ,&
&  0.71733E+00_JPRB, 0.73746E+00_JPRB, 0.77685E+00_JPRB, 0.75441E+00_JPRB, 0.94520E+00_JPRB, 0.10456E+01_JPRB, 0.10553E+01_JPRB ,&
&  0.10423E+01_JPRB, 0.10062E+01_JPRB, 0.12333E+01_JPRB, 0.13054E+01_JPRB, 0.12959E+01_JPRB, 0.12330E+01_JPRB, 0.11526E+01_JPRB ,&
&  0.10548E+01_JPRB, 0.99611E+00_JPRB, 0.96929E+00_JPRB, 0.95151E+00_JPRB, 0.92294E+00_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.98918E+00_JPRB, 0.94160E+00_JPRB, 0.89558E+00_JPRB, 0.86123E+00_JPRB, 0.83448E+00_JPRB, 0.80105E+00_JPRB, 0.75629E+00_JPRB ,&
&  0.72035E+00_JPRB, 0.70089E+00_JPRB, 0.68867E+00_JPRB, 0.67717E+00_JPRB, 0.67416E+00_JPRB, 0.67810E+00_JPRB, 0.68358E+00_JPRB ,&
&  0.68134E+00_JPRB, 0.61523E+00_JPRB, 0.49450E+00_JPRB, 0.41765E+00_JPRB, 0.37203E+00_JPRB, 0.36357E+00_JPRB, 0.38453E+00_JPRB ,&
&  0.42739E+00_JPRB, 0.53619E+00_JPRB, 0.69380E+00_JPRB, 0.79096E+00_JPRB, 0.85792E+00_JPRB, 0.89250E+00_JPRB, 0.87702E+00_JPRB ,&
&  0.89543E+00_JPRB, 0.74787E+00_JPRB, 0.73718E+00_JPRB, 0.68536E+00_JPRB, 0.70136E+00_JPRB, 0.74069E+00_JPRB, 0.74893E+00_JPRB ,&
&  0.73589E+00_JPRB, 0.76852E+00_JPRB, 0.80069E+00_JPRB, 0.85386E+00_JPRB, 0.88831E+00_JPRB, 0.93495E+00_JPRB, 0.10109E+01_JPRB ,&
&  0.10973E+01_JPRB, 0.11363E+01_JPRB, 0.10949E+01_JPRB, 0.10552E+01_JPRB, 0.12277E+01_JPRB, 0.14253E+01_JPRB, 0.15046E+01_JPRB ,&
&  0.14824E+01_JPRB, 0.15618E+01_JPRB, 0.22626E+01_JPRB, 0.26527E+01_JPRB, 0.28198E+01_JPRB, 0.30424E+01_JPRB, 0.29530E+01_JPRB ,&
&  0.26959E+01_JPRB, 0.24605E+01_JPRB, 0.23412E+01_JPRB, 0.20850E+01_JPRB, 0.23682E+01_JPRB, 0.24583E+01_JPRB, 0.24715E+01_JPRB ,&
&  0.24641E+01_JPRB, 0.23555E+01_JPRB, 0.22803E+01_JPRB, 0.23052E+01_JPRB, 0.23843E+01_JPRB, 0.24783E+01_JPRB, 0.23908E+01_JPRB ,&
&  0.21155E+01_JPRB, 0.21885E+01_JPRB, 0.18707E+01_JPRB, 0.15190E+01_JPRB, 0.12386E+01_JPRB, 0.11424E+01_JPRB, 0.10511E+01_JPRB ,&
&  0.95093E+00_JPRB, 0.90852E+00_JPRB, 0.97162E+00_JPRB, 0.10795E+01_JPRB, 0.12634E+01_JPRB, 0.14361E+01_JPRB, 0.14471E+01_JPRB ,&
&  0.13838E+01_JPRB, 0.15425E+01_JPRB, 0.15314E+01_JPRB, 0.14571E+01_JPRB, 0.14141E+01_JPRB, 0.13249E+01_JPRB, 0.12559E+01_JPRB ,&
&  0.11892E+01_JPRB, 0.11515E+01_JPRB, 0.11243E+01_JPRB, 0.10901E+01_JPRB, 0.10420E+01_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.10570E+01_JPRB, 0.10029E+01_JPRB, 0.95081E+00_JPRB, 0.90840E+00_JPRB, 0.86578E+00_JPRB, 0.82615E+00_JPRB, 0.78167E+00_JPRB ,&
&  0.74815E+00_JPRB, 0.72880E+00_JPRB, 0.71277E+00_JPRB, 0.70628E+00_JPRB, 0.70740E+00_JPRB, 0.70078E+00_JPRB, 0.69820E+00_JPRB ,&
&  0.68485E+00_JPRB, 0.57998E+00_JPRB, 0.47726E+00_JPRB, 0.44007E+00_JPRB, 0.40600E+00_JPRB, 0.37775E+00_JPRB, 0.39013E+00_JPRB ,&
&  0.46844E+00_JPRB, 0.67250E+00_JPRB, 0.86558E+00_JPRB, 0.99877E+00_JPRB, 0.10770E+01_JPRB, 0.11066E+01_JPRB, 0.10747E+01_JPRB ,&
&  0.97264E+00_JPRB, 0.98527E+00_JPRB, 0.98908E+00_JPRB, 0.94924E+00_JPRB, 0.89399E+00_JPRB, 0.87215E+00_JPRB, 0.85580E+00_JPRB ,&
&  0.83902E+00_JPRB, 0.86122E+00_JPRB, 0.89676E+00_JPRB, 0.93558E+00_JPRB, 0.99915E+00_JPRB, 0.10737E+01_JPRB, 0.11470E+01_JPRB ,&
&  0.11596E+01_JPRB, 0.11571E+01_JPRB, 0.10966E+01_JPRB, 0.10993E+01_JPRB, 0.11273E+01_JPRB, 0.15344E+01_JPRB, 0.17438E+01_JPRB ,&
&  0.21662E+01_JPRB, 0.25086E+01_JPRB, 0.25984E+01_JPRB, 0.30649E+01_JPRB, 0.26300E+01_JPRB, 0.29152E+01_JPRB, 0.31528E+01_JPRB ,&
&  0.27899E+01_JPRB, 0.24996E+01_JPRB, 0.24007E+01_JPRB, 0.20523E+01_JPRB, 0.20112E+01_JPRB, 0.29238E+01_JPRB, 0.31398E+01_JPRB ,&
&  0.32111E+01_JPRB, 0.33708E+01_JPRB, 0.31777E+01_JPRB, 0.30001E+01_JPRB, 0.28280E+01_JPRB, 0.25977E+01_JPRB, 0.19877E+01_JPRB ,&
&  0.19811E+01_JPRB, 0.17970E+01_JPRB, 0.18481E+01_JPRB, 0.16388E+01_JPRB, 0.14963E+01_JPRB, 0.13473E+01_JPRB, 0.11979E+01_JPRB ,&
&  0.11048E+01_JPRB, 0.10680E+01_JPRB, 0.13183E+01_JPRB, 0.16877E+01_JPRB, 0.19216E+01_JPRB, 0.17569E+01_JPRB, 0.20102E+01_JPRB ,&
&  0.19488E+01_JPRB, 0.17257E+01_JPRB, 0.14493E+01_JPRB, 0.13806E+01_JPRB, 0.13251E+01_JPRB, 0.12714E+01_JPRB, 0.12385E+01_JPRB ,&
&  0.12040E+01_JPRB, 0.11674E+01_JPRB, 0.11428E+01_JPRB, 0.11305E+01_JPRB, 0.11001E+01_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.10813E+01_JPRB, 0.10276E+01_JPRB, 0.97115E+00_JPRB, 0.92693E+00_JPRB, 0.88649E+00_JPRB, 0.84790E+00_JPRB, 0.81478E+00_JPRB ,&
&  0.79300E+00_JPRB, 0.76947E+00_JPRB, 0.73687E+00_JPRB, 0.71583E+00_JPRB, 0.70462E+00_JPRB, 0.71480E+00_JPRB, 0.70488E+00_JPRB ,&
&  0.68138E+00_JPRB, 0.60037E+00_JPRB, 0.49925E+00_JPRB, 0.46133E+00_JPRB, 0.46037E+00_JPRB, 0.43279E+00_JPRB, 0.43135E+00_JPRB ,&
&  0.58719E+00_JPRB, 0.91923E+00_JPRB, 0.11031E+01_JPRB, 0.11749E+01_JPRB, 0.12585E+01_JPRB, 0.12853E+01_JPRB, 0.12185E+01_JPRB ,&
&  0.12057E+01_JPRB, 0.11309E+01_JPRB, 0.10693E+01_JPRB, 0.10124E+01_JPRB, 0.96993E+00_JPRB, 0.92512E+00_JPRB, 0.88951E+00_JPRB ,&
&  0.88966E+00_JPRB, 0.89734E+00_JPRB, 0.93141E+00_JPRB, 0.94125E+00_JPRB, 0.10147E+01_JPRB, 0.10578E+01_JPRB, 0.10730E+01_JPRB ,&
&  0.10813E+01_JPRB, 0.11072E+01_JPRB, 0.10984E+01_JPRB, 0.11187E+01_JPRB, 0.13151E+01_JPRB, 0.17992E+01_JPRB, 0.22232E+01_JPRB ,&
&  0.24947E+01_JPRB, 0.27233E+01_JPRB, 0.30071E+01_JPRB, 0.32515E+01_JPRB, 0.37540E+01_JPRB, 0.36420E+01_JPRB, 0.35385E+01_JPRB ,&
&  0.26730E+01_JPRB, 0.21529E+01_JPRB, 0.17772E+01_JPRB, 0.14836E+01_JPRB, 0.23507E+01_JPRB, 0.38165E+01_JPRB, 0.43686E+01_JPRB ,&
&  0.43852E+01_JPRB, 0.45533E+01_JPRB, 0.43616E+01_JPRB, 0.36510E+01_JPRB, 0.30600E+01_JPRB, 0.24525E+01_JPRB, 0.27492E+01_JPRB ,&
&  0.26377E+01_JPRB, 0.25254E+01_JPRB, 0.22187E+01_JPRB, 0.17839E+01_JPRB, 0.14472E+01_JPRB, 0.15066E+01_JPRB, 0.13686E+01_JPRB ,&
&  0.12522E+01_JPRB, 0.13027E+01_JPRB, 0.20216E+01_JPRB, 0.24039E+01_JPRB, 0.23599E+01_JPRB, 0.22608E+01_JPRB, 0.24952E+01_JPRB ,&
&  0.20276E+01_JPRB, 0.16787E+01_JPRB, 0.14523E+01_JPRB, 0.13160E+01_JPRB, 0.12618E+01_JPRB, 0.12252E+01_JPRB, 0.12084E+01_JPRB ,&
&  0.11927E+01_JPRB, 0.11828E+01_JPRB, 0.11640E+01_JPRB, 0.11398E+01_JPRB, 0.11129E+01_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.11047E+01_JPRB, 0.10512E+01_JPRB, 0.10050E+01_JPRB, 0.95791E+00_JPRB, 0.91435E+00_JPRB, 0.88134E+00_JPRB, 0.85761E+00_JPRB ,&
&  0.82625E+00_JPRB, 0.79899E+00_JPRB, 0.76935E+00_JPRB, 0.74339E+00_JPRB, 0.73158E+00_JPRB, 0.73017E+00_JPRB, 0.71738E+00_JPRB ,&
&  0.69996E+00_JPRB, 0.66164E+00_JPRB, 0.54762E+00_JPRB, 0.50737E+00_JPRB, 0.50456E+00_JPRB, 0.50078E+00_JPRB, 0.53015E+00_JPRB ,&
&  0.83419E+00_JPRB, 0.12249E+01_JPRB, 0.12858E+01_JPRB, 0.12571E+01_JPRB, 0.13383E+01_JPRB, 0.11740E+01_JPRB, 0.11338E+01_JPRB ,&
&  0.12178E+01_JPRB, 0.11551E+01_JPRB, 0.10940E+01_JPRB, 0.10421E+01_JPRB, 0.10044E+01_JPRB, 0.97666E+00_JPRB, 0.95257E+00_JPRB ,&
&  0.93810E+00_JPRB, 0.93500E+00_JPRB, 0.97017E+00_JPRB, 0.10487E+01_JPRB, 0.10792E+01_JPRB, 0.10804E+01_JPRB, 0.10820E+01_JPRB ,&
&  0.10733E+01_JPRB, 0.10675E+01_JPRB, 0.10822E+01_JPRB, 0.12243E+01_JPRB, 0.15002E+01_JPRB, 0.20847E+01_JPRB, 0.30661E+01_JPRB ,&
&  0.30929E+01_JPRB, 0.34242E+01_JPRB, 0.33718E+01_JPRB, 0.42590E+01_JPRB, 0.46967E+01_JPRB, 0.43024E+01_JPRB, 0.38062E+01_JPRB ,&
&  0.27182E+01_JPRB, 0.21492E+01_JPRB, 0.19565E+01_JPRB, 0.21833E+01_JPRB, 0.29345E+01_JPRB, 0.38402E+01_JPRB, 0.48830E+01_JPRB ,&
&  0.43367E+01_JPRB, 0.42818E+01_JPRB, 0.42402E+01_JPRB, 0.38692E+01_JPRB, 0.23603E+01_JPRB, 0.18031E+01_JPRB, 0.25560E+01_JPRB ,&
&  0.25664E+01_JPRB, 0.15435E+01_JPRB, 0.10966E+01_JPRB, 0.12893E+01_JPRB, 0.96796E+00_JPRB, 0.10808E+01_JPRB, 0.14037E+01_JPRB ,&
&  0.14735E+01_JPRB, 0.17000E+01_JPRB, 0.24476E+01_JPRB, 0.23985E+01_JPRB, 0.24213E+01_JPRB, 0.27081E+01_JPRB, 0.23594E+01_JPRB ,&
&  0.19070E+01_JPRB, 0.15754E+01_JPRB, 0.15186E+01_JPRB, 0.13886E+01_JPRB, 0.13308E+01_JPRB, 0.12892E+01_JPRB, 0.12658E+01_JPRB ,&
&  0.12719E+01_JPRB, 0.12585E+01_JPRB, 0.12269E+01_JPRB, 0.11879E+01_JPRB, 0.11510E+01_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.11334E+01_JPRB, 0.10828E+01_JPRB, 0.10327E+01_JPRB, 0.99015E+00_JPRB, 0.95617E+00_JPRB, 0.92284E+00_JPRB, 0.87564E+00_JPRB ,&
&  0.82083E+00_JPRB, 0.78859E+00_JPRB, 0.77275E+00_JPRB, 0.75458E+00_JPRB, 0.74867E+00_JPRB, 0.75315E+00_JPRB, 0.74303E+00_JPRB ,&
&  0.71542E+00_JPRB, 0.68185E+00_JPRB, 0.63014E+00_JPRB, 0.58400E+00_JPRB, 0.56390E+00_JPRB, 0.54688E+00_JPRB, 0.68511E+00_JPRB ,&
&  0.11652E+01_JPRB, 0.15482E+01_JPRB, 0.14830E+01_JPRB, 0.13072E+01_JPRB, 0.11864E+01_JPRB, 0.10698E+01_JPRB, 0.11726E+01_JPRB ,&
&  0.11979E+01_JPRB, 0.11765E+01_JPRB, 0.11012E+01_JPRB, 0.10503E+01_JPRB, 0.10118E+01_JPRB, 0.97591E+00_JPRB, 0.96065E+00_JPRB ,&
&  0.95106E+00_JPRB, 0.95943E+00_JPRB, 0.10254E+01_JPRB, 0.10509E+01_JPRB, 0.10643E+01_JPRB, 0.10553E+01_JPRB, 0.10904E+01_JPRB ,&
&  0.10770E+01_JPRB, 0.10843E+01_JPRB, 0.11354E+01_JPRB, 0.13390E+01_JPRB, 0.18974E+01_JPRB, 0.27049E+01_JPRB, 0.29776E+01_JPRB ,&
&  0.30537E+01_JPRB, 0.32881E+01_JPRB, 0.39873E+01_JPRB, 0.43989E+01_JPRB, 0.46782E+01_JPRB, 0.44844E+01_JPRB, 0.36314E+01_JPRB ,&
&  0.31033E+01_JPRB, 0.27190E+01_JPRB, 0.23130E+01_JPRB, 0.25744E+01_JPRB, 0.36037E+01_JPRB, 0.38400E+01_JPRB, 0.45367E+01_JPRB ,&
&  0.44895E+01_JPRB, 0.40200E+01_JPRB, 0.31426E+01_JPRB, 0.23945E+01_JPRB, 0.24126E+01_JPRB, 0.18005E+01_JPRB, 0.11630E+01_JPRB ,&
&  0.10421E+01_JPRB, 0.93700E+00_JPRB, 0.83063E+00_JPRB, 0.83494E+00_JPRB, 0.79001E+00_JPRB, 0.95945E+00_JPRB, 0.13741E+01_JPRB ,&
&  0.17724E+01_JPRB, 0.20760E+01_JPRB, 0.23333E+01_JPRB, 0.22302E+01_JPRB, 0.21357E+01_JPRB, 0.25122E+01_JPRB, 0.25544E+01_JPRB ,&
&  0.16646E+01_JPRB, 0.14716E+01_JPRB, 0.14789E+01_JPRB, 0.14158E+01_JPRB, 0.13715E+01_JPRB, 0.13432E+01_JPRB, 0.13435E+01_JPRB ,&
&  0.13521E+01_JPRB, 0.13302E+01_JPRB, 0.12824E+01_JPRB, 0.12287E+01_JPRB, 0.11779E+01_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.11206E+01_JPRB, 0.10570E+01_JPRB, 0.10180E+01_JPRB, 0.99923E+00_JPRB, 0.97412E+00_JPRB, 0.92244E+00_JPRB, 0.84888E+00_JPRB ,&
&  0.79780E+00_JPRB, 0.77374E+00_JPRB, 0.76514E+00_JPRB, 0.75786E+00_JPRB, 0.75457E+00_JPRB, 0.75160E+00_JPRB, 0.73993E+00_JPRB ,&
&  0.70540E+00_JPRB, 0.68354E+00_JPRB, 0.68070E+00_JPRB, 0.65432E+00_JPRB, 0.64976E+00_JPRB, 0.64343E+00_JPRB, 0.88769E+00_JPRB ,&
&  0.15318E+01_JPRB, 0.18355E+01_JPRB, 0.16165E+01_JPRB, 0.13026E+01_JPRB, 0.12038E+01_JPRB, 0.11868E+01_JPRB, 0.12337E+01_JPRB ,&
&  0.11940E+01_JPRB, 0.11329E+01_JPRB, 0.10222E+01_JPRB, 0.98236E+00_JPRB, 0.97300E+00_JPRB, 0.96085E+00_JPRB, 0.95963E+00_JPRB ,&
&  0.95774E+00_JPRB, 0.99655E+00_JPRB, 0.10087E+01_JPRB, 0.10217E+01_JPRB, 0.10477E+01_JPRB, 0.10796E+01_JPRB, 0.11259E+01_JPRB ,&
&  0.11545E+01_JPRB, 0.11859E+01_JPRB, 0.13052E+01_JPRB, 0.17042E+01_JPRB, 0.22817E+01_JPRB, 0.25723E+01_JPRB, 0.30636E+01_JPRB ,&
&  0.35705E+01_JPRB, 0.39787E+01_JPRB, 0.38359E+01_JPRB, 0.46963E+01_JPRB, 0.46637E+01_JPRB, 0.40613E+01_JPRB, 0.34991E+01_JPRB ,&
&  0.31318E+01_JPRB, 0.33751E+01_JPRB, 0.27242E+01_JPRB, 0.24942E+01_JPRB, 0.39299E+01_JPRB, 0.43010E+01_JPRB, 0.45852E+01_JPRB ,&
&  0.48635E+01_JPRB, 0.41386E+01_JPRB, 0.32673E+01_JPRB, 0.27099E+01_JPRB, 0.33560E+01_JPRB, 0.30465E+01_JPRB, 0.13520E+01_JPRB ,&
&  0.10218E+01_JPRB, 0.91942E+00_JPRB, 0.79517E+00_JPRB, 0.73668E+00_JPRB, 0.72776E+00_JPRB, 0.90232E+00_JPRB, 0.15372E+01_JPRB ,&
&  0.16744E+01_JPRB, 0.21727E+01_JPRB, 0.22202E+01_JPRB, 0.21636E+01_JPRB, 0.19939E+01_JPRB, 0.19799E+01_JPRB, 0.21750E+01_JPRB ,&
&  0.18434E+01_JPRB, 0.16298E+01_JPRB, 0.14134E+01_JPRB, 0.13054E+01_JPRB, 0.12567E+01_JPRB, 0.12639E+01_JPRB, 0.13206E+01_JPRB ,&
&  0.13495E+01_JPRB, 0.13451E+01_JPRB, 0.13000E+01_JPRB, 0.12358E+01_JPRB, 0.11741E+01_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.10671E+01_JPRB, 0.10395E+01_JPRB, 0.10195E+01_JPRB, 0.98035E+00_JPRB, 0.94443E+00_JPRB, 0.89801E+00_JPRB, 0.83968E+00_JPRB ,&
&  0.80693E+00_JPRB, 0.79018E+00_JPRB, 0.78211E+00_JPRB, 0.76712E+00_JPRB, 0.74969E+00_JPRB, 0.73528E+00_JPRB, 0.73013E+00_JPRB ,&
&  0.70761E+00_JPRB, 0.67834E+00_JPRB, 0.67666E+00_JPRB, 0.70552E+00_JPRB, 0.74608E+00_JPRB, 0.84596E+00_JPRB, 0.11481E+01_JPRB ,&
&  0.18309E+01_JPRB, 0.20464E+01_JPRB, 0.17582E+01_JPRB, 0.15007E+01_JPRB, 0.13655E+01_JPRB, 0.13301E+01_JPRB, 0.12373E+01_JPRB ,&
&  0.11331E+01_JPRB, 0.10943E+01_JPRB, 0.98611E+00_JPRB, 0.98530E+00_JPRB, 0.99355E+00_JPRB, 0.10080E+01_JPRB, 0.10224E+01_JPRB ,&
&  0.10581E+01_JPRB, 0.10239E+01_JPRB, 0.10321E+01_JPRB, 0.10575E+01_JPRB, 0.10935E+01_JPRB, 0.11600E+01_JPRB, 0.12165E+01_JPRB ,&
&  0.12771E+01_JPRB, 0.13584E+01_JPRB, 0.16082E+01_JPRB, 0.20520E+01_JPRB, 0.21896E+01_JPRB, 0.27966E+01_JPRB, 0.31376E+01_JPRB ,&
&  0.34369E+01_JPRB, 0.35569E+01_JPRB, 0.34948E+01_JPRB, 0.40848E+01_JPRB, 0.43824E+01_JPRB, 0.39056E+01_JPRB, 0.34230E+01_JPRB ,&
&  0.35178E+01_JPRB, 0.30595E+01_JPRB, 0.27237E+01_JPRB, 0.24807E+01_JPRB, 0.32590E+01_JPRB, 0.54401E+01_JPRB, 0.42553E+01_JPRB ,&
&  0.40923E+01_JPRB, 0.46049E+01_JPRB, 0.37828E+01_JPRB, 0.32191E+01_JPRB, 0.34251E+01_JPRB, 0.36497E+01_JPRB, 0.27499E+01_JPRB ,&
&  0.11909E+01_JPRB, 0.88185E+00_JPRB, 0.87699E+00_JPRB, 0.85454E+00_JPRB, 0.75952E+00_JPRB, 0.91674E+00_JPRB, 0.22224E+01_JPRB ,&
&  0.18757E+01_JPRB, 0.23529E+01_JPRB, 0.22985E+01_JPRB, 0.21143E+01_JPRB, 0.19930E+01_JPRB, 0.17658E+01_JPRB, 0.17947E+01_JPRB ,&
&  0.14955E+01_JPRB, 0.13298E+01_JPRB, 0.12520E+01_JPRB, 0.11993E+01_JPRB, 0.11935E+01_JPRB, 0.12069E+01_JPRB, 0.12565E+01_JPRB ,&
&  0.12886E+01_JPRB, 0.12944E+01_JPRB, 0.12519E+01_JPRB, 0.11831E+01_JPRB, 0.11182E+01_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.10285E+01_JPRB, 0.10134E+01_JPRB, 0.98318E+00_JPRB, 0.94660E+00_JPRB, 0.91950E+00_JPRB, 0.87396E+00_JPRB, 0.83613E+00_JPRB ,&
&  0.82967E+00_JPRB, 0.81054E+00_JPRB, 0.79075E+00_JPRB, 0.76921E+00_JPRB, 0.74837E+00_JPRB, 0.72763E+00_JPRB, 0.71991E+00_JPRB ,&
&  0.71288E+00_JPRB, 0.68227E+00_JPRB, 0.67609E+00_JPRB, 0.72789E+00_JPRB, 0.89135E+00_JPRB, 0.11054E+01_JPRB, 0.12886E+01_JPRB ,&
&  0.20654E+01_JPRB, 0.22375E+01_JPRB, 0.16711E+01_JPRB, 0.15325E+01_JPRB, 0.15008E+01_JPRB, 0.14348E+01_JPRB, 0.12823E+01_JPRB ,&
&  0.11679E+01_JPRB, 0.11282E+01_JPRB, 0.10236E+01_JPRB, 0.10103E+01_JPRB, 0.10355E+01_JPRB, 0.10854E+01_JPRB, 0.11234E+01_JPRB ,&
&  0.10758E+01_JPRB, 0.10840E+01_JPRB, 0.11335E+01_JPRB, 0.11642E+01_JPRB, 0.12033E+01_JPRB, 0.12859E+01_JPRB, 0.13527E+01_JPRB ,&
&  0.14488E+01_JPRB, 0.16197E+01_JPRB, 0.19426E+01_JPRB, 0.21867E+01_JPRB, 0.25231E+01_JPRB, 0.27732E+01_JPRB, 0.31094E+01_JPRB ,&
&  0.31326E+01_JPRB, 0.32439E+01_JPRB, 0.34434E+01_JPRB, 0.36840E+01_JPRB, 0.38202E+01_JPRB, 0.36747E+01_JPRB, 0.34757E+01_JPRB ,&
&  0.35279E+01_JPRB, 0.30122E+01_JPRB, 0.26090E+01_JPRB, 0.25013E+01_JPRB, 0.30627E+01_JPRB, 0.48565E+01_JPRB, 0.54085E+01_JPRB ,&
&  0.54120E+01_JPRB, 0.38668E+01_JPRB, 0.33231E+01_JPRB, 0.35873E+01_JPRB, 0.33477E+01_JPRB, 0.32721E+01_JPRB, 0.39166E+01_JPRB ,&
&  0.33964E+01_JPRB, 0.18635E+01_JPRB, 0.15206E+01_JPRB, 0.18384E+01_JPRB, 0.11807E+01_JPRB, 0.95200E+00_JPRB, 0.14794E+01_JPRB ,&
&  0.17807E+01_JPRB, 0.20937E+01_JPRB, 0.20861E+01_JPRB, 0.20119E+01_JPRB, 0.19378E+01_JPRB, 0.16056E+01_JPRB, 0.14586E+01_JPRB ,&
&  0.12254E+01_JPRB, 0.11738E+01_JPRB, 0.11470E+01_JPRB, 0.10891E+01_JPRB, 0.10743E+01_JPRB, 0.11154E+01_JPRB, 0.11570E+01_JPRB ,&
&  0.11792E+01_JPRB, 0.11807E+01_JPRB, 0.11581E+01_JPRB, 0.11173E+01_JPRB, 0.10627E+01_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.99446E+00_JPRB, 0.98421E+00_JPRB, 0.97629E+00_JPRB, 0.93958E+00_JPRB, 0.90433E+00_JPRB, 0.86946E+00_JPRB, 0.86540E+00_JPRB ,&
&  0.84034E+00_JPRB, 0.81765E+00_JPRB, 0.79765E+00_JPRB, 0.76641E+00_JPRB, 0.75590E+00_JPRB, 0.73686E+00_JPRB, 0.74396E+00_JPRB ,&
&  0.75535E+00_JPRB, 0.73251E+00_JPRB, 0.75844E+00_JPRB, 0.93196E+00_JPRB, 0.12285E+01_JPRB, 0.18159E+01_JPRB, 0.14913E+01_JPRB ,&
&  0.20058E+01_JPRB, 0.23963E+01_JPRB, 0.17165E+01_JPRB, 0.16347E+01_JPRB, 0.16138E+01_JPRB, 0.15847E+01_JPRB, 0.13977E+01_JPRB ,&
&  0.12629E+01_JPRB, 0.12046E+01_JPRB, 0.11529E+01_JPRB, 0.11101E+01_JPRB, 0.11310E+01_JPRB, 0.11544E+01_JPRB, 0.11192E+01_JPRB ,&
&  0.11388E+01_JPRB, 0.12079E+01_JPRB, 0.12436E+01_JPRB, 0.12717E+01_JPRB, 0.13397E+01_JPRB, 0.14242E+01_JPRB, 0.15191E+01_JPRB ,&
&  0.16863E+01_JPRB, 0.19146E+01_JPRB, 0.22502E+01_JPRB, 0.23507E+01_JPRB, 0.25425E+01_JPRB, 0.28051E+01_JPRB, 0.29536E+01_JPRB ,&
&  0.28506E+01_JPRB, 0.27483E+01_JPRB, 0.30983E+01_JPRB, 0.34203E+01_JPRB, 0.34648E+01_JPRB, 0.33895E+01_JPRB, 0.34261E+01_JPRB ,&
&  0.36629E+01_JPRB, 0.35060E+01_JPRB, 0.35517E+01_JPRB, 0.31052E+01_JPRB, 0.36781E+01_JPRB, 0.48812E+01_JPRB, 0.49153E+01_JPRB ,&
&  0.58315E+01_JPRB, 0.49054E+01_JPRB, 0.44271E+01_JPRB, 0.38183E+01_JPRB, 0.32725E+01_JPRB, 0.30561E+01_JPRB, 0.31266E+01_JPRB ,&
&  0.31232E+01_JPRB, 0.33036E+01_JPRB, 0.30668E+01_JPRB, 0.19046E+01_JPRB, 0.13011E+01_JPRB, 0.10753E+01_JPRB, 0.12283E+01_JPRB ,&
&  0.15886E+01_JPRB, 0.17094E+01_JPRB, 0.17192E+01_JPRB, 0.24291E+01_JPRB, 0.16998E+01_JPRB, 0.13664E+01_JPRB, 0.11807E+01_JPRB ,&
&  0.10712E+01_JPRB, 0.10606E+01_JPRB, 0.10305E+01_JPRB, 0.98310E+00_JPRB, 0.98695E+00_JPRB, 0.10115E+01_JPRB, 0.10440E+01_JPRB ,&
&  0.10680E+01_JPRB, 0.10885E+01_JPRB, 0.10997E+01_JPRB, 0.10754E+01_JPRB, 0.10296E+01_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.95377E+00_JPRB, 0.95484E+00_JPRB, 0.93495E+00_JPRB, 0.90973E+00_JPRB, 0.89310E+00_JPRB, 0.88318E+00_JPRB, 0.89377E+00_JPRB ,&
&  0.82984E+00_JPRB, 0.80336E+00_JPRB, 0.78950E+00_JPRB, 0.77539E+00_JPRB, 0.78766E+00_JPRB, 0.80451E+00_JPRB, 0.83604E+00_JPRB ,&
&  0.87817E+00_JPRB, 0.92625E+00_JPRB, 0.10764E+01_JPRB, 0.14080E+01_JPRB, 0.18132E+01_JPRB, 0.23344E+01_JPRB, 0.22651E+01_JPRB ,&
&  0.19527E+01_JPRB, 0.26176E+01_JPRB, 0.17553E+01_JPRB, 0.17944E+01_JPRB, 0.17704E+01_JPRB, 0.19065E+01_JPRB, 0.17391E+01_JPRB ,&
&  0.15010E+01_JPRB, 0.14017E+01_JPRB, 0.13415E+01_JPRB, 0.12324E+01_JPRB, 0.12127E+01_JPRB, 0.11896E+01_JPRB, 0.12003E+01_JPRB ,&
&  0.12712E+01_JPRB, 0.13485E+01_JPRB, 0.13991E+01_JPRB, 0.14572E+01_JPRB, 0.15377E+01_JPRB, 0.16178E+01_JPRB, 0.17678E+01_JPRB ,&
&  0.20039E+01_JPRB, 0.22688E+01_JPRB, 0.23930E+01_JPRB, 0.24616E+01_JPRB, 0.25193E+01_JPRB, 0.27276E+01_JPRB, 0.28540E+01_JPRB ,&
&  0.28187E+01_JPRB, 0.27267E+01_JPRB, 0.29524E+01_JPRB, 0.30247E+01_JPRB, 0.30414E+01_JPRB, 0.33300E+01_JPRB, 0.33698E+01_JPRB ,&
&  0.37614E+01_JPRB, 0.35398E+01_JPRB, 0.40805E+01_JPRB, 0.38269E+01_JPRB, 0.40040E+01_JPRB, 0.44407E+01_JPRB, 0.38525E+01_JPRB ,&
&  0.36996E+01_JPRB, 0.38714E+01_JPRB, 0.37906E+01_JPRB, 0.35935E+01_JPRB, 0.32846E+01_JPRB, 0.30094E+01_JPRB, 0.26863E+01_JPRB ,&
&  0.23092E+01_JPRB, 0.22000E+01_JPRB, 0.19759E+01_JPRB, 0.13254E+01_JPRB, 0.10718E+01_JPRB, 0.98859E+00_JPRB, 0.11018E+01_JPRB ,&
&  0.13931E+01_JPRB, 0.15602E+01_JPRB, 0.18534E+01_JPRB, 0.18425E+01_JPRB, 0.13059E+01_JPRB, 0.10672E+01_JPRB, 0.10085E+01_JPRB ,&
&  0.97311E+00_JPRB, 0.95243E+00_JPRB, 0.89957E+00_JPRB, 0.87098E+00_JPRB, 0.88983E+00_JPRB, 0.93024E+00_JPRB, 0.95373E+00_JPRB ,&
&  0.98961E+00_JPRB, 0.10092E+01_JPRB, 0.10004E+01_JPRB, 0.97455E+00_JPRB, 0.96075E+00_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.99315E+00_JPRB, 0.10095E+01_JPRB, 0.10264E+01_JPRB, 0.10644E+01_JPRB, 0.11196E+01_JPRB, 0.11920E+01_JPRB, 0.11917E+01_JPRB ,&
&  0.10407E+01_JPRB, 0.79491E+00_JPRB, 0.81088E+00_JPRB, 0.84612E+00_JPRB, 0.91701E+00_JPRB, 0.98426E+00_JPRB, 0.10655E+01_JPRB ,&
&  0.11856E+01_JPRB, 0.13480E+01_JPRB, 0.16264E+01_JPRB, 0.19847E+01_JPRB, 0.23718E+01_JPRB, 0.28508E+01_JPRB, 0.28551E+01_JPRB ,&
&  0.24258E+01_JPRB, 0.19687E+01_JPRB, 0.20019E+01_JPRB, 0.19969E+01_JPRB, 0.18607E+01_JPRB, 0.20189E+01_JPRB, 0.19809E+01_JPRB ,&
&  0.17266E+01_JPRB, 0.15874E+01_JPRB, 0.15151E+01_JPRB, 0.14184E+01_JPRB, 0.13609E+01_JPRB, 0.14034E+01_JPRB, 0.14737E+01_JPRB ,&
&  0.15703E+01_JPRB, 0.16340E+01_JPRB, 0.16624E+01_JPRB, 0.16927E+01_JPRB, 0.17644E+01_JPRB, 0.18590E+01_JPRB, 0.20710E+01_JPRB ,&
&  0.23137E+01_JPRB, 0.25036E+01_JPRB, 0.25562E+01_JPRB, 0.24781E+01_JPRB, 0.25433E+01_JPRB, 0.26640E+01_JPRB, 0.27272E+01_JPRB ,&
&  0.27547E+01_JPRB, 0.28456E+01_JPRB, 0.29268E+01_JPRB, 0.30030E+01_JPRB, 0.29109E+01_JPRB, 0.33017E+01_JPRB, 0.33370E+01_JPRB ,&
&  0.35449E+01_JPRB, 0.34383E+01_JPRB, 0.39595E+01_JPRB, 0.35512E+01_JPRB, 0.39170E+01_JPRB, 0.38092E+01_JPRB, 0.33784E+01_JPRB ,&
&  0.32364E+01_JPRB, 0.34930E+01_JPRB, 0.35461E+01_JPRB, 0.33815E+01_JPRB, 0.32046E+01_JPRB, 0.26652E+01_JPRB, 0.26101E+01_JPRB ,&
&  0.20244E+01_JPRB, 0.20192E+01_JPRB, 0.14328E+01_JPRB, 0.10727E+01_JPRB, 0.87895E+00_JPRB, 0.96309E+00_JPRB, 0.11113E+01_JPRB ,&
&  0.13837E+01_JPRB, 0.14310E+01_JPRB, 0.13581E+01_JPRB, 0.13412E+01_JPRB, 0.11504E+01_JPRB, 0.97546E+00_JPRB, 0.94661E+00_JPRB ,&
&  0.94598E+00_JPRB, 0.90597E+00_JPRB, 0.85212E+00_JPRB, 0.84855E+00_JPRB, 0.87059E+00_JPRB, 0.92545E+00_JPRB, 0.94305E+00_JPRB ,&
&  0.96811E+00_JPRB, 0.96946E+00_JPRB, 0.96100E+00_JPRB, 0.95892E+00_JPRB, 0.96920E+00_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.10240E+01_JPRB, 0.10191E+01_JPRB, 0.10306E+01_JPRB, 0.10340E+01_JPRB, 0.10300E+01_JPRB, 0.97627E+00_JPRB, 0.93371E+00_JPRB ,&
&  0.88230E+00_JPRB, 0.83687E+00_JPRB, 0.90972E+00_JPRB, 0.10055E+01_JPRB, 0.11216E+01_JPRB, 0.12468E+01_JPRB, 0.13805E+01_JPRB ,&
&  0.15553E+01_JPRB, 0.17744E+01_JPRB, 0.20481E+01_JPRB, 0.22730E+01_JPRB, 0.24874E+01_JPRB, 0.27836E+01_JPRB, 0.30087E+01_JPRB ,&
&  0.25694E+01_JPRB, 0.17238E+01_JPRB, 0.20612E+01_JPRB, 0.22430E+01_JPRB, 0.20684E+01_JPRB, 0.20710E+01_JPRB, 0.20051E+01_JPRB ,&
&  0.18558E+01_JPRB, 0.17784E+01_JPRB, 0.17428E+01_JPRB, 0.17120E+01_JPRB, 0.16783E+01_JPRB, 0.17390E+01_JPRB, 0.17974E+01_JPRB ,&
&  0.18258E+01_JPRB, 0.18126E+01_JPRB, 0.18046E+01_JPRB, 0.18882E+01_JPRB, 0.20559E+01_JPRB, 0.21620E+01_JPRB, 0.23335E+01_JPRB ,&
&  0.24840E+01_JPRB, 0.26672E+01_JPRB, 0.27363E+01_JPRB, 0.27287E+01_JPRB, 0.28265E+01_JPRB, 0.29001E+01_JPRB, 0.28970E+01_JPRB ,&
&  0.28674E+01_JPRB, 0.28591E+01_JPRB, 0.29636E+01_JPRB, 0.32892E+01_JPRB, 0.34804E+01_JPRB, 0.31778E+01_JPRB, 0.33407E+01_JPRB ,&
&  0.34536E+01_JPRB, 0.35706E+01_JPRB, 0.34701E+01_JPRB, 0.44885E+01_JPRB, 0.33831E+01_JPRB, 0.33110E+01_JPRB, 0.27999E+01_JPRB ,&
&  0.30744E+01_JPRB, 0.31196E+01_JPRB, 0.32438E+01_JPRB, 0.32387E+01_JPRB, 0.30173E+01_JPRB, 0.25355E+01_JPRB, 0.25007E+01_JPRB ,&
&  0.21557E+01_JPRB, 0.18729E+01_JPRB, 0.13765E+01_JPRB, 0.10468E+01_JPRB, 0.99282E+00_JPRB, 0.14159E+01_JPRB, 0.14376E+01_JPRB ,&
&  0.13832E+01_JPRB, 0.12491E+01_JPRB, 0.13240E+01_JPRB, 0.14286E+01_JPRB, 0.12137E+01_JPRB, 0.94171E+00_JPRB, 0.88734E+00_JPRB ,&
&  0.89517E+00_JPRB, 0.89027E+00_JPRB, 0.85885E+00_JPRB, 0.86463E+00_JPRB, 0.89863E+00_JPRB, 0.95539E+00_JPRB, 0.98770E+00_JPRB ,&
&  0.99531E+00_JPRB, 0.98018E+00_JPRB, 0.98034E+00_JPRB, 0.98993E+00_JPRB, 0.10201E+01_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.98001E+00_JPRB, 0.96028E+00_JPRB, 0.94063E+00_JPRB, 0.91493E+00_JPRB, 0.88175E+00_JPRB, 0.86005E+00_JPRB, 0.86320E+00_JPRB ,&
&  0.87480E+00_JPRB, 0.92222E+00_JPRB, 0.10014E+01_JPRB, 0.10997E+01_JPRB, 0.12298E+01_JPRB, 0.13801E+01_JPRB, 0.15403E+01_JPRB ,&
&  0.17500E+01_JPRB, 0.19596E+01_JPRB, 0.20920E+01_JPRB, 0.21957E+01_JPRB, 0.23234E+01_JPRB, 0.25270E+01_JPRB, 0.27455E+01_JPRB ,&
&  0.29161E+01_JPRB, 0.28721E+01_JPRB, 0.26500E+01_JPRB, 0.19673E+01_JPRB, 0.17184E+01_JPRB, 0.20265E+01_JPRB, 0.20231E+01_JPRB ,&
&  0.20000E+01_JPRB, 0.19387E+01_JPRB, 0.18986E+01_JPRB, 0.18635E+01_JPRB, 0.18746E+01_JPRB, 0.19193E+01_JPRB, 0.19477E+01_JPRB ,&
&  0.19448E+01_JPRB, 0.19550E+01_JPRB, 0.20021E+01_JPRB, 0.21193E+01_JPRB, 0.23021E+01_JPRB, 0.24205E+01_JPRB, 0.25458E+01_JPRB ,&
&  0.26545E+01_JPRB, 0.28389E+01_JPRB, 0.29512E+01_JPRB, 0.30579E+01_JPRB, 0.30907E+01_JPRB, 0.31714E+01_JPRB, 0.33029E+01_JPRB ,&
&  0.33344E+01_JPRB, 0.33969E+01_JPRB, 0.35442E+01_JPRB, 0.38077E+01_JPRB, 0.39289E+01_JPRB, 0.34726E+01_JPRB, 0.33836E+01_JPRB ,&
&  0.36427E+01_JPRB, 0.39404E+01_JPRB, 0.36564E+01_JPRB, 0.46609E+01_JPRB, 0.29399E+01_JPRB, 0.30904E+01_JPRB, 0.29457E+01_JPRB ,&
&  0.27696E+01_JPRB, 0.27088E+01_JPRB, 0.28335E+01_JPRB, 0.28985E+01_JPRB, 0.27461E+01_JPRB, 0.22950E+01_JPRB, 0.24401E+01_JPRB ,&
&  0.22719E+01_JPRB, 0.17175E+01_JPRB, 0.13153E+01_JPRB, 0.96325E+00_JPRB, 0.11182E+01_JPRB, 0.16527E+01_JPRB, 0.13882E+01_JPRB ,&
&  0.10684E+01_JPRB, 0.12318E+01_JPRB, 0.13753E+01_JPRB, 0.15034E+01_JPRB, 0.13354E+01_JPRB, 0.86113E+00_JPRB, 0.85286E+00_JPRB ,&
&  0.87892E+00_JPRB, 0.89363E+00_JPRB, 0.91338E+00_JPRB, 0.92071E+00_JPRB, 0.94226E+00_JPRB, 0.98318E+00_JPRB, 0.10195E+01_JPRB ,&
&  0.10267E+01_JPRB, 0.10187E+01_JPRB, 0.10265E+01_JPRB, 0.10187E+01_JPRB, 0.10006E+01_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.88106E+00_JPRB, 0.84390E+00_JPRB, 0.81423E+00_JPRB, 0.80201E+00_JPRB, 0.80315E+00_JPRB, 0.81645E+00_JPRB, 0.84228E+00_JPRB ,&
&  0.88748E+00_JPRB, 0.94813E+00_JPRB, 0.10114E+01_JPRB, 0.10703E+01_JPRB, 0.11519E+01_JPRB, 0.12493E+01_JPRB, 0.13749E+01_JPRB ,&
&  0.15154E+01_JPRB, 0.16632E+01_JPRB, 0.17399E+01_JPRB, 0.17685E+01_JPRB, 0.18355E+01_JPRB, 0.19393E+01_JPRB, 0.21848E+01_JPRB ,&
&  0.25434E+01_JPRB, 0.28457E+01_JPRB, 0.30055E+01_JPRB, 0.27854E+01_JPRB, 0.21630E+01_JPRB, 0.21256E+01_JPRB, 0.23066E+01_JPRB ,&
&  0.23760E+01_JPRB, 0.21702E+01_JPRB, 0.20235E+01_JPRB, 0.20007E+01_JPRB, 0.20033E+01_JPRB, 0.20442E+01_JPRB, 0.20707E+01_JPRB ,&
&  0.20778E+01_JPRB, 0.21022E+01_JPRB, 0.21744E+01_JPRB, 0.22677E+01_JPRB, 0.24087E+01_JPRB, 0.25598E+01_JPRB, 0.27255E+01_JPRB ,&
&  0.28132E+01_JPRB, 0.29208E+01_JPRB, 0.29328E+01_JPRB, 0.28766E+01_JPRB, 0.29473E+01_JPRB, 0.32187E+01_JPRB, 0.36147E+01_JPRB ,&
&  0.38620E+01_JPRB, 0.38490E+01_JPRB, 0.41218E+01_JPRB, 0.44411E+01_JPRB, 0.43933E+01_JPRB, 0.40426E+01_JPRB, 0.39839E+01_JPRB ,&
&  0.41453E+01_JPRB, 0.46096E+01_JPRB, 0.33840E+01_JPRB, 0.39044E+01_JPRB, 0.39924E+01_JPRB, 0.33685E+01_JPRB, 0.27173E+01_JPRB ,&
&  0.24237E+01_JPRB, 0.21392E+01_JPRB, 0.18782E+01_JPRB, 0.19339E+01_JPRB, 0.19966E+01_JPRB, 0.20438E+01_JPRB, 0.21122E+01_JPRB ,&
&  0.20249E+01_JPRB, 0.15668E+01_JPRB, 0.11643E+01_JPRB, 0.95579E+00_JPRB, 0.10588E+01_JPRB, 0.13297E+01_JPRB, 0.11458E+01_JPRB ,&
&  0.10315E+01_JPRB, 0.11762E+01_JPRB, 0.12591E+01_JPRB, 0.15034E+01_JPRB, 0.15653E+01_JPRB, 0.83418E+00_JPRB, 0.84436E+00_JPRB ,&
&  0.86579E+00_JPRB, 0.90159E+00_JPRB, 0.91337E+00_JPRB, 0.91997E+00_JPRB, 0.92546E+00_JPRB, 0.95160E+00_JPRB, 0.98010E+00_JPRB ,&
&  0.99020E+00_JPRB, 0.99010E+00_JPRB, 0.98242E+00_JPRB, 0.95647E+00_JPRB, 0.91502E+00_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.77320E+00_JPRB, 0.76999E+00_JPRB, 0.75890E+00_JPRB, 0.77198E+00_JPRB, 0.78778E+00_JPRB, 0.78727E+00_JPRB, 0.80442E+00_JPRB ,&
&  0.82859E+00_JPRB, 0.85343E+00_JPRB, 0.87046E+00_JPRB, 0.88272E+00_JPRB, 0.94118E+00_JPRB, 0.10113E+01_JPRB, 0.10958E+01_JPRB ,&
&  0.11432E+01_JPRB, 0.11860E+01_JPRB, 0.11928E+01_JPRB, 0.11731E+01_JPRB, 0.11588E+01_JPRB, 0.11101E+01_JPRB, 0.10528E+01_JPRB ,&
&  0.11207E+01_JPRB, 0.14316E+01_JPRB, 0.19315E+01_JPRB, 0.23943E+01_JPRB, 0.28069E+01_JPRB, 0.24129E+01_JPRB, 0.25593E+01_JPRB ,&
&  0.22590E+01_JPRB, 0.18537E+01_JPRB, 0.19057E+01_JPRB, 0.20175E+01_JPRB, 0.21199E+01_JPRB, 0.21165E+01_JPRB, 0.21575E+01_JPRB ,&
&  0.21866E+01_JPRB, 0.22197E+01_JPRB, 0.22751E+01_JPRB, 0.23892E+01_JPRB, 0.24914E+01_JPRB, 0.25744E+01_JPRB, 0.26783E+01_JPRB ,&
&  0.27612E+01_JPRB, 0.28046E+01_JPRB, 0.28678E+01_JPRB, 0.26896E+01_JPRB, 0.29165E+01_JPRB, 0.33673E+01_JPRB, 0.37627E+01_JPRB ,&
&  0.39542E+01_JPRB, 0.40620E+01_JPRB, 0.44457E+01_JPRB, 0.46233E+01_JPRB, 0.47419E+01_JPRB, 0.46854E+01_JPRB, 0.46774E+01_JPRB ,&
&  0.47015E+01_JPRB, 0.43068E+01_JPRB, 0.31431E+01_JPRB, 0.33303E+01_JPRB, 0.29479E+01_JPRB, 0.23417E+01_JPRB, 0.22983E+01_JPRB ,&
&  0.20400E+01_JPRB, 0.17956E+01_JPRB, 0.15044E+01_JPRB, 0.14514E+01_JPRB, 0.15188E+01_JPRB, 0.16043E+01_JPRB, 0.17816E+01_JPRB ,&
&  0.15562E+01_JPRB, 0.12791E+01_JPRB, 0.10232E+01_JPRB, 0.96894E+00_JPRB, 0.96193E+00_JPRB, 0.10254E+01_JPRB, 0.10464E+01_JPRB ,&
&  0.10715E+01_JPRB, 0.98995E+00_JPRB, 0.10206E+01_JPRB, 0.11152E+01_JPRB, 0.85771E+00_JPRB, 0.76723E+00_JPRB, 0.77868E+00_JPRB ,&
&  0.79258E+00_JPRB, 0.80133E+00_JPRB, 0.80653E+00_JPRB, 0.80475E+00_JPRB, 0.81537E+00_JPRB, 0.83483E+00_JPRB, 0.84347E+00_JPRB ,&
&  0.84087E+00_JPRB, 0.81791E+00_JPRB, 0.79792E+00_JPRB, 0.78419E+00_JPRB, 0.77313E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.63812E+00_JPRB, 0.63965E+00_JPRB, 0.64227E+00_JPRB, 0.65081E+00_JPRB, 0.65187E+00_JPRB, 0.64883E+00_JPRB, 0.63779E+00_JPRB ,&
&  0.63085E+00_JPRB, 0.61728E+00_JPRB, 0.60876E+00_JPRB, 0.60388E+00_JPRB, 0.62040E+00_JPRB, 0.65678E+00_JPRB, 0.71074E+00_JPRB ,&
&  0.72676E+00_JPRB, 0.72363E+00_JPRB, 0.69505E+00_JPRB, 0.67777E+00_JPRB, 0.69944E+00_JPRB, 0.70135E+00_JPRB, 0.67618E+00_JPRB ,&
&  0.65519E+00_JPRB, 0.72159E+00_JPRB, 0.89996E+00_JPRB, 0.11624E+01_JPRB, 0.16295E+01_JPRB, 0.18855E+01_JPRB, 0.24041E+01_JPRB ,&
&  0.22774E+01_JPRB, 0.16791E+01_JPRB, 0.18126E+01_JPRB, 0.17502E+01_JPRB, 0.18738E+01_JPRB, 0.19421E+01_JPRB, 0.19683E+01_JPRB ,&
&  0.20312E+01_JPRB, 0.20954E+01_JPRB, 0.21560E+01_JPRB, 0.22232E+01_JPRB, 0.22754E+01_JPRB, 0.22855E+01_JPRB, 0.23008E+01_JPRB ,&
&  0.23136E+01_JPRB, 0.23056E+01_JPRB, 0.23682E+01_JPRB, 0.25449E+01_JPRB, 0.29766E+01_JPRB, 0.34627E+01_JPRB, 0.35479E+01_JPRB ,&
&  0.36011E+01_JPRB, 0.38859E+01_JPRB, 0.38366E+01_JPRB, 0.41026E+01_JPRB, 0.42192E+01_JPRB, 0.40989E+01_JPRB, 0.43749E+01_JPRB ,&
&  0.45430E+01_JPRB, 0.38025E+01_JPRB, 0.25402E+01_JPRB, 0.26130E+01_JPRB, 0.23686E+01_JPRB, 0.21073E+01_JPRB, 0.20987E+01_JPRB ,&
&  0.19184E+01_JPRB, 0.17471E+01_JPRB, 0.15673E+01_JPRB, 0.14522E+01_JPRB, 0.14012E+01_JPRB, 0.13165E+01_JPRB, 0.13013E+01_JPRB ,&
&  0.11952E+01_JPRB, 0.11417E+01_JPRB, 0.10896E+01_JPRB, 0.10522E+01_JPRB, 0.10697E+01_JPRB, 0.11348E+01_JPRB, 0.10079E+01_JPRB ,&
&  0.10081E+01_JPRB, 0.96181E+00_JPRB, 0.10715E+01_JPRB, 0.11415E+01_JPRB, 0.74918E+00_JPRB, 0.66162E+00_JPRB, 0.63482E+00_JPRB ,&
&  0.63857E+00_JPRB, 0.63321E+00_JPRB, 0.62949E+00_JPRB, 0.62105E+00_JPRB, 0.64554E+00_JPRB, 0.68436E+00_JPRB, 0.67877E+00_JPRB ,&
&  0.68408E+00_JPRB, 0.65798E+00_JPRB, 0.65418E+00_JPRB, 0.65725E+00_JPRB, 0.65803E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.60550E+00_JPRB, 0.59534E+00_JPRB, 0.58539E+00_JPRB, 0.57516E+00_JPRB, 0.58195E+00_JPRB, 0.58397E+00_JPRB, 0.59516E+00_JPRB ,&
&  0.60784E+00_JPRB, 0.60896E+00_JPRB, 0.61502E+00_JPRB, 0.61751E+00_JPRB, 0.63417E+00_JPRB, 0.64862E+00_JPRB, 0.68130E+00_JPRB ,&
&  0.71252E+00_JPRB, 0.72924E+00_JPRB, 0.75115E+00_JPRB, 0.77267E+00_JPRB, 0.78752E+00_JPRB, 0.80313E+00_JPRB, 0.78824E+00_JPRB ,&
&  0.76937E+00_JPRB, 0.76900E+00_JPRB, 0.80276E+00_JPRB, 0.93278E+00_JPRB, 0.11820E+01_JPRB, 0.15072E+01_JPRB, 0.21538E+01_JPRB ,&
&  0.21315E+01_JPRB, 0.15191E+01_JPRB, 0.13679E+01_JPRB, 0.12144E+01_JPRB, 0.12806E+01_JPRB, 0.13816E+01_JPRB, 0.13936E+01_JPRB ,&
&  0.14088E+01_JPRB, 0.14191E+01_JPRB, 0.14137E+01_JPRB, 0.13910E+01_JPRB, 0.13248E+01_JPRB, 0.12399E+01_JPRB, 0.11900E+01_JPRB ,&
&  0.12002E+01_JPRB, 0.12035E+01_JPRB, 0.13854E+01_JPRB, 0.18526E+01_JPRB, 0.22326E+01_JPRB, 0.28643E+01_JPRB, 0.27243E+01_JPRB ,&
&  0.27623E+01_JPRB, 0.28943E+01_JPRB, 0.28440E+01_JPRB, 0.30409E+01_JPRB, 0.31246E+01_JPRB, 0.32493E+01_JPRB, 0.32555E+01_JPRB ,&
&  0.34613E+01_JPRB, 0.30193E+01_JPRB, 0.21270E+01_JPRB, 0.22595E+01_JPRB, 0.20902E+01_JPRB, 0.20529E+01_JPRB, 0.19055E+01_JPRB ,&
&  0.17831E+01_JPRB, 0.16409E+01_JPRB, 0.15226E+01_JPRB, 0.14253E+01_JPRB, 0.13877E+01_JPRB, 0.13405E+01_JPRB, 0.12226E+01_JPRB ,&
&  0.11878E+01_JPRB, 0.11628E+01_JPRB, 0.11352E+01_JPRB, 0.10744E+01_JPRB, 0.11531E+01_JPRB, 0.10627E+01_JPRB, 0.13135E+01_JPRB ,&
&  0.94257E+00_JPRB, 0.85882E+00_JPRB, 0.69812E+00_JPRB, 0.86969E+00_JPRB, 0.66721E+00_JPRB, 0.57674E+00_JPRB, 0.57948E+00_JPRB ,&
&  0.56216E+00_JPRB, 0.54232E+00_JPRB, 0.52699E+00_JPRB, 0.52732E+00_JPRB, 0.54090E+00_JPRB, 0.56669E+00_JPRB, 0.56848E+00_JPRB ,&
&  0.57650E+00_JPRB, 0.58799E+00_JPRB, 0.60576E+00_JPRB, 0.62664E+00_JPRB, 0.62314E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.62808E+00_JPRB, 0.62909E+00_JPRB, 0.63828E+00_JPRB, 0.65472E+00_JPRB, 0.66348E+00_JPRB, 0.67031E+00_JPRB, 0.67570E+00_JPRB ,&
&  0.69514E+00_JPRB, 0.71734E+00_JPRB, 0.73790E+00_JPRB, 0.75813E+00_JPRB, 0.77690E+00_JPRB, 0.79283E+00_JPRB, 0.83087E+00_JPRB ,&
&  0.85143E+00_JPRB, 0.87960E+00_JPRB, 0.89891E+00_JPRB, 0.92015E+00_JPRB, 0.93869E+00_JPRB, 0.93116E+00_JPRB, 0.93749E+00_JPRB ,&
&  0.94832E+00_JPRB, 0.97851E+00_JPRB, 0.10086E+01_JPRB, 0.10632E+01_JPRB, 0.11340E+01_JPRB, 0.11529E+01_JPRB, 0.13068E+01_JPRB ,&
&  0.12984E+01_JPRB, 0.14775E+01_JPRB, 0.12957E+01_JPRB, 0.11099E+01_JPRB, 0.10659E+01_JPRB, 0.95895E+00_JPRB, 0.90711E+00_JPRB ,&
&  0.92669E+00_JPRB, 0.93533E+00_JPRB, 0.92131E+00_JPRB, 0.89712E+00_JPRB, 0.86254E+00_JPRB, 0.92784E+00_JPRB, 0.10543E+01_JPRB ,&
&  0.12726E+01_JPRB, 0.14011E+01_JPRB, 0.15676E+01_JPRB, 0.17081E+01_JPRB, 0.18973E+01_JPRB, 0.20088E+01_JPRB, 0.19848E+01_JPRB ,&
&  0.19875E+01_JPRB, 0.19588E+01_JPRB, 0.18438E+01_JPRB, 0.21175E+01_JPRB, 0.23346E+01_JPRB, 0.24335E+01_JPRB, 0.24550E+01_JPRB ,&
&  0.24397E+01_JPRB, 0.21598E+01_JPRB, 0.20816E+01_JPRB, 0.19488E+01_JPRB, 0.18103E+01_JPRB, 0.17891E+01_JPRB, 0.16773E+01_JPRB ,&
&  0.15556E+01_JPRB, 0.14732E+01_JPRB, 0.14436E+01_JPRB, 0.14402E+01_JPRB, 0.14504E+01_JPRB, 0.14392E+01_JPRB, 0.13229E+01_JPRB ,&
&  0.12370E+01_JPRB, 0.11695E+01_JPRB, 0.11160E+01_JPRB, 0.10691E+01_JPRB, 0.97998E+00_JPRB, 0.15876E+01_JPRB, 0.21075E+01_JPRB ,&
&  0.86660E+00_JPRB, 0.64625E+00_JPRB, 0.46565E+00_JPRB, 0.58893E+00_JPRB, 0.57338E+00_JPRB, 0.59596E+00_JPRB, 0.66716E+00_JPRB ,&
&  0.68238E+00_JPRB, 0.67026E+00_JPRB, 0.60689E+00_JPRB, 0.54544E+00_JPRB, 0.56105E+00_JPRB, 0.56579E+00_JPRB, 0.56820E+00_JPRB ,&
&  0.58355E+00_JPRB, 0.59263E+00_JPRB, 0.60479E+00_JPRB, 0.62609E+00_JPRB, 0.62235E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.65947E+00_JPRB, 0.67466E+00_JPRB, 0.69451E+00_JPRB, 0.72076E+00_JPRB, 0.74595E+00_JPRB, 0.76593E+00_JPRB, 0.77970E+00_JPRB ,&
&  0.80145E+00_JPRB, 0.82976E+00_JPRB, 0.84702E+00_JPRB, 0.85565E+00_JPRB, 0.87536E+00_JPRB, 0.89296E+00_JPRB, 0.92423E+00_JPRB ,&
&  0.94425E+00_JPRB, 0.98865E+00_JPRB, 0.10239E+01_JPRB, 0.10585E+01_JPRB, 0.10859E+01_JPRB, 0.11179E+01_JPRB, 0.11481E+01_JPRB ,&
&  0.11730E+01_JPRB, 0.12219E+01_JPRB, 0.12627E+01_JPRB, 0.12908E+01_JPRB, 0.13410E+01_JPRB, 0.13931E+01_JPRB, 0.11648E+01_JPRB ,&
&  0.14417E+01_JPRB, 0.13722E+01_JPRB, 0.12222E+01_JPRB, 0.11043E+01_JPRB, 0.10216E+01_JPRB, 0.97408E+00_JPRB, 0.95724E+00_JPRB ,&
&  0.98963E+00_JPRB, 0.11035E+01_JPRB, 0.11949E+01_JPRB, 0.12195E+01_JPRB, 0.12065E+01_JPRB, 0.13463E+01_JPRB, 0.16309E+01_JPRB ,&
&  0.17888E+01_JPRB, 0.18781E+01_JPRB, 0.19412E+01_JPRB, 0.19545E+01_JPRB, 0.20396E+01_JPRB, 0.21693E+01_JPRB, 0.22373E+01_JPRB ,&
&  0.22276E+01_JPRB, 0.21494E+01_JPRB, 0.19020E+01_JPRB, 0.23034E+01_JPRB, 0.22788E+01_JPRB, 0.21620E+01_JPRB, 0.19388E+01_JPRB ,&
&  0.17138E+01_JPRB, 0.15044E+01_JPRB, 0.15586E+01_JPRB, 0.16452E+01_JPRB, 0.15668E+01_JPRB, 0.15035E+01_JPRB, 0.13867E+01_JPRB ,&
&  0.12848E+01_JPRB, 0.12569E+01_JPRB, 0.12825E+01_JPRB, 0.13382E+01_JPRB, 0.13809E+01_JPRB, 0.13706E+01_JPRB, 0.12727E+01_JPRB ,&
&  0.11751E+01_JPRB, 0.10693E+01_JPRB, 0.99708E+00_JPRB, 0.94169E+00_JPRB, 0.88766E+00_JPRB, 0.90201E+00_JPRB, 0.12596E+01_JPRB ,&
&  0.74142E+00_JPRB, 0.47874E+00_JPRB, 0.50901E+00_JPRB, 0.53350E+00_JPRB, 0.57257E+00_JPRB, 0.61427E+00_JPRB, 0.77547E+00_JPRB ,&
&  0.98561E+00_JPRB, 0.11336E+01_JPRB, 0.11098E+01_JPRB, 0.84722E+00_JPRB, 0.67196E+00_JPRB, 0.61838E+00_JPRB, 0.61677E+00_JPRB ,&
&  0.62693E+00_JPRB, 0.62730E+00_JPRB, 0.63212E+00_JPRB, 0.63918E+00_JPRB, 0.65009E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.64374E+00_JPRB, 0.65481E+00_JPRB, 0.66918E+00_JPRB, 0.68468E+00_JPRB, 0.70355E+00_JPRB, 0.72282E+00_JPRB, 0.73751E+00_JPRB ,&
&  0.75312E+00_JPRB, 0.77110E+00_JPRB, 0.78775E+00_JPRB, 0.79920E+00_JPRB, 0.81277E+00_JPRB, 0.83856E+00_JPRB, 0.86890E+00_JPRB ,&
&  0.88352E+00_JPRB, 0.93149E+00_JPRB, 0.95559E+00_JPRB, 0.10015E+01_JPRB, 0.10441E+01_JPRB, 0.10956E+01_JPRB, 0.11345E+01_JPRB ,&
&  0.11739E+01_JPRB, 0.12308E+01_JPRB, 0.12519E+01_JPRB, 0.12427E+01_JPRB, 0.12613E+01_JPRB, 0.13512E+01_JPRB, 0.11124E+01_JPRB ,&
&  0.14063E+01_JPRB, 0.13912E+01_JPRB, 0.12774E+01_JPRB, 0.12330E+01_JPRB, 0.12027E+01_JPRB, 0.12266E+01_JPRB, 0.12748E+01_JPRB ,&
&  0.13514E+01_JPRB, 0.15356E+01_JPRB, 0.15670E+01_JPRB, 0.16331E+01_JPRB, 0.17508E+01_JPRB, 0.19995E+01_JPRB, 0.22280E+01_JPRB ,&
&  0.23195E+01_JPRB, 0.24963E+01_JPRB, 0.25833E+01_JPRB, 0.26658E+01_JPRB, 0.28013E+01_JPRB, 0.29434E+01_JPRB, 0.30433E+01_JPRB ,&
&  0.30052E+01_JPRB, 0.29367E+01_JPRB, 0.27006E+01_JPRB, 0.28921E+01_JPRB, 0.27164E+01_JPRB, 0.21781E+01_JPRB, 0.16794E+01_JPRB ,&
&  0.12177E+01_JPRB, 0.12463E+01_JPRB, 0.13181E+01_JPRB, 0.13745E+01_JPRB, 0.12788E+01_JPRB, 0.11979E+01_JPRB, 0.10294E+01_JPRB ,&
&  0.95688E+00_JPRB, 0.94350E+00_JPRB, 0.10008E+01_JPRB, 0.10835E+01_JPRB, 0.11296E+01_JPRB, 0.11139E+01_JPRB, 0.10357E+01_JPRB ,&
&  0.96274E+00_JPRB, 0.87379E+00_JPRB, 0.83050E+00_JPRB, 0.80713E+00_JPRB, 0.75098E+00_JPRB, 0.67275E+00_JPRB, 0.84678E+00_JPRB ,&
&  0.66035E+00_JPRB, 0.59221E+00_JPRB, 0.69217E+00_JPRB, 0.43345E+00_JPRB, 0.61455E+00_JPRB, 0.77930E+00_JPRB, 0.10437E+01_JPRB ,&
&  0.12506E+01_JPRB, 0.17636E+01_JPRB, 0.22123E+01_JPRB, 0.16825E+01_JPRB, 0.11651E+01_JPRB, 0.67681E+00_JPRB, 0.58781E+00_JPRB ,&
&  0.59822E+00_JPRB, 0.60498E+00_JPRB, 0.61497E+00_JPRB, 0.61671E+00_JPRB, 0.63034E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.59496E+00_JPRB, 0.59022E+00_JPRB, 0.59043E+00_JPRB, 0.58156E+00_JPRB, 0.57740E+00_JPRB, 0.57862E+00_JPRB, 0.56759E+00_JPRB ,&
&  0.57595E+00_JPRB, 0.59230E+00_JPRB, 0.59794E+00_JPRB, 0.61679E+00_JPRB, 0.62925E+00_JPRB, 0.67000E+00_JPRB, 0.72976E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.86541E+00_JPRB, 0.91342E+00_JPRB, 0.97137E+00_JPRB, 0.10209E+01_JPRB, 0.10737E+01_JPRB, 0.11190E+01_JPRB ,&
&  0.11826E+01_JPRB, 0.12598E+01_JPRB, 0.13406E+01_JPRB, 0.13971E+01_JPRB, 0.14102E+01_JPRB, 0.15494E+01_JPRB, 0.12441E+01_JPRB ,&
&  0.14621E+01_JPRB, 0.14994E+01_JPRB, 0.14364E+01_JPRB, 0.14214E+01_JPRB, 0.14140E+01_JPRB, 0.14283E+01_JPRB, 0.14532E+01_JPRB ,&
&  0.15236E+01_JPRB, 0.15891E+01_JPRB, 0.16531E+01_JPRB, 0.18746E+01_JPRB, 0.21168E+01_JPRB, 0.23231E+01_JPRB, 0.25197E+01_JPRB ,&
&  0.27089E+01_JPRB, 0.28997E+01_JPRB, 0.31360E+01_JPRB, 0.33788E+01_JPRB, 0.36250E+01_JPRB, 0.37775E+01_JPRB, 0.39566E+01_JPRB ,&
&  0.40437E+01_JPRB, 0.41003E+01_JPRB, 0.38200E+01_JPRB, 0.33209E+01_JPRB, 0.27793E+01_JPRB, 0.21702E+01_JPRB, 0.15723E+01_JPRB ,&
&  0.11640E+01_JPRB, 0.10817E+01_JPRB, 0.10730E+01_JPRB, 0.10663E+01_JPRB, 0.94842E+00_JPRB, 0.86990E+00_JPRB, 0.81945E+00_JPRB ,&
&  0.77032E+00_JPRB, 0.70937E+00_JPRB, 0.74334E+00_JPRB, 0.79979E+00_JPRB, 0.81479E+00_JPRB, 0.80454E+00_JPRB, 0.78104E+00_JPRB ,&
&  0.74712E+00_JPRB, 0.69717E+00_JPRB, 0.69330E+00_JPRB, 0.69153E+00_JPRB, 0.70339E+00_JPRB, 0.75125E+00_JPRB, 0.66928E+00_JPRB ,&
&  0.72495E+00_JPRB, 0.58336E+00_JPRB, 0.58243E+00_JPRB, 0.60473E+00_JPRB, 0.65264E+00_JPRB, 0.83614E+00_JPRB, 0.11650E+01_JPRB ,&
&  0.17567E+01_JPRB, 0.20069E+01_JPRB, 0.22527E+01_JPRB, 0.28029E+01_JPRB, 0.24426E+01_JPRB, 0.18081E+01_JPRB, 0.54354E+00_JPRB ,&
&  0.54326E+00_JPRB, 0.54896E+00_JPRB, 0.57346E+00_JPRB, 0.58298E+00_JPRB, 0.58828E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.50860E+00_JPRB, 0.49417E+00_JPRB, 0.48687E+00_JPRB, 0.48198E+00_JPRB, 0.46692E+00_JPRB, 0.46814E+00_JPRB, 0.46598E+00_JPRB ,&
&  0.48339E+00_JPRB, 0.50600E+00_JPRB, 0.54555E+00_JPRB, 0.59338E+00_JPRB, 0.65608E+00_JPRB, 0.70295E+00_JPRB, 0.75982E+00_JPRB ,&
&  0.79698E+00_JPRB, 0.84428E+00_JPRB, 0.89615E+00_JPRB, 0.94336E+00_JPRB, 0.99897E+00_JPRB, 0.10426E+01_JPRB, 0.11084E+01_JPRB ,&
&  0.11692E+01_JPRB, 0.12399E+01_JPRB, 0.13312E+01_JPRB, 0.14132E+01_JPRB, 0.14808E+01_JPRB, 0.15181E+01_JPRB, 0.12114E+01_JPRB ,&
&  0.15115E+01_JPRB, 0.15796E+01_JPRB, 0.15964E+01_JPRB, 0.15634E+01_JPRB, 0.15190E+01_JPRB, 0.14747E+01_JPRB, 0.14958E+01_JPRB ,&
&  0.15337E+01_JPRB, 0.15628E+01_JPRB, 0.15912E+01_JPRB, 0.16787E+01_JPRB, 0.21329E+01_JPRB, 0.23335E+01_JPRB, 0.25401E+01_JPRB ,&
&  0.27614E+01_JPRB, 0.30094E+01_JPRB, 0.32683E+01_JPRB, 0.35302E+01_JPRB, 0.37075E+01_JPRB, 0.38997E+01_JPRB, 0.41018E+01_JPRB ,&
&  0.42538E+01_JPRB, 0.43814E+01_JPRB, 0.41883E+01_JPRB, 0.32408E+01_JPRB, 0.26391E+01_JPRB, 0.21194E+01_JPRB, 0.13526E+01_JPRB ,&
&  0.10158E+01_JPRB, 0.92738E+00_JPRB, 0.92144E+00_JPRB, 0.88924E+00_JPRB, 0.78728E+00_JPRB, 0.71003E+00_JPRB, 0.66013E+00_JPRB ,&
&  0.59174E+00_JPRB, 0.57617E+00_JPRB, 0.57401E+00_JPRB, 0.61673E+00_JPRB, 0.64243E+00_JPRB, 0.61524E+00_JPRB, 0.59621E+00_JPRB ,&
&  0.56019E+00_JPRB, 0.53131E+00_JPRB, 0.56561E+00_JPRB, 0.61866E+00_JPRB, 0.73581E+00_JPRB, 0.85606E+00_JPRB, 0.92872E+00_JPRB ,&
&  0.95609E+00_JPRB, 0.59361E+00_JPRB, 0.58496E+00_JPRB, 0.58874E+00_JPRB, 0.61422E+00_JPRB, 0.66380E+00_JPRB, 0.75694E+00_JPRB ,&
&  0.95531E+00_JPRB, 0.13062E+01_JPRB, 0.11573E+01_JPRB, 0.88396E+00_JPRB, 0.95319E+00_JPRB, 0.87916E+00_JPRB, 0.49032E+00_JPRB ,&
&  0.49314E+00_JPRB, 0.49356E+00_JPRB, 0.50981E+00_JPRB, 0.51490E+00_JPRB, 0.51820E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.45008E+00_JPRB, 0.43435E+00_JPRB, 0.41688E+00_JPRB, 0.40797E+00_JPRB, 0.40045E+00_JPRB, 0.39222E+00_JPRB, 0.40377E+00_JPRB ,&
&  0.43333E+00_JPRB, 0.45996E+00_JPRB, 0.50433E+00_JPRB, 0.57073E+00_JPRB, 0.63447E+00_JPRB, 0.67273E+00_JPRB, 0.71517E+00_JPRB ,&
&  0.75193E+00_JPRB, 0.79206E+00_JPRB, 0.82600E+00_JPRB, 0.85839E+00_JPRB, 0.89980E+00_JPRB, 0.95076E+00_JPRB, 0.10194E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.11863E+01_JPRB, 0.12910E+01_JPRB, 0.14202E+01_JPRB, 0.15459E+01_JPRB, 0.16075E+01_JPRB, 0.16993E+01_JPRB ,&
&  0.12708E+01_JPRB, 0.15892E+01_JPRB, 0.16103E+01_JPRB, 0.15847E+01_JPRB, 0.15429E+01_JPRB, 0.15036E+01_JPRB, 0.14992E+01_JPRB ,&
&  0.15091E+01_JPRB, 0.14474E+01_JPRB, 0.14798E+01_JPRB, 0.15938E+01_JPRB, 0.18793E+01_JPRB, 0.20885E+01_JPRB, 0.22984E+01_JPRB ,&
&  0.25206E+01_JPRB, 0.27676E+01_JPRB, 0.30252E+01_JPRB, 0.32715E+01_JPRB, 0.34672E+01_JPRB, 0.36803E+01_JPRB, 0.38417E+01_JPRB ,&
&  0.40392E+01_JPRB, 0.42026E+01_JPRB, 0.42513E+01_JPRB, 0.34420E+01_JPRB, 0.26970E+01_JPRB, 0.22745E+01_JPRB, 0.15951E+01_JPRB ,&
&  0.10302E+01_JPRB, 0.80716E+00_JPRB, 0.81574E+00_JPRB, 0.83549E+00_JPRB, 0.76212E+00_JPRB, 0.69948E+00_JPRB, 0.63296E+00_JPRB ,&
&  0.58689E+00_JPRB, 0.52773E+00_JPRB, 0.51100E+00_JPRB, 0.52929E+00_JPRB, 0.56873E+00_JPRB, 0.54055E+00_JPRB, 0.49868E+00_JPRB ,&
&  0.46829E+00_JPRB, 0.45730E+00_JPRB, 0.50136E+00_JPRB, 0.55262E+00_JPRB, 0.62564E+00_JPRB, 0.66587E+00_JPRB, 0.67751E+00_JPRB ,&
&  0.71063E+00_JPRB, 0.62073E+00_JPRB, 0.60964E+00_JPRB, 0.61566E+00_JPRB, 0.61487E+00_JPRB, 0.62073E+00_JPRB, 0.63686E+00_JPRB ,&
&  0.67344E+00_JPRB, 0.73781E+00_JPRB, 0.75657E+00_JPRB, 0.70212E+00_JPRB, 0.64452E+00_JPRB, 0.59840E+00_JPRB, 0.55437E+00_JPRB ,&
&  0.55771E+00_JPRB, 0.54557E+00_JPRB, 0.53183E+00_JPRB, 0.50974E+00_JPRB, 0.48088E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.46134E+00_JPRB, 0.43416E+00_JPRB, 0.39472E+00_JPRB, 0.36032E+00_JPRB, 0.38358E+00_JPRB, 0.38444E+00_JPRB, 0.40064E+00_JPRB ,&
&  0.40746E+00_JPRB, 0.43371E+00_JPRB, 0.46064E+00_JPRB, 0.50065E+00_JPRB, 0.55208E+00_JPRB, 0.60414E+00_JPRB, 0.64823E+00_JPRB ,&
&  0.68796E+00_JPRB, 0.72736E+00_JPRB, 0.75810E+00_JPRB, 0.77107E+00_JPRB, 0.79089E+00_JPRB, 0.82355E+00_JPRB, 0.86310E+00_JPRB ,&
&  0.91509E+00_JPRB, 0.98319E+00_JPRB, 0.10773E+01_JPRB, 0.11980E+01_JPRB, 0.13403E+01_JPRB, 0.14248E+01_JPRB, 0.14843E+01_JPRB ,&
&  0.86148E+00_JPRB, 0.13375E+01_JPRB, 0.16189E+01_JPRB, 0.15645E+01_JPRB, 0.15301E+01_JPRB, 0.14972E+01_JPRB, 0.14823E+01_JPRB ,&
&  0.13992E+01_JPRB, 0.13125E+01_JPRB, 0.13304E+01_JPRB, 0.15557E+01_JPRB, 0.15724E+01_JPRB, 0.17602E+01_JPRB, 0.19193E+01_JPRB ,&
&  0.20904E+01_JPRB, 0.22713E+01_JPRB, 0.24791E+01_JPRB, 0.27354E+01_JPRB, 0.29947E+01_JPRB, 0.32103E+01_JPRB, 0.33997E+01_JPRB ,&
&  0.36524E+01_JPRB, 0.38840E+01_JPRB, 0.39612E+01_JPRB, 0.30308E+01_JPRB, 0.27162E+01_JPRB, 0.25168E+01_JPRB, 0.22063E+01_JPRB ,&
&  0.12500E+01_JPRB, 0.82234E+00_JPRB, 0.82022E+00_JPRB, 0.85022E+00_JPRB, 0.79368E+00_JPRB, 0.73986E+00_JPRB, 0.68567E+00_JPRB ,&
&  0.61760E+00_JPRB, 0.56475E+00_JPRB, 0.53081E+00_JPRB, 0.51911E+00_JPRB, 0.54186E+00_JPRB, 0.54590E+00_JPRB, 0.51966E+00_JPRB ,&
&  0.48918E+00_JPRB, 0.49068E+00_JPRB, 0.52267E+00_JPRB, 0.55001E+00_JPRB, 0.59225E+00_JPRB, 0.62125E+00_JPRB, 0.64828E+00_JPRB ,&
&  0.65716E+00_JPRB, 0.62060E+00_JPRB, 0.62309E+00_JPRB, 0.63565E+00_JPRB, 0.63947E+00_JPRB, 0.64178E+00_JPRB, 0.64053E+00_JPRB ,&
&  0.63962E+00_JPRB, 0.65819E+00_JPRB, 0.66895E+00_JPRB, 0.67703E+00_JPRB, 0.66352E+00_JPRB, 0.64641E+00_JPRB, 0.63870E+00_JPRB ,&
&  0.62259E+00_JPRB, 0.59865E+00_JPRB, 0.57351E+00_JPRB, 0.53345E+00_JPRB, 0.49451E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.48414E+00_JPRB, 0.45344E+00_JPRB, 0.41395E+00_JPRB, 0.39891E+00_JPRB, 0.42320E+00_JPRB, 0.44367E+00_JPRB, 0.43925E+00_JPRB ,&
&  0.43353E+00_JPRB, 0.42421E+00_JPRB, 0.41963E+00_JPRB, 0.42993E+00_JPRB, 0.46791E+00_JPRB, 0.52297E+00_JPRB, 0.56674E+00_JPRB ,&
&  0.60782E+00_JPRB, 0.64094E+00_JPRB, 0.66748E+00_JPRB, 0.67680E+00_JPRB, 0.67264E+00_JPRB, 0.68577E+00_JPRB, 0.70675E+00_JPRB ,&
&  0.74246E+00_JPRB, 0.79202E+00_JPRB, 0.85635E+00_JPRB, 0.94805E+00_JPRB, 0.10775E+01_JPRB, 0.12365E+01_JPRB, 0.14001E+01_JPRB ,&
&  0.10299E+01_JPRB, 0.51830E+00_JPRB, 0.13494E+01_JPRB, 0.16803E+01_JPRB, 0.15837E+01_JPRB, 0.15469E+01_JPRB, 0.14611E+01_JPRB ,&
&  0.13448E+01_JPRB, 0.13456E+01_JPRB, 0.12947E+01_JPRB, 0.14166E+01_JPRB, 0.13959E+01_JPRB, 0.14572E+01_JPRB, 0.15174E+01_JPRB ,&
&  0.16042E+01_JPRB, 0.17057E+01_JPRB, 0.18590E+01_JPRB, 0.20443E+01_JPRB, 0.22494E+01_JPRB, 0.25554E+01_JPRB, 0.27794E+01_JPRB ,&
&  0.29799E+01_JPRB, 0.31774E+01_JPRB, 0.32304E+01_JPRB, 0.23224E+01_JPRB, 0.18443E+01_JPRB, 0.15985E+01_JPRB, 0.14225E+01_JPRB ,&
&  0.11822E+01_JPRB, 0.90489E+00_JPRB, 0.86695E+00_JPRB, 0.89047E+00_JPRB, 0.85326E+00_JPRB, 0.77074E+00_JPRB, 0.76451E+00_JPRB ,&
&  0.67763E+00_JPRB, 0.62105E+00_JPRB, 0.57984E+00_JPRB, 0.54677E+00_JPRB, 0.55067E+00_JPRB, 0.56251E+00_JPRB, 0.56577E+00_JPRB ,&
&  0.54839E+00_JPRB, 0.54742E+00_JPRB, 0.56182E+00_JPRB, 0.56357E+00_JPRB, 0.56673E+00_JPRB, 0.57446E+00_JPRB, 0.58118E+00_JPRB ,&
&  0.61110E+00_JPRB, 0.61151E+00_JPRB, 0.62049E+00_JPRB, 0.63857E+00_JPRB, 0.64892E+00_JPRB, 0.63793E+00_JPRB, 0.64307E+00_JPRB ,&
&  0.65739E+00_JPRB, 0.67110E+00_JPRB, 0.66782E+00_JPRB, 0.67898E+00_JPRB, 0.67563E+00_JPRB, 0.67333E+00_JPRB, 0.68169E+00_JPRB ,&
&  0.69393E+00_JPRB, 0.69062E+00_JPRB, 0.65367E+00_JPRB, 0.57025E+00_JPRB, 0.52402E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.51459E+00_JPRB, 0.46965E+00_JPRB, 0.43343E+00_JPRB, 0.43499E+00_JPRB, 0.45663E+00_JPRB, 0.46352E+00_JPRB, 0.46313E+00_JPRB ,&
&  0.45380E+00_JPRB, 0.44252E+00_JPRB, 0.42465E+00_JPRB, 0.41639E+00_JPRB, 0.42149E+00_JPRB, 0.43525E+00_JPRB, 0.47400E+00_JPRB ,&
&  0.51251E+00_JPRB, 0.54894E+00_JPRB, 0.57890E+00_JPRB, 0.58205E+00_JPRB, 0.56525E+00_JPRB, 0.56380E+00_JPRB, 0.57495E+00_JPRB ,&
&  0.58847E+00_JPRB, 0.61854E+00_JPRB, 0.65873E+00_JPRB, 0.71400E+00_JPRB, 0.79803E+00_JPRB, 0.93781E+00_JPRB, 0.11535E+01_JPRB ,&
&  0.15005E+01_JPRB, 0.10391E+01_JPRB, 0.71687E+00_JPRB, 0.14362E+01_JPRB, 0.17537E+01_JPRB, 0.17865E+01_JPRB, 0.15041E+01_JPRB ,&
&  0.14544E+01_JPRB, 0.13860E+01_JPRB, 0.13273E+01_JPRB, 0.13811E+01_JPRB, 0.13076E+01_JPRB, 0.12956E+01_JPRB, 0.12839E+01_JPRB ,&
&  0.12703E+01_JPRB, 0.13462E+01_JPRB, 0.14488E+01_JPRB, 0.15478E+01_JPRB, 0.16469E+01_JPRB, 0.18584E+01_JPRB, 0.21278E+01_JPRB ,&
&  0.23908E+01_JPRB, 0.25568E+01_JPRB, 0.26132E+01_JPRB, 0.19430E+01_JPRB, 0.15929E+01_JPRB, 0.14260E+01_JPRB, 0.13411E+01_JPRB ,&
&  0.11695E+01_JPRB, 0.10608E+01_JPRB, 0.10252E+01_JPRB, 0.96467E+00_JPRB, 0.90557E+00_JPRB, 0.84519E+00_JPRB, 0.84845E+00_JPRB ,&
&  0.76837E+00_JPRB, 0.70016E+00_JPRB, 0.65751E+00_JPRB, 0.62967E+00_JPRB, 0.61750E+00_JPRB, 0.61780E+00_JPRB, 0.61287E+00_JPRB ,&
&  0.61499E+00_JPRB, 0.61716E+00_JPRB, 0.62041E+00_JPRB, 0.60463E+00_JPRB, 0.58919E+00_JPRB, 0.57898E+00_JPRB, 0.56418E+00_JPRB ,&
&  0.58032E+00_JPRB, 0.60455E+00_JPRB, 0.61693E+00_JPRB, 0.63671E+00_JPRB, 0.63726E+00_JPRB, 0.63187E+00_JPRB, 0.65087E+00_JPRB ,&
&  0.65908E+00_JPRB, 0.67663E+00_JPRB, 0.67343E+00_JPRB, 0.68020E+00_JPRB, 0.67195E+00_JPRB, 0.66694E+00_JPRB, 0.67521E+00_JPRB ,&
&  0.71683E+00_JPRB, 0.76502E+00_JPRB, 0.75437E+00_JPRB, 0.59866E+00_JPRB, 0.54436E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.50308E+00_JPRB, 0.48755E+00_JPRB, 0.44908E+00_JPRB, 0.45411E+00_JPRB, 0.45788E+00_JPRB, 0.46547E+00_JPRB, 0.46536E+00_JPRB ,&
&  0.46538E+00_JPRB, 0.46711E+00_JPRB, 0.46948E+00_JPRB, 0.46067E+00_JPRB, 0.43600E+00_JPRB, 0.42516E+00_JPRB, 0.42766E+00_JPRB ,&
&  0.43958E+00_JPRB, 0.46682E+00_JPRB, 0.49949E+00_JPRB, 0.50490E+00_JPRB, 0.50430E+00_JPRB, 0.49938E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.50090E+00_JPRB, 0.51054E+00_JPRB, 0.52333E+00_JPRB, 0.55260E+00_JPRB, 0.60498E+00_JPRB, 0.68639E+00_JPRB, 0.81876E+00_JPRB ,&
&  0.11423E+01_JPRB, 0.12002E+01_JPRB, 0.59248E+00_JPRB, 0.11336E+01_JPRB, 0.17360E+01_JPRB, 0.18334E+01_JPRB, 0.17396E+01_JPRB ,&
&  0.16394E+01_JPRB, 0.14647E+01_JPRB, 0.13790E+01_JPRB, 0.13133E+01_JPRB, 0.11976E+01_JPRB, 0.11366E+01_JPRB, 0.10766E+01_JPRB ,&
&  0.10425E+01_JPRB, 0.10878E+01_JPRB, 0.11945E+01_JPRB, 0.12756E+01_JPRB, 0.13039E+01_JPRB, 0.13677E+01_JPRB, 0.15671E+01_JPRB ,&
&  0.18630E+01_JPRB, 0.21630E+01_JPRB, 0.23002E+01_JPRB, 0.18167E+01_JPRB, 0.15858E+01_JPRB, 0.15852E+01_JPRB, 0.15222E+01_JPRB ,&
&  0.13980E+01_JPRB, 0.12350E+01_JPRB, 0.11223E+01_JPRB, 0.10443E+01_JPRB, 0.96500E+00_JPRB, 0.93060E+00_JPRB, 0.93798E+00_JPRB ,&
&  0.87409E+00_JPRB, 0.79160E+00_JPRB, 0.73998E+00_JPRB, 0.70682E+00_JPRB, 0.68341E+00_JPRB, 0.67139E+00_JPRB, 0.66238E+00_JPRB ,&
&  0.65646E+00_JPRB, 0.66344E+00_JPRB, 0.67383E+00_JPRB, 0.66474E+00_JPRB, 0.64293E+00_JPRB, 0.61685E+00_JPRB, 0.59200E+00_JPRB ,&
&  0.58639E+00_JPRB, 0.59397E+00_JPRB, 0.61575E+00_JPRB, 0.62628E+00_JPRB, 0.63203E+00_JPRB, 0.64097E+00_JPRB, 0.65219E+00_JPRB ,&
&  0.66655E+00_JPRB, 0.67588E+00_JPRB, 0.66542E+00_JPRB, 0.65130E+00_JPRB, 0.66744E+00_JPRB, 0.65531E+00_JPRB, 0.64486E+00_JPRB ,&
&  0.68130E+00_JPRB, 0.75416E+00_JPRB, 0.87082E+00_JPRB, 0.60495E+00_JPRB, 0.54324E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.49889E+00_JPRB, 0.47298E+00_JPRB, 0.46657E+00_JPRB, 0.46541E+00_JPRB, 0.46931E+00_JPRB, 0.47802E+00_JPRB, 0.48695E+00_JPRB ,&
&  0.48891E+00_JPRB, 0.48928E+00_JPRB, 0.48954E+00_JPRB, 0.48529E+00_JPRB, 0.47248E+00_JPRB, 0.45929E+00_JPRB, 0.44681E+00_JPRB ,&
&  0.44344E+00_JPRB, 0.44537E+00_JPRB, 0.45910E+00_JPRB, 0.46695E+00_JPRB, 0.46785E+00_JPRB, 0.46523E+00_JPRB, 0.46630E+00_JPRB ,&
&  0.47054E+00_JPRB, 0.47054E+00_JPRB, 0.46883E+00_JPRB, 0.48107E+00_JPRB, 0.50700E+00_JPRB, 0.55786E+00_JPRB, 0.64233E+00_JPRB ,&
&  0.87438E+00_JPRB, 0.10638E+01_JPRB, 0.10981E+01_JPRB, 0.16934E+01_JPRB, 0.20844E+01_JPRB, 0.20197E+01_JPRB, 0.19351E+01_JPRB ,&
&  0.17884E+01_JPRB, 0.15959E+01_JPRB, 0.15365E+01_JPRB, 0.13145E+01_JPRB, 0.11506E+01_JPRB, 0.10221E+01_JPRB, 0.92237E+00_JPRB ,&
&  0.84988E+00_JPRB, 0.83782E+00_JPRB, 0.90622E+00_JPRB, 0.10230E+01_JPRB, 0.10999E+01_JPRB, 0.11285E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.14121E+01_JPRB, 0.17881E+01_JPRB, 0.20961E+01_JPRB, 0.19446E+01_JPRB, 0.15825E+01_JPRB, 0.17480E+01_JPRB, 0.19320E+01_JPRB ,&
&  0.21852E+01_JPRB, 0.15872E+01_JPRB, 0.12546E+01_JPRB, 0.11235E+01_JPRB, 0.10235E+01_JPRB, 0.10925E+01_JPRB, 0.10342E+01_JPRB ,&
&  0.96207E+00_JPRB, 0.88698E+00_JPRB, 0.82099E+00_JPRB, 0.77888E+00_JPRB, 0.74728E+00_JPRB, 0.72389E+00_JPRB, 0.70855E+00_JPRB ,&
&  0.69939E+00_JPRB, 0.69506E+00_JPRB, 0.70372E+00_JPRB, 0.70571E+00_JPRB, 0.69590E+00_JPRB, 0.67250E+00_JPRB, 0.63982E+00_JPRB ,&
&  0.61322E+00_JPRB, 0.60470E+00_JPRB, 0.59270E+00_JPRB, 0.60548E+00_JPRB, 0.62272E+00_JPRB, 0.63251E+00_JPRB, 0.63397E+00_JPRB ,&
&  0.65063E+00_JPRB, 0.66721E+00_JPRB, 0.66325E+00_JPRB, 0.65200E+00_JPRB, 0.64977E+00_JPRB, 0.65043E+00_JPRB, 0.62320E+00_JPRB ,&
&  0.61086E+00_JPRB, 0.59904E+00_JPRB, 0.58725E+00_JPRB, 0.56676E+00_JPRB, 0.52809E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.50167E+00_JPRB, 0.48523E+00_JPRB, 0.48024E+00_JPRB, 0.48381E+00_JPRB, 0.48820E+00_JPRB, 0.49334E+00_JPRB, 0.50398E+00_JPRB ,&
&  0.50922E+00_JPRB, 0.50969E+00_JPRB, 0.50773E+00_JPRB, 0.50445E+00_JPRB, 0.49534E+00_JPRB, 0.48515E+00_JPRB, 0.47368E+00_JPRB ,&
&  0.46444E+00_JPRB, 0.44380E+00_JPRB, 0.43323E+00_JPRB, 0.44049E+00_JPRB, 0.44415E+00_JPRB, 0.44406E+00_JPRB, 0.44530E+00_JPRB ,&
&  0.44578E+00_JPRB, 0.45044E+00_JPRB, 0.45059E+00_JPRB, 0.45719E+00_JPRB, 0.47054E+00_JPRB, 0.50459E+00_JPRB, 0.55635E+00_JPRB ,&
&  0.71806E+00_JPRB, 0.99081E+00_JPRB, 0.10174E+01_JPRB, 0.19413E+01_JPRB, 0.21542E+01_JPRB, 0.20908E+01_JPRB, 0.18464E+01_JPRB ,&
&  0.16926E+01_JPRB, 0.17246E+01_JPRB, 0.14088E+01_JPRB, 0.12017E+01_JPRB, 0.10881E+01_JPRB, 0.97258E+00_JPRB, 0.84032E+00_JPRB ,&
&  0.74192E+00_JPRB, 0.69912E+00_JPRB, 0.69446E+00_JPRB, 0.74818E+00_JPRB, 0.84457E+00_JPRB, 0.93210E+00_JPRB, 0.10001E+01_JPRB ,&
&  0.11186E+01_JPRB, 0.13945E+01_JPRB, 0.17976E+01_JPRB, 0.17810E+01_JPRB, 0.16783E+01_JPRB, 0.19297E+01_JPRB, 0.24193E+01_JPRB ,&
&  0.29329E+01_JPRB, 0.23147E+01_JPRB, 0.15343E+01_JPRB, 0.12422E+01_JPRB, 0.11613E+01_JPRB, 0.11896E+01_JPRB, 0.11631E+01_JPRB ,&
&  0.10783E+01_JPRB, 0.10059E+01_JPRB, 0.91905E+00_JPRB, 0.86138E+00_JPRB, 0.82043E+00_JPRB, 0.77546E+00_JPRB, 0.74400E+00_JPRB ,&
&  0.72772E+00_JPRB, 0.71773E+00_JPRB, 0.71504E+00_JPRB, 0.70880E+00_JPRB, 0.70699E+00_JPRB, 0.70238E+00_JPRB, 0.68938E+00_JPRB ,&
&  0.66326E+00_JPRB, 0.62939E+00_JPRB, 0.59252E+00_JPRB, 0.58644E+00_JPRB, 0.60312E+00_JPRB, 0.61604E+00_JPRB, 0.62390E+00_JPRB ,&
&  0.65427E+00_JPRB, 0.66439E+00_JPRB, 0.65145E+00_JPRB, 0.62784E+00_JPRB, 0.62968E+00_JPRB, 0.65585E+00_JPRB, 0.62632E+00_JPRB ,&
&  0.62056E+00_JPRB, 0.60885E+00_JPRB, 0.59348E+00_JPRB, 0.57613E+00_JPRB, 0.53973E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.52921E+00_JPRB, 0.51562E+00_JPRB, 0.50677E+00_JPRB, 0.50500E+00_JPRB, 0.50083E+00_JPRB, 0.50224E+00_JPRB, 0.51156E+00_JPRB ,&
&  0.52566E+00_JPRB, 0.52934E+00_JPRB, 0.52668E+00_JPRB, 0.52307E+00_JPRB, 0.51630E+00_JPRB, 0.50667E+00_JPRB, 0.49494E+00_JPRB ,&
&  0.48455E+00_JPRB, 0.46880E+00_JPRB, 0.44813E+00_JPRB, 0.43300E+00_JPRB, 0.43604E+00_JPRB, 0.44095E+00_JPRB, 0.44163E+00_JPRB ,&
&  0.44103E+00_JPRB, 0.44740E+00_JPRB, 0.45008E+00_JPRB, 0.46018E+00_JPRB, 0.46792E+00_JPRB, 0.48806E+00_JPRB, 0.53410E+00_JPRB ,&
&  0.66212E+00_JPRB, 0.10888E+01_JPRB, 0.10564E+01_JPRB, 0.17605E+01_JPRB, 0.19955E+01_JPRB, 0.19250E+01_JPRB, 0.16797E+01_JPRB ,&
&  0.15529E+01_JPRB, 0.14655E+01_JPRB, 0.12491E+01_JPRB, 0.10741E+01_JPRB, 0.96328E+00_JPRB, 0.87527E+00_JPRB, 0.79194E+00_JPRB ,&
&  0.72735E+00_JPRB, 0.69054E+00_JPRB, 0.66893E+00_JPRB, 0.65912E+00_JPRB, 0.69167E+00_JPRB, 0.75783E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.91874E+00_JPRB, 0.10673E+01_JPRB, 0.14123E+01_JPRB, 0.17589E+01_JPRB, 0.16586E+01_JPRB, 0.19354E+01_JPRB, 0.28902E+01_JPRB ,&
&  0.34009E+01_JPRB, 0.33106E+01_JPRB, 0.21357E+01_JPRB, 0.15003E+01_JPRB, 0.13020E+01_JPRB, 0.13114E+01_JPRB, 0.13010E+01_JPRB ,&
&  0.12232E+01_JPRB, 0.11644E+01_JPRB, 0.10675E+01_JPRB, 0.96044E+00_JPRB, 0.88148E+00_JPRB, 0.82609E+00_JPRB, 0.77450E+00_JPRB ,&
&  0.73671E+00_JPRB, 0.72002E+00_JPRB, 0.70687E+00_JPRB, 0.69355E+00_JPRB, 0.68727E+00_JPRB, 0.68952E+00_JPRB, 0.69286E+00_JPRB ,&
&  0.68994E+00_JPRB, 0.66904E+00_JPRB, 0.62212E+00_JPRB, 0.59283E+00_JPRB, 0.59334E+00_JPRB, 0.60516E+00_JPRB, 0.61319E+00_JPRB ,&
&  0.65076E+00_JPRB, 0.65931E+00_JPRB, 0.63855E+00_JPRB, 0.62681E+00_JPRB, 0.62011E+00_JPRB, 0.66173E+00_JPRB, 0.63098E+00_JPRB ,&
&  0.61961E+00_JPRB, 0.60987E+00_JPRB, 0.60572E+00_JPRB, 0.59518E+00_JPRB, 0.56365E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.55628E+00_JPRB, 0.53860E+00_JPRB, 0.52917E+00_JPRB, 0.51724E+00_JPRB, 0.50820E+00_JPRB, 0.50585E+00_JPRB, 0.51365E+00_JPRB ,&
&  0.53062E+00_JPRB, 0.54344E+00_JPRB, 0.53689E+00_JPRB, 0.53292E+00_JPRB, 0.53057E+00_JPRB, 0.52223E+00_JPRB, 0.51060E+00_JPRB ,&
&  0.49890E+00_JPRB, 0.48754E+00_JPRB, 0.47346E+00_JPRB, 0.45773E+00_JPRB, 0.44671E+00_JPRB, 0.44649E+00_JPRB, 0.44445E+00_JPRB ,&
&  0.44270E+00_JPRB, 0.44675E+00_JPRB, 0.45127E+00_JPRB, 0.45942E+00_JPRB, 0.46781E+00_JPRB, 0.48289E+00_JPRB, 0.52332E+00_JPRB ,&
&  0.67071E+00_JPRB, 0.53688E+00_JPRB, 0.12349E+01_JPRB, 0.15485E+01_JPRB, 0.16021E+01_JPRB, 0.15005E+01_JPRB, 0.14355E+01_JPRB ,&
&  0.13780E+01_JPRB, 0.12421E+01_JPRB, 0.11236E+01_JPRB, 0.10204E+01_JPRB, 0.89587E+00_JPRB, 0.81444E+00_JPRB, 0.76473E+00_JPRB ,&
&  0.72552E+00_JPRB, 0.69155E+00_JPRB, 0.66478E+00_JPRB, 0.64152E+00_JPRB, 0.62963E+00_JPRB, 0.64728E+00_JPRB, 0.69272E+00_JPRB ,&
&  0.74690E+00_JPRB, 0.83124E+00_JPRB, 0.10699E+01_JPRB, 0.14093E+01_JPRB, 0.16258E+01_JPRB, 0.18735E+01_JPRB, 0.23710E+01_JPRB ,&
&  0.31250E+01_JPRB, 0.34441E+01_JPRB, 0.26126E+01_JPRB, 0.19264E+01_JPRB, 0.15525E+01_JPRB, 0.14609E+01_JPRB, 0.14616E+01_JPRB ,&
&  0.13913E+01_JPRB, 0.13018E+01_JPRB, 0.11881E+01_JPRB, 0.10585E+01_JPRB, 0.94663E+00_JPRB, 0.87156E+00_JPRB, 0.81794E+00_JPRB ,&
&  0.77153E+00_JPRB, 0.73274E+00_JPRB, 0.70757E+00_JPRB, 0.69090E+00_JPRB, 0.68016E+00_JPRB, 0.67670E+00_JPRB, 0.68107E+00_JPRB ,&
&  0.69097E+00_JPRB, 0.68905E+00_JPRB, 0.66088E+00_JPRB, 0.63259E+00_JPRB, 0.62804E+00_JPRB, 0.63721E+00_JPRB, 0.64877E+00_JPRB ,&
&  0.66738E+00_JPRB, 0.66820E+00_JPRB, 0.64552E+00_JPRB, 0.63387E+00_JPRB, 0.60678E+00_JPRB, 0.63898E+00_JPRB, 0.62588E+00_JPRB ,&
&  0.61518E+00_JPRB, 0.61133E+00_JPRB, 0.61509E+00_JPRB, 0.60732E+00_JPRB, 0.58447E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.57736E+00_JPRB, 0.55951E+00_JPRB, 0.54597E+00_JPRB, 0.52673E+00_JPRB, 0.51685E+00_JPRB, 0.51470E+00_JPRB, 0.52219E+00_JPRB ,&
&  0.53083E+00_JPRB, 0.53792E+00_JPRB, 0.53304E+00_JPRB, 0.52765E+00_JPRB, 0.52040E+00_JPRB, 0.51539E+00_JPRB, 0.50733E+00_JPRB ,&
&  0.50270E+00_JPRB, 0.49837E+00_JPRB, 0.49086E+00_JPRB, 0.48135E+00_JPRB, 0.46829E+00_JPRB, 0.46189E+00_JPRB, 0.45470E+00_JPRB ,&
&  0.44819E+00_JPRB, 0.44561E+00_JPRB, 0.44723E+00_JPRB, 0.45608E+00_JPRB, 0.46788E+00_JPRB, 0.48261E+00_JPRB, 0.51115E+00_JPRB ,&
&  0.67994E+00_JPRB, 0.82656E+00_JPRB, 0.13025E+01_JPRB, 0.13288E+01_JPRB, 0.12110E+01_JPRB, 0.10829E+01_JPRB, 0.10772E+01_JPRB ,&
&  0.10543E+01_JPRB, 0.99953E+00_JPRB, 0.93788E+00_JPRB, 0.89543E+00_JPRB, 0.85132E+00_JPRB, 0.82071E+00_JPRB, 0.77571E+00_JPRB ,&
&  0.73343E+00_JPRB, 0.68874E+00_JPRB, 0.64143E+00_JPRB, 0.60826E+00_JPRB, 0.58821E+00_JPRB, 0.58488E+00_JPRB, 0.60167E+00_JPRB ,&
&  0.62771E+00_JPRB, 0.66437E+00_JPRB, 0.79777E+00_JPRB, 0.10840E+01_JPRB, 0.15316E+01_JPRB, 0.16367E+01_JPRB, 0.22352E+01_JPRB ,&
&  0.30340E+01_JPRB, 0.29988E+01_JPRB, 0.26615E+01_JPRB, 0.23290E+01_JPRB, 0.19511E+01_JPRB, 0.18020E+01_JPRB, 0.17649E+01_JPRB ,&
&  0.16399E+01_JPRB, 0.14415E+01_JPRB, 0.12408E+01_JPRB, 0.10895E+01_JPRB, 0.98520E+00_JPRB, 0.92023E+00_JPRB, 0.88319E+00_JPRB ,&
&  0.84739E+00_JPRB, 0.80577E+00_JPRB, 0.76498E+00_JPRB, 0.73277E+00_JPRB, 0.71249E+00_JPRB, 0.70341E+00_JPRB, 0.70064E+00_JPRB ,&
&  0.70249E+00_JPRB, 0.70111E+00_JPRB, 0.69038E+00_JPRB, 0.67458E+00_JPRB, 0.68849E+00_JPRB, 0.71784E+00_JPRB, 0.72982E+00_JPRB ,&
&  0.71495E+00_JPRB, 0.69536E+00_JPRB, 0.67007E+00_JPRB, 0.64227E+00_JPRB, 0.61736E+00_JPRB, 0.63948E+00_JPRB, 0.62307E+00_JPRB ,&
&  0.60352E+00_JPRB, 0.60240E+00_JPRB, 0.61735E+00_JPRB, 0.61665E+00_JPRB, 0.60780E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.59649E+00_JPRB, 0.57596E+00_JPRB, 0.55764E+00_JPRB, 0.54063E+00_JPRB, 0.52662E+00_JPRB, 0.52211E+00_JPRB, 0.52450E+00_JPRB ,&
&  0.52868E+00_JPRB, 0.52851E+00_JPRB, 0.52370E+00_JPRB, 0.51544E+00_JPRB, 0.50961E+00_JPRB, 0.50442E+00_JPRB, 0.50023E+00_JPRB ,&
&  0.49342E+00_JPRB, 0.49419E+00_JPRB, 0.49043E+00_JPRB, 0.48339E+00_JPRB, 0.47525E+00_JPRB, 0.46540E+00_JPRB, 0.45603E+00_JPRB ,&
&  0.44982E+00_JPRB, 0.44753E+00_JPRB, 0.44807E+00_JPRB, 0.45255E+00_JPRB, 0.46516E+00_JPRB, 0.47821E+00_JPRB, 0.49616E+00_JPRB ,&
&  0.62368E+00_JPRB, 0.84827E+00_JPRB, 0.10082E+01_JPRB, 0.11110E+01_JPRB, 0.99631E+00_JPRB, 0.89044E+00_JPRB, 0.83954E+00_JPRB ,&
&  0.81793E+00_JPRB, 0.80879E+00_JPRB, 0.77768E+00_JPRB, 0.74276E+00_JPRB, 0.70269E+00_JPRB, 0.68385E+00_JPRB, 0.66196E+00_JPRB ,&
&  0.63695E+00_JPRB, 0.61416E+00_JPRB, 0.59164E+00_JPRB, 0.58162E+00_JPRB, 0.57394E+00_JPRB, 0.56504E+00_JPRB, 0.56069E+00_JPRB ,&
&  0.56610E+00_JPRB, 0.57769E+00_JPRB, 0.62229E+00_JPRB, 0.79393E+00_JPRB, 0.11718E+01_JPRB, 0.14717E+01_JPRB, 0.18590E+01_JPRB ,&
&  0.19703E+01_JPRB, 0.20737E+01_JPRB, 0.21492E+01_JPRB, 0.21214E+01_JPRB, 0.19769E+01_JPRB, 0.18848E+01_JPRB, 0.18735E+01_JPRB ,&
&  0.17630E+01_JPRB, 0.14922E+01_JPRB, 0.12339E+01_JPRB, 0.10758E+01_JPRB, 0.99680E+00_JPRB, 0.95757E+00_JPRB, 0.92495E+00_JPRB ,&
&  0.90223E+00_JPRB, 0.87936E+00_JPRB, 0.84635E+00_JPRB, 0.80718E+00_JPRB, 0.77110E+00_JPRB, 0.74087E+00_JPRB, 0.72203E+00_JPRB ,&
&  0.70678E+00_JPRB, 0.69898E+00_JPRB, 0.70461E+00_JPRB, 0.71976E+00_JPRB, 0.73629E+00_JPRB, 0.75169E+00_JPRB, 0.76129E+00_JPRB ,&
&  0.76838E+00_JPRB, 0.74764E+00_JPRB, 0.69571E+00_JPRB, 0.65985E+00_JPRB, 0.62801E+00_JPRB, 0.63192E+00_JPRB, 0.60956E+00_JPRB ,&
&  0.59232E+00_JPRB, 0.58740E+00_JPRB, 0.59926E+00_JPRB, 0.60726E+00_JPRB, 0.61297E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.67389E+00_JPRB, 0.62556E+00_JPRB, 0.58530E+00_JPRB, 0.55422E+00_JPRB, 0.53725E+00_JPRB, 0.52861E+00_JPRB, 0.52230E+00_JPRB ,&
&  0.51875E+00_JPRB, 0.51422E+00_JPRB, 0.50488E+00_JPRB, 0.49738E+00_JPRB, 0.49208E+00_JPRB, 0.48941E+00_JPRB, 0.48706E+00_JPRB ,&
&  0.47780E+00_JPRB, 0.47622E+00_JPRB, 0.47631E+00_JPRB, 0.47182E+00_JPRB, 0.46562E+00_JPRB, 0.45890E+00_JPRB, 0.45017E+00_JPRB ,&
&  0.44409E+00_JPRB, 0.44209E+00_JPRB, 0.44414E+00_JPRB, 0.44825E+00_JPRB, 0.45272E+00_JPRB, 0.46454E+00_JPRB, 0.48909E+00_JPRB ,&
&  0.59677E+00_JPRB, 0.70350E+00_JPRB, 0.79694E+00_JPRB, 0.84299E+00_JPRB, 0.75542E+00_JPRB, 0.71741E+00_JPRB, 0.70243E+00_JPRB ,&
&  0.70582E+00_JPRB, 0.71165E+00_JPRB, 0.68253E+00_JPRB, 0.63602E+00_JPRB, 0.59448E+00_JPRB, 0.57208E+00_JPRB, 0.55775E+00_JPRB ,&
&  0.54457E+00_JPRB, 0.53472E+00_JPRB, 0.52568E+00_JPRB, 0.52648E+00_JPRB, 0.52758E+00_JPRB, 0.52487E+00_JPRB, 0.52278E+00_JPRB ,&
&  0.52312E+00_JPRB, 0.52728E+00_JPRB, 0.53931E+00_JPRB, 0.60276E+00_JPRB, 0.78724E+00_JPRB, 0.99573E+00_JPRB, 0.12057E+01_JPRB ,&
&  0.13081E+01_JPRB, 0.14105E+01_JPRB, 0.15363E+01_JPRB, 0.16314E+01_JPRB, 0.16444E+01_JPRB, 0.16421E+01_JPRB, 0.16374E+01_JPRB ,&
&  0.15918E+01_JPRB, 0.14302E+01_JPRB, 0.12178E+01_JPRB, 0.10621E+01_JPRB, 0.97740E+00_JPRB, 0.93396E+00_JPRB, 0.90690E+00_JPRB ,&
&  0.88420E+00_JPRB, 0.86415E+00_JPRB, 0.83633E+00_JPRB, 0.79732E+00_JPRB, 0.75437E+00_JPRB, 0.72359E+00_JPRB, 0.70075E+00_JPRB ,&
&  0.68635E+00_JPRB, 0.68858E+00_JPRB, 0.70516E+00_JPRB, 0.73352E+00_JPRB, 0.75317E+00_JPRB, 0.76271E+00_JPRB, 0.76080E+00_JPRB ,&
&  0.75601E+00_JPRB, 0.75649E+00_JPRB, 0.69894E+00_JPRB, 0.65995E+00_JPRB, 0.67771E+00_JPRB, 0.63466E+00_JPRB, 0.59793E+00_JPRB ,&
&  0.58256E+00_JPRB, 0.57056E+00_JPRB, 0.56932E+00_JPRB, 0.58602E+00_JPRB, 0.69855E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.62138E+00_JPRB, 0.60536E+00_JPRB, 0.57881E+00_JPRB, 0.55362E+00_JPRB, 0.53769E+00_JPRB, 0.52846E+00_JPRB, 0.52298E+00_JPRB ,&
&  0.51659E+00_JPRB, 0.50822E+00_JPRB, 0.49845E+00_JPRB, 0.48584E+00_JPRB, 0.47781E+00_JPRB, 0.47386E+00_JPRB, 0.47104E+00_JPRB ,&
&  0.46630E+00_JPRB, 0.45691E+00_JPRB, 0.45384E+00_JPRB, 0.45102E+00_JPRB, 0.44962E+00_JPRB, 0.44796E+00_JPRB, 0.44320E+00_JPRB ,&
&  0.43721E+00_JPRB, 0.43444E+00_JPRB, 0.43806E+00_JPRB, 0.44028E+00_JPRB, 0.44277E+00_JPRB, 0.44856E+00_JPRB, 0.46501E+00_JPRB ,&
&  0.53985E+00_JPRB, 0.57459E+00_JPRB, 0.64706E+00_JPRB, 0.70626E+00_JPRB, 0.68215E+00_JPRB, 0.65446E+00_JPRB, 0.63967E+00_JPRB ,&
&  0.61932E+00_JPRB, 0.60510E+00_JPRB, 0.57780E+00_JPRB, 0.54235E+00_JPRB, 0.51315E+00_JPRB, 0.50515E+00_JPRB, 0.49908E+00_JPRB ,&
&  0.49409E+00_JPRB, 0.49166E+00_JPRB, 0.49033E+00_JPRB, 0.49008E+00_JPRB, 0.49038E+00_JPRB, 0.48963E+00_JPRB, 0.48875E+00_JPRB ,&
&  0.48996E+00_JPRB, 0.49411E+00_JPRB, 0.49977E+00_JPRB, 0.51585E+00_JPRB, 0.58502E+00_JPRB, 0.71689E+00_JPRB, 0.85961E+00_JPRB ,&
&  0.94909E+00_JPRB, 0.10256E+01_JPRB, 0.11376E+01_JPRB, 0.12322E+01_JPRB, 0.12917E+01_JPRB, 0.13145E+01_JPRB, 0.13090E+01_JPRB ,&
&  0.12936E+01_JPRB, 0.12374E+01_JPRB, 0.11159E+01_JPRB, 0.98621E+00_JPRB, 0.90159E+00_JPRB, 0.84813E+00_JPRB, 0.81090E+00_JPRB ,&
&  0.78371E+00_JPRB, 0.75114E+00_JPRB, 0.71859E+00_JPRB, 0.69052E+00_JPRB, 0.67191E+00_JPRB, 0.65500E+00_JPRB, 0.64232E+00_JPRB ,&
&  0.64718E+00_JPRB, 0.66933E+00_JPRB, 0.70057E+00_JPRB, 0.72033E+00_JPRB, 0.72534E+00_JPRB, 0.72572E+00_JPRB, 0.71860E+00_JPRB ,&
&  0.71326E+00_JPRB, 0.71261E+00_JPRB, 0.70172E+00_JPRB, 0.69027E+00_JPRB, 0.65980E+00_JPRB, 0.63270E+00_JPRB, 0.59247E+00_JPRB ,&
&  0.57078E+00_JPRB, 0.55797E+00_JPRB, 0.54551E+00_JPRB, 0.55815E+00_JPRB, 0.60825E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.55969E+00_JPRB, 0.55917E+00_JPRB, 0.55453E+00_JPRB, 0.54506E+00_JPRB, 0.53930E+00_JPRB, 0.53347E+00_JPRB, 0.52437E+00_JPRB ,&
&  0.51424E+00_JPRB, 0.50525E+00_JPRB, 0.49662E+00_JPRB, 0.48552E+00_JPRB, 0.47336E+00_JPRB, 0.46881E+00_JPRB, 0.46745E+00_JPRB ,&
&  0.46357E+00_JPRB, 0.45591E+00_JPRB, 0.44864E+00_JPRB, 0.44685E+00_JPRB, 0.44604E+00_JPRB, 0.44545E+00_JPRB, 0.44229E+00_JPRB ,&
&  0.43712E+00_JPRB, 0.43642E+00_JPRB, 0.43995E+00_JPRB, 0.44106E+00_JPRB, 0.43738E+00_JPRB, 0.43930E+00_JPRB, 0.45166E+00_JPRB ,&
&  0.51249E+00_JPRB, 0.47029E+00_JPRB, 0.51728E+00_JPRB, 0.60283E+00_JPRB, 0.58600E+00_JPRB, 0.57814E+00_JPRB, 0.55492E+00_JPRB ,&
&  0.53095E+00_JPRB, 0.51885E+00_JPRB, 0.50887E+00_JPRB, 0.49315E+00_JPRB, 0.47675E+00_JPRB, 0.46767E+00_JPRB, 0.46698E+00_JPRB ,&
&  0.47090E+00_JPRB, 0.47315E+00_JPRB, 0.47406E+00_JPRB, 0.46976E+00_JPRB, 0.46720E+00_JPRB, 0.46548E+00_JPRB, 0.46484E+00_JPRB ,&
&  0.46461E+00_JPRB, 0.46757E+00_JPRB, 0.47229E+00_JPRB, 0.47787E+00_JPRB, 0.49814E+00_JPRB, 0.56305E+00_JPRB, 0.66888E+00_JPRB ,&
&  0.76688E+00_JPRB, 0.84986E+00_JPRB, 0.92479E+00_JPRB, 0.10000E+01_JPRB, 0.10550E+01_JPRB, 0.11022E+01_JPRB, 0.11249E+01_JPRB ,&
&  0.11049E+01_JPRB, 0.10512E+01_JPRB, 0.98055E+00_JPRB, 0.89936E+00_JPRB, 0.82539E+00_JPRB, 0.76666E+00_JPRB, 0.72156E+00_JPRB ,&
&  0.68748E+00_JPRB, 0.66506E+00_JPRB, 0.64756E+00_JPRB, 0.62883E+00_JPRB, 0.61839E+00_JPRB, 0.61781E+00_JPRB, 0.62588E+00_JPRB ,&
&  0.64174E+00_JPRB, 0.65916E+00_JPRB, 0.67012E+00_JPRB, 0.67218E+00_JPRB, 0.66588E+00_JPRB, 0.65949E+00_JPRB, 0.65347E+00_JPRB ,&
&  0.65647E+00_JPRB, 0.66563E+00_JPRB, 0.67479E+00_JPRB, 0.65156E+00_JPRB, 0.65665E+00_JPRB, 0.64459E+00_JPRB, 0.61411E+00_JPRB ,&
&  0.58329E+00_JPRB, 0.56067E+00_JPRB, 0.54731E+00_JPRB, 0.48516E+00_JPRB, 0.55595E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.52808E+00_JPRB, 0.53010E+00_JPRB, 0.53033E+00_JPRB, 0.52947E+00_JPRB, 0.52738E+00_JPRB, 0.52165E+00_JPRB, 0.51197E+00_JPRB ,&
&  0.50193E+00_JPRB, 0.49211E+00_JPRB, 0.48496E+00_JPRB, 0.47761E+00_JPRB, 0.46655E+00_JPRB, 0.45934E+00_JPRB, 0.45796E+00_JPRB ,&
&  0.45610E+00_JPRB, 0.45094E+00_JPRB, 0.44503E+00_JPRB, 0.44410E+00_JPRB, 0.44215E+00_JPRB, 0.43930E+00_JPRB, 0.43530E+00_JPRB ,&
&  0.43118E+00_JPRB, 0.42993E+00_JPRB, 0.43057E+00_JPRB, 0.43186E+00_JPRB, 0.42893E+00_JPRB, 0.43047E+00_JPRB, 0.44262E+00_JPRB ,&
&  0.49036E+00_JPRB, 0.42737E+00_JPRB, 0.47166E+00_JPRB, 0.52025E+00_JPRB, 0.49972E+00_JPRB, 0.48263E+00_JPRB, 0.46786E+00_JPRB ,&
&  0.45721E+00_JPRB, 0.45315E+00_JPRB, 0.45115E+00_JPRB, 0.44721E+00_JPRB, 0.44310E+00_JPRB, 0.43818E+00_JPRB, 0.43660E+00_JPRB ,&
&  0.43734E+00_JPRB, 0.44243E+00_JPRB, 0.44539E+00_JPRB, 0.44562E+00_JPRB, 0.44253E+00_JPRB, 0.43928E+00_JPRB, 0.43657E+00_JPRB ,&
&  0.43555E+00_JPRB, 0.43697E+00_JPRB, 0.43965E+00_JPRB, 0.44441E+00_JPRB, 0.45513E+00_JPRB, 0.48350E+00_JPRB, 0.54245E+00_JPRB ,&
&  0.62081E+00_JPRB, 0.68715E+00_JPRB, 0.74833E+00_JPRB, 0.82090E+00_JPRB, 0.89071E+00_JPRB, 0.93904E+00_JPRB, 0.95246E+00_JPRB ,&
&  0.93195E+00_JPRB, 0.88352E+00_JPRB, 0.83996E+00_JPRB, 0.79540E+00_JPRB, 0.75214E+00_JPRB, 0.71534E+00_JPRB, 0.68911E+00_JPRB ,&
&  0.67176E+00_JPRB, 0.66046E+00_JPRB, 0.65429E+00_JPRB, 0.65041E+00_JPRB, 0.65134E+00_JPRB, 0.65394E+00_JPRB, 0.65826E+00_JPRB ,&
&  0.65586E+00_JPRB, 0.64939E+00_JPRB, 0.63177E+00_JPRB, 0.60715E+00_JPRB, 0.59018E+00_JPRB, 0.58167E+00_JPRB, 0.57838E+00_JPRB ,&
&  0.58125E+00_JPRB, 0.59092E+00_JPRB, 0.60418E+00_JPRB, 0.62519E+00_JPRB, 0.62975E+00_JPRB, 0.63277E+00_JPRB, 0.62150E+00_JPRB ,&
&  0.59568E+00_JPRB, 0.56827E+00_JPRB, 0.46619E+00_JPRB, 0.51578E+00_JPRB, 0.52960E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.53298E+00_JPRB, 0.52760E+00_JPRB, 0.52270E+00_JPRB, 0.51775E+00_JPRB, 0.51137E+00_JPRB, 0.50323E+00_JPRB, 0.49359E+00_JPRB ,&
&  0.48463E+00_JPRB, 0.47628E+00_JPRB, 0.46794E+00_JPRB, 0.46073E+00_JPRB, 0.45400E+00_JPRB, 0.45086E+00_JPRB, 0.45004E+00_JPRB ,&
&  0.45033E+00_JPRB, 0.44950E+00_JPRB, 0.44795E+00_JPRB, 0.44795E+00_JPRB, 0.44751E+00_JPRB, 0.44502E+00_JPRB, 0.44144E+00_JPRB ,&
&  0.43569E+00_JPRB, 0.43078E+00_JPRB, 0.42904E+00_JPRB, 0.43150E+00_JPRB, 0.43059E+00_JPRB, 0.43260E+00_JPRB, 0.44052E+00_JPRB ,&
&  0.42547E+00_JPRB, 0.39888E+00_JPRB, 0.43165E+00_JPRB, 0.46826E+00_JPRB, 0.45365E+00_JPRB, 0.43986E+00_JPRB, 0.43420E+00_JPRB ,&
&  0.42862E+00_JPRB, 0.42580E+00_JPRB, 0.42374E+00_JPRB, 0.42316E+00_JPRB, 0.42455E+00_JPRB, 0.42429E+00_JPRB, 0.42271E+00_JPRB ,&
&  0.42180E+00_JPRB, 0.42454E+00_JPRB, 0.42755E+00_JPRB, 0.42934E+00_JPRB, 0.42928E+00_JPRB, 0.42807E+00_JPRB, 0.42439E+00_JPRB ,&
&  0.42098E+00_JPRB, 0.41906E+00_JPRB, 0.41935E+00_JPRB, 0.42327E+00_JPRB, 0.43042E+00_JPRB, 0.44468E+00_JPRB, 0.47403E+00_JPRB ,&
&  0.52040E+00_JPRB, 0.57007E+00_JPRB, 0.61585E+00_JPRB, 0.66789E+00_JPRB, 0.73232E+00_JPRB, 0.79273E+00_JPRB, 0.83009E+00_JPRB ,&
&  0.82430E+00_JPRB, 0.78677E+00_JPRB, 0.74097E+00_JPRB, 0.70228E+00_JPRB, 0.67336E+00_JPRB, 0.65494E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64012E+00_JPRB, 0.63801E+00_JPRB, 0.63715E+00_JPRB, 0.63723E+00_JPRB, 0.63679E+00_JPRB, 0.63240E+00_JPRB, 0.62426E+00_JPRB ,&
&  0.60985E+00_JPRB, 0.58751E+00_JPRB, 0.56235E+00_JPRB, 0.54214E+00_JPRB, 0.52826E+00_JPRB, 0.52186E+00_JPRB, 0.52132E+00_JPRB ,&
&  0.52637E+00_JPRB, 0.53869E+00_JPRB, 0.55565E+00_JPRB, 0.58446E+00_JPRB, 0.60495E+00_JPRB, 0.61864E+00_JPRB, 0.62295E+00_JPRB ,&
&  0.60964E+00_JPRB, 0.58507E+00_JPRB, 0.56120E+00_JPRB, 0.54767E+00_JPRB, 0.54192E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.53082E+00_JPRB, 0.51668E+00_JPRB, 0.50282E+00_JPRB, 0.49340E+00_JPRB, 0.48628E+00_JPRB, 0.47933E+00_JPRB, 0.47147E+00_JPRB ,&
&  0.46218E+00_JPRB, 0.45308E+00_JPRB, 0.44470E+00_JPRB, 0.43690E+00_JPRB, 0.43304E+00_JPRB, 0.43123E+00_JPRB, 0.43174E+00_JPRB ,&
&  0.43439E+00_JPRB, 0.43901E+00_JPRB, 0.44083E+00_JPRB, 0.44047E+00_JPRB, 0.44109E+00_JPRB, 0.44087E+00_JPRB, 0.43903E+00_JPRB ,&
&  0.43475E+00_JPRB, 0.42959E+00_JPRB, 0.42677E+00_JPRB, 0.42750E+00_JPRB, 0.42771E+00_JPRB, 0.42838E+00_JPRB, 0.42846E+00_JPRB ,&
&  0.43121E+00_JPRB, 0.39609E+00_JPRB, 0.42153E+00_JPRB, 0.44034E+00_JPRB, 0.43511E+00_JPRB, 0.43033E+00_JPRB, 0.42472E+00_JPRB ,&
&  0.41753E+00_JPRB, 0.41371E+00_JPRB, 0.41187E+00_JPRB, 0.41348E+00_JPRB, 0.41665E+00_JPRB, 0.41809E+00_JPRB, 0.41826E+00_JPRB ,&
&  0.41543E+00_JPRB, 0.41550E+00_JPRB, 0.41506E+00_JPRB, 0.41530E+00_JPRB, 0.41629E+00_JPRB, 0.41744E+00_JPRB, 0.41760E+00_JPRB ,&
&  0.41623E+00_JPRB, 0.41323E+00_JPRB, 0.41028E+00_JPRB, 0.41013E+00_JPRB, 0.41186E+00_JPRB, 0.41653E+00_JPRB, 0.42717E+00_JPRB ,&
&  0.44690E+00_JPRB, 0.47229E+00_JPRB, 0.50277E+00_JPRB, 0.54135E+00_JPRB, 0.59437E+00_JPRB, 0.64850E+00_JPRB, 0.69080E+00_JPRB ,&
&  0.70481E+00_JPRB, 0.69541E+00_JPRB, 0.67283E+00_JPRB, 0.64438E+00_JPRB, 0.61818E+00_JPRB, 0.59350E+00_JPRB, 0.57190E+00_JPRB ,&
&  0.55605E+00_JPRB, 0.54383E+00_JPRB, 0.53508E+00_JPRB, 0.52974E+00_JPRB, 0.52455E+00_JPRB, 0.51696E+00_JPRB, 0.50936E+00_JPRB ,&
&  0.50099E+00_JPRB, 0.49327E+00_JPRB, 0.48724E+00_JPRB, 0.48193E+00_JPRB, 0.47646E+00_JPRB, 0.47199E+00_JPRB, 0.47117E+00_JPRB ,&
&  0.47543E+00_JPRB, 0.48426E+00_JPRB, 0.49973E+00_JPRB, 0.52137E+00_JPRB, 0.54601E+00_JPRB, 0.56559E+00_JPRB, 0.57695E+00_JPRB ,&
&  0.57817E+00_JPRB, 0.57397E+00_JPRB, 0.56339E+00_JPRB, 0.55256E+00_JPRB, 0.54325E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.50982E+00_JPRB, 0.49832E+00_JPRB, 0.48672E+00_JPRB, 0.47746E+00_JPRB, 0.47010E+00_JPRB, 0.46211E+00_JPRB, 0.45312E+00_JPRB ,&
&  0.44573E+00_JPRB, 0.43777E+00_JPRB, 0.43195E+00_JPRB, 0.42701E+00_JPRB, 0.42338E+00_JPRB, 0.42225E+00_JPRB, 0.42307E+00_JPRB ,&
&  0.42569E+00_JPRB, 0.43122E+00_JPRB, 0.43399E+00_JPRB, 0.43248E+00_JPRB, 0.43424E+00_JPRB, 0.43656E+00_JPRB, 0.43704E+00_JPRB ,&
&  0.43462E+00_JPRB, 0.43140E+00_JPRB, 0.42789E+00_JPRB, 0.42577E+00_JPRB, 0.42564E+00_JPRB, 0.42655E+00_JPRB, 0.42333E+00_JPRB ,&
&  0.41745E+00_JPRB, 0.41002E+00_JPRB, 0.42086E+00_JPRB, 0.42773E+00_JPRB, 0.42785E+00_JPRB, 0.42421E+00_JPRB, 0.41943E+00_JPRB ,&
&  0.41285E+00_JPRB, 0.40927E+00_JPRB, 0.41007E+00_JPRB, 0.41241E+00_JPRB, 0.41502E+00_JPRB, 0.41623E+00_JPRB, 0.41740E+00_JPRB ,&
&  0.41426E+00_JPRB, 0.41266E+00_JPRB, 0.41095E+00_JPRB, 0.40998E+00_JPRB, 0.41095E+00_JPRB, 0.41225E+00_JPRB, 0.41310E+00_JPRB ,&
&  0.41230E+00_JPRB, 0.41310E+00_JPRB, 0.41409E+00_JPRB, 0.41343E+00_JPRB, 0.41499E+00_JPRB, 0.41738E+00_JPRB, 0.42111E+00_JPRB ,&
&  0.42788E+00_JPRB, 0.43856E+00_JPRB, 0.45197E+00_JPRB, 0.47196E+00_JPRB, 0.50457E+00_JPRB, 0.54185E+00_JPRB, 0.57176E+00_JPRB ,&
&  0.58732E+00_JPRB, 0.58934E+00_JPRB, 0.58246E+00_JPRB, 0.56913E+00_JPRB, 0.55556E+00_JPRB, 0.54215E+00_JPRB, 0.52673E+00_JPRB ,&
&  0.51003E+00_JPRB, 0.49711E+00_JPRB, 0.48668E+00_JPRB, 0.47870E+00_JPRB, 0.47156E+00_JPRB, 0.46565E+00_JPRB, 0.46210E+00_JPRB ,&
&  0.46082E+00_JPRB, 0.46009E+00_JPRB, 0.45794E+00_JPRB, 0.45534E+00_JPRB, 0.45389E+00_JPRB, 0.45098E+00_JPRB, 0.44938E+00_JPRB ,&
&  0.45124E+00_JPRB, 0.45679E+00_JPRB, 0.46638E+00_JPRB, 0.48164E+00_JPRB, 0.49777E+00_JPRB, 0.51176E+00_JPRB, 0.52282E+00_JPRB ,&
&  0.53070E+00_JPRB, 0.53425E+00_JPRB, 0.53274E+00_JPRB, 0.52821E+00_JPRB, 0.52066E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.46790E+00_JPRB, 0.46222E+00_JPRB, 0.45534E+00_JPRB, 0.44844E+00_JPRB, 0.44188E+00_JPRB, 0.43524E+00_JPRB, 0.42919E+00_JPRB ,&
&  0.42401E+00_JPRB, 0.42092E+00_JPRB, 0.41904E+00_JPRB, 0.41760E+00_JPRB, 0.41431E+00_JPRB, 0.41288E+00_JPRB, 0.41369E+00_JPRB ,&
&  0.41623E+00_JPRB, 0.41878E+00_JPRB, 0.42050E+00_JPRB, 0.41995E+00_JPRB, 0.42223E+00_JPRB, 0.42441E+00_JPRB, 0.42459E+00_JPRB ,&
&  0.42370E+00_JPRB, 0.42281E+00_JPRB, 0.42063E+00_JPRB, 0.41788E+00_JPRB, 0.41774E+00_JPRB, 0.41790E+00_JPRB, 0.41393E+00_JPRB ,&
&  0.40647E+00_JPRB, 0.40781E+00_JPRB, 0.40007E+00_JPRB, 0.41338E+00_JPRB, 0.41643E+00_JPRB, 0.41128E+00_JPRB, 0.40502E+00_JPRB ,&
&  0.40302E+00_JPRB, 0.40419E+00_JPRB, 0.40770E+00_JPRB, 0.41013E+00_JPRB, 0.41103E+00_JPRB, 0.41130E+00_JPRB, 0.41123E+00_JPRB ,&
&  0.40382E+00_JPRB, 0.39959E+00_JPRB, 0.39940E+00_JPRB, 0.40044E+00_JPRB, 0.40066E+00_JPRB, 0.40143E+00_JPRB, 0.40338E+00_JPRB ,&
&  0.40491E+00_JPRB, 0.40594E+00_JPRB, 0.40710E+00_JPRB, 0.40972E+00_JPRB, 0.41156E+00_JPRB, 0.41246E+00_JPRB, 0.41438E+00_JPRB ,&
&  0.41760E+00_JPRB, 0.42238E+00_JPRB, 0.42794E+00_JPRB, 0.43478E+00_JPRB, 0.44929E+00_JPRB, 0.47075E+00_JPRB, 0.48981E+00_JPRB ,&
&  0.50032E+00_JPRB, 0.50393E+00_JPRB, 0.50309E+00_JPRB, 0.49703E+00_JPRB, 0.48740E+00_JPRB, 0.47874E+00_JPRB, 0.47084E+00_JPRB ,&
&  0.46060E+00_JPRB, 0.45340E+00_JPRB, 0.44704E+00_JPRB, 0.43935E+00_JPRB, 0.43252E+00_JPRB, 0.42781E+00_JPRB, 0.42520E+00_JPRB ,&
&  0.42476E+00_JPRB, 0.42628E+00_JPRB, 0.42658E+00_JPRB, 0.42754E+00_JPRB, 0.43025E+00_JPRB, 0.43348E+00_JPRB, 0.43343E+00_JPRB ,&
&  0.43217E+00_JPRB, 0.43198E+00_JPRB, 0.43430E+00_JPRB, 0.43935E+00_JPRB, 0.44516E+00_JPRB, 0.45250E+00_JPRB, 0.46045E+00_JPRB ,&
&  0.46774E+00_JPRB, 0.47280E+00_JPRB, 0.47487E+00_JPRB, 0.47529E+00_JPRB, 0.47259E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.42702E+00_JPRB, 0.42521E+00_JPRB, 0.42291E+00_JPRB, 0.41937E+00_JPRB, 0.41551E+00_JPRB, 0.41209E+00_JPRB, 0.40935E+00_JPRB ,&
&  0.40762E+00_JPRB, 0.40722E+00_JPRB, 0.40778E+00_JPRB, 0.40801E+00_JPRB, 0.40791E+00_JPRB, 0.40881E+00_JPRB, 0.40924E+00_JPRB ,&
&  0.41086E+00_JPRB, 0.41123E+00_JPRB, 0.40896E+00_JPRB, 0.40845E+00_JPRB, 0.41095E+00_JPRB, 0.41567E+00_JPRB, 0.41742E+00_JPRB ,&
&  0.41505E+00_JPRB, 0.41471E+00_JPRB, 0.41439E+00_JPRB, 0.41218E+00_JPRB, 0.41050E+00_JPRB, 0.40933E+00_JPRB, 0.40522E+00_JPRB ,&
&  0.39986E+00_JPRB, 0.39998E+00_JPRB, 0.40161E+00_JPRB, 0.40491E+00_JPRB, 0.40473E+00_JPRB, 0.40287E+00_JPRB, 0.40332E+00_JPRB ,&
&  0.40170E+00_JPRB, 0.40167E+00_JPRB, 0.40168E+00_JPRB, 0.40237E+00_JPRB, 0.40078E+00_JPRB, 0.39824E+00_JPRB, 0.39346E+00_JPRB ,&
&  0.38428E+00_JPRB, 0.38138E+00_JPRB, 0.38353E+00_JPRB, 0.38791E+00_JPRB, 0.38744E+00_JPRB, 0.38816E+00_JPRB, 0.39125E+00_JPRB ,&
&  0.39516E+00_JPRB, 0.39801E+00_JPRB, 0.39979E+00_JPRB, 0.40098E+00_JPRB, 0.40265E+00_JPRB, 0.40423E+00_JPRB, 0.40511E+00_JPRB ,&
&  0.40625E+00_JPRB, 0.40827E+00_JPRB, 0.41120E+00_JPRB, 0.41515E+00_JPRB, 0.42052E+00_JPRB, 0.42942E+00_JPRB, 0.43875E+00_JPRB ,&
&  0.44448E+00_JPRB, 0.44553E+00_JPRB, 0.44443E+00_JPRB, 0.44041E+00_JPRB, 0.43395E+00_JPRB, 0.42794E+00_JPRB, 0.42256E+00_JPRB ,&
&  0.41493E+00_JPRB, 0.40821E+00_JPRB, 0.40280E+00_JPRB, 0.39984E+00_JPRB, 0.39829E+00_JPRB, 0.39806E+00_JPRB, 0.39911E+00_JPRB ,&
&  0.40014E+00_JPRB, 0.40265E+00_JPRB, 0.40865E+00_JPRB, 0.41229E+00_JPRB, 0.41390E+00_JPRB, 0.41693E+00_JPRB, 0.41855E+00_JPRB ,&
&  0.41704E+00_JPRB, 0.41576E+00_JPRB, 0.41501E+00_JPRB, 0.41735E+00_JPRB, 0.41914E+00_JPRB, 0.41929E+00_JPRB, 0.42000E+00_JPRB ,&
&  0.42316E+00_JPRB, 0.42667E+00_JPRB, 0.42767E+00_JPRB, 0.42797E+00_JPRB, 0.42830E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.39268E+00_JPRB, 0.39279E+00_JPRB, 0.39222E+00_JPRB, 0.39118E+00_JPRB, 0.39016E+00_JPRB, 0.38936E+00_JPRB, 0.38955E+00_JPRB ,&
&  0.38988E+00_JPRB, 0.39069E+00_JPRB, 0.39174E+00_JPRB, 0.39261E+00_JPRB, 0.39440E+00_JPRB, 0.39645E+00_JPRB, 0.39814E+00_JPRB ,&
&  0.39992E+00_JPRB, 0.39938E+00_JPRB, 0.39669E+00_JPRB, 0.39402E+00_JPRB, 0.39473E+00_JPRB, 0.40009E+00_JPRB, 0.40600E+00_JPRB ,&
&  0.40737E+00_JPRB, 0.40828E+00_JPRB, 0.40858E+00_JPRB, 0.40726E+00_JPRB, 0.40548E+00_JPRB, 0.40481E+00_JPRB, 0.40102E+00_JPRB ,&
&  0.39619E+00_JPRB, 0.39459E+00_JPRB, 0.39404E+00_JPRB, 0.39153E+00_JPRB, 0.39134E+00_JPRB, 0.39397E+00_JPRB, 0.39331E+00_JPRB ,&
&  0.39265E+00_JPRB, 0.39110E+00_JPRB, 0.38812E+00_JPRB, 0.38538E+00_JPRB, 0.38205E+00_JPRB, 0.37571E+00_JPRB, 0.36837E+00_JPRB ,&
&  0.36257E+00_JPRB, 0.36274E+00_JPRB, 0.36813E+00_JPRB, 0.37375E+00_JPRB, 0.37269E+00_JPRB, 0.37161E+00_JPRB, 0.37310E+00_JPRB ,&
&  0.37669E+00_JPRB, 0.38125E+00_JPRB, 0.38418E+00_JPRB, 0.38344E+00_JPRB, 0.38236E+00_JPRB, 0.38426E+00_JPRB, 0.38695E+00_JPRB ,&
&  0.38821E+00_JPRB, 0.38963E+00_JPRB, 0.39071E+00_JPRB, 0.39252E+00_JPRB, 0.39406E+00_JPRB, 0.39511E+00_JPRB, 0.39786E+00_JPRB ,&
&  0.39895E+00_JPRB, 0.39811E+00_JPRB, 0.39476E+00_JPRB, 0.38933E+00_JPRB, 0.38434E+00_JPRB, 0.38018E+00_JPRB, 0.37853E+00_JPRB ,&
&  0.37646E+00_JPRB, 0.37675E+00_JPRB, 0.37627E+00_JPRB, 0.37352E+00_JPRB, 0.37025E+00_JPRB, 0.36998E+00_JPRB, 0.37340E+00_JPRB ,&
&  0.37835E+00_JPRB, 0.38209E+00_JPRB, 0.38791E+00_JPRB, 0.39052E+00_JPRB, 0.39226E+00_JPRB, 0.39470E+00_JPRB, 0.39547E+00_JPRB ,&
&  0.39299E+00_JPRB, 0.39096E+00_JPRB, 0.39059E+00_JPRB, 0.39357E+00_JPRB, 0.39446E+00_JPRB, 0.39336E+00_JPRB, 0.39033E+00_JPRB ,&
&  0.38923E+00_JPRB, 0.39083E+00_JPRB, 0.39133E+00_JPRB, 0.39090E+00_JPRB, 0.39140E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.36899E+00_JPRB, 0.37112E+00_JPRB, 0.37335E+00_JPRB, 0.37538E+00_JPRB, 0.37670E+00_JPRB, 0.37771E+00_JPRB, 0.37693E+00_JPRB ,&
&  0.37686E+00_JPRB, 0.37858E+00_JPRB, 0.38076E+00_JPRB, 0.38255E+00_JPRB, 0.38362E+00_JPRB, 0.38565E+00_JPRB, 0.38772E+00_JPRB ,&
&  0.38912E+00_JPRB, 0.38832E+00_JPRB, 0.38489E+00_JPRB, 0.38323E+00_JPRB, 0.38209E+00_JPRB, 0.38711E+00_JPRB, 0.39564E+00_JPRB ,&
&  0.39933E+00_JPRB, 0.40156E+00_JPRB, 0.40428E+00_JPRB, 0.40470E+00_JPRB, 0.40472E+00_JPRB, 0.40424E+00_JPRB, 0.39963E+00_JPRB ,&
&  0.39450E+00_JPRB, 0.38967E+00_JPRB, 0.38663E+00_JPRB, 0.38329E+00_JPRB, 0.38327E+00_JPRB, 0.38116E+00_JPRB, 0.37971E+00_JPRB ,&
&  0.37865E+00_JPRB, 0.37615E+00_JPRB, 0.37292E+00_JPRB, 0.36751E+00_JPRB, 0.36082E+00_JPRB, 0.35215E+00_JPRB, 0.34620E+00_JPRB ,&
&  0.34391E+00_JPRB, 0.34761E+00_JPRB, 0.35613E+00_JPRB, 0.35990E+00_JPRB, 0.35651E+00_JPRB, 0.35563E+00_JPRB, 0.35878E+00_JPRB ,&
&  0.36361E+00_JPRB, 0.36901E+00_JPRB, 0.37101E+00_JPRB, 0.37024E+00_JPRB, 0.36829E+00_JPRB, 0.36987E+00_JPRB, 0.37412E+00_JPRB ,&
&  0.37784E+00_JPRB, 0.37977E+00_JPRB, 0.37901E+00_JPRB, 0.37818E+00_JPRB, 0.37600E+00_JPRB, 0.37107E+00_JPRB, 0.36443E+00_JPRB ,&
&  0.35839E+00_JPRB, 0.35456E+00_JPRB, 0.35157E+00_JPRB, 0.34845E+00_JPRB, 0.34712E+00_JPRB, 0.34726E+00_JPRB, 0.34930E+00_JPRB ,&
&  0.35002E+00_JPRB, 0.35175E+00_JPRB, 0.35180E+00_JPRB, 0.34759E+00_JPRB, 0.34428E+00_JPRB, 0.34585E+00_JPRB, 0.34778E+00_JPRB ,&
&  0.35063E+00_JPRB, 0.35341E+00_JPRB, 0.35616E+00_JPRB, 0.35722E+00_JPRB, 0.35875E+00_JPRB, 0.36146E+00_JPRB, 0.36308E+00_JPRB ,&
&  0.36388E+00_JPRB, 0.36445E+00_JPRB, 0.36627E+00_JPRB, 0.36939E+00_JPRB, 0.37232E+00_JPRB, 0.37465E+00_JPRB, 0.37394E+00_JPRB ,&
&  0.37243E+00_JPRB, 0.37061E+00_JPRB, 0.37073E+00_JPRB, 0.37073E+00_JPRB, 0.36905E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.36352E+00_JPRB, 0.36326E+00_JPRB, 0.36641E+00_JPRB, 0.36984E+00_JPRB, 0.37059E+00_JPRB, 0.37026E+00_JPRB, 0.36769E+00_JPRB ,&
&  0.36596E+00_JPRB, 0.36711E+00_JPRB, 0.36890E+00_JPRB, 0.37020E+00_JPRB, 0.37236E+00_JPRB, 0.37532E+00_JPRB, 0.37769E+00_JPRB ,&
&  0.37782E+00_JPRB, 0.37599E+00_JPRB, 0.37286E+00_JPRB, 0.37337E+00_JPRB, 0.37328E+00_JPRB, 0.37538E+00_JPRB, 0.38289E+00_JPRB ,&
&  0.38826E+00_JPRB, 0.39188E+00_JPRB, 0.39491E+00_JPRB, 0.39518E+00_JPRB, 0.39444E+00_JPRB, 0.39573E+00_JPRB, 0.39223E+00_JPRB ,&
&  0.38591E+00_JPRB, 0.38002E+00_JPRB, 0.37464E+00_JPRB, 0.36769E+00_JPRB, 0.37027E+00_JPRB, 0.36775E+00_JPRB, 0.36467E+00_JPRB ,&
&  0.36048E+00_JPRB, 0.35507E+00_JPRB, 0.34874E+00_JPRB, 0.34279E+00_JPRB, 0.33839E+00_JPRB, 0.33433E+00_JPRB, 0.32913E+00_JPRB ,&
&  0.32750E+00_JPRB, 0.32969E+00_JPRB, 0.33536E+00_JPRB, 0.33770E+00_JPRB, 0.33491E+00_JPRB, 0.33565E+00_JPRB, 0.34233E+00_JPRB ,&
&  0.34929E+00_JPRB, 0.35264E+00_JPRB, 0.35256E+00_JPRB, 0.35316E+00_JPRB, 0.35377E+00_JPRB, 0.35515E+00_JPRB, 0.35582E+00_JPRB ,&
&  0.35476E+00_JPRB, 0.35308E+00_JPRB, 0.35053E+00_JPRB, 0.34467E+00_JPRB, 0.33783E+00_JPRB, 0.33210E+00_JPRB, 0.33002E+00_JPRB ,&
&  0.33294E+00_JPRB, 0.33309E+00_JPRB, 0.33357E+00_JPRB, 0.33543E+00_JPRB, 0.33607E+00_JPRB, 0.33535E+00_JPRB, 0.33440E+00_JPRB ,&
&  0.33569E+00_JPRB, 0.33468E+00_JPRB, 0.33590E+00_JPRB, 0.33307E+00_JPRB, 0.32867E+00_JPRB, 0.32447E+00_JPRB, 0.32051E+00_JPRB ,&
&  0.31831E+00_JPRB, 0.31772E+00_JPRB, 0.31985E+00_JPRB, 0.32056E+00_JPRB, 0.32237E+00_JPRB, 0.32455E+00_JPRB, 0.32641E+00_JPRB ,&
&  0.32748E+00_JPRB, 0.32893E+00_JPRB, 0.33169E+00_JPRB, 0.33717E+00_JPRB, 0.34296E+00_JPRB, 0.34807E+00_JPRB, 0.35323E+00_JPRB ,&
&  0.35797E+00_JPRB, 0.36127E+00_JPRB, 0.36704E+00_JPRB, 0.36801E+00_JPRB, 0.36692E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.36234E+00_JPRB, 0.35973E+00_JPRB, 0.35964E+00_JPRB, 0.36033E+00_JPRB, 0.35964E+00_JPRB, 0.35865E+00_JPRB, 0.35590E+00_JPRB ,&
&  0.35348E+00_JPRB, 0.35381E+00_JPRB, 0.35518E+00_JPRB, 0.35774E+00_JPRB, 0.36226E+00_JPRB, 0.36707E+00_JPRB, 0.36658E+00_JPRB ,&
&  0.36356E+00_JPRB, 0.36231E+00_JPRB, 0.36456E+00_JPRB, 0.36664E+00_JPRB, 0.36692E+00_JPRB, 0.36754E+00_JPRB, 0.37121E+00_JPRB ,&
&  0.37581E+00_JPRB, 0.37785E+00_JPRB, 0.38073E+00_JPRB, 0.38023E+00_JPRB, 0.37634E+00_JPRB, 0.37635E+00_JPRB, 0.37445E+00_JPRB ,&
&  0.36950E+00_JPRB, 0.36340E+00_JPRB, 0.35426E+00_JPRB, 0.35510E+00_JPRB, 0.35143E+00_JPRB, 0.34842E+00_JPRB, 0.34390E+00_JPRB ,&
&  0.33891E+00_JPRB, 0.33468E+00_JPRB, 0.32995E+00_JPRB, 0.32613E+00_JPRB, 0.32424E+00_JPRB, 0.32416E+00_JPRB, 0.32204E+00_JPRB ,&
&  0.32078E+00_JPRB, 0.32263E+00_JPRB, 0.32390E+00_JPRB, 0.32073E+00_JPRB, 0.32031E+00_JPRB, 0.32659E+00_JPRB, 0.33181E+00_JPRB ,&
&  0.33389E+00_JPRB, 0.33390E+00_JPRB, 0.33129E+00_JPRB, 0.33062E+00_JPRB, 0.32957E+00_JPRB, 0.32759E+00_JPRB, 0.32419E+00_JPRB ,&
&  0.31908E+00_JPRB, 0.31660E+00_JPRB, 0.31495E+00_JPRB, 0.31159E+00_JPRB, 0.30591E+00_JPRB, 0.28927E+00_JPRB, 0.28011E+00_JPRB ,&
&  0.28039E+00_JPRB, 0.28944E+00_JPRB, 0.30428E+00_JPRB, 0.31123E+00_JPRB, 0.31734E+00_JPRB, 0.32252E+00_JPRB, 0.32325E+00_JPRB ,&
&  0.32423E+00_JPRB, 0.30406E+00_JPRB, 0.29299E+00_JPRB, 0.28412E+00_JPRB, 0.27630E+00_JPRB, 0.27149E+00_JPRB, 0.27011E+00_JPRB ,&
&  0.27309E+00_JPRB, 0.27907E+00_JPRB, 0.28450E+00_JPRB, 0.28720E+00_JPRB, 0.28696E+00_JPRB, 0.28395E+00_JPRB, 0.28068E+00_JPRB ,&
&  0.28056E+00_JPRB, 0.28746E+00_JPRB, 0.30073E+00_JPRB, 0.31364E+00_JPRB, 0.32279E+00_JPRB, 0.33258E+00_JPRB, 0.34081E+00_JPRB ,&
&  0.34656E+00_JPRB, 0.35196E+00_JPRB, 0.35856E+00_JPRB, 0.36102E+00_JPRB, 0.36211E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.35046E+00_JPRB, 0.34672E+00_JPRB, 0.34671E+00_JPRB, 0.34699E+00_JPRB, 0.34576E+00_JPRB, 0.34555E+00_JPRB, 0.34477E+00_JPRB ,&
&  0.34434E+00_JPRB, 0.34239E+00_JPRB, 0.34050E+00_JPRB, 0.34094E+00_JPRB, 0.34308E+00_JPRB, 0.34553E+00_JPRB, 0.34622E+00_JPRB ,&
&  0.34613E+00_JPRB, 0.34759E+00_JPRB, 0.35103E+00_JPRB, 0.35458E+00_JPRB, 0.35524E+00_JPRB, 0.35549E+00_JPRB, 0.35728E+00_JPRB ,&
&  0.35957E+00_JPRB, 0.36036E+00_JPRB, 0.36310E+00_JPRB, 0.36083E+00_JPRB, 0.35622E+00_JPRB, 0.35428E+00_JPRB, 0.35259E+00_JPRB ,&
&  0.34603E+00_JPRB, 0.32263E+00_JPRB, 0.30748E+00_JPRB, 0.31204E+00_JPRB, 0.32764E+00_JPRB, 0.32697E+00_JPRB, 0.32478E+00_JPRB ,&
&  0.32249E+00_JPRB, 0.31940E+00_JPRB, 0.31675E+00_JPRB, 0.31575E+00_JPRB, 0.31588E+00_JPRB, 0.31589E+00_JPRB, 0.31453E+00_JPRB ,&
&  0.31503E+00_JPRB, 0.31995E+00_JPRB, 0.32591E+00_JPRB, 0.32289E+00_JPRB, 0.31831E+00_JPRB, 0.31126E+00_JPRB, 0.30680E+00_JPRB ,&
&  0.30260E+00_JPRB, 0.29833E+00_JPRB, 0.29565E+00_JPRB, 0.29481E+00_JPRB, 0.29455E+00_JPRB, 0.29252E+00_JPRB, 0.28907E+00_JPRB ,&
&  0.28178E+00_JPRB, 0.27266E+00_JPRB, 0.26327E+00_JPRB, 0.25404E+00_JPRB, 0.24730E+00_JPRB, 0.24255E+00_JPRB, 0.23887E+00_JPRB ,&
&  0.24074E+00_JPRB, 0.25236E+00_JPRB, 0.27124E+00_JPRB, 0.29126E+00_JPRB, 0.31052E+00_JPRB, 0.28846E+00_JPRB, 0.27781E+00_JPRB ,&
&  0.26645E+00_JPRB, 0.25669E+00_JPRB, 0.24992E+00_JPRB, 0.24522E+00_JPRB, 0.24187E+00_JPRB, 0.24006E+00_JPRB, 0.24008E+00_JPRB ,&
&  0.24316E+00_JPRB, 0.24733E+00_JPRB, 0.25118E+00_JPRB, 0.25227E+00_JPRB, 0.25168E+00_JPRB, 0.24993E+00_JPRB, 0.24894E+00_JPRB ,&
&  0.25020E+00_JPRB, 0.25575E+00_JPRB, 0.26491E+00_JPRB, 0.27824E+00_JPRB, 0.29026E+00_JPRB, 0.30266E+00_JPRB, 0.31698E+00_JPRB ,&
&  0.33218E+00_JPRB, 0.34625E+00_JPRB, 0.34991E+00_JPRB, 0.35317E+00_JPRB, 0.35341E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.34488E+00_JPRB, 0.34174E+00_JPRB, 0.33981E+00_JPRB, 0.34015E+00_JPRB, 0.34035E+00_JPRB, 0.34291E+00_JPRB, 0.34192E+00_JPRB ,&
&  0.33740E+00_JPRB, 0.33276E+00_JPRB, 0.33138E+00_JPRB, 0.33133E+00_JPRB, 0.33024E+00_JPRB, 0.32898E+00_JPRB, 0.32977E+00_JPRB ,&
&  0.32829E+00_JPRB, 0.33029E+00_JPRB, 0.33326E+00_JPRB, 0.33483E+00_JPRB, 0.33554E+00_JPRB, 0.33567E+00_JPRB, 0.33586E+00_JPRB ,&
&  0.32966E+00_JPRB, 0.32784E+00_JPRB, 0.33347E+00_JPRB, 0.33064E+00_JPRB, 0.32664E+00_JPRB, 0.32315E+00_JPRB, 0.32186E+00_JPRB ,&
&  0.31321E+00_JPRB, 0.30132E+00_JPRB, 0.28869E+00_JPRB, 0.29462E+00_JPRB, 0.31145E+00_JPRB, 0.30868E+00_JPRB, 0.30638E+00_JPRB ,&
&  0.30609E+00_JPRB, 0.30536E+00_JPRB, 0.30621E+00_JPRB, 0.30676E+00_JPRB, 0.30785E+00_JPRB, 0.30813E+00_JPRB, 0.30947E+00_JPRB ,&
&  0.31106E+00_JPRB, 0.31723E+00_JPRB, 0.32065E+00_JPRB, 0.30083E+00_JPRB, 0.28382E+00_JPRB, 0.27084E+00_JPRB, 0.25841E+00_JPRB ,&
&  0.24737E+00_JPRB, 0.24497E+00_JPRB, 0.24136E+00_JPRB, 0.23489E+00_JPRB, 0.23539E+00_JPRB, 0.23606E+00_JPRB, 0.23526E+00_JPRB ,&
&  0.23321E+00_JPRB, 0.23196E+00_JPRB, 0.23056E+00_JPRB, 0.22819E+00_JPRB, 0.22734E+00_JPRB, 0.22867E+00_JPRB, 0.23237E+00_JPRB ,&
&  0.23816E+00_JPRB, 0.24464E+00_JPRB, 0.25928E+00_JPRB, 0.27175E+00_JPRB, 0.26798E+00_JPRB, 0.26061E+00_JPRB, 0.25327E+00_JPRB ,&
&  0.24243E+00_JPRB, 0.23718E+00_JPRB, 0.23198E+00_JPRB, 0.22776E+00_JPRB, 0.22587E+00_JPRB, 0.22618E+00_JPRB, 0.22708E+00_JPRB ,&
&  0.22838E+00_JPRB, 0.23048E+00_JPRB, 0.23133E+00_JPRB, 0.23099E+00_JPRB, 0.23025E+00_JPRB, 0.23084E+00_JPRB, 0.23257E+00_JPRB ,&
&  0.23570E+00_JPRB, 0.24104E+00_JPRB, 0.24748E+00_JPRB, 0.25545E+00_JPRB, 0.26249E+00_JPRB, 0.26907E+00_JPRB, 0.27930E+00_JPRB ,&
&  0.28738E+00_JPRB, 0.30612E+00_JPRB, 0.33219E+00_JPRB, 0.35690E+00_JPRB, 0.35094E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.32631E+00_JPRB, 0.32383E+00_JPRB, 0.32129E+00_JPRB, 0.32101E+00_JPRB, 0.32173E+00_JPRB, 0.32441E+00_JPRB, 0.32189E+00_JPRB ,&
&  0.31720E+00_JPRB, 0.31540E+00_JPRB, 0.31672E+00_JPRB, 0.31662E+00_JPRB, 0.31614E+00_JPRB, 0.30586E+00_JPRB, 0.30005E+00_JPRB ,&
&  0.29319E+00_JPRB, 0.29101E+00_JPRB, 0.28926E+00_JPRB, 0.29014E+00_JPRB, 0.29234E+00_JPRB, 0.29367E+00_JPRB, 0.28943E+00_JPRB ,&
&  0.28497E+00_JPRB, 0.28075E+00_JPRB, 0.27633E+00_JPRB, 0.27443E+00_JPRB, 0.27574E+00_JPRB, 0.27709E+00_JPRB, 0.27719E+00_JPRB ,&
&  0.27791E+00_JPRB, 0.28071E+00_JPRB, 0.28340E+00_JPRB, 0.28702E+00_JPRB, 0.29974E+00_JPRB, 0.29770E+00_JPRB, 0.29639E+00_JPRB ,&
&  0.29674E+00_JPRB, 0.29722E+00_JPRB, 0.29839E+00_JPRB, 0.29920E+00_JPRB, 0.30345E+00_JPRB, 0.30639E+00_JPRB, 0.30855E+00_JPRB ,&
&  0.29492E+00_JPRB, 0.28727E+00_JPRB, 0.27955E+00_JPRB, 0.27025E+00_JPRB, 0.25905E+00_JPRB, 0.25009E+00_JPRB, 0.24184E+00_JPRB ,&
&  0.23486E+00_JPRB, 0.22833E+00_JPRB, 0.22237E+00_JPRB, 0.22005E+00_JPRB, 0.21920E+00_JPRB, 0.21792E+00_JPRB, 0.21782E+00_JPRB ,&
&  0.21840E+00_JPRB, 0.21919E+00_JPRB, 0.21970E+00_JPRB, 0.22127E+00_JPRB, 0.22373E+00_JPRB, 0.22761E+00_JPRB, 0.23175E+00_JPRB ,&
&  0.23792E+00_JPRB, 0.24243E+00_JPRB, 0.24775E+00_JPRB, 0.25226E+00_JPRB, 0.25079E+00_JPRB, 0.24561E+00_JPRB, 0.24007E+00_JPRB ,&
&  0.23332E+00_JPRB, 0.22948E+00_JPRB, 0.22560E+00_JPRB, 0.22282E+00_JPRB, 0.22126E+00_JPRB, 0.22035E+00_JPRB, 0.22066E+00_JPRB ,&
&  0.22080E+00_JPRB, 0.22128E+00_JPRB, 0.22084E+00_JPRB, 0.22111E+00_JPRB, 0.22200E+00_JPRB, 0.22391E+00_JPRB, 0.22641E+00_JPRB ,&
&  0.23026E+00_JPRB, 0.23472E+00_JPRB, 0.24004E+00_JPRB, 0.24588E+00_JPRB, 0.25449E+00_JPRB, 0.26570E+00_JPRB, 0.28153E+00_JPRB ,&
&  0.29801E+00_JPRB, 0.32837E+00_JPRB, 0.34045E+00_JPRB, 0.33249E+00_JPRB, 0.32977E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.31084E+00_JPRB, 0.31216E+00_JPRB, 0.31345E+00_JPRB, 0.31574E+00_JPRB, 0.31851E+00_JPRB, 0.31941E+00_JPRB, 0.31956E+00_JPRB ,&
&  0.31019E+00_JPRB, 0.30264E+00_JPRB, 0.29579E+00_JPRB, 0.28550E+00_JPRB, 0.27731E+00_JPRB, 0.26995E+00_JPRB, 0.26465E+00_JPRB ,&
&  0.25964E+00_JPRB, 0.25717E+00_JPRB, 0.25439E+00_JPRB, 0.25358E+00_JPRB, 0.25334E+00_JPRB, 0.25392E+00_JPRB, 0.25240E+00_JPRB ,&
&  0.25069E+00_JPRB, 0.25185E+00_JPRB, 0.25257E+00_JPRB, 0.25705E+00_JPRB, 0.26367E+00_JPRB, 0.27115E+00_JPRB, 0.27844E+00_JPRB ,&
&  0.28778E+00_JPRB, 0.29170E+00_JPRB, 0.29485E+00_JPRB, 0.29560E+00_JPRB, 0.29434E+00_JPRB, 0.29381E+00_JPRB, 0.29179E+00_JPRB ,&
&  0.29151E+00_JPRB, 0.29112E+00_JPRB, 0.29256E+00_JPRB, 0.29370E+00_JPRB, 0.29223E+00_JPRB, 0.28993E+00_JPRB, 0.28446E+00_JPRB ,&
&  0.27630E+00_JPRB, 0.26971E+00_JPRB, 0.26334E+00_JPRB, 0.25836E+00_JPRB, 0.25132E+00_JPRB, 0.24515E+00_JPRB, 0.23909E+00_JPRB ,&
&  0.23385E+00_JPRB, 0.22992E+00_JPRB, 0.22526E+00_JPRB, 0.22169E+00_JPRB, 0.22000E+00_JPRB, 0.21797E+00_JPRB, 0.21710E+00_JPRB ,&
&  0.21707E+00_JPRB, 0.21749E+00_JPRB, 0.21966E+00_JPRB, 0.22145E+00_JPRB, 0.22424E+00_JPRB, 0.22749E+00_JPRB, 0.22945E+00_JPRB ,&
&  0.23287E+00_JPRB, 0.23583E+00_JPRB, 0.23743E+00_JPRB, 0.23861E+00_JPRB, 0.23763E+00_JPRB, 0.23463E+00_JPRB, 0.23158E+00_JPRB ,&
&  0.22761E+00_JPRB, 0.22484E+00_JPRB, 0.22308E+00_JPRB, 0.22117E+00_JPRB, 0.22023E+00_JPRB, 0.21940E+00_JPRB, 0.21850E+00_JPRB ,&
&  0.21782E+00_JPRB, 0.21753E+00_JPRB, 0.21725E+00_JPRB, 0.21835E+00_JPRB, 0.21953E+00_JPRB, 0.22166E+00_JPRB, 0.22423E+00_JPRB ,&
&  0.22662E+00_JPRB, 0.23030E+00_JPRB, 0.23431E+00_JPRB, 0.23979E+00_JPRB, 0.24968E+00_JPRB, 0.25774E+00_JPRB, 0.27521E+00_JPRB ,&
&  0.28761E+00_JPRB, 0.31278E+00_JPRB, 0.32321E+00_JPRB, 0.31206E+00_JPRB, 0.31096E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.30037E+00_JPRB, 0.30253E+00_JPRB, 0.30347E+00_JPRB, 0.30544E+00_JPRB, 0.30738E+00_JPRB, 0.30802E+00_JPRB, 0.30825E+00_JPRB ,&
&  0.30582E+00_JPRB, 0.29966E+00_JPRB, 0.29493E+00_JPRB, 0.28838E+00_JPRB, 0.28209E+00_JPRB, 0.27539E+00_JPRB, 0.26938E+00_JPRB ,&
&  0.26306E+00_JPRB, 0.25733E+00_JPRB, 0.25151E+00_JPRB, 0.24659E+00_JPRB, 0.24308E+00_JPRB, 0.24016E+00_JPRB, 0.23710E+00_JPRB ,&
&  0.23447E+00_JPRB, 0.23562E+00_JPRB, 0.23828E+00_JPRB, 0.24337E+00_JPRB, 0.25131E+00_JPRB, 0.25861E+00_JPRB, 0.26659E+00_JPRB ,&
&  0.28040E+00_JPRB, 0.28382E+00_JPRB, 0.28380E+00_JPRB, 0.28590E+00_JPRB, 0.28537E+00_JPRB, 0.28592E+00_JPRB, 0.27604E+00_JPRB ,&
&  0.27860E+00_JPRB, 0.28073E+00_JPRB, 0.28854E+00_JPRB, 0.29162E+00_JPRB, 0.28630E+00_JPRB, 0.28198E+00_JPRB, 0.27404E+00_JPRB ,&
&  0.26763E+00_JPRB, 0.26250E+00_JPRB, 0.25910E+00_JPRB, 0.25705E+00_JPRB, 0.25224E+00_JPRB, 0.24868E+00_JPRB, 0.24273E+00_JPRB ,&
&  0.23783E+00_JPRB, 0.23458E+00_JPRB, 0.23053E+00_JPRB, 0.22803E+00_JPRB, 0.22599E+00_JPRB, 0.22401E+00_JPRB, 0.22250E+00_JPRB ,&
&  0.22168E+00_JPRB, 0.22100E+00_JPRB, 0.22169E+00_JPRB, 0.22250E+00_JPRB, 0.22405E+00_JPRB, 0.22530E+00_JPRB, 0.22556E+00_JPRB ,&
&  0.22583E+00_JPRB, 0.22628E+00_JPRB, 0.22643E+00_JPRB, 0.22706E+00_JPRB, 0.22650E+00_JPRB, 0.22555E+00_JPRB, 0.22438E+00_JPRB ,&
&  0.22312E+00_JPRB, 0.22263E+00_JPRB, 0.22177E+00_JPRB, 0.22123E+00_JPRB, 0.21945E+00_JPRB, 0.21827E+00_JPRB, 0.21766E+00_JPRB ,&
&  0.21710E+00_JPRB, 0.21760E+00_JPRB, 0.21781E+00_JPRB, 0.21874E+00_JPRB, 0.22015E+00_JPRB, 0.22143E+00_JPRB, 0.22364E+00_JPRB ,&
&  0.22640E+00_JPRB, 0.22957E+00_JPRB, 0.23497E+00_JPRB, 0.24062E+00_JPRB, 0.24683E+00_JPRB, 0.25451E+00_JPRB, 0.26133E+00_JPRB ,&
&  0.27104E+00_JPRB, 0.28609E+00_JPRB, 0.28826E+00_JPRB, 0.29246E+00_JPRB, 0.29736E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.28522E+00_JPRB, 0.28899E+00_JPRB, 0.29147E+00_JPRB, 0.29310E+00_JPRB, 0.29472E+00_JPRB, 0.29400E+00_JPRB, 0.29239E+00_JPRB ,&
&  0.28813E+00_JPRB, 0.28666E+00_JPRB, 0.28386E+00_JPRB, 0.28082E+00_JPRB, 0.27730E+00_JPRB, 0.27318E+00_JPRB, 0.26914E+00_JPRB ,&
&  0.26458E+00_JPRB, 0.25963E+00_JPRB, 0.25376E+00_JPRB, 0.24729E+00_JPRB, 0.24138E+00_JPRB, 0.23585E+00_JPRB, 0.23271E+00_JPRB ,&
&  0.23128E+00_JPRB, 0.23214E+00_JPRB, 0.23513E+00_JPRB, 0.23875E+00_JPRB, 0.24315E+00_JPRB, 0.24811E+00_JPRB, 0.25428E+00_JPRB ,&
&  0.25952E+00_JPRB, 0.26577E+00_JPRB, 0.26955E+00_JPRB, 0.27113E+00_JPRB, 0.26973E+00_JPRB, 0.26967E+00_JPRB, 0.26873E+00_JPRB ,&
&  0.26747E+00_JPRB, 0.26551E+00_JPRB, 0.26477E+00_JPRB, 0.26493E+00_JPRB, 0.26414E+00_JPRB, 0.26228E+00_JPRB, 0.25961E+00_JPRB ,&
&  0.25709E+00_JPRB, 0.25493E+00_JPRB, 0.25219E+00_JPRB, 0.24984E+00_JPRB, 0.24808E+00_JPRB, 0.24587E+00_JPRB, 0.24291E+00_JPRB ,&
&  0.24026E+00_JPRB, 0.23763E+00_JPRB, 0.23453E+00_JPRB, 0.23192E+00_JPRB, 0.22935E+00_JPRB, 0.22749E+00_JPRB, 0.22582E+00_JPRB ,&
&  0.22512E+00_JPRB, 0.22437E+00_JPRB, 0.22416E+00_JPRB, 0.22405E+00_JPRB, 0.22451E+00_JPRB, 0.22423E+00_JPRB, 0.22380E+00_JPRB ,&
&  0.22327E+00_JPRB, 0.22312E+00_JPRB, 0.22256E+00_JPRB, 0.22235E+00_JPRB, 0.22214E+00_JPRB, 0.22142E+00_JPRB, 0.22106E+00_JPRB ,&
&  0.22110E+00_JPRB, 0.22097E+00_JPRB, 0.22038E+00_JPRB, 0.22030E+00_JPRB, 0.21993E+00_JPRB, 0.21939E+00_JPRB, 0.21969E+00_JPRB ,&
&  0.21954E+00_JPRB, 0.21980E+00_JPRB, 0.22019E+00_JPRB, 0.22077E+00_JPRB, 0.22197E+00_JPRB, 0.22346E+00_JPRB, 0.22543E+00_JPRB ,&
&  0.22843E+00_JPRB, 0.23112E+00_JPRB, 0.23382E+00_JPRB, 0.23695E+00_JPRB, 0.24288E+00_JPRB, 0.24561E+00_JPRB, 0.25051E+00_JPRB ,&
&  0.25502E+00_JPRB, 0.25332E+00_JPRB, 0.27653E+00_JPRB, 0.27770E+00_JPRB, 0.28006E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.24138E+00_JPRB, 0.24650E+00_JPRB, 0.25020E+00_JPRB, 0.25345E+00_JPRB, 0.25048E+00_JPRB, 0.27056E+00_JPRB, 0.26869E+00_JPRB ,&
&  0.26881E+00_JPRB, 0.26288E+00_JPRB, 0.26426E+00_JPRB, 0.26307E+00_JPRB, 0.26073E+00_JPRB, 0.25770E+00_JPRB, 0.25450E+00_JPRB ,&
&  0.25117E+00_JPRB, 0.24721E+00_JPRB, 0.24359E+00_JPRB, 0.24009E+00_JPRB, 0.23720E+00_JPRB, 0.23466E+00_JPRB, 0.23329E+00_JPRB ,&
&  0.23336E+00_JPRB, 0.23491E+00_JPRB, 0.23757E+00_JPRB, 0.23898E+00_JPRB, 0.24051E+00_JPRB, 0.24212E+00_JPRB, 0.24384E+00_JPRB ,&
&  0.24526E+00_JPRB, 0.24718E+00_JPRB, 0.24818E+00_JPRB, 0.24778E+00_JPRB, 0.24678E+00_JPRB, 0.24642E+00_JPRB, 0.24637E+00_JPRB ,&
&  0.24694E+00_JPRB, 0.24725E+00_JPRB, 0.24738E+00_JPRB, 0.24765E+00_JPRB, 0.24791E+00_JPRB, 0.24741E+00_JPRB, 0.24619E+00_JPRB ,&
&  0.24515E+00_JPRB, 0.24430E+00_JPRB, 0.24262E+00_JPRB, 0.24101E+00_JPRB, 0.23950E+00_JPRB, 0.23793E+00_JPRB, 0.23629E+00_JPRB ,&
&  0.23511E+00_JPRB, 0.23393E+00_JPRB, 0.23245E+00_JPRB, 0.23073E+00_JPRB, 0.22913E+00_JPRB, 0.22772E+00_JPRB, 0.22661E+00_JPRB ,&
&  0.22638E+00_JPRB, 0.22578E+00_JPRB, 0.22527E+00_JPRB, 0.22476E+00_JPRB, 0.22463E+00_JPRB, 0.22417E+00_JPRB, 0.22384E+00_JPRB ,&
&  0.22359E+00_JPRB, 0.22327E+00_JPRB, 0.22291E+00_JPRB, 0.22256E+00_JPRB, 0.22231E+00_JPRB, 0.22198E+00_JPRB, 0.22166E+00_JPRB ,&
&  0.22145E+00_JPRB, 0.22129E+00_JPRB, 0.22110E+00_JPRB, 0.22070E+00_JPRB, 0.22030E+00_JPRB, 0.21982E+00_JPRB, 0.21981E+00_JPRB ,&
&  0.21997E+00_JPRB, 0.22026E+00_JPRB, 0.22078E+00_JPRB, 0.22100E+00_JPRB, 0.22190E+00_JPRB, 0.22277E+00_JPRB, 0.22380E+00_JPRB ,&
&  0.22489E+00_JPRB, 0.22654E+00_JPRB, 0.22836E+00_JPRB, 0.23033E+00_JPRB, 0.23343E+00_JPRB, 0.23445E+00_JPRB, 0.23527E+00_JPRB ,&
&  0.23628E+00_JPRB, 0.23989E+00_JPRB, 0.24078E+00_JPRB, 0.24056E+00_JPRB, 0.24119E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.22992E+00_JPRB, 0.23019E+00_JPRB, 0.23020E+00_JPRB, 0.23007E+00_JPRB, 0.22818E+00_JPRB, 0.22742E+00_JPRB, 0.22672E+00_JPRB ,&
&  0.22622E+00_JPRB, 0.22595E+00_JPRB, 0.22638E+00_JPRB, 0.22693E+00_JPRB, 0.22749E+00_JPRB, 0.22806E+00_JPRB, 0.22855E+00_JPRB ,&
&  0.22878E+00_JPRB, 0.22903E+00_JPRB, 0.22794E+00_JPRB, 0.22742E+00_JPRB, 0.22727E+00_JPRB, 0.22735E+00_JPRB, 0.22734E+00_JPRB ,&
&  0.22746E+00_JPRB, 0.22754E+00_JPRB, 0.22753E+00_JPRB, 0.22788E+00_JPRB, 0.22849E+00_JPRB, 0.22899E+00_JPRB, 0.22952E+00_JPRB ,&
&  0.23003E+00_JPRB, 0.23077E+00_JPRB, 0.23144E+00_JPRB, 0.23208E+00_JPRB, 0.23290E+00_JPRB, 0.23371E+00_JPRB, 0.23417E+00_JPRB ,&
&  0.23452E+00_JPRB, 0.23461E+00_JPRB, 0.23475E+00_JPRB, 0.23487E+00_JPRB, 0.23510E+00_JPRB, 0.23539E+00_JPRB, 0.23544E+00_JPRB ,&
&  0.23535E+00_JPRB, 0.23518E+00_JPRB, 0.23493E+00_JPRB, 0.23436E+00_JPRB, 0.23376E+00_JPRB, 0.23318E+00_JPRB, 0.23262E+00_JPRB ,&
&  0.23213E+00_JPRB, 0.23157E+00_JPRB, 0.23095E+00_JPRB, 0.23030E+00_JPRB, 0.22961E+00_JPRB, 0.22894E+00_JPRB, 0.22827E+00_JPRB ,&
&  0.22758E+00_JPRB, 0.22714E+00_JPRB, 0.22666E+00_JPRB, 0.22624E+00_JPRB, 0.22590E+00_JPRB, 0.22557E+00_JPRB, 0.22525E+00_JPRB ,&
&  0.22492E+00_JPRB, 0.22453E+00_JPRB, 0.22418E+00_JPRB, 0.22379E+00_JPRB, 0.22344E+00_JPRB, 0.22305E+00_JPRB, 0.22274E+00_JPRB ,&
&  0.22244E+00_JPRB, 0.22217E+00_JPRB, 0.22196E+00_JPRB, 0.22186E+00_JPRB, 0.22181E+00_JPRB, 0.22185E+00_JPRB, 0.22190E+00_JPRB ,&
&  0.22197E+00_JPRB, 0.22202E+00_JPRB, 0.22206E+00_JPRB, 0.22194E+00_JPRB, 0.22216E+00_JPRB, 0.22240E+00_JPRB, 0.22273E+00_JPRB ,&
&  0.22305E+00_JPRB, 0.22355E+00_JPRB, 0.22406E+00_JPRB, 0.22464E+00_JPRB, 0.22552E+00_JPRB, 0.22584E+00_JPRB, 0.22612E+00_JPRB ,&
&  0.22640E+00_JPRB, 0.22700E+00_JPRB, 0.22751E+00_JPRB, 0.22778E+00_JPRB, 0.22839E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB, 0.22572E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB, 0.57096E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.55204E+00_JPRB, 0.54909E+00_JPRB, 0.54640E+00_JPRB, 0.54354E+00_JPRB, 0.54055E+00_JPRB, 0.53702E+00_JPRB, 0.53324E+00_JPRB ,&
&  0.52980E+00_JPRB, 0.52670E+00_JPRB, 0.52393E+00_JPRB, 0.52140E+00_JPRB, 0.51930E+00_JPRB, 0.51761E+00_JPRB, 0.51682E+00_JPRB ,&
&  0.51647E+00_JPRB, 0.51615E+00_JPRB, 0.51604E+00_JPRB, 0.51600E+00_JPRB, 0.51572E+00_JPRB, 0.51529E+00_JPRB, 0.51474E+00_JPRB ,&
&  0.51445E+00_JPRB, 0.51425E+00_JPRB, 0.51411E+00_JPRB, 0.51439E+00_JPRB, 0.51486E+00_JPRB, 0.51532E+00_JPRB, 0.51575E+00_JPRB ,&
&  0.51663E+00_JPRB, 0.51771E+00_JPRB, 0.51920E+00_JPRB, 0.52105E+00_JPRB, 0.52476E+00_JPRB, 0.52872E+00_JPRB, 0.53275E+00_JPRB ,&
&  0.53653E+00_JPRB, 0.54029E+00_JPRB, 0.54330E+00_JPRB, 0.54579E+00_JPRB, 0.54805E+00_JPRB, 0.55093E+00_JPRB, 0.55416E+00_JPRB ,&
&  0.55766E+00_JPRB, 0.56159E+00_JPRB, 0.56632E+00_JPRB, 0.57095E+00_JPRB, 0.57521E+00_JPRB, 0.57959E+00_JPRB, 0.58440E+00_JPRB ,&
&  0.58910E+00_JPRB, 0.59388E+00_JPRB, 0.59873E+00_JPRB, 0.60384E+00_JPRB, 0.60870E+00_JPRB, 0.61455E+00_JPRB, 0.62154E+00_JPRB ,&
&  0.62974E+00_JPRB, 0.63604E+00_JPRB, 0.64345E+00_JPRB, 0.65147E+00_JPRB, 0.66392E+00_JPRB, 0.67020E+00_JPRB, 0.67624E+00_JPRB ,&
&  0.68144E+00_JPRB, 0.68999E+00_JPRB, 0.69534E+00_JPRB, 0.70052E+00_JPRB, 0.70522E+00_JPRB, 0.70946E+00_JPRB, 0.71223E+00_JPRB ,&
&  0.71463E+00_JPRB, 0.71673E+00_JPRB, 0.71717E+00_JPRB, 0.71694E+00_JPRB, 0.71572E+00_JPRB, 0.71405E+00_JPRB, 0.70946E+00_JPRB ,&
&  0.70534E+00_JPRB, 0.70139E+00_JPRB, 0.69770E+00_JPRB, 0.69221E+00_JPRB, 0.68655E+00_JPRB, 0.68067E+00_JPRB, 0.67463E+00_JPRB ,&
&  0.66675E+00_JPRB, 0.65864E+00_JPRB, 0.64986E+00_JPRB, 0.64048E+00_JPRB, 0.63146E+00_JPRB, 0.62201E+00_JPRB, 0.61168E+00_JPRB ,&
&  0.60003E+00_JPRB, 0.58914E+00_JPRB, 0.57681E+00_JPRB, 0.56460E+00_JPRB, 0.55573E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.53830E+00_JPRB, 0.52639E+00_JPRB, 0.52145E+00_JPRB, 0.51879E+00_JPRB, 0.51717E+00_JPRB, 0.51498E+00_JPRB, 0.51233E+00_JPRB ,&
&  0.51032E+00_JPRB, 0.50801E+00_JPRB, 0.50611E+00_JPRB, 0.50559E+00_JPRB, 0.50430E+00_JPRB, 0.50353E+00_JPRB, 0.50273E+00_JPRB ,&
&  0.50209E+00_JPRB, 0.50122E+00_JPRB, 0.50082E+00_JPRB, 0.49965E+00_JPRB, 0.49801E+00_JPRB, 0.49570E+00_JPRB, 0.49394E+00_JPRB ,&
&  0.49243E+00_JPRB, 0.49141E+00_JPRB, 0.48994E+00_JPRB, 0.48891E+00_JPRB, 0.48926E+00_JPRB, 0.48906E+00_JPRB, 0.48827E+00_JPRB ,&
&  0.49004E+00_JPRB, 0.49101E+00_JPRB, 0.49050E+00_JPRB, 0.48877E+00_JPRB, 0.48693E+00_JPRB, 0.48676E+00_JPRB, 0.48912E+00_JPRB ,&
&  0.49279E+00_JPRB, 0.50244E+00_JPRB, 0.50973E+00_JPRB, 0.51665E+00_JPRB, 0.52363E+00_JPRB, 0.53227E+00_JPRB, 0.54252E+00_JPRB ,&
&  0.55282E+00_JPRB, 0.55991E+00_JPRB, 0.56761E+00_JPRB, 0.57752E+00_JPRB, 0.58902E+00_JPRB, 0.60152E+00_JPRB, 0.61580E+00_JPRB ,&
&  0.62727E+00_JPRB, 0.63922E+00_JPRB, 0.65328E+00_JPRB, 0.66942E+00_JPRB, 0.68276E+00_JPRB, 0.70167E+00_JPRB, 0.72525E+00_JPRB ,&
&  0.75626E+00_JPRB, 0.77096E+00_JPRB, 0.78715E+00_JPRB, 0.80297E+00_JPRB, 0.82387E+00_JPRB, 0.83519E+00_JPRB, 0.84569E+00_JPRB ,&
&  0.85295E+00_JPRB, 0.86014E+00_JPRB, 0.85863E+00_JPRB, 0.85524E+00_JPRB, 0.85231E+00_JPRB, 0.84788E+00_JPRB, 0.84360E+00_JPRB ,&
&  0.83977E+00_JPRB, 0.83699E+00_JPRB, 0.82964E+00_JPRB, 0.82326E+00_JPRB, 0.81627E+00_JPRB, 0.80976E+00_JPRB, 0.79601E+00_JPRB ,&
&  0.78368E+00_JPRB, 0.77456E+00_JPRB, 0.76881E+00_JPRB, 0.75980E+00_JPRB, 0.75079E+00_JPRB, 0.74253E+00_JPRB, 0.73473E+00_JPRB ,&
&  0.72669E+00_JPRB, 0.71710E+00_JPRB, 0.70683E+00_JPRB, 0.69574E+00_JPRB, 0.68629E+00_JPRB, 0.67268E+00_JPRB, 0.65696E+00_JPRB ,&
&  0.63816E+00_JPRB, 0.62145E+00_JPRB, 0.60124E+00_JPRB, 0.57722E+00_JPRB, 0.55323E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.55741E+00_JPRB, 0.52907E+00_JPRB, 0.51448E+00_JPRB, 0.51113E+00_JPRB, 0.50998E+00_JPRB, 0.50888E+00_JPRB, 0.50777E+00_JPRB ,&
&  0.50595E+00_JPRB, 0.50523E+00_JPRB, 0.50654E+00_JPRB, 0.50768E+00_JPRB, 0.50775E+00_JPRB, 0.50772E+00_JPRB, 0.50662E+00_JPRB ,&
&  0.50496E+00_JPRB, 0.50171E+00_JPRB, 0.49857E+00_JPRB, 0.49356E+00_JPRB, 0.48839E+00_JPRB, 0.48193E+00_JPRB, 0.47357E+00_JPRB ,&
&  0.46596E+00_JPRB, 0.45897E+00_JPRB, 0.44967E+00_JPRB, 0.44660E+00_JPRB, 0.44489E+00_JPRB, 0.42485E+00_JPRB, 0.42017E+00_JPRB ,&
&  0.42540E+00_JPRB, 0.42669E+00_JPRB, 0.42783E+00_JPRB, 0.42911E+00_JPRB, 0.42946E+00_JPRB, 0.42253E+00_JPRB, 0.42114E+00_JPRB ,&
&  0.40677E+00_JPRB, 0.41801E+00_JPRB, 0.42656E+00_JPRB, 0.43470E+00_JPRB, 0.44359E+00_JPRB, 0.45687E+00_JPRB, 0.48062E+00_JPRB ,&
&  0.52407E+00_JPRB, 0.54394E+00_JPRB, 0.56614E+00_JPRB, 0.58662E+00_JPRB, 0.61444E+00_JPRB, 0.64468E+00_JPRB, 0.67275E+00_JPRB ,&
&  0.69591E+00_JPRB, 0.72086E+00_JPRB, 0.74969E+00_JPRB, 0.78895E+00_JPRB, 0.81903E+00_JPRB, 0.84703E+00_JPRB, 0.86652E+00_JPRB ,&
&  0.89143E+00_JPRB, 0.90982E+00_JPRB, 0.92259E+00_JPRB, 0.93095E+00_JPRB, 0.93946E+00_JPRB, 0.94220E+00_JPRB, 0.94569E+00_JPRB ,&
&  0.94602E+00_JPRB, 0.94509E+00_JPRB, 0.93606E+00_JPRB, 0.92113E+00_JPRB, 0.90918E+00_JPRB, 0.90222E+00_JPRB, 0.89261E+00_JPRB ,&
&  0.88279E+00_JPRB, 0.87230E+00_JPRB, 0.86022E+00_JPRB, 0.84615E+00_JPRB, 0.83077E+00_JPRB, 0.81771E+00_JPRB, 0.80214E+00_JPRB ,&
&  0.78922E+00_JPRB, 0.77910E+00_JPRB, 0.77099E+00_JPRB, 0.76033E+00_JPRB, 0.74933E+00_JPRB, 0.73852E+00_JPRB, 0.72639E+00_JPRB ,&
&  0.71486E+00_JPRB, 0.70491E+00_JPRB, 0.69678E+00_JPRB, 0.68760E+00_JPRB, 0.68144E+00_JPRB, 0.67232E+00_JPRB, 0.66396E+00_JPRB ,&
&  0.65410E+00_JPRB, 0.64533E+00_JPRB, 0.63089E+00_JPRB, 0.61008E+00_JPRB, 0.58201E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.59295E+00_JPRB, 0.56036E+00_JPRB, 0.52771E+00_JPRB, 0.50992E+00_JPRB, 0.50687E+00_JPRB, 0.50522E+00_JPRB, 0.50438E+00_JPRB ,&
&  0.50444E+00_JPRB, 0.50739E+00_JPRB, 0.51032E+00_JPRB, 0.51240E+00_JPRB, 0.51262E+00_JPRB, 0.51100E+00_JPRB, 0.50808E+00_JPRB ,&
&  0.50312E+00_JPRB, 0.49715E+00_JPRB, 0.49141E+00_JPRB, 0.48623E+00_JPRB, 0.48050E+00_JPRB, 0.47434E+00_JPRB, 0.46575E+00_JPRB ,&
&  0.45628E+00_JPRB, 0.44725E+00_JPRB, 0.41627E+00_JPRB, 0.41375E+00_JPRB, 0.40742E+00_JPRB, 0.40605E+00_JPRB, 0.40942E+00_JPRB ,&
&  0.41089E+00_JPRB, 0.40433E+00_JPRB, 0.39637E+00_JPRB, 0.38306E+00_JPRB, 0.37177E+00_JPRB, 0.35977E+00_JPRB, 0.35294E+00_JPRB ,&
&  0.35047E+00_JPRB, 0.35259E+00_JPRB, 0.35927E+00_JPRB, 0.36627E+00_JPRB, 0.37545E+00_JPRB, 0.38532E+00_JPRB, 0.40134E+00_JPRB ,&
&  0.44407E+00_JPRB, 0.50445E+00_JPRB, 0.57635E+00_JPRB, 0.61231E+00_JPRB, 0.65423E+00_JPRB, 0.69936E+00_JPRB, 0.73287E+00_JPRB ,&
&  0.76200E+00_JPRB, 0.79260E+00_JPRB, 0.82337E+00_JPRB, 0.85871E+00_JPRB, 0.88985E+00_JPRB, 0.91261E+00_JPRB, 0.94454E+00_JPRB ,&
&  0.97057E+00_JPRB, 0.98909E+00_JPRB, 0.99334E+00_JPRB, 0.99466E+00_JPRB, 0.99259E+00_JPRB, 0.98146E+00_JPRB, 0.97008E+00_JPRB ,&
&  0.95923E+00_JPRB, 0.94750E+00_JPRB, 0.93129E+00_JPRB, 0.90228E+00_JPRB, 0.88295E+00_JPRB, 0.87272E+00_JPRB, 0.85363E+00_JPRB ,&
&  0.83463E+00_JPRB, 0.81340E+00_JPRB, 0.79452E+00_JPRB, 0.76896E+00_JPRB, 0.75194E+00_JPRB, 0.74723E+00_JPRB, 0.74579E+00_JPRB ,&
&  0.74434E+00_JPRB, 0.74187E+00_JPRB, 0.73855E+00_JPRB, 0.73061E+00_JPRB, 0.71623E+00_JPRB, 0.70071E+00_JPRB, 0.68714E+00_JPRB ,&
&  0.67371E+00_JPRB, 0.65993E+00_JPRB, 0.64706E+00_JPRB, 0.63400E+00_JPRB, 0.62677E+00_JPRB, 0.62171E+00_JPRB, 0.62227E+00_JPRB ,&
&  0.62560E+00_JPRB, 0.63056E+00_JPRB, 0.63229E+00_JPRB, 0.62943E+00_JPRB, 0.61560E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.66517E+00_JPRB, 0.65240E+00_JPRB, 0.62671E+00_JPRB, 0.59150E+00_JPRB, 0.56518E+00_JPRB, 0.54779E+00_JPRB, 0.53960E+00_JPRB ,&
&  0.53493E+00_JPRB, 0.53257E+00_JPRB, 0.53071E+00_JPRB, 0.52800E+00_JPRB, 0.52388E+00_JPRB, 0.51789E+00_JPRB, 0.51233E+00_JPRB ,&
&  0.50220E+00_JPRB, 0.49318E+00_JPRB, 0.48322E+00_JPRB, 0.47773E+00_JPRB, 0.47315E+00_JPRB, 0.46953E+00_JPRB, 0.46666E+00_JPRB ,&
&  0.46287E+00_JPRB, 0.45793E+00_JPRB, 0.44110E+00_JPRB, 0.43413E+00_JPRB, 0.42748E+00_JPRB, 0.42837E+00_JPRB, 0.42750E+00_JPRB ,&
&  0.44263E+00_JPRB, 0.39848E+00_JPRB, 0.38956E+00_JPRB, 0.37525E+00_JPRB, 0.36429E+00_JPRB, 0.35165E+00_JPRB, 0.34017E+00_JPRB ,&
&  0.33325E+00_JPRB, 0.32724E+00_JPRB, 0.32614E+00_JPRB, 0.33321E+00_JPRB, 0.34048E+00_JPRB, 0.35363E+00_JPRB, 0.36845E+00_JPRB ,&
&  0.40240E+00_JPRB, 0.52128E+00_JPRB, 0.57718E+00_JPRB, 0.62050E+00_JPRB, 0.66296E+00_JPRB, 0.70634E+00_JPRB, 0.73170E+00_JPRB ,&
&  0.75591E+00_JPRB, 0.78831E+00_JPRB, 0.81036E+00_JPRB, 0.81468E+00_JPRB, 0.87520E+00_JPRB, 0.91927E+00_JPRB, 0.96119E+00_JPRB ,&
&  0.99835E+00_JPRB, 0.10272E+01_JPRB, 0.10496E+01_JPRB, 0.10628E+01_JPRB, 0.10773E+01_JPRB, 0.10684E+01_JPRB, 0.10492E+01_JPRB ,&
&  0.10311E+01_JPRB, 0.10171E+01_JPRB, 0.10037E+01_JPRB, 0.96373E+00_JPRB, 0.93331E+00_JPRB, 0.91411E+00_JPRB, 0.87740E+00_JPRB ,&
&  0.84045E+00_JPRB, 0.80629E+00_JPRB, 0.77320E+00_JPRB, 0.74068E+00_JPRB, 0.71407E+00_JPRB, 0.70007E+00_JPRB, 0.69904E+00_JPRB ,&
&  0.70362E+00_JPRB, 0.70609E+00_JPRB, 0.70009E+00_JPRB, 0.68764E+00_JPRB, 0.67188E+00_JPRB, 0.66167E+00_JPRB, 0.64609E+00_JPRB ,&
&  0.62862E+00_JPRB, 0.60681E+00_JPRB, 0.59263E+00_JPRB, 0.58276E+00_JPRB, 0.58011E+00_JPRB, 0.58345E+00_JPRB, 0.59121E+00_JPRB ,&
&  0.60169E+00_JPRB, 0.61593E+00_JPRB, 0.63376E+00_JPRB, 0.65054E+00_JPRB, 0.66215E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.64907E+00_JPRB, 0.65869E+00_JPRB, 0.66143E+00_JPRB, 0.65523E+00_JPRB, 0.64230E+00_JPRB, 0.62488E+00_JPRB, 0.60674E+00_JPRB ,&
&  0.59041E+00_JPRB, 0.57769E+00_JPRB, 0.56610E+00_JPRB, 0.55755E+00_JPRB, 0.54752E+00_JPRB, 0.54123E+00_JPRB, 0.53396E+00_JPRB ,&
&  0.53021E+00_JPRB, 0.52907E+00_JPRB, 0.52319E+00_JPRB, 0.51593E+00_JPRB, 0.50134E+00_JPRB, 0.48805E+00_JPRB, 0.47796E+00_JPRB ,&
&  0.46898E+00_JPRB, 0.46195E+00_JPRB, 0.44556E+00_JPRB, 0.43344E+00_JPRB, 0.42787E+00_JPRB, 0.42593E+00_JPRB, 0.43439E+00_JPRB ,&
&  0.43676E+00_JPRB, 0.44466E+00_JPRB, 0.45110E+00_JPRB, 0.44533E+00_JPRB, 0.43885E+00_JPRB, 0.35057E+00_JPRB, 0.32289E+00_JPRB ,&
&  0.31429E+00_JPRB, 0.30811E+00_JPRB, 0.30684E+00_JPRB, 0.31236E+00_JPRB, 0.32133E+00_JPRB, 0.34385E+00_JPRB, 0.36160E+00_JPRB ,&
&  0.38044E+00_JPRB, 0.50621E+00_JPRB, 0.55724E+00_JPRB, 0.60465E+00_JPRB, 0.63826E+00_JPRB, 0.66863E+00_JPRB, 0.69040E+00_JPRB ,&
&  0.70914E+00_JPRB, 0.73567E+00_JPRB, 0.74936E+00_JPRB, 0.77806E+00_JPRB, 0.83464E+00_JPRB, 0.93490E+00_JPRB, 0.10155E+01_JPRB ,&
&  0.11066E+01_JPRB, 0.11304E+01_JPRB, 0.11446E+01_JPRB, 0.11274E+01_JPRB, 0.11343E+01_JPRB, 0.11342E+01_JPRB, 0.11239E+01_JPRB ,&
&  0.11190E+01_JPRB, 0.10913E+01_JPRB, 0.10863E+01_JPRB, 0.10623E+01_JPRB, 0.10420E+01_JPRB, 0.10213E+01_JPRB, 0.95905E+00_JPRB ,&
&  0.88755E+00_JPRB, 0.83931E+00_JPRB, 0.79563E+00_JPRB, 0.74345E+00_JPRB, 0.70497E+00_JPRB, 0.67792E+00_JPRB, 0.67068E+00_JPRB ,&
&  0.66968E+00_JPRB, 0.67786E+00_JPRB, 0.66803E+00_JPRB, 0.65206E+00_JPRB, 0.64012E+00_JPRB, 0.63196E+00_JPRB, 0.61294E+00_JPRB ,&
&  0.59621E+00_JPRB, 0.57979E+00_JPRB, 0.57172E+00_JPRB, 0.56350E+00_JPRB, 0.55877E+00_JPRB, 0.55879E+00_JPRB, 0.56289E+00_JPRB ,&
&  0.57219E+00_JPRB, 0.58491E+00_JPRB, 0.60257E+00_JPRB, 0.62073E+00_JPRB, 0.63997E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.63142E+00_JPRB, 0.64779E+00_JPRB, 0.65637E+00_JPRB, 0.65430E+00_JPRB, 0.64033E+00_JPRB, 0.62154E+00_JPRB, 0.60508E+00_JPRB ,&
&  0.59003E+00_JPRB, 0.57582E+00_JPRB, 0.56285E+00_JPRB, 0.55315E+00_JPRB, 0.54483E+00_JPRB, 0.53967E+00_JPRB, 0.53613E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.52368E+00_JPRB, 0.51975E+00_JPRB, 0.52055E+00_JPRB, 0.52352E+00_JPRB, 0.52230E+00_JPRB, 0.51706E+00_JPRB ,&
&  0.49854E+00_JPRB, 0.47302E+00_JPRB, 0.44922E+00_JPRB, 0.42930E+00_JPRB, 0.41457E+00_JPRB, 0.41042E+00_JPRB, 0.41727E+00_JPRB ,&
&  0.42635E+00_JPRB, 0.42538E+00_JPRB, 0.42807E+00_JPRB, 0.44687E+00_JPRB, 0.46028E+00_JPRB, 0.40198E+00_JPRB, 0.32322E+00_JPRB ,&
&  0.31057E+00_JPRB, 0.29723E+00_JPRB, 0.29640E+00_JPRB, 0.30212E+00_JPRB, 0.31298E+00_JPRB, 0.32958E+00_JPRB, 0.35867E+00_JPRB ,&
&  0.40146E+00_JPRB, 0.49287E+00_JPRB, 0.53654E+00_JPRB, 0.57391E+00_JPRB, 0.61060E+00_JPRB, 0.64349E+00_JPRB, 0.66176E+00_JPRB ,&
&  0.67189E+00_JPRB, 0.69004E+00_JPRB, 0.72698E+00_JPRB, 0.76966E+00_JPRB, 0.84210E+00_JPRB, 0.98759E+00_JPRB, 0.11018E+01_JPRB ,&
&  0.12588E+01_JPRB, 0.13181E+01_JPRB, 0.12706E+01_JPRB, 0.11616E+01_JPRB, 0.11509E+01_JPRB, 0.11797E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.11897E+01_JPRB, 0.12291E+01_JPRB, 0.12118E+01_JPRB, 0.11567E+01_JPRB, 0.11210E+01_JPRB, 0.10999E+01_JPRB, 0.10158E+01_JPRB ,&
&  0.92363E+00_JPRB, 0.88839E+00_JPRB, 0.89312E+00_JPRB, 0.80621E+00_JPRB, 0.74702E+00_JPRB, 0.69172E+00_JPRB, 0.66278E+00_JPRB ,&
&  0.64587E+00_JPRB, 0.63147E+00_JPRB, 0.61132E+00_JPRB, 0.59469E+00_JPRB, 0.58667E+00_JPRB, 0.58161E+00_JPRB, 0.57409E+00_JPRB ,&
&  0.56640E+00_JPRB, 0.55939E+00_JPRB, 0.55251E+00_JPRB, 0.54779E+00_JPRB, 0.54363E+00_JPRB, 0.54251E+00_JPRB, 0.54493E+00_JPRB ,&
&  0.55227E+00_JPRB, 0.56457E+00_JPRB, 0.58090E+00_JPRB, 0.59613E+00_JPRB, 0.61497E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.63233E+00_JPRB, 0.64471E+00_JPRB, 0.63776E+00_JPRB, 0.61333E+00_JPRB, 0.57937E+00_JPRB, 0.53843E+00_JPRB, 0.51205E+00_JPRB ,&
&  0.49842E+00_JPRB, 0.49311E+00_JPRB, 0.48969E+00_JPRB, 0.48398E+00_JPRB, 0.48226E+00_JPRB, 0.48220E+00_JPRB, 0.48380E+00_JPRB ,&
&  0.48766E+00_JPRB, 0.49280E+00_JPRB, 0.49679E+00_JPRB, 0.49537E+00_JPRB, 0.49940E+00_JPRB, 0.50302E+00_JPRB, 0.50658E+00_JPRB ,&
&  0.50292E+00_JPRB, 0.48422E+00_JPRB, 0.45946E+00_JPRB, 0.44298E+00_JPRB, 0.42600E+00_JPRB, 0.42598E+00_JPRB, 0.41739E+00_JPRB ,&
&  0.39313E+00_JPRB, 0.38960E+00_JPRB, 0.41847E+00_JPRB, 0.44773E+00_JPRB, 0.44560E+00_JPRB, 0.46892E+00_JPRB, 0.35600E+00_JPRB ,&
&  0.31952E+00_JPRB, 0.30171E+00_JPRB, 0.29821E+00_JPRB, 0.30661E+00_JPRB, 0.32865E+00_JPRB, 0.35785E+00_JPRB, 0.38895E+00_JPRB ,&
&  0.44370E+00_JPRB, 0.48960E+00_JPRB, 0.51554E+00_JPRB, 0.54633E+00_JPRB, 0.58307E+00_JPRB, 0.62069E+00_JPRB, 0.64943E+00_JPRB ,&
&  0.66819E+00_JPRB, 0.69964E+00_JPRB, 0.75491E+00_JPRB, 0.84221E+00_JPRB, 0.10376E+01_JPRB, 0.10094E+01_JPRB, 0.10401E+01_JPRB ,&
&  0.13421E+01_JPRB, 0.15092E+01_JPRB, 0.13652E+01_JPRB, 0.12247E+01_JPRB, 0.11616E+01_JPRB, 0.11662E+01_JPRB, 0.11854E+01_JPRB ,&
&  0.12717E+01_JPRB, 0.13534E+01_JPRB, 0.13498E+01_JPRB, 0.12301E+01_JPRB, 0.11689E+01_JPRB, 0.11344E+01_JPRB, 0.10259E+01_JPRB ,&
&  0.90836E+00_JPRB, 0.93117E+00_JPRB, 0.98633E+00_JPRB, 0.83173E+00_JPRB, 0.72776E+00_JPRB, 0.66362E+00_JPRB, 0.63896E+00_JPRB ,&
&  0.62569E+00_JPRB, 0.61324E+00_JPRB, 0.59725E+00_JPRB, 0.58114E+00_JPRB, 0.55953E+00_JPRB, 0.54202E+00_JPRB, 0.52609E+00_JPRB ,&
&  0.51643E+00_JPRB, 0.50969E+00_JPRB, 0.50955E+00_JPRB, 0.51266E+00_JPRB, 0.51484E+00_JPRB, 0.51701E+00_JPRB, 0.52618E+00_JPRB ,&
&  0.54079E+00_JPRB, 0.54953E+00_JPRB, 0.56462E+00_JPRB, 0.58259E+00_JPRB, 0.60690E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.57198E+00_JPRB, 0.57487E+00_JPRB, 0.56210E+00_JPRB, 0.53338E+00_JPRB, 0.50494E+00_JPRB, 0.46676E+00_JPRB, 0.43926E+00_JPRB ,&
&  0.42421E+00_JPRB, 0.41767E+00_JPRB, 0.41658E+00_JPRB, 0.41764E+00_JPRB, 0.42109E+00_JPRB, 0.42705E+00_JPRB, 0.42813E+00_JPRB ,&
&  0.42998E+00_JPRB, 0.43013E+00_JPRB, 0.43312E+00_JPRB, 0.43971E+00_JPRB, 0.44719E+00_JPRB, 0.45555E+00_JPRB, 0.46436E+00_JPRB ,&
&  0.47299E+00_JPRB, 0.47080E+00_JPRB, 0.45383E+00_JPRB, 0.44086E+00_JPRB, 0.44599E+00_JPRB, 0.46589E+00_JPRB, 0.45450E+00_JPRB ,&
&  0.41555E+00_JPRB, 0.39101E+00_JPRB, 0.39242E+00_JPRB, 0.45264E+00_JPRB, 0.48938E+00_JPRB, 0.50875E+00_JPRB, 0.39673E+00_JPRB ,&
&  0.33819E+00_JPRB, 0.31564E+00_JPRB, 0.31639E+00_JPRB, 0.33996E+00_JPRB, 0.38531E+00_JPRB, 0.45120E+00_JPRB, 0.50560E+00_JPRB ,&
&  0.55052E+00_JPRB, 0.58901E+00_JPRB, 0.60031E+00_JPRB, 0.62311E+00_JPRB, 0.66064E+00_JPRB, 0.69493E+00_JPRB, 0.73236E+00_JPRB ,&
&  0.75550E+00_JPRB, 0.79418E+00_JPRB, 0.91353E+00_JPRB, 0.78960E+00_JPRB, 0.83944E+00_JPRB, 0.90031E+00_JPRB, 0.10160E+01_JPRB ,&
&  0.11846E+01_JPRB, 0.12352E+01_JPRB, 0.12337E+01_JPRB, 0.13063E+01_JPRB, 0.12020E+01_JPRB, 0.11173E+01_JPRB, 0.11092E+01_JPRB ,&
&  0.12655E+01_JPRB, 0.14126E+01_JPRB, 0.13858E+01_JPRB, 0.12709E+01_JPRB, 0.12310E+01_JPRB, 0.12078E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.91845E+00_JPRB, 0.82749E+00_JPRB, 0.75036E+00_JPRB, 0.65433E+00_JPRB, 0.61566E+00_JPRB, 0.59372E+00_JPRB, 0.58184E+00_JPRB ,&
&  0.57525E+00_JPRB, 0.56483E+00_JPRB, 0.55698E+00_JPRB, 0.52273E+00_JPRB, 0.48743E+00_JPRB, 0.45407E+00_JPRB, 0.43208E+00_JPRB ,&
&  0.41983E+00_JPRB, 0.41897E+00_JPRB, 0.42567E+00_JPRB, 0.44474E+00_JPRB, 0.47845E+00_JPRB, 0.50927E+00_JPRB, 0.53014E+00_JPRB ,&
&  0.53353E+00_JPRB, 0.52043E+00_JPRB, 0.51864E+00_JPRB, 0.52531E+00_JPRB, 0.55416E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.55541E+00_JPRB, 0.52292E+00_JPRB, 0.50227E+00_JPRB, 0.50713E+00_JPRB, 0.46992E+00_JPRB, 0.43462E+00_JPRB, 0.41553E+00_JPRB ,&
&  0.41680E+00_JPRB, 0.41396E+00_JPRB, 0.41639E+00_JPRB, 0.39813E+00_JPRB, 0.38221E+00_JPRB, 0.37282E+00_JPRB, 0.36908E+00_JPRB ,&
&  0.37844E+00_JPRB, 0.39001E+00_JPRB, 0.39852E+00_JPRB, 0.39685E+00_JPRB, 0.39505E+00_JPRB, 0.39903E+00_JPRB, 0.41019E+00_JPRB ,&
&  0.42641E+00_JPRB, 0.44029E+00_JPRB, 0.44122E+00_JPRB, 0.44081E+00_JPRB, 0.46513E+00_JPRB, 0.49412E+00_JPRB, 0.47043E+00_JPRB ,&
&  0.44067E+00_JPRB, 0.41786E+00_JPRB, 0.41393E+00_JPRB, 0.45799E+00_JPRB, 0.52427E+00_JPRB, 0.55656E+00_JPRB, 0.44047E+00_JPRB ,&
&  0.36245E+00_JPRB, 0.35216E+00_JPRB, 0.39015E+00_JPRB, 0.51137E+00_JPRB, 0.54829E+00_JPRB, 0.59515E+00_JPRB, 0.66525E+00_JPRB ,&
&  0.61625E+00_JPRB, 0.61907E+00_JPRB, 0.64387E+00_JPRB, 0.78582E+00_JPRB, 0.82235E+00_JPRB, 0.83740E+00_JPRB, 0.84931E+00_JPRB ,&
&  0.85736E+00_JPRB, 0.89189E+00_JPRB, 0.10341E+01_JPRB, 0.86633E+00_JPRB, 0.91161E+00_JPRB, 0.11192E+01_JPRB, 0.11155E+01_JPRB ,&
&  0.11669E+01_JPRB, 0.11805E+01_JPRB, 0.13065E+01_JPRB, 0.12477E+01_JPRB, 0.11249E+01_JPRB, 0.10693E+01_JPRB, 0.11375E+01_JPRB ,&
&  0.13502E+01_JPRB, 0.15252E+01_JPRB, 0.14869E+01_JPRB, 0.14305E+01_JPRB, 0.13973E+01_JPRB, 0.13360E+01_JPRB, 0.11924E+01_JPRB ,&
&  0.98632E+00_JPRB, 0.83078E+00_JPRB, 0.71347E+00_JPRB, 0.61105E+00_JPRB, 0.56245E+00_JPRB, 0.54410E+00_JPRB, 0.52597E+00_JPRB ,&
&  0.52209E+00_JPRB, 0.51690E+00_JPRB, 0.50927E+00_JPRB, 0.47092E+00_JPRB, 0.43626E+00_JPRB, 0.41409E+00_JPRB, 0.39881E+00_JPRB ,&
&  0.39202E+00_JPRB, 0.37883E+00_JPRB, 0.36954E+00_JPRB, 0.38263E+00_JPRB, 0.42346E+00_JPRB, 0.47439E+00_JPRB, 0.49706E+00_JPRB ,&
&  0.48722E+00_JPRB, 0.48164E+00_JPRB, 0.50792E+00_JPRB, 0.54795E+00_JPRB, 0.56591E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.57144E+00_JPRB, 0.56027E+00_JPRB, 0.55411E+00_JPRB, 0.54376E+00_JPRB, 0.52791E+00_JPRB, 0.48209E+00_JPRB, 0.44904E+00_JPRB ,&
&  0.45761E+00_JPRB, 0.43401E+00_JPRB, 0.42826E+00_JPRB, 0.40246E+00_JPRB, 0.38517E+00_JPRB, 0.36559E+00_JPRB, 0.35273E+00_JPRB ,&
&  0.34342E+00_JPRB, 0.35159E+00_JPRB, 0.37151E+00_JPRB, 0.37676E+00_JPRB, 0.37671E+00_JPRB, 0.37641E+00_JPRB, 0.38559E+00_JPRB ,&
&  0.39950E+00_JPRB, 0.41994E+00_JPRB, 0.43453E+00_JPRB, 0.44691E+00_JPRB, 0.46978E+00_JPRB, 0.49961E+00_JPRB, 0.50146E+00_JPRB ,&
&  0.49967E+00_JPRB, 0.48002E+00_JPRB, 0.45998E+00_JPRB, 0.49270E+00_JPRB, 0.55778E+00_JPRB, 0.61961E+00_JPRB, 0.59891E+00_JPRB ,&
&  0.43154E+00_JPRB, 0.43474E+00_JPRB, 0.60397E+00_JPRB, 0.65344E+00_JPRB, 0.70781E+00_JPRB, 0.74372E+00_JPRB, 0.79033E+00_JPRB ,&
&  0.83236E+00_JPRB, 0.87513E+00_JPRB, 0.90610E+00_JPRB, 0.93142E+00_JPRB, 0.94933E+00_JPRB, 0.97009E+00_JPRB, 0.96404E+00_JPRB ,&
&  0.10301E+01_JPRB, 0.83238E+00_JPRB, 0.78891E+00_JPRB, 0.84329E+00_JPRB, 0.11218E+01_JPRB, 0.11746E+01_JPRB, 0.11367E+01_JPRB ,&
&  0.11580E+01_JPRB, 0.11329E+01_JPRB, 0.11718E+01_JPRB, 0.12456E+01_JPRB, 0.12433E+01_JPRB, 0.12910E+01_JPRB, 0.13793E+01_JPRB ,&
&  0.15393E+01_JPRB, 0.17147E+01_JPRB, 0.17246E+01_JPRB, 0.16943E+01_JPRB, 0.16162E+01_JPRB, 0.14813E+01_JPRB, 0.12698E+01_JPRB ,&
&  0.10113E+01_JPRB, 0.83076E+00_JPRB, 0.68400E+00_JPRB, 0.58168E+00_JPRB, 0.52372E+00_JPRB, 0.49859E+00_JPRB, 0.48020E+00_JPRB ,&
&  0.47976E+00_JPRB, 0.47691E+00_JPRB, 0.46565E+00_JPRB, 0.43513E+00_JPRB, 0.41583E+00_JPRB, 0.42256E+00_JPRB, 0.42995E+00_JPRB ,&
&  0.42970E+00_JPRB, 0.41285E+00_JPRB, 0.38799E+00_JPRB, 0.39957E+00_JPRB, 0.43761E+00_JPRB, 0.47791E+00_JPRB, 0.49918E+00_JPRB ,&
&  0.49950E+00_JPRB, 0.50154E+00_JPRB, 0.51483E+00_JPRB, 0.54077E+00_JPRB, 0.55467E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.59789E+00_JPRB, 0.60061E+00_JPRB, 0.58632E+00_JPRB, 0.57300E+00_JPRB, 0.55597E+00_JPRB, 0.49912E+00_JPRB, 0.48677E+00_JPRB ,&
&  0.55114E+00_JPRB, 0.49659E+00_JPRB, 0.55149E+00_JPRB, 0.44875E+00_JPRB, 0.42620E+00_JPRB, 0.40539E+00_JPRB, 0.38445E+00_JPRB ,&
&  0.36045E+00_JPRB, 0.35394E+00_JPRB, 0.36170E+00_JPRB, 0.36872E+00_JPRB, 0.36948E+00_JPRB, 0.36658E+00_JPRB, 0.36951E+00_JPRB ,&
&  0.37770E+00_JPRB, 0.40018E+00_JPRB, 0.43117E+00_JPRB, 0.44815E+00_JPRB, 0.46762E+00_JPRB, 0.49990E+00_JPRB, 0.52080E+00_JPRB ,&
&  0.50497E+00_JPRB, 0.50629E+00_JPRB, 0.51585E+00_JPRB, 0.54117E+00_JPRB, 0.58479E+00_JPRB, 0.64777E+00_JPRB, 0.68849E+00_JPRB ,&
&  0.70666E+00_JPRB, 0.58870E+00_JPRB, 0.69744E+00_JPRB, 0.72107E+00_JPRB, 0.77885E+00_JPRB, 0.81135E+00_JPRB, 0.83444E+00_JPRB ,&
&  0.86741E+00_JPRB, 0.90539E+00_JPRB, 0.94818E+00_JPRB, 0.98719E+00_JPRB, 0.10269E+01_JPRB, 0.10634E+01_JPRB, 0.10629E+01_JPRB ,&
&  0.11448E+01_JPRB, 0.80892E+00_JPRB, 0.86798E+00_JPRB, 0.90458E+00_JPRB, 0.12246E+01_JPRB, 0.13634E+01_JPRB, 0.12943E+01_JPRB ,&
&  0.14415E+01_JPRB, 0.13522E+01_JPRB, 0.14254E+01_JPRB, 0.14938E+01_JPRB, 0.15828E+01_JPRB, 0.17128E+01_JPRB, 0.17873E+01_JPRB ,&
&  0.18136E+01_JPRB, 0.18887E+01_JPRB, 0.19829E+01_JPRB, 0.18567E+01_JPRB, 0.16589E+01_JPRB, 0.14270E+01_JPRB, 0.11606E+01_JPRB ,&
&  0.97745E+00_JPRB, 0.81975E+00_JPRB, 0.68685E+00_JPRB, 0.59670E+00_JPRB, 0.52720E+00_JPRB, 0.48240E+00_JPRB, 0.45772E+00_JPRB ,&
&  0.44541E+00_JPRB, 0.42978E+00_JPRB, 0.41823E+00_JPRB, 0.40043E+00_JPRB, 0.39737E+00_JPRB, 0.40678E+00_JPRB, 0.43716E+00_JPRB ,&
&  0.44885E+00_JPRB, 0.44551E+00_JPRB, 0.49010E+00_JPRB, 0.53877E+00_JPRB, 0.54499E+00_JPRB, 0.56252E+00_JPRB, 0.63059E+00_JPRB ,&
&  0.54958E+00_JPRB, 0.57205E+00_JPRB, 0.62202E+00_JPRB, 0.62035E+00_JPRB, 0.60086E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.64832E+00_JPRB, 0.64709E+00_JPRB, 0.62264E+00_JPRB, 0.59900E+00_JPRB, 0.58534E+00_JPRB, 0.55732E+00_JPRB, 0.52872E+00_JPRB ,&
&  0.57657E+00_JPRB, 0.56631E+00_JPRB, 0.56845E+00_JPRB, 0.57229E+00_JPRB, 0.56662E+00_JPRB, 0.50471E+00_JPRB, 0.44827E+00_JPRB ,&
&  0.40942E+00_JPRB, 0.38825E+00_JPRB, 0.39205E+00_JPRB, 0.39096E+00_JPRB, 0.38804E+00_JPRB, 0.38012E+00_JPRB, 0.37664E+00_JPRB ,&
&  0.37531E+00_JPRB, 0.38780E+00_JPRB, 0.41690E+00_JPRB, 0.44250E+00_JPRB, 0.45469E+00_JPRB, 0.47200E+00_JPRB, 0.50346E+00_JPRB ,&
&  0.49654E+00_JPRB, 0.50649E+00_JPRB, 0.52798E+00_JPRB, 0.50083E+00_JPRB, 0.60122E+00_JPRB, 0.64410E+00_JPRB, 0.66833E+00_JPRB ,&
&  0.71580E+00_JPRB, 0.75599E+00_JPRB, 0.77181E+00_JPRB, 0.77429E+00_JPRB, 0.79141E+00_JPRB, 0.80141E+00_JPRB, 0.82439E+00_JPRB ,&
&  0.84760E+00_JPRB, 0.88323E+00_JPRB, 0.93267E+00_JPRB, 0.99175E+00_JPRB, 0.11085E+01_JPRB, 0.10134E+01_JPRB, 0.11088E+01_JPRB ,&
&  0.11681E+01_JPRB, 0.11969E+01_JPRB, 0.12680E+01_JPRB, 0.11258E+01_JPRB, 0.14215E+01_JPRB, 0.15849E+01_JPRB, 0.14468E+01_JPRB ,&
&  0.14525E+01_JPRB, 0.13982E+01_JPRB, 0.16570E+01_JPRB, 0.18515E+01_JPRB, 0.20087E+01_JPRB, 0.21476E+01_JPRB, 0.21994E+01_JPRB ,&
&  0.21777E+01_JPRB, 0.23056E+01_JPRB, 0.22657E+01_JPRB, 0.20522E+01_JPRB, 0.17579E+01_JPRB, 0.15137E+01_JPRB, 0.12234E+01_JPRB ,&
&  0.98353E+00_JPRB, 0.79106E+00_JPRB, 0.67346E+00_JPRB, 0.61543E+00_JPRB, 0.52156E+00_JPRB, 0.48910E+00_JPRB, 0.45452E+00_JPRB ,&
&  0.42155E+00_JPRB, 0.39575E+00_JPRB, 0.38813E+00_JPRB, 0.38169E+00_JPRB, 0.38236E+00_JPRB, 0.39826E+00_JPRB, 0.41850E+00_JPRB ,&
&  0.47420E+00_JPRB, 0.57195E+00_JPRB, 0.77100E+00_JPRB, 0.74897E+00_JPRB, 0.73123E+00_JPRB, 0.71129E+00_JPRB, 0.62165E+00_JPRB ,&
&  0.58181E+00_JPRB, 0.67231E+00_JPRB, 0.65996E+00_JPRB, 0.64935E+00_JPRB, 0.63786E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.69304E+00_JPRB, 0.67952E+00_JPRB, 0.66199E+00_JPRB, 0.63888E+00_JPRB, 0.61213E+00_JPRB, 0.59048E+00_JPRB, 0.58151E+00_JPRB ,&
&  0.58393E+00_JPRB, 0.58370E+00_JPRB, 0.58570E+00_JPRB, 0.58223E+00_JPRB, 0.57097E+00_JPRB, 0.58771E+00_JPRB, 0.52055E+00_JPRB ,&
&  0.44681E+00_JPRB, 0.42114E+00_JPRB, 0.40324E+00_JPRB, 0.40651E+00_JPRB, 0.41702E+00_JPRB, 0.41070E+00_JPRB, 0.40176E+00_JPRB ,&
&  0.39369E+00_JPRB, 0.39060E+00_JPRB, 0.40105E+00_JPRB, 0.41995E+00_JPRB, 0.43460E+00_JPRB, 0.45090E+00_JPRB, 0.47339E+00_JPRB ,&
&  0.46527E+00_JPRB, 0.44976E+00_JPRB, 0.44847E+00_JPRB, 0.46235E+00_JPRB, 0.53569E+00_JPRB, 0.62919E+00_JPRB, 0.68091E+00_JPRB ,&
&  0.70137E+00_JPRB, 0.71668E+00_JPRB, 0.75116E+00_JPRB, 0.76402E+00_JPRB, 0.77044E+00_JPRB, 0.79036E+00_JPRB, 0.81563E+00_JPRB ,&
&  0.84307E+00_JPRB, 0.88074E+00_JPRB, 0.93204E+00_JPRB, 0.10158E+01_JPRB, 0.10352E+01_JPRB, 0.10229E+01_JPRB, 0.11285E+01_JPRB ,&
&  0.12161E+01_JPRB, 0.13481E+01_JPRB, 0.13849E+01_JPRB, 0.14620E+01_JPRB, 0.16300E+01_JPRB, 0.15982E+01_JPRB, 0.15413E+01_JPRB ,&
&  0.16154E+01_JPRB, 0.16440E+01_JPRB, 0.20453E+01_JPRB, 0.21905E+01_JPRB, 0.22392E+01_JPRB, 0.23742E+01_JPRB, 0.23815E+01_JPRB ,&
&  0.24185E+01_JPRB, 0.24297E+01_JPRB, 0.23912E+01_JPRB, 0.21712E+01_JPRB, 0.18329E+01_JPRB, 0.15607E+01_JPRB, 0.12139E+01_JPRB ,&
&  0.98411E+00_JPRB, 0.79208E+00_JPRB, 0.61265E+00_JPRB, 0.53941E+00_JPRB, 0.47424E+00_JPRB, 0.45888E+00_JPRB, 0.43634E+00_JPRB ,&
&  0.40686E+00_JPRB, 0.39710E+00_JPRB, 0.38790E+00_JPRB, 0.40383E+00_JPRB, 0.41428E+00_JPRB, 0.41375E+00_JPRB, 0.46157E+00_JPRB ,&
&  0.55673E+00_JPRB, 0.79300E+00_JPRB, 0.78818E+00_JPRB, 0.84177E+00_JPRB, 0.83065E+00_JPRB, 0.80271E+00_JPRB, 0.62512E+00_JPRB ,&
&  0.66563E+00_JPRB, 0.73919E+00_JPRB, 0.70225E+00_JPRB, 0.69446E+00_JPRB, 0.69551E+00_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.71653E+00_JPRB, 0.69742E+00_JPRB, 0.68395E+00_JPRB, 0.67008E+00_JPRB, 0.64760E+00_JPRB, 0.62282E+00_JPRB, 0.62315E+00_JPRB ,&
&  0.61856E+00_JPRB, 0.61094E+00_JPRB, 0.61119E+00_JPRB, 0.60914E+00_JPRB, 0.60132E+00_JPRB, 0.60412E+00_JPRB, 0.62170E+00_JPRB ,&
&  0.51935E+00_JPRB, 0.47936E+00_JPRB, 0.42675E+00_JPRB, 0.41270E+00_JPRB, 0.45715E+00_JPRB, 0.44084E+00_JPRB, 0.43347E+00_JPRB ,&
&  0.41626E+00_JPRB, 0.40684E+00_JPRB, 0.39451E+00_JPRB, 0.40110E+00_JPRB, 0.41725E+00_JPRB, 0.43417E+00_JPRB, 0.44782E+00_JPRB ,&
&  0.43143E+00_JPRB, 0.41115E+00_JPRB, 0.41088E+00_JPRB, 0.42849E+00_JPRB, 0.48223E+00_JPRB, 0.56229E+00_JPRB, 0.65276E+00_JPRB ,&
&  0.69285E+00_JPRB, 0.70570E+00_JPRB, 0.72749E+00_JPRB, 0.75228E+00_JPRB, 0.78449E+00_JPRB, 0.83574E+00_JPRB, 0.86948E+00_JPRB ,&
&  0.89344E+00_JPRB, 0.91452E+00_JPRB, 0.96111E+00_JPRB, 0.10308E+01_JPRB, 0.99330E+00_JPRB, 0.99486E+00_JPRB, 0.11307E+01_JPRB ,&
&  0.12955E+01_JPRB, 0.12833E+01_JPRB, 0.13468E+01_JPRB, 0.17676E+01_JPRB, 0.20228E+01_JPRB, 0.20521E+01_JPRB, 0.19630E+01_JPRB ,&
&  0.19970E+01_JPRB, 0.20811E+01_JPRB, 0.21924E+01_JPRB, 0.23349E+01_JPRB, 0.24186E+01_JPRB, 0.25222E+01_JPRB, 0.25200E+01_JPRB ,&
&  0.25138E+01_JPRB, 0.25169E+01_JPRB, 0.24255E+01_JPRB, 0.22090E+01_JPRB, 0.18025E+01_JPRB, 0.14861E+01_JPRB, 0.11462E+01_JPRB ,&
&  0.95053E+00_JPRB, 0.72806E+00_JPRB, 0.51966E+00_JPRB, 0.44123E+00_JPRB, 0.41277E+00_JPRB, 0.44203E+00_JPRB, 0.56200E+00_JPRB ,&
&  0.43846E+00_JPRB, 0.44342E+00_JPRB, 0.44787E+00_JPRB, 0.48127E+00_JPRB, 0.52187E+00_JPRB, 0.54421E+00_JPRB, 0.50704E+00_JPRB ,&
&  0.53054E+00_JPRB, 0.69487E+00_JPRB, 0.87239E+00_JPRB, 0.93861E+00_JPRB, 0.92354E+00_JPRB, 0.89903E+00_JPRB, 0.80666E+00_JPRB ,&
&  0.88823E+00_JPRB, 0.79476E+00_JPRB, 0.74408E+00_JPRB, 0.72772E+00_JPRB, 0.73532E+00_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.71360E+00_JPRB, 0.70906E+00_JPRB, 0.70668E+00_JPRB, 0.69693E+00_JPRB, 0.68668E+00_JPRB, 0.66013E+00_JPRB, 0.65827E+00_JPRB ,&
&  0.65151E+00_JPRB, 0.64385E+00_JPRB, 0.63469E+00_JPRB, 0.62923E+00_JPRB, 0.62393E+00_JPRB, 0.61791E+00_JPRB, 0.60873E+00_JPRB ,&
&  0.62865E+00_JPRB, 0.55846E+00_JPRB, 0.49059E+00_JPRB, 0.42201E+00_JPRB, 0.43594E+00_JPRB, 0.43138E+00_JPRB, 0.43268E+00_JPRB ,&
&  0.42272E+00_JPRB, 0.42337E+00_JPRB, 0.41035E+00_JPRB, 0.40178E+00_JPRB, 0.41467E+00_JPRB, 0.43472E+00_JPRB, 0.44567E+00_JPRB ,&
&  0.43628E+00_JPRB, 0.41693E+00_JPRB, 0.41527E+00_JPRB, 0.44406E+00_JPRB, 0.52979E+00_JPRB, 0.54077E+00_JPRB, 0.63874E+00_JPRB ,&
&  0.69217E+00_JPRB, 0.72512E+00_JPRB, 0.75413E+00_JPRB, 0.80115E+00_JPRB, 0.85623E+00_JPRB, 0.92856E+00_JPRB, 0.97157E+00_JPRB ,&
&  0.99678E+00_JPRB, 0.99171E+00_JPRB, 0.95615E+00_JPRB, 0.10212E+01_JPRB, 0.10818E+01_JPRB, 0.11171E+01_JPRB, 0.11629E+01_JPRB ,&
&  0.12351E+01_JPRB, 0.12725E+01_JPRB, 0.14445E+01_JPRB, 0.19248E+01_JPRB, 0.21940E+01_JPRB, 0.21989E+01_JPRB, 0.23002E+01_JPRB ,&
&  0.24380E+01_JPRB, 0.25758E+01_JPRB, 0.27069E+01_JPRB, 0.27223E+01_JPRB, 0.27405E+01_JPRB, 0.26491E+01_JPRB, 0.26103E+01_JPRB ,&
&  0.25414E+01_JPRB, 0.25547E+01_JPRB, 0.25118E+01_JPRB, 0.21764E+01_JPRB, 0.17513E+01_JPRB, 0.14002E+01_JPRB, 0.11988E+01_JPRB ,&
&  0.11247E+01_JPRB, 0.60936E+00_JPRB, 0.43820E+00_JPRB, 0.36473E+00_JPRB, 0.35387E+00_JPRB, 0.37165E+00_JPRB, 0.45206E+00_JPRB ,&
&  0.41392E+00_JPRB, 0.46567E+00_JPRB, 0.56146E+00_JPRB, 0.61252E+00_JPRB, 0.72519E+00_JPRB, 0.85574E+00_JPRB, 0.69849E+00_JPRB ,&
&  0.63296E+00_JPRB, 0.70952E+00_JPRB, 0.99280E+00_JPRB, 0.10463E+01_JPRB, 0.10090E+01_JPRB, 0.97798E+00_JPRB, 0.97843E+00_JPRB ,&
&  0.92808E+00_JPRB, 0.88821E+00_JPRB, 0.81050E+00_JPRB, 0.74942E+00_JPRB, 0.72787E+00_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.75765E+00_JPRB, 0.74322E+00_JPRB, 0.74282E+00_JPRB, 0.73801E+00_JPRB, 0.72836E+00_JPRB, 0.71078E+00_JPRB, 0.70608E+00_JPRB ,&
&  0.68519E+00_JPRB, 0.66638E+00_JPRB, 0.64915E+00_JPRB, 0.63884E+00_JPRB, 0.63303E+00_JPRB, 0.62557E+00_JPRB, 0.60806E+00_JPRB ,&
&  0.58785E+00_JPRB, 0.58389E+00_JPRB, 0.50123E+00_JPRB, 0.44343E+00_JPRB, 0.39404E+00_JPRB, 0.40384E+00_JPRB, 0.41896E+00_JPRB ,&
&  0.43915E+00_JPRB, 0.44733E+00_JPRB, 0.45861E+00_JPRB, 0.47204E+00_JPRB, 0.48817E+00_JPRB, 0.47783E+00_JPRB, 0.49616E+00_JPRB ,&
&  0.48825E+00_JPRB, 0.47211E+00_JPRB, 0.45163E+00_JPRB, 0.50859E+00_JPRB, 0.57412E+00_JPRB, 0.58034E+00_JPRB, 0.71063E+00_JPRB ,&
&  0.77936E+00_JPRB, 0.82743E+00_JPRB, 0.86525E+00_JPRB, 0.91251E+00_JPRB, 0.96304E+00_JPRB, 0.96451E+00_JPRB, 0.93250E+00_JPRB ,&
&  0.87639E+00_JPRB, 0.87450E+00_JPRB, 0.92803E+00_JPRB, 0.95853E+00_JPRB, 0.10852E+01_JPRB, 0.11373E+01_JPRB, 0.12158E+01_JPRB ,&
&  0.13398E+01_JPRB, 0.13499E+01_JPRB, 0.14225E+01_JPRB, 0.19746E+01_JPRB, 0.26129E+01_JPRB, 0.26352E+01_JPRB, 0.26959E+01_JPRB ,&
&  0.28728E+01_JPRB, 0.29150E+01_JPRB, 0.28659E+01_JPRB, 0.27300E+01_JPRB, 0.27122E+01_JPRB, 0.27267E+01_JPRB, 0.27500E+01_JPRB ,&
&  0.26234E+01_JPRB, 0.25597E+01_JPRB, 0.24577E+01_JPRB, 0.21959E+01_JPRB, 0.18174E+01_JPRB, 0.15961E+01_JPRB, 0.13819E+01_JPRB ,&
&  0.11810E+01_JPRB, 0.76797E+00_JPRB, 0.50440E+00_JPRB, 0.41919E+00_JPRB, 0.39568E+00_JPRB, 0.39739E+00_JPRB, 0.45866E+00_JPRB ,&
&  0.46808E+00_JPRB, 0.55804E+00_JPRB, 0.69187E+00_JPRB, 0.75519E+00_JPRB, 0.10839E+01_JPRB, 0.13025E+01_JPRB, 0.12232E+01_JPRB ,&
&  0.10224E+01_JPRB, 0.91697E+00_JPRB, 0.11712E+01_JPRB, 0.11835E+01_JPRB, 0.11419E+01_JPRB, 0.10483E+01_JPRB, 0.10240E+01_JPRB ,&
&  0.97570E+00_JPRB, 0.94563E+00_JPRB, 0.90061E+00_JPRB, 0.82957E+00_JPRB, 0.78080E+00_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.79480E+00_JPRB, 0.75931E+00_JPRB, 0.73902E+00_JPRB, 0.72970E+00_JPRB, 0.70462E+00_JPRB, 0.68735E+00_JPRB, 0.69917E+00_JPRB ,&
&  0.70384E+00_JPRB, 0.68240E+00_JPRB, 0.66415E+00_JPRB, 0.65793E+00_JPRB, 0.65746E+00_JPRB, 0.65066E+00_JPRB, 0.61589E+00_JPRB ,&
&  0.57378E+00_JPRB, 0.54677E+00_JPRB, 0.46111E+00_JPRB, 0.40743E+00_JPRB, 0.38444E+00_JPRB, 0.40352E+00_JPRB, 0.44264E+00_JPRB ,&
&  0.49383E+00_JPRB, 0.54162E+00_JPRB, 0.59447E+00_JPRB, 0.61234E+00_JPRB, 0.63030E+00_JPRB, 0.62789E+00_JPRB, 0.63028E+00_JPRB ,&
&  0.66866E+00_JPRB, 0.57043E+00_JPRB, 0.58735E+00_JPRB, 0.60837E+00_JPRB, 0.68336E+00_JPRB, 0.73728E+00_JPRB, 0.78890E+00_JPRB ,&
&  0.79922E+00_JPRB, 0.83231E+00_JPRB, 0.86003E+00_JPRB, 0.88679E+00_JPRB, 0.90720E+00_JPRB, 0.90680E+00_JPRB, 0.88801E+00_JPRB ,&
&  0.90781E+00_JPRB, 0.88957E+00_JPRB, 0.88769E+00_JPRB, 0.10810E+01_JPRB, 0.15022E+01_JPRB, 0.16850E+01_JPRB, 0.17157E+01_JPRB ,&
&  0.16288E+01_JPRB, 0.17083E+01_JPRB, 0.24899E+01_JPRB, 0.28167E+01_JPRB, 0.29444E+01_JPRB, 0.32856E+01_JPRB, 0.33861E+01_JPRB ,&
&  0.32370E+01_JPRB, 0.29353E+01_JPRB, 0.26670E+01_JPRB, 0.24075E+01_JPRB, 0.28682E+01_JPRB, 0.30323E+01_JPRB, 0.29882E+01_JPRB ,&
&  0.26530E+01_JPRB, 0.25566E+01_JPRB, 0.23007E+01_JPRB, 0.20615E+01_JPRB, 0.18911E+01_JPRB, 0.18893E+01_JPRB, 0.14999E+01_JPRB ,&
&  0.11411E+01_JPRB, 0.10314E+01_JPRB, 0.77910E+00_JPRB, 0.65465E+00_JPRB, 0.60548E+00_JPRB, 0.59915E+00_JPRB, 0.62206E+00_JPRB ,&
&  0.66697E+00_JPRB, 0.75516E+00_JPRB, 0.88835E+00_JPRB, 0.10397E+01_JPRB, 0.14100E+01_JPRB, 0.16942E+01_JPRB, 0.17205E+01_JPRB ,&
&  0.14235E+01_JPRB, 0.14516E+01_JPRB, 0.13747E+01_JPRB, 0.13200E+01_JPRB, 0.12841E+01_JPRB, 0.11581E+01_JPRB, 0.10949E+01_JPRB ,&
&  0.10086E+01_JPRB, 0.96683E+00_JPRB, 0.94169E+00_JPRB, 0.88788E+00_JPRB, 0.84615E+00_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.87121E+00_JPRB, 0.82965E+00_JPRB, 0.77276E+00_JPRB, 0.73294E+00_JPRB, 0.69985E+00_JPRB, 0.68214E+00_JPRB, 0.67901E+00_JPRB ,&
&  0.67852E+00_JPRB, 0.67361E+00_JPRB, 0.68226E+00_JPRB, 0.69395E+00_JPRB, 0.69724E+00_JPRB, 0.67633E+00_JPRB, 0.61762E+00_JPRB ,&
&  0.56857E+00_JPRB, 0.49686E+00_JPRB, 0.43347E+00_JPRB, 0.42858E+00_JPRB, 0.45428E+00_JPRB, 0.47739E+00_JPRB, 0.51624E+00_JPRB ,&
&  0.58540E+00_JPRB, 0.70625E+00_JPRB, 0.77680E+00_JPRB, 0.85207E+00_JPRB, 0.88326E+00_JPRB, 0.87499E+00_JPRB, 0.84270E+00_JPRB ,&
&  0.77560E+00_JPRB, 0.78414E+00_JPRB, 0.83433E+00_JPRB, 0.82934E+00_JPRB, 0.80979E+00_JPRB, 0.81202E+00_JPRB, 0.82123E+00_JPRB ,&
&  0.82096E+00_JPRB, 0.82345E+00_JPRB, 0.86456E+00_JPRB, 0.88781E+00_JPRB, 0.94016E+00_JPRB, 0.97037E+00_JPRB, 0.98793E+00_JPRB ,&
&  0.94658E+00_JPRB, 0.93480E+00_JPRB, 0.10694E+01_JPRB, 0.15867E+01_JPRB, 0.20489E+01_JPRB, 0.24018E+01_JPRB, 0.23330E+01_JPRB ,&
&  0.25908E+01_JPRB, 0.27986E+01_JPRB, 0.29356E+01_JPRB, 0.33110E+01_JPRB, 0.26910E+01_JPRB, 0.29485E+01_JPRB, 0.33388E+01_JPRB ,&
&  0.32143E+01_JPRB, 0.29112E+01_JPRB, 0.27481E+01_JPRB, 0.26388E+01_JPRB, 0.29210E+01_JPRB, 0.38742E+01_JPRB, 0.35631E+01_JPRB ,&
&  0.29627E+01_JPRB, 0.26411E+01_JPRB, 0.22852E+01_JPRB, 0.21957E+01_JPRB, 0.18915E+01_JPRB, 0.16488E+01_JPRB, 0.11717E+01_JPRB ,&
&  0.11372E+01_JPRB, 0.10033E+01_JPRB, 0.99268E+00_JPRB, 0.91234E+00_JPRB, 0.91065E+00_JPRB, 0.86868E+00_JPRB, 0.85480E+00_JPRB ,&
&  0.92980E+00_JPRB, 0.10143E+01_JPRB, 0.13058E+01_JPRB, 0.15735E+01_JPRB, 0.21924E+01_JPRB, 0.21182E+01_JPRB, 0.21452E+01_JPRB ,&
&  0.19583E+01_JPRB, 0.18529E+01_JPRB, 0.15478E+01_JPRB, 0.15287E+01_JPRB, 0.13848E+01_JPRB, 0.12735E+01_JPRB, 0.11849E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.97867E+00_JPRB, 0.92513E+00_JPRB, 0.88821E+00_JPRB, 0.88545E+00_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.83858E+00_JPRB, 0.80240E+00_JPRB, 0.76195E+00_JPRB, 0.72233E+00_JPRB, 0.68158E+00_JPRB, 0.66436E+00_JPRB, 0.66201E+00_JPRB ,&
&  0.66600E+00_JPRB, 0.67993E+00_JPRB, 0.68621E+00_JPRB, 0.68718E+00_JPRB, 0.68611E+00_JPRB, 0.68807E+00_JPRB, 0.62714E+00_JPRB ,&
&  0.56887E+00_JPRB, 0.50736E+00_JPRB, 0.44050E+00_JPRB, 0.43202E+00_JPRB, 0.50717E+00_JPRB, 0.56797E+00_JPRB, 0.55923E+00_JPRB ,&
&  0.74481E+00_JPRB, 0.93193E+00_JPRB, 0.10006E+01_JPRB, 0.10915E+01_JPRB, 0.11840E+01_JPRB, 0.12187E+01_JPRB, 0.11232E+01_JPRB ,&
&  0.10733E+01_JPRB, 0.10351E+01_JPRB, 0.94263E+00_JPRB, 0.86987E+00_JPRB, 0.82270E+00_JPRB, 0.79814E+00_JPRB, 0.77208E+00_JPRB ,&
&  0.76289E+00_JPRB, 0.77578E+00_JPRB, 0.79473E+00_JPRB, 0.82679E+00_JPRB, 0.89036E+00_JPRB, 0.86723E+00_JPRB, 0.91192E+00_JPRB ,&
&  0.96678E+00_JPRB, 0.10488E+01_JPRB, 0.13645E+01_JPRB, 0.20022E+01_JPRB, 0.27796E+01_JPRB, 0.32764E+01_JPRB, 0.33596E+01_JPRB ,&
&  0.31706E+01_JPRB, 0.31631E+01_JPRB, 0.33111E+01_JPRB, 0.33146E+01_JPRB, 0.32573E+01_JPRB, 0.32414E+01_JPRB, 0.34539E+01_JPRB ,&
&  0.27018E+01_JPRB, 0.22293E+01_JPRB, 0.19643E+01_JPRB, 0.22954E+01_JPRB, 0.36154E+01_JPRB, 0.46584E+01_JPRB, 0.46011E+01_JPRB ,&
&  0.38530E+01_JPRB, 0.32137E+01_JPRB, 0.27058E+01_JPRB, 0.23088E+01_JPRB, 0.22226E+01_JPRB, 0.15266E+01_JPRB, 0.16012E+01_JPRB ,&
&  0.15927E+01_JPRB, 0.15832E+01_JPRB, 0.14342E+01_JPRB, 0.11988E+01_JPRB, 0.10691E+01_JPRB, 0.11658E+01_JPRB, 0.12198E+01_JPRB ,&
&  0.13357E+01_JPRB, 0.14637E+01_JPRB, 0.22198E+01_JPRB, 0.25957E+01_JPRB, 0.29020E+01_JPRB, 0.25936E+01_JPRB, 0.23859E+01_JPRB ,&
&  0.19961E+01_JPRB, 0.19230E+01_JPRB, 0.18501E+01_JPRB, 0.17099E+01_JPRB, 0.15459E+01_JPRB, 0.14088E+01_JPRB, 0.12718E+01_JPRB ,&
&  0.11502E+01_JPRB, 0.10358E+01_JPRB, 0.93926E+00_JPRB, 0.87604E+00_JPRB, 0.85686E+00_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.84879E+00_JPRB, 0.81711E+00_JPRB, 0.77158E+00_JPRB, 0.70656E+00_JPRB, 0.67605E+00_JPRB, 0.67681E+00_JPRB, 0.69642E+00_JPRB ,&
&  0.70712E+00_JPRB, 0.70977E+00_JPRB, 0.69693E+00_JPRB, 0.67950E+00_JPRB, 0.67953E+00_JPRB, 0.66804E+00_JPRB, 0.62777E+00_JPRB ,&
&  0.59449E+00_JPRB, 0.55932E+00_JPRB, 0.47481E+00_JPRB, 0.48291E+00_JPRB, 0.58219E+00_JPRB, 0.67680E+00_JPRB, 0.68371E+00_JPRB ,&
&  0.10038E+01_JPRB, 0.11115E+01_JPRB, 0.10653E+01_JPRB, 0.11431E+01_JPRB, 0.14208E+01_JPRB, 0.13314E+01_JPRB, 0.12109E+01_JPRB ,&
&  0.11786E+01_JPRB, 0.10946E+01_JPRB, 0.96863E+00_JPRB, 0.87287E+00_JPRB, 0.81295E+00_JPRB, 0.77972E+00_JPRB, 0.75513E+00_JPRB ,&
&  0.75082E+00_JPRB, 0.76148E+00_JPRB, 0.81251E+00_JPRB, 0.87045E+00_JPRB, 0.85264E+00_JPRB, 0.83872E+00_JPRB, 0.92910E+00_JPRB ,&
&  0.10231E+01_JPRB, 0.11378E+01_JPRB, 0.14787E+01_JPRB, 0.21116E+01_JPRB, 0.30056E+01_JPRB, 0.37897E+01_JPRB, 0.47052E+01_JPRB ,&
&  0.41590E+01_JPRB, 0.42122E+01_JPRB, 0.37910E+01_JPRB, 0.43326E+01_JPRB, 0.38899E+01_JPRB, 0.36045E+01_JPRB, 0.34653E+01_JPRB ,&
&  0.24946E+01_JPRB, 0.21032E+01_JPRB, 0.23664E+01_JPRB, 0.35001E+01_JPRB, 0.38555E+01_JPRB, 0.43439E+01_JPRB, 0.52892E+01_JPRB ,&
&  0.41847E+01_JPRB, 0.34647E+01_JPRB, 0.30800E+01_JPRB, 0.29865E+01_JPRB, 0.19152E+01_JPRB, 0.14403E+01_JPRB, 0.20828E+01_JPRB ,&
&  0.19467E+01_JPRB, 0.11646E+01_JPRB, 0.88833E+00_JPRB, 0.10147E+01_JPRB, 0.80333E+00_JPRB, 0.94564E+00_JPRB, 0.13695E+01_JPRB ,&
&  0.15787E+01_JPRB, 0.18380E+01_JPRB, 0.28585E+01_JPRB, 0.27688E+01_JPRB, 0.28589E+01_JPRB, 0.29381E+01_JPRB, 0.24316E+01_JPRB ,&
&  0.21784E+01_JPRB, 0.18107E+01_JPRB, 0.18464E+01_JPRB, 0.16904E+01_JPRB, 0.15702E+01_JPRB, 0.14774E+01_JPRB, 0.13362E+01_JPRB ,&
&  0.12058E+01_JPRB, 0.10755E+01_JPRB, 0.96585E+00_JPRB, 0.89469E+00_JPRB, 0.86854E+00_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.84070E+00_JPRB, 0.80832E+00_JPRB, 0.79045E+00_JPRB, 0.75393E+00_JPRB, 0.72581E+00_JPRB, 0.72012E+00_JPRB, 0.73771E+00_JPRB ,&
&  0.72300E+00_JPRB, 0.69048E+00_JPRB, 0.65626E+00_JPRB, 0.64521E+00_JPRB, 0.65441E+00_JPRB, 0.65419E+00_JPRB, 0.62124E+00_JPRB ,&
&  0.59301E+00_JPRB, 0.56776E+00_JPRB, 0.56386E+00_JPRB, 0.59924E+00_JPRB, 0.75801E+00_JPRB, 0.79779E+00_JPRB, 0.96769E+00_JPRB ,&
&  0.12369E+01_JPRB, 0.12004E+01_JPRB, 0.11408E+01_JPRB, 0.12098E+01_JPRB, 0.12031E+01_JPRB, 0.11328E+01_JPRB, 0.11711E+01_JPRB ,&
&  0.11231E+01_JPRB, 0.10551E+01_JPRB, 0.95742E+00_JPRB, 0.88562E+00_JPRB, 0.81609E+00_JPRB, 0.76752E+00_JPRB, 0.75536E+00_JPRB ,&
&  0.75950E+00_JPRB, 0.80471E+00_JPRB, 0.89436E+00_JPRB, 0.85282E+00_JPRB, 0.84292E+00_JPRB, 0.85559E+00_JPRB, 0.96089E+00_JPRB ,&
&  0.10809E+01_JPRB, 0.12268E+01_JPRB, 0.15359E+01_JPRB, 0.22486E+01_JPRB, 0.37400E+01_JPRB, 0.49063E+01_JPRB, 0.48671E+01_JPRB ,&
&  0.43877E+01_JPRB, 0.42027E+01_JPRB, 0.43889E+01_JPRB, 0.45356E+01_JPRB, 0.38853E+01_JPRB, 0.36840E+01_JPRB, 0.33676E+01_JPRB ,&
&  0.28985E+01_JPRB, 0.26639E+01_JPRB, 0.26963E+01_JPRB, 0.37409E+01_JPRB, 0.45097E+01_JPRB, 0.45760E+01_JPRB, 0.51933E+01_JPRB ,&
&  0.46247E+01_JPRB, 0.37385E+01_JPRB, 0.28080E+01_JPRB, 0.22246E+01_JPRB, 0.22024E+01_JPRB, 0.18888E+01_JPRB, 0.13833E+01_JPRB ,&
&  0.11765E+01_JPRB, 0.97776E+00_JPRB, 0.82091E+00_JPRB, 0.80758E+00_JPRB, 0.70239E+00_JPRB, 0.81329E+00_JPRB, 0.12140E+01_JPRB ,&
&  0.17481E+01_JPRB, 0.20980E+01_JPRB, 0.26588E+01_JPRB, 0.27148E+01_JPRB, 0.27725E+01_JPRB, 0.31643E+01_JPRB, 0.30390E+01_JPRB ,&
&  0.19515E+01_JPRB, 0.17680E+01_JPRB, 0.18046E+01_JPRB, 0.17007E+01_JPRB, 0.15841E+01_JPRB, 0.14633E+01_JPRB, 0.13092E+01_JPRB ,&
&  0.11600E+01_JPRB, 0.10273E+01_JPRB, 0.94861E+00_JPRB, 0.92132E+00_JPRB, 0.89045E+00_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.77313E+00_JPRB, 0.76746E+00_JPRB, 0.76954E+00_JPRB, 0.75875E+00_JPRB, 0.74238E+00_JPRB, 0.74646E+00_JPRB, 0.72479E+00_JPRB ,&
&  0.68158E+00_JPRB, 0.64211E+00_JPRB, 0.62011E+00_JPRB, 0.62510E+00_JPRB, 0.63587E+00_JPRB, 0.64159E+00_JPRB, 0.61962E+00_JPRB ,&
&  0.58914E+00_JPRB, 0.58208E+00_JPRB, 0.62053E+00_JPRB, 0.72076E+00_JPRB, 0.10100E+01_JPRB, 0.99750E+00_JPRB, 0.12921E+01_JPRB ,&
&  0.14102E+01_JPRB, 0.12332E+01_JPRB, 0.11885E+01_JPRB, 0.11515E+01_JPRB, 0.11477E+01_JPRB, 0.10611E+01_JPRB, 0.10571E+01_JPRB ,&
&  0.96821E+00_JPRB, 0.97597E+00_JPRB, 0.88201E+00_JPRB, 0.82641E+00_JPRB, 0.77979E+00_JPRB, 0.76371E+00_JPRB, 0.78484E+00_JPRB ,&
&  0.83545E+00_JPRB, 0.90744E+00_JPRB, 0.88526E+00_JPRB, 0.86480E+00_JPRB, 0.89427E+00_JPRB, 0.92957E+00_JPRB, 0.10344E+01_JPRB ,&
&  0.11694E+01_JPRB, 0.13418E+01_JPRB, 0.16926E+01_JPRB, 0.28475E+01_JPRB, 0.39438E+01_JPRB, 0.42396E+01_JPRB, 0.47455E+01_JPRB ,&
&  0.49061E+01_JPRB, 0.49736E+01_JPRB, 0.42950E+01_JPRB, 0.47698E+01_JPRB, 0.39185E+01_JPRB, 0.35122E+01_JPRB, 0.32939E+01_JPRB ,&
&  0.29471E+01_JPRB, 0.31930E+01_JPRB, 0.30878E+01_JPRB, 0.33129E+01_JPRB, 0.45454E+01_JPRB, 0.52294E+01_JPRB, 0.51309E+01_JPRB ,&
&  0.48817E+01_JPRB, 0.40262E+01_JPRB, 0.31492E+01_JPRB, 0.26973E+01_JPRB, 0.33308E+01_JPRB, 0.34544E+01_JPRB, 0.16517E+01_JPRB ,&
&  0.11522E+01_JPRB, 0.10549E+01_JPRB, 0.80156E+00_JPRB, 0.68644E+00_JPRB, 0.65336E+00_JPRB, 0.78806E+00_JPRB, 0.13462E+01_JPRB ,&
&  0.15127E+01_JPRB, 0.23821E+01_JPRB, 0.26066E+01_JPRB, 0.27860E+01_JPRB, 0.26601E+01_JPRB, 0.26403E+01_JPRB, 0.28024E+01_JPRB ,&
&  0.28153E+01_JPRB, 0.24407E+01_JPRB, 0.20025E+01_JPRB, 0.17582E+01_JPRB, 0.15371E+01_JPRB, 0.13425E+01_JPRB, 0.11997E+01_JPRB ,&
&  0.11077E+01_JPRB, 0.10079E+01_JPRB, 0.94803E+00_JPRB, 0.90845E+00_JPRB, 0.82893E+00_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.69579E+00_JPRB, 0.71388E+00_JPRB, 0.73651E+00_JPRB, 0.72888E+00_JPRB, 0.70993E+00_JPRB, 0.71257E+00_JPRB, 0.67175E+00_JPRB ,&
&  0.63418E+00_JPRB, 0.61551E+00_JPRB, 0.60942E+00_JPRB, 0.62220E+00_JPRB, 0.65056E+00_JPRB, 0.67070E+00_JPRB, 0.65181E+00_JPRB ,&
&  0.61717E+00_JPRB, 0.61586E+00_JPRB, 0.66040E+00_JPRB, 0.94761E+00_JPRB, 0.12150E+01_JPRB, 0.14182E+01_JPRB, 0.15795E+01_JPRB ,&
&  0.15105E+01_JPRB, 0.13228E+01_JPRB, 0.12256E+01_JPRB, 0.12196E+01_JPRB, 0.11885E+01_JPRB, 0.11502E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.93764E+00_JPRB, 0.93939E+00_JPRB, 0.82772E+00_JPRB, 0.79255E+00_JPRB, 0.78101E+00_JPRB, 0.79668E+00_JPRB, 0.85144E+00_JPRB ,&
&  0.92038E+00_JPRB, 0.90550E+00_JPRB, 0.92398E+00_JPRB, 0.96045E+00_JPRB, 0.10193E+01_JPRB, 0.10580E+01_JPRB, 0.11464E+01_JPRB ,&
&  0.13074E+01_JPRB, 0.15646E+01_JPRB, 0.21026E+01_JPRB, 0.33931E+01_JPRB, 0.35755E+01_JPRB, 0.46107E+01_JPRB, 0.47843E+01_JPRB ,&
&  0.47820E+01_JPRB, 0.46659E+01_JPRB, 0.41458E+01_JPRB, 0.42897E+01_JPRB, 0.38325E+01_JPRB, 0.34766E+01_JPRB, 0.31694E+01_JPRB ,&
&  0.33175E+01_JPRB, 0.31943E+01_JPRB, 0.38699E+01_JPRB, 0.37358E+01_JPRB, 0.42051E+01_JPRB, 0.67340E+01_JPRB, 0.52193E+01_JPRB ,&
&  0.43330E+01_JPRB, 0.44395E+01_JPRB, 0.35678E+01_JPRB, 0.31364E+01_JPRB, 0.36958E+01_JPRB, 0.42307E+01_JPRB, 0.33477E+01_JPRB ,&
&  0.13688E+01_JPRB, 0.94735E+00_JPRB, 0.81732E+00_JPRB, 0.71749E+00_JPRB, 0.62010E+00_JPRB, 0.75119E+00_JPRB, 0.22976E+01_JPRB ,&
&  0.17122E+01_JPRB, 0.23708E+01_JPRB, 0.24480E+01_JPRB, 0.24611E+01_JPRB, 0.25836E+01_JPRB, 0.23542E+01_JPRB, 0.24811E+01_JPRB ,&
&  0.21126E+01_JPRB, 0.17191E+01_JPRB, 0.14328E+01_JPRB, 0.12378E+01_JPRB, 0.11347E+01_JPRB, 0.10828E+01_JPRB, 0.10568E+01_JPRB ,&
&  0.10238E+01_JPRB, 0.94757E+00_JPRB, 0.87613E+00_JPRB, 0.78258E+00_JPRB, 0.70152E+00_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.69951E+00_JPRB, 0.69978E+00_JPRB, 0.70561E+00_JPRB, 0.72514E+00_JPRB, 0.74067E+00_JPRB, 0.72102E+00_JPRB, 0.68356E+00_JPRB ,&
&  0.68797E+00_JPRB, 0.68301E+00_JPRB, 0.70861E+00_JPRB, 0.72858E+00_JPRB, 0.75531E+00_JPRB, 0.74695E+00_JPRB, 0.71981E+00_JPRB ,&
&  0.67771E+00_JPRB, 0.68494E+00_JPRB, 0.84914E+00_JPRB, 0.11830E+01_JPRB, 0.13791E+01_JPRB, 0.15020E+01_JPRB, 0.17110E+01_JPRB ,&
&  0.17313E+01_JPRB, 0.14154E+01_JPRB, 0.11991E+01_JPRB, 0.12539E+01_JPRB, 0.12751E+01_JPRB, 0.12967E+01_JPRB, 0.11239E+01_JPRB ,&
&  0.99460E+00_JPRB, 0.95213E+00_JPRB, 0.84611E+00_JPRB, 0.83409E+00_JPRB, 0.84505E+00_JPRB, 0.87213E+00_JPRB, 0.92095E+00_JPRB ,&
&  0.95332E+00_JPRB, 0.10156E+01_JPRB, 0.10978E+01_JPRB, 0.11664E+01_JPRB, 0.12172E+01_JPRB, 0.12510E+01_JPRB, 0.13446E+01_JPRB ,&
&  0.15513E+01_JPRB, 0.18949E+01_JPRB, 0.24974E+01_JPRB, 0.33364E+01_JPRB, 0.40355E+01_JPRB, 0.42688E+01_JPRB, 0.45682E+01_JPRB ,&
&  0.45662E+01_JPRB, 0.45119E+01_JPRB, 0.42607E+01_JPRB, 0.39332E+01_JPRB, 0.35383E+01_JPRB, 0.34521E+01_JPRB, 0.32164E+01_JPRB ,&
&  0.35223E+01_JPRB, 0.38844E+01_JPRB, 0.44623E+01_JPRB, 0.43094E+01_JPRB, 0.45099E+01_JPRB, 0.63464E+01_JPRB, 0.67028E+01_JPRB ,&
&  0.53807E+01_JPRB, 0.38942E+01_JPRB, 0.32783E+01_JPRB, 0.33863E+01_JPRB, 0.36221E+01_JPRB, 0.37559E+01_JPRB, 0.47593E+01_JPRB ,&
&  0.36963E+01_JPRB, 0.16728E+01_JPRB, 0.97124E+00_JPRB, 0.94189E+00_JPRB, 0.58337E+00_JPRB, 0.55159E+00_JPRB, 0.10374E+01_JPRB ,&
&  0.14226E+01_JPRB, 0.18227E+01_JPRB, 0.19045E+01_JPRB, 0.21150E+01_JPRB, 0.21538E+01_JPRB, 0.18223E+01_JPRB, 0.14485E+01_JPRB ,&
&  0.11326E+01_JPRB, 0.96740E+00_JPRB, 0.88717E+00_JPRB, 0.84933E+00_JPRB, 0.88126E+00_JPRB, 0.93110E+00_JPRB, 0.96079E+00_JPRB ,&
&  0.94008E+00_JPRB, 0.86893E+00_JPRB, 0.76447E+00_JPRB, 0.68889E+00_JPRB, 0.67677E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.77751E+00_JPRB, 0.76670E+00_JPRB, 0.77028E+00_JPRB, 0.79879E+00_JPRB, 0.81142E+00_JPRB, 0.81824E+00_JPRB, 0.87613E+00_JPRB ,&
&  0.90202E+00_JPRB, 0.90496E+00_JPRB, 0.91789E+00_JPRB, 0.91783E+00_JPRB, 0.89773E+00_JPRB, 0.84680E+00_JPRB, 0.80799E+00_JPRB ,&
&  0.80695E+00_JPRB, 0.93778E+00_JPRB, 0.12215E+01_JPRB, 0.15632E+01_JPRB, 0.16900E+01_JPRB, 0.20777E+01_JPRB, 0.16529E+01_JPRB ,&
&  0.16272E+01_JPRB, 0.17032E+01_JPRB, 0.13321E+01_JPRB, 0.13803E+01_JPRB, 0.13901E+01_JPRB, 0.14533E+01_JPRB, 0.11980E+01_JPRB ,&
&  0.10821E+01_JPRB, 0.10300E+01_JPRB, 0.93098E+00_JPRB, 0.91557E+00_JPRB, 0.94120E+00_JPRB, 0.99647E+00_JPRB, 0.10465E+01_JPRB ,&
&  0.11364E+01_JPRB, 0.12681E+01_JPRB, 0.13843E+01_JPRB, 0.14427E+01_JPRB, 0.14785E+01_JPRB, 0.15197E+01_JPRB, 0.16570E+01_JPRB ,&
&  0.18814E+01_JPRB, 0.22501E+01_JPRB, 0.28548E+01_JPRB, 0.35114E+01_JPRB, 0.39308E+01_JPRB, 0.39380E+01_JPRB, 0.41593E+01_JPRB ,&
&  0.39907E+01_JPRB, 0.37319E+01_JPRB, 0.37976E+01_JPRB, 0.36804E+01_JPRB, 0.34176E+01_JPRB, 0.32601E+01_JPRB, 0.31016E+01_JPRB ,&
&  0.38266E+01_JPRB, 0.49640E+01_JPRB, 0.62797E+01_JPRB, 0.54382E+01_JPRB, 0.53542E+01_JPRB, 0.65261E+01_JPRB, 0.67255E+01_JPRB ,&
&  0.57822E+01_JPRB, 0.42584E+01_JPRB, 0.37477E+01_JPRB, 0.33169E+01_JPRB, 0.31543E+01_JPRB, 0.30990E+01_JPRB, 0.35747E+01_JPRB ,&
&  0.36175E+01_JPRB, 0.34887E+01_JPRB, 0.18565E+01_JPRB, 0.84967E+00_JPRB, 0.61730E+00_JPRB, 0.67702E+00_JPRB, 0.84718E+00_JPRB ,&
&  0.11295E+01_JPRB, 0.13291E+01_JPRB, 0.14370E+01_JPRB, 0.18226E+01_JPRB, 0.16294E+01_JPRB, 0.13180E+01_JPRB, 0.10632E+01_JPRB ,&
&  0.91124E+00_JPRB, 0.79856E+00_JPRB, 0.74729E+00_JPRB, 0.75070E+00_JPRB, 0.81308E+00_JPRB, 0.88183E+00_JPRB, 0.90973E+00_JPRB ,&
&  0.85686E+00_JPRB, 0.77262E+00_JPRB, 0.73541E+00_JPRB, 0.73343E+00_JPRB, 0.75358E+00_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.89226E+00_JPRB, 0.91037E+00_JPRB, 0.95148E+00_JPRB, 0.10151E+01_JPRB, 0.10754E+01_JPRB, 0.11394E+01_JPRB, 0.11802E+01_JPRB ,&
&  0.11871E+01_JPRB, 0.11634E+01_JPRB, 0.11479E+01_JPRB, 0.11041E+01_JPRB, 0.10767E+01_JPRB, 0.10657E+01_JPRB, 0.11225E+01_JPRB ,&
&  0.12991E+01_JPRB, 0.15636E+01_JPRB, 0.18157E+01_JPRB, 0.19449E+01_JPRB, 0.19587E+01_JPRB, 0.21516E+01_JPRB, 0.20929E+01_JPRB ,&
&  0.15553E+01_JPRB, 0.21557E+01_JPRB, 0.15064E+01_JPRB, 0.15515E+01_JPRB, 0.15907E+01_JPRB, 0.17244E+01_JPRB, 0.14428E+01_JPRB ,&
&  0.12463E+01_JPRB, 0.11960E+01_JPRB, 0.11521E+01_JPRB, 0.11398E+01_JPRB, 0.11588E+01_JPRB, 0.12257E+01_JPRB, 0.13407E+01_JPRB ,&
&  0.14640E+01_JPRB, 0.15671E+01_JPRB, 0.16461E+01_JPRB, 0.16956E+01_JPRB, 0.17567E+01_JPRB, 0.18334E+01_JPRB, 0.20272E+01_JPRB ,&
&  0.22848E+01_JPRB, 0.26967E+01_JPRB, 0.31871E+01_JPRB, 0.37015E+01_JPRB, 0.36913E+01_JPRB, 0.36496E+01_JPRB, 0.37695E+01_JPRB ,&
&  0.36561E+01_JPRB, 0.34208E+01_JPRB, 0.34589E+01_JPRB, 0.32565E+01_JPRB, 0.30357E+01_JPRB, 0.31744E+01_JPRB, 0.32159E+01_JPRB ,&
&  0.44314E+01_JPRB, 0.52903E+01_JPRB, 0.71569E+01_JPRB, 0.65979E+01_JPRB, 0.57683E+01_JPRB, 0.60358E+01_JPRB, 0.51956E+01_JPRB ,&
&  0.39968E+01_JPRB, 0.35772E+01_JPRB, 0.32646E+01_JPRB, 0.29418E+01_JPRB, 0.27477E+01_JPRB, 0.26206E+01_JPRB, 0.27910E+01_JPRB ,&
&  0.28163E+01_JPRB, 0.26496E+01_JPRB, 0.19210E+01_JPRB, 0.96731E+00_JPRB, 0.72521E+00_JPRB, 0.69419E+00_JPRB, 0.82095E+00_JPRB ,&
&  0.10890E+01_JPRB, 0.12411E+01_JPRB, 0.12952E+01_JPRB, 0.12110E+01_JPRB, 0.10218E+01_JPRB, 0.86418E+00_JPRB, 0.75391E+00_JPRB ,&
&  0.72223E+00_JPRB, 0.71682E+00_JPRB, 0.72411E+00_JPRB, 0.78144E+00_JPRB, 0.84637E+00_JPRB, 0.88267E+00_JPRB, 0.82086E+00_JPRB ,&
&  0.77547E+00_JPRB, 0.77237E+00_JPRB, 0.80870E+00_JPRB, 0.84385E+00_JPRB, 0.87468E+00_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.10564E+01_JPRB, 0.11161E+01_JPRB, 0.12017E+01_JPRB, 0.13321E+01_JPRB, 0.15090E+01_JPRB, 0.16876E+01_JPRB, 0.16141E+01_JPRB ,&
&  0.15141E+01_JPRB, 0.13846E+01_JPRB, 0.14303E+01_JPRB, 0.14757E+01_JPRB, 0.15217E+01_JPRB, 0.16068E+01_JPRB, 0.17339E+01_JPRB ,&
&  0.18812E+01_JPRB, 0.20210E+01_JPRB, 0.21520E+01_JPRB, 0.22717E+01_JPRB, 0.23938E+01_JPRB, 0.27311E+01_JPRB, 0.28052E+01_JPRB ,&
&  0.23135E+01_JPRB, 0.16710E+01_JPRB, 0.18722E+01_JPRB, 0.16576E+01_JPRB, 0.16854E+01_JPRB, 0.18174E+01_JPRB, 0.17628E+01_JPRB ,&
&  0.15413E+01_JPRB, 0.14611E+01_JPRB, 0.14269E+01_JPRB, 0.14305E+01_JPRB, 0.14328E+01_JPRB, 0.15288E+01_JPRB, 0.16325E+01_JPRB ,&
&  0.17173E+01_JPRB, 0.17795E+01_JPRB, 0.18501E+01_JPRB, 0.19145E+01_JPRB, 0.19923E+01_JPRB, 0.21182E+01_JPRB, 0.23566E+01_JPRB ,&
&  0.26183E+01_JPRB, 0.29573E+01_JPRB, 0.33089E+01_JPRB, 0.34925E+01_JPRB, 0.34369E+01_JPRB, 0.33871E+01_JPRB, 0.34078E+01_JPRB ,&
&  0.33735E+01_JPRB, 0.33425E+01_JPRB, 0.32603E+01_JPRB, 0.31143E+01_JPRB, 0.29435E+01_JPRB, 0.33805E+01_JPRB, 0.38706E+01_JPRB ,&
&  0.49485E+01_JPRB, 0.56819E+01_JPRB, 0.72523E+01_JPRB, 0.63593E+01_JPRB, 0.54990E+01_JPRB, 0.49303E+01_JPRB, 0.41611E+01_JPRB ,&
&  0.33062E+01_JPRB, 0.31836E+01_JPRB, 0.29762E+01_JPRB, 0.25052E+01_JPRB, 0.21433E+01_JPRB, 0.18100E+01_JPRB, 0.20890E+01_JPRB ,&
&  0.21992E+01_JPRB, 0.22315E+01_JPRB, 0.14852E+01_JPRB, 0.10429E+01_JPRB, 0.80318E+00_JPRB, 0.74166E+00_JPRB, 0.87044E+00_JPRB ,&
&  0.11238E+01_JPRB, 0.11564E+01_JPRB, 0.10292E+01_JPRB, 0.98144E+00_JPRB, 0.87005E+00_JPRB, 0.73978E+00_JPRB, 0.64187E+00_JPRB ,&
&  0.66495E+00_JPRB, 0.68263E+00_JPRB, 0.71690E+00_JPRB, 0.77733E+00_JPRB, 0.80711E+00_JPRB, 0.84830E+00_JPRB, 0.84779E+00_JPRB ,&
&  0.88587E+00_JPRB, 0.92387E+00_JPRB, 0.94847E+00_JPRB, 0.97605E+00_JPRB, 0.10085E+01_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.11305E+01_JPRB, 0.11888E+01_JPRB, 0.12635E+01_JPRB, 0.13582E+01_JPRB, 0.14576E+01_JPRB, 0.15087E+01_JPRB, 0.15267E+01_JPRB ,&
&  0.15864E+01_JPRB, 0.16699E+01_JPRB, 0.17652E+01_JPRB, 0.18373E+01_JPRB, 0.18743E+01_JPRB, 0.19416E+01_JPRB, 0.20192E+01_JPRB ,&
&  0.20988E+01_JPRB, 0.21944E+01_JPRB, 0.23183E+01_JPRB, 0.24691E+01_JPRB, 0.26818E+01_JPRB, 0.29544E+01_JPRB, 0.30950E+01_JPRB ,&
&  0.24414E+01_JPRB, 0.16299E+01_JPRB, 0.19880E+01_JPRB, 0.19233E+01_JPRB, 0.18097E+01_JPRB, 0.18094E+01_JPRB, 0.17666E+01_JPRB ,&
&  0.16749E+01_JPRB, 0.16646E+01_JPRB, 0.16972E+01_JPRB, 0.17113E+01_JPRB, 0.17027E+01_JPRB, 0.17669E+01_JPRB, 0.18344E+01_JPRB ,&
&  0.18953E+01_JPRB, 0.19541E+01_JPRB, 0.20151E+01_JPRB, 0.20644E+01_JPRB, 0.21470E+01_JPRB, 0.22848E+01_JPRB, 0.25044E+01_JPRB ,&
&  0.27646E+01_JPRB, 0.29966E+01_JPRB, 0.32182E+01_JPRB, 0.33454E+01_JPRB, 0.32971E+01_JPRB, 0.32580E+01_JPRB, 0.32866E+01_JPRB ,&
&  0.33312E+01_JPRB, 0.32793E+01_JPRB, 0.33392E+01_JPRB, 0.35417E+01_JPRB, 0.38287E+01_JPRB, 0.38962E+01_JPRB, 0.47374E+01_JPRB ,&
&  0.55056E+01_JPRB, 0.58565E+01_JPRB, 0.62955E+01_JPRB, 0.73939E+01_JPRB, 0.47003E+01_JPRB, 0.41014E+01_JPRB, 0.33061E+01_JPRB ,&
&  0.33559E+01_JPRB, 0.31704E+01_JPRB, 0.27539E+01_JPRB, 0.21609E+01_JPRB, 0.17701E+01_JPRB, 0.13488E+01_JPRB, 0.14938E+01_JPRB ,&
&  0.17750E+01_JPRB, 0.17880E+01_JPRB, 0.13489E+01_JPRB, 0.11571E+01_JPRB, 0.98013E+00_JPRB, 0.10088E+01_JPRB, 0.11080E+01_JPRB ,&
&  0.11668E+01_JPRB, 0.10458E+01_JPRB, 0.99413E+00_JPRB, 0.10680E+01_JPRB, 0.91893E+00_JPRB, 0.65392E+00_JPRB, 0.55555E+00_JPRB ,&
&  0.56879E+00_JPRB, 0.61182E+00_JPRB, 0.67364E+00_JPRB, 0.74404E+00_JPRB, 0.78165E+00_JPRB, 0.82390E+00_JPRB, 0.86396E+00_JPRB ,&
&  0.94317E+00_JPRB, 0.10007E+01_JPRB, 0.10422E+01_JPRB, 0.10554E+01_JPRB, 0.10939E+01_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.11112E+01_JPRB, 0.11639E+01_JPRB, 0.12018E+01_JPRB, 0.12568E+01_JPRB, 0.13127E+01_JPRB, 0.13729E+01_JPRB, 0.14521E+01_JPRB ,&
&  0.15544E+01_JPRB, 0.16664E+01_JPRB, 0.17719E+01_JPRB, 0.18502E+01_JPRB, 0.19234E+01_JPRB, 0.19768E+01_JPRB, 0.20263E+01_JPRB ,&
&  0.20625E+01_JPRB, 0.21337E+01_JPRB, 0.22085E+01_JPRB, 0.23078E+01_JPRB, 0.24407E+01_JPRB, 0.25994E+01_JPRB, 0.27494E+01_JPRB ,&
&  0.28332E+01_JPRB, 0.26864E+01_JPRB, 0.23035E+01_JPRB, 0.16582E+01_JPRB, 0.14493E+01_JPRB, 0.16893E+01_JPRB, 0.17238E+01_JPRB ,&
&  0.17483E+01_JPRB, 0.17477E+01_JPRB, 0.17406E+01_JPRB, 0.18024E+01_JPRB, 0.18459E+01_JPRB, 0.18847E+01_JPRB, 0.19056E+01_JPRB ,&
&  0.19379E+01_JPRB, 0.19938E+01_JPRB, 0.20309E+01_JPRB, 0.20672E+01_JPRB, 0.21331E+01_JPRB, 0.22643E+01_JPRB, 0.24639E+01_JPRB ,&
&  0.27077E+01_JPRB, 0.29315E+01_JPRB, 0.30825E+01_JPRB, 0.31871E+01_JPRB, 0.31556E+01_JPRB, 0.32027E+01_JPRB, 0.33503E+01_JPRB ,&
&  0.35577E+01_JPRB, 0.36701E+01_JPRB, 0.38854E+01_JPRB, 0.42851E+01_JPRB, 0.45721E+01_JPRB, 0.44348E+01_JPRB, 0.49538E+01_JPRB ,&
&  0.55483E+01_JPRB, 0.57343E+01_JPRB, 0.56455E+01_JPRB, 0.71586E+01_JPRB, 0.44475E+01_JPRB, 0.42844E+01_JPRB, 0.38731E+01_JPRB ,&
&  0.35100E+01_JPRB, 0.34165E+01_JPRB, 0.26011E+01_JPRB, 0.18391E+01_JPRB, 0.15408E+01_JPRB, 0.12031E+01_JPRB, 0.13429E+01_JPRB ,&
&  0.15815E+01_JPRB, 0.14151E+01_JPRB, 0.11762E+01_JPRB, 0.11060E+01_JPRB, 0.11023E+01_JPRB, 0.12144E+01_JPRB, 0.12754E+01_JPRB ,&
&  0.10476E+01_JPRB, 0.97974E+00_JPRB, 0.98601E+00_JPRB, 0.12414E+01_JPRB, 0.12092E+01_JPRB, 0.61564E+00_JPRB, 0.51333E+00_JPRB ,&
&  0.53071E+00_JPRB, 0.57792E+00_JPRB, 0.63354E+00_JPRB, 0.67616E+00_JPRB, 0.71546E+00_JPRB, 0.76509E+00_JPRB, 0.81951E+00_JPRB ,&
&  0.88087E+00_JPRB, 0.95377E+00_JPRB, 0.10056E+01_JPRB, 0.10428E+01_JPRB, 0.10717E+01_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.90736E+00_JPRB, 0.95932E+00_JPRB, 0.10086E+01_JPRB, 0.10397E+01_JPRB, 0.10479E+01_JPRB, 0.10597E+01_JPRB, 0.10622E+01_JPRB ,&
&  0.10732E+01_JPRB, 0.10726E+01_JPRB, 0.11064E+01_JPRB, 0.11840E+01_JPRB, 0.13356E+01_JPRB, 0.14222E+01_JPRB, 0.15284E+01_JPRB ,&
&  0.15397E+01_JPRB, 0.15593E+01_JPRB, 0.15849E+01_JPRB, 0.16277E+01_JPRB, 0.17076E+01_JPRB, 0.18223E+01_JPRB, 0.20738E+01_JPRB ,&
&  0.25439E+01_JPRB, 0.28398E+01_JPRB, 0.27965E+01_JPRB, 0.24448E+01_JPRB, 0.19326E+01_JPRB, 0.17232E+01_JPRB, 0.19398E+01_JPRB ,&
&  0.19918E+01_JPRB, 0.18808E+01_JPRB, 0.18207E+01_JPRB, 0.18180E+01_JPRB, 0.18261E+01_JPRB, 0.18418E+01_JPRB, 0.18508E+01_JPRB ,&
&  0.18765E+01_JPRB, 0.19162E+01_JPRB, 0.19475E+01_JPRB, 0.19798E+01_JPRB, 0.20414E+01_JPRB, 0.21398E+01_JPRB, 0.22735E+01_JPRB ,&
&  0.24547E+01_JPRB, 0.26098E+01_JPRB, 0.27041E+01_JPRB, 0.27511E+01_JPRB, 0.28507E+01_JPRB, 0.30928E+01_JPRB, 0.34582E+01_JPRB ,&
&  0.37950E+01_JPRB, 0.39552E+01_JPRB, 0.43330E+01_JPRB, 0.47302E+01_JPRB, 0.47683E+01_JPRB, 0.47208E+01_JPRB, 0.49607E+01_JPRB ,&
&  0.51244E+01_JPRB, 0.51553E+01_JPRB, 0.44604E+01_JPRB, 0.61409E+01_JPRB, 0.62475E+01_JPRB, 0.52062E+01_JPRB, 0.40815E+01_JPRB ,&
&  0.36040E+01_JPRB, 0.32917E+01_JPRB, 0.27010E+01_JPRB, 0.20157E+01_JPRB, 0.16668E+01_JPRB, 0.15965E+01_JPRB, 0.14503E+01_JPRB ,&
&  0.14713E+01_JPRB, 0.12151E+01_JPRB, 0.11214E+01_JPRB, 0.11291E+01_JPRB, 0.10560E+01_JPRB, 0.11947E+01_JPRB, 0.10877E+01_JPRB ,&
&  0.85551E+00_JPRB, 0.85408E+00_JPRB, 0.93265E+00_JPRB, 0.13506E+01_JPRB, 0.20827E+01_JPRB, 0.58368E+00_JPRB, 0.46794E+00_JPRB ,&
&  0.48895E+00_JPRB, 0.52203E+00_JPRB, 0.56154E+00_JPRB, 0.57969E+00_JPRB, 0.58537E+00_JPRB, 0.62162E+00_JPRB, 0.67607E+00_JPRB ,&
&  0.70943E+00_JPRB, 0.73517E+00_JPRB, 0.76716E+00_JPRB, 0.82874E+00_JPRB, 0.86451E+00_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.62720E+00_JPRB, 0.65965E+00_JPRB, 0.68039E+00_JPRB, 0.70366E+00_JPRB, 0.71320E+00_JPRB, 0.70519E+00_JPRB, 0.67365E+00_JPRB ,&
&  0.64756E+00_JPRB, 0.62156E+00_JPRB, 0.61202E+00_JPRB, 0.59776E+00_JPRB, 0.62002E+00_JPRB, 0.66411E+00_JPRB, 0.69977E+00_JPRB ,&
&  0.78568E+00_JPRB, 0.73725E+00_JPRB, 0.73808E+00_JPRB, 0.77038E+00_JPRB, 0.77601E+00_JPRB, 0.81515E+00_JPRB, 0.95411E+00_JPRB ,&
&  0.11723E+01_JPRB, 0.14171E+01_JPRB, 0.17806E+01_JPRB, 0.23537E+01_JPRB, 0.24983E+01_JPRB, 0.19667E+01_JPRB, 0.20202E+01_JPRB ,&
&  0.18112E+01_JPRB, 0.15747E+01_JPRB, 0.16145E+01_JPRB, 0.16260E+01_JPRB, 0.16780E+01_JPRB, 0.16824E+01_JPRB, 0.17181E+01_JPRB ,&
&  0.17617E+01_JPRB, 0.18156E+01_JPRB, 0.18673E+01_JPRB, 0.19083E+01_JPRB, 0.19574E+01_JPRB, 0.20196E+01_JPRB, 0.21103E+01_JPRB ,&
&  0.21732E+01_JPRB, 0.22243E+01_JPRB, 0.23213E+01_JPRB, 0.23662E+01_JPRB, 0.26511E+01_JPRB, 0.30592E+01_JPRB, 0.34649E+01_JPRB ,&
&  0.37388E+01_JPRB, 0.39278E+01_JPRB, 0.42770E+01_JPRB, 0.43834E+01_JPRB, 0.44306E+01_JPRB, 0.44803E+01_JPRB, 0.44210E+01_JPRB ,&
&  0.42064E+01_JPRB, 0.34489E+01_JPRB, 0.31419E+01_JPRB, 0.48375E+01_JPRB, 0.46534E+01_JPRB, 0.36643E+01_JPRB, 0.36425E+01_JPRB ,&
&  0.33447E+01_JPRB, 0.31596E+01_JPRB, 0.27420E+01_JPRB, 0.22055E+01_JPRB, 0.18042E+01_JPRB, 0.15150E+01_JPRB, 0.13985E+01_JPRB ,&
&  0.12554E+01_JPRB, 0.10744E+01_JPRB, 0.10996E+01_JPRB, 0.11452E+01_JPRB, 0.10635E+01_JPRB, 0.10730E+01_JPRB, 0.95152E+00_JPRB ,&
&  0.87218E+00_JPRB, 0.72939E+00_JPRB, 0.80101E+00_JPRB, 0.10276E+01_JPRB, 0.81648E+00_JPRB, 0.48368E+00_JPRB, 0.42924E+00_JPRB ,&
&  0.44272E+00_JPRB, 0.45629E+00_JPRB, 0.46895E+00_JPRB, 0.47948E+00_JPRB, 0.48792E+00_JPRB, 0.50783E+00_JPRB, 0.52495E+00_JPRB ,&
&  0.54314E+00_JPRB, 0.57316E+00_JPRB, 0.58612E+00_JPRB, 0.60094E+00_JPRB, 0.61970E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.56250E+00_JPRB, 0.56023E+00_JPRB, 0.55329E+00_JPRB, 0.55447E+00_JPRB, 0.56012E+00_JPRB, 0.57472E+00_JPRB, 0.58400E+00_JPRB ,&
&  0.58286E+00_JPRB, 0.58812E+00_JPRB, 0.57149E+00_JPRB, 0.57318E+00_JPRB, 0.59009E+00_JPRB, 0.60151E+00_JPRB, 0.61274E+00_JPRB ,&
&  0.65121E+00_JPRB, 0.66514E+00_JPRB, 0.66284E+00_JPRB, 0.66766E+00_JPRB, 0.66184E+00_JPRB, 0.70869E+00_JPRB, 0.71301E+00_JPRB ,&
&  0.69549E+00_JPRB, 0.72125E+00_JPRB, 0.85078E+00_JPRB, 0.11802E+01_JPRB, 0.14854E+01_JPRB, 0.14865E+01_JPRB, 0.17052E+01_JPRB ,&
&  0.15909E+01_JPRB, 0.12679E+01_JPRB, 0.13685E+01_JPRB, 0.12205E+01_JPRB, 0.12539E+01_JPRB, 0.13140E+01_JPRB, 0.13243E+01_JPRB ,&
&  0.13520E+01_JPRB, 0.13508E+01_JPRB, 0.13733E+01_JPRB, 0.14071E+01_JPRB, 0.14262E+01_JPRB, 0.14541E+01_JPRB, 0.14913E+01_JPRB ,&
&  0.14343E+01_JPRB, 0.14888E+01_JPRB, 0.16760E+01_JPRB, 0.19984E+01_JPRB, 0.23654E+01_JPRB, 0.28733E+01_JPRB, 0.30996E+01_JPRB ,&
&  0.32598E+01_JPRB, 0.34438E+01_JPRB, 0.32532E+01_JPRB, 0.33523E+01_JPRB, 0.36314E+01_JPRB, 0.34030E+01_JPRB, 0.32545E+01_JPRB ,&
&  0.30959E+01_JPRB, 0.24338E+01_JPRB, 0.23225E+01_JPRB, 0.33709E+01_JPRB, 0.34985E+01_JPRB, 0.32420E+01_JPRB, 0.32356E+01_JPRB ,&
&  0.30704E+01_JPRB, 0.28162E+01_JPRB, 0.23841E+01_JPRB, 0.20344E+01_JPRB, 0.17288E+01_JPRB, 0.14191E+01_JPRB, 0.13029E+01_JPRB ,&
&  0.10681E+01_JPRB, 0.10423E+01_JPRB, 0.11764E+01_JPRB, 0.12177E+01_JPRB, 0.11788E+01_JPRB, 0.12304E+01_JPRB, 0.96597E+00_JPRB ,&
&  0.78573E+00_JPRB, 0.69783E+00_JPRB, 0.94085E+00_JPRB, 0.12495E+01_JPRB, 0.65456E+00_JPRB, 0.48208E+00_JPRB, 0.40547E+00_JPRB ,&
&  0.42236E+00_JPRB, 0.44099E+00_JPRB, 0.44739E+00_JPRB, 0.45593E+00_JPRB, 0.47287E+00_JPRB, 0.48763E+00_JPRB, 0.51081E+00_JPRB ,&
&  0.52520E+00_JPRB, 0.53671E+00_JPRB, 0.54867E+00_JPRB, 0.56293E+00_JPRB, 0.55929E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.61010E+00_JPRB, 0.61827E+00_JPRB, 0.62449E+00_JPRB, 0.64178E+00_JPRB, 0.66770E+00_JPRB, 0.67418E+00_JPRB, 0.68979E+00_JPRB ,&
&  0.69905E+00_JPRB, 0.71662E+00_JPRB, 0.73524E+00_JPRB, 0.75729E+00_JPRB, 0.77095E+00_JPRB, 0.77667E+00_JPRB, 0.79721E+00_JPRB ,&
&  0.81420E+00_JPRB, 0.80929E+00_JPRB, 0.81914E+00_JPRB, 0.83976E+00_JPRB, 0.84485E+00_JPRB, 0.87549E+00_JPRB, 0.87203E+00_JPRB ,&
&  0.90010E+00_JPRB, 0.90215E+00_JPRB, 0.92962E+00_JPRB, 0.10151E+01_JPRB, 0.11146E+01_JPRB, 0.13138E+01_JPRB, 0.16833E+01_JPRB ,&
&  0.14350E+01_JPRB, 0.10882E+01_JPRB, 0.10024E+01_JPRB, 0.86179E+00_JPRB, 0.86205E+00_JPRB, 0.89070E+00_JPRB, 0.89191E+00_JPRB ,&
&  0.88167E+00_JPRB, 0.85055E+00_JPRB, 0.85066E+00_JPRB, 0.86611E+00_JPRB, 0.93039E+00_JPRB, 0.10645E+01_JPRB, 0.11667E+01_JPRB ,&
&  0.11949E+01_JPRB, 0.13837E+01_JPRB, 0.16157E+01_JPRB, 0.19320E+01_JPRB, 0.21430E+01_JPRB, 0.27111E+01_JPRB, 0.27572E+01_JPRB ,&
&  0.26872E+01_JPRB, 0.26119E+01_JPRB, 0.22273E+01_JPRB, 0.23381E+01_JPRB, 0.25949E+01_JPRB, 0.25055E+01_JPRB, 0.22290E+01_JPRB ,&
&  0.22123E+01_JPRB, 0.20805E+01_JPRB, 0.21358E+01_JPRB, 0.27999E+01_JPRB, 0.29837E+01_JPRB, 0.30009E+01_JPRB, 0.28280E+01_JPRB ,&
&  0.26090E+01_JPRB, 0.22978E+01_JPRB, 0.20063E+01_JPRB, 0.18486E+01_JPRB, 0.17493E+01_JPRB, 0.15090E+01_JPRB, 0.12502E+01_JPRB ,&
&  0.10776E+01_JPRB, 0.11461E+01_JPRB, 0.12535E+01_JPRB, 0.12529E+01_JPRB, 0.13644E+01_JPRB, 0.11047E+01_JPRB, 0.12171E+01_JPRB ,&
&  0.72085E+00_JPRB, 0.67640E+00_JPRB, 0.57380E+00_JPRB, 0.74374E+00_JPRB, 0.56093E+00_JPRB, 0.46235E+00_JPRB, 0.46324E+00_JPRB ,&
&  0.47667E+00_JPRB, 0.48622E+00_JPRB, 0.50091E+00_JPRB, 0.51234E+00_JPRB, 0.52660E+00_JPRB, 0.55092E+00_JPRB, 0.57712E+00_JPRB ,&
&  0.59912E+00_JPRB, 0.58814E+00_JPRB, 0.58919E+00_JPRB, 0.59436E+00_JPRB, 0.60802E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.64906E+00_JPRB, 0.66799E+00_JPRB, 0.68713E+00_JPRB, 0.70661E+00_JPRB, 0.72367E+00_JPRB, 0.73983E+00_JPRB, 0.75666E+00_JPRB ,&
&  0.77336E+00_JPRB, 0.79271E+00_JPRB, 0.81304E+00_JPRB, 0.83500E+00_JPRB, 0.85316E+00_JPRB, 0.86133E+00_JPRB, 0.87423E+00_JPRB ,&
&  0.88385E+00_JPRB, 0.88735E+00_JPRB, 0.88073E+00_JPRB, 0.90429E+00_JPRB, 0.92934E+00_JPRB, 0.94620E+00_JPRB, 0.97226E+00_JPRB ,&
&  0.10017E+01_JPRB, 0.98803E+00_JPRB, 0.98477E+00_JPRB, 0.10113E+01_JPRB, 0.10293E+01_JPRB, 0.98299E+00_JPRB, 0.10846E+01_JPRB ,&
&  0.84676E+00_JPRB, 0.96288E+00_JPRB, 0.89073E+00_JPRB, 0.78532E+00_JPRB, 0.76301E+00_JPRB, 0.68381E+00_JPRB, 0.64800E+00_JPRB ,&
&  0.70956E+00_JPRB, 0.73702E+00_JPRB, 0.76957E+00_JPRB, 0.83694E+00_JPRB, 0.97198E+00_JPRB, 0.11850E+01_JPRB, 0.13515E+01_JPRB ,&
&  0.15651E+01_JPRB, 0.17741E+01_JPRB, 0.20153E+01_JPRB, 0.22830E+01_JPRB, 0.25829E+01_JPRB, 0.27492E+01_JPRB, 0.27148E+01_JPRB ,&
&  0.25874E+01_JPRB, 0.24300E+01_JPRB, 0.18361E+01_JPRB, 0.20728E+01_JPRB, 0.24540E+01_JPRB, 0.23842E+01_JPRB, 0.20176E+01_JPRB ,&
&  0.18100E+01_JPRB, 0.17831E+01_JPRB, 0.20077E+01_JPRB, 0.22427E+01_JPRB, 0.23857E+01_JPRB, 0.24236E+01_JPRB, 0.22844E+01_JPRB ,&
&  0.21233E+01_JPRB, 0.18370E+01_JPRB, 0.16966E+01_JPRB, 0.17220E+01_JPRB, 0.16779E+01_JPRB, 0.15298E+01_JPRB, 0.12484E+01_JPRB ,&
&  0.11544E+01_JPRB, 0.12001E+01_JPRB, 0.12556E+01_JPRB, 0.12361E+01_JPRB, 0.11821E+01_JPRB, 0.15264E+01_JPRB, 0.15784E+01_JPRB ,&
&  0.72465E+00_JPRB, 0.60758E+00_JPRB, 0.44394E+00_JPRB, 0.54238E+00_JPRB, 0.58106E+00_JPRB, 0.57849E+00_JPRB, 0.62940E+00_JPRB ,&
&  0.66243E+00_JPRB, 0.62539E+00_JPRB, 0.61678E+00_JPRB, 0.59416E+00_JPRB, 0.59386E+00_JPRB, 0.58984E+00_JPRB, 0.60320E+00_JPRB ,&
&  0.61117E+00_JPRB, 0.61145E+00_JPRB, 0.61888E+00_JPRB, 0.62925E+00_JPRB, 0.63642E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.67448E+00_JPRB, 0.69496E+00_JPRB, 0.71556E+00_JPRB, 0.73439E+00_JPRB, 0.75253E+00_JPRB, 0.76915E+00_JPRB, 0.78698E+00_JPRB ,&
&  0.80555E+00_JPRB, 0.82526E+00_JPRB, 0.84400E+00_JPRB, 0.86756E+00_JPRB, 0.89045E+00_JPRB, 0.90251E+00_JPRB, 0.92126E+00_JPRB ,&
&  0.92861E+00_JPRB, 0.96290E+00_JPRB, 0.97761E+00_JPRB, 0.10103E+01_JPRB, 0.10387E+01_JPRB, 0.10563E+01_JPRB, 0.10911E+01_JPRB ,&
&  0.11082E+01_JPRB, 0.11186E+01_JPRB, 0.11310E+01_JPRB, 0.11601E+01_JPRB, 0.11734E+01_JPRB, 0.12500E+01_JPRB, 0.10325E+01_JPRB ,&
&  0.10338E+01_JPRB, 0.95770E+00_JPRB, 0.89301E+00_JPRB, 0.84092E+00_JPRB, 0.79786E+00_JPRB, 0.75640E+00_JPRB, 0.74143E+00_JPRB ,&
&  0.77440E+00_JPRB, 0.81717E+00_JPRB, 0.88211E+00_JPRB, 0.97657E+00_JPRB, 0.11339E+01_JPRB, 0.13621E+01_JPRB, 0.15486E+01_JPRB ,&
&  0.17619E+01_JPRB, 0.20084E+01_JPRB, 0.22430E+01_JPRB, 0.24920E+01_JPRB, 0.27721E+01_JPRB, 0.30575E+01_JPRB, 0.32931E+01_JPRB ,&
&  0.33251E+01_JPRB, 0.32624E+01_JPRB, 0.27668E+01_JPRB, 0.30633E+01_JPRB, 0.32703E+01_JPRB, 0.29208E+01_JPRB, 0.22014E+01_JPRB ,&
&  0.17173E+01_JPRB, 0.14274E+01_JPRB, 0.14865E+01_JPRB, 0.17345E+01_JPRB, 0.18195E+01_JPRB, 0.18225E+01_JPRB, 0.17237E+01_JPRB ,&
&  0.16013E+01_JPRB, 0.14296E+01_JPRB, 0.13609E+01_JPRB, 0.13938E+01_JPRB, 0.14617E+01_JPRB, 0.14754E+01_JPRB, 0.13518E+01_JPRB ,&
&  0.12859E+01_JPRB, 0.12358E+01_JPRB, 0.11622E+01_JPRB, 0.10374E+01_JPRB, 0.97174E+00_JPRB, 0.87527E+00_JPRB, 0.95494E+00_JPRB ,&
&  0.62826E+00_JPRB, 0.52255E+00_JPRB, 0.54255E+00_JPRB, 0.57207E+00_JPRB, 0.64339E+00_JPRB, 0.70215E+00_JPRB, 0.87125E+00_JPRB ,&
&  0.10101E+01_JPRB, 0.10606E+01_JPRB, 0.99317E+00_JPRB, 0.81579E+00_JPRB, 0.67754E+00_JPRB, 0.62357E+00_JPRB, 0.62293E+00_JPRB ,&
&  0.62413E+00_JPRB, 0.62120E+00_JPRB, 0.62916E+00_JPRB, 0.64404E+00_JPRB, 0.65704E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.64522E+00_JPRB, 0.66328E+00_JPRB, 0.68376E+00_JPRB, 0.70057E+00_JPRB, 0.72272E+00_JPRB, 0.75476E+00_JPRB, 0.78633E+00_JPRB ,&
&  0.81290E+00_JPRB, 0.83197E+00_JPRB, 0.84519E+00_JPRB, 0.86580E+00_JPRB, 0.89273E+00_JPRB, 0.92947E+00_JPRB, 0.97705E+00_JPRB ,&
&  0.99566E+00_JPRB, 0.10361E+01_JPRB, 0.10567E+01_JPRB, 0.10988E+01_JPRB, 0.11163E+01_JPRB, 0.11672E+01_JPRB, 0.12037E+01_JPRB ,&
&  0.12233E+01_JPRB, 0.12333E+01_JPRB, 0.12439E+01_JPRB, 0.12686E+01_JPRB, 0.12539E+01_JPRB, 0.12951E+01_JPRB, 0.10305E+01_JPRB ,&
&  0.11182E+01_JPRB, 0.10784E+01_JPRB, 0.10009E+01_JPRB, 0.97011E+00_JPRB, 0.94397E+00_JPRB, 0.92724E+00_JPRB, 0.92522E+00_JPRB ,&
&  0.93160E+00_JPRB, 0.96056E+00_JPRB, 0.96263E+00_JPRB, 0.10301E+01_JPRB, 0.11758E+01_JPRB, 0.14013E+01_JPRB, 0.15968E+01_JPRB ,&
&  0.18250E+01_JPRB, 0.20578E+01_JPRB, 0.23023E+01_JPRB, 0.26173E+01_JPRB, 0.30283E+01_JPRB, 0.34630E+01_JPRB, 0.38951E+01_JPRB ,&
&  0.40620E+01_JPRB, 0.41409E+01_JPRB, 0.37863E+01_JPRB, 0.38288E+01_JPRB, 0.39061E+01_JPRB, 0.31721E+01_JPRB, 0.21860E+01_JPRB ,&
&  0.13408E+01_JPRB, 0.12264E+01_JPRB, 0.13425E+01_JPRB, 0.14385E+01_JPRB, 0.13302E+01_JPRB, 0.12468E+01_JPRB, 0.11358E+01_JPRB ,&
&  0.10283E+01_JPRB, 0.94061E+00_JPRB, 0.94837E+00_JPRB, 0.10351E+01_JPRB, 0.11411E+01_JPRB, 0.12583E+01_JPRB, 0.12872E+01_JPRB ,&
&  0.12474E+01_JPRB, 0.11352E+01_JPRB, 0.10152E+01_JPRB, 0.90472E+00_JPRB, 0.80057E+00_JPRB, 0.71119E+00_JPRB, 0.77948E+00_JPRB ,&
&  0.64800E+00_JPRB, 0.61776E+00_JPRB, 0.64244E+00_JPRB, 0.52343E+00_JPRB, 0.70457E+00_JPRB, 0.87201E+00_JPRB, 0.11207E+01_JPRB ,&
&  0.12489E+01_JPRB, 0.16906E+01_JPRB, 0.21072E+01_JPRB, 0.15081E+01_JPRB, 0.11198E+01_JPRB, 0.69490E+00_JPRB, 0.62344E+00_JPRB ,&
&  0.62278E+00_JPRB, 0.60923E+00_JPRB, 0.60941E+00_JPRB, 0.61940E+00_JPRB, 0.63040E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.60704E+00_JPRB, 0.61901E+00_JPRB, 0.63853E+00_JPRB, 0.64087E+00_JPRB, 0.64922E+00_JPRB, 0.66756E+00_JPRB, 0.68607E+00_JPRB ,&
&  0.70522E+00_JPRB, 0.71596E+00_JPRB, 0.72871E+00_JPRB, 0.74460E+00_JPRB, 0.76417E+00_JPRB, 0.81146E+00_JPRB, 0.88235E+00_JPRB ,&
&  0.93766E+00_JPRB, 0.99036E+00_JPRB, 0.10580E+01_JPRB, 0.10823E+01_JPRB, 0.11274E+01_JPRB, 0.11876E+01_JPRB, 0.12245E+01_JPRB ,&
&  0.12799E+01_JPRB, 0.13080E+01_JPRB, 0.13364E+01_JPRB, 0.14041E+01_JPRB, 0.13677E+01_JPRB, 0.13960E+01_JPRB, 0.10851E+01_JPRB ,&
&  0.11537E+01_JPRB, 0.11378E+01_JPRB, 0.10935E+01_JPRB, 0.10752E+01_JPRB, 0.10507E+01_JPRB, 0.10105E+01_JPRB, 0.98112E+00_JPRB ,&
&  0.96944E+00_JPRB, 0.93210E+00_JPRB, 0.89539E+00_JPRB, 0.96292E+00_JPRB, 0.10697E+01_JPRB, 0.12556E+01_JPRB, 0.14904E+01_JPRB ,&
&  0.17375E+01_JPRB, 0.19693E+01_JPRB, 0.22123E+01_JPRB, 0.25646E+01_JPRB, 0.30286E+01_JPRB, 0.35204E+01_JPRB, 0.40390E+01_JPRB ,&
&  0.43955E+01_JPRB, 0.46637E+01_JPRB, 0.47344E+01_JPRB, 0.40039E+01_JPRB, 0.37757E+01_JPRB, 0.28477E+01_JPRB, 0.18352E+01_JPRB ,&
&  0.13273E+01_JPRB, 0.11778E+01_JPRB, 0.11930E+01_JPRB, 0.11878E+01_JPRB, 0.10175E+01_JPRB, 0.91099E+00_JPRB, 0.79872E+00_JPRB ,&
&  0.71761E+00_JPRB, 0.69893E+00_JPRB, 0.73077E+00_JPRB, 0.82184E+00_JPRB, 0.92375E+00_JPRB, 0.10435E+01_JPRB, 0.10880E+01_JPRB ,&
&  0.10440E+01_JPRB, 0.94077E+00_JPRB, 0.85061E+00_JPRB, 0.77408E+00_JPRB, 0.74688E+00_JPRB, 0.76961E+00_JPRB, 0.70343E+00_JPRB ,&
&  0.67458E+00_JPRB, 0.60426E+00_JPRB, 0.59877E+00_JPRB, 0.65998E+00_JPRB, 0.74405E+00_JPRB, 0.96889E+00_JPRB, 0.13767E+01_JPRB ,&
&  0.19742E+01_JPRB, 0.21123E+01_JPRB, 0.25941E+01_JPRB, 0.34602E+01_JPRB, 0.25474E+01_JPRB, 0.17962E+01_JPRB, 0.56540E+00_JPRB ,&
&  0.58049E+00_JPRB, 0.57713E+00_JPRB, 0.57754E+00_JPRB, 0.57796E+00_JPRB, 0.58696E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.51140E+00_JPRB, 0.53309E+00_JPRB, 0.52368E+00_JPRB, 0.52657E+00_JPRB, 0.53307E+00_JPRB, 0.54393E+00_JPRB, 0.56692E+00_JPRB ,&
&  0.57241E+00_JPRB, 0.60361E+00_JPRB, 0.63663E+00_JPRB, 0.66305E+00_JPRB, 0.69984E+00_JPRB, 0.75153E+00_JPRB, 0.81645E+00_JPRB ,&
&  0.88388E+00_JPRB, 0.92378E+00_JPRB, 0.95843E+00_JPRB, 0.98354E+00_JPRB, 0.10293E+01_JPRB, 0.10916E+01_JPRB, 0.11262E+01_JPRB ,&
&  0.12000E+01_JPRB, 0.12520E+01_JPRB, 0.13153E+01_JPRB, 0.14065E+01_JPRB, 0.13469E+01_JPRB, 0.13391E+01_JPRB, 0.97631E+00_JPRB ,&
&  0.11143E+01_JPRB, 0.11296E+01_JPRB, 0.11204E+01_JPRB, 0.11036E+01_JPRB, 0.10643E+01_JPRB, 0.99826E+00_JPRB, 0.95839E+00_JPRB ,&
&  0.92790E+00_JPRB, 0.89484E+00_JPRB, 0.85597E+00_JPRB, 0.84376E+00_JPRB, 0.96006E+00_JPRB, 0.10704E+01_JPRB, 0.12844E+01_JPRB ,&
&  0.15406E+01_JPRB, 0.18262E+01_JPRB, 0.21031E+01_JPRB, 0.24415E+01_JPRB, 0.28842E+01_JPRB, 0.33918E+01_JPRB, 0.39094E+01_JPRB ,&
&  0.42971E+01_JPRB, 0.46217E+01_JPRB, 0.45752E+01_JPRB, 0.37795E+01_JPRB, 0.33214E+01_JPRB, 0.24601E+01_JPRB, 0.15700E+01_JPRB ,&
&  0.12346E+01_JPRB, 0.11409E+01_JPRB, 0.11514E+01_JPRB, 0.10823E+01_JPRB, 0.89159E+00_JPRB, 0.76294E+00_JPRB, 0.65865E+00_JPRB ,&
&  0.58384E+00_JPRB, 0.58025E+00_JPRB, 0.64275E+00_JPRB, 0.72534E+00_JPRB, 0.80572E+00_JPRB, 0.88580E+00_JPRB, 0.88486E+00_JPRB ,&
&  0.83364E+00_JPRB, 0.75040E+00_JPRB, 0.69337E+00_JPRB, 0.69209E+00_JPRB, 0.77465E+00_JPRB, 0.87423E+00_JPRB, 0.95086E+00_JPRB ,&
&  0.72551E+00_JPRB, 0.59070E+00_JPRB, 0.59841E+00_JPRB, 0.63037E+00_JPRB, 0.68000E+00_JPRB, 0.75662E+00_JPRB, 0.87676E+00_JPRB ,&
&  0.10527E+01_JPRB, 0.14155E+01_JPRB, 0.13820E+01_JPRB, 0.11406E+01_JPRB, 0.16488E+01_JPRB, 0.13719E+01_JPRB, 0.51469E+00_JPRB ,&
&  0.49510E+00_JPRB, 0.48655E+00_JPRB, 0.49201E+00_JPRB, 0.49568E+00_JPRB, 0.50785E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.47795E+00_JPRB, 0.46630E+00_JPRB, 0.44845E+00_JPRB, 0.45674E+00_JPRB, 0.45813E+00_JPRB, 0.47495E+00_JPRB, 0.49416E+00_JPRB ,&
&  0.52048E+00_JPRB, 0.57075E+00_JPRB, 0.61887E+00_JPRB, 0.65797E+00_JPRB, 0.68174E+00_JPRB, 0.71111E+00_JPRB, 0.73961E+00_JPRB ,&
&  0.75855E+00_JPRB, 0.77748E+00_JPRB, 0.80389E+00_JPRB, 0.84235E+00_JPRB, 0.89234E+00_JPRB, 0.94938E+00_JPRB, 0.10191E+01_JPRB ,&
&  0.10962E+01_JPRB, 0.11794E+01_JPRB, 0.12851E+01_JPRB, 0.13222E+01_JPRB, 0.13546E+01_JPRB, 0.13928E+01_JPRB, 0.13734E+01_JPRB ,&
&  0.97793E+00_JPRB, 0.11599E+01_JPRB, 0.11332E+01_JPRB, 0.10926E+01_JPRB, 0.10602E+01_JPRB, 0.10067E+01_JPRB, 0.96299E+00_JPRB ,&
&  0.93272E+00_JPRB, 0.87267E+00_JPRB, 0.84811E+00_JPRB, 0.80593E+00_JPRB, 0.85521E+00_JPRB, 0.91762E+00_JPRB, 0.10588E+01_JPRB ,&
&  0.12721E+01_JPRB, 0.15782E+01_JPRB, 0.19480E+01_JPRB, 0.23008E+01_JPRB, 0.26508E+01_JPRB, 0.30382E+01_JPRB, 0.35883E+01_JPRB ,&
&  0.41825E+01_JPRB, 0.45922E+01_JPRB, 0.48400E+01_JPRB, 0.41688E+01_JPRB, 0.32732E+01_JPRB, 0.27955E+01_JPRB, 0.20249E+01_JPRB ,&
&  0.12845E+01_JPRB, 0.10333E+01_JPRB, 0.10578E+01_JPRB, 0.10430E+01_JPRB, 0.87500E+00_JPRB, 0.75854E+00_JPRB, 0.65111E+00_JPRB ,&
&  0.56635E+00_JPRB, 0.52139E+00_JPRB, 0.54503E+00_JPRB, 0.63941E+00_JPRB, 0.69346E+00_JPRB, 0.72060E+00_JPRB, 0.70079E+00_JPRB ,&
&  0.67493E+00_JPRB, 0.61998E+00_JPRB, 0.56510E+00_JPRB, 0.56245E+00_JPRB, 0.59853E+00_JPRB, 0.63550E+00_JPRB, 0.63475E+00_JPRB ,&
&  0.61740E+00_JPRB, 0.61117E+00_JPRB, 0.61738E+00_JPRB, 0.63656E+00_JPRB, 0.65299E+00_JPRB, 0.67620E+00_JPRB, 0.70075E+00_JPRB ,&
&  0.73271E+00_JPRB, 0.77690E+00_JPRB, 0.76342E+00_JPRB, 0.75070E+00_JPRB, 0.83095E+00_JPRB, 0.88299E+00_JPRB, 0.57347E+00_JPRB ,&
&  0.49101E+00_JPRB, 0.45869E+00_JPRB, 0.45874E+00_JPRB, 0.45285E+00_JPRB, 0.48677E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.48293E+00_JPRB, 0.46454E+00_JPRB, 0.45364E+00_JPRB, 0.44527E+00_JPRB, 0.43284E+00_JPRB, 0.45542E+00_JPRB, 0.46025E+00_JPRB ,&
&  0.48333E+00_JPRB, 0.52236E+00_JPRB, 0.56972E+00_JPRB, 0.59656E+00_JPRB, 0.60465E+00_JPRB, 0.62994E+00_JPRB, 0.65112E+00_JPRB ,&
&  0.66364E+00_JPRB, 0.67204E+00_JPRB, 0.68522E+00_JPRB, 0.70295E+00_JPRB, 0.73511E+00_JPRB, 0.78460E+00_JPRB, 0.85884E+00_JPRB ,&
&  0.94350E+00_JPRB, 0.10337E+01_JPRB, 0.11299E+01_JPRB, 0.12191E+01_JPRB, 0.12785E+01_JPRB, 0.13259E+01_JPRB, 0.12922E+01_JPRB ,&
&  0.78670E+00_JPRB, 0.10540E+01_JPRB, 0.12055E+01_JPRB, 0.11351E+01_JPRB, 0.10728E+01_JPRB, 0.10190E+01_JPRB, 0.96614E+00_JPRB ,&
&  0.91168E+00_JPRB, 0.85301E+00_JPRB, 0.83395E+00_JPRB, 0.84900E+00_JPRB, 0.78499E+00_JPRB, 0.80655E+00_JPRB, 0.88110E+00_JPRB ,&
&  0.10175E+01_JPRB, 0.12445E+01_JPRB, 0.16553E+01_JPRB, 0.20974E+01_JPRB, 0.23444E+01_JPRB, 0.25884E+01_JPRB, 0.30416E+01_JPRB ,&
&  0.37353E+01_JPRB, 0.42706E+01_JPRB, 0.46560E+01_JPRB, 0.34889E+01_JPRB, 0.29379E+01_JPRB, 0.28338E+01_JPRB, 0.25384E+01_JPRB ,&
&  0.15068E+01_JPRB, 0.10483E+01_JPRB, 0.10616E+01_JPRB, 0.10648E+01_JPRB, 0.90737E+00_JPRB, 0.81811E+00_JPRB, 0.72040E+00_JPRB ,&
&  0.61193E+00_JPRB, 0.54121E+00_JPRB, 0.49674E+00_JPRB, 0.52677E+00_JPRB, 0.56126E+00_JPRB, 0.57866E+00_JPRB, 0.55249E+00_JPRB ,&
&  0.57437E+00_JPRB, 0.55075E+00_JPRB, 0.53583E+00_JPRB, 0.54116E+00_JPRB, 0.59351E+00_JPRB, 0.61620E+00_JPRB, 0.62546E+00_JPRB ,&
&  0.63952E+00_JPRB, 0.65050E+00_JPRB, 0.65332E+00_JPRB, 0.66361E+00_JPRB, 0.67163E+00_JPRB, 0.67240E+00_JPRB, 0.67062E+00_JPRB ,&
&  0.66346E+00_JPRB, 0.66518E+00_JPRB, 0.65065E+00_JPRB, 0.63269E+00_JPRB, 0.60340E+00_JPRB, 0.61749E+00_JPRB, 0.55382E+00_JPRB ,&
&  0.51049E+00_JPRB, 0.47983E+00_JPRB, 0.45842E+00_JPRB, 0.45378E+00_JPRB, 0.46209E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.49384E+00_JPRB, 0.48858E+00_JPRB, 0.48146E+00_JPRB, 0.47113E+00_JPRB, 0.45947E+00_JPRB, 0.45877E+00_JPRB, 0.44574E+00_JPRB ,&
&  0.44256E+00_JPRB, 0.46351E+00_JPRB, 0.49618E+00_JPRB, 0.52293E+00_JPRB, 0.54663E+00_JPRB, 0.55968E+00_JPRB, 0.57935E+00_JPRB ,&
&  0.58247E+00_JPRB, 0.58086E+00_JPRB, 0.57750E+00_JPRB, 0.57570E+00_JPRB, 0.58293E+00_JPRB, 0.61027E+00_JPRB, 0.65899E+00_JPRB ,&
&  0.72766E+00_JPRB, 0.81766E+00_JPRB, 0.92717E+00_JPRB, 0.10439E+01_JPRB, 0.11556E+01_JPRB, 0.12626E+01_JPRB, 0.13075E+01_JPRB ,&
&  0.90224E+00_JPRB, 0.52788E+00_JPRB, 0.10893E+01_JPRB, 0.12212E+01_JPRB, 0.11394E+01_JPRB, 0.10936E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.91066E+00_JPRB, 0.88214E+00_JPRB, 0.83950E+00_JPRB, 0.81454E+00_JPRB, 0.75264E+00_JPRB, 0.74840E+00_JPRB, 0.76771E+00_JPRB ,&
&  0.83721E+00_JPRB, 0.98440E+00_JPRB, 0.12768E+01_JPRB, 0.17630E+01_JPRB, 0.20894E+01_JPRB, 0.22202E+01_JPRB, 0.25125E+01_JPRB ,&
&  0.31010E+01_JPRB, 0.36112E+01_JPRB, 0.38182E+01_JPRB, 0.25535E+01_JPRB, 0.17494E+01_JPRB, 0.15450E+01_JPRB, 0.14284E+01_JPRB ,&
&  0.13529E+01_JPRB, 0.11050E+01_JPRB, 0.10850E+01_JPRB, 0.11055E+01_JPRB, 0.99252E+00_JPRB, 0.86103E+00_JPRB, 0.83781E+00_JPRB ,&
&  0.73430E+00_JPRB, 0.64630E+00_JPRB, 0.57522E+00_JPRB, 0.53966E+00_JPRB, 0.52411E+00_JPRB, 0.53267E+00_JPRB, 0.52814E+00_JPRB ,&
&  0.52566E+00_JPRB, 0.53590E+00_JPRB, 0.53602E+00_JPRB, 0.54530E+00_JPRB, 0.57804E+00_JPRB, 0.61006E+00_JPRB, 0.63065E+00_JPRB ,&
&  0.64437E+00_JPRB, 0.66197E+00_JPRB, 0.67396E+00_JPRB, 0.68007E+00_JPRB, 0.68231E+00_JPRB, 0.66982E+00_JPRB, 0.67549E+00_JPRB ,&
&  0.68424E+00_JPRB, 0.69082E+00_JPRB, 0.67638E+00_JPRB, 0.66595E+00_JPRB, 0.63003E+00_JPRB, 0.60561E+00_JPRB, 0.54738E+00_JPRB ,&
&  0.52439E+00_JPRB, 0.48739E+00_JPRB, 0.46114E+00_JPRB, 0.45521E+00_JPRB, 0.46616E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.49694E+00_JPRB, 0.49672E+00_JPRB, 0.49656E+00_JPRB, 0.49976E+00_JPRB, 0.49607E+00_JPRB, 0.49790E+00_JPRB, 0.48112E+00_JPRB ,&
&  0.45469E+00_JPRB, 0.44543E+00_JPRB, 0.45985E+00_JPRB, 0.46477E+00_JPRB, 0.46703E+00_JPRB, 0.48487E+00_JPRB, 0.50390E+00_JPRB ,&
&  0.51214E+00_JPRB, 0.51502E+00_JPRB, 0.51311E+00_JPRB, 0.51170E+00_JPRB, 0.50446E+00_JPRB, 0.50657E+00_JPRB, 0.52549E+00_JPRB ,&
&  0.55629E+00_JPRB, 0.61229E+00_JPRB, 0.69592E+00_JPRB, 0.79890E+00_JPRB, 0.91111E+00_JPRB, 0.10623E+01_JPRB, 0.11948E+01_JPRB ,&
&  0.14420E+01_JPRB, 0.10018E+01_JPRB, 0.64679E+00_JPRB, 0.10910E+01_JPRB, 0.12352E+01_JPRB, 0.12345E+01_JPRB, 0.10670E+01_JPRB ,&
&  0.10148E+01_JPRB, 0.97848E+00_JPRB, 0.94724E+00_JPRB, 0.88576E+00_JPRB, 0.78331E+00_JPRB, 0.74128E+00_JPRB, 0.70691E+00_JPRB ,&
&  0.70787E+00_JPRB, 0.77586E+00_JPRB, 0.94116E+00_JPRB, 0.12798E+01_JPRB, 0.17088E+01_JPRB, 0.19336E+01_JPRB, 0.20651E+01_JPRB ,&
&  0.24667E+01_JPRB, 0.29875E+01_JPRB, 0.31098E+01_JPRB, 0.20508E+01_JPRB, 0.14823E+01_JPRB, 0.13340E+01_JPRB, 0.13022E+01_JPRB ,&
&  0.12421E+01_JPRB, 0.12457E+01_JPRB, 0.12693E+01_JPRB, 0.11599E+01_JPRB, 0.10322E+01_JPRB, 0.93727E+00_JPRB, 0.93641E+00_JPRB ,&
&  0.83699E+00_JPRB, 0.75293E+00_JPRB, 0.68126E+00_JPRB, 0.60880E+00_JPRB, 0.55024E+00_JPRB, 0.54103E+00_JPRB, 0.53202E+00_JPRB ,&
&  0.53411E+00_JPRB, 0.54893E+00_JPRB, 0.54845E+00_JPRB, 0.56383E+00_JPRB, 0.57999E+00_JPRB, 0.59785E+00_JPRB, 0.61461E+00_JPRB ,&
&  0.62864E+00_JPRB, 0.64614E+00_JPRB, 0.66380E+00_JPRB, 0.66801E+00_JPRB, 0.66280E+00_JPRB, 0.65382E+00_JPRB, 0.66794E+00_JPRB ,&
&  0.67819E+00_JPRB, 0.69607E+00_JPRB, 0.69645E+00_JPRB, 0.70280E+00_JPRB, 0.67646E+00_JPRB, 0.65206E+00_JPRB, 0.61039E+00_JPRB ,&
&  0.60421E+00_JPRB, 0.56872E+00_JPRB, 0.53581E+00_JPRB, 0.49131E+00_JPRB, 0.49107E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.51461E+00_JPRB, 0.50649E+00_JPRB, 0.50520E+00_JPRB, 0.50744E+00_JPRB, 0.50717E+00_JPRB, 0.50993E+00_JPRB, 0.51178E+00_JPRB ,&
&  0.48715E+00_JPRB, 0.46308E+00_JPRB, 0.44634E+00_JPRB, 0.43799E+00_JPRB, 0.43496E+00_JPRB, 0.44736E+00_JPRB, 0.45243E+00_JPRB ,&
&  0.44968E+00_JPRB, 0.45001E+00_JPRB, 0.46734E+00_JPRB, 0.47146E+00_JPRB, 0.47037E+00_JPRB, 0.46636E+00_JPRB, 0.46614E+00_JPRB ,&
&  0.47651E+00_JPRB, 0.49903E+00_JPRB, 0.54371E+00_JPRB, 0.60686E+00_JPRB, 0.67251E+00_JPRB, 0.77886E+00_JPRB, 0.90509E+00_JPRB ,&
&  0.11210E+01_JPRB, 0.11390E+01_JPRB, 0.58979E+00_JPRB, 0.96131E+00_JPRB, 0.12743E+01_JPRB, 0.13118E+01_JPRB, 0.12289E+01_JPRB ,&
&  0.11520E+01_JPRB, 0.10874E+01_JPRB, 0.11403E+01_JPRB, 0.10745E+01_JPRB, 0.88749E+00_JPRB, 0.77110E+00_JPRB, 0.68834E+00_JPRB ,&
&  0.64572E+00_JPRB, 0.64989E+00_JPRB, 0.71366E+00_JPRB, 0.88265E+00_JPRB, 0.12184E+01_JPRB, 0.15321E+01_JPRB, 0.16898E+01_JPRB ,&
&  0.18814E+01_JPRB, 0.23712E+01_JPRB, 0.27100E+01_JPRB, 0.18652E+01_JPRB, 0.14843E+01_JPRB, 0.15073E+01_JPRB, 0.14864E+01_JPRB ,&
&  0.14120E+01_JPRB, 0.13760E+01_JPRB, 0.13415E+01_JPRB, 0.12281E+01_JPRB, 0.10898E+01_JPRB, 0.10273E+01_JPRB, 0.10285E+01_JPRB ,&
&  0.93458E+00_JPRB, 0.84975E+00_JPRB, 0.77379E+00_JPRB, 0.70143E+00_JPRB, 0.62420E+00_JPRB, 0.56905E+00_JPRB, 0.56503E+00_JPRB ,&
&  0.56506E+00_JPRB, 0.57893E+00_JPRB, 0.57800E+00_JPRB, 0.58670E+00_JPRB, 0.59564E+00_JPRB, 0.60475E+00_JPRB, 0.61073E+00_JPRB ,&
&  0.61906E+00_JPRB, 0.63082E+00_JPRB, 0.64532E+00_JPRB, 0.64601E+00_JPRB, 0.64502E+00_JPRB, 0.64326E+00_JPRB, 0.64731E+00_JPRB ,&
&  0.65764E+00_JPRB, 0.66994E+00_JPRB, 0.67540E+00_JPRB, 0.68159E+00_JPRB, 0.70504E+00_JPRB, 0.70429E+00_JPRB, 0.68918E+00_JPRB ,&
&  0.73272E+00_JPRB, 0.77090E+00_JPRB, 0.71243E+00_JPRB, 0.56404E+00_JPRB, 0.53279E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.54469E+00_JPRB, 0.53227E+00_JPRB, 0.52341E+00_JPRB, 0.52212E+00_JPRB, 0.51956E+00_JPRB, 0.51911E+00_JPRB, 0.52739E+00_JPRB ,&
&  0.51973E+00_JPRB, 0.50416E+00_JPRB, 0.48248E+00_JPRB, 0.45995E+00_JPRB, 0.44229E+00_JPRB, 0.44318E+00_JPRB, 0.44820E+00_JPRB ,&
&  0.44865E+00_JPRB, 0.44605E+00_JPRB, 0.44633E+00_JPRB, 0.45084E+00_JPRB, 0.44953E+00_JPRB, 0.44650E+00_JPRB, 0.44315E+00_JPRB ,&
&  0.44207E+00_JPRB, 0.44713E+00_JPRB, 0.46564E+00_JPRB, 0.49781E+00_JPRB, 0.53539E+00_JPRB, 0.59251E+00_JPRB, 0.67173E+00_JPRB ,&
&  0.84607E+00_JPRB, 0.95018E+00_JPRB, 0.10285E+01_JPRB, 0.15343E+01_JPRB, 0.16133E+01_JPRB, 0.15005E+01_JPRB, 0.14474E+01_JPRB ,&
&  0.13871E+01_JPRB, 0.13217E+01_JPRB, 0.14283E+01_JPRB, 0.11943E+01_JPRB, 0.99007E+00_JPRB, 0.81969E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.64415E+00_JPRB, 0.60574E+00_JPRB, 0.60718E+00_JPRB, 0.65372E+00_JPRB, 0.82200E+00_JPRB, 0.11163E+01_JPRB, 0.13504E+01_JPRB ,&
&  0.15223E+01_JPRB, 0.18493E+01_JPRB, 0.23325E+01_JPRB, 0.19625E+01_JPRB, 0.15831E+01_JPRB, 0.18131E+01_JPRB, 0.20446E+01_JPRB ,&
&  0.23102E+01_JPRB, 0.17093E+01_JPRB, 0.14770E+01_JPRB, 0.13273E+01_JPRB, 0.11516E+01_JPRB, 0.12438E+01_JPRB, 0.11092E+01_JPRB ,&
&  0.10148E+01_JPRB, 0.94188E+00_JPRB, 0.86561E+00_JPRB, 0.79361E+00_JPRB, 0.72504E+00_JPRB, 0.66064E+00_JPRB, 0.62571E+00_JPRB ,&
&  0.61997E+00_JPRB, 0.62301E+00_JPRB, 0.61918E+00_JPRB, 0.62294E+00_JPRB, 0.62109E+00_JPRB, 0.62651E+00_JPRB, 0.63076E+00_JPRB ,&
&  0.63555E+00_JPRB, 0.63524E+00_JPRB, 0.61992E+00_JPRB, 0.62226E+00_JPRB, 0.62511E+00_JPRB, 0.62345E+00_JPRB, 0.61913E+00_JPRB ,&
&  0.62698E+00_JPRB, 0.64276E+00_JPRB, 0.65314E+00_JPRB, 0.66298E+00_JPRB, 0.67787E+00_JPRB, 0.68072E+00_JPRB, 0.66507E+00_JPRB ,&
&  0.65807E+00_JPRB, 0.66664E+00_JPRB, 0.66446E+00_JPRB, 0.59791E+00_JPRB, 0.56720E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.55811E+00_JPRB, 0.54692E+00_JPRB, 0.53955E+00_JPRB, 0.53204E+00_JPRB, 0.52397E+00_JPRB, 0.52320E+00_JPRB, 0.52988E+00_JPRB ,&
&  0.53023E+00_JPRB, 0.52063E+00_JPRB, 0.50546E+00_JPRB, 0.49011E+00_JPRB, 0.46933E+00_JPRB, 0.45757E+00_JPRB, 0.45081E+00_JPRB ,&
&  0.44602E+00_JPRB, 0.44524E+00_JPRB, 0.44125E+00_JPRB, 0.43962E+00_JPRB, 0.43777E+00_JPRB, 0.42952E+00_JPRB, 0.42504E+00_JPRB ,&
&  0.42179E+00_JPRB, 0.42510E+00_JPRB, 0.43397E+00_JPRB, 0.45151E+00_JPRB, 0.47740E+00_JPRB, 0.51600E+00_JPRB, 0.56335E+00_JPRB ,&
&  0.67885E+00_JPRB, 0.85281E+00_JPRB, 0.84693E+00_JPRB, 0.16440E+01_JPRB, 0.16179E+01_JPRB, 0.15651E+01_JPRB, 0.14562E+01_JPRB ,&
&  0.14409E+01_JPRB, 0.15807E+01_JPRB, 0.14449E+01_JPRB, 0.12355E+01_JPRB, 0.10134E+01_JPRB, 0.85307E+00_JPRB, 0.73232E+00_JPRB ,&
&  0.65744E+00_JPRB, 0.61184E+00_JPRB, 0.59081E+00_JPRB, 0.58430E+00_JPRB, 0.60651E+00_JPRB, 0.77187E+00_JPRB, 0.10342E+01_JPRB ,&
&  0.12333E+01_JPRB, 0.14724E+01_JPRB, 0.19182E+01_JPRB, 0.18361E+01_JPRB, 0.17131E+01_JPRB, 0.20353E+01_JPRB, 0.26558E+01_JPRB ,&
&  0.32115E+01_JPRB, 0.24295E+01_JPRB, 0.17789E+01_JPRB, 0.15068E+01_JPRB, 0.13592E+01_JPRB, 0.13223E+01_JPRB, 0.12036E+01_JPRB ,&
&  0.10750E+01_JPRB, 0.10045E+01_JPRB, 0.94292E+00_JPRB, 0.88250E+00_JPRB, 0.81724E+00_JPRB, 0.76030E+00_JPRB, 0.70701E+00_JPRB ,&
&  0.68022E+00_JPRB, 0.66899E+00_JPRB, 0.66234E+00_JPRB, 0.66727E+00_JPRB, 0.66837E+00_JPRB, 0.67484E+00_JPRB, 0.67395E+00_JPRB ,&
&  0.67501E+00_JPRB, 0.66255E+00_JPRB, 0.62220E+00_JPRB, 0.61049E+00_JPRB, 0.61134E+00_JPRB, 0.60811E+00_JPRB, 0.60169E+00_JPRB ,&
&  0.61310E+00_JPRB, 0.62510E+00_JPRB, 0.62805E+00_JPRB, 0.63018E+00_JPRB, 0.65164E+00_JPRB, 0.66580E+00_JPRB, 0.66042E+00_JPRB ,&
&  0.64320E+00_JPRB, 0.62780E+00_JPRB, 0.61147E+00_JPRB, 0.58424E+00_JPRB, 0.57229E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.56201E+00_JPRB, 0.55397E+00_JPRB, 0.54631E+00_JPRB, 0.53647E+00_JPRB, 0.52338E+00_JPRB, 0.51748E+00_JPRB, 0.52229E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52369E+00_JPRB, 0.51334E+00_JPRB, 0.49788E+00_JPRB, 0.47725E+00_JPRB, 0.46661E+00_JPRB, 0.45949E+00_JPRB ,&
&  0.44849E+00_JPRB, 0.43995E+00_JPRB, 0.42822E+00_JPRB, 0.42324E+00_JPRB, 0.42256E+00_JPRB, 0.41713E+00_JPRB, 0.40651E+00_JPRB ,&
&  0.40847E+00_JPRB, 0.41627E+00_JPRB, 0.42597E+00_JPRB, 0.43598E+00_JPRB, 0.45581E+00_JPRB, 0.48663E+00_JPRB, 0.53169E+00_JPRB ,&
&  0.61814E+00_JPRB, 0.93811E+00_JPRB, 0.78503E+00_JPRB, 0.12871E+01_JPRB, 0.13165E+01_JPRB, 0.13985E+01_JPRB, 0.14142E+01_JPRB ,&
&  0.14418E+01_JPRB, 0.14773E+01_JPRB, 0.13940E+01_JPRB, 0.11705E+01_JPRB, 0.95459E+00_JPRB, 0.82005E+00_JPRB, 0.73565E+00_JPRB ,&
&  0.68015E+00_JPRB, 0.63737E+00_JPRB, 0.60654E+00_JPRB, 0.58530E+00_JPRB, 0.57054E+00_JPRB, 0.59247E+00_JPRB, 0.72956E+00_JPRB ,&
&  0.95878E+00_JPRB, 0.11692E+01_JPRB, 0.15072E+01_JPRB, 0.18673E+01_JPRB, 0.17206E+01_JPRB, 0.19467E+01_JPRB, 0.28703E+01_JPRB ,&
&  0.34658E+01_JPRB, 0.34647E+01_JPRB, 0.23133E+01_JPRB, 0.18088E+01_JPRB, 0.16002E+01_JPRB, 0.14730E+01_JPRB, 0.12836E+01_JPRB ,&
&  0.11118E+01_JPRB, 0.10295E+01_JPRB, 0.98417E+00_JPRB, 0.94352E+00_JPRB, 0.89241E+00_JPRB, 0.82331E+00_JPRB, 0.75783E+00_JPRB ,&
&  0.70430E+00_JPRB, 0.68847E+00_JPRB, 0.68598E+00_JPRB, 0.69028E+00_JPRB, 0.70071E+00_JPRB, 0.71153E+00_JPRB, 0.71798E+00_JPRB ,&
&  0.71381E+00_JPRB, 0.70243E+00_JPRB, 0.65031E+00_JPRB, 0.62415E+00_JPRB, 0.61726E+00_JPRB, 0.61134E+00_JPRB, 0.60360E+00_JPRB ,&
&  0.61214E+00_JPRB, 0.61756E+00_JPRB, 0.61599E+00_JPRB, 0.62258E+00_JPRB, 0.63777E+00_JPRB, 0.66084E+00_JPRB, 0.63661E+00_JPRB ,&
&  0.63711E+00_JPRB, 0.62281E+00_JPRB, 0.60583E+00_JPRB, 0.58417E+00_JPRB, 0.57377E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.56183E+00_JPRB, 0.55499E+00_JPRB, 0.54753E+00_JPRB, 0.53914E+00_JPRB, 0.52904E+00_JPRB, 0.52156E+00_JPRB, 0.51932E+00_JPRB ,&
&  0.51974E+00_JPRB, 0.52177E+00_JPRB, 0.51574E+00_JPRB, 0.49938E+00_JPRB, 0.48137E+00_JPRB, 0.46517E+00_JPRB, 0.45634E+00_JPRB ,&
&  0.44918E+00_JPRB, 0.43980E+00_JPRB, 0.42828E+00_JPRB, 0.41688E+00_JPRB, 0.41361E+00_JPRB, 0.41488E+00_JPRB, 0.40921E+00_JPRB ,&
&  0.41472E+00_JPRB, 0.42226E+00_JPRB, 0.42929E+00_JPRB, 0.43607E+00_JPRB, 0.45313E+00_JPRB, 0.48142E+00_JPRB, 0.52401E+00_JPRB ,&
&  0.60320E+00_JPRB, 0.52224E+00_JPRB, 0.86031E+00_JPRB, 0.10021E+01_JPRB, 0.10826E+01_JPRB, 0.11564E+01_JPRB, 0.13085E+01_JPRB ,&
&  0.14316E+01_JPRB, 0.13881E+01_JPRB, 0.12664E+01_JPRB, 0.10866E+01_JPRB, 0.92399E+00_JPRB, 0.80621E+00_JPRB, 0.73037E+00_JPRB ,&
&  0.68335E+00_JPRB, 0.64762E+00_JPRB, 0.61559E+00_JPRB, 0.58801E+00_JPRB, 0.56914E+00_JPRB, 0.56019E+00_JPRB, 0.57941E+00_JPRB ,&
&  0.67153E+00_JPRB, 0.84167E+00_JPRB, 0.10734E+01_JPRB, 0.14837E+01_JPRB, 0.17017E+01_JPRB, 0.18511E+01_JPRB, 0.22035E+01_JPRB ,&
&  0.29613E+01_JPRB, 0.34159E+01_JPRB, 0.27006E+01_JPRB, 0.21285E+01_JPRB, 0.18281E+01_JPRB, 0.15602E+01_JPRB, 0.13272E+01_JPRB ,&
&  0.11491E+01_JPRB, 0.10442E+01_JPRB, 0.98210E+00_JPRB, 0.94410E+00_JPRB, 0.91875E+00_JPRB, 0.88092E+00_JPRB, 0.82826E+00_JPRB ,&
&  0.78079E+00_JPRB, 0.74920E+00_JPRB, 0.73656E+00_JPRB, 0.73583E+00_JPRB, 0.73742E+00_JPRB, 0.74654E+00_JPRB, 0.74670E+00_JPRB ,&
&  0.73423E+00_JPRB, 0.71367E+00_JPRB, 0.67736E+00_JPRB, 0.64768E+00_JPRB, 0.63761E+00_JPRB, 0.62693E+00_JPRB, 0.61442E+00_JPRB ,&
&  0.60993E+00_JPRB, 0.61207E+00_JPRB, 0.61430E+00_JPRB, 0.62452E+00_JPRB, 0.62523E+00_JPRB, 0.64868E+00_JPRB, 0.62325E+00_JPRB ,&
&  0.61869E+00_JPRB, 0.60469E+00_JPRB, 0.59138E+00_JPRB, 0.57698E+00_JPRB, 0.57250E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.56607E+00_JPRB, 0.55920E+00_JPRB, 0.55020E+00_JPRB, 0.54079E+00_JPRB, 0.52995E+00_JPRB, 0.52073E+00_JPRB, 0.51582E+00_JPRB ,&
&  0.51311E+00_JPRB, 0.51310E+00_JPRB, 0.51347E+00_JPRB, 0.50168E+00_JPRB, 0.48596E+00_JPRB, 0.47167E+00_JPRB, 0.46308E+00_JPRB ,&
&  0.45717E+00_JPRB, 0.45071E+00_JPRB, 0.44234E+00_JPRB, 0.43155E+00_JPRB, 0.42616E+00_JPRB, 0.42309E+00_JPRB, 0.42114E+00_JPRB ,&
&  0.42614E+00_JPRB, 0.43053E+00_JPRB, 0.43631E+00_JPRB, 0.44480E+00_JPRB, 0.46073E+00_JPRB, 0.48654E+00_JPRB, 0.52734E+00_JPRB ,&
&  0.60534E+00_JPRB, 0.76249E+00_JPRB, 0.93835E+00_JPRB, 0.87797E+00_JPRB, 0.88993E+00_JPRB, 0.92536E+00_JPRB, 0.10681E+01_JPRB ,&
&  0.11785E+01_JPRB, 0.12117E+01_JPRB, 0.11195E+01_JPRB, 0.99279E+00_JPRB, 0.87964E+00_JPRB, 0.79129E+00_JPRB, 0.72859E+00_JPRB ,&
&  0.67151E+00_JPRB, 0.62122E+00_JPRB, 0.58554E+00_JPRB, 0.56069E+00_JPRB, 0.54437E+00_JPRB, 0.53453E+00_JPRB, 0.53193E+00_JPRB ,&
&  0.54708E+00_JPRB, 0.60412E+00_JPRB, 0.73781E+00_JPRB, 0.10146E+01_JPRB, 0.16155E+01_JPRB, 0.16297E+01_JPRB, 0.20383E+01_JPRB ,&
&  0.27964E+01_JPRB, 0.27323E+01_JPRB, 0.24577E+01_JPRB, 0.21152E+01_JPRB, 0.18236E+01_JPRB, 0.15741E+01_JPRB, 0.13648E+01_JPRB ,&
&  0.12180E+01_JPRB, 0.11126E+01_JPRB, 0.10211E+01_JPRB, 0.95251E+00_JPRB, 0.90728E+00_JPRB, 0.88602E+00_JPRB, 0.86681E+00_JPRB ,&
&  0.84256E+00_JPRB, 0.81507E+00_JPRB, 0.79517E+00_JPRB, 0.78566E+00_JPRB, 0.77656E+00_JPRB, 0.76978E+00_JPRB, 0.74774E+00_JPRB ,&
&  0.72158E+00_JPRB, 0.69640E+00_JPRB, 0.67314E+00_JPRB, 0.64784E+00_JPRB, 0.64230E+00_JPRB, 0.63519E+00_JPRB, 0.61933E+00_JPRB ,&
&  0.60274E+00_JPRB, 0.59932E+00_JPRB, 0.61136E+00_JPRB, 0.61633E+00_JPRB, 0.62590E+00_JPRB, 0.64787E+00_JPRB, 0.62065E+00_JPRB ,&
&  0.61047E+00_JPRB, 0.59087E+00_JPRB, 0.57404E+00_JPRB, 0.56802E+00_JPRB, 0.57481E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.56456E+00_JPRB, 0.55507E+00_JPRB, 0.54562E+00_JPRB, 0.53635E+00_JPRB, 0.52446E+00_JPRB, 0.51521E+00_JPRB, 0.50694E+00_JPRB ,&
&  0.50116E+00_JPRB, 0.49815E+00_JPRB, 0.49890E+00_JPRB, 0.49942E+00_JPRB, 0.49651E+00_JPRB, 0.48822E+00_JPRB, 0.47412E+00_JPRB ,&
&  0.46507E+00_JPRB, 0.45737E+00_JPRB, 0.44779E+00_JPRB, 0.43611E+00_JPRB, 0.42475E+00_JPRB, 0.41948E+00_JPRB, 0.41914E+00_JPRB ,&
&  0.42342E+00_JPRB, 0.42885E+00_JPRB, 0.43249E+00_JPRB, 0.44399E+00_JPRB, 0.46132E+00_JPRB, 0.48709E+00_JPRB, 0.52324E+00_JPRB ,&
&  0.58125E+00_JPRB, 0.74750E+00_JPRB, 0.78921E+00_JPRB, 0.78010E+00_JPRB, 0.78668E+00_JPRB, 0.80636E+00_JPRB, 0.88413E+00_JPRB ,&
&  0.96001E+00_JPRB, 0.10091E+01_JPRB, 0.98219E+00_JPRB, 0.88751E+00_JPRB, 0.80845E+00_JPRB, 0.75263E+00_JPRB, 0.69830E+00_JPRB ,&
&  0.64629E+00_JPRB, 0.60542E+00_JPRB, 0.57304E+00_JPRB, 0.55094E+00_JPRB, 0.53429E+00_JPRB, 0.52211E+00_JPRB, 0.51579E+00_JPRB ,&
&  0.51586E+00_JPRB, 0.52456E+00_JPRB, 0.56200E+00_JPRB, 0.65559E+00_JPRB, 0.10476E+01_JPRB, 0.13996E+01_JPRB, 0.17076E+01_JPRB ,&
&  0.18272E+01_JPRB, 0.18887E+01_JPRB, 0.19164E+01_JPRB, 0.18004E+01_JPRB, 0.16375E+01_JPRB, 0.14467E+01_JPRB, 0.13066E+01_JPRB ,&
&  0.12132E+01_JPRB, 0.11140E+01_JPRB, 0.10207E+01_JPRB, 0.93382E+00_JPRB, 0.87123E+00_JPRB, 0.83637E+00_JPRB, 0.81520E+00_JPRB ,&
&  0.79351E+00_JPRB, 0.77786E+00_JPRB, 0.76474E+00_JPRB, 0.75292E+00_JPRB, 0.73992E+00_JPRB, 0.72325E+00_JPRB, 0.70125E+00_JPRB ,&
&  0.67535E+00_JPRB, 0.64995E+00_JPRB, 0.63385E+00_JPRB, 0.62402E+00_JPRB, 0.61724E+00_JPRB, 0.60693E+00_JPRB, 0.59448E+00_JPRB ,&
&  0.59225E+00_JPRB, 0.59593E+00_JPRB, 0.60173E+00_JPRB, 0.60707E+00_JPRB, 0.60933E+00_JPRB, 0.62757E+00_JPRB, 0.60814E+00_JPRB ,&
&  0.59143E+00_JPRB, 0.57258E+00_JPRB, 0.55802E+00_JPRB, 0.55111E+00_JPRB, 0.56891E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.56801E+00_JPRB, 0.55456E+00_JPRB, 0.53922E+00_JPRB, 0.52099E+00_JPRB, 0.50566E+00_JPRB, 0.49623E+00_JPRB, 0.48979E+00_JPRB ,&
&  0.48642E+00_JPRB, 0.48511E+00_JPRB, 0.48400E+00_JPRB, 0.48223E+00_JPRB, 0.47940E+00_JPRB, 0.47580E+00_JPRB, 0.46844E+00_JPRB ,&
&  0.45946E+00_JPRB, 0.44872E+00_JPRB, 0.43989E+00_JPRB, 0.43215E+00_JPRB, 0.41866E+00_JPRB, 0.41050E+00_JPRB, 0.41176E+00_JPRB ,&
&  0.41637E+00_JPRB, 0.42184E+00_JPRB, 0.42608E+00_JPRB, 0.43670E+00_JPRB, 0.45753E+00_JPRB, 0.48299E+00_JPRB, 0.51370E+00_JPRB ,&
&  0.57215E+00_JPRB, 0.59009E+00_JPRB, 0.64206E+00_JPRB, 0.65589E+00_JPRB, 0.66112E+00_JPRB, 0.69513E+00_JPRB, 0.74372E+00_JPRB ,&
&  0.77546E+00_JPRB, 0.82923E+00_JPRB, 0.82116E+00_JPRB, 0.75666E+00_JPRB, 0.69175E+00_JPRB, 0.64570E+00_JPRB, 0.61680E+00_JPRB ,&
&  0.59469E+00_JPRB, 0.57540E+00_JPRB, 0.55878E+00_JPRB, 0.54802E+00_JPRB, 0.53761E+00_JPRB, 0.52733E+00_JPRB, 0.51718E+00_JPRB ,&
&  0.51143E+00_JPRB, 0.50797E+00_JPRB, 0.51061E+00_JPRB, 0.53224E+00_JPRB, 0.62151E+00_JPRB, 0.79727E+00_JPRB, 0.10420E+01_JPRB ,&
&  0.12439E+01_JPRB, 0.13669E+01_JPRB, 0.14723E+01_JPRB, 0.14797E+01_JPRB, 0.14187E+01_JPRB, 0.13399E+01_JPRB, 0.12534E+01_JPRB ,&
&  0.11369E+01_JPRB, 0.10030E+01_JPRB, 0.90209E+00_JPRB, 0.82922E+00_JPRB, 0.77868E+00_JPRB, 0.74671E+00_JPRB, 0.72745E+00_JPRB ,&
&  0.71283E+00_JPRB, 0.70900E+00_JPRB, 0.72486E+00_JPRB, 0.74135E+00_JPRB, 0.74035E+00_JPRB, 0.71427E+00_JPRB, 0.67675E+00_JPRB ,&
&  0.64300E+00_JPRB, 0.61672E+00_JPRB, 0.59652E+00_JPRB, 0.58175E+00_JPRB, 0.57058E+00_JPRB, 0.56252E+00_JPRB, 0.55403E+00_JPRB ,&
&  0.55079E+00_JPRB, 0.56170E+00_JPRB, 0.56075E+00_JPRB, 0.57272E+00_JPRB, 0.60178E+00_JPRB, 0.58855E+00_JPRB, 0.56857E+00_JPRB ,&
&  0.55367E+00_JPRB, 0.54522E+00_JPRB, 0.53612E+00_JPRB, 0.53004E+00_JPRB, 0.58172E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.49857E+00_JPRB, 0.49806E+00_JPRB, 0.48955E+00_JPRB, 0.47962E+00_JPRB, 0.47151E+00_JPRB, 0.46758E+00_JPRB, 0.46444E+00_JPRB ,&
&  0.46284E+00_JPRB, 0.46208E+00_JPRB, 0.46150E+00_JPRB, 0.45931E+00_JPRB, 0.45501E+00_JPRB, 0.44840E+00_JPRB, 0.44012E+00_JPRB ,&
&  0.43267E+00_JPRB, 0.42514E+00_JPRB, 0.41864E+00_JPRB, 0.41153E+00_JPRB, 0.40269E+00_JPRB, 0.39799E+00_JPRB, 0.40034E+00_JPRB ,&
&  0.40602E+00_JPRB, 0.41284E+00_JPRB, 0.41888E+00_JPRB, 0.42886E+00_JPRB, 0.45220E+00_JPRB, 0.47749E+00_JPRB, 0.50120E+00_JPRB ,&
&  0.55039E+00_JPRB, 0.52140E+00_JPRB, 0.54558E+00_JPRB, 0.57736E+00_JPRB, 0.60472E+00_JPRB, 0.62004E+00_JPRB, 0.63892E+00_JPRB ,&
&  0.65800E+00_JPRB, 0.67472E+00_JPRB, 0.67474E+00_JPRB, 0.64807E+00_JPRB, 0.59820E+00_JPRB, 0.55291E+00_JPRB, 0.52652E+00_JPRB ,&
&  0.51307E+00_JPRB, 0.50714E+00_JPRB, 0.50170E+00_JPRB, 0.49683E+00_JPRB, 0.49354E+00_JPRB, 0.49055E+00_JPRB, 0.48724E+00_JPRB ,&
&  0.48417E+00_JPRB, 0.48173E+00_JPRB, 0.48056E+00_JPRB, 0.48265E+00_JPRB, 0.49991E+00_JPRB, 0.55664E+00_JPRB, 0.68264E+00_JPRB ,&
&  0.86118E+00_JPRB, 0.10305E+01_JPRB, 0.11253E+01_JPRB, 0.11833E+01_JPRB, 0.12224E+01_JPRB, 0.11754E+01_JPRB, 0.10480E+01_JPRB ,&
&  0.92631E+00_JPRB, 0.82076E+00_JPRB, 0.74314E+00_JPRB, 0.68731E+00_JPRB, 0.64888E+00_JPRB, 0.62527E+00_JPRB, 0.61280E+00_JPRB ,&
&  0.60851E+00_JPRB, 0.60927E+00_JPRB, 0.61559E+00_JPRB, 0.62222E+00_JPRB, 0.62573E+00_JPRB, 0.62052E+00_JPRB, 0.60720E+00_JPRB ,&
&  0.58760E+00_JPRB, 0.57294E+00_JPRB, 0.55850E+00_JPRB, 0.54293E+00_JPRB, 0.52958E+00_JPRB, 0.51712E+00_JPRB, 0.50775E+00_JPRB ,&
&  0.50344E+00_JPRB, 0.50887E+00_JPRB, 0.52049E+00_JPRB, 0.54297E+00_JPRB, 0.53475E+00_JPRB, 0.52139E+00_JPRB, 0.50962E+00_JPRB ,&
&  0.50507E+00_JPRB, 0.50540E+00_JPRB, 0.49840E+00_JPRB, 0.49304E+00_JPRB, 0.49523E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.46301E+00_JPRB, 0.46141E+00_JPRB, 0.45458E+00_JPRB, 0.44732E+00_JPRB, 0.44422E+00_JPRB, 0.44365E+00_JPRB, 0.44414E+00_JPRB ,&
&  0.44299E+00_JPRB, 0.44006E+00_JPRB, 0.43604E+00_JPRB, 0.43324E+00_JPRB, 0.42928E+00_JPRB, 0.42352E+00_JPRB, 0.41643E+00_JPRB ,&
&  0.40974E+00_JPRB, 0.40349E+00_JPRB, 0.39868E+00_JPRB, 0.39346E+00_JPRB, 0.38944E+00_JPRB, 0.38942E+00_JPRB, 0.39104E+00_JPRB ,&
&  0.39691E+00_JPRB, 0.40462E+00_JPRB, 0.41095E+00_JPRB, 0.42485E+00_JPRB, 0.44097E+00_JPRB, 0.46438E+00_JPRB, 0.48746E+00_JPRB ,&
&  0.53058E+00_JPRB, 0.46327E+00_JPRB, 0.48255E+00_JPRB, 0.53038E+00_JPRB, 0.55707E+00_JPRB, 0.57428E+00_JPRB, 0.57074E+00_JPRB ,&
&  0.56752E+00_JPRB, 0.56869E+00_JPRB, 0.56673E+00_JPRB, 0.55957E+00_JPRB, 0.54113E+00_JPRB, 0.50754E+00_JPRB, 0.47498E+00_JPRB ,&
&  0.45237E+00_JPRB, 0.44451E+00_JPRB, 0.44115E+00_JPRB, 0.43629E+00_JPRB, 0.43142E+00_JPRB, 0.42938E+00_JPRB, 0.43014E+00_JPRB ,&
&  0.43049E+00_JPRB, 0.43026E+00_JPRB, 0.42923E+00_JPRB, 0.42819E+00_JPRB, 0.43058E+00_JPRB, 0.44623E+00_JPRB, 0.49488E+00_JPRB ,&
&  0.59245E+00_JPRB, 0.70567E+00_JPRB, 0.81138E+00_JPRB, 0.89519E+00_JPRB, 0.91619E+00_JPRB, 0.89223E+00_JPRB, 0.84175E+00_JPRB ,&
&  0.77883E+00_JPRB, 0.71707E+00_JPRB, 0.66830E+00_JPRB, 0.62950E+00_JPRB, 0.59843E+00_JPRB, 0.57435E+00_JPRB, 0.55682E+00_JPRB ,&
&  0.54738E+00_JPRB, 0.54093E+00_JPRB, 0.53644E+00_JPRB, 0.53032E+00_JPRB, 0.52426E+00_JPRB, 0.52000E+00_JPRB, 0.51815E+00_JPRB ,&
&  0.51431E+00_JPRB, 0.50931E+00_JPRB, 0.50474E+00_JPRB, 0.49977E+00_JPRB, 0.49427E+00_JPRB, 0.48877E+00_JPRB, 0.48424E+00_JPRB ,&
&  0.48282E+00_JPRB, 0.48331E+00_JPRB, 0.48729E+00_JPRB, 0.48761E+00_JPRB, 0.49591E+00_JPRB, 0.48651E+00_JPRB, 0.47947E+00_JPRB ,&
&  0.47684E+00_JPRB, 0.47357E+00_JPRB, 0.47202E+00_JPRB, 0.43535E+00_JPRB, 0.45917E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.43089E+00_JPRB, 0.43246E+00_JPRB, 0.43097E+00_JPRB, 0.42554E+00_JPRB, 0.42186E+00_JPRB, 0.41954E+00_JPRB, 0.41857E+00_JPRB ,&
&  0.41830E+00_JPRB, 0.41737E+00_JPRB, 0.41496E+00_JPRB, 0.41008E+00_JPRB, 0.40581E+00_JPRB, 0.40029E+00_JPRB, 0.39451E+00_JPRB ,&
&  0.38963E+00_JPRB, 0.38535E+00_JPRB, 0.38299E+00_JPRB, 0.37998E+00_JPRB, 0.37880E+00_JPRB, 0.38025E+00_JPRB, 0.38233E+00_JPRB ,&
&  0.38532E+00_JPRB, 0.38800E+00_JPRB, 0.39607E+00_JPRB, 0.40617E+00_JPRB, 0.42468E+00_JPRB, 0.43772E+00_JPRB, 0.45668E+00_JPRB ,&
&  0.49791E+00_JPRB, 0.42325E+00_JPRB, 0.45052E+00_JPRB, 0.48577E+00_JPRB, 0.50756E+00_JPRB, 0.52638E+00_JPRB, 0.51194E+00_JPRB ,&
&  0.50074E+00_JPRB, 0.49300E+00_JPRB, 0.48488E+00_JPRB, 0.47828E+00_JPRB, 0.47398E+00_JPRB, 0.46370E+00_JPRB, 0.45047E+00_JPRB ,&
&  0.43566E+00_JPRB, 0.42476E+00_JPRB, 0.41614E+00_JPRB, 0.40787E+00_JPRB, 0.40072E+00_JPRB, 0.39627E+00_JPRB, 0.39424E+00_JPRB ,&
&  0.39409E+00_JPRB, 0.39433E+00_JPRB, 0.39419E+00_JPRB, 0.39474E+00_JPRB, 0.39679E+00_JPRB, 0.40132E+00_JPRB, 0.41100E+00_JPRB ,&
&  0.43983E+00_JPRB, 0.49182E+00_JPRB, 0.55160E+00_JPRB, 0.60987E+00_JPRB, 0.65299E+00_JPRB, 0.67295E+00_JPRB, 0.68024E+00_JPRB ,&
&  0.66928E+00_JPRB, 0.64525E+00_JPRB, 0.61604E+00_JPRB, 0.58739E+00_JPRB, 0.56214E+00_JPRB, 0.54140E+00_JPRB, 0.52259E+00_JPRB ,&
&  0.50566E+00_JPRB, 0.48923E+00_JPRB, 0.47583E+00_JPRB, 0.46601E+00_JPRB, 0.45706E+00_JPRB, 0.44919E+00_JPRB, 0.44488E+00_JPRB ,&
&  0.44310E+00_JPRB, 0.44273E+00_JPRB, 0.44240E+00_JPRB, 0.44349E+00_JPRB, 0.44499E+00_JPRB, 0.44622E+00_JPRB, 0.44638E+00_JPRB ,&
&  0.44493E+00_JPRB, 0.44521E+00_JPRB, 0.44565E+00_JPRB, 0.45369E+00_JPRB, 0.45325E+00_JPRB, 0.45172E+00_JPRB, 0.44906E+00_JPRB ,&
&  0.44724E+00_JPRB, 0.44308E+00_JPRB, 0.39480E+00_JPRB, 0.41694E+00_JPRB, 0.42546E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.40975E+00_JPRB, 0.40903E+00_JPRB, 0.40959E+00_JPRB, 0.40912E+00_JPRB, 0.40821E+00_JPRB, 0.40664E+00_JPRB, 0.40400E+00_JPRB ,&
&  0.40012E+00_JPRB, 0.39744E+00_JPRB, 0.39499E+00_JPRB, 0.39177E+00_JPRB, 0.38823E+00_JPRB, 0.38517E+00_JPRB, 0.38222E+00_JPRB ,&
&  0.37918E+00_JPRB, 0.37599E+00_JPRB, 0.37394E+00_JPRB, 0.37261E+00_JPRB, 0.37104E+00_JPRB, 0.37058E+00_JPRB, 0.37175E+00_JPRB ,&
&  0.37194E+00_JPRB, 0.37449E+00_JPRB, 0.38410E+00_JPRB, 0.39508E+00_JPRB, 0.40845E+00_JPRB, 0.42053E+00_JPRB, 0.43534E+00_JPRB ,&
&  0.42196E+00_JPRB, 0.37994E+00_JPRB, 0.40378E+00_JPRB, 0.43139E+00_JPRB, 0.45929E+00_JPRB, 0.48140E+00_JPRB, 0.47151E+00_JPRB ,&
&  0.45758E+00_JPRB, 0.44256E+00_JPRB, 0.43002E+00_JPRB, 0.42086E+00_JPRB, 0.41844E+00_JPRB, 0.41650E+00_JPRB, 0.41384E+00_JPRB ,&
&  0.40980E+00_JPRB, 0.40291E+00_JPRB, 0.39583E+00_JPRB, 0.38943E+00_JPRB, 0.38504E+00_JPRB, 0.38151E+00_JPRB, 0.37848E+00_JPRB ,&
&  0.37725E+00_JPRB, 0.37719E+00_JPRB, 0.37730E+00_JPRB, 0.37761E+00_JPRB, 0.37857E+00_JPRB, 0.38156E+00_JPRB, 0.38533E+00_JPRB ,&
&  0.39023E+00_JPRB, 0.40242E+00_JPRB, 0.42817E+00_JPRB, 0.46047E+00_JPRB, 0.49373E+00_JPRB, 0.51969E+00_JPRB, 0.53780E+00_JPRB ,&
&  0.55041E+00_JPRB, 0.55535E+00_JPRB, 0.55053E+00_JPRB, 0.53624E+00_JPRB, 0.51755E+00_JPRB, 0.49939E+00_JPRB, 0.48188E+00_JPRB ,&
&  0.46446E+00_JPRB, 0.44535E+00_JPRB, 0.42709E+00_JPRB, 0.41552E+00_JPRB, 0.40904E+00_JPRB, 0.40320E+00_JPRB, 0.39882E+00_JPRB ,&
&  0.39712E+00_JPRB, 0.39734E+00_JPRB, 0.39868E+00_JPRB, 0.40014E+00_JPRB, 0.40176E+00_JPRB, 0.40432E+00_JPRB, 0.40662E+00_JPRB ,&
&  0.40841E+00_JPRB, 0.41176E+00_JPRB, 0.41699E+00_JPRB, 0.42704E+00_JPRB, 0.43361E+00_JPRB, 0.43631E+00_JPRB, 0.43498E+00_JPRB ,&
&  0.42915E+00_JPRB, 0.42187E+00_JPRB, 0.41468E+00_JPRB, 0.40958E+00_JPRB, 0.40894E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.39321E+00_JPRB, 0.39237E+00_JPRB, 0.39189E+00_JPRB, 0.39052E+00_JPRB, 0.38933E+00_JPRB, 0.38810E+00_JPRB, 0.38477E+00_JPRB ,&
&  0.38351E+00_JPRB, 0.38004E+00_JPRB, 0.37669E+00_JPRB, 0.37520E+00_JPRB, 0.37330E+00_JPRB, 0.37079E+00_JPRB, 0.36888E+00_JPRB ,&
&  0.36674E+00_JPRB, 0.36433E+00_JPRB, 0.36271E+00_JPRB, 0.36050E+00_JPRB, 0.35714E+00_JPRB, 0.35475E+00_JPRB, 0.35536E+00_JPRB ,&
&  0.35626E+00_JPRB, 0.35885E+00_JPRB, 0.36689E+00_JPRB, 0.37483E+00_JPRB, 0.38341E+00_JPRB, 0.39159E+00_JPRB, 0.39912E+00_JPRB ,&
&  0.41036E+00_JPRB, 0.36797E+00_JPRB, 0.38954E+00_JPRB, 0.40486E+00_JPRB, 0.42808E+00_JPRB, 0.44649E+00_JPRB, 0.44055E+00_JPRB ,&
&  0.42501E+00_JPRB, 0.40693E+00_JPRB, 0.39318E+00_JPRB, 0.38463E+00_JPRB, 0.38406E+00_JPRB, 0.38512E+00_JPRB, 0.38384E+00_JPRB ,&
&  0.38133E+00_JPRB, 0.37555E+00_JPRB, 0.36903E+00_JPRB, 0.36476E+00_JPRB, 0.36253E+00_JPRB, 0.36089E+00_JPRB, 0.35907E+00_JPRB ,&
&  0.35807E+00_JPRB, 0.35800E+00_JPRB, 0.35803E+00_JPRB, 0.35801E+00_JPRB, 0.35867E+00_JPRB, 0.36038E+00_JPRB, 0.36265E+00_JPRB ,&
&  0.36446E+00_JPRB, 0.36534E+00_JPRB, 0.36921E+00_JPRB, 0.37953E+00_JPRB, 0.39465E+00_JPRB, 0.41004E+00_JPRB, 0.42264E+00_JPRB ,&
&  0.43247E+00_JPRB, 0.44089E+00_JPRB, 0.44620E+00_JPRB, 0.44579E+00_JPRB, 0.43960E+00_JPRB, 0.42977E+00_JPRB, 0.41897E+00_JPRB ,&
&  0.40770E+00_JPRB, 0.39608E+00_JPRB, 0.38303E+00_JPRB, 0.37510E+00_JPRB, 0.37143E+00_JPRB, 0.36937E+00_JPRB, 0.36809E+00_JPRB ,&
&  0.36746E+00_JPRB, 0.36722E+00_JPRB, 0.36799E+00_JPRB, 0.36924E+00_JPRB, 0.37000E+00_JPRB, 0.37081E+00_JPRB, 0.37254E+00_JPRB ,&
&  0.37588E+00_JPRB, 0.38139E+00_JPRB, 0.38857E+00_JPRB, 0.39847E+00_JPRB, 0.40701E+00_JPRB, 0.41226E+00_JPRB, 0.41482E+00_JPRB ,&
&  0.41260E+00_JPRB, 0.40742E+00_JPRB, 0.40084E+00_JPRB, 0.39513E+00_JPRB, 0.39400E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.38098E+00_JPRB, 0.37776E+00_JPRB, 0.37553E+00_JPRB, 0.37180E+00_JPRB, 0.36897E+00_JPRB, 0.36685E+00_JPRB, 0.36448E+00_JPRB ,&
&  0.36277E+00_JPRB, 0.36293E+00_JPRB, 0.36162E+00_JPRB, 0.36083E+00_JPRB, 0.35975E+00_JPRB, 0.35696E+00_JPRB, 0.35361E+00_JPRB ,&
&  0.35180E+00_JPRB, 0.35017E+00_JPRB, 0.34872E+00_JPRB, 0.34704E+00_JPRB, 0.34537E+00_JPRB, 0.34299E+00_JPRB, 0.34309E+00_JPRB ,&
&  0.34421E+00_JPRB, 0.34688E+00_JPRB, 0.35009E+00_JPRB, 0.35383E+00_JPRB, 0.35945E+00_JPRB, 0.36701E+00_JPRB, 0.37409E+00_JPRB ,&
&  0.38162E+00_JPRB, 0.36998E+00_JPRB, 0.38307E+00_JPRB, 0.39054E+00_JPRB, 0.40371E+00_JPRB, 0.41415E+00_JPRB, 0.41333E+00_JPRB ,&
&  0.40675E+00_JPRB, 0.39347E+00_JPRB, 0.38135E+00_JPRB, 0.37337E+00_JPRB, 0.37107E+00_JPRB, 0.36953E+00_JPRB, 0.36496E+00_JPRB ,&
&  0.35907E+00_JPRB, 0.35189E+00_JPRB, 0.34629E+00_JPRB, 0.34328E+00_JPRB, 0.34231E+00_JPRB, 0.34231E+00_JPRB, 0.34113E+00_JPRB ,&
&  0.33982E+00_JPRB, 0.33844E+00_JPRB, 0.33777E+00_JPRB, 0.33747E+00_JPRB, 0.33708E+00_JPRB, 0.33730E+00_JPRB, 0.33815E+00_JPRB ,&
&  0.33909E+00_JPRB, 0.33969E+00_JPRB, 0.34004E+00_JPRB, 0.34172E+00_JPRB, 0.34589E+00_JPRB, 0.35193E+00_JPRB, 0.35796E+00_JPRB ,&
&  0.36217E+00_JPRB, 0.36631E+00_JPRB, 0.37001E+00_JPRB, 0.37272E+00_JPRB, 0.37197E+00_JPRB, 0.36758E+00_JPRB, 0.36153E+00_JPRB ,&
&  0.35632E+00_JPRB, 0.35170E+00_JPRB, 0.34731E+00_JPRB, 0.34384E+00_JPRB, 0.34197E+00_JPRB, 0.34175E+00_JPRB, 0.34214E+00_JPRB ,&
&  0.34274E+00_JPRB, 0.34349E+00_JPRB, 0.34445E+00_JPRB, 0.34593E+00_JPRB, 0.34681E+00_JPRB, 0.34781E+00_JPRB, 0.34913E+00_JPRB ,&
&  0.35270E+00_JPRB, 0.35720E+00_JPRB, 0.36361E+00_JPRB, 0.37312E+00_JPRB, 0.38166E+00_JPRB, 0.38764E+00_JPRB, 0.39248E+00_JPRB ,&
&  0.39418E+00_JPRB, 0.39289E+00_JPRB, 0.39020E+00_JPRB, 0.38766E+00_JPRB, 0.38490E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.35825E+00_JPRB, 0.35393E+00_JPRB, 0.35069E+00_JPRB, 0.34808E+00_JPRB, 0.34776E+00_JPRB, 0.34629E+00_JPRB, 0.34307E+00_JPRB ,&
&  0.34057E+00_JPRB, 0.34157E+00_JPRB, 0.34399E+00_JPRB, 0.34487E+00_JPRB, 0.34400E+00_JPRB, 0.34243E+00_JPRB, 0.33955E+00_JPRB ,&
&  0.33566E+00_JPRB, 0.33380E+00_JPRB, 0.33367E+00_JPRB, 0.33454E+00_JPRB, 0.33446E+00_JPRB, 0.33274E+00_JPRB, 0.33138E+00_JPRB ,&
&  0.33223E+00_JPRB, 0.33369E+00_JPRB, 0.33399E+00_JPRB, 0.33599E+00_JPRB, 0.33996E+00_JPRB, 0.34720E+00_JPRB, 0.35387E+00_JPRB ,&
&  0.35626E+00_JPRB, 0.35729E+00_JPRB, 0.35124E+00_JPRB, 0.36265E+00_JPRB, 0.36966E+00_JPRB, 0.37898E+00_JPRB, 0.38421E+00_JPRB ,&
&  0.38498E+00_JPRB, 0.38032E+00_JPRB, 0.36941E+00_JPRB, 0.36093E+00_JPRB, 0.35573E+00_JPRB, 0.35053E+00_JPRB, 0.34262E+00_JPRB ,&
&  0.33442E+00_JPRB, 0.32733E+00_JPRB, 0.32318E+00_JPRB, 0.32136E+00_JPRB, 0.32170E+00_JPRB, 0.32242E+00_JPRB, 0.32085E+00_JPRB ,&
&  0.31897E+00_JPRB, 0.31712E+00_JPRB, 0.31639E+00_JPRB, 0.31561E+00_JPRB, 0.31443E+00_JPRB, 0.31347E+00_JPRB, 0.31320E+00_JPRB ,&
&  0.31336E+00_JPRB, 0.31352E+00_JPRB, 0.31396E+00_JPRB, 0.31479E+00_JPRB, 0.31541E+00_JPRB, 0.31621E+00_JPRB, 0.31798E+00_JPRB ,&
&  0.31985E+00_JPRB, 0.32141E+00_JPRB, 0.32276E+00_JPRB, 0.32374E+00_JPRB, 0.32401E+00_JPRB, 0.32343E+00_JPRB, 0.32143E+00_JPRB ,&
&  0.31899E+00_JPRB, 0.31805E+00_JPRB, 0.31775E+00_JPRB, 0.31667E+00_JPRB, 0.31541E+00_JPRB, 0.31535E+00_JPRB, 0.31648E+00_JPRB ,&
&  0.31781E+00_JPRB, 0.31963E+00_JPRB, 0.32118E+00_JPRB, 0.32251E+00_JPRB, 0.32299E+00_JPRB, 0.32401E+00_JPRB, 0.32530E+00_JPRB ,&
&  0.32733E+00_JPRB, 0.33039E+00_JPRB, 0.33475E+00_JPRB, 0.34046E+00_JPRB, 0.34638E+00_JPRB, 0.35094E+00_JPRB, 0.35578E+00_JPRB ,&
&  0.36009E+00_JPRB, 0.36225E+00_JPRB, 0.36264E+00_JPRB, 0.36317E+00_JPRB, 0.36198E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.33187E+00_JPRB, 0.32871E+00_JPRB, 0.32631E+00_JPRB, 0.32567E+00_JPRB, 0.32705E+00_JPRB, 0.32678E+00_JPRB, 0.32445E+00_JPRB ,&
&  0.32227E+00_JPRB, 0.32324E+00_JPRB, 0.32626E+00_JPRB, 0.32794E+00_JPRB, 0.32863E+00_JPRB, 0.32940E+00_JPRB, 0.32824E+00_JPRB ,&
&  0.32481E+00_JPRB, 0.32146E+00_JPRB, 0.32204E+00_JPRB, 0.32374E+00_JPRB, 0.32484E+00_JPRB, 0.32339E+00_JPRB, 0.32167E+00_JPRB ,&
&  0.32215E+00_JPRB, 0.32224E+00_JPRB, 0.32387E+00_JPRB, 0.32558E+00_JPRB, 0.32917E+00_JPRB, 0.33505E+00_JPRB, 0.33936E+00_JPRB ,&
&  0.34060E+00_JPRB, 0.33881E+00_JPRB, 0.33802E+00_JPRB, 0.34061E+00_JPRB, 0.34364E+00_JPRB, 0.35061E+00_JPRB, 0.35770E+00_JPRB ,&
&  0.36007E+00_JPRB, 0.35677E+00_JPRB, 0.34621E+00_JPRB, 0.33719E+00_JPRB, 0.33142E+00_JPRB, 0.32552E+00_JPRB, 0.31974E+00_JPRB ,&
&  0.31370E+00_JPRB, 0.30931E+00_JPRB, 0.30680E+00_JPRB, 0.30579E+00_JPRB, 0.30556E+00_JPRB, 0.30501E+00_JPRB, 0.30281E+00_JPRB ,&
&  0.29995E+00_JPRB, 0.29775E+00_JPRB, 0.29606E+00_JPRB, 0.29475E+00_JPRB, 0.29353E+00_JPRB, 0.29294E+00_JPRB, 0.29227E+00_JPRB ,&
&  0.29195E+00_JPRB, 0.29235E+00_JPRB, 0.29315E+00_JPRB, 0.29316E+00_JPRB, 0.29298E+00_JPRB, 0.29234E+00_JPRB, 0.29223E+00_JPRB ,&
&  0.29282E+00_JPRB, 0.29237E+00_JPRB, 0.29197E+00_JPRB, 0.29281E+00_JPRB, 0.29351E+00_JPRB, 0.29439E+00_JPRB, 0.29472E+00_JPRB ,&
&  0.29459E+00_JPRB, 0.29479E+00_JPRB, 0.29529E+00_JPRB, 0.29546E+00_JPRB, 0.29535E+00_JPRB, 0.29524E+00_JPRB, 0.29547E+00_JPRB ,&
&  0.29688E+00_JPRB, 0.29866E+00_JPRB, 0.30000E+00_JPRB, 0.30122E+00_JPRB, 0.30250E+00_JPRB, 0.30461E+00_JPRB, 0.30547E+00_JPRB ,&
&  0.30606E+00_JPRB, 0.30747E+00_JPRB, 0.31049E+00_JPRB, 0.31456E+00_JPRB, 0.31842E+00_JPRB, 0.32111E+00_JPRB, 0.32406E+00_JPRB ,&
&  0.32681E+00_JPRB, 0.32921E+00_JPRB, 0.33072E+00_JPRB, 0.33142E+00_JPRB, 0.33212E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.30672E+00_JPRB, 0.30558E+00_JPRB, 0.30512E+00_JPRB, 0.30540E+00_JPRB, 0.30699E+00_JPRB, 0.30682E+00_JPRB, 0.30613E+00_JPRB ,&
&  0.30544E+00_JPRB, 0.30714E+00_JPRB, 0.31070E+00_JPRB, 0.31297E+00_JPRB, 0.31343E+00_JPRB, 0.31346E+00_JPRB, 0.31314E+00_JPRB ,&
&  0.31294E+00_JPRB, 0.31279E+00_JPRB, 0.31195E+00_JPRB, 0.31249E+00_JPRB, 0.31254E+00_JPRB, 0.31156E+00_JPRB, 0.31241E+00_JPRB ,&
&  0.31254E+00_JPRB, 0.31305E+00_JPRB, 0.31525E+00_JPRB, 0.31700E+00_JPRB, 0.32081E+00_JPRB, 0.32687E+00_JPRB, 0.32990E+00_JPRB ,&
&  0.32938E+00_JPRB, 0.32506E+00_JPRB, 0.32139E+00_JPRB, 0.32189E+00_JPRB, 0.32369E+00_JPRB, 0.32744E+00_JPRB, 0.33145E+00_JPRB ,&
&  0.33230E+00_JPRB, 0.32606E+00_JPRB, 0.31647E+00_JPRB, 0.30966E+00_JPRB, 0.30531E+00_JPRB, 0.30223E+00_JPRB, 0.29950E+00_JPRB ,&
&  0.29648E+00_JPRB, 0.29443E+00_JPRB, 0.29370E+00_JPRB, 0.29383E+00_JPRB, 0.29360E+00_JPRB, 0.29081E+00_JPRB, 0.28753E+00_JPRB ,&
&  0.28334E+00_JPRB, 0.28040E+00_JPRB, 0.27798E+00_JPRB, 0.27714E+00_JPRB, 0.27724E+00_JPRB, 0.27697E+00_JPRB, 0.27590E+00_JPRB ,&
&  0.27454E+00_JPRB, 0.27406E+00_JPRB, 0.27365E+00_JPRB, 0.27245E+00_JPRB, 0.27095E+00_JPRB, 0.27045E+00_JPRB, 0.26957E+00_JPRB ,&
&  0.26809E+00_JPRB, 0.26788E+00_JPRB, 0.26880E+00_JPRB, 0.27051E+00_JPRB, 0.27097E+00_JPRB, 0.27233E+00_JPRB, 0.27437E+00_JPRB ,&
&  0.27402E+00_JPRB, 0.27175E+00_JPRB, 0.27027E+00_JPRB, 0.27042E+00_JPRB, 0.27196E+00_JPRB, 0.27422E+00_JPRB, 0.27522E+00_JPRB ,&
&  0.27611E+00_JPRB, 0.27603E+00_JPRB, 0.27690E+00_JPRB, 0.27830E+00_JPRB, 0.28028E+00_JPRB, 0.28203E+00_JPRB, 0.28311E+00_JPRB ,&
&  0.28385E+00_JPRB, 0.28448E+00_JPRB, 0.28664E+00_JPRB, 0.28991E+00_JPRB, 0.29272E+00_JPRB, 0.29589E+00_JPRB, 0.29831E+00_JPRB ,&
&  0.30001E+00_JPRB, 0.30182E+00_JPRB, 0.30305E+00_JPRB, 0.30402E+00_JPRB, 0.30538E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.29049E+00_JPRB, 0.29115E+00_JPRB, 0.29231E+00_JPRB, 0.29264E+00_JPRB, 0.29346E+00_JPRB, 0.29382E+00_JPRB, 0.29479E+00_JPRB ,&
&  0.29512E+00_JPRB, 0.29653E+00_JPRB, 0.29865E+00_JPRB, 0.29867E+00_JPRB, 0.29841E+00_JPRB, 0.29835E+00_JPRB, 0.29916E+00_JPRB ,&
&  0.30162E+00_JPRB, 0.30297E+00_JPRB, 0.30274E+00_JPRB, 0.30289E+00_JPRB, 0.30210E+00_JPRB, 0.30130E+00_JPRB, 0.30327E+00_JPRB ,&
&  0.30381E+00_JPRB, 0.30460E+00_JPRB, 0.30652E+00_JPRB, 0.30815E+00_JPRB, 0.31193E+00_JPRB, 0.31656E+00_JPRB, 0.31985E+00_JPRB ,&
&  0.31994E+00_JPRB, 0.31676E+00_JPRB, 0.31135E+00_JPRB, 0.30714E+00_JPRB, 0.30625E+00_JPRB, 0.30713E+00_JPRB, 0.30933E+00_JPRB ,&
&  0.30838E+00_JPRB, 0.30257E+00_JPRB, 0.29519E+00_JPRB, 0.29110E+00_JPRB, 0.28950E+00_JPRB, 0.28729E+00_JPRB, 0.28386E+00_JPRB ,&
&  0.28122E+00_JPRB, 0.28064E+00_JPRB, 0.28099E+00_JPRB, 0.28038E+00_JPRB, 0.27875E+00_JPRB, 0.27577E+00_JPRB, 0.27194E+00_JPRB ,&
&  0.26854E+00_JPRB, 0.26517E+00_JPRB, 0.26305E+00_JPRB, 0.26162E+00_JPRB, 0.26151E+00_JPRB, 0.26166E+00_JPRB, 0.26074E+00_JPRB ,&
&  0.25872E+00_JPRB, 0.25737E+00_JPRB, 0.25565E+00_JPRB, 0.25250E+00_JPRB, 0.25027E+00_JPRB, 0.24963E+00_JPRB, 0.24870E+00_JPRB ,&
&  0.24744E+00_JPRB, 0.24735E+00_JPRB, 0.24753E+00_JPRB, 0.24745E+00_JPRB, 0.24878E+00_JPRB, 0.25093E+00_JPRB, 0.24874E+00_JPRB ,&
&  0.24676E+00_JPRB, 0.24542E+00_JPRB, 0.24548E+00_JPRB, 0.24702E+00_JPRB, 0.24961E+00_JPRB, 0.25244E+00_JPRB, 0.25479E+00_JPRB ,&
&  0.25578E+00_JPRB, 0.25601E+00_JPRB, 0.25631E+00_JPRB, 0.25652E+00_JPRB, 0.25789E+00_JPRB, 0.26058E+00_JPRB, 0.26370E+00_JPRB ,&
&  0.26510E+00_JPRB, 0.26761E+00_JPRB, 0.27039E+00_JPRB, 0.27389E+00_JPRB, 0.27596E+00_JPRB, 0.27790E+00_JPRB, 0.27968E+00_JPRB ,&
&  0.28102E+00_JPRB, 0.28273E+00_JPRB, 0.28502E+00_JPRB, 0.28689E+00_JPRB, 0.28886E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.27850E+00_JPRB, 0.27998E+00_JPRB, 0.28079E+00_JPRB, 0.28211E+00_JPRB, 0.28447E+00_JPRB, 0.28581E+00_JPRB, 0.28572E+00_JPRB ,&
&  0.28572E+00_JPRB, 0.28697E+00_JPRB, 0.28795E+00_JPRB, 0.28841E+00_JPRB, 0.28617E+00_JPRB, 0.28593E+00_JPRB, 0.28777E+00_JPRB ,&
&  0.29017E+00_JPRB, 0.29249E+00_JPRB, 0.29288E+00_JPRB, 0.29306E+00_JPRB, 0.29248E+00_JPRB, 0.29132E+00_JPRB, 0.29286E+00_JPRB ,&
&  0.29421E+00_JPRB, 0.29484E+00_JPRB, 0.29621E+00_JPRB, 0.29759E+00_JPRB, 0.30058E+00_JPRB, 0.30408E+00_JPRB, 0.30625E+00_JPRB ,&
&  0.30660E+00_JPRB, 0.30494E+00_JPRB, 0.29813E+00_JPRB, 0.29121E+00_JPRB, 0.29135E+00_JPRB, 0.29229E+00_JPRB, 0.29211E+00_JPRB ,&
&  0.28993E+00_JPRB, 0.28705E+00_JPRB, 0.28511E+00_JPRB, 0.28358E+00_JPRB, 0.28169E+00_JPRB, 0.27827E+00_JPRB, 0.27452E+00_JPRB ,&
&  0.27136E+00_JPRB, 0.27035E+00_JPRB, 0.26912E+00_JPRB, 0.26685E+00_JPRB, 0.26337E+00_JPRB, 0.26052E+00_JPRB, 0.25756E+00_JPRB ,&
&  0.25238E+00_JPRB, 0.24735E+00_JPRB, 0.24464E+00_JPRB, 0.24399E+00_JPRB, 0.24435E+00_JPRB, 0.24389E+00_JPRB, 0.24206E+00_JPRB ,&
&  0.23994E+00_JPRB, 0.23732E+00_JPRB, 0.23575E+00_JPRB, 0.23537E+00_JPRB, 0.23661E+00_JPRB, 0.23544E+00_JPRB, 0.23114E+00_JPRB ,&
&  0.22883E+00_JPRB, 0.22760E+00_JPRB, 0.22809E+00_JPRB, 0.22741E+00_JPRB, 0.22683E+00_JPRB, 0.22581E+00_JPRB, 0.22623E+00_JPRB ,&
&  0.22735E+00_JPRB, 0.22759E+00_JPRB, 0.23018E+00_JPRB, 0.23478E+00_JPRB, 0.23889E+00_JPRB, 0.24130E+00_JPRB, 0.24280E+00_JPRB ,&
&  0.24342E+00_JPRB, 0.24298E+00_JPRB, 0.24248E+00_JPRB, 0.24307E+00_JPRB, 0.24527E+00_JPRB, 0.24835E+00_JPRB, 0.25109E+00_JPRB ,&
&  0.25160E+00_JPRB, 0.25266E+00_JPRB, 0.25542E+00_JPRB, 0.25957E+00_JPRB, 0.26202E+00_JPRB, 0.26400E+00_JPRB, 0.26562E+00_JPRB ,&
&  0.26631E+00_JPRB, 0.26772E+00_JPRB, 0.27120E+00_JPRB, 0.27336E+00_JPRB, 0.27568E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.26602E+00_JPRB, 0.26696E+00_JPRB, 0.26886E+00_JPRB, 0.27105E+00_JPRB, 0.27197E+00_JPRB, 0.27138E+00_JPRB, 0.27048E+00_JPRB ,&
&  0.27078E+00_JPRB, 0.27287E+00_JPRB, 0.27509E+00_JPRB, 0.27571E+00_JPRB, 0.27606E+00_JPRB, 0.27653E+00_JPRB, 0.27838E+00_JPRB ,&
&  0.28108E+00_JPRB, 0.28423E+00_JPRB, 0.28436E+00_JPRB, 0.28427E+00_JPRB, 0.28394E+00_JPRB, 0.28300E+00_JPRB, 0.28424E+00_JPRB ,&
&  0.28733E+00_JPRB, 0.28757E+00_JPRB, 0.28736E+00_JPRB, 0.28659E+00_JPRB, 0.28818E+00_JPRB, 0.29129E+00_JPRB, 0.29309E+00_JPRB ,&
&  0.29170E+00_JPRB, 0.28475E+00_JPRB, 0.27654E+00_JPRB, 0.27599E+00_JPRB, 0.27762E+00_JPRB, 0.27927E+00_JPRB, 0.27940E+00_JPRB ,&
&  0.27891E+00_JPRB, 0.27806E+00_JPRB, 0.27701E+00_JPRB, 0.27597E+00_JPRB, 0.27469E+00_JPRB, 0.27089E+00_JPRB, 0.26643E+00_JPRB ,&
&  0.26289E+00_JPRB, 0.25970E+00_JPRB, 0.25653E+00_JPRB, 0.25210E+00_JPRB, 0.24757E+00_JPRB, 0.24429E+00_JPRB, 0.24051E+00_JPRB ,&
&  0.23584E+00_JPRB, 0.23255E+00_JPRB, 0.23156E+00_JPRB, 0.23151E+00_JPRB, 0.23163E+00_JPRB, 0.23082E+00_JPRB, 0.22978E+00_JPRB ,&
&  0.22796E+00_JPRB, 0.22658E+00_JPRB, 0.22507E+00_JPRB, 0.22424E+00_JPRB, 0.21897E+00_JPRB, 0.21090E+00_JPRB, 0.20517E+00_JPRB ,&
&  0.20459E+00_JPRB, 0.20740E+00_JPRB, 0.21120E+00_JPRB, 0.21221E+00_JPRB, 0.21437E+00_JPRB, 0.21546E+00_JPRB, 0.21509E+00_JPRB ,&
&  0.21235E+00_JPRB, 0.20896E+00_JPRB, 0.20935E+00_JPRB, 0.21047E+00_JPRB, 0.21090E+00_JPRB, 0.21207E+00_JPRB, 0.21350E+00_JPRB ,&
&  0.21637E+00_JPRB, 0.21872E+00_JPRB, 0.22106E+00_JPRB, 0.22350E+00_JPRB, 0.22522E+00_JPRB, 0.22524E+00_JPRB, 0.22305E+00_JPRB ,&
&  0.22291E+00_JPRB, 0.22846E+00_JPRB, 0.23829E+00_JPRB, 0.24856E+00_JPRB, 0.25119E+00_JPRB, 0.25356E+00_JPRB, 0.25567E+00_JPRB ,&
&  0.25671E+00_JPRB, 0.25995E+00_JPRB, 0.26265E+00_JPRB, 0.26403E+00_JPRB, 0.26494E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.25797E+00_JPRB, 0.25746E+00_JPRB, 0.25601E+00_JPRB, 0.25474E+00_JPRB, 0.25312E+00_JPRB, 0.25160E+00_JPRB, 0.25221E+00_JPRB ,&
&  0.25472E+00_JPRB, 0.25805E+00_JPRB, 0.26126E+00_JPRB, 0.26372E+00_JPRB, 0.26566E+00_JPRB, 0.26665E+00_JPRB, 0.26817E+00_JPRB ,&
&  0.26995E+00_JPRB, 0.27307E+00_JPRB, 0.27429E+00_JPRB, 0.27537E+00_JPRB, 0.27478E+00_JPRB, 0.27416E+00_JPRB, 0.27531E+00_JPRB ,&
&  0.27857E+00_JPRB, 0.28007E+00_JPRB, 0.27917E+00_JPRB, 0.27702E+00_JPRB, 0.27749E+00_JPRB, 0.27938E+00_JPRB, 0.27874E+00_JPRB ,&
&  0.27198E+00_JPRB, 0.25846E+00_JPRB, 0.24926E+00_JPRB, 0.25213E+00_JPRB, 0.26051E+00_JPRB, 0.26434E+00_JPRB, 0.26709E+00_JPRB ,&
&  0.26884E+00_JPRB, 0.26910E+00_JPRB, 0.26830E+00_JPRB, 0.26683E+00_JPRB, 0.26425E+00_JPRB, 0.25971E+00_JPRB, 0.25428E+00_JPRB ,&
&  0.24942E+00_JPRB, 0.24702E+00_JPRB, 0.24405E+00_JPRB, 0.23869E+00_JPRB, 0.23345E+00_JPRB, 0.23018E+00_JPRB, 0.22620E+00_JPRB ,&
&  0.22212E+00_JPRB, 0.21912E+00_JPRB, 0.21845E+00_JPRB, 0.21805E+00_JPRB, 0.21792E+00_JPRB, 0.21728E+00_JPRB, 0.21493E+00_JPRB ,&
&  0.21163E+00_JPRB, 0.20732E+00_JPRB, 0.20167E+00_JPRB, 0.19655E+00_JPRB, 0.19296E+00_JPRB, 0.18938E+00_JPRB, 0.18505E+00_JPRB ,&
&  0.18291E+00_JPRB, 0.18543E+00_JPRB, 0.19423E+00_JPRB, 0.20396E+00_JPRB, 0.20877E+00_JPRB, 0.19722E+00_JPRB, 0.19326E+00_JPRB ,&
&  0.19253E+00_JPRB, 0.19233E+00_JPRB, 0.19268E+00_JPRB, 0.19407E+00_JPRB, 0.19473E+00_JPRB, 0.19445E+00_JPRB, 0.19504E+00_JPRB ,&
&  0.19714E+00_JPRB, 0.19931E+00_JPRB, 0.20045E+00_JPRB, 0.20173E+00_JPRB, 0.20251E+00_JPRB, 0.20231E+00_JPRB, 0.20190E+00_JPRB ,&
&  0.20256E+00_JPRB, 0.20566E+00_JPRB, 0.21073E+00_JPRB, 0.21786E+00_JPRB, 0.22295E+00_JPRB, 0.22760E+00_JPRB, 0.23349E+00_JPRB ,&
&  0.24096E+00_JPRB, 0.24816E+00_JPRB, 0.25228E+00_JPRB, 0.25587E+00_JPRB, 0.25737E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.25047E+00_JPRB, 0.25091E+00_JPRB, 0.24692E+00_JPRB, 0.24343E+00_JPRB, 0.24170E+00_JPRB, 0.24123E+00_JPRB, 0.24257E+00_JPRB ,&
&  0.24567E+00_JPRB, 0.24811E+00_JPRB, 0.25052E+00_JPRB, 0.25262E+00_JPRB, 0.25352E+00_JPRB, 0.25338E+00_JPRB, 0.25442E+00_JPRB ,&
&  0.25547E+00_JPRB, 0.25818E+00_JPRB, 0.25988E+00_JPRB, 0.26270E+00_JPRB, 0.26303E+00_JPRB, 0.26383E+00_JPRB, 0.26509E+00_JPRB ,&
&  0.26466E+00_JPRB, 0.26513E+00_JPRB, 0.26655E+00_JPRB, 0.26424E+00_JPRB, 0.26269E+00_JPRB, 0.26038E+00_JPRB, 0.25850E+00_JPRB ,&
&  0.25193E+00_JPRB, 0.24515E+00_JPRB, 0.23861E+00_JPRB, 0.24220E+00_JPRB, 0.24993E+00_JPRB, 0.25148E+00_JPRB, 0.25540E+00_JPRB ,&
&  0.25654E+00_JPRB, 0.25492E+00_JPRB, 0.25330E+00_JPRB, 0.25013E+00_JPRB, 0.24708E+00_JPRB, 0.24364E+00_JPRB, 0.24088E+00_JPRB ,&
&  0.23779E+00_JPRB, 0.23846E+00_JPRB, 0.23438E+00_JPRB, 0.22080E+00_JPRB, 0.21386E+00_JPRB, 0.20951E+00_JPRB, 0.20249E+00_JPRB ,&
&  0.19543E+00_JPRB, 0.19350E+00_JPRB, 0.19311E+00_JPRB, 0.19072E+00_JPRB, 0.19061E+00_JPRB, 0.19093E+00_JPRB, 0.19085E+00_JPRB ,&
&  0.18950E+00_JPRB, 0.18923E+00_JPRB, 0.18788E+00_JPRB, 0.18586E+00_JPRB, 0.18391E+00_JPRB, 0.18310E+00_JPRB, 0.18348E+00_JPRB ,&
&  0.18498E+00_JPRB, 0.18660E+00_JPRB, 0.19225E+00_JPRB, 0.19579E+00_JPRB, 0.19095E+00_JPRB, 0.18839E+00_JPRB, 0.18885E+00_JPRB ,&
&  0.18487E+00_JPRB, 0.18412E+00_JPRB, 0.18411E+00_JPRB, 0.18410E+00_JPRB, 0.18453E+00_JPRB, 0.18547E+00_JPRB, 0.18541E+00_JPRB ,&
&  0.18610E+00_JPRB, 0.18748E+00_JPRB, 0.18785E+00_JPRB, 0.18786E+00_JPRB, 0.18756E+00_JPRB, 0.18799E+00_JPRB, 0.18921E+00_JPRB ,&
&  0.19065E+00_JPRB, 0.19338E+00_JPRB, 0.19623E+00_JPRB, 0.20042E+00_JPRB, 0.20379E+00_JPRB, 0.20648E+00_JPRB, 0.21136E+00_JPRB ,&
&  0.21550E+00_JPRB, 0.22491E+00_JPRB, 0.24068E+00_JPRB, 0.25048E+00_JPRB, 0.24635E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.24086E+00_JPRB, 0.23951E+00_JPRB, 0.23802E+00_JPRB, 0.23658E+00_JPRB, 0.23742E+00_JPRB, 0.23859E+00_JPRB, 0.24021E+00_JPRB ,&
&  0.24200E+00_JPRB, 0.24251E+00_JPRB, 0.24235E+00_JPRB, 0.24010E+00_JPRB, 0.23886E+00_JPRB, 0.23288E+00_JPRB, 0.23075E+00_JPRB ,&
&  0.22921E+00_JPRB, 0.23125E+00_JPRB, 0.23400E+00_JPRB, 0.23811E+00_JPRB, 0.23984E+00_JPRB, 0.24165E+00_JPRB, 0.24069E+00_JPRB ,&
&  0.23849E+00_JPRB, 0.23584E+00_JPRB, 0.23326E+00_JPRB, 0.23123E+00_JPRB, 0.23061E+00_JPRB, 0.22887E+00_JPRB, 0.22705E+00_JPRB ,&
&  0.22698E+00_JPRB, 0.22839E+00_JPRB, 0.22912E+00_JPRB, 0.22986E+00_JPRB, 0.23570E+00_JPRB, 0.23616E+00_JPRB, 0.23829E+00_JPRB ,&
&  0.23895E+00_JPRB, 0.23967E+00_JPRB, 0.23914E+00_JPRB, 0.23786E+00_JPRB, 0.23667E+00_JPRB, 0.23115E+00_JPRB, 0.22884E+00_JPRB ,&
&  0.22176E+00_JPRB, 0.21762E+00_JPRB, 0.21081E+00_JPRB, 0.20462E+00_JPRB, 0.19776E+00_JPRB, 0.19263E+00_JPRB, 0.18830E+00_JPRB ,&
&  0.18484E+00_JPRB, 0.18226E+00_JPRB, 0.18016E+00_JPRB, 0.17989E+00_JPRB, 0.17996E+00_JPRB, 0.17955E+00_JPRB, 0.17971E+00_JPRB ,&
&  0.18058E+00_JPRB, 0.18152E+00_JPRB, 0.18144E+00_JPRB, 0.18201E+00_JPRB, 0.18253E+00_JPRB, 0.18329E+00_JPRB, 0.18478E+00_JPRB ,&
&  0.18707E+00_JPRB, 0.18902E+00_JPRB, 0.18973E+00_JPRB, 0.19166E+00_JPRB, 0.19101E+00_JPRB, 0.18835E+00_JPRB, 0.18573E+00_JPRB ,&
&  0.18275E+00_JPRB, 0.18147E+00_JPRB, 0.18002E+00_JPRB, 0.17922E+00_JPRB, 0.17913E+00_JPRB, 0.17933E+00_JPRB, 0.18014E+00_JPRB ,&
&  0.18100E+00_JPRB, 0.18189E+00_JPRB, 0.18222E+00_JPRB, 0.18266E+00_JPRB, 0.18303E+00_JPRB, 0.18388E+00_JPRB, 0.18552E+00_JPRB ,&
&  0.18777E+00_JPRB, 0.19037E+00_JPRB, 0.19293E+00_JPRB, 0.19602E+00_JPRB, 0.20108E+00_JPRB, 0.20784E+00_JPRB, 0.21805E+00_JPRB ,&
&  0.22601E+00_JPRB, 0.23823E+00_JPRB, 0.23790E+00_JPRB, 0.24013E+00_JPRB, 0.24136E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.23374E+00_JPRB, 0.23358E+00_JPRB, 0.23274E+00_JPRB, 0.23219E+00_JPRB, 0.23264E+00_JPRB, 0.23344E+00_JPRB, 0.23453E+00_JPRB ,&
&  0.23112E+00_JPRB, 0.22864E+00_JPRB, 0.22430E+00_JPRB, 0.21952E+00_JPRB, 0.21530E+00_JPRB, 0.21204E+00_JPRB, 0.21017E+00_JPRB ,&
&  0.21000E+00_JPRB, 0.21219E+00_JPRB, 0.21386E+00_JPRB, 0.21593E+00_JPRB, 0.21607E+00_JPRB, 0.21626E+00_JPRB, 0.21481E+00_JPRB ,&
&  0.21325E+00_JPRB, 0.21264E+00_JPRB, 0.21289E+00_JPRB, 0.21451E+00_JPRB, 0.21717E+00_JPRB, 0.21888E+00_JPRB, 0.22096E+00_JPRB ,&
&  0.22375E+00_JPRB, 0.22295E+00_JPRB, 0.22368E+00_JPRB, 0.22306E+00_JPRB, 0.22346E+00_JPRB, 0.22450E+00_JPRB, 0.22504E+00_JPRB ,&
&  0.22611E+00_JPRB, 0.22714E+00_JPRB, 0.22718E+00_JPRB, 0.22649E+00_JPRB, 0.22224E+00_JPRB, 0.21978E+00_JPRB, 0.21517E+00_JPRB ,&
&  0.20963E+00_JPRB, 0.20463E+00_JPRB, 0.19881E+00_JPRB, 0.19473E+00_JPRB, 0.18988E+00_JPRB, 0.18702E+00_JPRB, 0.18481E+00_JPRB ,&
&  0.18272E+00_JPRB, 0.18194E+00_JPRB, 0.18081E+00_JPRB, 0.17959E+00_JPRB, 0.17929E+00_JPRB, 0.17807E+00_JPRB, 0.17798E+00_JPRB ,&
&  0.17828E+00_JPRB, 0.17857E+00_JPRB, 0.17958E+00_JPRB, 0.18003E+00_JPRB, 0.18084E+00_JPRB, 0.18158E+00_JPRB, 0.18212E+00_JPRB ,&
&  0.18327E+00_JPRB, 0.18478E+00_JPRB, 0.18479E+00_JPRB, 0.18515E+00_JPRB, 0.18433E+00_JPRB, 0.18250E+00_JPRB, 0.18052E+00_JPRB ,&
&  0.17838E+00_JPRB, 0.17673E+00_JPRB, 0.17622E+00_JPRB, 0.17549E+00_JPRB, 0.17564E+00_JPRB, 0.17603E+00_JPRB, 0.17620E+00_JPRB ,&
&  0.17686E+00_JPRB, 0.17746E+00_JPRB, 0.17832E+00_JPRB, 0.18024E+00_JPRB, 0.18157E+00_JPRB, 0.18329E+00_JPRB, 0.18511E+00_JPRB ,&
&  0.18656E+00_JPRB, 0.18920E+00_JPRB, 0.19154E+00_JPRB, 0.19472E+00_JPRB, 0.20207E+00_JPRB, 0.20840E+00_JPRB, 0.21971E+00_JPRB ,&
&  0.22453E+00_JPRB, 0.23531E+00_JPRB, 0.23293E+00_JPRB, 0.23316E+00_JPRB, 0.23419E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.22854E+00_JPRB, 0.22939E+00_JPRB, 0.22929E+00_JPRB, 0.23026E+00_JPRB, 0.23103E+00_JPRB, 0.23208E+00_JPRB, 0.23214E+00_JPRB ,&
&  0.23164E+00_JPRB, 0.22756E+00_JPRB, 0.22530E+00_JPRB, 0.22194E+00_JPRB, 0.21879E+00_JPRB, 0.21544E+00_JPRB, 0.21272E+00_JPRB ,&
&  0.21096E+00_JPRB, 0.20961E+00_JPRB, 0.20839E+00_JPRB, 0.20668E+00_JPRB, 0.20456E+00_JPRB, 0.20241E+00_JPRB, 0.20046E+00_JPRB ,&
&  0.19870E+00_JPRB, 0.19880E+00_JPRB, 0.20009E+00_JPRB, 0.20231E+00_JPRB, 0.20640E+00_JPRB, 0.20946E+00_JPRB, 0.21340E+00_JPRB ,&
&  0.21943E+00_JPRB, 0.21599E+00_JPRB, 0.21619E+00_JPRB, 0.21655E+00_JPRB, 0.21636E+00_JPRB, 0.21627E+00_JPRB, 0.21361E+00_JPRB ,&
&  0.21760E+00_JPRB, 0.21786E+00_JPRB, 0.22153E+00_JPRB, 0.22056E+00_JPRB, 0.21585E+00_JPRB, 0.21226E+00_JPRB, 0.20723E+00_JPRB ,&
&  0.20367E+00_JPRB, 0.19907E+00_JPRB, 0.19597E+00_JPRB, 0.19312E+00_JPRB, 0.18970E+00_JPRB, 0.18830E+00_JPRB, 0.18593E+00_JPRB ,&
&  0.18450E+00_JPRB, 0.18419E+00_JPRB, 0.18319E+00_JPRB, 0.18315E+00_JPRB, 0.18228E+00_JPRB, 0.18105E+00_JPRB, 0.18026E+00_JPRB ,&
&  0.17957E+00_JPRB, 0.17884E+00_JPRB, 0.17860E+00_JPRB, 0.17833E+00_JPRB, 0.17847E+00_JPRB, 0.17848E+00_JPRB, 0.17854E+00_JPRB ,&
&  0.17875E+00_JPRB, 0.17892E+00_JPRB, 0.17850E+00_JPRB, 0.17838E+00_JPRB, 0.17746E+00_JPRB, 0.17665E+00_JPRB, 0.17538E+00_JPRB ,&
&  0.17459E+00_JPRB, 0.17412E+00_JPRB, 0.17336E+00_JPRB, 0.17286E+00_JPRB, 0.17269E+00_JPRB, 0.17318E+00_JPRB, 0.17380E+00_JPRB ,&
&  0.17452E+00_JPRB, 0.17588E+00_JPRB, 0.17699E+00_JPRB, 0.17848E+00_JPRB, 0.18014E+00_JPRB, 0.18144E+00_JPRB, 0.18331E+00_JPRB ,&
&  0.18516E+00_JPRB, 0.18706E+00_JPRB, 0.19031E+00_JPRB, 0.19390E+00_JPRB, 0.19864E+00_JPRB, 0.20374E+00_JPRB, 0.20633E+00_JPRB ,&
&  0.20920E+00_JPRB, 0.21581E+00_JPRB, 0.21845E+00_JPRB, 0.22644E+00_JPRB, 0.22770E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.21884E+00_JPRB, 0.22043E+00_JPRB, 0.22197E+00_JPRB, 0.22344E+00_JPRB, 0.22514E+00_JPRB, 0.22582E+00_JPRB, 0.22579E+00_JPRB ,&
&  0.22342E+00_JPRB, 0.22290E+00_JPRB, 0.22137E+00_JPRB, 0.21942E+00_JPRB, 0.21785E+00_JPRB, 0.21473E+00_JPRB, 0.21187E+00_JPRB ,&
&  0.20901E+00_JPRB, 0.20627E+00_JPRB, 0.20386E+00_JPRB, 0.20053E+00_JPRB, 0.19736E+00_JPRB, 0.19421E+00_JPRB, 0.19337E+00_JPRB ,&
&  0.19308E+00_JPRB, 0.19369E+00_JPRB, 0.19552E+00_JPRB, 0.19696E+00_JPRB, 0.19906E+00_JPRB, 0.20129E+00_JPRB, 0.20464E+00_JPRB ,&
&  0.20779E+00_JPRB, 0.21011E+00_JPRB, 0.21042E+00_JPRB, 0.20922E+00_JPRB, 0.20831E+00_JPRB, 0.20810E+00_JPRB, 0.20778E+00_JPRB ,&
&  0.20712E+00_JPRB, 0.20683E+00_JPRB, 0.20632E+00_JPRB, 0.20591E+00_JPRB, 0.20452E+00_JPRB, 0.20196E+00_JPRB, 0.19935E+00_JPRB ,&
&  0.19689E+00_JPRB, 0.19471E+00_JPRB, 0.19262E+00_JPRB, 0.19078E+00_JPRB, 0.18969E+00_JPRB, 0.18860E+00_JPRB, 0.18721E+00_JPRB ,&
&  0.18639E+00_JPRB, 0.18554E+00_JPRB, 0.18456E+00_JPRB, 0.18384E+00_JPRB, 0.18306E+00_JPRB, 0.18250E+00_JPRB, 0.18186E+00_JPRB ,&
&  0.18139E+00_JPRB, 0.18064E+00_JPRB, 0.18001E+00_JPRB, 0.17931E+00_JPRB, 0.17902E+00_JPRB, 0.17822E+00_JPRB, 0.17747E+00_JPRB ,&
&  0.17652E+00_JPRB, 0.17593E+00_JPRB, 0.17484E+00_JPRB, 0.17420E+00_JPRB, 0.17343E+00_JPRB, 0.17272E+00_JPRB, 0.17222E+00_JPRB ,&
&  0.17224E+00_JPRB, 0.17218E+00_JPRB, 0.17165E+00_JPRB, 0.17149E+00_JPRB, 0.17148E+00_JPRB, 0.17190E+00_JPRB, 0.17324E+00_JPRB ,&
&  0.17411E+00_JPRB, 0.17506E+00_JPRB, 0.17598E+00_JPRB, 0.17668E+00_JPRB, 0.17789E+00_JPRB, 0.17905E+00_JPRB, 0.18024E+00_JPRB ,&
&  0.18165E+00_JPRB, 0.18285E+00_JPRB, 0.18425E+00_JPRB, 0.18589E+00_JPRB, 0.18997E+00_JPRB, 0.19273E+00_JPRB, 0.19558E+00_JPRB ,&
&  0.19754E+00_JPRB, 0.19598E+00_JPRB, 0.21031E+00_JPRB, 0.21303E+00_JPRB, 0.21509E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.19380E+00_JPRB, 0.19671E+00_JPRB, 0.19847E+00_JPRB, 0.19990E+00_JPRB, 0.19341E+00_JPRB, 0.20488E+00_JPRB, 0.20159E+00_JPRB ,&
&  0.20109E+00_JPRB, 0.20002E+00_JPRB, 0.20182E+00_JPRB, 0.20196E+00_JPRB, 0.20140E+00_JPRB, 0.20053E+00_JPRB, 0.19938E+00_JPRB ,&
&  0.19828E+00_JPRB, 0.19686E+00_JPRB, 0.19598E+00_JPRB, 0.19502E+00_JPRB, 0.19427E+00_JPRB, 0.19350E+00_JPRB, 0.19299E+00_JPRB ,&
&  0.19303E+00_JPRB, 0.19390E+00_JPRB, 0.19541E+00_JPRB, 0.19478E+00_JPRB, 0.19477E+00_JPRB, 0.19517E+00_JPRB, 0.19578E+00_JPRB ,&
&  0.19604E+00_JPRB, 0.19615E+00_JPRB, 0.19586E+00_JPRB, 0.19476E+00_JPRB, 0.19483E+00_JPRB, 0.19493E+00_JPRB, 0.19512E+00_JPRB ,&
&  0.19512E+00_JPRB, 0.19489E+00_JPRB, 0.19404E+00_JPRB, 0.19325E+00_JPRB, 0.19243E+00_JPRB, 0.19144E+00_JPRB, 0.19023E+00_JPRB ,&
&  0.18923E+00_JPRB, 0.18845E+00_JPRB, 0.18751E+00_JPRB, 0.18658E+00_JPRB, 0.18576E+00_JPRB, 0.18497E+00_JPRB, 0.18421E+00_JPRB ,&
&  0.18373E+00_JPRB, 0.18337E+00_JPRB, 0.18297E+00_JPRB, 0.18240E+00_JPRB, 0.18176E+00_JPRB, 0.18114E+00_JPRB, 0.18059E+00_JPRB ,&
&  0.18024E+00_JPRB, 0.17964E+00_JPRB, 0.17905E+00_JPRB, 0.17840E+00_JPRB, 0.17791E+00_JPRB, 0.17711E+00_JPRB, 0.17640E+00_JPRB ,&
&  0.17572E+00_JPRB, 0.17510E+00_JPRB, 0.17462E+00_JPRB, 0.17426E+00_JPRB, 0.17393E+00_JPRB, 0.17368E+00_JPRB, 0.17350E+00_JPRB ,&
&  0.17336E+00_JPRB, 0.17325E+00_JPRB, 0.17322E+00_JPRB, 0.17313E+00_JPRB, 0.17300E+00_JPRB, 0.17296E+00_JPRB, 0.17324E+00_JPRB ,&
&  0.17348E+00_JPRB, 0.17365E+00_JPRB, 0.17374E+00_JPRB, 0.17343E+00_JPRB, 0.17383E+00_JPRB, 0.17418E+00_JPRB, 0.17461E+00_JPRB ,&
&  0.17499E+00_JPRB, 0.17580E+00_JPRB, 0.17678E+00_JPRB, 0.17796E+00_JPRB, 0.18024E+00_JPRB, 0.18224E+00_JPRB, 0.18437E+00_JPRB ,&
&  0.18675E+00_JPRB, 0.19021E+00_JPRB, 0.19162E+00_JPRB, 0.19251E+00_JPRB, 0.19319E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.18279E+00_JPRB, 0.18272E+00_JPRB, 0.18250E+00_JPRB, 0.18169E+00_JPRB, 0.17932E+00_JPRB, 0.17889E+00_JPRB, 0.17886E+00_JPRB ,&
&  0.17904E+00_JPRB, 0.17916E+00_JPRB, 0.18007E+00_JPRB, 0.18106E+00_JPRB, 0.18203E+00_JPRB, 0.18278E+00_JPRB, 0.18343E+00_JPRB ,&
&  0.18378E+00_JPRB, 0.18413E+00_JPRB, 0.18297E+00_JPRB, 0.18241E+00_JPRB, 0.18225E+00_JPRB, 0.18225E+00_JPRB, 0.18225E+00_JPRB ,&
&  0.18220E+00_JPRB, 0.18219E+00_JPRB, 0.18207E+00_JPRB, 0.18228E+00_JPRB, 0.18261E+00_JPRB, 0.18289E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18345E+00_JPRB, 0.18366E+00_JPRB, 0.18380E+00_JPRB, 0.18391E+00_JPRB, 0.18426E+00_JPRB, 0.18444E+00_JPRB, 0.18437E+00_JPRB ,&
&  0.18421E+00_JPRB, 0.18386E+00_JPRB, 0.18347E+00_JPRB, 0.18306E+00_JPRB, 0.18277E+00_JPRB, 0.18249E+00_JPRB, 0.18221E+00_JPRB ,&
&  0.18192E+00_JPRB, 0.18162E+00_JPRB, 0.18129E+00_JPRB, 0.18091E+00_JPRB, 0.18056E+00_JPRB, 0.18030E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.17977E+00_JPRB, 0.17954E+00_JPRB, 0.17929E+00_JPRB, 0.17897E+00_JPRB, 0.17863E+00_JPRB, 0.17829E+00_JPRB, 0.17792E+00_JPRB ,&
&  0.17744E+00_JPRB, 0.17710E+00_JPRB, 0.17677E+00_JPRB, 0.17647E+00_JPRB, 0.17621E+00_JPRB, 0.17594E+00_JPRB, 0.17570E+00_JPRB ,&
&  0.17544E+00_JPRB, 0.17520E+00_JPRB, 0.17491E+00_JPRB, 0.17465E+00_JPRB, 0.17441E+00_JPRB, 0.17420E+00_JPRB, 0.17402E+00_JPRB ,&
&  0.17385E+00_JPRB, 0.17368E+00_JPRB, 0.17353E+00_JPRB, 0.17353E+00_JPRB, 0.17349E+00_JPRB, 0.17351E+00_JPRB, 0.17352E+00_JPRB ,&
&  0.17355E+00_JPRB, 0.17356E+00_JPRB, 0.17356E+00_JPRB, 0.17344E+00_JPRB, 0.17359E+00_JPRB, 0.17379E+00_JPRB, 0.17411E+00_JPRB ,&
&  0.17449E+00_JPRB, 0.17497E+00_JPRB, 0.17549E+00_JPRB, 0.17604E+00_JPRB, 0.17668E+00_JPRB, 0.17721E+00_JPRB, 0.17780E+00_JPRB ,&
&  0.17847E+00_JPRB, 0.17915E+00_JPRB, 0.17965E+00_JPRB, 0.18024E+00_JPRB, 0.18114E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB, 0.17676E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB ,&
&  0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB, 0.53476E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.54377E+00_JPRB, 0.54307E+00_JPRB, 0.54251E+00_JPRB, 0.54226E+00_JPRB, 0.54223E+00_JPRB, 0.54214E+00_JPRB, 0.54132E+00_JPRB ,&
&  0.54031E+00_JPRB, 0.53897E+00_JPRB, 0.53752E+00_JPRB, 0.53579E+00_JPRB, 0.53367E+00_JPRB, 0.53116E+00_JPRB, 0.52885E+00_JPRB ,&
&  0.52668E+00_JPRB, 0.52491E+00_JPRB, 0.52304E+00_JPRB, 0.52171E+00_JPRB, 0.52017E+00_JPRB, 0.51855E+00_JPRB, 0.51688E+00_JPRB ,&
&  0.51554E+00_JPRB, 0.51429E+00_JPRB, 0.51310E+00_JPRB, 0.51202E+00_JPRB, 0.51148E+00_JPRB, 0.51084E+00_JPRB, 0.51017E+00_JPRB ,&
&  0.50959E+00_JPRB, 0.50922E+00_JPRB, 0.50890E+00_JPRB, 0.50857E+00_JPRB, 0.50831E+00_JPRB, 0.50840E+00_JPRB, 0.50850E+00_JPRB ,&
&  0.50874E+00_JPRB, 0.50947E+00_JPRB, 0.51063E+00_JPRB, 0.51170E+00_JPRB, 0.51287E+00_JPRB, 0.51453E+00_JPRB, 0.51651E+00_JPRB ,&
&  0.51823E+00_JPRB, 0.51983E+00_JPRB, 0.52169E+00_JPRB, 0.52378E+00_JPRB, 0.52572E+00_JPRB, 0.52778E+00_JPRB, 0.52975E+00_JPRB ,&
&  0.53173E+00_JPRB, 0.53335E+00_JPRB, 0.53501E+00_JPRB, 0.53575E+00_JPRB, 0.53685E+00_JPRB, 0.53776E+00_JPRB, 0.53885E+00_JPRB ,&
&  0.53908E+00_JPRB, 0.53919E+00_JPRB, 0.53941E+00_JPRB, 0.53984E+00_JPRB, 0.54008E+00_JPRB, 0.54043E+00_JPRB, 0.54087E+00_JPRB ,&
&  0.54153E+00_JPRB, 0.54187E+00_JPRB, 0.54244E+00_JPRB, 0.54285E+00_JPRB, 0.54339E+00_JPRB, 0.54351E+00_JPRB, 0.54385E+00_JPRB ,&
&  0.54421E+00_JPRB, 0.54480E+00_JPRB, 0.54491E+00_JPRB, 0.54509E+00_JPRB, 0.54520E+00_JPRB, 0.54541E+00_JPRB, 0.54560E+00_JPRB ,&
&  0.54589E+00_JPRB, 0.54609E+00_JPRB, 0.54630E+00_JPRB, 0.54620E+00_JPRB, 0.54582E+00_JPRB, 0.54549E+00_JPRB, 0.54525E+00_JPRB ,&
&  0.54519E+00_JPRB, 0.54513E+00_JPRB, 0.54537E+00_JPRB, 0.54575E+00_JPRB, 0.54646E+00_JPRB, 0.54660E+00_JPRB, 0.54671E+00_JPRB ,&
&  0.54671E+00_JPRB, 0.54665E+00_JPRB, 0.54610E+00_JPRB, 0.54537E+00_JPRB, 0.54452E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.54604E+00_JPRB, 0.54462E+00_JPRB, 0.54371E+00_JPRB, 0.54350E+00_JPRB, 0.54360E+00_JPRB, 0.54355E+00_JPRB, 0.54342E+00_JPRB ,&
&  0.54217E+00_JPRB, 0.53925E+00_JPRB, 0.53578E+00_JPRB, 0.53194E+00_JPRB, 0.52722E+00_JPRB, 0.52286E+00_JPRB, 0.51987E+00_JPRB ,&
&  0.51673E+00_JPRB, 0.51357E+00_JPRB, 0.51051E+00_JPRB, 0.50794E+00_JPRB, 0.50505E+00_JPRB, 0.50202E+00_JPRB, 0.49905E+00_JPRB ,&
&  0.49665E+00_JPRB, 0.49391E+00_JPRB, 0.49086E+00_JPRB, 0.48705E+00_JPRB, 0.48445E+00_JPRB, 0.48170E+00_JPRB, 0.47927E+00_JPRB ,&
&  0.47623E+00_JPRB, 0.47472E+00_JPRB, 0.47383E+00_JPRB, 0.47373E+00_JPRB, 0.47587E+00_JPRB, 0.47760E+00_JPRB, 0.47781E+00_JPRB ,&
&  0.47722E+00_JPRB, 0.47458E+00_JPRB, 0.47493E+00_JPRB, 0.47584E+00_JPRB, 0.47764E+00_JPRB, 0.48139E+00_JPRB, 0.48631E+00_JPRB ,&
&  0.49332E+00_JPRB, 0.50175E+00_JPRB, 0.51034E+00_JPRB, 0.51699E+00_JPRB, 0.52353E+00_JPRB, 0.52954E+00_JPRB, 0.53388E+00_JPRB ,&
&  0.53924E+00_JPRB, 0.54377E+00_JPRB, 0.54843E+00_JPRB, 0.55260E+00_JPRB, 0.55731E+00_JPRB, 0.56100E+00_JPRB, 0.56376E+00_JPRB ,&
&  0.56445E+00_JPRB, 0.56409E+00_JPRB, 0.56377E+00_JPRB, 0.56365E+00_JPRB, 0.56250E+00_JPRB, 0.56172E+00_JPRB, 0.56160E+00_JPRB ,&
&  0.56197E+00_JPRB, 0.56196E+00_JPRB, 0.56284E+00_JPRB, 0.56393E+00_JPRB, 0.56518E+00_JPRB, 0.56479E+00_JPRB, 0.56495E+00_JPRB ,&
&  0.56486E+00_JPRB, 0.56495E+00_JPRB, 0.56400E+00_JPRB, 0.56218E+00_JPRB, 0.56065E+00_JPRB, 0.55997E+00_JPRB, 0.55885E+00_JPRB ,&
&  0.55776E+00_JPRB, 0.55697E+00_JPRB, 0.55673E+00_JPRB, 0.55613E+00_JPRB, 0.55494E+00_JPRB, 0.55386E+00_JPRB, 0.55345E+00_JPRB ,&
&  0.55302E+00_JPRB, 0.55228E+00_JPRB, 0.55256E+00_JPRB, 0.55297E+00_JPRB, 0.55486E+00_JPRB, 0.55524E+00_JPRB, 0.55536E+00_JPRB ,&
&  0.55461E+00_JPRB, 0.55410E+00_JPRB, 0.55257E+00_JPRB, 0.55039E+00_JPRB, 0.54784E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.55560E+00_JPRB, 0.55499E+00_JPRB, 0.55588E+00_JPRB, 0.55510E+00_JPRB, 0.55378E+00_JPRB, 0.55126E+00_JPRB, 0.54988E+00_JPRB ,&
&  0.54689E+00_JPRB, 0.54186E+00_JPRB, 0.53394E+00_JPRB, 0.52700E+00_JPRB, 0.52006E+00_JPRB, 0.51556E+00_JPRB, 0.51124E+00_JPRB ,&
&  0.50691E+00_JPRB, 0.50333E+00_JPRB, 0.49947E+00_JPRB, 0.49481E+00_JPRB, 0.48908E+00_JPRB, 0.48228E+00_JPRB, 0.47347E+00_JPRB ,&
&  0.46672E+00_JPRB, 0.46025E+00_JPRB, 0.45464E+00_JPRB, 0.45081E+00_JPRB, 0.44969E+00_JPRB, 0.43388E+00_JPRB, 0.43091E+00_JPRB ,&
&  0.43759E+00_JPRB, 0.42926E+00_JPRB, 0.42156E+00_JPRB, 0.40911E+00_JPRB, 0.40677E+00_JPRB, 0.41388E+00_JPRB, 0.41902E+00_JPRB ,&
&  0.41280E+00_JPRB, 0.43328E+00_JPRB, 0.43496E+00_JPRB, 0.43585E+00_JPRB, 0.43444E+00_JPRB, 0.43133E+00_JPRB, 0.43905E+00_JPRB ,&
&  0.45707E+00_JPRB, 0.46487E+00_JPRB, 0.48174E+00_JPRB, 0.49987E+00_JPRB, 0.51183E+00_JPRB, 0.52486E+00_JPRB, 0.53825E+00_JPRB ,&
&  0.55126E+00_JPRB, 0.56090E+00_JPRB, 0.56836E+00_JPRB, 0.57550E+00_JPRB, 0.58578E+00_JPRB, 0.59592E+00_JPRB, 0.60476E+00_JPRB ,&
&  0.61099E+00_JPRB, 0.61472E+00_JPRB, 0.61923E+00_JPRB, 0.62352E+00_JPRB, 0.62961E+00_JPRB, 0.63487E+00_JPRB, 0.64022E+00_JPRB ,&
&  0.64436E+00_JPRB, 0.64762E+00_JPRB, 0.64948E+00_JPRB, 0.64985E+00_JPRB, 0.64950E+00_JPRB, 0.64654E+00_JPRB, 0.64536E+00_JPRB ,&
&  0.64078E+00_JPRB, 0.63588E+00_JPRB, 0.62962E+00_JPRB, 0.62173E+00_JPRB, 0.61583E+00_JPRB, 0.61130E+00_JPRB, 0.60620E+00_JPRB ,&
&  0.60098E+00_JPRB, 0.59579E+00_JPRB, 0.59123E+00_JPRB, 0.58547E+00_JPRB, 0.58011E+00_JPRB, 0.57456E+00_JPRB, 0.57090E+00_JPRB ,&
&  0.56849E+00_JPRB, 0.56668E+00_JPRB, 0.56655E+00_JPRB, 0.56612E+00_JPRB, 0.56731E+00_JPRB, 0.56748E+00_JPRB, 0.56863E+00_JPRB ,&
&  0.56859E+00_JPRB, 0.56606E+00_JPRB, 0.56197E+00_JPRB, 0.55869E+00_JPRB, 0.55595E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.56134E+00_JPRB, 0.56181E+00_JPRB, 0.56529E+00_JPRB, 0.56404E+00_JPRB, 0.56202E+00_JPRB, 0.55615E+00_JPRB, 0.55349E+00_JPRB ,&
&  0.54726E+00_JPRB, 0.54119E+00_JPRB, 0.52847E+00_JPRB, 0.52022E+00_JPRB, 0.51253E+00_JPRB, 0.50797E+00_JPRB, 0.50408E+00_JPRB ,&
&  0.50101E+00_JPRB, 0.49711E+00_JPRB, 0.49232E+00_JPRB, 0.48526E+00_JPRB, 0.47649E+00_JPRB, 0.46643E+00_JPRB, 0.45257E+00_JPRB ,&
&  0.44030E+00_JPRB, 0.43158E+00_JPRB, 0.41942E+00_JPRB, 0.42306E+00_JPRB, 0.41772E+00_JPRB, 0.41371E+00_JPRB, 0.41147E+00_JPRB ,&
&  0.40528E+00_JPRB, 0.39614E+00_JPRB, 0.38367E+00_JPRB, 0.37531E+00_JPRB, 0.36862E+00_JPRB, 0.36138E+00_JPRB, 0.36920E+00_JPRB ,&
&  0.37518E+00_JPRB, 0.39335E+00_JPRB, 0.40079E+00_JPRB, 0.40278E+00_JPRB, 0.40573E+00_JPRB, 0.40636E+00_JPRB, 0.41325E+00_JPRB ,&
&  0.42258E+00_JPRB, 0.43370E+00_JPRB, 0.46969E+00_JPRB, 0.48894E+00_JPRB, 0.50004E+00_JPRB, 0.51370E+00_JPRB, 0.52494E+00_JPRB ,&
&  0.53886E+00_JPRB, 0.54906E+00_JPRB, 0.55759E+00_JPRB, 0.56846E+00_JPRB, 0.58414E+00_JPRB, 0.60232E+00_JPRB, 0.61818E+00_JPRB ,&
&  0.63254E+00_JPRB, 0.64125E+00_JPRB, 0.65253E+00_JPRB, 0.66518E+00_JPRB, 0.68289E+00_JPRB, 0.70199E+00_JPRB, 0.72237E+00_JPRB ,&
&  0.74341E+00_JPRB, 0.76494E+00_JPRB, 0.78198E+00_JPRB, 0.79693E+00_JPRB, 0.80382E+00_JPRB, 0.80358E+00_JPRB, 0.79924E+00_JPRB ,&
&  0.78540E+00_JPRB, 0.77060E+00_JPRB, 0.75138E+00_JPRB, 0.72313E+00_JPRB, 0.70220E+00_JPRB, 0.69154E+00_JPRB, 0.67146E+00_JPRB ,&
&  0.65499E+00_JPRB, 0.63973E+00_JPRB, 0.62602E+00_JPRB, 0.61063E+00_JPRB, 0.59674E+00_JPRB, 0.58359E+00_JPRB, 0.57468E+00_JPRB ,&
&  0.56780E+00_JPRB, 0.56369E+00_JPRB, 0.56167E+00_JPRB, 0.56100E+00_JPRB, 0.56277E+00_JPRB, 0.56429E+00_JPRB, 0.56846E+00_JPRB ,&
&  0.57027E+00_JPRB, 0.56940E+00_JPRB, 0.56554E+00_JPRB, 0.56285E+00_JPRB, 0.56050E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.57038E+00_JPRB, 0.56977E+00_JPRB, 0.56817E+00_JPRB, 0.56096E+00_JPRB, 0.55611E+00_JPRB, 0.55002E+00_JPRB, 0.54773E+00_JPRB ,&
&  0.54088E+00_JPRB, 0.53490E+00_JPRB, 0.52191E+00_JPRB, 0.51403E+00_JPRB, 0.50512E+00_JPRB, 0.50028E+00_JPRB, 0.49561E+00_JPRB ,&
&  0.49265E+00_JPRB, 0.48865E+00_JPRB, 0.48605E+00_JPRB, 0.47950E+00_JPRB, 0.47317E+00_JPRB, 0.46291E+00_JPRB, 0.45232E+00_JPRB ,&
&  0.44068E+00_JPRB, 0.42914E+00_JPRB, 0.41904E+00_JPRB, 0.42058E+00_JPRB, 0.41563E+00_JPRB, 0.41214E+00_JPRB, 0.41001E+00_JPRB ,&
&  0.41398E+00_JPRB, 0.38115E+00_JPRB, 0.38326E+00_JPRB, 0.37229E+00_JPRB, 0.35989E+00_JPRB, 0.34522E+00_JPRB, 0.33415E+00_JPRB ,&
&  0.33629E+00_JPRB, 0.33797E+00_JPRB, 0.34571E+00_JPRB, 0.35782E+00_JPRB, 0.36321E+00_JPRB, 0.37425E+00_JPRB, 0.38299E+00_JPRB ,&
&  0.38285E+00_JPRB, 0.43980E+00_JPRB, 0.46734E+00_JPRB, 0.48276E+00_JPRB, 0.49925E+00_JPRB, 0.51503E+00_JPRB, 0.52910E+00_JPRB ,&
&  0.54827E+00_JPRB, 0.57027E+00_JPRB, 0.59027E+00_JPRB, 0.60940E+00_JPRB, 0.63848E+00_JPRB, 0.65877E+00_JPRB, 0.67644E+00_JPRB ,&
&  0.69180E+00_JPRB, 0.69747E+00_JPRB, 0.70882E+00_JPRB, 0.72519E+00_JPRB, 0.74780E+00_JPRB, 0.77290E+00_JPRB, 0.80159E+00_JPRB ,&
&  0.83314E+00_JPRB, 0.86335E+00_JPRB, 0.89449E+00_JPRB, 0.92058E+00_JPRB, 0.93960E+00_JPRB, 0.95015E+00_JPRB, 0.96101E+00_JPRB ,&
&  0.94696E+00_JPRB, 0.92848E+00_JPRB, 0.89996E+00_JPRB, 0.86331E+00_JPRB, 0.81996E+00_JPRB, 0.77235E+00_JPRB, 0.72492E+00_JPRB ,&
&  0.70078E+00_JPRB, 0.67471E+00_JPRB, 0.65024E+00_JPRB, 0.62657E+00_JPRB, 0.60551E+00_JPRB, 0.58783E+00_JPRB, 0.57741E+00_JPRB ,&
&  0.56797E+00_JPRB, 0.56191E+00_JPRB, 0.55831E+00_JPRB, 0.55594E+00_JPRB, 0.55532E+00_JPRB, 0.55361E+00_JPRB, 0.55493E+00_JPRB ,&
&  0.55772E+00_JPRB, 0.56236E+00_JPRB, 0.56424E+00_JPRB, 0.56646E+00_JPRB, 0.56825E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.58051E+00_JPRB, 0.58339E+00_JPRB, 0.58195E+00_JPRB, 0.57751E+00_JPRB, 0.57360E+00_JPRB, 0.56417E+00_JPRB, 0.55721E+00_JPRB ,&
&  0.55162E+00_JPRB, 0.54628E+00_JPRB, 0.53739E+00_JPRB, 0.52804E+00_JPRB, 0.51608E+00_JPRB, 0.50754E+00_JPRB, 0.49833E+00_JPRB ,&
&  0.49121E+00_JPRB, 0.48283E+00_JPRB, 0.47379E+00_JPRB, 0.46546E+00_JPRB, 0.45906E+00_JPRB, 0.45113E+00_JPRB, 0.44569E+00_JPRB ,&
&  0.43920E+00_JPRB, 0.43046E+00_JPRB, 0.42407E+00_JPRB, 0.42408E+00_JPRB, 0.41869E+00_JPRB, 0.41541E+00_JPRB, 0.41734E+00_JPRB ,&
&  0.42322E+00_JPRB, 0.41933E+00_JPRB, 0.41476E+00_JPRB, 0.40117E+00_JPRB, 0.38015E+00_JPRB, 0.32961E+00_JPRB, 0.33529E+00_JPRB ,&
&  0.33127E+00_JPRB, 0.32464E+00_JPRB, 0.32694E+00_JPRB, 0.33313E+00_JPRB, 0.33985E+00_JPRB, 0.35872E+00_JPRB, 0.36149E+00_JPRB ,&
&  0.35733E+00_JPRB, 0.43698E+00_JPRB, 0.45500E+00_JPRB, 0.47676E+00_JPRB, 0.49919E+00_JPRB, 0.52290E+00_JPRB, 0.54788E+00_JPRB ,&
&  0.56219E+00_JPRB, 0.58507E+00_JPRB, 0.60537E+00_JPRB, 0.63831E+00_JPRB, 0.66651E+00_JPRB, 0.69902E+00_JPRB, 0.72676E+00_JPRB ,&
&  0.74028E+00_JPRB, 0.74757E+00_JPRB, 0.75828E+00_JPRB, 0.76529E+00_JPRB, 0.77701E+00_JPRB, 0.80064E+00_JPRB, 0.83487E+00_JPRB ,&
&  0.88183E+00_JPRB, 0.90422E+00_JPRB, 0.96278E+00_JPRB, 0.10043E+01_JPRB, 0.10227E+01_JPRB, 0.10528E+01_JPRB, 0.10891E+01_JPRB ,&
&  0.10782E+01_JPRB, 0.10717E+01_JPRB, 0.10497E+01_JPRB, 0.97378E+00_JPRB, 0.89396E+00_JPRB, 0.84036E+00_JPRB, 0.79116E+00_JPRB ,&
&  0.75390E+00_JPRB, 0.71702E+00_JPRB, 0.67695E+00_JPRB, 0.64615E+00_JPRB, 0.62220E+00_JPRB, 0.60455E+00_JPRB, 0.59504E+00_JPRB ,&
&  0.58414E+00_JPRB, 0.57457E+00_JPRB, 0.56613E+00_JPRB, 0.56311E+00_JPRB, 0.55558E+00_JPRB, 0.54924E+00_JPRB, 0.54274E+00_JPRB ,&
&  0.54270E+00_JPRB, 0.54545E+00_JPRB, 0.55451E+00_JPRB, 0.56413E+00_JPRB, 0.57085E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.58418E+00_JPRB, 0.59979E+00_JPRB, 0.60912E+00_JPRB, 0.61318E+00_JPRB, 0.61817E+00_JPRB, 0.60409E+00_JPRB, 0.59082E+00_JPRB ,&
&  0.58024E+00_JPRB, 0.56850E+00_JPRB, 0.55784E+00_JPRB, 0.54752E+00_JPRB, 0.53704E+00_JPRB, 0.52938E+00_JPRB, 0.52248E+00_JPRB ,&
&  0.51664E+00_JPRB, 0.48551E+00_JPRB, 0.46990E+00_JPRB, 0.46000E+00_JPRB, 0.45105E+00_JPRB, 0.44699E+00_JPRB, 0.44561E+00_JPRB ,&
&  0.43994E+00_JPRB, 0.43602E+00_JPRB, 0.43517E+00_JPRB, 0.43155E+00_JPRB, 0.42511E+00_JPRB, 0.41674E+00_JPRB, 0.41763E+00_JPRB ,&
&  0.42679E+00_JPRB, 0.41733E+00_JPRB, 0.40721E+00_JPRB, 0.39724E+00_JPRB, 0.38075E+00_JPRB, 0.35773E+00_JPRB, 0.34360E+00_JPRB ,&
&  0.33836E+00_JPRB, 0.32213E+00_JPRB, 0.32316E+00_JPRB, 0.32556E+00_JPRB, 0.32686E+00_JPRB, 0.33578E+00_JPRB, 0.35479E+00_JPRB ,&
&  0.37774E+00_JPRB, 0.44382E+00_JPRB, 0.46940E+00_JPRB, 0.50307E+00_JPRB, 0.53460E+00_JPRB, 0.56104E+00_JPRB, 0.58520E+00_JPRB ,&
&  0.60147E+00_JPRB, 0.62513E+00_JPRB, 0.65286E+00_JPRB, 0.68759E+00_JPRB, 0.72059E+00_JPRB, 0.74429E+00_JPRB, 0.77341E+00_JPRB ,&
&  0.81049E+00_JPRB, 0.80059E+00_JPRB, 0.80794E+00_JPRB, 0.79919E+00_JPRB, 0.80460E+00_JPRB, 0.83377E+00_JPRB, 0.82839E+00_JPRB ,&
&  0.85660E+00_JPRB, 0.91526E+00_JPRB, 0.10084E+01_JPRB, 0.11052E+01_JPRB, 0.11498E+01_JPRB, 0.12008E+01_JPRB, 0.12241E+01_JPRB ,&
&  0.11678E+01_JPRB, 0.11350E+01_JPRB, 0.11647E+01_JPRB, 0.10754E+01_JPRB, 0.97875E+00_JPRB, 0.89833E+00_JPRB, 0.84347E+00_JPRB ,&
&  0.78796E+00_JPRB, 0.73273E+00_JPRB, 0.69741E+00_JPRB, 0.66997E+00_JPRB, 0.64223E+00_JPRB, 0.61910E+00_JPRB, 0.61606E+00_JPRB ,&
&  0.61349E+00_JPRB, 0.60973E+00_JPRB, 0.60266E+00_JPRB, 0.59353E+00_JPRB, 0.57553E+00_JPRB, 0.55576E+00_JPRB, 0.53834E+00_JPRB ,&
&  0.53381E+00_JPRB, 0.53560E+00_JPRB, 0.54641E+00_JPRB, 0.55710E+00_JPRB, 0.56926E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.57739E+00_JPRB, 0.59027E+00_JPRB, 0.60105E+00_JPRB, 0.61253E+00_JPRB, 0.63347E+00_JPRB, 0.59832E+00_JPRB, 0.57397E+00_JPRB ,&
&  0.56092E+00_JPRB, 0.56183E+00_JPRB, 0.56205E+00_JPRB, 0.55628E+00_JPRB, 0.54609E+00_JPRB, 0.53625E+00_JPRB, 0.52861E+00_JPRB ,&
&  0.52231E+00_JPRB, 0.50734E+00_JPRB, 0.48282E+00_JPRB, 0.44658E+00_JPRB, 0.43705E+00_JPRB, 0.43538E+00_JPRB, 0.43859E+00_JPRB ,&
&  0.44008E+00_JPRB, 0.43596E+00_JPRB, 0.43543E+00_JPRB, 0.42877E+00_JPRB, 0.42330E+00_JPRB, 0.40961E+00_JPRB, 0.39772E+00_JPRB ,&
&  0.39948E+00_JPRB, 0.40006E+00_JPRB, 0.41470E+00_JPRB, 0.40469E+00_JPRB, 0.39983E+00_JPRB, 0.39980E+00_JPRB, 0.34200E+00_JPRB ,&
&  0.33123E+00_JPRB, 0.31915E+00_JPRB, 0.31563E+00_JPRB, 0.31536E+00_JPRB, 0.32758E+00_JPRB, 0.34000E+00_JPRB, 0.35947E+00_JPRB ,&
&  0.38785E+00_JPRB, 0.45440E+00_JPRB, 0.47742E+00_JPRB, 0.50561E+00_JPRB, 0.53427E+00_JPRB, 0.56565E+00_JPRB, 0.60600E+00_JPRB ,&
&  0.64260E+00_JPRB, 0.67762E+00_JPRB, 0.70486E+00_JPRB, 0.73355E+00_JPRB, 0.76344E+00_JPRB, 0.73370E+00_JPRB, 0.77613E+00_JPRB ,&
&  0.99806E+00_JPRB, 0.95385E+00_JPRB, 0.90159E+00_JPRB, 0.87138E+00_JPRB, 0.86796E+00_JPRB, 0.86700E+00_JPRB, 0.86777E+00_JPRB ,&
&  0.92237E+00_JPRB, 0.10286E+01_JPRB, 0.12001E+01_JPRB, 0.13266E+01_JPRB, 0.13330E+01_JPRB, 0.13541E+01_JPRB, 0.13406E+01_JPRB ,&
&  0.12510E+01_JPRB, 0.11938E+01_JPRB, 0.12586E+01_JPRB, 0.10401E+01_JPRB, 0.89087E+00_JPRB, 0.81569E+00_JPRB, 0.78519E+00_JPRB ,&
&  0.75006E+00_JPRB, 0.71143E+00_JPRB, 0.68962E+00_JPRB, 0.66267E+00_JPRB, 0.62690E+00_JPRB, 0.60642E+00_JPRB, 0.60352E+00_JPRB ,&
&  0.60158E+00_JPRB, 0.59672E+00_JPRB, 0.59325E+00_JPRB, 0.58673E+00_JPRB, 0.57325E+00_JPRB, 0.55727E+00_JPRB, 0.55258E+00_JPRB ,&
&  0.55385E+00_JPRB, 0.55461E+00_JPRB, 0.55586E+00_JPRB, 0.55788E+00_JPRB, 0.56470E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.56360E+00_JPRB, 0.57821E+00_JPRB, 0.59512E+00_JPRB, 0.59328E+00_JPRB, 0.59241E+00_JPRB, 0.54008E+00_JPRB, 0.48445E+00_JPRB ,&
&  0.44667E+00_JPRB, 0.45029E+00_JPRB, 0.46711E+00_JPRB, 0.48087E+00_JPRB, 0.48094E+00_JPRB, 0.47182E+00_JPRB, 0.47031E+00_JPRB ,&
&  0.47634E+00_JPRB, 0.47449E+00_JPRB, 0.45574E+00_JPRB, 0.44539E+00_JPRB, 0.42760E+00_JPRB, 0.42575E+00_JPRB, 0.42859E+00_JPRB ,&
&  0.43190E+00_JPRB, 0.43069E+00_JPRB, 0.42866E+00_JPRB, 0.42531E+00_JPRB, 0.42391E+00_JPRB, 0.42266E+00_JPRB, 0.41799E+00_JPRB ,&
&  0.40799E+00_JPRB, 0.40335E+00_JPRB, 0.40968E+00_JPRB, 0.42666E+00_JPRB, 0.42431E+00_JPRB, 0.42420E+00_JPRB, 0.35949E+00_JPRB ,&
&  0.33363E+00_JPRB, 0.31705E+00_JPRB, 0.31699E+00_JPRB, 0.32865E+00_JPRB, 0.35669E+00_JPRB, 0.38012E+00_JPRB, 0.43403E+00_JPRB ,&
&  0.48228E+00_JPRB, 0.50808E+00_JPRB, 0.52139E+00_JPRB, 0.53225E+00_JPRB, 0.55947E+00_JPRB, 0.61572E+00_JPRB, 0.68416E+00_JPRB ,&
&  0.72726E+00_JPRB, 0.75641E+00_JPRB, 0.75934E+00_JPRB, 0.62992E+00_JPRB, 0.61906E+00_JPRB, 0.66773E+00_JPRB, 0.73509E+00_JPRB ,&
&  0.82916E+00_JPRB, 0.84941E+00_JPRB, 0.85832E+00_JPRB, 0.89786E+00_JPRB, 0.88506E+00_JPRB, 0.87224E+00_JPRB, 0.85287E+00_JPRB ,&
&  0.93559E+00_JPRB, 0.10780E+01_JPRB, 0.13097E+01_JPRB, 0.14343E+01_JPRB, 0.14026E+01_JPRB, 0.14224E+01_JPRB, 0.13743E+01_JPRB ,&
&  0.13233E+01_JPRB, 0.11879E+01_JPRB, 0.10681E+01_JPRB, 0.92217E+00_JPRB, 0.80964E+00_JPRB, 0.74723E+00_JPRB, 0.71248E+00_JPRB ,&
&  0.68003E+00_JPRB, 0.64988E+00_JPRB, 0.64648E+00_JPRB, 0.63007E+00_JPRB, 0.59471E+00_JPRB, 0.56282E+00_JPRB, 0.53824E+00_JPRB ,&
&  0.52159E+00_JPRB, 0.51286E+00_JPRB, 0.51146E+00_JPRB, 0.50261E+00_JPRB, 0.50484E+00_JPRB, 0.51433E+00_JPRB, 0.51971E+00_JPRB ,&
&  0.52002E+00_JPRB, 0.51938E+00_JPRB, 0.52554E+00_JPRB, 0.53338E+00_JPRB, 0.54365E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.62246E+00_JPRB, 0.60353E+00_JPRB, 0.57517E+00_JPRB, 0.57363E+00_JPRB, 0.55116E+00_JPRB, 0.53662E+00_JPRB, 0.48938E+00_JPRB ,&
&  0.42999E+00_JPRB, 0.41072E+00_JPRB, 0.42863E+00_JPRB, 0.43212E+00_JPRB, 0.41714E+00_JPRB, 0.41181E+00_JPRB, 0.41659E+00_JPRB ,&
&  0.42429E+00_JPRB, 0.44506E+00_JPRB, 0.44403E+00_JPRB, 0.43077E+00_JPRB, 0.42327E+00_JPRB, 0.42109E+00_JPRB, 0.42558E+00_JPRB ,&
&  0.42834E+00_JPRB, 0.42954E+00_JPRB, 0.42455E+00_JPRB, 0.41578E+00_JPRB, 0.41530E+00_JPRB, 0.41732E+00_JPRB, 0.41847E+00_JPRB ,&
&  0.42141E+00_JPRB, 0.42150E+00_JPRB, 0.43354E+00_JPRB, 0.44548E+00_JPRB, 0.43969E+00_JPRB, 0.42804E+00_JPRB, 0.37357E+00_JPRB ,&
&  0.33819E+00_JPRB, 0.32824E+00_JPRB, 0.35440E+00_JPRB, 0.42930E+00_JPRB, 0.46438E+00_JPRB, 0.50393E+00_JPRB, 0.53604E+00_JPRB ,&
&  0.50775E+00_JPRB, 0.51140E+00_JPRB, 0.54539E+00_JPRB, 0.66124E+00_JPRB, 0.70443E+00_JPRB, 0.76387E+00_JPRB, 0.80699E+00_JPRB ,&
&  0.84045E+00_JPRB, 0.86657E+00_JPRB, 0.87349E+00_JPRB, 0.72536E+00_JPRB, 0.72616E+00_JPRB, 0.88733E+00_JPRB, 0.81843E+00_JPRB ,&
&  0.86746E+00_JPRB, 0.92183E+00_JPRB, 0.97180E+00_JPRB, 0.94285E+00_JPRB, 0.87508E+00_JPRB, 0.86981E+00_JPRB, 0.91858E+00_JPRB ,&
&  0.10142E+01_JPRB, 0.11749E+01_JPRB, 0.14905E+01_JPRB, 0.16669E+01_JPRB, 0.16405E+01_JPRB, 0.16255E+01_JPRB, 0.15937E+01_JPRB ,&
&  0.15641E+01_JPRB, 0.14172E+01_JPRB, 0.12260E+01_JPRB, 0.10471E+01_JPRB, 0.89422E+00_JPRB, 0.77607E+00_JPRB, 0.70747E+00_JPRB ,&
&  0.68027E+00_JPRB, 0.66070E+00_JPRB, 0.65731E+00_JPRB, 0.63822E+00_JPRB, 0.59581E+00_JPRB, 0.54999E+00_JPRB, 0.51349E+00_JPRB ,&
&  0.49028E+00_JPRB, 0.46493E+00_JPRB, 0.44050E+00_JPRB, 0.43378E+00_JPRB, 0.46045E+00_JPRB, 0.49919E+00_JPRB, 0.51342E+00_JPRB ,&
&  0.50226E+00_JPRB, 0.50466E+00_JPRB, 0.53460E+00_JPRB, 0.56790E+00_JPRB, 0.58470E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.62792E+00_JPRB, 0.60084E+00_JPRB, 0.57740E+00_JPRB, 0.55343E+00_JPRB, 0.54112E+00_JPRB, 0.52383E+00_JPRB, 0.47406E+00_JPRB ,&
&  0.42252E+00_JPRB, 0.39074E+00_JPRB, 0.39889E+00_JPRB, 0.38876E+00_JPRB, 0.37622E+00_JPRB, 0.36866E+00_JPRB, 0.36854E+00_JPRB ,&
&  0.37564E+00_JPRB, 0.39748E+00_JPRB, 0.41389E+00_JPRB, 0.41728E+00_JPRB, 0.41414E+00_JPRB, 0.41140E+00_JPRB, 0.41800E+00_JPRB ,&
&  0.42760E+00_JPRB, 0.43870E+00_JPRB, 0.44453E+00_JPRB, 0.44369E+00_JPRB, 0.43961E+00_JPRB, 0.43982E+00_JPRB, 0.44457E+00_JPRB ,&
&  0.45466E+00_JPRB, 0.46136E+00_JPRB, 0.45222E+00_JPRB, 0.45743E+00_JPRB, 0.43830E+00_JPRB, 0.43966E+00_JPRB, 0.42637E+00_JPRB ,&
&  0.36563E+00_JPRB, 0.37747E+00_JPRB, 0.49884E+00_JPRB, 0.53210E+00_JPRB, 0.54783E+00_JPRB, 0.58259E+00_JPRB, 0.62881E+00_JPRB ,&
&  0.67180E+00_JPRB, 0.71260E+00_JPRB, 0.74916E+00_JPRB, 0.78923E+00_JPRB, 0.82663E+00_JPRB, 0.87624E+00_JPRB, 0.91943E+00_JPRB ,&
&  0.99275E+00_JPRB, 0.81847E+00_JPRB, 0.76623E+00_JPRB, 0.81028E+00_JPRB, 0.10733E+01_JPRB, 0.11042E+01_JPRB, 0.10771E+01_JPRB ,&
&  0.10870E+01_JPRB, 0.10561E+01_JPRB, 0.10290E+01_JPRB, 0.10228E+01_JPRB, 0.10556E+01_JPRB, 0.11231E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.13491E+01_JPRB, 0.15623E+01_JPRB, 0.18916E+01_JPRB, 0.19734E+01_JPRB, 0.19484E+01_JPRB, 0.19363E+01_JPRB, 0.18758E+01_JPRB ,&
&  0.17956E+01_JPRB, 0.16370E+01_JPRB, 0.13754E+01_JPRB, 0.11564E+01_JPRB, 0.97538E+00_JPRB, 0.83860E+00_JPRB, 0.73147E+00_JPRB ,&
&  0.67776E+00_JPRB, 0.65918E+00_JPRB, 0.64690E+00_JPRB, 0.62853E+00_JPRB, 0.60088E+00_JPRB, 0.56985E+00_JPRB, 0.53561E+00_JPRB ,&
&  0.49350E+00_JPRB, 0.46158E+00_JPRB, 0.43424E+00_JPRB, 0.44553E+00_JPRB, 0.48485E+00_JPRB, 0.52671E+00_JPRB, 0.54425E+00_JPRB ,&
&  0.55447E+00_JPRB, 0.55995E+00_JPRB, 0.56169E+00_JPRB, 0.57603E+00_JPRB, 0.60125E+00_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.62648E+00_JPRB, 0.59024E+00_JPRB, 0.55717E+00_JPRB, 0.52998E+00_JPRB, 0.51753E+00_JPRB, 0.50438E+00_JPRB, 0.49155E+00_JPRB ,&
&  0.53677E+00_JPRB, 0.45543E+00_JPRB, 0.53629E+00_JPRB, 0.43429E+00_JPRB, 0.40217E+00_JPRB, 0.39619E+00_JPRB, 0.38041E+00_JPRB ,&
&  0.37085E+00_JPRB, 0.37264E+00_JPRB, 0.37805E+00_JPRB, 0.38437E+00_JPRB, 0.38349E+00_JPRB, 0.38181E+00_JPRB, 0.38862E+00_JPRB ,&
&  0.40313E+00_JPRB, 0.42431E+00_JPRB, 0.44960E+00_JPRB, 0.46200E+00_JPRB, 0.46725E+00_JPRB, 0.46742E+00_JPRB, 0.47726E+00_JPRB ,&
&  0.45176E+00_JPRB, 0.44675E+00_JPRB, 0.45585E+00_JPRB, 0.45745E+00_JPRB, 0.45350E+00_JPRB, 0.44349E+00_JPRB, 0.46138E+00_JPRB ,&
&  0.46060E+00_JPRB, 0.43707E+00_JPRB, 0.54427E+00_JPRB, 0.55818E+00_JPRB, 0.58655E+00_JPRB, 0.61799E+00_JPRB, 0.65264E+00_JPRB ,&
&  0.68919E+00_JPRB, 0.73387E+00_JPRB, 0.78492E+00_JPRB, 0.82249E+00_JPRB, 0.88139E+00_JPRB, 0.94750E+00_JPRB, 0.10023E+01_JPRB ,&
&  0.10800E+01_JPRB, 0.83347E+00_JPRB, 0.91795E+00_JPRB, 0.96444E+00_JPRB, 0.11773E+01_JPRB, 0.14220E+01_JPRB, 0.14175E+01_JPRB ,&
&  0.15070E+01_JPRB, 0.14480E+01_JPRB, 0.13852E+01_JPRB, 0.13377E+01_JPRB, 0.14021E+01_JPRB, 0.15043E+01_JPRB, 0.16278E+01_JPRB ,&
&  0.17964E+01_JPRB, 0.20264E+01_JPRB, 0.23740E+01_JPRB, 0.23891E+01_JPRB, 0.23474E+01_JPRB, 0.22803E+01_JPRB, 0.21570E+01_JPRB ,&
&  0.19860E+01_JPRB, 0.17779E+01_JPRB, 0.15159E+01_JPRB, 0.12358E+01_JPRB, 0.10460E+01_JPRB, 0.88636E+00_JPRB, 0.75708E+00_JPRB ,&
&  0.66411E+00_JPRB, 0.60991E+00_JPRB, 0.60036E+00_JPRB, 0.58244E+00_JPRB, 0.56975E+00_JPRB, 0.54955E+00_JPRB, 0.51331E+00_JPRB ,&
&  0.48303E+00_JPRB, 0.48898E+00_JPRB, 0.53248E+00_JPRB, 0.58724E+00_JPRB, 0.60867E+00_JPRB, 0.62114E+00_JPRB, 0.67519E+00_JPRB ,&
&  0.61492E+00_JPRB, 0.65388E+00_JPRB, 0.70052E+00_JPRB, 0.67763E+00_JPRB, 0.65865E+00_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.63475E+00_JPRB, 0.58645E+00_JPRB, 0.53244E+00_JPRB, 0.50976E+00_JPRB, 0.51741E+00_JPRB, 0.52795E+00_JPRB, 0.51382E+00_JPRB ,&
&  0.54987E+00_JPRB, 0.53595E+00_JPRB, 0.56284E+00_JPRB, 0.60105E+00_JPRB, 0.63942E+00_JPRB, 0.55693E+00_JPRB, 0.42920E+00_JPRB ,&
&  0.38905E+00_JPRB, 0.38991E+00_JPRB, 0.39560E+00_JPRB, 0.39281E+00_JPRB, 0.39195E+00_JPRB, 0.38800E+00_JPRB, 0.38799E+00_JPRB ,&
&  0.39327E+00_JPRB, 0.40514E+00_JPRB, 0.43737E+00_JPRB, 0.45070E+00_JPRB, 0.45771E+00_JPRB, 0.46024E+00_JPRB, 0.47371E+00_JPRB ,&
&  0.45005E+00_JPRB, 0.44338E+00_JPRB, 0.44373E+00_JPRB, 0.42002E+00_JPRB, 0.45546E+00_JPRB, 0.46884E+00_JPRB, 0.47985E+00_JPRB ,&
&  0.49383E+00_JPRB, 0.52126E+00_JPRB, 0.55392E+00_JPRB, 0.58281E+00_JPRB, 0.60911E+00_JPRB, 0.63645E+00_JPRB, 0.66636E+00_JPRB ,&
&  0.69805E+00_JPRB, 0.73860E+00_JPRB, 0.78678E+00_JPRB, 0.84391E+00_JPRB, 0.96994E+00_JPRB, 0.94619E+00_JPRB, 0.10384E+01_JPRB ,&
&  0.10858E+01_JPRB, 0.12065E+01_JPRB, 0.13300E+01_JPRB, 0.12466E+01_JPRB, 0.13499E+01_JPRB, 0.15184E+01_JPRB, 0.14887E+01_JPRB ,&
&  0.14880E+01_JPRB, 0.14368E+01_JPRB, 0.14884E+01_JPRB, 0.15856E+01_JPRB, 0.17356E+01_JPRB, 0.18637E+01_JPRB, 0.19928E+01_JPRB ,&
&  0.22169E+01_JPRB, 0.26366E+01_JPRB, 0.28068E+01_JPRB, 0.27289E+01_JPRB, 0.26241E+01_JPRB, 0.24977E+01_JPRB, 0.23056E+01_JPRB ,&
&  0.20884E+01_JPRB, 0.18403E+01_JPRB, 0.15647E+01_JPRB, 0.12879E+01_JPRB, 0.10525E+01_JPRB, 0.92339E+00_JPRB, 0.78489E+00_JPRB ,&
&  0.64700E+00_JPRB, 0.55677E+00_JPRB, 0.55367E+00_JPRB, 0.55245E+00_JPRB, 0.54301E+00_JPRB, 0.51436E+00_JPRB, 0.48893E+00_JPRB ,&
&  0.52009E+00_JPRB, 0.59521E+00_JPRB, 0.79234E+00_JPRB, 0.79350E+00_JPRB, 0.79663E+00_JPRB, 0.77872E+00_JPRB, 0.64342E+00_JPRB ,&
&  0.59637E+00_JPRB, 0.73126E+00_JPRB, 0.69409E+00_JPRB, 0.68931E+00_JPRB, 0.67238E+00_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.65604E+00_JPRB, 0.59037E+00_JPRB, 0.51942E+00_JPRB, 0.50218E+00_JPRB, 0.51980E+00_JPRB, 0.52905E+00_JPRB, 0.53542E+00_JPRB ,&
&  0.55180E+00_JPRB, 0.55959E+00_JPRB, 0.57172E+00_JPRB, 0.58698E+00_JPRB, 0.59270E+00_JPRB, 0.62284E+00_JPRB, 0.55772E+00_JPRB ,&
&  0.46509E+00_JPRB, 0.43792E+00_JPRB, 0.41636E+00_JPRB, 0.41252E+00_JPRB, 0.42970E+00_JPRB, 0.43012E+00_JPRB, 0.42858E+00_JPRB ,&
&  0.43471E+00_JPRB, 0.43426E+00_JPRB, 0.44507E+00_JPRB, 0.44588E+00_JPRB, 0.44555E+00_JPRB, 0.45203E+00_JPRB, 0.46267E+00_JPRB ,&
&  0.43494E+00_JPRB, 0.41342E+00_JPRB, 0.40140E+00_JPRB, 0.39522E+00_JPRB, 0.42467E+00_JPRB, 0.46718E+00_JPRB, 0.49375E+00_JPRB ,&
&  0.51222E+00_JPRB, 0.53262E+00_JPRB, 0.55583E+00_JPRB, 0.58435E+00_JPRB, 0.61533E+00_JPRB, 0.64680E+00_JPRB, 0.67223E+00_JPRB ,&
&  0.70008E+00_JPRB, 0.73304E+00_JPRB, 0.77961E+00_JPRB, 0.85873E+00_JPRB, 0.90340E+00_JPRB, 0.96364E+00_JPRB, 0.10965E+01_JPRB ,&
&  0.11901E+01_JPRB, 0.13132E+01_JPRB, 0.14253E+01_JPRB, 0.15918E+01_JPRB, 0.17163E+01_JPRB, 0.16707E+01_JPRB, 0.16123E+01_JPRB ,&
&  0.16166E+01_JPRB, 0.16082E+01_JPRB, 0.18258E+01_JPRB, 0.19787E+01_JPRB, 0.20792E+01_JPRB, 0.22277E+01_JPRB, 0.23600E+01_JPRB ,&
&  0.25982E+01_JPRB, 0.28517E+01_JPRB, 0.29106E+01_JPRB, 0.28207E+01_JPRB, 0.26958E+01_JPRB, 0.25733E+01_JPRB, 0.23583E+01_JPRB ,&
&  0.21138E+01_JPRB, 0.18251E+01_JPRB, 0.14955E+01_JPRB, 0.12210E+01_JPRB, 0.98071E+00_JPRB, 0.90572E+00_JPRB, 0.80893E+00_JPRB ,&
&  0.64248E+00_JPRB, 0.56377E+00_JPRB, 0.55724E+00_JPRB, 0.58366E+00_JPRB, 0.57764E+00_JPRB, 0.54118E+00_JPRB, 0.52514E+00_JPRB ,&
&  0.57563E+00_JPRB, 0.73528E+00_JPRB, 0.79310E+00_JPRB, 0.83433E+00_JPRB, 0.84690E+00_JPRB, 0.83212E+00_JPRB, 0.60121E+00_JPRB ,&
&  0.62425E+00_JPRB, 0.73829E+00_JPRB, 0.72160E+00_JPRB, 0.70876E+00_JPRB, 0.69249E+00_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.68250E+00_JPRB, 0.60918E+00_JPRB, 0.54399E+00_JPRB, 0.51710E+00_JPRB, 0.53245E+00_JPRB, 0.54861E+00_JPRB, 0.55890E+00_JPRB ,&
&  0.57460E+00_JPRB, 0.58622E+00_JPRB, 0.59538E+00_JPRB, 0.60374E+00_JPRB, 0.60814E+00_JPRB, 0.60093E+00_JPRB, 0.64745E+00_JPRB ,&
&  0.50818E+00_JPRB, 0.45946E+00_JPRB, 0.42667E+00_JPRB, 0.42740E+00_JPRB, 0.49329E+00_JPRB, 0.50606E+00_JPRB, 0.52287E+00_JPRB ,&
&  0.51806E+00_JPRB, 0.52066E+00_JPRB, 0.51284E+00_JPRB, 0.49897E+00_JPRB, 0.48517E+00_JPRB, 0.48115E+00_JPRB, 0.47022E+00_JPRB ,&
&  0.43482E+00_JPRB, 0.40923E+00_JPRB, 0.40468E+00_JPRB, 0.40942E+00_JPRB, 0.44173E+00_JPRB, 0.47004E+00_JPRB, 0.51982E+00_JPRB ,&
&  0.54890E+00_JPRB, 0.57300E+00_JPRB, 0.59771E+00_JPRB, 0.62459E+00_JPRB, 0.64820E+00_JPRB, 0.67917E+00_JPRB, 0.71517E+00_JPRB ,&
&  0.75248E+00_JPRB, 0.79101E+00_JPRB, 0.86150E+00_JPRB, 0.94527E+00_JPRB, 0.94734E+00_JPRB, 0.10132E+01_JPRB, 0.11757E+01_JPRB ,&
&  0.13317E+01_JPRB, 0.14059E+01_JPRB, 0.16259E+01_JPRB, 0.21995E+01_JPRB, 0.23461E+01_JPRB, 0.22733E+01_JPRB, 0.21593E+01_JPRB ,&
&  0.20990E+01_JPRB, 0.20507E+01_JPRB, 0.21208E+01_JPRB, 0.22378E+01_JPRB, 0.23455E+01_JPRB, 0.25451E+01_JPRB, 0.27294E+01_JPRB ,&
&  0.28354E+01_JPRB, 0.28717E+01_JPRB, 0.28313E+01_JPRB, 0.27278E+01_JPRB, 0.25504E+01_JPRB, 0.24121E+01_JPRB, 0.22571E+01_JPRB ,&
&  0.20499E+01_JPRB, 0.17910E+01_JPRB, 0.14077E+01_JPRB, 0.11604E+01_JPRB, 0.88321E+00_JPRB, 0.80215E+00_JPRB, 0.90994E+00_JPRB ,&
&  0.69000E+00_JPRB, 0.60209E+00_JPRB, 0.61021E+00_JPRB, 0.65618E+00_JPRB, 0.66544E+00_JPRB, 0.65615E+00_JPRB, 0.59106E+00_JPRB ,&
&  0.53603E+00_JPRB, 0.64309E+00_JPRB, 0.87824E+00_JPRB, 0.92933E+00_JPRB, 0.92484E+00_JPRB, 0.88526E+00_JPRB, 0.70988E+00_JPRB ,&
&  0.79313E+00_JPRB, 0.75015E+00_JPRB, 0.74390E+00_JPRB, 0.73395E+00_JPRB, 0.72280E+00_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.70075E+00_JPRB, 0.63027E+00_JPRB, 0.57682E+00_JPRB, 0.54964E+00_JPRB, 0.55912E+00_JPRB, 0.57907E+00_JPRB, 0.59130E+00_JPRB ,&
&  0.60059E+00_JPRB, 0.60644E+00_JPRB, 0.61135E+00_JPRB, 0.61650E+00_JPRB, 0.62015E+00_JPRB, 0.60234E+00_JPRB, 0.58733E+00_JPRB ,&
&  0.58079E+00_JPRB, 0.48597E+00_JPRB, 0.45606E+00_JPRB, 0.44573E+00_JPRB, 0.50302E+00_JPRB, 0.54926E+00_JPRB, 0.60001E+00_JPRB ,&
&  0.63974E+00_JPRB, 0.67293E+00_JPRB, 0.66155E+00_JPRB, 0.62783E+00_JPRB, 0.59312E+00_JPRB, 0.56518E+00_JPRB, 0.52037E+00_JPRB ,&
&  0.47069E+00_JPRB, 0.44833E+00_JPRB, 0.45209E+00_JPRB, 0.48520E+00_JPRB, 0.54461E+00_JPRB, 0.51189E+00_JPRB, 0.57434E+00_JPRB ,&
&  0.61044E+00_JPRB, 0.64487E+00_JPRB, 0.65959E+00_JPRB, 0.68192E+00_JPRB, 0.74133E+00_JPRB, 0.81637E+00_JPRB, 0.88366E+00_JPRB ,&
&  0.94464E+00_JPRB, 0.96271E+00_JPRB, 0.95270E+00_JPRB, 0.10236E+01_JPRB, 0.10959E+01_JPRB, 0.11411E+01_JPRB, 0.12324E+01_JPRB ,&
&  0.14179E+01_JPRB, 0.15417E+01_JPRB, 0.18296E+01_JPRB, 0.25217E+01_JPRB, 0.27215E+01_JPRB, 0.26543E+01_JPRB, 0.26697E+01_JPRB ,&
&  0.26597E+01_JPRB, 0.25938E+01_JPRB, 0.25522E+01_JPRB, 0.25058E+01_JPRB, 0.26489E+01_JPRB, 0.28935E+01_JPRB, 0.30820E+01_JPRB ,&
&  0.29740E+01_JPRB, 0.27601E+01_JPRB, 0.26656E+01_JPRB, 0.24783E+01_JPRB, 0.23001E+01_JPRB, 0.20989E+01_JPRB, 0.21091E+01_JPRB ,&
&  0.21108E+01_JPRB, 0.15298E+01_JPRB, 0.12499E+01_JPRB, 0.11683E+01_JPRB, 0.87483E+00_JPRB, 0.74709E+00_JPRB, 0.77092E+00_JPRB ,&
&  0.63188E+00_JPRB, 0.57891E+00_JPRB, 0.63714E+00_JPRB, 0.69924E+00_JPRB, 0.77443E+00_JPRB, 0.81936E+00_JPRB, 0.78337E+00_JPRB ,&
&  0.72621E+00_JPRB, 0.77252E+00_JPRB, 0.10546E+01_JPRB, 0.10930E+01_JPRB, 0.10480E+01_JPRB, 0.95654E+00_JPRB, 0.87467E+00_JPRB ,&
&  0.80564E+00_JPRB, 0.80457E+00_JPRB, 0.79508E+00_JPRB, 0.78268E+00_JPRB, 0.76534E+00_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.76459E+00_JPRB, 0.69117E+00_JPRB, 0.63098E+00_JPRB, 0.60879E+00_JPRB, 0.62244E+00_JPRB, 0.63410E+00_JPRB, 0.64139E+00_JPRB ,&
&  0.62684E+00_JPRB, 0.61819E+00_JPRB, 0.62718E+00_JPRB, 0.62734E+00_JPRB, 0.62625E+00_JPRB, 0.60863E+00_JPRB, 0.57203E+00_JPRB ,&
&  0.55321E+00_JPRB, 0.53035E+00_JPRB, 0.48406E+00_JPRB, 0.48657E+00_JPRB, 0.51164E+00_JPRB, 0.59990E+00_JPRB, 0.67546E+00_JPRB ,&
&  0.75193E+00_JPRB, 0.81254E+00_JPRB, 0.83388E+00_JPRB, 0.83613E+00_JPRB, 0.80236E+00_JPRB, 0.73844E+00_JPRB, 0.67623E+00_JPRB ,&
&  0.62123E+00_JPRB, 0.59814E+00_JPRB, 0.58231E+00_JPRB, 0.65771E+00_JPRB, 0.68922E+00_JPRB, 0.65398E+00_JPRB, 0.74342E+00_JPRB ,&
&  0.77923E+00_JPRB, 0.81186E+00_JPRB, 0.82814E+00_JPRB, 0.89222E+00_JPRB, 0.96253E+00_JPRB, 0.98787E+00_JPRB, 0.97935E+00_JPRB ,&
&  0.96516E+00_JPRB, 0.97859E+00_JPRB, 0.10210E+01_JPRB, 0.10106E+01_JPRB, 0.10653E+01_JPRB, 0.11968E+01_JPRB, 0.14379E+01_JPRB ,&
&  0.17346E+01_JPRB, 0.17312E+01_JPRB, 0.17659E+01_JPRB, 0.24920E+01_JPRB, 0.31467E+01_JPRB, 0.31322E+01_JPRB, 0.30461E+01_JPRB ,&
&  0.30620E+01_JPRB, 0.28891E+01_JPRB, 0.27208E+01_JPRB, 0.26761E+01_JPRB, 0.30496E+01_JPRB, 0.33194E+01_JPRB, 0.33904E+01_JPRB ,&
&  0.31601E+01_JPRB, 0.28370E+01_JPRB, 0.26715E+01_JPRB, 0.24717E+01_JPRB, 0.22043E+01_JPRB, 0.21030E+01_JPRB, 0.21437E+01_JPRB ,&
&  0.20963E+01_JPRB, 0.18540E+01_JPRB, 0.14200E+01_JPRB, 0.12705E+01_JPRB, 0.10085E+01_JPRB, 0.87238E+00_JPRB, 0.84863E+00_JPRB ,&
&  0.72121E+00_JPRB, 0.68353E+00_JPRB, 0.69800E+00_JPRB, 0.73916E+00_JPRB, 0.10381E+01_JPRB, 0.11233E+01_JPRB, 0.12065E+01_JPRB ,&
&  0.11922E+01_JPRB, 0.11398E+01_JPRB, 0.14186E+01_JPRB, 0.13963E+01_JPRB, 0.12604E+01_JPRB, 0.10822E+01_JPRB, 0.93426E+00_JPRB ,&
&  0.87536E+00_JPRB, 0.88584E+00_JPRB, 0.86940E+00_JPRB, 0.85156E+00_JPRB, 0.82897E+00_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.78089E+00_JPRB, 0.70545E+00_JPRB, 0.64605E+00_JPRB, 0.62575E+00_JPRB, 0.63243E+00_JPRB, 0.64175E+00_JPRB, 0.64317E+00_JPRB ,&
&  0.64030E+00_JPRB, 0.64268E+00_JPRB, 0.64338E+00_JPRB, 0.62764E+00_JPRB, 0.61055E+00_JPRB, 0.59058E+00_JPRB, 0.55907E+00_JPRB ,&
&  0.53396E+00_JPRB, 0.52431E+00_JPRB, 0.49066E+00_JPRB, 0.50022E+00_JPRB, 0.54972E+00_JPRB, 0.67456E+00_JPRB, 0.81516E+00_JPRB ,&
&  0.93753E+00_JPRB, 0.10117E+01_JPRB, 0.10569E+01_JPRB, 0.10770E+01_JPRB, 0.10914E+01_JPRB, 0.10551E+01_JPRB, 0.98258E+00_JPRB ,&
&  0.98147E+00_JPRB, 0.84510E+00_JPRB, 0.92359E+00_JPRB, 0.92394E+00_JPRB, 0.98707E+00_JPRB, 0.98881E+00_JPRB, 0.95995E+00_JPRB ,&
&  0.93118E+00_JPRB, 0.93809E+00_JPRB, 0.94032E+00_JPRB, 0.96960E+00_JPRB, 0.10071E+01_JPRB, 0.10269E+01_JPRB, 0.10511E+01_JPRB ,&
&  0.10769E+01_JPRB, 0.10382E+01_JPRB, 0.96807E+00_JPRB, 0.96656E+00_JPRB, 0.13024E+01_JPRB, 0.17677E+01_JPRB, 0.22004E+01_JPRB ,&
&  0.21598E+01_JPRB, 0.19740E+01_JPRB, 0.28183E+01_JPRB, 0.32214E+01_JPRB, 0.31800E+01_JPRB, 0.34609E+01_JPRB, 0.34012E+01_JPRB ,&
&  0.31073E+01_JPRB, 0.27374E+01_JPRB, 0.24556E+01_JPRB, 0.25786E+01_JPRB, 0.36200E+01_JPRB, 0.39157E+01_JPRB, 0.39098E+01_JPRB ,&
&  0.35099E+01_JPRB, 0.30865E+01_JPRB, 0.27039E+01_JPRB, 0.24409E+01_JPRB, 0.22944E+01_JPRB, 0.23821E+01_JPRB, 0.22357E+01_JPRB ,&
&  0.20684E+01_JPRB, 0.21636E+01_JPRB, 0.18111E+01_JPRB, 0.14300E+01_JPRB, 0.11928E+01_JPRB, 0.10781E+01_JPRB, 0.10044E+01_JPRB ,&
&  0.89295E+00_JPRB, 0.83607E+00_JPRB, 0.83887E+00_JPRB, 0.10112E+01_JPRB, 0.14527E+01_JPRB, 0.14919E+01_JPRB, 0.15206E+01_JPRB ,&
&  0.15737E+01_JPRB, 0.19513E+01_JPRB, 0.17838E+01_JPRB, 0.15970E+01_JPRB, 0.14112E+01_JPRB, 0.12203E+01_JPRB, 0.10748E+01_JPRB ,&
&  0.98084E+00_JPRB, 0.97578E+00_JPRB, 0.96084E+00_JPRB, 0.91649E+00_JPRB, 0.86251E+00_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.83465E+00_JPRB, 0.74819E+00_JPRB, 0.70498E+00_JPRB, 0.67838E+00_JPRB, 0.67208E+00_JPRB, 0.66386E+00_JPRB, 0.63648E+00_JPRB ,&
&  0.62010E+00_JPRB, 0.62978E+00_JPRB, 0.61801E+00_JPRB, 0.61218E+00_JPRB, 0.60634E+00_JPRB, 0.57839E+00_JPRB, 0.54854E+00_JPRB ,&
&  0.53685E+00_JPRB, 0.52263E+00_JPRB, 0.52055E+00_JPRB, 0.57806E+00_JPRB, 0.65136E+00_JPRB, 0.75454E+00_JPRB, 0.91973E+00_JPRB ,&
&  0.10701E+01_JPRB, 0.11886E+01_JPRB, 0.12510E+01_JPRB, 0.13588E+01_JPRB, 0.14404E+01_JPRB, 0.14770E+01_JPRB, 0.14071E+01_JPRB ,&
&  0.12578E+01_JPRB, 0.12708E+01_JPRB, 0.13590E+01_JPRB, 0.13110E+01_JPRB, 0.12754E+01_JPRB, 0.12172E+01_JPRB, 0.11567E+01_JPRB ,&
&  0.10934E+01_JPRB, 0.10455E+01_JPRB, 0.10413E+01_JPRB, 0.10619E+01_JPRB, 0.10954E+01_JPRB, 0.11595E+01_JPRB, 0.11527E+01_JPRB ,&
&  0.10749E+01_JPRB, 0.10273E+01_JPRB, 0.97855E+00_JPRB, 0.12152E+01_JPRB, 0.17941E+01_JPRB, 0.27247E+01_JPRB, 0.30269E+01_JPRB ,&
&  0.31898E+01_JPRB, 0.28702E+01_JPRB, 0.29390E+01_JPRB, 0.33676E+01_JPRB, 0.27623E+01_JPRB, 0.29862E+01_JPRB, 0.30835E+01_JPRB ,&
&  0.27827E+01_JPRB, 0.26395E+01_JPRB, 0.29663E+01_JPRB, 0.35638E+01_JPRB, 0.43721E+01_JPRB, 0.53356E+01_JPRB, 0.52323E+01_JPRB ,&
&  0.43206E+01_JPRB, 0.35389E+01_JPRB, 0.28133E+01_JPRB, 0.25821E+01_JPRB, 0.22858E+01_JPRB, 0.22220E+01_JPRB, 0.17891E+01_JPRB ,&
&  0.17999E+01_JPRB, 0.15474E+01_JPRB, 0.16462E+01_JPRB, 0.14467E+01_JPRB, 0.13218E+01_JPRB, 0.11865E+01_JPRB, 0.10817E+01_JPRB ,&
&  0.10287E+01_JPRB, 0.10075E+01_JPRB, 0.11066E+01_JPRB, 0.15023E+01_JPRB, 0.20039E+01_JPRB, 0.17302E+01_JPRB, 0.21039E+01_JPRB ,&
&  0.26866E+01_JPRB, 0.25773E+01_JPRB, 0.19293E+01_JPRB, 0.17472E+01_JPRB, 0.14910E+01_JPRB, 0.13071E+01_JPRB, 0.12178E+01_JPRB ,&
&  0.11153E+01_JPRB, 0.10617E+01_JPRB, 0.10246E+01_JPRB, 0.97732E+00_JPRB, 0.91971E+00_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.88801E+00_JPRB, 0.79637E+00_JPRB, 0.75607E+00_JPRB, 0.74694E+00_JPRB, 0.72947E+00_JPRB, 0.66959E+00_JPRB, 0.64496E+00_JPRB ,&
&  0.66167E+00_JPRB, 0.63809E+00_JPRB, 0.59871E+00_JPRB, 0.57817E+00_JPRB, 0.57950E+00_JPRB, 0.58456E+00_JPRB, 0.55975E+00_JPRB ,&
&  0.56689E+00_JPRB, 0.60539E+00_JPRB, 0.63504E+00_JPRB, 0.68179E+00_JPRB, 0.76788E+00_JPRB, 0.85347E+00_JPRB, 0.92643E+00_JPRB ,&
&  0.12559E+01_JPRB, 0.13915E+01_JPRB, 0.14093E+01_JPRB, 0.15463E+01_JPRB, 0.17028E+01_JPRB, 0.18039E+01_JPRB, 0.17590E+01_JPRB ,&
&  0.17757E+01_JPRB, 0.17968E+01_JPRB, 0.15407E+01_JPRB, 0.13622E+01_JPRB, 0.12574E+01_JPRB, 0.12005E+01_JPRB, 0.11369E+01_JPRB ,&
&  0.10543E+01_JPRB, 0.99805E+00_JPRB, 0.99103E+00_JPRB, 0.99015E+00_JPRB, 0.10318E+01_JPRB, 0.97448E+00_JPRB, 0.95639E+00_JPRB ,&
&  0.96451E+00_JPRB, 0.10123E+01_JPRB, 0.11075E+01_JPRB, 0.15405E+01_JPRB, 0.24651E+01_JPRB, 0.36316E+01_JPRB, 0.39420E+01_JPRB ,&
&  0.34755E+01_JPRB, 0.29836E+01_JPRB, 0.31421E+01_JPRB, 0.34844E+01_JPRB, 0.31021E+01_JPRB, 0.28619E+01_JPRB, 0.26795E+01_JPRB ,&
&  0.22561E+01_JPRB, 0.23336E+01_JPRB, 0.29328E+01_JPRB, 0.39682E+01_JPRB, 0.54679E+01_JPRB, 0.69539E+01_JPRB, 0.67569E+01_JPRB ,&
&  0.53469E+01_JPRB, 0.41968E+01_JPRB, 0.33523E+01_JPRB, 0.26509E+01_JPRB, 0.25400E+01_JPRB, 0.19287E+01_JPRB, 0.21365E+01_JPRB ,&
&  0.21375E+01_JPRB, 0.20326E+01_JPRB, 0.16812E+01_JPRB, 0.13750E+01_JPRB, 0.11156E+01_JPRB, 0.11983E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.11861E+01_JPRB, 0.12516E+01_JPRB, 0.20057E+01_JPRB, 0.23342E+01_JPRB, 0.24732E+01_JPRB, 0.24286E+01_JPRB, 0.32349E+01_JPRB ,&
&  0.30474E+01_JPRB, 0.27097E+01_JPRB, 0.22291E+01_JPRB, 0.18979E+01_JPRB, 0.16240E+01_JPRB, 0.14510E+01_JPRB, 0.13516E+01_JPRB ,&
&  0.12337E+01_JPRB, 0.11711E+01_JPRB, 0.11205E+01_JPRB, 0.10601E+01_JPRB, 0.98694E+00_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.90543E+00_JPRB, 0.85804E+00_JPRB, 0.83802E+00_JPRB, 0.78563E+00_JPRB, 0.70625E+00_JPRB, 0.66189E+00_JPRB, 0.66292E+00_JPRB ,&
&  0.65088E+00_JPRB, 0.62402E+00_JPRB, 0.59609E+00_JPRB, 0.57564E+00_JPRB, 0.57925E+00_JPRB, 0.59042E+00_JPRB, 0.59837E+00_JPRB ,&
&  0.66036E+00_JPRB, 0.79426E+00_JPRB, 0.80672E+00_JPRB, 0.85131E+00_JPRB, 0.89737E+00_JPRB, 0.95554E+00_JPRB, 0.10439E+01_JPRB ,&
&  0.14375E+01_JPRB, 0.14563E+01_JPRB, 0.13618E+01_JPRB, 0.15069E+01_JPRB, 0.18336E+01_JPRB, 0.18068E+01_JPRB, 0.17799E+01_JPRB ,&
&  0.18095E+01_JPRB, 0.17224E+01_JPRB, 0.14855E+01_JPRB, 0.12902E+01_JPRB, 0.11597E+01_JPRB, 0.10863E+01_JPRB, 0.10078E+01_JPRB ,&
&  0.96056E+00_JPRB, 0.96399E+00_JPRB, 0.98430E+00_JPRB, 0.10190E+01_JPRB, 0.99028E+00_JPRB, 0.95804E+00_JPRB, 0.96492E+00_JPRB ,&
&  0.98339E+00_JPRB, 0.10518E+01_JPRB, 0.12326E+01_JPRB, 0.18472E+01_JPRB, 0.28732E+01_JPRB, 0.41850E+01_JPRB, 0.47773E+01_JPRB ,&
&  0.38077E+01_JPRB, 0.35444E+01_JPRB, 0.36432E+01_JPRB, 0.46219E+01_JPRB, 0.30759E+01_JPRB, 0.27526E+01_JPRB, 0.26648E+01_JPRB ,&
&  0.25068E+01_JPRB, 0.29655E+01_JPRB, 0.40680E+01_JPRB, 0.53537E+01_JPRB, 0.57321E+01_JPRB, 0.67258E+01_JPRB, 0.77507E+01_JPRB ,&
&  0.56681E+01_JPRB, 0.43489E+01_JPRB, 0.36511E+01_JPRB, 0.33494E+01_JPRB, 0.21770E+01_JPRB, 0.14670E+01_JPRB, 0.20713E+01_JPRB ,&
&  0.20292E+01_JPRB, 0.11613E+01_JPRB, 0.85643E+00_JPRB, 0.96497E+00_JPRB, 0.71303E+00_JPRB, 0.80875E+00_JPRB, 0.11239E+01_JPRB ,&
&  0.13473E+01_JPRB, 0.15996E+01_JPRB, 0.30370E+01_JPRB, 0.29323E+01_JPRB, 0.29966E+01_JPRB, 0.36531E+01_JPRB, 0.40617E+01_JPRB ,&
&  0.35616E+01_JPRB, 0.23171E+01_JPRB, 0.22387E+01_JPRB, 0.20316E+01_JPRB, 0.17201E+01_JPRB, 0.14810E+01_JPRB, 0.13087E+01_JPRB ,&
&  0.11633E+01_JPRB, 0.10635E+01_JPRB, 0.10036E+01_JPRB, 0.97101E+00_JPRB, 0.95806E+00_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.80422E+00_JPRB, 0.80020E+00_JPRB, 0.77850E+00_JPRB, 0.70865E+00_JPRB, 0.66264E+00_JPRB, 0.65539E+00_JPRB, 0.64354E+00_JPRB ,&
&  0.62496E+00_JPRB, 0.59490E+00_JPRB, 0.57858E+00_JPRB, 0.58507E+00_JPRB, 0.59673E+00_JPRB, 0.61352E+00_JPRB, 0.64705E+00_JPRB ,&
&  0.70610E+00_JPRB, 0.81394E+00_JPRB, 0.96724E+00_JPRB, 0.10705E+01_JPRB, 0.11229E+01_JPRB, 0.10056E+01_JPRB, 0.12100E+01_JPRB ,&
&  0.14814E+01_JPRB, 0.13856E+01_JPRB, 0.13303E+01_JPRB, 0.15363E+01_JPRB, 0.16655E+01_JPRB, 0.15554E+01_JPRB, 0.15606E+01_JPRB ,&
&  0.15768E+01_JPRB, 0.15046E+01_JPRB, 0.13636E+01_JPRB, 0.12711E+01_JPRB, 0.11496E+01_JPRB, 0.10261E+01_JPRB, 0.10011E+01_JPRB ,&
&  0.10104E+01_JPRB, 0.10641E+01_JPRB, 0.11239E+01_JPRB, 0.10828E+01_JPRB, 0.10656E+01_JPRB, 0.10414E+01_JPRB, 0.10518E+01_JPRB ,&
&  0.10851E+01_JPRB, 0.11929E+01_JPRB, 0.14947E+01_JPRB, 0.23780E+01_JPRB, 0.38939E+01_JPRB, 0.56134E+01_JPRB, 0.46174E+01_JPRB ,&
&  0.34287E+01_JPRB, 0.30973E+01_JPRB, 0.40205E+01_JPRB, 0.43434E+01_JPRB, 0.29780E+01_JPRB, 0.28558E+01_JPRB, 0.27806E+01_JPRB ,&
&  0.31865E+01_JPRB, 0.38137E+01_JPRB, 0.42065E+01_JPRB, 0.53057E+01_JPRB, 0.66622E+01_JPRB, 0.68861E+01_JPRB, 0.70144E+01_JPRB ,&
&  0.56769E+01_JPRB, 0.42866E+01_JPRB, 0.30561E+01_JPRB, 0.23845E+01_JPRB, 0.25770E+01_JPRB, 0.18450E+01_JPRB, 0.11633E+01_JPRB ,&
&  0.90428E+00_JPRB, 0.70841E+00_JPRB, 0.56879E+00_JPRB, 0.55426E+00_JPRB, 0.48677E+00_JPRB, 0.64454E+00_JPRB, 0.10247E+01_JPRB ,&
&  0.18388E+01_JPRB, 0.25708E+01_JPRB, 0.33278E+01_JPRB, 0.34407E+01_JPRB, 0.31245E+01_JPRB, 0.39146E+01_JPRB, 0.47691E+01_JPRB ,&
&  0.29457E+01_JPRB, 0.22915E+01_JPRB, 0.22489E+01_JPRB, 0.18847E+01_JPRB, 0.14724E+01_JPRB, 0.12138E+01_JPRB, 0.10348E+01_JPRB ,&
&  0.92316E+00_JPRB, 0.85840E+00_JPRB, 0.82434E+00_JPRB, 0.81726E+00_JPRB, 0.82167E+00_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.73899E+00_JPRB, 0.72718E+00_JPRB, 0.70943E+00_JPRB, 0.67001E+00_JPRB, 0.64380E+00_JPRB, 0.63854E+00_JPRB, 0.63114E+00_JPRB ,&
&  0.60981E+00_JPRB, 0.58696E+00_JPRB, 0.58838E+00_JPRB, 0.60803E+00_JPRB, 0.63626E+00_JPRB, 0.67880E+00_JPRB, 0.72102E+00_JPRB ,&
&  0.77101E+00_JPRB, 0.90367E+00_JPRB, 0.11243E+01_JPRB, 0.13157E+01_JPRB, 0.14480E+01_JPRB, 0.11924E+01_JPRB, 0.13683E+01_JPRB ,&
&  0.14593E+01_JPRB, 0.13914E+01_JPRB, 0.14171E+01_JPRB, 0.15721E+01_JPRB, 0.16146E+01_JPRB, 0.14142E+01_JPRB, 0.14614E+01_JPRB ,&
&  0.14445E+01_JPRB, 0.13974E+01_JPRB, 0.13052E+01_JPRB, 0.12422E+01_JPRB, 0.11436E+01_JPRB, 0.11552E+01_JPRB, 0.12331E+01_JPRB ,&
&  0.12774E+01_JPRB, 0.13389E+01_JPRB, 0.12968E+01_JPRB, 0.12509E+01_JPRB, 0.12255E+01_JPRB, 0.12097E+01_JPRB, 0.12346E+01_JPRB ,&
&  0.13158E+01_JPRB, 0.15404E+01_JPRB, 0.21455E+01_JPRB, 0.35290E+01_JPRB, 0.47018E+01_JPRB, 0.47364E+01_JPRB, 0.42218E+01_JPRB ,&
&  0.37804E+01_JPRB, 0.38457E+01_JPRB, 0.39592E+01_JPRB, 0.44956E+01_JPRB, 0.31473E+01_JPRB, 0.28998E+01_JPRB, 0.28965E+01_JPRB ,&
&  0.33648E+01_JPRB, 0.45416E+01_JPRB, 0.49557E+01_JPRB, 0.56489E+01_JPRB, 0.81754E+01_JPRB, 0.84817E+01_JPRB, 0.65508E+01_JPRB ,&
&  0.55577E+01_JPRB, 0.41393E+01_JPRB, 0.32251E+01_JPRB, 0.26724E+01_JPRB, 0.35149E+01_JPRB, 0.33773E+01_JPRB, 0.12514E+01_JPRB ,&
&  0.74024E+00_JPRB, 0.58316E+00_JPRB, 0.43022E+00_JPRB, 0.41453E+00_JPRB, 0.43422E+00_JPRB, 0.62645E+00_JPRB, 0.14609E+01_JPRB ,&
&  0.17036E+01_JPRB, 0.27066E+01_JPRB, 0.30292E+01_JPRB, 0.32744E+01_JPRB, 0.30760E+01_JPRB, 0.30333E+01_JPRB, 0.35054E+01_JPRB ,&
&  0.31433E+01_JPRB, 0.24746E+01_JPRB, 0.18550E+01_JPRB, 0.14358E+01_JPRB, 0.11316E+01_JPRB, 0.97495E+00_JPRB, 0.88196E+00_JPRB ,&
&  0.83176E+00_JPRB, 0.80485E+00_JPRB, 0.78346E+00_JPRB, 0.76301E+00_JPRB, 0.73933E+00_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.67562E+00_JPRB, 0.67820E+00_JPRB, 0.69064E+00_JPRB, 0.68264E+00_JPRB, 0.67124E+00_JPRB, 0.66985E+00_JPRB, 0.65811E+00_JPRB ,&
&  0.63472E+00_JPRB, 0.61763E+00_JPRB, 0.63131E+00_JPRB, 0.66096E+00_JPRB, 0.70818E+00_JPRB, 0.76270E+00_JPRB, 0.80754E+00_JPRB ,&
&  0.88997E+00_JPRB, 0.10722E+01_JPRB, 0.13200E+01_JPRB, 0.16442E+01_JPRB, 0.17355E+01_JPRB, 0.15220E+01_JPRB, 0.15275E+01_JPRB ,&
&  0.15009E+01_JPRB, 0.15007E+01_JPRB, 0.14887E+01_JPRB, 0.16211E+01_JPRB, 0.16327E+01_JPRB, 0.15384E+01_JPRB, 0.15074E+01_JPRB ,&
&  0.14942E+01_JPRB, 0.14975E+01_JPRB, 0.14441E+01_JPRB, 0.14621E+01_JPRB, 0.15288E+01_JPRB, 0.16099E+01_JPRB, 0.16841E+01_JPRB ,&
&  0.17156E+01_JPRB, 0.16573E+01_JPRB, 0.15575E+01_JPRB, 0.14886E+01_JPRB, 0.14647E+01_JPRB, 0.14868E+01_JPRB, 0.15688E+01_JPRB ,&
&  0.17519E+01_JPRB, 0.22054E+01_JPRB, 0.31938E+01_JPRB, 0.44571E+01_JPRB, 0.43248E+01_JPRB, 0.46522E+01_JPRB, 0.42251E+01_JPRB ,&
&  0.38480E+01_JPRB, 0.38541E+01_JPRB, 0.36693E+01_JPRB, 0.39552E+01_JPRB, 0.33879E+01_JPRB, 0.32070E+01_JPRB, 0.32054E+01_JPRB ,&
&  0.44185E+01_JPRB, 0.52866E+01_JPRB, 0.64180E+01_JPRB, 0.68928E+01_JPRB, 0.84691E+01_JPRB, 0.10890E+02_JPRB, 0.67500E+01_JPRB ,&
&  0.47715E+01_JPRB, 0.43164E+01_JPRB, 0.33748E+01_JPRB, 0.28392E+01_JPRB, 0.32604E+01_JPRB, 0.39871E+01_JPRB, 0.28342E+01_JPRB ,&
&  0.93579E+00_JPRB, 0.59052E+00_JPRB, 0.50105E+00_JPRB, 0.49330E+00_JPRB, 0.41878E+00_JPRB, 0.63010E+00_JPRB, 0.20186E+01_JPRB ,&
&  0.18100E+01_JPRB, 0.23920E+01_JPRB, 0.26549E+01_JPRB, 0.28573E+01_JPRB, 0.28386E+01_JPRB, 0.21831E+01_JPRB, 0.20216E+01_JPRB ,&
&  0.14289E+01_JPRB, 0.11036E+01_JPRB, 0.98059E+00_JPRB, 0.91066E+00_JPRB, 0.81535E+00_JPRB, 0.76491E+00_JPRB, 0.72257E+00_JPRB ,&
&  0.70529E+00_JPRB, 0.69760E+00_JPRB, 0.68841E+00_JPRB, 0.67630E+00_JPRB, 0.67789E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.65500E+00_JPRB, 0.67825E+00_JPRB, 0.70424E+00_JPRB, 0.72923E+00_JPRB, 0.74020E+00_JPRB, 0.72840E+00_JPRB, 0.70877E+00_JPRB ,&
&  0.69881E+00_JPRB, 0.69289E+00_JPRB, 0.71325E+00_JPRB, 0.74762E+00_JPRB, 0.81305E+00_JPRB, 0.88527E+00_JPRB, 0.98886E+00_JPRB ,&
&  0.11490E+01_JPRB, 0.13658E+01_JPRB, 0.16220E+01_JPRB, 0.18764E+01_JPRB, 0.18842E+01_JPRB, 0.15955E+01_JPRB, 0.15361E+01_JPRB ,&
&  0.15732E+01_JPRB, 0.16652E+01_JPRB, 0.16007E+01_JPRB, 0.17697E+01_JPRB, 0.18703E+01_JPRB, 0.18635E+01_JPRB, 0.18230E+01_JPRB ,&
&  0.18140E+01_JPRB, 0.18252E+01_JPRB, 0.18421E+01_JPRB, 0.19188E+01_JPRB, 0.19982E+01_JPRB, 0.20545E+01_JPRB, 0.20700E+01_JPRB ,&
&  0.19919E+01_JPRB, 0.18782E+01_JPRB, 0.18112E+01_JPRB, 0.17989E+01_JPRB, 0.18349E+01_JPRB, 0.19015E+01_JPRB, 0.20698E+01_JPRB ,&
&  0.24410E+01_JPRB, 0.31106E+01_JPRB, 0.40147E+01_JPRB, 0.44029E+01_JPRB, 0.43082E+01_JPRB, 0.41653E+01_JPRB, 0.39263E+01_JPRB ,&
&  0.37366E+01_JPRB, 0.38094E+01_JPRB, 0.37586E+01_JPRB, 0.37081E+01_JPRB, 0.34912E+01_JPRB, 0.36119E+01_JPRB, 0.39159E+01_JPRB ,&
&  0.54352E+01_JPRB, 0.66622E+01_JPRB, 0.69330E+01_JPRB, 0.69447E+01_JPRB, 0.73183E+01_JPRB, 0.84599E+01_JPRB, 0.77505E+01_JPRB ,&
&  0.56251E+01_JPRB, 0.39409E+01_JPRB, 0.31502E+01_JPRB, 0.30669E+01_JPRB, 0.30139E+01_JPRB, 0.31510E+01_JPRB, 0.35562E+01_JPRB ,&
&  0.29335E+01_JPRB, 0.11508E+01_JPRB, 0.72503E+00_JPRB, 0.95769E+00_JPRB, 0.60205E+00_JPRB, 0.64999E+00_JPRB, 0.11084E+01_JPRB ,&
&  0.14221E+01_JPRB, 0.17771E+01_JPRB, 0.20221E+01_JPRB, 0.23149E+01_JPRB, 0.20351E+01_JPRB, 0.15525E+01_JPRB, 0.11600E+01_JPRB ,&
&  0.90110E+00_JPRB, 0.82214E+00_JPRB, 0.81574E+00_JPRB, 0.75938E+00_JPRB, 0.69786E+00_JPRB, 0.67525E+00_JPRB, 0.64889E+00_JPRB ,&
&  0.63425E+00_JPRB, 0.64837E+00_JPRB, 0.66067E+00_JPRB, 0.65515E+00_JPRB, 0.64358E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.70001E+00_JPRB, 0.73128E+00_JPRB, 0.78455E+00_JPRB, 0.80325E+00_JPRB, 0.79352E+00_JPRB, 0.79035E+00_JPRB, 0.80249E+00_JPRB ,&
&  0.81863E+00_JPRB, 0.83773E+00_JPRB, 0.87916E+00_JPRB, 0.95004E+00_JPRB, 0.10662E+01_JPRB, 0.11891E+01_JPRB, 0.13423E+01_JPRB ,&
&  0.15154E+01_JPRB, 0.16983E+01_JPRB, 0.18896E+01_JPRB, 0.20667E+01_JPRB, 0.21646E+01_JPRB, 0.22276E+01_JPRB, 0.14864E+01_JPRB ,&
&  0.13619E+01_JPRB, 0.18690E+01_JPRB, 0.17429E+01_JPRB, 0.19393E+01_JPRB, 0.20678E+01_JPRB, 0.21447E+01_JPRB, 0.20779E+01_JPRB ,&
&  0.20833E+01_JPRB, 0.21216E+01_JPRB, 0.21771E+01_JPRB, 0.22011E+01_JPRB, 0.22114E+01_JPRB, 0.22181E+01_JPRB, 0.21820E+01_JPRB ,&
&  0.21281E+01_JPRB, 0.20989E+01_JPRB, 0.21193E+01_JPRB, 0.21747E+01_JPRB, 0.22721E+01_JPRB, 0.24351E+01_JPRB, 0.27159E+01_JPRB ,&
&  0.31280E+01_JPRB, 0.36339E+01_JPRB, 0.41041E+01_JPRB, 0.39784E+01_JPRB, 0.38660E+01_JPRB, 0.37374E+01_JPRB, 0.37212E+01_JPRB ,&
&  0.35655E+01_JPRB, 0.33349E+01_JPRB, 0.35814E+01_JPRB, 0.38046E+01_JPRB, 0.37566E+01_JPRB, 0.39218E+01_JPRB, 0.45194E+01_JPRB ,&
&  0.61298E+01_JPRB, 0.70678E+01_JPRB, 0.74119E+01_JPRB, 0.62966E+01_JPRB, 0.58772E+01_JPRB, 0.64780E+01_JPRB, 0.66579E+01_JPRB ,&
&  0.56106E+01_JPRB, 0.42530E+01_JPRB, 0.34550E+01_JPRB, 0.29260E+01_JPRB, 0.25889E+01_JPRB, 0.24957E+01_JPRB, 0.26081E+01_JPRB ,&
&  0.26126E+01_JPRB, 0.27942E+01_JPRB, 0.21411E+01_JPRB, 0.12216E+01_JPRB, 0.87193E+00_JPRB, 0.87057E+00_JPRB, 0.10684E+01_JPRB ,&
&  0.12534E+01_JPRB, 0.13950E+01_JPRB, 0.14062E+01_JPRB, 0.19554E+01_JPRB, 0.12476E+01_JPRB, 0.95857E+00_JPRB, 0.77273E+00_JPRB ,&
&  0.73202E+00_JPRB, 0.74413E+00_JPRB, 0.74859E+00_JPRB, 0.70733E+00_JPRB, 0.66548E+00_JPRB, 0.66824E+00_JPRB, 0.65853E+00_JPRB ,&
&  0.67225E+00_JPRB, 0.68017E+00_JPRB, 0.67480E+00_JPRB, 0.67826E+00_JPRB, 0.69193E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.80353E+00_JPRB, 0.84243E+00_JPRB, 0.87918E+00_JPRB, 0.89031E+00_JPRB, 0.90624E+00_JPRB, 0.93213E+00_JPRB, 0.98262E+00_JPRB ,&
&  0.10410E+01_JPRB, 0.11060E+01_JPRB, 0.11890E+01_JPRB, 0.12931E+01_JPRB, 0.14282E+01_JPRB, 0.15527E+01_JPRB, 0.16876E+01_JPRB ,&
&  0.18078E+01_JPRB, 0.19084E+01_JPRB, 0.20066E+01_JPRB, 0.20980E+01_JPRB, 0.21583E+01_JPRB, 0.22470E+01_JPRB, 0.19089E+01_JPRB ,&
&  0.12871E+01_JPRB, 0.21326E+01_JPRB, 0.18334E+01_JPRB, 0.20347E+01_JPRB, 0.20858E+01_JPRB, 0.22319E+01_JPRB, 0.21992E+01_JPRB ,&
&  0.22127E+01_JPRB, 0.22289E+01_JPRB, 0.22313E+01_JPRB, 0.22487E+01_JPRB, 0.22549E+01_JPRB, 0.22612E+01_JPRB, 0.22663E+01_JPRB ,&
&  0.22868E+01_JPRB, 0.23428E+01_JPRB, 0.24328E+01_JPRB, 0.25378E+01_JPRB, 0.27092E+01_JPRB, 0.29428E+01_JPRB, 0.31743E+01_JPRB ,&
&  0.34251E+01_JPRB, 0.36883E+01_JPRB, 0.37610E+01_JPRB, 0.36212E+01_JPRB, 0.35306E+01_JPRB, 0.35936E+01_JPRB, 0.37640E+01_JPRB ,&
&  0.36630E+01_JPRB, 0.34885E+01_JPRB, 0.36779E+01_JPRB, 0.37413E+01_JPRB, 0.37540E+01_JPRB, 0.43245E+01_JPRB, 0.48710E+01_JPRB ,&
&  0.60488E+01_JPRB, 0.62583E+01_JPRB, 0.72185E+01_JPRB, 0.59550E+01_JPRB, 0.48794E+01_JPRB, 0.50687E+01_JPRB, 0.45908E+01_JPRB ,&
&  0.37168E+01_JPRB, 0.33991E+01_JPRB, 0.29400E+01_JPRB, 0.23849E+01_JPRB, 0.22207E+01_JPRB, 0.20599E+01_JPRB, 0.21391E+01_JPRB ,&
&  0.22048E+01_JPRB, 0.21784E+01_JPRB, 0.16251E+01_JPRB, 0.10015E+01_JPRB, 0.88779E+00_JPRB, 0.92310E+00_JPRB, 0.10817E+01_JPRB ,&
&  0.11457E+01_JPRB, 0.12332E+01_JPRB, 0.13179E+01_JPRB, 0.12257E+01_JPRB, 0.83642E+00_JPRB, 0.69225E+00_JPRB, 0.65255E+00_JPRB ,&
&  0.68694E+00_JPRB, 0.72334E+00_JPRB, 0.72230E+00_JPRB, 0.70852E+00_JPRB, 0.67924E+00_JPRB, 0.70801E+00_JPRB, 0.72032E+00_JPRB ,&
&  0.75357E+00_JPRB, 0.75217E+00_JPRB, 0.76999E+00_JPRB, 0.78475E+00_JPRB, 0.79188E+00_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.98671E+00_JPRB, 0.10552E+01_JPRB, 0.11410E+01_JPRB, 0.12222E+01_JPRB, 0.12716E+01_JPRB, 0.13139E+01_JPRB, 0.13381E+01_JPRB ,&
&  0.13264E+01_JPRB, 0.13151E+01_JPRB, 0.14120E+01_JPRB, 0.15082E+01_JPRB, 0.16286E+01_JPRB, 0.17421E+01_JPRB, 0.18509E+01_JPRB ,&
&  0.19365E+01_JPRB, 0.20162E+01_JPRB, 0.21041E+01_JPRB, 0.22247E+01_JPRB, 0.23582E+01_JPRB, 0.25879E+01_JPRB, 0.24390E+01_JPRB ,&
&  0.17764E+01_JPRB, 0.16102E+01_JPRB, 0.19515E+01_JPRB, 0.19836E+01_JPRB, 0.19301E+01_JPRB, 0.21005E+01_JPRB, 0.22350E+01_JPRB ,&
&  0.22156E+01_JPRB, 0.22439E+01_JPRB, 0.22482E+01_JPRB, 0.22682E+01_JPRB, 0.22555E+01_JPRB, 0.22988E+01_JPRB, 0.23598E+01_JPRB ,&
&  0.24543E+01_JPRB, 0.25682E+01_JPRB, 0.26938E+01_JPRB, 0.28482E+01_JPRB, 0.30173E+01_JPRB, 0.32053E+01_JPRB, 0.33716E+01_JPRB ,&
&  0.35102E+01_JPRB, 0.36374E+01_JPRB, 0.36298E+01_JPRB, 0.34751E+01_JPRB, 0.34997E+01_JPRB, 0.36332E+01_JPRB, 0.36925E+01_JPRB ,&
&  0.36777E+01_JPRB, 0.37621E+01_JPRB, 0.39483E+01_JPRB, 0.40241E+01_JPRB, 0.39791E+01_JPRB, 0.45771E+01_JPRB, 0.49772E+01_JPRB ,&
&  0.55137E+01_JPRB, 0.56122E+01_JPRB, 0.65780E+01_JPRB, 0.55058E+01_JPRB, 0.46018E+01_JPRB, 0.42462E+01_JPRB, 0.36605E+01_JPRB ,&
&  0.31284E+01_JPRB, 0.29871E+01_JPRB, 0.27594E+01_JPRB, 0.20556E+01_JPRB, 0.19417E+01_JPRB, 0.15856E+01_JPRB, 0.19083E+01_JPRB ,&
&  0.20773E+01_JPRB, 0.22519E+01_JPRB, 0.14158E+01_JPRB, 0.10558E+01_JPRB, 0.90411E+00_JPRB, 0.96668E+00_JPRB, 0.10181E+01_JPRB ,&
&  0.10111E+01_JPRB, 0.99976E+00_JPRB, 0.94354E+00_JPRB, 0.91148E+00_JPRB, 0.68223E+00_JPRB, 0.62071E+00_JPRB, 0.61344E+00_JPRB ,&
&  0.66305E+00_JPRB, 0.70616E+00_JPRB, 0.71353E+00_JPRB, 0.71812E+00_JPRB, 0.73646E+00_JPRB, 0.78247E+00_JPRB, 0.81314E+00_JPRB ,&
&  0.85168E+00_JPRB, 0.87231E+00_JPRB, 0.90036E+00_JPRB, 0.91433E+00_JPRB, 0.93532E+00_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.10148E+01_JPRB, 0.10714E+01_JPRB, 0.11616E+01_JPRB, 0.12365E+01_JPRB, 0.12857E+01_JPRB, 0.13012E+01_JPRB, 0.12910E+01_JPRB ,&
&  0.13033E+01_JPRB, 0.13465E+01_JPRB, 0.14317E+01_JPRB, 0.15041E+01_JPRB, 0.15865E+01_JPRB, 0.16670E+01_JPRB, 0.17479E+01_JPRB ,&
&  0.18212E+01_JPRB, 0.18880E+01_JPRB, 0.19814E+01_JPRB, 0.21033E+01_JPRB, 0.22498E+01_JPRB, 0.24861E+01_JPRB, 0.25561E+01_JPRB ,&
&  0.20545E+01_JPRB, 0.14289E+01_JPRB, 0.17125E+01_JPRB, 0.17929E+01_JPRB, 0.17403E+01_JPRB, 0.19195E+01_JPRB, 0.20415E+01_JPRB ,&
&  0.19907E+01_JPRB, 0.20447E+01_JPRB, 0.21324E+01_JPRB, 0.21894E+01_JPRB, 0.22461E+01_JPRB, 0.23650E+01_JPRB, 0.24725E+01_JPRB ,&
&  0.25958E+01_JPRB, 0.27097E+01_JPRB, 0.28185E+01_JPRB, 0.29368E+01_JPRB, 0.30469E+01_JPRB, 0.31732E+01_JPRB, 0.33259E+01_JPRB ,&
&  0.34924E+01_JPRB, 0.36104E+01_JPRB, 0.35912E+01_JPRB, 0.35504E+01_JPRB, 0.35740E+01_JPRB, 0.37105E+01_JPRB, 0.38392E+01_JPRB ,&
&  0.39155E+01_JPRB, 0.39102E+01_JPRB, 0.41975E+01_JPRB, 0.45867E+01_JPRB, 0.47669E+01_JPRB, 0.46230E+01_JPRB, 0.49330E+01_JPRB ,&
&  0.51592E+01_JPRB, 0.50973E+01_JPRB, 0.54213E+01_JPRB, 0.60202E+01_JPRB, 0.41067E+01_JPRB, 0.38749E+01_JPRB, 0.30637E+01_JPRB ,&
&  0.31988E+01_JPRB, 0.29966E+01_JPRB, 0.26906E+01_JPRB, 0.22659E+01_JPRB, 0.17219E+01_JPRB, 0.13441E+01_JPRB, 0.15605E+01_JPRB ,&
&  0.18584E+01_JPRB, 0.16868E+01_JPRB, 0.12233E+01_JPRB, 0.10214E+01_JPRB, 0.94616E+00_JPRB, 0.11060E+01_JPRB, 0.11089E+01_JPRB ,&
&  0.10001E+01_JPRB, 0.96215E+00_JPRB, 0.94025E+00_JPRB, 0.10779E+01_JPRB, 0.83552E+00_JPRB, 0.61694E+00_JPRB, 0.58171E+00_JPRB ,&
&  0.63286E+00_JPRB, 0.68404E+00_JPRB, 0.72916E+00_JPRB, 0.76320E+00_JPRB, 0.79507E+00_JPRB, 0.83160E+00_JPRB, 0.85521E+00_JPRB ,&
&  0.90660E+00_JPRB, 0.90416E+00_JPRB, 0.94579E+00_JPRB, 0.96530E+00_JPRB, 0.99260E+00_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.10043E+01_JPRB, 0.10376E+01_JPRB, 0.10716E+01_JPRB, 0.11182E+01_JPRB, 0.11616E+01_JPRB, 0.11799E+01_JPRB, 0.11814E+01_JPRB ,&
&  0.12011E+01_JPRB, 0.12467E+01_JPRB, 0.12977E+01_JPRB, 0.13426E+01_JPRB, 0.14070E+01_JPRB, 0.14644E+01_JPRB, 0.15253E+01_JPRB ,&
&  0.15892E+01_JPRB, 0.16730E+01_JPRB, 0.17950E+01_JPRB, 0.19285E+01_JPRB, 0.20455E+01_JPRB, 0.21972E+01_JPRB, 0.23525E+01_JPRB ,&
&  0.24979E+01_JPRB, 0.22513E+01_JPRB, 0.21177E+01_JPRB, 0.16859E+01_JPRB, 0.12651E+01_JPRB, 0.15979E+01_JPRB, 0.16524E+01_JPRB ,&
&  0.17219E+01_JPRB, 0.17822E+01_JPRB, 0.18407E+01_JPRB, 0.19342E+01_JPRB, 0.20992E+01_JPRB, 0.22448E+01_JPRB, 0.23554E+01_JPRB ,&
&  0.24488E+01_JPRB, 0.25285E+01_JPRB, 0.25933E+01_JPRB, 0.26679E+01_JPRB, 0.27560E+01_JPRB, 0.28410E+01_JPRB, 0.29334E+01_JPRB ,&
&  0.30647E+01_JPRB, 0.32223E+01_JPRB, 0.33285E+01_JPRB, 0.34485E+01_JPRB, 0.35108E+01_JPRB, 0.37205E+01_JPRB, 0.39540E+01_JPRB ,&
&  0.41007E+01_JPRB, 0.41804E+01_JPRB, 0.44600E+01_JPRB, 0.47955E+01_JPRB, 0.48385E+01_JPRB, 0.43117E+01_JPRB, 0.44913E+01_JPRB ,&
&  0.47534E+01_JPRB, 0.46130E+01_JPRB, 0.44973E+01_JPRB, 0.57877E+01_JPRB, 0.35482E+01_JPRB, 0.37170E+01_JPRB, 0.36044E+01_JPRB ,&
&  0.31201E+01_JPRB, 0.29585E+01_JPRB, 0.25949E+01_JPRB, 0.20105E+01_JPRB, 0.16145E+01_JPRB, 0.12214E+01_JPRB, 0.13981E+01_JPRB ,&
&  0.15563E+01_JPRB, 0.12990E+01_JPRB, 0.99947E+00_JPRB, 0.85369E+00_JPRB, 0.89451E+00_JPRB, 0.11265E+01_JPRB, 0.11469E+01_JPRB ,&
&  0.93099E+00_JPRB, 0.96524E+00_JPRB, 0.89773E+00_JPRB, 0.13047E+01_JPRB, 0.12510E+01_JPRB, 0.60650E+00_JPRB, 0.51566E+00_JPRB ,&
&  0.54887E+00_JPRB, 0.61631E+00_JPRB, 0.69450E+00_JPRB, 0.74621E+00_JPRB, 0.76423E+00_JPRB, 0.78871E+00_JPRB, 0.81300E+00_JPRB ,&
&  0.83986E+00_JPRB, 0.88304E+00_JPRB, 0.93800E+00_JPRB, 0.96416E+00_JPRB, 0.98590E+00_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.84981E+00_JPRB, 0.87714E+00_JPRB, 0.89803E+00_JPRB, 0.90495E+00_JPRB, 0.89859E+00_JPRB, 0.88765E+00_JPRB, 0.86208E+00_JPRB ,&
&  0.83357E+00_JPRB, 0.84648E+00_JPRB, 0.89376E+00_JPRB, 0.99981E+00_JPRB, 0.10784E+01_JPRB, 0.11268E+01_JPRB, 0.11920E+01_JPRB ,&
&  0.12585E+01_JPRB, 0.12887E+01_JPRB, 0.12845E+01_JPRB, 0.13189E+01_JPRB, 0.13464E+01_JPRB, 0.13738E+01_JPRB, 0.15614E+01_JPRB ,&
&  0.18432E+01_JPRB, 0.21802E+01_JPRB, 0.24923E+01_JPRB, 0.27305E+01_JPRB, 0.22447E+01_JPRB, 0.15965E+01_JPRB, 0.17027E+01_JPRB ,&
&  0.19044E+01_JPRB, 0.18142E+01_JPRB, 0.16216E+01_JPRB, 0.16347E+01_JPRB, 0.16802E+01_JPRB, 0.17810E+01_JPRB, 0.19101E+01_JPRB ,&
&  0.20007E+01_JPRB, 0.20636E+01_JPRB, 0.21243E+01_JPRB, 0.21765E+01_JPRB, 0.22297E+01_JPRB, 0.22837E+01_JPRB, 0.23517E+01_JPRB ,&
&  0.24062E+01_JPRB, 0.24557E+01_JPRB, 0.26093E+01_JPRB, 0.28298E+01_JPRB, 0.30667E+01_JPRB, 0.33856E+01_JPRB, 0.36621E+01_JPRB ,&
&  0.38287E+01_JPRB, 0.39603E+01_JPRB, 0.44148E+01_JPRB, 0.47103E+01_JPRB, 0.44575E+01_JPRB, 0.40467E+01_JPRB, 0.42114E+01_JPRB ,&
&  0.42417E+01_JPRB, 0.39698E+01_JPRB, 0.31441E+01_JPRB, 0.45487E+01_JPRB, 0.45672E+01_JPRB, 0.38942E+01_JPRB, 0.31736E+01_JPRB ,&
&  0.28155E+01_JPRB, 0.26681E+01_JPRB, 0.23882E+01_JPRB, 0.20024E+01_JPRB, 0.16157E+01_JPRB, 0.16037E+01_JPRB, 0.14348E+01_JPRB ,&
&  0.14110E+01_JPRB, 0.11342E+01_JPRB, 0.86772E+00_JPRB, 0.75632E+00_JPRB, 0.74939E+00_JPRB, 0.99309E+00_JPRB, 0.96849E+00_JPRB ,&
&  0.78823E+00_JPRB, 0.80949E+00_JPRB, 0.78171E+00_JPRB, 0.12438E+01_JPRB, 0.23279E+01_JPRB, 0.63272E+00_JPRB, 0.46499E+00_JPRB ,&
&  0.48799E+00_JPRB, 0.52867E+00_JPRB, 0.56799E+00_JPRB, 0.58188E+00_JPRB, 0.58386E+00_JPRB, 0.60890E+00_JPRB, 0.65165E+00_JPRB ,&
&  0.69929E+00_JPRB, 0.73443E+00_JPRB, 0.75269E+00_JPRB, 0.79147E+00_JPRB, 0.82252E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.65105E+00_JPRB, 0.66664E+00_JPRB, 0.66545E+00_JPRB, 0.67047E+00_JPRB, 0.66351E+00_JPRB, 0.66331E+00_JPRB, 0.64559E+00_JPRB ,&
&  0.63148E+00_JPRB, 0.64032E+00_JPRB, 0.67401E+00_JPRB, 0.73914E+00_JPRB, 0.75615E+00_JPRB, 0.77393E+00_JPRB, 0.77906E+00_JPRB ,&
&  0.80156E+00_JPRB, 0.78754E+00_JPRB, 0.78089E+00_JPRB, 0.75820E+00_JPRB, 0.76473E+00_JPRB, 0.80365E+00_JPRB, 0.89304E+00_JPRB ,&
&  0.10199E+01_JPRB, 0.11798E+01_JPRB, 0.14014E+01_JPRB, 0.16553E+01_JPRB, 0.22073E+01_JPRB, 0.18922E+01_JPRB, 0.16446E+01_JPRB ,&
&  0.17964E+01_JPRB, 0.15215E+01_JPRB, 0.14078E+01_JPRB, 0.13329E+01_JPRB, 0.12752E+01_JPRB, 0.12380E+01_JPRB, 0.12815E+01_JPRB ,&
&  0.13094E+01_JPRB, 0.13630E+01_JPRB, 0.14262E+01_JPRB, 0.14535E+01_JPRB, 0.14837E+01_JPRB, 0.14694E+01_JPRB, 0.14457E+01_JPRB ,&
&  0.14057E+01_JPRB, 0.14915E+01_JPRB, 0.16778E+01_JPRB, 0.18097E+01_JPRB, 0.21586E+01_JPRB, 0.24579E+01_JPRB, 0.27111E+01_JPRB ,&
&  0.28514E+01_JPRB, 0.30491E+01_JPRB, 0.34607E+01_JPRB, 0.35734E+01_JPRB, 0.36472E+01_JPRB, 0.34809E+01_JPRB, 0.33805E+01_JPRB ,&
&  0.31870E+01_JPRB, 0.25677E+01_JPRB, 0.21172E+01_JPRB, 0.31986E+01_JPRB, 0.32937E+01_JPRB, 0.26090E+01_JPRB, 0.27086E+01_JPRB ,&
&  0.24954E+01_JPRB, 0.23477E+01_JPRB, 0.22536E+01_JPRB, 0.19200E+01_JPRB, 0.15217E+01_JPRB, 0.14629E+01_JPRB, 0.14440E+01_JPRB ,&
&  0.12655E+01_JPRB, 0.95014E+00_JPRB, 0.73887E+00_JPRB, 0.67729E+00_JPRB, 0.68671E+00_JPRB, 0.84700E+00_JPRB, 0.87139E+00_JPRB ,&
&  0.86074E+00_JPRB, 0.68137E+00_JPRB, 0.71490E+00_JPRB, 0.10389E+01_JPRB, 0.89448E+00_JPRB, 0.52699E+00_JPRB, 0.41841E+00_JPRB ,&
&  0.43072E+00_JPRB, 0.45154E+00_JPRB, 0.47231E+00_JPRB, 0.48312E+00_JPRB, 0.48730E+00_JPRB, 0.49516E+00_JPRB, 0.52675E+00_JPRB ,&
&  0.55310E+00_JPRB, 0.57770E+00_JPRB, 0.59810E+00_JPRB, 0.62652E+00_JPRB, 0.64265E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.65220E+00_JPRB, 0.66459E+00_JPRB, 0.68236E+00_JPRB, 0.69340E+00_JPRB, 0.70603E+00_JPRB, 0.71406E+00_JPRB, 0.74323E+00_JPRB ,&
&  0.76458E+00_JPRB, 0.78399E+00_JPRB, 0.81138E+00_JPRB, 0.84070E+00_JPRB, 0.85234E+00_JPRB, 0.86461E+00_JPRB, 0.86962E+00_JPRB ,&
&  0.88524E+00_JPRB, 0.90172E+00_JPRB, 0.89745E+00_JPRB, 0.90163E+00_JPRB, 0.90588E+00_JPRB, 0.97206E+00_JPRB, 0.97108E+00_JPRB ,&
&  0.96178E+00_JPRB, 0.97729E+00_JPRB, 0.99165E+00_JPRB, 0.10472E+01_JPRB, 0.12113E+01_JPRB, 0.13435E+01_JPRB, 0.13571E+01_JPRB ,&
&  0.14257E+01_JPRB, 0.12156E+01_JPRB, 0.12169E+01_JPRB, 0.98087E+00_JPRB, 0.93513E+00_JPRB, 0.95363E+00_JPRB, 0.92683E+00_JPRB ,&
&  0.92465E+00_JPRB, 0.95666E+00_JPRB, 0.10211E+01_JPRB, 0.10298E+01_JPRB, 0.10863E+01_JPRB, 0.11324E+01_JPRB, 0.11606E+01_JPRB ,&
&  0.12131E+01_JPRB, 0.12824E+01_JPRB, 0.13571E+01_JPRB, 0.14815E+01_JPRB, 0.15824E+01_JPRB, 0.18766E+01_JPRB, 0.20016E+01_JPRB ,&
&  0.20502E+01_JPRB, 0.21520E+01_JPRB, 0.20040E+01_JPRB, 0.20383E+01_JPRB, 0.23290E+01_JPRB, 0.23310E+01_JPRB, 0.22720E+01_JPRB ,&
&  0.23416E+01_JPRB, 0.20919E+01_JPRB, 0.15434E+01_JPRB, 0.23098E+01_JPRB, 0.25697E+01_JPRB, 0.23554E+01_JPRB, 0.23094E+01_JPRB ,&
&  0.21463E+01_JPRB, 0.19146E+01_JPRB, 0.17484E+01_JPRB, 0.16613E+01_JPRB, 0.14447E+01_JPRB, 0.12885E+01_JPRB, 0.12369E+01_JPRB ,&
&  0.10190E+01_JPRB, 0.81461E+00_JPRB, 0.69213E+00_JPRB, 0.70844E+00_JPRB, 0.83703E+00_JPRB, 0.11578E+01_JPRB, 0.10582E+01_JPRB ,&
&  0.81916E+00_JPRB, 0.65366E+00_JPRB, 0.86225E+00_JPRB, 0.12404E+01_JPRB, 0.72971E+00_JPRB, 0.52418E+00_JPRB, 0.41840E+00_JPRB ,&
&  0.42321E+00_JPRB, 0.44872E+00_JPRB, 0.47429E+00_JPRB, 0.49417E+00_JPRB, 0.51847E+00_JPRB, 0.53996E+00_JPRB, 0.55659E+00_JPRB ,&
&  0.57296E+00_JPRB, 0.59335E+00_JPRB, 0.61083E+00_JPRB, 0.63111E+00_JPRB, 0.64803E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.77101E+00_JPRB, 0.80216E+00_JPRB, 0.82353E+00_JPRB, 0.84094E+00_JPRB, 0.85145E+00_JPRB, 0.86210E+00_JPRB, 0.88119E+00_JPRB ,&
&  0.89744E+00_JPRB, 0.91425E+00_JPRB, 0.93195E+00_JPRB, 0.95661E+00_JPRB, 0.98805E+00_JPRB, 0.10137E+01_JPRB, 0.10246E+01_JPRB ,&
&  0.10311E+01_JPRB, 0.10660E+01_JPRB, 0.10880E+01_JPRB, 0.11316E+01_JPRB, 0.11488E+01_JPRB, 0.11790E+01_JPRB, 0.11862E+01_JPRB ,&
&  0.11831E+01_JPRB, 0.11869E+01_JPRB, 0.11953E+01_JPRB, 0.12153E+01_JPRB, 0.12317E+01_JPRB, 0.13470E+01_JPRB, 0.15182E+01_JPRB ,&
&  0.13088E+01_JPRB, 0.12140E+01_JPRB, 0.99240E+00_JPRB, 0.82996E+00_JPRB, 0.83275E+00_JPRB, 0.86242E+00_JPRB, 0.88566E+00_JPRB ,&
&  0.91550E+00_JPRB, 0.95465E+00_JPRB, 0.10015E+01_JPRB, 0.10245E+01_JPRB, 0.11053E+01_JPRB, 0.12461E+01_JPRB, 0.13004E+01_JPRB ,&
&  0.13897E+01_JPRB, 0.14689E+01_JPRB, 0.15131E+01_JPRB, 0.16611E+01_JPRB, 0.16654E+01_JPRB, 0.20134E+01_JPRB, 0.21840E+01_JPRB ,&
&  0.22269E+01_JPRB, 0.21096E+01_JPRB, 0.16197E+01_JPRB, 0.16207E+01_JPRB, 0.17893E+01_JPRB, 0.17445E+01_JPRB, 0.16867E+01_JPRB ,&
&  0.19670E+01_JPRB, 0.19224E+01_JPRB, 0.16741E+01_JPRB, 0.21106E+01_JPRB, 0.22301E+01_JPRB, 0.21690E+01_JPRB, 0.19107E+01_JPRB ,&
&  0.16696E+01_JPRB, 0.13069E+01_JPRB, 0.12876E+01_JPRB, 0.12992E+01_JPRB, 0.12104E+01_JPRB, 0.11047E+01_JPRB, 0.96302E+00_JPRB ,&
&  0.82877E+00_JPRB, 0.73282E+00_JPRB, 0.69721E+00_JPRB, 0.73927E+00_JPRB, 0.10448E+01_JPRB, 0.10873E+01_JPRB, 0.13487E+01_JPRB ,&
&  0.68833E+00_JPRB, 0.65085E+00_JPRB, 0.54723E+00_JPRB, 0.68487E+00_JPRB, 0.58891E+00_JPRB, 0.45153E+00_JPRB, 0.44472E+00_JPRB ,&
&  0.49533E+00_JPRB, 0.54156E+00_JPRB, 0.57538E+00_JPRB, 0.59985E+00_JPRB, 0.62127E+00_JPRB, 0.64581E+00_JPRB, 0.65750E+00_JPRB ,&
&  0.67421E+00_JPRB, 0.69262E+00_JPRB, 0.71049E+00_JPRB, 0.73647E+00_JPRB, 0.75004E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.81714E+00_JPRB, 0.84220E+00_JPRB, 0.86422E+00_JPRB, 0.88493E+00_JPRB, 0.90678E+00_JPRB, 0.92864E+00_JPRB, 0.94749E+00_JPRB ,&
&  0.96455E+00_JPRB, 0.98414E+00_JPRB, 0.10007E+01_JPRB, 0.10169E+01_JPRB, 0.10403E+01_JPRB, 0.10617E+01_JPRB, 0.10751E+01_JPRB ,&
&  0.10952E+01_JPRB, 0.11235E+01_JPRB, 0.11570E+01_JPRB, 0.11863E+01_JPRB, 0.12011E+01_JPRB, 0.12086E+01_JPRB, 0.12082E+01_JPRB ,&
&  0.11971E+01_JPRB, 0.11919E+01_JPRB, 0.11991E+01_JPRB, 0.12168E+01_JPRB, 0.12503E+01_JPRB, 0.12373E+01_JPRB, 0.13248E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.12886E+01_JPRB, 0.10831E+01_JPRB, 0.96492E+00_JPRB, 0.98157E+00_JPRB, 0.94414E+00_JPRB, 0.96265E+00_JPRB ,&
&  0.10242E+01_JPRB, 0.10562E+01_JPRB, 0.10958E+01_JPRB, 0.11296E+01_JPRB, 0.12040E+01_JPRB, 0.13156E+01_JPRB, 0.13962E+01_JPRB ,&
&  0.15277E+01_JPRB, 0.16765E+01_JPRB, 0.17931E+01_JPRB, 0.19321E+01_JPRB, 0.22257E+01_JPRB, 0.24751E+01_JPRB, 0.25993E+01_JPRB ,&
&  0.25724E+01_JPRB, 0.24502E+01_JPRB, 0.18509E+01_JPRB, 0.19146E+01_JPRB, 0.20353E+01_JPRB, 0.18890E+01_JPRB, 0.17215E+01_JPRB ,&
&  0.17898E+01_JPRB, 0.15097E+01_JPRB, 0.17335E+01_JPRB, 0.17711E+01_JPRB, 0.16936E+01_JPRB, 0.15028E+01_JPRB, 0.12440E+01_JPRB ,&
&  0.10138E+01_JPRB, 0.84002E+00_JPRB, 0.85894E+00_JPRB, 0.98590E+00_JPRB, 0.94627E+00_JPRB, 0.78131E+00_JPRB, 0.71121E+00_JPRB ,&
&  0.64039E+00_JPRB, 0.57926E+00_JPRB, 0.56990E+00_JPRB, 0.64513E+00_JPRB, 0.81039E+00_JPRB, 0.13093E+01_JPRB, 0.13379E+01_JPRB ,&
&  0.65591E+00_JPRB, 0.59533E+00_JPRB, 0.43683E+00_JPRB, 0.49970E+00_JPRB, 0.51300E+00_JPRB, 0.47221E+00_JPRB, 0.51640E+00_JPRB ,&
&  0.62904E+00_JPRB, 0.72774E+00_JPRB, 0.73893E+00_JPRB, 0.67871E+00_JPRB, 0.65860E+00_JPRB, 0.66789E+00_JPRB, 0.68854E+00_JPRB ,&
&  0.70986E+00_JPRB, 0.72963E+00_JPRB, 0.74952E+00_JPRB, 0.77093E+00_JPRB, 0.79285E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.84135E+00_JPRB, 0.86737E+00_JPRB, 0.88935E+00_JPRB, 0.90774E+00_JPRB, 0.93058E+00_JPRB, 0.95465E+00_JPRB, 0.97572E+00_JPRB ,&
&  0.99484E+00_JPRB, 0.10122E+01_JPRB, 0.10316E+01_JPRB, 0.10496E+01_JPRB, 0.10697E+01_JPRB, 0.10932E+01_JPRB, 0.11181E+01_JPRB ,&
&  0.11420E+01_JPRB, 0.11579E+01_JPRB, 0.11918E+01_JPRB, 0.12328E+01_JPRB, 0.12604E+01_JPRB, 0.12708E+01_JPRB, 0.12739E+01_JPRB ,&
&  0.12777E+01_JPRB, 0.13031E+01_JPRB, 0.13209E+01_JPRB, 0.13406E+01_JPRB, 0.13722E+01_JPRB, 0.14534E+01_JPRB, 0.12530E+01_JPRB ,&
&  0.13798E+01_JPRB, 0.12859E+01_JPRB, 0.11281E+01_JPRB, 0.10769E+01_JPRB, 0.10707E+01_JPRB, 0.10727E+01_JPRB, 0.10876E+01_JPRB ,&
&  0.11296E+01_JPRB, 0.11347E+01_JPRB, 0.11598E+01_JPRB, 0.11911E+01_JPRB, 0.12639E+01_JPRB, 0.13623E+01_JPRB, 0.14691E+01_JPRB ,&
&  0.16411E+01_JPRB, 0.18601E+01_JPRB, 0.21155E+01_JPRB, 0.23942E+01_JPRB, 0.27254E+01_JPRB, 0.30037E+01_JPRB, 0.31823E+01_JPRB ,&
&  0.32204E+01_JPRB, 0.31839E+01_JPRB, 0.26991E+01_JPRB, 0.28529E+01_JPRB, 0.28213E+01_JPRB, 0.23384E+01_JPRB, 0.19167E+01_JPRB ,&
&  0.15812E+01_JPRB, 0.12304E+01_JPRB, 0.12901E+01_JPRB, 0.13325E+01_JPRB, 0.12147E+01_JPRB, 0.10269E+01_JPRB, 0.85253E+00_JPRB ,&
&  0.65636E+00_JPRB, 0.59131E+00_JPRB, 0.58879E+00_JPRB, 0.69329E+00_JPRB, 0.68513E+00_JPRB, 0.59585E+00_JPRB, 0.57643E+00_JPRB ,&
&  0.54972E+00_JPRB, 0.51454E+00_JPRB, 0.51835E+00_JPRB, 0.59691E+00_JPRB, 0.73818E+00_JPRB, 0.73933E+00_JPRB, 0.79967E+00_JPRB ,&
&  0.61160E+00_JPRB, 0.53660E+00_JPRB, 0.54121E+00_JPRB, 0.54954E+00_JPRB, 0.59125E+00_JPRB, 0.62414E+00_JPRB, 0.72605E+00_JPRB ,&
&  0.88854E+00_JPRB, 0.10992E+01_JPRB, 0.10710E+01_JPRB, 0.86270E+00_JPRB, 0.70478E+00_JPRB, 0.67533E+00_JPRB, 0.69326E+00_JPRB ,&
&  0.71659E+00_JPRB, 0.73611E+00_JPRB, 0.75948E+00_JPRB, 0.78484E+00_JPRB, 0.81201E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.84845E+00_JPRB, 0.87233E+00_JPRB, 0.89092E+00_JPRB, 0.90530E+00_JPRB, 0.92619E+00_JPRB, 0.94790E+00_JPRB, 0.96878E+00_JPRB ,&
&  0.98751E+00_JPRB, 0.10035E+01_JPRB, 0.10232E+01_JPRB, 0.10408E+01_JPRB, 0.10614E+01_JPRB, 0.10847E+01_JPRB, 0.11114E+01_JPRB ,&
&  0.11338E+01_JPRB, 0.11592E+01_JPRB, 0.11749E+01_JPRB, 0.12181E+01_JPRB, 0.12718E+01_JPRB, 0.13033E+01_JPRB, 0.13213E+01_JPRB ,&
&  0.13224E+01_JPRB, 0.13465E+01_JPRB, 0.13735E+01_JPRB, 0.14038E+01_JPRB, 0.14142E+01_JPRB, 0.14777E+01_JPRB, 0.12988E+01_JPRB ,&
&  0.13532E+01_JPRB, 0.12790E+01_JPRB, 0.11685E+01_JPRB, 0.11553E+01_JPRB, 0.11493E+01_JPRB, 0.11560E+01_JPRB, 0.11621E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11978E+01_JPRB, 0.12070E+01_JPRB, 0.12334E+01_JPRB, 0.12906E+01_JPRB, 0.13901E+01_JPRB, 0.15150E+01_JPRB ,&
&  0.16901E+01_JPRB, 0.19287E+01_JPRB, 0.22177E+01_JPRB, 0.25364E+01_JPRB, 0.28991E+01_JPRB, 0.32262E+01_JPRB, 0.35357E+01_JPRB ,&
&  0.37684E+01_JPRB, 0.39035E+01_JPRB, 0.35835E+01_JPRB, 0.35557E+01_JPRB, 0.33599E+01_JPRB, 0.23241E+01_JPRB, 0.16824E+01_JPRB ,&
&  0.11928E+01_JPRB, 0.11325E+01_JPRB, 0.11715E+01_JPRB, 0.11173E+01_JPRB, 0.89775E+00_JPRB, 0.77263E+00_JPRB, 0.63090E+00_JPRB ,&
&  0.51963E+00_JPRB, 0.47811E+00_JPRB, 0.48322E+00_JPRB, 0.53787E+00_JPRB, 0.52138E+00_JPRB, 0.48508E+00_JPRB, 0.49935E+00_JPRB ,&
&  0.49483E+00_JPRB, 0.50750E+00_JPRB, 0.53846E+00_JPRB, 0.59670E+00_JPRB, 0.65801E+00_JPRB, 0.65657E+00_JPRB, 0.70899E+00_JPRB ,&
&  0.65692E+00_JPRB, 0.64531E+00_JPRB, 0.62839E+00_JPRB, 0.52624E+00_JPRB, 0.70253E+00_JPRB, 0.80046E+00_JPRB, 0.93197E+00_JPRB ,&
&  0.10683E+01_JPRB, 0.14657E+01_JPRB, 0.17130E+01_JPRB, 0.13374E+01_JPRB, 0.10275E+01_JPRB, 0.73770E+00_JPRB, 0.70613E+00_JPRB ,&
&  0.72690E+00_JPRB, 0.73453E+00_JPRB, 0.75761E+00_JPRB, 0.78680E+00_JPRB, 0.81875E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.83310E+00_JPRB, 0.86045E+00_JPRB, 0.87417E+00_JPRB, 0.87568E+00_JPRB, 0.89313E+00_JPRB, 0.92304E+00_JPRB, 0.93682E+00_JPRB ,&
&  0.95195E+00_JPRB, 0.95585E+00_JPRB, 0.97299E+00_JPRB, 0.98980E+00_JPRB, 0.10070E+01_JPRB, 0.10260E+01_JPRB, 0.10464E+01_JPRB ,&
&  0.10612E+01_JPRB, 0.11009E+01_JPRB, 0.11225E+01_JPRB, 0.11673E+01_JPRB, 0.12150E+01_JPRB, 0.12806E+01_JPRB, 0.13169E+01_JPRB ,&
&  0.13222E+01_JPRB, 0.13571E+01_JPRB, 0.13963E+01_JPRB, 0.14451E+01_JPRB, 0.14396E+01_JPRB, 0.14490E+01_JPRB, 0.11535E+01_JPRB ,&
&  0.12653E+01_JPRB, 0.12366E+01_JPRB, 0.11907E+01_JPRB, 0.11810E+01_JPRB, 0.11679E+01_JPRB, 0.11506E+01_JPRB, 0.11325E+01_JPRB ,&
&  0.11429E+01_JPRB, 0.11483E+01_JPRB, 0.11324E+01_JPRB, 0.12173E+01_JPRB, 0.12813E+01_JPRB, 0.13680E+01_JPRB, 0.14952E+01_JPRB ,&
&  0.16382E+01_JPRB, 0.18467E+01_JPRB, 0.21180E+01_JPRB, 0.24475E+01_JPRB, 0.28093E+01_JPRB, 0.31710E+01_JPRB, 0.35307E+01_JPRB ,&
&  0.38642E+01_JPRB, 0.41285E+01_JPRB, 0.41768E+01_JPRB, 0.34273E+01_JPRB, 0.28637E+01_JPRB, 0.20164E+01_JPRB, 0.15223E+01_JPRB ,&
&  0.12469E+01_JPRB, 0.11265E+01_JPRB, 0.10879E+01_JPRB, 0.10057E+01_JPRB, 0.78147E+00_JPRB, 0.65381E+00_JPRB, 0.53725E+00_JPRB ,&
&  0.49090E+00_JPRB, 0.45475E+00_JPRB, 0.45240E+00_JPRB, 0.44995E+00_JPRB, 0.44834E+00_JPRB, 0.44372E+00_JPRB, 0.45921E+00_JPRB ,&
&  0.47034E+00_JPRB, 0.50075E+00_JPRB, 0.56590E+00_JPRB, 0.64887E+00_JPRB, 0.72416E+00_JPRB, 0.76235E+00_JPRB, 0.69901E+00_JPRB ,&
&  0.69856E+00_JPRB, 0.65740E+00_JPRB, 0.64632E+00_JPRB, 0.69398E+00_JPRB, 0.77703E+00_JPRB, 0.94773E+00_JPRB, 0.11548E+01_JPRB ,&
&  0.14095E+01_JPRB, 0.14610E+01_JPRB, 0.17623E+01_JPRB, 0.23166E+01_JPRB, 0.20912E+01_JPRB, 0.17799E+01_JPRB, 0.66852E+00_JPRB ,&
&  0.70541E+00_JPRB, 0.71603E+00_JPRB, 0.74732E+00_JPRB, 0.77807E+00_JPRB, 0.81643E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.68555E+00_JPRB, 0.71335E+00_JPRB, 0.73399E+00_JPRB, 0.73764E+00_JPRB, 0.76776E+00_JPRB, 0.79187E+00_JPRB, 0.80627E+00_JPRB ,&
&  0.81982E+00_JPRB, 0.84216E+00_JPRB, 0.85605E+00_JPRB, 0.88189E+00_JPRB, 0.90070E+00_JPRB, 0.91935E+00_JPRB, 0.93478E+00_JPRB ,&
&  0.95064E+00_JPRB, 0.98060E+00_JPRB, 0.10144E+01_JPRB, 0.10641E+01_JPRB, 0.11133E+01_JPRB, 0.11756E+01_JPRB, 0.12308E+01_JPRB ,&
&  0.12709E+01_JPRB, 0.13186E+01_JPRB, 0.13748E+01_JPRB, 0.14092E+01_JPRB, 0.13929E+01_JPRB, 0.13677E+01_JPRB, 0.10358E+01_JPRB ,&
&  0.11733E+01_JPRB, 0.11909E+01_JPRB, 0.11855E+01_JPRB, 0.11672E+01_JPRB, 0.11389E+01_JPRB, 0.11023E+01_JPRB, 0.10882E+01_JPRB ,&
&  0.10858E+01_JPRB, 0.10755E+01_JPRB, 0.10666E+01_JPRB, 0.10776E+01_JPRB, 0.12354E+01_JPRB, 0.13214E+01_JPRB, 0.14324E+01_JPRB ,&
&  0.15815E+01_JPRB, 0.17641E+01_JPRB, 0.19871E+01_JPRB, 0.23102E+01_JPRB, 0.26681E+01_JPRB, 0.30226E+01_JPRB, 0.33891E+01_JPRB ,&
&  0.36742E+01_JPRB, 0.38229E+01_JPRB, 0.38671E+01_JPRB, 0.30750E+01_JPRB, 0.25306E+01_JPRB, 0.19756E+01_JPRB, 0.14132E+01_JPRB ,&
&  0.11729E+01_JPRB, 0.10852E+01_JPRB, 0.10579E+01_JPRB, 0.94461E+00_JPRB, 0.75301E+00_JPRB, 0.60855E+00_JPRB, 0.50925E+00_JPRB ,&
&  0.46933E+00_JPRB, 0.43107E+00_JPRB, 0.42731E+00_JPRB, 0.42900E+00_JPRB, 0.43964E+00_JPRB, 0.44715E+00_JPRB, 0.46136E+00_JPRB ,&
&  0.48161E+00_JPRB, 0.51530E+00_JPRB, 0.57431E+00_JPRB, 0.66619E+00_JPRB, 0.79061E+00_JPRB, 0.92696E+00_JPRB, 0.89274E+00_JPRB ,&
&  0.72600E+00_JPRB, 0.63358E+00_JPRB, 0.62117E+00_JPRB, 0.62523E+00_JPRB, 0.65480E+00_JPRB, 0.71488E+00_JPRB, 0.78930E+00_JPRB ,&
&  0.91688E+00_JPRB, 0.99106E+00_JPRB, 0.89872E+00_JPRB, 0.75261E+00_JPRB, 0.10244E+01_JPRB, 0.10088E+01_JPRB, 0.54307E+00_JPRB ,&
&  0.59183E+00_JPRB, 0.58190E+00_JPRB, 0.60613E+00_JPRB, 0.63496E+00_JPRB, 0.67057E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.52590E+00_JPRB, 0.57218E+00_JPRB, 0.59198E+00_JPRB, 0.61679E+00_JPRB, 0.63523E+00_JPRB, 0.66103E+00_JPRB, 0.68394E+00_JPRB ,&
&  0.70565E+00_JPRB, 0.73194E+00_JPRB, 0.74666E+00_JPRB, 0.77291E+00_JPRB, 0.79296E+00_JPRB, 0.80928E+00_JPRB, 0.82355E+00_JPRB ,&
&  0.83815E+00_JPRB, 0.86603E+00_JPRB, 0.89531E+00_JPRB, 0.93045E+00_JPRB, 0.96597E+00_JPRB, 0.10139E+01_JPRB, 0.10818E+01_JPRB ,&
&  0.11641E+01_JPRB, 0.12423E+01_JPRB, 0.13082E+01_JPRB, 0.13601E+01_JPRB, 0.13840E+01_JPRB, 0.14327E+01_JPRB, 0.14191E+01_JPRB ,&
&  0.10302E+01_JPRB, 0.12157E+01_JPRB, 0.11798E+01_JPRB, 0.11321E+01_JPRB, 0.10973E+01_JPRB, 0.10683E+01_JPRB, 0.10675E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.10126E+01_JPRB, 0.10187E+01_JPRB, 0.10438E+01_JPRB, 0.11369E+01_JPRB, 0.12243E+01_JPRB, 0.13401E+01_JPRB ,&
&  0.14923E+01_JPRB, 0.16828E+01_JPRB, 0.18899E+01_JPRB, 0.21140E+01_JPRB, 0.24173E+01_JPRB, 0.27939E+01_JPRB, 0.31360E+01_JPRB ,&
&  0.34951E+01_JPRB, 0.38616E+01_JPRB, 0.40818E+01_JPRB, 0.34285E+01_JPRB, 0.28001E+01_JPRB, 0.24808E+01_JPRB, 0.18002E+01_JPRB ,&
&  0.11986E+01_JPRB, 0.97474E+00_JPRB, 0.97020E+00_JPRB, 0.92307E+00_JPRB, 0.73850E+00_JPRB, 0.61302E+00_JPRB, 0.52702E+00_JPRB ,&
&  0.48968E+00_JPRB, 0.45126E+00_JPRB, 0.43781E+00_JPRB, 0.43970E+00_JPRB, 0.44835E+00_JPRB, 0.45942E+00_JPRB, 0.48585E+00_JPRB ,&
&  0.50428E+00_JPRB, 0.52176E+00_JPRB, 0.54702E+00_JPRB, 0.57445E+00_JPRB, 0.62471E+00_JPRB, 0.70323E+00_JPRB, 0.71121E+00_JPRB ,&
&  0.67884E+00_JPRB, 0.64371E+00_JPRB, 0.61390E+00_JPRB, 0.61121E+00_JPRB, 0.60544E+00_JPRB, 0.60562E+00_JPRB, 0.62231E+00_JPRB ,&
&  0.64823E+00_JPRB, 0.66716E+00_JPRB, 0.63244E+00_JPRB, 0.60734E+00_JPRB, 0.60636E+00_JPRB, 0.57692E+00_JPRB, 0.52289E+00_JPRB ,&
&  0.50250E+00_JPRB, 0.48480E+00_JPRB, 0.46749E+00_JPRB, 0.47466E+00_JPRB, 0.49714E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.44295E+00_JPRB, 0.46167E+00_JPRB, 0.48298E+00_JPRB, 0.50344E+00_JPRB, 0.52214E+00_JPRB, 0.53893E+00_JPRB, 0.56544E+00_JPRB ,&
&  0.59489E+00_JPRB, 0.61518E+00_JPRB, 0.62358E+00_JPRB, 0.64234E+00_JPRB, 0.66391E+00_JPRB, 0.69083E+00_JPRB, 0.71161E+00_JPRB ,&
&  0.73505E+00_JPRB, 0.76826E+00_JPRB, 0.79649E+00_JPRB, 0.82694E+00_JPRB, 0.85758E+00_JPRB, 0.88925E+00_JPRB, 0.93597E+00_JPRB ,&
&  0.99666E+00_JPRB, 0.10765E+01_JPRB, 0.11691E+01_JPRB, 0.12549E+01_JPRB, 0.13290E+01_JPRB, 0.14071E+01_JPRB, 0.14176E+01_JPRB ,&
&  0.90345E+00_JPRB, 0.11376E+01_JPRB, 0.12755E+01_JPRB, 0.11671E+01_JPRB, 0.10794E+01_JPRB, 0.10348E+01_JPRB, 0.10234E+01_JPRB ,&
&  0.99270E+00_JPRB, 0.94483E+00_JPRB, 0.94519E+00_JPRB, 0.10346E+01_JPRB, 0.10352E+01_JPRB, 0.10966E+01_JPRB, 0.11799E+01_JPRB ,&
&  0.13026E+01_JPRB, 0.14550E+01_JPRB, 0.15960E+01_JPRB, 0.17634E+01_JPRB, 0.20051E+01_JPRB, 0.23095E+01_JPRB, 0.26852E+01_JPRB ,&
&  0.31584E+01_JPRB, 0.37040E+01_JPRB, 0.42116E+01_JPRB, 0.33707E+01_JPRB, 0.30498E+01_JPRB, 0.28624E+01_JPRB, 0.24347E+01_JPRB ,&
&  0.14329E+01_JPRB, 0.97810E+00_JPRB, 0.96021E+00_JPRB, 0.91746E+00_JPRB, 0.77103E+00_JPRB, 0.67109E+00_JPRB, 0.59917E+00_JPRB ,&
&  0.55223E+00_JPRB, 0.51527E+00_JPRB, 0.49629E+00_JPRB, 0.48869E+00_JPRB, 0.49017E+00_JPRB, 0.50821E+00_JPRB, 0.52860E+00_JPRB ,&
&  0.54568E+00_JPRB, 0.54781E+00_JPRB, 0.57049E+00_JPRB, 0.58119E+00_JPRB, 0.61612E+00_JPRB, 0.64536E+00_JPRB, 0.66308E+00_JPRB ,&
&  0.66951E+00_JPRB, 0.66114E+00_JPRB, 0.65573E+00_JPRB, 0.65178E+00_JPRB, 0.65094E+00_JPRB, 0.64895E+00_JPRB, 0.64569E+00_JPRB ,&
&  0.64096E+00_JPRB, 0.65380E+00_JPRB, 0.64496E+00_JPRB, 0.63617E+00_JPRB, 0.64054E+00_JPRB, 0.61335E+00_JPRB, 0.59787E+00_JPRB ,&
&  0.57668E+00_JPRB, 0.52874E+00_JPRB, 0.49359E+00_JPRB, 0.46709E+00_JPRB, 0.44870E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.46463E+00_JPRB, 0.44000E+00_JPRB, 0.44236E+00_JPRB, 0.44642E+00_JPRB, 0.46190E+00_JPRB, 0.47704E+00_JPRB, 0.47232E+00_JPRB ,&
&  0.47901E+00_JPRB, 0.48852E+00_JPRB, 0.50182E+00_JPRB, 0.51839E+00_JPRB, 0.53938E+00_JPRB, 0.56161E+00_JPRB, 0.57948E+00_JPRB ,&
&  0.60512E+00_JPRB, 0.63551E+00_JPRB, 0.66984E+00_JPRB, 0.70339E+00_JPRB, 0.73949E+00_JPRB, 0.76789E+00_JPRB, 0.79457E+00_JPRB ,&
&  0.82712E+00_JPRB, 0.88154E+00_JPRB, 0.95801E+00_JPRB, 0.10494E+01_JPRB, 0.11646E+01_JPRB, 0.12994E+01_JPRB, 0.13794E+01_JPRB ,&
&  0.10263E+01_JPRB, 0.60054E+00_JPRB, 0.11635E+01_JPRB, 0.12915E+01_JPRB, 0.11389E+01_JPRB, 0.10710E+01_JPRB, 0.99988E+00_JPRB ,&
&  0.95161E+00_JPRB, 0.95684E+00_JPRB, 0.92663E+00_JPRB, 0.96079E+00_JPRB, 0.95274E+00_JPRB, 0.97832E+00_JPRB, 0.10190E+01_JPRB ,&
&  0.10679E+01_JPRB, 0.11650E+01_JPRB, 0.13084E+01_JPRB, 0.13972E+01_JPRB, 0.15283E+01_JPRB, 0.17577E+01_JPRB, 0.20997E+01_JPRB ,&
&  0.25803E+01_JPRB, 0.31689E+01_JPRB, 0.37634E+01_JPRB, 0.29218E+01_JPRB, 0.22649E+01_JPRB, 0.17909E+01_JPRB, 0.14653E+01_JPRB ,&
&  0.12682E+01_JPRB, 0.10014E+01_JPRB, 0.94617E+00_JPRB, 0.90881E+00_JPRB, 0.81344E+00_JPRB, 0.69904E+00_JPRB, 0.67639E+00_JPRB ,&
&  0.62846E+00_JPRB, 0.59412E+00_JPRB, 0.57490E+00_JPRB, 0.56132E+00_JPRB, 0.55832E+00_JPRB, 0.57045E+00_JPRB, 0.58202E+00_JPRB ,&
&  0.59668E+00_JPRB, 0.60471E+00_JPRB, 0.62190E+00_JPRB, 0.64234E+00_JPRB, 0.64889E+00_JPRB, 0.65172E+00_JPRB, 0.66221E+00_JPRB ,&
&  0.66157E+00_JPRB, 0.67089E+00_JPRB, 0.68164E+00_JPRB, 0.68398E+00_JPRB, 0.68700E+00_JPRB, 0.67926E+00_JPRB, 0.68513E+00_JPRB ,&
&  0.68775E+00_JPRB, 0.69057E+00_JPRB, 0.68395E+00_JPRB, 0.69007E+00_JPRB, 0.69736E+00_JPRB, 0.69606E+00_JPRB, 0.71256E+00_JPRB ,&
&  0.70436E+00_JPRB, 0.67472E+00_JPRB, 0.61198E+00_JPRB, 0.54111E+00_JPRB, 0.50481E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.56113E+00_JPRB, 0.52182E+00_JPRB, 0.48797E+00_JPRB, 0.47003E+00_JPRB, 0.45788E+00_JPRB, 0.45287E+00_JPRB, 0.45131E+00_JPRB ,&
&  0.45487E+00_JPRB, 0.45910E+00_JPRB, 0.44688E+00_JPRB, 0.45165E+00_JPRB, 0.46484E+00_JPRB, 0.48013E+00_JPRB, 0.50001E+00_JPRB ,&
&  0.51252E+00_JPRB, 0.52323E+00_JPRB, 0.53568E+00_JPRB, 0.55738E+00_JPRB, 0.58618E+00_JPRB, 0.61838E+00_JPRB, 0.64915E+00_JPRB ,&
&  0.67971E+00_JPRB, 0.71276E+00_JPRB, 0.75868E+00_JPRB, 0.82145E+00_JPRB, 0.90997E+00_JPRB, 0.10353E+01_JPRB, 0.11698E+01_JPRB ,&
&  0.13358E+01_JPRB, 0.10031E+01_JPRB, 0.73417E+00_JPRB, 0.11648E+01_JPRB, 0.12387E+01_JPRB, 0.11764E+01_JPRB, 0.10231E+01_JPRB ,&
&  0.10152E+01_JPRB, 0.10222E+01_JPRB, 0.96879E+00_JPRB, 0.93958E+00_JPRB, 0.87890E+00_JPRB, 0.88258E+00_JPRB, 0.89506E+00_JPRB ,&
&  0.89531E+00_JPRB, 0.91415E+00_JPRB, 0.10084E+01_JPRB, 0.10773E+01_JPRB, 0.11341E+01_JPRB, 0.12812E+01_JPRB, 0.15848E+01_JPRB ,&
&  0.20019E+01_JPRB, 0.25486E+01_JPRB, 0.31702E+01_JPRB, 0.26685E+01_JPRB, 0.19855E+01_JPRB, 0.14830E+01_JPRB, 0.12944E+01_JPRB ,&
&  0.11851E+01_JPRB, 0.11136E+01_JPRB, 0.10146E+01_JPRB, 0.91905E+00_JPRB, 0.82461E+00_JPRB, 0.73890E+00_JPRB, 0.72216E+00_JPRB ,&
&  0.69483E+00_JPRB, 0.65298E+00_JPRB, 0.62471E+00_JPRB, 0.60054E+00_JPRB, 0.59634E+00_JPRB, 0.60574E+00_JPRB, 0.61519E+00_JPRB ,&
&  0.62420E+00_JPRB, 0.63493E+00_JPRB, 0.65413E+00_JPRB, 0.67792E+00_JPRB, 0.68396E+00_JPRB, 0.67810E+00_JPRB, 0.67566E+00_JPRB ,&
&  0.67189E+00_JPRB, 0.67198E+00_JPRB, 0.68655E+00_JPRB, 0.69263E+00_JPRB, 0.69050E+00_JPRB, 0.67825E+00_JPRB, 0.68976E+00_JPRB ,&
&  0.69143E+00_JPRB, 0.70158E+00_JPRB, 0.69946E+00_JPRB, 0.71064E+00_JPRB, 0.71701E+00_JPRB, 0.74099E+00_JPRB, 0.76680E+00_JPRB ,&
&  0.78870E+00_JPRB, 0.78460E+00_JPRB, 0.75021E+00_JPRB, 0.63935E+00_JPRB, 0.59629E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.60347E+00_JPRB, 0.57760E+00_JPRB, 0.55792E+00_JPRB, 0.53923E+00_JPRB, 0.51969E+00_JPRB, 0.49980E+00_JPRB, 0.48409E+00_JPRB ,&
&  0.46574E+00_JPRB, 0.46301E+00_JPRB, 0.45886E+00_JPRB, 0.45346E+00_JPRB, 0.44701E+00_JPRB, 0.44838E+00_JPRB, 0.45627E+00_JPRB ,&
&  0.46278E+00_JPRB, 0.46631E+00_JPRB, 0.47076E+00_JPRB, 0.47270E+00_JPRB, 0.47704E+00_JPRB, 0.48870E+00_JPRB, 0.50806E+00_JPRB ,&
&  0.53590E+00_JPRB, 0.56474E+00_JPRB, 0.60103E+00_JPRB, 0.64318E+00_JPRB, 0.68509E+00_JPRB, 0.75029E+00_JPRB, 0.83359E+00_JPRB ,&
&  0.10142E+01_JPRB, 0.10236E+01_JPRB, 0.64421E+00_JPRB, 0.10524E+01_JPRB, 0.13125E+01_JPRB, 0.12464E+01_JPRB, 0.11712E+01_JPRB ,&
&  0.11600E+01_JPRB, 0.10934E+01_JPRB, 0.10358E+01_JPRB, 0.93719E+00_JPRB, 0.83705E+00_JPRB, 0.81648E+00_JPRB, 0.81814E+00_JPRB ,&
&  0.79318E+00_JPRB, 0.77682E+00_JPRB, 0.79685E+00_JPRB, 0.82176E+00_JPRB, 0.84834E+00_JPRB, 0.91207E+00_JPRB, 0.11210E+01_JPRB ,&
&  0.14625E+01_JPRB, 0.19798E+01_JPRB, 0.25660E+01_JPRB, 0.24353E+01_JPRB, 0.18251E+01_JPRB, 0.15408E+01_JPRB, 0.14073E+01_JPRB ,&
&  0.13633E+01_JPRB, 0.12707E+01_JPRB, 0.10594E+01_JPRB, 0.92532E+00_JPRB, 0.82605E+00_JPRB, 0.75166E+00_JPRB, 0.74936E+00_JPRB ,&
&  0.72818E+00_JPRB, 0.69073E+00_JPRB, 0.66867E+00_JPRB, 0.63804E+00_JPRB, 0.62320E+00_JPRB, 0.61831E+00_JPRB, 0.62455E+00_JPRB ,&
&  0.63317E+00_JPRB, 0.64094E+00_JPRB, 0.65312E+00_JPRB, 0.66944E+00_JPRB, 0.68020E+00_JPRB, 0.68423E+00_JPRB, 0.68099E+00_JPRB ,&
&  0.67895E+00_JPRB, 0.68225E+00_JPRB, 0.69067E+00_JPRB, 0.69408E+00_JPRB, 0.69115E+00_JPRB, 0.68620E+00_JPRB, 0.67802E+00_JPRB ,&
&  0.67441E+00_JPRB, 0.67475E+00_JPRB, 0.66983E+00_JPRB, 0.66418E+00_JPRB, 0.69482E+00_JPRB, 0.70555E+00_JPRB, 0.71361E+00_JPRB ,&
&  0.75429E+00_JPRB, 0.78796E+00_JPRB, 0.78756E+00_JPRB, 0.67711E+00_JPRB, 0.63466E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.61380E+00_JPRB, 0.59312E+00_JPRB, 0.57696E+00_JPRB, 0.56453E+00_JPRB, 0.54895E+00_JPRB, 0.53332E+00_JPRB, 0.52052E+00_JPRB ,&
&  0.50237E+00_JPRB, 0.49122E+00_JPRB, 0.48335E+00_JPRB, 0.47507E+00_JPRB, 0.46704E+00_JPRB, 0.45949E+00_JPRB, 0.45325E+00_JPRB ,&
&  0.44790E+00_JPRB, 0.44373E+00_JPRB, 0.44116E+00_JPRB, 0.43576E+00_JPRB, 0.43059E+00_JPRB, 0.42891E+00_JPRB, 0.43265E+00_JPRB ,&
&  0.44932E+00_JPRB, 0.47316E+00_JPRB, 0.50057E+00_JPRB, 0.52799E+00_JPRB, 0.55097E+00_JPRB, 0.58216E+00_JPRB, 0.62099E+00_JPRB ,&
&  0.77322E+00_JPRB, 0.84823E+00_JPRB, 0.10114E+01_JPRB, 0.15694E+01_JPRB, 0.15976E+01_JPRB, 0.14286E+01_JPRB, 0.13995E+01_JPRB ,&
&  0.13810E+01_JPRB, 0.12797E+01_JPRB, 0.12144E+01_JPRB, 0.10193E+01_JPRB, 0.86617E+00_JPRB, 0.79139E+00_JPRB, 0.75535E+00_JPRB ,&
&  0.71654E+00_JPRB, 0.68301E+00_JPRB, 0.68200E+00_JPRB, 0.67733E+00_JPRB, 0.68719E+00_JPRB, 0.76587E+00_JPRB, 0.87393E+00_JPRB ,&
&  0.10948E+01_JPRB, 0.14738E+01_JPRB, 0.20056E+01_JPRB, 0.23552E+01_JPRB, 0.18318E+01_JPRB, 0.16934E+01_JPRB, 0.17209E+01_JPRB ,&
&  0.20731E+01_JPRB, 0.16212E+01_JPRB, 0.11932E+01_JPRB, 0.95792E+00_JPRB, 0.82601E+00_JPRB, 0.77856E+00_JPRB, 0.73807E+00_JPRB ,&
&  0.71796E+00_JPRB, 0.69672E+00_JPRB, 0.68049E+00_JPRB, 0.66063E+00_JPRB, 0.64736E+00_JPRB, 0.63571E+00_JPRB, 0.63106E+00_JPRB ,&
&  0.63229E+00_JPRB, 0.63565E+00_JPRB, 0.63961E+00_JPRB, 0.64811E+00_JPRB, 0.66290E+00_JPRB, 0.67411E+00_JPRB, 0.67733E+00_JPRB ,&
&  0.67816E+00_JPRB, 0.68381E+00_JPRB, 0.66894E+00_JPRB, 0.66965E+00_JPRB, 0.67640E+00_JPRB, 0.67270E+00_JPRB, 0.64906E+00_JPRB ,&
&  0.63920E+00_JPRB, 0.65074E+00_JPRB, 0.65335E+00_JPRB, 0.64840E+00_JPRB, 0.65852E+00_JPRB, 0.66925E+00_JPRB, 0.64999E+00_JPRB ,&
&  0.65456E+00_JPRB, 0.66258E+00_JPRB, 0.66960E+00_JPRB, 0.65387E+00_JPRB, 0.63589E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.61954E+00_JPRB, 0.60293E+00_JPRB, 0.58569E+00_JPRB, 0.56848E+00_JPRB, 0.54960E+00_JPRB, 0.53522E+00_JPRB, 0.52745E+00_JPRB ,&
&  0.51494E+00_JPRB, 0.50401E+00_JPRB, 0.49601E+00_JPRB, 0.49238E+00_JPRB, 0.48958E+00_JPRB, 0.48095E+00_JPRB, 0.46457E+00_JPRB ,&
&  0.44790E+00_JPRB, 0.43691E+00_JPRB, 0.43104E+00_JPRB, 0.42490E+00_JPRB, 0.41792E+00_JPRB, 0.41381E+00_JPRB, 0.41464E+00_JPRB ,&
&  0.42114E+00_JPRB, 0.43188E+00_JPRB, 0.45317E+00_JPRB, 0.47647E+00_JPRB, 0.49424E+00_JPRB, 0.50937E+00_JPRB, 0.52018E+00_JPRB ,&
&  0.60971E+00_JPRB, 0.75124E+00_JPRB, 0.82044E+00_JPRB, 0.13571E+01_JPRB, 0.14370E+01_JPRB, 0.13988E+01_JPRB, 0.13910E+01_JPRB ,&
&  0.13766E+01_JPRB, 0.14623E+01_JPRB, 0.12875E+01_JPRB, 0.11034E+01_JPRB, 0.93897E+00_JPRB, 0.82692E+00_JPRB, 0.74884E+00_JPRB ,&
&  0.69743E+00_JPRB, 0.65346E+00_JPRB, 0.62985E+00_JPRB, 0.61864E+00_JPRB, 0.61668E+00_JPRB, 0.65616E+00_JPRB, 0.76035E+00_JPRB ,&
&  0.91685E+00_JPRB, 0.11856E+01_JPRB, 0.15720E+01_JPRB, 0.18421E+01_JPRB, 0.17683E+01_JPRB, 0.17960E+01_JPRB, 0.20142E+01_JPRB ,&
&  0.25682E+01_JPRB, 0.22085E+01_JPRB, 0.14441E+01_JPRB, 0.10372E+01_JPRB, 0.85756E+00_JPRB, 0.78304E+00_JPRB, 0.73028E+00_JPRB ,&
&  0.70622E+00_JPRB, 0.68643E+00_JPRB, 0.67126E+00_JPRB, 0.65918E+00_JPRB, 0.64994E+00_JPRB, 0.64730E+00_JPRB, 0.64362E+00_JPRB ,&
&  0.63944E+00_JPRB, 0.63695E+00_JPRB, 0.63488E+00_JPRB, 0.63425E+00_JPRB, 0.63952E+00_JPRB, 0.65030E+00_JPRB, 0.66168E+00_JPRB ,&
&  0.66890E+00_JPRB, 0.67591E+00_JPRB, 0.66153E+00_JPRB, 0.64926E+00_JPRB, 0.65151E+00_JPRB, 0.64702E+00_JPRB, 0.62929E+00_JPRB ,&
&  0.63237E+00_JPRB, 0.64288E+00_JPRB, 0.64199E+00_JPRB, 0.62758E+00_JPRB, 0.63486E+00_JPRB, 0.65540E+00_JPRB, 0.63251E+00_JPRB ,&
&  0.62914E+00_JPRB, 0.63368E+00_JPRB, 0.64121E+00_JPRB, 0.63579E+00_JPRB, 0.62839E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.60894E+00_JPRB, 0.59994E+00_JPRB, 0.58465E+00_JPRB, 0.56689E+00_JPRB, 0.54892E+00_JPRB, 0.53366E+00_JPRB, 0.52584E+00_JPRB ,&
&  0.51665E+00_JPRB, 0.50724E+00_JPRB, 0.49880E+00_JPRB, 0.49423E+00_JPRB, 0.49240E+00_JPRB, 0.48421E+00_JPRB, 0.46905E+00_JPRB ,&
&  0.45300E+00_JPRB, 0.43834E+00_JPRB, 0.42655E+00_JPRB, 0.42009E+00_JPRB, 0.41301E+00_JPRB, 0.40948E+00_JPRB, 0.40905E+00_JPRB ,&
&  0.41016E+00_JPRB, 0.41630E+00_JPRB, 0.42983E+00_JPRB, 0.45187E+00_JPRB, 0.46989E+00_JPRB, 0.47903E+00_JPRB, 0.48575E+00_JPRB ,&
&  0.53865E+00_JPRB, 0.81483E+00_JPRB, 0.75668E+00_JPRB, 0.10631E+01_JPRB, 0.11363E+01_JPRB, 0.11874E+01_JPRB, 0.12540E+01_JPRB ,&
&  0.12844E+01_JPRB, 0.13206E+01_JPRB, 0.12494E+01_JPRB, 0.11281E+01_JPRB, 0.98527E+00_JPRB, 0.88082E+00_JPRB, 0.79558E+00_JPRB ,&
&  0.73477E+00_JPRB, 0.69027E+00_JPRB, 0.65515E+00_JPRB, 0.62623E+00_JPRB, 0.60751E+00_JPRB, 0.61320E+00_JPRB, 0.69273E+00_JPRB ,&
&  0.87333E+00_JPRB, 0.10982E+01_JPRB, 0.13510E+01_JPRB, 0.16280E+01_JPRB, 0.15678E+01_JPRB, 0.17761E+01_JPRB, 0.23855E+01_JPRB ,&
&  0.28296E+01_JPRB, 0.28150E+01_JPRB, 0.18585E+01_JPRB, 0.12134E+01_JPRB, 0.93679E+00_JPRB, 0.80503E+00_JPRB, 0.72773E+00_JPRB ,&
&  0.68423E+00_JPRB, 0.66131E+00_JPRB, 0.64884E+00_JPRB, 0.64314E+00_JPRB, 0.64294E+00_JPRB, 0.64851E+00_JPRB, 0.64866E+00_JPRB ,&
&  0.64863E+00_JPRB, 0.64733E+00_JPRB, 0.64576E+00_JPRB, 0.64088E+00_JPRB, 0.63368E+00_JPRB, 0.63321E+00_JPRB, 0.63949E+00_JPRB ,&
&  0.64980E+00_JPRB, 0.65893E+00_JPRB, 0.65127E+00_JPRB, 0.64214E+00_JPRB, 0.63882E+00_JPRB, 0.63397E+00_JPRB, 0.62364E+00_JPRB ,&
&  0.63048E+00_JPRB, 0.63411E+00_JPRB, 0.63216E+00_JPRB, 0.63144E+00_JPRB, 0.62643E+00_JPRB, 0.64895E+00_JPRB, 0.62323E+00_JPRB ,&
&  0.61711E+00_JPRB, 0.62066E+00_JPRB, 0.62385E+00_JPRB, 0.61665E+00_JPRB, 0.61238E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.58834E+00_JPRB, 0.58044E+00_JPRB, 0.56581E+00_JPRB, 0.54968E+00_JPRB, 0.53815E+00_JPRB, 0.52578E+00_JPRB, 0.51626E+00_JPRB ,&
&  0.50877E+00_JPRB, 0.50241E+00_JPRB, 0.49763E+00_JPRB, 0.49199E+00_JPRB, 0.48433E+00_JPRB, 0.47364E+00_JPRB, 0.46298E+00_JPRB ,&
&  0.45180E+00_JPRB, 0.44098E+00_JPRB, 0.43020E+00_JPRB, 0.41907E+00_JPRB, 0.41041E+00_JPRB, 0.40448E+00_JPRB, 0.40038E+00_JPRB ,&
&  0.39976E+00_JPRB, 0.40254E+00_JPRB, 0.41081E+00_JPRB, 0.42833E+00_JPRB, 0.44876E+00_JPRB, 0.46188E+00_JPRB, 0.47561E+00_JPRB ,&
&  0.53714E+00_JPRB, 0.48160E+00_JPRB, 0.82996E+00_JPRB, 0.92173E+00_JPRB, 0.89748E+00_JPRB, 0.91282E+00_JPRB, 0.10532E+01_JPRB ,&
&  0.11531E+01_JPRB, 0.11856E+01_JPRB, 0.11839E+01_JPRB, 0.11264E+01_JPRB, 0.10162E+01_JPRB, 0.93301E+00_JPRB, 0.86857E+00_JPRB ,&
&  0.82058E+00_JPRB, 0.78126E+00_JPRB, 0.73366E+00_JPRB, 0.68516E+00_JPRB, 0.64862E+00_JPRB, 0.64157E+00_JPRB, 0.68484E+00_JPRB ,&
&  0.87958E+00_JPRB, 0.10609E+01_JPRB, 0.12050E+01_JPRB, 0.13831E+01_JPRB, 0.14363E+01_JPRB, 0.17000E+01_JPRB, 0.20329E+01_JPRB ,&
&  0.27688E+01_JPRB, 0.28619E+01_JPRB, 0.21171E+01_JPRB, 0.14988E+01_JPRB, 0.11180E+01_JPRB, 0.91643E+00_JPRB, 0.77969E+00_JPRB ,&
&  0.70367E+00_JPRB, 0.65970E+00_JPRB, 0.63278E+00_JPRB, 0.62539E+00_JPRB, 0.63424E+00_JPRB, 0.65111E+00_JPRB, 0.66548E+00_JPRB ,&
&  0.68325E+00_JPRB, 0.69648E+00_JPRB, 0.69799E+00_JPRB, 0.69132E+00_JPRB, 0.66857E+00_JPRB, 0.64104E+00_JPRB, 0.62530E+00_JPRB ,&
&  0.62458E+00_JPRB, 0.63228E+00_JPRB, 0.63201E+00_JPRB, 0.62673E+00_JPRB, 0.62607E+00_JPRB, 0.62465E+00_JPRB, 0.62194E+00_JPRB ,&
&  0.62376E+00_JPRB, 0.62773E+00_JPRB, 0.62704E+00_JPRB, 0.63232E+00_JPRB, 0.61631E+00_JPRB, 0.64124E+00_JPRB, 0.62351E+00_JPRB ,&
&  0.61375E+00_JPRB, 0.60961E+00_JPRB, 0.60662E+00_JPRB, 0.59789E+00_JPRB, 0.59008E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.56635E+00_JPRB, 0.55785E+00_JPRB, 0.54287E+00_JPRB, 0.52710E+00_JPRB, 0.51703E+00_JPRB, 0.50941E+00_JPRB, 0.50399E+00_JPRB ,&
&  0.50077E+00_JPRB, 0.49360E+00_JPRB, 0.48674E+00_JPRB, 0.47704E+00_JPRB, 0.46460E+00_JPRB, 0.45399E+00_JPRB, 0.44742E+00_JPRB ,&
&  0.43921E+00_JPRB, 0.43502E+00_JPRB, 0.42992E+00_JPRB, 0.41978E+00_JPRB, 0.40864E+00_JPRB, 0.39937E+00_JPRB, 0.39334E+00_JPRB ,&
&  0.39053E+00_JPRB, 0.39278E+00_JPRB, 0.39862E+00_JPRB, 0.41201E+00_JPRB, 0.42471E+00_JPRB, 0.43812E+00_JPRB, 0.46017E+00_JPRB ,&
&  0.53731E+00_JPRB, 0.76558E+00_JPRB, 0.92873E+00_JPRB, 0.83376E+00_JPRB, 0.77514E+00_JPRB, 0.73595E+00_JPRB, 0.82846E+00_JPRB ,&
&  0.91311E+00_JPRB, 0.10002E+01_JPRB, 0.10487E+01_JPRB, 0.10621E+01_JPRB, 0.10057E+01_JPRB, 0.96289E+00_JPRB, 0.91939E+00_JPRB ,&
&  0.87376E+00_JPRB, 0.82369E+00_JPRB, 0.75863E+00_JPRB, 0.69407E+00_JPRB, 0.65900E+00_JPRB, 0.66305E+00_JPRB, 0.71998E+00_JPRB ,&
&  0.86341E+00_JPRB, 0.10101E+01_JPRB, 0.10855E+01_JPRB, 0.12344E+01_JPRB, 0.14701E+01_JPRB, 0.15052E+01_JPRB, 0.19914E+01_JPRB ,&
&  0.25835E+01_JPRB, 0.23304E+01_JPRB, 0.19470E+01_JPRB, 0.15520E+01_JPRB, 0.12429E+01_JPRB, 0.99218E+00_JPRB, 0.84819E+00_JPRB ,&
&  0.75078E+00_JPRB, 0.68711E+00_JPRB, 0.64217E+00_JPRB, 0.61939E+00_JPRB, 0.63012E+00_JPRB, 0.66497E+00_JPRB, 0.69475E+00_JPRB ,&
&  0.73608E+00_JPRB, 0.78099E+00_JPRB, 0.80725E+00_JPRB, 0.80586E+00_JPRB, 0.77396E+00_JPRB, 0.72245E+00_JPRB, 0.67286E+00_JPRB ,&
&  0.64131E+00_JPRB, 0.62362E+00_JPRB, 0.61912E+00_JPRB, 0.61243E+00_JPRB, 0.61696E+00_JPRB, 0.62775E+00_JPRB, 0.62614E+00_JPRB ,&
&  0.62211E+00_JPRB, 0.62446E+00_JPRB, 0.63660E+00_JPRB, 0.62922E+00_JPRB, 0.61443E+00_JPRB, 0.64349E+00_JPRB, 0.62689E+00_JPRB ,&
&  0.61198E+00_JPRB, 0.59867E+00_JPRB, 0.59027E+00_JPRB, 0.57358E+00_JPRB, 0.56734E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.55693E+00_JPRB, 0.54426E+00_JPRB, 0.52248E+00_JPRB, 0.50311E+00_JPRB, 0.49152E+00_JPRB, 0.48386E+00_JPRB, 0.48055E+00_JPRB ,&
&  0.47924E+00_JPRB, 0.47236E+00_JPRB, 0.46441E+00_JPRB, 0.45244E+00_JPRB, 0.44185E+00_JPRB, 0.43249E+00_JPRB, 0.42424E+00_JPRB ,&
&  0.41748E+00_JPRB, 0.41377E+00_JPRB, 0.41030E+00_JPRB, 0.40638E+00_JPRB, 0.40026E+00_JPRB, 0.39134E+00_JPRB, 0.38559E+00_JPRB ,&
&  0.38263E+00_JPRB, 0.38290E+00_JPRB, 0.38676E+00_JPRB, 0.39555E+00_JPRB, 0.40546E+00_JPRB, 0.41776E+00_JPRB, 0.43710E+00_JPRB ,&
&  0.48905E+00_JPRB, 0.65316E+00_JPRB, 0.68747E+00_JPRB, 0.68087E+00_JPRB, 0.67777E+00_JPRB, 0.67004E+00_JPRB, 0.70537E+00_JPRB ,&
&  0.77162E+00_JPRB, 0.83222E+00_JPRB, 0.88138E+00_JPRB, 0.89652E+00_JPRB, 0.86713E+00_JPRB, 0.83656E+00_JPRB, 0.79798E+00_JPRB ,&
&  0.75825E+00_JPRB, 0.71586E+00_JPRB, 0.67467E+00_JPRB, 0.63798E+00_JPRB, 0.61330E+00_JPRB, 0.62782E+00_JPRB, 0.67297E+00_JPRB ,&
&  0.75969E+00_JPRB, 0.83663E+00_JPRB, 0.89528E+00_JPRB, 0.10155E+01_JPRB, 0.12655E+01_JPRB, 0.14247E+01_JPRB, 0.16375E+01_JPRB ,&
&  0.16908E+01_JPRB, 0.16414E+01_JPRB, 0.15496E+01_JPRB, 0.14184E+01_JPRB, 0.12292E+01_JPRB, 0.10406E+01_JPRB, 0.88767E+00_JPRB ,&
&  0.77596E+00_JPRB, 0.70167E+00_JPRB, 0.65560E+00_JPRB, 0.63541E+00_JPRB, 0.64416E+00_JPRB, 0.67827E+00_JPRB, 0.72341E+00_JPRB ,&
&  0.78004E+00_JPRB, 0.83879E+00_JPRB, 0.87781E+00_JPRB, 0.89458E+00_JPRB, 0.88218E+00_JPRB, 0.84725E+00_JPRB, 0.79193E+00_JPRB ,&
&  0.74023E+00_JPRB, 0.69558E+00_JPRB, 0.66426E+00_JPRB, 0.65197E+00_JPRB, 0.65101E+00_JPRB, 0.65178E+00_JPRB, 0.65146E+00_JPRB ,&
&  0.65578E+00_JPRB, 0.65663E+00_JPRB, 0.65312E+00_JPRB, 0.63663E+00_JPRB, 0.61555E+00_JPRB, 0.63233E+00_JPRB, 0.61680E+00_JPRB ,&
&  0.60136E+00_JPRB, 0.58383E+00_JPRB, 0.56994E+00_JPRB, 0.55064E+00_JPRB, 0.55770E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.58959E+00_JPRB, 0.56147E+00_JPRB, 0.52469E+00_JPRB, 0.49647E+00_JPRB, 0.47705E+00_JPRB, 0.46307E+00_JPRB, 0.45561E+00_JPRB ,&
&  0.45337E+00_JPRB, 0.44421E+00_JPRB, 0.43014E+00_JPRB, 0.41814E+00_JPRB, 0.41196E+00_JPRB, 0.40711E+00_JPRB, 0.40075E+00_JPRB ,&
&  0.39567E+00_JPRB, 0.39110E+00_JPRB, 0.38784E+00_JPRB, 0.38655E+00_JPRB, 0.38583E+00_JPRB, 0.38428E+00_JPRB, 0.37993E+00_JPRB ,&
&  0.37356E+00_JPRB, 0.37172E+00_JPRB, 0.37282E+00_JPRB, 0.37938E+00_JPRB, 0.38871E+00_JPRB, 0.40013E+00_JPRB, 0.41976E+00_JPRB ,&
&  0.48007E+00_JPRB, 0.48828E+00_JPRB, 0.52727E+00_JPRB, 0.53732E+00_JPRB, 0.54927E+00_JPRB, 0.58484E+00_JPRB, 0.61889E+00_JPRB ,&
&  0.65334E+00_JPRB, 0.69842E+00_JPRB, 0.74935E+00_JPRB, 0.76841E+00_JPRB, 0.74572E+00_JPRB, 0.70705E+00_JPRB, 0.67264E+00_JPRB ,&
&  0.64105E+00_JPRB, 0.61917E+00_JPRB, 0.59933E+00_JPRB, 0.58359E+00_JPRB, 0.58260E+00_JPRB, 0.59956E+00_JPRB, 0.62707E+00_JPRB ,&
&  0.64823E+00_JPRB, 0.65628E+00_JPRB, 0.72878E+00_JPRB, 0.82466E+00_JPRB, 0.96339E+00_JPRB, 0.10655E+01_JPRB, 0.11675E+01_JPRB ,&
&  0.12540E+01_JPRB, 0.13033E+01_JPRB, 0.13293E+01_JPRB, 0.13089E+01_JPRB, 0.12212E+01_JPRB, 0.11085E+01_JPRB, 0.98515E+00_JPRB ,&
&  0.86783E+00_JPRB, 0.78643E+00_JPRB, 0.74241E+00_JPRB, 0.72128E+00_JPRB, 0.71822E+00_JPRB, 0.72633E+00_JPRB, 0.74723E+00_JPRB ,&
&  0.77981E+00_JPRB, 0.81149E+00_JPRB, 0.83741E+00_JPRB, 0.84938E+00_JPRB, 0.84597E+00_JPRB, 0.84023E+00_JPRB, 0.83050E+00_JPRB ,&
&  0.81703E+00_JPRB, 0.79679E+00_JPRB, 0.76878E+00_JPRB, 0.73854E+00_JPRB, 0.71193E+00_JPRB, 0.69346E+00_JPRB, 0.67911E+00_JPRB ,&
&  0.66503E+00_JPRB, 0.65601E+00_JPRB, 0.63157E+00_JPRB, 0.61642E+00_JPRB, 0.63080E+00_JPRB, 0.61641E+00_JPRB, 0.59872E+00_JPRB ,&
&  0.57687E+00_JPRB, 0.55809E+00_JPRB, 0.54063E+00_JPRB, 0.52146E+00_JPRB, 0.57421E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.50005E+00_JPRB, 0.49852E+00_JPRB, 0.48505E+00_JPRB, 0.46632E+00_JPRB, 0.45243E+00_JPRB, 0.43970E+00_JPRB, 0.42900E+00_JPRB ,&
&  0.42198E+00_JPRB, 0.41334E+00_JPRB, 0.40256E+00_JPRB, 0.39430E+00_JPRB, 0.39292E+00_JPRB, 0.39035E+00_JPRB, 0.38630E+00_JPRB ,&
&  0.37903E+00_JPRB, 0.37205E+00_JPRB, 0.36729E+00_JPRB, 0.36573E+00_JPRB, 0.36645E+00_JPRB, 0.36840E+00_JPRB, 0.36918E+00_JPRB ,&
&  0.36682E+00_JPRB, 0.36240E+00_JPRB, 0.36190E+00_JPRB, 0.36401E+00_JPRB, 0.36923E+00_JPRB, 0.37776E+00_JPRB, 0.39732E+00_JPRB ,&
&  0.44556E+00_JPRB, 0.42740E+00_JPRB, 0.44765E+00_JPRB, 0.46050E+00_JPRB, 0.47718E+00_JPRB, 0.51231E+00_JPRB, 0.54908E+00_JPRB ,&
&  0.57223E+00_JPRB, 0.58955E+00_JPRB, 0.62261E+00_JPRB, 0.64964E+00_JPRB, 0.64660E+00_JPRB, 0.62447E+00_JPRB, 0.59508E+00_JPRB ,&
&  0.56577E+00_JPRB, 0.54405E+00_JPRB, 0.52653E+00_JPRB, 0.51490E+00_JPRB, 0.51546E+00_JPRB, 0.52572E+00_JPRB, 0.53395E+00_JPRB ,&
&  0.52564E+00_JPRB, 0.53279E+00_JPRB, 0.58006E+00_JPRB, 0.65105E+00_JPRB, 0.74460E+00_JPRB, 0.83915E+00_JPRB, 0.93522E+00_JPRB ,&
&  0.10200E+01_JPRB, 0.10769E+01_JPRB, 0.11208E+01_JPRB, 0.11526E+01_JPRB, 0.11453E+01_JPRB, 0.11236E+01_JPRB, 0.10543E+01_JPRB ,&
&  0.96826E+00_JPRB, 0.89483E+00_JPRB, 0.84482E+00_JPRB, 0.80513E+00_JPRB, 0.77536E+00_JPRB, 0.75563E+00_JPRB, 0.74530E+00_JPRB ,&
&  0.74436E+00_JPRB, 0.74806E+00_JPRB, 0.74773E+00_JPRB, 0.74307E+00_JPRB, 0.73749E+00_JPRB, 0.73424E+00_JPRB, 0.73117E+00_JPRB ,&
&  0.72903E+00_JPRB, 0.72690E+00_JPRB, 0.71870E+00_JPRB, 0.70170E+00_JPRB, 0.67757E+00_JPRB, 0.65372E+00_JPRB, 0.63107E+00_JPRB ,&
&  0.61300E+00_JPRB, 0.60560E+00_JPRB, 0.59542E+00_JPRB, 0.59272E+00_JPRB, 0.57796E+00_JPRB, 0.56429E+00_JPRB, 0.55095E+00_JPRB ,&
&  0.53625E+00_JPRB, 0.52104E+00_JPRB, 0.50136E+00_JPRB, 0.48201E+00_JPRB, 0.47946E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.44532E+00_JPRB, 0.45176E+00_JPRB, 0.45166E+00_JPRB, 0.44020E+00_JPRB, 0.42721E+00_JPRB, 0.41956E+00_JPRB, 0.41036E+00_JPRB ,&
&  0.40006E+00_JPRB, 0.39162E+00_JPRB, 0.38117E+00_JPRB, 0.37674E+00_JPRB, 0.37440E+00_JPRB, 0.37347E+00_JPRB, 0.37153E+00_JPRB ,&
&  0.36669E+00_JPRB, 0.35916E+00_JPRB, 0.35338E+00_JPRB, 0.35093E+00_JPRB, 0.35162E+00_JPRB, 0.35333E+00_JPRB, 0.35504E+00_JPRB ,&
&  0.35391E+00_JPRB, 0.35292E+00_JPRB, 0.35162E+00_JPRB, 0.35105E+00_JPRB, 0.35234E+00_JPRB, 0.36004E+00_JPRB, 0.37556E+00_JPRB ,&
&  0.41619E+00_JPRB, 0.38195E+00_JPRB, 0.39316E+00_JPRB, 0.41620E+00_JPRB, 0.43948E+00_JPRB, 0.47174E+00_JPRB, 0.50348E+00_JPRB ,&
&  0.51949E+00_JPRB, 0.51911E+00_JPRB, 0.52846E+00_JPRB, 0.54753E+00_JPRB, 0.54711E+00_JPRB, 0.54027E+00_JPRB, 0.52681E+00_JPRB ,&
&  0.50659E+00_JPRB, 0.48320E+00_JPRB, 0.46059E+00_JPRB, 0.44734E+00_JPRB, 0.44807E+00_JPRB, 0.45511E+00_JPRB, 0.45699E+00_JPRB ,&
&  0.44296E+00_JPRB, 0.43825E+00_JPRB, 0.45870E+00_JPRB, 0.49080E+00_JPRB, 0.53330E+00_JPRB, 0.59930E+00_JPRB, 0.68233E+00_JPRB ,&
&  0.75152E+00_JPRB, 0.82144E+00_JPRB, 0.89281E+00_JPRB, 0.95016E+00_JPRB, 0.99974E+00_JPRB, 0.10162E+01_JPRB, 0.98871E+00_JPRB ,&
&  0.94390E+00_JPRB, 0.88911E+00_JPRB, 0.83677E+00_JPRB, 0.78776E+00_JPRB, 0.74664E+00_JPRB, 0.72000E+00_JPRB, 0.70262E+00_JPRB ,&
&  0.69020E+00_JPRB, 0.68009E+00_JPRB, 0.66513E+00_JPRB, 0.64480E+00_JPRB, 0.62402E+00_JPRB, 0.60661E+00_JPRB, 0.59150E+00_JPRB ,&
&  0.57835E+00_JPRB, 0.56494E+00_JPRB, 0.55459E+00_JPRB, 0.54764E+00_JPRB, 0.54052E+00_JPRB, 0.53466E+00_JPRB, 0.52995E+00_JPRB ,&
&  0.52479E+00_JPRB, 0.52442E+00_JPRB, 0.51989E+00_JPRB, 0.51251E+00_JPRB, 0.51747E+00_JPRB, 0.51054E+00_JPRB, 0.50479E+00_JPRB ,&
&  0.49382E+00_JPRB, 0.48118E+00_JPRB, 0.46742E+00_JPRB, 0.42474E+00_JPRB, 0.44005E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.40828E+00_JPRB, 0.41378E+00_JPRB, 0.41427E+00_JPRB, 0.40929E+00_JPRB, 0.40094E+00_JPRB, 0.39511E+00_JPRB, 0.38606E+00_JPRB ,&
&  0.37607E+00_JPRB, 0.37053E+00_JPRB, 0.36406E+00_JPRB, 0.36133E+00_JPRB, 0.36154E+00_JPRB, 0.35798E+00_JPRB, 0.35484E+00_JPRB ,&
&  0.35201E+00_JPRB, 0.34692E+00_JPRB, 0.34362E+00_JPRB, 0.34071E+00_JPRB, 0.34044E+00_JPRB, 0.34066E+00_JPRB, 0.34060E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33622E+00_JPRB, 0.33385E+00_JPRB, 0.33237E+00_JPRB, 0.33437E+00_JPRB, 0.34115E+00_JPRB, 0.35433E+00_JPRB ,&
&  0.38411E+00_JPRB, 0.34675E+00_JPRB, 0.36067E+00_JPRB, 0.37764E+00_JPRB, 0.40467E+00_JPRB, 0.42463E+00_JPRB, 0.43368E+00_JPRB ,&
&  0.44409E+00_JPRB, 0.45520E+00_JPRB, 0.46286E+00_JPRB, 0.46957E+00_JPRB, 0.46579E+00_JPRB, 0.45800E+00_JPRB, 0.45158E+00_JPRB ,&
&  0.44266E+00_JPRB, 0.43155E+00_JPRB, 0.42045E+00_JPRB, 0.40981E+00_JPRB, 0.40650E+00_JPRB, 0.40913E+00_JPRB, 0.40663E+00_JPRB ,&
&  0.39641E+00_JPRB, 0.38770E+00_JPRB, 0.38386E+00_JPRB, 0.38822E+00_JPRB, 0.40170E+00_JPRB, 0.42525E+00_JPRB, 0.47169E+00_JPRB ,&
&  0.52606E+00_JPRB, 0.58819E+00_JPRB, 0.65717E+00_JPRB, 0.71684E+00_JPRB, 0.76410E+00_JPRB, 0.79022E+00_JPRB, 0.79958E+00_JPRB ,&
&  0.78989E+00_JPRB, 0.77129E+00_JPRB, 0.75094E+00_JPRB, 0.72430E+00_JPRB, 0.68925E+00_JPRB, 0.65583E+00_JPRB, 0.62853E+00_JPRB ,&
&  0.60715E+00_JPRB, 0.58533E+00_JPRB, 0.56175E+00_JPRB, 0.53623E+00_JPRB, 0.50947E+00_JPRB, 0.48507E+00_JPRB, 0.46489E+00_JPRB ,&
&  0.44807E+00_JPRB, 0.43372E+00_JPRB, 0.42484E+00_JPRB, 0.42160E+00_JPRB, 0.42469E+00_JPRB, 0.42995E+00_JPRB, 0.43383E+00_JPRB ,&
&  0.43806E+00_JPRB, 0.44442E+00_JPRB, 0.44890E+00_JPRB, 0.45709E+00_JPRB, 0.45720E+00_JPRB, 0.45524E+00_JPRB, 0.45371E+00_JPRB ,&
&  0.44848E+00_JPRB, 0.43846E+00_JPRB, 0.38510E+00_JPRB, 0.40273E+00_JPRB, 0.40464E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.38171E+00_JPRB, 0.37872E+00_JPRB, 0.37570E+00_JPRB, 0.37747E+00_JPRB, 0.37641E+00_JPRB, 0.37285E+00_JPRB, 0.36666E+00_JPRB ,&
&  0.35681E+00_JPRB, 0.35010E+00_JPRB, 0.34515E+00_JPRB, 0.34388E+00_JPRB, 0.34665E+00_JPRB, 0.34505E+00_JPRB, 0.34085E+00_JPRB ,&
&  0.33757E+00_JPRB, 0.33385E+00_JPRB, 0.33109E+00_JPRB, 0.32786E+00_JPRB, 0.32519E+00_JPRB, 0.32403E+00_JPRB, 0.32334E+00_JPRB ,&
&  0.32176E+00_JPRB, 0.31905E+00_JPRB, 0.31670E+00_JPRB, 0.31563E+00_JPRB, 0.31823E+00_JPRB, 0.32446E+00_JPRB, 0.33424E+00_JPRB ,&
&  0.33174E+00_JPRB, 0.31395E+00_JPRB, 0.33151E+00_JPRB, 0.35327E+00_JPRB, 0.36740E+00_JPRB, 0.38059E+00_JPRB, 0.39060E+00_JPRB ,&
&  0.39970E+00_JPRB, 0.40253E+00_JPRB, 0.40518E+00_JPRB, 0.40691E+00_JPRB, 0.40634E+00_JPRB, 0.40214E+00_JPRB, 0.39741E+00_JPRB ,&
&  0.39386E+00_JPRB, 0.39117E+00_JPRB, 0.38803E+00_JPRB, 0.38239E+00_JPRB, 0.37707E+00_JPRB, 0.37263E+00_JPRB, 0.36832E+00_JPRB ,&
&  0.36336E+00_JPRB, 0.35829E+00_JPRB, 0.35485E+00_JPRB, 0.35440E+00_JPRB, 0.35762E+00_JPRB, 0.36381E+00_JPRB, 0.37525E+00_JPRB ,&
&  0.38946E+00_JPRB, 0.41453E+00_JPRB, 0.46014E+00_JPRB, 0.50862E+00_JPRB, 0.54847E+00_JPRB, 0.57673E+00_JPRB, 0.59991E+00_JPRB ,&
&  0.60814E+00_JPRB, 0.60771E+00_JPRB, 0.60383E+00_JPRB, 0.59479E+00_JPRB, 0.58015E+00_JPRB, 0.55971E+00_JPRB, 0.53485E+00_JPRB ,&
&  0.51215E+00_JPRB, 0.49198E+00_JPRB, 0.47232E+00_JPRB, 0.45324E+00_JPRB, 0.43439E+00_JPRB, 0.41680E+00_JPRB, 0.40164E+00_JPRB ,&
&  0.38850E+00_JPRB, 0.37678E+00_JPRB, 0.36813E+00_JPRB, 0.36376E+00_JPRB, 0.36480E+00_JPRB, 0.36936E+00_JPRB, 0.37600E+00_JPRB ,&
&  0.38210E+00_JPRB, 0.39041E+00_JPRB, 0.39835E+00_JPRB, 0.40621E+00_JPRB, 0.41172E+00_JPRB, 0.41146E+00_JPRB, 0.41059E+00_JPRB ,&
&  0.40820E+00_JPRB, 0.40151E+00_JPRB, 0.39566E+00_JPRB, 0.38836E+00_JPRB, 0.38294E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.35236E+00_JPRB, 0.34775E+00_JPRB, 0.34752E+00_JPRB, 0.35111E+00_JPRB, 0.35489E+00_JPRB, 0.35436E+00_JPRB, 0.35033E+00_JPRB ,&
&  0.34117E+00_JPRB, 0.33316E+00_JPRB, 0.32675E+00_JPRB, 0.32395E+00_JPRB, 0.32580E+00_JPRB, 0.32623E+00_JPRB, 0.32378E+00_JPRB ,&
&  0.32101E+00_JPRB, 0.31593E+00_JPRB, 0.31160E+00_JPRB, 0.30731E+00_JPRB, 0.30420E+00_JPRB, 0.30279E+00_JPRB, 0.30222E+00_JPRB ,&
&  0.30083E+00_JPRB, 0.29924E+00_JPRB, 0.29902E+00_JPRB, 0.29918E+00_JPRB, 0.30105E+00_JPRB, 0.30534E+00_JPRB, 0.31064E+00_JPRB ,&
&  0.31975E+00_JPRB, 0.30485E+00_JPRB, 0.32220E+00_JPRB, 0.32722E+00_JPRB, 0.33197E+00_JPRB, 0.33960E+00_JPRB, 0.34862E+00_JPRB ,&
&  0.35678E+00_JPRB, 0.36146E+00_JPRB, 0.36382E+00_JPRB, 0.36412E+00_JPRB, 0.36244E+00_JPRB, 0.36104E+00_JPRB, 0.35738E+00_JPRB ,&
&  0.35249E+00_JPRB, 0.35051E+00_JPRB, 0.34962E+00_JPRB, 0.34757E+00_JPRB, 0.34550E+00_JPRB, 0.34260E+00_JPRB, 0.33978E+00_JPRB ,&
&  0.33748E+00_JPRB, 0.33490E+00_JPRB, 0.33269E+00_JPRB, 0.33036E+00_JPRB, 0.32894E+00_JPRB, 0.32989E+00_JPRB, 0.33339E+00_JPRB ,&
&  0.33747E+00_JPRB, 0.34386E+00_JPRB, 0.35750E+00_JPRB, 0.37792E+00_JPRB, 0.40176E+00_JPRB, 0.42446E+00_JPRB, 0.44299E+00_JPRB ,&
&  0.45468E+00_JPRB, 0.46092E+00_JPRB, 0.46204E+00_JPRB, 0.46011E+00_JPRB, 0.45440E+00_JPRB, 0.44661E+00_JPRB, 0.43542E+00_JPRB ,&
&  0.42413E+00_JPRB, 0.41333E+00_JPRB, 0.40212E+00_JPRB, 0.39101E+00_JPRB, 0.37900E+00_JPRB, 0.36634E+00_JPRB, 0.35479E+00_JPRB ,&
&  0.34552E+00_JPRB, 0.33880E+00_JPRB, 0.33399E+00_JPRB, 0.33116E+00_JPRB, 0.32973E+00_JPRB, 0.33099E+00_JPRB, 0.33371E+00_JPRB ,&
&  0.33794E+00_JPRB, 0.34364E+00_JPRB, 0.35052E+00_JPRB, 0.35698E+00_JPRB, 0.36321E+00_JPRB, 0.36598E+00_JPRB, 0.36801E+00_JPRB ,&
&  0.36754E+00_JPRB, 0.36416E+00_JPRB, 0.36154E+00_JPRB, 0.35860E+00_JPRB, 0.35606E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.32477E+00_JPRB, 0.32305E+00_JPRB, 0.32558E+00_JPRB, 0.33011E+00_JPRB, 0.33642E+00_JPRB, 0.33556E+00_JPRB, 0.33335E+00_JPRB ,&
&  0.32801E+00_JPRB, 0.32097E+00_JPRB, 0.31417E+00_JPRB, 0.30789E+00_JPRB, 0.30526E+00_JPRB, 0.30587E+00_JPRB, 0.30575E+00_JPRB ,&
&  0.30447E+00_JPRB, 0.30091E+00_JPRB, 0.29596E+00_JPRB, 0.29119E+00_JPRB, 0.28772E+00_JPRB, 0.28583E+00_JPRB, 0.28425E+00_JPRB ,&
&  0.28336E+00_JPRB, 0.28207E+00_JPRB, 0.28297E+00_JPRB, 0.28303E+00_JPRB, 0.28383E+00_JPRB, 0.28758E+00_JPRB, 0.29067E+00_JPRB ,&
&  0.29223E+00_JPRB, 0.29634E+00_JPRB, 0.29554E+00_JPRB, 0.29590E+00_JPRB, 0.29936E+00_JPRB, 0.30710E+00_JPRB, 0.31675E+00_JPRB ,&
&  0.32526E+00_JPRB, 0.33071E+00_JPRB, 0.33296E+00_JPRB, 0.33163E+00_JPRB, 0.32896E+00_JPRB, 0.32659E+00_JPRB, 0.32509E+00_JPRB ,&
&  0.32333E+00_JPRB, 0.31971E+00_JPRB, 0.31570E+00_JPRB, 0.31230E+00_JPRB, 0.31088E+00_JPRB, 0.30994E+00_JPRB, 0.30874E+00_JPRB ,&
&  0.30739E+00_JPRB, 0.30602E+00_JPRB, 0.30350E+00_JPRB, 0.30029E+00_JPRB, 0.29734E+00_JPRB, 0.29652E+00_JPRB, 0.29816E+00_JPRB ,&
&  0.30163E+00_JPRB, 0.30625E+00_JPRB, 0.31295E+00_JPRB, 0.32161E+00_JPRB, 0.33051E+00_JPRB, 0.34124E+00_JPRB, 0.35084E+00_JPRB ,&
&  0.35795E+00_JPRB, 0.36164E+00_JPRB, 0.36254E+00_JPRB, 0.36188E+00_JPRB, 0.35991E+00_JPRB, 0.35677E+00_JPRB, 0.35324E+00_JPRB ,&
&  0.34973E+00_JPRB, 0.34582E+00_JPRB, 0.34138E+00_JPRB, 0.33627E+00_JPRB, 0.32949E+00_JPRB, 0.32227E+00_JPRB, 0.31640E+00_JPRB ,&
&  0.31237E+00_JPRB, 0.30957E+00_JPRB, 0.30758E+00_JPRB, 0.30582E+00_JPRB, 0.30481E+00_JPRB, 0.30492E+00_JPRB, 0.30645E+00_JPRB ,&
&  0.30941E+00_JPRB, 0.31352E+00_JPRB, 0.31857E+00_JPRB, 0.32379E+00_JPRB, 0.32751E+00_JPRB, 0.33053E+00_JPRB, 0.33346E+00_JPRB ,&
&  0.33435E+00_JPRB, 0.33242E+00_JPRB, 0.33100E+00_JPRB, 0.33018E+00_JPRB, 0.32793E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.30117E+00_JPRB, 0.30170E+00_JPRB, 0.30628E+00_JPRB, 0.31123E+00_JPRB, 0.31638E+00_JPRB, 0.31786E+00_JPRB, 0.31558E+00_JPRB ,&
&  0.31284E+00_JPRB, 0.30909E+00_JPRB, 0.30350E+00_JPRB, 0.29653E+00_JPRB, 0.29021E+00_JPRB, 0.28855E+00_JPRB, 0.28872E+00_JPRB ,&
&  0.28899E+00_JPRB, 0.28728E+00_JPRB, 0.28382E+00_JPRB, 0.27998E+00_JPRB, 0.27557E+00_JPRB, 0.27171E+00_JPRB, 0.26881E+00_JPRB ,&
&  0.26796E+00_JPRB, 0.26754E+00_JPRB, 0.26815E+00_JPRB, 0.26862E+00_JPRB, 0.26958E+00_JPRB, 0.27239E+00_JPRB, 0.27774E+00_JPRB ,&
&  0.27552E+00_JPRB, 0.27683E+00_JPRB, 0.27003E+00_JPRB, 0.27269E+00_JPRB, 0.27533E+00_JPRB, 0.28015E+00_JPRB, 0.28561E+00_JPRB ,&
&  0.29109E+00_JPRB, 0.29584E+00_JPRB, 0.29883E+00_JPRB, 0.29904E+00_JPRB, 0.29769E+00_JPRB, 0.29764E+00_JPRB, 0.29759E+00_JPRB ,&
&  0.29693E+00_JPRB, 0.29596E+00_JPRB, 0.29405E+00_JPRB, 0.29021E+00_JPRB, 0.28491E+00_JPRB, 0.28038E+00_JPRB, 0.27729E+00_JPRB ,&
&  0.27693E+00_JPRB, 0.27700E+00_JPRB, 0.27599E+00_JPRB, 0.27466E+00_JPRB, 0.27317E+00_JPRB, 0.27266E+00_JPRB, 0.27398E+00_JPRB ,&
&  0.27637E+00_JPRB, 0.27872E+00_JPRB, 0.28079E+00_JPRB, 0.28438E+00_JPRB, 0.28944E+00_JPRB, 0.29430E+00_JPRB, 0.29871E+00_JPRB ,&
&  0.30220E+00_JPRB, 0.30314E+00_JPRB, 0.30260E+00_JPRB, 0.30127E+00_JPRB, 0.30037E+00_JPRB, 0.29962E+00_JPRB, 0.29954E+00_JPRB ,&
&  0.29924E+00_JPRB, 0.29837E+00_JPRB, 0.29648E+00_JPRB, 0.29365E+00_JPRB, 0.29079E+00_JPRB, 0.28799E+00_JPRB, 0.28600E+00_JPRB ,&
&  0.28493E+00_JPRB, 0.28421E+00_JPRB, 0.28324E+00_JPRB, 0.28222E+00_JPRB, 0.28219E+00_JPRB, 0.28249E+00_JPRB, 0.28351E+00_JPRB ,&
&  0.28522E+00_JPRB, 0.28776E+00_JPRB, 0.29125E+00_JPRB, 0.29499E+00_JPRB, 0.29570E+00_JPRB, 0.29627E+00_JPRB, 0.29845E+00_JPRB ,&
&  0.30060E+00_JPRB, 0.30130E+00_JPRB, 0.30121E+00_JPRB, 0.30116E+00_JPRB, 0.30174E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.28150E+00_JPRB, 0.28377E+00_JPRB, 0.28782E+00_JPRB, 0.29381E+00_JPRB, 0.29709E+00_JPRB, 0.30044E+00_JPRB, 0.30025E+00_JPRB ,&
&  0.29777E+00_JPRB, 0.29434E+00_JPRB, 0.29021E+00_JPRB, 0.28505E+00_JPRB, 0.27956E+00_JPRB, 0.27540E+00_JPRB, 0.27395E+00_JPRB ,&
&  0.27387E+00_JPRB, 0.27286E+00_JPRB, 0.27022E+00_JPRB, 0.26667E+00_JPRB, 0.26272E+00_JPRB, 0.25955E+00_JPRB, 0.25711E+00_JPRB ,&
&  0.25637E+00_JPRB, 0.25676E+00_JPRB, 0.25777E+00_JPRB, 0.25835E+00_JPRB, 0.25861E+00_JPRB, 0.26089E+00_JPRB, 0.26477E+00_JPRB ,&
&  0.26272E+00_JPRB, 0.26044E+00_JPRB, 0.25514E+00_JPRB, 0.25469E+00_JPRB, 0.25609E+00_JPRB, 0.25837E+00_JPRB, 0.26069E+00_JPRB ,&
&  0.26275E+00_JPRB, 0.26548E+00_JPRB, 0.26848E+00_JPRB, 0.27079E+00_JPRB, 0.27231E+00_JPRB, 0.27373E+00_JPRB, 0.27464E+00_JPRB ,&
&  0.27489E+00_JPRB, 0.27382E+00_JPRB, 0.27195E+00_JPRB, 0.26853E+00_JPRB, 0.26413E+00_JPRB, 0.26115E+00_JPRB, 0.25983E+00_JPRB ,&
&  0.25859E+00_JPRB, 0.25868E+00_JPRB, 0.25890E+00_JPRB, 0.25849E+00_JPRB, 0.25768E+00_JPRB, 0.25770E+00_JPRB, 0.25868E+00_JPRB ,&
&  0.25961E+00_JPRB, 0.25974E+00_JPRB, 0.25930E+00_JPRB, 0.25985E+00_JPRB, 0.26188E+00_JPRB, 0.26467E+00_JPRB, 0.26694E+00_JPRB ,&
&  0.26854E+00_JPRB, 0.26839E+00_JPRB, 0.26726E+00_JPRB, 0.26508E+00_JPRB, 0.26345E+00_JPRB, 0.26348E+00_JPRB, 0.26496E+00_JPRB ,&
&  0.26619E+00_JPRB, 0.26648E+00_JPRB, 0.26673E+00_JPRB, 0.26542E+00_JPRB, 0.26393E+00_JPRB, 0.26337E+00_JPRB, 0.26309E+00_JPRB ,&
&  0.26243E+00_JPRB, 0.26153E+00_JPRB, 0.26066E+00_JPRB, 0.26017E+00_JPRB, 0.26038E+00_JPRB, 0.26023E+00_JPRB, 0.26087E+00_JPRB ,&
&  0.26320E+00_JPRB, 0.26610E+00_JPRB, 0.26896E+00_JPRB, 0.27106E+00_JPRB, 0.27212E+00_JPRB, 0.27199E+00_JPRB, 0.27370E+00_JPRB ,&
&  0.27658E+00_JPRB, 0.27866E+00_JPRB, 0.27851E+00_JPRB, 0.27802E+00_JPRB, 0.27955E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.26451E+00_JPRB, 0.26701E+00_JPRB, 0.27126E+00_JPRB, 0.27618E+00_JPRB, 0.27944E+00_JPRB, 0.28357E+00_JPRB, 0.28423E+00_JPRB ,&
&  0.28348E+00_JPRB, 0.28168E+00_JPRB, 0.27763E+00_JPRB, 0.27270E+00_JPRB, 0.26804E+00_JPRB, 0.26415E+00_JPRB, 0.26148E+00_JPRB ,&
&  0.26004E+00_JPRB, 0.25880E+00_JPRB, 0.25731E+00_JPRB, 0.25490E+00_JPRB, 0.25197E+00_JPRB, 0.24934E+00_JPRB, 0.24729E+00_JPRB ,&
&  0.24654E+00_JPRB, 0.24643E+00_JPRB, 0.24674E+00_JPRB, 0.24660E+00_JPRB, 0.24637E+00_JPRB, 0.24750E+00_JPRB, 0.24911E+00_JPRB ,&
&  0.24818E+00_JPRB, 0.24407E+00_JPRB, 0.24142E+00_JPRB, 0.24121E+00_JPRB, 0.24130E+00_JPRB, 0.24180E+00_JPRB, 0.24250E+00_JPRB ,&
&  0.24269E+00_JPRB, 0.24334E+00_JPRB, 0.24476E+00_JPRB, 0.24637E+00_JPRB, 0.24757E+00_JPRB, 0.25002E+00_JPRB, 0.25126E+00_JPRB ,&
&  0.25138E+00_JPRB, 0.25066E+00_JPRB, 0.24935E+00_JPRB, 0.24683E+00_JPRB, 0.24510E+00_JPRB, 0.24466E+00_JPRB, 0.24412E+00_JPRB ,&
&  0.24492E+00_JPRB, 0.24577E+00_JPRB, 0.24660E+00_JPRB, 0.24593E+00_JPRB, 0.24438E+00_JPRB, 0.24430E+00_JPRB, 0.24492E+00_JPRB ,&
&  0.24511E+00_JPRB, 0.24424E+00_JPRB, 0.24269E+00_JPRB, 0.24193E+00_JPRB, 0.24158E+00_JPRB, 0.24211E+00_JPRB, 0.24241E+00_JPRB ,&
&  0.24218E+00_JPRB, 0.24094E+00_JPRB, 0.24005E+00_JPRB, 0.23962E+00_JPRB, 0.23941E+00_JPRB, 0.23982E+00_JPRB, 0.23979E+00_JPRB ,&
&  0.24065E+00_JPRB, 0.24152E+00_JPRB, 0.24080E+00_JPRB, 0.24028E+00_JPRB, 0.23987E+00_JPRB, 0.23985E+00_JPRB, 0.23959E+00_JPRB ,&
&  0.23889E+00_JPRB, 0.23808E+00_JPRB, 0.23752E+00_JPRB, 0.23741E+00_JPRB, 0.23661E+00_JPRB, 0.23566E+00_JPRB, 0.23631E+00_JPRB ,&
&  0.23941E+00_JPRB, 0.24279E+00_JPRB, 0.24573E+00_JPRB, 0.24770E+00_JPRB, 0.24839E+00_JPRB, 0.24978E+00_JPRB, 0.25239E+00_JPRB ,&
&  0.25460E+00_JPRB, 0.25547E+00_JPRB, 0.25746E+00_JPRB, 0.25930E+00_JPRB, 0.26140E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.24975E+00_JPRB, 0.25304E+00_JPRB, 0.25549E+00_JPRB, 0.25882E+00_JPRB, 0.26188E+00_JPRB, 0.26663E+00_JPRB, 0.26853E+00_JPRB ,&
&  0.26835E+00_JPRB, 0.26799E+00_JPRB, 0.26651E+00_JPRB, 0.26402E+00_JPRB, 0.26015E+00_JPRB, 0.25618E+00_JPRB, 0.25247E+00_JPRB ,&
&  0.24963E+00_JPRB, 0.24735E+00_JPRB, 0.24556E+00_JPRB, 0.24395E+00_JPRB, 0.24168E+00_JPRB, 0.23955E+00_JPRB, 0.23793E+00_JPRB ,&
&  0.23716E+00_JPRB, 0.23642E+00_JPRB, 0.23603E+00_JPRB, 0.23586E+00_JPRB, 0.23554E+00_JPRB, 0.23515E+00_JPRB, 0.23510E+00_JPRB ,&
&  0.23401E+00_JPRB, 0.23056E+00_JPRB, 0.22859E+00_JPRB, 0.22796E+00_JPRB, 0.22736E+00_JPRB, 0.22778E+00_JPRB, 0.22739E+00_JPRB ,&
&  0.22712E+00_JPRB, 0.22762E+00_JPRB, 0.22906E+00_JPRB, 0.23077E+00_JPRB, 0.23096E+00_JPRB, 0.23159E+00_JPRB, 0.23279E+00_JPRB ,&
&  0.23324E+00_JPRB, 0.23259E+00_JPRB, 0.23088E+00_JPRB, 0.23041E+00_JPRB, 0.23114E+00_JPRB, 0.23130E+00_JPRB, 0.23180E+00_JPRB ,&
&  0.23254E+00_JPRB, 0.23374E+00_JPRB, 0.23443E+00_JPRB, 0.23292E+00_JPRB, 0.23141E+00_JPRB, 0.23087E+00_JPRB, 0.23092E+00_JPRB ,&
&  0.23072E+00_JPRB, 0.22987E+00_JPRB, 0.22797E+00_JPRB, 0.22629E+00_JPRB, 0.22377E+00_JPRB, 0.22177E+00_JPRB, 0.22079E+00_JPRB ,&
&  0.22130E+00_JPRB, 0.22180E+00_JPRB, 0.22320E+00_JPRB, 0.22444E+00_JPRB, 0.22512E+00_JPRB, 0.22522E+00_JPRB, 0.22383E+00_JPRB ,&
&  0.22223E+00_JPRB, 0.22161E+00_JPRB, 0.22060E+00_JPRB, 0.21957E+00_JPRB, 0.21979E+00_JPRB, 0.21969E+00_JPRB, 0.21916E+00_JPRB ,&
&  0.21854E+00_JPRB, 0.21881E+00_JPRB, 0.21903E+00_JPRB, 0.21852E+00_JPRB, 0.21781E+00_JPRB, 0.21687E+00_JPRB, 0.21732E+00_JPRB ,&
&  0.21872E+00_JPRB, 0.22077E+00_JPRB, 0.22319E+00_JPRB, 0.22586E+00_JPRB, 0.22867E+00_JPRB, 0.23119E+00_JPRB, 0.23362E+00_JPRB ,&
&  0.23664E+00_JPRB, 0.23767E+00_JPRB, 0.23958E+00_JPRB, 0.24262E+00_JPRB, 0.24561E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.23209E+00_JPRB, 0.23682E+00_JPRB, 0.23939E+00_JPRB, 0.24157E+00_JPRB, 0.24412E+00_JPRB, 0.24869E+00_JPRB, 0.25194E+00_JPRB ,&
&  0.25265E+00_JPRB, 0.25273E+00_JPRB, 0.25240E+00_JPRB, 0.25139E+00_JPRB, 0.24972E+00_JPRB, 0.24816E+00_JPRB, 0.24548E+00_JPRB ,&
&  0.24212E+00_JPRB, 0.23905E+00_JPRB, 0.23630E+00_JPRB, 0.23371E+00_JPRB, 0.23127E+00_JPRB, 0.22904E+00_JPRB, 0.22729E+00_JPRB ,&
&  0.22678E+00_JPRB, 0.22656E+00_JPRB, 0.22567E+00_JPRB, 0.22542E+00_JPRB, 0.22539E+00_JPRB, 0.22461E+00_JPRB, 0.22318E+00_JPRB ,&
&  0.22141E+00_JPRB, 0.21839E+00_JPRB, 0.21534E+00_JPRB, 0.21434E+00_JPRB, 0.21434E+00_JPRB, 0.21353E+00_JPRB, 0.21236E+00_JPRB ,&
&  0.21352E+00_JPRB, 0.21556E+00_JPRB, 0.21778E+00_JPRB, 0.21888E+00_JPRB, 0.21963E+00_JPRB, 0.22082E+00_JPRB, 0.22228E+00_JPRB ,&
&  0.22247E+00_JPRB, 0.22001E+00_JPRB, 0.21786E+00_JPRB, 0.21748E+00_JPRB, 0.21747E+00_JPRB, 0.21724E+00_JPRB, 0.21770E+00_JPRB ,&
&  0.21839E+00_JPRB, 0.21988E+00_JPRB, 0.22088E+00_JPRB, 0.21992E+00_JPRB, 0.21810E+00_JPRB, 0.21677E+00_JPRB, 0.21594E+00_JPRB ,&
&  0.21558E+00_JPRB, 0.21516E+00_JPRB, 0.21434E+00_JPRB, 0.21297E+00_JPRB, 0.21141E+00_JPRB, 0.20997E+00_JPRB, 0.20849E+00_JPRB ,&
&  0.20724E+00_JPRB, 0.20755E+00_JPRB, 0.20993E+00_JPRB, 0.21142E+00_JPRB, 0.21108E+00_JPRB, 0.20977E+00_JPRB, 0.20815E+00_JPRB ,&
&  0.20717E+00_JPRB, 0.20494E+00_JPRB, 0.20302E+00_JPRB, 0.20157E+00_JPRB, 0.20052E+00_JPRB, 0.19906E+00_JPRB, 0.19852E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.20038E+00_JPRB, 0.20077E+00_JPRB, 0.20070E+00_JPRB, 0.20024E+00_JPRB, 0.19988E+00_JPRB, 0.20075E+00_JPRB ,&
&  0.20198E+00_JPRB, 0.20397E+00_JPRB, 0.20616E+00_JPRB, 0.20906E+00_JPRB, 0.21182E+00_JPRB, 0.21526E+00_JPRB, 0.21807E+00_JPRB ,&
&  0.21986E+00_JPRB, 0.22040E+00_JPRB, 0.22162E+00_JPRB, 0.22352E+00_JPRB, 0.22679E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.21316E+00_JPRB, 0.21856E+00_JPRB, 0.22322E+00_JPRB, 0.22563E+00_JPRB, 0.22712E+00_JPRB, 0.22947E+00_JPRB, 0.23200E+00_JPRB ,&
&  0.23385E+00_JPRB, 0.23496E+00_JPRB, 0.23552E+00_JPRB, 0.23550E+00_JPRB, 0.23469E+00_JPRB, 0.23328E+00_JPRB, 0.23161E+00_JPRB ,&
&  0.22975E+00_JPRB, 0.22709E+00_JPRB, 0.22462E+00_JPRB, 0.22278E+00_JPRB, 0.22123E+00_JPRB, 0.21943E+00_JPRB, 0.21800E+00_JPRB ,&
&  0.21649E+00_JPRB, 0.21600E+00_JPRB, 0.21533E+00_JPRB, 0.21460E+00_JPRB, 0.21399E+00_JPRB, 0.21302E+00_JPRB, 0.21217E+00_JPRB ,&
&  0.20985E+00_JPRB, 0.20607E+00_JPRB, 0.20442E+00_JPRB, 0.20358E+00_JPRB, 0.20374E+00_JPRB, 0.20308E+00_JPRB, 0.20222E+00_JPRB ,&
&  0.20379E+00_JPRB, 0.20660E+00_JPRB, 0.20867E+00_JPRB, 0.20978E+00_JPRB, 0.21055E+00_JPRB, 0.21165E+00_JPRB, 0.21290E+00_JPRB ,&
&  0.21067E+00_JPRB, 0.20796E+00_JPRB, 0.20604E+00_JPRB, 0.20512E+00_JPRB, 0.20510E+00_JPRB, 0.20600E+00_JPRB, 0.20767E+00_JPRB ,&
&  0.20919E+00_JPRB, 0.21010E+00_JPRB, 0.20983E+00_JPRB, 0.20853E+00_JPRB, 0.20688E+00_JPRB, 0.20608E+00_JPRB, 0.20564E+00_JPRB ,&
&  0.20546E+00_JPRB, 0.20517E+00_JPRB, 0.20482E+00_JPRB, 0.20377E+00_JPRB, 0.19823E+00_JPRB, 0.19459E+00_JPRB, 0.19196E+00_JPRB ,&
&  0.19260E+00_JPRB, 0.19256E+00_JPRB, 0.19116E+00_JPRB, 0.19673E+00_JPRB, 0.20258E+00_JPRB, 0.19801E+00_JPRB, 0.19264E+00_JPRB ,&
&  0.18530E+00_JPRB, 0.18636E+00_JPRB, 0.18367E+00_JPRB, 0.17892E+00_JPRB, 0.17768E+00_JPRB, 0.17778E+00_JPRB, 0.17731E+00_JPRB ,&
&  0.17796E+00_JPRB, 0.17787E+00_JPRB, 0.17892E+00_JPRB, 0.17940E+00_JPRB, 0.17966E+00_JPRB, 0.18046E+00_JPRB, 0.18130E+00_JPRB ,&
&  0.18345E+00_JPRB, 0.18732E+00_JPRB, 0.19171E+00_JPRB, 0.19310E+00_JPRB, 0.19584E+00_JPRB, 0.19828E+00_JPRB, 0.19985E+00_JPRB ,&
&  0.20112E+00_JPRB, 0.20301E+00_JPRB, 0.20482E+00_JPRB, 0.20632E+00_JPRB, 0.20884E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.20011E+00_JPRB, 0.20250E+00_JPRB, 0.20542E+00_JPRB, 0.20832E+00_JPRB, 0.21039E+00_JPRB, 0.21243E+00_JPRB, 0.21423E+00_JPRB ,&
&  0.21583E+00_JPRB, 0.21694E+00_JPRB, 0.21769E+00_JPRB, 0.21812E+00_JPRB, 0.21843E+00_JPRB, 0.21819E+00_JPRB, 0.21752E+00_JPRB ,&
&  0.21681E+00_JPRB, 0.21553E+00_JPRB, 0.21385E+00_JPRB, 0.21249E+00_JPRB, 0.21164E+00_JPRB, 0.21054E+00_JPRB, 0.20892E+00_JPRB ,&
&  0.20716E+00_JPRB, 0.20557E+00_JPRB, 0.20429E+00_JPRB, 0.20306E+00_JPRB, 0.20298E+00_JPRB, 0.20355E+00_JPRB, 0.20404E+00_JPRB ,&
&  0.19973E+00_JPRB, 0.19455E+00_JPRB, 0.19560E+00_JPRB, 0.19662E+00_JPRB, 0.19454E+00_JPRB, 0.19580E+00_JPRB, 0.19651E+00_JPRB ,&
&  0.19750E+00_JPRB, 0.19883E+00_JPRB, 0.20017E+00_JPRB, 0.20083E+00_JPRB, 0.20124E+00_JPRB, 0.20190E+00_JPRB, 0.20126E+00_JPRB ,&
&  0.19924E+00_JPRB, 0.19755E+00_JPRB, 0.19610E+00_JPRB, 0.19463E+00_JPRB, 0.19435E+00_JPRB, 0.19602E+00_JPRB, 0.19705E+00_JPRB ,&
&  0.19620E+00_JPRB, 0.19484E+00_JPRB, 0.19357E+00_JPRB, 0.19395E+00_JPRB, 0.19477E+00_JPRB, 0.19521E+00_JPRB, 0.19315E+00_JPRB ,&
&  0.19194E+00_JPRB, 0.18916E+00_JPRB, 0.18660E+00_JPRB, 0.18408E+00_JPRB, 0.18282E+00_JPRB, 0.17930E+00_JPRB, 0.17554E+00_JPRB ,&
&  0.17477E+00_JPRB, 0.17876E+00_JPRB, 0.18490E+00_JPRB, 0.18705E+00_JPRB, 0.17649E+00_JPRB, 0.17733E+00_JPRB, 0.17755E+00_JPRB ,&
&  0.17306E+00_JPRB, 0.16791E+00_JPRB, 0.16415E+00_JPRB, 0.16255E+00_JPRB, 0.16193E+00_JPRB, 0.16132E+00_JPRB, 0.16171E+00_JPRB ,&
&  0.16253E+00_JPRB, 0.16295E+00_JPRB, 0.16351E+00_JPRB, 0.16469E+00_JPRB, 0.16586E+00_JPRB, 0.16705E+00_JPRB, 0.16811E+00_JPRB ,&
&  0.17048E+00_JPRB, 0.17338E+00_JPRB, 0.17562E+00_JPRB, 0.17854E+00_JPRB, 0.18167E+00_JPRB, 0.18475E+00_JPRB, 0.18774E+00_JPRB ,&
&  0.19105E+00_JPRB, 0.19295E+00_JPRB, 0.19454E+00_JPRB, 0.19736E+00_JPRB, 0.19874E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.19403E+00_JPRB, 0.19668E+00_JPRB, 0.19855E+00_JPRB, 0.20032E+00_JPRB, 0.20169E+00_JPRB, 0.20313E+00_JPRB, 0.20377E+00_JPRB ,&
&  0.20434E+00_JPRB, 0.20431E+00_JPRB, 0.20473E+00_JPRB, 0.20468E+00_JPRB, 0.20486E+00_JPRB, 0.20375E+00_JPRB, 0.20321E+00_JPRB ,&
&  0.20278E+00_JPRB, 0.20315E+00_JPRB, 0.20317E+00_JPRB, 0.20307E+00_JPRB, 0.20238E+00_JPRB, 0.20145E+00_JPRB, 0.19993E+00_JPRB ,&
&  0.19677E+00_JPRB, 0.19451E+00_JPRB, 0.19470E+00_JPRB, 0.19412E+00_JPRB, 0.19450E+00_JPRB, 0.19449E+00_JPRB, 0.19495E+00_JPRB ,&
&  0.19065E+00_JPRB, 0.18918E+00_JPRB, 0.18958E+00_JPRB, 0.18852E+00_JPRB, 0.19049E+00_JPRB, 0.19137E+00_JPRB, 0.19313E+00_JPRB ,&
&  0.19352E+00_JPRB, 0.19377E+00_JPRB, 0.19449E+00_JPRB, 0.19398E+00_JPRB, 0.19426E+00_JPRB, 0.19469E+00_JPRB, 0.19351E+00_JPRB ,&
&  0.19047E+00_JPRB, 0.18818E+00_JPRB, 0.18491E+00_JPRB, 0.17692E+00_JPRB, 0.17514E+00_JPRB, 0.17562E+00_JPRB, 0.17436E+00_JPRB ,&
&  0.17170E+00_JPRB, 0.17220E+00_JPRB, 0.17178E+00_JPRB, 0.17028E+00_JPRB, 0.17228E+00_JPRB, 0.17509E+00_JPRB, 0.17525E+00_JPRB ,&
&  0.17402E+00_JPRB, 0.17371E+00_JPRB, 0.17306E+00_JPRB, 0.17176E+00_JPRB, 0.17023E+00_JPRB, 0.16961E+00_JPRB, 0.16903E+00_JPRB ,&
&  0.16872E+00_JPRB, 0.16737E+00_JPRB, 0.17084E+00_JPRB, 0.17160E+00_JPRB, 0.16668E+00_JPRB, 0.16891E+00_JPRB, 0.16660E+00_JPRB ,&
&  0.15966E+00_JPRB, 0.15658E+00_JPRB, 0.15480E+00_JPRB, 0.15328E+00_JPRB, 0.15295E+00_JPRB, 0.15299E+00_JPRB, 0.15277E+00_JPRB ,&
&  0.15358E+00_JPRB, 0.15464E+00_JPRB, 0.15515E+00_JPRB, 0.15585E+00_JPRB, 0.15682E+00_JPRB, 0.15824E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16178E+00_JPRB, 0.16408E+00_JPRB, 0.16523E+00_JPRB, 0.16712E+00_JPRB, 0.16852E+00_JPRB, 0.17013E+00_JPRB, 0.17401E+00_JPRB ,&
&  0.17715E+00_JPRB, 0.18330E+00_JPRB, 0.19171E+00_JPRB, 0.19429E+00_JPRB, 0.19149E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.18885E+00_JPRB, 0.19087E+00_JPRB, 0.19293E+00_JPRB, 0.19455E+00_JPRB, 0.19620E+00_JPRB, 0.19757E+00_JPRB, 0.19829E+00_JPRB ,&
&  0.19840E+00_JPRB, 0.19773E+00_JPRB, 0.19773E+00_JPRB, 0.19635E+00_JPRB, 0.19504E+00_JPRB, 0.19080E+00_JPRB, 0.19030E+00_JPRB ,&
&  0.18931E+00_JPRB, 0.18998E+00_JPRB, 0.18954E+00_JPRB, 0.18983E+00_JPRB, 0.18911E+00_JPRB, 0.18874E+00_JPRB, 0.18706E+00_JPRB ,&
&  0.18457E+00_JPRB, 0.18272E+00_JPRB, 0.18094E+00_JPRB, 0.18044E+00_JPRB, 0.18124E+00_JPRB, 0.18181E+00_JPRB, 0.18147E+00_JPRB ,&
&  0.18150E+00_JPRB, 0.18178E+00_JPRB, 0.18221E+00_JPRB, 0.18220E+00_JPRB, 0.18748E+00_JPRB, 0.18967E+00_JPRB, 0.19078E+00_JPRB ,&
&  0.19054E+00_JPRB, 0.19079E+00_JPRB, 0.19083E+00_JPRB, 0.18987E+00_JPRB, 0.18890E+00_JPRB, 0.18462E+00_JPRB, 0.18190E+00_JPRB ,&
&  0.17410E+00_JPRB, 0.16850E+00_JPRB, 0.16425E+00_JPRB, 0.16165E+00_JPRB, 0.16063E+00_JPRB, 0.16070E+00_JPRB, 0.16030E+00_JPRB ,&
&  0.15878E+00_JPRB, 0.15737E+00_JPRB, 0.15650E+00_JPRB, 0.15725E+00_JPRB, 0.15846E+00_JPRB, 0.15978E+00_JPRB, 0.16127E+00_JPRB ,&
&  0.16256E+00_JPRB, 0.16292E+00_JPRB, 0.16292E+00_JPRB, 0.16339E+00_JPRB, 0.16377E+00_JPRB, 0.16432E+00_JPRB, 0.16474E+00_JPRB ,&
&  0.16579E+00_JPRB, 0.16493E+00_JPRB, 0.16558E+00_JPRB, 0.16675E+00_JPRB, 0.16435E+00_JPRB, 0.16052E+00_JPRB, 0.15673E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15110E+00_JPRB, 0.14942E+00_JPRB, 0.14832E+00_JPRB, 0.14804E+00_JPRB, 0.14806E+00_JPRB, 0.14855E+00_JPRB ,&
&  0.14920E+00_JPRB, 0.15016E+00_JPRB, 0.15079E+00_JPRB, 0.15164E+00_JPRB, 0.15269E+00_JPRB, 0.15385E+00_JPRB, 0.15522E+00_JPRB ,&
&  0.15692E+00_JPRB, 0.15853E+00_JPRB, 0.15968E+00_JPRB, 0.16128E+00_JPRB, 0.16429E+00_JPRB, 0.16828E+00_JPRB, 0.17429E+00_JPRB ,&
&  0.18090E+00_JPRB, 0.18619E+00_JPRB, 0.18326E+00_JPRB, 0.18584E+00_JPRB, 0.18788E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.18883E+00_JPRB, 0.19043E+00_JPRB, 0.19194E+00_JPRB, 0.19347E+00_JPRB, 0.19475E+00_JPRB, 0.19596E+00_JPRB, 0.19675E+00_JPRB ,&
&  0.19415E+00_JPRB, 0.19269E+00_JPRB, 0.18997E+00_JPRB, 0.18760E+00_JPRB, 0.18477E+00_JPRB, 0.18164E+00_JPRB, 0.18061E+00_JPRB ,&
&  0.17902E+00_JPRB, 0.17872E+00_JPRB, 0.17739E+00_JPRB, 0.17682E+00_JPRB, 0.17571E+00_JPRB, 0.17503E+00_JPRB, 0.17384E+00_JPRB ,&
&  0.17161E+00_JPRB, 0.17049E+00_JPRB, 0.16944E+00_JPRB, 0.16980E+00_JPRB, 0.17131E+00_JPRB, 0.17291E+00_JPRB, 0.17462E+00_JPRB ,&
&  0.17711E+00_JPRB, 0.17817E+00_JPRB, 0.18054E+00_JPRB, 0.18233E+00_JPRB, 0.18542E+00_JPRB, 0.18799E+00_JPRB, 0.18854E+00_JPRB ,&
&  0.18848E+00_JPRB, 0.18647E+00_JPRB, 0.18579E+00_JPRB, 0.18349E+00_JPRB, 0.17737E+00_JPRB, 0.17399E+00_JPRB, 0.16915E+00_JPRB ,&
&  0.16381E+00_JPRB, 0.15981E+00_JPRB, 0.15706E+00_JPRB, 0.15622E+00_JPRB, 0.15534E+00_JPRB, 0.15515E+00_JPRB, 0.15502E+00_JPRB ,&
&  0.15427E+00_JPRB, 0.15338E+00_JPRB, 0.15234E+00_JPRB, 0.15180E+00_JPRB, 0.15225E+00_JPRB, 0.15309E+00_JPRB, 0.15483E+00_JPRB ,&
&  0.15654E+00_JPRB, 0.15752E+00_JPRB, 0.15858E+00_JPRB, 0.15894E+00_JPRB, 0.15907E+00_JPRB, 0.15933E+00_JPRB, 0.15942E+00_JPRB ,&
&  0.16014E+00_JPRB, 0.16031E+00_JPRB, 0.15979E+00_JPRB, 0.15891E+00_JPRB, 0.15665E+00_JPRB, 0.15367E+00_JPRB, 0.15135E+00_JPRB ,&
&  0.14934E+00_JPRB, 0.14802E+00_JPRB, 0.14755E+00_JPRB, 0.14705E+00_JPRB, 0.14727E+00_JPRB, 0.14762E+00_JPRB, 0.14794E+00_JPRB ,&
&  0.14851E+00_JPRB, 0.14902E+00_JPRB, 0.14951E+00_JPRB, 0.15070E+00_JPRB, 0.15169E+00_JPRB, 0.15317E+00_JPRB, 0.15472E+00_JPRB ,&
&  0.15580E+00_JPRB, 0.15743E+00_JPRB, 0.15882E+00_JPRB, 0.16034E+00_JPRB, 0.16490E+00_JPRB, 0.16850E+00_JPRB, 0.17581E+00_JPRB ,&
&  0.18042E+00_JPRB, 0.18190E+00_JPRB, 0.18217E+00_JPRB, 0.18779E+00_JPRB, 0.18819E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.18941E+00_JPRB, 0.18997E+00_JPRB, 0.18995E+00_JPRB, 0.19093E+00_JPRB, 0.19168E+00_JPRB, 0.19250E+00_JPRB, 0.19178E+00_JPRB ,&
&  0.19117E+00_JPRB, 0.18971E+00_JPRB, 0.18856E+00_JPRB, 0.18549E+00_JPRB, 0.18305E+00_JPRB, 0.18009E+00_JPRB, 0.17755E+00_JPRB ,&
&  0.17528E+00_JPRB, 0.17311E+00_JPRB, 0.17109E+00_JPRB, 0.16933E+00_JPRB, 0.16771E+00_JPRB, 0.16626E+00_JPRB, 0.16485E+00_JPRB ,&
&  0.16276E+00_JPRB, 0.16220E+00_JPRB, 0.16223E+00_JPRB, 0.16353E+00_JPRB, 0.16618E+00_JPRB, 0.16789E+00_JPRB, 0.16994E+00_JPRB ,&
&  0.17220E+00_JPRB, 0.17147E+00_JPRB, 0.17454E+00_JPRB, 0.17750E+00_JPRB, 0.17993E+00_JPRB, 0.18194E+00_JPRB, 0.18317E+00_JPRB ,&
&  0.18373E+00_JPRB, 0.17857E+00_JPRB, 0.17966E+00_JPRB, 0.17525E+00_JPRB, 0.17051E+00_JPRB, 0.16845E+00_JPRB, 0.16446E+00_JPRB ,&
&  0.16274E+00_JPRB, 0.15963E+00_JPRB, 0.15763E+00_JPRB, 0.15593E+00_JPRB, 0.15389E+00_JPRB, 0.15345E+00_JPRB, 0.15267E+00_JPRB ,&
&  0.15225E+00_JPRB, 0.15238E+00_JPRB, 0.15163E+00_JPRB, 0.15143E+00_JPRB, 0.15101E+00_JPRB, 0.15064E+00_JPRB, 0.15110E+00_JPRB ,&
&  0.15178E+00_JPRB, 0.15257E+00_JPRB, 0.15330E+00_JPRB, 0.15340E+00_JPRB, 0.15333E+00_JPRB, 0.15332E+00_JPRB, 0.15324E+00_JPRB ,&
&  0.15353E+00_JPRB, 0.15356E+00_JPRB, 0.15316E+00_JPRB, 0.15270E+00_JPRB, 0.15162E+00_JPRB, 0.15053E+00_JPRB, 0.14946E+00_JPRB ,&
&  0.14850E+00_JPRB, 0.14799E+00_JPRB, 0.14754E+00_JPRB, 0.14741E+00_JPRB, 0.14741E+00_JPRB, 0.14773E+00_JPRB, 0.14806E+00_JPRB ,&
&  0.14836E+00_JPRB, 0.14911E+00_JPRB, 0.14965E+00_JPRB, 0.15055E+00_JPRB, 0.15164E+00_JPRB, 0.15262E+00_JPRB, 0.15411E+00_JPRB ,&
&  0.15573E+00_JPRB, 0.15749E+00_JPRB, 0.16001E+00_JPRB, 0.16220E+00_JPRB, 0.16497E+00_JPRB, 0.16854E+00_JPRB, 0.17117E+00_JPRB ,&
&  0.17400E+00_JPRB, 0.17670E+00_JPRB, 0.17684E+00_JPRB, 0.18410E+00_JPRB, 0.18815E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.18181E+00_JPRB, 0.18261E+00_JPRB, 0.18301E+00_JPRB, 0.18340E+00_JPRB, 0.18446E+00_JPRB, 0.18445E+00_JPRB, 0.18422E+00_JPRB ,&
&  0.18207E+00_JPRB, 0.18125E+00_JPRB, 0.17992E+00_JPRB, 0.17804E+00_JPRB, 0.17612E+00_JPRB, 0.17294E+00_JPRB, 0.17097E+00_JPRB ,&
&  0.16906E+00_JPRB, 0.16703E+00_JPRB, 0.16570E+00_JPRB, 0.16396E+00_JPRB, 0.16247E+00_JPRB, 0.16077E+00_JPRB, 0.16020E+00_JPRB ,&
&  0.15935E+00_JPRB, 0.15929E+00_JPRB, 0.16011E+00_JPRB, 0.16075E+00_JPRB, 0.16195E+00_JPRB, 0.16323E+00_JPRB, 0.16487E+00_JPRB ,&
&  0.16525E+00_JPRB, 0.16601E+00_JPRB, 0.16677E+00_JPRB, 0.16727E+00_JPRB, 0.16906E+00_JPRB, 0.17034E+00_JPRB, 0.17079E+00_JPRB ,&
&  0.17030E+00_JPRB, 0.16940E+00_JPRB, 0.16762E+00_JPRB, 0.16560E+00_JPRB, 0.16330E+00_JPRB, 0.16081E+00_JPRB, 0.15843E+00_JPRB ,&
&  0.15661E+00_JPRB, 0.15491E+00_JPRB, 0.15363E+00_JPRB, 0.15259E+00_JPRB, 0.15198E+00_JPRB, 0.15150E+00_JPRB, 0.15103E+00_JPRB ,&
&  0.15078E+00_JPRB, 0.15058E+00_JPRB, 0.15025E+00_JPRB, 0.15027E+00_JPRB, 0.15019E+00_JPRB, 0.15026E+00_JPRB, 0.15034E+00_JPRB ,&
&  0.15050E+00_JPRB, 0.15050E+00_JPRB, 0.15042E+00_JPRB, 0.15012E+00_JPRB, 0.14983E+00_JPRB, 0.14949E+00_JPRB, 0.14917E+00_JPRB ,&
&  0.14882E+00_JPRB, 0.14880E+00_JPRB, 0.14852E+00_JPRB, 0.14833E+00_JPRB, 0.14808E+00_JPRB, 0.14763E+00_JPRB, 0.14750E+00_JPRB ,&
&  0.14758E+00_JPRB, 0.14772E+00_JPRB, 0.14757E+00_JPRB, 0.14767E+00_JPRB, 0.14786E+00_JPRB, 0.14815E+00_JPRB, 0.14896E+00_JPRB ,&
&  0.14946E+00_JPRB, 0.14998E+00_JPRB, 0.15046E+00_JPRB, 0.15115E+00_JPRB, 0.15233E+00_JPRB, 0.15352E+00_JPRB, 0.15485E+00_JPRB ,&
&  0.15645E+00_JPRB, 0.15795E+00_JPRB, 0.15944E+00_JPRB, 0.16092E+00_JPRB, 0.16337E+00_JPRB, 0.16512E+00_JPRB, 0.16744E+00_JPRB ,&
&  0.16946E+00_JPRB, 0.16830E+00_JPRB, 0.17660E+00_JPRB, 0.17734E+00_JPRB, 0.17861E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.16587E+00_JPRB, 0.16599E+00_JPRB, 0.16521E+00_JPRB, 0.16436E+00_JPRB, 0.15782E+00_JPRB, 0.16412E+00_JPRB, 0.16346E+00_JPRB ,&
&  0.16397E+00_JPRB, 0.16407E+00_JPRB, 0.16545E+00_JPRB, 0.16565E+00_JPRB, 0.16536E+00_JPRB, 0.16521E+00_JPRB, 0.16495E+00_JPRB ,&
&  0.16462E+00_JPRB, 0.16386E+00_JPRB, 0.16347E+00_JPRB, 0.16290E+00_JPRB, 0.16233E+00_JPRB, 0.16137E+00_JPRB, 0.16035E+00_JPRB ,&
&  0.15989E+00_JPRB, 0.15995E+00_JPRB, 0.16032E+00_JPRB, 0.15867E+00_JPRB, 0.15793E+00_JPRB, 0.15745E+00_JPRB, 0.15721E+00_JPRB ,&
&  0.15680E+00_JPRB, 0.15668E+00_JPRB, 0.15668E+00_JPRB, 0.15662E+00_JPRB, 0.15790E+00_JPRB, 0.15829E+00_JPRB, 0.15859E+00_JPRB ,&
&  0.15864E+00_JPRB, 0.15849E+00_JPRB, 0.15782E+00_JPRB, 0.15720E+00_JPRB, 0.15657E+00_JPRB, 0.15547E+00_JPRB, 0.15432E+00_JPRB ,&
&  0.15335E+00_JPRB, 0.15260E+00_JPRB, 0.15175E+00_JPRB, 0.15111E+00_JPRB, 0.15058E+00_JPRB, 0.15008E+00_JPRB, 0.14976E+00_JPRB ,&
&  0.14968E+00_JPRB, 0.14970E+00_JPRB, 0.14969E+00_JPRB, 0.14968E+00_JPRB, 0.14964E+00_JPRB, 0.14961E+00_JPRB, 0.14961E+00_JPRB ,&
&  0.14993E+00_JPRB, 0.14992E+00_JPRB, 0.14976E+00_JPRB, 0.14948E+00_JPRB, 0.14936E+00_JPRB, 0.14905E+00_JPRB, 0.14884E+00_JPRB ,&
&  0.14867E+00_JPRB, 0.14845E+00_JPRB, 0.14836E+00_JPRB, 0.14834E+00_JPRB, 0.14834E+00_JPRB, 0.14834E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14858E+00_JPRB, 0.14878E+00_JPRB, 0.14914E+00_JPRB, 0.14943E+00_JPRB, 0.14970E+00_JPRB, 0.14993E+00_JPRB, 0.15042E+00_JPRB ,&
&  0.15094E+00_JPRB, 0.15145E+00_JPRB, 0.15195E+00_JPRB, 0.15229E+00_JPRB, 0.15303E+00_JPRB, 0.15378E+00_JPRB, 0.15459E+00_JPRB ,&
&  0.15546E+00_JPRB, 0.15656E+00_JPRB, 0.15770E+00_JPRB, 0.15885E+00_JPRB, 0.16037E+00_JPRB, 0.16160E+00_JPRB, 0.16280E+00_JPRB ,&
&  0.16414E+00_JPRB, 0.16660E+00_JPRB, 0.16726E+00_JPRB, 0.16738E+00_JPRB, 0.16685E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.16099E+00_JPRB, 0.16095E+00_JPRB, 0.16082E+00_JPRB, 0.16045E+00_JPRB, 0.15973E+00_JPRB, 0.15956E+00_JPRB, 0.15933E+00_JPRB ,&
&  0.15902E+00_JPRB, 0.15835E+00_JPRB, 0.15835E+00_JPRB, 0.15827E+00_JPRB, 0.15817E+00_JPRB, 0.15805E+00_JPRB, 0.15780E+00_JPRB ,&
&  0.15731E+00_JPRB, 0.15655E+00_JPRB, 0.15431E+00_JPRB, 0.15323E+00_JPRB, 0.15253E+00_JPRB, 0.15215E+00_JPRB, 0.15183E+00_JPRB ,&
&  0.15164E+00_JPRB, 0.15153E+00_JPRB, 0.15142E+00_JPRB, 0.15183E+00_JPRB, 0.15201E+00_JPRB, 0.15207E+00_JPRB, 0.15211E+00_JPRB ,&
&  0.15211E+00_JPRB, 0.15219E+00_JPRB, 0.15220E+00_JPRB, 0.15219E+00_JPRB, 0.15230E+00_JPRB, 0.15230E+00_JPRB, 0.15211E+00_JPRB ,&
&  0.15190E+00_JPRB, 0.15164E+00_JPRB, 0.15138E+00_JPRB, 0.15107E+00_JPRB, 0.15085E+00_JPRB, 0.15066E+00_JPRB, 0.15058E+00_JPRB ,&
&  0.15043E+00_JPRB, 0.15030E+00_JPRB, 0.15017E+00_JPRB, 0.15011E+00_JPRB, 0.15001E+00_JPRB, 0.14996E+00_JPRB, 0.14983E+00_JPRB ,&
&  0.14991E+00_JPRB, 0.14994E+00_JPRB, 0.15000E+00_JPRB, 0.15002E+00_JPRB, 0.15005E+00_JPRB, 0.15003E+00_JPRB, 0.14996E+00_JPRB ,&
&  0.14980E+00_JPRB, 0.14971E+00_JPRB, 0.14960E+00_JPRB, 0.14951E+00_JPRB, 0.14950E+00_JPRB, 0.14948E+00_JPRB, 0.14949E+00_JPRB ,&
&  0.14950E+00_JPRB, 0.14960E+00_JPRB, 0.14963E+00_JPRB, 0.14969E+00_JPRB, 0.14975E+00_JPRB, 0.14983E+00_JPRB, 0.14989E+00_JPRB ,&
&  0.14998E+00_JPRB, 0.15006E+00_JPRB, 0.15016E+00_JPRB, 0.15030E+00_JPRB, 0.15047E+00_JPRB, 0.15071E+00_JPRB, 0.15099E+00_JPRB ,&
&  0.15129E+00_JPRB, 0.15159E+00_JPRB, 0.15189E+00_JPRB, 0.15219E+00_JPRB, 0.15266E+00_JPRB, 0.15312E+00_JPRB, 0.15369E+00_JPRB ,&
&  0.15424E+00_JPRB, 0.15479E+00_JPRB, 0.15529E+00_JPRB, 0.15582E+00_JPRB, 0.15655E+00_JPRB, 0.15691E+00_JPRB, 0.15722E+00_JPRB ,&
&  0.15769E+00_JPRB, 0.15819E+00_JPRB, 0.15867E+00_JPRB, 0.15911E+00_JPRB, 0.15979E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB, 0.15156E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB, 0.50481E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.50672E+00_JPRB, 0.50563E+00_JPRB, 0.50472E+00_JPRB, 0.50366E+00_JPRB, 0.50230E+00_JPRB, 0.50117E+00_JPRB, 0.50008E+00_JPRB ,&
&  0.49918E+00_JPRB, 0.49865E+00_JPRB, 0.49838E+00_JPRB, 0.49827E+00_JPRB, 0.49836E+00_JPRB, 0.49843E+00_JPRB, 0.49853E+00_JPRB ,&
&  0.49854E+00_JPRB, 0.49864E+00_JPRB, 0.49869E+00_JPRB, 0.49849E+00_JPRB, 0.49813E+00_JPRB, 0.49767E+00_JPRB, 0.49702E+00_JPRB ,&
&  0.49603E+00_JPRB, 0.49509E+00_JPRB, 0.49425E+00_JPRB, 0.49335E+00_JPRB, 0.49256E+00_JPRB, 0.49172E+00_JPRB, 0.49080E+00_JPRB ,&
&  0.49003E+00_JPRB, 0.48969E+00_JPRB, 0.48949E+00_JPRB, 0.48929E+00_JPRB, 0.48936E+00_JPRB, 0.49026E+00_JPRB, 0.49109E+00_JPRB ,&
&  0.49202E+00_JPRB, 0.49314E+00_JPRB, 0.49500E+00_JPRB, 0.49684E+00_JPRB, 0.49857E+00_JPRB, 0.50012E+00_JPRB, 0.50127E+00_JPRB ,&
&  0.50153E+00_JPRB, 0.50132E+00_JPRB, 0.50079E+00_JPRB, 0.50019E+00_JPRB, 0.49910E+00_JPRB, 0.49817E+00_JPRB, 0.49740E+00_JPRB ,&
&  0.49646E+00_JPRB, 0.49567E+00_JPRB, 0.49487E+00_JPRB, 0.49368E+00_JPRB, 0.49133E+00_JPRB, 0.48960E+00_JPRB, 0.48863E+00_JPRB ,&
&  0.48831E+00_JPRB, 0.48806E+00_JPRB, 0.48816E+00_JPRB, 0.48853E+00_JPRB, 0.48942E+00_JPRB, 0.49050E+00_JPRB, 0.49184E+00_JPRB ,&
&  0.49333E+00_JPRB, 0.49521E+00_JPRB, 0.49773E+00_JPRB, 0.50037E+00_JPRB, 0.50323E+00_JPRB, 0.50650E+00_JPRB, 0.50947E+00_JPRB ,&
&  0.51243E+00_JPRB, 0.51553E+00_JPRB, 0.51820E+00_JPRB, 0.52092E+00_JPRB, 0.52369E+00_JPRB, 0.52664E+00_JPRB, 0.52894E+00_JPRB ,&
&  0.53024E+00_JPRB, 0.53123E+00_JPRB, 0.53201E+00_JPRB, 0.53183E+00_JPRB, 0.53101E+00_JPRB, 0.53035E+00_JPRB, 0.52963E+00_JPRB ,&
&  0.52866E+00_JPRB, 0.52735E+00_JPRB, 0.52610E+00_JPRB, 0.52467E+00_JPRB, 0.52265E+00_JPRB, 0.52047E+00_JPRB, 0.51839E+00_JPRB ,&
&  0.51633E+00_JPRB, 0.51408E+00_JPRB, 0.51207E+00_JPRB, 0.51015E+00_JPRB, 0.50839E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.51973E+00_JPRB, 0.51486E+00_JPRB, 0.51057E+00_JPRB, 0.50671E+00_JPRB, 0.50361E+00_JPRB, 0.50135E+00_JPRB, 0.49963E+00_JPRB ,&
&  0.49829E+00_JPRB, 0.49787E+00_JPRB, 0.49731E+00_JPRB, 0.49671E+00_JPRB, 0.49605E+00_JPRB, 0.49470E+00_JPRB, 0.49292E+00_JPRB ,&
&  0.49127E+00_JPRB, 0.49015E+00_JPRB, 0.48891E+00_JPRB, 0.48751E+00_JPRB, 0.48544E+00_JPRB, 0.48250E+00_JPRB, 0.47909E+00_JPRB ,&
&  0.47536E+00_JPRB, 0.47075E+00_JPRB, 0.46538E+00_JPRB, 0.45998E+00_JPRB, 0.45731E+00_JPRB, 0.45418E+00_JPRB, 0.45122E+00_JPRB ,&
&  0.44878E+00_JPRB, 0.44843E+00_JPRB, 0.44816E+00_JPRB, 0.44785E+00_JPRB, 0.44789E+00_JPRB, 0.44971E+00_JPRB, 0.45105E+00_JPRB ,&
&  0.45256E+00_JPRB, 0.45526E+00_JPRB, 0.46074E+00_JPRB, 0.46667E+00_JPRB, 0.47345E+00_JPRB, 0.48151E+00_JPRB, 0.49076E+00_JPRB ,&
&  0.49748E+00_JPRB, 0.50074E+00_JPRB, 0.50283E+00_JPRB, 0.50460E+00_JPRB, 0.50492E+00_JPRB, 0.50444E+00_JPRB, 0.50317E+00_JPRB ,&
&  0.50296E+00_JPRB, 0.50239E+00_JPRB, 0.50206E+00_JPRB, 0.50182E+00_JPRB, 0.49991E+00_JPRB, 0.49819E+00_JPRB, 0.49657E+00_JPRB ,&
&  0.49437E+00_JPRB, 0.49304E+00_JPRB, 0.49279E+00_JPRB, 0.49272E+00_JPRB, 0.49382E+00_JPRB, 0.49614E+00_JPRB, 0.49945E+00_JPRB ,&
&  0.50445E+00_JPRB, 0.51149E+00_JPRB, 0.52031E+00_JPRB, 0.53018E+00_JPRB, 0.54148E+00_JPRB, 0.55566E+00_JPRB, 0.56696E+00_JPRB ,&
&  0.57730E+00_JPRB, 0.58635E+00_JPRB, 0.59461E+00_JPRB, 0.60091E+00_JPRB, 0.60574E+00_JPRB, 0.61055E+00_JPRB, 0.61115E+00_JPRB ,&
&  0.61084E+00_JPRB, 0.60990E+00_JPRB, 0.60919E+00_JPRB, 0.60628E+00_JPRB, 0.60265E+00_JPRB, 0.59879E+00_JPRB, 0.59517E+00_JPRB ,&
&  0.58972E+00_JPRB, 0.58410E+00_JPRB, 0.57852E+00_JPRB, 0.57302E+00_JPRB, 0.56689E+00_JPRB, 0.56097E+00_JPRB, 0.55511E+00_JPRB ,&
&  0.54917E+00_JPRB, 0.54294E+00_JPRB, 0.53702E+00_JPRB, 0.53129E+00_JPRB, 0.52553E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.53383E+00_JPRB, 0.52541E+00_JPRB, 0.51786E+00_JPRB, 0.51132E+00_JPRB, 0.50580E+00_JPRB, 0.50174E+00_JPRB, 0.49883E+00_JPRB ,&
&  0.49679E+00_JPRB, 0.49617E+00_JPRB, 0.49546E+00_JPRB, 0.49427E+00_JPRB, 0.49268E+00_JPRB, 0.49239E+00_JPRB, 0.49075E+00_JPRB ,&
&  0.48803E+00_JPRB, 0.48428E+00_JPRB, 0.48039E+00_JPRB, 0.47566E+00_JPRB, 0.46955E+00_JPRB, 0.46275E+00_JPRB, 0.45452E+00_JPRB ,&
&  0.44878E+00_JPRB, 0.44086E+00_JPRB, 0.43203E+00_JPRB, 0.42231E+00_JPRB, 0.41555E+00_JPRB, 0.40018E+00_JPRB, 0.39421E+00_JPRB ,&
&  0.39005E+00_JPRB, 0.38298E+00_JPRB, 0.38026E+00_JPRB, 0.38081E+00_JPRB, 0.39557E+00_JPRB, 0.40690E+00_JPRB, 0.41216E+00_JPRB ,&
&  0.40053E+00_JPRB, 0.40150E+00_JPRB, 0.39905E+00_JPRB, 0.39769E+00_JPRB, 0.40011E+00_JPRB, 0.40652E+00_JPRB, 0.42883E+00_JPRB ,&
&  0.46812E+00_JPRB, 0.48297E+00_JPRB, 0.49043E+00_JPRB, 0.49625E+00_JPRB, 0.49818E+00_JPRB, 0.50011E+00_JPRB, 0.49957E+00_JPRB ,&
&  0.49970E+00_JPRB, 0.49902E+00_JPRB, 0.50058E+00_JPRB, 0.50168E+00_JPRB, 0.50372E+00_JPRB, 0.50565E+00_JPRB, 0.50747E+00_JPRB ,&
&  0.50465E+00_JPRB, 0.50277E+00_JPRB, 0.50163E+00_JPRB, 0.50260E+00_JPRB, 0.50527E+00_JPRB, 0.51163E+00_JPRB, 0.51978E+00_JPRB ,&
&  0.52908E+00_JPRB, 0.53656E+00_JPRB, 0.54925E+00_JPRB, 0.56654E+00_JPRB, 0.58722E+00_JPRB, 0.61291E+00_JPRB, 0.63905E+00_JPRB ,&
&  0.66524E+00_JPRB, 0.68729E+00_JPRB, 0.70902E+00_JPRB, 0.72086E+00_JPRB, 0.72825E+00_JPRB, 0.73367E+00_JPRB, 0.73463E+00_JPRB ,&
&  0.73407E+00_JPRB, 0.73087E+00_JPRB, 0.72702E+00_JPRB, 0.72123E+00_JPRB, 0.71338E+00_JPRB, 0.70251E+00_JPRB, 0.69019E+00_JPRB ,&
&  0.67568E+00_JPRB, 0.66139E+00_JPRB, 0.64714E+00_JPRB, 0.63304E+00_JPRB, 0.61909E+00_JPRB, 0.60626E+00_JPRB, 0.59481E+00_JPRB ,&
&  0.58391E+00_JPRB, 0.57304E+00_JPRB, 0.56301E+00_JPRB, 0.55335E+00_JPRB, 0.54363E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.54861E+00_JPRB, 0.53899E+00_JPRB, 0.52882E+00_JPRB, 0.51981E+00_JPRB, 0.51238E+00_JPRB, 0.50701E+00_JPRB, 0.50350E+00_JPRB ,&
&  0.50101E+00_JPRB, 0.49796E+00_JPRB, 0.49478E+00_JPRB, 0.49138E+00_JPRB, 0.48795E+00_JPRB, 0.48717E+00_JPRB, 0.48631E+00_JPRB ,&
&  0.48537E+00_JPRB, 0.48125E+00_JPRB, 0.47516E+00_JPRB, 0.46679E+00_JPRB, 0.45815E+00_JPRB, 0.45100E+00_JPRB, 0.44383E+00_JPRB ,&
&  0.43527E+00_JPRB, 0.42575E+00_JPRB, 0.40950E+00_JPRB, 0.39815E+00_JPRB, 0.38178E+00_JPRB, 0.37181E+00_JPRB, 0.36658E+00_JPRB ,&
&  0.36386E+00_JPRB, 0.36577E+00_JPRB, 0.37048E+00_JPRB, 0.37478E+00_JPRB, 0.37242E+00_JPRB, 0.36744E+00_JPRB, 0.36462E+00_JPRB ,&
&  0.35881E+00_JPRB, 0.35513E+00_JPRB, 0.35256E+00_JPRB, 0.35208E+00_JPRB, 0.35420E+00_JPRB, 0.36098E+00_JPRB, 0.37494E+00_JPRB ,&
&  0.40727E+00_JPRB, 0.44900E+00_JPRB, 0.48781E+00_JPRB, 0.50467E+00_JPRB, 0.49760E+00_JPRB, 0.49926E+00_JPRB, 0.49558E+00_JPRB ,&
&  0.49381E+00_JPRB, 0.49044E+00_JPRB, 0.48974E+00_JPRB, 0.49391E+00_JPRB, 0.49942E+00_JPRB, 0.50483E+00_JPRB, 0.51049E+00_JPRB ,&
&  0.51216E+00_JPRB, 0.51445E+00_JPRB, 0.51050E+00_JPRB, 0.51194E+00_JPRB, 0.51864E+00_JPRB, 0.52654E+00_JPRB, 0.52997E+00_JPRB ,&
&  0.53729E+00_JPRB, 0.54796E+00_JPRB, 0.56927E+00_JPRB, 0.60164E+00_JPRB, 0.63626E+00_JPRB, 0.66845E+00_JPRB, 0.69815E+00_JPRB ,&
&  0.72737E+00_JPRB, 0.76369E+00_JPRB, 0.79902E+00_JPRB, 0.81418E+00_JPRB, 0.81576E+00_JPRB, 0.82136E+00_JPRB, 0.81710E+00_JPRB ,&
&  0.81416E+00_JPRB, 0.80813E+00_JPRB, 0.80599E+00_JPRB, 0.80366E+00_JPRB, 0.79367E+00_JPRB, 0.77654E+00_JPRB, 0.75840E+00_JPRB ,&
&  0.73858E+00_JPRB, 0.71628E+00_JPRB, 0.69496E+00_JPRB, 0.67312E+00_JPRB, 0.65267E+00_JPRB, 0.63264E+00_JPRB, 0.61587E+00_JPRB ,&
&  0.59973E+00_JPRB, 0.58726E+00_JPRB, 0.57579E+00_JPRB, 0.56646E+00_JPRB, 0.55765E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.56427E+00_JPRB, 0.55795E+00_JPRB, 0.55150E+00_JPRB, 0.54310E+00_JPRB, 0.53453E+00_JPRB, 0.52679E+00_JPRB, 0.52090E+00_JPRB ,&
&  0.51627E+00_JPRB, 0.51144E+00_JPRB, 0.50690E+00_JPRB, 0.50124E+00_JPRB, 0.49595E+00_JPRB, 0.49141E+00_JPRB, 0.48766E+00_JPRB ,&
&  0.48529E+00_JPRB, 0.47926E+00_JPRB, 0.47282E+00_JPRB, 0.46479E+00_JPRB, 0.45727E+00_JPRB, 0.44902E+00_JPRB, 0.44051E+00_JPRB ,&
&  0.43332E+00_JPRB, 0.42059E+00_JPRB, 0.40530E+00_JPRB, 0.39555E+00_JPRB, 0.38018E+00_JPRB, 0.37401E+00_JPRB, 0.37323E+00_JPRB ,&
&  0.39002E+00_JPRB, 0.38358E+00_JPRB, 0.38795E+00_JPRB, 0.37521E+00_JPRB, 0.36400E+00_JPRB, 0.35352E+00_JPRB, 0.34592E+00_JPRB ,&
&  0.33999E+00_JPRB, 0.33516E+00_JPRB, 0.33469E+00_JPRB, 0.33676E+00_JPRB, 0.33856E+00_JPRB, 0.34665E+00_JPRB, 0.35989E+00_JPRB ,&
&  0.39542E+00_JPRB, 0.48438E+00_JPRB, 0.49661E+00_JPRB, 0.51144E+00_JPRB, 0.50881E+00_JPRB, 0.50884E+00_JPRB, 0.50923E+00_JPRB ,&
&  0.50831E+00_JPRB, 0.50622E+00_JPRB, 0.50137E+00_JPRB, 0.49719E+00_JPRB, 0.50501E+00_JPRB, 0.51135E+00_JPRB, 0.52027E+00_JPRB ,&
&  0.52378E+00_JPRB, 0.52805E+00_JPRB, 0.52788E+00_JPRB, 0.53021E+00_JPRB, 0.53954E+00_JPRB, 0.54322E+00_JPRB, 0.54177E+00_JPRB ,&
&  0.55299E+00_JPRB, 0.57067E+00_JPRB, 0.59856E+00_JPRB, 0.63378E+00_JPRB, 0.65994E+00_JPRB, 0.68248E+00_JPRB, 0.71647E+00_JPRB ,&
&  0.74075E+00_JPRB, 0.78082E+00_JPRB, 0.82392E+00_JPRB, 0.84420E+00_JPRB, 0.84903E+00_JPRB, 0.84573E+00_JPRB, 0.83995E+00_JPRB ,&
&  0.83995E+00_JPRB, 0.82870E+00_JPRB, 0.82352E+00_JPRB, 0.82115E+00_JPRB, 0.80882E+00_JPRB, 0.79166E+00_JPRB, 0.77360E+00_JPRB ,&
&  0.75157E+00_JPRB, 0.73152E+00_JPRB, 0.71425E+00_JPRB, 0.69437E+00_JPRB, 0.67371E+00_JPRB, 0.65363E+00_JPRB, 0.63418E+00_JPRB ,&
&  0.61535E+00_JPRB, 0.60013E+00_JPRB, 0.58605E+00_JPRB, 0.57707E+00_JPRB, 0.56916E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.57928E+00_JPRB, 0.57325E+00_JPRB, 0.56774E+00_JPRB, 0.56235E+00_JPRB, 0.55609E+00_JPRB, 0.54950E+00_JPRB, 0.54271E+00_JPRB ,&
&  0.53690E+00_JPRB, 0.53150E+00_JPRB, 0.52611E+00_JPRB, 0.51885E+00_JPRB, 0.51153E+00_JPRB, 0.50335E+00_JPRB, 0.49725E+00_JPRB ,&
&  0.49107E+00_JPRB, 0.48344E+00_JPRB, 0.47171E+00_JPRB, 0.46104E+00_JPRB, 0.45349E+00_JPRB, 0.44741E+00_JPRB, 0.43954E+00_JPRB ,&
&  0.43280E+00_JPRB, 0.42295E+00_JPRB, 0.40540E+00_JPRB, 0.38883E+00_JPRB, 0.37494E+00_JPRB, 0.37451E+00_JPRB, 0.40092E+00_JPRB ,&
&  0.41407E+00_JPRB, 0.41707E+00_JPRB, 0.41928E+00_JPRB, 0.41483E+00_JPRB, 0.40895E+00_JPRB, 0.36213E+00_JPRB, 0.35885E+00_JPRB ,&
&  0.34461E+00_JPRB, 0.33413E+00_JPRB, 0.33125E+00_JPRB, 0.33103E+00_JPRB, 0.33527E+00_JPRB, 0.34623E+00_JPRB, 0.36270E+00_JPRB ,&
&  0.39682E+00_JPRB, 0.49562E+00_JPRB, 0.51041E+00_JPRB, 0.52472E+00_JPRB, 0.52593E+00_JPRB, 0.52643E+00_JPRB, 0.52958E+00_JPRB ,&
&  0.52908E+00_JPRB, 0.53083E+00_JPRB, 0.53245E+00_JPRB, 0.53600E+00_JPRB, 0.53999E+00_JPRB, 0.53885E+00_JPRB, 0.54557E+00_JPRB ,&
&  0.55042E+00_JPRB, 0.55568E+00_JPRB, 0.55642E+00_JPRB, 0.55230E+00_JPRB, 0.55677E+00_JPRB, 0.55091E+00_JPRB, 0.54809E+00_JPRB ,&
&  0.57043E+00_JPRB, 0.60684E+00_JPRB, 0.63909E+00_JPRB, 0.66993E+00_JPRB, 0.69653E+00_JPRB, 0.72698E+00_JPRB, 0.76470E+00_JPRB ,&
&  0.79086E+00_JPRB, 0.82820E+00_JPRB, 0.87301E+00_JPRB, 0.88859E+00_JPRB, 0.88812E+00_JPRB, 0.88682E+00_JPRB, 0.89088E+00_JPRB ,&
&  0.89586E+00_JPRB, 0.90286E+00_JPRB, 0.87919E+00_JPRB, 0.86538E+00_JPRB, 0.83605E+00_JPRB, 0.79610E+00_JPRB, 0.76332E+00_JPRB ,&
&  0.72343E+00_JPRB, 0.68902E+00_JPRB, 0.66787E+00_JPRB, 0.65272E+00_JPRB, 0.64138E+00_JPRB, 0.63358E+00_JPRB, 0.62660E+00_JPRB ,&
&  0.62000E+00_JPRB, 0.61152E+00_JPRB, 0.60236E+00_JPRB, 0.59326E+00_JPRB, 0.58554E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.58770E+00_JPRB, 0.58432E+00_JPRB, 0.57904E+00_JPRB, 0.57406E+00_JPRB, 0.56689E+00_JPRB, 0.56134E+00_JPRB, 0.55597E+00_JPRB ,&
&  0.55222E+00_JPRB, 0.54977E+00_JPRB, 0.54729E+00_JPRB, 0.54429E+00_JPRB, 0.53996E+00_JPRB, 0.53558E+00_JPRB, 0.52922E+00_JPRB ,&
&  0.51902E+00_JPRB, 0.49010E+00_JPRB, 0.46872E+00_JPRB, 0.46010E+00_JPRB, 0.45254E+00_JPRB, 0.44685E+00_JPRB, 0.44246E+00_JPRB ,&
&  0.43354E+00_JPRB, 0.42408E+00_JPRB, 0.41107E+00_JPRB, 0.39150E+00_JPRB, 0.38266E+00_JPRB, 0.38756E+00_JPRB, 0.39581E+00_JPRB ,&
&  0.40990E+00_JPRB, 0.41967E+00_JPRB, 0.41439E+00_JPRB, 0.41396E+00_JPRB, 0.42920E+00_JPRB, 0.42402E+00_JPRB, 0.37591E+00_JPRB ,&
&  0.35151E+00_JPRB, 0.33587E+00_JPRB, 0.33065E+00_JPRB, 0.32942E+00_JPRB, 0.33413E+00_JPRB, 0.34897E+00_JPRB, 0.38822E+00_JPRB ,&
&  0.46314E+00_JPRB, 0.54599E+00_JPRB, 0.53722E+00_JPRB, 0.53489E+00_JPRB, 0.54048E+00_JPRB, 0.54434E+00_JPRB, 0.54854E+00_JPRB ,&
&  0.55241E+00_JPRB, 0.56053E+00_JPRB, 0.57152E+00_JPRB, 0.58176E+00_JPRB, 0.59446E+00_JPRB, 0.60527E+00_JPRB, 0.60947E+00_JPRB ,&
&  0.61701E+00_JPRB, 0.60153E+00_JPRB, 0.59447E+00_JPRB, 0.58319E+00_JPRB, 0.57814E+00_JPRB, 0.56496E+00_JPRB, 0.57067E+00_JPRB ,&
&  0.59912E+00_JPRB, 0.63377E+00_JPRB, 0.67051E+00_JPRB, 0.69984E+00_JPRB, 0.73110E+00_JPRB, 0.77414E+00_JPRB, 0.82123E+00_JPRB ,&
&  0.86302E+00_JPRB, 0.91959E+00_JPRB, 0.10020E+01_JPRB, 0.98922E+00_JPRB, 0.97316E+00_JPRB, 0.96528E+00_JPRB, 0.96917E+00_JPRB ,&
&  0.97007E+00_JPRB, 0.98252E+00_JPRB, 0.94113E+00_JPRB, 0.87792E+00_JPRB, 0.82265E+00_JPRB, 0.75569E+00_JPRB, 0.71483E+00_JPRB ,&
&  0.68778E+00_JPRB, 0.64706E+00_JPRB, 0.62683E+00_JPRB, 0.61586E+00_JPRB, 0.60936E+00_JPRB, 0.60131E+00_JPRB, 0.59443E+00_JPRB ,&
&  0.59187E+00_JPRB, 0.59117E+00_JPRB, 0.59215E+00_JPRB, 0.59112E+00_JPRB, 0.58986E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.57739E+00_JPRB, 0.58161E+00_JPRB, 0.58415E+00_JPRB, 0.58185E+00_JPRB, 0.56836E+00_JPRB, 0.54120E+00_JPRB, 0.52649E+00_JPRB ,&
&  0.51799E+00_JPRB, 0.51787E+00_JPRB, 0.51628E+00_JPRB, 0.51240E+00_JPRB, 0.51205E+00_JPRB, 0.51709E+00_JPRB, 0.52270E+00_JPRB ,&
&  0.52439E+00_JPRB, 0.51835E+00_JPRB, 0.50646E+00_JPRB, 0.48484E+00_JPRB, 0.46664E+00_JPRB, 0.45223E+00_JPRB, 0.44501E+00_JPRB ,&
&  0.44274E+00_JPRB, 0.43682E+00_JPRB, 0.42615E+00_JPRB, 0.41109E+00_JPRB, 0.39941E+00_JPRB, 0.38917E+00_JPRB, 0.39150E+00_JPRB ,&
&  0.40232E+00_JPRB, 0.40786E+00_JPRB, 0.41522E+00_JPRB, 0.42156E+00_JPRB, 0.42961E+00_JPRB, 0.45152E+00_JPRB, 0.40150E+00_JPRB ,&
&  0.36711E+00_JPRB, 0.35200E+00_JPRB, 0.34043E+00_JPRB, 0.33721E+00_JPRB, 0.34392E+00_JPRB, 0.36516E+00_JPRB, 0.41136E+00_JPRB ,&
&  0.52593E+00_JPRB, 0.54165E+00_JPRB, 0.54719E+00_JPRB, 0.55910E+00_JPRB, 0.56846E+00_JPRB, 0.57158E+00_JPRB, 0.57866E+00_JPRB ,&
&  0.59310E+00_JPRB, 0.61770E+00_JPRB, 0.64251E+00_JPRB, 0.64219E+00_JPRB, 0.63792E+00_JPRB, 0.62334E+00_JPRB, 0.68491E+00_JPRB ,&
&  0.87128E+00_JPRB, 0.80246E+00_JPRB, 0.70527E+00_JPRB, 0.63978E+00_JPRB, 0.60725E+00_JPRB, 0.59295E+00_JPRB, 0.60870E+00_JPRB ,&
&  0.63508E+00_JPRB, 0.66775E+00_JPRB, 0.71502E+00_JPRB, 0.75120E+00_JPRB, 0.78694E+00_JPRB, 0.83888E+00_JPRB, 0.91248E+00_JPRB ,&
&  0.98669E+00_JPRB, 0.10889E+01_JPRB, 0.13285E+01_JPRB, 0.11587E+01_JPRB, 0.10670E+01_JPRB, 0.10381E+01_JPRB, 0.10293E+01_JPRB ,&
&  0.99913E+00_JPRB, 0.10025E+01_JPRB, 0.93670E+00_JPRB, 0.84540E+00_JPRB, 0.79643E+00_JPRB, 0.74261E+00_JPRB, 0.70381E+00_JPRB ,&
&  0.66693E+00_JPRB, 0.63434E+00_JPRB, 0.61851E+00_JPRB, 0.60501E+00_JPRB, 0.60140E+00_JPRB, 0.59704E+00_JPRB, 0.59089E+00_JPRB ,&
&  0.58323E+00_JPRB, 0.57686E+00_JPRB, 0.57239E+00_JPRB, 0.57019E+00_JPRB, 0.57124E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.54824E+00_JPRB, 0.57423E+00_JPRB, 0.59983E+00_JPRB, 0.58605E+00_JPRB, 0.56541E+00_JPRB, 0.51426E+00_JPRB, 0.47717E+00_JPRB ,&
&  0.45364E+00_JPRB, 0.44966E+00_JPRB, 0.44908E+00_JPRB, 0.45158E+00_JPRB, 0.45743E+00_JPRB, 0.46726E+00_JPRB, 0.47355E+00_JPRB ,&
&  0.47964E+00_JPRB, 0.47713E+00_JPRB, 0.47600E+00_JPRB, 0.46988E+00_JPRB, 0.45871E+00_JPRB, 0.44451E+00_JPRB, 0.43445E+00_JPRB ,&
&  0.42967E+00_JPRB, 0.42825E+00_JPRB, 0.42422E+00_JPRB, 0.41729E+00_JPRB, 0.41195E+00_JPRB, 0.40417E+00_JPRB, 0.40851E+00_JPRB ,&
&  0.40264E+00_JPRB, 0.40176E+00_JPRB, 0.41425E+00_JPRB, 0.42815E+00_JPRB, 0.43926E+00_JPRB, 0.46067E+00_JPRB, 0.42522E+00_JPRB ,&
&  0.38522E+00_JPRB, 0.36638E+00_JPRB, 0.35893E+00_JPRB, 0.36395E+00_JPRB, 0.40312E+00_JPRB, 0.50875E+00_JPRB, 0.54264E+00_JPRB ,&
&  0.55968E+00_JPRB, 0.57308E+00_JPRB, 0.58842E+00_JPRB, 0.60865E+00_JPRB, 0.61306E+00_JPRB, 0.61763E+00_JPRB, 0.63348E+00_JPRB ,&
&  0.65859E+00_JPRB, 0.69868E+00_JPRB, 0.73788E+00_JPRB, 0.64187E+00_JPRB, 0.60197E+00_JPRB, 0.62252E+00_JPRB, 0.65693E+00_JPRB ,&
&  0.70512E+00_JPRB, 0.69550E+00_JPRB, 0.68173E+00_JPRB, 0.69999E+00_JPRB, 0.70077E+00_JPRB, 0.70263E+00_JPRB, 0.69567E+00_JPRB ,&
&  0.69912E+00_JPRB, 0.72725E+00_JPRB, 0.79814E+00_JPRB, 0.83606E+00_JPRB, 0.89606E+00_JPRB, 0.98671E+00_JPRB, 0.10666E+01_JPRB ,&
&  0.11570E+01_JPRB, 0.12232E+01_JPRB, 0.12541E+01_JPRB, 0.12066E+01_JPRB, 0.11712E+01_JPRB, 0.11699E+01_JPRB, 0.11487E+01_JPRB ,&
&  0.10647E+01_JPRB, 0.10052E+01_JPRB, 0.95516E+00_JPRB, 0.90367E+00_JPRB, 0.82844E+00_JPRB, 0.76672E+00_JPRB, 0.70303E+00_JPRB ,&
&  0.64149E+00_JPRB, 0.59674E+00_JPRB, 0.57488E+00_JPRB, 0.56257E+00_JPRB, 0.55998E+00_JPRB, 0.55839E+00_JPRB, 0.54994E+00_JPRB ,&
&  0.53613E+00_JPRB, 0.52113E+00_JPRB, 0.51567E+00_JPRB, 0.52128E+00_JPRB, 0.53885E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.60090E+00_JPRB, 0.59845E+00_JPRB, 0.56780E+00_JPRB, 0.57963E+00_JPRB, 0.55427E+00_JPRB, 0.53512E+00_JPRB, 0.50447E+00_JPRB ,&
&  0.48913E+00_JPRB, 0.46157E+00_JPRB, 0.44384E+00_JPRB, 0.41794E+00_JPRB, 0.40241E+00_JPRB, 0.39283E+00_JPRB, 0.40468E+00_JPRB ,&
&  0.42203E+00_JPRB, 0.43407E+00_JPRB, 0.44032E+00_JPRB, 0.43989E+00_JPRB, 0.43725E+00_JPRB, 0.43077E+00_JPRB, 0.42410E+00_JPRB ,&
&  0.41882E+00_JPRB, 0.42056E+00_JPRB, 0.41885E+00_JPRB, 0.41685E+00_JPRB, 0.41385E+00_JPRB, 0.41280E+00_JPRB, 0.41462E+00_JPRB ,&
&  0.40748E+00_JPRB, 0.40117E+00_JPRB, 0.40553E+00_JPRB, 0.42569E+00_JPRB, 0.44227E+00_JPRB, 0.46146E+00_JPRB, 0.43396E+00_JPRB ,&
&  0.39724E+00_JPRB, 0.38935E+00_JPRB, 0.40257E+00_JPRB, 0.50175E+00_JPRB, 0.52205E+00_JPRB, 0.52897E+00_JPRB, 0.55443E+00_JPRB ,&
&  0.53392E+00_JPRB, 0.53014E+00_JPRB, 0.54659E+00_JPRB, 0.61673E+00_JPRB, 0.63964E+00_JPRB, 0.65240E+00_JPRB, 0.67509E+00_JPRB ,&
&  0.70694E+00_JPRB, 0.75405E+00_JPRB, 0.80851E+00_JPRB, 0.71983E+00_JPRB, 0.75498E+00_JPRB, 0.89244E+00_JPRB, 0.80195E+00_JPRB ,&
&  0.76648E+00_JPRB, 0.74341E+00_JPRB, 0.76588E+00_JPRB, 0.77698E+00_JPRB, 0.76913E+00_JPRB, 0.79004E+00_JPRB, 0.80603E+00_JPRB ,&
&  0.78936E+00_JPRB, 0.82437E+00_JPRB, 0.91359E+00_JPRB, 0.10048E+01_JPRB, 0.10875E+01_JPRB, 0.11962E+01_JPRB, 0.13095E+01_JPRB ,&
&  0.14495E+01_JPRB, 0.15760E+01_JPRB, 0.16671E+01_JPRB, 0.16541E+01_JPRB, 0.16125E+01_JPRB, 0.15336E+01_JPRB, 0.14199E+01_JPRB ,&
&  0.12620E+01_JPRB, 0.11479E+01_JPRB, 0.10701E+01_JPRB, 0.10084E+01_JPRB, 0.93141E+00_JPRB, 0.83112E+00_JPRB, 0.70294E+00_JPRB ,&
&  0.60666E+00_JPRB, 0.54454E+00_JPRB, 0.51949E+00_JPRB, 0.51331E+00_JPRB, 0.51745E+00_JPRB, 0.52115E+00_JPRB, 0.51614E+00_JPRB ,&
&  0.50709E+00_JPRB, 0.50167E+00_JPRB, 0.50145E+00_JPRB, 0.51827E+00_JPRB, 0.56759E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.61555E+00_JPRB, 0.60198E+00_JPRB, 0.58617E+00_JPRB, 0.58175E+00_JPRB, 0.58660E+00_JPRB, 0.55660E+00_JPRB, 0.49805E+00_JPRB ,&
&  0.51291E+00_JPRB, 0.46500E+00_JPRB, 0.45189E+00_JPRB, 0.43907E+00_JPRB, 0.42704E+00_JPRB, 0.40177E+00_JPRB, 0.38005E+00_JPRB ,&
&  0.38046E+00_JPRB, 0.39410E+00_JPRB, 0.41000E+00_JPRB, 0.42285E+00_JPRB, 0.42650E+00_JPRB, 0.42271E+00_JPRB, 0.41751E+00_JPRB ,&
&  0.41387E+00_JPRB, 0.41515E+00_JPRB, 0.41917E+00_JPRB, 0.42485E+00_JPRB, 0.42600E+00_JPRB, 0.42667E+00_JPRB, 0.43056E+00_JPRB ,&
&  0.43298E+00_JPRB, 0.43327E+00_JPRB, 0.42565E+00_JPRB, 0.43491E+00_JPRB, 0.44167E+00_JPRB, 0.45717E+00_JPRB, 0.48401E+00_JPRB ,&
&  0.42021E+00_JPRB, 0.42568E+00_JPRB, 0.51715E+00_JPRB, 0.52124E+00_JPRB, 0.53342E+00_JPRB, 0.54672E+00_JPRB, 0.55674E+00_JPRB ,&
&  0.57191E+00_JPRB, 0.59660E+00_JPRB, 0.62708E+00_JPRB, 0.65433E+00_JPRB, 0.67762E+00_JPRB, 0.70691E+00_JPRB, 0.73016E+00_JPRB ,&
&  0.79569E+00_JPRB, 0.71672E+00_JPRB, 0.67681E+00_JPRB, 0.72245E+00_JPRB, 0.99302E+00_JPRB, 0.10001E+01_JPRB, 0.97308E+00_JPRB ,&
&  0.96041E+00_JPRB, 0.90949E+00_JPRB, 0.89228E+00_JPRB, 0.91066E+00_JPRB, 0.92195E+00_JPRB, 0.96680E+00_JPRB, 0.10310E+01_JPRB ,&
&  0.10295E+01_JPRB, 0.10964E+01_JPRB, 0.11864E+01_JPRB, 0.12684E+01_JPRB, 0.13585E+01_JPRB, 0.14934E+01_JPRB, 0.16455E+01_JPRB ,&
&  0.18096E+01_JPRB, 0.19520E+01_JPRB, 0.20174E+01_JPRB, 0.19894E+01_JPRB, 0.18988E+01_JPRB, 0.17790E+01_JPRB, 0.16294E+01_JPRB ,&
&  0.14356E+01_JPRB, 0.12965E+01_JPRB, 0.12019E+01_JPRB, 0.11097E+01_JPRB, 0.10261E+01_JPRB, 0.91603E+00_JPRB, 0.75843E+00_JPRB ,&
&  0.59584E+00_JPRB, 0.52583E+00_JPRB, 0.51318E+00_JPRB, 0.51149E+00_JPRB, 0.51570E+00_JPRB, 0.52413E+00_JPRB, 0.53236E+00_JPRB ,&
&  0.53630E+00_JPRB, 0.52817E+00_JPRB, 0.52889E+00_JPRB, 0.55301E+00_JPRB, 0.58295E+00_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.58612E+00_JPRB, 0.57838E+00_JPRB, 0.57637E+00_JPRB, 0.59384E+00_JPRB, 0.59435E+00_JPRB, 0.56297E+00_JPRB, 0.52320E+00_JPRB ,&
&  0.62970E+00_JPRB, 0.58412E+00_JPRB, 0.64370E+00_JPRB, 0.50674E+00_JPRB, 0.45224E+00_JPRB, 0.43579E+00_JPRB, 0.41579E+00_JPRB ,&
&  0.40015E+00_JPRB, 0.39343E+00_JPRB, 0.39244E+00_JPRB, 0.40231E+00_JPRB, 0.40812E+00_JPRB, 0.40634E+00_JPRB, 0.40390E+00_JPRB ,&
&  0.40270E+00_JPRB, 0.40640E+00_JPRB, 0.41632E+00_JPRB, 0.42262E+00_JPRB, 0.42770E+00_JPRB, 0.43292E+00_JPRB, 0.44225E+00_JPRB ,&
&  0.43012E+00_JPRB, 0.42772E+00_JPRB, 0.43428E+00_JPRB, 0.43804E+00_JPRB, 0.44787E+00_JPRB, 0.45369E+00_JPRB, 0.47016E+00_JPRB ,&
&  0.51547E+00_JPRB, 0.48483E+00_JPRB, 0.55902E+00_JPRB, 0.56270E+00_JPRB, 0.57074E+00_JPRB, 0.57959E+00_JPRB, 0.58761E+00_JPRB ,&
&  0.59573E+00_JPRB, 0.61304E+00_JPRB, 0.64014E+00_JPRB, 0.66439E+00_JPRB, 0.70085E+00_JPRB, 0.74419E+00_JPRB, 0.77421E+00_JPRB ,&
&  0.86494E+00_JPRB, 0.70803E+00_JPRB, 0.77078E+00_JPRB, 0.80487E+00_JPRB, 0.10080E+01_JPRB, 0.12387E+01_JPRB, 0.11833E+01_JPRB ,&
&  0.13233E+01_JPRB, 0.12196E+01_JPRB, 0.11385E+01_JPRB, 0.10751E+01_JPRB, 0.10955E+01_JPRB, 0.11490E+01_JPRB, 0.12189E+01_JPRB ,&
&  0.12828E+01_JPRB, 0.13541E+01_JPRB, 0.14769E+01_JPRB, 0.15859E+01_JPRB, 0.16997E+01_JPRB, 0.18320E+01_JPRB, 0.19588E+01_JPRB ,&
&  0.20733E+01_JPRB, 0.21457E+01_JPRB, 0.21857E+01_JPRB, 0.20652E+01_JPRB, 0.19575E+01_JPRB, 0.18417E+01_JPRB, 0.16172E+01_JPRB ,&
&  0.14435E+01_JPRB, 0.13098E+01_JPRB, 0.11995E+01_JPRB, 0.11177E+01_JPRB, 0.97481E+00_JPRB, 0.85599E+00_JPRB, 0.74360E+00_JPRB ,&
&  0.59908E+00_JPRB, 0.54676E+00_JPRB, 0.55819E+00_JPRB, 0.56439E+00_JPRB, 0.55950E+00_JPRB, 0.56292E+00_JPRB, 0.57151E+00_JPRB ,&
&  0.56187E+00_JPRB, 0.56238E+00_JPRB, 0.58574E+00_JPRB, 0.59054E+00_JPRB, 0.58952E+00_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.60730E+00_JPRB, 0.61354E+00_JPRB, 0.61904E+00_JPRB, 0.62300E+00_JPRB, 0.60679E+00_JPRB, 0.62482E+00_JPRB, 0.56996E+00_JPRB ,&
&  0.62651E+00_JPRB, 0.62730E+00_JPRB, 0.60914E+00_JPRB, 0.62287E+00_JPRB, 0.66412E+00_JPRB, 0.62313E+00_JPRB, 0.48999E+00_JPRB ,&
&  0.43199E+00_JPRB, 0.42496E+00_JPRB, 0.42641E+00_JPRB, 0.42661E+00_JPRB, 0.42759E+00_JPRB, 0.42588E+00_JPRB, 0.42541E+00_JPRB ,&
&  0.42552E+00_JPRB, 0.42497E+00_JPRB, 0.43090E+00_JPRB, 0.43721E+00_JPRB, 0.44003E+00_JPRB, 0.44336E+00_JPRB, 0.45399E+00_JPRB ,&
&  0.43830E+00_JPRB, 0.42786E+00_JPRB, 0.42297E+00_JPRB, 0.41261E+00_JPRB, 0.43196E+00_JPRB, 0.44777E+00_JPRB, 0.47453E+00_JPRB ,&
&  0.51380E+00_JPRB, 0.55377E+00_JPRB, 0.57345E+00_JPRB, 0.58940E+00_JPRB, 0.60304E+00_JPRB, 0.61780E+00_JPRB, 0.63358E+00_JPRB ,&
&  0.64846E+00_JPRB, 0.66697E+00_JPRB, 0.69389E+00_JPRB, 0.71434E+00_JPRB, 0.77465E+00_JPRB, 0.76366E+00_JPRB, 0.85746E+00_JPRB ,&
&  0.91794E+00_JPRB, 0.10419E+01_JPRB, 0.11215E+01_JPRB, 0.98901E+00_JPRB, 0.11442E+01_JPRB, 0.13190E+01_JPRB, 0.12989E+01_JPRB ,&
&  0.13159E+01_JPRB, 0.12862E+01_JPRB, 0.13100E+01_JPRB, 0.13571E+01_JPRB, 0.14966E+01_JPRB, 0.16000E+01_JPRB, 0.16485E+01_JPRB ,&
&  0.16954E+01_JPRB, 0.18866E+01_JPRB, 0.20149E+01_JPRB, 0.21004E+01_JPRB, 0.21410E+01_JPRB, 0.21756E+01_JPRB, 0.22307E+01_JPRB ,&
&  0.22707E+01_JPRB, 0.23054E+01_JPRB, 0.22935E+01_JPRB, 0.22155E+01_JPRB, 0.20985E+01_JPRB, 0.19381E+01_JPRB, 0.17221E+01_JPRB ,&
&  0.15286E+01_JPRB, 0.12892E+01_JPRB, 0.10952E+01_JPRB, 0.97473E+00_JPRB, 0.86211E+00_JPRB, 0.75158E+00_JPRB, 0.67459E+00_JPRB ,&
&  0.59858E+00_JPRB, 0.59140E+00_JPRB, 0.64522E+00_JPRB, 0.64097E+00_JPRB, 0.63987E+00_JPRB, 0.63701E+00_JPRB, 0.59078E+00_JPRB ,&
&  0.57699E+00_JPRB, 0.60617E+00_JPRB, 0.61369E+00_JPRB, 0.61103E+00_JPRB, 0.60722E+00_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.66885E+00_JPRB, 0.66650E+00_JPRB, 0.66714E+00_JPRB, 0.65674E+00_JPRB, 0.64566E+00_JPRB, 0.64372E+00_JPRB, 0.64369E+00_JPRB ,&
&  0.63920E+00_JPRB, 0.64167E+00_JPRB, 0.63942E+00_JPRB, 0.63002E+00_JPRB, 0.62690E+00_JPRB, 0.64688E+00_JPRB, 0.58633E+00_JPRB ,&
&  0.49251E+00_JPRB, 0.46726E+00_JPRB, 0.45790E+00_JPRB, 0.46399E+00_JPRB, 0.47656E+00_JPRB, 0.48145E+00_JPRB, 0.48336E+00_JPRB ,&
&  0.48738E+00_JPRB, 0.48165E+00_JPRB, 0.48532E+00_JPRB, 0.48675E+00_JPRB, 0.48396E+00_JPRB, 0.48530E+00_JPRB, 0.48685E+00_JPRB ,&
&  0.45836E+00_JPRB, 0.43800E+00_JPRB, 0.42701E+00_JPRB, 0.42054E+00_JPRB, 0.43453E+00_JPRB, 0.46073E+00_JPRB, 0.50035E+00_JPRB ,&
&  0.54752E+00_JPRB, 0.58203E+00_JPRB, 0.61200E+00_JPRB, 0.64198E+00_JPRB, 0.66586E+00_JPRB, 0.69183E+00_JPRB, 0.71571E+00_JPRB ,&
&  0.73469E+00_JPRB, 0.75089E+00_JPRB, 0.76105E+00_JPRB, 0.77839E+00_JPRB, 0.77914E+00_JPRB, 0.86244E+00_JPRB, 0.10416E+01_JPRB ,&
&  0.11064E+01_JPRB, 0.12083E+01_JPRB, 0.12584E+01_JPRB, 0.13922E+01_JPRB, 0.14896E+01_JPRB, 0.14558E+01_JPRB, 0.14729E+01_JPRB ,&
&  0.15796E+01_JPRB, 0.17010E+01_JPRB, 0.19780E+01_JPRB, 0.19942E+01_JPRB, 0.20418E+01_JPRB, 0.21124E+01_JPRB, 0.21001E+01_JPRB ,&
&  0.21606E+01_JPRB, 0.22429E+01_JPRB, 0.23685E+01_JPRB, 0.23797E+01_JPRB, 0.23466E+01_JPRB, 0.23668E+01_JPRB, 0.23631E+01_JPRB ,&
&  0.23864E+01_JPRB, 0.24096E+01_JPRB, 0.23019E+01_JPRB, 0.22015E+01_JPRB, 0.20083E+01_JPRB, 0.20007E+01_JPRB, 0.18522E+01_JPRB ,&
&  0.15511E+01_JPRB, 0.11564E+01_JPRB, 0.10441E+01_JPRB, 0.91904E+00_JPRB, 0.83314E+00_JPRB, 0.73377E+00_JPRB, 0.65583E+00_JPRB ,&
&  0.62273E+00_JPRB, 0.66493E+00_JPRB, 0.66914E+00_JPRB, 0.67812E+00_JPRB, 0.68798E+00_JPRB, 0.69441E+00_JPRB, 0.61832E+00_JPRB ,&
&  0.62214E+00_JPRB, 0.67004E+00_JPRB, 0.67054E+00_JPRB, 0.67198E+00_JPRB, 0.66989E+00_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.75235E+00_JPRB, 0.73987E+00_JPRB, 0.72894E+00_JPRB, 0.71794E+00_JPRB, 0.70406E+00_JPRB, 0.69106E+00_JPRB, 0.67808E+00_JPRB ,&
&  0.67214E+00_JPRB, 0.67741E+00_JPRB, 0.68035E+00_JPRB, 0.66390E+00_JPRB, 0.64797E+00_JPRB, 0.64248E+00_JPRB, 0.67601E+00_JPRB ,&
&  0.55095E+00_JPRB, 0.52941E+00_JPRB, 0.52775E+00_JPRB, 0.55243E+00_JPRB, 0.61737E+00_JPRB, 0.62761E+00_JPRB, 0.64578E+00_JPRB ,&
&  0.63702E+00_JPRB, 0.62416E+00_JPRB, 0.61679E+00_JPRB, 0.60716E+00_JPRB, 0.58895E+00_JPRB, 0.57803E+00_JPRB, 0.56096E+00_JPRB ,&
&  0.51307E+00_JPRB, 0.47565E+00_JPRB, 0.46239E+00_JPRB, 0.45760E+00_JPRB, 0.48157E+00_JPRB, 0.50960E+00_JPRB, 0.56260E+00_JPRB ,&
&  0.61129E+00_JPRB, 0.65612E+00_JPRB, 0.69898E+00_JPRB, 0.75108E+00_JPRB, 0.79343E+00_JPRB, 0.83394E+00_JPRB, 0.85940E+00_JPRB ,&
&  0.87050E+00_JPRB, 0.87049E+00_JPRB, 0.88231E+00_JPRB, 0.86432E+00_JPRB, 0.86268E+00_JPRB, 0.10694E+01_JPRB, 0.13398E+01_JPRB ,&
&  0.14326E+01_JPRB, 0.15220E+01_JPRB, 0.18953E+01_JPRB, 0.24038E+01_JPRB, 0.22823E+01_JPRB, 0.20875E+01_JPRB, 0.20220E+01_JPRB ,&
&  0.21447E+01_JPRB, 0.23252E+01_JPRB, 0.25183E+01_JPRB, 0.25820E+01_JPRB, 0.25677E+01_JPRB, 0.26345E+01_JPRB, 0.26517E+01_JPRB ,&
&  0.26377E+01_JPRB, 0.26673E+01_JPRB, 0.26304E+01_JPRB, 0.25509E+01_JPRB, 0.24812E+01_JPRB, 0.25272E+01_JPRB, 0.25214E+01_JPRB ,&
&  0.24854E+01_JPRB, 0.23843E+01_JPRB, 0.22037E+01_JPRB, 0.21040E+01_JPRB, 0.17690E+01_JPRB, 0.17126E+01_JPRB, 0.19058E+01_JPRB ,&
&  0.15715E+01_JPRB, 0.12371E+01_JPRB, 0.10551E+01_JPRB, 0.93783E+00_JPRB, 0.88226E+00_JPRB, 0.84426E+00_JPRB, 0.77190E+00_JPRB ,&
&  0.68791E+00_JPRB, 0.70983E+00_JPRB, 0.77186E+00_JPRB, 0.80710E+00_JPRB, 0.81666E+00_JPRB, 0.82607E+00_JPRB, 0.75895E+00_JPRB ,&
&  0.79225E+00_JPRB, 0.77048E+00_JPRB, 0.76381E+00_JPRB, 0.76508E+00_JPRB, 0.76039E+00_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.83684E+00_JPRB, 0.80781E+00_JPRB, 0.78941E+00_JPRB, 0.76091E+00_JPRB, 0.73516E+00_JPRB, 0.71856E+00_JPRB, 0.70807E+00_JPRB ,&
&  0.71006E+00_JPRB, 0.70933E+00_JPRB, 0.70456E+00_JPRB, 0.68501E+00_JPRB, 0.67107E+00_JPRB, 0.66127E+00_JPRB, 0.67783E+00_JPRB ,&
&  0.67763E+00_JPRB, 0.61057E+00_JPRB, 0.60074E+00_JPRB, 0.60784E+00_JPRB, 0.71659E+00_JPRB, 0.80897E+00_JPRB, 0.88537E+00_JPRB ,&
&  0.90803E+00_JPRB, 0.91924E+00_JPRB, 0.90547E+00_JPRB, 0.85793E+00_JPRB, 0.80676E+00_JPRB, 0.76076E+00_JPRB, 0.70801E+00_JPRB ,&
&  0.64597E+00_JPRB, 0.58660E+00_JPRB, 0.55854E+00_JPRB, 0.56382E+00_JPRB, 0.61409E+00_JPRB, 0.59505E+00_JPRB, 0.65246E+00_JPRB ,&
&  0.70921E+00_JPRB, 0.77329E+00_JPRB, 0.82883E+00_JPRB, 0.90050E+00_JPRB, 0.95018E+00_JPRB, 0.98708E+00_JPRB, 0.99371E+00_JPRB ,&
&  0.10095E+01_JPRB, 0.98118E+00_JPRB, 0.87485E+00_JPRB, 0.90985E+00_JPRB, 0.10064E+01_JPRB, 0.14075E+01_JPRB, 0.18284E+01_JPRB ,&
&  0.21009E+01_JPRB, 0.25172E+01_JPRB, 0.30317E+01_JPRB, 0.33372E+01_JPRB, 0.30044E+01_JPRB, 0.27687E+01_JPRB, 0.27879E+01_JPRB ,&
&  0.29495E+01_JPRB, 0.31346E+01_JPRB, 0.32009E+01_JPRB, 0.30619E+01_JPRB, 0.30933E+01_JPRB, 0.31621E+01_JPRB, 0.32179E+01_JPRB ,&
&  0.31910E+01_JPRB, 0.30694E+01_JPRB, 0.29091E+01_JPRB, 0.27571E+01_JPRB, 0.26573E+01_JPRB, 0.24634E+01_JPRB, 0.24071E+01_JPRB ,&
&  0.24443E+01_JPRB, 0.20803E+01_JPRB, 0.18732E+01_JPRB, 0.19437E+01_JPRB, 0.16027E+01_JPRB, 0.15664E+01_JPRB, 0.16802E+01_JPRB ,&
&  0.14313E+01_JPRB, 0.11956E+01_JPRB, 0.10698E+01_JPRB, 0.95015E+00_JPRB, 0.91422E+00_JPRB, 0.97647E+00_JPRB, 0.99365E+00_JPRB ,&
&  0.93685E+00_JPRB, 0.90667E+00_JPRB, 0.10441E+01_JPRB, 0.11143E+01_JPRB, 0.11495E+01_JPRB, 0.11114E+01_JPRB, 0.10535E+01_JPRB ,&
&  0.10000E+01_JPRB, 0.95986E+00_JPRB, 0.93259E+00_JPRB, 0.90445E+00_JPRB, 0.87469E+00_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.91385E+00_JPRB, 0.90166E+00_JPRB, 0.86801E+00_JPRB, 0.81949E+00_JPRB, 0.78475E+00_JPRB, 0.76740E+00_JPRB, 0.74408E+00_JPRB ,&
&  0.72052E+00_JPRB, 0.70266E+00_JPRB, 0.69649E+00_JPRB, 0.68208E+00_JPRB, 0.67757E+00_JPRB, 0.68082E+00_JPRB, 0.68225E+00_JPRB ,&
&  0.67579E+00_JPRB, 0.67547E+00_JPRB, 0.64212E+00_JPRB, 0.65496E+00_JPRB, 0.75497E+00_JPRB, 0.10377E+01_JPRB, 0.12377E+01_JPRB ,&
&  0.13121E+01_JPRB, 0.13191E+01_JPRB, 0.12866E+01_JPRB, 0.12588E+01_JPRB, 0.11900E+01_JPRB, 0.10746E+01_JPRB, 0.10015E+01_JPRB ,&
&  0.92593E+00_JPRB, 0.84046E+00_JPRB, 0.74964E+00_JPRB, 0.76312E+00_JPRB, 0.78750E+00_JPRB, 0.75001E+00_JPRB, 0.83733E+00_JPRB ,&
&  0.90565E+00_JPRB, 0.95463E+00_JPRB, 0.10050E+01_JPRB, 0.10517E+01_JPRB, 0.10734E+01_JPRB, 0.10095E+01_JPRB, 0.92479E+00_JPRB ,&
&  0.88933E+00_JPRB, 0.92133E+00_JPRB, 0.95109E+00_JPRB, 0.10797E+01_JPRB, 0.14584E+01_JPRB, 0.21240E+01_JPRB, 0.26910E+01_JPRB ,&
&  0.30517E+01_JPRB, 0.31960E+01_JPRB, 0.29816E+01_JPRB, 0.32476E+01_JPRB, 0.36881E+01_JPRB, 0.35725E+01_JPRB, 0.33937E+01_JPRB ,&
&  0.34052E+01_JPRB, 0.33955E+01_JPRB, 0.33331E+01_JPRB, 0.33847E+01_JPRB, 0.36615E+01_JPRB, 0.38119E+01_JPRB, 0.38109E+01_JPRB ,&
&  0.33288E+01_JPRB, 0.31447E+01_JPRB, 0.30494E+01_JPRB, 0.29137E+01_JPRB, 0.25940E+01_JPRB, 0.24645E+01_JPRB, 0.25111E+01_JPRB ,&
&  0.24383E+01_JPRB, 0.23844E+01_JPRB, 0.18882E+01_JPRB, 0.17681E+01_JPRB, 0.13698E+01_JPRB, 0.13228E+01_JPRB, 0.14794E+01_JPRB ,&
&  0.13538E+01_JPRB, 0.12336E+01_JPRB, 0.10712E+01_JPRB, 0.99768E+00_JPRB, 0.11344E+01_JPRB, 0.12783E+01_JPRB, 0.14871E+01_JPRB ,&
&  0.14646E+01_JPRB, 0.12836E+01_JPRB, 0.14369E+01_JPRB, 0.14799E+01_JPRB, 0.15409E+01_JPRB, 0.14619E+01_JPRB, 0.13218E+01_JPRB ,&
&  0.12126E+01_JPRB, 0.11333E+01_JPRB, 0.10542E+01_JPRB, 0.98087E+00_JPRB, 0.93686E+00_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.92739E+00_JPRB, 0.90527E+00_JPRB, 0.86367E+00_JPRB, 0.81569E+00_JPRB, 0.78433E+00_JPRB, 0.76019E+00_JPRB, 0.74108E+00_JPRB ,&
&  0.72411E+00_JPRB, 0.70646E+00_JPRB, 0.69556E+00_JPRB, 0.68469E+00_JPRB, 0.67542E+00_JPRB, 0.66816E+00_JPRB, 0.66237E+00_JPRB ,&
&  0.66363E+00_JPRB, 0.69713E+00_JPRB, 0.67903E+00_JPRB, 0.69905E+00_JPRB, 0.85460E+00_JPRB, 0.11912E+01_JPRB, 0.15078E+01_JPRB ,&
&  0.16453E+01_JPRB, 0.16239E+01_JPRB, 0.16020E+01_JPRB, 0.15754E+01_JPRB, 0.15609E+01_JPRB, 0.15022E+01_JPRB, 0.14161E+01_JPRB ,&
&  0.13120E+01_JPRB, 0.10663E+01_JPRB, 0.10222E+01_JPRB, 0.97122E+00_JPRB, 0.10184E+01_JPRB, 0.10481E+01_JPRB, 0.10355E+01_JPRB ,&
&  0.10168E+01_JPRB, 0.10231E+01_JPRB, 0.10329E+01_JPRB, 0.99871E+00_JPRB, 0.97292E+00_JPRB, 0.91555E+00_JPRB, 0.91159E+00_JPRB ,&
&  0.98743E+00_JPRB, 0.10412E+01_JPRB, 0.12851E+01_JPRB, 0.17234E+01_JPRB, 0.25064E+01_JPRB, 0.35180E+01_JPRB, 0.36364E+01_JPRB ,&
&  0.29811E+01_JPRB, 0.28404E+01_JPRB, 0.39030E+01_JPRB, 0.37197E+01_JPRB, 0.38239E+01_JPRB, 0.42467E+01_JPRB, 0.39887E+01_JPRB ,&
&  0.36239E+01_JPRB, 0.33658E+01_JPRB, 0.32852E+01_JPRB, 0.35035E+01_JPRB, 0.46377E+01_JPRB, 0.48472E+01_JPRB, 0.45219E+01_JPRB ,&
&  0.37719E+01_JPRB, 0.35705E+01_JPRB, 0.32693E+01_JPRB, 0.29319E+01_JPRB, 0.27276E+01_JPRB, 0.28460E+01_JPRB, 0.26845E+01_JPRB ,&
&  0.24733E+01_JPRB, 0.26307E+01_JPRB, 0.20861E+01_JPRB, 0.15963E+01_JPRB, 0.13285E+01_JPRB, 0.13522E+01_JPRB, 0.14343E+01_JPRB ,&
&  0.14046E+01_JPRB, 0.13140E+01_JPRB, 0.12238E+01_JPRB, 0.13728E+01_JPRB, 0.17712E+01_JPRB, 0.20401E+01_JPRB, 0.20040E+01_JPRB ,&
&  0.19630E+01_JPRB, 0.21587E+01_JPRB, 0.20515E+01_JPRB, 0.20345E+01_JPRB, 0.20673E+01_JPRB, 0.18502E+01_JPRB, 0.15558E+01_JPRB ,&
&  0.13076E+01_JPRB, 0.11532E+01_JPRB, 0.10579E+01_JPRB, 0.99474E+00_JPRB, 0.95135E+00_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.96311E+00_JPRB, 0.91927E+00_JPRB, 0.87799E+00_JPRB, 0.83856E+00_JPRB, 0.79825E+00_JPRB, 0.76831E+00_JPRB, 0.73923E+00_JPRB ,&
&  0.70835E+00_JPRB, 0.68751E+00_JPRB, 0.69037E+00_JPRB, 0.70115E+00_JPRB, 0.69926E+00_JPRB, 0.67127E+00_JPRB, 0.66578E+00_JPRB ,&
&  0.69906E+00_JPRB, 0.72624E+00_JPRB, 0.71669E+00_JPRB, 0.78644E+00_JPRB, 0.94052E+00_JPRB, 0.11667E+01_JPRB, 0.14890E+01_JPRB ,&
&  0.17043E+01_JPRB, 0.16597E+01_JPRB, 0.16117E+01_JPRB, 0.17306E+01_JPRB, 0.18831E+01_JPRB, 0.19324E+01_JPRB, 0.18647E+01_JPRB ,&
&  0.16184E+01_JPRB, 0.14667E+01_JPRB, 0.14317E+01_JPRB, 0.12986E+01_JPRB, 0.12400E+01_JPRB, 0.11635E+01_JPRB, 0.10667E+01_JPRB ,&
&  0.99247E+00_JPRB, 0.96994E+00_JPRB, 0.95793E+00_JPRB, 0.94808E+00_JPRB, 0.98546E+00_JPRB, 0.10347E+01_JPRB, 0.10771E+01_JPRB ,&
&  0.11736E+01_JPRB, 0.13980E+01_JPRB, 0.18125E+01_JPRB, 0.24678E+01_JPRB, 0.29770E+01_JPRB, 0.43306E+01_JPRB, 0.40416E+01_JPRB ,&
&  0.37232E+01_JPRB, 0.36705E+01_JPRB, 0.36458E+01_JPRB, 0.39915E+01_JPRB, 0.34647E+01_JPRB, 0.37541E+01_JPRB, 0.39079E+01_JPRB ,&
&  0.37510E+01_JPRB, 0.36516E+01_JPRB, 0.38607E+01_JPRB, 0.39908E+01_JPRB, 0.48616E+01_JPRB, 0.61850E+01_JPRB, 0.56479E+01_JPRB ,&
&  0.46144E+01_JPRB, 0.40669E+01_JPRB, 0.34875E+01_JPRB, 0.31423E+01_JPRB, 0.26386E+01_JPRB, 0.25997E+01_JPRB, 0.22614E+01_JPRB ,&
&  0.20485E+01_JPRB, 0.16508E+01_JPRB, 0.17682E+01_JPRB, 0.15025E+01_JPRB, 0.14052E+01_JPRB, 0.13733E+01_JPRB, 0.13691E+01_JPRB ,&
&  0.14152E+01_JPRB, 0.14496E+01_JPRB, 0.16464E+01_JPRB, 0.21000E+01_JPRB, 0.27022E+01_JPRB, 0.23951E+01_JPRB, 0.25772E+01_JPRB ,&
&  0.27488E+01_JPRB, 0.27284E+01_JPRB, 0.23557E+01_JPRB, 0.23020E+01_JPRB, 0.21245E+01_JPRB, 0.19386E+01_JPRB, 0.17123E+01_JPRB ,&
&  0.14677E+01_JPRB, 0.12550E+01_JPRB, 0.11351E+01_JPRB, 0.10615E+01_JPRB, 0.10072E+01_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.93105E+00_JPRB, 0.88660E+00_JPRB, 0.85559E+00_JPRB, 0.82826E+00_JPRB, 0.80128E+00_JPRB, 0.76064E+00_JPRB, 0.71259E+00_JPRB ,&
&  0.67610E+00_JPRB, 0.66219E+00_JPRB, 0.66381E+00_JPRB, 0.67588E+00_JPRB, 0.67641E+00_JPRB, 0.68866E+00_JPRB, 0.70079E+00_JPRB ,&
&  0.75101E+00_JPRB, 0.79448E+00_JPRB, 0.78509E+00_JPRB, 0.82134E+00_JPRB, 0.96097E+00_JPRB, 0.11371E+01_JPRB, 0.13009E+01_JPRB ,&
&  0.16591E+01_JPRB, 0.14986E+01_JPRB, 0.14387E+01_JPRB, 0.16530E+01_JPRB, 0.21173E+01_JPRB, 0.23822E+01_JPRB, 0.22752E+01_JPRB ,&
&  0.22321E+01_JPRB, 0.20095E+01_JPRB, 0.16363E+01_JPRB, 0.13564E+01_JPRB, 0.11551E+01_JPRB, 0.10145E+01_JPRB, 0.89654E+00_JPRB ,&
&  0.84376E+00_JPRB, 0.83685E+00_JPRB, 0.84946E+00_JPRB, 0.86666E+00_JPRB, 0.93096E+00_JPRB, 0.94806E+00_JPRB, 0.11268E+01_JPRB ,&
&  0.13296E+01_JPRB, 0.16011E+01_JPRB, 0.19844E+01_JPRB, 0.23827E+01_JPRB, 0.29731E+01_JPRB, 0.46881E+01_JPRB, 0.53124E+01_JPRB ,&
&  0.40735E+01_JPRB, 0.37987E+01_JPRB, 0.40756E+01_JPRB, 0.42623E+01_JPRB, 0.41699E+01_JPRB, 0.38076E+01_JPRB, 0.37230E+01_JPRB ,&
&  0.29738E+01_JPRB, 0.26672E+01_JPRB, 0.28062E+01_JPRB, 0.35787E+01_JPRB, 0.58149E+01_JPRB, 0.77120E+01_JPRB, 0.71055E+01_JPRB ,&
&  0.58651E+01_JPRB, 0.49203E+01_JPRB, 0.41220E+01_JPRB, 0.31007E+01_JPRB, 0.26775E+01_JPRB, 0.22928E+01_JPRB, 0.26333E+01_JPRB ,&
&  0.24472E+01_JPRB, 0.21805E+01_JPRB, 0.17813E+01_JPRB, 0.13662E+01_JPRB, 0.10883E+01_JPRB, 0.12574E+01_JPRB, 0.13507E+01_JPRB ,&
&  0.15013E+01_JPRB, 0.16750E+01_JPRB, 0.25687E+01_JPRB, 0.29049E+01_JPRB, 0.30224E+01_JPRB, 0.26977E+01_JPRB, 0.33041E+01_JPRB ,&
&  0.31155E+01_JPRB, 0.30873E+01_JPRB, 0.27708E+01_JPRB, 0.22417E+01_JPRB, 0.18440E+01_JPRB, 0.15661E+01_JPRB, 0.14098E+01_JPRB ,&
&  0.13380E+01_JPRB, 0.12740E+01_JPRB, 0.11806E+01_JPRB, 0.10871E+01_JPRB, 0.10097E+01_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.82154E+00_JPRB, 0.78431E+00_JPRB, 0.76359E+00_JPRB, 0.74549E+00_JPRB, 0.72235E+00_JPRB, 0.69168E+00_JPRB, 0.65704E+00_JPRB ,&
&  0.64040E+00_JPRB, 0.63592E+00_JPRB, 0.64142E+00_JPRB, 0.66539E+00_JPRB, 0.68616E+00_JPRB, 0.71817E+00_JPRB, 0.74253E+00_JPRB ,&
&  0.80109E+00_JPRB, 0.89714E+00_JPRB, 0.88337E+00_JPRB, 0.91355E+00_JPRB, 0.10228E+01_JPRB, 0.11152E+01_JPRB, 0.12619E+01_JPRB ,&
&  0.15769E+01_JPRB, 0.13990E+01_JPRB, 0.13774E+01_JPRB, 0.16182E+01_JPRB, 0.23030E+01_JPRB, 0.24078E+01_JPRB, 0.22173E+01_JPRB ,&
&  0.22778E+01_JPRB, 0.19507E+01_JPRB, 0.14477E+01_JPRB, 0.11726E+01_JPRB, 0.10034E+01_JPRB, 0.87181E+00_JPRB, 0.80684E+00_JPRB ,&
&  0.80721E+00_JPRB, 0.84206E+00_JPRB, 0.88532E+00_JPRB, 0.92052E+00_JPRB, 0.94174E+00_JPRB, 0.10400E+01_JPRB, 0.12567E+01_JPRB ,&
&  0.14710E+01_JPRB, 0.17060E+01_JPRB, 0.19118E+01_JPRB, 0.21884E+01_JPRB, 0.28754E+01_JPRB, 0.46813E+01_JPRB, 0.66432E+01_JPRB ,&
&  0.54314E+01_JPRB, 0.51287E+01_JPRB, 0.50562E+01_JPRB, 0.52286E+01_JPRB, 0.42627E+01_JPRB, 0.38261E+01_JPRB, 0.36101E+01_JPRB ,&
&  0.28764E+01_JPRB, 0.28109E+01_JPRB, 0.36132E+01_JPRB, 0.50509E+01_JPRB, 0.62329E+01_JPRB, 0.75245E+01_JPRB, 0.87517E+01_JPRB ,&
&  0.64923E+01_JPRB, 0.52744E+01_JPRB, 0.45127E+01_JPRB, 0.38344E+01_JPRB, 0.22073E+01_JPRB, 0.15443E+01_JPRB, 0.21005E+01_JPRB ,&
&  0.19117E+01_JPRB, 0.10829E+01_JPRB, 0.83825E+00_JPRB, 0.10417E+01_JPRB, 0.75717E+00_JPRB, 0.87051E+00_JPRB, 0.13283E+01_JPRB ,&
&  0.16516E+01_JPRB, 0.20170E+01_JPRB, 0.34291E+01_JPRB, 0.31790E+01_JPRB, 0.32277E+01_JPRB, 0.40308E+01_JPRB, 0.42014E+01_JPRB ,&
&  0.39114E+01_JPRB, 0.26840E+01_JPRB, 0.20933E+01_JPRB, 0.16007E+01_JPRB, 0.13042E+01_JPRB, 0.12069E+01_JPRB, 0.11553E+01_JPRB ,&
&  0.11135E+01_JPRB, 0.10550E+01_JPRB, 0.99978E+00_JPRB, 0.92447E+00_JPRB, 0.87381E+00_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.69796E+00_JPRB, 0.67141E+00_JPRB, 0.66716E+00_JPRB, 0.65898E+00_JPRB, 0.63142E+00_JPRB, 0.61471E+00_JPRB, 0.60579E+00_JPRB ,&
&  0.61039E+00_JPRB, 0.62131E+00_JPRB, 0.64105E+00_JPRB, 0.67818E+00_JPRB, 0.72470E+00_JPRB, 0.77265E+00_JPRB, 0.79828E+00_JPRB ,&
&  0.84208E+00_JPRB, 0.93898E+00_JPRB, 0.10219E+01_JPRB, 0.11044E+01_JPRB, 0.12295E+01_JPRB, 0.11599E+01_JPRB, 0.14650E+01_JPRB ,&
&  0.15389E+01_JPRB, 0.13631E+01_JPRB, 0.14231E+01_JPRB, 0.17071E+01_JPRB, 0.20028E+01_JPRB, 0.19291E+01_JPRB, 0.18080E+01_JPRB ,&
&  0.17043E+01_JPRB, 0.14377E+01_JPRB, 0.11690E+01_JPRB, 0.99733E+00_JPRB, 0.88343E+00_JPRB, 0.82833E+00_JPRB, 0.84964E+00_JPRB ,&
&  0.88978E+00_JPRB, 0.97916E+00_JPRB, 0.10453E+01_JPRB, 0.98853E+00_JPRB, 0.10622E+01_JPRB, 0.11811E+01_JPRB, 0.13694E+01_JPRB ,&
&  0.15421E+01_JPRB, 0.17109E+01_JPRB, 0.18918E+01_JPRB, 0.23123E+01_JPRB, 0.36629E+01_JPRB, 0.55642E+01_JPRB, 0.60331E+01_JPRB ,&
&  0.54436E+01_JPRB, 0.50437E+01_JPRB, 0.56709E+01_JPRB, 0.53383E+01_JPRB, 0.44935E+01_JPRB, 0.43702E+01_JPRB, 0.39852E+01_JPRB ,&
&  0.39483E+01_JPRB, 0.42928E+01_JPRB, 0.47998E+01_JPRB, 0.59655E+01_JPRB, 0.76357E+01_JPRB, 0.79498E+01_JPRB, 0.80820E+01_JPRB ,&
&  0.70752E+01_JPRB, 0.53956E+01_JPRB, 0.36605E+01_JPRB, 0.25326E+01_JPRB, 0.23519E+01_JPRB, 0.18547E+01_JPRB, 0.12038E+01_JPRB ,&
&  0.96518E+00_JPRB, 0.80174E+00_JPRB, 0.58127E+00_JPRB, 0.63990E+00_JPRB, 0.58016E+00_JPRB, 0.70073E+00_JPRB, 0.12013E+01_JPRB ,&
&  0.20711E+01_JPRB, 0.26913E+01_JPRB, 0.33840E+01_JPRB, 0.31430E+01_JPRB, 0.31447E+01_JPRB, 0.40838E+01_JPRB, 0.51280E+01_JPRB ,&
&  0.26454E+01_JPRB, 0.18589E+01_JPRB, 0.14233E+01_JPRB, 0.11725E+01_JPRB, 0.10808E+01_JPRB, 0.10261E+01_JPRB, 0.98252E+00_JPRB ,&
&  0.92431E+00_JPRB, 0.87326E+00_JPRB, 0.84826E+00_JPRB, 0.80642E+00_JPRB, 0.74635E+00_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.64225E+00_JPRB, 0.64295E+00_JPRB, 0.64455E+00_JPRB, 0.63227E+00_JPRB, 0.59533E+00_JPRB, 0.59348E+00_JPRB, 0.60778E+00_JPRB ,&
&  0.62811E+00_JPRB, 0.65442E+00_JPRB, 0.69419E+00_JPRB, 0.74904E+00_JPRB, 0.82282E+00_JPRB, 0.86788E+00_JPRB, 0.88635E+00_JPRB ,&
&  0.92125E+00_JPRB, 0.10382E+01_JPRB, 0.11899E+01_JPRB, 0.13570E+01_JPRB, 0.15112E+01_JPRB, 0.13181E+01_JPRB, 0.16753E+01_JPRB ,&
&  0.14927E+01_JPRB, 0.13851E+01_JPRB, 0.14506E+01_JPRB, 0.16173E+01_JPRB, 0.16787E+01_JPRB, 0.15700E+01_JPRB, 0.14528E+01_JPRB ,&
&  0.12867E+01_JPRB, 0.11971E+01_JPRB, 0.10109E+01_JPRB, 0.91390E+00_JPRB, 0.85494E+00_JPRB, 0.88266E+00_JPRB, 0.94206E+00_JPRB ,&
&  0.10134E+01_JPRB, 0.11214E+01_JPRB, 0.10951E+01_JPRB, 0.11563E+01_JPRB, 0.13216E+01_JPRB, 0.14592E+01_JPRB, 0.15970E+01_JPRB ,&
&  0.16868E+01_JPRB, 0.18663E+01_JPRB, 0.22334E+01_JPRB, 0.32763E+01_JPRB, 0.42963E+01_JPRB, 0.46685E+01_JPRB, 0.54256E+01_JPRB ,&
&  0.59183E+01_JPRB, 0.60174E+01_JPRB, 0.53393E+01_JPRB, 0.58565E+01_JPRB, 0.51265E+01_JPRB, 0.45180E+01_JPRB, 0.44227E+01_JPRB ,&
&  0.47006E+01_JPRB, 0.58286E+01_JPRB, 0.62646E+01_JPRB, 0.67671E+01_JPRB, 0.95619E+01_JPRB, 0.10394E+02_JPRB, 0.82885E+01_JPRB ,&
&  0.72934E+01_JPRB, 0.52134E+01_JPRB, 0.36274E+01_JPRB, 0.26087E+01_JPRB, 0.31469E+01_JPRB, 0.32266E+01_JPRB, 0.15946E+01_JPRB ,&
&  0.10364E+01_JPRB, 0.77370E+00_JPRB, 0.55023E+00_JPRB, 0.53879E+00_JPRB, 0.54458E+00_JPRB, 0.69433E+00_JPRB, 0.15173E+01_JPRB ,&
&  0.18798E+01_JPRB, 0.29943E+01_JPRB, 0.34000E+01_JPRB, 0.33012E+01_JPRB, 0.32554E+01_JPRB, 0.30554E+01_JPRB, 0.29800E+01_JPRB ,&
&  0.19911E+01_JPRB, 0.12772E+01_JPRB, 0.10089E+01_JPRB, 0.10102E+01_JPRB, 0.10139E+01_JPRB, 0.95885E+00_JPRB, 0.88354E+00_JPRB ,&
&  0.83547E+00_JPRB, 0.82620E+00_JPRB, 0.79375E+00_JPRB, 0.75592E+00_JPRB, 0.69773E+00_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.65273E+00_JPRB, 0.67276E+00_JPRB, 0.68642E+00_JPRB, 0.64931E+00_JPRB, 0.63567E+00_JPRB, 0.64519E+00_JPRB, 0.64360E+00_JPRB ,&
&  0.68040E+00_JPRB, 0.72855E+00_JPRB, 0.78065E+00_JPRB, 0.86762E+00_JPRB, 0.94937E+00_JPRB, 0.97336E+00_JPRB, 0.99390E+00_JPRB ,&
&  0.10367E+01_JPRB, 0.11761E+01_JPRB, 0.13944E+01_JPRB, 0.16538E+01_JPRB, 0.16963E+01_JPRB, 0.15641E+01_JPRB, 0.17436E+01_JPRB ,&
&  0.15038E+01_JPRB, 0.14585E+01_JPRB, 0.14314E+01_JPRB, 0.15557E+01_JPRB, 0.15670E+01_JPRB, 0.15179E+01_JPRB, 0.13909E+01_JPRB ,&
&  0.12108E+01_JPRB, 0.12095E+01_JPRB, 0.99374E+00_JPRB, 0.10218E+01_JPRB, 0.99967E+00_JPRB, 0.10638E+01_JPRB, 0.11134E+01_JPRB ,&
&  0.12052E+01_JPRB, 0.12224E+01_JPRB, 0.12612E+01_JPRB, 0.14016E+01_JPRB, 0.15954E+01_JPRB, 0.17647E+01_JPRB, 0.19282E+01_JPRB ,&
&  0.20506E+01_JPRB, 0.23904E+01_JPRB, 0.30613E+01_JPRB, 0.40837E+01_JPRB, 0.39220E+01_JPRB, 0.51572E+01_JPRB, 0.54078E+01_JPRB ,&
&  0.57466E+01_JPRB, 0.57160E+01_JPRB, 0.50729E+01_JPRB, 0.52875E+01_JPRB, 0.52766E+01_JPRB, 0.49415E+01_JPRB, 0.49563E+01_JPRB ,&
&  0.62102E+01_JPRB, 0.68661E+01_JPRB, 0.76800E+01_JPRB, 0.82406E+01_JPRB, 0.10768E+02_JPRB, 0.14639E+02_JPRB, 0.86959E+01_JPRB ,&
&  0.59575E+01_JPRB, 0.52469E+01_JPRB, 0.36723E+01_JPRB, 0.28048E+01_JPRB, 0.34129E+01_JPRB, 0.39335E+01_JPRB, 0.33425E+01_JPRB ,&
&  0.13723E+01_JPRB, 0.76174E+00_JPRB, 0.65084E+00_JPRB, 0.65798E+00_JPRB, 0.56550E+00_JPRB, 0.74480E+00_JPRB, 0.24115E+01_JPRB ,&
&  0.19456E+01_JPRB, 0.29904E+01_JPRB, 0.32038E+01_JPRB, 0.30918E+01_JPRB, 0.32582E+01_JPRB, 0.24865E+01_JPRB, 0.18832E+01_JPRB ,&
&  0.92228E+00_JPRB, 0.85100E+00_JPRB, 0.90796E+00_JPRB, 0.88795E+00_JPRB, 0.88208E+00_JPRB, 0.83269E+00_JPRB, 0.75772E+00_JPRB ,&
&  0.75118E+00_JPRB, 0.75294E+00_JPRB, 0.70982E+00_JPRB, 0.67400E+00_JPRB, 0.65356E+00_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.68940E+00_JPRB, 0.71120E+00_JPRB, 0.71200E+00_JPRB, 0.70865E+00_JPRB, 0.71436E+00_JPRB, 0.72474E+00_JPRB, 0.72206E+00_JPRB ,&
&  0.76404E+00_JPRB, 0.83298E+00_JPRB, 0.92554E+00_JPRB, 0.10375E+01_JPRB, 0.11090E+01_JPRB, 0.11178E+01_JPRB, 0.11469E+01_JPRB ,&
&  0.12342E+01_JPRB, 0.14149E+01_JPRB, 0.16429E+01_JPRB, 0.18018E+01_JPRB, 0.17526E+01_JPRB, 0.15855E+01_JPRB, 0.16410E+01_JPRB ,&
&  0.15721E+01_JPRB, 0.15602E+01_JPRB, 0.13824E+01_JPRB, 0.14975E+01_JPRB, 0.16347E+01_JPRB, 0.16998E+01_JPRB, 0.15670E+01_JPRB ,&
&  0.14475E+01_JPRB, 0.14280E+01_JPRB, 0.13064E+01_JPRB, 0.13126E+01_JPRB, 0.13426E+01_JPRB, 0.14129E+01_JPRB, 0.14673E+01_JPRB ,&
&  0.14269E+01_JPRB, 0.14650E+01_JPRB, 0.15817E+01_JPRB, 0.17687E+01_JPRB, 0.19889E+01_JPRB, 0.21912E+01_JPRB, 0.24208E+01_JPRB ,&
&  0.27260E+01_JPRB, 0.32401E+01_JPRB, 0.38922E+01_JPRB, 0.41053E+01_JPRB, 0.43943E+01_JPRB, 0.48208E+01_JPRB, 0.51939E+01_JPRB ,&
&  0.54668E+01_JPRB, 0.55923E+01_JPRB, 0.53839E+01_JPRB, 0.51899E+01_JPRB, 0.52782E+01_JPRB, 0.54919E+01_JPRB, 0.61118E+01_JPRB ,&
&  0.75411E+01_JPRB, 0.84294E+01_JPRB, 0.85719E+01_JPRB, 0.90280E+01_JPRB, 0.99937E+01_JPRB, 0.10716E+02_JPRB, 0.88758E+01_JPRB ,&
&  0.64252E+01_JPRB, 0.44398E+01_JPRB, 0.33806E+01_JPRB, 0.33894E+01_JPRB, 0.38485E+01_JPRB, 0.39077E+01_JPRB, 0.46990E+01_JPRB ,&
&  0.41949E+01_JPRB, 0.16168E+01_JPRB, 0.10839E+01_JPRB, 0.13726E+01_JPRB, 0.74536E+00_JPRB, 0.69969E+00_JPRB, 0.12715E+01_JPRB ,&
&  0.20140E+01_JPRB, 0.27353E+01_JPRB, 0.27786E+01_JPRB, 0.27102E+01_JPRB, 0.28785E+01_JPRB, 0.18283E+01_JPRB, 0.97356E+00_JPRB ,&
&  0.76443E+00_JPRB, 0.81189E+00_JPRB, 0.81071E+00_JPRB, 0.75340E+00_JPRB, 0.73450E+00_JPRB, 0.75794E+00_JPRB, 0.69284E+00_JPRB ,&
&  0.70292E+00_JPRB, 0.70794E+00_JPRB, 0.70958E+00_JPRB, 0.69933E+00_JPRB, 0.68338E+00_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.77439E+00_JPRB, 0.79488E+00_JPRB, 0.83062E+00_JPRB, 0.85401E+00_JPRB, 0.85417E+00_JPRB, 0.86695E+00_JPRB, 0.88014E+00_JPRB ,&
&  0.93672E+00_JPRB, 0.10336E+01_JPRB, 0.11402E+01_JPRB, 0.12408E+01_JPRB, 0.12771E+01_JPRB, 0.12906E+01_JPRB, 0.13690E+01_JPRB ,&
&  0.14892E+01_JPRB, 0.16642E+01_JPRB, 0.18296E+01_JPRB, 0.19469E+01_JPRB, 0.19574E+01_JPRB, 0.19866E+01_JPRB, 0.15916E+01_JPRB ,&
&  0.14301E+01_JPRB, 0.16875E+01_JPRB, 0.14568E+01_JPRB, 0.15721E+01_JPRB, 0.17302E+01_JPRB, 0.18625E+01_JPRB, 0.18311E+01_JPRB ,&
&  0.17825E+01_JPRB, 0.17518E+01_JPRB, 0.16858E+01_JPRB, 0.16836E+01_JPRB, 0.17506E+01_JPRB, 0.18097E+01_JPRB, 0.18390E+01_JPRB ,&
&  0.18885E+01_JPRB, 0.19410E+01_JPRB, 0.20755E+01_JPRB, 0.22623E+01_JPRB, 0.24680E+01_JPRB, 0.27050E+01_JPRB, 0.30399E+01_JPRB ,&
&  0.34125E+01_JPRB, 0.37493E+01_JPRB, 0.40733E+01_JPRB, 0.40363E+01_JPRB, 0.42922E+01_JPRB, 0.46376E+01_JPRB, 0.49352E+01_JPRB ,&
&  0.48637E+01_JPRB, 0.47290E+01_JPRB, 0.51378E+01_JPRB, 0.56857E+01_JPRB, 0.60435E+01_JPRB, 0.62416E+01_JPRB, 0.68669E+01_JPRB ,&
&  0.82235E+01_JPRB, 0.90902E+01_JPRB, 0.93186E+01_JPRB, 0.81135E+01_JPRB, 0.74356E+01_JPRB, 0.69660E+01_JPRB, 0.68281E+01_JPRB ,&
&  0.57505E+01_JPRB, 0.44773E+01_JPRB, 0.38004E+01_JPRB, 0.36756E+01_JPRB, 0.37156E+01_JPRB, 0.37570E+01_JPRB, 0.41223E+01_JPRB ,&
&  0.42328E+01_JPRB, 0.36497E+01_JPRB, 0.27746E+01_JPRB, 0.17146E+01_JPRB, 0.10944E+01_JPRB, 0.97282E+00_JPRB, 0.12469E+01_JPRB ,&
&  0.19241E+01_JPRB, 0.21468E+01_JPRB, 0.21556E+01_JPRB, 0.27225E+01_JPRB, 0.21341E+01_JPRB, 0.13014E+01_JPRB, 0.78319E+00_JPRB ,&
&  0.68337E+00_JPRB, 0.71966E+00_JPRB, 0.73089E+00_JPRB, 0.71560E+00_JPRB, 0.73989E+00_JPRB, 0.76620E+00_JPRB, 0.74903E+00_JPRB ,&
&  0.76430E+00_JPRB, 0.78045E+00_JPRB, 0.77005E+00_JPRB, 0.75407E+00_JPRB, 0.76248E+00_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.94973E+00_JPRB, 0.99340E+00_JPRB, 0.10273E+01_JPRB, 0.10478E+01_JPRB, 0.10619E+01_JPRB, 0.10417E+01_JPRB, 0.10583E+01_JPRB ,&
&  0.11388E+01_JPRB, 0.12359E+01_JPRB, 0.13214E+01_JPRB, 0.13753E+01_JPRB, 0.14003E+01_JPRB, 0.14531E+01_JPRB, 0.15574E+01_JPRB ,&
&  0.16531E+01_JPRB, 0.17698E+01_JPRB, 0.18639E+01_JPRB, 0.19676E+01_JPRB, 0.20357E+01_JPRB, 0.22177E+01_JPRB, 0.19180E+01_JPRB ,&
&  0.13647E+01_JPRB, 0.20069E+01_JPRB, 0.15612E+01_JPRB, 0.17064E+01_JPRB, 0.18180E+01_JPRB, 0.19698E+01_JPRB, 0.19591E+01_JPRB ,&
&  0.19616E+01_JPRB, 0.19996E+01_JPRB, 0.20091E+01_JPRB, 0.19852E+01_JPRB, 0.19899E+01_JPRB, 0.20841E+01_JPRB, 0.21904E+01_JPRB ,&
&  0.22891E+01_JPRB, 0.23714E+01_JPRB, 0.24966E+01_JPRB, 0.26602E+01_JPRB, 0.28516E+01_JPRB, 0.30909E+01_JPRB, 0.33772E+01_JPRB ,&
&  0.36164E+01_JPRB, 0.38446E+01_JPRB, 0.38988E+01_JPRB, 0.39414E+01_JPRB, 0.40455E+01_JPRB, 0.42643E+01_JPRB, 0.45194E+01_JPRB ,&
&  0.46172E+01_JPRB, 0.46800E+01_JPRB, 0.51665E+01_JPRB, 0.54893E+01_JPRB, 0.56669E+01_JPRB, 0.61722E+01_JPRB, 0.64703E+01_JPRB ,&
&  0.75851E+01_JPRB, 0.75627E+01_JPRB, 0.76481E+01_JPRB, 0.63402E+01_JPRB, 0.49679E+01_JPRB, 0.43239E+01_JPRB, 0.41180E+01_JPRB ,&
&  0.35666E+01_JPRB, 0.33804E+01_JPRB, 0.30925E+01_JPRB, 0.30536E+01_JPRB, 0.32924E+01_JPRB, 0.35541E+01_JPRB, 0.33776E+01_JPRB ,&
&  0.30938E+01_JPRB, 0.22003E+01_JPRB, 0.16826E+01_JPRB, 0.14140E+01_JPRB, 0.10608E+01_JPRB, 0.97107E+00_JPRB, 0.13269E+01_JPRB ,&
&  0.18387E+01_JPRB, 0.20091E+01_JPRB, 0.21819E+01_JPRB, 0.21156E+01_JPRB, 0.13470E+01_JPRB, 0.86613E+00_JPRB, 0.66622E+00_JPRB ,&
&  0.63699E+00_JPRB, 0.68168E+00_JPRB, 0.70503E+00_JPRB, 0.72444E+00_JPRB, 0.76583E+00_JPRB, 0.76502E+00_JPRB, 0.81320E+00_JPRB ,&
&  0.82476E+00_JPRB, 0.84867E+00_JPRB, 0.87229E+00_JPRB, 0.87827E+00_JPRB, 0.90553E+00_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.11903E+01_JPRB, 0.13023E+01_JPRB, 0.14083E+01_JPRB, 0.14764E+01_JPRB, 0.14309E+01_JPRB, 0.13770E+01_JPRB, 0.13377E+01_JPRB ,&
&  0.13073E+01_JPRB, 0.13147E+01_JPRB, 0.13803E+01_JPRB, 0.14401E+01_JPRB, 0.14923E+01_JPRB, 0.15777E+01_JPRB, 0.17007E+01_JPRB ,&
&  0.18239E+01_JPRB, 0.19438E+01_JPRB, 0.20615E+01_JPRB, 0.22045E+01_JPRB, 0.23821E+01_JPRB, 0.26664E+01_JPRB, 0.25641E+01_JPRB ,&
&  0.18338E+01_JPRB, 0.15941E+01_JPRB, 0.18143E+01_JPRB, 0.17681E+01_JPRB, 0.17720E+01_JPRB, 0.19198E+01_JPRB, 0.20079E+01_JPRB ,&
&  0.20444E+01_JPRB, 0.20856E+01_JPRB, 0.21265E+01_JPRB, 0.21230E+01_JPRB, 0.21307E+01_JPRB, 0.22560E+01_JPRB, 0.23861E+01_JPRB ,&
&  0.25117E+01_JPRB, 0.25970E+01_JPRB, 0.26961E+01_JPRB, 0.28251E+01_JPRB, 0.29753E+01_JPRB, 0.31483E+01_JPRB, 0.33468E+01_JPRB ,&
&  0.34766E+01_JPRB, 0.36259E+01_JPRB, 0.36883E+01_JPRB, 0.37167E+01_JPRB, 0.37971E+01_JPRB, 0.38626E+01_JPRB, 0.39560E+01_JPRB ,&
&  0.41661E+01_JPRB, 0.44957E+01_JPRB, 0.48333E+01_JPRB, 0.50172E+01_JPRB, 0.50084E+01_JPRB, 0.55001E+01_JPRB, 0.57391E+01_JPRB ,&
&  0.59676E+01_JPRB, 0.55472E+01_JPRB, 0.55270E+01_JPRB, 0.46495E+01_JPRB, 0.38439E+01_JPRB, 0.30979E+01_JPRB, 0.28406E+01_JPRB ,&
&  0.26283E+01_JPRB, 0.24450E+01_JPRB, 0.22243E+01_JPRB, 0.22043E+01_JPRB, 0.25501E+01_JPRB, 0.26935E+01_JPRB, 0.26996E+01_JPRB ,&
&  0.22064E+01_JPRB, 0.17132E+01_JPRB, 0.12495E+01_JPRB, 0.12477E+01_JPRB, 0.10178E+01_JPRB, 0.11284E+01_JPRB, 0.13692E+01_JPRB ,&
&  0.16269E+01_JPRB, 0.14990E+01_JPRB, 0.15055E+01_JPRB, 0.14387E+01_JPRB, 0.94298E+00_JPRB, 0.68484E+00_JPRB, 0.58310E+00_JPRB ,&
&  0.56019E+00_JPRB, 0.60357E+00_JPRB, 0.63597E+00_JPRB, 0.65624E+00_JPRB, 0.71176E+00_JPRB, 0.76542E+00_JPRB, 0.83143E+00_JPRB ,&
&  0.86127E+00_JPRB, 0.90499E+00_JPRB, 0.95555E+00_JPRB, 0.10081E+01_JPRB, 0.10885E+01_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.11970E+01_JPRB, 0.12625E+01_JPRB, 0.13240E+01_JPRB, 0.13035E+01_JPRB, 0.12945E+01_JPRB, 0.12597E+01_JPRB, 0.12272E+01_JPRB ,&
&  0.12629E+01_JPRB, 0.13073E+01_JPRB, 0.13786E+01_JPRB, 0.14410E+01_JPRB, 0.14966E+01_JPRB, 0.15909E+01_JPRB, 0.17030E+01_JPRB ,&
&  0.17760E+01_JPRB, 0.18453E+01_JPRB, 0.19224E+01_JPRB, 0.20311E+01_JPRB, 0.21475E+01_JPRB, 0.23137E+01_JPRB, 0.24098E+01_JPRB ,&
&  0.18967E+01_JPRB, 0.12799E+01_JPRB, 0.14863E+01_JPRB, 0.16502E+01_JPRB, 0.16672E+01_JPRB, 0.18318E+01_JPRB, 0.19449E+01_JPRB ,&
&  0.19063E+01_JPRB, 0.19561E+01_JPRB, 0.20401E+01_JPRB, 0.20556E+01_JPRB, 0.20535E+01_JPRB, 0.21990E+01_JPRB, 0.23452E+01_JPRB ,&
&  0.24844E+01_JPRB, 0.26057E+01_JPRB, 0.27062E+01_JPRB, 0.28185E+01_JPRB, 0.29152E+01_JPRB, 0.30104E+01_JPRB, 0.30969E+01_JPRB ,&
&  0.31882E+01_JPRB, 0.32356E+01_JPRB, 0.33100E+01_JPRB, 0.34552E+01_JPRB, 0.34851E+01_JPRB, 0.34826E+01_JPRB, 0.34663E+01_JPRB ,&
&  0.35888E+01_JPRB, 0.37912E+01_JPRB, 0.41383E+01_JPRB, 0.44457E+01_JPRB, 0.46662E+01_JPRB, 0.45882E+01_JPRB, 0.46275E+01_JPRB ,&
&  0.43434E+01_JPRB, 0.38318E+01_JPRB, 0.38654E+01_JPRB, 0.40366E+01_JPRB, 0.28127E+01_JPRB, 0.23634E+01_JPRB, 0.20188E+01_JPRB ,&
&  0.20981E+01_JPRB, 0.18443E+01_JPRB, 0.16239E+01_JPRB, 0.16105E+01_JPRB, 0.19233E+01_JPRB, 0.18911E+01_JPRB, 0.19252E+01_JPRB ,&
&  0.16518E+01_JPRB, 0.11803E+01_JPRB, 0.10594E+01_JPRB, 0.12106E+01_JPRB, 0.11950E+01_JPRB, 0.16205E+01_JPRB, 0.14357E+01_JPRB ,&
&  0.11992E+01_JPRB, 0.11310E+01_JPRB, 0.12152E+01_JPRB, 0.11978E+01_JPRB, 0.87530E+00_JPRB, 0.64901E+00_JPRB, 0.57204E+00_JPRB ,&
&  0.54186E+00_JPRB, 0.55037E+00_JPRB, 0.58287E+00_JPRB, 0.59859E+00_JPRB, 0.64491E+00_JPRB, 0.69353E+00_JPRB, 0.78611E+00_JPRB ,&
&  0.85929E+00_JPRB, 0.92300E+00_JPRB, 0.97495E+00_JPRB, 0.10389E+01_JPRB, 0.11124E+01_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.89461E+00_JPRB, 0.92938E+00_JPRB, 0.93181E+00_JPRB, 0.10008E+01_JPRB, 0.10707E+01_JPRB, 0.11017E+01_JPRB, 0.11267E+01_JPRB ,&
&  0.11826E+01_JPRB, 0.12704E+01_JPRB, 0.13439E+01_JPRB, 0.13994E+01_JPRB, 0.14604E+01_JPRB, 0.15456E+01_JPRB, 0.15750E+01_JPRB ,&
&  0.15655E+01_JPRB, 0.15621E+01_JPRB, 0.15812E+01_JPRB, 0.16107E+01_JPRB, 0.16639E+01_JPRB, 0.17240E+01_JPRB, 0.19105E+01_JPRB ,&
&  0.20268E+01_JPRB, 0.19804E+01_JPRB, 0.19132E+01_JPRB, 0.15514E+01_JPRB, 0.11419E+01_JPRB, 0.14638E+01_JPRB, 0.14701E+01_JPRB ,&
&  0.16148E+01_JPRB, 0.17092E+01_JPRB, 0.17440E+01_JPRB, 0.18500E+01_JPRB, 0.19516E+01_JPRB, 0.20624E+01_JPRB, 0.21935E+01_JPRB ,&
&  0.22939E+01_JPRB, 0.24049E+01_JPRB, 0.25098E+01_JPRB, 0.26070E+01_JPRB, 0.26811E+01_JPRB, 0.27290E+01_JPRB, 0.27451E+01_JPRB ,&
&  0.27712E+01_JPRB, 0.28314E+01_JPRB, 0.29067E+01_JPRB, 0.31061E+01_JPRB, 0.31554E+01_JPRB, 0.32038E+01_JPRB, 0.31261E+01_JPRB ,&
&  0.31494E+01_JPRB, 0.32298E+01_JPRB, 0.34660E+01_JPRB, 0.36553E+01_JPRB, 0.36751E+01_JPRB, 0.34232E+01_JPRB, 0.32680E+01_JPRB ,&
&  0.30510E+01_JPRB, 0.27800E+01_JPRB, 0.27891E+01_JPRB, 0.30543E+01_JPRB, 0.20076E+01_JPRB, 0.20072E+01_JPRB, 0.20676E+01_JPRB ,&
&  0.17803E+01_JPRB, 0.15862E+01_JPRB, 0.14442E+01_JPRB, 0.12867E+01_JPRB, 0.13822E+01_JPRB, 0.13988E+01_JPRB, 0.15919E+01_JPRB ,&
&  0.13726E+01_JPRB, 0.99137E+00_JPRB, 0.94473E+00_JPRB, 0.11328E+01_JPRB, 0.13049E+01_JPRB, 0.16496E+01_JPRB, 0.12522E+01_JPRB ,&
&  0.95284E+00_JPRB, 0.11370E+01_JPRB, 0.12265E+01_JPRB, 0.14281E+01_JPRB, 0.11921E+01_JPRB, 0.65665E+00_JPRB, 0.54868E+00_JPRB ,&
&  0.52752E+00_JPRB, 0.53754E+00_JPRB, 0.55576E+00_JPRB, 0.56812E+00_JPRB, 0.59303E+00_JPRB, 0.62694E+00_JPRB, 0.65486E+00_JPRB ,&
&  0.67811E+00_JPRB, 0.69352E+00_JPRB, 0.72796E+00_JPRB, 0.77794E+00_JPRB, 0.84567E+00_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.63396E+00_JPRB, 0.63507E+00_JPRB, 0.62233E+00_JPRB, 0.63784E+00_JPRB, 0.66620E+00_JPRB, 0.72989E+00_JPRB, 0.79155E+00_JPRB ,&
&  0.86319E+00_JPRB, 0.95489E+00_JPRB, 0.10872E+01_JPRB, 0.12103E+01_JPRB, 0.12510E+01_JPRB, 0.12409E+01_JPRB, 0.11396E+01_JPRB ,&
&  0.10077E+01_JPRB, 0.86977E+00_JPRB, 0.79552E+00_JPRB, 0.77867E+00_JPRB, 0.83223E+00_JPRB, 0.86381E+00_JPRB, 0.93704E+00_JPRB ,&
&  0.10568E+01_JPRB, 0.12544E+01_JPRB, 0.14681E+01_JPRB, 0.19425E+01_JPRB, 0.22327E+01_JPRB, 0.15548E+01_JPRB, 0.14174E+01_JPRB ,&
&  0.16262E+01_JPRB, 0.16924E+01_JPRB, 0.14297E+01_JPRB, 0.14294E+01_JPRB, 0.15708E+01_JPRB, 0.16882E+01_JPRB, 0.18353E+01_JPRB ,&
&  0.19098E+01_JPRB, 0.19900E+01_JPRB, 0.20622E+01_JPRB, 0.21040E+01_JPRB, 0.21219E+01_JPRB, 0.20870E+01_JPRB, 0.20542E+01_JPRB ,&
&  0.20359E+01_JPRB, 0.20203E+01_JPRB, 0.21414E+01_JPRB, 0.23028E+01_JPRB, 0.24788E+01_JPRB, 0.26094E+01_JPRB, 0.26309E+01_JPRB ,&
&  0.26462E+01_JPRB, 0.26884E+01_JPRB, 0.29495E+01_JPRB, 0.30080E+01_JPRB, 0.28192E+01_JPRB, 0.26239E+01_JPRB, 0.23644E+01_JPRB ,&
&  0.21069E+01_JPRB, 0.17775E+01_JPRB, 0.13719E+01_JPRB, 0.20308E+01_JPRB, 0.23256E+01_JPRB, 0.20056E+01_JPRB, 0.16544E+01_JPRB ,&
&  0.15371E+01_JPRB, 0.14244E+01_JPRB, 0.13022E+01_JPRB, 0.12205E+01_JPRB, 0.12244E+01_JPRB, 0.15334E+01_JPRB, 0.14302E+01_JPRB ,&
&  0.12291E+01_JPRB, 0.86600E+00_JPRB, 0.76362E+00_JPRB, 0.84427E+00_JPRB, 0.96737E+00_JPRB, 0.11601E+01_JPRB, 0.90189E+00_JPRB ,&
&  0.81676E+00_JPRB, 0.11217E+01_JPRB, 0.12647E+01_JPRB, 0.16962E+01_JPRB, 0.19906E+01_JPRB, 0.56812E+00_JPRB, 0.53467E+00_JPRB ,&
&  0.52714E+00_JPRB, 0.53386E+00_JPRB, 0.55582E+00_JPRB, 0.55508E+00_JPRB, 0.55723E+00_JPRB, 0.55920E+00_JPRB, 0.55459E+00_JPRB ,&
&  0.55812E+00_JPRB, 0.57538E+00_JPRB, 0.59418E+00_JPRB, 0.60285E+00_JPRB, 0.62189E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.61240E+00_JPRB, 0.61234E+00_JPRB, 0.61634E+00_JPRB, 0.62354E+00_JPRB, 0.62411E+00_JPRB, 0.62230E+00_JPRB, 0.63486E+00_JPRB ,&
&  0.65780E+00_JPRB, 0.70203E+00_JPRB, 0.76915E+00_JPRB, 0.80977E+00_JPRB, 0.79396E+00_JPRB, 0.74558E+00_JPRB, 0.72754E+00_JPRB ,&
&  0.72703E+00_JPRB, 0.72562E+00_JPRB, 0.75621E+00_JPRB, 0.80700E+00_JPRB, 0.87302E+00_JPRB, 0.89439E+00_JPRB, 0.95691E+00_JPRB ,&
&  0.95661E+00_JPRB, 0.98286E+00_JPRB, 0.10735E+01_JPRB, 0.11587E+01_JPRB, 0.17622E+01_JPRB, 0.16675E+01_JPRB, 0.13404E+01_JPRB ,&
&  0.12793E+01_JPRB, 0.12811E+01_JPRB, 0.12774E+01_JPRB, 0.11634E+01_JPRB, 0.11567E+01_JPRB, 0.11316E+01_JPRB, 0.11597E+01_JPRB ,&
&  0.11561E+01_JPRB, 0.12426E+01_JPRB, 0.13177E+01_JPRB, 0.14036E+01_JPRB, 0.14652E+01_JPRB, 0.14865E+01_JPRB, 0.15453E+01_JPRB ,&
&  0.15930E+01_JPRB, 0.16432E+01_JPRB, 0.17081E+01_JPRB, 0.15389E+01_JPRB, 0.16781E+01_JPRB, 0.18473E+01_JPRB, 0.19661E+01_JPRB ,&
&  0.20155E+01_JPRB, 0.20399E+01_JPRB, 0.22149E+01_JPRB, 0.21551E+01_JPRB, 0.20550E+01_JPRB, 0.18675E+01_JPRB, 0.16946E+01_JPRB ,&
&  0.15643E+01_JPRB, 0.11579E+01_JPRB, 0.75435E+00_JPRB, 0.10956E+01_JPRB, 0.13384E+01_JPRB, 0.11848E+01_JPRB, 0.12851E+01_JPRB ,&
&  0.12450E+01_JPRB, 0.11896E+01_JPRB, 0.11305E+01_JPRB, 0.11089E+01_JPRB, 0.10551E+01_JPRB, 0.12007E+01_JPRB, 0.12182E+01_JPRB ,&
&  0.10321E+01_JPRB, 0.77094E+00_JPRB, 0.65586E+00_JPRB, 0.65794E+00_JPRB, 0.71386E+00_JPRB, 0.81670E+00_JPRB, 0.89684E+00_JPRB ,&
&  0.96410E+00_JPRB, 0.81871E+00_JPRB, 0.91150E+00_JPRB, 0.10993E+01_JPRB, 0.70719E+00_JPRB, 0.48844E+00_JPRB, 0.48031E+00_JPRB ,&
&  0.48523E+00_JPRB, 0.49986E+00_JPRB, 0.52351E+00_JPRB, 0.53815E+00_JPRB, 0.54921E+00_JPRB, 0.55906E+00_JPRB, 0.57323E+00_JPRB ,&
&  0.58218E+00_JPRB, 0.59358E+00_JPRB, 0.59855E+00_JPRB, 0.60415E+00_JPRB, 0.60983E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.79043E+00_JPRB, 0.80521E+00_JPRB, 0.82568E+00_JPRB, 0.84660E+00_JPRB, 0.85836E+00_JPRB, 0.87705E+00_JPRB, 0.89471E+00_JPRB ,&
&  0.91894E+00_JPRB, 0.95438E+00_JPRB, 0.97713E+00_JPRB, 0.98557E+00_JPRB, 0.10010E+01_JPRB, 0.10221E+01_JPRB, 0.10327E+01_JPRB ,&
&  0.10589E+01_JPRB, 0.10715E+01_JPRB, 0.11300E+01_JPRB, 0.11585E+01_JPRB, 0.11983E+01_JPRB, 0.12506E+01_JPRB, 0.12596E+01_JPRB ,&
&  0.12520E+01_JPRB, 0.12410E+01_JPRB, 0.12556E+01_JPRB, 0.12556E+01_JPRB, 0.13426E+01_JPRB, 0.13581E+01_JPRB, 0.12950E+01_JPRB ,&
&  0.11097E+01_JPRB, 0.10416E+01_JPRB, 0.12236E+01_JPRB, 0.10473E+01_JPRB, 0.10445E+01_JPRB, 0.10571E+01_JPRB, 0.98204E+00_JPRB ,&
&  0.98863E+00_JPRB, 0.10474E+01_JPRB, 0.11180E+01_JPRB, 0.11873E+01_JPRB, 0.12811E+01_JPRB, 0.13305E+01_JPRB, 0.13657E+01_JPRB ,&
&  0.14094E+01_JPRB, 0.14607E+01_JPRB, 0.14896E+01_JPRB, 0.14533E+01_JPRB, 0.14412E+01_JPRB, 0.16811E+01_JPRB, 0.17836E+01_JPRB ,&
&  0.18246E+01_JPRB, 0.18713E+01_JPRB, 0.15702E+01_JPRB, 0.14591E+01_JPRB, 0.15358E+01_JPRB, 0.13655E+01_JPRB, 0.12996E+01_JPRB ,&
&  0.14194E+01_JPRB, 0.12255E+01_JPRB, 0.65867E+00_JPRB, 0.87359E+00_JPRB, 0.10309E+01_JPRB, 0.10890E+01_JPRB, 0.11113E+01_JPRB ,&
&  0.10510E+01_JPRB, 0.92913E+00_JPRB, 0.89258E+00_JPRB, 0.95374E+00_JPRB, 0.99754E+00_JPRB, 0.94638E+00_JPRB, 0.93354E+00_JPRB ,&
&  0.82255E+00_JPRB, 0.71323E+00_JPRB, 0.65211E+00_JPRB, 0.73334E+00_JPRB, 0.91468E+00_JPRB, 0.12482E+01_JPRB, 0.11985E+01_JPRB ,&
&  0.96395E+00_JPRB, 0.79512E+00_JPRB, 0.11418E+01_JPRB, 0.15449E+01_JPRB, 0.68783E+00_JPRB, 0.54689E+00_JPRB, 0.50591E+00_JPRB ,&
&  0.53697E+00_JPRB, 0.55333E+00_JPRB, 0.56530E+00_JPRB, 0.58876E+00_JPRB, 0.62358E+00_JPRB, 0.63803E+00_JPRB, 0.64293E+00_JPRB ,&
&  0.66201E+00_JPRB, 0.69236E+00_JPRB, 0.71506E+00_JPRB, 0.74286E+00_JPRB, 0.76752E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.92860E+00_JPRB, 0.94970E+00_JPRB, 0.96384E+00_JPRB, 0.97817E+00_JPRB, 0.99731E+00_JPRB, 0.10163E+01_JPRB, 0.10265E+01_JPRB ,&
&  0.10463E+01_JPRB, 0.10637E+01_JPRB, 0.10721E+01_JPRB, 0.10922E+01_JPRB, 0.11001E+01_JPRB, 0.11550E+01_JPRB, 0.11769E+01_JPRB ,&
&  0.12224E+01_JPRB, 0.12484E+01_JPRB, 0.12575E+01_JPRB, 0.12616E+01_JPRB, 0.12766E+01_JPRB, 0.12746E+01_JPRB, 0.12695E+01_JPRB ,&
&  0.12928E+01_JPRB, 0.13374E+01_JPRB, 0.13912E+01_JPRB, 0.14444E+01_JPRB, 0.14754E+01_JPRB, 0.15736E+01_JPRB, 0.17026E+01_JPRB ,&
&  0.12472E+01_JPRB, 0.12042E+01_JPRB, 0.12591E+01_JPRB, 0.11135E+01_JPRB, 0.11361E+01_JPRB, 0.11665E+01_JPRB, 0.11318E+01_JPRB ,&
&  0.11188E+01_JPRB, 0.11674E+01_JPRB, 0.12070E+01_JPRB, 0.12428E+01_JPRB, 0.12923E+01_JPRB, 0.13505E+01_JPRB, 0.13897E+01_JPRB ,&
&  0.14564E+01_JPRB, 0.15105E+01_JPRB, 0.15580E+01_JPRB, 0.16377E+01_JPRB, 0.15939E+01_JPRB, 0.18656E+01_JPRB, 0.19529E+01_JPRB ,&
&  0.19708E+01_JPRB, 0.19020E+01_JPRB, 0.14606E+01_JPRB, 0.13904E+01_JPRB, 0.14456E+01_JPRB, 0.13793E+01_JPRB, 0.13014E+01_JPRB ,&
&  0.14190E+01_JPRB, 0.13514E+01_JPRB, 0.95861E+00_JPRB, 0.10419E+01_JPRB, 0.10633E+01_JPRB, 0.10609E+01_JPRB, 0.98528E+00_JPRB ,&
&  0.89723E+00_JPRB, 0.77495E+00_JPRB, 0.73988E+00_JPRB, 0.76616E+00_JPRB, 0.77483E+00_JPRB, 0.75166E+00_JPRB, 0.76471E+00_JPRB ,&
&  0.74969E+00_JPRB, 0.71867E+00_JPRB, 0.73322E+00_JPRB, 0.71499E+00_JPRB, 0.99660E+00_JPRB, 0.10836E+01_JPRB, 0.15649E+01_JPRB ,&
&  0.80385E+00_JPRB, 0.74797E+00_JPRB, 0.68772E+00_JPRB, 0.95908E+00_JPRB, 0.69331E+00_JPRB, 0.52718E+00_JPRB, 0.55823E+00_JPRB ,&
&  0.62129E+00_JPRB, 0.65003E+00_JPRB, 0.65987E+00_JPRB, 0.67613E+00_JPRB, 0.72021E+00_JPRB, 0.74432E+00_JPRB, 0.76575E+00_JPRB ,&
&  0.76969E+00_JPRB, 0.80374E+00_JPRB, 0.85009E+00_JPRB, 0.88475E+00_JPRB, 0.89944E+00_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.98827E+00_JPRB, 0.10142E+01_JPRB, 0.10356E+01_JPRB, 0.10536E+01_JPRB, 0.10704E+01_JPRB, 0.10901E+01_JPRB, 0.11084E+01_JPRB ,&
&  0.11261E+01_JPRB, 0.11399E+01_JPRB, 0.11489E+01_JPRB, 0.11648E+01_JPRB, 0.11803E+01_JPRB, 0.12198E+01_JPRB, 0.12359E+01_JPRB ,&
&  0.12739E+01_JPRB, 0.12894E+01_JPRB, 0.12884E+01_JPRB, 0.13093E+01_JPRB, 0.12998E+01_JPRB, 0.12777E+01_JPRB, 0.12727E+01_JPRB ,&
&  0.12781E+01_JPRB, 0.12764E+01_JPRB, 0.12770E+01_JPRB, 0.12839E+01_JPRB, 0.12922E+01_JPRB, 0.12672E+01_JPRB, 0.13463E+01_JPRB ,&
&  0.10089E+01_JPRB, 0.13759E+01_JPRB, 0.14590E+01_JPRB, 0.13268E+01_JPRB, 0.12758E+01_JPRB, 0.11856E+01_JPRB, 0.11814E+01_JPRB ,&
&  0.12011E+01_JPRB, 0.12078E+01_JPRB, 0.12309E+01_JPRB, 0.12734E+01_JPRB, 0.13296E+01_JPRB, 0.13974E+01_JPRB, 0.14537E+01_JPRB ,&
&  0.15353E+01_JPRB, 0.16086E+01_JPRB, 0.16720E+01_JPRB, 0.17329E+01_JPRB, 0.18467E+01_JPRB, 0.19209E+01_JPRB, 0.19484E+01_JPRB ,&
&  0.19527E+01_JPRB, 0.19765E+01_JPRB, 0.15564E+01_JPRB, 0.15262E+01_JPRB, 0.16482E+01_JPRB, 0.15805E+01_JPRB, 0.14092E+01_JPRB ,&
&  0.14239E+01_JPRB, 0.13267E+01_JPRB, 0.12309E+01_JPRB, 0.11238E+01_JPRB, 0.10549E+01_JPRB, 0.98213E+00_JPRB, 0.89896E+00_JPRB ,&
&  0.74779E+00_JPRB, 0.63118E+00_JPRB, 0.60263E+00_JPRB, 0.64148E+00_JPRB, 0.65394E+00_JPRB, 0.64630E+00_JPRB, 0.61169E+00_JPRB ,&
&  0.60051E+00_JPRB, 0.60668E+00_JPRB, 0.63955E+00_JPRB, 0.71046E+00_JPRB, 0.77589E+00_JPRB, 0.13619E+01_JPRB, 0.15909E+01_JPRB ,&
&  0.80736E+00_JPRB, 0.71165E+00_JPRB, 0.55377E+00_JPRB, 0.70980E+00_JPRB, 0.71960E+00_JPRB, 0.65157E+00_JPRB, 0.68419E+00_JPRB ,&
&  0.78852E+00_JPRB, 0.84190E+00_JPRB, 0.86321E+00_JPRB, 0.79776E+00_JPRB, 0.81173E+00_JPRB, 0.79502E+00_JPRB, 0.79878E+00_JPRB ,&
&  0.81822E+00_JPRB, 0.84982E+00_JPRB, 0.88898E+00_JPRB, 0.92628E+00_JPRB, 0.95837E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.10118E+01_JPRB, 0.10370E+01_JPRB, 0.10622E+01_JPRB, 0.10895E+01_JPRB, 0.11169E+01_JPRB, 0.11370E+01_JPRB, 0.11595E+01_JPRB ,&
&  0.11834E+01_JPRB, 0.12019E+01_JPRB, 0.12100E+01_JPRB, 0.12190E+01_JPRB, 0.12337E+01_JPRB, 0.12702E+01_JPRB, 0.13007E+01_JPRB ,&
&  0.12879E+01_JPRB, 0.13045E+01_JPRB, 0.13148E+01_JPRB, 0.13226E+01_JPRB, 0.13115E+01_JPRB, 0.12802E+01_JPRB, 0.12898E+01_JPRB ,&
&  0.12837E+01_JPRB, 0.12860E+01_JPRB, 0.12903E+01_JPRB, 0.13189E+01_JPRB, 0.13630E+01_JPRB, 0.14586E+01_JPRB, 0.12319E+01_JPRB ,&
&  0.13576E+01_JPRB, 0.14873E+01_JPRB, 0.15123E+01_JPRB, 0.14136E+01_JPRB, 0.12934E+01_JPRB, 0.12617E+01_JPRB, 0.12594E+01_JPRB ,&
&  0.12757E+01_JPRB, 0.12624E+01_JPRB, 0.12782E+01_JPRB, 0.12992E+01_JPRB, 0.13368E+01_JPRB, 0.14036E+01_JPRB, 0.14542E+01_JPRB ,&
&  0.15333E+01_JPRB, 0.16566E+01_JPRB, 0.17789E+01_JPRB, 0.18932E+01_JPRB, 0.20375E+01_JPRB, 0.21804E+01_JPRB, 0.22642E+01_JPRB ,&
&  0.23075E+01_JPRB, 0.23586E+01_JPRB, 0.21555E+01_JPRB, 0.21816E+01_JPRB, 0.22886E+01_JPRB, 0.20695E+01_JPRB, 0.17324E+01_JPRB ,&
&  0.15807E+01_JPRB, 0.12696E+01_JPRB, 0.10959E+01_JPRB, 0.11966E+01_JPRB, 0.10757E+01_JPRB, 0.94606E+00_JPRB, 0.80871E+00_JPRB ,&
&  0.65519E+00_JPRB, 0.57405E+00_JPRB, 0.57129E+00_JPRB, 0.61259E+00_JPRB, 0.60621E+00_JPRB, 0.60715E+00_JPRB, 0.61932E+00_JPRB ,&
&  0.64451E+00_JPRB, 0.67046E+00_JPRB, 0.71468E+00_JPRB, 0.80244E+00_JPRB, 0.91087E+00_JPRB, 0.89652E+00_JPRB, 0.10430E+01_JPRB ,&
&  0.82341E+00_JPRB, 0.67932E+00_JPRB, 0.74170E+00_JPRB, 0.82355E+00_JPRB, 0.85535E+00_JPRB, 0.85059E+00_JPRB, 0.95254E+00_JPRB ,&
&  0.10975E+01_JPRB, 0.12335E+01_JPRB, 0.12428E+01_JPRB, 0.10249E+01_JPRB, 0.91061E+00_JPRB, 0.86155E+00_JPRB, 0.85532E+00_JPRB ,&
&  0.86499E+00_JPRB, 0.88428E+00_JPRB, 0.91612E+00_JPRB, 0.95276E+00_JPRB, 0.98487E+00_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.10139E+01_JPRB, 0.10407E+01_JPRB, 0.10676E+01_JPRB, 0.10929E+01_JPRB, 0.11118E+01_JPRB, 0.11263E+01_JPRB, 0.11370E+01_JPRB ,&
&  0.11566E+01_JPRB, 0.11763E+01_JPRB, 0.11909E+01_JPRB, 0.11988E+01_JPRB, 0.12063E+01_JPRB, 0.12260E+01_JPRB, 0.12529E+01_JPRB ,&
&  0.12859E+01_JPRB, 0.13038E+01_JPRB, 0.13423E+01_JPRB, 0.13283E+01_JPRB, 0.13132E+01_JPRB, 0.13204E+01_JPRB, 0.13513E+01_JPRB ,&
&  0.13617E+01_JPRB, 0.13560E+01_JPRB, 0.13546E+01_JPRB, 0.13909E+01_JPRB, 0.14308E+01_JPRB, 0.14894E+01_JPRB, 0.12981E+01_JPRB ,&
&  0.14663E+01_JPRB, 0.15829E+01_JPRB, 0.15567E+01_JPRB, 0.14962E+01_JPRB, 0.13812E+01_JPRB, 0.13485E+01_JPRB, 0.13423E+01_JPRB ,&
&  0.13371E+01_JPRB, 0.13380E+01_JPRB, 0.13358E+01_JPRB, 0.13464E+01_JPRB, 0.13713E+01_JPRB, 0.14092E+01_JPRB, 0.14688E+01_JPRB ,&
&  0.15351E+01_JPRB, 0.16316E+01_JPRB, 0.17819E+01_JPRB, 0.19358E+01_JPRB, 0.21424E+01_JPRB, 0.23447E+01_JPRB, 0.24937E+01_JPRB ,&
&  0.26432E+01_JPRB, 0.27944E+01_JPRB, 0.26453E+01_JPRB, 0.27282E+01_JPRB, 0.28913E+01_JPRB, 0.25603E+01_JPRB, 0.19756E+01_JPRB ,&
&  0.13876E+01_JPRB, 0.11953E+01_JPRB, 0.11953E+01_JPRB, 0.12382E+01_JPRB, 0.10727E+01_JPRB, 0.92464E+00_JPRB, 0.82268E+00_JPRB ,&
&  0.69664E+00_JPRB, 0.60201E+00_JPRB, 0.60912E+00_JPRB, 0.60275E+00_JPRB, 0.59285E+00_JPRB, 0.61822E+00_JPRB, 0.64954E+00_JPRB ,&
&  0.68700E+00_JPRB, 0.75373E+00_JPRB, 0.82395E+00_JPRB, 0.90044E+00_JPRB, 0.95035E+00_JPRB, 0.91464E+00_JPRB, 0.99504E+00_JPRB ,&
&  0.94544E+00_JPRB, 0.92376E+00_JPRB, 0.94760E+00_JPRB, 0.78136E+00_JPRB, 0.10590E+01_JPRB, 0.11285E+01_JPRB, 0.12459E+01_JPRB ,&
&  0.12151E+01_JPRB, 0.16296E+01_JPRB, 0.20048E+01_JPRB, 0.15663E+01_JPRB, 0.12379E+01_JPRB, 0.91638E+00_JPRB, 0.86440E+00_JPRB ,&
&  0.87173E+00_JPRB, 0.88752E+00_JPRB, 0.91578E+00_JPRB, 0.95136E+00_JPRB, 0.98368E+00_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.10070E+01_JPRB, 0.10389E+01_JPRB, 0.10733E+01_JPRB, 0.10953E+01_JPRB, 0.11076E+01_JPRB, 0.10948E+01_JPRB, 0.10833E+01_JPRB ,&
&  0.10842E+01_JPRB, 0.10939E+01_JPRB, 0.11102E+01_JPRB, 0.11279E+01_JPRB, 0.11387E+01_JPRB, 0.11489E+01_JPRB, 0.11688E+01_JPRB ,&
&  0.11939E+01_JPRB, 0.12260E+01_JPRB, 0.12681E+01_JPRB, 0.13011E+01_JPRB, 0.13368E+01_JPRB, 0.13655E+01_JPRB, 0.14055E+01_JPRB ,&
&  0.14396E+01_JPRB, 0.14920E+01_JPRB, 0.15054E+01_JPRB, 0.15405E+01_JPRB, 0.15611E+01_JPRB, 0.16112E+01_JPRB, 0.13070E+01_JPRB ,&
&  0.15026E+01_JPRB, 0.15778E+01_JPRB, 0.15686E+01_JPRB, 0.15031E+01_JPRB, 0.13960E+01_JPRB, 0.13462E+01_JPRB, 0.13149E+01_JPRB ,&
&  0.13092E+01_JPRB, 0.12877E+01_JPRB, 0.12406E+01_JPRB, 0.13154E+01_JPRB, 0.13806E+01_JPRB, 0.13817E+01_JPRB, 0.14489E+01_JPRB ,&
&  0.15207E+01_JPRB, 0.16220E+01_JPRB, 0.17497E+01_JPRB, 0.19387E+01_JPRB, 0.21769E+01_JPRB, 0.24009E+01_JPRB, 0.26254E+01_JPRB ,&
&  0.28581E+01_JPRB, 0.31065E+01_JPRB, 0.33058E+01_JPRB, 0.30174E+01_JPRB, 0.30152E+01_JPRB, 0.26699E+01_JPRB, 0.19832E+01_JPRB ,&
&  0.13362E+01_JPRB, 0.11599E+01_JPRB, 0.11924E+01_JPRB, 0.12447E+01_JPRB, 0.10887E+01_JPRB, 0.91667E+00_JPRB, 0.81183E+00_JPRB ,&
&  0.70542E+00_JPRB, 0.64686E+00_JPRB, 0.63141E+00_JPRB, 0.60462E+00_JPRB, 0.59441E+00_JPRB, 0.62669E+00_JPRB, 0.66536E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.78238E+00_JPRB, 0.86278E+00_JPRB, 0.94673E+00_JPRB, 0.10345E+01_JPRB, 0.10645E+01_JPRB, 0.98851E+00_JPRB ,&
&  0.10056E+01_JPRB, 0.10040E+01_JPRB, 0.10556E+01_JPRB, 0.11593E+01_JPRB, 0.12186E+01_JPRB, 0.14418E+01_JPRB, 0.16787E+01_JPRB ,&
&  0.19004E+01_JPRB, 0.17736E+01_JPRB, 0.20969E+01_JPRB, 0.29064E+01_JPRB, 0.26759E+01_JPRB, 0.21716E+01_JPRB, 0.84491E+00_JPRB ,&
&  0.84005E+00_JPRB, 0.84106E+00_JPRB, 0.87681E+00_JPRB, 0.92230E+00_JPRB, 0.97524E+00_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.84510E+00_JPRB, 0.89779E+00_JPRB, 0.95076E+00_JPRB, 0.10179E+01_JPRB, 0.10731E+01_JPRB, 0.10667E+01_JPRB, 0.10413E+01_JPRB ,&
&  0.10236E+01_JPRB, 0.10169E+01_JPRB, 0.10253E+01_JPRB, 0.10416E+01_JPRB, 0.10545E+01_JPRB, 0.10618E+01_JPRB, 0.10747E+01_JPRB ,&
&  0.10945E+01_JPRB, 0.11209E+01_JPRB, 0.11504E+01_JPRB, 0.11845E+01_JPRB, 0.12144E+01_JPRB, 0.12583E+01_JPRB, 0.13096E+01_JPRB ,&
&  0.13642E+01_JPRB, 0.14280E+01_JPRB, 0.14833E+01_JPRB, 0.15401E+01_JPRB, 0.15927E+01_JPRB, 0.15912E+01_JPRB, 0.11815E+01_JPRB ,&
&  0.14594E+01_JPRB, 0.15017E+01_JPRB, 0.15298E+01_JPRB, 0.14679E+01_JPRB, 0.13551E+01_JPRB, 0.12851E+01_JPRB, 0.12665E+01_JPRB ,&
&  0.12501E+01_JPRB, 0.12221E+01_JPRB, 0.11865E+01_JPRB, 0.11706E+01_JPRB, 0.12947E+01_JPRB, 0.13069E+01_JPRB, 0.13918E+01_JPRB ,&
&  0.14821E+01_JPRB, 0.15822E+01_JPRB, 0.16980E+01_JPRB, 0.18814E+01_JPRB, 0.21275E+01_JPRB, 0.23791E+01_JPRB, 0.26553E+01_JPRB ,&
&  0.29304E+01_JPRB, 0.31768E+01_JPRB, 0.33225E+01_JPRB, 0.29170E+01_JPRB, 0.27704E+01_JPRB, 0.25535E+01_JPRB, 0.17096E+01_JPRB ,&
&  0.13338E+01_JPRB, 0.12604E+01_JPRB, 0.13251E+01_JPRB, 0.12842E+01_JPRB, 0.11311E+01_JPRB, 0.98673E+00_JPRB, 0.86135E+00_JPRB ,&
&  0.71539E+00_JPRB, 0.64422E+00_JPRB, 0.62840E+00_JPRB, 0.61230E+00_JPRB, 0.60848E+00_JPRB, 0.64093E+00_JPRB, 0.68438E+00_JPRB ,&
&  0.73384E+00_JPRB, 0.79437E+00_JPRB, 0.84474E+00_JPRB, 0.88872E+00_JPRB, 0.94200E+00_JPRB, 0.10359E+01_JPRB, 0.10929E+01_JPRB ,&
&  0.99393E+00_JPRB, 0.93731E+00_JPRB, 0.97448E+00_JPRB, 0.10109E+01_JPRB, 0.10531E+01_JPRB, 0.11280E+01_JPRB, 0.12067E+01_JPRB ,&
&  0.13332E+01_JPRB, 0.14028E+01_JPRB, 0.11897E+01_JPRB, 0.99761E+00_JPRB, 0.13115E+01_JPRB, 0.13982E+01_JPRB, 0.73276E+00_JPRB ,&
&  0.70073E+00_JPRB, 0.71930E+00_JPRB, 0.73205E+00_JPRB, 0.76736E+00_JPRB, 0.80454E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.67791E+00_JPRB, 0.69500E+00_JPRB, 0.76820E+00_JPRB, 0.79074E+00_JPRB, 0.86281E+00_JPRB, 0.89265E+00_JPRB, 0.92413E+00_JPRB ,&
&  0.92970E+00_JPRB, 0.93227E+00_JPRB, 0.93323E+00_JPRB, 0.93777E+00_JPRB, 0.94340E+00_JPRB, 0.95406E+00_JPRB, 0.96320E+00_JPRB ,&
&  0.97766E+00_JPRB, 0.99706E+00_JPRB, 0.10201E+01_JPRB, 0.10450E+01_JPRB, 0.10781E+01_JPRB, 0.11181E+01_JPRB, 0.11704E+01_JPRB ,&
&  0.12368E+01_JPRB, 0.13150E+01_JPRB, 0.14018E+01_JPRB, 0.14986E+01_JPRB, 0.16016E+01_JPRB, 0.16790E+01_JPRB, 0.16918E+01_JPRB ,&
&  0.12152E+01_JPRB, 0.14698E+01_JPRB, 0.14518E+01_JPRB, 0.13932E+01_JPRB, 0.13140E+01_JPRB, 0.12620E+01_JPRB, 0.12515E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.11751E+01_JPRB, 0.11674E+01_JPRB, 0.11738E+01_JPRB, 0.11955E+01_JPRB, 0.12018E+01_JPRB, 0.12840E+01_JPRB ,&
&  0.13702E+01_JPRB, 0.14839E+01_JPRB, 0.16244E+01_JPRB, 0.17696E+01_JPRB, 0.20015E+01_JPRB, 0.23067E+01_JPRB, 0.26262E+01_JPRB ,&
&  0.29621E+01_JPRB, 0.33072E+01_JPRB, 0.35196E+01_JPRB, 0.31269E+01_JPRB, 0.27295E+01_JPRB, 0.25648E+01_JPRB, 0.18640E+01_JPRB ,&
&  0.14987E+01_JPRB, 0.13040E+01_JPRB, 0.13993E+01_JPRB, 0.14021E+01_JPRB, 0.12213E+01_JPRB, 0.10956E+01_JPRB, 0.97832E+00_JPRB ,&
&  0.82877E+00_JPRB, 0.70019E+00_JPRB, 0.65547E+00_JPRB, 0.64568E+00_JPRB, 0.64858E+00_JPRB, 0.67264E+00_JPRB, 0.69540E+00_JPRB ,&
&  0.72494E+00_JPRB, 0.75874E+00_JPRB, 0.78712E+00_JPRB, 0.81749E+00_JPRB, 0.83460E+00_JPRB, 0.85275E+00_JPRB, 0.85931E+00_JPRB ,&
&  0.87233E+00_JPRB, 0.90864E+00_JPRB, 0.91279E+00_JPRB, 0.93363E+00_JPRB, 0.93342E+00_JPRB, 0.95196E+00_JPRB, 0.97987E+00_JPRB ,&
&  0.10029E+01_JPRB, 0.10224E+01_JPRB, 0.10165E+01_JPRB, 0.98822E+00_JPRB, 0.92438E+00_JPRB, 0.84478E+00_JPRB, 0.68521E+00_JPRB ,&
&  0.65854E+00_JPRB, 0.63980E+00_JPRB, 0.64638E+00_JPRB, 0.65177E+00_JPRB, 0.66975E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.58550E+00_JPRB, 0.60160E+00_JPRB, 0.61894E+00_JPRB, 0.64069E+00_JPRB, 0.67394E+00_JPRB, 0.71280E+00_JPRB, 0.76307E+00_JPRB ,&
&  0.81358E+00_JPRB, 0.82822E+00_JPRB, 0.83133E+00_JPRB, 0.83266E+00_JPRB, 0.83567E+00_JPRB, 0.84648E+00_JPRB, 0.86114E+00_JPRB ,&
&  0.87557E+00_JPRB, 0.88926E+00_JPRB, 0.90221E+00_JPRB, 0.91688E+00_JPRB, 0.93609E+00_JPRB, 0.96545E+00_JPRB, 0.10050E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.11312E+01_JPRB, 0.12417E+01_JPRB, 0.13671E+01_JPRB, 0.15014E+01_JPRB, 0.16081E+01_JPRB, 0.16345E+01_JPRB ,&
&  0.98408E+00_JPRB, 0.12446E+01_JPRB, 0.14708E+01_JPRB, 0.14037E+01_JPRB, 0.13163E+01_JPRB, 0.12739E+01_JPRB, 0.12697E+01_JPRB ,&
&  0.12244E+01_JPRB, 0.11525E+01_JPRB, 0.11353E+01_JPRB, 0.11951E+01_JPRB, 0.11203E+01_JPRB, 0.11088E+01_JPRB, 0.11506E+01_JPRB ,&
&  0.12139E+01_JPRB, 0.12867E+01_JPRB, 0.14013E+01_JPRB, 0.15537E+01_JPRB, 0.17695E+01_JPRB, 0.20898E+01_JPRB, 0.24538E+01_JPRB ,&
&  0.28517E+01_JPRB, 0.33046E+01_JPRB, 0.37191E+01_JPRB, 0.33720E+01_JPRB, 0.33243E+01_JPRB, 0.34106E+01_JPRB, 0.30777E+01_JPRB ,&
&  0.20336E+01_JPRB, 0.15395E+01_JPRB, 0.15916E+01_JPRB, 0.15897E+01_JPRB, 0.13546E+01_JPRB, 0.12333E+01_JPRB, 0.11229E+01_JPRB ,&
&  0.10045E+01_JPRB, 0.87448E+00_JPRB, 0.73753E+00_JPRB, 0.69739E+00_JPRB, 0.69710E+00_JPRB, 0.71447E+00_JPRB, 0.73492E+00_JPRB ,&
&  0.74527E+00_JPRB, 0.74836E+00_JPRB, 0.75983E+00_JPRB, 0.77428E+00_JPRB, 0.80461E+00_JPRB, 0.81917E+00_JPRB, 0.82452E+00_JPRB ,&
&  0.84942E+00_JPRB, 0.86367E+00_JPRB, 0.88580E+00_JPRB, 0.91572E+00_JPRB, 0.94311E+00_JPRB, 0.95958E+00_JPRB, 0.95817E+00_JPRB ,&
&  0.94834E+00_JPRB, 0.96505E+00_JPRB, 0.98969E+00_JPRB, 0.98680E+00_JPRB, 0.94988E+00_JPRB, 0.85900E+00_JPRB, 0.78028E+00_JPRB ,&
&  0.69585E+00_JPRB, 0.67540E+00_JPRB, 0.64123E+00_JPRB, 0.61427E+00_JPRB, 0.59365E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.57241E+00_JPRB, 0.56709E+00_JPRB, 0.57079E+00_JPRB, 0.57585E+00_JPRB, 0.58799E+00_JPRB, 0.61475E+00_JPRB, 0.64548E+00_JPRB ,&
&  0.67044E+00_JPRB, 0.71316E+00_JPRB, 0.74242E+00_JPRB, 0.74345E+00_JPRB, 0.73801E+00_JPRB, 0.74282E+00_JPRB, 0.75486E+00_JPRB ,&
&  0.77306E+00_JPRB, 0.78756E+00_JPRB, 0.79623E+00_JPRB, 0.80424E+00_JPRB, 0.81674E+00_JPRB, 0.83047E+00_JPRB, 0.84917E+00_JPRB ,&
&  0.87907E+00_JPRB, 0.93144E+00_JPRB, 0.10115E+01_JPRB, 0.11330E+01_JPRB, 0.12964E+01_JPRB, 0.14781E+01_JPRB, 0.15804E+01_JPRB ,&
&  0.11700E+01_JPRB, 0.64894E+00_JPRB, 0.12601E+01_JPRB, 0.14968E+01_JPRB, 0.13758E+01_JPRB, 0.13362E+01_JPRB, 0.12872E+01_JPRB ,&
&  0.12271E+01_JPRB, 0.12218E+01_JPRB, 0.11814E+01_JPRB, 0.11770E+01_JPRB, 0.10661E+01_JPRB, 0.10311E+01_JPRB, 0.10277E+01_JPRB ,&
&  0.10505E+01_JPRB, 0.10909E+01_JPRB, 0.11708E+01_JPRB, 0.12885E+01_JPRB, 0.14577E+01_JPRB, 0.17244E+01_JPRB, 0.21264E+01_JPRB ,&
&  0.25610E+01_JPRB, 0.30782E+01_JPRB, 0.37580E+01_JPRB, 0.36687E+01_JPRB, 0.38340E+01_JPRB, 0.34955E+01_JPRB, 0.29209E+01_JPRB ,&
&  0.22799E+01_JPRB, 0.18287E+01_JPRB, 0.18311E+01_JPRB, 0.18465E+01_JPRB, 0.15607E+01_JPRB, 0.13158E+01_JPRB, 0.13056E+01_JPRB ,&
&  0.11342E+01_JPRB, 0.10556E+01_JPRB, 0.90836E+00_JPRB, 0.79951E+00_JPRB, 0.76079E+00_JPRB, 0.76880E+00_JPRB, 0.78239E+00_JPRB ,&
&  0.78287E+00_JPRB, 0.77955E+00_JPRB, 0.78401E+00_JPRB, 0.79315E+00_JPRB, 0.81007E+00_JPRB, 0.82131E+00_JPRB, 0.82569E+00_JPRB ,&
&  0.84842E+00_JPRB, 0.87372E+00_JPRB, 0.89852E+00_JPRB, 0.92384E+00_JPRB, 0.94370E+00_JPRB, 0.93186E+00_JPRB, 0.93678E+00_JPRB ,&
&  0.94710E+00_JPRB, 0.95296E+00_JPRB, 0.95563E+00_JPRB, 0.96876E+00_JPRB, 0.95989E+00_JPRB, 0.91124E+00_JPRB, 0.89284E+00_JPRB ,&
&  0.82843E+00_JPRB, 0.75638E+00_JPRB, 0.67018E+00_JPRB, 0.61763E+00_JPRB, 0.59741E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.65709E+00_JPRB, 0.62864E+00_JPRB, 0.58940E+00_JPRB, 0.57869E+00_JPRB, 0.57684E+00_JPRB, 0.57496E+00_JPRB, 0.59146E+00_JPRB ,&
&  0.60654E+00_JPRB, 0.62879E+00_JPRB, 0.63275E+00_JPRB, 0.65392E+00_JPRB, 0.66528E+00_JPRB, 0.66669E+00_JPRB, 0.67573E+00_JPRB ,&
&  0.68132E+00_JPRB, 0.68740E+00_JPRB, 0.69374E+00_JPRB, 0.70357E+00_JPRB, 0.71481E+00_JPRB, 0.71915E+00_JPRB, 0.72628E+00_JPRB ,&
&  0.74348E+00_JPRB, 0.76527E+00_JPRB, 0.79855E+00_JPRB, 0.87587E+00_JPRB, 0.10072E+01_JPRB, 0.11756E+01_JPRB, 0.13585E+01_JPRB ,&
&  0.14958E+01_JPRB, 0.10941E+01_JPRB, 0.80307E+00_JPRB, 0.13450E+01_JPRB, 0.15003E+01_JPRB, 0.14766E+01_JPRB, 0.13061E+01_JPRB ,&
&  0.13194E+01_JPRB, 0.12921E+01_JPRB, 0.12580E+01_JPRB, 0.12081E+01_JPRB, 0.10701E+01_JPRB, 0.99932E+00_JPRB, 0.94933E+00_JPRB ,&
&  0.92571E+00_JPRB, 0.92337E+00_JPRB, 0.96893E+00_JPRB, 0.10755E+01_JPRB, 0.12012E+01_JPRB, 0.13806E+01_JPRB, 0.17174E+01_JPRB ,&
&  0.21449E+01_JPRB, 0.26681E+01_JPRB, 0.34781E+01_JPRB, 0.38889E+01_JPRB, 0.44721E+01_JPRB, 0.38200E+01_JPRB, 0.31170E+01_JPRB ,&
&  0.24347E+01_JPRB, 0.23169E+01_JPRB, 0.23879E+01_JPRB, 0.21315E+01_JPRB, 0.17264E+01_JPRB, 0.15370E+01_JPRB, 0.15241E+01_JPRB ,&
&  0.13145E+01_JPRB, 0.12057E+01_JPRB, 0.11121E+01_JPRB, 0.97820E+00_JPRB, 0.88918E+00_JPRB, 0.85504E+00_JPRB, 0.84537E+00_JPRB ,&
&  0.83638E+00_JPRB, 0.81890E+00_JPRB, 0.81025E+00_JPRB, 0.81536E+00_JPRB, 0.82376E+00_JPRB, 0.82769E+00_JPRB, 0.83892E+00_JPRB ,&
&  0.85254E+00_JPRB, 0.87201E+00_JPRB, 0.89506E+00_JPRB, 0.90915E+00_JPRB, 0.90274E+00_JPRB, 0.88743E+00_JPRB, 0.90876E+00_JPRB ,&
&  0.91892E+00_JPRB, 0.94345E+00_JPRB, 0.94136E+00_JPRB, 0.94993E+00_JPRB, 0.94877E+00_JPRB, 0.95414E+00_JPRB, 0.95204E+00_JPRB ,&
&  0.94735E+00_JPRB, 0.91816E+00_JPRB, 0.83102E+00_JPRB, 0.71978E+00_JPRB, 0.69025E+00_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.73445E+00_JPRB, 0.70962E+00_JPRB, 0.67694E+00_JPRB, 0.64355E+00_JPRB, 0.62511E+00_JPRB, 0.60726E+00_JPRB, 0.58605E+00_JPRB ,&
&  0.57524E+00_JPRB, 0.56994E+00_JPRB, 0.55928E+00_JPRB, 0.56932E+00_JPRB, 0.59286E+00_JPRB, 0.60521E+00_JPRB, 0.61303E+00_JPRB ,&
&  0.62139E+00_JPRB, 0.62882E+00_JPRB, 0.62090E+00_JPRB, 0.61942E+00_JPRB, 0.62684E+00_JPRB, 0.63268E+00_JPRB, 0.63327E+00_JPRB ,&
&  0.63722E+00_JPRB, 0.64815E+00_JPRB, 0.66365E+00_JPRB, 0.69520E+00_JPRB, 0.76433E+00_JPRB, 0.86948E+00_JPRB, 0.10107E+01_JPRB ,&
&  0.12062E+01_JPRB, 0.11917E+01_JPRB, 0.69363E+00_JPRB, 0.11429E+01_JPRB, 0.15464E+01_JPRB, 0.15620E+01_JPRB, 0.14744E+01_JPRB ,&
&  0.14639E+01_JPRB, 0.14218E+01_JPRB, 0.13985E+01_JPRB, 0.12687E+01_JPRB, 0.11396E+01_JPRB, 0.10362E+01_JPRB, 0.94843E+00_JPRB ,&
&  0.87371E+00_JPRB, 0.82765E+00_JPRB, 0.82031E+00_JPRB, 0.87639E+00_JPRB, 0.97814E+00_JPRB, 0.10807E+01_JPRB, 0.12748E+01_JPRB ,&
&  0.16200E+01_JPRB, 0.20604E+01_JPRB, 0.27682E+01_JPRB, 0.34916E+01_JPRB, 0.46142E+01_JPRB, 0.46195E+01_JPRB, 0.37427E+01_JPRB ,&
&  0.29370E+01_JPRB, 0.28909E+01_JPRB, 0.28602E+01_JPRB, 0.24999E+01_JPRB, 0.19295E+01_JPRB, 0.17543E+01_JPRB, 0.17216E+01_JPRB ,&
&  0.15107E+01_JPRB, 0.13461E+01_JPRB, 0.12708E+01_JPRB, 0.11861E+01_JPRB, 0.10645E+01_JPRB, 0.95852E+00_JPRB, 0.90685E+00_JPRB ,&
&  0.88787E+00_JPRB, 0.86957E+00_JPRB, 0.85693E+00_JPRB, 0.84597E+00_JPRB, 0.84115E+00_JPRB, 0.84034E+00_JPRB, 0.84469E+00_JPRB ,&
&  0.85588E+00_JPRB, 0.86961E+00_JPRB, 0.88701E+00_JPRB, 0.88731E+00_JPRB, 0.88547E+00_JPRB, 0.88511E+00_JPRB, 0.88358E+00_JPRB ,&
&  0.88164E+00_JPRB, 0.88753E+00_JPRB, 0.89009E+00_JPRB, 0.86714E+00_JPRB, 0.91113E+00_JPRB, 0.91805E+00_JPRB, 0.94429E+00_JPRB ,&
&  0.97534E+00_JPRB, 0.97810E+00_JPRB, 0.91902E+00_JPRB, 0.78719E+00_JPRB, 0.76280E+00_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.75237E+00_JPRB, 0.73260E+00_JPRB, 0.71176E+00_JPRB, 0.69468E+00_JPRB, 0.68093E+00_JPRB, 0.66630E+00_JPRB, 0.64348E+00_JPRB ,&
&  0.61508E+00_JPRB, 0.59741E+00_JPRB, 0.56888E+00_JPRB, 0.56046E+00_JPRB, 0.56446E+00_JPRB, 0.56862E+00_JPRB, 0.57848E+00_JPRB ,&
&  0.58421E+00_JPRB, 0.58934E+00_JPRB, 0.59101E+00_JPRB, 0.59026E+00_JPRB, 0.59082E+00_JPRB, 0.58922E+00_JPRB, 0.58801E+00_JPRB ,&
&  0.58335E+00_JPRB, 0.58438E+00_JPRB, 0.59177E+00_JPRB, 0.60349E+00_JPRB, 0.63367E+00_JPRB, 0.68767E+00_JPRB, 0.76258E+00_JPRB ,&
&  0.93360E+00_JPRB, 0.10099E+01_JPRB, 0.10983E+01_JPRB, 0.15647E+01_JPRB, 0.18385E+01_JPRB, 0.17601E+01_JPRB, 0.17329E+01_JPRB ,&
&  0.17554E+01_JPRB, 0.17252E+01_JPRB, 0.16462E+01_JPRB, 0.13529E+01_JPRB, 0.11404E+01_JPRB, 0.10278E+01_JPRB, 0.94361E+00_JPRB ,&
&  0.86457E+00_JPRB, 0.80534E+00_JPRB, 0.76160E+00_JPRB, 0.75092E+00_JPRB, 0.78929E+00_JPRB, 0.87116E+00_JPRB, 0.96991E+00_JPRB ,&
&  0.11681E+01_JPRB, 0.15047E+01_JPRB, 0.19916E+01_JPRB, 0.26066E+01_JPRB, 0.37256E+01_JPRB, 0.45438E+01_JPRB, 0.43367E+01_JPRB ,&
&  0.40586E+01_JPRB, 0.37081E+01_JPRB, 0.32647E+01_JPRB, 0.28951E+01_JPRB, 0.21676E+01_JPRB, 0.20969E+01_JPRB, 0.18501E+01_JPRB ,&
&  0.16139E+01_JPRB, 0.14431E+01_JPRB, 0.13475E+01_JPRB, 0.12903E+01_JPRB, 0.12068E+01_JPRB, 0.10887E+01_JPRB, 0.99029E+00_JPRB ,&
&  0.93408E+00_JPRB, 0.91252E+00_JPRB, 0.90293E+00_JPRB, 0.88917E+00_JPRB, 0.87789E+00_JPRB, 0.87231E+00_JPRB, 0.87023E+00_JPRB ,&
&  0.86524E+00_JPRB, 0.86414E+00_JPRB, 0.85032E+00_JPRB, 0.85991E+00_JPRB, 0.85713E+00_JPRB, 0.84960E+00_JPRB, 0.83759E+00_JPRB ,&
&  0.84224E+00_JPRB, 0.86107E+00_JPRB, 0.86938E+00_JPRB, 0.86703E+00_JPRB, 0.86378E+00_JPRB, 0.86326E+00_JPRB, 0.85433E+00_JPRB ,&
&  0.85191E+00_JPRB, 0.82691E+00_JPRB, 0.79163E+00_JPRB, 0.77694E+00_JPRB, 0.76576E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.75085E+00_JPRB, 0.73360E+00_JPRB, 0.72092E+00_JPRB, 0.70576E+00_JPRB, 0.69137E+00_JPRB, 0.68329E+00_JPRB, 0.67560E+00_JPRB ,&
&  0.66190E+00_JPRB, 0.63131E+00_JPRB, 0.59938E+00_JPRB, 0.58353E+00_JPRB, 0.57746E+00_JPRB, 0.56527E+00_JPRB, 0.56612E+00_JPRB ,&
&  0.56827E+00_JPRB, 0.57399E+00_JPRB, 0.57082E+00_JPRB, 0.56904E+00_JPRB, 0.56820E+00_JPRB, 0.56731E+00_JPRB, 0.56382E+00_JPRB ,&
&  0.55652E+00_JPRB, 0.55264E+00_JPRB, 0.55626E+00_JPRB, 0.56181E+00_JPRB, 0.57287E+00_JPRB, 0.59780E+00_JPRB, 0.62708E+00_JPRB ,&
&  0.73674E+00_JPRB, 0.84898E+00_JPRB, 0.92738E+00_JPRB, 0.15583E+01_JPRB, 0.18125E+01_JPRB, 0.19066E+01_JPRB, 0.17982E+01_JPRB ,&
&  0.18195E+01_JPRB, 0.20120E+01_JPRB, 0.17322E+01_JPRB, 0.13458E+01_JPRB, 0.10760E+01_JPRB, 0.95302E+00_JPRB, 0.87678E+00_JPRB ,&
&  0.81334E+00_JPRB, 0.75641E+00_JPRB, 0.71971E+00_JPRB, 0.69860E+00_JPRB, 0.70608E+00_JPRB, 0.72975E+00_JPRB, 0.78033E+00_JPRB ,&
&  0.88102E+00_JPRB, 0.11054E+01_JPRB, 0.14458E+01_JPRB, 0.17564E+01_JPRB, 0.25750E+01_JPRB, 0.35161E+01_JPRB, 0.40662E+01_JPRB ,&
&  0.43342E+01_JPRB, 0.43075E+01_JPRB, 0.35400E+01_JPRB, 0.30526E+01_JPRB, 0.24832E+01_JPRB, 0.21797E+01_JPRB, 0.18779E+01_JPRB ,&
&  0.16686E+01_JPRB, 0.15040E+01_JPRB, 0.13850E+01_JPRB, 0.13161E+01_JPRB, 0.12490E+01_JPRB, 0.11678E+01_JPRB, 0.10664E+01_JPRB ,&
&  0.97802E+00_JPRB, 0.93616E+00_JPRB, 0.91795E+00_JPRB, 0.90487E+00_JPRB, 0.89985E+00_JPRB, 0.89281E+00_JPRB, 0.88402E+00_JPRB ,&
&  0.87464E+00_JPRB, 0.86389E+00_JPRB, 0.83898E+00_JPRB, 0.84018E+00_JPRB, 0.82627E+00_JPRB, 0.79828E+00_JPRB, 0.79390E+00_JPRB ,&
&  0.82507E+00_JPRB, 0.84934E+00_JPRB, 0.85252E+00_JPRB, 0.83361E+00_JPRB, 0.84671E+00_JPRB, 0.86256E+00_JPRB, 0.82694E+00_JPRB ,&
&  0.81466E+00_JPRB, 0.80187E+00_JPRB, 0.78072E+00_JPRB, 0.76814E+00_JPRB, 0.76159E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.74341E+00_JPRB, 0.71909E+00_JPRB, 0.70198E+00_JPRB, 0.68761E+00_JPRB, 0.67449E+00_JPRB, 0.66270E+00_JPRB, 0.65907E+00_JPRB ,&
&  0.65465E+00_JPRB, 0.64563E+00_JPRB, 0.62440E+00_JPRB, 0.59983E+00_JPRB, 0.58776E+00_JPRB, 0.58289E+00_JPRB, 0.57028E+00_JPRB ,&
&  0.56499E+00_JPRB, 0.56529E+00_JPRB, 0.56354E+00_JPRB, 0.55687E+00_JPRB, 0.55084E+00_JPRB, 0.54654E+00_JPRB, 0.54305E+00_JPRB ,&
&  0.53104E+00_JPRB, 0.52495E+00_JPRB, 0.52539E+00_JPRB, 0.52596E+00_JPRB, 0.52921E+00_JPRB, 0.55069E+00_JPRB, 0.58091E+00_JPRB ,&
&  0.64413E+00_JPRB, 0.88053E+00_JPRB, 0.90826E+00_JPRB, 0.12935E+01_JPRB, 0.14599E+01_JPRB, 0.16426E+01_JPRB, 0.17381E+01_JPRB ,&
&  0.18567E+01_JPRB, 0.19898E+01_JPRB, 0.16443E+01_JPRB, 0.12886E+01_JPRB, 0.99206E+00_JPRB, 0.83442E+00_JPRB, 0.76062E+00_JPRB ,&
&  0.72296E+00_JPRB, 0.68878E+00_JPRB, 0.66396E+00_JPRB, 0.65656E+00_JPRB, 0.66138E+00_JPRB, 0.67143E+00_JPRB, 0.68522E+00_JPRB ,&
&  0.73715E+00_JPRB, 0.86720E+00_JPRB, 0.11694E+01_JPRB, 0.15624E+01_JPRB, 0.17967E+01_JPRB, 0.24361E+01_JPRB, 0.33869E+01_JPRB ,&
&  0.41098E+01_JPRB, 0.44996E+01_JPRB, 0.37267E+01_JPRB, 0.30890E+01_JPRB, 0.26109E+01_JPRB, 0.22495E+01_JPRB, 0.19423E+01_JPRB ,&
&  0.17059E+01_JPRB, 0.15306E+01_JPRB, 0.14041E+01_JPRB, 0.13151E+01_JPRB, 0.12529E+01_JPRB, 0.12057E+01_JPRB, 0.11372E+01_JPRB ,&
&  0.10576E+01_JPRB, 0.99693E+00_JPRB, 0.95843E+00_JPRB, 0.93694E+00_JPRB, 0.92867E+00_JPRB, 0.91367E+00_JPRB, 0.88716E+00_JPRB ,&
&  0.86531E+00_JPRB, 0.85609E+00_JPRB, 0.83341E+00_JPRB, 0.83181E+00_JPRB, 0.82145E+00_JPRB, 0.79010E+00_JPRB, 0.77841E+00_JPRB ,&
&  0.81146E+00_JPRB, 0.82815E+00_JPRB, 0.83336E+00_JPRB, 0.83611E+00_JPRB, 0.84507E+00_JPRB, 0.88514E+00_JPRB, 0.83729E+00_JPRB ,&
&  0.80352E+00_JPRB, 0.78783E+00_JPRB, 0.77308E+00_JPRB, 0.76617E+00_JPRB, 0.75770E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.73101E+00_JPRB, 0.70540E+00_JPRB, 0.68458E+00_JPRB, 0.66404E+00_JPRB, 0.64871E+00_JPRB, 0.63265E+00_JPRB, 0.62201E+00_JPRB ,&
&  0.61712E+00_JPRB, 0.61666E+00_JPRB, 0.61385E+00_JPRB, 0.60367E+00_JPRB, 0.58681E+00_JPRB, 0.57381E+00_JPRB, 0.56500E+00_JPRB ,&
&  0.55616E+00_JPRB, 0.55415E+00_JPRB, 0.55310E+00_JPRB, 0.54486E+00_JPRB, 0.53582E+00_JPRB, 0.52792E+00_JPRB, 0.51936E+00_JPRB ,&
&  0.51137E+00_JPRB, 0.50682E+00_JPRB, 0.50339E+00_JPRB, 0.50034E+00_JPRB, 0.49866E+00_JPRB, 0.50952E+00_JPRB, 0.56079E+00_JPRB ,&
&  0.66731E+00_JPRB, 0.57907E+00_JPRB, 0.10685E+01_JPRB, 0.11926E+01_JPRB, 0.12392E+01_JPRB, 0.13196E+01_JPRB, 0.15229E+01_JPRB ,&
&  0.17390E+01_JPRB, 0.17115E+01_JPRB, 0.14928E+01_JPRB, 0.12176E+01_JPRB, 0.97442E+00_JPRB, 0.81232E+00_JPRB, 0.72766E+00_JPRB ,&
&  0.66831E+00_JPRB, 0.63040E+00_JPRB, 0.61375E+00_JPRB, 0.60440E+00_JPRB, 0.60761E+00_JPRB, 0.61989E+00_JPRB, 0.63055E+00_JPRB ,&
&  0.65953E+00_JPRB, 0.73440E+00_JPRB, 0.94048E+00_JPRB, 0.13390E+01_JPRB, 0.17478E+01_JPRB, 0.20616E+01_JPRB, 0.26774E+01_JPRB ,&
&  0.36374E+01_JPRB, 0.39084E+01_JPRB, 0.32398E+01_JPRB, 0.27411E+01_JPRB, 0.24200E+01_JPRB, 0.21864E+01_JPRB, 0.19561E+01_JPRB ,&
&  0.17343E+01_JPRB, 0.15208E+01_JPRB, 0.13665E+01_JPRB, 0.12598E+01_JPRB, 0.11987E+01_JPRB, 0.11749E+01_JPRB, 0.11540E+01_JPRB ,&
&  0.11256E+01_JPRB, 0.10843E+01_JPRB, 0.10428E+01_JPRB, 0.10040E+01_JPRB, 0.97360E+00_JPRB, 0.94278E+00_JPRB, 0.89851E+00_JPRB ,&
&  0.86184E+00_JPRB, 0.84331E+00_JPRB, 0.81960E+00_JPRB, 0.80439E+00_JPRB, 0.80118E+00_JPRB, 0.79350E+00_JPRB, 0.79137E+00_JPRB ,&
&  0.80136E+00_JPRB, 0.80516E+00_JPRB, 0.80658E+00_JPRB, 0.81525E+00_JPRB, 0.80640E+00_JPRB, 0.84743E+00_JPRB, 0.82781E+00_JPRB ,&
&  0.81759E+00_JPRB, 0.79667E+00_JPRB, 0.77139E+00_JPRB, 0.75759E+00_JPRB, 0.74824E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.71734E+00_JPRB, 0.69773E+00_JPRB, 0.66953E+00_JPRB, 0.64473E+00_JPRB, 0.62544E+00_JPRB, 0.60536E+00_JPRB, 0.59122E+00_JPRB ,&
&  0.58081E+00_JPRB, 0.58131E+00_JPRB, 0.58509E+00_JPRB, 0.58670E+00_JPRB, 0.58057E+00_JPRB, 0.56555E+00_JPRB, 0.55054E+00_JPRB ,&
&  0.54013E+00_JPRB, 0.53719E+00_JPRB, 0.53671E+00_JPRB, 0.53272E+00_JPRB, 0.52615E+00_JPRB, 0.51741E+00_JPRB, 0.50902E+00_JPRB ,&
&  0.50101E+00_JPRB, 0.49473E+00_JPRB, 0.48863E+00_JPRB, 0.48428E+00_JPRB, 0.47923E+00_JPRB, 0.48419E+00_JPRB, 0.53744E+00_JPRB ,&
&  0.68774E+00_JPRB, 0.83525E+00_JPRB, 0.10102E+01_JPRB, 0.97249E+00_JPRB, 0.97872E+00_JPRB, 0.10299E+01_JPRB, 0.11873E+01_JPRB ,&
&  0.12785E+01_JPRB, 0.13068E+01_JPRB, 0.12393E+01_JPRB, 0.10983E+01_JPRB, 0.93094E+00_JPRB, 0.80799E+00_JPRB, 0.71731E+00_JPRB ,&
&  0.64303E+00_JPRB, 0.59621E+00_JPRB, 0.56982E+00_JPRB, 0.56259E+00_JPRB, 0.56747E+00_JPRB, 0.57911E+00_JPRB, 0.58585E+00_JPRB ,&
&  0.60226E+00_JPRB, 0.64395E+00_JPRB, 0.79856E+00_JPRB, 0.11602E+01_JPRB, 0.17741E+01_JPRB, 0.18619E+01_JPRB, 0.25464E+01_JPRB ,&
&  0.32983E+01_JPRB, 0.29661E+01_JPRB, 0.26747E+01_JPRB, 0.24329E+01_JPRB, 0.22426E+01_JPRB, 0.20699E+01_JPRB, 0.18001E+01_JPRB ,&
&  0.15841E+01_JPRB, 0.13873E+01_JPRB, 0.12417E+01_JPRB, 0.11592E+01_JPRB, 0.11084E+01_JPRB, 0.10877E+01_JPRB, 0.10838E+01_JPRB ,&
&  0.10784E+01_JPRB, 0.10663E+01_JPRB, 0.10555E+01_JPRB, 0.10289E+01_JPRB, 0.98991E+00_JPRB, 0.94865E+00_JPRB, 0.90493E+00_JPRB ,&
&  0.86642E+00_JPRB, 0.83236E+00_JPRB, 0.79925E+00_JPRB, 0.77109E+00_JPRB, 0.76914E+00_JPRB, 0.77526E+00_JPRB, 0.77376E+00_JPRB ,&
&  0.76359E+00_JPRB, 0.75855E+00_JPRB, 0.77124E+00_JPRB, 0.77058E+00_JPRB, 0.77053E+00_JPRB, 0.81444E+00_JPRB, 0.81113E+00_JPRB ,&
&  0.80561E+00_JPRB, 0.78217E+00_JPRB, 0.75888E+00_JPRB, 0.73916E+00_JPRB, 0.73009E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.70807E+00_JPRB, 0.68309E+00_JPRB, 0.65439E+00_JPRB, 0.62774E+00_JPRB, 0.60372E+00_JPRB, 0.58523E+00_JPRB, 0.57381E+00_JPRB ,&
&  0.56411E+00_JPRB, 0.55812E+00_JPRB, 0.55981E+00_JPRB, 0.56461E+00_JPRB, 0.56387E+00_JPRB, 0.55470E+00_JPRB, 0.54126E+00_JPRB ,&
&  0.52772E+00_JPRB, 0.51763E+00_JPRB, 0.51064E+00_JPRB, 0.50811E+00_JPRB, 0.50371E+00_JPRB, 0.49896E+00_JPRB, 0.49105E+00_JPRB ,&
&  0.48496E+00_JPRB, 0.47959E+00_JPRB, 0.47160E+00_JPRB, 0.46333E+00_JPRB, 0.45757E+00_JPRB, 0.45913E+00_JPRB, 0.50249E+00_JPRB ,&
&  0.62433E+00_JPRB, 0.74653E+00_JPRB, 0.75494E+00_JPRB, 0.78504E+00_JPRB, 0.83558E+00_JPRB, 0.86126E+00_JPRB, 0.92838E+00_JPRB ,&
&  0.96659E+00_JPRB, 0.98568E+00_JPRB, 0.99152E+00_JPRB, 0.94227E+00_JPRB, 0.84981E+00_JPRB, 0.77710E+00_JPRB, 0.70528E+00_JPRB ,&
&  0.63962E+00_JPRB, 0.58831E+00_JPRB, 0.55564E+00_JPRB, 0.54278E+00_JPRB, 0.54556E+00_JPRB, 0.55347E+00_JPRB, 0.55664E+00_JPRB ,&
&  0.56263E+00_JPRB, 0.57245E+00_JPRB, 0.63153E+00_JPRB, 0.77692E+00_JPRB, 0.11484E+01_JPRB, 0.14044E+01_JPRB, 0.18073E+01_JPRB ,&
&  0.19082E+01_JPRB, 0.18831E+01_JPRB, 0.18291E+01_JPRB, 0.17671E+01_JPRB, 0.16898E+01_JPRB, 0.15706E+01_JPRB, 0.14653E+01_JPRB ,&
&  0.13382E+01_JPRB, 0.12083E+01_JPRB, 0.11067E+01_JPRB, 0.10382E+01_JPRB, 0.99523E+00_JPRB, 0.96990E+00_JPRB, 0.96080E+00_JPRB ,&
&  0.94606E+00_JPRB, 0.94164E+00_JPRB, 0.92861E+00_JPRB, 0.91831E+00_JPRB, 0.91137E+00_JPRB, 0.88700E+00_JPRB, 0.84796E+00_JPRB ,&
&  0.80665E+00_JPRB, 0.77142E+00_JPRB, 0.74569E+00_JPRB, 0.73183E+00_JPRB, 0.72502E+00_JPRB, 0.71907E+00_JPRB, 0.71321E+00_JPRB ,&
&  0.71816E+00_JPRB, 0.72094E+00_JPRB, 0.72441E+00_JPRB, 0.72098E+00_JPRB, 0.71357E+00_JPRB, 0.74638E+00_JPRB, 0.74542E+00_JPRB ,&
&  0.74504E+00_JPRB, 0.74197E+00_JPRB, 0.73013E+00_JPRB, 0.71241E+00_JPRB, 0.72032E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.71365E+00_JPRB, 0.69096E+00_JPRB, 0.65970E+00_JPRB, 0.62866E+00_JPRB, 0.59958E+00_JPRB, 0.57671E+00_JPRB, 0.56296E+00_JPRB ,&
&  0.55382E+00_JPRB, 0.54797E+00_JPRB, 0.54490E+00_JPRB, 0.54299E+00_JPRB, 0.54000E+00_JPRB, 0.53662E+00_JPRB, 0.53075E+00_JPRB ,&
&  0.52313E+00_JPRB, 0.51375E+00_JPRB, 0.50523E+00_JPRB, 0.49936E+00_JPRB, 0.49455E+00_JPRB, 0.49158E+00_JPRB, 0.48763E+00_JPRB ,&
&  0.47980E+00_JPRB, 0.47033E+00_JPRB, 0.45841E+00_JPRB, 0.44636E+00_JPRB, 0.43897E+00_JPRB, 0.43851E+00_JPRB, 0.46676E+00_JPRB ,&
&  0.55828E+00_JPRB, 0.57512E+00_JPRB, 0.61580E+00_JPRB, 0.65290E+00_JPRB, 0.69566E+00_JPRB, 0.72941E+00_JPRB, 0.75563E+00_JPRB ,&
&  0.77278E+00_JPRB, 0.79091E+00_JPRB, 0.81883E+00_JPRB, 0.81189E+00_JPRB, 0.75933E+00_JPRB, 0.71098E+00_JPRB, 0.66506E+00_JPRB ,&
&  0.62223E+00_JPRB, 0.58381E+00_JPRB, 0.55153E+00_JPRB, 0.53835E+00_JPRB, 0.53606E+00_JPRB, 0.53896E+00_JPRB, 0.53868E+00_JPRB ,&
&  0.54075E+00_JPRB, 0.53903E+00_JPRB, 0.54585E+00_JPRB, 0.57209E+00_JPRB, 0.63197E+00_JPRB, 0.74902E+00_JPRB, 0.94644E+00_JPRB ,&
&  0.10414E+01_JPRB, 0.10657E+01_JPRB, 0.10934E+01_JPRB, 0.11187E+01_JPRB, 0.10986E+01_JPRB, 0.10674E+01_JPRB, 0.10358E+01_JPRB ,&
&  0.10024E+01_JPRB, 0.96600E+00_JPRB, 0.93404E+00_JPRB, 0.89424E+00_JPRB, 0.86051E+00_JPRB, 0.83657E+00_JPRB, 0.82011E+00_JPRB ,&
&  0.80772E+00_JPRB, 0.80271E+00_JPRB, 0.79349E+00_JPRB, 0.77357E+00_JPRB, 0.75978E+00_JPRB, 0.74514E+00_JPRB, 0.72146E+00_JPRB ,&
&  0.68863E+00_JPRB, 0.66816E+00_JPRB, 0.65695E+00_JPRB, 0.65533E+00_JPRB, 0.65837E+00_JPRB, 0.66263E+00_JPRB, 0.66242E+00_JPRB ,&
&  0.66530E+00_JPRB, 0.67727E+00_JPRB, 0.66872E+00_JPRB, 0.66072E+00_JPRB, 0.69018E+00_JPRB, 0.69048E+00_JPRB, 0.69339E+00_JPRB ,&
&  0.69619E+00_JPRB, 0.69535E+00_JPRB, 0.68792E+00_JPRB, 0.67019E+00_JPRB, 0.71240E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.61415E+00_JPRB, 0.61726E+00_JPRB, 0.60278E+00_JPRB, 0.58450E+00_JPRB, 0.56715E+00_JPRB, 0.55372E+00_JPRB, 0.54261E+00_JPRB ,&
&  0.53491E+00_JPRB, 0.52933E+00_JPRB, 0.52587E+00_JPRB, 0.52376E+00_JPRB, 0.52048E+00_JPRB, 0.51516E+00_JPRB, 0.50928E+00_JPRB ,&
&  0.50426E+00_JPRB, 0.49816E+00_JPRB, 0.48871E+00_JPRB, 0.48213E+00_JPRB, 0.47830E+00_JPRB, 0.47330E+00_JPRB, 0.46808E+00_JPRB ,&
&  0.46163E+00_JPRB, 0.44912E+00_JPRB, 0.43664E+00_JPRB, 0.42841E+00_JPRB, 0.42204E+00_JPRB, 0.41948E+00_JPRB, 0.43696E+00_JPRB ,&
&  0.49691E+00_JPRB, 0.47554E+00_JPRB, 0.51354E+00_JPRB, 0.55516E+00_JPRB, 0.60166E+00_JPRB, 0.63044E+00_JPRB, 0.64933E+00_JPRB ,&
&  0.66166E+00_JPRB, 0.67980E+00_JPRB, 0.69404E+00_JPRB, 0.71242E+00_JPRB, 0.68766E+00_JPRB, 0.64931E+00_JPRB, 0.61755E+00_JPRB ,&
&  0.58832E+00_JPRB, 0.56175E+00_JPRB, 0.53987E+00_JPRB, 0.52521E+00_JPRB, 0.52094E+00_JPRB, 0.51936E+00_JPRB, 0.51489E+00_JPRB ,&
&  0.51494E+00_JPRB, 0.51207E+00_JPRB, 0.51077E+00_JPRB, 0.51752E+00_JPRB, 0.51668E+00_JPRB, 0.54685E+00_JPRB, 0.59656E+00_JPRB ,&
&  0.64191E+00_JPRB, 0.70328E+00_JPRB, 0.73638E+00_JPRB, 0.75926E+00_JPRB, 0.75828E+00_JPRB, 0.75485E+00_JPRB, 0.74991E+00_JPRB ,&
&  0.74487E+00_JPRB, 0.73835E+00_JPRB, 0.72923E+00_JPRB, 0.71556E+00_JPRB, 0.69943E+00_JPRB, 0.68555E+00_JPRB, 0.67345E+00_JPRB ,&
&  0.66358E+00_JPRB, 0.65550E+00_JPRB, 0.65027E+00_JPRB, 0.63588E+00_JPRB, 0.61320E+00_JPRB, 0.59401E+00_JPRB, 0.57924E+00_JPRB ,&
&  0.56683E+00_JPRB, 0.56220E+00_JPRB, 0.56261E+00_JPRB, 0.56995E+00_JPRB, 0.57737E+00_JPRB, 0.58542E+00_JPRB, 0.59015E+00_JPRB ,&
&  0.59424E+00_JPRB, 0.60486E+00_JPRB, 0.62103E+00_JPRB, 0.63987E+00_JPRB, 0.64732E+00_JPRB, 0.65137E+00_JPRB, 0.65347E+00_JPRB ,&
&  0.65316E+00_JPRB, 0.65138E+00_JPRB, 0.64135E+00_JPRB, 0.62870E+00_JPRB, 0.60295E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.55659E+00_JPRB, 0.56022E+00_JPRB, 0.55418E+00_JPRB, 0.54242E+00_JPRB, 0.53191E+00_JPRB, 0.52234E+00_JPRB, 0.51480E+00_JPRB ,&
&  0.50393E+00_JPRB, 0.49475E+00_JPRB, 0.49159E+00_JPRB, 0.49160E+00_JPRB, 0.49093E+00_JPRB, 0.48695E+00_JPRB, 0.48142E+00_JPRB ,&
&  0.47525E+00_JPRB, 0.46820E+00_JPRB, 0.45992E+00_JPRB, 0.45165E+00_JPRB, 0.44487E+00_JPRB, 0.43700E+00_JPRB, 0.43075E+00_JPRB ,&
&  0.42490E+00_JPRB, 0.41977E+00_JPRB, 0.41163E+00_JPRB, 0.40160E+00_JPRB, 0.39713E+00_JPRB, 0.39199E+00_JPRB, 0.40769E+00_JPRB ,&
&  0.46042E+00_JPRB, 0.40069E+00_JPRB, 0.42770E+00_JPRB, 0.47544E+00_JPRB, 0.52025E+00_JPRB, 0.56887E+00_JPRB, 0.59156E+00_JPRB ,&
&  0.60560E+00_JPRB, 0.61466E+00_JPRB, 0.61981E+00_JPRB, 0.62922E+00_JPRB, 0.62789E+00_JPRB, 0.60471E+00_JPRB, 0.57406E+00_JPRB ,&
&  0.55129E+00_JPRB, 0.52948E+00_JPRB, 0.51533E+00_JPRB, 0.50166E+00_JPRB, 0.49554E+00_JPRB, 0.49348E+00_JPRB, 0.48666E+00_JPRB ,&
&  0.48107E+00_JPRB, 0.48394E+00_JPRB, 0.48887E+00_JPRB, 0.49673E+00_JPRB, 0.50220E+00_JPRB, 0.50734E+00_JPRB, 0.50083E+00_JPRB ,&
&  0.49536E+00_JPRB, 0.51610E+00_JPRB, 0.55129E+00_JPRB, 0.57167E+00_JPRB, 0.58230E+00_JPRB, 0.58080E+00_JPRB, 0.57364E+00_JPRB ,&
&  0.56975E+00_JPRB, 0.56751E+00_JPRB, 0.56916E+00_JPRB, 0.56678E+00_JPRB, 0.56166E+00_JPRB, 0.55222E+00_JPRB, 0.54095E+00_JPRB ,&
&  0.53025E+00_JPRB, 0.52093E+00_JPRB, 0.51367E+00_JPRB, 0.50687E+00_JPRB, 0.49862E+00_JPRB, 0.48722E+00_JPRB, 0.48094E+00_JPRB ,&
&  0.47821E+00_JPRB, 0.47924E+00_JPRB, 0.48354E+00_JPRB, 0.49069E+00_JPRB, 0.50067E+00_JPRB, 0.50980E+00_JPRB, 0.52085E+00_JPRB ,&
&  0.53117E+00_JPRB, 0.54225E+00_JPRB, 0.55621E+00_JPRB, 0.56151E+00_JPRB, 0.58840E+00_JPRB, 0.59627E+00_JPRB, 0.60391E+00_JPRB ,&
&  0.60621E+00_JPRB, 0.60594E+00_JPRB, 0.60448E+00_JPRB, 0.53155E+00_JPRB, 0.55191E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.50912E+00_JPRB, 0.50934E+00_JPRB, 0.50452E+00_JPRB, 0.49815E+00_JPRB, 0.49266E+00_JPRB, 0.48670E+00_JPRB, 0.48169E+00_JPRB ,&
&  0.47417E+00_JPRB, 0.46481E+00_JPRB, 0.45819E+00_JPRB, 0.45557E+00_JPRB, 0.45441E+00_JPRB, 0.45158E+00_JPRB, 0.44626E+00_JPRB ,&
&  0.43940E+00_JPRB, 0.43205E+00_JPRB, 0.42510E+00_JPRB, 0.41675E+00_JPRB, 0.40902E+00_JPRB, 0.40189E+00_JPRB, 0.39603E+00_JPRB ,&
&  0.39013E+00_JPRB, 0.38375E+00_JPRB, 0.37743E+00_JPRB, 0.37303E+00_JPRB, 0.36953E+00_JPRB, 0.36806E+00_JPRB, 0.38086E+00_JPRB ,&
&  0.42088E+00_JPRB, 0.36261E+00_JPRB, 0.38299E+00_JPRB, 0.41237E+00_JPRB, 0.45549E+00_JPRB, 0.50934E+00_JPRB, 0.53899E+00_JPRB ,&
&  0.55398E+00_JPRB, 0.56248E+00_JPRB, 0.56365E+00_JPRB, 0.56400E+00_JPRB, 0.56704E+00_JPRB, 0.55999E+00_JPRB, 0.54199E+00_JPRB ,&
&  0.52290E+00_JPRB, 0.50439E+00_JPRB, 0.49020E+00_JPRB, 0.47969E+00_JPRB, 0.47095E+00_JPRB, 0.46475E+00_JPRB, 0.45586E+00_JPRB ,&
&  0.44525E+00_JPRB, 0.44370E+00_JPRB, 0.44572E+00_JPRB, 0.44648E+00_JPRB, 0.44711E+00_JPRB, 0.44439E+00_JPRB, 0.43154E+00_JPRB ,&
&  0.42068E+00_JPRB, 0.41526E+00_JPRB, 0.41654E+00_JPRB, 0.42410E+00_JPRB, 0.43205E+00_JPRB, 0.43665E+00_JPRB, 0.43509E+00_JPRB ,&
&  0.43112E+00_JPRB, 0.42503E+00_JPRB, 0.42261E+00_JPRB, 0.42253E+00_JPRB, 0.42260E+00_JPRB, 0.42354E+00_JPRB, 0.42261E+00_JPRB ,&
&  0.41987E+00_JPRB, 0.41906E+00_JPRB, 0.42101E+00_JPRB, 0.42317E+00_JPRB, 0.42363E+00_JPRB, 0.42228E+00_JPRB, 0.42025E+00_JPRB ,&
&  0.41727E+00_JPRB, 0.41538E+00_JPRB, 0.41686E+00_JPRB, 0.42267E+00_JPRB, 0.43077E+00_JPRB, 0.43985E+00_JPRB, 0.44985E+00_JPRB ,&
&  0.46092E+00_JPRB, 0.47253E+00_JPRB, 0.48545E+00_JPRB, 0.50718E+00_JPRB, 0.51825E+00_JPRB, 0.53022E+00_JPRB, 0.54157E+00_JPRB ,&
&  0.54310E+00_JPRB, 0.54032E+00_JPRB, 0.46020E+00_JPRB, 0.49326E+00_JPRB, 0.50217E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.47961E+00_JPRB, 0.47087E+00_JPRB, 0.46047E+00_JPRB, 0.45397E+00_JPRB, 0.45182E+00_JPRB, 0.44914E+00_JPRB, 0.44626E+00_JPRB ,&
&  0.44126E+00_JPRB, 0.43546E+00_JPRB, 0.42978E+00_JPRB, 0.42470E+00_JPRB, 0.42052E+00_JPRB, 0.41754E+00_JPRB, 0.41360E+00_JPRB ,&
&  0.40774E+00_JPRB, 0.40099E+00_JPRB, 0.39410E+00_JPRB, 0.38632E+00_JPRB, 0.37613E+00_JPRB, 0.36798E+00_JPRB, 0.36155E+00_JPRB ,&
&  0.35648E+00_JPRB, 0.35249E+00_JPRB, 0.35099E+00_JPRB, 0.34914E+00_JPRB, 0.34713E+00_JPRB, 0.34727E+00_JPRB, 0.35832E+00_JPRB ,&
&  0.35602E+00_JPRB, 0.33049E+00_JPRB, 0.34836E+00_JPRB, 0.37258E+00_JPRB, 0.40928E+00_JPRB, 0.46471E+00_JPRB, 0.50220E+00_JPRB ,&
&  0.52269E+00_JPRB, 0.53232E+00_JPRB, 0.53506E+00_JPRB, 0.53113E+00_JPRB, 0.52596E+00_JPRB, 0.52532E+00_JPRB, 0.51943E+00_JPRB ,&
&  0.50577E+00_JPRB, 0.48884E+00_JPRB, 0.47236E+00_JPRB, 0.46015E+00_JPRB, 0.44716E+00_JPRB, 0.43695E+00_JPRB, 0.42611E+00_JPRB ,&
&  0.41632E+00_JPRB, 0.41071E+00_JPRB, 0.40923E+00_JPRB, 0.40779E+00_JPRB, 0.40548E+00_JPRB, 0.39865E+00_JPRB, 0.38948E+00_JPRB ,&
&  0.38285E+00_JPRB, 0.37712E+00_JPRB, 0.37124E+00_JPRB, 0.36613E+00_JPRB, 0.36297E+00_JPRB, 0.36088E+00_JPRB, 0.35888E+00_JPRB ,&
&  0.35866E+00_JPRB, 0.35967E+00_JPRB, 0.36186E+00_JPRB, 0.36437E+00_JPRB, 0.36587E+00_JPRB, 0.36558E+00_JPRB, 0.36569E+00_JPRB ,&
&  0.36623E+00_JPRB, 0.36423E+00_JPRB, 0.36419E+00_JPRB, 0.36844E+00_JPRB, 0.37386E+00_JPRB, 0.37748E+00_JPRB, 0.37878E+00_JPRB ,&
&  0.37753E+00_JPRB, 0.37660E+00_JPRB, 0.37733E+00_JPRB, 0.38026E+00_JPRB, 0.38424E+00_JPRB, 0.39017E+00_JPRB, 0.39739E+00_JPRB ,&
&  0.40492E+00_JPRB, 0.41438E+00_JPRB, 0.42726E+00_JPRB, 0.44566E+00_JPRB, 0.45969E+00_JPRB, 0.47046E+00_JPRB, 0.47907E+00_JPRB ,&
&  0.48050E+00_JPRB, 0.48032E+00_JPRB, 0.48456E+00_JPRB, 0.48418E+00_JPRB, 0.48338E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.42664E+00_JPRB, 0.42128E+00_JPRB, 0.41477E+00_JPRB, 0.40949E+00_JPRB, 0.40717E+00_JPRB, 0.40547E+00_JPRB, 0.40198E+00_JPRB ,&
&  0.39886E+00_JPRB, 0.39602E+00_JPRB, 0.39298E+00_JPRB, 0.39040E+00_JPRB, 0.38814E+00_JPRB, 0.38640E+00_JPRB, 0.38309E+00_JPRB ,&
&  0.37759E+00_JPRB, 0.37058E+00_JPRB, 0.36274E+00_JPRB, 0.35620E+00_JPRB, 0.34816E+00_JPRB, 0.33988E+00_JPRB, 0.33375E+00_JPRB ,&
&  0.32868E+00_JPRB, 0.32527E+00_JPRB, 0.32408E+00_JPRB, 0.32240E+00_JPRB, 0.32329E+00_JPRB, 0.32650E+00_JPRB, 0.33521E+00_JPRB ,&
&  0.34746E+00_JPRB, 0.32832E+00_JPRB, 0.34512E+00_JPRB, 0.36016E+00_JPRB, 0.39075E+00_JPRB, 0.43347E+00_JPRB, 0.46276E+00_JPRB ,&
&  0.48184E+00_JPRB, 0.49701E+00_JPRB, 0.50773E+00_JPRB, 0.50997E+00_JPRB, 0.50295E+00_JPRB, 0.49841E+00_JPRB, 0.49519E+00_JPRB ,&
&  0.48657E+00_JPRB, 0.47371E+00_JPRB, 0.45636E+00_JPRB, 0.43811E+00_JPRB, 0.42152E+00_JPRB, 0.40920E+00_JPRB, 0.39944E+00_JPRB ,&
&  0.39169E+00_JPRB, 0.38520E+00_JPRB, 0.38022E+00_JPRB, 0.37250E+00_JPRB, 0.36363E+00_JPRB, 0.35470E+00_JPRB, 0.34636E+00_JPRB ,&
&  0.33817E+00_JPRB, 0.33012E+00_JPRB, 0.32302E+00_JPRB, 0.31709E+00_JPRB, 0.31380E+00_JPRB, 0.31223E+00_JPRB, 0.31163E+00_JPRB ,&
&  0.31202E+00_JPRB, 0.31388E+00_JPRB, 0.31639E+00_JPRB, 0.31917E+00_JPRB, 0.32160E+00_JPRB, 0.32220E+00_JPRB, 0.32034E+00_JPRB ,&
&  0.31987E+00_JPRB, 0.31968E+00_JPRB, 0.32024E+00_JPRB, 0.32248E+00_JPRB, 0.32404E+00_JPRB, 0.32594E+00_JPRB, 0.32926E+00_JPRB ,&
&  0.33210E+00_JPRB, 0.33357E+00_JPRB, 0.33477E+00_JPRB, 0.33623E+00_JPRB, 0.33815E+00_JPRB, 0.34140E+00_JPRB, 0.34633E+00_JPRB ,&
&  0.35215E+00_JPRB, 0.35874E+00_JPRB, 0.36834E+00_JPRB, 0.38110E+00_JPRB, 0.39331E+00_JPRB, 0.40368E+00_JPRB, 0.41221E+00_JPRB ,&
&  0.41567E+00_JPRB, 0.41847E+00_JPRB, 0.42358E+00_JPRB, 0.42646E+00_JPRB, 0.42878E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.38354E+00_JPRB, 0.38087E+00_JPRB, 0.37775E+00_JPRB, 0.37526E+00_JPRB, 0.37360E+00_JPRB, 0.37028E+00_JPRB, 0.36622E+00_JPRB ,&
&  0.36372E+00_JPRB, 0.36149E+00_JPRB, 0.36001E+00_JPRB, 0.35736E+00_JPRB, 0.35618E+00_JPRB, 0.35562E+00_JPRB, 0.35200E+00_JPRB ,&
&  0.34747E+00_JPRB, 0.34074E+00_JPRB, 0.33400E+00_JPRB, 0.32785E+00_JPRB, 0.32196E+00_JPRB, 0.31643E+00_JPRB, 0.31033E+00_JPRB ,&
&  0.30466E+00_JPRB, 0.30046E+00_JPRB, 0.29822E+00_JPRB, 0.29633E+00_JPRB, 0.29774E+00_JPRB, 0.30374E+00_JPRB, 0.31342E+00_JPRB ,&
&  0.32000E+00_JPRB, 0.32371E+00_JPRB, 0.33568E+00_JPRB, 0.34793E+00_JPRB, 0.37095E+00_JPRB, 0.40243E+00_JPRB, 0.42460E+00_JPRB ,&
&  0.43866E+00_JPRB, 0.45437E+00_JPRB, 0.47182E+00_JPRB, 0.48241E+00_JPRB, 0.48321E+00_JPRB, 0.47699E+00_JPRB, 0.47226E+00_JPRB ,&
&  0.46614E+00_JPRB, 0.45466E+00_JPRB, 0.44225E+00_JPRB, 0.42697E+00_JPRB, 0.41081E+00_JPRB, 0.39747E+00_JPRB, 0.38691E+00_JPRB ,&
&  0.37731E+00_JPRB, 0.36811E+00_JPRB, 0.35957E+00_JPRB, 0.35073E+00_JPRB, 0.34104E+00_JPRB, 0.33064E+00_JPRB, 0.31962E+00_JPRB ,&
&  0.30893E+00_JPRB, 0.29992E+00_JPRB, 0.29298E+00_JPRB, 0.28942E+00_JPRB, 0.28753E+00_JPRB, 0.28589E+00_JPRB, 0.28474E+00_JPRB ,&
&  0.28495E+00_JPRB, 0.28616E+00_JPRB, 0.28807E+00_JPRB, 0.29034E+00_JPRB, 0.29256E+00_JPRB, 0.29350E+00_JPRB, 0.29178E+00_JPRB ,&
&  0.28974E+00_JPRB, 0.28956E+00_JPRB, 0.29017E+00_JPRB, 0.28835E+00_JPRB, 0.28611E+00_JPRB, 0.28690E+00_JPRB, 0.28953E+00_JPRB ,&
&  0.29408E+00_JPRB, 0.29818E+00_JPRB, 0.30146E+00_JPRB, 0.30407E+00_JPRB, 0.30595E+00_JPRB, 0.30917E+00_JPRB, 0.31315E+00_JPRB ,&
&  0.31825E+00_JPRB, 0.32472E+00_JPRB, 0.33384E+00_JPRB, 0.34530E+00_JPRB, 0.35576E+00_JPRB, 0.36434E+00_JPRB, 0.36938E+00_JPRB ,&
&  0.37241E+00_JPRB, 0.37453E+00_JPRB, 0.37668E+00_JPRB, 0.37990E+00_JPRB, 0.38294E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.34096E+00_JPRB, 0.34097E+00_JPRB, 0.34165E+00_JPRB, 0.34186E+00_JPRB, 0.33910E+00_JPRB, 0.33627E+00_JPRB, 0.33299E+00_JPRB ,&
&  0.33162E+00_JPRB, 0.33071E+00_JPRB, 0.33056E+00_JPRB, 0.32797E+00_JPRB, 0.32614E+00_JPRB, 0.32529E+00_JPRB, 0.32212E+00_JPRB ,&
&  0.31886E+00_JPRB, 0.31128E+00_JPRB, 0.30308E+00_JPRB, 0.29698E+00_JPRB, 0.29311E+00_JPRB, 0.29042E+00_JPRB, 0.28510E+00_JPRB ,&
&  0.28126E+00_JPRB, 0.27823E+00_JPRB, 0.27680E+00_JPRB, 0.27694E+00_JPRB, 0.27914E+00_JPRB, 0.28513E+00_JPRB, 0.29504E+00_JPRB ,&
&  0.30229E+00_JPRB, 0.31205E+00_JPRB, 0.31280E+00_JPRB, 0.32753E+00_JPRB, 0.34243E+00_JPRB, 0.36320E+00_JPRB, 0.38121E+00_JPRB ,&
&  0.39333E+00_JPRB, 0.40269E+00_JPRB, 0.41541E+00_JPRB, 0.42852E+00_JPRB, 0.43453E+00_JPRB, 0.43642E+00_JPRB, 0.43499E+00_JPRB ,&
&  0.43129E+00_JPRB, 0.42565E+00_JPRB, 0.41905E+00_JPRB, 0.41081E+00_JPRB, 0.39953E+00_JPRB, 0.38658E+00_JPRB, 0.37378E+00_JPRB ,&
&  0.36212E+00_JPRB, 0.35070E+00_JPRB, 0.33940E+00_JPRB, 0.32714E+00_JPRB, 0.31483E+00_JPRB, 0.30341E+00_JPRB, 0.29200E+00_JPRB ,&
&  0.28254E+00_JPRB, 0.27530E+00_JPRB, 0.27000E+00_JPRB, 0.26623E+00_JPRB, 0.26342E+00_JPRB, 0.26067E+00_JPRB, 0.25940E+00_JPRB ,&
&  0.25949E+00_JPRB, 0.26107E+00_JPRB, 0.26352E+00_JPRB, 0.26556E+00_JPRB, 0.26805E+00_JPRB, 0.26857E+00_JPRB, 0.26582E+00_JPRB ,&
&  0.26289E+00_JPRB, 0.26198E+00_JPRB, 0.25830E+00_JPRB, 0.25619E+00_JPRB, 0.25590E+00_JPRB, 0.25650E+00_JPRB, 0.25852E+00_JPRB ,&
&  0.26243E+00_JPRB, 0.26780E+00_JPRB, 0.27281E+00_JPRB, 0.27721E+00_JPRB, 0.28108E+00_JPRB, 0.28492E+00_JPRB, 0.28934E+00_JPRB ,&
&  0.29402E+00_JPRB, 0.29854E+00_JPRB, 0.30558E+00_JPRB, 0.31352E+00_JPRB, 0.32031E+00_JPRB, 0.32759E+00_JPRB, 0.33196E+00_JPRB ,&
&  0.33360E+00_JPRB, 0.33416E+00_JPRB, 0.33608E+00_JPRB, 0.33902E+00_JPRB, 0.34085E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.30578E+00_JPRB, 0.30732E+00_JPRB, 0.30926E+00_JPRB, 0.30954E+00_JPRB, 0.30892E+00_JPRB, 0.30881E+00_JPRB, 0.30776E+00_JPRB ,&
&  0.30552E+00_JPRB, 0.30436E+00_JPRB, 0.30468E+00_JPRB, 0.30343E+00_JPRB, 0.30084E+00_JPRB, 0.29766E+00_JPRB, 0.29420E+00_JPRB ,&
&  0.29133E+00_JPRB, 0.28827E+00_JPRB, 0.28296E+00_JPRB, 0.27596E+00_JPRB, 0.26993E+00_JPRB, 0.26601E+00_JPRB, 0.26392E+00_JPRB ,&
&  0.26299E+00_JPRB, 0.26293E+00_JPRB, 0.26254E+00_JPRB, 0.26317E+00_JPRB, 0.26530E+00_JPRB, 0.27021E+00_JPRB, 0.27750E+00_JPRB ,&
&  0.28481E+00_JPRB, 0.29174E+00_JPRB, 0.29880E+00_JPRB, 0.30658E+00_JPRB, 0.31585E+00_JPRB, 0.32725E+00_JPRB, 0.33802E+00_JPRB ,&
&  0.34798E+00_JPRB, 0.35157E+00_JPRB, 0.35322E+00_JPRB, 0.35839E+00_JPRB, 0.36769E+00_JPRB, 0.37440E+00_JPRB, 0.37867E+00_JPRB ,&
&  0.38088E+00_JPRB, 0.37978E+00_JPRB, 0.37735E+00_JPRB, 0.37193E+00_JPRB, 0.36397E+00_JPRB, 0.35373E+00_JPRB, 0.34271E+00_JPRB ,&
&  0.33184E+00_JPRB, 0.32061E+00_JPRB, 0.30852E+00_JPRB, 0.29693E+00_JPRB, 0.28688E+00_JPRB, 0.27790E+00_JPRB, 0.26905E+00_JPRB ,&
&  0.26103E+00_JPRB, 0.25469E+00_JPRB, 0.24969E+00_JPRB, 0.24627E+00_JPRB, 0.24348E+00_JPRB, 0.24130E+00_JPRB, 0.24065E+00_JPRB ,&
&  0.24093E+00_JPRB, 0.24224E+00_JPRB, 0.24296E+00_JPRB, 0.24450E+00_JPRB, 0.24707E+00_JPRB, 0.24808E+00_JPRB, 0.24684E+00_JPRB ,&
&  0.24281E+00_JPRB, 0.23887E+00_JPRB, 0.23519E+00_JPRB, 0.23472E+00_JPRB, 0.23488E+00_JPRB, 0.23632E+00_JPRB, 0.23833E+00_JPRB ,&
&  0.24213E+00_JPRB, 0.24738E+00_JPRB, 0.25215E+00_JPRB, 0.25570E+00_JPRB, 0.25980E+00_JPRB, 0.26343E+00_JPRB, 0.26701E+00_JPRB ,&
&  0.27095E+00_JPRB, 0.27512E+00_JPRB, 0.28061E+00_JPRB, 0.28614E+00_JPRB, 0.29116E+00_JPRB, 0.29510E+00_JPRB, 0.29794E+00_JPRB ,&
&  0.29972E+00_JPRB, 0.30013E+00_JPRB, 0.30035E+00_JPRB, 0.30185E+00_JPRB, 0.30411E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.27469E+00_JPRB, 0.27680E+00_JPRB, 0.27958E+00_JPRB, 0.28183E+00_JPRB, 0.28262E+00_JPRB, 0.28208E+00_JPRB, 0.28126E+00_JPRB ,&
&  0.28105E+00_JPRB, 0.28091E+00_JPRB, 0.27947E+00_JPRB, 0.27703E+00_JPRB, 0.27458E+00_JPRB, 0.27201E+00_JPRB, 0.27023E+00_JPRB ,&
&  0.26847E+00_JPRB, 0.26642E+00_JPRB, 0.26267E+00_JPRB, 0.25708E+00_JPRB, 0.25128E+00_JPRB, 0.24793E+00_JPRB, 0.24638E+00_JPRB ,&
&  0.24631E+00_JPRB, 0.24687E+00_JPRB, 0.24938E+00_JPRB, 0.25114E+00_JPRB, 0.25358E+00_JPRB, 0.25768E+00_JPRB, 0.26336E+00_JPRB ,&
&  0.26872E+00_JPRB, 0.27387E+00_JPRB, 0.27917E+00_JPRB, 0.28376E+00_JPRB, 0.28833E+00_JPRB, 0.29402E+00_JPRB, 0.30025E+00_JPRB ,&
&  0.30743E+00_JPRB, 0.31228E+00_JPRB, 0.31288E+00_JPRB, 0.31375E+00_JPRB, 0.31661E+00_JPRB, 0.32014E+00_JPRB, 0.32250E+00_JPRB ,&
&  0.32424E+00_JPRB, 0.32287E+00_JPRB, 0.32036E+00_JPRB, 0.31644E+00_JPRB, 0.31204E+00_JPRB, 0.30702E+00_JPRB, 0.30026E+00_JPRB ,&
&  0.29214E+00_JPRB, 0.28334E+00_JPRB, 0.27490E+00_JPRB, 0.26687E+00_JPRB, 0.25917E+00_JPRB, 0.25206E+00_JPRB, 0.24566E+00_JPRB ,&
&  0.23961E+00_JPRB, 0.23496E+00_JPRB, 0.23045E+00_JPRB, 0.22769E+00_JPRB, 0.22600E+00_JPRB, 0.22402E+00_JPRB, 0.22357E+00_JPRB ,&
&  0.22262E+00_JPRB, 0.22211E+00_JPRB, 0.22205E+00_JPRB, 0.22366E+00_JPRB, 0.22676E+00_JPRB, 0.22864E+00_JPRB, 0.22742E+00_JPRB ,&
&  0.22412E+00_JPRB, 0.22213E+00_JPRB, 0.22103E+00_JPRB, 0.22022E+00_JPRB, 0.21797E+00_JPRB, 0.21777E+00_JPRB, 0.21982E+00_JPRB ,&
&  0.22428E+00_JPRB, 0.22904E+00_JPRB, 0.23297E+00_JPRB, 0.23539E+00_JPRB, 0.23802E+00_JPRB, 0.24026E+00_JPRB, 0.24225E+00_JPRB ,&
&  0.24438E+00_JPRB, 0.24593E+00_JPRB, 0.24708E+00_JPRB, 0.24860E+00_JPRB, 0.25105E+00_JPRB, 0.25341E+00_JPRB, 0.25572E+00_JPRB ,&
&  0.25825E+00_JPRB, 0.26221E+00_JPRB, 0.26624E+00_JPRB, 0.26919E+00_JPRB, 0.27261E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.25291E+00_JPRB, 0.25554E+00_JPRB, 0.25806E+00_JPRB, 0.25813E+00_JPRB, 0.25629E+00_JPRB, 0.25535E+00_JPRB, 0.25546E+00_JPRB ,&
&  0.25577E+00_JPRB, 0.25573E+00_JPRB, 0.25440E+00_JPRB, 0.25428E+00_JPRB, 0.25432E+00_JPRB, 0.25347E+00_JPRB, 0.25296E+00_JPRB ,&
&  0.25186E+00_JPRB, 0.24896E+00_JPRB, 0.24473E+00_JPRB, 0.23952E+00_JPRB, 0.23566E+00_JPRB, 0.23370E+00_JPRB, 0.23351E+00_JPRB ,&
&  0.23435E+00_JPRB, 0.23497E+00_JPRB, 0.23751E+00_JPRB, 0.23965E+00_JPRB, 0.24232E+00_JPRB, 0.24590E+00_JPRB, 0.24970E+00_JPRB ,&
&  0.25249E+00_JPRB, 0.25545E+00_JPRB, 0.25845E+00_JPRB, 0.26128E+00_JPRB, 0.26499E+00_JPRB, 0.27053E+00_JPRB, 0.27520E+00_JPRB ,&
&  0.28043E+00_JPRB, 0.28321E+00_JPRB, 0.28054E+00_JPRB, 0.27742E+00_JPRB, 0.27726E+00_JPRB, 0.27900E+00_JPRB, 0.28088E+00_JPRB ,&
&  0.28253E+00_JPRB, 0.28424E+00_JPRB, 0.28562E+00_JPRB, 0.28529E+00_JPRB, 0.28258E+00_JPRB, 0.27771E+00_JPRB, 0.27120E+00_JPRB ,&
&  0.26406E+00_JPRB, 0.25687E+00_JPRB, 0.24999E+00_JPRB, 0.24315E+00_JPRB, 0.23731E+00_JPRB, 0.23167E+00_JPRB, 0.22650E+00_JPRB ,&
&  0.22165E+00_JPRB, 0.21703E+00_JPRB, 0.21350E+00_JPRB, 0.21125E+00_JPRB, 0.21048E+00_JPRB, 0.20917E+00_JPRB, 0.20901E+00_JPRB ,&
&  0.20843E+00_JPRB, 0.20825E+00_JPRB, 0.20847E+00_JPRB, 0.20937E+00_JPRB, 0.21078E+00_JPRB, 0.21130E+00_JPRB, 0.21047E+00_JPRB ,&
&  0.21001E+00_JPRB, 0.21126E+00_JPRB, 0.21108E+00_JPRB, 0.20871E+00_JPRB, 0.20576E+00_JPRB, 0.20507E+00_JPRB, 0.20648E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.21215E+00_JPRB, 0.21473E+00_JPRB, 0.21698E+00_JPRB, 0.21887E+00_JPRB, 0.22119E+00_JPRB, 0.22329E+00_JPRB ,&
&  0.22508E+00_JPRB, 0.22604E+00_JPRB, 0.22766E+00_JPRB, 0.22939E+00_JPRB, 0.23119E+00_JPRB, 0.23245E+00_JPRB, 0.23232E+00_JPRB ,&
&  0.23183E+00_JPRB, 0.23338E+00_JPRB, 0.23824E+00_JPRB, 0.24547E+00_JPRB, 0.25001E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.23454E+00_JPRB, 0.23681E+00_JPRB, 0.23803E+00_JPRB, 0.23779E+00_JPRB, 0.23715E+00_JPRB, 0.23778E+00_JPRB, 0.23826E+00_JPRB ,&
&  0.23886E+00_JPRB, 0.23892E+00_JPRB, 0.23759E+00_JPRB, 0.23642E+00_JPRB, 0.23800E+00_JPRB, 0.23758E+00_JPRB, 0.23716E+00_JPRB ,&
&  0.23571E+00_JPRB, 0.23391E+00_JPRB, 0.23111E+00_JPRB, 0.22790E+00_JPRB, 0.22399E+00_JPRB, 0.22168E+00_JPRB, 0.22215E+00_JPRB ,&
&  0.22382E+00_JPRB, 0.22447E+00_JPRB, 0.22529E+00_JPRB, 0.22632E+00_JPRB, 0.22848E+00_JPRB, 0.23128E+00_JPRB, 0.23336E+00_JPRB ,&
&  0.23501E+00_JPRB, 0.23808E+00_JPRB, 0.24002E+00_JPRB, 0.23908E+00_JPRB, 0.24335E+00_JPRB, 0.24746E+00_JPRB, 0.25115E+00_JPRB ,&
&  0.25450E+00_JPRB, 0.25654E+00_JPRB, 0.25661E+00_JPRB, 0.25558E+00_JPRB, 0.25474E+00_JPRB, 0.25567E+00_JPRB, 0.25616E+00_JPRB ,&
&  0.25680E+00_JPRB, 0.25930E+00_JPRB, 0.26218E+00_JPRB, 0.26319E+00_JPRB, 0.26156E+00_JPRB, 0.25723E+00_JPRB, 0.25043E+00_JPRB ,&
&  0.24167E+00_JPRB, 0.23246E+00_JPRB, 0.22495E+00_JPRB, 0.21899E+00_JPRB, 0.21442E+00_JPRB, 0.21073E+00_JPRB, 0.20772E+00_JPRB ,&
&  0.20438E+00_JPRB, 0.20198E+00_JPRB, 0.20185E+00_JPRB, 0.20239E+00_JPRB, 0.20238E+00_JPRB, 0.20067E+00_JPRB, 0.19947E+00_JPRB ,&
&  0.19840E+00_JPRB, 0.19884E+00_JPRB, 0.19978E+00_JPRB, 0.20016E+00_JPRB, 0.20031E+00_JPRB, 0.20010E+00_JPRB, 0.19993E+00_JPRB ,&
&  0.20083E+00_JPRB, 0.20094E+00_JPRB, 0.19991E+00_JPRB, 0.19806E+00_JPRB, 0.19649E+00_JPRB, 0.19625E+00_JPRB, 0.19663E+00_JPRB ,&
&  0.19737E+00_JPRB, 0.19882E+00_JPRB, 0.20081E+00_JPRB, 0.20286E+00_JPRB, 0.20445E+00_JPRB, 0.20519E+00_JPRB, 0.20550E+00_JPRB ,&
&  0.20573E+00_JPRB, 0.20569E+00_JPRB, 0.20683E+00_JPRB, 0.21010E+00_JPRB, 0.21345E+00_JPRB, 0.21608E+00_JPRB, 0.21752E+00_JPRB ,&
&  0.21709E+00_JPRB, 0.21775E+00_JPRB, 0.22086E+00_JPRB, 0.22668E+00_JPRB, 0.23105E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.22081E+00_JPRB, 0.22294E+00_JPRB, 0.22358E+00_JPRB, 0.22311E+00_JPRB, 0.22253E+00_JPRB, 0.22338E+00_JPRB, 0.22464E+00_JPRB ,&
&  0.22527E+00_JPRB, 0.22532E+00_JPRB, 0.22432E+00_JPRB, 0.22240E+00_JPRB, 0.22092E+00_JPRB, 0.22107E+00_JPRB, 0.21953E+00_JPRB ,&
&  0.21826E+00_JPRB, 0.21875E+00_JPRB, 0.21778E+00_JPRB, 0.21654E+00_JPRB, 0.21440E+00_JPRB, 0.21323E+00_JPRB, 0.21396E+00_JPRB ,&
&  0.21615E+00_JPRB, 0.21677E+00_JPRB, 0.21742E+00_JPRB, 0.21829E+00_JPRB, 0.21995E+00_JPRB, 0.22198E+00_JPRB, 0.22327E+00_JPRB ,&
&  0.22415E+00_JPRB, 0.22321E+00_JPRB, 0.22164E+00_JPRB, 0.22427E+00_JPRB, 0.22752E+00_JPRB, 0.23174E+00_JPRB, 0.23456E+00_JPRB ,&
&  0.23703E+00_JPRB, 0.23890E+00_JPRB, 0.23980E+00_JPRB, 0.23973E+00_JPRB, 0.24005E+00_JPRB, 0.24180E+00_JPRB, 0.24311E+00_JPRB ,&
&  0.24360E+00_JPRB, 0.24328E+00_JPRB, 0.24185E+00_JPRB, 0.24046E+00_JPRB, 0.23703E+00_JPRB, 0.23187E+00_JPRB, 0.22416E+00_JPRB ,&
&  0.21612E+00_JPRB, 0.20965E+00_JPRB, 0.20478E+00_JPRB, 0.20087E+00_JPRB, 0.19816E+00_JPRB, 0.19791E+00_JPRB, 0.19788E+00_JPRB ,&
&  0.19743E+00_JPRB, 0.19697E+00_JPRB, 0.19830E+00_JPRB, 0.19894E+00_JPRB, 0.19475E+00_JPRB, 0.18967E+00_JPRB, 0.18512E+00_JPRB ,&
&  0.18405E+00_JPRB, 0.18498E+00_JPRB, 0.18640E+00_JPRB, 0.18855E+00_JPRB, 0.19203E+00_JPRB, 0.19282E+00_JPRB, 0.19177E+00_JPRB ,&
&  0.18631E+00_JPRB, 0.18108E+00_JPRB, 0.17798E+00_JPRB, 0.17554E+00_JPRB, 0.17409E+00_JPRB, 0.17418E+00_JPRB, 0.17496E+00_JPRB ,&
&  0.17718E+00_JPRB, 0.17900E+00_JPRB, 0.18151E+00_JPRB, 0.18287E+00_JPRB, 0.18271E+00_JPRB, 0.18250E+00_JPRB, 0.18181E+00_JPRB ,&
&  0.18324E+00_JPRB, 0.18772E+00_JPRB, 0.19442E+00_JPRB, 0.19746E+00_JPRB, 0.20165E+00_JPRB, 0.20453E+00_JPRB, 0.20592E+00_JPRB ,&
&  0.20680E+00_JPRB, 0.20726E+00_JPRB, 0.20761E+00_JPRB, 0.21171E+00_JPRB, 0.21732E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.20928E+00_JPRB, 0.21099E+00_JPRB, 0.21153E+00_JPRB, 0.21147E+00_JPRB, 0.21121E+00_JPRB, 0.21159E+00_JPRB, 0.21205E+00_JPRB ,&
&  0.21196E+00_JPRB, 0.21120E+00_JPRB, 0.20997E+00_JPRB, 0.20786E+00_JPRB, 0.20527E+00_JPRB, 0.20373E+00_JPRB, 0.20472E+00_JPRB ,&
&  0.20603E+00_JPRB, 0.20662E+00_JPRB, 0.20732E+00_JPRB, 0.20756E+00_JPRB, 0.20734E+00_JPRB, 0.20770E+00_JPRB, 0.20897E+00_JPRB ,&
&  0.21030E+00_JPRB, 0.21106E+00_JPRB, 0.21234E+00_JPRB, 0.21338E+00_JPRB, 0.21536E+00_JPRB, 0.21716E+00_JPRB, 0.21773E+00_JPRB ,&
&  0.21490E+00_JPRB, 0.20710E+00_JPRB, 0.20184E+00_JPRB, 0.20436E+00_JPRB, 0.21224E+00_JPRB, 0.21742E+00_JPRB, 0.22066E+00_JPRB ,&
&  0.22287E+00_JPRB, 0.22397E+00_JPRB, 0.22420E+00_JPRB, 0.22549E+00_JPRB, 0.22747E+00_JPRB, 0.22938E+00_JPRB, 0.22901E+00_JPRB ,&
&  0.22703E+00_JPRB, 0.22533E+00_JPRB, 0.22285E+00_JPRB, 0.21850E+00_JPRB, 0.21258E+00_JPRB, 0.20676E+00_JPRB, 0.20176E+00_JPRB ,&
&  0.19729E+00_JPRB, 0.19298E+00_JPRB, 0.18940E+00_JPRB, 0.18709E+00_JPRB, 0.18644E+00_JPRB, 0.18780E+00_JPRB, 0.18752E+00_JPRB ,&
&  0.18761E+00_JPRB, 0.18581E+00_JPRB, 0.18361E+00_JPRB, 0.18099E+00_JPRB, 0.17875E+00_JPRB, 0.17465E+00_JPRB, 0.16955E+00_JPRB ,&
&  0.16695E+00_JPRB, 0.16943E+00_JPRB, 0.17620E+00_JPRB, 0.18158E+00_JPRB, 0.18104E+00_JPRB, 0.17416E+00_JPRB, 0.17036E+00_JPRB ,&
&  0.16633E+00_JPRB, 0.16223E+00_JPRB, 0.15959E+00_JPRB, 0.15899E+00_JPRB, 0.15918E+00_JPRB, 0.15946E+00_JPRB, 0.16051E+00_JPRB ,&
&  0.16216E+00_JPRB, 0.16360E+00_JPRB, 0.16440E+00_JPRB, 0.16534E+00_JPRB, 0.16575E+00_JPRB, 0.16584E+00_JPRB, 0.16598E+00_JPRB ,&
&  0.16764E+00_JPRB, 0.17106E+00_JPRB, 0.17561E+00_JPRB, 0.18067E+00_JPRB, 0.18455E+00_JPRB, 0.18810E+00_JPRB, 0.19124E+00_JPRB ,&
&  0.19440E+00_JPRB, 0.19612E+00_JPRB, 0.19909E+00_JPRB, 0.20315E+00_JPRB, 0.20650E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.19620E+00_JPRB, 0.19994E+00_JPRB, 0.20143E+00_JPRB, 0.20143E+00_JPRB, 0.20033E+00_JPRB, 0.19936E+00_JPRB, 0.19911E+00_JPRB ,&
&  0.19891E+00_JPRB, 0.19779E+00_JPRB, 0.19695E+00_JPRB, 0.19632E+00_JPRB, 0.19562E+00_JPRB, 0.19481E+00_JPRB, 0.19506E+00_JPRB ,&
&  0.19633E+00_JPRB, 0.19818E+00_JPRB, 0.19964E+00_JPRB, 0.20041E+00_JPRB, 0.20012E+00_JPRB, 0.20055E+00_JPRB, 0.20147E+00_JPRB ,&
&  0.20146E+00_JPRB, 0.20159E+00_JPRB, 0.20475E+00_JPRB, 0.20499E+00_JPRB, 0.20585E+00_JPRB, 0.20583E+00_JPRB, 0.20583E+00_JPRB ,&
&  0.20094E+00_JPRB, 0.19563E+00_JPRB, 0.19288E+00_JPRB, 0.19507E+00_JPRB, 0.20303E+00_JPRB, 0.20447E+00_JPRB, 0.20795E+00_JPRB ,&
&  0.21011E+00_JPRB, 0.21113E+00_JPRB, 0.21255E+00_JPRB, 0.21378E+00_JPRB, 0.21526E+00_JPRB, 0.21449E+00_JPRB, 0.21232E+00_JPRB ,&
&  0.20949E+00_JPRB, 0.20848E+00_JPRB, 0.20519E+00_JPRB, 0.19103E+00_JPRB, 0.18242E+00_JPRB, 0.17752E+00_JPRB, 0.17237E+00_JPRB ,&
&  0.16768E+00_JPRB, 0.16514E+00_JPRB, 0.16454E+00_JPRB, 0.16320E+00_JPRB, 0.16408E+00_JPRB, 0.16622E+00_JPRB, 0.16716E+00_JPRB ,&
&  0.16690E+00_JPRB, 0.16736E+00_JPRB, 0.16707E+00_JPRB, 0.16612E+00_JPRB, 0.16436E+00_JPRB, 0.16327E+00_JPRB, 0.16268E+00_JPRB ,&
&  0.16323E+00_JPRB, 0.16295E+00_JPRB, 0.16694E+00_JPRB, 0.16817E+00_JPRB, 0.16311E+00_JPRB, 0.16182E+00_JPRB, 0.16004E+00_JPRB ,&
&  0.15485E+00_JPRB, 0.15194E+00_JPRB, 0.15031E+00_JPRB, 0.14968E+00_JPRB, 0.15069E+00_JPRB, 0.15214E+00_JPRB, 0.15289E+00_JPRB ,&
&  0.15317E+00_JPRB, 0.15426E+00_JPRB, 0.15477E+00_JPRB, 0.15533E+00_JPRB, 0.15572E+00_JPRB, 0.15657E+00_JPRB, 0.15783E+00_JPRB ,&
&  0.15939E+00_JPRB, 0.16222E+00_JPRB, 0.16500E+00_JPRB, 0.16812E+00_JPRB, 0.16996E+00_JPRB, 0.17139E+00_JPRB, 0.17473E+00_JPRB ,&
&  0.17667E+00_JPRB, 0.18374E+00_JPRB, 0.19401E+00_JPRB, 0.19685E+00_JPRB, 0.19170E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.18899E+00_JPRB, 0.19084E+00_JPRB, 0.19240E+00_JPRB, 0.19147E+00_JPRB, 0.19164E+00_JPRB, 0.19158E+00_JPRB, 0.19164E+00_JPRB ,&
&  0.19155E+00_JPRB, 0.19094E+00_JPRB, 0.19121E+00_JPRB, 0.18938E+00_JPRB, 0.18837E+00_JPRB, 0.18500E+00_JPRB, 0.18398E+00_JPRB ,&
&  0.18220E+00_JPRB, 0.18194E+00_JPRB, 0.18226E+00_JPRB, 0.18355E+00_JPRB, 0.18399E+00_JPRB, 0.18534E+00_JPRB, 0.18458E+00_JPRB ,&
&  0.18486E+00_JPRB, 0.18492E+00_JPRB, 0.18362E+00_JPRB, 0.18321E+00_JPRB, 0.18357E+00_JPRB, 0.18393E+00_JPRB, 0.18422E+00_JPRB ,&
&  0.18617E+00_JPRB, 0.18843E+00_JPRB, 0.19058E+00_JPRB, 0.19184E+00_JPRB, 0.19670E+00_JPRB, 0.19807E+00_JPRB, 0.20013E+00_JPRB ,&
&  0.20124E+00_JPRB, 0.20261E+00_JPRB, 0.20364E+00_JPRB, 0.20299E+00_JPRB, 0.20177E+00_JPRB, 0.19802E+00_JPRB, 0.19663E+00_JPRB ,&
&  0.18662E+00_JPRB, 0.17977E+00_JPRB, 0.17298E+00_JPRB, 0.16690E+00_JPRB, 0.16239E+00_JPRB, 0.15928E+00_JPRB, 0.15677E+00_JPRB ,&
&  0.15471E+00_JPRB, 0.15271E+00_JPRB, 0.15051E+00_JPRB, 0.15060E+00_JPRB, 0.15140E+00_JPRB, 0.15265E+00_JPRB, 0.15391E+00_JPRB ,&
&  0.15535E+00_JPRB, 0.15619E+00_JPRB, 0.15658E+00_JPRB, 0.15737E+00_JPRB, 0.15758E+00_JPRB, 0.15797E+00_JPRB, 0.15824E+00_JPRB ,&
&  0.15960E+00_JPRB, 0.16006E+00_JPRB, 0.16012E+00_JPRB, 0.16159E+00_JPRB, 0.15980E+00_JPRB, 0.15704E+00_JPRB, 0.15407E+00_JPRB ,&
&  0.15108E+00_JPRB, 0.14920E+00_JPRB, 0.14762E+00_JPRB, 0.14701E+00_JPRB, 0.14743E+00_JPRB, 0.14819E+00_JPRB, 0.14918E+00_JPRB ,&
&  0.14941E+00_JPRB, 0.15004E+00_JPRB, 0.15052E+00_JPRB, 0.15139E+00_JPRB, 0.15232E+00_JPRB, 0.15338E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15634E+00_JPRB, 0.15839E+00_JPRB, 0.16045E+00_JPRB, 0.16255E+00_JPRB, 0.16591E+00_JPRB, 0.16994E+00_JPRB, 0.17630E+00_JPRB ,&
&  0.18183E+00_JPRB, 0.18886E+00_JPRB, 0.18722E+00_JPRB, 0.19005E+00_JPRB, 0.19032E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.18430E+00_JPRB, 0.18508E+00_JPRB, 0.18317E+00_JPRB, 0.18353E+00_JPRB, 0.18428E+00_JPRB, 0.18468E+00_JPRB, 0.18515E+00_JPRB ,&
&  0.18269E+00_JPRB, 0.18188E+00_JPRB, 0.17960E+00_JPRB, 0.17762E+00_JPRB, 0.17518E+00_JPRB, 0.17266E+00_JPRB, 0.17073E+00_JPRB ,&
&  0.16887E+00_JPRB, 0.16820E+00_JPRB, 0.16732E+00_JPRB, 0.16815E+00_JPRB, 0.16817E+00_JPRB, 0.16984E+00_JPRB, 0.17120E+00_JPRB ,&
&  0.17194E+00_JPRB, 0.17297E+00_JPRB, 0.17161E+00_JPRB, 0.17131E+00_JPRB, 0.17300E+00_JPRB, 0.17453E+00_JPRB, 0.17714E+00_JPRB ,&
&  0.17986E+00_JPRB, 0.18297E+00_JPRB, 0.18805E+00_JPRB, 0.18986E+00_JPRB, 0.19211E+00_JPRB, 0.19352E+00_JPRB, 0.19349E+00_JPRB ,&
&  0.19321E+00_JPRB, 0.19313E+00_JPRB, 0.19395E+00_JPRB, 0.19365E+00_JPRB, 0.18676E+00_JPRB, 0.18151E+00_JPRB, 0.17596E+00_JPRB ,&
&  0.17083E+00_JPRB, 0.16593E+00_JPRB, 0.16154E+00_JPRB, 0.15859E+00_JPRB, 0.15548E+00_JPRB, 0.15324E+00_JPRB, 0.15150E+00_JPRB ,&
&  0.14973E+00_JPRB, 0.14872E+00_JPRB, 0.14740E+00_JPRB, 0.14703E+00_JPRB, 0.14733E+00_JPRB, 0.14816E+00_JPRB, 0.14929E+00_JPRB ,&
&  0.15062E+00_JPRB, 0.15155E+00_JPRB, 0.15271E+00_JPRB, 0.15382E+00_JPRB, 0.15463E+00_JPRB, 0.15515E+00_JPRB, 0.15492E+00_JPRB ,&
&  0.15517E+00_JPRB, 0.15546E+00_JPRB, 0.15504E+00_JPRB, 0.15500E+00_JPRB, 0.15386E+00_JPRB, 0.15253E+00_JPRB, 0.15116E+00_JPRB ,&
&  0.14931E+00_JPRB, 0.14789E+00_JPRB, 0.14737E+00_JPRB, 0.14709E+00_JPRB, 0.14780E+00_JPRB, 0.14887E+00_JPRB, 0.14958E+00_JPRB ,&
&  0.14995E+00_JPRB, 0.14989E+00_JPRB, 0.14993E+00_JPRB, 0.15109E+00_JPRB, 0.15228E+00_JPRB, 0.15380E+00_JPRB, 0.15538E+00_JPRB ,&
&  0.15588E+00_JPRB, 0.15719E+00_JPRB, 0.15833E+00_JPRB, 0.15975E+00_JPRB, 0.16458E+00_JPRB, 0.16836E+00_JPRB, 0.17560E+00_JPRB ,&
&  0.17903E+00_JPRB, 0.17995E+00_JPRB, 0.17916E+00_JPRB, 0.18354E+00_JPRB, 0.18393E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.18248E+00_JPRB, 0.18495E+00_JPRB, 0.18361E+00_JPRB, 0.18345E+00_JPRB, 0.18284E+00_JPRB, 0.18268E+00_JPRB, 0.18158E+00_JPRB ,&
&  0.18138E+00_JPRB, 0.17943E+00_JPRB, 0.17904E+00_JPRB, 0.17736E+00_JPRB, 0.17561E+00_JPRB, 0.17320E+00_JPRB, 0.17081E+00_JPRB ,&
&  0.16844E+00_JPRB, 0.16634E+00_JPRB, 0.16492E+00_JPRB, 0.16428E+00_JPRB, 0.16311E+00_JPRB, 0.16310E+00_JPRB, 0.16504E+00_JPRB ,&
&  0.16546E+00_JPRB, 0.16656E+00_JPRB, 0.16669E+00_JPRB, 0.16793E+00_JPRB, 0.16956E+00_JPRB, 0.17024E+00_JPRB, 0.17254E+00_JPRB ,&
&  0.17373E+00_JPRB, 0.17523E+00_JPRB, 0.18108E+00_JPRB, 0.18429E+00_JPRB, 0.18606E+00_JPRB, 0.18876E+00_JPRB, 0.18893E+00_JPRB ,&
&  0.18751E+00_JPRB, 0.18334E+00_JPRB, 0.18646E+00_JPRB, 0.18401E+00_JPRB, 0.17751E+00_JPRB, 0.17464E+00_JPRB, 0.16934E+00_JPRB ,&
&  0.16627E+00_JPRB, 0.16259E+00_JPRB, 0.15983E+00_JPRB, 0.15770E+00_JPRB, 0.15471E+00_JPRB, 0.15279E+00_JPRB, 0.15064E+00_JPRB ,&
&  0.14863E+00_JPRB, 0.14778E+00_JPRB, 0.14641E+00_JPRB, 0.14626E+00_JPRB, 0.14640E+00_JPRB, 0.14683E+00_JPRB, 0.14754E+00_JPRB ,&
&  0.14848E+00_JPRB, 0.14936E+00_JPRB, 0.15019E+00_JPRB, 0.15091E+00_JPRB, 0.15151E+00_JPRB, 0.15187E+00_JPRB, 0.15175E+00_JPRB ,&
&  0.15187E+00_JPRB, 0.15179E+00_JPRB, 0.15134E+00_JPRB, 0.15112E+00_JPRB, 0.15020E+00_JPRB, 0.14978E+00_JPRB, 0.14935E+00_JPRB ,&
&  0.14916E+00_JPRB, 0.14918E+00_JPRB, 0.14911E+00_JPRB, 0.14931E+00_JPRB, 0.14960E+00_JPRB, 0.15030E+00_JPRB, 0.15088E+00_JPRB ,&
&  0.15134E+00_JPRB, 0.15222E+00_JPRB, 0.15277E+00_JPRB, 0.15351E+00_JPRB, 0.15455E+00_JPRB, 0.15502E+00_JPRB, 0.15620E+00_JPRB ,&
&  0.15721E+00_JPRB, 0.15841E+00_JPRB, 0.16063E+00_JPRB, 0.16288E+00_JPRB, 0.16611E+00_JPRB, 0.17016E+00_JPRB, 0.17177E+00_JPRB ,&
&  0.17316E+00_JPRB, 0.17424E+00_JPRB, 0.17281E+00_JPRB, 0.17777E+00_JPRB, 0.18008E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.17606E+00_JPRB, 0.17792E+00_JPRB, 0.17910E+00_JPRB, 0.17993E+00_JPRB, 0.18117E+00_JPRB, 0.18172E+00_JPRB, 0.18222E+00_JPRB ,&
&  0.18035E+00_JPRB, 0.17974E+00_JPRB, 0.17869E+00_JPRB, 0.17739E+00_JPRB, 0.17647E+00_JPRB, 0.17415E+00_JPRB, 0.17186E+00_JPRB ,&
&  0.16932E+00_JPRB, 0.16706E+00_JPRB, 0.16570E+00_JPRB, 0.16437E+00_JPRB, 0.16300E+00_JPRB, 0.16188E+00_JPRB, 0.16372E+00_JPRB ,&
&  0.16478E+00_JPRB, 0.16548E+00_JPRB, 0.16676E+00_JPRB, 0.16666E+00_JPRB, 0.16798E+00_JPRB, 0.17005E+00_JPRB, 0.17282E+00_JPRB ,&
&  0.17459E+00_JPRB, 0.17744E+00_JPRB, 0.17912E+00_JPRB, 0.17996E+00_JPRB, 0.18113E+00_JPRB, 0.18195E+00_JPRB, 0.18165E+00_JPRB ,&
&  0.18035E+00_JPRB, 0.17862E+00_JPRB, 0.17659E+00_JPRB, 0.17398E+00_JPRB, 0.17110E+00_JPRB, 0.16831E+00_JPRB, 0.16553E+00_JPRB ,&
&  0.16349E+00_JPRB, 0.16164E+00_JPRB, 0.15968E+00_JPRB, 0.15789E+00_JPRB, 0.15619E+00_JPRB, 0.15465E+00_JPRB, 0.15311E+00_JPRB ,&
&  0.15189E+00_JPRB, 0.15093E+00_JPRB, 0.14974E+00_JPRB, 0.14910E+00_JPRB, 0.14847E+00_JPRB, 0.14820E+00_JPRB, 0.14807E+00_JPRB ,&
&  0.14847E+00_JPRB, 0.14899E+00_JPRB, 0.14953E+00_JPRB, 0.14984E+00_JPRB, 0.15012E+00_JPRB, 0.15026E+00_JPRB, 0.15052E+00_JPRB ,&
&  0.15076E+00_JPRB, 0.15105E+00_JPRB, 0.15067E+00_JPRB, 0.15033E+00_JPRB, 0.14984E+00_JPRB, 0.14930E+00_JPRB, 0.14913E+00_JPRB ,&
&  0.14942E+00_JPRB, 0.14966E+00_JPRB, 0.14951E+00_JPRB, 0.14979E+00_JPRB, 0.15029E+00_JPRB, 0.15081E+00_JPRB, 0.15211E+00_JPRB ,&
&  0.15282E+00_JPRB, 0.15351E+00_JPRB, 0.15414E+00_JPRB, 0.15464E+00_JPRB, 0.15541E+00_JPRB, 0.15602E+00_JPRB, 0.15679E+00_JPRB ,&
&  0.15772E+00_JPRB, 0.15863E+00_JPRB, 0.15989E+00_JPRB, 0.16149E+00_JPRB, 0.16528E+00_JPRB, 0.16780E+00_JPRB, 0.16998E+00_JPRB ,&
&  0.17099E+00_JPRB, 0.16760E+00_JPRB, 0.17478E+00_JPRB, 0.17391E+00_JPRB, 0.17367E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.16751E+00_JPRB, 0.16930E+00_JPRB, 0.17070E+00_JPRB, 0.17259E+00_JPRB, 0.16839E+00_JPRB, 0.17534E+00_JPRB, 0.17256E+00_JPRB ,&
&  0.16967E+00_JPRB, 0.16487E+00_JPRB, 0.16500E+00_JPRB, 0.16419E+00_JPRB, 0.16305E+00_JPRB, 0.16248E+00_JPRB, 0.16235E+00_JPRB ,&
&  0.16241E+00_JPRB, 0.16223E+00_JPRB, 0.16268E+00_JPRB, 0.16301E+00_JPRB, 0.16327E+00_JPRB, 0.16347E+00_JPRB, 0.16377E+00_JPRB ,&
&  0.16430E+00_JPRB, 0.16521E+00_JPRB, 0.16615E+00_JPRB, 0.16436E+00_JPRB, 0.16418E+00_JPRB, 0.16445E+00_JPRB, 0.16506E+00_JPRB ,&
&  0.16551E+00_JPRB, 0.16660E+00_JPRB, 0.16742E+00_JPRB, 0.16800E+00_JPRB, 0.16963E+00_JPRB, 0.16969E+00_JPRB, 0.16953E+00_JPRB ,&
&  0.16913E+00_JPRB, 0.16832E+00_JPRB, 0.16716E+00_JPRB, 0.16609E+00_JPRB, 0.16498E+00_JPRB, 0.16352E+00_JPRB, 0.16212E+00_JPRB ,&
&  0.16096E+00_JPRB, 0.16003E+00_JPRB, 0.15881E+00_JPRB, 0.15764E+00_JPRB, 0.15656E+00_JPRB, 0.15553E+00_JPRB, 0.15465E+00_JPRB ,&
&  0.15400E+00_JPRB, 0.15346E+00_JPRB, 0.15290E+00_JPRB, 0.15247E+00_JPRB, 0.15201E+00_JPRB, 0.15160E+00_JPRB, 0.15122E+00_JPRB ,&
&  0.15139E+00_JPRB, 0.15153E+00_JPRB, 0.15168E+00_JPRB, 0.15182E+00_JPRB, 0.15224E+00_JPRB, 0.15234E+00_JPRB, 0.15245E+00_JPRB ,&
&  0.15252E+00_JPRB, 0.15245E+00_JPRB, 0.15235E+00_JPRB, 0.15215E+00_JPRB, 0.15192E+00_JPRB, 0.15162E+00_JPRB, 0.15148E+00_JPRB ,&
&  0.15147E+00_JPRB, 0.15157E+00_JPRB, 0.15190E+00_JPRB, 0.15213E+00_JPRB, 0.15235E+00_JPRB, 0.15254E+00_JPRB, 0.15299E+00_JPRB ,&
&  0.15346E+00_JPRB, 0.15393E+00_JPRB, 0.15437E+00_JPRB, 0.15428E+00_JPRB, 0.15453E+00_JPRB, 0.15475E+00_JPRB, 0.15497E+00_JPRB ,&
&  0.15511E+00_JPRB, 0.15558E+00_JPRB, 0.15632E+00_JPRB, 0.15732E+00_JPRB, 0.15915E+00_JPRB, 0.16071E+00_JPRB, 0.16216E+00_JPRB ,&
&  0.16371E+00_JPRB, 0.16681E+00_JPRB, 0.16802E+00_JPRB, 0.16840E+00_JPRB, 0.16838E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.15826E+00_JPRB, 0.15905E+00_JPRB, 0.15960E+00_JPRB, 0.16004E+00_JPRB, 0.15985E+00_JPRB, 0.15991E+00_JPRB, 0.15996E+00_JPRB ,&
&  0.15992E+00_JPRB, 0.15988E+00_JPRB, 0.16014E+00_JPRB, 0.16038E+00_JPRB, 0.16064E+00_JPRB, 0.16092E+00_JPRB, 0.16126E+00_JPRB ,&
&  0.16139E+00_JPRB, 0.16168E+00_JPRB, 0.16051E+00_JPRB, 0.15955E+00_JPRB, 0.15893E+00_JPRB, 0.15854E+00_JPRB, 0.15834E+00_JPRB ,&
&  0.15815E+00_JPRB, 0.15804E+00_JPRB, 0.15789E+00_JPRB, 0.15871E+00_JPRB, 0.15924E+00_JPRB, 0.15972E+00_JPRB, 0.16009E+00_JPRB ,&
&  0.16022E+00_JPRB, 0.16044E+00_JPRB, 0.16062E+00_JPRB, 0.16081E+00_JPRB, 0.16103E+00_JPRB, 0.16100E+00_JPRB, 0.16088E+00_JPRB ,&
&  0.16068E+00_JPRB, 0.16037E+00_JPRB, 0.16000E+00_JPRB, 0.15965E+00_JPRB, 0.15936E+00_JPRB, 0.15903E+00_JPRB, 0.15867E+00_JPRB ,&
&  0.15833E+00_JPRB, 0.15792E+00_JPRB, 0.15754E+00_JPRB, 0.15717E+00_JPRB, 0.15682E+00_JPRB, 0.15652E+00_JPRB, 0.15620E+00_JPRB ,&
&  0.15598E+00_JPRB, 0.15578E+00_JPRB, 0.15558E+00_JPRB, 0.15543E+00_JPRB, 0.15522E+00_JPRB, 0.15513E+00_JPRB, 0.15494E+00_JPRB ,&
&  0.15488E+00_JPRB, 0.15482E+00_JPRB, 0.15478E+00_JPRB, 0.15472E+00_JPRB, 0.15473E+00_JPRB, 0.15472E+00_JPRB, 0.15472E+00_JPRB ,&
&  0.15470E+00_JPRB, 0.15483E+00_JPRB, 0.15472E+00_JPRB, 0.15457E+00_JPRB, 0.15440E+00_JPRB, 0.15424E+00_JPRB, 0.15408E+00_JPRB ,&
&  0.15394E+00_JPRB, 0.15379E+00_JPRB, 0.15370E+00_JPRB, 0.15372E+00_JPRB, 0.15375E+00_JPRB, 0.15381E+00_JPRB, 0.15388E+00_JPRB ,&
&  0.15397E+00_JPRB, 0.15408E+00_JPRB, 0.15419E+00_JPRB, 0.15416E+00_JPRB, 0.15419E+00_JPRB, 0.15422E+00_JPRB, 0.15424E+00_JPRB ,&
&  0.15426E+00_JPRB, 0.15431E+00_JPRB, 0.15440E+00_JPRB, 0.15452E+00_JPRB, 0.15478E+00_JPRB, 0.15489E+00_JPRB, 0.15505E+00_JPRB ,&
&  0.15527E+00_JPRB, 0.15566E+00_JPRB, 0.15601E+00_JPRB, 0.15638E+00_JPRB, 0.15681E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB, 0.15574E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB, 0.47068E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.46638E+00_JPRB, 0.46563E+00_JPRB, 0.46494E+00_JPRB, 0.46427E+00_JPRB, 0.46347E+00_JPRB, 0.46291E+00_JPRB, 0.46244E+00_JPRB ,&
&  0.46200E+00_JPRB, 0.46158E+00_JPRB, 0.46112E+00_JPRB, 0.46077E+00_JPRB, 0.46056E+00_JPRB, 0.46027E+00_JPRB, 0.45995E+00_JPRB ,&
&  0.45961E+00_JPRB, 0.45936E+00_JPRB, 0.45925E+00_JPRB, 0.45889E+00_JPRB, 0.45857E+00_JPRB, 0.45828E+00_JPRB, 0.45774E+00_JPRB ,&
&  0.45699E+00_JPRB, 0.45631E+00_JPRB, 0.45558E+00_JPRB, 0.45501E+00_JPRB, 0.45461E+00_JPRB, 0.45392E+00_JPRB, 0.45306E+00_JPRB ,&
&  0.45235E+00_JPRB, 0.45193E+00_JPRB, 0.45177E+00_JPRB, 0.45184E+00_JPRB, 0.45208E+00_JPRB, 0.45227E+00_JPRB, 0.45242E+00_JPRB ,&
&  0.45261E+00_JPRB, 0.45298E+00_JPRB, 0.45352E+00_JPRB, 0.45432E+00_JPRB, 0.45533E+00_JPRB, 0.45655E+00_JPRB, 0.45825E+00_JPRB ,&
&  0.46019E+00_JPRB, 0.46219E+00_JPRB, 0.46418E+00_JPRB, 0.46596E+00_JPRB, 0.46754E+00_JPRB, 0.46888E+00_JPRB, 0.46976E+00_JPRB ,&
&  0.47074E+00_JPRB, 0.47184E+00_JPRB, 0.47301E+00_JPRB, 0.47378E+00_JPRB, 0.47446E+00_JPRB, 0.47509E+00_JPRB, 0.47582E+00_JPRB ,&
&  0.47612E+00_JPRB, 0.47637E+00_JPRB, 0.47669E+00_JPRB, 0.47716E+00_JPRB, 0.47739E+00_JPRB, 0.47749E+00_JPRB, 0.47762E+00_JPRB ,&
&  0.47793E+00_JPRB, 0.47814E+00_JPRB, 0.47816E+00_JPRB, 0.47823E+00_JPRB, 0.47848E+00_JPRB, 0.47888E+00_JPRB, 0.47928E+00_JPRB ,&
&  0.47962E+00_JPRB, 0.47991E+00_JPRB, 0.48006E+00_JPRB, 0.48005E+00_JPRB, 0.47991E+00_JPRB, 0.47956E+00_JPRB, 0.47884E+00_JPRB ,&
&  0.47799E+00_JPRB, 0.47706E+00_JPRB, 0.47626E+00_JPRB, 0.47571E+00_JPRB, 0.47510E+00_JPRB, 0.47459E+00_JPRB, 0.47420E+00_JPRB ,&
&  0.47401E+00_JPRB, 0.47387E+00_JPRB, 0.47366E+00_JPRB, 0.47348E+00_JPRB, 0.47322E+00_JPRB, 0.47287E+00_JPRB, 0.47236E+00_JPRB ,&
&  0.47169E+00_JPRB, 0.47078E+00_JPRB, 0.46978E+00_JPRB, 0.46866E+00_JPRB, 0.46749E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.45650E+00_JPRB, 0.45498E+00_JPRB, 0.45391E+00_JPRB, 0.45323E+00_JPRB, 0.45255E+00_JPRB, 0.45195E+00_JPRB, 0.45143E+00_JPRB ,&
&  0.45086E+00_JPRB, 0.45085E+00_JPRB, 0.45052E+00_JPRB, 0.45029E+00_JPRB, 0.44995E+00_JPRB, 0.45004E+00_JPRB, 0.44983E+00_JPRB ,&
&  0.44948E+00_JPRB, 0.44856E+00_JPRB, 0.44804E+00_JPRB, 0.44708E+00_JPRB, 0.44717E+00_JPRB, 0.44740E+00_JPRB, 0.44877E+00_JPRB ,&
&  0.44888E+00_JPRB, 0.44881E+00_JPRB, 0.44806E+00_JPRB, 0.44513E+00_JPRB, 0.44254E+00_JPRB, 0.44005E+00_JPRB, 0.43801E+00_JPRB ,&
&  0.43626E+00_JPRB, 0.43509E+00_JPRB, 0.43496E+00_JPRB, 0.43534E+00_JPRB, 0.43766E+00_JPRB, 0.43828E+00_JPRB, 0.43841E+00_JPRB ,&
&  0.43780E+00_JPRB, 0.43545E+00_JPRB, 0.43594E+00_JPRB, 0.43670E+00_JPRB, 0.43801E+00_JPRB, 0.44111E+00_JPRB, 0.44389E+00_JPRB ,&
&  0.44739E+00_JPRB, 0.45149E+00_JPRB, 0.45777E+00_JPRB, 0.46233E+00_JPRB, 0.46705E+00_JPRB, 0.47113E+00_JPRB, 0.47328E+00_JPRB ,&
&  0.47519E+00_JPRB, 0.47690E+00_JPRB, 0.47860E+00_JPRB, 0.47993E+00_JPRB, 0.48103E+00_JPRB, 0.48219E+00_JPRB, 0.48357E+00_JPRB ,&
&  0.48456E+00_JPRB, 0.48574E+00_JPRB, 0.48690E+00_JPRB, 0.48799E+00_JPRB, 0.48866E+00_JPRB, 0.48911E+00_JPRB, 0.48978E+00_JPRB ,&
&  0.49050E+00_JPRB, 0.49034E+00_JPRB, 0.49010E+00_JPRB, 0.49059E+00_JPRB, 0.49115E+00_JPRB, 0.49172E+00_JPRB, 0.49139E+00_JPRB ,&
&  0.49126E+00_JPRB, 0.49086E+00_JPRB, 0.49063E+00_JPRB, 0.49007E+00_JPRB, 0.48934E+00_JPRB, 0.48864E+00_JPRB, 0.48762E+00_JPRB ,&
&  0.48610E+00_JPRB, 0.48442E+00_JPRB, 0.48305E+00_JPRB, 0.48220E+00_JPRB, 0.48105E+00_JPRB, 0.48050E+00_JPRB, 0.48015E+00_JPRB ,&
&  0.47989E+00_JPRB, 0.47838E+00_JPRB, 0.47668E+00_JPRB, 0.47473E+00_JPRB, 0.47283E+00_JPRB, 0.47090E+00_JPRB, 0.46911E+00_JPRB ,&
&  0.46728E+00_JPRB, 0.46524E+00_JPRB, 0.46305E+00_JPRB, 0.46078E+00_JPRB, 0.45858E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.45783E+00_JPRB, 0.45474E+00_JPRB, 0.45264E+00_JPRB, 0.45124E+00_JPRB, 0.44980E+00_JPRB, 0.44843E+00_JPRB, 0.44785E+00_JPRB ,&
&  0.44748E+00_JPRB, 0.44646E+00_JPRB, 0.44583E+00_JPRB, 0.44533E+00_JPRB, 0.44449E+00_JPRB, 0.44475E+00_JPRB, 0.44448E+00_JPRB ,&
&  0.44462E+00_JPRB, 0.44392E+00_JPRB, 0.44475E+00_JPRB, 0.44372E+00_JPRB, 0.44313E+00_JPRB, 0.44263E+00_JPRB, 0.44134E+00_JPRB ,&
&  0.44189E+00_JPRB, 0.44173E+00_JPRB, 0.43886E+00_JPRB, 0.43509E+00_JPRB, 0.43258E+00_JPRB, 0.41989E+00_JPRB, 0.42067E+00_JPRB ,&
&  0.42553E+00_JPRB, 0.41933E+00_JPRB, 0.41713E+00_JPRB, 0.41465E+00_JPRB, 0.41268E+00_JPRB, 0.41291E+00_JPRB, 0.41602E+00_JPRB ,&
&  0.41095E+00_JPRB, 0.41992E+00_JPRB, 0.42081E+00_JPRB, 0.42131E+00_JPRB, 0.42239E+00_JPRB, 0.42129E+00_JPRB, 0.42533E+00_JPRB ,&
&  0.44143E+00_JPRB, 0.44469E+00_JPRB, 0.45157E+00_JPRB, 0.45779E+00_JPRB, 0.46542E+00_JPRB, 0.47235E+00_JPRB, 0.47940E+00_JPRB ,&
&  0.48467E+00_JPRB, 0.48812E+00_JPRB, 0.49036E+00_JPRB, 0.49076E+00_JPRB, 0.49124E+00_JPRB, 0.49115E+00_JPRB, 0.49180E+00_JPRB ,&
&  0.49155E+00_JPRB, 0.49247E+00_JPRB, 0.49421E+00_JPRB, 0.49654E+00_JPRB, 0.49960E+00_JPRB, 0.50161E+00_JPRB, 0.50344E+00_JPRB ,&
&  0.50461E+00_JPRB, 0.50519E+00_JPRB, 0.50473E+00_JPRB, 0.50473E+00_JPRB, 0.50383E+00_JPRB, 0.50409E+00_JPRB, 0.50356E+00_JPRB ,&
&  0.50390E+00_JPRB, 0.50331E+00_JPRB, 0.50342E+00_JPRB, 0.50278E+00_JPRB, 0.50252E+00_JPRB, 0.50281E+00_JPRB, 0.50285E+00_JPRB ,&
&  0.50309E+00_JPRB, 0.50279E+00_JPRB, 0.50222E+00_JPRB, 0.50159E+00_JPRB, 0.50046E+00_JPRB, 0.49995E+00_JPRB, 0.49929E+00_JPRB ,&
&  0.49898E+00_JPRB, 0.49598E+00_JPRB, 0.49265E+00_JPRB, 0.48914E+00_JPRB, 0.48598E+00_JPRB, 0.48313E+00_JPRB, 0.48017E+00_JPRB ,&
&  0.47748E+00_JPRB, 0.47389E+00_JPRB, 0.47001E+00_JPRB, 0.46568E+00_JPRB, 0.46149E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.46860E+00_JPRB, 0.46419E+00_JPRB, 0.46033E+00_JPRB, 0.45758E+00_JPRB, 0.45516E+00_JPRB, 0.45309E+00_JPRB, 0.45155E+00_JPRB ,&
&  0.45021E+00_JPRB, 0.44930E+00_JPRB, 0.44868E+00_JPRB, 0.44822E+00_JPRB, 0.44773E+00_JPRB, 0.44704E+00_JPRB, 0.44616E+00_JPRB ,&
&  0.44689E+00_JPRB, 0.44609E+00_JPRB, 0.44681E+00_JPRB, 0.44503E+00_JPRB, 0.44371E+00_JPRB, 0.44127E+00_JPRB, 0.43635E+00_JPRB ,&
&  0.43461E+00_JPRB, 0.43225E+00_JPRB, 0.42026E+00_JPRB, 0.42576E+00_JPRB, 0.42197E+00_JPRB, 0.42033E+00_JPRB, 0.41828E+00_JPRB ,&
&  0.41208E+00_JPRB, 0.40888E+00_JPRB, 0.40444E+00_JPRB, 0.40382E+00_JPRB, 0.40548E+00_JPRB, 0.40293E+00_JPRB, 0.40408E+00_JPRB ,&
&  0.40060E+00_JPRB, 0.39941E+00_JPRB, 0.39978E+00_JPRB, 0.39994E+00_JPRB, 0.40287E+00_JPRB, 0.40364E+00_JPRB, 0.40683E+00_JPRB ,&
&  0.41468E+00_JPRB, 0.42565E+00_JPRB, 0.44359E+00_JPRB, 0.45270E+00_JPRB, 0.46370E+00_JPRB, 0.47231E+00_JPRB, 0.48067E+00_JPRB ,&
&  0.48593E+00_JPRB, 0.48954E+00_JPRB, 0.49207E+00_JPRB, 0.49341E+00_JPRB, 0.49323E+00_JPRB, 0.49047E+00_JPRB, 0.48975E+00_JPRB ,&
&  0.48942E+00_JPRB, 0.49000E+00_JPRB, 0.49116E+00_JPRB, 0.49241E+00_JPRB, 0.49517E+00_JPRB, 0.49788E+00_JPRB, 0.50159E+00_JPRB ,&
&  0.50526E+00_JPRB, 0.50936E+00_JPRB, 0.51018E+00_JPRB, 0.51217E+00_JPRB, 0.51253E+00_JPRB, 0.51310E+00_JPRB, 0.51327E+00_JPRB ,&
&  0.51360E+00_JPRB, 0.51405E+00_JPRB, 0.51526E+00_JPRB, 0.51116E+00_JPRB, 0.51065E+00_JPRB, 0.51704E+00_JPRB, 0.51751E+00_JPRB ,&
&  0.51856E+00_JPRB, 0.51752E+00_JPRB, 0.51544E+00_JPRB, 0.51346E+00_JPRB, 0.51176E+00_JPRB, 0.51082E+00_JPRB, 0.51016E+00_JPRB ,&
&  0.50931E+00_JPRB, 0.50546E+00_JPRB, 0.50153E+00_JPRB, 0.49799E+00_JPRB, 0.49416E+00_JPRB, 0.49133E+00_JPRB, 0.48856E+00_JPRB ,&
&  0.48663E+00_JPRB, 0.48506E+00_JPRB, 0.48312E+00_JPRB, 0.47940E+00_JPRB, 0.47436E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.48174E+00_JPRB, 0.47756E+00_JPRB, 0.47425E+00_JPRB, 0.47127E+00_JPRB, 0.46939E+00_JPRB, 0.46757E+00_JPRB, 0.46591E+00_JPRB ,&
&  0.46397E+00_JPRB, 0.46209E+00_JPRB, 0.45980E+00_JPRB, 0.45827E+00_JPRB, 0.45661E+00_JPRB, 0.45519E+00_JPRB, 0.45328E+00_JPRB ,&
&  0.45226E+00_JPRB, 0.45065E+00_JPRB, 0.45039E+00_JPRB, 0.44794E+00_JPRB, 0.44792E+00_JPRB, 0.44505E+00_JPRB, 0.43878E+00_JPRB ,&
&  0.43323E+00_JPRB, 0.42715E+00_JPRB, 0.41744E+00_JPRB, 0.42171E+00_JPRB, 0.42072E+00_JPRB, 0.42166E+00_JPRB, 0.41482E+00_JPRB ,&
&  0.41055E+00_JPRB, 0.39942E+00_JPRB, 0.40963E+00_JPRB, 0.40421E+00_JPRB, 0.40032E+00_JPRB, 0.39410E+00_JPRB, 0.38480E+00_JPRB ,&
&  0.38241E+00_JPRB, 0.37706E+00_JPRB, 0.37676E+00_JPRB, 0.37951E+00_JPRB, 0.38318E+00_JPRB, 0.38847E+00_JPRB, 0.39422E+00_JPRB ,&
&  0.39616E+00_JPRB, 0.42416E+00_JPRB, 0.44120E+00_JPRB, 0.45377E+00_JPRB, 0.46450E+00_JPRB, 0.47300E+00_JPRB, 0.48055E+00_JPRB ,&
&  0.48725E+00_JPRB, 0.49305E+00_JPRB, 0.49852E+00_JPRB, 0.49960E+00_JPRB, 0.50601E+00_JPRB, 0.50769E+00_JPRB, 0.50972E+00_JPRB ,&
&  0.51184E+00_JPRB, 0.51119E+00_JPRB, 0.51031E+00_JPRB, 0.50789E+00_JPRB, 0.50697E+00_JPRB, 0.50486E+00_JPRB, 0.50616E+00_JPRB ,&
&  0.50587E+00_JPRB, 0.50676E+00_JPRB, 0.50924E+00_JPRB, 0.51155E+00_JPRB, 0.51330E+00_JPRB, 0.51509E+00_JPRB, 0.51709E+00_JPRB ,&
&  0.51895E+00_JPRB, 0.52299E+00_JPRB, 0.52655E+00_JPRB, 0.53291E+00_JPRB, 0.53790E+00_JPRB, 0.53897E+00_JPRB, 0.53967E+00_JPRB ,&
&  0.54144E+00_JPRB, 0.53687E+00_JPRB, 0.53270E+00_JPRB, 0.52957E+00_JPRB, 0.52763E+00_JPRB, 0.52647E+00_JPRB, 0.52643E+00_JPRB ,&
&  0.52564E+00_JPRB, 0.52205E+00_JPRB, 0.51788E+00_JPRB, 0.51303E+00_JPRB, 0.50815E+00_JPRB, 0.50479E+00_JPRB, 0.50186E+00_JPRB ,&
&  0.49979E+00_JPRB, 0.49792E+00_JPRB, 0.49477E+00_JPRB, 0.49039E+00_JPRB, 0.48594E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.50007E+00_JPRB, 0.49667E+00_JPRB, 0.49418E+00_JPRB, 0.49131E+00_JPRB, 0.48966E+00_JPRB, 0.48777E+00_JPRB, 0.48613E+00_JPRB ,&
&  0.48475E+00_JPRB, 0.48191E+00_JPRB, 0.47842E+00_JPRB, 0.47420E+00_JPRB, 0.47104E+00_JPRB, 0.46806E+00_JPRB, 0.46473E+00_JPRB ,&
&  0.45962E+00_JPRB, 0.45389E+00_JPRB, 0.44789E+00_JPRB, 0.44294E+00_JPRB, 0.44271E+00_JPRB, 0.44055E+00_JPRB, 0.43779E+00_JPRB ,&
&  0.43206E+00_JPRB, 0.42554E+00_JPRB, 0.41831E+00_JPRB, 0.41794E+00_JPRB, 0.41868E+00_JPRB, 0.41339E+00_JPRB, 0.41427E+00_JPRB ,&
&  0.42219E+00_JPRB, 0.42391E+00_JPRB, 0.42326E+00_JPRB, 0.42269E+00_JPRB, 0.40543E+00_JPRB, 0.37515E+00_JPRB, 0.38600E+00_JPRB ,&
&  0.38419E+00_JPRB, 0.37429E+00_JPRB, 0.37091E+00_JPRB, 0.36891E+00_JPRB, 0.37280E+00_JPRB, 0.38237E+00_JPRB, 0.39246E+00_JPRB ,&
&  0.39764E+00_JPRB, 0.44452E+00_JPRB, 0.46360E+00_JPRB, 0.47649E+00_JPRB, 0.48422E+00_JPRB, 0.49120E+00_JPRB, 0.49851E+00_JPRB ,&
&  0.50472E+00_JPRB, 0.51398E+00_JPRB, 0.52582E+00_JPRB, 0.53671E+00_JPRB, 0.54649E+00_JPRB, 0.55387E+00_JPRB, 0.56101E+00_JPRB ,&
&  0.57028E+00_JPRB, 0.57020E+00_JPRB, 0.56210E+00_JPRB, 0.55252E+00_JPRB, 0.54388E+00_JPRB, 0.53506E+00_JPRB, 0.52742E+00_JPRB ,&
&  0.52322E+00_JPRB, 0.51904E+00_JPRB, 0.52146E+00_JPRB, 0.52118E+00_JPRB, 0.52181E+00_JPRB, 0.52369E+00_JPRB, 0.53044E+00_JPRB ,&
&  0.53492E+00_JPRB, 0.54138E+00_JPRB, 0.54767E+00_JPRB, 0.55616E+00_JPRB, 0.56198E+00_JPRB, 0.56278E+00_JPRB, 0.56305E+00_JPRB ,&
&  0.56159E+00_JPRB, 0.56515E+00_JPRB, 0.56527E+00_JPRB, 0.56717E+00_JPRB, 0.57159E+00_JPRB, 0.57338E+00_JPRB, 0.57594E+00_JPRB ,&
&  0.57714E+00_JPRB, 0.57701E+00_JPRB, 0.57362E+00_JPRB, 0.56771E+00_JPRB, 0.55960E+00_JPRB, 0.55050E+00_JPRB, 0.53991E+00_JPRB ,&
&  0.52993E+00_JPRB, 0.51974E+00_JPRB, 0.51134E+00_JPRB, 0.50642E+00_JPRB, 0.50268E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.52951E+00_JPRB, 0.52274E+00_JPRB, 0.51760E+00_JPRB, 0.51469E+00_JPRB, 0.51229E+00_JPRB, 0.50917E+00_JPRB, 0.50640E+00_JPRB ,&
&  0.50383E+00_JPRB, 0.50117E+00_JPRB, 0.49737E+00_JPRB, 0.49317E+00_JPRB, 0.48943E+00_JPRB, 0.48627E+00_JPRB, 0.48368E+00_JPRB ,&
&  0.47766E+00_JPRB, 0.46183E+00_JPRB, 0.45597E+00_JPRB, 0.44862E+00_JPRB, 0.43848E+00_JPRB, 0.43294E+00_JPRB, 0.42908E+00_JPRB ,&
&  0.42413E+00_JPRB, 0.42165E+00_JPRB, 0.42179E+00_JPRB, 0.41728E+00_JPRB, 0.41486E+00_JPRB, 0.41971E+00_JPRB, 0.42234E+00_JPRB ,&
&  0.41054E+00_JPRB, 0.41143E+00_JPRB, 0.42970E+00_JPRB, 0.43061E+00_JPRB, 0.41273E+00_JPRB, 0.40768E+00_JPRB, 0.40000E+00_JPRB ,&
&  0.39519E+00_JPRB, 0.37974E+00_JPRB, 0.37515E+00_JPRB, 0.37244E+00_JPRB, 0.37899E+00_JPRB, 0.38820E+00_JPRB, 0.41110E+00_JPRB ,&
&  0.43460E+00_JPRB, 0.47843E+00_JPRB, 0.49488E+00_JPRB, 0.50608E+00_JPRB, 0.51692E+00_JPRB, 0.52846E+00_JPRB, 0.54090E+00_JPRB ,&
&  0.55748E+00_JPRB, 0.57583E+00_JPRB, 0.59888E+00_JPRB, 0.61884E+00_JPRB, 0.63821E+00_JPRB, 0.64774E+00_JPRB, 0.64863E+00_JPRB ,&
&  0.66202E+00_JPRB, 0.66163E+00_JPRB, 0.64081E+00_JPRB, 0.62288E+00_JPRB, 0.60338E+00_JPRB, 0.58420E+00_JPRB, 0.56174E+00_JPRB ,&
&  0.55462E+00_JPRB, 0.56052E+00_JPRB, 0.55392E+00_JPRB, 0.54828E+00_JPRB, 0.54487E+00_JPRB, 0.54386E+00_JPRB, 0.54575E+00_JPRB ,&
&  0.54553E+00_JPRB, 0.56297E+00_JPRB, 0.62835E+00_JPRB, 0.64031E+00_JPRB, 0.64015E+00_JPRB, 0.63676E+00_JPRB, 0.63297E+00_JPRB ,&
&  0.63402E+00_JPRB, 0.64015E+00_JPRB, 0.64870E+00_JPRB, 0.65280E+00_JPRB, 0.65761E+00_JPRB, 0.65355E+00_JPRB, 0.65828E+00_JPRB ,&
&  0.65429E+00_JPRB, 0.64943E+00_JPRB, 0.64165E+00_JPRB, 0.63611E+00_JPRB, 0.62939E+00_JPRB, 0.62255E+00_JPRB, 0.61274E+00_JPRB ,&
&  0.60074E+00_JPRB, 0.58595E+00_JPRB, 0.57066E+00_JPRB, 0.55525E+00_JPRB, 0.54083E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.56933E+00_JPRB, 0.55072E+00_JPRB, 0.53856E+00_JPRB, 0.53405E+00_JPRB, 0.53213E+00_JPRB, 0.51779E+00_JPRB, 0.50729E+00_JPRB ,&
&  0.50100E+00_JPRB, 0.49994E+00_JPRB, 0.49831E+00_JPRB, 0.49426E+00_JPRB, 0.49235E+00_JPRB, 0.49060E+00_JPRB, 0.48991E+00_JPRB ,&
&  0.48782E+00_JPRB, 0.48205E+00_JPRB, 0.46988E+00_JPRB, 0.45172E+00_JPRB, 0.43806E+00_JPRB, 0.42850E+00_JPRB, 0.42258E+00_JPRB ,&
&  0.41856E+00_JPRB, 0.41783E+00_JPRB, 0.42297E+00_JPRB, 0.42383E+00_JPRB, 0.42290E+00_JPRB, 0.41931E+00_JPRB, 0.41497E+00_JPRB ,&
&  0.41530E+00_JPRB, 0.42345E+00_JPRB, 0.41674E+00_JPRB, 0.41906E+00_JPRB, 0.42870E+00_JPRB, 0.42496E+00_JPRB, 0.41162E+00_JPRB ,&
&  0.40102E+00_JPRB, 0.39065E+00_JPRB, 0.38521E+00_JPRB, 0.38424E+00_JPRB, 0.39523E+00_JPRB, 0.41242E+00_JPRB, 0.44307E+00_JPRB ,&
&  0.49015E+00_JPRB, 0.51226E+00_JPRB, 0.52382E+00_JPRB, 0.53622E+00_JPRB, 0.54960E+00_JPRB, 0.56429E+00_JPRB, 0.58695E+00_JPRB ,&
&  0.62178E+00_JPRB, 0.65805E+00_JPRB, 0.69352E+00_JPRB, 0.71721E+00_JPRB, 0.74373E+00_JPRB, 0.72010E+00_JPRB, 0.71714E+00_JPRB ,&
&  0.79462E+00_JPRB, 0.82484E+00_JPRB, 0.78280E+00_JPRB, 0.74399E+00_JPRB, 0.70420E+00_JPRB, 0.66817E+00_JPRB, 0.63761E+00_JPRB ,&
&  0.61709E+00_JPRB, 0.60438E+00_JPRB, 0.59400E+00_JPRB, 0.58023E+00_JPRB, 0.57037E+00_JPRB, 0.57163E+00_JPRB, 0.58673E+00_JPRB ,&
&  0.62094E+00_JPRB, 0.71873E+00_JPRB, 0.93371E+00_JPRB, 0.78944E+00_JPRB, 0.73528E+00_JPRB, 0.72374E+00_JPRB, 0.72518E+00_JPRB ,&
&  0.74135E+00_JPRB, 0.75497E+00_JPRB, 0.76223E+00_JPRB, 0.74988E+00_JPRB, 0.72540E+00_JPRB, 0.69677E+00_JPRB, 0.67751E+00_JPRB ,&
&  0.66159E+00_JPRB, 0.64582E+00_JPRB, 0.63461E+00_JPRB, 0.63111E+00_JPRB, 0.62403E+00_JPRB, 0.61616E+00_JPRB, 0.60982E+00_JPRB ,&
&  0.60453E+00_JPRB, 0.60214E+00_JPRB, 0.60239E+00_JPRB, 0.59794E+00_JPRB, 0.58962E+00_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.56819E+00_JPRB, 0.57543E+00_JPRB, 0.57631E+00_JPRB, 0.56907E+00_JPRB, 0.56087E+00_JPRB, 0.52773E+00_JPRB, 0.50213E+00_JPRB ,&
&  0.49011E+00_JPRB, 0.48623E+00_JPRB, 0.48789E+00_JPRB, 0.48882E+00_JPRB, 0.49054E+00_JPRB, 0.49154E+00_JPRB, 0.48931E+00_JPRB ,&
&  0.48255E+00_JPRB, 0.47604E+00_JPRB, 0.46963E+00_JPRB, 0.45867E+00_JPRB, 0.44463E+00_JPRB, 0.42998E+00_JPRB, 0.41881E+00_JPRB ,&
&  0.41168E+00_JPRB, 0.40956E+00_JPRB, 0.41496E+00_JPRB, 0.42115E+00_JPRB, 0.42745E+00_JPRB, 0.42990E+00_JPRB, 0.42931E+00_JPRB ,&
&  0.42149E+00_JPRB, 0.41644E+00_JPRB, 0.42272E+00_JPRB, 0.42578E+00_JPRB, 0.42958E+00_JPRB, 0.43577E+00_JPRB, 0.42455E+00_JPRB ,&
&  0.41279E+00_JPRB, 0.40286E+00_JPRB, 0.40275E+00_JPRB, 0.41341E+00_JPRB, 0.44678E+00_JPRB, 0.50006E+00_JPRB, 0.51621E+00_JPRB ,&
&  0.52980E+00_JPRB, 0.53993E+00_JPRB, 0.55122E+00_JPRB, 0.57034E+00_JPRB, 0.59354E+00_JPRB, 0.61982E+00_JPRB, 0.66585E+00_JPRB ,&
&  0.72842E+00_JPRB, 0.77682E+00_JPRB, 0.82592E+00_JPRB, 0.76253E+00_JPRB, 0.73725E+00_JPRB, 0.76131E+00_JPRB, 0.79440E+00_JPRB ,&
&  0.83302E+00_JPRB, 0.85148E+00_JPRB, 0.84616E+00_JPRB, 0.85441E+00_JPRB, 0.81892E+00_JPRB, 0.77206E+00_JPRB, 0.70703E+00_JPRB ,&
&  0.65875E+00_JPRB, 0.62568E+00_JPRB, 0.61020E+00_JPRB, 0.60035E+00_JPRB, 0.60926E+00_JPRB, 0.62936E+00_JPRB, 0.66350E+00_JPRB ,&
&  0.71783E+00_JPRB, 0.78304E+00_JPRB, 0.81789E+00_JPRB, 0.78807E+00_JPRB, 0.80263E+00_JPRB, 0.83787E+00_JPRB, 0.86464E+00_JPRB ,&
&  0.86470E+00_JPRB, 0.86158E+00_JPRB, 0.86520E+00_JPRB, 0.85613E+00_JPRB, 0.79910E+00_JPRB, 0.74797E+00_JPRB, 0.69017E+00_JPRB ,&
&  0.63512E+00_JPRB, 0.58609E+00_JPRB, 0.55974E+00_JPRB, 0.55244E+00_JPRB, 0.54661E+00_JPRB, 0.53983E+00_JPRB, 0.52873E+00_JPRB ,&
&  0.51916E+00_JPRB, 0.51863E+00_JPRB, 0.52828E+00_JPRB, 0.54029E+00_JPRB, 0.55710E+00_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.60131E+00_JPRB, 0.61379E+00_JPRB, 0.61601E+00_JPRB, 0.62029E+00_JPRB, 0.59347E+00_JPRB, 0.56867E+00_JPRB, 0.54430E+00_JPRB ,&
&  0.53507E+00_JPRB, 0.51872E+00_JPRB, 0.50896E+00_JPRB, 0.49413E+00_JPRB, 0.48078E+00_JPRB, 0.47343E+00_JPRB, 0.47475E+00_JPRB ,&
&  0.47807E+00_JPRB, 0.47553E+00_JPRB, 0.46921E+00_JPRB, 0.45605E+00_JPRB, 0.44388E+00_JPRB, 0.43316E+00_JPRB, 0.42415E+00_JPRB ,&
&  0.41750E+00_JPRB, 0.41705E+00_JPRB, 0.42041E+00_JPRB, 0.42756E+00_JPRB, 0.43300E+00_JPRB, 0.43715E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43407E+00_JPRB, 0.42687E+00_JPRB, 0.42887E+00_JPRB, 0.43759E+00_JPRB, 0.44143E+00_JPRB, 0.45096E+00_JPRB, 0.43668E+00_JPRB ,&
&  0.42366E+00_JPRB, 0.42405E+00_JPRB, 0.44072E+00_JPRB, 0.50069E+00_JPRB, 0.52152E+00_JPRB, 0.53092E+00_JPRB, 0.54291E+00_JPRB ,&
&  0.53457E+00_JPRB, 0.53450E+00_JPRB, 0.55275E+00_JPRB, 0.59770E+00_JPRB, 0.64134E+00_JPRB, 0.71011E+00_JPRB, 0.79090E+00_JPRB ,&
&  0.85597E+00_JPRB, 0.88412E+00_JPRB, 0.90491E+00_JPRB, 0.83924E+00_JPRB, 0.83780E+00_JPRB, 0.96542E+00_JPRB, 0.99246E+00_JPRB ,&
&  0.10137E+01_JPRB, 0.10149E+01_JPRB, 0.10380E+01_JPRB, 0.99957E+00_JPRB, 0.90047E+00_JPRB, 0.82701E+00_JPRB, 0.77142E+00_JPRB ,&
&  0.71434E+00_JPRB, 0.68096E+00_JPRB, 0.67551E+00_JPRB, 0.69551E+00_JPRB, 0.73838E+00_JPRB, 0.82157E+00_JPRB, 0.92629E+00_JPRB ,&
&  0.10264E+01_JPRB, 0.10864E+01_JPRB, 0.11224E+01_JPRB, 0.11125E+01_JPRB, 0.10957E+01_JPRB, 0.10723E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10352E+01_JPRB, 0.10289E+01_JPRB, 0.10192E+01_JPRB, 0.10069E+01_JPRB, 0.93588E+00_JPRB, 0.81788E+00_JPRB, 0.69974E+00_JPRB ,&
&  0.61645E+00_JPRB, 0.56695E+00_JPRB, 0.53279E+00_JPRB, 0.52483E+00_JPRB, 0.52711E+00_JPRB, 0.52896E+00_JPRB, 0.51609E+00_JPRB ,&
&  0.51132E+00_JPRB, 0.52105E+00_JPRB, 0.53613E+00_JPRB, 0.55263E+00_JPRB, 0.57556E+00_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.64372E+00_JPRB, 0.65320E+00_JPRB, 0.65849E+00_JPRB, 0.65446E+00_JPRB, 0.64108E+00_JPRB, 0.60845E+00_JPRB, 0.56426E+00_JPRB ,&
&  0.55449E+00_JPRB, 0.52564E+00_JPRB, 0.51290E+00_JPRB, 0.49993E+00_JPRB, 0.49019E+00_JPRB, 0.47587E+00_JPRB, 0.45989E+00_JPRB ,&
&  0.45781E+00_JPRB, 0.46345E+00_JPRB, 0.46792E+00_JPRB, 0.46469E+00_JPRB, 0.45648E+00_JPRB, 0.44608E+00_JPRB, 0.43671E+00_JPRB ,&
&  0.43076E+00_JPRB, 0.43130E+00_JPRB, 0.43837E+00_JPRB, 0.44776E+00_JPRB, 0.45158E+00_JPRB, 0.45402E+00_JPRB, 0.45857E+00_JPRB ,&
&  0.45880E+00_JPRB, 0.45422E+00_JPRB, 0.45110E+00_JPRB, 0.45488E+00_JPRB, 0.45679E+00_JPRB, 0.47044E+00_JPRB, 0.47714E+00_JPRB ,&
&  0.44745E+00_JPRB, 0.45801E+00_JPRB, 0.50916E+00_JPRB, 0.52673E+00_JPRB, 0.54039E+00_JPRB, 0.55216E+00_JPRB, 0.56389E+00_JPRB ,&
&  0.57647E+00_JPRB, 0.59434E+00_JPRB, 0.61401E+00_JPRB, 0.63238E+00_JPRB, 0.70024E+00_JPRB, 0.85118E+00_JPRB, 0.91441E+00_JPRB ,&
&  0.98074E+00_JPRB, 0.87699E+00_JPRB, 0.82175E+00_JPRB, 0.83158E+00_JPRB, 0.97749E+00_JPRB, 0.10596E+01_JPRB, 0.11037E+01_JPRB ,&
&  0.11605E+01_JPRB, 0.11523E+01_JPRB, 0.11281E+01_JPRB, 0.10720E+01_JPRB, 0.98844E+00_JPRB, 0.92593E+00_JPRB, 0.90116E+00_JPRB ,&
&  0.86650E+00_JPRB, 0.86381E+00_JPRB, 0.91056E+00_JPRB, 0.10141E+01_JPRB, 0.11331E+01_JPRB, 0.12340E+01_JPRB, 0.13059E+01_JPRB ,&
&  0.13746E+01_JPRB, 0.14068E+01_JPRB, 0.13897E+01_JPRB, 0.13568E+01_JPRB, 0.12997E+01_JPRB, 0.12662E+01_JPRB, 0.12535E+01_JPRB ,&
&  0.12368E+01_JPRB, 0.12220E+01_JPRB, 0.11907E+01_JPRB, 0.11450E+01_JPRB, 0.10698E+01_JPRB, 0.99540E+00_JPRB, 0.89168E+00_JPRB ,&
&  0.72784E+00_JPRB, 0.63233E+00_JPRB, 0.56504E+00_JPRB, 0.54614E+00_JPRB, 0.54870E+00_JPRB, 0.55593E+00_JPRB, 0.55697E+00_JPRB ,&
&  0.57160E+00_JPRB, 0.58103E+00_JPRB, 0.58334E+00_JPRB, 0.59858E+00_JPRB, 0.61967E+00_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.68599E+00_JPRB, 0.68874E+00_JPRB, 0.68640E+00_JPRB, 0.68458E+00_JPRB, 0.67800E+00_JPRB, 0.64695E+00_JPRB, 0.62028E+00_JPRB ,&
&  0.67408E+00_JPRB, 0.63334E+00_JPRB, 0.64914E+00_JPRB, 0.55680E+00_JPRB, 0.51878E+00_JPRB, 0.50842E+00_JPRB, 0.48858E+00_JPRB ,&
&  0.47909E+00_JPRB, 0.47939E+00_JPRB, 0.47625E+00_JPRB, 0.47545E+00_JPRB, 0.47273E+00_JPRB, 0.46260E+00_JPRB, 0.45441E+00_JPRB ,&
&  0.45134E+00_JPRB, 0.45523E+00_JPRB, 0.46480E+00_JPRB, 0.47170E+00_JPRB, 0.47758E+00_JPRB, 0.48260E+00_JPRB, 0.48718E+00_JPRB ,&
&  0.47677E+00_JPRB, 0.47223E+00_JPRB, 0.47274E+00_JPRB, 0.47009E+00_JPRB, 0.47376E+00_JPRB, 0.48377E+00_JPRB, 0.50184E+00_JPRB ,&
&  0.51953E+00_JPRB, 0.50680E+00_JPRB, 0.54239E+00_JPRB, 0.55906E+00_JPRB, 0.57260E+00_JPRB, 0.58425E+00_JPRB, 0.59420E+00_JPRB ,&
&  0.60508E+00_JPRB, 0.62168E+00_JPRB, 0.64119E+00_JPRB, 0.66030E+00_JPRB, 0.78094E+00_JPRB, 0.10166E+01_JPRB, 0.10380E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.87791E+00_JPRB, 0.97488E+00_JPRB, 0.10226E+01_JPRB, 0.11413E+01_JPRB, 0.12577E+01_JPRB, 0.12543E+01_JPRB ,&
&  0.13892E+01_JPRB, 0.14450E+01_JPRB, 0.13858E+01_JPRB, 0.12677E+01_JPRB, 0.11631E+01_JPRB, 0.11102E+01_JPRB, 0.10874E+01_JPRB ,&
&  0.11017E+01_JPRB, 0.10890E+01_JPRB, 0.12235E+01_JPRB, 0.13312E+01_JPRB, 0.13850E+01_JPRB, 0.14388E+01_JPRB, 0.15057E+01_JPRB ,&
&  0.15513E+01_JPRB, 0.15571E+01_JPRB, 0.15514E+01_JPRB, 0.14688E+01_JPRB, 0.14269E+01_JPRB, 0.13951E+01_JPRB, 0.13373E+01_JPRB ,&
&  0.12997E+01_JPRB, 0.12680E+01_JPRB, 0.12222E+01_JPRB, 0.12266E+01_JPRB, 0.11171E+01_JPRB, 0.10395E+01_JPRB, 0.98916E+00_JPRB ,&
&  0.85968E+00_JPRB, 0.71396E+00_JPRB, 0.66502E+00_JPRB, 0.64160E+00_JPRB, 0.62518E+00_JPRB, 0.62408E+00_JPRB, 0.63656E+00_JPRB ,&
&  0.62033E+00_JPRB, 0.62984E+00_JPRB, 0.66561E+00_JPRB, 0.67480E+00_JPRB, 0.67783E+00_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.72800E+00_JPRB, 0.73358E+00_JPRB, 0.73239E+00_JPRB, 0.73064E+00_JPRB, 0.72635E+00_JPRB, 0.71632E+00_JPRB, 0.67985E+00_JPRB ,&
&  0.70237E+00_JPRB, 0.68374E+00_JPRB, 0.65370E+00_JPRB, 0.63775E+00_JPRB, 0.64167E+00_JPRB, 0.59903E+00_JPRB, 0.51615E+00_JPRB ,&
&  0.48799E+00_JPRB, 0.49038E+00_JPRB, 0.49981E+00_JPRB, 0.50208E+00_JPRB, 0.50114E+00_JPRB, 0.49656E+00_JPRB, 0.49267E+00_JPRB ,&
&  0.49216E+00_JPRB, 0.49467E+00_JPRB, 0.50391E+00_JPRB, 0.51598E+00_JPRB, 0.51864E+00_JPRB, 0.52178E+00_JPRB, 0.52751E+00_JPRB ,&
&  0.51078E+00_JPRB, 0.49515E+00_JPRB, 0.48609E+00_JPRB, 0.47217E+00_JPRB, 0.49370E+00_JPRB, 0.51003E+00_JPRB, 0.53223E+00_JPRB ,&
&  0.55472E+00_JPRB, 0.57643E+00_JPRB, 0.59708E+00_JPRB, 0.61318E+00_JPRB, 0.62286E+00_JPRB, 0.63254E+00_JPRB, 0.64176E+00_JPRB ,&
&  0.65276E+00_JPRB, 0.66902E+00_JPRB, 0.69027E+00_JPRB, 0.70961E+00_JPRB, 0.94315E+00_JPRB, 0.11785E+01_JPRB, 0.12549E+01_JPRB ,&
&  0.12255E+01_JPRB, 0.13425E+01_JPRB, 0.14135E+01_JPRB, 0.13475E+01_JPRB, 0.14496E+01_JPRB, 0.15118E+01_JPRB, 0.14752E+01_JPRB ,&
&  0.15490E+01_JPRB, 0.15421E+01_JPRB, 0.15071E+01_JPRB, 0.14657E+01_JPRB, 0.14405E+01_JPRB, 0.13974E+01_JPRB, 0.13657E+01_JPRB ,&
&  0.13933E+01_JPRB, 0.15264E+01_JPRB, 0.15410E+01_JPRB, 0.15721E+01_JPRB, 0.15781E+01_JPRB, 0.16014E+01_JPRB, 0.16269E+01_JPRB ,&
&  0.16460E+01_JPRB, 0.16711E+01_JPRB, 0.16741E+01_JPRB, 0.16296E+01_JPRB, 0.15692E+01_JPRB, 0.15051E+01_JPRB, 0.14266E+01_JPRB ,&
&  0.13638E+01_JPRB, 0.12286E+01_JPRB, 0.11212E+01_JPRB, 0.10841E+01_JPRB, 0.10472E+01_JPRB, 0.96313E+00_JPRB, 0.93390E+00_JPRB ,&
&  0.89974E+00_JPRB, 0.82977E+00_JPRB, 0.84254E+00_JPRB, 0.79059E+00_JPRB, 0.75775E+00_JPRB, 0.72078E+00_JPRB, 0.65482E+00_JPRB ,&
&  0.62905E+00_JPRB, 0.67777E+00_JPRB, 0.70415E+00_JPRB, 0.71852E+00_JPRB, 0.72280E+00_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.78148E+00_JPRB, 0.79140E+00_JPRB, 0.79977E+00_JPRB, 0.79660E+00_JPRB, 0.77691E+00_JPRB, 0.75062E+00_JPRB, 0.72694E+00_JPRB ,&
&  0.71168E+00_JPRB, 0.69155E+00_JPRB, 0.67238E+00_JPRB, 0.65640E+00_JPRB, 0.64932E+00_JPRB, 0.64896E+00_JPRB, 0.60498E+00_JPRB ,&
&  0.54160E+00_JPRB, 0.52796E+00_JPRB, 0.51566E+00_JPRB, 0.51673E+00_JPRB, 0.52711E+00_JPRB, 0.53209E+00_JPRB, 0.53686E+00_JPRB ,&
&  0.55153E+00_JPRB, 0.56078E+00_JPRB, 0.56469E+00_JPRB, 0.56924E+00_JPRB, 0.57230E+00_JPRB, 0.58029E+00_JPRB, 0.58792E+00_JPRB ,&
&  0.56273E+00_JPRB, 0.53064E+00_JPRB, 0.51344E+00_JPRB, 0.50711E+00_JPRB, 0.52945E+00_JPRB, 0.55989E+00_JPRB, 0.59401E+00_JPRB ,&
&  0.62369E+00_JPRB, 0.64792E+00_JPRB, 0.67021E+00_JPRB, 0.68705E+00_JPRB, 0.69406E+00_JPRB, 0.70135E+00_JPRB, 0.71328E+00_JPRB ,&
&  0.73384E+00_JPRB, 0.74527E+00_JPRB, 0.75638E+00_JPRB, 0.78499E+00_JPRB, 0.11409E+01_JPRB, 0.15630E+01_JPRB, 0.17618E+01_JPRB ,&
&  0.17547E+01_JPRB, 0.17904E+01_JPRB, 0.18259E+01_JPRB, 0.19330E+01_JPRB, 0.19349E+01_JPRB, 0.18281E+01_JPRB, 0.17268E+01_JPRB ,&
&  0.17451E+01_JPRB, 0.16618E+01_JPRB, 0.17649E+01_JPRB, 0.17498E+01_JPRB, 0.17556E+01_JPRB, 0.17785E+01_JPRB, 0.17556E+01_JPRB ,&
&  0.17913E+01_JPRB, 0.18187E+01_JPRB, 0.17942E+01_JPRB, 0.17340E+01_JPRB, 0.17045E+01_JPRB, 0.17225E+01_JPRB, 0.17386E+01_JPRB ,&
&  0.17941E+01_JPRB, 0.18367E+01_JPRB, 0.17583E+01_JPRB, 0.16954E+01_JPRB, 0.16154E+01_JPRB, 0.16911E+01_JPRB, 0.15595E+01_JPRB ,&
&  0.13416E+01_JPRB, 0.10793E+01_JPRB, 0.10634E+01_JPRB, 0.10478E+01_JPRB, 0.10515E+01_JPRB, 0.10182E+01_JPRB, 0.95043E+00_JPRB ,&
&  0.94847E+00_JPRB, 0.10097E+01_JPRB, 0.98689E+00_JPRB, 0.89999E+00_JPRB, 0.86012E+00_JPRB, 0.80157E+00_JPRB, 0.69314E+00_JPRB ,&
&  0.68739E+00_JPRB, 0.74605E+00_JPRB, 0.75685E+00_JPRB, 0.76032E+00_JPRB, 0.76604E+00_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.85596E+00_JPRB, 0.84599E+00_JPRB, 0.84834E+00_JPRB, 0.84050E+00_JPRB, 0.81794E+00_JPRB, 0.79014E+00_JPRB, 0.76145E+00_JPRB ,&
&  0.74088E+00_JPRB, 0.72593E+00_JPRB, 0.71251E+00_JPRB, 0.70115E+00_JPRB, 0.69421E+00_JPRB, 0.68182E+00_JPRB, 0.68367E+00_JPRB ,&
&  0.59754E+00_JPRB, 0.56769E+00_JPRB, 0.54922E+00_JPRB, 0.54940E+00_JPRB, 0.58759E+00_JPRB, 0.58646E+00_JPRB, 0.59255E+00_JPRB ,&
&  0.60128E+00_JPRB, 0.60924E+00_JPRB, 0.60361E+00_JPRB, 0.60376E+00_JPRB, 0.61455E+00_JPRB, 0.64368E+00_JPRB, 0.67757E+00_JPRB ,&
&  0.65643E+00_JPRB, 0.60971E+00_JPRB, 0.59183E+00_JPRB, 0.58778E+00_JPRB, 0.62313E+00_JPRB, 0.65072E+00_JPRB, 0.70870E+00_JPRB ,&
&  0.73817E+00_JPRB, 0.75519E+00_JPRB, 0.76479E+00_JPRB, 0.77160E+00_JPRB, 0.78269E+00_JPRB, 0.80396E+00_JPRB, 0.83245E+00_JPRB ,&
&  0.85762E+00_JPRB, 0.84961E+00_JPRB, 0.85588E+00_JPRB, 0.92126E+00_JPRB, 0.14523E+01_JPRB, 0.20424E+01_JPRB, 0.23935E+01_JPRB ,&
&  0.25522E+01_JPRB, 0.24679E+01_JPRB, 0.24842E+01_JPRB, 0.27953E+01_JPRB, 0.27085E+01_JPRB, 0.24410E+01_JPRB, 0.21898E+01_JPRB ,&
&  0.20730E+01_JPRB, 0.19655E+01_JPRB, 0.20293E+01_JPRB, 0.20853E+01_JPRB, 0.20193E+01_JPRB, 0.20109E+01_JPRB, 0.19988E+01_JPRB ,&
&  0.20000E+01_JPRB, 0.20086E+01_JPRB, 0.19289E+01_JPRB, 0.18462E+01_JPRB, 0.17816E+01_JPRB, 0.18354E+01_JPRB, 0.18733E+01_JPRB ,&
&  0.19787E+01_JPRB, 0.19510E+01_JPRB, 0.17448E+01_JPRB, 0.17341E+01_JPRB, 0.14139E+01_JPRB, 0.13424E+01_JPRB, 0.15145E+01_JPRB ,&
&  0.12922E+01_JPRB, 0.11092E+01_JPRB, 0.10495E+01_JPRB, 0.10647E+01_JPRB, 0.10872E+01_JPRB, 0.11393E+01_JPRB, 0.10932E+01_JPRB ,&
&  0.97936E+00_JPRB, 0.10255E+01_JPRB, 0.11252E+01_JPRB, 0.10331E+01_JPRB, 0.98137E+00_JPRB, 0.92560E+00_JPRB, 0.83638E+00_JPRB ,&
&  0.85966E+00_JPRB, 0.84526E+00_JPRB, 0.83827E+00_JPRB, 0.83842E+00_JPRB, 0.84871E+00_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.89597E+00_JPRB, 0.85914E+00_JPRB, 0.87230E+00_JPRB, 0.83351E+00_JPRB, 0.81320E+00_JPRB, 0.79450E+00_JPRB, 0.78132E+00_JPRB ,&
&  0.77328E+00_JPRB, 0.76355E+00_JPRB, 0.75279E+00_JPRB, 0.74129E+00_JPRB, 0.73276E+00_JPRB, 0.71957E+00_JPRB, 0.70818E+00_JPRB ,&
&  0.70217E+00_JPRB, 0.64515E+00_JPRB, 0.62847E+00_JPRB, 0.60095E+00_JPRB, 0.63254E+00_JPRB, 0.63385E+00_JPRB, 0.64452E+00_JPRB ,&
&  0.65951E+00_JPRB, 0.67951E+00_JPRB, 0.67776E+00_JPRB, 0.67967E+00_JPRB, 0.69630E+00_JPRB, 0.74587E+00_JPRB, 0.80797E+00_JPRB ,&
&  0.82362E+00_JPRB, 0.78325E+00_JPRB, 0.76210E+00_JPRB, 0.78207E+00_JPRB, 0.85987E+00_JPRB, 0.80939E+00_JPRB, 0.84178E+00_JPRB ,&
&  0.85189E+00_JPRB, 0.84938E+00_JPRB, 0.84822E+00_JPRB, 0.86079E+00_JPRB, 0.90117E+00_JPRB, 0.95490E+00_JPRB, 0.97914E+00_JPRB ,&
&  0.98811E+00_JPRB, 0.93226E+00_JPRB, 0.89882E+00_JPRB, 0.11695E+01_JPRB, 0.18487E+01_JPRB, 0.26630E+01_JPRB, 0.30044E+01_JPRB ,&
&  0.30832E+01_JPRB, 0.30316E+01_JPRB, 0.30064E+01_JPRB, 0.32760E+01_JPRB, 0.30940E+01_JPRB, 0.27838E+01_JPRB, 0.26057E+01_JPRB ,&
&  0.24771E+01_JPRB, 0.23988E+01_JPRB, 0.24173E+01_JPRB, 0.22764E+01_JPRB, 0.21233E+01_JPRB, 0.20326E+01_JPRB, 0.19687E+01_JPRB ,&
&  0.18819E+01_JPRB, 0.18626E+01_JPRB, 0.18436E+01_JPRB, 0.17377E+01_JPRB, 0.16908E+01_JPRB, 0.16641E+01_JPRB, 0.18290E+01_JPRB ,&
&  0.20478E+01_JPRB, 0.15991E+01_JPRB, 0.13469E+01_JPRB, 0.14682E+01_JPRB, 0.11402E+01_JPRB, 0.10978E+01_JPRB, 0.12117E+01_JPRB ,&
&  0.11273E+01_JPRB, 0.10863E+01_JPRB, 0.11184E+01_JPRB, 0.10812E+01_JPRB, 0.11187E+01_JPRB, 0.12125E+01_JPRB, 0.11992E+01_JPRB ,&
&  0.11385E+01_JPRB, 0.10995E+01_JPRB, 0.12821E+01_JPRB, 0.11913E+01_JPRB, 0.11273E+01_JPRB, 0.10694E+01_JPRB, 0.10188E+01_JPRB ,&
&  0.99123E+00_JPRB, 0.96454E+00_JPRB, 0.93564E+00_JPRB, 0.91952E+00_JPRB, 0.91769E+00_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.91151E+00_JPRB, 0.87229E+00_JPRB, 0.90318E+00_JPRB, 0.88030E+00_JPRB, 0.87111E+00_JPRB, 0.86140E+00_JPRB, 0.85162E+00_JPRB ,&
&  0.82726E+00_JPRB, 0.81657E+00_JPRB, 0.80417E+00_JPRB, 0.77943E+00_JPRB, 0.76049E+00_JPRB, 0.74479E+00_JPRB, 0.73641E+00_JPRB ,&
&  0.72349E+00_JPRB, 0.72802E+00_JPRB, 0.68521E+00_JPRB, 0.67181E+00_JPRB, 0.65052E+00_JPRB, 0.67428E+00_JPRB, 0.69752E+00_JPRB ,&
&  0.73676E+00_JPRB, 0.77681E+00_JPRB, 0.81742E+00_JPRB, 0.86961E+00_JPRB, 0.92249E+00_JPRB, 0.97109E+00_JPRB, 0.10580E+01_JPRB ,&
&  0.10898E+01_JPRB, 0.10380E+01_JPRB, 0.98374E+00_JPRB, 0.10110E+01_JPRB, 0.10406E+01_JPRB, 0.97792E+00_JPRB, 0.99328E+00_JPRB ,&
&  0.98150E+00_JPRB, 0.97442E+00_JPRB, 0.97819E+00_JPRB, 0.10182E+01_JPRB, 0.10269E+01_JPRB, 0.96638E+00_JPRB, 0.90548E+00_JPRB ,&
&  0.88246E+00_JPRB, 0.91143E+00_JPRB, 0.11158E+01_JPRB, 0.15988E+01_JPRB, 0.24253E+01_JPRB, 0.33894E+01_JPRB, 0.36251E+01_JPRB ,&
&  0.36381E+01_JPRB, 0.32813E+01_JPRB, 0.25666E+01_JPRB, 0.28125E+01_JPRB, 0.33823E+01_JPRB, 0.31549E+01_JPRB, 0.28322E+01_JPRB ,&
&  0.27960E+01_JPRB, 0.25943E+01_JPRB, 0.24960E+01_JPRB, 0.22697E+01_JPRB, 0.21065E+01_JPRB, 0.19498E+01_JPRB, 0.18090E+01_JPRB ,&
&  0.16929E+01_JPRB, 0.16505E+01_JPRB, 0.16417E+01_JPRB, 0.16574E+01_JPRB, 0.16567E+01_JPRB, 0.17861E+01_JPRB, 0.20205E+01_JPRB ,&
&  0.20246E+01_JPRB, 0.18679E+01_JPRB, 0.14543E+01_JPRB, 0.13959E+01_JPRB, 0.10672E+01_JPRB, 0.10236E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.10860E+01_JPRB, 0.11788E+01_JPRB, 0.13038E+01_JPRB, 0.12933E+01_JPRB, 0.14428E+01_JPRB, 0.14803E+01_JPRB, 0.14701E+01_JPRB ,&
&  0.14114E+01_JPRB, 0.12902E+01_JPRB, 0.14969E+01_JPRB, 0.13680E+01_JPRB, 0.12681E+01_JPRB, 0.11876E+01_JPRB, 0.11220E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.10468E+01_JPRB, 0.10016E+01_JPRB, 0.97815E+00_JPRB, 0.95476E+00_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.90438E+00_JPRB, 0.88128E+00_JPRB, 0.91766E+00_JPRB, 0.89133E+00_JPRB, 0.89673E+00_JPRB, 0.86556E+00_JPRB, 0.85365E+00_JPRB ,&
&  0.84591E+00_JPRB, 0.83000E+00_JPRB, 0.80580E+00_JPRB, 0.78061E+00_JPRB, 0.76193E+00_JPRB, 0.74699E+00_JPRB, 0.74436E+00_JPRB ,&
&  0.73586E+00_JPRB, 0.72035E+00_JPRB, 0.65652E+00_JPRB, 0.62440E+00_JPRB, 0.61281E+00_JPRB, 0.64989E+00_JPRB, 0.71889E+00_JPRB ,&
&  0.80358E+00_JPRB, 0.88442E+00_JPRB, 0.10011E+01_JPRB, 0.10910E+01_JPRB, 0.11737E+01_JPRB, 0.12467E+01_JPRB, 0.13403E+01_JPRB ,&
&  0.14226E+01_JPRB, 0.12405E+01_JPRB, 0.12064E+01_JPRB, 0.11212E+01_JPRB, 0.11065E+01_JPRB, 0.10915E+01_JPRB, 0.10540E+01_JPRB ,&
&  0.10235E+01_JPRB, 0.10214E+01_JPRB, 0.10181E+01_JPRB, 0.97742E+00_JPRB, 0.92477E+00_JPRB, 0.89106E+00_JPRB, 0.89209E+00_JPRB ,&
&  0.94149E+00_JPRB, 0.10820E+01_JPRB, 0.15596E+01_JPRB, 0.22755E+01_JPRB, 0.34273E+01_JPRB, 0.43871E+01_JPRB, 0.43251E+01_JPRB ,&
&  0.33455E+01_JPRB, 0.29385E+01_JPRB, 0.37891E+01_JPRB, 0.33040E+01_JPRB, 0.33081E+01_JPRB, 0.34489E+01_JPRB, 0.31741E+01_JPRB ,&
&  0.27843E+01_JPRB, 0.23430E+01_JPRB, 0.22708E+01_JPRB, 0.21471E+01_JPRB, 0.27007E+01_JPRB, 0.23130E+01_JPRB, 0.20282E+01_JPRB ,&
&  0.18521E+01_JPRB, 0.18189E+01_JPRB, 0.17782E+01_JPRB, 0.18274E+01_JPRB, 0.20190E+01_JPRB, 0.23814E+01_JPRB, 0.24697E+01_JPRB ,&
&  0.22121E+01_JPRB, 0.21975E+01_JPRB, 0.18281E+01_JPRB, 0.14836E+01_JPRB, 0.12487E+01_JPRB, 0.12169E+01_JPRB, 0.12212E+01_JPRB ,&
&  0.12527E+01_JPRB, 0.13067E+01_JPRB, 0.15265E+01_JPRB, 0.17759E+01_JPRB, 0.20080E+01_JPRB, 0.19911E+01_JPRB, 0.17358E+01_JPRB ,&
&  0.16850E+01_JPRB, 0.18840E+01_JPRB, 0.18746E+01_JPRB, 0.15536E+01_JPRB, 0.13871E+01_JPRB, 0.12851E+01_JPRB, 0.12092E+01_JPRB ,&
&  0.11339E+01_JPRB, 0.10676E+01_JPRB, 0.10207E+01_JPRB, 0.98533E+00_JPRB, 0.95747E+00_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.93715E+00_JPRB, 0.92085E+00_JPRB, 0.89682E+00_JPRB, 0.88360E+00_JPRB, 0.87909E+00_JPRB, 0.83478E+00_JPRB, 0.80636E+00_JPRB ,&
&  0.78299E+00_JPRB, 0.76155E+00_JPRB, 0.76894E+00_JPRB, 0.77615E+00_JPRB, 0.77015E+00_JPRB, 0.75403E+00_JPRB, 0.74968E+00_JPRB ,&
&  0.73987E+00_JPRB, 0.68399E+00_JPRB, 0.62925E+00_JPRB, 0.63453E+00_JPRB, 0.63611E+00_JPRB, 0.64314E+00_JPRB, 0.73207E+00_JPRB ,&
&  0.86561E+00_JPRB, 0.10310E+01_JPRB, 0.11920E+01_JPRB, 0.13022E+01_JPRB, 0.13571E+01_JPRB, 0.14139E+01_JPRB, 0.14848E+01_JPRB ,&
&  0.14043E+01_JPRB, 0.14000E+01_JPRB, 0.13849E+01_JPRB, 0.12712E+01_JPRB, 0.11878E+01_JPRB, 0.11069E+01_JPRB, 0.10656E+01_JPRB ,&
&  0.10455E+01_JPRB, 0.10554E+01_JPRB, 0.10224E+01_JPRB, 0.97048E+00_JPRB, 0.98688E+00_JPRB, 0.10027E+01_JPRB, 0.10065E+01_JPRB ,&
&  0.10264E+01_JPRB, 0.13455E+01_JPRB, 0.19667E+01_JPRB, 0.28043E+01_JPRB, 0.30698E+01_JPRB, 0.39727E+01_JPRB, 0.39572E+01_JPRB ,&
&  0.44404E+01_JPRB, 0.41888E+01_JPRB, 0.39022E+01_JPRB, 0.42225E+01_JPRB, 0.34247E+01_JPRB, 0.33346E+01_JPRB, 0.31943E+01_JPRB ,&
&  0.28378E+01_JPRB, 0.26639E+01_JPRB, 0.28974E+01_JPRB, 0.26622E+01_JPRB, 0.28706E+01_JPRB, 0.31939E+01_JPRB, 0.28434E+01_JPRB ,&
&  0.26781E+01_JPRB, 0.25331E+01_JPRB, 0.23667E+01_JPRB, 0.24764E+01_JPRB, 0.24825E+01_JPRB, 0.25449E+01_JPRB, 0.21635E+01_JPRB ,&
&  0.20367E+01_JPRB, 0.17135E+01_JPRB, 0.18157E+01_JPRB, 0.16142E+01_JPRB, 0.15368E+01_JPRB, 0.14555E+01_JPRB, 0.13967E+01_JPRB ,&
&  0.14750E+01_JPRB, 0.15245E+01_JPRB, 0.19029E+01_JPRB, 0.22978E+01_JPRB, 0.26655E+01_JPRB, 0.23964E+01_JPRB, 0.21497E+01_JPRB ,&
&  0.22998E+01_JPRB, 0.23731E+01_JPRB, 0.20119E+01_JPRB, 0.16661E+01_JPRB, 0.14472E+01_JPRB, 0.13477E+01_JPRB, 0.12750E+01_JPRB ,&
&  0.11964E+01_JPRB, 0.11206E+01_JPRB, 0.10692E+01_JPRB, 0.10289E+01_JPRB, 0.99211E+00_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.87722E+00_JPRB, 0.86824E+00_JPRB, 0.84151E+00_JPRB, 0.83929E+00_JPRB, 0.84660E+00_JPRB, 0.81091E+00_JPRB, 0.79189E+00_JPRB ,&
&  0.76286E+00_JPRB, 0.74869E+00_JPRB, 0.75562E+00_JPRB, 0.77102E+00_JPRB, 0.76179E+00_JPRB, 0.75793E+00_JPRB, 0.74729E+00_JPRB ,&
&  0.73577E+00_JPRB, 0.69690E+00_JPRB, 0.64910E+00_JPRB, 0.64405E+00_JPRB, 0.67088E+00_JPRB, 0.68731E+00_JPRB, 0.75209E+00_JPRB ,&
&  0.99389E+00_JPRB, 0.12228E+01_JPRB, 0.13531E+01_JPRB, 0.14233E+01_JPRB, 0.14346E+01_JPRB, 0.14415E+01_JPRB, 0.14215E+01_JPRB ,&
&  0.14017E+01_JPRB, 0.13196E+01_JPRB, 0.12492E+01_JPRB, 0.11930E+01_JPRB, 0.11413E+01_JPRB, 0.10949E+01_JPRB, 0.10545E+01_JPRB ,&
&  0.10242E+01_JPRB, 0.95675E+00_JPRB, 0.89159E+00_JPRB, 0.84895E+00_JPRB, 0.87981E+00_JPRB, 0.86625E+00_JPRB, 0.93179E+00_JPRB ,&
&  0.11394E+01_JPRB, 0.15063E+01_JPRB, 0.20034E+01_JPRB, 0.26465E+01_JPRB, 0.30912E+01_JPRB, 0.40072E+01_JPRB, 0.45558E+01_JPRB ,&
&  0.43557E+01_JPRB, 0.39136E+01_JPRB, 0.43591E+01_JPRB, 0.48651E+01_JPRB, 0.44665E+01_JPRB, 0.37675E+01_JPRB, 0.34785E+01_JPRB ,&
&  0.26463E+01_JPRB, 0.21475E+01_JPRB, 0.20325E+01_JPRB, 0.23581E+01_JPRB, 0.35669E+01_JPRB, 0.45725E+01_JPRB, 0.43467E+01_JPRB ,&
&  0.42237E+01_JPRB, 0.39442E+01_JPRB, 0.36140E+01_JPRB, 0.31222E+01_JPRB, 0.28227E+01_JPRB, 0.25680E+01_JPRB, 0.31466E+01_JPRB ,&
&  0.30305E+01_JPRB, 0.29482E+01_JPRB, 0.24587E+01_JPRB, 0.19531E+01_JPRB, 0.15838E+01_JPRB, 0.16699E+01_JPRB, 0.16460E+01_JPRB ,&
&  0.17211E+01_JPRB, 0.18589E+01_JPRB, 0.29705E+01_JPRB, 0.33920E+01_JPRB, 0.32903E+01_JPRB, 0.31353E+01_JPRB, 0.30698E+01_JPRB ,&
&  0.27956E+01_JPRB, 0.27617E+01_JPRB, 0.22659E+01_JPRB, 0.17463E+01_JPRB, 0.14429E+01_JPRB, 0.12604E+01_JPRB, 0.11542E+01_JPRB ,&
&  0.11208E+01_JPRB, 0.10895E+01_JPRB, 0.10408E+01_JPRB, 0.97747E+00_JPRB, 0.92970E+00_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.81806E+00_JPRB, 0.80758E+00_JPRB, 0.77604E+00_JPRB, 0.77511E+00_JPRB, 0.78780E+00_JPRB, 0.79475E+00_JPRB, 0.77950E+00_JPRB ,&
&  0.75857E+00_JPRB, 0.77131E+00_JPRB, 0.77754E+00_JPRB, 0.77162E+00_JPRB, 0.75588E+00_JPRB, 0.76042E+00_JPRB, 0.75647E+00_JPRB ,&
&  0.76479E+00_JPRB, 0.78572E+00_JPRB, 0.71488E+00_JPRB, 0.71791E+00_JPRB, 0.75505E+00_JPRB, 0.80320E+00_JPRB, 0.87992E+00_JPRB ,&
&  0.11975E+01_JPRB, 0.13502E+01_JPRB, 0.13516E+01_JPRB, 0.14060E+01_JPRB, 0.14676E+01_JPRB, 0.13178E+01_JPRB, 0.12875E+01_JPRB ,&
&  0.13034E+01_JPRB, 0.12252E+01_JPRB, 0.11677E+01_JPRB, 0.11222E+01_JPRB, 0.11022E+01_JPRB, 0.10897E+01_JPRB, 0.10426E+01_JPRB ,&
&  0.96510E+00_JPRB, 0.88713E+00_JPRB, 0.85585E+00_JPRB, 0.85066E+00_JPRB, 0.79686E+00_JPRB, 0.80617E+00_JPRB, 0.88994E+00_JPRB ,&
&  0.11709E+01_JPRB, 0.15474E+01_JPRB, 0.18365E+01_JPRB, 0.22798E+01_JPRB, 0.27938E+01_JPRB, 0.38058E+01_JPRB, 0.53591E+01_JPRB ,&
&  0.49409E+01_JPRB, 0.48601E+01_JPRB, 0.47792E+01_JPRB, 0.54565E+01_JPRB, 0.45757E+01_JPRB, 0.38863E+01_JPRB, 0.34610E+01_JPRB ,&
&  0.25615E+01_JPRB, 0.21734E+01_JPRB, 0.24857E+01_JPRB, 0.32325E+01_JPRB, 0.36344E+01_JPRB, 0.42211E+01_JPRB, 0.52323E+01_JPRB ,&
&  0.46969E+01_JPRB, 0.45130E+01_JPRB, 0.43101E+01_JPRB, 0.40356E+01_JPRB, 0.24230E+01_JPRB, 0.19602E+01_JPRB, 0.27975E+01_JPRB ,&
&  0.28808E+01_JPRB, 0.18590E+01_JPRB, 0.12153E+01_JPRB, 0.14941E+01_JPRB, 0.10804E+01_JPRB, 0.11705E+01_JPRB, 0.15742E+01_JPRB ,&
&  0.19401E+01_JPRB, 0.25000E+01_JPRB, 0.41449E+01_JPRB, 0.39215E+01_JPRB, 0.37001E+01_JPRB, 0.43623E+01_JPRB, 0.38441E+01_JPRB ,&
&  0.35776E+01_JPRB, 0.27017E+01_JPRB, 0.21886E+01_JPRB, 0.15935E+01_JPRB, 0.12069E+01_JPRB, 0.10113E+01_JPRB, 0.93882E+00_JPRB ,&
&  0.91406E+00_JPRB, 0.91870E+00_JPRB, 0.91077E+00_JPRB, 0.87980E+00_JPRB, 0.84728E+00_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.83622E+00_JPRB, 0.78202E+00_JPRB, 0.71650E+00_JPRB, 0.71900E+00_JPRB, 0.73578E+00_JPRB, 0.76026E+00_JPRB, 0.75024E+00_JPRB ,&
&  0.75888E+00_JPRB, 0.78305E+00_JPRB, 0.81353E+00_JPRB, 0.81159E+00_JPRB, 0.78263E+00_JPRB, 0.78757E+00_JPRB, 0.77251E+00_JPRB ,&
&  0.77365E+00_JPRB, 0.81722E+00_JPRB, 0.85239E+00_JPRB, 0.89846E+00_JPRB, 0.96149E+00_JPRB, 0.99502E+00_JPRB, 0.11788E+01_JPRB ,&
&  0.13769E+01_JPRB, 0.13684E+01_JPRB, 0.13539E+01_JPRB, 0.13892E+01_JPRB, 0.13308E+01_JPRB, 0.11619E+01_JPRB, 0.12196E+01_JPRB ,&
&  0.12270E+01_JPRB, 0.11802E+01_JPRB, 0.11150E+01_JPRB, 0.10721E+01_JPRB, 0.10646E+01_JPRB, 0.10587E+01_JPRB, 0.10187E+01_JPRB ,&
&  0.93976E+00_JPRB, 0.91803E+00_JPRB, 0.89023E+00_JPRB, 0.79835E+00_JPRB, 0.76423E+00_JPRB, 0.79494E+00_JPRB, 0.90207E+00_JPRB ,&
&  0.11660E+01_JPRB, 0.14709E+01_JPRB, 0.16587E+01_JPRB, 0.19683E+01_JPRB, 0.30899E+01_JPRB, 0.39354E+01_JPRB, 0.41054E+01_JPRB ,&
&  0.40359E+01_JPRB, 0.40848E+01_JPRB, 0.52184E+01_JPRB, 0.52594E+01_JPRB, 0.48146E+01_JPRB, 0.45606E+01_JPRB, 0.38005E+01_JPRB ,&
&  0.32867E+01_JPRB, 0.29806E+01_JPRB, 0.32017E+01_JPRB, 0.37655E+01_JPRB, 0.43987E+01_JPRB, 0.44078E+01_JPRB, 0.53780E+01_JPRB ,&
&  0.54411E+01_JPRB, 0.51107E+01_JPRB, 0.39953E+01_JPRB, 0.30074E+01_JPRB, 0.27160E+01_JPRB, 0.19898E+01_JPRB, 0.13658E+01_JPRB ,&
&  0.11900E+01_JPRB, 0.11258E+01_JPRB, 0.99119E+00_JPRB, 0.99657E+00_JPRB, 0.88523E+00_JPRB, 0.98547E+00_JPRB, 0.13807E+01_JPRB ,&
&  0.23997E+01_JPRB, 0.35110E+01_JPRB, 0.41080E+01_JPRB, 0.36254E+01_JPRB, 0.34297E+01_JPRB, 0.42760E+01_JPRB, 0.48060E+01_JPRB ,&
&  0.27815E+01_JPRB, 0.22528E+01_JPRB, 0.15771E+01_JPRB, 0.11376E+01_JPRB, 0.94951E+00_JPRB, 0.84337E+00_JPRB, 0.80778E+00_JPRB ,&
&  0.80222E+00_JPRB, 0.79672E+00_JPRB, 0.80514E+00_JPRB, 0.82070E+00_JPRB, 0.82161E+00_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.81841E+00_JPRB, 0.73417E+00_JPRB, 0.66942E+00_JPRB, 0.65933E+00_JPRB, 0.69140E+00_JPRB, 0.70918E+00_JPRB, 0.72297E+00_JPRB ,&
&  0.75858E+00_JPRB, 0.81021E+00_JPRB, 0.84029E+00_JPRB, 0.82655E+00_JPRB, 0.81082E+00_JPRB, 0.80585E+00_JPRB, 0.79241E+00_JPRB ,&
&  0.81103E+00_JPRB, 0.89172E+00_JPRB, 0.10336E+01_JPRB, 0.11643E+01_JPRB, 0.13018E+01_JPRB, 0.12650E+01_JPRB, 0.14438E+01_JPRB ,&
&  0.14045E+01_JPRB, 0.13478E+01_JPRB, 0.14200E+01_JPRB, 0.13960E+01_JPRB, 0.13260E+01_JPRB, 0.12190E+01_JPRB, 0.12341E+01_JPRB ,&
&  0.11739E+01_JPRB, 0.11516E+01_JPRB, 0.10754E+01_JPRB, 0.10760E+01_JPRB, 0.10721E+01_JPRB, 0.10477E+01_JPRB, 0.99536E+00_JPRB ,&
&  0.94767E+00_JPRB, 0.96030E+00_JPRB, 0.83927E+00_JPRB, 0.81758E+00_JPRB, 0.82462E+00_JPRB, 0.88728E+00_JPRB, 0.10055E+01_JPRB ,&
&  0.12279E+01_JPRB, 0.14731E+01_JPRB, 0.17000E+01_JPRB, 0.23093E+01_JPRB, 0.28764E+01_JPRB, 0.28363E+01_JPRB, 0.36554E+01_JPRB ,&
&  0.43439E+01_JPRB, 0.48049E+01_JPRB, 0.48093E+01_JPRB, 0.54588E+01_JPRB, 0.52508E+01_JPRB, 0.46643E+01_JPRB, 0.41824E+01_JPRB ,&
&  0.35957E+01_JPRB, 0.37963E+01_JPRB, 0.46670E+01_JPRB, 0.50533E+01_JPRB, 0.61768E+01_JPRB, 0.58821E+01_JPRB, 0.61995E+01_JPRB ,&
&  0.63924E+01_JPRB, 0.58109E+01_JPRB, 0.46552E+01_JPRB, 0.36656E+01_JPRB, 0.39499E+01_JPRB, 0.34687E+01_JPRB, 0.18370E+01_JPRB ,&
&  0.15028E+01_JPRB, 0.12992E+01_JPRB, 0.10365E+01_JPRB, 0.89447E+00_JPRB, 0.84069E+00_JPRB, 0.99372E+00_JPRB, 0.16557E+01_JPRB ,&
&  0.21198E+01_JPRB, 0.36850E+01_JPRB, 0.42607E+01_JPRB, 0.38060E+01_JPRB, 0.33746E+01_JPRB, 0.41233E+01_JPRB, 0.36141E+01_JPRB ,&
&  0.27035E+01_JPRB, 0.19877E+01_JPRB, 0.11553E+01_JPRB, 0.90441E+00_JPRB, 0.84806E+00_JPRB, 0.80665E+00_JPRB, 0.76268E+00_JPRB ,&
&  0.72001E+00_JPRB, 0.71733E+00_JPRB, 0.74010E+00_JPRB, 0.76534E+00_JPRB, 0.77865E+00_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.71351E+00_JPRB, 0.65247E+00_JPRB, 0.62798E+00_JPRB, 0.61382E+00_JPRB, 0.66029E+00_JPRB, 0.71409E+00_JPRB, 0.72593E+00_JPRB ,&
&  0.78137E+00_JPRB, 0.83701E+00_JPRB, 0.86521E+00_JPRB, 0.85602E+00_JPRB, 0.82992E+00_JPRB, 0.82877E+00_JPRB, 0.85659E+00_JPRB ,&
&  0.92858E+00_JPRB, 0.10675E+01_JPRB, 0.12698E+01_JPRB, 0.14848E+01_JPRB, 0.15681E+01_JPRB, 0.15684E+01_JPRB, 0.15359E+01_JPRB ,&
&  0.13502E+01_JPRB, 0.13420E+01_JPRB, 0.14581E+01_JPRB, 0.14874E+01_JPRB, 0.13619E+01_JPRB, 0.13213E+01_JPRB, 0.12395E+01_JPRB ,&
&  0.11655E+01_JPRB, 0.11543E+01_JPRB, 0.11265E+01_JPRB, 0.11342E+01_JPRB, 0.11082E+01_JPRB, 0.10703E+01_JPRB, 0.10423E+01_JPRB ,&
&  0.10141E+01_JPRB, 0.90322E+00_JPRB, 0.84561E+00_JPRB, 0.84762E+00_JPRB, 0.94626E+00_JPRB, 0.10997E+01_JPRB, 0.12692E+01_JPRB ,&
&  0.14805E+01_JPRB, 0.17248E+01_JPRB, 0.20728E+01_JPRB, 0.27074E+01_JPRB, 0.24538E+01_JPRB, 0.31688E+01_JPRB, 0.37206E+01_JPRB ,&
&  0.39983E+01_JPRB, 0.42323E+01_JPRB, 0.41570E+01_JPRB, 0.47271E+01_JPRB, 0.50455E+01_JPRB, 0.46275E+01_JPRB, 0.40651E+01_JPRB ,&
&  0.41297E+01_JPRB, 0.43305E+01_JPRB, 0.60625E+01_JPRB, 0.69202E+01_JPRB, 0.76553E+01_JPRB, 0.98676E+01_JPRB, 0.72682E+01_JPRB ,&
&  0.61044E+01_JPRB, 0.63020E+01_JPRB, 0.53046E+01_JPRB, 0.41969E+01_JPRB, 0.41951E+01_JPRB, 0.45044E+01_JPRB, 0.42331E+01_JPRB ,&
&  0.19804E+01_JPRB, 0.12493E+01_JPRB, 0.11645E+01_JPRB, 0.10413E+01_JPRB, 0.87654E+00_JPRB, 0.10075E+01_JPRB, 0.24515E+01_JPRB ,&
&  0.20828E+01_JPRB, 0.34134E+01_JPRB, 0.38884E+01_JPRB, 0.34195E+01_JPRB, 0.32690E+01_JPRB, 0.33867E+01_JPRB, 0.33662E+01_JPRB ,&
&  0.20094E+01_JPRB, 0.14048E+01_JPRB, 0.99276E+00_JPRB, 0.83105E+00_JPRB, 0.80389E+00_JPRB, 0.78358E+00_JPRB, 0.74294E+00_JPRB ,&
&  0.70264E+00_JPRB, 0.67983E+00_JPRB, 0.69470E+00_JPRB, 0.68919E+00_JPRB, 0.72159E+00_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.65330E+00_JPRB, 0.63771E+00_JPRB, 0.66996E+00_JPRB, 0.69363E+00_JPRB, 0.72103E+00_JPRB, 0.74026E+00_JPRB, 0.77337E+00_JPRB ,&
&  0.83873E+00_JPRB, 0.89503E+00_JPRB, 0.93382E+00_JPRB, 0.92847E+00_JPRB, 0.91131E+00_JPRB, 0.95342E+00_JPRB, 0.10405E+01_JPRB ,&
&  0.11602E+01_JPRB, 0.13223E+01_JPRB, 0.15117E+01_JPRB, 0.16401E+01_JPRB, 0.16952E+01_JPRB, 0.15483E+01_JPRB, 0.14048E+01_JPRB ,&
&  0.12794E+01_JPRB, 0.13048E+01_JPRB, 0.13334E+01_JPRB, 0.13965E+01_JPRB, 0.14033E+01_JPRB, 0.13139E+01_JPRB, 0.12267E+01_JPRB ,&
&  0.11855E+01_JPRB, 0.11726E+01_JPRB, 0.11453E+01_JPRB, 0.11570E+01_JPRB, 0.11715E+01_JPRB, 0.11740E+01_JPRB, 0.11320E+01_JPRB ,&
&  0.10207E+01_JPRB, 0.96783E+00_JPRB, 0.94562E+00_JPRB, 0.10251E+01_JPRB, 0.12359E+01_JPRB, 0.14719E+01_JPRB, 0.17012E+01_JPRB ,&
&  0.19404E+01_JPRB, 0.21799E+01_JPRB, 0.24858E+01_JPRB, 0.26912E+01_JPRB, 0.30227E+01_JPRB, 0.33403E+01_JPRB, 0.35888E+01_JPRB ,&
&  0.36652E+01_JPRB, 0.39643E+01_JPRB, 0.42545E+01_JPRB, 0.44784E+01_JPRB, 0.46475E+01_JPRB, 0.44496E+01_JPRB, 0.41564E+01_JPRB ,&
&  0.46236E+01_JPRB, 0.56871E+01_JPRB, 0.70952E+01_JPRB, 0.77352E+01_JPRB, 0.81480E+01_JPRB, 0.99180E+01_JPRB, 0.97631E+01_JPRB ,&
&  0.76025E+01_JPRB, 0.55532E+01_JPRB, 0.47220E+01_JPRB, 0.45389E+01_JPRB, 0.46317E+01_JPRB, 0.48854E+01_JPRB, 0.62762E+01_JPRB ,&
&  0.58525E+01_JPRB, 0.27563E+01_JPRB, 0.17541E+01_JPRB, 0.18410E+01_JPRB, 0.10589E+01_JPRB, 0.93224E+00_JPRB, 0.15141E+01_JPRB ,&
&  0.20685E+01_JPRB, 0.30478E+01_JPRB, 0.35480E+01_JPRB, 0.28983E+01_JPRB, 0.28612E+01_JPRB, 0.24891E+01_JPRB, 0.21320E+01_JPRB ,&
&  0.14250E+01_JPRB, 0.10675E+01_JPRB, 0.84318E+00_JPRB, 0.77773E+00_JPRB, 0.74401E+00_JPRB, 0.75153E+00_JPRB, 0.75656E+00_JPRB ,&
&  0.70773E+00_JPRB, 0.64866E+00_JPRB, 0.65084E+00_JPRB, 0.65847E+00_JPRB, 0.65161E+00_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.69094E+00_JPRB, 0.71280E+00_JPRB, 0.77277E+00_JPRB, 0.82673E+00_JPRB, 0.85618E+00_JPRB, 0.87147E+00_JPRB, 0.87840E+00_JPRB ,&
&  0.92607E+00_JPRB, 0.99626E+00_JPRB, 0.10379E+01_JPRB, 0.10649E+01_JPRB, 0.11079E+01_JPRB, 0.11882E+01_JPRB, 0.12842E+01_JPRB ,&
&  0.13949E+01_JPRB, 0.15164E+01_JPRB, 0.16426E+01_JPRB, 0.17745E+01_JPRB, 0.19462E+01_JPRB, 0.20213E+01_JPRB, 0.12661E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.13635E+01_JPRB, 0.12907E+01_JPRB, 0.13678E+01_JPRB, 0.14074E+01_JPRB, 0.14198E+01_JPRB, 0.12627E+01_JPRB ,&
&  0.11792E+01_JPRB, 0.11528E+01_JPRB, 0.11419E+01_JPRB, 0.11820E+01_JPRB, 0.12531E+01_JPRB, 0.13300E+01_JPRB, 0.13172E+01_JPRB ,&
&  0.13035E+01_JPRB, 0.13338E+01_JPRB, 0.13913E+01_JPRB, 0.15924E+01_JPRB, 0.18781E+01_JPRB, 0.21099E+01_JPRB, 0.22749E+01_JPRB ,&
&  0.24108E+01_JPRB, 0.25710E+01_JPRB, 0.27980E+01_JPRB, 0.29608E+01_JPRB, 0.32945E+01_JPRB, 0.34661E+01_JPRB, 0.36185E+01_JPRB ,&
&  0.35686E+01_JPRB, 0.34596E+01_JPRB, 0.38822E+01_JPRB, 0.42018E+01_JPRB, 0.42633E+01_JPRB, 0.41392E+01_JPRB, 0.42846E+01_JPRB ,&
&  0.55777E+01_JPRB, 0.71849E+01_JPRB, 0.89634E+01_JPRB, 0.80264E+01_JPRB, 0.76906E+01_JPRB, 0.85513E+01_JPRB, 0.92574E+01_JPRB ,&
&  0.76038E+01_JPRB, 0.58510E+01_JPRB, 0.53023E+01_JPRB, 0.48343E+01_JPRB, 0.48553E+01_JPRB, 0.48492E+01_JPRB, 0.54903E+01_JPRB ,&
&  0.57240E+01_JPRB, 0.43220E+01_JPRB, 0.24995E+01_JPRB, 0.13959E+01_JPRB, 0.10111E+01_JPRB, 0.10658E+01_JPRB, 0.13580E+01_JPRB ,&
&  0.21606E+01_JPRB, 0.27216E+01_JPRB, 0.31703E+01_JPRB, 0.33118E+01_JPRB, 0.20939E+01_JPRB, 0.16130E+01_JPRB, 0.12991E+01_JPRB ,&
&  0.89041E+00_JPRB, 0.74578E+00_JPRB, 0.73479E+00_JPRB, 0.72296E+00_JPRB, 0.69708E+00_JPRB, 0.72563E+00_JPRB, 0.77033E+00_JPRB ,&
&  0.72602E+00_JPRB, 0.64789E+00_JPRB, 0.65077E+00_JPRB, 0.65406E+00_JPRB, 0.66473E+00_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.75459E+00_JPRB, 0.79920E+00_JPRB, 0.86212E+00_JPRB, 0.95466E+00_JPRB, 0.10501E+01_JPRB, 0.11204E+01_JPRB, 0.10725E+01_JPRB ,&
&  0.10280E+01_JPRB, 0.10716E+01_JPRB, 0.11277E+01_JPRB, 0.11898E+01_JPRB, 0.12582E+01_JPRB, 0.13361E+01_JPRB, 0.14135E+01_JPRB ,&
&  0.14959E+01_JPRB, 0.15832E+01_JPRB, 0.16733E+01_JPRB, 0.17929E+01_JPRB, 0.19305E+01_JPRB, 0.19998E+01_JPRB, 0.14823E+01_JPRB ,&
&  0.11101E+01_JPRB, 0.17049E+01_JPRB, 0.13338E+01_JPRB, 0.14192E+01_JPRB, 0.14674E+01_JPRB, 0.15810E+01_JPRB, 0.13663E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.11659E+01_JPRB, 0.11911E+01_JPRB, 0.11957E+01_JPRB, 0.12615E+01_JPRB, 0.14083E+01_JPRB, 0.15718E+01_JPRB ,&
&  0.16505E+01_JPRB, 0.17649E+01_JPRB, 0.19358E+01_JPRB, 0.21789E+01_JPRB, 0.23704E+01_JPRB, 0.25296E+01_JPRB, 0.26545E+01_JPRB ,&
&  0.27739E+01_JPRB, 0.29368E+01_JPRB, 0.30650E+01_JPRB, 0.32747E+01_JPRB, 0.33832E+01_JPRB, 0.35722E+01_JPRB, 0.37138E+01_JPRB ,&
&  0.36402E+01_JPRB, 0.35202E+01_JPRB, 0.37152E+01_JPRB, 0.37246E+01_JPRB, 0.38786E+01_JPRB, 0.45858E+01_JPRB, 0.50767E+01_JPRB ,&
&  0.65738E+01_JPRB, 0.73688E+01_JPRB, 0.88934E+01_JPRB, 0.71215E+01_JPRB, 0.64166E+01_JPRB, 0.66855E+01_JPRB, 0.61469E+01_JPRB ,&
&  0.51167E+01_JPRB, 0.49231E+01_JPRB, 0.47214E+01_JPRB, 0.47014E+01_JPRB, 0.47361E+01_JPRB, 0.47229E+01_JPRB, 0.47109E+01_JPRB ,&
&  0.37379E+01_JPRB, 0.20289E+01_JPRB, 0.14038E+01_JPRB, 0.10679E+01_JPRB, 0.89324E+00_JPRB, 0.99472E+00_JPRB, 0.15417E+01_JPRB ,&
&  0.24492E+01_JPRB, 0.28041E+01_JPRB, 0.30759E+01_JPRB, 0.22784E+01_JPRB, 0.13225E+01_JPRB, 0.98363E+00_JPRB, 0.77311E+00_JPRB ,&
&  0.60751E+00_JPRB, 0.58763E+00_JPRB, 0.63706E+00_JPRB, 0.65326E+00_JPRB, 0.64024E+00_JPRB, 0.66301E+00_JPRB, 0.68477E+00_JPRB ,&
&  0.67072E+00_JPRB, 0.66372E+00_JPRB, 0.69886E+00_JPRB, 0.69720E+00_JPRB, 0.71908E+00_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.81590E+00_JPRB, 0.87533E+00_JPRB, 0.98101E+00_JPRB, 0.11239E+01_JPRB, 0.13187E+01_JPRB, 0.15061E+01_JPRB, 0.15538E+01_JPRB ,&
&  0.13152E+01_JPRB, 0.10878E+01_JPRB, 0.11844E+01_JPRB, 0.12788E+01_JPRB, 0.13604E+01_JPRB, 0.14514E+01_JPRB, 0.15353E+01_JPRB ,&
&  0.16424E+01_JPRB, 0.17539E+01_JPRB, 0.18747E+01_JPRB, 0.20242E+01_JPRB, 0.21882E+01_JPRB, 0.23377E+01_JPRB, 0.22165E+01_JPRB ,&
&  0.16086E+01_JPRB, 0.13455E+01_JPRB, 0.15842E+01_JPRB, 0.14634E+01_JPRB, 0.14011E+01_JPRB, 0.15076E+01_JPRB, 0.15130E+01_JPRB ,&
&  0.13471E+01_JPRB, 0.12295E+01_JPRB, 0.12720E+01_JPRB, 0.12473E+01_JPRB, 0.11189E+01_JPRB, 0.12984E+01_JPRB, 0.15781E+01_JPRB ,&
&  0.17520E+01_JPRB, 0.19134E+01_JPRB, 0.21205E+01_JPRB, 0.23515E+01_JPRB, 0.25234E+01_JPRB, 0.27315E+01_JPRB, 0.29149E+01_JPRB ,&
&  0.30600E+01_JPRB, 0.32259E+01_JPRB, 0.33015E+01_JPRB, 0.33263E+01_JPRB, 0.34655E+01_JPRB, 0.36368E+01_JPRB, 0.37090E+01_JPRB ,&
&  0.37239E+01_JPRB, 0.38944E+01_JPRB, 0.40731E+01_JPRB, 0.42437E+01_JPRB, 0.43581E+01_JPRB, 0.52018E+01_JPRB, 0.58000E+01_JPRB ,&
&  0.65099E+01_JPRB, 0.66249E+01_JPRB, 0.72418E+01_JPRB, 0.56337E+01_JPRB, 0.50372E+01_JPRB, 0.46772E+01_JPRB, 0.43428E+01_JPRB ,&
&  0.40516E+01_JPRB, 0.41417E+01_JPRB, 0.40885E+01_JPRB, 0.42081E+01_JPRB, 0.44268E+01_JPRB, 0.41392E+01_JPRB, 0.39104E+01_JPRB ,&
&  0.23252E+01_JPRB, 0.15334E+01_JPRB, 0.10643E+01_JPRB, 0.98262E+00_JPRB, 0.93911E+00_JPRB, 0.10945E+01_JPRB, 0.14604E+01_JPRB ,&
&  0.20176E+01_JPRB, 0.18213E+01_JPRB, 0.16726E+01_JPRB, 0.15083E+01_JPRB, 0.92311E+00_JPRB, 0.68327E+00_JPRB, 0.57750E+00_JPRB ,&
&  0.53561E+00_JPRB, 0.55029E+00_JPRB, 0.58317E+00_JPRB, 0.58998E+00_JPRB, 0.57647E+00_JPRB, 0.59759E+00_JPRB, 0.63615E+00_JPRB ,&
&  0.65788E+00_JPRB, 0.67989E+00_JPRB, 0.72286E+00_JPRB, 0.73697E+00_JPRB, 0.78054E+00_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.78730E+00_JPRB, 0.87351E+00_JPRB, 0.94000E+00_JPRB, 0.10049E+01_JPRB, 0.10407E+01_JPRB, 0.11031E+01_JPRB, 0.10978E+01_JPRB ,&
&  0.10281E+01_JPRB, 0.97977E+00_JPRB, 0.10569E+01_JPRB, 0.11558E+01_JPRB, 0.12445E+01_JPRB, 0.12993E+01_JPRB, 0.13571E+01_JPRB ,&
&  0.14253E+01_JPRB, 0.14948E+01_JPRB, 0.15739E+01_JPRB, 0.16490E+01_JPRB, 0.17581E+01_JPRB, 0.18835E+01_JPRB, 0.19587E+01_JPRB ,&
&  0.16715E+01_JPRB, 0.11946E+01_JPRB, 0.14337E+01_JPRB, 0.15350E+01_JPRB, 0.13583E+01_JPRB, 0.14037E+01_JPRB, 0.14905E+01_JPRB ,&
&  0.13135E+01_JPRB, 0.12324E+01_JPRB, 0.12591E+01_JPRB, 0.12689E+01_JPRB, 0.10518E+01_JPRB, 0.11726E+01_JPRB, 0.14147E+01_JPRB ,&
&  0.16389E+01_JPRB, 0.18595E+01_JPRB, 0.20824E+01_JPRB, 0.22772E+01_JPRB, 0.24762E+01_JPRB, 0.27268E+01_JPRB, 0.29563E+01_JPRB ,&
&  0.31475E+01_JPRB, 0.32893E+01_JPRB, 0.33922E+01_JPRB, 0.34930E+01_JPRB, 0.36014E+01_JPRB, 0.37747E+01_JPRB, 0.38317E+01_JPRB ,&
&  0.39477E+01_JPRB, 0.40284E+01_JPRB, 0.44322E+01_JPRB, 0.49318E+01_JPRB, 0.53089E+01_JPRB, 0.52090E+01_JPRB, 0.55207E+01_JPRB ,&
&  0.55650E+01_JPRB, 0.53160E+01_JPRB, 0.53431E+01_JPRB, 0.52662E+01_JPRB, 0.35107E+01_JPRB, 0.32158E+01_JPRB, 0.32091E+01_JPRB ,&
&  0.36018E+01_JPRB, 0.35070E+01_JPRB, 0.34699E+01_JPRB, 0.35321E+01_JPRB, 0.34741E+01_JPRB, 0.32518E+01_JPRB, 0.27663E+01_JPRB ,&
&  0.19101E+01_JPRB, 0.12931E+01_JPRB, 0.10388E+01_JPRB, 0.10372E+01_JPRB, 0.10874E+01_JPRB, 0.14394E+01_JPRB, 0.12803E+01_JPRB ,&
&  0.15212E+01_JPRB, 0.12432E+01_JPRB, 0.11656E+01_JPRB, 0.12890E+01_JPRB, 0.92916E+00_JPRB, 0.67341E+00_JPRB, 0.57079E+00_JPRB ,&
&  0.53516E+00_JPRB, 0.54854E+00_JPRB, 0.55284E+00_JPRB, 0.54546E+00_JPRB, 0.54021E+00_JPRB, 0.56579E+00_JPRB, 0.59709E+00_JPRB ,&
&  0.61774E+00_JPRB, 0.64654E+00_JPRB, 0.67811E+00_JPRB, 0.71306E+00_JPRB, 0.76073E+00_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.64872E+00_JPRB, 0.67202E+00_JPRB, 0.70767E+00_JPRB, 0.75687E+00_JPRB, 0.79078E+00_JPRB, 0.81724E+00_JPRB, 0.81932E+00_JPRB ,&
&  0.84588E+00_JPRB, 0.87444E+00_JPRB, 0.91918E+00_JPRB, 0.95753E+00_JPRB, 0.98109E+00_JPRB, 0.10379E+01_JPRB, 0.10796E+01_JPRB ,&
&  0.10920E+01_JPRB, 0.10887E+01_JPRB, 0.11400E+01_JPRB, 0.12250E+01_JPRB, 0.13853E+01_JPRB, 0.16405E+01_JPRB, 0.19052E+01_JPRB ,&
&  0.21468E+01_JPRB, 0.21230E+01_JPRB, 0.20328E+01_JPRB, 0.13414E+01_JPRB, 0.97041E+00_JPRB, 0.11135E+01_JPRB, 0.11854E+01_JPRB ,&
&  0.12480E+01_JPRB, 0.12690E+01_JPRB, 0.11669E+01_JPRB, 0.11031E+01_JPRB, 0.10740E+01_JPRB, 0.11103E+01_JPRB, 0.12355E+01_JPRB ,&
&  0.14039E+01_JPRB, 0.16026E+01_JPRB, 0.18306E+01_JPRB, 0.20157E+01_JPRB, 0.22153E+01_JPRB, 0.24698E+01_JPRB, 0.27398E+01_JPRB ,&
&  0.29670E+01_JPRB, 0.31600E+01_JPRB, 0.33155E+01_JPRB, 0.35599E+01_JPRB, 0.37215E+01_JPRB, 0.39518E+01_JPRB, 0.41178E+01_JPRB ,&
&  0.43058E+01_JPRB, 0.43868E+01_JPRB, 0.47250E+01_JPRB, 0.51043E+01_JPRB, 0.51525E+01_JPRB, 0.46689E+01_JPRB, 0.45006E+01_JPRB ,&
&  0.44567E+01_JPRB, 0.41234E+01_JPRB, 0.39973E+01_JPRB, 0.41286E+01_JPRB, 0.27096E+01_JPRB, 0.27416E+01_JPRB, 0.31032E+01_JPRB ,&
&  0.30298E+01_JPRB, 0.29881E+01_JPRB, 0.27583E+01_JPRB, 0.23787E+01_JPRB, 0.22330E+01_JPRB, 0.21181E+01_JPRB, 0.19450E+01_JPRB ,&
&  0.16600E+01_JPRB, 0.12606E+01_JPRB, 0.10278E+01_JPRB, 0.88432E+00_JPRB, 0.10207E+01_JPRB, 0.14748E+01_JPRB, 0.11396E+01_JPRB ,&
&  0.10064E+01_JPRB, 0.11097E+01_JPRB, 0.10163E+01_JPRB, 0.13840E+01_JPRB, 0.12782E+01_JPRB, 0.83632E+00_JPRB, 0.63355E+00_JPRB ,&
&  0.53846E+00_JPRB, 0.52742E+00_JPRB, 0.53667E+00_JPRB, 0.52788E+00_JPRB, 0.53406E+00_JPRB, 0.55923E+00_JPRB, 0.56367E+00_JPRB ,&
&  0.59930E+00_JPRB, 0.61313E+00_JPRB, 0.63205E+00_JPRB, 0.64908E+00_JPRB, 0.63800E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.51220E+00_JPRB, 0.51243E+00_JPRB, 0.52327E+00_JPRB, 0.56760E+00_JPRB, 0.59172E+00_JPRB, 0.60560E+00_JPRB, 0.63751E+00_JPRB ,&
&  0.67625E+00_JPRB, 0.74193E+00_JPRB, 0.78920E+00_JPRB, 0.80676E+00_JPRB, 0.77977E+00_JPRB, 0.75288E+00_JPRB, 0.74363E+00_JPRB ,&
&  0.74400E+00_JPRB, 0.69448E+00_JPRB, 0.66478E+00_JPRB, 0.69183E+00_JPRB, 0.75740E+00_JPRB, 0.87841E+00_JPRB, 0.10231E+01_JPRB ,&
&  0.12218E+01_JPRB, 0.14236E+01_JPRB, 0.16649E+01_JPRB, 0.22469E+01_JPRB, 0.23901E+01_JPRB, 0.12049E+01_JPRB, 0.12567E+01_JPRB ,&
&  0.16090E+01_JPRB, 0.16475E+01_JPRB, 0.12922E+01_JPRB, 0.11516E+01_JPRB, 0.10871E+01_JPRB, 0.10574E+01_JPRB, 0.10918E+01_JPRB ,&
&  0.12034E+01_JPRB, 0.13236E+01_JPRB, 0.14908E+01_JPRB, 0.17042E+01_JPRB, 0.19290E+01_JPRB, 0.21335E+01_JPRB, 0.23825E+01_JPRB ,&
&  0.25986E+01_JPRB, 0.27202E+01_JPRB, 0.28776E+01_JPRB, 0.30998E+01_JPRB, 0.34070E+01_JPRB, 0.37938E+01_JPRB, 0.41463E+01_JPRB ,&
&  0.44276E+01_JPRB, 0.45626E+01_JPRB, 0.48852E+01_JPRB, 0.50758E+01_JPRB, 0.46467E+01_JPRB, 0.41981E+01_JPRB, 0.38024E+01_JPRB ,&
&  0.34350E+01_JPRB, 0.29334E+01_JPRB, 0.25583E+01_JPRB, 0.30532E+01_JPRB, 0.31442E+01_JPRB, 0.27324E+01_JPRB, 0.25931E+01_JPRB ,&
&  0.26149E+01_JPRB, 0.25329E+01_JPRB, 0.22655E+01_JPRB, 0.18020E+01_JPRB, 0.14992E+01_JPRB, 0.16100E+01_JPRB, 0.15359E+01_JPRB ,&
&  0.13903E+01_JPRB, 0.10976E+01_JPRB, 0.92671E+00_JPRB, 0.83989E+00_JPRB, 0.81743E+00_JPRB, 0.11010E+01_JPRB, 0.87918E+00_JPRB ,&
&  0.72297E+00_JPRB, 0.91065E+00_JPRB, 0.86577E+00_JPRB, 0.12973E+01_JPRB, 0.21841E+01_JPRB, 0.11295E+01_JPRB, 0.69279E+00_JPRB ,&
&  0.53310E+00_JPRB, 0.52649E+00_JPRB, 0.54031E+00_JPRB, 0.54379E+00_JPRB, 0.54185E+00_JPRB, 0.55431E+00_JPRB, 0.55578E+00_JPRB ,&
&  0.56383E+00_JPRB, 0.55648E+00_JPRB, 0.55867E+00_JPRB, 0.54035E+00_JPRB, 0.50934E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.57922E+00_JPRB, 0.57926E+00_JPRB, 0.56734E+00_JPRB, 0.55974E+00_JPRB, 0.56108E+00_JPRB, 0.56629E+00_JPRB, 0.57589E+00_JPRB ,&
&  0.59255E+00_JPRB, 0.61696E+00_JPRB, 0.64158E+00_JPRB, 0.67540E+00_JPRB, 0.68455E+00_JPRB, 0.65625E+00_JPRB, 0.66415E+00_JPRB ,&
&  0.66239E+00_JPRB, 0.67793E+00_JPRB, 0.67214E+00_JPRB, 0.75245E+00_JPRB, 0.77830E+00_JPRB, 0.78911E+00_JPRB, 0.88375E+00_JPRB ,&
&  0.87520E+00_JPRB, 0.97021E+00_JPRB, 0.10438E+01_JPRB, 0.11704E+01_JPRB, 0.19267E+01_JPRB, 0.17707E+01_JPRB, 0.12519E+01_JPRB ,&
&  0.13863E+01_JPRB, 0.14223E+01_JPRB, 0.12353E+01_JPRB, 0.10779E+01_JPRB, 0.10299E+01_JPRB, 0.95691E+00_JPRB, 0.93739E+00_JPRB ,&
&  0.99609E+00_JPRB, 0.10831E+01_JPRB, 0.12525E+01_JPRB, 0.14239E+01_JPRB, 0.15681E+01_JPRB, 0.17044E+01_JPRB, 0.18606E+01_JPRB ,&
&  0.19744E+01_JPRB, 0.20896E+01_JPRB, 0.21453E+01_JPRB, 0.22218E+01_JPRB, 0.25994E+01_JPRB, 0.29774E+01_JPRB, 0.32756E+01_JPRB ,&
&  0.35108E+01_JPRB, 0.37348E+01_JPRB, 0.40991E+01_JPRB, 0.39504E+01_JPRB, 0.35780E+01_JPRB, 0.33531E+01_JPRB, 0.28083E+01_JPRB ,&
&  0.22748E+01_JPRB, 0.17648E+01_JPRB, 0.15590E+01_JPRB, 0.19135E+01_JPRB, 0.19956E+01_JPRB, 0.17539E+01_JPRB, 0.20115E+01_JPRB ,&
&  0.20762E+01_JPRB, 0.20975E+01_JPRB, 0.18249E+01_JPRB, 0.16124E+01_JPRB, 0.13704E+01_JPRB, 0.13392E+01_JPRB, 0.12866E+01_JPRB ,&
&  0.11128E+01_JPRB, 0.90556E+00_JPRB, 0.79572E+00_JPRB, 0.73082E+00_JPRB, 0.69885E+00_JPRB, 0.83050E+00_JPRB, 0.89783E+00_JPRB ,&
&  0.91271E+00_JPRB, 0.81226E+00_JPRB, 0.82139E+00_JPRB, 0.12157E+01_JPRB, 0.11487E+01_JPRB, 0.71099E+00_JPRB, 0.53916E+00_JPRB ,&
&  0.51272E+00_JPRB, 0.52161E+00_JPRB, 0.52485E+00_JPRB, 0.52584E+00_JPRB, 0.53793E+00_JPRB, 0.55144E+00_JPRB, 0.54134E+00_JPRB ,&
&  0.54225E+00_JPRB, 0.53577E+00_JPRB, 0.53147E+00_JPRB, 0.53554E+00_JPRB, 0.55064E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.72085E+00_JPRB, 0.72508E+00_JPRB, 0.73045E+00_JPRB, 0.73901E+00_JPRB, 0.75617E+00_JPRB, 0.77469E+00_JPRB, 0.78754E+00_JPRB ,&
&  0.81578E+00_JPRB, 0.83917E+00_JPRB, 0.86407E+00_JPRB, 0.87483E+00_JPRB, 0.89272E+00_JPRB, 0.90845E+00_JPRB, 0.91487E+00_JPRB ,&
&  0.93628E+00_JPRB, 0.96152E+00_JPRB, 0.96276E+00_JPRB, 0.97352E+00_JPRB, 0.10018E+01_JPRB, 0.10206E+01_JPRB, 0.10028E+01_JPRB ,&
&  0.98068E+00_JPRB, 0.95404E+00_JPRB, 0.97469E+00_JPRB, 0.10137E+01_JPRB, 0.12088E+01_JPRB, 0.13321E+01_JPRB, 0.13177E+01_JPRB ,&
&  0.12188E+01_JPRB, 0.12447E+01_JPRB, 0.13675E+01_JPRB, 0.11243E+01_JPRB, 0.10368E+01_JPRB, 0.99386E+00_JPRB, 0.94759E+00_JPRB ,&
&  0.96500E+00_JPRB, 0.10089E+01_JPRB, 0.10593E+01_JPRB, 0.11163E+01_JPRB, 0.11921E+01_JPRB, 0.12254E+01_JPRB, 0.12850E+01_JPRB ,&
&  0.12999E+01_JPRB, 0.13427E+01_JPRB, 0.14073E+01_JPRB, 0.15443E+01_JPRB, 0.18115E+01_JPRB, 0.21736E+01_JPRB, 0.22278E+01_JPRB ,&
&  0.22641E+01_JPRB, 0.24703E+01_JPRB, 0.22957E+01_JPRB, 0.22003E+01_JPRB, 0.21225E+01_JPRB, 0.18206E+01_JPRB, 0.15227E+01_JPRB ,&
&  0.14772E+01_JPRB, 0.13504E+01_JPRB, 0.10243E+01_JPRB, 0.12750E+01_JPRB, 0.14425E+01_JPRB, 0.14871E+01_JPRB, 0.16526E+01_JPRB ,&
&  0.17288E+01_JPRB, 0.15437E+01_JPRB, 0.13072E+01_JPRB, 0.12906E+01_JPRB, 0.12789E+01_JPRB, 0.10995E+01_JPRB, 0.10481E+01_JPRB ,&
&  0.90543E+00_JPRB, 0.80615E+00_JPRB, 0.75305E+00_JPRB, 0.74975E+00_JPRB, 0.81177E+00_JPRB, 0.11081E+01_JPRB, 0.11844E+01_JPRB ,&
&  0.99984E+00_JPRB, 0.83783E+00_JPRB, 0.94860E+00_JPRB, 0.14436E+01_JPRB, 0.84162E+00_JPRB, 0.64805E+00_JPRB, 0.57051E+00_JPRB ,&
&  0.58335E+00_JPRB, 0.58785E+00_JPRB, 0.59092E+00_JPRB, 0.59352E+00_JPRB, 0.59808E+00_JPRB, 0.61518E+00_JPRB, 0.60979E+00_JPRB ,&
&  0.60630E+00_JPRB, 0.60558E+00_JPRB, 0.62340E+00_JPRB, 0.64736E+00_JPRB, 0.68936E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.81615E+00_JPRB, 0.80916E+00_JPRB, 0.82046E+00_JPRB, 0.83708E+00_JPRB, 0.85229E+00_JPRB, 0.86945E+00_JPRB, 0.89863E+00_JPRB ,&
&  0.93420E+00_JPRB, 0.96354E+00_JPRB, 0.99278E+00_JPRB, 0.10141E+01_JPRB, 0.10218E+01_JPRB, 0.10357E+01_JPRB, 0.10489E+01_JPRB ,&
&  0.10635E+01_JPRB, 0.10586E+01_JPRB, 0.10775E+01_JPRB, 0.11092E+01_JPRB, 0.11452E+01_JPRB, 0.11600E+01_JPRB, 0.11730E+01_JPRB ,&
&  0.11887E+01_JPRB, 0.12296E+01_JPRB, 0.12776E+01_JPRB, 0.13356E+01_JPRB, 0.14333E+01_JPRB, 0.16049E+01_JPRB, 0.17772E+01_JPRB ,&
&  0.13189E+01_JPRB, 0.13934E+01_JPRB, 0.13654E+01_JPRB, 0.11543E+01_JPRB, 0.10872E+01_JPRB, 0.11308E+01_JPRB, 0.11141E+01_JPRB ,&
&  0.10971E+01_JPRB, 0.11300E+01_JPRB, 0.11585E+01_JPRB, 0.11886E+01_JPRB, 0.12223E+01_JPRB, 0.12703E+01_JPRB, 0.12741E+01_JPRB ,&
&  0.13074E+01_JPRB, 0.13328E+01_JPRB, 0.13597E+01_JPRB, 0.14317E+01_JPRB, 0.14213E+01_JPRB, 0.17675E+01_JPRB, 0.17198E+01_JPRB ,&
&  0.16202E+01_JPRB, 0.16317E+01_JPRB, 0.12598E+01_JPRB, 0.12788E+01_JPRB, 0.12878E+01_JPRB, 0.11152E+01_JPRB, 0.10040E+01_JPRB ,&
&  0.12886E+01_JPRB, 0.14044E+01_JPRB, 0.10863E+01_JPRB, 0.12573E+01_JPRB, 0.13388E+01_JPRB, 0.13763E+01_JPRB, 0.13891E+01_JPRB ,&
&  0.13564E+01_JPRB, 0.11967E+01_JPRB, 0.10634E+01_JPRB, 0.99093E+00_JPRB, 0.95939E+00_JPRB, 0.91729E+00_JPRB, 0.86102E+00_JPRB ,&
&  0.79649E+00_JPRB, 0.75906E+00_JPRB, 0.71735E+00_JPRB, 0.69662E+00_JPRB, 0.93787E+00_JPRB, 0.10230E+01_JPRB, 0.15402E+01_JPRB ,&
&  0.93735E+00_JPRB, 0.81031E+00_JPRB, 0.67781E+00_JPRB, 0.98087E+00_JPRB, 0.79159E+00_JPRB, 0.62586E+00_JPRB, 0.67723E+00_JPRB ,&
&  0.73145E+00_JPRB, 0.76664E+00_JPRB, 0.75304E+00_JPRB, 0.72427E+00_JPRB, 0.71299E+00_JPRB, 0.71404E+00_JPRB, 0.69298E+00_JPRB ,&
&  0.68532E+00_JPRB, 0.70420E+00_JPRB, 0.73862E+00_JPRB, 0.75159E+00_JPRB, 0.78508E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.86014E+00_JPRB, 0.87779E+00_JPRB, 0.89736E+00_JPRB, 0.92046E+00_JPRB, 0.94273E+00_JPRB, 0.96325E+00_JPRB, 0.98709E+00_JPRB ,&
&  0.10177E+01_JPRB, 0.10410E+01_JPRB, 0.10645E+01_JPRB, 0.10788E+01_JPRB, 0.10860E+01_JPRB, 0.11005E+01_JPRB, 0.11055E+01_JPRB ,&
&  0.11086E+01_JPRB, 0.11113E+01_JPRB, 0.11294E+01_JPRB, 0.11412E+01_JPRB, 0.11435E+01_JPRB, 0.11425E+01_JPRB, 0.11595E+01_JPRB ,&
&  0.11625E+01_JPRB, 0.11969E+01_JPRB, 0.12255E+01_JPRB, 0.12618E+01_JPRB, 0.12930E+01_JPRB, 0.13052E+01_JPRB, 0.13879E+01_JPRB ,&
&  0.10961E+01_JPRB, 0.16273E+01_JPRB, 0.16266E+01_JPRB, 0.13336E+01_JPRB, 0.12814E+01_JPRB, 0.12226E+01_JPRB, 0.12218E+01_JPRB ,&
&  0.12393E+01_JPRB, 0.12643E+01_JPRB, 0.13048E+01_JPRB, 0.13672E+01_JPRB, 0.14247E+01_JPRB, 0.14777E+01_JPRB, 0.15035E+01_JPRB ,&
&  0.15447E+01_JPRB, 0.15862E+01_JPRB, 0.16297E+01_JPRB, 0.16607E+01_JPRB, 0.17126E+01_JPRB, 0.17477E+01_JPRB, 0.17103E+01_JPRB ,&
&  0.15914E+01_JPRB, 0.14837E+01_JPRB, 0.10842E+01_JPRB, 0.10517E+01_JPRB, 0.11179E+01_JPRB, 0.10615E+01_JPRB, 0.95058E+00_JPRB ,&
&  0.12264E+01_JPRB, 0.13453E+01_JPRB, 0.12778E+01_JPRB, 0.12592E+01_JPRB, 0.12339E+01_JPRB, 0.11514E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.10055E+01_JPRB, 0.87990E+00_JPRB, 0.80629E+00_JPRB, 0.76118E+00_JPRB, 0.76487E+00_JPRB, 0.75172E+00_JPRB, 0.71923E+00_JPRB ,&
&  0.67112E+00_JPRB, 0.66128E+00_JPRB, 0.67703E+00_JPRB, 0.69479E+00_JPRB, 0.73453E+00_JPRB, 0.13113E+01_JPRB, 0.15791E+01_JPRB ,&
&  0.81840E+00_JPRB, 0.72447E+00_JPRB, 0.59349E+00_JPRB, 0.76731E+00_JPRB, 0.79817E+00_JPRB, 0.80776E+00_JPRB, 0.92284E+00_JPRB ,&
&  0.10183E+01_JPRB, 0.10313E+01_JPRB, 0.94031E+00_JPRB, 0.83282E+00_JPRB, 0.79115E+00_JPRB, 0.77658E+00_JPRB, 0.75421E+00_JPRB ,&
&  0.75471E+00_JPRB, 0.77996E+00_JPRB, 0.80361E+00_JPRB, 0.82180E+00_JPRB, 0.84105E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.90699E+00_JPRB, 0.92284E+00_JPRB, 0.93869E+00_JPRB, 0.96574E+00_JPRB, 0.99532E+00_JPRB, 0.10232E+01_JPRB, 0.10459E+01_JPRB ,&
&  0.10694E+01_JPRB, 0.10869E+01_JPRB, 0.11080E+01_JPRB, 0.11262E+01_JPRB, 0.11395E+01_JPRB, 0.11478E+01_JPRB, 0.11492E+01_JPRB ,&
&  0.11526E+01_JPRB, 0.11579E+01_JPRB, 0.11725E+01_JPRB, 0.11844E+01_JPRB, 0.11847E+01_JPRB, 0.11941E+01_JPRB, 0.12137E+01_JPRB ,&
&  0.12269E+01_JPRB, 0.12402E+01_JPRB, 0.12739E+01_JPRB, 0.13214E+01_JPRB, 0.13866E+01_JPRB, 0.15203E+01_JPRB, 0.13898E+01_JPRB ,&
&  0.15282E+01_JPRB, 0.17315E+01_JPRB, 0.17569E+01_JPRB, 0.14547E+01_JPRB, 0.13382E+01_JPRB, 0.13121E+01_JPRB, 0.13218E+01_JPRB ,&
&  0.13432E+01_JPRB, 0.13613E+01_JPRB, 0.13945E+01_JPRB, 0.14531E+01_JPRB, 0.15415E+01_JPRB, 0.16313E+01_JPRB, 0.16794E+01_JPRB ,&
&  0.17448E+01_JPRB, 0.18221E+01_JPRB, 0.19047E+01_JPRB, 0.19827E+01_JPRB, 0.21057E+01_JPRB, 0.21742E+01_JPRB, 0.21475E+01_JPRB ,&
&  0.20469E+01_JPRB, 0.19745E+01_JPRB, 0.16886E+01_JPRB, 0.16138E+01_JPRB, 0.16678E+01_JPRB, 0.14702E+01_JPRB, 0.13053E+01_JPRB ,&
&  0.14233E+01_JPRB, 0.12371E+01_JPRB, 0.10947E+01_JPRB, 0.12047E+01_JPRB, 0.11393E+01_JPRB, 0.10017E+01_JPRB, 0.94727E+00_JPRB ,&
&  0.81645E+00_JPRB, 0.70410E+00_JPRB, 0.63517E+00_JPRB, 0.62337E+00_JPRB, 0.63434E+00_JPRB, 0.66462E+00_JPRB, 0.66203E+00_JPRB ,&
&  0.67286E+00_JPRB, 0.70418E+00_JPRB, 0.73209E+00_JPRB, 0.77058E+00_JPRB, 0.91004E+00_JPRB, 0.87948E+00_JPRB, 0.10722E+01_JPRB ,&
&  0.85102E+00_JPRB, 0.71273E+00_JPRB, 0.80025E+00_JPRB, 0.90351E+00_JPRB, 0.96473E+00_JPRB, 0.10270E+01_JPRB, 0.11797E+01_JPRB ,&
&  0.13582E+01_JPRB, 0.14997E+01_JPRB, 0.13553E+01_JPRB, 0.11524E+01_JPRB, 0.94263E+00_JPRB, 0.85393E+00_JPRB, 0.81813E+00_JPRB ,&
&  0.81465E+00_JPRB, 0.84182E+00_JPRB, 0.86279E+00_JPRB, 0.87414E+00_JPRB, 0.89012E+00_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.91612E+00_JPRB, 0.93309E+00_JPRB, 0.94566E+00_JPRB, 0.96467E+00_JPRB, 0.98828E+00_JPRB, 0.10150E+01_JPRB, 0.10352E+01_JPRB ,&
&  0.10573E+01_JPRB, 0.10748E+01_JPRB, 0.10911E+01_JPRB, 0.11107E+01_JPRB, 0.11251E+01_JPRB, 0.11455E+01_JPRB, 0.11522E+01_JPRB ,&
&  0.11725E+01_JPRB, 0.11699E+01_JPRB, 0.11804E+01_JPRB, 0.11917E+01_JPRB, 0.12020E+01_JPRB, 0.12115E+01_JPRB, 0.12288E+01_JPRB ,&
&  0.12400E+01_JPRB, 0.12380E+01_JPRB, 0.12478E+01_JPRB, 0.12634E+01_JPRB, 0.12689E+01_JPRB, 0.13518E+01_JPRB, 0.12281E+01_JPRB ,&
&  0.16817E+01_JPRB, 0.18996E+01_JPRB, 0.18441E+01_JPRB, 0.15526E+01_JPRB, 0.14375E+01_JPRB, 0.14149E+01_JPRB, 0.14227E+01_JPRB ,&
&  0.14297E+01_JPRB, 0.14434E+01_JPRB, 0.14531E+01_JPRB, 0.14948E+01_JPRB, 0.15869E+01_JPRB, 0.17213E+01_JPRB, 0.18370E+01_JPRB ,&
&  0.19268E+01_JPRB, 0.20405E+01_JPRB, 0.21811E+01_JPRB, 0.23395E+01_JPRB, 0.25397E+01_JPRB, 0.26616E+01_JPRB, 0.27058E+01_JPRB ,&
&  0.26978E+01_JPRB, 0.26702E+01_JPRB, 0.24381E+01_JPRB, 0.25419E+01_JPRB, 0.26134E+01_JPRB, 0.21072E+01_JPRB, 0.16972E+01_JPRB ,&
&  0.13929E+01_JPRB, 0.11923E+01_JPRB, 0.11788E+01_JPRB, 0.11698E+01_JPRB, 0.10426E+01_JPRB, 0.95184E+00_JPRB, 0.87445E+00_JPRB ,&
&  0.73178E+00_JPRB, 0.65875E+00_JPRB, 0.61972E+00_JPRB, 0.58283E+00_JPRB, 0.58393E+00_JPRB, 0.60696E+00_JPRB, 0.66334E+00_JPRB ,&
&  0.70088E+00_JPRB, 0.74274E+00_JPRB, 0.79675E+00_JPRB, 0.84107E+00_JPRB, 0.92202E+00_JPRB, 0.90897E+00_JPRB, 0.10542E+01_JPRB ,&
&  0.98376E+00_JPRB, 0.95654E+00_JPRB, 0.99174E+00_JPRB, 0.81483E+00_JPRB, 0.10604E+01_JPRB, 0.11446E+01_JPRB, 0.12917E+01_JPRB ,&
&  0.13377E+01_JPRB, 0.18012E+01_JPRB, 0.21690E+01_JPRB, 0.18224E+01_JPRB, 0.13294E+01_JPRB, 0.92682E+00_JPRB, 0.83458E+00_JPRB ,&
&  0.83325E+00_JPRB, 0.85525E+00_JPRB, 0.87738E+00_JPRB, 0.88863E+00_JPRB, 0.90191E+00_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.88504E+00_JPRB, 0.90047E+00_JPRB, 0.91459E+00_JPRB, 0.92483E+00_JPRB, 0.92741E+00_JPRB, 0.93983E+00_JPRB, 0.96233E+00_JPRB ,&
&  0.98285E+00_JPRB, 0.10030E+01_JPRB, 0.10226E+01_JPRB, 0.10394E+01_JPRB, 0.10578E+01_JPRB, 0.10704E+01_JPRB, 0.10966E+01_JPRB ,&
&  0.11338E+01_JPRB, 0.11694E+01_JPRB, 0.11898E+01_JPRB, 0.12284E+01_JPRB, 0.12538E+01_JPRB, 0.12672E+01_JPRB, 0.13154E+01_JPRB ,&
&  0.13638E+01_JPRB, 0.14248E+01_JPRB, 0.14693E+01_JPRB, 0.14948E+01_JPRB, 0.14884E+01_JPRB, 0.15747E+01_JPRB, 0.14195E+01_JPRB ,&
&  0.17928E+01_JPRB, 0.20127E+01_JPRB, 0.19072E+01_JPRB, 0.16202E+01_JPRB, 0.15170E+01_JPRB, 0.14710E+01_JPRB, 0.14366E+01_JPRB ,&
&  0.14374E+01_JPRB, 0.14217E+01_JPRB, 0.13741E+01_JPRB, 0.14650E+01_JPRB, 0.15653E+01_JPRB, 0.17161E+01_JPRB, 0.18908E+01_JPRB ,&
&  0.20364E+01_JPRB, 0.21946E+01_JPRB, 0.23639E+01_JPRB, 0.25447E+01_JPRB, 0.27778E+01_JPRB, 0.29747E+01_JPRB, 0.31291E+01_JPRB ,&
&  0.32501E+01_JPRB, 0.33355E+01_JPRB, 0.34095E+01_JPRB, 0.32172E+01_JPRB, 0.32012E+01_JPRB, 0.28161E+01_JPRB, 0.20956E+01_JPRB ,&
&  0.13493E+01_JPRB, 0.11333E+01_JPRB, 0.11468E+01_JPRB, 0.11293E+01_JPRB, 0.10367E+01_JPRB, 0.88840E+00_JPRB, 0.82956E+00_JPRB ,&
&  0.70361E+00_JPRB, 0.62861E+00_JPRB, 0.60639E+00_JPRB, 0.58643E+00_JPRB, 0.57792E+00_JPRB, 0.61583E+00_JPRB, 0.64950E+00_JPRB ,&
&  0.70441E+00_JPRB, 0.78157E+00_JPRB, 0.84753E+00_JPRB, 0.90650E+00_JPRB, 0.10245E+01_JPRB, 0.11343E+01_JPRB, 0.10530E+01_JPRB ,&
&  0.10703E+01_JPRB, 0.10410E+01_JPRB, 0.10591E+01_JPRB, 0.11729E+01_JPRB, 0.11386E+01_JPRB, 0.13395E+01_JPRB, 0.15565E+01_JPRB ,&
&  0.18606E+01_JPRB, 0.17943E+01_JPRB, 0.20935E+01_JPRB, 0.31224E+01_JPRB, 0.26707E+01_JPRB, 0.22565E+01_JPRB, 0.85329E+00_JPRB ,&
&  0.84439E+00_JPRB, 0.86013E+00_JPRB, 0.88445E+00_JPRB, 0.88674E+00_JPRB, 0.88921E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.79781E+00_JPRB, 0.83200E+00_JPRB, 0.83768E+00_JPRB, 0.86471E+00_JPRB, 0.86337E+00_JPRB, 0.85384E+00_JPRB, 0.86536E+00_JPRB ,&
&  0.88157E+00_JPRB, 0.89472E+00_JPRB, 0.91941E+00_JPRB, 0.94418E+00_JPRB, 0.96305E+00_JPRB, 0.97026E+00_JPRB, 0.98689E+00_JPRB ,&
&  0.10153E+01_JPRB, 0.10533E+01_JPRB, 0.11000E+01_JPRB, 0.11629E+01_JPRB, 0.12084E+01_JPRB, 0.12443E+01_JPRB, 0.12998E+01_JPRB ,&
&  0.13458E+01_JPRB, 0.14336E+01_JPRB, 0.15290E+01_JPRB, 0.16220E+01_JPRB, 0.16804E+01_JPRB, 0.17056E+01_JPRB, 0.13717E+01_JPRB ,&
&  0.18695E+01_JPRB, 0.19317E+01_JPRB, 0.18661E+01_JPRB, 0.16422E+01_JPRB, 0.15278E+01_JPRB, 0.14548E+01_JPRB, 0.14120E+01_JPRB ,&
&  0.13908E+01_JPRB, 0.13621E+01_JPRB, 0.13246E+01_JPRB, 0.13122E+01_JPRB, 0.15159E+01_JPRB, 0.16211E+01_JPRB, 0.17857E+01_JPRB ,&
&  0.19609E+01_JPRB, 0.21236E+01_JPRB, 0.22910E+01_JPRB, 0.24738E+01_JPRB, 0.27189E+01_JPRB, 0.29982E+01_JPRB, 0.32660E+01_JPRB ,&
&  0.35046E+01_JPRB, 0.36432E+01_JPRB, 0.38442E+01_JPRB, 0.34488E+01_JPRB, 0.33525E+01_JPRB, 0.31528E+01_JPRB, 0.19978E+01_JPRB ,&
&  0.12320E+01_JPRB, 0.11373E+01_JPRB, 0.11940E+01_JPRB, 0.11560E+01_JPRB, 0.10327E+01_JPRB, 0.88480E+00_JPRB, 0.78061E+00_JPRB ,&
&  0.67857E+00_JPRB, 0.64748E+00_JPRB, 0.62232E+00_JPRB, 0.60249E+00_JPRB, 0.59435E+00_JPRB, 0.62468E+00_JPRB, 0.65406E+00_JPRB ,&
&  0.71785E+00_JPRB, 0.76889E+00_JPRB, 0.83164E+00_JPRB, 0.89373E+00_JPRB, 0.98129E+00_JPRB, 0.10898E+01_JPRB, 0.11955E+01_JPRB ,&
&  0.10330E+01_JPRB, 0.94083E+00_JPRB, 0.96314E+00_JPRB, 0.10171E+01_JPRB, 0.10518E+01_JPRB, 0.11378E+01_JPRB, 0.12418E+01_JPRB ,&
&  0.13706E+01_JPRB, 0.14650E+01_JPRB, 0.13359E+01_JPRB, 0.11190E+01_JPRB, 0.14707E+01_JPRB, 0.15148E+01_JPRB, 0.71779E+00_JPRB ,&
&  0.70884E+00_JPRB, 0.74256E+00_JPRB, 0.76258E+00_JPRB, 0.78557E+00_JPRB, 0.80710E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.65481E+00_JPRB, 0.66288E+00_JPRB, 0.66401E+00_JPRB, 0.70177E+00_JPRB, 0.71196E+00_JPRB, 0.72987E+00_JPRB, 0.74482E+00_JPRB ,&
&  0.75917E+00_JPRB, 0.78346E+00_JPRB, 0.81147E+00_JPRB, 0.84074E+00_JPRB, 0.86445E+00_JPRB, 0.88215E+00_JPRB, 0.89682E+00_JPRB ,&
&  0.91235E+00_JPRB, 0.93141E+00_JPRB, 0.95660E+00_JPRB, 0.99064E+00_JPRB, 0.10459E+01_JPRB, 0.11082E+01_JPRB, 0.11744E+01_JPRB ,&
&  0.12386E+01_JPRB, 0.13237E+01_JPRB, 0.14275E+01_JPRB, 0.15511E+01_JPRB, 0.16771E+01_JPRB, 0.17677E+01_JPRB, 0.18198E+01_JPRB ,&
&  0.14588E+01_JPRB, 0.17735E+01_JPRB, 0.17497E+01_JPRB, 0.16181E+01_JPRB, 0.15028E+01_JPRB, 0.14420E+01_JPRB, 0.13982E+01_JPRB ,&
&  0.13698E+01_JPRB, 0.13017E+01_JPRB, 0.12999E+01_JPRB, 0.13326E+01_JPRB, 0.14321E+01_JPRB, 0.15026E+01_JPRB, 0.16185E+01_JPRB ,&
&  0.17802E+01_JPRB, 0.19106E+01_JPRB, 0.20693E+01_JPRB, 0.22329E+01_JPRB, 0.24545E+01_JPRB, 0.27943E+01_JPRB, 0.31897E+01_JPRB ,&
&  0.36123E+01_JPRB, 0.39477E+01_JPRB, 0.41980E+01_JPRB, 0.37190E+01_JPRB, 0.34169E+01_JPRB, 0.31112E+01_JPRB, 0.19377E+01_JPRB ,&
&  0.12974E+01_JPRB, 0.10976E+01_JPRB, 0.11652E+01_JPRB, 0.12066E+01_JPRB, 0.10552E+01_JPRB, 0.92497E+00_JPRB, 0.80160E+00_JPRB ,&
&  0.70708E+00_JPRB, 0.65208E+00_JPRB, 0.63452E+00_JPRB, 0.61176E+00_JPRB, 0.60893E+00_JPRB, 0.62864E+00_JPRB, 0.64979E+00_JPRB ,&
&  0.69279E+00_JPRB, 0.71223E+00_JPRB, 0.76187E+00_JPRB, 0.80597E+00_JPRB, 0.85238E+00_JPRB, 0.88969E+00_JPRB, 0.92321E+00_JPRB ,&
&  0.94462E+00_JPRB, 0.95365E+00_JPRB, 0.95726E+00_JPRB, 0.98225E+00_JPRB, 0.99153E+00_JPRB, 0.10079E+01_JPRB, 0.10348E+01_JPRB ,&
&  0.10642E+01_JPRB, 0.10806E+01_JPRB, 0.10545E+01_JPRB, 0.10247E+01_JPRB, 0.96990E+00_JPRB, 0.91722E+00_JPRB, 0.70564E+00_JPRB ,&
&  0.66806E+00_JPRB, 0.62891E+00_JPRB, 0.64579E+00_JPRB, 0.66722E+00_JPRB, 0.65935E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.57914E+00_JPRB, 0.56684E+00_JPRB, 0.55354E+00_JPRB, 0.55815E+00_JPRB, 0.56407E+00_JPRB, 0.58719E+00_JPRB, 0.58913E+00_JPRB ,&
&  0.60497E+00_JPRB, 0.65240E+00_JPRB, 0.70981E+00_JPRB, 0.74198E+00_JPRB, 0.76793E+00_JPRB, 0.79326E+00_JPRB, 0.81474E+00_JPRB ,&
&  0.82696E+00_JPRB, 0.83615E+00_JPRB, 0.84537E+00_JPRB, 0.85796E+00_JPRB, 0.88209E+00_JPRB, 0.91997E+00_JPRB, 0.98019E+00_JPRB ,&
&  0.10417E+01_JPRB, 0.11089E+01_JPRB, 0.11932E+01_JPRB, 0.13003E+01_JPRB, 0.14148E+01_JPRB, 0.14783E+01_JPRB, 0.15184E+01_JPRB ,&
&  0.94688E+00_JPRB, 0.14540E+01_JPRB, 0.17324E+01_JPRB, 0.16384E+01_JPRB, 0.15074E+01_JPRB, 0.14364E+01_JPRB, 0.13812E+01_JPRB ,&
&  0.12950E+01_JPRB, 0.12072E+01_JPRB, 0.12009E+01_JPRB, 0.13397E+01_JPRB, 0.13299E+01_JPRB, 0.13680E+01_JPRB, 0.14112E+01_JPRB ,&
&  0.15311E+01_JPRB, 0.16199E+01_JPRB, 0.16720E+01_JPRB, 0.17476E+01_JPRB, 0.18758E+01_JPRB, 0.21618E+01_JPRB, 0.26486E+01_JPRB ,&
&  0.33044E+01_JPRB, 0.39970E+01_JPRB, 0.43919E+01_JPRB, 0.37788E+01_JPRB, 0.38902E+01_JPRB, 0.35452E+01_JPRB, 0.28659E+01_JPRB ,&
&  0.16493E+01_JPRB, 0.11760E+01_JPRB, 0.12049E+01_JPRB, 0.12317E+01_JPRB, 0.10950E+01_JPRB, 0.99473E+00_JPRB, 0.88629E+00_JPRB ,&
&  0.77487E+00_JPRB, 0.71201E+00_JPRB, 0.67665E+00_JPRB, 0.65196E+00_JPRB, 0.64141E+00_JPRB, 0.64320E+00_JPRB, 0.65782E+00_JPRB ,&
&  0.66594E+00_JPRB, 0.68268E+00_JPRB, 0.71068E+00_JPRB, 0.74360E+00_JPRB, 0.78612E+00_JPRB, 0.82923E+00_JPRB, 0.87466E+00_JPRB ,&
&  0.90798E+00_JPRB, 0.93679E+00_JPRB, 0.96134E+00_JPRB, 0.98033E+00_JPRB, 0.98901E+00_JPRB, 0.99808E+00_JPRB, 0.98689E+00_JPRB ,&
&  0.97744E+00_JPRB, 0.99247E+00_JPRB, 0.99861E+00_JPRB, 0.98626E+00_JPRB, 0.94461E+00_JPRB, 0.90265E+00_JPRB, 0.86446E+00_JPRB ,&
&  0.79579E+00_JPRB, 0.69992E+00_JPRB, 0.61748E+00_JPRB, 0.59543E+00_JPRB, 0.58309E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.54274E+00_JPRB, 0.51877E+00_JPRB, 0.51073E+00_JPRB, 0.51211E+00_JPRB, 0.52342E+00_JPRB, 0.53047E+00_JPRB, 0.53020E+00_JPRB ,&
&  0.53343E+00_JPRB, 0.54529E+00_JPRB, 0.56020E+00_JPRB, 0.60843E+00_JPRB, 0.65895E+00_JPRB, 0.69680E+00_JPRB, 0.72077E+00_JPRB ,&
&  0.74001E+00_JPRB, 0.74510E+00_JPRB, 0.74164E+00_JPRB, 0.74299E+00_JPRB, 0.74583E+00_JPRB, 0.75934E+00_JPRB, 0.79025E+00_JPRB ,&
&  0.84294E+00_JPRB, 0.90013E+00_JPRB, 0.97187E+00_JPRB, 0.10666E+01_JPRB, 0.11864E+01_JPRB, 0.13030E+01_JPRB, 0.13909E+01_JPRB ,&
&  0.10639E+01_JPRB, 0.60437E+00_JPRB, 0.14921E+01_JPRB, 0.17713E+01_JPRB, 0.15758E+01_JPRB, 0.14976E+01_JPRB, 0.13768E+01_JPRB ,&
&  0.12177E+01_JPRB, 0.11667E+01_JPRB, 0.11253E+01_JPRB, 0.11919E+01_JPRB, 0.12075E+01_JPRB, 0.12471E+01_JPRB, 0.12839E+01_JPRB ,&
&  0.13528E+01_JPRB, 0.14504E+01_JPRB, 0.14702E+01_JPRB, 0.14514E+01_JPRB, 0.14600E+01_JPRB, 0.15657E+01_JPRB, 0.18861E+01_JPRB ,&
&  0.25326E+01_JPRB, 0.34918E+01_JPRB, 0.43922E+01_JPRB, 0.39538E+01_JPRB, 0.40944E+01_JPRB, 0.32009E+01_JPRB, 0.22829E+01_JPRB ,&
&  0.16698E+01_JPRB, 0.13160E+01_JPRB, 0.12634E+01_JPRB, 0.12764E+01_JPRB, 0.11650E+01_JPRB, 0.10294E+01_JPRB, 0.99935E+00_JPRB ,&
&  0.86726E+00_JPRB, 0.78442E+00_JPRB, 0.72662E+00_JPRB, 0.69485E+00_JPRB, 0.67948E+00_JPRB, 0.67629E+00_JPRB, 0.67927E+00_JPRB ,&
&  0.67872E+00_JPRB, 0.68548E+00_JPRB, 0.69561E+00_JPRB, 0.72199E+00_JPRB, 0.75923E+00_JPRB, 0.81155E+00_JPRB, 0.85460E+00_JPRB ,&
&  0.87976E+00_JPRB, 0.89943E+00_JPRB, 0.91869E+00_JPRB, 0.92998E+00_JPRB, 0.93849E+00_JPRB, 0.91753E+00_JPRB, 0.91201E+00_JPRB ,&
&  0.92246E+00_JPRB, 0.93937E+00_JPRB, 0.96073E+00_JPRB, 0.98908E+00_JPRB, 0.95629E+00_JPRB, 0.92685E+00_JPRB, 0.94259E+00_JPRB ,&
&  0.92687E+00_JPRB, 0.85614E+00_JPRB, 0.73061E+00_JPRB, 0.61182E+00_JPRB, 0.56806E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.58616E+00_JPRB, 0.56220E+00_JPRB, 0.53416E+00_JPRB, 0.52732E+00_JPRB, 0.53722E+00_JPRB, 0.53223E+00_JPRB, 0.52931E+00_JPRB ,&
&  0.52500E+00_JPRB, 0.52077E+00_JPRB, 0.50579E+00_JPRB, 0.51418E+00_JPRB, 0.54388E+00_JPRB, 0.58832E+00_JPRB, 0.62488E+00_JPRB ,&
&  0.65254E+00_JPRB, 0.66245E+00_JPRB, 0.66220E+00_JPRB, 0.66105E+00_JPRB, 0.66027E+00_JPRB, 0.66064E+00_JPRB, 0.66532E+00_JPRB ,&
&  0.68567E+00_JPRB, 0.71981E+00_JPRB, 0.76526E+00_JPRB, 0.81642E+00_JPRB, 0.88627E+00_JPRB, 0.99951E+00_JPRB, 0.11808E+01_JPRB ,&
&  0.14652E+01_JPRB, 0.10957E+01_JPRB, 0.74501E+00_JPRB, 0.15154E+01_JPRB, 0.16864E+01_JPRB, 0.16549E+01_JPRB, 0.14145E+01_JPRB ,&
&  0.13085E+01_JPRB, 0.11779E+01_JPRB, 0.10967E+01_JPRB, 0.11327E+01_JPRB, 0.10793E+01_JPRB, 0.11078E+01_JPRB, 0.11692E+01_JPRB ,&
&  0.12230E+01_JPRB, 0.12806E+01_JPRB, 0.13297E+01_JPRB, 0.13189E+01_JPRB, 0.12700E+01_JPRB, 0.12538E+01_JPRB, 0.13659E+01_JPRB ,&
&  0.17640E+01_JPRB, 0.25831E+01_JPRB, 0.39045E+01_JPRB, 0.40309E+01_JPRB, 0.41696E+01_JPRB, 0.29159E+01_JPRB, 0.21050E+01_JPRB ,&
&  0.17075E+01_JPRB, 0.16239E+01_JPRB, 0.15276E+01_JPRB, 0.13680E+01_JPRB, 0.11984E+01_JPRB, 0.10948E+01_JPRB, 0.11006E+01_JPRB ,&
&  0.98017E+00_JPRB, 0.89184E+00_JPRB, 0.81761E+00_JPRB, 0.77095E+00_JPRB, 0.74089E+00_JPRB, 0.72105E+00_JPRB, 0.70702E+00_JPRB ,&
&  0.70909E+00_JPRB, 0.70955E+00_JPRB, 0.71929E+00_JPRB, 0.73196E+00_JPRB, 0.75090E+00_JPRB, 0.78111E+00_JPRB, 0.82134E+00_JPRB ,&
&  0.85195E+00_JPRB, 0.86637E+00_JPRB, 0.87226E+00_JPRB, 0.86974E+00_JPRB, 0.84642E+00_JPRB, 0.83125E+00_JPRB, 0.85248E+00_JPRB ,&
&  0.85244E+00_JPRB, 0.86803E+00_JPRB, 0.87301E+00_JPRB, 0.90124E+00_JPRB, 0.92228E+00_JPRB, 0.93334E+00_JPRB, 0.95882E+00_JPRB ,&
&  0.99266E+00_JPRB, 0.97306E+00_JPRB, 0.86915E+00_JPRB, 0.66801E+00_JPRB, 0.61306E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.65683E+00_JPRB, 0.63844E+00_JPRB, 0.62428E+00_JPRB, 0.62337E+00_JPRB, 0.62177E+00_JPRB, 0.60610E+00_JPRB, 0.58538E+00_JPRB ,&
&  0.55868E+00_JPRB, 0.53852E+00_JPRB, 0.51018E+00_JPRB, 0.48351E+00_JPRB, 0.47850E+00_JPRB, 0.49146E+00_JPRB, 0.51934E+00_JPRB ,&
&  0.54649E+00_JPRB, 0.56054E+00_JPRB, 0.57082E+00_JPRB, 0.58067E+00_JPRB, 0.58938E+00_JPRB, 0.59298E+00_JPRB, 0.59155E+00_JPRB ,&
&  0.59587E+00_JPRB, 0.60836E+00_JPRB, 0.62801E+00_JPRB, 0.65059E+00_JPRB, 0.68142E+00_JPRB, 0.73395E+00_JPRB, 0.83403E+00_JPRB ,&
&  0.10442E+01_JPRB, 0.11560E+01_JPRB, 0.63402E+00_JPRB, 0.12031E+01_JPRB, 0.16809E+01_JPRB, 0.16668E+01_JPRB, 0.15711E+01_JPRB ,&
&  0.14381E+01_JPRB, 0.12284E+01_JPRB, 0.11756E+01_JPRB, 0.11101E+01_JPRB, 0.10147E+01_JPRB, 0.99484E+00_JPRB, 0.10181E+01_JPRB ,&
&  0.10365E+01_JPRB, 0.10255E+01_JPRB, 0.10456E+01_JPRB, 0.10960E+01_JPRB, 0.10923E+01_JPRB, 0.10559E+01_JPRB, 0.10768E+01_JPRB ,&
&  0.12705E+01_JPRB, 0.18333E+01_JPRB, 0.30691E+01_JPRB, 0.38378E+01_JPRB, 0.41020E+01_JPRB, 0.31833E+01_JPRB, 0.23578E+01_JPRB ,&
&  0.20303E+01_JPRB, 0.20596E+01_JPRB, 0.18866E+01_JPRB, 0.15612E+01_JPRB, 0.12940E+01_JPRB, 0.12072E+01_JPRB, 0.11840E+01_JPRB ,&
&  0.10806E+01_JPRB, 0.98581E+00_JPRB, 0.92535E+00_JPRB, 0.87631E+00_JPRB, 0.83780E+00_JPRB, 0.79838E+00_JPRB, 0.76625E+00_JPRB ,&
&  0.75524E+00_JPRB, 0.74864E+00_JPRB, 0.74774E+00_JPRB, 0.74856E+00_JPRB, 0.74959E+00_JPRB, 0.75777E+00_JPRB, 0.78014E+00_JPRB ,&
&  0.81001E+00_JPRB, 0.82555E+00_JPRB, 0.83230E+00_JPRB, 0.81368E+00_JPRB, 0.79758E+00_JPRB, 0.79778E+00_JPRB, 0.81525E+00_JPRB ,&
&  0.81965E+00_JPRB, 0.83040E+00_JPRB, 0.83017E+00_JPRB, 0.83061E+00_JPRB, 0.87719E+00_JPRB, 0.89379E+00_JPRB, 0.92954E+00_JPRB ,&
&  0.97670E+00_JPRB, 0.10379E+01_JPRB, 0.10310E+01_JPRB, 0.74761E+00_JPRB, 0.69518E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.71241E+00_JPRB, 0.68935E+00_JPRB, 0.68573E+00_JPRB, 0.68648E+00_JPRB, 0.68430E+00_JPRB, 0.67531E+00_JPRB, 0.65612E+00_JPRB ,&
&  0.63192E+00_JPRB, 0.61316E+00_JPRB, 0.59101E+00_JPRB, 0.56233E+00_JPRB, 0.52118E+00_JPRB, 0.48387E+00_JPRB, 0.47880E+00_JPRB ,&
&  0.49619E+00_JPRB, 0.50407E+00_JPRB, 0.51125E+00_JPRB, 0.52052E+00_JPRB, 0.53056E+00_JPRB, 0.54244E+00_JPRB, 0.55042E+00_JPRB ,&
&  0.55347E+00_JPRB, 0.55883E+00_JPRB, 0.56853E+00_JPRB, 0.57812E+00_JPRB, 0.59213E+00_JPRB, 0.61920E+00_JPRB, 0.66826E+00_JPRB ,&
&  0.80594E+00_JPRB, 0.99697E+00_JPRB, 0.10373E+01_JPRB, 0.16978E+01_JPRB, 0.19692E+01_JPRB, 0.18488E+01_JPRB, 0.18099E+01_JPRB ,&
&  0.17338E+01_JPRB, 0.15284E+01_JPRB, 0.13787E+01_JPRB, 0.11530E+01_JPRB, 0.10227E+01_JPRB, 0.96194E+00_JPRB, 0.93505E+00_JPRB ,&
&  0.91446E+00_JPRB, 0.88551E+00_JPRB, 0.85484E+00_JPRB, 0.86255E+00_JPRB, 0.87640E+00_JPRB, 0.87275E+00_JPRB, 0.88733E+00_JPRB ,&
&  0.98041E+00_JPRB, 0.13198E+01_JPRB, 0.22005E+01_JPRB, 0.36009E+01_JPRB, 0.40612E+01_JPRB, 0.36395E+01_JPRB, 0.29785E+01_JPRB ,&
&  0.29570E+01_JPRB, 0.27705E+01_JPRB, 0.24623E+01_JPRB, 0.19329E+01_JPRB, 0.15231E+01_JPRB, 0.14845E+01_JPRB, 0.12750E+01_JPRB ,&
&  0.11454E+01_JPRB, 0.10571E+01_JPRB, 0.10038E+01_JPRB, 0.96433E+00_JPRB, 0.93181E+00_JPRB, 0.88953E+00_JPRB, 0.83974E+00_JPRB ,&
&  0.80068E+00_JPRB, 0.78038E+00_JPRB, 0.76617E+00_JPRB, 0.76011E+00_JPRB, 0.75319E+00_JPRB, 0.74793E+00_JPRB, 0.75130E+00_JPRB ,&
&  0.77167E+00_JPRB, 0.78750E+00_JPRB, 0.76560E+00_JPRB, 0.76001E+00_JPRB, 0.76345E+00_JPRB, 0.76538E+00_JPRB, 0.76799E+00_JPRB ,&
&  0.76893E+00_JPRB, 0.79092E+00_JPRB, 0.81421E+00_JPRB, 0.81809E+00_JPRB, 0.83643E+00_JPRB, 0.85989E+00_JPRB, 0.87188E+00_JPRB ,&
&  0.88217E+00_JPRB, 0.86960E+00_JPRB, 0.84884E+00_JPRB, 0.77820E+00_JPRB, 0.74596E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.73865E+00_JPRB, 0.71753E+00_JPRB, 0.70481E+00_JPRB, 0.69791E+00_JPRB, 0.69320E+00_JPRB, 0.68865E+00_JPRB, 0.67333E+00_JPRB ,&
&  0.65589E+00_JPRB, 0.64509E+00_JPRB, 0.62761E+00_JPRB, 0.60954E+00_JPRB, 0.57400E+00_JPRB, 0.52877E+00_JPRB, 0.50768E+00_JPRB ,&
&  0.49570E+00_JPRB, 0.49371E+00_JPRB, 0.50215E+00_JPRB, 0.50656E+00_JPRB, 0.50961E+00_JPRB, 0.51729E+00_JPRB, 0.52423E+00_JPRB ,&
&  0.53122E+00_JPRB, 0.53350E+00_JPRB, 0.53976E+00_JPRB, 0.54565E+00_JPRB, 0.55036E+00_JPRB, 0.56780E+00_JPRB, 0.59728E+00_JPRB ,&
&  0.68982E+00_JPRB, 0.94524E+00_JPRB, 0.83929E+00_JPRB, 0.16441E+01_JPRB, 0.18825E+01_JPRB, 0.18771E+01_JPRB, 0.17354E+01_JPRB ,&
&  0.17116E+01_JPRB, 0.16991E+01_JPRB, 0.13796E+01_JPRB, 0.11715E+01_JPRB, 0.10278E+01_JPRB, 0.95259E+00_JPRB, 0.90702E+00_JPRB ,&
&  0.87368E+00_JPRB, 0.83886E+00_JPRB, 0.79460E+00_JPRB, 0.76180E+00_JPRB, 0.75212E+00_JPRB, 0.74846E+00_JPRB, 0.76055E+00_JPRB ,&
&  0.80134E+00_JPRB, 0.98379E+00_JPRB, 0.15595E+01_JPRB, 0.27859E+01_JPRB, 0.38847E+01_JPRB, 0.40249E+01_JPRB, 0.36021E+01_JPRB ,&
&  0.36240E+01_JPRB, 0.36635E+01_JPRB, 0.32887E+01_JPRB, 0.25205E+01_JPRB, 0.19829E+01_JPRB, 0.17147E+01_JPRB, 0.14443E+01_JPRB ,&
&  0.12693E+01_JPRB, 0.11591E+01_JPRB, 0.10920E+01_JPRB, 0.10358E+01_JPRB, 0.99076E+00_JPRB, 0.94466E+00_JPRB, 0.88760E+00_JPRB ,&
&  0.83565E+00_JPRB, 0.79401E+00_JPRB, 0.77279E+00_JPRB, 0.76022E+00_JPRB, 0.75670E+00_JPRB, 0.75244E+00_JPRB, 0.74891E+00_JPRB ,&
&  0.75366E+00_JPRB, 0.76845E+00_JPRB, 0.73535E+00_JPRB, 0.72498E+00_JPRB, 0.72975E+00_JPRB, 0.73311E+00_JPRB, 0.74276E+00_JPRB ,&
&  0.76388E+00_JPRB, 0.77310E+00_JPRB, 0.78313E+00_JPRB, 0.77877E+00_JPRB, 0.79818E+00_JPRB, 0.84194E+00_JPRB, 0.84730E+00_JPRB ,&
&  0.84991E+00_JPRB, 0.83429E+00_JPRB, 0.81398E+00_JPRB, 0.78295E+00_JPRB, 0.75992E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.74378E+00_JPRB, 0.72772E+00_JPRB, 0.70750E+00_JPRB, 0.68877E+00_JPRB, 0.68404E+00_JPRB, 0.68203E+00_JPRB, 0.66943E+00_JPRB ,&
&  0.65681E+00_JPRB, 0.64249E+00_JPRB, 0.62224E+00_JPRB, 0.59872E+00_JPRB, 0.57252E+00_JPRB, 0.54050E+00_JPRB, 0.51108E+00_JPRB ,&
&  0.49729E+00_JPRB, 0.48643E+00_JPRB, 0.48826E+00_JPRB, 0.49391E+00_JPRB, 0.50218E+00_JPRB, 0.50777E+00_JPRB, 0.51459E+00_JPRB ,&
&  0.52087E+00_JPRB, 0.52429E+00_JPRB, 0.52838E+00_JPRB, 0.53182E+00_JPRB, 0.53288E+00_JPRB, 0.54020E+00_JPRB, 0.57226E+00_JPRB ,&
&  0.64333E+00_JPRB, 0.10157E+01_JPRB, 0.83828E+00_JPRB, 0.13618E+01_JPRB, 0.15709E+01_JPRB, 0.16672E+01_JPRB, 0.15632E+01_JPRB ,&
&  0.15568E+01_JPRB, 0.14816E+01_JPRB, 0.12811E+01_JPRB, 0.11285E+01_JPRB, 0.10148E+01_JPRB, 0.94032E+00_JPRB, 0.88765E+00_JPRB ,&
&  0.85077E+00_JPRB, 0.80694E+00_JPRB, 0.75762E+00_JPRB, 0.71449E+00_JPRB, 0.69424E+00_JPRB, 0.68491E+00_JPRB, 0.68741E+00_JPRB ,&
&  0.70156E+00_JPRB, 0.78696E+00_JPRB, 0.11076E+01_JPRB, 0.21035E+01_JPRB, 0.32752E+01_JPRB, 0.39787E+01_JPRB, 0.41307E+01_JPRB ,&
&  0.42381E+01_JPRB, 0.45188E+01_JPRB, 0.40951E+01_JPRB, 0.32275E+01_JPRB, 0.25517E+01_JPRB, 0.20633E+01_JPRB, 0.16917E+01_JPRB ,&
&  0.14647E+01_JPRB, 0.13278E+01_JPRB, 0.12332E+01_JPRB, 0.11541E+01_JPRB, 0.10784E+01_JPRB, 0.99469E+00_JPRB, 0.90889E+00_JPRB ,&
&  0.84333E+00_JPRB, 0.79866E+00_JPRB, 0.77093E+00_JPRB, 0.76324E+00_JPRB, 0.76168E+00_JPRB, 0.76064E+00_JPRB, 0.76085E+00_JPRB ,&
&  0.76248E+00_JPRB, 0.76847E+00_JPRB, 0.73855E+00_JPRB, 0.72547E+00_JPRB, 0.72294E+00_JPRB, 0.72895E+00_JPRB, 0.73389E+00_JPRB ,&
&  0.76301E+00_JPRB, 0.76859E+00_JPRB, 0.76041E+00_JPRB, 0.76029E+00_JPRB, 0.77863E+00_JPRB, 0.83471E+00_JPRB, 0.84250E+00_JPRB ,&
&  0.83692E+00_JPRB, 0.81571E+00_JPRB, 0.79534E+00_JPRB, 0.77995E+00_JPRB, 0.76361E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.74533E+00_JPRB, 0.72565E+00_JPRB, 0.71249E+00_JPRB, 0.70023E+00_JPRB, 0.69564E+00_JPRB, 0.68954E+00_JPRB, 0.67383E+00_JPRB ,&
&  0.65184E+00_JPRB, 0.62595E+00_JPRB, 0.59994E+00_JPRB, 0.57799E+00_JPRB, 0.56107E+00_JPRB, 0.54252E+00_JPRB, 0.52721E+00_JPRB ,&
&  0.51383E+00_JPRB, 0.49845E+00_JPRB, 0.48848E+00_JPRB, 0.48450E+00_JPRB, 0.48725E+00_JPRB, 0.49205E+00_JPRB, 0.49897E+00_JPRB ,&
&  0.50755E+00_JPRB, 0.51328E+00_JPRB, 0.51426E+00_JPRB, 0.51638E+00_JPRB, 0.51439E+00_JPRB, 0.52022E+00_JPRB, 0.55447E+00_JPRB ,&
&  0.63713E+00_JPRB, 0.59995E+00_JPRB, 0.93206E+00_JPRB, 0.11046E+01_JPRB, 0.12457E+01_JPRB, 0.13800E+01_JPRB, 0.14493E+01_JPRB ,&
&  0.14817E+01_JPRB, 0.13367E+01_JPRB, 0.11657E+01_JPRB, 0.10527E+01_JPRB, 0.96363E+00_JPRB, 0.89658E+00_JPRB, 0.84315E+00_JPRB ,&
&  0.79092E+00_JPRB, 0.75108E+00_JPRB, 0.71917E+00_JPRB, 0.67883E+00_JPRB, 0.65479E+00_JPRB, 0.64631E+00_JPRB, 0.64497E+00_JPRB ,&
&  0.64174E+00_JPRB, 0.67199E+00_JPRB, 0.82812E+00_JPRB, 0.13598E+01_JPRB, 0.26626E+01_JPRB, 0.36797E+01_JPRB, 0.43827E+01_JPRB ,&
&  0.52214E+01_JPRB, 0.55000E+01_JPRB, 0.48793E+01_JPRB, 0.39170E+01_JPRB, 0.29946E+01_JPRB, 0.23745E+01_JPRB, 0.18722E+01_JPRB ,&
&  0.15821E+01_JPRB, 0.14202E+01_JPRB, 0.13254E+01_JPRB, 0.12497E+01_JPRB, 0.11702E+01_JPRB, 0.10623E+01_JPRB, 0.94145E+00_JPRB ,&
&  0.84664E+00_JPRB, 0.79286E+00_JPRB, 0.75990E+00_JPRB, 0.74162E+00_JPRB, 0.73292E+00_JPRB, 0.73397E+00_JPRB, 0.74223E+00_JPRB ,&
&  0.75428E+00_JPRB, 0.76781E+00_JPRB, 0.74266E+00_JPRB, 0.72625E+00_JPRB, 0.72579E+00_JPRB, 0.73797E+00_JPRB, 0.74983E+00_JPRB ,&
&  0.75131E+00_JPRB, 0.74104E+00_JPRB, 0.72763E+00_JPRB, 0.73043E+00_JPRB, 0.74798E+00_JPRB, 0.82516E+00_JPRB, 0.83684E+00_JPRB ,&
&  0.83168E+00_JPRB, 0.81241E+00_JPRB, 0.78415E+00_JPRB, 0.77413E+00_JPRB, 0.76807E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.74177E+00_JPRB, 0.72429E+00_JPRB, 0.72140E+00_JPRB, 0.72898E+00_JPRB, 0.73561E+00_JPRB, 0.72245E+00_JPRB, 0.68436E+00_JPRB ,&
&  0.64921E+00_JPRB, 0.61890E+00_JPRB, 0.59049E+00_JPRB, 0.56786E+00_JPRB, 0.55290E+00_JPRB, 0.54220E+00_JPRB, 0.53117E+00_JPRB ,&
&  0.52315E+00_JPRB, 0.51613E+00_JPRB, 0.50121E+00_JPRB, 0.48734E+00_JPRB, 0.48109E+00_JPRB, 0.47999E+00_JPRB, 0.48378E+00_JPRB ,&
&  0.48827E+00_JPRB, 0.49367E+00_JPRB, 0.49748E+00_JPRB, 0.50011E+00_JPRB, 0.49968E+00_JPRB, 0.50676E+00_JPRB, 0.53712E+00_JPRB ,&
&  0.63144E+00_JPRB, 0.80415E+00_JPRB, 0.92527E+00_JPRB, 0.94098E+00_JPRB, 0.10226E+01_JPRB, 0.11682E+01_JPRB, 0.13229E+01_JPRB ,&
&  0.13153E+01_JPRB, 0.12044E+01_JPRB, 0.10560E+01_JPRB, 0.95361E+00_JPRB, 0.87657E+00_JPRB, 0.82204E+00_JPRB, 0.76656E+00_JPRB ,&
&  0.72025E+00_JPRB, 0.68890E+00_JPRB, 0.65758E+00_JPRB, 0.62979E+00_JPRB, 0.62045E+00_JPRB, 0.62235E+00_JPRB, 0.62393E+00_JPRB ,&
&  0.62175E+00_JPRB, 0.62305E+00_JPRB, 0.69031E+00_JPRB, 0.10442E+01_JPRB, 0.21341E+01_JPRB, 0.28674E+01_JPRB, 0.41688E+01_JPRB ,&
&  0.54339E+01_JPRB, 0.53757E+01_JPRB, 0.48182E+01_JPRB, 0.40086E+01_JPRB, 0.31276E+01_JPRB, 0.23903E+01_JPRB, 0.19274E+01_JPRB ,&
&  0.16370E+01_JPRB, 0.14098E+01_JPRB, 0.12733E+01_JPRB, 0.11868E+01_JPRB, 0.11164E+01_JPRB, 0.10499E+01_JPRB, 0.96896E+00_JPRB ,&
&  0.88486E+00_JPRB, 0.81548E+00_JPRB, 0.76731E+00_JPRB, 0.73425E+00_JPRB, 0.71694E+00_JPRB, 0.71267E+00_JPRB, 0.71357E+00_JPRB ,&
&  0.71209E+00_JPRB, 0.72207E+00_JPRB, 0.72788E+00_JPRB, 0.72054E+00_JPRB, 0.72636E+00_JPRB, 0.74072E+00_JPRB, 0.74883E+00_JPRB ,&
&  0.72831E+00_JPRB, 0.71126E+00_JPRB, 0.70777E+00_JPRB, 0.70574E+00_JPRB, 0.74044E+00_JPRB, 0.82421E+00_JPRB, 0.82377E+00_JPRB ,&
&  0.81731E+00_JPRB, 0.80228E+00_JPRB, 0.78560E+00_JPRB, 0.76602E+00_JPRB, 0.76591E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.73286E+00_JPRB, 0.71341E+00_JPRB, 0.70476E+00_JPRB, 0.71856E+00_JPRB, 0.73627E+00_JPRB, 0.72903E+00_JPRB, 0.69318E+00_JPRB ,&
&  0.64594E+00_JPRB, 0.60774E+00_JPRB, 0.58099E+00_JPRB, 0.56215E+00_JPRB, 0.54798E+00_JPRB, 0.53584E+00_JPRB, 0.52258E+00_JPRB ,&
&  0.51153E+00_JPRB, 0.50545E+00_JPRB, 0.50104E+00_JPRB, 0.49512E+00_JPRB, 0.48424E+00_JPRB, 0.47567E+00_JPRB, 0.47271E+00_JPRB ,&
&  0.47110E+00_JPRB, 0.47141E+00_JPRB, 0.47659E+00_JPRB, 0.48248E+00_JPRB, 0.48469E+00_JPRB, 0.48872E+00_JPRB, 0.50925E+00_JPRB ,&
&  0.57947E+00_JPRB, 0.72330E+00_JPRB, 0.72762E+00_JPRB, 0.78991E+00_JPRB, 0.89776E+00_JPRB, 0.10249E+01_JPRB, 0.11366E+01_JPRB ,&
&  0.11435E+01_JPRB, 0.10860E+01_JPRB, 0.98108E+00_JPRB, 0.87519E+00_JPRB, 0.79048E+00_JPRB, 0.74097E+00_JPRB, 0.70565E+00_JPRB ,&
&  0.67268E+00_JPRB, 0.63905E+00_JPRB, 0.60981E+00_JPRB, 0.58797E+00_JPRB, 0.57353E+00_JPRB, 0.56897E+00_JPRB, 0.57123E+00_JPRB ,&
&  0.57375E+00_JPRB, 0.57432E+00_JPRB, 0.58867E+00_JPRB, 0.71485E+00_JPRB, 0.12898E+01_JPRB, 0.20794E+01_JPRB, 0.29637E+01_JPRB ,&
&  0.32537E+01_JPRB, 0.34007E+01_JPRB, 0.33639E+01_JPRB, 0.31485E+01_JPRB, 0.27617E+01_JPRB, 0.23787E+01_JPRB, 0.19656E+01_JPRB ,&
&  0.16614E+01_JPRB, 0.14558E+01_JPRB, 0.13075E+01_JPRB, 0.11826E+01_JPRB, 0.10709E+01_JPRB, 0.98146E+00_JPRB, 0.91905E+00_JPRB ,&
&  0.87374E+00_JPRB, 0.83474E+00_JPRB, 0.79814E+00_JPRB, 0.76470E+00_JPRB, 0.73942E+00_JPRB, 0.72456E+00_JPRB, 0.72234E+00_JPRB ,&
&  0.71742E+00_JPRB, 0.70397E+00_JPRB, 0.70156E+00_JPRB, 0.70422E+00_JPRB, 0.71042E+00_JPRB, 0.71145E+00_JPRB, 0.70928E+00_JPRB ,&
&  0.71246E+00_JPRB, 0.71434E+00_JPRB, 0.71118E+00_JPRB, 0.70887E+00_JPRB, 0.73450E+00_JPRB, 0.79595E+00_JPRB, 0.81230E+00_JPRB ,&
&  0.81145E+00_JPRB, 0.79524E+00_JPRB, 0.78220E+00_JPRB, 0.77155E+00_JPRB, 0.76435E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.82318E+00_JPRB, 0.74370E+00_JPRB, 0.69773E+00_JPRB, 0.68259E+00_JPRB, 0.68424E+00_JPRB, 0.67238E+00_JPRB, 0.64965E+00_JPRB ,&
&  0.62524E+00_JPRB, 0.60436E+00_JPRB, 0.58664E+00_JPRB, 0.56680E+00_JPRB, 0.54717E+00_JPRB, 0.53040E+00_JPRB, 0.51449E+00_JPRB ,&
&  0.49937E+00_JPRB, 0.49047E+00_JPRB, 0.48410E+00_JPRB, 0.47983E+00_JPRB, 0.47668E+00_JPRB, 0.47051E+00_JPRB, 0.46374E+00_JPRB ,&
&  0.45855E+00_JPRB, 0.45137E+00_JPRB, 0.44939E+00_JPRB, 0.45577E+00_JPRB, 0.46182E+00_JPRB, 0.46641E+00_JPRB, 0.48443E+00_JPRB ,&
&  0.54392E+00_JPRB, 0.58605E+00_JPRB, 0.64557E+00_JPRB, 0.69919E+00_JPRB, 0.75091E+00_JPRB, 0.82092E+00_JPRB, 0.85117E+00_JPRB ,&
&  0.83502E+00_JPRB, 0.84243E+00_JPRB, 0.82651E+00_JPRB, 0.77029E+00_JPRB, 0.70752E+00_JPRB, 0.66594E+00_JPRB, 0.63690E+00_JPRB ,&
&  0.61262E+00_JPRB, 0.58564E+00_JPRB, 0.55778E+00_JPRB, 0.53573E+00_JPRB, 0.52303E+00_JPRB, 0.51883E+00_JPRB, 0.51946E+00_JPRB ,&
&  0.52199E+00_JPRB, 0.52681E+00_JPRB, 0.53225E+00_JPRB, 0.55247E+00_JPRB, 0.68564E+00_JPRB, 0.10356E+01_JPRB, 0.15842E+01_JPRB ,&
&  0.18599E+01_JPRB, 0.19574E+01_JPRB, 0.20335E+01_JPRB, 0.20482E+01_JPRB, 0.19150E+01_JPRB, 0.17662E+01_JPRB, 0.16030E+01_JPRB ,&
&  0.14397E+01_JPRB, 0.12685E+01_JPRB, 0.11235E+01_JPRB, 0.10100E+01_JPRB, 0.92209E+00_JPRB, 0.86152E+00_JPRB, 0.82819E+00_JPRB ,&
&  0.81353E+00_JPRB, 0.80564E+00_JPRB, 0.79831E+00_JPRB, 0.78267E+00_JPRB, 0.76217E+00_JPRB, 0.74723E+00_JPRB, 0.73764E+00_JPRB ,&
&  0.72531E+00_JPRB, 0.71336E+00_JPRB, 0.70376E+00_JPRB, 0.69712E+00_JPRB, 0.69431E+00_JPRB, 0.69066E+00_JPRB, 0.69011E+00_JPRB ,&
&  0.69308E+00_JPRB, 0.70209E+00_JPRB, 0.69211E+00_JPRB, 0.69222E+00_JPRB, 0.74250E+00_JPRB, 0.76171E+00_JPRB, 0.78249E+00_JPRB ,&
&  0.80085E+00_JPRB, 0.79397E+00_JPRB, 0.77332E+00_JPRB, 0.76830E+00_JPRB, 0.87907E+00_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.73732E+00_JPRB, 0.70553E+00_JPRB, 0.66235E+00_JPRB, 0.63631E+00_JPRB, 0.61865E+00_JPRB, 0.59652E+00_JPRB, 0.58014E+00_JPRB ,&
&  0.56714E+00_JPRB, 0.55599E+00_JPRB, 0.54255E+00_JPRB, 0.52638E+00_JPRB, 0.51086E+00_JPRB, 0.49998E+00_JPRB, 0.49059E+00_JPRB ,&
&  0.48162E+00_JPRB, 0.47304E+00_JPRB, 0.46493E+00_JPRB, 0.45877E+00_JPRB, 0.45177E+00_JPRB, 0.44627E+00_JPRB, 0.44142E+00_JPRB ,&
&  0.43564E+00_JPRB, 0.42959E+00_JPRB, 0.42206E+00_JPRB, 0.42569E+00_JPRB, 0.43468E+00_JPRB, 0.44330E+00_JPRB, 0.45466E+00_JPRB ,&
&  0.49454E+00_JPRB, 0.50290E+00_JPRB, 0.55362E+00_JPRB, 0.61012E+00_JPRB, 0.64865E+00_JPRB, 0.68136E+00_JPRB, 0.68885E+00_JPRB ,&
&  0.67299E+00_JPRB, 0.68258E+00_JPRB, 0.67322E+00_JPRB, 0.64811E+00_JPRB, 0.62855E+00_JPRB, 0.58993E+00_JPRB, 0.55567E+00_JPRB ,&
&  0.53186E+00_JPRB, 0.51668E+00_JPRB, 0.50313E+00_JPRB, 0.48832E+00_JPRB, 0.47939E+00_JPRB, 0.47775E+00_JPRB, 0.48122E+00_JPRB ,&
&  0.48481E+00_JPRB, 0.49037E+00_JPRB, 0.49617E+00_JPRB, 0.50410E+00_JPRB, 0.52562E+00_JPRB, 0.63030E+00_JPRB, 0.90245E+00_JPRB ,&
&  0.11086E+01_JPRB, 0.12337E+01_JPRB, 0.13188E+01_JPRB, 0.13562E+01_JPRB, 0.13180E+01_JPRB, 0.12400E+01_JPRB, 0.11447E+01_JPRB ,&
&  0.10443E+01_JPRB, 0.93893E+00_JPRB, 0.84349E+00_JPRB, 0.76980E+00_JPRB, 0.71327E+00_JPRB, 0.67653E+00_JPRB, 0.65862E+00_JPRB ,&
&  0.65858E+00_JPRB, 0.67102E+00_JPRB, 0.68855E+00_JPRB, 0.70497E+00_JPRB, 0.71363E+00_JPRB, 0.71305E+00_JPRB, 0.70771E+00_JPRB ,&
&  0.69807E+00_JPRB, 0.68557E+00_JPRB, 0.67220E+00_JPRB, 0.66134E+00_JPRB, 0.65313E+00_JPRB, 0.64758E+00_JPRB, 0.64479E+00_JPRB ,&
&  0.64421E+00_JPRB, 0.64662E+00_JPRB, 0.65405E+00_JPRB, 0.67280E+00_JPRB, 0.68112E+00_JPRB, 0.69685E+00_JPRB, 0.70792E+00_JPRB ,&
&  0.72081E+00_JPRB, 0.72424E+00_JPRB, 0.71480E+00_JPRB, 0.69078E+00_JPRB, 0.70329E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.63950E+00_JPRB, 0.63130E+00_JPRB, 0.60978E+00_JPRB, 0.58610E+00_JPRB, 0.56585E+00_JPRB, 0.54711E+00_JPRB, 0.53129E+00_JPRB ,&
&  0.51872E+00_JPRB, 0.50758E+00_JPRB, 0.49557E+00_JPRB, 0.48430E+00_JPRB, 0.47306E+00_JPRB, 0.46398E+00_JPRB, 0.45515E+00_JPRB ,&
&  0.44913E+00_JPRB, 0.44488E+00_JPRB, 0.44066E+00_JPRB, 0.43540E+00_JPRB, 0.42983E+00_JPRB, 0.42420E+00_JPRB, 0.41784E+00_JPRB ,&
&  0.41270E+00_JPRB, 0.40989E+00_JPRB, 0.40359E+00_JPRB, 0.40162E+00_JPRB, 0.40815E+00_JPRB, 0.41705E+00_JPRB, 0.42697E+00_JPRB ,&
&  0.45733E+00_JPRB, 0.42696E+00_JPRB, 0.44042E+00_JPRB, 0.49911E+00_JPRB, 0.54088E+00_JPRB, 0.59026E+00_JPRB, 0.61704E+00_JPRB ,&
&  0.62549E+00_JPRB, 0.63377E+00_JPRB, 0.62090E+00_JPRB, 0.60068E+00_JPRB, 0.56794E+00_JPRB, 0.53924E+00_JPRB, 0.51722E+00_JPRB ,&
&  0.49816E+00_JPRB, 0.48176E+00_JPRB, 0.46868E+00_JPRB, 0.45789E+00_JPRB, 0.44857E+00_JPRB, 0.44470E+00_JPRB, 0.44584E+00_JPRB ,&
&  0.44811E+00_JPRB, 0.45107E+00_JPRB, 0.45537E+00_JPRB, 0.46319E+00_JPRB, 0.47377E+00_JPRB, 0.49768E+00_JPRB, 0.58753E+00_JPRB ,&
&  0.73184E+00_JPRB, 0.83969E+00_JPRB, 0.93342E+00_JPRB, 0.98746E+00_JPRB, 0.98617E+00_JPRB, 0.93974E+00_JPRB, 0.85294E+00_JPRB ,&
&  0.76064E+00_JPRB, 0.68971E+00_JPRB, 0.64655E+00_JPRB, 0.62094E+00_JPRB, 0.60763E+00_JPRB, 0.59845E+00_JPRB, 0.59126E+00_JPRB ,&
&  0.58492E+00_JPRB, 0.58188E+00_JPRB, 0.58504E+00_JPRB, 0.59391E+00_JPRB, 0.60647E+00_JPRB, 0.61652E+00_JPRB, 0.62016E+00_JPRB ,&
&  0.61861E+00_JPRB, 0.61396E+00_JPRB, 0.60950E+00_JPRB, 0.60431E+00_JPRB, 0.59824E+00_JPRB, 0.59093E+00_JPRB, 0.58685E+00_JPRB ,&
&  0.58718E+00_JPRB, 0.59140E+00_JPRB, 0.59988E+00_JPRB, 0.59954E+00_JPRB, 0.62228E+00_JPRB, 0.63147E+00_JPRB, 0.63742E+00_JPRB ,&
&  0.64227E+00_JPRB, 0.64050E+00_JPRB, 0.63719E+00_JPRB, 0.56394E+00_JPRB, 0.62285E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.54954E+00_JPRB, 0.54806E+00_JPRB, 0.53436E+00_JPRB, 0.51894E+00_JPRB, 0.50340E+00_JPRB, 0.48848E+00_JPRB, 0.47741E+00_JPRB ,&
&  0.46930E+00_JPRB, 0.45888E+00_JPRB, 0.44981E+00_JPRB, 0.44375E+00_JPRB, 0.43723E+00_JPRB, 0.43136E+00_JPRB, 0.42463E+00_JPRB ,&
&  0.41932E+00_JPRB, 0.41601E+00_JPRB, 0.41360E+00_JPRB, 0.41059E+00_JPRB, 0.40706E+00_JPRB, 0.40265E+00_JPRB, 0.39822E+00_JPRB ,&
&  0.39402E+00_JPRB, 0.39110E+00_JPRB, 0.38701E+00_JPRB, 0.38453E+00_JPRB, 0.38707E+00_JPRB, 0.39349E+00_JPRB, 0.39964E+00_JPRB ,&
&  0.42365E+00_JPRB, 0.37472E+00_JPRB, 0.38854E+00_JPRB, 0.41737E+00_JPRB, 0.44954E+00_JPRB, 0.49221E+00_JPRB, 0.52750E+00_JPRB ,&
&  0.54489E+00_JPRB, 0.56204E+00_JPRB, 0.55611E+00_JPRB, 0.54302E+00_JPRB, 0.51967E+00_JPRB, 0.49582E+00_JPRB, 0.47641E+00_JPRB ,&
&  0.46404E+00_JPRB, 0.45506E+00_JPRB, 0.44438E+00_JPRB, 0.43248E+00_JPRB, 0.42129E+00_JPRB, 0.41352E+00_JPRB, 0.41037E+00_JPRB ,&
&  0.41100E+00_JPRB, 0.41220E+00_JPRB, 0.41244E+00_JPRB, 0.41538E+00_JPRB, 0.42241E+00_JPRB, 0.43327E+00_JPRB, 0.45768E+00_JPRB ,&
&  0.51177E+00_JPRB, 0.58455E+00_JPRB, 0.64750E+00_JPRB, 0.70678E+00_JPRB, 0.74431E+00_JPRB, 0.75513E+00_JPRB, 0.71864E+00_JPRB ,&
&  0.66763E+00_JPRB, 0.62067E+00_JPRB, 0.59019E+00_JPRB, 0.57432E+00_JPRB, 0.56484E+00_JPRB, 0.55821E+00_JPRB, 0.54852E+00_JPRB ,&
&  0.53563E+00_JPRB, 0.52045E+00_JPRB, 0.50691E+00_JPRB, 0.49397E+00_JPRB, 0.48500E+00_JPRB, 0.48265E+00_JPRB, 0.48543E+00_JPRB ,&
&  0.48965E+00_JPRB, 0.49278E+00_JPRB, 0.49550E+00_JPRB, 0.49885E+00_JPRB, 0.50173E+00_JPRB, 0.50467E+00_JPRB, 0.50696E+00_JPRB ,&
&  0.51147E+00_JPRB, 0.51761E+00_JPRB, 0.52443E+00_JPRB, 0.53733E+00_JPRB, 0.54725E+00_JPRB, 0.55387E+00_JPRB, 0.55860E+00_JPRB ,&
&  0.55992E+00_JPRB, 0.56510E+00_JPRB, 0.48710E+00_JPRB, 0.52259E+00_JPRB, 0.54020E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.48329E+00_JPRB, 0.48236E+00_JPRB, 0.47765E+00_JPRB, 0.46637E+00_JPRB, 0.45344E+00_JPRB, 0.44231E+00_JPRB, 0.43529E+00_JPRB ,&
&  0.43090E+00_JPRB, 0.42473E+00_JPRB, 0.41948E+00_JPRB, 0.41600E+00_JPRB, 0.41163E+00_JPRB, 0.40663E+00_JPRB, 0.39946E+00_JPRB ,&
&  0.39348E+00_JPRB, 0.38881E+00_JPRB, 0.38536E+00_JPRB, 0.38330E+00_JPRB, 0.38212E+00_JPRB, 0.38078E+00_JPRB, 0.37859E+00_JPRB ,&
&  0.37632E+00_JPRB, 0.37494E+00_JPRB, 0.37365E+00_JPRB, 0.37315E+00_JPRB, 0.37354E+00_JPRB, 0.37731E+00_JPRB, 0.38331E+00_JPRB ,&
&  0.37112E+00_JPRB, 0.34566E+00_JPRB, 0.35791E+00_JPRB, 0.37416E+00_JPRB, 0.39352E+00_JPRB, 0.42514E+00_JPRB, 0.45855E+00_JPRB ,&
&  0.47794E+00_JPRB, 0.48967E+00_JPRB, 0.48253E+00_JPRB, 0.47110E+00_JPRB, 0.45853E+00_JPRB, 0.44557E+00_JPRB, 0.43520E+00_JPRB ,&
&  0.42548E+00_JPRB, 0.41871E+00_JPRB, 0.41281E+00_JPRB, 0.40431E+00_JPRB, 0.39404E+00_JPRB, 0.38562E+00_JPRB, 0.38057E+00_JPRB ,&
&  0.37883E+00_JPRB, 0.37960E+00_JPRB, 0.38085E+00_JPRB, 0.38241E+00_JPRB, 0.38560E+00_JPRB, 0.39195E+00_JPRB, 0.40177E+00_JPRB ,&
&  0.41469E+00_JPRB, 0.43949E+00_JPRB, 0.47475E+00_JPRB, 0.50922E+00_JPRB, 0.54208E+00_JPRB, 0.56552E+00_JPRB, 0.57936E+00_JPRB ,&
&  0.57980E+00_JPRB, 0.56699E+00_JPRB, 0.54906E+00_JPRB, 0.53021E+00_JPRB, 0.50783E+00_JPRB, 0.48092E+00_JPRB, 0.45231E+00_JPRB ,&
&  0.42849E+00_JPRB, 0.41050E+00_JPRB, 0.39891E+00_JPRB, 0.39377E+00_JPRB, 0.39216E+00_JPRB, 0.39427E+00_JPRB, 0.39856E+00_JPRB ,&
&  0.40410E+00_JPRB, 0.41017E+00_JPRB, 0.41482E+00_JPRB, 0.42022E+00_JPRB, 0.42530E+00_JPRB, 0.43095E+00_JPRB, 0.43700E+00_JPRB ,&
&  0.44383E+00_JPRB, 0.45035E+00_JPRB, 0.45726E+00_JPRB, 0.46817E+00_JPRB, 0.47698E+00_JPRB, 0.48511E+00_JPRB, 0.48972E+00_JPRB ,&
&  0.48816E+00_JPRB, 0.48716E+00_JPRB, 0.48563E+00_JPRB, 0.48258E+00_JPRB, 0.48458E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.43560E+00_JPRB, 0.43350E+00_JPRB, 0.43146E+00_JPRB, 0.42846E+00_JPRB, 0.42346E+00_JPRB, 0.41625E+00_JPRB, 0.41056E+00_JPRB ,&
&  0.40688E+00_JPRB, 0.40288E+00_JPRB, 0.39955E+00_JPRB, 0.39547E+00_JPRB, 0.39053E+00_JPRB, 0.38444E+00_JPRB, 0.37630E+00_JPRB ,&
&  0.36881E+00_JPRB, 0.36190E+00_JPRB, 0.35743E+00_JPRB, 0.35538E+00_JPRB, 0.35547E+00_JPRB, 0.35564E+00_JPRB, 0.35411E+00_JPRB ,&
&  0.35317E+00_JPRB, 0.35313E+00_JPRB, 0.35306E+00_JPRB, 0.35302E+00_JPRB, 0.35332E+00_JPRB, 0.35652E+00_JPRB, 0.36020E+00_JPRB ,&
&  0.36572E+00_JPRB, 0.34217E+00_JPRB, 0.35271E+00_JPRB, 0.35662E+00_JPRB, 0.36714E+00_JPRB, 0.38561E+00_JPRB, 0.40759E+00_JPRB ,&
&  0.42443E+00_JPRB, 0.42529E+00_JPRB, 0.41874E+00_JPRB, 0.40940E+00_JPRB, 0.40294E+00_JPRB, 0.39678E+00_JPRB, 0.39003E+00_JPRB ,&
&  0.38353E+00_JPRB, 0.37911E+00_JPRB, 0.37523E+00_JPRB, 0.37134E+00_JPRB, 0.36588E+00_JPRB, 0.35894E+00_JPRB, 0.35220E+00_JPRB ,&
&  0.34739E+00_JPRB, 0.34565E+00_JPRB, 0.34645E+00_JPRB, 0.34843E+00_JPRB, 0.35099E+00_JPRB, 0.35545E+00_JPRB, 0.36120E+00_JPRB ,&
&  0.36778E+00_JPRB, 0.37218E+00_JPRB, 0.37803E+00_JPRB, 0.38947E+00_JPRB, 0.40270E+00_JPRB, 0.41342E+00_JPRB, 0.42024E+00_JPRB ,&
&  0.42340E+00_JPRB, 0.42085E+00_JPRB, 0.41444E+00_JPRB, 0.40509E+00_JPRB, 0.39428E+00_JPRB, 0.38317E+00_JPRB, 0.37348E+00_JPRB ,&
&  0.36569E+00_JPRB, 0.35896E+00_JPRB, 0.35315E+00_JPRB, 0.34909E+00_JPRB, 0.34731E+00_JPRB, 0.34716E+00_JPRB, 0.34850E+00_JPRB ,&
&  0.35221E+00_JPRB, 0.35697E+00_JPRB, 0.36114E+00_JPRB, 0.36533E+00_JPRB, 0.36921E+00_JPRB, 0.37267E+00_JPRB, 0.37727E+00_JPRB ,&
&  0.38348E+00_JPRB, 0.38832E+00_JPRB, 0.39261E+00_JPRB, 0.39853E+00_JPRB, 0.40556E+00_JPRB, 0.41321E+00_JPRB, 0.42042E+00_JPRB ,&
&  0.42543E+00_JPRB, 0.42806E+00_JPRB, 0.43127E+00_JPRB, 0.43293E+00_JPRB, 0.43616E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.40428E+00_JPRB, 0.40295E+00_JPRB, 0.40050E+00_JPRB, 0.39894E+00_JPRB, 0.39656E+00_JPRB, 0.39298E+00_JPRB, 0.38891E+00_JPRB ,&
&  0.38559E+00_JPRB, 0.38251E+00_JPRB, 0.37931E+00_JPRB, 0.37492E+00_JPRB, 0.36886E+00_JPRB, 0.36291E+00_JPRB, 0.35588E+00_JPRB ,&
&  0.34918E+00_JPRB, 0.34332E+00_JPRB, 0.33931E+00_JPRB, 0.33701E+00_JPRB, 0.33511E+00_JPRB, 0.33354E+00_JPRB, 0.33167E+00_JPRB ,&
&  0.33062E+00_JPRB, 0.33032E+00_JPRB, 0.33043E+00_JPRB, 0.33099E+00_JPRB, 0.33170E+00_JPRB, 0.33442E+00_JPRB, 0.33620E+00_JPRB ,&
&  0.33859E+00_JPRB, 0.33773E+00_JPRB, 0.34479E+00_JPRB, 0.34922E+00_JPRB, 0.35630E+00_JPRB, 0.36276E+00_JPRB, 0.36873E+00_JPRB ,&
&  0.37394E+00_JPRB, 0.37642E+00_JPRB, 0.37242E+00_JPRB, 0.36635E+00_JPRB, 0.36194E+00_JPRB, 0.35955E+00_JPRB, 0.35454E+00_JPRB ,&
&  0.34938E+00_JPRB, 0.34560E+00_JPRB, 0.34259E+00_JPRB, 0.34045E+00_JPRB, 0.33821E+00_JPRB, 0.33490E+00_JPRB, 0.33124E+00_JPRB ,&
&  0.32660E+00_JPRB, 0.32314E+00_JPRB, 0.32044E+00_JPRB, 0.31942E+00_JPRB, 0.31967E+00_JPRB, 0.32098E+00_JPRB, 0.32303E+00_JPRB ,&
&  0.32643E+00_JPRB, 0.32996E+00_JPRB, 0.33190E+00_JPRB, 0.33310E+00_JPRB, 0.33475E+00_JPRB, 0.33754E+00_JPRB, 0.33984E+00_JPRB ,&
&  0.34046E+00_JPRB, 0.33969E+00_JPRB, 0.33766E+00_JPRB, 0.33453E+00_JPRB, 0.33066E+00_JPRB, 0.32741E+00_JPRB, 0.32467E+00_JPRB ,&
&  0.32205E+00_JPRB, 0.31922E+00_JPRB, 0.31637E+00_JPRB, 0.31478E+00_JPRB, 0.31389E+00_JPRB, 0.31361E+00_JPRB, 0.31471E+00_JPRB ,&
&  0.31722E+00_JPRB, 0.32031E+00_JPRB, 0.32383E+00_JPRB, 0.32719E+00_JPRB, 0.33038E+00_JPRB, 0.33308E+00_JPRB, 0.33643E+00_JPRB ,&
&  0.34179E+00_JPRB, 0.34770E+00_JPRB, 0.35356E+00_JPRB, 0.35912E+00_JPRB, 0.36530E+00_JPRB, 0.37263E+00_JPRB, 0.38044E+00_JPRB ,&
&  0.38582E+00_JPRB, 0.38950E+00_JPRB, 0.39386E+00_JPRB, 0.39866E+00_JPRB, 0.40288E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.36556E+00_JPRB, 0.36737E+00_JPRB, 0.36794E+00_JPRB, 0.36881E+00_JPRB, 0.36874E+00_JPRB, 0.36730E+00_JPRB, 0.36428E+00_JPRB ,&
&  0.36088E+00_JPRB, 0.35698E+00_JPRB, 0.35307E+00_JPRB, 0.34797E+00_JPRB, 0.34258E+00_JPRB, 0.33811E+00_JPRB, 0.33314E+00_JPRB ,&
&  0.32801E+00_JPRB, 0.32323E+00_JPRB, 0.31989E+00_JPRB, 0.31721E+00_JPRB, 0.31461E+00_JPRB, 0.31212E+00_JPRB, 0.30888E+00_JPRB ,&
&  0.30607E+00_JPRB, 0.30345E+00_JPRB, 0.30381E+00_JPRB, 0.30635E+00_JPRB, 0.30844E+00_JPRB, 0.30992E+00_JPRB, 0.31120E+00_JPRB ,&
&  0.31153E+00_JPRB, 0.31878E+00_JPRB, 0.31300E+00_JPRB, 0.32267E+00_JPRB, 0.32527E+00_JPRB, 0.32588E+00_JPRB, 0.32625E+00_JPRB ,&
&  0.32848E+00_JPRB, 0.33107E+00_JPRB, 0.33277E+00_JPRB, 0.32963E+00_JPRB, 0.32516E+00_JPRB, 0.32168E+00_JPRB, 0.31780E+00_JPRB ,&
&  0.31257E+00_JPRB, 0.31045E+00_JPRB, 0.31047E+00_JPRB, 0.31083E+00_JPRB, 0.31097E+00_JPRB, 0.31054E+00_JPRB, 0.30941E+00_JPRB ,&
&  0.30780E+00_JPRB, 0.30543E+00_JPRB, 0.30263E+00_JPRB, 0.29985E+00_JPRB, 0.29809E+00_JPRB, 0.29608E+00_JPRB, 0.29427E+00_JPRB ,&
&  0.29362E+00_JPRB, 0.29432E+00_JPRB, 0.29511E+00_JPRB, 0.29539E+00_JPRB, 0.29489E+00_JPRB, 0.29458E+00_JPRB, 0.29472E+00_JPRB ,&
&  0.29446E+00_JPRB, 0.29391E+00_JPRB, 0.29303E+00_JPRB, 0.29161E+00_JPRB, 0.28995E+00_JPRB, 0.28899E+00_JPRB, 0.28872E+00_JPRB ,&
&  0.28814E+00_JPRB, 0.28644E+00_JPRB, 0.28401E+00_JPRB, 0.28352E+00_JPRB, 0.28353E+00_JPRB, 0.28387E+00_JPRB, 0.28448E+00_JPRB ,&
&  0.28581E+00_JPRB, 0.28822E+00_JPRB, 0.29096E+00_JPRB, 0.29370E+00_JPRB, 0.29644E+00_JPRB, 0.30020E+00_JPRB, 0.30459E+00_JPRB ,&
&  0.30944E+00_JPRB, 0.31469E+00_JPRB, 0.31989E+00_JPRB, 0.32494E+00_JPRB, 0.32927E+00_JPRB, 0.33413E+00_JPRB, 0.34021E+00_JPRB ,&
&  0.34527E+00_JPRB, 0.34923E+00_JPRB, 0.35268E+00_JPRB, 0.35706E+00_JPRB, 0.36187E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.33327E+00_JPRB, 0.33810E+00_JPRB, 0.34090E+00_JPRB, 0.34232E+00_JPRB, 0.34278E+00_JPRB, 0.34140E+00_JPRB, 0.33891E+00_JPRB ,&
&  0.33504E+00_JPRB, 0.33115E+00_JPRB, 0.32689E+00_JPRB, 0.32210E+00_JPRB, 0.31776E+00_JPRB, 0.31407E+00_JPRB, 0.31082E+00_JPRB ,&
&  0.30751E+00_JPRB, 0.30445E+00_JPRB, 0.30194E+00_JPRB, 0.29939E+00_JPRB, 0.29640E+00_JPRB, 0.29260E+00_JPRB, 0.28771E+00_JPRB ,&
&  0.28477E+00_JPRB, 0.28247E+00_JPRB, 0.28336E+00_JPRB, 0.28625E+00_JPRB, 0.28851E+00_JPRB, 0.28986E+00_JPRB, 0.29094E+00_JPRB ,&
&  0.29243E+00_JPRB, 0.29409E+00_JPRB, 0.29551E+00_JPRB, 0.29821E+00_JPRB, 0.30065E+00_JPRB, 0.30037E+00_JPRB, 0.29867E+00_JPRB ,&
&  0.29726E+00_JPRB, 0.29764E+00_JPRB, 0.29855E+00_JPRB, 0.29893E+00_JPRB, 0.29813E+00_JPRB, 0.29573E+00_JPRB, 0.29185E+00_JPRB ,&
&  0.28525E+00_JPRB, 0.28037E+00_JPRB, 0.27929E+00_JPRB, 0.28043E+00_JPRB, 0.28236E+00_JPRB, 0.28384E+00_JPRB, 0.28475E+00_JPRB ,&
&  0.28504E+00_JPRB, 0.28446E+00_JPRB, 0.28333E+00_JPRB, 0.28197E+00_JPRB, 0.28017E+00_JPRB, 0.27811E+00_JPRB, 0.27690E+00_JPRB ,&
&  0.27558E+00_JPRB, 0.27446E+00_JPRB, 0.27379E+00_JPRB, 0.27344E+00_JPRB, 0.27236E+00_JPRB, 0.27074E+00_JPRB, 0.26981E+00_JPRB ,&
&  0.26855E+00_JPRB, 0.26785E+00_JPRB, 0.26681E+00_JPRB, 0.26578E+00_JPRB, 0.26444E+00_JPRB, 0.26313E+00_JPRB, 0.26269E+00_JPRB ,&
&  0.26275E+00_JPRB, 0.26160E+00_JPRB, 0.26024E+00_JPRB, 0.26020E+00_JPRB, 0.26086E+00_JPRB, 0.26185E+00_JPRB, 0.26264E+00_JPRB ,&
&  0.26341E+00_JPRB, 0.26440E+00_JPRB, 0.26656E+00_JPRB, 0.26904E+00_JPRB, 0.27188E+00_JPRB, 0.27531E+00_JPRB, 0.27920E+00_JPRB ,&
&  0.28375E+00_JPRB, 0.28901E+00_JPRB, 0.29488E+00_JPRB, 0.30037E+00_JPRB, 0.30509E+00_JPRB, 0.30827E+00_JPRB, 0.31219E+00_JPRB ,&
&  0.31681E+00_JPRB, 0.31988E+00_JPRB, 0.32237E+00_JPRB, 0.32533E+00_JPRB, 0.32860E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.30167E+00_JPRB, 0.30662E+00_JPRB, 0.31045E+00_JPRB, 0.31270E+00_JPRB, 0.31291E+00_JPRB, 0.31176E+00_JPRB, 0.30920E+00_JPRB ,&
&  0.30540E+00_JPRB, 0.30154E+00_JPRB, 0.29770E+00_JPRB, 0.29458E+00_JPRB, 0.29264E+00_JPRB, 0.29108E+00_JPRB, 0.28979E+00_JPRB ,&
&  0.28795E+00_JPRB, 0.28570E+00_JPRB, 0.28292E+00_JPRB, 0.28047E+00_JPRB, 0.27749E+00_JPRB, 0.27332E+00_JPRB, 0.27024E+00_JPRB ,&
&  0.26809E+00_JPRB, 0.26603E+00_JPRB, 0.26538E+00_JPRB, 0.26661E+00_JPRB, 0.26898E+00_JPRB, 0.27132E+00_JPRB, 0.27388E+00_JPRB ,&
&  0.27566E+00_JPRB, 0.27663E+00_JPRB, 0.27736E+00_JPRB, 0.27721E+00_JPRB, 0.27717E+00_JPRB, 0.27685E+00_JPRB, 0.27614E+00_JPRB ,&
&  0.27532E+00_JPRB, 0.27454E+00_JPRB, 0.27319E+00_JPRB, 0.27287E+00_JPRB, 0.27258E+00_JPRB, 0.27125E+00_JPRB, 0.26900E+00_JPRB ,&
&  0.26543E+00_JPRB, 0.26059E+00_JPRB, 0.25678E+00_JPRB, 0.25490E+00_JPRB, 0.25435E+00_JPRB, 0.25469E+00_JPRB, 0.25561E+00_JPRB ,&
&  0.25681E+00_JPRB, 0.25847E+00_JPRB, 0.26000E+00_JPRB, 0.26075E+00_JPRB, 0.26062E+00_JPRB, 0.26046E+00_JPRB, 0.25926E+00_JPRB ,&
&  0.25808E+00_JPRB, 0.25732E+00_JPRB, 0.25678E+00_JPRB, 0.25618E+00_JPRB, 0.25409E+00_JPRB, 0.25150E+00_JPRB, 0.25092E+00_JPRB ,&
&  0.24947E+00_JPRB, 0.24852E+00_JPRB, 0.24713E+00_JPRB, 0.24642E+00_JPRB, 0.24555E+00_JPRB, 0.24426E+00_JPRB, 0.24290E+00_JPRB ,&
&  0.24182E+00_JPRB, 0.24126E+00_JPRB, 0.24095E+00_JPRB, 0.24020E+00_JPRB, 0.24074E+00_JPRB, 0.24164E+00_JPRB, 0.24314E+00_JPRB ,&
&  0.24435E+00_JPRB, 0.24551E+00_JPRB, 0.24714E+00_JPRB, 0.24883E+00_JPRB, 0.25142E+00_JPRB, 0.25446E+00_JPRB, 0.25795E+00_JPRB ,&
&  0.26091E+00_JPRB, 0.26487E+00_JPRB, 0.26937E+00_JPRB, 0.27359E+00_JPRB, 0.27694E+00_JPRB, 0.27933E+00_JPRB, 0.28251E+00_JPRB ,&
&  0.28757E+00_JPRB, 0.29155E+00_JPRB, 0.29335E+00_JPRB, 0.29556E+00_JPRB, 0.29771E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.27843E+00_JPRB, 0.28006E+00_JPRB, 0.28177E+00_JPRB, 0.28373E+00_JPRB, 0.28508E+00_JPRB, 0.28561E+00_JPRB, 0.28428E+00_JPRB ,&
&  0.28171E+00_JPRB, 0.27876E+00_JPRB, 0.27579E+00_JPRB, 0.27328E+00_JPRB, 0.27183E+00_JPRB, 0.27099E+00_JPRB, 0.27050E+00_JPRB ,&
&  0.27009E+00_JPRB, 0.26962E+00_JPRB, 0.26772E+00_JPRB, 0.26487E+00_JPRB, 0.26112E+00_JPRB, 0.25789E+00_JPRB, 0.25693E+00_JPRB ,&
&  0.25534E+00_JPRB, 0.25419E+00_JPRB, 0.25336E+00_JPRB, 0.25249E+00_JPRB, 0.25214E+00_JPRB, 0.25309E+00_JPRB, 0.25489E+00_JPRB ,&
&  0.25679E+00_JPRB, 0.25835E+00_JPRB, 0.25957E+00_JPRB, 0.26041E+00_JPRB, 0.25986E+00_JPRB, 0.25878E+00_JPRB, 0.25748E+00_JPRB ,&
&  0.25587E+00_JPRB, 0.25429E+00_JPRB, 0.25288E+00_JPRB, 0.25102E+00_JPRB, 0.24834E+00_JPRB, 0.24512E+00_JPRB, 0.24280E+00_JPRB ,&
&  0.24102E+00_JPRB, 0.23937E+00_JPRB, 0.23769E+00_JPRB, 0.23681E+00_JPRB, 0.23618E+00_JPRB, 0.23593E+00_JPRB, 0.23601E+00_JPRB ,&
&  0.23646E+00_JPRB, 0.23774E+00_JPRB, 0.23950E+00_JPRB, 0.24143E+00_JPRB, 0.24202E+00_JPRB, 0.24222E+00_JPRB, 0.24247E+00_JPRB ,&
&  0.24182E+00_JPRB, 0.24052E+00_JPRB, 0.23874E+00_JPRB, 0.23674E+00_JPRB, 0.23544E+00_JPRB, 0.23447E+00_JPRB, 0.23339E+00_JPRB ,&
&  0.23148E+00_JPRB, 0.23027E+00_JPRB, 0.22961E+00_JPRB, 0.22946E+00_JPRB, 0.22963E+00_JPRB, 0.22964E+00_JPRB, 0.22891E+00_JPRB ,&
&  0.22803E+00_JPRB, 0.22728E+00_JPRB, 0.22637E+00_JPRB, 0.22550E+00_JPRB, 0.22397E+00_JPRB, 0.22454E+00_JPRB, 0.22560E+00_JPRB ,&
&  0.22602E+00_JPRB, 0.22613E+00_JPRB, 0.22638E+00_JPRB, 0.22768E+00_JPRB, 0.22916E+00_JPRB, 0.23110E+00_JPRB, 0.23374E+00_JPRB ,&
&  0.23596E+00_JPRB, 0.23888E+00_JPRB, 0.24308E+00_JPRB, 0.24700E+00_JPRB, 0.24986E+00_JPRB, 0.25127E+00_JPRB, 0.25406E+00_JPRB ,&
&  0.26002E+00_JPRB, 0.26623E+00_JPRB, 0.27002E+00_JPRB, 0.27373E+00_JPRB, 0.27706E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.25602E+00_JPRB, 0.25677E+00_JPRB, 0.25753E+00_JPRB, 0.25861E+00_JPRB, 0.26019E+00_JPRB, 0.26114E+00_JPRB, 0.26190E+00_JPRB ,&
&  0.26131E+00_JPRB, 0.25961E+00_JPRB, 0.25761E+00_JPRB, 0.25548E+00_JPRB, 0.25458E+00_JPRB, 0.25456E+00_JPRB, 0.25479E+00_JPRB ,&
&  0.25441E+00_JPRB, 0.25432E+00_JPRB, 0.25261E+00_JPRB, 0.24963E+00_JPRB, 0.24609E+00_JPRB, 0.24365E+00_JPRB, 0.24297E+00_JPRB ,&
&  0.24241E+00_JPRB, 0.24148E+00_JPRB, 0.24102E+00_JPRB, 0.24044E+00_JPRB, 0.23820E+00_JPRB, 0.23703E+00_JPRB, 0.23738E+00_JPRB ,&
&  0.23815E+00_JPRB, 0.23955E+00_JPRB, 0.23973E+00_JPRB, 0.23730E+00_JPRB, 0.23951E+00_JPRB, 0.23924E+00_JPRB, 0.23734E+00_JPRB ,&
&  0.23491E+00_JPRB, 0.23239E+00_JPRB, 0.23010E+00_JPRB, 0.22783E+00_JPRB, 0.22552E+00_JPRB, 0.22375E+00_JPRB, 0.22257E+00_JPRB ,&
&  0.22222E+00_JPRB, 0.22238E+00_JPRB, 0.22283E+00_JPRB, 0.22375E+00_JPRB, 0.22439E+00_JPRB, 0.22475E+00_JPRB, 0.22480E+00_JPRB ,&
&  0.22493E+00_JPRB, 0.22513E+00_JPRB, 0.22553E+00_JPRB, 0.22576E+00_JPRB, 0.22603E+00_JPRB, 0.22626E+00_JPRB, 0.22613E+00_JPRB ,&
&  0.22505E+00_JPRB, 0.22324E+00_JPRB, 0.22081E+00_JPRB, 0.21988E+00_JPRB, 0.21981E+00_JPRB, 0.21827E+00_JPRB, 0.21584E+00_JPRB ,&
&  0.21355E+00_JPRB, 0.21309E+00_JPRB, 0.21427E+00_JPRB, 0.21553E+00_JPRB, 0.21653E+00_JPRB, 0.21686E+00_JPRB, 0.21657E+00_JPRB ,&
&  0.21557E+00_JPRB, 0.21338E+00_JPRB, 0.21085E+00_JPRB, 0.20888E+00_JPRB, 0.20779E+00_JPRB, 0.20785E+00_JPRB, 0.20880E+00_JPRB ,&
&  0.20896E+00_JPRB, 0.20907E+00_JPRB, 0.21021E+00_JPRB, 0.21125E+00_JPRB, 0.21204E+00_JPRB, 0.21297E+00_JPRB, 0.21450E+00_JPRB ,&
&  0.21658E+00_JPRB, 0.21994E+00_JPRB, 0.22404E+00_JPRB, 0.22854E+00_JPRB, 0.23169E+00_JPRB, 0.23100E+00_JPRB, 0.23187E+00_JPRB ,&
&  0.23669E+00_JPRB, 0.24234E+00_JPRB, 0.24686E+00_JPRB, 0.25037E+00_JPRB, 0.25361E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.23700E+00_JPRB, 0.23808E+00_JPRB, 0.23902E+00_JPRB, 0.23984E+00_JPRB, 0.24140E+00_JPRB, 0.24264E+00_JPRB, 0.24422E+00_JPRB ,&
&  0.24535E+00_JPRB, 0.24409E+00_JPRB, 0.24244E+00_JPRB, 0.24110E+00_JPRB, 0.24059E+00_JPRB, 0.23998E+00_JPRB, 0.23949E+00_JPRB ,&
&  0.24025E+00_JPRB, 0.23964E+00_JPRB, 0.23704E+00_JPRB, 0.23509E+00_JPRB, 0.23182E+00_JPRB, 0.22967E+00_JPRB, 0.22926E+00_JPRB ,&
&  0.23085E+00_JPRB, 0.23052E+00_JPRB, 0.22978E+00_JPRB, 0.22866E+00_JPRB, 0.22718E+00_JPRB, 0.22639E+00_JPRB, 0.22522E+00_JPRB ,&
&  0.22453E+00_JPRB, 0.22344E+00_JPRB, 0.21876E+00_JPRB, 0.21929E+00_JPRB, 0.21889E+00_JPRB, 0.21885E+00_JPRB, 0.21829E+00_JPRB ,&
&  0.21764E+00_JPRB, 0.21661E+00_JPRB, 0.21481E+00_JPRB, 0.21282E+00_JPRB, 0.21114E+00_JPRB, 0.21021E+00_JPRB, 0.21009E+00_JPRB ,&
&  0.21047E+00_JPRB, 0.21150E+00_JPRB, 0.21318E+00_JPRB, 0.21528E+00_JPRB, 0.21619E+00_JPRB, 0.21685E+00_JPRB, 0.21787E+00_JPRB ,&
&  0.21829E+00_JPRB, 0.21834E+00_JPRB, 0.21830E+00_JPRB, 0.21816E+00_JPRB, 0.21777E+00_JPRB, 0.21685E+00_JPRB, 0.21612E+00_JPRB ,&
&  0.21443E+00_JPRB, 0.21198E+00_JPRB, 0.21029E+00_JPRB, 0.20822E+00_JPRB, 0.20257E+00_JPRB, 0.19238E+00_JPRB, 0.18671E+00_JPRB ,&
&  0.18694E+00_JPRB, 0.19165E+00_JPRB, 0.19723E+00_JPRB, 0.20052E+00_JPRB, 0.20585E+00_JPRB, 0.20602E+00_JPRB, 0.20229E+00_JPRB ,&
&  0.19383E+00_JPRB, 0.18882E+00_JPRB, 0.18522E+00_JPRB, 0.18102E+00_JPRB, 0.17850E+00_JPRB, 0.17812E+00_JPRB, 0.17856E+00_JPRB ,&
&  0.18062E+00_JPRB, 0.18346E+00_JPRB, 0.18638E+00_JPRB, 0.18805E+00_JPRB, 0.18825E+00_JPRB, 0.18703E+00_JPRB, 0.18552E+00_JPRB ,&
&  0.18531E+00_JPRB, 0.18940E+00_JPRB, 0.19771E+00_JPRB, 0.20659E+00_JPRB, 0.21101E+00_JPRB, 0.21285E+00_JPRB, 0.21473E+00_JPRB ,&
&  0.21852E+00_JPRB, 0.22354E+00_JPRB, 0.22852E+00_JPRB, 0.23198E+00_JPRB, 0.23480E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.21786E+00_JPRB, 0.21897E+00_JPRB, 0.22013E+00_JPRB, 0.22130E+00_JPRB, 0.22258E+00_JPRB, 0.22406E+00_JPRB, 0.22535E+00_JPRB ,&
&  0.22682E+00_JPRB, 0.22778E+00_JPRB, 0.22800E+00_JPRB, 0.22695E+00_JPRB, 0.22619E+00_JPRB, 0.22621E+00_JPRB, 0.22698E+00_JPRB ,&
&  0.22509E+00_JPRB, 0.22222E+00_JPRB, 0.22101E+00_JPRB, 0.22081E+00_JPRB, 0.22020E+00_JPRB, 0.21991E+00_JPRB, 0.22067E+00_JPRB ,&
&  0.22160E+00_JPRB, 0.22112E+00_JPRB, 0.22066E+00_JPRB, 0.21938E+00_JPRB, 0.21835E+00_JPRB, 0.21753E+00_JPRB, 0.21641E+00_JPRB ,&
&  0.21290E+00_JPRB, 0.20291E+00_JPRB, 0.19652E+00_JPRB, 0.20003E+00_JPRB, 0.20672E+00_JPRB, 0.20599E+00_JPRB, 0.20605E+00_JPRB ,&
&  0.20563E+00_JPRB, 0.20432E+00_JPRB, 0.20284E+00_JPRB, 0.20146E+00_JPRB, 0.20090E+00_JPRB, 0.20123E+00_JPRB, 0.20214E+00_JPRB ,&
&  0.20323E+00_JPRB, 0.20555E+00_JPRB, 0.20809E+00_JPRB, 0.20919E+00_JPRB, 0.21031E+00_JPRB, 0.21086E+00_JPRB, 0.21103E+00_JPRB ,&
&  0.21052E+00_JPRB, 0.20936E+00_JPRB, 0.20844E+00_JPRB, 0.20753E+00_JPRB, 0.20666E+00_JPRB, 0.20538E+00_JPRB, 0.20150E+00_JPRB ,&
&  0.19624E+00_JPRB, 0.19096E+00_JPRB, 0.18456E+00_JPRB, 0.17816E+00_JPRB, 0.17248E+00_JPRB, 0.16835E+00_JPRB, 0.16492E+00_JPRB ,&
&  0.16451E+00_JPRB, 0.16844E+00_JPRB, 0.17637E+00_JPRB, 0.18592E+00_JPRB, 0.19117E+00_JPRB, 0.18156E+00_JPRB, 0.17553E+00_JPRB ,&
&  0.16973E+00_JPRB, 0.16511E+00_JPRB, 0.16235E+00_JPRB, 0.16093E+00_JPRB, 0.15997E+00_JPRB, 0.16003E+00_JPRB, 0.16069E+00_JPRB ,&
&  0.16207E+00_JPRB, 0.16354E+00_JPRB, 0.16491E+00_JPRB, 0.16579E+00_JPRB, 0.16613E+00_JPRB, 0.16599E+00_JPRB, 0.16560E+00_JPRB ,&
&  0.16614E+00_JPRB, 0.16855E+00_JPRB, 0.17269E+00_JPRB, 0.17752E+00_JPRB, 0.18177E+00_JPRB, 0.18587E+00_JPRB, 0.19144E+00_JPRB ,&
&  0.19843E+00_JPRB, 0.20390E+00_JPRB, 0.20855E+00_JPRB, 0.21339E+00_JPRB, 0.21616E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.20340E+00_JPRB, 0.20539E+00_JPRB, 0.20605E+00_JPRB, 0.20728E+00_JPRB, 0.20763E+00_JPRB, 0.20886E+00_JPRB, 0.20980E+00_JPRB ,&
&  0.21215E+00_JPRB, 0.21431E+00_JPRB, 0.21478E+00_JPRB, 0.21293E+00_JPRB, 0.21173E+00_JPRB, 0.20877E+00_JPRB, 0.20760E+00_JPRB ,&
&  0.20728E+00_JPRB, 0.20828E+00_JPRB, 0.20846E+00_JPRB, 0.20854E+00_JPRB, 0.20849E+00_JPRB, 0.20844E+00_JPRB, 0.20895E+00_JPRB ,&
&  0.20688E+00_JPRB, 0.20591E+00_JPRB, 0.20792E+00_JPRB, 0.20696E+00_JPRB, 0.20617E+00_JPRB, 0.20472E+00_JPRB, 0.20391E+00_JPRB ,&
&  0.19978E+00_JPRB, 0.19395E+00_JPRB, 0.18745E+00_JPRB, 0.19059E+00_JPRB, 0.19771E+00_JPRB, 0.19525E+00_JPRB, 0.19457E+00_JPRB ,&
&  0.19408E+00_JPRB, 0.19299E+00_JPRB, 0.19263E+00_JPRB, 0.19223E+00_JPRB, 0.19296E+00_JPRB, 0.19370E+00_JPRB, 0.19556E+00_JPRB ,&
&  0.19812E+00_JPRB, 0.20168E+00_JPRB, 0.20158E+00_JPRB, 0.19519E+00_JPRB, 0.19214E+00_JPRB, 0.19106E+00_JPRB, 0.18767E+00_JPRB ,&
&  0.18366E+00_JPRB, 0.18224E+00_JPRB, 0.18111E+00_JPRB, 0.17836E+00_JPRB, 0.17745E+00_JPRB, 0.17695E+00_JPRB, 0.17440E+00_JPRB ,&
&  0.17111E+00_JPRB, 0.16866E+00_JPRB, 0.16526E+00_JPRB, 0.16178E+00_JPRB, 0.15940E+00_JPRB, 0.15831E+00_JPRB, 0.15844E+00_JPRB ,&
&  0.16033E+00_JPRB, 0.16224E+00_JPRB, 0.16811E+00_JPRB, 0.17183E+00_JPRB, 0.16868E+00_JPRB, 0.16558E+00_JPRB, 0.16210E+00_JPRB ,&
&  0.15669E+00_JPRB, 0.15422E+00_JPRB, 0.15227E+00_JPRB, 0.15087E+00_JPRB, 0.15079E+00_JPRB, 0.15157E+00_JPRB, 0.15213E+00_JPRB ,&
&  0.15289E+00_JPRB, 0.15395E+00_JPRB, 0.15446E+00_JPRB, 0.15472E+00_JPRB, 0.15479E+00_JPRB, 0.15508E+00_JPRB, 0.15576E+00_JPRB ,&
&  0.15660E+00_JPRB, 0.15855E+00_JPRB, 0.16110E+00_JPRB, 0.16375E+00_JPRB, 0.16578E+00_JPRB, 0.16743E+00_JPRB, 0.17088E+00_JPRB ,&
&  0.17477E+00_JPRB, 0.18377E+00_JPRB, 0.19643E+00_JPRB, 0.20547E+00_JPRB, 0.20190E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.20059E+00_JPRB, 0.20189E+00_JPRB, 0.20160E+00_JPRB, 0.20176E+00_JPRB, 0.20127E+00_JPRB, 0.20225E+00_JPRB, 0.20137E+00_JPRB ,&
&  0.20116E+00_JPRB, 0.19988E+00_JPRB, 0.19867E+00_JPRB, 0.19582E+00_JPRB, 0.19427E+00_JPRB, 0.18820E+00_JPRB, 0.18652E+00_JPRB ,&
&  0.18505E+00_JPRB, 0.18583E+00_JPRB, 0.18602E+00_JPRB, 0.18649E+00_JPRB, 0.18730E+00_JPRB, 0.18887E+00_JPRB, 0.18874E+00_JPRB ,&
&  0.18755E+00_JPRB, 0.18550E+00_JPRB, 0.18309E+00_JPRB, 0.18089E+00_JPRB, 0.18122E+00_JPRB, 0.18081E+00_JPRB, 0.17978E+00_JPRB ,&
&  0.17879E+00_JPRB, 0.17937E+00_JPRB, 0.17982E+00_JPRB, 0.18115E+00_JPRB, 0.18603E+00_JPRB, 0.18431E+00_JPRB, 0.18316E+00_JPRB ,&
&  0.18309E+00_JPRB, 0.18314E+00_JPRB, 0.18380E+00_JPRB, 0.18435E+00_JPRB, 0.18659E+00_JPRB, 0.18614E+00_JPRB, 0.18900E+00_JPRB ,&
&  0.18680E+00_JPRB, 0.18538E+00_JPRB, 0.18332E+00_JPRB, 0.18038E+00_JPRB, 0.17640E+00_JPRB, 0.17386E+00_JPRB, 0.17146E+00_JPRB ,&
&  0.16880E+00_JPRB, 0.16659E+00_JPRB, 0.16373E+00_JPRB, 0.16270E+00_JPRB, 0.16120E+00_JPRB, 0.15988E+00_JPRB, 0.15869E+00_JPRB ,&
&  0.15783E+00_JPRB, 0.15660E+00_JPRB, 0.15557E+00_JPRB, 0.15488E+00_JPRB, 0.15471E+00_JPRB, 0.15488E+00_JPRB, 0.15597E+00_JPRB ,&
&  0.15800E+00_JPRB, 0.15965E+00_JPRB, 0.16095E+00_JPRB, 0.16212E+00_JPRB, 0.16068E+00_JPRB, 0.15805E+00_JPRB, 0.15545E+00_JPRB ,&
&  0.15188E+00_JPRB, 0.14996E+00_JPRB, 0.14814E+00_JPRB, 0.14706E+00_JPRB, 0.14678E+00_JPRB, 0.14672E+00_JPRB, 0.14698E+00_JPRB ,&
&  0.14725E+00_JPRB, 0.14773E+00_JPRB, 0.14809E+00_JPRB, 0.14857E+00_JPRB, 0.14921E+00_JPRB, 0.14984E+00_JPRB, 0.15079E+00_JPRB ,&
&  0.15221E+00_JPRB, 0.15401E+00_JPRB, 0.15631E+00_JPRB, 0.15889E+00_JPRB, 0.16218E+00_JPRB, 0.16650E+00_JPRB, 0.17276E+00_JPRB ,&
&  0.17910E+00_JPRB, 0.18932E+00_JPRB, 0.19033E+00_JPRB, 0.19633E+00_JPRB, 0.19966E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.19816E+00_JPRB, 0.20004E+00_JPRB, 0.20072E+00_JPRB, 0.20160E+00_JPRB, 0.20117E+00_JPRB, 0.19871E+00_JPRB, 0.19616E+00_JPRB ,&
&  0.19058E+00_JPRB, 0.18611E+00_JPRB, 0.18069E+00_JPRB, 0.17567E+00_JPRB, 0.17090E+00_JPRB, 0.16787E+00_JPRB, 0.16607E+00_JPRB ,&
&  0.16599E+00_JPRB, 0.16737E+00_JPRB, 0.16837E+00_JPRB, 0.16896E+00_JPRB, 0.16928E+00_JPRB, 0.17023E+00_JPRB, 0.17024E+00_JPRB ,&
&  0.16975E+00_JPRB, 0.16866E+00_JPRB, 0.16787E+00_JPRB, 0.16866E+00_JPRB, 0.17064E+00_JPRB, 0.17268E+00_JPRB, 0.17438E+00_JPRB ,&
&  0.17585E+00_JPRB, 0.17509E+00_JPRB, 0.17506E+00_JPRB, 0.17424E+00_JPRB, 0.17377E+00_JPRB, 0.17320E+00_JPRB, 0.17311E+00_JPRB ,&
&  0.17404E+00_JPRB, 0.17477E+00_JPRB, 0.17671E+00_JPRB, 0.17773E+00_JPRB, 0.17662E+00_JPRB, 0.17879E+00_JPRB, 0.17920E+00_JPRB ,&
&  0.17855E+00_JPRB, 0.17547E+00_JPRB, 0.17211E+00_JPRB, 0.16945E+00_JPRB, 0.16664E+00_JPRB, 0.16431E+00_JPRB, 0.16240E+00_JPRB ,&
&  0.16066E+00_JPRB, 0.15973E+00_JPRB, 0.15780E+00_JPRB, 0.15619E+00_JPRB, 0.15516E+00_JPRB, 0.15409E+00_JPRB, 0.15343E+00_JPRB ,&
&  0.15302E+00_JPRB, 0.15228E+00_JPRB, 0.15216E+00_JPRB, 0.15177E+00_JPRB, 0.15189E+00_JPRB, 0.15204E+00_JPRB, 0.15240E+00_JPRB ,&
&  0.15319E+00_JPRB, 0.15416E+00_JPRB, 0.15408E+00_JPRB, 0.15395E+00_JPRB, 0.15245E+00_JPRB, 0.15080E+00_JPRB, 0.14939E+00_JPRB ,&
&  0.14754E+00_JPRB, 0.14620E+00_JPRB, 0.14537E+00_JPRB, 0.14456E+00_JPRB, 0.14443E+00_JPRB, 0.14447E+00_JPRB, 0.14444E+00_JPRB ,&
&  0.14466E+00_JPRB, 0.14501E+00_JPRB, 0.14552E+00_JPRB, 0.14655E+00_JPRB, 0.14747E+00_JPRB, 0.14878E+00_JPRB, 0.15022E+00_JPRB ,&
&  0.15135E+00_JPRB, 0.15339E+00_JPRB, 0.15548E+00_JPRB, 0.15787E+00_JPRB, 0.16280E+00_JPRB, 0.16710E+00_JPRB, 0.17481E+00_JPRB ,&
&  0.18069E+00_JPRB, 0.18981E+00_JPRB, 0.18776E+00_JPRB, 0.19218E+00_JPRB, 0.19605E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.19725E+00_JPRB, 0.19844E+00_JPRB, 0.19818E+00_JPRB, 0.19843E+00_JPRB, 0.19726E+00_JPRB, 0.19547E+00_JPRB, 0.19287E+00_JPRB ,&
&  0.19012E+00_JPRB, 0.18437E+00_JPRB, 0.18027E+00_JPRB, 0.17573E+00_JPRB, 0.17192E+00_JPRB, 0.16856E+00_JPRB, 0.16618E+00_JPRB ,&
&  0.16511E+00_JPRB, 0.16472E+00_JPRB, 0.16450E+00_JPRB, 0.16360E+00_JPRB, 0.16197E+00_JPRB, 0.16089E+00_JPRB, 0.15951E+00_JPRB ,&
&  0.15850E+00_JPRB, 0.15843E+00_JPRB, 0.15926E+00_JPRB, 0.16107E+00_JPRB, 0.16431E+00_JPRB, 0.16662E+00_JPRB, 0.16871E+00_JPRB ,&
&  0.17074E+00_JPRB, 0.16712E+00_JPRB, 0.16694E+00_JPRB, 0.16730E+00_JPRB, 0.16724E+00_JPRB, 0.16659E+00_JPRB, 0.16543E+00_JPRB ,&
&  0.16852E+00_JPRB, 0.16932E+00_JPRB, 0.17440E+00_JPRB, 0.17417E+00_JPRB, 0.17318E+00_JPRB, 0.17444E+00_JPRB, 0.17315E+00_JPRB ,&
&  0.17289E+00_JPRB, 0.16980E+00_JPRB, 0.16751E+00_JPRB, 0.16423E+00_JPRB, 0.16096E+00_JPRB, 0.15931E+00_JPRB, 0.15740E+00_JPRB ,&
&  0.15677E+00_JPRB, 0.15662E+00_JPRB, 0.15522E+00_JPRB, 0.15435E+00_JPRB, 0.15275E+00_JPRB, 0.15094E+00_JPRB, 0.14982E+00_JPRB ,&
&  0.14868E+00_JPRB, 0.14777E+00_JPRB, 0.14730E+00_JPRB, 0.14683E+00_JPRB, 0.14664E+00_JPRB, 0.14654E+00_JPRB, 0.14644E+00_JPRB ,&
&  0.14647E+00_JPRB, 0.14640E+00_JPRB, 0.14587E+00_JPRB, 0.14558E+00_JPRB, 0.14459E+00_JPRB, 0.14391E+00_JPRB, 0.14335E+00_JPRB ,&
&  0.14286E+00_JPRB, 0.14269E+00_JPRB, 0.14261E+00_JPRB, 0.14267E+00_JPRB, 0.14282E+00_JPRB, 0.14330E+00_JPRB, 0.14386E+00_JPRB ,&
&  0.14443E+00_JPRB, 0.14538E+00_JPRB, 0.14612E+00_JPRB, 0.14717E+00_JPRB, 0.14842E+00_JPRB, 0.14949E+00_JPRB, 0.15117E+00_JPRB ,&
&  0.15302E+00_JPRB, 0.15516E+00_JPRB, 0.15825E+00_JPRB, 0.16100E+00_JPRB, 0.16436E+00_JPRB, 0.16896E+00_JPRB, 0.17393E+00_JPRB ,&
&  0.17937E+00_JPRB, 0.18556E+00_JPRB, 0.18615E+00_JPRB, 0.19175E+00_JPRB, 0.19531E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.18964E+00_JPRB, 0.19032E+00_JPRB, 0.19097E+00_JPRB, 0.19122E+00_JPRB, 0.19129E+00_JPRB, 0.19051E+00_JPRB, 0.18952E+00_JPRB ,&
&  0.18565E+00_JPRB, 0.18284E+00_JPRB, 0.17971E+00_JPRB, 0.17659E+00_JPRB, 0.17384E+00_JPRB, 0.17100E+00_JPRB, 0.16848E+00_JPRB ,&
&  0.16619E+00_JPRB, 0.16447E+00_JPRB, 0.16290E+00_JPRB, 0.16077E+00_JPRB, 0.15809E+00_JPRB, 0.15563E+00_JPRB, 0.15408E+00_JPRB ,&
&  0.15364E+00_JPRB, 0.15395E+00_JPRB, 0.15536E+00_JPRB, 0.15681E+00_JPRB, 0.15835E+00_JPRB, 0.15951E+00_JPRB, 0.16068E+00_JPRB ,&
&  0.16011E+00_JPRB, 0.15974E+00_JPRB, 0.15939E+00_JPRB, 0.15877E+00_JPRB, 0.15939E+00_JPRB, 0.16023E+00_JPRB, 0.16075E+00_JPRB ,&
&  0.16137E+00_JPRB, 0.16291E+00_JPRB, 0.16403E+00_JPRB, 0.16507E+00_JPRB, 0.16502E+00_JPRB, 0.16434E+00_JPRB, 0.16294E+00_JPRB ,&
&  0.16116E+00_JPRB, 0.15925E+00_JPRB, 0.15753E+00_JPRB, 0.15606E+00_JPRB, 0.15493E+00_JPRB, 0.15365E+00_JPRB, 0.15256E+00_JPRB ,&
&  0.15202E+00_JPRB, 0.15153E+00_JPRB, 0.15082E+00_JPRB, 0.15013E+00_JPRB, 0.14905E+00_JPRB, 0.14811E+00_JPRB, 0.14714E+00_JPRB ,&
&  0.14626E+00_JPRB, 0.14542E+00_JPRB, 0.14466E+00_JPRB, 0.14387E+00_JPRB, 0.14333E+00_JPRB, 0.14285E+00_JPRB, 0.14238E+00_JPRB ,&
&  0.14188E+00_JPRB, 0.14171E+00_JPRB, 0.14136E+00_JPRB, 0.14124E+00_JPRB, 0.14103E+00_JPRB, 0.14088E+00_JPRB, 0.14107E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14212E+00_JPRB, 0.14239E+00_JPRB, 0.14274E+00_JPRB, 0.14310E+00_JPRB, 0.14357E+00_JPRB, 0.14431E+00_JPRB ,&
&  0.14489E+00_JPRB, 0.14559E+00_JPRB, 0.14616E+00_JPRB, 0.14684E+00_JPRB, 0.14811E+00_JPRB, 0.14942E+00_JPRB, 0.15092E+00_JPRB ,&
&  0.15278E+00_JPRB, 0.15472E+00_JPRB, 0.15696E+00_JPRB, 0.15913E+00_JPRB, 0.16206E+00_JPRB, 0.16379E+00_JPRB, 0.16709E+00_JPRB ,&
&  0.17072E+00_JPRB, 0.17192E+00_JPRB, 0.18417E+00_JPRB, 0.18536E+00_JPRB, 0.18702E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.16348E+00_JPRB, 0.16548E+00_JPRB, 0.16684E+00_JPRB, 0.16846E+00_JPRB, 0.16726E+00_JPRB, 0.17909E+00_JPRB, 0.17705E+00_JPRB ,&
&  0.17589E+00_JPRB, 0.17125E+00_JPRB, 0.17095E+00_JPRB, 0.16932E+00_JPRB, 0.16730E+00_JPRB, 0.16504E+00_JPRB, 0.16296E+00_JPRB ,&
&  0.16103E+00_JPRB, 0.15888E+00_JPRB, 0.15735E+00_JPRB, 0.15612E+00_JPRB, 0.15499E+00_JPRB, 0.15397E+00_JPRB, 0.15317E+00_JPRB ,&
&  0.15314E+00_JPRB, 0.15370E+00_JPRB, 0.15476E+00_JPRB, 0.15462E+00_JPRB, 0.15430E+00_JPRB, 0.15388E+00_JPRB, 0.15365E+00_JPRB ,&
&  0.15307E+00_JPRB, 0.15265E+00_JPRB, 0.15226E+00_JPRB, 0.15170E+00_JPRB, 0.15235E+00_JPRB, 0.15302E+00_JPRB, 0.15370E+00_JPRB ,&
&  0.15433E+00_JPRB, 0.15495E+00_JPRB, 0.15494E+00_JPRB, 0.15482E+00_JPRB, 0.15443E+00_JPRB, 0.15393E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15237E+00_JPRB, 0.15156E+00_JPRB, 0.15062E+00_JPRB, 0.14999E+00_JPRB, 0.14943E+00_JPRB, 0.14891E+00_JPRB, 0.14846E+00_JPRB ,&
&  0.14822E+00_JPRB, 0.14807E+00_JPRB, 0.14788E+00_JPRB, 0.14758E+00_JPRB, 0.14721E+00_JPRB, 0.14683E+00_JPRB, 0.14650E+00_JPRB ,&
&  0.14625E+00_JPRB, 0.14576E+00_JPRB, 0.14533E+00_JPRB, 0.14487E+00_JPRB, 0.14452E+00_JPRB, 0.14406E+00_JPRB, 0.14368E+00_JPRB ,&
&  0.14332E+00_JPRB, 0.14304E+00_JPRB, 0.14295E+00_JPRB, 0.14289E+00_JPRB, 0.14284E+00_JPRB, 0.14288E+00_JPRB, 0.14300E+00_JPRB ,&
&  0.14318E+00_JPRB, 0.14338E+00_JPRB, 0.14366E+00_JPRB, 0.14387E+00_JPRB, 0.14407E+00_JPRB, 0.14424E+00_JPRB, 0.14465E+00_JPRB ,&
&  0.14514E+00_JPRB, 0.14566E+00_JPRB, 0.14614E+00_JPRB, 0.14634E+00_JPRB, 0.14712E+00_JPRB, 0.14795E+00_JPRB, 0.14885E+00_JPRB ,&
&  0.14980E+00_JPRB, 0.15102E+00_JPRB, 0.15235E+00_JPRB, 0.15380E+00_JPRB, 0.15550E+00_JPRB, 0.15659E+00_JPRB, 0.15782E+00_JPRB ,&
&  0.15947E+00_JPRB, 0.16222E+00_JPRB, 0.16314E+00_JPRB, 0.16348E+00_JPRB, 0.16363E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.15272E+00_JPRB, 0.15252E+00_JPRB, 0.15228E+00_JPRB, 0.15202E+00_JPRB, 0.15120E+00_JPRB, 0.15081E+00_JPRB, 0.15047E+00_JPRB ,&
&  0.15027E+00_JPRB, 0.15014E+00_JPRB, 0.15022E+00_JPRB, 0.15035E+00_JPRB, 0.15049E+00_JPRB, 0.15062E+00_JPRB, 0.15072E+00_JPRB ,&
&  0.15069E+00_JPRB, 0.15069E+00_JPRB, 0.15001E+00_JPRB, 0.14947E+00_JPRB, 0.14914E+00_JPRB, 0.14889E+00_JPRB, 0.14856E+00_JPRB ,&
&  0.14829E+00_JPRB, 0.14796E+00_JPRB, 0.14757E+00_JPRB, 0.14731E+00_JPRB, 0.14732E+00_JPRB, 0.14718E+00_JPRB, 0.14719E+00_JPRB ,&
&  0.14718E+00_JPRB, 0.14735E+00_JPRB, 0.14742E+00_JPRB, 0.14751E+00_JPRB, 0.14787E+00_JPRB, 0.14819E+00_JPRB, 0.14827E+00_JPRB ,&
&  0.14830E+00_JPRB, 0.14813E+00_JPRB, 0.14795E+00_JPRB, 0.14782E+00_JPRB, 0.14776E+00_JPRB, 0.14776E+00_JPRB, 0.14772E+00_JPRB ,&
&  0.14761E+00_JPRB, 0.14743E+00_JPRB, 0.14720E+00_JPRB, 0.14694E+00_JPRB, 0.14670E+00_JPRB, 0.14659E+00_JPRB, 0.14658E+00_JPRB ,&
&  0.14657E+00_JPRB, 0.14646E+00_JPRB, 0.14627E+00_JPRB, 0.14596E+00_JPRB, 0.14567E+00_JPRB, 0.14538E+00_JPRB, 0.14511E+00_JPRB ,&
&  0.14485E+00_JPRB, 0.14472E+00_JPRB, 0.14459E+00_JPRB, 0.14449E+00_JPRB, 0.14442E+00_JPRB, 0.14431E+00_JPRB, 0.14420E+00_JPRB ,&
&  0.14411E+00_JPRB, 0.14403E+00_JPRB, 0.14394E+00_JPRB, 0.14387E+00_JPRB, 0.14382E+00_JPRB, 0.14378E+00_JPRB, 0.14378E+00_JPRB ,&
&  0.14379E+00_JPRB, 0.14381E+00_JPRB, 0.14385E+00_JPRB, 0.14399E+00_JPRB, 0.14415E+00_JPRB, 0.14435E+00_JPRB, 0.14459E+00_JPRB ,&
&  0.14485E+00_JPRB, 0.14514E+00_JPRB, 0.14544E+00_JPRB, 0.14566E+00_JPRB, 0.14601E+00_JPRB, 0.14636E+00_JPRB, 0.14673E+00_JPRB ,&
&  0.14709E+00_JPRB, 0.14753E+00_JPRB, 0.14803E+00_JPRB, 0.14859E+00_JPRB, 0.14922E+00_JPRB, 0.14957E+00_JPRB, 0.15000E+00_JPRB ,&
&  0.15051E+00_JPRB, 0.15120E+00_JPRB, 0.15157E+00_JPRB, 0.15184E+00_JPRB, 0.15213E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB, 0.14615E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB, 0.45346E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.45585E+00_JPRB, 0.45573E+00_JPRB, 0.45561E+00_JPRB, 0.45554E+00_JPRB, 0.45547E+00_JPRB, 0.45549E+00_JPRB, 0.45547E+00_JPRB ,&
&  0.45560E+00_JPRB, 0.45571E+00_JPRB, 0.45577E+00_JPRB, 0.45576E+00_JPRB, 0.45585E+00_JPRB, 0.45589E+00_JPRB, 0.45587E+00_JPRB ,&
&  0.45579E+00_JPRB, 0.45577E+00_JPRB, 0.45555E+00_JPRB, 0.45529E+00_JPRB, 0.45493E+00_JPRB, 0.45470E+00_JPRB, 0.45432E+00_JPRB ,&
&  0.45402E+00_JPRB, 0.45353E+00_JPRB, 0.45306E+00_JPRB, 0.45230E+00_JPRB, 0.45175E+00_JPRB, 0.45113E+00_JPRB, 0.45059E+00_JPRB ,&
&  0.45011E+00_JPRB, 0.44981E+00_JPRB, 0.44951E+00_JPRB, 0.44910E+00_JPRB, 0.44878E+00_JPRB, 0.44886E+00_JPRB, 0.44904E+00_JPRB ,&
&  0.44920E+00_JPRB, 0.44943E+00_JPRB, 0.44976E+00_JPRB, 0.44988E+00_JPRB, 0.44980E+00_JPRB, 0.44980E+00_JPRB, 0.44974E+00_JPRB ,&
&  0.44961E+00_JPRB, 0.44937E+00_JPRB, 0.44913E+00_JPRB, 0.44907E+00_JPRB, 0.44903E+00_JPRB, 0.44884E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44857E+00_JPRB, 0.44843E+00_JPRB, 0.44822E+00_JPRB, 0.44789E+00_JPRB, 0.44766E+00_JPRB, 0.44741E+00_JPRB, 0.44714E+00_JPRB ,&
&  0.44685E+00_JPRB, 0.44651E+00_JPRB, 0.44616E+00_JPRB, 0.44578E+00_JPRB, 0.44535E+00_JPRB, 0.44499E+00_JPRB, 0.44472E+00_JPRB ,&
&  0.44453E+00_JPRB, 0.44447E+00_JPRB, 0.44447E+00_JPRB, 0.44450E+00_JPRB, 0.44460E+00_JPRB, 0.44481E+00_JPRB, 0.44511E+00_JPRB ,&
&  0.44550E+00_JPRB, 0.44596E+00_JPRB, 0.44659E+00_JPRB, 0.44735E+00_JPRB, 0.44808E+00_JPRB, 0.44882E+00_JPRB, 0.44963E+00_JPRB ,&
&  0.45037E+00_JPRB, 0.45094E+00_JPRB, 0.45134E+00_JPRB, 0.45175E+00_JPRB, 0.45244E+00_JPRB, 0.45295E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45366E+00_JPRB, 0.45406E+00_JPRB, 0.45442E+00_JPRB, 0.45482E+00_JPRB, 0.45498E+00_JPRB, 0.45518E+00_JPRB, 0.45539E+00_JPRB ,&
&  0.45559E+00_JPRB, 0.45571E+00_JPRB, 0.45580E+00_JPRB, 0.45586E+00_JPRB, 0.45587E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.45259E+00_JPRB, 0.45239E+00_JPRB, 0.45242E+00_JPRB, 0.45262E+00_JPRB, 0.45292E+00_JPRB, 0.45325E+00_JPRB, 0.45365E+00_JPRB ,&
&  0.45418E+00_JPRB, 0.45451E+00_JPRB, 0.45445E+00_JPRB, 0.45471E+00_JPRB, 0.45533E+00_JPRB, 0.45605E+00_JPRB, 0.45598E+00_JPRB ,&
&  0.45599E+00_JPRB, 0.45594E+00_JPRB, 0.45608E+00_JPRB, 0.45573E+00_JPRB, 0.45518E+00_JPRB, 0.45436E+00_JPRB, 0.45377E+00_JPRB ,&
&  0.45324E+00_JPRB, 0.45242E+00_JPRB, 0.45140E+00_JPRB, 0.44954E+00_JPRB, 0.44792E+00_JPRB, 0.44624E+00_JPRB, 0.44458E+00_JPRB ,&
&  0.44238E+00_JPRB, 0.44049E+00_JPRB, 0.43898E+00_JPRB, 0.43771E+00_JPRB, 0.43946E+00_JPRB, 0.44061E+00_JPRB, 0.44126E+00_JPRB ,&
&  0.44131E+00_JPRB, 0.43939E+00_JPRB, 0.43876E+00_JPRB, 0.43838E+00_JPRB, 0.43810E+00_JPRB, 0.43964E+00_JPRB, 0.44066E+00_JPRB ,&
&  0.44149E+00_JPRB, 0.44220E+00_JPRB, 0.44403E+00_JPRB, 0.44494E+00_JPRB, 0.44558E+00_JPRB, 0.44597E+00_JPRB, 0.44589E+00_JPRB ,&
&  0.44629E+00_JPRB, 0.44672E+00_JPRB, 0.44708E+00_JPRB, 0.44700E+00_JPRB, 0.44687E+00_JPRB, 0.44671E+00_JPRB, 0.44624E+00_JPRB ,&
&  0.44543E+00_JPRB, 0.44480E+00_JPRB, 0.44404E+00_JPRB, 0.44319E+00_JPRB, 0.44219E+00_JPRB, 0.44153E+00_JPRB, 0.44096E+00_JPRB ,&
&  0.44042E+00_JPRB, 0.44005E+00_JPRB, 0.43982E+00_JPRB, 0.43962E+00_JPRB, 0.43951E+00_JPRB, 0.43972E+00_JPRB, 0.44028E+00_JPRB ,&
&  0.44118E+00_JPRB, 0.44225E+00_JPRB, 0.44348E+00_JPRB, 0.44486E+00_JPRB, 0.44609E+00_JPRB, 0.44721E+00_JPRB, 0.44873E+00_JPRB ,&
&  0.45044E+00_JPRB, 0.45176E+00_JPRB, 0.45278E+00_JPRB, 0.45365E+00_JPRB, 0.45483E+00_JPRB, 0.45545E+00_JPRB, 0.45569E+00_JPRB ,&
&  0.45576E+00_JPRB, 0.45625E+00_JPRB, 0.45674E+00_JPRB, 0.45729E+00_JPRB, 0.45738E+00_JPRB, 0.45709E+00_JPRB, 0.45655E+00_JPRB ,&
&  0.45591E+00_JPRB, 0.45534E+00_JPRB, 0.45460E+00_JPRB, 0.45379E+00_JPRB, 0.45310E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.44978E+00_JPRB, 0.45108E+00_JPRB, 0.45286E+00_JPRB, 0.45483E+00_JPRB, 0.45711E+00_JPRB, 0.45867E+00_JPRB, 0.46040E+00_JPRB ,&
&  0.46155E+00_JPRB, 0.46242E+00_JPRB, 0.46243E+00_JPRB, 0.46314E+00_JPRB, 0.46341E+00_JPRB, 0.46390E+00_JPRB, 0.46349E+00_JPRB ,&
&  0.46349E+00_JPRB, 0.46281E+00_JPRB, 0.46348E+00_JPRB, 0.46370E+00_JPRB, 0.46392E+00_JPRB, 0.46399E+00_JPRB, 0.46091E+00_JPRB ,&
&  0.45792E+00_JPRB, 0.45594E+00_JPRB, 0.45213E+00_JPRB, 0.44852E+00_JPRB, 0.44607E+00_JPRB, 0.42838E+00_JPRB, 0.42296E+00_JPRB ,&
&  0.42899E+00_JPRB, 0.43306E+00_JPRB, 0.43688E+00_JPRB, 0.43972E+00_JPRB, 0.43310E+00_JPRB, 0.42397E+00_JPRB, 0.41862E+00_JPRB ,&
&  0.40335E+00_JPRB, 0.40669E+00_JPRB, 0.40906E+00_JPRB, 0.41102E+00_JPRB, 0.41484E+00_JPRB, 0.41621E+00_JPRB, 0.42021E+00_JPRB ,&
&  0.43037E+00_JPRB, 0.43299E+00_JPRB, 0.43652E+00_JPRB, 0.43943E+00_JPRB, 0.44115E+00_JPRB, 0.44321E+00_JPRB, 0.44770E+00_JPRB ,&
&  0.44979E+00_JPRB, 0.45165E+00_JPRB, 0.45296E+00_JPRB, 0.45296E+00_JPRB, 0.45315E+00_JPRB, 0.45259E+00_JPRB, 0.45182E+00_JPRB ,&
&  0.44999E+00_JPRB, 0.44907E+00_JPRB, 0.44844E+00_JPRB, 0.44770E+00_JPRB, 0.44730E+00_JPRB, 0.44569E+00_JPRB, 0.44448E+00_JPRB ,&
&  0.44373E+00_JPRB, 0.44340E+00_JPRB, 0.44347E+00_JPRB, 0.44412E+00_JPRB, 0.44542E+00_JPRB, 0.44702E+00_JPRB, 0.44844E+00_JPRB ,&
&  0.45021E+00_JPRB, 0.45168E+00_JPRB, 0.45349E+00_JPRB, 0.45525E+00_JPRB, 0.45652E+00_JPRB, 0.45753E+00_JPRB, 0.45852E+00_JPRB ,&
&  0.46001E+00_JPRB, 0.46166E+00_JPRB, 0.46246E+00_JPRB, 0.46222E+00_JPRB, 0.46321E+00_JPRB, 0.46388E+00_JPRB, 0.46399E+00_JPRB ,&
&  0.46376E+00_JPRB, 0.46376E+00_JPRB, 0.46276E+00_JPRB, 0.46124E+00_JPRB, 0.45939E+00_JPRB, 0.45773E+00_JPRB, 0.45599E+00_JPRB ,&
&  0.45422E+00_JPRB, 0.45257E+00_JPRB, 0.45119E+00_JPRB, 0.45000E+00_JPRB, 0.44940E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.45435E+00_JPRB, 0.45773E+00_JPRB, 0.46204E+00_JPRB, 0.46458E+00_JPRB, 0.46783E+00_JPRB, 0.46907E+00_JPRB, 0.47101E+00_JPRB ,&
&  0.47159E+00_JPRB, 0.47255E+00_JPRB, 0.47317E+00_JPRB, 0.47345E+00_JPRB, 0.47336E+00_JPRB, 0.47288E+00_JPRB, 0.47250E+00_JPRB ,&
&  0.47254E+00_JPRB, 0.47216E+00_JPRB, 0.47287E+00_JPRB, 0.47315E+00_JPRB, 0.47338E+00_JPRB, 0.47325E+00_JPRB, 0.46976E+00_JPRB ,&
&  0.46548E+00_JPRB, 0.46162E+00_JPRB, 0.43926E+00_JPRB, 0.43682E+00_JPRB, 0.43274E+00_JPRB, 0.43059E+00_JPRB, 0.43157E+00_JPRB ,&
&  0.42933E+00_JPRB, 0.42638E+00_JPRB, 0.41616E+00_JPRB, 0.40561E+00_JPRB, 0.39715E+00_JPRB, 0.38777E+00_JPRB, 0.38652E+00_JPRB ,&
&  0.38511E+00_JPRB, 0.38771E+00_JPRB, 0.39045E+00_JPRB, 0.39247E+00_JPRB, 0.39839E+00_JPRB, 0.40224E+00_JPRB, 0.40920E+00_JPRB ,&
&  0.41792E+00_JPRB, 0.42407E+00_JPRB, 0.42465E+00_JPRB, 0.42345E+00_JPRB, 0.43479E+00_JPRB, 0.44229E+00_JPRB, 0.44907E+00_JPRB ,&
&  0.45326E+00_JPRB, 0.45427E+00_JPRB, 0.45666E+00_JPRB, 0.45703E+00_JPRB, 0.45779E+00_JPRB, 0.45980E+00_JPRB, 0.46031E+00_JPRB ,&
&  0.46163E+00_JPRB, 0.46138E+00_JPRB, 0.46026E+00_JPRB, 0.45853E+00_JPRB, 0.45540E+00_JPRB, 0.45340E+00_JPRB, 0.45213E+00_JPRB ,&
&  0.45227E+00_JPRB, 0.45367E+00_JPRB, 0.45558E+00_JPRB, 0.45843E+00_JPRB, 0.46143E+00_JPRB, 0.46355E+00_JPRB, 0.46462E+00_JPRB ,&
&  0.46515E+00_JPRB, 0.46514E+00_JPRB, 0.46738E+00_JPRB, 0.46684E+00_JPRB, 0.46759E+00_JPRB, 0.47145E+00_JPRB, 0.47047E+00_JPRB ,&
&  0.47101E+00_JPRB, 0.47315E+00_JPRB, 0.47448E+00_JPRB, 0.47561E+00_JPRB, 0.47643E+00_JPRB, 0.47667E+00_JPRB, 0.47714E+00_JPRB ,&
&  0.47693E+00_JPRB, 0.47589E+00_JPRB, 0.47365E+00_JPRB, 0.47074E+00_JPRB, 0.46755E+00_JPRB, 0.46433E+00_JPRB, 0.46023E+00_JPRB ,&
&  0.45648E+00_JPRB, 0.45303E+00_JPRB, 0.45104E+00_JPRB, 0.45013E+00_JPRB, 0.45162E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.46585E+00_JPRB, 0.46943E+00_JPRB, 0.47442E+00_JPRB, 0.47625E+00_JPRB, 0.47907E+00_JPRB, 0.48014E+00_JPRB, 0.48124E+00_JPRB ,&
&  0.48173E+00_JPRB, 0.48232E+00_JPRB, 0.48220E+00_JPRB, 0.48225E+00_JPRB, 0.48178E+00_JPRB, 0.48168E+00_JPRB, 0.48121E+00_JPRB ,&
&  0.48091E+00_JPRB, 0.48042E+00_JPRB, 0.48032E+00_JPRB, 0.48006E+00_JPRB, 0.48058E+00_JPRB, 0.48073E+00_JPRB, 0.47905E+00_JPRB ,&
&  0.47559E+00_JPRB, 0.46786E+00_JPRB, 0.45091E+00_JPRB, 0.44679E+00_JPRB, 0.44298E+00_JPRB, 0.44292E+00_JPRB, 0.44166E+00_JPRB ,&
&  0.43848E+00_JPRB, 0.40716E+00_JPRB, 0.40855E+00_JPRB, 0.40099E+00_JPRB, 0.39725E+00_JPRB, 0.39357E+00_JPRB, 0.38573E+00_JPRB ,&
&  0.38290E+00_JPRB, 0.37995E+00_JPRB, 0.37951E+00_JPRB, 0.38371E+00_JPRB, 0.38799E+00_JPRB, 0.39466E+00_JPRB, 0.40277E+00_JPRB ,&
&  0.40341E+00_JPRB, 0.41616E+00_JPRB, 0.42310E+00_JPRB, 0.42739E+00_JPRB, 0.43875E+00_JPRB, 0.44789E+00_JPRB, 0.45050E+00_JPRB ,&
&  0.45357E+00_JPRB, 0.45333E+00_JPRB, 0.45628E+00_JPRB, 0.46120E+00_JPRB, 0.46675E+00_JPRB, 0.46723E+00_JPRB, 0.46813E+00_JPRB ,&
&  0.46836E+00_JPRB, 0.46860E+00_JPRB, 0.46869E+00_JPRB, 0.46814E+00_JPRB, 0.46772E+00_JPRB, 0.46876E+00_JPRB, 0.47040E+00_JPRB ,&
&  0.47401E+00_JPRB, 0.47761E+00_JPRB, 0.48160E+00_JPRB, 0.48481E+00_JPRB, 0.48721E+00_JPRB, 0.48914E+00_JPRB, 0.49112E+00_JPRB ,&
&  0.49292E+00_JPRB, 0.49398E+00_JPRB, 0.49312E+00_JPRB, 0.49245E+00_JPRB, 0.49050E+00_JPRB, 0.48763E+00_JPRB, 0.48690E+00_JPRB ,&
&  0.48847E+00_JPRB, 0.48916E+00_JPRB, 0.48939E+00_JPRB, 0.48979E+00_JPRB, 0.49084E+00_JPRB, 0.49115E+00_JPRB, 0.49131E+00_JPRB ,&
&  0.49095E+00_JPRB, 0.48974E+00_JPRB, 0.48753E+00_JPRB, 0.48282E+00_JPRB, 0.47767E+00_JPRB, 0.47431E+00_JPRB, 0.46886E+00_JPRB ,&
&  0.46383E+00_JPRB, 0.45873E+00_JPRB, 0.45720E+00_JPRB, 0.45692E+00_JPRB, 0.46080E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.48047E+00_JPRB, 0.48585E+00_JPRB, 0.49145E+00_JPRB, 0.49425E+00_JPRB, 0.49704E+00_JPRB, 0.49748E+00_JPRB, 0.49843E+00_JPRB ,&
&  0.49882E+00_JPRB, 0.49963E+00_JPRB, 0.49973E+00_JPRB, 0.49970E+00_JPRB, 0.49906E+00_JPRB, 0.49824E+00_JPRB, 0.49683E+00_JPRB ,&
&  0.49467E+00_JPRB, 0.49258E+00_JPRB, 0.48973E+00_JPRB, 0.48758E+00_JPRB, 0.48638E+00_JPRB, 0.48722E+00_JPRB, 0.48638E+00_JPRB ,&
&  0.48302E+00_JPRB, 0.47585E+00_JPRB, 0.46281E+00_JPRB, 0.45622E+00_JPRB, 0.45467E+00_JPRB, 0.44755E+00_JPRB, 0.44668E+00_JPRB ,&
&  0.44439E+00_JPRB, 0.44382E+00_JPRB, 0.43921E+00_JPRB, 0.43659E+00_JPRB, 0.42755E+00_JPRB, 0.38493E+00_JPRB, 0.38074E+00_JPRB ,&
&  0.37761E+00_JPRB, 0.37237E+00_JPRB, 0.37174E+00_JPRB, 0.37310E+00_JPRB, 0.37750E+00_JPRB, 0.39153E+00_JPRB, 0.40411E+00_JPRB ,&
&  0.40417E+00_JPRB, 0.41929E+00_JPRB, 0.42492E+00_JPRB, 0.43052E+00_JPRB, 0.44065E+00_JPRB, 0.44986E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45637E+00_JPRB, 0.46043E+00_JPRB, 0.46727E+00_JPRB, 0.47429E+00_JPRB, 0.47932E+00_JPRB, 0.48197E+00_JPRB, 0.48533E+00_JPRB ,&
&  0.48859E+00_JPRB, 0.49202E+00_JPRB, 0.49623E+00_JPRB, 0.49868E+00_JPRB, 0.49900E+00_JPRB, 0.49816E+00_JPRB, 0.49857E+00_JPRB ,&
&  0.50195E+00_JPRB, 0.50434E+00_JPRB, 0.51033E+00_JPRB, 0.50900E+00_JPRB, 0.50982E+00_JPRB, 0.50995E+00_JPRB, 0.51279E+00_JPRB ,&
&  0.51391E+00_JPRB, 0.52069E+00_JPRB, 0.52532E+00_JPRB, 0.52802E+00_JPRB, 0.52161E+00_JPRB, 0.51542E+00_JPRB, 0.51107E+00_JPRB ,&
&  0.50911E+00_JPRB, 0.50909E+00_JPRB, 0.50484E+00_JPRB, 0.50354E+00_JPRB, 0.50338E+00_JPRB, 0.50371E+00_JPRB, 0.50430E+00_JPRB ,&
&  0.50405E+00_JPRB, 0.50170E+00_JPRB, 0.49761E+00_JPRB, 0.49144E+00_JPRB, 0.48580E+00_JPRB, 0.48068E+00_JPRB, 0.47168E+00_JPRB ,&
&  0.46536E+00_JPRB, 0.46071E+00_JPRB, 0.46134E+00_JPRB, 0.46442E+00_JPRB, 0.47176E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.49180E+00_JPRB, 0.49971E+00_JPRB, 0.50766E+00_JPRB, 0.51423E+00_JPRB, 0.51710E+00_JPRB, 0.51703E+00_JPRB, 0.51508E+00_JPRB ,&
&  0.51298E+00_JPRB, 0.51207E+00_JPRB, 0.51109E+00_JPRB, 0.51044E+00_JPRB, 0.50978E+00_JPRB, 0.50846E+00_JPRB, 0.50643E+00_JPRB ,&
&  0.50508E+00_JPRB, 0.49446E+00_JPRB, 0.48751E+00_JPRB, 0.48602E+00_JPRB, 0.48510E+00_JPRB, 0.48571E+00_JPRB, 0.48714E+00_JPRB ,&
&  0.48392E+00_JPRB, 0.47684E+00_JPRB, 0.47095E+00_JPRB, 0.46282E+00_JPRB, 0.45467E+00_JPRB, 0.45537E+00_JPRB, 0.45419E+00_JPRB ,&
&  0.44397E+00_JPRB, 0.43580E+00_JPRB, 0.43981E+00_JPRB, 0.44107E+00_JPRB, 0.42545E+00_JPRB, 0.40661E+00_JPRB, 0.38718E+00_JPRB ,&
&  0.38579E+00_JPRB, 0.37680E+00_JPRB, 0.37508E+00_JPRB, 0.37632E+00_JPRB, 0.38267E+00_JPRB, 0.39008E+00_JPRB, 0.40345E+00_JPRB ,&
&  0.40849E+00_JPRB, 0.41321E+00_JPRB, 0.42417E+00_JPRB, 0.43605E+00_JPRB, 0.44496E+00_JPRB, 0.45431E+00_JPRB, 0.46091E+00_JPRB ,&
&  0.47049E+00_JPRB, 0.48049E+00_JPRB, 0.48934E+00_JPRB, 0.49675E+00_JPRB, 0.50431E+00_JPRB, 0.50678E+00_JPRB, 0.51048E+00_JPRB ,&
&  0.53375E+00_JPRB, 0.55326E+00_JPRB, 0.56302E+00_JPRB, 0.56965E+00_JPRB, 0.57035E+00_JPRB, 0.56715E+00_JPRB, 0.55604E+00_JPRB ,&
&  0.55058E+00_JPRB, 0.55203E+00_JPRB, 0.55127E+00_JPRB, 0.55173E+00_JPRB, 0.55045E+00_JPRB, 0.54737E+00_JPRB, 0.54077E+00_JPRB ,&
&  0.53127E+00_JPRB, 0.53410E+00_JPRB, 0.58724E+00_JPRB, 0.59607E+00_JPRB, 0.58555E+00_JPRB, 0.56730E+00_JPRB, 0.54815E+00_JPRB ,&
&  0.53443E+00_JPRB, 0.52605E+00_JPRB, 0.52117E+00_JPRB, 0.51737E+00_JPRB, 0.51468E+00_JPRB, 0.51054E+00_JPRB, 0.51085E+00_JPRB ,&
&  0.50880E+00_JPRB, 0.50436E+00_JPRB, 0.49941E+00_JPRB, 0.49340E+00_JPRB, 0.48742E+00_JPRB, 0.48228E+00_JPRB, 0.47509E+00_JPRB ,&
&  0.46951E+00_JPRB, 0.46592E+00_JPRB, 0.46829E+00_JPRB, 0.47428E+00_JPRB, 0.48348E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.50163E+00_JPRB, 0.51080E+00_JPRB, 0.52167E+00_JPRB, 0.52946E+00_JPRB, 0.53076E+00_JPRB, 0.51979E+00_JPRB, 0.51004E+00_JPRB ,&
&  0.50382E+00_JPRB, 0.50147E+00_JPRB, 0.50080E+00_JPRB, 0.49948E+00_JPRB, 0.49896E+00_JPRB, 0.49845E+00_JPRB, 0.49798E+00_JPRB ,&
&  0.49782E+00_JPRB, 0.49605E+00_JPRB, 0.49164E+00_JPRB, 0.48435E+00_JPRB, 0.47980E+00_JPRB, 0.47901E+00_JPRB, 0.48017E+00_JPRB ,&
&  0.47996E+00_JPRB, 0.47771E+00_JPRB, 0.47611E+00_JPRB, 0.46788E+00_JPRB, 0.45916E+00_JPRB, 0.44867E+00_JPRB, 0.44182E+00_JPRB ,&
&  0.44184E+00_JPRB, 0.44318E+00_JPRB, 0.43745E+00_JPRB, 0.43801E+00_JPRB, 0.43942E+00_JPRB, 0.42147E+00_JPRB, 0.40145E+00_JPRB ,&
&  0.39032E+00_JPRB, 0.38280E+00_JPRB, 0.37941E+00_JPRB, 0.38053E+00_JPRB, 0.39487E+00_JPRB, 0.41585E+00_JPRB, 0.42310E+00_JPRB ,&
&  0.40703E+00_JPRB, 0.41919E+00_JPRB, 0.43511E+00_JPRB, 0.44536E+00_JPRB, 0.45540E+00_JPRB, 0.46882E+00_JPRB, 0.48246E+00_JPRB ,&
&  0.49434E+00_JPRB, 0.50429E+00_JPRB, 0.51017E+00_JPRB, 0.52223E+00_JPRB, 0.52797E+00_JPRB, 0.52205E+00_JPRB, 0.53037E+00_JPRB ,&
&  0.59340E+00_JPRB, 0.64808E+00_JPRB, 0.66604E+00_JPRB, 0.66569E+00_JPRB, 0.65801E+00_JPRB, 0.64395E+00_JPRB, 0.63367E+00_JPRB ,&
&  0.62419E+00_JPRB, 0.61423E+00_JPRB, 0.60523E+00_JPRB, 0.59705E+00_JPRB, 0.58952E+00_JPRB, 0.58059E+00_JPRB, 0.57473E+00_JPRB ,&
&  0.56919E+00_JPRB, 0.58189E+00_JPRB, 0.74083E+00_JPRB, 0.70995E+00_JPRB, 0.66418E+00_JPRB, 0.63392E+00_JPRB, 0.60445E+00_JPRB ,&
&  0.58276E+00_JPRB, 0.56825E+00_JPRB, 0.55944E+00_JPRB, 0.54946E+00_JPRB, 0.52881E+00_JPRB, 0.51332E+00_JPRB, 0.50608E+00_JPRB ,&
&  0.50237E+00_JPRB, 0.49586E+00_JPRB, 0.49011E+00_JPRB, 0.48475E+00_JPRB, 0.48051E+00_JPRB, 0.47841E+00_JPRB, 0.47487E+00_JPRB ,&
&  0.47250E+00_JPRB, 0.47513E+00_JPRB, 0.48032E+00_JPRB, 0.48875E+00_JPRB, 0.49554E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.50849E+00_JPRB, 0.52755E+00_JPRB, 0.54204E+00_JPRB, 0.54725E+00_JPRB, 0.54441E+00_JPRB, 0.51665E+00_JPRB, 0.49159E+00_JPRB ,&
&  0.47748E+00_JPRB, 0.47304E+00_JPRB, 0.47395E+00_JPRB, 0.47520E+00_JPRB, 0.47748E+00_JPRB, 0.48059E+00_JPRB, 0.48284E+00_JPRB ,&
&  0.48283E+00_JPRB, 0.48240E+00_JPRB, 0.48208E+00_JPRB, 0.48035E+00_JPRB, 0.47720E+00_JPRB, 0.47464E+00_JPRB, 0.47418E+00_JPRB ,&
&  0.47370E+00_JPRB, 0.47142E+00_JPRB, 0.46899E+00_JPRB, 0.46389E+00_JPRB, 0.46069E+00_JPRB, 0.45752E+00_JPRB, 0.45208E+00_JPRB ,&
&  0.44374E+00_JPRB, 0.44317E+00_JPRB, 0.44907E+00_JPRB, 0.45095E+00_JPRB, 0.45154E+00_JPRB, 0.43947E+00_JPRB, 0.41642E+00_JPRB ,&
&  0.40109E+00_JPRB, 0.39459E+00_JPRB, 0.40103E+00_JPRB, 0.42406E+00_JPRB, 0.44014E+00_JPRB, 0.42484E+00_JPRB, 0.43379E+00_JPRB ,&
&  0.43744E+00_JPRB, 0.44037E+00_JPRB, 0.44616E+00_JPRB, 0.45973E+00_JPRB, 0.47621E+00_JPRB, 0.49206E+00_JPRB, 0.50466E+00_JPRB ,&
&  0.51641E+00_JPRB, 0.52610E+00_JPRB, 0.53631E+00_JPRB, 0.52452E+00_JPRB, 0.52365E+00_JPRB, 0.55526E+00_JPRB, 0.59483E+00_JPRB ,&
&  0.63662E+00_JPRB, 0.66292E+00_JPRB, 0.67150E+00_JPRB, 0.69684E+00_JPRB, 0.70975E+00_JPRB, 0.70915E+00_JPRB, 0.69999E+00_JPRB ,&
&  0.68795E+00_JPRB, 0.66460E+00_JPRB, 0.64412E+00_JPRB, 0.63165E+00_JPRB, 0.62954E+00_JPRB, 0.63413E+00_JPRB, 0.63380E+00_JPRB ,&
&  0.63065E+00_JPRB, 0.61386E+00_JPRB, 0.59898E+00_JPRB, 0.56771E+00_JPRB, 0.56611E+00_JPRB, 0.57911E+00_JPRB, 0.59053E+00_JPRB ,&
&  0.58718E+00_JPRB, 0.57978E+00_JPRB, 0.57349E+00_JPRB, 0.56299E+00_JPRB, 0.52857E+00_JPRB, 0.50517E+00_JPRB, 0.49102E+00_JPRB ,&
&  0.48279E+00_JPRB, 0.47192E+00_JPRB, 0.47051E+00_JPRB, 0.46784E+00_JPRB, 0.47006E+00_JPRB, 0.47343E+00_JPRB, 0.47116E+00_JPRB ,&
&  0.47048E+00_JPRB, 0.47210E+00_JPRB, 0.47833E+00_JPRB, 0.48522E+00_JPRB, 0.49551E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.55981E+00_JPRB, 0.56714E+00_JPRB, 0.56407E+00_JPRB, 0.56464E+00_JPRB, 0.55176E+00_JPRB, 0.53608E+00_JPRB, 0.51941E+00_JPRB ,&
&  0.51305E+00_JPRB, 0.49805E+00_JPRB, 0.48217E+00_JPRB, 0.46647E+00_JPRB, 0.45756E+00_JPRB, 0.45378E+00_JPRB, 0.45656E+00_JPRB ,&
&  0.46296E+00_JPRB, 0.47076E+00_JPRB, 0.47599E+00_JPRB, 0.47478E+00_JPRB, 0.47125E+00_JPRB, 0.46956E+00_JPRB, 0.47015E+00_JPRB ,&
&  0.46938E+00_JPRB, 0.46766E+00_JPRB, 0.46508E+00_JPRB, 0.46194E+00_JPRB, 0.45933E+00_JPRB, 0.45740E+00_JPRB, 0.45725E+00_JPRB ,&
&  0.45769E+00_JPRB, 0.45734E+00_JPRB, 0.46041E+00_JPRB, 0.46181E+00_JPRB, 0.46360E+00_JPRB, 0.45334E+00_JPRB, 0.43317E+00_JPRB ,&
&  0.41642E+00_JPRB, 0.41516E+00_JPRB, 0.44303E+00_JPRB, 0.43625E+00_JPRB, 0.44869E+00_JPRB, 0.45873E+00_JPRB, 0.45863E+00_JPRB ,&
&  0.45797E+00_JPRB, 0.46078E+00_JPRB, 0.47376E+00_JPRB, 0.49157E+00_JPRB, 0.50576E+00_JPRB, 0.52115E+00_JPRB, 0.53454E+00_JPRB ,&
&  0.55085E+00_JPRB, 0.56718E+00_JPRB, 0.57643E+00_JPRB, 0.56367E+00_JPRB, 0.61062E+00_JPRB, 0.76749E+00_JPRB, 0.78980E+00_JPRB ,&
&  0.80325E+00_JPRB, 0.81299E+00_JPRB, 0.83725E+00_JPRB, 0.85397E+00_JPRB, 0.82917E+00_JPRB, 0.80817E+00_JPRB, 0.79238E+00_JPRB ,&
&  0.76182E+00_JPRB, 0.74020E+00_JPRB, 0.72562E+00_JPRB, 0.73361E+00_JPRB, 0.73540E+00_JPRB, 0.73929E+00_JPRB, 0.74642E+00_JPRB ,&
&  0.75144E+00_JPRB, 0.73712E+00_JPRB, 0.71885E+00_JPRB, 0.68708E+00_JPRB, 0.66997E+00_JPRB, 0.65812E+00_JPRB, 0.65134E+00_JPRB ,&
&  0.63953E+00_JPRB, 0.63146E+00_JPRB, 0.62132E+00_JPRB, 0.60342E+00_JPRB, 0.56113E+00_JPRB, 0.52071E+00_JPRB, 0.49585E+00_JPRB ,&
&  0.47793E+00_JPRB, 0.46266E+00_JPRB, 0.46032E+00_JPRB, 0.46289E+00_JPRB, 0.46955E+00_JPRB, 0.47732E+00_JPRB, 0.47598E+00_JPRB ,&
&  0.47971E+00_JPRB, 0.48728E+00_JPRB, 0.50275E+00_JPRB, 0.52291E+00_JPRB, 0.54148E+00_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.60269E+00_JPRB, 0.60364E+00_JPRB, 0.59555E+00_JPRB, 0.58761E+00_JPRB, 0.57827E+00_JPRB, 0.55851E+00_JPRB, 0.52829E+00_JPRB ,&
&  0.52215E+00_JPRB, 0.49153E+00_JPRB, 0.47249E+00_JPRB, 0.46627E+00_JPRB, 0.46293E+00_JPRB, 0.45607E+00_JPRB, 0.44691E+00_JPRB ,&
&  0.44775E+00_JPRB, 0.45705E+00_JPRB, 0.46788E+00_JPRB, 0.47236E+00_JPRB, 0.47437E+00_JPRB, 0.47273E+00_JPRB, 0.47242E+00_JPRB ,&
&  0.47222E+00_JPRB, 0.47218E+00_JPRB, 0.47253E+00_JPRB, 0.46845E+00_JPRB, 0.46516E+00_JPRB, 0.46430E+00_JPRB, 0.46316E+00_JPRB ,&
&  0.46745E+00_JPRB, 0.46943E+00_JPRB, 0.47159E+00_JPRB, 0.47065E+00_JPRB, 0.46782E+00_JPRB, 0.46754E+00_JPRB, 0.44575E+00_JPRB ,&
&  0.43101E+00_JPRB, 0.44368E+00_JPRB, 0.44350E+00_JPRB, 0.45176E+00_JPRB, 0.46331E+00_JPRB, 0.47192E+00_JPRB, 0.48008E+00_JPRB ,&
&  0.48649E+00_JPRB, 0.49494E+00_JPRB, 0.50573E+00_JPRB, 0.51927E+00_JPRB, 0.53672E+00_JPRB, 0.55443E+00_JPRB, 0.56973E+00_JPRB ,&
&  0.59928E+00_JPRB, 0.57840E+00_JPRB, 0.58008E+00_JPRB, 0.63315E+00_JPRB, 0.82153E+00_JPRB, 0.87856E+00_JPRB, 0.92376E+00_JPRB ,&
&  0.97472E+00_JPRB, 0.99597E+00_JPRB, 0.99715E+00_JPRB, 0.10024E+01_JPRB, 0.98195E+00_JPRB, 0.96047E+00_JPRB, 0.95726E+00_JPRB ,&
&  0.93539E+00_JPRB, 0.93267E+00_JPRB, 0.92786E+00_JPRB, 0.94025E+00_JPRB, 0.94017E+00_JPRB, 0.95303E+00_JPRB, 0.96338E+00_JPRB ,&
&  0.95649E+00_JPRB, 0.93184E+00_JPRB, 0.88282E+00_JPRB, 0.83171E+00_JPRB, 0.78443E+00_JPRB, 0.75713E+00_JPRB, 0.73450E+00_JPRB ,&
&  0.70971E+00_JPRB, 0.69139E+00_JPRB, 0.66927E+00_JPRB, 0.63812E+00_JPRB, 0.59816E+00_JPRB, 0.56550E+00_JPRB, 0.53909E+00_JPRB ,&
&  0.50247E+00_JPRB, 0.48619E+00_JPRB, 0.48420E+00_JPRB, 0.48533E+00_JPRB, 0.49222E+00_JPRB, 0.50717E+00_JPRB, 0.50713E+00_JPRB ,&
&  0.52331E+00_JPRB, 0.53762E+00_JPRB, 0.54513E+00_JPRB, 0.55868E+00_JPRB, 0.57810E+00_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.64370E+00_JPRB, 0.64161E+00_JPRB, 0.63176E+00_JPRB, 0.62163E+00_JPRB, 0.61055E+00_JPRB, 0.58131E+00_JPRB, 0.56041E+00_JPRB ,&
&  0.58870E+00_JPRB, 0.55089E+00_JPRB, 0.54299E+00_JPRB, 0.49858E+00_JPRB, 0.47627E+00_JPRB, 0.46467E+00_JPRB, 0.45449E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.46062E+00_JPRB, 0.46785E+00_JPRB, 0.47465E+00_JPRB, 0.47845E+00_JPRB, 0.47759E+00_JPRB, 0.47537E+00_JPRB ,&
&  0.47500E+00_JPRB, 0.47815E+00_JPRB, 0.48147E+00_JPRB, 0.48207E+00_JPRB, 0.48247E+00_JPRB, 0.48221E+00_JPRB, 0.47863E+00_JPRB ,&
&  0.47462E+00_JPRB, 0.47535E+00_JPRB, 0.48195E+00_JPRB, 0.48128E+00_JPRB, 0.48108E+00_JPRB, 0.47988E+00_JPRB, 0.47837E+00_JPRB ,&
&  0.46664E+00_JPRB, 0.45988E+00_JPRB, 0.46614E+00_JPRB, 0.47586E+00_JPRB, 0.48754E+00_JPRB, 0.49775E+00_JPRB, 0.50749E+00_JPRB ,&
&  0.51941E+00_JPRB, 0.53367E+00_JPRB, 0.55009E+00_JPRB, 0.56678E+00_JPRB, 0.58598E+00_JPRB, 0.60903E+00_JPRB, 0.62601E+00_JPRB ,&
&  0.67892E+00_JPRB, 0.63800E+00_JPRB, 0.74914E+00_JPRB, 0.84421E+00_JPRB, 0.10094E+01_JPRB, 0.11344E+01_JPRB, 0.11516E+01_JPRB ,&
&  0.12457E+01_JPRB, 0.12594E+01_JPRB, 0.12629E+01_JPRB, 0.12343E+01_JPRB, 0.12025E+01_JPRB, 0.11850E+01_JPRB, 0.11863E+01_JPRB ,&
&  0.11976E+01_JPRB, 0.11802E+01_JPRB, 0.12561E+01_JPRB, 0.12514E+01_JPRB, 0.12389E+01_JPRB, 0.12431E+01_JPRB, 0.12268E+01_JPRB ,&
&  0.11911E+01_JPRB, 0.11333E+01_JPRB, 0.10560E+01_JPRB, 0.93574E+00_JPRB, 0.88162E+00_JPRB, 0.83669E+00_JPRB, 0.77163E+00_JPRB ,&
&  0.73534E+00_JPRB, 0.69646E+00_JPRB, 0.66139E+00_JPRB, 0.65363E+00_JPRB, 0.60487E+00_JPRB, 0.56826E+00_JPRB, 0.54500E+00_JPRB ,&
&  0.51666E+00_JPRB, 0.50746E+00_JPRB, 0.51698E+00_JPRB, 0.52978E+00_JPRB, 0.53823E+00_JPRB, 0.54276E+00_JPRB, 0.56390E+00_JPRB ,&
&  0.55609E+00_JPRB, 0.57735E+00_JPRB, 0.61596E+00_JPRB, 0.62840E+00_JPRB, 0.63706E+00_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.70231E+00_JPRB, 0.69567E+00_JPRB, 0.68336E+00_JPRB, 0.66979E+00_JPRB, 0.64556E+00_JPRB, 0.61488E+00_JPRB, 0.58340E+00_JPRB ,&
&  0.59426E+00_JPRB, 0.58246E+00_JPRB, 0.56827E+00_JPRB, 0.54883E+00_JPRB, 0.52829E+00_JPRB, 0.50042E+00_JPRB, 0.46580E+00_JPRB ,&
&  0.45852E+00_JPRB, 0.46969E+00_JPRB, 0.48212E+00_JPRB, 0.49068E+00_JPRB, 0.49104E+00_JPRB, 0.48989E+00_JPRB, 0.49019E+00_JPRB ,&
&  0.49192E+00_JPRB, 0.49472E+00_JPRB, 0.49885E+00_JPRB, 0.50198E+00_JPRB, 0.50194E+00_JPRB, 0.50217E+00_JPRB, 0.50272E+00_JPRB ,&
&  0.48960E+00_JPRB, 0.48167E+00_JPRB, 0.47994E+00_JPRB, 0.47784E+00_JPRB, 0.50097E+00_JPRB, 0.51358E+00_JPRB, 0.52466E+00_JPRB ,&
&  0.52746E+00_JPRB, 0.52531E+00_JPRB, 0.52661E+00_JPRB, 0.53867E+00_JPRB, 0.55077E+00_JPRB, 0.55680E+00_JPRB, 0.56633E+00_JPRB ,&
&  0.58090E+00_JPRB, 0.59860E+00_JPRB, 0.61868E+00_JPRB, 0.64510E+00_JPRB, 0.67943E+00_JPRB, 0.68141E+00_JPRB, 0.72568E+00_JPRB ,&
&  0.79735E+00_JPRB, 0.92590E+00_JPRB, 0.10692E+01_JPRB, 0.10949E+01_JPRB, 0.12489E+01_JPRB, 0.13586E+01_JPRB, 0.13708E+01_JPRB ,&
&  0.14344E+01_JPRB, 0.14425E+01_JPRB, 0.15102E+01_JPRB, 0.15189E+01_JPRB, 0.15348E+01_JPRB, 0.15460E+01_JPRB, 0.15438E+01_JPRB ,&
&  0.15556E+01_JPRB, 0.16043E+01_JPRB, 0.16461E+01_JPRB, 0.16509E+01_JPRB, 0.16074E+01_JPRB, 0.15420E+01_JPRB, 0.14524E+01_JPRB ,&
&  0.13665E+01_JPRB, 0.12712E+01_JPRB, 0.11762E+01_JPRB, 0.11000E+01_JPRB, 0.10121E+01_JPRB, 0.92187E+00_JPRB, 0.83607E+00_JPRB ,&
&  0.75432E+00_JPRB, 0.66287E+00_JPRB, 0.62299E+00_JPRB, 0.62013E+00_JPRB, 0.60656E+00_JPRB, 0.56228E+00_JPRB, 0.53856E+00_JPRB ,&
&  0.53172E+00_JPRB, 0.54550E+00_JPRB, 0.56683E+00_JPRB, 0.57852E+00_JPRB, 0.59991E+00_JPRB, 0.62176E+00_JPRB, 0.59501E+00_JPRB ,&
&  0.57943E+00_JPRB, 0.63654E+00_JPRB, 0.67046E+00_JPRB, 0.68943E+00_JPRB, 0.70000E+00_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.75586E+00_JPRB, 0.74227E+00_JPRB, 0.72840E+00_JPRB, 0.71434E+00_JPRB, 0.68897E+00_JPRB, 0.65403E+00_JPRB, 0.62902E+00_JPRB ,&
&  0.62035E+00_JPRB, 0.60275E+00_JPRB, 0.58432E+00_JPRB, 0.56626E+00_JPRB, 0.55191E+00_JPRB, 0.53692E+00_JPRB, 0.51787E+00_JPRB ,&
&  0.50093E+00_JPRB, 0.50712E+00_JPRB, 0.50339E+00_JPRB, 0.50547E+00_JPRB, 0.51210E+00_JPRB, 0.50872E+00_JPRB, 0.50854E+00_JPRB ,&
&  0.51635E+00_JPRB, 0.51937E+00_JPRB, 0.51897E+00_JPRB, 0.51643E+00_JPRB, 0.51523E+00_JPRB, 0.51866E+00_JPRB, 0.52042E+00_JPRB ,&
&  0.50322E+00_JPRB, 0.48902E+00_JPRB, 0.47923E+00_JPRB, 0.48095E+00_JPRB, 0.50650E+00_JPRB, 0.54430E+00_JPRB, 0.57496E+00_JPRB ,&
&  0.59989E+00_JPRB, 0.61865E+00_JPRB, 0.62635E+00_JPRB, 0.63293E+00_JPRB, 0.63439E+00_JPRB, 0.63298E+00_JPRB, 0.63386E+00_JPRB ,&
&  0.63940E+00_JPRB, 0.65278E+00_JPRB, 0.67340E+00_JPRB, 0.71133E+00_JPRB, 0.73869E+00_JPRB, 0.76938E+00_JPRB, 0.85557E+00_JPRB ,&
&  0.99428E+00_JPRB, 0.11714E+01_JPRB, 0.12709E+01_JPRB, 0.14345E+01_JPRB, 0.15710E+01_JPRB, 0.16169E+01_JPRB, 0.16218E+01_JPRB ,&
&  0.16720E+01_JPRB, 0.16486E+01_JPRB, 0.17711E+01_JPRB, 0.17562E+01_JPRB, 0.17656E+01_JPRB, 0.18393E+01_JPRB, 0.18346E+01_JPRB ,&
&  0.18130E+01_JPRB, 0.17935E+01_JPRB, 0.18650E+01_JPRB, 0.18627E+01_JPRB, 0.18255E+01_JPRB, 0.17626E+01_JPRB, 0.16507E+01_JPRB ,&
&  0.15414E+01_JPRB, 0.14216E+01_JPRB, 0.12053E+01_JPRB, 0.10421E+01_JPRB, 0.88522E+00_JPRB, 0.86897E+00_JPRB, 0.79592E+00_JPRB ,&
&  0.71978E+00_JPRB, 0.62645E+00_JPRB, 0.61947E+00_JPRB, 0.61999E+00_JPRB, 0.63137E+00_JPRB, 0.61094E+00_JPRB, 0.57390E+00_JPRB ,&
&  0.56978E+00_JPRB, 0.59464E+00_JPRB, 0.60312E+00_JPRB, 0.61951E+00_JPRB, 0.65210E+00_JPRB, 0.70009E+00_JPRB, 0.62923E+00_JPRB ,&
&  0.64625E+00_JPRB, 0.71613E+00_JPRB, 0.74661E+00_JPRB, 0.76025E+00_JPRB, 0.76261E+00_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.77862E+00_JPRB, 0.75889E+00_JPRB, 0.74050E+00_JPRB, 0.72592E+00_JPRB, 0.70482E+00_JPRB, 0.68608E+00_JPRB, 0.66307E+00_JPRB ,&
&  0.64215E+00_JPRB, 0.61905E+00_JPRB, 0.60182E+00_JPRB, 0.59039E+00_JPRB, 0.58188E+00_JPRB, 0.56661E+00_JPRB, 0.56917E+00_JPRB ,&
&  0.54018E+00_JPRB, 0.53690E+00_JPRB, 0.51006E+00_JPRB, 0.51532E+00_JPRB, 0.54917E+00_JPRB, 0.53104E+00_JPRB, 0.53039E+00_JPRB ,&
&  0.53235E+00_JPRB, 0.53872E+00_JPRB, 0.53312E+00_JPRB, 0.52628E+00_JPRB, 0.52323E+00_JPRB, 0.52566E+00_JPRB, 0.53273E+00_JPRB ,&
&  0.52111E+00_JPRB, 0.49397E+00_JPRB, 0.48093E+00_JPRB, 0.48656E+00_JPRB, 0.52134E+00_JPRB, 0.56621E+00_JPRB, 0.61736E+00_JPRB ,&
&  0.64789E+00_JPRB, 0.67416E+00_JPRB, 0.69293E+00_JPRB, 0.70450E+00_JPRB, 0.70646E+00_JPRB, 0.70726E+00_JPRB, 0.71073E+00_JPRB ,&
&  0.71477E+00_JPRB, 0.73003E+00_JPRB, 0.76131E+00_JPRB, 0.80081E+00_JPRB, 0.81049E+00_JPRB, 0.87887E+00_JPRB, 0.10620E+01_JPRB ,&
&  0.13456E+01_JPRB, 0.14407E+01_JPRB, 0.15460E+01_JPRB, 0.18676E+01_JPRB, 0.20439E+01_JPRB, 0.20890E+01_JPRB, 0.20854E+01_JPRB ,&
&  0.20849E+01_JPRB, 0.20542E+01_JPRB, 0.20500E+01_JPRB, 0.21150E+01_JPRB, 0.20838E+01_JPRB, 0.20675E+01_JPRB, 0.20005E+01_JPRB ,&
&  0.19190E+01_JPRB, 0.19264E+01_JPRB, 0.19512E+01_JPRB, 0.19005E+01_JPRB, 0.18163E+01_JPRB, 0.17807E+01_JPRB, 0.17234E+01_JPRB ,&
&  0.16215E+01_JPRB, 0.14161E+01_JPRB, 0.10463E+01_JPRB, 0.91970E+00_JPRB, 0.72059E+00_JPRB, 0.71992E+00_JPRB, 0.82415E+00_JPRB ,&
&  0.79189E+00_JPRB, 0.73239E+00_JPRB, 0.69581E+00_JPRB, 0.68291E+00_JPRB, 0.68450E+00_JPRB, 0.70298E+00_JPRB, 0.66870E+00_JPRB ,&
&  0.62605E+00_JPRB, 0.63370E+00_JPRB, 0.65248E+00_JPRB, 0.69122E+00_JPRB, 0.75292E+00_JPRB, 0.79917E+00_JPRB, 0.74915E+00_JPRB ,&
&  0.79716E+00_JPRB, 0.81768E+00_JPRB, 0.82364E+00_JPRB, 0.82052E+00_JPRB, 0.80333E+00_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.76646E+00_JPRB, 0.74653E+00_JPRB, 0.72619E+00_JPRB, 0.71489E+00_JPRB, 0.70783E+00_JPRB, 0.70427E+00_JPRB, 0.68712E+00_JPRB ,&
&  0.65823E+00_JPRB, 0.63509E+00_JPRB, 0.62065E+00_JPRB, 0.61780E+00_JPRB, 0.61285E+00_JPRB, 0.59671E+00_JPRB, 0.59174E+00_JPRB ,&
&  0.61387E+00_JPRB, 0.59674E+00_JPRB, 0.54830E+00_JPRB, 0.51539E+00_JPRB, 0.53777E+00_JPRB, 0.53339E+00_JPRB, 0.53922E+00_JPRB ,&
&  0.53361E+00_JPRB, 0.55234E+00_JPRB, 0.54810E+00_JPRB, 0.54619E+00_JPRB, 0.54807E+00_JPRB, 0.55970E+00_JPRB, 0.58887E+00_JPRB ,&
&  0.59092E+00_JPRB, 0.55057E+00_JPRB, 0.53084E+00_JPRB, 0.54794E+00_JPRB, 0.61101E+00_JPRB, 0.60738E+00_JPRB, 0.65210E+00_JPRB ,&
&  0.68663E+00_JPRB, 0.72047E+00_JPRB, 0.74916E+00_JPRB, 0.77473E+00_JPRB, 0.79253E+00_JPRB, 0.80535E+00_JPRB, 0.81958E+00_JPRB ,&
&  0.83911E+00_JPRB, 0.85618E+00_JPRB, 0.85479E+00_JPRB, 0.88522E+00_JPRB, 0.97291E+00_JPRB, 0.11867E+01_JPRB, 0.14322E+01_JPRB ,&
&  0.16064E+01_JPRB, 0.16324E+01_JPRB, 0.16747E+01_JPRB, 0.19860E+01_JPRB, 0.22427E+01_JPRB, 0.23485E+01_JPRB, 0.24576E+01_JPRB ,&
&  0.24752E+01_JPRB, 0.24623E+01_JPRB, 0.24176E+01_JPRB, 0.22197E+01_JPRB, 0.19485E+01_JPRB, 0.17826E+01_JPRB, 0.17248E+01_JPRB ,&
&  0.16776E+01_JPRB, 0.17005E+01_JPRB, 0.17780E+01_JPRB, 0.17445E+01_JPRB, 0.17278E+01_JPRB, 0.16511E+01_JPRB, 0.17278E+01_JPRB ,&
&  0.16715E+01_JPRB, 0.10256E+01_JPRB, 0.79427E+00_JPRB, 0.83111E+00_JPRB, 0.69832E+00_JPRB, 0.70291E+00_JPRB, 0.77579E+00_JPRB ,&
&  0.75093E+00_JPRB, 0.74381E+00_JPRB, 0.76998E+00_JPRB, 0.77268E+00_JPRB, 0.77289E+00_JPRB, 0.81610E+00_JPRB, 0.81797E+00_JPRB ,&
&  0.78158E+00_JPRB, 0.73888E+00_JPRB, 0.76589E+00_JPRB, 0.82355E+00_JPRB, 0.89429E+00_JPRB, 0.91787E+00_JPRB, 0.91089E+00_JPRB ,&
&  0.90556E+00_JPRB, 0.89486E+00_JPRB, 0.87560E+00_JPRB, 0.83846E+00_JPRB, 0.79443E+00_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.78696E+00_JPRB, 0.76836E+00_JPRB, 0.74609E+00_JPRB, 0.74220E+00_JPRB, 0.74336E+00_JPRB, 0.73709E+00_JPRB, 0.71435E+00_JPRB ,&
&  0.66858E+00_JPRB, 0.64674E+00_JPRB, 0.63871E+00_JPRB, 0.63782E+00_JPRB, 0.63855E+00_JPRB, 0.63343E+00_JPRB, 0.62764E+00_JPRB ,&
&  0.61336E+00_JPRB, 0.63040E+00_JPRB, 0.57627E+00_JPRB, 0.54555E+00_JPRB, 0.50904E+00_JPRB, 0.52332E+00_JPRB, 0.54097E+00_JPRB ,&
&  0.55617E+00_JPRB, 0.56687E+00_JPRB, 0.59224E+00_JPRB, 0.62879E+00_JPRB, 0.66470E+00_JPRB, 0.69893E+00_JPRB, 0.77264E+00_JPRB ,&
&  0.76284E+00_JPRB, 0.70046E+00_JPRB, 0.64391E+00_JPRB, 0.66848E+00_JPRB, 0.70822E+00_JPRB, 0.67466E+00_JPRB, 0.73856E+00_JPRB ,&
&  0.78289E+00_JPRB, 0.82157E+00_JPRB, 0.85844E+00_JPRB, 0.87972E+00_JPRB, 0.88534E+00_JPRB, 0.85950E+00_JPRB, 0.84838E+00_JPRB ,&
&  0.84988E+00_JPRB, 0.87253E+00_JPRB, 0.91021E+00_JPRB, 0.94667E+00_JPRB, 0.11708E+01_JPRB, 0.14619E+01_JPRB, 0.16757E+01_JPRB ,&
&  0.18465E+01_JPRB, 0.15917E+01_JPRB, 0.14400E+01_JPRB, 0.19101E+01_JPRB, 0.26941E+01_JPRB, 0.29237E+01_JPRB, 0.27192E+01_JPRB ,&
&  0.28449E+01_JPRB, 0.25617E+01_JPRB, 0.22129E+01_JPRB, 0.19830E+01_JPRB, 0.18214E+01_JPRB, 0.17097E+01_JPRB, 0.16612E+01_JPRB ,&
&  0.15641E+01_JPRB, 0.15634E+01_JPRB, 0.16504E+01_JPRB, 0.17009E+01_JPRB, 0.16918E+01_JPRB, 0.16734E+01_JPRB, 0.17135E+01_JPRB ,&
&  0.15972E+01_JPRB, 0.12896E+01_JPRB, 0.91200E+00_JPRB, 0.83745E+00_JPRB, 0.68747E+00_JPRB, 0.67685E+00_JPRB, 0.74829E+00_JPRB ,&
&  0.73994E+00_JPRB, 0.78040E+00_JPRB, 0.80103E+00_JPRB, 0.79577E+00_JPRB, 0.86723E+00_JPRB, 0.90927E+00_JPRB, 0.98931E+00_JPRB ,&
&  0.10374E+01_JPRB, 0.90545E+00_JPRB, 0.96148E+00_JPRB, 0.10263E+01_JPRB, 0.10647E+01_JPRB, 0.10427E+01_JPRB, 0.10007E+01_JPRB ,&
&  0.97023E+00_JPRB, 0.94133E+00_JPRB, 0.90081E+00_JPRB, 0.84662E+00_JPRB, 0.80895E+00_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.79591E+00_JPRB, 0.78073E+00_JPRB, 0.75481E+00_JPRB, 0.75135E+00_JPRB, 0.74693E+00_JPRB, 0.73628E+00_JPRB, 0.72318E+00_JPRB ,&
&  0.70830E+00_JPRB, 0.70362E+00_JPRB, 0.69295E+00_JPRB, 0.68343E+00_JPRB, 0.67805E+00_JPRB, 0.66565E+00_JPRB, 0.65203E+00_JPRB ,&
&  0.63074E+00_JPRB, 0.62111E+00_JPRB, 0.55768E+00_JPRB, 0.51915E+00_JPRB, 0.49798E+00_JPRB, 0.50788E+00_JPRB, 0.53544E+00_JPRB ,&
&  0.57748E+00_JPRB, 0.63271E+00_JPRB, 0.72681E+00_JPRB, 0.81726E+00_JPRB, 0.94644E+00_JPRB, 0.10577E+01_JPRB, 0.11404E+01_JPRB ,&
&  0.11523E+01_JPRB, 0.90872E+00_JPRB, 0.88621E+00_JPRB, 0.83467E+00_JPRB, 0.84961E+00_JPRB, 0.83361E+00_JPRB, 0.84311E+00_JPRB ,&
&  0.85057E+00_JPRB, 0.87891E+00_JPRB, 0.90175E+00_JPRB, 0.89063E+00_JPRB, 0.87166E+00_JPRB, 0.86019E+00_JPRB, 0.89334E+00_JPRB ,&
&  0.93602E+00_JPRB, 0.94217E+00_JPRB, 0.97811E+00_JPRB, 0.11553E+01_JPRB, 0.15364E+01_JPRB, 0.18507E+01_JPRB, 0.20532E+01_JPRB ,&
&  0.19480E+01_JPRB, 0.18633E+01_JPRB, 0.25986E+01_JPRB, 0.29095E+01_JPRB, 0.27176E+01_JPRB, 0.30399E+01_JPRB, 0.30109E+01_JPRB ,&
&  0.26944E+01_JPRB, 0.21593E+01_JPRB, 0.19054E+01_JPRB, 0.17352E+01_JPRB, 0.20259E+01_JPRB, 0.19137E+01_JPRB, 0.17050E+01_JPRB ,&
&  0.15225E+01_JPRB, 0.16120E+01_JPRB, 0.16548E+01_JPRB, 0.17315E+01_JPRB, 0.18324E+01_JPRB, 0.18692E+01_JPRB, 0.16955E+01_JPRB ,&
&  0.14784E+01_JPRB, 0.15409E+01_JPRB, 0.12281E+01_JPRB, 0.92955E+00_JPRB, 0.79189E+00_JPRB, 0.78677E+00_JPRB, 0.80606E+00_JPRB ,&
&  0.82521E+00_JPRB, 0.83767E+00_JPRB, 0.84066E+00_JPRB, 0.89574E+00_JPRB, 0.10133E+01_JPRB, 0.10890E+01_JPRB, 0.11431E+01_JPRB ,&
&  0.11287E+01_JPRB, 0.12139E+01_JPRB, 0.12235E+01_JPRB, 0.12150E+01_JPRB, 0.11634E+01_JPRB, 0.10729E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.97890E+00_JPRB, 0.95877E+00_JPRB, 0.90598E+00_JPRB, 0.84858E+00_JPRB, 0.80605E+00_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.80912E+00_JPRB, 0.80341E+00_JPRB, 0.79206E+00_JPRB, 0.77675E+00_JPRB, 0.76390E+00_JPRB, 0.75779E+00_JPRB, 0.76075E+00_JPRB ,&
&  0.75572E+00_JPRB, 0.73794E+00_JPRB, 0.73874E+00_JPRB, 0.74787E+00_JPRB, 0.74313E+00_JPRB, 0.71647E+00_JPRB, 0.68802E+00_JPRB ,&
&  0.66735E+00_JPRB, 0.60893E+00_JPRB, 0.53730E+00_JPRB, 0.53615E+00_JPRB, 0.53470E+00_JPRB, 0.52363E+00_JPRB, 0.55644E+00_JPRB ,&
&  0.61733E+00_JPRB, 0.74535E+00_JPRB, 0.90174E+00_JPRB, 0.10905E+01_JPRB, 0.12253E+01_JPRB, 0.12882E+01_JPRB, 0.13171E+01_JPRB ,&
&  0.12134E+01_JPRB, 0.11537E+01_JPRB, 0.11091E+01_JPRB, 0.10441E+01_JPRB, 0.10175E+01_JPRB, 0.10004E+01_JPRB, 0.96500E+00_JPRB ,&
&  0.92905E+00_JPRB, 0.92215E+00_JPRB, 0.92534E+00_JPRB, 0.90026E+00_JPRB, 0.90324E+00_JPRB, 0.93154E+00_JPRB, 0.97623E+00_JPRB ,&
&  0.97852E+00_JPRB, 0.98842E+00_JPRB, 0.10934E+01_JPRB, 0.13916E+01_JPRB, 0.15100E+01_JPRB, 0.17622E+01_JPRB, 0.20206E+01_JPRB ,&
&  0.27555E+01_JPRB, 0.30309E+01_JPRB, 0.28696E+01_JPRB, 0.35273E+01_JPRB, 0.22187E+01_JPRB, 0.24720E+01_JPRB, 0.27502E+01_JPRB ,&
&  0.25175E+01_JPRB, 0.23579E+01_JPRB, 0.24752E+01_JPRB, 0.22137E+01_JPRB, 0.19125E+01_JPRB, 0.25647E+01_JPRB, 0.21087E+01_JPRB ,&
&  0.17879E+01_JPRB, 0.17897E+01_JPRB, 0.18920E+01_JPRB, 0.21073E+01_JPRB, 0.18258E+01_JPRB, 0.16137E+01_JPRB, 0.13037E+01_JPRB ,&
&  0.12781E+01_JPRB, 0.11456E+01_JPRB, 0.12190E+01_JPRB, 0.10641E+01_JPRB, 0.10403E+01_JPRB, 0.95849E+00_JPRB, 0.88551E+00_JPRB ,&
&  0.89098E+00_JPRB, 0.88745E+00_JPRB, 0.91882E+00_JPRB, 0.10652E+01_JPRB, 0.13012E+01_JPRB, 0.12314E+01_JPRB, 0.13158E+01_JPRB ,&
&  0.13838E+01_JPRB, 0.14100E+01_JPRB, 0.13230E+01_JPRB, 0.12606E+01_JPRB, 0.11288E+01_JPRB, 0.10479E+01_JPRB, 0.10087E+01_JPRB ,&
&  0.97871E+00_JPRB, 0.92358E+00_JPRB, 0.86240E+00_JPRB, 0.83883E+00_JPRB, 0.80583E+00_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.81739E+00_JPRB, 0.80345E+00_JPRB, 0.80176E+00_JPRB, 0.79270E+00_JPRB, 0.79593E+00_JPRB, 0.79475E+00_JPRB, 0.79547E+00_JPRB ,&
&  0.78391E+00_JPRB, 0.77022E+00_JPRB, 0.77551E+00_JPRB, 0.77751E+00_JPRB, 0.76620E+00_JPRB, 0.74710E+00_JPRB, 0.70864E+00_JPRB ,&
&  0.68526E+00_JPRB, 0.64131E+00_JPRB, 0.55757E+00_JPRB, 0.53645E+00_JPRB, 0.55199E+00_JPRB, 0.55158E+00_JPRB, 0.57891E+00_JPRB ,&
&  0.73277E+00_JPRB, 0.94768E+00_JPRB, 0.11057E+01_JPRB, 0.12530E+01_JPRB, 0.13497E+01_JPRB, 0.13251E+01_JPRB, 0.12911E+01_JPRB ,&
&  0.12908E+01_JPRB, 0.11499E+01_JPRB, 0.10594E+01_JPRB, 0.10486E+01_JPRB, 0.10173E+01_JPRB, 0.97895E+00_JPRB, 0.94544E+00_JPRB ,&
&  0.91154E+00_JPRB, 0.87971E+00_JPRB, 0.85851E+00_JPRB, 0.84530E+00_JPRB, 0.86974E+00_JPRB, 0.90603E+00_JPRB, 0.95116E+00_JPRB ,&
&  0.98393E+00_JPRB, 0.10568E+01_JPRB, 0.12796E+01_JPRB, 0.15763E+01_JPRB, 0.17849E+01_JPRB, 0.20300E+01_JPRB, 0.26198E+01_JPRB ,&
&  0.29521E+01_JPRB, 0.29323E+01_JPRB, 0.31519E+01_JPRB, 0.34373E+01_JPRB, 0.30430E+01_JPRB, 0.25333E+01_JPRB, 0.26615E+01_JPRB ,&
&  0.20634E+01_JPRB, 0.18396E+01_JPRB, 0.18022E+01_JPRB, 0.18582E+01_JPRB, 0.24179E+01_JPRB, 0.35547E+01_JPRB, 0.31317E+01_JPRB ,&
&  0.27821E+01_JPRB, 0.24134E+01_JPRB, 0.24473E+01_JPRB, 0.22465E+01_JPRB, 0.19403E+01_JPRB, 0.14194E+01_JPRB, 0.16998E+01_JPRB ,&
&  0.17324E+01_JPRB, 0.17489E+01_JPRB, 0.15701E+01_JPRB, 0.12832E+01_JPRB, 0.10634E+01_JPRB, 0.11222E+01_JPRB, 0.10767E+01_JPRB ,&
&  0.10415E+01_JPRB, 0.10292E+01_JPRB, 0.12903E+01_JPRB, 0.14370E+01_JPRB, 0.15262E+01_JPRB, 0.15622E+01_JPRB, 0.16989E+01_JPRB ,&
&  0.15863E+01_JPRB, 0.15566E+01_JPRB, 0.14245E+01_JPRB, 0.12274E+01_JPRB, 0.11015E+01_JPRB, 0.10368E+01_JPRB, 0.98842E+00_JPRB ,&
&  0.94003E+00_JPRB, 0.87005E+00_JPRB, 0.83969E+00_JPRB, 0.81533E+00_JPRB, 0.80131E+00_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.81897E+00_JPRB, 0.81967E+00_JPRB, 0.81514E+00_JPRB, 0.81115E+00_JPRB, 0.79940E+00_JPRB, 0.78482E+00_JPRB, 0.79013E+00_JPRB ,&
&  0.78929E+00_JPRB, 0.77794E+00_JPRB, 0.76033E+00_JPRB, 0.74863E+00_JPRB, 0.73399E+00_JPRB, 0.72892E+00_JPRB, 0.71412E+00_JPRB ,&
&  0.70548E+00_JPRB, 0.70352E+00_JPRB, 0.60446E+00_JPRB, 0.58122E+00_JPRB, 0.58922E+00_JPRB, 0.60678E+00_JPRB, 0.63982E+00_JPRB ,&
&  0.87160E+00_JPRB, 0.11114E+01_JPRB, 0.11973E+01_JPRB, 0.12808E+01_JPRB, 0.13410E+01_JPRB, 0.13041E+01_JPRB, 0.12947E+01_JPRB ,&
&  0.12985E+01_JPRB, 0.12152E+01_JPRB, 0.11144E+01_JPRB, 0.10493E+01_JPRB, 0.98468E+00_JPRB, 0.94499E+00_JPRB, 0.91420E+00_JPRB ,&
&  0.88115E+00_JPRB, 0.85376E+00_JPRB, 0.84398E+00_JPRB, 0.85622E+00_JPRB, 0.85430E+00_JPRB, 0.92419E+00_JPRB, 0.10043E+01_JPRB ,&
&  0.10827E+01_JPRB, 0.12535E+01_JPRB, 0.14714E+01_JPRB, 0.17843E+01_JPRB, 0.20876E+01_JPRB, 0.23986E+01_JPRB, 0.33419E+01_JPRB ,&
&  0.32593E+01_JPRB, 0.32899E+01_JPRB, 0.33789E+01_JPRB, 0.42010E+01_JPRB, 0.29891E+01_JPRB, 0.24558E+01_JPRB, 0.25208E+01_JPRB ,&
&  0.19390E+01_JPRB, 0.17892E+01_JPRB, 0.21146E+01_JPRB, 0.27581E+01_JPRB, 0.28820E+01_JPRB, 0.32023E+01_JPRB, 0.41354E+01_JPRB ,&
&  0.36537E+01_JPRB, 0.31684E+01_JPRB, 0.29603E+01_JPRB, 0.28629E+01_JPRB, 0.14829E+01_JPRB, 0.11437E+01_JPRB, 0.16892E+01_JPRB ,&
&  0.17438E+01_JPRB, 0.11867E+01_JPRB, 0.91598E+00_JPRB, 0.10922E+01_JPRB, 0.85912E+00_JPRB, 0.95300E+00_JPRB, 0.12022E+01_JPRB ,&
&  0.12163E+01_JPRB, 0.13170E+01_JPRB, 0.18375E+01_JPRB, 0.17830E+01_JPRB, 0.17264E+01_JPRB, 0.21899E+01_JPRB, 0.20459E+01_JPRB ,&
&  0.18276E+01_JPRB, 0.14988E+01_JPRB, 0.13635E+01_JPRB, 0.11678E+01_JPRB, 0.10622E+01_JPRB, 0.98446E+00_JPRB, 0.92443E+00_JPRB ,&
&  0.85808E+00_JPRB, 0.82596E+00_JPRB, 0.83015E+00_JPRB, 0.83035E+00_JPRB, 0.80608E+00_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.82239E+00_JPRB, 0.81143E+00_JPRB, 0.79677E+00_JPRB, 0.80165E+00_JPRB, 0.78542E+00_JPRB, 0.76407E+00_JPRB, 0.79457E+00_JPRB ,&
&  0.79657E+00_JPRB, 0.76280E+00_JPRB, 0.72819E+00_JPRB, 0.71651E+00_JPRB, 0.71177E+00_JPRB, 0.73596E+00_JPRB, 0.74438E+00_JPRB ,&
&  0.75119E+00_JPRB, 0.74684E+00_JPRB, 0.72217E+00_JPRB, 0.70806E+00_JPRB, 0.71317E+00_JPRB, 0.72571E+00_JPRB, 0.81537E+00_JPRB ,&
&  0.10810E+01_JPRB, 0.12132E+01_JPRB, 0.12161E+01_JPRB, 0.12476E+01_JPRB, 0.12836E+01_JPRB, 0.12635E+01_JPRB, 0.13327E+01_JPRB ,&
&  0.12943E+01_JPRB, 0.11898E+01_JPRB, 0.11025E+01_JPRB, 0.10260E+01_JPRB, 0.98374E+00_JPRB, 0.95531E+00_JPRB, 0.93015E+00_JPRB ,&
&  0.88871E+00_JPRB, 0.86038E+00_JPRB, 0.87315E+00_JPRB, 0.85754E+00_JPRB, 0.89445E+00_JPRB, 0.98979E+00_JPRB, 0.10973E+01_JPRB ,&
&  0.12368E+01_JPRB, 0.14436E+01_JPRB, 0.16582E+01_JPRB, 0.20312E+01_JPRB, 0.25970E+01_JPRB, 0.28800E+01_JPRB, 0.26939E+01_JPRB ,&
&  0.24974E+01_JPRB, 0.24812E+01_JPRB, 0.32918E+01_JPRB, 0.35967E+01_JPRB, 0.31048E+01_JPRB, 0.27706E+01_JPRB, 0.25455E+01_JPRB ,&
&  0.22501E+01_JPRB, 0.21695E+01_JPRB, 0.24566E+01_JPRB, 0.31916E+01_JPRB, 0.35891E+01_JPRB, 0.33781E+01_JPRB, 0.42880E+01_JPRB ,&
&  0.43027E+01_JPRB, 0.37412E+01_JPRB, 0.25841E+01_JPRB, 0.19083E+01_JPRB, 0.16696E+01_JPRB, 0.13573E+01_JPRB, 0.98580E+00_JPRB ,&
&  0.89805E+00_JPRB, 0.88839E+00_JPRB, 0.83115E+00_JPRB, 0.83545E+00_JPRB, 0.80487E+00_JPRB, 0.95929E+00_JPRB, 0.11872E+01_JPRB ,&
&  0.15206E+01_JPRB, 0.18799E+01_JPRB, 0.20689E+01_JPRB, 0.20129E+01_JPRB, 0.18997E+01_JPRB, 0.24588E+01_JPRB, 0.22223E+01_JPRB ,&
&  0.15693E+01_JPRB, 0.13496E+01_JPRB, 0.12151E+01_JPRB, 0.10823E+01_JPRB, 0.99177E+00_JPRB, 0.90941E+00_JPRB, 0.84929E+00_JPRB ,&
&  0.81953E+00_JPRB, 0.81951E+00_JPRB, 0.82295E+00_JPRB, 0.81950E+00_JPRB, 0.80052E+00_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.82312E+00_JPRB, 0.80777E+00_JPRB, 0.78812E+00_JPRB, 0.77416E+00_JPRB, 0.74301E+00_JPRB, 0.75072E+00_JPRB, 0.79883E+00_JPRB ,&
&  0.80032E+00_JPRB, 0.74112E+00_JPRB, 0.70553E+00_JPRB, 0.70502E+00_JPRB, 0.73660E+00_JPRB, 0.76912E+00_JPRB, 0.77554E+00_JPRB ,&
&  0.79465E+00_JPRB, 0.80065E+00_JPRB, 0.84927E+00_JPRB, 0.87806E+00_JPRB, 0.94556E+00_JPRB, 0.94594E+00_JPRB, 0.11147E+01_JPRB ,&
&  0.12876E+01_JPRB, 0.12515E+01_JPRB, 0.12314E+01_JPRB, 0.12618E+01_JPRB, 0.13054E+01_JPRB, 0.13283E+01_JPRB, 0.13456E+01_JPRB ,&
&  0.12404E+01_JPRB, 0.11439E+01_JPRB, 0.10509E+01_JPRB, 0.10007E+01_JPRB, 0.98093E+00_JPRB, 0.93943E+00_JPRB, 0.91416E+00_JPRB ,&
&  0.88887E+00_JPRB, 0.88924E+00_JPRB, 0.87123E+00_JPRB, 0.89713E+00_JPRB, 0.99957E+00_JPRB, 0.11288E+01_JPRB, 0.12703E+01_JPRB ,&
&  0.14328E+01_JPRB, 0.16727E+01_JPRB, 0.19410E+01_JPRB, 0.23762E+01_JPRB, 0.23548E+01_JPRB, 0.20428E+01_JPRB, 0.23416E+01_JPRB ,&
&  0.25896E+01_JPRB, 0.28385E+01_JPRB, 0.27966E+01_JPRB, 0.35160E+01_JPRB, 0.32977E+01_JPRB, 0.27385E+01_JPRB, 0.24871E+01_JPRB ,&
&  0.22391E+01_JPRB, 0.26565E+01_JPRB, 0.30745E+01_JPRB, 0.35150E+01_JPRB, 0.46363E+01_JPRB, 0.44053E+01_JPRB, 0.47492E+01_JPRB ,&
&  0.49193E+01_JPRB, 0.40565E+01_JPRB, 0.29165E+01_JPRB, 0.21758E+01_JPRB, 0.24309E+01_JPRB, 0.24301E+01_JPRB, 0.11625E+01_JPRB ,&
&  0.96567E+00_JPRB, 0.90452E+00_JPRB, 0.87591E+00_JPRB, 0.81947E+00_JPRB, 0.80673E+00_JPRB, 0.98541E+00_JPRB, 0.15643E+01_JPRB ,&
&  0.15089E+01_JPRB, 0.23793E+01_JPRB, 0.22473E+01_JPRB, 0.21691E+01_JPRB, 0.20070E+01_JPRB, 0.24989E+01_JPRB, 0.20717E+01_JPRB ,&
&  0.16353E+01_JPRB, 0.14074E+01_JPRB, 0.11864E+01_JPRB, 0.10687E+01_JPRB, 0.95677E+00_JPRB, 0.86884E+00_JPRB, 0.84087E+00_JPRB ,&
&  0.81598E+00_JPRB, 0.78755E+00_JPRB, 0.78098E+00_JPRB, 0.78853E+00_JPRB, 0.80183E+00_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.80795E+00_JPRB, 0.79152E+00_JPRB, 0.78488E+00_JPRB, 0.78250E+00_JPRB, 0.76124E+00_JPRB, 0.79341E+00_JPRB, 0.81007E+00_JPRB ,&
&  0.78997E+00_JPRB, 0.74907E+00_JPRB, 0.73097E+00_JPRB, 0.74466E+00_JPRB, 0.77763E+00_JPRB, 0.81308E+00_JPRB, 0.84410E+00_JPRB ,&
&  0.88027E+00_JPRB, 0.92360E+00_JPRB, 0.99582E+00_JPRB, 0.11480E+01_JPRB, 0.12395E+01_JPRB, 0.12676E+01_JPRB, 0.14217E+01_JPRB ,&
&  0.13642E+01_JPRB, 0.12153E+01_JPRB, 0.12486E+01_JPRB, 0.13184E+01_JPRB, 0.13004E+01_JPRB, 0.12802E+01_JPRB, 0.12446E+01_JPRB ,&
&  0.11599E+01_JPRB, 0.11033E+01_JPRB, 0.10378E+01_JPRB, 0.10347E+01_JPRB, 0.10378E+01_JPRB, 0.99600E+00_JPRB, 0.97737E+00_JPRB ,&
&  0.96406E+00_JPRB, 0.94425E+00_JPRB, 0.95378E+00_JPRB, 0.99875E+00_JPRB, 0.11407E+01_JPRB, 0.12730E+01_JPRB, 0.14421E+01_JPRB ,&
&  0.16585E+01_JPRB, 0.18817E+01_JPRB, 0.21362E+01_JPRB, 0.24495E+01_JPRB, 0.20709E+01_JPRB, 0.24772E+01_JPRB, 0.26179E+01_JPRB ,&
&  0.27586E+01_JPRB, 0.26884E+01_JPRB, 0.24482E+01_JPRB, 0.29316E+01_JPRB, 0.30429E+01_JPRB, 0.25874E+01_JPRB, 0.23162E+01_JPRB ,&
&  0.26055E+01_JPRB, 0.29704E+01_JPRB, 0.37094E+01_JPRB, 0.43137E+01_JPRB, 0.49419E+01_JPRB, 0.68343E+01_JPRB, 0.48198E+01_JPRB ,&
&  0.40215E+01_JPRB, 0.39286E+01_JPRB, 0.32182E+01_JPRB, 0.26362E+01_JPRB, 0.28190E+01_JPRB, 0.35039E+01_JPRB, 0.25417E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.84471E+00_JPRB, 0.92254E+00_JPRB, 0.92760E+00_JPRB, 0.83189E+00_JPRB, 0.98504E+00_JPRB, 0.24033E+01_JPRB ,&
&  0.18516E+01_JPRB, 0.25748E+01_JPRB, 0.22535E+01_JPRB, 0.21193E+01_JPRB, 0.21222E+01_JPRB, 0.23827E+01_JPRB, 0.22726E+01_JPRB ,&
&  0.15833E+01_JPRB, 0.13342E+01_JPRB, 0.11412E+01_JPRB, 0.10182E+01_JPRB, 0.91217E+00_JPRB, 0.84944E+00_JPRB, 0.81453E+00_JPRB ,&
&  0.78882E+00_JPRB, 0.77676E+00_JPRB, 0.77360E+00_JPRB, 0.77976E+00_JPRB, 0.79513E+00_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.81323E+00_JPRB, 0.78210E+00_JPRB, 0.77382E+00_JPRB, 0.76410E+00_JPRB, 0.80649E+00_JPRB, 0.84842E+00_JPRB, 0.81283E+00_JPRB ,&
&  0.81829E+00_JPRB, 0.81610E+00_JPRB, 0.82594E+00_JPRB, 0.81960E+00_JPRB, 0.84217E+00_JPRB, 0.91368E+00_JPRB, 0.97628E+00_JPRB ,&
&  0.10490E+01_JPRB, 0.11279E+01_JPRB, 0.12322E+01_JPRB, 0.13939E+01_JPRB, 0.15337E+01_JPRB, 0.14942E+01_JPRB, 0.14505E+01_JPRB ,&
&  0.13380E+01_JPRB, 0.12338E+01_JPRB, 0.12144E+01_JPRB, 0.13327E+01_JPRB, 0.13325E+01_JPRB, 0.12937E+01_JPRB, 0.12809E+01_JPRB ,&
&  0.12153E+01_JPRB, 0.11819E+01_JPRB, 0.11331E+01_JPRB, 0.11421E+01_JPRB, 0.11168E+01_JPRB, 0.11304E+01_JPRB, 0.12003E+01_JPRB ,&
&  0.11661E+01_JPRB, 0.11501E+01_JPRB, 0.11706E+01_JPRB, 0.12235E+01_JPRB, 0.13672E+01_JPRB, 0.15021E+01_JPRB, 0.16780E+01_JPRB ,&
&  0.18783E+01_JPRB, 0.20526E+01_JPRB, 0.22750E+01_JPRB, 0.23336E+01_JPRB, 0.23647E+01_JPRB, 0.24753E+01_JPRB, 0.27127E+01_JPRB ,&
&  0.27816E+01_JPRB, 0.26952E+01_JPRB, 0.25747E+01_JPRB, 0.26314E+01_JPRB, 0.26097E+01_JPRB, 0.24400E+01_JPRB, 0.24185E+01_JPRB ,&
&  0.30344E+01_JPRB, 0.39016E+01_JPRB, 0.45261E+01_JPRB, 0.50764E+01_JPRB, 0.54161E+01_JPRB, 0.68384E+01_JPRB, 0.62231E+01_JPRB ,&
&  0.47007E+01_JPRB, 0.35856E+01_JPRB, 0.31858E+01_JPRB, 0.32876E+01_JPRB, 0.33920E+01_JPRB, 0.33897E+01_JPRB, 0.36781E+01_JPRB ,&
&  0.35586E+01_JPRB, 0.19997E+01_JPRB, 0.15253E+01_JPRB, 0.20227E+01_JPRB, 0.13913E+01_JPRB, 0.10311E+01_JPRB, 0.14153E+01_JPRB ,&
&  0.19814E+01_JPRB, 0.23765E+01_JPRB, 0.22984E+01_JPRB, 0.20869E+01_JPRB, 0.22113E+01_JPRB, 0.21857E+01_JPRB, 0.19073E+01_JPRB ,&
&  0.13977E+01_JPRB, 0.11596E+01_JPRB, 0.10170E+01_JPRB, 0.92184E+00_JPRB, 0.84347E+00_JPRB, 0.81439E+00_JPRB, 0.80913E+00_JPRB ,&
&  0.79059E+00_JPRB, 0.79545E+00_JPRB, 0.79549E+00_JPRB, 0.79251E+00_JPRB, 0.80894E+00_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.79710E+00_JPRB, 0.77254E+00_JPRB, 0.75607E+00_JPRB, 0.76323E+00_JPRB, 0.83400E+00_JPRB, 0.87610E+00_JPRB, 0.86937E+00_JPRB ,&
&  0.86673E+00_JPRB, 0.87279E+00_JPRB, 0.90868E+00_JPRB, 0.89940E+00_JPRB, 0.96479E+00_JPRB, 0.11040E+01_JPRB, 0.12030E+01_JPRB ,&
&  0.12812E+01_JPRB, 0.13643E+01_JPRB, 0.14712E+01_JPRB, 0.16410E+01_JPRB, 0.18675E+01_JPRB, 0.21500E+01_JPRB, 0.13393E+01_JPRB ,&
&  0.11942E+01_JPRB, 0.12872E+01_JPRB, 0.12802E+01_JPRB, 0.14316E+01_JPRB, 0.14066E+01_JPRB, 0.13933E+01_JPRB, 0.13399E+01_JPRB ,&
&  0.13373E+01_JPRB, 0.13271E+01_JPRB, 0.13377E+01_JPRB, 0.13785E+01_JPRB, 0.14052E+01_JPRB, 0.15411E+01_JPRB, 0.16130E+01_JPRB ,&
&  0.15764E+01_JPRB, 0.15695E+01_JPRB, 0.16370E+01_JPRB, 0.17055E+01_JPRB, 0.17939E+01_JPRB, 0.18784E+01_JPRB, 0.19888E+01_JPRB ,&
&  0.21126E+01_JPRB, 0.22633E+01_JPRB, 0.24655E+01_JPRB, 0.23550E+01_JPRB, 0.23456E+01_JPRB, 0.24121E+01_JPRB, 0.25888E+01_JPRB ,&
&  0.24800E+01_JPRB, 0.22363E+01_JPRB, 0.23220E+01_JPRB, 0.24606E+01_JPRB, 0.24283E+01_JPRB, 0.23822E+01_JPRB, 0.28244E+01_JPRB ,&
&  0.39033E+01_JPRB, 0.49797E+01_JPRB, 0.61571E+01_JPRB, 0.55371E+01_JPRB, 0.55705E+01_JPRB, 0.60246E+01_JPRB, 0.56878E+01_JPRB ,&
&  0.50989E+01_JPRB, 0.41607E+01_JPRB, 0.37390E+01_JPRB, 0.35984E+01_JPRB, 0.33231E+01_JPRB, 0.29038E+01_JPRB, 0.28524E+01_JPRB ,&
&  0.33213E+01_JPRB, 0.40301E+01_JPRB, 0.33799E+01_JPRB, 0.21918E+01_JPRB, 0.15914E+01_JPRB, 0.11853E+01_JPRB, 0.13556E+01_JPRB ,&
&  0.20961E+01_JPRB, 0.21154E+01_JPRB, 0.20565E+01_JPRB, 0.23285E+01_JPRB, 0.20043E+01_JPRB, 0.17913E+01_JPRB, 0.14316E+01_JPRB ,&
&  0.11061E+01_JPRB, 0.91331E+00_JPRB, 0.84070E+00_JPRB, 0.78170E+00_JPRB, 0.77626E+00_JPRB, 0.78413E+00_JPRB, 0.80595E+00_JPRB ,&
&  0.82584E+00_JPRB, 0.82730E+00_JPRB, 0.81409E+00_JPRB, 0.80318E+00_JPRB, 0.80934E+00_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.80646E+00_JPRB, 0.78285E+00_JPRB, 0.76067E+00_JPRB, 0.78680E+00_JPRB, 0.85805E+00_JPRB, 0.89152E+00_JPRB, 0.90456E+00_JPRB ,&
&  0.91402E+00_JPRB, 0.91379E+00_JPRB, 0.96977E+00_JPRB, 0.10062E+01_JPRB, 0.11320E+01_JPRB, 0.12679E+01_JPRB, 0.13767E+01_JPRB ,&
&  0.14667E+01_JPRB, 0.15418E+01_JPRB, 0.16517E+01_JPRB, 0.18111E+01_JPRB, 0.19681E+01_JPRB, 0.20939E+01_JPRB, 0.16086E+01_JPRB ,&
&  0.10975E+01_JPRB, 0.15745E+01_JPRB, 0.14210E+01_JPRB, 0.15748E+01_JPRB, 0.16011E+01_JPRB, 0.17004E+01_JPRB, 0.16241E+01_JPRB ,&
&  0.16183E+01_JPRB, 0.16019E+01_JPRB, 0.16926E+01_JPRB, 0.17469E+01_JPRB, 0.17943E+01_JPRB, 0.19084E+01_JPRB, 0.20069E+01_JPRB ,&
&  0.20693E+01_JPRB, 0.21426E+01_JPRB, 0.21684E+01_JPRB, 0.21634E+01_JPRB, 0.21777E+01_JPRB, 0.22058E+01_JPRB, 0.22624E+01_JPRB ,&
&  0.23872E+01_JPRB, 0.25467E+01_JPRB, 0.25839E+01_JPRB, 0.25509E+01_JPRB, 0.24092E+01_JPRB, 0.24616E+01_JPRB, 0.25429E+01_JPRB ,&
&  0.23565E+01_JPRB, 0.21195E+01_JPRB, 0.22225E+01_JPRB, 0.22982E+01_JPRB, 0.24391E+01_JPRB, 0.30749E+01_JPRB, 0.37601E+01_JPRB ,&
&  0.48847E+01_JPRB, 0.53854E+01_JPRB, 0.68261E+01_JPRB, 0.50170E+01_JPRB, 0.47638E+01_JPRB, 0.50288E+01_JPRB, 0.43776E+01_JPRB ,&
&  0.37637E+01_JPRB, 0.37029E+01_JPRB, 0.34598E+01_JPRB, 0.34140E+01_JPRB, 0.31693E+01_JPRB, 0.27405E+01_JPRB, 0.24119E+01_JPRB ,&
&  0.24307E+01_JPRB, 0.25327E+01_JPRB, 0.22032E+01_JPRB, 0.15325E+01_JPRB, 0.13275E+01_JPRB, 0.12896E+01_JPRB, 0.17452E+01_JPRB ,&
&  0.21048E+01_JPRB, 0.21307E+01_JPRB, 0.21349E+01_JPRB, 0.20451E+01_JPRB, 0.15136E+01_JPRB, 0.12297E+01_JPRB, 0.93852E+00_JPRB ,&
&  0.74759E+00_JPRB, 0.67825E+00_JPRB, 0.73495E+00_JPRB, 0.72779E+00_JPRB, 0.74929E+00_JPRB, 0.77796E+00_JPRB, 0.79375E+00_JPRB ,&
&  0.80886E+00_JPRB, 0.81245E+00_JPRB, 0.81595E+00_JPRB, 0.81290E+00_JPRB, 0.81632E+00_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.75311E+00_JPRB, 0.74550E+00_JPRB, 0.75038E+00_JPRB, 0.83311E+00_JPRB, 0.96115E+00_JPRB, 0.10642E+01_JPRB, 0.10843E+01_JPRB ,&
&  0.10938E+01_JPRB, 0.93640E+00_JPRB, 0.10033E+01_JPRB, 0.10584E+01_JPRB, 0.11796E+01_JPRB, 0.13153E+01_JPRB, 0.14670E+01_JPRB ,&
&  0.16112E+01_JPRB, 0.17515E+01_JPRB, 0.18756E+01_JPRB, 0.20143E+01_JPRB, 0.21897E+01_JPRB, 0.23639E+01_JPRB, 0.21051E+01_JPRB ,&
&  0.15478E+01_JPRB, 0.13224E+01_JPRB, 0.15837E+01_JPRB, 0.16223E+01_JPRB, 0.16452E+01_JPRB, 0.17590E+01_JPRB, 0.18153E+01_JPRB ,&
&  0.18658E+01_JPRB, 0.18106E+01_JPRB, 0.18748E+01_JPRB, 0.19476E+01_JPRB, 0.19511E+01_JPRB, 0.20757E+01_JPRB, 0.22271E+01_JPRB ,&
&  0.23268E+01_JPRB, 0.24068E+01_JPRB, 0.24278E+01_JPRB, 0.24165E+01_JPRB, 0.24280E+01_JPRB, 0.24845E+01_JPRB, 0.25769E+01_JPRB ,&
&  0.27332E+01_JPRB, 0.29185E+01_JPRB, 0.30113E+01_JPRB, 0.28526E+01_JPRB, 0.27826E+01_JPRB, 0.28365E+01_JPRB, 0.27972E+01_JPRB ,&
&  0.26490E+01_JPRB, 0.26360E+01_JPRB, 0.28079E+01_JPRB, 0.31279E+01_JPRB, 0.33159E+01_JPRB, 0.41323E+01_JPRB, 0.46053E+01_JPRB ,&
&  0.51632E+01_JPRB, 0.54515E+01_JPRB, 0.60346E+01_JPRB, 0.42257E+01_JPRB, 0.41650E+01_JPRB, 0.40797E+01_JPRB, 0.36585E+01_JPRB ,&
&  0.33730E+01_JPRB, 0.33566E+01_JPRB, 0.32123E+01_JPRB, 0.30466E+01_JPRB, 0.28886E+01_JPRB, 0.23833E+01_JPRB, 0.22408E+01_JPRB ,&
&  0.20345E+01_JPRB, 0.21662E+01_JPRB, 0.17380E+01_JPRB, 0.15243E+01_JPRB, 0.13438E+01_JPRB, 0.14337E+01_JPRB, 0.16338E+01_JPRB ,&
&  0.19141E+01_JPRB, 0.19190E+01_JPRB, 0.17341E+01_JPRB, 0.13868E+01_JPRB, 0.92742E+00_JPRB, 0.73304E+00_JPRB, 0.64078E+00_JPRB ,&
&  0.63799E+00_JPRB, 0.65873E+00_JPRB, 0.68736E+00_JPRB, 0.70562E+00_JPRB, 0.72425E+00_JPRB, 0.74498E+00_JPRB, 0.74809E+00_JPRB ,&
&  0.74497E+00_JPRB, 0.73422E+00_JPRB, 0.72754E+00_JPRB, 0.75765E+00_JPRB, 0.77067E+00_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.65827E+00_JPRB, 0.64510E+00_JPRB, 0.65583E+00_JPRB, 0.74500E+00_JPRB, 0.84046E+00_JPRB, 0.89958E+00_JPRB, 0.87915E+00_JPRB ,&
&  0.84139E+00_JPRB, 0.81010E+00_JPRB, 0.87021E+00_JPRB, 0.94152E+00_JPRB, 0.10414E+01_JPRB, 0.11789E+01_JPRB, 0.13236E+01_JPRB ,&
&  0.14831E+01_JPRB, 0.16086E+01_JPRB, 0.17030E+01_JPRB, 0.18212E+01_JPRB, 0.19743E+01_JPRB, 0.21299E+01_JPRB, 0.22002E+01_JPRB ,&
&  0.17563E+01_JPRB, 0.12105E+01_JPRB, 0.14202E+01_JPRB, 0.15864E+01_JPRB, 0.17174E+01_JPRB, 0.18222E+01_JPRB, 0.18543E+01_JPRB ,&
&  0.17407E+01_JPRB, 0.17491E+01_JPRB, 0.18389E+01_JPRB, 0.19185E+01_JPRB, 0.19248E+01_JPRB, 0.20670E+01_JPRB, 0.22165E+01_JPRB ,&
&  0.23754E+01_JPRB, 0.24921E+01_JPRB, 0.25594E+01_JPRB, 0.26247E+01_JPRB, 0.26929E+01_JPRB, 0.28005E+01_JPRB, 0.29747E+01_JPRB ,&
&  0.31955E+01_JPRB, 0.33971E+01_JPRB, 0.35468E+01_JPRB, 0.35360E+01_JPRB, 0.35081E+01_JPRB, 0.35945E+01_JPRB, 0.35649E+01_JPRB ,&
&  0.35581E+01_JPRB, 0.35492E+01_JPRB, 0.39335E+01_JPRB, 0.44779E+01_JPRB, 0.47982E+01_JPRB, 0.43896E+01_JPRB, 0.48039E+01_JPRB ,&
&  0.49060E+01_JPRB, 0.47749E+01_JPRB, 0.44132E+01_JPRB, 0.45098E+01_JPRB, 0.30979E+01_JPRB, 0.31579E+01_JPRB, 0.30070E+01_JPRB ,&
&  0.31863E+01_JPRB, 0.30593E+01_JPRB, 0.29020E+01_JPRB, 0.26738E+01_JPRB, 0.26393E+01_JPRB, 0.21531E+01_JPRB, 0.19311E+01_JPRB ,&
&  0.18512E+01_JPRB, 0.18593E+01_JPRB, 0.16695E+01_JPRB, 0.16089E+01_JPRB, 0.14794E+01_JPRB, 0.15677E+01_JPRB, 0.15367E+01_JPRB ,&
&  0.16578E+01_JPRB, 0.14350E+01_JPRB, 0.12114E+01_JPRB, 0.96217E+00_JPRB, 0.64872E+00_JPRB, 0.56225E+00_JPRB, 0.57670E+00_JPRB ,&
&  0.60118E+00_JPRB, 0.62665E+00_JPRB, 0.63153E+00_JPRB, 0.65566E+00_JPRB, 0.67149E+00_JPRB, 0.66332E+00_JPRB, 0.64670E+00_JPRB ,&
&  0.64448E+00_JPRB, 0.66217E+00_JPRB, 0.66090E+00_JPRB, 0.66438E+00_JPRB, 0.66650E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.56500E+00_JPRB, 0.57036E+00_JPRB, 0.59248E+00_JPRB, 0.63755E+00_JPRB, 0.70694E+00_JPRB, 0.74571E+00_JPRB, 0.75007E+00_JPRB ,&
&  0.73004E+00_JPRB, 0.73937E+00_JPRB, 0.77269E+00_JPRB, 0.78321E+00_JPRB, 0.83933E+00_JPRB, 0.94697E+00_JPRB, 0.10524E+01_JPRB ,&
&  0.11933E+01_JPRB, 0.13221E+01_JPRB, 0.14392E+01_JPRB, 0.16181E+01_JPRB, 0.17975E+01_JPRB, 0.20101E+01_JPRB, 0.21351E+01_JPRB ,&
&  0.21045E+01_JPRB, 0.18082E+01_JPRB, 0.17374E+01_JPRB, 0.14542E+01_JPRB, 0.13182E+01_JPRB, 0.15830E+01_JPRB, 0.15564E+01_JPRB ,&
&  0.15728E+01_JPRB, 0.16296E+01_JPRB, 0.16554E+01_JPRB, 0.17425E+01_JPRB, 0.18717E+01_JPRB, 0.20095E+01_JPRB, 0.21757E+01_JPRB ,&
&  0.23642E+01_JPRB, 0.25424E+01_JPRB, 0.27033E+01_JPRB, 0.28538E+01_JPRB, 0.29666E+01_JPRB, 0.31160E+01_JPRB, 0.33332E+01_JPRB ,&
&  0.35779E+01_JPRB, 0.37774E+01_JPRB, 0.40017E+01_JPRB, 0.42867E+01_JPRB, 0.42644E+01_JPRB, 0.44065E+01_JPRB, 0.44693E+01_JPRB ,&
&  0.45066E+01_JPRB, 0.44520E+01_JPRB, 0.44906E+01_JPRB, 0.46719E+01_JPRB, 0.46320E+01_JPRB, 0.40991E+01_JPRB, 0.40560E+01_JPRB ,&
&  0.41575E+01_JPRB, 0.38063E+01_JPRB, 0.32483E+01_JPRB, 0.36036E+01_JPRB, 0.23870E+01_JPRB, 0.26635E+01_JPRB, 0.28828E+01_JPRB ,&
&  0.27463E+01_JPRB, 0.26042E+01_JPRB, 0.23749E+01_JPRB, 0.22850E+01_JPRB, 0.21993E+01_JPRB, 0.19156E+01_JPRB, 0.18138E+01_JPRB ,&
&  0.17536E+01_JPRB, 0.14574E+01_JPRB, 0.13493E+01_JPRB, 0.14223E+01_JPRB, 0.15223E+01_JPRB, 0.14110E+01_JPRB, 0.11921E+01_JPRB ,&
&  0.10561E+01_JPRB, 0.11995E+01_JPRB, 0.10822E+01_JPRB, 0.11871E+01_JPRB, 0.95053E+00_JPRB, 0.54427E+00_JPRB, 0.54232E+00_JPRB ,&
&  0.55672E+00_JPRB, 0.57220E+00_JPRB, 0.57358E+00_JPRB, 0.59810E+00_JPRB, 0.60201E+00_JPRB, 0.59241E+00_JPRB, 0.59471E+00_JPRB ,&
&  0.60211E+00_JPRB, 0.59935E+00_JPRB, 0.59966E+00_JPRB, 0.59308E+00_JPRB, 0.58522E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.57959E+00_JPRB, 0.57496E+00_JPRB, 0.58132E+00_JPRB, 0.57906E+00_JPRB, 0.58972E+00_JPRB, 0.58964E+00_JPRB, 0.59499E+00_JPRB ,&
&  0.60934E+00_JPRB, 0.63049E+00_JPRB, 0.62346E+00_JPRB, 0.60093E+00_JPRB, 0.59685E+00_JPRB, 0.60867E+00_JPRB, 0.65022E+00_JPRB ,&
&  0.74470E+00_JPRB, 0.84293E+00_JPRB, 0.92625E+00_JPRB, 0.10430E+01_JPRB, 0.12604E+01_JPRB, 0.16340E+01_JPRB, 0.19960E+01_JPRB ,&
&  0.22638E+01_JPRB, 0.24508E+01_JPRB, 0.24511E+01_JPRB, 0.25644E+01_JPRB, 0.20469E+01_JPRB, 0.15147E+01_JPRB, 0.16941E+01_JPRB ,&
&  0.18425E+01_JPRB, 0.18345E+01_JPRB, 0.16598E+01_JPRB, 0.16498E+01_JPRB, 0.16874E+01_JPRB, 0.18067E+01_JPRB, 0.20398E+01_JPRB ,&
&  0.22746E+01_JPRB, 0.25221E+01_JPRB, 0.27312E+01_JPRB, 0.29167E+01_JPRB, 0.30760E+01_JPRB, 0.32601E+01_JPRB, 0.34365E+01_JPRB ,&
&  0.36287E+01_JPRB, 0.37763E+01_JPRB, 0.39300E+01_JPRB, 0.42137E+01_JPRB, 0.44891E+01_JPRB, 0.47405E+01_JPRB, 0.48706E+01_JPRB ,&
&  0.48838E+01_JPRB, 0.45919E+01_JPRB, 0.45956E+01_JPRB, 0.46152E+01_JPRB, 0.43043E+01_JPRB, 0.38777E+01_JPRB, 0.36832E+01_JPRB ,&
&  0.34633E+01_JPRB, 0.31231E+01_JPRB, 0.23450E+01_JPRB, 0.27682E+01_JPRB, 0.30673E+01_JPRB, 0.26713E+01_JPRB, 0.24914E+01_JPRB ,&
&  0.22840E+01_JPRB, 0.21463E+01_JPRB, 0.19711E+01_JPRB, 0.19469E+01_JPRB, 0.20266E+01_JPRB, 0.23820E+01_JPRB, 0.18189E+01_JPRB ,&
&  0.16041E+01_JPRB, 0.11320E+01_JPRB, 0.10005E+01_JPRB, 0.10845E+01_JPRB, 0.11740E+01_JPRB, 0.11002E+01_JPRB, 0.87376E+00_JPRB ,&
&  0.85174E+00_JPRB, 0.10559E+01_JPRB, 0.11211E+01_JPRB, 0.15620E+01_JPRB, 0.18705E+01_JPRB, 0.56845E+00_JPRB, 0.51406E+00_JPRB ,&
&  0.52068E+00_JPRB, 0.52456E+00_JPRB, 0.52715E+00_JPRB, 0.53803E+00_JPRB, 0.53324E+00_JPRB, 0.53994E+00_JPRB, 0.55638E+00_JPRB ,&
&  0.56752E+00_JPRB, 0.57319E+00_JPRB, 0.57839E+00_JPRB, 0.57046E+00_JPRB, 0.57824E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.65953E+00_JPRB, 0.65965E+00_JPRB, 0.66303E+00_JPRB, 0.65779E+00_JPRB, 0.65250E+00_JPRB, 0.65018E+00_JPRB, 0.65158E+00_JPRB ,&
&  0.65160E+00_JPRB, 0.64166E+00_JPRB, 0.63123E+00_JPRB, 0.62266E+00_JPRB, 0.61035E+00_JPRB, 0.58286E+00_JPRB, 0.54379E+00_JPRB ,&
&  0.55118E+00_JPRB, 0.56376E+00_JPRB, 0.58072E+00_JPRB, 0.62159E+00_JPRB, 0.70855E+00_JPRB, 0.84098E+00_JPRB, 0.10203E+01_JPRB ,&
&  0.13678E+01_JPRB, 0.17518E+01_JPRB, 0.20260E+01_JPRB, 0.21195E+01_JPRB, 0.23536E+01_JPRB, 0.17524E+01_JPRB, 0.16462E+01_JPRB ,&
&  0.18169E+01_JPRB, 0.15992E+01_JPRB, 0.15325E+01_JPRB, 0.15015E+01_JPRB, 0.15376E+01_JPRB, 0.16308E+01_JPRB, 0.17952E+01_JPRB ,&
&  0.20392E+01_JPRB, 0.22909E+01_JPRB, 0.25151E+01_JPRB, 0.27426E+01_JPRB, 0.29587E+01_JPRB, 0.31828E+01_JPRB, 0.33682E+01_JPRB ,&
&  0.34874E+01_JPRB, 0.36269E+01_JPRB, 0.38318E+01_JPRB, 0.39309E+01_JPRB, 0.42953E+01_JPRB, 0.45332E+01_JPRB, 0.46393E+01_JPRB ,&
&  0.45166E+01_JPRB, 0.42756E+01_JPRB, 0.42523E+01_JPRB, 0.39932E+01_JPRB, 0.37123E+01_JPRB, 0.34548E+01_JPRB, 0.31663E+01_JPRB ,&
&  0.28851E+01_JPRB, 0.24948E+01_JPRB, 0.19559E+01_JPRB, 0.21340E+01_JPRB, 0.21898E+01_JPRB, 0.19334E+01_JPRB, 0.21928E+01_JPRB ,&
&  0.20153E+01_JPRB, 0.19443E+01_JPRB, 0.18338E+01_JPRB, 0.17255E+01_JPRB, 0.16422E+01_JPRB, 0.18313E+01_JPRB, 0.16814E+01_JPRB ,&
&  0.12637E+01_JPRB, 0.95380E+00_JPRB, 0.79284E+00_JPRB, 0.78253E+00_JPRB, 0.84531E+00_JPRB, 0.90220E+00_JPRB, 0.87100E+00_JPRB ,&
&  0.92111E+00_JPRB, 0.86821E+00_JPRB, 0.91859E+00_JPRB, 0.11287E+01_JPRB, 0.80607E+00_JPRB, 0.54128E+00_JPRB, 0.49789E+00_JPRB ,&
&  0.49029E+00_JPRB, 0.49318E+00_JPRB, 0.50077E+00_JPRB, 0.52346E+00_JPRB, 0.54502E+00_JPRB, 0.56443E+00_JPRB, 0.58001E+00_JPRB ,&
&  0.58914E+00_JPRB, 0.60965E+00_JPRB, 0.62916E+00_JPRB, 0.64635E+00_JPRB, 0.65914E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.75603E+00_JPRB, 0.76967E+00_JPRB, 0.76369E+00_JPRB, 0.77139E+00_JPRB, 0.77503E+00_JPRB, 0.78814E+00_JPRB, 0.78587E+00_JPRB ,&
&  0.78047E+00_JPRB, 0.78557E+00_JPRB, 0.76534E+00_JPRB, 0.77007E+00_JPRB, 0.76671E+00_JPRB, 0.77006E+00_JPRB, 0.75976E+00_JPRB ,&
&  0.76355E+00_JPRB, 0.78061E+00_JPRB, 0.78932E+00_JPRB, 0.79593E+00_JPRB, 0.78966E+00_JPRB, 0.76958E+00_JPRB, 0.74932E+00_JPRB ,&
&  0.79216E+00_JPRB, 0.90090E+00_JPRB, 0.10290E+01_JPRB, 0.11241E+01_JPRB, 0.13424E+01_JPRB, 0.13501E+01_JPRB, 0.14711E+01_JPRB ,&
&  0.15550E+01_JPRB, 0.14170E+01_JPRB, 0.15270E+01_JPRB, 0.13614E+01_JPRB, 0.13616E+01_JPRB, 0.13444E+01_JPRB, 0.13557E+01_JPRB ,&
&  0.14194E+01_JPRB, 0.15341E+01_JPRB, 0.16655E+01_JPRB, 0.18127E+01_JPRB, 0.19385E+01_JPRB, 0.19935E+01_JPRB, 0.20304E+01_JPRB ,&
&  0.20548E+01_JPRB, 0.22481E+01_JPRB, 0.27658E+01_JPRB, 0.31745E+01_JPRB, 0.36375E+01_JPRB, 0.38617E+01_JPRB, 0.38841E+01_JPRB ,&
&  0.38232E+01_JPRB, 0.38041E+01_JPRB, 0.32418E+01_JPRB, 0.28056E+01_JPRB, 0.26826E+01_JPRB, 0.23725E+01_JPRB, 0.22548E+01_JPRB ,&
&  0.23032E+01_JPRB, 0.21140E+01_JPRB, 0.16172E+01_JPRB, 0.17022E+01_JPRB, 0.17789E+01_JPRB, 0.17118E+01_JPRB, 0.18853E+01_JPRB ,&
&  0.16491E+01_JPRB, 0.15053E+01_JPRB, 0.14424E+01_JPRB, 0.14093E+01_JPRB, 0.13621E+01_JPRB, 0.12736E+01_JPRB, 0.12362E+01_JPRB ,&
&  0.10248E+01_JPRB, 0.89565E+00_JPRB, 0.83215E+00_JPRB, 0.81334E+00_JPRB, 0.90032E+00_JPRB, 0.10893E+01_JPRB, 0.91311E+00_JPRB ,&
&  0.91552E+00_JPRB, 0.84145E+00_JPRB, 0.10426E+01_JPRB, 0.14272E+01_JPRB, 0.73353E+00_JPRB, 0.57543E+00_JPRB, 0.50764E+00_JPRB ,&
&  0.52387E+00_JPRB, 0.55586E+00_JPRB, 0.58167E+00_JPRB, 0.61475E+00_JPRB, 0.63198E+00_JPRB, 0.63481E+00_JPRB, 0.64541E+00_JPRB ,&
&  0.67313E+00_JPRB, 0.70061E+00_JPRB, 0.73862E+00_JPRB, 0.74324E+00_JPRB, 0.74011E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.81532E+00_JPRB, 0.83193E+00_JPRB, 0.83328E+00_JPRB, 0.84946E+00_JPRB, 0.86964E+00_JPRB, 0.87494E+00_JPRB, 0.87638E+00_JPRB ,&
&  0.86619E+00_JPRB, 0.86824E+00_JPRB, 0.86454E+00_JPRB, 0.88240E+00_JPRB, 0.88510E+00_JPRB, 0.88759E+00_JPRB, 0.89037E+00_JPRB ,&
&  0.90571E+00_JPRB, 0.91846E+00_JPRB, 0.95085E+00_JPRB, 0.95586E+00_JPRB, 0.95217E+00_JPRB, 0.95909E+00_JPRB, 0.98052E+00_JPRB ,&
&  0.10057E+01_JPRB, 0.10347E+01_JPRB, 0.10730E+01_JPRB, 0.11068E+01_JPRB, 0.12206E+01_JPRB, 0.13820E+01_JPRB, 0.16656E+01_JPRB ,&
&  0.14976E+01_JPRB, 0.14738E+01_JPRB, 0.14003E+01_JPRB, 0.12127E+01_JPRB, 0.12031E+01_JPRB, 0.12302E+01_JPRB, 0.11822E+01_JPRB ,&
&  0.11321E+01_JPRB, 0.11155E+01_JPRB, 0.10985E+01_JPRB, 0.10989E+01_JPRB, 0.11134E+01_JPRB, 0.10958E+01_JPRB, 0.10863E+01_JPRB ,&
&  0.10452E+01_JPRB, 0.10726E+01_JPRB, 0.13639E+01_JPRB, 0.17549E+01_JPRB, 0.21232E+01_JPRB, 0.24588E+01_JPRB, 0.22242E+01_JPRB ,&
&  0.21315E+01_JPRB, 0.22569E+01_JPRB, 0.20392E+01_JPRB, 0.18642E+01_JPRB, 0.16792E+01_JPRB, 0.14976E+01_JPRB, 0.14014E+01_JPRB ,&
&  0.15261E+01_JPRB, 0.16139E+01_JPRB, 0.14472E+01_JPRB, 0.16366E+01_JPRB, 0.16430E+01_JPRB, 0.16467E+01_JPRB, 0.14523E+01_JPRB ,&
&  0.12631E+01_JPRB, 0.11629E+01_JPRB, 0.11439E+01_JPRB, 0.11278E+01_JPRB, 0.11330E+01_JPRB, 0.10738E+01_JPRB, 0.97873E+00_JPRB ,&
&  0.92847E+00_JPRB, 0.85542E+00_JPRB, 0.83156E+00_JPRB, 0.78077E+00_JPRB, 0.93874E+00_JPRB, 0.95229E+00_JPRB, 0.12281E+01_JPRB ,&
&  0.91419E+00_JPRB, 0.82699E+00_JPRB, 0.72544E+00_JPRB, 0.10584E+01_JPRB, 0.69567E+00_JPRB, 0.53430E+00_JPRB, 0.55327E+00_JPRB ,&
&  0.62603E+00_JPRB, 0.66652E+00_JPRB, 0.68384E+00_JPRB, 0.71204E+00_JPRB, 0.72066E+00_JPRB, 0.71478E+00_JPRB, 0.72295E+00_JPRB ,&
&  0.76019E+00_JPRB, 0.76844E+00_JPRB, 0.77622E+00_JPRB, 0.77840E+00_JPRB, 0.79612E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.84186E+00_JPRB, 0.86732E+00_JPRB, 0.88262E+00_JPRB, 0.89775E+00_JPRB, 0.91749E+00_JPRB, 0.92733E+00_JPRB, 0.93484E+00_JPRB ,&
&  0.93723E+00_JPRB, 0.93418E+00_JPRB, 0.94187E+00_JPRB, 0.95624E+00_JPRB, 0.96441E+00_JPRB, 0.96259E+00_JPRB, 0.96928E+00_JPRB ,&
&  0.98503E+00_JPRB, 0.10103E+01_JPRB, 0.10371E+01_JPRB, 0.10313E+01_JPRB, 0.10277E+01_JPRB, 0.10432E+01_JPRB, 0.10647E+01_JPRB ,&
&  0.10667E+01_JPRB, 0.10630E+01_JPRB, 0.10716E+01_JPRB, 0.10832E+01_JPRB, 0.11051E+01_JPRB, 0.11110E+01_JPRB, 0.12115E+01_JPRB ,&
&  0.11387E+01_JPRB, 0.15916E+01_JPRB, 0.15323E+01_JPRB, 0.13332E+01_JPRB, 0.13197E+01_JPRB, 0.12335E+01_JPRB, 0.12269E+01_JPRB ,&
&  0.12326E+01_JPRB, 0.12042E+01_JPRB, 0.11815E+01_JPRB, 0.11593E+01_JPRB, 0.11687E+01_JPRB, 0.11883E+01_JPRB, 0.11624E+01_JPRB ,&
&  0.11784E+01_JPRB, 0.12102E+01_JPRB, 0.13067E+01_JPRB, 0.14360E+01_JPRB, 0.15581E+01_JPRB, 0.15954E+01_JPRB, 0.14814E+01_JPRB ,&
&  0.14013E+01_JPRB, 0.12741E+01_JPRB, 0.10436E+01_JPRB, 0.10423E+01_JPRB, 0.10147E+01_JPRB, 0.83837E+00_JPRB, 0.76433E+00_JPRB ,&
&  0.10271E+01_JPRB, 0.11703E+01_JPRB, 0.14709E+01_JPRB, 0.14883E+01_JPRB, 0.14228E+01_JPRB, 0.14170E+01_JPRB, 0.12340E+01_JPRB ,&
&  0.10970E+01_JPRB, 0.10008E+01_JPRB, 0.98664E+00_JPRB, 0.94912E+00_JPRB, 0.95405E+00_JPRB, 0.93563E+00_JPRB, 0.86347E+00_JPRB ,&
&  0.80423E+00_JPRB, 0.77448E+00_JPRB, 0.76715E+00_JPRB, 0.79052E+00_JPRB, 0.77612E+00_JPRB, 0.12300E+01_JPRB, 0.19296E+01_JPRB ,&
&  0.10160E+01_JPRB, 0.74715E+00_JPRB, 0.53321E+00_JPRB, 0.67545E+00_JPRB, 0.60834E+00_JPRB, 0.57100E+00_JPRB, 0.62749E+00_JPRB ,&
&  0.74209E+00_JPRB, 0.79001E+00_JPRB, 0.79850E+00_JPRB, 0.79053E+00_JPRB, 0.76779E+00_JPRB, 0.74682E+00_JPRB, 0.73813E+00_JPRB ,&
&  0.75443E+00_JPRB, 0.77069E+00_JPRB, 0.78917E+00_JPRB, 0.79994E+00_JPRB, 0.81618E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.85728E+00_JPRB, 0.88171E+00_JPRB, 0.90066E+00_JPRB, 0.92222E+00_JPRB, 0.94487E+00_JPRB, 0.96556E+00_JPRB, 0.98466E+00_JPRB ,&
&  0.10003E+01_JPRB, 0.10133E+01_JPRB, 0.10254E+01_JPRB, 0.10412E+01_JPRB, 0.10616E+01_JPRB, 0.10778E+01_JPRB, 0.10986E+01_JPRB ,&
&  0.11304E+01_JPRB, 0.11564E+01_JPRB, 0.11849E+01_JPRB, 0.11832E+01_JPRB, 0.11844E+01_JPRB, 0.12079E+01_JPRB, 0.12303E+01_JPRB ,&
&  0.12385E+01_JPRB, 0.12543E+01_JPRB, 0.12802E+01_JPRB, 0.13106E+01_JPRB, 0.13502E+01_JPRB, 0.14264E+01_JPRB, 0.13697E+01_JPRB ,&
&  0.14920E+01_JPRB, 0.16666E+01_JPRB, 0.16203E+01_JPRB, 0.14682E+01_JPRB, 0.14031E+01_JPRB, 0.13918E+01_JPRB, 0.14218E+01_JPRB ,&
&  0.14765E+01_JPRB, 0.14746E+01_JPRB, 0.14808E+01_JPRB, 0.14753E+01_JPRB, 0.14862E+01_JPRB, 0.15214E+01_JPRB, 0.15554E+01_JPRB ,&
&  0.16016E+01_JPRB, 0.16610E+01_JPRB, 0.17456E+01_JPRB, 0.18560E+01_JPRB, 0.19781E+01_JPRB, 0.19747E+01_JPRB, 0.19166E+01_JPRB ,&
&  0.17802E+01_JPRB, 0.16374E+01_JPRB, 0.12726E+01_JPRB, 0.11805E+01_JPRB, 0.10973E+01_JPRB, 0.94618E+00_JPRB, 0.82355E+00_JPRB ,&
&  0.10407E+01_JPRB, 0.10981E+01_JPRB, 0.12322E+01_JPRB, 0.14102E+01_JPRB, 0.13104E+01_JPRB, 0.12079E+01_JPRB, 0.10762E+01_JPRB ,&
&  0.97238E+00_JPRB, 0.89942E+00_JPRB, 0.87046E+00_JPRB, 0.83355E+00_JPRB, 0.82621E+00_JPRB, 0.82952E+00_JPRB, 0.76672E+00_JPRB ,&
&  0.74074E+00_JPRB, 0.73719E+00_JPRB, 0.74409E+00_JPRB, 0.76317E+00_JPRB, 0.83910E+00_JPRB, 0.86374E+00_JPRB, 0.12410E+01_JPRB ,&
&  0.93775E+00_JPRB, 0.61814E+00_JPRB, 0.64229E+00_JPRB, 0.66920E+00_JPRB, 0.64298E+00_JPRB, 0.63152E+00_JPRB, 0.70641E+00_JPRB ,&
&  0.85768E+00_JPRB, 0.97674E+00_JPRB, 0.10046E+01_JPRB, 0.95026E+00_JPRB, 0.85876E+00_JPRB, 0.80554E+00_JPRB, 0.77796E+00_JPRB ,&
&  0.78870E+00_JPRB, 0.79908E+00_JPRB, 0.81242E+00_JPRB, 0.81756E+00_JPRB, 0.83137E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.85110E+00_JPRB, 0.87859E+00_JPRB, 0.90132E+00_JPRB, 0.92347E+00_JPRB, 0.94404E+00_JPRB, 0.96359E+00_JPRB, 0.98537E+00_JPRB ,&
&  0.10003E+01_JPRB, 0.10214E+01_JPRB, 0.10467E+01_JPRB, 0.10724E+01_JPRB, 0.10946E+01_JPRB, 0.11241E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.11847E+01_JPRB, 0.12090E+01_JPRB, 0.12332E+01_JPRB, 0.12367E+01_JPRB, 0.12382E+01_JPRB, 0.12530E+01_JPRB, 0.12637E+01_JPRB ,&
&  0.12604E+01_JPRB, 0.12652E+01_JPRB, 0.12785E+01_JPRB, 0.13034E+01_JPRB, 0.13150E+01_JPRB, 0.13747E+01_JPRB, 0.12258E+01_JPRB ,&
&  0.16049E+01_JPRB, 0.18112E+01_JPRB, 0.17491E+01_JPRB, 0.16128E+01_JPRB, 0.15401E+01_JPRB, 0.15781E+01_JPRB, 0.16314E+01_JPRB ,&
&  0.17031E+01_JPRB, 0.18052E+01_JPRB, 0.17771E+01_JPRB, 0.17717E+01_JPRB, 0.17992E+01_JPRB, 0.18617E+01_JPRB, 0.19563E+01_JPRB ,&
&  0.20547E+01_JPRB, 0.21633E+01_JPRB, 0.22810E+01_JPRB, 0.23963E+01_JPRB, 0.25020E+01_JPRB, 0.25711E+01_JPRB, 0.25776E+01_JPRB ,&
&  0.25027E+01_JPRB, 0.23924E+01_JPRB, 0.21404E+01_JPRB, 0.20943E+01_JPRB, 0.19699E+01_JPRB, 0.16157E+01_JPRB, 0.12755E+01_JPRB ,&
&  0.12050E+01_JPRB, 0.12157E+01_JPRB, 0.11950E+01_JPRB, 0.13050E+01_JPRB, 0.12295E+01_JPRB, 0.11237E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.91364E+00_JPRB, 0.83594E+00_JPRB, 0.81445E+00_JPRB, 0.76709E+00_JPRB, 0.73184E+00_JPRB, 0.70742E+00_JPRB, 0.68771E+00_JPRB ,&
&  0.69740E+00_JPRB, 0.71463E+00_JPRB, 0.74033E+00_JPRB, 0.76316E+00_JPRB, 0.77809E+00_JPRB, 0.73925E+00_JPRB, 0.10454E+01_JPRB ,&
&  0.87107E+00_JPRB, 0.73338E+00_JPRB, 0.86083E+00_JPRB, 0.57677E+00_JPRB, 0.67157E+00_JPRB, 0.73570E+00_JPRB, 0.86273E+00_JPRB ,&
&  0.94927E+00_JPRB, 0.13246E+01_JPRB, 0.18105E+01_JPRB, 0.14468E+01_JPRB, 0.11672E+01_JPRB, 0.88974E+00_JPRB, 0.79888E+00_JPRB ,&
&  0.79757E+00_JPRB, 0.80099E+00_JPRB, 0.81018E+00_JPRB, 0.81375E+00_JPRB, 0.82636E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.83849E+00_JPRB, 0.85967E+00_JPRB, 0.88406E+00_JPRB, 0.90615E+00_JPRB, 0.92070E+00_JPRB, 0.93361E+00_JPRB, 0.94994E+00_JPRB ,&
&  0.95876E+00_JPRB, 0.96126E+00_JPRB, 0.97387E+00_JPRB, 0.99442E+00_JPRB, 0.10198E+01_JPRB, 0.10479E+01_JPRB, 0.10815E+01_JPRB ,&
&  0.11265E+01_JPRB, 0.11677E+01_JPRB, 0.12204E+01_JPRB, 0.12584E+01_JPRB, 0.12832E+01_JPRB, 0.13085E+01_JPRB, 0.13316E+01_JPRB ,&
&  0.13699E+01_JPRB, 0.13944E+01_JPRB, 0.14190E+01_JPRB, 0.14716E+01_JPRB, 0.15118E+01_JPRB, 0.15598E+01_JPRB, 0.13159E+01_JPRB ,&
&  0.17293E+01_JPRB, 0.20333E+01_JPRB, 0.19183E+01_JPRB, 0.17402E+01_JPRB, 0.16484E+01_JPRB, 0.16658E+01_JPRB, 0.16889E+01_JPRB ,&
&  0.17386E+01_JPRB, 0.17757E+01_JPRB, 0.17462E+01_JPRB, 0.19050E+01_JPRB, 0.20279E+01_JPRB, 0.20941E+01_JPRB, 0.22438E+01_JPRB ,&
&  0.23967E+01_JPRB, 0.25451E+01_JPRB, 0.26863E+01_JPRB, 0.28187E+01_JPRB, 0.29319E+01_JPRB, 0.30545E+01_JPRB, 0.30945E+01_JPRB ,&
&  0.30997E+01_JPRB, 0.31002E+01_JPRB, 0.30678E+01_JPRB, 0.27817E+01_JPRB, 0.26504E+01_JPRB, 0.24783E+01_JPRB, 0.19839E+01_JPRB ,&
&  0.15671E+01_JPRB, 0.12430E+01_JPRB, 0.11367E+01_JPRB, 0.11620E+01_JPRB, 0.10858E+01_JPRB, 0.97446E+00_JPRB, 0.88522E+00_JPRB ,&
&  0.80630E+00_JPRB, 0.75627E+00_JPRB, 0.75686E+00_JPRB, 0.71765E+00_JPRB, 0.63400E+00_JPRB, 0.64730E+00_JPRB, 0.67066E+00_JPRB ,&
&  0.66199E+00_JPRB, 0.68034E+00_JPRB, 0.70749E+00_JPRB, 0.71309E+00_JPRB, 0.74725E+00_JPRB, 0.83058E+00_JPRB, 0.86758E+00_JPRB ,&
&  0.92419E+00_JPRB, 0.76702E+00_JPRB, 0.75536E+00_JPRB, 0.79822E+00_JPRB, 0.76242E+00_JPRB, 0.98212E+00_JPRB, 0.13307E+01_JPRB ,&
&  0.19218E+01_JPRB, 0.20741E+01_JPRB, 0.26862E+01_JPRB, 0.41779E+01_JPRB, 0.31767E+01_JPRB, 0.21183E+01_JPRB, 0.82791E+00_JPRB ,&
&  0.82969E+00_JPRB, 0.82318E+00_JPRB, 0.83845E+00_JPRB, 0.83556E+00_JPRB, 0.83905E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.72945E+00_JPRB, 0.72831E+00_JPRB, 0.75506E+00_JPRB, 0.78332E+00_JPRB, 0.80907E+00_JPRB, 0.83347E+00_JPRB, 0.85189E+00_JPRB ,&
&  0.87864E+00_JPRB, 0.89998E+00_JPRB, 0.90556E+00_JPRB, 0.90299E+00_JPRB, 0.91591E+00_JPRB, 0.92016E+00_JPRB, 0.94451E+00_JPRB ,&
&  0.98210E+00_JPRB, 0.10250E+01_JPRB, 0.10829E+01_JPRB, 0.11207E+01_JPRB, 0.11575E+01_JPRB, 0.12148E+01_JPRB, 0.12686E+01_JPRB ,&
&  0.13281E+01_JPRB, 0.14025E+01_JPRB, 0.14690E+01_JPRB, 0.15417E+01_JPRB, 0.16108E+01_JPRB, 0.16026E+01_JPRB, 0.12472E+01_JPRB ,&
&  0.18016E+01_JPRB, 0.20915E+01_JPRB, 0.20364E+01_JPRB, 0.18367E+01_JPRB, 0.17089E+01_JPRB, 0.16670E+01_JPRB, 0.16768E+01_JPRB ,&
&  0.16899E+01_JPRB, 0.16707E+01_JPRB, 0.16588E+01_JPRB, 0.17119E+01_JPRB, 0.20447E+01_JPRB, 0.20972E+01_JPRB, 0.22620E+01_JPRB ,&
&  0.24292E+01_JPRB, 0.25843E+01_JPRB, 0.27179E+01_JPRB, 0.28328E+01_JPRB, 0.29198E+01_JPRB, 0.30268E+01_JPRB, 0.31419E+01_JPRB ,&
&  0.32460E+01_JPRB, 0.33072E+01_JPRB, 0.34134E+01_JPRB, 0.31075E+01_JPRB, 0.29627E+01_JPRB, 0.31831E+01_JPRB, 0.28240E+01_JPRB ,&
&  0.18211E+01_JPRB, 0.12593E+01_JPRB, 0.12379E+01_JPRB, 0.11568E+01_JPRB, 0.10213E+01_JPRB, 0.89658E+00_JPRB, 0.81690E+00_JPRB ,&
&  0.76265E+00_JPRB, 0.72091E+00_JPRB, 0.69895E+00_JPRB, 0.66084E+00_JPRB, 0.65017E+00_JPRB, 0.66448E+00_JPRB, 0.66603E+00_JPRB ,&
&  0.68793E+00_JPRB, 0.66091E+00_JPRB, 0.65254E+00_JPRB, 0.66249E+00_JPRB, 0.71612E+00_JPRB, 0.82976E+00_JPRB, 0.10567E+01_JPRB ,&
&  0.11934E+01_JPRB, 0.73834E+00_JPRB, 0.75779E+00_JPRB, 0.78763E+00_JPRB, 0.84998E+00_JPRB, 0.97472E+00_JPRB, 0.11732E+01_JPRB ,&
&  0.15154E+01_JPRB, 0.19905E+01_JPRB, 0.19207E+01_JPRB, 0.15255E+01_JPRB, 0.19186E+01_JPRB, 0.16147E+01_JPRB, 0.73344E+00_JPRB ,&
&  0.73779E+00_JPRB, 0.71708E+00_JPRB, 0.72347E+00_JPRB, 0.73092E+00_JPRB, 0.73374E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.59856E+00_JPRB, 0.58674E+00_JPRB, 0.59537E+00_JPRB, 0.61484E+00_JPRB, 0.63195E+00_JPRB, 0.66494E+00_JPRB, 0.70556E+00_JPRB ,&
&  0.75649E+00_JPRB, 0.79011E+00_JPRB, 0.80231E+00_JPRB, 0.80572E+00_JPRB, 0.81521E+00_JPRB, 0.82072E+00_JPRB, 0.83612E+00_JPRB ,&
&  0.86406E+00_JPRB, 0.89246E+00_JPRB, 0.93019E+00_JPRB, 0.96268E+00_JPRB, 0.99006E+00_JPRB, 0.10500E+01_JPRB, 0.11177E+01_JPRB ,&
&  0.12010E+01_JPRB, 0.12898E+01_JPRB, 0.13965E+01_JPRB, 0.15114E+01_JPRB, 0.16293E+01_JPRB, 0.16761E+01_JPRB, 0.16854E+01_JPRB ,&
&  0.13210E+01_JPRB, 0.19232E+01_JPRB, 0.19967E+01_JPRB, 0.18641E+01_JPRB, 0.17293E+01_JPRB, 0.16925E+01_JPRB, 0.16536E+01_JPRB ,&
&  0.16146E+01_JPRB, 0.15091E+01_JPRB, 0.15389E+01_JPRB, 0.16421E+01_JPRB, 0.18538E+01_JPRB, 0.19623E+01_JPRB, 0.21113E+01_JPRB ,&
&  0.22543E+01_JPRB, 0.23734E+01_JPRB, 0.24785E+01_JPRB, 0.25827E+01_JPRB, 0.27152E+01_JPRB, 0.28935E+01_JPRB, 0.31234E+01_JPRB ,&
&  0.33247E+01_JPRB, 0.34802E+01_JPRB, 0.35880E+01_JPRB, 0.33664E+01_JPRB, 0.31335E+01_JPRB, 0.34793E+01_JPRB, 0.33139E+01_JPRB ,&
&  0.20581E+01_JPRB, 0.13342E+01_JPRB, 0.13546E+01_JPRB, 0.13375E+01_JPRB, 0.11238E+01_JPRB, 0.98888E+00_JPRB, 0.88141E+00_JPRB ,&
&  0.76067E+00_JPRB, 0.68389E+00_JPRB, 0.68792E+00_JPRB, 0.68260E+00_JPRB, 0.68844E+00_JPRB, 0.68956E+00_JPRB, 0.70172E+00_JPRB ,&
&  0.72184E+00_JPRB, 0.69243E+00_JPRB, 0.67293E+00_JPRB, 0.66360E+00_JPRB, 0.67988E+00_JPRB, 0.70677E+00_JPRB, 0.73672E+00_JPRB ,&
&  0.79671E+00_JPRB, 0.74942E+00_JPRB, 0.75485E+00_JPRB, 0.79478E+00_JPRB, 0.83398E+00_JPRB, 0.88329E+00_JPRB, 0.93655E+00_JPRB ,&
&  0.97978E+00_JPRB, 0.10237E+01_JPRB, 0.10603E+01_JPRB, 0.10595E+01_JPRB, 0.97696E+00_JPRB, 0.89503E+00_JPRB, 0.65530E+00_JPRB ,&
&  0.64018E+00_JPRB, 0.62403E+00_JPRB, 0.62434E+00_JPRB, 0.62146E+00_JPRB, 0.61911E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.52597E+00_JPRB, 0.51878E+00_JPRB, 0.52497E+00_JPRB, 0.53496E+00_JPRB, 0.55344E+00_JPRB, 0.58424E+00_JPRB, 0.62921E+00_JPRB ,&
&  0.67953E+00_JPRB, 0.70783E+00_JPRB, 0.73095E+00_JPRB, 0.72231E+00_JPRB, 0.72476E+00_JPRB, 0.74234E+00_JPRB, 0.75374E+00_JPRB ,&
&  0.78375E+00_JPRB, 0.81523E+00_JPRB, 0.83334E+00_JPRB, 0.83820E+00_JPRB, 0.84994E+00_JPRB, 0.88285E+00_JPRB, 0.92922E+00_JPRB ,&
&  0.98666E+00_JPRB, 0.10655E+01_JPRB, 0.11741E+01_JPRB, 0.13216E+01_JPRB, 0.14838E+01_JPRB, 0.15424E+01_JPRB, 0.15329E+01_JPRB ,&
&  0.78664E+00_JPRB, 0.13876E+01_JPRB, 0.18805E+01_JPRB, 0.18742E+01_JPRB, 0.18049E+01_JPRB, 0.17379E+01_JPRB, 0.16932E+01_JPRB ,&
&  0.15266E+01_JPRB, 0.13448E+01_JPRB, 0.13182E+01_JPRB, 0.15658E+01_JPRB, 0.16130E+01_JPRB, 0.17570E+01_JPRB, 0.18844E+01_JPRB ,&
&  0.20218E+01_JPRB, 0.21386E+01_JPRB, 0.22152E+01_JPRB, 0.22839E+01_JPRB, 0.23984E+01_JPRB, 0.25699E+01_JPRB, 0.27843E+01_JPRB ,&
&  0.30583E+01_JPRB, 0.33933E+01_JPRB, 0.36339E+01_JPRB, 0.34885E+01_JPRB, 0.34065E+01_JPRB, 0.36484E+01_JPRB, 0.45263E+01_JPRB ,&
&  0.27881E+01_JPRB, 0.15930E+01_JPRB, 0.15736E+01_JPRB, 0.15813E+01_JPRB, 0.12926E+01_JPRB, 0.11421E+01_JPRB, 0.10212E+01_JPRB ,&
&  0.85494E+00_JPRB, 0.77899E+00_JPRB, 0.74509E+00_JPRB, 0.72282E+00_JPRB, 0.73506E+00_JPRB, 0.75999E+00_JPRB, 0.80449E+00_JPRB ,&
&  0.82234E+00_JPRB, 0.80646E+00_JPRB, 0.78113E+00_JPRB, 0.76983E+00_JPRB, 0.76224E+00_JPRB, 0.76607E+00_JPRB, 0.76041E+00_JPRB ,&
&  0.77602E+00_JPRB, 0.76998E+00_JPRB, 0.77916E+00_JPRB, 0.81336E+00_JPRB, 0.85967E+00_JPRB, 0.87431E+00_JPRB, 0.87593E+00_JPRB ,&
&  0.84336E+00_JPRB, 0.85639E+00_JPRB, 0.86341E+00_JPRB, 0.84210E+00_JPRB, 0.81168E+00_JPRB, 0.75769E+00_JPRB, 0.66264E+00_JPRB ,&
&  0.61645E+00_JPRB, 0.56680E+00_JPRB, 0.53996E+00_JPRB, 0.52823E+00_JPRB, 0.53753E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.50601E+00_JPRB, 0.49953E+00_JPRB, 0.49928E+00_JPRB, 0.51921E+00_JPRB, 0.51681E+00_JPRB, 0.53242E+00_JPRB, 0.55356E+00_JPRB ,&
&  0.60369E+00_JPRB, 0.64865E+00_JPRB, 0.68839E+00_JPRB, 0.68468E+00_JPRB, 0.68330E+00_JPRB, 0.66876E+00_JPRB, 0.67247E+00_JPRB ,&
&  0.68516E+00_JPRB, 0.70756E+00_JPRB, 0.73241E+00_JPRB, 0.74340E+00_JPRB, 0.75130E+00_JPRB, 0.76129E+00_JPRB, 0.77736E+00_JPRB ,&
&  0.80297E+00_JPRB, 0.84097E+00_JPRB, 0.91148E+00_JPRB, 0.10415E+01_JPRB, 0.12474E+01_JPRB, 0.14585E+01_JPRB, 0.16093E+01_JPRB ,&
&  0.11118E+01_JPRB, 0.50857E+00_JPRB, 0.13813E+01_JPRB, 0.18405E+01_JPRB, 0.18247E+01_JPRB, 0.17414E+01_JPRB, 0.16653E+01_JPRB ,&
&  0.14481E+01_JPRB, 0.13296E+01_JPRB, 0.11826E+01_JPRB, 0.13098E+01_JPRB, 0.12801E+01_JPRB, 0.13846E+01_JPRB, 0.15374E+01_JPRB ,&
&  0.16987E+01_JPRB, 0.18469E+01_JPRB, 0.19833E+01_JPRB, 0.20429E+01_JPRB, 0.20725E+01_JPRB, 0.21454E+01_JPRB, 0.22920E+01_JPRB ,&
&  0.25521E+01_JPRB, 0.29814E+01_JPRB, 0.35374E+01_JPRB, 0.37840E+01_JPRB, 0.44037E+01_JPRB, 0.41629E+01_JPRB, 0.45106E+01_JPRB ,&
&  0.29277E+01_JPRB, 0.19528E+01_JPRB, 0.18198E+01_JPRB, 0.18332E+01_JPRB, 0.15093E+01_JPRB, 0.12359E+01_JPRB, 0.12150E+01_JPRB ,&
&  0.98584E+00_JPRB, 0.88129E+00_JPRB, 0.84781E+00_JPRB, 0.81478E+00_JPRB, 0.79019E+00_JPRB, 0.81893E+00_JPRB, 0.86869E+00_JPRB ,&
&  0.90609E+00_JPRB, 0.91161E+00_JPRB, 0.89454E+00_JPRB, 0.87887E+00_JPRB, 0.86576E+00_JPRB, 0.84279E+00_JPRB, 0.82292E+00_JPRB ,&
&  0.83098E+00_JPRB, 0.84303E+00_JPRB, 0.85951E+00_JPRB, 0.87657E+00_JPRB, 0.89629E+00_JPRB, 0.88136E+00_JPRB, 0.87079E+00_JPRB ,&
&  0.86512E+00_JPRB, 0.87575E+00_JPRB, 0.85626E+00_JPRB, 0.85307E+00_JPRB, 0.83259E+00_JPRB, 0.79100E+00_JPRB, 0.75808E+00_JPRB ,&
&  0.69942E+00_JPRB, 0.61029E+00_JPRB, 0.54863E+00_JPRB, 0.52891E+00_JPRB, 0.51198E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.53544E+00_JPRB, 0.52346E+00_JPRB, 0.52225E+00_JPRB, 0.53305E+00_JPRB, 0.54282E+00_JPRB, 0.55224E+00_JPRB, 0.53841E+00_JPRB ,&
&  0.55362E+00_JPRB, 0.59097E+00_JPRB, 0.61252E+00_JPRB, 0.63437E+00_JPRB, 0.62980E+00_JPRB, 0.61774E+00_JPRB, 0.61014E+00_JPRB ,&
&  0.60693E+00_JPRB, 0.61049E+00_JPRB, 0.62605E+00_JPRB, 0.63421E+00_JPRB, 0.64031E+00_JPRB, 0.64941E+00_JPRB, 0.65482E+00_JPRB ,&
&  0.66137E+00_JPRB, 0.67594E+00_JPRB, 0.70699E+00_JPRB, 0.76963E+00_JPRB, 0.88189E+00_JPRB, 0.10782E+01_JPRB, 0.13575E+01_JPRB ,&
&  0.18384E+01_JPRB, 0.13271E+01_JPRB, 0.64997E+00_JPRB, 0.14562E+01_JPRB, 0.18766E+01_JPRB, 0.18784E+01_JPRB, 0.16056E+01_JPRB ,&
&  0.15006E+01_JPRB, 0.13084E+01_JPRB, 0.11807E+01_JPRB, 0.12249E+01_JPRB, 0.10649E+01_JPRB, 0.10278E+01_JPRB, 0.10826E+01_JPRB ,&
&  0.12044E+01_JPRB, 0.13892E+01_JPRB, 0.15894E+01_JPRB, 0.17292E+01_JPRB, 0.18066E+01_JPRB, 0.18439E+01_JPRB, 0.19029E+01_JPRB ,&
&  0.20751E+01_JPRB, 0.23468E+01_JPRB, 0.28334E+01_JPRB, 0.36827E+01_JPRB, 0.53880E+01_JPRB, 0.50483E+01_JPRB, 0.50239E+01_JPRB ,&
&  0.31199E+01_JPRB, 0.25961E+01_JPRB, 0.23832E+01_JPRB, 0.21568E+01_JPRB, 0.17122E+01_JPRB, 0.14804E+01_JPRB, 0.14813E+01_JPRB ,&
&  0.11840E+01_JPRB, 0.99976E+00_JPRB, 0.93457E+00_JPRB, 0.88239E+00_JPRB, 0.85440E+00_JPRB, 0.86471E+00_JPRB, 0.90134E+00_JPRB ,&
&  0.94344E+00_JPRB, 0.94403E+00_JPRB, 0.92685E+00_JPRB, 0.91118E+00_JPRB, 0.90091E+00_JPRB, 0.88923E+00_JPRB, 0.87429E+00_JPRB ,&
&  0.86919E+00_JPRB, 0.88161E+00_JPRB, 0.90413E+00_JPRB, 0.89920E+00_JPRB, 0.87391E+00_JPRB, 0.85851E+00_JPRB, 0.88494E+00_JPRB ,&
&  0.87296E+00_JPRB, 0.89168E+00_JPRB, 0.87646E+00_JPRB, 0.88286E+00_JPRB, 0.85723E+00_JPRB, 0.83678E+00_JPRB, 0.81826E+00_JPRB ,&
&  0.78478E+00_JPRB, 0.70926E+00_JPRB, 0.62696E+00_JPRB, 0.54519E+00_JPRB, 0.53452E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.59786E+00_JPRB, 0.58149E+00_JPRB, 0.58445E+00_JPRB, 0.57843E+00_JPRB, 0.56858E+00_JPRB, 0.56768E+00_JPRB, 0.56219E+00_JPRB ,&
&  0.55259E+00_JPRB, 0.54762E+00_JPRB, 0.54088E+00_JPRB, 0.56940E+00_JPRB, 0.58403E+00_JPRB, 0.57393E+00_JPRB, 0.56403E+00_JPRB ,&
&  0.55632E+00_JPRB, 0.55677E+00_JPRB, 0.56191E+00_JPRB, 0.56624E+00_JPRB, 0.56859E+00_JPRB, 0.57506E+00_JPRB, 0.58082E+00_JPRB ,&
&  0.57770E+00_JPRB, 0.57883E+00_JPRB, 0.58875E+00_JPRB, 0.61291E+00_JPRB, 0.66163E+00_JPRB, 0.74812E+00_JPRB, 0.90440E+00_JPRB ,&
&  0.12709E+01_JPRB, 0.13010E+01_JPRB, 0.62812E+00_JPRB, 0.11612E+01_JPRB, 0.18838E+01_JPRB, 0.19616E+01_JPRB, 0.18463E+01_JPRB ,&
&  0.16695E+01_JPRB, 0.13258E+01_JPRB, 0.12344E+01_JPRB, 0.12384E+01_JPRB, 0.10719E+01_JPRB, 0.94943E+00_JPRB, 0.87745E+00_JPRB ,&
&  0.85806E+00_JPRB, 0.91821E+00_JPRB, 0.10655E+01_JPRB, 0.12406E+01_JPRB, 0.13978E+01_JPRB, 0.14959E+01_JPRB, 0.15490E+01_JPRB ,&
&  0.16243E+01_JPRB, 0.17755E+01_JPRB, 0.20914E+01_JPRB, 0.25247E+01_JPRB, 0.43591E+01_JPRB, 0.53636E+01_JPRB, 0.56661E+01_JPRB ,&
&  0.41562E+01_JPRB, 0.35745E+01_JPRB, 0.30825E+01_JPRB, 0.25950E+01_JPRB, 0.20165E+01_JPRB, 0.18383E+01_JPRB, 0.18101E+01_JPRB ,&
&  0.14762E+01_JPRB, 0.11980E+01_JPRB, 0.10742E+01_JPRB, 0.98910E+00_JPRB, 0.92906E+00_JPRB, 0.90447E+00_JPRB, 0.89787E+00_JPRB ,&
&  0.91551E+00_JPRB, 0.92603E+00_JPRB, 0.91145E+00_JPRB, 0.89795E+00_JPRB, 0.89768E+00_JPRB, 0.89733E+00_JPRB, 0.89588E+00_JPRB ,&
&  0.88679E+00_JPRB, 0.89616E+00_JPRB, 0.91460E+00_JPRB, 0.90788E+00_JPRB, 0.91248E+00_JPRB, 0.92553E+00_JPRB, 0.89540E+00_JPRB ,&
&  0.86433E+00_JPRB, 0.86924E+00_JPRB, 0.86234E+00_JPRB, 0.83835E+00_JPRB, 0.88034E+00_JPRB, 0.87516E+00_JPRB, 0.86603E+00_JPRB ,&
&  0.90778E+00_JPRB, 0.10166E+01_JPRB, 0.97301E+00_JPRB, 0.66388E+00_JPRB, 0.61178E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.64364E+00_JPRB, 0.61849E+00_JPRB, 0.61400E+00_JPRB, 0.60852E+00_JPRB, 0.59585E+00_JPRB, 0.58676E+00_JPRB, 0.58314E+00_JPRB ,&
&  0.56981E+00_JPRB, 0.54705E+00_JPRB, 0.52609E+00_JPRB, 0.52594E+00_JPRB, 0.53774E+00_JPRB, 0.54765E+00_JPRB, 0.53864E+00_JPRB ,&
&  0.52643E+00_JPRB, 0.52297E+00_JPRB, 0.53292E+00_JPRB, 0.53810E+00_JPRB, 0.53694E+00_JPRB, 0.53934E+00_JPRB, 0.53703E+00_JPRB ,&
&  0.53495E+00_JPRB, 0.53045E+00_JPRB, 0.53144E+00_JPRB, 0.54288E+00_JPRB, 0.56486E+00_JPRB, 0.60634E+00_JPRB, 0.67093E+00_JPRB ,&
&  0.88676E+00_JPRB, 0.10626E+01_JPRB, 0.13078E+01_JPRB, 0.20132E+01_JPRB, 0.24705E+01_JPRB, 0.23273E+01_JPRB, 0.21498E+01_JPRB ,&
&  0.19803E+01_JPRB, 0.16699E+01_JPRB, 0.14922E+01_JPRB, 0.13180E+01_JPRB, 0.11196E+01_JPRB, 0.97237E+00_JPRB, 0.87313E+00_JPRB ,&
&  0.81694E+00_JPRB, 0.78691E+00_JPRB, 0.79955E+00_JPRB, 0.86201E+00_JPRB, 0.96059E+00_JPRB, 0.10697E+01_JPRB, 0.11490E+01_JPRB ,&
&  0.12170E+01_JPRB, 0.13253E+01_JPRB, 0.15364E+01_JPRB, 0.18108E+01_JPRB, 0.28499E+01_JPRB, 0.45651E+01_JPRB, 0.54664E+01_JPRB ,&
&  0.54466E+01_JPRB, 0.48330E+01_JPRB, 0.39647E+01_JPRB, 0.31769E+01_JPRB, 0.23296E+01_JPRB, 0.23603E+01_JPRB, 0.20918E+01_JPRB ,&
&  0.17602E+01_JPRB, 0.14908E+01_JPRB, 0.12915E+01_JPRB, 0.11504E+01_JPRB, 0.10428E+01_JPRB, 0.96027E+00_JPRB, 0.91628E+00_JPRB ,&
&  0.90025E+00_JPRB, 0.90125E+00_JPRB, 0.88868E+00_JPRB, 0.87052E+00_JPRB, 0.87617E+00_JPRB, 0.88521E+00_JPRB, 0.89182E+00_JPRB ,&
&  0.88733E+00_JPRB, 0.89090E+00_JPRB, 0.87086E+00_JPRB, 0.88900E+00_JPRB, 0.91474E+00_JPRB, 0.90701E+00_JPRB, 0.87603E+00_JPRB ,&
&  0.83694E+00_JPRB, 0.84422E+00_JPRB, 0.84916E+00_JPRB, 0.84385E+00_JPRB, 0.86172E+00_JPRB, 0.87795E+00_JPRB, 0.87236E+00_JPRB ,&
&  0.88604E+00_JPRB, 0.86063E+00_JPRB, 0.80603E+00_JPRB, 0.71240E+00_JPRB, 0.67373E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.69377E+00_JPRB, 0.66657E+00_JPRB, 0.64809E+00_JPRB, 0.63395E+00_JPRB, 0.62733E+00_JPRB, 0.62742E+00_JPRB, 0.62219E+00_JPRB ,&
&  0.61117E+00_JPRB, 0.58871E+00_JPRB, 0.55921E+00_JPRB, 0.53331E+00_JPRB, 0.53156E+00_JPRB, 0.52921E+00_JPRB, 0.52840E+00_JPRB ,&
&  0.51798E+00_JPRB, 0.51397E+00_JPRB, 0.51759E+00_JPRB, 0.51763E+00_JPRB, 0.51543E+00_JPRB, 0.51181E+00_JPRB, 0.50955E+00_JPRB ,&
&  0.51290E+00_JPRB, 0.51358E+00_JPRB, 0.51256E+00_JPRB, 0.51517E+00_JPRB, 0.52508E+00_JPRB, 0.55104E+00_JPRB, 0.59323E+00_JPRB ,&
&  0.71666E+00_JPRB, 0.97801E+00_JPRB, 0.11675E+01_JPRB, 0.22844E+01_JPRB, 0.26498E+01_JPRB, 0.25599E+01_JPRB, 0.22442E+01_JPRB ,&
&  0.19608E+01_JPRB, 0.19248E+01_JPRB, 0.16237E+01_JPRB, 0.13422E+01_JPRB, 0.10997E+01_JPRB, 0.96285E+00_JPRB, 0.86316E+00_JPRB ,&
&  0.79319E+00_JPRB, 0.75522E+00_JPRB, 0.74350E+00_JPRB, 0.74545E+00_JPRB, 0.75592E+00_JPRB, 0.79715E+00_JPRB, 0.84363E+00_JPRB ,&
&  0.89687E+00_JPRB, 0.10143E+01_JPRB, 0.12280E+01_JPRB, 0.14292E+01_JPRB, 0.20568E+01_JPRB, 0.33979E+01_JPRB, 0.45716E+01_JPRB ,&
&  0.51391E+01_JPRB, 0.48559E+01_JPRB, 0.42701E+01_JPRB, 0.34779E+01_JPRB, 0.27919E+01_JPRB, 0.25274E+01_JPRB, 0.22415E+01_JPRB ,&
&  0.19537E+01_JPRB, 0.17594E+01_JPRB, 0.15025E+01_JPRB, 0.13166E+01_JPRB, 0.12155E+01_JPRB, 0.10941E+01_JPRB, 0.99034E+00_JPRB ,&
&  0.92637E+00_JPRB, 0.89679E+00_JPRB, 0.87837E+00_JPRB, 0.85859E+00_JPRB, 0.85339E+00_JPRB, 0.86446E+00_JPRB, 0.87421E+00_JPRB ,&
&  0.87027E+00_JPRB, 0.87392E+00_JPRB, 0.83714E+00_JPRB, 0.83465E+00_JPRB, 0.86205E+00_JPRB, 0.86287E+00_JPRB, 0.83843E+00_JPRB ,&
&  0.81918E+00_JPRB, 0.81791E+00_JPRB, 0.82119E+00_JPRB, 0.81423E+00_JPRB, 0.81038E+00_JPRB, 0.88303E+00_JPRB, 0.88236E+00_JPRB ,&
&  0.88320E+00_JPRB, 0.81986E+00_JPRB, 0.78375E+00_JPRB, 0.74310E+00_JPRB, 0.72003E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.71759E+00_JPRB, 0.69939E+00_JPRB, 0.68403E+00_JPRB, 0.66688E+00_JPRB, 0.65672E+00_JPRB, 0.65203E+00_JPRB, 0.64468E+00_JPRB ,&
&  0.62973E+00_JPRB, 0.60784E+00_JPRB, 0.58347E+00_JPRB, 0.55083E+00_JPRB, 0.53126E+00_JPRB, 0.52683E+00_JPRB, 0.53262E+00_JPRB ,&
&  0.52428E+00_JPRB, 0.50911E+00_JPRB, 0.50934E+00_JPRB, 0.51228E+00_JPRB, 0.51083E+00_JPRB, 0.50332E+00_JPRB, 0.49470E+00_JPRB ,&
&  0.49729E+00_JPRB, 0.50140E+00_JPRB, 0.50450E+00_JPRB, 0.50694E+00_JPRB, 0.51094E+00_JPRB, 0.52293E+00_JPRB, 0.55593E+00_JPRB ,&
&  0.65741E+00_JPRB, 0.11719E+01_JPRB, 0.12107E+01_JPRB, 0.21128E+01_JPRB, 0.22835E+01_JPRB, 0.22388E+01_JPRB, 0.20321E+01_JPRB ,&
&  0.18218E+01_JPRB, 0.16888E+01_JPRB, 0.14327E+01_JPRB, 0.12391E+01_JPRB, 0.10524E+01_JPRB, 0.91369E+00_JPRB, 0.82529E+00_JPRB ,&
&  0.76590E+00_JPRB, 0.72103E+00_JPRB, 0.69452E+00_JPRB, 0.68676E+00_JPRB, 0.68374E+00_JPRB, 0.68992E+00_JPRB, 0.70599E+00_JPRB ,&
&  0.73281E+00_JPRB, 0.81205E+00_JPRB, 0.10173E+01_JPRB, 0.14733E+01_JPRB, 0.18532E+01_JPRB, 0.27954E+01_JPRB, 0.42071E+01_JPRB ,&
&  0.49692E+01_JPRB, 0.48309E+01_JPRB, 0.40354E+01_JPRB, 0.33289E+01_JPRB, 0.27978E+01_JPRB, 0.24390E+01_JPRB, 0.21399E+01_JPRB ,&
&  0.19367E+01_JPRB, 0.17882E+01_JPRB, 0.16345E+01_JPRB, 0.14469E+01_JPRB, 0.13110E+01_JPRB, 0.12100E+01_JPRB, 0.11073E+01_JPRB ,&
&  0.10047E+01_JPRB, 0.92392E+00_JPRB, 0.87870E+00_JPRB, 0.84477E+00_JPRB, 0.82491E+00_JPRB, 0.82987E+00_JPRB, 0.83983E+00_JPRB ,&
&  0.83540E+00_JPRB, 0.83744E+00_JPRB, 0.80417E+00_JPRB, 0.79663E+00_JPRB, 0.81976E+00_JPRB, 0.82501E+00_JPRB, 0.80253E+00_JPRB ,&
&  0.79996E+00_JPRB, 0.80397E+00_JPRB, 0.80691E+00_JPRB, 0.82037E+00_JPRB, 0.80228E+00_JPRB, 0.88214E+00_JPRB, 0.87114E+00_JPRB ,&
&  0.86917E+00_JPRB, 0.82006E+00_JPRB, 0.78060E+00_JPRB, 0.76557E+00_JPRB, 0.74509E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.72583E+00_JPRB, 0.70869E+00_JPRB, 0.69578E+00_JPRB, 0.68751E+00_JPRB, 0.68141E+00_JPRB, 0.66908E+00_JPRB, 0.65524E+00_JPRB ,&
&  0.64192E+00_JPRB, 0.62624E+00_JPRB, 0.60105E+00_JPRB, 0.56506E+00_JPRB, 0.53671E+00_JPRB, 0.52379E+00_JPRB, 0.52625E+00_JPRB ,&
&  0.52677E+00_JPRB, 0.51768E+00_JPRB, 0.51111E+00_JPRB, 0.51012E+00_JPRB, 0.50884E+00_JPRB, 0.50111E+00_JPRB, 0.49119E+00_JPRB ,&
&  0.48385E+00_JPRB, 0.48474E+00_JPRB, 0.48847E+00_JPRB, 0.49401E+00_JPRB, 0.49897E+00_JPRB, 0.50621E+00_JPRB, 0.52913E+00_JPRB ,&
&  0.62489E+00_JPRB, 0.63691E+00_JPRB, 0.13839E+01_JPRB, 0.18097E+01_JPRB, 0.18405E+01_JPRB, 0.17806E+01_JPRB, 0.17575E+01_JPRB ,&
&  0.16300E+01_JPRB, 0.14429E+01_JPRB, 0.12290E+01_JPRB, 0.10522E+01_JPRB, 0.92016E+00_JPRB, 0.83319E+00_JPRB, 0.76895E+00_JPRB ,&
&  0.71937E+00_JPRB, 0.68140E+00_JPRB, 0.65075E+00_JPRB, 0.63523E+00_JPRB, 0.63803E+00_JPRB, 0.64552E+00_JPRB, 0.65485E+00_JPRB ,&
&  0.66989E+00_JPRB, 0.70406E+00_JPRB, 0.81654E+00_JPRB, 0.11748E+01_JPRB, 0.19787E+01_JPRB, 0.26838E+01_JPRB, 0.38306E+01_JPRB ,&
&  0.47911E+01_JPRB, 0.43394E+01_JPRB, 0.34918E+01_JPRB, 0.28967E+01_JPRB, 0.24666E+01_JPRB, 0.21664E+01_JPRB, 0.19400E+01_JPRB ,&
&  0.17683E+01_JPRB, 0.16460E+01_JPRB, 0.15449E+01_JPRB, 0.14329E+01_JPRB, 0.13370E+01_JPRB, 0.12619E+01_JPRB, 0.11858E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.10122E+01_JPRB, 0.93460E+00_JPRB, 0.86776E+00_JPRB, 0.81325E+00_JPRB, 0.79811E+00_JPRB, 0.79732E+00_JPRB ,&
&  0.79001E+00_JPRB, 0.78985E+00_JPRB, 0.77763E+00_JPRB, 0.78104E+00_JPRB, 0.80828E+00_JPRB, 0.81171E+00_JPRB, 0.80174E+00_JPRB ,&
&  0.77934E+00_JPRB, 0.78914E+00_JPRB, 0.78852E+00_JPRB, 0.78545E+00_JPRB, 0.75277E+00_JPRB, 0.84265E+00_JPRB, 0.83645E+00_JPRB ,&
&  0.83306E+00_JPRB, 0.81991E+00_JPRB, 0.78216E+00_JPRB, 0.76821E+00_JPRB, 0.75075E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.72566E+00_JPRB, 0.70491E+00_JPRB, 0.69317E+00_JPRB, 0.68556E+00_JPRB, 0.68003E+00_JPRB, 0.67471E+00_JPRB, 0.66082E+00_JPRB ,&
&  0.64327E+00_JPRB, 0.63281E+00_JPRB, 0.61823E+00_JPRB, 0.59815E+00_JPRB, 0.56878E+00_JPRB, 0.54667E+00_JPRB, 0.54079E+00_JPRB ,&
&  0.53679E+00_JPRB, 0.53001E+00_JPRB, 0.51795E+00_JPRB, 0.51109E+00_JPRB, 0.50340E+00_JPRB, 0.49393E+00_JPRB, 0.48813E+00_JPRB ,&
&  0.47768E+00_JPRB, 0.47321E+00_JPRB, 0.47568E+00_JPRB, 0.48231E+00_JPRB, 0.48858E+00_JPRB, 0.49664E+00_JPRB, 0.50745E+00_JPRB ,&
&  0.58188E+00_JPRB, 0.89898E+00_JPRB, 0.12582E+01_JPRB, 0.14611E+01_JPRB, 0.14860E+01_JPRB, 0.14012E+01_JPRB, 0.13986E+01_JPRB ,&
&  0.12658E+01_JPRB, 0.11405E+01_JPRB, 0.10090E+01_JPRB, 0.88581E+00_JPRB, 0.79093E+00_JPRB, 0.74126E+00_JPRB, 0.71315E+00_JPRB ,&
&  0.67981E+00_JPRB, 0.65015E+00_JPRB, 0.62327E+00_JPRB, 0.60046E+00_JPRB, 0.59250E+00_JPRB, 0.60288E+00_JPRB, 0.61921E+00_JPRB ,&
&  0.62752E+00_JPRB, 0.64137E+00_JPRB, 0.69763E+00_JPRB, 0.91778E+00_JPRB, 0.15849E+01_JPRB, 0.19538E+01_JPRB, 0.30097E+01_JPRB ,&
&  0.36025E+01_JPRB, 0.32189E+01_JPRB, 0.26694E+01_JPRB, 0.22685E+01_JPRB, 0.19462E+01_JPRB, 0.17028E+01_JPRB, 0.15268E+01_JPRB ,&
&  0.14255E+01_JPRB, 0.13555E+01_JPRB, 0.12947E+01_JPRB, 0.12364E+01_JPRB, 0.11823E+01_JPRB, 0.11401E+01_JPRB, 0.11055E+01_JPRB ,&
&  0.10668E+01_JPRB, 0.10177E+01_JPRB, 0.96274E+00_JPRB, 0.90036E+00_JPRB, 0.83770E+00_JPRB, 0.79756E+00_JPRB, 0.77532E+00_JPRB ,&
&  0.75691E+00_JPRB, 0.75390E+00_JPRB, 0.75350E+00_JPRB, 0.76353E+00_JPRB, 0.78961E+00_JPRB, 0.80153E+00_JPRB, 0.79917E+00_JPRB ,&
&  0.76616E+00_JPRB, 0.75496E+00_JPRB, 0.74842E+00_JPRB, 0.71974E+00_JPRB, 0.71747E+00_JPRB, 0.79003E+00_JPRB, 0.77658E+00_JPRB ,&
&  0.76498E+00_JPRB, 0.76060E+00_JPRB, 0.75090E+00_JPRB, 0.74262E+00_JPRB, 0.75101E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.73889E+00_JPRB, 0.70919E+00_JPRB, 0.69066E+00_JPRB, 0.67927E+00_JPRB, 0.67360E+00_JPRB, 0.66779E+00_JPRB, 0.65533E+00_JPRB ,&
&  0.63978E+00_JPRB, 0.62457E+00_JPRB, 0.61176E+00_JPRB, 0.60237E+00_JPRB, 0.58274E+00_JPRB, 0.55021E+00_JPRB, 0.52496E+00_JPRB ,&
&  0.51720E+00_JPRB, 0.51082E+00_JPRB, 0.50540E+00_JPRB, 0.49588E+00_JPRB, 0.48776E+00_JPRB, 0.48426E+00_JPRB, 0.48049E+00_JPRB ,&
&  0.47611E+00_JPRB, 0.47172E+00_JPRB, 0.47304E+00_JPRB, 0.47770E+00_JPRB, 0.48212E+00_JPRB, 0.48619E+00_JPRB, 0.49218E+00_JPRB ,&
&  0.52884E+00_JPRB, 0.71539E+00_JPRB, 0.87400E+00_JPRB, 0.10967E+01_JPRB, 0.11602E+01_JPRB, 0.11050E+01_JPRB, 0.10669E+01_JPRB ,&
&  0.10031E+01_JPRB, 0.92952E+00_JPRB, 0.85945E+00_JPRB, 0.79183E+00_JPRB, 0.71719E+00_JPRB, 0.67485E+00_JPRB, 0.65516E+00_JPRB ,&
&  0.64053E+00_JPRB, 0.62012E+00_JPRB, 0.59787E+00_JPRB, 0.57484E+00_JPRB, 0.55937E+00_JPRB, 0.56306E+00_JPRB, 0.57420E+00_JPRB ,&
&  0.57322E+00_JPRB, 0.57532E+00_JPRB, 0.59006E+00_JPRB, 0.67659E+00_JPRB, 0.10116E+01_JPRB, 0.13458E+01_JPRB, 0.18179E+01_JPRB ,&
&  0.18615E+01_JPRB, 0.19646E+01_JPRB, 0.17938E+01_JPRB, 0.16005E+01_JPRB, 0.14348E+01_JPRB, 0.13067E+01_JPRB, 0.12099E+01_JPRB ,&
&  0.11433E+01_JPRB, 0.10889E+01_JPRB, 0.10484E+01_JPRB, 0.10181E+01_JPRB, 0.99361E+00_JPRB, 0.96431E+00_JPRB, 0.94831E+00_JPRB ,&
&  0.93947E+00_JPRB, 0.91866E+00_JPRB, 0.89270E+00_JPRB, 0.86277E+00_JPRB, 0.82641E+00_JPRB, 0.79523E+00_JPRB, 0.77193E+00_JPRB ,&
&  0.75094E+00_JPRB, 0.72917E+00_JPRB, 0.72234E+00_JPRB, 0.73141E+00_JPRB, 0.74168E+00_JPRB, 0.74932E+00_JPRB, 0.74509E+00_JPRB ,&
&  0.73593E+00_JPRB, 0.72835E+00_JPRB, 0.71598E+00_JPRB, 0.69017E+00_JPRB, 0.68480E+00_JPRB, 0.73350E+00_JPRB, 0.72555E+00_JPRB ,&
&  0.71080E+00_JPRB, 0.70051E+00_JPRB, 0.70263E+00_JPRB, 0.71033E+00_JPRB, 0.76475E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.76618E+00_JPRB, 0.73256E+00_JPRB, 0.69788E+00_JPRB, 0.67598E+00_JPRB, 0.66125E+00_JPRB, 0.64893E+00_JPRB, 0.63306E+00_JPRB ,&
&  0.61804E+00_JPRB, 0.60275E+00_JPRB, 0.58991E+00_JPRB, 0.58036E+00_JPRB, 0.57000E+00_JPRB, 0.54756E+00_JPRB, 0.51707E+00_JPRB ,&
&  0.49607E+00_JPRB, 0.48957E+00_JPRB, 0.48519E+00_JPRB, 0.47740E+00_JPRB, 0.47043E+00_JPRB, 0.46819E+00_JPRB, 0.46577E+00_JPRB ,&
&  0.46620E+00_JPRB, 0.46731E+00_JPRB, 0.47113E+00_JPRB, 0.47322E+00_JPRB, 0.47134E+00_JPRB, 0.47772E+00_JPRB, 0.47770E+00_JPRB ,&
&  0.50698E+00_JPRB, 0.55689E+00_JPRB, 0.66460E+00_JPRB, 0.78648E+00_JPRB, 0.86911E+00_JPRB, 0.89899E+00_JPRB, 0.88670E+00_JPRB ,&
&  0.86206E+00_JPRB, 0.83413E+00_JPRB, 0.79228E+00_JPRB, 0.74856E+00_JPRB, 0.69878E+00_JPRB, 0.64495E+00_JPRB, 0.61296E+00_JPRB ,&
&  0.60311E+00_JPRB, 0.59148E+00_JPRB, 0.57880E+00_JPRB, 0.56300E+00_JPRB, 0.54769E+00_JPRB, 0.54295E+00_JPRB, 0.54435E+00_JPRB ,&
&  0.54032E+00_JPRB, 0.53594E+00_JPRB, 0.53236E+00_JPRB, 0.53961E+00_JPRB, 0.59526E+00_JPRB, 0.75612E+00_JPRB, 0.97127E+00_JPRB ,&
&  0.10594E+01_JPRB, 0.11237E+01_JPRB, 0.11589E+01_JPRB, 0.11506E+01_JPRB, 0.11016E+01_JPRB, 0.10296E+01_JPRB, 0.95389E+00_JPRB ,&
&  0.89805E+00_JPRB, 0.85943E+00_JPRB, 0.83899E+00_JPRB, 0.83207E+00_JPRB, 0.82966E+00_JPRB, 0.82216E+00_JPRB, 0.80715E+00_JPRB ,&
&  0.79471E+00_JPRB, 0.78879E+00_JPRB, 0.78842E+00_JPRB, 0.78274E+00_JPRB, 0.77437E+00_JPRB, 0.76407E+00_JPRB, 0.75710E+00_JPRB ,&
&  0.75552E+00_JPRB, 0.74749E+00_JPRB, 0.72715E+00_JPRB, 0.70864E+00_JPRB, 0.70827E+00_JPRB, 0.70853E+00_JPRB, 0.70415E+00_JPRB ,&
&  0.69878E+00_JPRB, 0.70064E+00_JPRB, 0.68743E+00_JPRB, 0.68884E+00_JPRB, 0.71391E+00_JPRB, 0.69804E+00_JPRB, 0.68956E+00_JPRB ,&
&  0.67049E+00_JPRB, 0.65942E+00_JPRB, 0.66345E+00_JPRB, 0.68144E+00_JPRB, 0.75320E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.64628E+00_JPRB, 0.64097E+00_JPRB, 0.63273E+00_JPRB, 0.62529E+00_JPRB, 0.61382E+00_JPRB, 0.60278E+00_JPRB, 0.58849E+00_JPRB ,&
&  0.57512E+00_JPRB, 0.56204E+00_JPRB, 0.55077E+00_JPRB, 0.54701E+00_JPRB, 0.53972E+00_JPRB, 0.52884E+00_JPRB, 0.50736E+00_JPRB ,&
&  0.48926E+00_JPRB, 0.47533E+00_JPRB, 0.46675E+00_JPRB, 0.46286E+00_JPRB, 0.46024E+00_JPRB, 0.45691E+00_JPRB, 0.45568E+00_JPRB ,&
&  0.45677E+00_JPRB, 0.45680E+00_JPRB, 0.45954E+00_JPRB, 0.45937E+00_JPRB, 0.46237E+00_JPRB, 0.45831E+00_JPRB, 0.45559E+00_JPRB ,&
&  0.47656E+00_JPRB, 0.48445E+00_JPRB, 0.54647E+00_JPRB, 0.61576E+00_JPRB, 0.68454E+00_JPRB, 0.73177E+00_JPRB, 0.77175E+00_JPRB ,&
&  0.79644E+00_JPRB, 0.78040E+00_JPRB, 0.74844E+00_JPRB, 0.70241E+00_JPRB, 0.66639E+00_JPRB, 0.62743E+00_JPRB, 0.58842E+00_JPRB ,&
&  0.56888E+00_JPRB, 0.55914E+00_JPRB, 0.55103E+00_JPRB, 0.54385E+00_JPRB, 0.53637E+00_JPRB, 0.53205E+00_JPRB, 0.53004E+00_JPRB ,&
&  0.52345E+00_JPRB, 0.51528E+00_JPRB, 0.50552E+00_JPRB, 0.50226E+00_JPRB, 0.50482E+00_JPRB, 0.54218E+00_JPRB, 0.64254E+00_JPRB ,&
&  0.69637E+00_JPRB, 0.72541E+00_JPRB, 0.75087E+00_JPRB, 0.78916E+00_JPRB, 0.81631E+00_JPRB, 0.81702E+00_JPRB, 0.77740E+00_JPRB ,&
&  0.73867E+00_JPRB, 0.70935E+00_JPRB, 0.69298E+00_JPRB, 0.68651E+00_JPRB, 0.68767E+00_JPRB, 0.68954E+00_JPRB, 0.69135E+00_JPRB ,&
&  0.68455E+00_JPRB, 0.67724E+00_JPRB, 0.67545E+00_JPRB, 0.67712E+00_JPRB, 0.68222E+00_JPRB, 0.68678E+00_JPRB, 0.69400E+00_JPRB ,&
&  0.70772E+00_JPRB, 0.72514E+00_JPRB, 0.73860E+00_JPRB, 0.72593E+00_JPRB, 0.70224E+00_JPRB, 0.68501E+00_JPRB, 0.67369E+00_JPRB ,&
&  0.66595E+00_JPRB, 0.66669E+00_JPRB, 0.67148E+00_JPRB, 0.68547E+00_JPRB, 0.68642E+00_JPRB, 0.67363E+00_JPRB, 0.65543E+00_JPRB ,&
&  0.63880E+00_JPRB, 0.62774E+00_JPRB, 0.62740E+00_JPRB, 0.63521E+00_JPRB, 0.62219E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.57698E+00_JPRB, 0.57471E+00_JPRB, 0.57034E+00_JPRB, 0.56722E+00_JPRB, 0.56038E+00_JPRB, 0.55262E+00_JPRB, 0.54451E+00_JPRB ,&
&  0.53407E+00_JPRB, 0.52381E+00_JPRB, 0.51497E+00_JPRB, 0.51225E+00_JPRB, 0.51043E+00_JPRB, 0.50853E+00_JPRB, 0.50141E+00_JPRB ,&
&  0.49042E+00_JPRB, 0.47970E+00_JPRB, 0.47007E+00_JPRB, 0.46384E+00_JPRB, 0.46154E+00_JPRB, 0.45873E+00_JPRB, 0.45504E+00_JPRB ,&
&  0.45268E+00_JPRB, 0.45315E+00_JPRB, 0.45231E+00_JPRB, 0.45114E+00_JPRB, 0.44929E+00_JPRB, 0.44376E+00_JPRB, 0.44345E+00_JPRB ,&
&  0.45515E+00_JPRB, 0.42233E+00_JPRB, 0.45654E+00_JPRB, 0.49252E+00_JPRB, 0.52353E+00_JPRB, 0.57366E+00_JPRB, 0.63821E+00_JPRB ,&
&  0.69526E+00_JPRB, 0.70780E+00_JPRB, 0.69575E+00_JPRB, 0.66262E+00_JPRB, 0.63714E+00_JPRB, 0.61797E+00_JPRB, 0.58994E+00_JPRB ,&
&  0.56212E+00_JPRB, 0.54364E+00_JPRB, 0.53077E+00_JPRB, 0.52047E+00_JPRB, 0.51315E+00_JPRB, 0.50689E+00_JPRB, 0.50128E+00_JPRB ,&
&  0.49343E+00_JPRB, 0.48543E+00_JPRB, 0.47839E+00_JPRB, 0.47490E+00_JPRB, 0.47652E+00_JPRB, 0.48353E+00_JPRB, 0.51367E+00_JPRB ,&
&  0.54495E+00_JPRB, 0.56409E+00_JPRB, 0.56939E+00_JPRB, 0.58680E+00_JPRB, 0.61324E+00_JPRB, 0.63589E+00_JPRB, 0.64188E+00_JPRB ,&
&  0.62675E+00_JPRB, 0.60310E+00_JPRB, 0.58282E+00_JPRB, 0.56816E+00_JPRB, 0.56113E+00_JPRB, 0.56308E+00_JPRB, 0.57209E+00_JPRB ,&
&  0.57650E+00_JPRB, 0.57628E+00_JPRB, 0.57713E+00_JPRB, 0.58127E+00_JPRB, 0.58709E+00_JPRB, 0.59729E+00_JPRB, 0.61144E+00_JPRB ,&
&  0.62916E+00_JPRB, 0.64948E+00_JPRB, 0.67237E+00_JPRB, 0.68977E+00_JPRB, 0.69323E+00_JPRB, 0.68935E+00_JPRB, 0.68020E+00_JPRB ,&
&  0.66963E+00_JPRB, 0.66106E+00_JPRB, 0.65315E+00_JPRB, 0.63347E+00_JPRB, 0.65321E+00_JPRB, 0.65004E+00_JPRB, 0.63237E+00_JPRB ,&
&  0.61656E+00_JPRB, 0.59929E+00_JPRB, 0.59036E+00_JPRB, 0.53864E+00_JPRB, 0.57073E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.52115E+00_JPRB, 0.51925E+00_JPRB, 0.51462E+00_JPRB, 0.51025E+00_JPRB, 0.50727E+00_JPRB, 0.50279E+00_JPRB, 0.49931E+00_JPRB ,&
&  0.49432E+00_JPRB, 0.49075E+00_JPRB, 0.48439E+00_JPRB, 0.48040E+00_JPRB, 0.47854E+00_JPRB, 0.47847E+00_JPRB, 0.47803E+00_JPRB ,&
&  0.47431E+00_JPRB, 0.46749E+00_JPRB, 0.46026E+00_JPRB, 0.45441E+00_JPRB, 0.44988E+00_JPRB, 0.44496E+00_JPRB, 0.44028E+00_JPRB ,&
&  0.43614E+00_JPRB, 0.43249E+00_JPRB, 0.42922E+00_JPRB, 0.42894E+00_JPRB, 0.42886E+00_JPRB, 0.42781E+00_JPRB, 0.42546E+00_JPRB ,&
&  0.43178E+00_JPRB, 0.38146E+00_JPRB, 0.39559E+00_JPRB, 0.41153E+00_JPRB, 0.42877E+00_JPRB, 0.46961E+00_JPRB, 0.52720E+00_JPRB ,&
&  0.57797E+00_JPRB, 0.61463E+00_JPRB, 0.62508E+00_JPRB, 0.61080E+00_JPRB, 0.59241E+00_JPRB, 0.58019E+00_JPRB, 0.56978E+00_JPRB ,&
&  0.55025E+00_JPRB, 0.53291E+00_JPRB, 0.51668E+00_JPRB, 0.50090E+00_JPRB, 0.48595E+00_JPRB, 0.47318E+00_JPRB, 0.46208E+00_JPRB ,&
&  0.45502E+00_JPRB, 0.45205E+00_JPRB, 0.45009E+00_JPRB, 0.44871E+00_JPRB, 0.44948E+00_JPRB, 0.45218E+00_JPRB, 0.45532E+00_JPRB ,&
&  0.46240E+00_JPRB, 0.47582E+00_JPRB, 0.48212E+00_JPRB, 0.48753E+00_JPRB, 0.49876E+00_JPRB, 0.51551E+00_JPRB, 0.52426E+00_JPRB ,&
&  0.52466E+00_JPRB, 0.51856E+00_JPRB, 0.50955E+00_JPRB, 0.49997E+00_JPRB, 0.49141E+00_JPRB, 0.48415E+00_JPRB, 0.48301E+00_JPRB ,&
&  0.48323E+00_JPRB, 0.48570E+00_JPRB, 0.48915E+00_JPRB, 0.49587E+00_JPRB, 0.50396E+00_JPRB, 0.51237E+00_JPRB, 0.52101E+00_JPRB ,&
&  0.53009E+00_JPRB, 0.54033E+00_JPRB, 0.55468E+00_JPRB, 0.57090E+00_JPRB, 0.58390E+00_JPRB, 0.59027E+00_JPRB, 0.59175E+00_JPRB ,&
&  0.59004E+00_JPRB, 0.58758E+00_JPRB, 0.58130E+00_JPRB, 0.58260E+00_JPRB, 0.58152E+00_JPRB, 0.58202E+00_JPRB, 0.57500E+00_JPRB ,&
&  0.56589E+00_JPRB, 0.55673E+00_JPRB, 0.48330E+00_JPRB, 0.51258E+00_JPRB, 0.51820E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.48654E+00_JPRB, 0.48111E+00_JPRB, 0.47575E+00_JPRB, 0.46984E+00_JPRB, 0.46507E+00_JPRB, 0.46180E+00_JPRB, 0.46051E+00_JPRB ,&
&  0.46156E+00_JPRB, 0.46309E+00_JPRB, 0.46199E+00_JPRB, 0.45698E+00_JPRB, 0.45224E+00_JPRB, 0.44943E+00_JPRB, 0.44846E+00_JPRB ,&
&  0.44681E+00_JPRB, 0.44337E+00_JPRB, 0.43844E+00_JPRB, 0.43364E+00_JPRB, 0.42795E+00_JPRB, 0.42304E+00_JPRB, 0.41880E+00_JPRB ,&
&  0.41525E+00_JPRB, 0.41351E+00_JPRB, 0.41213E+00_JPRB, 0.41118E+00_JPRB, 0.41142E+00_JPRB, 0.41081E+00_JPRB, 0.41477E+00_JPRB ,&
&  0.38818E+00_JPRB, 0.35609E+00_JPRB, 0.36917E+00_JPRB, 0.38175E+00_JPRB, 0.39400E+00_JPRB, 0.42125E+00_JPRB, 0.45873E+00_JPRB ,&
&  0.50102E+00_JPRB, 0.53686E+00_JPRB, 0.55745E+00_JPRB, 0.55463E+00_JPRB, 0.54164E+00_JPRB, 0.53273E+00_JPRB, 0.52804E+00_JPRB ,&
&  0.51789E+00_JPRB, 0.50568E+00_JPRB, 0.49545E+00_JPRB, 0.48335E+00_JPRB, 0.47040E+00_JPRB, 0.45846E+00_JPRB, 0.44887E+00_JPRB ,&
&  0.44142E+00_JPRB, 0.43695E+00_JPRB, 0.43571E+00_JPRB, 0.43417E+00_JPRB, 0.43288E+00_JPRB, 0.43319E+00_JPRB, 0.43254E+00_JPRB ,&
&  0.43287E+00_JPRB, 0.43373E+00_JPRB, 0.43747E+00_JPRB, 0.44067E+00_JPRB, 0.44505E+00_JPRB, 0.44867E+00_JPRB, 0.45008E+00_JPRB ,&
&  0.44862E+00_JPRB, 0.44747E+00_JPRB, 0.44798E+00_JPRB, 0.44596E+00_JPRB, 0.43894E+00_JPRB, 0.43127E+00_JPRB, 0.42768E+00_JPRB ,&
&  0.42758E+00_JPRB, 0.42855E+00_JPRB, 0.43004E+00_JPRB, 0.43340E+00_JPRB, 0.43818E+00_JPRB, 0.44367E+00_JPRB, 0.44954E+00_JPRB ,&
&  0.45582E+00_JPRB, 0.46176E+00_JPRB, 0.46743E+00_JPRB, 0.47432E+00_JPRB, 0.48422E+00_JPRB, 0.49318E+00_JPRB, 0.50084E+00_JPRB ,&
&  0.50703E+00_JPRB, 0.51208E+00_JPRB, 0.51622E+00_JPRB, 0.52212E+00_JPRB, 0.52735E+00_JPRB, 0.53035E+00_JPRB, 0.52776E+00_JPRB ,&
&  0.52072E+00_JPRB, 0.51034E+00_JPRB, 0.50550E+00_JPRB, 0.49707E+00_JPRB, 0.49049E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.43820E+00_JPRB, 0.43492E+00_JPRB, 0.43061E+00_JPRB, 0.42596E+00_JPRB, 0.42120E+00_JPRB, 0.41893E+00_JPRB, 0.41846E+00_JPRB ,&
&  0.42067E+00_JPRB, 0.42164E+00_JPRB, 0.42377E+00_JPRB, 0.42226E+00_JPRB, 0.41981E+00_JPRB, 0.41790E+00_JPRB, 0.41584E+00_JPRB ,&
&  0.41342E+00_JPRB, 0.41017E+00_JPRB, 0.40693E+00_JPRB, 0.40437E+00_JPRB, 0.40133E+00_JPRB, 0.39702E+00_JPRB, 0.39296E+00_JPRB ,&
&  0.38979E+00_JPRB, 0.38824E+00_JPRB, 0.38722E+00_JPRB, 0.38638E+00_JPRB, 0.38553E+00_JPRB, 0.38614E+00_JPRB, 0.38637E+00_JPRB ,&
&  0.38845E+00_JPRB, 0.35837E+00_JPRB, 0.36921E+00_JPRB, 0.37346E+00_JPRB, 0.38148E+00_JPRB, 0.39865E+00_JPRB, 0.41950E+00_JPRB ,&
&  0.44230E+00_JPRB, 0.46415E+00_JPRB, 0.47984E+00_JPRB, 0.48357E+00_JPRB, 0.47799E+00_JPRB, 0.47424E+00_JPRB, 0.47499E+00_JPRB ,&
&  0.47328E+00_JPRB, 0.46880E+00_JPRB, 0.46486E+00_JPRB, 0.46009E+00_JPRB, 0.45305E+00_JPRB, 0.44308E+00_JPRB, 0.43335E+00_JPRB ,&
&  0.42777E+00_JPRB, 0.42463E+00_JPRB, 0.42181E+00_JPRB, 0.41871E+00_JPRB, 0.41347E+00_JPRB, 0.40848E+00_JPRB, 0.40409E+00_JPRB ,&
&  0.40004E+00_JPRB, 0.39468E+00_JPRB, 0.39187E+00_JPRB, 0.39122E+00_JPRB, 0.39022E+00_JPRB, 0.39079E+00_JPRB, 0.39127E+00_JPRB ,&
&  0.39004E+00_JPRB, 0.38739E+00_JPRB, 0.38614E+00_JPRB, 0.38651E+00_JPRB, 0.38568E+00_JPRB, 0.38445E+00_JPRB, 0.38275E+00_JPRB ,&
&  0.38287E+00_JPRB, 0.38333E+00_JPRB, 0.38309E+00_JPRB, 0.38357E+00_JPRB, 0.38532E+00_JPRB, 0.38883E+00_JPRB, 0.39296E+00_JPRB ,&
&  0.39750E+00_JPRB, 0.40249E+00_JPRB, 0.40704E+00_JPRB, 0.41119E+00_JPRB, 0.41714E+00_JPRB, 0.42295E+00_JPRB, 0.43026E+00_JPRB ,&
&  0.43855E+00_JPRB, 0.44620E+00_JPRB, 0.45301E+00_JPRB, 0.45934E+00_JPRB, 0.46364E+00_JPRB, 0.46505E+00_JPRB, 0.46479E+00_JPRB ,&
&  0.46318E+00_JPRB, 0.45666E+00_JPRB, 0.45128E+00_JPRB, 0.44534E+00_JPRB, 0.44031E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.40157E+00_JPRB, 0.39767E+00_JPRB, 0.39317E+00_JPRB, 0.38967E+00_JPRB, 0.38646E+00_JPRB, 0.38317E+00_JPRB, 0.38190E+00_JPRB ,&
&  0.38238E+00_JPRB, 0.38411E+00_JPRB, 0.38682E+00_JPRB, 0.38832E+00_JPRB, 0.38900E+00_JPRB, 0.38994E+00_JPRB, 0.38974E+00_JPRB ,&
&  0.38890E+00_JPRB, 0.38738E+00_JPRB, 0.38527E+00_JPRB, 0.38234E+00_JPRB, 0.37885E+00_JPRB, 0.37558E+00_JPRB, 0.37127E+00_JPRB ,&
&  0.36820E+00_JPRB, 0.36632E+00_JPRB, 0.36518E+00_JPRB, 0.36470E+00_JPRB, 0.36554E+00_JPRB, 0.36674E+00_JPRB, 0.36783E+00_JPRB ,&
&  0.36706E+00_JPRB, 0.35775E+00_JPRB, 0.36333E+00_JPRB, 0.36603E+00_JPRB, 0.37405E+00_JPRB, 0.38805E+00_JPRB, 0.40076E+00_JPRB ,&
&  0.40932E+00_JPRB, 0.41966E+00_JPRB, 0.42978E+00_JPRB, 0.43595E+00_JPRB, 0.43491E+00_JPRB, 0.43297E+00_JPRB, 0.43482E+00_JPRB ,&
&  0.43603E+00_JPRB, 0.43509E+00_JPRB, 0.43416E+00_JPRB, 0.43239E+00_JPRB, 0.42928E+00_JPRB, 0.42570E+00_JPRB, 0.42008E+00_JPRB ,&
&  0.41445E+00_JPRB, 0.41023E+00_JPRB, 0.40588E+00_JPRB, 0.40139E+00_JPRB, 0.39486E+00_JPRB, 0.38723E+00_JPRB, 0.37833E+00_JPRB ,&
&  0.36887E+00_JPRB, 0.36211E+00_JPRB, 0.35784E+00_JPRB, 0.35424E+00_JPRB, 0.35116E+00_JPRB, 0.34984E+00_JPRB, 0.34824E+00_JPRB ,&
&  0.34771E+00_JPRB, 0.34829E+00_JPRB, 0.34741E+00_JPRB, 0.34700E+00_JPRB, 0.34802E+00_JPRB, 0.34858E+00_JPRB, 0.35019E+00_JPRB ,&
&  0.35039E+00_JPRB, 0.35035E+00_JPRB, 0.35045E+00_JPRB, 0.35021E+00_JPRB, 0.34958E+00_JPRB, 0.35095E+00_JPRB, 0.35361E+00_JPRB ,&
&  0.35650E+00_JPRB, 0.36078E+00_JPRB, 0.36538E+00_JPRB, 0.36918E+00_JPRB, 0.37270E+00_JPRB, 0.37834E+00_JPRB, 0.38559E+00_JPRB ,&
&  0.39386E+00_JPRB, 0.40279E+00_JPRB, 0.41095E+00_JPRB, 0.41741E+00_JPRB, 0.41969E+00_JPRB, 0.42026E+00_JPRB, 0.42043E+00_JPRB ,&
&  0.41877E+00_JPRB, 0.41330E+00_JPRB, 0.41031E+00_JPRB, 0.40799E+00_JPRB, 0.40492E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.35549E+00_JPRB, 0.35410E+00_JPRB, 0.35297E+00_JPRB, 0.35258E+00_JPRB, 0.35235E+00_JPRB, 0.35231E+00_JPRB, 0.35295E+00_JPRB ,&
&  0.35448E+00_JPRB, 0.35580E+00_JPRB, 0.35968E+00_JPRB, 0.36175E+00_JPRB, 0.36262E+00_JPRB, 0.36403E+00_JPRB, 0.36498E+00_JPRB ,&
&  0.36395E+00_JPRB, 0.36300E+00_JPRB, 0.36181E+00_JPRB, 0.35942E+00_JPRB, 0.35555E+00_JPRB, 0.35334E+00_JPRB, 0.35131E+00_JPRB ,&
&  0.34819E+00_JPRB, 0.34447E+00_JPRB, 0.34118E+00_JPRB, 0.33934E+00_JPRB, 0.33921E+00_JPRB, 0.33864E+00_JPRB, 0.33903E+00_JPRB ,&
&  0.33888E+00_JPRB, 0.34217E+00_JPRB, 0.33041E+00_JPRB, 0.33997E+00_JPRB, 0.34891E+00_JPRB, 0.35777E+00_JPRB, 0.36718E+00_JPRB ,&
&  0.37381E+00_JPRB, 0.38036E+00_JPRB, 0.38832E+00_JPRB, 0.39162E+00_JPRB, 0.39130E+00_JPRB, 0.38827E+00_JPRB, 0.38750E+00_JPRB ,&
&  0.38707E+00_JPRB, 0.38444E+00_JPRB, 0.38300E+00_JPRB, 0.38358E+00_JPRB, 0.38435E+00_JPRB, 0.38339E+00_JPRB, 0.38157E+00_JPRB ,&
&  0.37998E+00_JPRB, 0.37723E+00_JPRB, 0.37248E+00_JPRB, 0.36712E+00_JPRB, 0.36151E+00_JPRB, 0.35475E+00_JPRB, 0.34577E+00_JPRB ,&
&  0.33560E+00_JPRB, 0.32753E+00_JPRB, 0.32085E+00_JPRB, 0.31667E+00_JPRB, 0.31483E+00_JPRB, 0.31337E+00_JPRB, 0.31216E+00_JPRB ,&
&  0.31218E+00_JPRB, 0.31275E+00_JPRB, 0.31281E+00_JPRB, 0.31251E+00_JPRB, 0.31330E+00_JPRB, 0.31417E+00_JPRB, 0.31718E+00_JPRB ,&
&  0.32067E+00_JPRB, 0.32262E+00_JPRB, 0.32370E+00_JPRB, 0.32457E+00_JPRB, 0.32345E+00_JPRB, 0.32418E+00_JPRB, 0.32611E+00_JPRB ,&
&  0.32820E+00_JPRB, 0.33086E+00_JPRB, 0.33271E+00_JPRB, 0.33458E+00_JPRB, 0.33717E+00_JPRB, 0.34102E+00_JPRB, 0.34688E+00_JPRB ,&
&  0.35263E+00_JPRB, 0.35887E+00_JPRB, 0.36551E+00_JPRB, 0.37078E+00_JPRB, 0.37387E+00_JPRB, 0.37476E+00_JPRB, 0.37494E+00_JPRB ,&
&  0.37306E+00_JPRB, 0.36852E+00_JPRB, 0.36391E+00_JPRB, 0.36051E+00_JPRB, 0.35769E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.32249E+00_JPRB, 0.32290E+00_JPRB, 0.32422E+00_JPRB, 0.32480E+00_JPRB, 0.32486E+00_JPRB, 0.32578E+00_JPRB, 0.32886E+00_JPRB ,&
&  0.33173E+00_JPRB, 0.33380E+00_JPRB, 0.33578E+00_JPRB, 0.33705E+00_JPRB, 0.33782E+00_JPRB, 0.33988E+00_JPRB, 0.34147E+00_JPRB ,&
&  0.34191E+00_JPRB, 0.34096E+00_JPRB, 0.33972E+00_JPRB, 0.33806E+00_JPRB, 0.33571E+00_JPRB, 0.33312E+00_JPRB, 0.33093E+00_JPRB ,&
&  0.32805E+00_JPRB, 0.32481E+00_JPRB, 0.32160E+00_JPRB, 0.32004E+00_JPRB, 0.31956E+00_JPRB, 0.31934E+00_JPRB, 0.31992E+00_JPRB ,&
&  0.32050E+00_JPRB, 0.32284E+00_JPRB, 0.32567E+00_JPRB, 0.32909E+00_JPRB, 0.33250E+00_JPRB, 0.33705E+00_JPRB, 0.34208E+00_JPRB ,&
&  0.34685E+00_JPRB, 0.34938E+00_JPRB, 0.35156E+00_JPRB, 0.35025E+00_JPRB, 0.35078E+00_JPRB, 0.35097E+00_JPRB, 0.35093E+00_JPRB ,&
&  0.34980E+00_JPRB, 0.34457E+00_JPRB, 0.34048E+00_JPRB, 0.34033E+00_JPRB, 0.34153E+00_JPRB, 0.34248E+00_JPRB, 0.34304E+00_JPRB ,&
&  0.34236E+00_JPRB, 0.34094E+00_JPRB, 0.33775E+00_JPRB, 0.33331E+00_JPRB, 0.32857E+00_JPRB, 0.32294E+00_JPRB, 0.31505E+00_JPRB ,&
&  0.30773E+00_JPRB, 0.30179E+00_JPRB, 0.29713E+00_JPRB, 0.29305E+00_JPRB, 0.29036E+00_JPRB, 0.28756E+00_JPRB, 0.28708E+00_JPRB ,&
&  0.28787E+00_JPRB, 0.29007E+00_JPRB, 0.29121E+00_JPRB, 0.29039E+00_JPRB, 0.28918E+00_JPRB, 0.28929E+00_JPRB, 0.29163E+00_JPRB ,&
&  0.29594E+00_JPRB, 0.29941E+00_JPRB, 0.30078E+00_JPRB, 0.30085E+00_JPRB, 0.30038E+00_JPRB, 0.30162E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30515E+00_JPRB, 0.30578E+00_JPRB, 0.30685E+00_JPRB, 0.30665E+00_JPRB, 0.30853E+00_JPRB, 0.31180E+00_JPRB, 0.31549E+00_JPRB ,&
&  0.31907E+00_JPRB, 0.32354E+00_JPRB, 0.32795E+00_JPRB, 0.33122E+00_JPRB, 0.33392E+00_JPRB, 0.33368E+00_JPRB, 0.33352E+00_JPRB ,&
&  0.33309E+00_JPRB, 0.33131E+00_JPRB, 0.32954E+00_JPRB, 0.32714E+00_JPRB, 0.32422E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.29456E+00_JPRB, 0.29760E+00_JPRB, 0.30137E+00_JPRB, 0.30223E+00_JPRB, 0.30266E+00_JPRB, 0.30424E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.31004E+00_JPRB, 0.31110E+00_JPRB, 0.31158E+00_JPRB, 0.31191E+00_JPRB, 0.31280E+00_JPRB, 0.31486E+00_JPRB, 0.31593E+00_JPRB ,&
&  0.31553E+00_JPRB, 0.31498E+00_JPRB, 0.31413E+00_JPRB, 0.31338E+00_JPRB, 0.31171E+00_JPRB, 0.30924E+00_JPRB, 0.30708E+00_JPRB ,&
&  0.30413E+00_JPRB, 0.30138E+00_JPRB, 0.29926E+00_JPRB, 0.29754E+00_JPRB, 0.29648E+00_JPRB, 0.29623E+00_JPRB, 0.29755E+00_JPRB ,&
&  0.29930E+00_JPRB, 0.30164E+00_JPRB, 0.30456E+00_JPRB, 0.30797E+00_JPRB, 0.31137E+00_JPRB, 0.31527E+00_JPRB, 0.31873E+00_JPRB ,&
&  0.32149E+00_JPRB, 0.32390E+00_JPRB, 0.32556E+00_JPRB, 0.32291E+00_JPRB, 0.31973E+00_JPRB, 0.31948E+00_JPRB, 0.31972E+00_JPRB ,&
&  0.31745E+00_JPRB, 0.31369E+00_JPRB, 0.30872E+00_JPRB, 0.30599E+00_JPRB, 0.30530E+00_JPRB, 0.30430E+00_JPRB, 0.30302E+00_JPRB ,&
&  0.30215E+00_JPRB, 0.30142E+00_JPRB, 0.30008E+00_JPRB, 0.29781E+00_JPRB, 0.29423E+00_JPRB, 0.28943E+00_JPRB, 0.28470E+00_JPRB ,&
&  0.28173E+00_JPRB, 0.27964E+00_JPRB, 0.27959E+00_JPRB, 0.27748E+00_JPRB, 0.27328E+00_JPRB, 0.26932E+00_JPRB, 0.26894E+00_JPRB ,&
&  0.26955E+00_JPRB, 0.27068E+00_JPRB, 0.27091E+00_JPRB, 0.26975E+00_JPRB, 0.26980E+00_JPRB, 0.27036E+00_JPRB, 0.27173E+00_JPRB ,&
&  0.27599E+00_JPRB, 0.27914E+00_JPRB, 0.27921E+00_JPRB, 0.27825E+00_JPRB, 0.27637E+00_JPRB, 0.27602E+00_JPRB, 0.27693E+00_JPRB ,&
&  0.27795E+00_JPRB, 0.27846E+00_JPRB, 0.27880E+00_JPRB, 0.27824E+00_JPRB, 0.27829E+00_JPRB, 0.27973E+00_JPRB, 0.28158E+00_JPRB ,&
&  0.28386E+00_JPRB, 0.28667E+00_JPRB, 0.28912E+00_JPRB, 0.29154E+00_JPRB, 0.29372E+00_JPRB, 0.29462E+00_JPRB, 0.29546E+00_JPRB ,&
&  0.29639E+00_JPRB, 0.29641E+00_JPRB, 0.29595E+00_JPRB, 0.29562E+00_JPRB, 0.29520E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.27320E+00_JPRB, 0.27751E+00_JPRB, 0.28304E+00_JPRB, 0.28431E+00_JPRB, 0.28664E+00_JPRB, 0.28911E+00_JPRB, 0.29265E+00_JPRB ,&
&  0.29516E+00_JPRB, 0.29572E+00_JPRB, 0.29458E+00_JPRB, 0.29336E+00_JPRB, 0.29283E+00_JPRB, 0.29286E+00_JPRB, 0.29256E+00_JPRB ,&
&  0.29249E+00_JPRB, 0.29270E+00_JPRB, 0.29262E+00_JPRB, 0.29233E+00_JPRB, 0.29094E+00_JPRB, 0.28890E+00_JPRB, 0.28660E+00_JPRB ,&
&  0.28294E+00_JPRB, 0.28044E+00_JPRB, 0.27949E+00_JPRB, 0.27905E+00_JPRB, 0.27813E+00_JPRB, 0.27850E+00_JPRB, 0.28017E+00_JPRB ,&
&  0.28387E+00_JPRB, 0.28679E+00_JPRB, 0.29054E+00_JPRB, 0.29433E+00_JPRB, 0.29652E+00_JPRB, 0.29811E+00_JPRB, 0.30000E+00_JPRB ,&
&  0.30118E+00_JPRB, 0.30146E+00_JPRB, 0.30151E+00_JPRB, 0.30148E+00_JPRB, 0.29881E+00_JPRB, 0.29829E+00_JPRB, 0.29844E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29513E+00_JPRB, 0.29137E+00_JPRB, 0.28815E+00_JPRB, 0.28492E+00_JPRB, 0.28255E+00_JPRB, 0.28056E+00_JPRB ,&
&  0.27907E+00_JPRB, 0.27830E+00_JPRB, 0.27817E+00_JPRB, 0.27658E+00_JPRB, 0.27130E+00_JPRB, 0.26616E+00_JPRB, 0.26332E+00_JPRB ,&
&  0.26203E+00_JPRB, 0.26174E+00_JPRB, 0.26316E+00_JPRB, 0.26190E+00_JPRB, 0.25800E+00_JPRB, 0.25485E+00_JPRB, 0.25376E+00_JPRB ,&
&  0.25251E+00_JPRB, 0.25209E+00_JPRB, 0.25302E+00_JPRB, 0.25357E+00_JPRB, 0.25395E+00_JPRB, 0.25386E+00_JPRB, 0.25494E+00_JPRB ,&
&  0.25626E+00_JPRB, 0.25805E+00_JPRB, 0.25802E+00_JPRB, 0.25529E+00_JPRB, 0.25149E+00_JPRB, 0.24884E+00_JPRB, 0.24866E+00_JPRB ,&
&  0.25060E+00_JPRB, 0.25243E+00_JPRB, 0.25311E+00_JPRB, 0.25344E+00_JPRB, 0.25456E+00_JPRB, 0.25703E+00_JPRB, 0.26021E+00_JPRB ,&
&  0.26159E+00_JPRB, 0.26292E+00_JPRB, 0.26478E+00_JPRB, 0.26668E+00_JPRB, 0.26786E+00_JPRB, 0.26841E+00_JPRB, 0.26845E+00_JPRB ,&
&  0.26775E+00_JPRB, 0.26839E+00_JPRB, 0.26989E+00_JPRB, 0.27171E+00_JPRB, 0.27288E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.25869E+00_JPRB, 0.26185E+00_JPRB, 0.26497E+00_JPRB, 0.26668E+00_JPRB, 0.26846E+00_JPRB, 0.27038E+00_JPRB, 0.27381E+00_JPRB ,&
&  0.27792E+00_JPRB, 0.27944E+00_JPRB, 0.27794E+00_JPRB, 0.27527E+00_JPRB, 0.27361E+00_JPRB, 0.27300E+00_JPRB, 0.27295E+00_JPRB ,&
&  0.27289E+00_JPRB, 0.27279E+00_JPRB, 0.27196E+00_JPRB, 0.27000E+00_JPRB, 0.26723E+00_JPRB, 0.26441E+00_JPRB, 0.26221E+00_JPRB ,&
&  0.26167E+00_JPRB, 0.26290E+00_JPRB, 0.26501E+00_JPRB, 0.26510E+00_JPRB, 0.26520E+00_JPRB, 0.26646E+00_JPRB, 0.26856E+00_JPRB ,&
&  0.27248E+00_JPRB, 0.27431E+00_JPRB, 0.27609E+00_JPRB, 0.27209E+00_JPRB, 0.27628E+00_JPRB, 0.27777E+00_JPRB, 0.27812E+00_JPRB ,&
&  0.27925E+00_JPRB, 0.28019E+00_JPRB, 0.28124E+00_JPRB, 0.28061E+00_JPRB, 0.27641E+00_JPRB, 0.27485E+00_JPRB, 0.27597E+00_JPRB ,&
&  0.27694E+00_JPRB, 0.27541E+00_JPRB, 0.27250E+00_JPRB, 0.26975E+00_JPRB, 0.26740E+00_JPRB, 0.26569E+00_JPRB, 0.26427E+00_JPRB ,&
&  0.26360E+00_JPRB, 0.26281E+00_JPRB, 0.26080E+00_JPRB, 0.25552E+00_JPRB, 0.24954E+00_JPRB, 0.24559E+00_JPRB, 0.24279E+00_JPRB ,&
&  0.24275E+00_JPRB, 0.24469E+00_JPRB, 0.24647E+00_JPRB, 0.24666E+00_JPRB, 0.24534E+00_JPRB, 0.24284E+00_JPRB, 0.23935E+00_JPRB ,&
&  0.23452E+00_JPRB, 0.23268E+00_JPRB, 0.23233E+00_JPRB, 0.23151E+00_JPRB, 0.23189E+00_JPRB, 0.23366E+00_JPRB, 0.23549E+00_JPRB ,&
&  0.23776E+00_JPRB, 0.23770E+00_JPRB, 0.23622E+00_JPRB, 0.23373E+00_JPRB, 0.23117E+00_JPRB, 0.23102E+00_JPRB, 0.23183E+00_JPRB ,&
&  0.23268E+00_JPRB, 0.23348E+00_JPRB, 0.23461E+00_JPRB, 0.23603E+00_JPRB, 0.23781E+00_JPRB, 0.24023E+00_JPRB, 0.24217E+00_JPRB ,&
&  0.24208E+00_JPRB, 0.24090E+00_JPRB, 0.24198E+00_JPRB, 0.24384E+00_JPRB, 0.24480E+00_JPRB, 0.24534E+00_JPRB, 0.24569E+00_JPRB ,&
&  0.24527E+00_JPRB, 0.24524E+00_JPRB, 0.24714E+00_JPRB, 0.25171E+00_JPRB, 0.25589E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.24564E+00_JPRB, 0.24702E+00_JPRB, 0.24723E+00_JPRB, 0.24771E+00_JPRB, 0.24837E+00_JPRB, 0.25029E+00_JPRB, 0.25452E+00_JPRB ,&
&  0.25872E+00_JPRB, 0.26070E+00_JPRB, 0.25919E+00_JPRB, 0.25692E+00_JPRB, 0.25576E+00_JPRB, 0.25522E+00_JPRB, 0.25510E+00_JPRB ,&
&  0.25497E+00_JPRB, 0.25395E+00_JPRB, 0.25099E+00_JPRB, 0.24686E+00_JPRB, 0.24372E+00_JPRB, 0.24273E+00_JPRB, 0.24294E+00_JPRB ,&
&  0.24656E+00_JPRB, 0.24903E+00_JPRB, 0.25136E+00_JPRB, 0.25353E+00_JPRB, 0.25417E+00_JPRB, 0.25391E+00_JPRB, 0.25542E+00_JPRB ,&
&  0.25633E+00_JPRB, 0.25365E+00_JPRB, 0.24817E+00_JPRB, 0.25198E+00_JPRB, 0.25414E+00_JPRB, 0.25542E+00_JPRB, 0.25466E+00_JPRB ,&
&  0.25527E+00_JPRB, 0.25738E+00_JPRB, 0.25954E+00_JPRB, 0.26121E+00_JPRB, 0.26047E+00_JPRB, 0.25752E+00_JPRB, 0.25672E+00_JPRB ,&
&  0.25710E+00_JPRB, 0.25606E+00_JPRB, 0.25329E+00_JPRB, 0.25075E+00_JPRB, 0.24790E+00_JPRB, 0.24660E+00_JPRB, 0.24643E+00_JPRB ,&
&  0.24565E+00_JPRB, 0.24354E+00_JPRB, 0.23925E+00_JPRB, 0.23500E+00_JPRB, 0.23185E+00_JPRB, 0.22872E+00_JPRB, 0.22725E+00_JPRB ,&
&  0.22760E+00_JPRB, 0.22876E+00_JPRB, 0.22881E+00_JPRB, 0.22726E+00_JPRB, 0.22206E+00_JPRB, 0.21241E+00_JPRB, 0.20524E+00_JPRB ,&
&  0.20397E+00_JPRB, 0.20844E+00_JPRB, 0.21630E+00_JPRB, 0.21821E+00_JPRB, 0.22066E+00_JPRB, 0.22221E+00_JPRB, 0.22340E+00_JPRB ,&
&  0.22144E+00_JPRB, 0.20864E+00_JPRB, 0.20298E+00_JPRB, 0.19963E+00_JPRB, 0.19652E+00_JPRB, 0.19628E+00_JPRB, 0.19800E+00_JPRB ,&
&  0.20147E+00_JPRB, 0.20566E+00_JPRB, 0.20871E+00_JPRB, 0.21053E+00_JPRB, 0.21113E+00_JPRB, 0.21098E+00_JPRB, 0.21029E+00_JPRB ,&
&  0.21060E+00_JPRB, 0.21461E+00_JPRB, 0.22256E+00_JPRB, 0.23030E+00_JPRB, 0.23134E+00_JPRB, 0.23289E+00_JPRB, 0.23318E+00_JPRB ,&
&  0.23350E+00_JPRB, 0.23545E+00_JPRB, 0.23662E+00_JPRB, 0.23937E+00_JPRB, 0.24282E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.23450E+00_JPRB, 0.23576E+00_JPRB, 0.23599E+00_JPRB, 0.23509E+00_JPRB, 0.23382E+00_JPRB, 0.23397E+00_JPRB, 0.23535E+00_JPRB ,&
&  0.23827E+00_JPRB, 0.24037E+00_JPRB, 0.24083E+00_JPRB, 0.24081E+00_JPRB, 0.23987E+00_JPRB, 0.23784E+00_JPRB, 0.23616E+00_JPRB ,&
&  0.23488E+00_JPRB, 0.23416E+00_JPRB, 0.23358E+00_JPRB, 0.23340E+00_JPRB, 0.23329E+00_JPRB, 0.23410E+00_JPRB, 0.23487E+00_JPRB ,&
&  0.23837E+00_JPRB, 0.23829E+00_JPRB, 0.23757E+00_JPRB, 0.23732E+00_JPRB, 0.23711E+00_JPRB, 0.23710E+00_JPRB, 0.23693E+00_JPRB ,&
&  0.23395E+00_JPRB, 0.22625E+00_JPRB, 0.21966E+00_JPRB, 0.22544E+00_JPRB, 0.23684E+00_JPRB, 0.23715E+00_JPRB, 0.23732E+00_JPRB ,&
&  0.23871E+00_JPRB, 0.24155E+00_JPRB, 0.24365E+00_JPRB, 0.24464E+00_JPRB, 0.24343E+00_JPRB, 0.24167E+00_JPRB, 0.24111E+00_JPRB ,&
&  0.24135E+00_JPRB, 0.23964E+00_JPRB, 0.23438E+00_JPRB, 0.22827E+00_JPRB, 0.22217E+00_JPRB, 0.21982E+00_JPRB, 0.21829E+00_JPRB ,&
&  0.21581E+00_JPRB, 0.21436E+00_JPRB, 0.21385E+00_JPRB, 0.21340E+00_JPRB, 0.21384E+00_JPRB, 0.21167E+00_JPRB, 0.20843E+00_JPRB ,&
&  0.20518E+00_JPRB, 0.20104E+00_JPRB, 0.19618E+00_JPRB, 0.19105E+00_JPRB, 0.18669E+00_JPRB, 0.18186E+00_JPRB, 0.17853E+00_JPRB ,&
&  0.17802E+00_JPRB, 0.18222E+00_JPRB, 0.19213E+00_JPRB, 0.20214E+00_JPRB, 0.20843E+00_JPRB, 0.19838E+00_JPRB, 0.19187E+00_JPRB ,&
&  0.18447E+00_JPRB, 0.17826E+00_JPRB, 0.17471E+00_JPRB, 0.17394E+00_JPRB, 0.17411E+00_JPRB, 0.17497E+00_JPRB, 0.17670E+00_JPRB ,&
&  0.17897E+00_JPRB, 0.18181E+00_JPRB, 0.18396E+00_JPRB, 0.18556E+00_JPRB, 0.18671E+00_JPRB, 0.18702E+00_JPRB, 0.18679E+00_JPRB ,&
&  0.18654E+00_JPRB, 0.18784E+00_JPRB, 0.19027E+00_JPRB, 0.19462E+00_JPRB, 0.19752E+00_JPRB, 0.20133E+00_JPRB, 0.20645E+00_JPRB ,&
&  0.21403E+00_JPRB, 0.22285E+00_JPRB, 0.22646E+00_JPRB, 0.22892E+00_JPRB, 0.23200E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.22265E+00_JPRB, 0.22725E+00_JPRB, 0.22963E+00_JPRB, 0.22906E+00_JPRB, 0.22860E+00_JPRB, 0.22908E+00_JPRB, 0.22971E+00_JPRB ,&
&  0.23064E+00_JPRB, 0.23006E+00_JPRB, 0.22852E+00_JPRB, 0.22704E+00_JPRB, 0.22548E+00_JPRB, 0.22351E+00_JPRB, 0.22225E+00_JPRB ,&
&  0.22130E+00_JPRB, 0.22146E+00_JPRB, 0.22138E+00_JPRB, 0.22248E+00_JPRB, 0.22313E+00_JPRB, 0.22531E+00_JPRB, 0.22364E+00_JPRB ,&
&  0.21988E+00_JPRB, 0.21772E+00_JPRB, 0.21998E+00_JPRB, 0.22026E+00_JPRB, 0.22075E+00_JPRB, 0.22144E+00_JPRB, 0.22281E+00_JPRB ,&
&  0.22017E+00_JPRB, 0.21724E+00_JPRB, 0.21378E+00_JPRB, 0.21963E+00_JPRB, 0.22766E+00_JPRB, 0.22680E+00_JPRB, 0.22632E+00_JPRB ,&
&  0.22791E+00_JPRB, 0.23057E+00_JPRB, 0.23183E+00_JPRB, 0.23201E+00_JPRB, 0.23051E+00_JPRB, 0.22981E+00_JPRB, 0.22861E+00_JPRB ,&
&  0.22566E+00_JPRB, 0.22297E+00_JPRB, 0.21392E+00_JPRB, 0.19756E+00_JPRB, 0.19144E+00_JPRB, 0.18870E+00_JPRB, 0.18483E+00_JPRB ,&
&  0.17970E+00_JPRB, 0.17757E+00_JPRB, 0.17746E+00_JPRB, 0.17563E+00_JPRB, 0.17579E+00_JPRB, 0.17541E+00_JPRB, 0.17430E+00_JPRB ,&
&  0.17289E+00_JPRB, 0.17194E+00_JPRB, 0.17044E+00_JPRB, 0.16800E+00_JPRB, 0.16617E+00_JPRB, 0.16584E+00_JPRB, 0.16674E+00_JPRB ,&
&  0.16878E+00_JPRB, 0.17176E+00_JPRB, 0.18036E+00_JPRB, 0.18536E+00_JPRB, 0.18343E+00_JPRB, 0.17971E+00_JPRB, 0.17332E+00_JPRB ,&
&  0.16743E+00_JPRB, 0.16562E+00_JPRB, 0.16433E+00_JPRB, 0.16325E+00_JPRB, 0.16361E+00_JPRB, 0.16469E+00_JPRB, 0.16579E+00_JPRB ,&
&  0.16712E+00_JPRB, 0.16871E+00_JPRB, 0.16968E+00_JPRB, 0.17023E+00_JPRB, 0.17075E+00_JPRB, 0.17102E+00_JPRB, 0.17152E+00_JPRB ,&
&  0.17161E+00_JPRB, 0.17299E+00_JPRB, 0.17413E+00_JPRB, 0.17652E+00_JPRB, 0.17813E+00_JPRB, 0.18027E+00_JPRB, 0.18485E+00_JPRB ,&
&  0.18856E+00_JPRB, 0.19964E+00_JPRB, 0.21466E+00_JPRB, 0.22162E+00_JPRB, 0.21853E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.21435E+00_JPRB, 0.21672E+00_JPRB, 0.21875E+00_JPRB, 0.21732E+00_JPRB, 0.21729E+00_JPRB, 0.21750E+00_JPRB, 0.21859E+00_JPRB ,&
&  0.21907E+00_JPRB, 0.21852E+00_JPRB, 0.21812E+00_JPRB, 0.21647E+00_JPRB, 0.21492E+00_JPRB, 0.20820E+00_JPRB, 0.20542E+00_JPRB ,&
&  0.20321E+00_JPRB, 0.20385E+00_JPRB, 0.20436E+00_JPRB, 0.20635E+00_JPRB, 0.20639E+00_JPRB, 0.20533E+00_JPRB, 0.20211E+00_JPRB ,&
&  0.19968E+00_JPRB, 0.19831E+00_JPRB, 0.19677E+00_JPRB, 0.19712E+00_JPRB, 0.19926E+00_JPRB, 0.20149E+00_JPRB, 0.20291E+00_JPRB ,&
&  0.20369E+00_JPRB, 0.20586E+00_JPRB, 0.20779E+00_JPRB, 0.21064E+00_JPRB, 0.21720E+00_JPRB, 0.21787E+00_JPRB, 0.21879E+00_JPRB ,&
&  0.21973E+00_JPRB, 0.22087E+00_JPRB, 0.22148E+00_JPRB, 0.22106E+00_JPRB, 0.21926E+00_JPRB, 0.21399E+00_JPRB, 0.21256E+00_JPRB ,&
&  0.20029E+00_JPRB, 0.19306E+00_JPRB, 0.18633E+00_JPRB, 0.18143E+00_JPRB, 0.17587E+00_JPRB, 0.17290E+00_JPRB, 0.16955E+00_JPRB ,&
&  0.16670E+00_JPRB, 0.16360E+00_JPRB, 0.16131E+00_JPRB, 0.16033E+00_JPRB, 0.16002E+00_JPRB, 0.15975E+00_JPRB, 0.15908E+00_JPRB ,&
&  0.15920E+00_JPRB, 0.15876E+00_JPRB, 0.15845E+00_JPRB, 0.15813E+00_JPRB, 0.15792E+00_JPRB, 0.15806E+00_JPRB, 0.15887E+00_JPRB ,&
&  0.16092E+00_JPRB, 0.16278E+00_JPRB, 0.16472E+00_JPRB, 0.16661E+00_JPRB, 0.16619E+00_JPRB, 0.16441E+00_JPRB, 0.16174E+00_JPRB ,&
&  0.15941E+00_JPRB, 0.15790E+00_JPRB, 0.15628E+00_JPRB, 0.15586E+00_JPRB, 0.15650E+00_JPRB, 0.15729E+00_JPRB, 0.15866E+00_JPRB ,&
&  0.15988E+00_JPRB, 0.16087E+00_JPRB, 0.16154E+00_JPRB, 0.16198E+00_JPRB, 0.16268E+00_JPRB, 0.16333E+00_JPRB, 0.16444E+00_JPRB ,&
&  0.16567E+00_JPRB, 0.16719E+00_JPRB, 0.16857E+00_JPRB, 0.17047E+00_JPRB, 0.17334E+00_JPRB, 0.17762E+00_JPRB, 0.18498E+00_JPRB ,&
&  0.19203E+00_JPRB, 0.21047E+00_JPRB, 0.21275E+00_JPRB, 0.21274E+00_JPRB, 0.21433E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.20511E+00_JPRB, 0.20704E+00_JPRB, 0.20828E+00_JPRB, 0.20949E+00_JPRB, 0.20955E+00_JPRB, 0.20971E+00_JPRB, 0.21036E+00_JPRB ,&
&  0.20643E+00_JPRB, 0.20368E+00_JPRB, 0.19906E+00_JPRB, 0.19480E+00_JPRB, 0.19073E+00_JPRB, 0.18735E+00_JPRB, 0.18530E+00_JPRB ,&
&  0.18419E+00_JPRB, 0.18428E+00_JPRB, 0.18436E+00_JPRB, 0.18490E+00_JPRB, 0.18398E+00_JPRB, 0.18298E+00_JPRB, 0.18206E+00_JPRB ,&
&  0.18099E+00_JPRB, 0.18153E+00_JPRB, 0.18218E+00_JPRB, 0.18561E+00_JPRB, 0.19042E+00_JPRB, 0.19579E+00_JPRB, 0.19972E+00_JPRB ,&
&  0.20244E+00_JPRB, 0.20453E+00_JPRB, 0.20718E+00_JPRB, 0.20832E+00_JPRB, 0.20950E+00_JPRB, 0.21081E+00_JPRB, 0.21126E+00_JPRB ,&
&  0.21255E+00_JPRB, 0.21250E+00_JPRB, 0.21229E+00_JPRB, 0.21241E+00_JPRB, 0.20386E+00_JPRB, 0.19762E+00_JPRB, 0.19123E+00_JPRB ,&
&  0.18516E+00_JPRB, 0.17898E+00_JPRB, 0.17339E+00_JPRB, 0.17032E+00_JPRB, 0.16695E+00_JPRB, 0.16501E+00_JPRB, 0.16331E+00_JPRB ,&
&  0.16203E+00_JPRB, 0.16070E+00_JPRB, 0.15883E+00_JPRB, 0.15697E+00_JPRB, 0.15588E+00_JPRB, 0.15497E+00_JPRB, 0.15383E+00_JPRB ,&
&  0.15374E+00_JPRB, 0.15310E+00_JPRB, 0.15307E+00_JPRB, 0.15272E+00_JPRB, 0.15247E+00_JPRB, 0.15226E+00_JPRB, 0.15229E+00_JPRB ,&
&  0.15272E+00_JPRB, 0.15355E+00_JPRB, 0.15390E+00_JPRB, 0.15434E+00_JPRB, 0.15426E+00_JPRB, 0.15361E+00_JPRB, 0.15308E+00_JPRB ,&
&  0.15250E+00_JPRB, 0.15204E+00_JPRB, 0.15208E+00_JPRB, 0.15199E+00_JPRB, 0.15252E+00_JPRB, 0.15323E+00_JPRB, 0.15392E+00_JPRB ,&
&  0.15488E+00_JPRB, 0.15559E+00_JPRB, 0.15633E+00_JPRB, 0.15726E+00_JPRB, 0.15804E+00_JPRB, 0.15905E+00_JPRB, 0.16021E+00_JPRB ,&
&  0.16109E+00_JPRB, 0.16285E+00_JPRB, 0.16432E+00_JPRB, 0.16616E+00_JPRB, 0.16969E+00_JPRB, 0.17333E+00_JPRB, 0.18198E+00_JPRB ,&
&  0.18993E+00_JPRB, 0.20635E+00_JPRB, 0.20763E+00_JPRB, 0.20312E+00_JPRB, 0.20387E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.19948E+00_JPRB, 0.20088E+00_JPRB, 0.20148E+00_JPRB, 0.20242E+00_JPRB, 0.20276E+00_JPRB, 0.20419E+00_JPRB, 0.20425E+00_JPRB ,&
&  0.20434E+00_JPRB, 0.20054E+00_JPRB, 0.19823E+00_JPRB, 0.19554E+00_JPRB, 0.19319E+00_JPRB, 0.19041E+00_JPRB, 0.18772E+00_JPRB ,&
&  0.18562E+00_JPRB, 0.18375E+00_JPRB, 0.18175E+00_JPRB, 0.17900E+00_JPRB, 0.17623E+00_JPRB, 0.17427E+00_JPRB, 0.17291E+00_JPRB ,&
&  0.17160E+00_JPRB, 0.17249E+00_JPRB, 0.17421E+00_JPRB, 0.17757E+00_JPRB, 0.18205E+00_JPRB, 0.18574E+00_JPRB, 0.19064E+00_JPRB ,&
&  0.19663E+00_JPRB, 0.19844E+00_JPRB, 0.20092E+00_JPRB, 0.20202E+00_JPRB, 0.20169E+00_JPRB, 0.20226E+00_JPRB, 0.20008E+00_JPRB ,&
&  0.20022E+00_JPRB, 0.19742E+00_JPRB, 0.20188E+00_JPRB, 0.19868E+00_JPRB, 0.19187E+00_JPRB, 0.18884E+00_JPRB, 0.18269E+00_JPRB ,&
&  0.17832E+00_JPRB, 0.17355E+00_JPRB, 0.17039E+00_JPRB, 0.16859E+00_JPRB, 0.16544E+00_JPRB, 0.16374E+00_JPRB, 0.16112E+00_JPRB ,&
&  0.15870E+00_JPRB, 0.15792E+00_JPRB, 0.15643E+00_JPRB, 0.15583E+00_JPRB, 0.15502E+00_JPRB, 0.15389E+00_JPRB, 0.15266E+00_JPRB ,&
&  0.15185E+00_JPRB, 0.15115E+00_JPRB, 0.15080E+00_JPRB, 0.15029E+00_JPRB, 0.14957E+00_JPRB, 0.14884E+00_JPRB, 0.14800E+00_JPRB ,&
&  0.14745E+00_JPRB, 0.14719E+00_JPRB, 0.14700E+00_JPRB, 0.14699E+00_JPRB, 0.14696E+00_JPRB, 0.14677E+00_JPRB, 0.14692E+00_JPRB ,&
&  0.14721E+00_JPRB, 0.14784E+00_JPRB, 0.14837E+00_JPRB, 0.14908E+00_JPRB, 0.14971E+00_JPRB, 0.15054E+00_JPRB, 0.15114E+00_JPRB ,&
&  0.15182E+00_JPRB, 0.15278E+00_JPRB, 0.15362E+00_JPRB, 0.15461E+00_JPRB, 0.15568E+00_JPRB, 0.15658E+00_JPRB, 0.15802E+00_JPRB ,&
&  0.15949E+00_JPRB, 0.16095E+00_JPRB, 0.16300E+00_JPRB, 0.16508E+00_JPRB, 0.16775E+00_JPRB, 0.17222E+00_JPRB, 0.17730E+00_JPRB ,&
&  0.18367E+00_JPRB, 0.19306E+00_JPRB, 0.19340E+00_JPRB, 0.19646E+00_JPRB, 0.19814E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.19868E+00_JPRB, 0.19994E+00_JPRB, 0.20099E+00_JPRB, 0.20150E+00_JPRB, 0.20211E+00_JPRB, 0.20212E+00_JPRB, 0.20200E+00_JPRB ,&
&  0.19911E+00_JPRB, 0.19789E+00_JPRB, 0.19608E+00_JPRB, 0.19403E+00_JPRB, 0.19222E+00_JPRB, 0.18977E+00_JPRB, 0.18746E+00_JPRB ,&
&  0.18531E+00_JPRB, 0.18282E+00_JPRB, 0.18031E+00_JPRB, 0.17735E+00_JPRB, 0.17455E+00_JPRB, 0.17193E+00_JPRB, 0.17057E+00_JPRB ,&
&  0.16996E+00_JPRB, 0.17041E+00_JPRB, 0.17213E+00_JPRB, 0.17347E+00_JPRB, 0.17537E+00_JPRB, 0.17777E+00_JPRB, 0.18151E+00_JPRB ,&
&  0.18428E+00_JPRB, 0.18773E+00_JPRB, 0.18999E+00_JPRB, 0.19065E+00_JPRB, 0.18996E+00_JPRB, 0.18960E+00_JPRB, 0.18870E+00_JPRB ,&
&  0.18680E+00_JPRB, 0.18483E+00_JPRB, 0.18315E+00_JPRB, 0.18133E+00_JPRB, 0.17935E+00_JPRB, 0.17698E+00_JPRB, 0.17402E+00_JPRB ,&
&  0.17136E+00_JPRB, 0.16902E+00_JPRB, 0.16680E+00_JPRB, 0.16505E+00_JPRB, 0.16334E+00_JPRB, 0.16179E+00_JPRB, 0.16008E+00_JPRB ,&
&  0.15877E+00_JPRB, 0.15764E+00_JPRB, 0.15620E+00_JPRB, 0.15505E+00_JPRB, 0.15373E+00_JPRB, 0.15263E+00_JPRB, 0.15144E+00_JPRB ,&
&  0.15054E+00_JPRB, 0.14976E+00_JPRB, 0.14912E+00_JPRB, 0.14852E+00_JPRB, 0.14816E+00_JPRB, 0.14782E+00_JPRB, 0.14743E+00_JPRB ,&
&  0.14709E+00_JPRB, 0.14711E+00_JPRB, 0.14716E+00_JPRB, 0.14723E+00_JPRB, 0.14735E+00_JPRB, 0.14706E+00_JPRB, 0.14713E+00_JPRB ,&
&  0.14746E+00_JPRB, 0.14779E+00_JPRB, 0.14785E+00_JPRB, 0.14828E+00_JPRB, 0.14896E+00_JPRB, 0.14979E+00_JPRB, 0.15103E+00_JPRB ,&
&  0.15199E+00_JPRB, 0.15298E+00_JPRB, 0.15391E+00_JPRB, 0.15486E+00_JPRB, 0.15610E+00_JPRB, 0.15744E+00_JPRB, 0.15883E+00_JPRB ,&
&  0.16047E+00_JPRB, 0.16201E+00_JPRB, 0.16360E+00_JPRB, 0.16525E+00_JPRB, 0.16789E+00_JPRB, 0.17041E+00_JPRB, 0.17430E+00_JPRB ,&
&  0.17821E+00_JPRB, 0.17999E+00_JPRB, 0.19337E+00_JPRB, 0.19442E+00_JPRB, 0.19599E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.17734E+00_JPRB, 0.17959E+00_JPRB, 0.18109E+00_JPRB, 0.18269E+00_JPRB, 0.18052E+00_JPRB, 0.19215E+00_JPRB, 0.19008E+00_JPRB ,&
&  0.18973E+00_JPRB, 0.18476E+00_JPRB, 0.18520E+00_JPRB, 0.18449E+00_JPRB, 0.18307E+00_JPRB, 0.18146E+00_JPRB, 0.17966E+00_JPRB ,&
&  0.17790E+00_JPRB, 0.17568E+00_JPRB, 0.17386E+00_JPRB, 0.17236E+00_JPRB, 0.17097E+00_JPRB, 0.16978E+00_JPRB, 0.16858E+00_JPRB ,&
&  0.16846E+00_JPRB, 0.16916E+00_JPRB, 0.17026E+00_JPRB, 0.16956E+00_JPRB, 0.16994E+00_JPRB, 0.17080E+00_JPRB, 0.17198E+00_JPRB ,&
&  0.17295E+00_JPRB, 0.17432E+00_JPRB, 0.17521E+00_JPRB, 0.17547E+00_JPRB, 0.17589E+00_JPRB, 0.17519E+00_JPRB, 0.17469E+00_JPRB ,&
&  0.17398E+00_JPRB, 0.17299E+00_JPRB, 0.17173E+00_JPRB, 0.17065E+00_JPRB, 0.16945E+00_JPRB, 0.16792E+00_JPRB, 0.16635E+00_JPRB ,&
&  0.16493E+00_JPRB, 0.16368E+00_JPRB, 0.16233E+00_JPRB, 0.16122E+00_JPRB, 0.16015E+00_JPRB, 0.15908E+00_JPRB, 0.15810E+00_JPRB ,&
&  0.15732E+00_JPRB, 0.15661E+00_JPRB, 0.15586E+00_JPRB, 0.15508E+00_JPRB, 0.15422E+00_JPRB, 0.15339E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15224E+00_JPRB, 0.15192E+00_JPRB, 0.15160E+00_JPRB, 0.15125E+00_JPRB, 0.15116E+00_JPRB, 0.15105E+00_JPRB, 0.15091E+00_JPRB ,&
&  0.15075E+00_JPRB, 0.15050E+00_JPRB, 0.15042E+00_JPRB, 0.15033E+00_JPRB, 0.15026E+00_JPRB, 0.15020E+00_JPRB, 0.15031E+00_JPRB ,&
&  0.15049E+00_JPRB, 0.15073E+00_JPRB, 0.15123E+00_JPRB, 0.15177E+00_JPRB, 0.15233E+00_JPRB, 0.15288E+00_JPRB, 0.15367E+00_JPRB ,&
&  0.15453E+00_JPRB, 0.15534E+00_JPRB, 0.15605E+00_JPRB, 0.15664E+00_JPRB, 0.15759E+00_JPRB, 0.15849E+00_JPRB, 0.15937E+00_JPRB ,&
&  0.16033E+00_JPRB, 0.16156E+00_JPRB, 0.16284E+00_JPRB, 0.16415E+00_JPRB, 0.16569E+00_JPRB, 0.16697E+00_JPRB, 0.16826E+00_JPRB ,&
&  0.16979E+00_JPRB, 0.17261E+00_JPRB, 0.17448E+00_JPRB, 0.17570E+00_JPRB, 0.17699E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.16287E+00_JPRB, 0.16302E+00_JPRB, 0.16308E+00_JPRB, 0.16307E+00_JPRB, 0.16287E+00_JPRB, 0.16282E+00_JPRB, 0.16278E+00_JPRB ,&
&  0.16275E+00_JPRB, 0.16256E+00_JPRB, 0.16276E+00_JPRB, 0.16286E+00_JPRB, 0.16289E+00_JPRB, 0.16289E+00_JPRB, 0.16275E+00_JPRB ,&
&  0.16245E+00_JPRB, 0.16208E+00_JPRB, 0.16111E+00_JPRB, 0.16094E+00_JPRB, 0.16095E+00_JPRB, 0.16116E+00_JPRB, 0.16130E+00_JPRB ,&
&  0.16154E+00_JPRB, 0.16179E+00_JPRB, 0.16199E+00_JPRB, 0.16259E+00_JPRB, 0.16286E+00_JPRB, 0.16315E+00_JPRB, 0.16345E+00_JPRB ,&
&  0.16373E+00_JPRB, 0.16405E+00_JPRB, 0.16430E+00_JPRB, 0.16456E+00_JPRB, 0.16479E+00_JPRB, 0.16473E+00_JPRB, 0.16452E+00_JPRB ,&
&  0.16419E+00_JPRB, 0.16367E+00_JPRB, 0.16313E+00_JPRB, 0.16264E+00_JPRB, 0.16220E+00_JPRB, 0.16174E+00_JPRB, 0.16130E+00_JPRB ,&
&  0.16082E+00_JPRB, 0.16029E+00_JPRB, 0.15973E+00_JPRB, 0.15922E+00_JPRB, 0.15870E+00_JPRB, 0.15824E+00_JPRB, 0.15772E+00_JPRB ,&
&  0.15732E+00_JPRB, 0.15692E+00_JPRB, 0.15654E+00_JPRB, 0.15613E+00_JPRB, 0.15579E+00_JPRB, 0.15544E+00_JPRB, 0.15510E+00_JPRB ,&
&  0.15474E+00_JPRB, 0.15456E+00_JPRB, 0.15430E+00_JPRB, 0.15408E+00_JPRB, 0.15388E+00_JPRB, 0.15373E+00_JPRB, 0.15353E+00_JPRB ,&
&  0.15334E+00_JPRB, 0.15315E+00_JPRB, 0.15309E+00_JPRB, 0.15295E+00_JPRB, 0.15291E+00_JPRB, 0.15289E+00_JPRB, 0.15295E+00_JPRB ,&
&  0.15301E+00_JPRB, 0.15311E+00_JPRB, 0.15328E+00_JPRB, 0.15356E+00_JPRB, 0.15381E+00_JPRB, 0.15408E+00_JPRB, 0.15436E+00_JPRB ,&
&  0.15467E+00_JPRB, 0.15498E+00_JPRB, 0.15531E+00_JPRB, 0.15563E+00_JPRB, 0.15602E+00_JPRB, 0.15643E+00_JPRB, 0.15688E+00_JPRB ,&
&  0.15729E+00_JPRB, 0.15774E+00_JPRB, 0.15812E+00_JPRB, 0.15858E+00_JPRB, 0.15895E+00_JPRB, 0.15919E+00_JPRB, 0.15941E+00_JPRB ,&
&  0.15980E+00_JPRB, 0.16017E+00_JPRB, 0.16062E+00_JPRB, 0.16103E+00_JPRB, 0.16175E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB, 0.15636E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB, 0.44869E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.44301E+00_JPRB, 0.44308E+00_JPRB, 0.44313E+00_JPRB, 0.44317E+00_JPRB, 0.44320E+00_JPRB, 0.44325E+00_JPRB, 0.44330E+00_JPRB ,&
&  0.44330E+00_JPRB, 0.44344E+00_JPRB, 0.44365E+00_JPRB, 0.44391E+00_JPRB, 0.44414E+00_JPRB, 0.44437E+00_JPRB, 0.44458E+00_JPRB ,&
&  0.44476E+00_JPRB, 0.44486E+00_JPRB, 0.44502E+00_JPRB, 0.44535E+00_JPRB, 0.44581E+00_JPRB, 0.44636E+00_JPRB, 0.44712E+00_JPRB ,&
&  0.44743E+00_JPRB, 0.44738E+00_JPRB, 0.44737E+00_JPRB, 0.44778E+00_JPRB, 0.44772E+00_JPRB, 0.44773E+00_JPRB, 0.44778E+00_JPRB ,&
&  0.44782E+00_JPRB, 0.44779E+00_JPRB, 0.44765E+00_JPRB, 0.44749E+00_JPRB, 0.44745E+00_JPRB, 0.44757E+00_JPRB, 0.44772E+00_JPRB ,&
&  0.44795E+00_JPRB, 0.44814E+00_JPRB, 0.44863E+00_JPRB, 0.44917E+00_JPRB, 0.44980E+00_JPRB, 0.45023E+00_JPRB, 0.45069E+00_JPRB ,&
&  0.45098E+00_JPRB, 0.45130E+00_JPRB, 0.45182E+00_JPRB, 0.45254E+00_JPRB, 0.45333E+00_JPRB, 0.45407E+00_JPRB, 0.45465E+00_JPRB ,&
&  0.45553E+00_JPRB, 0.45667E+00_JPRB, 0.45787E+00_JPRB, 0.45918E+00_JPRB, 0.46025E+00_JPRB, 0.46134E+00_JPRB, 0.46250E+00_JPRB ,&
&  0.46351E+00_JPRB, 0.46412E+00_JPRB, 0.46485E+00_JPRB, 0.46562E+00_JPRB, 0.46619E+00_JPRB, 0.46588E+00_JPRB, 0.46566E+00_JPRB ,&
&  0.46547E+00_JPRB, 0.46506E+00_JPRB, 0.46377E+00_JPRB, 0.46269E+00_JPRB, 0.46167E+00_JPRB, 0.46059E+00_JPRB, 0.45940E+00_JPRB ,&
&  0.45833E+00_JPRB, 0.45738E+00_JPRB, 0.45634E+00_JPRB, 0.45524E+00_JPRB, 0.45424E+00_JPRB, 0.45329E+00_JPRB, 0.45235E+00_JPRB ,&
&  0.45149E+00_JPRB, 0.45076E+00_JPRB, 0.45005E+00_JPRB, 0.44939E+00_JPRB, 0.44865E+00_JPRB, 0.44794E+00_JPRB, 0.44723E+00_JPRB ,&
&  0.44659E+00_JPRB, 0.44593E+00_JPRB, 0.44533E+00_JPRB, 0.44474E+00_JPRB, 0.44431E+00_JPRB, 0.44400E+00_JPRB, 0.44372E+00_JPRB ,&
&  0.44348E+00_JPRB, 0.44332E+00_JPRB, 0.44324E+00_JPRB, 0.44313E+00_JPRB, 0.44302E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.44046E+00_JPRB, 0.44079E+00_JPRB, 0.44108E+00_JPRB, 0.44132E+00_JPRB, 0.44165E+00_JPRB, 0.44204E+00_JPRB, 0.44237E+00_JPRB ,&
&  0.44261E+00_JPRB, 0.44306E+00_JPRB, 0.44358E+00_JPRB, 0.44411E+00_JPRB, 0.44460E+00_JPRB, 0.44516E+00_JPRB, 0.44567E+00_JPRB ,&
&  0.44622E+00_JPRB, 0.44675E+00_JPRB, 0.44731E+00_JPRB, 0.44782E+00_JPRB, 0.44837E+00_JPRB, 0.44877E+00_JPRB, 0.45022E+00_JPRB ,&
&  0.45150E+00_JPRB, 0.45214E+00_JPRB, 0.45180E+00_JPRB, 0.44832E+00_JPRB, 0.44579E+00_JPRB, 0.44354E+00_JPRB, 0.44170E+00_JPRB ,&
&  0.43981E+00_JPRB, 0.43992E+00_JPRB, 0.44034E+00_JPRB, 0.44078E+00_JPRB, 0.44264E+00_JPRB, 0.44236E+00_JPRB, 0.44192E+00_JPRB ,&
&  0.44075E+00_JPRB, 0.43767E+00_JPRB, 0.43804E+00_JPRB, 0.43854E+00_JPRB, 0.43909E+00_JPRB, 0.44068E+00_JPRB, 0.44183E+00_JPRB ,&
&  0.44324E+00_JPRB, 0.44482E+00_JPRB, 0.44782E+00_JPRB, 0.45120E+00_JPRB, 0.45428E+00_JPRB, 0.45685E+00_JPRB, 0.45933E+00_JPRB ,&
&  0.46149E+00_JPRB, 0.46368E+00_JPRB, 0.46604E+00_JPRB, 0.46877E+00_JPRB, 0.47088E+00_JPRB, 0.47321E+00_JPRB, 0.47517E+00_JPRB ,&
&  0.47637E+00_JPRB, 0.47694E+00_JPRB, 0.47848E+00_JPRB, 0.48071E+00_JPRB, 0.48299E+00_JPRB, 0.48341E+00_JPRB, 0.48301E+00_JPRB ,&
&  0.48189E+00_JPRB, 0.48063E+00_JPRB, 0.47839E+00_JPRB, 0.47631E+00_JPRB, 0.47406E+00_JPRB, 0.47177E+00_JPRB, 0.46904E+00_JPRB ,&
&  0.46627E+00_JPRB, 0.46361E+00_JPRB, 0.46155E+00_JPRB, 0.45972E+00_JPRB, 0.45784E+00_JPRB, 0.45609E+00_JPRB, 0.45490E+00_JPRB ,&
&  0.45371E+00_JPRB, 0.45267E+00_JPRB, 0.45157E+00_JPRB, 0.45049E+00_JPRB, 0.44879E+00_JPRB, 0.44712E+00_JPRB, 0.44536E+00_JPRB ,&
&  0.44392E+00_JPRB, 0.44252E+00_JPRB, 0.44136E+00_JPRB, 0.44039E+00_JPRB, 0.43994E+00_JPRB, 0.43970E+00_JPRB, 0.43957E+00_JPRB ,&
&  0.43948E+00_JPRB, 0.43964E+00_JPRB, 0.43986E+00_JPRB, 0.44003E+00_JPRB, 0.44021E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.44376E+00_JPRB, 0.44440E+00_JPRB, 0.44508E+00_JPRB, 0.44548E+00_JPRB, 0.44627E+00_JPRB, 0.44697E+00_JPRB, 0.44764E+00_JPRB ,&
&  0.44827E+00_JPRB, 0.44867E+00_JPRB, 0.44898E+00_JPRB, 0.44932E+00_JPRB, 0.44956E+00_JPRB, 0.45026E+00_JPRB, 0.45092E+00_JPRB ,&
&  0.45152E+00_JPRB, 0.45188E+00_JPRB, 0.45301E+00_JPRB, 0.45411E+00_JPRB, 0.45478E+00_JPRB, 0.45542E+00_JPRB, 0.45266E+00_JPRB ,&
&  0.45096E+00_JPRB, 0.44997E+00_JPRB, 0.44865E+00_JPRB, 0.44538E+00_JPRB, 0.44380E+00_JPRB, 0.43278E+00_JPRB, 0.43603E+00_JPRB ,&
&  0.44911E+00_JPRB, 0.44827E+00_JPRB, 0.44555E+00_JPRB, 0.44084E+00_JPRB, 0.42415E+00_JPRB, 0.41671E+00_JPRB, 0.41544E+00_JPRB ,&
&  0.40444E+00_JPRB, 0.41380E+00_JPRB, 0.41635E+00_JPRB, 0.41741E+00_JPRB, 0.41980E+00_JPRB, 0.41822E+00_JPRB, 0.41864E+00_JPRB ,&
&  0.42598E+00_JPRB, 0.42629E+00_JPRB, 0.43008E+00_JPRB, 0.43473E+00_JPRB, 0.43979E+00_JPRB, 0.44454E+00_JPRB, 0.45001E+00_JPRB ,&
&  0.45448E+00_JPRB, 0.45775E+00_JPRB, 0.45995E+00_JPRB, 0.46077E+00_JPRB, 0.46186E+00_JPRB, 0.46311E+00_JPRB, 0.46574E+00_JPRB ,&
&  0.47010E+00_JPRB, 0.47479E+00_JPRB, 0.47736E+00_JPRB, 0.47684E+00_JPRB, 0.47769E+00_JPRB, 0.47831E+00_JPRB, 0.47888E+00_JPRB ,&
&  0.47878E+00_JPRB, 0.47813E+00_JPRB, 0.47587E+00_JPRB, 0.47528E+00_JPRB, 0.47497E+00_JPRB, 0.47383E+00_JPRB, 0.47205E+00_JPRB ,&
&  0.46950E+00_JPRB, 0.46594E+00_JPRB, 0.46295E+00_JPRB, 0.46112E+00_JPRB, 0.45925E+00_JPRB, 0.45764E+00_JPRB, 0.45691E+00_JPRB ,&
&  0.45559E+00_JPRB, 0.45452E+00_JPRB, 0.45267E+00_JPRB, 0.45015E+00_JPRB, 0.44756E+00_JPRB, 0.44546E+00_JPRB, 0.44329E+00_JPRB ,&
&  0.44208E+00_JPRB, 0.44141E+00_JPRB, 0.44105E+00_JPRB, 0.44044E+00_JPRB, 0.44039E+00_JPRB, 0.44055E+00_JPRB, 0.44094E+00_JPRB ,&
&  0.44137E+00_JPRB, 0.44192E+00_JPRB, 0.44229E+00_JPRB, 0.44278E+00_JPRB, 0.44301E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.44511E+00_JPRB, 0.44528E+00_JPRB, 0.44568E+00_JPRB, 0.44601E+00_JPRB, 0.44676E+00_JPRB, 0.44739E+00_JPRB, 0.44889E+00_JPRB ,&
&  0.45034E+00_JPRB, 0.45172E+00_JPRB, 0.45304E+00_JPRB, 0.45413E+00_JPRB, 0.45522E+00_JPRB, 0.45564E+00_JPRB, 0.45580E+00_JPRB ,&
&  0.45585E+00_JPRB, 0.45571E+00_JPRB, 0.45627E+00_JPRB, 0.45710E+00_JPRB, 0.45734E+00_JPRB, 0.45803E+00_JPRB, 0.45500E+00_JPRB ,&
&  0.45278E+00_JPRB, 0.45147E+00_JPRB, 0.44028E+00_JPRB, 0.44567E+00_JPRB, 0.44447E+00_JPRB, 0.44506E+00_JPRB, 0.44327E+00_JPRB ,&
&  0.43152E+00_JPRB, 0.42650E+00_JPRB, 0.41357E+00_JPRB, 0.40251E+00_JPRB, 0.40286E+00_JPRB, 0.39471E+00_JPRB, 0.39511E+00_JPRB ,&
&  0.39322E+00_JPRB, 0.39008E+00_JPRB, 0.38875E+00_JPRB, 0.38798E+00_JPRB, 0.39012E+00_JPRB, 0.39193E+00_JPRB, 0.39401E+00_JPRB ,&
&  0.39892E+00_JPRB, 0.40322E+00_JPRB, 0.40963E+00_JPRB, 0.41054E+00_JPRB, 0.41774E+00_JPRB, 0.42574E+00_JPRB, 0.43402E+00_JPRB ,&
&  0.43910E+00_JPRB, 0.43798E+00_JPRB, 0.44064E+00_JPRB, 0.44512E+00_JPRB, 0.44883E+00_JPRB, 0.45261E+00_JPRB, 0.45810E+00_JPRB ,&
&  0.46503E+00_JPRB, 0.47161E+00_JPRB, 0.47399E+00_JPRB, 0.47290E+00_JPRB, 0.47074E+00_JPRB, 0.46885E+00_JPRB, 0.46856E+00_JPRB ,&
&  0.46823E+00_JPRB, 0.46975E+00_JPRB, 0.47003E+00_JPRB, 0.47159E+00_JPRB, 0.47271E+00_JPRB, 0.47536E+00_JPRB, 0.47725E+00_JPRB ,&
&  0.47806E+00_JPRB, 0.47762E+00_JPRB, 0.47823E+00_JPRB, 0.47442E+00_JPRB, 0.47327E+00_JPRB, 0.47194E+00_JPRB, 0.46778E+00_JPRB ,&
&  0.46393E+00_JPRB, 0.46075E+00_JPRB, 0.45752E+00_JPRB, 0.45461E+00_JPRB, 0.45229E+00_JPRB, 0.45062E+00_JPRB, 0.44958E+00_JPRB ,&
&  0.44912E+00_JPRB, 0.44843E+00_JPRB, 0.44762E+00_JPRB, 0.44575E+00_JPRB, 0.44535E+00_JPRB, 0.44401E+00_JPRB, 0.44330E+00_JPRB ,&
&  0.44286E+00_JPRB, 0.44337E+00_JPRB, 0.44339E+00_JPRB, 0.44408E+00_JPRB, 0.44485E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.44257E+00_JPRB, 0.44471E+00_JPRB, 0.44833E+00_JPRB, 0.45148E+00_JPRB, 0.45432E+00_JPRB, 0.45666E+00_JPRB, 0.45789E+00_JPRB ,&
&  0.45879E+00_JPRB, 0.45908E+00_JPRB, 0.45942E+00_JPRB, 0.45992E+00_JPRB, 0.46080E+00_JPRB, 0.46149E+00_JPRB, 0.46241E+00_JPRB ,&
&  0.46269E+00_JPRB, 0.46274E+00_JPRB, 0.46251E+00_JPRB, 0.46228E+00_JPRB, 0.46223E+00_JPRB, 0.46295E+00_JPRB, 0.46216E+00_JPRB ,&
&  0.46241E+00_JPRB, 0.45962E+00_JPRB, 0.45133E+00_JPRB, 0.45378E+00_JPRB, 0.45441E+00_JPRB, 0.45526E+00_JPRB, 0.45076E+00_JPRB ,&
&  0.43917E+00_JPRB, 0.41505E+00_JPRB, 0.42661E+00_JPRB, 0.41747E+00_JPRB, 0.40797E+00_JPRB, 0.39836E+00_JPRB, 0.38500E+00_JPRB ,&
&  0.37690E+00_JPRB, 0.36662E+00_JPRB, 0.35903E+00_JPRB, 0.35649E+00_JPRB, 0.35639E+00_JPRB, 0.36065E+00_JPRB, 0.36648E+00_JPRB ,&
&  0.36957E+00_JPRB, 0.37825E+00_JPRB, 0.38305E+00_JPRB, 0.38912E+00_JPRB, 0.39644E+00_JPRB, 0.40582E+00_JPRB, 0.40955E+00_JPRB ,&
&  0.41623E+00_JPRB, 0.41939E+00_JPRB, 0.42643E+00_JPRB, 0.43529E+00_JPRB, 0.44751E+00_JPRB, 0.45506E+00_JPRB, 0.45980E+00_JPRB ,&
&  0.46489E+00_JPRB, 0.46719E+00_JPRB, 0.46826E+00_JPRB, 0.46869E+00_JPRB, 0.47009E+00_JPRB, 0.47177E+00_JPRB, 0.47389E+00_JPRB ,&
&  0.47607E+00_JPRB, 0.47943E+00_JPRB, 0.48309E+00_JPRB, 0.48727E+00_JPRB, 0.48996E+00_JPRB, 0.49516E+00_JPRB, 0.50111E+00_JPRB ,&
&  0.50608E+00_JPRB, 0.50844E+00_JPRB, 0.51325E+00_JPRB, 0.50793E+00_JPRB, 0.50571E+00_JPRB, 0.49639E+00_JPRB, 0.48957E+00_JPRB ,&
&  0.48457E+00_JPRB, 0.47823E+00_JPRB, 0.47297E+00_JPRB, 0.46858E+00_JPRB, 0.46535E+00_JPRB, 0.46224E+00_JPRB, 0.45968E+00_JPRB ,&
&  0.45698E+00_JPRB, 0.45364E+00_JPRB, 0.45134E+00_JPRB, 0.44841E+00_JPRB, 0.44659E+00_JPRB, 0.44445E+00_JPRB, 0.44337E+00_JPRB ,&
&  0.44259E+00_JPRB, 0.44237E+00_JPRB, 0.44197E+00_JPRB, 0.44246E+00_JPRB, 0.44269E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.44643E+00_JPRB, 0.45016E+00_JPRB, 0.45472E+00_JPRB, 0.45861E+00_JPRB, 0.46051E+00_JPRB, 0.46209E+00_JPRB, 0.46121E+00_JPRB ,&
&  0.46064E+00_JPRB, 0.45954E+00_JPRB, 0.45872E+00_JPRB, 0.45862E+00_JPRB, 0.45932E+00_JPRB, 0.46051E+00_JPRB, 0.46209E+00_JPRB ,&
&  0.46350E+00_JPRB, 0.46528E+00_JPRB, 0.46591E+00_JPRB, 0.46676E+00_JPRB, 0.46775E+00_JPRB, 0.47026E+00_JPRB, 0.47116E+00_JPRB ,&
&  0.47241E+00_JPRB, 0.46874E+00_JPRB, 0.46029E+00_JPRB, 0.45742E+00_JPRB, 0.46114E+00_JPRB, 0.45354E+00_JPRB, 0.45501E+00_JPRB ,&
&  0.46029E+00_JPRB, 0.46120E+00_JPRB, 0.45338E+00_JPRB, 0.44737E+00_JPRB, 0.43252E+00_JPRB, 0.38373E+00_JPRB, 0.38169E+00_JPRB ,&
&  0.37244E+00_JPRB, 0.35710E+00_JPRB, 0.34919E+00_JPRB, 0.34242E+00_JPRB, 0.33953E+00_JPRB, 0.34550E+00_JPRB, 0.35499E+00_JPRB ,&
&  0.35800E+00_JPRB, 0.37047E+00_JPRB, 0.37516E+00_JPRB, 0.38231E+00_JPRB, 0.39006E+00_JPRB, 0.40076E+00_JPRB, 0.40697E+00_JPRB ,&
&  0.41382E+00_JPRB, 0.42091E+00_JPRB, 0.43099E+00_JPRB, 0.44404E+00_JPRB, 0.45475E+00_JPRB, 0.46002E+00_JPRB, 0.45880E+00_JPRB ,&
&  0.46355E+00_JPRB, 0.45929E+00_JPRB, 0.46169E+00_JPRB, 0.46493E+00_JPRB, 0.46683E+00_JPRB, 0.47065E+00_JPRB, 0.47205E+00_JPRB ,&
&  0.47807E+00_JPRB, 0.48807E+00_JPRB, 0.49934E+00_JPRB, 0.50554E+00_JPRB, 0.51087E+00_JPRB, 0.51693E+00_JPRB, 0.52664E+00_JPRB ,&
&  0.53587E+00_JPRB, 0.54715E+00_JPRB, 0.55669E+00_JPRB, 0.54508E+00_JPRB, 0.54028E+00_JPRB, 0.52792E+00_JPRB, 0.51891E+00_JPRB ,&
&  0.50814E+00_JPRB, 0.50094E+00_JPRB, 0.48888E+00_JPRB, 0.48223E+00_JPRB, 0.47561E+00_JPRB, 0.47001E+00_JPRB, 0.46506E+00_JPRB ,&
&  0.46094E+00_JPRB, 0.45711E+00_JPRB, 0.45446E+00_JPRB, 0.45271E+00_JPRB, 0.45069E+00_JPRB, 0.44827E+00_JPRB, 0.44667E+00_JPRB ,&
&  0.44386E+00_JPRB, 0.44217E+00_JPRB, 0.44096E+00_JPRB, 0.44135E+00_JPRB, 0.44275E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.44407E+00_JPRB, 0.44917E+00_JPRB, 0.45368E+00_JPRB, 0.45967E+00_JPRB, 0.46046E+00_JPRB, 0.46249E+00_JPRB, 0.45882E+00_JPRB ,&
&  0.45625E+00_JPRB, 0.45492E+00_JPRB, 0.45432E+00_JPRB, 0.45548E+00_JPRB, 0.45720E+00_JPRB, 0.45923E+00_JPRB, 0.46216E+00_JPRB ,&
&  0.46491E+00_JPRB, 0.46333E+00_JPRB, 0.46478E+00_JPRB, 0.46885E+00_JPRB, 0.47044E+00_JPRB, 0.47344E+00_JPRB, 0.47517E+00_JPRB ,&
&  0.47521E+00_JPRB, 0.47275E+00_JPRB, 0.47024E+00_JPRB, 0.46405E+00_JPRB, 0.46169E+00_JPRB, 0.46387E+00_JPRB, 0.46654E+00_JPRB ,&
&  0.45705E+00_JPRB, 0.45251E+00_JPRB, 0.46333E+00_JPRB, 0.46560E+00_JPRB, 0.43963E+00_JPRB, 0.41534E+00_JPRB, 0.39868E+00_JPRB ,&
&  0.38094E+00_JPRB, 0.35727E+00_JPRB, 0.34869E+00_JPRB, 0.34298E+00_JPRB, 0.34494E+00_JPRB, 0.34828E+00_JPRB, 0.35639E+00_JPRB ,&
&  0.36266E+00_JPRB, 0.37075E+00_JPRB, 0.37999E+00_JPRB, 0.39173E+00_JPRB, 0.40000E+00_JPRB, 0.40895E+00_JPRB, 0.41651E+00_JPRB ,&
&  0.42464E+00_JPRB, 0.43462E+00_JPRB, 0.44745E+00_JPRB, 0.46336E+00_JPRB, 0.47425E+00_JPRB, 0.48183E+00_JPRB, 0.48391E+00_JPRB ,&
&  0.50037E+00_JPRB, 0.48887E+00_JPRB, 0.49105E+00_JPRB, 0.49216E+00_JPRB, 0.49241E+00_JPRB, 0.49483E+00_JPRB, 0.49631E+00_JPRB ,&
&  0.50553E+00_JPRB, 0.52028E+00_JPRB, 0.52505E+00_JPRB, 0.52816E+00_JPRB, 0.53092E+00_JPRB, 0.53507E+00_JPRB, 0.54737E+00_JPRB ,&
&  0.55923E+00_JPRB, 0.58279E+00_JPRB, 0.64182E+00_JPRB, 0.61312E+00_JPRB, 0.59310E+00_JPRB, 0.57310E+00_JPRB, 0.55304E+00_JPRB ,&
&  0.53481E+00_JPRB, 0.51846E+00_JPRB, 0.50231E+00_JPRB, 0.48780E+00_JPRB, 0.47749E+00_JPRB, 0.46798E+00_JPRB, 0.46588E+00_JPRB ,&
&  0.46370E+00_JPRB, 0.46067E+00_JPRB, 0.45811E+00_JPRB, 0.45586E+00_JPRB, 0.45173E+00_JPRB, 0.44824E+00_JPRB, 0.44612E+00_JPRB ,&
&  0.44224E+00_JPRB, 0.44109E+00_JPRB, 0.43957E+00_JPRB, 0.44012E+00_JPRB, 0.44093E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.44239E+00_JPRB, 0.45298E+00_JPRB, 0.46123E+00_JPRB, 0.46687E+00_JPRB, 0.46585E+00_JPRB, 0.46655E+00_JPRB, 0.46087E+00_JPRB ,&
&  0.45793E+00_JPRB, 0.45845E+00_JPRB, 0.45938E+00_JPRB, 0.46011E+00_JPRB, 0.46075E+00_JPRB, 0.46111E+00_JPRB, 0.46208E+00_JPRB ,&
&  0.46390E+00_JPRB, 0.46625E+00_JPRB, 0.46767E+00_JPRB, 0.46966E+00_JPRB, 0.47243E+00_JPRB, 0.47603E+00_JPRB, 0.47944E+00_JPRB ,&
&  0.48037E+00_JPRB, 0.47813E+00_JPRB, 0.47882E+00_JPRB, 0.47525E+00_JPRB, 0.47143E+00_JPRB, 0.46724E+00_JPRB, 0.46492E+00_JPRB ,&
&  0.46956E+00_JPRB, 0.47615E+00_JPRB, 0.47090E+00_JPRB, 0.46659E+00_JPRB, 0.46396E+00_JPRB, 0.44202E+00_JPRB, 0.41637E+00_JPRB ,&
&  0.39066E+00_JPRB, 0.36861E+00_JPRB, 0.35823E+00_JPRB, 0.35340E+00_JPRB, 0.35762E+00_JPRB, 0.36746E+00_JPRB, 0.37770E+00_JPRB ,&
&  0.38065E+00_JPRB, 0.39454E+00_JPRB, 0.40868E+00_JPRB, 0.41769E+00_JPRB, 0.42511E+00_JPRB, 0.43233E+00_JPRB, 0.43933E+00_JPRB ,&
&  0.45049E+00_JPRB, 0.46622E+00_JPRB, 0.49252E+00_JPRB, 0.51986E+00_JPRB, 0.54091E+00_JPRB, 0.54757E+00_JPRB, 0.57803E+00_JPRB ,&
&  0.65861E+00_JPRB, 0.60544E+00_JPRB, 0.57651E+00_JPRB, 0.56494E+00_JPRB, 0.55361E+00_JPRB, 0.54947E+00_JPRB, 0.55302E+00_JPRB ,&
&  0.56184E+00_JPRB, 0.57045E+00_JPRB, 0.57160E+00_JPRB, 0.56964E+00_JPRB, 0.57087E+00_JPRB, 0.57601E+00_JPRB, 0.58879E+00_JPRB ,&
&  0.60416E+00_JPRB, 0.64797E+00_JPRB, 0.75778E+00_JPRB, 0.66362E+00_JPRB, 0.61244E+00_JPRB, 0.58771E+00_JPRB, 0.56521E+00_JPRB ,&
&  0.54629E+00_JPRB, 0.53002E+00_JPRB, 0.51334E+00_JPRB, 0.49314E+00_JPRB, 0.47724E+00_JPRB, 0.46379E+00_JPRB, 0.45877E+00_JPRB ,&
&  0.45470E+00_JPRB, 0.45075E+00_JPRB, 0.45011E+00_JPRB, 0.44786E+00_JPRB, 0.44480E+00_JPRB, 0.44535E+00_JPRB, 0.44509E+00_JPRB ,&
&  0.44105E+00_JPRB, 0.43794E+00_JPRB, 0.43574E+00_JPRB, 0.43522E+00_JPRB, 0.43633E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.45345E+00_JPRB, 0.46168E+00_JPRB, 0.47293E+00_JPRB, 0.49132E+00_JPRB, 0.50012E+00_JPRB, 0.49420E+00_JPRB, 0.47866E+00_JPRB ,&
&  0.46944E+00_JPRB, 0.46542E+00_JPRB, 0.46857E+00_JPRB, 0.47266E+00_JPRB, 0.47448E+00_JPRB, 0.47424E+00_JPRB, 0.47252E+00_JPRB ,&
&  0.46897E+00_JPRB, 0.46741E+00_JPRB, 0.46786E+00_JPRB, 0.46980E+00_JPRB, 0.47272E+00_JPRB, 0.47590E+00_JPRB, 0.48016E+00_JPRB ,&
&  0.48352E+00_JPRB, 0.48341E+00_JPRB, 0.48247E+00_JPRB, 0.47974E+00_JPRB, 0.47940E+00_JPRB, 0.48170E+00_JPRB, 0.48074E+00_JPRB ,&
&  0.47489E+00_JPRB, 0.47838E+00_JPRB, 0.48327E+00_JPRB, 0.47585E+00_JPRB, 0.47256E+00_JPRB, 0.45689E+00_JPRB, 0.43183E+00_JPRB ,&
&  0.40963E+00_JPRB, 0.38870E+00_JPRB, 0.38254E+00_JPRB, 0.38489E+00_JPRB, 0.39289E+00_JPRB, 0.40033E+00_JPRB, 0.41365E+00_JPRB ,&
&  0.42525E+00_JPRB, 0.43479E+00_JPRB, 0.44204E+00_JPRB, 0.44618E+00_JPRB, 0.45113E+00_JPRB, 0.45961E+00_JPRB, 0.47656E+00_JPRB ,&
&  0.50750E+00_JPRB, 0.54703E+00_JPRB, 0.58843E+00_JPRB, 0.56916E+00_JPRB, 0.57217E+00_JPRB, 0.59180E+00_JPRB, 0.62073E+00_JPRB ,&
&  0.64526E+00_JPRB, 0.63928E+00_JPRB, 0.63231E+00_JPRB, 0.65403E+00_JPRB, 0.66509E+00_JPRB, 0.67862E+00_JPRB, 0.67485E+00_JPRB ,&
&  0.66317E+00_JPRB, 0.64868E+00_JPRB, 0.63926E+00_JPRB, 0.63940E+00_JPRB, 0.65124E+00_JPRB, 0.66211E+00_JPRB, 0.66233E+00_JPRB ,&
&  0.64668E+00_JPRB, 0.61981E+00_JPRB, 0.58897E+00_JPRB, 0.55702E+00_JPRB, 0.54998E+00_JPRB, 0.55261E+00_JPRB, 0.55321E+00_JPRB ,&
&  0.54284E+00_JPRB, 0.52890E+00_JPRB, 0.51445E+00_JPRB, 0.49509E+00_JPRB, 0.47609E+00_JPRB, 0.45720E+00_JPRB, 0.44680E+00_JPRB ,&
&  0.44044E+00_JPRB, 0.43728E+00_JPRB, 0.43959E+00_JPRB, 0.43386E+00_JPRB, 0.43540E+00_JPRB, 0.43920E+00_JPRB, 0.43568E+00_JPRB ,&
&  0.43259E+00_JPRB, 0.43232E+00_JPRB, 0.42914E+00_JPRB, 0.43179E+00_JPRB, 0.43824E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.47828E+00_JPRB, 0.50231E+00_JPRB, 0.52408E+00_JPRB, 0.53941E+00_JPRB, 0.54314E+00_JPRB, 0.54226E+00_JPRB, 0.52838E+00_JPRB ,&
&  0.52765E+00_JPRB, 0.50452E+00_JPRB, 0.48966E+00_JPRB, 0.47308E+00_JPRB, 0.46030E+00_JPRB, 0.45227E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45766E+00_JPRB, 0.46258E+00_JPRB, 0.46562E+00_JPRB, 0.46651E+00_JPRB, 0.46898E+00_JPRB, 0.47301E+00_JPRB, 0.47865E+00_JPRB ,&
&  0.48473E+00_JPRB, 0.48819E+00_JPRB, 0.49061E+00_JPRB, 0.49179E+00_JPRB, 0.49230E+00_JPRB, 0.49190E+00_JPRB, 0.48760E+00_JPRB ,&
&  0.48755E+00_JPRB, 0.48551E+00_JPRB, 0.48618E+00_JPRB, 0.48223E+00_JPRB, 0.47976E+00_JPRB, 0.46812E+00_JPRB, 0.44092E+00_JPRB ,&
&  0.42491E+00_JPRB, 0.41700E+00_JPRB, 0.42086E+00_JPRB, 0.42981E+00_JPRB, 0.43850E+00_JPRB, 0.44876E+00_JPRB, 0.45428E+00_JPRB ,&
&  0.45288E+00_JPRB, 0.45357E+00_JPRB, 0.46285E+00_JPRB, 0.47144E+00_JPRB, 0.47904E+00_JPRB, 0.49126E+00_JPRB, 0.50529E+00_JPRB ,&
&  0.53157E+00_JPRB, 0.57265E+00_JPRB, 0.63936E+00_JPRB, 0.65647E+00_JPRB, 0.71513E+00_JPRB, 0.84954E+00_JPRB, 0.76282E+00_JPRB ,&
&  0.77096E+00_JPRB, 0.78020E+00_JPRB, 0.80430E+00_JPRB, 0.85011E+00_JPRB, 0.82800E+00_JPRB, 0.81492E+00_JPRB, 0.80531E+00_JPRB ,&
&  0.77160E+00_JPRB, 0.75818E+00_JPRB, 0.74680E+00_JPRB, 0.76271E+00_JPRB, 0.75871E+00_JPRB, 0.75364E+00_JPRB, 0.74466E+00_JPRB ,&
&  0.71139E+00_JPRB, 0.66943E+00_JPRB, 0.62715E+00_JPRB, 0.58716E+00_JPRB, 0.56709E+00_JPRB, 0.55546E+00_JPRB, 0.54782E+00_JPRB ,&
&  0.53746E+00_JPRB, 0.52911E+00_JPRB, 0.52134E+00_JPRB, 0.50898E+00_JPRB, 0.48823E+00_JPRB, 0.46416E+00_JPRB, 0.45366E+00_JPRB ,&
&  0.44481E+00_JPRB, 0.43863E+00_JPRB, 0.43510E+00_JPRB, 0.43245E+00_JPRB, 0.43589E+00_JPRB, 0.43741E+00_JPRB, 0.43608E+00_JPRB ,&
&  0.43815E+00_JPRB, 0.43488E+00_JPRB, 0.43162E+00_JPRB, 0.43628E+00_JPRB, 0.45549E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.51884E+00_JPRB, 0.55065E+00_JPRB, 0.56468E+00_JPRB, 0.57354E+00_JPRB, 0.58145E+00_JPRB, 0.58002E+00_JPRB, 0.55259E+00_JPRB ,&
&  0.54944E+00_JPRB, 0.49994E+00_JPRB, 0.48177E+00_JPRB, 0.46665E+00_JPRB, 0.45961E+00_JPRB, 0.45210E+00_JPRB, 0.44316E+00_JPRB ,&
&  0.44494E+00_JPRB, 0.45399E+00_JPRB, 0.46414E+00_JPRB, 0.46859E+00_JPRB, 0.47232E+00_JPRB, 0.47515E+00_JPRB, 0.47828E+00_JPRB ,&
&  0.48361E+00_JPRB, 0.49022E+00_JPRB, 0.49700E+00_JPRB, 0.49930E+00_JPRB, 0.49913E+00_JPRB, 0.49842E+00_JPRB, 0.49669E+00_JPRB ,&
&  0.49533E+00_JPRB, 0.48842E+00_JPRB, 0.48838E+00_JPRB, 0.48725E+00_JPRB, 0.48239E+00_JPRB, 0.48082E+00_JPRB, 0.46630E+00_JPRB ,&
&  0.44368E+00_JPRB, 0.45292E+00_JPRB, 0.46188E+00_JPRB, 0.47224E+00_JPRB, 0.48079E+00_JPRB, 0.48743E+00_JPRB, 0.49084E+00_JPRB ,&
&  0.49128E+00_JPRB, 0.49080E+00_JPRB, 0.49311E+00_JPRB, 0.49754E+00_JPRB, 0.50308E+00_JPRB, 0.51057E+00_JPRB, 0.51742E+00_JPRB ,&
&  0.52703E+00_JPRB, 0.51547E+00_JPRB, 0.56124E+00_JPRB, 0.68740E+00_JPRB, 0.95988E+00_JPRB, 0.93901E+00_JPRB, 0.95367E+00_JPRB ,&
&  0.10108E+01_JPRB, 0.98677E+00_JPRB, 0.99015E+00_JPRB, 0.99450E+00_JPRB, 0.98284E+00_JPRB, 0.96479E+00_JPRB, 0.96526E+00_JPRB ,&
&  0.93324E+00_JPRB, 0.94599E+00_JPRB, 0.92724E+00_JPRB, 0.91715E+00_JPRB, 0.89012E+00_JPRB, 0.87693E+00_JPRB, 0.85372E+00_JPRB ,&
&  0.81708E+00_JPRB, 0.78406E+00_JPRB, 0.72977E+00_JPRB, 0.66735E+00_JPRB, 0.61803E+00_JPRB, 0.59194E+00_JPRB, 0.56903E+00_JPRB ,&
&  0.55247E+00_JPRB, 0.54318E+00_JPRB, 0.53242E+00_JPRB, 0.52586E+00_JPRB, 0.51276E+00_JPRB, 0.49793E+00_JPRB, 0.49241E+00_JPRB ,&
&  0.47374E+00_JPRB, 0.46328E+00_JPRB, 0.45383E+00_JPRB, 0.44961E+00_JPRB, 0.45591E+00_JPRB, 0.46350E+00_JPRB, 0.46861E+00_JPRB ,&
&  0.47667E+00_JPRB, 0.47244E+00_JPRB, 0.47166E+00_JPRB, 0.47815E+00_JPRB, 0.49246E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.55426E+00_JPRB, 0.57779E+00_JPRB, 0.58872E+00_JPRB, 0.60067E+00_JPRB, 0.61501E+00_JPRB, 0.61327E+00_JPRB, 0.61081E+00_JPRB ,&
&  0.63484E+00_JPRB, 0.60000E+00_JPRB, 0.63641E+00_JPRB, 0.52521E+00_JPRB, 0.48215E+00_JPRB, 0.47224E+00_JPRB, 0.45982E+00_JPRB ,&
&  0.45794E+00_JPRB, 0.46313E+00_JPRB, 0.46827E+00_JPRB, 0.47296E+00_JPRB, 0.47878E+00_JPRB, 0.47900E+00_JPRB, 0.47952E+00_JPRB ,&
&  0.48266E+00_JPRB, 0.48997E+00_JPRB, 0.50197E+00_JPRB, 0.50598E+00_JPRB, 0.50931E+00_JPRB, 0.51333E+00_JPRB, 0.50977E+00_JPRB ,&
&  0.50134E+00_JPRB, 0.50088E+00_JPRB, 0.50078E+00_JPRB, 0.49385E+00_JPRB, 0.48666E+00_JPRB, 0.48669E+00_JPRB, 0.49767E+00_JPRB ,&
&  0.49848E+00_JPRB, 0.48825E+00_JPRB, 0.50212E+00_JPRB, 0.50909E+00_JPRB, 0.51305E+00_JPRB, 0.51455E+00_JPRB, 0.51481E+00_JPRB ,&
&  0.51605E+00_JPRB, 0.51614E+00_JPRB, 0.51976E+00_JPRB, 0.52184E+00_JPRB, 0.52454E+00_JPRB, 0.52948E+00_JPRB, 0.53043E+00_JPRB ,&
&  0.54462E+00_JPRB, 0.53601E+00_JPRB, 0.64761E+00_JPRB, 0.77165E+00_JPRB, 0.10245E+01_JPRB, 0.11550E+01_JPRB, 0.11509E+01_JPRB ,&
&  0.12883E+01_JPRB, 0.12140E+01_JPRB, 0.12601E+01_JPRB, 0.12313E+01_JPRB, 0.11972E+01_JPRB, 0.11513E+01_JPRB, 0.10988E+01_JPRB ,&
&  0.10886E+01_JPRB, 0.10886E+01_JPRB, 0.11281E+01_JPRB, 0.10961E+01_JPRB, 0.10318E+01_JPRB, 0.10083E+01_JPRB, 0.99235E+00_JPRB ,&
&  0.98402E+00_JPRB, 0.96842E+00_JPRB, 0.92005E+00_JPRB, 0.78989E+00_JPRB, 0.72543E+00_JPRB, 0.67378E+00_JPRB, 0.61105E+00_JPRB ,&
&  0.58062E+00_JPRB, 0.55046E+00_JPRB, 0.52803E+00_JPRB, 0.52396E+00_JPRB, 0.51392E+00_JPRB, 0.50511E+00_JPRB, 0.50297E+00_JPRB ,&
&  0.48787E+00_JPRB, 0.48207E+00_JPRB, 0.48089E+00_JPRB, 0.48459E+00_JPRB, 0.48918E+00_JPRB, 0.49582E+00_JPRB, 0.50556E+00_JPRB ,&
&  0.50328E+00_JPRB, 0.49985E+00_JPRB, 0.50964E+00_JPRB, 0.51572E+00_JPRB, 0.53123E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.57328E+00_JPRB, 0.59421E+00_JPRB, 0.61153E+00_JPRB, 0.62783E+00_JPRB, 0.64978E+00_JPRB, 0.67307E+00_JPRB, 0.67216E+00_JPRB ,&
&  0.68499E+00_JPRB, 0.68886E+00_JPRB, 0.68588E+00_JPRB, 0.67292E+00_JPRB, 0.64675E+00_JPRB, 0.57298E+00_JPRB, 0.48229E+00_JPRB ,&
&  0.45612E+00_JPRB, 0.46043E+00_JPRB, 0.47023E+00_JPRB, 0.47434E+00_JPRB, 0.47656E+00_JPRB, 0.47957E+00_JPRB, 0.47984E+00_JPRB ,&
&  0.48370E+00_JPRB, 0.48980E+00_JPRB, 0.50070E+00_JPRB, 0.51427E+00_JPRB, 0.52633E+00_JPRB, 0.53137E+00_JPRB, 0.52876E+00_JPRB ,&
&  0.51734E+00_JPRB, 0.51239E+00_JPRB, 0.50413E+00_JPRB, 0.49307E+00_JPRB, 0.49781E+00_JPRB, 0.50557E+00_JPRB, 0.52255E+00_JPRB ,&
&  0.53768E+00_JPRB, 0.54868E+00_JPRB, 0.55343E+00_JPRB, 0.55873E+00_JPRB, 0.56156E+00_JPRB, 0.55867E+00_JPRB, 0.55238E+00_JPRB ,&
&  0.54691E+00_JPRB, 0.54307E+00_JPRB, 0.54079E+00_JPRB, 0.54290E+00_JPRB, 0.54293E+00_JPRB, 0.55357E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.59408E+00_JPRB, 0.68505E+00_JPRB, 0.82348E+00_JPRB, 0.85937E+00_JPRB, 0.11073E+01_JPRB, 0.12233E+01_JPRB, 0.12916E+01_JPRB ,&
&  0.13307E+01_JPRB, 0.12654E+01_JPRB, 0.14051E+01_JPRB, 0.14362E+01_JPRB, 0.14414E+01_JPRB, 0.13915E+01_JPRB, 0.12889E+01_JPRB ,&
&  0.12380E+01_JPRB, 0.13135E+01_JPRB, 0.13420E+01_JPRB, 0.13215E+01_JPRB, 0.12593E+01_JPRB, 0.12437E+01_JPRB, 0.12340E+01_JPRB ,&
&  0.12550E+01_JPRB, 0.12681E+01_JPRB, 0.12014E+01_JPRB, 0.10696E+01_JPRB, 0.92096E+00_JPRB, 0.79633E+00_JPRB, 0.69887E+00_JPRB ,&
&  0.62308E+00_JPRB, 0.55187E+00_JPRB, 0.52590E+00_JPRB, 0.51894E+00_JPRB, 0.51881E+00_JPRB, 0.50684E+00_JPRB, 0.50358E+00_JPRB ,&
&  0.50359E+00_JPRB, 0.50446E+00_JPRB, 0.51044E+00_JPRB, 0.51380E+00_JPRB, 0.52575E+00_JPRB, 0.53312E+00_JPRB, 0.51869E+00_JPRB ,&
&  0.50833E+00_JPRB, 0.51497E+00_JPRB, 0.52197E+00_JPRB, 0.53237E+00_JPRB, 0.55133E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.59910E+00_JPRB, 0.62271E+00_JPRB, 0.64056E+00_JPRB, 0.65603E+00_JPRB, 0.68151E+00_JPRB, 0.71906E+00_JPRB, 0.73498E+00_JPRB ,&
&  0.73459E+00_JPRB, 0.73159E+00_JPRB, 0.72714E+00_JPRB, 0.71637E+00_JPRB, 0.69447E+00_JPRB, 0.65902E+00_JPRB, 0.57761E+00_JPRB ,&
&  0.49062E+00_JPRB, 0.48325E+00_JPRB, 0.47124E+00_JPRB, 0.47611E+00_JPRB, 0.48132E+00_JPRB, 0.48004E+00_JPRB, 0.47708E+00_JPRB ,&
&  0.48217E+00_JPRB, 0.48828E+00_JPRB, 0.49996E+00_JPRB, 0.51430E+00_JPRB, 0.51882E+00_JPRB, 0.52434E+00_JPRB, 0.53180E+00_JPRB ,&
&  0.52145E+00_JPRB, 0.50821E+00_JPRB, 0.49880E+00_JPRB, 0.49506E+00_JPRB, 0.50946E+00_JPRB, 0.53285E+00_JPRB, 0.55438E+00_JPRB ,&
&  0.57017E+00_JPRB, 0.58070E+00_JPRB, 0.58932E+00_JPRB, 0.59629E+00_JPRB, 0.60016E+00_JPRB, 0.59721E+00_JPRB, 0.58738E+00_JPRB ,&
&  0.57508E+00_JPRB, 0.56736E+00_JPRB, 0.56312E+00_JPRB, 0.56727E+00_JPRB, 0.57329E+00_JPRB, 0.60641E+00_JPRB, 0.64902E+00_JPRB ,&
&  0.71283E+00_JPRB, 0.82112E+00_JPRB, 0.90460E+00_JPRB, 0.10396E+01_JPRB, 0.12404E+01_JPRB, 0.13788E+01_JPRB, 0.14440E+01_JPRB ,&
&  0.15068E+01_JPRB, 0.14673E+01_JPRB, 0.16875E+01_JPRB, 0.16943E+01_JPRB, 0.16066E+01_JPRB, 0.15773E+01_JPRB, 0.14598E+01_JPRB ,&
&  0.13688E+01_JPRB, 0.13055E+01_JPRB, 0.13829E+01_JPRB, 0.14339E+01_JPRB, 0.14654E+01_JPRB, 0.15115E+01_JPRB, 0.15157E+01_JPRB ,&
&  0.15289E+01_JPRB, 0.15167E+01_JPRB, 0.13028E+01_JPRB, 0.10853E+01_JPRB, 0.87318E+00_JPRB, 0.83626E+00_JPRB, 0.74161E+00_JPRB ,&
&  0.64263E+00_JPRB, 0.55081E+00_JPRB, 0.53947E+00_JPRB, 0.53201E+00_JPRB, 0.54117E+00_JPRB, 0.53821E+00_JPRB, 0.52938E+00_JPRB ,&
&  0.53508E+00_JPRB, 0.54704E+00_JPRB, 0.54510E+00_JPRB, 0.55064E+00_JPRB, 0.56472E+00_JPRB, 0.56727E+00_JPRB, 0.54548E+00_JPRB ,&
&  0.53823E+00_JPRB, 0.54088E+00_JPRB, 0.53548E+00_JPRB, 0.54568E+00_JPRB, 0.56884E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.62958E+00_JPRB, 0.64998E+00_JPRB, 0.66608E+00_JPRB, 0.68885E+00_JPRB, 0.72401E+00_JPRB, 0.76302E+00_JPRB, 0.77551E+00_JPRB ,&
&  0.77213E+00_JPRB, 0.76573E+00_JPRB, 0.75617E+00_JPRB, 0.74001E+00_JPRB, 0.72173E+00_JPRB, 0.68610E+00_JPRB, 0.64660E+00_JPRB ,&
&  0.53533E+00_JPRB, 0.50322E+00_JPRB, 0.46382E+00_JPRB, 0.46487E+00_JPRB, 0.50371E+00_JPRB, 0.49578E+00_JPRB, 0.49482E+00_JPRB ,&
&  0.49060E+00_JPRB, 0.49724E+00_JPRB, 0.50326E+00_JPRB, 0.51218E+00_JPRB, 0.50887E+00_JPRB, 0.51737E+00_JPRB, 0.52935E+00_JPRB ,&
&  0.52203E+00_JPRB, 0.51252E+00_JPRB, 0.51228E+00_JPRB, 0.51864E+00_JPRB, 0.53811E+00_JPRB, 0.56653E+00_JPRB, 0.60151E+00_JPRB ,&
&  0.61991E+00_JPRB, 0.63640E+00_JPRB, 0.64578E+00_JPRB, 0.65224E+00_JPRB, 0.65405E+00_JPRB, 0.64920E+00_JPRB, 0.63848E+00_JPRB ,&
&  0.62358E+00_JPRB, 0.61123E+00_JPRB, 0.61259E+00_JPRB, 0.62438E+00_JPRB, 0.62474E+00_JPRB, 0.67574E+00_JPRB, 0.78188E+00_JPRB ,&
&  0.91862E+00_JPRB, 0.97818E+00_JPRB, 0.10699E+01_JPRB, 0.13483E+01_JPRB, 0.15651E+01_JPRB, 0.16866E+01_JPRB, 0.17011E+01_JPRB ,&
&  0.16997E+01_JPRB, 0.16790E+01_JPRB, 0.16780E+01_JPRB, 0.17080E+01_JPRB, 0.16479E+01_JPRB, 0.16660E+01_JPRB, 0.15784E+01_JPRB ,&
&  0.14412E+01_JPRB, 0.14181E+01_JPRB, 0.14985E+01_JPRB, 0.15633E+01_JPRB, 0.15921E+01_JPRB, 0.16491E+01_JPRB, 0.16949E+01_JPRB ,&
&  0.16684E+01_JPRB, 0.14715E+01_JPRB, 0.11007E+01_JPRB, 0.97695E+00_JPRB, 0.74082E+00_JPRB, 0.71838E+00_JPRB, 0.78573E+00_JPRB ,&
&  0.75965E+00_JPRB, 0.66922E+00_JPRB, 0.60564E+00_JPRB, 0.57702E+00_JPRB, 0.57658E+00_JPRB, 0.62528E+00_JPRB, 0.61187E+00_JPRB ,&
&  0.58080E+00_JPRB, 0.59163E+00_JPRB, 0.60420E+00_JPRB, 0.61748E+00_JPRB, 0.63394E+00_JPRB, 0.63676E+00_JPRB, 0.61359E+00_JPRB ,&
&  0.61505E+00_JPRB, 0.59893E+00_JPRB, 0.58565E+00_JPRB, 0.58815E+00_JPRB, 0.60500E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.68043E+00_JPRB, 0.69067E+00_JPRB, 0.70694E+00_JPRB, 0.73208E+00_JPRB, 0.77273E+00_JPRB, 0.80411E+00_JPRB, 0.80637E+00_JPRB ,&
&  0.79830E+00_JPRB, 0.78978E+00_JPRB, 0.77550E+00_JPRB, 0.75855E+00_JPRB, 0.72997E+00_JPRB, 0.68517E+00_JPRB, 0.64355E+00_JPRB ,&
&  0.63193E+00_JPRB, 0.56074E+00_JPRB, 0.50604E+00_JPRB, 0.46977E+00_JPRB, 0.49351E+00_JPRB, 0.49514E+00_JPRB, 0.50766E+00_JPRB ,&
&  0.50591E+00_JPRB, 0.52125E+00_JPRB, 0.52328E+00_JPRB, 0.51980E+00_JPRB, 0.51498E+00_JPRB, 0.53277E+00_JPRB, 0.55474E+00_JPRB ,&
&  0.56827E+00_JPRB, 0.57292E+00_JPRB, 0.57732E+00_JPRB, 0.59772E+00_JPRB, 0.63692E+00_JPRB, 0.62004E+00_JPRB, 0.65793E+00_JPRB ,&
&  0.67838E+00_JPRB, 0.69316E+00_JPRB, 0.70423E+00_JPRB, 0.71857E+00_JPRB, 0.72352E+00_JPRB, 0.72920E+00_JPRB, 0.73545E+00_JPRB ,&
&  0.74327E+00_JPRB, 0.74250E+00_JPRB, 0.71529E+00_JPRB, 0.70859E+00_JPRB, 0.71988E+00_JPRB, 0.82229E+00_JPRB, 0.88270E+00_JPRB ,&
&  0.95285E+00_JPRB, 0.95558E+00_JPRB, 0.10041E+01_JPRB, 0.13144E+01_JPRB, 0.15736E+01_JPRB, 0.16684E+01_JPRB, 0.17479E+01_JPRB ,&
&  0.17841E+01_JPRB, 0.17968E+01_JPRB, 0.18564E+01_JPRB, 0.18785E+01_JPRB, 0.18811E+01_JPRB, 0.18605E+01_JPRB, 0.18158E+01_JPRB ,&
&  0.17054E+01_JPRB, 0.16701E+01_JPRB, 0.17285E+01_JPRB, 0.16998E+01_JPRB, 0.16796E+01_JPRB, 0.16089E+01_JPRB, 0.17238E+01_JPRB ,&
&  0.17216E+01_JPRB, 0.10441E+01_JPRB, 0.83403E+00_JPRB, 0.93697E+00_JPRB, 0.77973E+00_JPRB, 0.74671E+00_JPRB, 0.78333E+00_JPRB ,&
&  0.74002E+00_JPRB, 0.69451E+00_JPRB, 0.67426E+00_JPRB, 0.64022E+00_JPRB, 0.64452E+00_JPRB, 0.75244E+00_JPRB, 0.74332E+00_JPRB ,&
&  0.71059E+00_JPRB, 0.70061E+00_JPRB, 0.72875E+00_JPRB, 0.73179E+00_JPRB, 0.73127E+00_JPRB, 0.73974E+00_JPRB, 0.72627E+00_JPRB ,&
&  0.69513E+00_JPRB, 0.67166E+00_JPRB, 0.65645E+00_JPRB, 0.65673E+00_JPRB, 0.66886E+00_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.76487E+00_JPRB, 0.76558E+00_JPRB, 0.77132E+00_JPRB, 0.79141E+00_JPRB, 0.82635E+00_JPRB, 0.84805E+00_JPRB, 0.83953E+00_JPRB ,&
&  0.82892E+00_JPRB, 0.81227E+00_JPRB, 0.79535E+00_JPRB, 0.77076E+00_JPRB, 0.73131E+00_JPRB, 0.68554E+00_JPRB, 0.64329E+00_JPRB ,&
&  0.61581E+00_JPRB, 0.58651E+00_JPRB, 0.51650E+00_JPRB, 0.48775E+00_JPRB, 0.46645E+00_JPRB, 0.48128E+00_JPRB, 0.50021E+00_JPRB ,&
&  0.52036E+00_JPRB, 0.52993E+00_JPRB, 0.53455E+00_JPRB, 0.53980E+00_JPRB, 0.54760E+00_JPRB, 0.57977E+00_JPRB, 0.62757E+00_JPRB ,&
&  0.65624E+00_JPRB, 0.67139E+00_JPRB, 0.65705E+00_JPRB, 0.70484E+00_JPRB, 0.71472E+00_JPRB, 0.69280E+00_JPRB, 0.74585E+00_JPRB ,&
&  0.77487E+00_JPRB, 0.79585E+00_JPRB, 0.80043E+00_JPRB, 0.81715E+00_JPRB, 0.82669E+00_JPRB, 0.80799E+00_JPRB, 0.78421E+00_JPRB ,&
&  0.75414E+00_JPRB, 0.73742E+00_JPRB, 0.73507E+00_JPRB, 0.72951E+00_JPRB, 0.74073E+00_JPRB, 0.80247E+00_JPRB, 0.80935E+00_JPRB ,&
&  0.80236E+00_JPRB, 0.68740E+00_JPRB, 0.65138E+00_JPRB, 0.92628E+00_JPRB, 0.13633E+01_JPRB, 0.16159E+01_JPRB, 0.16056E+01_JPRB ,&
&  0.17644E+01_JPRB, 0.18770E+01_JPRB, 0.20346E+01_JPRB, 0.21261E+01_JPRB, 0.21337E+01_JPRB, 0.20648E+01_JPRB, 0.19920E+01_JPRB ,&
&  0.19250E+01_JPRB, 0.18992E+01_JPRB, 0.18879E+01_JPRB, 0.18738E+01_JPRB, 0.18396E+01_JPRB, 0.18780E+01_JPRB, 0.19901E+01_JPRB ,&
&  0.18514E+01_JPRB, 0.15811E+01_JPRB, 0.10631E+01_JPRB, 0.10430E+01_JPRB, 0.80909E+00_JPRB, 0.76817E+00_JPRB, 0.81160E+00_JPRB ,&
&  0.76542E+00_JPRB, 0.76631E+00_JPRB, 0.74591E+00_JPRB, 0.71289E+00_JPRB, 0.77091E+00_JPRB, 0.89334E+00_JPRB, 0.93174E+00_JPRB ,&
&  0.95640E+00_JPRB, 0.88832E+00_JPRB, 0.94245E+00_JPRB, 0.94339E+00_JPRB, 0.93384E+00_JPRB, 0.91949E+00_JPRB, 0.89193E+00_JPRB ,&
&  0.84520E+00_JPRB, 0.79882E+00_JPRB, 0.76985E+00_JPRB, 0.75979E+00_JPRB, 0.75933E+00_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.82659E+00_JPRB, 0.81119E+00_JPRB, 0.80570E+00_JPRB, 0.81869E+00_JPRB, 0.84486E+00_JPRB, 0.85713E+00_JPRB, 0.84286E+00_JPRB ,&
&  0.83127E+00_JPRB, 0.81541E+00_JPRB, 0.80357E+00_JPRB, 0.76587E+00_JPRB, 0.72878E+00_JPRB, 0.68111E+00_JPRB, 0.63685E+00_JPRB ,&
&  0.60832E+00_JPRB, 0.56077E+00_JPRB, 0.49880E+00_JPRB, 0.47260E+00_JPRB, 0.45277E+00_JPRB, 0.45742E+00_JPRB, 0.48149E+00_JPRB ,&
&  0.51170E+00_JPRB, 0.53420E+00_JPRB, 0.57014E+00_JPRB, 0.58744E+00_JPRB, 0.61774E+00_JPRB, 0.67463E+00_JPRB, 0.74034E+00_JPRB ,&
&  0.83300E+00_JPRB, 0.74099E+00_JPRB, 0.76952E+00_JPRB, 0.77030E+00_JPRB, 0.79777E+00_JPRB, 0.80609E+00_JPRB, 0.80746E+00_JPRB ,&
&  0.82021E+00_JPRB, 0.83069E+00_JPRB, 0.85010E+00_JPRB, 0.84798E+00_JPRB, 0.83169E+00_JPRB, 0.81193E+00_JPRB, 0.79702E+00_JPRB ,&
&  0.79349E+00_JPRB, 0.78277E+00_JPRB, 0.75740E+00_JPRB, 0.72858E+00_JPRB, 0.79350E+00_JPRB, 0.84187E+00_JPRB, 0.79523E+00_JPRB ,&
&  0.66178E+00_JPRB, 0.67670E+00_JPRB, 0.88882E+00_JPRB, 0.11726E+01_JPRB, 0.14740E+01_JPRB, 0.17582E+01_JPRB, 0.19294E+01_JPRB ,&
&  0.20884E+01_JPRB, 0.21866E+01_JPRB, 0.22785E+01_JPRB, 0.20440E+01_JPRB, 0.22092E+01_JPRB, 0.21568E+01_JPRB, 0.20637E+01_JPRB ,&
&  0.19687E+01_JPRB, 0.19650E+01_JPRB, 0.19091E+01_JPRB, 0.19720E+01_JPRB, 0.21099E+01_JPRB, 0.23115E+01_JPRB, 0.20543E+01_JPRB ,&
&  0.17249E+01_JPRB, 0.18165E+01_JPRB, 0.15490E+01_JPRB, 0.12214E+01_JPRB, 0.96124E+00_JPRB, 0.92392E+00_JPRB, 0.89996E+00_JPRB ,&
&  0.86504E+00_JPRB, 0.83770E+00_JPRB, 0.80139E+00_JPRB, 0.84380E+00_JPRB, 0.97802E+00_JPRB, 0.10908E+01_JPRB, 0.10988E+01_JPRB ,&
&  0.10768E+01_JPRB, 0.11572E+01_JPRB, 0.11783E+01_JPRB, 0.11708E+01_JPRB, 0.11777E+01_JPRB, 0.11482E+01_JPRB, 0.10867E+01_JPRB ,&
&  0.10120E+01_JPRB, 0.95187E+00_JPRB, 0.91082E+00_JPRB, 0.87624E+00_JPRB, 0.84778E+00_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.87223E+00_JPRB, 0.85130E+00_JPRB, 0.84523E+00_JPRB, 0.85430E+00_JPRB, 0.87293E+00_JPRB, 0.87067E+00_JPRB, 0.85425E+00_JPRB ,&
&  0.83111E+00_JPRB, 0.80532E+00_JPRB, 0.77409E+00_JPRB, 0.74326E+00_JPRB, 0.71758E+00_JPRB, 0.67733E+00_JPRB, 0.63978E+00_JPRB ,&
&  0.60466E+00_JPRB, 0.54680E+00_JPRB, 0.50042E+00_JPRB, 0.49633E+00_JPRB, 0.48454E+00_JPRB, 0.46276E+00_JPRB, 0.47693E+00_JPRB ,&
&  0.50885E+00_JPRB, 0.55439E+00_JPRB, 0.60971E+00_JPRB, 0.68015E+00_JPRB, 0.74556E+00_JPRB, 0.82985E+00_JPRB, 0.89176E+00_JPRB ,&
&  0.86024E+00_JPRB, 0.85050E+00_JPRB, 0.86827E+00_JPRB, 0.86940E+00_JPRB, 0.84037E+00_JPRB, 0.83564E+00_JPRB, 0.84759E+00_JPRB ,&
&  0.86467E+00_JPRB, 0.88030E+00_JPRB, 0.88689E+00_JPRB, 0.87783E+00_JPRB, 0.87511E+00_JPRB, 0.87160E+00_JPRB, 0.87257E+00_JPRB ,&
&  0.82978E+00_JPRB, 0.79081E+00_JPRB, 0.74682E+00_JPRB, 0.76652E+00_JPRB, 0.70417E+00_JPRB, 0.67743E+00_JPRB, 0.67082E+00_JPRB ,&
&  0.75415E+00_JPRB, 0.90931E+00_JPRB, 0.10198E+01_JPRB, 0.14867E+01_JPRB, 0.15269E+01_JPRB, 0.16505E+01_JPRB, 0.20005E+01_JPRB ,&
&  0.22026E+01_JPRB, 0.22731E+01_JPRB, 0.22744E+01_JPRB, 0.18502E+01_JPRB, 0.16328E+01_JPRB, 0.21396E+01_JPRB, 0.21551E+01_JPRB ,&
&  0.20578E+01_JPRB, 0.20271E+01_JPRB, 0.20802E+01_JPRB, 0.24124E+01_JPRB, 0.20084E+01_JPRB, 0.17760E+01_JPRB, 0.14118E+01_JPRB ,&
&  0.14715E+01_JPRB, 0.12854E+01_JPRB, 0.14422E+01_JPRB, 0.12624E+01_JPRB, 0.11911E+01_JPRB, 0.10936E+01_JPRB, 0.96994E+00_JPRB ,&
&  0.91200E+00_JPRB, 0.87964E+00_JPRB, 0.88782E+00_JPRB, 0.10527E+01_JPRB, 0.12517E+01_JPRB, 0.12907E+01_JPRB, 0.13483E+01_JPRB ,&
&  0.13895E+01_JPRB, 0.14157E+01_JPRB, 0.13634E+01_JPRB, 0.13662E+01_JPRB, 0.13243E+01_JPRB, 0.12725E+01_JPRB, 0.12113E+01_JPRB ,&
&  0.11471E+01_JPRB, 0.10839E+01_JPRB, 0.10172E+01_JPRB, 0.95558E+00_JPRB, 0.90619E+00_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.91258E+00_JPRB, 0.89404E+00_JPRB, 0.89194E+00_JPRB, 0.89682E+00_JPRB, 0.89416E+00_JPRB, 0.87630E+00_JPRB, 0.85170E+00_JPRB ,&
&  0.82581E+00_JPRB, 0.78347E+00_JPRB, 0.74186E+00_JPRB, 0.70978E+00_JPRB, 0.69146E+00_JPRB, 0.67434E+00_JPRB, 0.64712E+00_JPRB ,&
&  0.61374E+00_JPRB, 0.58262E+00_JPRB, 0.53546E+00_JPRB, 0.51353E+00_JPRB, 0.53470E+00_JPRB, 0.50311E+00_JPRB, 0.48135E+00_JPRB ,&
&  0.52695E+00_JPRB, 0.58658E+00_JPRB, 0.67046E+00_JPRB, 0.78261E+00_JPRB, 0.88506E+00_JPRB, 0.98194E+00_JPRB, 0.10342E+01_JPRB ,&
&  0.99219E+00_JPRB, 0.92794E+00_JPRB, 0.87786E+00_JPRB, 0.83310E+00_JPRB, 0.79758E+00_JPRB, 0.79873E+00_JPRB, 0.81256E+00_JPRB ,&
&  0.84664E+00_JPRB, 0.87753E+00_JPRB, 0.86689E+00_JPRB, 0.82900E+00_JPRB, 0.81984E+00_JPRB, 0.77681E+00_JPRB, 0.74837E+00_JPRB ,&
&  0.75446E+00_JPRB, 0.75065E+00_JPRB, 0.74290E+00_JPRB, 0.79989E+00_JPRB, 0.72157E+00_JPRB, 0.67129E+00_JPRB, 0.69109E+00_JPRB ,&
&  0.78280E+00_JPRB, 0.94711E+00_JPRB, 0.11074E+01_JPRB, 0.14925E+01_JPRB, 0.20090E+01_JPRB, 0.20840E+01_JPRB, 0.22881E+01_JPRB ,&
&  0.19598E+01_JPRB, 0.16462E+01_JPRB, 0.14268E+01_JPRB, 0.12463E+01_JPRB, 0.15783E+01_JPRB, 0.23016E+01_JPRB, 0.25287E+01_JPRB ,&
&  0.24825E+01_JPRB, 0.24775E+01_JPRB, 0.27018E+01_JPRB, 0.25217E+01_JPRB, 0.18767E+01_JPRB, 0.13405E+01_JPRB, 0.17206E+01_JPRB ,&
&  0.18402E+01_JPRB, 0.18139E+01_JPRB, 0.16316E+01_JPRB, 0.13360E+01_JPRB, 0.10780E+01_JPRB, 0.11998E+01_JPRB, 0.11104E+01_JPRB ,&
&  0.99625E+00_JPRB, 0.94601E+00_JPRB, 0.11169E+01_JPRB, 0.14106E+01_JPRB, 0.15291E+01_JPRB, 0.17093E+01_JPRB, 0.17611E+01_JPRB ,&
&  0.16817E+01_JPRB, 0.16225E+01_JPRB, 0.15378E+01_JPRB, 0.14380E+01_JPRB, 0.13531E+01_JPRB, 0.12633E+01_JPRB, 0.11957E+01_JPRB ,&
&  0.11372E+01_JPRB, 0.10875E+01_JPRB, 0.10338E+01_JPRB, 0.98353E+00_JPRB, 0.94205E+00_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.93964E+00_JPRB, 0.92948E+00_JPRB, 0.92091E+00_JPRB, 0.90965E+00_JPRB, 0.89282E+00_JPRB, 0.86939E+00_JPRB, 0.84085E+00_JPRB ,&
&  0.79829E+00_JPRB, 0.75821E+00_JPRB, 0.71675E+00_JPRB, 0.69427E+00_JPRB, 0.67121E+00_JPRB, 0.64944E+00_JPRB, 0.63639E+00_JPRB ,&
&  0.63354E+00_JPRB, 0.64394E+00_JPRB, 0.58540E+00_JPRB, 0.56359E+00_JPRB, 0.57258E+00_JPRB, 0.56629E+00_JPRB, 0.50464E+00_JPRB ,&
&  0.56813E+00_JPRB, 0.63401E+00_JPRB, 0.70799E+00_JPRB, 0.83792E+00_JPRB, 0.10447E+01_JPRB, 0.10279E+01_JPRB, 0.10054E+01_JPRB ,&
&  0.98877E+00_JPRB, 0.94423E+00_JPRB, 0.90074E+00_JPRB, 0.84286E+00_JPRB, 0.80473E+00_JPRB, 0.81189E+00_JPRB, 0.82979E+00_JPRB ,&
&  0.86601E+00_JPRB, 0.88342E+00_JPRB, 0.85405E+00_JPRB, 0.80850E+00_JPRB, 0.76836E+00_JPRB, 0.74439E+00_JPRB, 0.72761E+00_JPRB ,&
&  0.73057E+00_JPRB, 0.73075E+00_JPRB, 0.73536E+00_JPRB, 0.80949E+00_JPRB, 0.74404E+00_JPRB, 0.71489E+00_JPRB, 0.84729E+00_JPRB ,&
&  0.97481E+00_JPRB, 0.12571E+01_JPRB, 0.13726E+01_JPRB, 0.16765E+01_JPRB, 0.20083E+01_JPRB, 0.21445E+01_JPRB, 0.22407E+01_JPRB ,&
&  0.17620E+01_JPRB, 0.15355E+01_JPRB, 0.17107E+01_JPRB, 0.19606E+01_JPRB, 0.19438E+01_JPRB, 0.21472E+01_JPRB, 0.35173E+01_JPRB ,&
&  0.25634E+01_JPRB, 0.27086E+01_JPRB, 0.30237E+01_JPRB, 0.29957E+01_JPRB, 0.14040E+01_JPRB, 0.96497E+00_JPRB, 0.14768E+01_JPRB ,&
&  0.15056E+01_JPRB, 0.93080E+00_JPRB, 0.72874E+00_JPRB, 0.91024E+00_JPRB, 0.73068E+00_JPRB, 0.83624E+00_JPRB, 0.10357E+01_JPRB ,&
&  0.10947E+01_JPRB, 0.11194E+01_JPRB, 0.13632E+01_JPRB, 0.14887E+01_JPRB, 0.16212E+01_JPRB, 0.22207E+01_JPRB, 0.21120E+01_JPRB ,&
&  0.19766E+01_JPRB, 0.17026E+01_JPRB, 0.16207E+01_JPRB, 0.14431E+01_JPRB, 0.13180E+01_JPRB, 0.12087E+01_JPRB, 0.11235E+01_JPRB ,&
&  0.10625E+01_JPRB, 0.10181E+01_JPRB, 0.98708E+00_JPRB, 0.96239E+00_JPRB, 0.95245E+00_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.95538E+00_JPRB, 0.94376E+00_JPRB, 0.93089E+00_JPRB, 0.91377E+00_JPRB, 0.89080E+00_JPRB, 0.85940E+00_JPRB, 0.81865E+00_JPRB ,&
&  0.77157E+00_JPRB, 0.72919E+00_JPRB, 0.69527E+00_JPRB, 0.67237E+00_JPRB, 0.64876E+00_JPRB, 0.62998E+00_JPRB, 0.62931E+00_JPRB ,&
&  0.63801E+00_JPRB, 0.65770E+00_JPRB, 0.65315E+00_JPRB, 0.63738E+00_JPRB, 0.63976E+00_JPRB, 0.63343E+00_JPRB, 0.60096E+00_JPRB ,&
&  0.66795E+00_JPRB, 0.71147E+00_JPRB, 0.76325E+00_JPRB, 0.86021E+00_JPRB, 0.95876E+00_JPRB, 0.94742E+00_JPRB, 0.97861E+00_JPRB ,&
&  0.96503E+00_JPRB, 0.95458E+00_JPRB, 0.90540E+00_JPRB, 0.85939E+00_JPRB, 0.84176E+00_JPRB, 0.84450E+00_JPRB, 0.87659E+00_JPRB ,&
&  0.89082E+00_JPRB, 0.89694E+00_JPRB, 0.85487E+00_JPRB, 0.78896E+00_JPRB, 0.75849E+00_JPRB, 0.73502E+00_JPRB, 0.72212E+00_JPRB ,&
&  0.72132E+00_JPRB, 0.74025E+00_JPRB, 0.76646E+00_JPRB, 0.86010E+00_JPRB, 0.10307E+01_JPRB, 0.11288E+01_JPRB, 0.10422E+01_JPRB ,&
&  0.99720E+00_JPRB, 0.11803E+01_JPRB, 0.15391E+01_JPRB, 0.16698E+01_JPRB, 0.18716E+01_JPRB, 0.20549E+01_JPRB, 0.20344E+01_JPRB ,&
&  0.19849E+01_JPRB, 0.21943E+01_JPRB, 0.22246E+01_JPRB, 0.25291E+01_JPRB, 0.27267E+01_JPRB, 0.22218E+01_JPRB, 0.28265E+01_JPRB ,&
&  0.28823E+01_JPRB, 0.27755E+01_JPRB, 0.21985E+01_JPRB, 0.15588E+01_JPRB, 0.16623E+01_JPRB, 0.11935E+01_JPRB, 0.76036E+00_JPRB ,&
&  0.67311E+00_JPRB, 0.62988E+00_JPRB, 0.59835E+00_JPRB, 0.62416E+00_JPRB, 0.62991E+00_JPRB, 0.77568E+00_JPRB, 0.98755E+00_JPRB ,&
&  0.12915E+01_JPRB, 0.14285E+01_JPRB, 0.14580E+01_JPRB, 0.14910E+01_JPRB, 0.15272E+01_JPRB, 0.19981E+01_JPRB, 0.20478E+01_JPRB ,&
&  0.17787E+01_JPRB, 0.16623E+01_JPRB, 0.15033E+01_JPRB, 0.13562E+01_JPRB, 0.12546E+01_JPRB, 0.11723E+01_JPRB, 0.11082E+01_JPRB ,&
&  0.10658E+01_JPRB, 0.10340E+01_JPRB, 0.10098E+01_JPRB, 0.98987E+00_JPRB, 0.96869E+00_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.94155E+00_JPRB, 0.92680E+00_JPRB, 0.91293E+00_JPRB, 0.88945E+00_JPRB, 0.86085E+00_JPRB, 0.82626E+00_JPRB, 0.78075E+00_JPRB ,&
&  0.72721E+00_JPRB, 0.68525E+00_JPRB, 0.65880E+00_JPRB, 0.63611E+00_JPRB, 0.62450E+00_JPRB, 0.61834E+00_JPRB, 0.62879E+00_JPRB ,&
&  0.64501E+00_JPRB, 0.66858E+00_JPRB, 0.70781E+00_JPRB, 0.72086E+00_JPRB, 0.78910E+00_JPRB, 0.78568E+00_JPRB, 0.78995E+00_JPRB ,&
&  0.83161E+00_JPRB, 0.84450E+00_JPRB, 0.87873E+00_JPRB, 0.92442E+00_JPRB, 0.98597E+00_JPRB, 0.97766E+00_JPRB, 0.97120E+00_JPRB ,&
&  0.95945E+00_JPRB, 0.97357E+00_JPRB, 0.93354E+00_JPRB, 0.89777E+00_JPRB, 0.87517E+00_JPRB, 0.88977E+00_JPRB, 0.89568E+00_JPRB ,&
&  0.90853E+00_JPRB, 0.90317E+00_JPRB, 0.86520E+00_JPRB, 0.80503E+00_JPRB, 0.77300E+00_JPRB, 0.75697E+00_JPRB, 0.76269E+00_JPRB ,&
&  0.77181E+00_JPRB, 0.81910E+00_JPRB, 0.87555E+00_JPRB, 0.10071E+01_JPRB, 0.98143E+00_JPRB, 0.84087E+00_JPRB, 0.95814E+00_JPRB ,&
&  0.10665E+01_JPRB, 0.13525E+01_JPRB, 0.14698E+01_JPRB, 0.17059E+01_JPRB, 0.18026E+01_JPRB, 0.17859E+01_JPRB, 0.19477E+01_JPRB ,&
&  0.21782E+01_JPRB, 0.30634E+01_JPRB, 0.23969E+01_JPRB, 0.24355E+01_JPRB, 0.37682E+01_JPRB, 0.30451E+01_JPRB, 0.28573E+01_JPRB ,&
&  0.32661E+01_JPRB, 0.27665E+01_JPRB, 0.24962E+01_JPRB, 0.20476E+01_JPRB, 0.30674E+01_JPRB, 0.30941E+01_JPRB, 0.10343E+01_JPRB ,&
&  0.74638E+00_JPRB, 0.66728E+00_JPRB, 0.61450E+00_JPRB, 0.57682E+00_JPRB, 0.61589E+00_JPRB, 0.80566E+00_JPRB, 0.11733E+01_JPRB ,&
&  0.13620E+01_JPRB, 0.15598E+01_JPRB, 0.15383E+01_JPRB, 0.15762E+01_JPRB, 0.15103E+01_JPRB, 0.17311E+01_JPRB, 0.17037E+01_JPRB ,&
&  0.15856E+01_JPRB, 0.15130E+01_JPRB, 0.13474E+01_JPRB, 0.12326E+01_JPRB, 0.11671E+01_JPRB, 0.11186E+01_JPRB, 0.10722E+01_JPRB ,&
&  0.10358E+01_JPRB, 0.10148E+01_JPRB, 0.99612E+00_JPRB, 0.97561E+00_JPRB, 0.95761E+00_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.90496E+00_JPRB, 0.89162E+00_JPRB, 0.87165E+00_JPRB, 0.84012E+00_JPRB, 0.82115E+00_JPRB, 0.79578E+00_JPRB, 0.74841E+00_JPRB ,&
&  0.70091E+00_JPRB, 0.67241E+00_JPRB, 0.64374E+00_JPRB, 0.61986E+00_JPRB, 0.61727E+00_JPRB, 0.62199E+00_JPRB, 0.64269E+00_JPRB ,&
&  0.66983E+00_JPRB, 0.70353E+00_JPRB, 0.74081E+00_JPRB, 0.84108E+00_JPRB, 0.96230E+00_JPRB, 0.97628E+00_JPRB, 0.11065E+01_JPRB ,&
&  0.10853E+01_JPRB, 0.10438E+01_JPRB, 0.10548E+01_JPRB, 0.10592E+01_JPRB, 0.10654E+01_JPRB, 0.10231E+01_JPRB, 0.10043E+01_JPRB ,&
&  0.97263E+00_JPRB, 0.10021E+01_JPRB, 0.96564E+00_JPRB, 0.94362E+00_JPRB, 0.94234E+00_JPRB, 0.94031E+00_JPRB, 0.93662E+00_JPRB ,&
&  0.93358E+00_JPRB, 0.92174E+00_JPRB, 0.89283E+00_JPRB, 0.84876E+00_JPRB, 0.81805E+00_JPRB, 0.80440E+00_JPRB, 0.81773E+00_JPRB ,&
&  0.86211E+00_JPRB, 0.93778E+00_JPRB, 0.10128E+01_JPRB, 0.11030E+01_JPRB, 0.90813E+00_JPRB, 0.10494E+01_JPRB, 0.11159E+01_JPRB ,&
&  0.12672E+01_JPRB, 0.14015E+01_JPRB, 0.14098E+01_JPRB, 0.15302E+01_JPRB, 0.17240E+01_JPRB, 0.17714E+01_JPRB, 0.19745E+01_JPRB ,&
&  0.25633E+01_JPRB, 0.25595E+01_JPRB, 0.22770E+01_JPRB, 0.25203E+01_JPRB, 0.33746E+01_JPRB, 0.49083E+01_JPRB, 0.26165E+01_JPRB ,&
&  0.21101E+01_JPRB, 0.26574E+01_JPRB, 0.25603E+01_JPRB, 0.23780E+01_JPRB, 0.32786E+01_JPRB, 0.40794E+01_JPRB, 0.27038E+01_JPRB ,&
&  0.88388E+00_JPRB, 0.67231E+00_JPRB, 0.72521E+00_JPRB, 0.67482E+00_JPRB, 0.65829E+00_JPRB, 0.84748E+00_JPRB, 0.17938E+01_JPRB ,&
&  0.15110E+01_JPRB, 0.16431E+01_JPRB, 0.16235E+01_JPRB, 0.15874E+01_JPRB, 0.15380E+01_JPRB, 0.16130E+01_JPRB, 0.16148E+01_JPRB ,&
&  0.13247E+01_JPRB, 0.12472E+01_JPRB, 0.11321E+01_JPRB, 0.10756E+01_JPRB, 0.10513E+01_JPRB, 0.10337E+01_JPRB, 0.99853E+00_JPRB ,&
&  0.97265E+00_JPRB, 0.95717E+00_JPRB, 0.94316E+00_JPRB, 0.93145E+00_JPRB, 0.92029E+00_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.86116E+00_JPRB, 0.84455E+00_JPRB, 0.81765E+00_JPRB, 0.80512E+00_JPRB, 0.81395E+00_JPRB, 0.78787E+00_JPRB, 0.75448E+00_JPRB ,&
&  0.70769E+00_JPRB, 0.68407E+00_JPRB, 0.65769E+00_JPRB, 0.64726E+00_JPRB, 0.64206E+00_JPRB, 0.65537E+00_JPRB, 0.68182E+00_JPRB ,&
&  0.71740E+00_JPRB, 0.75530E+00_JPRB, 0.80378E+00_JPRB, 0.95310E+00_JPRB, 0.11615E+01_JPRB, 0.11559E+01_JPRB, 0.12910E+01_JPRB ,&
&  0.14491E+01_JPRB, 0.13653E+01_JPRB, 0.12507E+01_JPRB, 0.12036E+01_JPRB, 0.11795E+01_JPRB, 0.11349E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.10219E+01_JPRB, 0.10674E+01_JPRB, 0.10294E+01_JPRB, 0.10007E+01_JPRB, 0.97160E+00_JPRB, 0.97287E+00_JPRB, 0.99186E+00_JPRB ,&
&  0.97559E+00_JPRB, 0.98057E+00_JPRB, 0.95011E+00_JPRB, 0.92446E+00_JPRB, 0.90191E+00_JPRB, 0.89596E+00_JPRB, 0.94505E+00_JPRB ,&
&  0.10346E+01_JPRB, 0.11267E+01_JPRB, 0.11817E+01_JPRB, 0.11660E+01_JPRB, 0.11805E+01_JPRB, 0.12069E+01_JPRB, 0.13436E+01_JPRB ,&
&  0.14571E+01_JPRB, 0.15281E+01_JPRB, 0.15676E+01_JPRB, 0.15914E+01_JPRB, 0.17513E+01_JPRB, 0.19562E+01_JPRB, 0.22360E+01_JPRB ,&
&  0.26746E+01_JPRB, 0.27914E+01_JPRB, 0.24637E+01_JPRB, 0.26055E+01_JPRB, 0.31434E+01_JPRB, 0.37982E+01_JPRB, 0.33275E+01_JPRB ,&
&  0.25977E+01_JPRB, 0.21245E+01_JPRB, 0.22638E+01_JPRB, 0.27448E+01_JPRB, 0.30539E+01_JPRB, 0.29704E+01_JPRB, 0.34702E+01_JPRB ,&
&  0.29398E+01_JPRB, 0.14840E+01_JPRB, 0.11938E+01_JPRB, 0.13942E+01_JPRB, 0.10672E+01_JPRB, 0.10873E+01_JPRB, 0.14312E+01_JPRB ,&
&  0.15219E+01_JPRB, 0.16588E+01_JPRB, 0.16687E+01_JPRB, 0.14743E+01_JPRB, 0.14966E+01_JPRB, 0.15134E+01_JPRB, 0.13873E+01_JPRB ,&
&  0.11858E+01_JPRB, 0.10897E+01_JPRB, 0.10188E+01_JPRB, 0.97408E+00_JPRB, 0.94782E+00_JPRB, 0.91948E+00_JPRB, 0.90203E+00_JPRB ,&
&  0.88212E+00_JPRB, 0.87018E+00_JPRB, 0.86572E+00_JPRB, 0.86314E+00_JPRB, 0.86685E+00_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.82557E+00_JPRB, 0.80692E+00_JPRB, 0.81479E+00_JPRB, 0.83983E+00_JPRB, 0.84212E+00_JPRB, 0.83661E+00_JPRB, 0.80866E+00_JPRB ,&
&  0.74407E+00_JPRB, 0.73036E+00_JPRB, 0.73281E+00_JPRB, 0.72578E+00_JPRB, 0.71226E+00_JPRB, 0.72396E+00_JPRB, 0.74844E+00_JPRB ,&
&  0.78625E+00_JPRB, 0.85713E+00_JPRB, 0.98130E+00_JPRB, 0.12488E+01_JPRB, 0.14908E+01_JPRB, 0.17864E+01_JPRB, 0.13240E+01_JPRB ,&
&  0.14496E+01_JPRB, 0.15618E+01_JPRB, 0.13750E+01_JPRB, 0.13227E+01_JPRB, 0.12967E+01_JPRB, 0.12733E+01_JPRB, 0.11725E+01_JPRB ,&
&  0.11444E+01_JPRB, 0.11565E+01_JPRB, 0.11093E+01_JPRB, 0.10357E+01_JPRB, 0.10034E+01_JPRB, 0.10625E+01_JPRB, 0.10601E+01_JPRB ,&
&  0.10637E+01_JPRB, 0.10872E+01_JPRB, 0.10464E+01_JPRB, 0.10428E+01_JPRB, 0.10305E+01_JPRB, 0.10361E+01_JPRB, 0.11112E+01_JPRB ,&
&  0.12111E+01_JPRB, 0.12928E+01_JPRB, 0.13457E+01_JPRB, 0.12933E+01_JPRB, 0.13601E+01_JPRB, 0.14537E+01_JPRB, 0.16261E+01_JPRB ,&
&  0.16572E+01_JPRB, 0.15175E+01_JPRB, 0.15817E+01_JPRB, 0.17622E+01_JPRB, 0.19748E+01_JPRB, 0.20756E+01_JPRB, 0.23306E+01_JPRB ,&
&  0.29196E+01_JPRB, 0.33518E+01_JPRB, 0.34464E+01_JPRB, 0.28903E+01_JPRB, 0.29146E+01_JPRB, 0.33729E+01_JPRB, 0.35311E+01_JPRB ,&
&  0.31816E+01_JPRB, 0.27077E+01_JPRB, 0.27361E+01_JPRB, 0.26403E+01_JPRB, 0.25231E+01_JPRB, 0.23290E+01_JPRB, 0.24242E+01_JPRB ,&
&  0.26883E+01_JPRB, 0.28770E+01_JPRB, 0.21943E+01_JPRB, 0.17607E+01_JPRB, 0.16785E+01_JPRB, 0.14912E+01_JPRB, 0.14377E+01_JPRB ,&
&  0.16044E+01_JPRB, 0.16042E+01_JPRB, 0.15896E+01_JPRB, 0.17569E+01_JPRB, 0.15529E+01_JPRB, 0.13921E+01_JPRB, 0.11429E+01_JPRB ,&
&  0.95394E+00_JPRB, 0.89376E+00_JPRB, 0.85756E+00_JPRB, 0.83778E+00_JPRB, 0.84762E+00_JPRB, 0.84591E+00_JPRB, 0.84532E+00_JPRB ,&
&  0.83373E+00_JPRB, 0.83533E+00_JPRB, 0.84279E+00_JPRB, 0.84897E+00_JPRB, 0.84433E+00_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.81573E+00_JPRB, 0.82642E+00_JPRB, 0.85367E+00_JPRB, 0.88304E+00_JPRB, 0.90141E+00_JPRB, 0.93037E+00_JPRB, 0.91497E+00_JPRB ,&
&  0.83131E+00_JPRB, 0.77687E+00_JPRB, 0.78841E+00_JPRB, 0.77404E+00_JPRB, 0.76365E+00_JPRB, 0.77746E+00_JPRB, 0.81472E+00_JPRB ,&
&  0.88302E+00_JPRB, 0.10324E+01_JPRB, 0.12634E+01_JPRB, 0.15600E+01_JPRB, 0.16943E+01_JPRB, 0.17790E+01_JPRB, 0.17445E+01_JPRB ,&
&  0.13322E+01_JPRB, 0.17019E+01_JPRB, 0.13911E+01_JPRB, 0.14009E+01_JPRB, 0.13890E+01_JPRB, 0.14903E+01_JPRB, 0.14086E+01_JPRB ,&
&  0.12723E+01_JPRB, 0.11916E+01_JPRB, 0.11996E+01_JPRB, 0.11446E+01_JPRB, 0.11337E+01_JPRB, 0.11363E+01_JPRB, 0.11501E+01_JPRB ,&
&  0.11352E+01_JPRB, 0.11344E+01_JPRB, 0.11534E+01_JPRB, 0.11767E+01_JPRB, 0.11736E+01_JPRB, 0.11889E+01_JPRB, 0.12329E+01_JPRB ,&
&  0.13473E+01_JPRB, 0.14461E+01_JPRB, 0.14777E+01_JPRB, 0.15244E+01_JPRB, 0.15678E+01_JPRB, 0.17669E+01_JPRB, 0.19552E+01_JPRB ,&
&  0.19183E+01_JPRB, 0.17263E+01_JPRB, 0.17624E+01_JPRB, 0.18843E+01_JPRB, 0.19948E+01_JPRB, 0.23088E+01_JPRB, 0.25078E+01_JPRB ,&
&  0.31294E+01_JPRB, 0.30874E+01_JPRB, 0.36260E+01_JPRB, 0.28402E+01_JPRB, 0.27161E+01_JPRB, 0.30898E+01_JPRB, 0.27768E+01_JPRB ,&
&  0.23246E+01_JPRB, 0.24942E+01_JPRB, 0.24716E+01_JPRB, 0.24784E+01_JPRB, 0.23746E+01_JPRB, 0.21398E+01_JPRB, 0.20061E+01_JPRB ,&
&  0.20391E+01_JPRB, 0.21461E+01_JPRB, 0.20633E+01_JPRB, 0.17436E+01_JPRB, 0.16295E+01_JPRB, 0.14887E+01_JPRB, 0.15621E+01_JPRB ,&
&  0.17529E+01_JPRB, 0.18749E+01_JPRB, 0.19402E+01_JPRB, 0.18387E+01_JPRB, 0.12591E+01_JPRB, 0.10397E+01_JPRB, 0.89950E+00_JPRB ,&
&  0.83210E+00_JPRB, 0.79719E+00_JPRB, 0.79748E+00_JPRB, 0.81463E+00_JPRB, 0.83122E+00_JPRB, 0.83912E+00_JPRB, 0.83106E+00_JPRB ,&
&  0.82828E+00_JPRB, 0.83102E+00_JPRB, 0.83149E+00_JPRB, 0.82757E+00_JPRB, 0.82198E+00_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.85544E+00_JPRB, 0.88352E+00_JPRB, 0.92348E+00_JPRB, 0.98512E+00_JPRB, 0.10426E+01_JPRB, 0.11347E+01_JPRB, 0.12144E+01_JPRB ,&
&  0.10898E+01_JPRB, 0.75605E+00_JPRB, 0.77284E+00_JPRB, 0.79159E+00_JPRB, 0.82056E+00_JPRB, 0.86725E+00_JPRB, 0.94024E+00_JPRB ,&
&  0.10678E+01_JPRB, 0.12366E+01_JPRB, 0.14275E+01_JPRB, 0.15993E+01_JPRB, 0.17568E+01_JPRB, 0.18492E+01_JPRB, 0.17960E+01_JPRB ,&
&  0.14443E+01_JPRB, 0.13363E+01_JPRB, 0.15657E+01_JPRB, 0.15443E+01_JPRB, 0.15526E+01_JPRB, 0.15581E+01_JPRB, 0.15584E+01_JPRB ,&
&  0.13922E+01_JPRB, 0.13129E+01_JPRB, 0.13262E+01_JPRB, 0.13324E+01_JPRB, 0.12487E+01_JPRB, 0.12832E+01_JPRB, 0.13127E+01_JPRB ,&
&  0.13141E+01_JPRB, 0.13139E+01_JPRB, 0.13500E+01_JPRB, 0.13944E+01_JPRB, 0.13799E+01_JPRB, 0.13910E+01_JPRB, 0.14534E+01_JPRB ,&
&  0.16032E+01_JPRB, 0.17182E+01_JPRB, 0.17864E+01_JPRB, 0.17987E+01_JPRB, 0.19189E+01_JPRB, 0.21244E+01_JPRB, 0.21922E+01_JPRB ,&
&  0.20916E+01_JPRB, 0.20602E+01_JPRB, 0.20985E+01_JPRB, 0.21859E+01_JPRB, 0.21429E+01_JPRB, 0.25595E+01_JPRB, 0.27072E+01_JPRB ,&
&  0.29239E+01_JPRB, 0.29295E+01_JPRB, 0.34043E+01_JPRB, 0.25028E+01_JPRB, 0.26419E+01_JPRB, 0.27245E+01_JPRB, 0.23697E+01_JPRB ,&
&  0.20938E+01_JPRB, 0.21786E+01_JPRB, 0.21200E+01_JPRB, 0.21386E+01_JPRB, 0.21090E+01_JPRB, 0.18510E+01_JPRB, 0.19158E+01_JPRB ,&
&  0.18273E+01_JPRB, 0.19977E+01_JPRB, 0.18987E+01_JPRB, 0.18211E+01_JPRB, 0.15790E+01_JPRB, 0.15090E+01_JPRB, 0.15724E+01_JPRB ,&
&  0.18139E+01_JPRB, 0.19594E+01_JPRB, 0.17644E+01_JPRB, 0.14087E+01_JPRB, 0.94226E+00_JPRB, 0.80361E+00_JPRB, 0.78795E+00_JPRB ,&
&  0.78231E+00_JPRB, 0.79271E+00_JPRB, 0.80968E+00_JPRB, 0.83012E+00_JPRB, 0.84117E+00_JPRB, 0.83941E+00_JPRB, 0.83490E+00_JPRB ,&
&  0.83828E+00_JPRB, 0.83993E+00_JPRB, 0.83647E+00_JPRB, 0.83781E+00_JPRB, 0.84399E+00_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.87321E+00_JPRB, 0.88499E+00_JPRB, 0.90037E+00_JPRB, 0.89949E+00_JPRB, 0.88760E+00_JPRB, 0.85907E+00_JPRB, 0.79773E+00_JPRB ,&
&  0.74106E+00_JPRB, 0.71699E+00_JPRB, 0.75249E+00_JPRB, 0.79956E+00_JPRB, 0.85183E+00_JPRB, 0.91718E+00_JPRB, 0.10169E+01_JPRB ,&
&  0.11407E+01_JPRB, 0.12458E+01_JPRB, 0.13541E+01_JPRB, 0.15318E+01_JPRB, 0.17187E+01_JPRB, 0.17269E+01_JPRB, 0.17225E+01_JPRB ,&
&  0.15052E+01_JPRB, 0.12006E+01_JPRB, 0.15086E+01_JPRB, 0.17366E+01_JPRB, 0.17271E+01_JPRB, 0.17048E+01_JPRB, 0.16505E+01_JPRB ,&
&  0.14587E+01_JPRB, 0.14951E+01_JPRB, 0.15662E+01_JPRB, 0.15953E+01_JPRB, 0.14742E+01_JPRB, 0.15085E+01_JPRB, 0.15453E+01_JPRB ,&
&  0.15849E+01_JPRB, 0.16083E+01_JPRB, 0.16837E+01_JPRB, 0.17302E+01_JPRB, 0.17396E+01_JPRB, 0.17964E+01_JPRB, 0.18928E+01_JPRB ,&
&  0.19940E+01_JPRB, 0.20735E+01_JPRB, 0.21875E+01_JPRB, 0.22631E+01_JPRB, 0.23455E+01_JPRB, 0.24565E+01_JPRB, 0.24259E+01_JPRB ,&
&  0.23249E+01_JPRB, 0.22288E+01_JPRB, 0.23494E+01_JPRB, 0.26324E+01_JPRB, 0.28458E+01_JPRB, 0.25666E+01_JPRB, 0.27971E+01_JPRB ,&
&  0.27617E+01_JPRB, 0.27424E+01_JPRB, 0.26628E+01_JPRB, 0.29901E+01_JPRB, 0.20619E+01_JPRB, 0.22232E+01_JPRB, 0.19078E+01_JPRB ,&
&  0.19812E+01_JPRB, 0.18930E+01_JPRB, 0.18112E+01_JPRB, 0.17912E+01_JPRB, 0.17936E+01_JPRB, 0.16168E+01_JPRB, 0.16898E+01_JPRB ,&
&  0.17762E+01_JPRB, 0.18332E+01_JPRB, 0.18330E+01_JPRB, 0.18585E+01_JPRB, 0.17512E+01_JPRB, 0.17505E+01_JPRB, 0.17776E+01_JPRB ,&
&  0.19533E+01_JPRB, 0.16796E+01_JPRB, 0.13561E+01_JPRB, 0.10597E+01_JPRB, 0.84500E+00_JPRB, 0.73823E+00_JPRB, 0.77427E+00_JPRB ,&
&  0.79466E+00_JPRB, 0.81101E+00_JPRB, 0.81743E+00_JPRB, 0.82877E+00_JPRB, 0.83576E+00_JPRB, 0.83655E+00_JPRB, 0.84853E+00_JPRB ,&
&  0.85382E+00_JPRB, 0.85722E+00_JPRB, 0.85691E+00_JPRB, 0.85876E+00_JPRB, 0.86402E+00_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.86686E+00_JPRB, 0.85801E+00_JPRB, 0.83288E+00_JPRB, 0.78726E+00_JPRB, 0.73477E+00_JPRB, 0.68113E+00_JPRB, 0.64788E+00_JPRB ,&
&  0.61780E+00_JPRB, 0.63687E+00_JPRB, 0.68518E+00_JPRB, 0.75232E+00_JPRB, 0.83342E+00_JPRB, 0.92064E+00_JPRB, 0.10087E+01_JPRB ,&
&  0.10655E+01_JPRB, 0.11164E+01_JPRB, 0.12039E+01_JPRB, 0.14037E+01_JPRB, 0.16169E+01_JPRB, 0.17524E+01_JPRB, 0.18073E+01_JPRB ,&
&  0.18315E+01_JPRB, 0.18331E+01_JPRB, 0.17571E+01_JPRB, 0.14782E+01_JPRB, 0.13364E+01_JPRB, 0.16799E+01_JPRB, 0.15959E+01_JPRB ,&
&  0.15945E+01_JPRB, 0.15957E+01_JPRB, 0.16380E+01_JPRB, 0.16918E+01_JPRB, 0.16960E+01_JPRB, 0.17181E+01_JPRB, 0.17906E+01_JPRB ,&
&  0.18561E+01_JPRB, 0.19346E+01_JPRB, 0.20111E+01_JPRB, 0.20619E+01_JPRB, 0.21319E+01_JPRB, 0.22239E+01_JPRB, 0.23105E+01_JPRB ,&
&  0.23863E+01_JPRB, 0.25051E+01_JPRB, 0.26448E+01_JPRB, 0.27548E+01_JPRB, 0.27240E+01_JPRB, 0.27759E+01_JPRB, 0.27179E+01_JPRB ,&
&  0.26272E+01_JPRB, 0.25575E+01_JPRB, 0.25644E+01_JPRB, 0.27254E+01_JPRB, 0.27707E+01_JPRB, 0.24155E+01_JPRB, 0.24138E+01_JPRB ,&
&  0.25338E+01_JPRB, 0.25303E+01_JPRB, 0.21938E+01_JPRB, 0.27645E+01_JPRB, 0.16244E+01_JPRB, 0.18988E+01_JPRB, 0.19842E+01_JPRB ,&
&  0.18616E+01_JPRB, 0.16908E+01_JPRB, 0.16263E+01_JPRB, 0.16240E+01_JPRB, 0.15824E+01_JPRB, 0.15201E+01_JPRB, 0.16529E+01_JPRB ,&
&  0.17196E+01_JPRB, 0.17195E+01_JPRB, 0.17844E+01_JPRB, 0.17931E+01_JPRB, 0.17937E+01_JPRB, 0.16949E+01_JPRB, 0.15308E+01_JPRB ,&
&  0.13518E+01_JPRB, 0.14400E+01_JPRB, 0.12759E+01_JPRB, 0.11830E+01_JPRB, 0.97639E+00_JPRB, 0.74690E+00_JPRB, 0.78540E+00_JPRB ,&
&  0.80355E+00_JPRB, 0.80791E+00_JPRB, 0.82464E+00_JPRB, 0.83415E+00_JPRB, 0.82760E+00_JPRB, 0.83111E+00_JPRB, 0.84602E+00_JPRB ,&
&  0.84593E+00_JPRB, 0.84633E+00_JPRB, 0.85027E+00_JPRB, 0.85294E+00_JPRB, 0.86156E+00_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.73946E+00_JPRB, 0.72244E+00_JPRB, 0.69583E+00_JPRB, 0.66771E+00_JPRB, 0.64037E+00_JPRB, 0.60567E+00_JPRB, 0.60299E+00_JPRB ,&
&  0.58716E+00_JPRB, 0.59306E+00_JPRB, 0.61577E+00_JPRB, 0.63883E+00_JPRB, 0.68111E+00_JPRB, 0.74818E+00_JPRB, 0.81449E+00_JPRB ,&
&  0.85871E+00_JPRB, 0.87769E+00_JPRB, 0.94240E+00_JPRB, 0.10430E+01_JPRB, 0.12140E+01_JPRB, 0.14930E+01_JPRB, 0.19426E+01_JPRB ,&
&  0.21543E+01_JPRB, 0.22878E+01_JPRB, 0.22640E+01_JPRB, 0.21254E+01_JPRB, 0.18474E+01_JPRB, 0.18445E+01_JPRB, 0.20376E+01_JPRB ,&
&  0.21564E+01_JPRB, 0.20295E+01_JPRB, 0.18191E+01_JPRB, 0.18195E+01_JPRB, 0.18519E+01_JPRB, 0.19033E+01_JPRB, 0.19808E+01_JPRB ,&
&  0.20697E+01_JPRB, 0.21370E+01_JPRB, 0.22030E+01_JPRB, 0.22922E+01_JPRB, 0.23690E+01_JPRB, 0.24461E+01_JPRB, 0.25463E+01_JPRB ,&
&  0.26539E+01_JPRB, 0.27499E+01_JPRB, 0.28299E+01_JPRB, 0.29073E+01_JPRB, 0.30429E+01_JPRB, 0.31718E+01_JPRB, 0.31717E+01_JPRB ,&
&  0.30993E+01_JPRB, 0.28847E+01_JPRB, 0.29191E+01_JPRB, 0.30334E+01_JPRB, 0.28852E+01_JPRB, 0.25884E+01_JPRB, 0.24835E+01_JPRB ,&
&  0.25510E+01_JPRB, 0.25860E+01_JPRB, 0.17645E+01_JPRB, 0.21796E+01_JPRB, 0.23813E+01_JPRB, 0.20583E+01_JPRB, 0.17958E+01_JPRB ,&
&  0.16692E+01_JPRB, 0.15438E+01_JPRB, 0.15383E+01_JPRB, 0.15103E+01_JPRB, 0.16021E+01_JPRB, 0.15790E+01_JPRB, 0.13841E+01_JPRB ,&
&  0.14346E+01_JPRB, 0.14144E+01_JPRB, 0.15213E+01_JPRB, 0.15798E+01_JPRB, 0.15269E+01_JPRB, 0.13924E+01_JPRB, 0.11855E+01_JPRB ,&
&  0.10543E+01_JPRB, 0.12903E+01_JPRB, 0.12057E+01_JPRB, 0.13252E+01_JPRB, 0.12208E+01_JPRB, 0.74351E+00_JPRB, 0.77957E+00_JPRB ,&
&  0.78652E+00_JPRB, 0.79450E+00_JPRB, 0.80544E+00_JPRB, 0.80604E+00_JPRB, 0.80612E+00_JPRB, 0.80831E+00_JPRB, 0.81546E+00_JPRB ,&
&  0.80821E+00_JPRB, 0.79580E+00_JPRB, 0.78406E+00_JPRB, 0.77598E+00_JPRB, 0.76149E+00_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.67943E+00_JPRB, 0.67622E+00_JPRB, 0.67249E+00_JPRB, 0.67506E+00_JPRB, 0.68421E+00_JPRB, 0.68383E+00_JPRB, 0.67262E+00_JPRB ,&
&  0.65171E+00_JPRB, 0.64126E+00_JPRB, 0.61795E+00_JPRB, 0.59237E+00_JPRB, 0.59386E+00_JPRB, 0.64264E+00_JPRB, 0.69429E+00_JPRB ,&
&  0.72662E+00_JPRB, 0.74256E+00_JPRB, 0.74474E+00_JPRB, 0.74918E+00_JPRB, 0.78958E+00_JPRB, 0.82178E+00_JPRB, 0.10315E+01_JPRB ,&
&  0.14189E+01_JPRB, 0.19763E+01_JPRB, 0.23944E+01_JPRB, 0.26130E+01_JPRB, 0.24736E+01_JPRB, 0.20878E+01_JPRB, 0.23619E+01_JPRB ,&
&  0.21678E+01_JPRB, 0.17711E+01_JPRB, 0.16777E+01_JPRB, 0.16884E+01_JPRB, 0.17406E+01_JPRB, 0.18041E+01_JPRB, 0.18969E+01_JPRB ,&
&  0.19886E+01_JPRB, 0.20846E+01_JPRB, 0.21857E+01_JPRB, 0.23037E+01_JPRB, 0.24285E+01_JPRB, 0.25660E+01_JPRB, 0.27059E+01_JPRB ,&
&  0.28677E+01_JPRB, 0.29674E+01_JPRB, 0.31099E+01_JPRB, 0.29954E+01_JPRB, 0.32649E+01_JPRB, 0.34932E+01_JPRB, 0.36009E+01_JPRB ,&
&  0.35418E+01_JPRB, 0.33918E+01_JPRB, 0.34532E+01_JPRB, 0.34254E+01_JPRB, 0.33789E+01_JPRB, 0.31814E+01_JPRB, 0.29201E+01_JPRB ,&
&  0.29728E+01_JPRB, 0.27591E+01_JPRB, 0.18232E+01_JPRB, 0.17940E+01_JPRB, 0.19250E+01_JPRB, 0.15597E+01_JPRB, 0.16985E+01_JPRB ,&
&  0.15324E+01_JPRB, 0.14154E+01_JPRB, 0.13051E+01_JPRB, 0.12476E+01_JPRB, 0.13490E+01_JPRB, 0.14600E+01_JPRB, 0.12539E+01_JPRB ,&
&  0.11444E+01_JPRB, 0.11225E+01_JPRB, 0.11743E+01_JPRB, 0.12266E+01_JPRB, 0.11992E+01_JPRB, 0.11281E+01_JPRB, 0.10477E+01_JPRB ,&
&  0.10661E+01_JPRB, 0.10266E+01_JPRB, 0.90991E+00_JPRB, 0.90793E+00_JPRB, 0.73984E+00_JPRB, 0.70782E+00_JPRB, 0.72750E+00_JPRB ,&
&  0.73214E+00_JPRB, 0.71901E+00_JPRB, 0.72043E+00_JPRB, 0.72025E+00_JPRB, 0.72654E+00_JPRB, 0.71733E+00_JPRB, 0.72598E+00_JPRB ,&
&  0.72695E+00_JPRB, 0.72142E+00_JPRB, 0.71357E+00_JPRB, 0.71290E+00_JPRB, 0.69618E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.77445E+00_JPRB, 0.78367E+00_JPRB, 0.79994E+00_JPRB, 0.81343E+00_JPRB, 0.83523E+00_JPRB, 0.82524E+00_JPRB, 0.81343E+00_JPRB ,&
&  0.78188E+00_JPRB, 0.77558E+00_JPRB, 0.76841E+00_JPRB, 0.76482E+00_JPRB, 0.77120E+00_JPRB, 0.79645E+00_JPRB, 0.84387E+00_JPRB ,&
&  0.85875E+00_JPRB, 0.86415E+00_JPRB, 0.85659E+00_JPRB, 0.84499E+00_JPRB, 0.81018E+00_JPRB, 0.77812E+00_JPRB, 0.75316E+00_JPRB ,&
&  0.77668E+00_JPRB, 0.98143E+00_JPRB, 0.12533E+01_JPRB, 0.15129E+01_JPRB, 0.18147E+01_JPRB, 0.17120E+01_JPRB, 0.19762E+01_JPRB ,&
&  0.21185E+01_JPRB, 0.16383E+01_JPRB, 0.16974E+01_JPRB, 0.15211E+01_JPRB, 0.15448E+01_JPRB, 0.15639E+01_JPRB, 0.16507E+01_JPRB ,&
&  0.17665E+01_JPRB, 0.18541E+01_JPRB, 0.19243E+01_JPRB, 0.20027E+01_JPRB, 0.21041E+01_JPRB, 0.22005E+01_JPRB, 0.22644E+01_JPRB ,&
&  0.23149E+01_JPRB, 0.22587E+01_JPRB, 0.23265E+01_JPRB, 0.27591E+01_JPRB, 0.31447E+01_JPRB, 0.33526E+01_JPRB, 0.35105E+01_JPRB ,&
&  0.37201E+01_JPRB, 0.38415E+01_JPRB, 0.34876E+01_JPRB, 0.34263E+01_JPRB, 0.36027E+01_JPRB, 0.32288E+01_JPRB, 0.32098E+01_JPRB ,&
&  0.33190E+01_JPRB, 0.28570E+01_JPRB, 0.16919E+01_JPRB, 0.17742E+01_JPRB, 0.18037E+01_JPRB, 0.16849E+01_JPRB, 0.16923E+01_JPRB ,&
&  0.14624E+01_JPRB, 0.13236E+01_JPRB, 0.11522E+01_JPRB, 0.10835E+01_JPRB, 0.10890E+01_JPRB, 0.11724E+01_JPRB, 0.11353E+01_JPRB ,&
&  0.10113E+01_JPRB, 0.97385E+00_JPRB, 0.10617E+01_JPRB, 0.11510E+01_JPRB, 0.11503E+01_JPRB, 0.10763E+01_JPRB, 0.97661E+00_JPRB ,&
&  0.10195E+01_JPRB, 0.97107E+00_JPRB, 0.10954E+01_JPRB, 0.11007E+01_JPRB, 0.73378E+00_JPRB, 0.72341E+00_JPRB, 0.71829E+00_JPRB ,&
&  0.72783E+00_JPRB, 0.72135E+00_JPRB, 0.70976E+00_JPRB, 0.71330E+00_JPRB, 0.70900E+00_JPRB, 0.70339E+00_JPRB, 0.72313E+00_JPRB ,&
&  0.74988E+00_JPRB, 0.74994E+00_JPRB, 0.75009E+00_JPRB, 0.75285E+00_JPRB, 0.76187E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.88338E+00_JPRB, 0.90349E+00_JPRB, 0.92781E+00_JPRB, 0.93225E+00_JPRB, 0.93919E+00_JPRB, 0.93298E+00_JPRB, 0.92914E+00_JPRB ,&
&  0.91620E+00_JPRB, 0.91306E+00_JPRB, 0.91601E+00_JPRB, 0.92128E+00_JPRB, 0.93225E+00_JPRB, 0.94590E+00_JPRB, 0.97773E+00_JPRB ,&
&  0.97852E+00_JPRB, 0.98420E+00_JPRB, 0.99215E+00_JPRB, 0.10080E+01_JPRB, 0.10234E+01_JPRB, 0.10205E+01_JPRB, 0.99366E+00_JPRB ,&
&  0.97546E+00_JPRB, 0.97371E+00_JPRB, 0.98794E+00_JPRB, 0.10180E+01_JPRB, 0.11877E+01_JPRB, 0.15050E+01_JPRB, 0.20149E+01_JPRB ,&
&  0.19259E+01_JPRB, 0.16608E+01_JPRB, 0.15059E+01_JPRB, 0.13060E+01_JPRB, 0.12920E+01_JPRB, 0.13251E+01_JPRB, 0.12742E+01_JPRB ,&
&  0.12682E+01_JPRB, 0.12491E+01_JPRB, 0.12725E+01_JPRB, 0.13291E+01_JPRB, 0.13774E+01_JPRB, 0.13664E+01_JPRB, 0.13431E+01_JPRB ,&
&  0.12356E+01_JPRB, 0.12038E+01_JPRB, 0.13006E+01_JPRB, 0.15589E+01_JPRB, 0.20940E+01_JPRB, 0.24693E+01_JPRB, 0.23607E+01_JPRB ,&
&  0.26160E+01_JPRB, 0.30649E+01_JPRB, 0.31743E+01_JPRB, 0.31696E+01_JPRB, 0.30342E+01_JPRB, 0.28362E+01_JPRB, 0.27089E+01_JPRB ,&
&  0.26525E+01_JPRB, 0.23491E+01_JPRB, 0.17390E+01_JPRB, 0.18139E+01_JPRB, 0.17259E+01_JPRB, 0.17616E+01_JPRB, 0.15861E+01_JPRB ,&
&  0.13881E+01_JPRB, 0.12598E+01_JPRB, 0.11502E+01_JPRB, 0.10444E+01_JPRB, 0.98798E+00_JPRB, 0.10211E+01_JPRB, 0.10061E+01_JPRB ,&
&  0.94728E+00_JPRB, 0.96495E+00_JPRB, 0.97203E+00_JPRB, 0.99033E+00_JPRB, 0.11549E+01_JPRB, 0.10156E+01_JPRB, 0.11200E+01_JPRB ,&
&  0.99059E+00_JPRB, 0.95828E+00_JPRB, 0.78497E+00_JPRB, 0.11177E+01_JPRB, 0.76301E+00_JPRB, 0.68331E+00_JPRB, 0.71053E+00_JPRB ,&
&  0.74965E+00_JPRB, 0.76817E+00_JPRB, 0.79364E+00_JPRB, 0.81504E+00_JPRB, 0.81216E+00_JPRB, 0.80750E+00_JPRB, 0.82201E+00_JPRB ,&
&  0.83674E+00_JPRB, 0.84240E+00_JPRB, 0.84926E+00_JPRB, 0.85321E+00_JPRB, 0.86776E+00_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.95585E+00_JPRB, 0.97391E+00_JPRB, 0.98887E+00_JPRB, 0.99949E+00_JPRB, 0.10044E+01_JPRB, 0.10084E+01_JPRB, 0.10129E+01_JPRB ,&
&  0.10141E+01_JPRB, 0.10123E+01_JPRB, 0.10111E+01_JPRB, 0.10266E+01_JPRB, 0.10486E+01_JPRB, 0.10587E+01_JPRB, 0.10835E+01_JPRB ,&
&  0.10790E+01_JPRB, 0.10932E+01_JPRB, 0.11146E+01_JPRB, 0.11308E+01_JPRB, 0.11574E+01_JPRB, 0.11579E+01_JPRB, 0.11426E+01_JPRB ,&
&  0.11388E+01_JPRB, 0.11296E+01_JPRB, 0.11317E+01_JPRB, 0.11356E+01_JPRB, 0.11503E+01_JPRB, 0.11713E+01_JPRB, 0.13493E+01_JPRB ,&
&  0.14240E+01_JPRB, 0.17715E+01_JPRB, 0.16610E+01_JPRB, 0.14955E+01_JPRB, 0.14980E+01_JPRB, 0.13667E+01_JPRB, 0.12984E+01_JPRB ,&
&  0.11971E+01_JPRB, 0.10995E+01_JPRB, 0.10651E+01_JPRB, 0.10598E+01_JPRB, 0.11147E+01_JPRB, 0.11816E+01_JPRB, 0.12252E+01_JPRB ,&
&  0.13176E+01_JPRB, 0.13900E+01_JPRB, 0.14805E+01_JPRB, 0.15486E+01_JPRB, 0.16068E+01_JPRB, 0.17940E+01_JPRB, 0.17080E+01_JPRB ,&
&  0.17649E+01_JPRB, 0.19790E+01_JPRB, 0.20806E+01_JPRB, 0.21572E+01_JPRB, 0.20199E+01_JPRB, 0.18653E+01_JPRB, 0.16670E+01_JPRB ,&
&  0.14735E+01_JPRB, 0.14208E+01_JPRB, 0.15593E+01_JPRB, 0.15679E+01_JPRB, 0.15525E+01_JPRB, 0.15797E+01_JPRB, 0.14362E+01_JPRB ,&
&  0.13000E+01_JPRB, 0.12017E+01_JPRB, 0.11000E+01_JPRB, 0.10054E+01_JPRB, 0.95817E+00_JPRB, 0.94736E+00_JPRB, 0.92005E+00_JPRB ,&
&  0.88309E+00_JPRB, 0.87030E+00_JPRB, 0.87815E+00_JPRB, 0.90474E+00_JPRB, 0.91617E+00_JPRB, 0.13533E+01_JPRB, 0.16729E+01_JPRB ,&
&  0.10782E+01_JPRB, 0.85527E+00_JPRB, 0.59048E+00_JPRB, 0.83791E+00_JPRB, 0.67325E+00_JPRB, 0.66616E+00_JPRB, 0.73682E+00_JPRB ,&
&  0.83420E+00_JPRB, 0.90818E+00_JPRB, 0.93698E+00_JPRB, 0.96848E+00_JPRB, 0.87135E+00_JPRB, 0.85509E+00_JPRB, 0.85311E+00_JPRB ,&
&  0.86326E+00_JPRB, 0.88030E+00_JPRB, 0.89749E+00_JPRB, 0.91487E+00_JPRB, 0.93575E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.99555E+00_JPRB, 0.10099E+01_JPRB, 0.10231E+01_JPRB, 0.10400E+01_JPRB, 0.10551E+01_JPRB, 0.10741E+01_JPRB, 0.10865E+01_JPRB ,&
&  0.10980E+01_JPRB, 0.11058E+01_JPRB, 0.11067E+01_JPRB, 0.11206E+01_JPRB, 0.11344E+01_JPRB, 0.11602E+01_JPRB, 0.11900E+01_JPRB ,&
&  0.12241E+01_JPRB, 0.12602E+01_JPRB, 0.13009E+01_JPRB, 0.13250E+01_JPRB, 0.13445E+01_JPRB, 0.13662E+01_JPRB, 0.13863E+01_JPRB ,&
&  0.13993E+01_JPRB, 0.14039E+01_JPRB, 0.14243E+01_JPRB, 0.14619E+01_JPRB, 0.15068E+01_JPRB, 0.15688E+01_JPRB, 0.13788E+01_JPRB ,&
&  0.16567E+01_JPRB, 0.18605E+01_JPRB, 0.18696E+01_JPRB, 0.17555E+01_JPRB, 0.17385E+01_JPRB, 0.17338E+01_JPRB, 0.17647E+01_JPRB ,&
&  0.18299E+01_JPRB, 0.17747E+01_JPRB, 0.16968E+01_JPRB, 0.15680E+01_JPRB, 0.15021E+01_JPRB, 0.15385E+01_JPRB, 0.16231E+01_JPRB ,&
&  0.16776E+01_JPRB, 0.17377E+01_JPRB, 0.17522E+01_JPRB, 0.17514E+01_JPRB, 0.17597E+01_JPRB, 0.17515E+01_JPRB, 0.16529E+01_JPRB ,&
&  0.15702E+01_JPRB, 0.15469E+01_JPRB, 0.13447E+01_JPRB, 0.13519E+01_JPRB, 0.13454E+01_JPRB, 0.12274E+01_JPRB, 0.10505E+01_JPRB ,&
&  0.10026E+01_JPRB, 0.94969E+00_JPRB, 0.10190E+01_JPRB, 0.13484E+01_JPRB, 0.14274E+01_JPRB, 0.13915E+01_JPRB, 0.13072E+01_JPRB ,&
&  0.11877E+01_JPRB, 0.10755E+01_JPRB, 0.95078E+00_JPRB, 0.87638E+00_JPRB, 0.85267E+00_JPRB, 0.87748E+00_JPRB, 0.88542E+00_JPRB ,&
&  0.87341E+00_JPRB, 0.85299E+00_JPRB, 0.84382E+00_JPRB, 0.83897E+00_JPRB, 0.91972E+00_JPRB, 0.10458E+01_JPRB, 0.15077E+01_JPRB ,&
&  0.11097E+01_JPRB, 0.68025E+00_JPRB, 0.65656E+00_JPRB, 0.72800E+00_JPRB, 0.62254E+00_JPRB, 0.62500E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.93857E+00_JPRB, 0.10622E+01_JPRB, 0.10512E+01_JPRB, 0.10263E+01_JPRB, 0.97411E+00_JPRB, 0.97570E+00_JPRB, 0.95767E+00_JPRB ,&
&  0.94594E+00_JPRB, 0.94903E+00_JPRB, 0.95648E+00_JPRB, 0.96507E+00_JPRB, 0.97919E+00_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.10174E+01_JPRB, 0.10346E+01_JPRB, 0.10500E+01_JPRB, 0.10649E+01_JPRB, 0.10765E+01_JPRB, 0.10971E+01_JPRB, 0.11056E+01_JPRB ,&
&  0.11179E+01_JPRB, 0.11249E+01_JPRB, 0.11415E+01_JPRB, 0.11628E+01_JPRB, 0.11833E+01_JPRB, 0.12111E+01_JPRB, 0.12451E+01_JPRB ,&
&  0.12869E+01_JPRB, 0.13186E+01_JPRB, 0.13549E+01_JPRB, 0.13755E+01_JPRB, 0.13723E+01_JPRB, 0.13831E+01_JPRB, 0.13916E+01_JPRB ,&
&  0.13842E+01_JPRB, 0.13785E+01_JPRB, 0.13978E+01_JPRB, 0.14267E+01_JPRB, 0.14334E+01_JPRB, 0.14596E+01_JPRB, 0.11668E+01_JPRB ,&
&  0.17500E+01_JPRB, 0.20199E+01_JPRB, 0.21199E+01_JPRB, 0.20148E+01_JPRB, 0.19706E+01_JPRB, 0.20383E+01_JPRB, 0.21251E+01_JPRB ,&
&  0.22402E+01_JPRB, 0.23477E+01_JPRB, 0.23053E+01_JPRB, 0.21843E+01_JPRB, 0.20663E+01_JPRB, 0.20851E+01_JPRB, 0.21241E+01_JPRB ,&
&  0.21630E+01_JPRB, 0.21932E+01_JPRB, 0.21863E+01_JPRB, 0.21684E+01_JPRB, 0.21539E+01_JPRB, 0.21235E+01_JPRB, 0.20531E+01_JPRB ,&
&  0.19812E+01_JPRB, 0.19013E+01_JPRB, 0.16756E+01_JPRB, 0.18153E+01_JPRB, 0.16931E+01_JPRB, 0.14121E+01_JPRB, 0.11362E+01_JPRB ,&
&  0.99238E+00_JPRB, 0.96240E+00_JPRB, 0.11490E+01_JPRB, 0.12663E+01_JPRB, 0.12311E+01_JPRB, 0.11881E+01_JPRB, 0.11246E+01_JPRB ,&
&  0.10021E+01_JPRB, 0.89061E+00_JPRB, 0.81815E+00_JPRB, 0.79407E+00_JPRB, 0.80206E+00_JPRB, 0.82873E+00_JPRB, 0.84182E+00_JPRB ,&
&  0.84034E+00_JPRB, 0.83818E+00_JPRB, 0.82463E+00_JPRB, 0.80927E+00_JPRB, 0.82605E+00_JPRB, 0.84558E+00_JPRB, 0.12433E+01_JPRB ,&
&  0.10304E+01_JPRB, 0.75935E+00_JPRB, 0.94032E+00_JPRB, 0.48348E+00_JPRB, 0.56670E+00_JPRB, 0.56364E+00_JPRB, 0.65856E+00_JPRB ,&
&  0.79335E+00_JPRB, 0.11218E+01_JPRB, 0.15408E+01_JPRB, 0.13528E+01_JPRB, 0.12278E+01_JPRB, 0.11206E+01_JPRB, 0.10309E+01_JPRB ,&
&  0.99689E+00_JPRB, 0.98767E+00_JPRB, 0.98766E+00_JPRB, 0.98725E+00_JPRB, 0.99921E+00_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.10484E+01_JPRB, 0.10685E+01_JPRB, 0.10715E+01_JPRB, 0.10889E+01_JPRB, 0.10700E+01_JPRB, 0.10884E+01_JPRB, 0.10907E+01_JPRB ,&
&  0.11103E+01_JPRB, 0.11229E+01_JPRB, 0.11438E+01_JPRB, 0.11641E+01_JPRB, 0.11942E+01_JPRB, 0.12224E+01_JPRB, 0.12471E+01_JPRB ,&
&  0.12794E+01_JPRB, 0.13151E+01_JPRB, 0.13447E+01_JPRB, 0.13751E+01_JPRB, 0.13689E+01_JPRB, 0.13847E+01_JPRB, 0.13934E+01_JPRB ,&
&  0.14291E+01_JPRB, 0.14274E+01_JPRB, 0.14438E+01_JPRB, 0.14579E+01_JPRB, 0.14521E+01_JPRB, 0.14886E+01_JPRB, 0.11234E+01_JPRB ,&
&  0.18463E+01_JPRB, 0.21275E+01_JPRB, 0.22941E+01_JPRB, 0.22267E+01_JPRB, 0.21494E+01_JPRB, 0.21540E+01_JPRB, 0.21957E+01_JPRB ,&
&  0.23010E+01_JPRB, 0.23895E+01_JPRB, 0.23877E+01_JPRB, 0.25555E+01_JPRB, 0.26360E+01_JPRB, 0.25908E+01_JPRB, 0.26713E+01_JPRB ,&
&  0.27378E+01_JPRB, 0.27978E+01_JPRB, 0.28445E+01_JPRB, 0.28903E+01_JPRB, 0.29267E+01_JPRB, 0.30088E+01_JPRB, 0.30577E+01_JPRB ,&
&  0.30377E+01_JPRB, 0.30375E+01_JPRB, 0.30226E+01_JPRB, 0.28629E+01_JPRB, 0.26887E+01_JPRB, 0.22482E+01_JPRB, 0.18265E+01_JPRB ,&
&  0.12659E+01_JPRB, 0.10743E+01_JPRB, 0.11281E+01_JPRB, 0.11976E+01_JPRB, 0.11346E+01_JPRB, 0.10679E+01_JPRB, 0.96530E+00_JPRB ,&
&  0.84576E+00_JPRB, 0.80815E+00_JPRB, 0.78210E+00_JPRB, 0.78381E+00_JPRB, 0.80055E+00_JPRB, 0.81953E+00_JPRB, 0.83206E+00_JPRB ,&
&  0.83776E+00_JPRB, 0.83990E+00_JPRB, 0.81138E+00_JPRB, 0.78086E+00_JPRB, 0.79159E+00_JPRB, 0.87730E+00_JPRB, 0.87940E+00_JPRB ,&
&  0.10208E+01_JPRB, 0.81819E+00_JPRB, 0.74736E+00_JPRB, 0.69366E+00_JPRB, 0.59285E+00_JPRB, 0.60329E+00_JPRB, 0.76959E+00_JPRB ,&
&  0.12626E+01_JPRB, 0.16359E+01_JPRB, 0.23451E+01_JPRB, 0.39758E+01_JPRB, 0.28132E+01_JPRB, 0.20905E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.10308E+01_JPRB, 0.10115E+01_JPRB, 0.10028E+01_JPRB, 0.99716E+00_JPRB, 0.10206E+01_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.96943E+00_JPRB, 0.99688E+00_JPRB, 0.10266E+01_JPRB, 0.10537E+01_JPRB, 0.10653E+01_JPRB, 0.10696E+01_JPRB, 0.10703E+01_JPRB ,&
&  0.10960E+01_JPRB, 0.10778E+01_JPRB, 0.11028E+01_JPRB, 0.10834E+01_JPRB, 0.11058E+01_JPRB, 0.10970E+01_JPRB, 0.11101E+01_JPRB ,&
&  0.11324E+01_JPRB, 0.11684E+01_JPRB, 0.12117E+01_JPRB, 0.12371E+01_JPRB, 0.12520E+01_JPRB, 0.12996E+01_JPRB, 0.13629E+01_JPRB ,&
&  0.14402E+01_JPRB, 0.14842E+01_JPRB, 0.15358E+01_JPRB, 0.15682E+01_JPRB, 0.15535E+01_JPRB, 0.15369E+01_JPRB, 0.11393E+01_JPRB ,&
&  0.17645E+01_JPRB, 0.21496E+01_JPRB, 0.24252E+01_JPRB, 0.24344E+01_JPRB, 0.23143E+01_JPRB, 0.21999E+01_JPRB, 0.22297E+01_JPRB ,&
&  0.22866E+01_JPRB, 0.23044E+01_JPRB, 0.23159E+01_JPRB, 0.24080E+01_JPRB, 0.27502E+01_JPRB, 0.27109E+01_JPRB, 0.27969E+01_JPRB ,&
&  0.28919E+01_JPRB, 0.30021E+01_JPRB, 0.31429E+01_JPRB, 0.33106E+01_JPRB, 0.34734E+01_JPRB, 0.36323E+01_JPRB, 0.38037E+01_JPRB ,&
&  0.39356E+01_JPRB, 0.39863E+01_JPRB, 0.40274E+01_JPRB, 0.35817E+01_JPRB, 0.32906E+01_JPRB, 0.33031E+01_JPRB, 0.24508E+01_JPRB ,&
&  0.14363E+01_JPRB, 0.11750E+01_JPRB, 0.12024E+01_JPRB, 0.12109E+01_JPRB, 0.11488E+01_JPRB, 0.10180E+01_JPRB, 0.91828E+00_JPRB ,&
&  0.84694E+00_JPRB, 0.81915E+00_JPRB, 0.80216E+00_JPRB, 0.82333E+00_JPRB, 0.83524E+00_JPRB, 0.85194E+00_JPRB, 0.86119E+00_JPRB ,&
&  0.87377E+00_JPRB, 0.85192E+00_JPRB, 0.81596E+00_JPRB, 0.79733E+00_JPRB, 0.82369E+00_JPRB, 0.94167E+00_JPRB, 0.11363E+01_JPRB ,&
&  0.12290E+01_JPRB, 0.75850E+00_JPRB, 0.76368E+00_JPRB, 0.71066E+00_JPRB, 0.69619E+00_JPRB, 0.74798E+00_JPRB, 0.92190E+00_JPRB ,&
&  0.13788E+01_JPRB, 0.21147E+01_JPRB, 0.23759E+01_JPRB, 0.20396E+01_JPRB, 0.25057E+01_JPRB, 0.18054E+01_JPRB, 0.10190E+01_JPRB ,&
&  0.10018E+01_JPRB, 0.97443E+00_JPRB, 0.93855E+00_JPRB, 0.92911E+00_JPRB, 0.93996E+00_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.80923E+00_JPRB, 0.82744E+00_JPRB, 0.85164E+00_JPRB, 0.88890E+00_JPRB, 0.93024E+00_JPRB, 0.95891E+00_JPRB, 0.98801E+00_JPRB ,&
&  0.99914E+00_JPRB, 0.10169E+01_JPRB, 0.10333E+01_JPRB, 0.99797E+00_JPRB, 0.10125E+01_JPRB, 0.10020E+01_JPRB, 0.10010E+01_JPRB ,&
&  0.10044E+01_JPRB, 0.10263E+01_JPRB, 0.10540E+01_JPRB, 0.10841E+01_JPRB, 0.11293E+01_JPRB, 0.11854E+01_JPRB, 0.12568E+01_JPRB ,&
&  0.13453E+01_JPRB, 0.14257E+01_JPRB, 0.15173E+01_JPRB, 0.15768E+01_JPRB, 0.16329E+01_JPRB, 0.16645E+01_JPRB, 0.16512E+01_JPRB ,&
&  0.10712E+01_JPRB, 0.19139E+01_JPRB, 0.21696E+01_JPRB, 0.23158E+01_JPRB, 0.24218E+01_JPRB, 0.23092E+01_JPRB, 0.23238E+01_JPRB ,&
&  0.22551E+01_JPRB, 0.21288E+01_JPRB, 0.21892E+01_JPRB, 0.23312E+01_JPRB, 0.25068E+01_JPRB, 0.24874E+01_JPRB, 0.25211E+01_JPRB ,&
&  0.25797E+01_JPRB, 0.26966E+01_JPRB, 0.28544E+01_JPRB, 0.30384E+01_JPRB, 0.32608E+01_JPRB, 0.35218E+01_JPRB, 0.38390E+01_JPRB ,&
&  0.41684E+01_JPRB, 0.44674E+01_JPRB, 0.47293E+01_JPRB, 0.42025E+01_JPRB, 0.37124E+01_JPRB, 0.38045E+01_JPRB, 0.31667E+01_JPRB ,&
&  0.17944E+01_JPRB, 0.12185E+01_JPRB, 0.12515E+01_JPRB, 0.13513E+01_JPRB, 0.12430E+01_JPRB, 0.11307E+01_JPRB, 0.10094E+01_JPRB ,&
&  0.93382E+00_JPRB, 0.88753E+00_JPRB, 0.91008E+00_JPRB, 0.88530E+00_JPRB, 0.91700E+00_JPRB, 0.91810E+00_JPRB, 0.93085E+00_JPRB ,&
&  0.94336E+00_JPRB, 0.91957E+00_JPRB, 0.89994E+00_JPRB, 0.88716E+00_JPRB, 0.86713E+00_JPRB, 0.85030E+00_JPRB, 0.84201E+00_JPRB ,&
&  0.85140E+00_JPRB, 0.81024E+00_JPRB, 0.78576E+00_JPRB, 0.79877E+00_JPRB, 0.81858E+00_JPRB, 0.84877E+00_JPRB, 0.94095E+00_JPRB ,&
&  0.10947E+01_JPRB, 0.12624E+01_JPRB, 0.15462E+01_JPRB, 0.16350E+01_JPRB, 0.15749E+01_JPRB, 0.11949E+01_JPRB, 0.88677E+00_JPRB ,&
&  0.88337E+00_JPRB, 0.84156E+00_JPRB, 0.80784E+00_JPRB, 0.78916E+00_JPRB, 0.79371E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.73247E+00_JPRB, 0.74607E+00_JPRB, 0.73996E+00_JPRB, 0.76502E+00_JPRB, 0.78540E+00_JPRB, 0.81554E+00_JPRB, 0.84674E+00_JPRB ,&
&  0.87299E+00_JPRB, 0.89721E+00_JPRB, 0.93057E+00_JPRB, 0.91500E+00_JPRB, 0.93019E+00_JPRB, 0.92600E+00_JPRB, 0.90814E+00_JPRB ,&
&  0.90867E+00_JPRB, 0.92403E+00_JPRB, 0.94051E+00_JPRB, 0.95835E+00_JPRB, 0.98164E+00_JPRB, 0.10250E+01_JPRB, 0.10770E+01_JPRB ,&
&  0.11424E+01_JPRB, 0.12252E+01_JPRB, 0.13355E+01_JPRB, 0.14344E+01_JPRB, 0.15273E+01_JPRB, 0.15558E+01_JPRB, 0.14857E+01_JPRB ,&
&  0.74109E+00_JPRB, 0.10484E+01_JPRB, 0.17740E+01_JPRB, 0.20414E+01_JPRB, 0.21776E+01_JPRB, 0.22463E+01_JPRB, 0.22488E+01_JPRB ,&
&  0.21174E+01_JPRB, 0.19234E+01_JPRB, 0.19604E+01_JPRB, 0.22570E+01_JPRB, 0.22249E+01_JPRB, 0.22255E+01_JPRB, 0.22276E+01_JPRB ,&
&  0.22536E+01_JPRB, 0.23216E+01_JPRB, 0.24313E+01_JPRB, 0.25745E+01_JPRB, 0.28018E+01_JPRB, 0.30966E+01_JPRB, 0.34780E+01_JPRB ,&
&  0.39435E+01_JPRB, 0.44263E+01_JPRB, 0.48600E+01_JPRB, 0.42998E+01_JPRB, 0.42022E+01_JPRB, 0.41705E+01_JPRB, 0.43228E+01_JPRB ,&
&  0.23012E+01_JPRB, 0.13637E+01_JPRB, 0.14059E+01_JPRB, 0.15089E+01_JPRB, 0.13500E+01_JPRB, 0.12687E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.11158E+01_JPRB, 0.10441E+01_JPRB, 0.10240E+01_JPRB, 0.99495E+00_JPRB, 0.98343E+00_JPRB, 0.99763E+00_JPRB, 0.10250E+01_JPRB ,&
&  0.10253E+01_JPRB, 0.10225E+01_JPRB, 0.10187E+01_JPRB, 0.10169E+01_JPRB, 0.98234E+00_JPRB, 0.94810E+00_JPRB, 0.93382E+00_JPRB ,&
&  0.93279E+00_JPRB, 0.91053E+00_JPRB, 0.91595E+00_JPRB, 0.94889E+00_JPRB, 0.98255E+00_JPRB, 0.99979E+00_JPRB, 0.10124E+01_JPRB ,&
&  0.99862E+00_JPRB, 0.10109E+01_JPRB, 0.11318E+01_JPRB, 0.11763E+01_JPRB, 0.11639E+01_JPRB, 0.98375E+00_JPRB, 0.81725E+00_JPRB ,&
&  0.79870E+00_JPRB, 0.77169E+00_JPRB, 0.74633E+00_JPRB, 0.73814E+00_JPRB, 0.73019E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.67284E+00_JPRB, 0.65562E+00_JPRB, 0.64080E+00_JPRB, 0.64715E+00_JPRB, 0.65709E+00_JPRB, 0.67634E+00_JPRB, 0.72797E+00_JPRB ,&
&  0.77012E+00_JPRB, 0.78678E+00_JPRB, 0.80927E+00_JPRB, 0.81314E+00_JPRB, 0.82665E+00_JPRB, 0.81799E+00_JPRB, 0.82247E+00_JPRB ,&
&  0.81588E+00_JPRB, 0.82739E+00_JPRB, 0.84752E+00_JPRB, 0.86679E+00_JPRB, 0.88117E+00_JPRB, 0.89690E+00_JPRB, 0.92020E+00_JPRB ,&
&  0.94968E+00_JPRB, 0.99278E+00_JPRB, 0.10644E+01_JPRB, 0.11753E+01_JPRB, 0.13452E+01_JPRB, 0.15441E+01_JPRB, 0.16732E+01_JPRB ,&
&  0.12649E+01_JPRB, 0.58725E+00_JPRB, 0.11529E+01_JPRB, 0.17743E+01_JPRB, 0.18726E+01_JPRB, 0.19629E+01_JPRB, 0.19255E+01_JPRB ,&
&  0.17957E+01_JPRB, 0.17070E+01_JPRB, 0.16813E+01_JPRB, 0.19622E+01_JPRB, 0.20034E+01_JPRB, 0.20148E+01_JPRB, 0.20121E+01_JPRB ,&
&  0.20002E+01_JPRB, 0.20201E+01_JPRB, 0.20615E+01_JPRB, 0.21502E+01_JPRB, 0.23236E+01_JPRB, 0.25993E+01_JPRB, 0.29605E+01_JPRB ,&
&  0.34479E+01_JPRB, 0.41138E+01_JPRB, 0.47767E+01_JPRB, 0.44378E+01_JPRB, 0.45705E+01_JPRB, 0.40522E+01_JPRB, 0.34841E+01_JPRB ,&
&  0.21144E+01_JPRB, 0.14979E+01_JPRB, 0.15067E+01_JPRB, 0.16508E+01_JPRB, 0.14846E+01_JPRB, 0.12826E+01_JPRB, 0.13398E+01_JPRB ,&
&  0.12423E+01_JPRB, 0.11745E+01_JPRB, 0.11319E+01_JPRB, 0.10915E+01_JPRB, 0.10746E+01_JPRB, 0.10687E+01_JPRB, 0.10681E+01_JPRB ,&
&  0.10626E+01_JPRB, 0.10712E+01_JPRB, 0.10851E+01_JPRB, 0.10804E+01_JPRB, 0.10541E+01_JPRB, 0.10402E+01_JPRB, 0.10466E+01_JPRB ,&
&  0.10558E+01_JPRB, 0.10464E+01_JPRB, 0.10598E+01_JPRB, 0.10766E+01_JPRB, 0.10895E+01_JPRB, 0.10624E+01_JPRB, 0.10596E+01_JPRB ,&
&  0.10447E+01_JPRB, 0.10392E+01_JPRB, 0.10262E+01_JPRB, 0.99962E+00_JPRB, 0.98136E+00_JPRB, 0.90928E+00_JPRB, 0.82176E+00_JPRB ,&
&  0.79286E+00_JPRB, 0.76048E+00_JPRB, 0.73679E+00_JPRB, 0.71746E+00_JPRB, 0.69671E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.72067E+00_JPRB, 0.70372E+00_JPRB, 0.67200E+00_JPRB, 0.62958E+00_JPRB, 0.62322E+00_JPRB, 0.61989E+00_JPRB, 0.63192E+00_JPRB ,&
&  0.67459E+00_JPRB, 0.72806E+00_JPRB, 0.73801E+00_JPRB, 0.73530E+00_JPRB, 0.71541E+00_JPRB, 0.71531E+00_JPRB, 0.71968E+00_JPRB ,&
&  0.73108E+00_JPRB, 0.73987E+00_JPRB, 0.75970E+00_JPRB, 0.78569E+00_JPRB, 0.81174E+00_JPRB, 0.83161E+00_JPRB, 0.83715E+00_JPRB ,&
&  0.84442E+00_JPRB, 0.84748E+00_JPRB, 0.86754E+00_JPRB, 0.90125E+00_JPRB, 0.97186E+00_JPRB, 0.11132E+01_JPRB, 0.13433E+01_JPRB ,&
&  0.18934E+01_JPRB, 0.15456E+01_JPRB, 0.77688E+00_JPRB, 0.14573E+01_JPRB, 0.17152E+01_JPRB, 0.18670E+01_JPRB, 0.16291E+01_JPRB ,&
&  0.15748E+01_JPRB, 0.14319E+01_JPRB, 0.13748E+01_JPRB, 0.15509E+01_JPRB, 0.16115E+01_JPRB, 0.16715E+01_JPRB, 0.16862E+01_JPRB ,&
&  0.16388E+01_JPRB, 0.16567E+01_JPRB, 0.16879E+01_JPRB, 0.17387E+01_JPRB, 0.18301E+01_JPRB, 0.20342E+01_JPRB, 0.23991E+01_JPRB ,&
&  0.28601E+01_JPRB, 0.34818E+01_JPRB, 0.44151E+01_JPRB, 0.48404E+01_JPRB, 0.52870E+01_JPRB, 0.39485E+01_JPRB, 0.30435E+01_JPRB ,&
&  0.19858E+01_JPRB, 0.17838E+01_JPRB, 0.18503E+01_JPRB, 0.18326E+01_JPRB, 0.16536E+01_JPRB, 0.14820E+01_JPRB, 0.15331E+01_JPRB ,&
&  0.14006E+01_JPRB, 0.13085E+01_JPRB, 0.12604E+01_JPRB, 0.12120E+01_JPRB, 0.11758E+01_JPRB, 0.11544E+01_JPRB, 0.11383E+01_JPRB ,&
&  0.11215E+01_JPRB, 0.11263E+01_JPRB, 0.11345E+01_JPRB, 0.11419E+01_JPRB, 0.11419E+01_JPRB, 0.11297E+01_JPRB, 0.11359E+01_JPRB ,&
&  0.11397E+01_JPRB, 0.11429E+01_JPRB, 0.11485E+01_JPRB, 0.11630E+01_JPRB, 0.11291E+01_JPRB, 0.10778E+01_JPRB, 0.10838E+01_JPRB ,&
&  0.10746E+01_JPRB, 0.10834E+01_JPRB, 0.10617E+01_JPRB, 0.10670E+01_JPRB, 0.10096E+01_JPRB, 0.93061E+00_JPRB, 0.85492E+00_JPRB ,&
&  0.84580E+00_JPRB, 0.83588E+00_JPRB, 0.79393E+00_JPRB, 0.73998E+00_JPRB, 0.72149E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.78642E+00_JPRB, 0.79256E+00_JPRB, 0.78442E+00_JPRB, 0.74008E+00_JPRB, 0.69813E+00_JPRB, 0.65745E+00_JPRB, 0.62879E+00_JPRB ,&
&  0.62296E+00_JPRB, 0.62937E+00_JPRB, 0.65851E+00_JPRB, 0.68159E+00_JPRB, 0.67364E+00_JPRB, 0.65224E+00_JPRB, 0.64486E+00_JPRB ,&
&  0.65094E+00_JPRB, 0.67065E+00_JPRB, 0.68562E+00_JPRB, 0.71077E+00_JPRB, 0.73577E+00_JPRB, 0.75555E+00_JPRB, 0.77693E+00_JPRB ,&
&  0.76621E+00_JPRB, 0.76759E+00_JPRB, 0.77829E+00_JPRB, 0.78821E+00_JPRB, 0.80901E+00_JPRB, 0.85824E+00_JPRB, 0.91177E+00_JPRB ,&
&  0.11057E+01_JPRB, 0.17573E+01_JPRB, 0.10612E+01_JPRB, 0.14786E+01_JPRB, 0.17516E+01_JPRB, 0.17859E+01_JPRB, 0.16124E+01_JPRB ,&
&  0.14636E+01_JPRB, 0.12757E+01_JPRB, 0.12557E+01_JPRB, 0.12885E+01_JPRB, 0.12078E+01_JPRB, 0.12350E+01_JPRB, 0.12656E+01_JPRB ,&
&  0.12784E+01_JPRB, 0.13186E+01_JPRB, 0.13533E+01_JPRB, 0.13874E+01_JPRB, 0.14116E+01_JPRB, 0.14847E+01_JPRB, 0.17011E+01_JPRB ,&
&  0.21120E+01_JPRB, 0.26929E+01_JPRB, 0.36842E+01_JPRB, 0.45871E+01_JPRB, 0.56192E+01_JPRB, 0.43647E+01_JPRB, 0.32125E+01_JPRB ,&
&  0.23381E+01_JPRB, 0.22082E+01_JPRB, 0.22058E+01_JPRB, 0.21618E+01_JPRB, 0.19271E+01_JPRB, 0.18825E+01_JPRB, 0.19104E+01_JPRB ,&
&  0.16926E+01_JPRB, 0.15164E+01_JPRB, 0.14279E+01_JPRB, 0.13666E+01_JPRB, 0.13022E+01_JPRB, 0.12549E+01_JPRB, 0.12238E+01_JPRB ,&
&  0.11984E+01_JPRB, 0.11892E+01_JPRB, 0.12013E+01_JPRB, 0.12109E+01_JPRB, 0.12214E+01_JPRB, 0.12170E+01_JPRB, 0.12079E+01_JPRB ,&
&  0.11850E+01_JPRB, 0.11722E+01_JPRB, 0.11682E+01_JPRB, 0.11606E+01_JPRB, 0.11307E+01_JPRB, 0.10922E+01_JPRB, 0.10847E+01_JPRB ,&
&  0.10857E+01_JPRB, 0.11081E+01_JPRB, 0.11019E+01_JPRB, 0.10703E+01_JPRB, 0.10938E+01_JPRB, 0.10295E+01_JPRB, 0.95524E+00_JPRB ,&
&  0.97923E+00_JPRB, 0.11197E+01_JPRB, 0.11670E+01_JPRB, 0.81628E+00_JPRB, 0.79106E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.82023E+00_JPRB, 0.81780E+00_JPRB, 0.80204E+00_JPRB, 0.79667E+00_JPRB, 0.76182E+00_JPRB, 0.72232E+00_JPRB, 0.68605E+00_JPRB ,&
&  0.66145E+00_JPRB, 0.64734E+00_JPRB, 0.64938E+00_JPRB, 0.64849E+00_JPRB, 0.64894E+00_JPRB, 0.64099E+00_JPRB, 0.63611E+00_JPRB ,&
&  0.62350E+00_JPRB, 0.62834E+00_JPRB, 0.64352E+00_JPRB, 0.66899E+00_JPRB, 0.69894E+00_JPRB, 0.70922E+00_JPRB, 0.71919E+00_JPRB ,&
&  0.72098E+00_JPRB, 0.71712E+00_JPRB, 0.71725E+00_JPRB, 0.72530E+00_JPRB, 0.74283E+00_JPRB, 0.76391E+00_JPRB, 0.79464E+00_JPRB ,&
&  0.85952E+00_JPRB, 0.15247E+01_JPRB, 0.22875E+01_JPRB, 0.25984E+01_JPRB, 0.22670E+01_JPRB, 0.19014E+01_JPRB, 0.16998E+01_JPRB ,&
&  0.15896E+01_JPRB, 0.14442E+01_JPRB, 0.14448E+01_JPRB, 0.12644E+01_JPRB, 0.10839E+01_JPRB, 0.10211E+01_JPRB, 0.10382E+01_JPRB ,&
&  0.11043E+01_JPRB, 0.11319E+01_JPRB, 0.11469E+01_JPRB, 0.11467E+01_JPRB, 0.11242E+01_JPRB, 0.11263E+01_JPRB, 0.11883E+01_JPRB ,&
&  0.13782E+01_JPRB, 0.17831E+01_JPRB, 0.25194E+01_JPRB, 0.37977E+01_JPRB, 0.51766E+01_JPRB, 0.46362E+01_JPRB, 0.37334E+01_JPRB ,&
&  0.34267E+01_JPRB, 0.31064E+01_JPRB, 0.28120E+01_JPRB, 0.26464E+01_JPRB, 0.21245E+01_JPRB, 0.25362E+01_JPRB, 0.23750E+01_JPRB ,&
&  0.21790E+01_JPRB, 0.20419E+01_JPRB, 0.18519E+01_JPRB, 0.16547E+01_JPRB, 0.15094E+01_JPRB, 0.13789E+01_JPRB, 0.12829E+01_JPRB ,&
&  0.12518E+01_JPRB, 0.12576E+01_JPRB, 0.12742E+01_JPRB, 0.13038E+01_JPRB, 0.13356E+01_JPRB, 0.13437E+01_JPRB, 0.13154E+01_JPRB ,&
&  0.12477E+01_JPRB, 0.11897E+01_JPRB, 0.11076E+01_JPRB, 0.10927E+01_JPRB, 0.10789E+01_JPRB, 0.10425E+01_JPRB, 0.10206E+01_JPRB ,&
&  0.10396E+01_JPRB, 0.10829E+01_JPRB, 0.10982E+01_JPRB, 0.10865E+01_JPRB, 0.10714E+01_JPRB, 0.10663E+01_JPRB, 0.98844E+00_JPRB ,&
&  0.95294E+00_JPRB, 0.99934E+00_JPRB, 0.95307E+00_JPRB, 0.87701E+00_JPRB, 0.84587E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.84890E+00_JPRB, 0.81983E+00_JPRB, 0.80103E+00_JPRB, 0.80081E+00_JPRB, 0.77915E+00_JPRB, 0.76315E+00_JPRB, 0.75536E+00_JPRB ,&
&  0.71431E+00_JPRB, 0.68372E+00_JPRB, 0.65891E+00_JPRB, 0.65045E+00_JPRB, 0.65040E+00_JPRB, 0.64381E+00_JPRB, 0.64670E+00_JPRB ,&
&  0.63836E+00_JPRB, 0.62684E+00_JPRB, 0.62705E+00_JPRB, 0.64638E+00_JPRB, 0.67473E+00_JPRB, 0.68304E+00_JPRB, 0.69768E+00_JPRB ,&
&  0.71246E+00_JPRB, 0.70839E+00_JPRB, 0.69749E+00_JPRB, 0.69547E+00_JPRB, 0.69912E+00_JPRB, 0.69605E+00_JPRB, 0.71377E+00_JPRB ,&
&  0.78119E+00_JPRB, 0.15741E+01_JPRB, 0.16951E+01_JPRB, 0.28845E+01_JPRB, 0.25104E+01_JPRB, 0.21332E+01_JPRB, 0.16613E+01_JPRB ,&
&  0.16219E+01_JPRB, 0.18259E+01_JPRB, 0.15751E+01_JPRB, 0.13706E+01_JPRB, 0.11614E+01_JPRB, 0.10033E+01_JPRB, 0.96607E+00_JPRB ,&
&  0.10028E+01_JPRB, 0.10408E+01_JPRB, 0.10579E+01_JPRB, 0.10197E+01_JPRB, 0.97463E+00_JPRB, 0.95935E+00_JPRB, 0.97589E+00_JPRB ,&
&  0.10266E+01_JPRB, 0.11630E+01_JPRB, 0.15425E+01_JPRB, 0.23183E+01_JPRB, 0.40198E+01_JPRB, 0.48081E+01_JPRB, 0.43748E+01_JPRB ,&
&  0.41859E+01_JPRB, 0.43087E+01_JPRB, 0.38930E+01_JPRB, 0.34314E+01_JPRB, 0.28350E+01_JPRB, 0.28193E+01_JPRB, 0.26565E+01_JPRB ,&
&  0.24831E+01_JPRB, 0.24281E+01_JPRB, 0.23349E+01_JPRB, 0.21476E+01_JPRB, 0.19236E+01_JPRB, 0.16940E+01_JPRB, 0.14844E+01_JPRB ,&
&  0.13408E+01_JPRB, 0.12774E+01_JPRB, 0.12539E+01_JPRB, 0.12924E+01_JPRB, 0.13641E+01_JPRB, 0.14123E+01_JPRB, 0.14198E+01_JPRB ,&
&  0.13631E+01_JPRB, 0.12758E+01_JPRB, 0.11229E+01_JPRB, 0.10455E+01_JPRB, 0.10321E+01_JPRB, 0.99689E+00_JPRB, 0.98038E+00_JPRB ,&
&  0.10203E+01_JPRB, 0.10571E+01_JPRB, 0.10653E+01_JPRB, 0.10501E+01_JPRB, 0.10483E+01_JPRB, 0.10853E+01_JPRB, 0.10241E+01_JPRB ,&
&  0.98269E+00_JPRB, 0.99680E+00_JPRB, 0.10007E+01_JPRB, 0.93622E+00_JPRB, 0.89499E+00_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.89761E+00_JPRB, 0.85132E+00_JPRB, 0.81674E+00_JPRB, 0.80136E+00_JPRB, 0.79308E+00_JPRB, 0.78160E+00_JPRB, 0.77074E+00_JPRB ,&
&  0.74640E+00_JPRB, 0.71634E+00_JPRB, 0.69108E+00_JPRB, 0.66986E+00_JPRB, 0.66732E+00_JPRB, 0.66067E+00_JPRB, 0.65342E+00_JPRB ,&
&  0.64243E+00_JPRB, 0.63389E+00_JPRB, 0.62971E+00_JPRB, 0.63659E+00_JPRB, 0.65246E+00_JPRB, 0.66380E+00_JPRB, 0.67865E+00_JPRB ,&
&  0.68544E+00_JPRB, 0.69416E+00_JPRB, 0.69496E+00_JPRB, 0.68979E+00_JPRB, 0.68100E+00_JPRB, 0.66835E+00_JPRB, 0.65486E+00_JPRB ,&
&  0.75777E+00_JPRB, 0.17865E+01_JPRB, 0.14786E+01_JPRB, 0.26016E+01_JPRB, 0.22790E+01_JPRB, 0.20560E+01_JPRB, 0.16304E+01_JPRB ,&
&  0.15338E+01_JPRB, 0.16819E+01_JPRB, 0.15552E+01_JPRB, 0.14136E+01_JPRB, 0.12785E+01_JPRB, 0.11389E+01_JPRB, 0.10459E+01_JPRB ,&
&  0.10136E+01_JPRB, 0.10082E+01_JPRB, 0.99411E+00_JPRB, 0.96050E+00_JPRB, 0.93607E+00_JPRB, 0.91494E+00_JPRB, 0.91460E+00_JPRB ,&
&  0.92779E+00_JPRB, 0.96752E+00_JPRB, 0.11119E+01_JPRB, 0.16012E+01_JPRB, 0.26648E+01_JPRB, 0.40623E+01_JPRB, 0.50044E+01_JPRB ,&
&  0.51631E+01_JPRB, 0.56090E+01_JPRB, 0.50016E+01_JPRB, 0.42615E+01_JPRB, 0.36793E+01_JPRB, 0.33306E+01_JPRB, 0.30223E+01_JPRB ,&
&  0.27092E+01_JPRB, 0.25211E+01_JPRB, 0.24611E+01_JPRB, 0.24174E+01_JPRB, 0.23237E+01_JPRB, 0.21535E+01_JPRB, 0.19317E+01_JPRB ,&
&  0.16996E+01_JPRB, 0.15357E+01_JPRB, 0.14244E+01_JPRB, 0.13777E+01_JPRB, 0.13824E+01_JPRB, 0.14148E+01_JPRB, 0.14298E+01_JPRB ,&
&  0.13784E+01_JPRB, 0.13110E+01_JPRB, 0.11775E+01_JPRB, 0.10733E+01_JPRB, 0.10457E+01_JPRB, 0.10090E+01_JPRB, 0.98269E+00_JPRB ,&
&  0.10222E+01_JPRB, 0.10496E+01_JPRB, 0.10424E+01_JPRB, 0.10341E+01_JPRB, 0.10158E+01_JPRB, 0.10862E+01_JPRB, 0.10264E+01_JPRB ,&
&  0.99239E+00_JPRB, 0.98502E+00_JPRB, 0.98304E+00_JPRB, 0.96056E+00_JPRB, 0.93991E+00_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.93410E+00_JPRB, 0.89939E+00_JPRB, 0.86464E+00_JPRB, 0.83894E+00_JPRB, 0.81742E+00_JPRB, 0.80437E+00_JPRB, 0.79149E+00_JPRB ,&
&  0.76558E+00_JPRB, 0.73905E+00_JPRB, 0.72223E+00_JPRB, 0.70903E+00_JPRB, 0.69798E+00_JPRB, 0.69005E+00_JPRB, 0.67373E+00_JPRB ,&
&  0.65581E+00_JPRB, 0.63385E+00_JPRB, 0.63974E+00_JPRB, 0.65073E+00_JPRB, 0.65884E+00_JPRB, 0.66620E+00_JPRB, 0.68526E+00_JPRB ,&
&  0.68015E+00_JPRB, 0.67921E+00_JPRB, 0.68135E+00_JPRB, 0.68225E+00_JPRB, 0.67171E+00_JPRB, 0.67059E+00_JPRB, 0.64539E+00_JPRB ,&
&  0.73068E+00_JPRB, 0.82440E+00_JPRB, 0.16178E+01_JPRB, 0.22209E+01_JPRB, 0.20499E+01_JPRB, 0.18457E+01_JPRB, 0.16612E+01_JPRB ,&
&  0.16352E+01_JPRB, 0.15590E+01_JPRB, 0.14194E+01_JPRB, 0.13302E+01_JPRB, 0.12438E+01_JPRB, 0.11728E+01_JPRB, 0.11152E+01_JPRB ,&
&  0.10725E+01_JPRB, 0.10218E+01_JPRB, 0.98201E+00_JPRB, 0.93955E+00_JPRB, 0.91215E+00_JPRB, 0.89445E+00_JPRB, 0.89038E+00_JPRB ,&
&  0.88975E+00_JPRB, 0.89286E+00_JPRB, 0.93564E+00_JPRB, 0.11869E+01_JPRB, 0.20746E+01_JPRB, 0.32643E+01_JPRB, 0.46206E+01_JPRB ,&
&  0.56679E+01_JPRB, 0.60547E+01_JPRB, 0.55088E+01_JPRB, 0.48387E+01_JPRB, 0.41897E+01_JPRB, 0.36962E+01_JPRB, 0.33158E+01_JPRB ,&
&  0.29707E+01_JPRB, 0.26741E+01_JPRB, 0.24452E+01_JPRB, 0.23246E+01_JPRB, 0.22529E+01_JPRB, 0.21962E+01_JPRB, 0.21216E+01_JPRB ,&
&  0.20102E+01_JPRB, 0.18536E+01_JPRB, 0.16924E+01_JPRB, 0.15622E+01_JPRB, 0.14814E+01_JPRB, 0.14512E+01_JPRB, 0.14271E+01_JPRB ,&
&  0.13584E+01_JPRB, 0.12904E+01_JPRB, 0.11908E+01_JPRB, 0.11217E+01_JPRB, 0.11126E+01_JPRB, 0.10885E+01_JPRB, 0.10638E+01_JPRB ,&
&  0.10574E+01_JPRB, 0.10634E+01_JPRB, 0.10490E+01_JPRB, 0.10451E+01_JPRB, 0.10085E+01_JPRB, 0.10796E+01_JPRB, 0.10412E+01_JPRB ,&
&  0.10023E+01_JPRB, 0.99045E+00_JPRB, 0.97078E+00_JPRB, 0.95830E+00_JPRB, 0.95309E+00_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.96430E+00_JPRB, 0.94729E+00_JPRB, 0.90758E+00_JPRB, 0.86904E+00_JPRB, 0.84183E+00_JPRB, 0.82259E+00_JPRB, 0.80587E+00_JPRB ,&
&  0.78486E+00_JPRB, 0.75776E+00_JPRB, 0.73717E+00_JPRB, 0.72812E+00_JPRB, 0.72633E+00_JPRB, 0.72196E+00_JPRB, 0.70423E+00_JPRB ,&
&  0.68134E+00_JPRB, 0.66310E+00_JPRB, 0.66350E+00_JPRB, 0.66964E+00_JPRB, 0.67810E+00_JPRB, 0.67044E+00_JPRB, 0.68009E+00_JPRB ,&
&  0.68508E+00_JPRB, 0.68164E+00_JPRB, 0.67850E+00_JPRB, 0.67550E+00_JPRB, 0.67010E+00_JPRB, 0.66555E+00_JPRB, 0.64677E+00_JPRB ,&
&  0.69407E+00_JPRB, 0.96482E+00_JPRB, 0.15000E+01_JPRB, 0.18888E+01_JPRB, 0.18849E+01_JPRB, 0.16795E+01_JPRB, 0.15555E+01_JPRB ,&
&  0.14539E+01_JPRB, 0.13876E+01_JPRB, 0.12739E+01_JPRB, 0.11983E+01_JPRB, 0.11317E+01_JPRB, 0.10826E+01_JPRB, 0.10444E+01_JPRB ,&
&  0.10190E+01_JPRB, 0.99287E+00_JPRB, 0.95803E+00_JPRB, 0.92458E+00_JPRB, 0.89943E+00_JPRB, 0.87891E+00_JPRB, 0.86940E+00_JPRB ,&
&  0.86720E+00_JPRB, 0.86214E+00_JPRB, 0.87062E+00_JPRB, 0.97364E+00_JPRB, 0.16044E+01_JPRB, 0.20821E+01_JPRB, 0.36374E+01_JPRB ,&
&  0.49016E+01_JPRB, 0.46400E+01_JPRB, 0.44214E+01_JPRB, 0.40558E+01_JPRB, 0.35989E+01_JPRB, 0.32210E+01_JPRB, 0.28321E+01_JPRB ,&
&  0.24982E+01_JPRB, 0.22354E+01_JPRB, 0.20213E+01_JPRB, 0.18758E+01_JPRB, 0.17858E+01_JPRB, 0.17406E+01_JPRB, 0.17252E+01_JPRB ,&
&  0.17145E+01_JPRB, 0.16992E+01_JPRB, 0.16673E+01_JPRB, 0.16128E+01_JPRB, 0.15332E+01_JPRB, 0.14504E+01_JPRB, 0.13693E+01_JPRB ,&
&  0.12847E+01_JPRB, 0.12307E+01_JPRB, 0.11663E+01_JPRB, 0.11268E+01_JPRB, 0.11473E+01_JPRB, 0.11775E+01_JPRB, 0.11547E+01_JPRB ,&
&  0.11007E+01_JPRB, 0.10792E+01_JPRB, 0.10748E+01_JPRB, 0.10386E+01_JPRB, 0.99905E+00_JPRB, 0.10822E+01_JPRB, 0.10616E+01_JPRB ,&
&  0.10261E+01_JPRB, 0.10030E+01_JPRB, 0.98319E+00_JPRB, 0.96092E+00_JPRB, 0.96990E+00_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.99042E+00_JPRB, 0.95542E+00_JPRB, 0.92137E+00_JPRB, 0.88450E+00_JPRB, 0.85832E+00_JPRB, 0.83034E+00_JPRB, 0.80327E+00_JPRB ,&
&  0.78031E+00_JPRB, 0.76697E+00_JPRB, 0.75503E+00_JPRB, 0.74094E+00_JPRB, 0.74032E+00_JPRB, 0.73416E+00_JPRB, 0.71886E+00_JPRB ,&
&  0.69581E+00_JPRB, 0.67854E+00_JPRB, 0.67166E+00_JPRB, 0.67144E+00_JPRB, 0.67994E+00_JPRB, 0.68389E+00_JPRB, 0.69057E+00_JPRB ,&
&  0.69516E+00_JPRB, 0.67666E+00_JPRB, 0.66573E+00_JPRB, 0.66597E+00_JPRB, 0.66543E+00_JPRB, 0.66736E+00_JPRB, 0.65371E+00_JPRB ,&
&  0.67777E+00_JPRB, 0.82171E+00_JPRB, 0.11331E+01_JPRB, 0.15406E+01_JPRB, 0.15951E+01_JPRB, 0.14984E+01_JPRB, 0.14008E+01_JPRB ,&
&  0.13153E+01_JPRB, 0.12425E+01_JPRB, 0.11493E+01_JPRB, 0.10463E+01_JPRB, 0.96746E+00_JPRB, 0.92031E+00_JPRB, 0.89469E+00_JPRB ,&
&  0.88747E+00_JPRB, 0.88174E+00_JPRB, 0.86917E+00_JPRB, 0.85787E+00_JPRB, 0.84512E+00_JPRB, 0.83386E+00_JPRB, 0.82874E+00_JPRB ,&
&  0.82344E+00_JPRB, 0.81694E+00_JPRB, 0.81524E+00_JPRB, 0.83900E+00_JPRB, 0.11166E+01_JPRB, 0.14086E+01_JPRB, 0.21021E+01_JPRB ,&
&  0.24069E+01_JPRB, 0.25394E+01_JPRB, 0.26626E+01_JPRB, 0.26323E+01_JPRB, 0.24750E+01_JPRB, 0.23150E+01_JPRB, 0.21485E+01_JPRB ,&
&  0.19678E+01_JPRB, 0.17840E+01_JPRB, 0.16567E+01_JPRB, 0.15635E+01_JPRB, 0.14768E+01_JPRB, 0.14125E+01_JPRB, 0.13735E+01_JPRB ,&
&  0.13669E+01_JPRB, 0.13915E+01_JPRB, 0.14191E+01_JPRB, 0.14370E+01_JPRB, 0.14196E+01_JPRB, 0.13799E+01_JPRB, 0.13182E+01_JPRB ,&
&  0.12343E+01_JPRB, 0.11674E+01_JPRB, 0.11353E+01_JPRB, 0.11310E+01_JPRB, 0.11417E+01_JPRB, 0.11468E+01_JPRB, 0.11523E+01_JPRB ,&
&  0.11512E+01_JPRB, 0.11291E+01_JPRB, 0.11052E+01_JPRB, 0.10719E+01_JPRB, 0.10354E+01_JPRB, 0.10982E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.10547E+01_JPRB, 0.10247E+01_JPRB, 0.10039E+01_JPRB, 0.97938E+00_JPRB, 0.10136E+01_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.10595E+01_JPRB, 0.99683E+00_JPRB, 0.94667E+00_JPRB, 0.91019E+00_JPRB, 0.88638E+00_JPRB, 0.85539E+00_JPRB, 0.81800E+00_JPRB ,&
&  0.79013E+00_JPRB, 0.77978E+00_JPRB, 0.76867E+00_JPRB, 0.74978E+00_JPRB, 0.73593E+00_JPRB, 0.72176E+00_JPRB, 0.70899E+00_JPRB ,&
&  0.69020E+00_JPRB, 0.68178E+00_JPRB, 0.67433E+00_JPRB, 0.67440E+00_JPRB, 0.67197E+00_JPRB, 0.67392E+00_JPRB, 0.68351E+00_JPRB ,&
&  0.68614E+00_JPRB, 0.66747E+00_JPRB, 0.66466E+00_JPRB, 0.66437E+00_JPRB, 0.66270E+00_JPRB, 0.66168E+00_JPRB, 0.65164E+00_JPRB ,&
&  0.68180E+00_JPRB, 0.72180E+00_JPRB, 0.91362E+00_JPRB, 0.11025E+01_JPRB, 0.11926E+01_JPRB, 0.12449E+01_JPRB, 0.12221E+01_JPRB ,&
&  0.11770E+01_JPRB, 0.11093E+01_JPRB, 0.10334E+01_JPRB, 0.95592E+00_JPRB, 0.88483E+00_JPRB, 0.83774E+00_JPRB, 0.81157E+00_JPRB ,&
&  0.79732E+00_JPRB, 0.79222E+00_JPRB, 0.78613E+00_JPRB, 0.78240E+00_JPRB, 0.77834E+00_JPRB, 0.77151E+00_JPRB, 0.77144E+00_JPRB ,&
&  0.77687E+00_JPRB, 0.76720E+00_JPRB, 0.76400E+00_JPRB, 0.77949E+00_JPRB, 0.82995E+00_JPRB, 0.94585E+00_JPRB, 0.12580E+01_JPRB ,&
&  0.14576E+01_JPRB, 0.16073E+01_JPRB, 0.17679E+01_JPRB, 0.18307E+01_JPRB, 0.17885E+01_JPRB, 0.17072E+01_JPRB, 0.15916E+01_JPRB ,&
&  0.15010E+01_JPRB, 0.14351E+01_JPRB, 0.13744E+01_JPRB, 0.13131E+01_JPRB, 0.12594E+01_JPRB, 0.12196E+01_JPRB, 0.11924E+01_JPRB ,&
&  0.11735E+01_JPRB, 0.11790E+01_JPRB, 0.11967E+01_JPRB, 0.12147E+01_JPRB, 0.12229E+01_JPRB, 0.12201E+01_JPRB, 0.12054E+01_JPRB ,&
&  0.11633E+01_JPRB, 0.11059E+01_JPRB, 0.10718E+01_JPRB, 0.10748E+01_JPRB, 0.11024E+01_JPRB, 0.11169E+01_JPRB, 0.11229E+01_JPRB ,&
&  0.11319E+01_JPRB, 0.11343E+01_JPRB, 0.10989E+01_JPRB, 0.10739E+01_JPRB, 0.11305E+01_JPRB, 0.11104E+01_JPRB, 0.10835E+01_JPRB ,&
&  0.10523E+01_JPRB, 0.10321E+01_JPRB, 0.10077E+01_JPRB, 0.97842E+00_JPRB, 0.10467E+01_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.90941E+00_JPRB, 0.90715E+00_JPRB, 0.88607E+00_JPRB, 0.86689E+00_JPRB, 0.85685E+00_JPRB, 0.84663E+00_JPRB, 0.82689E+00_JPRB ,&
&  0.80422E+00_JPRB, 0.78297E+00_JPRB, 0.76954E+00_JPRB, 0.75082E+00_JPRB, 0.73187E+00_JPRB, 0.71979E+00_JPRB, 0.70729E+00_JPRB ,&
&  0.68966E+00_JPRB, 0.68269E+00_JPRB, 0.68039E+00_JPRB, 0.67644E+00_JPRB, 0.66999E+00_JPRB, 0.66549E+00_JPRB, 0.67062E+00_JPRB ,&
&  0.67078E+00_JPRB, 0.66995E+00_JPRB, 0.67107E+00_JPRB, 0.66629E+00_JPRB, 0.65917E+00_JPRB, 0.65637E+00_JPRB, 0.64438E+00_JPRB ,&
&  0.65339E+00_JPRB, 0.62908E+00_JPRB, 0.72369E+00_JPRB, 0.83846E+00_JPRB, 0.93425E+00_JPRB, 0.10014E+01_JPRB, 0.10537E+01_JPRB ,&
&  0.10524E+01_JPRB, 0.10006E+01_JPRB, 0.94283E+00_JPRB, 0.90204E+00_JPRB, 0.85825E+00_JPRB, 0.82324E+00_JPRB, 0.79438E+00_JPRB ,&
&  0.77538E+00_JPRB, 0.76535E+00_JPRB, 0.75778E+00_JPRB, 0.75070E+00_JPRB, 0.74326E+00_JPRB, 0.73715E+00_JPRB, 0.73480E+00_JPRB ,&
&  0.73142E+00_JPRB, 0.73227E+00_JPRB, 0.73518E+00_JPRB, 0.73473E+00_JPRB, 0.74785E+00_JPRB, 0.81266E+00_JPRB, 0.94999E+00_JPRB ,&
&  0.10451E+01_JPRB, 0.11500E+01_JPRB, 0.12883E+01_JPRB, 0.13667E+01_JPRB, 0.13538E+01_JPRB, 0.13141E+01_JPRB, 0.12580E+01_JPRB ,&
&  0.12107E+01_JPRB, 0.11573E+01_JPRB, 0.11242E+01_JPRB, 0.10900E+01_JPRB, 0.10531E+01_JPRB, 0.10231E+01_JPRB, 0.10063E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.99928E+00_JPRB, 0.10094E+01_JPRB, 0.10244E+01_JPRB, 0.10324E+01_JPRB, 0.10377E+01_JPRB, 0.10456E+01_JPRB ,&
&  0.10510E+01_JPRB, 0.10356E+01_JPRB, 0.10086E+01_JPRB, 0.10110E+01_JPRB, 0.10374E+01_JPRB, 0.10587E+01_JPRB, 0.10718E+01_JPRB ,&
&  0.10835E+01_JPRB, 0.10950E+01_JPRB, 0.11008E+01_JPRB, 0.11128E+01_JPRB, 0.10994E+01_JPRB, 0.10653E+01_JPRB, 0.10333E+01_JPRB ,&
&  0.10057E+01_JPRB, 0.98003E+00_JPRB, 0.96147E+00_JPRB, 0.93770E+00_JPRB, 0.86149E+00_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.85347E+00_JPRB, 0.84563E+00_JPRB, 0.82902E+00_JPRB, 0.81219E+00_JPRB, 0.80436E+00_JPRB, 0.80183E+00_JPRB, 0.80139E+00_JPRB ,&
&  0.79558E+00_JPRB, 0.78611E+00_JPRB, 0.77472E+00_JPRB, 0.76035E+00_JPRB, 0.74460E+00_JPRB, 0.73102E+00_JPRB, 0.71838E+00_JPRB ,&
&  0.69876E+00_JPRB, 0.68698E+00_JPRB, 0.68572E+00_JPRB, 0.68276E+00_JPRB, 0.67962E+00_JPRB, 0.67430E+00_JPRB, 0.67330E+00_JPRB ,&
&  0.67587E+00_JPRB, 0.67792E+00_JPRB, 0.67921E+00_JPRB, 0.67254E+00_JPRB, 0.66628E+00_JPRB, 0.66470E+00_JPRB, 0.65929E+00_JPRB ,&
&  0.65914E+00_JPRB, 0.56070E+00_JPRB, 0.62997E+00_JPRB, 0.72259E+00_JPRB, 0.77045E+00_JPRB, 0.83839E+00_JPRB, 0.92258E+00_JPRB ,&
&  0.96040E+00_JPRB, 0.94131E+00_JPRB, 0.90376E+00_JPRB, 0.86997E+00_JPRB, 0.83369E+00_JPRB, 0.80176E+00_JPRB, 0.77527E+00_JPRB ,&
&  0.75841E+00_JPRB, 0.74831E+00_JPRB, 0.74000E+00_JPRB, 0.72961E+00_JPRB, 0.71874E+00_JPRB, 0.71376E+00_JPRB, 0.71081E+00_JPRB ,&
&  0.71087E+00_JPRB, 0.71492E+00_JPRB, 0.71751E+00_JPRB, 0.72526E+00_JPRB, 0.73291E+00_JPRB, 0.75749E+00_JPRB, 0.82268E+00_JPRB ,&
&  0.87035E+00_JPRB, 0.92859E+00_JPRB, 0.10118E+01_JPRB, 0.10499E+01_JPRB, 0.10470E+01_JPRB, 0.10346E+01_JPRB, 0.10122E+01_JPRB ,&
&  0.98326E+00_JPRB, 0.94202E+00_JPRB, 0.91766E+00_JPRB, 0.89405E+00_JPRB, 0.87592E+00_JPRB, 0.86643E+00_JPRB, 0.87078E+00_JPRB ,&
&  0.87655E+00_JPRB, 0.87684E+00_JPRB, 0.87996E+00_JPRB, 0.88732E+00_JPRB, 0.89179E+00_JPRB, 0.89292E+00_JPRB, 0.90413E+00_JPRB ,&
&  0.92709E+00_JPRB, 0.95212E+00_JPRB, 0.95865E+00_JPRB, 0.95200E+00_JPRB, 0.95459E+00_JPRB, 0.97166E+00_JPRB, 0.98808E+00_JPRB ,&
&  0.10014E+01_JPRB, 0.10102E+01_JPRB, 0.10228E+01_JPRB, 0.98417E+00_JPRB, 0.10257E+01_JPRB, 0.10079E+01_JPRB, 0.98234E+00_JPRB ,&
&  0.95619E+00_JPRB, 0.93203E+00_JPRB, 0.91663E+00_JPRB, 0.78690E+00_JPRB, 0.82972E+00_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.78851E+00_JPRB, 0.77992E+00_JPRB, 0.76528E+00_JPRB, 0.75088E+00_JPRB, 0.74087E+00_JPRB, 0.73705E+00_JPRB, 0.73572E+00_JPRB ,&
&  0.73963E+00_JPRB, 0.74331E+00_JPRB, 0.74398E+00_JPRB, 0.74080E+00_JPRB, 0.73520E+00_JPRB, 0.72606E+00_JPRB, 0.71816E+00_JPRB ,&
&  0.70477E+00_JPRB, 0.69050E+00_JPRB, 0.68561E+00_JPRB, 0.68598E+00_JPRB, 0.68422E+00_JPRB, 0.67733E+00_JPRB, 0.67294E+00_JPRB ,&
&  0.67126E+00_JPRB, 0.67378E+00_JPRB, 0.66786E+00_JPRB, 0.66363E+00_JPRB, 0.65796E+00_JPRB, 0.65909E+00_JPRB, 0.65919E+00_JPRB ,&
&  0.65780E+00_JPRB, 0.53360E+00_JPRB, 0.60406E+00_JPRB, 0.66390E+00_JPRB, 0.69192E+00_JPRB, 0.74680E+00_JPRB, 0.80439E+00_JPRB ,&
&  0.84436E+00_JPRB, 0.86194E+00_JPRB, 0.85422E+00_JPRB, 0.83606E+00_JPRB, 0.80867E+00_JPRB, 0.78240E+00_JPRB, 0.75897E+00_JPRB ,&
&  0.73682E+00_JPRB, 0.72048E+00_JPRB, 0.70695E+00_JPRB, 0.69480E+00_JPRB, 0.68777E+00_JPRB, 0.68078E+00_JPRB, 0.67458E+00_JPRB ,&
&  0.67495E+00_JPRB, 0.68169E+00_JPRB, 0.68254E+00_JPRB, 0.68576E+00_JPRB, 0.69036E+00_JPRB, 0.69506E+00_JPRB, 0.72040E+00_JPRB ,&
&  0.74594E+00_JPRB, 0.76891E+00_JPRB, 0.79410E+00_JPRB, 0.81106E+00_JPRB, 0.81443E+00_JPRB, 0.82054E+00_JPRB, 0.81852E+00_JPRB ,&
&  0.81397E+00_JPRB, 0.80090E+00_JPRB, 0.78602E+00_JPRB, 0.77215E+00_JPRB, 0.76092E+00_JPRB, 0.75309E+00_JPRB, 0.75373E+00_JPRB ,&
&  0.75913E+00_JPRB, 0.75446E+00_JPRB, 0.74908E+00_JPRB, 0.74875E+00_JPRB, 0.75091E+00_JPRB, 0.75679E+00_JPRB, 0.77028E+00_JPRB ,&
&  0.79326E+00_JPRB, 0.82324E+00_JPRB, 0.84782E+00_JPRB, 0.85759E+00_JPRB, 0.85921E+00_JPRB, 0.86651E+00_JPRB, 0.87880E+00_JPRB ,&
&  0.89263E+00_JPRB, 0.90118E+00_JPRB, 0.90920E+00_JPRB, 0.93322E+00_JPRB, 0.93531E+00_JPRB, 0.92322E+00_JPRB, 0.90397E+00_JPRB ,&
&  0.87570E+00_JPRB, 0.86080E+00_JPRB, 0.70218E+00_JPRB, 0.76344E+00_JPRB, 0.77694E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.73833E+00_JPRB, 0.72640E+00_JPRB, 0.70969E+00_JPRB, 0.69645E+00_JPRB, 0.68612E+00_JPRB, 0.68238E+00_JPRB, 0.68019E+00_JPRB ,&
&  0.68438E+00_JPRB, 0.69212E+00_JPRB, 0.70004E+00_JPRB, 0.70493E+00_JPRB, 0.70865E+00_JPRB, 0.70899E+00_JPRB, 0.70535E+00_JPRB ,&
&  0.70236E+00_JPRB, 0.69143E+00_JPRB, 0.68288E+00_JPRB, 0.68316E+00_JPRB, 0.68066E+00_JPRB, 0.67742E+00_JPRB, 0.67286E+00_JPRB ,&
&  0.66803E+00_JPRB, 0.66371E+00_JPRB, 0.65352E+00_JPRB, 0.64834E+00_JPRB, 0.64677E+00_JPRB, 0.64900E+00_JPRB, 0.65847E+00_JPRB ,&
&  0.58305E+00_JPRB, 0.50353E+00_JPRB, 0.56413E+00_JPRB, 0.62727E+00_JPRB, 0.65908E+00_JPRB, 0.69903E+00_JPRB, 0.73505E+00_JPRB ,&
&  0.78227E+00_JPRB, 0.81189E+00_JPRB, 0.82767E+00_JPRB, 0.82229E+00_JPRB, 0.80468E+00_JPRB, 0.78438E+00_JPRB, 0.76306E+00_JPRB ,&
&  0.73665E+00_JPRB, 0.70628E+00_JPRB, 0.68618E+00_JPRB, 0.67627E+00_JPRB, 0.66794E+00_JPRB, 0.65903E+00_JPRB, 0.65341E+00_JPRB ,&
&  0.65115E+00_JPRB, 0.64969E+00_JPRB, 0.64776E+00_JPRB, 0.64671E+00_JPRB, 0.64883E+00_JPRB, 0.65177E+00_JPRB, 0.65791E+00_JPRB ,&
&  0.66790E+00_JPRB, 0.67802E+00_JPRB, 0.68634E+00_JPRB, 0.68999E+00_JPRB, 0.69370E+00_JPRB, 0.70336E+00_JPRB, 0.70990E+00_JPRB ,&
&  0.71571E+00_JPRB, 0.71560E+00_JPRB, 0.71237E+00_JPRB, 0.70707E+00_JPRB, 0.69866E+00_JPRB, 0.69038E+00_JPRB, 0.68478E+00_JPRB ,&
&  0.68249E+00_JPRB, 0.67665E+00_JPRB, 0.67493E+00_JPRB, 0.67767E+00_JPRB, 0.68242E+00_JPRB, 0.68810E+00_JPRB, 0.69413E+00_JPRB ,&
&  0.70581E+00_JPRB, 0.72654E+00_JPRB, 0.74366E+00_JPRB, 0.75486E+00_JPRB, 0.76216E+00_JPRB, 0.76815E+00_JPRB, 0.77764E+00_JPRB ,&
&  0.78717E+00_JPRB, 0.79713E+00_JPRB, 0.80984E+00_JPRB, 0.82416E+00_JPRB, 0.82850E+00_JPRB, 0.82408E+00_JPRB, 0.81319E+00_JPRB ,&
&  0.79611E+00_JPRB, 0.77382E+00_JPRB, 0.76551E+00_JPRB, 0.75419E+00_JPRB, 0.74522E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.65418E+00_JPRB, 0.64596E+00_JPRB, 0.63406E+00_JPRB, 0.62704E+00_JPRB, 0.62419E+00_JPRB, 0.62710E+00_JPRB, 0.63135E+00_JPRB ,&
&  0.63582E+00_JPRB, 0.63749E+00_JPRB, 0.64502E+00_JPRB, 0.65320E+00_JPRB, 0.65854E+00_JPRB, 0.66246E+00_JPRB, 0.66211E+00_JPRB ,&
&  0.66232E+00_JPRB, 0.66277E+00_JPRB, 0.65884E+00_JPRB, 0.65545E+00_JPRB, 0.65045E+00_JPRB, 0.65143E+00_JPRB, 0.65136E+00_JPRB ,&
&  0.64950E+00_JPRB, 0.64526E+00_JPRB, 0.63763E+00_JPRB, 0.62793E+00_JPRB, 0.62335E+00_JPRB, 0.62227E+00_JPRB, 0.61974E+00_JPRB ,&
&  0.62554E+00_JPRB, 0.54448E+00_JPRB, 0.58267E+00_JPRB, 0.60062E+00_JPRB, 0.62482E+00_JPRB, 0.65343E+00_JPRB, 0.68476E+00_JPRB ,&
&  0.71674E+00_JPRB, 0.74273E+00_JPRB, 0.75229E+00_JPRB, 0.75267E+00_JPRB, 0.73941E+00_JPRB, 0.71770E+00_JPRB, 0.69304E+00_JPRB ,&
&  0.67209E+00_JPRB, 0.65611E+00_JPRB, 0.63747E+00_JPRB, 0.62657E+00_JPRB, 0.61771E+00_JPRB, 0.60641E+00_JPRB, 0.59667E+00_JPRB ,&
&  0.59214E+00_JPRB, 0.58929E+00_JPRB, 0.58879E+00_JPRB, 0.59052E+00_JPRB, 0.59351E+00_JPRB, 0.59583E+00_JPRB, 0.59601E+00_JPRB ,&
&  0.59711E+00_JPRB, 0.59973E+00_JPRB, 0.60194E+00_JPRB, 0.60588E+00_JPRB, 0.61121E+00_JPRB, 0.61902E+00_JPRB, 0.62072E+00_JPRB ,&
&  0.61660E+00_JPRB, 0.61847E+00_JPRB, 0.62080E+00_JPRB, 0.62029E+00_JPRB, 0.61953E+00_JPRB, 0.61822E+00_JPRB, 0.61475E+00_JPRB ,&
&  0.61179E+00_JPRB, 0.61302E+00_JPRB, 0.61508E+00_JPRB, 0.62065E+00_JPRB, 0.62466E+00_JPRB, 0.63374E+00_JPRB, 0.64210E+00_JPRB ,&
&  0.65116E+00_JPRB, 0.66582E+00_JPRB, 0.68144E+00_JPRB, 0.69366E+00_JPRB, 0.70114E+00_JPRB, 0.70654E+00_JPRB, 0.71190E+00_JPRB ,&
&  0.71654E+00_JPRB, 0.72128E+00_JPRB, 0.72670E+00_JPRB, 0.72890E+00_JPRB, 0.72575E+00_JPRB, 0.72035E+00_JPRB, 0.71447E+00_JPRB ,&
&  0.70145E+00_JPRB, 0.68432E+00_JPRB, 0.67461E+00_JPRB, 0.66612E+00_JPRB, 0.65875E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.59901E+00_JPRB, 0.59302E+00_JPRB, 0.58729E+00_JPRB, 0.58623E+00_JPRB, 0.58515E+00_JPRB, 0.58710E+00_JPRB, 0.58981E+00_JPRB ,&
&  0.59652E+00_JPRB, 0.60090E+00_JPRB, 0.61028E+00_JPRB, 0.61807E+00_JPRB, 0.62207E+00_JPRB, 0.62679E+00_JPRB, 0.62989E+00_JPRB ,&
&  0.63133E+00_JPRB, 0.63328E+00_JPRB, 0.63453E+00_JPRB, 0.63923E+00_JPRB, 0.64022E+00_JPRB, 0.63977E+00_JPRB, 0.63888E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63397E+00_JPRB, 0.62801E+00_JPRB, 0.62004E+00_JPRB, 0.61028E+00_JPRB, 0.60574E+00_JPRB, 0.59925E+00_JPRB ,&
&  0.60005E+00_JPRB, 0.57003E+00_JPRB, 0.58193E+00_JPRB, 0.58024E+00_JPRB, 0.59600E+00_JPRB, 0.62094E+00_JPRB, 0.65394E+00_JPRB ,&
&  0.67642E+00_JPRB, 0.68792E+00_JPRB, 0.68896E+00_JPRB, 0.68587E+00_JPRB, 0.67085E+00_JPRB, 0.65546E+00_JPRB, 0.63817E+00_JPRB ,&
&  0.61861E+00_JPRB, 0.60762E+00_JPRB, 0.59387E+00_JPRB, 0.58378E+00_JPRB, 0.57957E+00_JPRB, 0.57653E+00_JPRB, 0.57145E+00_JPRB ,&
&  0.56372E+00_JPRB, 0.55701E+00_JPRB, 0.55740E+00_JPRB, 0.56080E+00_JPRB, 0.56447E+00_JPRB, 0.56684E+00_JPRB, 0.56578E+00_JPRB ,&
&  0.55965E+00_JPRB, 0.55522E+00_JPRB, 0.55518E+00_JPRB, 0.55725E+00_JPRB, 0.55577E+00_JPRB, 0.55088E+00_JPRB, 0.54549E+00_JPRB ,&
&  0.54500E+00_JPRB, 0.54802E+00_JPRB, 0.55141E+00_JPRB, 0.55580E+00_JPRB, 0.55862E+00_JPRB, 0.56196E+00_JPRB, 0.56327E+00_JPRB ,&
&  0.56752E+00_JPRB, 0.57163E+00_JPRB, 0.58030E+00_JPRB, 0.58862E+00_JPRB, 0.59027E+00_JPRB, 0.59867E+00_JPRB, 0.60772E+00_JPRB ,&
&  0.61734E+00_JPRB, 0.62894E+00_JPRB, 0.63890E+00_JPRB, 0.64685E+00_JPRB, 0.65491E+00_JPRB, 0.65954E+00_JPRB, 0.66565E+00_JPRB ,&
&  0.67355E+00_JPRB, 0.67545E+00_JPRB, 0.68130E+00_JPRB, 0.68153E+00_JPRB, 0.67641E+00_JPRB, 0.66794E+00_JPRB, 0.65584E+00_JPRB ,&
&  0.64168E+00_JPRB, 0.62368E+00_JPRB, 0.61316E+00_JPRB, 0.60628E+00_JPRB, 0.60202E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.53233E+00_JPRB, 0.52957E+00_JPRB, 0.52942E+00_JPRB, 0.53237E+00_JPRB, 0.53580E+00_JPRB, 0.53845E+00_JPRB, 0.54399E+00_JPRB ,&
&  0.55386E+00_JPRB, 0.56173E+00_JPRB, 0.57139E+00_JPRB, 0.58129E+00_JPRB, 0.58891E+00_JPRB, 0.59410E+00_JPRB, 0.59695E+00_JPRB ,&
&  0.59548E+00_JPRB, 0.59314E+00_JPRB, 0.59641E+00_JPRB, 0.60132E+00_JPRB, 0.60289E+00_JPRB, 0.60260E+00_JPRB, 0.60268E+00_JPRB ,&
&  0.60211E+00_JPRB, 0.59962E+00_JPRB, 0.59628E+00_JPRB, 0.59130E+00_JPRB, 0.58550E+00_JPRB, 0.58243E+00_JPRB, 0.57970E+00_JPRB ,&
&  0.57272E+00_JPRB, 0.57261E+00_JPRB, 0.52199E+00_JPRB, 0.53691E+00_JPRB, 0.54955E+00_JPRB, 0.57031E+00_JPRB, 0.59074E+00_JPRB ,&
&  0.60514E+00_JPRB, 0.61393E+00_JPRB, 0.61812E+00_JPRB, 0.61609E+00_JPRB, 0.60343E+00_JPRB, 0.59552E+00_JPRB, 0.58662E+00_JPRB ,&
&  0.57537E+00_JPRB, 0.56735E+00_JPRB, 0.55408E+00_JPRB, 0.53990E+00_JPRB, 0.53509E+00_JPRB, 0.53559E+00_JPRB, 0.53555E+00_JPRB ,&
&  0.53167E+00_JPRB, 0.52801E+00_JPRB, 0.52925E+00_JPRB, 0.53014E+00_JPRB, 0.52988E+00_JPRB, 0.52461E+00_JPRB, 0.51796E+00_JPRB ,&
&  0.50762E+00_JPRB, 0.49765E+00_JPRB, 0.49086E+00_JPRB, 0.48632E+00_JPRB, 0.48274E+00_JPRB, 0.47878E+00_JPRB, 0.47473E+00_JPRB ,&
&  0.47939E+00_JPRB, 0.49173E+00_JPRB, 0.50308E+00_JPRB, 0.51217E+00_JPRB, 0.51813E+00_JPRB, 0.52197E+00_JPRB, 0.52707E+00_JPRB ,&
&  0.53297E+00_JPRB, 0.54321E+00_JPRB, 0.55051E+00_JPRB, 0.55692E+00_JPRB, 0.56144E+00_JPRB, 0.56577E+00_JPRB, 0.57083E+00_JPRB ,&
&  0.57532E+00_JPRB, 0.58296E+00_JPRB, 0.58917E+00_JPRB, 0.59657E+00_JPRB, 0.60136E+00_JPRB, 0.60728E+00_JPRB, 0.61354E+00_JPRB ,&
&  0.62030E+00_JPRB, 0.62611E+00_JPRB, 0.62834E+00_JPRB, 0.62639E+00_JPRB, 0.61890E+00_JPRB, 0.60909E+00_JPRB, 0.59873E+00_JPRB ,&
&  0.58541E+00_JPRB, 0.56984E+00_JPRB, 0.55431E+00_JPRB, 0.54424E+00_JPRB, 0.53570E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.48214E+00_JPRB, 0.48233E+00_JPRB, 0.48660E+00_JPRB, 0.49092E+00_JPRB, 0.49414E+00_JPRB, 0.49891E+00_JPRB, 0.50843E+00_JPRB ,&
&  0.51809E+00_JPRB, 0.52743E+00_JPRB, 0.53388E+00_JPRB, 0.54170E+00_JPRB, 0.55272E+00_JPRB, 0.56024E+00_JPRB, 0.56627E+00_JPRB ,&
&  0.56884E+00_JPRB, 0.56935E+00_JPRB, 0.57057E+00_JPRB, 0.57386E+00_JPRB, 0.57354E+00_JPRB, 0.57000E+00_JPRB, 0.56879E+00_JPRB ,&
&  0.56871E+00_JPRB, 0.56697E+00_JPRB, 0.56660E+00_JPRB, 0.56741E+00_JPRB, 0.56474E+00_JPRB, 0.56053E+00_JPRB, 0.55964E+00_JPRB ,&
&  0.55945E+00_JPRB, 0.54554E+00_JPRB, 0.53766E+00_JPRB, 0.53050E+00_JPRB, 0.53401E+00_JPRB, 0.54536E+00_JPRB, 0.55793E+00_JPRB ,&
&  0.56551E+00_JPRB, 0.57060E+00_JPRB, 0.57033E+00_JPRB, 0.56940E+00_JPRB, 0.56469E+00_JPRB, 0.55324E+00_JPRB, 0.54355E+00_JPRB ,&
&  0.53700E+00_JPRB, 0.52754E+00_JPRB, 0.50490E+00_JPRB, 0.49173E+00_JPRB, 0.48827E+00_JPRB, 0.49097E+00_JPRB, 0.49508E+00_JPRB ,&
&  0.49676E+00_JPRB, 0.49416E+00_JPRB, 0.48796E+00_JPRB, 0.48069E+00_JPRB, 0.47082E+00_JPRB, 0.45948E+00_JPRB, 0.44957E+00_JPRB ,&
&  0.44405E+00_JPRB, 0.44012E+00_JPRB, 0.43696E+00_JPRB, 0.43980E+00_JPRB, 0.43929E+00_JPRB, 0.43583E+00_JPRB, 0.43843E+00_JPRB ,&
&  0.45076E+00_JPRB, 0.46625E+00_JPRB, 0.48056E+00_JPRB, 0.48841E+00_JPRB, 0.49170E+00_JPRB, 0.49543E+00_JPRB, 0.49966E+00_JPRB ,&
&  0.50544E+00_JPRB, 0.51097E+00_JPRB, 0.51614E+00_JPRB, 0.52298E+00_JPRB, 0.53138E+00_JPRB, 0.53879E+00_JPRB, 0.54188E+00_JPRB ,&
&  0.54451E+00_JPRB, 0.54637E+00_JPRB, 0.54817E+00_JPRB, 0.55408E+00_JPRB, 0.56252E+00_JPRB, 0.56842E+00_JPRB, 0.57300E+00_JPRB ,&
&  0.57758E+00_JPRB, 0.57964E+00_JPRB, 0.57808E+00_JPRB, 0.57367E+00_JPRB, 0.56797E+00_JPRB, 0.55690E+00_JPRB, 0.54300E+00_JPRB ,&
&  0.52693E+00_JPRB, 0.51202E+00_JPRB, 0.50086E+00_JPRB, 0.49121E+00_JPRB, 0.48439E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.44427E+00_JPRB, 0.44364E+00_JPRB, 0.44615E+00_JPRB, 0.45218E+00_JPRB, 0.45631E+00_JPRB, 0.46015E+00_JPRB, 0.46788E+00_JPRB ,&
&  0.47499E+00_JPRB, 0.48133E+00_JPRB, 0.48884E+00_JPRB, 0.49699E+00_JPRB, 0.50326E+00_JPRB, 0.51234E+00_JPRB, 0.52101E+00_JPRB ,&
&  0.52661E+00_JPRB, 0.52893E+00_JPRB, 0.53031E+00_JPRB, 0.53152E+00_JPRB, 0.53145E+00_JPRB, 0.52900E+00_JPRB, 0.52862E+00_JPRB ,&
&  0.52682E+00_JPRB, 0.52478E+00_JPRB, 0.52692E+00_JPRB, 0.52724E+00_JPRB, 0.52880E+00_JPRB, 0.53136E+00_JPRB, 0.53121E+00_JPRB ,&
&  0.52878E+00_JPRB, 0.51544E+00_JPRB, 0.51043E+00_JPRB, 0.50734E+00_JPRB, 0.51148E+00_JPRB, 0.51250E+00_JPRB, 0.51104E+00_JPRB ,&
&  0.50466E+00_JPRB, 0.50072E+00_JPRB, 0.50255E+00_JPRB, 0.50486E+00_JPRB, 0.49906E+00_JPRB, 0.48599E+00_JPRB, 0.47570E+00_JPRB ,&
&  0.46762E+00_JPRB, 0.45532E+00_JPRB, 0.43736E+00_JPRB, 0.42664E+00_JPRB, 0.42203E+00_JPRB, 0.41713E+00_JPRB, 0.41270E+00_JPRB ,&
&  0.41039E+00_JPRB, 0.40964E+00_JPRB, 0.41105E+00_JPRB, 0.41024E+00_JPRB, 0.40664E+00_JPRB, 0.40225E+00_JPRB, 0.40062E+00_JPRB ,&
&  0.40347E+00_JPRB, 0.40622E+00_JPRB, 0.41004E+00_JPRB, 0.41278E+00_JPRB, 0.41239E+00_JPRB, 0.41303E+00_JPRB, 0.41720E+00_JPRB ,&
&  0.42391E+00_JPRB, 0.43047E+00_JPRB, 0.43656E+00_JPRB, 0.43909E+00_JPRB, 0.44255E+00_JPRB, 0.44968E+00_JPRB, 0.45036E+00_JPRB ,&
&  0.44876E+00_JPRB, 0.45039E+00_JPRB, 0.45438E+00_JPRB, 0.46240E+00_JPRB, 0.47477E+00_JPRB, 0.48285E+00_JPRB, 0.48406E+00_JPRB ,&
&  0.48411E+00_JPRB, 0.48643E+00_JPRB, 0.49058E+00_JPRB, 0.50054E+00_JPRB, 0.51116E+00_JPRB, 0.51869E+00_JPRB, 0.52172E+00_JPRB ,&
&  0.51841E+00_JPRB, 0.51143E+00_JPRB, 0.49902E+00_JPRB, 0.48460E+00_JPRB, 0.47194E+00_JPRB, 0.46210E+00_JPRB, 0.45405E+00_JPRB ,&
&  0.44611E+00_JPRB, 0.44257E+00_JPRB, 0.44211E+00_JPRB, 0.44381E+00_JPRB, 0.44449E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.41492E+00_JPRB, 0.41654E+00_JPRB, 0.41864E+00_JPRB, 0.42147E+00_JPRB, 0.42454E+00_JPRB, 0.42641E+00_JPRB, 0.43018E+00_JPRB ,&
&  0.43235E+00_JPRB, 0.43888E+00_JPRB, 0.44961E+00_JPRB, 0.46038E+00_JPRB, 0.46819E+00_JPRB, 0.47481E+00_JPRB, 0.48133E+00_JPRB ,&
&  0.48667E+00_JPRB, 0.49373E+00_JPRB, 0.49466E+00_JPRB, 0.49227E+00_JPRB, 0.48965E+00_JPRB, 0.48766E+00_JPRB, 0.48588E+00_JPRB ,&
&  0.48339E+00_JPRB, 0.48327E+00_JPRB, 0.48703E+00_JPRB, 0.49125E+00_JPRB, 0.49556E+00_JPRB, 0.49669E+00_JPRB, 0.49523E+00_JPRB ,&
&  0.49867E+00_JPRB, 0.48894E+00_JPRB, 0.48254E+00_JPRB, 0.47510E+00_JPRB, 0.46859E+00_JPRB, 0.45705E+00_JPRB, 0.44854E+00_JPRB ,&
&  0.43842E+00_JPRB, 0.42950E+00_JPRB, 0.42568E+00_JPRB, 0.42161E+00_JPRB, 0.41185E+00_JPRB, 0.40051E+00_JPRB, 0.39214E+00_JPRB ,&
&  0.38658E+00_JPRB, 0.38094E+00_JPRB, 0.37416E+00_JPRB, 0.36858E+00_JPRB, 0.36405E+00_JPRB, 0.36249E+00_JPRB, 0.36424E+00_JPRB ,&
&  0.36844E+00_JPRB, 0.37244E+00_JPRB, 0.37484E+00_JPRB, 0.37549E+00_JPRB, 0.37394E+00_JPRB, 0.37140E+00_JPRB, 0.37000E+00_JPRB ,&
&  0.37084E+00_JPRB, 0.37260E+00_JPRB, 0.37483E+00_JPRB, 0.38004E+00_JPRB, 0.38103E+00_JPRB, 0.37974E+00_JPRB, 0.37455E+00_JPRB ,&
&  0.37263E+00_JPRB, 0.37191E+00_JPRB, 0.37397E+00_JPRB, 0.37931E+00_JPRB, 0.38787E+00_JPRB, 0.39609E+00_JPRB, 0.40186E+00_JPRB ,&
&  0.40460E+00_JPRB, 0.40614E+00_JPRB, 0.40906E+00_JPRB, 0.41268E+00_JPRB, 0.41419E+00_JPRB, 0.41358E+00_JPRB, 0.41213E+00_JPRB ,&
&  0.41495E+00_JPRB, 0.42251E+00_JPRB, 0.43067E+00_JPRB, 0.43619E+00_JPRB, 0.43904E+00_JPRB, 0.43816E+00_JPRB, 0.43129E+00_JPRB ,&
&  0.41867E+00_JPRB, 0.40417E+00_JPRB, 0.39008E+00_JPRB, 0.37820E+00_JPRB, 0.37006E+00_JPRB, 0.36617E+00_JPRB, 0.36906E+00_JPRB ,&
&  0.37686E+00_JPRB, 0.38684E+00_JPRB, 0.39599E+00_JPRB, 0.40275E+00_JPRB, 0.41002E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.38633E+00_JPRB, 0.38618E+00_JPRB, 0.38769E+00_JPRB, 0.38659E+00_JPRB, 0.38387E+00_JPRB, 0.38172E+00_JPRB, 0.38506E+00_JPRB ,&
&  0.39180E+00_JPRB, 0.40342E+00_JPRB, 0.41892E+00_JPRB, 0.43257E+00_JPRB, 0.44079E+00_JPRB, 0.44578E+00_JPRB, 0.45342E+00_JPRB ,&
&  0.46057E+00_JPRB, 0.46853E+00_JPRB, 0.46875E+00_JPRB, 0.45995E+00_JPRB, 0.45250E+00_JPRB, 0.44857E+00_JPRB, 0.44808E+00_JPRB ,&
&  0.44742E+00_JPRB, 0.44917E+00_JPRB, 0.45231E+00_JPRB, 0.45562E+00_JPRB, 0.45489E+00_JPRB, 0.45297E+00_JPRB, 0.45644E+00_JPRB ,&
&  0.45426E+00_JPRB, 0.44434E+00_JPRB, 0.43421E+00_JPRB, 0.40917E+00_JPRB, 0.40829E+00_JPRB, 0.39779E+00_JPRB, 0.38977E+00_JPRB ,&
&  0.37900E+00_JPRB, 0.36568E+00_JPRB, 0.35600E+00_JPRB, 0.34981E+00_JPRB, 0.34704E+00_JPRB, 0.34451E+00_JPRB, 0.34244E+00_JPRB ,&
&  0.34100E+00_JPRB, 0.34064E+00_JPRB, 0.34191E+00_JPRB, 0.33953E+00_JPRB, 0.33836E+00_JPRB, 0.33914E+00_JPRB, 0.34092E+00_JPRB ,&
&  0.34087E+00_JPRB, 0.34086E+00_JPRB, 0.34089E+00_JPRB, 0.34008E+00_JPRB, 0.33850E+00_JPRB, 0.33609E+00_JPRB, 0.33384E+00_JPRB ,&
&  0.33307E+00_JPRB, 0.33442E+00_JPRB, 0.33509E+00_JPRB, 0.33153E+00_JPRB, 0.32409E+00_JPRB, 0.31486E+00_JPRB, 0.30900E+00_JPRB ,&
&  0.30685E+00_JPRB, 0.30940E+00_JPRB, 0.31762E+00_JPRB, 0.32907E+00_JPRB, 0.33957E+00_JPRB, 0.34574E+00_JPRB, 0.34940E+00_JPRB ,&
&  0.35358E+00_JPRB, 0.35143E+00_JPRB, 0.34744E+00_JPRB, 0.34394E+00_JPRB, 0.34080E+00_JPRB, 0.33669E+00_JPRB, 0.33660E+00_JPRB ,&
&  0.33930E+00_JPRB, 0.34203E+00_JPRB, 0.34294E+00_JPRB, 0.34198E+00_JPRB, 0.34002E+00_JPRB, 0.33583E+00_JPRB, 0.32991E+00_JPRB ,&
&  0.32251E+00_JPRB, 0.31384E+00_JPRB, 0.31196E+00_JPRB, 0.31263E+00_JPRB, 0.31640E+00_JPRB, 0.32417E+00_JPRB, 0.33484E+00_JPRB ,&
&  0.34603E+00_JPRB, 0.35520E+00_JPRB, 0.36516E+00_JPRB, 0.37443E+00_JPRB, 0.38273E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.36824E+00_JPRB, 0.36971E+00_JPRB, 0.36713E+00_JPRB, 0.36430E+00_JPRB, 0.36398E+00_JPRB, 0.36551E+00_JPRB, 0.37108E+00_JPRB ,&
&  0.38077E+00_JPRB, 0.39346E+00_JPRB, 0.40563E+00_JPRB, 0.41570E+00_JPRB, 0.42475E+00_JPRB, 0.42871E+00_JPRB, 0.43462E+00_JPRB ,&
&  0.44104E+00_JPRB, 0.44692E+00_JPRB, 0.44767E+00_JPRB, 0.43736E+00_JPRB, 0.42736E+00_JPRB, 0.42204E+00_JPRB, 0.42201E+00_JPRB ,&
&  0.42374E+00_JPRB, 0.42631E+00_JPRB, 0.43238E+00_JPRB, 0.43456E+00_JPRB, 0.42857E+00_JPRB, 0.42702E+00_JPRB, 0.42473E+00_JPRB ,&
&  0.41896E+00_JPRB, 0.40718E+00_JPRB, 0.37461E+00_JPRB, 0.36869E+00_JPRB, 0.36027E+00_JPRB, 0.35113E+00_JPRB, 0.34210E+00_JPRB ,&
&  0.33179E+00_JPRB, 0.32143E+00_JPRB, 0.31437E+00_JPRB, 0.31022E+00_JPRB, 0.31089E+00_JPRB, 0.31211E+00_JPRB, 0.31159E+00_JPRB ,&
&  0.31357E+00_JPRB, 0.31603E+00_JPRB, 0.31819E+00_JPRB, 0.31841E+00_JPRB, 0.31798E+00_JPRB, 0.31692E+00_JPRB, 0.31252E+00_JPRB ,&
&  0.30856E+00_JPRB, 0.30653E+00_JPRB, 0.30500E+00_JPRB, 0.30369E+00_JPRB, 0.30361E+00_JPRB, 0.30412E+00_JPRB, 0.30433E+00_JPRB ,&
&  0.30221E+00_JPRB, 0.29879E+00_JPRB, 0.29495E+00_JPRB, 0.28810E+00_JPRB, 0.27599E+00_JPRB, 0.26109E+00_JPRB, 0.25180E+00_JPRB ,&
&  0.25049E+00_JPRB, 0.26136E+00_JPRB, 0.28268E+00_JPRB, 0.29247E+00_JPRB, 0.29987E+00_JPRB, 0.30369E+00_JPRB, 0.30788E+00_JPRB ,&
&  0.30259E+00_JPRB, 0.27887E+00_JPRB, 0.26558E+00_JPRB, 0.25831E+00_JPRB, 0.25284E+00_JPRB, 0.24879E+00_JPRB, 0.24850E+00_JPRB ,&
&  0.25170E+00_JPRB, 0.25791E+00_JPRB, 0.26429E+00_JPRB, 0.26743E+00_JPRB, 0.26669E+00_JPRB, 0.26353E+00_JPRB, 0.26019E+00_JPRB ,&
&  0.25764E+00_JPRB, 0.26240E+00_JPRB, 0.27714E+00_JPRB, 0.29868E+00_JPRB, 0.30691E+00_JPRB, 0.31330E+00_JPRB, 0.31927E+00_JPRB ,&
&  0.32571E+00_JPRB, 0.33540E+00_JPRB, 0.34706E+00_JPRB, 0.35628E+00_JPRB, 0.36441E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.34127E+00_JPRB, 0.34538E+00_JPRB, 0.34798E+00_JPRB, 0.35137E+00_JPRB, 0.35897E+00_JPRB, 0.36951E+00_JPRB, 0.38157E+00_JPRB ,&
&  0.39286E+00_JPRB, 0.40216E+00_JPRB, 0.40679E+00_JPRB, 0.41198E+00_JPRB, 0.41690E+00_JPRB, 0.41921E+00_JPRB, 0.42429E+00_JPRB ,&
&  0.42498E+00_JPRB, 0.42151E+00_JPRB, 0.41537E+00_JPRB, 0.40727E+00_JPRB, 0.40296E+00_JPRB, 0.40414E+00_JPRB, 0.40694E+00_JPRB ,&
&  0.40871E+00_JPRB, 0.41009E+00_JPRB, 0.41006E+00_JPRB, 0.40654E+00_JPRB, 0.40320E+00_JPRB, 0.40121E+00_JPRB, 0.39701E+00_JPRB ,&
&  0.38730E+00_JPRB, 0.33756E+00_JPRB, 0.30004E+00_JPRB, 0.29599E+00_JPRB, 0.31206E+00_JPRB, 0.31418E+00_JPRB, 0.30914E+00_JPRB ,&
&  0.30347E+00_JPRB, 0.29869E+00_JPRB, 0.29593E+00_JPRB, 0.29377E+00_JPRB, 0.29103E+00_JPRB, 0.28852E+00_JPRB, 0.28906E+00_JPRB ,&
&  0.29023E+00_JPRB, 0.28795E+00_JPRB, 0.28487E+00_JPRB, 0.27853E+00_JPRB, 0.27398E+00_JPRB, 0.27222E+00_JPRB, 0.27079E+00_JPRB ,&
&  0.26661E+00_JPRB, 0.26247E+00_JPRB, 0.26244E+00_JPRB, 0.26370E+00_JPRB, 0.26566E+00_JPRB, 0.26699E+00_JPRB, 0.26433E+00_JPRB ,&
&  0.25876E+00_JPRB, 0.25227E+00_JPRB, 0.24421E+00_JPRB, 0.23449E+00_JPRB, 0.22526E+00_JPRB, 0.21855E+00_JPRB, 0.21326E+00_JPRB ,&
&  0.21256E+00_JPRB, 0.21855E+00_JPRB, 0.23194E+00_JPRB, 0.24754E+00_JPRB, 0.26528E+00_JPRB, 0.25567E+00_JPRB, 0.24501E+00_JPRB ,&
&  0.23420E+00_JPRB, 0.22533E+00_JPRB, 0.21881E+00_JPRB, 0.21583E+00_JPRB, 0.21465E+00_JPRB, 0.21460E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21786E+00_JPRB, 0.22056E+00_JPRB, 0.22287E+00_JPRB, 0.22394E+00_JPRB, 0.22450E+00_JPRB, 0.22495E+00_JPRB, 0.22474E+00_JPRB ,&
&  0.22411E+00_JPRB, 0.22656E+00_JPRB, 0.23343E+00_JPRB, 0.24202E+00_JPRB, 0.25027E+00_JPRB, 0.25812E+00_JPRB, 0.27112E+00_JPRB ,&
&  0.28708E+00_JPRB, 0.30688E+00_JPRB, 0.31956E+00_JPRB, 0.32928E+00_JPRB, 0.33758E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.33435E+00_JPRB, 0.34330E+00_JPRB, 0.35521E+00_JPRB, 0.36556E+00_JPRB, 0.37467E+00_JPRB, 0.38327E+00_JPRB, 0.39090E+00_JPRB ,&
&  0.39509E+00_JPRB, 0.39750E+00_JPRB, 0.39845E+00_JPRB, 0.40200E+00_JPRB, 0.39892E+00_JPRB, 0.39192E+00_JPRB, 0.38632E+00_JPRB ,&
&  0.38165E+00_JPRB, 0.37803E+00_JPRB, 0.37273E+00_JPRB, 0.36992E+00_JPRB, 0.36961E+00_JPRB, 0.36961E+00_JPRB, 0.36389E+00_JPRB ,&
&  0.35328E+00_JPRB, 0.35213E+00_JPRB, 0.35672E+00_JPRB, 0.35121E+00_JPRB, 0.34543E+00_JPRB, 0.33985E+00_JPRB, 0.33388E+00_JPRB ,&
&  0.31774E+00_JPRB, 0.29757E+00_JPRB, 0.27078E+00_JPRB, 0.26705E+00_JPRB, 0.28283E+00_JPRB, 0.27626E+00_JPRB, 0.27954E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28041E+00_JPRB, 0.27893E+00_JPRB, 0.27733E+00_JPRB, 0.27376E+00_JPRB, 0.27253E+00_JPRB, 0.27085E+00_JPRB ,&
&  0.26498E+00_JPRB, 0.25995E+00_JPRB, 0.25190E+00_JPRB, 0.23409E+00_JPRB, 0.22924E+00_JPRB, 0.22777E+00_JPRB, 0.22305E+00_JPRB ,&
&  0.21545E+00_JPRB, 0.21000E+00_JPRB, 0.20792E+00_JPRB, 0.20407E+00_JPRB, 0.20624E+00_JPRB, 0.20824E+00_JPRB, 0.20820E+00_JPRB ,&
&  0.20816E+00_JPRB, 0.20775E+00_JPRB, 0.20544E+00_JPRB, 0.20158E+00_JPRB, 0.19855E+00_JPRB, 0.19760E+00_JPRB, 0.19856E+00_JPRB ,&
&  0.20103E+00_JPRB, 0.20560E+00_JPRB, 0.21665E+00_JPRB, 0.22526E+00_JPRB, 0.22420E+00_JPRB, 0.21991E+00_JPRB, 0.21088E+00_JPRB ,&
&  0.20279E+00_JPRB, 0.19978E+00_JPRB, 0.19715E+00_JPRB, 0.19571E+00_JPRB, 0.19598E+00_JPRB, 0.19740E+00_JPRB, 0.19815E+00_JPRB ,&
&  0.19923E+00_JPRB, 0.20032E+00_JPRB, 0.20115E+00_JPRB, 0.20163E+00_JPRB, 0.20244E+00_JPRB, 0.20338E+00_JPRB, 0.20517E+00_JPRB ,&
&  0.20779E+00_JPRB, 0.21224E+00_JPRB, 0.21803E+00_JPRB, 0.22428E+00_JPRB, 0.22884E+00_JPRB, 0.23111E+00_JPRB, 0.23529E+00_JPRB ,&
&  0.24121E+00_JPRB, 0.25648E+00_JPRB, 0.28517E+00_JPRB, 0.32995E+00_JPRB, 0.33361E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.33418E+00_JPRB, 0.34632E+00_JPRB, 0.36159E+00_JPRB, 0.37114E+00_JPRB, 0.37665E+00_JPRB, 0.37945E+00_JPRB, 0.37850E+00_JPRB ,&
&  0.37515E+00_JPRB, 0.37458E+00_JPRB, 0.38562E+00_JPRB, 0.36747E+00_JPRB, 0.34993E+00_JPRB, 0.31688E+00_JPRB, 0.30035E+00_JPRB ,&
&  0.28782E+00_JPRB, 0.28561E+00_JPRB, 0.28523E+00_JPRB, 0.28987E+00_JPRB, 0.29362E+00_JPRB, 0.29655E+00_JPRB, 0.29269E+00_JPRB ,&
&  0.28724E+00_JPRB, 0.27796E+00_JPRB, 0.27018E+00_JPRB, 0.26293E+00_JPRB, 0.26306E+00_JPRB, 0.26191E+00_JPRB, 0.25929E+00_JPRB ,&
&  0.25634E+00_JPRB, 0.25452E+00_JPRB, 0.25449E+00_JPRB, 0.25491E+00_JPRB, 0.26649E+00_JPRB, 0.26556E+00_JPRB, 0.26578E+00_JPRB ,&
&  0.26550E+00_JPRB, 0.26545E+00_JPRB, 0.26444E+00_JPRB, 0.26266E+00_JPRB, 0.25781E+00_JPRB, 0.24410E+00_JPRB, 0.24317E+00_JPRB ,&
&  0.23416E+00_JPRB, 0.22363E+00_JPRB, 0.21339E+00_JPRB, 0.20627E+00_JPRB, 0.20194E+00_JPRB, 0.20016E+00_JPRB, 0.19552E+00_JPRB ,&
&  0.19055E+00_JPRB, 0.18477E+00_JPRB, 0.18214E+00_JPRB, 0.18204E+00_JPRB, 0.18267E+00_JPRB, 0.18327E+00_JPRB, 0.18376E+00_JPRB ,&
&  0.18507E+00_JPRB, 0.18569E+00_JPRB, 0.18609E+00_JPRB, 0.18684E+00_JPRB, 0.18759E+00_JPRB, 0.18870E+00_JPRB, 0.19055E+00_JPRB ,&
&  0.19381E+00_JPRB, 0.19729E+00_JPRB, 0.20053E+00_JPRB, 0.20337E+00_JPRB, 0.20237E+00_JPRB, 0.19886E+00_JPRB, 0.19424E+00_JPRB ,&
&  0.19084E+00_JPRB, 0.18966E+00_JPRB, 0.18855E+00_JPRB, 0.18841E+00_JPRB, 0.18853E+00_JPRB, 0.18892E+00_JPRB, 0.18952E+00_JPRB ,&
&  0.19036E+00_JPRB, 0.19107E+00_JPRB, 0.19194E+00_JPRB, 0.19306E+00_JPRB, 0.19464E+00_JPRB, 0.19606E+00_JPRB, 0.19805E+00_JPRB ,&
&  0.20102E+00_JPRB, 0.20430E+00_JPRB, 0.20896E+00_JPRB, 0.21321E+00_JPRB, 0.21906E+00_JPRB, 0.22733E+00_JPRB, 0.24086E+00_JPRB ,&
&  0.25532E+00_JPRB, 0.28931E+00_JPRB, 0.30479E+00_JPRB, 0.31626E+00_JPRB, 0.32393E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.33445E+00_JPRB, 0.34413E+00_JPRB, 0.35821E+00_JPRB, 0.36737E+00_JPRB, 0.37460E+00_JPRB, 0.37048E+00_JPRB, 0.36655E+00_JPRB ,&
&  0.36241E+00_JPRB, 0.34849E+00_JPRB, 0.33130E+00_JPRB, 0.30215E+00_JPRB, 0.27762E+00_JPRB, 0.25888E+00_JPRB, 0.24833E+00_JPRB ,&
&  0.24189E+00_JPRB, 0.23967E+00_JPRB, 0.23746E+00_JPRB, 0.23680E+00_JPRB, 0.23537E+00_JPRB, 0.23680E+00_JPRB, 0.23797E+00_JPRB ,&
&  0.23743E+00_JPRB, 0.23639E+00_JPRB, 0.23587E+00_JPRB, 0.23901E+00_JPRB, 0.24513E+00_JPRB, 0.24991E+00_JPRB, 0.25453E+00_JPRB ,&
&  0.25780E+00_JPRB, 0.25510E+00_JPRB, 0.25643E+00_JPRB, 0.25454E+00_JPRB, 0.25277E+00_JPRB, 0.25325E+00_JPRB, 0.25446E+00_JPRB ,&
&  0.25487E+00_JPRB, 0.25516E+00_JPRB, 0.25429E+00_JPRB, 0.25102E+00_JPRB, 0.23901E+00_JPRB, 0.23284E+00_JPRB, 0.22488E+00_JPRB ,&
&  0.21685E+00_JPRB, 0.20745E+00_JPRB, 0.19978E+00_JPRB, 0.19614E+00_JPRB, 0.19164E+00_JPRB, 0.18995E+00_JPRB, 0.18673E+00_JPRB ,&
&  0.18308E+00_JPRB, 0.17964E+00_JPRB, 0.17746E+00_JPRB, 0.17638E+00_JPRB, 0.17602E+00_JPRB, 0.17486E+00_JPRB, 0.17448E+00_JPRB ,&
&  0.17482E+00_JPRB, 0.17518E+00_JPRB, 0.17610E+00_JPRB, 0.17698E+00_JPRB, 0.17792E+00_JPRB, 0.17876E+00_JPRB, 0.17975E+00_JPRB ,&
&  0.18140E+00_JPRB, 0.18344E+00_JPRB, 0.18435E+00_JPRB, 0.18514E+00_JPRB, 0.18483E+00_JPRB, 0.18426E+00_JPRB, 0.18341E+00_JPRB ,&
&  0.18273E+00_JPRB, 0.18251E+00_JPRB, 0.18250E+00_JPRB, 0.18241E+00_JPRB, 0.18287E+00_JPRB, 0.18376E+00_JPRB, 0.18467E+00_JPRB ,&
&  0.18623E+00_JPRB, 0.18763E+00_JPRB, 0.18924E+00_JPRB, 0.19117E+00_JPRB, 0.19335E+00_JPRB, 0.19595E+00_JPRB, 0.19886E+00_JPRB ,&
&  0.20143E+00_JPRB, 0.20475E+00_JPRB, 0.20759E+00_JPRB, 0.20989E+00_JPRB, 0.21546E+00_JPRB, 0.22275E+00_JPRB, 0.23735E+00_JPRB ,&
&  0.25384E+00_JPRB, 0.29331E+00_JPRB, 0.33512E+00_JPRB, 0.32103E+00_JPRB, 0.32628E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.30738E+00_JPRB, 0.31838E+00_JPRB, 0.32713E+00_JPRB, 0.33262E+00_JPRB, 0.33814E+00_JPRB, 0.34022E+00_JPRB, 0.34821E+00_JPRB ,&
&  0.34277E+00_JPRB, 0.32480E+00_JPRB, 0.31365E+00_JPRB, 0.29718E+00_JPRB, 0.27691E+00_JPRB, 0.26027E+00_JPRB, 0.25181E+00_JPRB ,&
&  0.24675E+00_JPRB, 0.24266E+00_JPRB, 0.23778E+00_JPRB, 0.23244E+00_JPRB, 0.22709E+00_JPRB, 0.22310E+00_JPRB, 0.22057E+00_JPRB ,&
&  0.21826E+00_JPRB, 0.21898E+00_JPRB, 0.22171E+00_JPRB, 0.22616E+00_JPRB, 0.23252E+00_JPRB, 0.23678E+00_JPRB, 0.24015E+00_JPRB ,&
&  0.24204E+00_JPRB, 0.23280E+00_JPRB, 0.23481E+00_JPRB, 0.23704E+00_JPRB, 0.23586E+00_JPRB, 0.23928E+00_JPRB, 0.24476E+00_JPRB ,&
&  0.24205E+00_JPRB, 0.23451E+00_JPRB, 0.23695E+00_JPRB, 0.23158E+00_JPRB, 0.22308E+00_JPRB, 0.22139E+00_JPRB, 0.21555E+00_JPRB ,&
&  0.21289E+00_JPRB, 0.20573E+00_JPRB, 0.20060E+00_JPRB, 0.19721E+00_JPRB, 0.19141E+00_JPRB, 0.18904E+00_JPRB, 0.18552E+00_JPRB ,&
&  0.18195E+00_JPRB, 0.17951E+00_JPRB, 0.17661E+00_JPRB, 0.17525E+00_JPRB, 0.17407E+00_JPRB, 0.17279E+00_JPRB, 0.17179E+00_JPRB ,&
&  0.17122E+00_JPRB, 0.17069E+00_JPRB, 0.17071E+00_JPRB, 0.17072E+00_JPRB, 0.17067E+00_JPRB, 0.17098E+00_JPRB, 0.17114E+00_JPRB ,&
&  0.17175E+00_JPRB, 0.17249E+00_JPRB, 0.17308E+00_JPRB, 0.17406E+00_JPRB, 0.17447E+00_JPRB, 0.17503E+00_JPRB, 0.17543E+00_JPRB ,&
&  0.17587E+00_JPRB, 0.17653E+00_JPRB, 0.17684E+00_JPRB, 0.17751E+00_JPRB, 0.17826E+00_JPRB, 0.17975E+00_JPRB, 0.18123E+00_JPRB ,&
&  0.18305E+00_JPRB, 0.18518E+00_JPRB, 0.18744E+00_JPRB, 0.18984E+00_JPRB, 0.19282E+00_JPRB, 0.19554E+00_JPRB, 0.19914E+00_JPRB ,&
&  0.20303E+00_JPRB, 0.20675E+00_JPRB, 0.21119E+00_JPRB, 0.21676E+00_JPRB, 0.22414E+00_JPRB, 0.23389E+00_JPRB, 0.24274E+00_JPRB ,&
&  0.25286E+00_JPRB, 0.27871E+00_JPRB, 0.28513E+00_JPRB, 0.28988E+00_JPRB, 0.29864E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.29888E+00_JPRB, 0.30386E+00_JPRB, 0.30770E+00_JPRB, 0.31017E+00_JPRB, 0.31256E+00_JPRB, 0.31421E+00_JPRB, 0.31483E+00_JPRB ,&
&  0.30835E+00_JPRB, 0.30340E+00_JPRB, 0.29640E+00_JPRB, 0.28794E+00_JPRB, 0.28021E+00_JPRB, 0.27286E+00_JPRB, 0.26468E+00_JPRB ,&
&  0.25796E+00_JPRB, 0.25163E+00_JPRB, 0.24508E+00_JPRB, 0.23819E+00_JPRB, 0.23099E+00_JPRB, 0.22373E+00_JPRB, 0.22041E+00_JPRB ,&
&  0.21954E+00_JPRB, 0.22082E+00_JPRB, 0.22303E+00_JPRB, 0.22457E+00_JPRB, 0.22548E+00_JPRB, 0.22602E+00_JPRB, 0.22484E+00_JPRB ,&
&  0.22106E+00_JPRB, 0.22182E+00_JPRB, 0.22289E+00_JPRB, 0.22359E+00_JPRB, 0.22525E+00_JPRB, 0.22664E+00_JPRB, 0.22647E+00_JPRB ,&
&  0.22457E+00_JPRB, 0.22205E+00_JPRB, 0.21862E+00_JPRB, 0.21542E+00_JPRB, 0.21425E+00_JPRB, 0.21211E+00_JPRB, 0.20966E+00_JPRB ,&
&  0.20782E+00_JPRB, 0.20393E+00_JPRB, 0.19986E+00_JPRB, 0.19698E+00_JPRB, 0.19314E+00_JPRB, 0.19054E+00_JPRB, 0.18714E+00_JPRB ,&
&  0.18441E+00_JPRB, 0.18197E+00_JPRB, 0.17929E+00_JPRB, 0.17723E+00_JPRB, 0.17531E+00_JPRB, 0.17389E+00_JPRB, 0.17247E+00_JPRB ,&
&  0.17164E+00_JPRB, 0.17084E+00_JPRB, 0.17033E+00_JPRB, 0.16975E+00_JPRB, 0.16939E+00_JPRB, 0.16903E+00_JPRB, 0.16874E+00_JPRB ,&
&  0.16858E+00_JPRB, 0.16890E+00_JPRB, 0.16912E+00_JPRB, 0.16952E+00_JPRB, 0.16999E+00_JPRB, 0.17026E+00_JPRB, 0.17079E+00_JPRB ,&
&  0.17183E+00_JPRB, 0.17296E+00_JPRB, 0.17368E+00_JPRB, 0.17463E+00_JPRB, 0.17571E+00_JPRB, 0.17702E+00_JPRB, 0.17882E+00_JPRB ,&
&  0.18071E+00_JPRB, 0.18283E+00_JPRB, 0.18522E+00_JPRB, 0.18758E+00_JPRB, 0.19064E+00_JPRB, 0.19377E+00_JPRB, 0.19708E+00_JPRB ,&
&  0.20103E+00_JPRB, 0.20523E+00_JPRB, 0.20957E+00_JPRB, 0.21436E+00_JPRB, 0.22124E+00_JPRB, 0.22723E+00_JPRB, 0.23464E+00_JPRB ,&
&  0.24181E+00_JPRB, 0.24705E+00_JPRB, 0.27945E+00_JPRB, 0.28427E+00_JPRB, 0.29107E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.22474E+00_JPRB, 0.23140E+00_JPRB, 0.23801E+00_JPRB, 0.24741E+00_JPRB, 0.25346E+00_JPRB, 0.28921E+00_JPRB, 0.28416E+00_JPRB ,&
&  0.28098E+00_JPRB, 0.26775E+00_JPRB, 0.26642E+00_JPRB, 0.26288E+00_JPRB, 0.25838E+00_JPRB, 0.25352E+00_JPRB, 0.24858E+00_JPRB ,&
&  0.24396E+00_JPRB, 0.23851E+00_JPRB, 0.23391E+00_JPRB, 0.23026E+00_JPRB, 0.22700E+00_JPRB, 0.22380E+00_JPRB, 0.22102E+00_JPRB ,&
&  0.21985E+00_JPRB, 0.21974E+00_JPRB, 0.21898E+00_JPRB, 0.21472E+00_JPRB, 0.21122E+00_JPRB, 0.20909E+00_JPRB, 0.20847E+00_JPRB ,&
&  0.20843E+00_JPRB, 0.21022E+00_JPRB, 0.21140E+00_JPRB, 0.21237E+00_JPRB, 0.21378E+00_JPRB, 0.21291E+00_JPRB, 0.21204E+00_JPRB ,&
&  0.21079E+00_JPRB, 0.20943E+00_JPRB, 0.20800E+00_JPRB, 0.20676E+00_JPRB, 0.20552E+00_JPRB, 0.20369E+00_JPRB, 0.20212E+00_JPRB ,&
&  0.20060E+00_JPRB, 0.19860E+00_JPRB, 0.19586E+00_JPRB, 0.19404E+00_JPRB, 0.19190E+00_JPRB, 0.19009E+00_JPRB, 0.18826E+00_JPRB ,&
&  0.18689E+00_JPRB, 0.18565E+00_JPRB, 0.18421E+00_JPRB, 0.18247E+00_JPRB, 0.18074E+00_JPRB, 0.17922E+00_JPRB, 0.17778E+00_JPRB ,&
&  0.17700E+00_JPRB, 0.17619E+00_JPRB, 0.17538E+00_JPRB, 0.17452E+00_JPRB, 0.17409E+00_JPRB, 0.17364E+00_JPRB, 0.17323E+00_JPRB ,&
&  0.17288E+00_JPRB, 0.17268E+00_JPRB, 0.17269E+00_JPRB, 0.17276E+00_JPRB, 0.17285E+00_JPRB, 0.17321E+00_JPRB, 0.17366E+00_JPRB ,&
&  0.17426E+00_JPRB, 0.17501E+00_JPRB, 0.17606E+00_JPRB, 0.17714E+00_JPRB, 0.17810E+00_JPRB, 0.17888E+00_JPRB, 0.17988E+00_JPRB ,&
&  0.18150E+00_JPRB, 0.18334E+00_JPRB, 0.18536E+00_JPRB, 0.18690E+00_JPRB, 0.18912E+00_JPRB, 0.19112E+00_JPRB, 0.19312E+00_JPRB ,&
&  0.19510E+00_JPRB, 0.19749E+00_JPRB, 0.20017E+00_JPRB, 0.20305E+00_JPRB, 0.20639E+00_JPRB, 0.20920E+00_JPRB, 0.21183E+00_JPRB ,&
&  0.21450E+00_JPRB, 0.21834E+00_JPRB, 0.22032E+00_JPRB, 0.22142E+00_JPRB, 0.22281E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.20362E+00_JPRB, 0.20373E+00_JPRB, 0.20414E+00_JPRB, 0.20460E+00_JPRB, 0.20503E+00_JPRB, 0.20572E+00_JPRB, 0.20590E+00_JPRB ,&
&  0.20589E+00_JPRB, 0.20578E+00_JPRB, 0.20613E+00_JPRB, 0.20653E+00_JPRB, 0.20677E+00_JPRB, 0.20678E+00_JPRB, 0.20638E+00_JPRB ,&
&  0.20569E+00_JPRB, 0.20486E+00_JPRB, 0.20356E+00_JPRB, 0.20298E+00_JPRB, 0.20256E+00_JPRB, 0.20216E+00_JPRB, 0.20139E+00_JPRB ,&
&  0.20070E+00_JPRB, 0.19950E+00_JPRB, 0.19785E+00_JPRB, 0.19660E+00_JPRB, 0.19641E+00_JPRB, 0.19672E+00_JPRB, 0.19736E+00_JPRB ,&
&  0.19787E+00_JPRB, 0.19820E+00_JPRB, 0.19832E+00_JPRB, 0.19851E+00_JPRB, 0.19878E+00_JPRB, 0.19863E+00_JPRB, 0.19837E+00_JPRB ,&
&  0.19803E+00_JPRB, 0.19748E+00_JPRB, 0.19695E+00_JPRB, 0.19648E+00_JPRB, 0.19605E+00_JPRB, 0.19554E+00_JPRB, 0.19493E+00_JPRB ,&
&  0.19423E+00_JPRB, 0.19337E+00_JPRB, 0.19234E+00_JPRB, 0.19151E+00_JPRB, 0.19064E+00_JPRB, 0.18995E+00_JPRB, 0.18910E+00_JPRB ,&
&  0.18839E+00_JPRB, 0.18774E+00_JPRB, 0.18708E+00_JPRB, 0.18647E+00_JPRB, 0.18590E+00_JPRB, 0.18534E+00_JPRB, 0.18480E+00_JPRB ,&
&  0.18428E+00_JPRB, 0.18386E+00_JPRB, 0.18338E+00_JPRB, 0.18294E+00_JPRB, 0.18263E+00_JPRB, 0.18238E+00_JPRB, 0.18216E+00_JPRB ,&
&  0.18197E+00_JPRB, 0.18191E+00_JPRB, 0.18190E+00_JPRB, 0.18187E+00_JPRB, 0.18187E+00_JPRB, 0.18198E+00_JPRB, 0.18210E+00_JPRB ,&
&  0.18226E+00_JPRB, 0.18238E+00_JPRB, 0.18251E+00_JPRB, 0.18270E+00_JPRB, 0.18291E+00_JPRB, 0.18315E+00_JPRB, 0.18348E+00_JPRB ,&
&  0.18388E+00_JPRB, 0.18435E+00_JPRB, 0.18485E+00_JPRB, 0.18516E+00_JPRB, 0.18584E+00_JPRB, 0.18660E+00_JPRB, 0.18749E+00_JPRB ,&
&  0.18847E+00_JPRB, 0.18964E+00_JPRB, 0.19093E+00_JPRB, 0.19244E+00_JPRB, 0.19400E+00_JPRB, 0.19499E+00_JPRB, 0.19598E+00_JPRB ,&
&  0.19701E+00_JPRB, 0.19822E+00_JPRB, 0.19937E+00_JPRB, 0.20055E+00_JPRB, 0.20196E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB, 0.18859E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB, 0.39729E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.40502E+00_JPRB, 0.40467E+00_JPRB, 0.40433E+00_JPRB, 0.40394E+00_JPRB, 0.40345E+00_JPRB, 0.40294E+00_JPRB, 0.40251E+00_JPRB ,&
&  0.40208E+00_JPRB, 0.40177E+00_JPRB, 0.40124E+00_JPRB, 0.40071E+00_JPRB, 0.40017E+00_JPRB, 0.39969E+00_JPRB, 0.39917E+00_JPRB ,&
&  0.39865E+00_JPRB, 0.39812E+00_JPRB, 0.39773E+00_JPRB, 0.39725E+00_JPRB, 0.39673E+00_JPRB, 0.39625E+00_JPRB, 0.39583E+00_JPRB ,&
&  0.39543E+00_JPRB, 0.39497E+00_JPRB, 0.39449E+00_JPRB, 0.39391E+00_JPRB, 0.39353E+00_JPRB, 0.39299E+00_JPRB, 0.39254E+00_JPRB ,&
&  0.39203E+00_JPRB, 0.39167E+00_JPRB, 0.39127E+00_JPRB, 0.39089E+00_JPRB, 0.39045E+00_JPRB, 0.39037E+00_JPRB, 0.39010E+00_JPRB ,&
&  0.38996E+00_JPRB, 0.38982E+00_JPRB, 0.38982E+00_JPRB, 0.38977E+00_JPRB, 0.38977E+00_JPRB, 0.38981E+00_JPRB, 0.39010E+00_JPRB ,&
&  0.39034E+00_JPRB, 0.39063E+00_JPRB, 0.39109E+00_JPRB, 0.39150E+00_JPRB, 0.39187E+00_JPRB, 0.39226E+00_JPRB, 0.39270E+00_JPRB ,&
&  0.39331E+00_JPRB, 0.39390E+00_JPRB, 0.39446E+00_JPRB, 0.39504E+00_JPRB, 0.39560E+00_JPRB, 0.39604E+00_JPRB, 0.39642E+00_JPRB ,&
&  0.39662E+00_JPRB, 0.39691E+00_JPRB, 0.39718E+00_JPRB, 0.39748E+00_JPRB, 0.39755E+00_JPRB, 0.39774E+00_JPRB, 0.39783E+00_JPRB ,&
&  0.39790E+00_JPRB, 0.39771E+00_JPRB, 0.39775E+00_JPRB, 0.39768E+00_JPRB, 0.39773E+00_JPRB, 0.39770E+00_JPRB, 0.39792E+00_JPRB ,&
&  0.39808E+00_JPRB, 0.39845E+00_JPRB, 0.39884E+00_JPRB, 0.39940E+00_JPRB, 0.39981E+00_JPRB, 0.40029E+00_JPRB, 0.40072E+00_JPRB ,&
&  0.40119E+00_JPRB, 0.40169E+00_JPRB, 0.40224E+00_JPRB, 0.40262E+00_JPRB, 0.40296E+00_JPRB, 0.40336E+00_JPRB, 0.40379E+00_JPRB ,&
&  0.40435E+00_JPRB, 0.40470E+00_JPRB, 0.40507E+00_JPRB, 0.40538E+00_JPRB, 0.40550E+00_JPRB, 0.40562E+00_JPRB, 0.40563E+00_JPRB ,&
&  0.40558E+00_JPRB, 0.40552E+00_JPRB, 0.40542E+00_JPRB, 0.40535E+00_JPRB, 0.40525E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.41143E+00_JPRB, 0.41125E+00_JPRB, 0.41091E+00_JPRB, 0.41034E+00_JPRB, 0.40964E+00_JPRB, 0.40879E+00_JPRB, 0.40796E+00_JPRB ,&
&  0.40701E+00_JPRB, 0.40609E+00_JPRB, 0.40497E+00_JPRB, 0.40387E+00_JPRB, 0.40274E+00_JPRB, 0.40169E+00_JPRB, 0.40059E+00_JPRB ,&
&  0.39956E+00_JPRB, 0.39852E+00_JPRB, 0.39782E+00_JPRB, 0.39708E+00_JPRB, 0.39625E+00_JPRB, 0.39523E+00_JPRB, 0.39437E+00_JPRB ,&
&  0.39347E+00_JPRB, 0.39238E+00_JPRB, 0.39118E+00_JPRB, 0.38905E+00_JPRB, 0.38724E+00_JPRB, 0.38557E+00_JPRB, 0.38414E+00_JPRB ,&
&  0.38284E+00_JPRB, 0.38162E+00_JPRB, 0.38067E+00_JPRB, 0.38006E+00_JPRB, 0.38073E+00_JPRB, 0.38084E+00_JPRB, 0.38063E+00_JPRB ,&
&  0.38028E+00_JPRB, 0.37897E+00_JPRB, 0.37854E+00_JPRB, 0.37840E+00_JPRB, 0.37840E+00_JPRB, 0.37927E+00_JPRB, 0.37998E+00_JPRB ,&
&  0.38075E+00_JPRB, 0.38138E+00_JPRB, 0.38265E+00_JPRB, 0.38386E+00_JPRB, 0.38498E+00_JPRB, 0.38620E+00_JPRB, 0.38690E+00_JPRB ,&
&  0.38769E+00_JPRB, 0.38887E+00_JPRB, 0.39001E+00_JPRB, 0.39121E+00_JPRB, 0.39192E+00_JPRB, 0.39259E+00_JPRB, 0.39309E+00_JPRB ,&
&  0.39352E+00_JPRB, 0.39410E+00_JPRB, 0.39475E+00_JPRB, 0.39537E+00_JPRB, 0.39580E+00_JPRB, 0.39615E+00_JPRB, 0.39636E+00_JPRB ,&
&  0.39643E+00_JPRB, 0.39611E+00_JPRB, 0.39595E+00_JPRB, 0.39575E+00_JPRB, 0.39539E+00_JPRB, 0.39498E+00_JPRB, 0.39484E+00_JPRB ,&
&  0.39470E+00_JPRB, 0.39443E+00_JPRB, 0.39401E+00_JPRB, 0.39423E+00_JPRB, 0.39451E+00_JPRB, 0.39464E+00_JPRB, 0.39465E+00_JPRB ,&
&  0.39596E+00_JPRB, 0.39780E+00_JPRB, 0.39994E+00_JPRB, 0.40278E+00_JPRB, 0.40491E+00_JPRB, 0.40627E+00_JPRB, 0.40722E+00_JPRB ,&
&  0.40761E+00_JPRB, 0.40800E+00_JPRB, 0.40838E+00_JPRB, 0.40899E+00_JPRB, 0.40942E+00_JPRB, 0.41018E+00_JPRB, 0.41117E+00_JPRB ,&
&  0.41186E+00_JPRB, 0.41207E+00_JPRB, 0.41205E+00_JPRB, 0.41188E+00_JPRB, 0.41165E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.41383E+00_JPRB, 0.41304E+00_JPRB, 0.41227E+00_JPRB, 0.41105E+00_JPRB, 0.41030E+00_JPRB, 0.40923E+00_JPRB, 0.40827E+00_JPRB ,&
&  0.40711E+00_JPRB, 0.40579E+00_JPRB, 0.40480E+00_JPRB, 0.40401E+00_JPRB, 0.40316E+00_JPRB, 0.40236E+00_JPRB, 0.40146E+00_JPRB ,&
&  0.40053E+00_JPRB, 0.39945E+00_JPRB, 0.39901E+00_JPRB, 0.39830E+00_JPRB, 0.39748E+00_JPRB, 0.39655E+00_JPRB, 0.39415E+00_JPRB ,&
&  0.39183E+00_JPRB, 0.38898E+00_JPRB, 0.38605E+00_JPRB, 0.38355E+00_JPRB, 0.38194E+00_JPRB, 0.37050E+00_JPRB, 0.36861E+00_JPRB ,&
&  0.37309E+00_JPRB, 0.37556E+00_JPRB, 0.37841E+00_JPRB, 0.38139E+00_JPRB, 0.37554E+00_JPRB, 0.36818E+00_JPRB, 0.36388E+00_JPRB ,&
&  0.35333E+00_JPRB, 0.35741E+00_JPRB, 0.35947E+00_JPRB, 0.36142E+00_JPRB, 0.36440E+00_JPRB, 0.36449E+00_JPRB, 0.36643E+00_JPRB ,&
&  0.37345E+00_JPRB, 0.37348E+00_JPRB, 0.37438E+00_JPRB, 0.37539E+00_JPRB, 0.37637E+00_JPRB, 0.37735E+00_JPRB, 0.37922E+00_JPRB ,&
&  0.38069E+00_JPRB, 0.38241E+00_JPRB, 0.38430E+00_JPRB, 0.38520E+00_JPRB, 0.38617E+00_JPRB, 0.38748E+00_JPRB, 0.38872E+00_JPRB ,&
&  0.38934E+00_JPRB, 0.39015E+00_JPRB, 0.39104E+00_JPRB, 0.39186E+00_JPRB, 0.39298E+00_JPRB, 0.39369E+00_JPRB, 0.39424E+00_JPRB ,&
&  0.39463E+00_JPRB, 0.39448E+00_JPRB, 0.39438E+00_JPRB, 0.39446E+00_JPRB, 0.39432E+00_JPRB, 0.39407E+00_JPRB, 0.39416E+00_JPRB ,&
&  0.39452E+00_JPRB, 0.39478E+00_JPRB, 0.39534E+00_JPRB, 0.39624E+00_JPRB, 0.39847E+00_JPRB, 0.40155E+00_JPRB, 0.40766E+00_JPRB ,&
&  0.41265E+00_JPRB, 0.41695E+00_JPRB, 0.41850E+00_JPRB, 0.42004E+00_JPRB, 0.42117E+00_JPRB, 0.42135E+00_JPRB, 0.42123E+00_JPRB ,&
&  0.42041E+00_JPRB, 0.41903E+00_JPRB, 0.41766E+00_JPRB, 0.41632E+00_JPRB, 0.41555E+00_JPRB, 0.41500E+00_JPRB, 0.41503E+00_JPRB ,&
&  0.41549E+00_JPRB, 0.41624E+00_JPRB, 0.41641E+00_JPRB, 0.41559E+00_JPRB, 0.41459E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.41922E+00_JPRB, 0.41733E+00_JPRB, 0.41660E+00_JPRB, 0.41500E+00_JPRB, 0.41336E+00_JPRB, 0.41130E+00_JPRB, 0.40963E+00_JPRB ,&
&  0.40778E+00_JPRB, 0.40617E+00_JPRB, 0.40483E+00_JPRB, 0.40363E+00_JPRB, 0.40308E+00_JPRB, 0.40244E+00_JPRB, 0.40206E+00_JPRB ,&
&  0.40151E+00_JPRB, 0.40073E+00_JPRB, 0.40033E+00_JPRB, 0.39968E+00_JPRB, 0.39900E+00_JPRB, 0.39818E+00_JPRB, 0.39617E+00_JPRB ,&
&  0.39267E+00_JPRB, 0.38792E+00_JPRB, 0.37743E+00_JPRB, 0.37920E+00_JPRB, 0.37781E+00_JPRB, 0.37644E+00_JPRB, 0.37693E+00_JPRB ,&
&  0.37175E+00_JPRB, 0.37034E+00_JPRB, 0.36456E+00_JPRB, 0.35926E+00_JPRB, 0.35722E+00_JPRB, 0.35185E+00_JPRB, 0.35168E+00_JPRB ,&
&  0.34973E+00_JPRB, 0.34876E+00_JPRB, 0.34939E+00_JPRB, 0.34914E+00_JPRB, 0.35234E+00_JPRB, 0.35401E+00_JPRB, 0.35652E+00_JPRB ,&
&  0.36088E+00_JPRB, 0.36442E+00_JPRB, 0.36686E+00_JPRB, 0.36328E+00_JPRB, 0.36513E+00_JPRB, 0.36697E+00_JPRB, 0.36930E+00_JPRB ,&
&  0.37145E+00_JPRB, 0.37292E+00_JPRB, 0.37438E+00_JPRB, 0.37545E+00_JPRB, 0.37685E+00_JPRB, 0.37893E+00_JPRB, 0.38113E+00_JPRB ,&
&  0.38374E+00_JPRB, 0.38557E+00_JPRB, 0.38677E+00_JPRB, 0.38797E+00_JPRB, 0.38927E+00_JPRB, 0.39047E+00_JPRB, 0.39195E+00_JPRB ,&
&  0.39368E+00_JPRB, 0.39588E+00_JPRB, 0.39822E+00_JPRB, 0.40068E+00_JPRB, 0.40295E+00_JPRB, 0.40598E+00_JPRB, 0.40955E+00_JPRB ,&
&  0.41488E+00_JPRB, 0.41981E+00_JPRB, 0.42748E+00_JPRB, 0.43218E+00_JPRB, 0.44187E+00_JPRB, 0.45131E+00_JPRB, 0.45721E+00_JPRB ,&
&  0.45621E+00_JPRB, 0.45341E+00_JPRB, 0.45040E+00_JPRB, 0.44700E+00_JPRB, 0.44409E+00_JPRB, 0.44068E+00_JPRB, 0.43832E+00_JPRB ,&
&  0.43610E+00_JPRB, 0.43392E+00_JPRB, 0.43176E+00_JPRB, 0.42955E+00_JPRB, 0.42768E+00_JPRB, 0.42558E+00_JPRB, 0.42416E+00_JPRB ,&
&  0.42356E+00_JPRB, 0.42378E+00_JPRB, 0.42366E+00_JPRB, 0.42282E+00_JPRB, 0.42117E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.42533E+00_JPRB, 0.42485E+00_JPRB, 0.42556E+00_JPRB, 0.42474E+00_JPRB, 0.42282E+00_JPRB, 0.42092E+00_JPRB, 0.41726E+00_JPRB ,&
&  0.41368E+00_JPRB, 0.41021E+00_JPRB, 0.40720E+00_JPRB, 0.40501E+00_JPRB, 0.40325E+00_JPRB, 0.40183E+00_JPRB, 0.40131E+00_JPRB ,&
&  0.40047E+00_JPRB, 0.40046E+00_JPRB, 0.40050E+00_JPRB, 0.40065E+00_JPRB, 0.40127E+00_JPRB, 0.40150E+00_JPRB, 0.40149E+00_JPRB ,&
&  0.39923E+00_JPRB, 0.39349E+00_JPRB, 0.38575E+00_JPRB, 0.38714E+00_JPRB, 0.38795E+00_JPRB, 0.38951E+00_JPRB, 0.39123E+00_JPRB ,&
&  0.38997E+00_JPRB, 0.37437E+00_JPRB, 0.37908E+00_JPRB, 0.37174E+00_JPRB, 0.36773E+00_JPRB, 0.36319E+00_JPRB, 0.35197E+00_JPRB ,&
&  0.34803E+00_JPRB, 0.34137E+00_JPRB, 0.33765E+00_JPRB, 0.33770E+00_JPRB, 0.33930E+00_JPRB, 0.34368E+00_JPRB, 0.34918E+00_JPRB ,&
&  0.35020E+00_JPRB, 0.35850E+00_JPRB, 0.35962E+00_JPRB, 0.35944E+00_JPRB, 0.36147E+00_JPRB, 0.36380E+00_JPRB, 0.36294E+00_JPRB ,&
&  0.36383E+00_JPRB, 0.36359E+00_JPRB, 0.36304E+00_JPRB, 0.36500E+00_JPRB, 0.36951E+00_JPRB, 0.37182E+00_JPRB, 0.37419E+00_JPRB ,&
&  0.37737E+00_JPRB, 0.37927E+00_JPRB, 0.38046E+00_JPRB, 0.38211E+00_JPRB, 0.38430E+00_JPRB, 0.38648E+00_JPRB, 0.38979E+00_JPRB ,&
&  0.39350E+00_JPRB, 0.39985E+00_JPRB, 0.40578E+00_JPRB, 0.41460E+00_JPRB, 0.42289E+00_JPRB, 0.43394E+00_JPRB, 0.44369E+00_JPRB ,&
&  0.45388E+00_JPRB, 0.46216E+00_JPRB, 0.47162E+00_JPRB, 0.48055E+00_JPRB, 0.48817E+00_JPRB, 0.48641E+00_JPRB, 0.47815E+00_JPRB ,&
&  0.47106E+00_JPRB, 0.46091E+00_JPRB, 0.45498E+00_JPRB, 0.45029E+00_JPRB, 0.44710E+00_JPRB, 0.44570E+00_JPRB, 0.44420E+00_JPRB ,&
&  0.44360E+00_JPRB, 0.44224E+00_JPRB, 0.44045E+00_JPRB, 0.43785E+00_JPRB, 0.43566E+00_JPRB, 0.43328E+00_JPRB, 0.43166E+00_JPRB ,&
&  0.42938E+00_JPRB, 0.42851E+00_JPRB, 0.42720E+00_JPRB, 0.42640E+00_JPRB, 0.42578E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.43140E+00_JPRB, 0.43357E+00_JPRB, 0.43308E+00_JPRB, 0.43184E+00_JPRB, 0.42784E+00_JPRB, 0.42446E+00_JPRB, 0.41918E+00_JPRB ,&
&  0.41417E+00_JPRB, 0.40919E+00_JPRB, 0.40476E+00_JPRB, 0.40184E+00_JPRB, 0.39971E+00_JPRB, 0.39826E+00_JPRB, 0.39765E+00_JPRB ,&
&  0.39758E+00_JPRB, 0.39811E+00_JPRB, 0.39863E+00_JPRB, 0.39922E+00_JPRB, 0.40062E+00_JPRB, 0.40229E+00_JPRB, 0.40408E+00_JPRB ,&
&  0.40444E+00_JPRB, 0.40190E+00_JPRB, 0.39466E+00_JPRB, 0.39406E+00_JPRB, 0.39795E+00_JPRB, 0.39560E+00_JPRB, 0.40352E+00_JPRB ,&
&  0.40892E+00_JPRB, 0.41137E+00_JPRB, 0.40569E+00_JPRB, 0.40188E+00_JPRB, 0.39429E+00_JPRB, 0.36132E+00_JPRB, 0.35862E+00_JPRB ,&
&  0.35242E+00_JPRB, 0.34358E+00_JPRB, 0.33963E+00_JPRB, 0.33646E+00_JPRB, 0.33596E+00_JPRB, 0.34134E+00_JPRB, 0.34903E+00_JPRB ,&
&  0.35122E+00_JPRB, 0.36405E+00_JPRB, 0.36428E+00_JPRB, 0.36401E+00_JPRB, 0.36521E+00_JPRB, 0.36826E+00_JPRB, 0.36778E+00_JPRB ,&
&  0.36888E+00_JPRB, 0.36933E+00_JPRB, 0.36923E+00_JPRB, 0.37208E+00_JPRB, 0.37495E+00_JPRB, 0.37725E+00_JPRB, 0.38097E+00_JPRB ,&
&  0.38858E+00_JPRB, 0.39365E+00_JPRB, 0.39673E+00_JPRB, 0.39955E+00_JPRB, 0.40417E+00_JPRB, 0.40917E+00_JPRB, 0.41586E+00_JPRB ,&
&  0.42101E+00_JPRB, 0.42682E+00_JPRB, 0.43377E+00_JPRB, 0.44041E+00_JPRB, 0.44733E+00_JPRB, 0.45460E+00_JPRB, 0.46288E+00_JPRB ,&
&  0.46705E+00_JPRB, 0.47355E+00_JPRB, 0.48108E+00_JPRB, 0.49495E+00_JPRB, 0.49611E+00_JPRB, 0.48622E+00_JPRB, 0.47820E+00_JPRB ,&
&  0.47264E+00_JPRB, 0.47276E+00_JPRB, 0.46776E+00_JPRB, 0.46478E+00_JPRB, 0.46138E+00_JPRB, 0.46038E+00_JPRB, 0.45814E+00_JPRB ,&
&  0.45660E+00_JPRB, 0.45388E+00_JPRB, 0.45009E+00_JPRB, 0.44568E+00_JPRB, 0.44231E+00_JPRB, 0.43865E+00_JPRB, 0.43591E+00_JPRB ,&
&  0.43255E+00_JPRB, 0.43096E+00_JPRB, 0.42985E+00_JPRB, 0.42969E+00_JPRB, 0.42931E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.43655E+00_JPRB, 0.43830E+00_JPRB, 0.43474E+00_JPRB, 0.43295E+00_JPRB, 0.42675E+00_JPRB, 0.42083E+00_JPRB, 0.41338E+00_JPRB ,&
&  0.40786E+00_JPRB, 0.40381E+00_JPRB, 0.39984E+00_JPRB, 0.39836E+00_JPRB, 0.39795E+00_JPRB, 0.39803E+00_JPRB, 0.39935E+00_JPRB ,&
&  0.40083E+00_JPRB, 0.39930E+00_JPRB, 0.40067E+00_JPRB, 0.40328E+00_JPRB, 0.40529E+00_JPRB, 0.40708E+00_JPRB, 0.40883E+00_JPRB ,&
&  0.40796E+00_JPRB, 0.40687E+00_JPRB, 0.40548E+00_JPRB, 0.40126E+00_JPRB, 0.39967E+00_JPRB, 0.40557E+00_JPRB, 0.41400E+00_JPRB ,&
&  0.41043E+00_JPRB, 0.41129E+00_JPRB, 0.41551E+00_JPRB, 0.41585E+00_JPRB, 0.40494E+00_JPRB, 0.38859E+00_JPRB, 0.37039E+00_JPRB ,&
&  0.35763E+00_JPRB, 0.34227E+00_JPRB, 0.33566E+00_JPRB, 0.33236E+00_JPRB, 0.33538E+00_JPRB, 0.34166E+00_JPRB, 0.35324E+00_JPRB ,&
&  0.36088E+00_JPRB, 0.36472E+00_JPRB, 0.36875E+00_JPRB, 0.37282E+00_JPRB, 0.37468E+00_JPRB, 0.37800E+00_JPRB, 0.37961E+00_JPRB ,&
&  0.38373E+00_JPRB, 0.38606E+00_JPRB, 0.38788E+00_JPRB, 0.39229E+00_JPRB, 0.39942E+00_JPRB, 0.40677E+00_JPRB, 0.41858E+00_JPRB ,&
&  0.44752E+00_JPRB, 0.45768E+00_JPRB, 0.46456E+00_JPRB, 0.46776E+00_JPRB, 0.47211E+00_JPRB, 0.47368E+00_JPRB, 0.46797E+00_JPRB ,&
&  0.46702E+00_JPRB, 0.47314E+00_JPRB, 0.47345E+00_JPRB, 0.47449E+00_JPRB, 0.47674E+00_JPRB, 0.47882E+00_JPRB, 0.48187E+00_JPRB ,&
&  0.47764E+00_JPRB, 0.48595E+00_JPRB, 0.53073E+00_JPRB, 0.54499E+00_JPRB, 0.53625E+00_JPRB, 0.52138E+00_JPRB, 0.50833E+00_JPRB ,&
&  0.49868E+00_JPRB, 0.49050E+00_JPRB, 0.48366E+00_JPRB, 0.47804E+00_JPRB, 0.47395E+00_JPRB, 0.47167E+00_JPRB, 0.47121E+00_JPRB ,&
&  0.46689E+00_JPRB, 0.46173E+00_JPRB, 0.45603E+00_JPRB, 0.45173E+00_JPRB, 0.44789E+00_JPRB, 0.44353E+00_JPRB, 0.44065E+00_JPRB ,&
&  0.43775E+00_JPRB, 0.43707E+00_JPRB, 0.43673E+00_JPRB, 0.43709E+00_JPRB, 0.43701E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.43732E+00_JPRB, 0.44051E+00_JPRB, 0.44160E+00_JPRB, 0.43575E+00_JPRB, 0.42343E+00_JPRB, 0.41210E+00_JPRB, 0.40400E+00_JPRB ,&
&  0.40129E+00_JPRB, 0.40021E+00_JPRB, 0.39933E+00_JPRB, 0.39896E+00_JPRB, 0.39933E+00_JPRB, 0.40048E+00_JPRB, 0.40236E+00_JPRB ,&
&  0.40497E+00_JPRB, 0.40730E+00_JPRB, 0.40821E+00_JPRB, 0.40813E+00_JPRB, 0.40946E+00_JPRB, 0.41254E+00_JPRB, 0.41599E+00_JPRB ,&
&  0.41753E+00_JPRB, 0.41662E+00_JPRB, 0.41704E+00_JPRB, 0.41548E+00_JPRB, 0.41437E+00_JPRB, 0.41260E+00_JPRB, 0.41280E+00_JPRB ,&
&  0.41811E+00_JPRB, 0.42257E+00_JPRB, 0.41751E+00_JPRB, 0.41862E+00_JPRB, 0.42290E+00_JPRB, 0.40831E+00_JPRB, 0.38572E+00_JPRB ,&
&  0.36152E+00_JPRB, 0.34323E+00_JPRB, 0.33545E+00_JPRB, 0.33373E+00_JPRB, 0.33922E+00_JPRB, 0.35166E+00_JPRB, 0.36695E+00_JPRB ,&
&  0.37057E+00_JPRB, 0.37503E+00_JPRB, 0.38205E+00_JPRB, 0.38696E+00_JPRB, 0.38948E+00_JPRB, 0.39070E+00_JPRB, 0.39214E+00_JPRB ,&
&  0.39546E+00_JPRB, 0.40252E+00_JPRB, 0.41467E+00_JPRB, 0.43151E+00_JPRB, 0.45737E+00_JPRB, 0.47297E+00_JPRB, 0.50155E+00_JPRB ,&
&  0.54608E+00_JPRB, 0.54543E+00_JPRB, 0.54756E+00_JPRB, 0.54378E+00_JPRB, 0.53974E+00_JPRB, 0.53326E+00_JPRB, 0.52899E+00_JPRB ,&
&  0.52544E+00_JPRB, 0.52096E+00_JPRB, 0.51391E+00_JPRB, 0.50710E+00_JPRB, 0.50689E+00_JPRB, 0.51135E+00_JPRB, 0.52319E+00_JPRB ,&
&  0.54333E+00_JPRB, 0.57967E+00_JPRB, 0.67361E+00_JPRB, 0.61198E+00_JPRB, 0.57956E+00_JPRB, 0.56473E+00_JPRB, 0.54651E+00_JPRB ,&
&  0.53212E+00_JPRB, 0.52288E+00_JPRB, 0.51718E+00_JPRB, 0.50990E+00_JPRB, 0.49612E+00_JPRB, 0.48594E+00_JPRB, 0.47845E+00_JPRB ,&
&  0.47201E+00_JPRB, 0.46561E+00_JPRB, 0.46101E+00_JPRB, 0.45769E+00_JPRB, 0.45372E+00_JPRB, 0.45006E+00_JPRB, 0.44762E+00_JPRB ,&
&  0.44395E+00_JPRB, 0.44146E+00_JPRB, 0.44035E+00_JPRB, 0.43846E+00_JPRB, 0.43749E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.43998E+00_JPRB, 0.44358E+00_JPRB, 0.44041E+00_JPRB, 0.43436E+00_JPRB, 0.42108E+00_JPRB, 0.41026E+00_JPRB, 0.40287E+00_JPRB ,&
&  0.40174E+00_JPRB, 0.39913E+00_JPRB, 0.39807E+00_JPRB, 0.39829E+00_JPRB, 0.39957E+00_JPRB, 0.40217E+00_JPRB, 0.40408E+00_JPRB ,&
&  0.40544E+00_JPRB, 0.40780E+00_JPRB, 0.41062E+00_JPRB, 0.41268E+00_JPRB, 0.41457E+00_JPRB, 0.41703E+00_JPRB, 0.42003E+00_JPRB ,&
&  0.42334E+00_JPRB, 0.42480E+00_JPRB, 0.42547E+00_JPRB, 0.42617E+00_JPRB, 0.42930E+00_JPRB, 0.43381E+00_JPRB, 0.43508E+00_JPRB ,&
&  0.42940E+00_JPRB, 0.42641E+00_JPRB, 0.43014E+00_JPRB, 0.43107E+00_JPRB, 0.43426E+00_JPRB, 0.42424E+00_JPRB, 0.39806E+00_JPRB ,&
&  0.37041E+00_JPRB, 0.35185E+00_JPRB, 0.34912E+00_JPRB, 0.35663E+00_JPRB, 0.37060E+00_JPRB, 0.37928E+00_JPRB, 0.38709E+00_JPRB ,&
&  0.39338E+00_JPRB, 0.39588E+00_JPRB, 0.39831E+00_JPRB, 0.40373E+00_JPRB, 0.40750E+00_JPRB, 0.40877E+00_JPRB, 0.41129E+00_JPRB ,&
&  0.41790E+00_JPRB, 0.42849E+00_JPRB, 0.43398E+00_JPRB, 0.43132E+00_JPRB, 0.45194E+00_JPRB, 0.51533E+00_JPRB, 0.56705E+00_JPRB ,&
&  0.57988E+00_JPRB, 0.58315E+00_JPRB, 0.58619E+00_JPRB, 0.60308E+00_JPRB, 0.60673E+00_JPRB, 0.60819E+00_JPRB, 0.60236E+00_JPRB ,&
&  0.58943E+00_JPRB, 0.57344E+00_JPRB, 0.56078E+00_JPRB, 0.55863E+00_JPRB, 0.57249E+00_JPRB, 0.60097E+00_JPRB, 0.63016E+00_JPRB ,&
&  0.64799E+00_JPRB, 0.63450E+00_JPRB, 0.58562E+00_JPRB, 0.54179E+00_JPRB, 0.52888E+00_JPRB, 0.53277E+00_JPRB, 0.53937E+00_JPRB ,&
&  0.53636E+00_JPRB, 0.53165E+00_JPRB, 0.52660E+00_JPRB, 0.52094E+00_JPRB, 0.49827E+00_JPRB, 0.48361E+00_JPRB, 0.47172E+00_JPRB ,&
&  0.46335E+00_JPRB, 0.45759E+00_JPRB, 0.45841E+00_JPRB, 0.45601E+00_JPRB, 0.45305E+00_JPRB, 0.44917E+00_JPRB, 0.44312E+00_JPRB ,&
&  0.43884E+00_JPRB, 0.43610E+00_JPRB, 0.43393E+00_JPRB, 0.43321E+00_JPRB, 0.43461E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.44838E+00_JPRB, 0.44719E+00_JPRB, 0.44558E+00_JPRB, 0.43941E+00_JPRB, 0.43418E+00_JPRB, 0.43185E+00_JPRB, 0.43037E+00_JPRB ,&
&  0.43865E+00_JPRB, 0.42417E+00_JPRB, 0.41103E+00_JPRB, 0.40133E+00_JPRB, 0.39590E+00_JPRB, 0.39423E+00_JPRB, 0.39771E+00_JPRB ,&
&  0.40327E+00_JPRB, 0.40869E+00_JPRB, 0.41312E+00_JPRB, 0.41748E+00_JPRB, 0.42158E+00_JPRB, 0.42417E+00_JPRB, 0.42681E+00_JPRB ,&
&  0.43034E+00_JPRB, 0.43395E+00_JPRB, 0.43747E+00_JPRB, 0.44323E+00_JPRB, 0.44779E+00_JPRB, 0.45136E+00_JPRB, 0.45036E+00_JPRB ,&
&  0.44583E+00_JPRB, 0.44129E+00_JPRB, 0.44122E+00_JPRB, 0.44297E+00_JPRB, 0.44370E+00_JPRB, 0.43504E+00_JPRB, 0.40837E+00_JPRB ,&
&  0.38291E+00_JPRB, 0.37294E+00_JPRB, 0.38335E+00_JPRB, 0.40089E+00_JPRB, 0.40716E+00_JPRB, 0.41244E+00_JPRB, 0.41374E+00_JPRB ,&
&  0.41133E+00_JPRB, 0.41084E+00_JPRB, 0.41711E+00_JPRB, 0.42581E+00_JPRB, 0.42852E+00_JPRB, 0.43079E+00_JPRB, 0.43498E+00_JPRB ,&
&  0.44551E+00_JPRB, 0.45715E+00_JPRB, 0.46387E+00_JPRB, 0.45952E+00_JPRB, 0.51595E+00_JPRB, 0.71715E+00_JPRB, 0.71241E+00_JPRB ,&
&  0.71558E+00_JPRB, 0.71499E+00_JPRB, 0.72442E+00_JPRB, 0.73212E+00_JPRB, 0.71225E+00_JPRB, 0.69951E+00_JPRB, 0.69152E+00_JPRB ,&
&  0.66799E+00_JPRB, 0.65643E+00_JPRB, 0.65574E+00_JPRB, 0.68175E+00_JPRB, 0.70335E+00_JPRB, 0.71737E+00_JPRB, 0.72857E+00_JPRB ,&
&  0.72191E+00_JPRB, 0.68466E+00_JPRB, 0.63904E+00_JPRB, 0.59745E+00_JPRB, 0.57665E+00_JPRB, 0.56347E+00_JPRB, 0.55543E+00_JPRB ,&
&  0.54622E+00_JPRB, 0.54262E+00_JPRB, 0.54202E+00_JPRB, 0.53907E+00_JPRB, 0.51854E+00_JPRB, 0.49325E+00_JPRB, 0.47787E+00_JPRB ,&
&  0.46832E+00_JPRB, 0.46005E+00_JPRB, 0.45285E+00_JPRB, 0.44775E+00_JPRB, 0.44850E+00_JPRB, 0.44894E+00_JPRB, 0.44423E+00_JPRB ,&
&  0.44378E+00_JPRB, 0.43984E+00_JPRB, 0.43952E+00_JPRB, 0.44133E+00_JPRB, 0.44677E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.45231E+00_JPRB, 0.45411E+00_JPRB, 0.45413E+00_JPRB, 0.45201E+00_JPRB, 0.45300E+00_JPRB, 0.45267E+00_JPRB, 0.44571E+00_JPRB ,&
&  0.45972E+00_JPRB, 0.42867E+00_JPRB, 0.41488E+00_JPRB, 0.40832E+00_JPRB, 0.40735E+00_JPRB, 0.40376E+00_JPRB, 0.39861E+00_JPRB ,&
&  0.40040E+00_JPRB, 0.40810E+00_JPRB, 0.41715E+00_JPRB, 0.42355E+00_JPRB, 0.42896E+00_JPRB, 0.43203E+00_JPRB, 0.43464E+00_JPRB ,&
&  0.43838E+00_JPRB, 0.44399E+00_JPRB, 0.45207E+00_JPRB, 0.45834E+00_JPRB, 0.46189E+00_JPRB, 0.46518E+00_JPRB, 0.46435E+00_JPRB ,&
&  0.46200E+00_JPRB, 0.45648E+00_JPRB, 0.45209E+00_JPRB, 0.45011E+00_JPRB, 0.44805E+00_JPRB, 0.44813E+00_JPRB, 0.43836E+00_JPRB ,&
&  0.41153E+00_JPRB, 0.41114E+00_JPRB, 0.42783E+00_JPRB, 0.43491E+00_JPRB, 0.43529E+00_JPRB, 0.43293E+00_JPRB, 0.43340E+00_JPRB ,&
&  0.43552E+00_JPRB, 0.43767E+00_JPRB, 0.44248E+00_JPRB, 0.44475E+00_JPRB, 0.44818E+00_JPRB, 0.45165E+00_JPRB, 0.46294E+00_JPRB ,&
&  0.48834E+00_JPRB, 0.47306E+00_JPRB, 0.47902E+00_JPRB, 0.51693E+00_JPRB, 0.68569E+00_JPRB, 0.75742E+00_JPRB, 0.80711E+00_JPRB ,&
&  0.88110E+00_JPRB, 0.88039E+00_JPRB, 0.86226E+00_JPRB, 0.85049E+00_JPRB, 0.84444E+00_JPRB, 0.83518E+00_JPRB, 0.83421E+00_JPRB ,&
&  0.80984E+00_JPRB, 0.81378E+00_JPRB, 0.82817E+00_JPRB, 0.85322E+00_JPRB, 0.85738E+00_JPRB, 0.85511E+00_JPRB, 0.84309E+00_JPRB ,&
&  0.81838E+00_JPRB, 0.78214E+00_JPRB, 0.72929E+00_JPRB, 0.67796E+00_JPRB, 0.63544E+00_JPRB, 0.61391E+00_JPRB, 0.59891E+00_JPRB ,&
&  0.58603E+00_JPRB, 0.57866E+00_JPRB, 0.57101E+00_JPRB, 0.56138E+00_JPRB, 0.54955E+00_JPRB, 0.53731E+00_JPRB, 0.53326E+00_JPRB ,&
&  0.50937E+00_JPRB, 0.49414E+00_JPRB, 0.47674E+00_JPRB, 0.46607E+00_JPRB, 0.46428E+00_JPRB, 0.46465E+00_JPRB, 0.46416E+00_JPRB ,&
&  0.46701E+00_JPRB, 0.45847E+00_JPRB, 0.45623E+00_JPRB, 0.45541E+00_JPRB, 0.45525E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.45737E+00_JPRB, 0.46315E+00_JPRB, 0.46440E+00_JPRB, 0.46532E+00_JPRB, 0.47074E+00_JPRB, 0.47285E+00_JPRB, 0.47462E+00_JPRB ,&
&  0.51601E+00_JPRB, 0.49221E+00_JPRB, 0.50234E+00_JPRB, 0.45057E+00_JPRB, 0.42906E+00_JPRB, 0.41965E+00_JPRB, 0.40731E+00_JPRB ,&
&  0.40565E+00_JPRB, 0.41010E+00_JPRB, 0.41582E+00_JPRB, 0.42293E+00_JPRB, 0.43095E+00_JPRB, 0.43508E+00_JPRB, 0.43868E+00_JPRB ,&
&  0.44348E+00_JPRB, 0.45012E+00_JPRB, 0.45726E+00_JPRB, 0.46225E+00_JPRB, 0.46596E+00_JPRB, 0.47027E+00_JPRB, 0.46839E+00_JPRB ,&
&  0.45901E+00_JPRB, 0.45488E+00_JPRB, 0.45386E+00_JPRB, 0.45135E+00_JPRB, 0.45315E+00_JPRB, 0.45454E+00_JPRB, 0.46262E+00_JPRB ,&
&  0.46412E+00_JPRB, 0.45604E+00_JPRB, 0.46390E+00_JPRB, 0.46127E+00_JPRB, 0.45940E+00_JPRB, 0.45648E+00_JPRB, 0.45348E+00_JPRB ,&
&  0.45267E+00_JPRB, 0.45520E+00_JPRB, 0.46123E+00_JPRB, 0.46616E+00_JPRB, 0.47062E+00_JPRB, 0.48045E+00_JPRB, 0.50387E+00_JPRB ,&
&  0.54389E+00_JPRB, 0.51242E+00_JPRB, 0.58200E+00_JPRB, 0.61776E+00_JPRB, 0.80317E+00_JPRB, 0.99367E+00_JPRB, 0.10092E+01_JPRB ,&
&  0.11354E+01_JPRB, 0.11121E+01_JPRB, 0.10692E+01_JPRB, 0.10366E+01_JPRB, 0.10275E+01_JPRB, 0.99088E+00_JPRB, 0.94331E+00_JPRB ,&
&  0.93640E+00_JPRB, 0.93023E+00_JPRB, 0.98578E+00_JPRB, 0.98589E+00_JPRB, 0.95577E+00_JPRB, 0.93591E+00_JPRB, 0.89915E+00_JPRB ,&
&  0.86913E+00_JPRB, 0.83600E+00_JPRB, 0.80178E+00_JPRB, 0.73440E+00_JPRB, 0.70723E+00_JPRB, 0.68165E+00_JPRB, 0.63658E+00_JPRB ,&
&  0.61625E+00_JPRB, 0.59171E+00_JPRB, 0.56862E+00_JPRB, 0.57337E+00_JPRB, 0.55631E+00_JPRB, 0.54680E+00_JPRB, 0.54183E+00_JPRB ,&
&  0.52584E+00_JPRB, 0.50922E+00_JPRB, 0.51090E+00_JPRB, 0.50581E+00_JPRB, 0.49430E+00_JPRB, 0.48442E+00_JPRB, 0.48235E+00_JPRB ,&
&  0.47542E+00_JPRB, 0.47202E+00_JPRB, 0.47595E+00_JPRB, 0.46700E+00_JPRB, 0.46039E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.47170E+00_JPRB, 0.47517E+00_JPRB, 0.47418E+00_JPRB, 0.47837E+00_JPRB, 0.48815E+00_JPRB, 0.49778E+00_JPRB, 0.50581E+00_JPRB ,&
&  0.52599E+00_JPRB, 0.53370E+00_JPRB, 0.53355E+00_JPRB, 0.52619E+00_JPRB, 0.51210E+00_JPRB, 0.47130E+00_JPRB, 0.41796E+00_JPRB ,&
&  0.40560E+00_JPRB, 0.41313E+00_JPRB, 0.42276E+00_JPRB, 0.42769E+00_JPRB, 0.43389E+00_JPRB, 0.44013E+00_JPRB, 0.44546E+00_JPRB ,&
&  0.45147E+00_JPRB, 0.45597E+00_JPRB, 0.46150E+00_JPRB, 0.46648E+00_JPRB, 0.46945E+00_JPRB, 0.47400E+00_JPRB, 0.47613E+00_JPRB ,&
&  0.46190E+00_JPRB, 0.45428E+00_JPRB, 0.45041E+00_JPRB, 0.45184E+00_JPRB, 0.46502E+00_JPRB, 0.47297E+00_JPRB, 0.48288E+00_JPRB ,&
&  0.49189E+00_JPRB, 0.49680E+00_JPRB, 0.49593E+00_JPRB, 0.49412E+00_JPRB, 0.49090E+00_JPRB, 0.48931E+00_JPRB, 0.48620E+00_JPRB ,&
&  0.48469E+00_JPRB, 0.48618E+00_JPRB, 0.49277E+00_JPRB, 0.49802E+00_JPRB, 0.50591E+00_JPRB, 0.52372E+00_JPRB, 0.57910E+00_JPRB ,&
&  0.64043E+00_JPRB, 0.71676E+00_JPRB, 0.75958E+00_JPRB, 0.76254E+00_JPRB, 0.92076E+00_JPRB, 0.10471E+01_JPRB, 0.10836E+01_JPRB ,&
&  0.11431E+01_JPRB, 0.10937E+01_JPRB, 0.11480E+01_JPRB, 0.12036E+01_JPRB, 0.12378E+01_JPRB, 0.11983E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.11019E+01_JPRB, 0.11735E+01_JPRB, 0.11733E+01_JPRB, 0.11651E+01_JPRB, 0.11116E+01_JPRB, 0.10720E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.10083E+01_JPRB, 0.99412E+00_JPRB, 0.94665E+00_JPRB, 0.89341E+00_JPRB, 0.83296E+00_JPRB, 0.77281E+00_JPRB, 0.71249E+00_JPRB ,&
&  0.65580E+00_JPRB, 0.59010E+00_JPRB, 0.56517E+00_JPRB, 0.56264E+00_JPRB, 0.56611E+00_JPRB, 0.54901E+00_JPRB, 0.54299E+00_JPRB ,&
&  0.53957E+00_JPRB, 0.53936E+00_JPRB, 0.55031E+00_JPRB, 0.53615E+00_JPRB, 0.51556E+00_JPRB, 0.49301E+00_JPRB, 0.47941E+00_JPRB ,&
&  0.46858E+00_JPRB, 0.47426E+00_JPRB, 0.47823E+00_JPRB, 0.47460E+00_JPRB, 0.47055E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.48392E+00_JPRB, 0.48100E+00_JPRB, 0.48121E+00_JPRB, 0.49022E+00_JPRB, 0.50406E+00_JPRB, 0.51931E+00_JPRB, 0.53837E+00_JPRB ,&
&  0.55630E+00_JPRB, 0.56534E+00_JPRB, 0.55818E+00_JPRB, 0.54492E+00_JPRB, 0.53386E+00_JPRB, 0.52749E+00_JPRB, 0.49651E+00_JPRB ,&
&  0.44919E+00_JPRB, 0.44468E+00_JPRB, 0.43628E+00_JPRB, 0.43853E+00_JPRB, 0.44818E+00_JPRB, 0.45449E+00_JPRB, 0.46060E+00_JPRB ,&
&  0.46966E+00_JPRB, 0.47313E+00_JPRB, 0.47514E+00_JPRB, 0.47672E+00_JPRB, 0.48035E+00_JPRB, 0.49190E+00_JPRB, 0.50003E+00_JPRB ,&
&  0.48020E+00_JPRB, 0.46730E+00_JPRB, 0.46599E+00_JPRB, 0.47293E+00_JPRB, 0.48746E+00_JPRB, 0.50410E+00_JPRB, 0.51609E+00_JPRB ,&
&  0.52658E+00_JPRB, 0.53303E+00_JPRB, 0.53470E+00_JPRB, 0.53360E+00_JPRB, 0.53095E+00_JPRB, 0.52709E+00_JPRB, 0.52560E+00_JPRB ,&
&  0.52518E+00_JPRB, 0.52810E+00_JPRB, 0.53586E+00_JPRB, 0.54377E+00_JPRB, 0.55183E+00_JPRB, 0.59562E+00_JPRB, 0.69687E+00_JPRB ,&
&  0.81837E+00_JPRB, 0.94409E+00_JPRB, 0.97669E+00_JPRB, 0.10638E+01_JPRB, 0.11451E+01_JPRB, 0.12226E+01_JPRB, 0.12580E+01_JPRB ,&
&  0.12833E+01_JPRB, 0.12106E+01_JPRB, 0.13512E+01_JPRB, 0.14126E+01_JPRB, 0.13849E+01_JPRB, 0.13982E+01_JPRB, 0.13197E+01_JPRB ,&
&  0.12708E+01_JPRB, 0.12332E+01_JPRB, 0.12791E+01_JPRB, 0.12811E+01_JPRB, 0.12488E+01_JPRB, 0.12568E+01_JPRB, 0.12059E+01_JPRB ,&
&  0.11749E+01_JPRB, 0.11267E+01_JPRB, 0.97958E+00_JPRB, 0.86704E+00_JPRB, 0.75859E+00_JPRB, 0.78173E+00_JPRB, 0.73569E+00_JPRB ,&
&  0.67403E+00_JPRB, 0.59611E+00_JPRB, 0.59506E+00_JPRB, 0.59167E+00_JPRB, 0.60272E+00_JPRB, 0.60612E+00_JPRB, 0.58174E+00_JPRB ,&
&  0.58070E+00_JPRB, 0.59314E+00_JPRB, 0.57728E+00_JPRB, 0.56096E+00_JPRB, 0.54126E+00_JPRB, 0.51552E+00_JPRB, 0.50020E+00_JPRB ,&
&  0.49922E+00_JPRB, 0.50269E+00_JPRB, 0.49864E+00_JPRB, 0.49062E+00_JPRB, 0.48625E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.51191E+00_JPRB, 0.50929E+00_JPRB, 0.51094E+00_JPRB, 0.51960E+00_JPRB, 0.53249E+00_JPRB, 0.55069E+00_JPRB, 0.57013E+00_JPRB ,&
&  0.58497E+00_JPRB, 0.58768E+00_JPRB, 0.57455E+00_JPRB, 0.56434E+00_JPRB, 0.55779E+00_JPRB, 0.55043E+00_JPRB, 0.55185E+00_JPRB ,&
&  0.49436E+00_JPRB, 0.47024E+00_JPRB, 0.44414E+00_JPRB, 0.44358E+00_JPRB, 0.46249E+00_JPRB, 0.46900E+00_JPRB, 0.47570E+00_JPRB ,&
&  0.48144E+00_JPRB, 0.48988E+00_JPRB, 0.49147E+00_JPRB, 0.49427E+00_JPRB, 0.50050E+00_JPRB, 0.51794E+00_JPRB, 0.53697E+00_JPRB ,&
&  0.52113E+00_JPRB, 0.50646E+00_JPRB, 0.50985E+00_JPRB, 0.52186E+00_JPRB, 0.53275E+00_JPRB, 0.55199E+00_JPRB, 0.57498E+00_JPRB ,&
&  0.58345E+00_JPRB, 0.58312E+00_JPRB, 0.57740E+00_JPRB, 0.57444E+00_JPRB, 0.57358E+00_JPRB, 0.57250E+00_JPRB, 0.57709E+00_JPRB ,&
&  0.58393E+00_JPRB, 0.59014E+00_JPRB, 0.59895E+00_JPRB, 0.60440E+00_JPRB, 0.60608E+00_JPRB, 0.67727E+00_JPRB, 0.80881E+00_JPRB ,&
&  0.97962E+00_JPRB, 0.10139E+01_JPRB, 0.10831E+01_JPRB, 0.13694E+01_JPRB, 0.15163E+01_JPRB, 0.15724E+01_JPRB, 0.15283E+01_JPRB ,&
&  0.14752E+01_JPRB, 0.13824E+01_JPRB, 0.13754E+01_JPRB, 0.13991E+01_JPRB, 0.13821E+01_JPRB, 0.14541E+01_JPRB, 0.14161E+01_JPRB ,&
&  0.13299E+01_JPRB, 0.13045E+01_JPRB, 0.13015E+01_JPRB, 0.12900E+01_JPRB, 0.12502E+01_JPRB, 0.12543E+01_JPRB, 0.12188E+01_JPRB ,&
&  0.11593E+01_JPRB, 0.10087E+01_JPRB, 0.79942E+00_JPRB, 0.75595E+00_JPRB, 0.64601E+00_JPRB, 0.67327E+00_JPRB, 0.77668E+00_JPRB ,&
&  0.78521E+00_JPRB, 0.73916E+00_JPRB, 0.69473E+00_JPRB, 0.67225E+00_JPRB, 0.66700E+00_JPRB, 0.70244E+00_JPRB, 0.66819E+00_JPRB ,&
&  0.63010E+00_JPRB, 0.63883E+00_JPRB, 0.63503E+00_JPRB, 0.61717E+00_JPRB, 0.59166E+00_JPRB, 0.56925E+00_JPRB, 0.55490E+00_JPRB ,&
&  0.55164E+00_JPRB, 0.54923E+00_JPRB, 0.54248E+00_JPRB, 0.52525E+00_JPRB, 0.51448E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.55827E+00_JPRB, 0.55776E+00_JPRB, 0.55639E+00_JPRB, 0.56354E+00_JPRB, 0.57485E+00_JPRB, 0.59013E+00_JPRB, 0.60560E+00_JPRB ,&
&  0.61265E+00_JPRB, 0.60577E+00_JPRB, 0.59047E+00_JPRB, 0.58604E+00_JPRB, 0.58007E+00_JPRB, 0.56974E+00_JPRB, 0.56733E+00_JPRB ,&
&  0.57740E+00_JPRB, 0.52307E+00_JPRB, 0.48335E+00_JPRB, 0.45438E+00_JPRB, 0.46389E+00_JPRB, 0.46937E+00_JPRB, 0.48120E+00_JPRB ,&
&  0.48915E+00_JPRB, 0.50634E+00_JPRB, 0.51320E+00_JPRB, 0.51791E+00_JPRB, 0.52967E+00_JPRB, 0.55713E+00_JPRB, 0.58357E+00_JPRB ,&
&  0.58656E+00_JPRB, 0.58864E+00_JPRB, 0.60008E+00_JPRB, 0.62255E+00_JPRB, 0.64291E+00_JPRB, 0.62140E+00_JPRB, 0.64393E+00_JPRB ,&
&  0.64554E+00_JPRB, 0.63996E+00_JPRB, 0.62822E+00_JPRB, 0.63079E+00_JPRB, 0.63839E+00_JPRB, 0.65097E+00_JPRB, 0.66621E+00_JPRB ,&
&  0.68408E+00_JPRB, 0.68717E+00_JPRB, 0.66709E+00_JPRB, 0.66345E+00_JPRB, 0.67972E+00_JPRB, 0.79873E+00_JPRB, 0.87242E+00_JPRB ,&
&  0.99716E+00_JPRB, 0.99216E+00_JPRB, 0.99974E+00_JPRB, 0.12644E+01_JPRB, 0.15236E+01_JPRB, 0.16314E+01_JPRB, 0.16524E+01_JPRB ,&
&  0.16172E+01_JPRB, 0.15588E+01_JPRB, 0.15295E+01_JPRB, 0.14633E+01_JPRB, 0.14642E+01_JPRB, 0.14994E+01_JPRB, 0.14598E+01_JPRB ,&
&  0.13652E+01_JPRB, 0.12888E+01_JPRB, 0.12643E+01_JPRB, 0.12173E+01_JPRB, 0.11767E+01_JPRB, 0.11002E+01_JPRB, 0.11209E+01_JPRB ,&
&  0.11320E+01_JPRB, 0.73893E+00_JPRB, 0.63654E+00_JPRB, 0.70883E+00_JPRB, 0.63532E+00_JPRB, 0.65605E+00_JPRB, 0.73270E+00_JPRB ,&
&  0.73895E+00_JPRB, 0.73695E+00_JPRB, 0.76094E+00_JPRB, 0.74036E+00_JPRB, 0.75361E+00_JPRB, 0.78399E+00_JPRB, 0.76435E+00_JPRB ,&
&  0.73889E+00_JPRB, 0.70726E+00_JPRB, 0.70914E+00_JPRB, 0.68748E+00_JPRB, 0.65764E+00_JPRB, 0.63544E+00_JPRB, 0.61809E+00_JPRB ,&
&  0.60263E+00_JPRB, 0.59321E+00_JPRB, 0.58818E+00_JPRB, 0.57209E+00_JPRB, 0.55962E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.62598E+00_JPRB, 0.62530E+00_JPRB, 0.62496E+00_JPRB, 0.62851E+00_JPRB, 0.63264E+00_JPRB, 0.64213E+00_JPRB, 0.64738E+00_JPRB ,&
&  0.64265E+00_JPRB, 0.62368E+00_JPRB, 0.60860E+00_JPRB, 0.61003E+00_JPRB, 0.59913E+00_JPRB, 0.58619E+00_JPRB, 0.58634E+00_JPRB ,&
&  0.60001E+00_JPRB, 0.58660E+00_JPRB, 0.51349E+00_JPRB, 0.47802E+00_JPRB, 0.45206E+00_JPRB, 0.46168E+00_JPRB, 0.47294E+00_JPRB ,&
&  0.49449E+00_JPRB, 0.50896E+00_JPRB, 0.51827E+00_JPRB, 0.53276E+00_JPRB, 0.56306E+00_JPRB, 0.60282E+00_JPRB, 0.64826E+00_JPRB ,&
&  0.66987E+00_JPRB, 0.69278E+00_JPRB, 0.68328E+00_JPRB, 0.72518E+00_JPRB, 0.73863E+00_JPRB, 0.70712E+00_JPRB, 0.73234E+00_JPRB ,&
&  0.72979E+00_JPRB, 0.72623E+00_JPRB, 0.71631E+00_JPRB, 0.72296E+00_JPRB, 0.72952E+00_JPRB, 0.72031E+00_JPRB, 0.70527E+00_JPRB ,&
&  0.69450E+00_JPRB, 0.69093E+00_JPRB, 0.68524E+00_JPRB, 0.67768E+00_JPRB, 0.70942E+00_JPRB, 0.78983E+00_JPRB, 0.82163E+00_JPRB ,&
&  0.89383E+00_JPRB, 0.78045E+00_JPRB, 0.70638E+00_JPRB, 0.96495E+00_JPRB, 0.12961E+01_JPRB, 0.14199E+01_JPRB, 0.13938E+01_JPRB ,&
&  0.15116E+01_JPRB, 0.15449E+01_JPRB, 0.15836E+01_JPRB, 0.16250E+01_JPRB, 0.16227E+01_JPRB, 0.16031E+01_JPRB, 0.15364E+01_JPRB ,&
&  0.14048E+01_JPRB, 0.13126E+01_JPRB, 0.12681E+01_JPRB, 0.12917E+01_JPRB, 0.12416E+01_JPRB, 0.12020E+01_JPRB, 0.11700E+01_JPRB ,&
&  0.10348E+01_JPRB, 0.91937E+00_JPRB, 0.71802E+00_JPRB, 0.72716E+00_JPRB, 0.61740E+00_JPRB, 0.62368E+00_JPRB, 0.69424E+00_JPRB ,&
&  0.69972E+00_JPRB, 0.74356E+00_JPRB, 0.76264E+00_JPRB, 0.75024E+00_JPRB, 0.84624E+00_JPRB, 0.87876E+00_JPRB, 0.86236E+00_JPRB ,&
&  0.84331E+00_JPRB, 0.77073E+00_JPRB, 0.78733E+00_JPRB, 0.78999E+00_JPRB, 0.77086E+00_JPRB, 0.75007E+00_JPRB, 0.72706E+00_JPRB ,&
&  0.70060E+00_JPRB, 0.67763E+00_JPRB, 0.66321E+00_JPRB, 0.64294E+00_JPRB, 0.62959E+00_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.69285E+00_JPRB, 0.67825E+00_JPRB, 0.66986E+00_JPRB, 0.66871E+00_JPRB, 0.67076E+00_JPRB, 0.67326E+00_JPRB, 0.67254E+00_JPRB ,&
&  0.66438E+00_JPRB, 0.64232E+00_JPRB, 0.63556E+00_JPRB, 0.63230E+00_JPRB, 0.61296E+00_JPRB, 0.59811E+00_JPRB, 0.60216E+00_JPRB ,&
&  0.61996E+00_JPRB, 0.58284E+00_JPRB, 0.49862E+00_JPRB, 0.45562E+00_JPRB, 0.43376E+00_JPRB, 0.43898E+00_JPRB, 0.45651E+00_JPRB ,&
&  0.47942E+00_JPRB, 0.49722E+00_JPRB, 0.51808E+00_JPRB, 0.54699E+00_JPRB, 0.59194E+00_JPRB, 0.64150E+00_JPRB, 0.73079E+00_JPRB ,&
&  0.84219E+00_JPRB, 0.77113E+00_JPRB, 0.79388E+00_JPRB, 0.79190E+00_JPRB, 0.82480E+00_JPRB, 0.81583E+00_JPRB, 0.79727E+00_JPRB ,&
&  0.76681E+00_JPRB, 0.74841E+00_JPRB, 0.73319E+00_JPRB, 0.71396E+00_JPRB, 0.69600E+00_JPRB, 0.68720E+00_JPRB, 0.69396E+00_JPRB ,&
&  0.72104E+00_JPRB, 0.72964E+00_JPRB, 0.71613E+00_JPRB, 0.72617E+00_JPRB, 0.80451E+00_JPRB, 0.82223E+00_JPRB, 0.81326E+00_JPRB ,&
&  0.76404E+00_JPRB, 0.79900E+00_JPRB, 0.10323E+01_JPRB, 0.12053E+01_JPRB, 0.13271E+01_JPRB, 0.14311E+01_JPRB, 0.14651E+01_JPRB ,&
&  0.15529E+01_JPRB, 0.15195E+01_JPRB, 0.15507E+01_JPRB, 0.13980E+01_JPRB, 0.15589E+01_JPRB, 0.15570E+01_JPRB, 0.15150E+01_JPRB ,&
&  0.14448E+01_JPRB, 0.14203E+01_JPRB, 0.13720E+01_JPRB, 0.14101E+01_JPRB, 0.14008E+01_JPRB, 0.13880E+01_JPRB, 0.11070E+01_JPRB ,&
&  0.91405E+00_JPRB, 0.95422E+00_JPRB, 0.86752E+00_JPRB, 0.77397E+00_JPRB, 0.68941E+00_JPRB, 0.69788E+00_JPRB, 0.71183E+00_JPRB ,&
&  0.72739E+00_JPRB, 0.73947E+00_JPRB, 0.73500E+00_JPRB, 0.77246E+00_JPRB, 0.88411E+00_JPRB, 0.99896E+00_JPRB, 0.99975E+00_JPRB ,&
&  0.92759E+00_JPRB, 0.94601E+00_JPRB, 0.95308E+00_JPRB, 0.93059E+00_JPRB, 0.95350E+00_JPRB, 0.92866E+00_JPRB, 0.88489E+00_JPRB ,&
&  0.83872E+00_JPRB, 0.80510E+00_JPRB, 0.77778E+00_JPRB, 0.74775E+00_JPRB, 0.71531E+00_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.76261E+00_JPRB, 0.74406E+00_JPRB, 0.72798E+00_JPRB, 0.71750E+00_JPRB, 0.70834E+00_JPRB, 0.70282E+00_JPRB, 0.69585E+00_JPRB ,&
&  0.67919E+00_JPRB, 0.65104E+00_JPRB, 0.65298E+00_JPRB, 0.64455E+00_JPRB, 0.62869E+00_JPRB, 0.61542E+00_JPRB, 0.62181E+00_JPRB ,&
&  0.63862E+00_JPRB, 0.57474E+00_JPRB, 0.49206E+00_JPRB, 0.48083E+00_JPRB, 0.46479E+00_JPRB, 0.44239E+00_JPRB, 0.44891E+00_JPRB ,&
&  0.46835E+00_JPRB, 0.49071E+00_JPRB, 0.51260E+00_JPRB, 0.56416E+00_JPRB, 0.63429E+00_JPRB, 0.72132E+00_JPRB, 0.84175E+00_JPRB ,&
&  0.86239E+00_JPRB, 0.89042E+00_JPRB, 0.94757E+00_JPRB, 0.91375E+00_JPRB, 0.87504E+00_JPRB, 0.83402E+00_JPRB, 0.79566E+00_JPRB ,&
&  0.76836E+00_JPRB, 0.74846E+00_JPRB, 0.73276E+00_JPRB, 0.72014E+00_JPRB, 0.72221E+00_JPRB, 0.72555E+00_JPRB, 0.73215E+00_JPRB ,&
&  0.74211E+00_JPRB, 0.73299E+00_JPRB, 0.71005E+00_JPRB, 0.78229E+00_JPRB, 0.71207E+00_JPRB, 0.68810E+00_JPRB, 0.69479E+00_JPRB ,&
&  0.87715E+00_JPRB, 0.10234E+01_JPRB, 0.11240E+01_JPRB, 0.13702E+01_JPRB, 0.12175E+01_JPRB, 0.13212E+01_JPRB, 0.15893E+01_JPRB ,&
&  0.16505E+01_JPRB, 0.16477E+01_JPRB, 0.17483E+01_JPRB, 0.14345E+01_JPRB, 0.12877E+01_JPRB, 0.16952E+01_JPRB, 0.17053E+01_JPRB ,&
&  0.16425E+01_JPRB, 0.16136E+01_JPRB, 0.15956E+01_JPRB, 0.17558E+01_JPRB, 0.12365E+01_JPRB, 0.10231E+01_JPRB, 0.81976E+00_JPRB ,&
&  0.81033E+00_JPRB, 0.77515E+00_JPRB, 0.85802E+00_JPRB, 0.81757E+00_JPRB, 0.80711E+00_JPRB, 0.78129E+00_JPRB, 0.73833E+00_JPRB ,&
&  0.73184E+00_JPRB, 0.72973E+00_JPRB, 0.73842E+00_JPRB, 0.84116E+00_JPRB, 0.10199E+01_JPRB, 0.10520E+01_JPRB, 0.11421E+01_JPRB ,&
&  0.11637E+01_JPRB, 0.12225E+01_JPRB, 0.11672E+01_JPRB, 0.11621E+01_JPRB, 0.11354E+01_JPRB, 0.10860E+01_JPRB, 0.10168E+01_JPRB ,&
&  0.96388E+00_JPRB, 0.91709E+00_JPRB, 0.86969E+00_JPRB, 0.82743E+00_JPRB, 0.79117E+00_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.81798E+00_JPRB, 0.78989E+00_JPRB, 0.76459E+00_JPRB, 0.74352E+00_JPRB, 0.73249E+00_JPRB, 0.72283E+00_JPRB, 0.70904E+00_JPRB ,&
&  0.68144E+00_JPRB, 0.65712E+00_JPRB, 0.65991E+00_JPRB, 0.63568E+00_JPRB, 0.61885E+00_JPRB, 0.61976E+00_JPRB, 0.64082E+00_JPRB ,&
&  0.65212E+00_JPRB, 0.62519E+00_JPRB, 0.53525E+00_JPRB, 0.49470E+00_JPRB, 0.49734E+00_JPRB, 0.46785E+00_JPRB, 0.45667E+00_JPRB ,&
&  0.47975E+00_JPRB, 0.50437E+00_JPRB, 0.54835E+00_JPRB, 0.64195E+00_JPRB, 0.74803E+00_JPRB, 0.86546E+00_JPRB, 0.99762E+00_JPRB ,&
&  0.10167E+01_JPRB, 0.10024E+01_JPRB, 0.95547E+00_JPRB, 0.89527E+00_JPRB, 0.83148E+00_JPRB, 0.78888E+00_JPRB, 0.75714E+00_JPRB ,&
&  0.73487E+00_JPRB, 0.72511E+00_JPRB, 0.72763E+00_JPRB, 0.72079E+00_JPRB, 0.72895E+00_JPRB, 0.70111E+00_JPRB, 0.69353E+00_JPRB ,&
&  0.70365E+00_JPRB, 0.71367E+00_JPRB, 0.72688E+00_JPRB, 0.83993E+00_JPRB, 0.79201E+00_JPRB, 0.72841E+00_JPRB, 0.75679E+00_JPRB ,&
&  0.86956E+00_JPRB, 0.96125E+00_JPRB, 0.11132E+01_JPRB, 0.13427E+01_JPRB, 0.16539E+01_JPRB, 0.17780E+01_JPRB, 0.19115E+01_JPRB ,&
&  0.15579E+01_JPRB, 0.13830E+01_JPRB, 0.14127E+01_JPRB, 0.13525E+01_JPRB, 0.17304E+01_JPRB, 0.23522E+01_JPRB, 0.23769E+01_JPRB ,&
&  0.22642E+01_JPRB, 0.21373E+01_JPRB, 0.21224E+01_JPRB, 0.17232E+01_JPRB, 0.12177E+01_JPRB, 0.91226E+00_JPRB, 0.10350E+01_JPRB ,&
&  0.10732E+01_JPRB, 0.10967E+01_JPRB, 0.99664E+00_JPRB, 0.86420E+00_JPRB, 0.76575E+00_JPRB, 0.86728E+00_JPRB, 0.84814E+00_JPRB ,&
&  0.79992E+00_JPRB, 0.77893E+00_JPRB, 0.92451E+00_JPRB, 0.11334E+01_JPRB, 0.12506E+01_JPRB, 0.13133E+01_JPRB, 0.15060E+01_JPRB ,&
&  0.15762E+01_JPRB, 0.16462E+01_JPRB, 0.15765E+01_JPRB, 0.14709E+01_JPRB, 0.13595E+01_JPRB, 0.12356E+01_JPRB, 0.11338E+01_JPRB ,&
&  0.10545E+01_JPRB, 0.98745E+00_JPRB, 0.93030E+00_JPRB, 0.88257E+00_JPRB, 0.84731E+00_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.86249E+00_JPRB, 0.82384E+00_JPRB, 0.78991E+00_JPRB, 0.76919E+00_JPRB, 0.75244E+00_JPRB, 0.73569E+00_JPRB, 0.71089E+00_JPRB ,&
&  0.67428E+00_JPRB, 0.67043E+00_JPRB, 0.66427E+00_JPRB, 0.63575E+00_JPRB, 0.62500E+00_JPRB, 0.62918E+00_JPRB, 0.64389E+00_JPRB ,&
&  0.65741E+00_JPRB, 0.68528E+00_JPRB, 0.58898E+00_JPRB, 0.54603E+00_JPRB, 0.53667E+00_JPRB, 0.51959E+00_JPRB, 0.48324E+00_JPRB ,&
&  0.50771E+00_JPRB, 0.54071E+00_JPRB, 0.60081E+00_JPRB, 0.72159E+00_JPRB, 0.89851E+00_JPRB, 0.97217E+00_JPRB, 0.99414E+00_JPRB ,&
&  0.10264E+01_JPRB, 0.98819E+00_JPRB, 0.92948E+00_JPRB, 0.87322E+00_JPRB, 0.81190E+00_JPRB, 0.78151E+00_JPRB, 0.76433E+00_JPRB ,&
&  0.75700E+00_JPRB, 0.77039E+00_JPRB, 0.77496E+00_JPRB, 0.79516E+00_JPRB, 0.76732E+00_JPRB, 0.72860E+00_JPRB, 0.71375E+00_JPRB ,&
&  0.70621E+00_JPRB, 0.71712E+00_JPRB, 0.74494E+00_JPRB, 0.86781E+00_JPRB, 0.91460E+00_JPRB, 0.86210E+00_JPRB, 0.96035E+00_JPRB ,&
&  0.10318E+01_JPRB, 0.11708E+01_JPRB, 0.12430E+01_JPRB, 0.14860E+01_JPRB, 0.17004E+01_JPRB, 0.18147E+01_JPRB, 0.19543E+01_JPRB ,&
&  0.18411E+01_JPRB, 0.19535E+01_JPRB, 0.23916E+01_JPRB, 0.29207E+01_JPRB, 0.25755E+01_JPRB, 0.25693E+01_JPRB, 0.34598E+01_JPRB ,&
&  0.25303E+01_JPRB, 0.24706E+01_JPRB, 0.25534E+01_JPRB, 0.24589E+01_JPRB, 0.10252E+01_JPRB, 0.79375E+00_JPRB, 0.10686E+01_JPRB ,&
&  0.10961E+01_JPRB, 0.76569E+00_JPRB, 0.62427E+00_JPRB, 0.70852E+00_JPRB, 0.60651E+00_JPRB, 0.69552E+00_JPRB, 0.92109E+00_JPRB ,&
&  0.96038E+00_JPRB, 0.98280E+00_JPRB, 0.12727E+01_JPRB, 0.14267E+01_JPRB, 0.14227E+01_JPRB, 0.17577E+01_JPRB, 0.19800E+01_JPRB ,&
&  0.20523E+01_JPRB, 0.19465E+01_JPRB, 0.19514E+01_JPRB, 0.17697E+01_JPRB, 0.15577E+01_JPRB, 0.13733E+01_JPRB, 0.12297E+01_JPRB ,&
&  0.11101E+01_JPRB, 0.10281E+01_JPRB, 0.97461E+00_JPRB, 0.92718E+00_JPRB, 0.89146E+00_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.85563E+00_JPRB, 0.81947E+00_JPRB, 0.78434E+00_JPRB, 0.76175E+00_JPRB, 0.74819E+00_JPRB, 0.73139E+00_JPRB, 0.70550E+00_JPRB ,&
&  0.68040E+00_JPRB, 0.69195E+00_JPRB, 0.66588E+00_JPRB, 0.64332E+00_JPRB, 0.62755E+00_JPRB, 0.62662E+00_JPRB, 0.64196E+00_JPRB ,&
&  0.65423E+00_JPRB, 0.67154E+00_JPRB, 0.65274E+00_JPRB, 0.60985E+00_JPRB, 0.59344E+00_JPRB, 0.56739E+00_JPRB, 0.54536E+00_JPRB ,&
&  0.57001E+00_JPRB, 0.61718E+00_JPRB, 0.69157E+00_JPRB, 0.81058E+00_JPRB, 0.90415E+00_JPRB, 0.94747E+00_JPRB, 0.10142E+01_JPRB ,&
&  0.10052E+01_JPRB, 0.95762E+00_JPRB, 0.90649E+00_JPRB, 0.85771E+00_JPRB, 0.81869E+00_JPRB, 0.79921E+00_JPRB, 0.78218E+00_JPRB ,&
&  0.77792E+00_JPRB, 0.78485E+00_JPRB, 0.79895E+00_JPRB, 0.76751E+00_JPRB, 0.73681E+00_JPRB, 0.71560E+00_JPRB, 0.69853E+00_JPRB ,&
&  0.69721E+00_JPRB, 0.73793E+00_JPRB, 0.79767E+00_JPRB, 0.88730E+00_JPRB, 0.10450E+01_JPRB, 0.11023E+01_JPRB, 0.10199E+01_JPRB ,&
&  0.95226E+00_JPRB, 0.10529E+01_JPRB, 0.13749E+01_JPRB, 0.14156E+01_JPRB, 0.13999E+01_JPRB, 0.15006E+01_JPRB, 0.15394E+01_JPRB ,&
&  0.17646E+01_JPRB, 0.22485E+01_JPRB, 0.23689E+01_JPRB, 0.29366E+01_JPRB, 0.31621E+01_JPRB, 0.24000E+01_JPRB, 0.27178E+01_JPRB ,&
&  0.26798E+01_JPRB, 0.23281E+01_JPRB, 0.17843E+01_JPRB, 0.13074E+01_JPRB, 0.13436E+01_JPRB, 0.11232E+01_JPRB, 0.72706E+00_JPRB ,&
&  0.65202E+00_JPRB, 0.61982E+00_JPRB, 0.59119E+00_JPRB, 0.59391E+00_JPRB, 0.58913E+00_JPRB, 0.70631E+00_JPRB, 0.91504E+00_JPRB ,&
&  0.11990E+01_JPRB, 0.13787E+01_JPRB, 0.15537E+01_JPRB, 0.15994E+01_JPRB, 0.15017E+01_JPRB, 0.18087E+01_JPRB, 0.22672E+01_JPRB ,&
&  0.19421E+01_JPRB, 0.18029E+01_JPRB, 0.17685E+01_JPRB, 0.16096E+01_JPRB, 0.14445E+01_JPRB, 0.13014E+01_JPRB, 0.11786E+01_JPRB ,&
&  0.10843E+01_JPRB, 0.10215E+01_JPRB, 0.97622E+00_JPRB, 0.93364E+00_JPRB, 0.89447E+00_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.83293E+00_JPRB, 0.80605E+00_JPRB, 0.78706E+00_JPRB, 0.76995E+00_JPRB, 0.75153E+00_JPRB, 0.73965E+00_JPRB, 0.70737E+00_JPRB ,&
&  0.70500E+00_JPRB, 0.69714E+00_JPRB, 0.65605E+00_JPRB, 0.62597E+00_JPRB, 0.60983E+00_JPRB, 0.61212E+00_JPRB, 0.62901E+00_JPRB ,&
&  0.63940E+00_JPRB, 0.65916E+00_JPRB, 0.70295E+00_JPRB, 0.69120E+00_JPRB, 0.72347E+00_JPRB, 0.67021E+00_JPRB, 0.67329E+00_JPRB ,&
&  0.68515E+00_JPRB, 0.74424E+00_JPRB, 0.83383E+00_JPRB, 0.90148E+00_JPRB, 0.94690E+00_JPRB, 0.10067E+01_JPRB, 0.10047E+01_JPRB ,&
&  0.97934E+00_JPRB, 0.96558E+00_JPRB, 0.89722E+00_JPRB, 0.85022E+00_JPRB, 0.82975E+00_JPRB, 0.80341E+00_JPRB, 0.78667E+00_JPRB ,&
&  0.78875E+00_JPRB, 0.80411E+00_JPRB, 0.77362E+00_JPRB, 0.75910E+00_JPRB, 0.75322E+00_JPRB, 0.74054E+00_JPRB, 0.72835E+00_JPRB ,&
&  0.74947E+00_JPRB, 0.80333E+00_JPRB, 0.87499E+00_JPRB, 0.94981E+00_JPRB, 0.94630E+00_JPRB, 0.82730E+00_JPRB, 0.94102E+00_JPRB ,&
&  0.10449E+01_JPRB, 0.12478E+01_JPRB, 0.13058E+01_JPRB, 0.15087E+01_JPRB, 0.14115E+01_JPRB, 0.13630E+01_JPRB, 0.15198E+01_JPRB ,&
&  0.18851E+01_JPRB, 0.26431E+01_JPRB, 0.23370E+01_JPRB, 0.26851E+01_JPRB, 0.35968E+01_JPRB, 0.27472E+01_JPRB, 0.23301E+01_JPRB ,&
&  0.25122E+01_JPRB, 0.21017E+01_JPRB, 0.17291E+01_JPRB, 0.14866E+01_JPRB, 0.19887E+01_JPRB, 0.21068E+01_JPRB, 0.98507E+00_JPRB ,&
&  0.76105E+00_JPRB, 0.71859E+00_JPRB, 0.68287E+00_JPRB, 0.64046E+00_JPRB, 0.64248E+00_JPRB, 0.80815E+00_JPRB, 0.11704E+01_JPRB ,&
&  0.13405E+01_JPRB, 0.16850E+01_JPRB, 0.18007E+01_JPRB, 0.17566E+01_JPRB, 0.15186E+01_JPRB, 0.17628E+01_JPRB, 0.17998E+01_JPRB ,&
&  0.19624E+01_JPRB, 0.18759E+01_JPRB, 0.17016E+01_JPRB, 0.15340E+01_JPRB, 0.13705E+01_JPRB, 0.12331E+01_JPRB, 0.11247E+01_JPRB ,&
&  0.10359E+01_JPRB, 0.97664E+00_JPRB, 0.93443E+00_JPRB, 0.90094E+00_JPRB, 0.86424E+00_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.83636E+00_JPRB, 0.81172E+00_JPRB, 0.78915E+00_JPRB, 0.76614E+00_JPRB, 0.75642E+00_JPRB, 0.74127E+00_JPRB, 0.71342E+00_JPRB ,&
&  0.73890E+00_JPRB, 0.69098E+00_JPRB, 0.64358E+00_JPRB, 0.61266E+00_JPRB, 0.60306E+00_JPRB, 0.60557E+00_JPRB, 0.61710E+00_JPRB ,&
&  0.63621E+00_JPRB, 0.65282E+00_JPRB, 0.70520E+00_JPRB, 0.82975E+00_JPRB, 0.88380E+00_JPRB, 0.82773E+00_JPRB, 0.87518E+00_JPRB ,&
&  0.86067E+00_JPRB, 0.90816E+00_JPRB, 0.99668E+00_JPRB, 0.10103E+01_JPRB, 0.10375E+01_JPRB, 0.10722E+01_JPRB, 0.10187E+01_JPRB ,&
&  0.96911E+00_JPRB, 0.96588E+00_JPRB, 0.90390E+00_JPRB, 0.87153E+00_JPRB, 0.84602E+00_JPRB, 0.83822E+00_JPRB, 0.82534E+00_JPRB ,&
&  0.83209E+00_JPRB, 0.80700E+00_JPRB, 0.79299E+00_JPRB, 0.79218E+00_JPRB, 0.79504E+00_JPRB, 0.79255E+00_JPRB, 0.80528E+00_JPRB ,&
&  0.85497E+00_JPRB, 0.90627E+00_JPRB, 0.97803E+00_JPRB, 0.10274E+01_JPRB, 0.85102E+00_JPRB, 0.97823E+00_JPRB, 0.10178E+01_JPRB ,&
&  0.10646E+01_JPRB, 0.11334E+01_JPRB, 0.11280E+01_JPRB, 0.12694E+01_JPRB, 0.13849E+01_JPRB, 0.13695E+01_JPRB, 0.15087E+01_JPRB ,&
&  0.21134E+01_JPRB, 0.21590E+01_JPRB, 0.22146E+01_JPRB, 0.26519E+01_JPRB, 0.30036E+01_JPRB, 0.40334E+01_JPRB, 0.20962E+01_JPRB ,&
&  0.16130E+01_JPRB, 0.19128E+01_JPRB, 0.17938E+01_JPRB, 0.17071E+01_JPRB, 0.20956E+01_JPRB, 0.26784E+01_JPRB, 0.22560E+01_JPRB ,&
&  0.93934E+00_JPRB, 0.74458E+00_JPRB, 0.83123E+00_JPRB, 0.83624E+00_JPRB, 0.76829E+00_JPRB, 0.97559E+00_JPRB, 0.18531E+01_JPRB ,&
&  0.17600E+01_JPRB, 0.19706E+01_JPRB, 0.19237E+01_JPRB, 0.16790E+01_JPRB, 0.15216E+01_JPRB, 0.16927E+01_JPRB, 0.20113E+01_JPRB ,&
&  0.18809E+01_JPRB, 0.16688E+01_JPRB, 0.15187E+01_JPRB, 0.13974E+01_JPRB, 0.12550E+01_JPRB, 0.11264E+01_JPRB, 0.10244E+01_JPRB ,&
&  0.95509E+00_JPRB, 0.91779E+00_JPRB, 0.89799E+00_JPRB, 0.88035E+00_JPRB, 0.85958E+00_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.80547E+00_JPRB, 0.78796E+00_JPRB, 0.77564E+00_JPRB, 0.76843E+00_JPRB, 0.77372E+00_JPRB, 0.75860E+00_JPRB, 0.74119E+00_JPRB ,&
&  0.77289E+00_JPRB, 0.68828E+00_JPRB, 0.64208E+00_JPRB, 0.60827E+00_JPRB, 0.60422E+00_JPRB, 0.61447E+00_JPRB, 0.62224E+00_JPRB ,&
&  0.63029E+00_JPRB, 0.65775E+00_JPRB, 0.73603E+00_JPRB, 0.90790E+00_JPRB, 0.10169E+01_JPRB, 0.97250E+00_JPRB, 0.10032E+01_JPRB ,&
&  0.10799E+01_JPRB, 0.10840E+01_JPRB, 0.10540E+01_JPRB, 0.10537E+01_JPRB, 0.11316E+01_JPRB, 0.11260E+01_JPRB, 0.10556E+01_JPRB ,&
&  0.99545E+00_JPRB, 0.99247E+00_JPRB, 0.94922E+00_JPRB, 0.92073E+00_JPRB, 0.91173E+00_JPRB, 0.90458E+00_JPRB, 0.90411E+00_JPRB ,&
&  0.87110E+00_JPRB, 0.85764E+00_JPRB, 0.85467E+00_JPRB, 0.85408E+00_JPRB, 0.87390E+00_JPRB, 0.87233E+00_JPRB, 0.91408E+00_JPRB ,&
&  0.97942E+00_JPRB, 0.10250E+01_JPRB, 0.10711E+01_JPRB, 0.10166E+01_JPRB, 0.10245E+01_JPRB, 0.10333E+01_JPRB, 0.10815E+01_JPRB ,&
&  0.10901E+01_JPRB, 0.11343E+01_JPRB, 0.12051E+01_JPRB, 0.12768E+01_JPRB, 0.14154E+01_JPRB, 0.15300E+01_JPRB, 0.17346E+01_JPRB ,&
&  0.21323E+01_JPRB, 0.23020E+01_JPRB, 0.22521E+01_JPRB, 0.24232E+01_JPRB, 0.27628E+01_JPRB, 0.34181E+01_JPRB, 0.26552E+01_JPRB ,&
&  0.19713E+01_JPRB, 0.16297E+01_JPRB, 0.17181E+01_JPRB, 0.20982E+01_JPRB, 0.24025E+01_JPRB, 0.27832E+01_JPRB, 0.34940E+01_JPRB ,&
&  0.37882E+01_JPRB, 0.21688E+01_JPRB, 0.17535E+01_JPRB, 0.20320E+01_JPRB, 0.15838E+01_JPRB, 0.13760E+01_JPRB, 0.17090E+01_JPRB ,&
&  0.19168E+01_JPRB, 0.19976E+01_JPRB, 0.19056E+01_JPRB, 0.15822E+01_JPRB, 0.15673E+01_JPRB, 0.18006E+01_JPRB, 0.18944E+01_JPRB ,&
&  0.16181E+01_JPRB, 0.14662E+01_JPRB, 0.13172E+01_JPRB, 0.11674E+01_JPRB, 0.10495E+01_JPRB, 0.96812E+00_JPRB, 0.91957E+00_JPRB ,&
&  0.89860E+00_JPRB, 0.88584E+00_JPRB, 0.87225E+00_JPRB, 0.85079E+00_JPRB, 0.82730E+00_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.78521E+00_JPRB, 0.78730E+00_JPRB, 0.79853E+00_JPRB, 0.80362E+00_JPRB, 0.79713E+00_JPRB, 0.77279E+00_JPRB, 0.78640E+00_JPRB ,&
&  0.81019E+00_JPRB, 0.70405E+00_JPRB, 0.64555E+00_JPRB, 0.62074E+00_JPRB, 0.62433E+00_JPRB, 0.64042E+00_JPRB, 0.64813E+00_JPRB ,&
&  0.66325E+00_JPRB, 0.71464E+00_JPRB, 0.86960E+00_JPRB, 0.10225E+01_JPRB, 0.11838E+01_JPRB, 0.14297E+01_JPRB, 0.10643E+01_JPRB ,&
&  0.11176E+01_JPRB, 0.11347E+01_JPRB, 0.10799E+01_JPRB, 0.11312E+01_JPRB, 0.11840E+01_JPRB, 0.11746E+01_JPRB, 0.11069E+01_JPRB ,&
&  0.10498E+01_JPRB, 0.10573E+01_JPRB, 0.10337E+01_JPRB, 0.10098E+01_JPRB, 0.10071E+01_JPRB, 0.10176E+01_JPRB, 0.98281E+00_JPRB ,&
&  0.98216E+00_JPRB, 0.97309E+00_JPRB, 0.94848E+00_JPRB, 0.96566E+00_JPRB, 0.98627E+00_JPRB, 0.99597E+00_JPRB, 0.10520E+01_JPRB ,&
&  0.11007E+01_JPRB, 0.11257E+01_JPRB, 0.11329E+01_JPRB, 0.10765E+01_JPRB, 0.11104E+01_JPRB, 0.11379E+01_JPRB, 0.11846E+01_JPRB ,&
&  0.11369E+01_JPRB, 0.10648E+01_JPRB, 0.12196E+01_JPRB, 0.14247E+01_JPRB, 0.15884E+01_JPRB, 0.16405E+01_JPRB, 0.18445E+01_JPRB ,&
&  0.23005E+01_JPRB, 0.26490E+01_JPRB, 0.27798E+01_JPRB, 0.23050E+01_JPRB, 0.25202E+01_JPRB, 0.29844E+01_JPRB, 0.30436E+01_JPRB ,&
&  0.27669E+01_JPRB, 0.22396E+01_JPRB, 0.22627E+01_JPRB, 0.23500E+01_JPRB, 0.24431E+01_JPRB, 0.27539E+01_JPRB, 0.28112E+01_JPRB ,&
&  0.32592E+01_JPRB, 0.39674E+01_JPRB, 0.33716E+01_JPRB, 0.26612E+01_JPRB, 0.22443E+01_JPRB, 0.19285E+01_JPRB, 0.18959E+01_JPRB ,&
&  0.20845E+01_JPRB, 0.19555E+01_JPRB, 0.17651E+01_JPRB, 0.18024E+01_JPRB, 0.16376E+01_JPRB, 0.16970E+01_JPRB, 0.15937E+01_JPRB ,&
&  0.13257E+01_JPRB, 0.11572E+01_JPRB, 0.99676E+00_JPRB, 0.90824E+00_JPRB, 0.86036E+00_JPRB, 0.84962E+00_JPRB, 0.84333E+00_JPRB ,&
&  0.84223E+00_JPRB, 0.83511E+00_JPRB, 0.82312E+00_JPRB, 0.81329E+00_JPRB, 0.79720E+00_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.80373E+00_JPRB, 0.81311E+00_JPRB, 0.83482E+00_JPRB, 0.84204E+00_JPRB, 0.82798E+00_JPRB, 0.82891E+00_JPRB, 0.85254E+00_JPRB ,&
&  0.85258E+00_JPRB, 0.70522E+00_JPRB, 0.67914E+00_JPRB, 0.67972E+00_JPRB, 0.69413E+00_JPRB, 0.69900E+00_JPRB, 0.70183E+00_JPRB ,&
&  0.74048E+00_JPRB, 0.84832E+00_JPRB, 0.10179E+01_JPRB, 0.11354E+01_JPRB, 0.12489E+01_JPRB, 0.13987E+01_JPRB, 0.13405E+01_JPRB ,&
&  0.10718E+01_JPRB, 0.12635E+01_JPRB, 0.11513E+01_JPRB, 0.12669E+01_JPRB, 0.12794E+01_JPRB, 0.13291E+01_JPRB, 0.12512E+01_JPRB ,&
&  0.11777E+01_JPRB, 0.11786E+01_JPRB, 0.12193E+01_JPRB, 0.11763E+01_JPRB, 0.11456E+01_JPRB, 0.10952E+01_JPRB, 0.11115E+01_JPRB ,&
&  0.11180E+01_JPRB, 0.11032E+01_JPRB, 0.10976E+01_JPRB, 0.11259E+01_JPRB, 0.11446E+01_JPRB, 0.11493E+01_JPRB, 0.11646E+01_JPRB ,&
&  0.11777E+01_JPRB, 0.11846E+01_JPRB, 0.11631E+01_JPRB, 0.11820E+01_JPRB, 0.12036E+01_JPRB, 0.12684E+01_JPRB, 0.13254E+01_JPRB ,&
&  0.13018E+01_JPRB, 0.12476E+01_JPRB, 0.13589E+01_JPRB, 0.14899E+01_JPRB, 0.16068E+01_JPRB, 0.18297E+01_JPRB, 0.19841E+01_JPRB ,&
&  0.24272E+01_JPRB, 0.24860E+01_JPRB, 0.29188E+01_JPRB, 0.21382E+01_JPRB, 0.22538E+01_JPRB, 0.26219E+01_JPRB, 0.25101E+01_JPRB ,&
&  0.20339E+01_JPRB, 0.20670E+01_JPRB, 0.21361E+01_JPRB, 0.22975E+01_JPRB, 0.24153E+01_JPRB, 0.25378E+01_JPRB, 0.23874E+01_JPRB ,&
&  0.24351E+01_JPRB, 0.26242E+01_JPRB, 0.25465E+01_JPRB, 0.21349E+01_JPRB, 0.19742E+01_JPRB, 0.17726E+01_JPRB, 0.19292E+01_JPRB ,&
&  0.20815E+01_JPRB, 0.20069E+01_JPRB, 0.19963E+01_JPRB, 0.19178E+01_JPRB, 0.14992E+01_JPRB, 0.13852E+01_JPRB, 0.11972E+01_JPRB ,&
&  0.10096E+01_JPRB, 0.90346E+00_JPRB, 0.80606E+00_JPRB, 0.79479E+00_JPRB, 0.79919E+00_JPRB, 0.79912E+00_JPRB, 0.81411E+00_JPRB ,&
&  0.81221E+00_JPRB, 0.80743E+00_JPRB, 0.81090E+00_JPRB, 0.80893E+00_JPRB, 0.80085E+00_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.83696E+00_JPRB, 0.84704E+00_JPRB, 0.87103E+00_JPRB, 0.90011E+00_JPRB, 0.92962E+00_JPRB, 0.97558E+00_JPRB, 0.93955E+00_JPRB ,&
&  0.90377E+00_JPRB, 0.73681E+00_JPRB, 0.74892E+00_JPRB, 0.75758E+00_JPRB, 0.76959E+00_JPRB, 0.76357E+00_JPRB, 0.78446E+00_JPRB ,&
&  0.86718E+00_JPRB, 0.99446E+00_JPRB, 0.11256E+01_JPRB, 0.12546E+01_JPRB, 0.13807E+01_JPRB, 0.15392E+01_JPRB, 0.15381E+01_JPRB ,&
&  0.12335E+01_JPRB, 0.12030E+01_JPRB, 0.13040E+01_JPRB, 0.13757E+01_JPRB, 0.13618E+01_JPRB, 0.14351E+01_JPRB, 0.14664E+01_JPRB ,&
&  0.14301E+01_JPRB, 0.14030E+01_JPRB, 0.14642E+01_JPRB, 0.13378E+01_JPRB, 0.11805E+01_JPRB, 0.11965E+01_JPRB, 0.12346E+01_JPRB ,&
&  0.12538E+01_JPRB, 0.12626E+01_JPRB, 0.12783E+01_JPRB, 0.13114E+01_JPRB, 0.13269E+01_JPRB, 0.13076E+01_JPRB, 0.12836E+01_JPRB ,&
&  0.12804E+01_JPRB, 0.12958E+01_JPRB, 0.12963E+01_JPRB, 0.12828E+01_JPRB, 0.13379E+01_JPRB, 0.14067E+01_JPRB, 0.14586E+01_JPRB ,&
&  0.14969E+01_JPRB, 0.15644E+01_JPRB, 0.16356E+01_JPRB, 0.17320E+01_JPRB, 0.17402E+01_JPRB, 0.20289E+01_JPRB, 0.21854E+01_JPRB ,&
&  0.23976E+01_JPRB, 0.24399E+01_JPRB, 0.26660E+01_JPRB, 0.19211E+01_JPRB, 0.21886E+01_JPRB, 0.23125E+01_JPRB, 0.22191E+01_JPRB ,&
&  0.19655E+01_JPRB, 0.20339E+01_JPRB, 0.20523E+01_JPRB, 0.21070E+01_JPRB, 0.22054E+01_JPRB, 0.20923E+01_JPRB, 0.21224E+01_JPRB ,&
&  0.20078E+01_JPRB, 0.22135E+01_JPRB, 0.21200E+01_JPRB, 0.19912E+01_JPRB, 0.17661E+01_JPRB, 0.16803E+01_JPRB, 0.17688E+01_JPRB ,&
&  0.19278E+01_JPRB, 0.18861E+01_JPRB, 0.17869E+01_JPRB, 0.15569E+01_JPRB, 0.11996E+01_JPRB, 0.10629E+01_JPRB, 0.92000E+00_JPRB ,&
&  0.86892E+00_JPRB, 0.80710E+00_JPRB, 0.78537E+00_JPRB, 0.79804E+00_JPRB, 0.79878E+00_JPRB, 0.82908E+00_JPRB, 0.82841E+00_JPRB ,&
&  0.81583E+00_JPRB, 0.82399E+00_JPRB, 0.83309E+00_JPRB, 0.83512E+00_JPRB, 0.83483E+00_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.82390E+00_JPRB, 0.81840E+00_JPRB, 0.81643E+00_JPRB, 0.82534E+00_JPRB, 0.84571E+00_JPRB, 0.85859E+00_JPRB, 0.81388E+00_JPRB ,&
&  0.79797E+00_JPRB, 0.79458E+00_JPRB, 0.82462E+00_JPRB, 0.83675E+00_JPRB, 0.84143E+00_JPRB, 0.84765E+00_JPRB, 0.89344E+00_JPRB ,&
&  0.97500E+00_JPRB, 0.10971E+01_JPRB, 0.12212E+01_JPRB, 0.13023E+01_JPRB, 0.13769E+01_JPRB, 0.14808E+01_JPRB, 0.15909E+01_JPRB ,&
&  0.14143E+01_JPRB, 0.11738E+01_JPRB, 0.13756E+01_JPRB, 0.14796E+01_JPRB, 0.14390E+01_JPRB, 0.15633E+01_JPRB, 0.17755E+01_JPRB ,&
&  0.15236E+01_JPRB, 0.14635E+01_JPRB, 0.14789E+01_JPRB, 0.14051E+01_JPRB, 0.12779E+01_JPRB, 0.13140E+01_JPRB, 0.13642E+01_JPRB ,&
&  0.14006E+01_JPRB, 0.14211E+01_JPRB, 0.14681E+01_JPRB, 0.15117E+01_JPRB, 0.15264E+01_JPRB, 0.15015E+01_JPRB, 0.14657E+01_JPRB ,&
&  0.14806E+01_JPRB, 0.15113E+01_JPRB, 0.15503E+01_JPRB, 0.15509E+01_JPRB, 0.15866E+01_JPRB, 0.16598E+01_JPRB, 0.17278E+01_JPRB ,&
&  0.17897E+01_JPRB, 0.18045E+01_JPRB, 0.19062E+01_JPRB, 0.21017E+01_JPRB, 0.22751E+01_JPRB, 0.20722E+01_JPRB, 0.22583E+01_JPRB ,&
&  0.22196E+01_JPRB, 0.21332E+01_JPRB, 0.20250E+01_JPRB, 0.23577E+01_JPRB, 0.16745E+01_JPRB, 0.19073E+01_JPRB, 0.17677E+01_JPRB ,&
&  0.19919E+01_JPRB, 0.19410E+01_JPRB, 0.19275E+01_JPRB, 0.19802E+01_JPRB, 0.20494E+01_JPRB, 0.18855E+01_JPRB, 0.18659E+01_JPRB ,&
&  0.19060E+01_JPRB, 0.19371E+01_JPRB, 0.19184E+01_JPRB, 0.19094E+01_JPRB, 0.17783E+01_JPRB, 0.17528E+01_JPRB, 0.17271E+01_JPRB ,&
&  0.17813E+01_JPRB, 0.16069E+01_JPRB, 0.14045E+01_JPRB, 0.11738E+01_JPRB, 0.94807E+00_JPRB, 0.87782E+00_JPRB, 0.86194E+00_JPRB ,&
&  0.86822E+00_JPRB, 0.88285E+00_JPRB, 0.84258E+00_JPRB, 0.82311E+00_JPRB, 0.82796E+00_JPRB, 0.85586E+00_JPRB, 0.84541E+00_JPRB ,&
&  0.84932E+00_JPRB, 0.84642E+00_JPRB, 0.85246E+00_JPRB, 0.84936E+00_JPRB, 0.84104E+00_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.82079E+00_JPRB, 0.81191E+00_JPRB, 0.80129E+00_JPRB, 0.81180E+00_JPRB, 0.81730E+00_JPRB, 0.81356E+00_JPRB, 0.80197E+00_JPRB ,&
&  0.80756E+00_JPRB, 0.83637E+00_JPRB, 0.87014E+00_JPRB, 0.88795E+00_JPRB, 0.89190E+00_JPRB, 0.90221E+00_JPRB, 0.94896E+00_JPRB ,&
&  0.10232E+01_JPRB, 0.11362E+01_JPRB, 0.12741E+01_JPRB, 0.13723E+01_JPRB, 0.14555E+01_JPRB, 0.15028E+01_JPRB, 0.16018E+01_JPRB ,&
&  0.15922E+01_JPRB, 0.16160E+01_JPRB, 0.15690E+01_JPRB, 0.13179E+01_JPRB, 0.12427E+01_JPRB, 0.15371E+01_JPRB, 0.15467E+01_JPRB ,&
&  0.14748E+01_JPRB, 0.14403E+01_JPRB, 0.14186E+01_JPRB, 0.14015E+01_JPRB, 0.13972E+01_JPRB, 0.14518E+01_JPRB, 0.15105E+01_JPRB ,&
&  0.15456E+01_JPRB, 0.16020E+01_JPRB, 0.16579E+01_JPRB, 0.16980E+01_JPRB, 0.17206E+01_JPRB, 0.17532E+01_JPRB, 0.17617E+01_JPRB ,&
&  0.18275E+01_JPRB, 0.18994E+01_JPRB, 0.19756E+01_JPRB, 0.20031E+01_JPRB, 0.19782E+01_JPRB, 0.20388E+01_JPRB, 0.20892E+01_JPRB ,&
&  0.21414E+01_JPRB, 0.21330E+01_JPRB, 0.21143E+01_JPRB, 0.21977E+01_JPRB, 0.22274E+01_JPRB, 0.19293E+01_JPRB, 0.19129E+01_JPRB ,&
&  0.20073E+01_JPRB, 0.19315E+01_JPRB, 0.17033E+01_JPRB, 0.21164E+01_JPRB, 0.13546E+01_JPRB, 0.16874E+01_JPRB, 0.19054E+01_JPRB ,&
&  0.18486E+01_JPRB, 0.18157E+01_JPRB, 0.18277E+01_JPRB, 0.18544E+01_JPRB, 0.18485E+01_JPRB, 0.17178E+01_JPRB, 0.17920E+01_JPRB ,&
&  0.18579E+01_JPRB, 0.18214E+01_JPRB, 0.17906E+01_JPRB, 0.17679E+01_JPRB, 0.17156E+01_JPRB, 0.15850E+01_JPRB, 0.14011E+01_JPRB ,&
&  0.12059E+01_JPRB, 0.13618E+01_JPRB, 0.12110E+01_JPRB, 0.11089E+01_JPRB, 0.95103E+00_JPRB, 0.82879E+00_JPRB, 0.85868E+00_JPRB ,&
&  0.88376E+00_JPRB, 0.87305E+00_JPRB, 0.85100E+00_JPRB, 0.85566E+00_JPRB, 0.85793E+00_JPRB, 0.86404E+00_JPRB, 0.85349E+00_JPRB ,&
&  0.85651E+00_JPRB, 0.85874E+00_JPRB, 0.85820E+00_JPRB, 0.84754E+00_JPRB, 0.83455E+00_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.79254E+00_JPRB, 0.78442E+00_JPRB, 0.77769E+00_JPRB, 0.80779E+00_JPRB, 0.80238E+00_JPRB, 0.78968E+00_JPRB, 0.79588E+00_JPRB ,&
&  0.80707E+00_JPRB, 0.82434E+00_JPRB, 0.84812E+00_JPRB, 0.87309E+00_JPRB, 0.90767E+00_JPRB, 0.96105E+00_JPRB, 0.10165E+01_JPRB ,&
&  0.10812E+01_JPRB, 0.11540E+01_JPRB, 0.12745E+01_JPRB, 0.14270E+01_JPRB, 0.16193E+01_JPRB, 0.17166E+01_JPRB, 0.17124E+01_JPRB ,&
&  0.16001E+01_JPRB, 0.16499E+01_JPRB, 0.17314E+01_JPRB, 0.17910E+01_JPRB, 0.15633E+01_JPRB, 0.16760E+01_JPRB, 0.18187E+01_JPRB ,&
&  0.18084E+01_JPRB, 0.16328E+01_JPRB, 0.15076E+01_JPRB, 0.14975E+01_JPRB, 0.15015E+01_JPRB, 0.15706E+01_JPRB, 0.16431E+01_JPRB ,&
&  0.17007E+01_JPRB, 0.17701E+01_JPRB, 0.18196E+01_JPRB, 0.18717E+01_JPRB, 0.19474E+01_JPRB, 0.20622E+01_JPRB, 0.21193E+01_JPRB ,&
&  0.21744E+01_JPRB, 0.22860E+01_JPRB, 0.23380E+01_JPRB, 0.22989E+01_JPRB, 0.23606E+01_JPRB, 0.23987E+01_JPRB, 0.24259E+01_JPRB ,&
&  0.24824E+01_JPRB, 0.23520E+01_JPRB, 0.23734E+01_JPRB, 0.24193E+01_JPRB, 0.22946E+01_JPRB, 0.20314E+01_JPRB, 0.19501E+01_JPRB ,&
&  0.19732E+01_JPRB, 0.19390E+01_JPRB, 0.14064E+01_JPRB, 0.18786E+01_JPRB, 0.22361E+01_JPRB, 0.19232E+01_JPRB, 0.18460E+01_JPRB ,&
&  0.17460E+01_JPRB, 0.17529E+01_JPRB, 0.17988E+01_JPRB, 0.18414E+01_JPRB, 0.18618E+01_JPRB, 0.18630E+01_JPRB, 0.17510E+01_JPRB ,&
&  0.18945E+01_JPRB, 0.18084E+01_JPRB, 0.16977E+01_JPRB, 0.16088E+01_JPRB, 0.14902E+01_JPRB, 0.12990E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.10084E+01_JPRB, 0.12520E+01_JPRB, 0.11905E+01_JPRB, 0.12470E+01_JPRB, 0.12197E+01_JPRB, 0.83780E+00_JPRB, 0.88003E+00_JPRB ,&
&  0.88364E+00_JPRB, 0.88036E+00_JPRB, 0.86451E+00_JPRB, 0.84067E+00_JPRB, 0.84041E+00_JPRB, 0.84540E+00_JPRB, 0.84824E+00_JPRB ,&
&  0.84550E+00_JPRB, 0.83482E+00_JPRB, 0.82950E+00_JPRB, 0.81258E+00_JPRB, 0.80286E+00_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.80035E+00_JPRB, 0.80131E+00_JPRB, 0.79631E+00_JPRB, 0.80959E+00_JPRB, 0.80272E+00_JPRB, 0.81698E+00_JPRB, 0.80033E+00_JPRB ,&
&  0.78381E+00_JPRB, 0.77984E+00_JPRB, 0.78435E+00_JPRB, 0.78972E+00_JPRB, 0.81312E+00_JPRB, 0.86300E+00_JPRB, 0.92562E+00_JPRB ,&
&  0.98731E+00_JPRB, 0.10022E+01_JPRB, 0.10434E+01_JPRB, 0.11519E+01_JPRB, 0.13820E+01_JPRB, 0.16068E+01_JPRB, 0.16590E+01_JPRB ,&
&  0.17542E+01_JPRB, 0.19036E+01_JPRB, 0.20343E+01_JPRB, 0.21691E+01_JPRB, 0.21708E+01_JPRB, 0.21594E+01_JPRB, 0.22175E+01_JPRB ,&
&  0.19210E+01_JPRB, 0.15744E+01_JPRB, 0.15187E+01_JPRB, 0.15398E+01_JPRB, 0.15928E+01_JPRB, 0.16540E+01_JPRB, 0.17081E+01_JPRB ,&
&  0.17675E+01_JPRB, 0.18431E+01_JPRB, 0.19131E+01_JPRB, 0.20112E+01_JPRB, 0.21502E+01_JPRB, 0.23102E+01_JPRB, 0.24110E+01_JPRB ,&
&  0.24698E+01_JPRB, 0.25542E+01_JPRB, 0.26668E+01_JPRB, 0.25676E+01_JPRB, 0.28657E+01_JPRB, 0.29640E+01_JPRB, 0.29995E+01_JPRB ,&
&  0.29428E+01_JPRB, 0.27630E+01_JPRB, 0.27538E+01_JPRB, 0.27288E+01_JPRB, 0.26859E+01_JPRB, 0.24123E+01_JPRB, 0.22174E+01_JPRB ,&
&  0.21713E+01_JPRB, 0.18410E+01_JPRB, 0.13292E+01_JPRB, 0.14367E+01_JPRB, 0.16014E+01_JPRB, 0.14555E+01_JPRB, 0.18058E+01_JPRB ,&
&  0.17322E+01_JPRB, 0.16892E+01_JPRB, 0.16934E+01_JPRB, 0.17197E+01_JPRB, 0.18027E+01_JPRB, 0.17802E+01_JPRB, 0.17353E+01_JPRB ,&
&  0.17590E+01_JPRB, 0.16273E+01_JPRB, 0.14834E+01_JPRB, 0.13359E+01_JPRB, 0.11855E+01_JPRB, 0.10625E+01_JPRB, 0.99979E+00_JPRB ,&
&  0.99716E+00_JPRB, 0.10119E+01_JPRB, 0.95829E+00_JPRB, 0.90665E+00_JPRB, 0.80757E+00_JPRB, 0.81107E+00_JPRB, 0.82162E+00_JPRB ,&
&  0.85388E+00_JPRB, 0.85179E+00_JPRB, 0.84073E+00_JPRB, 0.82124E+00_JPRB, 0.81101E+00_JPRB, 0.80403E+00_JPRB, 0.80310E+00_JPRB ,&
&  0.81335E+00_JPRB, 0.82625E+00_JPRB, 0.82211E+00_JPRB, 0.81569E+00_JPRB, 0.80764E+00_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.80237E+00_JPRB, 0.79870E+00_JPRB, 0.79784E+00_JPRB, 0.81316E+00_JPRB, 0.82592E+00_JPRB, 0.81489E+00_JPRB, 0.82210E+00_JPRB ,&
&  0.79962E+00_JPRB, 0.78170E+00_JPRB, 0.79184E+00_JPRB, 0.79718E+00_JPRB, 0.81355E+00_JPRB, 0.82847E+00_JPRB, 0.83713E+00_JPRB ,&
&  0.83452E+00_JPRB, 0.82652E+00_JPRB, 0.79248E+00_JPRB, 0.80182E+00_JPRB, 0.88671E+00_JPRB, 0.10301E+01_JPRB, 0.13030E+01_JPRB ,&
&  0.16067E+01_JPRB, 0.17613E+01_JPRB, 0.18788E+01_JPRB, 0.19090E+01_JPRB, 0.18279E+01_JPRB, 0.17038E+01_JPRB, 0.22218E+01_JPRB ,&
&  0.21647E+01_JPRB, 0.15596E+01_JPRB, 0.16110E+01_JPRB, 0.15176E+01_JPRB, 0.15707E+01_JPRB, 0.15950E+01_JPRB, 0.16462E+01_JPRB ,&
&  0.16883E+01_JPRB, 0.17696E+01_JPRB, 0.18875E+01_JPRB, 0.20450E+01_JPRB, 0.22055E+01_JPRB, 0.23752E+01_JPRB, 0.25083E+01_JPRB ,&
&  0.25940E+01_JPRB, 0.26759E+01_JPRB, 0.28334E+01_JPRB, 0.28831E+01_JPRB, 0.29104E+01_JPRB, 0.30639E+01_JPRB, 0.32246E+01_JPRB ,&
&  0.33366E+01_JPRB, 0.35261E+01_JPRB, 0.30800E+01_JPRB, 0.30261E+01_JPRB, 0.31142E+01_JPRB, 0.26288E+01_JPRB, 0.24671E+01_JPRB ,&
&  0.23742E+01_JPRB, 0.18278E+01_JPRB, 0.12133E+01_JPRB, 0.12889E+01_JPRB, 0.13999E+01_JPRB, 0.14857E+01_JPRB, 0.16029E+01_JPRB ,&
&  0.15409E+01_JPRB, 0.14701E+01_JPRB, 0.14772E+01_JPRB, 0.15622E+01_JPRB, 0.15832E+01_JPRB, 0.15282E+01_JPRB, 0.14594E+01_JPRB ,&
&  0.14138E+01_JPRB, 0.12830E+01_JPRB, 0.12110E+01_JPRB, 0.11029E+01_JPRB, 0.10268E+01_JPRB, 0.97078E+00_JPRB, 0.90298E+00_JPRB ,&
&  0.95273E+00_JPRB, 0.10144E+01_JPRB, 0.11605E+01_JPRB, 0.10586E+01_JPRB, 0.79114E+00_JPRB, 0.81231E+00_JPRB, 0.80840E+00_JPRB ,&
&  0.81688E+00_JPRB, 0.82204E+00_JPRB, 0.83810E+00_JPRB, 0.83104E+00_JPRB, 0.81352E+00_JPRB, 0.79635E+00_JPRB, 0.78126E+00_JPRB ,&
&  0.79617E+00_JPRB, 0.81504E+00_JPRB, 0.82383E+00_JPRB, 0.81217E+00_JPRB, 0.80256E+00_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.86838E+00_JPRB, 0.87126E+00_JPRB, 0.88871E+00_JPRB, 0.89217E+00_JPRB, 0.89178E+00_JPRB, 0.89924E+00_JPRB, 0.90066E+00_JPRB ,&
&  0.89404E+00_JPRB, 0.90099E+00_JPRB, 0.91080E+00_JPRB, 0.92215E+00_JPRB, 0.93692E+00_JPRB, 0.95024E+00_JPRB, 0.95846E+00_JPRB ,&
&  0.94978E+00_JPRB, 0.94125E+00_JPRB, 0.96581E+00_JPRB, 0.98526E+00_JPRB, 0.98247E+00_JPRB, 0.97688E+00_JPRB, 0.99694E+00_JPRB ,&
&  0.10495E+01_JPRB, 0.10818E+01_JPRB, 0.10927E+01_JPRB, 0.10812E+01_JPRB, 0.11185E+01_JPRB, 0.13330E+01_JPRB, 0.20704E+01_JPRB ,&
&  0.21026E+01_JPRB, 0.15810E+01_JPRB, 0.14542E+01_JPRB, 0.13254E+01_JPRB, 0.13607E+01_JPRB, 0.14537E+01_JPRB, 0.14661E+01_JPRB ,&
&  0.14572E+01_JPRB, 0.15023E+01_JPRB, 0.15521E+01_JPRB, 0.15996E+01_JPRB, 0.16729E+01_JPRB, 0.17296E+01_JPRB, 0.17608E+01_JPRB ,&
&  0.17315E+01_JPRB, 0.17290E+01_JPRB, 0.17904E+01_JPRB, 0.19010E+01_JPRB, 0.21853E+01_JPRB, 0.26289E+01_JPRB, 0.25458E+01_JPRB ,&
&  0.25506E+01_JPRB, 0.27648E+01_JPRB, 0.29223E+01_JPRB, 0.29927E+01_JPRB, 0.28453E+01_JPRB, 0.25306E+01_JPRB, 0.22632E+01_JPRB ,&
&  0.19920E+01_JPRB, 0.15995E+01_JPRB, 0.12920E+01_JPRB, 0.14467E+01_JPRB, 0.14821E+01_JPRB, 0.15277E+01_JPRB, 0.14116E+01_JPRB ,&
&  0.13387E+01_JPRB, 0.12086E+01_JPRB, 0.11957E+01_JPRB, 0.12585E+01_JPRB, 0.13174E+01_JPRB, 0.12896E+01_JPRB, 0.12222E+01_JPRB ,&
&  0.11268E+01_JPRB, 0.10860E+01_JPRB, 0.10809E+01_JPRB, 0.10287E+01_JPRB, 0.10090E+01_JPRB, 0.86286E+00_JPRB, 0.87666E+00_JPRB ,&
&  0.90561E+00_JPRB, 0.97024E+00_JPRB, 0.81594E+00_JPRB, 0.10443E+01_JPRB, 0.79065E+00_JPRB, 0.74885E+00_JPRB, 0.76261E+00_JPRB ,&
&  0.78396E+00_JPRB, 0.81321E+00_JPRB, 0.82018E+00_JPRB, 0.90201E+00_JPRB, 0.88814E+00_JPRB, 0.87807E+00_JPRB, 0.88357E+00_JPRB ,&
&  0.85489E+00_JPRB, 0.84935E+00_JPRB, 0.84956E+00_JPRB, 0.86056E+00_JPRB, 0.86487E+00_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.93337E+00_JPRB, 0.93691E+00_JPRB, 0.94385E+00_JPRB, 0.95536E+00_JPRB, 0.96584E+00_JPRB, 0.97613E+00_JPRB, 0.98030E+00_JPRB ,&
&  0.97502E+00_JPRB, 0.97753E+00_JPRB, 0.98785E+00_JPRB, 0.10039E+01_JPRB, 0.10191E+01_JPRB, 0.10321E+01_JPRB, 0.10478E+01_JPRB ,&
&  0.10474E+01_JPRB, 0.10514E+01_JPRB, 0.10716E+01_JPRB, 0.10946E+01_JPRB, 0.11067E+01_JPRB, 0.11062E+01_JPRB, 0.11094E+01_JPRB ,&
&  0.11044E+01_JPRB, 0.11153E+01_JPRB, 0.11466E+01_JPRB, 0.11697E+01_JPRB, 0.11999E+01_JPRB, 0.12754E+01_JPRB, 0.15035E+01_JPRB ,&
&  0.15385E+01_JPRB, 0.17213E+01_JPRB, 0.15505E+01_JPRB, 0.13945E+01_JPRB, 0.13927E+01_JPRB, 0.13341E+01_JPRB, 0.12932E+01_JPRB ,&
&  0.12493E+01_JPRB, 0.12287E+01_JPRB, 0.11812E+01_JPRB, 0.11889E+01_JPRB, 0.11814E+01_JPRB, 0.12186E+01_JPRB, 0.12169E+01_JPRB ,&
&  0.12755E+01_JPRB, 0.13402E+01_JPRB, 0.14676E+01_JPRB, 0.15834E+01_JPRB, 0.17401E+01_JPRB, 0.18417E+01_JPRB, 0.16746E+01_JPRB ,&
&  0.16410E+01_JPRB, 0.18203E+01_JPRB, 0.20831E+01_JPRB, 0.22659E+01_JPRB, 0.21281E+01_JPRB, 0.18910E+01_JPRB, 0.16885E+01_JPRB ,&
&  0.14851E+01_JPRB, 0.13522E+01_JPRB, 0.15098E+01_JPRB, 0.14750E+01_JPRB, 0.14788E+01_JPRB, 0.14022E+01_JPRB, 0.12497E+01_JPRB ,&
&  0.11089E+01_JPRB, 0.94873E+00_JPRB, 0.93123E+00_JPRB, 0.93161E+00_JPRB, 0.95677E+00_JPRB, 0.93939E+00_JPRB, 0.92353E+00_JPRB ,&
&  0.86739E+00_JPRB, 0.82066E+00_JPRB, 0.88111E+00_JPRB, 0.94907E+00_JPRB, 0.10069E+01_JPRB, 0.12338E+01_JPRB, 0.11895E+01_JPRB ,&
&  0.85466E+00_JPRB, 0.76524E+00_JPRB, 0.55487E+00_JPRB, 0.76489E+00_JPRB, 0.67205E+00_JPRB, 0.70452E+00_JPRB, 0.78239E+00_JPRB ,&
&  0.89160E+00_JPRB, 0.98632E+00_JPRB, 0.10511E+01_JPRB, 0.10129E+01_JPRB, 0.93921E+00_JPRB, 0.92472E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.92178E+00_JPRB, 0.92521E+00_JPRB, 0.92624E+00_JPRB, 0.92645E+00_JPRB, 0.92301E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.96927E+00_JPRB, 0.97980E+00_JPRB, 0.99448E+00_JPRB, 0.10133E+01_JPRB, 0.10317E+01_JPRB, 0.10487E+01_JPRB, 0.10625E+01_JPRB ,&
&  0.10725E+01_JPRB, 0.10881E+01_JPRB, 0.11126E+01_JPRB, 0.11412E+01_JPRB, 0.11540E+01_JPRB, 0.11714E+01_JPRB, 0.11770E+01_JPRB ,&
&  0.12026E+01_JPRB, 0.12325E+01_JPRB, 0.12473E+01_JPRB, 0.12644E+01_JPRB, 0.12768E+01_JPRB, 0.12936E+01_JPRB, 0.13153E+01_JPRB ,&
&  0.13350E+01_JPRB, 0.13793E+01_JPRB, 0.14447E+01_JPRB, 0.14985E+01_JPRB, 0.15736E+01_JPRB, 0.17127E+01_JPRB, 0.15005E+01_JPRB ,&
&  0.16965E+01_JPRB, 0.17407E+01_JPRB, 0.16602E+01_JPRB, 0.14830E+01_JPRB, 0.14238E+01_JPRB, 0.13709E+01_JPRB, 0.13337E+01_JPRB ,&
&  0.13135E+01_JPRB, 0.12710E+01_JPRB, 0.12929E+01_JPRB, 0.12932E+01_JPRB, 0.12972E+01_JPRB, 0.13847E+01_JPRB, 0.14449E+01_JPRB ,&
&  0.14824E+01_JPRB, 0.15234E+01_JPRB, 0.15821E+01_JPRB, 0.16194E+01_JPRB, 0.16834E+01_JPRB, 0.17118E+01_JPRB, 0.16295E+01_JPRB ,&
&  0.15138E+01_JPRB, 0.14580E+01_JPRB, 0.14031E+01_JPRB, 0.14320E+01_JPRB, 0.13959E+01_JPRB, 0.12204E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.11494E+01_JPRB, 0.11115E+01_JPRB, 0.11498E+01_JPRB, 0.15453E+01_JPRB, 0.14586E+01_JPRB, 0.13438E+01_JPRB, 0.11168E+01_JPRB ,&
&  0.97992E+00_JPRB, 0.80742E+00_JPRB, 0.74382E+00_JPRB, 0.75048E+00_JPRB, 0.73882E+00_JPRB, 0.73302E+00_JPRB, 0.70703E+00_JPRB ,&
&  0.66254E+00_JPRB, 0.64634E+00_JPRB, 0.71009E+00_JPRB, 0.82038E+00_JPRB, 0.98201E+00_JPRB, 0.10457E+01_JPRB, 0.13685E+01_JPRB ,&
&  0.92235E+00_JPRB, 0.56670E+00_JPRB, 0.63943E+00_JPRB, 0.66815E+00_JPRB, 0.64136E+00_JPRB, 0.65099E+00_JPRB, 0.77871E+00_JPRB ,&
&  0.98647E+00_JPRB, 0.10485E+01_JPRB, 0.10681E+01_JPRB, 0.10170E+01_JPRB, 0.10090E+01_JPRB, 0.10075E+01_JPRB, 0.99112E+00_JPRB ,&
&  0.98451E+00_JPRB, 0.98167E+00_JPRB, 0.97944E+00_JPRB, 0.97038E+00_JPRB, 0.96526E+00_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.10149E+01_JPRB, 0.10297E+01_JPRB, 0.10520E+01_JPRB, 0.10820E+01_JPRB, 0.10988E+01_JPRB, 0.11247E+01_JPRB, 0.11366E+01_JPRB ,&
&  0.11508E+01_JPRB, 0.11828E+01_JPRB, 0.12126E+01_JPRB, 0.12451E+01_JPRB, 0.12758E+01_JPRB, 0.13051E+01_JPRB, 0.13335E+01_JPRB ,&
&  0.13572E+01_JPRB, 0.13808E+01_JPRB, 0.13921E+01_JPRB, 0.13919E+01_JPRB, 0.13776E+01_JPRB, 0.13711E+01_JPRB, 0.13540E+01_JPRB ,&
&  0.13384E+01_JPRB, 0.13427E+01_JPRB, 0.13569E+01_JPRB, 0.13680E+01_JPRB, 0.13872E+01_JPRB, 0.14799E+01_JPRB, 0.11174E+01_JPRB ,&
&  0.16251E+01_JPRB, 0.17177E+01_JPRB, 0.17205E+01_JPRB, 0.15944E+01_JPRB, 0.15337E+01_JPRB, 0.15536E+01_JPRB, 0.16111E+01_JPRB ,&
&  0.17016E+01_JPRB, 0.18250E+01_JPRB, 0.19439E+01_JPRB, 0.18688E+01_JPRB, 0.17619E+01_JPRB, 0.18080E+01_JPRB, 0.18701E+01_JPRB ,&
&  0.19089E+01_JPRB, 0.19226E+01_JPRB, 0.19389E+01_JPRB, 0.19216E+01_JPRB, 0.19081E+01_JPRB, 0.18852E+01_JPRB, 0.18255E+01_JPRB ,&
&  0.17331E+01_JPRB, 0.16106E+01_JPRB, 0.14507E+01_JPRB, 0.15587E+01_JPRB, 0.15627E+01_JPRB, 0.13081E+01_JPRB, 0.93356E+00_JPRB ,&
&  0.85170E+00_JPRB, 0.99512E+00_JPRB, 0.11934E+01_JPRB, 0.15753E+01_JPRB, 0.13935E+01_JPRB, 0.12320E+01_JPRB, 0.11040E+01_JPRB ,&
&  0.91228E+00_JPRB, 0.78087E+00_JPRB, 0.70187E+00_JPRB, 0.69460E+00_JPRB, 0.69437E+00_JPRB, 0.70554E+00_JPRB, 0.68950E+00_JPRB ,&
&  0.66698E+00_JPRB, 0.65082E+00_JPRB, 0.68699E+00_JPRB, 0.75308E+00_JPRB, 0.86687E+00_JPRB, 0.86670E+00_JPRB, 0.12944E+01_JPRB ,&
&  0.93678E+00_JPRB, 0.76411E+00_JPRB, 0.10060E+01_JPRB, 0.57310E+00_JPRB, 0.70908E+00_JPRB, 0.79295E+00_JPRB, 0.90758E+00_JPRB ,&
&  0.93859E+00_JPRB, 0.12354E+01_JPRB, 0.15326E+01_JPRB, 0.13591E+01_JPRB, 0.12243E+01_JPRB, 0.11114E+01_JPRB, 0.10660E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.10427E+01_JPRB, 0.10384E+01_JPRB, 0.10292E+01_JPRB, 0.10164E+01_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.10932E+01_JPRB, 0.10982E+01_JPRB, 0.10941E+01_JPRB, 0.11278E+01_JPRB, 0.11280E+01_JPRB, 0.11462E+01_JPRB, 0.11497E+01_JPRB ,&
&  0.11748E+01_JPRB, 0.11962E+01_JPRB, 0.12219E+01_JPRB, 0.12627E+01_JPRB, 0.12790E+01_JPRB, 0.12989E+01_JPRB, 0.13323E+01_JPRB ,&
&  0.13647E+01_JPRB, 0.14050E+01_JPRB, 0.14068E+01_JPRB, 0.13959E+01_JPRB, 0.13659E+01_JPRB, 0.13477E+01_JPRB, 0.13248E+01_JPRB ,&
&  0.13059E+01_JPRB, 0.13037E+01_JPRB, 0.13147E+01_JPRB, 0.13353E+01_JPRB, 0.13555E+01_JPRB, 0.14079E+01_JPRB, 0.98472E+00_JPRB ,&
&  0.15750E+01_JPRB, 0.17724E+01_JPRB, 0.17458E+01_JPRB, 0.17302E+01_JPRB, 0.16598E+01_JPRB, 0.17135E+01_JPRB, 0.18112E+01_JPRB ,&
&  0.19520E+01_JPRB, 0.20930E+01_JPRB, 0.21682E+01_JPRB, 0.22057E+01_JPRB, 0.22092E+01_JPRB, 0.22940E+01_JPRB, 0.23750E+01_JPRB ,&
&  0.24309E+01_JPRB, 0.24666E+01_JPRB, 0.25070E+01_JPRB, 0.25275E+01_JPRB, 0.25598E+01_JPRB, 0.26036E+01_JPRB, 0.25979E+01_JPRB ,&
&  0.25449E+01_JPRB, 0.24953E+01_JPRB, 0.25211E+01_JPRB, 0.26692E+01_JPRB, 0.26423E+01_JPRB, 0.22588E+01_JPRB, 0.17120E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.85801E+00_JPRB, 0.11491E+01_JPRB, 0.15182E+01_JPRB, 0.13204E+01_JPRB, 0.11565E+01_JPRB, 0.10195E+01_JPRB ,&
&  0.93067E+00_JPRB, 0.81554E+00_JPRB, 0.72313E+00_JPRB, 0.71614E+00_JPRB, 0.74107E+00_JPRB, 0.79655E+00_JPRB, 0.78352E+00_JPRB ,&
&  0.79859E+00_JPRB, 0.80954E+00_JPRB, 0.80698E+00_JPRB, 0.83462E+00_JPRB, 0.94587E+00_JPRB, 0.10204E+01_JPRB, 0.94672E+00_JPRB ,&
&  0.10792E+01_JPRB, 0.91139E+00_JPRB, 0.89928E+00_JPRB, 0.98572E+00_JPRB, 0.99841E+00_JPRB, 0.11826E+01_JPRB, 0.14736E+01_JPRB ,&
&  0.19218E+01_JPRB, 0.18702E+01_JPRB, 0.24760E+01_JPRB, 0.37725E+01_JPRB, 0.29652E+01_JPRB, 0.21225E+01_JPRB, 0.11569E+01_JPRB ,&
&  0.11440E+01_JPRB, 0.11181E+01_JPRB, 0.11282E+01_JPRB, 0.11122E+01_JPRB, 0.10994E+01_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.10510E+01_JPRB, 0.10374E+01_JPRB, 0.10392E+01_JPRB, 0.10401E+01_JPRB, 0.10383E+01_JPRB, 0.10489E+01_JPRB, 0.10730E+01_JPRB ,&
&  0.10859E+01_JPRB, 0.11239E+01_JPRB, 0.11825E+01_JPRB, 0.12221E+01_JPRB, 0.12743E+01_JPRB, 0.12835E+01_JPRB, 0.12555E+01_JPRB ,&
&  0.12519E+01_JPRB, 0.12891E+01_JPRB, 0.13234E+01_JPRB, 0.13386E+01_JPRB, 0.13049E+01_JPRB, 0.12887E+01_JPRB, 0.13094E+01_JPRB ,&
&  0.13532E+01_JPRB, 0.13913E+01_JPRB, 0.14295E+01_JPRB, 0.14858E+01_JPRB, 0.14977E+01_JPRB, 0.14264E+01_JPRB, 0.10028E+01_JPRB ,&
&  0.14632E+01_JPRB, 0.17395E+01_JPRB, 0.18328E+01_JPRB, 0.18169E+01_JPRB, 0.18316E+01_JPRB, 0.17797E+01_JPRB, 0.18667E+01_JPRB ,&
&  0.20137E+01_JPRB, 0.21695E+01_JPRB, 0.22058E+01_JPRB, 0.21533E+01_JPRB, 0.24830E+01_JPRB, 0.25762E+01_JPRB, 0.27108E+01_JPRB ,&
&  0.28231E+01_JPRB, 0.29244E+01_JPRB, 0.30234E+01_JPRB, 0.31391E+01_JPRB, 0.32393E+01_JPRB, 0.33347E+01_JPRB, 0.34090E+01_JPRB ,&
&  0.34280E+01_JPRB, 0.34284E+01_JPRB, 0.35105E+01_JPRB, 0.33725E+01_JPRB, 0.32891E+01_JPRB, 0.34004E+01_JPRB, 0.28697E+01_JPRB ,&
&  0.16160E+01_JPRB, 0.10319E+01_JPRB, 0.12778E+01_JPRB, 0.14413E+01_JPRB, 0.12324E+01_JPRB, 0.11042E+01_JPRB, 0.10587E+01_JPRB ,&
&  0.94232E+00_JPRB, 0.83192E+00_JPRB, 0.79928E+00_JPRB, 0.84566E+00_JPRB, 0.92825E+00_JPRB, 0.10034E+01_JPRB, 0.10205E+01_JPRB ,&
&  0.98603E+00_JPRB, 0.96708E+00_JPRB, 0.97206E+00_JPRB, 0.10012E+01_JPRB, 0.10754E+01_JPRB, 0.12318E+01_JPRB, 0.14520E+01_JPRB ,&
&  0.13851E+01_JPRB, 0.10177E+01_JPRB, 0.10961E+01_JPRB, 0.11455E+01_JPRB, 0.12445E+01_JPRB, 0.13961E+01_JPRB, 0.16926E+01_JPRB ,&
&  0.21569E+01_JPRB, 0.26482E+01_JPRB, 0.25844E+01_JPRB, 0.21640E+01_JPRB, 0.27230E+01_JPRB, 0.18336E+01_JPRB, 0.99134E+00_JPRB ,&
&  0.10235E+01_JPRB, 0.10432E+01_JPRB, 0.10641E+01_JPRB, 0.10627E+01_JPRB, 0.10598E+01_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.90794E+00_JPRB, 0.89191E+00_JPRB, 0.88147E+00_JPRB, 0.87205E+00_JPRB, 0.88423E+00_JPRB, 0.92023E+00_JPRB, 0.96203E+00_JPRB ,&
&  0.10058E+01_JPRB, 0.10556E+01_JPRB, 0.11209E+01_JPRB, 0.11476E+01_JPRB, 0.11898E+01_JPRB, 0.11920E+01_JPRB, 0.11845E+01_JPRB ,&
&  0.11662E+01_JPRB, 0.11816E+01_JPRB, 0.12101E+01_JPRB, 0.12396E+01_JPRB, 0.12540E+01_JPRB, 0.12878E+01_JPRB, 0.13323E+01_JPRB ,&
&  0.13904E+01_JPRB, 0.14469E+01_JPRB, 0.15194E+01_JPRB, 0.16354E+01_JPRB, 0.16784E+01_JPRB, 0.16905E+01_JPRB, 0.17179E+01_JPRB ,&
&  0.95859E+00_JPRB, 0.15550E+01_JPRB, 0.17182E+01_JPRB, 0.17295E+01_JPRB, 0.18848E+01_JPRB, 0.19353E+01_JPRB, 0.19932E+01_JPRB ,&
&  0.21236E+01_JPRB, 0.21280E+01_JPRB, 0.21648E+01_JPRB, 0.22878E+01_JPRB, 0.25146E+01_JPRB, 0.26231E+01_JPRB, 0.27409E+01_JPRB ,&
&  0.28013E+01_JPRB, 0.28533E+01_JPRB, 0.29713E+01_JPRB, 0.31208E+01_JPRB, 0.32759E+01_JPRB, 0.34313E+01_JPRB, 0.36272E+01_JPRB ,&
&  0.37682E+01_JPRB, 0.38881E+01_JPRB, 0.41073E+01_JPRB, 0.37621E+01_JPRB, 0.35178E+01_JPRB, 0.37527E+01_JPRB, 0.34455E+01_JPRB ,&
&  0.23465E+01_JPRB, 0.13101E+01_JPRB, 0.13048E+01_JPRB, 0.14900E+01_JPRB, 0.12511E+01_JPRB, 0.11682E+01_JPRB, 0.10822E+01_JPRB ,&
&  0.94660E+00_JPRB, 0.87032E+00_JPRB, 0.93562E+00_JPRB, 0.10146E+01_JPRB, 0.11032E+01_JPRB, 0.11778E+01_JPRB, 0.11873E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.10772E+01_JPRB, 0.10249E+01_JPRB, 0.10096E+01_JPRB, 0.10251E+01_JPRB, 0.10457E+01_JPRB, 0.10410E+01_JPRB ,&
&  0.10394E+01_JPRB, 0.10244E+01_JPRB, 0.10617E+01_JPRB, 0.10930E+01_JPRB, 0.11275E+01_JPRB, 0.11593E+01_JPRB, 0.12950E+01_JPRB ,&
&  0.15254E+01_JPRB, 0.17341E+01_JPRB, 0.19830E+01_JPRB, 0.19329E+01_JPRB, 0.17487E+01_JPRB, 0.12635E+01_JPRB, 0.86091E+00_JPRB ,&
&  0.88298E+00_JPRB, 0.90580E+00_JPRB, 0.91123E+00_JPRB, 0.92431E+00_JPRB, 0.91642E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.81655E+00_JPRB, 0.80936E+00_JPRB, 0.80251E+00_JPRB, 0.80784E+00_JPRB, 0.81758E+00_JPRB, 0.86795E+00_JPRB, 0.94933E+00_JPRB ,&
&  0.10293E+01_JPRB, 0.10791E+01_JPRB, 0.11235E+01_JPRB, 0.11478E+01_JPRB, 0.11532E+01_JPRB, 0.11173E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.11141E+01_JPRB, 0.11152E+01_JPRB, 0.11408E+01_JPRB, 0.11801E+01_JPRB, 0.12160E+01_JPRB, 0.12557E+01_JPRB, 0.13010E+01_JPRB ,&
&  0.13576E+01_JPRB, 0.14406E+01_JPRB, 0.15281E+01_JPRB, 0.16582E+01_JPRB, 0.17611E+01_JPRB, 0.18288E+01_JPRB, 0.17982E+01_JPRB ,&
&  0.84352E+00_JPRB, 0.99500E+00_JPRB, 0.15444E+01_JPRB, 0.15508E+01_JPRB, 0.16849E+01_JPRB, 0.18723E+01_JPRB, 0.21264E+01_JPRB ,&
&  0.21031E+01_JPRB, 0.19176E+01_JPRB, 0.19932E+01_JPRB, 0.23492E+01_JPRB, 0.23728E+01_JPRB, 0.24767E+01_JPRB, 0.25667E+01_JPRB ,&
&  0.25956E+01_JPRB, 0.25952E+01_JPRB, 0.26315E+01_JPRB, 0.27507E+01_JPRB, 0.29201E+01_JPRB, 0.31040E+01_JPRB, 0.33231E+01_JPRB ,&
&  0.35492E+01_JPRB, 0.38025E+01_JPRB, 0.40566E+01_JPRB, 0.34530E+01_JPRB, 0.36939E+01_JPRB, 0.40513E+01_JPRB, 0.44305E+01_JPRB ,&
&  0.33748E+01_JPRB, 0.19494E+01_JPRB, 0.15842E+01_JPRB, 0.16966E+01_JPRB, 0.14288E+01_JPRB, 0.13904E+01_JPRB, 0.11810E+01_JPRB ,&
&  0.11347E+01_JPRB, 0.10561E+01_JPRB, 0.10930E+01_JPRB, 0.11670E+01_JPRB, 0.12449E+01_JPRB, 0.13018E+01_JPRB, 0.13007E+01_JPRB ,&
&  0.12265E+01_JPRB, 0.11768E+01_JPRB, 0.11486E+01_JPRB, 0.11427E+01_JPRB, 0.11508E+01_JPRB, 0.11569E+01_JPRB, 0.11451E+01_JPRB ,&
&  0.11300E+01_JPRB, 0.11270E+01_JPRB, 0.11669E+01_JPRB, 0.11926E+01_JPRB, 0.12167E+01_JPRB, 0.12401E+01_JPRB, 0.12604E+01_JPRB ,&
&  0.12098E+01_JPRB, 0.11774E+01_JPRB, 0.12584E+01_JPRB, 0.11579E+01_JPRB, 0.10276E+01_JPRB, 0.88301E+00_JPRB, 0.75103E+00_JPRB ,&
&  0.77246E+00_JPRB, 0.78360E+00_JPRB, 0.78286E+00_JPRB, 0.79276E+00_JPRB, 0.81159E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.72933E+00_JPRB, 0.73579E+00_JPRB, 0.72955E+00_JPRB, 0.74815E+00_JPRB, 0.75077E+00_JPRB, 0.82109E+00_JPRB, 0.89196E+00_JPRB ,&
&  0.10034E+01_JPRB, 0.10866E+01_JPRB, 0.11408E+01_JPRB, 0.11577E+01_JPRB, 0.11408E+01_JPRB, 0.10905E+01_JPRB, 0.10429E+01_JPRB ,&
&  0.10455E+01_JPRB, 0.10507E+01_JPRB, 0.10645E+01_JPRB, 0.10883E+01_JPRB, 0.11138E+01_JPRB, 0.11524E+01_JPRB, 0.11920E+01_JPRB ,&
&  0.12448E+01_JPRB, 0.13248E+01_JPRB, 0.14255E+01_JPRB, 0.15722E+01_JPRB, 0.17624E+01_JPRB, 0.20222E+01_JPRB, 0.22864E+01_JPRB ,&
&  0.17934E+01_JPRB, 0.91245E+00_JPRB, 0.13367E+01_JPRB, 0.13759E+01_JPRB, 0.13086E+01_JPRB, 0.15583E+01_JPRB, 0.18948E+01_JPRB ,&
&  0.18629E+01_JPRB, 0.18135E+01_JPRB, 0.18310E+01_JPRB, 0.21692E+01_JPRB, 0.22152E+01_JPRB, 0.22935E+01_JPRB, 0.23573E+01_JPRB ,&
&  0.24020E+01_JPRB, 0.24068E+01_JPRB, 0.24090E+01_JPRB, 0.25093E+01_JPRB, 0.26741E+01_JPRB, 0.28842E+01_JPRB, 0.31044E+01_JPRB ,&
&  0.33588E+01_JPRB, 0.36396E+01_JPRB, 0.39858E+01_JPRB, 0.34413E+01_JPRB, 0.38563E+01_JPRB, 0.39102E+01_JPRB, 0.34679E+01_JPRB ,&
&  0.32754E+01_JPRB, 0.22183E+01_JPRB, 0.17214E+01_JPRB, 0.18428E+01_JPRB, 0.16319E+01_JPRB, 0.12728E+01_JPRB, 0.12707E+01_JPRB ,&
&  0.12064E+01_JPRB, 0.12093E+01_JPRB, 0.11954E+01_JPRB, 0.12781E+01_JPRB, 0.13257E+01_JPRB, 0.13582E+01_JPRB, 0.13374E+01_JPRB ,&
&  0.13155E+01_JPRB, 0.12663E+01_JPRB, 0.12443E+01_JPRB, 0.12428E+01_JPRB, 0.12427E+01_JPRB, 0.12663E+01_JPRB, 0.12643E+01_JPRB ,&
&  0.12547E+01_JPRB, 0.12679E+01_JPRB, 0.13123E+01_JPRB, 0.13619E+01_JPRB, 0.13800E+01_JPRB, 0.13039E+01_JPRB, 0.12370E+01_JPRB ,&
&  0.11916E+01_JPRB, 0.11457E+01_JPRB, 0.10358E+01_JPRB, 0.91278E+00_JPRB, 0.87110E+00_JPRB, 0.79473E+00_JPRB, 0.73201E+00_JPRB ,&
&  0.74167E+00_JPRB, 0.73964E+00_JPRB, 0.72664E+00_JPRB, 0.72699E+00_JPRB, 0.73123E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.77500E+00_JPRB, 0.77115E+00_JPRB, 0.74122E+00_JPRB, 0.72398E+00_JPRB, 0.72057E+00_JPRB, 0.74980E+00_JPRB, 0.79357E+00_JPRB ,&
&  0.86205E+00_JPRB, 0.95444E+00_JPRB, 0.10309E+01_JPRB, 0.10696E+01_JPRB, 0.10639E+01_JPRB, 0.10712E+01_JPRB, 0.10312E+01_JPRB ,&
&  0.98737E+00_JPRB, 0.98165E+00_JPRB, 0.99440E+00_JPRB, 0.10025E+01_JPRB, 0.10183E+01_JPRB, 0.10407E+01_JPRB, 0.10660E+01_JPRB ,&
&  0.10798E+01_JPRB, 0.10928E+01_JPRB, 0.11098E+01_JPRB, 0.11940E+01_JPRB, 0.13363E+01_JPRB, 0.15579E+01_JPRB, 0.19623E+01_JPRB ,&
&  0.28447E+01_JPRB, 0.28337E+01_JPRB, 0.14777E+01_JPRB, 0.15756E+01_JPRB, 0.13940E+01_JPRB, 0.14971E+01_JPRB, 0.15018E+01_JPRB ,&
&  0.16569E+01_JPRB, 0.16047E+01_JPRB, 0.16370E+01_JPRB, 0.19081E+01_JPRB, 0.19719E+01_JPRB, 0.20522E+01_JPRB, 0.21121E+01_JPRB ,&
&  0.21998E+01_JPRB, 0.22385E+01_JPRB, 0.22113E+01_JPRB, 0.22363E+01_JPRB, 0.23813E+01_JPRB, 0.25815E+01_JPRB, 0.28129E+01_JPRB ,&
&  0.30942E+01_JPRB, 0.33523E+01_JPRB, 0.37062E+01_JPRB, 0.34408E+01_JPRB, 0.38215E+01_JPRB, 0.35398E+01_JPRB, 0.28531E+01_JPRB ,&
&  0.24169E+01_JPRB, 0.21489E+01_JPRB, 0.20206E+01_JPRB, 0.19935E+01_JPRB, 0.18014E+01_JPRB, 0.13942E+01_JPRB, 0.13742E+01_JPRB ,&
&  0.13044E+01_JPRB, 0.12742E+01_JPRB, 0.12626E+01_JPRB, 0.12868E+01_JPRB, 0.13388E+01_JPRB, 0.13564E+01_JPRB, 0.13654E+01_JPRB ,&
&  0.13460E+01_JPRB, 0.13108E+01_JPRB, 0.12926E+01_JPRB, 0.12866E+01_JPRB, 0.12871E+01_JPRB, 0.13121E+01_JPRB, 0.13230E+01_JPRB ,&
&  0.13344E+01_JPRB, 0.13610E+01_JPRB, 0.14047E+01_JPRB, 0.14501E+01_JPRB, 0.14205E+01_JPRB, 0.13308E+01_JPRB, 0.12341E+01_JPRB ,&
&  0.12020E+01_JPRB, 0.12029E+01_JPRB, 0.11183E+01_JPRB, 0.10526E+01_JPRB, 0.98710E+00_JPRB, 0.96479E+00_JPRB, 0.93248E+00_JPRB ,&
&  0.92446E+00_JPRB, 0.92106E+00_JPRB, 0.88257E+00_JPRB, 0.77488E+00_JPRB, 0.77488E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.87762E+00_JPRB, 0.83569E+00_JPRB, 0.82998E+00_JPRB, 0.82528E+00_JPRB, 0.81273E+00_JPRB, 0.81245E+00_JPRB, 0.77393E+00_JPRB ,&
&  0.77087E+00_JPRB, 0.81986E+00_JPRB, 0.88629E+00_JPRB, 0.92444E+00_JPRB, 0.94785E+00_JPRB, 0.95017E+00_JPRB, 0.92947E+00_JPRB ,&
&  0.90579E+00_JPRB, 0.91795E+00_JPRB, 0.93474E+00_JPRB, 0.94451E+00_JPRB, 0.96230E+00_JPRB, 0.97807E+00_JPRB, 0.10019E+01_JPRB ,&
&  0.10206E+01_JPRB, 0.99782E+00_JPRB, 0.96808E+00_JPRB, 0.96013E+00_JPRB, 0.10175E+01_JPRB, 0.11089E+01_JPRB, 0.12379E+01_JPRB ,&
&  0.18342E+01_JPRB, 0.37468E+01_JPRB, 0.19467E+01_JPRB, 0.17787E+01_JPRB, 0.16724E+01_JPRB, 0.15620E+01_JPRB, 0.14773E+01_JPRB ,&
&  0.15377E+01_JPRB, 0.13794E+01_JPRB, 0.13820E+01_JPRB, 0.15870E+01_JPRB, 0.16513E+01_JPRB, 0.17082E+01_JPRB, 0.17870E+01_JPRB ,&
&  0.18560E+01_JPRB, 0.19055E+01_JPRB, 0.19296E+01_JPRB, 0.19003E+01_JPRB, 0.20022E+01_JPRB, 0.21556E+01_JPRB, 0.23687E+01_JPRB ,&
&  0.26113E+01_JPRB, 0.29212E+01_JPRB, 0.33354E+01_JPRB, 0.33059E+01_JPRB, 0.36012E+01_JPRB, 0.33832E+01_JPRB, 0.27699E+01_JPRB ,&
&  0.24163E+01_JPRB, 0.23440E+01_JPRB, 0.21569E+01_JPRB, 0.20521E+01_JPRB, 0.18797E+01_JPRB, 0.15873E+01_JPRB, 0.15746E+01_JPRB ,&
&  0.14105E+01_JPRB, 0.13449E+01_JPRB, 0.13365E+01_JPRB, 0.13303E+01_JPRB, 0.13527E+01_JPRB, 0.13579E+01_JPRB, 0.13632E+01_JPRB ,&
&  0.13484E+01_JPRB, 0.13201E+01_JPRB, 0.13028E+01_JPRB, 0.13057E+01_JPRB, 0.13070E+01_JPRB, 0.13069E+01_JPRB, 0.13240E+01_JPRB ,&
&  0.13397E+01_JPRB, 0.13768E+01_JPRB, 0.14339E+01_JPRB, 0.14364E+01_JPRB, 0.13895E+01_JPRB, 0.12967E+01_JPRB, 0.12025E+01_JPRB ,&
&  0.11691E+01_JPRB, 0.11739E+01_JPRB, 0.11377E+01_JPRB, 0.10876E+01_JPRB, 0.10836E+01_JPRB, 0.10562E+01_JPRB, 0.10841E+01_JPRB ,&
&  0.11628E+01_JPRB, 0.12737E+01_JPRB, 0.13591E+01_JPRB, 0.91020E+00_JPRB, 0.90703E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.93215E+00_JPRB, 0.88715E+00_JPRB, 0.88400E+00_JPRB, 0.90875E+00_JPRB, 0.90804E+00_JPRB, 0.88901E+00_JPRB, 0.84725E+00_JPRB ,&
&  0.79457E+00_JPRB, 0.79505E+00_JPRB, 0.81640E+00_JPRB, 0.83792E+00_JPRB, 0.85311E+00_JPRB, 0.86678E+00_JPRB, 0.85822E+00_JPRB ,&
&  0.83759E+00_JPRB, 0.84569E+00_JPRB, 0.86173E+00_JPRB, 0.88944E+00_JPRB, 0.91044E+00_JPRB, 0.93140E+00_JPRB, 0.96065E+00_JPRB ,&
&  0.96762E+00_JPRB, 0.95973E+00_JPRB, 0.94332E+00_JPRB, 0.93008E+00_JPRB, 0.92893E+00_JPRB, 0.95363E+00_JPRB, 0.10092E+01_JPRB ,&
&  0.12971E+01_JPRB, 0.32722E+01_JPRB, 0.31772E+01_JPRB, 0.22532E+01_JPRB, 0.17571E+01_JPRB, 0.15083E+01_JPRB, 0.14074E+01_JPRB ,&
&  0.15227E+01_JPRB, 0.15469E+01_JPRB, 0.16211E+01_JPRB, 0.14973E+01_JPRB, 0.14470E+01_JPRB, 0.14744E+01_JPRB, 0.15329E+01_JPRB ,&
&  0.15416E+01_JPRB, 0.15555E+01_JPRB, 0.15640E+01_JPRB, 0.15519E+01_JPRB, 0.15739E+01_JPRB, 0.17025E+01_JPRB, 0.18668E+01_JPRB ,&
&  0.20323E+01_JPRB, 0.23216E+01_JPRB, 0.28284E+01_JPRB, 0.33320E+01_JPRB, 0.33669E+01_JPRB, 0.34550E+01_JPRB, 0.32764E+01_JPRB ,&
&  0.34166E+01_JPRB, 0.30211E+01_JPRB, 0.24356E+01_JPRB, 0.20225E+01_JPRB, 0.17592E+01_JPRB, 0.18109E+01_JPRB, 0.17115E+01_JPRB ,&
&  0.14966E+01_JPRB, 0.13856E+01_JPRB, 0.13462E+01_JPRB, 0.13439E+01_JPRB, 0.13515E+01_JPRB, 0.13540E+01_JPRB, 0.13266E+01_JPRB ,&
&  0.13046E+01_JPRB, 0.12942E+01_JPRB, 0.12918E+01_JPRB, 0.13001E+01_JPRB, 0.12989E+01_JPRB, 0.12978E+01_JPRB, 0.13115E+01_JPRB ,&
&  0.13117E+01_JPRB, 0.13317E+01_JPRB, 0.13407E+01_JPRB, 0.13794E+01_JPRB, 0.13370E+01_JPRB, 0.12387E+01_JPRB, 0.11511E+01_JPRB ,&
&  0.11194E+01_JPRB, 0.11384E+01_JPRB, 0.11424E+01_JPRB, 0.11262E+01_JPRB, 0.11076E+01_JPRB, 0.10830E+01_JPRB, 0.10558E+01_JPRB ,&
&  0.10642E+01_JPRB, 0.10743E+01_JPRB, 0.10325E+01_JPRB, 0.96769E+00_JPRB, 0.97522E+00_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.99006E+00_JPRB, 0.95897E+00_JPRB, 0.93548E+00_JPRB, 0.94162E+00_JPRB, 0.94824E+00_JPRB, 0.93569E+00_JPRB, 0.91116E+00_JPRB ,&
&  0.88422E+00_JPRB, 0.85579E+00_JPRB, 0.86068E+00_JPRB, 0.84947E+00_JPRB, 0.84135E+00_JPRB, 0.83510E+00_JPRB, 0.83277E+00_JPRB ,&
&  0.83727E+00_JPRB, 0.84738E+00_JPRB, 0.83701E+00_JPRB, 0.83972E+00_JPRB, 0.86338E+00_JPRB, 0.88832E+00_JPRB, 0.89819E+00_JPRB ,&
&  0.90680E+00_JPRB, 0.91649E+00_JPRB, 0.91440E+00_JPRB, 0.91139E+00_JPRB, 0.89756E+00_JPRB, 0.90233E+00_JPRB, 0.92528E+00_JPRB ,&
&  0.10918E+01_JPRB, 0.25896E+01_JPRB, 0.18807E+01_JPRB, 0.22556E+01_JPRB, 0.17256E+01_JPRB, 0.14874E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.13562E+01_JPRB, 0.16766E+01_JPRB, 0.15290E+01_JPRB, 0.14312E+01_JPRB, 0.13539E+01_JPRB, 0.13375E+01_JPRB, 0.13369E+01_JPRB ,&
&  0.13389E+01_JPRB, 0.13567E+01_JPRB, 0.13462E+01_JPRB, 0.13541E+01_JPRB, 0.13374E+01_JPRB, 0.13979E+01_JPRB, 0.15003E+01_JPRB ,&
&  0.15939E+01_JPRB, 0.17474E+01_JPRB, 0.20953E+01_JPRB, 0.25293E+01_JPRB, 0.33729E+01_JPRB, 0.38300E+01_JPRB, 0.40519E+01_JPRB ,&
&  0.42323E+01_JPRB, 0.36631E+01_JPRB, 0.29175E+01_JPRB, 0.22822E+01_JPRB, 0.19121E+01_JPRB, 0.18418E+01_JPRB, 0.18153E+01_JPRB ,&
&  0.16282E+01_JPRB, 0.14906E+01_JPRB, 0.14095E+01_JPRB, 0.13717E+01_JPRB, 0.13777E+01_JPRB, 0.13774E+01_JPRB, 0.13257E+01_JPRB ,&
&  0.12850E+01_JPRB, 0.12821E+01_JPRB, 0.12915E+01_JPRB, 0.13022E+01_JPRB, 0.12916E+01_JPRB, 0.12667E+01_JPRB, 0.12666E+01_JPRB ,&
&  0.12759E+01_JPRB, 0.12846E+01_JPRB, 0.12848E+01_JPRB, 0.13522E+01_JPRB, 0.13072E+01_JPRB, 0.12289E+01_JPRB, 0.11506E+01_JPRB ,&
&  0.11321E+01_JPRB, 0.11292E+01_JPRB, 0.11217E+01_JPRB, 0.11023E+01_JPRB, 0.11221E+01_JPRB, 0.11399E+01_JPRB, 0.10720E+01_JPRB ,&
&  0.10606E+01_JPRB, 0.10524E+01_JPRB, 0.10385E+01_JPRB, 0.10042E+01_JPRB, 0.10126E+01_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.99876E+00_JPRB, 0.97602E+00_JPRB, 0.97748E+00_JPRB, 0.97771E+00_JPRB, 0.97050E+00_JPRB, 0.94777E+00_JPRB, 0.93316E+00_JPRB ,&
&  0.94046E+00_JPRB, 0.93060E+00_JPRB, 0.91736E+00_JPRB, 0.88832E+00_JPRB, 0.87161E+00_JPRB, 0.85256E+00_JPRB, 0.84209E+00_JPRB ,&
&  0.84619E+00_JPRB, 0.85001E+00_JPRB, 0.84232E+00_JPRB, 0.84139E+00_JPRB, 0.83730E+00_JPRB, 0.85034E+00_JPRB, 0.86354E+00_JPRB ,&
&  0.87560E+00_JPRB, 0.88408E+00_JPRB, 0.89808E+00_JPRB, 0.90297E+00_JPRB, 0.89423E+00_JPRB, 0.88096E+00_JPRB, 0.88797E+00_JPRB ,&
&  0.10207E+01_JPRB, 0.25512E+01_JPRB, 0.15807E+01_JPRB, 0.22375E+01_JPRB, 0.17031E+01_JPRB, 0.14933E+01_JPRB, 0.12477E+01_JPRB ,&
&  0.13444E+01_JPRB, 0.16115E+01_JPRB, 0.15026E+01_JPRB, 0.13521E+01_JPRB, 0.13056E+01_JPRB, 0.12738E+01_JPRB, 0.12354E+01_JPRB ,&
&  0.12261E+01_JPRB, 0.12503E+01_JPRB, 0.12507E+01_JPRB, 0.12622E+01_JPRB, 0.12654E+01_JPRB, 0.12497E+01_JPRB, 0.12809E+01_JPRB ,&
&  0.13168E+01_JPRB, 0.13788E+01_JPRB, 0.15928E+01_JPRB, 0.23212E+01_JPRB, 0.29829E+01_JPRB, 0.40538E+01_JPRB, 0.52061E+01_JPRB ,&
&  0.50895E+01_JPRB, 0.43687E+01_JPRB, 0.31434E+01_JPRB, 0.25666E+01_JPRB, 0.21656E+01_JPRB, 0.19854E+01_JPRB, 0.18840E+01_JPRB ,&
&  0.17754E+01_JPRB, 0.16831E+01_JPRB, 0.15442E+01_JPRB, 0.14741E+01_JPRB, 0.14557E+01_JPRB, 0.14371E+01_JPRB, 0.13979E+01_JPRB ,&
&  0.13529E+01_JPRB, 0.13457E+01_JPRB, 0.13515E+01_JPRB, 0.13460E+01_JPRB, 0.13286E+01_JPRB, 0.12893E+01_JPRB, 0.12694E+01_JPRB ,&
&  0.12797E+01_JPRB, 0.12782E+01_JPRB, 0.12909E+01_JPRB, 0.13885E+01_JPRB, 0.13453E+01_JPRB, 0.12482E+01_JPRB, 0.11786E+01_JPRB ,&
&  0.11722E+01_JPRB, 0.11413E+01_JPRB, 0.11284E+01_JPRB, 0.11289E+01_JPRB, 0.11428E+01_JPRB, 0.12081E+01_JPRB, 0.11314E+01_JPRB ,&
&  0.10843E+01_JPRB, 0.10452E+01_JPRB, 0.10220E+01_JPRB, 0.10080E+01_JPRB, 0.10079E+01_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.10322E+01_JPRB, 0.10167E+01_JPRB, 0.10149E+01_JPRB, 0.10271E+01_JPRB, 0.10103E+01_JPRB, 0.98359E+00_JPRB, 0.95823E+00_JPRB ,&
&  0.97052E+00_JPRB, 0.96693E+00_JPRB, 0.95619E+00_JPRB, 0.91919E+00_JPRB, 0.89900E+00_JPRB, 0.88228E+00_JPRB, 0.87625E+00_JPRB ,&
&  0.87992E+00_JPRB, 0.87123E+00_JPRB, 0.85512E+00_JPRB, 0.84656E+00_JPRB, 0.84570E+00_JPRB, 0.84581E+00_JPRB, 0.86133E+00_JPRB ,&
&  0.85434E+00_JPRB, 0.86036E+00_JPRB, 0.88199E+00_JPRB, 0.89013E+00_JPRB, 0.89564E+00_JPRB, 0.87634E+00_JPRB, 0.87333E+00_JPRB ,&
&  0.94123E+00_JPRB, 0.10823E+01_JPRB, 0.17378E+01_JPRB, 0.20489E+01_JPRB, 0.17108E+01_JPRB, 0.15314E+01_JPRB, 0.14253E+01_JPRB ,&
&  0.14689E+01_JPRB, 0.15080E+01_JPRB, 0.14341E+01_JPRB, 0.13380E+01_JPRB, 0.12854E+01_JPRB, 0.12279E+01_JPRB, 0.11842E+01_JPRB ,&
&  0.11462E+01_JPRB, 0.11307E+01_JPRB, 0.11438E+01_JPRB, 0.11763E+01_JPRB, 0.11956E+01_JPRB, 0.11896E+01_JPRB, 0.11602E+01_JPRB ,&
&  0.11479E+01_JPRB, 0.11516E+01_JPRB, 0.12427E+01_JPRB, 0.17336E+01_JPRB, 0.28567E+01_JPRB, 0.41884E+01_JPRB, 0.51445E+01_JPRB ,&
&  0.52859E+01_JPRB, 0.43755E+01_JPRB, 0.31947E+01_JPRB, 0.27101E+01_JPRB, 0.23700E+01_JPRB, 0.21143E+01_JPRB, 0.19014E+01_JPRB ,&
&  0.17801E+01_JPRB, 0.17446E+01_JPRB, 0.16834E+01_JPRB, 0.15930E+01_JPRB, 0.15074E+01_JPRB, 0.14584E+01_JPRB, 0.14210E+01_JPRB ,&
&  0.13940E+01_JPRB, 0.14025E+01_JPRB, 0.14169E+01_JPRB, 0.13852E+01_JPRB, 0.13471E+01_JPRB, 0.13056E+01_JPRB, 0.12852E+01_JPRB ,&
&  0.12944E+01_JPRB, 0.13087E+01_JPRB, 0.13314E+01_JPRB, 0.13922E+01_JPRB, 0.13624E+01_JPRB, 0.13084E+01_JPRB, 0.12443E+01_JPRB ,&
&  0.11874E+01_JPRB, 0.11422E+01_JPRB, 0.11204E+01_JPRB, 0.11555E+01_JPRB, 0.11515E+01_JPRB, 0.12350E+01_JPRB, 0.11984E+01_JPRB ,&
&  0.11302E+01_JPRB, 0.10826E+01_JPRB, 0.10566E+01_JPRB, 0.10447E+01_JPRB, 0.10455E+01_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.10728E+01_JPRB, 0.10661E+01_JPRB, 0.10749E+01_JPRB, 0.10844E+01_JPRB, 0.10725E+01_JPRB, 0.10413E+01_JPRB, 0.10105E+01_JPRB ,&
&  0.99128E+00_JPRB, 0.97301E+00_JPRB, 0.95098E+00_JPRB, 0.91854E+00_JPRB, 0.90209E+00_JPRB, 0.90215E+00_JPRB, 0.90689E+00_JPRB ,&
&  0.91227E+00_JPRB, 0.90137E+00_JPRB, 0.88317E+00_JPRB, 0.86371E+00_JPRB, 0.86045E+00_JPRB, 0.86288E+00_JPRB, 0.87428E+00_JPRB ,&
&  0.86919E+00_JPRB, 0.86745E+00_JPRB, 0.87282E+00_JPRB, 0.87928E+00_JPRB, 0.88657E+00_JPRB, 0.87598E+00_JPRB, 0.86450E+00_JPRB ,&
&  0.89077E+00_JPRB, 0.12553E+01_JPRB, 0.15640E+01_JPRB, 0.16960E+01_JPRB, 0.16478E+01_JPRB, 0.15244E+01_JPRB, 0.14700E+01_JPRB ,&
&  0.13930E+01_JPRB, 0.13940E+01_JPRB, 0.13876E+01_JPRB, 0.13378E+01_JPRB, 0.12757E+01_JPRB, 0.12226E+01_JPRB, 0.11825E+01_JPRB ,&
&  0.11417E+01_JPRB, 0.11094E+01_JPRB, 0.10884E+01_JPRB, 0.10965E+01_JPRB, 0.11161E+01_JPRB, 0.11372E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.10888E+01_JPRB, 0.10602E+01_JPRB, 0.10688E+01_JPRB, 0.13156E+01_JPRB, 0.22170E+01_JPRB, 0.27893E+01_JPRB, 0.41145E+01_JPRB ,&
&  0.45754E+01_JPRB, 0.34316E+01_JPRB, 0.27398E+01_JPRB, 0.24182E+01_JPRB, 0.21893E+01_JPRB, 0.19592E+01_JPRB, 0.17860E+01_JPRB ,&
&  0.17049E+01_JPRB, 0.16614E+01_JPRB, 0.16572E+01_JPRB, 0.16101E+01_JPRB, 0.15042E+01_JPRB, 0.14014E+01_JPRB, 0.13504E+01_JPRB ,&
&  0.13238E+01_JPRB, 0.13037E+01_JPRB, 0.13029E+01_JPRB, 0.13077E+01_JPRB, 0.12960E+01_JPRB, 0.12547E+01_JPRB, 0.12342E+01_JPRB ,&
&  0.12366E+01_JPRB, 0.12660E+01_JPRB, 0.13147E+01_JPRB, 0.13350E+01_JPRB, 0.13486E+01_JPRB, 0.13567E+01_JPRB, 0.12908E+01_JPRB ,&
&  0.11962E+01_JPRB, 0.11376E+01_JPRB, 0.11241E+01_JPRB, 0.11360E+01_JPRB, 0.11697E+01_JPRB, 0.12676E+01_JPRB, 0.12283E+01_JPRB ,&
&  0.11671E+01_JPRB, 0.11145E+01_JPRB, 0.10884E+01_JPRB, 0.10711E+01_JPRB, 0.10867E+01_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.11015E+01_JPRB, 0.10911E+01_JPRB, 0.10991E+01_JPRB, 0.11040E+01_JPRB, 0.10930E+01_JPRB, 0.10608E+01_JPRB, 0.10337E+01_JPRB ,&
&  0.10188E+01_JPRB, 0.98078E+00_JPRB, 0.94269E+00_JPRB, 0.91491E+00_JPRB, 0.91394E+00_JPRB, 0.91980E+00_JPRB, 0.92270E+00_JPRB ,&
&  0.91468E+00_JPRB, 0.91271E+00_JPRB, 0.89805E+00_JPRB, 0.88489E+00_JPRB, 0.87869E+00_JPRB, 0.87375E+00_JPRB, 0.86174E+00_JPRB ,&
&  0.86346E+00_JPRB, 0.86311E+00_JPRB, 0.86748E+00_JPRB, 0.87569E+00_JPRB, 0.87326E+00_JPRB, 0.87626E+00_JPRB, 0.86995E+00_JPRB ,&
&  0.86024E+00_JPRB, 0.98693E+00_JPRB, 0.11427E+01_JPRB, 0.13845E+01_JPRB, 0.14698E+01_JPRB, 0.14454E+01_JPRB, 0.13939E+01_JPRB ,&
&  0.13320E+01_JPRB, 0.13027E+01_JPRB, 0.13000E+01_JPRB, 0.12710E+01_JPRB, 0.12255E+01_JPRB, 0.11878E+01_JPRB, 0.11536E+01_JPRB ,&
&  0.11264E+01_JPRB, 0.11036E+01_JPRB, 0.10972E+01_JPRB, 0.10864E+01_JPRB, 0.10770E+01_JPRB, 0.10862E+01_JPRB, 0.10995E+01_JPRB ,&
&  0.10610E+01_JPRB, 0.10275E+01_JPRB, 0.10023E+01_JPRB, 0.10490E+01_JPRB, 0.14555E+01_JPRB, 0.19740E+01_JPRB, 0.26808E+01_JPRB ,&
&  0.25353E+01_JPRB, 0.22347E+01_JPRB, 0.20424E+01_JPRB, 0.18984E+01_JPRB, 0.17996E+01_JPRB, 0.16620E+01_JPRB, 0.15430E+01_JPRB ,&
&  0.14350E+01_JPRB, 0.13881E+01_JPRB, 0.13512E+01_JPRB, 0.13330E+01_JPRB, 0.13040E+01_JPRB, 0.12826E+01_JPRB, 0.12286E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.11726E+01_JPRB, 0.11677E+01_JPRB, 0.11814E+01_JPRB, 0.12063E+01_JPRB, 0.12090E+01_JPRB, 0.12078E+01_JPRB ,&
&  0.12214E+01_JPRB, 0.12344E+01_JPRB, 0.12765E+01_JPRB, 0.13279E+01_JPRB, 0.13117E+01_JPRB, 0.12742E+01_JPRB, 0.12452E+01_JPRB ,&
&  0.12199E+01_JPRB, 0.11745E+01_JPRB, 0.11347E+01_JPRB, 0.11483E+01_JPRB, 0.11598E+01_JPRB, 0.12465E+01_JPRB, 0.12314E+01_JPRB ,&
&  0.11854E+01_JPRB, 0.11255E+01_JPRB, 0.10986E+01_JPRB, 0.10822E+01_JPRB, 0.11015E+01_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.11298E+01_JPRB, 0.11341E+01_JPRB, 0.11285E+01_JPRB, 0.11229E+01_JPRB, 0.11090E+01_JPRB, 0.10787E+01_JPRB, 0.10461E+01_JPRB ,&
&  0.10223E+01_JPRB, 0.99430E+00_JPRB, 0.95354E+00_JPRB, 0.93032E+00_JPRB, 0.92292E+00_JPRB, 0.92212E+00_JPRB, 0.91790E+00_JPRB ,&
&  0.91339E+00_JPRB, 0.91725E+00_JPRB, 0.91040E+00_JPRB, 0.90985E+00_JPRB, 0.90183E+00_JPRB, 0.89659E+00_JPRB, 0.87607E+00_JPRB ,&
&  0.86228E+00_JPRB, 0.85285E+00_JPRB, 0.85073E+00_JPRB, 0.85820E+00_JPRB, 0.86514E+00_JPRB, 0.87007E+00_JPRB, 0.87627E+00_JPRB ,&
&  0.87624E+00_JPRB, 0.81713E+00_JPRB, 0.91323E+00_JPRB, 0.10714E+01_JPRB, 0.11831E+01_JPRB, 0.12672E+01_JPRB, 0.12511E+01_JPRB ,&
&  0.12275E+01_JPRB, 0.12029E+01_JPRB, 0.12015E+01_JPRB, 0.11776E+01_JPRB, 0.11371E+01_JPRB, 0.10998E+01_JPRB, 0.10756E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.10633E+01_JPRB, 0.10580E+01_JPRB, 0.10517E+01_JPRB, 0.10469E+01_JPRB, 0.10468E+01_JPRB, 0.10590E+01_JPRB ,&
&  0.10537E+01_JPRB, 0.10216E+01_JPRB, 0.98567E+00_JPRB, 0.96320E+00_JPRB, 0.10240E+01_JPRB, 0.12048E+01_JPRB, 0.15245E+01_JPRB ,&
&  0.16972E+01_JPRB, 0.16067E+01_JPRB, 0.15331E+01_JPRB, 0.14685E+01_JPRB, 0.14158E+01_JPRB, 0.13825E+01_JPRB, 0.13459E+01_JPRB ,&
&  0.13184E+01_JPRB, 0.12966E+01_JPRB, 0.12835E+01_JPRB, 0.12670E+01_JPRB, 0.12487E+01_JPRB, 0.12301E+01_JPRB, 0.12048E+01_JPRB ,&
&  0.11827E+01_JPRB, 0.11717E+01_JPRB, 0.11590E+01_JPRB, 0.11662E+01_JPRB, 0.11738E+01_JPRB, 0.11787E+01_JPRB, 0.11841E+01_JPRB ,&
&  0.11999E+01_JPRB, 0.12340E+01_JPRB, 0.12580E+01_JPRB, 0.12810E+01_JPRB, 0.12705E+01_JPRB, 0.12421E+01_JPRB, 0.12251E+01_JPRB ,&
&  0.12016E+01_JPRB, 0.11801E+01_JPRB, 0.11203E+01_JPRB, 0.11199E+01_JPRB, 0.12185E+01_JPRB, 0.12081E+01_JPRB, 0.11938E+01_JPRB ,&
&  0.11837E+01_JPRB, 0.11545E+01_JPRB, 0.11078E+01_JPRB, 0.10608E+01_JPRB, 0.10938E+01_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.10223E+01_JPRB, 0.10575E+01_JPRB, 0.10595E+01_JPRB, 0.10597E+01_JPRB, 0.10576E+01_JPRB, 0.10428E+01_JPRB, 0.10222E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.98771E+00_JPRB, 0.96969E+00_JPRB, 0.94569E+00_JPRB, 0.92981E+00_JPRB, 0.91980E+00_JPRB, 0.91047E+00_JPRB ,&
&  0.90587E+00_JPRB, 0.90114E+00_JPRB, 0.89727E+00_JPRB, 0.89258E+00_JPRB, 0.88262E+00_JPRB, 0.87424E+00_JPRB, 0.86032E+00_JPRB ,&
&  0.85088E+00_JPRB, 0.84967E+00_JPRB, 0.85023E+00_JPRB, 0.85480E+00_JPRB, 0.86316E+00_JPRB, 0.87270E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.88288E+00_JPRB, 0.78055E+00_JPRB, 0.81322E+00_JPRB, 0.90209E+00_JPRB, 0.10076E+01_JPRB, 0.11034E+01_JPRB, 0.11363E+01_JPRB ,&
&  0.11409E+01_JPRB, 0.11353E+01_JPRB, 0.11187E+01_JPRB, 0.10969E+01_JPRB, 0.10735E+01_JPRB, 0.10575E+01_JPRB, 0.10264E+01_JPRB ,&
&  0.10072E+01_JPRB, 0.10006E+01_JPRB, 0.10046E+01_JPRB, 0.10046E+01_JPRB, 0.99989E+00_JPRB, 0.99480E+00_JPRB, 0.99809E+00_JPRB ,&
&  0.99818E+00_JPRB, 0.98058E+00_JPRB, 0.95728E+00_JPRB, 0.94082E+00_JPRB, 0.94968E+00_JPRB, 0.10262E+01_JPRB, 0.11598E+01_JPRB ,&
&  0.12861E+01_JPRB, 0.12795E+01_JPRB, 0.12436E+01_JPRB, 0.12049E+01_JPRB, 0.11869E+01_JPRB, 0.11827E+01_JPRB, 0.11853E+01_JPRB ,&
&  0.11790E+01_JPRB, 0.11727E+01_JPRB, 0.11847E+01_JPRB, 0.11888E+01_JPRB, 0.11934E+01_JPRB, 0.11917E+01_JPRB, 0.11883E+01_JPRB ,&
&  0.11780E+01_JPRB, 0.11641E+01_JPRB, 0.11574E+01_JPRB, 0.11562E+01_JPRB, 0.11636E+01_JPRB, 0.11653E+01_JPRB, 0.11583E+01_JPRB ,&
&  0.11486E+01_JPRB, 0.11604E+01_JPRB, 0.11992E+01_JPRB, 0.12319E+01_JPRB, 0.12227E+01_JPRB, 0.12031E+01_JPRB, 0.11853E+01_JPRB ,&
&  0.11789E+01_JPRB, 0.11627E+01_JPRB, 0.11504E+01_JPRB, 0.11792E+01_JPRB, 0.11746E+01_JPRB, 0.11598E+01_JPRB, 0.11455E+01_JPRB ,&
&  0.11351E+01_JPRB, 0.11133E+01_JPRB, 0.10911E+01_JPRB, 0.10535E+01_JPRB, 0.96396E+00_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.93641E+00_JPRB, 0.97223E+00_JPRB, 0.99274E+00_JPRB, 0.99884E+00_JPRB, 0.10072E+01_JPRB, 0.10115E+01_JPRB, 0.10092E+01_JPRB ,&
&  0.10071E+01_JPRB, 0.10017E+01_JPRB, 0.98861E+00_JPRB, 0.96793E+00_JPRB, 0.94673E+00_JPRB, 0.92328E+00_JPRB, 0.90471E+00_JPRB ,&
&  0.89847E+00_JPRB, 0.89434E+00_JPRB, 0.88059E+00_JPRB, 0.86778E+00_JPRB, 0.85568E+00_JPRB, 0.84602E+00_JPRB, 0.84194E+00_JPRB ,&
&  0.83691E+00_JPRB, 0.83482E+00_JPRB, 0.84031E+00_JPRB, 0.84780E+00_JPRB, 0.85785E+00_JPRB, 0.86907E+00_JPRB, 0.88385E+00_JPRB ,&
&  0.88816E+00_JPRB, 0.70477E+00_JPRB, 0.73317E+00_JPRB, 0.81758E+00_JPRB, 0.88057E+00_JPRB, 0.95828E+00_JPRB, 0.10297E+01_JPRB ,&
&  0.10559E+01_JPRB, 0.10798E+01_JPRB, 0.10714E+01_JPRB, 0.10552E+01_JPRB, 0.10506E+01_JPRB, 0.10309E+01_JPRB, 0.99064E+00_JPRB ,&
&  0.96001E+00_JPRB, 0.95675E+00_JPRB, 0.96795E+00_JPRB, 0.97504E+00_JPRB, 0.97234E+00_JPRB, 0.96500E+00_JPRB, 0.96208E+00_JPRB ,&
&  0.96048E+00_JPRB, 0.95934E+00_JPRB, 0.95690E+00_JPRB, 0.95124E+00_JPRB, 0.96406E+00_JPRB, 0.10020E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.10914E+01_JPRB, 0.10988E+01_JPRB, 0.10909E+01_JPRB, 0.10728E+01_JPRB, 0.10650E+01_JPRB, 0.10761E+01_JPRB, 0.10851E+01_JPRB ,&
&  0.10864E+01_JPRB, 0.10865E+01_JPRB, 0.10952E+01_JPRB, 0.11017E+01_JPRB, 0.11138E+01_JPRB, 0.11261E+01_JPRB, 0.11464E+01_JPRB ,&
&  0.11614E+01_JPRB, 0.11616E+01_JPRB, 0.11584E+01_JPRB, 0.11464E+01_JPRB, 0.11254E+01_JPRB, 0.11109E+01_JPRB, 0.11052E+01_JPRB ,&
&  0.11014E+01_JPRB, 0.11088E+01_JPRB, 0.11299E+01_JPRB, 0.11528E+01_JPRB, 0.11564E+01_JPRB, 0.11542E+01_JPRB, 0.11525E+01_JPRB ,&
&  0.11551E+01_JPRB, 0.11541E+01_JPRB, 0.11571E+01_JPRB, 0.10933E+01_JPRB, 0.11299E+01_JPRB, 0.11224E+01_JPRB, 0.10966E+01_JPRB ,&
&  0.10669E+01_JPRB, 0.10323E+01_JPRB, 0.10098E+01_JPRB, 0.84499E+00_JPRB, 0.90913E+00_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.88545E+00_JPRB, 0.90639E+00_JPRB, 0.91246E+00_JPRB, 0.92716E+00_JPRB, 0.93763E+00_JPRB, 0.94523E+00_JPRB, 0.95578E+00_JPRB ,&
&  0.96267E+00_JPRB, 0.95936E+00_JPRB, 0.95282E+00_JPRB, 0.93898E+00_JPRB, 0.91577E+00_JPRB, 0.89090E+00_JPRB, 0.87052E+00_JPRB ,&
&  0.85580E+00_JPRB, 0.85317E+00_JPRB, 0.84498E+00_JPRB, 0.83435E+00_JPRB, 0.82779E+00_JPRB, 0.82423E+00_JPRB, 0.82614E+00_JPRB ,&
&  0.83383E+00_JPRB, 0.83785E+00_JPRB, 0.83287E+00_JPRB, 0.83173E+00_JPRB, 0.84071E+00_JPRB, 0.85688E+00_JPRB, 0.86449E+00_JPRB ,&
&  0.86654E+00_JPRB, 0.67075E+00_JPRB, 0.71404E+00_JPRB, 0.76516E+00_JPRB, 0.80932E+00_JPRB, 0.87324E+00_JPRB, 0.94231E+00_JPRB ,&
&  0.98692E+00_JPRB, 0.10139E+01_JPRB, 0.10149E+01_JPRB, 0.10046E+01_JPRB, 0.98546E+00_JPRB, 0.96770E+00_JPRB, 0.94681E+00_JPRB ,&
&  0.92874E+00_JPRB, 0.92307E+00_JPRB, 0.92734E+00_JPRB, 0.93214E+00_JPRB, 0.93424E+00_JPRB, 0.93844E+00_JPRB, 0.93319E+00_JPRB ,&
&  0.93190E+00_JPRB, 0.92996E+00_JPRB, 0.92990E+00_JPRB, 0.93202E+00_JPRB, 0.93450E+00_JPRB, 0.94202E+00_JPRB, 0.95620E+00_JPRB ,&
&  0.96086E+00_JPRB, 0.96762E+00_JPRB, 0.96936E+00_JPRB, 0.96453E+00_JPRB, 0.96703E+00_JPRB, 0.97719E+00_JPRB, 0.99316E+00_JPRB ,&
&  0.10134E+01_JPRB, 0.10179E+01_JPRB, 0.10253E+01_JPRB, 0.10348E+01_JPRB, 0.10493E+01_JPRB, 0.10658E+01_JPRB, 0.10856E+01_JPRB ,&
&  0.10972E+01_JPRB, 0.10969E+01_JPRB, 0.10963E+01_JPRB, 0.10873E+01_JPRB, 0.10847E+01_JPRB, 0.10660E+01_JPRB, 0.10444E+01_JPRB ,&
&  0.10404E+01_JPRB, 0.10412E+01_JPRB, 0.10448E+01_JPRB, 0.10575E+01_JPRB, 0.10638E+01_JPRB, 0.10608E+01_JPRB, 0.10651E+01_JPRB ,&
&  0.10727E+01_JPRB, 0.10761E+01_JPRB, 0.10677E+01_JPRB, 0.10614E+01_JPRB, 0.10438E+01_JPRB, 0.10285E+01_JPRB, 0.10026E+01_JPRB ,&
&  0.97164E+00_JPRB, 0.94771E+00_JPRB, 0.74974E+00_JPRB, 0.83598E+00_JPRB, 0.85797E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.87591E+00_JPRB, 0.88172E+00_JPRB, 0.88511E+00_JPRB, 0.89008E+00_JPRB, 0.89202E+00_JPRB, 0.89199E+00_JPRB, 0.89782E+00_JPRB ,&
&  0.90413E+00_JPRB, 0.90061E+00_JPRB, 0.89317E+00_JPRB, 0.89064E+00_JPRB, 0.88141E+00_JPRB, 0.86777E+00_JPRB, 0.85665E+00_JPRB ,&
&  0.84261E+00_JPRB, 0.83706E+00_JPRB, 0.83363E+00_JPRB, 0.82993E+00_JPRB, 0.82316E+00_JPRB, 0.81483E+00_JPRB, 0.81846E+00_JPRB ,&
&  0.82538E+00_JPRB, 0.83276E+00_JPRB, 0.83486E+00_JPRB, 0.83898E+00_JPRB, 0.84514E+00_JPRB, 0.85203E+00_JPRB, 0.86425E+00_JPRB ,&
&  0.75282E+00_JPRB, 0.62695E+00_JPRB, 0.67982E+00_JPRB, 0.73371E+00_JPRB, 0.78072E+00_JPRB, 0.85212E+00_JPRB, 0.90455E+00_JPRB ,&
&  0.94456E+00_JPRB, 0.97712E+00_JPRB, 0.98421E+00_JPRB, 0.96184E+00_JPRB, 0.93872E+00_JPRB, 0.93114E+00_JPRB, 0.93030E+00_JPRB ,&
&  0.92677E+00_JPRB, 0.92561E+00_JPRB, 0.92984E+00_JPRB, 0.93451E+00_JPRB, 0.93936E+00_JPRB, 0.94582E+00_JPRB, 0.94821E+00_JPRB ,&
&  0.94627E+00_JPRB, 0.93387E+00_JPRB, 0.92111E+00_JPRB, 0.90156E+00_JPRB, 0.88511E+00_JPRB, 0.88216E+00_JPRB, 0.88074E+00_JPRB ,&
&  0.87372E+00_JPRB, 0.87639E+00_JPRB, 0.88482E+00_JPRB, 0.89315E+00_JPRB, 0.90576E+00_JPRB, 0.92026E+00_JPRB, 0.94057E+00_JPRB ,&
&  0.96431E+00_JPRB, 0.98381E+00_JPRB, 0.99868E+00_JPRB, 0.10146E+01_JPRB, 0.10219E+01_JPRB, 0.10307E+01_JPRB, 0.10468E+01_JPRB ,&
&  0.10597E+01_JPRB, 0.10480E+01_JPRB, 0.10367E+01_JPRB, 0.10259E+01_JPRB, 0.10230E+01_JPRB, 0.10216E+01_JPRB, 0.10125E+01_JPRB ,&
&  0.10033E+01_JPRB, 0.99716E+00_JPRB, 0.99457E+00_JPRB, 0.99932E+00_JPRB, 0.99689E+00_JPRB, 0.98923E+00_JPRB, 0.98385E+00_JPRB ,&
&  0.97870E+00_JPRB, 0.96622E+00_JPRB, 0.95248E+00_JPRB, 0.94381E+00_JPRB, 0.93082E+00_JPRB, 0.92078E+00_JPRB, 0.91046E+00_JPRB ,&
&  0.90169E+00_JPRB, 0.88396E+00_JPRB, 0.88557E+00_JPRB, 0.87709E+00_JPRB, 0.87246E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.85028E+00_JPRB, 0.85343E+00_JPRB, 0.85753E+00_JPRB, 0.85836E+00_JPRB, 0.85529E+00_JPRB, 0.85398E+00_JPRB, 0.85696E+00_JPRB ,&
&  0.85967E+00_JPRB, 0.85311E+00_JPRB, 0.84621E+00_JPRB, 0.84096E+00_JPRB, 0.83900E+00_JPRB, 0.83054E+00_JPRB, 0.81737E+00_JPRB ,&
&  0.80801E+00_JPRB, 0.80701E+00_JPRB, 0.81233E+00_JPRB, 0.81528E+00_JPRB, 0.80676E+00_JPRB, 0.79777E+00_JPRB, 0.79490E+00_JPRB ,&
&  0.79623E+00_JPRB, 0.80254E+00_JPRB, 0.81297E+00_JPRB, 0.81910E+00_JPRB, 0.82798E+00_JPRB, 0.84228E+00_JPRB, 0.84350E+00_JPRB ,&
&  0.84155E+00_JPRB, 0.70398E+00_JPRB, 0.74260E+00_JPRB, 0.75820E+00_JPRB, 0.78888E+00_JPRB, 0.83341E+00_JPRB, 0.87043E+00_JPRB ,&
&  0.90577E+00_JPRB, 0.92580E+00_JPRB, 0.93462E+00_JPRB, 0.92806E+00_JPRB, 0.91268E+00_JPRB, 0.89847E+00_JPRB, 0.89491E+00_JPRB ,&
&  0.89437E+00_JPRB, 0.90185E+00_JPRB, 0.91604E+00_JPRB, 0.92596E+00_JPRB, 0.92965E+00_JPRB, 0.93090E+00_JPRB, 0.93357E+00_JPRB ,&
&  0.93052E+00_JPRB, 0.91436E+00_JPRB, 0.89117E+00_JPRB, 0.86270E+00_JPRB, 0.84010E+00_JPRB, 0.82964E+00_JPRB, 0.81942E+00_JPRB ,&
&  0.80525E+00_JPRB, 0.80024E+00_JPRB, 0.80508E+00_JPRB, 0.81335E+00_JPRB, 0.82681E+00_JPRB, 0.84172E+00_JPRB, 0.85429E+00_JPRB ,&
&  0.86756E+00_JPRB, 0.88692E+00_JPRB, 0.90766E+00_JPRB, 0.92692E+00_JPRB, 0.94407E+00_JPRB, 0.95790E+00_JPRB, 0.96665E+00_JPRB ,&
&  0.97085E+00_JPRB, 0.96575E+00_JPRB, 0.95210E+00_JPRB, 0.94210E+00_JPRB, 0.93859E+00_JPRB, 0.94364E+00_JPRB, 0.94515E+00_JPRB ,&
&  0.93715E+00_JPRB, 0.93015E+00_JPRB, 0.92609E+00_JPRB, 0.91436E+00_JPRB, 0.90106E+00_JPRB, 0.88833E+00_JPRB, 0.87522E+00_JPRB ,&
&  0.86517E+00_JPRB, 0.86065E+00_JPRB, 0.85652E+00_JPRB, 0.84951E+00_JPRB, 0.83844E+00_JPRB, 0.83212E+00_JPRB, 0.82689E+00_JPRB ,&
&  0.82197E+00_JPRB, 0.81638E+00_JPRB, 0.81689E+00_JPRB, 0.82639E+00_JPRB, 0.84057E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.81094E+00_JPRB, 0.81638E+00_JPRB, 0.82527E+00_JPRB, 0.83576E+00_JPRB, 0.83957E+00_JPRB, 0.83970E+00_JPRB, 0.83821E+00_JPRB ,&
&  0.83982E+00_JPRB, 0.83249E+00_JPRB, 0.82718E+00_JPRB, 0.82406E+00_JPRB, 0.82355E+00_JPRB, 0.82021E+00_JPRB, 0.81086E+00_JPRB ,&
&  0.80297E+00_JPRB, 0.79979E+00_JPRB, 0.80412E+00_JPRB, 0.80545E+00_JPRB, 0.79012E+00_JPRB, 0.78517E+00_JPRB, 0.77738E+00_JPRB ,&
&  0.77626E+00_JPRB, 0.77634E+00_JPRB, 0.77920E+00_JPRB, 0.78528E+00_JPRB, 0.80167E+00_JPRB, 0.81331E+00_JPRB, 0.81956E+00_JPRB ,&
&  0.81771E+00_JPRB, 0.73901E+00_JPRB, 0.76688E+00_JPRB, 0.77921E+00_JPRB, 0.80190E+00_JPRB, 0.82474E+00_JPRB, 0.84504E+00_JPRB ,&
&  0.87144E+00_JPRB, 0.88597E+00_JPRB, 0.89203E+00_JPRB, 0.89321E+00_JPRB, 0.89484E+00_JPRB, 0.89472E+00_JPRB, 0.88791E+00_JPRB ,&
&  0.88368E+00_JPRB, 0.89125E+00_JPRB, 0.90839E+00_JPRB, 0.91748E+00_JPRB, 0.92209E+00_JPRB, 0.92459E+00_JPRB, 0.92144E+00_JPRB ,&
&  0.91435E+00_JPRB, 0.89966E+00_JPRB, 0.87494E+00_JPRB, 0.84513E+00_JPRB, 0.81143E+00_JPRB, 0.78783E+00_JPRB, 0.77496E+00_JPRB ,&
&  0.76797E+00_JPRB, 0.77028E+00_JPRB, 0.77481E+00_JPRB, 0.77983E+00_JPRB, 0.78632E+00_JPRB, 0.79808E+00_JPRB, 0.80512E+00_JPRB ,&
&  0.81508E+00_JPRB, 0.82410E+00_JPRB, 0.83683E+00_JPRB, 0.85656E+00_JPRB, 0.87577E+00_JPRB, 0.88597E+00_JPRB, 0.88577E+00_JPRB ,&
&  0.89077E+00_JPRB, 0.89331E+00_JPRB, 0.88919E+00_JPRB, 0.88827E+00_JPRB, 0.88364E+00_JPRB, 0.88445E+00_JPRB, 0.88177E+00_JPRB ,&
&  0.87280E+00_JPRB, 0.86587E+00_JPRB, 0.85745E+00_JPRB, 0.84150E+00_JPRB, 0.82725E+00_JPRB, 0.81666E+00_JPRB, 0.80783E+00_JPRB ,&
&  0.80622E+00_JPRB, 0.80966E+00_JPRB, 0.81475E+00_JPRB, 0.80935E+00_JPRB, 0.79452E+00_JPRB, 0.78476E+00_JPRB, 0.77361E+00_JPRB ,&
&  0.76601E+00_JPRB, 0.76781E+00_JPRB, 0.76767E+00_JPRB, 0.77500E+00_JPRB, 0.79404E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.76630E+00_JPRB, 0.77872E+00_JPRB, 0.79434E+00_JPRB, 0.81051E+00_JPRB, 0.82093E+00_JPRB, 0.82753E+00_JPRB, 0.82591E+00_JPRB ,&
&  0.81571E+00_JPRB, 0.80556E+00_JPRB, 0.80775E+00_JPRB, 0.80805E+00_JPRB, 0.80725E+00_JPRB, 0.80548E+00_JPRB, 0.79874E+00_JPRB ,&
&  0.78897E+00_JPRB, 0.78056E+00_JPRB, 0.77695E+00_JPRB, 0.77389E+00_JPRB, 0.76395E+00_JPRB, 0.75628E+00_JPRB, 0.74799E+00_JPRB ,&
&  0.74775E+00_JPRB, 0.75026E+00_JPRB, 0.75302E+00_JPRB, 0.75312E+00_JPRB, 0.76100E+00_JPRB, 0.77116E+00_JPRB, 0.77539E+00_JPRB ,&
&  0.76967E+00_JPRB, 0.77638E+00_JPRB, 0.69366E+00_JPRB, 0.76352E+00_JPRB, 0.78869E+00_JPRB, 0.80380E+00_JPRB, 0.81944E+00_JPRB ,&
&  0.83002E+00_JPRB, 0.84164E+00_JPRB, 0.84997E+00_JPRB, 0.85471E+00_JPRB, 0.87206E+00_JPRB, 0.88626E+00_JPRB, 0.88988E+00_JPRB ,&
&  0.89146E+00_JPRB, 0.89057E+00_JPRB, 0.89201E+00_JPRB, 0.89251E+00_JPRB, 0.89742E+00_JPRB, 0.89780E+00_JPRB, 0.89207E+00_JPRB ,&
&  0.88407E+00_JPRB, 0.87218E+00_JPRB, 0.85441E+00_JPRB, 0.83363E+00_JPRB, 0.80104E+00_JPRB, 0.77254E+00_JPRB, 0.75433E+00_JPRB ,&
&  0.73985E+00_JPRB, 0.73625E+00_JPRB, 0.73209E+00_JPRB, 0.72810E+00_JPRB, 0.72780E+00_JPRB, 0.73172E+00_JPRB, 0.73264E+00_JPRB ,&
&  0.74054E+00_JPRB, 0.74983E+00_JPRB, 0.75419E+00_JPRB, 0.76175E+00_JPRB, 0.77179E+00_JPRB, 0.78624E+00_JPRB, 0.79994E+00_JPRB ,&
&  0.81269E+00_JPRB, 0.82155E+00_JPRB, 0.82302E+00_JPRB, 0.82297E+00_JPRB, 0.82010E+00_JPRB, 0.81839E+00_JPRB, 0.81093E+00_JPRB ,&
&  0.80502E+00_JPRB, 0.80177E+00_JPRB, 0.79507E+00_JPRB, 0.78268E+00_JPRB, 0.77480E+00_JPRB, 0.76813E+00_JPRB, 0.76626E+00_JPRB ,&
&  0.76688E+00_JPRB, 0.76811E+00_JPRB, 0.76565E+00_JPRB, 0.76634E+00_JPRB, 0.75985E+00_JPRB, 0.73904E+00_JPRB, 0.72260E+00_JPRB ,&
&  0.72131E+00_JPRB, 0.72629E+00_JPRB, 0.73090E+00_JPRB, 0.73965E+00_JPRB, 0.75430E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.78404E+00_JPRB, 0.79005E+00_JPRB, 0.79912E+00_JPRB, 0.81251E+00_JPRB, 0.82666E+00_JPRB, 0.83598E+00_JPRB, 0.83586E+00_JPRB ,&
&  0.82520E+00_JPRB, 0.81230E+00_JPRB, 0.80829E+00_JPRB, 0.80801E+00_JPRB, 0.80532E+00_JPRB, 0.80406E+00_JPRB, 0.79954E+00_JPRB ,&
&  0.78831E+00_JPRB, 0.77120E+00_JPRB, 0.76306E+00_JPRB, 0.74911E+00_JPRB, 0.73595E+00_JPRB, 0.72517E+00_JPRB, 0.72514E+00_JPRB ,&
&  0.73377E+00_JPRB, 0.74378E+00_JPRB, 0.75612E+00_JPRB, 0.77011E+00_JPRB, 0.77751E+00_JPRB, 0.77594E+00_JPRB, 0.77258E+00_JPRB ,&
&  0.76984E+00_JPRB, 0.76696E+00_JPRB, 0.77765E+00_JPRB, 0.78656E+00_JPRB, 0.79258E+00_JPRB, 0.80386E+00_JPRB, 0.81323E+00_JPRB ,&
&  0.82122E+00_JPRB, 0.83268E+00_JPRB, 0.85138E+00_JPRB, 0.86516E+00_JPRB, 0.87522E+00_JPRB, 0.88595E+00_JPRB, 0.88957E+00_JPRB ,&
&  0.89577E+00_JPRB, 0.90705E+00_JPRB, 0.90932E+00_JPRB, 0.90034E+00_JPRB, 0.88608E+00_JPRB, 0.87705E+00_JPRB, 0.87206E+00_JPRB ,&
&  0.86289E+00_JPRB, 0.84350E+00_JPRB, 0.82873E+00_JPRB, 0.81680E+00_JPRB, 0.79751E+00_JPRB, 0.77692E+00_JPRB, 0.75287E+00_JPRB ,&
&  0.72567E+00_JPRB, 0.70544E+00_JPRB, 0.68986E+00_JPRB, 0.67585E+00_JPRB, 0.67265E+00_JPRB, 0.66444E+00_JPRB, 0.65861E+00_JPRB ,&
&  0.66398E+00_JPRB, 0.68313E+00_JPRB, 0.69765E+00_JPRB, 0.70316E+00_JPRB, 0.70621E+00_JPRB, 0.72249E+00_JPRB, 0.74214E+00_JPRB ,&
&  0.75752E+00_JPRB, 0.76411E+00_JPRB, 0.76034E+00_JPRB, 0.76274E+00_JPRB, 0.76778E+00_JPRB, 0.77187E+00_JPRB, 0.76544E+00_JPRB ,&
&  0.75638E+00_JPRB, 0.75044E+00_JPRB, 0.74891E+00_JPRB, 0.74496E+00_JPRB, 0.74625E+00_JPRB, 0.74601E+00_JPRB, 0.74320E+00_JPRB ,&
&  0.73754E+00_JPRB, 0.72842E+00_JPRB, 0.71655E+00_JPRB, 0.70900E+00_JPRB, 0.69796E+00_JPRB, 0.68156E+00_JPRB, 0.68194E+00_JPRB ,&
&  0.69306E+00_JPRB, 0.71120E+00_JPRB, 0.72860E+00_JPRB, 0.74916E+00_JPRB, 0.77308E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.81012E+00_JPRB, 0.81793E+00_JPRB, 0.82336E+00_JPRB, 0.82731E+00_JPRB, 0.83599E+00_JPRB, 0.84702E+00_JPRB, 0.84699E+00_JPRB ,&
&  0.83244E+00_JPRB, 0.81885E+00_JPRB, 0.80947E+00_JPRB, 0.80704E+00_JPRB, 0.80891E+00_JPRB, 0.81132E+00_JPRB, 0.80553E+00_JPRB ,&
&  0.78420E+00_JPRB, 0.76141E+00_JPRB, 0.74742E+00_JPRB, 0.73357E+00_JPRB, 0.72495E+00_JPRB, 0.71945E+00_JPRB, 0.72624E+00_JPRB ,&
&  0.73632E+00_JPRB, 0.75219E+00_JPRB, 0.76508E+00_JPRB, 0.77438E+00_JPRB, 0.77808E+00_JPRB, 0.76659E+00_JPRB, 0.76783E+00_JPRB ,&
&  0.77485E+00_JPRB, 0.77702E+00_JPRB, 0.79201E+00_JPRB, 0.79141E+00_JPRB, 0.77814E+00_JPRB, 0.76738E+00_JPRB, 0.75682E+00_JPRB ,&
&  0.75416E+00_JPRB, 0.76548E+00_JPRB, 0.79078E+00_JPRB, 0.82741E+00_JPRB, 0.84869E+00_JPRB, 0.85684E+00_JPRB, 0.85376E+00_JPRB ,&
&  0.85643E+00_JPRB, 0.86835E+00_JPRB, 0.87506E+00_JPRB, 0.86679E+00_JPRB, 0.84849E+00_JPRB, 0.83359E+00_JPRB, 0.82048E+00_JPRB ,&
&  0.81080E+00_JPRB, 0.79909E+00_JPRB, 0.79111E+00_JPRB, 0.78107E+00_JPRB, 0.75911E+00_JPRB, 0.73309E+00_JPRB, 0.71035E+00_JPRB ,&
&  0.69338E+00_JPRB, 0.67664E+00_JPRB, 0.65631E+00_JPRB, 0.64986E+00_JPRB, 0.65388E+00_JPRB, 0.63109E+00_JPRB, 0.60675E+00_JPRB ,&
&  0.61095E+00_JPRB, 0.64299E+00_JPRB, 0.66820E+00_JPRB, 0.67879E+00_JPRB, 0.68020E+00_JPRB, 0.68484E+00_JPRB, 0.68851E+00_JPRB ,&
&  0.69525E+00_JPRB, 0.70409E+00_JPRB, 0.71083E+00_JPRB, 0.72208E+00_JPRB, 0.73366E+00_JPRB, 0.73190E+00_JPRB, 0.71723E+00_JPRB ,&
&  0.70119E+00_JPRB, 0.69194E+00_JPRB, 0.68800E+00_JPRB, 0.69008E+00_JPRB, 0.69338E+00_JPRB, 0.69143E+00_JPRB, 0.68562E+00_JPRB ,&
&  0.67377E+00_JPRB, 0.65531E+00_JPRB, 0.63963E+00_JPRB, 0.63061E+00_JPRB, 0.62985E+00_JPRB, 0.63620E+00_JPRB, 0.65236E+00_JPRB ,&
&  0.68367E+00_JPRB, 0.73138E+00_JPRB, 0.76696E+00_JPRB, 0.78721E+00_JPRB, 0.79284E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.83815E+00_JPRB, 0.84618E+00_JPRB, 0.85577E+00_JPRB, 0.86885E+00_JPRB, 0.88092E+00_JPRB, 0.87773E+00_JPRB, 0.86514E+00_JPRB ,&
&  0.84143E+00_JPRB, 0.83245E+00_JPRB, 0.83712E+00_JPRB, 0.83958E+00_JPRB, 0.84136E+00_JPRB, 0.83920E+00_JPRB, 0.82303E+00_JPRB ,&
&  0.78523E+00_JPRB, 0.75505E+00_JPRB, 0.74061E+00_JPRB, 0.73308E+00_JPRB, 0.73067E+00_JPRB, 0.73757E+00_JPRB, 0.74576E+00_JPRB ,&
&  0.76229E+00_JPRB, 0.77203E+00_JPRB, 0.78396E+00_JPRB, 0.79342E+00_JPRB, 0.79620E+00_JPRB, 0.80015E+00_JPRB, 0.80118E+00_JPRB ,&
&  0.78696E+00_JPRB, 0.78712E+00_JPRB, 0.78844E+00_JPRB, 0.77868E+00_JPRB, 0.75709E+00_JPRB, 0.73539E+00_JPRB, 0.70881E+00_JPRB ,&
&  0.69115E+00_JPRB, 0.68222E+00_JPRB, 0.70774E+00_JPRB, 0.76965E+00_JPRB, 0.81327E+00_JPRB, 0.82139E+00_JPRB, 0.81335E+00_JPRB ,&
&  0.81517E+00_JPRB, 0.82252E+00_JPRB, 0.82606E+00_JPRB, 0.81461E+00_JPRB, 0.79492E+00_JPRB, 0.77760E+00_JPRB, 0.76108E+00_JPRB ,&
&  0.75554E+00_JPRB, 0.75480E+00_JPRB, 0.73747E+00_JPRB, 0.70529E+00_JPRB, 0.67762E+00_JPRB, 0.65805E+00_JPRB, 0.64773E+00_JPRB ,&
&  0.64139E+00_JPRB, 0.63162E+00_JPRB, 0.62544E+00_JPRB, 0.63013E+00_JPRB, 0.61022E+00_JPRB, 0.59254E+00_JPRB, 0.59675E+00_JPRB ,&
&  0.61364E+00_JPRB, 0.61970E+00_JPRB, 0.62939E+00_JPRB, 0.63296E+00_JPRB, 0.63041E+00_JPRB, 0.62709E+00_JPRB, 0.62853E+00_JPRB ,&
&  0.63745E+00_JPRB, 0.64766E+00_JPRB, 0.65516E+00_JPRB, 0.66005E+00_JPRB, 0.66492E+00_JPRB, 0.65269E+00_JPRB, 0.63113E+00_JPRB ,&
&  0.61329E+00_JPRB, 0.60975E+00_JPRB, 0.61398E+00_JPRB, 0.61312E+00_JPRB, 0.61144E+00_JPRB, 0.61163E+00_JPRB, 0.61042E+00_JPRB ,&
&  0.60774E+00_JPRB, 0.60054E+00_JPRB, 0.59186E+00_JPRB, 0.59188E+00_JPRB, 0.60297E+00_JPRB, 0.62563E+00_JPRB, 0.66093E+00_JPRB ,&
&  0.71308E+00_JPRB, 0.75240E+00_JPRB, 0.79068E+00_JPRB, 0.80660E+00_JPRB, 0.81052E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.87320E+00_JPRB, 0.89503E+00_JPRB, 0.91373E+00_JPRB, 0.92637E+00_JPRB, 0.92812E+00_JPRB, 0.92117E+00_JPRB, 0.90509E+00_JPRB ,&
&  0.87485E+00_JPRB, 0.85763E+00_JPRB, 0.86653E+00_JPRB, 0.88127E+00_JPRB, 0.89022E+00_JPRB, 0.89020E+00_JPRB, 0.85848E+00_JPRB ,&
&  0.80367E+00_JPRB, 0.78292E+00_JPRB, 0.77991E+00_JPRB, 0.78945E+00_JPRB, 0.80751E+00_JPRB, 0.81417E+00_JPRB, 0.80167E+00_JPRB ,&
&  0.78904E+00_JPRB, 0.79041E+00_JPRB, 0.79311E+00_JPRB, 0.80006E+00_JPRB, 0.78955E+00_JPRB, 0.76856E+00_JPRB, 0.75937E+00_JPRB ,&
&  0.75651E+00_JPRB, 0.74859E+00_JPRB, 0.74036E+00_JPRB, 0.69154E+00_JPRB, 0.71475E+00_JPRB, 0.70315E+00_JPRB, 0.68468E+00_JPRB ,&
&  0.66751E+00_JPRB, 0.66990E+00_JPRB, 0.71183E+00_JPRB, 0.75859E+00_JPRB, 0.77017E+00_JPRB, 0.77279E+00_JPRB, 0.76762E+00_JPRB ,&
&  0.76403E+00_JPRB, 0.76444E+00_JPRB, 0.76583E+00_JPRB, 0.75765E+00_JPRB, 0.73726E+00_JPRB, 0.71092E+00_JPRB, 0.68210E+00_JPRB ,&
&  0.66471E+00_JPRB, 0.63983E+00_JPRB, 0.59639E+00_JPRB, 0.57613E+00_JPRB, 0.57431E+00_JPRB, 0.58236E+00_JPRB, 0.58619E+00_JPRB ,&
&  0.58319E+00_JPRB, 0.57580E+00_JPRB, 0.57951E+00_JPRB, 0.57645E+00_JPRB, 0.55473E+00_JPRB, 0.54148E+00_JPRB, 0.53557E+00_JPRB ,&
&  0.52453E+00_JPRB, 0.52402E+00_JPRB, 0.52931E+00_JPRB, 0.54137E+00_JPRB, 0.55203E+00_JPRB, 0.56657E+00_JPRB, 0.57277E+00_JPRB ,&
&  0.57512E+00_JPRB, 0.57139E+00_JPRB, 0.56516E+00_JPRB, 0.55279E+00_JPRB, 0.53397E+00_JPRB, 0.52670E+00_JPRB, 0.52810E+00_JPRB ,&
&  0.53698E+00_JPRB, 0.54292E+00_JPRB, 0.54239E+00_JPRB, 0.54124E+00_JPRB, 0.52963E+00_JPRB, 0.51694E+00_JPRB, 0.50738E+00_JPRB ,&
&  0.50302E+00_JPRB, 0.50618E+00_JPRB, 0.52195E+00_JPRB, 0.54713E+00_JPRB, 0.57936E+00_JPRB, 0.61850E+00_JPRB, 0.66342E+00_JPRB ,&
&  0.69500E+00_JPRB, 0.71338E+00_JPRB, 0.73773E+00_JPRB, 0.78563E+00_JPRB, 0.83206E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.87551E+00_JPRB, 0.90238E+00_JPRB, 0.93166E+00_JPRB, 0.94847E+00_JPRB, 0.94921E+00_JPRB, 0.94591E+00_JPRB, 0.93070E+00_JPRB ,&
&  0.89968E+00_JPRB, 0.87433E+00_JPRB, 0.87014E+00_JPRB, 0.87408E+00_JPRB, 0.88587E+00_JPRB, 0.90455E+00_JPRB, 0.90585E+00_JPRB ,&
&  0.87885E+00_JPRB, 0.85929E+00_JPRB, 0.84951E+00_JPRB, 0.84226E+00_JPRB, 0.82501E+00_JPRB, 0.80014E+00_JPRB, 0.78389E+00_JPRB ,&
&  0.77922E+00_JPRB, 0.77906E+00_JPRB, 0.77013E+00_JPRB, 0.75626E+00_JPRB, 0.72472E+00_JPRB, 0.68577E+00_JPRB, 0.65505E+00_JPRB ,&
&  0.64415E+00_JPRB, 0.63446E+00_JPRB, 0.59410E+00_JPRB, 0.60977E+00_JPRB, 0.61376E+00_JPRB, 0.61418E+00_JPRB, 0.61157E+00_JPRB ,&
&  0.62347E+00_JPRB, 0.65682E+00_JPRB, 0.69003E+00_JPRB, 0.69981E+00_JPRB, 0.70920E+00_JPRB, 0.71091E+00_JPRB, 0.71236E+00_JPRB ,&
&  0.71688E+00_JPRB, 0.70887E+00_JPRB, 0.67922E+00_JPRB, 0.63523E+00_JPRB, 0.59185E+00_JPRB, 0.55808E+00_JPRB, 0.52934E+00_JPRB ,&
&  0.50963E+00_JPRB, 0.49725E+00_JPRB, 0.48754E+00_JPRB, 0.48423E+00_JPRB, 0.49205E+00_JPRB, 0.50728E+00_JPRB, 0.50230E+00_JPRB ,&
&  0.48406E+00_JPRB, 0.46714E+00_JPRB, 0.44487E+00_JPRB, 0.41461E+00_JPRB, 0.38351E+00_JPRB, 0.34537E+00_JPRB, 0.32558E+00_JPRB ,&
&  0.33035E+00_JPRB, 0.36997E+00_JPRB, 0.42201E+00_JPRB, 0.44215E+00_JPRB, 0.46967E+00_JPRB, 0.49670E+00_JPRB, 0.48623E+00_JPRB ,&
&  0.43801E+00_JPRB, 0.37024E+00_JPRB, 0.34722E+00_JPRB, 0.33417E+00_JPRB, 0.32715E+00_JPRB, 0.32227E+00_JPRB, 0.32104E+00_JPRB ,&
&  0.32527E+00_JPRB, 0.33640E+00_JPRB, 0.34604E+00_JPRB, 0.34307E+00_JPRB, 0.33281E+00_JPRB, 0.32689E+00_JPRB, 0.32785E+00_JPRB ,&
&  0.33802E+00_JPRB, 0.36673E+00_JPRB, 0.42994E+00_JPRB, 0.54454E+00_JPRB, 0.58177E+00_JPRB, 0.61550E+00_JPRB, 0.65423E+00_JPRB ,&
&  0.69136E+00_JPRB, 0.72499E+00_JPRB, 0.75354E+00_JPRB, 0.78860E+00_JPRB, 0.83577E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.81392E+00_JPRB, 0.83249E+00_JPRB, 0.85400E+00_JPRB, 0.86535E+00_JPRB, 0.86651E+00_JPRB, 0.86357E+00_JPRB, 0.85488E+00_JPRB ,&
&  0.83996E+00_JPRB, 0.82203E+00_JPRB, 0.80879E+00_JPRB, 0.79973E+00_JPRB, 0.79556E+00_JPRB, 0.79826E+00_JPRB, 0.80328E+00_JPRB ,&
&  0.80211E+00_JPRB, 0.79779E+00_JPRB, 0.78539E+00_JPRB, 0.76927E+00_JPRB, 0.75225E+00_JPRB, 0.73912E+00_JPRB, 0.72914E+00_JPRB ,&
&  0.71942E+00_JPRB, 0.70478E+00_JPRB, 0.68193E+00_JPRB, 0.64656E+00_JPRB, 0.60469E+00_JPRB, 0.56641E+00_JPRB, 0.53641E+00_JPRB ,&
&  0.51001E+00_JPRB, 0.44358E+00_JPRB, 0.40690E+00_JPRB, 0.41790E+00_JPRB, 0.50836E+00_JPRB, 0.54048E+00_JPRB, 0.56517E+00_JPRB ,&
&  0.60180E+00_JPRB, 0.64076E+00_JPRB, 0.65951E+00_JPRB, 0.65995E+00_JPRB, 0.65438E+00_JPRB, 0.64170E+00_JPRB, 0.62197E+00_JPRB ,&
&  0.59690E+00_JPRB, 0.56628E+00_JPRB, 0.54013E+00_JPRB, 0.51081E+00_JPRB, 0.48080E+00_JPRB, 0.45874E+00_JPRB, 0.44341E+00_JPRB ,&
&  0.43282E+00_JPRB, 0.42305E+00_JPRB, 0.41522E+00_JPRB, 0.40545E+00_JPRB, 0.39636E+00_JPRB, 0.37536E+00_JPRB, 0.35311E+00_JPRB ,&
&  0.33047E+00_JPRB, 0.31266E+00_JPRB, 0.29410E+00_JPRB, 0.27581E+00_JPRB, 0.26083E+00_JPRB, 0.25101E+00_JPRB, 0.24371E+00_JPRB ,&
&  0.24164E+00_JPRB, 0.24818E+00_JPRB, 0.26759E+00_JPRB, 0.30056E+00_JPRB, 0.33731E+00_JPRB, 0.31050E+00_JPRB, 0.29050E+00_JPRB ,&
&  0.26910E+00_JPRB, 0.25676E+00_JPRB, 0.25197E+00_JPRB, 0.25125E+00_JPRB, 0.25238E+00_JPRB, 0.25453E+00_JPRB, 0.25686E+00_JPRB ,&
&  0.26021E+00_JPRB, 0.26338E+00_JPRB, 0.26665E+00_JPRB, 0.26903E+00_JPRB, 0.27049E+00_JPRB, 0.27316E+00_JPRB, 0.27786E+00_JPRB ,&
&  0.28544E+00_JPRB, 0.29789E+00_JPRB, 0.31796E+00_JPRB, 0.34410E+00_JPRB, 0.37165E+00_JPRB, 0.39649E+00_JPRB, 0.44964E+00_JPRB ,&
&  0.53712E+00_JPRB, 0.64866E+00_JPRB, 0.70669E+00_JPRB, 0.76345E+00_JPRB, 0.79187E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.74997E+00_JPRB, 0.76253E+00_JPRB, 0.77292E+00_JPRB, 0.77518E+00_JPRB, 0.77452E+00_JPRB, 0.76948E+00_JPRB, 0.75735E+00_JPRB ,&
&  0.73759E+00_JPRB, 0.71125E+00_JPRB, 0.68559E+00_JPRB, 0.66515E+00_JPRB, 0.65232E+00_JPRB, 0.65519E+00_JPRB, 0.66359E+00_JPRB ,&
&  0.68077E+00_JPRB, 0.70606E+00_JPRB, 0.71106E+00_JPRB, 0.70416E+00_JPRB, 0.66830E+00_JPRB, 0.63096E+00_JPRB, 0.59662E+00_JPRB ,&
&  0.56598E+00_JPRB, 0.53679E+00_JPRB, 0.52588E+00_JPRB, 0.50686E+00_JPRB, 0.49347E+00_JPRB, 0.47657E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.41334E+00_JPRB, 0.38305E+00_JPRB, 0.36683E+00_JPRB, 0.37851E+00_JPRB, 0.46560E+00_JPRB, 0.52662E+00_JPRB, 0.53738E+00_JPRB ,&
&  0.57162E+00_JPRB, 0.59596E+00_JPRB, 0.59098E+00_JPRB, 0.57098E+00_JPRB, 0.54933E+00_JPRB, 0.52587E+00_JPRB, 0.50630E+00_JPRB ,&
&  0.48683E+00_JPRB, 0.46041E+00_JPRB, 0.41062E+00_JPRB, 0.34010E+00_JPRB, 0.30481E+00_JPRB, 0.29146E+00_JPRB, 0.27709E+00_JPRB ,&
&  0.26627E+00_JPRB, 0.25781E+00_JPRB, 0.25440E+00_JPRB, 0.25075E+00_JPRB, 0.25013E+00_JPRB, 0.24630E+00_JPRB, 0.24286E+00_JPRB ,&
&  0.23816E+00_JPRB, 0.23469E+00_JPRB, 0.22982E+00_JPRB, 0.22506E+00_JPRB, 0.22140E+00_JPRB, 0.22084E+00_JPRB, 0.22068E+00_JPRB ,&
&  0.22183E+00_JPRB, 0.22602E+00_JPRB, 0.23809E+00_JPRB, 0.24468E+00_JPRB, 0.24520E+00_JPRB, 0.24093E+00_JPRB, 0.23368E+00_JPRB ,&
&  0.22678E+00_JPRB, 0.22510E+00_JPRB, 0.22363E+00_JPRB, 0.22419E+00_JPRB, 0.22704E+00_JPRB, 0.23077E+00_JPRB, 0.23370E+00_JPRB ,&
&  0.23601E+00_JPRB, 0.23847E+00_JPRB, 0.24070E+00_JPRB, 0.24315E+00_JPRB, 0.24619E+00_JPRB, 0.24980E+00_JPRB, 0.25478E+00_JPRB ,&
&  0.26006E+00_JPRB, 0.26752E+00_JPRB, 0.27797E+00_JPRB, 0.29317E+00_JPRB, 0.30696E+00_JPRB, 0.31594E+00_JPRB, 0.32829E+00_JPRB ,&
&  0.34456E+00_JPRB, 0.39227E+00_JPRB, 0.52713E+00_JPRB, 0.75139E+00_JPRB, 0.78380E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.67222E+00_JPRB, 0.68418E+00_JPRB, 0.69854E+00_JPRB, 0.70186E+00_JPRB, 0.71203E+00_JPRB, 0.71084E+00_JPRB, 0.69343E+00_JPRB ,&
&  0.66521E+00_JPRB, 0.63848E+00_JPRB, 0.62388E+00_JPRB, 0.60545E+00_JPRB, 0.57749E+00_JPRB, 0.51751E+00_JPRB, 0.49461E+00_JPRB ,&
&  0.47596E+00_JPRB, 0.47546E+00_JPRB, 0.46605E+00_JPRB, 0.46604E+00_JPRB, 0.45488E+00_JPRB, 0.44346E+00_JPRB, 0.42475E+00_JPRB ,&
&  0.40750E+00_JPRB, 0.39061E+00_JPRB, 0.37882E+00_JPRB, 0.37330E+00_JPRB, 0.37096E+00_JPRB, 0.36605E+00_JPRB, 0.35340E+00_JPRB ,&
&  0.34455E+00_JPRB, 0.34278E+00_JPRB, 0.34527E+00_JPRB, 0.35387E+00_JPRB, 0.39652E+00_JPRB, 0.45306E+00_JPRB, 0.45394E+00_JPRB ,&
&  0.47624E+00_JPRB, 0.48948E+00_JPRB, 0.50244E+00_JPRB, 0.49135E+00_JPRB, 0.45851E+00_JPRB, 0.41910E+00_JPRB, 0.36560E+00_JPRB ,&
&  0.31468E+00_JPRB, 0.28566E+00_JPRB, 0.26210E+00_JPRB, 0.24545E+00_JPRB, 0.23396E+00_JPRB, 0.22934E+00_JPRB, 0.22339E+00_JPRB ,&
&  0.21954E+00_JPRB, 0.21657E+00_JPRB, 0.21537E+00_JPRB, 0.21481E+00_JPRB, 0.21408E+00_JPRB, 0.21201E+00_JPRB, 0.21145E+00_JPRB ,&
&  0.21207E+00_JPRB, 0.21224E+00_JPRB, 0.21202E+00_JPRB, 0.21291E+00_JPRB, 0.21394E+00_JPRB, 0.21544E+00_JPRB, 0.21686E+00_JPRB ,&
&  0.21901E+00_JPRB, 0.22020E+00_JPRB, 0.22200E+00_JPRB, 0.22398E+00_JPRB, 0.22297E+00_JPRB, 0.22184E+00_JPRB, 0.21905E+00_JPRB ,&
&  0.21589E+00_JPRB, 0.21568E+00_JPRB, 0.21516E+00_JPRB, 0.21578E+00_JPRB, 0.21747E+00_JPRB, 0.21971E+00_JPRB, 0.22264E+00_JPRB ,&
&  0.22542E+00_JPRB, 0.22830E+00_JPRB, 0.23049E+00_JPRB, 0.23273E+00_JPRB, 0.23648E+00_JPRB, 0.24046E+00_JPRB, 0.24552E+00_JPRB ,&
&  0.25075E+00_JPRB, 0.25673E+00_JPRB, 0.26496E+00_JPRB, 0.27564E+00_JPRB, 0.29116E+00_JPRB, 0.30847E+00_JPRB, 0.33530E+00_JPRB ,&
&  0.35938E+00_JPRB, 0.45834E+00_JPRB, 0.69370E+00_JPRB, 0.62365E+00_JPRB, 0.64388E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.63545E+00_JPRB, 0.65234E+00_JPRB, 0.67714E+00_JPRB, 0.66996E+00_JPRB, 0.68994E+00_JPRB, 0.67652E+00_JPRB, 0.65346E+00_JPRB ,&
&  0.60611E+00_JPRB, 0.54964E+00_JPRB, 0.50289E+00_JPRB, 0.45078E+00_JPRB, 0.40995E+00_JPRB, 0.38041E+00_JPRB, 0.36584E+00_JPRB ,&
&  0.35845E+00_JPRB, 0.35608E+00_JPRB, 0.35437E+00_JPRB, 0.35335E+00_JPRB, 0.34826E+00_JPRB, 0.34473E+00_JPRB, 0.33883E+00_JPRB ,&
&  0.33294E+00_JPRB, 0.32897E+00_JPRB, 0.32577E+00_JPRB, 0.33073E+00_JPRB, 0.33677E+00_JPRB, 0.34303E+00_JPRB, 0.34619E+00_JPRB ,&
&  0.34889E+00_JPRB, 0.35031E+00_JPRB, 0.35317E+00_JPRB, 0.35109E+00_JPRB, 0.36231E+00_JPRB, 0.38338E+00_JPRB, 0.40112E+00_JPRB ,&
&  0.41833E+00_JPRB, 0.43262E+00_JPRB, 0.42841E+00_JPRB, 0.40449E+00_JPRB, 0.33861E+00_JPRB, 0.30449E+00_JPRB, 0.27431E+00_JPRB ,&
&  0.25519E+00_JPRB, 0.24100E+00_JPRB, 0.23139E+00_JPRB, 0.22592E+00_JPRB, 0.22144E+00_JPRB, 0.21903E+00_JPRB, 0.21645E+00_JPRB ,&
&  0.21401E+00_JPRB, 0.21257E+00_JPRB, 0.21089E+00_JPRB, 0.21007E+00_JPRB, 0.20910E+00_JPRB, 0.20780E+00_JPRB, 0.20690E+00_JPRB ,&
&  0.20716E+00_JPRB, 0.20703E+00_JPRB, 0.20825E+00_JPRB, 0.20929E+00_JPRB, 0.21128E+00_JPRB, 0.21291E+00_JPRB, 0.21521E+00_JPRB ,&
&  0.21718E+00_JPRB, 0.21883E+00_JPRB, 0.21870E+00_JPRB, 0.21826E+00_JPRB, 0.21737E+00_JPRB, 0.21689E+00_JPRB, 0.21605E+00_JPRB ,&
&  0.21492E+00_JPRB, 0.21515E+00_JPRB, 0.21550E+00_JPRB, 0.21591E+00_JPRB, 0.21711E+00_JPRB, 0.21876E+00_JPRB, 0.22124E+00_JPRB ,&
&  0.22451E+00_JPRB, 0.22792E+00_JPRB, 0.23080E+00_JPRB, 0.23389E+00_JPRB, 0.23787E+00_JPRB, 0.24237E+00_JPRB, 0.24795E+00_JPRB ,&
&  0.25275E+00_JPRB, 0.26119E+00_JPRB, 0.27300E+00_JPRB, 0.28538E+00_JPRB, 0.30472E+00_JPRB, 0.32271E+00_JPRB, 0.35271E+00_JPRB ,&
&  0.40080E+00_JPRB, 0.57059E+00_JPRB, 0.85357E+00_JPRB, 0.63031E+00_JPRB, 0.62044E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.55572E+00_JPRB, 0.56500E+00_JPRB, 0.57915E+00_JPRB, 0.59067E+00_JPRB, 0.61460E+00_JPRB, 0.62808E+00_JPRB, 0.67346E+00_JPRB ,&
&  0.62780E+00_JPRB, 0.54920E+00_JPRB, 0.51713E+00_JPRB, 0.48213E+00_JPRB, 0.44238E+00_JPRB, 0.41370E+00_JPRB, 0.39353E+00_JPRB ,&
&  0.38133E+00_JPRB, 0.37089E+00_JPRB, 0.36075E+00_JPRB, 0.34820E+00_JPRB, 0.33538E+00_JPRB, 0.32124E+00_JPRB, 0.31078E+00_JPRB ,&
&  0.30460E+00_JPRB, 0.30311E+00_JPRB, 0.30268E+00_JPRB, 0.30672E+00_JPRB, 0.31227E+00_JPRB, 0.31961E+00_JPRB, 0.32820E+00_JPRB ,&
&  0.34823E+00_JPRB, 0.35513E+00_JPRB, 0.35623E+00_JPRB, 0.35598E+00_JPRB, 0.35697E+00_JPRB, 0.35946E+00_JPRB, 0.34995E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.37803E+00_JPRB, 0.38376E+00_JPRB, 0.36430E+00_JPRB, 0.30979E+00_JPRB, 0.27735E+00_JPRB, 0.25471E+00_JPRB ,&
&  0.24641E+00_JPRB, 0.23645E+00_JPRB, 0.23023E+00_JPRB, 0.22425E+00_JPRB, 0.22024E+00_JPRB, 0.21835E+00_JPRB, 0.21644E+00_JPRB ,&
&  0.21511E+00_JPRB, 0.21425E+00_JPRB, 0.21228E+00_JPRB, 0.21143E+00_JPRB, 0.21030E+00_JPRB, 0.20969E+00_JPRB, 0.20938E+00_JPRB ,&
&  0.20922E+00_JPRB, 0.20893E+00_JPRB, 0.20916E+00_JPRB, 0.20971E+00_JPRB, 0.21065E+00_JPRB, 0.21139E+00_JPRB, 0.21207E+00_JPRB ,&
&  0.21300E+00_JPRB, 0.21387E+00_JPRB, 0.21411E+00_JPRB, 0.21465E+00_JPRB, 0.21512E+00_JPRB, 0.21635E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21752E+00_JPRB, 0.21804E+00_JPRB, 0.21806E+00_JPRB, 0.21873E+00_JPRB, 0.21971E+00_JPRB, 0.22163E+00_JPRB, 0.22378E+00_JPRB ,&
&  0.22653E+00_JPRB, 0.22977E+00_JPRB, 0.23333E+00_JPRB, 0.23692E+00_JPRB, 0.24161E+00_JPRB, 0.24668E+00_JPRB, 0.25332E+00_JPRB ,&
&  0.26052E+00_JPRB, 0.26984E+00_JPRB, 0.28258E+00_JPRB, 0.29797E+00_JPRB, 0.31682E+00_JPRB, 0.33512E+00_JPRB, 0.35542E+00_JPRB ,&
&  0.38981E+00_JPRB, 0.48835E+00_JPRB, 0.54860E+00_JPRB, 0.53804E+00_JPRB, 0.54928E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.53817E+00_JPRB, 0.54075E+00_JPRB, 0.54351E+00_JPRB, 0.54776E+00_JPRB, 0.55231E+00_JPRB, 0.55670E+00_JPRB, 0.56041E+00_JPRB ,&
&  0.54202E+00_JPRB, 0.51891E+00_JPRB, 0.49986E+00_JPRB, 0.47717E+00_JPRB, 0.45646E+00_JPRB, 0.43911E+00_JPRB, 0.41891E+00_JPRB ,&
&  0.39870E+00_JPRB, 0.37855E+00_JPRB, 0.35586E+00_JPRB, 0.33723E+00_JPRB, 0.31890E+00_JPRB, 0.30409E+00_JPRB, 0.29524E+00_JPRB ,&
&  0.29033E+00_JPRB, 0.28896E+00_JPRB, 0.29036E+00_JPRB, 0.29308E+00_JPRB, 0.29573E+00_JPRB, 0.30031E+00_JPRB, 0.30673E+00_JPRB ,&
&  0.31562E+00_JPRB, 0.33044E+00_JPRB, 0.34373E+00_JPRB, 0.34845E+00_JPRB, 0.34475E+00_JPRB, 0.33616E+00_JPRB, 0.32873E+00_JPRB ,&
&  0.31981E+00_JPRB, 0.30766E+00_JPRB, 0.29278E+00_JPRB, 0.27802E+00_JPRB, 0.26428E+00_JPRB, 0.25430E+00_JPRB, 0.24592E+00_JPRB ,&
&  0.23879E+00_JPRB, 0.23248E+00_JPRB, 0.22760E+00_JPRB, 0.22432E+00_JPRB, 0.22152E+00_JPRB, 0.21928E+00_JPRB, 0.21713E+00_JPRB ,&
&  0.21610E+00_JPRB, 0.21503E+00_JPRB, 0.21439E+00_JPRB, 0.21405E+00_JPRB, 0.21355E+00_JPRB, 0.21341E+00_JPRB, 0.21331E+00_JPRB ,&
&  0.21297E+00_JPRB, 0.21235E+00_JPRB, 0.21201E+00_JPRB, 0.21179E+00_JPRB, 0.21223E+00_JPRB, 0.21260E+00_JPRB, 0.21319E+00_JPRB ,&
&  0.21412E+00_JPRB, 0.21600E+00_JPRB, 0.21717E+00_JPRB, 0.21827E+00_JPRB, 0.21885E+00_JPRB, 0.21935E+00_JPRB, 0.22002E+00_JPRB ,&
&  0.22112E+00_JPRB, 0.22244E+00_JPRB, 0.22361E+00_JPRB, 0.22518E+00_JPRB, 0.22678E+00_JPRB, 0.22898E+00_JPRB, 0.23135E+00_JPRB ,&
&  0.23404E+00_JPRB, 0.23735E+00_JPRB, 0.24147E+00_JPRB, 0.24519E+00_JPRB, 0.25104E+00_JPRB, 0.25693E+00_JPRB, 0.26356E+00_JPRB ,&
&  0.27041E+00_JPRB, 0.27782E+00_JPRB, 0.28758E+00_JPRB, 0.29918E+00_JPRB, 0.31367E+00_JPRB, 0.32178E+00_JPRB, 0.33272E+00_JPRB ,&
&  0.34957E+00_JPRB, 0.38508E+00_JPRB, 0.52115E+00_JPRB, 0.53475E+00_JPRB, 0.53957E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.31897E+00_JPRB, 0.32475E+00_JPRB, 0.32873E+00_JPRB, 0.33772E+00_JPRB, 0.35009E+00_JPRB, 0.44755E+00_JPRB, 0.44255E+00_JPRB ,&
&  0.44169E+00_JPRB, 0.39865E+00_JPRB, 0.39070E+00_JPRB, 0.37856E+00_JPRB, 0.36323E+00_JPRB, 0.34428E+00_JPRB, 0.33256E+00_JPRB ,&
&  0.32280E+00_JPRB, 0.31142E+00_JPRB, 0.30080E+00_JPRB, 0.29455E+00_JPRB, 0.28959E+00_JPRB, 0.28519E+00_JPRB, 0.28259E+00_JPRB ,&
&  0.28151E+00_JPRB, 0.28184E+00_JPRB, 0.28251E+00_JPRB, 0.28341E+00_JPRB, 0.28669E+00_JPRB, 0.29030E+00_JPRB, 0.29394E+00_JPRB ,&
&  0.29565E+00_JPRB, 0.29992E+00_JPRB, 0.30082E+00_JPRB, 0.29767E+00_JPRB, 0.29257E+00_JPRB, 0.28534E+00_JPRB, 0.27802E+00_JPRB ,&
&  0.27050E+00_JPRB, 0.26352E+00_JPRB, 0.25776E+00_JPRB, 0.25255E+00_JPRB, 0.24661E+00_JPRB, 0.24096E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23400E+00_JPRB, 0.23061E+00_JPRB, 0.22793E+00_JPRB, 0.22659E+00_JPRB, 0.22509E+00_JPRB, 0.22408E+00_JPRB, 0.22333E+00_JPRB ,&
&  0.22313E+00_JPRB, 0.22302E+00_JPRB, 0.22310E+00_JPRB, 0.22339E+00_JPRB, 0.22337E+00_JPRB, 0.22330E+00_JPRB, 0.22327E+00_JPRB ,&
&  0.22384E+00_JPRB, 0.22388E+00_JPRB, 0.22382E+00_JPRB, 0.22360E+00_JPRB, 0.22430E+00_JPRB, 0.22454E+00_JPRB, 0.22496E+00_JPRB ,&
&  0.22560E+00_JPRB, 0.22648E+00_JPRB, 0.22715E+00_JPRB, 0.22789E+00_JPRB, 0.22862E+00_JPRB, 0.22949E+00_JPRB, 0.23051E+00_JPRB ,&
&  0.23155E+00_JPRB, 0.23307E+00_JPRB, 0.23463E+00_JPRB, 0.23600E+00_JPRB, 0.23723E+00_JPRB, 0.23890E+00_JPRB, 0.24077E+00_JPRB ,&
&  0.24357E+00_JPRB, 0.24628E+00_JPRB, 0.24906E+00_JPRB, 0.25070E+00_JPRB, 0.25338E+00_JPRB, 0.25564E+00_JPRB, 0.25805E+00_JPRB ,&
&  0.26088E+00_JPRB, 0.26544E+00_JPRB, 0.27213E+00_JPRB, 0.27948E+00_JPRB, 0.28769E+00_JPRB, 0.29445E+00_JPRB, 0.29888E+00_JPRB ,&
&  0.30268E+00_JPRB, 0.30686E+00_JPRB, 0.31237E+00_JPRB, 0.31529E+00_JPRB, 0.31760E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.28408E+00_JPRB, 0.28407E+00_JPRB, 0.28413E+00_JPRB, 0.28377E+00_JPRB, 0.28217E+00_JPRB, 0.28091E+00_JPRB, 0.27977E+00_JPRB ,&
&  0.27915E+00_JPRB, 0.27823E+00_JPRB, 0.27778E+00_JPRB, 0.27756E+00_JPRB, 0.27713E+00_JPRB, 0.27652E+00_JPRB, 0.27572E+00_JPRB ,&
&  0.27468E+00_JPRB, 0.27333E+00_JPRB, 0.27189E+00_JPRB, 0.27155E+00_JPRB, 0.27156E+00_JPRB, 0.27160E+00_JPRB, 0.27155E+00_JPRB ,&
&  0.27140E+00_JPRB, 0.27125E+00_JPRB, 0.27068E+00_JPRB, 0.27073E+00_JPRB, 0.27071E+00_JPRB, 0.27070E+00_JPRB, 0.27079E+00_JPRB ,&
&  0.27099E+00_JPRB, 0.27119E+00_JPRB, 0.27097E+00_JPRB, 0.27051E+00_JPRB, 0.26962E+00_JPRB, 0.26794E+00_JPRB, 0.26529E+00_JPRB ,&
&  0.26227E+00_JPRB, 0.25926E+00_JPRB, 0.25685E+00_JPRB, 0.25463E+00_JPRB, 0.25261E+00_JPRB, 0.25076E+00_JPRB, 0.24923E+00_JPRB ,&
&  0.24741E+00_JPRB, 0.24560E+00_JPRB, 0.24394E+00_JPRB, 0.24275E+00_JPRB, 0.24145E+00_JPRB, 0.24031E+00_JPRB, 0.23926E+00_JPRB ,&
&  0.23853E+00_JPRB, 0.23790E+00_JPRB, 0.23735E+00_JPRB, 0.23707E+00_JPRB, 0.23685E+00_JPRB, 0.23665E+00_JPRB, 0.23642E+00_JPRB ,&
&  0.23622E+00_JPRB, 0.23608E+00_JPRB, 0.23597E+00_JPRB, 0.23597E+00_JPRB, 0.23608E+00_JPRB, 0.23600E+00_JPRB, 0.23588E+00_JPRB ,&
&  0.23586E+00_JPRB, 0.23615E+00_JPRB, 0.23635E+00_JPRB, 0.23667E+00_JPRB, 0.23716E+00_JPRB, 0.23784E+00_JPRB, 0.23857E+00_JPRB ,&
&  0.23931E+00_JPRB, 0.24017E+00_JPRB, 0.24110E+00_JPRB, 0.24224E+00_JPRB, 0.24347E+00_JPRB, 0.24469E+00_JPRB, 0.24586E+00_JPRB ,&
&  0.24711E+00_JPRB, 0.24826E+00_JPRB, 0.24935E+00_JPRB, 0.25012E+00_JPRB, 0.25121E+00_JPRB, 0.25232E+00_JPRB, 0.25362E+00_JPRB ,&
&  0.25511E+00_JPRB, 0.25709E+00_JPRB, 0.25985E+00_JPRB, 0.26315E+00_JPRB, 0.26733E+00_JPRB, 0.27038E+00_JPRB, 0.27348E+00_JPRB ,&
&  0.27568E+00_JPRB, 0.27742E+00_JPRB, 0.27871E+00_JPRB, 0.28035E+00_JPRB, 0.28217E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB ,&
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB ,&
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB ,&
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB ,&
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB ,&
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB ,&
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB ,&
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB ,&
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB ,&
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB ,&
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB ,&
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB ,&
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB ,&
&  0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB, 0.25351E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2080:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_A1B2080
