!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_A1B2090 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS2090* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB, 0.25830E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.26716E+00_JPRB, 0.26691E+00_JPRB, 0.26668E+00_JPRB, 0.26649E+00_JPRB, 0.26630E+00_JPRB, 0.26619E+00_JPRB, 0.26611E+00_JPRB ,&
&  0.26596E+00_JPRB, 0.26583E+00_JPRB, 0.26559E+00_JPRB, 0.26534E+00_JPRB, 0.26514E+00_JPRB, 0.26489E+00_JPRB, 0.26457E+00_JPRB ,&
&  0.26417E+00_JPRB, 0.26384E+00_JPRB, 0.26338E+00_JPRB, 0.26281E+00_JPRB, 0.26222E+00_JPRB, 0.26161E+00_JPRB, 0.26087E+00_JPRB ,&
&  0.26001E+00_JPRB, 0.25918E+00_JPRB, 0.25828E+00_JPRB, 0.25732E+00_JPRB, 0.25631E+00_JPRB, 0.25523E+00_JPRB, 0.25427E+00_JPRB ,&
&  0.25354E+00_JPRB, 0.25272E+00_JPRB, 0.25190E+00_JPRB, 0.25108E+00_JPRB, 0.25040E+00_JPRB, 0.24988E+00_JPRB, 0.24933E+00_JPRB ,&
&  0.24880E+00_JPRB, 0.24836E+00_JPRB, 0.24830E+00_JPRB, 0.24837E+00_JPRB, 0.24854E+00_JPRB, 0.24899E+00_JPRB, 0.24978E+00_JPRB ,&
&  0.25042E+00_JPRB, 0.25099E+00_JPRB, 0.25170E+00_JPRB, 0.25249E+00_JPRB, 0.25333E+00_JPRB, 0.25419E+00_JPRB, 0.25494E+00_JPRB ,&
&  0.25552E+00_JPRB, 0.25599E+00_JPRB, 0.25651E+00_JPRB, 0.25693E+00_JPRB, 0.25718E+00_JPRB, 0.25742E+00_JPRB, 0.25768E+00_JPRB ,&
&  0.25787E+00_JPRB, 0.25800E+00_JPRB, 0.25814E+00_JPRB, 0.25839E+00_JPRB, 0.25862E+00_JPRB, 0.25888E+00_JPRB, 0.25928E+00_JPRB ,&
&  0.25983E+00_JPRB, 0.26023E+00_JPRB, 0.26071E+00_JPRB, 0.26135E+00_JPRB, 0.26218E+00_JPRB, 0.26317E+00_JPRB, 0.26396E+00_JPRB ,&
&  0.26480E+00_JPRB, 0.26566E+00_JPRB, 0.26663E+00_JPRB, 0.26733E+00_JPRB, 0.26807E+00_JPRB, 0.26873E+00_JPRB, 0.26938E+00_JPRB ,&
&  0.26955E+00_JPRB, 0.26983E+00_JPRB, 0.27019E+00_JPRB, 0.27051E+00_JPRB, 0.27028E+00_JPRB, 0.27017E+00_JPRB, 0.27009E+00_JPRB ,&
&  0.27031E+00_JPRB, 0.26986E+00_JPRB, 0.26969E+00_JPRB, 0.26943E+00_JPRB, 0.26934E+00_JPRB, 0.26892E+00_JPRB, 0.26865E+00_JPRB ,&
&  0.26835E+00_JPRB, 0.26814E+00_JPRB, 0.26787E+00_JPRB, 0.26761E+00_JPRB, 0.26736E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.27576E+00_JPRB, 0.27532E+00_JPRB, 0.27481E+00_JPRB, 0.27425E+00_JPRB, 0.27367E+00_JPRB, 0.27327E+00_JPRB, 0.27285E+00_JPRB ,&
&  0.27236E+00_JPRB, 0.27196E+00_JPRB, 0.27136E+00_JPRB, 0.27087E+00_JPRB, 0.27045E+00_JPRB, 0.27033E+00_JPRB, 0.27003E+00_JPRB ,&
&  0.26967E+00_JPRB, 0.26907E+00_JPRB, 0.26853E+00_JPRB, 0.26753E+00_JPRB, 0.26638E+00_JPRB, 0.26482E+00_JPRB, 0.26333E+00_JPRB ,&
&  0.26154E+00_JPRB, 0.25947E+00_JPRB, 0.25763E+00_JPRB, 0.25568E+00_JPRB, 0.25380E+00_JPRB, 0.25190E+00_JPRB, 0.25054E+00_JPRB ,&
&  0.24967E+00_JPRB, 0.24845E+00_JPRB, 0.24738E+00_JPRB, 0.24638E+00_JPRB, 0.24523E+00_JPRB, 0.24437E+00_JPRB, 0.24354E+00_JPRB ,&
&  0.24206E+00_JPRB, 0.24050E+00_JPRB, 0.24033E+00_JPRB, 0.24037E+00_JPRB, 0.24083E+00_JPRB, 0.24207E+00_JPRB, 0.24449E+00_JPRB ,&
&  0.24769E+00_JPRB, 0.25076E+00_JPRB, 0.25402E+00_JPRB, 0.25692E+00_JPRB, 0.25926E+00_JPRB, 0.26102E+00_JPRB, 0.26243E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26386E+00_JPRB, 0.26416E+00_JPRB, 0.26415E+00_JPRB, 0.26377E+00_JPRB, 0.26365E+00_JPRB, 0.26372E+00_JPRB ,&
&  0.26392E+00_JPRB, 0.26319E+00_JPRB, 0.26299E+00_JPRB, 0.26321E+00_JPRB, 0.26379E+00_JPRB, 0.26430E+00_JPRB, 0.26537E+00_JPRB ,&
&  0.26696E+00_JPRB, 0.26892E+00_JPRB, 0.27073E+00_JPRB, 0.27315E+00_JPRB, 0.27599E+00_JPRB, 0.27928E+00_JPRB, 0.28127E+00_JPRB ,&
&  0.28372E+00_JPRB, 0.28629E+00_JPRB, 0.28900E+00_JPRB, 0.28963E+00_JPRB, 0.29034E+00_JPRB, 0.29086E+00_JPRB, 0.29155E+00_JPRB ,&
&  0.29050E+00_JPRB, 0.29010E+00_JPRB, 0.28985E+00_JPRB, 0.29024E+00_JPRB, 0.28893E+00_JPRB, 0.28831E+00_JPRB, 0.28767E+00_JPRB ,&
&  0.28802E+00_JPRB, 0.28610E+00_JPRB, 0.28511E+00_JPRB, 0.28386E+00_JPRB, 0.28321E+00_JPRB, 0.28142E+00_JPRB, 0.28006E+00_JPRB ,&
&  0.27896E+00_JPRB, 0.27836E+00_JPRB, 0.27754E+00_JPRB, 0.27685E+00_JPRB, 0.27634E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.28322E+00_JPRB, 0.28173E+00_JPRB, 0.27944E+00_JPRB, 0.27647E+00_JPRB, 0.27385E+00_JPRB, 0.27189E+00_JPRB, 0.26985E+00_JPRB ,&
&  0.26823E+00_JPRB, 0.26683E+00_JPRB, 0.26591E+00_JPRB, 0.26529E+00_JPRB, 0.26469E+00_JPRB, 0.26532E+00_JPRB, 0.26604E+00_JPRB ,&
&  0.26663E+00_JPRB, 0.26671E+00_JPRB, 0.26779E+00_JPRB, 0.26774E+00_JPRB, 0.26723E+00_JPRB, 0.26573E+00_JPRB, 0.26337E+00_JPRB ,&
&  0.26060E+00_JPRB, 0.25732E+00_JPRB, 0.25345E+00_JPRB, 0.24897E+00_JPRB, 0.24481E+00_JPRB, 0.22998E+00_JPRB, 0.22663E+00_JPRB ,&
&  0.22767E+00_JPRB, 0.22533E+00_JPRB, 0.22357E+00_JPRB, 0.22266E+00_JPRB, 0.22175E+00_JPRB, 0.22031E+00_JPRB, 0.21883E+00_JPRB ,&
&  0.20730E+00_JPRB, 0.20635E+00_JPRB, 0.20463E+00_JPRB, 0.20352E+00_JPRB, 0.20502E+00_JPRB, 0.21061E+00_JPRB, 0.22002E+00_JPRB ,&
&  0.23746E+00_JPRB, 0.24458E+00_JPRB, 0.25079E+00_JPRB, 0.25805E+00_JPRB, 0.26374E+00_JPRB, 0.26791E+00_JPRB, 0.27191E+00_JPRB ,&
&  0.27507E+00_JPRB, 0.27807E+00_JPRB, 0.28014E+00_JPRB, 0.28107E+00_JPRB, 0.28088E+00_JPRB, 0.28061E+00_JPRB, 0.28032E+00_JPRB ,&
&  0.28019E+00_JPRB, 0.28009E+00_JPRB, 0.28030E+00_JPRB, 0.28028E+00_JPRB, 0.28005E+00_JPRB, 0.28016E+00_JPRB, 0.28185E+00_JPRB ,&
&  0.28440E+00_JPRB, 0.28844E+00_JPRB, 0.29195E+00_JPRB, 0.29705E+00_JPRB, 0.30169E+00_JPRB, 0.30768E+00_JPRB, 0.31005E+00_JPRB ,&
&  0.31299E+00_JPRB, 0.31487E+00_JPRB, 0.31739E+00_JPRB, 0.31733E+00_JPRB, 0.31718E+00_JPRB, 0.31655E+00_JPRB, 0.31655E+00_JPRB ,&
&  0.31496E+00_JPRB, 0.31383E+00_JPRB, 0.31227E+00_JPRB, 0.31195E+00_JPRB, 0.30999E+00_JPRB, 0.30940E+00_JPRB, 0.30807E+00_JPRB ,&
&  0.30853E+00_JPRB, 0.30635E+00_JPRB, 0.30557E+00_JPRB, 0.30298E+00_JPRB, 0.30080E+00_JPRB, 0.29687E+00_JPRB, 0.29354E+00_JPRB ,&
&  0.29079E+00_JPRB, 0.28874E+00_JPRB, 0.28707E+00_JPRB, 0.28568E+00_JPRB, 0.28433E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.28725E+00_JPRB, 0.28284E+00_JPRB, 0.27795E+00_JPRB, 0.27384E+00_JPRB, 0.27013E+00_JPRB, 0.26745E+00_JPRB, 0.26495E+00_JPRB ,&
&  0.26309E+00_JPRB, 0.26138E+00_JPRB, 0.26019E+00_JPRB, 0.25920E+00_JPRB, 0.25854E+00_JPRB, 0.25840E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25871E+00_JPRB, 0.25894E+00_JPRB, 0.25973E+00_JPRB, 0.26016E+00_JPRB, 0.26086E+00_JPRB, 0.26023E+00_JPRB, 0.25803E+00_JPRB ,&
&  0.25537E+00_JPRB, 0.25224E+00_JPRB, 0.24030E+00_JPRB, 0.23615E+00_JPRB, 0.22625E+00_JPRB, 0.21939E+00_JPRB, 0.21554E+00_JPRB ,&
&  0.20863E+00_JPRB, 0.20521E+00_JPRB, 0.19830E+00_JPRB, 0.19403E+00_JPRB, 0.19249E+00_JPRB, 0.18673E+00_JPRB, 0.18413E+00_JPRB ,&
&  0.18020E+00_JPRB, 0.17359E+00_JPRB, 0.17094E+00_JPRB, 0.16973E+00_JPRB, 0.17189E+00_JPRB, 0.17744E+00_JPRB, 0.18728E+00_JPRB ,&
&  0.20561E+00_JPRB, 0.22595E+00_JPRB, 0.25366E+00_JPRB, 0.26149E+00_JPRB, 0.26325E+00_JPRB, 0.26964E+00_JPRB, 0.27453E+00_JPRB ,&
&  0.27752E+00_JPRB, 0.28412E+00_JPRB, 0.29143E+00_JPRB, 0.29395E+00_JPRB, 0.29580E+00_JPRB, 0.29632E+00_JPRB, 0.29644E+00_JPRB ,&
&  0.29682E+00_JPRB, 0.29938E+00_JPRB, 0.30092E+00_JPRB, 0.30258E+00_JPRB, 0.30422E+00_JPRB, 0.30584E+00_JPRB, 0.30949E+00_JPRB ,&
&  0.31256E+00_JPRB, 0.31982E+00_JPRB, 0.32578E+00_JPRB, 0.33540E+00_JPRB, 0.33933E+00_JPRB, 0.34420E+00_JPRB, 0.34536E+00_JPRB ,&
&  0.34748E+00_JPRB, 0.34861E+00_JPRB, 0.34955E+00_JPRB, 0.34657E+00_JPRB, 0.34591E+00_JPRB, 0.34791E+00_JPRB, 0.34698E+00_JPRB ,&
&  0.34562E+00_JPRB, 0.34446E+00_JPRB, 0.34283E+00_JPRB, 0.34133E+00_JPRB, 0.34014E+00_JPRB, 0.33983E+00_JPRB, 0.33889E+00_JPRB ,&
&  0.33961E+00_JPRB, 0.33800E+00_JPRB, 0.33630E+00_JPRB, 0.33294E+00_JPRB, 0.32852E+00_JPRB, 0.32251E+00_JPRB, 0.31516E+00_JPRB ,&
&  0.30894E+00_JPRB, 0.30429E+00_JPRB, 0.29910E+00_JPRB, 0.29525E+00_JPRB, 0.29113E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.27720E+00_JPRB, 0.27152E+00_JPRB, 0.26693E+00_JPRB, 0.26379E+00_JPRB, 0.26117E+00_JPRB, 0.25962E+00_JPRB, 0.25801E+00_JPRB ,&
&  0.25724E+00_JPRB, 0.25613E+00_JPRB, 0.25564E+00_JPRB, 0.25501E+00_JPRB, 0.25493E+00_JPRB, 0.25515E+00_JPRB, 0.25578E+00_JPRB ,&
&  0.25664E+00_JPRB, 0.25721E+00_JPRB, 0.25803E+00_JPRB, 0.25751E+00_JPRB, 0.25722E+00_JPRB, 0.25634E+00_JPRB, 0.25549E+00_JPRB ,&
&  0.25246E+00_JPRB, 0.24840E+00_JPRB, 0.24000E+00_JPRB, 0.23533E+00_JPRB, 0.22665E+00_JPRB, 0.22082E+00_JPRB, 0.21514E+00_JPRB ,&
&  0.21137E+00_JPRB, 0.19958E+00_JPRB, 0.20231E+00_JPRB, 0.19436E+00_JPRB, 0.18651E+00_JPRB, 0.17884E+00_JPRB, 0.17036E+00_JPRB ,&
&  0.16694E+00_JPRB, 0.15858E+00_JPRB, 0.15552E+00_JPRB, 0.15551E+00_JPRB, 0.15694E+00_JPRB, 0.16317E+00_JPRB, 0.17275E+00_JPRB ,&
&  0.19089E+00_JPRB, 0.23674E+00_JPRB, 0.25189E+00_JPRB, 0.25951E+00_JPRB, 0.25884E+00_JPRB, 0.26018E+00_JPRB, 0.25882E+00_JPRB ,&
&  0.26623E+00_JPRB, 0.27537E+00_JPRB, 0.28509E+00_JPRB, 0.29094E+00_JPRB, 0.29931E+00_JPRB, 0.30379E+00_JPRB, 0.30696E+00_JPRB ,&
&  0.31075E+00_JPRB, 0.31708E+00_JPRB, 0.32251E+00_JPRB, 0.32827E+00_JPRB, 0.33619E+00_JPRB, 0.34479E+00_JPRB, 0.35492E+00_JPRB ,&
&  0.36089E+00_JPRB, 0.37313E+00_JPRB, 0.37726E+00_JPRB, 0.38295E+00_JPRB, 0.38354E+00_JPRB, 0.38239E+00_JPRB, 0.38220E+00_JPRB ,&
&  0.38345E+00_JPRB, 0.38512E+00_JPRB, 0.38645E+00_JPRB, 0.38567E+00_JPRB, 0.38529E+00_JPRB, 0.38011E+00_JPRB, 0.37775E+00_JPRB ,&
&  0.37672E+00_JPRB, 0.37465E+00_JPRB, 0.37208E+00_JPRB, 0.36867E+00_JPRB, 0.36468E+00_JPRB, 0.36020E+00_JPRB, 0.35747E+00_JPRB ,&
&  0.35402E+00_JPRB, 0.35019E+00_JPRB, 0.34570E+00_JPRB, 0.34141E+00_JPRB, 0.33682E+00_JPRB, 0.33212E+00_JPRB, 0.32417E+00_JPRB ,&
&  0.31394E+00_JPRB, 0.30531E+00_JPRB, 0.29695E+00_JPRB, 0.29038E+00_JPRB, 0.28350E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.26912E+00_JPRB, 0.26442E+00_JPRB, 0.26075E+00_JPRB, 0.25920E+00_JPRB, 0.25750E+00_JPRB, 0.25703E+00_JPRB, 0.25581E+00_JPRB ,&
&  0.25516E+00_JPRB, 0.25422E+00_JPRB, 0.25371E+00_JPRB, 0.25300E+00_JPRB, 0.25274E+00_JPRB, 0.25254E+00_JPRB, 0.25293E+00_JPRB ,&
&  0.25347E+00_JPRB, 0.25468E+00_JPRB, 0.25597E+00_JPRB, 0.25611E+00_JPRB, 0.25647E+00_JPRB, 0.25614E+00_JPRB, 0.25547E+00_JPRB ,&
&  0.25303E+00_JPRB, 0.24904E+00_JPRB, 0.24084E+00_JPRB, 0.23488E+00_JPRB, 0.22976E+00_JPRB, 0.22261E+00_JPRB, 0.22192E+00_JPRB ,&
&  0.22025E+00_JPRB, 0.21536E+00_JPRB, 0.20743E+00_JPRB, 0.20459E+00_JPRB, 0.19954E+00_JPRB, 0.17941E+00_JPRB, 0.17868E+00_JPRB ,&
&  0.17244E+00_JPRB, 0.15974E+00_JPRB, 0.15587E+00_JPRB, 0.15323E+00_JPRB, 0.15431E+00_JPRB, 0.15990E+00_JPRB, 0.16876E+00_JPRB ,&
&  0.18679E+00_JPRB, 0.24168E+00_JPRB, 0.25515E+00_JPRB, 0.26131E+00_JPRB, 0.25823E+00_JPRB, 0.26189E+00_JPRB, 0.26321E+00_JPRB ,&
&  0.26906E+00_JPRB, 0.27359E+00_JPRB, 0.27944E+00_JPRB, 0.28404E+00_JPRB, 0.28936E+00_JPRB, 0.29117E+00_JPRB, 0.29620E+00_JPRB ,&
&  0.30589E+00_JPRB, 0.31705E+00_JPRB, 0.32628E+00_JPRB, 0.33891E+00_JPRB, 0.35699E+00_JPRB, 0.39135E+00_JPRB, 0.42936E+00_JPRB ,&
&  0.43290E+00_JPRB, 0.42941E+00_JPRB, 0.42749E+00_JPRB, 0.42341E+00_JPRB, 0.42219E+00_JPRB, 0.42134E+00_JPRB, 0.42339E+00_JPRB ,&
&  0.42861E+00_JPRB, 0.43373E+00_JPRB, 0.42704E+00_JPRB, 0.41233E+00_JPRB, 0.40591E+00_JPRB, 0.39504E+00_JPRB, 0.39239E+00_JPRB ,&
&  0.38873E+00_JPRB, 0.39267E+00_JPRB, 0.38835E+00_JPRB, 0.38188E+00_JPRB, 0.37594E+00_JPRB, 0.36959E+00_JPRB, 0.36534E+00_JPRB ,&
&  0.36019E+00_JPRB, 0.35418E+00_JPRB, 0.34802E+00_JPRB, 0.34333E+00_JPRB, 0.33932E+00_JPRB, 0.33617E+00_JPRB, 0.33097E+00_JPRB ,&
&  0.31996E+00_JPRB, 0.30567E+00_JPRB, 0.29466E+00_JPRB, 0.28402E+00_JPRB, 0.27544E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.26629E+00_JPRB, 0.26417E+00_JPRB, 0.26329E+00_JPRB, 0.26441E+00_JPRB, 0.26483E+00_JPRB, 0.26549E+00_JPRB, 0.26399E+00_JPRB ,&
&  0.26241E+00_JPRB, 0.26062E+00_JPRB, 0.25896E+00_JPRB, 0.25723E+00_JPRB, 0.25598E+00_JPRB, 0.25458E+00_JPRB, 0.25352E+00_JPRB ,&
&  0.25247E+00_JPRB, 0.24874E+00_JPRB, 0.24731E+00_JPRB, 0.24849E+00_JPRB, 0.25022E+00_JPRB, 0.25179E+00_JPRB, 0.25382E+00_JPRB ,&
&  0.25245E+00_JPRB, 0.24890E+00_JPRB, 0.24385E+00_JPRB, 0.23708E+00_JPRB, 0.23174E+00_JPRB, 0.22961E+00_JPRB, 0.22641E+00_JPRB ,&
&  0.22010E+00_JPRB, 0.21490E+00_JPRB, 0.21838E+00_JPRB, 0.21728E+00_JPRB, 0.20603E+00_JPRB, 0.20132E+00_JPRB, 0.19053E+00_JPRB ,&
&  0.17779E+00_JPRB, 0.16203E+00_JPRB, 0.15819E+00_JPRB, 0.15656E+00_JPRB, 0.16020E+00_JPRB, 0.16666E+00_JPRB, 0.18066E+00_JPRB ,&
&  0.20400E+00_JPRB, 0.24744E+00_JPRB, 0.26024E+00_JPRB, 0.26809E+00_JPRB, 0.27264E+00_JPRB, 0.27480E+00_JPRB, 0.27653E+00_JPRB ,&
&  0.28131E+00_JPRB, 0.28483E+00_JPRB, 0.29007E+00_JPRB, 0.29605E+00_JPRB, 0.30381E+00_JPRB, 0.30968E+00_JPRB, 0.31248E+00_JPRB ,&
&  0.32821E+00_JPRB, 0.33325E+00_JPRB, 0.33617E+00_JPRB, 0.35771E+00_JPRB, 0.40481E+00_JPRB, 0.45715E+00_JPRB, 0.48258E+00_JPRB ,&
&  0.48122E+00_JPRB, 0.47638E+00_JPRB, 0.45926E+00_JPRB, 0.44934E+00_JPRB, 0.44463E+00_JPRB, 0.44341E+00_JPRB, 0.44504E+00_JPRB ,&
&  0.45006E+00_JPRB, 0.46039E+00_JPRB, 0.49294E+00_JPRB, 0.46820E+00_JPRB, 0.45237E+00_JPRB, 0.43688E+00_JPRB, 0.41935E+00_JPRB ,&
&  0.40558E+00_JPRB, 0.39761E+00_JPRB, 0.39210E+00_JPRB, 0.38503E+00_JPRB, 0.37847E+00_JPRB, 0.37301E+00_JPRB, 0.37124E+00_JPRB ,&
&  0.36602E+00_JPRB, 0.35915E+00_JPRB, 0.35016E+00_JPRB, 0.34587E+00_JPRB, 0.34184E+00_JPRB, 0.33773E+00_JPRB, 0.33321E+00_JPRB ,&
&  0.32174E+00_JPRB, 0.30566E+00_JPRB, 0.29269E+00_JPRB, 0.28117E+00_JPRB, 0.27222E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.26989E+00_JPRB, 0.26991E+00_JPRB, 0.27241E+00_JPRB, 0.27742E+00_JPRB, 0.28168E+00_JPRB, 0.27800E+00_JPRB, 0.27338E+00_JPRB ,&
&  0.27025E+00_JPRB, 0.26926E+00_JPRB, 0.26894E+00_JPRB, 0.26697E+00_JPRB, 0.26527E+00_JPRB, 0.26337E+00_JPRB, 0.26128E+00_JPRB ,&
&  0.25897E+00_JPRB, 0.25637E+00_JPRB, 0.25329E+00_JPRB, 0.24906E+00_JPRB, 0.24673E+00_JPRB, 0.24735E+00_JPRB, 0.24984E+00_JPRB ,&
&  0.25221E+00_JPRB, 0.25171E+00_JPRB, 0.24954E+00_JPRB, 0.24361E+00_JPRB, 0.23687E+00_JPRB, 0.23159E+00_JPRB, 0.23136E+00_JPRB ,&
&  0.22853E+00_JPRB, 0.22301E+00_JPRB, 0.21751E+00_JPRB, 0.22147E+00_JPRB, 0.22346E+00_JPRB, 0.21230E+00_JPRB, 0.19900E+00_JPRB ,&
&  0.18537E+00_JPRB, 0.17330E+00_JPRB, 0.16770E+00_JPRB, 0.16743E+00_JPRB, 0.17385E+00_JPRB, 0.18623E+00_JPRB, 0.20368E+00_JPRB ,&
&  0.23071E+00_JPRB, 0.25320E+00_JPRB, 0.26478E+00_JPRB, 0.27407E+00_JPRB, 0.27672E+00_JPRB, 0.27698E+00_JPRB, 0.27718E+00_JPRB ,&
&  0.27934E+00_JPRB, 0.28192E+00_JPRB, 0.28628E+00_JPRB, 0.29011E+00_JPRB, 0.29952E+00_JPRB, 0.31399E+00_JPRB, 0.34188E+00_JPRB ,&
&  0.41908E+00_JPRB, 0.41702E+00_JPRB, 0.37311E+00_JPRB, 0.39601E+00_JPRB, 0.44527E+00_JPRB, 0.49179E+00_JPRB, 0.52812E+00_JPRB ,&
&  0.53515E+00_JPRB, 0.50965E+00_JPRB, 0.48678E+00_JPRB, 0.47446E+00_JPRB, 0.46624E+00_JPRB, 0.46050E+00_JPRB, 0.45946E+00_JPRB ,&
&  0.46382E+00_JPRB, 0.47819E+00_JPRB, 0.54445E+00_JPRB, 0.47633E+00_JPRB, 0.43705E+00_JPRB, 0.42085E+00_JPRB, 0.40418E+00_JPRB ,&
&  0.39059E+00_JPRB, 0.38361E+00_JPRB, 0.38339E+00_JPRB, 0.38499E+00_JPRB, 0.37055E+00_JPRB, 0.35743E+00_JPRB, 0.34779E+00_JPRB ,&
&  0.34374E+00_JPRB, 0.33844E+00_JPRB, 0.33539E+00_JPRB, 0.33725E+00_JPRB, 0.33579E+00_JPRB, 0.32864E+00_JPRB, 0.32451E+00_JPRB ,&
&  0.31698E+00_JPRB, 0.30303E+00_JPRB, 0.29341E+00_JPRB, 0.28420E+00_JPRB, 0.27497E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.26900E+00_JPRB, 0.27694E+00_JPRB, 0.28802E+00_JPRB, 0.29616E+00_JPRB, 0.29917E+00_JPRB, 0.28632E+00_JPRB, 0.27313E+00_JPRB ,&
&  0.26621E+00_JPRB, 0.26346E+00_JPRB, 0.26450E+00_JPRB, 0.26508E+00_JPRB, 0.26509E+00_JPRB, 0.26513E+00_JPRB, 0.26475E+00_JPRB ,&
&  0.26205E+00_JPRB, 0.25810E+00_JPRB, 0.25436E+00_JPRB, 0.25071E+00_JPRB, 0.24775E+00_JPRB, 0.24676E+00_JPRB, 0.24744E+00_JPRB ,&
&  0.24919E+00_JPRB, 0.24982E+00_JPRB, 0.24909E+00_JPRB, 0.24539E+00_JPRB, 0.24230E+00_JPRB, 0.23986E+00_JPRB, 0.23570E+00_JPRB ,&
&  0.22874E+00_JPRB, 0.22864E+00_JPRB, 0.22744E+00_JPRB, 0.22632E+00_JPRB, 0.22897E+00_JPRB, 0.22221E+00_JPRB, 0.20869E+00_JPRB ,&
&  0.19446E+00_JPRB, 0.18411E+00_JPRB, 0.18203E+00_JPRB, 0.18864E+00_JPRB, 0.20421E+00_JPRB, 0.22821E+00_JPRB, 0.24026E+00_JPRB ,&
&  0.25084E+00_JPRB, 0.26008E+00_JPRB, 0.26586E+00_JPRB, 0.27161E+00_JPRB, 0.27610E+00_JPRB, 0.27835E+00_JPRB, 0.27919E+00_JPRB ,&
&  0.27903E+00_JPRB, 0.27937E+00_JPRB, 0.28095E+00_JPRB, 0.26877E+00_JPRB, 0.27831E+00_JPRB, 0.33216E+00_JPRB, 0.36925E+00_JPRB ,&
&  0.41222E+00_JPRB, 0.41609E+00_JPRB, 0.42046E+00_JPRB, 0.45128E+00_JPRB, 0.48452E+00_JPRB, 0.51347E+00_JPRB, 0.53956E+00_JPRB ,&
&  0.54966E+00_JPRB, 0.52123E+00_JPRB, 0.49239E+00_JPRB, 0.48267E+00_JPRB, 0.47927E+00_JPRB, 0.47483E+00_JPRB, 0.45883E+00_JPRB ,&
&  0.44694E+00_JPRB, 0.42610E+00_JPRB, 0.39966E+00_JPRB, 0.37700E+00_JPRB, 0.37348E+00_JPRB, 0.38297E+00_JPRB, 0.39617E+00_JPRB ,&
&  0.39572E+00_JPRB, 0.39339E+00_JPRB, 0.39235E+00_JPRB, 0.39413E+00_JPRB, 0.36330E+00_JPRB, 0.34276E+00_JPRB, 0.32823E+00_JPRB ,&
&  0.32092E+00_JPRB, 0.31706E+00_JPRB, 0.31498E+00_JPRB, 0.31997E+00_JPRB, 0.32022E+00_JPRB, 0.31733E+00_JPRB, 0.30685E+00_JPRB ,&
&  0.29266E+00_JPRB, 0.28141E+00_JPRB, 0.27351E+00_JPRB, 0.26870E+00_JPRB, 0.26754E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.29904E+00_JPRB, 0.30625E+00_JPRB, 0.31066E+00_JPRB, 0.31437E+00_JPRB, 0.31190E+00_JPRB, 0.30645E+00_JPRB, 0.30014E+00_JPRB ,&
&  0.30159E+00_JPRB, 0.29425E+00_JPRB, 0.28757E+00_JPRB, 0.27722E+00_JPRB, 0.26846E+00_JPRB, 0.26077E+00_JPRB, 0.25927E+00_JPRB ,&
&  0.26086E+00_JPRB, 0.26268E+00_JPRB, 0.26128E+00_JPRB, 0.25678E+00_JPRB, 0.25205E+00_JPRB, 0.24916E+00_JPRB, 0.24868E+00_JPRB ,&
&  0.24945E+00_JPRB, 0.25081E+00_JPRB, 0.25093E+00_JPRB, 0.24947E+00_JPRB, 0.24777E+00_JPRB, 0.24560E+00_JPRB, 0.24277E+00_JPRB ,&
&  0.23808E+00_JPRB, 0.23573E+00_JPRB, 0.23479E+00_JPRB, 0.23507E+00_JPRB, 0.23819E+00_JPRB, 0.23199E+00_JPRB, 0.21906E+00_JPRB ,&
&  0.20481E+00_JPRB, 0.20050E+00_JPRB, 0.20891E+00_JPRB, 0.23647E+00_JPRB, 0.25203E+00_JPRB, 0.26464E+00_JPRB, 0.27549E+00_JPRB ,&
&  0.27463E+00_JPRB, 0.27223E+00_JPRB, 0.27619E+00_JPRB, 0.28896E+00_JPRB, 0.29133E+00_JPRB, 0.29189E+00_JPRB, 0.28998E+00_JPRB ,&
&  0.28723E+00_JPRB, 0.28703E+00_JPRB, 0.28773E+00_JPRB, 0.28086E+00_JPRB, 0.33578E+00_JPRB, 0.42766E+00_JPRB, 0.42868E+00_JPRB ,&
&  0.45680E+00_JPRB, 0.47415E+00_JPRB, 0.50593E+00_JPRB, 0.50991E+00_JPRB, 0.49505E+00_JPRB, 0.50568E+00_JPRB, 0.54531E+00_JPRB ,&
&  0.55664E+00_JPRB, 0.54064E+00_JPRB, 0.51793E+00_JPRB, 0.50928E+00_JPRB, 0.48893E+00_JPRB, 0.46788E+00_JPRB, 0.44864E+00_JPRB ,&
&  0.44326E+00_JPRB, 0.43255E+00_JPRB, 0.41298E+00_JPRB, 0.39064E+00_JPRB, 0.38263E+00_JPRB, 0.38244E+00_JPRB, 0.38929E+00_JPRB ,&
&  0.39169E+00_JPRB, 0.39686E+00_JPRB, 0.40018E+00_JPRB, 0.40270E+00_JPRB, 0.37426E+00_JPRB, 0.34161E+00_JPRB, 0.32060E+00_JPRB ,&
&  0.30610E+00_JPRB, 0.29582E+00_JPRB, 0.28700E+00_JPRB, 0.29074E+00_JPRB, 0.29806E+00_JPRB, 0.29902E+00_JPRB, 0.28696E+00_JPRB ,&
&  0.27767E+00_JPRB, 0.27370E+00_JPRB, 0.27296E+00_JPRB, 0.28132E+00_JPRB, 0.29091E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.31838E+00_JPRB, 0.32719E+00_JPRB, 0.32877E+00_JPRB, 0.32845E+00_JPRB, 0.33104E+00_JPRB, 0.32658E+00_JPRB, 0.31516E+00_JPRB ,&
&  0.31588E+00_JPRB, 0.29822E+00_JPRB, 0.29051E+00_JPRB, 0.28247E+00_JPRB, 0.27691E+00_JPRB, 0.27020E+00_JPRB, 0.26041E+00_JPRB ,&
&  0.25756E+00_JPRB, 0.26025E+00_JPRB, 0.26496E+00_JPRB, 0.26470E+00_JPRB, 0.26125E+00_JPRB, 0.25701E+00_JPRB, 0.25456E+00_JPRB ,&
&  0.25394E+00_JPRB, 0.25501E+00_JPRB, 0.25697E+00_JPRB, 0.25537E+00_JPRB, 0.25218E+00_JPRB, 0.24952E+00_JPRB, 0.24931E+00_JPRB ,&
&  0.24661E+00_JPRB, 0.24280E+00_JPRB, 0.24377E+00_JPRB, 0.24435E+00_JPRB, 0.24794E+00_JPRB, 0.25087E+00_JPRB, 0.24325E+00_JPRB ,&
&  0.22649E+00_JPRB, 0.23168E+00_JPRB, 0.26074E+00_JPRB, 0.27832E+00_JPRB, 0.29445E+00_JPRB, 0.30169E+00_JPRB, 0.30684E+00_JPRB ,&
&  0.31111E+00_JPRB, 0.31616E+00_JPRB, 0.31876E+00_JPRB, 0.32112E+00_JPRB, 0.32120E+00_JPRB, 0.31936E+00_JPRB, 0.31584E+00_JPRB ,&
&  0.31399E+00_JPRB, 0.29016E+00_JPRB, 0.28414E+00_JPRB, 0.30412E+00_JPRB, 0.44579E+00_JPRB, 0.45730E+00_JPRB, 0.45940E+00_JPRB ,&
&  0.48953E+00_JPRB, 0.50199E+00_JPRB, 0.53126E+00_JPRB, 0.54845E+00_JPRB, 0.54242E+00_JPRB, 0.54115E+00_JPRB, 0.57224E+00_JPRB ,&
&  0.59627E+00_JPRB, 0.59192E+00_JPRB, 0.56972E+00_JPRB, 0.54280E+00_JPRB, 0.50633E+00_JPRB, 0.49029E+00_JPRB, 0.48017E+00_JPRB ,&
&  0.48134E+00_JPRB, 0.47793E+00_JPRB, 0.45706E+00_JPRB, 0.43180E+00_JPRB, 0.41423E+00_JPRB, 0.41510E+00_JPRB, 0.41824E+00_JPRB ,&
&  0.41919E+00_JPRB, 0.42429E+00_JPRB, 0.42125E+00_JPRB, 0.41441E+00_JPRB, 0.39561E+00_JPRB, 0.37642E+00_JPRB, 0.36509E+00_JPRB ,&
&  0.33739E+00_JPRB, 0.30932E+00_JPRB, 0.28915E+00_JPRB, 0.28847E+00_JPRB, 0.29562E+00_JPRB, 0.29643E+00_JPRB, 0.28919E+00_JPRB ,&
&  0.28815E+00_JPRB, 0.28655E+00_JPRB, 0.28646E+00_JPRB, 0.29603E+00_JPRB, 0.30543E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.32549E+00_JPRB, 0.33308E+00_JPRB, 0.33510E+00_JPRB, 0.33520E+00_JPRB, 0.34173E+00_JPRB, 0.33802E+00_JPRB, 0.33900E+00_JPRB ,&
&  0.35793E+00_JPRB, 0.34804E+00_JPRB, 0.36379E+00_JPRB, 0.31534E+00_JPRB, 0.28972E+00_JPRB, 0.28061E+00_JPRB, 0.27008E+00_JPRB ,&
&  0.26704E+00_JPRB, 0.27047E+00_JPRB, 0.27230E+00_JPRB, 0.27209E+00_JPRB, 0.27115E+00_JPRB, 0.26507E+00_JPRB, 0.26153E+00_JPRB ,&
&  0.26089E+00_JPRB, 0.26247E+00_JPRB, 0.26508E+00_JPRB, 0.26311E+00_JPRB, 0.26024E+00_JPRB, 0.25802E+00_JPRB, 0.25596E+00_JPRB ,&
&  0.25249E+00_JPRB, 0.25341E+00_JPRB, 0.25569E+00_JPRB, 0.25482E+00_JPRB, 0.25728E+00_JPRB, 0.26315E+00_JPRB, 0.26536E+00_JPRB ,&
&  0.27423E+00_JPRB, 0.27555E+00_JPRB, 0.30204E+00_JPRB, 0.32101E+00_JPRB, 0.33198E+00_JPRB, 0.33539E+00_JPRB, 0.33822E+00_JPRB ,&
&  0.34440E+00_JPRB, 0.35281E+00_JPRB, 0.36193E+00_JPRB, 0.36910E+00_JPRB, 0.36993E+00_JPRB, 0.36878E+00_JPRB, 0.36435E+00_JPRB ,&
&  0.36187E+00_JPRB, 0.31665E+00_JPRB, 0.35978E+00_JPRB, 0.38392E+00_JPRB, 0.48709E+00_JPRB, 0.59417E+00_JPRB, 0.57223E+00_JPRB ,&
&  0.65151E+00_JPRB, 0.60706E+00_JPRB, 0.61062E+00_JPRB, 0.59876E+00_JPRB, 0.58935E+00_JPRB, 0.59811E+00_JPRB, 0.62524E+00_JPRB ,&
&  0.68234E+00_JPRB, 0.65986E+00_JPRB, 0.65099E+00_JPRB, 0.59508E+00_JPRB, 0.55844E+00_JPRB, 0.55332E+00_JPRB, 0.54493E+00_JPRB ,&
&  0.54375E+00_JPRB, 0.54256E+00_JPRB, 0.53169E+00_JPRB, 0.48068E+00_JPRB, 0.46816E+00_JPRB, 0.46372E+00_JPRB, 0.43965E+00_JPRB ,&
&  0.44650E+00_JPRB, 0.44221E+00_JPRB, 0.42254E+00_JPRB, 0.43619E+00_JPRB, 0.40701E+00_JPRB, 0.38843E+00_JPRB, 0.38432E+00_JPRB ,&
&  0.37047E+00_JPRB, 0.33156E+00_JPRB, 0.31610E+00_JPRB, 0.31253E+00_JPRB, 0.30579E+00_JPRB, 0.30043E+00_JPRB, 0.30170E+00_JPRB ,&
&  0.29514E+00_JPRB, 0.29749E+00_JPRB, 0.30856E+00_JPRB, 0.31193E+00_JPRB, 0.31669E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.33387E+00_JPRB, 0.33624E+00_JPRB, 0.33903E+00_JPRB, 0.34239E+00_JPRB, 0.35549E+00_JPRB, 0.36208E+00_JPRB, 0.36226E+00_JPRB ,&
&  0.37703E+00_JPRB, 0.37558E+00_JPRB, 0.37583E+00_JPRB, 0.37231E+00_JPRB, 0.36619E+00_JPRB, 0.33342E+00_JPRB, 0.28614E+00_JPRB ,&
&  0.27064E+00_JPRB, 0.27246E+00_JPRB, 0.27868E+00_JPRB, 0.28053E+00_JPRB, 0.27699E+00_JPRB, 0.27135E+00_JPRB, 0.26772E+00_JPRB ,&
&  0.26857E+00_JPRB, 0.26996E+00_JPRB, 0.27249E+00_JPRB, 0.27281E+00_JPRB, 0.27062E+00_JPRB, 0.26947E+00_JPRB, 0.26842E+00_JPRB ,&
&  0.26404E+00_JPRB, 0.26535E+00_JPRB, 0.26798E+00_JPRB, 0.26415E+00_JPRB, 0.27604E+00_JPRB, 0.28716E+00_JPRB, 0.29247E+00_JPRB ,&
&  0.31106E+00_JPRB, 0.33469E+00_JPRB, 0.35147E+00_JPRB, 0.36741E+00_JPRB, 0.37137E+00_JPRB, 0.37377E+00_JPRB, 0.38211E+00_JPRB ,&
&  0.39647E+00_JPRB, 0.41024E+00_JPRB, 0.42329E+00_JPRB, 0.43386E+00_JPRB, 0.43878E+00_JPRB, 0.43096E+00_JPRB, 0.44672E+00_JPRB ,&
&  0.45085E+00_JPRB, 0.47747E+00_JPRB, 0.53301E+00_JPRB, 0.52576E+00_JPRB, 0.61282E+00_JPRB, 0.66813E+00_JPRB, 0.64160E+00_JPRB ,&
&  0.66250E+00_JPRB, 0.62341E+00_JPRB, 0.68012E+00_JPRB, 0.71445E+00_JPRB, 0.74744E+00_JPRB, 0.75525E+00_JPRB, 0.77872E+00_JPRB ,&
&  0.84916E+00_JPRB, 0.90879E+00_JPRB, 0.77705E+00_JPRB, 0.71516E+00_JPRB, 0.66453E+00_JPRB, 0.65896E+00_JPRB, 0.62671E+00_JPRB ,&
&  0.62722E+00_JPRB, 0.63985E+00_JPRB, 0.62599E+00_JPRB, 0.60719E+00_JPRB, 0.57101E+00_JPRB, 0.53421E+00_JPRB, 0.49940E+00_JPRB ,&
&  0.47777E+00_JPRB, 0.43470E+00_JPRB, 0.41345E+00_JPRB, 0.41330E+00_JPRB, 0.41161E+00_JPRB, 0.38035E+00_JPRB, 0.38611E+00_JPRB ,&
&  0.39223E+00_JPRB, 0.36621E+00_JPRB, 0.34553E+00_JPRB, 0.32927E+00_JPRB, 0.32394E+00_JPRB, 0.31609E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.29179E+00_JPRB, 0.30739E+00_JPRB, 0.31844E+00_JPRB, 0.32370E+00_JPRB, 0.32970E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.34664E+00_JPRB, 0.34544E+00_JPRB, 0.34433E+00_JPRB, 0.35110E+00_JPRB, 0.36589E+00_JPRB, 0.37192E+00_JPRB, 0.38205E+00_JPRB ,&
&  0.38845E+00_JPRB, 0.38877E+00_JPRB, 0.39124E+00_JPRB, 0.38437E+00_JPRB, 0.38028E+00_JPRB, 0.37660E+00_JPRB, 0.34914E+00_JPRB ,&
&  0.31188E+00_JPRB, 0.30152E+00_JPRB, 0.29146E+00_JPRB, 0.28989E+00_JPRB, 0.28914E+00_JPRB, 0.28280E+00_JPRB, 0.27860E+00_JPRB ,&
&  0.28087E+00_JPRB, 0.27985E+00_JPRB, 0.27963E+00_JPRB, 0.27970E+00_JPRB, 0.27960E+00_JPRB, 0.28125E+00_JPRB, 0.28171E+00_JPRB ,&
&  0.27711E+00_JPRB, 0.28044E+00_JPRB, 0.28595E+00_JPRB, 0.28434E+00_JPRB, 0.30170E+00_JPRB, 0.32379E+00_JPRB, 0.34148E+00_JPRB ,&
&  0.36637E+00_JPRB, 0.38757E+00_JPRB, 0.40080E+00_JPRB, 0.41125E+00_JPRB, 0.41425E+00_JPRB, 0.42601E+00_JPRB, 0.44535E+00_JPRB ,&
&  0.46907E+00_JPRB, 0.48657E+00_JPRB, 0.50127E+00_JPRB, 0.51782E+00_JPRB, 0.51426E+00_JPRB, 0.52735E+00_JPRB, 0.57406E+00_JPRB ,&
&  0.63271E+00_JPRB, 0.70742E+00_JPRB, 0.73830E+00_JPRB, 0.76698E+00_JPRB, 0.82949E+00_JPRB, 0.81936E+00_JPRB, 0.74610E+00_JPRB ,&
&  0.73850E+00_JPRB, 0.68951E+00_JPRB, 0.90249E+00_JPRB, 0.90492E+00_JPRB, 0.87621E+00_JPRB, 0.91050E+00_JPRB, 0.88800E+00_JPRB ,&
&  0.89126E+00_JPRB, 0.85794E+00_JPRB, 0.80110E+00_JPRB, 0.77382E+00_JPRB, 0.75253E+00_JPRB, 0.78951E+00_JPRB, 0.74651E+00_JPRB ,&
&  0.75325E+00_JPRB, 0.74578E+00_JPRB, 0.64329E+00_JPRB, 0.56525E+00_JPRB, 0.49117E+00_JPRB, 0.51098E+00_JPRB, 0.49303E+00_JPRB ,&
&  0.47088E+00_JPRB, 0.41165E+00_JPRB, 0.42719E+00_JPRB, 0.43007E+00_JPRB, 0.43537E+00_JPRB, 0.44547E+00_JPRB, 0.44236E+00_JPRB ,&
&  0.43904E+00_JPRB, 0.43743E+00_JPRB, 0.39200E+00_JPRB, 0.36216E+00_JPRB, 0.34922E+00_JPRB, 0.33409E+00_JPRB, 0.31178E+00_JPRB ,&
&  0.30716E+00_JPRB, 0.32519E+00_JPRB, 0.33792E+00_JPRB, 0.34453E+00_JPRB, 0.34776E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.36461E+00_JPRB, 0.36143E+00_JPRB, 0.35741E+00_JPRB, 0.36618E+00_JPRB, 0.37769E+00_JPRB, 0.38211E+00_JPRB, 0.39181E+00_JPRB ,&
&  0.40182E+00_JPRB, 0.40362E+00_JPRB, 0.39899E+00_JPRB, 0.39324E+00_JPRB, 0.39007E+00_JPRB, 0.38777E+00_JPRB, 0.38865E+00_JPRB ,&
&  0.34688E+00_JPRB, 0.32477E+00_JPRB, 0.30068E+00_JPRB, 0.29901E+00_JPRB, 0.31738E+00_JPRB, 0.30559E+00_JPRB, 0.30280E+00_JPRB ,&
&  0.29806E+00_JPRB, 0.29773E+00_JPRB, 0.29228E+00_JPRB, 0.28879E+00_JPRB, 0.28792E+00_JPRB, 0.29087E+00_JPRB, 0.29664E+00_JPRB ,&
&  0.29712E+00_JPRB, 0.30949E+00_JPRB, 0.32399E+00_JPRB, 0.32586E+00_JPRB, 0.34789E+00_JPRB, 0.37117E+00_JPRB, 0.40162E+00_JPRB ,&
&  0.42981E+00_JPRB, 0.44011E+00_JPRB, 0.44626E+00_JPRB, 0.45973E+00_JPRB, 0.47541E+00_JPRB, 0.50369E+00_JPRB, 0.53656E+00_JPRB ,&
&  0.57117E+00_JPRB, 0.59822E+00_JPRB, 0.61648E+00_JPRB, 0.62719E+00_JPRB, 0.61868E+00_JPRB, 0.65518E+00_JPRB, 0.73166E+00_JPRB ,&
&  0.85902E+00_JPRB, 0.90906E+00_JPRB, 0.92733E+00_JPRB, 0.10853E+01_JPRB, 0.11672E+01_JPRB, 0.11087E+01_JPRB, 0.95471E+00_JPRB ,&
&  0.86149E+00_JPRB, 0.78101E+00_JPRB, 0.85090E+00_JPRB, 0.89033E+00_JPRB, 0.85946E+00_JPRB, 0.93476E+00_JPRB, 0.89595E+00_JPRB ,&
&  0.81928E+00_JPRB, 0.75807E+00_JPRB, 0.73079E+00_JPRB, 0.76946E+00_JPRB, 0.76354E+00_JPRB, 0.81378E+00_JPRB, 0.80753E+00_JPRB ,&
&  0.80580E+00_JPRB, 0.69202E+00_JPRB, 0.52404E+00_JPRB, 0.48362E+00_JPRB, 0.39456E+00_JPRB, 0.40174E+00_JPRB, 0.48142E+00_JPRB ,&
&  0.48249E+00_JPRB, 0.48420E+00_JPRB, 0.47701E+00_JPRB, 0.47749E+00_JPRB, 0.48014E+00_JPRB, 0.54889E+00_JPRB, 0.56506E+00_JPRB ,&
&  0.49993E+00_JPRB, 0.49570E+00_JPRB, 0.47106E+00_JPRB, 0.44117E+00_JPRB, 0.41440E+00_JPRB, 0.38113E+00_JPRB, 0.35677E+00_JPRB ,&
&  0.35375E+00_JPRB, 0.35565E+00_JPRB, 0.36276E+00_JPRB, 0.36446E+00_JPRB, 0.36395E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.39017E+00_JPRB, 0.38757E+00_JPRB, 0.38108E+00_JPRB, 0.38462E+00_JPRB, 0.39109E+00_JPRB, 0.39711E+00_JPRB, 0.40694E+00_JPRB ,&
&  0.41762E+00_JPRB, 0.41620E+00_JPRB, 0.40767E+00_JPRB, 0.40441E+00_JPRB, 0.40161E+00_JPRB, 0.39648E+00_JPRB, 0.39658E+00_JPRB ,&
&  0.40445E+00_JPRB, 0.38208E+00_JPRB, 0.34325E+00_JPRB, 0.31246E+00_JPRB, 0.32248E+00_JPRB, 0.32098E+00_JPRB, 0.32975E+00_JPRB ,&
&  0.32429E+00_JPRB, 0.33363E+00_JPRB, 0.32344E+00_JPRB, 0.31742E+00_JPRB, 0.31349E+00_JPRB, 0.31832E+00_JPRB, 0.33260E+00_JPRB ,&
&  0.35318E+00_JPRB, 0.38750E+00_JPRB, 0.41981E+00_JPRB, 0.42611E+00_JPRB, 0.45207E+00_JPRB, 0.44209E+00_JPRB, 0.47249E+00_JPRB ,&
&  0.49595E+00_JPRB, 0.50072E+00_JPRB, 0.50697E+00_JPRB, 0.52949E+00_JPRB, 0.56498E+00_JPRB, 0.60666E+00_JPRB, 0.65931E+00_JPRB ,&
&  0.70866E+00_JPRB, 0.72739E+00_JPRB, 0.70661E+00_JPRB, 0.69890E+00_JPRB, 0.73374E+00_JPRB, 0.80904E+00_JPRB, 0.82323E+00_JPRB ,&
&  0.89531E+00_JPRB, 0.92483E+00_JPRB, 0.89650E+00_JPRB, 0.11114E+01_JPRB, 0.12721E+01_JPRB, 0.12596E+01_JPRB, 0.11429E+01_JPRB ,&
&  0.10465E+01_JPRB, 0.95986E+00_JPRB, 0.10021E+01_JPRB, 0.99837E+00_JPRB, 0.97276E+00_JPRB, 0.95333E+00_JPRB, 0.88760E+00_JPRB ,&
&  0.78758E+00_JPRB, 0.74053E+00_JPRB, 0.75680E+00_JPRB, 0.77175E+00_JPRB, 0.78782E+00_JPRB, 0.76027E+00_JPRB, 0.77881E+00_JPRB ,&
&  0.81211E+00_JPRB, 0.47837E+00_JPRB, 0.39425E+00_JPRB, 0.44417E+00_JPRB, 0.38736E+00_JPRB, 0.39481E+00_JPRB, 0.44001E+00_JPRB ,&
&  0.43704E+00_JPRB, 0.44072E+00_JPRB, 0.48609E+00_JPRB, 0.51543E+00_JPRB, 0.56805E+00_JPRB, 0.66272E+00_JPRB, 0.65876E+00_JPRB ,&
&  0.62592E+00_JPRB, 0.57794E+00_JPRB, 0.58023E+00_JPRB, 0.55528E+00_JPRB, 0.51267E+00_JPRB, 0.45676E+00_JPRB, 0.41599E+00_JPRB ,&
&  0.39833E+00_JPRB, 0.39553E+00_JPRB, 0.39501E+00_JPRB, 0.39212E+00_JPRB, 0.38790E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.43108E+00_JPRB, 0.42706E+00_JPRB, 0.41132E+00_JPRB, 0.40662E+00_JPRB, 0.41487E+00_JPRB, 0.42688E+00_JPRB, 0.43561E+00_JPRB ,&
&  0.43686E+00_JPRB, 0.43175E+00_JPRB, 0.42517E+00_JPRB, 0.42275E+00_JPRB, 0.41612E+00_JPRB, 0.40399E+00_JPRB, 0.39815E+00_JPRB ,&
&  0.39664E+00_JPRB, 0.39703E+00_JPRB, 0.35861E+00_JPRB, 0.34238E+00_JPRB, 0.32291E+00_JPRB, 0.33101E+00_JPRB, 0.34305E+00_JPRB ,&
&  0.35812E+00_JPRB, 0.35481E+00_JPRB, 0.34760E+00_JPRB, 0.34839E+00_JPRB, 0.34970E+00_JPRB, 0.36234E+00_JPRB, 0.41198E+00_JPRB ,&
&  0.47084E+00_JPRB, 0.53327E+00_JPRB, 0.54159E+00_JPRB, 0.56486E+00_JPRB, 0.55088E+00_JPRB, 0.52309E+00_JPRB, 0.56709E+00_JPRB ,&
&  0.58047E+00_JPRB, 0.58114E+00_JPRB, 0.58645E+00_JPRB, 0.61555E+00_JPRB, 0.65599E+00_JPRB, 0.67869E+00_JPRB, 0.71459E+00_JPRB ,&
&  0.73434E+00_JPRB, 0.73846E+00_JPRB, 0.72810E+00_JPRB, 0.70694E+00_JPRB, 0.75672E+00_JPRB, 0.78652E+00_JPRB, 0.74101E+00_JPRB ,&
&  0.77166E+00_JPRB, 0.69618E+00_JPRB, 0.57869E+00_JPRB, 0.82693E+00_JPRB, 0.12605E+01_JPRB, 0.12971E+01_JPRB, 0.10664E+01_JPRB ,&
&  0.10799E+01_JPRB, 0.10608E+01_JPRB, 0.11384E+01_JPRB, 0.11870E+01_JPRB, 0.11272E+01_JPRB, 0.10138E+01_JPRB, 0.91739E+00_JPRB ,&
&  0.82902E+00_JPRB, 0.80711E+00_JPRB, 0.84295E+00_JPRB, 0.99154E+00_JPRB, 0.10099E+01_JPRB, 0.98922E+00_JPRB, 0.93524E+00_JPRB ,&
&  0.77168E+00_JPRB, 0.64549E+00_JPRB, 0.46946E+00_JPRB, 0.47793E+00_JPRB, 0.39048E+00_JPRB, 0.39226E+00_JPRB, 0.43402E+00_JPRB ,&
&  0.42741E+00_JPRB, 0.45587E+00_JPRB, 0.50566E+00_JPRB, 0.54812E+00_JPRB, 0.71328E+00_JPRB, 0.80962E+00_JPRB, 0.83614E+00_JPRB ,&
&  0.79969E+00_JPRB, 0.70202E+00_JPRB, 0.70585E+00_JPRB, 0.65300E+00_JPRB, 0.61524E+00_JPRB, 0.55027E+00_JPRB, 0.49582E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.45328E+00_JPRB, 0.44499E+00_JPRB, 0.43738E+00_JPRB, 0.42996E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.45170E+00_JPRB, 0.43625E+00_JPRB, 0.40852E+00_JPRB, 0.40663E+00_JPRB, 0.41837E+00_JPRB, 0.44013E+00_JPRB, 0.45542E+00_JPRB ,&
&  0.45760E+00_JPRB, 0.45227E+00_JPRB, 0.44263E+00_JPRB, 0.43671E+00_JPRB, 0.42435E+00_JPRB, 0.40603E+00_JPRB, 0.39907E+00_JPRB ,&
&  0.39290E+00_JPRB, 0.38134E+00_JPRB, 0.35049E+00_JPRB, 0.33351E+00_JPRB, 0.31964E+00_JPRB, 0.32350E+00_JPRB, 0.33837E+00_JPRB ,&
&  0.35259E+00_JPRB, 0.36372E+00_JPRB, 0.37870E+00_JPRB, 0.39010E+00_JPRB, 0.39644E+00_JPRB, 0.43993E+00_JPRB, 0.53363E+00_JPRB ,&
&  0.70026E+00_JPRB, 0.65751E+00_JPRB, 0.67226E+00_JPRB, 0.63731E+00_JPRB, 0.62474E+00_JPRB, 0.62377E+00_JPRB, 0.61545E+00_JPRB ,&
&  0.58965E+00_JPRB, 0.57705E+00_JPRB, 0.58958E+00_JPRB, 0.61483E+00_JPRB, 0.66924E+00_JPRB, 0.71343E+00_JPRB, 0.78217E+00_JPRB ,&
&  0.80673E+00_JPRB, 0.78075E+00_JPRB, 0.72947E+00_JPRB, 0.72667E+00_JPRB, 0.78433E+00_JPRB, 0.78414E+00_JPRB, 0.75771E+00_JPRB ,&
&  0.65234E+00_JPRB, 0.58392E+00_JPRB, 0.67711E+00_JPRB, 0.82892E+00_JPRB, 0.11014E+01_JPRB, 0.12839E+01_JPRB, 0.11910E+01_JPRB ,&
&  0.11813E+01_JPRB, 0.11549E+01_JPRB, 0.11612E+01_JPRB, 0.93095E+00_JPRB, 0.10079E+01_JPRB, 0.95221E+00_JPRB, 0.89720E+00_JPRB ,&
&  0.85927E+00_JPRB, 0.88162E+00_JPRB, 0.98075E+00_JPRB, 0.11612E+01_JPRB, 0.12805E+01_JPRB, 0.13160E+01_JPRB, 0.96998E+00_JPRB ,&
&  0.72996E+00_JPRB, 0.76887E+00_JPRB, 0.69135E+00_JPRB, 0.57998E+00_JPRB, 0.49483E+00_JPRB, 0.50236E+00_JPRB, 0.49859E+00_JPRB ,&
&  0.49909E+00_JPRB, 0.50850E+00_JPRB, 0.56895E+00_JPRB, 0.69570E+00_JPRB, 0.92040E+00_JPRB, 0.10678E+01_JPRB, 0.10403E+01_JPRB ,&
&  0.91790E+00_JPRB, 0.91666E+00_JPRB, 0.83859E+00_JPRB, 0.73878E+00_JPRB, 0.69844E+00_JPRB, 0.64332E+00_JPRB, 0.58646E+00_JPRB ,&
&  0.54277E+00_JPRB, 0.52430E+00_JPRB, 0.50985E+00_JPRB, 0.49200E+00_JPRB, 0.46689E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.47436E+00_JPRB, 0.45728E+00_JPRB, 0.43146E+00_JPRB, 0.43064E+00_JPRB, 0.44873E+00_JPRB, 0.47004E+00_JPRB, 0.47816E+00_JPRB ,&
&  0.46905E+00_JPRB, 0.46001E+00_JPRB, 0.44865E+00_JPRB, 0.44806E+00_JPRB, 0.44090E+00_JPRB, 0.41797E+00_JPRB, 0.40397E+00_JPRB ,&
&  0.40125E+00_JPRB, 0.37432E+00_JPRB, 0.34442E+00_JPRB, 0.34489E+00_JPRB, 0.34258E+00_JPRB, 0.32461E+00_JPRB, 0.33314E+00_JPRB ,&
&  0.34954E+00_JPRB, 0.37062E+00_JPRB, 0.39300E+00_JPRB, 0.42968E+00_JPRB, 0.48874E+00_JPRB, 0.58865E+00_JPRB, 0.71575E+00_JPRB ,&
&  0.74112E+00_JPRB, 0.74285E+00_JPRB, 0.76329E+00_JPRB, 0.72660E+00_JPRB, 0.65686E+00_JPRB, 0.62091E+00_JPRB, 0.60740E+00_JPRB ,&
&  0.58634E+00_JPRB, 0.57774E+00_JPRB, 0.61096E+00_JPRB, 0.67266E+00_JPRB, 0.75562E+00_JPRB, 0.80467E+00_JPRB, 0.84618E+00_JPRB ,&
&  0.80614E+00_JPRB, 0.79658E+00_JPRB, 0.78766E+00_JPRB, 0.86203E+00_JPRB, 0.66279E+00_JPRB, 0.63930E+00_JPRB, 0.62158E+00_JPRB ,&
&  0.71907E+00_JPRB, 0.69296E+00_JPRB, 0.69123E+00_JPRB, 0.85150E+00_JPRB, 0.82803E+00_JPRB, 0.97812E+00_JPRB, 0.11648E+01_JPRB ,&
&  0.12633E+01_JPRB, 0.12675E+01_JPRB, 0.12439E+01_JPRB, 0.77310E+00_JPRB, 0.70279E+00_JPRB, 0.90371E+00_JPRB, 0.96403E+00_JPRB ,&
&  0.97409E+00_JPRB, 0.10858E+01_JPRB, 0.12388E+01_JPRB, 0.14838E+01_JPRB, 0.11625E+01_JPRB, 0.96345E+00_JPRB, 0.72798E+00_JPRB ,&
&  0.70415E+00_JPRB, 0.65198E+00_JPRB, 0.75264E+00_JPRB, 0.68815E+00_JPRB, 0.68151E+00_JPRB, 0.65342E+00_JPRB, 0.60435E+00_JPRB ,&
&  0.60735E+00_JPRB, 0.63103E+00_JPRB, 0.73402E+00_JPRB, 0.97670E+00_JPRB, 0.12482E+01_JPRB, 0.12446E+01_JPRB, 0.13005E+01_JPRB ,&
&  0.12253E+01_JPRB, 0.10732E+01_JPRB, 0.90695E+00_JPRB, 0.82359E+00_JPRB, 0.77124E+00_JPRB, 0.72974E+00_JPRB, 0.68339E+00_JPRB ,&
&  0.63045E+00_JPRB, 0.58961E+00_JPRB, 0.55782E+00_JPRB, 0.52620E+00_JPRB, 0.49468E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.51118E+00_JPRB, 0.49294E+00_JPRB, 0.46382E+00_JPRB, 0.46703E+00_JPRB, 0.49189E+00_JPRB, 0.50194E+00_JPRB, 0.49239E+00_JPRB ,&
&  0.48436E+00_JPRB, 0.46826E+00_JPRB, 0.45950E+00_JPRB, 0.45084E+00_JPRB, 0.44073E+00_JPRB, 0.43843E+00_JPRB, 0.42293E+00_JPRB ,&
&  0.42189E+00_JPRB, 0.40741E+00_JPRB, 0.36876E+00_JPRB, 0.34907E+00_JPRB, 0.35705E+00_JPRB, 0.33827E+00_JPRB, 0.32858E+00_JPRB ,&
&  0.35824E+00_JPRB, 0.39298E+00_JPRB, 0.43839E+00_JPRB, 0.50933E+00_JPRB, 0.61285E+00_JPRB, 0.79476E+00_JPRB, 0.89235E+00_JPRB ,&
&  0.84448E+00_JPRB, 0.80012E+00_JPRB, 0.71301E+00_JPRB, 0.63790E+00_JPRB, 0.58263E+00_JPRB, 0.55976E+00_JPRB, 0.55209E+00_JPRB ,&
&  0.55293E+00_JPRB, 0.57071E+00_JPRB, 0.62821E+00_JPRB, 0.70816E+00_JPRB, 0.78229E+00_JPRB, 0.80134E+00_JPRB, 0.79912E+00_JPRB ,&
&  0.82696E+00_JPRB, 0.89510E+00_JPRB, 0.91245E+00_JPRB, 0.85127E+00_JPRB, 0.76791E+00_JPRB, 0.65995E+00_JPRB, 0.66963E+00_JPRB ,&
&  0.71528E+00_JPRB, 0.66202E+00_JPRB, 0.69599E+00_JPRB, 0.87242E+00_JPRB, 0.96986E+00_JPRB, 0.10050E+01_JPRB, 0.12070E+01_JPRB ,&
&  0.98616E+00_JPRB, 0.79006E+00_JPRB, 0.67442E+00_JPRB, 0.57279E+00_JPRB, 0.81539E+00_JPRB, 0.11236E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.12976E+01_JPRB, 0.14168E+01_JPRB, 0.15799E+01_JPRB, 0.14584E+01_JPRB, 0.10875E+01_JPRB, 0.86120E+00_JPRB, 0.10276E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.10851E+01_JPRB, 0.10346E+01_JPRB, 0.88786E+00_JPRB, 0.72834E+00_JPRB, 0.81535E+00_JPRB, 0.80309E+00_JPRB ,&
&  0.80779E+00_JPRB, 0.82723E+00_JPRB, 0.10488E+01_JPRB, 0.13612E+01_JPRB, 0.15254E+01_JPRB, 0.16919E+01_JPRB, 0.16970E+01_JPRB ,&
&  0.14174E+01_JPRB, 0.12792E+01_JPRB, 0.10798E+01_JPRB, 0.92825E+00_JPRB, 0.84590E+00_JPRB, 0.79326E+00_JPRB, 0.75204E+00_JPRB ,&
&  0.69824E+00_JPRB, 0.65812E+00_JPRB, 0.61865E+00_JPRB, 0.57064E+00_JPRB, 0.53286E+00_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.56105E+00_JPRB, 0.53459E+00_JPRB, 0.50012E+00_JPRB, 0.50424E+00_JPRB, 0.53433E+00_JPRB, 0.52621E+00_JPRB, 0.50644E+00_JPRB ,&
&  0.50319E+00_JPRB, 0.48454E+00_JPRB, 0.47209E+00_JPRB, 0.45630E+00_JPRB, 0.44190E+00_JPRB, 0.43490E+00_JPRB, 0.43517E+00_JPRB ,&
&  0.45059E+00_JPRB, 0.47151E+00_JPRB, 0.42305E+00_JPRB, 0.38996E+00_JPRB, 0.37778E+00_JPRB, 0.36377E+00_JPRB, 0.33636E+00_JPRB ,&
&  0.37961E+00_JPRB, 0.44007E+00_JPRB, 0.50845E+00_JPRB, 0.61619E+00_JPRB, 0.78645E+00_JPRB, 0.84704E+00_JPRB, 0.84553E+00_JPRB ,&
&  0.81268E+00_JPRB, 0.74374E+00_JPRB, 0.66252E+00_JPRB, 0.59400E+00_JPRB, 0.55946E+00_JPRB, 0.55037E+00_JPRB, 0.54872E+00_JPRB ,&
&  0.56591E+00_JPRB, 0.62144E+00_JPRB, 0.68605E+00_JPRB, 0.75564E+00_JPRB, 0.81920E+00_JPRB, 0.85945E+00_JPRB, 0.88373E+00_JPRB ,&
&  0.93374E+00_JPRB, 0.99124E+00_JPRB, 0.10079E+01_JPRB, 0.95990E+00_JPRB, 0.92223E+00_JPRB, 0.80509E+00_JPRB, 0.93749E+00_JPRB ,&
&  0.88537E+00_JPRB, 0.91405E+00_JPRB, 0.88845E+00_JPRB, 0.11613E+01_JPRB, 0.10512E+01_JPRB, 0.10116E+01_JPRB, 0.11129E+01_JPRB ,&
&  0.86553E+00_JPRB, 0.80398E+00_JPRB, 0.91522E+00_JPRB, 0.10466E+01_JPRB, 0.11428E+01_JPRB, 0.12540E+01_JPRB, 0.17703E+01_JPRB ,&
&  0.14961E+01_JPRB, 0.16975E+01_JPRB, 0.18393E+01_JPRB, 0.17738E+01_JPRB, 0.82870E+00_JPRB, 0.62793E+00_JPRB, 0.95638E+00_JPRB ,&
&  0.10410E+01_JPRB, 0.72176E+00_JPRB, 0.55201E+00_JPRB, 0.67647E+00_JPRB, 0.53402E+00_JPRB, 0.62442E+00_JPRB, 0.86261E+00_JPRB ,&
&  0.97204E+00_JPRB, 0.11191E+01_JPRB, 0.15350E+01_JPRB, 0.16782E+01_JPRB, 0.16294E+01_JPRB, 0.20551E+01_JPRB, 0.17941E+01_JPRB ,&
&  0.15721E+01_JPRB, 0.13264E+01_JPRB, 0.12095E+01_JPRB, 0.10516E+01_JPRB, 0.94303E+00_JPRB, 0.88108E+00_JPRB, 0.83022E+00_JPRB ,&
&  0.77320E+00_JPRB, 0.72414E+00_JPRB, 0.67889E+00_JPRB, 0.63110E+00_JPRB, 0.59205E+00_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.59851E+00_JPRB, 0.57385E+00_JPRB, 0.54632E+00_JPRB, 0.54864E+00_JPRB, 0.57005E+00_JPRB, 0.54045E+00_JPRB, 0.52711E+00_JPRB ,&
&  0.50835E+00_JPRB, 0.49519E+00_JPRB, 0.47608E+00_JPRB, 0.45525E+00_JPRB, 0.43883E+00_JPRB, 0.43972E+00_JPRB, 0.44769E+00_JPRB ,&
&  0.46048E+00_JPRB, 0.49915E+00_JPRB, 0.52478E+00_JPRB, 0.49089E+00_JPRB, 0.46238E+00_JPRB, 0.42315E+00_JPRB, 0.39738E+00_JPRB ,&
&  0.44128E+00_JPRB, 0.52851E+00_JPRB, 0.60308E+00_JPRB, 0.67196E+00_JPRB, 0.74238E+00_JPRB, 0.73943E+00_JPRB, 0.76259E+00_JPRB ,&
&  0.74904E+00_JPRB, 0.68463E+00_JPRB, 0.62623E+00_JPRB, 0.58108E+00_JPRB, 0.56315E+00_JPRB, 0.56009E+00_JPRB, 0.58529E+00_JPRB ,&
&  0.62781E+00_JPRB, 0.67453E+00_JPRB, 0.71586E+00_JPRB, 0.79595E+00_JPRB, 0.89015E+00_JPRB, 0.93891E+00_JPRB, 0.98216E+00_JPRB ,&
&  0.10204E+01_JPRB, 0.10381E+01_JPRB, 0.10431E+01_JPRB, 0.10521E+01_JPRB, 0.11232E+01_JPRB, 0.10192E+01_JPRB, 0.87294E+00_JPRB ,&
&  0.80046E+00_JPRB, 0.91679E+00_JPRB, 0.13161E+01_JPRB, 0.13409E+01_JPRB, 0.10836E+01_JPRB, 0.10850E+01_JPRB, 0.10514E+01_JPRB ,&
&  0.10243E+01_JPRB, 0.11810E+01_JPRB, 0.13171E+01_JPRB, 0.14508E+01_JPRB, 0.15329E+01_JPRB, 0.12731E+01_JPRB, 0.17453E+01_JPRB ,&
&  0.18478E+01_JPRB, 0.18017E+01_JPRB, 0.13522E+01_JPRB, 0.94251E+00_JPRB, 0.94050E+00_JPRB, 0.70664E+00_JPRB, 0.50824E+00_JPRB ,&
&  0.47942E+00_JPRB, 0.45883E+00_JPRB, 0.42316E+00_JPRB, 0.44224E+00_JPRB, 0.44661E+00_JPRB, 0.58616E+00_JPRB, 0.87041E+00_JPRB ,&
&  0.12423E+01_JPRB, 0.15493E+01_JPRB, 0.17607E+01_JPRB, 0.18066E+01_JPRB, 0.17237E+01_JPRB, 0.20160E+01_JPRB, 0.19116E+01_JPRB ,&
&  0.13875E+01_JPRB, 0.12521E+01_JPRB, 0.11185E+01_JPRB, 0.10644E+01_JPRB, 0.10120E+01_JPRB, 0.96191E+00_JPRB, 0.89724E+00_JPRB ,&
&  0.82839E+00_JPRB, 0.77841E+00_JPRB, 0.73458E+00_JPRB, 0.67874E+00_JPRB, 0.63320E+00_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.63296E+00_JPRB, 0.61262E+00_JPRB, 0.60988E+00_JPRB, 0.60372E+00_JPRB, 0.60062E+00_JPRB, 0.55836E+00_JPRB, 0.54626E+00_JPRB ,&
&  0.51115E+00_JPRB, 0.49539E+00_JPRB, 0.46608E+00_JPRB, 0.44907E+00_JPRB, 0.43870E+00_JPRB, 0.44849E+00_JPRB, 0.46847E+00_JPRB ,&
&  0.49000E+00_JPRB, 0.55070E+00_JPRB, 0.64186E+00_JPRB, 0.66803E+00_JPRB, 0.67432E+00_JPRB, 0.61790E+00_JPRB, 0.57476E+00_JPRB ,&
&  0.57448E+00_JPRB, 0.67580E+00_JPRB, 0.70349E+00_JPRB, 0.71903E+00_JPRB, 0.74382E+00_JPRB, 0.73454E+00_JPRB, 0.71353E+00_JPRB ,&
&  0.70049E+00_JPRB, 0.65428E+00_JPRB, 0.59987E+00_JPRB, 0.57566E+00_JPRB, 0.57238E+00_JPRB, 0.58947E+00_JPRB, 0.62414E+00_JPRB ,&
&  0.68461E+00_JPRB, 0.70855E+00_JPRB, 0.75641E+00_JPRB, 0.86132E+00_JPRB, 0.94968E+00_JPRB, 0.98763E+00_JPRB, 0.10422E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.10898E+01_JPRB, 0.10783E+01_JPRB, 0.10506E+01_JPRB, 0.90708E+00_JPRB, 0.76983E+00_JPRB, 0.92488E+00_JPRB ,&
&  0.10650E+01_JPRB, 0.13293E+01_JPRB, 0.15069E+01_JPRB, 0.16597E+01_JPRB, 0.12304E+01_JPRB, 0.11203E+01_JPRB, 0.11108E+01_JPRB ,&
&  0.12150E+01_JPRB, 0.18563E+01_JPRB, 0.16686E+01_JPRB, 0.14969E+01_JPRB, 0.17742E+01_JPRB, 0.14752E+01_JPRB, 0.15593E+01_JPRB ,&
&  0.18711E+01_JPRB, 0.17542E+01_JPRB, 0.13512E+01_JPRB, 0.10518E+01_JPRB, 0.12712E+01_JPRB, 0.11965E+01_JPRB, 0.63879E+00_JPRB ,&
&  0.52540E+00_JPRB, 0.48971E+00_JPRB, 0.46087E+00_JPRB, 0.44054E+00_JPRB, 0.44940E+00_JPRB, 0.62625E+00_JPRB, 0.11099E+01_JPRB ,&
&  0.14503E+01_JPRB, 0.18821E+01_JPRB, 0.19487E+01_JPRB, 0.19013E+01_JPRB, 0.17522E+01_JPRB, 0.18770E+01_JPRB, 0.17977E+01_JPRB ,&
&  0.14609E+01_JPRB, 0.13038E+01_JPRB, 0.11678E+01_JPRB, 0.11012E+01_JPRB, 0.10479E+01_JPRB, 0.99851E+00_JPRB, 0.94042E+00_JPRB ,&
&  0.87515E+00_JPRB, 0.82073E+00_JPRB, 0.77070E+00_JPRB, 0.71140E+00_JPRB, 0.66365E+00_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.66066E+00_JPRB, 0.65028E+00_JPRB, 0.66126E+00_JPRB, 0.66258E+00_JPRB, 0.63882E+00_JPRB, 0.60149E+00_JPRB, 0.55824E+00_JPRB ,&
&  0.52065E+00_JPRB, 0.48986E+00_JPRB, 0.46657E+00_JPRB, 0.45501E+00_JPRB, 0.45270E+00_JPRB, 0.46730E+00_JPRB, 0.48892E+00_JPRB ,&
&  0.53535E+00_JPRB, 0.61801E+00_JPRB, 0.71606E+00_JPRB, 0.87518E+00_JPRB, 0.88420E+00_JPRB, 0.80419E+00_JPRB, 0.83640E+00_JPRB ,&
&  0.83828E+00_JPRB, 0.88520E+00_JPRB, 0.87294E+00_JPRB, 0.81836E+00_JPRB, 0.77718E+00_JPRB, 0.75802E+00_JPRB, 0.73044E+00_JPRB ,&
&  0.67601E+00_JPRB, 0.64708E+00_JPRB, 0.60389E+00_JPRB, 0.60410E+00_JPRB, 0.61577E+00_JPRB, 0.63975E+00_JPRB, 0.68531E+00_JPRB ,&
&  0.70525E+00_JPRB, 0.71111E+00_JPRB, 0.79411E+00_JPRB, 0.90031E+00_JPRB, 0.97398E+00_JPRB, 0.10376E+01_JPRB, 0.11103E+01_JPRB ,&
&  0.11316E+01_JPRB, 0.11288E+01_JPRB, 0.11024E+01_JPRB, 0.10640E+01_JPRB, 0.87808E+00_JPRB, 0.10028E+01_JPRB, 0.10484E+01_JPRB ,&
&  0.11676E+01_JPRB, 0.13257E+01_JPRB, 0.13945E+01_JPRB, 0.15332E+01_JPRB, 0.12999E+01_JPRB, 0.11602E+01_JPRB, 0.11179E+01_JPRB ,&
&  0.14393E+01_JPRB, 0.15450E+01_JPRB, 0.13821E+01_JPRB, 0.14556E+01_JPRB, 0.17399E+01_JPRB, 0.21490E+01_JPRB, 0.14217E+01_JPRB ,&
&  0.12722E+01_JPRB, 0.16059E+01_JPRB, 0.13498E+01_JPRB, 0.11803E+01_JPRB, 0.13559E+01_JPRB, 0.14975E+01_JPRB, 0.11409E+01_JPRB ,&
&  0.63529E+00_JPRB, 0.52785E+00_JPRB, 0.53998E+00_JPRB, 0.57122E+00_JPRB, 0.56635E+00_JPRB, 0.79000E+00_JPRB, 0.18854E+01_JPRB ,&
&  0.19943E+01_JPRB, 0.22309E+01_JPRB, 0.21036E+01_JPRB, 0.18896E+01_JPRB, 0.17584E+01_JPRB, 0.17750E+01_JPRB, 0.16949E+01_JPRB ,&
&  0.14411E+01_JPRB, 0.12989E+01_JPRB, 0.11832E+01_JPRB, 0.11049E+01_JPRB, 0.10376E+01_JPRB, 0.97447E+00_JPRB, 0.91443E+00_JPRB ,&
&  0.86816E+00_JPRB, 0.81912E+00_JPRB, 0.77509E+00_JPRB, 0.72520E+00_JPRB, 0.68317E+00_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.67509E+00_JPRB, 0.67196E+00_JPRB, 0.68380E+00_JPRB, 0.68607E+00_JPRB, 0.68580E+00_JPRB, 0.67729E+00_JPRB, 0.61630E+00_JPRB ,&
&  0.54603E+00_JPRB, 0.49752E+00_JPRB, 0.47579E+00_JPRB, 0.47047E+00_JPRB, 0.47759E+00_JPRB, 0.49725E+00_JPRB, 0.51975E+00_JPRB ,&
&  0.59715E+00_JPRB, 0.67538E+00_JPRB, 0.80789E+00_JPRB, 0.98061E+00_JPRB, 0.10556E+01_JPRB, 0.94724E+00_JPRB, 0.97025E+00_JPRB ,&
&  0.11142E+01_JPRB, 0.10903E+01_JPRB, 0.10006E+01_JPRB, 0.95355E+00_JPRB, 0.91237E+00_JPRB, 0.87126E+00_JPRB, 0.79965E+00_JPRB ,&
&  0.72967E+00_JPRB, 0.72612E+00_JPRB, 0.68139E+00_JPRB, 0.67862E+00_JPRB, 0.68222E+00_JPRB, 0.72010E+00_JPRB, 0.71311E+00_JPRB ,&
&  0.69939E+00_JPRB, 0.74919E+00_JPRB, 0.83677E+00_JPRB, 0.91973E+00_JPRB, 0.99920E+00_JPRB, 0.10723E+01_JPRB, 0.11575E+01_JPRB ,&
&  0.11980E+01_JPRB, 0.11806E+01_JPRB, 0.11322E+01_JPRB, 0.10434E+01_JPRB, 0.10310E+01_JPRB, 0.10522E+01_JPRB, 0.11350E+01_JPRB ,&
&  0.11827E+01_JPRB, 0.12819E+01_JPRB, 0.13331E+01_JPRB, 0.13442E+01_JPRB, 0.12327E+01_JPRB, 0.11620E+01_JPRB, 0.11423E+01_JPRB ,&
&  0.13575E+01_JPRB, 0.13897E+01_JPRB, 0.12394E+01_JPRB, 0.14863E+01_JPRB, 0.18286E+01_JPRB, 0.20910E+01_JPRB, 0.19369E+01_JPRB ,&
&  0.16512E+01_JPRB, 0.13778E+01_JPRB, 0.12605E+01_JPRB, 0.13628E+01_JPRB, 0.14587E+01_JPRB, 0.15058E+01_JPRB, 0.17594E+01_JPRB ,&
&  0.17649E+01_JPRB, 0.11670E+01_JPRB, 0.10166E+01_JPRB, 0.12524E+01_JPRB, 0.10939E+01_JPRB, 0.11423E+01_JPRB, 0.15899E+01_JPRB ,&
&  0.19903E+01_JPRB, 0.22425E+01_JPRB, 0.21261E+01_JPRB, 0.18807E+01_JPRB, 0.18118E+01_JPRB, 0.17313E+01_JPRB, 0.16355E+01_JPRB ,&
&  0.13973E+01_JPRB, 0.12267E+01_JPRB, 0.11141E+01_JPRB, 0.10359E+01_JPRB, 0.96244E+00_JPRB, 0.90076E+00_JPRB, 0.86286E+00_JPRB ,&
&  0.83001E+00_JPRB, 0.79254E+00_JPRB, 0.75906E+00_JPRB, 0.72027E+00_JPRB, 0.68750E+00_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.68365E+00_JPRB, 0.68829E+00_JPRB, 0.69796E+00_JPRB, 0.71129E+00_JPRB, 0.76265E+00_JPRB, 0.77457E+00_JPRB, 0.71274E+00_JPRB ,&
&  0.64034E+00_JPRB, 0.52591E+00_JPRB, 0.50465E+00_JPRB, 0.50627E+00_JPRB, 0.51884E+00_JPRB, 0.54248E+00_JPRB, 0.59481E+00_JPRB ,&
&  0.67313E+00_JPRB, 0.75416E+00_JPRB, 0.92267E+00_JPRB, 0.11148E+01_JPRB, 0.13079E+01_JPRB, 0.14925E+01_JPRB, 0.10454E+01_JPRB ,&
&  0.10178E+01_JPRB, 0.10905E+01_JPRB, 0.10456E+01_JPRB, 0.10492E+01_JPRB, 0.10259E+01_JPRB, 0.10037E+01_JPRB, 0.88957E+00_JPRB ,&
&  0.81803E+00_JPRB, 0.81289E+00_JPRB, 0.76851E+00_JPRB, 0.73595E+00_JPRB, 0.75454E+00_JPRB, 0.75991E+00_JPRB, 0.71470E+00_JPRB ,&
&  0.73576E+00_JPRB, 0.79154E+00_JPRB, 0.86733E+00_JPRB, 0.94292E+00_JPRB, 0.10243E+01_JPRB, 0.11129E+01_JPRB, 0.11718E+01_JPRB ,&
&  0.12339E+01_JPRB, 0.12350E+01_JPRB, 0.11866E+01_JPRB, 0.10808E+01_JPRB, 0.10809E+01_JPRB, 0.10922E+01_JPRB, 0.11085E+01_JPRB ,&
&  0.10514E+01_JPRB, 0.10164E+01_JPRB, 0.11550E+01_JPRB, 0.12242E+01_JPRB, 0.11670E+01_JPRB, 0.10789E+01_JPRB, 0.10760E+01_JPRB ,&
&  0.13067E+01_JPRB, 0.14708E+01_JPRB, 0.16229E+01_JPRB, 0.17062E+01_JPRB, 0.19419E+01_JPRB, 0.21011E+01_JPRB, 0.21559E+01_JPRB ,&
&  0.21516E+01_JPRB, 0.18444E+01_JPRB, 0.16551E+01_JPRB, 0.16285E+01_JPRB, 0.15736E+01_JPRB, 0.15821E+01_JPRB, 0.17196E+01_JPRB ,&
&  0.19277E+01_JPRB, 0.22295E+01_JPRB, 0.22820E+01_JPRB, 0.19269E+01_JPRB, 0.17404E+01_JPRB, 0.15275E+01_JPRB, 0.16423E+01_JPRB ,&
&  0.19347E+01_JPRB, 0.19704E+01_JPRB, 0.19235E+01_JPRB, 0.20279E+01_JPRB, 0.16851E+01_JPRB, 0.15495E+01_JPRB, 0.14256E+01_JPRB ,&
&  0.12881E+01_JPRB, 0.11492E+01_JPRB, 0.10549E+01_JPRB, 0.98048E+00_JPRB, 0.89576E+00_JPRB, 0.83967E+00_JPRB, 0.81472E+00_JPRB ,&
&  0.78634E+00_JPRB, 0.76547E+00_JPRB, 0.73971E+00_JPRB, 0.71198E+00_JPRB, 0.69237E+00_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.69334E+00_JPRB, 0.69515E+00_JPRB, 0.70640E+00_JPRB, 0.72691E+00_JPRB, 0.82175E+00_JPRB, 0.85646E+00_JPRB, 0.85192E+00_JPRB ,&
&  0.82877E+00_JPRB, 0.58006E+00_JPRB, 0.54500E+00_JPRB, 0.54772E+00_JPRB, 0.56971E+00_JPRB, 0.60794E+00_JPRB, 0.67668E+00_JPRB ,&
&  0.74547E+00_JPRB, 0.85116E+00_JPRB, 0.10263E+01_JPRB, 0.12483E+01_JPRB, 0.14181E+01_JPRB, 0.14810E+01_JPRB, 0.13567E+01_JPRB ,&
&  0.91988E+00_JPRB, 0.11531E+01_JPRB, 0.11033E+01_JPRB, 0.11571E+01_JPRB, 0.11385E+01_JPRB, 0.10865E+01_JPRB, 0.96862E+00_JPRB ,&
&  0.89968E+00_JPRB, 0.89018E+00_JPRB, 0.87058E+00_JPRB, 0.86643E+00_JPRB, 0.85340E+00_JPRB, 0.78763E+00_JPRB, 0.77124E+00_JPRB ,&
&  0.80451E+00_JPRB, 0.87289E+00_JPRB, 0.95168E+00_JPRB, 0.10186E+01_JPRB, 0.11025E+01_JPRB, 0.11773E+01_JPRB, 0.12248E+01_JPRB ,&
&  0.12810E+01_JPRB, 0.12864E+01_JPRB, 0.12012E+01_JPRB, 0.11565E+01_JPRB, 0.11346E+01_JPRB, 0.11433E+01_JPRB, 0.11339E+01_JPRB ,&
&  0.10628E+01_JPRB, 0.10083E+01_JPRB, 0.10833E+01_JPRB, 0.10742E+01_JPRB, 0.10234E+01_JPRB, 0.10700E+01_JPRB, 0.11259E+01_JPRB ,&
&  0.14116E+01_JPRB, 0.15511E+01_JPRB, 0.20363E+01_JPRB, 0.18660E+01_JPRB, 0.18918E+01_JPRB, 0.20869E+01_JPRB, 0.20209E+01_JPRB ,&
&  0.18117E+01_JPRB, 0.18279E+01_JPRB, 0.17246E+01_JPRB, 0.17516E+01_JPRB, 0.17235E+01_JPRB, 0.17616E+01_JPRB, 0.17871E+01_JPRB ,&
&  0.18811E+01_JPRB, 0.20426E+01_JPRB, 0.22520E+01_JPRB, 0.19021E+01_JPRB, 0.17339E+01_JPRB, 0.16167E+01_JPRB, 0.17171E+01_JPRB ,&
&  0.18759E+01_JPRB, 0.18962E+01_JPRB, 0.19973E+01_JPRB, 0.18977E+01_JPRB, 0.14910E+01_JPRB, 0.13678E+01_JPRB, 0.12699E+01_JPRB ,&
&  0.11814E+01_JPRB, 0.10813E+01_JPRB, 0.10045E+01_JPRB, 0.92305E+00_JPRB, 0.85259E+00_JPRB, 0.81743E+00_JPRB, 0.79459E+00_JPRB ,&
&  0.77194E+00_JPRB, 0.75581E+00_JPRB, 0.73925E+00_JPRB, 0.71678E+00_JPRB, 0.70041E+00_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.71486E+00_JPRB, 0.72178E+00_JPRB, 0.73876E+00_JPRB, 0.75283E+00_JPRB, 0.85014E+00_JPRB, 0.96830E+00_JPRB, 0.10365E+01_JPRB ,&
&  0.98310E+00_JPRB, 0.61082E+00_JPRB, 0.58810E+00_JPRB, 0.59594E+00_JPRB, 0.62090E+00_JPRB, 0.68026E+00_JPRB, 0.75470E+00_JPRB ,&
&  0.85724E+00_JPRB, 0.98134E+00_JPRB, 0.11585E+01_JPRB, 0.13760E+01_JPRB, 0.15126E+01_JPRB, 0.16474E+01_JPRB, 0.15913E+01_JPRB ,&
&  0.11281E+01_JPRB, 0.99526E+00_JPRB, 0.11490E+01_JPRB, 0.11889E+01_JPRB, 0.11444E+01_JPRB, 0.11418E+01_JPRB, 0.11237E+01_JPRB ,&
&  0.10671E+01_JPRB, 0.10159E+01_JPRB, 0.10119E+01_JPRB, 0.98839E+00_JPRB, 0.88436E+00_JPRB, 0.85924E+00_JPRB, 0.86999E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.98059E+00_JPRB, 0.10521E+01_JPRB, 0.11125E+01_JPRB, 0.11927E+01_JPRB, 0.12482E+01_JPRB, 0.12984E+01_JPRB ,&
&  0.13206E+01_JPRB, 0.13353E+01_JPRB, 0.12724E+01_JPRB, 0.11972E+01_JPRB, 0.11776E+01_JPRB, 0.11598E+01_JPRB, 0.11214E+01_JPRB ,&
&  0.10872E+01_JPRB, 0.11083E+01_JPRB, 0.11182E+01_JPRB, 0.10775E+01_JPRB, 0.10470E+01_JPRB, 0.12009E+01_JPRB, 0.13472E+01_JPRB ,&
&  0.15504E+01_JPRB, 0.17917E+01_JPRB, 0.23324E+01_JPRB, 0.18343E+01_JPRB, 0.18480E+01_JPRB, 0.19663E+01_JPRB, 0.19388E+01_JPRB ,&
&  0.18083E+01_JPRB, 0.18433E+01_JPRB, 0.17976E+01_JPRB, 0.17804E+01_JPRB, 0.17918E+01_JPRB, 0.17298E+01_JPRB, 0.19653E+01_JPRB ,&
&  0.18802E+01_JPRB, 0.21313E+01_JPRB, 0.20172E+01_JPRB, 0.19174E+01_JPRB, 0.16624E+01_JPRB, 0.16229E+01_JPRB, 0.16208E+01_JPRB ,&
&  0.17362E+01_JPRB, 0.17342E+01_JPRB, 0.16559E+01_JPRB, 0.15630E+01_JPRB, 0.13322E+01_JPRB, 0.12445E+01_JPRB, 0.11596E+01_JPRB ,&
&  0.10679E+01_JPRB, 0.98794E+00_JPRB, 0.94059E+00_JPRB, 0.89419E+00_JPRB, 0.86443E+00_JPRB, 0.83550E+00_JPRB, 0.80408E+00_JPRB ,&
&  0.77828E+00_JPRB, 0.76086E+00_JPRB, 0.74592E+00_JPRB, 0.72713E+00_JPRB, 0.71595E+00_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.72449E+00_JPRB, 0.71922E+00_JPRB, 0.72253E+00_JPRB, 0.70618E+00_JPRB, 0.70305E+00_JPRB, 0.70684E+00_JPRB, 0.68770E+00_JPRB ,&
&  0.65735E+00_JPRB, 0.62469E+00_JPRB, 0.62753E+00_JPRB, 0.65394E+00_JPRB, 0.69590E+00_JPRB, 0.76610E+00_JPRB, 0.85982E+00_JPRB ,&
&  0.97970E+00_JPRB, 0.10857E+01_JPRB, 0.12478E+01_JPRB, 0.13927E+01_JPRB, 0.14955E+01_JPRB, 0.15605E+01_JPRB, 0.15456E+01_JPRB ,&
&  0.12353E+01_JPRB, 0.96608E+00_JPRB, 0.11912E+01_JPRB, 0.12946E+01_JPRB, 0.12430E+01_JPRB, 0.12529E+01_JPRB, 0.12266E+01_JPRB ,&
&  0.11331E+01_JPRB, 0.11097E+01_JPRB, 0.11094E+01_JPRB, 0.10422E+01_JPRB, 0.94632E+00_JPRB, 0.95488E+00_JPRB, 0.99244E+00_JPRB ,&
&  0.10492E+01_JPRB, 0.10973E+01_JPRB, 0.11548E+01_JPRB, 0.12142E+01_JPRB, 0.12723E+01_JPRB, 0.13494E+01_JPRB, 0.13959E+01_JPRB ,&
&  0.14032E+01_JPRB, 0.13960E+01_JPRB, 0.13706E+01_JPRB, 0.12831E+01_JPRB, 0.12226E+01_JPRB, 0.11881E+01_JPRB, 0.11489E+01_JPRB ,&
&  0.11414E+01_JPRB, 0.11329E+01_JPRB, 0.11779E+01_JPRB, 0.12766E+01_JPRB, 0.14417E+01_JPRB, 0.14338E+01_JPRB, 0.16145E+01_JPRB ,&
&  0.17458E+01_JPRB, 0.20319E+01_JPRB, 0.21149E+01_JPRB, 0.23599E+01_JPRB, 0.15126E+01_JPRB, 0.17281E+01_JPRB, 0.16242E+01_JPRB ,&
&  0.18670E+01_JPRB, 0.17745E+01_JPRB, 0.17612E+01_JPRB, 0.17955E+01_JPRB, 0.19107E+01_JPRB, 0.18519E+01_JPRB, 0.19291E+01_JPRB ,&
&  0.19277E+01_JPRB, 0.19099E+01_JPRB, 0.17800E+01_JPRB, 0.17504E+01_JPRB, 0.16488E+01_JPRB, 0.17054E+01_JPRB, 0.15537E+01_JPRB ,&
&  0.15833E+01_JPRB, 0.15496E+01_JPRB, 0.15056E+01_JPRB, 0.14684E+01_JPRB, 0.12715E+01_JPRB, 0.10930E+01_JPRB, 0.10071E+01_JPRB ,&
&  0.96621E+00_JPRB, 0.93225E+00_JPRB, 0.89199E+00_JPRB, 0.86265E+00_JPRB, 0.84549E+00_JPRB, 0.82867E+00_JPRB, 0.80986E+00_JPRB ,&
&  0.78983E+00_JPRB, 0.77260E+00_JPRB, 0.75863E+00_JPRB, 0.74331E+00_JPRB, 0.73235E+00_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.71202E+00_JPRB, 0.70119E+00_JPRB, 0.70014E+00_JPRB, 0.69514E+00_JPRB, 0.68851E+00_JPRB, 0.67122E+00_JPRB, 0.66467E+00_JPRB ,&
&  0.65892E+00_JPRB, 0.66656E+00_JPRB, 0.67223E+00_JPRB, 0.69751E+00_JPRB, 0.73331E+00_JPRB, 0.79565E+00_JPRB, 0.85758E+00_JPRB ,&
&  0.92985E+00_JPRB, 0.10246E+01_JPRB, 0.12033E+01_JPRB, 0.14189E+01_JPRB, 0.15706E+01_JPRB, 0.16391E+01_JPRB, 0.15265E+01_JPRB ,&
&  0.14042E+01_JPRB, 0.13720E+01_JPRB, 0.13240E+01_JPRB, 0.11091E+01_JPRB, 0.10466E+01_JPRB, 0.12628E+01_JPRB, 0.12141E+01_JPRB ,&
&  0.11522E+01_JPRB, 0.11310E+01_JPRB, 0.10970E+01_JPRB, 0.10618E+01_JPRB, 0.10360E+01_JPRB, 0.10529E+01_JPRB, 0.10907E+01_JPRB ,&
&  0.11395E+01_JPRB, 0.11864E+01_JPRB, 0.12494E+01_JPRB, 0.13128E+01_JPRB, 0.13792E+01_JPRB, 0.14788E+01_JPRB, 0.15263E+01_JPRB ,&
&  0.15154E+01_JPRB, 0.15021E+01_JPRB, 0.14662E+01_JPRB, 0.13832E+01_JPRB, 0.12860E+01_JPRB, 0.12689E+01_JPRB, 0.12815E+01_JPRB ,&
&  0.13048E+01_JPRB, 0.13422E+01_JPRB, 0.14006E+01_JPRB, 0.15595E+01_JPRB, 0.17045E+01_JPRB, 0.16143E+01_JPRB, 0.16333E+01_JPRB ,&
&  0.18530E+01_JPRB, 0.21087E+01_JPRB, 0.18794E+01_JPRB, 0.21762E+01_JPRB, 0.13020E+01_JPRB, 0.15602E+01_JPRB, 0.17767E+01_JPRB ,&
&  0.16815E+01_JPRB, 0.16675E+01_JPRB, 0.17377E+01_JPRB, 0.18671E+01_JPRB, 0.19911E+01_JPRB, 0.18436E+01_JPRB, 0.18468E+01_JPRB ,&
&  0.17756E+01_JPRB, 0.16130E+01_JPRB, 0.15697E+01_JPRB, 0.15455E+01_JPRB, 0.15839E+01_JPRB, 0.15815E+01_JPRB, 0.13589E+01_JPRB ,&
&  0.12297E+01_JPRB, 0.14022E+01_JPRB, 0.14574E+01_JPRB, 0.14572E+01_JPRB, 0.12331E+01_JPRB, 0.95417E+00_JPRB, 0.92622E+00_JPRB ,&
&  0.91733E+00_JPRB, 0.87918E+00_JPRB, 0.85718E+00_JPRB, 0.84331E+00_JPRB, 0.83630E+00_JPRB, 0.82460E+00_JPRB, 0.81356E+00_JPRB ,&
&  0.80227E+00_JPRB, 0.78346E+00_JPRB, 0.76656E+00_JPRB, 0.74824E+00_JPRB, 0.72884E+00_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.68768E+00_JPRB, 0.68472E+00_JPRB, 0.68790E+00_JPRB, 0.68449E+00_JPRB, 0.65421E+00_JPRB, 0.63494E+00_JPRB, 0.62938E+00_JPRB ,&
&  0.63858E+00_JPRB, 0.65423E+00_JPRB, 0.66626E+00_JPRB, 0.67492E+00_JPRB, 0.70266E+00_JPRB, 0.72008E+00_JPRB, 0.75054E+00_JPRB ,&
&  0.80635E+00_JPRB, 0.89636E+00_JPRB, 0.10644E+01_JPRB, 0.13290E+01_JPRB, 0.15703E+01_JPRB, 0.17837E+01_JPRB, 0.16424E+01_JPRB ,&
&  0.15030E+01_JPRB, 0.14291E+01_JPRB, 0.14134E+01_JPRB, 0.14084E+01_JPRB, 0.12496E+01_JPRB, 0.13281E+01_JPRB, 0.13903E+01_JPRB ,&
&  0.13959E+01_JPRB, 0.13031E+01_JPRB, 0.12061E+01_JPRB, 0.11659E+01_JPRB, 0.11369E+01_JPRB, 0.11446E+01_JPRB, 0.11784E+01_JPRB ,&
&  0.12220E+01_JPRB, 0.12679E+01_JPRB, 0.13412E+01_JPRB, 0.14332E+01_JPRB, 0.15438E+01_JPRB, 0.16460E+01_JPRB, 0.16844E+01_JPRB ,&
&  0.16730E+01_JPRB, 0.16597E+01_JPRB, 0.16304E+01_JPRB, 0.15351E+01_JPRB, 0.14963E+01_JPRB, 0.15188E+01_JPRB, 0.15705E+01_JPRB ,&
&  0.16409E+01_JPRB, 0.16204E+01_JPRB, 0.17025E+01_JPRB, 0.18802E+01_JPRB, 0.19183E+01_JPRB, 0.18589E+01_JPRB, 0.18473E+01_JPRB ,&
&  0.19822E+01_JPRB, 0.21709E+01_JPRB, 0.14996E+01_JPRB, 0.17649E+01_JPRB, 0.20704E+01_JPRB, 0.17569E+01_JPRB, 0.16378E+01_JPRB ,&
&  0.16177E+01_JPRB, 0.16467E+01_JPRB, 0.17453E+01_JPRB, 0.18736E+01_JPRB, 0.20040E+01_JPRB, 0.20909E+01_JPRB, 0.16323E+01_JPRB ,&
&  0.15568E+01_JPRB, 0.14075E+01_JPRB, 0.13802E+01_JPRB, 0.13925E+01_JPRB, 0.14256E+01_JPRB, 0.13873E+01_JPRB, 0.12076E+01_JPRB ,&
&  0.11542E+01_JPRB, 0.13008E+01_JPRB, 0.13319E+01_JPRB, 0.14706E+01_JPRB, 0.14568E+01_JPRB, 0.88981E+00_JPRB, 0.90432E+00_JPRB ,&
&  0.88593E+00_JPRB, 0.85599E+00_JPRB, 0.84250E+00_JPRB, 0.83157E+00_JPRB, 0.82558E+00_JPRB, 0.81803E+00_JPRB, 0.81173E+00_JPRB ,&
&  0.79699E+00_JPRB, 0.77421E+00_JPRB, 0.75030E+00_JPRB, 0.72658E+00_JPRB, 0.70468E+00_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.67506E+00_JPRB, 0.67676E+00_JPRB, 0.67982E+00_JPRB, 0.66055E+00_JPRB, 0.63059E+00_JPRB, 0.61631E+00_JPRB, 0.60788E+00_JPRB ,&
&  0.60980E+00_JPRB, 0.62607E+00_JPRB, 0.63608E+00_JPRB, 0.63389E+00_JPRB, 0.63142E+00_JPRB, 0.64624E+00_JPRB, 0.68894E+00_JPRB ,&
&  0.74448E+00_JPRB, 0.81525E+00_JPRB, 0.94492E+00_JPRB, 0.12100E+01_JPRB, 0.15378E+01_JPRB, 0.17884E+01_JPRB, 0.17210E+01_JPRB ,&
&  0.15830E+01_JPRB, 0.15774E+01_JPRB, 0.15835E+01_JPRB, 0.16087E+01_JPRB, 0.16197E+01_JPRB, 0.16639E+01_JPRB, 0.16939E+01_JPRB ,&
&  0.14803E+01_JPRB, 0.12214E+01_JPRB, 0.11871E+01_JPRB, 0.11855E+01_JPRB, 0.12027E+01_JPRB, 0.12174E+01_JPRB, 0.12505E+01_JPRB ,&
&  0.12933E+01_JPRB, 0.13592E+01_JPRB, 0.14535E+01_JPRB, 0.16013E+01_JPRB, 0.17576E+01_JPRB, 0.18487E+01_JPRB, 0.18931E+01_JPRB ,&
&  0.19166E+01_JPRB, 0.19302E+01_JPRB, 0.19782E+01_JPRB, 0.17889E+01_JPRB, 0.18509E+01_JPRB, 0.19059E+01_JPRB, 0.19592E+01_JPRB ,&
&  0.19462E+01_JPRB, 0.18909E+01_JPRB, 0.20300E+01_JPRB, 0.21290E+01_JPRB, 0.22235E+01_JPRB, 0.22082E+01_JPRB, 0.21489E+01_JPRB ,&
&  0.22248E+01_JPRB, 0.20295E+01_JPRB, 0.13530E+01_JPRB, 0.13283E+01_JPRB, 0.14256E+01_JPRB, 0.13174E+01_JPRB, 0.15723E+01_JPRB ,&
&  0.15409E+01_JPRB, 0.15721E+01_JPRB, 0.16086E+01_JPRB, 0.16637E+01_JPRB, 0.17248E+01_JPRB, 0.17548E+01_JPRB, 0.14810E+01_JPRB ,&
&  0.13347E+01_JPRB, 0.12394E+01_JPRB, 0.12174E+01_JPRB, 0.11951E+01_JPRB, 0.11812E+01_JPRB, 0.11433E+01_JPRB, 0.11195E+01_JPRB ,&
&  0.11219E+01_JPRB, 0.11103E+01_JPRB, 0.10618E+01_JPRB, 0.10450E+01_JPRB, 0.95780E+00_JPRB, 0.88696E+00_JPRB, 0.88317E+00_JPRB ,&
&  0.87334E+00_JPRB, 0.84024E+00_JPRB, 0.83112E+00_JPRB, 0.81866E+00_JPRB, 0.81038E+00_JPRB, 0.80089E+00_JPRB, 0.78823E+00_JPRB ,&
&  0.76724E+00_JPRB, 0.74629E+00_JPRB, 0.72482E+00_JPRB, 0.70100E+00_JPRB, 0.68174E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.67499E+00_JPRB, 0.67882E+00_JPRB, 0.67509E+00_JPRB, 0.65776E+00_JPRB, 0.65948E+00_JPRB, 0.66474E+00_JPRB, 0.65075E+00_JPRB ,&
&  0.64136E+00_JPRB, 0.63487E+00_JPRB, 0.64895E+00_JPRB, 0.65040E+00_JPRB, 0.65425E+00_JPRB, 0.66871E+00_JPRB, 0.68611E+00_JPRB ,&
&  0.70457E+00_JPRB, 0.71978E+00_JPRB, 0.78597E+00_JPRB, 0.92174E+00_JPRB, 0.11746E+01_JPRB, 0.15037E+01_JPRB, 0.17313E+01_JPRB ,&
&  0.18539E+01_JPRB, 0.19659E+01_JPRB, 0.20640E+01_JPRB, 0.21411E+01_JPRB, 0.19851E+01_JPRB, 0.20021E+01_JPRB, 0.21333E+01_JPRB ,&
&  0.15581E+01_JPRB, 0.11534E+01_JPRB, 0.12504E+01_JPRB, 0.11761E+01_JPRB, 0.12030E+01_JPRB, 0.12615E+01_JPRB, 0.13099E+01_JPRB ,&
&  0.13782E+01_JPRB, 0.14636E+01_JPRB, 0.15732E+01_JPRB, 0.17265E+01_JPRB, 0.18838E+01_JPRB, 0.19933E+01_JPRB, 0.20762E+01_JPRB ,&
&  0.21460E+01_JPRB, 0.22229E+01_JPRB, 0.23416E+01_JPRB, 0.23222E+01_JPRB, 0.22720E+01_JPRB, 0.23572E+01_JPRB, 0.23976E+01_JPRB ,&
&  0.24502E+01_JPRB, 0.24462E+01_JPRB, 0.21539E+01_JPRB, 0.22300E+01_JPRB, 0.24975E+01_JPRB, 0.23681E+01_JPRB, 0.23326E+01_JPRB ,&
&  0.24861E+01_JPRB, 0.20829E+01_JPRB, 0.12472E+01_JPRB, 0.11910E+01_JPRB, 0.12874E+01_JPRB, 0.13939E+01_JPRB, 0.14987E+01_JPRB ,&
&  0.15201E+01_JPRB, 0.15159E+01_JPRB, 0.14792E+01_JPRB, 0.14684E+01_JPRB, 0.14119E+01_JPRB, 0.13555E+01_JPRB, 0.12934E+01_JPRB ,&
&  0.11648E+01_JPRB, 0.11293E+01_JPRB, 0.10915E+01_JPRB, 0.10506E+01_JPRB, 0.10353E+01_JPRB, 0.10057E+01_JPRB, 0.10030E+01_JPRB ,&
&  0.10695E+01_JPRB, 0.10622E+01_JPRB, 0.11247E+01_JPRB, 0.10179E+01_JPRB, 0.89715E+00_JPRB, 0.90709E+00_JPRB, 0.89111E+00_JPRB ,&
&  0.90797E+00_JPRB, 0.88513E+00_JPRB, 0.86336E+00_JPRB, 0.84167E+00_JPRB, 0.82475E+00_JPRB, 0.80135E+00_JPRB, 0.77041E+00_JPRB ,&
&  0.73933E+00_JPRB, 0.71579E+00_JPRB, 0.70101E+00_JPRB, 0.68660E+00_JPRB, 0.67223E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.67206E+00_JPRB, 0.66005E+00_JPRB, 0.66891E+00_JPRB, 0.70749E+00_JPRB, 0.68553E+00_JPRB, 0.70617E+00_JPRB, 0.70086E+00_JPRB ,&
&  0.70026E+00_JPRB, 0.69622E+00_JPRB, 0.70982E+00_JPRB, 0.72647E+00_JPRB, 0.75049E+00_JPRB, 0.77153E+00_JPRB, 0.77821E+00_JPRB ,&
&  0.78923E+00_JPRB, 0.80085E+00_JPRB, 0.82010E+00_JPRB, 0.85411E+00_JPRB, 0.91515E+00_JPRB, 0.10159E+01_JPRB, 0.12129E+01_JPRB ,&
&  0.14112E+01_JPRB, 0.15083E+01_JPRB, 0.15385E+01_JPRB, 0.15295E+01_JPRB, 0.16885E+01_JPRB, 0.21051E+01_JPRB, 0.24032E+01_JPRB ,&
&  0.14405E+01_JPRB, 0.12383E+01_JPRB, 0.11460E+01_JPRB, 0.10517E+01_JPRB, 0.10955E+01_JPRB, 0.12324E+01_JPRB, 0.13075E+01_JPRB ,&
&  0.13515E+01_JPRB, 0.14332E+01_JPRB, 0.15389E+01_JPRB, 0.16703E+01_JPRB, 0.17845E+01_JPRB, 0.18949E+01_JPRB, 0.20113E+01_JPRB ,&
&  0.21382E+01_JPRB, 0.23045E+01_JPRB, 0.24807E+01_JPRB, 0.26726E+01_JPRB, 0.26539E+01_JPRB, 0.29620E+01_JPRB, 0.30281E+01_JPRB ,&
&  0.30708E+01_JPRB, 0.29269E+01_JPRB, 0.25411E+01_JPRB, 0.25867E+01_JPRB, 0.27727E+01_JPRB, 0.27459E+01_JPRB, 0.25381E+01_JPRB ,&
&  0.25284E+01_JPRB, 0.19369E+01_JPRB, 0.13338E+01_JPRB, 0.13117E+01_JPRB, 0.13225E+01_JPRB, 0.13754E+01_JPRB, 0.13318E+01_JPRB ,&
&  0.13663E+01_JPRB, 0.13264E+01_JPRB, 0.12831E+01_JPRB, 0.12381E+01_JPRB, 0.12005E+01_JPRB, 0.12103E+01_JPRB, 0.12098E+01_JPRB ,&
&  0.11092E+01_JPRB, 0.10787E+01_JPRB, 0.10150E+01_JPRB, 0.97631E+00_JPRB, 0.98551E+00_JPRB, 0.85533E+00_JPRB, 0.94287E+00_JPRB ,&
&  0.97450E+00_JPRB, 0.99095E+00_JPRB, 0.89495E+00_JPRB, 0.10752E+01_JPRB, 0.92623E+00_JPRB, 0.88458E+00_JPRB, 0.89440E+00_JPRB ,&
&  0.92075E+00_JPRB, 0.88160E+00_JPRB, 0.86162E+00_JPRB, 0.84375E+00_JPRB, 0.83459E+00_JPRB, 0.81176E+00_JPRB, 0.76271E+00_JPRB ,&
&  0.72586E+00_JPRB, 0.70226E+00_JPRB, 0.69065E+00_JPRB, 0.68081E+00_JPRB, 0.67901E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.65886E+00_JPRB, 0.66574E+00_JPRB, 0.72831E+00_JPRB, 0.69000E+00_JPRB, 0.68952E+00_JPRB, 0.70581E+00_JPRB, 0.72408E+00_JPRB ,&
&  0.72726E+00_JPRB, 0.73920E+00_JPRB, 0.76239E+00_JPRB, 0.80015E+00_JPRB, 0.83659E+00_JPRB, 0.86479E+00_JPRB, 0.89587E+00_JPRB ,&
&  0.91984E+00_JPRB, 0.93436E+00_JPRB, 0.94241E+00_JPRB, 0.95137E+00_JPRB, 0.97962E+00_JPRB, 0.10118E+01_JPRB, 0.10594E+01_JPRB ,&
&  0.11195E+01_JPRB, 0.11346E+01_JPRB, 0.11341E+01_JPRB, 0.11637E+01_JPRB, 0.11946E+01_JPRB, 0.13021E+01_JPRB, 0.15001E+01_JPRB ,&
&  0.11580E+01_JPRB, 0.12176E+01_JPRB, 0.11640E+01_JPRB, 0.11008E+01_JPRB, 0.11377E+01_JPRB, 0.11245E+01_JPRB, 0.11775E+01_JPRB ,&
&  0.12685E+01_JPRB, 0.12898E+01_JPRB, 0.13521E+01_JPRB, 0.14094E+01_JPRB, 0.14781E+01_JPRB, 0.15721E+01_JPRB, 0.16670E+01_JPRB ,&
&  0.17891E+01_JPRB, 0.19433E+01_JPRB, 0.20656E+01_JPRB, 0.22602E+01_JPRB, 0.24452E+01_JPRB, 0.26387E+01_JPRB, 0.27437E+01_JPRB ,&
&  0.28114E+01_JPRB, 0.28584E+01_JPRB, 0.26403E+01_JPRB, 0.26512E+01_JPRB, 0.26577E+01_JPRB, 0.25317E+01_JPRB, 0.22261E+01_JPRB ,&
&  0.19128E+01_JPRB, 0.16420E+01_JPRB, 0.14592E+01_JPRB, 0.13522E+01_JPRB, 0.13142E+01_JPRB, 0.12632E+01_JPRB, 0.12174E+01_JPRB ,&
&  0.11989E+01_JPRB, 0.11367E+01_JPRB, 0.10789E+01_JPRB, 0.10243E+01_JPRB, 0.10081E+01_JPRB, 0.10274E+01_JPRB, 0.10274E+01_JPRB ,&
&  0.98107E+00_JPRB, 0.94097E+00_JPRB, 0.90119E+00_JPRB, 0.89203E+00_JPRB, 0.91105E+00_JPRB, 0.11427E+01_JPRB, 0.10098E+01_JPRB ,&
&  0.82446E+00_JPRB, 0.74434E+00_JPRB, 0.64046E+00_JPRB, 0.86500E+00_JPRB, 0.84509E+00_JPRB, 0.83659E+00_JPRB, 0.86969E+00_JPRB ,&
&  0.88177E+00_JPRB, 0.87431E+00_JPRB, 0.90125E+00_JPRB, 0.86000E+00_JPRB, 0.82488E+00_JPRB, 0.80515E+00_JPRB, 0.79554E+00_JPRB ,&
&  0.74272E+00_JPRB, 0.69093E+00_JPRB, 0.64170E+00_JPRB, 0.64581E+00_JPRB, 0.61379E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.59687E+00_JPRB, 0.64848E+00_JPRB, 0.74133E+00_JPRB, 0.67390E+00_JPRB, 0.67286E+00_JPRB, 0.68910E+00_JPRB, 0.71375E+00_JPRB ,&
&  0.73965E+00_JPRB, 0.77269E+00_JPRB, 0.80798E+00_JPRB, 0.85660E+00_JPRB, 0.89915E+00_JPRB, 0.93641E+00_JPRB, 0.97159E+00_JPRB ,&
&  0.10035E+01_JPRB, 0.10308E+01_JPRB, 0.10570E+01_JPRB, 0.10697E+01_JPRB, 0.10818E+01_JPRB, 0.11051E+01_JPRB, 0.11349E+01_JPRB ,&
&  0.11899E+01_JPRB, 0.12169E+01_JPRB, 0.12507E+01_JPRB, 0.13053E+01_JPRB, 0.13433E+01_JPRB, 0.13801E+01_JPRB, 0.11891E+01_JPRB ,&
&  0.13031E+01_JPRB, 0.11968E+01_JPRB, 0.11490E+01_JPRB, 0.11313E+01_JPRB, 0.11087E+01_JPRB, 0.10983E+01_JPRB, 0.11110E+01_JPRB ,&
&  0.11627E+01_JPRB, 0.11373E+01_JPRB, 0.11604E+01_JPRB, 0.11551E+01_JPRB, 0.11957E+01_JPRB, 0.12827E+01_JPRB, 0.13357E+01_JPRB ,&
&  0.14461E+01_JPRB, 0.15825E+01_JPRB, 0.17237E+01_JPRB, 0.18319E+01_JPRB, 0.19311E+01_JPRB, 0.19354E+01_JPRB, 0.19393E+01_JPRB ,&
&  0.19950E+01_JPRB, 0.21064E+01_JPRB, 0.20639E+01_JPRB, 0.21077E+01_JPRB, 0.20945E+01_JPRB, 0.19258E+01_JPRB, 0.15296E+01_JPRB ,&
&  0.13318E+01_JPRB, 0.13916E+01_JPRB, 0.12561E+01_JPRB, 0.14278E+01_JPRB, 0.12918E+01_JPRB, 0.11591E+01_JPRB, 0.11171E+01_JPRB ,&
&  0.10408E+01_JPRB, 0.96913E+00_JPRB, 0.91620E+00_JPRB, 0.83742E+00_JPRB, 0.83913E+00_JPRB, 0.84690E+00_JPRB, 0.85072E+00_JPRB ,&
&  0.80449E+00_JPRB, 0.75833E+00_JPRB, 0.71972E+00_JPRB, 0.73964E+00_JPRB, 0.90526E+00_JPRB, 0.94530E+00_JPRB, 0.10881E+01_JPRB ,&
&  0.69818E+00_JPRB, 0.46284E+00_JPRB, 0.56828E+00_JPRB, 0.67583E+00_JPRB, 0.69440E+00_JPRB, 0.72184E+00_JPRB, 0.78384E+00_JPRB ,&
&  0.85487E+00_JPRB, 0.85354E+00_JPRB, 0.82540E+00_JPRB, 0.82292E+00_JPRB, 0.81315E+00_JPRB, 0.80962E+00_JPRB, 0.72832E+00_JPRB ,&
&  0.70027E+00_JPRB, 0.64482E+00_JPRB, 0.61378E+00_JPRB, 0.59980E+00_JPRB, 0.57180E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.58305E+00_JPRB, 0.66329E+00_JPRB, 0.73312E+00_JPRB, 0.66514E+00_JPRB, 0.67441E+00_JPRB, 0.69537E+00_JPRB, 0.72687E+00_JPRB ,&
&  0.76296E+00_JPRB, 0.80547E+00_JPRB, 0.84986E+00_JPRB, 0.90218E+00_JPRB, 0.94350E+00_JPRB, 0.98156E+00_JPRB, 0.10225E+01_JPRB ,&
&  0.10649E+01_JPRB, 0.10802E+01_JPRB, 0.11146E+01_JPRB, 0.11212E+01_JPRB, 0.11460E+01_JPRB, 0.11490E+01_JPRB, 0.11695E+01_JPRB ,&
&  0.12155E+01_JPRB, 0.12446E+01_JPRB, 0.12761E+01_JPRB, 0.13003E+01_JPRB, 0.12948E+01_JPRB, 0.13404E+01_JPRB, 0.10541E+01_JPRB ,&
&  0.12655E+01_JPRB, 0.11816E+01_JPRB, 0.11177E+01_JPRB, 0.11281E+01_JPRB, 0.11112E+01_JPRB, 0.11175E+01_JPRB, 0.11176E+01_JPRB ,&
&  0.11161E+01_JPRB, 0.11277E+01_JPRB, 0.11514E+01_JPRB, 0.11412E+01_JPRB, 0.10940E+01_JPRB, 0.11445E+01_JPRB, 0.11974E+01_JPRB ,&
&  0.12711E+01_JPRB, 0.13606E+01_JPRB, 0.14305E+01_JPRB, 0.14750E+01_JPRB, 0.14984E+01_JPRB, 0.14728E+01_JPRB, 0.14438E+01_JPRB ,&
&  0.14113E+01_JPRB, 0.14273E+01_JPRB, 0.13489E+01_JPRB, 0.14310E+01_JPRB, 0.15660E+01_JPRB, 0.13351E+01_JPRB, 0.10284E+01_JPRB ,&
&  0.88756E+00_JPRB, 0.10997E+01_JPRB, 0.12874E+01_JPRB, 0.14293E+01_JPRB, 0.12229E+01_JPRB, 0.11148E+01_JPRB, 0.10241E+01_JPRB ,&
&  0.91820E+00_JPRB, 0.83288E+00_JPRB, 0.78195E+00_JPRB, 0.71035E+00_JPRB, 0.68001E+00_JPRB, 0.67572E+00_JPRB, 0.67371E+00_JPRB ,&
&  0.65324E+00_JPRB, 0.63418E+00_JPRB, 0.62615E+00_JPRB, 0.64046E+00_JPRB, 0.68384E+00_JPRB, 0.71045E+00_JPRB, 0.11800E+01_JPRB ,&
&  0.71964E+00_JPRB, 0.52210E+00_JPRB, 0.78041E+00_JPRB, 0.43595E+00_JPRB, 0.55098E+00_JPRB, 0.60348E+00_JPRB, 0.64834E+00_JPRB ,&
&  0.61177E+00_JPRB, 0.77800E+00_JPRB, 0.11263E+01_JPRB, 0.11583E+01_JPRB, 0.12176E+01_JPRB, 0.12924E+01_JPRB, 0.77142E+00_JPRB ,&
&  0.62276E+00_JPRB, 0.61188E+00_JPRB, 0.61459E+00_JPRB, 0.56677E+00_JPRB, 0.56229E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.56195E+00_JPRB, 0.62404E+00_JPRB, 0.69472E+00_JPRB, 0.74165E+00_JPRB, 0.72091E+00_JPRB, 0.71915E+00_JPRB, 0.73599E+00_JPRB ,&
&  0.76056E+00_JPRB, 0.79310E+00_JPRB, 0.82713E+00_JPRB, 0.87206E+00_JPRB, 0.90284E+00_JPRB, 0.94185E+00_JPRB, 0.99559E+00_JPRB ,&
&  0.10596E+01_JPRB, 0.10738E+01_JPRB, 0.10958E+01_JPRB, 0.11207E+01_JPRB, 0.11426E+01_JPRB, 0.11674E+01_JPRB, 0.11891E+01_JPRB ,&
&  0.11979E+01_JPRB, 0.12167E+01_JPRB, 0.12261E+01_JPRB, 0.12659E+01_JPRB, 0.12865E+01_JPRB, 0.14069E+01_JPRB, 0.10263E+01_JPRB ,&
&  0.12106E+01_JPRB, 0.11517E+01_JPRB, 0.10831E+01_JPRB, 0.11038E+01_JPRB, 0.11019E+01_JPRB, 0.10904E+01_JPRB, 0.10794E+01_JPRB ,&
&  0.11127E+01_JPRB, 0.11298E+01_JPRB, 0.11439E+01_JPRB, 0.12482E+01_JPRB, 0.12489E+01_JPRB, 0.12489E+01_JPRB, 0.13622E+01_JPRB ,&
&  0.14693E+01_JPRB, 0.15596E+01_JPRB, 0.15917E+01_JPRB, 0.16429E+01_JPRB, 0.16458E+01_JPRB, 0.16180E+01_JPRB, 0.15760E+01_JPRB ,&
&  0.15246E+01_JPRB, 0.15139E+01_JPRB, 0.15558E+01_JPRB, 0.16770E+01_JPRB, 0.17168E+01_JPRB, 0.15154E+01_JPRB, 0.12357E+01_JPRB ,&
&  0.91543E+00_JPRB, 0.82691E+00_JPRB, 0.11474E+01_JPRB, 0.13233E+01_JPRB, 0.11531E+01_JPRB, 0.10759E+01_JPRB, 0.92617E+00_JPRB ,&
&  0.84480E+00_JPRB, 0.75529E+00_JPRB, 0.71195E+00_JPRB, 0.65202E+00_JPRB, 0.61960E+00_JPRB, 0.61748E+00_JPRB, 0.64755E+00_JPRB ,&
&  0.66043E+00_JPRB, 0.64435E+00_JPRB, 0.63731E+00_JPRB, 0.63265E+00_JPRB, 0.69547E+00_JPRB, 0.82756E+00_JPRB, 0.83045E+00_JPRB ,&
&  0.88280E+00_JPRB, 0.66583E+00_JPRB, 0.60317E+00_JPRB, 0.63489E+00_JPRB, 0.53500E+00_JPRB, 0.57285E+00_JPRB, 0.63181E+00_JPRB ,&
&  0.81235E+00_JPRB, 0.89497E+00_JPRB, 0.12318E+01_JPRB, 0.34333E+01_JPRB, 0.39550E+01_JPRB, 0.49558E+01_JPRB, 0.19804E+01_JPRB ,&
&  0.92512E+00_JPRB, 0.61799E+00_JPRB, 0.55078E+00_JPRB, 0.55004E+00_JPRB, 0.54268E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.53311E+00_JPRB, 0.58149E+00_JPRB, 0.62112E+00_JPRB, 0.67719E+00_JPRB, 0.69723E+00_JPRB, 0.70290E+00_JPRB, 0.70277E+00_JPRB ,&
&  0.69267E+00_JPRB, 0.69952E+00_JPRB, 0.72214E+00_JPRB, 0.77684E+00_JPRB, 0.83462E+00_JPRB, 0.89109E+00_JPRB, 0.95205E+00_JPRB ,&
&  0.10294E+01_JPRB, 0.10797E+01_JPRB, 0.10677E+01_JPRB, 0.10750E+01_JPRB, 0.11064E+01_JPRB, 0.11400E+01_JPRB, 0.11518E+01_JPRB ,&
&  0.11518E+01_JPRB, 0.11726E+01_JPRB, 0.11982E+01_JPRB, 0.12461E+01_JPRB, 0.13059E+01_JPRB, 0.13503E+01_JPRB, 0.97070E+00_JPRB ,&
&  0.11706E+01_JPRB, 0.11155E+01_JPRB, 0.10942E+01_JPRB, 0.11107E+01_JPRB, 0.10701E+01_JPRB, 0.10082E+01_JPRB, 0.10035E+01_JPRB ,&
&  0.10587E+01_JPRB, 0.11421E+01_JPRB, 0.12357E+01_JPRB, 0.13172E+01_JPRB, 0.14653E+01_JPRB, 0.14967E+01_JPRB, 0.17009E+01_JPRB ,&
&  0.18750E+01_JPRB, 0.19774E+01_JPRB, 0.20284E+01_JPRB, 0.20749E+01_JPRB, 0.21343E+01_JPRB, 0.22234E+01_JPRB, 0.23260E+01_JPRB ,&
&  0.23866E+01_JPRB, 0.24174E+01_JPRB, 0.24637E+01_JPRB, 0.25083E+01_JPRB, 0.22417E+01_JPRB, 0.23591E+01_JPRB, 0.18862E+01_JPRB ,&
&  0.12589E+01_JPRB, 0.10157E+01_JPRB, 0.12059E+01_JPRB, 0.12569E+01_JPRB, 0.11432E+01_JPRB, 0.10469E+01_JPRB, 0.93064E+00_JPRB ,&
&  0.80603E+00_JPRB, 0.72812E+00_JPRB, 0.68024E+00_JPRB, 0.63649E+00_JPRB, 0.59849E+00_JPRB, 0.59812E+00_JPRB, 0.61227E+00_JPRB ,&
&  0.61987E+00_JPRB, 0.61722E+00_JPRB, 0.61464E+00_JPRB, 0.64012E+00_JPRB, 0.70888E+00_JPRB, 0.89789E+00_JPRB, 0.12127E+01_JPRB ,&
&  0.14460E+01_JPRB, 0.71268E+00_JPRB, 0.61713E+00_JPRB, 0.61246E+00_JPRB, 0.63026E+00_JPRB, 0.66600E+00_JPRB, 0.72324E+00_JPRB ,&
&  0.87842E+00_JPRB, 0.11212E+01_JPRB, 0.10432E+01_JPRB, 0.12417E+01_JPRB, 0.25198E+01_JPRB, 0.27509E+01_JPRB, 0.15880E+01_JPRB ,&
&  0.10015E+01_JPRB, 0.70993E+00_JPRB, 0.57648E+00_JPRB, 0.53406E+00_JPRB, 0.52316E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.53810E+00_JPRB, 0.55305E+00_JPRB, 0.57387E+00_JPRB, 0.59861E+00_JPRB, 0.59591E+00_JPRB, 0.59719E+00_JPRB, 0.58511E+00_JPRB ,&
&  0.58298E+00_JPRB, 0.60153E+00_JPRB, 0.64802E+00_JPRB, 0.72661E+00_JPRB, 0.78443E+00_JPRB, 0.86421E+00_JPRB, 0.97231E+00_JPRB ,&
&  0.10078E+01_JPRB, 0.99090E+00_JPRB, 0.98149E+00_JPRB, 0.98589E+00_JPRB, 0.10058E+01_JPRB, 0.10350E+01_JPRB, 0.10612E+01_JPRB ,&
&  0.10882E+01_JPRB, 0.11489E+01_JPRB, 0.12005E+01_JPRB, 0.12738E+01_JPRB, 0.13433E+01_JPRB, 0.13722E+01_JPRB, 0.14211E+01_JPRB ,&
&  0.84070E+00_JPRB, 0.11592E+01_JPRB, 0.11321E+01_JPRB, 0.11063E+01_JPRB, 0.96577E+00_JPRB, 0.87514E+00_JPRB, 0.90048E+00_JPRB ,&
&  0.10487E+01_JPRB, 0.12123E+01_JPRB, 0.14403E+01_JPRB, 0.14821E+01_JPRB, 0.16631E+01_JPRB, 0.17882E+01_JPRB, 0.20254E+01_JPRB ,&
&  0.21259E+01_JPRB, 0.22226E+01_JPRB, 0.23436E+01_JPRB, 0.24774E+01_JPRB, 0.26183E+01_JPRB, 0.27945E+01_JPRB, 0.29587E+01_JPRB ,&
&  0.30867E+01_JPRB, 0.31428E+01_JPRB, 0.32595E+01_JPRB, 0.29309E+01_JPRB, 0.25518E+01_JPRB, 0.26702E+01_JPRB, 0.25841E+01_JPRB ,&
&  0.21378E+01_JPRB, 0.14626E+01_JPRB, 0.13795E+01_JPRB, 0.13839E+01_JPRB, 0.12304E+01_JPRB, 0.11247E+01_JPRB, 0.10030E+01_JPRB ,&
&  0.85613E+00_JPRB, 0.72034E+00_JPRB, 0.66121E+00_JPRB, 0.61460E+00_JPRB, 0.59565E+00_JPRB, 0.62064E+00_JPRB, 0.65845E+00_JPRB ,&
&  0.65902E+00_JPRB, 0.65029E+00_JPRB, 0.64088E+00_JPRB, 0.65652E+00_JPRB, 0.70185E+00_JPRB, 0.74400E+00_JPRB, 0.77840E+00_JPRB ,&
&  0.74658E+00_JPRB, 0.70576E+00_JPRB, 0.69623E+00_JPRB, 0.70608E+00_JPRB, 0.72525E+00_JPRB, 0.74269E+00_JPRB, 0.77550E+00_JPRB ,&
&  0.79283E+00_JPRB, 0.79047E+00_JPRB, 0.87985E+00_JPRB, 0.10750E+01_JPRB, 0.11825E+01_JPRB, 0.12646E+01_JPRB, 0.10449E+01_JPRB ,&
&  0.83621E+00_JPRB, 0.70511E+00_JPRB, 0.62356E+00_JPRB, 0.56978E+00_JPRB, 0.52729E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.52910E+00_JPRB, 0.53004E+00_JPRB, 0.52761E+00_JPRB, 0.53811E+00_JPRB, 0.53710E+00_JPRB, 0.53789E+00_JPRB, 0.53459E+00_JPRB ,&
&  0.55922E+00_JPRB, 0.60748E+00_JPRB, 0.68126E+00_JPRB, 0.76483E+00_JPRB, 0.82579E+00_JPRB, 0.90081E+00_JPRB, 0.93674E+00_JPRB ,&
&  0.93198E+00_JPRB, 0.91257E+00_JPRB, 0.90837E+00_JPRB, 0.93008E+00_JPRB, 0.94559E+00_JPRB, 0.97468E+00_JPRB, 0.10073E+01_JPRB ,&
&  0.10421E+01_JPRB, 0.10942E+01_JPRB, 0.11594E+01_JPRB, 0.12364E+01_JPRB, 0.13289E+01_JPRB, 0.13848E+01_JPRB, 0.13996E+01_JPRB ,&
&  0.54244E+00_JPRB, 0.75486E+00_JPRB, 0.10688E+01_JPRB, 0.11205E+01_JPRB, 0.87286E+00_JPRB, 0.77837E+00_JPRB, 0.72114E+00_JPRB ,&
&  0.86965E+00_JPRB, 0.11405E+01_JPRB, 0.13829E+01_JPRB, 0.16939E+01_JPRB, 0.17708E+01_JPRB, 0.19672E+01_JPRB, 0.21200E+01_JPRB ,&
&  0.21762E+01_JPRB, 0.22515E+01_JPRB, 0.23887E+01_JPRB, 0.25578E+01_JPRB, 0.27196E+01_JPRB, 0.28759E+01_JPRB, 0.30421E+01_JPRB ,&
&  0.32224E+01_JPRB, 0.33354E+01_JPRB, 0.34701E+01_JPRB, 0.29489E+01_JPRB, 0.30597E+01_JPRB, 0.32438E+01_JPRB, 0.36915E+01_JPRB ,&
&  0.30607E+01_JPRB, 0.18648E+01_JPRB, 0.14362E+01_JPRB, 0.14729E+01_JPRB, 0.13162E+01_JPRB, 0.12168E+01_JPRB, 0.10577E+01_JPRB ,&
&  0.89090E+00_JPRB, 0.76690E+00_JPRB, 0.70062E+00_JPRB, 0.67965E+00_JPRB, 0.68275E+00_JPRB, 0.71587E+00_JPRB, 0.74282E+00_JPRB ,&
&  0.73333E+00_JPRB, 0.72527E+00_JPRB, 0.71606E+00_JPRB, 0.71222E+00_JPRB, 0.75146E+00_JPRB, 0.79056E+00_JPRB, 0.80780E+00_JPRB ,&
&  0.80722E+00_JPRB, 0.80556E+00_JPRB, 0.81625E+00_JPRB, 0.83546E+00_JPRB, 0.85995E+00_JPRB, 0.87203E+00_JPRB, 0.87596E+00_JPRB ,&
&  0.78671E+00_JPRB, 0.74118E+00_JPRB, 0.74009E+00_JPRB, 0.73413E+00_JPRB, 0.75088E+00_JPRB, 0.78376E+00_JPRB, 0.73784E+00_JPRB ,&
&  0.70889E+00_JPRB, 0.69751E+00_JPRB, 0.66928E+00_JPRB, 0.60565E+00_JPRB, 0.54208E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.54576E+00_JPRB, 0.54537E+00_JPRB, 0.53567E+00_JPRB, 0.53480E+00_JPRB, 0.52708E+00_JPRB, 0.53348E+00_JPRB, 0.54820E+00_JPRB ,&
&  0.57928E+00_JPRB, 0.63354E+00_JPRB, 0.70061E+00_JPRB, 0.76051E+00_JPRB, 0.83029E+00_JPRB, 0.88353E+00_JPRB, 0.88594E+00_JPRB ,&
&  0.86929E+00_JPRB, 0.86588E+00_JPRB, 0.85491E+00_JPRB, 0.87858E+00_JPRB, 0.90012E+00_JPRB, 0.93351E+00_JPRB, 0.97321E+00_JPRB ,&
&  0.10213E+01_JPRB, 0.10826E+01_JPRB, 0.11591E+01_JPRB, 0.12490E+01_JPRB, 0.13758E+01_JPRB, 0.15180E+01_JPRB, 0.16477E+01_JPRB ,&
&  0.11106E+01_JPRB, 0.58268E+00_JPRB, 0.10088E+01_JPRB, 0.11629E+01_JPRB, 0.90825E+00_JPRB, 0.80141E+00_JPRB, 0.68276E+00_JPRB ,&
&  0.68190E+00_JPRB, 0.10178E+01_JPRB, 0.13165E+01_JPRB, 0.16092E+01_JPRB, 0.17279E+01_JPRB, 0.19121E+01_JPRB, 0.20480E+01_JPRB ,&
&  0.21194E+01_JPRB, 0.21962E+01_JPRB, 0.22869E+01_JPRB, 0.23914E+01_JPRB, 0.25260E+01_JPRB, 0.26668E+01_JPRB, 0.28450E+01_JPRB ,&
&  0.30622E+01_JPRB, 0.32200E+01_JPRB, 0.33354E+01_JPRB, 0.28803E+01_JPRB, 0.30833E+01_JPRB, 0.29453E+01_JPRB, 0.28173E+01_JPRB ,&
&  0.25387E+01_JPRB, 0.15667E+01_JPRB, 0.12026E+01_JPRB, 0.14456E+01_JPRB, 0.13345E+01_JPRB, 0.10278E+01_JPRB, 0.10891E+01_JPRB ,&
&  0.95250E+00_JPRB, 0.83103E+00_JPRB, 0.80169E+00_JPRB, 0.80148E+00_JPRB, 0.77340E+00_JPRB, 0.78172E+00_JPRB, 0.78296E+00_JPRB ,&
&  0.80809E+00_JPRB, 0.79539E+00_JPRB, 0.77434E+00_JPRB, 0.78451E+00_JPRB, 0.80992E+00_JPRB, 0.84433E+00_JPRB, 0.86780E+00_JPRB ,&
&  0.89170E+00_JPRB, 0.90805E+00_JPRB, 0.92120E+00_JPRB, 0.94382E+00_JPRB, 0.96249E+00_JPRB, 0.90687E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.81512E+00_JPRB, 0.79279E+00_JPRB, 0.77652E+00_JPRB, 0.76476E+00_JPRB, 0.75128E+00_JPRB, 0.76148E+00_JPRB, 0.79179E+00_JPRB ,&
&  0.82267E+00_JPRB, 0.82821E+00_JPRB, 0.75955E+00_JPRB, 0.63776E+00_JPRB, 0.56747E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.54331E+00_JPRB, 0.53447E+00_JPRB, 0.52413E+00_JPRB, 0.53759E+00_JPRB, 0.53634E+00_JPRB, 0.54449E+00_JPRB, 0.56347E+00_JPRB ,&
&  0.59426E+00_JPRB, 0.63369E+00_JPRB, 0.66450E+00_JPRB, 0.70985E+00_JPRB, 0.77091E+00_JPRB, 0.82587E+00_JPRB, 0.83015E+00_JPRB ,&
&  0.81726E+00_JPRB, 0.81132E+00_JPRB, 0.81541E+00_JPRB, 0.82408E+00_JPRB, 0.85098E+00_JPRB, 0.88576E+00_JPRB, 0.92068E+00_JPRB ,&
&  0.96954E+00_JPRB, 0.10372E+01_JPRB, 0.11083E+01_JPRB, 0.11946E+01_JPRB, 0.13018E+01_JPRB, 0.14735E+01_JPRB, 0.17057E+01_JPRB ,&
&  0.23457E+01_JPRB, 0.22600E+01_JPRB, 0.90714E+00_JPRB, 0.11758E+01_JPRB, 0.10482E+01_JPRB, 0.89130E+00_JPRB, 0.72157E+00_JPRB ,&
&  0.67215E+00_JPRB, 0.85795E+00_JPRB, 0.11987E+01_JPRB, 0.15150E+01_JPRB, 0.15438E+01_JPRB, 0.16935E+01_JPRB, 0.18232E+01_JPRB ,&
&  0.19668E+01_JPRB, 0.20624E+01_JPRB, 0.21070E+01_JPRB, 0.21270E+01_JPRB, 0.21862E+01_JPRB, 0.22871E+01_JPRB, 0.24702E+01_JPRB ,&
&  0.27455E+01_JPRB, 0.30439E+01_JPRB, 0.32693E+01_JPRB, 0.30248E+01_JPRB, 0.31584E+01_JPRB, 0.27628E+01_JPRB, 0.24493E+01_JPRB ,&
&  0.18936E+01_JPRB, 0.15932E+01_JPRB, 0.15054E+01_JPRB, 0.14962E+01_JPRB, 0.12865E+01_JPRB, 0.11388E+01_JPRB, 0.12091E+01_JPRB ,&
&  0.11139E+01_JPRB, 0.98495E+00_JPRB, 0.90919E+00_JPRB, 0.86136E+00_JPRB, 0.84448E+00_JPRB, 0.83810E+00_JPRB, 0.84307E+00_JPRB ,&
&  0.86582E+00_JPRB, 0.86705E+00_JPRB, 0.84201E+00_JPRB, 0.86421E+00_JPRB, 0.88823E+00_JPRB, 0.91392E+00_JPRB, 0.93298E+00_JPRB ,&
&  0.95635E+00_JPRB, 0.98121E+00_JPRB, 0.99463E+00_JPRB, 0.99776E+00_JPRB, 0.94064E+00_JPRB, 0.88073E+00_JPRB, 0.88639E+00_JPRB ,&
&  0.84399E+00_JPRB, 0.84584E+00_JPRB, 0.85409E+00_JPRB, 0.85938E+00_JPRB, 0.84373E+00_JPRB, 0.86559E+00_JPRB, 0.87761E+00_JPRB ,&
&  0.91546E+00_JPRB, 0.94810E+00_JPRB, 0.86296E+00_JPRB, 0.65928E+00_JPRB, 0.57827E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.58114E+00_JPRB, 0.57006E+00_JPRB, 0.56386E+00_JPRB, 0.56960E+00_JPRB, 0.57336E+00_JPRB, 0.56453E+00_JPRB, 0.56677E+00_JPRB ,&
&  0.57731E+00_JPRB, 0.59203E+00_JPRB, 0.59102E+00_JPRB, 0.62330E+00_JPRB, 0.69624E+00_JPRB, 0.77092E+00_JPRB, 0.77459E+00_JPRB ,&
&  0.76241E+00_JPRB, 0.74886E+00_JPRB, 0.74231E+00_JPRB, 0.75596E+00_JPRB, 0.77000E+00_JPRB, 0.81249E+00_JPRB, 0.85575E+00_JPRB ,&
&  0.89874E+00_JPRB, 0.94368E+00_JPRB, 0.98477E+00_JPRB, 0.10264E+01_JPRB, 0.11004E+01_JPRB, 0.12128E+01_JPRB, 0.15105E+01_JPRB ,&
&  0.20520E+01_JPRB, 0.37177E+01_JPRB, 0.11535E+01_JPRB, 0.11776E+01_JPRB, 0.11232E+01_JPRB, 0.99100E+00_JPRB, 0.87391E+00_JPRB ,&
&  0.83409E+00_JPRB, 0.79396E+00_JPRB, 0.10847E+01_JPRB, 0.13967E+01_JPRB, 0.13588E+01_JPRB, 0.14061E+01_JPRB, 0.14894E+01_JPRB ,&
&  0.16758E+01_JPRB, 0.18090E+01_JPRB, 0.18908E+01_JPRB, 0.18782E+01_JPRB, 0.18549E+01_JPRB, 0.18412E+01_JPRB, 0.18958E+01_JPRB ,&
&  0.21175E+01_JPRB, 0.25378E+01_JPRB, 0.29826E+01_JPRB, 0.30543E+01_JPRB, 0.32840E+01_JPRB, 0.28907E+01_JPRB, 0.26242E+01_JPRB ,&
&  0.21602E+01_JPRB, 0.19632E+01_JPRB, 0.19403E+01_JPRB, 0.17730E+01_JPRB, 0.14531E+01_JPRB, 0.13058E+01_JPRB, 0.13145E+01_JPRB ,&
&  0.12457E+01_JPRB, 0.11222E+01_JPRB, 0.10009E+01_JPRB, 0.90833E+00_JPRB, 0.89368E+00_JPRB, 0.91977E+00_JPRB, 0.89936E+00_JPRB ,&
&  0.92662E+00_JPRB, 0.96488E+00_JPRB, 0.94010E+00_JPRB, 0.97410E+00_JPRB, 0.99106E+00_JPRB, 0.99585E+00_JPRB, 0.99511E+00_JPRB ,&
&  0.99638E+00_JPRB, 0.10198E+01_JPRB, 0.10255E+01_JPRB, 0.98345E+00_JPRB, 0.89860E+00_JPRB, 0.84588E+00_JPRB, 0.84930E+00_JPRB ,&
&  0.85411E+00_JPRB, 0.88219E+00_JPRB, 0.88824E+00_JPRB, 0.86189E+00_JPRB, 0.89609E+00_JPRB, 0.89269E+00_JPRB, 0.89994E+00_JPRB ,&
&  0.92934E+00_JPRB, 0.99915E+00_JPRB, 0.10410E+01_JPRB, 0.74571E+00_JPRB, 0.63615E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.65212E+00_JPRB, 0.63057E+00_JPRB, 0.61006E+00_JPRB, 0.59230E+00_JPRB, 0.58170E+00_JPRB, 0.58670E+00_JPRB, 0.59105E+00_JPRB ,&
&  0.58608E+00_JPRB, 0.58663E+00_JPRB, 0.58141E+00_JPRB, 0.58626E+00_JPRB, 0.60983E+00_JPRB, 0.65631E+00_JPRB, 0.70386E+00_JPRB ,&
&  0.70330E+00_JPRB, 0.67138E+00_JPRB, 0.66603E+00_JPRB, 0.66129E+00_JPRB, 0.67989E+00_JPRB, 0.70469E+00_JPRB, 0.75064E+00_JPRB ,&
&  0.81153E+00_JPRB, 0.85820E+00_JPRB, 0.88188E+00_JPRB, 0.89413E+00_JPRB, 0.92138E+00_JPRB, 0.96389E+00_JPRB, 0.12384E+01_JPRB ,&
&  0.18222E+01_JPRB, 0.37128E+01_JPRB, 0.28359E+01_JPRB, 0.18206E+01_JPRB, 0.12195E+01_JPRB, 0.10202E+01_JPRB, 0.98947E+00_JPRB ,&
&  0.10696E+01_JPRB, 0.10313E+01_JPRB, 0.12141E+01_JPRB, 0.12720E+01_JPRB, 0.12070E+01_JPRB, 0.11480E+01_JPRB, 0.11987E+01_JPRB ,&
&  0.13471E+01_JPRB, 0.14909E+01_JPRB, 0.15927E+01_JPRB, 0.16147E+01_JPRB, 0.15778E+01_JPRB, 0.15468E+01_JPRB, 0.15289E+01_JPRB ,&
&  0.15287E+01_JPRB, 0.16985E+01_JPRB, 0.23112E+01_JPRB, 0.29677E+01_JPRB, 0.33565E+01_JPRB, 0.31165E+01_JPRB, 0.29486E+01_JPRB ,&
&  0.28718E+01_JPRB, 0.25581E+01_JPRB, 0.23560E+01_JPRB, 0.20198E+01_JPRB, 0.16082E+01_JPRB, 0.16322E+01_JPRB, 0.13979E+01_JPRB ,&
&  0.13022E+01_JPRB, 0.12096E+01_JPRB, 0.11012E+01_JPRB, 0.98162E+00_JPRB, 0.97463E+00_JPRB, 0.98162E+00_JPRB, 0.95382E+00_JPRB ,&
&  0.98320E+00_JPRB, 0.10200E+01_JPRB, 0.10004E+01_JPRB, 0.10071E+01_JPRB, 0.10099E+01_JPRB, 0.10201E+01_JPRB, 0.10236E+01_JPRB ,&
&  0.10237E+01_JPRB, 0.10460E+01_JPRB, 0.97980E+00_JPRB, 0.91156E+00_JPRB, 0.87439E+00_JPRB, 0.82951E+00_JPRB, 0.81525E+00_JPRB ,&
&  0.84498E+00_JPRB, 0.90692E+00_JPRB, 0.91145E+00_JPRB, 0.91750E+00_JPRB, 0.90286E+00_JPRB, 0.90407E+00_JPRB, 0.87290E+00_JPRB ,&
&  0.86584E+00_JPRB, 0.85282E+00_JPRB, 0.88038E+00_JPRB, 0.80833E+00_JPRB, 0.71334E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.70154E+00_JPRB, 0.67738E+00_JPRB, 0.62717E+00_JPRB, 0.58551E+00_JPRB, 0.57929E+00_JPRB, 0.58478E+00_JPRB, 0.59499E+00_JPRB ,&
&  0.59822E+00_JPRB, 0.59832E+00_JPRB, 0.59481E+00_JPRB, 0.58808E+00_JPRB, 0.59315E+00_JPRB, 0.58188E+00_JPRB, 0.60536E+00_JPRB ,&
&  0.63003E+00_JPRB, 0.62924E+00_JPRB, 0.63837E+00_JPRB, 0.61725E+00_JPRB, 0.61606E+00_JPRB, 0.61216E+00_JPRB, 0.63755E+00_JPRB ,&
&  0.69677E+00_JPRB, 0.76622E+00_JPRB, 0.80732E+00_JPRB, 0.82074E+00_JPRB, 0.83776E+00_JPRB, 0.88414E+00_JPRB, 0.10643E+01_JPRB ,&
&  0.15460E+01_JPRB, 0.31992E+01_JPRB, 0.15899E+01_JPRB, 0.21904E+01_JPRB, 0.14556E+01_JPRB, 0.11057E+01_JPRB, 0.93041E+00_JPRB ,&
&  0.10004E+01_JPRB, 0.12123E+01_JPRB, 0.11274E+01_JPRB, 0.11272E+01_JPRB, 0.10742E+01_JPRB, 0.10366E+01_JPRB, 0.10291E+01_JPRB ,&
&  0.10742E+01_JPRB, 0.11982E+01_JPRB, 0.13203E+01_JPRB, 0.13753E+01_JPRB, 0.13431E+01_JPRB, 0.12730E+01_JPRB, 0.12560E+01_JPRB ,&
&  0.12497E+01_JPRB, 0.12677E+01_JPRB, 0.15287E+01_JPRB, 0.19339E+01_JPRB, 0.29287E+01_JPRB, 0.32953E+01_JPRB, 0.34036E+01_JPRB ,&
&  0.34979E+01_JPRB, 0.32502E+01_JPRB, 0.27499E+01_JPRB, 0.22922E+01_JPRB, 0.18983E+01_JPRB, 0.17973E+01_JPRB, 0.15416E+01_JPRB ,&
&  0.13790E+01_JPRB, 0.12680E+01_JPRB, 0.11666E+01_JPRB, 0.10706E+01_JPRB, 0.10216E+01_JPRB, 0.10095E+01_JPRB, 0.98514E+00_JPRB ,&
&  0.10207E+01_JPRB, 0.10650E+01_JPRB, 0.10525E+01_JPRB, 0.10247E+01_JPRB, 0.10124E+01_JPRB, 0.10157E+01_JPRB, 0.10146E+01_JPRB ,&
&  0.10261E+01_JPRB, 0.10542E+01_JPRB, 0.98593E+00_JPRB, 0.87964E+00_JPRB, 0.85529E+00_JPRB, 0.85128E+00_JPRB, 0.84142E+00_JPRB ,&
&  0.89192E+00_JPRB, 0.94137E+00_JPRB, 0.93944E+00_JPRB, 0.90741E+00_JPRB, 0.91034E+00_JPRB, 0.94606E+00_JPRB, 0.89077E+00_JPRB ,&
&  0.85504E+00_JPRB, 0.83321E+00_JPRB, 0.82014E+00_JPRB, 0.80026E+00_JPRB, 0.75897E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.70541E+00_JPRB, 0.66108E+00_JPRB, 0.60903E+00_JPRB, 0.58457E+00_JPRB, 0.58924E+00_JPRB, 0.57619E+00_JPRB, 0.58376E+00_JPRB ,&
&  0.60362E+00_JPRB, 0.60478E+00_JPRB, 0.60434E+00_JPRB, 0.59437E+00_JPRB, 0.58604E+00_JPRB, 0.57010E+00_JPRB, 0.55930E+00_JPRB ,&
&  0.57627E+00_JPRB, 0.62054E+00_JPRB, 0.63686E+00_JPRB, 0.62554E+00_JPRB, 0.61466E+00_JPRB, 0.59716E+00_JPRB, 0.59234E+00_JPRB ,&
&  0.61744E+00_JPRB, 0.66738E+00_JPRB, 0.71963E+00_JPRB, 0.74691E+00_JPRB, 0.76588E+00_JPRB, 0.79017E+00_JPRB, 0.90660E+00_JPRB ,&
&  0.12362E+01_JPRB, 0.30283E+01_JPRB, 0.13287E+01_JPRB, 0.22546E+01_JPRB, 0.16428E+01_JPRB, 0.12744E+01_JPRB, 0.99379E+00_JPRB ,&
&  0.97829E+00_JPRB, 0.10827E+01_JPRB, 0.10400E+01_JPRB, 0.10429E+01_JPRB, 0.10106E+01_JPRB, 0.98286E+00_JPRB, 0.98313E+00_JPRB ,&
&  0.98537E+00_JPRB, 0.99797E+00_JPRB, 0.10504E+01_JPRB, 0.10967E+01_JPRB, 0.11027E+01_JPRB, 0.10653E+01_JPRB, 0.10288E+01_JPRB ,&
&  0.10146E+01_JPRB, 0.10214E+01_JPRB, 0.11536E+01_JPRB, 0.14727E+01_JPRB, 0.21432E+01_JPRB, 0.30524E+01_JPRB, 0.40387E+01_JPRB ,&
&  0.44433E+01_JPRB, 0.41021E+01_JPRB, 0.33184E+01_JPRB, 0.27433E+01_JPRB, 0.23295E+01_JPRB, 0.21412E+01_JPRB, 0.18392E+01_JPRB ,&
&  0.15953E+01_JPRB, 0.14314E+01_JPRB, 0.12989E+01_JPRB, 0.12133E+01_JPRB, 0.11333E+01_JPRB, 0.11074E+01_JPRB, 0.10786E+01_JPRB ,&
&  0.10860E+01_JPRB, 0.11440E+01_JPRB, 0.11406E+01_JPRB, 0.10962E+01_JPRB, 0.10352E+01_JPRB, 0.10119E+01_JPRB, 0.98979E+00_JPRB ,&
&  0.98903E+00_JPRB, 0.10215E+01_JPRB, 0.99560E+00_JPRB, 0.91341E+00_JPRB, 0.87370E+00_JPRB, 0.87240E+00_JPRB, 0.87634E+00_JPRB ,&
&  0.93386E+00_JPRB, 0.97596E+00_JPRB, 0.96906E+00_JPRB, 0.95515E+00_JPRB, 0.93460E+00_JPRB, 0.97937E+00_JPRB, 0.91844E+00_JPRB ,&
&  0.85816E+00_JPRB, 0.82929E+00_JPRB, 0.80620E+00_JPRB, 0.76951E+00_JPRB, 0.75646E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.71155E+00_JPRB, 0.65616E+00_JPRB, 0.61879E+00_JPRB, 0.59235E+00_JPRB, 0.59394E+00_JPRB, 0.58797E+00_JPRB, 0.58437E+00_JPRB ,&
&  0.60165E+00_JPRB, 0.61199E+00_JPRB, 0.62443E+00_JPRB, 0.61366E+00_JPRB, 0.59207E+00_JPRB, 0.58636E+00_JPRB, 0.57597E+00_JPRB ,&
&  0.56956E+00_JPRB, 0.60142E+00_JPRB, 0.63265E+00_JPRB, 0.64240E+00_JPRB, 0.62962E+00_JPRB, 0.59951E+00_JPRB, 0.58428E+00_JPRB ,&
&  0.59056E+00_JPRB, 0.60871E+00_JPRB, 0.63779E+00_JPRB, 0.66894E+00_JPRB, 0.69502E+00_JPRB, 0.71873E+00_JPRB, 0.75732E+00_JPRB ,&
&  0.89106E+00_JPRB, 0.11469E+01_JPRB, 0.17218E+01_JPRB, 0.21367E+01_JPRB, 0.17299E+01_JPRB, 0.13549E+01_JPRB, 0.11108E+01_JPRB ,&
&  0.10600E+01_JPRB, 0.10129E+01_JPRB, 0.97964E+00_JPRB, 0.95336E+00_JPRB, 0.91842E+00_JPRB, 0.89543E+00_JPRB, 0.89842E+00_JPRB ,&
&  0.88536E+00_JPRB, 0.87200E+00_JPRB, 0.86003E+00_JPRB, 0.86139E+00_JPRB, 0.86559E+00_JPRB, 0.86108E+00_JPRB, 0.85009E+00_JPRB ,&
&  0.85025E+00_JPRB, 0.85909E+00_JPRB, 0.91471E+00_JPRB, 0.11465E+01_JPRB, 0.18271E+01_JPRB, 0.27307E+01_JPRB, 0.41014E+01_JPRB ,&
&  0.50503E+01_JPRB, 0.45536E+01_JPRB, 0.37897E+01_JPRB, 0.31722E+01_JPRB, 0.27288E+01_JPRB, 0.24813E+01_JPRB, 0.22235E+01_JPRB ,&
&  0.19858E+01_JPRB, 0.17449E+01_JPRB, 0.15461E+01_JPRB, 0.14211E+01_JPRB, 0.12860E+01_JPRB, 0.12222E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.11903E+01_JPRB, 0.11872E+01_JPRB, 0.11908E+01_JPRB, 0.12248E+01_JPRB, 0.11607E+01_JPRB, 0.10883E+01_JPRB, 0.10226E+01_JPRB ,&
&  0.97794E+00_JPRB, 0.98932E+00_JPRB, 0.97630E+00_JPRB, 0.94830E+00_JPRB, 0.93662E+00_JPRB, 0.93375E+00_JPRB, 0.93546E+00_JPRB ,&
&  0.94993E+00_JPRB, 0.97694E+00_JPRB, 0.98548E+00_JPRB, 0.98729E+00_JPRB, 0.92559E+00_JPRB, 0.99039E+00_JPRB, 0.93211E+00_JPRB ,&
&  0.87473E+00_JPRB, 0.83813E+00_JPRB, 0.81410E+00_JPRB, 0.78441E+00_JPRB, 0.77022E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.77329E+00_JPRB, 0.71720E+00_JPRB, 0.66550E+00_JPRB, 0.63350E+00_JPRB, 0.60802E+00_JPRB, 0.59791E+00_JPRB, 0.59172E+00_JPRB ,&
&  0.59966E+00_JPRB, 0.60727E+00_JPRB, 0.62203E+00_JPRB, 0.61953E+00_JPRB, 0.60030E+00_JPRB, 0.59911E+00_JPRB, 0.61331E+00_JPRB ,&
&  0.60965E+00_JPRB, 0.61594E+00_JPRB, 0.62821E+00_JPRB, 0.62018E+00_JPRB, 0.60549E+00_JPRB, 0.58908E+00_JPRB, 0.57402E+00_JPRB ,&
&  0.57717E+00_JPRB, 0.58329E+00_JPRB, 0.59676E+00_JPRB, 0.61831E+00_JPRB, 0.64697E+00_JPRB, 0.67541E+00_JPRB, 0.69701E+00_JPRB ,&
&  0.72528E+00_JPRB, 0.10913E+01_JPRB, 0.15424E+01_JPRB, 0.18381E+01_JPRB, 0.16967E+01_JPRB, 0.13629E+01_JPRB, 0.11928E+01_JPRB ,&
&  0.10514E+01_JPRB, 0.97748E+00_JPRB, 0.93077E+00_JPRB, 0.89899E+00_JPRB, 0.85456E+00_JPRB, 0.83167E+00_JPRB, 0.82865E+00_JPRB ,&
&  0.81284E+00_JPRB, 0.78175E+00_JPRB, 0.74799E+00_JPRB, 0.72247E+00_JPRB, 0.71652E+00_JPRB, 0.72261E+00_JPRB, 0.72998E+00_JPRB ,&
&  0.73766E+00_JPRB, 0.74950E+00_JPRB, 0.78564E+00_JPRB, 0.88260E+00_JPRB, 0.14072E+01_JPRB, 0.17326E+01_JPRB, 0.31573E+01_JPRB ,&
&  0.39100E+01_JPRB, 0.33856E+01_JPRB, 0.30463E+01_JPRB, 0.28063E+01_JPRB, 0.25523E+01_JPRB, 0.23232E+01_JPRB, 0.21799E+01_JPRB ,&
&  0.20328E+01_JPRB, 0.18625E+01_JPRB, 0.17186E+01_JPRB, 0.16158E+01_JPRB, 0.15155E+01_JPRB, 0.14407E+01_JPRB, 0.13447E+01_JPRB ,&
&  0.13054E+01_JPRB, 0.12548E+01_JPRB, 0.12335E+01_JPRB, 0.12573E+01_JPRB, 0.12722E+01_JPRB, 0.12075E+01_JPRB, 0.11294E+01_JPRB ,&
&  0.10463E+01_JPRB, 0.10051E+01_JPRB, 0.98157E+00_JPRB, 0.96445E+00_JPRB, 0.97593E+00_JPRB, 0.99256E+00_JPRB, 0.99375E+00_JPRB ,&
&  0.96782E+00_JPRB, 0.96518E+00_JPRB, 0.99083E+00_JPRB, 0.97857E+00_JPRB, 0.92912E+00_JPRB, 0.99985E+00_JPRB, 0.95078E+00_JPRB ,&
&  0.89291E+00_JPRB, 0.84074E+00_JPRB, 0.80785E+00_JPRB, 0.77624E+00_JPRB, 0.79860E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.81345E+00_JPRB, 0.77393E+00_JPRB, 0.72875E+00_JPRB, 0.69563E+00_JPRB, 0.66696E+00_JPRB, 0.64633E+00_JPRB, 0.64234E+00_JPRB ,&
&  0.63737E+00_JPRB, 0.62855E+00_JPRB, 0.63493E+00_JPRB, 0.63409E+00_JPRB, 0.63034E+00_JPRB, 0.62332E+00_JPRB, 0.63476E+00_JPRB ,&
&  0.64498E+00_JPRB, 0.64387E+00_JPRB, 0.63777E+00_JPRB, 0.62639E+00_JPRB, 0.61363E+00_JPRB, 0.59791E+00_JPRB, 0.58814E+00_JPRB ,&
&  0.58202E+00_JPRB, 0.58308E+00_JPRB, 0.59046E+00_JPRB, 0.60416E+00_JPRB, 0.62808E+00_JPRB, 0.65790E+00_JPRB, 0.68671E+00_JPRB ,&
&  0.68153E+00_JPRB, 0.79523E+00_JPRB, 0.10558E+01_JPRB, 0.14783E+01_JPRB, 0.15809E+01_JPRB, 0.13719E+01_JPRB, 0.11970E+01_JPRB ,&
&  0.10704E+01_JPRB, 0.98042E+00_JPRB, 0.91723E+00_JPRB, 0.87580E+00_JPRB, 0.83298E+00_JPRB, 0.81147E+00_JPRB, 0.79117E+00_JPRB ,&
&  0.76730E+00_JPRB, 0.73800E+00_JPRB, 0.70254E+00_JPRB, 0.68045E+00_JPRB, 0.67599E+00_JPRB, 0.68223E+00_JPRB, 0.68289E+00_JPRB ,&
&  0.68098E+00_JPRB, 0.69134E+00_JPRB, 0.70390E+00_JPRB, 0.73448E+00_JPRB, 0.97618E+00_JPRB, 0.12418E+01_JPRB, 0.18364E+01_JPRB ,&
&  0.18635E+01_JPRB, 0.17677E+01_JPRB, 0.17546E+01_JPRB, 0.17380E+01_JPRB, 0.17628E+01_JPRB, 0.17771E+01_JPRB, 0.18090E+01_JPRB ,&
&  0.18311E+01_JPRB, 0.18076E+01_JPRB, 0.17503E+01_JPRB, 0.17055E+01_JPRB, 0.16607E+01_JPRB, 0.16158E+01_JPRB, 0.15253E+01_JPRB ,&
&  0.14516E+01_JPRB, 0.13905E+01_JPRB, 0.13489E+01_JPRB, 0.13037E+01_JPRB, 0.12922E+01_JPRB, 0.12582E+01_JPRB, 0.11973E+01_JPRB ,&
&  0.11152E+01_JPRB, 0.10402E+01_JPRB, 0.10186E+01_JPRB, 0.10098E+01_JPRB, 0.99968E+00_JPRB, 0.98537E+00_JPRB, 0.98756E+00_JPRB ,&
&  0.99784E+00_JPRB, 0.10001E+01_JPRB, 0.99234E+00_JPRB, 0.97923E+00_JPRB, 0.93509E+00_JPRB, 0.99352E+00_JPRB, 0.96049E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.85893E+00_JPRB, 0.80673E+00_JPRB, 0.77843E+00_JPRB, 0.82542E+00_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.90102E+00_JPRB, 0.85692E+00_JPRB, 0.81235E+00_JPRB, 0.75775E+00_JPRB, 0.71514E+00_JPRB, 0.69689E+00_JPRB, 0.68602E+00_JPRB ,&
&  0.66941E+00_JPRB, 0.65334E+00_JPRB, 0.65271E+00_JPRB, 0.65808E+00_JPRB, 0.66141E+00_JPRB, 0.66144E+00_JPRB, 0.65409E+00_JPRB ,&
&  0.65202E+00_JPRB, 0.65605E+00_JPRB, 0.64965E+00_JPRB, 0.63838E+00_JPRB, 0.62597E+00_JPRB, 0.61562E+00_JPRB, 0.60775E+00_JPRB ,&
&  0.60368E+00_JPRB, 0.60706E+00_JPRB, 0.60951E+00_JPRB, 0.62058E+00_JPRB, 0.64032E+00_JPRB, 0.66314E+00_JPRB, 0.69165E+00_JPRB ,&
&  0.69258E+00_JPRB, 0.63278E+00_JPRB, 0.80465E+00_JPRB, 0.10363E+01_JPRB, 0.12174E+01_JPRB, 0.12398E+01_JPRB, 0.11562E+01_JPRB ,&
&  0.10891E+01_JPRB, 0.10052E+01_JPRB, 0.93859E+00_JPRB, 0.88524E+00_JPRB, 0.83801E+00_JPRB, 0.80004E+00_JPRB, 0.77231E+00_JPRB ,&
&  0.74177E+00_JPRB, 0.71677E+00_JPRB, 0.69262E+00_JPRB, 0.67095E+00_JPRB, 0.66705E+00_JPRB, 0.66405E+00_JPRB, 0.65996E+00_JPRB ,&
&  0.65784E+00_JPRB, 0.66008E+00_JPRB, 0.66448E+00_JPRB, 0.67423E+00_JPRB, 0.71012E+00_JPRB, 0.79544E+00_JPRB, 0.96516E+00_JPRB ,&
&  0.10551E+01_JPRB, 0.11074E+01_JPRB, 0.11227E+01_JPRB, 0.11228E+01_JPRB, 0.11467E+01_JPRB, 0.12023E+01_JPRB, 0.12908E+01_JPRB ,&
&  0.13985E+01_JPRB, 0.14896E+01_JPRB, 0.15564E+01_JPRB, 0.15979E+01_JPRB, 0.16000E+01_JPRB, 0.15759E+01_JPRB, 0.15319E+01_JPRB ,&
&  0.14842E+01_JPRB, 0.14117E+01_JPRB, 0.13410E+01_JPRB, 0.12844E+01_JPRB, 0.12493E+01_JPRB, 0.12245E+01_JPRB, 0.11853E+01_JPRB ,&
&  0.11293E+01_JPRB, 0.10662E+01_JPRB, 0.10266E+01_JPRB, 0.10086E+01_JPRB, 0.99955E+00_JPRB, 0.98567E+00_JPRB, 0.98494E+00_JPRB ,&
&  0.98359E+00_JPRB, 0.98790E+00_JPRB, 0.95183E+00_JPRB, 0.95564E+00_JPRB, 0.99736E+00_JPRB, 0.96696E+00_JPRB, 0.93929E+00_JPRB ,&
&  0.90210E+00_JPRB, 0.86295E+00_JPRB, 0.82503E+00_JPRB, 0.79345E+00_JPRB, 0.89046E+00_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.83786E+00_JPRB, 0.83496E+00_JPRB, 0.80516E+00_JPRB, 0.76303E+00_JPRB, 0.72340E+00_JPRB, 0.69131E+00_JPRB, 0.67521E+00_JPRB ,&
&  0.66284E+00_JPRB, 0.65433E+00_JPRB, 0.65953E+00_JPRB, 0.67187E+00_JPRB, 0.68643E+00_JPRB, 0.69079E+00_JPRB, 0.68620E+00_JPRB ,&
&  0.67518E+00_JPRB, 0.65787E+00_JPRB, 0.64712E+00_JPRB, 0.64086E+00_JPRB, 0.63165E+00_JPRB, 0.62776E+00_JPRB, 0.62621E+00_JPRB ,&
&  0.62968E+00_JPRB, 0.63514E+00_JPRB, 0.63843E+00_JPRB, 0.64775E+00_JPRB, 0.66298E+00_JPRB, 0.68589E+00_JPRB, 0.70941E+00_JPRB ,&
&  0.71111E+00_JPRB, 0.60965E+00_JPRB, 0.67657E+00_JPRB, 0.82513E+00_JPRB, 0.97622E+00_JPRB, 0.10569E+01_JPRB, 0.10686E+01_JPRB ,&
&  0.10409E+01_JPRB, 0.10019E+01_JPRB, 0.95259E+00_JPRB, 0.89687E+00_JPRB, 0.84222E+00_JPRB, 0.79437E+00_JPRB, 0.74981E+00_JPRB ,&
&  0.71631E+00_JPRB, 0.69253E+00_JPRB, 0.67916E+00_JPRB, 0.66994E+00_JPRB, 0.66295E+00_JPRB, 0.64835E+00_JPRB, 0.64046E+00_JPRB ,&
&  0.64197E+00_JPRB, 0.64513E+00_JPRB, 0.64990E+00_JPRB, 0.65628E+00_JPRB, 0.67717E+00_JPRB, 0.70818E+00_JPRB, 0.74134E+00_JPRB ,&
&  0.77487E+00_JPRB, 0.80871E+00_JPRB, 0.82838E+00_JPRB, 0.83508E+00_JPRB, 0.84005E+00_JPRB, 0.88609E+00_JPRB, 0.93067E+00_JPRB ,&
&  0.99300E+00_JPRB, 0.10904E+01_JPRB, 0.12052E+01_JPRB, 0.12765E+01_JPRB, 0.12888E+01_JPRB, 0.12587E+01_JPRB, 0.12071E+01_JPRB ,&
&  0.11713E+01_JPRB, 0.11313E+01_JPRB, 0.10910E+01_JPRB, 0.10585E+01_JPRB, 0.10404E+01_JPRB, 0.10397E+01_JPRB, 0.10399E+01_JPRB ,&
&  0.10353E+01_JPRB, 0.10185E+01_JPRB, 0.10010E+01_JPRB, 0.10011E+01_JPRB, 0.10055E+01_JPRB, 0.99951E+00_JPRB, 0.98497E+00_JPRB ,&
&  0.96640E+00_JPRB, 0.95010E+00_JPRB, 0.93708E+00_JPRB, 0.95361E+00_JPRB, 0.92830E+00_JPRB, 0.91188E+00_JPRB, 0.88758E+00_JPRB ,&
&  0.86499E+00_JPRB, 0.83877E+00_JPRB, 0.80421E+00_JPRB, 0.77422E+00_JPRB, 0.77047E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.74679E+00_JPRB, 0.78317E+00_JPRB, 0.78880E+00_JPRB, 0.76300E+00_JPRB, 0.73236E+00_JPRB, 0.71200E+00_JPRB, 0.69847E+00_JPRB ,&
&  0.69502E+00_JPRB, 0.69014E+00_JPRB, 0.69361E+00_JPRB, 0.70578E+00_JPRB, 0.71906E+00_JPRB, 0.72282E+00_JPRB, 0.70719E+00_JPRB ,&
&  0.68618E+00_JPRB, 0.66924E+00_JPRB, 0.66088E+00_JPRB, 0.65724E+00_JPRB, 0.65260E+00_JPRB, 0.65050E+00_JPRB, 0.65906E+00_JPRB ,&
&  0.66945E+00_JPRB, 0.68204E+00_JPRB, 0.69561E+00_JPRB, 0.70686E+00_JPRB, 0.71538E+00_JPRB, 0.72355E+00_JPRB, 0.74312E+00_JPRB ,&
&  0.75323E+00_JPRB, 0.58222E+00_JPRB, 0.60902E+00_JPRB, 0.72072E+00_JPRB, 0.81879E+00_JPRB, 0.93483E+00_JPRB, 0.99355E+00_JPRB ,&
&  0.99338E+00_JPRB, 0.96899E+00_JPRB, 0.93095E+00_JPRB, 0.88687E+00_JPRB, 0.83768E+00_JPRB, 0.79238E+00_JPRB, 0.74890E+00_JPRB ,&
&  0.71923E+00_JPRB, 0.69695E+00_JPRB, 0.68374E+00_JPRB, 0.66828E+00_JPRB, 0.65896E+00_JPRB, 0.65318E+00_JPRB, 0.64827E+00_JPRB ,&
&  0.64506E+00_JPRB, 0.64050E+00_JPRB, 0.64321E+00_JPRB, 0.65030E+00_JPRB, 0.65308E+00_JPRB, 0.66229E+00_JPRB, 0.66341E+00_JPRB ,&
&  0.68166E+00_JPRB, 0.70592E+00_JPRB, 0.71713E+00_JPRB, 0.72025E+00_JPRB, 0.72769E+00_JPRB, 0.74447E+00_JPRB, 0.76744E+00_JPRB ,&
&  0.79294E+00_JPRB, 0.84451E+00_JPRB, 0.89914E+00_JPRB, 0.95130E+00_JPRB, 0.98201E+00_JPRB, 0.98546E+00_JPRB, 0.97266E+00_JPRB ,&
&  0.97354E+00_JPRB, 0.96615E+00_JPRB, 0.95327E+00_JPRB, 0.93515E+00_JPRB, 0.91259E+00_JPRB, 0.90038E+00_JPRB, 0.90421E+00_JPRB ,&
&  0.91733E+00_JPRB, 0.93713E+00_JPRB, 0.94934E+00_JPRB, 0.96715E+00_JPRB, 0.96897E+00_JPRB, 0.95503E+00_JPRB, 0.94014E+00_JPRB ,&
&  0.92403E+00_JPRB, 0.90221E+00_JPRB, 0.88348E+00_JPRB, 0.82271E+00_JPRB, 0.86814E+00_JPRB, 0.86703E+00_JPRB, 0.85054E+00_JPRB ,&
&  0.83495E+00_JPRB, 0.80255E+00_JPRB, 0.78404E+00_JPRB, 0.63297E+00_JPRB, 0.70573E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.71685E+00_JPRB, 0.74881E+00_JPRB, 0.75959E+00_JPRB, 0.75219E+00_JPRB, 0.73395E+00_JPRB, 0.72391E+00_JPRB, 0.71749E+00_JPRB ,&
&  0.71356E+00_JPRB, 0.71016E+00_JPRB, 0.71972E+00_JPRB, 0.72744E+00_JPRB, 0.73088E+00_JPRB, 0.72412E+00_JPRB, 0.71636E+00_JPRB ,&
&  0.70330E+00_JPRB, 0.68675E+00_JPRB, 0.67490E+00_JPRB, 0.66475E+00_JPRB, 0.66389E+00_JPRB, 0.67213E+00_JPRB, 0.68855E+00_JPRB ,&
&  0.70970E+00_JPRB, 0.72465E+00_JPRB, 0.73692E+00_JPRB, 0.74980E+00_JPRB, 0.75266E+00_JPRB, 0.75337E+00_JPRB, 0.75709E+00_JPRB ,&
&  0.76394E+00_JPRB, 0.57932E+00_JPRB, 0.62026E+00_JPRB, 0.67637E+00_JPRB, 0.72824E+00_JPRB, 0.80509E+00_JPRB, 0.87025E+00_JPRB ,&
&  0.89414E+00_JPRB, 0.86909E+00_JPRB, 0.84869E+00_JPRB, 0.82296E+00_JPRB, 0.79232E+00_JPRB, 0.76469E+00_JPRB, 0.73329E+00_JPRB ,&
&  0.70972E+00_JPRB, 0.69276E+00_JPRB, 0.68530E+00_JPRB, 0.67890E+00_JPRB, 0.67173E+00_JPRB, 0.66210E+00_JPRB, 0.65056E+00_JPRB ,&
&  0.63895E+00_JPRB, 0.63355E+00_JPRB, 0.63083E+00_JPRB, 0.62857E+00_JPRB, 0.63154E+00_JPRB, 0.63697E+00_JPRB, 0.64074E+00_JPRB ,&
&  0.65057E+00_JPRB, 0.65992E+00_JPRB, 0.66456E+00_JPRB, 0.66494E+00_JPRB, 0.66533E+00_JPRB, 0.66903E+00_JPRB, 0.68670E+00_JPRB ,&
&  0.69971E+00_JPRB, 0.72158E+00_JPRB, 0.74951E+00_JPRB, 0.79110E+00_JPRB, 0.82559E+00_JPRB, 0.83957E+00_JPRB, 0.84434E+00_JPRB ,&
&  0.84539E+00_JPRB, 0.83657E+00_JPRB, 0.81925E+00_JPRB, 0.80153E+00_JPRB, 0.78843E+00_JPRB, 0.78452E+00_JPRB, 0.78142E+00_JPRB ,&
&  0.78725E+00_JPRB, 0.79925E+00_JPRB, 0.80926E+00_JPRB, 0.82528E+00_JPRB, 0.84320E+00_JPRB, 0.84715E+00_JPRB, 0.82853E+00_JPRB ,&
&  0.81197E+00_JPRB, 0.80400E+00_JPRB, 0.79721E+00_JPRB, 0.81124E+00_JPRB, 0.81211E+00_JPRB, 0.80536E+00_JPRB, 0.78685E+00_JPRB ,&
&  0.77269E+00_JPRB, 0.76140E+00_JPRB, 0.57771E+00_JPRB, 0.66166E+00_JPRB, 0.68357E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.72644E+00_JPRB, 0.72295E+00_JPRB, 0.71692E+00_JPRB, 0.71994E+00_JPRB, 0.72406E+00_JPRB, 0.72401E+00_JPRB, 0.72446E+00_JPRB ,&
&  0.72317E+00_JPRB, 0.72068E+00_JPRB, 0.72969E+00_JPRB, 0.73783E+00_JPRB, 0.73959E+00_JPRB, 0.73436E+00_JPRB, 0.72916E+00_JPRB ,&
&  0.72548E+00_JPRB, 0.71840E+00_JPRB, 0.71104E+00_JPRB, 0.70871E+00_JPRB, 0.71209E+00_JPRB, 0.72658E+00_JPRB, 0.74822E+00_JPRB ,&
&  0.76437E+00_JPRB, 0.76395E+00_JPRB, 0.76774E+00_JPRB, 0.77890E+00_JPRB, 0.78149E+00_JPRB, 0.78316E+00_JPRB, 0.79025E+00_JPRB ,&
&  0.67225E+00_JPRB, 0.54698E+00_JPRB, 0.60983E+00_JPRB, 0.66593E+00_JPRB, 0.70817E+00_JPRB, 0.75664E+00_JPRB, 0.81166E+00_JPRB ,&
&  0.82876E+00_JPRB, 0.81638E+00_JPRB, 0.80386E+00_JPRB, 0.78745E+00_JPRB, 0.76628E+00_JPRB, 0.75227E+00_JPRB, 0.74125E+00_JPRB ,&
&  0.73925E+00_JPRB, 0.72903E+00_JPRB, 0.71298E+00_JPRB, 0.70001E+00_JPRB, 0.68951E+00_JPRB, 0.67595E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.64879E+00_JPRB, 0.63694E+00_JPRB, 0.63397E+00_JPRB, 0.63305E+00_JPRB, 0.63629E+00_JPRB, 0.63696E+00_JPRB, 0.63256E+00_JPRB ,&
&  0.62954E+00_JPRB, 0.63186E+00_JPRB, 0.64149E+00_JPRB, 0.65025E+00_JPRB, 0.65353E+00_JPRB, 0.65280E+00_JPRB, 0.66206E+00_JPRB ,&
&  0.66922E+00_JPRB, 0.68237E+00_JPRB, 0.69911E+00_JPRB, 0.72624E+00_JPRB, 0.74730E+00_JPRB, 0.75464E+00_JPRB, 0.75859E+00_JPRB ,&
&  0.76533E+00_JPRB, 0.76001E+00_JPRB, 0.74399E+00_JPRB, 0.73239E+00_JPRB, 0.72688E+00_JPRB, 0.71943E+00_JPRB, 0.71655E+00_JPRB ,&
&  0.72251E+00_JPRB, 0.72766E+00_JPRB, 0.72658E+00_JPRB, 0.73253E+00_JPRB, 0.74338E+00_JPRB, 0.75509E+00_JPRB, 0.76360E+00_JPRB ,&
&  0.75039E+00_JPRB, 0.75387E+00_JPRB, 0.75309E+00_JPRB, 0.76092E+00_JPRB, 0.76973E+00_JPRB, 0.75811E+00_JPRB, 0.74721E+00_JPRB ,&
&  0.73716E+00_JPRB, 0.71531E+00_JPRB, 0.71512E+00_JPRB, 0.71547E+00_JPRB, 0.71755E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.69627E+00_JPRB, 0.68776E+00_JPRB, 0.68187E+00_JPRB, 0.69012E+00_JPRB, 0.70513E+00_JPRB, 0.71706E+00_JPRB, 0.71833E+00_JPRB ,&
&  0.71938E+00_JPRB, 0.72493E+00_JPRB, 0.73388E+00_JPRB, 0.73666E+00_JPRB, 0.73035E+00_JPRB, 0.71449E+00_JPRB, 0.70685E+00_JPRB ,&
&  0.70575E+00_JPRB, 0.70782E+00_JPRB, 0.70864E+00_JPRB, 0.71375E+00_JPRB, 0.73403E+00_JPRB, 0.76524E+00_JPRB, 0.79474E+00_JPRB ,&
&  0.80938E+00_JPRB, 0.80416E+00_JPRB, 0.79807E+00_JPRB, 0.79298E+00_JPRB, 0.79167E+00_JPRB, 0.79643E+00_JPRB, 0.79741E+00_JPRB ,&
&  0.78910E+00_JPRB, 0.64283E+00_JPRB, 0.69758E+00_JPRB, 0.71750E+00_JPRB, 0.74033E+00_JPRB, 0.76765E+00_JPRB, 0.80083E+00_JPRB ,&
&  0.80486E+00_JPRB, 0.79571E+00_JPRB, 0.77743E+00_JPRB, 0.76470E+00_JPRB, 0.74988E+00_JPRB, 0.74600E+00_JPRB, 0.74387E+00_JPRB ,&
&  0.74234E+00_JPRB, 0.73952E+00_JPRB, 0.71767E+00_JPRB, 0.68253E+00_JPRB, 0.66350E+00_JPRB, 0.65248E+00_JPRB, 0.64350E+00_JPRB ,&
&  0.63525E+00_JPRB, 0.62378E+00_JPRB, 0.61250E+00_JPRB, 0.60309E+00_JPRB, 0.60504E+00_JPRB, 0.60373E+00_JPRB, 0.59164E+00_JPRB ,&
&  0.58696E+00_JPRB, 0.58538E+00_JPRB, 0.59341E+00_JPRB, 0.60417E+00_JPRB, 0.60865E+00_JPRB, 0.60418E+00_JPRB, 0.61335E+00_JPRB ,&
&  0.63455E+00_JPRB, 0.64843E+00_JPRB, 0.66045E+00_JPRB, 0.67563E+00_JPRB, 0.68345E+00_JPRB, 0.68332E+00_JPRB, 0.68499E+00_JPRB ,&
&  0.69211E+00_JPRB, 0.69109E+00_JPRB, 0.68336E+00_JPRB, 0.67739E+00_JPRB, 0.67891E+00_JPRB, 0.68005E+00_JPRB, 0.69000E+00_JPRB ,&
&  0.69932E+00_JPRB, 0.70277E+00_JPRB, 0.69928E+00_JPRB, 0.70223E+00_JPRB, 0.70840E+00_JPRB, 0.71838E+00_JPRB, 0.72466E+00_JPRB ,&
&  0.73104E+00_JPRB, 0.72740E+00_JPRB, 0.71448E+00_JPRB, 0.71400E+00_JPRB, 0.71812E+00_JPRB, 0.71471E+00_JPRB, 0.70213E+00_JPRB ,&
&  0.69149E+00_JPRB, 0.68299E+00_JPRB, 0.68541E+00_JPRB, 0.68725E+00_JPRB, 0.69242E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.69378E+00_JPRB, 0.68994E+00_JPRB, 0.68861E+00_JPRB, 0.69928E+00_JPRB, 0.71658E+00_JPRB, 0.72094E+00_JPRB, 0.72266E+00_JPRB ,&
&  0.73521E+00_JPRB, 0.74472E+00_JPRB, 0.74450E+00_JPRB, 0.74320E+00_JPRB, 0.73745E+00_JPRB, 0.72397E+00_JPRB, 0.71378E+00_JPRB ,&
&  0.71950E+00_JPRB, 0.72560E+00_JPRB, 0.72846E+00_JPRB, 0.75303E+00_JPRB, 0.78531E+00_JPRB, 0.82345E+00_JPRB, 0.84939E+00_JPRB ,&
&  0.85874E+00_JPRB, 0.85156E+00_JPRB, 0.84786E+00_JPRB, 0.84247E+00_JPRB, 0.83704E+00_JPRB, 0.83867E+00_JPRB, 0.83312E+00_JPRB ,&
&  0.82318E+00_JPRB, 0.72912E+00_JPRB, 0.75812E+00_JPRB, 0.76727E+00_JPRB, 0.78247E+00_JPRB, 0.80580E+00_JPRB, 0.83183E+00_JPRB ,&
&  0.83081E+00_JPRB, 0.82637E+00_JPRB, 0.81715E+00_JPRB, 0.79773E+00_JPRB, 0.78061E+00_JPRB, 0.77070E+00_JPRB, 0.75416E+00_JPRB ,&
&  0.72705E+00_JPRB, 0.69024E+00_JPRB, 0.65756E+00_JPRB, 0.63541E+00_JPRB, 0.62552E+00_JPRB, 0.62337E+00_JPRB, 0.61962E+00_JPRB ,&
&  0.61848E+00_JPRB, 0.61770E+00_JPRB, 0.60780E+00_JPRB, 0.59886E+00_JPRB, 0.58972E+00_JPRB, 0.58391E+00_JPRB, 0.57149E+00_JPRB ,&
&  0.56356E+00_JPRB, 0.55299E+00_JPRB, 0.55257E+00_JPRB, 0.57259E+00_JPRB, 0.59345E+00_JPRB, 0.59792E+00_JPRB, 0.61060E+00_JPRB ,&
&  0.62763E+00_JPRB, 0.63466E+00_JPRB, 0.63594E+00_JPRB, 0.63295E+00_JPRB, 0.62548E+00_JPRB, 0.62492E+00_JPRB, 0.63286E+00_JPRB ,&
&  0.64005E+00_JPRB, 0.65008E+00_JPRB, 0.65365E+00_JPRB, 0.65836E+00_JPRB, 0.66515E+00_JPRB, 0.67035E+00_JPRB, 0.68132E+00_JPRB ,&
&  0.68694E+00_JPRB, 0.69097E+00_JPRB, 0.70229E+00_JPRB, 0.71374E+00_JPRB, 0.71860E+00_JPRB, 0.72088E+00_JPRB, 0.72989E+00_JPRB ,&
&  0.73324E+00_JPRB, 0.73108E+00_JPRB, 0.72237E+00_JPRB, 0.70975E+00_JPRB, 0.70055E+00_JPRB, 0.68705E+00_JPRB, 0.66723E+00_JPRB ,&
&  0.65034E+00_JPRB, 0.64847E+00_JPRB, 0.65784E+00_JPRB, 0.66786E+00_JPRB, 0.68476E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.68654E+00_JPRB, 0.69792E+00_JPRB, 0.71093E+00_JPRB, 0.72364E+00_JPRB, 0.73692E+00_JPRB, 0.73677E+00_JPRB, 0.73828E+00_JPRB ,&
&  0.75767E+00_JPRB, 0.76084E+00_JPRB, 0.76030E+00_JPRB, 0.75182E+00_JPRB, 0.74564E+00_JPRB, 0.74566E+00_JPRB, 0.73690E+00_JPRB ,&
&  0.73261E+00_JPRB, 0.73917E+00_JPRB, 0.75786E+00_JPRB, 0.78633E+00_JPRB, 0.81413E+00_JPRB, 0.85049E+00_JPRB, 0.87740E+00_JPRB ,&
&  0.88181E+00_JPRB, 0.87116E+00_JPRB, 0.87127E+00_JPRB, 0.87442E+00_JPRB, 0.87453E+00_JPRB, 0.85751E+00_JPRB, 0.85291E+00_JPRB ,&
&  0.83280E+00_JPRB, 0.83838E+00_JPRB, 0.70842E+00_JPRB, 0.77776E+00_JPRB, 0.82137E+00_JPRB, 0.85494E+00_JPRB, 0.86359E+00_JPRB ,&
&  0.85543E+00_JPRB, 0.84289E+00_JPRB, 0.82044E+00_JPRB, 0.79488E+00_JPRB, 0.78563E+00_JPRB, 0.76532E+00_JPRB, 0.74231E+00_JPRB ,&
&  0.69378E+00_JPRB, 0.63186E+00_JPRB, 0.60029E+00_JPRB, 0.58889E+00_JPRB, 0.58346E+00_JPRB, 0.58510E+00_JPRB, 0.59088E+00_JPRB ,&
&  0.59500E+00_JPRB, 0.59596E+00_JPRB, 0.58539E+00_JPRB, 0.56945E+00_JPRB, 0.55064E+00_JPRB, 0.54077E+00_JPRB, 0.53832E+00_JPRB ,&
&  0.53492E+00_JPRB, 0.52222E+00_JPRB, 0.52560E+00_JPRB, 0.55564E+00_JPRB, 0.58077E+00_JPRB, 0.59830E+00_JPRB, 0.60565E+00_JPRB ,&
&  0.61074E+00_JPRB, 0.60429E+00_JPRB, 0.60035E+00_JPRB, 0.59134E+00_JPRB, 0.56822E+00_JPRB, 0.56206E+00_JPRB, 0.58314E+00_JPRB ,&
&  0.60699E+00_JPRB, 0.62061E+00_JPRB, 0.62870E+00_JPRB, 0.64093E+00_JPRB, 0.65384E+00_JPRB, 0.66108E+00_JPRB, 0.66729E+00_JPRB ,&
&  0.67188E+00_JPRB, 0.67641E+00_JPRB, 0.68983E+00_JPRB, 0.70475E+00_JPRB, 0.71162E+00_JPRB, 0.71270E+00_JPRB, 0.71258E+00_JPRB ,&
&  0.70958E+00_JPRB, 0.70188E+00_JPRB, 0.69295E+00_JPRB, 0.68440E+00_JPRB, 0.66424E+00_JPRB, 0.64005E+00_JPRB, 0.62209E+00_JPRB ,&
&  0.61309E+00_JPRB, 0.62320E+00_JPRB, 0.63086E+00_JPRB, 0.64336E+00_JPRB, 0.66987E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.72854E+00_JPRB, 0.74237E+00_JPRB, 0.75379E+00_JPRB, 0.77485E+00_JPRB, 0.79415E+00_JPRB, 0.79165E+00_JPRB, 0.78475E+00_JPRB ,&
&  0.77987E+00_JPRB, 0.77539E+00_JPRB, 0.76833E+00_JPRB, 0.75882E+00_JPRB, 0.75240E+00_JPRB, 0.75189E+00_JPRB, 0.75262E+00_JPRB ,&
&  0.75423E+00_JPRB, 0.77438E+00_JPRB, 0.80751E+00_JPRB, 0.83359E+00_JPRB, 0.86208E+00_JPRB, 0.90563E+00_JPRB, 0.92634E+00_JPRB ,&
&  0.91004E+00_JPRB, 0.90928E+00_JPRB, 0.89843E+00_JPRB, 0.88381E+00_JPRB, 0.88554E+00_JPRB, 0.87612E+00_JPRB, 0.84777E+00_JPRB ,&
&  0.84110E+00_JPRB, 0.83344E+00_JPRB, 0.82929E+00_JPRB, 0.85712E+00_JPRB, 0.87866E+00_JPRB, 0.87797E+00_JPRB, 0.87366E+00_JPRB ,&
&  0.85187E+00_JPRB, 0.81499E+00_JPRB, 0.78081E+00_JPRB, 0.76460E+00_JPRB, 0.76289E+00_JPRB, 0.74276E+00_JPRB, 0.71467E+00_JPRB ,&
&  0.65717E+00_JPRB, 0.61003E+00_JPRB, 0.58404E+00_JPRB, 0.56869E+00_JPRB, 0.56388E+00_JPRB, 0.56145E+00_JPRB, 0.55734E+00_JPRB ,&
&  0.56298E+00_JPRB, 0.55821E+00_JPRB, 0.53106E+00_JPRB, 0.50269E+00_JPRB, 0.49825E+00_JPRB, 0.50730E+00_JPRB, 0.52722E+00_JPRB ,&
&  0.54236E+00_JPRB, 0.53798E+00_JPRB, 0.54500E+00_JPRB, 0.55290E+00_JPRB, 0.57019E+00_JPRB, 0.58703E+00_JPRB, 0.58787E+00_JPRB ,&
&  0.58756E+00_JPRB, 0.58214E+00_JPRB, 0.56890E+00_JPRB, 0.56029E+00_JPRB, 0.54907E+00_JPRB, 0.54470E+00_JPRB, 0.55622E+00_JPRB ,&
&  0.57373E+00_JPRB, 0.58351E+00_JPRB, 0.59441E+00_JPRB, 0.60802E+00_JPRB, 0.61898E+00_JPRB, 0.62728E+00_JPRB, 0.63387E+00_JPRB ,&
&  0.64404E+00_JPRB, 0.65866E+00_JPRB, 0.67356E+00_JPRB, 0.68658E+00_JPRB, 0.68654E+00_JPRB, 0.67228E+00_JPRB, 0.64679E+00_JPRB ,&
&  0.62977E+00_JPRB, 0.61643E+00_JPRB, 0.61321E+00_JPRB, 0.61869E+00_JPRB, 0.61383E+00_JPRB, 0.59433E+00_JPRB, 0.58455E+00_JPRB ,&
&  0.59534E+00_JPRB, 0.61921E+00_JPRB, 0.64199E+00_JPRB, 0.67288E+00_JPRB, 0.70525E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.76738E+00_JPRB, 0.79886E+00_JPRB, 0.82314E+00_JPRB, 0.83204E+00_JPRB, 0.83007E+00_JPRB, 0.82150E+00_JPRB, 0.79909E+00_JPRB ,&
&  0.77474E+00_JPRB, 0.75752E+00_JPRB, 0.73919E+00_JPRB, 0.71275E+00_JPRB, 0.69082E+00_JPRB, 0.68629E+00_JPRB, 0.69890E+00_JPRB ,&
&  0.74516E+00_JPRB, 0.78626E+00_JPRB, 0.80302E+00_JPRB, 0.81691E+00_JPRB, 0.86701E+00_JPRB, 0.90911E+00_JPRB, 0.91848E+00_JPRB ,&
&  0.91372E+00_JPRB, 0.91552E+00_JPRB, 0.87904E+00_JPRB, 0.84578E+00_JPRB, 0.84572E+00_JPRB, 0.85082E+00_JPRB, 0.83220E+00_JPRB ,&
&  0.85759E+00_JPRB, 0.87448E+00_JPRB, 0.86804E+00_JPRB, 0.84263E+00_JPRB, 0.83480E+00_JPRB, 0.82426E+00_JPRB, 0.81239E+00_JPRB ,&
&  0.79054E+00_JPRB, 0.74246E+00_JPRB, 0.71284E+00_JPRB, 0.71148E+00_JPRB, 0.70441E+00_JPRB, 0.69703E+00_JPRB, 0.68337E+00_JPRB ,&
&  0.64328E+00_JPRB, 0.60744E+00_JPRB, 0.58058E+00_JPRB, 0.56214E+00_JPRB, 0.55063E+00_JPRB, 0.54173E+00_JPRB, 0.53248E+00_JPRB ,&
&  0.52452E+00_JPRB, 0.49767E+00_JPRB, 0.47004E+00_JPRB, 0.46440E+00_JPRB, 0.47058E+00_JPRB, 0.47796E+00_JPRB, 0.50541E+00_JPRB ,&
&  0.53044E+00_JPRB, 0.52883E+00_JPRB, 0.52525E+00_JPRB, 0.52773E+00_JPRB, 0.53656E+00_JPRB, 0.53676E+00_JPRB, 0.52640E+00_JPRB ,&
&  0.52114E+00_JPRB, 0.51857E+00_JPRB, 0.50901E+00_JPRB, 0.50191E+00_JPRB, 0.50121E+00_JPRB, 0.50520E+00_JPRB, 0.51965E+00_JPRB ,&
&  0.52859E+00_JPRB, 0.52995E+00_JPRB, 0.54354E+00_JPRB, 0.56071E+00_JPRB, 0.57101E+00_JPRB, 0.57912E+00_JPRB, 0.58493E+00_JPRB ,&
&  0.59844E+00_JPRB, 0.61795E+00_JPRB, 0.63300E+00_JPRB, 0.63507E+00_JPRB, 0.61206E+00_JPRB, 0.58772E+00_JPRB, 0.56860E+00_JPRB ,&
&  0.56031E+00_JPRB, 0.56481E+00_JPRB, 0.57246E+00_JPRB, 0.57982E+00_JPRB, 0.58652E+00_JPRB, 0.59099E+00_JPRB, 0.59030E+00_JPRB ,&
&  0.60986E+00_JPRB, 0.64097E+00_JPRB, 0.66611E+00_JPRB, 0.69607E+00_JPRB, 0.72729E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.83470E+00_JPRB, 0.85229E+00_JPRB, 0.86415E+00_JPRB, 0.86496E+00_JPRB, 0.84949E+00_JPRB, 0.82281E+00_JPRB, 0.79995E+00_JPRB ,&
&  0.77392E+00_JPRB, 0.74864E+00_JPRB, 0.72462E+00_JPRB, 0.69191E+00_JPRB, 0.67432E+00_JPRB, 0.67780E+00_JPRB, 0.72688E+00_JPRB ,&
&  0.79226E+00_JPRB, 0.79724E+00_JPRB, 0.79139E+00_JPRB, 0.81688E+00_JPRB, 0.86960E+00_JPRB, 0.90159E+00_JPRB, 0.92261E+00_JPRB ,&
&  0.92552E+00_JPRB, 0.89256E+00_JPRB, 0.85942E+00_JPRB, 0.84595E+00_JPRB, 0.86919E+00_JPRB, 0.88663E+00_JPRB, 0.89139E+00_JPRB ,&
&  0.88674E+00_JPRB, 0.86537E+00_JPRB, 0.84611E+00_JPRB, 0.82971E+00_JPRB, 0.83355E+00_JPRB, 0.80844E+00_JPRB, 0.75610E+00_JPRB ,&
&  0.68971E+00_JPRB, 0.66332E+00_JPRB, 0.65211E+00_JPRB, 0.63785E+00_JPRB, 0.62703E+00_JPRB, 0.62723E+00_JPRB, 0.61810E+00_JPRB ,&
&  0.60458E+00_JPRB, 0.58897E+00_JPRB, 0.57407E+00_JPRB, 0.55646E+00_JPRB, 0.54265E+00_JPRB, 0.52306E+00_JPRB, 0.50472E+00_JPRB ,&
&  0.48772E+00_JPRB, 0.46675E+00_JPRB, 0.44755E+00_JPRB, 0.43424E+00_JPRB, 0.42852E+00_JPRB, 0.43912E+00_JPRB, 0.47933E+00_JPRB ,&
&  0.51072E+00_JPRB, 0.50770E+00_JPRB, 0.50529E+00_JPRB, 0.50923E+00_JPRB, 0.50202E+00_JPRB, 0.49142E+00_JPRB, 0.48556E+00_JPRB ,&
&  0.48031E+00_JPRB, 0.47617E+00_JPRB, 0.47428E+00_JPRB, 0.47603E+00_JPRB, 0.47817E+00_JPRB, 0.47636E+00_JPRB, 0.47878E+00_JPRB ,&
&  0.48343E+00_JPRB, 0.48870E+00_JPRB, 0.50574E+00_JPRB, 0.53018E+00_JPRB, 0.55083E+00_JPRB, 0.56546E+00_JPRB, 0.57902E+00_JPRB ,&
&  0.59998E+00_JPRB, 0.61281E+00_JPRB, 0.60019E+00_JPRB, 0.58817E+00_JPRB, 0.58192E+00_JPRB, 0.58705E+00_JPRB, 0.59759E+00_JPRB ,&
&  0.61825E+00_JPRB, 0.63114E+00_JPRB, 0.63728E+00_JPRB, 0.63880E+00_JPRB, 0.64203E+00_JPRB, 0.64291E+00_JPRB, 0.65544E+00_JPRB ,&
&  0.68917E+00_JPRB, 0.71828E+00_JPRB, 0.72560E+00_JPRB, 0.75815E+00_JPRB, 0.79985E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.87442E+00_JPRB, 0.87868E+00_JPRB, 0.86891E+00_JPRB, 0.86457E+00_JPRB, 0.85401E+00_JPRB, 0.84043E+00_JPRB, 0.81811E+00_JPRB ,&
&  0.79187E+00_JPRB, 0.76377E+00_JPRB, 0.73723E+00_JPRB, 0.71551E+00_JPRB, 0.71955E+00_JPRB, 0.75227E+00_JPRB, 0.77989E+00_JPRB ,&
&  0.76904E+00_JPRB, 0.76528E+00_JPRB, 0.78254E+00_JPRB, 0.81332E+00_JPRB, 0.84675E+00_JPRB, 0.87758E+00_JPRB, 0.89969E+00_JPRB ,&
&  0.91762E+00_JPRB, 0.89942E+00_JPRB, 0.90312E+00_JPRB, 0.90513E+00_JPRB, 0.89888E+00_JPRB, 0.88068E+00_JPRB, 0.85172E+00_JPRB ,&
&  0.82306E+00_JPRB, 0.79587E+00_JPRB, 0.76222E+00_JPRB, 0.70332E+00_JPRB, 0.77642E+00_JPRB, 0.69976E+00_JPRB, 0.67117E+00_JPRB ,&
&  0.64385E+00_JPRB, 0.62569E+00_JPRB, 0.60536E+00_JPRB, 0.58333E+00_JPRB, 0.57580E+00_JPRB, 0.57125E+00_JPRB, 0.56338E+00_JPRB ,&
&  0.55688E+00_JPRB, 0.54986E+00_JPRB, 0.53919E+00_JPRB, 0.52555E+00_JPRB, 0.51272E+00_JPRB, 0.49896E+00_JPRB, 0.48631E+00_JPRB ,&
&  0.46800E+00_JPRB, 0.44119E+00_JPRB, 0.41299E+00_JPRB, 0.39232E+00_JPRB, 0.38164E+00_JPRB, 0.39100E+00_JPRB, 0.42383E+00_JPRB ,&
&  0.45752E+00_JPRB, 0.46313E+00_JPRB, 0.47273E+00_JPRB, 0.47535E+00_JPRB, 0.46661E+00_JPRB, 0.45634E+00_JPRB, 0.43855E+00_JPRB ,&
&  0.42497E+00_JPRB, 0.42260E+00_JPRB, 0.42342E+00_JPRB, 0.42713E+00_JPRB, 0.43231E+00_JPRB, 0.43958E+00_JPRB, 0.44974E+00_JPRB ,&
&  0.45909E+00_JPRB, 0.46239E+00_JPRB, 0.47468E+00_JPRB, 0.50142E+00_JPRB, 0.53230E+00_JPRB, 0.56357E+00_JPRB, 0.59029E+00_JPRB ,&
&  0.60838E+00_JPRB, 0.62099E+00_JPRB, 0.62925E+00_JPRB, 0.64441E+00_JPRB, 0.66007E+00_JPRB, 0.67585E+00_JPRB, 0.68406E+00_JPRB ,&
&  0.68172E+00_JPRB, 0.68535E+00_JPRB, 0.70245E+00_JPRB, 0.73063E+00_JPRB, 0.75036E+00_JPRB, 0.78413E+00_JPRB, 0.80519E+00_JPRB ,&
&  0.82753E+00_JPRB, 0.83321E+00_JPRB, 0.82774E+00_JPRB, 0.84021E+00_JPRB, 0.85694E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.86034E+00_JPRB, 0.87085E+00_JPRB, 0.86863E+00_JPRB, 0.86753E+00_JPRB, 0.86929E+00_JPRB, 0.86231E+00_JPRB, 0.84830E+00_JPRB ,&
&  0.83421E+00_JPRB, 0.80775E+00_JPRB, 0.77741E+00_JPRB, 0.76383E+00_JPRB, 0.77138E+00_JPRB, 0.77130E+00_JPRB, 0.75272E+00_JPRB ,&
&  0.75113E+00_JPRB, 0.76507E+00_JPRB, 0.78819E+00_JPRB, 0.81389E+00_JPRB, 0.84845E+00_JPRB, 0.88426E+00_JPRB, 0.91101E+00_JPRB ,&
&  0.92156E+00_JPRB, 0.91960E+00_JPRB, 0.91190E+00_JPRB, 0.88516E+00_JPRB, 0.84939E+00_JPRB, 0.82186E+00_JPRB, 0.80098E+00_JPRB ,&
&  0.77522E+00_JPRB, 0.72740E+00_JPRB, 0.65800E+00_JPRB, 0.65238E+00_JPRB, 0.58859E+00_JPRB, 0.56852E+00_JPRB, 0.59075E+00_JPRB ,&
&  0.60609E+00_JPRB, 0.58816E+00_JPRB, 0.56436E+00_JPRB, 0.54464E+00_JPRB, 0.54091E+00_JPRB, 0.53958E+00_JPRB, 0.53452E+00_JPRB ,&
&  0.53069E+00_JPRB, 0.52398E+00_JPRB, 0.50993E+00_JPRB, 0.49702E+00_JPRB, 0.48484E+00_JPRB, 0.47266E+00_JPRB, 0.46434E+00_JPRB ,&
&  0.43623E+00_JPRB, 0.39324E+00_JPRB, 0.36124E+00_JPRB, 0.34738E+00_JPRB, 0.33973E+00_JPRB, 0.34344E+00_JPRB, 0.35619E+00_JPRB ,&
&  0.36646E+00_JPRB, 0.36699E+00_JPRB, 0.36133E+00_JPRB, 0.34871E+00_JPRB, 0.32930E+00_JPRB, 0.28392E+00_JPRB, 0.25788E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.30706E+00_JPRB, 0.34196E+00_JPRB, 0.35118E+00_JPRB, 0.36058E+00_JPRB, 0.36293E+00_JPRB, 0.34830E+00_JPRB ,&
&  0.32012E+00_JPRB, 0.28981E+00_JPRB, 0.28382E+00_JPRB, 0.28595E+00_JPRB, 0.29049E+00_JPRB, 0.29889E+00_JPRB, 0.31467E+00_JPRB ,&
&  0.33857E+00_JPRB, 0.37708E+00_JPRB, 0.39923E+00_JPRB, 0.40193E+00_JPRB, 0.39466E+00_JPRB, 0.38319E+00_JPRB, 0.38113E+00_JPRB ,&
&  0.39503E+00_JPRB, 0.45571E+00_JPRB, 0.57751E+00_JPRB, 0.72477E+00_JPRB, 0.72430E+00_JPRB, 0.76834E+00_JPRB, 0.80304E+00_JPRB ,&
&  0.83272E+00_JPRB, 0.84601E+00_JPRB, 0.83265E+00_JPRB, 0.83385E+00_JPRB, 0.84039E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.82074E+00_JPRB, 0.84071E+00_JPRB, 0.85170E+00_JPRB, 0.86574E+00_JPRB, 0.86602E+00_JPRB, 0.83591E+00_JPRB, 0.82139E+00_JPRB ,&
&  0.82612E+00_JPRB, 0.82129E+00_JPRB, 0.81048E+00_JPRB, 0.80376E+00_JPRB, 0.79487E+00_JPRB, 0.78505E+00_JPRB, 0.77679E+00_JPRB ,&
&  0.76857E+00_JPRB, 0.76367E+00_JPRB, 0.77074E+00_JPRB, 0.78473E+00_JPRB, 0.79623E+00_JPRB, 0.80256E+00_JPRB, 0.80755E+00_JPRB ,&
&  0.80707E+00_JPRB, 0.77401E+00_JPRB, 0.72891E+00_JPRB, 0.69606E+00_JPRB, 0.67952E+00_JPRB, 0.66563E+00_JPRB, 0.63252E+00_JPRB ,&
&  0.56496E+00_JPRB, 0.44362E+00_JPRB, 0.38733E+00_JPRB, 0.42845E+00_JPRB, 0.54649E+00_JPRB, 0.51144E+00_JPRB, 0.53211E+00_JPRB ,&
&  0.56266E+00_JPRB, 0.57849E+00_JPRB, 0.57434E+00_JPRB, 0.56392E+00_JPRB, 0.56054E+00_JPRB, 0.56421E+00_JPRB, 0.56093E+00_JPRB ,&
&  0.55760E+00_JPRB, 0.54172E+00_JPRB, 0.53226E+00_JPRB, 0.52150E+00_JPRB, 0.50398E+00_JPRB, 0.47829E+00_JPRB, 0.46274E+00_JPRB ,&
&  0.42454E+00_JPRB, 0.38689E+00_JPRB, 0.37593E+00_JPRB, 0.37663E+00_JPRB, 0.34581E+00_JPRB, 0.33325E+00_JPRB, 0.31837E+00_JPRB ,&
&  0.27840E+00_JPRB, 0.25695E+00_JPRB, 0.22477E+00_JPRB, 0.20455E+00_JPRB, 0.18959E+00_JPRB, 0.17549E+00_JPRB, 0.17344E+00_JPRB ,&
&  0.17811E+00_JPRB, 0.19039E+00_JPRB, 0.21171E+00_JPRB, 0.24589E+00_JPRB, 0.27705E+00_JPRB, 0.23374E+00_JPRB, 0.21560E+00_JPRB ,&
&  0.20157E+00_JPRB, 0.19852E+00_JPRB, 0.20400E+00_JPRB, 0.21045E+00_JPRB, 0.21707E+00_JPRB, 0.22487E+00_JPRB, 0.23233E+00_JPRB ,&
&  0.23769E+00_JPRB, 0.24072E+00_JPRB, 0.24164E+00_JPRB, 0.24098E+00_JPRB, 0.24002E+00_JPRB, 0.24089E+00_JPRB, 0.24437E+00_JPRB ,&
&  0.25050E+00_JPRB, 0.26272E+00_JPRB, 0.28655E+00_JPRB, 0.32178E+00_JPRB, 0.36790E+00_JPRB, 0.42819E+00_JPRB, 0.52039E+00_JPRB ,&
&  0.64526E+00_JPRB, 0.80960E+00_JPRB, 0.85406E+00_JPRB, 0.86303E+00_JPRB, 0.83227E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.82671E+00_JPRB, 0.82442E+00_JPRB, 0.83229E+00_JPRB, 0.82203E+00_JPRB, 0.80305E+00_JPRB, 0.79409E+00_JPRB, 0.81115E+00_JPRB ,&
&  0.83099E+00_JPRB, 0.80161E+00_JPRB, 0.78000E+00_JPRB, 0.76081E+00_JPRB, 0.74721E+00_JPRB, 0.73344E+00_JPRB, 0.71136E+00_JPRB ,&
&  0.69136E+00_JPRB, 0.68410E+00_JPRB, 0.68227E+00_JPRB, 0.67727E+00_JPRB, 0.66707E+00_JPRB, 0.65703E+00_JPRB, 0.63935E+00_JPRB ,&
&  0.60668E+00_JPRB, 0.57242E+00_JPRB, 0.55448E+00_JPRB, 0.53094E+00_JPRB, 0.51417E+00_JPRB, 0.49476E+00_JPRB, 0.46715E+00_JPRB ,&
&  0.42223E+00_JPRB, 0.36429E+00_JPRB, 0.32838E+00_JPRB, 0.36484E+00_JPRB, 0.45973E+00_JPRB, 0.45349E+00_JPRB, 0.47276E+00_JPRB ,&
&  0.50744E+00_JPRB, 0.52870E+00_JPRB, 0.53925E+00_JPRB, 0.54273E+00_JPRB, 0.55204E+00_JPRB, 0.55774E+00_JPRB, 0.55434E+00_JPRB ,&
&  0.53982E+00_JPRB, 0.48769E+00_JPRB, 0.44523E+00_JPRB, 0.34234E+00_JPRB, 0.27100E+00_JPRB, 0.23340E+00_JPRB, 0.20423E+00_JPRB ,&
&  0.18527E+00_JPRB, 0.17402E+00_JPRB, 0.17147E+00_JPRB, 0.17114E+00_JPRB, 0.17639E+00_JPRB, 0.18188E+00_JPRB, 0.18174E+00_JPRB ,&
&  0.17785E+00_JPRB, 0.17001E+00_JPRB, 0.15877E+00_JPRB, 0.15192E+00_JPRB, 0.14836E+00_JPRB, 0.14844E+00_JPRB, 0.15034E+00_JPRB ,&
&  0.15184E+00_JPRB, 0.15595E+00_JPRB, 0.16840E+00_JPRB, 0.17933E+00_JPRB, 0.19278E+00_JPRB, 0.17470E+00_JPRB, 0.16000E+00_JPRB ,&
&  0.15484E+00_JPRB, 0.15913E+00_JPRB, 0.16576E+00_JPRB, 0.17077E+00_JPRB, 0.17606E+00_JPRB, 0.18057E+00_JPRB, 0.18511E+00_JPRB ,&
&  0.18951E+00_JPRB, 0.19258E+00_JPRB, 0.19416E+00_JPRB, 0.19587E+00_JPRB, 0.19744E+00_JPRB, 0.19976E+00_JPRB, 0.20268E+00_JPRB ,&
&  0.20766E+00_JPRB, 0.21697E+00_JPRB, 0.22911E+00_JPRB, 0.24916E+00_JPRB, 0.26798E+00_JPRB, 0.28675E+00_JPRB, 0.30829E+00_JPRB ,&
&  0.32921E+00_JPRB, 0.40946E+00_JPRB, 0.60316E+00_JPRB, 0.92436E+00_JPRB, 0.91784E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.77607E+00_JPRB, 0.78855E+00_JPRB, 0.80562E+00_JPRB, 0.78217E+00_JPRB, 0.76692E+00_JPRB, 0.77032E+00_JPRB, 0.77117E+00_JPRB ,&
&  0.74808E+00_JPRB, 0.71297E+00_JPRB, 0.66390E+00_JPRB, 0.62354E+00_JPRB, 0.55823E+00_JPRB, 0.47424E+00_JPRB, 0.43664E+00_JPRB ,&
&  0.40243E+00_JPRB, 0.39891E+00_JPRB, 0.40120E+00_JPRB, 0.41728E+00_JPRB, 0.42615E+00_JPRB, 0.42276E+00_JPRB, 0.40112E+00_JPRB ,&
&  0.37415E+00_JPRB, 0.34709E+00_JPRB, 0.32961E+00_JPRB, 0.31576E+00_JPRB, 0.31559E+00_JPRB, 0.31379E+00_JPRB, 0.30730E+00_JPRB ,&
&  0.29879E+00_JPRB, 0.30314E+00_JPRB, 0.32055E+00_JPRB, 0.34879E+00_JPRB, 0.39817E+00_JPRB, 0.37811E+00_JPRB, 0.38462E+00_JPRB ,&
&  0.41468E+00_JPRB, 0.43636E+00_JPRB, 0.44503E+00_JPRB, 0.44822E+00_JPRB, 0.44152E+00_JPRB, 0.46630E+00_JPRB, 0.40991E+00_JPRB ,&
&  0.30289E+00_JPRB, 0.24394E+00_JPRB, 0.20190E+00_JPRB, 0.17489E+00_JPRB, 0.15690E+00_JPRB, 0.14642E+00_JPRB, 0.13489E+00_JPRB ,&
&  0.12956E+00_JPRB, 0.12561E+00_JPRB, 0.12464E+00_JPRB, 0.12610E+00_JPRB, 0.12852E+00_JPRB, 0.13157E+00_JPRB, 0.13258E+00_JPRB ,&
&  0.13429E+00_JPRB, 0.13429E+00_JPRB, 0.13341E+00_JPRB, 0.13312E+00_JPRB, 0.13204E+00_JPRB, 0.13258E+00_JPRB, 0.13242E+00_JPRB ,&
&  0.13231E+00_JPRB, 0.13225E+00_JPRB, 0.13379E+00_JPRB, 0.13430E+00_JPRB, 0.13506E+00_JPRB, 0.13207E+00_JPRB, 0.13033E+00_JPRB ,&
&  0.13046E+00_JPRB, 0.13244E+00_JPRB, 0.13572E+00_JPRB, 0.13951E+00_JPRB, 0.14473E+00_JPRB, 0.14892E+00_JPRB, 0.15371E+00_JPRB ,&
&  0.15788E+00_JPRB, 0.16193E+00_JPRB, 0.16590E+00_JPRB, 0.16996E+00_JPRB, 0.17430E+00_JPRB, 0.17865E+00_JPRB, 0.18322E+00_JPRB ,&
&  0.18794E+00_JPRB, 0.19613E+00_JPRB, 0.20624E+00_JPRB, 0.22383E+00_JPRB, 0.24752E+00_JPRB, 0.27442E+00_JPRB, 0.32153E+00_JPRB ,&
&  0.38174E+00_JPRB, 0.61117E+00_JPRB, 0.91106E+00_JPRB, 0.73964E+00_JPRB, 0.75752E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.76736E+00_JPRB, 0.78573E+00_JPRB, 0.80202E+00_JPRB, 0.79944E+00_JPRB, 0.78223E+00_JPRB, 0.72464E+00_JPRB, 0.66132E+00_JPRB ,&
&  0.58276E+00_JPRB, 0.51671E+00_JPRB, 0.44746E+00_JPRB, 0.38748E+00_JPRB, 0.33717E+00_JPRB, 0.29853E+00_JPRB, 0.27543E+00_JPRB ,&
&  0.25919E+00_JPRB, 0.25211E+00_JPRB, 0.24951E+00_JPRB, 0.25413E+00_JPRB, 0.25660E+00_JPRB, 0.26208E+00_JPRB, 0.25653E+00_JPRB ,&
&  0.24964E+00_JPRB, 0.24165E+00_JPRB, 0.23966E+00_JPRB, 0.24840E+00_JPRB, 0.26158E+00_JPRB, 0.27442E+00_JPRB, 0.29109E+00_JPRB ,&
&  0.30939E+00_JPRB, 0.31528E+00_JPRB, 0.32937E+00_JPRB, 0.33308E+00_JPRB, 0.31842E+00_JPRB, 0.32014E+00_JPRB, 0.32601E+00_JPRB ,&
&  0.33922E+00_JPRB, 0.35783E+00_JPRB, 0.36851E+00_JPRB, 0.38868E+00_JPRB, 0.33600E+00_JPRB, 0.27594E+00_JPRB, 0.22429E+00_JPRB ,&
&  0.18587E+00_JPRB, 0.16403E+00_JPRB, 0.14878E+00_JPRB, 0.14103E+00_JPRB, 0.13360E+00_JPRB, 0.12953E+00_JPRB, 0.12519E+00_JPRB ,&
&  0.12323E+00_JPRB, 0.12308E+00_JPRB, 0.12251E+00_JPRB, 0.12210E+00_JPRB, 0.12238E+00_JPRB, 0.12291E+00_JPRB, 0.12227E+00_JPRB ,&
&  0.12146E+00_JPRB, 0.12111E+00_JPRB, 0.12058E+00_JPRB, 0.12015E+00_JPRB, 0.11929E+00_JPRB, 0.11904E+00_JPRB, 0.11743E+00_JPRB ,&
&  0.11653E+00_JPRB, 0.11511E+00_JPRB, 0.11428E+00_JPRB, 0.11413E+00_JPRB, 0.11441E+00_JPRB, 0.11471E+00_JPRB, 0.11525E+00_JPRB ,&
&  0.11638E+00_JPRB, 0.11739E+00_JPRB, 0.11911E+00_JPRB, 0.12083E+00_JPRB, 0.12357E+00_JPRB, 0.12610E+00_JPRB, 0.12964E+00_JPRB ,&
&  0.13341E+00_JPRB, 0.13796E+00_JPRB, 0.14315E+00_JPRB, 0.14896E+00_JPRB, 0.15401E+00_JPRB, 0.15898E+00_JPRB, 0.16441E+00_JPRB ,&
&  0.16874E+00_JPRB, 0.17579E+00_JPRB, 0.18378E+00_JPRB, 0.19695E+00_JPRB, 0.22440E+00_JPRB, 0.24813E+00_JPRB, 0.29685E+00_JPRB ,&
&  0.37969E+00_JPRB, 0.63408E+00_JPRB, 0.86574E+00_JPRB, 0.71769E+00_JPRB, 0.74890E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.64652E+00_JPRB, 0.67764E+00_JPRB, 0.70204E+00_JPRB, 0.70425E+00_JPRB, 0.70317E+00_JPRB, 0.66597E+00_JPRB, 0.60817E+00_JPRB ,&
&  0.50149E+00_JPRB, 0.43186E+00_JPRB, 0.38801E+00_JPRB, 0.34668E+00_JPRB, 0.30918E+00_JPRB, 0.28400E+00_JPRB, 0.26679E+00_JPRB ,&
&  0.25373E+00_JPRB, 0.24656E+00_JPRB, 0.24111E+00_JPRB, 0.23651E+00_JPRB, 0.23158E+00_JPRB, 0.22682E+00_JPRB, 0.21814E+00_JPRB ,&
&  0.21291E+00_JPRB, 0.20943E+00_JPRB, 0.20805E+00_JPRB, 0.21050E+00_JPRB, 0.21741E+00_JPRB, 0.22453E+00_JPRB, 0.23525E+00_JPRB ,&
&  0.24861E+00_JPRB, 0.26253E+00_JPRB, 0.26987E+00_JPRB, 0.27318E+00_JPRB, 0.27615E+00_JPRB, 0.27433E+00_JPRB, 0.25178E+00_JPRB ,&
&  0.25678E+00_JPRB, 0.27969E+00_JPRB, 0.29806E+00_JPRB, 0.30837E+00_JPRB, 0.26420E+00_JPRB, 0.22300E+00_JPRB, 0.18242E+00_JPRB ,&
&  0.16292E+00_JPRB, 0.14768E+00_JPRB, 0.13978E+00_JPRB, 0.13383E+00_JPRB, 0.12950E+00_JPRB, 0.12692E+00_JPRB, 0.12463E+00_JPRB ,&
&  0.12253E+00_JPRB, 0.12161E+00_JPRB, 0.12016E+00_JPRB, 0.11948E+00_JPRB, 0.11846E+00_JPRB, 0.11823E+00_JPRB, 0.11698E+00_JPRB ,&
&  0.11570E+00_JPRB, 0.11466E+00_JPRB, 0.11397E+00_JPRB, 0.11359E+00_JPRB, 0.11277E+00_JPRB, 0.11192E+00_JPRB, 0.11009E+00_JPRB ,&
&  0.10874E+00_JPRB, 0.10743E+00_JPRB, 0.10641E+00_JPRB, 0.10638E+00_JPRB, 0.10678E+00_JPRB, 0.10841E+00_JPRB, 0.10929E+00_JPRB ,&
&  0.10942E+00_JPRB, 0.10944E+00_JPRB, 0.10942E+00_JPRB, 0.11007E+00_JPRB, 0.11107E+00_JPRB, 0.11287E+00_JPRB, 0.11538E+00_JPRB ,&
&  0.11842E+00_JPRB, 0.12307E+00_JPRB, 0.12780E+00_JPRB, 0.13313E+00_JPRB, 0.13815E+00_JPRB, 0.14319E+00_JPRB, 0.14972E+00_JPRB ,&
&  0.15645E+00_JPRB, 0.16424E+00_JPRB, 0.17567E+00_JPRB, 0.18795E+00_JPRB, 0.21000E+00_JPRB, 0.22953E+00_JPRB, 0.26120E+00_JPRB ,&
&  0.33176E+00_JPRB, 0.47885E+00_JPRB, 0.56444E+00_JPRB, 0.57571E+00_JPRB, 0.60879E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.46762E+00_JPRB, 0.47912E+00_JPRB, 0.49423E+00_JPRB, 0.50340E+00_JPRB, 0.51337E+00_JPRB, 0.48076E+00_JPRB, 0.45506E+00_JPRB ,&
&  0.40252E+00_JPRB, 0.37252E+00_JPRB, 0.34757E+00_JPRB, 0.32466E+00_JPRB, 0.30428E+00_JPRB, 0.28642E+00_JPRB, 0.26925E+00_JPRB ,&
&  0.25822E+00_JPRB, 0.25154E+00_JPRB, 0.24243E+00_JPRB, 0.23234E+00_JPRB, 0.22159E+00_JPRB, 0.21220E+00_JPRB, 0.20719E+00_JPRB ,&
&  0.20264E+00_JPRB, 0.19937E+00_JPRB, 0.19807E+00_JPRB, 0.19836E+00_JPRB, 0.20032E+00_JPRB, 0.20069E+00_JPRB, 0.20149E+00_JPRB ,&
&  0.19871E+00_JPRB, 0.20584E+00_JPRB, 0.21518E+00_JPRB, 0.21944E+00_JPRB, 0.22724E+00_JPRB, 0.22581E+00_JPRB, 0.22107E+00_JPRB ,&
&  0.21414E+00_JPRB, 0.20584E+00_JPRB, 0.19656E+00_JPRB, 0.18507E+00_JPRB, 0.17463E+00_JPRB, 0.16684E+00_JPRB, 0.15921E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.14908E+00_JPRB, 0.14277E+00_JPRB, 0.13735E+00_JPRB, 0.13266E+00_JPRB, 0.12875E+00_JPRB, 0.12520E+00_JPRB ,&
&  0.12254E+00_JPRB, 0.12093E+00_JPRB, 0.11935E+00_JPRB, 0.11880E+00_JPRB, 0.11761E+00_JPRB, 0.11647E+00_JPRB, 0.11534E+00_JPRB ,&
&  0.11467E+00_JPRB, 0.11343E+00_JPRB, 0.11245E+00_JPRB, 0.11156E+00_JPRB, 0.11025E+00_JPRB, 0.10912E+00_JPRB, 0.10785E+00_JPRB ,&
&  0.10673E+00_JPRB, 0.10611E+00_JPRB, 0.10574E+00_JPRB, 0.10561E+00_JPRB, 0.10581E+00_JPRB, 0.10571E+00_JPRB, 0.10589E+00_JPRB ,&
&  0.10613E+00_JPRB, 0.10651E+00_JPRB, 0.10646E+00_JPRB, 0.10672E+00_JPRB, 0.10750E+00_JPRB, 0.10860E+00_JPRB, 0.11131E+00_JPRB ,&
&  0.11445E+00_JPRB, 0.11850E+00_JPRB, 0.12294E+00_JPRB, 0.12813E+00_JPRB, 0.13335E+00_JPRB, 0.13861E+00_JPRB, 0.14436E+00_JPRB ,&
&  0.15188E+00_JPRB, 0.15863E+00_JPRB, 0.16641E+00_JPRB, 0.17817E+00_JPRB, 0.20307E+00_JPRB, 0.21704E+00_JPRB, 0.23220E+00_JPRB ,&
&  0.25910E+00_JPRB, 0.29632E+00_JPRB, 0.39962E+00_JPRB, 0.43248E+00_JPRB, 0.44982E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.21106E+00_JPRB, 0.21128E+00_JPRB, 0.21225E+00_JPRB, 0.21897E+00_JPRB, 0.23189E+00_JPRB, 0.30908E+00_JPRB, 0.30045E+00_JPRB ,&
&  0.29342E+00_JPRB, 0.26509E+00_JPRB, 0.26137E+00_JPRB, 0.25418E+00_JPRB, 0.24458E+00_JPRB, 0.23665E+00_JPRB, 0.23189E+00_JPRB ,&
&  0.22675E+00_JPRB, 0.22115E+00_JPRB, 0.21786E+00_JPRB, 0.21460E+00_JPRB, 0.21142E+00_JPRB, 0.20787E+00_JPRB, 0.20516E+00_JPRB ,&
&  0.20277E+00_JPRB, 0.20112E+00_JPRB, 0.19929E+00_JPRB, 0.19573E+00_JPRB, 0.19416E+00_JPRB, 0.19101E+00_JPRB, 0.18629E+00_JPRB ,&
&  0.18014E+00_JPRB, 0.17677E+00_JPRB, 0.17303E+00_JPRB, 0.16967E+00_JPRB, 0.17225E+00_JPRB, 0.17117E+00_JPRB, 0.16893E+00_JPRB ,&
&  0.16544E+00_JPRB, 0.16182E+00_JPRB, 0.15715E+00_JPRB, 0.15269E+00_JPRB, 0.14893E+00_JPRB, 0.14586E+00_JPRB, 0.14317E+00_JPRB ,&
&  0.14104E+00_JPRB, 0.13923E+00_JPRB, 0.13683E+00_JPRB, 0.13463E+00_JPRB, 0.13231E+00_JPRB, 0.13025E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12654E+00_JPRB, 0.12498E+00_JPRB, 0.12336E+00_JPRB, 0.12169E+00_JPRB, 0.12036E+00_JPRB, 0.11864E+00_JPRB, 0.11734E+00_JPRB ,&
&  0.11619E+00_JPRB, 0.11477E+00_JPRB, 0.11371E+00_JPRB, 0.11241E+00_JPRB, 0.11146E+00_JPRB, 0.11066E+00_JPRB, 0.10990E+00_JPRB ,&
&  0.10887E+00_JPRB, 0.10818E+00_JPRB, 0.10790E+00_JPRB, 0.10767E+00_JPRB, 0.10732E+00_JPRB, 0.10733E+00_JPRB, 0.10760E+00_JPRB ,&
&  0.10800E+00_JPRB, 0.10847E+00_JPRB, 0.10929E+00_JPRB, 0.11027E+00_JPRB, 0.11174E+00_JPRB, 0.11370E+00_JPRB, 0.11682E+00_JPRB ,&
&  0.12022E+00_JPRB, 0.12400E+00_JPRB, 0.12793E+00_JPRB, 0.13242E+00_JPRB, 0.13696E+00_JPRB, 0.14158E+00_JPRB, 0.14687E+00_JPRB ,&
&  0.15264E+00_JPRB, 0.15707E+00_JPRB, 0.16113E+00_JPRB, 0.16542E+00_JPRB, 0.17416E+00_JPRB, 0.18074E+00_JPRB, 0.18584E+00_JPRB ,&
&  0.19155E+00_JPRB, 0.19924E+00_JPRB, 0.20408E+00_JPRB, 0.20745E+00_JPRB, 0.20952E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.18419E+00_JPRB, 0.18673E+00_JPRB, 0.18843E+00_JPRB, 0.18951E+00_JPRB, 0.18929E+00_JPRB, 0.19021E+00_JPRB, 0.19118E+00_JPRB ,&
&  0.19246E+00_JPRB, 0.19404E+00_JPRB, 0.19514E+00_JPRB, 0.19616E+00_JPRB, 0.19666E+00_JPRB, 0.19667E+00_JPRB, 0.19667E+00_JPRB ,&
&  0.19649E+00_JPRB, 0.19626E+00_JPRB, 0.19313E+00_JPRB, 0.19031E+00_JPRB, 0.18832E+00_JPRB, 0.18722E+00_JPRB, 0.18622E+00_JPRB ,&
&  0.18538E+00_JPRB, 0.18421E+00_JPRB, 0.18279E+00_JPRB, 0.18063E+00_JPRB, 0.17741E+00_JPRB, 0.17426E+00_JPRB, 0.17104E+00_JPRB ,&
&  0.16814E+00_JPRB, 0.16575E+00_JPRB, 0.16354E+00_JPRB, 0.16131E+00_JPRB, 0.15993E+00_JPRB, 0.15836E+00_JPRB, 0.15676E+00_JPRB ,&
&  0.15477E+00_JPRB, 0.15263E+00_JPRB, 0.15059E+00_JPRB, 0.14851E+00_JPRB, 0.14650E+00_JPRB, 0.14452E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14155E+00_JPRB, 0.14039E+00_JPRB, 0.13911E+00_JPRB, 0.13794E+00_JPRB, 0.13671E+00_JPRB, 0.13555E+00_JPRB, 0.13425E+00_JPRB ,&
&  0.13335E+00_JPRB, 0.13242E+00_JPRB, 0.13147E+00_JPRB, 0.13060E+00_JPRB, 0.12972E+00_JPRB, 0.12887E+00_JPRB, 0.12791E+00_JPRB ,&
&  0.12704E+00_JPRB, 0.12647E+00_JPRB, 0.12587E+00_JPRB, 0.12521E+00_JPRB, 0.12459E+00_JPRB, 0.12416E+00_JPRB, 0.12379E+00_JPRB ,&
&  0.12344E+00_JPRB, 0.12340E+00_JPRB, 0.12354E+00_JPRB, 0.12370E+00_JPRB, 0.12385E+00_JPRB, 0.12422E+00_JPRB, 0.12478E+00_JPRB ,&
&  0.12544E+00_JPRB, 0.12616E+00_JPRB, 0.12709E+00_JPRB, 0.12841E+00_JPRB, 0.12987E+00_JPRB, 0.13155E+00_JPRB, 0.13339E+00_JPRB ,&
&  0.13538E+00_JPRB, 0.13752E+00_JPRB, 0.13979E+00_JPRB, 0.14205E+00_JPRB, 0.14447E+00_JPRB, 0.14688E+00_JPRB, 0.14921E+00_JPRB ,&
&  0.15163E+00_JPRB, 0.15401E+00_JPRB, 0.15644E+00_JPRB, 0.15865E+00_JPRB, 0.16199E+00_JPRB, 0.16417E+00_JPRB, 0.16646E+00_JPRB ,&
&  0.16861E+00_JPRB, 0.17110E+00_JPRB, 0.17361E+00_JPRB, 0.17672E+00_JPRB, 0.17981E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB, 0.29771E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.30617E+00_JPRB, 0.30546E+00_JPRB, 0.30447E+00_JPRB, 0.30366E+00_JPRB, 0.30257E+00_JPRB, 0.30180E+00_JPRB, 0.30098E+00_JPRB ,&
&  0.30031E+00_JPRB, 0.29959E+00_JPRB, 0.29911E+00_JPRB, 0.29834E+00_JPRB, 0.29767E+00_JPRB, 0.29704E+00_JPRB, 0.29656E+00_JPRB ,&
&  0.29602E+00_JPRB, 0.29559E+00_JPRB, 0.29518E+00_JPRB, 0.29477E+00_JPRB, 0.29444E+00_JPRB, 0.29395E+00_JPRB, 0.29317E+00_JPRB ,&
&  0.29169E+00_JPRB, 0.28995E+00_JPRB, 0.28847E+00_JPRB, 0.28761E+00_JPRB, 0.28711E+00_JPRB, 0.28673E+00_JPRB, 0.28644E+00_JPRB ,&
&  0.28641E+00_JPRB, 0.28671E+00_JPRB, 0.28714E+00_JPRB, 0.28752E+00_JPRB, 0.28803E+00_JPRB, 0.28903E+00_JPRB, 0.28973E+00_JPRB ,&
&  0.29023E+00_JPRB, 0.29069E+00_JPRB, 0.29158E+00_JPRB, 0.29232E+00_JPRB, 0.29314E+00_JPRB, 0.29397E+00_JPRB, 0.29519E+00_JPRB ,&
&  0.29642E+00_JPRB, 0.29759E+00_JPRB, 0.29879E+00_JPRB, 0.29995E+00_JPRB, 0.30108E+00_JPRB, 0.30226E+00_JPRB, 0.30363E+00_JPRB ,&
&  0.30504E+00_JPRB, 0.30628E+00_JPRB, 0.30737E+00_JPRB, 0.30815E+00_JPRB, 0.30876E+00_JPRB, 0.30890E+00_JPRB, 0.30893E+00_JPRB ,&
&  0.30855E+00_JPRB, 0.30855E+00_JPRB, 0.30821E+00_JPRB, 0.30789E+00_JPRB, 0.30725E+00_JPRB, 0.30708E+00_JPRB, 0.30674E+00_JPRB ,&
&  0.30680E+00_JPRB, 0.30665E+00_JPRB, 0.30676E+00_JPRB, 0.30682E+00_JPRB, 0.30724E+00_JPRB, 0.30785E+00_JPRB, 0.30854E+00_JPRB ,&
&  0.30936E+00_JPRB, 0.31021E+00_JPRB, 0.31125E+00_JPRB, 0.31155E+00_JPRB, 0.31185E+00_JPRB, 0.31217E+00_JPRB, 0.31270E+00_JPRB ,&
&  0.31301E+00_JPRB, 0.31332E+00_JPRB, 0.31373E+00_JPRB, 0.31401E+00_JPRB, 0.31404E+00_JPRB, 0.31395E+00_JPRB, 0.31388E+00_JPRB ,&
&  0.31362E+00_JPRB, 0.31340E+00_JPRB, 0.31312E+00_JPRB, 0.31292E+00_JPRB, 0.31237E+00_JPRB, 0.31174E+00_JPRB, 0.31091E+00_JPRB ,&
&  0.31027E+00_JPRB, 0.30943E+00_JPRB, 0.30871E+00_JPRB, 0.30788E+00_JPRB, 0.30710E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.33195E+00_JPRB, 0.33053E+00_JPRB, 0.32904E+00_JPRB, 0.32763E+00_JPRB, 0.32564E+00_JPRB, 0.32363E+00_JPRB, 0.32155E+00_JPRB ,&
&  0.31942E+00_JPRB, 0.31701E+00_JPRB, 0.31450E+00_JPRB, 0.31186E+00_JPRB, 0.30907E+00_JPRB, 0.30642E+00_JPRB, 0.30428E+00_JPRB ,&
&  0.30242E+00_JPRB, 0.30042E+00_JPRB, 0.29892E+00_JPRB, 0.29687E+00_JPRB, 0.29475E+00_JPRB, 0.29299E+00_JPRB, 0.29218E+00_JPRB ,&
&  0.29029E+00_JPRB, 0.28781E+00_JPRB, 0.28517E+00_JPRB, 0.28228E+00_JPRB, 0.27964E+00_JPRB, 0.27693E+00_JPRB, 0.27406E+00_JPRB ,&
&  0.27120E+00_JPRB, 0.26923E+00_JPRB, 0.26765E+00_JPRB, 0.26606E+00_JPRB, 0.26447E+00_JPRB, 0.26461E+00_JPRB, 0.26459E+00_JPRB ,&
&  0.26470E+00_JPRB, 0.26533E+00_JPRB, 0.26772E+00_JPRB, 0.26994E+00_JPRB, 0.27221E+00_JPRB, 0.27447E+00_JPRB, 0.27802E+00_JPRB ,&
&  0.28132E+00_JPRB, 0.28525E+00_JPRB, 0.28986E+00_JPRB, 0.29423E+00_JPRB, 0.29762E+00_JPRB, 0.29979E+00_JPRB, 0.30247E+00_JPRB ,&
&  0.30412E+00_JPRB, 0.30639E+00_JPRB, 0.30784E+00_JPRB, 0.30830E+00_JPRB, 0.30772E+00_JPRB, 0.30713E+00_JPRB, 0.30700E+00_JPRB ,&
&  0.30649E+00_JPRB, 0.30726E+00_JPRB, 0.30792E+00_JPRB, 0.30857E+00_JPRB, 0.30825E+00_JPRB, 0.30911E+00_JPRB, 0.30970E+00_JPRB ,&
&  0.31148E+00_JPRB, 0.31444E+00_JPRB, 0.31728E+00_JPRB, 0.32051E+00_JPRB, 0.32466E+00_JPRB, 0.32979E+00_JPRB, 0.33365E+00_JPRB ,&
&  0.33754E+00_JPRB, 0.34022E+00_JPRB, 0.34191E+00_JPRB, 0.34210E+00_JPRB, 0.34235E+00_JPRB, 0.34267E+00_JPRB, 0.34380E+00_JPRB ,&
&  0.34402E+00_JPRB, 0.34447E+00_JPRB, 0.34498E+00_JPRB, 0.34523E+00_JPRB, 0.34443E+00_JPRB, 0.34363E+00_JPRB, 0.34321E+00_JPRB ,&
&  0.34261E+00_JPRB, 0.34194E+00_JPRB, 0.34134E+00_JPRB, 0.34095E+00_JPRB, 0.34027E+00_JPRB, 0.33952E+00_JPRB, 0.33879E+00_JPRB ,&
&  0.33809E+00_JPRB, 0.33701E+00_JPRB, 0.33578E+00_JPRB, 0.33456E+00_JPRB, 0.33344E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.34251E+00_JPRB, 0.34240E+00_JPRB, 0.34219E+00_JPRB, 0.34182E+00_JPRB, 0.34076E+00_JPRB, 0.33942E+00_JPRB, 0.33765E+00_JPRB ,&
&  0.33546E+00_JPRB, 0.33229E+00_JPRB, 0.32900E+00_JPRB, 0.32552E+00_JPRB, 0.32231E+00_JPRB, 0.31973E+00_JPRB, 0.31726E+00_JPRB ,&
&  0.31451E+00_JPRB, 0.31079E+00_JPRB, 0.30666E+00_JPRB, 0.30113E+00_JPRB, 0.29603E+00_JPRB, 0.29164E+00_JPRB, 0.28676E+00_JPRB ,&
&  0.27974E+00_JPRB, 0.27222E+00_JPRB, 0.26561E+00_JPRB, 0.25882E+00_JPRB, 0.25378E+00_JPRB, 0.23724E+00_JPRB, 0.23246E+00_JPRB ,&
&  0.22945E+00_JPRB, 0.22648E+00_JPRB, 0.22564E+00_JPRB, 0.22537E+00_JPRB, 0.22753E+00_JPRB, 0.22539E+00_JPRB, 0.22426E+00_JPRB ,&
&  0.21562E+00_JPRB, 0.21642E+00_JPRB, 0.21833E+00_JPRB, 0.22086E+00_JPRB, 0.22631E+00_JPRB, 0.23267E+00_JPRB, 0.24313E+00_JPRB ,&
&  0.26225E+00_JPRB, 0.27049E+00_JPRB, 0.28270E+00_JPRB, 0.29199E+00_JPRB, 0.29708E+00_JPRB, 0.29952E+00_JPRB, 0.30190E+00_JPRB ,&
&  0.30455E+00_JPRB, 0.30832E+00_JPRB, 0.31033E+00_JPRB, 0.31158E+00_JPRB, 0.31109E+00_JPRB, 0.30987E+00_JPRB, 0.30911E+00_JPRB ,&
&  0.30785E+00_JPRB, 0.30823E+00_JPRB, 0.30948E+00_JPRB, 0.31308E+00_JPRB, 0.31820E+00_JPRB, 0.32476E+00_JPRB, 0.33213E+00_JPRB ,&
&  0.33993E+00_JPRB, 0.34893E+00_JPRB, 0.35598E+00_JPRB, 0.36299E+00_JPRB, 0.37043E+00_JPRB, 0.37670E+00_JPRB, 0.38157E+00_JPRB ,&
&  0.38497E+00_JPRB, 0.38463E+00_JPRB, 0.38091E+00_JPRB, 0.37686E+00_JPRB, 0.37261E+00_JPRB, 0.36946E+00_JPRB, 0.36825E+00_JPRB ,&
&  0.36713E+00_JPRB, 0.36653E+00_JPRB, 0.36619E+00_JPRB, 0.36443E+00_JPRB, 0.36270E+00_JPRB, 0.36065E+00_JPRB, 0.35864E+00_JPRB ,&
&  0.35631E+00_JPRB, 0.35433E+00_JPRB, 0.35262E+00_JPRB, 0.35100E+00_JPRB, 0.34973E+00_JPRB, 0.34790E+00_JPRB, 0.34646E+00_JPRB ,&
&  0.34490E+00_JPRB, 0.34456E+00_JPRB, 0.34377E+00_JPRB, 0.34322E+00_JPRB, 0.34264E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.32416E+00_JPRB, 0.32462E+00_JPRB, 0.32548E+00_JPRB, 0.32605E+00_JPRB, 0.32710E+00_JPRB, 0.32783E+00_JPRB, 0.32878E+00_JPRB ,&
&  0.32932E+00_JPRB, 0.32988E+00_JPRB, 0.32972E+00_JPRB, 0.32916E+00_JPRB, 0.32794E+00_JPRB, 0.32682E+00_JPRB, 0.32545E+00_JPRB ,&
&  0.32334E+00_JPRB, 0.31873E+00_JPRB, 0.31238E+00_JPRB, 0.30514E+00_JPRB, 0.29688E+00_JPRB, 0.28794E+00_JPRB, 0.27763E+00_JPRB ,&
&  0.26694E+00_JPRB, 0.25705E+00_JPRB, 0.24308E+00_JPRB, 0.23816E+00_JPRB, 0.22899E+00_JPRB, 0.22217E+00_JPRB, 0.21752E+00_JPRB ,&
&  0.21179E+00_JPRB, 0.20782E+00_JPRB, 0.20368E+00_JPRB, 0.19916E+00_JPRB, 0.19565E+00_JPRB, 0.18977E+00_JPRB, 0.18662E+00_JPRB ,&
&  0.18441E+00_JPRB, 0.18341E+00_JPRB, 0.18353E+00_JPRB, 0.18505E+00_JPRB, 0.18864E+00_JPRB, 0.19330E+00_JPRB, 0.20245E+00_JPRB ,&
&  0.22210E+00_JPRB, 0.25059E+00_JPRB, 0.29319E+00_JPRB, 0.30090E+00_JPRB, 0.29973E+00_JPRB, 0.30510E+00_JPRB, 0.31059E+00_JPRB ,&
&  0.31574E+00_JPRB, 0.31905E+00_JPRB, 0.32148E+00_JPRB, 0.32295E+00_JPRB, 0.32496E+00_JPRB, 0.32667E+00_JPRB, 0.32832E+00_JPRB ,&
&  0.32995E+00_JPRB, 0.33416E+00_JPRB, 0.34005E+00_JPRB, 0.34737E+00_JPRB, 0.35455E+00_JPRB, 0.35971E+00_JPRB, 0.36519E+00_JPRB ,&
&  0.36961E+00_JPRB, 0.37497E+00_JPRB, 0.38237E+00_JPRB, 0.39105E+00_JPRB, 0.40024E+00_JPRB, 0.40556E+00_JPRB, 0.40944E+00_JPRB ,&
&  0.41128E+00_JPRB, 0.40797E+00_JPRB, 0.40191E+00_JPRB, 0.39514E+00_JPRB, 0.39037E+00_JPRB, 0.39121E+00_JPRB, 0.38900E+00_JPRB ,&
&  0.38753E+00_JPRB, 0.38340E+00_JPRB, 0.37754E+00_JPRB, 0.37065E+00_JPRB, 0.36547E+00_JPRB, 0.36069E+00_JPRB, 0.35746E+00_JPRB ,&
&  0.35453E+00_JPRB, 0.35165E+00_JPRB, 0.34817E+00_JPRB, 0.34367E+00_JPRB, 0.33928E+00_JPRB, 0.33469E+00_JPRB, 0.33114E+00_JPRB ,&
&  0.32793E+00_JPRB, 0.32614E+00_JPRB, 0.32466E+00_JPRB, 0.32414E+00_JPRB, 0.32370E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.31353E+00_JPRB, 0.31669E+00_JPRB, 0.31976E+00_JPRB, 0.32202E+00_JPRB, 0.32383E+00_JPRB, 0.32522E+00_JPRB, 0.32568E+00_JPRB ,&
&  0.32585E+00_JPRB, 0.32575E+00_JPRB, 0.32590E+00_JPRB, 0.32597E+00_JPRB, 0.32632E+00_JPRB, 0.32704E+00_JPRB, 0.32753E+00_JPRB ,&
&  0.32790E+00_JPRB, 0.32433E+00_JPRB, 0.31710E+00_JPRB, 0.30897E+00_JPRB, 0.30047E+00_JPRB, 0.28980E+00_JPRB, 0.27727E+00_JPRB ,&
&  0.26620E+00_JPRB, 0.25785E+00_JPRB, 0.24800E+00_JPRB, 0.24161E+00_JPRB, 0.23419E+00_JPRB, 0.23020E+00_JPRB, 0.22443E+00_JPRB ,&
&  0.22341E+00_JPRB, 0.20739E+00_JPRB, 0.20512E+00_JPRB, 0.19502E+00_JPRB, 0.18693E+00_JPRB, 0.18035E+00_JPRB, 0.17555E+00_JPRB ,&
&  0.17405E+00_JPRB, 0.17232E+00_JPRB, 0.17241E+00_JPRB, 0.17466E+00_JPRB, 0.17775E+00_JPRB, 0.18425E+00_JPRB, 0.19195E+00_JPRB ,&
&  0.21009E+00_JPRB, 0.26992E+00_JPRB, 0.29030E+00_JPRB, 0.29650E+00_JPRB, 0.30060E+00_JPRB, 0.31103E+00_JPRB, 0.32046E+00_JPRB ,&
&  0.33153E+00_JPRB, 0.33719E+00_JPRB, 0.34113E+00_JPRB, 0.34217E+00_JPRB, 0.35124E+00_JPRB, 0.35931E+00_JPRB, 0.36560E+00_JPRB ,&
&  0.37202E+00_JPRB, 0.37796E+00_JPRB, 0.38494E+00_JPRB, 0.38811E+00_JPRB, 0.38684E+00_JPRB, 0.38388E+00_JPRB, 0.38118E+00_JPRB ,&
&  0.38087E+00_JPRB, 0.38143E+00_JPRB, 0.38981E+00_JPRB, 0.40231E+00_JPRB, 0.41759E+00_JPRB, 0.42859E+00_JPRB, 0.43320E+00_JPRB ,&
&  0.43440E+00_JPRB, 0.43281E+00_JPRB, 0.43426E+00_JPRB, 0.43819E+00_JPRB, 0.43283E+00_JPRB, 0.42294E+00_JPRB, 0.41683E+00_JPRB ,&
&  0.41082E+00_JPRB, 0.39870E+00_JPRB, 0.38906E+00_JPRB, 0.38072E+00_JPRB, 0.37411E+00_JPRB, 0.36788E+00_JPRB, 0.36202E+00_JPRB ,&
&  0.35513E+00_JPRB, 0.34807E+00_JPRB, 0.34047E+00_JPRB, 0.33081E+00_JPRB, 0.32284E+00_JPRB, 0.31465E+00_JPRB, 0.30986E+00_JPRB ,&
&  0.30593E+00_JPRB, 0.30540E+00_JPRB, 0.30517E+00_JPRB, 0.30754E+00_JPRB, 0.30954E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.30892E+00_JPRB, 0.31624E+00_JPRB, 0.32223E+00_JPRB, 0.32642E+00_JPRB, 0.32898E+00_JPRB, 0.33039E+00_JPRB, 0.33000E+00_JPRB ,&
&  0.32922E+00_JPRB, 0.32778E+00_JPRB, 0.32701E+00_JPRB, 0.32590E+00_JPRB, 0.32576E+00_JPRB, 0.32497E+00_JPRB, 0.32466E+00_JPRB ,&
&  0.32469E+00_JPRB, 0.32393E+00_JPRB, 0.31999E+00_JPRB, 0.31246E+00_JPRB, 0.30365E+00_JPRB, 0.29400E+00_JPRB, 0.28097E+00_JPRB ,&
&  0.26848E+00_JPRB, 0.26066E+00_JPRB, 0.25288E+00_JPRB, 0.24605E+00_JPRB, 0.24222E+00_JPRB, 0.23698E+00_JPRB, 0.23664E+00_JPRB ,&
&  0.23506E+00_JPRB, 0.23007E+00_JPRB, 0.22376E+00_JPRB, 0.21753E+00_JPRB, 0.21018E+00_JPRB, 0.18715E+00_JPRB, 0.18332E+00_JPRB ,&
&  0.17700E+00_JPRB, 0.17176E+00_JPRB, 0.17048E+00_JPRB, 0.17129E+00_JPRB, 0.17377E+00_JPRB, 0.18088E+00_JPRB, 0.19061E+00_JPRB ,&
&  0.20566E+00_JPRB, 0.27497E+00_JPRB, 0.28860E+00_JPRB, 0.29844E+00_JPRB, 0.31344E+00_JPRB, 0.32782E+00_JPRB, 0.34005E+00_JPRB ,&
&  0.35064E+00_JPRB, 0.35825E+00_JPRB, 0.36772E+00_JPRB, 0.37535E+00_JPRB, 0.38427E+00_JPRB, 0.39164E+00_JPRB, 0.39577E+00_JPRB ,&
&  0.39834E+00_JPRB, 0.39638E+00_JPRB, 0.40635E+00_JPRB, 0.40965E+00_JPRB, 0.40636E+00_JPRB, 0.41329E+00_JPRB, 0.41560E+00_JPRB ,&
&  0.41155E+00_JPRB, 0.40423E+00_JPRB, 0.40954E+00_JPRB, 0.42670E+00_JPRB, 0.45267E+00_JPRB, 0.47262E+00_JPRB, 0.47660E+00_JPRB ,&
&  0.47657E+00_JPRB, 0.47223E+00_JPRB, 0.46773E+00_JPRB, 0.46590E+00_JPRB, 0.45277E+00_JPRB, 0.43399E+00_JPRB, 0.42077E+00_JPRB ,&
&  0.40997E+00_JPRB, 0.40804E+00_JPRB, 0.40195E+00_JPRB, 0.39480E+00_JPRB, 0.38662E+00_JPRB, 0.37771E+00_JPRB, 0.36598E+00_JPRB ,&
&  0.35045E+00_JPRB, 0.33331E+00_JPRB, 0.32000E+00_JPRB, 0.30820E+00_JPRB, 0.30197E+00_JPRB, 0.29482E+00_JPRB, 0.29160E+00_JPRB ,&
&  0.28874E+00_JPRB, 0.28972E+00_JPRB, 0.29149E+00_JPRB, 0.29617E+00_JPRB, 0.30141E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.31441E+00_JPRB, 0.32633E+00_JPRB, 0.33597E+00_JPRB, 0.34160E+00_JPRB, 0.34275E+00_JPRB, 0.34292E+00_JPRB, 0.34148E+00_JPRB ,&
&  0.34049E+00_JPRB, 0.33915E+00_JPRB, 0.33733E+00_JPRB, 0.33425E+00_JPRB, 0.33107E+00_JPRB, 0.32724E+00_JPRB, 0.32462E+00_JPRB ,&
&  0.32171E+00_JPRB, 0.31622E+00_JPRB, 0.31251E+00_JPRB, 0.30878E+00_JPRB, 0.30243E+00_JPRB, 0.29431E+00_JPRB, 0.28350E+00_JPRB ,&
&  0.27168E+00_JPRB, 0.26303E+00_JPRB, 0.25742E+00_JPRB, 0.25099E+00_JPRB, 0.24542E+00_JPRB, 0.24150E+00_JPRB, 0.23973E+00_JPRB ,&
&  0.23971E+00_JPRB, 0.23683E+00_JPRB, 0.23511E+00_JPRB, 0.22698E+00_JPRB, 0.21618E+00_JPRB, 0.21223E+00_JPRB, 0.19709E+00_JPRB ,&
&  0.18420E+00_JPRB, 0.17518E+00_JPRB, 0.17299E+00_JPRB, 0.17327E+00_JPRB, 0.17733E+00_JPRB, 0.18617E+00_JPRB, 0.20579E+00_JPRB ,&
&  0.23962E+00_JPRB, 0.29322E+00_JPRB, 0.30418E+00_JPRB, 0.31973E+00_JPRB, 0.33715E+00_JPRB, 0.35373E+00_JPRB, 0.36565E+00_JPRB ,&
&  0.37368E+00_JPRB, 0.38037E+00_JPRB, 0.39122E+00_JPRB, 0.40101E+00_JPRB, 0.41774E+00_JPRB, 0.43261E+00_JPRB, 0.44438E+00_JPRB ,&
&  0.45528E+00_JPRB, 0.43504E+00_JPRB, 0.43626E+00_JPRB, 0.43774E+00_JPRB, 0.44924E+00_JPRB, 0.45737E+00_JPRB, 0.44898E+00_JPRB ,&
&  0.44445E+00_JPRB, 0.45854E+00_JPRB, 0.45890E+00_JPRB, 0.46790E+00_JPRB, 0.48914E+00_JPRB, 0.51252E+00_JPRB, 0.52003E+00_JPRB ,&
&  0.50859E+00_JPRB, 0.50972E+00_JPRB, 0.53841E+00_JPRB, 0.51289E+00_JPRB, 0.49667E+00_JPRB, 0.47161E+00_JPRB, 0.44296E+00_JPRB ,&
&  0.42757E+00_JPRB, 0.41894E+00_JPRB, 0.41455E+00_JPRB, 0.40829E+00_JPRB, 0.39875E+00_JPRB, 0.38497E+00_JPRB, 0.36334E+00_JPRB ,&
&  0.33618E+00_JPRB, 0.31634E+00_JPRB, 0.30259E+00_JPRB, 0.29382E+00_JPRB, 0.28850E+00_JPRB, 0.28258E+00_JPRB, 0.27842E+00_JPRB ,&
&  0.27646E+00_JPRB, 0.27924E+00_JPRB, 0.28542E+00_JPRB, 0.29524E+00_JPRB, 0.30528E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.33031E+00_JPRB, 0.34311E+00_JPRB, 0.35097E+00_JPRB, 0.35223E+00_JPRB, 0.34824E+00_JPRB, 0.33938E+00_JPRB, 0.33196E+00_JPRB ,&
&  0.32953E+00_JPRB, 0.32948E+00_JPRB, 0.33028E+00_JPRB, 0.32796E+00_JPRB, 0.32535E+00_JPRB, 0.32281E+00_JPRB, 0.32110E+00_JPRB ,&
&  0.31940E+00_JPRB, 0.31793E+00_JPRB, 0.31618E+00_JPRB, 0.31075E+00_JPRB, 0.30349E+00_JPRB, 0.29474E+00_JPRB, 0.28526E+00_JPRB ,&
&  0.27592E+00_JPRB, 0.26886E+00_JPRB, 0.26338E+00_JPRB, 0.25897E+00_JPRB, 0.25327E+00_JPRB, 0.24990E+00_JPRB, 0.24722E+00_JPRB ,&
&  0.24504E+00_JPRB, 0.24533E+00_JPRB, 0.24423E+00_JPRB, 0.24045E+00_JPRB, 0.23197E+00_JPRB, 0.22133E+00_JPRB, 0.20712E+00_JPRB ,&
&  0.19525E+00_JPRB, 0.18765E+00_JPRB, 0.18333E+00_JPRB, 0.18311E+00_JPRB, 0.18957E+00_JPRB, 0.20409E+00_JPRB, 0.23421E+00_JPRB ,&
&  0.29256E+00_JPRB, 0.31175E+00_JPRB, 0.32338E+00_JPRB, 0.33940E+00_JPRB, 0.35441E+00_JPRB, 0.36724E+00_JPRB, 0.37900E+00_JPRB ,&
&  0.38827E+00_JPRB, 0.39728E+00_JPRB, 0.40798E+00_JPRB, 0.41559E+00_JPRB, 0.42890E+00_JPRB, 0.42923E+00_JPRB, 0.45668E+00_JPRB ,&
&  0.53097E+00_JPRB, 0.52766E+00_JPRB, 0.49001E+00_JPRB, 0.49037E+00_JPRB, 0.50053E+00_JPRB, 0.50979E+00_JPRB, 0.52118E+00_JPRB ,&
&  0.52933E+00_JPRB, 0.53061E+00_JPRB, 0.53069E+00_JPRB, 0.52748E+00_JPRB, 0.53260E+00_JPRB, 0.54031E+00_JPRB, 0.54051E+00_JPRB ,&
&  0.53392E+00_JPRB, 0.55241E+00_JPRB, 0.60201E+00_JPRB, 0.50316E+00_JPRB, 0.46017E+00_JPRB, 0.44725E+00_JPRB, 0.42973E+00_JPRB ,&
&  0.41875E+00_JPRB, 0.41583E+00_JPRB, 0.41967E+00_JPRB, 0.41780E+00_JPRB, 0.39426E+00_JPRB, 0.36285E+00_JPRB, 0.33067E+00_JPRB ,&
&  0.31304E+00_JPRB, 0.30348E+00_JPRB, 0.29329E+00_JPRB, 0.28630E+00_JPRB, 0.28166E+00_JPRB, 0.27661E+00_JPRB, 0.27177E+00_JPRB ,&
&  0.27403E+00_JPRB, 0.28060E+00_JPRB, 0.29073E+00_JPRB, 0.30686E+00_JPRB, 0.31965E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.33030E+00_JPRB, 0.34806E+00_JPRB, 0.35579E+00_JPRB, 0.35078E+00_JPRB, 0.34436E+00_JPRB, 0.32531E+00_JPRB, 0.30703E+00_JPRB ,&
&  0.30036E+00_JPRB, 0.29600E+00_JPRB, 0.29519E+00_JPRB, 0.29475E+00_JPRB, 0.29565E+00_JPRB, 0.29810E+00_JPRB, 0.30019E+00_JPRB ,&
&  0.29954E+00_JPRB, 0.29863E+00_JPRB, 0.29882E+00_JPRB, 0.29928E+00_JPRB, 0.29637E+00_JPRB, 0.28953E+00_JPRB, 0.28224E+00_JPRB ,&
&  0.27318E+00_JPRB, 0.26708E+00_JPRB, 0.26300E+00_JPRB, 0.26023E+00_JPRB, 0.25880E+00_JPRB, 0.25923E+00_JPRB, 0.26006E+00_JPRB ,&
&  0.25836E+00_JPRB, 0.25677E+00_JPRB, 0.25715E+00_JPRB, 0.25593E+00_JPRB, 0.24842E+00_JPRB, 0.23629E+00_JPRB, 0.21955E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20184E+00_JPRB, 0.20314E+00_JPRB, 0.21285E+00_JPRB, 0.23935E+00_JPRB, 0.29079E+00_JPRB, 0.31019E+00_JPRB ,&
&  0.32664E+00_JPRB, 0.33750E+00_JPRB, 0.34898E+00_JPRB, 0.36462E+00_JPRB, 0.37774E+00_JPRB, 0.38863E+00_JPRB, 0.39911E+00_JPRB ,&
&  0.41376E+00_JPRB, 0.42846E+00_JPRB, 0.44777E+00_JPRB, 0.41277E+00_JPRB, 0.42083E+00_JPRB, 0.48776E+00_JPRB, 0.50474E+00_JPRB ,&
&  0.54227E+00_JPRB, 0.54897E+00_JPRB, 0.54703E+00_JPRB, 0.57979E+00_JPRB, 0.58778E+00_JPRB, 0.60170E+00_JPRB, 0.60033E+00_JPRB ,&
&  0.59112E+00_JPRB, 0.57555E+00_JPRB, 0.56087E+00_JPRB, 0.54846E+00_JPRB, 0.55120E+00_JPRB, 0.56328E+00_JPRB, 0.55417E+00_JPRB ,&
&  0.54370E+00_JPRB, 0.53809E+00_JPRB, 0.50190E+00_JPRB, 0.44221E+00_JPRB, 0.42106E+00_JPRB, 0.41766E+00_JPRB, 0.42159E+00_JPRB ,&
&  0.41899E+00_JPRB, 0.41994E+00_JPRB, 0.42040E+00_JPRB, 0.41425E+00_JPRB, 0.37379E+00_JPRB, 0.33620E+00_JPRB, 0.30618E+00_JPRB ,&
&  0.29023E+00_JPRB, 0.28622E+00_JPRB, 0.28059E+00_JPRB, 0.28000E+00_JPRB, 0.27736E+00_JPRB, 0.27667E+00_JPRB, 0.27369E+00_JPRB ,&
&  0.27507E+00_JPRB, 0.28054E+00_JPRB, 0.29315E+00_JPRB, 0.30798E+00_JPRB, 0.31912E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.35458E+00_JPRB, 0.36089E+00_JPRB, 0.35635E+00_JPRB, 0.35391E+00_JPRB, 0.34658E+00_JPRB, 0.33720E+00_JPRB, 0.32486E+00_JPRB ,&
&  0.32669E+00_JPRB, 0.30861E+00_JPRB, 0.29536E+00_JPRB, 0.28337E+00_JPRB, 0.27607E+00_JPRB, 0.27159E+00_JPRB, 0.27393E+00_JPRB ,&
&  0.28099E+00_JPRB, 0.28987E+00_JPRB, 0.29464E+00_JPRB, 0.29342E+00_JPRB, 0.28915E+00_JPRB, 0.28103E+00_JPRB, 0.27550E+00_JPRB ,&
&  0.26844E+00_JPRB, 0.26499E+00_JPRB, 0.26580E+00_JPRB, 0.26908E+00_JPRB, 0.27252E+00_JPRB, 0.27293E+00_JPRB, 0.26999E+00_JPRB ,&
&  0.26537E+00_JPRB, 0.26286E+00_JPRB, 0.26454E+00_JPRB, 0.26727E+00_JPRB, 0.26389E+00_JPRB, 0.24871E+00_JPRB, 0.23222E+00_JPRB ,&
&  0.21933E+00_JPRB, 0.21901E+00_JPRB, 0.24019E+00_JPRB, 0.29398E+00_JPRB, 0.31364E+00_JPRB, 0.32835E+00_JPRB, 0.34033E+00_JPRB ,&
&  0.33687E+00_JPRB, 0.34163E+00_JPRB, 0.36356E+00_JPRB, 0.40523E+00_JPRB, 0.41457E+00_JPRB, 0.42439E+00_JPRB, 0.43477E+00_JPRB ,&
&  0.45470E+00_JPRB, 0.45542E+00_JPRB, 0.47009E+00_JPRB, 0.45902E+00_JPRB, 0.55090E+00_JPRB, 0.72841E+00_JPRB, 0.65582E+00_JPRB ,&
&  0.65457E+00_JPRB, 0.64919E+00_JPRB, 0.67986E+00_JPRB, 0.68423E+00_JPRB, 0.66840E+00_JPRB, 0.65968E+00_JPRB, 0.64881E+00_JPRB ,&
&  0.61813E+00_JPRB, 0.59979E+00_JPRB, 0.57704E+00_JPRB, 0.57200E+00_JPRB, 0.56678E+00_JPRB, 0.56833E+00_JPRB, 0.56385E+00_JPRB ,&
&  0.55556E+00_JPRB, 0.54661E+00_JPRB, 0.51154E+00_JPRB, 0.46457E+00_JPRB, 0.44039E+00_JPRB, 0.42013E+00_JPRB, 0.40763E+00_JPRB ,&
&  0.40084E+00_JPRB, 0.40783E+00_JPRB, 0.41736E+00_JPRB, 0.41459E+00_JPRB, 0.37837E+00_JPRB, 0.33685E+00_JPRB, 0.30179E+00_JPRB ,&
&  0.27987E+00_JPRB, 0.27282E+00_JPRB, 0.26205E+00_JPRB, 0.25817E+00_JPRB, 0.26435E+00_JPRB, 0.27293E+00_JPRB, 0.27712E+00_JPRB ,&
&  0.28149E+00_JPRB, 0.28956E+00_JPRB, 0.30848E+00_JPRB, 0.32944E+00_JPRB, 0.34400E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.35697E+00_JPRB, 0.36525E+00_JPRB, 0.36960E+00_JPRB, 0.37425E+00_JPRB, 0.37647E+00_JPRB, 0.36660E+00_JPRB, 0.34642E+00_JPRB ,&
&  0.35325E+00_JPRB, 0.30991E+00_JPRB, 0.29602E+00_JPRB, 0.28768E+00_JPRB, 0.28629E+00_JPRB, 0.28146E+00_JPRB, 0.27308E+00_JPRB ,&
&  0.27304E+00_JPRB, 0.28026E+00_JPRB, 0.29229E+00_JPRB, 0.29686E+00_JPRB, 0.29455E+00_JPRB, 0.28669E+00_JPRB, 0.27651E+00_JPRB ,&
&  0.26751E+00_JPRB, 0.26523E+00_JPRB, 0.26993E+00_JPRB, 0.27751E+00_JPRB, 0.28181E+00_JPRB, 0.28087E+00_JPRB, 0.27810E+00_JPRB ,&
&  0.27478E+00_JPRB, 0.27180E+00_JPRB, 0.27143E+00_JPRB, 0.27290E+00_JPRB, 0.27365E+00_JPRB, 0.26983E+00_JPRB, 0.25798E+00_JPRB ,&
&  0.24365E+00_JPRB, 0.25203E+00_JPRB, 0.30679E+00_JPRB, 0.33199E+00_JPRB, 0.35208E+00_JPRB, 0.36291E+00_JPRB, 0.37380E+00_JPRB ,&
&  0.38824E+00_JPRB, 0.40497E+00_JPRB, 0.42494E+00_JPRB, 0.44957E+00_JPRB, 0.46552E+00_JPRB, 0.48000E+00_JPRB, 0.49466E+00_JPRB ,&
&  0.53702E+00_JPRB, 0.45862E+00_JPRB, 0.43873E+00_JPRB, 0.47233E+00_JPRB, 0.70954E+00_JPRB, 0.77835E+00_JPRB, 0.77612E+00_JPRB ,&
&  0.78883E+00_JPRB, 0.74103E+00_JPRB, 0.73524E+00_JPRB, 0.74517E+00_JPRB, 0.74398E+00_JPRB, 0.73601E+00_JPRB, 0.72564E+00_JPRB ,&
&  0.70465E+00_JPRB, 0.67838E+00_JPRB, 0.63769E+00_JPRB, 0.60924E+00_JPRB, 0.58972E+00_JPRB, 0.59189E+00_JPRB, 0.58064E+00_JPRB ,&
&  0.56267E+00_JPRB, 0.55552E+00_JPRB, 0.52179E+00_JPRB, 0.48414E+00_JPRB, 0.45567E+00_JPRB, 0.43694E+00_JPRB, 0.42272E+00_JPRB ,&
&  0.41345E+00_JPRB, 0.41564E+00_JPRB, 0.42165E+00_JPRB, 0.42362E+00_JPRB, 0.40125E+00_JPRB, 0.36771E+00_JPRB, 0.33696E+00_JPRB ,&
&  0.30351E+00_JPRB, 0.28705E+00_JPRB, 0.26842E+00_JPRB, 0.26492E+00_JPRB, 0.27471E+00_JPRB, 0.28659E+00_JPRB, 0.29702E+00_JPRB ,&
&  0.30913E+00_JPRB, 0.31888E+00_JPRB, 0.32899E+00_JPRB, 0.34216E+00_JPRB, 0.34909E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.35934E+00_JPRB, 0.37189E+00_JPRB, 0.38326E+00_JPRB, 0.39482E+00_JPRB, 0.39973E+00_JPRB, 0.38277E+00_JPRB, 0.37734E+00_JPRB ,&
&  0.41676E+00_JPRB, 0.36762E+00_JPRB, 0.38703E+00_JPRB, 0.32767E+00_JPRB, 0.30475E+00_JPRB, 0.29830E+00_JPRB, 0.28715E+00_JPRB ,&
&  0.28457E+00_JPRB, 0.29015E+00_JPRB, 0.29621E+00_JPRB, 0.30103E+00_JPRB, 0.30188E+00_JPRB, 0.29090E+00_JPRB, 0.28085E+00_JPRB ,&
&  0.27085E+00_JPRB, 0.26833E+00_JPRB, 0.27412E+00_JPRB, 0.27999E+00_JPRB, 0.28498E+00_JPRB, 0.28604E+00_JPRB, 0.28379E+00_JPRB ,&
&  0.27766E+00_JPRB, 0.27603E+00_JPRB, 0.27852E+00_JPRB, 0.27917E+00_JPRB, 0.28159E+00_JPRB, 0.28072E+00_JPRB, 0.28170E+00_JPRB ,&
&  0.28714E+00_JPRB, 0.29550E+00_JPRB, 0.33451E+00_JPRB, 0.36174E+00_JPRB, 0.38070E+00_JPRB, 0.39055E+00_JPRB, 0.40472E+00_JPRB ,&
&  0.41922E+00_JPRB, 0.43737E+00_JPRB, 0.46463E+00_JPRB, 0.49846E+00_JPRB, 0.52712E+00_JPRB, 0.54937E+00_JPRB, 0.57605E+00_JPRB ,&
&  0.62620E+00_JPRB, 0.50681E+00_JPRB, 0.58966E+00_JPRB, 0.59787E+00_JPRB, 0.74040E+00_JPRB, 0.90379E+00_JPRB, 0.85974E+00_JPRB ,&
&  0.92508E+00_JPRB, 0.86205E+00_JPRB, 0.84869E+00_JPRB, 0.82502E+00_JPRB, 0.81898E+00_JPRB, 0.81344E+00_JPRB, 0.80605E+00_JPRB ,&
&  0.82507E+00_JPRB, 0.74270E+00_JPRB, 0.72349E+00_JPRB, 0.67745E+00_JPRB, 0.64435E+00_JPRB, 0.63760E+00_JPRB, 0.60895E+00_JPRB ,&
&  0.58751E+00_JPRB, 0.57666E+00_JPRB, 0.55377E+00_JPRB, 0.49509E+00_JPRB, 0.47707E+00_JPRB, 0.46134E+00_JPRB, 0.42868E+00_JPRB ,&
&  0.41655E+00_JPRB, 0.40346E+00_JPRB, 0.39116E+00_JPRB, 0.41106E+00_JPRB, 0.39841E+00_JPRB, 0.36899E+00_JPRB, 0.33841E+00_JPRB ,&
&  0.31228E+00_JPRB, 0.29444E+00_JPRB, 0.30005E+00_JPRB, 0.30333E+00_JPRB, 0.30213E+00_JPRB, 0.30566E+00_JPRB, 0.32640E+00_JPRB ,&
&  0.32751E+00_JPRB, 0.33690E+00_JPRB, 0.35486E+00_JPRB, 0.35908E+00_JPRB, 0.35737E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.36748E+00_JPRB, 0.38283E+00_JPRB, 0.39348E+00_JPRB, 0.40869E+00_JPRB, 0.41676E+00_JPRB, 0.40778E+00_JPRB, 0.38900E+00_JPRB ,&
&  0.39773E+00_JPRB, 0.39602E+00_JPRB, 0.39482E+00_JPRB, 0.39651E+00_JPRB, 0.39863E+00_JPRB, 0.36073E+00_JPRB, 0.29766E+00_JPRB ,&
&  0.27808E+00_JPRB, 0.28305E+00_JPRB, 0.29595E+00_JPRB, 0.30406E+00_JPRB, 0.30252E+00_JPRB, 0.29443E+00_JPRB, 0.28782E+00_JPRB ,&
&  0.28320E+00_JPRB, 0.28036E+00_JPRB, 0.28369E+00_JPRB, 0.28609E+00_JPRB, 0.28734E+00_JPRB, 0.29040E+00_JPRB, 0.29189E+00_JPRB ,&
&  0.28656E+00_JPRB, 0.28625E+00_JPRB, 0.28608E+00_JPRB, 0.28131E+00_JPRB, 0.29271E+00_JPRB, 0.29300E+00_JPRB, 0.29428E+00_JPRB ,&
&  0.31020E+00_JPRB, 0.33451E+00_JPRB, 0.35808E+00_JPRB, 0.38683E+00_JPRB, 0.41094E+00_JPRB, 0.42642E+00_JPRB, 0.44259E+00_JPRB ,&
&  0.46307E+00_JPRB, 0.49235E+00_JPRB, 0.52790E+00_JPRB, 0.56916E+00_JPRB, 0.61502E+00_JPRB, 0.61188E+00_JPRB, 0.68249E+00_JPRB ,&
&  0.69430E+00_JPRB, 0.73639E+00_JPRB, 0.75761E+00_JPRB, 0.69494E+00_JPRB, 0.83884E+00_JPRB, 0.96327E+00_JPRB, 0.90886E+00_JPRB ,&
&  0.90529E+00_JPRB, 0.82683E+00_JPRB, 0.87883E+00_JPRB, 0.92847E+00_JPRB, 0.96558E+00_JPRB, 0.93105E+00_JPRB, 0.90088E+00_JPRB ,&
&  0.94023E+00_JPRB, 0.95244E+00_JPRB, 0.82158E+00_JPRB, 0.77219E+00_JPRB, 0.72446E+00_JPRB, 0.70779E+00_JPRB, 0.66104E+00_JPRB ,&
&  0.63863E+00_JPRB, 0.62901E+00_JPRB, 0.60257E+00_JPRB, 0.57149E+00_JPRB, 0.53551E+00_JPRB, 0.50379E+00_JPRB, 0.47369E+00_JPRB ,&
&  0.43861E+00_JPRB, 0.39442E+00_JPRB, 0.37494E+00_JPRB, 0.37821E+00_JPRB, 0.38812E+00_JPRB, 0.35253E+00_JPRB, 0.32128E+00_JPRB ,&
&  0.31071E+00_JPRB, 0.31301E+00_JPRB, 0.33541E+00_JPRB, 0.33599E+00_JPRB, 0.33279E+00_JPRB, 0.32940E+00_JPRB, 0.32131E+00_JPRB ,&
&  0.31801E+00_JPRB, 0.34573E+00_JPRB, 0.35821E+00_JPRB, 0.36032E+00_JPRB, 0.36055E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.38356E+00_JPRB, 0.40189E+00_JPRB, 0.41033E+00_JPRB, 0.42631E+00_JPRB, 0.43792E+00_JPRB, 0.42547E+00_JPRB, 0.41990E+00_JPRB ,&
&  0.41746E+00_JPRB, 0.41374E+00_JPRB, 0.40796E+00_JPRB, 0.40558E+00_JPRB, 0.40473E+00_JPRB, 0.39784E+00_JPRB, 0.35543E+00_JPRB ,&
&  0.30607E+00_JPRB, 0.30411E+00_JPRB, 0.30010E+00_JPRB, 0.30797E+00_JPRB, 0.31562E+00_JPRB, 0.30671E+00_JPRB, 0.29966E+00_JPRB ,&
&  0.29943E+00_JPRB, 0.29556E+00_JPRB, 0.29354E+00_JPRB, 0.29467E+00_JPRB, 0.29861E+00_JPRB, 0.30571E+00_JPRB, 0.30912E+00_JPRB ,&
&  0.30096E+00_JPRB, 0.29642E+00_JPRB, 0.29319E+00_JPRB, 0.29293E+00_JPRB, 0.30352E+00_JPRB, 0.30892E+00_JPRB, 0.31416E+00_JPRB ,&
&  0.33103E+00_JPRB, 0.35497E+00_JPRB, 0.37960E+00_JPRB, 0.41528E+00_JPRB, 0.44957E+00_JPRB, 0.47794E+00_JPRB, 0.50796E+00_JPRB ,&
&  0.55670E+00_JPRB, 0.60523E+00_JPRB, 0.63719E+00_JPRB, 0.68138E+00_JPRB, 0.71458E+00_JPRB, 0.76176E+00_JPRB, 0.80411E+00_JPRB ,&
&  0.83598E+00_JPRB, 0.85465E+00_JPRB, 0.82793E+00_JPRB, 0.84300E+00_JPRB, 0.97021E+00_JPRB, 0.10222E+01_JPRB, 0.99034E+00_JPRB ,&
&  0.10023E+01_JPRB, 0.91767E+00_JPRB, 0.11178E+01_JPRB, 0.11452E+01_JPRB, 0.10700E+01_JPRB, 0.10631E+01_JPRB, 0.99019E+00_JPRB ,&
&  0.98037E+00_JPRB, 0.93740E+00_JPRB, 0.87226E+00_JPRB, 0.82584E+00_JPRB, 0.78485E+00_JPRB, 0.79224E+00_JPRB, 0.74578E+00_JPRB ,&
&  0.73660E+00_JPRB, 0.71290E+00_JPRB, 0.61760E+00_JPRB, 0.55254E+00_JPRB, 0.48767E+00_JPRB, 0.50950E+00_JPRB, 0.49115E+00_JPRB ,&
&  0.44976E+00_JPRB, 0.38938E+00_JPRB, 0.39732E+00_JPRB, 0.40297E+00_JPRB, 0.42143E+00_JPRB, 0.40846E+00_JPRB, 0.35658E+00_JPRB ,&
&  0.33719E+00_JPRB, 0.35115E+00_JPRB, 0.35314E+00_JPRB, 0.35495E+00_JPRB, 0.35589E+00_JPRB, 0.35270E+00_JPRB, 0.32931E+00_JPRB ,&
&  0.32578E+00_JPRB, 0.35131E+00_JPRB, 0.36439E+00_JPRB, 0.36638E+00_JPRB, 0.37287E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.40456E+00_JPRB, 0.42729E+00_JPRB, 0.43851E+00_JPRB, 0.44883E+00_JPRB, 0.45899E+00_JPRB, 0.44579E+00_JPRB, 0.44067E+00_JPRB ,&
&  0.44164E+00_JPRB, 0.43528E+00_JPRB, 0.42135E+00_JPRB, 0.41812E+00_JPRB, 0.41659E+00_JPRB, 0.40966E+00_JPRB, 0.39887E+00_JPRB ,&
&  0.33356E+00_JPRB, 0.32020E+00_JPRB, 0.30053E+00_JPRB, 0.30509E+00_JPRB, 0.33927E+00_JPRB, 0.32897E+00_JPRB, 0.32307E+00_JPRB ,&
&  0.31810E+00_JPRB, 0.31536E+00_JPRB, 0.30841E+00_JPRB, 0.30711E+00_JPRB, 0.31138E+00_JPRB, 0.32392E+00_JPRB, 0.33731E+00_JPRB ,&
&  0.33107E+00_JPRB, 0.32050E+00_JPRB, 0.31367E+00_JPRB, 0.31262E+00_JPRB, 0.32124E+00_JPRB, 0.33205E+00_JPRB, 0.34636E+00_JPRB ,&
&  0.36633E+00_JPRB, 0.38758E+00_JPRB, 0.42213E+00_JPRB, 0.47287E+00_JPRB, 0.51648E+00_JPRB, 0.55457E+00_JPRB, 0.61378E+00_JPRB ,&
&  0.69097E+00_JPRB, 0.75200E+00_JPRB, 0.79039E+00_JPRB, 0.82995E+00_JPRB, 0.84325E+00_JPRB, 0.89078E+00_JPRB, 0.90841E+00_JPRB ,&
&  0.98053E+00_JPRB, 0.95250E+00_JPRB, 0.94279E+00_JPRB, 0.10578E+01_JPRB, 0.11841E+01_JPRB, 0.12034E+01_JPRB, 0.11394E+01_JPRB ,&
&  0.11092E+01_JPRB, 0.10360E+01_JPRB, 0.10919E+01_JPRB, 0.11388E+01_JPRB, 0.10805E+01_JPRB, 0.11099E+01_JPRB, 0.10214E+01_JPRB ,&
&  0.91679E+00_JPRB, 0.85274E+00_JPRB, 0.82623E+00_JPRB, 0.83027E+00_JPRB, 0.79905E+00_JPRB, 0.83498E+00_JPRB, 0.81707E+00_JPRB ,&
&  0.82479E+00_JPRB, 0.72602E+00_JPRB, 0.55956E+00_JPRB, 0.52175E+00_JPRB, 0.43197E+00_JPRB, 0.44267E+00_JPRB, 0.52932E+00_JPRB ,&
&  0.51351E+00_JPRB, 0.47028E+00_JPRB, 0.46149E+00_JPRB, 0.46659E+00_JPRB, 0.48966E+00_JPRB, 0.53088E+00_JPRB, 0.46510E+00_JPRB ,&
&  0.39624E+00_JPRB, 0.39176E+00_JPRB, 0.39603E+00_JPRB, 0.39198E+00_JPRB, 0.38819E+00_JPRB, 0.38967E+00_JPRB, 0.38378E+00_JPRB ,&
&  0.37975E+00_JPRB, 0.37946E+00_JPRB, 0.38379E+00_JPRB, 0.38561E+00_JPRB, 0.39536E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.42915E+00_JPRB, 0.44889E+00_JPRB, 0.46311E+00_JPRB, 0.46915E+00_JPRB, 0.48143E+00_JPRB, 0.47399E+00_JPRB, 0.46537E+00_JPRB ,&
&  0.46346E+00_JPRB, 0.45376E+00_JPRB, 0.43618E+00_JPRB, 0.43243E+00_JPRB, 0.42774E+00_JPRB, 0.41837E+00_JPRB, 0.40834E+00_JPRB ,&
&  0.40907E+00_JPRB, 0.36917E+00_JPRB, 0.33172E+00_JPRB, 0.31013E+00_JPRB, 0.33529E+00_JPRB, 0.33249E+00_JPRB, 0.33888E+00_JPRB ,&
&  0.33304E+00_JPRB, 0.34142E+00_JPRB, 0.33890E+00_JPRB, 0.33659E+00_JPRB, 0.34577E+00_JPRB, 0.36324E+00_JPRB, 0.37910E+00_JPRB ,&
&  0.37997E+00_JPRB, 0.37701E+00_JPRB, 0.36712E+00_JPRB, 0.36682E+00_JPRB, 0.38625E+00_JPRB, 0.38385E+00_JPRB, 0.40665E+00_JPRB ,&
&  0.42720E+00_JPRB, 0.44932E+00_JPRB, 0.48434E+00_JPRB, 0.54125E+00_JPRB, 0.59484E+00_JPRB, 0.64713E+00_JPRB, 0.72332E+00_JPRB ,&
&  0.79354E+00_JPRB, 0.82234E+00_JPRB, 0.82478E+00_JPRB, 0.84775E+00_JPRB, 0.91696E+00_JPRB, 0.10041E+01_JPRB, 0.99623E+00_JPRB ,&
&  0.99071E+00_JPRB, 0.92662E+00_JPRB, 0.87399E+00_JPRB, 0.10620E+01_JPRB, 0.12285E+01_JPRB, 0.12248E+01_JPRB, 0.11789E+01_JPRB ,&
&  0.11749E+01_JPRB, 0.11645E+01_JPRB, 0.12349E+01_JPRB, 0.12357E+01_JPRB, 0.12058E+01_JPRB, 0.11565E+01_JPRB, 0.10362E+01_JPRB ,&
&  0.90729E+00_JPRB, 0.84884E+00_JPRB, 0.87534E+00_JPRB, 0.88626E+00_JPRB, 0.88959E+00_JPRB, 0.84951E+00_JPRB, 0.89082E+00_JPRB ,&
&  0.95565E+00_JPRB, 0.55149E+00_JPRB, 0.45228E+00_JPRB, 0.51642E+00_JPRB, 0.45380E+00_JPRB, 0.46026E+00_JPRB, 0.50890E+00_JPRB ,&
&  0.49036E+00_JPRB, 0.47805E+00_JPRB, 0.51921E+00_JPRB, 0.52963E+00_JPRB, 0.57441E+00_JPRB, 0.63427E+00_JPRB, 0.56723E+00_JPRB ,&
&  0.50368E+00_JPRB, 0.47139E+00_JPRB, 0.47225E+00_JPRB, 0.45841E+00_JPRB, 0.45834E+00_JPRB, 0.45106E+00_JPRB, 0.44625E+00_JPRB ,&
&  0.43476E+00_JPRB, 0.42792E+00_JPRB, 0.42133E+00_JPRB, 0.42143E+00_JPRB, 0.42545E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.47339E+00_JPRB, 0.48935E+00_JPRB, 0.49839E+00_JPRB, 0.50477E+00_JPRB, 0.51475E+00_JPRB, 0.51205E+00_JPRB, 0.49832E+00_JPRB ,&
&  0.48425E+00_JPRB, 0.47114E+00_JPRB, 0.45245E+00_JPRB, 0.44688E+00_JPRB, 0.43828E+00_JPRB, 0.42713E+00_JPRB, 0.41778E+00_JPRB ,&
&  0.41659E+00_JPRB, 0.40345E+00_JPRB, 0.34659E+00_JPRB, 0.32513E+00_JPRB, 0.30915E+00_JPRB, 0.32172E+00_JPRB, 0.33371E+00_JPRB ,&
&  0.35159E+00_JPRB, 0.35848E+00_JPRB, 0.36468E+00_JPRB, 0.37746E+00_JPRB, 0.40148E+00_JPRB, 0.41936E+00_JPRB, 0.44750E+00_JPRB ,&
&  0.45476E+00_JPRB, 0.46268E+00_JPRB, 0.43747E+00_JPRB, 0.44685E+00_JPRB, 0.46942E+00_JPRB, 0.46664E+00_JPRB, 0.51003E+00_JPRB ,&
&  0.53841E+00_JPRB, 0.57266E+00_JPRB, 0.60782E+00_JPRB, 0.63666E+00_JPRB, 0.67200E+00_JPRB, 0.69392E+00_JPRB, 0.74432E+00_JPRB ,&
&  0.77929E+00_JPRB, 0.79635E+00_JPRB, 0.79017E+00_JPRB, 0.79579E+00_JPRB, 0.83666E+00_JPRB, 0.88425E+00_JPRB, 0.88163E+00_JPRB ,&
&  0.90529E+00_JPRB, 0.76655E+00_JPRB, 0.58711E+00_JPRB, 0.86155E+00_JPRB, 0.12111E+01_JPRB, 0.11671E+01_JPRB, 0.10028E+01_JPRB ,&
&  0.10750E+01_JPRB, 0.11409E+01_JPRB, 0.12800E+01_JPRB, 0.13069E+01_JPRB, 0.12714E+01_JPRB, 0.11979E+01_JPRB, 0.11185E+01_JPRB ,&
&  0.10367E+01_JPRB, 0.10512E+01_JPRB, 0.11342E+01_JPRB, 0.12692E+01_JPRB, 0.12858E+01_JPRB, 0.12598E+01_JPRB, 0.11936E+01_JPRB ,&
&  0.10015E+01_JPRB, 0.82818E+00_JPRB, 0.57990E+00_JPRB, 0.59650E+00_JPRB, 0.48115E+00_JPRB, 0.47843E+00_JPRB, 0.52043E+00_JPRB ,&
&  0.50252E+00_JPRB, 0.52777E+00_JPRB, 0.57921E+00_JPRB, 0.60892E+00_JPRB, 0.74138E+00_JPRB, 0.73440E+00_JPRB, 0.69270E+00_JPRB ,&
&  0.62066E+00_JPRB, 0.54840E+00_JPRB, 0.55869E+00_JPRB, 0.54891E+00_JPRB, 0.55149E+00_JPRB, 0.53696E+00_JPRB, 0.52181E+00_JPRB ,&
&  0.50230E+00_JPRB, 0.48760E+00_JPRB, 0.47820E+00_JPRB, 0.47054E+00_JPRB, 0.46690E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.51363E+00_JPRB, 0.51427E+00_JPRB, 0.51008E+00_JPRB, 0.50802E+00_JPRB, 0.51108E+00_JPRB, 0.50836E+00_JPRB, 0.49964E+00_JPRB ,&
&  0.49573E+00_JPRB, 0.48402E+00_JPRB, 0.46927E+00_JPRB, 0.45668E+00_JPRB, 0.45014E+00_JPRB, 0.43955E+00_JPRB, 0.43161E+00_JPRB ,&
&  0.42847E+00_JPRB, 0.39084E+00_JPRB, 0.32932E+00_JPRB, 0.30335E+00_JPRB, 0.29047E+00_JPRB, 0.30172E+00_JPRB, 0.32179E+00_JPRB ,&
&  0.34655E+00_JPRB, 0.36311E+00_JPRB, 0.39204E+00_JPRB, 0.43193E+00_JPRB, 0.47968E+00_JPRB, 0.52050E+00_JPRB, 0.55133E+00_JPRB ,&
&  0.63787E+00_JPRB, 0.57363E+00_JPRB, 0.58239E+00_JPRB, 0.56521E+00_JPRB, 0.59713E+00_JPRB, 0.61222E+00_JPRB, 0.61517E+00_JPRB ,&
&  0.61602E+00_JPRB, 0.62007E+00_JPRB, 0.62786E+00_JPRB, 0.62856E+00_JPRB, 0.64058E+00_JPRB, 0.67357E+00_JPRB, 0.74197E+00_JPRB ,&
&  0.80403E+00_JPRB, 0.79526E+00_JPRB, 0.76405E+00_JPRB, 0.78754E+00_JPRB, 0.84119E+00_JPRB, 0.87129E+00_JPRB, 0.86729E+00_JPRB ,&
&  0.82642E+00_JPRB, 0.76930E+00_JPRB, 0.92436E+00_JPRB, 0.10478E+01_JPRB, 0.12103E+01_JPRB, 0.12226E+01_JPRB, 0.11334E+01_JPRB ,&
&  0.11833E+01_JPRB, 0.12090E+01_JPRB, 0.12830E+01_JPRB, 0.10352E+01_JPRB, 0.11523E+01_JPRB, 0.11506E+01_JPRB, 0.11444E+01_JPRB ,&
&  0.11468E+01_JPRB, 0.12295E+01_JPRB, 0.13322E+01_JPRB, 0.15248E+01_JPRB, 0.16665E+01_JPRB, 0.17520E+01_JPRB, 0.13192E+01_JPRB ,&
&  0.99594E+00_JPRB, 0.10336E+01_JPRB, 0.92225E+00_JPRB, 0.74689E+00_JPRB, 0.61500E+00_JPRB, 0.61236E+00_JPRB, 0.60113E+00_JPRB ,&
&  0.59488E+00_JPRB, 0.61933E+00_JPRB, 0.66256E+00_JPRB, 0.80946E+00_JPRB, 0.95810E+00_JPRB, 0.95098E+00_JPRB, 0.84935E+00_JPRB ,&
&  0.71417E+00_JPRB, 0.71857E+00_JPRB, 0.68728E+00_JPRB, 0.65792E+00_JPRB, 0.67116E+00_JPRB, 0.64685E+00_JPRB, 0.60648E+00_JPRB ,&
&  0.56793E+00_JPRB, 0.54838E+00_JPRB, 0.53576E+00_JPRB, 0.52395E+00_JPRB, 0.51460E+00_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.54493E+00_JPRB, 0.54187E+00_JPRB, 0.53387E+00_JPRB, 0.52781E+00_JPRB, 0.52125E+00_JPRB, 0.51286E+00_JPRB, 0.50446E+00_JPRB ,&
&  0.50525E+00_JPRB, 0.48191E+00_JPRB, 0.47329E+00_JPRB, 0.46618E+00_JPRB, 0.46169E+00_JPRB, 0.45493E+00_JPRB, 0.45070E+00_JPRB ,&
&  0.44638E+00_JPRB, 0.37743E+00_JPRB, 0.31581E+00_JPRB, 0.31118E+00_JPRB, 0.31213E+00_JPRB, 0.30668E+00_JPRB, 0.32265E+00_JPRB ,&
&  0.34623E+00_JPRB, 0.37406E+00_JPRB, 0.41448E+00_JPRB, 0.49956E+00_JPRB, 0.58875E+00_JPRB, 0.66402E+00_JPRB, 0.73276E+00_JPRB ,&
&  0.72796E+00_JPRB, 0.73840E+00_JPRB, 0.76094E+00_JPRB, 0.71872E+00_JPRB, 0.68031E+00_JPRB, 0.65976E+00_JPRB, 0.65599E+00_JPRB ,&
&  0.65451E+00_JPRB, 0.64413E+00_JPRB, 0.64835E+00_JPRB, 0.65885E+00_JPRB, 0.67946E+00_JPRB, 0.72459E+00_JPRB, 0.78497E+00_JPRB ,&
&  0.81227E+00_JPRB, 0.79265E+00_JPRB, 0.77758E+00_JPRB, 0.83595E+00_JPRB, 0.76766E+00_JPRB, 0.73420E+00_JPRB, 0.71112E+00_JPRB ,&
&  0.89935E+00_JPRB, 0.10145E+01_JPRB, 0.10138E+01_JPRB, 0.11152E+01_JPRB, 0.10290E+01_JPRB, 0.10720E+01_JPRB, 0.12223E+01_JPRB ,&
&  0.13210E+01_JPRB, 0.13307E+01_JPRB, 0.13760E+01_JPRB, 0.94963E+00_JPRB, 0.84838E+00_JPRB, 0.11446E+01_JPRB, 0.12203E+01_JPRB ,&
&  0.12495E+01_JPRB, 0.14279E+01_JPRB, 0.16848E+01_JPRB, 0.19893E+01_JPRB, 0.15210E+01_JPRB, 0.12945E+01_JPRB, 0.96555E+00_JPRB ,&
&  0.92162E+00_JPRB, 0.83209E+00_JPRB, 0.92195E+00_JPRB, 0.84525E+00_JPRB, 0.81209E+00_JPRB, 0.77624E+00_JPRB, 0.72068E+00_JPRB ,&
&  0.71447E+00_JPRB, 0.71273E+00_JPRB, 0.82481E+00_JPRB, 0.10593E+01_JPRB, 0.12515E+01_JPRB, 0.11273E+01_JPRB, 0.10742E+01_JPRB ,&
&  0.10181E+01_JPRB, 0.94752E+00_JPRB, 0.84831E+00_JPRB, 0.81627E+00_JPRB, 0.77390E+00_JPRB, 0.72274E+00_JPRB, 0.66519E+00_JPRB ,&
&  0.61889E+00_JPRB, 0.59419E+00_JPRB, 0.57219E+00_JPRB, 0.55669E+00_JPRB, 0.54850E+00_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.57769E+00_JPRB, 0.56925E+00_JPRB, 0.56031E+00_JPRB, 0.55064E+00_JPRB, 0.54028E+00_JPRB, 0.52891E+00_JPRB, 0.52127E+00_JPRB ,&
&  0.52612E+00_JPRB, 0.49929E+00_JPRB, 0.47785E+00_JPRB, 0.47099E+00_JPRB, 0.46999E+00_JPRB, 0.47089E+00_JPRB, 0.47258E+00_JPRB ,&
&  0.46931E+00_JPRB, 0.42147E+00_JPRB, 0.33917E+00_JPRB, 0.31738E+00_JPRB, 0.33400E+00_JPRB, 0.32604E+00_JPRB, 0.32735E+00_JPRB ,&
&  0.35960E+00_JPRB, 0.40324E+00_JPRB, 0.47618E+00_JPRB, 0.61030E+00_JPRB, 0.73472E+00_JPRB, 0.84582E+00_JPRB, 0.91476E+00_JPRB ,&
&  0.91771E+00_JPRB, 0.84222E+00_JPRB, 0.74992E+00_JPRB, 0.66687E+00_JPRB, 0.62641E+00_JPRB, 0.61152E+00_JPRB, 0.61901E+00_JPRB ,&
&  0.63343E+00_JPRB, 0.64216E+00_JPRB, 0.65242E+00_JPRB, 0.65743E+00_JPRB, 0.69564E+00_JPRB, 0.72092E+00_JPRB, 0.76429E+00_JPRB ,&
&  0.78609E+00_JPRB, 0.78372E+00_JPRB, 0.80986E+00_JPRB, 0.85625E+00_JPRB, 0.83285E+00_JPRB, 0.77539E+00_JPRB, 0.76660E+00_JPRB ,&
&  0.85556E+00_JPRB, 0.95137E+00_JPRB, 0.10289E+01_JPRB, 0.11331E+01_JPRB, 0.12338E+01_JPRB, 0.12727E+01_JPRB, 0.14244E+01_JPRB ,&
&  0.12178E+01_JPRB, 0.98510E+00_JPRB, 0.82807E+00_JPRB, 0.71671E+00_JPRB, 0.98041E+00_JPRB, 0.13835E+01_JPRB, 0.15638E+01_JPRB ,&
&  0.17460E+01_JPRB, 0.19425E+01_JPRB, 0.21520E+01_JPRB, 0.19199E+01_JPRB, 0.14656E+01_JPRB, 0.10940E+01_JPRB, 0.12705E+01_JPRB ,&
&  0.12940E+01_JPRB, 0.12498E+01_JPRB, 0.11413E+01_JPRB, 0.99821E+00_JPRB, 0.79614E+00_JPRB, 0.89487E+00_JPRB, 0.86008E+00_JPRB ,&
&  0.82726E+00_JPRB, 0.82160E+00_JPRB, 0.11244E+01_JPRB, 0.14310E+01_JPRB, 0.14818E+01_JPRB, 0.14628E+01_JPRB, 0.13991E+01_JPRB ,&
&  0.12679E+01_JPRB, 0.11879E+01_JPRB, 0.10561E+01_JPRB, 0.93708E+00_JPRB, 0.83990E+00_JPRB, 0.75298E+00_JPRB, 0.68604E+00_JPRB ,&
&  0.64739E+00_JPRB, 0.62308E+00_JPRB, 0.59764E+00_JPRB, 0.58804E+00_JPRB, 0.58194E+00_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.62907E+00_JPRB, 0.61442E+00_JPRB, 0.60149E+00_JPRB, 0.58984E+00_JPRB, 0.57646E+00_JPRB, 0.55576E+00_JPRB, 0.54943E+00_JPRB ,&
&  0.54754E+00_JPRB, 0.52207E+00_JPRB, 0.49774E+00_JPRB, 0.48990E+00_JPRB, 0.49636E+00_JPRB, 0.49991E+00_JPRB, 0.49793E+00_JPRB ,&
&  0.49505E+00_JPRB, 0.50018E+00_JPRB, 0.39421E+00_JPRB, 0.35203E+00_JPRB, 0.35244E+00_JPRB, 0.34834E+00_JPRB, 0.33544E+00_JPRB ,&
&  0.38634E+00_JPRB, 0.45936E+00_JPRB, 0.55363E+00_JPRB, 0.70504E+00_JPRB, 0.91361E+00_JPRB, 0.92086E+00_JPRB, 0.91852E+00_JPRB ,&
&  0.91790E+00_JPRB, 0.84967E+00_JPRB, 0.75889E+00_JPRB, 0.69289E+00_JPRB, 0.65886E+00_JPRB, 0.64779E+00_JPRB, 0.65465E+00_JPRB ,&
&  0.67104E+00_JPRB, 0.68266E+00_JPRB, 0.68958E+00_JPRB, 0.71210E+00_JPRB, 0.71704E+00_JPRB, 0.73245E+00_JPRB, 0.77496E+00_JPRB ,&
&  0.78871E+00_JPRB, 0.77697E+00_JPRB, 0.80241E+00_JPRB, 0.88365E+00_JPRB, 0.92481E+00_JPRB, 0.84975E+00_JPRB, 0.10058E+01_JPRB ,&
&  0.94163E+00_JPRB, 0.10618E+01_JPRB, 0.11348E+01_JPRB, 0.14393E+01_JPRB, 0.14966E+01_JPRB, 0.13922E+01_JPRB, 0.14283E+01_JPRB ,&
&  0.11795E+01_JPRB, 0.10581E+01_JPRB, 0.11625E+01_JPRB, 0.12867E+01_JPRB, 0.13874E+01_JPRB, 0.14864E+01_JPRB, 0.22306E+01_JPRB ,&
&  0.19207E+01_JPRB, 0.21344E+01_JPRB, 0.22754E+01_JPRB, 0.21802E+01_JPRB, 0.10550E+01_JPRB, 0.76416E+00_JPRB, 0.10578E+01_JPRB ,&
&  0.11307E+01_JPRB, 0.79388E+00_JPRB, 0.58634E+00_JPRB, 0.70246E+00_JPRB, 0.54600E+00_JPRB, 0.64169E+00_JPRB, 0.82905E+00_JPRB ,&
&  0.88240E+00_JPRB, 0.97143E+00_JPRB, 0.13787E+01_JPRB, 0.16089E+01_JPRB, 0.16608E+01_JPRB, 0.19221E+01_JPRB, 0.16377E+01_JPRB ,&
&  0.14933E+01_JPRB, 0.13168E+01_JPRB, 0.12449E+01_JPRB, 0.10733E+01_JPRB, 0.93614E+00_JPRB, 0.84486E+00_JPRB, 0.77776E+00_JPRB ,&
&  0.73042E+00_JPRB, 0.69050E+00_JPRB, 0.66389E+00_JPRB, 0.65128E+00_JPRB, 0.64154E+00_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.69367E+00_JPRB, 0.67009E+00_JPRB, 0.64970E+00_JPRB, 0.63419E+00_JPRB, 0.61259E+00_JPRB, 0.59007E+00_JPRB, 0.58539E+00_JPRB ,&
&  0.57173E+00_JPRB, 0.54832E+00_JPRB, 0.53187E+00_JPRB, 0.51586E+00_JPRB, 0.51647E+00_JPRB, 0.51912E+00_JPRB, 0.50652E+00_JPRB ,&
&  0.49152E+00_JPRB, 0.48398E+00_JPRB, 0.45453E+00_JPRB, 0.41547E+00_JPRB, 0.39446E+00_JPRB, 0.37937E+00_JPRB, 0.38540E+00_JPRB ,&
&  0.45972E+00_JPRB, 0.57890E+00_JPRB, 0.68354E+00_JPRB, 0.80227E+00_JPRB, 0.90988E+00_JPRB, 0.89575E+00_JPRB, 0.95186E+00_JPRB ,&
&  0.96556E+00_JPRB, 0.91988E+00_JPRB, 0.83969E+00_JPRB, 0.76825E+00_JPRB, 0.73076E+00_JPRB, 0.70941E+00_JPRB, 0.69582E+00_JPRB ,&
&  0.70003E+00_JPRB, 0.71626E+00_JPRB, 0.72994E+00_JPRB, 0.72539E+00_JPRB, 0.71831E+00_JPRB, 0.73774E+00_JPRB, 0.78077E+00_JPRB ,&
&  0.78999E+00_JPRB, 0.77518E+00_JPRB, 0.81155E+00_JPRB, 0.86687E+00_JPRB, 0.10002E+01_JPRB, 0.10294E+01_JPRB, 0.87457E+00_JPRB ,&
&  0.78117E+00_JPRB, 0.86518E+00_JPRB, 0.11966E+01_JPRB, 0.13992E+01_JPRB, 0.13932E+01_JPRB, 0.13579E+01_JPRB, 0.12609E+01_JPRB ,&
&  0.12363E+01_JPRB, 0.14387E+01_JPRB, 0.15705E+01_JPRB, 0.17587E+01_JPRB, 0.18471E+01_JPRB, 0.14975E+01_JPRB, 0.20048E+01_JPRB ,&
&  0.21443E+01_JPRB, 0.21221E+01_JPRB, 0.17025E+01_JPRB, 0.11764E+01_JPRB, 0.11526E+01_JPRB, 0.82661E+00_JPRB, 0.56841E+00_JPRB ,&
&  0.53355E+00_JPRB, 0.51053E+00_JPRB, 0.46569E+00_JPRB, 0.47143E+00_JPRB, 0.46017E+00_JPRB, 0.58168E+00_JPRB, 0.79321E+00_JPRB ,&
&  0.10286E+01_JPRB, 0.12752E+01_JPRB, 0.15840E+01_JPRB, 0.17870E+01_JPRB, 0.17055E+01_JPRB, 0.18898E+01_JPRB, 0.18121E+01_JPRB ,&
&  0.14656E+01_JPRB, 0.13432E+01_JPRB, 0.12184E+01_JPRB, 0.10766E+01_JPRB, 0.97220E+00_JPRB, 0.90605E+00_JPRB, 0.85649E+00_JPRB ,&
&  0.80806E+00_JPRB, 0.76681E+00_JPRB, 0.74081E+00_JPRB, 0.72627E+00_JPRB, 0.71290E+00_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.74786E+00_JPRB, 0.72230E+00_JPRB, 0.70105E+00_JPRB, 0.68419E+00_JPRB, 0.65986E+00_JPRB, 0.63769E+00_JPRB, 0.62762E+00_JPRB ,&
&  0.60474E+00_JPRB, 0.57863E+00_JPRB, 0.55707E+00_JPRB, 0.54032E+00_JPRB, 0.53431E+00_JPRB, 0.53160E+00_JPRB, 0.51978E+00_JPRB ,&
&  0.50392E+00_JPRB, 0.49093E+00_JPRB, 0.48793E+00_JPRB, 0.47323E+00_JPRB, 0.47003E+00_JPRB, 0.46806E+00_JPRB, 0.50472E+00_JPRB ,&
&  0.61127E+00_JPRB, 0.77725E+00_JPRB, 0.87557E+00_JPRB, 0.96456E+00_JPRB, 0.10539E+01_JPRB, 0.10720E+01_JPRB, 0.10761E+01_JPRB ,&
&  0.10561E+01_JPRB, 0.10122E+01_JPRB, 0.92390E+00_JPRB, 0.83649E+00_JPRB, 0.77802E+00_JPRB, 0.74819E+00_JPRB, 0.73587E+00_JPRB ,&
&  0.73382E+00_JPRB, 0.75861E+00_JPRB, 0.75524E+00_JPRB, 0.73339E+00_JPRB, 0.72187E+00_JPRB, 0.75445E+00_JPRB, 0.79201E+00_JPRB ,&
&  0.80018E+00_JPRB, 0.82103E+00_JPRB, 0.87131E+00_JPRB, 0.88938E+00_JPRB, 0.83869E+00_JPRB, 0.71612E+00_JPRB, 0.78625E+00_JPRB ,&
&  0.87949E+00_JPRB, 0.10811E+01_JPRB, 0.11421E+01_JPRB, 0.14829E+01_JPRB, 0.15004E+01_JPRB, 0.13607E+01_JPRB, 0.13482E+01_JPRB ,&
&  0.14248E+01_JPRB, 0.18943E+01_JPRB, 0.16784E+01_JPRB, 0.16736E+01_JPRB, 0.20179E+01_JPRB, 0.16528E+01_JPRB, 0.17707E+01_JPRB ,&
&  0.20689E+01_JPRB, 0.19243E+01_JPRB, 0.15984E+01_JPRB, 0.11953E+01_JPRB, 0.14134E+01_JPRB, 0.12583E+01_JPRB, 0.66464E+00_JPRB ,&
&  0.55223E+00_JPRB, 0.51524E+00_JPRB, 0.48708E+00_JPRB, 0.46922E+00_JPRB, 0.46768E+00_JPRB, 0.63856E+00_JPRB, 0.10133E+01_JPRB ,&
&  0.11452E+01_JPRB, 0.15549E+01_JPRB, 0.18278E+01_JPRB, 0.19309E+01_JPRB, 0.17076E+01_JPRB, 0.16440E+01_JPRB, 0.16564E+01_JPRB ,&
&  0.14458E+01_JPRB, 0.14006E+01_JPRB, 0.12482E+01_JPRB, 0.11386E+01_JPRB, 0.10507E+01_JPRB, 0.98256E+00_JPRB, 0.92260E+00_JPRB ,&
&  0.86698E+00_JPRB, 0.83052E+00_JPRB, 0.80372E+00_JPRB, 0.79251E+00_JPRB, 0.77134E+00_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.78485E+00_JPRB, 0.76693E+00_JPRB, 0.74186E+00_JPRB, 0.72244E+00_JPRB, 0.69869E+00_JPRB, 0.67329E+00_JPRB, 0.67054E+00_JPRB ,&
&  0.65055E+00_JPRB, 0.61922E+00_JPRB, 0.58810E+00_JPRB, 0.57354E+00_JPRB, 0.56059E+00_JPRB, 0.54407E+00_JPRB, 0.52914E+00_JPRB ,&
&  0.51309E+00_JPRB, 0.50401E+00_JPRB, 0.51032E+00_JPRB, 0.55784E+00_JPRB, 0.58052E+00_JPRB, 0.59256E+00_JPRB, 0.64517E+00_JPRB ,&
&  0.83600E+00_JPRB, 0.96553E+00_JPRB, 0.10176E+01_JPRB, 0.10698E+01_JPRB, 0.10794E+01_JPRB, 0.10713E+01_JPRB, 0.10638E+01_JPRB ,&
&  0.10301E+01_JPRB, 0.99615E+00_JPRB, 0.92880E+00_JPRB, 0.86161E+00_JPRB, 0.80869E+00_JPRB, 0.78672E+00_JPRB, 0.78061E+00_JPRB ,&
&  0.79919E+00_JPRB, 0.78737E+00_JPRB, 0.76580E+00_JPRB, 0.73422E+00_JPRB, 0.72999E+00_JPRB, 0.76824E+00_JPRB, 0.80686E+00_JPRB ,&
&  0.82563E+00_JPRB, 0.88454E+00_JPRB, 0.92199E+00_JPRB, 0.89822E+00_JPRB, 0.74200E+00_JPRB, 0.82560E+00_JPRB, 0.86023E+00_JPRB ,&
&  0.91025E+00_JPRB, 0.97431E+00_JPRB, 0.98214E+00_JPRB, 0.12476E+01_JPRB, 0.14344E+01_JPRB, 0.13507E+01_JPRB, 0.12953E+01_JPRB ,&
&  0.15919E+01_JPRB, 0.16277E+01_JPRB, 0.15000E+01_JPRB, 0.15732E+01_JPRB, 0.17509E+01_JPRB, 0.21652E+01_JPRB, 0.14706E+01_JPRB ,&
&  0.13487E+01_JPRB, 0.16111E+01_JPRB, 0.14117E+01_JPRB, 0.11060E+01_JPRB, 0.12675E+01_JPRB, 0.13876E+01_JPRB, 0.11370E+01_JPRB ,&
&  0.63138E+00_JPRB, 0.52725E+00_JPRB, 0.55248E+00_JPRB, 0.59182E+00_JPRB, 0.55298E+00_JPRB, 0.75722E+00_JPRB, 0.16455E+01_JPRB ,&
&  0.15574E+01_JPRB, 0.18792E+01_JPRB, 0.19537E+01_JPRB, 0.18403E+01_JPRB, 0.16037E+01_JPRB, 0.14652E+01_JPRB, 0.15551E+01_JPRB ,&
&  0.13903E+01_JPRB, 0.13086E+01_JPRB, 0.12440E+01_JPRB, 0.11830E+01_JPRB, 0.11086E+01_JPRB, 0.10236E+01_JPRB, 0.95301E+00_JPRB ,&
&  0.90125E+00_JPRB, 0.87079E+00_JPRB, 0.84330E+00_JPRB, 0.82290E+00_JPRB, 0.79931E+00_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.82894E+00_JPRB, 0.80009E+00_JPRB, 0.77311E+00_JPRB, 0.75581E+00_JPRB, 0.72866E+00_JPRB, 0.70532E+00_JPRB, 0.72516E+00_JPRB ,&
&  0.71328E+00_JPRB, 0.66996E+00_JPRB, 0.61725E+00_JPRB, 0.59541E+00_JPRB, 0.57666E+00_JPRB, 0.55631E+00_JPRB, 0.54313E+00_JPRB ,&
&  0.53167E+00_JPRB, 0.52890E+00_JPRB, 0.53159E+00_JPRB, 0.57994E+00_JPRB, 0.62352E+00_JPRB, 0.60968E+00_JPRB, 0.69338E+00_JPRB ,&
&  0.96373E+00_JPRB, 0.10833E+01_JPRB, 0.10978E+01_JPRB, 0.11293E+01_JPRB, 0.11299E+01_JPRB, 0.11118E+01_JPRB, 0.10835E+01_JPRB ,&
&  0.10017E+01_JPRB, 0.96248E+00_JPRB, 0.89162E+00_JPRB, 0.86421E+00_JPRB, 0.84225E+00_JPRB, 0.82401E+00_JPRB, 0.82299E+00_JPRB ,&
&  0.79507E+00_JPRB, 0.77222E+00_JPRB, 0.74565E+00_JPRB, 0.71740E+00_JPRB, 0.73924E+00_JPRB, 0.79302E+00_JPRB, 0.84202E+00_JPRB ,&
&  0.88546E+00_JPRB, 0.94486E+00_JPRB, 0.95163E+00_JPRB, 0.87192E+00_JPRB, 0.84372E+00_JPRB, 0.84272E+00_JPRB, 0.88347E+00_JPRB ,&
&  0.88820E+00_JPRB, 0.95246E+00_JPRB, 0.10480E+01_JPRB, 0.11717E+01_JPRB, 0.13022E+01_JPRB, 0.13097E+01_JPRB, 0.12924E+01_JPRB ,&
&  0.14527E+01_JPRB, 0.14524E+01_JPRB, 0.13346E+01_JPRB, 0.14634E+01_JPRB, 0.16249E+01_JPRB, 0.19910E+01_JPRB, 0.19547E+01_JPRB ,&
&  0.16105E+01_JPRB, 0.12826E+01_JPRB, 0.11632E+01_JPRB, 0.11371E+01_JPRB, 0.12112E+01_JPRB, 0.13039E+01_JPRB, 0.16374E+01_JPRB ,&
&  0.16428E+01_JPRB, 0.11654E+01_JPRB, 0.10043E+01_JPRB, 0.12616E+01_JPRB, 0.11122E+01_JPRB, 0.10604E+01_JPRB, 0.13921E+01_JPRB ,&
&  0.17102E+01_JPRB, 0.19505E+01_JPRB, 0.19054E+01_JPRB, 0.17144E+01_JPRB, 0.15922E+01_JPRB, 0.14619E+01_JPRB, 0.14864E+01_JPRB ,&
&  0.13807E+01_JPRB, 0.12928E+01_JPRB, 0.12610E+01_JPRB, 0.12197E+01_JPRB, 0.11372E+01_JPRB, 0.10398E+01_JPRB, 0.95825E+00_JPRB ,&
&  0.91410E+00_JPRB, 0.88917E+00_JPRB, 0.87052E+00_JPRB, 0.85817E+00_JPRB, 0.84458E+00_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.85769E+00_JPRB, 0.82432E+00_JPRB, 0.80267E+00_JPRB, 0.78208E+00_JPRB, 0.75224E+00_JPRB, 0.74662E+00_JPRB, 0.79810E+00_JPRB ,&
&  0.80019E+00_JPRB, 0.70930E+00_JPRB, 0.65908E+00_JPRB, 0.61835E+00_JPRB, 0.58991E+00_JPRB, 0.57166E+00_JPRB, 0.56294E+00_JPRB ,&
&  0.55619E+00_JPRB, 0.55886E+00_JPRB, 0.55935E+00_JPRB, 0.61266E+00_JPRB, 0.71347E+00_JPRB, 0.88304E+00_JPRB, 0.73133E+00_JPRB ,&
&  0.10148E+01_JPRB, 0.12429E+01_JPRB, 0.12407E+01_JPRB, 0.11874E+01_JPRB, 0.11144E+01_JPRB, 0.11194E+01_JPRB, 0.10600E+01_JPRB ,&
&  0.96162E+00_JPRB, 0.91509E+00_JPRB, 0.86099E+00_JPRB, 0.83202E+00_JPRB, 0.82615E+00_JPRB, 0.81017E+00_JPRB, 0.77937E+00_JPRB ,&
&  0.75709E+00_JPRB, 0.73866E+00_JPRB, 0.72330E+00_JPRB, 0.72878E+00_JPRB, 0.76221E+00_JPRB, 0.81773E+00_JPRB, 0.88528E+00_JPRB ,&
&  0.96379E+00_JPRB, 0.99871E+00_JPRB, 0.98050E+00_JPRB, 0.89309E+00_JPRB, 0.88594E+00_JPRB, 0.88094E+00_JPRB, 0.89075E+00_JPRB ,&
&  0.86166E+00_JPRB, 0.83915E+00_JPRB, 0.98728E+00_JPRB, 0.11509E+01_JPRB, 0.12323E+01_JPRB, 0.11854E+01_JPRB, 0.11629E+01_JPRB ,&
&  0.13622E+01_JPRB, 0.15347E+01_JPRB, 0.16565E+01_JPRB, 0.15935E+01_JPRB, 0.17655E+01_JPRB, 0.19751E+01_JPRB, 0.21289E+01_JPRB ,&
&  0.20346E+01_JPRB, 0.15682E+01_JPRB, 0.13279E+01_JPRB, 0.12002E+01_JPRB, 0.11635E+01_JPRB, 0.12568E+01_JPRB, 0.15395E+01_JPRB ,&
&  0.18733E+01_JPRB, 0.23350E+01_JPRB, 0.23466E+01_JPRB, 0.19046E+01_JPRB, 0.16959E+01_JPRB, 0.15402E+01_JPRB, 0.14712E+01_JPRB ,&
&  0.18706E+01_JPRB, 0.19434E+01_JPRB, 0.18544E+01_JPRB, 0.20337E+01_JPRB, 0.16828E+01_JPRB, 0.15113E+01_JPRB, 0.14407E+01_JPRB ,&
&  0.13856E+01_JPRB, 0.13198E+01_JPRB, 0.12575E+01_JPRB, 0.11957E+01_JPRB, 0.10967E+01_JPRB, 0.10076E+01_JPRB, 0.95502E+00_JPRB ,&
&  0.92557E+00_JPRB, 0.91083E+00_JPRB, 0.89606E+00_JPRB, 0.88831E+00_JPRB, 0.87930E+00_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.86481E+00_JPRB, 0.83538E+00_JPRB, 0.80105E+00_JPRB, 0.79171E+00_JPRB, 0.78316E+00_JPRB, 0.79332E+00_JPRB, 0.82388E+00_JPRB ,&
&  0.91038E+00_JPRB, 0.76181E+00_JPRB, 0.68051E+00_JPRB, 0.62385E+00_JPRB, 0.59844E+00_JPRB, 0.59216E+00_JPRB, 0.58685E+00_JPRB ,&
&  0.58999E+00_JPRB, 0.60041E+00_JPRB, 0.62454E+00_JPRB, 0.65919E+00_JPRB, 0.78041E+00_JPRB, 0.97269E+00_JPRB, 0.10103E+01_JPRB ,&
&  0.11477E+01_JPRB, 0.15889E+01_JPRB, 0.13630E+01_JPRB, 0.12159E+01_JPRB, 0.11391E+01_JPRB, 0.12106E+01_JPRB, 0.10895E+01_JPRB ,&
&  0.96161E+00_JPRB, 0.93130E+00_JPRB, 0.91286E+00_JPRB, 0.83920E+00_JPRB, 0.80850E+00_JPRB, 0.77245E+00_JPRB, 0.76245E+00_JPRB ,&
&  0.75108E+00_JPRB, 0.74007E+00_JPRB, 0.72873E+00_JPRB, 0.75116E+00_JPRB, 0.79892E+00_JPRB, 0.86440E+00_JPRB, 0.96268E+00_JPRB ,&
&  0.10548E+01_JPRB, 0.10326E+01_JPRB, 0.95926E+00_JPRB, 0.92440E+00_JPRB, 0.91373E+00_JPRB, 0.92276E+00_JPRB, 0.93040E+00_JPRB ,&
&  0.91038E+00_JPRB, 0.88288E+00_JPRB, 0.97054E+00_JPRB, 0.10155E+01_JPRB, 0.10342E+01_JPRB, 0.11093E+01_JPRB, 0.11085E+01_JPRB ,&
&  0.13477E+01_JPRB, 0.14232E+01_JPRB, 0.17904E+01_JPRB, 0.16400E+01_JPRB, 0.18367E+01_JPRB, 0.20037E+01_JPRB, 0.18928E+01_JPRB ,&
&  0.15912E+01_JPRB, 0.14334E+01_JPRB, 0.12350E+01_JPRB, 0.11598E+01_JPRB, 0.12136E+01_JPRB, 0.13918E+01_JPRB, 0.16129E+01_JPRB ,&
&  0.18631E+01_JPRB, 0.21357E+01_JPRB, 0.23638E+01_JPRB, 0.19424E+01_JPRB, 0.17686E+01_JPRB, 0.16609E+01_JPRB, 0.16882E+01_JPRB ,&
&  0.19646E+01_JPRB, 0.20433E+01_JPRB, 0.20998E+01_JPRB, 0.20572E+01_JPRB, 0.16204E+01_JPRB, 0.14482E+01_JPRB, 0.13664E+01_JPRB ,&
&  0.13118E+01_JPRB, 0.12549E+01_JPRB, 0.11608E+01_JPRB, 0.10818E+01_JPRB, 0.10234E+01_JPRB, 0.98496E+00_JPRB, 0.94463E+00_JPRB ,&
&  0.92178E+00_JPRB, 0.89881E+00_JPRB, 0.88676E+00_JPRB, 0.88768E+00_JPRB, 0.88155E+00_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.84815E+00_JPRB, 0.82892E+00_JPRB, 0.82375E+00_JPRB, 0.83180E+00_JPRB, 0.83883E+00_JPRB, 0.88333E+00_JPRB, 0.93394E+00_JPRB ,&
&  0.10636E+01_JPRB, 0.76115E+00_JPRB, 0.66593E+00_JPRB, 0.63239E+00_JPRB, 0.62240E+00_JPRB, 0.61733E+00_JPRB, 0.62201E+00_JPRB ,&
&  0.63066E+00_JPRB, 0.65766E+00_JPRB, 0.69997E+00_JPRB, 0.77323E+00_JPRB, 0.95079E+00_JPRB, 0.11992E+01_JPRB, 0.13657E+01_JPRB ,&
&  0.14271E+01_JPRB, 0.14405E+01_JPRB, 0.14085E+01_JPRB, 0.12242E+01_JPRB, 0.11426E+01_JPRB, 0.11832E+01_JPRB, 0.11533E+01_JPRB ,&
&  0.10523E+01_JPRB, 0.96685E+00_JPRB, 0.90782E+00_JPRB, 0.83219E+00_JPRB, 0.79438E+00_JPRB, 0.77635E+00_JPRB, 0.76353E+00_JPRB ,&
&  0.74197E+00_JPRB, 0.72760E+00_JPRB, 0.73177E+00_JPRB, 0.77923E+00_JPRB, 0.84286E+00_JPRB, 0.93924E+00_JPRB, 0.10850E+01_JPRB ,&
&  0.11180E+01_JPRB, 0.10518E+01_JPRB, 0.99387E+00_JPRB, 0.93307E+00_JPRB, 0.93527E+00_JPRB, 0.94455E+00_JPRB, 0.94864E+00_JPRB ,&
&  0.94303E+00_JPRB, 0.97145E+00_JPRB, 0.10003E+01_JPRB, 0.10028E+01_JPRB, 0.97530E+00_JPRB, 0.11019E+01_JPRB, 0.11586E+01_JPRB ,&
&  0.13331E+01_JPRB, 0.15150E+01_JPRB, 0.19311E+01_JPRB, 0.15828E+01_JPRB, 0.18748E+01_JPRB, 0.19243E+01_JPRB, 0.17501E+01_JPRB ,&
&  0.14857E+01_JPRB, 0.13324E+01_JPRB, 0.12143E+01_JPRB, 0.11974E+01_JPRB, 0.13534E+01_JPRB, 0.15351E+01_JPRB, 0.19770E+01_JPRB ,&
&  0.19908E+01_JPRB, 0.22177E+01_JPRB, 0.21049E+01_JPRB, 0.20480E+01_JPRB, 0.18209E+01_JPRB, 0.17371E+01_JPRB, 0.17389E+01_JPRB ,&
&  0.19158E+01_JPRB, 0.19324E+01_JPRB, 0.18284E+01_JPRB, 0.17087E+01_JPRB, 0.14369E+01_JPRB, 0.12915E+01_JPRB, 0.12213E+01_JPRB ,&
&  0.11518E+01_JPRB, 0.10810E+01_JPRB, 0.10297E+01_JPRB, 0.10116E+01_JPRB, 0.98399E+00_JPRB, 0.95524E+00_JPRB, 0.92619E+00_JPRB ,&
&  0.90933E+00_JPRB, 0.89496E+00_JPRB, 0.88643E+00_JPRB, 0.88159E+00_JPRB, 0.87172E+00_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.81540E+00_JPRB, 0.80327E+00_JPRB, 0.80418E+00_JPRB, 0.79535E+00_JPRB, 0.79130E+00_JPRB, 0.79534E+00_JPRB, 0.78261E+00_JPRB ,&
&  0.72680E+00_JPRB, 0.67969E+00_JPRB, 0.66338E+00_JPRB, 0.66136E+00_JPRB, 0.65580E+00_JPRB, 0.64715E+00_JPRB, 0.65117E+00_JPRB ,&
&  0.67160E+00_JPRB, 0.71687E+00_JPRB, 0.79155E+00_JPRB, 0.93789E+00_JPRB, 0.11305E+01_JPRB, 0.13515E+01_JPRB, 0.15417E+01_JPRB ,&
&  0.15273E+01_JPRB, 0.11190E+01_JPRB, 0.12798E+01_JPRB, 0.12889E+01_JPRB, 0.11422E+01_JPRB, 0.11284E+01_JPRB, 0.11132E+01_JPRB ,&
&  0.98900E+00_JPRB, 0.94299E+00_JPRB, 0.91491E+00_JPRB, 0.87044E+00_JPRB, 0.80236E+00_JPRB, 0.77929E+00_JPRB, 0.75505E+00_JPRB ,&
&  0.74140E+00_JPRB, 0.73987E+00_JPRB, 0.76583E+00_JPRB, 0.84719E+00_JPRB, 0.95040E+00_JPRB, 0.11004E+01_JPRB, 0.11858E+01_JPRB ,&
&  0.11510E+01_JPRB, 0.11078E+01_JPRB, 0.10845E+01_JPRB, 0.10183E+01_JPRB, 0.10093E+01_JPRB, 0.10215E+01_JPRB, 0.10235E+01_JPRB ,&
&  0.10207E+01_JPRB, 0.10158E+01_JPRB, 0.10542E+01_JPRB, 0.11232E+01_JPRB, 0.11848E+01_JPRB, 0.11381E+01_JPRB, 0.12752E+01_JPRB ,&
&  0.14197E+01_JPRB, 0.16932E+01_JPRB, 0.18760E+01_JPRB, 0.21423E+01_JPRB, 0.15707E+01_JPRB, 0.17153E+01_JPRB, 0.14476E+01_JPRB ,&
&  0.14331E+01_JPRB, 0.13106E+01_JPRB, 0.13339E+01_JPRB, 0.14135E+01_JPRB, 0.16359E+01_JPRB, 0.18119E+01_JPRB, 0.22119E+01_JPRB ,&
&  0.20503E+01_JPRB, 0.19908E+01_JPRB, 0.19183E+01_JPRB, 0.20060E+01_JPRB, 0.18747E+01_JPRB, 0.18745E+01_JPRB, 0.17559E+01_JPRB ,&
&  0.17811E+01_JPRB, 0.16656E+01_JPRB, 0.15264E+01_JPRB, 0.14455E+01_JPRB, 0.12882E+01_JPRB, 0.10959E+01_JPRB, 0.10169E+01_JPRB ,&
&  0.96669E+00_JPRB, 0.96366E+00_JPRB, 0.95927E+00_JPRB, 0.95348E+00_JPRB, 0.94111E+00_JPRB, 0.92486E+00_JPRB, 0.90934E+00_JPRB ,&
&  0.89775E+00_JPRB, 0.88460E+00_JPRB, 0.87407E+00_JPRB, 0.86287E+00_JPRB, 0.84255E+00_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.78172E+00_JPRB, 0.76866E+00_JPRB, 0.76728E+00_JPRB, 0.76356E+00_JPRB, 0.76123E+00_JPRB, 0.75423E+00_JPRB, 0.74060E+00_JPRB ,&
&  0.71524E+00_JPRB, 0.70309E+00_JPRB, 0.70005E+00_JPRB, 0.69634E+00_JPRB, 0.67871E+00_JPRB, 0.67105E+00_JPRB, 0.69077E+00_JPRB ,&
&  0.74339E+00_JPRB, 0.83340E+00_JPRB, 0.95362E+00_JPRB, 0.10937E+01_JPRB, 0.12764E+01_JPRB, 0.14114E+01_JPRB, 0.14498E+01_JPRB ,&
&  0.15525E+01_JPRB, 0.15945E+01_JPRB, 0.14265E+01_JPRB, 0.10551E+01_JPRB, 0.93987E+00_JPRB, 0.10850E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.94572E+00_JPRB, 0.91311E+00_JPRB, 0.88377E+00_JPRB, 0.85238E+00_JPRB, 0.81686E+00_JPRB, 0.79390E+00_JPRB, 0.77905E+00_JPRB ,&
&  0.77953E+00_JPRB, 0.80598E+00_JPRB, 0.88186E+00_JPRB, 0.10248E+01_JPRB, 0.11639E+01_JPRB, 0.12593E+01_JPRB, 0.12625E+01_JPRB ,&
&  0.12563E+01_JPRB, 0.12673E+01_JPRB, 0.12586E+01_JPRB, 0.11970E+01_JPRB, 0.11499E+01_JPRB, 0.11628E+01_JPRB, 0.11734E+01_JPRB ,&
&  0.11790E+01_JPRB, 0.11809E+01_JPRB, 0.11879E+01_JPRB, 0.12476E+01_JPRB, 0.12927E+01_JPRB, 0.12234E+01_JPRB, 0.13016E+01_JPRB ,&
&  0.15548E+01_JPRB, 0.19215E+01_JPRB, 0.19140E+01_JPRB, 0.22702E+01_JPRB, 0.13597E+01_JPRB, 0.15163E+01_JPRB, 0.15330E+01_JPRB ,&
&  0.13531E+01_JPRB, 0.13985E+01_JPRB, 0.14813E+01_JPRB, 0.16323E+01_JPRB, 0.18544E+01_JPRB, 0.19270E+01_JPRB, 0.21016E+01_JPRB ,&
&  0.18984E+01_JPRB, 0.17492E+01_JPRB, 0.17344E+01_JPRB, 0.17974E+01_JPRB, 0.17981E+01_JPRB, 0.17511E+01_JPRB, 0.15490E+01_JPRB ,&
&  0.13678E+01_JPRB, 0.14416E+01_JPRB, 0.13361E+01_JPRB, 0.13293E+01_JPRB, 0.11827E+01_JPRB, 0.95332E+00_JPRB, 0.95190E+00_JPRB ,&
&  0.94602E+00_JPRB, 0.93144E+00_JPRB, 0.92805E+00_JPRB, 0.91049E+00_JPRB, 0.89744E+00_JPRB, 0.88577E+00_JPRB, 0.87463E+00_JPRB ,&
&  0.86757E+00_JPRB, 0.85853E+00_JPRB, 0.84527E+00_JPRB, 0.82724E+00_JPRB, 0.80454E+00_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.75563E+00_JPRB, 0.74584E+00_JPRB, 0.75026E+00_JPRB, 0.73940E+00_JPRB, 0.72597E+00_JPRB, 0.72523E+00_JPRB, 0.72773E+00_JPRB ,&
&  0.72075E+00_JPRB, 0.71653E+00_JPRB, 0.70803E+00_JPRB, 0.70152E+00_JPRB, 0.70345E+00_JPRB, 0.72968E+00_JPRB, 0.78185E+00_JPRB ,&
&  0.85304E+00_JPRB, 0.96249E+00_JPRB, 0.11064E+01_JPRB, 0.12404E+01_JPRB, 0.13957E+01_JPRB, 0.14663E+01_JPRB, 0.14375E+01_JPRB ,&
&  0.14872E+01_JPRB, 0.15497E+01_JPRB, 0.15378E+01_JPRB, 0.15137E+01_JPRB, 0.12245E+01_JPRB, 0.12586E+01_JPRB, 0.12936E+01_JPRB ,&
&  0.12239E+01_JPRB, 0.10303E+01_JPRB, 0.92075E+00_JPRB, 0.88693E+00_JPRB, 0.85095E+00_JPRB, 0.84421E+00_JPRB, 0.86368E+00_JPRB ,&
&  0.91083E+00_JPRB, 0.99204E+00_JPRB, 0.11196E+01_JPRB, 0.12619E+01_JPRB, 0.13545E+01_JPRB, 0.13944E+01_JPRB, 0.14102E+01_JPRB ,&
&  0.14364E+01_JPRB, 0.14637E+01_JPRB, 0.14942E+01_JPRB, 0.14489E+01_JPRB, 0.14369E+01_JPRB, 0.14364E+01_JPRB, 0.14755E+01_JPRB ,&
&  0.15153E+01_JPRB, 0.14544E+01_JPRB, 0.14610E+01_JPRB, 0.15454E+01_JPRB, 0.15533E+01_JPRB, 0.15329E+01_JPRB, 0.15920E+01_JPRB ,&
&  0.18371E+01_JPRB, 0.22483E+01_JPRB, 0.17898E+01_JPRB, 0.18618E+01_JPRB, 0.20831E+01_JPRB, 0.16939E+01_JPRB, 0.14321E+01_JPRB ,&
&  0.14113E+01_JPRB, 0.14705E+01_JPRB, 0.15503E+01_JPRB, 0.16924E+01_JPRB, 0.19126E+01_JPRB, 0.21596E+01_JPRB, 0.18517E+01_JPRB ,&
&  0.16835E+01_JPRB, 0.15450E+01_JPRB, 0.15466E+01_JPRB, 0.15591E+01_JPRB, 0.15446E+01_JPRB, 0.15114E+01_JPRB, 0.13653E+01_JPRB ,&
&  0.12559E+01_JPRB, 0.13212E+01_JPRB, 0.13023E+01_JPRB, 0.13302E+01_JPRB, 0.14190E+01_JPRB, 0.98497E+00_JPRB, 0.96188E+00_JPRB ,&
&  0.91093E+00_JPRB, 0.90185E+00_JPRB, 0.89400E+00_JPRB, 0.87942E+00_JPRB, 0.86463E+00_JPRB, 0.84962E+00_JPRB, 0.84070E+00_JPRB ,&
&  0.83176E+00_JPRB, 0.81677E+00_JPRB, 0.80483E+00_JPRB, 0.79558E+00_JPRB, 0.77205E+00_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.74066E+00_JPRB, 0.73844E+00_JPRB, 0.74120E+00_JPRB, 0.73480E+00_JPRB, 0.72290E+00_JPRB, 0.72445E+00_JPRB, 0.71898E+00_JPRB ,&
&  0.70310E+00_JPRB, 0.68778E+00_JPRB, 0.66988E+00_JPRB, 0.67957E+00_JPRB, 0.70078E+00_JPRB, 0.74455E+00_JPRB, 0.79998E+00_JPRB ,&
&  0.87576E+00_JPRB, 0.98995E+00_JPRB, 0.11428E+01_JPRB, 0.12609E+01_JPRB, 0.14208E+01_JPRB, 0.14867E+01_JPRB, 0.15171E+01_JPRB ,&
&  0.15744E+01_JPRB, 0.16415E+01_JPRB, 0.16890E+01_JPRB, 0.17013E+01_JPRB, 0.16461E+01_JPRB, 0.16145E+01_JPRB, 0.16327E+01_JPRB ,&
&  0.13415E+01_JPRB, 0.10362E+01_JPRB, 0.97149E+00_JPRB, 0.94726E+00_JPRB, 0.94717E+00_JPRB, 0.96269E+00_JPRB, 0.10204E+01_JPRB ,&
&  0.11119E+01_JPRB, 0.12132E+01_JPRB, 0.13421E+01_JPRB, 0.14409E+01_JPRB, 0.15223E+01_JPRB, 0.15822E+01_JPRB, 0.16231E+01_JPRB ,&
&  0.16912E+01_JPRB, 0.17652E+01_JPRB, 0.18817E+01_JPRB, 0.18004E+01_JPRB, 0.19530E+01_JPRB, 0.19527E+01_JPRB, 0.19650E+01_JPRB ,&
&  0.19166E+01_JPRB, 0.17902E+01_JPRB, 0.18479E+01_JPRB, 0.19299E+01_JPRB, 0.20150E+01_JPRB, 0.20298E+01_JPRB, 0.20086E+01_JPRB ,&
&  0.22441E+01_JPRB, 0.23077E+01_JPRB, 0.17251E+01_JPRB, 0.15130E+01_JPRB, 0.15244E+01_JPRB, 0.12656E+01_JPRB, 0.14000E+01_JPRB ,&
&  0.13921E+01_JPRB, 0.14434E+01_JPRB, 0.15081E+01_JPRB, 0.16431E+01_JPRB, 0.17811E+01_JPRB, 0.18655E+01_JPRB, 0.16574E+01_JPRB ,&
&  0.14232E+01_JPRB, 0.13202E+01_JPRB, 0.12963E+01_JPRB, 0.12772E+01_JPRB, 0.12877E+01_JPRB, 0.12870E+01_JPRB, 0.12502E+01_JPRB ,&
&  0.12047E+01_JPRB, 0.11332E+01_JPRB, 0.10937E+01_JPRB, 0.99248E+00_JPRB, 0.91884E+00_JPRB, 0.91893E+00_JPRB, 0.87220E+00_JPRB ,&
&  0.86394E+00_JPRB, 0.87307E+00_JPRB, 0.86922E+00_JPRB, 0.85624E+00_JPRB, 0.84832E+00_JPRB, 0.82902E+00_JPRB, 0.81943E+00_JPRB ,&
&  0.81409E+00_JPRB, 0.79908E+00_JPRB, 0.78456E+00_JPRB, 0.77620E+00_JPRB, 0.75447E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.71789E+00_JPRB, 0.70879E+00_JPRB, 0.70819E+00_JPRB, 0.71024E+00_JPRB, 0.69940E+00_JPRB, 0.68994E+00_JPRB, 0.68192E+00_JPRB ,&
&  0.65110E+00_JPRB, 0.64771E+00_JPRB, 0.63508E+00_JPRB, 0.64169E+00_JPRB, 0.66724E+00_JPRB, 0.69192E+00_JPRB, 0.73303E+00_JPRB ,&
&  0.77448E+00_JPRB, 0.83060E+00_JPRB, 0.91969E+00_JPRB, 0.11048E+01_JPRB, 0.13101E+01_JPRB, 0.14576E+01_JPRB, 0.16316E+01_JPRB ,&
&  0.16927E+01_JPRB, 0.17444E+01_JPRB, 0.18590E+01_JPRB, 0.20075E+01_JPRB, 0.20641E+01_JPRB, 0.20812E+01_JPRB, 0.19958E+01_JPRB ,&
&  0.14536E+01_JPRB, 0.10187E+01_JPRB, 0.10442E+01_JPRB, 0.99129E+00_JPRB, 0.10307E+01_JPRB, 0.10831E+01_JPRB, 0.11466E+01_JPRB ,&
&  0.12463E+01_JPRB, 0.13477E+01_JPRB, 0.14524E+01_JPRB, 0.15373E+01_JPRB, 0.16417E+01_JPRB, 0.17555E+01_JPRB, 0.18193E+01_JPRB ,&
&  0.18766E+01_JPRB, 0.19420E+01_JPRB, 0.20018E+01_JPRB, 0.19674E+01_JPRB, 0.20679E+01_JPRB, 0.22096E+01_JPRB, 0.23243E+01_JPRB ,&
&  0.24329E+01_JPRB, 0.24273E+01_JPRB, 0.20445E+01_JPRB, 0.21150E+01_JPRB, 0.24039E+01_JPRB, 0.22528E+01_JPRB, 0.23081E+01_JPRB ,&
&  0.26668E+01_JPRB, 0.27327E+01_JPRB, 0.15833E+01_JPRB, 0.13569E+01_JPRB, 0.13554E+01_JPRB, 0.13057E+01_JPRB, 0.13248E+01_JPRB ,&
&  0.13224E+01_JPRB, 0.13847E+01_JPRB, 0.14735E+01_JPRB, 0.15222E+01_JPRB, 0.15358E+01_JPRB, 0.14481E+01_JPRB, 0.13601E+01_JPRB ,&
&  0.11861E+01_JPRB, 0.11045E+01_JPRB, 0.11060E+01_JPRB, 0.11207E+01_JPRB, 0.11285E+01_JPRB, 0.11469E+01_JPRB, 0.10820E+01_JPRB ,&
&  0.10559E+01_JPRB, 0.10240E+01_JPRB, 0.11038E+01_JPRB, 0.99119E+00_JPRB, 0.84425E+00_JPRB, 0.84883E+00_JPRB, 0.82589E+00_JPRB ,&
&  0.85796E+00_JPRB, 0.87127E+00_JPRB, 0.87475E+00_JPRB, 0.87351E+00_JPRB, 0.86929E+00_JPRB, 0.85598E+00_JPRB, 0.84059E+00_JPRB ,&
&  0.82172E+00_JPRB, 0.80069E+00_JPRB, 0.77739E+00_JPRB, 0.75535E+00_JPRB, 0.73486E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.65189E+00_JPRB, 0.64193E+00_JPRB, 0.64805E+00_JPRB, 0.62637E+00_JPRB, 0.64550E+00_JPRB, 0.62990E+00_JPRB, 0.65831E+00_JPRB ,&
&  0.63422E+00_JPRB, 0.62883E+00_JPRB, 0.62823E+00_JPRB, 0.63888E+00_JPRB, 0.65034E+00_JPRB, 0.66751E+00_JPRB, 0.68094E+00_JPRB ,&
&  0.70384E+00_JPRB, 0.74774E+00_JPRB, 0.80869E+00_JPRB, 0.87827E+00_JPRB, 0.96883E+00_JPRB, 0.10577E+01_JPRB, 0.11339E+01_JPRB ,&
&  0.12366E+01_JPRB, 0.13236E+01_JPRB, 0.13694E+01_JPRB, 0.14998E+01_JPRB, 0.18838E+01_JPRB, 0.23537E+01_JPRB, 0.23850E+01_JPRB ,&
&  0.15074E+01_JPRB, 0.10771E+01_JPRB, 0.99048E+00_JPRB, 0.93064E+00_JPRB, 0.98830E+00_JPRB, 0.11214E+01_JPRB, 0.11834E+01_JPRB ,&
&  0.12432E+01_JPRB, 0.13165E+01_JPRB, 0.14058E+01_JPRB, 0.15025E+01_JPRB, 0.16221E+01_JPRB, 0.17477E+01_JPRB, 0.18162E+01_JPRB ,&
&  0.18272E+01_JPRB, 0.18727E+01_JPRB, 0.18913E+01_JPRB, 0.20383E+01_JPRB, 0.22796E+01_JPRB, 0.25656E+01_JPRB, 0.27807E+01_JPRB ,&
&  0.28967E+01_JPRB, 0.28416E+01_JPRB, 0.24076E+01_JPRB, 0.24314E+01_JPRB, 0.25657E+01_JPRB, 0.25181E+01_JPRB, 0.24234E+01_JPRB ,&
&  0.26579E+01_JPRB, 0.25014E+01_JPRB, 0.16291E+01_JPRB, 0.14616E+01_JPRB, 0.13675E+01_JPRB, 0.13023E+01_JPRB, 0.12254E+01_JPRB ,&
&  0.12614E+01_JPRB, 0.13181E+01_JPRB, 0.13563E+01_JPRB, 0.13434E+01_JPRB, 0.12972E+01_JPRB, 0.12342E+01_JPRB, 0.12162E+01_JPRB ,&
&  0.11156E+01_JPRB, 0.10960E+01_JPRB, 0.10885E+01_JPRB, 0.10473E+01_JPRB, 0.11315E+01_JPRB, 0.10014E+01_JPRB, 0.10192E+01_JPRB ,&
&  0.91006E+00_JPRB, 0.88831E+00_JPRB, 0.77629E+00_JPRB, 0.94071E+00_JPRB, 0.83813E+00_JPRB, 0.76599E+00_JPRB, 0.78797E+00_JPRB ,&
&  0.80328E+00_JPRB, 0.82641E+00_JPRB, 0.81856E+00_JPRB, 0.82101E+00_JPRB, 0.82605E+00_JPRB, 0.81553E+00_JPRB, 0.80502E+00_JPRB ,&
&  0.78987E+00_JPRB, 0.77182E+00_JPRB, 0.73904E+00_JPRB, 0.70874E+00_JPRB, 0.67939E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.64019E+00_JPRB, 0.64539E+00_JPRB, 0.63250E+00_JPRB, 0.60880E+00_JPRB, 0.59171E+00_JPRB, 0.58308E+00_JPRB, 0.59117E+00_JPRB ,&
&  0.60079E+00_JPRB, 0.62122E+00_JPRB, 0.63739E+00_JPRB, 0.65517E+00_JPRB, 0.66408E+00_JPRB, 0.67100E+00_JPRB, 0.68622E+00_JPRB ,&
&  0.70638E+00_JPRB, 0.72707E+00_JPRB, 0.75294E+00_JPRB, 0.78844E+00_JPRB, 0.82062E+00_JPRB, 0.86073E+00_JPRB, 0.87656E+00_JPRB ,&
&  0.87422E+00_JPRB, 0.89833E+00_JPRB, 0.91542E+00_JPRB, 0.95690E+00_JPRB, 0.11479E+01_JPRB, 0.13431E+01_JPRB, 0.15511E+01_JPRB ,&
&  0.11779E+01_JPRB, 0.11804E+01_JPRB, 0.10643E+01_JPRB, 0.98716E+00_JPRB, 0.10167E+01_JPRB, 0.10113E+01_JPRB, 0.10417E+01_JPRB ,&
&  0.10873E+01_JPRB, 0.11126E+01_JPRB, 0.11377E+01_JPRB, 0.11731E+01_JPRB, 0.12129E+01_JPRB, 0.12926E+01_JPRB, 0.12662E+01_JPRB ,&
&  0.12601E+01_JPRB, 0.13229E+01_JPRB, 0.14001E+01_JPRB, 0.16129E+01_JPRB, 0.19181E+01_JPRB, 0.21950E+01_JPRB, 0.22269E+01_JPRB ,&
&  0.22556E+01_JPRB, 0.24157E+01_JPRB, 0.23548E+01_JPRB, 0.23660E+01_JPRB, 0.23265E+01_JPRB, 0.22421E+01_JPRB, 0.22428E+01_JPRB ,&
&  0.21271E+01_JPRB, 0.19158E+01_JPRB, 0.17063E+01_JPRB, 0.14734E+01_JPRB, 0.13175E+01_JPRB, 0.11990E+01_JPRB, 0.11598E+01_JPRB ,&
&  0.11443E+01_JPRB, 0.11773E+01_JPRB, 0.11939E+01_JPRB, 0.11475E+01_JPRB, 0.11177E+01_JPRB, 0.10775E+01_JPRB, 0.10757E+01_JPRB ,&
&  0.10619E+01_JPRB, 0.98001E+00_JPRB, 0.93163E+00_JPRB, 0.91498E+00_JPRB, 0.91489E+00_JPRB, 0.13198E+01_JPRB, 0.12365E+01_JPRB ,&
&  0.80345E+00_JPRB, 0.66502E+00_JPRB, 0.55971E+00_JPRB, 0.80158E+00_JPRB, 0.70455E+00_JPRB, 0.69748E+00_JPRB, 0.71818E+00_JPRB ,&
&  0.73957E+00_JPRB, 0.75434E+00_JPRB, 0.76343E+00_JPRB, 0.75406E+00_JPRB, 0.75093E+00_JPRB, 0.74812E+00_JPRB, 0.73137E+00_JPRB ,&
&  0.71763E+00_JPRB, 0.69981E+00_JPRB, 0.69481E+00_JPRB, 0.66216E+00_JPRB, 0.64531E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.61997E+00_JPRB, 0.61964E+00_JPRB, 0.59822E+00_JPRB, 0.56799E+00_JPRB, 0.55402E+00_JPRB, 0.56274E+00_JPRB, 0.57863E+00_JPRB ,&
&  0.59361E+00_JPRB, 0.61373E+00_JPRB, 0.63702E+00_JPRB, 0.65970E+00_JPRB, 0.67988E+00_JPRB, 0.70162E+00_JPRB, 0.72730E+00_JPRB ,&
&  0.75165E+00_JPRB, 0.78166E+00_JPRB, 0.81187E+00_JPRB, 0.84191E+00_JPRB, 0.86751E+00_JPRB, 0.91292E+00_JPRB, 0.94403E+00_JPRB ,&
&  0.99416E+00_JPRB, 0.10444E+01_JPRB, 0.11053E+01_JPRB, 0.11633E+01_JPRB, 0.12352E+01_JPRB, 0.12844E+01_JPRB, 0.11004E+01_JPRB ,&
&  0.12520E+01_JPRB, 0.11983E+01_JPRB, 0.10956E+01_JPRB, 0.10150E+01_JPRB, 0.96004E+00_JPRB, 0.92066E+00_JPRB, 0.88099E+00_JPRB ,&
&  0.86572E+00_JPRB, 0.84947E+00_JPRB, 0.81906E+00_JPRB, 0.79977E+00_JPRB, 0.80012E+00_JPRB, 0.86449E+00_JPRB, 0.86762E+00_JPRB ,&
&  0.94826E+00_JPRB, 0.10854E+01_JPRB, 0.12027E+01_JPRB, 0.13916E+01_JPRB, 0.15563E+01_JPRB, 0.16700E+01_JPRB, 0.16124E+01_JPRB ,&
&  0.16335E+01_JPRB, 0.17825E+01_JPRB, 0.17554E+01_JPRB, 0.17974E+01_JPRB, 0.17756E+01_JPRB, 0.16900E+01_JPRB, 0.16333E+01_JPRB ,&
&  0.14528E+01_JPRB, 0.14025E+01_JPRB, 0.14332E+01_JPRB, 0.15332E+01_JPRB, 0.13095E+01_JPRB, 0.11626E+01_JPRB, 0.11388E+01_JPRB ,&
&  0.10625E+01_JPRB, 0.10724E+01_JPRB, 0.10759E+01_JPRB, 0.10187E+01_JPRB, 0.97748E+00_JPRB, 0.94344E+00_JPRB, 0.94195E+00_JPRB ,&
&  0.94549E+00_JPRB, 0.84926E+00_JPRB, 0.81594E+00_JPRB, 0.84033E+00_JPRB, 0.92020E+00_JPRB, 0.96168E+00_JPRB, 0.11436E+01_JPRB ,&
&  0.73914E+00_JPRB, 0.47787E+00_JPRB, 0.53238E+00_JPRB, 0.62833E+00_JPRB, 0.60222E+00_JPRB, 0.60633E+00_JPRB, 0.65141E+00_JPRB ,&
&  0.69487E+00_JPRB, 0.71969E+00_JPRB, 0.71077E+00_JPRB, 0.73333E+00_JPRB, 0.76713E+00_JPRB, 0.75450E+00_JPRB, 0.69775E+00_JPRB ,&
&  0.70386E+00_JPRB, 0.66779E+00_JPRB, 0.64834E+00_JPRB, 0.63836E+00_JPRB, 0.63106E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.59419E+00_JPRB, 0.58548E+00_JPRB, 0.56438E+00_JPRB, 0.53357E+00_JPRB, 0.52230E+00_JPRB, 0.54026E+00_JPRB, 0.56558E+00_JPRB ,&
&  0.58476E+00_JPRB, 0.60827E+00_JPRB, 0.63268E+00_JPRB, 0.65944E+00_JPRB, 0.68893E+00_JPRB, 0.71632E+00_JPRB, 0.73741E+00_JPRB ,&
&  0.75208E+00_JPRB, 0.77192E+00_JPRB, 0.78924E+00_JPRB, 0.80308E+00_JPRB, 0.80719E+00_JPRB, 0.80903E+00_JPRB, 0.80907E+00_JPRB ,&
&  0.84165E+00_JPRB, 0.87414E+00_JPRB, 0.91670E+00_JPRB, 0.95862E+00_JPRB, 0.98364E+00_JPRB, 0.97052E+00_JPRB, 0.78949E+00_JPRB ,&
&  0.11404E+01_JPRB, 0.11566E+01_JPRB, 0.10762E+01_JPRB, 0.99394E+00_JPRB, 0.91699E+00_JPRB, 0.86154E+00_JPRB, 0.80703E+00_JPRB ,&
&  0.76063E+00_JPRB, 0.79063E+00_JPRB, 0.77622E+00_JPRB, 0.78655E+00_JPRB, 0.76392E+00_JPRB, 0.81876E+00_JPRB, 0.87974E+00_JPRB ,&
&  0.10013E+01_JPRB, 0.11339E+01_JPRB, 0.12527E+01_JPRB, 0.13557E+01_JPRB, 0.14074E+01_JPRB, 0.14481E+01_JPRB, 0.13851E+01_JPRB ,&
&  0.13909E+01_JPRB, 0.13932E+01_JPRB, 0.13018E+01_JPRB, 0.13365E+01_JPRB, 0.13789E+01_JPRB, 0.11971E+01_JPRB, 0.10672E+01_JPRB ,&
&  0.96660E+00_JPRB, 0.10854E+01_JPRB, 0.14125E+01_JPRB, 0.15443E+01_JPRB, 0.13189E+01_JPRB, 0.12253E+01_JPRB, 0.11315E+01_JPRB ,&
&  0.10296E+01_JPRB, 0.10106E+01_JPRB, 0.98616E+00_JPRB, 0.93854E+00_JPRB, 0.86275E+00_JPRB, 0.82030E+00_JPRB, 0.81992E+00_JPRB ,&
&  0.82609E+00_JPRB, 0.75091E+00_JPRB, 0.70013E+00_JPRB, 0.73579E+00_JPRB, 0.75953E+00_JPRB, 0.70279E+00_JPRB, 0.10679E+01_JPRB ,&
&  0.74226E+00_JPRB, 0.55359E+00_JPRB, 0.72412E+00_JPRB, 0.41849E+00_JPRB, 0.51272E+00_JPRB, 0.50473E+00_JPRB, 0.52795E+00_JPRB ,&
&  0.45701E+00_JPRB, 0.53369E+00_JPRB, 0.89151E+00_JPRB, 0.12213E+01_JPRB, 0.14059E+01_JPRB, 0.13942E+01_JPRB, 0.79041E+00_JPRB ,&
&  0.69282E+00_JPRB, 0.66562E+00_JPRB, 0.64561E+00_JPRB, 0.62256E+00_JPRB, 0.59807E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.56826E+00_JPRB, 0.53672E+00_JPRB, 0.52639E+00_JPRB, 0.52116E+00_JPRB, 0.51227E+00_JPRB, 0.53142E+00_JPRB, 0.54674E+00_JPRB ,&
&  0.56865E+00_JPRB, 0.59604E+00_JPRB, 0.62942E+00_JPRB, 0.65869E+00_JPRB, 0.70060E+00_JPRB, 0.74097E+00_JPRB, 0.77195E+00_JPRB ,&
&  0.79599E+00_JPRB, 0.81352E+00_JPRB, 0.82541E+00_JPRB, 0.82949E+00_JPRB, 0.82813E+00_JPRB, 0.82579E+00_JPRB, 0.82061E+00_JPRB ,&
&  0.81533E+00_JPRB, 0.82892E+00_JPRB, 0.86170E+00_JPRB, 0.89966E+00_JPRB, 0.91658E+00_JPRB, 0.93468E+00_JPRB, 0.61120E+00_JPRB ,&
&  0.99574E+00_JPRB, 0.10366E+01_JPRB, 0.10180E+01_JPRB, 0.95082E+00_JPRB, 0.88259E+00_JPRB, 0.83190E+00_JPRB, 0.79242E+00_JPRB ,&
&  0.76876E+00_JPRB, 0.78897E+00_JPRB, 0.78884E+00_JPRB, 0.89459E+00_JPRB, 0.97135E+00_JPRB, 0.97761E+00_JPRB, 0.10829E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.12963E+01_JPRB, 0.13815E+01_JPRB, 0.14484E+01_JPRB, 0.14997E+01_JPRB, 0.15484E+01_JPRB, 0.15693E+01_JPRB ,&
&  0.15788E+01_JPRB, 0.15405E+01_JPRB, 0.14909E+01_JPRB, 0.15459E+01_JPRB, 0.15658E+01_JPRB, 0.13432E+01_JPRB, 0.12247E+01_JPRB ,&
&  0.91391E+00_JPRB, 0.84579E+00_JPRB, 0.12378E+01_JPRB, 0.14939E+01_JPRB, 0.13555E+01_JPRB, 0.12866E+01_JPRB, 0.11457E+01_JPRB ,&
&  0.10100E+01_JPRB, 0.93746E+00_JPRB, 0.89339E+00_JPRB, 0.84529E+00_JPRB, 0.78403E+00_JPRB, 0.74177E+00_JPRB, 0.72008E+00_JPRB ,&
&  0.70091E+00_JPRB, 0.63161E+00_JPRB, 0.59613E+00_JPRB, 0.60915E+00_JPRB, 0.61637E+00_JPRB, 0.69176E+00_JPRB, 0.69435E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.76200E+00_JPRB, 0.62105E+00_JPRB, 0.58996E+00_JPRB, 0.48542E+00_JPRB, 0.46073E+00_JPRB, 0.45974E+00_JPRB ,&
&  0.48206E+00_JPRB, 0.44144E+00_JPRB, 0.93658E+00_JPRB, 0.37196E+01_JPRB, 0.36806E+01_JPRB, 0.46727E+01_JPRB, 0.24316E+01_JPRB ,&
&  0.11462E+01_JPRB, 0.70526E+00_JPRB, 0.60974E+00_JPRB, 0.60003E+00_JPRB, 0.58152E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.51527E+00_JPRB, 0.49682E+00_JPRB, 0.47983E+00_JPRB, 0.49150E+00_JPRB, 0.50045E+00_JPRB, 0.50694E+00_JPRB, 0.53374E+00_JPRB ,&
&  0.58922E+00_JPRB, 0.60326E+00_JPRB, 0.63022E+00_JPRB, 0.67331E+00_JPRB, 0.73602E+00_JPRB, 0.78370E+00_JPRB, 0.82715E+00_JPRB ,&
&  0.87066E+00_JPRB, 0.89506E+00_JPRB, 0.88638E+00_JPRB, 0.86951E+00_JPRB, 0.86449E+00_JPRB, 0.86622E+00_JPRB, 0.87612E+00_JPRB ,&
&  0.87504E+00_JPRB, 0.88876E+00_JPRB, 0.91852E+00_JPRB, 0.95440E+00_JPRB, 0.98318E+00_JPRB, 0.96460E+00_JPRB, 0.61327E+00_JPRB ,&
&  0.80786E+00_JPRB, 0.87039E+00_JPRB, 0.90193E+00_JPRB, 0.88272E+00_JPRB, 0.82581E+00_JPRB, 0.76537E+00_JPRB, 0.76054E+00_JPRB ,&
&  0.81267E+00_JPRB, 0.88929E+00_JPRB, 0.96298E+00_JPRB, 0.10238E+01_JPRB, 0.11990E+01_JPRB, 0.12257E+01_JPRB, 0.14105E+01_JPRB ,&
&  0.15108E+01_JPRB, 0.15902E+01_JPRB, 0.16936E+01_JPRB, 0.17579E+01_JPRB, 0.18430E+01_JPRB, 0.19456E+01_JPRB, 0.20412E+01_JPRB ,&
&  0.20856E+01_JPRB, 0.20812E+01_JPRB, 0.21470E+01_JPRB, 0.23470E+01_JPRB, 0.21597E+01_JPRB, 0.23987E+01_JPRB, 0.19486E+01_JPRB ,&
&  0.13155E+01_JPRB, 0.11081E+01_JPRB, 0.13302E+01_JPRB, 0.14586E+01_JPRB, 0.13715E+01_JPRB, 0.13309E+01_JPRB, 0.12055E+01_JPRB ,&
&  0.10275E+01_JPRB, 0.90620E+00_JPRB, 0.85143E+00_JPRB, 0.80826E+00_JPRB, 0.75939E+00_JPRB, 0.70605E+00_JPRB, 0.66261E+00_JPRB ,&
&  0.61252E+00_JPRB, 0.57758E+00_JPRB, 0.58994E+00_JPRB, 0.58598E+00_JPRB, 0.61029E+00_JPRB, 0.71657E+00_JPRB, 0.96830E+00_JPRB ,&
&  0.14980E+01_JPRB, 0.80925E+00_JPRB, 0.66090E+00_JPRB, 0.59664E+00_JPRB, 0.53047E+00_JPRB, 0.49300E+00_JPRB, 0.47137E+00_JPRB ,&
&  0.47598E+00_JPRB, 0.53617E+00_JPRB, 0.66703E+00_JPRB, 0.13875E+01_JPRB, 0.25398E+01_JPRB, 0.26029E+01_JPRB, 0.17500E+01_JPRB ,&
&  0.10580E+01_JPRB, 0.72831E+00_JPRB, 0.58251E+00_JPRB, 0.54649E+00_JPRB, 0.52099E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.48938E+00_JPRB, 0.47719E+00_JPRB, 0.47776E+00_JPRB, 0.47753E+00_JPRB, 0.51340E+00_JPRB, 0.53729E+00_JPRB, 0.55889E+00_JPRB ,&
&  0.59132E+00_JPRB, 0.61164E+00_JPRB, 0.64370E+00_JPRB, 0.70207E+00_JPRB, 0.76418E+00_JPRB, 0.81356E+00_JPRB, 0.83544E+00_JPRB ,&
&  0.84604E+00_JPRB, 0.84066E+00_JPRB, 0.82466E+00_JPRB, 0.80495E+00_JPRB, 0.81411E+00_JPRB, 0.81454E+00_JPRB, 0.83860E+00_JPRB ,&
&  0.87639E+00_JPRB, 0.91618E+00_JPRB, 0.96540E+00_JPRB, 0.10112E+01_JPRB, 0.10562E+01_JPRB, 0.10814E+01_JPRB, 0.10551E+01_JPRB ,&
&  0.51356E+00_JPRB, 0.72600E+00_JPRB, 0.77039E+00_JPRB, 0.76385E+00_JPRB, 0.72057E+00_JPRB, 0.70508E+00_JPRB, 0.77665E+00_JPRB ,&
&  0.88536E+00_JPRB, 0.94919E+00_JPRB, 0.10870E+01_JPRB, 0.11613E+01_JPRB, 0.14157E+01_JPRB, 0.15085E+01_JPRB, 0.16721E+01_JPRB ,&
&  0.17917E+01_JPRB, 0.18801E+01_JPRB, 0.20008E+01_JPRB, 0.21291E+01_JPRB, 0.22731E+01_JPRB, 0.24323E+01_JPRB, 0.26296E+01_JPRB ,&
&  0.28167E+01_JPRB, 0.30513E+01_JPRB, 0.33634E+01_JPRB, 0.33536E+01_JPRB, 0.31100E+01_JPRB, 0.33340E+01_JPRB, 0.31666E+01_JPRB ,&
&  0.26589E+01_JPRB, 0.18914E+01_JPRB, 0.18018E+01_JPRB, 0.16325E+01_JPRB, 0.14688E+01_JPRB, 0.13950E+01_JPRB, 0.12655E+01_JPRB ,&
&  0.10799E+01_JPRB, 0.97342E+00_JPRB, 0.88399E+00_JPRB, 0.82687E+00_JPRB, 0.75806E+00_JPRB, 0.70850E+00_JPRB, 0.67233E+00_JPRB ,&
&  0.64764E+00_JPRB, 0.66103E+00_JPRB, 0.67636E+00_JPRB, 0.65733E+00_JPRB, 0.62872E+00_JPRB, 0.65251E+00_JPRB, 0.72480E+00_JPRB ,&
&  0.82536E+00_JPRB, 0.73669E+00_JPRB, 0.67140E+00_JPRB, 0.66935E+00_JPRB, 0.65331E+00_JPRB, 0.63689E+00_JPRB, 0.62801E+00_JPRB ,&
&  0.63697E+00_JPRB, 0.62444E+00_JPRB, 0.71832E+00_JPRB, 0.11027E+01_JPRB, 0.14789E+01_JPRB, 0.17230E+01_JPRB, 0.14222E+01_JPRB ,&
&  0.10079E+01_JPRB, 0.74558E+00_JPRB, 0.59475E+00_JPRB, 0.55370E+00_JPRB, 0.53130E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.53508E+00_JPRB, 0.52479E+00_JPRB, 0.52497E+00_JPRB, 0.52203E+00_JPRB, 0.55486E+00_JPRB, 0.59145E+00_JPRB, 0.60191E+00_JPRB ,&
&  0.59662E+00_JPRB, 0.61381E+00_JPRB, 0.63030E+00_JPRB, 0.66085E+00_JPRB, 0.72481E+00_JPRB, 0.75428E+00_JPRB, 0.78899E+00_JPRB ,&
&  0.82273E+00_JPRB, 0.81764E+00_JPRB, 0.78544E+00_JPRB, 0.76812E+00_JPRB, 0.78720E+00_JPRB, 0.80145E+00_JPRB, 0.83310E+00_JPRB ,&
&  0.88039E+00_JPRB, 0.93561E+00_JPRB, 0.10029E+01_JPRB, 0.10872E+01_JPRB, 0.11829E+01_JPRB, 0.12590E+01_JPRB, 0.12467E+01_JPRB ,&
&  0.54228E+00_JPRB, 0.52856E+00_JPRB, 0.66237E+00_JPRB, 0.67059E+00_JPRB, 0.63720E+00_JPRB, 0.67073E+00_JPRB, 0.74863E+00_JPRB ,&
&  0.82883E+00_JPRB, 0.85402E+00_JPRB, 0.99351E+00_JPRB, 0.13802E+01_JPRB, 0.15504E+01_JPRB, 0.16972E+01_JPRB, 0.18362E+01_JPRB ,&
&  0.19519E+01_JPRB, 0.20761E+01_JPRB, 0.21862E+01_JPRB, 0.22844E+01_JPRB, 0.24087E+01_JPRB, 0.25694E+01_JPRB, 0.27862E+01_JPRB ,&
&  0.30639E+01_JPRB, 0.34739E+01_JPRB, 0.38753E+01_JPRB, 0.35751E+01_JPRB, 0.36796E+01_JPRB, 0.39092E+01_JPRB, 0.44684E+01_JPRB ,&
&  0.40043E+01_JPRB, 0.27687E+01_JPRB, 0.20957E+01_JPRB, 0.17643E+01_JPRB, 0.14173E+01_JPRB, 0.13067E+01_JPRB, 0.11846E+01_JPRB ,&
&  0.10804E+01_JPRB, 0.10077E+01_JPRB, 0.93944E+00_JPRB, 0.86361E+00_JPRB, 0.77055E+00_JPRB, 0.73705E+00_JPRB, 0.71881E+00_JPRB ,&
&  0.74470E+00_JPRB, 0.76373E+00_JPRB, 0.76151E+00_JPRB, 0.75746E+00_JPRB, 0.71840E+00_JPRB, 0.73014E+00_JPRB, 0.77357E+00_JPRB ,&
&  0.81146E+00_JPRB, 0.83512E+00_JPRB, 0.84608E+00_JPRB, 0.85757E+00_JPRB, 0.87360E+00_JPRB, 0.88075E+00_JPRB, 0.85312E+00_JPRB ,&
&  0.81430E+00_JPRB, 0.78050E+00_JPRB, 0.78387E+00_JPRB, 0.77735E+00_JPRB, 0.93229E+00_JPRB, 0.12317E+01_JPRB, 0.12711E+01_JPRB ,&
&  0.10829E+01_JPRB, 0.83064E+00_JPRB, 0.66928E+00_JPRB, 0.61860E+00_JPRB, 0.59473E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.64086E+00_JPRB, 0.60832E+00_JPRB, 0.59951E+00_JPRB, 0.58293E+00_JPRB, 0.59632E+00_JPRB, 0.62228E+00_JPRB, 0.64183E+00_JPRB ,&
&  0.64599E+00_JPRB, 0.64451E+00_JPRB, 0.64457E+00_JPRB, 0.66291E+00_JPRB, 0.68898E+00_JPRB, 0.71705E+00_JPRB, 0.75213E+00_JPRB ,&
&  0.77475E+00_JPRB, 0.78520E+00_JPRB, 0.75688E+00_JPRB, 0.71836E+00_JPRB, 0.72471E+00_JPRB, 0.74290E+00_JPRB, 0.77313E+00_JPRB ,&
&  0.81628E+00_JPRB, 0.86460E+00_JPRB, 0.93250E+00_JPRB, 0.10520E+01_JPRB, 0.12349E+01_JPRB, 0.14539E+01_JPRB, 0.16777E+01_JPRB ,&
&  0.13113E+01_JPRB, 0.62956E+00_JPRB, 0.64706E+00_JPRB, 0.72290E+00_JPRB, 0.66656E+00_JPRB, 0.70167E+00_JPRB, 0.73078E+00_JPRB ,&
&  0.76139E+00_JPRB, 0.83615E+00_JPRB, 0.90499E+00_JPRB, 0.11339E+01_JPRB, 0.14527E+01_JPRB, 0.16367E+01_JPRB, 0.17977E+01_JPRB ,&
&  0.19245E+01_JPRB, 0.20152E+01_JPRB, 0.20709E+01_JPRB, 0.21278E+01_JPRB, 0.22330E+01_JPRB, 0.23935E+01_JPRB, 0.26097E+01_JPRB ,&
&  0.29093E+01_JPRB, 0.32900E+01_JPRB, 0.37529E+01_JPRB, 0.34499E+01_JPRB, 0.38997E+01_JPRB, 0.39096E+01_JPRB, 0.39678E+01_JPRB ,&
&  0.36516E+01_JPRB, 0.24745E+01_JPRB, 0.18699E+01_JPRB, 0.17194E+01_JPRB, 0.13460E+01_JPRB, 0.10075E+01_JPRB, 0.10676E+01_JPRB ,&
&  0.10396E+01_JPRB, 0.10110E+01_JPRB, 0.98807E+00_JPRB, 0.89351E+00_JPRB, 0.80878E+00_JPRB, 0.78014E+00_JPRB, 0.82644E+00_JPRB ,&
&  0.86407E+00_JPRB, 0.89010E+00_JPRB, 0.87671E+00_JPRB, 0.87168E+00_JPRB, 0.84423E+00_JPRB, 0.85984E+00_JPRB, 0.89561E+00_JPRB ,&
&  0.92912E+00_JPRB, 0.96222E+00_JPRB, 0.97938E+00_JPRB, 0.10076E+01_JPRB, 0.10447E+01_JPRB, 0.99100E+00_JPRB, 0.91939E+00_JPRB ,&
&  0.90589E+00_JPRB, 0.89695E+00_JPRB, 0.85370E+00_JPRB, 0.83445E+00_JPRB, 0.82022E+00_JPRB, 0.85543E+00_JPRB, 0.96002E+00_JPRB ,&
&  0.94505E+00_JPRB, 0.83370E+00_JPRB, 0.72033E+00_JPRB, 0.68926E+00_JPRB, 0.69459E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.72670E+00_JPRB, 0.65704E+00_JPRB, 0.61511E+00_JPRB, 0.61029E+00_JPRB, 0.59772E+00_JPRB, 0.60385E+00_JPRB, 0.61323E+00_JPRB ,&
&  0.61136E+00_JPRB, 0.63059E+00_JPRB, 0.62439E+00_JPRB, 0.63736E+00_JPRB, 0.65725E+00_JPRB, 0.66362E+00_JPRB, 0.68702E+00_JPRB ,&
&  0.71082E+00_JPRB, 0.73734E+00_JPRB, 0.75822E+00_JPRB, 0.72077E+00_JPRB, 0.71151E+00_JPRB, 0.71198E+00_JPRB, 0.72509E+00_JPRB ,&
&  0.75587E+00_JPRB, 0.79925E+00_JPRB, 0.84901E+00_JPRB, 0.91342E+00_JPRB, 0.10096E+01_JPRB, 0.11852E+01_JPRB, 0.15179E+01_JPRB ,&
&  0.23509E+01_JPRB, 0.18844E+01_JPRB, 0.65425E+00_JPRB, 0.81808E+00_JPRB, 0.80641E+00_JPRB, 0.80069E+00_JPRB, 0.70715E+00_JPRB ,&
&  0.74589E+00_JPRB, 0.83283E+00_JPRB, 0.85801E+00_JPRB, 0.93529E+00_JPRB, 0.11133E+01_JPRB, 0.13662E+01_JPRB, 0.15711E+01_JPRB ,&
&  0.17049E+01_JPRB, 0.17993E+01_JPRB, 0.18367E+01_JPRB, 0.18749E+01_JPRB, 0.19322E+01_JPRB, 0.20437E+01_JPRB, 0.22221E+01_JPRB ,&
&  0.24878E+01_JPRB, 0.28689E+01_JPRB, 0.33729E+01_JPRB, 0.33341E+01_JPRB, 0.40991E+01_JPRB, 0.39352E+01_JPRB, 0.36663E+01_JPRB ,&
&  0.30211E+01_JPRB, 0.24306E+01_JPRB, 0.20486E+01_JPRB, 0.16614E+01_JPRB, 0.11976E+01_JPRB, 0.97060E+00_JPRB, 0.99740E+00_JPRB ,&
&  0.10011E+01_JPRB, 0.90552E+00_JPRB, 0.88305E+00_JPRB, 0.86821E+00_JPRB, 0.81999E+00_JPRB, 0.87057E+00_JPRB, 0.94965E+00_JPRB ,&
&  0.97611E+00_JPRB, 0.97619E+00_JPRB, 0.98145E+00_JPRB, 0.96569E+00_JPRB, 0.95393E+00_JPRB, 0.96033E+00_JPRB, 0.99084E+00_JPRB ,&
&  0.10072E+01_JPRB, 0.10245E+01_JPRB, 0.10262E+01_JPRB, 0.10529E+01_JPRB, 0.10792E+01_JPRB, 0.10157E+01_JPRB, 0.93394E+00_JPRB ,&
&  0.92023E+00_JPRB, 0.93418E+00_JPRB, 0.91276E+00_JPRB, 0.89968E+00_JPRB, 0.86409E+00_JPRB, 0.83448E+00_JPRB, 0.83320E+00_JPRB ,&
&  0.91595E+00_JPRB, 0.85549E+00_JPRB, 0.80066E+00_JPRB, 0.73324E+00_JPRB, 0.75354E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.75711E+00_JPRB, 0.71633E+00_JPRB, 0.64519E+00_JPRB, 0.61771E+00_JPRB, 0.61348E+00_JPRB, 0.60373E+00_JPRB, 0.58894E+00_JPRB ,&
&  0.56748E+00_JPRB, 0.56068E+00_JPRB, 0.58667E+00_JPRB, 0.61739E+00_JPRB, 0.64441E+00_JPRB, 0.64680E+00_JPRB, 0.65421E+00_JPRB ,&
&  0.65944E+00_JPRB, 0.68118E+00_JPRB, 0.72252E+00_JPRB, 0.73120E+00_JPRB, 0.71713E+00_JPRB, 0.70909E+00_JPRB, 0.70911E+00_JPRB ,&
&  0.71674E+00_JPRB, 0.72858E+00_JPRB, 0.75759E+00_JPRB, 0.79127E+00_JPRB, 0.84784E+00_JPRB, 0.94501E+00_JPRB, 0.11570E+01_JPRB ,&
&  0.16360E+01_JPRB, 0.28018E+01_JPRB, 0.79370E+00_JPRB, 0.10183E+01_JPRB, 0.97042E+00_JPRB, 0.88054E+00_JPRB, 0.81572E+00_JPRB ,&
&  0.79933E+00_JPRB, 0.81683E+00_JPRB, 0.88659E+00_JPRB, 0.89981E+00_JPRB, 0.83460E+00_JPRB, 0.98406E+00_JPRB, 0.12103E+01_JPRB ,&
&  0.14165E+01_JPRB, 0.15131E+01_JPRB, 0.15479E+01_JPRB, 0.15399E+01_JPRB, 0.15604E+01_JPRB, 0.16043E+01_JPRB, 0.17053E+01_JPRB ,&
&  0.18919E+01_JPRB, 0.21986E+01_JPRB, 0.26455E+01_JPRB, 0.28315E+01_JPRB, 0.37554E+01_JPRB, 0.39978E+01_JPRB, 0.38225E+01_JPRB ,&
&  0.30816E+01_JPRB, 0.27678E+01_JPRB, 0.24385E+01_JPRB, 0.18176E+01_JPRB, 0.13113E+01_JPRB, 0.10555E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.99979E+00_JPRB, 0.91543E+00_JPRB, 0.90029E+00_JPRB, 0.87973E+00_JPRB, 0.88553E+00_JPRB, 0.93976E+00_JPRB, 0.99690E+00_JPRB ,&
&  0.10055E+01_JPRB, 0.10166E+01_JPRB, 0.10079E+01_JPRB, 0.99457E+00_JPRB, 0.99567E+00_JPRB, 0.10025E+01_JPRB, 0.10347E+01_JPRB ,&
&  0.10461E+01_JPRB, 0.10540E+01_JPRB, 0.10509E+01_JPRB, 0.10225E+01_JPRB, 0.10248E+01_JPRB, 0.98697E+00_JPRB, 0.93601E+00_JPRB ,&
&  0.93861E+00_JPRB, 0.95717E+00_JPRB, 0.94325E+00_JPRB, 0.90657E+00_JPRB, 0.93087E+00_JPRB, 0.90733E+00_JPRB, 0.87984E+00_JPRB ,&
&  0.91696E+00_JPRB, 0.93317E+00_JPRB, 0.91285E+00_JPRB, 0.78984E+00_JPRB, 0.79847E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.78158E+00_JPRB, 0.74302E+00_JPRB, 0.69727E+00_JPRB, 0.64159E+00_JPRB, 0.64525E+00_JPRB, 0.65529E+00_JPRB, 0.63085E+00_JPRB ,&
&  0.58443E+00_JPRB, 0.58003E+00_JPRB, 0.59045E+00_JPRB, 0.61351E+00_JPRB, 0.63275E+00_JPRB, 0.63480E+00_JPRB, 0.65488E+00_JPRB ,&
&  0.64678E+00_JPRB, 0.64253E+00_JPRB, 0.66412E+00_JPRB, 0.69776E+00_JPRB, 0.70058E+00_JPRB, 0.69841E+00_JPRB, 0.71042E+00_JPRB ,&
&  0.71626E+00_JPRB, 0.70428E+00_JPRB, 0.69932E+00_JPRB, 0.70892E+00_JPRB, 0.74673E+00_JPRB, 0.81282E+00_JPRB, 0.93129E+00_JPRB ,&
&  0.13918E+01_JPRB, 0.31389E+01_JPRB, 0.22912E+01_JPRB, 0.17931E+01_JPRB, 0.12705E+01_JPRB, 0.10066E+01_JPRB, 0.90099E+00_JPRB ,&
&  0.94461E+00_JPRB, 0.93501E+00_JPRB, 0.10166E+01_JPRB, 0.89585E+00_JPRB, 0.79812E+00_JPRB, 0.77906E+00_JPRB, 0.90239E+00_JPRB ,&
&  0.10347E+01_JPRB, 0.11406E+01_JPRB, 0.12208E+01_JPRB, 0.12305E+01_JPRB, 0.12420E+01_JPRB, 0.12731E+01_JPRB, 0.13166E+01_JPRB ,&
&  0.14164E+01_JPRB, 0.16198E+01_JPRB, 0.19658E+01_JPRB, 0.24069E+01_JPRB, 0.29483E+01_JPRB, 0.35193E+01_JPRB, 0.40894E+01_JPRB ,&
&  0.39024E+01_JPRB, 0.34274E+01_JPRB, 0.28006E+01_JPRB, 0.19732E+01_JPRB, 0.13751E+01_JPRB, 0.12851E+01_JPRB, 0.10695E+01_JPRB ,&
&  0.10088E+01_JPRB, 0.94498E+00_JPRB, 0.94330E+00_JPRB, 0.96221E+00_JPRB, 0.98948E+00_JPRB, 0.10253E+01_JPRB, 0.10488E+01_JPRB ,&
&  0.10560E+01_JPRB, 0.10472E+01_JPRB, 0.10409E+01_JPRB, 0.10236E+01_JPRB, 0.10160E+01_JPRB, 0.10161E+01_JPRB, 0.10267E+01_JPRB ,&
&  0.10310E+01_JPRB, 0.10442E+01_JPRB, 0.99773E+00_JPRB, 0.98531E+00_JPRB, 0.98019E+00_JPRB, 0.96031E+00_JPRB, 0.91260E+00_JPRB ,&
&  0.92422E+00_JPRB, 0.96511E+00_JPRB, 0.97772E+00_JPRB, 0.96396E+00_JPRB, 0.96270E+00_JPRB, 0.97298E+00_JPRB, 0.94354E+00_JPRB ,&
&  0.94853E+00_JPRB, 0.93415E+00_JPRB, 0.87515E+00_JPRB, 0.83670E+00_JPRB, 0.83284E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.82301E+00_JPRB, 0.79169E+00_JPRB, 0.75482E+00_JPRB, 0.71028E+00_JPRB, 0.68592E+00_JPRB, 0.69114E+00_JPRB, 0.70837E+00_JPRB ,&
&  0.68604E+00_JPRB, 0.66354E+00_JPRB, 0.65603E+00_JPRB, 0.67083E+00_JPRB, 0.69265E+00_JPRB, 0.69604E+00_JPRB, 0.69561E+00_JPRB ,&
&  0.68154E+00_JPRB, 0.67489E+00_JPRB, 0.66197E+00_JPRB, 0.66862E+00_JPRB, 0.71737E+00_JPRB, 0.71466E+00_JPRB, 0.70357E+00_JPRB ,&
&  0.72129E+00_JPRB, 0.71711E+00_JPRB, 0.69593E+00_JPRB, 0.69333E+00_JPRB, 0.70185E+00_JPRB, 0.74151E+00_JPRB, 0.82521E+00_JPRB ,&
&  0.12987E+01_JPRB, 0.29135E+01_JPRB, 0.15757E+01_JPRB, 0.20122E+01_JPRB, 0.13910E+01_JPRB, 0.11223E+01_JPRB, 0.90926E+00_JPRB ,&
&  0.92670E+00_JPRB, 0.11155E+01_JPRB, 0.99341E+00_JPRB, 0.91042E+00_JPRB, 0.81076E+00_JPRB, 0.76906E+00_JPRB, 0.82383E+00_JPRB ,&
&  0.89266E+00_JPRB, 0.91982E+00_JPRB, 0.96028E+00_JPRB, 0.99106E+00_JPRB, 0.98148E+00_JPRB, 0.98448E+00_JPRB, 0.10193E+01_JPRB ,&
&  0.10670E+01_JPRB, 0.11760E+01_JPRB, 0.13991E+01_JPRB, 0.17330E+01_JPRB, 0.25093E+01_JPRB, 0.33774E+01_JPRB, 0.43324E+01_JPRB ,&
&  0.43839E+01_JPRB, 0.43268E+01_JPRB, 0.32412E+01_JPRB, 0.21759E+01_JPRB, 0.15408E+01_JPRB, 0.13329E+01_JPRB, 0.11560E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.10270E+01_JPRB, 0.99434E+00_JPRB, 0.10040E+01_JPRB, 0.10728E+01_JPRB, 0.11128E+01_JPRB, 0.10953E+01_JPRB ,&
&  0.10919E+01_JPRB, 0.10704E+01_JPRB, 0.10585E+01_JPRB, 0.10339E+01_JPRB, 0.10333E+01_JPRB, 0.10273E+01_JPRB, 0.10150E+01_JPRB ,&
&  0.10071E+01_JPRB, 0.10133E+01_JPRB, 0.97998E+00_JPRB, 0.97890E+00_JPRB, 0.97640E+00_JPRB, 0.96247E+00_JPRB, 0.92019E+00_JPRB ,&
&  0.93792E+00_JPRB, 0.95874E+00_JPRB, 0.96600E+00_JPRB, 0.95198E+00_JPRB, 0.95359E+00_JPRB, 0.99924E+00_JPRB, 0.97571E+00_JPRB ,&
&  0.97500E+00_JPRB, 0.95495E+00_JPRB, 0.89953E+00_JPRB, 0.87700E+00_JPRB, 0.86966E+00_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.86836E+00_JPRB, 0.83031E+00_JPRB, 0.80180E+00_JPRB, 0.76818E+00_JPRB, 0.74255E+00_JPRB, 0.72745E+00_JPRB, 0.73380E+00_JPRB ,&
&  0.72916E+00_JPRB, 0.70453E+00_JPRB, 0.68780E+00_JPRB, 0.69030E+00_JPRB, 0.68925E+00_JPRB, 0.69392E+00_JPRB, 0.69747E+00_JPRB ,&
&  0.69122E+00_JPRB, 0.67894E+00_JPRB, 0.67433E+00_JPRB, 0.67825E+00_JPRB, 0.70528E+00_JPRB, 0.74216E+00_JPRB, 0.74929E+00_JPRB ,&
&  0.73951E+00_JPRB, 0.73247E+00_JPRB, 0.71238E+00_JPRB, 0.69334E+00_JPRB, 0.69008E+00_JPRB, 0.70840E+00_JPRB, 0.77113E+00_JPRB ,&
&  0.12082E+01_JPRB, 0.29894E+01_JPRB, 0.14188E+01_JPRB, 0.20672E+01_JPRB, 0.14632E+01_JPRB, 0.11826E+01_JPRB, 0.96013E+00_JPRB ,&
&  0.97196E+00_JPRB, 0.10792E+01_JPRB, 0.10137E+01_JPRB, 0.94030E+00_JPRB, 0.85857E+00_JPRB, 0.81277E+00_JPRB, 0.80476E+00_JPRB ,&
&  0.85099E+00_JPRB, 0.85720E+00_JPRB, 0.85514E+00_JPRB, 0.86616E+00_JPRB, 0.85098E+00_JPRB, 0.84297E+00_JPRB, 0.85487E+00_JPRB ,&
&  0.85876E+00_JPRB, 0.88168E+00_JPRB, 0.99206E+00_JPRB, 0.14180E+01_JPRB, 0.20080E+01_JPRB, 0.31988E+01_JPRB, 0.49721E+01_JPRB ,&
&  0.49423E+01_JPRB, 0.50850E+01_JPRB, 0.36272E+01_JPRB, 0.24201E+01_JPRB, 0.16701E+01_JPRB, 0.13822E+01_JPRB, 0.12699E+01_JPRB ,&
&  0.11691E+01_JPRB, 0.10983E+01_JPRB, 0.10558E+01_JPRB, 0.10582E+01_JPRB, 0.11012E+01_JPRB, 0.11063E+01_JPRB, 0.11154E+01_JPRB ,&
&  0.11121E+01_JPRB, 0.10592E+01_JPRB, 0.10354E+01_JPRB, 0.10212E+01_JPRB, 0.10261E+01_JPRB, 0.10231E+01_JPRB, 0.10095E+01_JPRB ,&
&  0.99739E+00_JPRB, 0.98953E+00_JPRB, 0.97138E+00_JPRB, 0.98181E+00_JPRB, 0.99103E+00_JPRB, 0.98129E+00_JPRB, 0.93802E+00_JPRB ,&
&  0.94649E+00_JPRB, 0.95648E+00_JPRB, 0.94886E+00_JPRB, 0.96120E+00_JPRB, 0.96507E+00_JPRB, 0.10180E+01_JPRB, 0.97521E+00_JPRB ,&
&  0.96911E+00_JPRB, 0.95088E+00_JPRB, 0.91009E+00_JPRB, 0.89726E+00_JPRB, 0.90031E+00_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.90362E+00_JPRB, 0.88047E+00_JPRB, 0.86138E+00_JPRB, 0.83330E+00_JPRB, 0.79389E+00_JPRB, 0.77286E+00_JPRB, 0.75988E+00_JPRB ,&
&  0.75381E+00_JPRB, 0.72872E+00_JPRB, 0.71009E+00_JPRB, 0.70962E+00_JPRB, 0.71474E+00_JPRB, 0.70686E+00_JPRB, 0.70434E+00_JPRB ,&
&  0.70992E+00_JPRB, 0.71235E+00_JPRB, 0.70310E+00_JPRB, 0.69366E+00_JPRB, 0.69428E+00_JPRB, 0.72777E+00_JPRB, 0.74229E+00_JPRB ,&
&  0.74795E+00_JPRB, 0.73836E+00_JPRB, 0.71614E+00_JPRB, 0.68758E+00_JPRB, 0.67990E+00_JPRB, 0.68785E+00_JPRB, 0.73189E+00_JPRB ,&
&  0.10563E+01_JPRB, 0.12688E+01_JPRB, 0.20988E+01_JPRB, 0.20958E+01_JPRB, 0.15161E+01_JPRB, 0.12441E+01_JPRB, 0.10980E+01_JPRB ,&
&  0.11119E+01_JPRB, 0.10807E+01_JPRB, 0.10162E+01_JPRB, 0.95612E+00_JPRB, 0.90046E+00_JPRB, 0.85433E+00_JPRB, 0.82335E+00_JPRB ,&
&  0.81952E+00_JPRB, 0.81802E+00_JPRB, 0.81533E+00_JPRB, 0.81760E+00_JPRB, 0.81328E+00_JPRB, 0.81239E+00_JPRB, 0.81225E+00_JPRB ,&
&  0.81304E+00_JPRB, 0.81180E+00_JPRB, 0.83618E+00_JPRB, 0.10151E+01_JPRB, 0.17218E+01_JPRB, 0.28219E+01_JPRB, 0.44542E+01_JPRB ,&
&  0.52963E+01_JPRB, 0.49095E+01_JPRB, 0.34403E+01_JPRB, 0.24752E+01_JPRB, 0.17576E+01_JPRB, 0.13835E+01_JPRB, 0.12758E+01_JPRB ,&
&  0.12300E+01_JPRB, 0.11888E+01_JPRB, 0.11214E+01_JPRB, 0.11054E+01_JPRB, 0.11052E+01_JPRB, 0.10848E+01_JPRB, 0.10804E+01_JPRB ,&
&  0.10698E+01_JPRB, 0.10621E+01_JPRB, 0.10465E+01_JPRB, 0.10283E+01_JPRB, 0.10079E+01_JPRB, 0.98980E+00_JPRB, 0.97938E+00_JPRB ,&
&  0.97191E+00_JPRB, 0.95715E+00_JPRB, 0.95199E+00_JPRB, 0.97786E+00_JPRB, 0.10272E+01_JPRB, 0.10334E+01_JPRB, 0.10114E+01_JPRB ,&
&  0.97053E+00_JPRB, 0.95022E+00_JPRB, 0.94115E+00_JPRB, 0.96362E+00_JPRB, 0.95172E+00_JPRB, 0.10361E+01_JPRB, 0.99739E+00_JPRB ,&
&  0.98121E+00_JPRB, 0.96545E+00_JPRB, 0.93283E+00_JPRB, 0.90846E+00_JPRB, 0.90416E+00_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.91320E+00_JPRB, 0.88425E+00_JPRB, 0.86648E+00_JPRB, 0.85405E+00_JPRB, 0.82924E+00_JPRB, 0.80266E+00_JPRB, 0.79634E+00_JPRB ,&
&  0.79388E+00_JPRB, 0.77558E+00_JPRB, 0.75353E+00_JPRB, 0.72514E+00_JPRB, 0.71799E+00_JPRB, 0.72231E+00_JPRB, 0.72740E+00_JPRB ,&
&  0.72644E+00_JPRB, 0.72714E+00_JPRB, 0.72591E+00_JPRB, 0.72407E+00_JPRB, 0.72240E+00_JPRB, 0.74398E+00_JPRB, 0.75270E+00_JPRB ,&
&  0.74468E+00_JPRB, 0.74553E+00_JPRB, 0.72415E+00_JPRB, 0.70168E+00_JPRB, 0.69103E+00_JPRB, 0.67834E+00_JPRB, 0.67542E+00_JPRB ,&
&  0.88157E+00_JPRB, 0.13524E+01_JPRB, 0.19493E+01_JPRB, 0.18396E+01_JPRB, 0.14942E+01_JPRB, 0.12298E+01_JPRB, 0.11631E+01_JPRB ,&
&  0.11274E+01_JPRB, 0.10836E+01_JPRB, 0.10196E+01_JPRB, 0.96588E+00_JPRB, 0.91855E+00_JPRB, 0.88568E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.83579E+00_JPRB, 0.83088E+00_JPRB, 0.81159E+00_JPRB, 0.80094E+00_JPRB, 0.79808E+00_JPRB, 0.80250E+00_JPRB, 0.80665E+00_JPRB ,&
&  0.81158E+00_JPRB, 0.81301E+00_JPRB, 0.81998E+00_JPRB, 0.88401E+00_JPRB, 0.13050E+01_JPRB, 0.16079E+01_JPRB, 0.29679E+01_JPRB ,&
&  0.42567E+01_JPRB, 0.34474E+01_JPRB, 0.26299E+01_JPRB, 0.21573E+01_JPRB, 0.17562E+01_JPRB, 0.14066E+01_JPRB, 0.12463E+01_JPRB ,&
&  0.11854E+01_JPRB, 0.11536E+01_JPRB, 0.11020E+01_JPRB, 0.10771E+01_JPRB, 0.10560E+01_JPRB, 0.10420E+01_JPRB, 0.10318E+01_JPRB ,&
&  0.10102E+01_JPRB, 0.10079E+01_JPRB, 0.10047E+01_JPRB, 0.10013E+01_JPRB, 0.98338E+00_JPRB, 0.97168E+00_JPRB, 0.96064E+00_JPRB ,&
&  0.95204E+00_JPRB, 0.93284E+00_JPRB, 0.92716E+00_JPRB, 0.95232E+00_JPRB, 0.10284E+01_JPRB, 0.10811E+01_JPRB, 0.10701E+01_JPRB ,&
&  0.10139E+01_JPRB, 0.96901E+00_JPRB, 0.96790E+00_JPRB, 0.96527E+00_JPRB, 0.97561E+00_JPRB, 0.10829E+01_JPRB, 0.10325E+01_JPRB ,&
&  0.10003E+01_JPRB, 0.98838E+00_JPRB, 0.96567E+00_JPRB, 0.93627E+00_JPRB, 0.92913E+00_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.94991E+00_JPRB, 0.91649E+00_JPRB, 0.89034E+00_JPRB, 0.86256E+00_JPRB, 0.85385E+00_JPRB, 0.83644E+00_JPRB, 0.81379E+00_JPRB ,&
&  0.80143E+00_JPRB, 0.80095E+00_JPRB, 0.79460E+00_JPRB, 0.77866E+00_JPRB, 0.76448E+00_JPRB, 0.75905E+00_JPRB, 0.75897E+00_JPRB ,&
&  0.75752E+00_JPRB, 0.75617E+00_JPRB, 0.75354E+00_JPRB, 0.74590E+00_JPRB, 0.74913E+00_JPRB, 0.75832E+00_JPRB, 0.76743E+00_JPRB ,&
&  0.75017E+00_JPRB, 0.73617E+00_JPRB, 0.72417E+00_JPRB, 0.71207E+00_JPRB, 0.70865E+00_JPRB, 0.69058E+00_JPRB, 0.66744E+00_JPRB ,&
&  0.74652E+00_JPRB, 0.10003E+01_JPRB, 0.13706E+01_JPRB, 0.16747E+01_JPRB, 0.14918E+01_JPRB, 0.12936E+01_JPRB, 0.12065E+01_JPRB ,&
&  0.11457E+01_JPRB, 0.10674E+01_JPRB, 0.10083E+01_JPRB, 0.96429E+00_JPRB, 0.94272E+00_JPRB, 0.91565E+00_JPRB, 0.86702E+00_JPRB ,&
&  0.83214E+00_JPRB, 0.81290E+00_JPRB, 0.80588E+00_JPRB, 0.80575E+00_JPRB, 0.80018E+00_JPRB, 0.81309E+00_JPRB, 0.82347E+00_JPRB ,&
&  0.82996E+00_JPRB, 0.82651E+00_JPRB, 0.81917E+00_JPRB, 0.82687E+00_JPRB, 0.10008E+01_JPRB, 0.11565E+01_JPRB, 0.16165E+01_JPRB ,&
&  0.18487E+01_JPRB, 0.19564E+01_JPRB, 0.18713E+01_JPRB, 0.17874E+01_JPRB, 0.16487E+01_JPRB, 0.14102E+01_JPRB, 0.12281E+01_JPRB ,&
&  0.11277E+01_JPRB, 0.10882E+01_JPRB, 0.10470E+01_JPRB, 0.10297E+01_JPRB, 0.10128E+01_JPRB, 0.99476E+00_JPRB, 0.98691E+00_JPRB ,&
&  0.97741E+00_JPRB, 0.96854E+00_JPRB, 0.96073E+00_JPRB, 0.96723E+00_JPRB, 0.95794E+00_JPRB, 0.93513E+00_JPRB, 0.91773E+00_JPRB ,&
&  0.91169E+00_JPRB, 0.89959E+00_JPRB, 0.91028E+00_JPRB, 0.94723E+00_JPRB, 0.98274E+00_JPRB, 0.10039E+01_JPRB, 0.10253E+01_JPRB ,&
&  0.10418E+01_JPRB, 0.10300E+01_JPRB, 0.10042E+01_JPRB, 0.10050E+01_JPRB, 0.10111E+01_JPRB, 0.10995E+01_JPRB, 0.10557E+01_JPRB ,&
&  0.10280E+01_JPRB, 0.10095E+01_JPRB, 0.98972E+00_JPRB, 0.94741E+00_JPRB, 0.97509E+00_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.95767E+00_JPRB, 0.94517E+00_JPRB, 0.92730E+00_JPRB, 0.89547E+00_JPRB, 0.88107E+00_JPRB, 0.86603E+00_JPRB, 0.84923E+00_JPRB ,&
&  0.84091E+00_JPRB, 0.83321E+00_JPRB, 0.82764E+00_JPRB, 0.81440E+00_JPRB, 0.80102E+00_JPRB, 0.79304E+00_JPRB, 0.78273E+00_JPRB ,&
&  0.78349E+00_JPRB, 0.78400E+00_JPRB, 0.77617E+00_JPRB, 0.76807E+00_JPRB, 0.76487E+00_JPRB, 0.76241E+00_JPRB, 0.76658E+00_JPRB ,&
&  0.75787E+00_JPRB, 0.74729E+00_JPRB, 0.74624E+00_JPRB, 0.73831E+00_JPRB, 0.72559E+00_JPRB, 0.71485E+00_JPRB, 0.68455E+00_JPRB ,&
&  0.68427E+00_JPRB, 0.72568E+00_JPRB, 0.99505E+00_JPRB, 0.13001E+01_JPRB, 0.12945E+01_JPRB, 0.12723E+01_JPRB, 0.11975E+01_JPRB ,&
&  0.11301E+01_JPRB, 0.10603E+01_JPRB, 0.98214E+00_JPRB, 0.93164E+00_JPRB, 0.91922E+00_JPRB, 0.90314E+00_JPRB, 0.88151E+00_JPRB ,&
&  0.85497E+00_JPRB, 0.83074E+00_JPRB, 0.81511E+00_JPRB, 0.82053E+00_JPRB, 0.81720E+00_JPRB, 0.82159E+00_JPRB, 0.83109E+00_JPRB ,&
&  0.83317E+00_JPRB, 0.83200E+00_JPRB, 0.81852E+00_JPRB, 0.81006E+00_JPRB, 0.80911E+00_JPRB, 0.83220E+00_JPRB, 0.95487E+00_JPRB ,&
&  0.11315E+01_JPRB, 0.12173E+01_JPRB, 0.12656E+01_JPRB, 0.13180E+01_JPRB, 0.13470E+01_JPRB, 0.13124E+01_JPRB, 0.12141E+01_JPRB ,&
&  0.11320E+01_JPRB, 0.10815E+01_JPRB, 0.10275E+01_JPRB, 0.10273E+01_JPRB, 0.10091E+01_JPRB, 0.98434E+00_JPRB, 0.96142E+00_JPRB ,&
&  0.93671E+00_JPRB, 0.93177E+00_JPRB, 0.91671E+00_JPRB, 0.90887E+00_JPRB, 0.90150E+00_JPRB, 0.88813E+00_JPRB, 0.87250E+00_JPRB ,&
&  0.86877E+00_JPRB, 0.86369E+00_JPRB, 0.87193E+00_JPRB, 0.89987E+00_JPRB, 0.93470E+00_JPRB, 0.95693E+00_JPRB, 0.98018E+00_JPRB ,&
&  0.99973E+00_JPRB, 0.10239E+01_JPRB, 0.99309E+00_JPRB, 0.98788E+00_JPRB, 0.10888E+01_JPRB, 0.10882E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10258E+01_JPRB, 0.10023E+01_JPRB, 0.98788E+00_JPRB, 0.93689E+00_JPRB, 0.93252E+00_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.85654E+00_JPRB, 0.88515E+00_JPRB, 0.88207E+00_JPRB, 0.85890E+00_JPRB, 0.84616E+00_JPRB, 0.84140E+00_JPRB, 0.84081E+00_JPRB ,&
&  0.83756E+00_JPRB, 0.82790E+00_JPRB, 0.83324E+00_JPRB, 0.84034E+00_JPRB, 0.83869E+00_JPRB, 0.82789E+00_JPRB, 0.82010E+00_JPRB ,&
&  0.81818E+00_JPRB, 0.82198E+00_JPRB, 0.82694E+00_JPRB, 0.81691E+00_JPRB, 0.79617E+00_JPRB, 0.78248E+00_JPRB, 0.77033E+00_JPRB ,&
&  0.76850E+00_JPRB, 0.76805E+00_JPRB, 0.76557E+00_JPRB, 0.76681E+00_JPRB, 0.74611E+00_JPRB, 0.72309E+00_JPRB, 0.70411E+00_JPRB ,&
&  0.68819E+00_JPRB, 0.61399E+00_JPRB, 0.74483E+00_JPRB, 0.95821E+00_JPRB, 0.10670E+01_JPRB, 0.10742E+01_JPRB, 0.10260E+01_JPRB ,&
&  0.99481E+00_JPRB, 0.95983E+00_JPRB, 0.92098E+00_JPRB, 0.88209E+00_JPRB, 0.86091E+00_JPRB, 0.84879E+00_JPRB, 0.84020E+00_JPRB ,&
&  0.83175E+00_JPRB, 0.82204E+00_JPRB, 0.81614E+00_JPRB, 0.80013E+00_JPRB, 0.78712E+00_JPRB, 0.78600E+00_JPRB, 0.78929E+00_JPRB ,&
&  0.79603E+00_JPRB, 0.79417E+00_JPRB, 0.79561E+00_JPRB, 0.79726E+00_JPRB, 0.77967E+00_JPRB, 0.78029E+00_JPRB, 0.79785E+00_JPRB ,&
&  0.85237E+00_JPRB, 0.89995E+00_JPRB, 0.92941E+00_JPRB, 0.99969E+00_JPRB, 0.10573E+01_JPRB, 0.11053E+01_JPRB, 0.11031E+01_JPRB ,&
&  0.10733E+01_JPRB, 0.10485E+01_JPRB, 0.10207E+01_JPRB, 0.10081E+01_JPRB, 0.99026E+00_JPRB, 0.95504E+00_JPRB, 0.91954E+00_JPRB ,&
&  0.90303E+00_JPRB, 0.89272E+00_JPRB, 0.87377E+00_JPRB, 0.86140E+00_JPRB, 0.84864E+00_JPRB, 0.82932E+00_JPRB, 0.81311E+00_JPRB ,&
&  0.80218E+00_JPRB, 0.80069E+00_JPRB, 0.80468E+00_JPRB, 0.82134E+00_JPRB, 0.84729E+00_JPRB, 0.87782E+00_JPRB, 0.90841E+00_JPRB ,&
&  0.93669E+00_JPRB, 0.95722E+00_JPRB, 0.98437E+00_JPRB, 0.10323E+01_JPRB, 0.10322E+01_JPRB, 0.10243E+01_JPRB, 0.10282E+01_JPRB ,&
&  0.10140E+01_JPRB, 0.98091E+00_JPRB, 0.94540E+00_JPRB, 0.88629E+00_JPRB, 0.80532E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.82526E+00_JPRB, 0.84232E+00_JPRB, 0.84231E+00_JPRB, 0.83401E+00_JPRB, 0.82742E+00_JPRB, 0.82398E+00_JPRB, 0.82848E+00_JPRB ,&
&  0.82754E+00_JPRB, 0.82224E+00_JPRB, 0.82563E+00_JPRB, 0.82858E+00_JPRB, 0.83124E+00_JPRB, 0.82940E+00_JPRB, 0.83487E+00_JPRB ,&
&  0.83618E+00_JPRB, 0.83823E+00_JPRB, 0.84177E+00_JPRB, 0.83581E+00_JPRB, 0.81629E+00_JPRB, 0.80241E+00_JPRB, 0.78838E+00_JPRB ,&
&  0.78701E+00_JPRB, 0.78635E+00_JPRB, 0.78667E+00_JPRB, 0.78026E+00_JPRB, 0.76055E+00_JPRB, 0.74369E+00_JPRB, 0.73306E+00_JPRB ,&
&  0.73249E+00_JPRB, 0.58898E+00_JPRB, 0.65019E+00_JPRB, 0.83649E+00_JPRB, 0.92283E+00_JPRB, 0.90550E+00_JPRB, 0.90054E+00_JPRB ,&
&  0.90979E+00_JPRB, 0.90760E+00_JPRB, 0.88784E+00_JPRB, 0.85711E+00_JPRB, 0.81731E+00_JPRB, 0.80587E+00_JPRB, 0.78812E+00_JPRB ,&
&  0.77037E+00_JPRB, 0.76360E+00_JPRB, 0.75789E+00_JPRB, 0.75804E+00_JPRB, 0.74914E+00_JPRB, 0.73859E+00_JPRB, 0.74056E+00_JPRB ,&
&  0.75014E+00_JPRB, 0.75222E+00_JPRB, 0.75905E+00_JPRB, 0.76582E+00_JPRB, 0.76230E+00_JPRB, 0.77881E+00_JPRB, 0.78959E+00_JPRB ,&
&  0.80503E+00_JPRB, 0.81778E+00_JPRB, 0.82795E+00_JPRB, 0.84561E+00_JPRB, 0.87903E+00_JPRB, 0.94094E+00_JPRB, 0.97941E+00_JPRB ,&
&  0.97626E+00_JPRB, 0.96176E+00_JPRB, 0.96095E+00_JPRB, 0.95724E+00_JPRB, 0.94427E+00_JPRB, 0.91940E+00_JPRB, 0.89203E+00_JPRB ,&
&  0.87464E+00_JPRB, 0.85845E+00_JPRB, 0.84371E+00_JPRB, 0.82632E+00_JPRB, 0.80768E+00_JPRB, 0.79091E+00_JPRB, 0.77778E+00_JPRB ,&
&  0.76932E+00_JPRB, 0.76605E+00_JPRB, 0.77375E+00_JPRB, 0.79228E+00_JPRB, 0.81641E+00_JPRB, 0.84395E+00_JPRB, 0.87272E+00_JPRB ,&
&  0.90583E+00_JPRB, 0.94114E+00_JPRB, 0.97629E+00_JPRB, 0.93563E+00_JPRB, 0.99529E+00_JPRB, 0.10059E+01_JPRB, 0.10106E+01_JPRB ,&
&  0.99923E+00_JPRB, 0.95990E+00_JPRB, 0.90358E+00_JPRB, 0.71182E+00_JPRB, 0.78848E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.81844E+00_JPRB, 0.82404E+00_JPRB, 0.82729E+00_JPRB, 0.81621E+00_JPRB, 0.80233E+00_JPRB, 0.79778E+00_JPRB, 0.80539E+00_JPRB ,&
&  0.80195E+00_JPRB, 0.78912E+00_JPRB, 0.79415E+00_JPRB, 0.79807E+00_JPRB, 0.80542E+00_JPRB, 0.80917E+00_JPRB, 0.81344E+00_JPRB ,&
&  0.81786E+00_JPRB, 0.82211E+00_JPRB, 0.82061E+00_JPRB, 0.82914E+00_JPRB, 0.82182E+00_JPRB, 0.81622E+00_JPRB, 0.81322E+00_JPRB ,&
&  0.80463E+00_JPRB, 0.79564E+00_JPRB, 0.78820E+00_JPRB, 0.77799E+00_JPRB, 0.77046E+00_JPRB, 0.76169E+00_JPRB, 0.74649E+00_JPRB ,&
&  0.75788E+00_JPRB, 0.60449E+00_JPRB, 0.67921E+00_JPRB, 0.80885E+00_JPRB, 0.81346E+00_JPRB, 0.81076E+00_JPRB, 0.82454E+00_JPRB ,&
&  0.84474E+00_JPRB, 0.85547E+00_JPRB, 0.84691E+00_JPRB, 0.82263E+00_JPRB, 0.77782E+00_JPRB, 0.75516E+00_JPRB, 0.74152E+00_JPRB ,&
&  0.72644E+00_JPRB, 0.71560E+00_JPRB, 0.70923E+00_JPRB, 0.71103E+00_JPRB, 0.71124E+00_JPRB, 0.70825E+00_JPRB, 0.70548E+00_JPRB ,&
&  0.70966E+00_JPRB, 0.72019E+00_JPRB, 0.72777E+00_JPRB, 0.73560E+00_JPRB, 0.74285E+00_JPRB, 0.74044E+00_JPRB, 0.74111E+00_JPRB ,&
&  0.74297E+00_JPRB, 0.74763E+00_JPRB, 0.75046E+00_JPRB, 0.75877E+00_JPRB, 0.77390E+00_JPRB, 0.81356E+00_JPRB, 0.84905E+00_JPRB ,&
&  0.86089E+00_JPRB, 0.86266E+00_JPRB, 0.86490E+00_JPRB, 0.86716E+00_JPRB, 0.87008E+00_JPRB, 0.86818E+00_JPRB, 0.85596E+00_JPRB ,&
&  0.84141E+00_JPRB, 0.82284E+00_JPRB, 0.81080E+00_JPRB, 0.80135E+00_JPRB, 0.78955E+00_JPRB, 0.78182E+00_JPRB, 0.78051E+00_JPRB ,&
&  0.77299E+00_JPRB, 0.76116E+00_JPRB, 0.76407E+00_JPRB, 0.77907E+00_JPRB, 0.79307E+00_JPRB, 0.80325E+00_JPRB, 0.82251E+00_JPRB ,&
&  0.85701E+00_JPRB, 0.89362E+00_JPRB, 0.92357E+00_JPRB, 0.95504E+00_JPRB, 0.97053E+00_JPRB, 0.97611E+00_JPRB, 0.97053E+00_JPRB ,&
&  0.95227E+00_JPRB, 0.93052E+00_JPRB, 0.68111E+00_JPRB, 0.78168E+00_JPRB, 0.80698E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.83009E+00_JPRB, 0.82040E+00_JPRB, 0.80844E+00_JPRB, 0.79189E+00_JPRB, 0.77614E+00_JPRB, 0.76777E+00_JPRB, 0.77424E+00_JPRB ,&
&  0.77362E+00_JPRB, 0.76216E+00_JPRB, 0.77215E+00_JPRB, 0.78535E+00_JPRB, 0.79609E+00_JPRB, 0.80603E+00_JPRB, 0.80972E+00_JPRB ,&
&  0.81394E+00_JPRB, 0.81154E+00_JPRB, 0.81518E+00_JPRB, 0.82751E+00_JPRB, 0.84606E+00_JPRB, 0.84757E+00_JPRB, 0.84444E+00_JPRB ,&
&  0.82187E+00_JPRB, 0.81048E+00_JPRB, 0.80009E+00_JPRB, 0.78751E+00_JPRB, 0.77624E+00_JPRB, 0.77245E+00_JPRB, 0.77529E+00_JPRB ,&
&  0.66165E+00_JPRB, 0.56555E+00_JPRB, 0.67272E+00_JPRB, 0.76340E+00_JPRB, 0.76784E+00_JPRB, 0.76975E+00_JPRB, 0.78713E+00_JPRB ,&
&  0.79323E+00_JPRB, 0.79458E+00_JPRB, 0.79208E+00_JPRB, 0.77955E+00_JPRB, 0.74798E+00_JPRB, 0.71438E+00_JPRB, 0.69587E+00_JPRB ,&
&  0.68466E+00_JPRB, 0.68939E+00_JPRB, 0.69130E+00_JPRB, 0.69863E+00_JPRB, 0.69894E+00_JPRB, 0.69208E+00_JPRB, 0.69089E+00_JPRB ,&
&  0.69206E+00_JPRB, 0.70250E+00_JPRB, 0.71259E+00_JPRB, 0.72030E+00_JPRB, 0.72363E+00_JPRB, 0.72486E+00_JPRB, 0.72239E+00_JPRB ,&
&  0.72112E+00_JPRB, 0.72082E+00_JPRB, 0.71929E+00_JPRB, 0.72387E+00_JPRB, 0.73330E+00_JPRB, 0.75189E+00_JPRB, 0.77945E+00_JPRB ,&
&  0.79631E+00_JPRB, 0.80214E+00_JPRB, 0.80527E+00_JPRB, 0.80469E+00_JPRB, 0.81333E+00_JPRB, 0.81832E+00_JPRB, 0.80994E+00_JPRB ,&
&  0.79807E+00_JPRB, 0.78890E+00_JPRB, 0.78944E+00_JPRB, 0.78889E+00_JPRB, 0.79027E+00_JPRB, 0.78910E+00_JPRB, 0.78239E+00_JPRB ,&
&  0.77318E+00_JPRB, 0.75955E+00_JPRB, 0.74972E+00_JPRB, 0.75011E+00_JPRB, 0.76116E+00_JPRB, 0.77027E+00_JPRB, 0.78912E+00_JPRB ,&
&  0.82218E+00_JPRB, 0.86170E+00_JPRB, 0.89465E+00_JPRB, 0.93027E+00_JPRB, 0.95080E+00_JPRB, 0.95979E+00_JPRB, 0.95196E+00_JPRB ,&
&  0.93223E+00_JPRB, 0.89289E+00_JPRB, 0.87827E+00_JPRB, 0.85379E+00_JPRB, 0.84101E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.79518E+00_JPRB, 0.78804E+00_JPRB, 0.77368E+00_JPRB, 0.74809E+00_JPRB, 0.71508E+00_JPRB, 0.72383E+00_JPRB, 0.73602E+00_JPRB ,&
&  0.72999E+00_JPRB, 0.72081E+00_JPRB, 0.73147E+00_JPRB, 0.75667E+00_JPRB, 0.78137E+00_JPRB, 0.79560E+00_JPRB, 0.79810E+00_JPRB ,&
&  0.79334E+00_JPRB, 0.79556E+00_JPRB, 0.80402E+00_JPRB, 0.82122E+00_JPRB, 0.82032E+00_JPRB, 0.80231E+00_JPRB, 0.79115E+00_JPRB ,&
&  0.78792E+00_JPRB, 0.78602E+00_JPRB, 0.78339E+00_JPRB, 0.76666E+00_JPRB, 0.75568E+00_JPRB, 0.75291E+00_JPRB, 0.75364E+00_JPRB ,&
&  0.74932E+00_JPRB, 0.60997E+00_JPRB, 0.71213E+00_JPRB, 0.72246E+00_JPRB, 0.73316E+00_JPRB, 0.72850E+00_JPRB, 0.72948E+00_JPRB ,&
&  0.73441E+00_JPRB, 0.74999E+00_JPRB, 0.74244E+00_JPRB, 0.71713E+00_JPRB, 0.69053E+00_JPRB, 0.66266E+00_JPRB, 0.64382E+00_JPRB ,&
&  0.63447E+00_JPRB, 0.63740E+00_JPRB, 0.64744E+00_JPRB, 0.65795E+00_JPRB, 0.66920E+00_JPRB, 0.67107E+00_JPRB, 0.67513E+00_JPRB ,&
&  0.67803E+00_JPRB, 0.68344E+00_JPRB, 0.68626E+00_JPRB, 0.69231E+00_JPRB, 0.69490E+00_JPRB, 0.69202E+00_JPRB, 0.68488E+00_JPRB ,&
&  0.67472E+00_JPRB, 0.66921E+00_JPRB, 0.66967E+00_JPRB, 0.67323E+00_JPRB, 0.68336E+00_JPRB, 0.70163E+00_JPRB, 0.72198E+00_JPRB ,&
&  0.74347E+00_JPRB, 0.75740E+00_JPRB, 0.76509E+00_JPRB, 0.76732E+00_JPRB, 0.77489E+00_JPRB, 0.78275E+00_JPRB, 0.78006E+00_JPRB ,&
&  0.77019E+00_JPRB, 0.75942E+00_JPRB, 0.75409E+00_JPRB, 0.74791E+00_JPRB, 0.74594E+00_JPRB, 0.74464E+00_JPRB, 0.73937E+00_JPRB ,&
&  0.73129E+00_JPRB, 0.73003E+00_JPRB, 0.73197E+00_JPRB, 0.72196E+00_JPRB, 0.72446E+00_JPRB, 0.73562E+00_JPRB, 0.75546E+00_JPRB ,&
&  0.77998E+00_JPRB, 0.81196E+00_JPRB, 0.84341E+00_JPRB, 0.86921E+00_JPRB, 0.88395E+00_JPRB, 0.88776E+00_JPRB, 0.88095E+00_JPRB ,&
&  0.86286E+00_JPRB, 0.83811E+00_JPRB, 0.81964E+00_JPRB, 0.80958E+00_JPRB, 0.81025E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.75654E+00_JPRB, 0.75686E+00_JPRB, 0.74653E+00_JPRB, 0.70876E+00_JPRB, 0.68023E+00_JPRB, 0.68997E+00_JPRB, 0.69888E+00_JPRB ,&
&  0.69831E+00_JPRB, 0.71319E+00_JPRB, 0.73834E+00_JPRB, 0.76387E+00_JPRB, 0.78093E+00_JPRB, 0.78085E+00_JPRB, 0.78494E+00_JPRB ,&
&  0.78253E+00_JPRB, 0.78047E+00_JPRB, 0.79042E+00_JPRB, 0.79531E+00_JPRB, 0.78462E+00_JPRB, 0.76357E+00_JPRB, 0.76105E+00_JPRB ,&
&  0.76702E+00_JPRB, 0.76275E+00_JPRB, 0.74459E+00_JPRB, 0.73230E+00_JPRB, 0.72243E+00_JPRB, 0.72776E+00_JPRB, 0.73453E+00_JPRB ,&
&  0.72866E+00_JPRB, 0.67730E+00_JPRB, 0.71512E+00_JPRB, 0.72602E+00_JPRB, 0.73297E+00_JPRB, 0.72144E+00_JPRB, 0.72598E+00_JPRB ,&
&  0.73701E+00_JPRB, 0.74369E+00_JPRB, 0.72349E+00_JPRB, 0.70010E+00_JPRB, 0.67924E+00_JPRB, 0.66200E+00_JPRB, 0.63435E+00_JPRB ,&
&  0.62391E+00_JPRB, 0.62082E+00_JPRB, 0.63416E+00_JPRB, 0.64835E+00_JPRB, 0.66475E+00_JPRB, 0.67146E+00_JPRB, 0.68192E+00_JPRB ,&
&  0.68417E+00_JPRB, 0.67872E+00_JPRB, 0.67373E+00_JPRB, 0.67000E+00_JPRB, 0.66945E+00_JPRB, 0.66373E+00_JPRB, 0.65526E+00_JPRB ,&
&  0.64858E+00_JPRB, 0.64740E+00_JPRB, 0.65255E+00_JPRB, 0.66239E+00_JPRB, 0.67530E+00_JPRB, 0.69681E+00_JPRB, 0.71455E+00_JPRB ,&
&  0.73569E+00_JPRB, 0.74363E+00_JPRB, 0.74998E+00_JPRB, 0.76303E+00_JPRB, 0.77053E+00_JPRB, 0.76808E+00_JPRB, 0.76426E+00_JPRB ,&
&  0.75940E+00_JPRB, 0.75165E+00_JPRB, 0.75444E+00_JPRB, 0.75180E+00_JPRB, 0.75087E+00_JPRB, 0.74141E+00_JPRB, 0.72565E+00_JPRB ,&
&  0.71973E+00_JPRB, 0.71554E+00_JPRB, 0.70743E+00_JPRB, 0.70504E+00_JPRB, 0.70305E+00_JPRB, 0.72137E+00_JPRB, 0.75337E+00_JPRB ,&
&  0.77079E+00_JPRB, 0.79400E+00_JPRB, 0.82710E+00_JPRB, 0.84832E+00_JPRB, 0.84707E+00_JPRB, 0.84465E+00_JPRB, 0.83172E+00_JPRB ,&
&  0.81183E+00_JPRB, 0.79792E+00_JPRB, 0.78539E+00_JPRB, 0.78086E+00_JPRB, 0.77693E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.71523E+00_JPRB, 0.73669E+00_JPRB, 0.70417E+00_JPRB, 0.68670E+00_JPRB, 0.66655E+00_JPRB, 0.66148E+00_JPRB, 0.65227E+00_JPRB ,&
&  0.66282E+00_JPRB, 0.67986E+00_JPRB, 0.70362E+00_JPRB, 0.71156E+00_JPRB, 0.71839E+00_JPRB, 0.72118E+00_JPRB, 0.73811E+00_JPRB ,&
&  0.73351E+00_JPRB, 0.72531E+00_JPRB, 0.72845E+00_JPRB, 0.71815E+00_JPRB, 0.70937E+00_JPRB, 0.71418E+00_JPRB, 0.72297E+00_JPRB ,&
&  0.73294E+00_JPRB, 0.72420E+00_JPRB, 0.70691E+00_JPRB, 0.68046E+00_JPRB, 0.66801E+00_JPRB, 0.66540E+00_JPRB, 0.67291E+00_JPRB ,&
&  0.67811E+00_JPRB, 0.70169E+00_JPRB, 0.63887E+00_JPRB, 0.69462E+00_JPRB, 0.70280E+00_JPRB, 0.70103E+00_JPRB, 0.69935E+00_JPRB ,&
&  0.70381E+00_JPRB, 0.69328E+00_JPRB, 0.68434E+00_JPRB, 0.67563E+00_JPRB, 0.65310E+00_JPRB, 0.63495E+00_JPRB, 0.61568E+00_JPRB ,&
&  0.59854E+00_JPRB, 0.58807E+00_JPRB, 0.59324E+00_JPRB, 0.61080E+00_JPRB, 0.62883E+00_JPRB, 0.63884E+00_JPRB, 0.64496E+00_JPRB ,&
&  0.64816E+00_JPRB, 0.64079E+00_JPRB, 0.63084E+00_JPRB, 0.63097E+00_JPRB, 0.63748E+00_JPRB, 0.64242E+00_JPRB, 0.64570E+00_JPRB ,&
&  0.64626E+00_JPRB, 0.62388E+00_JPRB, 0.61163E+00_JPRB, 0.62834E+00_JPRB, 0.64676E+00_JPRB, 0.66771E+00_JPRB, 0.69500E+00_JPRB ,&
&  0.71606E+00_JPRB, 0.72456E+00_JPRB, 0.72448E+00_JPRB, 0.73101E+00_JPRB, 0.73702E+00_JPRB, 0.74491E+00_JPRB, 0.75112E+00_JPRB ,&
&  0.73967E+00_JPRB, 0.73647E+00_JPRB, 0.73689E+00_JPRB, 0.73648E+00_JPRB, 0.72695E+00_JPRB, 0.70342E+00_JPRB, 0.67088E+00_JPRB ,&
&  0.65596E+00_JPRB, 0.64104E+00_JPRB, 0.63012E+00_JPRB, 0.62707E+00_JPRB, 0.62890E+00_JPRB, 0.65518E+00_JPRB, 0.69305E+00_JPRB ,&
&  0.73066E+00_JPRB, 0.76088E+00_JPRB, 0.78681E+00_JPRB, 0.81048E+00_JPRB, 0.80848E+00_JPRB, 0.78596E+00_JPRB, 0.77553E+00_JPRB ,&
&  0.75975E+00_JPRB, 0.74312E+00_JPRB, 0.73189E+00_JPRB, 0.72299E+00_JPRB, 0.71352E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.71814E+00_JPRB, 0.71960E+00_JPRB, 0.70504E+00_JPRB, 0.69497E+00_JPRB, 0.67836E+00_JPRB, 0.65646E+00_JPRB, 0.63730E+00_JPRB ,&
&  0.63883E+00_JPRB, 0.65407E+00_JPRB, 0.66878E+00_JPRB, 0.67943E+00_JPRB, 0.69485E+00_JPRB, 0.69795E+00_JPRB, 0.70439E+00_JPRB ,&
&  0.70017E+00_JPRB, 0.66313E+00_JPRB, 0.65731E+00_JPRB, 0.66497E+00_JPRB, 0.66682E+00_JPRB, 0.67281E+00_JPRB, 0.68353E+00_JPRB ,&
&  0.69338E+00_JPRB, 0.68862E+00_JPRB, 0.68047E+00_JPRB, 0.65097E+00_JPRB, 0.64429E+00_JPRB, 0.64917E+00_JPRB, 0.65403E+00_JPRB ,&
&  0.66403E+00_JPRB, 0.68024E+00_JPRB, 0.68365E+00_JPRB, 0.69155E+00_JPRB, 0.70029E+00_JPRB, 0.70441E+00_JPRB, 0.70093E+00_JPRB ,&
&  0.67498E+00_JPRB, 0.64899E+00_JPRB, 0.63825E+00_JPRB, 0.62090E+00_JPRB, 0.59919E+00_JPRB, 0.58395E+00_JPRB, 0.57619E+00_JPRB ,&
&  0.57191E+00_JPRB, 0.57693E+00_JPRB, 0.58311E+00_JPRB, 0.59707E+00_JPRB, 0.60229E+00_JPRB, 0.60639E+00_JPRB, 0.61160E+00_JPRB ,&
&  0.61106E+00_JPRB, 0.60689E+00_JPRB, 0.60701E+00_JPRB, 0.61691E+00_JPRB, 0.62294E+00_JPRB, 0.62075E+00_JPRB, 0.61641E+00_JPRB ,&
&  0.60394E+00_JPRB, 0.59102E+00_JPRB, 0.59896E+00_JPRB, 0.61442E+00_JPRB, 0.62192E+00_JPRB, 0.64833E+00_JPRB, 0.68241E+00_JPRB ,&
&  0.71144E+00_JPRB, 0.73013E+00_JPRB, 0.73001E+00_JPRB, 0.72579E+00_JPRB, 0.72334E+00_JPRB, 0.73019E+00_JPRB, 0.74209E+00_JPRB ,&
&  0.72587E+00_JPRB, 0.71659E+00_JPRB, 0.71457E+00_JPRB, 0.69526E+00_JPRB, 0.65867E+00_JPRB, 0.62688E+00_JPRB, 0.60355E+00_JPRB ,&
&  0.58501E+00_JPRB, 0.57451E+00_JPRB, 0.57816E+00_JPRB, 0.58153E+00_JPRB, 0.59418E+00_JPRB, 0.62992E+00_JPRB, 0.66738E+00_JPRB ,&
&  0.70808E+00_JPRB, 0.72897E+00_JPRB, 0.73561E+00_JPRB, 0.74359E+00_JPRB, 0.74903E+00_JPRB, 0.73753E+00_JPRB, 0.72787E+00_JPRB ,&
&  0.72476E+00_JPRB, 0.71620E+00_JPRB, 0.70683E+00_JPRB, 0.69727E+00_JPRB, 0.70480E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.71295E+00_JPRB, 0.70563E+00_JPRB, 0.69237E+00_JPRB, 0.66636E+00_JPRB, 0.64885E+00_JPRB, 0.62169E+00_JPRB, 0.60631E+00_JPRB ,&
&  0.60857E+00_JPRB, 0.60989E+00_JPRB, 0.61495E+00_JPRB, 0.62415E+00_JPRB, 0.62196E+00_JPRB, 0.61720E+00_JPRB, 0.61255E+00_JPRB ,&
&  0.61560E+00_JPRB, 0.61606E+00_JPRB, 0.62454E+00_JPRB, 0.64004E+00_JPRB, 0.65640E+00_JPRB, 0.66569E+00_JPRB, 0.67549E+00_JPRB ,&
&  0.68298E+00_JPRB, 0.67680E+00_JPRB, 0.67242E+00_JPRB, 0.64205E+00_JPRB, 0.62440E+00_JPRB, 0.62954E+00_JPRB, 0.63688E+00_JPRB ,&
&  0.64680E+00_JPRB, 0.65708E+00_JPRB, 0.66689E+00_JPRB, 0.67237E+00_JPRB, 0.65840E+00_JPRB, 0.65456E+00_JPRB, 0.64373E+00_JPRB ,&
&  0.60087E+00_JPRB, 0.55898E+00_JPRB, 0.53633E+00_JPRB, 0.52885E+00_JPRB, 0.52390E+00_JPRB, 0.52994E+00_JPRB, 0.53741E+00_JPRB ,&
&  0.55045E+00_JPRB, 0.56238E+00_JPRB, 0.56539E+00_JPRB, 0.56853E+00_JPRB, 0.56999E+00_JPRB, 0.56908E+00_JPRB, 0.56942E+00_JPRB ,&
&  0.56664E+00_JPRB, 0.56500E+00_JPRB, 0.57235E+00_JPRB, 0.57308E+00_JPRB, 0.56444E+00_JPRB, 0.56203E+00_JPRB, 0.56029E+00_JPRB ,&
&  0.55837E+00_JPRB, 0.55653E+00_JPRB, 0.55374E+00_JPRB, 0.56402E+00_JPRB, 0.58969E+00_JPRB, 0.61929E+00_JPRB, 0.64498E+00_JPRB ,&
&  0.65986E+00_JPRB, 0.67198E+00_JPRB, 0.67507E+00_JPRB, 0.67685E+00_JPRB, 0.67621E+00_JPRB, 0.66731E+00_JPRB, 0.66393E+00_JPRB ,&
&  0.65622E+00_JPRB, 0.63719E+00_JPRB, 0.61161E+00_JPRB, 0.57180E+00_JPRB, 0.53295E+00_JPRB, 0.51052E+00_JPRB, 0.50975E+00_JPRB ,&
&  0.52173E+00_JPRB, 0.53410E+00_JPRB, 0.54043E+00_JPRB, 0.54842E+00_JPRB, 0.56306E+00_JPRB, 0.58068E+00_JPRB, 0.59933E+00_JPRB ,&
&  0.61099E+00_JPRB, 0.61100E+00_JPRB, 0.60848E+00_JPRB, 0.61623E+00_JPRB, 0.62676E+00_JPRB, 0.64424E+00_JPRB, 0.66351E+00_JPRB ,&
&  0.68045E+00_JPRB, 0.68010E+00_JPRB, 0.68060E+00_JPRB, 0.68200E+00_JPRB, 0.70200E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.70688E+00_JPRB, 0.70585E+00_JPRB, 0.67780E+00_JPRB, 0.64644E+00_JPRB, 0.62776E+00_JPRB, 0.60693E+00_JPRB, 0.59010E+00_JPRB ,&
&  0.58850E+00_JPRB, 0.58566E+00_JPRB, 0.58327E+00_JPRB, 0.58035E+00_JPRB, 0.57830E+00_JPRB, 0.57501E+00_JPRB, 0.56285E+00_JPRB ,&
&  0.55842E+00_JPRB, 0.56496E+00_JPRB, 0.58731E+00_JPRB, 0.61249E+00_JPRB, 0.63277E+00_JPRB, 0.65915E+00_JPRB, 0.67038E+00_JPRB ,&
&  0.68217E+00_JPRB, 0.69468E+00_JPRB, 0.69696E+00_JPRB, 0.66421E+00_JPRB, 0.63726E+00_JPRB, 0.63840E+00_JPRB, 0.64952E+00_JPRB ,&
&  0.64984E+00_JPRB, 0.65285E+00_JPRB, 0.65614E+00_JPRB, 0.65230E+00_JPRB, 0.63130E+00_JPRB, 0.59299E+00_JPRB, 0.56279E+00_JPRB ,&
&  0.53776E+00_JPRB, 0.51835E+00_JPRB, 0.51235E+00_JPRB, 0.50095E+00_JPRB, 0.49927E+00_JPRB, 0.50938E+00_JPRB, 0.52271E+00_JPRB ,&
&  0.53651E+00_JPRB, 0.54273E+00_JPRB, 0.54969E+00_JPRB, 0.54616E+00_JPRB, 0.54178E+00_JPRB, 0.54448E+00_JPRB, 0.54426E+00_JPRB ,&
&  0.54357E+00_JPRB, 0.53444E+00_JPRB, 0.53181E+00_JPRB, 0.52499E+00_JPRB, 0.52117E+00_JPRB, 0.52179E+00_JPRB, 0.52501E+00_JPRB ,&
&  0.52655E+00_JPRB, 0.53239E+00_JPRB, 0.53428E+00_JPRB, 0.53658E+00_JPRB, 0.54606E+00_JPRB, 0.56263E+00_JPRB, 0.57924E+00_JPRB ,&
&  0.59169E+00_JPRB, 0.60152E+00_JPRB, 0.60919E+00_JPRB, 0.61504E+00_JPRB, 0.61492E+00_JPRB, 0.61037E+00_JPRB, 0.59662E+00_JPRB ,&
&  0.56627E+00_JPRB, 0.53361E+00_JPRB, 0.50872E+00_JPRB, 0.48877E+00_JPRB, 0.48494E+00_JPRB, 0.49560E+00_JPRB, 0.51299E+00_JPRB ,&
&  0.52617E+00_JPRB, 0.53001E+00_JPRB, 0.52387E+00_JPRB, 0.52201E+00_JPRB, 0.52679E+00_JPRB, 0.53619E+00_JPRB, 0.54351E+00_JPRB ,&
&  0.54270E+00_JPRB, 0.53951E+00_JPRB, 0.54333E+00_JPRB, 0.55027E+00_JPRB, 0.56419E+00_JPRB, 0.58174E+00_JPRB, 0.60070E+00_JPRB ,&
&  0.61958E+00_JPRB, 0.63432E+00_JPRB, 0.65348E+00_JPRB, 0.67295E+00_JPRB, 0.69850E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.68859E+00_JPRB, 0.66162E+00_JPRB, 0.63304E+00_JPRB, 0.60842E+00_JPRB, 0.59657E+00_JPRB, 0.57902E+00_JPRB, 0.56428E+00_JPRB ,&
&  0.56336E+00_JPRB, 0.57106E+00_JPRB, 0.56611E+00_JPRB, 0.55984E+00_JPRB, 0.55026E+00_JPRB, 0.54243E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.52931E+00_JPRB, 0.54085E+00_JPRB, 0.56652E+00_JPRB, 0.57933E+00_JPRB, 0.58558E+00_JPRB, 0.61616E+00_JPRB, 0.64758E+00_JPRB ,&
&  0.65684E+00_JPRB, 0.67086E+00_JPRB, 0.67511E+00_JPRB, 0.64995E+00_JPRB, 0.63352E+00_JPRB, 0.62717E+00_JPRB, 0.63099E+00_JPRB ,&
&  0.62621E+00_JPRB, 0.63030E+00_JPRB, 0.62211E+00_JPRB, 0.55210E+00_JPRB, 0.55101E+00_JPRB, 0.52034E+00_JPRB, 0.50969E+00_JPRB ,&
&  0.50426E+00_JPRB, 0.50941E+00_JPRB, 0.50555E+00_JPRB, 0.49648E+00_JPRB, 0.49293E+00_JPRB, 0.49624E+00_JPRB, 0.49775E+00_JPRB ,&
&  0.49338E+00_JPRB, 0.48281E+00_JPRB, 0.48731E+00_JPRB, 0.49373E+00_JPRB, 0.49747E+00_JPRB, 0.50459E+00_JPRB, 0.51513E+00_JPRB ,&
&  0.50490E+00_JPRB, 0.49569E+00_JPRB, 0.48714E+00_JPRB, 0.48754E+00_JPRB, 0.49427E+00_JPRB, 0.50883E+00_JPRB, 0.51397E+00_JPRB ,&
&  0.51209E+00_JPRB, 0.51106E+00_JPRB, 0.51227E+00_JPRB, 0.51866E+00_JPRB, 0.53346E+00_JPRB, 0.54826E+00_JPRB, 0.54950E+00_JPRB ,&
&  0.55063E+00_JPRB, 0.55524E+00_JPRB, 0.56017E+00_JPRB, 0.56198E+00_JPRB, 0.55801E+00_JPRB, 0.54288E+00_JPRB, 0.52533E+00_JPRB ,&
&  0.50871E+00_JPRB, 0.49178E+00_JPRB, 0.47744E+00_JPRB, 0.47289E+00_JPRB, 0.48606E+00_JPRB, 0.51531E+00_JPRB, 0.53162E+00_JPRB ,&
&  0.53551E+00_JPRB, 0.53753E+00_JPRB, 0.54086E+00_JPRB, 0.54183E+00_JPRB, 0.54372E+00_JPRB, 0.54170E+00_JPRB, 0.53506E+00_JPRB ,&
&  0.52376E+00_JPRB, 0.51262E+00_JPRB, 0.50992E+00_JPRB, 0.51418E+00_JPRB, 0.51928E+00_JPRB, 0.53127E+00_JPRB, 0.54770E+00_JPRB ,&
&  0.56947E+00_JPRB, 0.60747E+00_JPRB, 0.63955E+00_JPRB, 0.66451E+00_JPRB, 0.68275E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.66531E+00_JPRB, 0.63987E+00_JPRB, 0.62290E+00_JPRB, 0.60957E+00_JPRB, 0.59141E+00_JPRB, 0.57755E+00_JPRB, 0.57196E+00_JPRB ,&
&  0.58014E+00_JPRB, 0.57866E+00_JPRB, 0.56866E+00_JPRB, 0.56023E+00_JPRB, 0.54411E+00_JPRB, 0.52973E+00_JPRB, 0.52386E+00_JPRB ,&
&  0.53234E+00_JPRB, 0.54857E+00_JPRB, 0.56848E+00_JPRB, 0.57667E+00_JPRB, 0.57605E+00_JPRB, 0.58454E+00_JPRB, 0.60384E+00_JPRB ,&
&  0.63249E+00_JPRB, 0.65247E+00_JPRB, 0.65951E+00_JPRB, 0.64632E+00_JPRB, 0.62876E+00_JPRB, 0.62060E+00_JPRB, 0.60626E+00_JPRB ,&
&  0.58951E+00_JPRB, 0.59292E+00_JPRB, 0.52543E+00_JPRB, 0.51788E+00_JPRB, 0.49458E+00_JPRB, 0.48009E+00_JPRB, 0.47626E+00_JPRB ,&
&  0.48922E+00_JPRB, 0.49314E+00_JPRB, 0.48476E+00_JPRB, 0.47266E+00_JPRB, 0.46441E+00_JPRB, 0.45335E+00_JPRB, 0.44312E+00_JPRB ,&
&  0.43601E+00_JPRB, 0.43858E+00_JPRB, 0.44761E+00_JPRB, 0.45693E+00_JPRB, 0.46798E+00_JPRB, 0.47852E+00_JPRB, 0.47504E+00_JPRB ,&
&  0.46337E+00_JPRB, 0.46135E+00_JPRB, 0.46834E+00_JPRB, 0.48298E+00_JPRB, 0.49902E+00_JPRB, 0.50657E+00_JPRB, 0.51035E+00_JPRB ,&
&  0.50254E+00_JPRB, 0.48575E+00_JPRB, 0.47132E+00_JPRB, 0.45943E+00_JPRB, 0.43368E+00_JPRB, 0.37047E+00_JPRB, 0.34767E+00_JPRB ,&
&  0.35849E+00_JPRB, 0.41723E+00_JPRB, 0.48756E+00_JPRB, 0.47923E+00_JPRB, 0.47479E+00_JPRB, 0.46053E+00_JPRB, 0.42424E+00_JPRB ,&
&  0.38253E+00_JPRB, 0.32104E+00_JPRB, 0.29955E+00_JPRB, 0.29597E+00_JPRB, 0.29510E+00_JPRB, 0.30076E+00_JPRB, 0.31295E+00_JPRB ,&
&  0.32903E+00_JPRB, 0.35584E+00_JPRB, 0.37322E+00_JPRB, 0.37980E+00_JPRB, 0.37819E+00_JPRB, 0.36204E+00_JPRB, 0.34813E+00_JPRB ,&
&  0.33889E+00_JPRB, 0.36370E+00_JPRB, 0.42226E+00_JPRB, 0.50494E+00_JPRB, 0.50408E+00_JPRB, 0.52870E+00_JPRB, 0.55350E+00_JPRB ,&
&  0.57710E+00_JPRB, 0.61671E+00_JPRB, 0.64276E+00_JPRB, 0.64852E+00_JPRB, 0.66695E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.62532E+00_JPRB, 0.61364E+00_JPRB, 0.60577E+00_JPRB, 0.60025E+00_JPRB, 0.59359E+00_JPRB, 0.59247E+00_JPRB, 0.59184E+00_JPRB ,&
&  0.58602E+00_JPRB, 0.57749E+00_JPRB, 0.57816E+00_JPRB, 0.56072E+00_JPRB, 0.53601E+00_JPRB, 0.51947E+00_JPRB, 0.51716E+00_JPRB ,&
&  0.52104E+00_JPRB, 0.52819E+00_JPRB, 0.53731E+00_JPRB, 0.54607E+00_JPRB, 0.55108E+00_JPRB, 0.56288E+00_JPRB, 0.58388E+00_JPRB ,&
&  0.60833E+00_JPRB, 0.62207E+00_JPRB, 0.62627E+00_JPRB, 0.61827E+00_JPRB, 0.60592E+00_JPRB, 0.59313E+00_JPRB, 0.58223E+00_JPRB ,&
&  0.57370E+00_JPRB, 0.47191E+00_JPRB, 0.40383E+00_JPRB, 0.41028E+00_JPRB, 0.47977E+00_JPRB, 0.46368E+00_JPRB, 0.46165E+00_JPRB ,&
&  0.47400E+00_JPRB, 0.47416E+00_JPRB, 0.46094E+00_JPRB, 0.44392E+00_JPRB, 0.42802E+00_JPRB, 0.41657E+00_JPRB, 0.40820E+00_JPRB ,&
&  0.40432E+00_JPRB, 0.41095E+00_JPRB, 0.41905E+00_JPRB, 0.43165E+00_JPRB, 0.43486E+00_JPRB, 0.42219E+00_JPRB, 0.40696E+00_JPRB ,&
&  0.39227E+00_JPRB, 0.38705E+00_JPRB, 0.38941E+00_JPRB, 0.39583E+00_JPRB, 0.40429E+00_JPRB, 0.41092E+00_JPRB, 0.40898E+00_JPRB ,&
&  0.38014E+00_JPRB, 0.35718E+00_JPRB, 0.32904E+00_JPRB, 0.30252E+00_JPRB, 0.27942E+00_JPRB, 0.26390E+00_JPRB, 0.25599E+00_JPRB ,&
&  0.25409E+00_JPRB, 0.26142E+00_JPRB, 0.28281E+00_JPRB, 0.33185E+00_JPRB, 0.39928E+00_JPRB, 0.32033E+00_JPRB, 0.28116E+00_JPRB ,&
&  0.25954E+00_JPRB, 0.24646E+00_JPRB, 0.24150E+00_JPRB, 0.24218E+00_JPRB, 0.24621E+00_JPRB, 0.25223E+00_JPRB, 0.25695E+00_JPRB ,&
&  0.26070E+00_JPRB, 0.26410E+00_JPRB, 0.26604E+00_JPRB, 0.26698E+00_JPRB, 0.26848E+00_JPRB, 0.26958E+00_JPRB, 0.26849E+00_JPRB ,&
&  0.26695E+00_JPRB, 0.27301E+00_JPRB, 0.29038E+00_JPRB, 0.31786E+00_JPRB, 0.35030E+00_JPRB, 0.38539E+00_JPRB, 0.43733E+00_JPRB ,&
&  0.50282E+00_JPRB, 0.57788E+00_JPRB, 0.59574E+00_JPRB, 0.60835E+00_JPRB, 0.61881E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.56805E+00_JPRB, 0.57185E+00_JPRB, 0.57617E+00_JPRB, 0.57864E+00_JPRB, 0.57329E+00_JPRB, 0.56605E+00_JPRB, 0.56140E+00_JPRB ,&
&  0.56649E+00_JPRB, 0.56633E+00_JPRB, 0.54280E+00_JPRB, 0.52437E+00_JPRB, 0.51375E+00_JPRB, 0.51010E+00_JPRB, 0.50697E+00_JPRB ,&
&  0.50079E+00_JPRB, 0.49865E+00_JPRB, 0.49989E+00_JPRB, 0.51119E+00_JPRB, 0.52059E+00_JPRB, 0.52880E+00_JPRB, 0.53623E+00_JPRB ,&
&  0.53171E+00_JPRB, 0.54449E+00_JPRB, 0.56614E+00_JPRB, 0.56542E+00_JPRB, 0.56276E+00_JPRB, 0.55565E+00_JPRB, 0.54129E+00_JPRB ,&
&  0.49335E+00_JPRB, 0.42939E+00_JPRB, 0.36795E+00_JPRB, 0.37937E+00_JPRB, 0.45671E+00_JPRB, 0.46598E+00_JPRB, 0.45050E+00_JPRB ,&
&  0.46088E+00_JPRB, 0.45672E+00_JPRB, 0.43887E+00_JPRB, 0.42572E+00_JPRB, 0.41028E+00_JPRB, 0.40020E+00_JPRB, 0.39314E+00_JPRB ,&
&  0.39026E+00_JPRB, 0.38904E+00_JPRB, 0.37425E+00_JPRB, 0.32586E+00_JPRB, 0.29752E+00_JPRB, 0.28021E+00_JPRB, 0.26250E+00_JPRB ,&
&  0.24858E+00_JPRB, 0.23967E+00_JPRB, 0.23805E+00_JPRB, 0.23826E+00_JPRB, 0.24291E+00_JPRB, 0.24490E+00_JPRB, 0.24677E+00_JPRB ,&
&  0.24654E+00_JPRB, 0.24541E+00_JPRB, 0.24134E+00_JPRB, 0.23679E+00_JPRB, 0.23362E+00_JPRB, 0.23312E+00_JPRB, 0.23284E+00_JPRB ,&
&  0.23547E+00_JPRB, 0.23948E+00_JPRB, 0.25599E+00_JPRB, 0.27372E+00_JPRB, 0.27284E+00_JPRB, 0.25764E+00_JPRB, 0.24282E+00_JPRB ,&
&  0.23304E+00_JPRB, 0.22810E+00_JPRB, 0.22492E+00_JPRB, 0.22436E+00_JPRB, 0.22718E+00_JPRB, 0.23119E+00_JPRB, 0.23227E+00_JPRB ,&
&  0.23214E+00_JPRB, 0.23281E+00_JPRB, 0.23290E+00_JPRB, 0.23339E+00_JPRB, 0.23407E+00_JPRB, 0.23660E+00_JPRB, 0.24137E+00_JPRB ,&
&  0.24700E+00_JPRB, 0.25713E+00_JPRB, 0.26865E+00_JPRB, 0.28413E+00_JPRB, 0.29548E+00_JPRB, 0.30380E+00_JPRB, 0.31454E+00_JPRB ,&
&  0.33208E+00_JPRB, 0.37038E+00_JPRB, 0.43931E+00_JPRB, 0.56543E+00_JPRB, 0.56036E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.53652E+00_JPRB, 0.54806E+00_JPRB, 0.56060E+00_JPRB, 0.55260E+00_JPRB, 0.53900E+00_JPRB, 0.53136E+00_JPRB, 0.53109E+00_JPRB ,&
&  0.53067E+00_JPRB, 0.52772E+00_JPRB, 0.51547E+00_JPRB, 0.50279E+00_JPRB, 0.48478E+00_JPRB, 0.43657E+00_JPRB, 0.41826E+00_JPRB ,&
&  0.40071E+00_JPRB, 0.40323E+00_JPRB, 0.40785E+00_JPRB, 0.41453E+00_JPRB, 0.42864E+00_JPRB, 0.43959E+00_JPRB, 0.44173E+00_JPRB ,&
&  0.43188E+00_JPRB, 0.41364E+00_JPRB, 0.39615E+00_JPRB, 0.38254E+00_JPRB, 0.38220E+00_JPRB, 0.38235E+00_JPRB, 0.37550E+00_JPRB ,&
&  0.36198E+00_JPRB, 0.35588E+00_JPRB, 0.35560E+00_JPRB, 0.37556E+00_JPRB, 0.42805E+00_JPRB, 0.44759E+00_JPRB, 0.42237E+00_JPRB ,&
&  0.42676E+00_JPRB, 0.42200E+00_JPRB, 0.41805E+00_JPRB, 0.41950E+00_JPRB, 0.40950E+00_JPRB, 0.39057E+00_JPRB, 0.35678E+00_JPRB ,&
&  0.31286E+00_JPRB, 0.29008E+00_JPRB, 0.26894E+00_JPRB, 0.24918E+00_JPRB, 0.23412E+00_JPRB, 0.22529E+00_JPRB, 0.21771E+00_JPRB ,&
&  0.21275E+00_JPRB, 0.20860E+00_JPRB, 0.20583E+00_JPRB, 0.20560E+00_JPRB, 0.20693E+00_JPRB, 0.20757E+00_JPRB, 0.20841E+00_JPRB ,&
&  0.20972E+00_JPRB, 0.21038E+00_JPRB, 0.21071E+00_JPRB, 0.21186E+00_JPRB, 0.21322E+00_JPRB, 0.21576E+00_JPRB, 0.21933E+00_JPRB ,&
&  0.22375E+00_JPRB, 0.22750E+00_JPRB, 0.23115E+00_JPRB, 0.23721E+00_JPRB, 0.23775E+00_JPRB, 0.23659E+00_JPRB, 0.22882E+00_JPRB ,&
&  0.22320E+00_JPRB, 0.22068E+00_JPRB, 0.21811E+00_JPRB, 0.21740E+00_JPRB, 0.21847E+00_JPRB, 0.22035E+00_JPRB, 0.22223E+00_JPRB ,&
&  0.22352E+00_JPRB, 0.22357E+00_JPRB, 0.22277E+00_JPRB, 0.22270E+00_JPRB, 0.22329E+00_JPRB, 0.22346E+00_JPRB, 0.22699E+00_JPRB ,&
&  0.23465E+00_JPRB, 0.24595E+00_JPRB, 0.25851E+00_JPRB, 0.27583E+00_JPRB, 0.29201E+00_JPRB, 0.30844E+00_JPRB, 0.33594E+00_JPRB ,&
&  0.37548E+00_JPRB, 0.48375E+00_JPRB, 0.55336E+00_JPRB, 0.49181E+00_JPRB, 0.51593E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.50854E+00_JPRB, 0.51616E+00_JPRB, 0.52731E+00_JPRB, 0.53207E+00_JPRB, 0.54026E+00_JPRB, 0.52867E+00_JPRB, 0.52167E+00_JPRB ,&
&  0.49145E+00_JPRB, 0.46920E+00_JPRB, 0.43811E+00_JPRB, 0.39921E+00_JPRB, 0.37307E+00_JPRB, 0.35283E+00_JPRB, 0.34569E+00_JPRB ,&
&  0.33787E+00_JPRB, 0.33293E+00_JPRB, 0.32814E+00_JPRB, 0.32625E+00_JPRB, 0.32508E+00_JPRB, 0.32619E+00_JPRB, 0.32599E+00_JPRB ,&
&  0.32238E+00_JPRB, 0.31676E+00_JPRB, 0.31014E+00_JPRB, 0.31311E+00_JPRB, 0.32604E+00_JPRB, 0.34101E+00_JPRB, 0.35860E+00_JPRB ,&
&  0.37887E+00_JPRB, 0.38967E+00_JPRB, 0.39989E+00_JPRB, 0.40400E+00_JPRB, 0.38860E+00_JPRB, 0.39115E+00_JPRB, 0.39401E+00_JPRB ,&
&  0.40204E+00_JPRB, 0.40899E+00_JPRB, 0.40611E+00_JPRB, 0.40187E+00_JPRB, 0.35112E+00_JPRB, 0.31801E+00_JPRB, 0.28940E+00_JPRB ,&
&  0.26794E+00_JPRB, 0.25412E+00_JPRB, 0.24259E+00_JPRB, 0.23279E+00_JPRB, 0.22297E+00_JPRB, 0.21661E+00_JPRB, 0.21181E+00_JPRB ,&
&  0.20792E+00_JPRB, 0.20615E+00_JPRB, 0.20446E+00_JPRB, 0.20391E+00_JPRB, 0.20421E+00_JPRB, 0.20358E+00_JPRB, 0.20345E+00_JPRB ,&
&  0.20315E+00_JPRB, 0.20238E+00_JPRB, 0.20232E+00_JPRB, 0.20341E+00_JPRB, 0.20529E+00_JPRB, 0.20843E+00_JPRB, 0.20953E+00_JPRB ,&
&  0.21045E+00_JPRB, 0.21104E+00_JPRB, 0.21108E+00_JPRB, 0.21293E+00_JPRB, 0.21391E+00_JPRB, 0.21408E+00_JPRB, 0.21249E+00_JPRB ,&
&  0.21085E+00_JPRB, 0.20962E+00_JPRB, 0.20946E+00_JPRB, 0.20993E+00_JPRB, 0.21132E+00_JPRB, 0.21352E+00_JPRB, 0.21403E+00_JPRB ,&
&  0.21491E+00_JPRB, 0.21456E+00_JPRB, 0.21423E+00_JPRB, 0.21601E+00_JPRB, 0.21811E+00_JPRB, 0.22043E+00_JPRB, 0.22358E+00_JPRB ,&
&  0.22537E+00_JPRB, 0.23236E+00_JPRB, 0.24517E+00_JPRB, 0.26484E+00_JPRB, 0.29509E+00_JPRB, 0.31155E+00_JPRB, 0.35074E+00_JPRB ,&
&  0.41302E+00_JPRB, 0.55648E+00_JPRB, 0.62085E+00_JPRB, 0.47810E+00_JPRB, 0.49315E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.45074E+00_JPRB, 0.46300E+00_JPRB, 0.48024E+00_JPRB, 0.49096E+00_JPRB, 0.50513E+00_JPRB, 0.50783E+00_JPRB, 0.50234E+00_JPRB ,&
&  0.48370E+00_JPRB, 0.45417E+00_JPRB, 0.43755E+00_JPRB, 0.41695E+00_JPRB, 0.39568E+00_JPRB, 0.37424E+00_JPRB, 0.36470E+00_JPRB ,&
&  0.35318E+00_JPRB, 0.34022E+00_JPRB, 0.32699E+00_JPRB, 0.31559E+00_JPRB, 0.30467E+00_JPRB, 0.29452E+00_JPRB, 0.28863E+00_JPRB ,&
&  0.28257E+00_JPRB, 0.28197E+00_JPRB, 0.28136E+00_JPRB, 0.28552E+00_JPRB, 0.29678E+00_JPRB, 0.31571E+00_JPRB, 0.34162E+00_JPRB ,&
&  0.38053E+00_JPRB, 0.38621E+00_JPRB, 0.37141E+00_JPRB, 0.36508E+00_JPRB, 0.35776E+00_JPRB, 0.35143E+00_JPRB, 0.33808E+00_JPRB ,&
&  0.34544E+00_JPRB, 0.35755E+00_JPRB, 0.37315E+00_JPRB, 0.36218E+00_JPRB, 0.31676E+00_JPRB, 0.29655E+00_JPRB, 0.27296E+00_JPRB ,&
&  0.26062E+00_JPRB, 0.25040E+00_JPRB, 0.24232E+00_JPRB, 0.23460E+00_JPRB, 0.22667E+00_JPRB, 0.22177E+00_JPRB, 0.21641E+00_JPRB ,&
&  0.21257E+00_JPRB, 0.21096E+00_JPRB, 0.20855E+00_JPRB, 0.20850E+00_JPRB, 0.20832E+00_JPRB, 0.20756E+00_JPRB, 0.20682E+00_JPRB ,&
&  0.20557E+00_JPRB, 0.20421E+00_JPRB, 0.20357E+00_JPRB, 0.20408E+00_JPRB, 0.20491E+00_JPRB, 0.20574E+00_JPRB, 0.20381E+00_JPRB ,&
&  0.20254E+00_JPRB, 0.20129E+00_JPRB, 0.20113E+00_JPRB, 0.20231E+00_JPRB, 0.20267E+00_JPRB, 0.20346E+00_JPRB, 0.20360E+00_JPRB ,&
&  0.20370E+00_JPRB, 0.20427E+00_JPRB, 0.20398E+00_JPRB, 0.20471E+00_JPRB, 0.20503E+00_JPRB, 0.20619E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20788E+00_JPRB, 0.20988E+00_JPRB, 0.21200E+00_JPRB, 0.21502E+00_JPRB, 0.21915E+00_JPRB, 0.22195E+00_JPRB, 0.22642E+00_JPRB ,&
&  0.23178E+00_JPRB, 0.24056E+00_JPRB, 0.25716E+00_JPRB, 0.27580E+00_JPRB, 0.29891E+00_JPRB, 0.31038E+00_JPRB, 0.31891E+00_JPRB ,&
&  0.33228E+00_JPRB, 0.36596E+00_JPRB, 0.41668E+00_JPRB, 0.43809E+00_JPRB, 0.44347E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.44855E+00_JPRB, 0.44996E+00_JPRB, 0.45085E+00_JPRB, 0.44763E+00_JPRB, 0.44657E+00_JPRB, 0.44660E+00_JPRB, 0.44386E+00_JPRB ,&
&  0.43591E+00_JPRB, 0.43352E+00_JPRB, 0.42857E+00_JPRB, 0.42135E+00_JPRB, 0.41425E+00_JPRB, 0.40276E+00_JPRB, 0.38641E+00_JPRB ,&
&  0.36450E+00_JPRB, 0.34518E+00_JPRB, 0.33065E+00_JPRB, 0.31719E+00_JPRB, 0.30402E+00_JPRB, 0.29116E+00_JPRB, 0.28603E+00_JPRB ,&
&  0.28152E+00_JPRB, 0.27964E+00_JPRB, 0.27912E+00_JPRB, 0.28261E+00_JPRB, 0.29019E+00_JPRB, 0.29966E+00_JPRB, 0.31003E+00_JPRB ,&
&  0.32177E+00_JPRB, 0.33274E+00_JPRB, 0.33407E+00_JPRB, 0.32682E+00_JPRB, 0.31993E+00_JPRB, 0.31561E+00_JPRB, 0.31227E+00_JPRB ,&
&  0.30834E+00_JPRB, 0.30004E+00_JPRB, 0.29058E+00_JPRB, 0.28064E+00_JPRB, 0.27060E+00_JPRB, 0.26328E+00_JPRB, 0.25470E+00_JPRB ,&
&  0.24841E+00_JPRB, 0.24265E+00_JPRB, 0.23653E+00_JPRB, 0.23127E+00_JPRB, 0.22768E+00_JPRB, 0.22471E+00_JPRB, 0.22171E+00_JPRB ,&
&  0.22020E+00_JPRB, 0.21872E+00_JPRB, 0.21706E+00_JPRB, 0.21545E+00_JPRB, 0.21343E+00_JPRB, 0.21189E+00_JPRB, 0.21049E+00_JPRB ,&
&  0.20942E+00_JPRB, 0.20848E+00_JPRB, 0.20790E+00_JPRB, 0.20731E+00_JPRB, 0.20681E+00_JPRB, 0.20581E+00_JPRB, 0.20464E+00_JPRB ,&
&  0.20389E+00_JPRB, 0.20358E+00_JPRB, 0.20280E+00_JPRB, 0.20237E+00_JPRB, 0.20203E+00_JPRB, 0.20129E+00_JPRB, 0.20175E+00_JPRB ,&
&  0.20291E+00_JPRB, 0.20421E+00_JPRB, 0.20450E+00_JPRB, 0.20558E+00_JPRB, 0.20688E+00_JPRB, 0.20851E+00_JPRB, 0.21106E+00_JPRB ,&
&  0.21300E+00_JPRB, 0.21503E+00_JPRB, 0.21756E+00_JPRB, 0.21979E+00_JPRB, 0.22323E+00_JPRB, 0.22748E+00_JPRB, 0.23271E+00_JPRB ,&
&  0.23985E+00_JPRB, 0.24846E+00_JPRB, 0.26056E+00_JPRB, 0.27535E+00_JPRB, 0.29650E+00_JPRB, 0.30472E+00_JPRB, 0.31271E+00_JPRB ,&
&  0.31938E+00_JPRB, 0.33256E+00_JPRB, 0.41502E+00_JPRB, 0.42988E+00_JPRB, 0.44482E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.32274E+00_JPRB, 0.33303E+00_JPRB, 0.34165E+00_JPRB, 0.34988E+00_JPRB, 0.35163E+00_JPRB, 0.41644E+00_JPRB, 0.39662E+00_JPRB ,&
&  0.38705E+00_JPRB, 0.35924E+00_JPRB, 0.36042E+00_JPRB, 0.35619E+00_JPRB, 0.34977E+00_JPRB, 0.34254E+00_JPRB, 0.33639E+00_JPRB ,&
&  0.33027E+00_JPRB, 0.32255E+00_JPRB, 0.31775E+00_JPRB, 0.31179E+00_JPRB, 0.30610E+00_JPRB, 0.30040E+00_JPRB, 0.29700E+00_JPRB ,&
&  0.29447E+00_JPRB, 0.29302E+00_JPRB, 0.29193E+00_JPRB, 0.29084E+00_JPRB, 0.29325E+00_JPRB, 0.29447E+00_JPRB, 0.29259E+00_JPRB ,&
&  0.28887E+00_JPRB, 0.28536E+00_JPRB, 0.28101E+00_JPRB, 0.27658E+00_JPRB, 0.27364E+00_JPRB, 0.27098E+00_JPRB, 0.26821E+00_JPRB ,&
&  0.26453E+00_JPRB, 0.26044E+00_JPRB, 0.25625E+00_JPRB, 0.25293E+00_JPRB, 0.24976E+00_JPRB, 0.24650E+00_JPRB, 0.24345E+00_JPRB ,&
&  0.24027E+00_JPRB, 0.23713E+00_JPRB, 0.23398E+00_JPRB, 0.23191E+00_JPRB, 0.23019E+00_JPRB, 0.22859E+00_JPRB, 0.22714E+00_JPRB ,&
&  0.22653E+00_JPRB, 0.22580E+00_JPRB, 0.22489E+00_JPRB, 0.22375E+00_JPRB, 0.22269E+00_JPRB, 0.22153E+00_JPRB, 0.22059E+00_JPRB ,&
&  0.22044E+00_JPRB, 0.21985E+00_JPRB, 0.21923E+00_JPRB, 0.21844E+00_JPRB, 0.21806E+00_JPRB, 0.21725E+00_JPRB, 0.21624E+00_JPRB ,&
&  0.21533E+00_JPRB, 0.21449E+00_JPRB, 0.21429E+00_JPRB, 0.21402E+00_JPRB, 0.21375E+00_JPRB, 0.21353E+00_JPRB, 0.21392E+00_JPRB ,&
&  0.21441E+00_JPRB, 0.21501E+00_JPRB, 0.21600E+00_JPRB, 0.21720E+00_JPRB, 0.21826E+00_JPRB, 0.21930E+00_JPRB, 0.22076E+00_JPRB ,&
&  0.22271E+00_JPRB, 0.22467E+00_JPRB, 0.22686E+00_JPRB, 0.22888E+00_JPRB, 0.23230E+00_JPRB, 0.23598E+00_JPRB, 0.24004E+00_JPRB ,&
&  0.24501E+00_JPRB, 0.25182E+00_JPRB, 0.25967E+00_JPRB, 0.26674E+00_JPRB, 0.27372E+00_JPRB, 0.28127E+00_JPRB, 0.28749E+00_JPRB ,&
&  0.29292E+00_JPRB, 0.29824E+00_JPRB, 0.30378E+00_JPRB, 0.30985E+00_JPRB, 0.31646E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.28043E+00_JPRB, 0.28315E+00_JPRB, 0.28546E+00_JPRB, 0.28711E+00_JPRB, 0.28733E+00_JPRB, 0.28900E+00_JPRB, 0.28936E+00_JPRB ,&
&  0.28952E+00_JPRB, 0.28939E+00_JPRB, 0.29015E+00_JPRB, 0.29097E+00_JPRB, 0.29151E+00_JPRB, 0.29180E+00_JPRB, 0.29184E+00_JPRB ,&
&  0.29135E+00_JPRB, 0.29038E+00_JPRB, 0.28656E+00_JPRB, 0.28444E+00_JPRB, 0.28319E+00_JPRB, 0.28262E+00_JPRB, 0.28232E+00_JPRB ,&
&  0.28156E+00_JPRB, 0.28065E+00_JPRB, 0.27916E+00_JPRB, 0.27893E+00_JPRB, 0.27811E+00_JPRB, 0.27704E+00_JPRB, 0.27553E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27288E+00_JPRB, 0.27144E+00_JPRB, 0.26996E+00_JPRB, 0.26815E+00_JPRB, 0.26595E+00_JPRB, 0.26319E+00_JPRB ,&
&  0.26035E+00_JPRB, 0.25744E+00_JPRB, 0.25502E+00_JPRB, 0.25279E+00_JPRB, 0.25085E+00_JPRB, 0.24930E+00_JPRB, 0.24818E+00_JPRB ,&
&  0.24695E+00_JPRB, 0.24555E+00_JPRB, 0.24413E+00_JPRB, 0.24293E+00_JPRB, 0.24176E+00_JPRB, 0.24074E+00_JPRB, 0.24022E+00_JPRB ,&
&  0.23992E+00_JPRB, 0.23956E+00_JPRB, 0.23911E+00_JPRB, 0.23872E+00_JPRB, 0.23821E+00_JPRB, 0.23775E+00_JPRB, 0.23728E+00_JPRB ,&
&  0.23689E+00_JPRB, 0.23665E+00_JPRB, 0.23628E+00_JPRB, 0.23590E+00_JPRB, 0.23566E+00_JPRB, 0.23526E+00_JPRB, 0.23481E+00_JPRB ,&
&  0.23429E+00_JPRB, 0.23399E+00_JPRB, 0.23366E+00_JPRB, 0.23324E+00_JPRB, 0.23286E+00_JPRB, 0.23259E+00_JPRB, 0.23259E+00_JPRB ,&
&  0.23255E+00_JPRB, 0.23244E+00_JPRB, 0.23250E+00_JPRB, 0.23323E+00_JPRB, 0.23401E+00_JPRB, 0.23494E+00_JPRB, 0.23599E+00_JPRB ,&
&  0.23737E+00_JPRB, 0.23881E+00_JPRB, 0.24042E+00_JPRB, 0.24196E+00_JPRB, 0.24402E+00_JPRB, 0.24606E+00_JPRB, 0.24810E+00_JPRB ,&
&  0.25007E+00_JPRB, 0.25209E+00_JPRB, 0.25413E+00_JPRB, 0.25621E+00_JPRB, 0.25892E+00_JPRB, 0.26123E+00_JPRB, 0.26384E+00_JPRB ,&
&  0.26632E+00_JPRB, 0.26906E+00_JPRB, 0.27187E+00_JPRB, 0.27433E+00_JPRB, 0.27685E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB, 0.25522E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB, 0.31098E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.31748E+00_JPRB, 0.31757E+00_JPRB, 0.31768E+00_JPRB, 0.31787E+00_JPRB, 0.31791E+00_JPRB, 0.31783E+00_JPRB, 0.31762E+00_JPRB ,&
&  0.31737E+00_JPRB, 0.31706E+00_JPRB, 0.31652E+00_JPRB, 0.31599E+00_JPRB, 0.31549E+00_JPRB, 0.31499E+00_JPRB, 0.31420E+00_JPRB ,&
&  0.31332E+00_JPRB, 0.31230E+00_JPRB, 0.31129E+00_JPRB, 0.31046E+00_JPRB, 0.31027E+00_JPRB, 0.31018E+00_JPRB, 0.31009E+00_JPRB ,&
&  0.30995E+00_JPRB, 0.30992E+00_JPRB, 0.30990E+00_JPRB, 0.30987E+00_JPRB, 0.30987E+00_JPRB, 0.30974E+00_JPRB, 0.30962E+00_JPRB ,&
&  0.30945E+00_JPRB, 0.30932E+00_JPRB, 0.30908E+00_JPRB, 0.30890E+00_JPRB, 0.30864E+00_JPRB, 0.30861E+00_JPRB, 0.30848E+00_JPRB ,&
&  0.30845E+00_JPRB, 0.30833E+00_JPRB, 0.30827E+00_JPRB, 0.30818E+00_JPRB, 0.30815E+00_JPRB, 0.30791E+00_JPRB, 0.30767E+00_JPRB ,&
&  0.30740E+00_JPRB, 0.30727E+00_JPRB, 0.30748E+00_JPRB, 0.30792E+00_JPRB, 0.30836E+00_JPRB, 0.30887E+00_JPRB, 0.30931E+00_JPRB ,&
&  0.30975E+00_JPRB, 0.31008E+00_JPRB, 0.31061E+00_JPRB, 0.31111E+00_JPRB, 0.31168E+00_JPRB, 0.31215E+00_JPRB, 0.31271E+00_JPRB ,&
&  0.31323E+00_JPRB, 0.31400E+00_JPRB, 0.31500E+00_JPRB, 0.31613E+00_JPRB, 0.31720E+00_JPRB, 0.31818E+00_JPRB, 0.31924E+00_JPRB ,&
&  0.32036E+00_JPRB, 0.32142E+00_JPRB, 0.32221E+00_JPRB, 0.32309E+00_JPRB, 0.32399E+00_JPRB, 0.32489E+00_JPRB, 0.32552E+00_JPRB ,&
&  0.32619E+00_JPRB, 0.32692E+00_JPRB, 0.32772E+00_JPRB, 0.32794E+00_JPRB, 0.32808E+00_JPRB, 0.32812E+00_JPRB, 0.32797E+00_JPRB ,&
&  0.32740E+00_JPRB, 0.32681E+00_JPRB, 0.32623E+00_JPRB, 0.32565E+00_JPRB, 0.32484E+00_JPRB, 0.32392E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32180E+00_JPRB, 0.32052E+00_JPRB, 0.31934E+00_JPRB, 0.31854E+00_JPRB, 0.31814E+00_JPRB, 0.31769E+00_JPRB, 0.31729E+00_JPRB ,&
&  0.31699E+00_JPRB, 0.31704E+00_JPRB, 0.31716E+00_JPRB, 0.31729E+00_JPRB, 0.31745E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.32237E+00_JPRB, 0.32262E+00_JPRB, 0.32306E+00_JPRB, 0.32338E+00_JPRB, 0.32314E+00_JPRB, 0.32298E+00_JPRB, 0.32282E+00_JPRB ,&
&  0.32295E+00_JPRB, 0.32311E+00_JPRB, 0.32276E+00_JPRB, 0.32209E+00_JPRB, 0.32141E+00_JPRB, 0.32070E+00_JPRB, 0.31986E+00_JPRB ,&
&  0.31913E+00_JPRB, 0.31834E+00_JPRB, 0.31728E+00_JPRB, 0.31523E+00_JPRB, 0.31376E+00_JPRB, 0.31311E+00_JPRB, 0.31345E+00_JPRB ,&
&  0.31339E+00_JPRB, 0.31290E+00_JPRB, 0.31174E+00_JPRB, 0.31042E+00_JPRB, 0.30889E+00_JPRB, 0.30736E+00_JPRB, 0.30571E+00_JPRB ,&
&  0.30424E+00_JPRB, 0.30282E+00_JPRB, 0.30176E+00_JPRB, 0.30076E+00_JPRB, 0.30087E+00_JPRB, 0.30094E+00_JPRB, 0.30102E+00_JPRB ,&
&  0.30107E+00_JPRB, 0.30091E+00_JPRB, 0.30130E+00_JPRB, 0.30172E+00_JPRB, 0.30218E+00_JPRB, 0.30281E+00_JPRB, 0.30289E+00_JPRB ,&
&  0.30242E+00_JPRB, 0.30210E+00_JPRB, 0.30310E+00_JPRB, 0.30359E+00_JPRB, 0.30384E+00_JPRB, 0.30390E+00_JPRB, 0.30495E+00_JPRB ,&
&  0.30595E+00_JPRB, 0.30681E+00_JPRB, 0.30784E+00_JPRB, 0.30961E+00_JPRB, 0.31095E+00_JPRB, 0.31238E+00_JPRB, 0.31407E+00_JPRB ,&
&  0.31590E+00_JPRB, 0.31794E+00_JPRB, 0.32014E+00_JPRB, 0.32237E+00_JPRB, 0.32437E+00_JPRB, 0.32610E+00_JPRB, 0.32815E+00_JPRB ,&
&  0.33034E+00_JPRB, 0.33288E+00_JPRB, 0.33441E+00_JPRB, 0.33605E+00_JPRB, 0.33788E+00_JPRB, 0.34058E+00_JPRB, 0.34154E+00_JPRB ,&
&  0.34238E+00_JPRB, 0.34314E+00_JPRB, 0.34402E+00_JPRB, 0.34437E+00_JPRB, 0.34469E+00_JPRB, 0.34501E+00_JPRB, 0.34521E+00_JPRB ,&
&  0.34506E+00_JPRB, 0.34441E+00_JPRB, 0.34331E+00_JPRB, 0.34190E+00_JPRB, 0.34022E+00_JPRB, 0.33862E+00_JPRB, 0.33678E+00_JPRB ,&
&  0.33528E+00_JPRB, 0.33329E+00_JPRB, 0.33142E+00_JPRB, 0.33035E+00_JPRB, 0.32997E+00_JPRB, 0.32850E+00_JPRB, 0.32674E+00_JPRB ,&
&  0.32520E+00_JPRB, 0.32408E+00_JPRB, 0.32346E+00_JPRB, 0.32288E+00_JPRB, 0.32260E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.32711E+00_JPRB, 0.32741E+00_JPRB, 0.32781E+00_JPRB, 0.32811E+00_JPRB, 0.32776E+00_JPRB, 0.32762E+00_JPRB, 0.32738E+00_JPRB ,&
&  0.32709E+00_JPRB, 0.32641E+00_JPRB, 0.32589E+00_JPRB, 0.32549E+00_JPRB, 0.32514E+00_JPRB, 0.32501E+00_JPRB, 0.32482E+00_JPRB ,&
&  0.32480E+00_JPRB, 0.32449E+00_JPRB, 0.32408E+00_JPRB, 0.32273E+00_JPRB, 0.32063E+00_JPRB, 0.31791E+00_JPRB, 0.31587E+00_JPRB ,&
&  0.31362E+00_JPRB, 0.31121E+00_JPRB, 0.30827E+00_JPRB, 0.30509E+00_JPRB, 0.30200E+00_JPRB, 0.28314E+00_JPRB, 0.27714E+00_JPRB ,&
&  0.27667E+00_JPRB, 0.27677E+00_JPRB, 0.27892E+00_JPRB, 0.28202E+00_JPRB, 0.28542E+00_JPRB, 0.28201E+00_JPRB, 0.28104E+00_JPRB ,&
&  0.26965E+00_JPRB, 0.27055E+00_JPRB, 0.27152E+00_JPRB, 0.27270E+00_JPRB, 0.27573E+00_JPRB, 0.27850E+00_JPRB, 0.28348E+00_JPRB ,&
&  0.29495E+00_JPRB, 0.29622E+00_JPRB, 0.29725E+00_JPRB, 0.29858E+00_JPRB, 0.29980E+00_JPRB, 0.30094E+00_JPRB, 0.30231E+00_JPRB ,&
&  0.30306E+00_JPRB, 0.30386E+00_JPRB, 0.30495E+00_JPRB, 0.30661E+00_JPRB, 0.30879E+00_JPRB, 0.31139E+00_JPRB, 0.31429E+00_JPRB ,&
&  0.31715E+00_JPRB, 0.32061E+00_JPRB, 0.32428E+00_JPRB, 0.32807E+00_JPRB, 0.33192E+00_JPRB, 0.33562E+00_JPRB, 0.33956E+00_JPRB ,&
&  0.34305E+00_JPRB, 0.34668E+00_JPRB, 0.34903E+00_JPRB, 0.35172E+00_JPRB, 0.35337E+00_JPRB, 0.35561E+00_JPRB, 0.35607E+00_JPRB ,&
&  0.35672E+00_JPRB, 0.35586E+00_JPRB, 0.35565E+00_JPRB, 0.35504E+00_JPRB, 0.35437E+00_JPRB, 0.35408E+00_JPRB, 0.35351E+00_JPRB ,&
&  0.35317E+00_JPRB, 0.35249E+00_JPRB, 0.35170E+00_JPRB, 0.35037E+00_JPRB, 0.34816E+00_JPRB, 0.34562E+00_JPRB, 0.34325E+00_JPRB ,&
&  0.34208E+00_JPRB, 0.34174E+00_JPRB, 0.34176E+00_JPRB, 0.34225E+00_JPRB, 0.34186E+00_JPRB, 0.34039E+00_JPRB, 0.33821E+00_JPRB ,&
&  0.33603E+00_JPRB, 0.33348E+00_JPRB, 0.33096E+00_JPRB, 0.32894E+00_JPRB, 0.32768E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.33305E+00_JPRB, 0.33384E+00_JPRB, 0.33446E+00_JPRB, 0.33498E+00_JPRB, 0.33489E+00_JPRB, 0.33486E+00_JPRB, 0.33452E+00_JPRB ,&
&  0.33407E+00_JPRB, 0.33351E+00_JPRB, 0.33293E+00_JPRB, 0.33249E+00_JPRB, 0.33192E+00_JPRB, 0.33159E+00_JPRB, 0.33089E+00_JPRB ,&
&  0.33069E+00_JPRB, 0.32967E+00_JPRB, 0.32831E+00_JPRB, 0.32533E+00_JPRB, 0.32245E+00_JPRB, 0.31905E+00_JPRB, 0.31437E+00_JPRB ,&
&  0.30958E+00_JPRB, 0.30520E+00_JPRB, 0.29082E+00_JPRB, 0.28616E+00_JPRB, 0.27811E+00_JPRB, 0.27355E+00_JPRB, 0.27049E+00_JPRB ,&
&  0.26766E+00_JPRB, 0.26606E+00_JPRB, 0.26117E+00_JPRB, 0.25657E+00_JPRB, 0.25232E+00_JPRB, 0.24552E+00_JPRB, 0.24133E+00_JPRB ,&
&  0.23678E+00_JPRB, 0.23463E+00_JPRB, 0.23524E+00_JPRB, 0.23780E+00_JPRB, 0.24411E+00_JPRB, 0.25183E+00_JPRB, 0.26210E+00_JPRB ,&
&  0.27549E+00_JPRB, 0.28704E+00_JPRB, 0.29809E+00_JPRB, 0.29752E+00_JPRB, 0.29965E+00_JPRB, 0.30147E+00_JPRB, 0.30233E+00_JPRB ,&
&  0.29899E+00_JPRB, 0.29762E+00_JPRB, 0.30190E+00_JPRB, 0.30343E+00_JPRB, 0.30590E+00_JPRB, 0.30921E+00_JPRB, 0.31284E+00_JPRB ,&
&  0.31702E+00_JPRB, 0.32182E+00_JPRB, 0.32711E+00_JPRB, 0.33308E+00_JPRB, 0.34041E+00_JPRB, 0.34688E+00_JPRB, 0.35179E+00_JPRB ,&
&  0.35560E+00_JPRB, 0.36063E+00_JPRB, 0.36361E+00_JPRB, 0.36640E+00_JPRB, 0.36897E+00_JPRB, 0.37337E+00_JPRB, 0.37682E+00_JPRB ,&
&  0.37913E+00_JPRB, 0.37802E+00_JPRB, 0.37534E+00_JPRB, 0.36885E+00_JPRB, 0.36578E+00_JPRB, 0.36882E+00_JPRB, 0.36751E+00_JPRB ,&
&  0.36685E+00_JPRB, 0.36508E+00_JPRB, 0.36347E+00_JPRB, 0.36116E+00_JPRB, 0.35881E+00_JPRB, 0.35532E+00_JPRB, 0.35212E+00_JPRB ,&
&  0.35008E+00_JPRB, 0.35031E+00_JPRB, 0.35085E+00_JPRB, 0.35149E+00_JPRB, 0.35053E+00_JPRB, 0.34915E+00_JPRB, 0.34721E+00_JPRB ,&
&  0.34463E+00_JPRB, 0.34136E+00_JPRB, 0.33792E+00_JPRB, 0.33508E+00_JPRB, 0.33334E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.34435E+00_JPRB, 0.34489E+00_JPRB, 0.34470E+00_JPRB, 0.34492E+00_JPRB, 0.34461E+00_JPRB, 0.34452E+00_JPRB, 0.34376E+00_JPRB ,&
&  0.34278E+00_JPRB, 0.34183E+00_JPRB, 0.34073E+00_JPRB, 0.33966E+00_JPRB, 0.33840E+00_JPRB, 0.33727E+00_JPRB, 0.33656E+00_JPRB ,&
&  0.33588E+00_JPRB, 0.33439E+00_JPRB, 0.33179E+00_JPRB, 0.32841E+00_JPRB, 0.32552E+00_JPRB, 0.32208E+00_JPRB, 0.31836E+00_JPRB ,&
&  0.31414E+00_JPRB, 0.30876E+00_JPRB, 0.29964E+00_JPRB, 0.29508E+00_JPRB, 0.28850E+00_JPRB, 0.28493E+00_JPRB, 0.28058E+00_JPRB ,&
&  0.28051E+00_JPRB, 0.26125E+00_JPRB, 0.25937E+00_JPRB, 0.25112E+00_JPRB, 0.24493E+00_JPRB, 0.24016E+00_JPRB, 0.23179E+00_JPRB ,&
&  0.22807E+00_JPRB, 0.22361E+00_JPRB, 0.22202E+00_JPRB, 0.22423E+00_JPRB, 0.22865E+00_JPRB, 0.23774E+00_JPRB, 0.24984E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.29518E+00_JPRB, 0.30168E+00_JPRB, 0.30552E+00_JPRB, 0.30946E+00_JPRB, 0.31209E+00_JPRB, 0.30781E+00_JPRB ,&
&  0.30558E+00_JPRB, 0.30297E+00_JPRB, 0.30181E+00_JPRB, 0.30246E+00_JPRB, 0.30687E+00_JPRB, 0.30881E+00_JPRB, 0.31275E+00_JPRB ,&
&  0.31730E+00_JPRB, 0.32410E+00_JPRB, 0.33179E+00_JPRB, 0.34123E+00_JPRB, 0.35397E+00_JPRB, 0.36535E+00_JPRB, 0.37506E+00_JPRB ,&
&  0.38146E+00_JPRB, 0.38450E+00_JPRB, 0.38344E+00_JPRB, 0.38540E+00_JPRB, 0.38879E+00_JPRB, 0.39578E+00_JPRB, 0.39970E+00_JPRB ,&
&  0.40235E+00_JPRB, 0.40497E+00_JPRB, 0.40377E+00_JPRB, 0.40101E+00_JPRB, 0.39505E+00_JPRB, 0.38891E+00_JPRB, 0.38603E+00_JPRB ,&
&  0.38717E+00_JPRB, 0.38426E+00_JPRB, 0.38002E+00_JPRB, 0.37439E+00_JPRB, 0.37099E+00_JPRB, 0.36647E+00_JPRB, 0.36301E+00_JPRB ,&
&  0.36054E+00_JPRB, 0.35997E+00_JPRB, 0.35853E+00_JPRB, 0.35819E+00_JPRB, 0.35651E+00_JPRB, 0.35566E+00_JPRB, 0.35453E+00_JPRB ,&
&  0.35193E+00_JPRB, 0.34971E+00_JPRB, 0.34620E+00_JPRB, 0.34424E+00_JPRB, 0.34438E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.35836E+00_JPRB, 0.35861E+00_JPRB, 0.35882E+00_JPRB, 0.35865E+00_JPRB, 0.35804E+00_JPRB, 0.35788E+00_JPRB, 0.35818E+00_JPRB ,&
&  0.35717E+00_JPRB, 0.35662E+00_JPRB, 0.35557E+00_JPRB, 0.35464E+00_JPRB, 0.35341E+00_JPRB, 0.35276E+00_JPRB, 0.35200E+00_JPRB ,&
&  0.35044E+00_JPRB, 0.34761E+00_JPRB, 0.34361E+00_JPRB, 0.33960E+00_JPRB, 0.33563E+00_JPRB, 0.33287E+00_JPRB, 0.32889E+00_JPRB ,&
&  0.32484E+00_JPRB, 0.31950E+00_JPRB, 0.31148E+00_JPRB, 0.30502E+00_JPRB, 0.30236E+00_JPRB, 0.29746E+00_JPRB, 0.29772E+00_JPRB ,&
&  0.29552E+00_JPRB, 0.29253E+00_JPRB, 0.28479E+00_JPRB, 0.28234E+00_JPRB, 0.27761E+00_JPRB, 0.24923E+00_JPRB, 0.24186E+00_JPRB ,&
&  0.23217E+00_JPRB, 0.22247E+00_JPRB, 0.21881E+00_JPRB, 0.21777E+00_JPRB, 0.22021E+00_JPRB, 0.22932E+00_JPRB, 0.24226E+00_JPRB ,&
&  0.25844E+00_JPRB, 0.29936E+00_JPRB, 0.31037E+00_JPRB, 0.31962E+00_JPRB, 0.32734E+00_JPRB, 0.33144E+00_JPRB, 0.33111E+00_JPRB ,&
&  0.32984E+00_JPRB, 0.32906E+00_JPRB, 0.33019E+00_JPRB, 0.33208E+00_JPRB, 0.33161E+00_JPRB, 0.32788E+00_JPRB, 0.32858E+00_JPRB ,&
&  0.32581E+00_JPRB, 0.32986E+00_JPRB, 0.33903E+00_JPRB, 0.35152E+00_JPRB, 0.36619E+00_JPRB, 0.37537E+00_JPRB, 0.38770E+00_JPRB ,&
&  0.39922E+00_JPRB, 0.40590E+00_JPRB, 0.41597E+00_JPRB, 0.42410E+00_JPRB, 0.42726E+00_JPRB, 0.42885E+00_JPRB, 0.42706E+00_JPRB ,&
&  0.42390E+00_JPRB, 0.43174E+00_JPRB, 0.43496E+00_JPRB, 0.43122E+00_JPRB, 0.42465E+00_JPRB, 0.41455E+00_JPRB, 0.40625E+00_JPRB ,&
&  0.40259E+00_JPRB, 0.40402E+00_JPRB, 0.40028E+00_JPRB, 0.39096E+00_JPRB, 0.38411E+00_JPRB, 0.37832E+00_JPRB, 0.37359E+00_JPRB ,&
&  0.37011E+00_JPRB, 0.36847E+00_JPRB, 0.36596E+00_JPRB, 0.36459E+00_JPRB, 0.36377E+00_JPRB, 0.36329E+00_JPRB, 0.36309E+00_JPRB ,&
&  0.36241E+00_JPRB, 0.36250E+00_JPRB, 0.36163E+00_JPRB, 0.35976E+00_JPRB, 0.35951E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.37516E+00_JPRB, 0.37740E+00_JPRB, 0.37955E+00_JPRB, 0.38009E+00_JPRB, 0.38066E+00_JPRB, 0.38056E+00_JPRB, 0.37824E+00_JPRB ,&
&  0.37364E+00_JPRB, 0.37003E+00_JPRB, 0.36631E+00_JPRB, 0.36527E+00_JPRB, 0.36454E+00_JPRB, 0.36424E+00_JPRB, 0.36401E+00_JPRB ,&
&  0.36224E+00_JPRB, 0.35462E+00_JPRB, 0.34934E+00_JPRB, 0.34670E+00_JPRB, 0.34409E+00_JPRB, 0.34280E+00_JPRB, 0.34124E+00_JPRB ,&
&  0.33724E+00_JPRB, 0.33158E+00_JPRB, 0.32609E+00_JPRB, 0.31608E+00_JPRB, 0.30831E+00_JPRB, 0.30719E+00_JPRB, 0.31012E+00_JPRB ,&
&  0.30741E+00_JPRB, 0.29840E+00_JPRB, 0.29885E+00_JPRB, 0.29708E+00_JPRB, 0.28610E+00_JPRB, 0.28363E+00_JPRB, 0.25922E+00_JPRB ,&
&  0.24143E+00_JPRB, 0.22746E+00_JPRB, 0.22226E+00_JPRB, 0.21917E+00_JPRB, 0.22282E+00_JPRB, 0.23216E+00_JPRB, 0.25376E+00_JPRB ,&
&  0.27771E+00_JPRB, 0.30425E+00_JPRB, 0.31380E+00_JPRB, 0.32643E+00_JPRB, 0.33780E+00_JPRB, 0.35175E+00_JPRB, 0.36493E+00_JPRB ,&
&  0.37189E+00_JPRB, 0.37623E+00_JPRB, 0.38096E+00_JPRB, 0.38256E+00_JPRB, 0.38671E+00_JPRB, 0.38849E+00_JPRB, 0.38644E+00_JPRB ,&
&  0.38873E+00_JPRB, 0.38877E+00_JPRB, 0.39354E+00_JPRB, 0.39988E+00_JPRB, 0.40735E+00_JPRB, 0.40954E+00_JPRB, 0.41041E+00_JPRB ,&
&  0.41534E+00_JPRB, 0.43052E+00_JPRB, 0.44966E+00_JPRB, 0.47130E+00_JPRB, 0.47429E+00_JPRB, 0.46857E+00_JPRB, 0.45410E+00_JPRB ,&
&  0.43786E+00_JPRB, 0.44711E+00_JPRB, 0.50376E+00_JPRB, 0.49013E+00_JPRB, 0.47967E+00_JPRB, 0.46735E+00_JPRB, 0.44500E+00_JPRB ,&
&  0.42107E+00_JPRB, 0.41222E+00_JPRB, 0.40565E+00_JPRB, 0.39431E+00_JPRB, 0.38600E+00_JPRB, 0.37789E+00_JPRB, 0.37265E+00_JPRB ,&
&  0.36964E+00_JPRB, 0.36999E+00_JPRB, 0.36895E+00_JPRB, 0.36915E+00_JPRB, 0.36852E+00_JPRB, 0.36812E+00_JPRB, 0.36932E+00_JPRB ,&
&  0.37010E+00_JPRB, 0.37223E+00_JPRB, 0.37424E+00_JPRB, 0.37528E+00_JPRB, 0.37538E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.39201E+00_JPRB, 0.39891E+00_JPRB, 0.40422E+00_JPRB, 0.41253E+00_JPRB, 0.41962E+00_JPRB, 0.41826E+00_JPRB, 0.41275E+00_JPRB ,&
&  0.40163E+00_JPRB, 0.38809E+00_JPRB, 0.37960E+00_JPRB, 0.37310E+00_JPRB, 0.36980E+00_JPRB, 0.36831E+00_JPRB, 0.36688E+00_JPRB ,&
&  0.36556E+00_JPRB, 0.36376E+00_JPRB, 0.35911E+00_JPRB, 0.35218E+00_JPRB, 0.34738E+00_JPRB, 0.34591E+00_JPRB, 0.34506E+00_JPRB ,&
&  0.34330E+00_JPRB, 0.33974E+00_JPRB, 0.33479E+00_JPRB, 0.32533E+00_JPRB, 0.31705E+00_JPRB, 0.31776E+00_JPRB, 0.31377E+00_JPRB ,&
&  0.30963E+00_JPRB, 0.30741E+00_JPRB, 0.30616E+00_JPRB, 0.30377E+00_JPRB, 0.30069E+00_JPRB, 0.29298E+00_JPRB, 0.27484E+00_JPRB ,&
&  0.25482E+00_JPRB, 0.23829E+00_JPRB, 0.22816E+00_JPRB, 0.22527E+00_JPRB, 0.23088E+00_JPRB, 0.24621E+00_JPRB, 0.26983E+00_JPRB ,&
&  0.30200E+00_JPRB, 0.31639E+00_JPRB, 0.33190E+00_JPRB, 0.35109E+00_JPRB, 0.36827E+00_JPRB, 0.38753E+00_JPRB, 0.40406E+00_JPRB ,&
&  0.41879E+00_JPRB, 0.43683E+00_JPRB, 0.45728E+00_JPRB, 0.48006E+00_JPRB, 0.50769E+00_JPRB, 0.48656E+00_JPRB, 0.49676E+00_JPRB ,&
&  0.55313E+00_JPRB, 0.55514E+00_JPRB, 0.53578E+00_JPRB, 0.51105E+00_JPRB, 0.49349E+00_JPRB, 0.48445E+00_JPRB, 0.48109E+00_JPRB ,&
&  0.48150E+00_JPRB, 0.48752E+00_JPRB, 0.49941E+00_JPRB, 0.51235E+00_JPRB, 0.51355E+00_JPRB, 0.50579E+00_JPRB, 0.48463E+00_JPRB ,&
&  0.46548E+00_JPRB, 0.47562E+00_JPRB, 0.60782E+00_JPRB, 0.54411E+00_JPRB, 0.49560E+00_JPRB, 0.47173E+00_JPRB, 0.44470E+00_JPRB ,&
&  0.42482E+00_JPRB, 0.41113E+00_JPRB, 0.39908E+00_JPRB, 0.39105E+00_JPRB, 0.37914E+00_JPRB, 0.36558E+00_JPRB, 0.35387E+00_JPRB ,&
&  0.35139E+00_JPRB, 0.35270E+00_JPRB, 0.35527E+00_JPRB, 0.35802E+00_JPRB, 0.35951E+00_JPRB, 0.36242E+00_JPRB, 0.36597E+00_JPRB ,&
&  0.36671E+00_JPRB, 0.36916E+00_JPRB, 0.37452E+00_JPRB, 0.38005E+00_JPRB, 0.38566E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.38328E+00_JPRB, 0.40139E+00_JPRB, 0.42572E+00_JPRB, 0.44023E+00_JPRB, 0.44739E+00_JPRB, 0.43405E+00_JPRB, 0.42079E+00_JPRB ,&
&  0.40674E+00_JPRB, 0.38560E+00_JPRB, 0.37505E+00_JPRB, 0.36821E+00_JPRB, 0.36408E+00_JPRB, 0.36100E+00_JPRB, 0.35850E+00_JPRB ,&
&  0.35625E+00_JPRB, 0.35334E+00_JPRB, 0.35146E+00_JPRB, 0.35220E+00_JPRB, 0.35337E+00_JPRB, 0.35142E+00_JPRB, 0.34787E+00_JPRB ,&
&  0.34475E+00_JPRB, 0.34143E+00_JPRB, 0.33434E+00_JPRB, 0.32417E+00_JPRB, 0.32037E+00_JPRB, 0.32500E+00_JPRB, 0.32350E+00_JPRB ,&
&  0.31936E+00_JPRB, 0.31378E+00_JPRB, 0.31262E+00_JPRB, 0.31351E+00_JPRB, 0.31096E+00_JPRB, 0.30491E+00_JPRB, 0.28844E+00_JPRB ,&
&  0.26625E+00_JPRB, 0.24854E+00_JPRB, 0.24231E+00_JPRB, 0.25210E+00_JPRB, 0.27284E+00_JPRB, 0.30616E+00_JPRB, 0.31823E+00_JPRB ,&
&  0.32939E+00_JPRB, 0.34348E+00_JPRB, 0.35836E+00_JPRB, 0.37955E+00_JPRB, 0.40360E+00_JPRB, 0.42461E+00_JPRB, 0.44273E+00_JPRB ,&
&  0.46676E+00_JPRB, 0.48970E+00_JPRB, 0.51853E+00_JPRB, 0.48593E+00_JPRB, 0.49522E+00_JPRB, 0.53371E+00_JPRB, 0.55138E+00_JPRB ,&
&  0.56168E+00_JPRB, 0.57800E+00_JPRB, 0.59164E+00_JPRB, 0.60937E+00_JPRB, 0.59716E+00_JPRB, 0.57828E+00_JPRB, 0.55871E+00_JPRB ,&
&  0.54941E+00_JPRB, 0.53809E+00_JPRB, 0.53276E+00_JPRB, 0.54445E+00_JPRB, 0.55628E+00_JPRB, 0.55144E+00_JPRB, 0.53277E+00_JPRB ,&
&  0.50600E+00_JPRB, 0.46871E+00_JPRB, 0.43474E+00_JPRB, 0.41309E+00_JPRB, 0.41198E+00_JPRB, 0.41910E+00_JPRB, 0.42038E+00_JPRB ,&
&  0.41662E+00_JPRB, 0.41294E+00_JPRB, 0.40686E+00_JPRB, 0.40195E+00_JPRB, 0.37652E+00_JPRB, 0.35772E+00_JPRB, 0.34311E+00_JPRB ,&
&  0.33924E+00_JPRB, 0.33855E+00_JPRB, 0.34024E+00_JPRB, 0.34715E+00_JPRB, 0.34907E+00_JPRB, 0.35251E+00_JPRB, 0.35519E+00_JPRB ,&
&  0.35519E+00_JPRB, 0.35502E+00_JPRB, 0.35938E+00_JPRB, 0.36479E+00_JPRB, 0.37232E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.43209E+00_JPRB, 0.44595E+00_JPRB, 0.45739E+00_JPRB, 0.46470E+00_JPRB, 0.45643E+00_JPRB, 0.45260E+00_JPRB, 0.44999E+00_JPRB ,&
&  0.45368E+00_JPRB, 0.41241E+00_JPRB, 0.38882E+00_JPRB, 0.36940E+00_JPRB, 0.35600E+00_JPRB, 0.34288E+00_JPRB, 0.33824E+00_JPRB ,&
&  0.33862E+00_JPRB, 0.34056E+00_JPRB, 0.34213E+00_JPRB, 0.34334E+00_JPRB, 0.34466E+00_JPRB, 0.34457E+00_JPRB, 0.34533E+00_JPRB ,&
&  0.34736E+00_JPRB, 0.34719E+00_JPRB, 0.34002E+00_JPRB, 0.33405E+00_JPRB, 0.33229E+00_JPRB, 0.33375E+00_JPRB, 0.33112E+00_JPRB ,&
&  0.32763E+00_JPRB, 0.32346E+00_JPRB, 0.32145E+00_JPRB, 0.32275E+00_JPRB, 0.32156E+00_JPRB, 0.31634E+00_JPRB, 0.29721E+00_JPRB ,&
&  0.27116E+00_JPRB, 0.26284E+00_JPRB, 0.27625E+00_JPRB, 0.32004E+00_JPRB, 0.33354E+00_JPRB, 0.34217E+00_JPRB, 0.35463E+00_JPRB ,&
&  0.35180E+00_JPRB, 0.35476E+00_JPRB, 0.37313E+00_JPRB, 0.41655E+00_JPRB, 0.43801E+00_JPRB, 0.45675E+00_JPRB, 0.47659E+00_JPRB ,&
&  0.50795E+00_JPRB, 0.52906E+00_JPRB, 0.55127E+00_JPRB, 0.51668E+00_JPRB, 0.56498E+00_JPRB, 0.68739E+00_JPRB, 0.66132E+00_JPRB ,&
&  0.65711E+00_JPRB, 0.65639E+00_JPRB, 0.67664E+00_JPRB, 0.68041E+00_JPRB, 0.66528E+00_JPRB, 0.64350E+00_JPRB, 0.62865E+00_JPRB ,&
&  0.60726E+00_JPRB, 0.59293E+00_JPRB, 0.59008E+00_JPRB, 0.60936E+00_JPRB, 0.60153E+00_JPRB, 0.58722E+00_JPRB, 0.56886E+00_JPRB ,&
&  0.54073E+00_JPRB, 0.50266E+00_JPRB, 0.46710E+00_JPRB, 0.44043E+00_JPRB, 0.43221E+00_JPRB, 0.43172E+00_JPRB, 0.42828E+00_JPRB ,&
&  0.42011E+00_JPRB, 0.41948E+00_JPRB, 0.42252E+00_JPRB, 0.42341E+00_JPRB, 0.39691E+00_JPRB, 0.36646E+00_JPRB, 0.34404E+00_JPRB ,&
&  0.32907E+00_JPRB, 0.32086E+00_JPRB, 0.31484E+00_JPRB, 0.31988E+00_JPRB, 0.32904E+00_JPRB, 0.33897E+00_JPRB, 0.34111E+00_JPRB ,&
&  0.34470E+00_JPRB, 0.34871E+00_JPRB, 0.36074E+00_JPRB, 0.38407E+00_JPRB, 0.40939E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.45454E+00_JPRB, 0.47367E+00_JPRB, 0.48889E+00_JPRB, 0.48590E+00_JPRB, 0.47620E+00_JPRB, 0.46634E+00_JPRB, 0.46081E+00_JPRB ,&
&  0.46679E+00_JPRB, 0.40235E+00_JPRB, 0.37875E+00_JPRB, 0.36332E+00_JPRB, 0.35468E+00_JPRB, 0.33935E+00_JPRB, 0.32527E+00_JPRB ,&
&  0.32255E+00_JPRB, 0.32801E+00_JPRB, 0.33738E+00_JPRB, 0.34341E+00_JPRB, 0.34832E+00_JPRB, 0.35143E+00_JPRB, 0.35239E+00_JPRB ,&
&  0.35393E+00_JPRB, 0.35556E+00_JPRB, 0.35194E+00_JPRB, 0.34699E+00_JPRB, 0.34428E+00_JPRB, 0.34444E+00_JPRB, 0.34517E+00_JPRB ,&
&  0.34502E+00_JPRB, 0.34339E+00_JPRB, 0.33648E+00_JPRB, 0.33429E+00_JPRB, 0.33295E+00_JPRB, 0.33450E+00_JPRB, 0.33477E+00_JPRB ,&
&  0.28855E+00_JPRB, 0.29078E+00_JPRB, 0.33157E+00_JPRB, 0.33879E+00_JPRB, 0.34948E+00_JPRB, 0.35572E+00_JPRB, 0.36170E+00_JPRB ,&
&  0.37246E+00_JPRB, 0.39264E+00_JPRB, 0.42036E+00_JPRB, 0.45338E+00_JPRB, 0.48023E+00_JPRB, 0.50066E+00_JPRB, 0.51805E+00_JPRB ,&
&  0.55945E+00_JPRB, 0.51217E+00_JPRB, 0.49184E+00_JPRB, 0.53235E+00_JPRB, 0.64420E+00_JPRB, 0.70253E+00_JPRB, 0.75095E+00_JPRB ,&
&  0.77913E+00_JPRB, 0.76222E+00_JPRB, 0.74520E+00_JPRB, 0.73757E+00_JPRB, 0.72387E+00_JPRB, 0.70690E+00_JPRB, 0.70417E+00_JPRB ,&
&  0.68742E+00_JPRB, 0.68449E+00_JPRB, 0.68858E+00_JPRB, 0.68916E+00_JPRB, 0.65887E+00_JPRB, 0.63667E+00_JPRB, 0.61260E+00_JPRB ,&
&  0.58355E+00_JPRB, 0.55051E+00_JPRB, 0.51309E+00_JPRB, 0.48290E+00_JPRB, 0.46156E+00_JPRB, 0.45844E+00_JPRB, 0.45460E+00_JPRB ,&
&  0.44582E+00_JPRB, 0.44161E+00_JPRB, 0.43698E+00_JPRB, 0.43275E+00_JPRB, 0.41595E+00_JPRB, 0.39854E+00_JPRB, 0.38843E+00_JPRB ,&
&  0.36128E+00_JPRB, 0.34388E+00_JPRB, 0.32280E+00_JPRB, 0.32240E+00_JPRB, 0.33521E+00_JPRB, 0.34860E+00_JPRB, 0.35811E+00_JPRB ,&
&  0.37287E+00_JPRB, 0.38235E+00_JPRB, 0.38847E+00_JPRB, 0.40097E+00_JPRB, 0.42421E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.46786E+00_JPRB, 0.48073E+00_JPRB, 0.48862E+00_JPRB, 0.48712E+00_JPRB, 0.48817E+00_JPRB, 0.48177E+00_JPRB, 0.48819E+00_JPRB ,&
&  0.52524E+00_JPRB, 0.48113E+00_JPRB, 0.51575E+00_JPRB, 0.41631E+00_JPRB, 0.37167E+00_JPRB, 0.35089E+00_JPRB, 0.33351E+00_JPRB ,&
&  0.32918E+00_JPRB, 0.33491E+00_JPRB, 0.34025E+00_JPRB, 0.34590E+00_JPRB, 0.35622E+00_JPRB, 0.35623E+00_JPRB, 0.35562E+00_JPRB ,&
&  0.35894E+00_JPRB, 0.36188E+00_JPRB, 0.36771E+00_JPRB, 0.35962E+00_JPRB, 0.35756E+00_JPRB, 0.35807E+00_JPRB, 0.36595E+00_JPRB ,&
&  0.35376E+00_JPRB, 0.35136E+00_JPRB, 0.35906E+00_JPRB, 0.35746E+00_JPRB, 0.35677E+00_JPRB, 0.35534E+00_JPRB, 0.35670E+00_JPRB ,&
&  0.36344E+00_JPRB, 0.34201E+00_JPRB, 0.36239E+00_JPRB, 0.36984E+00_JPRB, 0.37552E+00_JPRB, 0.38252E+00_JPRB, 0.38828E+00_JPRB ,&
&  0.39857E+00_JPRB, 0.41670E+00_JPRB, 0.44426E+00_JPRB, 0.47326E+00_JPRB, 0.50652E+00_JPRB, 0.53608E+00_JPRB, 0.55810E+00_JPRB ,&
&  0.60916E+00_JPRB, 0.53337E+00_JPRB, 0.62694E+00_JPRB, 0.67009E+00_JPRB, 0.77556E+00_JPRB, 0.91410E+00_JPRB, 0.94753E+00_JPRB ,&
&  0.10189E+01_JPRB, 0.96179E+00_JPRB, 0.91916E+00_JPRB, 0.87149E+00_JPRB, 0.82553E+00_JPRB, 0.79437E+00_JPRB, 0.77454E+00_JPRB ,&
&  0.78350E+00_JPRB, 0.78799E+00_JPRB, 0.83283E+00_JPRB, 0.80057E+00_JPRB, 0.73704E+00_JPRB, 0.69909E+00_JPRB, 0.66152E+00_JPRB ,&
&  0.62971E+00_JPRB, 0.60750E+00_JPRB, 0.58422E+00_JPRB, 0.53386E+00_JPRB, 0.52056E+00_JPRB, 0.51179E+00_JPRB, 0.48209E+00_JPRB ,&
&  0.46760E+00_JPRB, 0.44564E+00_JPRB, 0.42258E+00_JPRB, 0.42832E+00_JPRB, 0.41040E+00_JPRB, 0.39714E+00_JPRB, 0.39386E+00_JPRB ,&
&  0.37961E+00_JPRB, 0.35607E+00_JPRB, 0.35808E+00_JPRB, 0.36415E+00_JPRB, 0.36368E+00_JPRB, 0.36779E+00_JPRB, 0.39033E+00_JPRB ,&
&  0.39617E+00_JPRB, 0.40802E+00_JPRB, 0.43237E+00_JPRB, 0.44237E+00_JPRB, 0.45663E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.47511E+00_JPRB, 0.47450E+00_JPRB, 0.47636E+00_JPRB, 0.47962E+00_JPRB, 0.49231E+00_JPRB, 0.51550E+00_JPRB, 0.50520E+00_JPRB ,&
&  0.53213E+00_JPRB, 0.53375E+00_JPRB, 0.52428E+00_JPRB, 0.51552E+00_JPRB, 0.49457E+00_JPRB, 0.42756E+00_JPRB, 0.34875E+00_JPRB ,&
&  0.32608E+00_JPRB, 0.33145E+00_JPRB, 0.34254E+00_JPRB, 0.34767E+00_JPRB, 0.35484E+00_JPRB, 0.35384E+00_JPRB, 0.35455E+00_JPRB ,&
&  0.35915E+00_JPRB, 0.36457E+00_JPRB, 0.37057E+00_JPRB, 0.37329E+00_JPRB, 0.36894E+00_JPRB, 0.37602E+00_JPRB, 0.39183E+00_JPRB ,&
&  0.37130E+00_JPRB, 0.35927E+00_JPRB, 0.35674E+00_JPRB, 0.34846E+00_JPRB, 0.36456E+00_JPRB, 0.37142E+00_JPRB, 0.37813E+00_JPRB ,&
&  0.38599E+00_JPRB, 0.39353E+00_JPRB, 0.39472E+00_JPRB, 0.39992E+00_JPRB, 0.40898E+00_JPRB, 0.42535E+00_JPRB, 0.44197E+00_JPRB ,&
&  0.46151E+00_JPRB, 0.48253E+00_JPRB, 0.51030E+00_JPRB, 0.54783E+00_JPRB, 0.59478E+00_JPRB, 0.60087E+00_JPRB, 0.65073E+00_JPRB ,&
&  0.69555E+00_JPRB, 0.78080E+00_JPRB, 0.88333E+00_JPRB, 0.84955E+00_JPRB, 0.96167E+00_JPRB, 0.10557E+01_JPRB, 0.10442E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.99796E+00_JPRB, 0.10020E+01_JPRB, 0.10014E+01_JPRB, 0.99731E+00_JPRB, 0.96710E+00_JPRB, 0.91005E+00_JPRB ,&
&  0.89120E+00_JPRB, 0.99864E+00_JPRB, 0.99815E+00_JPRB, 0.93145E+00_JPRB, 0.83942E+00_JPRB, 0.78469E+00_JPRB, 0.72665E+00_JPRB ,&
&  0.70536E+00_JPRB, 0.70366E+00_JPRB, 0.67769E+00_JPRB, 0.65060E+00_JPRB, 0.60713E+00_JPRB, 0.56211E+00_JPRB, 0.52592E+00_JPRB ,&
&  0.48519E+00_JPRB, 0.43214E+00_JPRB, 0.40856E+00_JPRB, 0.40520E+00_JPRB, 0.40273E+00_JPRB, 0.38467E+00_JPRB, 0.38027E+00_JPRB ,&
&  0.38124E+00_JPRB, 0.37905E+00_JPRB, 0.39397E+00_JPRB, 0.39389E+00_JPRB, 0.39494E+00_JPRB, 0.39456E+00_JPRB, 0.38654E+00_JPRB ,&
&  0.38436E+00_JPRB, 0.41578E+00_JPRB, 0.43898E+00_JPRB, 0.45635E+00_JPRB, 0.46698E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.47387E+00_JPRB, 0.47125E+00_JPRB, 0.46457E+00_JPRB, 0.47308E+00_JPRB, 0.49801E+00_JPRB, 0.52104E+00_JPRB, 0.53541E+00_JPRB ,&
&  0.53568E+00_JPRB, 0.53455E+00_JPRB, 0.53143E+00_JPRB, 0.52455E+00_JPRB, 0.50870E+00_JPRB, 0.49056E+00_JPRB, 0.43660E+00_JPRB ,&
&  0.36596E+00_JPRB, 0.35988E+00_JPRB, 0.34233E+00_JPRB, 0.34479E+00_JPRB, 0.35931E+00_JPRB, 0.35743E+00_JPRB, 0.35786E+00_JPRB ,&
&  0.36352E+00_JPRB, 0.36890E+00_JPRB, 0.37502E+00_JPRB, 0.37560E+00_JPRB, 0.37632E+00_JPRB, 0.40464E+00_JPRB, 0.40331E+00_JPRB ,&
&  0.37300E+00_JPRB, 0.35481E+00_JPRB, 0.35158E+00_JPRB, 0.35631E+00_JPRB, 0.36907E+00_JPRB, 0.38488E+00_JPRB, 0.39289E+00_JPRB ,&
&  0.40012E+00_JPRB, 0.40785E+00_JPRB, 0.41242E+00_JPRB, 0.42502E+00_JPRB, 0.44514E+00_JPRB, 0.47056E+00_JPRB, 0.49332E+00_JPRB ,&
&  0.51191E+00_JPRB, 0.53648E+00_JPRB, 0.57139E+00_JPRB, 0.61999E+00_JPRB, 0.65776E+00_JPRB, 0.70392E+00_JPRB, 0.77463E+00_JPRB ,&
&  0.83950E+00_JPRB, 0.93554E+00_JPRB, 0.99616E+00_JPRB, 0.10687E+01_JPRB, 0.11838E+01_JPRB, 0.11878E+01_JPRB, 0.11416E+01_JPRB ,&
&  0.11237E+01_JPRB, 0.10632E+01_JPRB, 0.12070E+01_JPRB, 0.12090E+01_JPRB, 0.11548E+01_JPRB, 0.11420E+01_JPRB, 0.10620E+01_JPRB ,&
&  0.10087E+01_JPRB, 0.99353E+00_JPRB, 0.96920E+00_JPRB, 0.93749E+00_JPRB, 0.89952E+00_JPRB, 0.89826E+00_JPRB, 0.83968E+00_JPRB ,&
&  0.83305E+00_JPRB, 0.80575E+00_JPRB, 0.68779E+00_JPRB, 0.60463E+00_JPRB, 0.53063E+00_JPRB, 0.53816E+00_JPRB, 0.52498E+00_JPRB ,&
&  0.48310E+00_JPRB, 0.42212E+00_JPRB, 0.41893E+00_JPRB, 0.41338E+00_JPRB, 0.41608E+00_JPRB, 0.41790E+00_JPRB, 0.40421E+00_JPRB ,&
&  0.40887E+00_JPRB, 0.42694E+00_JPRB, 0.42704E+00_JPRB, 0.43417E+00_JPRB, 0.43355E+00_JPRB, 0.42497E+00_JPRB, 0.38697E+00_JPRB ,&
&  0.38640E+00_JPRB, 0.41205E+00_JPRB, 0.43107E+00_JPRB, 0.45181E+00_JPRB, 0.46689E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.48447E+00_JPRB, 0.48493E+00_JPRB, 0.47960E+00_JPRB, 0.48688E+00_JPRB, 0.50996E+00_JPRB, 0.53310E+00_JPRB, 0.54686E+00_JPRB ,&
&  0.54761E+00_JPRB, 0.54972E+00_JPRB, 0.54285E+00_JPRB, 0.53404E+00_JPRB, 0.52300E+00_JPRB, 0.50340E+00_JPRB, 0.50405E+00_JPRB ,&
&  0.40789E+00_JPRB, 0.38021E+00_JPRB, 0.33707E+00_JPRB, 0.33180E+00_JPRB, 0.36801E+00_JPRB, 0.36619E+00_JPRB, 0.36968E+00_JPRB ,&
&  0.36905E+00_JPRB, 0.37722E+00_JPRB, 0.37711E+00_JPRB, 0.37810E+00_JPRB, 0.39609E+00_JPRB, 0.42319E+00_JPRB, 0.41033E+00_JPRB ,&
&  0.38707E+00_JPRB, 0.37060E+00_JPRB, 0.36971E+00_JPRB, 0.37181E+00_JPRB, 0.37958E+00_JPRB, 0.39506E+00_JPRB, 0.40728E+00_JPRB ,&
&  0.41805E+00_JPRB, 0.44099E+00_JPRB, 0.46141E+00_JPRB, 0.48714E+00_JPRB, 0.51712E+00_JPRB, 0.55610E+00_JPRB, 0.58510E+00_JPRB ,&
&  0.61470E+00_JPRB, 0.65227E+00_JPRB, 0.69602E+00_JPRB, 0.75221E+00_JPRB, 0.78448E+00_JPRB, 0.86232E+00_JPRB, 0.95225E+00_JPRB ,&
&  0.11078E+01_JPRB, 0.11656E+01_JPRB, 0.12085E+01_JPRB, 0.13958E+01_JPRB, 0.14601E+01_JPRB, 0.14212E+01_JPRB, 0.12961E+01_JPRB ,&
&  0.12212E+01_JPRB, 0.11600E+01_JPRB, 0.11700E+01_JPRB, 0.11991E+01_JPRB, 0.11780E+01_JPRB, 0.11995E+01_JPRB, 0.11330E+01_JPRB ,&
&  0.10185E+01_JPRB, 0.96670E+00_JPRB, 0.94108E+00_JPRB, 0.94820E+00_JPRB, 0.90621E+00_JPRB, 0.91464E+00_JPRB, 0.88057E+00_JPRB ,&
&  0.88209E+00_JPRB, 0.76204E+00_JPRB, 0.58349E+00_JPRB, 0.53769E+00_JPRB, 0.45182E+00_JPRB, 0.46160E+00_JPRB, 0.55030E+00_JPRB ,&
&  0.53255E+00_JPRB, 0.48482E+00_JPRB, 0.45319E+00_JPRB, 0.43599E+00_JPRB, 0.43358E+00_JPRB, 0.47945E+00_JPRB, 0.47214E+00_JPRB ,&
&  0.44068E+00_JPRB, 0.46167E+00_JPRB, 0.48448E+00_JPRB, 0.49883E+00_JPRB, 0.49665E+00_JPRB, 0.48860E+00_JPRB, 0.45420E+00_JPRB ,&
&  0.45439E+00_JPRB, 0.45453E+00_JPRB, 0.46122E+00_JPRB, 0.47303E+00_JPRB, 0.48031E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.51545E+00_JPRB, 0.51487E+00_JPRB, 0.51244E+00_JPRB, 0.51916E+00_JPRB, 0.53497E+00_JPRB, 0.55408E+00_JPRB, 0.56690E+00_JPRB ,&
&  0.56458E+00_JPRB, 0.56358E+00_JPRB, 0.55522E+00_JPRB, 0.54495E+00_JPRB, 0.53401E+00_JPRB, 0.51790E+00_JPRB, 0.50831E+00_JPRB ,&
&  0.51384E+00_JPRB, 0.43559E+00_JPRB, 0.36720E+00_JPRB, 0.32996E+00_JPRB, 0.34948E+00_JPRB, 0.35633E+00_JPRB, 0.37023E+00_JPRB ,&
&  0.36851E+00_JPRB, 0.38681E+00_JPRB, 0.38749E+00_JPRB, 0.39216E+00_JPRB, 0.43046E+00_JPRB, 0.44053E+00_JPRB, 0.43174E+00_JPRB ,&
&  0.42926E+00_JPRB, 0.41346E+00_JPRB, 0.40829E+00_JPRB, 0.41578E+00_JPRB, 0.43183E+00_JPRB, 0.42051E+00_JPRB, 0.44514E+00_JPRB ,&
&  0.47463E+00_JPRB, 0.50678E+00_JPRB, 0.53535E+00_JPRB, 0.57861E+00_JPRB, 0.63173E+00_JPRB, 0.67154E+00_JPRB, 0.71179E+00_JPRB ,&
&  0.74978E+00_JPRB, 0.77522E+00_JPRB, 0.78752E+00_JPRB, 0.83211E+00_JPRB, 0.90821E+00_JPRB, 0.10423E+01_JPRB, 0.10499E+01_JPRB ,&
&  0.11163E+01_JPRB, 0.11731E+01_JPRB, 0.11573E+01_JPRB, 0.13854E+01_JPRB, 0.15050E+01_JPRB, 0.14766E+01_JPRB, 0.13995E+01_JPRB ,&
&  0.13447E+01_JPRB, 0.13012E+01_JPRB, 0.13515E+01_JPRB, 0.13477E+01_JPRB, 0.13298E+01_JPRB, 0.12877E+01_JPRB, 0.11810E+01_JPRB ,&
&  0.10625E+01_JPRB, 0.98933E+00_JPRB, 0.95363E+00_JPRB, 0.91296E+00_JPRB, 0.87647E+00_JPRB, 0.84484E+00_JPRB, 0.86744E+00_JPRB ,&
&  0.92631E+00_JPRB, 0.55886E+00_JPRB, 0.46154E+00_JPRB, 0.51512E+00_JPRB, 0.44817E+00_JPRB, 0.45730E+00_JPRB, 0.50252E+00_JPRB ,&
&  0.49319E+00_JPRB, 0.48417E+00_JPRB, 0.49324E+00_JPRB, 0.47996E+00_JPRB, 0.49036E+00_JPRB, 0.57625E+00_JPRB, 0.57967E+00_JPRB ,&
&  0.54853E+00_JPRB, 0.52860E+00_JPRB, 0.56274E+00_JPRB, 0.56751E+00_JPRB, 0.56955E+00_JPRB, 0.56149E+00_JPRB, 0.54147E+00_JPRB ,&
&  0.52479E+00_JPRB, 0.51969E+00_JPRB, 0.51534E+00_JPRB, 0.51503E+00_JPRB, 0.51504E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.57695E+00_JPRB, 0.57813E+00_JPRB, 0.57882E+00_JPRB, 0.57808E+00_JPRB, 0.58241E+00_JPRB, 0.59590E+00_JPRB, 0.60292E+00_JPRB ,&
&  0.59686E+00_JPRB, 0.59028E+00_JPRB, 0.57662E+00_JPRB, 0.56317E+00_JPRB, 0.55002E+00_JPRB, 0.53770E+00_JPRB, 0.52705E+00_JPRB ,&
&  0.51634E+00_JPRB, 0.48241E+00_JPRB, 0.38958E+00_JPRB, 0.34758E+00_JPRB, 0.32366E+00_JPRB, 0.33994E+00_JPRB, 0.35846E+00_JPRB ,&
&  0.36971E+00_JPRB, 0.38463E+00_JPRB, 0.39653E+00_JPRB, 0.44238E+00_JPRB, 0.50312E+00_JPRB, 0.48754E+00_JPRB, 0.50646E+00_JPRB ,&
&  0.50995E+00_JPRB, 0.49432E+00_JPRB, 0.46429E+00_JPRB, 0.48532E+00_JPRB, 0.48042E+00_JPRB, 0.47542E+00_JPRB, 0.54617E+00_JPRB ,&
&  0.59441E+00_JPRB, 0.63088E+00_JPRB, 0.66861E+00_JPRB, 0.70733E+00_JPRB, 0.74131E+00_JPRB, 0.74229E+00_JPRB, 0.75999E+00_JPRB ,&
&  0.78836E+00_JPRB, 0.81593E+00_JPRB, 0.85501E+00_JPRB, 0.87867E+00_JPRB, 0.93226E+00_JPRB, 0.99669E+00_JPRB, 0.98732E+00_JPRB ,&
&  0.10296E+01_JPRB, 0.93848E+00_JPRB, 0.76135E+00_JPRB, 0.10013E+01_JPRB, 0.14297E+01_JPRB, 0.14990E+01_JPRB, 0.13736E+01_JPRB ,&
&  0.14555E+01_JPRB, 0.14701E+01_JPRB, 0.15775E+01_JPRB, 0.16460E+01_JPRB, 0.15619E+01_JPRB, 0.14448E+01_JPRB, 0.13156E+01_JPRB ,&
&  0.11714E+01_JPRB, 0.10780E+01_JPRB, 0.10365E+01_JPRB, 0.10824E+01_JPRB, 0.10479E+01_JPRB, 0.10456E+01_JPRB, 0.10403E+01_JPRB ,&
&  0.91481E+00_JPRB, 0.76765E+00_JPRB, 0.56566E+00_JPRB, 0.57718E+00_JPRB, 0.47070E+00_JPRB, 0.47255E+00_JPRB, 0.51317E+00_JPRB ,&
&  0.49770E+00_JPRB, 0.52085E+00_JPRB, 0.53449E+00_JPRB, 0.53079E+00_JPRB, 0.62133E+00_JPRB, 0.71031E+00_JPRB, 0.77159E+00_JPRB ,&
&  0.75042E+00_JPRB, 0.65362E+00_JPRB, 0.68673E+00_JPRB, 0.67231E+00_JPRB, 0.68013E+00_JPRB, 0.67007E+00_JPRB, 0.65539E+00_JPRB ,&
&  0.63471E+00_JPRB, 0.61260E+00_JPRB, 0.59456E+00_JPRB, 0.58270E+00_JPRB, 0.57776E+00_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.61082E+00_JPRB, 0.60154E+00_JPRB, 0.59624E+00_JPRB, 0.59462E+00_JPRB, 0.59904E+00_JPRB, 0.60958E+00_JPRB, 0.61023E+00_JPRB ,&
&  0.60725E+00_JPRB, 0.59760E+00_JPRB, 0.58419E+00_JPRB, 0.56941E+00_JPRB, 0.55615E+00_JPRB, 0.54646E+00_JPRB, 0.53811E+00_JPRB ,&
&  0.52313E+00_JPRB, 0.45839E+00_JPRB, 0.36660E+00_JPRB, 0.32438E+00_JPRB, 0.30763E+00_JPRB, 0.31574E+00_JPRB, 0.34466E+00_JPRB ,&
&  0.37205E+00_JPRB, 0.39604E+00_JPRB, 0.43928E+00_JPRB, 0.51254E+00_JPRB, 0.57375E+00_JPRB, 0.58911E+00_JPRB, 0.61390E+00_JPRB ,&
&  0.69464E+00_JPRB, 0.56897E+00_JPRB, 0.56642E+00_JPRB, 0.56638E+00_JPRB, 0.62352E+00_JPRB, 0.66912E+00_JPRB, 0.66140E+00_JPRB ,&
&  0.64942E+00_JPRB, 0.67456E+00_JPRB, 0.70180E+00_JPRB, 0.73146E+00_JPRB, 0.75220E+00_JPRB, 0.78623E+00_JPRB, 0.86623E+00_JPRB ,&
&  0.93783E+00_JPRB, 0.95669E+00_JPRB, 0.91453E+00_JPRB, 0.89536E+00_JPRB, 0.96189E+00_JPRB, 0.10029E+01_JPRB, 0.98418E+00_JPRB ,&
&  0.88586E+00_JPRB, 0.83830E+00_JPRB, 0.97197E+00_JPRB, 0.11673E+01_JPRB, 0.14014E+01_JPRB, 0.15749E+01_JPRB, 0.15850E+01_JPRB ,&
&  0.16483E+01_JPRB, 0.16442E+01_JPRB, 0.16954E+01_JPRB, 0.15377E+01_JPRB, 0.16214E+01_JPRB, 0.15508E+01_JPRB, 0.14689E+01_JPRB ,&
&  0.14223E+01_JPRB, 0.14003E+01_JPRB, 0.13454E+01_JPRB, 0.13741E+01_JPRB, 0.14595E+01_JPRB, 0.14800E+01_JPRB, 0.11741E+01_JPRB ,&
&  0.92749E+00_JPRB, 0.92621E+00_JPRB, 0.80397E+00_JPRB, 0.69864E+00_JPRB, 0.60734E+00_JPRB, 0.60724E+00_JPRB, 0.60060E+00_JPRB ,&
&  0.59219E+00_JPRB, 0.59397E+00_JPRB, 0.58626E+00_JPRB, 0.63509E+00_JPRB, 0.73154E+00_JPRB, 0.84374E+00_JPRB, 0.91272E+00_JPRB ,&
&  0.83745E+00_JPRB, 0.88299E+00_JPRB, 0.85705E+00_JPRB, 0.82085E+00_JPRB, 0.84717E+00_JPRB, 0.84037E+00_JPRB, 0.80454E+00_JPRB ,&
&  0.75575E+00_JPRB, 0.71790E+00_JPRB, 0.68447E+00_JPRB, 0.65322E+00_JPRB, 0.62805E+00_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.69345E+00_JPRB, 0.67182E+00_JPRB, 0.65596E+00_JPRB, 0.64771E+00_JPRB, 0.64910E+00_JPRB, 0.65609E+00_JPRB, 0.64961E+00_JPRB ,&
&  0.62991E+00_JPRB, 0.60876E+00_JPRB, 0.59454E+00_JPRB, 0.58369E+00_JPRB, 0.56982E+00_JPRB, 0.55705E+00_JPRB, 0.54778E+00_JPRB ,&
&  0.54130E+00_JPRB, 0.44159E+00_JPRB, 0.34654E+00_JPRB, 0.32892E+00_JPRB, 0.32233E+00_JPRB, 0.31082E+00_JPRB, 0.33144E+00_JPRB ,&
&  0.36640E+00_JPRB, 0.41817E+00_JPRB, 0.48249E+00_JPRB, 0.60149E+00_JPRB, 0.67060E+00_JPRB, 0.72019E+00_JPRB, 0.75069E+00_JPRB ,&
&  0.71665E+00_JPRB, 0.70202E+00_JPRB, 0.73306E+00_JPRB, 0.72873E+00_JPRB, 0.71157E+00_JPRB, 0.71505E+00_JPRB, 0.69710E+00_JPRB ,&
&  0.69374E+00_JPRB, 0.71007E+00_JPRB, 0.74095E+00_JPRB, 0.78657E+00_JPRB, 0.84744E+00_JPRB, 0.91845E+00_JPRB, 0.10142E+01_JPRB ,&
&  0.10259E+01_JPRB, 0.98912E+00_JPRB, 0.88757E+00_JPRB, 0.88611E+00_JPRB, 0.81953E+00_JPRB, 0.82645E+00_JPRB, 0.82311E+00_JPRB ,&
&  0.10121E+01_JPRB, 0.10649E+01_JPRB, 0.10350E+01_JPRB, 0.12464E+01_JPRB, 0.12318E+01_JPRB, 0.13602E+01_JPRB, 0.16898E+01_JPRB ,&
&  0.17852E+01_JPRB, 0.17981E+01_JPRB, 0.18322E+01_JPRB, 0.14661E+01_JPRB, 0.12694E+01_JPRB, 0.17211E+01_JPRB, 0.18096E+01_JPRB ,&
&  0.18749E+01_JPRB, 0.19572E+01_JPRB, 0.19542E+01_JPRB, 0.20399E+01_JPRB, 0.17033E+01_JPRB, 0.13638E+01_JPRB, 0.10055E+01_JPRB ,&
&  0.92940E+00_JPRB, 0.84898E+00_JPRB, 0.90064E+00_JPRB, 0.80363E+00_JPRB, 0.77413E+00_JPRB, 0.72161E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68046E+00_JPRB, 0.67402E+00_JPRB, 0.71330E+00_JPRB, 0.78707E+00_JPRB, 0.90147E+00_JPRB, 0.91442E+00_JPRB, 0.10367E+01_JPRB ,&
&  0.10775E+01_JPRB, 0.10467E+01_JPRB, 0.94996E+00_JPRB, 0.95110E+00_JPRB, 0.92080E+00_JPRB, 0.90175E+00_JPRB, 0.87610E+00_JPRB ,&
&  0.84020E+00_JPRB, 0.80381E+00_JPRB, 0.77441E+00_JPRB, 0.74501E+00_JPRB, 0.71816E+00_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.76078E+00_JPRB, 0.73795E+00_JPRB, 0.71924E+00_JPRB, 0.70931E+00_JPRB, 0.70401E+00_JPRB, 0.69465E+00_JPRB, 0.67105E+00_JPRB ,&
&  0.63998E+00_JPRB, 0.61317E+00_JPRB, 0.58918E+00_JPRB, 0.57509E+00_JPRB, 0.55825E+00_JPRB, 0.55551E+00_JPRB, 0.55936E+00_JPRB ,&
&  0.55351E+00_JPRB, 0.48672E+00_JPRB, 0.37260E+00_JPRB, 0.33910E+00_JPRB, 0.34340E+00_JPRB, 0.32811E+00_JPRB, 0.32936E+00_JPRB ,&
&  0.38254E+00_JPRB, 0.45647E+00_JPRB, 0.54640E+00_JPRB, 0.70436E+00_JPRB, 0.78628E+00_JPRB, 0.86838E+00_JPRB, 0.89987E+00_JPRB ,&
&  0.90698E+00_JPRB, 0.80659E+00_JPRB, 0.74764E+00_JPRB, 0.70007E+00_JPRB, 0.66387E+00_JPRB, 0.64361E+00_JPRB, 0.65757E+00_JPRB ,&
&  0.67790E+00_JPRB, 0.72796E+00_JPRB, 0.78593E+00_JPRB, 0.81084E+00_JPRB, 0.87191E+00_JPRB, 0.91710E+00_JPRB, 0.96934E+00_JPRB ,&
&  0.98065E+00_JPRB, 0.95442E+00_JPRB, 0.92651E+00_JPRB, 0.96842E+00_JPRB, 0.94845E+00_JPRB, 0.88937E+00_JPRB, 0.91204E+00_JPRB ,&
&  0.10197E+01_JPRB, 0.10288E+01_JPRB, 0.10689E+01_JPRB, 0.12251E+01_JPRB, 0.14441E+01_JPRB, 0.15032E+01_JPRB, 0.17222E+01_JPRB ,&
&  0.14530E+01_JPRB, 0.12849E+01_JPRB, 0.11190E+01_JPRB, 0.98332E+00_JPRB, 0.12841E+01_JPRB, 0.21171E+01_JPRB, 0.22447E+01_JPRB ,&
&  0.22566E+01_JPRB, 0.23972E+01_JPRB, 0.24480E+01_JPRB, 0.21156E+01_JPRB, 0.16805E+01_JPRB, 0.13040E+01_JPRB, 0.15043E+01_JPRB ,&
&  0.14623E+01_JPRB, 0.15247E+01_JPRB, 0.13395E+01_JPRB, 0.10457E+01_JPRB, 0.83771E+00_JPRB, 0.91119E+00_JPRB, 0.87728E+00_JPRB ,&
&  0.85031E+00_JPRB, 0.85073E+00_JPRB, 0.10505E+01_JPRB, 0.10635E+01_JPRB, 0.10743E+01_JPRB, 0.11626E+01_JPRB, 0.12449E+01_JPRB ,&
&  0.11433E+01_JPRB, 0.11107E+01_JPRB, 0.10745E+01_JPRB, 0.10163E+01_JPRB, 0.97830E+00_JPRB, 0.93697E+00_JPRB, 0.89817E+00_JPRB ,&
&  0.87118E+00_JPRB, 0.84912E+00_JPRB, 0.82338E+00_JPRB, 0.80447E+00_JPRB, 0.78264E+00_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.80990E+00_JPRB, 0.78747E+00_JPRB, 0.77018E+00_JPRB, 0.75384E+00_JPRB, 0.73668E+00_JPRB, 0.71184E+00_JPRB, 0.67982E+00_JPRB ,&
&  0.64575E+00_JPRB, 0.61342E+00_JPRB, 0.59000E+00_JPRB, 0.57407E+00_JPRB, 0.55707E+00_JPRB, 0.55234E+00_JPRB, 0.55653E+00_JPRB ,&
&  0.55762E+00_JPRB, 0.56665E+00_JPRB, 0.42833E+00_JPRB, 0.36844E+00_JPRB, 0.36426E+00_JPRB, 0.35848E+00_JPRB, 0.34881E+00_JPRB ,&
&  0.41880E+00_JPRB, 0.51398E+00_JPRB, 0.59033E+00_JPRB, 0.75193E+00_JPRB, 0.94600E+00_JPRB, 0.91262E+00_JPRB, 0.88082E+00_JPRB ,&
&  0.88156E+00_JPRB, 0.81258E+00_JPRB, 0.74341E+00_JPRB, 0.70231E+00_JPRB, 0.67547E+00_JPRB, 0.67633E+00_JPRB, 0.69827E+00_JPRB ,&
&  0.74824E+00_JPRB, 0.81699E+00_JPRB, 0.82945E+00_JPRB, 0.85751E+00_JPRB, 0.88063E+00_JPRB, 0.94422E+00_JPRB, 0.10064E+01_JPRB ,&
&  0.97662E+00_JPRB, 0.94958E+00_JPRB, 0.97647E+00_JPRB, 0.10764E+01_JPRB, 0.10676E+01_JPRB, 0.10116E+01_JPRB, 0.11360E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.12222E+01_JPRB, 0.12008E+01_JPRB, 0.13755E+01_JPRB, 0.14880E+01_JPRB, 0.14708E+01_JPRB, 0.15467E+01_JPRB ,&
&  0.12324E+01_JPRB, 0.11020E+01_JPRB, 0.11481E+01_JPRB, 0.12803E+01_JPRB, 0.14716E+01_JPRB, 0.18343E+01_JPRB, 0.27268E+01_JPRB ,&
&  0.22983E+01_JPRB, 0.24414E+01_JPRB, 0.24445E+01_JPRB, 0.22943E+01_JPRB, 0.12482E+01_JPRB, 0.98491E+00_JPRB, 0.15267E+01_JPRB ,&
&  0.16528E+01_JPRB, 0.10422E+01_JPRB, 0.72664E+00_JPRB, 0.84403E+00_JPRB, 0.65888E+00_JPRB, 0.76610E+00_JPRB, 0.98837E+00_JPRB ,&
&  0.10568E+01_JPRB, 0.11187E+01_JPRB, 0.13782E+01_JPRB, 0.12448E+01_JPRB, 0.12076E+01_JPRB, 0.15684E+01_JPRB, 0.14144E+01_JPRB ,&
&  0.13060E+01_JPRB, 0.12135E+01_JPRB, 0.11960E+01_JPRB, 0.10683E+01_JPRB, 0.99324E+00_JPRB, 0.94920E+00_JPRB, 0.92663E+00_JPRB ,&
&  0.91744E+00_JPRB, 0.90540E+00_JPRB, 0.88957E+00_JPRB, 0.86526E+00_JPRB, 0.83516E+00_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.83184E+00_JPRB, 0.81553E+00_JPRB, 0.80291E+00_JPRB, 0.78677E+00_JPRB, 0.76176E+00_JPRB, 0.72687E+00_JPRB, 0.69001E+00_JPRB ,&
&  0.65939E+00_JPRB, 0.62253E+00_JPRB, 0.59607E+00_JPRB, 0.57757E+00_JPRB, 0.56229E+00_JPRB, 0.55450E+00_JPRB, 0.55329E+00_JPRB ,&
&  0.54623E+00_JPRB, 0.54378E+00_JPRB, 0.50557E+00_JPRB, 0.44695E+00_JPRB, 0.43313E+00_JPRB, 0.42607E+00_JPRB, 0.42939E+00_JPRB ,&
&  0.50565E+00_JPRB, 0.59782E+00_JPRB, 0.68785E+00_JPRB, 0.80398E+00_JPRB, 0.89683E+00_JPRB, 0.85355E+00_JPRB, 0.88016E+00_JPRB ,&
&  0.86547E+00_JPRB, 0.82378E+00_JPRB, 0.76407E+00_JPRB, 0.74152E+00_JPRB, 0.74101E+00_JPRB, 0.75436E+00_JPRB, 0.77960E+00_JPRB ,&
&  0.81998E+00_JPRB, 0.85748E+00_JPRB, 0.85244E+00_JPRB, 0.83831E+00_JPRB, 0.89003E+00_JPRB, 0.98184E+00_JPRB, 0.10310E+01_JPRB ,&
&  0.99999E+00_JPRB, 0.10050E+01_JPRB, 0.10338E+01_JPRB, 0.11243E+01_JPRB, 0.12485E+01_JPRB, 0.12456E+01_JPRB, 0.10767E+01_JPRB ,&
&  0.97239E+00_JPRB, 0.10511E+01_JPRB, 0.13210E+01_JPRB, 0.13250E+01_JPRB, 0.13539E+01_JPRB, 0.13952E+01_JPRB, 0.14169E+01_JPRB ,&
&  0.13852E+01_JPRB, 0.14012E+01_JPRB, 0.14194E+01_JPRB, 0.16406E+01_JPRB, 0.18786E+01_JPRB, 0.19114E+01_JPRB, 0.26710E+01_JPRB ,&
&  0.26595E+01_JPRB, 0.24227E+01_JPRB, 0.18511E+01_JPRB, 0.13773E+01_JPRB, 0.13563E+01_JPRB, 0.10741E+01_JPRB, 0.73114E+00_JPRB ,&
&  0.69328E+00_JPRB, 0.65434E+00_JPRB, 0.57756E+00_JPRB, 0.56991E+00_JPRB, 0.53991E+00_JPRB, 0.68509E+00_JPRB, 0.98175E+00_JPRB ,&
&  0.13297E+01_JPRB, 0.15534E+01_JPRB, 0.15845E+01_JPRB, 0.14552E+01_JPRB, 0.13380E+01_JPRB, 0.17102E+01_JPRB, 0.16090E+01_JPRB ,&
&  0.12763E+01_JPRB, 0.12227E+01_JPRB, 0.11534E+01_JPRB, 0.10619E+01_JPRB, 0.10151E+01_JPRB, 0.97750E+00_JPRB, 0.95467E+00_JPRB ,&
&  0.93560E+00_JPRB, 0.91971E+00_JPRB, 0.90190E+00_JPRB, 0.87608E+00_JPRB, 0.85306E+00_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.85139E+00_JPRB, 0.83883E+00_JPRB, 0.82313E+00_JPRB, 0.80299E+00_JPRB, 0.77383E+00_JPRB, 0.74073E+00_JPRB, 0.70925E+00_JPRB ,&
&  0.68420E+00_JPRB, 0.64884E+00_JPRB, 0.61703E+00_JPRB, 0.59269E+00_JPRB, 0.57417E+00_JPRB, 0.56248E+00_JPRB, 0.55882E+00_JPRB ,&
&  0.55660E+00_JPRB, 0.54714E+00_JPRB, 0.55922E+00_JPRB, 0.53292E+00_JPRB, 0.56505E+00_JPRB, 0.57883E+00_JPRB, 0.60866E+00_JPRB ,&
&  0.67518E+00_JPRB, 0.76374E+00_JPRB, 0.88263E+00_JPRB, 0.93963E+00_JPRB, 0.99059E+00_JPRB, 0.97869E+00_JPRB, 0.94768E+00_JPRB ,&
&  0.91085E+00_JPRB, 0.89844E+00_JPRB, 0.85994E+00_JPRB, 0.84098E+00_JPRB, 0.82574E+00_JPRB, 0.82295E+00_JPRB, 0.83497E+00_JPRB ,&
&  0.85632E+00_JPRB, 0.88178E+00_JPRB, 0.84320E+00_JPRB, 0.85266E+00_JPRB, 0.92247E+00_JPRB, 0.10018E+01_JPRB, 0.10533E+01_JPRB ,&
&  0.10679E+01_JPRB, 0.10761E+01_JPRB, 0.11179E+01_JPRB, 0.12024E+01_JPRB, 0.11250E+01_JPRB, 0.96293E+00_JPRB, 0.10552E+01_JPRB ,&
&  0.11357E+01_JPRB, 0.13016E+01_JPRB, 0.12857E+01_JPRB, 0.15037E+01_JPRB, 0.14709E+01_JPRB, 0.13808E+01_JPRB, 0.14797E+01_JPRB ,&
&  0.15879E+01_JPRB, 0.19529E+01_JPRB, 0.16865E+01_JPRB, 0.16962E+01_JPRB, 0.23946E+01_JPRB, 0.22448E+01_JPRB, 0.24000E+01_JPRB ,&
&  0.25954E+01_JPRB, 0.21427E+01_JPRB, 0.17211E+01_JPRB, 0.14922E+01_JPRB, 0.19260E+01_JPRB, 0.18039E+01_JPRB, 0.92214E+00_JPRB ,&
&  0.74894E+00_JPRB, 0.66038E+00_JPRB, 0.60074E+00_JPRB, 0.55295E+00_JPRB, 0.53242E+00_JPRB, 0.72262E+00_JPRB, 0.12659E+01_JPRB ,&
&  0.13322E+01_JPRB, 0.19321E+01_JPRB, 0.19385E+01_JPRB, 0.17613E+01_JPRB, 0.15718E+01_JPRB, 0.17429E+01_JPRB, 0.16222E+01_JPRB ,&
&  0.13529E+01_JPRB, 0.12853E+01_JPRB, 0.11502E+01_JPRB, 0.10582E+01_JPRB, 0.98874E+00_JPRB, 0.94991E+00_JPRB, 0.93588E+00_JPRB ,&
&  0.92207E+00_JPRB, 0.90806E+00_JPRB, 0.89538E+00_JPRB, 0.88054E+00_JPRB, 0.86632E+00_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.85401E+00_JPRB, 0.83219E+00_JPRB, 0.81761E+00_JPRB, 0.80118E+00_JPRB, 0.78327E+00_JPRB, 0.76076E+00_JPRB, 0.73265E+00_JPRB ,&
&  0.70723E+00_JPRB, 0.67166E+00_JPRB, 0.63853E+00_JPRB, 0.61017E+00_JPRB, 0.58746E+00_JPRB, 0.57176E+00_JPRB, 0.56698E+00_JPRB ,&
&  0.56316E+00_JPRB, 0.56330E+00_JPRB, 0.57360E+00_JPRB, 0.61969E+00_JPRB, 0.68656E+00_JPRB, 0.71182E+00_JPRB, 0.80669E+00_JPRB ,&
&  0.93405E+00_JPRB, 0.10572E+01_JPRB, 0.11943E+01_JPRB, 0.12173E+01_JPRB, 0.12068E+01_JPRB, 0.11504E+01_JPRB, 0.10922E+01_JPRB ,&
&  0.10409E+01_JPRB, 0.10371E+01_JPRB, 0.98302E+00_JPRB, 0.93879E+00_JPRB, 0.90474E+00_JPRB, 0.89137E+00_JPRB, 0.89125E+00_JPRB ,&
&  0.91897E+00_JPRB, 0.88355E+00_JPRB, 0.85371E+00_JPRB, 0.87943E+00_JPRB, 0.94508E+00_JPRB, 0.10345E+01_JPRB, 0.10963E+01_JPRB ,&
&  0.11165E+01_JPRB, 0.11131E+01_JPRB, 0.11514E+01_JPRB, 0.12209E+01_JPRB, 0.10145E+01_JPRB, 0.11564E+01_JPRB, 0.11596E+01_JPRB ,&
&  0.11809E+01_JPRB, 0.12437E+01_JPRB, 0.12355E+01_JPRB, 0.14356E+01_JPRB, 0.14859E+01_JPRB, 0.13980E+01_JPRB, 0.14472E+01_JPRB ,&
&  0.17793E+01_JPRB, 0.17061E+01_JPRB, 0.16293E+01_JPRB, 0.18985E+01_JPRB, 0.23596E+01_JPRB, 0.32684E+01_JPRB, 0.21254E+01_JPRB ,&
&  0.18063E+01_JPRB, 0.19851E+01_JPRB, 0.17055E+01_JPRB, 0.15224E+01_JPRB, 0.18218E+01_JPRB, 0.19960E+01_JPRB, 0.16443E+01_JPRB ,&
&  0.83926E+00_JPRB, 0.64872E+00_JPRB, 0.66811E+00_JPRB, 0.68218E+00_JPRB, 0.65310E+00_JPRB, 0.85415E+00_JPRB, 0.18613E+01_JPRB ,&
&  0.16134E+01_JPRB, 0.20937E+01_JPRB, 0.20331E+01_JPRB, 0.17967E+01_JPRB, 0.16211E+01_JPRB, 0.16449E+01_JPRB, 0.15839E+01_JPRB ,&
&  0.12939E+01_JPRB, 0.11652E+01_JPRB, 0.10530E+01_JPRB, 0.98114E+00_JPRB, 0.94839E+00_JPRB, 0.92825E+00_JPRB, 0.91957E+00_JPRB ,&
&  0.90881E+00_JPRB, 0.89672E+00_JPRB, 0.89063E+00_JPRB, 0.88596E+00_JPRB, 0.87493E+00_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.82952E+00_JPRB, 0.81638E+00_JPRB, 0.80689E+00_JPRB, 0.80362E+00_JPRB, 0.79425E+00_JPRB, 0.76490E+00_JPRB, 0.73499E+00_JPRB ,&
&  0.70769E+00_JPRB, 0.67248E+00_JPRB, 0.64410E+00_JPRB, 0.61494E+00_JPRB, 0.59331E+00_JPRB, 0.57865E+00_JPRB, 0.57242E+00_JPRB ,&
&  0.57669E+00_JPRB, 0.57906E+00_JPRB, 0.59161E+00_JPRB, 0.66944E+00_JPRB, 0.78749E+00_JPRB, 0.82102E+00_JPRB, 0.92690E+00_JPRB ,&
&  0.13246E+01_JPRB, 0.15313E+01_JPRB, 0.14754E+01_JPRB, 0.14528E+01_JPRB, 0.14461E+01_JPRB, 0.14105E+01_JPRB, 0.13258E+01_JPRB ,&
&  0.12094E+01_JPRB, 0.11420E+01_JPRB, 0.10412E+01_JPRB, 0.97911E+00_JPRB, 0.94365E+00_JPRB, 0.92920E+00_JPRB, 0.93557E+00_JPRB ,&
&  0.91923E+00_JPRB, 0.89187E+00_JPRB, 0.87389E+00_JPRB, 0.89778E+00_JPRB, 0.97349E+00_JPRB, 0.10578E+01_JPRB, 0.11255E+01_JPRB ,&
&  0.11374E+01_JPRB, 0.11316E+01_JPRB, 0.11425E+01_JPRB, 0.11265E+01_JPRB, 0.11506E+01_JPRB, 0.11779E+01_JPRB, 0.12189E+01_JPRB ,&
&  0.12206E+01_JPRB, 0.13024E+01_JPRB, 0.13928E+01_JPRB, 0.14214E+01_JPRB, 0.14333E+01_JPRB, 0.14384E+01_JPRB, 0.15051E+01_JPRB ,&
&  0.17179E+01_JPRB, 0.17268E+01_JPRB, 0.17923E+01_JPRB, 0.22335E+01_JPRB, 0.25189E+01_JPRB, 0.28315E+01_JPRB, 0.25621E+01_JPRB ,&
&  0.20801E+01_JPRB, 0.15918E+01_JPRB, 0.14541E+01_JPRB, 0.15447E+01_JPRB, 0.17105E+01_JPRB, 0.17432E+01_JPRB, 0.22339E+01_JPRB ,&
&  0.21979E+01_JPRB, 0.15285E+01_JPRB, 0.13045E+01_JPRB, 0.16098E+01_JPRB, 0.13845E+01_JPRB, 0.12472E+01_JPRB, 0.15325E+01_JPRB ,&
&  0.17387E+01_JPRB, 0.20572E+01_JPRB, 0.19165E+01_JPRB, 0.16252E+01_JPRB, 0.16029E+01_JPRB, 0.15254E+01_JPRB, 0.13930E+01_JPRB ,&
&  0.11835E+01_JPRB, 0.10971E+01_JPRB, 0.10350E+01_JPRB, 0.97771E+00_JPRB, 0.93688E+00_JPRB, 0.91780E+00_JPRB, 0.90694E+00_JPRB ,&
&  0.88844E+00_JPRB, 0.87856E+00_JPRB, 0.87390E+00_JPRB, 0.86481E+00_JPRB, 0.84626E+00_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.80633E+00_JPRB, 0.80503E+00_JPRB, 0.80945E+00_JPRB, 0.78531E+00_JPRB, 0.75358E+00_JPRB, 0.72459E+00_JPRB, 0.70982E+00_JPRB ,&
&  0.69107E+00_JPRB, 0.66161E+00_JPRB, 0.63335E+00_JPRB, 0.61097E+00_JPRB, 0.59021E+00_JPRB, 0.57965E+00_JPRB, 0.57698E+00_JPRB ,&
&  0.58525E+00_JPRB, 0.59925E+00_JPRB, 0.65199E+00_JPRB, 0.76768E+00_JPRB, 0.90886E+00_JPRB, 0.11311E+01_JPRB, 0.10295E+01_JPRB ,&
&  0.13905E+01_JPRB, 0.18388E+01_JPRB, 0.17949E+01_JPRB, 0.17279E+01_JPRB, 0.16571E+01_JPRB, 0.15897E+01_JPRB, 0.14285E+01_JPRB ,&
&  0.12743E+01_JPRB, 0.11888E+01_JPRB, 0.10841E+01_JPRB, 0.10161E+01_JPRB, 0.99009E+00_JPRB, 0.97722E+00_JPRB, 0.94342E+00_JPRB ,&
&  0.91675E+00_JPRB, 0.89136E+00_JPRB, 0.87954E+00_JPRB, 0.90504E+00_JPRB, 0.96075E+00_JPRB, 0.10518E+01_JPRB, 0.11117E+01_JPRB ,&
&  0.11383E+01_JPRB, 0.11382E+01_JPRB, 0.11320E+01_JPRB, 0.10759E+01_JPRB, 0.11191E+01_JPRB, 0.11617E+01_JPRB, 0.12111E+01_JPRB ,&
&  0.11869E+01_JPRB, 0.11480E+01_JPRB, 0.12965E+01_JPRB, 0.14246E+01_JPRB, 0.14556E+01_JPRB, 0.14057E+01_JPRB, 0.14597E+01_JPRB ,&
&  0.17457E+01_JPRB, 0.20018E+01_JPRB, 0.24498E+01_JPRB, 0.25771E+01_JPRB, 0.26432E+01_JPRB, 0.26687E+01_JPRB, 0.25688E+01_JPRB ,&
&  0.24750E+01_JPRB, 0.19748E+01_JPRB, 0.17223E+01_JPRB, 0.16164E+01_JPRB, 0.15919E+01_JPRB, 0.16456E+01_JPRB, 0.20293E+01_JPRB ,&
&  0.23837E+01_JPRB, 0.25666E+01_JPRB, 0.26028E+01_JPRB, 0.22376E+01_JPRB, 0.19529E+01_JPRB, 0.16763E+01_JPRB, 0.15941E+01_JPRB ,&
&  0.18842E+01_JPRB, 0.19524E+01_JPRB, 0.17902E+01_JPRB, 0.19433E+01_JPRB, 0.15936E+01_JPRB, 0.15098E+01_JPRB, 0.13596E+01_JPRB ,&
&  0.12188E+01_JPRB, 0.11200E+01_JPRB, 0.10392E+01_JPRB, 0.98550E+00_JPRB, 0.95818E+00_JPRB, 0.91194E+00_JPRB, 0.87668E+00_JPRB ,&
&  0.85219E+00_JPRB, 0.84326E+00_JPRB, 0.83366E+00_JPRB, 0.82518E+00_JPRB, 0.81190E+00_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.81686E+00_JPRB, 0.81742E+00_JPRB, 0.80797E+00_JPRB, 0.77383E+00_JPRB, 0.74840E+00_JPRB, 0.73032E+00_JPRB, 0.69497E+00_JPRB ,&
&  0.67104E+00_JPRB, 0.64967E+00_JPRB, 0.62515E+00_JPRB, 0.60460E+00_JPRB, 0.59340E+00_JPRB, 0.58076E+00_JPRB, 0.58606E+00_JPRB ,&
&  0.62446E+00_JPRB, 0.68934E+00_JPRB, 0.80246E+00_JPRB, 0.91115E+00_JPRB, 0.10627E+01_JPRB, 0.13301E+01_JPRB, 0.14625E+01_JPRB ,&
&  0.15030E+01_JPRB, 0.21922E+01_JPRB, 0.19485E+01_JPRB, 0.18245E+01_JPRB, 0.17215E+01_JPRB, 0.15892E+01_JPRB, 0.14081E+01_JPRB ,&
&  0.12312E+01_JPRB, 0.11677E+01_JPRB, 0.11200E+01_JPRB, 0.10637E+01_JPRB, 0.10399E+01_JPRB, 0.96838E+00_JPRB, 0.95456E+00_JPRB ,&
&  0.93620E+00_JPRB, 0.92111E+00_JPRB, 0.90310E+00_JPRB, 0.93219E+00_JPRB, 0.99446E+00_JPRB, 0.10612E+01_JPRB, 0.11230E+01_JPRB ,&
&  0.11455E+01_JPRB, 0.11379E+01_JPRB, 0.10938E+01_JPRB, 0.10749E+01_JPRB, 0.10802E+01_JPRB, 0.11293E+01_JPRB, 0.11881E+01_JPRB ,&
&  0.12021E+01_JPRB, 0.11775E+01_JPRB, 0.12631E+01_JPRB, 0.12912E+01_JPRB, 0.13022E+01_JPRB, 0.14136E+01_JPRB, 0.14718E+01_JPRB ,&
&  0.17883E+01_JPRB, 0.18428E+01_JPRB, 0.26003E+01_JPRB, 0.24116E+01_JPRB, 0.23628E+01_JPRB, 0.24383E+01_JPRB, 0.22534E+01_JPRB ,&
&  0.19095E+01_JPRB, 0.17598E+01_JPRB, 0.15876E+01_JPRB, 0.15389E+01_JPRB, 0.15862E+01_JPRB, 0.17349E+01_JPRB, 0.20232E+01_JPRB ,&
&  0.20872E+01_JPRB, 0.20278E+01_JPRB, 0.20558E+01_JPRB, 0.17650E+01_JPRB, 0.16987E+01_JPRB, 0.15832E+01_JPRB, 0.15554E+01_JPRB ,&
&  0.18630E+01_JPRB, 0.19181E+01_JPRB, 0.19206E+01_JPRB, 0.18332E+01_JPRB, 0.14983E+01_JPRB, 0.13867E+01_JPRB, 0.12453E+01_JPRB ,&
&  0.11409E+01_JPRB, 0.10575E+01_JPRB, 0.10092E+01_JPRB, 0.96422E+00_JPRB, 0.91733E+00_JPRB, 0.87094E+00_JPRB, 0.84208E+00_JPRB ,&
&  0.82063E+00_JPRB, 0.81086E+00_JPRB, 0.81201E+00_JPRB, 0.81052E+00_JPRB, 0.81124E+00_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.86474E+00_JPRB, 0.87678E+00_JPRB, 0.88501E+00_JPRB, 0.91916E+00_JPRB, 0.96594E+00_JPRB, 0.10336E+01_JPRB, 0.10666E+01_JPRB ,&
&  0.92540E+00_JPRB, 0.63923E+00_JPRB, 0.62215E+00_JPRB, 0.60964E+00_JPRB, 0.59897E+00_JPRB, 0.60347E+00_JPRB, 0.64643E+00_JPRB ,&
&  0.71208E+00_JPRB, 0.79614E+00_JPRB, 0.89289E+00_JPRB, 0.10296E+01_JPRB, 0.12253E+01_JPRB, 0.15972E+01_JPRB, 0.18947E+01_JPRB ,&
&  0.17630E+01_JPRB, 0.18868E+01_JPRB, 0.19996E+01_JPRB, 0.17675E+01_JPRB, 0.16228E+01_JPRB, 0.15110E+01_JPRB, 0.14193E+01_JPRB ,&
&  0.12726E+01_JPRB, 0.11973E+01_JPRB, 0.11740E+01_JPRB, 0.11524E+01_JPRB, 0.10031E+01_JPRB, 0.98332E+00_JPRB, 0.97958E+00_JPRB ,&
&  0.97702E+00_JPRB, 0.98010E+00_JPRB, 0.98312E+00_JPRB, 0.10222E+01_JPRB, 0.10738E+01_JPRB, 0.11318E+01_JPRB, 0.11756E+01_JPRB ,&
&  0.11872E+01_JPRB, 0.11769E+01_JPRB, 0.11459E+01_JPRB, 0.10777E+01_JPRB, 0.10779E+01_JPRB, 0.11139E+01_JPRB, 0.11626E+01_JPRB ,&
&  0.11953E+01_JPRB, 0.12454E+01_JPRB, 0.12767E+01_JPRB, 0.12819E+01_JPRB, 0.12650E+01_JPRB, 0.14652E+01_JPRB, 0.15785E+01_JPRB ,&
&  0.17844E+01_JPRB, 0.20386E+01_JPRB, 0.27960E+01_JPRB, 0.20721E+01_JPRB, 0.21213E+01_JPRB, 0.21629E+01_JPRB, 0.20068E+01_JPRB ,&
&  0.17209E+01_JPRB, 0.16060E+01_JPRB, 0.14745E+01_JPRB, 0.14554E+01_JPRB, 0.15880E+01_JPRB, 0.17299E+01_JPRB, 0.20713E+01_JPRB ,&
&  0.17815E+01_JPRB, 0.18168E+01_JPRB, 0.16252E+01_JPRB, 0.16117E+01_JPRB, 0.15541E+01_JPRB, 0.14683E+01_JPRB, 0.14098E+01_JPRB ,&
&  0.16711E+01_JPRB, 0.16445E+01_JPRB, 0.15449E+01_JPRB, 0.14368E+01_JPRB, 0.12125E+01_JPRB, 0.11180E+01_JPRB, 0.10218E+01_JPRB ,&
&  0.97883E+00_JPRB, 0.96216E+00_JPRB, 0.95779E+00_JPRB, 0.92483E+00_JPRB, 0.90020E+00_JPRB, 0.86962E+00_JPRB, 0.84936E+00_JPRB ,&
&  0.83612E+00_JPRB, 0.83151E+00_JPRB, 0.83214E+00_JPRB, 0.84016E+00_JPRB, 0.85068E+00_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.93873E+00_JPRB, 0.92976E+00_JPRB, 0.92423E+00_JPRB, 0.92009E+00_JPRB, 0.91009E+00_JPRB, 0.87947E+00_JPRB, 0.81620E+00_JPRB ,&
&  0.72741E+00_JPRB, 0.65797E+00_JPRB, 0.64385E+00_JPRB, 0.63821E+00_JPRB, 0.64370E+00_JPRB, 0.67314E+00_JPRB, 0.72603E+00_JPRB ,&
&  0.78254E+00_JPRB, 0.87745E+00_JPRB, 0.10098E+01_JPRB, 0.11807E+01_JPRB, 0.14108E+01_JPRB, 0.17693E+01_JPRB, 0.21220E+01_JPRB ,&
&  0.20640E+01_JPRB, 0.15076E+01_JPRB, 0.17145E+01_JPRB, 0.16857E+01_JPRB, 0.15687E+01_JPRB, 0.14435E+01_JPRB, 0.13178E+01_JPRB ,&
&  0.11940E+01_JPRB, 0.11526E+01_JPRB, 0.11283E+01_JPRB, 0.10849E+01_JPRB, 0.10241E+01_JPRB, 0.10424E+01_JPRB, 0.10651E+01_JPRB ,&
&  0.10879E+01_JPRB, 0.11048E+01_JPRB, 0.11071E+01_JPRB, 0.11454E+01_JPRB, 0.11779E+01_JPRB, 0.12316E+01_JPRB, 0.12922E+01_JPRB ,&
&  0.13102E+01_JPRB, 0.13076E+01_JPRB, 0.12890E+01_JPRB, 0.12106E+01_JPRB, 0.11913E+01_JPRB, 0.12341E+01_JPRB, 0.12746E+01_JPRB ,&
&  0.13018E+01_JPRB, 0.12923E+01_JPRB, 0.13471E+01_JPRB, 0.14771E+01_JPRB, 0.16396E+01_JPRB, 0.15637E+01_JPRB, 0.17963E+01_JPRB ,&
&  0.19632E+01_JPRB, 0.22848E+01_JPRB, 0.22286E+01_JPRB, 0.25247E+01_JPRB, 0.16721E+01_JPRB, 0.17955E+01_JPRB, 0.16009E+01_JPRB ,&
&  0.16056E+01_JPRB, 0.14533E+01_JPRB, 0.13981E+01_JPRB, 0.14759E+01_JPRB, 0.16609E+01_JPRB, 0.18760E+01_JPRB, 0.18453E+01_JPRB ,&
&  0.14793E+01_JPRB, 0.14075E+01_JPRB, 0.14395E+01_JPRB, 0.15490E+01_JPRB, 0.15245E+01_JPRB, 0.14650E+01_JPRB, 0.13139E+01_JPRB ,&
&  0.13805E+01_JPRB, 0.13513E+01_JPRB, 0.13005E+01_JPRB, 0.12079E+01_JPRB, 0.10131E+01_JPRB, 0.91434E+00_JPRB, 0.91740E+00_JPRB ,&
&  0.93739E+00_JPRB, 0.95547E+00_JPRB, 0.95806E+00_JPRB, 0.94894E+00_JPRB, 0.92454E+00_JPRB, 0.90273E+00_JPRB, 0.88960E+00_JPRB ,&
&  0.88187E+00_JPRB, 0.88978E+00_JPRB, 0.89966E+00_JPRB, 0.91030E+00_JPRB, 0.92664E+00_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.92891E+00_JPRB, 0.90566E+00_JPRB, 0.88138E+00_JPRB, 0.85860E+00_JPRB, 0.83812E+00_JPRB, 0.79118E+00_JPRB, 0.74166E+00_JPRB ,&
&  0.70945E+00_JPRB, 0.69274E+00_JPRB, 0.69268E+00_JPRB, 0.69904E+00_JPRB, 0.72267E+00_JPRB, 0.75087E+00_JPRB, 0.80175E+00_JPRB ,&
&  0.88055E+00_JPRB, 0.99264E+00_JPRB, 0.11473E+01_JPRB, 0.13365E+01_JPRB, 0.15748E+01_JPRB, 0.18467E+01_JPRB, 0.20023E+01_JPRB ,&
&  0.21384E+01_JPRB, 0.21304E+01_JPRB, 0.18577E+01_JPRB, 0.13665E+01_JPRB, 0.12512E+01_JPRB, 0.12956E+01_JPRB, 0.12161E+01_JPRB ,&
&  0.12012E+01_JPRB, 0.11684E+01_JPRB, 0.11319E+01_JPRB, 0.11284E+01_JPRB, 0.11326E+01_JPRB, 0.11671E+01_JPRB, 0.11963E+01_JPRB ,&
&  0.12182E+01_JPRB, 0.12367E+01_JPRB, 0.12631E+01_JPRB, 0.13211E+01_JPRB, 0.13667E+01_JPRB, 0.14241E+01_JPRB, 0.14699E+01_JPRB ,&
&  0.14963E+01_JPRB, 0.15338E+01_JPRB, 0.15183E+01_JPRB, 0.14715E+01_JPRB, 0.14267E+01_JPRB, 0.14666E+01_JPRB, 0.15190E+01_JPRB ,&
&  0.15631E+01_JPRB, 0.15743E+01_JPRB, 0.16095E+01_JPRB, 0.17783E+01_JPRB, 0.19282E+01_JPRB, 0.17918E+01_JPRB, 0.18687E+01_JPRB ,&
&  0.21657E+01_JPRB, 0.23807E+01_JPRB, 0.18362E+01_JPRB, 0.21486E+01_JPRB, 0.12913E+01_JPRB, 0.15472E+01_JPRB, 0.16083E+01_JPRB ,&
&  0.14588E+01_JPRB, 0.14154E+01_JPRB, 0.14187E+01_JPRB, 0.15470E+01_JPRB, 0.17268E+01_JPRB, 0.17923E+01_JPRB, 0.15697E+01_JPRB ,&
&  0.12952E+01_JPRB, 0.12513E+01_JPRB, 0.13000E+01_JPRB, 0.13631E+01_JPRB, 0.13627E+01_JPRB, 0.12689E+01_JPRB, 0.10439E+01_JPRB ,&
&  0.92276E+00_JPRB, 0.11572E+01_JPRB, 0.11883E+01_JPRB, 0.12335E+01_JPRB, 0.10375E+01_JPRB, 0.88921E+00_JPRB, 0.94235E+00_JPRB ,&
&  0.98570E+00_JPRB, 0.10075E+01_JPRB, 0.10057E+01_JPRB, 0.98906E+00_JPRB, 0.96477E+00_JPRB, 0.94487E+00_JPRB, 0.93664E+00_JPRB ,&
&  0.93583E+00_JPRB, 0.93195E+00_JPRB, 0.93714E+00_JPRB, 0.94446E+00_JPRB, 0.94480E+00_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.85575E+00_JPRB, 0.82781E+00_JPRB, 0.80670E+00_JPRB, 0.78937E+00_JPRB, 0.77841E+00_JPRB, 0.75559E+00_JPRB, 0.72639E+00_JPRB ,&
&  0.71105E+00_JPRB, 0.70506E+00_JPRB, 0.71488E+00_JPRB, 0.74184E+00_JPRB, 0.78142E+00_JPRB, 0.82128E+00_JPRB, 0.89692E+00_JPRB ,&
&  0.98738E+00_JPRB, 0.11217E+01_JPRB, 0.12943E+01_JPRB, 0.15392E+01_JPRB, 0.17388E+01_JPRB, 0.19194E+01_JPRB, 0.19491E+01_JPRB ,&
&  0.20264E+01_JPRB, 0.20281E+01_JPRB, 0.19131E+01_JPRB, 0.17707E+01_JPRB, 0.13996E+01_JPRB, 0.13928E+01_JPRB, 0.15315E+01_JPRB ,&
&  0.15837E+01_JPRB, 0.13794E+01_JPRB, 0.12475E+01_JPRB, 0.12565E+01_JPRB, 0.12598E+01_JPRB, 0.13021E+01_JPRB, 0.13410E+01_JPRB ,&
&  0.13760E+01_JPRB, 0.14140E+01_JPRB, 0.14509E+01_JPRB, 0.15091E+01_JPRB, 0.15700E+01_JPRB, 0.16031E+01_JPRB, 0.16428E+01_JPRB ,&
&  0.16592E+01_JPRB, 0.16873E+01_JPRB, 0.17064E+01_JPRB, 0.16934E+01_JPRB, 0.17320E+01_JPRB, 0.17957E+01_JPRB, 0.19076E+01_JPRB ,&
&  0.20193E+01_JPRB, 0.19996E+01_JPRB, 0.21291E+01_JPRB, 0.23425E+01_JPRB, 0.23730E+01_JPRB, 0.22900E+01_JPRB, 0.22832E+01_JPRB ,&
&  0.24994E+01_JPRB, 0.26889E+01_JPRB, 0.15220E+01_JPRB, 0.16559E+01_JPRB, 0.18762E+01_JPRB, 0.15996E+01_JPRB, 0.14365E+01_JPRB ,&
&  0.14076E+01_JPRB, 0.14008E+01_JPRB, 0.14574E+01_JPRB, 0.15445E+01_JPRB, 0.17027E+01_JPRB, 0.19887E+01_JPRB, 0.14241E+01_JPRB ,&
&  0.11227E+01_JPRB, 0.10620E+01_JPRB, 0.10820E+01_JPRB, 0.11134E+01_JPRB, 0.11134E+01_JPRB, 0.10120E+01_JPRB, 0.86314E+00_JPRB ,&
&  0.85012E+00_JPRB, 0.11780E+01_JPRB, 0.13168E+01_JPRB, 0.15033E+01_JPRB, 0.13876E+01_JPRB, 0.89911E+00_JPRB, 0.98060E+00_JPRB ,&
&  0.10070E+01_JPRB, 0.10123E+01_JPRB, 0.98033E+00_JPRB, 0.96801E+00_JPRB, 0.95858E+00_JPRB, 0.94747E+00_JPRB, 0.94573E+00_JPRB ,&
&  0.94000E+00_JPRB, 0.94194E+00_JPRB, 0.94566E+00_JPRB, 0.92177E+00_JPRB, 0.88496E+00_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.78669E+00_JPRB, 0.76378E+00_JPRB, 0.75114E+00_JPRB, 0.73485E+00_JPRB, 0.72714E+00_JPRB, 0.71839E+00_JPRB, 0.71484E+00_JPRB ,&
&  0.71338E+00_JPRB, 0.71643E+00_JPRB, 0.73377E+00_JPRB, 0.76402E+00_JPRB, 0.80783E+00_JPRB, 0.86731E+00_JPRB, 0.97219E+00_JPRB ,&
&  0.10766E+01_JPRB, 0.12352E+01_JPRB, 0.14263E+01_JPRB, 0.16446E+01_JPRB, 0.17943E+01_JPRB, 0.18332E+01_JPRB, 0.19276E+01_JPRB ,&
&  0.19892E+01_JPRB, 0.20040E+01_JPRB, 0.19687E+01_JPRB, 0.19043E+01_JPRB, 0.18058E+01_JPRB, 0.18055E+01_JPRB, 0.18820E+01_JPRB ,&
&  0.16987E+01_JPRB, 0.13648E+01_JPRB, 0.13228E+01_JPRB, 0.13537E+01_JPRB, 0.13930E+01_JPRB, 0.14200E+01_JPRB, 0.14712E+01_JPRB ,&
&  0.15150E+01_JPRB, 0.15498E+01_JPRB, 0.15879E+01_JPRB, 0.16325E+01_JPRB, 0.17104E+01_JPRB, 0.17568E+01_JPRB, 0.18214E+01_JPRB ,&
&  0.18713E+01_JPRB, 0.19277E+01_JPRB, 0.20338E+01_JPRB, 0.19380E+01_JPRB, 0.21155E+01_JPRB, 0.22491E+01_JPRB, 0.23728E+01_JPRB ,&
&  0.24173E+01_JPRB, 0.23988E+01_JPRB, 0.25863E+01_JPRB, 0.27332E+01_JPRB, 0.28447E+01_JPRB, 0.28779E+01_JPRB, 0.28034E+01_JPRB ,&
&  0.29080E+01_JPRB, 0.27036E+01_JPRB, 0.15071E+01_JPRB, 0.13159E+01_JPRB, 0.13824E+01_JPRB, 0.12235E+01_JPRB, 0.13926E+01_JPRB ,&
&  0.13790E+01_JPRB, 0.14006E+01_JPRB, 0.14376E+01_JPRB, 0.15197E+01_JPRB, 0.16058E+01_JPRB, 0.18425E+01_JPRB, 0.13211E+01_JPRB ,&
&  0.92312E+00_JPRB, 0.86078E+00_JPRB, 0.85688E+00_JPRB, 0.87713E+00_JPRB, 0.91124E+00_JPRB, 0.89414E+00_JPRB, 0.87917E+00_JPRB ,&
&  0.93043E+00_JPRB, 0.97427E+00_JPRB, 0.10169E+01_JPRB, 0.99216E+00_JPRB, 0.86793E+00_JPRB, 0.86471E+00_JPRB, 0.89809E+00_JPRB ,&
&  0.92054E+00_JPRB, 0.92896E+00_JPRB, 0.91919E+00_JPRB, 0.91885E+00_JPRB, 0.90683E+00_JPRB, 0.90022E+00_JPRB, 0.89332E+00_JPRB ,&
&  0.88182E+00_JPRB, 0.87757E+00_JPRB, 0.86379E+00_JPRB, 0.82829E+00_JPRB, 0.80392E+00_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.70914E+00_JPRB, 0.71108E+00_JPRB, 0.70561E+00_JPRB, 0.70352E+00_JPRB, 0.70357E+00_JPRB, 0.68775E+00_JPRB, 0.70773E+00_JPRB ,&
&  0.70158E+00_JPRB, 0.72106E+00_JPRB, 0.74219E+00_JPRB, 0.76411E+00_JPRB, 0.80804E+00_JPRB, 0.85336E+00_JPRB, 0.91653E+00_JPRB ,&
&  0.10248E+01_JPRB, 0.12009E+01_JPRB, 0.14231E+01_JPRB, 0.16973E+01_JPRB, 0.17772E+01_JPRB, 0.18233E+01_JPRB, 0.19190E+01_JPRB ,&
&  0.20425E+01_JPRB, 0.21384E+01_JPRB, 0.21875E+01_JPRB, 0.22492E+01_JPRB, 0.22972E+01_JPRB, 0.22190E+01_JPRB, 0.21033E+01_JPRB ,&
&  0.17218E+01_JPRB, 0.13375E+01_JPRB, 0.14551E+01_JPRB, 0.13829E+01_JPRB, 0.14577E+01_JPRB, 0.15245E+01_JPRB, 0.15766E+01_JPRB ,&
&  0.16313E+01_JPRB, 0.16698E+01_JPRB, 0.17235E+01_JPRB, 0.17877E+01_JPRB, 0.18778E+01_JPRB, 0.19657E+01_JPRB, 0.20672E+01_JPRB ,&
&  0.21391E+01_JPRB, 0.22144E+01_JPRB, 0.22785E+01_JPRB, 0.22495E+01_JPRB, 0.23854E+01_JPRB, 0.25759E+01_JPRB, 0.27504E+01_JPRB ,&
&  0.28897E+01_JPRB, 0.28686E+01_JPRB, 0.25819E+01_JPRB, 0.27014E+01_JPRB, 0.29032E+01_JPRB, 0.27879E+01_JPRB, 0.28926E+01_JPRB ,&
&  0.31815E+01_JPRB, 0.28064E+01_JPRB, 0.14107E+01_JPRB, 0.12160E+01_JPRB, 0.12632E+01_JPRB, 0.12872E+01_JPRB, 0.13085E+01_JPRB ,&
&  0.13065E+01_JPRB, 0.13193E+01_JPRB, 0.13545E+01_JPRB, 0.13898E+01_JPRB, 0.13658E+01_JPRB, 0.13667E+01_JPRB, 0.11589E+01_JPRB ,&
&  0.81422E+00_JPRB, 0.79927E+00_JPRB, 0.83377E+00_JPRB, 0.86553E+00_JPRB, 0.91653E+00_JPRB, 0.89464E+00_JPRB, 0.83110E+00_JPRB ,&
&  0.89657E+00_JPRB, 0.98016E+00_JPRB, 0.11157E+01_JPRB, 0.10073E+01_JPRB, 0.78538E+00_JPRB, 0.79492E+00_JPRB, 0.79761E+00_JPRB ,&
&  0.82058E+00_JPRB, 0.82226E+00_JPRB, 0.83385E+00_JPRB, 0.83946E+00_JPRB, 0.84891E+00_JPRB, 0.83386E+00_JPRB, 0.81633E+00_JPRB ,&
&  0.79587E+00_JPRB, 0.77800E+00_JPRB, 0.76211E+00_JPRB, 0.73708E+00_JPRB, 0.71862E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.64146E+00_JPRB, 0.61261E+00_JPRB, 0.58970E+00_JPRB, 0.59228E+00_JPRB, 0.59768E+00_JPRB, 0.58418E+00_JPRB, 0.61242E+00_JPRB ,&
&  0.58686E+00_JPRB, 0.60767E+00_JPRB, 0.61889E+00_JPRB, 0.61690E+00_JPRB, 0.65420E+00_JPRB, 0.69302E+00_JPRB, 0.74274E+00_JPRB ,&
&  0.81168E+00_JPRB, 0.92143E+00_JPRB, 0.10579E+01_JPRB, 0.12268E+01_JPRB, 0.14295E+01_JPRB, 0.15212E+01_JPRB, 0.16161E+01_JPRB ,&
&  0.17538E+01_JPRB, 0.19371E+01_JPRB, 0.22009E+01_JPRB, 0.24972E+01_JPRB, 0.28469E+01_JPRB, 0.26719E+01_JPRB, 0.22620E+01_JPRB ,&
&  0.16905E+01_JPRB, 0.13593E+01_JPRB, 0.13091E+01_JPRB, 0.12327E+01_JPRB, 0.13239E+01_JPRB, 0.15089E+01_JPRB, 0.16044E+01_JPRB ,&
&  0.16674E+01_JPRB, 0.17383E+01_JPRB, 0.18179E+01_JPRB, 0.19070E+01_JPRB, 0.20068E+01_JPRB, 0.21205E+01_JPRB, 0.21910E+01_JPRB ,&
&  0.22329E+01_JPRB, 0.22793E+01_JPRB, 0.23258E+01_JPRB, 0.24707E+01_JPRB, 0.26501E+01_JPRB, 0.28769E+01_JPRB, 0.29378E+01_JPRB ,&
&  0.29551E+01_JPRB, 0.28810E+01_JPRB, 0.25018E+01_JPRB, 0.24789E+01_JPRB, 0.25566E+01_JPRB, 0.25849E+01_JPRB, 0.26229E+01_JPRB ,&
&  0.28503E+01_JPRB, 0.22820E+01_JPRB, 0.14144E+01_JPRB, 0.13623E+01_JPRB, 0.13391E+01_JPRB, 0.13473E+01_JPRB, 0.11983E+01_JPRB ,&
&  0.11883E+01_JPRB, 0.11818E+01_JPRB, 0.11845E+01_JPRB, 0.11507E+01_JPRB, 0.11491E+01_JPRB, 0.11409E+01_JPRB, 0.10754E+01_JPRB ,&
&  0.92550E+00_JPRB, 0.88783E+00_JPRB, 0.83408E+00_JPRB, 0.78907E+00_JPRB, 0.89087E+00_JPRB, 0.83507E+00_JPRB, 0.85966E+00_JPRB ,&
&  0.81659E+00_JPRB, 0.87645E+00_JPRB, 0.78617E+00_JPRB, 0.95827E+00_JPRB, 0.76275E+00_JPRB, 0.71731E+00_JPRB, 0.70722E+00_JPRB ,&
&  0.72697E+00_JPRB, 0.73645E+00_JPRB, 0.74773E+00_JPRB, 0.73641E+00_JPRB, 0.75115E+00_JPRB, 0.77782E+00_JPRB, 0.77721E+00_JPRB ,&
&  0.73332E+00_JPRB, 0.71702E+00_JPRB, 0.69060E+00_JPRB, 0.65861E+00_JPRB, 0.65226E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.55536E+00_JPRB, 0.54916E+00_JPRB, 0.54784E+00_JPRB, 0.54513E+00_JPRB, 0.55367E+00_JPRB, 0.55452E+00_JPRB, 0.55437E+00_JPRB ,&
&  0.55288E+00_JPRB, 0.56241E+00_JPRB, 0.56915E+00_JPRB, 0.58014E+00_JPRB, 0.59296E+00_JPRB, 0.62314E+00_JPRB, 0.65735E+00_JPRB ,&
&  0.69793E+00_JPRB, 0.74487E+00_JPRB, 0.79651E+00_JPRB, 0.87087E+00_JPRB, 0.97506E+00_JPRB, 0.10463E+01_JPRB, 0.11252E+01_JPRB ,&
&  0.12284E+01_JPRB, 0.13852E+01_JPRB, 0.15562E+01_JPRB, 0.17852E+01_JPRB, 0.20456E+01_JPRB, 0.20838E+01_JPRB, 0.19292E+01_JPRB ,&
&  0.14603E+01_JPRB, 0.14554E+01_JPRB, 0.13767E+01_JPRB, 0.12624E+01_JPRB, 0.12946E+01_JPRB, 0.12750E+01_JPRB, 0.13205E+01_JPRB ,&
&  0.13855E+01_JPRB, 0.14067E+01_JPRB, 0.14166E+01_JPRB, 0.14509E+01_JPRB, 0.14808E+01_JPRB, 0.15105E+01_JPRB, 0.15053E+01_JPRB ,&
&  0.15090E+01_JPRB, 0.15789E+01_JPRB, 0.18392E+01_JPRB, 0.21697E+01_JPRB, 0.24082E+01_JPRB, 0.25072E+01_JPRB, 0.23892E+01_JPRB ,&
&  0.22156E+01_JPRB, 0.21082E+01_JPRB, 0.19738E+01_JPRB, 0.19464E+01_JPRB, 0.20049E+01_JPRB, 0.20321E+01_JPRB, 0.21593E+01_JPRB ,&
&  0.20372E+01_JPRB, 0.16945E+01_JPRB, 0.15320E+01_JPRB, 0.14230E+01_JPRB, 0.13497E+01_JPRB, 0.12691E+01_JPRB, 0.11814E+01_JPRB ,&
&  0.11765E+01_JPRB, 0.11252E+01_JPRB, 0.10816E+01_JPRB, 0.10072E+01_JPRB, 0.94691E+00_JPRB, 0.91691E+00_JPRB, 0.89413E+00_JPRB ,&
&  0.81699E+00_JPRB, 0.75761E+00_JPRB, 0.72822E+00_JPRB, 0.74694E+00_JPRB, 0.80344E+00_JPRB, 0.11245E+01_JPRB, 0.10836E+01_JPRB ,&
&  0.73575E+00_JPRB, 0.64031E+00_JPRB, 0.53916E+00_JPRB, 0.71638E+00_JPRB, 0.65614E+00_JPRB, 0.64513E+00_JPRB, 0.64829E+00_JPRB ,&
&  0.65338E+00_JPRB, 0.65085E+00_JPRB, 0.63099E+00_JPRB, 0.62731E+00_JPRB, 0.63592E+00_JPRB, 0.64295E+00_JPRB, 0.63195E+00_JPRB ,&
&  0.59547E+00_JPRB, 0.57820E+00_JPRB, 0.60251E+00_JPRB, 0.56528E+00_JPRB, 0.55841E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.50928E+00_JPRB, 0.50885E+00_JPRB, 0.52209E+00_JPRB, 0.53486E+00_JPRB, 0.54175E+00_JPRB, 0.54672E+00_JPRB, 0.55531E+00_JPRB ,&
&  0.55961E+00_JPRB, 0.56656E+00_JPRB, 0.57498E+00_JPRB, 0.59116E+00_JPRB, 0.61521E+00_JPRB, 0.65192E+00_JPRB, 0.69367E+00_JPRB ,&
&  0.74457E+00_JPRB, 0.79950E+00_JPRB, 0.86076E+00_JPRB, 0.92752E+00_JPRB, 0.10106E+01_JPRB, 0.11262E+01_JPRB, 0.12842E+01_JPRB ,&
&  0.13711E+01_JPRB, 0.14803E+01_JPRB, 0.16118E+01_JPRB, 0.17215E+01_JPRB, 0.17321E+01_JPRB, 0.16760E+01_JPRB, 0.13509E+01_JPRB ,&
&  0.15359E+01_JPRB, 0.14646E+01_JPRB, 0.13433E+01_JPRB, 0.12228E+01_JPRB, 0.11396E+01_JPRB, 0.10737E+01_JPRB, 0.10237E+01_JPRB ,&
&  0.10127E+01_JPRB, 0.97932E+00_JPRB, 0.92993E+00_JPRB, 0.89350E+00_JPRB, 0.87844E+00_JPRB, 0.90263E+00_JPRB, 0.89464E+00_JPRB ,&
&  0.99026E+00_JPRB, 0.11981E+01_JPRB, 0.14620E+01_JPRB, 0.17814E+01_JPRB, 0.19610E+01_JPRB, 0.19347E+01_JPRB, 0.16282E+01_JPRB ,&
&  0.14469E+01_JPRB, 0.13719E+01_JPRB, 0.13447E+01_JPRB, 0.14163E+01_JPRB, 0.14272E+01_JPRB, 0.14926E+01_JPRB, 0.15232E+01_JPRB ,&
&  0.14972E+01_JPRB, 0.13308E+01_JPRB, 0.13122E+01_JPRB, 0.15393E+01_JPRB, 0.13919E+01_JPRB, 0.12407E+01_JPRB, 0.12283E+01_JPRB ,&
&  0.11443E+01_JPRB, 0.10876E+01_JPRB, 0.10568E+01_JPRB, 0.98814E+00_JPRB, 0.91372E+00_JPRB, 0.85938E+00_JPRB, 0.82921E+00_JPRB ,&
&  0.76154E+00_JPRB, 0.68964E+00_JPRB, 0.64181E+00_JPRB, 0.64732E+00_JPRB, 0.73267E+00_JPRB, 0.81188E+00_JPRB, 0.11079E+01_JPRB ,&
&  0.70741E+00_JPRB, 0.48085E+00_JPRB, 0.52306E+00_JPRB, 0.58964E+00_JPRB, 0.56886E+00_JPRB, 0.58064E+00_JPRB, 0.64140E+00_JPRB ,&
&  0.71071E+00_JPRB, 0.77867E+00_JPRB, 0.77590E+00_JPRB, 0.66763E+00_JPRB, 0.62447E+00_JPRB, 0.60220E+00_JPRB, 0.57684E+00_JPRB ,&
&  0.55679E+00_JPRB, 0.52033E+00_JPRB, 0.53269E+00_JPRB, 0.51675E+00_JPRB, 0.51378E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.48036E+00_JPRB, 0.47860E+00_JPRB, 0.48618E+00_JPRB, 0.49369E+00_JPRB, 0.49501E+00_JPRB, 0.49615E+00_JPRB, 0.49684E+00_JPRB ,&
&  0.49893E+00_JPRB, 0.49821E+00_JPRB, 0.49998E+00_JPRB, 0.50687E+00_JPRB, 0.52200E+00_JPRB, 0.55482E+00_JPRB, 0.59256E+00_JPRB ,&
&  0.62533E+00_JPRB, 0.65542E+00_JPRB, 0.68418E+00_JPRB, 0.70881E+00_JPRB, 0.73429E+00_JPRB, 0.75119E+00_JPRB, 0.80247E+00_JPRB ,&
&  0.86777E+00_JPRB, 0.94599E+00_JPRB, 0.10250E+01_JPRB, 0.10815E+01_JPRB, 0.11272E+01_JPRB, 0.10762E+01_JPRB, 0.86629E+00_JPRB ,&
&  0.13547E+01_JPRB, 0.13410E+01_JPRB, 0.12260E+01_JPRB, 0.11056E+01_JPRB, 0.10209E+01_JPRB, 0.94572E+00_JPRB, 0.86835E+00_JPRB ,&
&  0.79582E+00_JPRB, 0.82468E+00_JPRB, 0.79977E+00_JPRB, 0.84651E+00_JPRB, 0.79768E+00_JPRB, 0.85344E+00_JPRB, 0.10133E+01_JPRB ,&
&  0.11444E+01_JPRB, 0.12733E+01_JPRB, 0.14284E+01_JPRB, 0.15877E+01_JPRB, 0.16757E+01_JPRB, 0.16427E+01_JPRB, 0.14735E+01_JPRB ,&
&  0.13758E+01_JPRB, 0.13418E+01_JPRB, 0.12838E+01_JPRB, 0.13858E+01_JPRB, 0.13381E+01_JPRB, 0.12559E+01_JPRB, 0.12640E+01_JPRB ,&
&  0.11184E+01_JPRB, 0.11246E+01_JPRB, 0.12700E+01_JPRB, 0.15486E+01_JPRB, 0.14107E+01_JPRB, 0.13066E+01_JPRB, 0.12640E+01_JPRB ,&
&  0.11738E+01_JPRB, 0.10984E+01_JPRB, 0.10450E+01_JPRB, 0.97540E+00_JPRB, 0.90379E+00_JPRB, 0.84210E+00_JPRB, 0.78266E+00_JPRB ,&
&  0.72932E+00_JPRB, 0.66589E+00_JPRB, 0.61661E+00_JPRB, 0.59429E+00_JPRB, 0.60645E+00_JPRB, 0.60978E+00_JPRB, 0.93935E+00_JPRB ,&
&  0.72007E+00_JPRB, 0.59151E+00_JPRB, 0.63538E+00_JPRB, 0.38671E+00_JPRB, 0.47200E+00_JPRB, 0.51681E+00_JPRB, 0.63191E+00_JPRB ,&
&  0.79755E+00_JPRB, 0.12820E+01_JPRB, 0.20081E+01_JPRB, 0.18120E+01_JPRB, 0.13799E+01_JPRB, 0.88228E+00_JPRB, 0.52294E+00_JPRB ,&
&  0.50235E+00_JPRB, 0.49773E+00_JPRB, 0.50215E+00_JPRB, 0.49258E+00_JPRB, 0.48849E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.43850E+00_JPRB, 0.42925E+00_JPRB, 0.43695E+00_JPRB, 0.44624E+00_JPRB, 0.44201E+00_JPRB, 0.44111E+00_JPRB, 0.43588E+00_JPRB ,&
&  0.42636E+00_JPRB, 0.42018E+00_JPRB, 0.44178E+00_JPRB, 0.48252E+00_JPRB, 0.53868E+00_JPRB, 0.61107E+00_JPRB, 0.68144E+00_JPRB ,&
&  0.72477E+00_JPRB, 0.77340E+00_JPRB, 0.81363E+00_JPRB, 0.84518E+00_JPRB, 0.84222E+00_JPRB, 0.83721E+00_JPRB, 0.81768E+00_JPRB ,&
&  0.80355E+00_JPRB, 0.81149E+00_JPRB, 0.84817E+00_JPRB, 0.88103E+00_JPRB, 0.94618E+00_JPRB, 0.10251E+01_JPRB, 0.68687E+00_JPRB ,&
&  0.11165E+01_JPRB, 0.11275E+01_JPRB, 0.10768E+01_JPRB, 0.98029E+00_JPRB, 0.92983E+00_JPRB, 0.85014E+00_JPRB, 0.78394E+00_JPRB ,&
&  0.78429E+00_JPRB, 0.82897E+00_JPRB, 0.86543E+00_JPRB, 0.10028E+01_JPRB, 0.11546E+01_JPRB, 0.12392E+01_JPRB, 0.14175E+01_JPRB ,&
&  0.15624E+01_JPRB, 0.17443E+01_JPRB, 0.19360E+01_JPRB, 0.20686E+01_JPRB, 0.21804E+01_JPRB, 0.22293E+01_JPRB, 0.22192E+01_JPRB ,&
&  0.20788E+01_JPRB, 0.19840E+01_JPRB, 0.20420E+01_JPRB, 0.21874E+01_JPRB, 0.19210E+01_JPRB, 0.18007E+01_JPRB, 0.15157E+01_JPRB ,&
&  0.11400E+01_JPRB, 0.10472E+01_JPRB, 0.11888E+01_JPRB, 0.14429E+01_JPRB, 0.14213E+01_JPRB, 0.13392E+01_JPRB, 0.12245E+01_JPRB ,&
&  0.11302E+01_JPRB, 0.10336E+01_JPRB, 0.96308E+00_JPRB, 0.89964E+00_JPRB, 0.84382E+00_JPRB, 0.78972E+00_JPRB, 0.73327E+00_JPRB ,&
&  0.70098E+00_JPRB, 0.65811E+00_JPRB, 0.61016E+00_JPRB, 0.59978E+00_JPRB, 0.61211E+00_JPRB, 0.67894E+00_JPRB, 0.71625E+00_JPRB ,&
&  0.82677E+00_JPRB, 0.63932E+00_JPRB, 0.56530E+00_JPRB, 0.54734E+00_JPRB, 0.47400E+00_JPRB, 0.51475E+00_JPRB, 0.68407E+00_JPRB ,&
&  0.10923E+01_JPRB, 0.16322E+01_JPRB, 0.27785E+01_JPRB, 0.50080E+01_JPRB, 0.27671E+01_JPRB, 0.21316E+01_JPRB, 0.50736E+00_JPRB ,&
&  0.48546E+00_JPRB, 0.48739E+00_JPRB, 0.48438E+00_JPRB, 0.46746E+00_JPRB, 0.46228E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.42815E+00_JPRB, 0.41967E+00_JPRB, 0.42778E+00_JPRB, 0.42584E+00_JPRB, 0.42910E+00_JPRB, 0.43569E+00_JPRB, 0.43973E+00_JPRB ,&
&  0.44325E+00_JPRB, 0.45966E+00_JPRB, 0.54798E+00_JPRB, 0.66899E+00_JPRB, 0.78252E+00_JPRB, 0.86784E+00_JPRB, 0.90350E+00_JPRB ,&
&  0.93586E+00_JPRB, 0.94461E+00_JPRB, 0.92560E+00_JPRB, 0.90100E+00_JPRB, 0.90213E+00_JPRB, 0.93874E+00_JPRB, 0.96670E+00_JPRB ,&
&  0.99100E+00_JPRB, 0.10132E+01_JPRB, 0.10360E+01_JPRB, 0.10746E+01_JPRB, 0.11126E+01_JPRB, 0.10512E+01_JPRB, 0.66214E+00_JPRB ,&
&  0.94662E+00_JPRB, 0.10046E+01_JPRB, 0.10119E+01_JPRB, 0.93768E+00_JPRB, 0.87127E+00_JPRB, 0.83288E+00_JPRB, 0.89133E+00_JPRB ,&
&  0.98188E+00_JPRB, 0.10451E+01_JPRB, 0.10872E+01_JPRB, 0.11656E+01_JPRB, 0.16272E+01_JPRB, 0.17959E+01_JPRB, 0.20887E+01_JPRB ,&
&  0.23053E+01_JPRB, 0.25362E+01_JPRB, 0.27017E+01_JPRB, 0.28887E+01_JPRB, 0.30458E+01_JPRB, 0.31551E+01_JPRB, 0.33088E+01_JPRB ,&
&  0.33631E+01_JPRB, 0.34395E+01_JPRB, 0.34454E+01_JPRB, 0.33477E+01_JPRB, 0.27369E+01_JPRB, 0.29101E+01_JPRB, 0.20013E+01_JPRB ,&
&  0.14392E+01_JPRB, 0.12500E+01_JPRB, 0.13849E+01_JPRB, 0.14325E+01_JPRB, 0.13406E+01_JPRB, 0.12253E+01_JPRB, 0.10826E+01_JPRB ,&
&  0.10005E+01_JPRB, 0.93691E+00_JPRB, 0.88391E+00_JPRB, 0.83042E+00_JPRB, 0.80711E+00_JPRB, 0.77351E+00_JPRB, 0.74344E+00_JPRB ,&
&  0.73205E+00_JPRB, 0.69583E+00_JPRB, 0.65072E+00_JPRB, 0.65181E+00_JPRB, 0.68458E+00_JPRB, 0.81442E+00_JPRB, 0.10865E+01_JPRB ,&
&  0.12115E+01_JPRB, 0.70841E+00_JPRB, 0.64879E+00_JPRB, 0.60198E+00_JPRB, 0.58605E+00_JPRB, 0.61602E+00_JPRB, 0.70085E+00_JPRB ,&
&  0.91755E+00_JPRB, 0.16307E+01_JPRB, 0.25215E+01_JPRB, 0.19210E+01_JPRB, 0.21945E+01_JPRB, 0.15067E+01_JPRB, 0.60774E+00_JPRB ,&
&  0.47131E+00_JPRB, 0.46101E+00_JPRB, 0.46631E+00_JPRB, 0.45640E+00_JPRB, 0.44349E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.47929E+00_JPRB, 0.46820E+00_JPRB, 0.46720E+00_JPRB, 0.47722E+00_JPRB, 0.48348E+00_JPRB, 0.50755E+00_JPRB, 0.53774E+00_JPRB ,&
&  0.55302E+00_JPRB, 0.62214E+00_JPRB, 0.77660E+00_JPRB, 0.90799E+00_JPRB, 0.96105E+00_JPRB, 0.99877E+00_JPRB, 0.94978E+00_JPRB ,&
&  0.93502E+00_JPRB, 0.90192E+00_JPRB, 0.85083E+00_JPRB, 0.84110E+00_JPRB, 0.85431E+00_JPRB, 0.88491E+00_JPRB, 0.92569E+00_JPRB ,&
&  0.97246E+00_JPRB, 0.10210E+01_JPRB, 0.10604E+01_JPRB, 0.10971E+01_JPRB, 0.11474E+01_JPRB, 0.11280E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.56413E+00_JPRB, 0.93891E+00_JPRB, 0.95122E+00_JPRB, 0.90085E+00_JPRB, 0.85738E+00_JPRB, 0.87430E+00_JPRB, 0.10200E+01_JPRB ,&
&  0.11775E+01_JPRB, 0.12386E+01_JPRB, 0.14237E+01_JPRB, 0.15442E+01_JPRB, 0.19929E+01_JPRB, 0.22610E+01_JPRB, 0.25068E+01_JPRB ,&
&  0.27342E+01_JPRB, 0.29056E+01_JPRB, 0.30916E+01_JPRB, 0.32769E+01_JPRB, 0.34598E+01_JPRB, 0.36147E+01_JPRB, 0.38008E+01_JPRB ,&
&  0.40168E+01_JPRB, 0.43154E+01_JPRB, 0.46210E+01_JPRB, 0.44514E+01_JPRB, 0.37790E+01_JPRB, 0.35433E+01_JPRB, 0.28394E+01_JPRB ,&
&  0.19338E+01_JPRB, 0.12594E+01_JPRB, 0.12888E+01_JPRB, 0.13672E+01_JPRB, 0.12665E+01_JPRB, 0.11611E+01_JPRB, 0.10620E+01_JPRB ,&
&  0.98981E+00_JPRB, 0.91615E+00_JPRB, 0.82528E+00_JPRB, 0.81702E+00_JPRB, 0.83073E+00_JPRB, 0.81634E+00_JPRB, 0.79696E+00_JPRB ,&
&  0.79703E+00_JPRB, 0.77530E+00_JPRB, 0.74855E+00_JPRB, 0.72512E+00_JPRB, 0.72930E+00_JPRB, 0.76962E+00_JPRB, 0.80365E+00_JPRB ,&
&  0.83200E+00_JPRB, 0.76211E+00_JPRB, 0.71163E+00_JPRB, 0.71351E+00_JPRB, 0.71402E+00_JPRB, 0.72086E+00_JPRB, 0.71849E+00_JPRB ,&
&  0.77148E+00_JPRB, 0.93048E+00_JPRB, 0.13215E+01_JPRB, 0.17175E+01_JPRB, 0.13936E+01_JPRB, 0.10991E+01_JPRB, 0.62499E+00_JPRB ,&
&  0.49803E+00_JPRB, 0.48128E+00_JPRB, 0.49922E+00_JPRB, 0.50278E+00_JPRB, 0.50171E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.58653E+00_JPRB, 0.57413E+00_JPRB, 0.55735E+00_JPRB, 0.57338E+00_JPRB, 0.60658E+00_JPRB, 0.64851E+00_JPRB, 0.66639E+00_JPRB ,&
&  0.68959E+00_JPRB, 0.78907E+00_JPRB, 0.92063E+00_JPRB, 0.10367E+01_JPRB, 0.10557E+01_JPRB, 0.10231E+01_JPRB, 0.92284E+00_JPRB ,&
&  0.89487E+00_JPRB, 0.86695E+00_JPRB, 0.84979E+00_JPRB, 0.83169E+00_JPRB, 0.83263E+00_JPRB, 0.85957E+00_JPRB, 0.90781E+00_JPRB ,&
&  0.96932E+00_JPRB, 0.10354E+01_JPRB, 0.11101E+01_JPRB, 0.11889E+01_JPRB, 0.12555E+01_JPRB, 0.12663E+01_JPRB, 0.11498E+01_JPRB ,&
&  0.49547E+00_JPRB, 0.61490E+00_JPRB, 0.91238E+00_JPRB, 0.92035E+00_JPRB, 0.90839E+00_JPRB, 0.94734E+00_JPRB, 0.10794E+01_JPRB ,&
&  0.12151E+01_JPRB, 0.12897E+01_JPRB, 0.14934E+01_JPRB, 0.18760E+01_JPRB, 0.20863E+01_JPRB, 0.23744E+01_JPRB, 0.25440E+01_JPRB ,&
&  0.27155E+01_JPRB, 0.28904E+01_JPRB, 0.30846E+01_JPRB, 0.32818E+01_JPRB, 0.34795E+01_JPRB, 0.36622E+01_JPRB, 0.38991E+01_JPRB ,&
&  0.42196E+01_JPRB, 0.45662E+01_JPRB, 0.48607E+01_JPRB, 0.43948E+01_JPRB, 0.46711E+01_JPRB, 0.40957E+01_JPRB, 0.35298E+01_JPRB ,&
&  0.21048E+01_JPRB, 0.13103E+01_JPRB, 0.13219E+01_JPRB, 0.14135E+01_JPRB, 0.12505E+01_JPRB, 0.11089E+01_JPRB, 0.97224E+00_JPRB ,&
&  0.90501E+00_JPRB, 0.83043E+00_JPRB, 0.83691E+00_JPRB, 0.84993E+00_JPRB, 0.83678E+00_JPRB, 0.87196E+00_JPRB, 0.82937E+00_JPRB ,&
&  0.84030E+00_JPRB, 0.84993E+00_JPRB, 0.84056E+00_JPRB, 0.84415E+00_JPRB, 0.84176E+00_JPRB, 0.84167E+00_JPRB, 0.82710E+00_JPRB ,&
&  0.83032E+00_JPRB, 0.82214E+00_JPRB, 0.82557E+00_JPRB, 0.83645E+00_JPRB, 0.83478E+00_JPRB, 0.82338E+00_JPRB, 0.78143E+00_JPRB ,&
&  0.74006E+00_JPRB, 0.75467E+00_JPRB, 0.85841E+00_JPRB, 0.99255E+00_JPRB, 0.95317E+00_JPRB, 0.82240E+00_JPRB, 0.65146E+00_JPRB ,&
&  0.63692E+00_JPRB, 0.65784E+00_JPRB, 0.65993E+00_JPRB, 0.64017E+00_JPRB, 0.61576E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.69826E+00_JPRB, 0.67225E+00_JPRB, 0.63411E+00_JPRB, 0.62747E+00_JPRB, 0.68256E+00_JPRB, 0.73042E+00_JPRB, 0.73256E+00_JPRB ,&
&  0.74007E+00_JPRB, 0.80390E+00_JPRB, 0.91980E+00_JPRB, 0.10117E+01_JPRB, 0.10410E+01_JPRB, 0.10325E+01_JPRB, 0.93833E+00_JPRB ,&
&  0.90166E+00_JPRB, 0.87789E+00_JPRB, 0.85064E+00_JPRB, 0.83768E+00_JPRB, 0.83294E+00_JPRB, 0.83821E+00_JPRB, 0.85800E+00_JPRB ,&
&  0.88773E+00_JPRB, 0.94383E+00_JPRB, 0.10391E+01_JPRB, 0.11717E+01_JPRB, 0.13690E+01_JPRB, 0.15951E+01_JPRB, 0.17891E+01_JPRB ,&
&  0.13333E+01_JPRB, 0.59030E+00_JPRB, 0.68069E+00_JPRB, 0.96871E+00_JPRB, 0.94636E+00_JPRB, 0.10090E+01_JPRB, 0.10298E+01_JPRB ,&
&  0.11168E+01_JPRB, 0.12954E+01_JPRB, 0.14869E+01_JPRB, 0.18323E+01_JPRB, 0.20603E+01_JPRB, 0.22943E+01_JPRB, 0.24053E+01_JPRB ,&
&  0.25203E+01_JPRB, 0.26572E+01_JPRB, 0.27955E+01_JPRB, 0.29393E+01_JPRB, 0.30994E+01_JPRB, 0.32802E+01_JPRB, 0.35502E+01_JPRB ,&
&  0.38918E+01_JPRB, 0.42194E+01_JPRB, 0.44533E+01_JPRB, 0.38366E+01_JPRB, 0.40538E+01_JPRB, 0.32214E+01_JPRB, 0.24404E+01_JPRB ,&
&  0.19157E+01_JPRB, 0.15519E+01_JPRB, 0.14317E+01_JPRB, 0.14703E+01_JPRB, 0.12657E+01_JPRB, 0.98652E+00_JPRB, 0.96819E+00_JPRB ,&
&  0.87863E+00_JPRB, 0.85000E+00_JPRB, 0.85369E+00_JPRB, 0.80164E+00_JPRB, 0.85763E+00_JPRB, 0.87833E+00_JPRB, 0.89091E+00_JPRB ,&
&  0.87262E+00_JPRB, 0.89102E+00_JPRB, 0.91503E+00_JPRB, 0.95370E+00_JPRB, 0.94025E+00_JPRB, 0.93516E+00_JPRB, 0.91402E+00_JPRB ,&
&  0.91565E+00_JPRB, 0.92045E+00_JPRB, 0.93156E+00_JPRB, 0.94996E+00_JPRB, 0.95271E+00_JPRB, 0.87981E+00_JPRB, 0.83044E+00_JPRB ,&
&  0.80664E+00_JPRB, 0.79814E+00_JPRB, 0.77078E+00_JPRB, 0.81515E+00_JPRB, 0.82631E+00_JPRB, 0.83652E+00_JPRB, 0.86945E+00_JPRB ,&
&  0.88271E+00_JPRB, 0.88340E+00_JPRB, 0.83333E+00_JPRB, 0.77682E+00_JPRB, 0.74048E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.75836E+00_JPRB, 0.71358E+00_JPRB, 0.68137E+00_JPRB, 0.69154E+00_JPRB, 0.74117E+00_JPRB, 0.79246E+00_JPRB, 0.79091E+00_JPRB ,&
&  0.77671E+00_JPRB, 0.79706E+00_JPRB, 0.86625E+00_JPRB, 0.92603E+00_JPRB, 0.93467E+00_JPRB, 0.94390E+00_JPRB, 0.96340E+00_JPRB ,&
&  0.91291E+00_JPRB, 0.88033E+00_JPRB, 0.86415E+00_JPRB, 0.84417E+00_JPRB, 0.84289E+00_JPRB, 0.83673E+00_JPRB, 0.83845E+00_JPRB ,&
&  0.85449E+00_JPRB, 0.87518E+00_JPRB, 0.92690E+00_JPRB, 0.10082E+01_JPRB, 0.11456E+01_JPRB, 0.14353E+01_JPRB, 0.18782E+01_JPRB ,&
&  0.26132E+01_JPRB, 0.17281E+01_JPRB, 0.67418E+00_JPRB, 0.90635E+00_JPRB, 0.10051E+01_JPRB, 0.10643E+01_JPRB, 0.97690E+00_JPRB ,&
&  0.10199E+01_JPRB, 0.11183E+01_JPRB, 0.12959E+01_JPRB, 0.16683E+01_JPRB, 0.18934E+01_JPRB, 0.20650E+01_JPRB, 0.21729E+01_JPRB ,&
&  0.22187E+01_JPRB, 0.23008E+01_JPRB, 0.23722E+01_JPRB, 0.24356E+01_JPRB, 0.25118E+01_JPRB, 0.26272E+01_JPRB, 0.28229E+01_JPRB ,&
&  0.31929E+01_JPRB, 0.36476E+01_JPRB, 0.40345E+01_JPRB, 0.35145E+01_JPRB, 0.35307E+01_JPRB, 0.27393E+01_JPRB, 0.22818E+01_JPRB ,&
&  0.19829E+01_JPRB, 0.19516E+01_JPRB, 0.18668E+01_JPRB, 0.15735E+01_JPRB, 0.12824E+01_JPRB, 0.11010E+01_JPRB, 0.11269E+01_JPRB ,&
&  0.96781E+00_JPRB, 0.88633E+00_JPRB, 0.86303E+00_JPRB, 0.88803E+00_JPRB, 0.90896E+00_JPRB, 0.91700E+00_JPRB, 0.91490E+00_JPRB ,&
&  0.92887E+00_JPRB, 0.92901E+00_JPRB, 0.97015E+00_JPRB, 0.99743E+00_JPRB, 0.10167E+01_JPRB, 0.10538E+01_JPRB, 0.10500E+01_JPRB ,&
&  0.10340E+01_JPRB, 0.10240E+01_JPRB, 0.10272E+01_JPRB, 0.10390E+01_JPRB, 0.98664E+00_JPRB, 0.90785E+00_JPRB, 0.90054E+00_JPRB ,&
&  0.89413E+00_JPRB, 0.89086E+00_JPRB, 0.85851E+00_JPRB, 0.86993E+00_JPRB, 0.87952E+00_JPRB, 0.94096E+00_JPRB, 0.10147E+01_JPRB ,&
&  0.10470E+01_JPRB, 0.10191E+01_JPRB, 0.95021E+00_JPRB, 0.82204E+00_JPRB, 0.80708E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.81338E+00_JPRB, 0.80114E+00_JPRB, 0.77440E+00_JPRB, 0.75619E+00_JPRB, 0.77892E+00_JPRB, 0.81356E+00_JPRB, 0.81272E+00_JPRB ,&
&  0.78831E+00_JPRB, 0.77475E+00_JPRB, 0.81015E+00_JPRB, 0.84749E+00_JPRB, 0.84515E+00_JPRB, 0.86243E+00_JPRB, 0.87191E+00_JPRB ,&
&  0.90453E+00_JPRB, 0.89797E+00_JPRB, 0.87791E+00_JPRB, 0.85089E+00_JPRB, 0.84792E+00_JPRB, 0.83470E+00_JPRB, 0.83374E+00_JPRB ,&
&  0.84349E+00_JPRB, 0.85704E+00_JPRB, 0.88430E+00_JPRB, 0.92863E+00_JPRB, 0.97981E+00_JPRB, 0.10974E+01_JPRB, 0.13422E+01_JPRB ,&
&  0.16629E+01_JPRB, 0.21600E+01_JPRB, 0.80528E+00_JPRB, 0.10392E+01_JPRB, 0.12488E+01_JPRB, 0.12001E+01_JPRB, 0.10962E+01_JPRB ,&
&  0.10379E+01_JPRB, 0.98215E+00_JPRB, 0.10729E+01_JPRB, 0.13424E+01_JPRB, 0.15668E+01_JPRB, 0.17181E+01_JPRB, 0.17269E+01_JPRB ,&
&  0.17746E+01_JPRB, 0.18639E+01_JPRB, 0.20119E+01_JPRB, 0.20517E+01_JPRB, 0.20529E+01_JPRB, 0.20420E+01_JPRB, 0.21147E+01_JPRB ,&
&  0.23714E+01_JPRB, 0.28150E+01_JPRB, 0.34718E+01_JPRB, 0.32421E+01_JPRB, 0.32443E+01_JPRB, 0.28456E+01_JPRB, 0.25168E+01_JPRB ,&
&  0.22589E+01_JPRB, 0.22579E+01_JPRB, 0.21419E+01_JPRB, 0.17755E+01_JPRB, 0.14116E+01_JPRB, 0.12456E+01_JPRB, 0.12332E+01_JPRB ,&
&  0.10790E+01_JPRB, 0.97835E+00_JPRB, 0.96173E+00_JPRB, 0.96316E+00_JPRB, 0.10041E+01_JPRB, 0.98997E+00_JPRB, 0.96487E+00_JPRB ,&
&  0.97774E+00_JPRB, 0.95860E+00_JPRB, 0.97679E+00_JPRB, 0.10107E+01_JPRB, 0.10451E+01_JPRB, 0.10997E+01_JPRB, 0.11242E+01_JPRB ,&
&  0.11160E+01_JPRB, 0.11162E+01_JPRB, 0.11011E+01_JPRB, 0.10498E+01_JPRB, 0.97996E+00_JPRB, 0.94045E+00_JPRB, 0.93294E+00_JPRB ,&
&  0.94371E+00_JPRB, 0.95193E+00_JPRB, 0.92019E+00_JPRB, 0.87788E+00_JPRB, 0.91275E+00_JPRB, 0.92948E+00_JPRB, 0.10213E+01_JPRB ,&
&  0.11304E+01_JPRB, 0.12009E+01_JPRB, 0.11560E+01_JPRB, 0.88212E+00_JPRB, 0.86865E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.89918E+00_JPRB, 0.86403E+00_JPRB, 0.84476E+00_JPRB, 0.82504E+00_JPRB, 0.80725E+00_JPRB, 0.83916E+00_JPRB, 0.83102E+00_JPRB ,&
&  0.80742E+00_JPRB, 0.77904E+00_JPRB, 0.79693E+00_JPRB, 0.84382E+00_JPRB, 0.84641E+00_JPRB, 0.82650E+00_JPRB, 0.81248E+00_JPRB ,&
&  0.81147E+00_JPRB, 0.82789E+00_JPRB, 0.83109E+00_JPRB, 0.84368E+00_JPRB, 0.84550E+00_JPRB, 0.83595E+00_JPRB, 0.82996E+00_JPRB ,&
&  0.84597E+00_JPRB, 0.85711E+00_JPRB, 0.86989E+00_JPRB, 0.88005E+00_JPRB, 0.89997E+00_JPRB, 0.91308E+00_JPRB, 0.10239E+01_JPRB ,&
&  0.11817E+01_JPRB, 0.18381E+01_JPRB, 0.15158E+01_JPRB, 0.17919E+01_JPRB, 0.16783E+01_JPRB, 0.15164E+01_JPRB, 0.12973E+01_JPRB ,&
&  0.12770E+01_JPRB, 0.11849E+01_JPRB, 0.12410E+01_JPRB, 0.11318E+01_JPRB, 0.11853E+01_JPRB, 0.12846E+01_JPRB, 0.12835E+01_JPRB ,&
&  0.12564E+01_JPRB, 0.12483E+01_JPRB, 0.14427E+01_JPRB, 0.15365E+01_JPRB, 0.15793E+01_JPRB, 0.15910E+01_JPRB, 0.16103E+01_JPRB ,&
&  0.17417E+01_JPRB, 0.19765E+01_JPRB, 0.25969E+01_JPRB, 0.30627E+01_JPRB, 0.29914E+01_JPRB, 0.29452E+01_JPRB, 0.28874E+01_JPRB ,&
&  0.29249E+01_JPRB, 0.26528E+01_JPRB, 0.23665E+01_JPRB, 0.19542E+01_JPRB, 0.15528E+01_JPRB, 0.15030E+01_JPRB, 0.13072E+01_JPRB ,&
&  0.11546E+01_JPRB, 0.10581E+01_JPRB, 0.10464E+01_JPRB, 0.10554E+01_JPRB, 0.10788E+01_JPRB, 0.10552E+01_JPRB, 0.10132E+01_JPRB ,&
&  0.99622E+00_JPRB, 0.10000E+01_JPRB, 0.98278E+00_JPRB, 0.10015E+01_JPRB, 0.10180E+01_JPRB, 0.10516E+01_JPRB, 0.10808E+01_JPRB ,&
&  0.11152E+01_JPRB, 0.11374E+01_JPRB, 0.10923E+01_JPRB, 0.10480E+01_JPRB, 0.99451E+00_JPRB, 0.96129E+00_JPRB, 0.93180E+00_JPRB ,&
&  0.94339E+00_JPRB, 0.96753E+00_JPRB, 0.96456E+00_JPRB, 0.92720E+00_JPRB, 0.90573E+00_JPRB, 0.89974E+00_JPRB, 0.90996E+00_JPRB ,&
&  0.93486E+00_JPRB, 0.99130E+00_JPRB, 0.99144E+00_JPRB, 0.93767E+00_JPRB, 0.92415E+00_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.94908E+00_JPRB, 0.93336E+00_JPRB, 0.89365E+00_JPRB, 0.88703E+00_JPRB, 0.85859E+00_JPRB, 0.88491E+00_JPRB, 0.89021E+00_JPRB ,&
&  0.85671E+00_JPRB, 0.82486E+00_JPRB, 0.80431E+00_JPRB, 0.82463E+00_JPRB, 0.84659E+00_JPRB, 0.83385E+00_JPRB, 0.79453E+00_JPRB ,&
&  0.78144E+00_JPRB, 0.77881E+00_JPRB, 0.77811E+00_JPRB, 0.78242E+00_JPRB, 0.79941E+00_JPRB, 0.82381E+00_JPRB, 0.82228E+00_JPRB ,&
&  0.81941E+00_JPRB, 0.85082E+00_JPRB, 0.88002E+00_JPRB, 0.88089E+00_JPRB, 0.87342E+00_JPRB, 0.86479E+00_JPRB, 0.91166E+00_JPRB ,&
&  0.98109E+00_JPRB, 0.19265E+01_JPRB, 0.14367E+01_JPRB, 0.23059E+01_JPRB, 0.18379E+01_JPRB, 0.16185E+01_JPRB, 0.13165E+01_JPRB ,&
&  0.12095E+01_JPRB, 0.13017E+01_JPRB, 0.11541E+01_JPRB, 0.10355E+01_JPRB, 0.97893E+00_JPRB, 0.98388E+00_JPRB, 0.96462E+00_JPRB ,&
&  0.90078E+00_JPRB, 0.91227E+00_JPRB, 0.97739E+00_JPRB, 0.10782E+01_JPRB, 0.11512E+01_JPRB, 0.11861E+01_JPRB, 0.12367E+01_JPRB ,&
&  0.13196E+01_JPRB, 0.14286E+01_JPRB, 0.18360E+01_JPRB, 0.23683E+01_JPRB, 0.28654E+01_JPRB, 0.30563E+01_JPRB, 0.33139E+01_JPRB ,&
&  0.34116E+01_JPRB, 0.31659E+01_JPRB, 0.26319E+01_JPRB, 0.21272E+01_JPRB, 0.17695E+01_JPRB, 0.16382E+01_JPRB, 0.14922E+01_JPRB ,&
&  0.13405E+01_JPRB, 0.11993E+01_JPRB, 0.11676E+01_JPRB, 0.11461E+01_JPRB, 0.11706E+01_JPRB, 0.11485E+01_JPRB, 0.10766E+01_JPRB ,&
&  0.10169E+01_JPRB, 0.10008E+01_JPRB, 0.10069E+01_JPRB, 0.10219E+01_JPRB, 0.10413E+01_JPRB, 0.10407E+01_JPRB, 0.10419E+01_JPRB ,&
&  0.10720E+01_JPRB, 0.11133E+01_JPRB, 0.10897E+01_JPRB, 0.10790E+01_JPRB, 0.10384E+01_JPRB, 0.10046E+01_JPRB, 0.98670E+00_JPRB ,&
&  0.10135E+01_JPRB, 0.10052E+01_JPRB, 0.96774E+00_JPRB, 0.92300E+00_JPRB, 0.90330E+00_JPRB, 0.91239E+00_JPRB, 0.88419E+00_JPRB ,&
&  0.88868E+00_JPRB, 0.93424E+00_JPRB, 0.98547E+00_JPRB, 0.94972E+00_JPRB, 0.94563E+00_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.96415E+00_JPRB, 0.96340E+00_JPRB, 0.91408E+00_JPRB, 0.90044E+00_JPRB, 0.87731E+00_JPRB, 0.87292E+00_JPRB, 0.88540E+00_JPRB ,&
&  0.87792E+00_JPRB, 0.85385E+00_JPRB, 0.82308E+00_JPRB, 0.81034E+00_JPRB, 0.82237E+00_JPRB, 0.83478E+00_JPRB, 0.80216E+00_JPRB ,&
&  0.76456E+00_JPRB, 0.75147E+00_JPRB, 0.74421E+00_JPRB, 0.74721E+00_JPRB, 0.75898E+00_JPRB, 0.77669E+00_JPRB, 0.78689E+00_JPRB ,&
&  0.78306E+00_JPRB, 0.82184E+00_JPRB, 0.86559E+00_JPRB, 0.88171E+00_JPRB, 0.86683E+00_JPRB, 0.85886E+00_JPRB, 0.88384E+00_JPRB ,&
&  0.99687E+00_JPRB, 0.23232E+01_JPRB, 0.15280E+01_JPRB, 0.23623E+01_JPRB, 0.18796E+01_JPRB, 0.16047E+01_JPRB, 0.12963E+01_JPRB ,&
&  0.12107E+01_JPRB, 0.11991E+01_JPRB, 0.11082E+01_JPRB, 0.10158E+01_JPRB, 0.95287E+00_JPRB, 0.91276E+00_JPRB, 0.88170E+00_JPRB ,&
&  0.85333E+00_JPRB, 0.83892E+00_JPRB, 0.84305E+00_JPRB, 0.90120E+00_JPRB, 0.95066E+00_JPRB, 0.97827E+00_JPRB, 0.10011E+01_JPRB ,&
&  0.10484E+01_JPRB, 0.11207E+01_JPRB, 0.13636E+01_JPRB, 0.20424E+01_JPRB, 0.25551E+01_JPRB, 0.30544E+01_JPRB, 0.44421E+01_JPRB ,&
&  0.42970E+01_JPRB, 0.39505E+01_JPRB, 0.29302E+01_JPRB, 0.23385E+01_JPRB, 0.19547E+01_JPRB, 0.17315E+01_JPRB, 0.16378E+01_JPRB ,&
&  0.15349E+01_JPRB, 0.13971E+01_JPRB, 0.13373E+01_JPRB, 0.12949E+01_JPRB, 0.12576E+01_JPRB, 0.12148E+01_JPRB, 0.11476E+01_JPRB ,&
&  0.10887E+01_JPRB, 0.10445E+01_JPRB, 0.10482E+01_JPRB, 0.10583E+01_JPRB, 0.10686E+01_JPRB, 0.10525E+01_JPRB, 0.10418E+01_JPRB ,&
&  0.10521E+01_JPRB, 0.10870E+01_JPRB, 0.10782E+01_JPRB, 0.10823E+01_JPRB, 0.10703E+01_JPRB, 0.10435E+01_JPRB, 0.10244E+01_JPRB ,&
&  0.10636E+01_JPRB, 0.10583E+01_JPRB, 0.99506E+00_JPRB, 0.96273E+00_JPRB, 0.92598E+00_JPRB, 0.94519E+00_JPRB, 0.89666E+00_JPRB ,&
&  0.87839E+00_JPRB, 0.87842E+00_JPRB, 0.94786E+00_JPRB, 0.95484E+00_JPRB, 0.96831E+00_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.97184E+00_JPRB, 0.95648E+00_JPRB, 0.93327E+00_JPRB, 0.89495E+00_JPRB, 0.88163E+00_JPRB, 0.86564E+00_JPRB, 0.85770E+00_JPRB ,&
&  0.85595E+00_JPRB, 0.84393E+00_JPRB, 0.83078E+00_JPRB, 0.81131E+00_JPRB, 0.80384E+00_JPRB, 0.81527E+00_JPRB, 0.80983E+00_JPRB ,&
&  0.79175E+00_JPRB, 0.76798E+00_JPRB, 0.75986E+00_JPRB, 0.75053E+00_JPRB, 0.74089E+00_JPRB, 0.74221E+00_JPRB, 0.75288E+00_JPRB ,&
&  0.75843E+00_JPRB, 0.77689E+00_JPRB, 0.80722E+00_JPRB, 0.83462E+00_JPRB, 0.83278E+00_JPRB, 0.82682E+00_JPRB, 0.86260E+00_JPRB ,&
&  0.10126E+01_JPRB, 0.10082E+01_JPRB, 0.19915E+01_JPRB, 0.22520E+01_JPRB, 0.18904E+01_JPRB, 0.16392E+01_JPRB, 0.13962E+01_JPRB ,&
&  0.12931E+01_JPRB, 0.11550E+01_JPRB, 0.10456E+01_JPRB, 0.96272E+00_JPRB, 0.91791E+00_JPRB, 0.86548E+00_JPRB, 0.85105E+00_JPRB ,&
&  0.84565E+00_JPRB, 0.82846E+00_JPRB, 0.82497E+00_JPRB, 0.84401E+00_JPRB, 0.85561E+00_JPRB, 0.86906E+00_JPRB, 0.88022E+00_JPRB ,&
&  0.91117E+00_JPRB, 0.95495E+00_JPRB, 0.10833E+01_JPRB, 0.15201E+01_JPRB, 0.23385E+01_JPRB, 0.31647E+01_JPRB, 0.44281E+01_JPRB ,&
&  0.50222E+01_JPRB, 0.45845E+01_JPRB, 0.32494E+01_JPRB, 0.25206E+01_JPRB, 0.21281E+01_JPRB, 0.18669E+01_JPRB, 0.17455E+01_JPRB ,&
&  0.16561E+01_JPRB, 0.15494E+01_JPRB, 0.14678E+01_JPRB, 0.14629E+01_JPRB, 0.13751E+01_JPRB, 0.12893E+01_JPRB, 0.12080E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.11119E+01_JPRB, 0.10687E+01_JPRB, 0.10426E+01_JPRB, 0.10519E+01_JPRB, 0.10507E+01_JPRB, 0.10380E+01_JPRB ,&
&  0.10366E+01_JPRB, 0.10537E+01_JPRB, 0.10561E+01_JPRB, 0.10561E+01_JPRB, 0.10795E+01_JPRB, 0.10838E+01_JPRB, 0.10840E+01_JPRB ,&
&  0.10730E+01_JPRB, 0.10747E+01_JPRB, 0.10305E+01_JPRB, 0.10049E+01_JPRB, 0.94711E+00_JPRB, 0.98479E+00_JPRB, 0.93294E+00_JPRB ,&
&  0.89600E+00_JPRB, 0.86929E+00_JPRB, 0.88660E+00_JPRB, 0.93926E+00_JPRB, 0.97361E+00_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.94903E+00_JPRB, 0.93556E+00_JPRB, 0.92053E+00_JPRB, 0.90087E+00_JPRB, 0.88243E+00_JPRB, 0.87347E+00_JPRB, 0.86378E+00_JPRB ,&
&  0.86629E+00_JPRB, 0.86871E+00_JPRB, 0.86032E+00_JPRB, 0.83550E+00_JPRB, 0.81524E+00_JPRB, 0.81208E+00_JPRB, 0.81783E+00_JPRB ,&
&  0.81243E+00_JPRB, 0.80187E+00_JPRB, 0.80516E+00_JPRB, 0.78820E+00_JPRB, 0.76645E+00_JPRB, 0.75289E+00_JPRB, 0.74807E+00_JPRB ,&
&  0.75330E+00_JPRB, 0.76434E+00_JPRB, 0.79058E+00_JPRB, 0.81570E+00_JPRB, 0.81854E+00_JPRB, 0.81062E+00_JPRB, 0.79824E+00_JPRB ,&
&  0.92687E+00_JPRB, 0.12897E+01_JPRB, 0.20899E+01_JPRB, 0.20182E+01_JPRB, 0.17729E+01_JPRB, 0.15206E+01_JPRB, 0.14029E+01_JPRB ,&
&  0.12407E+01_JPRB, 0.11443E+01_JPRB, 0.10470E+01_JPRB, 0.95686E+00_JPRB, 0.89320E+00_JPRB, 0.85744E+00_JPRB, 0.84054E+00_JPRB ,&
&  0.81809E+00_JPRB, 0.80350E+00_JPRB, 0.80423E+00_JPRB, 0.81010E+00_JPRB, 0.81537E+00_JPRB, 0.81438E+00_JPRB, 0.81341E+00_JPRB ,&
&  0.83715E+00_JPRB, 0.87440E+00_JPRB, 0.94374E+00_JPRB, 0.11771E+01_JPRB, 0.19968E+01_JPRB, 0.24525E+01_JPRB, 0.38080E+01_JPRB ,&
&  0.51752E+01_JPRB, 0.43163E+01_JPRB, 0.33271E+01_JPRB, 0.26697E+01_JPRB, 0.22634E+01_JPRB, 0.19993E+01_JPRB, 0.18438E+01_JPRB ,&
&  0.16926E+01_JPRB, 0.16019E+01_JPRB, 0.15264E+01_JPRB, 0.14449E+01_JPRB, 0.13885E+01_JPRB, 0.13027E+01_JPRB, 0.12282E+01_JPRB ,&
&  0.12035E+01_JPRB, 0.11578E+01_JPRB, 0.11073E+01_JPRB, 0.10531E+01_JPRB, 0.10398E+01_JPRB, 0.10377E+01_JPRB, 0.10312E+01_JPRB ,&
&  0.10317E+01_JPRB, 0.10309E+01_JPRB, 0.10319E+01_JPRB, 0.10377E+01_JPRB, 0.10761E+01_JPRB, 0.11092E+01_JPRB, 0.11032E+01_JPRB ,&
&  0.10561E+01_JPRB, 0.10431E+01_JPRB, 0.10541E+01_JPRB, 0.10136E+01_JPRB, 0.98386E+00_JPRB, 0.10509E+01_JPRB, 0.98558E+00_JPRB ,&
&  0.92495E+00_JPRB, 0.89179E+00_JPRB, 0.87238E+00_JPRB, 0.88486E+00_JPRB, 0.94609E+00_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.94622E+00_JPRB, 0.93916E+00_JPRB, 0.92365E+00_JPRB, 0.91239E+00_JPRB, 0.91084E+00_JPRB, 0.89979E+00_JPRB, 0.89204E+00_JPRB ,&
&  0.88890E+00_JPRB, 0.87920E+00_JPRB, 0.87140E+00_JPRB, 0.85774E+00_JPRB, 0.84068E+00_JPRB, 0.82679E+00_JPRB, 0.82287E+00_JPRB ,&
&  0.82406E+00_JPRB, 0.81572E+00_JPRB, 0.82041E+00_JPRB, 0.82247E+00_JPRB, 0.81941E+00_JPRB, 0.80700E+00_JPRB, 0.79453E+00_JPRB ,&
&  0.78757E+00_JPRB, 0.77450E+00_JPRB, 0.77134E+00_JPRB, 0.79171E+00_JPRB, 0.80996E+00_JPRB, 0.81193E+00_JPRB, 0.79731E+00_JPRB ,&
&  0.84320E+00_JPRB, 0.11057E+01_JPRB, 0.15874E+01_JPRB, 0.17213E+01_JPRB, 0.16213E+01_JPRB, 0.14751E+01_JPRB, 0.13203E+01_JPRB ,&
&  0.11900E+01_JPRB, 0.10836E+01_JPRB, 0.10226E+01_JPRB, 0.97229E+00_JPRB, 0.92176E+00_JPRB, 0.88031E+00_JPRB, 0.84641E+00_JPRB ,&
&  0.81055E+00_JPRB, 0.79599E+00_JPRB, 0.78828E+00_JPRB, 0.78684E+00_JPRB, 0.78600E+00_JPRB, 0.78457E+00_JPRB, 0.79041E+00_JPRB ,&
&  0.80702E+00_JPRB, 0.83230E+00_JPRB, 0.87127E+00_JPRB, 0.95432E+00_JPRB, 0.13466E+01_JPRB, 0.19906E+01_JPRB, 0.28619E+01_JPRB ,&
&  0.30955E+01_JPRB, 0.31531E+01_JPRB, 0.28653E+01_JPRB, 0.24585E+01_JPRB, 0.21299E+01_JPRB, 0.18953E+01_JPRB, 0.17417E+01_JPRB ,&
&  0.15956E+01_JPRB, 0.14940E+01_JPRB, 0.14220E+01_JPRB, 0.13342E+01_JPRB, 0.12425E+01_JPRB, 0.11914E+01_JPRB, 0.11939E+01_JPRB ,&
&  0.12119E+01_JPRB, 0.11932E+01_JPRB, 0.11404E+01_JPRB, 0.10778E+01_JPRB, 0.10149E+01_JPRB, 0.98902E+00_JPRB, 0.98049E+00_JPRB ,&
&  0.97472E+00_JPRB, 0.98559E+00_JPRB, 0.10129E+01_JPRB, 0.10436E+01_JPRB, 0.10631E+01_JPRB, 0.10696E+01_JPRB, 0.10659E+01_JPRB ,&
&  0.10752E+01_JPRB, 0.10627E+01_JPRB, 0.10444E+01_JPRB, 0.10374E+01_JPRB, 0.10113E+01_JPRB, 0.10697E+01_JPRB, 0.10252E+01_JPRB ,&
&  0.96146E+00_JPRB, 0.91611E+00_JPRB, 0.89278E+00_JPRB, 0.88042E+00_JPRB, 0.92848E+00_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.96246E+00_JPRB, 0.95644E+00_JPRB, 0.93984E+00_JPRB, 0.92746E+00_JPRB, 0.91167E+00_JPRB, 0.89757E+00_JPRB, 0.88554E+00_JPRB ,&
&  0.87977E+00_JPRB, 0.86505E+00_JPRB, 0.85790E+00_JPRB, 0.85058E+00_JPRB, 0.84780E+00_JPRB, 0.83988E+00_JPRB, 0.82754E+00_JPRB ,&
&  0.81425E+00_JPRB, 0.80705E+00_JPRB, 0.80533E+00_JPRB, 0.81004E+00_JPRB, 0.82024E+00_JPRB, 0.82670E+00_JPRB, 0.82157E+00_JPRB ,&
&  0.81244E+00_JPRB, 0.79818E+00_JPRB, 0.78070E+00_JPRB, 0.79437E+00_JPRB, 0.80996E+00_JPRB, 0.81424E+00_JPRB, 0.79945E+00_JPRB ,&
&  0.81412E+00_JPRB, 0.88925E+00_JPRB, 0.11774E+01_JPRB, 0.12868E+01_JPRB, 0.12960E+01_JPRB, 0.13242E+01_JPRB, 0.12460E+01_JPRB ,&
&  0.11662E+01_JPRB, 0.10646E+01_JPRB, 0.98265E+00_JPRB, 0.94248E+00_JPRB, 0.91755E+00_JPRB, 0.87915E+00_JPRB, 0.84762E+00_JPRB ,&
&  0.80523E+00_JPRB, 0.78514E+00_JPRB, 0.77214E+00_JPRB, 0.76714E+00_JPRB, 0.76460E+00_JPRB, 0.76558E+00_JPRB, 0.77190E+00_JPRB ,&
&  0.78189E+00_JPRB, 0.79515E+00_JPRB, 0.81651E+00_JPRB, 0.85819E+00_JPRB, 0.94717E+00_JPRB, 0.12231E+01_JPRB, 0.16440E+01_JPRB ,&
&  0.19286E+01_JPRB, 0.21366E+01_JPRB, 0.22827E+01_JPRB, 0.22640E+01_JPRB, 0.20731E+01_JPRB, 0.18487E+01_JPRB, 0.16820E+01_JPRB ,&
&  0.15430E+01_JPRB, 0.14093E+01_JPRB, 0.12961E+01_JPRB, 0.12037E+01_JPRB, 0.11215E+01_JPRB, 0.10869E+01_JPRB, 0.11202E+01_JPRB ,&
&  0.11411E+01_JPRB, 0.11343E+01_JPRB, 0.10943E+01_JPRB, 0.10446E+01_JPRB, 0.98399E+00_JPRB, 0.94720E+00_JPRB, 0.93810E+00_JPRB ,&
&  0.94188E+00_JPRB, 0.95828E+00_JPRB, 0.98157E+00_JPRB, 0.10079E+01_JPRB, 0.10297E+01_JPRB, 0.10402E+01_JPRB, 0.10417E+01_JPRB ,&
&  0.10490E+01_JPRB, 0.10507E+01_JPRB, 0.99465E+00_JPRB, 0.97892E+00_JPRB, 0.10609E+01_JPRB, 0.10471E+01_JPRB, 0.10280E+01_JPRB ,&
&  0.98837E+00_JPRB, 0.93068E+00_JPRB, 0.89096E+00_JPRB, 0.86354E+00_JPRB, 0.92003E+00_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.83455E+00_JPRB, 0.86249E+00_JPRB, 0.87293E+00_JPRB, 0.87553E+00_JPRB, 0.86650E+00_JPRB, 0.85182E+00_JPRB, 0.83954E+00_JPRB ,&
&  0.83475E+00_JPRB, 0.82149E+00_JPRB, 0.81617E+00_JPRB, 0.81450E+00_JPRB, 0.81846E+00_JPRB, 0.82634E+00_JPRB, 0.82706E+00_JPRB ,&
&  0.81864E+00_JPRB, 0.80553E+00_JPRB, 0.79691E+00_JPRB, 0.79694E+00_JPRB, 0.80714E+00_JPRB, 0.81797E+00_JPRB, 0.82505E+00_JPRB ,&
&  0.82787E+00_JPRB, 0.82146E+00_JPRB, 0.81817E+00_JPRB, 0.82719E+00_JPRB, 0.83488E+00_JPRB, 0.82662E+00_JPRB, 0.80648E+00_JPRB ,&
&  0.80252E+00_JPRB, 0.75701E+00_JPRB, 0.86771E+00_JPRB, 0.98553E+00_JPRB, 0.10794E+01_JPRB, 0.11180E+01_JPRB, 0.11169E+01_JPRB ,&
&  0.10751E+01_JPRB, 0.10233E+01_JPRB, 0.96137E+00_JPRB, 0.91230E+00_JPRB, 0.87581E+00_JPRB, 0.84946E+00_JPRB, 0.83208E+00_JPRB ,&
&  0.80657E+00_JPRB, 0.78263E+00_JPRB, 0.76161E+00_JPRB, 0.75007E+00_JPRB, 0.74600E+00_JPRB, 0.74250E+00_JPRB, 0.73959E+00_JPRB ,&
&  0.73704E+00_JPRB, 0.73800E+00_JPRB, 0.75152E+00_JPRB, 0.77978E+00_JPRB, 0.82446E+00_JPRB, 0.93362E+00_JPRB, 0.11256E+01_JPRB ,&
&  0.12990E+01_JPRB, 0.14817E+01_JPRB, 0.16606E+01_JPRB, 0.18012E+01_JPRB, 0.18012E+01_JPRB, 0.17164E+01_JPRB, 0.15959E+01_JPRB ,&
&  0.14554E+01_JPRB, 0.12864E+01_JPRB, 0.11385E+01_JPRB, 0.10509E+01_JPRB, 0.10210E+01_JPRB, 0.10244E+01_JPRB, 0.10558E+01_JPRB ,&
&  0.10706E+01_JPRB, 0.10613E+01_JPRB, 0.10189E+01_JPRB, 0.97550E+00_JPRB, 0.94567E+00_JPRB, 0.91533E+00_JPRB, 0.90577E+00_JPRB ,&
&  0.90347E+00_JPRB, 0.91204E+00_JPRB, 0.93055E+00_JPRB, 0.94845E+00_JPRB, 0.97239E+00_JPRB, 0.98779E+00_JPRB, 0.99043E+00_JPRB ,&
&  0.98898E+00_JPRB, 0.98406E+00_JPRB, 0.97950E+00_JPRB, 0.99973E+00_JPRB, 0.98627E+00_JPRB, 0.97820E+00_JPRB, 0.96720E+00_JPRB ,&
&  0.94556E+00_JPRB, 0.90675E+00_JPRB, 0.86342E+00_JPRB, 0.83902E+00_JPRB, 0.77947E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.78181E+00_JPRB, 0.80826E+00_JPRB, 0.82095E+00_JPRB, 0.82459E+00_JPRB, 0.82383E+00_JPRB, 0.81277E+00_JPRB, 0.79913E+00_JPRB ,&
&  0.78243E+00_JPRB, 0.77598E+00_JPRB, 0.77400E+00_JPRB, 0.77898E+00_JPRB, 0.78528E+00_JPRB, 0.79434E+00_JPRB, 0.80694E+00_JPRB ,&
&  0.81894E+00_JPRB, 0.82498E+00_JPRB, 0.82975E+00_JPRB, 0.82825E+00_JPRB, 0.82677E+00_JPRB, 0.82922E+00_JPRB, 0.83356E+00_JPRB ,&
&  0.84245E+00_JPRB, 0.84540E+00_JPRB, 0.84210E+00_JPRB, 0.85035E+00_JPRB, 0.86316E+00_JPRB, 0.85259E+00_JPRB, 0.83330E+00_JPRB ,&
&  0.81894E+00_JPRB, 0.69248E+00_JPRB, 0.71820E+00_JPRB, 0.82963E+00_JPRB, 0.91657E+00_JPRB, 0.10063E+01_JPRB, 0.10320E+01_JPRB ,&
&  0.10115E+01_JPRB, 0.96866E+00_JPRB, 0.92419E+00_JPRB, 0.88307E+00_JPRB, 0.85459E+00_JPRB, 0.82369E+00_JPRB, 0.80242E+00_JPRB ,&
&  0.78820E+00_JPRB, 0.77292E+00_JPRB, 0.75047E+00_JPRB, 0.73233E+00_JPRB, 0.71770E+00_JPRB, 0.70316E+00_JPRB, 0.69541E+00_JPRB ,&
&  0.69721E+00_JPRB, 0.70931E+00_JPRB, 0.72522E+00_JPRB, 0.74795E+00_JPRB, 0.78841E+00_JPRB, 0.84315E+00_JPRB, 0.92953E+00_JPRB ,&
&  0.10240E+01_JPRB, 0.11274E+01_JPRB, 0.12296E+01_JPRB, 0.13200E+01_JPRB, 0.13963E+01_JPRB, 0.14078E+01_JPRB, 0.13432E+01_JPRB ,&
&  0.12252E+01_JPRB, 0.11277E+01_JPRB, 0.10548E+01_JPRB, 0.10066E+01_JPRB, 0.99774E+00_JPRB, 0.10115E+01_JPRB, 0.10202E+01_JPRB ,&
&  0.10075E+01_JPRB, 0.97560E+00_JPRB, 0.93354E+00_JPRB, 0.89541E+00_JPRB, 0.87529E+00_JPRB, 0.86254E+00_JPRB, 0.84816E+00_JPRB ,&
&  0.83596E+00_JPRB, 0.83248E+00_JPRB, 0.83902E+00_JPRB, 0.85772E+00_JPRB, 0.88206E+00_JPRB, 0.90824E+00_JPRB, 0.92160E+00_JPRB ,&
&  0.92937E+00_JPRB, 0.93668E+00_JPRB, 0.95214E+00_JPRB, 0.89898E+00_JPRB, 0.93148E+00_JPRB, 0.93720E+00_JPRB, 0.93464E+00_JPRB ,&
&  0.91621E+00_JPRB, 0.88848E+00_JPRB, 0.85315E+00_JPRB, 0.70559E+00_JPRB, 0.75471E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.76309E+00_JPRB, 0.77139E+00_JPRB, 0.77827E+00_JPRB, 0.77651E+00_JPRB, 0.77394E+00_JPRB, 0.76729E+00_JPRB, 0.75298E+00_JPRB ,&
&  0.73936E+00_JPRB, 0.73448E+00_JPRB, 0.73527E+00_JPRB, 0.73730E+00_JPRB, 0.74078E+00_JPRB, 0.74733E+00_JPRB, 0.75882E+00_JPRB ,&
&  0.77249E+00_JPRB, 0.78681E+00_JPRB, 0.80363E+00_JPRB, 0.81916E+00_JPRB, 0.82256E+00_JPRB, 0.81914E+00_JPRB, 0.81908E+00_JPRB ,&
&  0.82503E+00_JPRB, 0.82385E+00_JPRB, 0.81726E+00_JPRB, 0.81951E+00_JPRB, 0.82761E+00_JPRB, 0.81881E+00_JPRB, 0.80075E+00_JPRB ,&
&  0.78817E+00_JPRB, 0.64431E+00_JPRB, 0.68032E+00_JPRB, 0.74577E+00_JPRB, 0.81722E+00_JPRB, 0.88652E+00_JPRB, 0.91535E+00_JPRB ,&
&  0.90762E+00_JPRB, 0.87453E+00_JPRB, 0.84565E+00_JPRB, 0.81013E+00_JPRB, 0.79427E+00_JPRB, 0.78214E+00_JPRB, 0.76675E+00_JPRB ,&
&  0.74451E+00_JPRB, 0.72135E+00_JPRB, 0.69590E+00_JPRB, 0.67381E+00_JPRB, 0.65631E+00_JPRB, 0.64839E+00_JPRB, 0.64623E+00_JPRB ,&
&  0.64692E+00_JPRB, 0.65808E+00_JPRB, 0.66474E+00_JPRB, 0.67654E+00_JPRB, 0.69688E+00_JPRB, 0.72919E+00_JPRB, 0.77390E+00_JPRB ,&
&  0.83349E+00_JPRB, 0.90364E+00_JPRB, 0.97292E+00_JPRB, 0.10217E+01_JPRB, 0.10537E+01_JPRB, 0.10788E+01_JPRB, 0.10744E+01_JPRB ,&
&  0.10341E+01_JPRB, 0.98995E+00_JPRB, 0.98017E+00_JPRB, 0.97753E+00_JPRB, 0.96410E+00_JPRB, 0.94982E+00_JPRB, 0.92788E+00_JPRB ,&
&  0.89527E+00_JPRB, 0.85798E+00_JPRB, 0.82643E+00_JPRB, 0.79738E+00_JPRB, 0.77903E+00_JPRB, 0.76948E+00_JPRB, 0.75046E+00_JPRB ,&
&  0.73779E+00_JPRB, 0.73557E+00_JPRB, 0.74371E+00_JPRB, 0.76441E+00_JPRB, 0.79232E+00_JPRB, 0.81802E+00_JPRB, 0.83304E+00_JPRB ,&
&  0.83918E+00_JPRB, 0.84626E+00_JPRB, 0.85174E+00_JPRB, 0.87503E+00_JPRB, 0.88034E+00_JPRB, 0.88136E+00_JPRB, 0.87813E+00_JPRB ,&
&  0.87145E+00_JPRB, 0.86411E+00_JPRB, 0.66692E+00_JPRB, 0.72961E+00_JPRB, 0.74410E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.78135E+00_JPRB, 0.76459E+00_JPRB, 0.75620E+00_JPRB, 0.75139E+00_JPRB, 0.74765E+00_JPRB, 0.74130E+00_JPRB, 0.72854E+00_JPRB ,&
&  0.71575E+00_JPRB, 0.70952E+00_JPRB, 0.70385E+00_JPRB, 0.69620E+00_JPRB, 0.69798E+00_JPRB, 0.70424E+00_JPRB, 0.71537E+00_JPRB ,&
&  0.72754E+00_JPRB, 0.74305E+00_JPRB, 0.75790E+00_JPRB, 0.77662E+00_JPRB, 0.78894E+00_JPRB, 0.79749E+00_JPRB, 0.80193E+00_JPRB ,&
&  0.80487E+00_JPRB, 0.80517E+00_JPRB, 0.79768E+00_JPRB, 0.78804E+00_JPRB, 0.78532E+00_JPRB, 0.78632E+00_JPRB, 0.78133E+00_JPRB ,&
&  0.68299E+00_JPRB, 0.59627E+00_JPRB, 0.64985E+00_JPRB, 0.71179E+00_JPRB, 0.77127E+00_JPRB, 0.82116E+00_JPRB, 0.84310E+00_JPRB ,&
&  0.83212E+00_JPRB, 0.80418E+00_JPRB, 0.77766E+00_JPRB, 0.74619E+00_JPRB, 0.73374E+00_JPRB, 0.72882E+00_JPRB, 0.72023E+00_JPRB ,&
&  0.71044E+00_JPRB, 0.69879E+00_JPRB, 0.68430E+00_JPRB, 0.66179E+00_JPRB, 0.64604E+00_JPRB, 0.63460E+00_JPRB, 0.63282E+00_JPRB ,&
&  0.63521E+00_JPRB, 0.63918E+00_JPRB, 0.64278E+00_JPRB, 0.64578E+00_JPRB, 0.65359E+00_JPRB, 0.66667E+00_JPRB, 0.68396E+00_JPRB ,&
&  0.71100E+00_JPRB, 0.74329E+00_JPRB, 0.78594E+00_JPRB, 0.81571E+00_JPRB, 0.83402E+00_JPRB, 0.85072E+00_JPRB, 0.87244E+00_JPRB ,&
&  0.88515E+00_JPRB, 0.88126E+00_JPRB, 0.85774E+00_JPRB, 0.84507E+00_JPRB, 0.84158E+00_JPRB, 0.83590E+00_JPRB, 0.82078E+00_JPRB ,&
&  0.79455E+00_JPRB, 0.76301E+00_JPRB, 0.73102E+00_JPRB, 0.71195E+00_JPRB, 0.70774E+00_JPRB, 0.70536E+00_JPRB, 0.69890E+00_JPRB ,&
&  0.69327E+00_JPRB, 0.69220E+00_JPRB, 0.69765E+00_JPRB, 0.70915E+00_JPRB, 0.72453E+00_JPRB, 0.74304E+00_JPRB, 0.75769E+00_JPRB ,&
&  0.77177E+00_JPRB, 0.78556E+00_JPRB, 0.79592E+00_JPRB, 0.81281E+00_JPRB, 0.82985E+00_JPRB, 0.83778E+00_JPRB, 0.83280E+00_JPRB ,&
&  0.82955E+00_JPRB, 0.81691E+00_JPRB, 0.80737E+00_JPRB, 0.80294E+00_JPRB, 0.79520E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.75052E+00_JPRB, 0.73318E+00_JPRB, 0.72335E+00_JPRB, 0.71946E+00_JPRB, 0.71104E+00_JPRB, 0.69220E+00_JPRB, 0.67664E+00_JPRB ,&
&  0.66252E+00_JPRB, 0.65514E+00_JPRB, 0.65236E+00_JPRB, 0.65192E+00_JPRB, 0.65794E+00_JPRB, 0.66360E+00_JPRB, 0.67044E+00_JPRB ,&
&  0.68176E+00_JPRB, 0.69993E+00_JPRB, 0.71548E+00_JPRB, 0.72981E+00_JPRB, 0.74266E+00_JPRB, 0.75538E+00_JPRB, 0.75877E+00_JPRB ,&
&  0.76145E+00_JPRB, 0.76890E+00_JPRB, 0.77037E+00_JPRB, 0.75799E+00_JPRB, 0.74490E+00_JPRB, 0.74732E+00_JPRB, 0.74422E+00_JPRB ,&
&  0.73460E+00_JPRB, 0.62084E+00_JPRB, 0.67344E+00_JPRB, 0.69759E+00_JPRB, 0.72500E+00_JPRB, 0.75029E+00_JPRB, 0.76379E+00_JPRB ,&
&  0.76024E+00_JPRB, 0.74382E+00_JPRB, 0.72439E+00_JPRB, 0.69756E+00_JPRB, 0.68953E+00_JPRB, 0.68288E+00_JPRB, 0.67454E+00_JPRB ,&
&  0.66909E+00_JPRB, 0.66365E+00_JPRB, 0.65594E+00_JPRB, 0.64506E+00_JPRB, 0.63091E+00_JPRB, 0.62044E+00_JPRB, 0.61122E+00_JPRB ,&
&  0.60897E+00_JPRB, 0.60176E+00_JPRB, 0.59508E+00_JPRB, 0.59575E+00_JPRB, 0.60755E+00_JPRB, 0.61398E+00_JPRB, 0.61617E+00_JPRB ,&
&  0.62000E+00_JPRB, 0.62748E+00_JPRB, 0.64274E+00_JPRB, 0.66343E+00_JPRB, 0.68031E+00_JPRB, 0.69295E+00_JPRB, 0.71050E+00_JPRB ,&
&  0.73779E+00_JPRB, 0.75747E+00_JPRB, 0.75894E+00_JPRB, 0.74474E+00_JPRB, 0.73429E+00_JPRB, 0.73088E+00_JPRB, 0.71651E+00_JPRB ,&
&  0.69348E+00_JPRB, 0.67684E+00_JPRB, 0.67010E+00_JPRB, 0.66661E+00_JPRB, 0.66128E+00_JPRB, 0.65472E+00_JPRB, 0.65371E+00_JPRB ,&
&  0.65475E+00_JPRB, 0.65473E+00_JPRB, 0.65360E+00_JPRB, 0.65622E+00_JPRB, 0.66217E+00_JPRB, 0.66992E+00_JPRB, 0.68065E+00_JPRB ,&
&  0.69630E+00_JPRB, 0.71068E+00_JPRB, 0.71869E+00_JPRB, 0.72934E+00_JPRB, 0.74252E+00_JPRB, 0.74848E+00_JPRB, 0.76220E+00_JPRB ,&
&  0.76186E+00_JPRB, 0.75681E+00_JPRB, 0.75464E+00_JPRB, 0.75963E+00_JPRB, 0.76158E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.71801E+00_JPRB, 0.69854E+00_JPRB, 0.68699E+00_JPRB, 0.67569E+00_JPRB, 0.66767E+00_JPRB, 0.65003E+00_JPRB, 0.64009E+00_JPRB ,&
&  0.63377E+00_JPRB, 0.62990E+00_JPRB, 0.62953E+00_JPRB, 0.63009E+00_JPRB, 0.64033E+00_JPRB, 0.64688E+00_JPRB, 0.64936E+00_JPRB ,&
&  0.65697E+00_JPRB, 0.67448E+00_JPRB, 0.69063E+00_JPRB, 0.70417E+00_JPRB, 0.71486E+00_JPRB, 0.72261E+00_JPRB, 0.72252E+00_JPRB ,&
&  0.72628E+00_JPRB, 0.73430E+00_JPRB, 0.73822E+00_JPRB, 0.72824E+00_JPRB, 0.71559E+00_JPRB, 0.70890E+00_JPRB, 0.70910E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.65335E+00_JPRB, 0.66767E+00_JPRB, 0.67938E+00_JPRB, 0.69288E+00_JPRB, 0.69965E+00_JPRB, 0.70676E+00_JPRB ,&
&  0.70834E+00_JPRB, 0.69182E+00_JPRB, 0.67101E+00_JPRB, 0.65575E+00_JPRB, 0.65905E+00_JPRB, 0.65660E+00_JPRB, 0.64777E+00_JPRB ,&
&  0.64427E+00_JPRB, 0.64141E+00_JPRB, 0.63442E+00_JPRB, 0.62177E+00_JPRB, 0.61033E+00_JPRB, 0.60268E+00_JPRB, 0.59765E+00_JPRB ,&
&  0.58662E+00_JPRB, 0.57750E+00_JPRB, 0.57256E+00_JPRB, 0.56703E+00_JPRB, 0.56241E+00_JPRB, 0.56690E+00_JPRB, 0.57103E+00_JPRB ,&
&  0.57191E+00_JPRB, 0.57498E+00_JPRB, 0.58518E+00_JPRB, 0.60189E+00_JPRB, 0.61676E+00_JPRB, 0.63296E+00_JPRB, 0.65117E+00_JPRB ,&
&  0.66989E+00_JPRB, 0.68754E+00_JPRB, 0.69479E+00_JPRB, 0.69986E+00_JPRB, 0.69419E+00_JPRB, 0.68422E+00_JPRB, 0.66417E+00_JPRB ,&
&  0.64193E+00_JPRB, 0.63596E+00_JPRB, 0.63532E+00_JPRB, 0.63973E+00_JPRB, 0.63703E+00_JPRB, 0.63374E+00_JPRB, 0.63215E+00_JPRB ,&
&  0.62820E+00_JPRB, 0.62465E+00_JPRB, 0.61958E+00_JPRB, 0.61484E+00_JPRB, 0.61794E+00_JPRB, 0.62350E+00_JPRB, 0.63658E+00_JPRB ,&
&  0.65428E+00_JPRB, 0.67320E+00_JPRB, 0.69271E+00_JPRB, 0.70572E+00_JPRB, 0.70990E+00_JPRB, 0.71256E+00_JPRB, 0.71561E+00_JPRB ,&
&  0.71640E+00_JPRB, 0.71429E+00_JPRB, 0.71968E+00_JPRB, 0.72363E+00_JPRB, 0.72792E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.66549E+00_JPRB, 0.64060E+00_JPRB, 0.62263E+00_JPRB, 0.61603E+00_JPRB, 0.61259E+00_JPRB, 0.60073E+00_JPRB, 0.59566E+00_JPRB ,&
&  0.60174E+00_JPRB, 0.60357E+00_JPRB, 0.61070E+00_JPRB, 0.61046E+00_JPRB, 0.60979E+00_JPRB, 0.61242E+00_JPRB, 0.60484E+00_JPRB ,&
&  0.60492E+00_JPRB, 0.61397E+00_JPRB, 0.62464E+00_JPRB, 0.63904E+00_JPRB, 0.65116E+00_JPRB, 0.66031E+00_JPRB, 0.66808E+00_JPRB ,&
&  0.67116E+00_JPRB, 0.67768E+00_JPRB, 0.68126E+00_JPRB, 0.67705E+00_JPRB, 0.66609E+00_JPRB, 0.65750E+00_JPRB, 0.65595E+00_JPRB ,&
&  0.65451E+00_JPRB, 0.64850E+00_JPRB, 0.60419E+00_JPRB, 0.62804E+00_JPRB, 0.63498E+00_JPRB, 0.63829E+00_JPRB, 0.64045E+00_JPRB ,&
&  0.63515E+00_JPRB, 0.62019E+00_JPRB, 0.61305E+00_JPRB, 0.60088E+00_JPRB, 0.59948E+00_JPRB, 0.60545E+00_JPRB, 0.60417E+00_JPRB ,&
&  0.60020E+00_JPRB, 0.59017E+00_JPRB, 0.57655E+00_JPRB, 0.56592E+00_JPRB, 0.56331E+00_JPRB, 0.56141E+00_JPRB, 0.55393E+00_JPRB ,&
&  0.54877E+00_JPRB, 0.54797E+00_JPRB, 0.54628E+00_JPRB, 0.54044E+00_JPRB, 0.52572E+00_JPRB, 0.51232E+00_JPRB, 0.51020E+00_JPRB ,&
&  0.51448E+00_JPRB, 0.52276E+00_JPRB, 0.53821E+00_JPRB, 0.55762E+00_JPRB, 0.57814E+00_JPRB, 0.59729E+00_JPRB, 0.61428E+00_JPRB ,&
&  0.62101E+00_JPRB, 0.62437E+00_JPRB, 0.62834E+00_JPRB, 0.63443E+00_JPRB, 0.63639E+00_JPRB, 0.62878E+00_JPRB, 0.60870E+00_JPRB ,&
&  0.58694E+00_JPRB, 0.58050E+00_JPRB, 0.58349E+00_JPRB, 0.59158E+00_JPRB, 0.59536E+00_JPRB, 0.59445E+00_JPRB, 0.58788E+00_JPRB ,&
&  0.58395E+00_JPRB, 0.57567E+00_JPRB, 0.56681E+00_JPRB, 0.56242E+00_JPRB, 0.56207E+00_JPRB, 0.56801E+00_JPRB, 0.58530E+00_JPRB ,&
&  0.60612E+00_JPRB, 0.62935E+00_JPRB, 0.65006E+00_JPRB, 0.65740E+00_JPRB, 0.66105E+00_JPRB, 0.65985E+00_JPRB, 0.65065E+00_JPRB ,&
&  0.65463E+00_JPRB, 0.67326E+00_JPRB, 0.68251E+00_JPRB, 0.68369E+00_JPRB, 0.68102E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.61542E+00_JPRB, 0.58690E+00_JPRB, 0.57567E+00_JPRB, 0.57749E+00_JPRB, 0.58094E+00_JPRB, 0.57684E+00_JPRB, 0.58421E+00_JPRB ,&
&  0.59543E+00_JPRB, 0.60393E+00_JPRB, 0.60862E+00_JPRB, 0.60491E+00_JPRB, 0.60245E+00_JPRB, 0.59951E+00_JPRB, 0.58479E+00_JPRB ,&
&  0.57994E+00_JPRB, 0.57822E+00_JPRB, 0.58237E+00_JPRB, 0.58949E+00_JPRB, 0.59726E+00_JPRB, 0.60947E+00_JPRB, 0.61853E+00_JPRB ,&
&  0.62448E+00_JPRB, 0.62819E+00_JPRB, 0.63235E+00_JPRB, 0.63253E+00_JPRB, 0.62893E+00_JPRB, 0.61977E+00_JPRB, 0.61272E+00_JPRB ,&
&  0.60849E+00_JPRB, 0.59687E+00_JPRB, 0.58649E+00_JPRB, 0.58506E+00_JPRB, 0.58801E+00_JPRB, 0.59087E+00_JPRB, 0.59018E+00_JPRB ,&
&  0.57737E+00_JPRB, 0.56078E+00_JPRB, 0.55910E+00_JPRB, 0.55881E+00_JPRB, 0.56389E+00_JPRB, 0.57589E+00_JPRB, 0.57925E+00_JPRB ,&
&  0.57608E+00_JPRB, 0.56208E+00_JPRB, 0.54308E+00_JPRB, 0.53955E+00_JPRB, 0.54184E+00_JPRB, 0.53532E+00_JPRB, 0.52888E+00_JPRB ,&
&  0.52779E+00_JPRB, 0.52485E+00_JPRB, 0.51889E+00_JPRB, 0.51069E+00_JPRB, 0.49786E+00_JPRB, 0.48132E+00_JPRB, 0.47303E+00_JPRB ,&
&  0.47321E+00_JPRB, 0.48960E+00_JPRB, 0.51803E+00_JPRB, 0.54949E+00_JPRB, 0.57903E+00_JPRB, 0.59329E+00_JPRB, 0.59226E+00_JPRB ,&
&  0.57912E+00_JPRB, 0.57514E+00_JPRB, 0.57978E+00_JPRB, 0.58453E+00_JPRB, 0.58632E+00_JPRB, 0.57633E+00_JPRB, 0.55926E+00_JPRB ,&
&  0.54660E+00_JPRB, 0.53724E+00_JPRB, 0.53595E+00_JPRB, 0.54048E+00_JPRB, 0.54496E+00_JPRB, 0.54591E+00_JPRB, 0.53996E+00_JPRB ,&
&  0.53395E+00_JPRB, 0.52752E+00_JPRB, 0.52356E+00_JPRB, 0.52614E+00_JPRB, 0.53124E+00_JPRB, 0.53640E+00_JPRB, 0.54934E+00_JPRB ,&
&  0.56807E+00_JPRB, 0.58680E+00_JPRB, 0.59104E+00_JPRB, 0.58660E+00_JPRB, 0.58588E+00_JPRB, 0.58293E+00_JPRB, 0.58011E+00_JPRB ,&
&  0.59006E+00_JPRB, 0.61213E+00_JPRB, 0.62789E+00_JPRB, 0.63645E+00_JPRB, 0.63638E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.54090E+00_JPRB, 0.52412E+00_JPRB, 0.52464E+00_JPRB, 0.53189E+00_JPRB, 0.53601E+00_JPRB, 0.54386E+00_JPRB, 0.56338E+00_JPRB ,&
&  0.57479E+00_JPRB, 0.58204E+00_JPRB, 0.58490E+00_JPRB, 0.57969E+00_JPRB, 0.57875E+00_JPRB, 0.57152E+00_JPRB, 0.55876E+00_JPRB ,&
&  0.55345E+00_JPRB, 0.55239E+00_JPRB, 0.55403E+00_JPRB, 0.55785E+00_JPRB, 0.56013E+00_JPRB, 0.56301E+00_JPRB, 0.56630E+00_JPRB ,&
&  0.56807E+00_JPRB, 0.56806E+00_JPRB, 0.56720E+00_JPRB, 0.56741E+00_JPRB, 0.56642E+00_JPRB, 0.55878E+00_JPRB, 0.55014E+00_JPRB ,&
&  0.54074E+00_JPRB, 0.53147E+00_JPRB, 0.52874E+00_JPRB, 0.53300E+00_JPRB, 0.53787E+00_JPRB, 0.54082E+00_JPRB, 0.53845E+00_JPRB ,&
&  0.53380E+00_JPRB, 0.53122E+00_JPRB, 0.53362E+00_JPRB, 0.54052E+00_JPRB, 0.54903E+00_JPRB, 0.55853E+00_JPRB, 0.56353E+00_JPRB ,&
&  0.55792E+00_JPRB, 0.54034E+00_JPRB, 0.52967E+00_JPRB, 0.52911E+00_JPRB, 0.52538E+00_JPRB, 0.51086E+00_JPRB, 0.50073E+00_JPRB ,&
&  0.49598E+00_JPRB, 0.48719E+00_JPRB, 0.47790E+00_JPRB, 0.46521E+00_JPRB, 0.45354E+00_JPRB, 0.44916E+00_JPRB, 0.44809E+00_JPRB ,&
&  0.45206E+00_JPRB, 0.47158E+00_JPRB, 0.50600E+00_JPRB, 0.53895E+00_JPRB, 0.55298E+00_JPRB, 0.54619E+00_JPRB, 0.52871E+00_JPRB ,&
&  0.52013E+00_JPRB, 0.51604E+00_JPRB, 0.51700E+00_JPRB, 0.51684E+00_JPRB, 0.51393E+00_JPRB, 0.50555E+00_JPRB, 0.49843E+00_JPRB ,&
&  0.48822E+00_JPRB, 0.47703E+00_JPRB, 0.47251E+00_JPRB, 0.46963E+00_JPRB, 0.46654E+00_JPRB, 0.46654E+00_JPRB, 0.46568E+00_JPRB ,&
&  0.46636E+00_JPRB, 0.47032E+00_JPRB, 0.48181E+00_JPRB, 0.48808E+00_JPRB, 0.48882E+00_JPRB, 0.47864E+00_JPRB, 0.47987E+00_JPRB ,&
&  0.48287E+00_JPRB, 0.48066E+00_JPRB, 0.47598E+00_JPRB, 0.47566E+00_JPRB, 0.47546E+00_JPRB, 0.47719E+00_JPRB, 0.48873E+00_JPRB ,&
&  0.51542E+00_JPRB, 0.53874E+00_JPRB, 0.55775E+00_JPRB, 0.56300E+00_JPRB, 0.55856E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.48576E+00_JPRB, 0.49093E+00_JPRB, 0.50396E+00_JPRB, 0.51473E+00_JPRB, 0.51828E+00_JPRB, 0.52792E+00_JPRB, 0.53918E+00_JPRB ,&
&  0.54827E+00_JPRB, 0.54998E+00_JPRB, 0.54625E+00_JPRB, 0.53608E+00_JPRB, 0.52404E+00_JPRB, 0.51407E+00_JPRB, 0.51013E+00_JPRB ,&
&  0.51231E+00_JPRB, 0.51535E+00_JPRB, 0.52087E+00_JPRB, 0.52394E+00_JPRB, 0.52766E+00_JPRB, 0.52655E+00_JPRB, 0.52707E+00_JPRB ,&
&  0.52896E+00_JPRB, 0.53004E+00_JPRB, 0.53057E+00_JPRB, 0.52689E+00_JPRB, 0.52393E+00_JPRB, 0.52134E+00_JPRB, 0.51267E+00_JPRB ,&
&  0.50304E+00_JPRB, 0.49657E+00_JPRB, 0.50042E+00_JPRB, 0.51175E+00_JPRB, 0.51895E+00_JPRB, 0.51603E+00_JPRB, 0.51461E+00_JPRB ,&
&  0.51484E+00_JPRB, 0.51977E+00_JPRB, 0.52770E+00_JPRB, 0.53948E+00_JPRB, 0.54772E+00_JPRB, 0.55866E+00_JPRB, 0.55923E+00_JPRB ,&
&  0.55279E+00_JPRB, 0.54356E+00_JPRB, 0.54061E+00_JPRB, 0.53933E+00_JPRB, 0.51032E+00_JPRB, 0.47858E+00_JPRB, 0.46305E+00_JPRB ,&
&  0.45472E+00_JPRB, 0.44433E+00_JPRB, 0.42855E+00_JPRB, 0.41749E+00_JPRB, 0.41635E+00_JPRB, 0.42121E+00_JPRB, 0.43282E+00_JPRB ,&
&  0.44217E+00_JPRB, 0.45564E+00_JPRB, 0.47110E+00_JPRB, 0.48101E+00_JPRB, 0.48100E+00_JPRB, 0.46847E+00_JPRB, 0.45676E+00_JPRB ,&
&  0.44965E+00_JPRB, 0.44745E+00_JPRB, 0.44579E+00_JPRB, 0.44373E+00_JPRB, 0.44194E+00_JPRB, 0.44147E+00_JPRB, 0.44045E+00_JPRB ,&
&  0.43625E+00_JPRB, 0.42909E+00_JPRB, 0.42037E+00_JPRB, 0.41541E+00_JPRB, 0.41442E+00_JPRB, 0.41440E+00_JPRB, 0.41609E+00_JPRB ,&
&  0.42509E+00_JPRB, 0.43644E+00_JPRB, 0.44549E+00_JPRB, 0.44938E+00_JPRB, 0.44620E+00_JPRB, 0.43821E+00_JPRB, 0.43343E+00_JPRB ,&
&  0.42988E+00_JPRB, 0.42736E+00_JPRB, 0.42398E+00_JPRB, 0.42240E+00_JPRB, 0.42216E+00_JPRB, 0.42942E+00_JPRB, 0.44508E+00_JPRB ,&
&  0.46451E+00_JPRB, 0.48982E+00_JPRB, 0.50117E+00_JPRB, 0.49949E+00_JPRB, 0.49211E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.45625E+00_JPRB, 0.46897E+00_JPRB, 0.48508E+00_JPRB, 0.49650E+00_JPRB, 0.50144E+00_JPRB, 0.50688E+00_JPRB, 0.50936E+00_JPRB ,&
&  0.50414E+00_JPRB, 0.50194E+00_JPRB, 0.49638E+00_JPRB, 0.48844E+00_JPRB, 0.48153E+00_JPRB, 0.47803E+00_JPRB, 0.47838E+00_JPRB ,&
&  0.48050E+00_JPRB, 0.48233E+00_JPRB, 0.47886E+00_JPRB, 0.47715E+00_JPRB, 0.48115E+00_JPRB, 0.48656E+00_JPRB, 0.49009E+00_JPRB ,&
&  0.49155E+00_JPRB, 0.49304E+00_JPRB, 0.49781E+00_JPRB, 0.50083E+00_JPRB, 0.50093E+00_JPRB, 0.49430E+00_JPRB, 0.48935E+00_JPRB ,&
&  0.48430E+00_JPRB, 0.48251E+00_JPRB, 0.48896E+00_JPRB, 0.48103E+00_JPRB, 0.49575E+00_JPRB, 0.49765E+00_JPRB, 0.50313E+00_JPRB ,&
&  0.51131E+00_JPRB, 0.52492E+00_JPRB, 0.53546E+00_JPRB, 0.54346E+00_JPRB, 0.54583E+00_JPRB, 0.54894E+00_JPRB, 0.53910E+00_JPRB ,&
&  0.52539E+00_JPRB, 0.50682E+00_JPRB, 0.48810E+00_JPRB, 0.46773E+00_JPRB, 0.44814E+00_JPRB, 0.43298E+00_JPRB, 0.42565E+00_JPRB ,&
&  0.41671E+00_JPRB, 0.40282E+00_JPRB, 0.38946E+00_JPRB, 0.38426E+00_JPRB, 0.38892E+00_JPRB, 0.39909E+00_JPRB, 0.41485E+00_JPRB ,&
&  0.42494E+00_JPRB, 0.42827E+00_JPRB, 0.42530E+00_JPRB, 0.42112E+00_JPRB, 0.41673E+00_JPRB, 0.40932E+00_JPRB, 0.40493E+00_JPRB ,&
&  0.40135E+00_JPRB, 0.39558E+00_JPRB, 0.39254E+00_JPRB, 0.39419E+00_JPRB, 0.39729E+00_JPRB, 0.39935E+00_JPRB, 0.39793E+00_JPRB ,&
&  0.39506E+00_JPRB, 0.39055E+00_JPRB, 0.38248E+00_JPRB, 0.37818E+00_JPRB, 0.37857E+00_JPRB, 0.38299E+00_JPRB, 0.39050E+00_JPRB ,&
&  0.39293E+00_JPRB, 0.39897E+00_JPRB, 0.40488E+00_JPRB, 0.40702E+00_JPRB, 0.40958E+00_JPRB, 0.41161E+00_JPRB, 0.41170E+00_JPRB ,&
&  0.40996E+00_JPRB, 0.40494E+00_JPRB, 0.39807E+00_JPRB, 0.39519E+00_JPRB, 0.39639E+00_JPRB, 0.40325E+00_JPRB, 0.41249E+00_JPRB ,&
&  0.42154E+00_JPRB, 0.43540E+00_JPRB, 0.44583E+00_JPRB, 0.44808E+00_JPRB, 0.45017E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.43601E+00_JPRB, 0.44753E+00_JPRB, 0.46103E+00_JPRB, 0.47042E+00_JPRB, 0.46946E+00_JPRB, 0.46943E+00_JPRB, 0.47554E+00_JPRB ,&
&  0.46934E+00_JPRB, 0.46205E+00_JPRB, 0.45612E+00_JPRB, 0.45010E+00_JPRB, 0.44801E+00_JPRB, 0.44670E+00_JPRB, 0.44968E+00_JPRB ,&
&  0.45185E+00_JPRB, 0.45081E+00_JPRB, 0.44926E+00_JPRB, 0.45011E+00_JPRB, 0.45680E+00_JPRB, 0.46512E+00_JPRB, 0.47204E+00_JPRB ,&
&  0.47336E+00_JPRB, 0.47692E+00_JPRB, 0.48265E+00_JPRB, 0.48751E+00_JPRB, 0.48995E+00_JPRB, 0.49140E+00_JPRB, 0.49012E+00_JPRB ,&
&  0.48750E+00_JPRB, 0.48516E+00_JPRB, 0.46816E+00_JPRB, 0.48166E+00_JPRB, 0.48613E+00_JPRB, 0.48695E+00_JPRB, 0.49237E+00_JPRB ,&
&  0.50017E+00_JPRB, 0.50907E+00_JPRB, 0.51326E+00_JPRB, 0.51453E+00_JPRB, 0.50975E+00_JPRB, 0.50031E+00_JPRB, 0.47599E+00_JPRB ,&
&  0.45560E+00_JPRB, 0.43442E+00_JPRB, 0.42117E+00_JPRB, 0.41308E+00_JPRB, 0.40668E+00_JPRB, 0.40120E+00_JPRB, 0.39603E+00_JPRB ,&
&  0.38892E+00_JPRB, 0.38256E+00_JPRB, 0.37866E+00_JPRB, 0.37865E+00_JPRB, 0.38277E+00_JPRB, 0.39367E+00_JPRB, 0.40714E+00_JPRB ,&
&  0.41328E+00_JPRB, 0.40599E+00_JPRB, 0.39809E+00_JPRB, 0.38934E+00_JPRB, 0.37508E+00_JPRB, 0.34779E+00_JPRB, 0.33096E+00_JPRB ,&
&  0.32690E+00_JPRB, 0.33948E+00_JPRB, 0.36072E+00_JPRB, 0.36673E+00_JPRB, 0.37197E+00_JPRB, 0.36952E+00_JPRB, 0.36409E+00_JPRB ,&
&  0.35516E+00_JPRB, 0.33176E+00_JPRB, 0.31913E+00_JPRB, 0.31542E+00_JPRB, 0.31319E+00_JPRB, 0.31353E+00_JPRB, 0.31613E+00_JPRB ,&
&  0.32041E+00_JPRB, 0.32768E+00_JPRB, 0.33277E+00_JPRB, 0.33485E+00_JPRB, 0.33791E+00_JPRB, 0.33812E+00_JPRB, 0.33706E+00_JPRB ,&
&  0.33351E+00_JPRB, 0.34028E+00_JPRB, 0.35876E+00_JPRB, 0.38710E+00_JPRB, 0.39356E+00_JPRB, 0.40126E+00_JPRB, 0.41014E+00_JPRB ,&
&  0.41555E+00_JPRB, 0.42071E+00_JPRB, 0.42116E+00_JPRB, 0.42197E+00_JPRB, 0.42710E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.41230E+00_JPRB, 0.41938E+00_JPRB, 0.42789E+00_JPRB, 0.43897E+00_JPRB, 0.44144E+00_JPRB, 0.44307E+00_JPRB, 0.45213E+00_JPRB ,&
&  0.45241E+00_JPRB, 0.44937E+00_JPRB, 0.44613E+00_JPRB, 0.44641E+00_JPRB, 0.44857E+00_JPRB, 0.45100E+00_JPRB, 0.45059E+00_JPRB ,&
&  0.44153E+00_JPRB, 0.43634E+00_JPRB, 0.43913E+00_JPRB, 0.44219E+00_JPRB, 0.44678E+00_JPRB, 0.45326E+00_JPRB, 0.45890E+00_JPRB ,&
&  0.46195E+00_JPRB, 0.46556E+00_JPRB, 0.46806E+00_JPRB, 0.47429E+00_JPRB, 0.48387E+00_JPRB, 0.49044E+00_JPRB, 0.49663E+00_JPRB ,&
&  0.48222E+00_JPRB, 0.43448E+00_JPRB, 0.40405E+00_JPRB, 0.41033E+00_JPRB, 0.45947E+00_JPRB, 0.46984E+00_JPRB, 0.47071E+00_JPRB ,&
&  0.47374E+00_JPRB, 0.48136E+00_JPRB, 0.48090E+00_JPRB, 0.47505E+00_JPRB, 0.46319E+00_JPRB, 0.44416E+00_JPRB, 0.42096E+00_JPRB ,&
&  0.40546E+00_JPRB, 0.39520E+00_JPRB, 0.38840E+00_JPRB, 0.38169E+00_JPRB, 0.37819E+00_JPRB, 0.37318E+00_JPRB, 0.36546E+00_JPRB ,&
&  0.36241E+00_JPRB, 0.36110E+00_JPRB, 0.36221E+00_JPRB, 0.36377E+00_JPRB, 0.36563E+00_JPRB, 0.36689E+00_JPRB, 0.36370E+00_JPRB ,&
&  0.35158E+00_JPRB, 0.33594E+00_JPRB, 0.32111E+00_JPRB, 0.30496E+00_JPRB, 0.29115E+00_JPRB, 0.28257E+00_JPRB, 0.27568E+00_JPRB ,&
&  0.27303E+00_JPRB, 0.27891E+00_JPRB, 0.29604E+00_JPRB, 0.31566E+00_JPRB, 0.34727E+00_JPRB, 0.32250E+00_JPRB, 0.29884E+00_JPRB ,&
&  0.28732E+00_JPRB, 0.28164E+00_JPRB, 0.27845E+00_JPRB, 0.27831E+00_JPRB, 0.27982E+00_JPRB, 0.28024E+00_JPRB, 0.28032E+00_JPRB ,&
&  0.28164E+00_JPRB, 0.28408E+00_JPRB, 0.28598E+00_JPRB, 0.28764E+00_JPRB, 0.29020E+00_JPRB, 0.29314E+00_JPRB, 0.29470E+00_JPRB ,&
&  0.29550E+00_JPRB, 0.30034E+00_JPRB, 0.31002E+00_JPRB, 0.32126E+00_JPRB, 0.33240E+00_JPRB, 0.34397E+00_JPRB, 0.36143E+00_JPRB ,&
&  0.38323E+00_JPRB, 0.40941E+00_JPRB, 0.40902E+00_JPRB, 0.40921E+00_JPRB, 0.40935E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.40022E+00_JPRB, 0.41062E+00_JPRB, 0.41604E+00_JPRB, 0.42143E+00_JPRB, 0.42372E+00_JPRB, 0.42521E+00_JPRB, 0.43318E+00_JPRB ,&
&  0.43799E+00_JPRB, 0.44178E+00_JPRB, 0.44414E+00_JPRB, 0.44521E+00_JPRB, 0.44113E+00_JPRB, 0.43464E+00_JPRB, 0.43265E+00_JPRB ,&
&  0.43150E+00_JPRB, 0.43200E+00_JPRB, 0.43099E+00_JPRB, 0.43100E+00_JPRB, 0.43153E+00_JPRB, 0.43630E+00_JPRB, 0.44310E+00_JPRB ,&
&  0.44468E+00_JPRB, 0.44911E+00_JPRB, 0.46185E+00_JPRB, 0.46955E+00_JPRB, 0.47440E+00_JPRB, 0.47389E+00_JPRB, 0.46805E+00_JPRB ,&
&  0.44037E+00_JPRB, 0.41237E+00_JPRB, 0.38516E+00_JPRB, 0.38659E+00_JPRB, 0.43954E+00_JPRB, 0.44580E+00_JPRB, 0.44516E+00_JPRB ,&
&  0.44363E+00_JPRB, 0.44791E+00_JPRB, 0.44812E+00_JPRB, 0.43354E+00_JPRB, 0.42292E+00_JPRB, 0.40826E+00_JPRB, 0.39563E+00_JPRB ,&
&  0.38475E+00_JPRB, 0.37711E+00_JPRB, 0.36384E+00_JPRB, 0.33643E+00_JPRB, 0.31894E+00_JPRB, 0.30995E+00_JPRB, 0.29613E+00_JPRB ,&
&  0.28570E+00_JPRB, 0.28168E+00_JPRB, 0.28070E+00_JPRB, 0.28009E+00_JPRB, 0.28414E+00_JPRB, 0.28609E+00_JPRB, 0.28468E+00_JPRB ,&
&  0.28131E+00_JPRB, 0.27851E+00_JPRB, 0.27479E+00_JPRB, 0.26965E+00_JPRB, 0.26695E+00_JPRB, 0.26616E+00_JPRB, 0.26669E+00_JPRB ,&
&  0.27067E+00_JPRB, 0.27679E+00_JPRB, 0.29028E+00_JPRB, 0.29982E+00_JPRB, 0.29606E+00_JPRB, 0.28461E+00_JPRB, 0.27418E+00_JPRB ,&
&  0.26736E+00_JPRB, 0.26454E+00_JPRB, 0.26240E+00_JPRB, 0.26020E+00_JPRB, 0.26077E+00_JPRB, 0.26278E+00_JPRB, 0.26423E+00_JPRB ,&
&  0.26452E+00_JPRB, 0.26625E+00_JPRB, 0.26727E+00_JPRB, 0.26851E+00_JPRB, 0.26959E+00_JPRB, 0.27134E+00_JPRB, 0.27373E+00_JPRB ,&
&  0.27601E+00_JPRB, 0.28130E+00_JPRB, 0.28715E+00_JPRB, 0.29374E+00_JPRB, 0.29789E+00_JPRB, 0.30102E+00_JPRB, 0.30724E+00_JPRB ,&
&  0.31350E+00_JPRB, 0.32918E+00_JPRB, 0.36017E+00_JPRB, 0.39687E+00_JPRB, 0.39092E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.38763E+00_JPRB, 0.39644E+00_JPRB, 0.40418E+00_JPRB, 0.41021E+00_JPRB, 0.41402E+00_JPRB, 0.41482E+00_JPRB, 0.41760E+00_JPRB ,&
&  0.42514E+00_JPRB, 0.42958E+00_JPRB, 0.43286E+00_JPRB, 0.43214E+00_JPRB, 0.43162E+00_JPRB, 0.41302E+00_JPRB, 0.40835E+00_JPRB ,&
&  0.40231E+00_JPRB, 0.39810E+00_JPRB, 0.39231E+00_JPRB, 0.39225E+00_JPRB, 0.39203E+00_JPRB, 0.39403E+00_JPRB, 0.39479E+00_JPRB ,&
&  0.39500E+00_JPRB, 0.39317E+00_JPRB, 0.39004E+00_JPRB, 0.38812E+00_JPRB, 0.38936E+00_JPRB, 0.39074E+00_JPRB, 0.38776E+00_JPRB ,&
&  0.38342E+00_JPRB, 0.38246E+00_JPRB, 0.38568E+00_JPRB, 0.39283E+00_JPRB, 0.43318E+00_JPRB, 0.42790E+00_JPRB, 0.42556E+00_JPRB ,&
&  0.42225E+00_JPRB, 0.41807E+00_JPRB, 0.41749E+00_JPRB, 0.41088E+00_JPRB, 0.40132E+00_JPRB, 0.37993E+00_JPRB, 0.36804E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.31461E+00_JPRB, 0.29787E+00_JPRB, 0.28695E+00_JPRB, 0.27663E+00_JPRB, 0.27064E+00_JPRB, 0.26525E+00_JPRB ,&
&  0.26117E+00_JPRB, 0.25797E+00_JPRB, 0.25510E+00_JPRB, 0.25559E+00_JPRB, 0.25731E+00_JPRB, 0.25826E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25975E+00_JPRB, 0.26025E+00_JPRB, 0.26091E+00_JPRB, 0.26228E+00_JPRB, 0.26407E+00_JPRB, 0.26644E+00_JPRB, 0.26944E+00_JPRB ,&
&  0.27434E+00_JPRB, 0.27820E+00_JPRB, 0.27971E+00_JPRB, 0.28235E+00_JPRB, 0.28202E+00_JPRB, 0.27629E+00_JPRB, 0.26975E+00_JPRB ,&
&  0.26291E+00_JPRB, 0.25933E+00_JPRB, 0.25583E+00_JPRB, 0.25337E+00_JPRB, 0.25278E+00_JPRB, 0.25275E+00_JPRB, 0.25468E+00_JPRB ,&
&  0.25681E+00_JPRB, 0.25870E+00_JPRB, 0.26002E+00_JPRB, 0.26147E+00_JPRB, 0.26334E+00_JPRB, 0.26479E+00_JPRB, 0.26666E+00_JPRB ,&
&  0.26905E+00_JPRB, 0.27228E+00_JPRB, 0.27711E+00_JPRB, 0.28270E+00_JPRB, 0.28979E+00_JPRB, 0.30035E+00_JPRB, 0.31893E+00_JPRB ,&
&  0.34353E+00_JPRB, 0.39923E+00_JPRB, 0.40446E+00_JPRB, 0.37445E+00_JPRB, 0.38054E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.36440E+00_JPRB, 0.37126E+00_JPRB, 0.37835E+00_JPRB, 0.38595E+00_JPRB, 0.39544E+00_JPRB, 0.39957E+00_JPRB, 0.40157E+00_JPRB ,&
&  0.39749E+00_JPRB, 0.39291E+00_JPRB, 0.38208E+00_JPRB, 0.37217E+00_JPRB, 0.36269E+00_JPRB, 0.35516E+00_JPRB, 0.34932E+00_JPRB ,&
&  0.34597E+00_JPRB, 0.34340E+00_JPRB, 0.33943E+00_JPRB, 0.33612E+00_JPRB, 0.33320E+00_JPRB, 0.33324E+00_JPRB, 0.33350E+00_JPRB ,&
&  0.33284E+00_JPRB, 0.33304E+00_JPRB, 0.33172E+00_JPRB, 0.33692E+00_JPRB, 0.34609E+00_JPRB, 0.35728E+00_JPRB, 0.37189E+00_JPRB ,&
&  0.39235E+00_JPRB, 0.40350E+00_JPRB, 0.41906E+00_JPRB, 0.41948E+00_JPRB, 0.41767E+00_JPRB, 0.41389E+00_JPRB, 0.41080E+00_JPRB ,&
&  0.40615E+00_JPRB, 0.39944E+00_JPRB, 0.39451E+00_JPRB, 0.38833E+00_JPRB, 0.35899E+00_JPRB, 0.33598E+00_JPRB, 0.31830E+00_JPRB ,&
&  0.30304E+00_JPRB, 0.28866E+00_JPRB, 0.27599E+00_JPRB, 0.26960E+00_JPRB, 0.26442E+00_JPRB, 0.26136E+00_JPRB, 0.25815E+00_JPRB ,&
&  0.25474E+00_JPRB, 0.25349E+00_JPRB, 0.25154E+00_JPRB, 0.25081E+00_JPRB, 0.25180E+00_JPRB, 0.25172E+00_JPRB, 0.25248E+00_JPRB ,&
&  0.25318E+00_JPRB, 0.25428E+00_JPRB, 0.25681E+00_JPRB, 0.25871E+00_JPRB, 0.26068E+00_JPRB, 0.26252E+00_JPRB, 0.26346E+00_JPRB ,&
&  0.26531E+00_JPRB, 0.26688E+00_JPRB, 0.26684E+00_JPRB, 0.26610E+00_JPRB, 0.26430E+00_JPRB, 0.26031E+00_JPRB, 0.25632E+00_JPRB ,&
&  0.25206E+00_JPRB, 0.24898E+00_JPRB, 0.24777E+00_JPRB, 0.24676E+00_JPRB, 0.24757E+00_JPRB, 0.24833E+00_JPRB, 0.24924E+00_JPRB ,&
&  0.25113E+00_JPRB, 0.25292E+00_JPRB, 0.25500E+00_JPRB, 0.25794E+00_JPRB, 0.26020E+00_JPRB, 0.26213E+00_JPRB, 0.26459E+00_JPRB ,&
&  0.26491E+00_JPRB, 0.26922E+00_JPRB, 0.27468E+00_JPRB, 0.28176E+00_JPRB, 0.29399E+00_JPRB, 0.30398E+00_JPRB, 0.32666E+00_JPRB ,&
&  0.36209E+00_JPRB, 0.42926E+00_JPRB, 0.42305E+00_JPRB, 0.35502E+00_JPRB, 0.35833E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.35317E+00_JPRB, 0.35602E+00_JPRB, 0.35934E+00_JPRB, 0.36340E+00_JPRB, 0.36840E+00_JPRB, 0.37289E+00_JPRB, 0.37773E+00_JPRB ,&
&  0.38063E+00_JPRB, 0.37739E+00_JPRB, 0.37799E+00_JPRB, 0.37361E+00_JPRB, 0.36915E+00_JPRB, 0.36345E+00_JPRB, 0.35740E+00_JPRB ,&
&  0.35104E+00_JPRB, 0.34206E+00_JPRB, 0.33244E+00_JPRB, 0.32394E+00_JPRB, 0.31675E+00_JPRB, 0.31113E+00_JPRB, 0.30664E+00_JPRB ,&
&  0.30353E+00_JPRB, 0.30388E+00_JPRB, 0.30599E+00_JPRB, 0.31203E+00_JPRB, 0.32217E+00_JPRB, 0.33474E+00_JPRB, 0.35381E+00_JPRB ,&
&  0.38074E+00_JPRB, 0.38292E+00_JPRB, 0.38800E+00_JPRB, 0.38710E+00_JPRB, 0.38297E+00_JPRB, 0.38098E+00_JPRB, 0.37705E+00_JPRB ,&
&  0.37759E+00_JPRB, 0.37004E+00_JPRB, 0.37984E+00_JPRB, 0.36338E+00_JPRB, 0.33655E+00_JPRB, 0.32117E+00_JPRB, 0.30286E+00_JPRB ,&
&  0.28943E+00_JPRB, 0.27801E+00_JPRB, 0.26945E+00_JPRB, 0.26583E+00_JPRB, 0.26201E+00_JPRB, 0.26022E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25266E+00_JPRB, 0.25071E+00_JPRB, 0.24756E+00_JPRB, 0.24727E+00_JPRB, 0.24733E+00_JPRB, 0.24674E+00_JPRB, 0.24682E+00_JPRB ,&
&  0.24722E+00_JPRB, 0.24825E+00_JPRB, 0.25013E+00_JPRB, 0.25183E+00_JPRB, 0.25285E+00_JPRB, 0.25345E+00_JPRB, 0.25331E+00_JPRB ,&
&  0.25378E+00_JPRB, 0.25348E+00_JPRB, 0.25264E+00_JPRB, 0.25131E+00_JPRB, 0.24930E+00_JPRB, 0.24721E+00_JPRB, 0.24520E+00_JPRB ,&
&  0.24358E+00_JPRB, 0.24297E+00_JPRB, 0.24246E+00_JPRB, 0.24268E+00_JPRB, 0.24328E+00_JPRB, 0.24505E+00_JPRB, 0.24672E+00_JPRB ,&
&  0.24863E+00_JPRB, 0.25129E+00_JPRB, 0.25364E+00_JPRB, 0.25652E+00_JPRB, 0.25914E+00_JPRB, 0.26069E+00_JPRB, 0.26478E+00_JPRB ,&
&  0.27004E+00_JPRB, 0.27715E+00_JPRB, 0.28544E+00_JPRB, 0.29342E+00_JPRB, 0.30355E+00_JPRB, 0.31907E+00_JPRB, 0.34017E+00_JPRB ,&
&  0.36167E+00_JPRB, 0.38499E+00_JPRB, 0.37913E+00_JPRB, 0.35143E+00_JPRB, 0.35007E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.35467E+00_JPRB, 0.35281E+00_JPRB, 0.35335E+00_JPRB, 0.35437E+00_JPRB, 0.35800E+00_JPRB, 0.36328E+00_JPRB, 0.36902E+00_JPRB ,&
&  0.36858E+00_JPRB, 0.37172E+00_JPRB, 0.37086E+00_JPRB, 0.36821E+00_JPRB, 0.36378E+00_JPRB, 0.35959E+00_JPRB, 0.35483E+00_JPRB ,&
&  0.34805E+00_JPRB, 0.33934E+00_JPRB, 0.33084E+00_JPRB, 0.32219E+00_JPRB, 0.31395E+00_JPRB, 0.30538E+00_JPRB, 0.30028E+00_JPRB ,&
&  0.29700E+00_JPRB, 0.29613E+00_JPRB, 0.29814E+00_JPRB, 0.30091E+00_JPRB, 0.30629E+00_JPRB, 0.31362E+00_JPRB, 0.32335E+00_JPRB ,&
&  0.33288E+00_JPRB, 0.34358E+00_JPRB, 0.34914E+00_JPRB, 0.34903E+00_JPRB, 0.34881E+00_JPRB, 0.34675E+00_JPRB, 0.34329E+00_JPRB ,&
&  0.33677E+00_JPRB, 0.32935E+00_JPRB, 0.32078E+00_JPRB, 0.31171E+00_JPRB, 0.30246E+00_JPRB, 0.29372E+00_JPRB, 0.28561E+00_JPRB ,&
&  0.27901E+00_JPRB, 0.27342E+00_JPRB, 0.26829E+00_JPRB, 0.26400E+00_JPRB, 0.26068E+00_JPRB, 0.25820E+00_JPRB, 0.25519E+00_JPRB ,&
&  0.25267E+00_JPRB, 0.25044E+00_JPRB, 0.24780E+00_JPRB, 0.24596E+00_JPRB, 0.24432E+00_JPRB, 0.24324E+00_JPRB, 0.24234E+00_JPRB ,&
&  0.24223E+00_JPRB, 0.24213E+00_JPRB, 0.24272E+00_JPRB, 0.24333E+00_JPRB, 0.24405E+00_JPRB, 0.24400E+00_JPRB, 0.24377E+00_JPRB ,&
&  0.24335E+00_JPRB, 0.24294E+00_JPRB, 0.24158E+00_JPRB, 0.24052E+00_JPRB, 0.23979E+00_JPRB, 0.23899E+00_JPRB, 0.23896E+00_JPRB ,&
&  0.23980E+00_JPRB, 0.24068E+00_JPRB, 0.24140E+00_JPRB, 0.24301E+00_JPRB, 0.24463E+00_JPRB, 0.24697E+00_JPRB, 0.24977E+00_JPRB ,&
&  0.25184E+00_JPRB, 0.25428E+00_JPRB, 0.25699E+00_JPRB, 0.26014E+00_JPRB, 0.26389E+00_JPRB, 0.26779E+00_JPRB, 0.27235E+00_JPRB ,&
&  0.27749E+00_JPRB, 0.28276E+00_JPRB, 0.28869E+00_JPRB, 0.29522E+00_JPRB, 0.30312E+00_JPRB, 0.30981E+00_JPRB, 0.31982E+00_JPRB ,&
&  0.32775E+00_JPRB, 0.32696E+00_JPRB, 0.36734E+00_JPRB, 0.36102E+00_JPRB, 0.35686E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.29690E+00_JPRB, 0.30341E+00_JPRB, 0.30811E+00_JPRB, 0.31292E+00_JPRB, 0.30862E+00_JPRB, 0.34581E+00_JPRB, 0.34425E+00_JPRB ,&
&  0.34494E+00_JPRB, 0.33164E+00_JPRB, 0.33220E+00_JPRB, 0.33084E+00_JPRB, 0.32790E+00_JPRB, 0.32358E+00_JPRB, 0.31958E+00_JPRB ,&
&  0.31606E+00_JPRB, 0.31135E+00_JPRB, 0.30588E+00_JPRB, 0.30121E+00_JPRB, 0.29747E+00_JPRB, 0.29428E+00_JPRB, 0.29156E+00_JPRB ,&
&  0.29094E+00_JPRB, 0.29220E+00_JPRB, 0.29463E+00_JPRB, 0.29354E+00_JPRB, 0.29458E+00_JPRB, 0.29653E+00_JPRB, 0.29882E+00_JPRB ,&
&  0.30098E+00_JPRB, 0.30283E+00_JPRB, 0.30289E+00_JPRB, 0.30122E+00_JPRB, 0.30003E+00_JPRB, 0.29767E+00_JPRB, 0.29549E+00_JPRB ,&
&  0.29286E+00_JPRB, 0.28996E+00_JPRB, 0.28654E+00_JPRB, 0.28380E+00_JPRB, 0.28081E+00_JPRB, 0.27717E+00_JPRB, 0.27367E+00_JPRB ,&
&  0.27066E+00_JPRB, 0.26797E+00_JPRB, 0.26504E+00_JPRB, 0.26232E+00_JPRB, 0.25973E+00_JPRB, 0.25709E+00_JPRB, 0.25475E+00_JPRB ,&
&  0.25297E+00_JPRB, 0.25147E+00_JPRB, 0.24997E+00_JPRB, 0.24840E+00_JPRB, 0.24686E+00_JPRB, 0.24537E+00_JPRB, 0.24415E+00_JPRB ,&
&  0.24395E+00_JPRB, 0.24325E+00_JPRB, 0.24257E+00_JPRB, 0.24218E+00_JPRB, 0.24222E+00_JPRB, 0.24174E+00_JPRB, 0.24154E+00_JPRB ,&
&  0.24149E+00_JPRB, 0.24154E+00_JPRB, 0.24177E+00_JPRB, 0.24208E+00_JPRB, 0.24250E+00_JPRB, 0.24305E+00_JPRB, 0.24394E+00_JPRB ,&
&  0.24500E+00_JPRB, 0.24628E+00_JPRB, 0.24765E+00_JPRB, 0.24906E+00_JPRB, 0.25051E+00_JPRB, 0.25204E+00_JPRB, 0.25375E+00_JPRB ,&
&  0.25548E+00_JPRB, 0.25717E+00_JPRB, 0.25882E+00_JPRB, 0.26044E+00_JPRB, 0.26306E+00_JPRB, 0.26558E+00_JPRB, 0.26828E+00_JPRB ,&
&  0.27119E+00_JPRB, 0.27460E+00_JPRB, 0.27818E+00_JPRB, 0.28204E+00_JPRB, 0.28634E+00_JPRB, 0.28790E+00_JPRB, 0.28913E+00_JPRB ,&
&  0.28994E+00_JPRB, 0.29282E+00_JPRB, 0.29489E+00_JPRB, 0.29470E+00_JPRB, 0.29441E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.26892E+00_JPRB, 0.26909E+00_JPRB, 0.26922E+00_JPRB, 0.26933E+00_JPRB, 0.26821E+00_JPRB, 0.26875E+00_JPRB, 0.26922E+00_JPRB ,&
&  0.26997E+00_JPRB, 0.27133E+00_JPRB, 0.27290E+00_JPRB, 0.27409E+00_JPRB, 0.27494E+00_JPRB, 0.27545E+00_JPRB, 0.27581E+00_JPRB ,&
&  0.27584E+00_JPRB, 0.27604E+00_JPRB, 0.27287E+00_JPRB, 0.27175E+00_JPRB, 0.27140E+00_JPRB, 0.27145E+00_JPRB, 0.27167E+00_JPRB ,&
&  0.27182E+00_JPRB, 0.27184E+00_JPRB, 0.27137E+00_JPRB, 0.27167E+00_JPRB, 0.27199E+00_JPRB, 0.27221E+00_JPRB, 0.27246E+00_JPRB ,&
&  0.27269E+00_JPRB, 0.27288E+00_JPRB, 0.27281E+00_JPRB, 0.27260E+00_JPRB, 0.27244E+00_JPRB, 0.27192E+00_JPRB, 0.27105E+00_JPRB ,&
&  0.27000E+00_JPRB, 0.26868E+00_JPRB, 0.26737E+00_JPRB, 0.26617E+00_JPRB, 0.26513E+00_JPRB, 0.26424E+00_JPRB, 0.26338E+00_JPRB ,&
&  0.26237E+00_JPRB, 0.26104E+00_JPRB, 0.25964E+00_JPRB, 0.25828E+00_JPRB, 0.25698E+00_JPRB, 0.25580E+00_JPRB, 0.25456E+00_JPRB ,&
&  0.25360E+00_JPRB, 0.25279E+00_JPRB, 0.25198E+00_JPRB, 0.25129E+00_JPRB, 0.25061E+00_JPRB, 0.24998E+00_JPRB, 0.24936E+00_JPRB ,&
&  0.24855E+00_JPRB, 0.24808E+00_JPRB, 0.24757E+00_JPRB, 0.24732E+00_JPRB, 0.24737E+00_JPRB, 0.24738E+00_JPRB, 0.24748E+00_JPRB ,&
&  0.24765E+00_JPRB, 0.24787E+00_JPRB, 0.24800E+00_JPRB, 0.24821E+00_JPRB, 0.24852E+00_JPRB, 0.24893E+00_JPRB, 0.24943E+00_JPRB ,&
&  0.24998E+00_JPRB, 0.25057E+00_JPRB, 0.25122E+00_JPRB, 0.25194E+00_JPRB, 0.25270E+00_JPRB, 0.25354E+00_JPRB, 0.25433E+00_JPRB ,&
&  0.25513E+00_JPRB, 0.25588E+00_JPRB, 0.25662E+00_JPRB, 0.25720E+00_JPRB, 0.25801E+00_JPRB, 0.25880E+00_JPRB, 0.25974E+00_JPRB ,&
&  0.26069E+00_JPRB, 0.26170E+00_JPRB, 0.26258E+00_JPRB, 0.26353E+00_JPRB, 0.26457E+00_JPRB, 0.26441E+00_JPRB, 0.26431E+00_JPRB ,&
&  0.26428E+00_JPRB, 0.26453E+00_JPRB, 0.26453E+00_JPRB, 0.26486E+00_JPRB, 0.26570E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB, 0.25422E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB, 0.42063E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.42294E+00_JPRB, 0.42319E+00_JPRB, 0.42334E+00_JPRB, 0.42340E+00_JPRB, 0.42349E+00_JPRB, 0.42376E+00_JPRB, 0.42396E+00_JPRB ,&
&  0.42424E+00_JPRB, 0.42459E+00_JPRB, 0.42523E+00_JPRB, 0.42559E+00_JPRB, 0.42595E+00_JPRB, 0.42617E+00_JPRB, 0.42643E+00_JPRB ,&
&  0.42662E+00_JPRB, 0.42678E+00_JPRB, 0.42684E+00_JPRB, 0.42699E+00_JPRB, 0.42720E+00_JPRB, 0.42754E+00_JPRB, 0.42784E+00_JPRB ,&
&  0.42779E+00_JPRB, 0.42744E+00_JPRB, 0.42719E+00_JPRB, 0.42685E+00_JPRB, 0.42640E+00_JPRB, 0.42557E+00_JPRB, 0.42453E+00_JPRB ,&
&  0.42336E+00_JPRB, 0.42222E+00_JPRB, 0.42129E+00_JPRB, 0.42040E+00_JPRB, 0.41976E+00_JPRB, 0.41921E+00_JPRB, 0.41887E+00_JPRB ,&
&  0.41839E+00_JPRB, 0.41817E+00_JPRB, 0.41796E+00_JPRB, 0.41784E+00_JPRB, 0.41742E+00_JPRB, 0.41709E+00_JPRB, 0.41661E+00_JPRB ,&
&  0.41609E+00_JPRB, 0.41537E+00_JPRB, 0.41479E+00_JPRB, 0.41405E+00_JPRB, 0.41324E+00_JPRB, 0.41205E+00_JPRB, 0.41094E+00_JPRB ,&
&  0.40984E+00_JPRB, 0.40868E+00_JPRB, 0.40743E+00_JPRB, 0.40638E+00_JPRB, 0.40560E+00_JPRB, 0.40485E+00_JPRB, 0.40393E+00_JPRB ,&
&  0.40358E+00_JPRB, 0.40375E+00_JPRB, 0.40389E+00_JPRB, 0.40381E+00_JPRB, 0.40409E+00_JPRB, 0.40447E+00_JPRB, 0.40493E+00_JPRB ,&
&  0.40533E+00_JPRB, 0.40602E+00_JPRB, 0.40704E+00_JPRB, 0.40798E+00_JPRB, 0.40893E+00_JPRB, 0.40990E+00_JPRB, 0.41102E+00_JPRB ,&
&  0.41205E+00_JPRB, 0.41304E+00_JPRB, 0.41398E+00_JPRB, 0.41500E+00_JPRB, 0.41561E+00_JPRB, 0.41609E+00_JPRB, 0.41625E+00_JPRB ,&
&  0.41650E+00_JPRB, 0.41663E+00_JPRB, 0.41681E+00_JPRB, 0.41696E+00_JPRB, 0.41733E+00_JPRB, 0.41764E+00_JPRB, 0.41798E+00_JPRB ,&
&  0.41838E+00_JPRB, 0.41891E+00_JPRB, 0.41935E+00_JPRB, 0.41972E+00_JPRB, 0.42012E+00_JPRB, 0.42070E+00_JPRB, 0.42116E+00_JPRB ,&
&  0.42152E+00_JPRB, 0.42193E+00_JPRB, 0.42230E+00_JPRB, 0.42262E+00_JPRB, 0.42278E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.41798E+00_JPRB, 0.41963E+00_JPRB, 0.42064E+00_JPRB, 0.42156E+00_JPRB, 0.42248E+00_JPRB, 0.42389E+00_JPRB, 0.42502E+00_JPRB ,&
&  0.42624E+00_JPRB, 0.42788E+00_JPRB, 0.43002E+00_JPRB, 0.43145E+00_JPRB, 0.43255E+00_JPRB, 0.43328E+00_JPRB, 0.43397E+00_JPRB ,&
&  0.43419E+00_JPRB, 0.43405E+00_JPRB, 0.43351E+00_JPRB, 0.43315E+00_JPRB, 0.43281E+00_JPRB, 0.43252E+00_JPRB, 0.43204E+00_JPRB ,&
&  0.43092E+00_JPRB, 0.42944E+00_JPRB, 0.42779E+00_JPRB, 0.42536E+00_JPRB, 0.42253E+00_JPRB, 0.41943E+00_JPRB, 0.41602E+00_JPRB ,&
&  0.41263E+00_JPRB, 0.40989E+00_JPRB, 0.40720E+00_JPRB, 0.40463E+00_JPRB, 0.40343E+00_JPRB, 0.40242E+00_JPRB, 0.40143E+00_JPRB ,&
&  0.40043E+00_JPRB, 0.39907E+00_JPRB, 0.39883E+00_JPRB, 0.39886E+00_JPRB, 0.39922E+00_JPRB, 0.40027E+00_JPRB, 0.40125E+00_JPRB ,&
&  0.40187E+00_JPRB, 0.40187E+00_JPRB, 0.40143E+00_JPRB, 0.40124E+00_JPRB, 0.40104E+00_JPRB, 0.40056E+00_JPRB, 0.39961E+00_JPRB ,&
&  0.39888E+00_JPRB, 0.39766E+00_JPRB, 0.39616E+00_JPRB, 0.39447E+00_JPRB, 0.39333E+00_JPRB, 0.39202E+00_JPRB, 0.39064E+00_JPRB ,&
&  0.38966E+00_JPRB, 0.38921E+00_JPRB, 0.38893E+00_JPRB, 0.38875E+00_JPRB, 0.38913E+00_JPRB, 0.38980E+00_JPRB, 0.39047E+00_JPRB ,&
&  0.39120E+00_JPRB, 0.39265E+00_JPRB, 0.39450E+00_JPRB, 0.39602E+00_JPRB, 0.39738E+00_JPRB, 0.39867E+00_JPRB, 0.40034E+00_JPRB ,&
&  0.40146E+00_JPRB, 0.40218E+00_JPRB, 0.40212E+00_JPRB, 0.40272E+00_JPRB, 0.40308E+00_JPRB, 0.40384E+00_JPRB, 0.40404E+00_JPRB ,&
&  0.40498E+00_JPRB, 0.40531E+00_JPRB, 0.40543E+00_JPRB, 0.40574E+00_JPRB, 0.40620E+00_JPRB, 0.40657E+00_JPRB, 0.40663E+00_JPRB ,&
&  0.40671E+00_JPRB, 0.40695E+00_JPRB, 0.40716E+00_JPRB, 0.40734E+00_JPRB, 0.40798E+00_JPRB, 0.40881E+00_JPRB, 0.40975E+00_JPRB ,&
&  0.41081E+00_JPRB, 0.41234E+00_JPRB, 0.41401E+00_JPRB, 0.41532E+00_JPRB, 0.41652E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.41298E+00_JPRB, 0.41431E+00_JPRB, 0.41531E+00_JPRB, 0.41684E+00_JPRB, 0.41961E+00_JPRB, 0.42294E+00_JPRB, 0.42511E+00_JPRB ,&
&  0.42796E+00_JPRB, 0.43130E+00_JPRB, 0.43491E+00_JPRB, 0.43716E+00_JPRB, 0.43883E+00_JPRB, 0.43965E+00_JPRB, 0.43965E+00_JPRB ,&
&  0.43866E+00_JPRB, 0.43731E+00_JPRB, 0.43602E+00_JPRB, 0.43471E+00_JPRB, 0.43335E+00_JPRB, 0.43185E+00_JPRB, 0.42912E+00_JPRB ,&
&  0.42657E+00_JPRB, 0.42539E+00_JPRB, 0.42162E+00_JPRB, 0.41689E+00_JPRB, 0.41282E+00_JPRB, 0.38280E+00_JPRB, 0.37231E+00_JPRB ,&
&  0.37006E+00_JPRB, 0.36769E+00_JPRB, 0.36851E+00_JPRB, 0.36977E+00_JPRB, 0.37556E+00_JPRB, 0.37238E+00_JPRB, 0.37043E+00_JPRB ,&
&  0.35332E+00_JPRB, 0.35232E+00_JPRB, 0.34915E+00_JPRB, 0.34749E+00_JPRB, 0.34930E+00_JPRB, 0.35154E+00_JPRB, 0.35837E+00_JPRB ,&
&  0.37524E+00_JPRB, 0.38047E+00_JPRB, 0.38289E+00_JPRB, 0.38436E+00_JPRB, 0.38547E+00_JPRB, 0.38596E+00_JPRB, 0.38643E+00_JPRB ,&
&  0.38670E+00_JPRB, 0.38646E+00_JPRB, 0.38607E+00_JPRB, 0.38529E+00_JPRB, 0.38439E+00_JPRB, 0.38318E+00_JPRB, 0.38221E+00_JPRB ,&
&  0.38153E+00_JPRB, 0.38154E+00_JPRB, 0.38202E+00_JPRB, 0.38235E+00_JPRB, 0.38311E+00_JPRB, 0.38344E+00_JPRB, 0.38378E+00_JPRB ,&
&  0.38392E+00_JPRB, 0.38416E+00_JPRB, 0.38499E+00_JPRB, 0.38606E+00_JPRB, 0.38740E+00_JPRB, 0.38783E+00_JPRB, 0.38894E+00_JPRB ,&
&  0.38975E+00_JPRB, 0.39105E+00_JPRB, 0.39185E+00_JPRB, 0.39376E+00_JPRB, 0.39474E+00_JPRB, 0.39682E+00_JPRB, 0.39845E+00_JPRB ,&
&  0.40118E+00_JPRB, 0.40279E+00_JPRB, 0.40438E+00_JPRB, 0.40541E+00_JPRB, 0.40638E+00_JPRB, 0.40708E+00_JPRB, 0.40713E+00_JPRB ,&
&  0.40714E+00_JPRB, 0.40652E+00_JPRB, 0.40592E+00_JPRB, 0.40510E+00_JPRB, 0.40470E+00_JPRB, 0.40416E+00_JPRB, 0.40404E+00_JPRB ,&
&  0.40372E+00_JPRB, 0.40468E+00_JPRB, 0.40666E+00_JPRB, 0.40909E+00_JPRB, 0.41112E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.42408E+00_JPRB, 0.42414E+00_JPRB, 0.42358E+00_JPRB, 0.42509E+00_JPRB, 0.42714E+00_JPRB, 0.43117E+00_JPRB, 0.43187E+00_JPRB ,&
&  0.43558E+00_JPRB, 0.43921E+00_JPRB, 0.44299E+00_JPRB, 0.44443E+00_JPRB, 0.44536E+00_JPRB, 0.44498E+00_JPRB, 0.44325E+00_JPRB ,&
&  0.44123E+00_JPRB, 0.43801E+00_JPRB, 0.43525E+00_JPRB, 0.43183E+00_JPRB, 0.42827E+00_JPRB, 0.42645E+00_JPRB, 0.42488E+00_JPRB ,&
&  0.42433E+00_JPRB, 0.42284E+00_JPRB, 0.39491E+00_JPRB, 0.38475E+00_JPRB, 0.36990E+00_JPRB, 0.36155E+00_JPRB, 0.35703E+00_JPRB ,&
&  0.35257E+00_JPRB, 0.34812E+00_JPRB, 0.34235E+00_JPRB, 0.33363E+00_JPRB, 0.32274E+00_JPRB, 0.30804E+00_JPRB, 0.30069E+00_JPRB ,&
&  0.29437E+00_JPRB, 0.29271E+00_JPRB, 0.29365E+00_JPRB, 0.29744E+00_JPRB, 0.30516E+00_JPRB, 0.31426E+00_JPRB, 0.32580E+00_JPRB ,&
&  0.34200E+00_JPRB, 0.35805E+00_JPRB, 0.36731E+00_JPRB, 0.36579E+00_JPRB, 0.37459E+00_JPRB, 0.37887E+00_JPRB, 0.38106E+00_JPRB ,&
&  0.38191E+00_JPRB, 0.38029E+00_JPRB, 0.38268E+00_JPRB, 0.38201E+00_JPRB, 0.38172E+00_JPRB, 0.38204E+00_JPRB, 0.38279E+00_JPRB ,&
&  0.38454E+00_JPRB, 0.38624E+00_JPRB, 0.38732E+00_JPRB, 0.38805E+00_JPRB, 0.38806E+00_JPRB, 0.38747E+00_JPRB, 0.38680E+00_JPRB ,&
&  0.38620E+00_JPRB, 0.38573E+00_JPRB, 0.38543E+00_JPRB, 0.38557E+00_JPRB, 0.38652E+00_JPRB, 0.38706E+00_JPRB, 0.38702E+00_JPRB ,&
&  0.38774E+00_JPRB, 0.38902E+00_JPRB, 0.39049E+00_JPRB, 0.38988E+00_JPRB, 0.39183E+00_JPRB, 0.39765E+00_JPRB, 0.40059E+00_JPRB ,&
&  0.40598E+00_JPRB, 0.41019E+00_JPRB, 0.41532E+00_JPRB, 0.41915E+00_JPRB, 0.42227E+00_JPRB, 0.42352E+00_JPRB, 0.42302E+00_JPRB ,&
&  0.42219E+00_JPRB, 0.42088E+00_JPRB, 0.42009E+00_JPRB, 0.41836E+00_JPRB, 0.41723E+00_JPRB, 0.41552E+00_JPRB, 0.41502E+00_JPRB ,&
&  0.41462E+00_JPRB, 0.41550E+00_JPRB, 0.41541E+00_JPRB, 0.41852E+00_JPRB, 0.42125E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.43952E+00_JPRB, 0.43925E+00_JPRB, 0.43970E+00_JPRB, 0.44355E+00_JPRB, 0.45069E+00_JPRB, 0.45882E+00_JPRB, 0.45759E+00_JPRB ,&
&  0.45694E+00_JPRB, 0.45366E+00_JPRB, 0.45421E+00_JPRB, 0.45284E+00_JPRB, 0.45062E+00_JPRB, 0.44747E+00_JPRB, 0.44471E+00_JPRB ,&
&  0.44237E+00_JPRB, 0.43676E+00_JPRB, 0.42997E+00_JPRB, 0.42869E+00_JPRB, 0.43036E+00_JPRB, 0.43125E+00_JPRB, 0.42962E+00_JPRB ,&
&  0.42560E+00_JPRB, 0.41628E+00_JPRB, 0.39417E+00_JPRB, 0.37750E+00_JPRB, 0.36204E+00_JPRB, 0.35649E+00_JPRB, 0.34735E+00_JPRB ,&
&  0.35238E+00_JPRB, 0.32263E+00_JPRB, 0.32193E+00_JPRB, 0.30889E+00_JPRB, 0.29836E+00_JPRB, 0.28701E+00_JPRB, 0.27676E+00_JPRB ,&
&  0.27340E+00_JPRB, 0.26995E+00_JPRB, 0.26956E+00_JPRB, 0.27474E+00_JPRB, 0.28157E+00_JPRB, 0.29377E+00_JPRB, 0.30807E+00_JPRB ,&
&  0.32123E+00_JPRB, 0.35756E+00_JPRB, 0.36760E+00_JPRB, 0.37123E+00_JPRB, 0.37978E+00_JPRB, 0.38533E+00_JPRB, 0.38586E+00_JPRB ,&
&  0.38562E+00_JPRB, 0.38381E+00_JPRB, 0.38572E+00_JPRB, 0.38671E+00_JPRB, 0.39022E+00_JPRB, 0.38999E+00_JPRB, 0.38992E+00_JPRB ,&
&  0.39171E+00_JPRB, 0.39368E+00_JPRB, 0.39606E+00_JPRB, 0.39803E+00_JPRB, 0.39999E+00_JPRB, 0.40118E+00_JPRB, 0.40188E+00_JPRB ,&
&  0.40186E+00_JPRB, 0.40147E+00_JPRB, 0.40080E+00_JPRB, 0.40117E+00_JPRB, 0.40145E+00_JPRB, 0.40364E+00_JPRB, 0.40444E+00_JPRB ,&
&  0.40516E+00_JPRB, 0.40528E+00_JPRB, 0.40481E+00_JPRB, 0.40563E+00_JPRB, 0.40806E+00_JPRB, 0.41027E+00_JPRB, 0.41446E+00_JPRB ,&
&  0.42157E+00_JPRB, 0.42642E+00_JPRB, 0.43225E+00_JPRB, 0.43720E+00_JPRB, 0.44119E+00_JPRB, 0.44143E+00_JPRB, 0.43938E+00_JPRB ,&
&  0.43799E+00_JPRB, 0.43649E+00_JPRB, 0.43498E+00_JPRB, 0.43251E+00_JPRB, 0.43016E+00_JPRB, 0.42736E+00_JPRB, 0.42586E+00_JPRB ,&
&  0.42545E+00_JPRB, 0.42747E+00_JPRB, 0.42757E+00_JPRB, 0.43117E+00_JPRB, 0.43556E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.45089E+00_JPRB, 0.45297E+00_JPRB, 0.45811E+00_JPRB, 0.46425E+00_JPRB, 0.47059E+00_JPRB, 0.47713E+00_JPRB, 0.47956E+00_JPRB ,&
&  0.48094E+00_JPRB, 0.47728E+00_JPRB, 0.47221E+00_JPRB, 0.45991E+00_JPRB, 0.45161E+00_JPRB, 0.44752E+00_JPRB, 0.44274E+00_JPRB ,&
&  0.43821E+00_JPRB, 0.43626E+00_JPRB, 0.43519E+00_JPRB, 0.43546E+00_JPRB, 0.43353E+00_JPRB, 0.43256E+00_JPRB, 0.42638E+00_JPRB ,&
&  0.41323E+00_JPRB, 0.40167E+00_JPRB, 0.38340E+00_JPRB, 0.36676E+00_JPRB, 0.35550E+00_JPRB, 0.34654E+00_JPRB, 0.34625E+00_JPRB ,&
&  0.34449E+00_JPRB, 0.34361E+00_JPRB, 0.34269E+00_JPRB, 0.34151E+00_JPRB, 0.34391E+00_JPRB, 0.29011E+00_JPRB, 0.27629E+00_JPRB ,&
&  0.26711E+00_JPRB, 0.26099E+00_JPRB, 0.25955E+00_JPRB, 0.26029E+00_JPRB, 0.26541E+00_JPRB, 0.28036E+00_JPRB, 0.30130E+00_JPRB ,&
&  0.31914E+00_JPRB, 0.36877E+00_JPRB, 0.37758E+00_JPRB, 0.38588E+00_JPRB, 0.39803E+00_JPRB, 0.40578E+00_JPRB, 0.40932E+00_JPRB ,&
&  0.41481E+00_JPRB, 0.42311E+00_JPRB, 0.43137E+00_JPRB, 0.43802E+00_JPRB, 0.43653E+00_JPRB, 0.43621E+00_JPRB, 0.43336E+00_JPRB ,&
&  0.42670E+00_JPRB, 0.41996E+00_JPRB, 0.41808E+00_JPRB, 0.41733E+00_JPRB, 0.41753E+00_JPRB, 0.41790E+00_JPRB, 0.41772E+00_JPRB ,&
&  0.41956E+00_JPRB, 0.42043E+00_JPRB, 0.42429E+00_JPRB, 0.42531E+00_JPRB, 0.42769E+00_JPRB, 0.43104E+00_JPRB, 0.43319E+00_JPRB ,&
&  0.43311E+00_JPRB, 0.43327E+00_JPRB, 0.43045E+00_JPRB, 0.42666E+00_JPRB, 0.42523E+00_JPRB, 0.42368E+00_JPRB, 0.42556E+00_JPRB ,&
&  0.42880E+00_JPRB, 0.43582E+00_JPRB, 0.43942E+00_JPRB, 0.44359E+00_JPRB, 0.44729E+00_JPRB, 0.44754E+00_JPRB, 0.44774E+00_JPRB ,&
&  0.44742E+00_JPRB, 0.44776E+00_JPRB, 0.44652E+00_JPRB, 0.44435E+00_JPRB, 0.44043E+00_JPRB, 0.43643E+00_JPRB, 0.43301E+00_JPRB ,&
&  0.43316E+00_JPRB, 0.43597E+00_JPRB, 0.43683E+00_JPRB, 0.44014E+00_JPRB, 0.44565E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.46141E+00_JPRB, 0.46529E+00_JPRB, 0.47290E+00_JPRB, 0.47885E+00_JPRB, 0.48697E+00_JPRB, 0.49296E+00_JPRB, 0.49523E+00_JPRB ,&
&  0.50254E+00_JPRB, 0.49780E+00_JPRB, 0.48396E+00_JPRB, 0.46641E+00_JPRB, 0.44916E+00_JPRB, 0.43836E+00_JPRB, 0.43340E+00_JPRB ,&
&  0.43340E+00_JPRB, 0.42932E+00_JPRB, 0.42653E+00_JPRB, 0.42571E+00_JPRB, 0.42459E+00_JPRB, 0.42332E+00_JPRB, 0.41790E+00_JPRB ,&
&  0.40269E+00_JPRB, 0.39153E+00_JPRB, 0.37970E+00_JPRB, 0.36487E+00_JPRB, 0.35198E+00_JPRB, 0.34511E+00_JPRB, 0.34846E+00_JPRB ,&
&  0.35187E+00_JPRB, 0.34600E+00_JPRB, 0.34441E+00_JPRB, 0.34275E+00_JPRB, 0.34114E+00_JPRB, 0.33794E+00_JPRB, 0.29113E+00_JPRB ,&
&  0.27530E+00_JPRB, 0.26321E+00_JPRB, 0.25883E+00_JPRB, 0.25947E+00_JPRB, 0.27013E+00_JPRB, 0.28792E+00_JPRB, 0.31988E+00_JPRB ,&
&  0.35260E+00_JPRB, 0.38202E+00_JPRB, 0.39872E+00_JPRB, 0.41215E+00_JPRB, 0.42681E+00_JPRB, 0.44044E+00_JPRB, 0.44947E+00_JPRB ,&
&  0.46226E+00_JPRB, 0.47776E+00_JPRB, 0.48968E+00_JPRB, 0.49919E+00_JPRB, 0.50411E+00_JPRB, 0.50256E+00_JPRB, 0.50122E+00_JPRB ,&
&  0.49417E+00_JPRB, 0.45778E+00_JPRB, 0.44144E+00_JPRB, 0.43054E+00_JPRB, 0.42363E+00_JPRB, 0.42376E+00_JPRB, 0.42342E+00_JPRB ,&
&  0.42819E+00_JPRB, 0.43433E+00_JPRB, 0.43690E+00_JPRB, 0.43920E+00_JPRB, 0.44064E+00_JPRB, 0.44053E+00_JPRB, 0.44267E+00_JPRB ,&
&  0.44694E+00_JPRB, 0.45440E+00_JPRB, 0.46932E+00_JPRB, 0.45959E+00_JPRB, 0.44663E+00_JPRB, 0.43443E+00_JPRB, 0.42815E+00_JPRB ,&
&  0.42673E+00_JPRB, 0.42925E+00_JPRB, 0.43445E+00_JPRB, 0.43681E+00_JPRB, 0.43949E+00_JPRB, 0.44274E+00_JPRB, 0.45045E+00_JPRB ,&
&  0.45227E+00_JPRB, 0.45444E+00_JPRB, 0.45507E+00_JPRB, 0.45463E+00_JPRB, 0.45061E+00_JPRB, 0.44659E+00_JPRB, 0.44390E+00_JPRB ,&
&  0.44469E+00_JPRB, 0.44714E+00_JPRB, 0.44919E+00_JPRB, 0.45241E+00_JPRB, 0.45728E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.46408E+00_JPRB, 0.47305E+00_JPRB, 0.48685E+00_JPRB, 0.49651E+00_JPRB, 0.50643E+00_JPRB, 0.49915E+00_JPRB, 0.49101E+00_JPRB ,&
&  0.48865E+00_JPRB, 0.48532E+00_JPRB, 0.47452E+00_JPRB, 0.46087E+00_JPRB, 0.44353E+00_JPRB, 0.43052E+00_JPRB, 0.42573E+00_JPRB ,&
&  0.42548E+00_JPRB, 0.42330E+00_JPRB, 0.42051E+00_JPRB, 0.41464E+00_JPRB, 0.40867E+00_JPRB, 0.40777E+00_JPRB, 0.40347E+00_JPRB ,&
&  0.39356E+00_JPRB, 0.38572E+00_JPRB, 0.37734E+00_JPRB, 0.36809E+00_JPRB, 0.35852E+00_JPRB, 0.35578E+00_JPRB, 0.35488E+00_JPRB ,&
&  0.35353E+00_JPRB, 0.35422E+00_JPRB, 0.35705E+00_JPRB, 0.35109E+00_JPRB, 0.35170E+00_JPRB, 0.35556E+00_JPRB, 0.32184E+00_JPRB ,&
&  0.29197E+00_JPRB, 0.27266E+00_JPRB, 0.26252E+00_JPRB, 0.26317E+00_JPRB, 0.27774E+00_JPRB, 0.30786E+00_JPRB, 0.34533E+00_JPRB ,&
&  0.38453E+00_JPRB, 0.41364E+00_JPRB, 0.43605E+00_JPRB, 0.45144E+00_JPRB, 0.46331E+00_JPRB, 0.47715E+00_JPRB, 0.49157E+00_JPRB ,&
&  0.50863E+00_JPRB, 0.53110E+00_JPRB, 0.54757E+00_JPRB, 0.56349E+00_JPRB, 0.57045E+00_JPRB, 0.53748E+00_JPRB, 0.56135E+00_JPRB ,&
&  0.66503E+00_JPRB, 0.55966E+00_JPRB, 0.47892E+00_JPRB, 0.44746E+00_JPRB, 0.44300E+00_JPRB, 0.44877E+00_JPRB, 0.45726E+00_JPRB ,&
&  0.46679E+00_JPRB, 0.46959E+00_JPRB, 0.46691E+00_JPRB, 0.46305E+00_JPRB, 0.46240E+00_JPRB, 0.46431E+00_JPRB, 0.47441E+00_JPRB ,&
&  0.49084E+00_JPRB, 0.57220E+00_JPRB, 0.72815E+00_JPRB, 0.55305E+00_JPRB, 0.48802E+00_JPRB, 0.44902E+00_JPRB, 0.42143E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.40919E+00_JPRB, 0.41386E+00_JPRB, 0.41660E+00_JPRB, 0.41553E+00_JPRB, 0.41743E+00_JPRB, 0.42172E+00_JPRB ,&
&  0.43177E+00_JPRB, 0.44235E+00_JPRB, 0.44935E+00_JPRB, 0.45528E+00_JPRB, 0.45510E+00_JPRB, 0.45094E+00_JPRB, 0.44726E+00_JPRB ,&
&  0.45078E+00_JPRB, 0.45568E+00_JPRB, 0.45951E+00_JPRB, 0.46116E+00_JPRB, 0.46147E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.46362E+00_JPRB, 0.48863E+00_JPRB, 0.51661E+00_JPRB, 0.51755E+00_JPRB, 0.51795E+00_JPRB, 0.48970E+00_JPRB, 0.45897E+00_JPRB ,&
&  0.43843E+00_JPRB, 0.42697E+00_JPRB, 0.42306E+00_JPRB, 0.41597E+00_JPRB, 0.41044E+00_JPRB, 0.40810E+00_JPRB, 0.40627E+00_JPRB ,&
&  0.40522E+00_JPRB, 0.40333E+00_JPRB, 0.40093E+00_JPRB, 0.39686E+00_JPRB, 0.39568E+00_JPRB, 0.39872E+00_JPRB, 0.39614E+00_JPRB ,&
&  0.38888E+00_JPRB, 0.38479E+00_JPRB, 0.37441E+00_JPRB, 0.36160E+00_JPRB, 0.35602E+00_JPRB, 0.35873E+00_JPRB, 0.36536E+00_JPRB ,&
&  0.36303E+00_JPRB, 0.35810E+00_JPRB, 0.36442E+00_JPRB, 0.37417E+00_JPRB, 0.36608E+00_JPRB, 0.36694E+00_JPRB, 0.33212E+00_JPRB ,&
&  0.29859E+00_JPRB, 0.27866E+00_JPRB, 0.27332E+00_JPRB, 0.29459E+00_JPRB, 0.33640E+00_JPRB, 0.39389E+00_JPRB, 0.42412E+00_JPRB ,&
&  0.44685E+00_JPRB, 0.46938E+00_JPRB, 0.48949E+00_JPRB, 0.50998E+00_JPRB, 0.53745E+00_JPRB, 0.56405E+00_JPRB, 0.59431E+00_JPRB ,&
&  0.61600E+00_JPRB, 0.62519E+00_JPRB, 0.61961E+00_JPRB, 0.55772E+00_JPRB, 0.53858E+00_JPRB, 0.52010E+00_JPRB, 0.54973E+00_JPRB ,&
&  0.58236E+00_JPRB, 0.54206E+00_JPRB, 0.51411E+00_JPRB, 0.51545E+00_JPRB, 0.51541E+00_JPRB, 0.52423E+00_JPRB, 0.52718E+00_JPRB ,&
&  0.51770E+00_JPRB, 0.49991E+00_JPRB, 0.48275E+00_JPRB, 0.47058E+00_JPRB, 0.47183E+00_JPRB, 0.48299E+00_JPRB, 0.49581E+00_JPRB ,&
&  0.51891E+00_JPRB, 0.57158E+00_JPRB, 0.56866E+00_JPRB, 0.45214E+00_JPRB, 0.43356E+00_JPRB, 0.42589E+00_JPRB, 0.41917E+00_JPRB ,&
&  0.40721E+00_JPRB, 0.40038E+00_JPRB, 0.40081E+00_JPRB, 0.40192E+00_JPRB, 0.39664E+00_JPRB, 0.39122E+00_JPRB, 0.38878E+00_JPRB ,&
&  0.39163E+00_JPRB, 0.39922E+00_JPRB, 0.41003E+00_JPRB, 0.43185E+00_JPRB, 0.44457E+00_JPRB, 0.44910E+00_JPRB, 0.44485E+00_JPRB ,&
&  0.44202E+00_JPRB, 0.44441E+00_JPRB, 0.44504E+00_JPRB, 0.44291E+00_JPRB, 0.44845E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.54215E+00_JPRB, 0.54425E+00_JPRB, 0.53249E+00_JPRB, 0.52976E+00_JPRB, 0.51268E+00_JPRB, 0.50101E+00_JPRB, 0.47545E+00_JPRB ,&
&  0.46826E+00_JPRB, 0.44222E+00_JPRB, 0.41735E+00_JPRB, 0.39134E+00_JPRB, 0.37915E+00_JPRB, 0.37481E+00_JPRB, 0.37695E+00_JPRB ,&
&  0.38230E+00_JPRB, 0.38845E+00_JPRB, 0.39286E+00_JPRB, 0.38804E+00_JPRB, 0.39121E+00_JPRB, 0.39399E+00_JPRB, 0.39240E+00_JPRB ,&
&  0.38912E+00_JPRB, 0.38809E+00_JPRB, 0.38215E+00_JPRB, 0.37279E+00_JPRB, 0.36781E+00_JPRB, 0.36659E+00_JPRB, 0.36304E+00_JPRB ,&
&  0.35621E+00_JPRB, 0.35664E+00_JPRB, 0.36926E+00_JPRB, 0.39410E+00_JPRB, 0.37637E+00_JPRB, 0.37308E+00_JPRB, 0.33116E+00_JPRB ,&
&  0.29948E+00_JPRB, 0.29579E+00_JPRB, 0.32025E+00_JPRB, 0.38641E+00_JPRB, 0.40945E+00_JPRB, 0.44052E+00_JPRB, 0.45944E+00_JPRB ,&
&  0.45863E+00_JPRB, 0.47271E+00_JPRB, 0.51324E+00_JPRB, 0.58991E+00_JPRB, 0.61747E+00_JPRB, 0.65308E+00_JPRB, 0.68763E+00_JPRB ,&
&  0.71195E+00_JPRB, 0.72450E+00_JPRB, 0.72500E+00_JPRB, 0.68453E+00_JPRB, 0.69366E+00_JPRB, 0.72464E+00_JPRB, 0.61507E+00_JPRB ,&
&  0.59997E+00_JPRB, 0.57511E+00_JPRB, 0.57660E+00_JPRB, 0.60358E+00_JPRB, 0.57925E+00_JPRB, 0.58209E+00_JPRB, 0.56611E+00_JPRB ,&
&  0.52900E+00_JPRB, 0.50790E+00_JPRB, 0.49351E+00_JPRB, 0.49021E+00_JPRB, 0.49252E+00_JPRB, 0.50420E+00_JPRB, 0.52276E+00_JPRB ,&
&  0.54578E+00_JPRB, 0.55513E+00_JPRB, 0.52627E+00_JPRB, 0.46064E+00_JPRB, 0.43623E+00_JPRB, 0.42499E+00_JPRB, 0.41638E+00_JPRB ,&
&  0.40737E+00_JPRB, 0.40254E+00_JPRB, 0.40394E+00_JPRB, 0.40379E+00_JPRB, 0.39736E+00_JPRB, 0.38424E+00_JPRB, 0.37293E+00_JPRB ,&
&  0.36835E+00_JPRB, 0.36905E+00_JPRB, 0.36965E+00_JPRB, 0.38877E+00_JPRB, 0.41247E+00_JPRB, 0.43272E+00_JPRB, 0.43610E+00_JPRB ,&
&  0.43822E+00_JPRB, 0.44812E+00_JPRB, 0.45987E+00_JPRB, 0.48671E+00_JPRB, 0.51163E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.55324E+00_JPRB, 0.55694E+00_JPRB, 0.55272E+00_JPRB, 0.54204E+00_JPRB, 0.53745E+00_JPRB, 0.51128E+00_JPRB, 0.44871E+00_JPRB ,&
&  0.45028E+00_JPRB, 0.40485E+00_JPRB, 0.38651E+00_JPRB, 0.37672E+00_JPRB, 0.37674E+00_JPRB, 0.37168E+00_JPRB, 0.36760E+00_JPRB ,&
&  0.36778E+00_JPRB, 0.37758E+00_JPRB, 0.38723E+00_JPRB, 0.38611E+00_JPRB, 0.38981E+00_JPRB, 0.39306E+00_JPRB, 0.39166E+00_JPRB ,&
&  0.39429E+00_JPRB, 0.39730E+00_JPRB, 0.39771E+00_JPRB, 0.39308E+00_JPRB, 0.38200E+00_JPRB, 0.37350E+00_JPRB, 0.36454E+00_JPRB ,&
&  0.35866E+00_JPRB, 0.36036E+00_JPRB, 0.37094E+00_JPRB, 0.39460E+00_JPRB, 0.38574E+00_JPRB, 0.37507E+00_JPRB, 0.36319E+00_JPRB ,&
&  0.31659E+00_JPRB, 0.33355E+00_JPRB, 0.39992E+00_JPRB, 0.42475E+00_JPRB, 0.44997E+00_JPRB, 0.48146E+00_JPRB, 0.52506E+00_JPRB ,&
&  0.56374E+00_JPRB, 0.60412E+00_JPRB, 0.64930E+00_JPRB, 0.70032E+00_JPRB, 0.75444E+00_JPRB, 0.80095E+00_JPRB, 0.82965E+00_JPRB ,&
&  0.86449E+00_JPRB, 0.70673E+00_JPRB, 0.68076E+00_JPRB, 0.78897E+00_JPRB, 0.96012E+00_JPRB, 0.77378E+00_JPRB, 0.70360E+00_JPRB ,&
&  0.67197E+00_JPRB, 0.63579E+00_JPRB, 0.62275E+00_JPRB, 0.64839E+00_JPRB, 0.64368E+00_JPRB, 0.65232E+00_JPRB, 0.65871E+00_JPRB ,&
&  0.60399E+00_JPRB, 0.57443E+00_JPRB, 0.55180E+00_JPRB, 0.55241E+00_JPRB, 0.56140E+00_JPRB, 0.57343E+00_JPRB, 0.57715E+00_JPRB ,&
&  0.57038E+00_JPRB, 0.55817E+00_JPRB, 0.53219E+00_JPRB, 0.49403E+00_JPRB, 0.46732E+00_JPRB, 0.45205E+00_JPRB, 0.44001E+00_JPRB ,&
&  0.42956E+00_JPRB, 0.41828E+00_JPRB, 0.40861E+00_JPRB, 0.40312E+00_JPRB, 0.40111E+00_JPRB, 0.40131E+00_JPRB, 0.40727E+00_JPRB ,&
&  0.39409E+00_JPRB, 0.38567E+00_JPRB, 0.37682E+00_JPRB, 0.38872E+00_JPRB, 0.41395E+00_JPRB, 0.43747E+00_JPRB, 0.45360E+00_JPRB ,&
&  0.47514E+00_JPRB, 0.48821E+00_JPRB, 0.48422E+00_JPRB, 0.49886E+00_JPRB, 0.51894E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.56094E+00_JPRB, 0.56875E+00_JPRB, 0.56420E+00_JPRB, 0.56437E+00_JPRB, 0.55826E+00_JPRB, 0.51167E+00_JPRB, 0.46970E+00_JPRB ,&
&  0.52933E+00_JPRB, 0.45921E+00_JPRB, 0.48863E+00_JPRB, 0.41159E+00_JPRB, 0.38689E+00_JPRB, 0.38136E+00_JPRB, 0.36564E+00_JPRB ,&
&  0.34906E+00_JPRB, 0.35887E+00_JPRB, 0.37067E+00_JPRB, 0.37332E+00_JPRB, 0.37923E+00_JPRB, 0.38564E+00_JPRB, 0.38542E+00_JPRB ,&
&  0.38969E+00_JPRB, 0.39849E+00_JPRB, 0.41004E+00_JPRB, 0.40222E+00_JPRB, 0.39014E+00_JPRB, 0.38343E+00_JPRB, 0.37583E+00_JPRB ,&
&  0.36235E+00_JPRB, 0.35691E+00_JPRB, 0.36870E+00_JPRB, 0.38910E+00_JPRB, 0.39981E+00_JPRB, 0.38376E+00_JPRB, 0.37604E+00_JPRB ,&
&  0.37746E+00_JPRB, 0.38032E+00_JPRB, 0.43165E+00_JPRB, 0.45852E+00_JPRB, 0.49471E+00_JPRB, 0.54210E+00_JPRB, 0.58889E+00_JPRB ,&
&  0.64338E+00_JPRB, 0.70034E+00_JPRB, 0.76173E+00_JPRB, 0.83168E+00_JPRB, 0.89423E+00_JPRB, 0.95530E+00_JPRB, 0.10052E+01_JPRB ,&
&  0.10730E+01_JPRB, 0.80796E+00_JPRB, 0.97630E+00_JPRB, 0.10371E+01_JPRB, 0.11829E+01_JPRB, 0.11179E+01_JPRB, 0.96616E+00_JPRB ,&
&  0.96449E+00_JPRB, 0.84651E+00_JPRB, 0.77267E+00_JPRB, 0.70927E+00_JPRB, 0.69003E+00_JPRB, 0.71370E+00_JPRB, 0.73932E+00_JPRB ,&
&  0.77956E+00_JPRB, 0.76083E+00_JPRB, 0.72295E+00_JPRB, 0.71638E+00_JPRB, 0.71615E+00_JPRB, 0.71238E+00_JPRB, 0.69394E+00_JPRB ,&
&  0.66220E+00_JPRB, 0.62542E+00_JPRB, 0.58659E+00_JPRB, 0.53845E+00_JPRB, 0.51417E+00_JPRB, 0.49359E+00_JPRB, 0.47179E+00_JPRB ,&
&  0.45955E+00_JPRB, 0.43537E+00_JPRB, 0.40900E+00_JPRB, 0.39410E+00_JPRB, 0.38311E+00_JPRB, 0.38040E+00_JPRB, 0.38856E+00_JPRB ,&
&  0.39118E+00_JPRB, 0.38983E+00_JPRB, 0.41836E+00_JPRB, 0.44010E+00_JPRB, 0.45300E+00_JPRB, 0.46900E+00_JPRB, 0.51623E+00_JPRB ,&
&  0.49683E+00_JPRB, 0.51443E+00_JPRB, 0.54902E+00_JPRB, 0.55356E+00_JPRB, 0.55602E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.57681E+00_JPRB, 0.59194E+00_JPRB, 0.58055E+00_JPRB, 0.58247E+00_JPRB, 0.55904E+00_JPRB, 0.54531E+00_JPRB, 0.48776E+00_JPRB ,&
&  0.51037E+00_JPRB, 0.49343E+00_JPRB, 0.47648E+00_JPRB, 0.47263E+00_JPRB, 0.48377E+00_JPRB, 0.42101E+00_JPRB, 0.36735E+00_JPRB ,&
&  0.34192E+00_JPRB, 0.33777E+00_JPRB, 0.35549E+00_JPRB, 0.36533E+00_JPRB, 0.36786E+00_JPRB, 0.37257E+00_JPRB, 0.36898E+00_JPRB ,&
&  0.38602E+00_JPRB, 0.39445E+00_JPRB, 0.40558E+00_JPRB, 0.40765E+00_JPRB, 0.40172E+00_JPRB, 0.39843E+00_JPRB, 0.39786E+00_JPRB ,&
&  0.38340E+00_JPRB, 0.37761E+00_JPRB, 0.36996E+00_JPRB, 0.36903E+00_JPRB, 0.41510E+00_JPRB, 0.40870E+00_JPRB, 0.40216E+00_JPRB ,&
&  0.41082E+00_JPRB, 0.43122E+00_JPRB, 0.46187E+00_JPRB, 0.50009E+00_JPRB, 0.54653E+00_JPRB, 0.59604E+00_JPRB, 0.65040E+00_JPRB ,&
&  0.70233E+00_JPRB, 0.76873E+00_JPRB, 0.81760E+00_JPRB, 0.90753E+00_JPRB, 0.97230E+00_JPRB, 0.91920E+00_JPRB, 0.10829E+01_JPRB ,&
&  0.11755E+01_JPRB, 0.13024E+01_JPRB, 0.14196E+01_JPRB, 0.12800E+01_JPRB, 0.14221E+01_JPRB, 0.12959E+01_JPRB, 0.10932E+01_JPRB ,&
&  0.10528E+01_JPRB, 0.96427E+00_JPRB, 0.93320E+00_JPRB, 0.90544E+00_JPRB, 0.93407E+00_JPRB, 0.90033E+00_JPRB, 0.88458E+00_JPRB ,&
&  0.93029E+00_JPRB, 0.10754E+01_JPRB, 0.10222E+01_JPRB, 0.10365E+01_JPRB, 0.10097E+01_JPRB, 0.97478E+00_JPRB, 0.90011E+00_JPRB ,&
&  0.83741E+00_JPRB, 0.76629E+00_JPRB, 0.69958E+00_JPRB, 0.64537E+00_JPRB, 0.58446E+00_JPRB, 0.53942E+00_JPRB, 0.50563E+00_JPRB ,&
&  0.46893E+00_JPRB, 0.42170E+00_JPRB, 0.40124E+00_JPRB, 0.38869E+00_JPRB, 0.38291E+00_JPRB, 0.36880E+00_JPRB, 0.37013E+00_JPRB ,&
&  0.38577E+00_JPRB, 0.41929E+00_JPRB, 0.47129E+00_JPRB, 0.49018E+00_JPRB, 0.51946E+00_JPRB, 0.55466E+00_JPRB, 0.52476E+00_JPRB ,&
&  0.50217E+00_JPRB, 0.56670E+00_JPRB, 0.57569E+00_JPRB, 0.58014E+00_JPRB, 0.57478E+00_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.62209E+00_JPRB, 0.63338E+00_JPRB, 0.61977E+00_JPRB, 0.60466E+00_JPRB, 0.56803E+00_JPRB, 0.54450E+00_JPRB, 0.51011E+00_JPRB ,&
&  0.51299E+00_JPRB, 0.50022E+00_JPRB, 0.48616E+00_JPRB, 0.48140E+00_JPRB, 0.48487E+00_JPRB, 0.47347E+00_JPRB, 0.41474E+00_JPRB ,&
&  0.36944E+00_JPRB, 0.35841E+00_JPRB, 0.34673E+00_JPRB, 0.35704E+00_JPRB, 0.37009E+00_JPRB, 0.36944E+00_JPRB, 0.37857E+00_JPRB ,&
&  0.39819E+00_JPRB, 0.40434E+00_JPRB, 0.40530E+00_JPRB, 0.40462E+00_JPRB, 0.40399E+00_JPRB, 0.40833E+00_JPRB, 0.40817E+00_JPRB ,&
&  0.39188E+00_JPRB, 0.37957E+00_JPRB, 0.36866E+00_JPRB, 0.37266E+00_JPRB, 0.41634E+00_JPRB, 0.43304E+00_JPRB, 0.43367E+00_JPRB ,&
&  0.44509E+00_JPRB, 0.46274E+00_JPRB, 0.48389E+00_JPRB, 0.52234E+00_JPRB, 0.57458E+00_JPRB, 0.63749E+00_JPRB, 0.70403E+00_JPRB ,&
&  0.76909E+00_JPRB, 0.83964E+00_JPRB, 0.89673E+00_JPRB, 0.96830E+00_JPRB, 0.95024E+00_JPRB, 0.10129E+01_JPRB, 0.11785E+01_JPRB ,&
&  0.12833E+01_JPRB, 0.14104E+01_JPRB, 0.14760E+01_JPRB, 0.16016E+01_JPRB, 0.16786E+01_JPRB, 0.15100E+01_JPRB, 0.13212E+01_JPRB ,&
&  0.12131E+01_JPRB, 0.11392E+01_JPRB, 0.14047E+01_JPRB, 0.12428E+01_JPRB, 0.11349E+01_JPRB, 0.11203E+01_JPRB, 0.10893E+01_JPRB ,&
&  0.11299E+01_JPRB, 0.12155E+01_JPRB, 0.12539E+01_JPRB, 0.12872E+01_JPRB, 0.12888E+01_JPRB, 0.12865E+01_JPRB, 0.12100E+01_JPRB ,&
&  0.11527E+01_JPRB, 0.10437E+01_JPRB, 0.82730E+00_JPRB, 0.67272E+00_JPRB, 0.56110E+00_JPRB, 0.55208E+00_JPRB, 0.51627E+00_JPRB ,&
&  0.46953E+00_JPRB, 0.41973E+00_JPRB, 0.40956E+00_JPRB, 0.39607E+00_JPRB, 0.39430E+00_JPRB, 0.38877E+00_JPRB, 0.37790E+00_JPRB ,&
&  0.39000E+00_JPRB, 0.43670E+00_JPRB, 0.46272E+00_JPRB, 0.50887E+00_JPRB, 0.56474E+00_JPRB, 0.63377E+00_JPRB, 0.54542E+00_JPRB ,&
&  0.56930E+00_JPRB, 0.63474E+00_JPRB, 0.64921E+00_JPRB, 0.65072E+00_JPRB, 0.63178E+00_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.71701E+00_JPRB, 0.70883E+00_JPRB, 0.68536E+00_JPRB, 0.64393E+00_JPRB, 0.59439E+00_JPRB, 0.55236E+00_JPRB, 0.51390E+00_JPRB ,&
&  0.51039E+00_JPRB, 0.49758E+00_JPRB, 0.48557E+00_JPRB, 0.48512E+00_JPRB, 0.48596E+00_JPRB, 0.48317E+00_JPRB, 0.48124E+00_JPRB ,&
&  0.40906E+00_JPRB, 0.39567E+00_JPRB, 0.36745E+00_JPRB, 0.37491E+00_JPRB, 0.42163E+00_JPRB, 0.39085E+00_JPRB, 0.39692E+00_JPRB ,&
&  0.40241E+00_JPRB, 0.41381E+00_JPRB, 0.40803E+00_JPRB, 0.40709E+00_JPRB, 0.41543E+00_JPRB, 0.42447E+00_JPRB, 0.42350E+00_JPRB ,&
&  0.40696E+00_JPRB, 0.38938E+00_JPRB, 0.38480E+00_JPRB, 0.39146E+00_JPRB, 0.43349E+00_JPRB, 0.46714E+00_JPRB, 0.48690E+00_JPRB ,&
&  0.50589E+00_JPRB, 0.52255E+00_JPRB, 0.53551E+00_JPRB, 0.56528E+00_JPRB, 0.61288E+00_JPRB, 0.67063E+00_JPRB, 0.75378E+00_JPRB ,&
&  0.83575E+00_JPRB, 0.91084E+00_JPRB, 0.99119E+00_JPRB, 0.10685E+01_JPRB, 0.10672E+01_JPRB, 0.11898E+01_JPRB, 0.13240E+01_JPRB ,&
&  0.14486E+01_JPRB, 0.14017E+01_JPRB, 0.14181E+01_JPRB, 0.16984E+01_JPRB, 0.18759E+01_JPRB, 0.18046E+01_JPRB, 0.16136E+01_JPRB ,&
&  0.14418E+01_JPRB, 0.13227E+01_JPRB, 0.13586E+01_JPRB, 0.13138E+01_JPRB, 0.12094E+01_JPRB, 0.12536E+01_JPRB, 0.12690E+01_JPRB ,&
&  0.13031E+01_JPRB, 0.13731E+01_JPRB, 0.14646E+01_JPRB, 0.15297E+01_JPRB, 0.14916E+01_JPRB, 0.15118E+01_JPRB, 0.14617E+01_JPRB ,&
&  0.13913E+01_JPRB, 0.11694E+01_JPRB, 0.84372E+00_JPRB, 0.67563E+00_JPRB, 0.51628E+00_JPRB, 0.49229E+00_JPRB, 0.55352E+00_JPRB ,&
&  0.53684E+00_JPRB, 0.50771E+00_JPRB, 0.47304E+00_JPRB, 0.44561E+00_JPRB, 0.43230E+00_JPRB, 0.44393E+00_JPRB, 0.43545E+00_JPRB ,&
&  0.42478E+00_JPRB, 0.46990E+00_JPRB, 0.54640E+00_JPRB, 0.63922E+00_JPRB, 0.70553E+00_JPRB, 0.75107E+00_JPRB, 0.69667E+00_JPRB ,&
&  0.75777E+00_JPRB, 0.75473E+00_JPRB, 0.75593E+00_JPRB, 0.76474E+00_JPRB, 0.74882E+00_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.81378E+00_JPRB, 0.78988E+00_JPRB, 0.74675E+00_JPRB, 0.68051E+00_JPRB, 0.60815E+00_JPRB, 0.54892E+00_JPRB, 0.52028E+00_JPRB ,&
&  0.51472E+00_JPRB, 0.50286E+00_JPRB, 0.48854E+00_JPRB, 0.48646E+00_JPRB, 0.48666E+00_JPRB, 0.48285E+00_JPRB, 0.48705E+00_JPRB ,&
&  0.51249E+00_JPRB, 0.47937E+00_JPRB, 0.41790E+00_JPRB, 0.37728E+00_JPRB, 0.39831E+00_JPRB, 0.39570E+00_JPRB, 0.40385E+00_JPRB ,&
&  0.40580E+00_JPRB, 0.42568E+00_JPRB, 0.41028E+00_JPRB, 0.41477E+00_JPRB, 0.44086E+00_JPRB, 0.45829E+00_JPRB, 0.46607E+00_JPRB ,&
&  0.46322E+00_JPRB, 0.45492E+00_JPRB, 0.45724E+00_JPRB, 0.48232E+00_JPRB, 0.53241E+00_JPRB, 0.52397E+00_JPRB, 0.55965E+00_JPRB ,&
&  0.57557E+00_JPRB, 0.59861E+00_JPRB, 0.60452E+00_JPRB, 0.63780E+00_JPRB, 0.69771E+00_JPRB, 0.79698E+00_JPRB, 0.87336E+00_JPRB ,&
&  0.93409E+00_JPRB, 0.98329E+00_JPRB, 0.10097E+01_JPRB, 0.10901E+01_JPRB, 0.12056E+01_JPRB, 0.13559E+01_JPRB, 0.13265E+01_JPRB ,&
&  0.13405E+01_JPRB, 0.12782E+01_JPRB, 0.12367E+01_JPRB, 0.14826E+01_JPRB, 0.17249E+01_JPRB, 0.17758E+01_JPRB, 0.17285E+01_JPRB ,&
&  0.16456E+01_JPRB, 0.15617E+01_JPRB, 0.16023E+01_JPRB, 0.15344E+01_JPRB, 0.14853E+01_JPRB, 0.15179E+01_JPRB, 0.15740E+01_JPRB ,&
&  0.15592E+01_JPRB, 0.16197E+01_JPRB, 0.17247E+01_JPRB, 0.17269E+01_JPRB, 0.17045E+01_JPRB, 0.16362E+01_JPRB, 0.16335E+01_JPRB ,&
&  0.15736E+01_JPRB, 0.96420E+00_JPRB, 0.75770E+00_JPRB, 0.77490E+00_JPRB, 0.58755E+00_JPRB, 0.50956E+00_JPRB, 0.52994E+00_JPRB ,&
&  0.50935E+00_JPRB, 0.50511E+00_JPRB, 0.52141E+00_JPRB, 0.50679E+00_JPRB, 0.50005E+00_JPRB, 0.53069E+00_JPRB, 0.54450E+00_JPRB ,&
&  0.57054E+00_JPRB, 0.60002E+00_JPRB, 0.74548E+00_JPRB, 0.83967E+00_JPRB, 0.88044E+00_JPRB, 0.89242E+00_JPRB, 0.88941E+00_JPRB ,&
&  0.87256E+00_JPRB, 0.85803E+00_JPRB, 0.84735E+00_JPRB, 0.84537E+00_JPRB, 0.84174E+00_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.92390E+00_JPRB, 0.89108E+00_JPRB, 0.83071E+00_JPRB, 0.74811E+00_JPRB, 0.65417E+00_JPRB, 0.58296E+00_JPRB, 0.56060E+00_JPRB ,&
&  0.54572E+00_JPRB, 0.53624E+00_JPRB, 0.52028E+00_JPRB, 0.51079E+00_JPRB, 0.50690E+00_JPRB, 0.50130E+00_JPRB, 0.50348E+00_JPRB ,&
&  0.50787E+00_JPRB, 0.51250E+00_JPRB, 0.42995E+00_JPRB, 0.37305E+00_JPRB, 0.34339E+00_JPRB, 0.36235E+00_JPRB, 0.37999E+00_JPRB ,&
&  0.40201E+00_JPRB, 0.42390E+00_JPRB, 0.42917E+00_JPRB, 0.47304E+00_JPRB, 0.51603E+00_JPRB, 0.53608E+00_JPRB, 0.56956E+00_JPRB ,&
&  0.57732E+00_JPRB, 0.57220E+00_JPRB, 0.56475E+00_JPRB, 0.61232E+00_JPRB, 0.62530E+00_JPRB, 0.59401E+00_JPRB, 0.65419E+00_JPRB ,&
&  0.68324E+00_JPRB, 0.71219E+00_JPRB, 0.73611E+00_JPRB, 0.78871E+00_JPRB, 0.85100E+00_JPRB, 0.88083E+00_JPRB, 0.87009E+00_JPRB ,&
&  0.89327E+00_JPRB, 0.94180E+00_JPRB, 0.10124E+01_JPRB, 0.10722E+01_JPRB, 0.11526E+01_JPRB, 0.12366E+01_JPRB, 0.11978E+01_JPRB ,&
&  0.11995E+01_JPRB, 0.10348E+01_JPRB, 0.89992E+00_JPRB, 0.11383E+01_JPRB, 0.15833E+01_JPRB, 0.17625E+01_JPRB, 0.16568E+01_JPRB ,&
&  0.17587E+01_JPRB, 0.17796E+01_JPRB, 0.18489E+01_JPRB, 0.18150E+01_JPRB, 0.17500E+01_JPRB, 0.17779E+01_JPRB, 0.18720E+01_JPRB ,&
&  0.18923E+01_JPRB, 0.19739E+01_JPRB, 0.20263E+01_JPRB, 0.20004E+01_JPRB, 0.19555E+01_JPRB, 0.19580E+01_JPRB, 0.19616E+01_JPRB ,&
&  0.17797E+01_JPRB, 0.14829E+01_JPRB, 0.10385E+01_JPRB, 0.98166E+00_JPRB, 0.69507E+00_JPRB, 0.57012E+00_JPRB, 0.56289E+00_JPRB ,&
&  0.51560E+00_JPRB, 0.52891E+00_JPRB, 0.55757E+00_JPRB, 0.55692E+00_JPRB, 0.62039E+00_JPRB, 0.67282E+00_JPRB, 0.74851E+00_JPRB ,&
&  0.81693E+00_JPRB, 0.82905E+00_JPRB, 0.99107E+00_JPRB, 0.10279E+01_JPRB, 0.10529E+01_JPRB, 0.10728E+01_JPRB, 0.10742E+01_JPRB ,&
&  0.10467E+01_JPRB, 0.10128E+01_JPRB, 0.98458E+00_JPRB, 0.95757E+00_JPRB, 0.94088E+00_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.95366E+00_JPRB, 0.90477E+00_JPRB, 0.82496E+00_JPRB, 0.74747E+00_JPRB, 0.66225E+00_JPRB, 0.59011E+00_JPRB, 0.57749E+00_JPRB ,&
&  0.59912E+00_JPRB, 0.59818E+00_JPRB, 0.58871E+00_JPRB, 0.56505E+00_JPRB, 0.55236E+00_JPRB, 0.54451E+00_JPRB, 0.53445E+00_JPRB ,&
&  0.52384E+00_JPRB, 0.47617E+00_JPRB, 0.38676E+00_JPRB, 0.33641E+00_JPRB, 0.32230E+00_JPRB, 0.32458E+00_JPRB, 0.35452E+00_JPRB ,&
&  0.38904E+00_JPRB, 0.44460E+00_JPRB, 0.50104E+00_JPRB, 0.56951E+00_JPRB, 0.65647E+00_JPRB, 0.74690E+00_JPRB, 0.78289E+00_JPRB ,&
&  0.81164E+00_JPRB, 0.70160E+00_JPRB, 0.73332E+00_JPRB, 0.72604E+00_JPRB, 0.75995E+00_JPRB, 0.76123E+00_JPRB, 0.77086E+00_JPRB ,&
&  0.76266E+00_JPRB, 0.78961E+00_JPRB, 0.81851E+00_JPRB, 0.84575E+00_JPRB, 0.86014E+00_JPRB, 0.86778E+00_JPRB, 0.87632E+00_JPRB ,&
&  0.93357E+00_JPRB, 0.99837E+00_JPRB, 0.10385E+01_JPRB, 0.10659E+01_JPRB, 0.11297E+01_JPRB, 0.12137E+01_JPRB, 0.12159E+01_JPRB ,&
&  0.11192E+01_JPRB, 0.10516E+01_JPRB, 0.11942E+01_JPRB, 0.13450E+01_JPRB, 0.15196E+01_JPRB, 0.17420E+01_JPRB, 0.18378E+01_JPRB ,&
&  0.19015E+01_JPRB, 0.18460E+01_JPRB, 0.18787E+01_JPRB, 0.17658E+01_JPRB, 0.19770E+01_JPRB, 0.20253E+01_JPRB, 0.21062E+01_JPRB ,&
&  0.21721E+01_JPRB, 0.23073E+01_JPRB, 0.22428E+01_JPRB, 0.21708E+01_JPRB, 0.22069E+01_JPRB, 0.23239E+01_JPRB, 0.20846E+01_JPRB ,&
&  0.17481E+01_JPRB, 0.18646E+01_JPRB, 0.16423E+01_JPRB, 0.13497E+01_JPRB, 0.10259E+01_JPRB, 0.87579E+00_JPRB, 0.74673E+00_JPRB ,&
&  0.65825E+00_JPRB, 0.61765E+00_JPRB, 0.62379E+00_JPRB, 0.71354E+00_JPRB, 0.85821E+00_JPRB, 0.98172E+00_JPRB, 0.10516E+01_JPRB ,&
&  0.10866E+01_JPRB, 0.13230E+01_JPRB, 0.13172E+01_JPRB, 0.12747E+01_JPRB, 0.12828E+01_JPRB, 0.12872E+01_JPRB, 0.12559E+01_JPRB ,&
&  0.11792E+01_JPRB, 0.11215E+01_JPRB, 0.10761E+01_JPRB, 0.10344E+01_JPRB, 0.98989E+00_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.97204E+00_JPRB, 0.92552E+00_JPRB, 0.84398E+00_JPRB, 0.75998E+00_JPRB, 0.67979E+00_JPRB, 0.61329E+00_JPRB, 0.59584E+00_JPRB ,&
&  0.60651E+00_JPRB, 0.61947E+00_JPRB, 0.63066E+00_JPRB, 0.63590E+00_JPRB, 0.63376E+00_JPRB, 0.61442E+00_JPRB, 0.58993E+00_JPRB ,&
&  0.57065E+00_JPRB, 0.46114E+00_JPRB, 0.36686E+00_JPRB, 0.33901E+00_JPRB, 0.32426E+00_JPRB, 0.31909E+00_JPRB, 0.33891E+00_JPRB ,&
&  0.38942E+00_JPRB, 0.49946E+00_JPRB, 0.62277E+00_JPRB, 0.77682E+00_JPRB, 0.91369E+00_JPRB, 0.10112E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.93629E+00_JPRB, 0.93223E+00_JPRB, 0.93164E+00_JPRB, 0.89571E+00_JPRB, 0.88518E+00_JPRB, 0.85367E+00_JPRB, 0.83539E+00_JPRB ,&
&  0.82738E+00_JPRB, 0.82796E+00_JPRB, 0.87268E+00_JPRB, 0.90645E+00_JPRB, 0.93316E+00_JPRB, 0.95729E+00_JPRB, 0.98234E+00_JPRB ,&
&  0.10037E+01_JPRB, 0.10289E+01_JPRB, 0.10410E+01_JPRB, 0.11131E+01_JPRB, 0.95785E+00_JPRB, 0.98670E+00_JPRB, 0.10832E+01_JPRB ,&
&  0.12951E+01_JPRB, 0.12981E+01_JPRB, 0.12551E+01_JPRB, 0.13905E+01_JPRB, 0.13245E+01_JPRB, 0.14246E+01_JPRB, 0.17109E+01_JPRB ,&
&  0.18274E+01_JPRB, 0.18546E+01_JPRB, 0.19671E+01_JPRB, 0.17248E+01_JPRB, 0.16961E+01_JPRB, 0.23864E+01_JPRB, 0.25421E+01_JPRB ,&
&  0.25707E+01_JPRB, 0.25783E+01_JPRB, 0.24661E+01_JPRB, 0.24602E+01_JPRB, 0.21355E+01_JPRB, 0.18638E+01_JPRB, 0.14387E+01_JPRB ,&
&  0.15247E+01_JPRB, 0.14712E+01_JPRB, 0.16249E+01_JPRB, 0.14840E+01_JPRB, 0.13705E+01_JPRB, 0.12095E+01_JPRB, 0.99070E+00_JPRB ,&
&  0.85273E+00_JPRB, 0.77818E+00_JPRB, 0.77705E+00_JPRB, 0.97772E+00_JPRB, 0.12366E+01_JPRB, 0.12940E+01_JPRB, 0.15365E+01_JPRB ,&
&  0.16515E+01_JPRB, 0.16987E+01_JPRB, 0.15104E+01_JPRB, 0.14830E+01_JPRB, 0.14168E+01_JPRB, 0.13530E+01_JPRB, 0.12734E+01_JPRB ,&
&  0.12063E+01_JPRB, 0.11491E+01_JPRB, 0.10964E+01_JPRB, 0.10462E+01_JPRB, 0.10011E+01_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.98254E+00_JPRB, 0.94448E+00_JPRB, 0.86529E+00_JPRB, 0.78137E+00_JPRB, 0.70439E+00_JPRB, 0.65109E+00_JPRB, 0.63741E+00_JPRB ,&
&  0.64931E+00_JPRB, 0.66264E+00_JPRB, 0.66902E+00_JPRB, 0.66598E+00_JPRB, 0.65774E+00_JPRB, 0.65624E+00_JPRB, 0.64799E+00_JPRB ,&
&  0.62099E+00_JPRB, 0.51441E+00_JPRB, 0.39061E+00_JPRB, 0.34479E+00_JPRB, 0.35031E+00_JPRB, 0.33861E+00_JPRB, 0.34670E+00_JPRB ,&
&  0.43882E+00_JPRB, 0.60416E+00_JPRB, 0.82306E+00_JPRB, 0.10242E+01_JPRB, 0.11875E+01_JPRB, 0.12191E+01_JPRB, 0.11585E+01_JPRB ,&
&  0.11560E+01_JPRB, 0.10936E+01_JPRB, 0.97258E+00_JPRB, 0.88184E+00_JPRB, 0.84240E+00_JPRB, 0.82723E+00_JPRB, 0.80359E+00_JPRB ,&
&  0.78595E+00_JPRB, 0.79463E+00_JPRB, 0.82384E+00_JPRB, 0.84610E+00_JPRB, 0.88056E+00_JPRB, 0.89315E+00_JPRB, 0.89600E+00_JPRB ,&
&  0.92215E+00_JPRB, 0.96587E+00_JPRB, 0.10221E+01_JPRB, 0.11172E+01_JPRB, 0.10262E+01_JPRB, 0.10353E+01_JPRB, 0.11806E+01_JPRB ,&
&  0.13025E+01_JPRB, 0.13127E+01_JPRB, 0.13106E+01_JPRB, 0.13504E+01_JPRB, 0.14548E+01_JPRB, 0.14538E+01_JPRB, 0.16819E+01_JPRB ,&
&  0.16297E+01_JPRB, 0.15587E+01_JPRB, 0.15652E+01_JPRB, 0.15251E+01_JPRB, 0.19086E+01_JPRB, 0.26246E+01_JPRB, 0.29646E+01_JPRB ,&
&  0.29201E+01_JPRB, 0.28471E+01_JPRB, 0.27533E+01_JPRB, 0.24127E+01_JPRB, 0.18980E+01_JPRB, 0.14325E+01_JPRB, 0.16833E+01_JPRB ,&
&  0.18213E+01_JPRB, 0.18643E+01_JPRB, 0.17687E+01_JPRB, 0.15008E+01_JPRB, 0.13085E+01_JPRB, 0.14462E+01_JPRB, 0.13056E+01_JPRB ,&
&  0.10866E+01_JPRB, 0.99109E+00_JPRB, 0.11666E+01_JPRB, 0.14169E+01_JPRB, 0.15654E+01_JPRB, 0.16988E+01_JPRB, 0.18962E+01_JPRB ,&
&  0.18226E+01_JPRB, 0.17871E+01_JPRB, 0.16738E+01_JPRB, 0.15291E+01_JPRB, 0.14089E+01_JPRB, 0.13178E+01_JPRB, 0.12414E+01_JPRB ,&
&  0.11965E+01_JPRB, 0.11600E+01_JPRB, 0.11105E+01_JPRB, 0.10637E+01_JPRB, 0.10147E+01_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.99814E+00_JPRB, 0.96211E+00_JPRB, 0.88043E+00_JPRB, 0.79041E+00_JPRB, 0.73043E+00_JPRB, 0.69379E+00_JPRB, 0.67739E+00_JPRB ,&
&  0.67962E+00_JPRB, 0.68194E+00_JPRB, 0.68441E+00_JPRB, 0.68196E+00_JPRB, 0.67562E+00_JPRB, 0.66720E+00_JPRB, 0.67006E+00_JPRB ,&
&  0.65171E+00_JPRB, 0.62808E+00_JPRB, 0.46232E+00_JPRB, 0.39458E+00_JPRB, 0.38545E+00_JPRB, 0.38750E+00_JPRB, 0.39012E+00_JPRB ,&
&  0.53379E+00_JPRB, 0.80783E+00_JPRB, 0.10504E+01_JPRB, 0.12372E+01_JPRB, 0.13741E+01_JPRB, 0.12159E+01_JPRB, 0.11623E+01_JPRB ,&
&  0.11887E+01_JPRB, 0.11400E+01_JPRB, 0.10253E+01_JPRB, 0.92517E+00_JPRB, 0.84339E+00_JPRB, 0.80830E+00_JPRB, 0.79484E+00_JPRB ,&
&  0.78451E+00_JPRB, 0.79003E+00_JPRB, 0.80929E+00_JPRB, 0.84682E+00_JPRB, 0.86761E+00_JPRB, 0.87794E+00_JPRB, 0.89148E+00_JPRB ,&
&  0.90611E+00_JPRB, 0.93432E+00_JPRB, 0.97753E+00_JPRB, 0.11166E+01_JPRB, 0.11079E+01_JPRB, 0.11756E+01_JPRB, 0.14251E+01_JPRB ,&
&  0.14310E+01_JPRB, 0.14664E+01_JPRB, 0.14136E+01_JPRB, 0.14438E+01_JPRB, 0.14783E+01_JPRB, 0.15062E+01_JPRB, 0.16996E+01_JPRB ,&
&  0.16687E+01_JPRB, 0.16888E+01_JPRB, 0.18423E+01_JPRB, 0.19590E+01_JPRB, 0.20787E+01_JPRB, 0.24407E+01_JPRB, 0.33452E+01_JPRB ,&
&  0.29105E+01_JPRB, 0.28751E+01_JPRB, 0.28085E+01_JPRB, 0.25156E+01_JPRB, 0.13492E+01_JPRB, 0.10014E+01_JPRB, 0.14581E+01_JPRB ,&
&  0.15409E+01_JPRB, 0.99238E+00_JPRB, 0.76447E+00_JPRB, 0.10341E+01_JPRB, 0.83572E+00_JPRB, 0.98466E+00_JPRB, 0.13159E+01_JPRB ,&
&  0.13394E+01_JPRB, 0.12281E+01_JPRB, 0.14478E+01_JPRB, 0.15568E+01_JPRB, 0.16139E+01_JPRB, 0.19414E+01_JPRB, 0.20298E+01_JPRB ,&
&  0.19677E+01_JPRB, 0.17250E+01_JPRB, 0.17091E+01_JPRB, 0.15365E+01_JPRB, 0.14135E+01_JPRB, 0.13443E+01_JPRB, 0.12909E+01_JPRB ,&
&  0.12232E+01_JPRB, 0.11503E+01_JPRB, 0.10960E+01_JPRB, 0.10623E+01_JPRB, 0.10266E+01_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.99616E+00_JPRB, 0.95756E+00_JPRB, 0.87337E+00_JPRB, 0.78496E+00_JPRB, 0.73943E+00_JPRB, 0.70800E+00_JPRB, 0.68962E+00_JPRB ,&
&  0.68995E+00_JPRB, 0.69623E+00_JPRB, 0.70061E+00_JPRB, 0.70293E+00_JPRB, 0.69964E+00_JPRB, 0.69448E+00_JPRB, 0.69057E+00_JPRB ,&
&  0.67191E+00_JPRB, 0.63135E+00_JPRB, 0.54763E+00_JPRB, 0.48134E+00_JPRB, 0.46270E+00_JPRB, 0.45104E+00_JPRB, 0.48656E+00_JPRB ,&
&  0.64136E+00_JPRB, 0.99118E+00_JPRB, 0.12745E+01_JPRB, 0.13820E+01_JPRB, 0.12930E+01_JPRB, 0.11716E+01_JPRB, 0.12398E+01_JPRB ,&
&  0.12512E+01_JPRB, 0.11794E+01_JPRB, 0.10419E+01_JPRB, 0.95273E+00_JPRB, 0.88231E+00_JPRB, 0.84039E+00_JPRB, 0.82593E+00_JPRB ,&
&  0.80669E+00_JPRB, 0.80694E+00_JPRB, 0.83716E+00_JPRB, 0.85581E+00_JPRB, 0.86356E+00_JPRB, 0.86892E+00_JPRB, 0.87410E+00_JPRB ,&
&  0.89609E+00_JPRB, 0.93890E+00_JPRB, 0.10072E+01_JPRB, 0.11262E+01_JPRB, 0.13495E+01_JPRB, 0.14720E+01_JPRB, 0.14137E+01_JPRB ,&
&  0.13306E+01_JPRB, 0.13629E+01_JPRB, 0.15296E+01_JPRB, 0.14584E+01_JPRB, 0.14325E+01_JPRB, 0.14780E+01_JPRB, 0.16347E+01_JPRB ,&
&  0.19017E+01_JPRB, 0.20700E+01_JPRB, 0.19587E+01_JPRB, 0.20739E+01_JPRB, 0.22351E+01_JPRB, 0.21017E+01_JPRB, 0.27781E+01_JPRB ,&
&  0.30040E+01_JPRB, 0.25971E+01_JPRB, 0.19920E+01_JPRB, 0.14577E+01_JPRB, 0.13934E+01_JPRB, 0.10665E+01_JPRB, 0.68383E+00_JPRB ,&
&  0.62466E+00_JPRB, 0.59447E+00_JPRB, 0.55442E+00_JPRB, 0.61567E+00_JPRB, 0.62335E+00_JPRB, 0.81694E+00_JPRB, 0.12123E+01_JPRB ,&
&  0.15148E+01_JPRB, 0.15433E+01_JPRB, 0.16234E+01_JPRB, 0.16197E+01_JPRB, 0.15917E+01_JPRB, 0.18960E+01_JPRB, 0.20903E+01_JPRB ,&
&  0.17935E+01_JPRB, 0.16450E+01_JPRB, 0.15995E+01_JPRB, 0.14449E+01_JPRB, 0.13836E+01_JPRB, 0.13402E+01_JPRB, 0.12673E+01_JPRB ,&
&  0.11725E+01_JPRB, 0.10824E+01_JPRB, 0.10402E+01_JPRB, 0.10259E+01_JPRB, 0.10194E+01_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.99090E+00_JPRB, 0.94741E+00_JPRB, 0.87671E+00_JPRB, 0.80290E+00_JPRB, 0.77107E+00_JPRB, 0.74794E+00_JPRB, 0.72939E+00_JPRB ,&
&  0.72859E+00_JPRB, 0.73288E+00_JPRB, 0.74089E+00_JPRB, 0.74616E+00_JPRB, 0.74344E+00_JPRB, 0.73409E+00_JPRB, 0.72201E+00_JPRB ,&
&  0.70164E+00_JPRB, 0.65192E+00_JPRB, 0.61315E+00_JPRB, 0.56078E+00_JPRB, 0.54707E+00_JPRB, 0.52728E+00_JPRB, 0.58778E+00_JPRB ,&
&  0.78819E+00_JPRB, 0.11916E+01_JPRB, 0.14844E+01_JPRB, 0.15130E+01_JPRB, 0.14244E+01_JPRB, 0.13578E+01_JPRB, 0.13452E+01_JPRB ,&
&  0.12752E+01_JPRB, 0.11973E+01_JPRB, 0.10706E+01_JPRB, 0.98058E+00_JPRB, 0.91356E+00_JPRB, 0.87099E+00_JPRB, 0.84634E+00_JPRB ,&
&  0.82883E+00_JPRB, 0.85296E+00_JPRB, 0.85681E+00_JPRB, 0.86323E+00_JPRB, 0.86281E+00_JPRB, 0.86396E+00_JPRB, 0.87641E+00_JPRB ,&
&  0.91754E+00_JPRB, 0.99226E+00_JPRB, 0.11174E+01_JPRB, 0.13163E+01_JPRB, 0.13509E+01_JPRB, 0.12328E+01_JPRB, 0.13229E+01_JPRB ,&
&  0.13748E+01_JPRB, 0.14794E+01_JPRB, 0.14477E+01_JPRB, 0.15293E+01_JPRB, 0.14647E+01_JPRB, 0.14309E+01_JPRB, 0.16148E+01_JPRB ,&
&  0.19266E+01_JPRB, 0.23046E+01_JPRB, 0.19833E+01_JPRB, 0.19290E+01_JPRB, 0.24389E+01_JPRB, 0.22946E+01_JPRB, 0.25025E+01_JPRB ,&
&  0.28022E+01_JPRB, 0.23885E+01_JPRB, 0.19404E+01_JPRB, 0.15350E+01_JPRB, 0.18196E+01_JPRB, 0.17485E+01_JPRB, 0.94367E+00_JPRB ,&
&  0.70874E+00_JPRB, 0.61594E+00_JPRB, 0.56685E+00_JPRB, 0.55590E+00_JPRB, 0.56779E+00_JPRB, 0.78267E+00_JPRB, 0.13416E+01_JPRB ,&
&  0.14931E+01_JPRB, 0.17246E+01_JPRB, 0.17802E+01_JPRB, 0.17427E+01_JPRB, 0.16966E+01_JPRB, 0.17126E+01_JPRB, 0.18020E+01_JPRB ,&
&  0.18837E+01_JPRB, 0.17406E+01_JPRB, 0.15362E+01_JPRB, 0.13942E+01_JPRB, 0.13056E+01_JPRB, 0.11998E+01_JPRB, 0.10991E+01_JPRB ,&
&  0.10230E+01_JPRB, 0.99242E+00_JPRB, 0.98411E+00_JPRB, 0.99516E+00_JPRB, 0.10142E+01_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.98775E+00_JPRB, 0.96873E+00_JPRB, 0.90997E+00_JPRB, 0.84373E+00_JPRB, 0.81115E+00_JPRB, 0.78873E+00_JPRB, 0.77439E+00_JPRB ,&
&  0.77663E+00_JPRB, 0.78192E+00_JPRB, 0.77968E+00_JPRB, 0.77059E+00_JPRB, 0.76663E+00_JPRB, 0.75117E+00_JPRB, 0.73535E+00_JPRB ,&
&  0.71358E+00_JPRB, 0.66858E+00_JPRB, 0.62923E+00_JPRB, 0.62238E+00_JPRB, 0.60681E+00_JPRB, 0.59945E+00_JPRB, 0.69734E+00_JPRB ,&
&  0.99790E+00_JPRB, 0.14658E+01_JPRB, 0.16653E+01_JPRB, 0.16710E+01_JPRB, 0.15706E+01_JPRB, 0.14761E+01_JPRB, 0.13127E+01_JPRB ,&
&  0.11486E+01_JPRB, 0.11174E+01_JPRB, 0.10160E+01_JPRB, 0.97405E+00_JPRB, 0.94299E+00_JPRB, 0.91259E+00_JPRB, 0.89187E+00_JPRB ,&
&  0.88633E+00_JPRB, 0.86936E+00_JPRB, 0.86286E+00_JPRB, 0.86074E+00_JPRB, 0.86173E+00_JPRB, 0.87019E+00_JPRB, 0.91639E+00_JPRB ,&
&  0.10014E+01_JPRB, 0.10988E+01_JPRB, 0.12642E+01_JPRB, 0.14160E+01_JPRB, 0.11753E+01_JPRB, 0.12816E+01_JPRB, 0.13102E+01_JPRB ,&
&  0.13219E+01_JPRB, 0.13534E+01_JPRB, 0.12953E+01_JPRB, 0.13471E+01_JPRB, 0.14283E+01_JPRB, 0.14335E+01_JPRB, 0.15595E+01_JPRB ,&
&  0.20322E+01_JPRB, 0.19122E+01_JPRB, 0.17617E+01_JPRB, 0.18342E+01_JPRB, 0.22381E+01_JPRB, 0.32552E+01_JPRB, 0.22991E+01_JPRB ,&
&  0.20903E+01_JPRB, 0.23435E+01_JPRB, 0.19544E+01_JPRB, 0.16454E+01_JPRB, 0.18680E+01_JPRB, 0.21782E+01_JPRB, 0.19639E+01_JPRB ,&
&  0.92616E+00_JPRB, 0.66979E+00_JPRB, 0.66399E+00_JPRB, 0.66736E+00_JPRB, 0.63948E+00_JPRB, 0.79477E+00_JPRB, 0.18561E+01_JPRB ,&
&  0.16387E+01_JPRB, 0.18559E+01_JPRB, 0.18473E+01_JPRB, 0.17277E+01_JPRB, 0.17267E+01_JPRB, 0.17549E+01_JPRB, 0.18054E+01_JPRB ,&
&  0.15981E+01_JPRB, 0.14360E+01_JPRB, 0.13067E+01_JPRB, 0.11711E+01_JPRB, 0.10671E+01_JPRB, 0.99623E+00_JPRB, 0.95933E+00_JPRB ,&
&  0.94030E+00_JPRB, 0.93710E+00_JPRB, 0.94917E+00_JPRB, 0.96930E+00_JPRB, 0.98738E+00_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.98734E+00_JPRB, 0.98844E+00_JPRB, 0.93391E+00_JPRB, 0.87198E+00_JPRB, 0.83588E+00_JPRB, 0.81722E+00_JPRB, 0.80024E+00_JPRB ,&
&  0.85313E+00_JPRB, 0.84233E+00_JPRB, 0.80488E+00_JPRB, 0.77309E+00_JPRB, 0.76522E+00_JPRB, 0.74720E+00_JPRB, 0.73321E+00_JPRB ,&
&  0.71008E+00_JPRB, 0.68060E+00_JPRB, 0.64869E+00_JPRB, 0.63023E+00_JPRB, 0.64729E+00_JPRB, 0.61837E+00_JPRB, 0.72281E+00_JPRB ,&
&  0.11885E+01_JPRB, 0.18023E+01_JPRB, 0.17390E+01_JPRB, 0.17034E+01_JPRB, 0.15974E+01_JPRB, 0.15003E+01_JPRB, 0.12233E+01_JPRB ,&
&  0.10918E+01_JPRB, 0.10789E+01_JPRB, 0.10010E+01_JPRB, 0.98534E+00_JPRB, 0.97037E+00_JPRB, 0.95344E+00_JPRB, 0.93661E+00_JPRB ,&
&  0.89441E+00_JPRB, 0.87303E+00_JPRB, 0.88075E+00_JPRB, 0.88653E+00_JPRB, 0.88749E+00_JPRB, 0.92585E+00_JPRB, 0.10020E+01_JPRB ,&
&  0.10859E+01_JPRB, 0.11992E+01_JPRB, 0.13705E+01_JPRB, 0.13552E+01_JPRB, 0.12823E+01_JPRB, 0.12729E+01_JPRB, 0.13032E+01_JPRB ,&
&  0.12862E+01_JPRB, 0.13042E+01_JPRB, 0.13073E+01_JPRB, 0.12895E+01_JPRB, 0.13588E+01_JPRB, 0.14591E+01_JPRB, 0.15627E+01_JPRB ,&
&  0.17923E+01_JPRB, 0.16891E+01_JPRB, 0.15764E+01_JPRB, 0.17465E+01_JPRB, 0.21520E+01_JPRB, 0.29318E+01_JPRB, 0.29765E+01_JPRB ,&
&  0.26091E+01_JPRB, 0.19336E+01_JPRB, 0.16805E+01_JPRB, 0.18330E+01_JPRB, 0.19920E+01_JPRB, 0.21194E+01_JPRB, 0.26921E+01_JPRB ,&
&  0.26194E+01_JPRB, 0.15448E+01_JPRB, 0.12695E+01_JPRB, 0.15503E+01_JPRB, 0.11280E+01_JPRB, 0.88293E+00_JPRB, 0.12894E+01_JPRB ,&
&  0.15488E+01_JPRB, 0.17567E+01_JPRB, 0.17991E+01_JPRB, 0.17244E+01_JPRB, 0.17507E+01_JPRB, 0.16809E+01_JPRB, 0.15021E+01_JPRB ,&
&  0.13220E+01_JPRB, 0.11789E+01_JPRB, 0.10742E+01_JPRB, 0.10126E+01_JPRB, 0.98765E+00_JPRB, 0.96990E+00_JPRB, 0.94403E+00_JPRB ,&
&  0.92962E+00_JPRB, 0.92475E+00_JPRB, 0.93835E+00_JPRB, 0.96800E+00_JPRB, 0.98659E+00_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.98924E+00_JPRB, 0.97912E+00_JPRB, 0.92993E+00_JPRB, 0.88103E+00_JPRB, 0.84861E+00_JPRB, 0.83497E+00_JPRB, 0.83422E+00_JPRB ,&
&  0.10175E+01_JPRB, 0.89800E+00_JPRB, 0.81201E+00_JPRB, 0.77243E+00_JPRB, 0.75182E+00_JPRB, 0.73825E+00_JPRB, 0.72249E+00_JPRB ,&
&  0.70830E+00_JPRB, 0.68918E+00_JPRB, 0.67616E+00_JPRB, 0.67620E+00_JPRB, 0.71405E+00_JPRB, 0.88119E+00_JPRB, 0.86204E+00_JPRB ,&
&  0.12652E+01_JPRB, 0.20458E+01_JPRB, 0.18158E+01_JPRB, 0.17353E+01_JPRB, 0.16030E+01_JPRB, 0.15381E+01_JPRB, 0.12262E+01_JPRB ,&
&  0.11063E+01_JPRB, 0.10715E+01_JPRB, 0.10296E+01_JPRB, 0.10081E+01_JPRB, 0.10071E+01_JPRB, 0.99660E+00_JPRB, 0.92799E+00_JPRB ,&
&  0.88983E+00_JPRB, 0.88654E+00_JPRB, 0.90757E+00_JPRB, 0.92200E+00_JPRB, 0.94965E+00_JPRB, 0.10164E+01_JPRB, 0.10725E+01_JPRB ,&
&  0.11598E+01_JPRB, 0.13419E+01_JPRB, 0.14543E+01_JPRB, 0.13139E+01_JPRB, 0.12861E+01_JPRB, 0.13029E+01_JPRB, 0.13218E+01_JPRB ,&
&  0.12451E+01_JPRB, 0.11642E+01_JPRB, 0.12442E+01_JPRB, 0.13182E+01_JPRB, 0.13723E+01_JPRB, 0.14057E+01_JPRB, 0.14472E+01_JPRB ,&
&  0.16486E+01_JPRB, 0.17424E+01_JPRB, 0.19255E+01_JPRB, 0.19272E+01_JPRB, 0.21679E+01_JPRB, 0.27282E+01_JPRB, 0.29575E+01_JPRB ,&
&  0.30431E+01_JPRB, 0.24440E+01_JPRB, 0.22330E+01_JPRB, 0.22084E+01_JPRB, 0.21341E+01_JPRB, 0.20403E+01_JPRB, 0.22197E+01_JPRB ,&
&  0.24260E+01_JPRB, 0.27650E+01_JPRB, 0.24727E+01_JPRB, 0.19658E+01_JPRB, 0.15202E+01_JPRB, 0.11300E+01_JPRB, 0.12694E+01_JPRB ,&
&  0.15643E+01_JPRB, 0.16351E+01_JPRB, 0.16351E+01_JPRB, 0.20627E+01_JPRB, 0.16815E+01_JPRB, 0.14156E+01_JPRB, 0.11841E+01_JPRB ,&
&  0.10544E+01_JPRB, 0.10260E+01_JPRB, 0.10229E+01_JPRB, 0.10024E+01_JPRB, 0.98075E+00_JPRB, 0.96112E+00_JPRB, 0.94354E+00_JPRB ,&
&  0.93462E+00_JPRB, 0.93947E+00_JPRB, 0.95987E+00_JPRB, 0.98269E+00_JPRB, 0.99693E+00_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.95933E+00_JPRB, 0.93167E+00_JPRB, 0.89865E+00_JPRB, 0.86822E+00_JPRB, 0.84485E+00_JPRB, 0.84615E+00_JPRB, 0.88896E+00_JPRB ,&
&  0.11589E+01_JPRB, 0.89337E+00_JPRB, 0.79599E+00_JPRB, 0.76484E+00_JPRB, 0.74977E+00_JPRB, 0.72925E+00_JPRB, 0.72262E+00_JPRB ,&
&  0.72235E+00_JPRB, 0.72165E+00_JPRB, 0.73658E+00_JPRB, 0.77928E+00_JPRB, 0.90037E+00_JPRB, 0.12011E+01_JPRB, 0.13905E+01_JPRB ,&
&  0.14356E+01_JPRB, 0.23543E+01_JPRB, 0.18993E+01_JPRB, 0.17671E+01_JPRB, 0.15995E+01_JPRB, 0.15151E+01_JPRB, 0.13170E+01_JPRB ,&
&  0.11612E+01_JPRB, 0.10945E+01_JPRB, 0.10652E+01_JPRB, 0.10288E+01_JPRB, 0.10027E+01_JPRB, 0.95119E+00_JPRB, 0.92507E+00_JPRB ,&
&  0.92123E+00_JPRB, 0.93858E+00_JPRB, 0.95887E+00_JPRB, 0.98367E+00_JPRB, 0.10209E+01_JPRB, 0.10988E+01_JPRB, 0.11894E+01_JPRB ,&
&  0.13406E+01_JPRB, 0.15054E+01_JPRB, 0.14173E+01_JPRB, 0.13206E+01_JPRB, 0.12809E+01_JPRB, 0.13121E+01_JPRB, 0.13320E+01_JPRB ,&
&  0.12743E+01_JPRB, 0.12052E+01_JPRB, 0.12637E+01_JPRB, 0.12850E+01_JPRB, 0.12858E+01_JPRB, 0.13758E+01_JPRB, 0.13992E+01_JPRB ,&
&  0.16576E+01_JPRB, 0.18017E+01_JPRB, 0.24217E+01_JPRB, 0.23350E+01_JPRB, 0.21251E+01_JPRB, 0.25785E+01_JPRB, 0.25737E+01_JPRB ,&
&  0.22011E+01_JPRB, 0.21043E+01_JPRB, 0.21262E+01_JPRB, 0.22884E+01_JPRB, 0.22722E+01_JPRB, 0.21969E+01_JPRB, 0.20547E+01_JPRB ,&
&  0.18497E+01_JPRB, 0.19190E+01_JPRB, 0.16796E+01_JPRB, 0.14174E+01_JPRB, 0.13863E+01_JPRB, 0.12173E+01_JPRB, 0.13867E+01_JPRB ,&
&  0.16690E+01_JPRB, 0.17352E+01_JPRB, 0.18497E+01_JPRB, 0.18645E+01_JPRB, 0.13783E+01_JPRB, 0.11256E+01_JPRB, 0.10134E+01_JPRB ,&
&  0.10053E+01_JPRB, 0.10255E+01_JPRB, 0.10185E+01_JPRB, 0.99268E+00_JPRB, 0.98073E+00_JPRB, 0.97300E+00_JPRB, 0.96636E+00_JPRB ,&
&  0.96892E+00_JPRB, 0.97671E+00_JPRB, 0.98333E+00_JPRB, 0.98343E+00_JPRB, 0.98271E+00_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.95300E+00_JPRB, 0.94224E+00_JPRB, 0.92655E+00_JPRB, 0.92313E+00_JPRB, 0.94950E+00_JPRB, 0.97572E+00_JPRB, 0.10930E+01_JPRB ,&
&  0.13037E+01_JPRB, 0.87912E+00_JPRB, 0.77826E+00_JPRB, 0.75671E+00_JPRB, 0.74855E+00_JPRB, 0.74780E+00_JPRB, 0.76899E+00_JPRB ,&
&  0.78855E+00_JPRB, 0.80975E+00_JPRB, 0.85816E+00_JPRB, 0.99161E+00_JPRB, 0.12670E+01_JPRB, 0.17128E+01_JPRB, 0.19176E+01_JPRB ,&
&  0.18954E+01_JPRB, 0.19426E+01_JPRB, 0.20345E+01_JPRB, 0.17275E+01_JPRB, 0.15998E+01_JPRB, 0.15142E+01_JPRB, 0.14756E+01_JPRB ,&
&  0.12708E+01_JPRB, 0.11664E+01_JPRB, 0.11092E+01_JPRB, 0.10459E+01_JPRB, 0.98357E+00_JPRB, 0.96388E+00_JPRB, 0.97872E+00_JPRB ,&
&  0.10012E+01_JPRB, 0.10250E+01_JPRB, 0.10547E+01_JPRB, 0.10973E+01_JPRB, 0.11738E+01_JPRB, 0.12951E+01_JPRB, 0.14256E+01_JPRB ,&
&  0.15531E+01_JPRB, 0.15877E+01_JPRB, 0.14632E+01_JPRB, 0.13527E+01_JPRB, 0.13067E+01_JPRB, 0.13355E+01_JPRB, 0.13437E+01_JPRB ,&
&  0.13203E+01_JPRB, 0.13184E+01_JPRB, 0.13324E+01_JPRB, 0.13267E+01_JPRB, 0.12722E+01_JPRB, 0.14109E+01_JPRB, 0.14899E+01_JPRB ,&
&  0.17516E+01_JPRB, 0.20495E+01_JPRB, 0.27321E+01_JPRB, 0.22465E+01_JPRB, 0.23287E+01_JPRB, 0.24398E+01_JPRB, 0.23811E+01_JPRB ,&
&  0.19770E+01_JPRB, 0.20037E+01_JPRB, 0.21632E+01_JPRB, 0.22514E+01_JPRB, 0.23789E+01_JPRB, 0.21957E+01_JPRB, 0.21330E+01_JPRB ,&
&  0.16398E+01_JPRB, 0.15999E+01_JPRB, 0.12860E+01_JPRB, 0.13188E+01_JPRB, 0.13057E+01_JPRB, 0.12597E+01_JPRB, 0.13595E+01_JPRB ,&
&  0.16116E+01_JPRB, 0.16679E+01_JPRB, 0.15495E+01_JPRB, 0.13763E+01_JPRB, 0.10678E+01_JPRB, 0.98813E+00_JPRB, 0.97338E+00_JPRB ,&
&  0.98326E+00_JPRB, 0.10311E+01_JPRB, 0.10197E+01_JPRB, 0.10132E+01_JPRB, 0.10004E+01_JPRB, 0.99113E+00_JPRB, 0.99142E+00_JPRB ,&
&  0.99561E+00_JPRB, 0.99523E+00_JPRB, 0.99198E+00_JPRB, 0.98091E+00_JPRB, 0.96995E+00_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.93932E+00_JPRB, 0.92866E+00_JPRB, 0.91893E+00_JPRB, 0.89997E+00_JPRB, 0.87780E+00_JPRB, 0.89045E+00_JPRB, 0.89194E+00_JPRB ,&
&  0.85061E+00_JPRB, 0.79702E+00_JPRB, 0.79610E+00_JPRB, 0.79761E+00_JPRB, 0.80420E+00_JPRB, 0.81982E+00_JPRB, 0.85322E+00_JPRB ,&
&  0.88586E+00_JPRB, 0.94662E+00_JPRB, 0.10931E+01_JPRB, 0.13522E+01_JPRB, 0.16530E+01_JPRB, 0.19845E+01_JPRB, 0.23551E+01_JPRB ,&
&  0.22602E+01_JPRB, 0.15660E+01_JPRB, 0.18772E+01_JPRB, 0.18200E+01_JPRB, 0.16268E+01_JPRB, 0.14716E+01_JPRB, 0.13886E+01_JPRB ,&
&  0.13188E+01_JPRB, 0.12616E+01_JPRB, 0.12116E+01_JPRB, 0.11534E+01_JPRB, 0.10997E+01_JPRB, 0.11254E+01_JPRB, 0.11591E+01_JPRB ,&
&  0.11876E+01_JPRB, 0.12271E+01_JPRB, 0.12791E+01_JPRB, 0.13601E+01_JPRB, 0.14815E+01_JPRB, 0.16013E+01_JPRB, 0.16866E+01_JPRB ,&
&  0.17133E+01_JPRB, 0.17232E+01_JPRB, 0.16533E+01_JPRB, 0.15583E+01_JPRB, 0.14720E+01_JPRB, 0.14588E+01_JPRB, 0.14226E+01_JPRB ,&
&  0.13976E+01_JPRB, 0.13700E+01_JPRB, 0.14105E+01_JPRB, 0.14972E+01_JPRB, 0.15541E+01_JPRB, 0.14981E+01_JPRB, 0.17024E+01_JPRB ,&
&  0.19338E+01_JPRB, 0.22689E+01_JPRB, 0.24363E+01_JPRB, 0.29974E+01_JPRB, 0.20720E+01_JPRB, 0.21656E+01_JPRB, 0.18718E+01_JPRB ,&
&  0.19265E+01_JPRB, 0.19065E+01_JPRB, 0.20351E+01_JPRB, 0.21653E+01_JPRB, 0.23408E+01_JPRB, 0.21023E+01_JPRB, 0.18815E+01_JPRB ,&
&  0.15287E+01_JPRB, 0.12819E+01_JPRB, 0.11187E+01_JPRB, 0.11942E+01_JPRB, 0.12579E+01_JPRB, 0.13571E+01_JPRB, 0.13446E+01_JPRB ,&
&  0.14182E+01_JPRB, 0.13700E+01_JPRB, 0.12442E+01_JPRB, 0.12213E+01_JPRB, 0.10914E+01_JPRB, 0.10003E+01_JPRB, 0.10038E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.10438E+01_JPRB, 0.10252E+01_JPRB, 0.10032E+01_JPRB, 0.98527E+00_JPRB, 0.97382E+00_JPRB, 0.97714E+00_JPRB ,&
&  0.97260E+00_JPRB, 0.96937E+00_JPRB, 0.96644E+00_JPRB, 0.95631E+00_JPRB, 0.94565E+00_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.90077E+00_JPRB, 0.88281E+00_JPRB, 0.84711E+00_JPRB, 0.82017E+00_JPRB, 0.82231E+00_JPRB, 0.82505E+00_JPRB, 0.80733E+00_JPRB ,&
&  0.80866E+00_JPRB, 0.82081E+00_JPRB, 0.83719E+00_JPRB, 0.86079E+00_JPRB, 0.88512E+00_JPRB, 0.90636E+00_JPRB, 0.96202E+00_JPRB ,&
&  0.10322E+01_JPRB, 0.11661E+01_JPRB, 0.13762E+01_JPRB, 0.15915E+01_JPRB, 0.17886E+01_JPRB, 0.20098E+01_JPRB, 0.22196E+01_JPRB ,&
&  0.24699E+01_JPRB, 0.25235E+01_JPRB, 0.22975E+01_JPRB, 0.15879E+01_JPRB, 0.13790E+01_JPRB, 0.14868E+01_JPRB, 0.14611E+01_JPRB ,&
&  0.14323E+01_JPRB, 0.13812E+01_JPRB, 0.13426E+01_JPRB, 0.13303E+01_JPRB, 0.13497E+01_JPRB, 0.13943E+01_JPRB, 0.14289E+01_JPRB ,&
&  0.14828E+01_JPRB, 0.15427E+01_JPRB, 0.16107E+01_JPRB, 0.16856E+01_JPRB, 0.17461E+01_JPRB, 0.18065E+01_JPRB, 0.18689E+01_JPRB ,&
&  0.19128E+01_JPRB, 0.19867E+01_JPRB, 0.20273E+01_JPRB, 0.19780E+01_JPRB, 0.18448E+01_JPRB, 0.17794E+01_JPRB, 0.16930E+01_JPRB ,&
&  0.16558E+01_JPRB, 0.16255E+01_JPRB, 0.16417E+01_JPRB, 0.17492E+01_JPRB, 0.18272E+01_JPRB, 0.16982E+01_JPRB, 0.17878E+01_JPRB ,&
&  0.21406E+01_JPRB, 0.24811E+01_JPRB, 0.22398E+01_JPRB, 0.28036E+01_JPRB, 0.16682E+01_JPRB, 0.18763E+01_JPRB, 0.18800E+01_JPRB ,&
&  0.16403E+01_JPRB, 0.17029E+01_JPRB, 0.18441E+01_JPRB, 0.19631E+01_JPRB, 0.19866E+01_JPRB, 0.17803E+01_JPRB, 0.16168E+01_JPRB ,&
&  0.14020E+01_JPRB, 0.11481E+01_JPRB, 0.10141E+01_JPRB, 0.10519E+01_JPRB, 0.11668E+01_JPRB, 0.12712E+01_JPRB, 0.11169E+01_JPRB ,&
&  0.95270E+00_JPRB, 0.11834E+01_JPRB, 0.12419E+01_JPRB, 0.13878E+01_JPRB, 0.12702E+01_JPRB, 0.10070E+01_JPRB, 0.10301E+01_JPRB ,&
&  0.10526E+01_JPRB, 0.10357E+01_JPRB, 0.10224E+01_JPRB, 0.99310E+00_JPRB, 0.98111E+00_JPRB, 0.96835E+00_JPRB, 0.95668E+00_JPRB ,&
&  0.95158E+00_JPRB, 0.94938E+00_JPRB, 0.94519E+00_JPRB, 0.92756E+00_JPRB, 0.91054E+00_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.84397E+00_JPRB, 0.80864E+00_JPRB, 0.77991E+00_JPRB, 0.77136E+00_JPRB, 0.77646E+00_JPRB, 0.77913E+00_JPRB, 0.78646E+00_JPRB ,&
&  0.80700E+00_JPRB, 0.83199E+00_JPRB, 0.86050E+00_JPRB, 0.90063E+00_JPRB, 0.94150E+00_JPRB, 0.98566E+00_JPRB, 0.10550E+01_JPRB ,&
&  0.11405E+01_JPRB, 0.12610E+01_JPRB, 0.14244E+01_JPRB, 0.16014E+01_JPRB, 0.17602E+01_JPRB, 0.19374E+01_JPRB, 0.21144E+01_JPRB ,&
&  0.23104E+01_JPRB, 0.23862E+01_JPRB, 0.23190E+01_JPRB, 0.21691E+01_JPRB, 0.17226E+01_JPRB, 0.17695E+01_JPRB, 0.18857E+01_JPRB ,&
&  0.19231E+01_JPRB, 0.16532E+01_JPRB, 0.15686E+01_JPRB, 0.15526E+01_JPRB, 0.15765E+01_JPRB, 0.16330E+01_JPRB, 0.16778E+01_JPRB ,&
&  0.17289E+01_JPRB, 0.17750E+01_JPRB, 0.18188E+01_JPRB, 0.18547E+01_JPRB, 0.18919E+01_JPRB, 0.19627E+01_JPRB, 0.20930E+01_JPRB ,&
&  0.21447E+01_JPRB, 0.22008E+01_JPRB, 0.22530E+01_JPRB, 0.22521E+01_JPRB, 0.23495E+01_JPRB, 0.24131E+01_JPRB, 0.23326E+01_JPRB ,&
&  0.22493E+01_JPRB, 0.20888E+01_JPRB, 0.21401E+01_JPRB, 0.23029E+01_JPRB, 0.23214E+01_JPRB, 0.21858E+01_JPRB, 0.22034E+01_JPRB ,&
&  0.24923E+01_JPRB, 0.27914E+01_JPRB, 0.19543E+01_JPRB, 0.21511E+01_JPRB, 0.24279E+01_JPRB, 0.18970E+01_JPRB, 0.15082E+01_JPRB ,&
&  0.13102E+01_JPRB, 0.13518E+01_JPRB, 0.15364E+01_JPRB, 0.16098E+01_JPRB, 0.16074E+01_JPRB, 0.18055E+01_JPRB, 0.14154E+01_JPRB ,&
&  0.12459E+01_JPRB, 0.10230E+01_JPRB, 0.92125E+00_JPRB, 0.93110E+00_JPRB, 0.10024E+01_JPRB, 0.10314E+01_JPRB, 0.88098E+00_JPRB ,&
&  0.85409E+00_JPRB, 0.11703E+01_JPRB, 0.13230E+01_JPRB, 0.15882E+01_JPRB, 0.15867E+01_JPRB, 0.99099E+00_JPRB, 0.10031E+01_JPRB ,&
&  0.10091E+01_JPRB, 0.10032E+01_JPRB, 0.98483E+00_JPRB, 0.96583E+00_JPRB, 0.95019E+00_JPRB, 0.93605E+00_JPRB, 0.92725E+00_JPRB ,&
&  0.92087E+00_JPRB, 0.91479E+00_JPRB, 0.90916E+00_JPRB, 0.89287E+00_JPRB, 0.87032E+00_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.75928E+00_JPRB, 0.73578E+00_JPRB, 0.72404E+00_JPRB, 0.73296E+00_JPRB, 0.75226E+00_JPRB, 0.76580E+00_JPRB, 0.77052E+00_JPRB ,&
&  0.78653E+00_JPRB, 0.80919E+00_JPRB, 0.84481E+00_JPRB, 0.87664E+00_JPRB, 0.91699E+00_JPRB, 0.95534E+00_JPRB, 0.10182E+01_JPRB ,&
&  0.10812E+01_JPRB, 0.11553E+01_JPRB, 0.12826E+01_JPRB, 0.14725E+01_JPRB, 0.16298E+01_JPRB, 0.17723E+01_JPRB, 0.19835E+01_JPRB ,&
&  0.21893E+01_JPRB, 0.23072E+01_JPRB, 0.24367E+01_JPRB, 0.24850E+01_JPRB, 0.23645E+01_JPRB, 0.22094E+01_JPRB, 0.22384E+01_JPRB ,&
&  0.18697E+01_JPRB, 0.15711E+01_JPRB, 0.16149E+01_JPRB, 0.16784E+01_JPRB, 0.17514E+01_JPRB, 0.17788E+01_JPRB, 0.18103E+01_JPRB ,&
&  0.18501E+01_JPRB, 0.18777E+01_JPRB, 0.19036E+01_JPRB, 0.19371E+01_JPRB, 0.19974E+01_JPRB, 0.21035E+01_JPRB, 0.22520E+01_JPRB ,&
&  0.23058E+01_JPRB, 0.23815E+01_JPRB, 0.25057E+01_JPRB, 0.23290E+01_JPRB, 0.25381E+01_JPRB, 0.28241E+01_JPRB, 0.29880E+01_JPRB ,&
&  0.29547E+01_JPRB, 0.27471E+01_JPRB, 0.27846E+01_JPRB, 0.28591E+01_JPRB, 0.28834E+01_JPRB, 0.28189E+01_JPRB, 0.27743E+01_JPRB ,&
&  0.29205E+01_JPRB, 0.27745E+01_JPRB, 0.19643E+01_JPRB, 0.17286E+01_JPRB, 0.17278E+01_JPRB, 0.13346E+01_JPRB, 0.12694E+01_JPRB ,&
&  0.10948E+01_JPRB, 0.11136E+01_JPRB, 0.12731E+01_JPRB, 0.13951E+01_JPRB, 0.14882E+01_JPRB, 0.17095E+01_JPRB, 0.13871E+01_JPRB ,&
&  0.10738E+01_JPRB, 0.87694E+00_JPRB, 0.77258E+00_JPRB, 0.75510E+00_JPRB, 0.78663E+00_JPRB, 0.81935E+00_JPRB, 0.82140E+00_JPRB ,&
&  0.87182E+00_JPRB, 0.91169E+00_JPRB, 0.94510E+00_JPRB, 0.98437E+00_JPRB, 0.93103E+00_JPRB, 0.95247E+00_JPRB, 0.94666E+00_JPRB ,&
&  0.92749E+00_JPRB, 0.93012E+00_JPRB, 0.92281E+00_JPRB, 0.90288E+00_JPRB, 0.89342E+00_JPRB, 0.88130E+00_JPRB, 0.87684E+00_JPRB ,&
&  0.87465E+00_JPRB, 0.86830E+00_JPRB, 0.85381E+00_JPRB, 0.83489E+00_JPRB, 0.79907E+00_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.69133E+00_JPRB, 0.67171E+00_JPRB, 0.67833E+00_JPRB, 0.69736E+00_JPRB, 0.70809E+00_JPRB, 0.71866E+00_JPRB, 0.72976E+00_JPRB ,&
&  0.73316E+00_JPRB, 0.73453E+00_JPRB, 0.73969E+00_JPRB, 0.74137E+00_JPRB, 0.77459E+00_JPRB, 0.81259E+00_JPRB, 0.86187E+00_JPRB ,&
&  0.93580E+00_JPRB, 0.10024E+01_JPRB, 0.10635E+01_JPRB, 0.11745E+01_JPRB, 0.12547E+01_JPRB, 0.13315E+01_JPRB, 0.14569E+01_JPRB ,&
&  0.16156E+01_JPRB, 0.17899E+01_JPRB, 0.19553E+01_JPRB, 0.21191E+01_JPRB, 0.23257E+01_JPRB, 0.25229E+01_JPRB, 0.25424E+01_JPRB ,&
&  0.19465E+01_JPRB, 0.15227E+01_JPRB, 0.16981E+01_JPRB, 0.16304E+01_JPRB, 0.17142E+01_JPRB, 0.17767E+01_JPRB, 0.18035E+01_JPRB ,&
&  0.18480E+01_JPRB, 0.18800E+01_JPRB, 0.19090E+01_JPRB, 0.19434E+01_JPRB, 0.20202E+01_JPRB, 0.21194E+01_JPRB, 0.22533E+01_JPRB ,&
&  0.23360E+01_JPRB, 0.24098E+01_JPRB, 0.24451E+01_JPRB, 0.24588E+01_JPRB, 0.26507E+01_JPRB, 0.29529E+01_JPRB, 0.31809E+01_JPRB ,&
&  0.33073E+01_JPRB, 0.34097E+01_JPRB, 0.30888E+01_JPRB, 0.30892E+01_JPRB, 0.30376E+01_JPRB, 0.27337E+01_JPRB, 0.27629E+01_JPRB ,&
&  0.29334E+01_JPRB, 0.27073E+01_JPRB, 0.17139E+01_JPRB, 0.15647E+01_JPRB, 0.14648E+01_JPRB, 0.12918E+01_JPRB, 0.11548E+01_JPRB ,&
&  0.94687E+00_JPRB, 0.93293E+00_JPRB, 0.10215E+01_JPRB, 0.12221E+01_JPRB, 0.13525E+01_JPRB, 0.14261E+01_JPRB, 0.12649E+01_JPRB ,&
&  0.96854E+00_JPRB, 0.79992E+00_JPRB, 0.70844E+00_JPRB, 0.71853E+00_JPRB, 0.74253E+00_JPRB, 0.80810E+00_JPRB, 0.76110E+00_JPRB ,&
&  0.81572E+00_JPRB, 0.89155E+00_JPRB, 0.10691E+01_JPRB, 0.97977E+00_JPRB, 0.82786E+00_JPRB, 0.89188E+00_JPRB, 0.87616E+00_JPRB ,&
&  0.86416E+00_JPRB, 0.86317E+00_JPRB, 0.85437E+00_JPRB, 0.84526E+00_JPRB, 0.83810E+00_JPRB, 0.84075E+00_JPRB, 0.82776E+00_JPRB ,&
&  0.81747E+00_JPRB, 0.80015E+00_JPRB, 0.78853E+00_JPRB, 0.74885E+00_JPRB, 0.71818E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.63261E+00_JPRB, 0.63578E+00_JPRB, 0.64415E+00_JPRB, 0.66082E+00_JPRB, 0.65810E+00_JPRB, 0.65585E+00_JPRB, 0.68167E+00_JPRB ,&
&  0.66487E+00_JPRB, 0.65199E+00_JPRB, 0.64980E+00_JPRB, 0.62422E+00_JPRB, 0.64243E+00_JPRB, 0.66020E+00_JPRB, 0.69802E+00_JPRB ,&
&  0.73752E+00_JPRB, 0.76239E+00_JPRB, 0.78471E+00_JPRB, 0.81794E+00_JPRB, 0.87971E+00_JPRB, 0.92721E+00_JPRB, 0.96775E+00_JPRB ,&
&  0.10384E+01_JPRB, 0.11480E+01_JPRB, 0.12691E+01_JPRB, 0.15477E+01_JPRB, 0.21904E+01_JPRB, 0.27030E+01_JPRB, 0.26938E+01_JPRB ,&
&  0.20334E+01_JPRB, 0.15177E+01_JPRB, 0.14601E+01_JPRB, 0.13463E+01_JPRB, 0.14231E+01_JPRB, 0.15831E+01_JPRB, 0.16429E+01_JPRB ,&
&  0.16394E+01_JPRB, 0.16642E+01_JPRB, 0.16780E+01_JPRB, 0.16988E+01_JPRB, 0.17292E+01_JPRB, 0.17510E+01_JPRB, 0.17395E+01_JPRB ,&
&  0.16795E+01_JPRB, 0.16072E+01_JPRB, 0.16678E+01_JPRB, 0.20173E+01_JPRB, 0.23431E+01_JPRB, 0.28304E+01_JPRB, 0.27894E+01_JPRB ,&
&  0.26715E+01_JPRB, 0.26041E+01_JPRB, 0.25764E+01_JPRB, 0.25223E+01_JPRB, 0.24242E+01_JPRB, 0.23121E+01_JPRB, 0.23485E+01_JPRB ,&
&  0.24811E+01_JPRB, 0.23060E+01_JPRB, 0.15283E+01_JPRB, 0.15207E+01_JPRB, 0.13957E+01_JPRB, 0.13387E+01_JPRB, 0.10728E+01_JPRB ,&
&  0.86643E+00_JPRB, 0.81708E+00_JPRB, 0.83599E+00_JPRB, 0.91221E+00_JPRB, 0.10674E+01_JPRB, 0.11605E+01_JPRB, 0.10614E+01_JPRB ,&
&  0.90407E+00_JPRB, 0.77333E+00_JPRB, 0.69448E+00_JPRB, 0.65843E+00_JPRB, 0.73284E+00_JPRB, 0.75175E+00_JPRB, 0.88332E+00_JPRB ,&
&  0.70730E+00_JPRB, 0.77886E+00_JPRB, 0.72227E+00_JPRB, 0.92816E+00_JPRB, 0.78213E+00_JPRB, 0.75335E+00_JPRB, 0.74318E+00_JPRB ,&
&  0.74030E+00_JPRB, 0.72444E+00_JPRB, 0.70881E+00_JPRB, 0.68794E+00_JPRB, 0.66709E+00_JPRB, 0.69646E+00_JPRB, 0.73255E+00_JPRB ,&
&  0.69991E+00_JPRB, 0.71675E+00_JPRB, 0.69578E+00_JPRB, 0.66183E+00_JPRB, 0.65439E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.61502E+00_JPRB, 0.62730E+00_JPRB, 0.62862E+00_JPRB, 0.63543E+00_JPRB, 0.64943E+00_JPRB, 0.65910E+00_JPRB, 0.67322E+00_JPRB ,&
&  0.68850E+00_JPRB, 0.69551E+00_JPRB, 0.70258E+00_JPRB, 0.71193E+00_JPRB, 0.71807E+00_JPRB, 0.73850E+00_JPRB, 0.75886E+00_JPRB ,&
&  0.78553E+00_JPRB, 0.80652E+00_JPRB, 0.82867E+00_JPRB, 0.86269E+00_JPRB, 0.90579E+00_JPRB, 0.97357E+00_JPRB, 0.10452E+01_JPRB ,&
&  0.11382E+01_JPRB, 0.12472E+01_JPRB, 0.13763E+01_JPRB, 0.14810E+01_JPRB, 0.15353E+01_JPRB, 0.15637E+01_JPRB, 0.17167E+01_JPRB ,&
&  0.15077E+01_JPRB, 0.16032E+01_JPRB, 0.15307E+01_JPRB, 0.13743E+01_JPRB, 0.13641E+01_JPRB, 0.12696E+01_JPRB, 0.12475E+01_JPRB ,&
&  0.12631E+01_JPRB, 0.12161E+01_JPRB, 0.11930E+01_JPRB, 0.11732E+01_JPRB, 0.11502E+01_JPRB, 0.10947E+01_JPRB, 0.10274E+01_JPRB ,&
&  0.10197E+01_JPRB, 0.10920E+01_JPRB, 0.12454E+01_JPRB, 0.15586E+01_JPRB, 0.18674E+01_JPRB, 0.19823E+01_JPRB, 0.18516E+01_JPRB ,&
&  0.17719E+01_JPRB, 0.17630E+01_JPRB, 0.16792E+01_JPRB, 0.15984E+01_JPRB, 0.16682E+01_JPRB, 0.17246E+01_JPRB, 0.17839E+01_JPRB ,&
&  0.17159E+01_JPRB, 0.15311E+01_JPRB, 0.14372E+01_JPRB, 0.13358E+01_JPRB, 0.12694E+01_JPRB, 0.11941E+01_JPRB, 0.10387E+01_JPRB ,&
&  0.87228E+00_JPRB, 0.77626E+00_JPRB, 0.73363E+00_JPRB, 0.75490E+00_JPRB, 0.85576E+00_JPRB, 0.87607E+00_JPRB, 0.81508E+00_JPRB ,&
&  0.71004E+00_JPRB, 0.65164E+00_JPRB, 0.61956E+00_JPRB, 0.60516E+00_JPRB, 0.61874E+00_JPRB, 0.10630E+01_JPRB, 0.14394E+01_JPRB ,&
&  0.66373E+00_JPRB, 0.57405E+00_JPRB, 0.48576E+00_JPRB, 0.67094E+00_JPRB, 0.63431E+00_JPRB, 0.63918E+00_JPRB, 0.63923E+00_JPRB ,&
&  0.63084E+00_JPRB, 0.62987E+00_JPRB, 0.62747E+00_JPRB, 0.61160E+00_JPRB, 0.60014E+00_JPRB, 0.59812E+00_JPRB, 0.62402E+00_JPRB ,&
&  0.62049E+00_JPRB, 0.62103E+00_JPRB, 0.61456E+00_JPRB, 0.62413E+00_JPRB, 0.61967E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.61297E+00_JPRB, 0.62694E+00_JPRB, 0.64375E+00_JPRB, 0.66463E+00_JPRB, 0.67841E+00_JPRB, 0.69756E+00_JPRB, 0.71314E+00_JPRB ,&
&  0.73479E+00_JPRB, 0.75219E+00_JPRB, 0.76694E+00_JPRB, 0.78230E+00_JPRB, 0.80193E+00_JPRB, 0.82613E+00_JPRB, 0.85706E+00_JPRB ,&
&  0.89069E+00_JPRB, 0.92434E+00_JPRB, 0.96122E+00_JPRB, 0.10035E+01_JPRB, 0.10532E+01_JPRB, 0.11142E+01_JPRB, 0.11720E+01_JPRB ,&
&  0.12257E+01_JPRB, 0.12794E+01_JPRB, 0.13329E+01_JPRB, 0.13745E+01_JPRB, 0.13886E+01_JPRB, 0.14077E+01_JPRB, 0.12525E+01_JPRB ,&
&  0.15321E+01_JPRB, 0.15100E+01_JPRB, 0.14134E+01_JPRB, 0.12911E+01_JPRB, 0.11788E+01_JPRB, 0.10779E+01_JPRB, 0.99255E+00_JPRB ,&
&  0.10058E+01_JPRB, 0.96777E+00_JPRB, 0.95614E+00_JPRB, 0.95726E+00_JPRB, 0.90018E+00_JPRB, 0.90537E+00_JPRB, 0.96025E+00_JPRB ,&
&  0.10562E+01_JPRB, 0.11947E+01_JPRB, 0.12734E+01_JPRB, 0.13674E+01_JPRB, 0.14933E+01_JPRB, 0.14385E+01_JPRB, 0.13755E+01_JPRB ,&
&  0.12735E+01_JPRB, 0.12036E+01_JPRB, 0.97005E+00_JPRB, 0.10927E+01_JPRB, 0.11416E+01_JPRB, 0.11937E+01_JPRB, 0.12724E+01_JPRB ,&
&  0.11877E+01_JPRB, 0.98909E+00_JPRB, 0.10299E+01_JPRB, 0.12309E+01_JPRB, 0.11665E+01_JPRB, 0.10633E+01_JPRB, 0.97266E+00_JPRB ,&
&  0.87625E+00_JPRB, 0.80135E+00_JPRB, 0.73698E+00_JPRB, 0.71414E+00_JPRB, 0.75098E+00_JPRB, 0.74606E+00_JPRB, 0.68569E+00_JPRB ,&
&  0.61144E+00_JPRB, 0.58340E+00_JPRB, 0.57496E+00_JPRB, 0.57364E+00_JPRB, 0.63144E+00_JPRB, 0.68233E+00_JPRB, 0.10038E+01_JPRB ,&
&  0.66039E+00_JPRB, 0.42214E+00_JPRB, 0.49500E+00_JPRB, 0.58515E+00_JPRB, 0.56402E+00_JPRB, 0.59559E+00_JPRB, 0.62889E+00_JPRB ,&
&  0.66106E+00_JPRB, 0.67308E+00_JPRB, 0.63094E+00_JPRB, 0.62907E+00_JPRB, 0.62003E+00_JPRB, 0.61068E+00_JPRB, 0.60112E+00_JPRB ,&
&  0.59127E+00_JPRB, 0.59269E+00_JPRB, 0.60604E+00_JPRB, 0.60503E+00_JPRB, 0.60631E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.54054E+00_JPRB, 0.54999E+00_JPRB, 0.56159E+00_JPRB, 0.57159E+00_JPRB, 0.57774E+00_JPRB, 0.58915E+00_JPRB, 0.59751E+00_JPRB ,&
&  0.60754E+00_JPRB, 0.61166E+00_JPRB, 0.61960E+00_JPRB, 0.62631E+00_JPRB, 0.63553E+00_JPRB, 0.64936E+00_JPRB, 0.66318E+00_JPRB ,&
&  0.67333E+00_JPRB, 0.68938E+00_JPRB, 0.70857E+00_JPRB, 0.72114E+00_JPRB, 0.75449E+00_JPRB, 0.81778E+00_JPRB, 0.88928E+00_JPRB ,&
&  0.94600E+00_JPRB, 0.10186E+01_JPRB, 0.10858E+01_JPRB, 0.11128E+01_JPRB, 0.11083E+01_JPRB, 0.11276E+01_JPRB, 0.98034E+00_JPRB ,&
&  0.13702E+01_JPRB, 0.13658E+01_JPRB, 0.12432E+01_JPRB, 0.11704E+01_JPRB, 0.10992E+01_JPRB, 0.10292E+01_JPRB, 0.98344E+00_JPRB ,&
&  0.99235E+00_JPRB, 0.11648E+01_JPRB, 0.11944E+01_JPRB, 0.12440E+01_JPRB, 0.11687E+01_JPRB, 0.11735E+01_JPRB, 0.13026E+01_JPRB ,&
&  0.14545E+01_JPRB, 0.14992E+01_JPRB, 0.14917E+01_JPRB, 0.15595E+01_JPRB, 0.16430E+01_JPRB, 0.16850E+01_JPRB, 0.16980E+01_JPRB ,&
&  0.16341E+01_JPRB, 0.15715E+01_JPRB, 0.14637E+01_JPRB, 0.17132E+01_JPRB, 0.15374E+01_JPRB, 0.13578E+01_JPRB, 0.12423E+01_JPRB ,&
&  0.96970E+00_JPRB, 0.91249E+00_JPRB, 0.97292E+00_JPRB, 0.10384E+01_JPRB, 0.98981E+00_JPRB, 0.97200E+00_JPRB, 0.92068E+00_JPRB ,&
&  0.86302E+00_JPRB, 0.82622E+00_JPRB, 0.78037E+00_JPRB, 0.76389E+00_JPRB, 0.76399E+00_JPRB, 0.72759E+00_JPRB, 0.65399E+00_JPRB ,&
&  0.59796E+00_JPRB, 0.58643E+00_JPRB, 0.57136E+00_JPRB, 0.56672E+00_JPRB, 0.58908E+00_JPRB, 0.56210E+00_JPRB, 0.76333E+00_JPRB ,&
&  0.62828E+00_JPRB, 0.53970E+00_JPRB, 0.69079E+00_JPRB, 0.41683E+00_JPRB, 0.52387E+00_JPRB, 0.63873E+00_JPRB, 0.82807E+00_JPRB ,&
&  0.88128E+00_JPRB, 0.11395E+01_JPRB, 0.14869E+01_JPRB, 0.11782E+01_JPRB, 0.89438E+00_JPRB, 0.60177E+00_JPRB, 0.56734E+00_JPRB ,&
&  0.55746E+00_JPRB, 0.54775E+00_JPRB, 0.55062E+00_JPRB, 0.54347E+00_JPRB, 0.54183E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.47105E+00_JPRB, 0.47350E+00_JPRB, 0.48173E+00_JPRB, 0.48715E+00_JPRB, 0.49268E+00_JPRB, 0.49660E+00_JPRB, 0.50064E+00_JPRB ,&
&  0.51441E+00_JPRB, 0.52246E+00_JPRB, 0.54317E+00_JPRB, 0.55884E+00_JPRB, 0.57275E+00_JPRB, 0.60336E+00_JPRB, 0.62787E+00_JPRB ,&
&  0.66748E+00_JPRB, 0.71884E+00_JPRB, 0.73827E+00_JPRB, 0.76786E+00_JPRB, 0.78494E+00_JPRB, 0.80305E+00_JPRB, 0.81574E+00_JPRB ,&
&  0.88092E+00_JPRB, 0.10083E+01_JPRB, 0.10961E+01_JPRB, 0.11868E+01_JPRB, 0.11888E+01_JPRB, 0.12356E+01_JPRB, 0.99804E+00_JPRB ,&
&  0.13310E+01_JPRB, 0.13574E+01_JPRB, 0.12864E+01_JPRB, 0.12599E+01_JPRB, 0.12503E+01_JPRB, 0.12483E+01_JPRB, 0.12427E+01_JPRB ,&
&  0.12994E+01_JPRB, 0.14009E+01_JPRB, 0.14158E+01_JPRB, 0.15170E+01_JPRB, 0.16135E+01_JPRB, 0.16385E+01_JPRB, 0.18165E+01_JPRB ,&
&  0.19675E+01_JPRB, 0.20884E+01_JPRB, 0.22447E+01_JPRB, 0.24406E+01_JPRB, 0.25816E+01_JPRB, 0.26705E+01_JPRB, 0.27036E+01_JPRB ,&
&  0.27111E+01_JPRB, 0.27905E+01_JPRB, 0.30186E+01_JPRB, 0.29979E+01_JPRB, 0.23987E+01_JPRB, 0.19742E+01_JPRB, 0.15317E+01_JPRB ,&
&  0.10506E+01_JPRB, 0.86175E+00_JPRB, 0.85701E+00_JPRB, 0.90550E+00_JPRB, 0.87249E+00_JPRB, 0.85025E+00_JPRB, 0.82459E+00_JPRB ,&
&  0.80361E+00_JPRB, 0.81359E+00_JPRB, 0.81654E+00_JPRB, 0.81111E+00_JPRB, 0.78267E+00_JPRB, 0.71433E+00_JPRB, 0.63723E+00_JPRB ,&
&  0.61274E+00_JPRB, 0.59641E+00_JPRB, 0.59246E+00_JPRB, 0.60641E+00_JPRB, 0.63163E+00_JPRB, 0.69774E+00_JPRB, 0.67526E+00_JPRB ,&
&  0.73474E+00_JPRB, 0.56447E+00_JPRB, 0.56247E+00_JPRB, 0.59500E+00_JPRB, 0.60952E+00_JPRB, 0.84268E+00_JPRB, 0.12391E+01_JPRB ,&
&  0.18333E+01_JPRB, 0.20995E+01_JPRB, 0.27840E+01_JPRB, 0.40873E+01_JPRB, 0.25741E+01_JPRB, 0.16445E+01_JPRB, 0.53902E+00_JPRB ,&
&  0.52849E+00_JPRB, 0.50389E+00_JPRB, 0.49673E+00_JPRB, 0.48161E+00_JPRB, 0.47658E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.38843E+00_JPRB, 0.38240E+00_JPRB, 0.39245E+00_JPRB, 0.40950E+00_JPRB, 0.42234E+00_JPRB, 0.45132E+00_JPRB, 0.48623E+00_JPRB ,&
&  0.52449E+00_JPRB, 0.57168E+00_JPRB, 0.61044E+00_JPRB, 0.63665E+00_JPRB, 0.68219E+00_JPRB, 0.74664E+00_JPRB, 0.77249E+00_JPRB ,&
&  0.82577E+00_JPRB, 0.85746E+00_JPRB, 0.87027E+00_JPRB, 0.87801E+00_JPRB, 0.90346E+00_JPRB, 0.94790E+00_JPRB, 0.10005E+01_JPRB ,&
&  0.10486E+01_JPRB, 0.11093E+01_JPRB, 0.11935E+01_JPRB, 0.12490E+01_JPRB, 0.12753E+01_JPRB, 0.12818E+01_JPRB, 0.10221E+01_JPRB ,&
&  0.13276E+01_JPRB, 0.14102E+01_JPRB, 0.14576E+01_JPRB, 0.14524E+01_JPRB, 0.14431E+01_JPRB, 0.14448E+01_JPRB, 0.14904E+01_JPRB ,&
&  0.15716E+01_JPRB, 0.16084E+01_JPRB, 0.16303E+01_JPRB, 0.16603E+01_JPRB, 0.21080E+01_JPRB, 0.21972E+01_JPRB, 0.23958E+01_JPRB ,&
&  0.25775E+01_JPRB, 0.27392E+01_JPRB, 0.29582E+01_JPRB, 0.31910E+01_JPRB, 0.34329E+01_JPRB, 0.36411E+01_JPRB, 0.38280E+01_JPRB ,&
&  0.39988E+01_JPRB, 0.41854E+01_JPRB, 0.43678E+01_JPRB, 0.37620E+01_JPRB, 0.29913E+01_JPRB, 0.24498E+01_JPRB, 0.15507E+01_JPRB ,&
&  0.99782E+00_JPRB, 0.86898E+00_JPRB, 0.88651E+00_JPRB, 0.88138E+00_JPRB, 0.80378E+00_JPRB, 0.76246E+00_JPRB, 0.72932E+00_JPRB ,&
&  0.73742E+00_JPRB, 0.76706E+00_JPRB, 0.79116E+00_JPRB, 0.79366E+00_JPRB, 0.75576E+00_JPRB, 0.69357E+00_JPRB, 0.63510E+00_JPRB ,&
&  0.64377E+00_JPRB, 0.63844E+00_JPRB, 0.63486E+00_JPRB, 0.67204E+00_JPRB, 0.70741E+00_JPRB, 0.82797E+00_JPRB, 0.99286E+00_JPRB ,&
&  0.96533E+00_JPRB, 0.59714E+00_JPRB, 0.58772E+00_JPRB, 0.59800E+00_JPRB, 0.62175E+00_JPRB, 0.69494E+00_JPRB, 0.85116E+00_JPRB ,&
&  0.12004E+01_JPRB, 0.19638E+01_JPRB, 0.22323E+01_JPRB, 0.14760E+01_JPRB, 0.19988E+01_JPRB, 0.13001E+01_JPRB, 0.52394E+00_JPRB ,&
&  0.49626E+00_JPRB, 0.48139E+00_JPRB, 0.45400E+00_JPRB, 0.42396E+00_JPRB, 0.40336E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.39363E+00_JPRB, 0.37612E+00_JPRB, 0.38627E+00_JPRB, 0.41033E+00_JPRB, 0.43905E+00_JPRB, 0.48250E+00_JPRB, 0.53511E+00_JPRB ,&
&  0.58091E+00_JPRB, 0.62515E+00_JPRB, 0.70098E+00_JPRB, 0.71750E+00_JPRB, 0.73117E+00_JPRB, 0.76983E+00_JPRB, 0.79440E+00_JPRB ,&
&  0.82229E+00_JPRB, 0.84238E+00_JPRB, 0.86669E+00_JPRB, 0.89080E+00_JPRB, 0.93072E+00_JPRB, 0.97610E+00_JPRB, 0.10136E+01_JPRB ,&
&  0.10633E+01_JPRB, 0.11188E+01_JPRB, 0.12027E+01_JPRB, 0.12918E+01_JPRB, 0.13751E+01_JPRB, 0.13936E+01_JPRB, 0.14238E+01_JPRB ,&
&  0.10213E+01_JPRB, 0.14010E+01_JPRB, 0.14555E+01_JPRB, 0.14870E+01_JPRB, 0.14862E+01_JPRB, 0.14839E+01_JPRB, 0.15416E+01_JPRB ,&
&  0.16278E+01_JPRB, 0.16208E+01_JPRB, 0.17318E+01_JPRB, 0.19234E+01_JPRB, 0.23989E+01_JPRB, 0.25178E+01_JPRB, 0.26694E+01_JPRB ,&
&  0.28379E+01_JPRB, 0.30002E+01_JPRB, 0.31889E+01_JPRB, 0.34170E+01_JPRB, 0.36504E+01_JPRB, 0.38714E+01_JPRB, 0.40976E+01_JPRB ,&
&  0.42985E+01_JPRB, 0.44835E+01_JPRB, 0.46907E+01_JPRB, 0.39040E+01_JPRB, 0.32385E+01_JPRB, 0.26022E+01_JPRB, 0.17046E+01_JPRB ,&
&  0.10958E+01_JPRB, 0.85606E+00_JPRB, 0.87721E+00_JPRB, 0.91122E+00_JPRB, 0.81165E+00_JPRB, 0.75213E+00_JPRB, 0.67908E+00_JPRB ,&
&  0.67215E+00_JPRB, 0.68878E+00_JPRB, 0.70545E+00_JPRB, 0.70501E+00_JPRB, 0.70709E+00_JPRB, 0.65855E+00_JPRB, 0.62894E+00_JPRB ,&
&  0.64132E+00_JPRB, 0.63348E+00_JPRB, 0.63382E+00_JPRB, 0.65144E+00_JPRB, 0.68878E+00_JPRB, 0.72811E+00_JPRB, 0.73136E+00_JPRB ,&
&  0.73439E+00_JPRB, 0.66236E+00_JPRB, 0.63690E+00_JPRB, 0.65338E+00_JPRB, 0.65255E+00_JPRB, 0.65002E+00_JPRB, 0.67233E+00_JPRB ,&
&  0.75726E+00_JPRB, 0.86165E+00_JPRB, 0.95145E+00_JPRB, 0.94104E+00_JPRB, 0.98656E+00_JPRB, 0.77680E+00_JPRB, 0.57892E+00_JPRB ,&
&  0.54323E+00_JPRB, 0.53210E+00_JPRB, 0.48858E+00_JPRB, 0.45825E+00_JPRB, 0.42369E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.48708E+00_JPRB, 0.43111E+00_JPRB, 0.42625E+00_JPRB, 0.46795E+00_JPRB, 0.51073E+00_JPRB, 0.55468E+00_JPRB, 0.59477E+00_JPRB ,&
&  0.63580E+00_JPRB, 0.65821E+00_JPRB, 0.70267E+00_JPRB, 0.72308E+00_JPRB, 0.73806E+00_JPRB, 0.74099E+00_JPRB, 0.74528E+00_JPRB ,&
&  0.76950E+00_JPRB, 0.79675E+00_JPRB, 0.81990E+00_JPRB, 0.85896E+00_JPRB, 0.89990E+00_JPRB, 0.93923E+00_JPRB, 0.98655E+00_JPRB ,&
&  0.10327E+01_JPRB, 0.10864E+01_JPRB, 0.11421E+01_JPRB, 0.12110E+01_JPRB, 0.12630E+01_JPRB, 0.12815E+01_JPRB, 0.12269E+01_JPRB ,&
&  0.66742E+00_JPRB, 0.10771E+01_JPRB, 0.13978E+01_JPRB, 0.14216E+01_JPRB, 0.14354E+01_JPRB, 0.14355E+01_JPRB, 0.15092E+01_JPRB ,&
&  0.15068E+01_JPRB, 0.14625E+01_JPRB, 0.16005E+01_JPRB, 0.21945E+01_JPRB, 0.24565E+01_JPRB, 0.26001E+01_JPRB, 0.26921E+01_JPRB ,&
&  0.28085E+01_JPRB, 0.28995E+01_JPRB, 0.30112E+01_JPRB, 0.31315E+01_JPRB, 0.32722E+01_JPRB, 0.34080E+01_JPRB, 0.35326E+01_JPRB ,&
&  0.36372E+01_JPRB, 0.36968E+01_JPRB, 0.36852E+01_JPRB, 0.29104E+01_JPRB, 0.28499E+01_JPRB, 0.24053E+01_JPRB, 0.20477E+01_JPRB ,&
&  0.12757E+01_JPRB, 0.90632E+00_JPRB, 0.89002E+00_JPRB, 0.91773E+00_JPRB, 0.81609E+00_JPRB, 0.75374E+00_JPRB, 0.67174E+00_JPRB ,&
&  0.66741E+00_JPRB, 0.66810E+00_JPRB, 0.67702E+00_JPRB, 0.66966E+00_JPRB, 0.66977E+00_JPRB, 0.62224E+00_JPRB, 0.63884E+00_JPRB ,&
&  0.65392E+00_JPRB, 0.64619E+00_JPRB, 0.65165E+00_JPRB, 0.67786E+00_JPRB, 0.71372E+00_JPRB, 0.74959E+00_JPRB, 0.76118E+00_JPRB ,&
&  0.75340E+00_JPRB, 0.73697E+00_JPRB, 0.73606E+00_JPRB, 0.73539E+00_JPRB, 0.73366E+00_JPRB, 0.73096E+00_JPRB, 0.71945E+00_JPRB ,&
&  0.70611E+00_JPRB, 0.71863E+00_JPRB, 0.75311E+00_JPRB, 0.70026E+00_JPRB, 0.74168E+00_JPRB, 0.67310E+00_JPRB, 0.65744E+00_JPRB ,&
&  0.64098E+00_JPRB, 0.62322E+00_JPRB, 0.57752E+00_JPRB, 0.55472E+00_JPRB, 0.52440E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.58519E+00_JPRB, 0.52079E+00_JPRB, 0.46753E+00_JPRB, 0.48270E+00_JPRB, 0.54583E+00_JPRB, 0.59954E+00_JPRB, 0.62770E+00_JPRB ,&
&  0.66038E+00_JPRB, 0.66852E+00_JPRB, 0.67440E+00_JPRB, 0.68342E+00_JPRB, 0.70518E+00_JPRB, 0.71123E+00_JPRB, 0.73169E+00_JPRB ,&
&  0.74187E+00_JPRB, 0.74781E+00_JPRB, 0.75922E+00_JPRB, 0.79255E+00_JPRB, 0.83298E+00_JPRB, 0.87413E+00_JPRB, 0.91313E+00_JPRB ,&
&  0.95513E+00_JPRB, 0.10020E+01_JPRB, 0.10699E+01_JPRB, 0.11543E+01_JPRB, 0.12671E+01_JPRB, 0.13910E+01_JPRB, 0.14832E+01_JPRB ,&
&  0.10716E+01_JPRB, 0.58985E+00_JPRB, 0.10908E+01_JPRB, 0.13689E+01_JPRB, 0.13468E+01_JPRB, 0.13449E+01_JPRB, 0.13399E+01_JPRB ,&
&  0.13349E+01_JPRB, 0.13736E+01_JPRB, 0.14428E+01_JPRB, 0.18933E+01_JPRB, 0.23286E+01_JPRB, 0.24894E+01_JPRB, 0.25475E+01_JPRB ,&
&  0.26050E+01_JPRB, 0.26278E+01_JPRB, 0.26662E+01_JPRB, 0.27274E+01_JPRB, 0.28047E+01_JPRB, 0.28553E+01_JPRB, 0.28916E+01_JPRB ,&
&  0.29217E+01_JPRB, 0.29080E+01_JPRB, 0.27975E+01_JPRB, 0.20272E+01_JPRB, 0.18226E+01_JPRB, 0.15902E+01_JPRB, 0.14527E+01_JPRB ,&
&  0.12817E+01_JPRB, 0.10081E+01_JPRB, 0.93204E+00_JPRB, 0.92024E+00_JPRB, 0.83651E+00_JPRB, 0.72692E+00_JPRB, 0.70043E+00_JPRB ,&
&  0.69912E+00_JPRB, 0.67953E+00_JPRB, 0.65693E+00_JPRB, 0.63944E+00_JPRB, 0.66701E+00_JPRB, 0.66418E+00_JPRB, 0.66081E+00_JPRB ,&
&  0.65505E+00_JPRB, 0.68514E+00_JPRB, 0.71612E+00_JPRB, 0.74181E+00_JPRB, 0.76872E+00_JPRB, 0.80076E+00_JPRB, 0.80343E+00_JPRB ,&
&  0.81251E+00_JPRB, 0.80595E+00_JPRB, 0.80552E+00_JPRB, 0.78913E+00_JPRB, 0.77536E+00_JPRB, 0.74797E+00_JPRB, 0.74623E+00_JPRB ,&
&  0.75101E+00_JPRB, 0.75860E+00_JPRB, 0.75428E+00_JPRB, 0.74308E+00_JPRB, 0.76505E+00_JPRB, 0.75815E+00_JPRB, 0.76560E+00_JPRB ,&
&  0.76240E+00_JPRB, 0.74207E+00_JPRB, 0.68564E+00_JPRB, 0.64072E+00_JPRB, 0.62190E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.63344E+00_JPRB, 0.59687E+00_JPRB, 0.53995E+00_JPRB, 0.50294E+00_JPRB, 0.52467E+00_JPRB, 0.58999E+00_JPRB, 0.64197E+00_JPRB ,&
&  0.66146E+00_JPRB, 0.65981E+00_JPRB, 0.65012E+00_JPRB, 0.64828E+00_JPRB, 0.66086E+00_JPRB, 0.65656E+00_JPRB, 0.66688E+00_JPRB ,&
&  0.68309E+00_JPRB, 0.69246E+00_JPRB, 0.70548E+00_JPRB, 0.72625E+00_JPRB, 0.74849E+00_JPRB, 0.78533E+00_JPRB, 0.82102E+00_JPRB ,&
&  0.84846E+00_JPRB, 0.87393E+00_JPRB, 0.91282E+00_JPRB, 0.96558E+00_JPRB, 0.10463E+01_JPRB, 0.11980E+01_JPRB, 0.14641E+01_JPRB ,&
&  0.19626E+01_JPRB, 0.13550E+01_JPRB, 0.73171E+00_JPRB, 0.12541E+01_JPRB, 0.13700E+01_JPRB, 0.13336E+01_JPRB, 0.12068E+01_JPRB ,&
&  0.12164E+01_JPRB, 0.12133E+01_JPRB, 0.12643E+01_JPRB, 0.14780E+01_JPRB, 0.17350E+01_JPRB, 0.20494E+01_JPRB, 0.22376E+01_JPRB ,&
&  0.22830E+01_JPRB, 0.23019E+01_JPRB, 0.23174E+01_JPRB, 0.23582E+01_JPRB, 0.24265E+01_JPRB, 0.24942E+01_JPRB, 0.25088E+01_JPRB ,&
&  0.25324E+01_JPRB, 0.25797E+01_JPRB, 0.25098E+01_JPRB, 0.18665E+01_JPRB, 0.17197E+01_JPRB, 0.16585E+01_JPRB, 0.15779E+01_JPRB ,&
&  0.13881E+01_JPRB, 0.12526E+01_JPRB, 0.11425E+01_JPRB, 0.98417E+00_JPRB, 0.86452E+00_JPRB, 0.79320E+00_JPRB, 0.79911E+00_JPRB ,&
&  0.77961E+00_JPRB, 0.72700E+00_JPRB, 0.70087E+00_JPRB, 0.67939E+00_JPRB, 0.69777E+00_JPRB, 0.72166E+00_JPRB, 0.71991E+00_JPRB ,&
&  0.73532E+00_JPRB, 0.76116E+00_JPRB, 0.78046E+00_JPRB, 0.79914E+00_JPRB, 0.82367E+00_JPRB, 0.84535E+00_JPRB, 0.85630E+00_JPRB ,&
&  0.86814E+00_JPRB, 0.86467E+00_JPRB, 0.85849E+00_JPRB, 0.84517E+00_JPRB, 0.80823E+00_JPRB, 0.77425E+00_JPRB, 0.78254E+00_JPRB ,&
&  0.77979E+00_JPRB, 0.78644E+00_JPRB, 0.77711E+00_JPRB, 0.78763E+00_JPRB, 0.79139E+00_JPRB, 0.81112E+00_JPRB, 0.84047E+00_JPRB ,&
&  0.86702E+00_JPRB, 0.85016E+00_JPRB, 0.76809E+00_JPRB, 0.67447E+00_JPRB, 0.65164E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.66161E+00_JPRB, 0.64952E+00_JPRB, 0.61852E+00_JPRB, 0.55391E+00_JPRB, 0.53376E+00_JPRB, 0.54933E+00_JPRB, 0.61078E+00_JPRB ,&
&  0.63732E+00_JPRB, 0.64433E+00_JPRB, 0.62961E+00_JPRB, 0.61396E+00_JPRB, 0.60595E+00_JPRB, 0.60784E+00_JPRB, 0.62447E+00_JPRB ,&
&  0.64012E+00_JPRB, 0.63363E+00_JPRB, 0.63152E+00_JPRB, 0.63979E+00_JPRB, 0.66044E+00_JPRB, 0.69133E+00_JPRB, 0.73164E+00_JPRB ,&
&  0.76404E+00_JPRB, 0.77923E+00_JPRB, 0.78938E+00_JPRB, 0.82287E+00_JPRB, 0.86530E+00_JPRB, 0.88157E+00_JPRB, 0.94742E+00_JPRB ,&
&  0.12838E+01_JPRB, 0.18084E+01_JPRB, 0.85111E+00_JPRB, 0.12997E+01_JPRB, 0.15172E+01_JPRB, 0.13712E+01_JPRB, 0.12582E+01_JPRB ,&
&  0.11939E+01_JPRB, 0.10577E+01_JPRB, 0.10172E+01_JPRB, 0.11609E+01_JPRB, 0.12971E+01_JPRB, 0.14177E+01_JPRB, 0.16056E+01_JPRB ,&
&  0.17758E+01_JPRB, 0.18810E+01_JPRB, 0.19638E+01_JPRB, 0.20269E+01_JPRB, 0.21191E+01_JPRB, 0.22532E+01_JPRB, 0.23755E+01_JPRB ,&
&  0.24658E+01_JPRB, 0.25991E+01_JPRB, 0.26753E+01_JPRB, 0.21319E+01_JPRB, 0.20301E+01_JPRB, 0.21676E+01_JPRB, 0.20498E+01_JPRB ,&
&  0.17749E+01_JPRB, 0.15769E+01_JPRB, 0.13493E+01_JPRB, 0.11065E+01_JPRB, 0.94337E+00_JPRB, 0.87835E+00_JPRB, 0.86826E+00_JPRB ,&
&  0.82852E+00_JPRB, 0.78430E+00_JPRB, 0.77481E+00_JPRB, 0.77396E+00_JPRB, 0.77478E+00_JPRB, 0.78053E+00_JPRB, 0.79358E+00_JPRB ,&
&  0.82085E+00_JPRB, 0.84613E+00_JPRB, 0.86619E+00_JPRB, 0.87766E+00_JPRB, 0.88873E+00_JPRB, 0.89788E+00_JPRB, 0.89015E+00_JPRB ,&
&  0.89280E+00_JPRB, 0.88599E+00_JPRB, 0.87791E+00_JPRB, 0.86124E+00_JPRB, 0.83753E+00_JPRB, 0.82545E+00_JPRB, 0.81892E+00_JPRB ,&
&  0.81083E+00_JPRB, 0.79972E+00_JPRB, 0.77465E+00_JPRB, 0.76158E+00_JPRB, 0.79428E+00_JPRB, 0.77930E+00_JPRB, 0.82198E+00_JPRB ,&
&  0.88785E+00_JPRB, 0.93486E+00_JPRB, 0.88754E+00_JPRB, 0.69244E+00_JPRB, 0.66914E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.68514E+00_JPRB, 0.67819E+00_JPRB, 0.66953E+00_JPRB, 0.62550E+00_JPRB, 0.58564E+00_JPRB, 0.56824E+00_JPRB, 0.58353E+00_JPRB ,&
&  0.61783E+00_JPRB, 0.64125E+00_JPRB, 0.64002E+00_JPRB, 0.62903E+00_JPRB, 0.62596E+00_JPRB, 0.63202E+00_JPRB, 0.64182E+00_JPRB ,&
&  0.64296E+00_JPRB, 0.63362E+00_JPRB, 0.62472E+00_JPRB, 0.63033E+00_JPRB, 0.63337E+00_JPRB, 0.64377E+00_JPRB, 0.66778E+00_JPRB ,&
&  0.70035E+00_JPRB, 0.71648E+00_JPRB, 0.71836E+00_JPRB, 0.72542E+00_JPRB, 0.75645E+00_JPRB, 0.75967E+00_JPRB, 0.77094E+00_JPRB ,&
&  0.88826E+00_JPRB, 0.15015E+01_JPRB, 0.16957E+01_JPRB, 0.20408E+01_JPRB, 0.18431E+01_JPRB, 0.15232E+01_JPRB, 0.14120E+01_JPRB ,&
&  0.13505E+01_JPRB, 0.11747E+01_JPRB, 0.11096E+01_JPRB, 0.10150E+01_JPRB, 0.10791E+01_JPRB, 0.11431E+01_JPRB, 0.11741E+01_JPRB ,&
&  0.12649E+01_JPRB, 0.14443E+01_JPRB, 0.15528E+01_JPRB, 0.16559E+01_JPRB, 0.17523E+01_JPRB, 0.18684E+01_JPRB, 0.20429E+01_JPRB ,&
&  0.22516E+01_JPRB, 0.25236E+01_JPRB, 0.28485E+01_JPRB, 0.26718E+01_JPRB, 0.23461E+01_JPRB, 0.26950E+01_JPRB, 0.27989E+01_JPRB ,&
&  0.26643E+01_JPRB, 0.20771E+01_JPRB, 0.16218E+01_JPRB, 0.12885E+01_JPRB, 0.10658E+01_JPRB, 0.10820E+01_JPRB, 0.96048E+00_JPRB ,&
&  0.87484E+00_JPRB, 0.85420E+00_JPRB, 0.84006E+00_JPRB, 0.84695E+00_JPRB, 0.84925E+00_JPRB, 0.84117E+00_JPRB, 0.84860E+00_JPRB ,&
&  0.87087E+00_JPRB, 0.88955E+00_JPRB, 0.90419E+00_JPRB, 0.91439E+00_JPRB, 0.92853E+00_JPRB, 0.93670E+00_JPRB, 0.92555E+00_JPRB ,&
&  0.90412E+00_JPRB, 0.89121E+00_JPRB, 0.84355E+00_JPRB, 0.84027E+00_JPRB, 0.83883E+00_JPRB, 0.84030E+00_JPRB, 0.81538E+00_JPRB ,&
&  0.81096E+00_JPRB, 0.81686E+00_JPRB, 0.79662E+00_JPRB, 0.78159E+00_JPRB, 0.79125E+00_JPRB, 0.78330E+00_JPRB, 0.75182E+00_JPRB ,&
&  0.75355E+00_JPRB, 0.74191E+00_JPRB, 0.73236E+00_JPRB, 0.71012E+00_JPRB, 0.69538E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.72149E+00_JPRB, 0.71495E+00_JPRB, 0.71340E+00_JPRB, 0.69280E+00_JPRB, 0.66670E+00_JPRB, 0.64665E+00_JPRB, 0.63795E+00_JPRB ,&
&  0.64017E+00_JPRB, 0.65166E+00_JPRB, 0.66140E+00_JPRB, 0.65081E+00_JPRB, 0.64621E+00_JPRB, 0.65614E+00_JPRB, 0.65988E+00_JPRB ,&
&  0.66022E+00_JPRB, 0.65667E+00_JPRB, 0.65392E+00_JPRB, 0.65326E+00_JPRB, 0.64988E+00_JPRB, 0.63878E+00_JPRB, 0.63849E+00_JPRB ,&
&  0.65295E+00_JPRB, 0.66676E+00_JPRB, 0.67254E+00_JPRB, 0.67352E+00_JPRB, 0.69092E+00_JPRB, 0.69726E+00_JPRB, 0.70847E+00_JPRB ,&
&  0.76087E+00_JPRB, 0.13479E+01_JPRB, 0.13121E+01_JPRB, 0.23122E+01_JPRB, 0.19722E+01_JPRB, 0.16584E+01_JPRB, 0.14081E+01_JPRB ,&
&  0.13398E+01_JPRB, 0.14190E+01_JPRB, 0.12050E+01_JPRB, 0.10469E+01_JPRB, 0.98886E+00_JPRB, 0.10076E+01_JPRB, 0.10040E+01_JPRB ,&
&  0.99266E+00_JPRB, 0.10313E+01_JPRB, 0.11036E+01_JPRB, 0.12109E+01_JPRB, 0.13348E+01_JPRB, 0.14406E+01_JPRB, 0.15691E+01_JPRB ,&
&  0.17270E+01_JPRB, 0.19986E+01_JPRB, 0.24307E+01_JPRB, 0.25244E+01_JPRB, 0.25815E+01_JPRB, 0.30136E+01_JPRB, 0.34783E+01_JPRB ,&
&  0.34461E+01_JPRB, 0.27964E+01_JPRB, 0.20028E+01_JPRB, 0.14983E+01_JPRB, 0.12586E+01_JPRB, 0.11886E+01_JPRB, 0.10614E+01_JPRB ,&
&  0.96126E+00_JPRB, 0.92129E+00_JPRB, 0.90591E+00_JPRB, 0.89481E+00_JPRB, 0.89288E+00_JPRB, 0.89060E+00_JPRB, 0.89095E+00_JPRB ,&
&  0.90147E+00_JPRB, 0.92725E+00_JPRB, 0.93070E+00_JPRB, 0.92963E+00_JPRB, 0.93409E+00_JPRB, 0.93694E+00_JPRB, 0.92755E+00_JPRB ,&
&  0.92190E+00_JPRB, 0.91088E+00_JPRB, 0.85181E+00_JPRB, 0.82400E+00_JPRB, 0.82660E+00_JPRB, 0.82428E+00_JPRB, 0.82036E+00_JPRB ,&
&  0.84114E+00_JPRB, 0.84178E+00_JPRB, 0.81635E+00_JPRB, 0.77909E+00_JPRB, 0.78829E+00_JPRB, 0.80575E+00_JPRB, 0.75752E+00_JPRB ,&
&  0.74704E+00_JPRB, 0.74027E+00_JPRB, 0.73880E+00_JPRB, 0.73557E+00_JPRB, 0.72851E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.72731E+00_JPRB, 0.72080E+00_JPRB, 0.72102E+00_JPRB, 0.71236E+00_JPRB, 0.69334E+00_JPRB, 0.67523E+00_JPRB, 0.66941E+00_JPRB ,&
&  0.66450E+00_JPRB, 0.66684E+00_JPRB, 0.67454E+00_JPRB, 0.66700E+00_JPRB, 0.66862E+00_JPRB, 0.67481E+00_JPRB, 0.67612E+00_JPRB ,&
&  0.66898E+00_JPRB, 0.66499E+00_JPRB, 0.65945E+00_JPRB, 0.65542E+00_JPRB, 0.65375E+00_JPRB, 0.64373E+00_JPRB, 0.63830E+00_JPRB ,&
&  0.63770E+00_JPRB, 0.64337E+00_JPRB, 0.64897E+00_JPRB, 0.65099E+00_JPRB, 0.66170E+00_JPRB, 0.66654E+00_JPRB, 0.67218E+00_JPRB ,&
&  0.72514E+00_JPRB, 0.13786E+01_JPRB, 0.12003E+01_JPRB, 0.20768E+01_JPRB, 0.18434E+01_JPRB, 0.15393E+01_JPRB, 0.12872E+01_JPRB ,&
&  0.12691E+01_JPRB, 0.13020E+01_JPRB, 0.12030E+01_JPRB, 0.11110E+01_JPRB, 0.10376E+01_JPRB, 0.97932E+00_JPRB, 0.94114E+00_JPRB ,&
&  0.90754E+00_JPRB, 0.89010E+00_JPRB, 0.89500E+00_JPRB, 0.92332E+00_JPRB, 0.99716E+00_JPRB, 0.10900E+01_JPRB, 0.11762E+01_JPRB ,&
&  0.12716E+01_JPRB, 0.14309E+01_JPRB, 0.17343E+01_JPRB, 0.22283E+01_JPRB, 0.23659E+01_JPRB, 0.27330E+01_JPRB, 0.40586E+01_JPRB ,&
&  0.42117E+01_JPRB, 0.36582E+01_JPRB, 0.25231E+01_JPRB, 0.17305E+01_JPRB, 0.14193E+01_JPRB, 0.12931E+01_JPRB, 0.11744E+01_JPRB ,&
&  0.10640E+01_JPRB, 0.99132E+00_JPRB, 0.97535E+00_JPRB, 0.93844E+00_JPRB, 0.91070E+00_JPRB, 0.92025E+00_JPRB, 0.94379E+00_JPRB ,&
&  0.95904E+00_JPRB, 0.94941E+00_JPRB, 0.95619E+00_JPRB, 0.93959E+00_JPRB, 0.92380E+00_JPRB, 0.90641E+00_JPRB, 0.90179E+00_JPRB ,&
&  0.90488E+00_JPRB, 0.91102E+00_JPRB, 0.87581E+00_JPRB, 0.83931E+00_JPRB, 0.83312E+00_JPRB, 0.83441E+00_JPRB, 0.82665E+00_JPRB ,&
&  0.85937E+00_JPRB, 0.85708E+00_JPRB, 0.83021E+00_JPRB, 0.80927E+00_JPRB, 0.78767E+00_JPRB, 0.80663E+00_JPRB, 0.75237E+00_JPRB ,&
&  0.73398E+00_JPRB, 0.72795E+00_JPRB, 0.72969E+00_JPRB, 0.73741E+00_JPRB, 0.73962E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.73908E+00_JPRB, 0.72851E+00_JPRB, 0.72708E+00_JPRB, 0.72389E+00_JPRB, 0.71421E+00_JPRB, 0.70219E+00_JPRB, 0.70343E+00_JPRB ,&
&  0.70534E+00_JPRB, 0.70575E+00_JPRB, 0.69633E+00_JPRB, 0.68036E+00_JPRB, 0.67850E+00_JPRB, 0.67599E+00_JPRB, 0.67201E+00_JPRB ,&
&  0.66758E+00_JPRB, 0.66642E+00_JPRB, 0.65738E+00_JPRB, 0.65243E+00_JPRB, 0.65467E+00_JPRB, 0.65197E+00_JPRB, 0.64336E+00_JPRB ,&
&  0.63418E+00_JPRB, 0.62915E+00_JPRB, 0.63523E+00_JPRB, 0.64154E+00_JPRB, 0.65566E+00_JPRB, 0.65881E+00_JPRB, 0.66259E+00_JPRB ,&
&  0.72924E+00_JPRB, 0.67450E+00_JPRB, 0.13937E+01_JPRB, 0.17837E+01_JPRB, 0.17359E+01_JPRB, 0.14996E+01_JPRB, 0.13307E+01_JPRB ,&
&  0.12948E+01_JPRB, 0.12226E+01_JPRB, 0.11711E+01_JPRB, 0.10914E+01_JPRB, 0.10110E+01_JPRB, 0.92823E+00_JPRB, 0.89538E+00_JPRB ,&
&  0.87933E+00_JPRB, 0.86065E+00_JPRB, 0.85543E+00_JPRB, 0.85655E+00_JPRB, 0.86910E+00_JPRB, 0.89449E+00_JPRB, 0.94334E+00_JPRB ,&
&  0.99476E+00_JPRB, 0.10582E+01_JPRB, 0.11932E+01_JPRB, 0.15711E+01_JPRB, 0.21479E+01_JPRB, 0.26157E+01_JPRB, 0.36098E+01_JPRB ,&
&  0.45391E+01_JPRB, 0.42422E+01_JPRB, 0.30001E+01_JPRB, 0.20176E+01_JPRB, 0.15803E+01_JPRB, 0.14091E+01_JPRB, 0.12787E+01_JPRB ,&
&  0.11725E+01_JPRB, 0.10714E+01_JPRB, 0.10152E+01_JPRB, 0.97169E+00_JPRB, 0.93566E+00_JPRB, 0.91637E+00_JPRB, 0.94802E+00_JPRB ,&
&  0.98243E+00_JPRB, 0.98891E+00_JPRB, 0.96484E+00_JPRB, 0.93979E+00_JPRB, 0.91913E+00_JPRB, 0.89948E+00_JPRB, 0.89466E+00_JPRB ,&
&  0.89749E+00_JPRB, 0.90729E+00_JPRB, 0.88758E+00_JPRB, 0.86375E+00_JPRB, 0.86100E+00_JPRB, 0.87327E+00_JPRB, 0.87673E+00_JPRB ,&
&  0.86729E+00_JPRB, 0.85584E+00_JPRB, 0.82858E+00_JPRB, 0.81905E+00_JPRB, 0.77543E+00_JPRB, 0.80043E+00_JPRB, 0.75073E+00_JPRB ,&
&  0.72868E+00_JPRB, 0.72520E+00_JPRB, 0.73186E+00_JPRB, 0.73772E+00_JPRB, 0.75588E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.74990E+00_JPRB, 0.73031E+00_JPRB, 0.72507E+00_JPRB, 0.73025E+00_JPRB, 0.72866E+00_JPRB, 0.73317E+00_JPRB, 0.73769E+00_JPRB ,&
&  0.73404E+00_JPRB, 0.71791E+00_JPRB, 0.70330E+00_JPRB, 0.68710E+00_JPRB, 0.68163E+00_JPRB, 0.67429E+00_JPRB, 0.67288E+00_JPRB ,&
&  0.66605E+00_JPRB, 0.66216E+00_JPRB, 0.65536E+00_JPRB, 0.65373E+00_JPRB, 0.65410E+00_JPRB, 0.64933E+00_JPRB, 0.62983E+00_JPRB ,&
&  0.62493E+00_JPRB, 0.62434E+00_JPRB, 0.63297E+00_JPRB, 0.64473E+00_JPRB, 0.65908E+00_JPRB, 0.66608E+00_JPRB, 0.66997E+00_JPRB ,&
&  0.72630E+00_JPRB, 0.87191E+00_JPRB, 0.13441E+01_JPRB, 0.14541E+01_JPRB, 0.14685E+01_JPRB, 0.13547E+01_JPRB, 0.12616E+01_JPRB ,&
&  0.11857E+01_JPRB, 0.11519E+01_JPRB, 0.11454E+01_JPRB, 0.10635E+01_JPRB, 0.94788E+00_JPRB, 0.85901E+00_JPRB, 0.83063E+00_JPRB ,&
&  0.81782E+00_JPRB, 0.80770E+00_JPRB, 0.80396E+00_JPRB, 0.80655E+00_JPRB, 0.80528E+00_JPRB, 0.80740E+00_JPRB, 0.82342E+00_JPRB ,&
&  0.84454E+00_JPRB, 0.87510E+00_JPRB, 0.92841E+00_JPRB, 0.11233E+01_JPRB, 0.18120E+01_JPRB, 0.20526E+01_JPRB, 0.31483E+01_JPRB ,&
&  0.46082E+01_JPRB, 0.41213E+01_JPRB, 0.31695E+01_JPRB, 0.23081E+01_JPRB, 0.18224E+01_JPRB, 0.15905E+01_JPRB, 0.14163E+01_JPRB ,&
&  0.12804E+01_JPRB, 0.11663E+01_JPRB, 0.10735E+01_JPRB, 0.10241E+01_JPRB, 0.98004E+00_JPRB, 0.94338E+00_JPRB, 0.93897E+00_JPRB ,&
&  0.96222E+00_JPRB, 0.97892E+00_JPRB, 0.96653E+00_JPRB, 0.93866E+00_JPRB, 0.91687E+00_JPRB, 0.91351E+00_JPRB, 0.91081E+00_JPRB ,&
&  0.91156E+00_JPRB, 0.91166E+00_JPRB, 0.89817E+00_JPRB, 0.86770E+00_JPRB, 0.87089E+00_JPRB, 0.89409E+00_JPRB, 0.89420E+00_JPRB ,&
&  0.85946E+00_JPRB, 0.82591E+00_JPRB, 0.83342E+00_JPRB, 0.81002E+00_JPRB, 0.78979E+00_JPRB, 0.81178E+00_JPRB, 0.75367E+00_JPRB ,&
&  0.72242E+00_JPRB, 0.71706E+00_JPRB, 0.73443E+00_JPRB, 0.75456E+00_JPRB, 0.78750E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.77748E+00_JPRB, 0.74001E+00_JPRB, 0.72089E+00_JPRB, 0.72190E+00_JPRB, 0.73056E+00_JPRB, 0.73046E+00_JPRB, 0.73552E+00_JPRB ,&
&  0.72933E+00_JPRB, 0.71522E+00_JPRB, 0.69911E+00_JPRB, 0.68430E+00_JPRB, 0.67688E+00_JPRB, 0.67167E+00_JPRB, 0.66787E+00_JPRB ,&
&  0.66509E+00_JPRB, 0.66232E+00_JPRB, 0.65672E+00_JPRB, 0.66023E+00_JPRB, 0.66253E+00_JPRB, 0.66109E+00_JPRB, 0.63951E+00_JPRB ,&
&  0.62903E+00_JPRB, 0.63088E+00_JPRB, 0.63680E+00_JPRB, 0.64215E+00_JPRB, 0.65091E+00_JPRB, 0.66358E+00_JPRB, 0.67600E+00_JPRB ,&
&  0.70303E+00_JPRB, 0.84361E+00_JPRB, 0.11036E+01_JPRB, 0.12821E+01_JPRB, 0.13355E+01_JPRB, 0.12998E+01_JPRB, 0.12162E+01_JPRB ,&
&  0.11481E+01_JPRB, 0.11060E+01_JPRB, 0.10648E+01_JPRB, 0.96539E+00_JPRB, 0.86512E+00_JPRB, 0.81229E+00_JPRB, 0.78724E+00_JPRB ,&
&  0.77761E+00_JPRB, 0.77686E+00_JPRB, 0.77436E+00_JPRB, 0.77213E+00_JPRB, 0.76926E+00_JPRB, 0.76987E+00_JPRB, 0.77514E+00_JPRB ,&
&  0.78398E+00_JPRB, 0.79156E+00_JPRB, 0.81281E+00_JPRB, 0.85179E+00_JPRB, 0.11948E+01_JPRB, 0.16618E+01_JPRB, 0.23960E+01_JPRB ,&
&  0.27421E+01_JPRB, 0.28106E+01_JPRB, 0.25620E+01_JPRB, 0.22278E+01_JPRB, 0.19349E+01_JPRB, 0.17055E+01_JPRB, 0.15102E+01_JPRB ,&
&  0.13396E+01_JPRB, 0.11900E+01_JPRB, 0.10698E+01_JPRB, 0.99778E+00_JPRB, 0.95755E+00_JPRB, 0.92149E+00_JPRB, 0.90422E+00_JPRB ,&
&  0.91063E+00_JPRB, 0.92281E+00_JPRB, 0.92709E+00_JPRB, 0.92355E+00_JPRB, 0.91013E+00_JPRB, 0.89796E+00_JPRB, 0.89132E+00_JPRB ,&
&  0.89425E+00_JPRB, 0.89202E+00_JPRB, 0.89935E+00_JPRB, 0.88801E+00_JPRB, 0.87555E+00_JPRB, 0.86485E+00_JPRB, 0.86062E+00_JPRB ,&
&  0.86290E+00_JPRB, 0.83709E+00_JPRB, 0.82298E+00_JPRB, 0.81399E+00_JPRB, 0.79185E+00_JPRB, 0.81180E+00_JPRB, 0.75676E+00_JPRB ,&
&  0.72727E+00_JPRB, 0.72067E+00_JPRB, 0.72543E+00_JPRB, 0.74118E+00_JPRB, 0.78221E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.80258E+00_JPRB, 0.77381E+00_JPRB, 0.74502E+00_JPRB, 0.72784E+00_JPRB, 0.72212E+00_JPRB, 0.72134E+00_JPRB, 0.72176E+00_JPRB ,&
&  0.71933E+00_JPRB, 0.71403E+00_JPRB, 0.69684E+00_JPRB, 0.68384E+00_JPRB, 0.67487E+00_JPRB, 0.67226E+00_JPRB, 0.66995E+00_JPRB ,&
&  0.67003E+00_JPRB, 0.66895E+00_JPRB, 0.66428E+00_JPRB, 0.66331E+00_JPRB, 0.66492E+00_JPRB, 0.66001E+00_JPRB, 0.64352E+00_JPRB ,&
&  0.62664E+00_JPRB, 0.62828E+00_JPRB, 0.63409E+00_JPRB, 0.63822E+00_JPRB, 0.64600E+00_JPRB, 0.65630E+00_JPRB, 0.67321E+00_JPRB ,&
&  0.70188E+00_JPRB, 0.74121E+00_JPRB, 0.96619E+00_JPRB, 0.11075E+01_JPRB, 0.11732E+01_JPRB, 0.11981E+01_JPRB, 0.11488E+01_JPRB ,&
&  0.10847E+01_JPRB, 0.10034E+01_JPRB, 0.93725E+00_JPRB, 0.86704E+00_JPRB, 0.82047E+00_JPRB, 0.78765E+00_JPRB, 0.76640E+00_JPRB ,&
&  0.75629E+00_JPRB, 0.75303E+00_JPRB, 0.75032E+00_JPRB, 0.74629E+00_JPRB, 0.74416E+00_JPRB, 0.74378E+00_JPRB, 0.74502E+00_JPRB ,&
&  0.74865E+00_JPRB, 0.75654E+00_JPRB, 0.76632E+00_JPRB, 0.78107E+00_JPRB, 0.83084E+00_JPRB, 0.10157E+01_JPRB, 0.14125E+01_JPRB ,&
&  0.17722E+01_JPRB, 0.19809E+01_JPRB, 0.19470E+01_JPRB, 0.18736E+01_JPRB, 0.18068E+01_JPRB, 0.16647E+01_JPRB, 0.15143E+01_JPRB ,&
&  0.13649E+01_JPRB, 0.12251E+01_JPRB, 0.10832E+01_JPRB, 0.97193E+00_JPRB, 0.90284E+00_JPRB, 0.86489E+00_JPRB, 0.84574E+00_JPRB ,&
&  0.84716E+00_JPRB, 0.86299E+00_JPRB, 0.87207E+00_JPRB, 0.88304E+00_JPRB, 0.88103E+00_JPRB, 0.87038E+00_JPRB, 0.85782E+00_JPRB ,&
&  0.84302E+00_JPRB, 0.82735E+00_JPRB, 0.82457E+00_JPRB, 0.81847E+00_JPRB, 0.81245E+00_JPRB, 0.81241E+00_JPRB, 0.81233E+00_JPRB ,&
&  0.81367E+00_JPRB, 0.82427E+00_JPRB, 0.79211E+00_JPRB, 0.78373E+00_JPRB, 0.84364E+00_JPRB, 0.80879E+00_JPRB, 0.77047E+00_JPRB ,&
&  0.74064E+00_JPRB, 0.72941E+00_JPRB, 0.72879E+00_JPRB, 0.74613E+00_JPRB, 0.79131E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.71794E+00_JPRB, 0.71776E+00_JPRB, 0.71514E+00_JPRB, 0.71595E+00_JPRB, 0.71535E+00_JPRB, 0.70717E+00_JPRB, 0.70102E+00_JPRB ,&
&  0.70042E+00_JPRB, 0.69761E+00_JPRB, 0.69419E+00_JPRB, 0.68694E+00_JPRB, 0.67636E+00_JPRB, 0.66878E+00_JPRB, 0.66796E+00_JPRB ,&
&  0.66477E+00_JPRB, 0.66359E+00_JPRB, 0.66314E+00_JPRB, 0.65888E+00_JPRB, 0.65314E+00_JPRB, 0.64426E+00_JPRB, 0.62977E+00_JPRB ,&
&  0.61178E+00_JPRB, 0.60627E+00_JPRB, 0.61534E+00_JPRB, 0.62320E+00_JPRB, 0.62863E+00_JPRB, 0.63163E+00_JPRB, 0.64341E+00_JPRB ,&
&  0.68343E+00_JPRB, 0.68593E+00_JPRB, 0.77792E+00_JPRB, 0.91365E+00_JPRB, 0.99203E+00_JPRB, 0.98277E+00_JPRB, 0.97597E+00_JPRB ,&
&  0.94626E+00_JPRB, 0.91143E+00_JPRB, 0.87221E+00_JPRB, 0.82600E+00_JPRB, 0.78615E+00_JPRB, 0.75967E+00_JPRB, 0.74830E+00_JPRB ,&
&  0.74163E+00_JPRB, 0.73552E+00_JPRB, 0.73353E+00_JPRB, 0.72621E+00_JPRB, 0.71982E+00_JPRB, 0.71492E+00_JPRB, 0.71680E+00_JPRB ,&
&  0.72010E+00_JPRB, 0.72452E+00_JPRB, 0.72849E+00_JPRB, 0.73493E+00_JPRB, 0.75040E+00_JPRB, 0.80551E+00_JPRB, 0.98375E+00_JPRB ,&
&  0.12200E+01_JPRB, 0.14418E+01_JPRB, 0.15443E+01_JPRB, 0.15200E+01_JPRB, 0.14876E+01_JPRB, 0.14419E+01_JPRB, 0.13335E+01_JPRB ,&
&  0.11951E+01_JPRB, 0.11029E+01_JPRB, 0.10328E+01_JPRB, 0.97170E+00_JPRB, 0.92711E+00_JPRB, 0.89189E+00_JPRB, 0.86505E+00_JPRB ,&
&  0.84907E+00_JPRB, 0.84464E+00_JPRB, 0.84273E+00_JPRB, 0.84171E+00_JPRB, 0.83643E+00_JPRB, 0.81910E+00_JPRB, 0.79271E+00_JPRB ,&
&  0.76184E+00_JPRB, 0.74842E+00_JPRB, 0.74979E+00_JPRB, 0.74503E+00_JPRB, 0.74133E+00_JPRB, 0.74777E+00_JPRB, 0.76030E+00_JPRB ,&
&  0.76575E+00_JPRB, 0.77632E+00_JPRB, 0.78580E+00_JPRB, 0.81489E+00_JPRB, 0.81085E+00_JPRB, 0.79064E+00_JPRB, 0.77799E+00_JPRB ,&
&  0.75571E+00_JPRB, 0.73481E+00_JPRB, 0.72067E+00_JPRB, 0.72341E+00_JPRB, 0.68059E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.70573E+00_JPRB, 0.70394E+00_JPRB, 0.69880E+00_JPRB, 0.69850E+00_JPRB, 0.70039E+00_JPRB, 0.68976E+00_JPRB, 0.68025E+00_JPRB ,&
&  0.67556E+00_JPRB, 0.67737E+00_JPRB, 0.68154E+00_JPRB, 0.68334E+00_JPRB, 0.67825E+00_JPRB, 0.66810E+00_JPRB, 0.66215E+00_JPRB ,&
&  0.65323E+00_JPRB, 0.64527E+00_JPRB, 0.64347E+00_JPRB, 0.64286E+00_JPRB, 0.63721E+00_JPRB, 0.62959E+00_JPRB, 0.62311E+00_JPRB ,&
&  0.60959E+00_JPRB, 0.60071E+00_JPRB, 0.59901E+00_JPRB, 0.60252E+00_JPRB, 0.60651E+00_JPRB, 0.60569E+00_JPRB, 0.62363E+00_JPRB ,&
&  0.67344E+00_JPRB, 0.61607E+00_JPRB, 0.62211E+00_JPRB, 0.73274E+00_JPRB, 0.80631E+00_JPRB, 0.82268E+00_JPRB, 0.84191E+00_JPRB ,&
&  0.86373E+00_JPRB, 0.85632E+00_JPRB, 0.83558E+00_JPRB, 0.79174E+00_JPRB, 0.75788E+00_JPRB, 0.73514E+00_JPRB, 0.71594E+00_JPRB ,&
&  0.70216E+00_JPRB, 0.69103E+00_JPRB, 0.68659E+00_JPRB, 0.67880E+00_JPRB, 0.67563E+00_JPRB, 0.66674E+00_JPRB, 0.66401E+00_JPRB ,&
&  0.66526E+00_JPRB, 0.67176E+00_JPRB, 0.67902E+00_JPRB, 0.68477E+00_JPRB, 0.69777E+00_JPRB, 0.72444E+00_JPRB, 0.79557E+00_JPRB ,&
&  0.92315E+00_JPRB, 0.11006E+01_JPRB, 0.12259E+01_JPRB, 0.12663E+01_JPRB, 0.12492E+01_JPRB, 0.11999E+01_JPRB, 0.11248E+01_JPRB ,&
&  0.10412E+01_JPRB, 0.97165E+00_JPRB, 0.92195E+00_JPRB, 0.89510E+00_JPRB, 0.87572E+00_JPRB, 0.85993E+00_JPRB, 0.84418E+00_JPRB ,&
&  0.83044E+00_JPRB, 0.82176E+00_JPRB, 0.81561E+00_JPRB, 0.80721E+00_JPRB, 0.79546E+00_JPRB, 0.77009E+00_JPRB, 0.73374E+00_JPRB ,&
&  0.70549E+00_JPRB, 0.68838E+00_JPRB, 0.68505E+00_JPRB, 0.68264E+00_JPRB, 0.68407E+00_JPRB, 0.69794E+00_JPRB, 0.71559E+00_JPRB ,&
&  0.72516E+00_JPRB, 0.73669E+00_JPRB, 0.75945E+00_JPRB, 0.73972E+00_JPRB, 0.77453E+00_JPRB, 0.77445E+00_JPRB, 0.77522E+00_JPRB ,&
&  0.76430E+00_JPRB, 0.74813E+00_JPRB, 0.73630E+00_JPRB, 0.62895E+00_JPRB, 0.68759E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.69442E+00_JPRB, 0.69171E+00_JPRB, 0.68044E+00_JPRB, 0.67345E+00_JPRB, 0.66874E+00_JPRB, 0.66338E+00_JPRB, 0.65419E+00_JPRB ,&
&  0.64897E+00_JPRB, 0.64867E+00_JPRB, 0.64842E+00_JPRB, 0.64949E+00_JPRB, 0.65044E+00_JPRB, 0.64565E+00_JPRB, 0.63770E+00_JPRB ,&
&  0.62944E+00_JPRB, 0.62170E+00_JPRB, 0.62316E+00_JPRB, 0.62242E+00_JPRB, 0.61716E+00_JPRB, 0.61286E+00_JPRB, 0.60720E+00_JPRB ,&
&  0.59286E+00_JPRB, 0.57846E+00_JPRB, 0.56721E+00_JPRB, 0.57025E+00_JPRB, 0.57699E+00_JPRB, 0.57239E+00_JPRB, 0.58789E+00_JPRB ,&
&  0.64108E+00_JPRB, 0.56787E+00_JPRB, 0.59487E+00_JPRB, 0.65674E+00_JPRB, 0.68696E+00_JPRB, 0.71692E+00_JPRB, 0.73831E+00_JPRB ,&
&  0.75316E+00_JPRB, 0.76186E+00_JPRB, 0.73152E+00_JPRB, 0.69591E+00_JPRB, 0.66497E+00_JPRB, 0.64023E+00_JPRB, 0.62282E+00_JPRB ,&
&  0.61504E+00_JPRB, 0.61674E+00_JPRB, 0.61861E+00_JPRB, 0.61520E+00_JPRB, 0.61219E+00_JPRB, 0.60741E+00_JPRB, 0.60455E+00_JPRB ,&
&  0.60360E+00_JPRB, 0.60600E+00_JPRB, 0.61093E+00_JPRB, 0.62217E+00_JPRB, 0.63430E+00_JPRB, 0.64883E+00_JPRB, 0.67456E+00_JPRB ,&
&  0.72520E+00_JPRB, 0.81650E+00_JPRB, 0.91870E+00_JPRB, 0.97622E+00_JPRB, 0.98057E+00_JPRB, 0.95217E+00_JPRB, 0.90416E+00_JPRB ,&
&  0.85493E+00_JPRB, 0.81687E+00_JPRB, 0.79361E+00_JPRB, 0.77930E+00_JPRB, 0.76509E+00_JPRB, 0.75269E+00_JPRB, 0.74272E+00_JPRB ,&
&  0.73441E+00_JPRB, 0.72410E+00_JPRB, 0.71896E+00_JPRB, 0.71129E+00_JPRB, 0.69853E+00_JPRB, 0.67862E+00_JPRB, 0.65862E+00_JPRB ,&
&  0.63993E+00_JPRB, 0.62544E+00_JPRB, 0.61825E+00_JPRB, 0.61966E+00_JPRB, 0.62518E+00_JPRB, 0.63829E+00_JPRB, 0.65685E+00_JPRB ,&
&  0.67108E+00_JPRB, 0.68228E+00_JPRB, 0.69482E+00_JPRB, 0.72031E+00_JPRB, 0.73732E+00_JPRB, 0.75236E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75939E+00_JPRB, 0.75969E+00_JPRB, 0.60510E+00_JPRB, 0.66877E+00_JPRB, 0.68587E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.69409E+00_JPRB, 0.67759E+00_JPRB, 0.65765E+00_JPRB, 0.64408E+00_JPRB, 0.63746E+00_JPRB, 0.63539E+00_JPRB, 0.63240E+00_JPRB ,&
&  0.62746E+00_JPRB, 0.62771E+00_JPRB, 0.62876E+00_JPRB, 0.62971E+00_JPRB, 0.63025E+00_JPRB, 0.62721E+00_JPRB, 0.61957E+00_JPRB ,&
&  0.61074E+00_JPRB, 0.59937E+00_JPRB, 0.59768E+00_JPRB, 0.59705E+00_JPRB, 0.59074E+00_JPRB, 0.58629E+00_JPRB, 0.58573E+00_JPRB ,&
&  0.57926E+00_JPRB, 0.56923E+00_JPRB, 0.55117E+00_JPRB, 0.54550E+00_JPRB, 0.54836E+00_JPRB, 0.54860E+00_JPRB, 0.55545E+00_JPRB ,&
&  0.53721E+00_JPRB, 0.51358E+00_JPRB, 0.56566E+00_JPRB, 0.59855E+00_JPRB, 0.61241E+00_JPRB, 0.62527E+00_JPRB, 0.63961E+00_JPRB ,&
&  0.63501E+00_JPRB, 0.63650E+00_JPRB, 0.62051E+00_JPRB, 0.59479E+00_JPRB, 0.57727E+00_JPRB, 0.56969E+00_JPRB, 0.56782E+00_JPRB ,&
&  0.57307E+00_JPRB, 0.57720E+00_JPRB, 0.57832E+00_JPRB, 0.57827E+00_JPRB, 0.57948E+00_JPRB, 0.57654E+00_JPRB, 0.57462E+00_JPRB ,&
&  0.57627E+00_JPRB, 0.57986E+00_JPRB, 0.58187E+00_JPRB, 0.58619E+00_JPRB, 0.59517E+00_JPRB, 0.60865E+00_JPRB, 0.62382E+00_JPRB ,&
&  0.64143E+00_JPRB, 0.66966E+00_JPRB, 0.71823E+00_JPRB, 0.76018E+00_JPRB, 0.77488E+00_JPRB, 0.76070E+00_JPRB, 0.73450E+00_JPRB ,&
&  0.71167E+00_JPRB, 0.69730E+00_JPRB, 0.68737E+00_JPRB, 0.67860E+00_JPRB, 0.66590E+00_JPRB, 0.65380E+00_JPRB, 0.64182E+00_JPRB ,&
&  0.63492E+00_JPRB, 0.62535E+00_JPRB, 0.61680E+00_JPRB, 0.60773E+00_JPRB, 0.60074E+00_JPRB, 0.59727E+00_JPRB, 0.59572E+00_JPRB ,&
&  0.59317E+00_JPRB, 0.58647E+00_JPRB, 0.58246E+00_JPRB, 0.58474E+00_JPRB, 0.58840E+00_JPRB, 0.59610E+00_JPRB, 0.60755E+00_JPRB ,&
&  0.62165E+00_JPRB, 0.63479E+00_JPRB, 0.64893E+00_JPRB, 0.67584E+00_JPRB, 0.70288E+00_JPRB, 0.72604E+00_JPRB, 0.73950E+00_JPRB ,&
&  0.74734E+00_JPRB, 0.74010E+00_JPRB, 0.73646E+00_JPRB, 0.72355E+00_JPRB, 0.70739E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.64000E+00_JPRB, 0.61974E+00_JPRB, 0.60381E+00_JPRB, 0.59343E+00_JPRB, 0.58780E+00_JPRB, 0.58721E+00_JPRB, 0.58491E+00_JPRB ,&
&  0.58339E+00_JPRB, 0.58038E+00_JPRB, 0.57848E+00_JPRB, 0.58111E+00_JPRB, 0.58254E+00_JPRB, 0.58324E+00_JPRB, 0.58458E+00_JPRB ,&
&  0.57916E+00_JPRB, 0.56655E+00_JPRB, 0.55650E+00_JPRB, 0.55381E+00_JPRB, 0.55483E+00_JPRB, 0.55087E+00_JPRB, 0.54946E+00_JPRB ,&
&  0.55250E+00_JPRB, 0.54460E+00_JPRB, 0.53133E+00_JPRB, 0.52277E+00_JPRB, 0.52331E+00_JPRB, 0.52094E+00_JPRB, 0.51826E+00_JPRB ,&
&  0.53866E+00_JPRB, 0.48984E+00_JPRB, 0.53688E+00_JPRB, 0.54421E+00_JPRB, 0.54976E+00_JPRB, 0.55274E+00_JPRB, 0.55210E+00_JPRB ,&
&  0.54774E+00_JPRB, 0.54181E+00_JPRB, 0.53564E+00_JPRB, 0.52977E+00_JPRB, 0.52607E+00_JPRB, 0.52150E+00_JPRB, 0.51762E+00_JPRB ,&
&  0.51879E+00_JPRB, 0.52067E+00_JPRB, 0.52592E+00_JPRB, 0.53202E+00_JPRB, 0.53986E+00_JPRB, 0.54271E+00_JPRB, 0.54524E+00_JPRB ,&
&  0.54808E+00_JPRB, 0.54978E+00_JPRB, 0.54976E+00_JPRB, 0.54869E+00_JPRB, 0.54729E+00_JPRB, 0.55189E+00_JPRB, 0.55841E+00_JPRB ,&
&  0.56500E+00_JPRB, 0.57764E+00_JPRB, 0.59834E+00_JPRB, 0.61711E+00_JPRB, 0.62581E+00_JPRB, 0.62282E+00_JPRB, 0.61241E+00_JPRB ,&
&  0.60678E+00_JPRB, 0.60802E+00_JPRB, 0.61020E+00_JPRB, 0.60610E+00_JPRB, 0.59873E+00_JPRB, 0.59171E+00_JPRB, 0.58088E+00_JPRB ,&
&  0.56675E+00_JPRB, 0.55452E+00_JPRB, 0.54665E+00_JPRB, 0.54234E+00_JPRB, 0.54114E+00_JPRB, 0.53933E+00_JPRB, 0.53747E+00_JPRB ,&
&  0.53783E+00_JPRB, 0.53678E+00_JPRB, 0.53735E+00_JPRB, 0.54154E+00_JPRB, 0.54537E+00_JPRB, 0.55124E+00_JPRB, 0.55887E+00_JPRB ,&
&  0.56940E+00_JPRB, 0.58095E+00_JPRB, 0.59218E+00_JPRB, 0.61059E+00_JPRB, 0.63683E+00_JPRB, 0.65597E+00_JPRB, 0.66819E+00_JPRB ,&
&  0.67943E+00_JPRB, 0.68220E+00_JPRB, 0.67629E+00_JPRB, 0.66772E+00_JPRB, 0.65527E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.59576E+00_JPRB, 0.58288E+00_JPRB, 0.57295E+00_JPRB, 0.56561E+00_JPRB, 0.56111E+00_JPRB, 0.56042E+00_JPRB, 0.55849E+00_JPRB ,&
&  0.55663E+00_JPRB, 0.55513E+00_JPRB, 0.54910E+00_JPRB, 0.54290E+00_JPRB, 0.54274E+00_JPRB, 0.54858E+00_JPRB, 0.55321E+00_JPRB ,&
&  0.55049E+00_JPRB, 0.54022E+00_JPRB, 0.53056E+00_JPRB, 0.52861E+00_JPRB, 0.53157E+00_JPRB, 0.53404E+00_JPRB, 0.53372E+00_JPRB ,&
&  0.53543E+00_JPRB, 0.53648E+00_JPRB, 0.52828E+00_JPRB, 0.51160E+00_JPRB, 0.50386E+00_JPRB, 0.49939E+00_JPRB, 0.50382E+00_JPRB ,&
&  0.51091E+00_JPRB, 0.50494E+00_JPRB, 0.51686E+00_JPRB, 0.51449E+00_JPRB, 0.51339E+00_JPRB, 0.50947E+00_JPRB, 0.50267E+00_JPRB ,&
&  0.49727E+00_JPRB, 0.49180E+00_JPRB, 0.48840E+00_JPRB, 0.48682E+00_JPRB, 0.48717E+00_JPRB, 0.48846E+00_JPRB, 0.49297E+00_JPRB ,&
&  0.49411E+00_JPRB, 0.49241E+00_JPRB, 0.49505E+00_JPRB, 0.50215E+00_JPRB, 0.51099E+00_JPRB, 0.51609E+00_JPRB, 0.52072E+00_JPRB ,&
&  0.52452E+00_JPRB, 0.52665E+00_JPRB, 0.52954E+00_JPRB, 0.53035E+00_JPRB, 0.52670E+00_JPRB, 0.52590E+00_JPRB, 0.52571E+00_JPRB ,&
&  0.52805E+00_JPRB, 0.53316E+00_JPRB, 0.54323E+00_JPRB, 0.55560E+00_JPRB, 0.56086E+00_JPRB, 0.55676E+00_JPRB, 0.55138E+00_JPRB ,&
&  0.55184E+00_JPRB, 0.55765E+00_JPRB, 0.56419E+00_JPRB, 0.56825E+00_JPRB, 0.56819E+00_JPRB, 0.56092E+00_JPRB, 0.55154E+00_JPRB ,&
&  0.54132E+00_JPRB, 0.53075E+00_JPRB, 0.52119E+00_JPRB, 0.51500E+00_JPRB, 0.51007E+00_JPRB, 0.50637E+00_JPRB, 0.50401E+00_JPRB ,&
&  0.50300E+00_JPRB, 0.50284E+00_JPRB, 0.50575E+00_JPRB, 0.51183E+00_JPRB, 0.51520E+00_JPRB, 0.52339E+00_JPRB, 0.53377E+00_JPRB ,&
&  0.54433E+00_JPRB, 0.55412E+00_JPRB, 0.56569E+00_JPRB, 0.58241E+00_JPRB, 0.60135E+00_JPRB, 0.61536E+00_JPRB, 0.62641E+00_JPRB ,&
&  0.63359E+00_JPRB, 0.63290E+00_JPRB, 0.62544E+00_JPRB, 0.61697E+00_JPRB, 0.60775E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.55202E+00_JPRB, 0.54426E+00_JPRB, 0.53822E+00_JPRB, 0.53322E+00_JPRB, 0.52915E+00_JPRB, 0.52787E+00_JPRB, 0.52633E+00_JPRB ,&
&  0.52463E+00_JPRB, 0.52228E+00_JPRB, 0.52108E+00_JPRB, 0.51803E+00_JPRB, 0.51538E+00_JPRB, 0.51842E+00_JPRB, 0.52403E+00_JPRB ,&
&  0.52273E+00_JPRB, 0.51614E+00_JPRB, 0.51097E+00_JPRB, 0.51101E+00_JPRB, 0.51448E+00_JPRB, 0.51708E+00_JPRB, 0.52005E+00_JPRB ,&
&  0.51989E+00_JPRB, 0.51922E+00_JPRB, 0.51771E+00_JPRB, 0.50234E+00_JPRB, 0.48756E+00_JPRB, 0.48033E+00_JPRB, 0.47614E+00_JPRB ,&
&  0.47548E+00_JPRB, 0.47995E+00_JPRB, 0.46418E+00_JPRB, 0.47344E+00_JPRB, 0.46934E+00_JPRB, 0.46319E+00_JPRB, 0.45979E+00_JPRB ,&
&  0.45777E+00_JPRB, 0.45543E+00_JPRB, 0.45382E+00_JPRB, 0.45589E+00_JPRB, 0.45863E+00_JPRB, 0.45956E+00_JPRB, 0.46143E+00_JPRB ,&
&  0.46126E+00_JPRB, 0.46384E+00_JPRB, 0.46608E+00_JPRB, 0.47125E+00_JPRB, 0.47524E+00_JPRB, 0.47779E+00_JPRB, 0.48102E+00_JPRB ,&
&  0.48550E+00_JPRB, 0.48966E+00_JPRB, 0.49364E+00_JPRB, 0.49697E+00_JPRB, 0.49829E+00_JPRB, 0.49800E+00_JPRB, 0.49384E+00_JPRB ,&
&  0.49048E+00_JPRB, 0.49030E+00_JPRB, 0.49414E+00_JPRB, 0.50302E+00_JPRB, 0.51096E+00_JPRB, 0.51224E+00_JPRB, 0.51142E+00_JPRB ,&
&  0.51254E+00_JPRB, 0.51703E+00_JPRB, 0.51884E+00_JPRB, 0.51852E+00_JPRB, 0.51713E+00_JPRB, 0.51482E+00_JPRB, 0.51041E+00_JPRB ,&
&  0.50424E+00_JPRB, 0.49902E+00_JPRB, 0.49360E+00_JPRB, 0.48222E+00_JPRB, 0.47349E+00_JPRB, 0.47247E+00_JPRB, 0.47387E+00_JPRB ,&
&  0.47326E+00_JPRB, 0.47160E+00_JPRB, 0.47264E+00_JPRB, 0.47819E+00_JPRB, 0.47816E+00_JPRB, 0.48479E+00_JPRB, 0.49849E+00_JPRB ,&
&  0.51257E+00_JPRB, 0.52491E+00_JPRB, 0.53459E+00_JPRB, 0.54462E+00_JPRB, 0.55455E+00_JPRB, 0.56214E+00_JPRB, 0.56879E+00_JPRB ,&
&  0.57211E+00_JPRB, 0.57072E+00_JPRB, 0.56558E+00_JPRB, 0.56239E+00_JPRB, 0.55949E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.52673E+00_JPRB, 0.52504E+00_JPRB, 0.52328E+00_JPRB, 0.52111E+00_JPRB, 0.51810E+00_JPRB, 0.51547E+00_JPRB, 0.51104E+00_JPRB ,&
&  0.50632E+00_JPRB, 0.50253E+00_JPRB, 0.50098E+00_JPRB, 0.50059E+00_JPRB, 0.50212E+00_JPRB, 0.50635E+00_JPRB, 0.51324E+00_JPRB ,&
&  0.51342E+00_JPRB, 0.50681E+00_JPRB, 0.50379E+00_JPRB, 0.50395E+00_JPRB, 0.50643E+00_JPRB, 0.50819E+00_JPRB, 0.51094E+00_JPRB ,&
&  0.51272E+00_JPRB, 0.51328E+00_JPRB, 0.51249E+00_JPRB, 0.50553E+00_JPRB, 0.48897E+00_JPRB, 0.47897E+00_JPRB, 0.47013E+00_JPRB ,&
&  0.46485E+00_JPRB, 0.46030E+00_JPRB, 0.45744E+00_JPRB, 0.45635E+00_JPRB, 0.45526E+00_JPRB, 0.45342E+00_JPRB, 0.45331E+00_JPRB ,&
&  0.45139E+00_JPRB, 0.44887E+00_JPRB, 0.44749E+00_JPRB, 0.44806E+00_JPRB, 0.44979E+00_JPRB, 0.45141E+00_JPRB, 0.45136E+00_JPRB ,&
&  0.45242E+00_JPRB, 0.45619E+00_JPRB, 0.45733E+00_JPRB, 0.45759E+00_JPRB, 0.45748E+00_JPRB, 0.45707E+00_JPRB, 0.45721E+00_JPRB ,&
&  0.45956E+00_JPRB, 0.46466E+00_JPRB, 0.46898E+00_JPRB, 0.47332E+00_JPRB, 0.47517E+00_JPRB, 0.47604E+00_JPRB, 0.47601E+00_JPRB ,&
&  0.47290E+00_JPRB, 0.46891E+00_JPRB, 0.46751E+00_JPRB, 0.47039E+00_JPRB, 0.47627E+00_JPRB, 0.48239E+00_JPRB, 0.48564E+00_JPRB ,&
&  0.48821E+00_JPRB, 0.48949E+00_JPRB, 0.48771E+00_JPRB, 0.48696E+00_JPRB, 0.48466E+00_JPRB, 0.48040E+00_JPRB, 0.47800E+00_JPRB ,&
&  0.47560E+00_JPRB, 0.47340E+00_JPRB, 0.47135E+00_JPRB, 0.46241E+00_JPRB, 0.45481E+00_JPRB, 0.45022E+00_JPRB, 0.44720E+00_JPRB ,&
&  0.44605E+00_JPRB, 0.44511E+00_JPRB, 0.44480E+00_JPRB, 0.44604E+00_JPRB, 0.44808E+00_JPRB, 0.45468E+00_JPRB, 0.46782E+00_JPRB ,&
&  0.48727E+00_JPRB, 0.50532E+00_JPRB, 0.51516E+00_JPRB, 0.52214E+00_JPRB, 0.52657E+00_JPRB, 0.52879E+00_JPRB, 0.53104E+00_JPRB ,&
&  0.53455E+00_JPRB, 0.53569E+00_JPRB, 0.53347E+00_JPRB, 0.53069E+00_JPRB, 0.52904E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.49951E+00_JPRB, 0.49966E+00_JPRB, 0.49821E+00_JPRB, 0.49688E+00_JPRB, 0.49543E+00_JPRB, 0.49413E+00_JPRB, 0.49214E+00_JPRB ,&
&  0.48731E+00_JPRB, 0.48107E+00_JPRB, 0.47749E+00_JPRB, 0.47593E+00_JPRB, 0.48168E+00_JPRB, 0.49013E+00_JPRB, 0.49766E+00_JPRB ,&
&  0.49688E+00_JPRB, 0.48790E+00_JPRB, 0.48518E+00_JPRB, 0.48719E+00_JPRB, 0.48978E+00_JPRB, 0.49260E+00_JPRB, 0.49639E+00_JPRB ,&
&  0.50054E+00_JPRB, 0.50477E+00_JPRB, 0.50663E+00_JPRB, 0.50409E+00_JPRB, 0.49762E+00_JPRB, 0.48401E+00_JPRB, 0.47210E+00_JPRB ,&
&  0.46418E+00_JPRB, 0.45800E+00_JPRB, 0.45444E+00_JPRB, 0.45344E+00_JPRB, 0.45121E+00_JPRB, 0.44741E+00_JPRB, 0.44207E+00_JPRB ,&
&  0.43742E+00_JPRB, 0.43527E+00_JPRB, 0.43747E+00_JPRB, 0.43919E+00_JPRB, 0.44008E+00_JPRB, 0.43982E+00_JPRB, 0.43693E+00_JPRB ,&
&  0.43537E+00_JPRB, 0.43496E+00_JPRB, 0.43476E+00_JPRB, 0.43644E+00_JPRB, 0.43423E+00_JPRB, 0.43237E+00_JPRB, 0.43451E+00_JPRB ,&
&  0.43907E+00_JPRB, 0.44305E+00_JPRB, 0.44667E+00_JPRB, 0.44944E+00_JPRB, 0.45059E+00_JPRB, 0.45271E+00_JPRB, 0.45406E+00_JPRB ,&
&  0.45315E+00_JPRB, 0.44959E+00_JPRB, 0.44627E+00_JPRB, 0.44677E+00_JPRB, 0.44911E+00_JPRB, 0.45177E+00_JPRB, 0.45530E+00_JPRB ,&
&  0.45599E+00_JPRB, 0.45101E+00_JPRB, 0.44600E+00_JPRB, 0.44160E+00_JPRB, 0.43758E+00_JPRB, 0.43450E+00_JPRB, 0.43300E+00_JPRB ,&
&  0.43261E+00_JPRB, 0.42957E+00_JPRB, 0.42365E+00_JPRB, 0.41876E+00_JPRB, 0.41816E+00_JPRB, 0.41741E+00_JPRB, 0.41361E+00_JPRB ,&
&  0.41401E+00_JPRB, 0.41655E+00_JPRB, 0.41545E+00_JPRB, 0.41357E+00_JPRB, 0.41557E+00_JPRB, 0.42104E+00_JPRB, 0.43224E+00_JPRB ,&
&  0.44717E+00_JPRB, 0.46429E+00_JPRB, 0.48005E+00_JPRB, 0.49361E+00_JPRB, 0.50157E+00_JPRB, 0.50342E+00_JPRB, 0.50162E+00_JPRB ,&
&  0.50088E+00_JPRB, 0.50157E+00_JPRB, 0.50058E+00_JPRB, 0.49884E+00_JPRB, 0.49875E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.47199E+00_JPRB, 0.47184E+00_JPRB, 0.47202E+00_JPRB, 0.47312E+00_JPRB, 0.47384E+00_JPRB, 0.47282E+00_JPRB, 0.47254E+00_JPRB ,&
&  0.46939E+00_JPRB, 0.46482E+00_JPRB, 0.46203E+00_JPRB, 0.46358E+00_JPRB, 0.47275E+00_JPRB, 0.48446E+00_JPRB, 0.49153E+00_JPRB ,&
&  0.48719E+00_JPRB, 0.47485E+00_JPRB, 0.47189E+00_JPRB, 0.47421E+00_JPRB, 0.47644E+00_JPRB, 0.48058E+00_JPRB, 0.48670E+00_JPRB ,&
&  0.49228E+00_JPRB, 0.49610E+00_JPRB, 0.50030E+00_JPRB, 0.50202E+00_JPRB, 0.50228E+00_JPRB, 0.49522E+00_JPRB, 0.48406E+00_JPRB ,&
&  0.47332E+00_JPRB, 0.46555E+00_JPRB, 0.46076E+00_JPRB, 0.45721E+00_JPRB, 0.45214E+00_JPRB, 0.44701E+00_JPRB, 0.44036E+00_JPRB ,&
&  0.43712E+00_JPRB, 0.43683E+00_JPRB, 0.43774E+00_JPRB, 0.43588E+00_JPRB, 0.43334E+00_JPRB, 0.42822E+00_JPRB, 0.42398E+00_JPRB ,&
&  0.42237E+00_JPRB, 0.42366E+00_JPRB, 0.42501E+00_JPRB, 0.42420E+00_JPRB, 0.42089E+00_JPRB, 0.41855E+00_JPRB, 0.41998E+00_JPRB ,&
&  0.42587E+00_JPRB, 0.42571E+00_JPRB, 0.42473E+00_JPRB, 0.42552E+00_JPRB, 0.42771E+00_JPRB, 0.42968E+00_JPRB, 0.42911E+00_JPRB ,&
&  0.42845E+00_JPRB, 0.42841E+00_JPRB, 0.42870E+00_JPRB, 0.43059E+00_JPRB, 0.43070E+00_JPRB, 0.42859E+00_JPRB, 0.42307E+00_JPRB ,&
&  0.41468E+00_JPRB, 0.40412E+00_JPRB, 0.39978E+00_JPRB, 0.39669E+00_JPRB, 0.39374E+00_JPRB, 0.39314E+00_JPRB, 0.39455E+00_JPRB ,&
&  0.39320E+00_JPRB, 0.38704E+00_JPRB, 0.38424E+00_JPRB, 0.38478E+00_JPRB, 0.38385E+00_JPRB, 0.38610E+00_JPRB, 0.38994E+00_JPRB ,&
&  0.40017E+00_JPRB, 0.40589E+00_JPRB, 0.40759E+00_JPRB, 0.40549E+00_JPRB, 0.40583E+00_JPRB, 0.40761E+00_JPRB, 0.41198E+00_JPRB ,&
&  0.41856E+00_JPRB, 0.42565E+00_JPRB, 0.43295E+00_JPRB, 0.44103E+00_JPRB, 0.45112E+00_JPRB, 0.46090E+00_JPRB, 0.46462E+00_JPRB ,&
&  0.46482E+00_JPRB, 0.46515E+00_JPRB, 0.46639E+00_JPRB, 0.46791E+00_JPRB, 0.47080E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.43800E+00_JPRB, 0.44026E+00_JPRB, 0.44252E+00_JPRB, 0.44405E+00_JPRB, 0.44671E+00_JPRB, 0.45154E+00_JPRB, 0.45347E+00_JPRB ,&
&  0.45157E+00_JPRB, 0.44943E+00_JPRB, 0.44931E+00_JPRB, 0.45276E+00_JPRB, 0.45946E+00_JPRB, 0.46646E+00_JPRB, 0.47419E+00_JPRB ,&
&  0.47014E+00_JPRB, 0.45769E+00_JPRB, 0.45316E+00_JPRB, 0.45520E+00_JPRB, 0.46005E+00_JPRB, 0.46545E+00_JPRB, 0.47213E+00_JPRB ,&
&  0.47848E+00_JPRB, 0.48266E+00_JPRB, 0.48703E+00_JPRB, 0.49099E+00_JPRB, 0.49356E+00_JPRB, 0.49184E+00_JPRB, 0.48742E+00_JPRB ,&
&  0.48210E+00_JPRB, 0.47672E+00_JPRB, 0.47198E+00_JPRB, 0.45628E+00_JPRB, 0.45368E+00_JPRB, 0.44469E+00_JPRB, 0.43720E+00_JPRB ,&
&  0.43202E+00_JPRB, 0.42774E+00_JPRB, 0.42503E+00_JPRB, 0.42198E+00_JPRB, 0.41771E+00_JPRB, 0.41049E+00_JPRB, 0.40845E+00_JPRB ,&
&  0.40757E+00_JPRB, 0.40868E+00_JPRB, 0.40412E+00_JPRB, 0.40173E+00_JPRB, 0.39909E+00_JPRB, 0.39703E+00_JPRB, 0.40130E+00_JPRB ,&
&  0.40646E+00_JPRB, 0.40397E+00_JPRB, 0.40003E+00_JPRB, 0.39971E+00_JPRB, 0.40399E+00_JPRB, 0.40562E+00_JPRB, 0.40659E+00_JPRB ,&
&  0.40837E+00_JPRB, 0.40889E+00_JPRB, 0.40427E+00_JPRB, 0.39916E+00_JPRB, 0.39054E+00_JPRB, 0.37974E+00_JPRB, 0.37683E+00_JPRB ,&
&  0.37385E+00_JPRB, 0.36809E+00_JPRB, 0.36464E+00_JPRB, 0.36355E+00_JPRB, 0.36318E+00_JPRB, 0.36401E+00_JPRB, 0.36411E+00_JPRB ,&
&  0.36409E+00_JPRB, 0.36058E+00_JPRB, 0.35542E+00_JPRB, 0.35096E+00_JPRB, 0.34489E+00_JPRB, 0.34762E+00_JPRB, 0.35751E+00_JPRB ,&
&  0.36968E+00_JPRB, 0.37708E+00_JPRB, 0.38190E+00_JPRB, 0.38536E+00_JPRB, 0.38847E+00_JPRB, 0.38993E+00_JPRB, 0.39231E+00_JPRB ,&
&  0.39548E+00_JPRB, 0.39694E+00_JPRB, 0.39561E+00_JPRB, 0.39565E+00_JPRB, 0.39873E+00_JPRB, 0.40388E+00_JPRB, 0.41010E+00_JPRB ,&
&  0.41627E+00_JPRB, 0.42145E+00_JPRB, 0.42577E+00_JPRB, 0.43119E+00_JPRB, 0.43472E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.41044E+00_JPRB, 0.41477E+00_JPRB, 0.41885E+00_JPRB, 0.42332E+00_JPRB, 0.42967E+00_JPRB, 0.43294E+00_JPRB, 0.43396E+00_JPRB ,&
&  0.43524E+00_JPRB, 0.43576E+00_JPRB, 0.43726E+00_JPRB, 0.44160E+00_JPRB, 0.44771E+00_JPRB, 0.45211E+00_JPRB, 0.45787E+00_JPRB ,&
&  0.45795E+00_JPRB, 0.44926E+00_JPRB, 0.44136E+00_JPRB, 0.44108E+00_JPRB, 0.44659E+00_JPRB, 0.45233E+00_JPRB, 0.45958E+00_JPRB ,&
&  0.46675E+00_JPRB, 0.47197E+00_JPRB, 0.47494E+00_JPRB, 0.47644E+00_JPRB, 0.47841E+00_JPRB, 0.47802E+00_JPRB, 0.47497E+00_JPRB ,&
&  0.47076E+00_JPRB, 0.46177E+00_JPRB, 0.44228E+00_JPRB, 0.43953E+00_JPRB, 0.43079E+00_JPRB, 0.42318E+00_JPRB, 0.41790E+00_JPRB ,&
&  0.41363E+00_JPRB, 0.41014E+00_JPRB, 0.40617E+00_JPRB, 0.40274E+00_JPRB, 0.39881E+00_JPRB, 0.39180E+00_JPRB, 0.38823E+00_JPRB ,&
&  0.38500E+00_JPRB, 0.38233E+00_JPRB, 0.37971E+00_JPRB, 0.37994E+00_JPRB, 0.38035E+00_JPRB, 0.38318E+00_JPRB, 0.38457E+00_JPRB ,&
&  0.38143E+00_JPRB, 0.37774E+00_JPRB, 0.37491E+00_JPRB, 0.37654E+00_JPRB, 0.37880E+00_JPRB, 0.38036E+00_JPRB, 0.38094E+00_JPRB ,&
&  0.37844E+00_JPRB, 0.37383E+00_JPRB, 0.36904E+00_JPRB, 0.36351E+00_JPRB, 0.35804E+00_JPRB, 0.33236E+00_JPRB, 0.32114E+00_JPRB ,&
&  0.31791E+00_JPRB, 0.32693E+00_JPRB, 0.33929E+00_JPRB, 0.33815E+00_JPRB, 0.34037E+00_JPRB, 0.34050E+00_JPRB, 0.33798E+00_JPRB ,&
&  0.32902E+00_JPRB, 0.31551E+00_JPRB, 0.30648E+00_JPRB, 0.30247E+00_JPRB, 0.29961E+00_JPRB, 0.30111E+00_JPRB, 0.30728E+00_JPRB ,&
&  0.31745E+00_JPRB, 0.32915E+00_JPRB, 0.33801E+00_JPRB, 0.34205E+00_JPRB, 0.34096E+00_JPRB, 0.33787E+00_JPRB, 0.33375E+00_JPRB ,&
&  0.32929E+00_JPRB, 0.33507E+00_JPRB, 0.34937E+00_JPRB, 0.36935E+00_JPRB, 0.37201E+00_JPRB, 0.37708E+00_JPRB, 0.38051E+00_JPRB ,&
&  0.38574E+00_JPRB, 0.39231E+00_JPRB, 0.39798E+00_JPRB, 0.40393E+00_JPRB, 0.40757E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.38878E+00_JPRB, 0.39375E+00_JPRB, 0.40052E+00_JPRB, 0.40805E+00_JPRB, 0.41345E+00_JPRB, 0.41594E+00_JPRB, 0.41857E+00_JPRB ,&
&  0.42193E+00_JPRB, 0.42334E+00_JPRB, 0.42439E+00_JPRB, 0.42717E+00_JPRB, 0.43281E+00_JPRB, 0.43590E+00_JPRB, 0.44075E+00_JPRB ,&
&  0.44269E+00_JPRB, 0.44052E+00_JPRB, 0.43371E+00_JPRB, 0.43173E+00_JPRB, 0.43530E+00_JPRB, 0.44008E+00_JPRB, 0.44534E+00_JPRB ,&
&  0.45079E+00_JPRB, 0.45578E+00_JPRB, 0.45815E+00_JPRB, 0.45881E+00_JPRB, 0.45898E+00_JPRB, 0.45882E+00_JPRB, 0.45615E+00_JPRB ,&
&  0.45092E+00_JPRB, 0.41140E+00_JPRB, 0.38486E+00_JPRB, 0.38822E+00_JPRB, 0.39976E+00_JPRB, 0.39410E+00_JPRB, 0.39254E+00_JPRB ,&
&  0.39096E+00_JPRB, 0.38741E+00_JPRB, 0.38212E+00_JPRB, 0.37781E+00_JPRB, 0.37246E+00_JPRB, 0.36556E+00_JPRB, 0.36004E+00_JPRB ,&
&  0.35873E+00_JPRB, 0.36360E+00_JPRB, 0.36513E+00_JPRB, 0.36727E+00_JPRB, 0.36594E+00_JPRB, 0.36782E+00_JPRB, 0.36655E+00_JPRB ,&
&  0.36108E+00_JPRB, 0.35485E+00_JPRB, 0.35278E+00_JPRB, 0.35239E+00_JPRB, 0.35348E+00_JPRB, 0.35615E+00_JPRB, 0.35599E+00_JPRB ,&
&  0.34289E+00_JPRB, 0.33135E+00_JPRB, 0.31282E+00_JPRB, 0.29485E+00_JPRB, 0.28173E+00_JPRB, 0.27578E+00_JPRB, 0.27146E+00_JPRB ,&
&  0.27063E+00_JPRB, 0.27653E+00_JPRB, 0.29329E+00_JPRB, 0.31365E+00_JPRB, 0.32744E+00_JPRB, 0.30917E+00_JPRB, 0.29356E+00_JPRB ,&
&  0.28404E+00_JPRB, 0.27899E+00_JPRB, 0.27468E+00_JPRB, 0.27316E+00_JPRB, 0.27361E+00_JPRB, 0.27595E+00_JPRB, 0.27930E+00_JPRB ,&
&  0.28404E+00_JPRB, 0.28846E+00_JPRB, 0.29126E+00_JPRB, 0.29261E+00_JPRB, 0.29336E+00_JPRB, 0.29412E+00_JPRB, 0.29352E+00_JPRB ,&
&  0.29336E+00_JPRB, 0.29676E+00_JPRB, 0.30364E+00_JPRB, 0.31249E+00_JPRB, 0.32071E+00_JPRB, 0.32986E+00_JPRB, 0.34274E+00_JPRB ,&
&  0.35698E+00_JPRB, 0.36869E+00_JPRB, 0.37286E+00_JPRB, 0.37888E+00_JPRB, 0.38412E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.37997E+00_JPRB, 0.38671E+00_JPRB, 0.39167E+00_JPRB, 0.40152E+00_JPRB, 0.40437E+00_JPRB, 0.40728E+00_JPRB, 0.40973E+00_JPRB ,&
&  0.41598E+00_JPRB, 0.41960E+00_JPRB, 0.41743E+00_JPRB, 0.41687E+00_JPRB, 0.41604E+00_JPRB, 0.41486E+00_JPRB, 0.41591E+00_JPRB ,&
&  0.41605E+00_JPRB, 0.41819E+00_JPRB, 0.41584E+00_JPRB, 0.41308E+00_JPRB, 0.41503E+00_JPRB, 0.41985E+00_JPRB, 0.42309E+00_JPRB ,&
&  0.41791E+00_JPRB, 0.41648E+00_JPRB, 0.42500E+00_JPRB, 0.42707E+00_JPRB, 0.42826E+00_JPRB, 0.43020E+00_JPRB, 0.42993E+00_JPRB ,&
&  0.41606E+00_JPRB, 0.39062E+00_JPRB, 0.36305E+00_JPRB, 0.36468E+00_JPRB, 0.37933E+00_JPRB, 0.37364E+00_JPRB, 0.37197E+00_JPRB ,&
&  0.36906E+00_JPRB, 0.36452E+00_JPRB, 0.36019E+00_JPRB, 0.35812E+00_JPRB, 0.35534E+00_JPRB, 0.35067E+00_JPRB, 0.34908E+00_JPRB ,&
&  0.34797E+00_JPRB, 0.35659E+00_JPRB, 0.35177E+00_JPRB, 0.32969E+00_JPRB, 0.31311E+00_JPRB, 0.30404E+00_JPRB, 0.29246E+00_JPRB ,&
&  0.27898E+00_JPRB, 0.27115E+00_JPRB, 0.26994E+00_JPRB, 0.26812E+00_JPRB, 0.26964E+00_JPRB, 0.27076E+00_JPRB, 0.27189E+00_JPRB ,&
&  0.27274E+00_JPRB, 0.27253E+00_JPRB, 0.26792E+00_JPRB, 0.26196E+00_JPRB, 0.25871E+00_JPRB, 0.25871E+00_JPRB, 0.26143E+00_JPRB ,&
&  0.26734E+00_JPRB, 0.27361E+00_JPRB, 0.28801E+00_JPRB, 0.29578E+00_JPRB, 0.28711E+00_JPRB, 0.28169E+00_JPRB, 0.27554E+00_JPRB ,&
&  0.26506E+00_JPRB, 0.26106E+00_JPRB, 0.25773E+00_JPRB, 0.25587E+00_JPRB, 0.25674E+00_JPRB, 0.26009E+00_JPRB, 0.26188E+00_JPRB ,&
&  0.26447E+00_JPRB, 0.26721E+00_JPRB, 0.26833E+00_JPRB, 0.26923E+00_JPRB, 0.26998E+00_JPRB, 0.27189E+00_JPRB, 0.27383E+00_JPRB ,&
&  0.27546E+00_JPRB, 0.27839E+00_JPRB, 0.28194E+00_JPRB, 0.28668E+00_JPRB, 0.29085E+00_JPRB, 0.29332E+00_JPRB, 0.30056E+00_JPRB ,&
&  0.30903E+00_JPRB, 0.32650E+00_JPRB, 0.35498E+00_JPRB, 0.37958E+00_JPRB, 0.37525E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.38180E+00_JPRB, 0.38610E+00_JPRB, 0.38675E+00_JPRB, 0.39002E+00_JPRB, 0.39486E+00_JPRB, 0.40050E+00_JPRB, 0.40522E+00_JPRB ,&
&  0.41043E+00_JPRB, 0.41017E+00_JPRB, 0.40507E+00_JPRB, 0.39788E+00_JPRB, 0.39188E+00_JPRB, 0.37513E+00_JPRB, 0.36636E+00_JPRB ,&
&  0.35917E+00_JPRB, 0.35888E+00_JPRB, 0.35941E+00_JPRB, 0.36112E+00_JPRB, 0.36218E+00_JPRB, 0.36722E+00_JPRB, 0.36870E+00_JPRB ,&
&  0.36609E+00_JPRB, 0.36088E+00_JPRB, 0.35558E+00_JPRB, 0.35461E+00_JPRB, 0.35720E+00_JPRB, 0.35911E+00_JPRB, 0.35615E+00_JPRB ,&
&  0.35247E+00_JPRB, 0.35222E+00_JPRB, 0.35279E+00_JPRB, 0.35423E+00_JPRB, 0.36383E+00_JPRB, 0.35884E+00_JPRB, 0.35432E+00_JPRB ,&
&  0.34719E+00_JPRB, 0.34106E+00_JPRB, 0.33683E+00_JPRB, 0.33714E+00_JPRB, 0.34020E+00_JPRB, 0.33970E+00_JPRB, 0.34235E+00_JPRB ,&
&  0.33080E+00_JPRB, 0.32113E+00_JPRB, 0.30604E+00_JPRB, 0.29408E+00_JPRB, 0.28207E+00_JPRB, 0.27387E+00_JPRB, 0.26627E+00_JPRB ,&
&  0.25979E+00_JPRB, 0.25439E+00_JPRB, 0.24930E+00_JPRB, 0.24765E+00_JPRB, 0.24854E+00_JPRB, 0.24859E+00_JPRB, 0.24814E+00_JPRB ,&
&  0.24868E+00_JPRB, 0.24964E+00_JPRB, 0.25038E+00_JPRB, 0.25139E+00_JPRB, 0.25301E+00_JPRB, 0.25568E+00_JPRB, 0.25928E+00_JPRB ,&
&  0.26489E+00_JPRB, 0.26937E+00_JPRB, 0.27178E+00_JPRB, 0.27298E+00_JPRB, 0.26983E+00_JPRB, 0.26447E+00_JPRB, 0.25904E+00_JPRB ,&
&  0.25356E+00_JPRB, 0.25100E+00_JPRB, 0.24869E+00_JPRB, 0.24758E+00_JPRB, 0.24738E+00_JPRB, 0.24828E+00_JPRB, 0.25016E+00_JPRB ,&
&  0.25206E+00_JPRB, 0.25445E+00_JPRB, 0.25603E+00_JPRB, 0.25790E+00_JPRB, 0.26015E+00_JPRB, 0.26210E+00_JPRB, 0.26412E+00_JPRB ,&
&  0.26637E+00_JPRB, 0.26813E+00_JPRB, 0.27116E+00_JPRB, 0.27490E+00_JPRB, 0.28203E+00_JPRB, 0.29040E+00_JPRB, 0.30531E+00_JPRB ,&
&  0.32279E+00_JPRB, 0.35514E+00_JPRB, 0.36537E+00_JPRB, 0.37278E+00_JPRB, 0.38003E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.37790E+00_JPRB, 0.38042E+00_JPRB, 0.38438E+00_JPRB, 0.38989E+00_JPRB, 0.39678E+00_JPRB, 0.40307E+00_JPRB, 0.40457E+00_JPRB ,&
&  0.39514E+00_JPRB, 0.38226E+00_JPRB, 0.36403E+00_JPRB, 0.34963E+00_JPRB, 0.33564E+00_JPRB, 0.32524E+00_JPRB, 0.31679E+00_JPRB ,&
&  0.31137E+00_JPRB, 0.31098E+00_JPRB, 0.31138E+00_JPRB, 0.31262E+00_JPRB, 0.31157E+00_JPRB, 0.31261E+00_JPRB, 0.31291E+00_JPRB ,&
&  0.31165E+00_JPRB, 0.31150E+00_JPRB, 0.31141E+00_JPRB, 0.31488E+00_JPRB, 0.32171E+00_JPRB, 0.32800E+00_JPRB, 0.33424E+00_JPRB ,&
&  0.34156E+00_JPRB, 0.34362E+00_JPRB, 0.34780E+00_JPRB, 0.34648E+00_JPRB, 0.34298E+00_JPRB, 0.33847E+00_JPRB, 0.33216E+00_JPRB ,&
&  0.32770E+00_JPRB, 0.32375E+00_JPRB, 0.32247E+00_JPRB, 0.32292E+00_JPRB, 0.32304E+00_JPRB, 0.32519E+00_JPRB, 0.31925E+00_JPRB ,&
&  0.31195E+00_JPRB, 0.30136E+00_JPRB, 0.28913E+00_JPRB, 0.28007E+00_JPRB, 0.27141E+00_JPRB, 0.26519E+00_JPRB, 0.25965E+00_JPRB ,&
&  0.25453E+00_JPRB, 0.25110E+00_JPRB, 0.24752E+00_JPRB, 0.24491E+00_JPRB, 0.24408E+00_JPRB, 0.24226E+00_JPRB, 0.24124E+00_JPRB ,&
&  0.24082E+00_JPRB, 0.24125E+00_JPRB, 0.24291E+00_JPRB, 0.24442E+00_JPRB, 0.24649E+00_JPRB, 0.24882E+00_JPRB, 0.25082E+00_JPRB ,&
&  0.25347E+00_JPRB, 0.25561E+00_JPRB, 0.25586E+00_JPRB, 0.25526E+00_JPRB, 0.25312E+00_JPRB, 0.25032E+00_JPRB, 0.24785E+00_JPRB ,&
&  0.24534E+00_JPRB, 0.24357E+00_JPRB, 0.24291E+00_JPRB, 0.24204E+00_JPRB, 0.24249E+00_JPRB, 0.24368E+00_JPRB, 0.24461E+00_JPRB ,&
&  0.24641E+00_JPRB, 0.24788E+00_JPRB, 0.24961E+00_JPRB, 0.25247E+00_JPRB, 0.25515E+00_JPRB, 0.25749E+00_JPRB, 0.25988E+00_JPRB ,&
&  0.26004E+00_JPRB, 0.26177E+00_JPRB, 0.26414E+00_JPRB, 0.26841E+00_JPRB, 0.28085E+00_JPRB, 0.28959E+00_JPRB, 0.30970E+00_JPRB ,&
&  0.32701E+00_JPRB, 0.36213E+00_JPRB, 0.36207E+00_JPRB, 0.36791E+00_JPRB, 0.37427E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.37253E+00_JPRB, 0.37926E+00_JPRB, 0.38607E+00_JPRB, 0.39125E+00_JPRB, 0.39490E+00_JPRB, 0.39803E+00_JPRB, 0.39639E+00_JPRB ,&
&  0.39034E+00_JPRB, 0.37537E+00_JPRB, 0.36402E+00_JPRB, 0.35286E+00_JPRB, 0.34348E+00_JPRB, 0.33454E+00_JPRB, 0.32621E+00_JPRB ,&
&  0.31817E+00_JPRB, 0.31148E+00_JPRB, 0.30592E+00_JPRB, 0.30105E+00_JPRB, 0.29442E+00_JPRB, 0.28858E+00_JPRB, 0.28389E+00_JPRB ,&
&  0.28001E+00_JPRB, 0.27987E+00_JPRB, 0.28220E+00_JPRB, 0.28662E+00_JPRB, 0.29488E+00_JPRB, 0.30319E+00_JPRB, 0.31248E+00_JPRB ,&
&  0.32672E+00_JPRB, 0.33028E+00_JPRB, 0.32922E+00_JPRB, 0.32854E+00_JPRB, 0.32357E+00_JPRB, 0.31739E+00_JPRB, 0.30818E+00_JPRB ,&
&  0.30925E+00_JPRB, 0.30729E+00_JPRB, 0.31467E+00_JPRB, 0.31485E+00_JPRB, 0.31360E+00_JPRB, 0.31382E+00_JPRB, 0.30640E+00_JPRB ,&
&  0.30098E+00_JPRB, 0.29170E+00_JPRB, 0.28453E+00_JPRB, 0.27703E+00_JPRB, 0.26959E+00_JPRB, 0.26517E+00_JPRB, 0.25942E+00_JPRB ,&
&  0.25517E+00_JPRB, 0.25199E+00_JPRB, 0.24788E+00_JPRB, 0.24528E+00_JPRB, 0.24302E+00_JPRB, 0.24069E+00_JPRB, 0.23947E+00_JPRB ,&
&  0.23855E+00_JPRB, 0.23821E+00_JPRB, 0.23833E+00_JPRB, 0.23853E+00_JPRB, 0.23901E+00_JPRB, 0.23992E+00_JPRB, 0.24062E+00_JPRB ,&
&  0.24139E+00_JPRB, 0.24166E+00_JPRB, 0.24130E+00_JPRB, 0.24087E+00_JPRB, 0.23973E+00_JPRB, 0.23897E+00_JPRB, 0.23798E+00_JPRB ,&
&  0.23738E+00_JPRB, 0.23771E+00_JPRB, 0.23802E+00_JPRB, 0.23873E+00_JPRB, 0.23946E+00_JPRB, 0.24105E+00_JPRB, 0.24254E+00_JPRB ,&
&  0.24413E+00_JPRB, 0.24611E+00_JPRB, 0.24774E+00_JPRB, 0.24981E+00_JPRB, 0.25228E+00_JPRB, 0.25394E+00_JPRB, 0.25684E+00_JPRB ,&
&  0.25968E+00_JPRB, 0.26323E+00_JPRB, 0.26929E+00_JPRB, 0.27600E+00_JPRB, 0.28503E+00_JPRB, 0.29608E+00_JPRB, 0.31002E+00_JPRB ,&
&  0.32779E+00_JPRB, 0.35266E+00_JPRB, 0.35430E+00_JPRB, 0.36195E+00_JPRB, 0.36718E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.36232E+00_JPRB, 0.36725E+00_JPRB, 0.37144E+00_JPRB, 0.37520E+00_JPRB, 0.37853E+00_JPRB, 0.37944E+00_JPRB, 0.37853E+00_JPRB ,&
&  0.36940E+00_JPRB, 0.36270E+00_JPRB, 0.35450E+00_JPRB, 0.34716E+00_JPRB, 0.34037E+00_JPRB, 0.33221E+00_JPRB, 0.32528E+00_JPRB ,&
&  0.31817E+00_JPRB, 0.31106E+00_JPRB, 0.30432E+00_JPRB, 0.29641E+00_JPRB, 0.28817E+00_JPRB, 0.28028E+00_JPRB, 0.27549E+00_JPRB ,&
&  0.27275E+00_JPRB, 0.27265E+00_JPRB, 0.27567E+00_JPRB, 0.27955E+00_JPRB, 0.28455E+00_JPRB, 0.29097E+00_JPRB, 0.29787E+00_JPRB ,&
&  0.30221E+00_JPRB, 0.30457E+00_JPRB, 0.30407E+00_JPRB, 0.30248E+00_JPRB, 0.30039E+00_JPRB, 0.29866E+00_JPRB, 0.29597E+00_JPRB ,&
&  0.29254E+00_JPRB, 0.29023E+00_JPRB, 0.28962E+00_JPRB, 0.29082E+00_JPRB, 0.29071E+00_JPRB, 0.28890E+00_JPRB, 0.28497E+00_JPRB ,&
&  0.28052E+00_JPRB, 0.27602E+00_JPRB, 0.27136E+00_JPRB, 0.26700E+00_JPRB, 0.26342E+00_JPRB, 0.25965E+00_JPRB, 0.25558E+00_JPRB ,&
&  0.25301E+00_JPRB, 0.25030E+00_JPRB, 0.24733E+00_JPRB, 0.24452E+00_JPRB, 0.24217E+00_JPRB, 0.24036E+00_JPRB, 0.23883E+00_JPRB ,&
&  0.23766E+00_JPRB, 0.23676E+00_JPRB, 0.23604E+00_JPRB, 0.23550E+00_JPRB, 0.23498E+00_JPRB, 0.23417E+00_JPRB, 0.23338E+00_JPRB ,&
&  0.23259E+00_JPRB, 0.23229E+00_JPRB, 0.23181E+00_JPRB, 0.23184E+00_JPRB, 0.23212E+00_JPRB, 0.23228E+00_JPRB, 0.23282E+00_JPRB ,&
&  0.23396E+00_JPRB, 0.23516E+00_JPRB, 0.23579E+00_JPRB, 0.23692E+00_JPRB, 0.23802E+00_JPRB, 0.23944E+00_JPRB, 0.24131E+00_JPRB ,&
&  0.24249E+00_JPRB, 0.24380E+00_JPRB, 0.24535E+00_JPRB, 0.24702E+00_JPRB, 0.24978E+00_JPRB, 0.25278E+00_JPRB, 0.25610E+00_JPRB ,&
&  0.26036E+00_JPRB, 0.26449E+00_JPRB, 0.26920E+00_JPRB, 0.27477E+00_JPRB, 0.28472E+00_JPRB, 0.29141E+00_JPRB, 0.29997E+00_JPRB ,&
&  0.30846E+00_JPRB, 0.31304E+00_JPRB, 0.34754E+00_JPRB, 0.35021E+00_JPRB, 0.35568E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.28958E+00_JPRB, 0.29619E+00_JPRB, 0.30054E+00_JPRB, 0.30515E+00_JPRB, 0.30310E+00_JPRB, 0.34170E+00_JPRB, 0.34078E+00_JPRB ,&
&  0.33977E+00_JPRB, 0.32572E+00_JPRB, 0.32355E+00_JPRB, 0.31886E+00_JPRB, 0.31330E+00_JPRB, 0.30717E+00_JPRB, 0.30187E+00_JPRB ,&
&  0.29702E+00_JPRB, 0.29160E+00_JPRB, 0.28720E+00_JPRB, 0.28312E+00_JPRB, 0.27978E+00_JPRB, 0.27682E+00_JPRB, 0.27474E+00_JPRB ,&
&  0.27422E+00_JPRB, 0.27573E+00_JPRB, 0.27891E+00_JPRB, 0.28064E+00_JPRB, 0.28134E+00_JPRB, 0.28192E+00_JPRB, 0.28251E+00_JPRB ,&
&  0.28250E+00_JPRB, 0.28241E+00_JPRB, 0.28120E+00_JPRB, 0.27896E+00_JPRB, 0.27703E+00_JPRB, 0.27553E+00_JPRB, 0.27438E+00_JPRB ,&
&  0.27329E+00_JPRB, 0.27223E+00_JPRB, 0.27109E+00_JPRB, 0.27040E+00_JPRB, 0.26937E+00_JPRB, 0.26791E+00_JPRB, 0.26581E+00_JPRB ,&
&  0.26367E+00_JPRB, 0.26148E+00_JPRB, 0.25885E+00_JPRB, 0.25675E+00_JPRB, 0.25470E+00_JPRB, 0.25263E+00_JPRB, 0.25056E+00_JPRB ,&
&  0.24939E+00_JPRB, 0.24824E+00_JPRB, 0.24681E+00_JPRB, 0.24500E+00_JPRB, 0.24363E+00_JPRB, 0.24218E+00_JPRB, 0.24091E+00_JPRB ,&
&  0.24007E+00_JPRB, 0.23920E+00_JPRB, 0.23832E+00_JPRB, 0.23740E+00_JPRB, 0.23673E+00_JPRB, 0.23596E+00_JPRB, 0.23525E+00_JPRB ,&
&  0.23462E+00_JPRB, 0.23391E+00_JPRB, 0.23374E+00_JPRB, 0.23366E+00_JPRB, 0.23374E+00_JPRB, 0.23379E+00_JPRB, 0.23411E+00_JPRB ,&
&  0.23460E+00_JPRB, 0.23531E+00_JPRB, 0.23617E+00_JPRB, 0.23699E+00_JPRB, 0.23778E+00_JPRB, 0.23861E+00_JPRB, 0.23982E+00_JPRB ,&
&  0.24113E+00_JPRB, 0.24253E+00_JPRB, 0.24398E+00_JPRB, 0.24511E+00_JPRB, 0.24749E+00_JPRB, 0.24974E+00_JPRB, 0.25212E+00_JPRB ,&
&  0.25486E+00_JPRB, 0.25809E+00_JPRB, 0.26157E+00_JPRB, 0.26529E+00_JPRB, 0.27019E+00_JPRB, 0.27325E+00_JPRB, 0.27546E+00_JPRB ,&
&  0.27790E+00_JPRB, 0.28202E+00_JPRB, 0.28353E+00_JPRB, 0.28486E+00_JPRB, 0.28688E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.26271E+00_JPRB, 0.26299E+00_JPRB, 0.26307E+00_JPRB, 0.26302E+00_JPRB, 0.26102E+00_JPRB, 0.26135E+00_JPRB, 0.26138E+00_JPRB ,&
&  0.26161E+00_JPRB, 0.26187E+00_JPRB, 0.26250E+00_JPRB, 0.26325E+00_JPRB, 0.26400E+00_JPRB, 0.26468E+00_JPRB, 0.26528E+00_JPRB ,&
&  0.26565E+00_JPRB, 0.26613E+00_JPRB, 0.26480E+00_JPRB, 0.26400E+00_JPRB, 0.26391E+00_JPRB, 0.26402E+00_JPRB, 0.26400E+00_JPRB ,&
&  0.26394E+00_JPRB, 0.26382E+00_JPRB, 0.26335E+00_JPRB, 0.26301E+00_JPRB, 0.26286E+00_JPRB, 0.26265E+00_JPRB, 0.26233E+00_JPRB ,&
&  0.26194E+00_JPRB, 0.26162E+00_JPRB, 0.26125E+00_JPRB, 0.26082E+00_JPRB, 0.26069E+00_JPRB, 0.26051E+00_JPRB, 0.25992E+00_JPRB ,&
&  0.25921E+00_JPRB, 0.25818E+00_JPRB, 0.25724E+00_JPRB, 0.25639E+00_JPRB, 0.25570E+00_JPRB, 0.25534E+00_JPRB, 0.25498E+00_JPRB ,&
&  0.25448E+00_JPRB, 0.25381E+00_JPRB, 0.25317E+00_JPRB, 0.25238E+00_JPRB, 0.25169E+00_JPRB, 0.25107E+00_JPRB, 0.25034E+00_JPRB ,&
&  0.24953E+00_JPRB, 0.24871E+00_JPRB, 0.24786E+00_JPRB, 0.24692E+00_JPRB, 0.24601E+00_JPRB, 0.24509E+00_JPRB, 0.24408E+00_JPRB ,&
&  0.24309E+00_JPRB, 0.24266E+00_JPRB, 0.24217E+00_JPRB, 0.24175E+00_JPRB, 0.24144E+00_JPRB, 0.24119E+00_JPRB, 0.24095E+00_JPRB ,&
&  0.24076E+00_JPRB, 0.24056E+00_JPRB, 0.24046E+00_JPRB, 0.24037E+00_JPRB, 0.24037E+00_JPRB, 0.24044E+00_JPRB, 0.24061E+00_JPRB ,&
&  0.24082E+00_JPRB, 0.24108E+00_JPRB, 0.24144E+00_JPRB, 0.24203E+00_JPRB, 0.24261E+00_JPRB, 0.24329E+00_JPRB, 0.24395E+00_JPRB ,&
&  0.24469E+00_JPRB, 0.24541E+00_JPRB, 0.24615E+00_JPRB, 0.24685E+00_JPRB, 0.24779E+00_JPRB, 0.24869E+00_JPRB, 0.24968E+00_JPRB ,&
&  0.25067E+00_JPRB, 0.25173E+00_JPRB, 0.25278E+00_JPRB, 0.25391E+00_JPRB, 0.25519E+00_JPRB, 0.25557E+00_JPRB, 0.25596E+00_JPRB ,&
&  0.25645E+00_JPRB, 0.25717E+00_JPRB, 0.25782E+00_JPRB, 0.25869E+00_JPRB, 0.26015E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB, 0.24990E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB, 0.39751E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.39702E+00_JPRB, 0.39789E+00_JPRB, 0.39872E+00_JPRB, 0.39935E+00_JPRB, 0.39978E+00_JPRB, 0.40004E+00_JPRB, 0.40017E+00_JPRB ,&
&  0.40008E+00_JPRB, 0.40020E+00_JPRB, 0.40042E+00_JPRB, 0.40063E+00_JPRB, 0.40082E+00_JPRB, 0.40092E+00_JPRB, 0.40089E+00_JPRB ,&
&  0.40116E+00_JPRB, 0.40215E+00_JPRB, 0.40272E+00_JPRB, 0.40307E+00_JPRB, 0.40339E+00_JPRB, 0.40365E+00_JPRB, 0.40387E+00_JPRB ,&
&  0.40411E+00_JPRB, 0.40438E+00_JPRB, 0.40466E+00_JPRB, 0.40499E+00_JPRB, 0.40547E+00_JPRB, 0.40591E+00_JPRB, 0.40640E+00_JPRB ,&
&  0.40713E+00_JPRB, 0.40829E+00_JPRB, 0.40968E+00_JPRB, 0.41129E+00_JPRB, 0.41329E+00_JPRB, 0.41561E+00_JPRB, 0.41760E+00_JPRB ,&
&  0.41927E+00_JPRB, 0.42042E+00_JPRB, 0.42041E+00_JPRB, 0.41958E+00_JPRB, 0.41793E+00_JPRB, 0.41585E+00_JPRB, 0.41340E+00_JPRB ,&
&  0.41054E+00_JPRB, 0.40800E+00_JPRB, 0.40570E+00_JPRB, 0.40358E+00_JPRB, 0.40154E+00_JPRB, 0.39969E+00_JPRB, 0.39789E+00_JPRB ,&
&  0.39639E+00_JPRB, 0.39493E+00_JPRB, 0.39356E+00_JPRB, 0.39214E+00_JPRB, 0.39070E+00_JPRB, 0.38922E+00_JPRB, 0.38774E+00_JPRB ,&
&  0.38656E+00_JPRB, 0.38555E+00_JPRB, 0.38482E+00_JPRB, 0.38412E+00_JPRB, 0.38365E+00_JPRB, 0.38367E+00_JPRB, 0.38388E+00_JPRB ,&
&  0.38403E+00_JPRB, 0.38432E+00_JPRB, 0.38473E+00_JPRB, 0.38493E+00_JPRB, 0.38510E+00_JPRB, 0.38519E+00_JPRB, 0.38521E+00_JPRB ,&
&  0.38521E+00_JPRB, 0.38543E+00_JPRB, 0.38562E+00_JPRB, 0.38599E+00_JPRB, 0.38632E+00_JPRB, 0.38673E+00_JPRB, 0.38721E+00_JPRB ,&
&  0.38756E+00_JPRB, 0.38806E+00_JPRB, 0.38851E+00_JPRB, 0.38917E+00_JPRB, 0.38976E+00_JPRB, 0.39026E+00_JPRB, 0.39066E+00_JPRB ,&
&  0.39103E+00_JPRB, 0.39126E+00_JPRB, 0.39157E+00_JPRB, 0.39177E+00_JPRB, 0.39213E+00_JPRB, 0.39252E+00_JPRB, 0.39295E+00_JPRB ,&
&  0.39341E+00_JPRB, 0.39401E+00_JPRB, 0.39455E+00_JPRB, 0.39534E+00_JPRB, 0.39615E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.39986E+00_JPRB, 0.40113E+00_JPRB, 0.40191E+00_JPRB, 0.40227E+00_JPRB, 0.40246E+00_JPRB, 0.40211E+00_JPRB, 0.40114E+00_JPRB ,&
&  0.39954E+00_JPRB, 0.39805E+00_JPRB, 0.39685E+00_JPRB, 0.39544E+00_JPRB, 0.39358E+00_JPRB, 0.39297E+00_JPRB, 0.39272E+00_JPRB ,&
&  0.39137E+00_JPRB, 0.38976E+00_JPRB, 0.38817E+00_JPRB, 0.38701E+00_JPRB, 0.38574E+00_JPRB, 0.38437E+00_JPRB, 0.38351E+00_JPRB ,&
&  0.38304E+00_JPRB, 0.38238E+00_JPRB, 0.38133E+00_JPRB, 0.37986E+00_JPRB, 0.37946E+00_JPRB, 0.37902E+00_JPRB, 0.37844E+00_JPRB ,&
&  0.37761E+00_JPRB, 0.37599E+00_JPRB, 0.37466E+00_JPRB, 0.37411E+00_JPRB, 0.37655E+00_JPRB, 0.37984E+00_JPRB, 0.38421E+00_JPRB ,&
&  0.38980E+00_JPRB, 0.39495E+00_JPRB, 0.40081E+00_JPRB, 0.40631E+00_JPRB, 0.41057E+00_JPRB, 0.41563E+00_JPRB, 0.41915E+00_JPRB ,&
&  0.42056E+00_JPRB, 0.41972E+00_JPRB, 0.41878E+00_JPRB, 0.41865E+00_JPRB, 0.41871E+00_JPRB, 0.41836E+00_JPRB, 0.41732E+00_JPRB ,&
&  0.41607E+00_JPRB, 0.41473E+00_JPRB, 0.41311E+00_JPRB, 0.41065E+00_JPRB, 0.40760E+00_JPRB, 0.40457E+00_JPRB, 0.40137E+00_JPRB ,&
&  0.39845E+00_JPRB, 0.39506E+00_JPRB, 0.39189E+00_JPRB, 0.38886E+00_JPRB, 0.38616E+00_JPRB, 0.38315E+00_JPRB, 0.38034E+00_JPRB ,&
&  0.37852E+00_JPRB, 0.37826E+00_JPRB, 0.37847E+00_JPRB, 0.37800E+00_JPRB, 0.37790E+00_JPRB, 0.37794E+00_JPRB, 0.37859E+00_JPRB ,&
&  0.37943E+00_JPRB, 0.38107E+00_JPRB, 0.38286E+00_JPRB, 0.38503E+00_JPRB, 0.38693E+00_JPRB, 0.38873E+00_JPRB, 0.39018E+00_JPRB ,&
&  0.39148E+00_JPRB, 0.39251E+00_JPRB, 0.39337E+00_JPRB, 0.39415E+00_JPRB, 0.39484E+00_JPRB, 0.39535E+00_JPRB, 0.39554E+00_JPRB ,&
&  0.39559E+00_JPRB, 0.39534E+00_JPRB, 0.39498E+00_JPRB, 0.39439E+00_JPRB, 0.39407E+00_JPRB, 0.39330E+00_JPRB, 0.39315E+00_JPRB ,&
&  0.39323E+00_JPRB, 0.39400E+00_JPRB, 0.39495E+00_JPRB, 0.39650E+00_JPRB, 0.39820E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.39921E+00_JPRB, 0.40173E+00_JPRB, 0.40344E+00_JPRB, 0.40427E+00_JPRB, 0.40462E+00_JPRB, 0.40487E+00_JPRB, 0.40458E+00_JPRB ,&
&  0.40398E+00_JPRB, 0.40291E+00_JPRB, 0.40208E+00_JPRB, 0.40088E+00_JPRB, 0.39939E+00_JPRB, 0.39952E+00_JPRB, 0.39829E+00_JPRB ,&
&  0.39630E+00_JPRB, 0.39398E+00_JPRB, 0.39199E+00_JPRB, 0.38958E+00_JPRB, 0.38672E+00_JPRB, 0.38310E+00_JPRB, 0.37726E+00_JPRB ,&
&  0.37365E+00_JPRB, 0.36948E+00_JPRB, 0.36460E+00_JPRB, 0.36324E+00_JPRB, 0.36191E+00_JPRB, 0.34792E+00_JPRB, 0.34791E+00_JPRB ,&
&  0.35976E+00_JPRB, 0.36447E+00_JPRB, 0.36652E+00_JPRB, 0.36889E+00_JPRB, 0.35449E+00_JPRB, 0.33774E+00_JPRB, 0.32716E+00_JPRB ,&
&  0.31077E+00_JPRB, 0.32746E+00_JPRB, 0.33795E+00_JPRB, 0.34268E+00_JPRB, 0.34458E+00_JPRB, 0.34683E+00_JPRB, 0.36649E+00_JPRB ,&
&  0.40694E+00_JPRB, 0.41590E+00_JPRB, 0.42568E+00_JPRB, 0.43630E+00_JPRB, 0.44648E+00_JPRB, 0.45326E+00_JPRB, 0.46014E+00_JPRB ,&
&  0.46310E+00_JPRB, 0.46453E+00_JPRB, 0.46173E+00_JPRB, 0.45678E+00_JPRB, 0.44732E+00_JPRB, 0.43946E+00_JPRB, 0.43245E+00_JPRB ,&
&  0.42758E+00_JPRB, 0.42241E+00_JPRB, 0.41697E+00_JPRB, 0.41069E+00_JPRB, 0.40472E+00_JPRB, 0.39816E+00_JPRB, 0.39160E+00_JPRB ,&
&  0.38475E+00_JPRB, 0.37774E+00_JPRB, 0.37259E+00_JPRB, 0.36867E+00_JPRB, 0.36817E+00_JPRB, 0.36858E+00_JPRB, 0.37032E+00_JPRB ,&
&  0.37208E+00_JPRB, 0.37427E+00_JPRB, 0.37642E+00_JPRB, 0.37974E+00_JPRB, 0.38268E+00_JPRB, 0.38495E+00_JPRB, 0.38690E+00_JPRB ,&
&  0.38848E+00_JPRB, 0.38973E+00_JPRB, 0.39073E+00_JPRB, 0.39137E+00_JPRB, 0.39238E+00_JPRB, 0.39354E+00_JPRB, 0.39425E+00_JPRB ,&
&  0.39525E+00_JPRB, 0.39623E+00_JPRB, 0.39718E+00_JPRB, 0.39745E+00_JPRB, 0.39709E+00_JPRB, 0.39505E+00_JPRB, 0.39299E+00_JPRB ,&
&  0.39110E+00_JPRB, 0.39101E+00_JPRB, 0.39157E+00_JPRB, 0.39359E+00_JPRB, 0.39611E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.41026E+00_JPRB, 0.41235E+00_JPRB, 0.41470E+00_JPRB, 0.41630E+00_JPRB, 0.41665E+00_JPRB, 0.41646E+00_JPRB, 0.41573E+00_JPRB ,&
&  0.41469E+00_JPRB, 0.41395E+00_JPRB, 0.41265E+00_JPRB, 0.41045E+00_JPRB, 0.40873E+00_JPRB, 0.40674E+00_JPRB, 0.40461E+00_JPRB ,&
&  0.40153E+00_JPRB, 0.39846E+00_JPRB, 0.39597E+00_JPRB, 0.39273E+00_JPRB, 0.38976E+00_JPRB, 0.38724E+00_JPRB, 0.38297E+00_JPRB ,&
&  0.37999E+00_JPRB, 0.37391E+00_JPRB, 0.35161E+00_JPRB, 0.35141E+00_JPRB, 0.34662E+00_JPRB, 0.34572E+00_JPRB, 0.34834E+00_JPRB ,&
&  0.34316E+00_JPRB, 0.34269E+00_JPRB, 0.33575E+00_JPRB, 0.32519E+00_JPRB, 0.31922E+00_JPRB, 0.30950E+00_JPRB, 0.30931E+00_JPRB ,&
&  0.30762E+00_JPRB, 0.31104E+00_JPRB, 0.31903E+00_JPRB, 0.32326E+00_JPRB, 0.32512E+00_JPRB, 0.32723E+00_JPRB, 0.33629E+00_JPRB ,&
&  0.36140E+00_JPRB, 0.39929E+00_JPRB, 0.44883E+00_JPRB, 0.47125E+00_JPRB, 0.48965E+00_JPRB, 0.50839E+00_JPRB, 0.52545E+00_JPRB ,&
&  0.53695E+00_JPRB, 0.54619E+00_JPRB, 0.54417E+00_JPRB, 0.54008E+00_JPRB, 0.52692E+00_JPRB, 0.50922E+00_JPRB, 0.48896E+00_JPRB ,&
&  0.47009E+00_JPRB, 0.45641E+00_JPRB, 0.44570E+00_JPRB, 0.43879E+00_JPRB, 0.43403E+00_JPRB, 0.42973E+00_JPRB, 0.42688E+00_JPRB ,&
&  0.42001E+00_JPRB, 0.40734E+00_JPRB, 0.39578E+00_JPRB, 0.38593E+00_JPRB, 0.38214E+00_JPRB, 0.38072E+00_JPRB, 0.38298E+00_JPRB ,&
&  0.38563E+00_JPRB, 0.39023E+00_JPRB, 0.39571E+00_JPRB, 0.39766E+00_JPRB, 0.40125E+00_JPRB, 0.40591E+00_JPRB, 0.40604E+00_JPRB ,&
&  0.40605E+00_JPRB, 0.40572E+00_JPRB, 0.40444E+00_JPRB, 0.40257E+00_JPRB, 0.40031E+00_JPRB, 0.39792E+00_JPRB, 0.39616E+00_JPRB ,&
&  0.39572E+00_JPRB, 0.39677E+00_JPRB, 0.40037E+00_JPRB, 0.40455E+00_JPRB, 0.40961E+00_JPRB, 0.41206E+00_JPRB, 0.41112E+00_JPRB ,&
&  0.40668E+00_JPRB, 0.40469E+00_JPRB, 0.40336E+00_JPRB, 0.40544E+00_JPRB, 0.40775E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.42159E+00_JPRB, 0.42687E+00_JPRB, 0.43467E+00_JPRB, 0.44104E+00_JPRB, 0.44588E+00_JPRB, 0.44593E+00_JPRB, 0.44033E+00_JPRB ,&
&  0.43458E+00_JPRB, 0.42893E+00_JPRB, 0.42410E+00_JPRB, 0.41981E+00_JPRB, 0.41655E+00_JPRB, 0.41243E+00_JPRB, 0.40824E+00_JPRB ,&
&  0.40472E+00_JPRB, 0.40236E+00_JPRB, 0.40098E+00_JPRB, 0.40006E+00_JPRB, 0.40027E+00_JPRB, 0.39940E+00_JPRB, 0.39918E+00_JPRB ,&
&  0.39916E+00_JPRB, 0.39153E+00_JPRB, 0.37340E+00_JPRB, 0.36440E+00_JPRB, 0.35638E+00_JPRB, 0.35566E+00_JPRB, 0.35588E+00_JPRB ,&
&  0.35463E+00_JPRB, 0.31870E+00_JPRB, 0.32300E+00_JPRB, 0.31408E+00_JPRB, 0.30818E+00_JPRB, 0.30764E+00_JPRB, 0.30062E+00_JPRB ,&
&  0.29602E+00_JPRB, 0.29089E+00_JPRB, 0.29214E+00_JPRB, 0.29791E+00_JPRB, 0.29968E+00_JPRB, 0.30817E+00_JPRB, 0.31982E+00_JPRB ,&
&  0.34891E+00_JPRB, 0.44429E+00_JPRB, 0.48341E+00_JPRB, 0.50727E+00_JPRB, 0.52643E+00_JPRB, 0.55112E+00_JPRB, 0.57545E+00_JPRB ,&
&  0.59218E+00_JPRB, 0.61404E+00_JPRB, 0.61977E+00_JPRB, 0.61196E+00_JPRB, 0.61016E+00_JPRB, 0.59776E+00_JPRB, 0.57299E+00_JPRB ,&
&  0.54226E+00_JPRB, 0.50726E+00_JPRB, 0.48475E+00_JPRB, 0.47089E+00_JPRB, 0.45930E+00_JPRB, 0.44799E+00_JPRB, 0.43969E+00_JPRB ,&
&  0.44044E+00_JPRB, 0.43341E+00_JPRB, 0.42587E+00_JPRB, 0.41360E+00_JPRB, 0.40674E+00_JPRB, 0.40303E+00_JPRB, 0.40303E+00_JPRB ,&
&  0.40501E+00_JPRB, 0.40932E+00_JPRB, 0.41500E+00_JPRB, 0.42017E+00_JPRB, 0.42302E+00_JPRB, 0.42279E+00_JPRB, 0.42568E+00_JPRB ,&
&  0.42684E+00_JPRB, 0.42625E+00_JPRB, 0.42494E+00_JPRB, 0.42285E+00_JPRB, 0.42038E+00_JPRB, 0.41865E+00_JPRB, 0.41976E+00_JPRB ,&
&  0.42157E+00_JPRB, 0.42289E+00_JPRB, 0.42388E+00_JPRB, 0.42615E+00_JPRB, 0.43033E+00_JPRB, 0.43416E+00_JPRB, 0.43549E+00_JPRB ,&
&  0.42948E+00_JPRB, 0.42362E+00_JPRB, 0.41846E+00_JPRB, 0.41770E+00_JPRB, 0.41829E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.41328E+00_JPRB, 0.42695E+00_JPRB, 0.44596E+00_JPRB, 0.46095E+00_JPRB, 0.47287E+00_JPRB, 0.47151E+00_JPRB, 0.46287E+00_JPRB ,&
&  0.45361E+00_JPRB, 0.44612E+00_JPRB, 0.43926E+00_JPRB, 0.43239E+00_JPRB, 0.42424E+00_JPRB, 0.41797E+00_JPRB, 0.41279E+00_JPRB ,&
&  0.40989E+00_JPRB, 0.40832E+00_JPRB, 0.40739E+00_JPRB, 0.40644E+00_JPRB, 0.40613E+00_JPRB, 0.40699E+00_JPRB, 0.40743E+00_JPRB ,&
&  0.40857E+00_JPRB, 0.40344E+00_JPRB, 0.38868E+00_JPRB, 0.37400E+00_JPRB, 0.36688E+00_JPRB, 0.35898E+00_JPRB, 0.36099E+00_JPRB ,&
&  0.36036E+00_JPRB, 0.35950E+00_JPRB, 0.35538E+00_JPRB, 0.35405E+00_JPRB, 0.34309E+00_JPRB, 0.29696E+00_JPRB, 0.29926E+00_JPRB ,&
&  0.28994E+00_JPRB, 0.28028E+00_JPRB, 0.27916E+00_JPRB, 0.28009E+00_JPRB, 0.28279E+00_JPRB, 0.29875E+00_JPRB, 0.31815E+00_JPRB ,&
&  0.34717E+00_JPRB, 0.45441E+00_JPRB, 0.48281E+00_JPRB, 0.51370E+00_JPRB, 0.54127E+00_JPRB, 0.57692E+00_JPRB, 0.61653E+00_JPRB ,&
&  0.63914E+00_JPRB, 0.66357E+00_JPRB, 0.67724E+00_JPRB, 0.68411E+00_JPRB, 0.67787E+00_JPRB, 0.66973E+00_JPRB, 0.66073E+00_JPRB ,&
&  0.64057E+00_JPRB, 0.58313E+00_JPRB, 0.54284E+00_JPRB, 0.51925E+00_JPRB, 0.49900E+00_JPRB, 0.48367E+00_JPRB, 0.47100E+00_JPRB ,&
&  0.46411E+00_JPRB, 0.44543E+00_JPRB, 0.43928E+00_JPRB, 0.43082E+00_JPRB, 0.42623E+00_JPRB, 0.42282E+00_JPRB, 0.41866E+00_JPRB ,&
&  0.41285E+00_JPRB, 0.41238E+00_JPRB, 0.41745E+00_JPRB, 0.42006E+00_JPRB, 0.42078E+00_JPRB, 0.41826E+00_JPRB, 0.41509E+00_JPRB ,&
&  0.40972E+00_JPRB, 0.41113E+00_JPRB, 0.40926E+00_JPRB, 0.40887E+00_JPRB, 0.40993E+00_JPRB, 0.41601E+00_JPRB, 0.42365E+00_JPRB ,&
&  0.42969E+00_JPRB, 0.43214E+00_JPRB, 0.43223E+00_JPRB, 0.43281E+00_JPRB, 0.43213E+00_JPRB, 0.43060E+00_JPRB, 0.42677E+00_JPRB ,&
&  0.42331E+00_JPRB, 0.42090E+00_JPRB, 0.41564E+00_JPRB, 0.41084E+00_JPRB, 0.40860E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.40928E+00_JPRB, 0.43349E+00_JPRB, 0.46085E+00_JPRB, 0.48004E+00_JPRB, 0.48981E+00_JPRB, 0.47840E+00_JPRB, 0.46469E+00_JPRB ,&
&  0.45350E+00_JPRB, 0.44461E+00_JPRB, 0.43853E+00_JPRB, 0.43379E+00_JPRB, 0.42798E+00_JPRB, 0.42245E+00_JPRB, 0.41725E+00_JPRB ,&
&  0.41333E+00_JPRB, 0.40488E+00_JPRB, 0.40060E+00_JPRB, 0.40065E+00_JPRB, 0.40121E+00_JPRB, 0.40416E+00_JPRB, 0.40736E+00_JPRB ,&
&  0.40700E+00_JPRB, 0.40071E+00_JPRB, 0.39462E+00_JPRB, 0.38196E+00_JPRB, 0.36851E+00_JPRB, 0.36153E+00_JPRB, 0.36238E+00_JPRB ,&
&  0.35620E+00_JPRB, 0.34260E+00_JPRB, 0.35445E+00_JPRB, 0.36384E+00_JPRB, 0.35067E+00_JPRB, 0.34336E+00_JPRB, 0.30856E+00_JPRB ,&
&  0.29529E+00_JPRB, 0.27968E+00_JPRB, 0.27607E+00_JPRB, 0.27693E+00_JPRB, 0.28472E+00_JPRB, 0.30042E+00_JPRB, 0.33488E+00_JPRB ,&
&  0.38749E+00_JPRB, 0.46465E+00_JPRB, 0.48539E+00_JPRB, 0.52441E+00_JPRB, 0.56041E+00_JPRB, 0.59230E+00_JPRB, 0.61555E+00_JPRB ,&
&  0.65083E+00_JPRB, 0.69622E+00_JPRB, 0.71879E+00_JPRB, 0.71486E+00_JPRB, 0.71313E+00_JPRB, 0.71676E+00_JPRB, 0.73188E+00_JPRB ,&
&  0.75959E+00_JPRB, 0.66843E+00_JPRB, 0.60655E+00_JPRB, 0.56268E+00_JPRB, 0.53340E+00_JPRB, 0.52067E+00_JPRB, 0.48406E+00_JPRB ,&
&  0.46805E+00_JPRB, 0.46225E+00_JPRB, 0.45302E+00_JPRB, 0.44720E+00_JPRB, 0.44237E+00_JPRB, 0.43811E+00_JPRB, 0.43091E+00_JPRB ,&
&  0.42202E+00_JPRB, 0.42159E+00_JPRB, 0.45082E+00_JPRB, 0.45562E+00_JPRB, 0.44601E+00_JPRB, 0.42286E+00_JPRB, 0.40065E+00_JPRB ,&
&  0.38618E+00_JPRB, 0.38169E+00_JPRB, 0.38104E+00_JPRB, 0.38073E+00_JPRB, 0.38493E+00_JPRB, 0.39185E+00_JPRB, 0.40314E+00_JPRB ,&
&  0.40673E+00_JPRB, 0.41536E+00_JPRB, 0.42333E+00_JPRB, 0.42725E+00_JPRB, 0.42634E+00_JPRB, 0.42310E+00_JPRB, 0.42032E+00_JPRB ,&
&  0.41787E+00_JPRB, 0.41522E+00_JPRB, 0.41151E+00_JPRB, 0.40596E+00_JPRB, 0.40200E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.39881E+00_JPRB, 0.43163E+00_JPRB, 0.47465E+00_JPRB, 0.50452E+00_JPRB, 0.50631E+00_JPRB, 0.48046E+00_JPRB, 0.45701E+00_JPRB ,&
&  0.44289E+00_JPRB, 0.43661E+00_JPRB, 0.43462E+00_JPRB, 0.43041E+00_JPRB, 0.42573E+00_JPRB, 0.42062E+00_JPRB, 0.41556E+00_JPRB ,&
&  0.41120E+00_JPRB, 0.40717E+00_JPRB, 0.40213E+00_JPRB, 0.39527E+00_JPRB, 0.39075E+00_JPRB, 0.39018E+00_JPRB, 0.39196E+00_JPRB ,&
&  0.39556E+00_JPRB, 0.39451E+00_JPRB, 0.39201E+00_JPRB, 0.38514E+00_JPRB, 0.37498E+00_JPRB, 0.36404E+00_JPRB, 0.35494E+00_JPRB ,&
&  0.34998E+00_JPRB, 0.35006E+00_JPRB, 0.35069E+00_JPRB, 0.36442E+00_JPRB, 0.35984E+00_JPRB, 0.35512E+00_JPRB, 0.33213E+00_JPRB ,&
&  0.31176E+00_JPRB, 0.28995E+00_JPRB, 0.28172E+00_JPRB, 0.28508E+00_JPRB, 0.30475E+00_JPRB, 0.33575E+00_JPRB, 0.39018E+00_JPRB ,&
&  0.48844E+00_JPRB, 0.53731E+00_JPRB, 0.57743E+00_JPRB, 0.61524E+00_JPRB, 0.64468E+00_JPRB, 0.67549E+00_JPRB, 0.71155E+00_JPRB ,&
&  0.75585E+00_JPRB, 0.79042E+00_JPRB, 0.80552E+00_JPRB, 0.77416E+00_JPRB, 0.75372E+00_JPRB, 0.68135E+00_JPRB, 0.71963E+00_JPRB ,&
&  0.90445E+00_JPRB, 0.77971E+00_JPRB, 0.68664E+00_JPRB, 0.63556E+00_JPRB, 0.59646E+00_JPRB, 0.56134E+00_JPRB, 0.52824E+00_JPRB ,&
&  0.49866E+00_JPRB, 0.48163E+00_JPRB, 0.47131E+00_JPRB, 0.44770E+00_JPRB, 0.42518E+00_JPRB, 0.42182E+00_JPRB, 0.44057E+00_JPRB ,&
&  0.49005E+00_JPRB, 0.57054E+00_JPRB, 0.69373E+00_JPRB, 0.49628E+00_JPRB, 0.43504E+00_JPRB, 0.39336E+00_JPRB, 0.37622E+00_JPRB ,&
&  0.37272E+00_JPRB, 0.37319E+00_JPRB, 0.37491E+00_JPRB, 0.37290E+00_JPRB, 0.37063E+00_JPRB, 0.37278E+00_JPRB, 0.37548E+00_JPRB ,&
&  0.37756E+00_JPRB, 0.38617E+00_JPRB, 0.39827E+00_JPRB, 0.40795E+00_JPRB, 0.41426E+00_JPRB, 0.41710E+00_JPRB, 0.41999E+00_JPRB ,&
&  0.41981E+00_JPRB, 0.41604E+00_JPRB, 0.41075E+00_JPRB, 0.39952E+00_JPRB, 0.39176E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.39291E+00_JPRB, 0.43244E+00_JPRB, 0.48967E+00_JPRB, 0.52236E+00_JPRB, 0.52280E+00_JPRB, 0.48217E+00_JPRB, 0.43986E+00_JPRB ,&
&  0.41630E+00_JPRB, 0.40164E+00_JPRB, 0.39924E+00_JPRB, 0.39681E+00_JPRB, 0.39525E+00_JPRB, 0.39416E+00_JPRB, 0.39294E+00_JPRB ,&
&  0.39035E+00_JPRB, 0.38687E+00_JPRB, 0.38293E+00_JPRB, 0.37999E+00_JPRB, 0.37681E+00_JPRB, 0.37417E+00_JPRB, 0.37355E+00_JPRB ,&
&  0.37501E+00_JPRB, 0.37515E+00_JPRB, 0.37379E+00_JPRB, 0.37163E+00_JPRB, 0.37141E+00_JPRB, 0.37388E+00_JPRB, 0.37273E+00_JPRB ,&
&  0.36247E+00_JPRB, 0.35283E+00_JPRB, 0.35835E+00_JPRB, 0.37307E+00_JPRB, 0.36938E+00_JPRB, 0.36775E+00_JPRB, 0.34388E+00_JPRB ,&
&  0.31904E+00_JPRB, 0.30202E+00_JPRB, 0.31122E+00_JPRB, 0.34841E+00_JPRB, 0.41279E+00_JPRB, 0.53891E+00_JPRB, 0.61351E+00_JPRB ,&
&  0.68128E+00_JPRB, 0.72374E+00_JPRB, 0.74987E+00_JPRB, 0.77622E+00_JPRB, 0.82552E+00_JPRB, 0.87541E+00_JPRB, 0.91091E+00_JPRB ,&
&  0.93417E+00_JPRB, 0.94145E+00_JPRB, 0.94052E+00_JPRB, 0.70773E+00_JPRB, 0.66751E+00_JPRB, 0.69896E+00_JPRB, 0.73922E+00_JPRB ,&
&  0.76198E+00_JPRB, 0.71444E+00_JPRB, 0.69413E+00_JPRB, 0.70126E+00_JPRB, 0.65593E+00_JPRB, 0.61653E+00_JPRB, 0.58100E+00_JPRB ,&
&  0.54865E+00_JPRB, 0.52844E+00_JPRB, 0.49003E+00_JPRB, 0.45553E+00_JPRB, 0.45457E+00_JPRB, 0.45640E+00_JPRB, 0.45970E+00_JPRB ,&
&  0.47443E+00_JPRB, 0.48292E+00_JPRB, 0.45057E+00_JPRB, 0.37414E+00_JPRB, 0.35373E+00_JPRB, 0.34619E+00_JPRB, 0.34680E+00_JPRB ,&
&  0.34968E+00_JPRB, 0.35290E+00_JPRB, 0.35766E+00_JPRB, 0.35844E+00_JPRB, 0.35674E+00_JPRB, 0.35606E+00_JPRB, 0.35526E+00_JPRB ,&
&  0.35258E+00_JPRB, 0.35474E+00_JPRB, 0.36234E+00_JPRB, 0.36861E+00_JPRB, 0.38206E+00_JPRB, 0.39467E+00_JPRB, 0.40573E+00_JPRB ,&
&  0.40377E+00_JPRB, 0.40118E+00_JPRB, 0.39679E+00_JPRB, 0.38553E+00_JPRB, 0.37886E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.43199E+00_JPRB, 0.47130E+00_JPRB, 0.50403E+00_JPRB, 0.52818E+00_JPRB, 0.52106E+00_JPRB, 0.49532E+00_JPRB, 0.44179E+00_JPRB ,&
&  0.43012E+00_JPRB, 0.40119E+00_JPRB, 0.39287E+00_JPRB, 0.37970E+00_JPRB, 0.36900E+00_JPRB, 0.36026E+00_JPRB, 0.35661E+00_JPRB ,&
&  0.35896E+00_JPRB, 0.36284E+00_JPRB, 0.36537E+00_JPRB, 0.36624E+00_JPRB, 0.36423E+00_JPRB, 0.36286E+00_JPRB, 0.36239E+00_JPRB ,&
&  0.36224E+00_JPRB, 0.36230E+00_JPRB, 0.36353E+00_JPRB, 0.36518E+00_JPRB, 0.36796E+00_JPRB, 0.37064E+00_JPRB, 0.36862E+00_JPRB ,&
&  0.36133E+00_JPRB, 0.35505E+00_JPRB, 0.36583E+00_JPRB, 0.38211E+00_JPRB, 0.37876E+00_JPRB, 0.38182E+00_JPRB, 0.35502E+00_JPRB ,&
&  0.32790E+00_JPRB, 0.33993E+00_JPRB, 0.41360E+00_JPRB, 0.57382E+00_JPRB, 0.63164E+00_JPRB, 0.69099E+00_JPRB, 0.73644E+00_JPRB ,&
&  0.69844E+00_JPRB, 0.71096E+00_JPRB, 0.77192E+00_JPRB, 0.94769E+00_JPRB, 0.97777E+00_JPRB, 0.10168E+01_JPRB, 0.10583E+01_JPRB ,&
&  0.11169E+01_JPRB, 0.11021E+01_JPRB, 0.10580E+01_JPRB, 0.85838E+00_JPRB, 0.95205E+00_JPRB, 0.11038E+01_JPRB, 0.93161E+00_JPRB ,&
&  0.88031E+00_JPRB, 0.83841E+00_JPRB, 0.84326E+00_JPRB, 0.78985E+00_JPRB, 0.71258E+00_JPRB, 0.66193E+00_JPRB, 0.63407E+00_JPRB ,&
&  0.59367E+00_JPRB, 0.56583E+00_JPRB, 0.51146E+00_JPRB, 0.49598E+00_JPRB, 0.50231E+00_JPRB, 0.48730E+00_JPRB, 0.47107E+00_JPRB ,&
&  0.45451E+00_JPRB, 0.42463E+00_JPRB, 0.39881E+00_JPRB, 0.37201E+00_JPRB, 0.35527E+00_JPRB, 0.35134E+00_JPRB, 0.35233E+00_JPRB ,&
&  0.35349E+00_JPRB, 0.35296E+00_JPRB, 0.35538E+00_JPRB, 0.35777E+00_JPRB, 0.35489E+00_JPRB, 0.35155E+00_JPRB, 0.35318E+00_JPRB ,&
&  0.35182E+00_JPRB, 0.34695E+00_JPRB, 0.34384E+00_JPRB, 0.35063E+00_JPRB, 0.36024E+00_JPRB, 0.37348E+00_JPRB, 0.38685E+00_JPRB ,&
&  0.39339E+00_JPRB, 0.40141E+00_JPRB, 0.39944E+00_JPRB, 0.40078E+00_JPRB, 0.40408E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.46159E+00_JPRB, 0.48523E+00_JPRB, 0.51129E+00_JPRB, 0.53025E+00_JPRB, 0.54418E+00_JPRB, 0.49282E+00_JPRB, 0.44724E+00_JPRB ,&
&  0.47318E+00_JPRB, 0.41233E+00_JPRB, 0.38738E+00_JPRB, 0.37329E+00_JPRB, 0.36818E+00_JPRB, 0.35736E+00_JPRB, 0.34700E+00_JPRB ,&
&  0.34317E+00_JPRB, 0.34636E+00_JPRB, 0.35425E+00_JPRB, 0.35817E+00_JPRB, 0.35907E+00_JPRB, 0.35919E+00_JPRB, 0.35844E+00_JPRB ,&
&  0.35854E+00_JPRB, 0.35981E+00_JPRB, 0.36138E+00_JPRB, 0.36353E+00_JPRB, 0.36402E+00_JPRB, 0.36491E+00_JPRB, 0.36354E+00_JPRB ,&
&  0.35852E+00_JPRB, 0.35850E+00_JPRB, 0.37052E+00_JPRB, 0.39093E+00_JPRB, 0.39793E+00_JPRB, 0.40823E+00_JPRB, 0.41283E+00_JPRB ,&
&  0.35766E+00_JPRB, 0.41210E+00_JPRB, 0.61403E+00_JPRB, 0.67958E+00_JPRB, 0.72751E+00_JPRB, 0.79272E+00_JPRB, 0.85033E+00_JPRB ,&
&  0.89421E+00_JPRB, 0.93109E+00_JPRB, 0.96161E+00_JPRB, 0.10024E+01_JPRB, 0.10567E+01_JPRB, 0.11041E+01_JPRB, 0.11655E+01_JPRB ,&
&  0.13175E+01_JPRB, 0.10013E+01_JPRB, 0.89701E+00_JPRB, 0.93253E+00_JPRB, 0.13212E+01_JPRB, 0.13403E+01_JPRB, 0.11251E+01_JPRB ,&
&  0.10090E+01_JPRB, 0.92366E+00_JPRB, 0.87375E+00_JPRB, 0.80650E+00_JPRB, 0.74614E+00_JPRB, 0.71351E+00_JPRB, 0.68836E+00_JPRB ,&
&  0.63624E+00_JPRB, 0.61105E+00_JPRB, 0.55165E+00_JPRB, 0.54089E+00_JPRB, 0.54112E+00_JPRB, 0.51459E+00_JPRB, 0.47569E+00_JPRB ,&
&  0.44387E+00_JPRB, 0.41806E+00_JPRB, 0.39840E+00_JPRB, 0.38513E+00_JPRB, 0.37625E+00_JPRB, 0.37064E+00_JPRB, 0.36666E+00_JPRB ,&
&  0.36449E+00_JPRB, 0.36211E+00_JPRB, 0.35501E+00_JPRB, 0.35519E+00_JPRB, 0.35643E+00_JPRB, 0.35867E+00_JPRB, 0.36617E+00_JPRB ,&
&  0.36802E+00_JPRB, 0.36325E+00_JPRB, 0.35645E+00_JPRB, 0.36245E+00_JPRB, 0.37489E+00_JPRB, 0.39479E+00_JPRB, 0.41693E+00_JPRB ,&
&  0.43961E+00_JPRB, 0.44381E+00_JPRB, 0.42335E+00_JPRB, 0.41886E+00_JPRB, 0.42815E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.48388E+00_JPRB, 0.50437E+00_JPRB, 0.51939E+00_JPRB, 0.53730E+00_JPRB, 0.55262E+00_JPRB, 0.50691E+00_JPRB, 0.47844E+00_JPRB ,&
&  0.54155E+00_JPRB, 0.46510E+00_JPRB, 0.51368E+00_JPRB, 0.42476E+00_JPRB, 0.38513E+00_JPRB, 0.36930E+00_JPRB, 0.35705E+00_JPRB ,&
&  0.34565E+00_JPRB, 0.35141E+00_JPRB, 0.35799E+00_JPRB, 0.36022E+00_JPRB, 0.36710E+00_JPRB, 0.36332E+00_JPRB, 0.35992E+00_JPRB ,&
&  0.35780E+00_JPRB, 0.35772E+00_JPRB, 0.36302E+00_JPRB, 0.36269E+00_JPRB, 0.36156E+00_JPRB, 0.36002E+00_JPRB, 0.35540E+00_JPRB ,&
&  0.34498E+00_JPRB, 0.34856E+00_JPRB, 0.37026E+00_JPRB, 0.39077E+00_JPRB, 0.40917E+00_JPRB, 0.42711E+00_JPRB, 0.46005E+00_JPRB ,&
&  0.49018E+00_JPRB, 0.51202E+00_JPRB, 0.65441E+00_JPRB, 0.69612E+00_JPRB, 0.74536E+00_JPRB, 0.78124E+00_JPRB, 0.82520E+00_JPRB ,&
&  0.87008E+00_JPRB, 0.90886E+00_JPRB, 0.93296E+00_JPRB, 0.97811E+00_JPRB, 0.10646E+01_JPRB, 0.11247E+01_JPRB, 0.12235E+01_JPRB ,&
&  0.14082E+01_JPRB, 0.10199E+01_JPRB, 0.11996E+01_JPRB, 0.11669E+01_JPRB, 0.13708E+01_JPRB, 0.15025E+01_JPRB, 0.14374E+01_JPRB ,&
&  0.14258E+01_JPRB, 0.11813E+01_JPRB, 0.10203E+01_JPRB, 0.88694E+00_JPRB, 0.79141E+00_JPRB, 0.75863E+00_JPRB, 0.73538E+00_JPRB ,&
&  0.74001E+00_JPRB, 0.71862E+00_JPRB, 0.62111E+00_JPRB, 0.58057E+00_JPRB, 0.56762E+00_JPRB, 0.55692E+00_JPRB, 0.51551E+00_JPRB ,&
&  0.49038E+00_JPRB, 0.47412E+00_JPRB, 0.45313E+00_JPRB, 0.43301E+00_JPRB, 0.41720E+00_JPRB, 0.40700E+00_JPRB, 0.39414E+00_JPRB ,&
&  0.39032E+00_JPRB, 0.38008E+00_JPRB, 0.35606E+00_JPRB, 0.34498E+00_JPRB, 0.34296E+00_JPRB, 0.34101E+00_JPRB, 0.35181E+00_JPRB ,&
&  0.36373E+00_JPRB, 0.36605E+00_JPRB, 0.39117E+00_JPRB, 0.42578E+00_JPRB, 0.43678E+00_JPRB, 0.44669E+00_JPRB, 0.49869E+00_JPRB ,&
&  0.46915E+00_JPRB, 0.48216E+00_JPRB, 0.48571E+00_JPRB, 0.46760E+00_JPRB, 0.46619E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.50917E+00_JPRB, 0.51895E+00_JPRB, 0.52756E+00_JPRB, 0.54586E+00_JPRB, 0.55829E+00_JPRB, 0.56006E+00_JPRB, 0.52357E+00_JPRB ,&
&  0.56357E+00_JPRB, 0.54334E+00_JPRB, 0.52463E+00_JPRB, 0.52126E+00_JPRB, 0.51213E+00_JPRB, 0.44048E+00_JPRB, 0.36685E+00_JPRB ,&
&  0.34579E+00_JPRB, 0.34302E+00_JPRB, 0.35914E+00_JPRB, 0.36243E+00_JPRB, 0.36782E+00_JPRB, 0.36724E+00_JPRB, 0.36294E+00_JPRB ,&
&  0.36157E+00_JPRB, 0.36029E+00_JPRB, 0.36553E+00_JPRB, 0.36625E+00_JPRB, 0.36174E+00_JPRB, 0.36026E+00_JPRB, 0.35891E+00_JPRB ,&
&  0.34422E+00_JPRB, 0.34098E+00_JPRB, 0.35517E+00_JPRB, 0.35891E+00_JPRB, 0.41024E+00_JPRB, 0.43691E+00_JPRB, 0.47510E+00_JPRB ,&
&  0.51444E+00_JPRB, 0.55644E+00_JPRB, 0.62721E+00_JPRB, 0.68237E+00_JPRB, 0.71969E+00_JPRB, 0.74860E+00_JPRB, 0.79151E+00_JPRB ,&
&  0.82646E+00_JPRB, 0.87339E+00_JPRB, 0.92700E+00_JPRB, 0.10130E+01_JPRB, 0.11401E+01_JPRB, 0.10429E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.13413E+01_JPRB, 0.14963E+01_JPRB, 0.16313E+01_JPRB, 0.14779E+01_JPRB, 0.16196E+01_JPRB, 0.16726E+01_JPRB, 0.15645E+01_JPRB ,&
&  0.14268E+01_JPRB, 0.12073E+01_JPRB, 0.11284E+01_JPRB, 0.10654E+01_JPRB, 0.10319E+01_JPRB, 0.94709E+00_JPRB, 0.87102E+00_JPRB ,&
&  0.89892E+00_JPRB, 0.10151E+01_JPRB, 0.80097E+00_JPRB, 0.73587E+00_JPRB, 0.68116E+00_JPRB, 0.66294E+00_JPRB, 0.62862E+00_JPRB ,&
&  0.61086E+00_JPRB, 0.60781E+00_JPRB, 0.58508E+00_JPRB, 0.55546E+00_JPRB, 0.49967E+00_JPRB, 0.47396E+00_JPRB, 0.44379E+00_JPRB ,&
&  0.42697E+00_JPRB, 0.40087E+00_JPRB, 0.36411E+00_JPRB, 0.34340E+00_JPRB, 0.33103E+00_JPRB, 0.32877E+00_JPRB, 0.34403E+00_JPRB ,&
&  0.37856E+00_JPRB, 0.41348E+00_JPRB, 0.49196E+00_JPRB, 0.51233E+00_JPRB, 0.52674E+00_JPRB, 0.55845E+00_JPRB, 0.49717E+00_JPRB ,&
&  0.46392E+00_JPRB, 0.52841E+00_JPRB, 0.51346E+00_JPRB, 0.50100E+00_JPRB, 0.50056E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.55065E+00_JPRB, 0.55487E+00_JPRB, 0.55777E+00_JPRB, 0.56572E+00_JPRB, 0.57368E+00_JPRB, 0.56490E+00_JPRB, 0.56281E+00_JPRB ,&
&  0.56789E+00_JPRB, 0.56293E+00_JPRB, 0.54587E+00_JPRB, 0.53422E+00_JPRB, 0.51911E+00_JPRB, 0.51581E+00_JPRB, 0.44985E+00_JPRB ,&
&  0.38049E+00_JPRB, 0.36590E+00_JPRB, 0.35316E+00_JPRB, 0.35899E+00_JPRB, 0.37695E+00_JPRB, 0.37472E+00_JPRB, 0.37275E+00_JPRB ,&
&  0.37117E+00_JPRB, 0.36814E+00_JPRB, 0.36847E+00_JPRB, 0.36860E+00_JPRB, 0.37046E+00_JPRB, 0.38034E+00_JPRB, 0.38419E+00_JPRB ,&
&  0.36065E+00_JPRB, 0.33683E+00_JPRB, 0.33414E+00_JPRB, 0.34803E+00_JPRB, 0.39501E+00_JPRB, 0.44455E+00_JPRB, 0.48507E+00_JPRB ,&
&  0.52321E+00_JPRB, 0.56393E+00_JPRB, 0.61194E+00_JPRB, 0.66897E+00_JPRB, 0.70557E+00_JPRB, 0.74110E+00_JPRB, 0.78021E+00_JPRB ,&
&  0.82468E+00_JPRB, 0.88707E+00_JPRB, 0.96842E+00_JPRB, 0.10889E+01_JPRB, 0.10882E+01_JPRB, 0.11228E+01_JPRB, 0.12677E+01_JPRB ,&
&  0.13727E+01_JPRB, 0.14933E+01_JPRB, 0.16002E+01_JPRB, 0.18025E+01_JPRB, 0.19043E+01_JPRB, 0.17823E+01_JPRB, 0.16076E+01_JPRB ,&
&  0.14653E+01_JPRB, 0.12662E+01_JPRB, 0.13742E+01_JPRB, 0.12321E+01_JPRB, 0.11216E+01_JPRB, 0.10972E+01_JPRB, 0.10322E+01_JPRB ,&
&  0.10090E+01_JPRB, 0.10476E+01_JPRB, 0.10105E+01_JPRB, 0.99516E+00_JPRB, 0.92524E+00_JPRB, 0.90828E+00_JPRB, 0.84854E+00_JPRB ,&
&  0.84804E+00_JPRB, 0.83610E+00_JPRB, 0.72238E+00_JPRB, 0.62512E+00_JPRB, 0.53741E+00_JPRB, 0.51664E+00_JPRB, 0.47768E+00_JPRB ,&
&  0.44841E+00_JPRB, 0.41254E+00_JPRB, 0.38314E+00_JPRB, 0.35330E+00_JPRB, 0.34681E+00_JPRB, 0.35516E+00_JPRB, 0.37091E+00_JPRB ,&
&  0.41647E+00_JPRB, 0.49956E+00_JPRB, 0.53274E+00_JPRB, 0.55855E+00_JPRB, 0.57667E+00_JPRB, 0.60527E+00_JPRB, 0.49194E+00_JPRB ,&
&  0.49219E+00_JPRB, 0.54887E+00_JPRB, 0.54514E+00_JPRB, 0.53928E+00_JPRB, 0.54005E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.61850E+00_JPRB, 0.61764E+00_JPRB, 0.61342E+00_JPRB, 0.60732E+00_JPRB, 0.59874E+00_JPRB, 0.58770E+00_JPRB, 0.58720E+00_JPRB ,&
&  0.58865E+00_JPRB, 0.57852E+00_JPRB, 0.56349E+00_JPRB, 0.55132E+00_JPRB, 0.53905E+00_JPRB, 0.53372E+00_JPRB, 0.54441E+00_JPRB ,&
&  0.43945E+00_JPRB, 0.40262E+00_JPRB, 0.35729E+00_JPRB, 0.35561E+00_JPRB, 0.41173E+00_JPRB, 0.39047E+00_JPRB, 0.38221E+00_JPRB ,&
&  0.37393E+00_JPRB, 0.37750E+00_JPRB, 0.37580E+00_JPRB, 0.39174E+00_JPRB, 0.40749E+00_JPRB, 0.42756E+00_JPRB, 0.42911E+00_JPRB ,&
&  0.39659E+00_JPRB, 0.35927E+00_JPRB, 0.35010E+00_JPRB, 0.35744E+00_JPRB, 0.39640E+00_JPRB, 0.43459E+00_JPRB, 0.50332E+00_JPRB ,&
&  0.55196E+00_JPRB, 0.59572E+00_JPRB, 0.63732E+00_JPRB, 0.69111E+00_JPRB, 0.73268E+00_JPRB, 0.77936E+00_JPRB, 0.83678E+00_JPRB ,&
&  0.90072E+00_JPRB, 0.97827E+00_JPRB, 0.10592E+01_JPRB, 0.11609E+01_JPRB, 0.11820E+01_JPRB, 0.12327E+01_JPRB, 0.12991E+01_JPRB ,&
&  0.13724E+01_JPRB, 0.14199E+01_JPRB, 0.15504E+01_JPRB, 0.19616E+01_JPRB, 0.21804E+01_JPRB, 0.21370E+01_JPRB, 0.18783E+01_JPRB ,&
&  0.16194E+01_JPRB, 0.13734E+01_JPRB, 0.13097E+01_JPRB, 0.12069E+01_JPRB, 0.11100E+01_JPRB, 0.11451E+01_JPRB, 0.11065E+01_JPRB ,&
&  0.10589E+01_JPRB, 0.10753E+01_JPRB, 0.10865E+01_JPRB, 0.11218E+01_JPRB, 0.10776E+01_JPRB, 0.10844E+01_JPRB, 0.10544E+01_JPRB ,&
&  0.10602E+01_JPRB, 0.97470E+00_JPRB, 0.75421E+00_JPRB, 0.64819E+00_JPRB, 0.54819E+00_JPRB, 0.51483E+00_JPRB, 0.57715E+00_JPRB ,&
&  0.52046E+00_JPRB, 0.48182E+00_JPRB, 0.44713E+00_JPRB, 0.41926E+00_JPRB, 0.42626E+00_JPRB, 0.47124E+00_JPRB, 0.44934E+00_JPRB ,&
&  0.44827E+00_JPRB, 0.52551E+00_JPRB, 0.62213E+00_JPRB, 0.69872E+00_JPRB, 0.73418E+00_JPRB, 0.73135E+00_JPRB, 0.62318E+00_JPRB ,&
&  0.63457E+00_JPRB, 0.61929E+00_JPRB, 0.61940E+00_JPRB, 0.61093E+00_JPRB, 0.61162E+00_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.71494E+00_JPRB, 0.70659E+00_JPRB, 0.68976E+00_JPRB, 0.66135E+00_JPRB, 0.63996E+00_JPRB, 0.62767E+00_JPRB, 0.62315E+00_JPRB ,&
&  0.61398E+00_JPRB, 0.59559E+00_JPRB, 0.58164E+00_JPRB, 0.56698E+00_JPRB, 0.56156E+00_JPRB, 0.56166E+00_JPRB, 0.56089E+00_JPRB ,&
&  0.57985E+00_JPRB, 0.49773E+00_JPRB, 0.41502E+00_JPRB, 0.36590E+00_JPRB, 0.38646E+00_JPRB, 0.37781E+00_JPRB, 0.38166E+00_JPRB ,&
&  0.38046E+00_JPRB, 0.40727E+00_JPRB, 0.41396E+00_JPRB, 0.44345E+00_JPRB, 0.48047E+00_JPRB, 0.52008E+00_JPRB, 0.52244E+00_JPRB ,&
&  0.48411E+00_JPRB, 0.43288E+00_JPRB, 0.41224E+00_JPRB, 0.42192E+00_JPRB, 0.47141E+00_JPRB, 0.46829E+00_JPRB, 0.53324E+00_JPRB ,&
&  0.57902E+00_JPRB, 0.62723E+00_JPRB, 0.67988E+00_JPRB, 0.72810E+00_JPRB, 0.79039E+00_JPRB, 0.86374E+00_JPRB, 0.94049E+00_JPRB ,&
&  0.10205E+01_JPRB, 0.10816E+01_JPRB, 0.11047E+01_JPRB, 0.11459E+01_JPRB, 0.12500E+01_JPRB, 0.13512E+01_JPRB, 0.12793E+01_JPRB ,&
&  0.13036E+01_JPRB, 0.13365E+01_JPRB, 0.14421E+01_JPRB, 0.18276E+01_JPRB, 0.21218E+01_JPRB, 0.21688E+01_JPRB, 0.20398E+01_JPRB ,&
&  0.18302E+01_JPRB, 0.16273E+01_JPRB, 0.15596E+01_JPRB, 0.13968E+01_JPRB, 0.13054E+01_JPRB, 0.12995E+01_JPRB, 0.12814E+01_JPRB ,&
&  0.12368E+01_JPRB, 0.12414E+01_JPRB, 0.12665E+01_JPRB, 0.12338E+01_JPRB, 0.12329E+01_JPRB, 0.12253E+01_JPRB, 0.12615E+01_JPRB ,&
&  0.13674E+01_JPRB, 0.95912E+00_JPRB, 0.77453E+00_JPRB, 0.79633E+00_JPRB, 0.66452E+00_JPRB, 0.58763E+00_JPRB, 0.59255E+00_JPRB ,&
&  0.52446E+00_JPRB, 0.51907E+00_JPRB, 0.54919E+00_JPRB, 0.52708E+00_JPRB, 0.57662E+00_JPRB, 0.66135E+00_JPRB, 0.61069E+00_JPRB ,&
&  0.60531E+00_JPRB, 0.62709E+00_JPRB, 0.79044E+00_JPRB, 0.89568E+00_JPRB, 0.93151E+00_JPRB, 0.89798E+00_JPRB, 0.84176E+00_JPRB ,&
&  0.77277E+00_JPRB, 0.74418E+00_JPRB, 0.73829E+00_JPRB, 0.73039E+00_JPRB, 0.72480E+00_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.82745E+00_JPRB, 0.80682E+00_JPRB, 0.78442E+00_JPRB, 0.75433E+00_JPRB, 0.73141E+00_JPRB, 0.70803E+00_JPRB, 0.67592E+00_JPRB ,&
&  0.64368E+00_JPRB, 0.61799E+00_JPRB, 0.59938E+00_JPRB, 0.59002E+00_JPRB, 0.58997E+00_JPRB, 0.59588E+00_JPRB, 0.59701E+00_JPRB ,&
&  0.60353E+00_JPRB, 0.57419E+00_JPRB, 0.46634E+00_JPRB, 0.39721E+00_JPRB, 0.35392E+00_JPRB, 0.36177E+00_JPRB, 0.37181E+00_JPRB ,&
&  0.39663E+00_JPRB, 0.43017E+00_JPRB, 0.49175E+00_JPRB, 0.57653E+00_JPRB, 0.63041E+00_JPRB, 0.65485E+00_JPRB, 0.66225E+00_JPRB ,&
&  0.61498E+00_JPRB, 0.56805E+00_JPRB, 0.51463E+00_JPRB, 0.53831E+00_JPRB, 0.54920E+00_JPRB, 0.54341E+00_JPRB, 0.60466E+00_JPRB ,&
&  0.65067E+00_JPRB, 0.70648E+00_JPRB, 0.75498E+00_JPRB, 0.82357E+00_JPRB, 0.88225E+00_JPRB, 0.92785E+00_JPRB, 0.96895E+00_JPRB ,&
&  0.10024E+01_JPRB, 0.10430E+01_JPRB, 0.10715E+01_JPRB, 0.10891E+01_JPRB, 0.11457E+01_JPRB, 0.12086E+01_JPRB, 0.11827E+01_JPRB ,&
&  0.12622E+01_JPRB, 0.12100E+01_JPRB, 0.12067E+01_JPRB, 0.17150E+01_JPRB, 0.24054E+01_JPRB, 0.23888E+01_JPRB, 0.21331E+01_JPRB ,&
&  0.20630E+01_JPRB, 0.19872E+01_JPRB, 0.19919E+01_JPRB, 0.19250E+01_JPRB, 0.17672E+01_JPRB, 0.17235E+01_JPRB, 0.16624E+01_JPRB ,&
&  0.16124E+01_JPRB, 0.15724E+01_JPRB, 0.15223E+01_JPRB, 0.15417E+01_JPRB, 0.15616E+01_JPRB, 0.16153E+01_JPRB, 0.16726E+01_JPRB ,&
&  0.16599E+01_JPRB, 0.14727E+01_JPRB, 0.10446E+01_JPRB, 0.98724E+00_JPRB, 0.79223E+00_JPRB, 0.70958E+00_JPRB, 0.70038E+00_JPRB ,&
&  0.61942E+00_JPRB, 0.63459E+00_JPRB, 0.66370E+00_JPRB, 0.64370E+00_JPRB, 0.79602E+00_JPRB, 0.87495E+00_JPRB, 0.88681E+00_JPRB ,&
&  0.87793E+00_JPRB, 0.85391E+00_JPRB, 0.10628E+01_JPRB, 0.11314E+01_JPRB, 0.11288E+01_JPRB, 0.10832E+01_JPRB, 0.10246E+01_JPRB ,&
&  0.94798E+00_JPRB, 0.90161E+00_JPRB, 0.88102E+00_JPRB, 0.86844E+00_JPRB, 0.84590E+00_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.90192E+00_JPRB, 0.85966E+00_JPRB, 0.81846E+00_JPRB, 0.78774E+00_JPRB, 0.76415E+00_JPRB, 0.73392E+00_JPRB, 0.69345E+00_JPRB ,&
&  0.66233E+00_JPRB, 0.64509E+00_JPRB, 0.63314E+00_JPRB, 0.62207E+00_JPRB, 0.61927E+00_JPRB, 0.62306E+00_JPRB, 0.62815E+00_JPRB ,&
&  0.62546E+00_JPRB, 0.56557E+00_JPRB, 0.45184E+00_JPRB, 0.38033E+00_JPRB, 0.33922E+00_JPRB, 0.33459E+00_JPRB, 0.35677E+00_JPRB ,&
&  0.39874E+00_JPRB, 0.50038E+00_JPRB, 0.65110E+00_JPRB, 0.75010E+00_JPRB, 0.82368E+00_JPRB, 0.87149E+00_JPRB, 0.86676E+00_JPRB ,&
&  0.89739E+00_JPRB, 0.74220E+00_JPRB, 0.72755E+00_JPRB, 0.67044E+00_JPRB, 0.68240E+00_JPRB, 0.71945E+00_JPRB, 0.72577E+00_JPRB ,&
&  0.71085E+00_JPRB, 0.74225E+00_JPRB, 0.77374E+00_JPRB, 0.82673E+00_JPRB, 0.86141E+00_JPRB, 0.90828E+00_JPRB, 0.98430E+00_JPRB ,&
&  0.10703E+01_JPRB, 0.11085E+01_JPRB, 0.10656E+01_JPRB, 0.10242E+01_JPRB, 0.11926E+01_JPRB, 0.13763E+01_JPRB, 0.14337E+01_JPRB ,&
&  0.13888E+01_JPRB, 0.14492E+01_JPRB, 0.21005E+01_JPRB, 0.24243E+01_JPRB, 0.25477E+01_JPRB, 0.27163E+01_JPRB, 0.26121E+01_JPRB ,&
&  0.23874E+01_JPRB, 0.21785E+01_JPRB, 0.20678E+01_JPRB, 0.18331E+01_JPRB, 0.20603E+01_JPRB, 0.21301E+01_JPRB, 0.21399E+01_JPRB ,&
&  0.21231E+01_JPRB, 0.20230E+01_JPRB, 0.19529E+01_JPRB, 0.19670E+01_JPRB, 0.20268E+01_JPRB, 0.20989E+01_JPRB, 0.20198E+01_JPRB ,&
&  0.17868E+01_JPRB, 0.18465E+01_JPRB, 0.15828E+01_JPRB, 0.12904E+01_JPRB, 0.10566E+01_JPRB, 0.97625E+00_JPRB, 0.89939E+00_JPRB ,&
&  0.81538E+00_JPRB, 0.77837E+00_JPRB, 0.82464E+00_JPRB, 0.90790E+00_JPRB, 0.10553E+01_JPRB, 0.11889E+01_JPRB, 0.11998E+01_JPRB ,&
&  0.11551E+01_JPRB, 0.13029E+01_JPRB, 0.13091E+01_JPRB, 0.12574E+01_JPRB, 0.12319E+01_JPRB, 0.11618E+01_JPRB, 0.11101E+01_JPRB ,&
&  0.10612E+01_JPRB, 0.10357E+01_JPRB, 0.10166E+01_JPRB, 0.98948E+00_JPRB, 0.94826E+00_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.96128E+00_JPRB, 0.91261E+00_JPRB, 0.86558E+00_JPRB, 0.82796E+00_JPRB, 0.79045E+00_JPRB, 0.75500E+00_JPRB, 0.71502E+00_JPRB ,&
&  0.68454E+00_JPRB, 0.66655E+00_JPRB, 0.65232E+00_JPRB, 0.64783E+00_JPRB, 0.65028E+00_JPRB, 0.64431E+00_JPRB, 0.64157E+00_JPRB ,&
&  0.62803E+00_JPRB, 0.52780E+00_JPRB, 0.43149E+00_JPRB, 0.39867E+00_JPRB, 0.37086E+00_JPRB, 0.34713E+00_JPRB, 0.36074E+00_JPRB ,&
&  0.43355E+00_JPRB, 0.62142E+00_JPRB, 0.80555E+00_JPRB, 0.94912E+00_JPRB, 0.10516E+01_JPRB, 0.11081E+01_JPRB, 0.10906E+01_JPRB ,&
&  0.98309E+00_JPRB, 0.99168E+00_JPRB, 0.98583E+00_JPRB, 0.94016E+00_JPRB, 0.87989E+00_JPRB, 0.85486E+00_JPRB, 0.83578E+00_JPRB ,&
&  0.81658E+00_JPRB, 0.83701E+00_JPRB, 0.87177E+00_JPRB, 0.91023E+00_JPRB, 0.97354E+00_JPRB, 0.10476E+01_JPRB, 0.11199E+01_JPRB ,&
&  0.11311E+01_JPRB, 0.11267E+01_JPRB, 0.10641E+01_JPRB, 0.10634E+01_JPRB, 0.10814E+01_JPRB, 0.14485E+01_JPRB, 0.16197E+01_JPRB ,&
&  0.19940E+01_JPRB, 0.22906E+01_JPRB, 0.23680E+01_JPRB, 0.27944E+01_JPRB, 0.23931E+01_JPRB, 0.26284E+01_JPRB, 0.27899E+01_JPRB ,&
&  0.24744E+01_JPRB, 0.22222E+01_JPRB, 0.21313E+01_JPRB, 0.18181E+01_JPRB, 0.17553E+01_JPRB, 0.25226E+01_JPRB, 0.26963E+01_JPRB ,&
&  0.27388E+01_JPRB, 0.28589E+01_JPRB, 0.26916E+01_JPRB, 0.25379E+01_JPRB, 0.23855E+01_JPRB, 0.21864E+01_JPRB, 0.16796E+01_JPRB ,&
&  0.16749E+01_JPRB, 0.15224E+01_JPRB, 0.15655E+01_JPRB, 0.13918E+01_JPRB, 0.12723E+01_JPRB, 0.11482E+01_JPRB, 0.10232E+01_JPRB ,&
&  0.94476E+00_JPRB, 0.91111E+00_JPRB, 0.11044E+01_JPRB, 0.13982E+01_JPRB, 0.15864E+01_JPRB, 0.14451E+01_JPRB, 0.16552E+01_JPRB ,&
&  0.16279E+01_JPRB, 0.14665E+01_JPRB, 0.12463E+01_JPRB, 0.12031E+01_JPRB, 0.11690E+01_JPRB, 0.11299E+01_JPRB, 0.11084E+01_JPRB ,&
&  0.10833E+01_JPRB, 0.10543E+01_JPRB, 0.10355E+01_JPRB, 0.10270E+01_JPRB, 0.10002E+01_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.98204E+00_JPRB, 0.93361E+00_JPRB, 0.88294E+00_JPRB, 0.84350E+00_JPRB, 0.80802E+00_JPRB, 0.77375E+00_JPRB, 0.74409E+00_JPRB ,&
&  0.72452E+00_JPRB, 0.70363E+00_JPRB, 0.67487E+00_JPRB, 0.65629E+00_JPRB, 0.64625E+00_JPRB, 0.65762E+00_JPRB, 0.64810E+00_JPRB ,&
&  0.62401E+00_JPRB, 0.54472E+00_JPRB, 0.45025E+00_JPRB, 0.41660E+00_JPRB, 0.41980E+00_JPRB, 0.39655E+00_JPRB, 0.39711E+00_JPRB ,&
&  0.53868E+00_JPRB, 0.84254E+00_JPRB, 0.10232E+01_JPRB, 0.11219E+01_JPRB, 0.12477E+01_JPRB, 0.13076E+01_JPRB, 0.12512E+01_JPRB ,&
&  0.12337E+01_JPRB, 0.11387E+01_JPRB, 0.10667E+01_JPRB, 0.10042E+01_JPRB, 0.95771E+00_JPRB, 0.90856E+00_JPRB, 0.86899E+00_JPRB ,&
&  0.86696E+00_JPRB, 0.87313E+00_JPRB, 0.90615E+00_JPRB, 0.91511E+00_JPRB, 0.98750E+00_JPRB, 0.10287E+01_JPRB, 0.10420E+01_JPRB ,&
&  0.10485E+01_JPRB, 0.10722E+01_JPRB, 0.10603E+01_JPRB, 0.10728E+01_JPRB, 0.12398E+01_JPRB, 0.16528E+01_JPRB, 0.20203E+01_JPRB ,&
&  0.22567E+01_JPRB, 0.24568E+01_JPRB, 0.27168E+01_JPRB, 0.29891E+01_JPRB, 0.34713E+01_JPRB, 0.33615E+01_JPRB, 0.32132E+01_JPRB ,&
&  0.24140E+01_JPRB, 0.19396E+01_JPRB, 0.16014E+01_JPRB, 0.13330E+01_JPRB, 0.20314E+01_JPRB, 0.32389E+01_JPRB, 0.36979E+01_JPRB ,&
&  0.36976E+01_JPRB, 0.38299E+01_JPRB, 0.36660E+01_JPRB, 0.30697E+01_JPRB, 0.25724E+01_JPRB, 0.20635E+01_JPRB, 0.23069E+01_JPRB ,&
&  0.22189E+01_JPRB, 0.21259E+01_JPRB, 0.18704E+01_JPRB, 0.15104E+01_JPRB, 0.12293E+01_JPRB, 0.12785E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.10664E+01_JPRB, 0.11008E+01_JPRB, 0.16694E+01_JPRB, 0.19787E+01_JPRB, 0.19453E+01_JPRB, 0.18634E+01_JPRB, 0.20772E+01_JPRB ,&
&  0.17004E+01_JPRB, 0.14376E+01_JPRB, 0.12641E+01_JPRB, 0.11634E+01_JPRB, 0.11288E+01_JPRB, 0.11024E+01_JPRB, 0.10896E+01_JPRB ,&
&  0.10755E+01_JPRB, 0.10669E+01_JPRB, 0.10523E+01_JPRB, 0.10331E+01_JPRB, 0.10103E+01_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.10012E+01_JPRB, 0.95292E+00_JPRB, 0.91172E+00_JPRB, 0.87002E+00_JPRB, 0.83209E+00_JPRB, 0.80341E+00_JPRB, 0.78242E+00_JPRB ,&
&  0.75441E+00_JPRB, 0.73072E+00_JPRB, 0.70417E+00_JPRB, 0.67981E+00_JPRB, 0.66899E+00_JPRB, 0.66899E+00_JPRB, 0.65744E+00_JPRB ,&
&  0.63993E+00_JPRB, 0.60249E+00_JPRB, 0.49435E+00_JPRB, 0.45859E+00_JPRB, 0.45887E+00_JPRB, 0.45731E+00_JPRB, 0.48441E+00_JPRB ,&
&  0.75625E+00_JPRB, 0.11127E+01_JPRB, 0.11825E+01_JPRB, 0.11917E+01_JPRB, 0.13295E+01_JPRB, 0.11870E+01_JPRB, 0.11525E+01_JPRB ,&
&  0.12340E+01_JPRB, 0.11523E+01_JPRB, 0.10828E+01_JPRB, 0.10255E+01_JPRB, 0.98458E+00_JPRB, 0.95432E+00_JPRB, 0.92760E+00_JPRB ,&
&  0.91089E+00_JPRB, 0.90641E+00_JPRB, 0.94072E+00_JPRB, 0.10177E+01_JPRB, 0.10457E+01_JPRB, 0.10442E+01_JPRB, 0.10444E+01_JPRB ,&
&  0.10344E+01_JPRB, 0.10265E+01_JPRB, 0.10357E+01_JPRB, 0.11636E+01_JPRB, 0.13949E+01_JPRB, 0.18831E+01_JPRB, 0.27382E+01_JPRB ,&
&  0.27536E+01_JPRB, 0.30437E+01_JPRB, 0.30214E+01_JPRB, 0.39680E+01_JPRB, 0.44170E+01_JPRB, 0.40350E+01_JPRB, 0.35191E+01_JPRB ,&
&  0.24971E+01_JPRB, 0.19576E+01_JPRB, 0.17680E+01_JPRB, 0.19268E+01_JPRB, 0.25076E+01_JPRB, 0.32337E+01_JPRB, 0.41030E+01_JPRB ,&
&  0.36367E+01_JPRB, 0.35903E+01_JPRB, 0.35565E+01_JPRB, 0.32435E+01_JPRB, 0.19949E+01_JPRB, 0.15302E+01_JPRB, 0.21461E+01_JPRB ,&
&  0.21554E+01_JPRB, 0.13102E+01_JPRB, 0.94111E+00_JPRB, 0.11012E+01_JPRB, 0.83362E+00_JPRB, 0.92516E+00_JPRB, 0.11890E+01_JPRB ,&
&  0.12427E+01_JPRB, 0.14177E+01_JPRB, 0.20144E+01_JPRB, 0.19860E+01_JPRB, 0.20123E+01_JPRB, 0.22189E+01_JPRB, 0.19932E+01_JPRB ,&
&  0.16287E+01_JPRB, 0.13710E+01_JPRB, 0.13421E+01_JPRB, 0.12434E+01_JPRB, 0.12015E+01_JPRB, 0.11672E+01_JPRB, 0.11422E+01_JPRB ,&
&  0.11419E+01_JPRB, 0.11272E+01_JPRB, 0.11004E+01_JPRB, 0.10699E+01_JPRB, 0.10418E+01_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.10220E+01_JPRB, 0.97658E+00_JPRB, 0.93125E+00_JPRB, 0.89400E+00_JPRB, 0.86572E+00_JPRB, 0.83685E+00_JPRB, 0.79476E+00_JPRB ,&
&  0.74616E+00_JPRB, 0.71793E+00_JPRB, 0.70319E+00_JPRB, 0.68611E+00_JPRB, 0.68177E+00_JPRB, 0.68726E+00_JPRB, 0.67880E+00_JPRB ,&
&  0.65272E+00_JPRB, 0.62150E+00_JPRB, 0.57462E+00_JPRB, 0.53201E+00_JPRB, 0.51410E+00_JPRB, 0.49841E+00_JPRB, 0.62067E+00_JPRB ,&
&  0.10450E+01_JPRB, 0.13987E+01_JPRB, 0.13658E+01_JPRB, 0.12316E+01_JPRB, 0.11595E+01_JPRB, 0.10649E+01_JPRB, 0.11765E+01_JPRB ,&
&  0.11936E+01_JPRB, 0.11602E+01_JPRB, 0.10771E+01_JPRB, 0.10215E+01_JPRB, 0.98090E+00_JPRB, 0.94380E+00_JPRB, 0.92761E+00_JPRB ,&
&  0.91708E+00_JPRB, 0.92470E+00_JPRB, 0.98948E+00_JPRB, 0.10129E+01_JPRB, 0.10233E+01_JPRB, 0.10129E+01_JPRB, 0.10468E+01_JPRB ,&
&  0.10307E+01_JPRB, 0.10331E+01_JPRB, 0.10738E+01_JPRB, 0.12574E+01_JPRB, 0.17452E+01_JPRB, 0.24111E+01_JPRB, 0.26328E+01_JPRB ,&
&  0.27001E+01_JPRB, 0.29058E+01_JPRB, 0.35531E+01_JPRB, 0.40652E+01_JPRB, 0.43911E+01_JPRB, 0.42065E+01_JPRB, 0.33627E+01_JPRB ,&
&  0.28571E+01_JPRB, 0.24791E+01_JPRB, 0.20760E+01_JPRB, 0.22582E+01_JPRB, 0.30561E+01_JPRB, 0.32192E+01_JPRB, 0.37854E+01_JPRB ,&
&  0.37453E+01_JPRB, 0.33591E+01_JPRB, 0.26407E+01_JPRB, 0.20206E+01_JPRB, 0.20298E+01_JPRB, 0.15235E+01_JPRB, 0.99628E+00_JPRB ,&
&  0.89688E+00_JPRB, 0.81042E+00_JPRB, 0.72196E+00_JPRB, 0.72583E+00_JPRB, 0.68702E+00_JPRB, 0.82337E+00_JPRB, 0.11604E+01_JPRB ,&
&  0.14769E+01_JPRB, 0.17162E+01_JPRB, 0.19271E+01_JPRB, 0.18589E+01_JPRB, 0.17829E+01_JPRB, 0.20803E+01_JPRB, 0.22146E+01_JPRB ,&
&  0.14539E+01_JPRB, 0.13114E+01_JPRB, 0.13270E+01_JPRB, 0.12806E+01_JPRB, 0.12470E+01_JPRB, 0.12164E+01_JPRB, 0.12047E+01_JPRB ,&
&  0.12029E+01_JPRB, 0.11807E+01_JPRB, 0.11411E+01_JPRB, 0.11003E+01_JPRB, 0.10599E+01_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.10059E+01_JPRB, 0.94897E+00_JPRB, 0.91489E+00_JPRB, 0.89973E+00_JPRB, 0.87827E+00_JPRB, 0.83227E+00_JPRB, 0.76779E+00_JPRB ,&
&  0.72408E+00_JPRB, 0.70257E+00_JPRB, 0.69383E+00_JPRB, 0.68744E+00_JPRB, 0.68624E+00_JPRB, 0.68440E+00_JPRB, 0.67453E+00_JPRB ,&
&  0.64214E+00_JPRB, 0.62213E+00_JPRB, 0.62246E+00_JPRB, 0.59721E+00_JPRB, 0.59105E+00_JPRB, 0.58337E+00_JPRB, 0.79636E+00_JPRB ,&
&  0.13607E+01_JPRB, 0.16526E+01_JPRB, 0.14947E+01_JPRB, 0.12285E+01_JPRB, 0.11690E+01_JPRB, 0.11709E+01_JPRB, 0.12193E+01_JPRB ,&
&  0.11702E+01_JPRB, 0.11016E+01_JPRB, 0.98541E+00_JPRB, 0.94274E+00_JPRB, 0.93300E+00_JPRB, 0.92178E+00_JPRB, 0.92034E+00_JPRB ,&
&  0.91808E+00_JPRB, 0.95703E+00_JPRB, 0.96757E+00_JPRB, 0.97820E+00_JPRB, 0.10017E+01_JPRB, 0.10321E+01_JPRB, 0.10737E+01_JPRB ,&
&  0.10960E+01_JPRB, 0.11187E+01_JPRB, 0.12202E+01_JPRB, 0.15712E+01_JPRB, 0.20473E+01_JPRB, 0.22781E+01_JPRB, 0.27069E+01_JPRB ,&
&  0.31502E+01_JPRB, 0.35208E+01_JPRB, 0.34332E+01_JPRB, 0.43101E+01_JPRB, 0.43492E+01_JPRB, 0.37775E+01_JPRB, 0.32277E+01_JPRB ,&
&  0.28618E+01_JPRB, 0.30159E+01_JPRB, 0.24095E+01_JPRB, 0.21955E+01_JPRB, 0.33354E+01_JPRB, 0.35995E+01_JPRB, 0.38128E+01_JPRB ,&
&  0.40391E+01_JPRB, 0.34480E+01_JPRB, 0.27353E+01_JPRB, 0.22638E+01_JPRB, 0.27762E+01_JPRB, 0.25059E+01_JPRB, 0.11469E+01_JPRB ,&
&  0.87808E+00_JPRB, 0.79462E+00_JPRB, 0.69125E+00_JPRB, 0.64280E+00_JPRB, 0.63331E+00_JPRB, 0.77131E+00_JPRB, 0.12855E+01_JPRB ,&
&  0.13918E+01_JPRB, 0.17895E+01_JPRB, 0.18351E+01_JPRB, 0.18009E+01_JPRB, 0.16855E+01_JPRB, 0.16962E+01_JPRB, 0.19781E+01_JPRB ,&
&  0.16605E+01_JPRB, 0.14751E+01_JPRB, 0.12810E+01_JPRB, 0.11862E+01_JPRB, 0.11396E+01_JPRB, 0.11384E+01_JPRB, 0.11788E+01_JPRB ,&
&  0.11991E+01_JPRB, 0.11954E+01_JPRB, 0.11564E+01_JPRB, 0.11024E+01_JPRB, 0.10513E+01_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.95381E+00_JPRB, 0.92966E+00_JPRB, 0.91191E+00_JPRB, 0.87680E+00_JPRB, 0.84709E+00_JPRB, 0.80859E+00_JPRB, 0.75996E+00_JPRB ,&
&  0.73264E+00_JPRB, 0.71818E+00_JPRB, 0.70978E+00_JPRB, 0.69648E+00_JPRB, 0.68186E+00_JPRB, 0.66906E+00_JPRB, 0.66556E+00_JPRB ,&
&  0.64481E+00_JPRB, 0.61683E+00_JPRB, 0.61748E+00_JPRB, 0.64021E+00_JPRB, 0.67217E+00_JPRB, 0.75446E+00_JPRB, 0.10158E+01_JPRB ,&
&  0.16166E+01_JPRB, 0.18390E+01_JPRB, 0.16255E+01_JPRB, 0.14149E+01_JPRB, 0.13010E+01_JPRB, 0.12765E+01_JPRB, 0.11872E+01_JPRB ,&
&  0.10843E+01_JPRB, 0.10456E+01_JPRB, 0.93757E+00_JPRB, 0.93704E+00_JPRB, 0.94851E+00_JPRB, 0.96444E+00_JPRB, 0.97754E+00_JPRB ,&
&  0.10132E+01_JPRB, 0.97898E+00_JPRB, 0.98433E+00_JPRB, 0.10076E+01_JPRB, 0.10418E+01_JPRB, 0.11025E+01_JPRB, 0.11500E+01_JPRB ,&
&  0.12009E+01_JPRB, 0.12674E+01_JPRB, 0.14837E+01_JPRB, 0.18679E+01_JPRB, 0.19553E+01_JPRB, 0.24829E+01_JPRB, 0.27809E+01_JPRB ,&
&  0.30396E+01_JPRB, 0.31605E+01_JPRB, 0.31411E+01_JPRB, 0.37316E+01_JPRB, 0.40381E+01_JPRB, 0.35944E+01_JPRB, 0.31057E+01_JPRB ,&
&  0.31253E+01_JPRB, 0.27031E+01_JPRB, 0.23890E+01_JPRB, 0.21749E+01_JPRB, 0.27903E+01_JPRB, 0.45385E+01_JPRB, 0.35482E+01_JPRB ,&
&  0.34065E+01_JPRB, 0.38227E+01_JPRB, 0.31448E+01_JPRB, 0.26650E+01_JPRB, 0.28249E+01_JPRB, 0.29595E+01_JPRB, 0.22332E+01_JPRB ,&
&  0.10092E+01_JPRB, 0.76318E+00_JPRB, 0.75716E+00_JPRB, 0.73771E+00_JPRB, 0.65793E+00_JPRB, 0.77989E+00_JPRB, 0.18207E+01_JPRB ,&
&  0.15438E+01_JPRB, 0.19252E+01_JPRB, 0.18906E+01_JPRB, 0.17537E+01_JPRB, 0.16822E+01_JPRB, 0.15182E+01_JPRB, 0.16050E+01_JPRB ,&
&  0.13429E+01_JPRB, 0.12038E+01_JPRB, 0.11387E+01_JPRB, 0.10891E+01_JPRB, 0.10813E+01_JPRB, 0.10881E+01_JPRB, 0.11265E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.11563E+01_JPRB, 0.11188E+01_JPRB, 0.10567E+01_JPRB, 0.99935E+00_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.91648E+00_JPRB, 0.90266E+00_JPRB, 0.87471E+00_JPRB, 0.84547E+00_JPRB, 0.82676E+00_JPRB, 0.79022E+00_JPRB, 0.75972E+00_JPRB ,&
&  0.75561E+00_JPRB, 0.73770E+00_JPRB, 0.71864E+00_JPRB, 0.69928E+00_JPRB, 0.68186E+00_JPRB, 0.66306E+00_JPRB, 0.65653E+00_JPRB ,&
&  0.65058E+00_JPRB, 0.62108E+00_JPRB, 0.61605E+00_JPRB, 0.66229E+00_JPRB, 0.80363E+00_JPRB, 0.98946E+00_JPRB, 0.11418E+01_JPRB ,&
&  0.18155E+01_JPRB, 0.20018E+01_JPRB, 0.15242E+01_JPRB, 0.14197E+01_JPRB, 0.14038E+01_JPRB, 0.13520E+01_JPRB, 0.12071E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.10677E+01_JPRB, 0.96793E+00_JPRB, 0.96005E+00_JPRB, 0.98800E+00_JPRB, 0.10366E+01_JPRB, 0.10720E+01_JPRB ,&
&  0.10243E+01_JPRB, 0.10302E+01_JPRB, 0.10754E+01_JPRB, 0.11057E+01_JPRB, 0.11400E+01_JPRB, 0.12118E+01_JPRB, 0.12662E+01_JPRB ,&
&  0.13461E+01_JPRB, 0.14929E+01_JPRB, 0.17762E+01_JPRB, 0.19776E+01_JPRB, 0.22576E+01_JPRB, 0.24700E+01_JPRB, 0.27570E+01_JPRB ,&
&  0.27753E+01_JPRB, 0.28861E+01_JPRB, 0.31015E+01_JPRB, 0.33573E+01_JPRB, 0.35032E+01_JPRB, 0.33479E+01_JPRB, 0.31062E+01_JPRB ,&
&  0.30988E+01_JPRB, 0.26390E+01_JPRB, 0.22770E+01_JPRB, 0.21782E+01_JPRB, 0.26234E+01_JPRB, 0.40646E+01_JPRB, 0.44975E+01_JPRB ,&
&  0.44795E+01_JPRB, 0.32194E+01_JPRB, 0.27647E+01_JPRB, 0.29606E+01_JPRB, 0.27635E+01_JPRB, 0.26626E+01_JPRB, 0.31261E+01_JPRB ,&
&  0.27073E+01_JPRB, 0.15219E+01_JPRB, 0.12532E+01_JPRB, 0.14947E+01_JPRB, 0.98245E+00_JPRB, 0.80198E+00_JPRB, 0.12237E+01_JPRB ,&
&  0.14628E+01_JPRB, 0.17159E+01_JPRB, 0.17150E+01_JPRB, 0.16616E+01_JPRB, 0.16433E+01_JPRB, 0.13949E+01_JPRB, 0.13055E+01_JPRB ,&
&  0.11071E+01_JPRB, 0.10708E+01_JPRB, 0.10504E+01_JPRB, 0.99461E+00_JPRB, 0.97927E+00_JPRB, 0.10147E+01_JPRB, 0.10465E+01_JPRB ,&
&  0.10591E+01_JPRB, 0.10566E+01_JPRB, 0.10327E+01_JPRB, 0.99512E+00_JPRB, 0.94662E+00_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.88897E+00_JPRB, 0.87849E+00_JPRB, 0.87313E+00_JPRB, 0.84500E+00_JPRB, 0.81788E+00_JPRB, 0.79061E+00_JPRB, 0.79011E+00_JPRB ,&
&  0.76703E+00_JPRB, 0.74580E+00_JPRB, 0.72803E+00_JPRB, 0.69952E+00_JPRB, 0.69141E+00_JPRB, 0.67399E+00_JPRB, 0.68170E+00_JPRB ,&
&  0.69322E+00_JPRB, 0.67051E+00_JPRB, 0.69381E+00_JPRB, 0.85234E+00_JPRB, 0.11134E+01_JPRB, 0.16238E+01_JPRB, 0.13227E+01_JPRB ,&
&  0.17485E+01_JPRB, 0.21362E+01_JPRB, 0.15628E+01_JPRB, 0.14984E+01_JPRB, 0.14880E+01_JPRB, 0.14616E+01_JPRB, 0.12991E+01_JPRB ,&
&  0.11814E+01_JPRB, 0.11319E+01_JPRB, 0.10883E+01_JPRB, 0.10537E+01_JPRB, 0.10768E+01_JPRB, 0.10973E+01_JPRB, 0.10610E+01_JPRB ,&
&  0.10775E+01_JPRB, 0.11392E+01_JPRB, 0.11714E+01_JPRB, 0.11968E+01_JPRB, 0.12552E+01_JPRB, 0.13256E+01_JPRB, 0.14033E+01_JPRB ,&
&  0.15467E+01_JPRB, 0.17464E+01_JPRB, 0.20427E+01_JPRB, 0.21175E+01_JPRB, 0.22752E+01_JPRB, 0.24990E+01_JPRB, 0.26285E+01_JPRB ,&
&  0.25392E+01_JPRB, 0.24625E+01_JPRB, 0.28017E+01_JPRB, 0.31136E+01_JPRB, 0.31547E+01_JPRB, 0.30513E+01_JPRB, 0.30201E+01_JPRB ,&
&  0.31791E+01_JPRB, 0.30234E+01_JPRB, 0.30437E+01_JPRB, 0.26548E+01_JPRB, 0.31064E+01_JPRB, 0.40721E+01_JPRB, 0.40960E+01_JPRB ,&
&  0.48158E+01_JPRB, 0.40621E+01_JPRB, 0.36652E+01_JPRB, 0.31561E+01_JPRB, 0.26991E+01_JPRB, 0.24917E+01_JPRB, 0.25197E+01_JPRB ,&
&  0.24941E+01_JPRB, 0.26131E+01_JPRB, 0.24270E+01_JPRB, 0.15280E+01_JPRB, 0.10683E+01_JPRB, 0.89471E+00_JPRB, 0.10190E+01_JPRB ,&
&  0.13098E+01_JPRB, 0.14199E+01_JPRB, 0.14371E+01_JPRB, 0.20203E+01_JPRB, 0.14638E+01_JPRB, 0.12153E+01_JPRB, 0.10724E+01_JPRB ,&
&  0.98157E+00_JPRB, 0.97769E+00_JPRB, 0.95059E+00_JPRB, 0.90804E+00_JPRB, 0.91214E+00_JPRB, 0.93076E+00_JPRB, 0.95415E+00_JPRB ,&
&  0.96674E+00_JPRB, 0.97936E+00_JPRB, 0.98583E+00_JPRB, 0.96374E+00_JPRB, 0.92237E+00_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.85914E+00_JPRB, 0.86024E+00_JPRB, 0.84439E+00_JPRB, 0.82541E+00_JPRB, 0.81413E+00_JPRB, 0.80908E+00_JPRB, 0.82209E+00_JPRB ,&
&  0.76013E+00_JPRB, 0.73564E+00_JPRB, 0.72444E+00_JPRB, 0.71261E+00_JPRB, 0.72484E+00_JPRB, 0.74096E+00_JPRB, 0.77099E+00_JPRB ,&
&  0.81080E+00_JPRB, 0.85300E+00_JPRB, 0.98965E+00_JPRB, 0.12882E+01_JPRB, 0.16415E+01_JPRB, 0.20879E+01_JPRB, 0.20048E+01_JPRB ,&
&  0.16975E+01_JPRB, 0.23296E+01_JPRB, 0.15969E+01_JPRB, 0.16334E+01_JPRB, 0.16034E+01_JPRB, 0.17152E+01_JPRB, 0.15848E+01_JPRB ,&
&  0.13916E+01_JPRB, 0.13072E+01_JPRB, 0.12591E+01_JPRB, 0.11625E+01_JPRB, 0.11458E+01_JPRB, 0.11211E+01_JPRB, 0.11283E+01_JPRB ,&
&  0.11904E+01_JPRB, 0.12571E+01_JPRB, 0.13004E+01_JPRB, 0.13508E+01_JPRB, 0.14207E+01_JPRB, 0.14862E+01_JPRB, 0.16141E+01_JPRB ,&
&  0.18239E+01_JPRB, 0.20598E+01_JPRB, 0.21616E+01_JPRB, 0.22097E+01_JPRB, 0.22522E+01_JPRB, 0.24382E+01_JPRB, 0.25584E+01_JPRB ,&
&  0.25357E+01_JPRB, 0.24653E+01_JPRB, 0.26782E+01_JPRB, 0.27378E+01_JPRB, 0.27333E+01_JPRB, 0.29540E+01_JPRB, 0.29403E+01_JPRB ,&
&  0.32412E+01_JPRB, 0.30302E+01_JPRB, 0.34650E+01_JPRB, 0.32227E+01_JPRB, 0.33583E+01_JPRB, 0.37076E+01_JPRB, 0.32245E+01_JPRB ,&
&  0.30831E+01_JPRB, 0.32151E+01_JPRB, 0.31391E+01_JPRB, 0.29702E+01_JPRB, 0.27031E+01_JPRB, 0.24563E+01_JPRB, 0.21738E+01_JPRB ,&
&  0.18689E+01_JPRB, 0.17768E+01_JPRB, 0.16162E+01_JPRB, 0.10939E+01_JPRB, 0.89100E+00_JPRB, 0.82510E+00_JPRB, 0.92066E+00_JPRB ,&
&  0.11670E+01_JPRB, 0.13300E+01_JPRB, 0.15982E+01_JPRB, 0.15767E+01_JPRB, 0.11640E+01_JPRB, 0.97099E+00_JPRB, 0.92822E+00_JPRB ,&
&  0.89800E+00_JPRB, 0.88158E+00_JPRB, 0.83294E+00_JPRB, 0.80953E+00_JPRB, 0.82865E+00_JPRB, 0.86381E+00_JPRB, 0.87799E+00_JPRB ,&
&  0.90426E+00_JPRB, 0.91713E+00_JPRB, 0.90651E+00_JPRB, 0.88120E+00_JPRB, 0.86667E+00_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.90804E+00_JPRB, 0.92686E+00_JPRB, 0.94676E+00_JPRB, 0.98743E+00_JPRB, 0.10459E+01_JPRB, 0.11217E+01_JPRB, 0.11249E+01_JPRB ,&
&  0.97662E+00_JPRB, 0.73304E+00_JPRB, 0.74932E+00_JPRB, 0.78331E+00_JPRB, 0.84918E+00_JPRB, 0.91186E+00_JPRB, 0.98605E+00_JPRB ,&
&  0.10953E+01_JPRB, 0.12430E+01_JPRB, 0.14962E+01_JPRB, 0.18113E+01_JPRB, 0.21386E+01_JPRB, 0.25477E+01_JPRB, 0.25227E+01_JPRB ,&
&  0.20823E+01_JPRB, 0.17500E+01_JPRB, 0.18317E+01_JPRB, 0.18227E+01_JPRB, 0.16881E+01_JPRB, 0.18211E+01_JPRB, 0.17880E+01_JPRB ,&
&  0.15737E+01_JPRB, 0.14591E+01_JPRB, 0.13995E+01_JPRB, 0.13145E+01_JPRB, 0.12647E+01_JPRB, 0.13001E+01_JPRB, 0.13611E+01_JPRB ,&
&  0.14429E+01_JPRB, 0.14969E+01_JPRB, 0.15217E+01_JPRB, 0.15485E+01_JPRB, 0.16121E+01_JPRB, 0.16894E+01_JPRB, 0.18774E+01_JPRB ,&
&  0.20960E+01_JPRB, 0.22658E+01_JPRB, 0.22995E+01_JPRB, 0.22158E+01_JPRB, 0.22701E+01_JPRB, 0.23819E+01_JPRB, 0.24440E+01_JPRB ,&
&  0.24780E+01_JPRB, 0.25624E+01_JPRB, 0.26297E+01_JPRB, 0.26763E+01_JPRB, 0.25734E+01_JPRB, 0.28943E+01_JPRB, 0.28949E+01_JPRB ,&
&  0.30477E+01_JPRB, 0.29370E+01_JPRB, 0.33571E+01_JPRB, 0.29915E+01_JPRB, 0.32784E+01_JPRB, 0.31870E+01_JPRB, 0.28365E+01_JPRB ,&
&  0.27181E+01_JPRB, 0.29107E+01_JPRB, 0.29465E+01_JPRB, 0.27956E+01_JPRB, 0.26464E+01_JPRB, 0.21860E+01_JPRB, 0.21167E+01_JPRB ,&
&  0.16509E+01_JPRB, 0.16600E+01_JPRB, 0.12087E+01_JPRB, 0.91157E+00_JPRB, 0.74449E+00_JPRB, 0.80450E+00_JPRB, 0.92879E+00_JPRB ,&
&  0.11749E+01_JPRB, 0.12399E+01_JPRB, 0.12041E+01_JPRB, 0.12012E+01_JPRB, 0.10523E+01_JPRB, 0.89804E+00_JPRB, 0.87606E+00_JPRB ,&
&  0.87902E+00_JPRB, 0.84378E+00_JPRB, 0.79279E+00_JPRB, 0.79045E+00_JPRB, 0.81201E+00_JPRB, 0.86125E+00_JPRB, 0.87234E+00_JPRB ,&
&  0.89059E+00_JPRB, 0.88834E+00_JPRB, 0.87810E+00_JPRB, 0.87383E+00_JPRB, 0.88399E+00_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.94712E+00_JPRB, 0.94519E+00_JPRB, 0.95972E+00_JPRB, 0.96572E+00_JPRB, 0.96429E+00_JPRB, 0.91312E+00_JPRB, 0.87282E+00_JPRB ,&
&  0.82299E+00_JPRB, 0.77693E+00_JPRB, 0.84543E+00_JPRB, 0.93399E+00_JPRB, 0.10406E+01_JPRB, 0.11552E+01_JPRB, 0.12762E+01_JPRB ,&
&  0.14353E+01_JPRB, 0.16340E+01_JPRB, 0.18817E+01_JPRB, 0.20807E+01_JPRB, 0.22668E+01_JPRB, 0.25235E+01_JPRB, 0.27242E+01_JPRB ,&
&  0.23098E+01_JPRB, 0.15678E+01_JPRB, 0.18973E+01_JPRB, 0.20658E+01_JPRB, 0.18720E+01_JPRB, 0.18642E+01_JPRB, 0.18085E+01_JPRB ,&
&  0.16796E+01_JPRB, 0.16139E+01_JPRB, 0.15846E+01_JPRB, 0.15581E+01_JPRB, 0.15307E+01_JPRB, 0.15823E+01_JPRB, 0.16327E+01_JPRB ,&
&  0.16567E+01_JPRB, 0.16450E+01_JPRB, 0.16364E+01_JPRB, 0.17114E+01_JPRB, 0.18633E+01_JPRB, 0.19518E+01_JPRB, 0.21067E+01_JPRB ,&
&  0.22428E+01_JPRB, 0.24099E+01_JPRB, 0.24619E+01_JPRB, 0.24268E+01_JPRB, 0.25018E+01_JPRB, 0.25619E+01_JPRB, 0.25592E+01_JPRB ,&
&  0.25414E+01_JPRB, 0.25365E+01_JPRB, 0.26207E+01_JPRB, 0.28843E+01_JPRB, 0.30356E+01_JPRB, 0.27629E+01_JPRB, 0.28815E+01_JPRB ,&
&  0.29572E+01_JPRB, 0.30318E+01_JPRB, 0.29407E+01_JPRB, 0.37707E+01_JPRB, 0.28402E+01_JPRB, 0.27808E+01_JPRB, 0.23632E+01_JPRB ,&
&  0.25932E+01_JPRB, 0.26184E+01_JPRB, 0.27003E+01_JPRB, 0.26961E+01_JPRB, 0.25078E+01_JPRB, 0.20911E+01_JPRB, 0.20342E+01_JPRB ,&
&  0.17618E+01_JPRB, 0.15535E+01_JPRB, 0.11699E+01_JPRB, 0.90780E+00_JPRB, 0.85112E+00_JPRB, 0.11610E+01_JPRB, 0.11962E+01_JPRB ,&
&  0.11877E+01_JPRB, 0.11119E+01_JPRB, 0.12067E+01_JPRB, 0.12857E+01_JPRB, 0.11238E+01_JPRB, 0.87142E+00_JPRB, 0.82464E+00_JPRB ,&
&  0.83215E+00_JPRB, 0.82941E+00_JPRB, 0.80065E+00_JPRB, 0.80654E+00_JPRB, 0.83700E+00_JPRB, 0.88866E+00_JPRB, 0.91631E+00_JPRB ,&
&  0.92085E+00_JPRB, 0.90502E+00_JPRB, 0.90409E+00_JPRB, 0.91281E+00_JPRB, 0.94162E+00_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.91153E+00_JPRB, 0.89668E+00_JPRB, 0.88088E+00_JPRB, 0.85818E+00_JPRB, 0.82727E+00_JPRB, 0.80667E+00_JPRB, 0.80950E+00_JPRB ,&
&  0.81895E+00_JPRB, 0.86201E+00_JPRB, 0.93497E+00_JPRB, 0.10252E+01_JPRB, 0.11452E+01_JPRB, 0.12834E+01_JPRB, 0.14305E+01_JPRB ,&
&  0.16235E+01_JPRB, 0.18171E+01_JPRB, 0.19423E+01_JPRB, 0.20422E+01_JPRB, 0.21631E+01_JPRB, 0.23584E+01_JPRB, 0.25704E+01_JPRB ,&
&  0.27330E+01_JPRB, 0.26957E+01_JPRB, 0.24945E+01_JPRB, 0.18372E+01_JPRB, 0.15697E+01_JPRB, 0.18372E+01_JPRB, 0.18329E+01_JPRB ,&
&  0.18103E+01_JPRB, 0.17539E+01_JPRB, 0.17190E+01_JPRB, 0.16869E+01_JPRB, 0.16961E+01_JPRB, 0.17344E+01_JPRB, 0.17590E+01_JPRB ,&
&  0.17556E+01_JPRB, 0.17623E+01_JPRB, 0.18037E+01_JPRB, 0.19092E+01_JPRB, 0.20767E+01_JPRB, 0.21807E+01_JPRB, 0.22920E+01_JPRB ,&
&  0.23879E+01_JPRB, 0.25511E+01_JPRB, 0.26485E+01_JPRB, 0.27226E+01_JPRB, 0.27289E+01_JPRB, 0.27797E+01_JPRB, 0.28762E+01_JPRB ,&
&  0.28999E+01_JPRB, 0.29472E+01_JPRB, 0.30669E+01_JPRB, 0.32813E+01_JPRB, 0.33744E+01_JPRB, 0.29765E+01_JPRB, 0.28889E+01_JPRB ,&
&  0.30930E+01_JPRB, 0.33270E+01_JPRB, 0.30805E+01_JPRB, 0.39088E+01_JPRB, 0.24765E+01_JPRB, 0.26079E+01_JPRB, 0.25013E+01_JPRB ,&
&  0.23573E+01_JPRB, 0.22946E+01_JPRB, 0.23934E+01_JPRB, 0.24446E+01_JPRB, 0.23302E+01_JPRB, 0.19271E+01_JPRB, 0.19977E+01_JPRB ,&
&  0.18754E+01_JPRB, 0.14427E+01_JPRB, 0.11305E+01_JPRB, 0.84448E+00_JPRB, 0.96439E+00_JPRB, 0.13619E+01_JPRB, 0.11854E+01_JPRB ,&
&  0.95102E+00_JPRB, 0.11244E+01_JPRB, 0.12591E+01_JPRB, 0.13715E+01_JPRB, 0.12553E+01_JPRB, 0.79737E+00_JPRB, 0.79169E+00_JPRB ,&
&  0.81663E+00_JPRB, 0.83182E+00_JPRB, 0.85054E+00_JPRB, 0.85496E+00_JPRB, 0.87356E+00_JPRB, 0.91183E+00_JPRB, 0.94585E+00_JPRB ,&
&  0.95351E+00_JPRB, 0.94542E+00_JPRB, 0.95090E+00_JPRB, 0.94294E+00_JPRB, 0.92745E+00_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.83174E+00_JPRB, 0.79855E+00_JPRB, 0.77216E+00_JPRB, 0.76091E+00_JPRB, 0.76143E+00_JPRB, 0.77388E+00_JPRB, 0.79743E+00_JPRB ,&
&  0.83805E+00_JPRB, 0.89339E+00_JPRB, 0.95082E+00_JPRB, 0.10030E+01_JPRB, 0.10771E+01_JPRB, 0.11677E+01_JPRB, 0.12855E+01_JPRB ,&
&  0.14176E+01_JPRB, 0.15577E+01_JPRB, 0.16318E+01_JPRB, 0.16632E+01_JPRB, 0.17311E+01_JPRB, 0.18372E+01_JPRB, 0.20783E+01_JPRB ,&
&  0.24207E+01_JPRB, 0.27061E+01_JPRB, 0.28495E+01_JPRB, 0.26116E+01_JPRB, 0.20018E+01_JPRB, 0.19375E+01_JPRB, 0.21028E+01_JPRB ,&
&  0.21615E+01_JPRB, 0.19617E+01_JPRB, 0.18245E+01_JPRB, 0.18033E+01_JPRB, 0.18044E+01_JPRB, 0.18409E+01_JPRB, 0.18647E+01_JPRB ,&
&  0.18708E+01_JPRB, 0.18907E+01_JPRB, 0.19538E+01_JPRB, 0.20363E+01_JPRB, 0.21637E+01_JPRB, 0.22998E+01_JPRB, 0.24506E+01_JPRB ,&
&  0.25246E+01_JPRB, 0.26174E+01_JPRB, 0.26237E+01_JPRB, 0.25657E+01_JPRB, 0.26090E+01_JPRB, 0.28147E+01_JPRB, 0.31244E+01_JPRB ,&
&  0.33181E+01_JPRB, 0.32946E+01_JPRB, 0.35124E+01_JPRB, 0.37683E+01_JPRB, 0.37197E+01_JPRB, 0.34220E+01_JPRB, 0.33687E+01_JPRB ,&
&  0.34966E+01_JPRB, 0.38740E+01_JPRB, 0.28477E+01_JPRB, 0.32830E+01_JPRB, 0.33702E+01_JPRB, 0.28537E+01_JPRB, 0.23106E+01_JPRB ,&
&  0.20823E+01_JPRB, 0.18436E+01_JPRB, 0.16485E+01_JPRB, 0.16952E+01_JPRB, 0.17399E+01_JPRB, 0.17642E+01_JPRB, 0.17556E+01_JPRB ,&
&  0.16953E+01_JPRB, 0.13419E+01_JPRB, 0.10198E+01_JPRB, 0.84859E+00_JPRB, 0.92649E+00_JPRB, 0.11377E+01_JPRB, 0.10212E+01_JPRB ,&
&  0.93945E+00_JPRB, 0.10792E+01_JPRB, 0.11596E+01_JPRB, 0.13957E+01_JPRB, 0.14861E+01_JPRB, 0.77856E+00_JPRB, 0.78810E+00_JPRB ,&
&  0.80669E+00_JPRB, 0.84002E+00_JPRB, 0.85067E+00_JPRB, 0.85555E+00_JPRB, 0.85985E+00_JPRB, 0.88447E+00_JPRB, 0.91273E+00_JPRB ,&
&  0.92338E+00_JPRB, 0.92410E+00_JPRB, 0.91955E+00_JPRB, 0.89768E+00_JPRB, 0.86096E+00_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.74476E+00_JPRB, 0.74271E+00_JPRB, 0.73179E+00_JPRB, 0.74367E+00_JPRB, 0.75827E+00_JPRB, 0.75638E+00_JPRB, 0.77196E+00_JPRB ,&
&  0.79444E+00_JPRB, 0.81709E+00_JPRB, 0.83133E+00_JPRB, 0.84077E+00_JPRB, 0.89364E+00_JPRB, 0.95809E+00_JPRB, 0.10361E+01_JPRB ,&
&  0.10803E+01_JPRB, 0.11214E+01_JPRB, 0.11299E+01_JPRB, 0.11143E+01_JPRB, 0.11043E+01_JPRB, 0.10632E+01_JPRB, 0.10118E+01_JPRB ,&
&  0.10774E+01_JPRB, 0.13713E+01_JPRB, 0.18432E+01_JPRB, 0.22717E+01_JPRB, 0.26456E+01_JPRB, 0.22429E+01_JPRB, 0.23496E+01_JPRB ,&
&  0.20542E+01_JPRB, 0.16724E+01_JPRB, 0.17138E+01_JPRB, 0.18147E+01_JPRB, 0.19060E+01_JPRB, 0.19013E+01_JPRB, 0.19378E+01_JPRB ,&
&  0.19626E+01_JPRB, 0.19898E+01_JPRB, 0.20367E+01_JPRB, 0.21364E+01_JPRB, 0.22256E+01_JPRB, 0.22972E+01_JPRB, 0.23897E+01_JPRB ,&
&  0.24597E+01_JPRB, 0.24926E+01_JPRB, 0.25407E+01_JPRB, 0.23768E+01_JPRB, 0.25539E+01_JPRB, 0.29123E+01_JPRB, 0.32253E+01_JPRB ,&
&  0.33726E+01_JPRB, 0.34484E+01_JPRB, 0.37573E+01_JPRB, 0.38958E+01_JPRB, 0.39862E+01_JPRB, 0.39333E+01_JPRB, 0.39285E+01_JPRB ,&
&  0.39475E+01_JPRB, 0.36167E+01_JPRB, 0.26418E+01_JPRB, 0.28034E+01_JPRB, 0.24980E+01_JPRB, 0.19980E+01_JPRB, 0.19745E+01_JPRB ,&
&  0.17709E+01_JPRB, 0.15786E+01_JPRB, 0.13632E+01_JPRB, 0.13206E+01_JPRB, 0.13602E+01_JPRB, 0.14163E+01_JPRB, 0.15161E+01_JPRB ,&
&  0.13405E+01_JPRB, 0.11224E+01_JPRB, 0.91612E+00_JPRB, 0.86854E+00_JPRB, 0.85096E+00_JPRB, 0.90459E+00_JPRB, 0.93531E+00_JPRB ,&
&  0.96636E+00_JPRB, 0.88771E+00_JPRB, 0.90937E+00_JPRB, 0.97838E+00_JPRB, 0.78581E+00_JPRB, 0.72387E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.74299E+00_JPRB, 0.75119E+00_JPRB, 0.75634E+00_JPRB, 0.75507E+00_JPRB, 0.76530E+00_JPRB, 0.78337E+00_JPRB, 0.79252E+00_JPRB ,&
&  0.79302E+00_JPRB, 0.77563E+00_JPRB, 0.76124E+00_JPRB, 0.75137E+00_JPRB, 0.74281E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.62181E+00_JPRB, 0.62328E+00_JPRB, 0.62548E+00_JPRB, 0.63333E+00_JPRB, 0.63373E+00_JPRB, 0.62989E+00_JPRB, 0.61854E+00_JPRB ,&
&  0.61144E+00_JPRB, 0.59751E+00_JPRB, 0.58820E+00_JPRB, 0.58220E+00_JPRB, 0.59645E+00_JPRB, 0.62978E+00_JPRB, 0.68058E+00_JPRB ,&
&  0.69555E+00_JPRB, 0.69276E+00_JPRB, 0.66663E+00_JPRB, 0.65164E+00_JPRB, 0.67411E+00_JPRB, 0.67749E+00_JPRB, 0.65349E+00_JPRB ,&
&  0.63274E+00_JPRB, 0.69551E+00_JPRB, 0.86442E+00_JPRB, 0.11085E+01_JPRB, 0.15428E+01_JPRB, 0.17686E+01_JPRB, 0.22366E+01_JPRB ,&
&  0.20716E+01_JPRB, 0.15189E+01_JPRB, 0.16344E+01_JPRB, 0.15751E+01_JPRB, 0.16822E+01_JPRB, 0.17404E+01_JPRB, 0.17615E+01_JPRB ,&
&  0.18154E+01_JPRB, 0.18685E+01_JPRB, 0.19189E+01_JPRB, 0.19752E+01_JPRB, 0.20168E+01_JPRB, 0.20210E+01_JPRB, 0.20321E+01_JPRB ,&
&  0.20377E+01_JPRB, 0.20234E+01_JPRB, 0.20748E+01_JPRB, 0.22202E+01_JPRB, 0.25689E+01_JPRB, 0.29642E+01_JPRB, 0.30265E+01_JPRB ,&
&  0.30606E+01_JPRB, 0.32855E+01_JPRB, 0.32309E+01_JPRB, 0.34476E+01_JPRB, 0.35424E+01_JPRB, 0.34401E+01_JPRB, 0.36710E+01_JPRB ,&
&  0.38107E+01_JPRB, 0.31962E+01_JPRB, 0.21392E+01_JPRB, 0.22156E+01_JPRB, 0.20255E+01_JPRB, 0.18209E+01_JPRB, 0.18174E+01_JPRB ,&
&  0.16692E+01_JPRB, 0.15339E+01_JPRB, 0.13938E+01_JPRB, 0.13090E+01_JPRB, 0.12655E+01_JPRB, 0.11892E+01_JPRB, 0.11673E+01_JPRB ,&
&  0.10692E+01_JPRB, 0.10146E+01_JPRB, 0.97091E+00_JPRB, 0.93417E+00_JPRB, 0.93276E+00_JPRB, 0.96615E+00_JPRB, 0.87454E+00_JPRB ,&
&  0.88554E+00_JPRB, 0.83534E+00_JPRB, 0.91716E+00_JPRB, 0.96951E+00_JPRB, 0.68088E+00_JPRB, 0.62942E+00_JPRB, 0.60599E+00_JPRB ,&
&  0.60880E+00_JPRB, 0.60351E+00_JPRB, 0.60034E+00_JPRB, 0.59291E+00_JPRB, 0.61588E+00_JPRB, 0.65157E+00_JPRB, 0.64820E+00_JPRB ,&
&  0.65542E+00_JPRB, 0.63437E+00_JPRB, 0.63393E+00_JPRB, 0.63895E+00_JPRB, 0.64109E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.59218E+00_JPRB, 0.58174E+00_JPRB, 0.57145E+00_JPRB, 0.56088E+00_JPRB, 0.56699E+00_JPRB, 0.56810E+00_JPRB, 0.57813E+00_JPRB ,&
&  0.58963E+00_JPRB, 0.58960E+00_JPRB, 0.59445E+00_JPRB, 0.59611E+00_JPRB, 0.61119E+00_JPRB, 0.62440E+00_JPRB, 0.65573E+00_JPRB ,&
&  0.68539E+00_JPRB, 0.70088E+00_JPRB, 0.72188E+00_JPRB, 0.74228E+00_JPRB, 0.75681E+00_JPRB, 0.77195E+00_JPRB, 0.75654E+00_JPRB ,&
&  0.73779E+00_JPRB, 0.73749E+00_JPRB, 0.77082E+00_JPRB, 0.89517E+00_JPRB, 0.11330E+01_JPRB, 0.14442E+01_JPRB, 0.20584E+01_JPRB ,&
&  0.19989E+01_JPRB, 0.13849E+01_JPRB, 0.12482E+01_JPRB, 0.11065E+01_JPRB, 0.11628E+01_JPRB, 0.12522E+01_JPRB, 0.12591E+01_JPRB ,&
&  0.12670E+01_JPRB, 0.12729E+01_JPRB, 0.12653E+01_JPRB, 0.12431E+01_JPRB, 0.11831E+01_JPRB, 0.11074E+01_JPRB, 0.10636E+01_JPRB ,&
&  0.10663E+01_JPRB, 0.10639E+01_JPRB, 0.12175E+01_JPRB, 0.16186E+01_JPRB, 0.19271E+01_JPRB, 0.24548E+01_JPRB, 0.23316E+01_JPRB ,&
&  0.23568E+01_JPRB, 0.24517E+01_JPRB, 0.23991E+01_JPRB, 0.25630E+01_JPRB, 0.26345E+01_JPRB, 0.27389E+01_JPRB, 0.27450E+01_JPRB ,&
&  0.29143E+01_JPRB, 0.25499E+01_JPRB, 0.18096E+01_JPRB, 0.19384E+01_JPRB, 0.18049E+01_JPRB, 0.17799E+01_JPRB, 0.16553E+01_JPRB ,&
&  0.15539E+01_JPRB, 0.14365E+01_JPRB, 0.13409E+01_JPRB, 0.12636E+01_JPRB, 0.12320E+01_JPRB, 0.11953E+01_JPRB, 0.10920E+01_JPRB ,&
&  0.10572E+01_JPRB, 0.10274E+01_JPRB, 0.10023E+01_JPRB, 0.94675E+00_JPRB, 0.10032E+01_JPRB, 0.89063E+00_JPRB, 0.10967E+01_JPRB ,&
&  0.80769E+00_JPRB, 0.73877E+00_JPRB, 0.61351E+00_JPRB, 0.75918E+00_JPRB, 0.62487E+00_JPRB, 0.55717E+00_JPRB, 0.56164E+00_JPRB ,&
&  0.54441E+00_JPRB, 0.52520E+00_JPRB, 0.51042E+00_JPRB, 0.51131E+00_JPRB, 0.52475E+00_JPRB, 0.54888E+00_JPRB, 0.55178E+00_JPRB ,&
&  0.56051E+00_JPRB, 0.57318E+00_JPRB, 0.59162E+00_JPRB, 0.61271E+00_JPRB, 0.60974E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.61082E+00_JPRB, 0.61107E+00_JPRB, 0.61929E+00_JPRB, 0.63455E+00_JPRB, 0.64216E+00_JPRB, 0.64779E+00_JPRB, 0.65187E+00_JPRB ,&
&  0.66982E+00_JPRB, 0.69050E+00_JPRB, 0.70968E+00_JPRB, 0.72862E+00_JPRB, 0.74569E+00_JPRB, 0.76031E+00_JPRB, 0.79646E+00_JPRB ,&
&  0.81525E+00_JPRB, 0.84144E+00_JPRB, 0.85944E+00_JPRB, 0.87883E+00_JPRB, 0.89570E+00_JPRB, 0.88849E+00_JPRB, 0.89357E+00_JPRB ,&
&  0.90332E+00_JPRB, 0.93128E+00_JPRB, 0.95964E+00_JPRB, 0.10104E+01_JPRB, 0.10762E+01_JPRB, 0.10908E+01_JPRB, 0.12272E+01_JPRB ,&
&  0.11969E+01_JPRB, 0.13607E+01_JPRB, 0.11922E+01_JPRB, 0.10203E+01_JPRB, 0.97876E+00_JPRB, 0.88195E+00_JPRB, 0.83458E+00_JPRB ,&
&  0.85045E+00_JPRB, 0.85360E+00_JPRB, 0.83609E+00_JPRB, 0.81025E+00_JPRB, 0.77734E+00_JPRB, 0.83452E+00_JPRB, 0.93361E+00_JPRB ,&
&  0.11186E+01_JPRB, 0.12262E+01_JPRB, 0.13677E+01_JPRB, 0.14884E+01_JPRB, 0.16464E+01_JPRB, 0.17314E+01_JPRB, 0.17063E+01_JPRB ,&
&  0.17040E+01_JPRB, 0.16794E+01_JPRB, 0.15753E+01_JPRB, 0.18025E+01_JPRB, 0.19831E+01_JPRB, 0.20671E+01_JPRB, 0.20864E+01_JPRB ,&
&  0.20693E+01_JPRB, 0.18406E+01_JPRB, 0.17867E+01_JPRB, 0.16863E+01_JPRB, 0.15742E+01_JPRB, 0.15559E+01_JPRB, 0.14610E+01_JPRB ,&
&  0.13615E+01_JPRB, 0.12928E+01_JPRB, 0.12676E+01_JPRB, 0.12657E+01_JPRB, 0.12725E+01_JPRB, 0.12632E+01_JPRB, 0.11617E+01_JPRB ,&
&  0.10870E+01_JPRB, 0.10256E+01_JPRB, 0.97793E+00_JPRB, 0.93505E+00_JPRB, 0.83871E+00_JPRB, 0.12676E+01_JPRB, 0.16825E+01_JPRB ,&
&  0.73284E+00_JPRB, 0.56099E+00_JPRB, 0.42888E+00_JPRB, 0.54701E+00_JPRB, 0.55066E+00_JPRB, 0.58266E+00_JPRB, 0.65371E+00_JPRB ,&
&  0.66920E+00_JPRB, 0.65744E+00_JPRB, 0.59456E+00_JPRB, 0.53351E+00_JPRB, 0.54802E+00_JPRB, 0.55129E+00_JPRB, 0.55379E+00_JPRB ,&
&  0.56861E+00_JPRB, 0.57771E+00_JPRB, 0.58949E+00_JPRB, 0.61001E+00_JPRB, 0.60582E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.63846E+00_JPRB, 0.65255E+00_JPRB, 0.67117E+00_JPRB, 0.69595E+00_JPRB, 0.71957E+00_JPRB, 0.73804E+00_JPRB, 0.75053E+00_JPRB ,&
&  0.77071E+00_JPRB, 0.79716E+00_JPRB, 0.81307E+00_JPRB, 0.82092E+00_JPRB, 0.83893E+00_JPRB, 0.85514E+00_JPRB, 0.88424E+00_JPRB ,&
&  0.90250E+00_JPRB, 0.94382E+00_JPRB, 0.97597E+00_JPRB, 0.10079E+01_JPRB, 0.10328E+01_JPRB, 0.10625E+01_JPRB, 0.10905E+01_JPRB ,&
&  0.11134E+01_JPRB, 0.11586E+01_JPRB, 0.11972E+01_JPRB, 0.12244E+01_JPRB, 0.12707E+01_JPRB, 0.13125E+01_JPRB, 0.10864E+01_JPRB ,&
&  0.13272E+01_JPRB, 0.12639E+01_JPRB, 0.11220E+01_JPRB, 0.10109E+01_JPRB, 0.93456E+00_JPRB, 0.88887E+00_JPRB, 0.87080E+00_JPRB ,&
&  0.89559E+00_JPRB, 0.98888E+00_JPRB, 0.10624E+01_JPRB, 0.10771E+01_JPRB, 0.10628E+01_JPRB, 0.11796E+01_JPRB, 0.14169E+01_JPRB ,&
&  0.15510E+01_JPRB, 0.16243E+01_JPRB, 0.16757E+01_JPRB, 0.16818E+01_JPRB, 0.17510E+01_JPRB, 0.18586E+01_JPRB, 0.19113E+01_JPRB ,&
&  0.19001E+01_JPRB, 0.18304E+01_JPRB, 0.16218E+01_JPRB, 0.19550E+01_JPRB, 0.19397E+01_JPRB, 0.18476E+01_JPRB, 0.16625E+01_JPRB ,&
&  0.14718E+01_JPRB, 0.13006E+01_JPRB, 0.13504E+01_JPRB, 0.14377E+01_JPRB, 0.13731E+01_JPRB, 0.13192E+01_JPRB, 0.12177E+01_JPRB ,&
&  0.11336E+01_JPRB, 0.11084E+01_JPRB, 0.11292E+01_JPRB, 0.11761E+01_JPRB, 0.12109E+01_JPRB, 0.12029E+01_JPRB, 0.11192E+01_JPRB ,&
&  0.10358E+01_JPRB, 0.94363E+00_JPRB, 0.87950E+00_JPRB, 0.82801E+00_JPRB, 0.76940E+00_JPRB, 0.75107E+00_JPRB, 0.10312E+01_JPRB ,&
&  0.63451E+00_JPRB, 0.42622E+00_JPRB, 0.46262E+00_JPRB, 0.50363E+00_JPRB, 0.54680E+00_JPRB, 0.60107E+00_JPRB, 0.76331E+00_JPRB ,&
&  0.97356E+00_JPRB, 0.11215E+01_JPRB, 0.10979E+01_JPRB, 0.83524E+00_JPRB, 0.65752E+00_JPRB, 0.60236E+00_JPRB, 0.60097E+00_JPRB ,&
&  0.61037E+00_JPRB, 0.61019E+00_JPRB, 0.61436E+00_JPRB, 0.62040E+00_JPRB, 0.63029E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.62319E+00_JPRB, 0.63339E+00_JPRB, 0.64684E+00_JPRB, 0.66136E+00_JPRB, 0.67900E+00_JPRB, 0.69692E+00_JPRB, 0.71075E+00_JPRB ,&
&  0.72519E+00_JPRB, 0.74206E+00_JPRB, 0.75779E+00_JPRB, 0.76856E+00_JPRB, 0.78085E+00_JPRB, 0.80450E+00_JPRB, 0.83233E+00_JPRB ,&
&  0.84462E+00_JPRB, 0.88863E+00_JPRB, 0.90881E+00_JPRB, 0.94996E+00_JPRB, 0.98740E+00_JPRB, 0.10333E+01_JPRB, 0.10674E+01_JPRB ,&
&  0.11005E+01_JPRB, 0.11491E+01_JPRB, 0.11644E+01_JPRB, 0.11531E+01_JPRB, 0.11657E+01_JPRB, 0.12405E+01_JPRB, 0.10120E+01_JPRB ,&
&  0.12825E+01_JPRB, 0.12671E+01_JPRB, 0.11585E+01_JPRB, 0.11134E+01_JPRB, 0.10815E+01_JPRB, 0.10957E+01_JPRB, 0.11312E+01_JPRB ,&
&  0.11915E+01_JPRB, 0.13421E+01_JPRB, 0.13655E+01_JPRB, 0.14207E+01_JPRB, 0.15199E+01_JPRB, 0.17255E+01_JPRB, 0.19147E+01_JPRB ,&
&  0.19848E+01_JPRB, 0.21302E+01_JPRB, 0.21977E+01_JPRB, 0.22639E+01_JPRB, 0.23770E+01_JPRB, 0.24938E+01_JPRB, 0.25744E+01_JPRB ,&
&  0.25412E+01_JPRB, 0.24821E+01_JPRB, 0.22837E+01_JPRB, 0.24398E+01_JPRB, 0.22987E+01_JPRB, 0.18626E+01_JPRB, 0.14538E+01_JPRB ,&
&  0.10658E+01_JPRB, 0.10925E+01_JPRB, 0.11582E+01_JPRB, 0.12150E+01_JPRB, 0.11345E+01_JPRB, 0.10654E+01_JPRB, 0.92138E+00_JPRB ,&
&  0.85940E+00_JPRB, 0.84616E+00_JPRB, 0.89451E+00_JPRB, 0.96352E+00_JPRB, 0.10036E+01_JPRB, 0.99010E+00_JPRB, 0.92274E+00_JPRB ,&
&  0.85939E+00_JPRB, 0.78115E+00_JPRB, 0.74041E+00_JPRB, 0.71645E+00_JPRB, 0.66292E+00_JPRB, 0.58546E+00_JPRB, 0.72317E+00_JPRB ,&
&  0.58554E+00_JPRB, 0.53047E+00_JPRB, 0.62082E+00_JPRB, 0.41621E+00_JPRB, 0.59969E+00_JPRB, 0.76164E+00_JPRB, 0.10252E+01_JPRB ,&
&  0.12334E+01_JPRB, 0.17453E+01_JPRB, 0.21972E+01_JPRB, 0.16701E+01_JPRB, 0.11515E+01_JPRB, 0.66120E+00_JPRB, 0.57208E+00_JPRB ,&
&  0.58153E+00_JPRB, 0.58782E+00_JPRB, 0.59726E+00_JPRB, 0.59833E+00_JPRB, 0.61105E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.57789E+00_JPRB, 0.57292E+00_JPRB, 0.57285E+00_JPRB, 0.56392E+00_JPRB, 0.55954E+00_JPRB, 0.56068E+00_JPRB, 0.54930E+00_JPRB ,&
&  0.55793E+00_JPRB, 0.57397E+00_JPRB, 0.57951E+00_JPRB, 0.59745E+00_JPRB, 0.60881E+00_JPRB, 0.64674E+00_JPRB, 0.70209E+00_JPRB ,&
&  0.74079E+00_JPRB, 0.82477E+00_JPRB, 0.86604E+00_JPRB, 0.91665E+00_JPRB, 0.95926E+00_JPRB, 0.10040E+01_JPRB, 0.10423E+01_JPRB ,&
&  0.10972E+01_JPRB, 0.11631E+01_JPRB, 0.12331E+01_JPRB, 0.12808E+01_JPRB, 0.12875E+01_JPRB, 0.14042E+01_JPRB, 0.11232E+01_JPRB ,&
&  0.13191E+01_JPRB, 0.13483E+01_JPRB, 0.12860E+01_JPRB, 0.12673E+01_JPRB, 0.12542E+01_JPRB, 0.12608E+01_JPRB, 0.12775E+01_JPRB ,&
&  0.13331E+01_JPRB, 0.13850E+01_JPRB, 0.14368E+01_JPRB, 0.16195E+01_JPRB, 0.18200E+01_JPRB, 0.19851E+01_JPRB, 0.21451E+01_JPRB ,&
&  0.22999E+01_JPRB, 0.24534E+01_JPRB, 0.26476E+01_JPRB, 0.28472E+01_JPRB, 0.30504E+01_JPRB, 0.31775E+01_JPRB, 0.33237E+01_JPRB ,&
&  0.33917E+01_JPRB, 0.34367E+01_JPRB, 0.32038E+01_JPRB, 0.27936E+01_JPRB, 0.23486E+01_JPRB, 0.18560E+01_JPRB, 0.13699E+01_JPRB ,&
&  0.10297E+01_JPRB, 0.95922E+00_JPRB, 0.95617E+00_JPRB, 0.96177E+00_JPRB, 0.86156E+00_JPRB, 0.79314E+00_JPRB, 0.75021E+00_JPRB ,&
&  0.70717E+00_JPRB, 0.65176E+00_JPRB, 0.68057E+00_JPRB, 0.72984E+00_JPRB, 0.74221E+00_JPRB, 0.73202E+00_JPRB, 0.70944E+00_JPRB ,&
&  0.67864E+00_JPRB, 0.63326E+00_JPRB, 0.62751E+00_JPRB, 0.62359E+00_JPRB, 0.63103E+00_JPRB, 0.66897E+00_JPRB, 0.59906E+00_JPRB ,&
&  0.65440E+00_JPRB, 0.54688E+00_JPRB, 0.55159E+00_JPRB, 0.57799E+00_JPRB, 0.63640E+00_JPRB, 0.81698E+00_JPRB, 0.11435E+01_JPRB ,&
&  0.17307E+01_JPRB, 0.19858E+01_JPRB, 0.22364E+01_JPRB, 0.27883E+01_JPRB, 0.24304E+01_JPRB, 0.17951E+01_JPRB, 0.53064E+00_JPRB ,&
&  0.52961E+00_JPRB, 0.53485E+00_JPRB, 0.55811E+00_JPRB, 0.56724E+00_JPRB, 0.57224E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.49556E+00_JPRB, 0.48184E+00_JPRB, 0.47465E+00_JPRB, 0.46946E+00_JPRB, 0.45449E+00_JPRB, 0.45545E+00_JPRB, 0.45292E+00_JPRB ,&
&  0.46971E+00_JPRB, 0.49119E+00_JPRB, 0.52904E+00_JPRB, 0.57427E+00_JPRB, 0.63289E+00_JPRB, 0.67492E+00_JPRB, 0.72520E+00_JPRB ,&
&  0.75639E+00_JPRB, 0.79671E+00_JPRB, 0.84223E+00_JPRB, 0.88188E+00_JPRB, 0.92952E+00_JPRB, 0.96452E+00_JPRB, 0.10211E+01_JPRB ,&
&  0.10727E+01_JPRB, 0.11328E+01_JPRB, 0.12122E+01_JPRB, 0.12831E+01_JPRB, 0.13422E+01_JPRB, 0.13677E+01_JPRB, 0.10905E+01_JPRB ,&
&  0.13549E+01_JPRB, 0.14081E+01_JPRB, 0.14181E+01_JPRB, 0.13847E+01_JPRB, 0.13411E+01_JPRB, 0.12995E+01_JPRB, 0.13149E+01_JPRB ,&
&  0.13450E+01_JPRB, 0.13683E+01_JPRB, 0.13921E+01_JPRB, 0.14629E+01_JPRB, 0.18367E+01_JPRB, 0.19973E+01_JPRB, 0.21642E+01_JPRB ,&
&  0.23436E+01_JPRB, 0.25450E+01_JPRB, 0.27551E+01_JPRB, 0.29684E+01_JPRB, 0.31143E+01_JPRB, 0.32736E+01_JPRB, 0.34420E+01_JPRB ,&
&  0.35637E+01_JPRB, 0.36660E+01_JPRB, 0.35074E+01_JPRB, 0.27236E+01_JPRB, 0.22316E+01_JPRB, 0.18107E+01_JPRB, 0.11887E+01_JPRB ,&
&  0.90863E+00_JPRB, 0.83447E+00_JPRB, 0.83526E+00_JPRB, 0.81599E+00_JPRB, 0.73008E+00_JPRB, 0.66048E+00_JPRB, 0.61657E+00_JPRB ,&
&  0.55614E+00_JPRB, 0.54299E+00_JPRB, 0.53907E+00_JPRB, 0.57523E+00_JPRB, 0.59627E+00_JPRB, 0.57120E+00_JPRB, 0.55336E+00_JPRB ,&
&  0.52058E+00_JPRB, 0.49416E+00_JPRB, 0.52296E+00_JPRB, 0.56883E+00_JPRB, 0.67125E+00_JPRB, 0.77496E+00_JPRB, 0.83410E+00_JPRB ,&
&  0.86940E+00_JPRB, 0.55459E+00_JPRB, 0.55700E+00_JPRB, 0.56546E+00_JPRB, 0.59267E+00_JPRB, 0.64228E+00_JPRB, 0.73545E+00_JPRB ,&
&  0.93130E+00_JPRB, 0.12767E+01_JPRB, 0.11339E+01_JPRB, 0.86490E+00_JPRB, 0.93874E+00_JPRB, 0.86508E+00_JPRB, 0.47695E+00_JPRB ,&
&  0.47948E+00_JPRB, 0.48040E+00_JPRB, 0.49637E+00_JPRB, 0.50130E+00_JPRB, 0.50447E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.43857E+00_JPRB, 0.42399E+00_JPRB, 0.40714E+00_JPRB, 0.39817E+00_JPRB, 0.39041E+00_JPRB, 0.38183E+00_JPRB, 0.39250E+00_JPRB ,&
&  0.42046E+00_JPRB, 0.44530E+00_JPRB, 0.48691E+00_JPRB, 0.54892E+00_JPRB, 0.60677E+00_JPRB, 0.63952E+00_JPRB, 0.67631E+00_JPRB ,&
&  0.70760E+00_JPRB, 0.74241E+00_JPRB, 0.77162E+00_JPRB, 0.79877E+00_JPRB, 0.83378E+00_JPRB, 0.87681E+00_JPRB, 0.93491E+00_JPRB ,&
&  0.10050E+01_JPRB, 0.10774E+01_JPRB, 0.11677E+01_JPRB, 0.12803E+01_JPRB, 0.13894E+01_JPRB, 0.14400E+01_JPRB, 0.15122E+01_JPRB ,&
&  0.11389E+01_JPRB, 0.14104E+01_JPRB, 0.14250E+01_JPRB, 0.13994E+01_JPRB, 0.13608E+01_JPRB, 0.13257E+01_JPRB, 0.13206E+01_JPRB ,&
&  0.13287E+01_JPRB, 0.12752E+01_JPRB, 0.13013E+01_JPRB, 0.13959E+01_JPRB, 0.16290E+01_JPRB, 0.17977E+01_JPRB, 0.19675E+01_JPRB ,&
&  0.21468E+01_JPRB, 0.23463E+01_JPRB, 0.25542E+01_JPRB, 0.27533E+01_JPRB, 0.29124E+01_JPRB, 0.30880E+01_JPRB, 0.32194E+01_JPRB ,&
&  0.33792E+01_JPRB, 0.35095E+01_JPRB, 0.35485E+01_JPRB, 0.28811E+01_JPRB, 0.22739E+01_JPRB, 0.19276E+01_JPRB, 0.13757E+01_JPRB ,&
&  0.91626E+00_JPRB, 0.73362E+00_JPRB, 0.74473E+00_JPRB, 0.76573E+00_JPRB, 0.70452E+00_JPRB, 0.64900E+00_JPRB, 0.59076E+00_JPRB ,&
&  0.55258E+00_JPRB, 0.49925E+00_JPRB, 0.48416E+00_JPRB, 0.49960E+00_JPRB, 0.53376E+00_JPRB, 0.50858E+00_JPRB, 0.47137E+00_JPRB ,&
&  0.44379E+00_JPRB, 0.43324E+00_JPRB, 0.47280E+00_JPRB, 0.51908E+00_JPRB, 0.58411E+00_JPRB, 0.61996E+00_JPRB, 0.62768E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.58825E+00_JPRB, 0.58007E+00_JPRB, 0.58624E+00_JPRB, 0.58659E+00_JPRB, 0.59302E+00_JPRB, 0.60906E+00_JPRB ,&
&  0.64595E+00_JPRB, 0.71029E+00_JPRB, 0.73046E+00_JPRB, 0.67753E+00_JPRB, 0.62479E+00_JPRB, 0.58066E+00_JPRB, 0.53697E+00_JPRB ,&
&  0.54029E+00_JPRB, 0.52845E+00_JPRB, 0.51530E+00_JPRB, 0.49490E+00_JPRB, 0.46765E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.44900E+00_JPRB, 0.42343E+00_JPRB, 0.38540E+00_JPRB, 0.35157E+00_JPRB, 0.37376E+00_JPRB, 0.37369E+00_JPRB, 0.38860E+00_JPRB ,&
&  0.39433E+00_JPRB, 0.41899E+00_JPRB, 0.44379E+00_JPRB, 0.48089E+00_JPRB, 0.52742E+00_JPRB, 0.57399E+00_JPRB, 0.61276E+00_JPRB ,&
&  0.64749E+00_JPRB, 0.68199E+00_JPRB, 0.70906E+00_JPRB, 0.71990E+00_JPRB, 0.73673E+00_JPRB, 0.76470E+00_JPRB, 0.79784E+00_JPRB ,&
&  0.84128E+00_JPRB, 0.89874E+00_JPRB, 0.97902E+00_JPRB, 0.10820E+01_JPRB, 0.12038E+01_JPRB, 0.12731E+01_JPRB, 0.13180E+01_JPRB ,&
&  0.77482E+00_JPRB, 0.11890E+01_JPRB, 0.14284E+01_JPRB, 0.13796E+01_JPRB, 0.13492E+01_JPRB, 0.13230E+01_JPRB, 0.13110E+01_JPRB ,&
&  0.12420E+01_JPRB, 0.11677E+01_JPRB, 0.11798E+01_JPRB, 0.13671E+01_JPRB, 0.13748E+01_JPRB, 0.15288E+01_JPRB, 0.16586E+01_JPRB ,&
&  0.17973E+01_JPRB, 0.19431E+01_JPRB, 0.21104E+01_JPRB, 0.23168E+01_JPRB, 0.25256E+01_JPRB, 0.27005E+01_JPRB, 0.28556E+01_JPRB ,&
&  0.30605E+01_JPRB, 0.32446E+01_JPRB, 0.33021E+01_JPRB, 0.25358E+01_JPRB, 0.22791E+01_JPRB, 0.21164E+01_JPRB, 0.18643E+01_JPRB ,&
&  0.10917E+01_JPRB, 0.74702E+00_JPRB, 0.74549E+00_JPRB, 0.77353E+00_JPRB, 0.72622E+00_JPRB, 0.68038E+00_JPRB, 0.63406E+00_JPRB ,&
&  0.57615E+00_JPRB, 0.52943E+00_JPRB, 0.50060E+00_JPRB, 0.49078E+00_JPRB, 0.51114E+00_JPRB, 0.51473E+00_JPRB, 0.49197E+00_JPRB ,&
&  0.46472E+00_JPRB, 0.46615E+00_JPRB, 0.49551E+00_JPRB, 0.52040E+00_JPRB, 0.55831E+00_JPRB, 0.58452E+00_JPRB, 0.60702E+00_JPRB ,&
&  0.61776E+00_JPRB, 0.58811E+00_JPRB, 0.59176E+00_JPRB, 0.60360E+00_JPRB, 0.60649E+00_JPRB, 0.60797E+00_JPRB, 0.60585E+00_JPRB ,&
&  0.60491E+00_JPRB, 0.62297E+00_JPRB, 0.63527E+00_JPRB, 0.64432E+00_JPRB, 0.63417E+00_JPRB, 0.62013E+00_JPRB, 0.61502E+00_JPRB ,&
&  0.60104E+00_JPRB, 0.57866E+00_JPRB, 0.55421E+00_JPRB, 0.51634E+00_JPRB, 0.47993E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.47063E+00_JPRB, 0.44200E+00_JPRB, 0.40366E+00_JPRB, 0.38867E+00_JPRB, 0.41143E+00_JPRB, 0.43024E+00_JPRB, 0.42509E+00_JPRB ,&
&  0.41901E+00_JPRB, 0.40976E+00_JPRB, 0.40480E+00_JPRB, 0.41404E+00_JPRB, 0.44916E+00_JPRB, 0.49940E+00_JPRB, 0.53793E+00_JPRB ,&
&  0.57479E+00_JPRB, 0.60457E+00_JPRB, 0.62874E+00_JPRB, 0.63723E+00_JPRB, 0.63368E+00_JPRB, 0.64505E+00_JPRB, 0.66244E+00_JPRB ,&
&  0.69213E+00_JPRB, 0.73436E+00_JPRB, 0.78930E+00_JPRB, 0.86732E+00_JPRB, 0.97579E+00_JPRB, 0.11087E+01_JPRB, 0.12447E+01_JPRB ,&
&  0.91952E+00_JPRB, 0.47784E+00_JPRB, 0.11971E+01_JPRB, 0.14816E+01_JPRB, 0.13989E+01_JPRB, 0.13704E+01_JPRB, 0.12961E+01_JPRB ,&
&  0.11993E+01_JPRB, 0.12024E+01_JPRB, 0.11581E+01_JPRB, 0.12590E+01_JPRB, 0.12325E+01_JPRB, 0.12806E+01_JPRB, 0.13294E+01_JPRB ,&
&  0.14002E+01_JPRB, 0.14826E+01_JPRB, 0.16065E+01_JPRB, 0.17559E+01_JPRB, 0.19217E+01_JPRB, 0.21693E+01_JPRB, 0.23503E+01_JPRB ,&
&  0.25149E+01_JPRB, 0.26707E+01_JPRB, 0.27090E+01_JPRB, 0.19626E+01_JPRB, 0.15772E+01_JPRB, 0.13803E+01_JPRB, 0.12370E+01_JPRB ,&
&  0.10426E+01_JPRB, 0.81668E+00_JPRB, 0.78411E+00_JPRB, 0.80477E+00_JPRB, 0.77320E+00_JPRB, 0.70288E+00_JPRB, 0.70035E+00_JPRB ,&
&  0.62703E+00_JPRB, 0.57739E+00_JPRB, 0.54237E+00_JPRB, 0.51426E+00_JPRB, 0.51867E+00_JPRB, 0.52909E+00_JPRB, 0.53213E+00_JPRB ,&
&  0.51717E+00_JPRB, 0.51680E+00_JPRB, 0.52993E+00_JPRB, 0.53169E+00_JPRB, 0.53494E+00_JPRB, 0.54258E+00_JPRB, 0.54824E+00_JPRB ,&
&  0.57691E+00_JPRB, 0.57885E+00_JPRB, 0.58793E+00_JPRB, 0.60506E+00_JPRB, 0.61468E+00_JPRB, 0.60338E+00_JPRB, 0.60687E+00_JPRB ,&
&  0.61960E+00_JPRB, 0.63228E+00_JPRB, 0.63051E+00_JPRB, 0.64190E+00_JPRB, 0.64072E+00_JPRB, 0.64024E+00_JPRB, 0.65105E+00_JPRB ,&
&  0.66666E+00_JPRB, 0.66636E+00_JPRB, 0.63123E+00_JPRB, 0.55033E+00_JPRB, 0.50735E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.49808E+00_JPRB, 0.45723E+00_JPRB, 0.42163E+00_JPRB, 0.42228E+00_JPRB, 0.44214E+00_JPRB, 0.44790E+00_JPRB, 0.44662E+00_JPRB ,&
&  0.43717E+00_JPRB, 0.42629E+00_JPRB, 0.40917E+00_JPRB, 0.40129E+00_JPRB, 0.40592E+00_JPRB, 0.41814E+00_JPRB, 0.45379E+00_JPRB ,&
&  0.48923E+00_JPRB, 0.52227E+00_JPRB, 0.54966E+00_JPRB, 0.55304E+00_JPRB, 0.53889E+00_JPRB, 0.53832E+00_JPRB, 0.54815E+00_JPRB ,&
&  0.55872E+00_JPRB, 0.58423E+00_JPRB, 0.61843E+00_JPRB, 0.66563E+00_JPRB, 0.73653E+00_JPRB, 0.85352E+00_JPRB, 0.10344E+01_JPRB ,&
&  0.13210E+01_JPRB, 0.92004E+00_JPRB, 0.64667E+00_JPRB, 0.12720E+01_JPRB, 0.15513E+01_JPRB, 0.15812E+01_JPRB, 0.13393E+01_JPRB ,&
&  0.12966E+01_JPRB, 0.12375E+01_JPRB, 0.11912E+01_JPRB, 0.12378E+01_JPRB, 0.11657E+01_JPRB, 0.11520E+01_JPRB, 0.11382E+01_JPRB ,&
&  0.11251E+01_JPRB, 0.11863E+01_JPRB, 0.12703E+01_JPRB, 0.13510E+01_JPRB, 0.14304E+01_JPRB, 0.16023E+01_JPRB, 0.18203E+01_JPRB ,&
&  0.20343E+01_JPRB, 0.21666E+01_JPRB, 0.22090E+01_JPRB, 0.16577E+01_JPRB, 0.13759E+01_JPRB, 0.12421E+01_JPRB, 0.11737E+01_JPRB ,&
&  0.10334E+01_JPRB, 0.94803E+00_JPRB, 0.91975E+00_JPRB, 0.86649E+00_JPRB, 0.81519E+00_JPRB, 0.76467E+00_JPRB, 0.77013E+00_JPRB ,&
&  0.70372E+00_JPRB, 0.64627E+00_JPRB, 0.61031E+00_JPRB, 0.58703E+00_JPRB, 0.57720E+00_JPRB, 0.57818E+00_JPRB, 0.57357E+00_JPRB ,&
&  0.57592E+00_JPRB, 0.57782E+00_JPRB, 0.58090E+00_JPRB, 0.56711E+00_JPRB, 0.55426E+00_JPRB, 0.54598E+00_JPRB, 0.53256E+00_JPRB ,&
&  0.54866E+00_JPRB, 0.57278E+00_JPRB, 0.58398E+00_JPRB, 0.60207E+00_JPRB, 0.60244E+00_JPRB, 0.59691E+00_JPRB, 0.61372E+00_JPRB ,&
&  0.62055E+00_JPRB, 0.63725E+00_JPRB, 0.63481E+00_JPRB, 0.64224E+00_JPRB, 0.63561E+00_JPRB, 0.63207E+00_JPRB, 0.64150E+00_JPRB ,&
&  0.68498E+00_JPRB, 0.73569E+00_JPRB, 0.72872E+00_JPRB, 0.57635E+00_JPRB, 0.52526E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.48673E+00_JPRB, 0.47378E+00_JPRB, 0.43561E+00_JPRB, 0.43954E+00_JPRB, 0.44217E+00_JPRB, 0.44868E+00_JPRB, 0.44724E+00_JPRB ,&
&  0.44635E+00_JPRB, 0.44802E+00_JPRB, 0.45044E+00_JPRB, 0.44236E+00_JPRB, 0.41920E+00_JPRB, 0.40903E+00_JPRB, 0.41194E+00_JPRB ,&
&  0.42336E+00_JPRB, 0.44862E+00_JPRB, 0.47890E+00_JPRB, 0.48375E+00_JPRB, 0.48417E+00_JPRB, 0.48007E+00_JPRB, 0.47684E+00_JPRB ,&
&  0.48080E+00_JPRB, 0.48882E+00_JPRB, 0.49962E+00_JPRB, 0.52496E+00_JPRB, 0.57020E+00_JPRB, 0.63983E+00_JPRB, 0.75144E+00_JPRB ,&
&  0.10234E+01_JPRB, 0.10724E+01_JPRB, 0.54294E+00_JPRB, 0.10149E+01_JPRB, 0.15394E+01_JPRB, 0.16241E+01_JPRB, 0.15439E+01_JPRB ,&
&  0.14574E+01_JPRB, 0.13042E+01_JPRB, 0.12373E+01_JPRB, 0.11848E+01_JPRB, 0.10796E+01_JPRB, 0.10241E+01_JPRB, 0.96967E+00_JPRB ,&
&  0.93690E+00_JPRB, 0.97160E+00_JPRB, 0.10587E+01_JPRB, 0.11259E+01_JPRB, 0.11479E+01_JPRB, 0.11988E+01_JPRB, 0.13605E+01_JPRB ,&
&  0.16004E+01_JPRB, 0.18443E+01_JPRB, 0.19536E+01_JPRB, 0.15544E+01_JPRB, 0.13666E+01_JPRB, 0.13689E+01_JPRB, 0.13185E+01_JPRB ,&
&  0.12184E+01_JPRB, 0.10923E+01_JPRB, 0.10013E+01_JPRB, 0.93321E+00_JPRB, 0.86450E+00_JPRB, 0.83614E+00_JPRB, 0.84495E+00_JPRB ,&
&  0.79254E+00_JPRB, 0.72376E+00_JPRB, 0.68097E+00_JPRB, 0.65392E+00_JPRB, 0.63455E+00_JPRB, 0.62493E+00_JPRB, 0.61721E+00_JPRB ,&
&  0.61236E+00_JPRB, 0.61870E+00_JPRB, 0.62754E+00_JPRB, 0.61970E+00_JPRB, 0.60083E+00_JPRB, 0.57792E+00_JPRB, 0.55645E+00_JPRB ,&
&  0.55277E+00_JPRB, 0.56153E+00_JPRB, 0.58233E+00_JPRB, 0.59119E+00_JPRB, 0.59575E+00_JPRB, 0.60412E+00_JPRB, 0.61325E+00_JPRB ,&
&  0.62550E+00_JPRB, 0.63517E+00_JPRB, 0.62589E+00_JPRB, 0.61418E+00_JPRB, 0.63140E+00_JPRB, 0.62045E+00_JPRB, 0.61102E+00_JPRB ,&
&  0.64852E+00_JPRB, 0.72353E+00_JPRB, 0.84520E+00_JPRB, 0.58278E+00_JPRB, 0.52402E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.48155E+00_JPRB, 0.45798E+00_JPRB, 0.45113E+00_JPRB, 0.44895E+00_JPRB, 0.45161E+00_JPRB, 0.45899E+00_JPRB, 0.46640E+00_JPRB ,&
&  0.46750E+00_JPRB, 0.46760E+00_JPRB, 0.46806E+00_JPRB, 0.46417E+00_JPRB, 0.45230E+00_JPRB, 0.44039E+00_JPRB, 0.42938E+00_JPRB ,&
&  0.42710E+00_JPRB, 0.42913E+00_JPRB, 0.44218E+00_JPRB, 0.44980E+00_JPRB, 0.45078E+00_JPRB, 0.44828E+00_JPRB, 0.44917E+00_JPRB ,&
&  0.45298E+00_JPRB, 0.45254E+00_JPRB, 0.45038E+00_JPRB, 0.46138E+00_JPRB, 0.48411E+00_JPRB, 0.52848E+00_JPRB, 0.60006E+00_JPRB ,&
&  0.79667E+00_JPRB, 0.95857E+00_JPRB, 0.10053E+01_JPRB, 0.15271E+01_JPRB, 0.18669E+01_JPRB, 0.18027E+01_JPRB, 0.17202E+01_JPRB ,&
&  0.15864E+01_JPRB, 0.14178E+01_JPRB, 0.13777E+01_JPRB, 0.11860E+01_JPRB, 0.10433E+01_JPRB, 0.93042E+00_JPRB, 0.84242E+00_JPRB ,&
&  0.77814E+00_JPRB, 0.76450E+00_JPRB, 0.81974E+00_JPRB, 0.91486E+00_JPRB, 0.97714E+00_JPRB, 0.99960E+00_JPRB, 0.10631E+01_JPRB ,&
&  0.12301E+01_JPRB, 0.15361E+01_JPRB, 0.17854E+01_JPRB, 0.16570E+01_JPRB, 0.13601E+01_JPRB, 0.14973E+01_JPRB, 0.16491E+01_JPRB ,&
&  0.18590E+01_JPRB, 0.13803E+01_JPRB, 0.11080E+01_JPRB, 0.99832E+00_JPRB, 0.91337E+00_JPRB, 0.97328E+00_JPRB, 0.92483E+00_JPRB ,&
&  0.86587E+00_JPRB, 0.80382E+00_JPRB, 0.74883E+00_JPRB, 0.71419E+00_JPRB, 0.68816E+00_JPRB, 0.66922E+00_JPRB, 0.65644E+00_JPRB ,&
&  0.64884E+00_JPRB, 0.64538E+00_JPRB, 0.65365E+00_JPRB, 0.65567E+00_JPRB, 0.64743E+00_JPRB, 0.62696E+00_JPRB, 0.59808E+00_JPRB ,&
&  0.57513E+00_JPRB, 0.56914E+00_JPRB, 0.55875E+00_JPRB, 0.57064E+00_JPRB, 0.58584E+00_JPRB, 0.59405E+00_JPRB, 0.59361E+00_JPRB ,&
&  0.60757E+00_JPRB, 0.62400E+00_JPRB, 0.62256E+00_JPRB, 0.61508E+00_JPRB, 0.61469E+00_JPRB, 0.61650E+00_JPRB, 0.58943E+00_JPRB ,&
&  0.57818E+00_JPRB, 0.56733E+00_JPRB, 0.56080E+00_JPRB, 0.54389E+00_JPRB, 0.50809E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.48231E+00_JPRB, 0.46739E+00_JPRB, 0.46232E+00_JPRB, 0.46505E+00_JPRB, 0.46812E+00_JPRB, 0.47180E+00_JPRB, 0.48112E+00_JPRB ,&
&  0.48568E+00_JPRB, 0.48610E+00_JPRB, 0.48418E+00_JPRB, 0.48117E+00_JPRB, 0.47291E+00_JPRB, 0.46378E+00_JPRB, 0.45351E+00_JPRB ,&
&  0.44568E+00_JPRB, 0.42655E+00_JPRB, 0.41685E+00_JPRB, 0.42442E+00_JPRB, 0.42823E+00_JPRB, 0.42788E+00_JPRB, 0.42884E+00_JPRB ,&
&  0.42918E+00_JPRB, 0.43356E+00_JPRB, 0.43367E+00_JPRB, 0.43981E+00_JPRB, 0.45193E+00_JPRB, 0.48211E+00_JPRB, 0.52555E+00_JPRB ,&
&  0.66341E+00_JPRB, 0.89761E+00_JPRB, 0.94370E+00_JPRB, 0.17499E+01_JPRB, 0.19338E+01_JPRB, 0.18724E+01_JPRB, 0.16474E+01_JPRB ,&
&  0.15072E+01_JPRB, 0.15368E+01_JPRB, 0.12695E+01_JPRB, 0.10902E+01_JPRB, 0.99008E+00_JPRB, 0.88908E+00_JPRB, 0.77349E+00_JPRB ,&
&  0.68679E+00_JPRB, 0.64869E+00_JPRB, 0.64379E+00_JPRB, 0.68729E+00_JPRB, 0.76549E+00_JPRB, 0.83652E+00_JPRB, 0.89236E+00_JPRB ,&
&  0.98894E+00_JPRB, 0.12136E+01_JPRB, 0.15412E+01_JPRB, 0.15220E+01_JPRB, 0.14358E+01_JPRB, 0.16407E+01_JPRB, 0.20389E+01_JPRB ,&
&  0.24609E+01_JPRB, 0.19717E+01_JPRB, 0.13349E+01_JPRB, 0.10939E+01_JPRB, 0.10268E+01_JPRB, 0.10518E+01_JPRB, 0.10302E+01_JPRB ,&
&  0.96073E+00_JPRB, 0.90157E+00_JPRB, 0.82974E+00_JPRB, 0.78209E+00_JPRB, 0.74816E+00_JPRB, 0.71019E+00_JPRB, 0.68386E+00_JPRB ,&
&  0.67079E+00_JPRB, 0.66312E+00_JPRB, 0.66215E+00_JPRB, 0.65759E+00_JPRB, 0.65647E+00_JPRB, 0.65254E+00_JPRB, 0.64178E+00_JPRB ,&
&  0.61944E+00_JPRB, 0.58923E+00_JPRB, 0.55612E+00_JPRB, 0.55141E+00_JPRB, 0.56676E+00_JPRB, 0.57680E+00_JPRB, 0.58251E+00_JPRB ,&
&  0.60964E+00_JPRB, 0.61977E+00_JPRB, 0.61034E+00_JPRB, 0.59058E+00_JPRB, 0.59534E+00_JPRB, 0.62313E+00_JPRB, 0.59280E+00_JPRB ,&
&  0.58796E+00_JPRB, 0.57761E+00_JPRB, 0.56508E+00_JPRB, 0.55109E+00_JPRB, 0.51766E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.50683E+00_JPRB, 0.49441E+00_JPRB, 0.48582E+00_JPRB, 0.48360E+00_JPRB, 0.47864E+00_JPRB, 0.47900E+00_JPRB, 0.48718E+00_JPRB ,&
&  0.50043E+00_JPRB, 0.50400E+00_JPRB, 0.50166E+00_JPRB, 0.49837E+00_JPRB, 0.49220E+00_JPRB, 0.48362E+00_JPRB, 0.47288E+00_JPRB ,&
&  0.46339E+00_JPRB, 0.44887E+00_JPRB, 0.42994E+00_JPRB, 0.41622E+00_JPRB, 0.41968E+00_JPRB, 0.42456E+00_JPRB, 0.42488E+00_JPRB ,&
&  0.42412E+00_JPRB, 0.43033E+00_JPRB, 0.43293E+00_JPRB, 0.44291E+00_JPRB, 0.45039E+00_JPRB, 0.46858E+00_JPRB, 0.50686E+00_JPRB ,&
&  0.61546E+00_JPRB, 0.97197E+00_JPRB, 0.97161E+00_JPRB, 0.15811E+01_JPRB, 0.17923E+01_JPRB, 0.17272E+01_JPRB, 0.15029E+01_JPRB ,&
&  0.13897E+01_JPRB, 0.13153E+01_JPRB, 0.11289E+01_JPRB, 0.97766E+00_JPRB, 0.88070E+00_JPRB, 0.80406E+00_JPRB, 0.73158E+00_JPRB ,&
&  0.67508E+00_JPRB, 0.64271E+00_JPRB, 0.62378E+00_JPRB, 0.61435E+00_JPRB, 0.64009E+00_JPRB, 0.69368E+00_JPRB, 0.75630E+00_JPRB ,&
&  0.82606E+00_JPRB, 0.94650E+00_JPRB, 0.12281E+01_JPRB, 0.15084E+01_JPRB, 0.14188E+01_JPRB, 0.16424E+01_JPRB, 0.24168E+01_JPRB ,&
&  0.28398E+01_JPRB, 0.27783E+01_JPRB, 0.18215E+01_JPRB, 0.13032E+01_JPRB, 0.11418E+01_JPRB, 0.11496E+01_JPRB, 0.11417E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.10315E+01_JPRB, 0.95214E+00_JPRB, 0.86421E+00_JPRB, 0.79891E+00_JPRB, 0.75262E+00_JPRB, 0.70855E+00_JPRB ,&
&  0.67606E+00_JPRB, 0.66213E+00_JPRB, 0.65176E+00_JPRB, 0.64134E+00_JPRB, 0.63660E+00_JPRB, 0.63899E+00_JPRB, 0.64230E+00_JPRB ,&
&  0.64051E+00_JPRB, 0.62238E+00_JPRB, 0.58053E+00_JPRB, 0.55463E+00_JPRB, 0.55491E+00_JPRB, 0.56461E+00_JPRB, 0.57085E+00_JPRB ,&
&  0.60486E+00_JPRB, 0.61374E+00_JPRB, 0.59767E+00_JPRB, 0.58986E+00_JPRB, 0.58648E+00_JPRB, 0.63072E+00_JPRB, 0.59835E+00_JPRB ,&
&  0.58713E+00_JPRB, 0.57846E+00_JPRB, 0.57545E+00_JPRB, 0.56754E+00_JPRB, 0.53903E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.53184E+00_JPRB, 0.51534E+00_JPRB, 0.50615E+00_JPRB, 0.49423E+00_JPRB, 0.48478E+00_JPRB, 0.48174E+00_JPRB, 0.48854E+00_JPRB ,&
&  0.50455E+00_JPRB, 0.51724E+00_JPRB, 0.51111E+00_JPRB, 0.50759E+00_JPRB, 0.50565E+00_JPRB, 0.49815E+00_JPRB, 0.48729E+00_JPRB ,&
&  0.47638E+00_JPRB, 0.46580E+00_JPRB, 0.45289E+00_JPRB, 0.43878E+00_JPRB, 0.42887E+00_JPRB, 0.42890E+00_JPRB, 0.42703E+00_JPRB ,&
&  0.42529E+00_JPRB, 0.42918E+00_JPRB, 0.43379E+00_JPRB, 0.44207E+00_JPRB, 0.45043E+00_JPRB, 0.46468E+00_JPRB, 0.49803E+00_JPRB ,&
&  0.62211E+00_JPRB, 0.50078E+00_JPRB, 0.11239E+01_JPRB, 0.13907E+01_JPRB, 0.14381E+01_JPRB, 0.13486E+01_JPRB, 0.12892E+01_JPRB ,&
&  0.12423E+01_JPRB, 0.11216E+01_JPRB, 0.10185E+01_JPRB, 0.92842E+00_JPRB, 0.82006E+00_JPRB, 0.74961E+00_JPRB, 0.70676E+00_JPRB ,&
&  0.67290E+00_JPRB, 0.64329E+00_JPRB, 0.61963E+00_JPRB, 0.59952E+00_JPRB, 0.58893E+00_JPRB, 0.60305E+00_JPRB, 0.64063E+00_JPRB ,&
&  0.68532E+00_JPRB, 0.75421E+00_JPRB, 0.94917E+00_JPRB, 0.12241E+01_JPRB, 0.13923E+01_JPRB, 0.15903E+01_JPRB, 0.19949E+01_JPRB ,&
&  0.26130E+01_JPRB, 0.28786E+01_JPRB, 0.22040E+01_JPRB, 0.16483E+01_JPRB, 0.13459E+01_JPRB, 0.12711E+01_JPRB, 0.12720E+01_JPRB ,&
&  0.12158E+01_JPRB, 0.11441E+01_JPRB, 0.10506E+01_JPRB, 0.94357E+00_JPRB, 0.85091E+00_JPRB, 0.78912E+00_JPRB, 0.74536E+00_JPRB ,&
&  0.70636E+00_JPRB, 0.67233E+00_JPRB, 0.65033E+00_JPRB, 0.63639E+00_JPRB, 0.62763E+00_JPRB, 0.62485E+00_JPRB, 0.62904E+00_JPRB ,&
&  0.63852E+00_JPRB, 0.63711E+00_JPRB, 0.61276E+00_JPRB, 0.58734E+00_JPRB, 0.58316E+00_JPRB, 0.59129E+00_JPRB, 0.60124E+00_JPRB ,&
&  0.61773E+00_JPRB, 0.62002E+00_JPRB, 0.60352E+00_JPRB, 0.59652E+00_JPRB, 0.57308E+00_JPRB, 0.60739E+00_JPRB, 0.59402E+00_JPRB ,&
&  0.58331E+00_JPRB, 0.58021E+00_JPRB, 0.58449E+00_JPRB, 0.57817E+00_JPRB, 0.55789E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.55151E+00_JPRB, 0.53492E+00_JPRB, 0.52199E+00_JPRB, 0.50316E+00_JPRB, 0.49315E+00_JPRB, 0.49038E+00_JPRB, 0.49686E+00_JPRB ,&
&  0.50501E+00_JPRB, 0.51234E+00_JPRB, 0.50801E+00_JPRB, 0.50316E+00_JPRB, 0.49641E+00_JPRB, 0.49197E+00_JPRB, 0.48437E+00_JPRB ,&
&  0.48009E+00_JPRB, 0.47606E+00_JPRB, 0.46910E+00_JPRB, 0.46064E+00_JPRB, 0.44872E+00_JPRB, 0.44305E+00_JPRB, 0.43643E+00_JPRB ,&
&  0.43036E+00_JPRB, 0.42802E+00_JPRB, 0.42991E+00_JPRB, 0.43880E+00_JPRB, 0.45046E+00_JPRB, 0.46459E+00_JPRB, 0.48888E+00_JPRB ,&
&  0.62994E+00_JPRB, 0.74003E+00_JPRB, 0.11596E+01_JPRB, 0.11847E+01_JPRB, 0.10860E+01_JPRB, 0.97885E+00_JPRB, 0.97696E+00_JPRB ,&
&  0.95939E+00_JPRB, 0.90977E+00_JPRB, 0.85550E+00_JPRB, 0.81830E+00_JPRB, 0.78019E+00_JPRB, 0.75377E+00_JPRB, 0.71488E+00_JPRB ,&
&  0.67819E+00_JPRB, 0.63932E+00_JPRB, 0.59790E+00_JPRB, 0.56984E+00_JPRB, 0.55314E+00_JPRB, 0.55060E+00_JPRB, 0.56489E+00_JPRB ,&
&  0.58650E+00_JPRB, 0.61668E+00_JPRB, 0.72550E+00_JPRB, 0.95905E+00_JPRB, 0.13202E+01_JPRB, 0.14030E+01_JPRB, 0.18950E+01_JPRB ,&
&  0.25558E+01_JPRB, 0.25212E+01_JPRB, 0.22462E+01_JPRB, 0.19905E+01_JPRB, 0.16805E+01_JPRB, 0.15564E+01_JPRB, 0.15247E+01_JPRB ,&
&  0.14214E+01_JPRB, 0.12586E+01_JPRB, 0.10926E+01_JPRB, 0.96714E+00_JPRB, 0.88040E+00_JPRB, 0.82654E+00_JPRB, 0.79724E+00_JPRB ,&
&  0.76890E+00_JPRB, 0.73433E+00_JPRB, 0.69885E+00_JPRB, 0.67075E+00_JPRB, 0.65340E+00_JPRB, 0.64598E+00_JPRB, 0.64392E+00_JPRB ,&
&  0.64578E+00_JPRB, 0.64508E+00_JPRB, 0.63656E+00_JPRB, 0.62169E+00_JPRB, 0.63424E+00_JPRB, 0.66110E+00_JPRB, 0.67177E+00_JPRB ,&
&  0.65782E+00_JPRB, 0.64277E+00_JPRB, 0.62580E+00_JPRB, 0.60458E+00_JPRB, 0.58616E+00_JPRB, 0.61013E+00_JPRB, 0.59307E+00_JPRB ,&
&  0.57344E+00_JPRB, 0.57256E+00_JPRB, 0.58749E+00_JPRB, 0.58729E+00_JPRB, 0.57981E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.56991E+00_JPRB, 0.55051E+00_JPRB, 0.53301E+00_JPRB, 0.51651E+00_JPRB, 0.50270E+00_JPRB, 0.49789E+00_JPRB, 0.49953E+00_JPRB ,&
&  0.50353E+00_JPRB, 0.50401E+00_JPRB, 0.49995E+00_JPRB, 0.49223E+00_JPRB, 0.48691E+00_JPRB, 0.48227E+00_JPRB, 0.47863E+00_JPRB ,&
&  0.47217E+00_JPRB, 0.47304E+00_JPRB, 0.46945E+00_JPRB, 0.46285E+00_JPRB, 0.45530E+00_JPRB, 0.44618E+00_JPRB, 0.43747E+00_JPRB ,&
&  0.43185E+00_JPRB, 0.42999E+00_JPRB, 0.43087E+00_JPRB, 0.43542E+00_JPRB, 0.44787E+00_JPRB, 0.46044E+00_JPRB, 0.47664E+00_JPRB ,&
&  0.58338E+00_JPRB, 0.76330E+00_JPRB, 0.90075E+00_JPRB, 0.99294E+00_JPRB, 0.89436E+00_JPRB, 0.80889E+00_JPRB, 0.76929E+00_JPRB ,&
&  0.75242E+00_JPRB, 0.74372E+00_JPRB, 0.71564E+00_JPRB, 0.68528E+00_JPRB, 0.65096E+00_JPRB, 0.63532E+00_JPRB, 0.61675E+00_JPRB ,&
&  0.59516E+00_JPRB, 0.57568E+00_JPRB, 0.55589E+00_JPRB, 0.54777E+00_JPRB, 0.54182E+00_JPRB, 0.53453E+00_JPRB, 0.53066E+00_JPRB ,&
&  0.53494E+00_JPRB, 0.54438E+00_JPRB, 0.58113E+00_JPRB, 0.72231E+00_JPRB, 0.10334E+01_JPRB, 0.12788E+01_JPRB, 0.15988E+01_JPRB ,&
&  0.16886E+01_JPRB, 0.17719E+01_JPRB, 0.18349E+01_JPRB, 0.18205E+01_JPRB, 0.17041E+01_JPRB, 0.16272E+01_JPRB, 0.16176E+01_JPRB ,&
&  0.15250E+01_JPRB, 0.13006E+01_JPRB, 0.10854E+01_JPRB, 0.95389E+00_JPRB, 0.88781E+00_JPRB, 0.85544E+00_JPRB, 0.82876E+00_JPRB ,&
&  0.81109E+00_JPRB, 0.79354E+00_JPRB, 0.76625E+00_JPRB, 0.73282E+00_JPRB, 0.70170E+00_JPRB, 0.67601E+00_JPRB, 0.66026E+00_JPRB ,&
&  0.64746E+00_JPRB, 0.64139E+00_JPRB, 0.64721E+00_JPRB, 0.66004E+00_JPRB, 0.67363E+00_JPRB, 0.68686E+00_JPRB, 0.69651E+00_JPRB ,&
&  0.70528E+00_JPRB, 0.69127E+00_JPRB, 0.64974E+00_JPRB, 0.62231E+00_JPRB, 0.59667E+00_JPRB, 0.60287E+00_JPRB, 0.58173E+00_JPRB ,&
&  0.56413E+00_JPRB, 0.55934E+00_JPRB, 0.57120E+00_JPRB, 0.57922E+00_JPRB, 0.58541E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.64860E+00_JPRB, 0.60068E+00_JPRB, 0.56092E+00_JPRB, 0.53006E+00_JPRB, 0.51298E+00_JPRB, 0.50399E+00_JPRB, 0.49752E+00_JPRB ,&
&  0.49420E+00_JPRB, 0.49058E+00_JPRB, 0.48227E+00_JPRB, 0.47551E+00_JPRB, 0.47081E+00_JPRB, 0.46866E+00_JPRB, 0.46689E+00_JPRB ,&
&  0.45822E+00_JPRB, 0.45687E+00_JPRB, 0.45709E+00_JPRB, 0.45281E+00_JPRB, 0.44686E+00_JPRB, 0.44052E+00_JPRB, 0.43230E+00_JPRB ,&
&  0.42670E+00_JPRB, 0.42498E+00_JPRB, 0.42722E+00_JPRB, 0.43135E+00_JPRB, 0.43573E+00_JPRB, 0.44731E+00_JPRB, 0.47054E+00_JPRB ,&
&  0.56168E+00_JPRB, 0.63980E+00_JPRB, 0.72115E+00_JPRB, 0.76254E+00_JPRB, 0.68885E+00_JPRB, 0.66090E+00_JPRB, 0.65100E+00_JPRB ,&
&  0.65538E+00_JPRB, 0.66046E+00_JPRB, 0.63485E+00_JPRB, 0.59403E+00_JPRB, 0.55789E+00_JPRB, 0.53853E+00_JPRB, 0.52622E+00_JPRB ,&
&  0.51470E+00_JPRB, 0.50631E+00_JPRB, 0.49859E+00_JPRB, 0.49995E+00_JPRB, 0.50150E+00_JPRB, 0.49933E+00_JPRB, 0.49762E+00_JPRB ,&
&  0.49783E+00_JPRB, 0.50120E+00_JPRB, 0.51117E+00_JPRB, 0.56416E+00_JPRB, 0.71656E+00_JPRB, 0.88836E+00_JPRB, 0.10611E+01_JPRB ,&
&  0.11454E+01_JPRB, 0.12305E+01_JPRB, 0.13363E+01_JPRB, 0.14187E+01_JPRB, 0.14300E+01_JPRB, 0.14267E+01_JPRB, 0.14205E+01_JPRB ,&
&  0.13807E+01_JPRB, 0.12461E+01_JPRB, 0.10698E+01_JPRB, 0.94012E+00_JPRB, 0.86965E+00_JPRB, 0.83380E+00_JPRB, 0.81207E+00_JPRB ,&
&  0.79410E+00_JPRB, 0.77811E+00_JPRB, 0.75510E+00_JPRB, 0.72253E+00_JPRB, 0.68629E+00_JPRB, 0.66032E+00_JPRB, 0.64108E+00_JPRB ,&
&  0.62925E+00_JPRB, 0.63148E+00_JPRB, 0.64563E+00_JPRB, 0.66964E+00_JPRB, 0.68645E+00_JPRB, 0.69549E+00_JPRB, 0.69542E+00_JPRB ,&
&  0.69410E+00_JPRB, 0.69972E+00_JPRB, 0.65211E+00_JPRB, 0.62257E+00_JPRB, 0.64440E+00_JPRB, 0.60625E+00_JPRB, 0.57294E+00_JPRB ,&
&  0.55715E+00_JPRB, 0.54499E+00_JPRB, 0.54381E+00_JPRB, 0.56089E+00_JPRB, 0.67379E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.59856E+00_JPRB, 0.58210E+00_JPRB, 0.55540E+00_JPRB, 0.52994E+00_JPRB, 0.51344E+00_JPRB, 0.50364E+00_JPRB, 0.49792E+00_JPRB ,&
&  0.49199E+00_JPRB, 0.48469E+00_JPRB, 0.47612E+00_JPRB, 0.46466E+00_JPRB, 0.45745E+00_JPRB, 0.45411E+00_JPRB, 0.45183E+00_JPRB ,&
&  0.44771E+00_JPRB, 0.43876E+00_JPRB, 0.43593E+00_JPRB, 0.43330E+00_JPRB, 0.43204E+00_JPRB, 0.43056E+00_JPRB, 0.42604E+00_JPRB ,&
&  0.42030E+00_JPRB, 0.41771E+00_JPRB, 0.42152E+00_JPRB, 0.42390E+00_JPRB, 0.42642E+00_JPRB, 0.43200E+00_JPRB, 0.44745E+00_JPRB ,&
&  0.51281E+00_JPRB, 0.53082E+00_JPRB, 0.59412E+00_JPRB, 0.64648E+00_JPRB, 0.62880E+00_JPRB, 0.60765E+00_JPRB, 0.59599E+00_JPRB ,&
&  0.57951E+00_JPRB, 0.56785E+00_JPRB, 0.54415E+00_JPRB, 0.51244E+00_JPRB, 0.48631E+00_JPRB, 0.47978E+00_JPRB, 0.47442E+00_JPRB ,&
&  0.47002E+00_JPRB, 0.46826E+00_JPRB, 0.46752E+00_JPRB, 0.46776E+00_JPRB, 0.46823E+00_JPRB, 0.46776E+00_JPRB, 0.46711E+00_JPRB ,&
&  0.46830E+00_JPRB, 0.47208E+00_JPRB, 0.47685E+00_JPRB, 0.49057E+00_JPRB, 0.54821E+00_JPRB, 0.65773E+00_JPRB, 0.77648E+00_JPRB ,&
&  0.85185E+00_JPRB, 0.91727E+00_JPRB, 0.10115E+01_JPRB, 0.10907E+01_JPRB, 0.11396E+01_JPRB, 0.11568E+01_JPRB, 0.11491E+01_JPRB ,&
&  0.11325E+01_JPRB, 0.10852E+01_JPRB, 0.98424E+00_JPRB, 0.87624E+00_JPRB, 0.80563E+00_JPRB, 0.76167E+00_JPRB, 0.73121E+00_JPRB ,&
&  0.70916E+00_JPRB, 0.68224E+00_JPRB, 0.65484E+00_JPRB, 0.63130E+00_JPRB, 0.61607E+00_JPRB, 0.60220E+00_JPRB, 0.59144E+00_JPRB ,&
&  0.59523E+00_JPRB, 0.61357E+00_JPRB, 0.63948E+00_JPRB, 0.65596E+00_JPRB, 0.66078E+00_JPRB, 0.66257E+00_JPRB, 0.65821E+00_JPRB ,&
&  0.65623E+00_JPRB, 0.65929E+00_JPRB, 0.65391E+00_JPRB, 0.65016E+00_JPRB, 0.62483E+00_JPRB, 0.60276E+00_JPRB, 0.56799E+00_JPRB ,&
&  0.54781E+00_JPRB, 0.53524E+00_JPRB, 0.52275E+00_JPRB, 0.53614E+00_JPRB, 0.58656E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.53824E+00_JPRB, 0.53704E+00_JPRB, 0.53181E+00_JPRB, 0.52147E+00_JPRB, 0.51476E+00_JPRB, 0.50851E+00_JPRB, 0.49952E+00_JPRB ,&
&  0.49000E+00_JPRB, 0.48170E+00_JPRB, 0.47399E+00_JPRB, 0.46413E+00_JPRB, 0.45323E+00_JPRB, 0.44959E+00_JPRB, 0.44881E+00_JPRB ,&
&  0.44544E+00_JPRB, 0.43828E+00_JPRB, 0.43141E+00_JPRB, 0.42983E+00_JPRB, 0.42916E+00_JPRB, 0.42869E+00_JPRB, 0.42567E+00_JPRB ,&
&  0.42067E+00_JPRB, 0.42008E+00_JPRB, 0.42376E+00_JPRB, 0.42505E+00_JPRB, 0.42149E+00_JPRB, 0.42340E+00_JPRB, 0.43475E+00_JPRB ,&
&  0.48967E+00_JPRB, 0.44229E+00_JPRB, 0.48280E+00_JPRB, 0.55949E+00_JPRB, 0.54719E+00_JPRB, 0.54212E+00_JPRB, 0.52310E+00_JPRB ,&
&  0.50327E+00_JPRB, 0.49356E+00_JPRB, 0.48520E+00_JPRB, 0.47076E+00_JPRB, 0.45540E+00_JPRB, 0.44683E+00_JPRB, 0.44607E+00_JPRB ,&
&  0.44960E+00_JPRB, 0.45184E+00_JPRB, 0.45297E+00_JPRB, 0.44916E+00_JPRB, 0.44702E+00_JPRB, 0.44570E+00_JPRB, 0.44539E+00_JPRB ,&
&  0.44541E+00_JPRB, 0.44831E+00_JPRB, 0.45263E+00_JPRB, 0.45738E+00_JPRB, 0.47465E+00_JPRB, 0.52975E+00_JPRB, 0.61896E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.77627E+00_JPRB, 0.83937E+00_JPRB, 0.90108E+00_JPRB, 0.94455E+00_JPRB, 0.98123E+00_JPRB, 0.99699E+00_JPRB ,&
&  0.97694E+00_JPRB, 0.92928E+00_JPRB, 0.86953E+00_JPRB, 0.80215E+00_JPRB, 0.74122E+00_JPRB, 0.69302E+00_JPRB, 0.65590E+00_JPRB ,&
&  0.62795E+00_JPRB, 0.60951E+00_JPRB, 0.59510E+00_JPRB, 0.57947E+00_JPRB, 0.57071E+00_JPRB, 0.57050E+00_JPRB, 0.57710E+00_JPRB ,&
&  0.58961E+00_JPRB, 0.60346E+00_JPRB, 0.61235E+00_JPRB, 0.61430E+00_JPRB, 0.60991E+00_JPRB, 0.60567E+00_JPRB, 0.60189E+00_JPRB ,&
&  0.60678E+00_JPRB, 0.61759E+00_JPRB, 0.62889E+00_JPRB, 0.61292E+00_JPRB, 0.62080E+00_JPRB, 0.61193E+00_JPRB, 0.58617E+00_JPRB ,&
&  0.55898E+00_JPRB, 0.53805E+00_JPRB, 0.52598E+00_JPRB, 0.46492E+00_JPRB, 0.53514E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.50625E+00_JPRB, 0.50781E+00_JPRB, 0.50735E+00_JPRB, 0.50577E+00_JPRB, 0.50325E+00_JPRB, 0.49747E+00_JPRB, 0.48819E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.46948E+00_JPRB, 0.46303E+00_JPRB, 0.45655E+00_JPRB, 0.44674E+00_JPRB, 0.44061E+00_JPRB, 0.43986E+00_JPRB ,&
&  0.43841E+00_JPRB, 0.43364E+00_JPRB, 0.42806E+00_JPRB, 0.42731E+00_JPRB, 0.42554E+00_JPRB, 0.42289E+00_JPRB, 0.41911E+00_JPRB ,&
&  0.41517E+00_JPRB, 0.41402E+00_JPRB, 0.41478E+00_JPRB, 0.41621E+00_JPRB, 0.41338E+00_JPRB, 0.41487E+00_JPRB, 0.42591E+00_JPRB ,&
&  0.46922E+00_JPRB, 0.40547E+00_JPRB, 0.44495E+00_JPRB, 0.48867E+00_JPRB, 0.47213E+00_JPRB, 0.45797E+00_JPRB, 0.44557E+00_JPRB ,&
&  0.43673E+00_JPRB, 0.43382E+00_JPRB, 0.43246E+00_JPRB, 0.42889E+00_JPRB, 0.42495E+00_JPRB, 0.42021E+00_JPRB, 0.41858E+00_JPRB ,&
&  0.41907E+00_JPRB, 0.42377E+00_JPRB, 0.42656E+00_JPRB, 0.42686E+00_JPRB, 0.42406E+00_JPRB, 0.42113E+00_JPRB, 0.41876E+00_JPRB ,&
&  0.41803E+00_JPRB, 0.41959E+00_JPRB, 0.42221E+00_JPRB, 0.42660E+00_JPRB, 0.43621E+00_JPRB, 0.46091E+00_JPRB, 0.51103E+00_JPRB ,&
&  0.57846E+00_JPRB, 0.63694E+00_JPRB, 0.68955E+00_JPRB, 0.74962E+00_JPRB, 0.80618E+00_JPRB, 0.84395E+00_JPRB, 0.85231E+00_JPRB ,&
&  0.83235E+00_JPRB, 0.78912E+00_JPRB, 0.75160E+00_JPRB, 0.71443E+00_JPRB, 0.67854E+00_JPRB, 0.64832E+00_JPRB, 0.62696E+00_JPRB ,&
&  0.61301E+00_JPRB, 0.60390E+00_JPRB, 0.59872E+00_JPRB, 0.59507E+00_JPRB, 0.59560E+00_JPRB, 0.59750E+00_JPRB, 0.60108E+00_JPRB ,&
&  0.59909E+00_JPRB, 0.59367E+00_JPRB, 0.57898E+00_JPRB, 0.55869E+00_JPRB, 0.54523E+00_JPRB, 0.53885E+00_JPRB, 0.53677E+00_JPRB ,&
&  0.54041E+00_JPRB, 0.55054E+00_JPRB, 0.56422E+00_JPRB, 0.58677E+00_JPRB, 0.59341E+00_JPRB, 0.59818E+00_JPRB, 0.58980E+00_JPRB ,&
&  0.56776E+00_JPRB, 0.54361E+00_JPRB, 0.44425E+00_JPRB, 0.49408E+00_JPRB, 0.50797E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.50895E+00_JPRB, 0.50368E+00_JPRB, 0.49885E+00_JPRB, 0.49404E+00_JPRB, 0.48799E+00_JPRB, 0.48027E+00_JPRB, 0.47115E+00_JPRB ,&
&  0.46279E+00_JPRB, 0.45516E+00_JPRB, 0.44758E+00_JPRB, 0.44118E+00_JPRB, 0.43527E+00_JPRB, 0.43276E+00_JPRB, 0.43232E+00_JPRB ,&
&  0.43284E+00_JPRB, 0.43224E+00_JPRB, 0.43101E+00_JPRB, 0.43125E+00_JPRB, 0.43096E+00_JPRB, 0.42864E+00_JPRB, 0.42526E+00_JPRB ,&
&  0.41974E+00_JPRB, 0.41501E+00_JPRB, 0.41341E+00_JPRB, 0.41595E+00_JPRB, 0.41513E+00_JPRB, 0.41703E+00_JPRB, 0.42449E+00_JPRB ,&
&  0.40813E+00_JPRB, 0.38003E+00_JPRB, 0.41040E+00_JPRB, 0.44357E+00_JPRB, 0.43198E+00_JPRB, 0.42080E+00_JPRB, 0.41614E+00_JPRB ,&
&  0.41137E+00_JPRB, 0.40910E+00_JPRB, 0.40736E+00_JPRB, 0.40693E+00_JPRB, 0.40836E+00_JPRB, 0.40806E+00_JPRB, 0.40641E+00_JPRB ,&
&  0.40539E+00_JPRB, 0.40797E+00_JPRB, 0.41079E+00_JPRB, 0.41244E+00_JPRB, 0.41244E+00_JPRB, 0.41141E+00_JPRB, 0.40795E+00_JPRB ,&
&  0.40475E+00_JPRB, 0.40299E+00_JPRB, 0.40334E+00_JPRB, 0.40719E+00_JPRB, 0.41405E+00_JPRB, 0.42717E+00_JPRB, 0.45303E+00_JPRB ,&
&  0.49324E+00_JPRB, 0.53633E+00_JPRB, 0.57578E+00_JPRB, 0.61911E+00_JPRB, 0.67153E+00_JPRB, 0.72003E+00_JPRB, 0.74901E+00_JPRB ,&
&  0.74151E+00_JPRB, 0.70847E+00_JPRB, 0.66958E+00_JPRB, 0.63722E+00_JPRB, 0.61317E+00_JPRB, 0.59783E+00_JPRB, 0.58871E+00_JPRB ,&
&  0.58536E+00_JPRB, 0.58345E+00_JPRB, 0.58236E+00_JPRB, 0.58220E+00_JPRB, 0.58174E+00_JPRB, 0.57801E+00_JPRB, 0.57134E+00_JPRB ,&
&  0.55976E+00_JPRB, 0.54180E+00_JPRB, 0.52157E+00_JPRB, 0.50533E+00_JPRB, 0.49420E+00_JPRB, 0.48940E+00_JPRB, 0.48953E+00_JPRB ,&
&  0.49475E+00_JPRB, 0.50678E+00_JPRB, 0.52320E+00_JPRB, 0.55149E+00_JPRB, 0.57149E+00_JPRB, 0.58516E+00_JPRB, 0.58975E+00_JPRB ,&
&  0.57816E+00_JPRB, 0.55598E+00_JPRB, 0.53454E+00_JPRB, 0.52241E+00_JPRB, 0.51754E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.50518E+00_JPRB, 0.49205E+00_JPRB, 0.47907E+00_JPRB, 0.47036E+00_JPRB, 0.46391E+00_JPRB, 0.45756E+00_JPRB, 0.45038E+00_JPRB ,&
&  0.44192E+00_JPRB, 0.43371E+00_JPRB, 0.42612E+00_JPRB, 0.41902E+00_JPRB, 0.41559E+00_JPRB, 0.41407E+00_JPRB, 0.41475E+00_JPRB ,&
&  0.41750E+00_JPRB, 0.42219E+00_JPRB, 0.42423E+00_JPRB, 0.42409E+00_JPRB, 0.42481E+00_JPRB, 0.42466E+00_JPRB, 0.42295E+00_JPRB ,&
&  0.41886E+00_JPRB, 0.41389E+00_JPRB, 0.41122E+00_JPRB, 0.41203E+00_JPRB, 0.41228E+00_JPRB, 0.41287E+00_JPRB, 0.41275E+00_JPRB ,&
&  0.41479E+00_JPRB, 0.37963E+00_JPRB, 0.40369E+00_JPRB, 0.41984E+00_JPRB, 0.41568E+00_JPRB, 0.41238E+00_JPRB, 0.40777E+00_JPRB ,&
&  0.40120E+00_JPRB, 0.39770E+00_JPRB, 0.39603E+00_JPRB, 0.39766E+00_JPRB, 0.40074E+00_JPRB, 0.40208E+00_JPRB, 0.40220E+00_JPRB ,&
&  0.39941E+00_JPRB, 0.39950E+00_JPRB, 0.39916E+00_JPRB, 0.39949E+00_JPRB, 0.40050E+00_JPRB, 0.40164E+00_JPRB, 0.40184E+00_JPRB ,&
&  0.40060E+00_JPRB, 0.39775E+00_JPRB, 0.39493E+00_JPRB, 0.39486E+00_JPRB, 0.39663E+00_JPRB, 0.40114E+00_JPRB, 0.41084E+00_JPRB ,&
&  0.42815E+00_JPRB, 0.45034E+00_JPRB, 0.47658E+00_JPRB, 0.50875E+00_JPRB, 0.55251E+00_JPRB, 0.59654E+00_JPRB, 0.63072E+00_JPRB ,&
&  0.64092E+00_JPRB, 0.63257E+00_JPRB, 0.61291E+00_JPRB, 0.58856E+00_JPRB, 0.56646E+00_JPRB, 0.54559E+00_JPRB, 0.52711E+00_JPRB ,&
&  0.51345E+00_JPRB, 0.50276E+00_JPRB, 0.49520E+00_JPRB, 0.49095E+00_JPRB, 0.48698E+00_JPRB, 0.48095E+00_JPRB, 0.47503E+00_JPRB ,&
&  0.46868E+00_JPRB, 0.46292E+00_JPRB, 0.45857E+00_JPRB, 0.45466E+00_JPRB, 0.45037E+00_JPRB, 0.44678E+00_JPRB, 0.44646E+00_JPRB ,&
&  0.45067E+00_JPRB, 0.45914E+00_JPRB, 0.47386E+00_JPRB, 0.49452E+00_JPRB, 0.51780E+00_JPRB, 0.53613E+00_JPRB, 0.54646E+00_JPRB ,&
&  0.54734E+00_JPRB, 0.54330E+00_JPRB, 0.53378E+00_JPRB, 0.52450E+00_JPRB, 0.51652E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.48515E+00_JPRB, 0.47463E+00_JPRB, 0.46395E+00_JPRB, 0.45551E+00_JPRB, 0.44891E+00_JPRB, 0.44176E+00_JPRB, 0.43372E+00_JPRB ,&
&  0.42719E+00_JPRB, 0.41996E+00_JPRB, 0.41468E+00_JPRB, 0.41013E+00_JPRB, 0.40673E+00_JPRB, 0.40572E+00_JPRB, 0.40663E+00_JPRB ,&
&  0.40927E+00_JPRB, 0.41477E+00_JPRB, 0.41768E+00_JPRB, 0.41636E+00_JPRB, 0.41820E+00_JPRB, 0.42058E+00_JPRB, 0.42115E+00_JPRB ,&
&  0.41889E+00_JPRB, 0.41584E+00_JPRB, 0.41247E+00_JPRB, 0.41046E+00_JPRB, 0.41039E+00_JPRB, 0.41127E+00_JPRB, 0.40800E+00_JPRB ,&
&  0.40199E+00_JPRB, 0.39357E+00_JPRB, 0.40376E+00_JPRB, 0.40969E+00_JPRB, 0.40996E+00_JPRB, 0.40706E+00_JPRB, 0.40305E+00_JPRB ,&
&  0.39717E+00_JPRB, 0.39388E+00_JPRB, 0.39472E+00_JPRB, 0.39696E+00_JPRB, 0.39945E+00_JPRB, 0.40055E+00_JPRB, 0.40161E+00_JPRB ,&
&  0.39849E+00_JPRB, 0.39695E+00_JPRB, 0.39540E+00_JPRB, 0.39461E+00_JPRB, 0.39563E+00_JPRB, 0.39693E+00_JPRB, 0.39776E+00_JPRB ,&
&  0.39697E+00_JPRB, 0.39781E+00_JPRB, 0.39888E+00_JPRB, 0.39833E+00_JPRB, 0.39998E+00_JPRB, 0.40241E+00_JPRB, 0.40607E+00_JPRB ,&
&  0.41222E+00_JPRB, 0.42165E+00_JPRB, 0.43318E+00_JPRB, 0.45000E+00_JPRB, 0.47703E+00_JPRB, 0.50740E+00_JPRB, 0.53150E+00_JPRB ,&
&  0.54386E+00_JPRB, 0.54502E+00_JPRB, 0.53878E+00_JPRB, 0.52690E+00_JPRB, 0.51489E+00_JPRB, 0.50327E+00_JPRB, 0.48980E+00_JPRB ,&
&  0.47499E+00_JPRB, 0.46379E+00_JPRB, 0.45504E+00_JPRB, 0.44852E+00_JPRB, 0.44255E+00_JPRB, 0.43785E+00_JPRB, 0.43541E+00_JPRB ,&
&  0.43514E+00_JPRB, 0.43533E+00_JPRB, 0.43416E+00_JPRB, 0.43249E+00_JPRB, 0.43180E+00_JPRB, 0.42950E+00_JPRB, 0.42825E+00_JPRB ,&
&  0.43023E+00_JPRB, 0.43568E+00_JPRB, 0.44500E+00_JPRB, 0.45985E+00_JPRB, 0.47511E+00_JPRB, 0.48804E+00_JPRB, 0.49796E+00_JPRB ,&
&  0.50475E+00_JPRB, 0.50761E+00_JPRB, 0.50596E+00_JPRB, 0.50176E+00_JPRB, 0.49497E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.44616E+00_JPRB, 0.44096E+00_JPRB, 0.43476E+00_JPRB, 0.42862E+00_JPRB, 0.42285E+00_JPRB, 0.41698E+00_JPRB, 0.41163E+00_JPRB ,&
&  0.40698E+00_JPRB, 0.40425E+00_JPRB, 0.40259E+00_JPRB, 0.40130E+00_JPRB, 0.39812E+00_JPRB, 0.39676E+00_JPRB, 0.39759E+00_JPRB ,&
&  0.40011E+00_JPRB, 0.40265E+00_JPRB, 0.40447E+00_JPRB, 0.40407E+00_JPRB, 0.40644E+00_JPRB, 0.40872E+00_JPRB, 0.40899E+00_JPRB ,&
&  0.40824E+00_JPRB, 0.40751E+00_JPRB, 0.40549E+00_JPRB, 0.40286E+00_JPRB, 0.40280E+00_JPRB, 0.40298E+00_JPRB, 0.39904E+00_JPRB ,&
&  0.39175E+00_JPRB, 0.39241E+00_JPRB, 0.38469E+00_JPRB, 0.39724E+00_JPRB, 0.39999E+00_JPRB, 0.39508E+00_JPRB, 0.38912E+00_JPRB ,&
&  0.38746E+00_JPRB, 0.38880E+00_JPRB, 0.39229E+00_JPRB, 0.39460E+00_JPRB, 0.39536E+00_JPRB, 0.39554E+00_JPRB, 0.39548E+00_JPRB ,&
&  0.38826E+00_JPRB, 0.38420E+00_JPRB, 0.38413E+00_JPRB, 0.38524E+00_JPRB, 0.38550E+00_JPRB, 0.38631E+00_JPRB, 0.38830E+00_JPRB ,&
&  0.38984E+00_JPRB, 0.39090E+00_JPRB, 0.39211E+00_JPRB, 0.39479E+00_JPRB, 0.39671E+00_JPRB, 0.39768E+00_JPRB, 0.39963E+00_JPRB ,&
&  0.40279E+00_JPRB, 0.40728E+00_JPRB, 0.41219E+00_JPRB, 0.41778E+00_JPRB, 0.42963E+00_JPRB, 0.44705E+00_JPRB, 0.46227E+00_JPRB ,&
&  0.47032E+00_JPRB, 0.47269E+00_JPRB, 0.47139E+00_JPRB, 0.46556E+00_JPRB, 0.45661E+00_JPRB, 0.44877E+00_JPRB, 0.44184E+00_JPRB ,&
&  0.43266E+00_JPRB, 0.42657E+00_JPRB, 0.42138E+00_JPRB, 0.41500E+00_JPRB, 0.40940E+00_JPRB, 0.40574E+00_JPRB, 0.40389E+00_JPRB ,&
&  0.40404E+00_JPRB, 0.40604E+00_JPRB, 0.40683E+00_JPRB, 0.40826E+00_JPRB, 0.41128E+00_JPRB, 0.41468E+00_JPRB, 0.41476E+00_JPRB ,&
&  0.41356E+00_JPRB, 0.41334E+00_JPRB, 0.41554E+00_JPRB, 0.42053E+00_JPRB, 0.42620E+00_JPRB, 0.43319E+00_JPRB, 0.44065E+00_JPRB ,&
&  0.44735E+00_JPRB, 0.45177E+00_JPRB, 0.45325E+00_JPRB, 0.45333E+00_JPRB, 0.45060E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.40938E+00_JPRB, 0.40766E+00_JPRB, 0.40558E+00_JPRB, 0.40238E+00_JPRB, 0.39890E+00_JPRB, 0.39579E+00_JPRB, 0.39330E+00_JPRB ,&
&  0.39172E+00_JPRB, 0.39143E+00_JPRB, 0.39205E+00_JPRB, 0.39231E+00_JPRB, 0.39222E+00_JPRB, 0.39310E+00_JPRB, 0.39350E+00_JPRB ,&
&  0.39510E+00_JPRB, 0.39546E+00_JPRB, 0.39333E+00_JPRB, 0.39293E+00_JPRB, 0.39550E+00_JPRB, 0.40026E+00_JPRB, 0.40211E+00_JPRB ,&
&  0.39986E+00_JPRB, 0.39966E+00_JPRB, 0.39947E+00_JPRB, 0.39741E+00_JPRB, 0.39586E+00_JPRB, 0.39474E+00_JPRB, 0.39066E+00_JPRB ,&
&  0.38549E+00_JPRB, 0.38540E+00_JPRB, 0.38707E+00_JPRB, 0.38999E+00_JPRB, 0.38947E+00_JPRB, 0.38730E+00_JPRB, 0.38766E+00_JPRB ,&
&  0.38601E+00_JPRB, 0.38599E+00_JPRB, 0.38601E+00_JPRB, 0.38673E+00_JPRB, 0.38519E+00_JPRB, 0.38279E+00_JPRB, 0.37830E+00_JPRB ,&
&  0.36946E+00_JPRB, 0.36675E+00_JPRB, 0.36894E+00_JPRB, 0.37329E+00_JPRB, 0.37287E+00_JPRB, 0.37366E+00_JPRB, 0.37676E+00_JPRB ,&
&  0.38066E+00_JPRB, 0.38350E+00_JPRB, 0.38528E+00_JPRB, 0.38651E+00_JPRB, 0.38820E+00_JPRB, 0.38982E+00_JPRB, 0.39073E+00_JPRB ,&
&  0.39190E+00_JPRB, 0.39393E+00_JPRB, 0.39671E+00_JPRB, 0.40027E+00_JPRB, 0.40457E+00_JPRB, 0.41140E+00_JPRB, 0.41843E+00_JPRB ,&
&  0.42238E+00_JPRB, 0.42232E+00_JPRB, 0.42057E+00_JPRB, 0.41620E+00_JPRB, 0.40999E+00_JPRB, 0.40467E+00_JPRB, 0.40024E+00_JPRB ,&
&  0.39369E+00_JPRB, 0.38795E+00_JPRB, 0.38339E+00_JPRB, 0.38110E+00_JPRB, 0.38004E+00_JPRB, 0.38019E+00_JPRB, 0.38152E+00_JPRB ,&
&  0.38273E+00_JPRB, 0.38528E+00_JPRB, 0.39131E+00_JPRB, 0.39513E+00_JPRB, 0.39687E+00_JPRB, 0.39991E+00_JPRB, 0.40149E+00_JPRB ,&
&  0.39997E+00_JPRB, 0.39869E+00_JPRB, 0.39784E+00_JPRB, 0.40009E+00_JPRB, 0.40190E+00_JPRB, 0.40219E+00_JPRB, 0.40295E+00_JPRB ,&
&  0.40607E+00_JPRB, 0.40946E+00_JPRB, 0.41031E+00_JPRB, 0.41047E+00_JPRB, 0.41067E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.37736E+00_JPRB, 0.37749E+00_JPRB, 0.37698E+00_JPRB, 0.37600E+00_JPRB, 0.37504E+00_JPRB, 0.37429E+00_JPRB, 0.37449E+00_JPRB ,&
&  0.37482E+00_JPRB, 0.37562E+00_JPRB, 0.37664E+00_JPRB, 0.37749E+00_JPRB, 0.37925E+00_JPRB, 0.38126E+00_JPRB, 0.38288E+00_JPRB ,&
&  0.38462E+00_JPRB, 0.38407E+00_JPRB, 0.38154E+00_JPRB, 0.37902E+00_JPRB, 0.37975E+00_JPRB, 0.38508E+00_JPRB, 0.39099E+00_JPRB ,&
&  0.39243E+00_JPRB, 0.39343E+00_JPRB, 0.39384E+00_JPRB, 0.39264E+00_JPRB, 0.39101E+00_JPRB, 0.39040E+00_JPRB, 0.38665E+00_JPRB ,&
&  0.38194E+00_JPRB, 0.38032E+00_JPRB, 0.37984E+00_JPRB, 0.37718E+00_JPRB, 0.37673E+00_JPRB, 0.37902E+00_JPRB, 0.37804E+00_JPRB ,&
&  0.37722E+00_JPRB, 0.37566E+00_JPRB, 0.37278E+00_JPRB, 0.37022E+00_JPRB, 0.36712E+00_JPRB, 0.36115E+00_JPRB, 0.35420E+00_JPRB ,&
&  0.34868E+00_JPRB, 0.34889E+00_JPRB, 0.35421E+00_JPRB, 0.35982E+00_JPRB, 0.35892E+00_JPRB, 0.35799E+00_JPRB, 0.35954E+00_JPRB ,&
&  0.36311E+00_JPRB, 0.36760E+00_JPRB, 0.37048E+00_JPRB, 0.36972E+00_JPRB, 0.36866E+00_JPRB, 0.37054E+00_JPRB, 0.37321E+00_JPRB ,&
&  0.37450E+00_JPRB, 0.37594E+00_JPRB, 0.37704E+00_JPRB, 0.37877E+00_JPRB, 0.38005E+00_JPRB, 0.38040E+00_JPRB, 0.38212E+00_JPRB ,&
&  0.38217E+00_JPRB, 0.38053E+00_JPRB, 0.37678E+00_JPRB, 0.37133E+00_JPRB, 0.36662E+00_JPRB, 0.36299E+00_JPRB, 0.36184E+00_JPRB ,&
&  0.36020E+00_JPRB, 0.36084E+00_JPRB, 0.36058E+00_JPRB, 0.35799E+00_JPRB, 0.35477E+00_JPRB, 0.35453E+00_JPRB, 0.35794E+00_JPRB ,&
&  0.36282E+00_JPRB, 0.36641E+00_JPRB, 0.37213E+00_JPRB, 0.37478E+00_JPRB, 0.37658E+00_JPRB, 0.37900E+00_JPRB, 0.37973E+00_JPRB ,&
&  0.37723E+00_JPRB, 0.37521E+00_JPRB, 0.37485E+00_JPRB, 0.37778E+00_JPRB, 0.37864E+00_JPRB, 0.37760E+00_JPRB, 0.37474E+00_JPRB ,&
&  0.37378E+00_JPRB, 0.37543E+00_JPRB, 0.37597E+00_JPRB, 0.37557E+00_JPRB, 0.37608E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.35467E+00_JPRB, 0.35679E+00_JPRB, 0.35899E+00_JPRB, 0.36097E+00_JPRB, 0.36228E+00_JPRB, 0.36329E+00_JPRB, 0.36253E+00_JPRB ,&
&  0.36244E+00_JPRB, 0.36414E+00_JPRB, 0.36629E+00_JPRB, 0.36806E+00_JPRB, 0.36909E+00_JPRB, 0.37106E+00_JPRB, 0.37304E+00_JPRB ,&
&  0.37436E+00_JPRB, 0.37357E+00_JPRB, 0.37029E+00_JPRB, 0.36879E+00_JPRB, 0.36768E+00_JPRB, 0.37262E+00_JPRB, 0.38106E+00_JPRB ,&
&  0.38473E+00_JPRB, 0.38698E+00_JPRB, 0.38974E+00_JPRB, 0.39024E+00_JPRB, 0.39036E+00_JPRB, 0.38993E+00_JPRB, 0.38537E+00_JPRB ,&
&  0.38032E+00_JPRB, 0.37556E+00_JPRB, 0.37264E+00_JPRB, 0.36930E+00_JPRB, 0.36915E+00_JPRB, 0.36687E+00_JPRB, 0.36524E+00_JPRB ,&
&  0.36408E+00_JPRB, 0.36160E+00_JPRB, 0.35854E+00_JPRB, 0.35337E+00_JPRB, 0.34696E+00_JPRB, 0.33866E+00_JPRB, 0.33300E+00_JPRB ,&
&  0.33083E+00_JPRB, 0.33445E+00_JPRB, 0.34286E+00_JPRB, 0.34670E+00_JPRB, 0.34351E+00_JPRB, 0.34281E+00_JPRB, 0.34603E+00_JPRB ,&
&  0.35082E+00_JPRB, 0.35611E+00_JPRB, 0.35801E+00_JPRB, 0.35721E+00_JPRB, 0.35531E+00_JPRB, 0.35688E+00_JPRB, 0.36108E+00_JPRB ,&
&  0.36477E+00_JPRB, 0.36672E+00_JPRB, 0.36602E+00_JPRB, 0.36522E+00_JPRB, 0.36298E+00_JPRB, 0.35782E+00_JPRB, 0.35084E+00_JPRB ,&
&  0.34450E+00_JPRB, 0.34050E+00_JPRB, 0.33740E+00_JPRB, 0.33425E+00_JPRB, 0.33296E+00_JPRB, 0.33318E+00_JPRB, 0.33528E+00_JPRB ,&
&  0.33606E+00_JPRB, 0.33785E+00_JPRB, 0.33791E+00_JPRB, 0.33372E+00_JPRB, 0.33041E+00_JPRB, 0.33189E+00_JPRB, 0.33373E+00_JPRB ,&
&  0.33648E+00_JPRB, 0.33915E+00_JPRB, 0.34184E+00_JPRB, 0.34291E+00_JPRB, 0.34446E+00_JPRB, 0.34711E+00_JPRB, 0.34868E+00_JPRB ,&
&  0.34945E+00_JPRB, 0.35001E+00_JPRB, 0.35181E+00_JPRB, 0.35485E+00_JPRB, 0.35766E+00_JPRB, 0.35986E+00_JPRB, 0.35910E+00_JPRB ,&
&  0.35764E+00_JPRB, 0.35597E+00_JPRB, 0.35621E+00_JPRB, 0.35631E+00_JPRB, 0.35470E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.34943E+00_JPRB, 0.34924E+00_JPRB, 0.35240E+00_JPRB, 0.35585E+00_JPRB, 0.35665E+00_JPRB, 0.35641E+00_JPRB, 0.35392E+00_JPRB ,&
&  0.35225E+00_JPRB, 0.35341E+00_JPRB, 0.35518E+00_JPRB, 0.35645E+00_JPRB, 0.35854E+00_JPRB, 0.36136E+00_JPRB, 0.36360E+00_JPRB ,&
&  0.36370E+00_JPRB, 0.36193E+00_JPRB, 0.35887E+00_JPRB, 0.35948E+00_JPRB, 0.35943E+00_JPRB, 0.36146E+00_JPRB, 0.36886E+00_JPRB ,&
&  0.37415E+00_JPRB, 0.37773E+00_JPRB, 0.38075E+00_JPRB, 0.38105E+00_JPRB, 0.38037E+00_JPRB, 0.38170E+00_JPRB, 0.37828E+00_JPRB ,&
&  0.37205E+00_JPRB, 0.36620E+00_JPRB, 0.36101E+00_JPRB, 0.35406E+00_JPRB, 0.35660E+00_JPRB, 0.35406E+00_JPRB, 0.35096E+00_JPRB ,&
&  0.34683E+00_JPRB, 0.34157E+00_JPRB, 0.33546E+00_JPRB, 0.32973E+00_JPRB, 0.32552E+00_JPRB, 0.32163E+00_JPRB, 0.31657E+00_JPRB ,&
&  0.31497E+00_JPRB, 0.31702E+00_JPRB, 0.32267E+00_JPRB, 0.32520E+00_JPRB, 0.32266E+00_JPRB, 0.32353E+00_JPRB, 0.33021E+00_JPRB ,&
&  0.33706E+00_JPRB, 0.34029E+00_JPRB, 0.34018E+00_JPRB, 0.34076E+00_JPRB, 0.34138E+00_JPRB, 0.34274E+00_JPRB, 0.34340E+00_JPRB ,&
&  0.34236E+00_JPRB, 0.34072E+00_JPRB, 0.33820E+00_JPRB, 0.33233E+00_JPRB, 0.32543E+00_JPRB, 0.31971E+00_JPRB, 0.31756E+00_JPRB ,&
&  0.32037E+00_JPRB, 0.32038E+00_JPRB, 0.32063E+00_JPRB, 0.32235E+00_JPRB, 0.32294E+00_JPRB, 0.32219E+00_JPRB, 0.32121E+00_JPRB ,&
&  0.32247E+00_JPRB, 0.32160E+00_JPRB, 0.32296E+00_JPRB, 0.32015E+00_JPRB, 0.31574E+00_JPRB, 0.31154E+00_JPRB, 0.30759E+00_JPRB ,&
&  0.30534E+00_JPRB, 0.30465E+00_JPRB, 0.30672E+00_JPRB, 0.30741E+00_JPRB, 0.30920E+00_JPRB, 0.31134E+00_JPRB, 0.31317E+00_JPRB ,&
&  0.31420E+00_JPRB, 0.31556E+00_JPRB, 0.31816E+00_JPRB, 0.32341E+00_JPRB, 0.32898E+00_JPRB, 0.33387E+00_JPRB, 0.33888E+00_JPRB ,&
&  0.34352E+00_JPRB, 0.34680E+00_JPRB, 0.35257E+00_JPRB, 0.35365E+00_JPRB, 0.35269E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.34845E+00_JPRB, 0.34599E+00_JPRB, 0.34600E+00_JPRB, 0.34679E+00_JPRB, 0.34626E+00_JPRB, 0.34543E+00_JPRB, 0.34281E+00_JPRB ,&
&  0.34047E+00_JPRB, 0.34083E+00_JPRB, 0.34219E+00_JPRB, 0.34470E+00_JPRB, 0.34912E+00_JPRB, 0.35381E+00_JPRB, 0.35325E+00_JPRB ,&
&  0.35019E+00_JPRB, 0.34889E+00_JPRB, 0.35108E+00_JPRB, 0.35320E+00_JPRB, 0.35355E+00_JPRB, 0.35417E+00_JPRB, 0.35778E+00_JPRB ,&
&  0.36234E+00_JPRB, 0.36435E+00_JPRB, 0.36720E+00_JPRB, 0.36671E+00_JPRB, 0.36290E+00_JPRB, 0.36294E+00_JPRB, 0.36107E+00_JPRB ,&
&  0.35616E+00_JPRB, 0.35021E+00_JPRB, 0.34116E+00_JPRB, 0.34207E+00_JPRB, 0.33843E+00_JPRB, 0.33546E+00_JPRB, 0.33105E+00_JPRB ,&
&  0.32620E+00_JPRB, 0.32215E+00_JPRB, 0.31761E+00_JPRB, 0.31393E+00_JPRB, 0.31207E+00_JPRB, 0.31199E+00_JPRB, 0.30986E+00_JPRB ,&
&  0.30854E+00_JPRB, 0.31020E+00_JPRB, 0.31151E+00_JPRB, 0.30870E+00_JPRB, 0.30858E+00_JPRB, 0.31498E+00_JPRB, 0.32015E+00_JPRB ,&
&  0.32211E+00_JPRB, 0.32207E+00_JPRB, 0.31950E+00_JPRB, 0.31884E+00_JPRB, 0.31779E+00_JPRB, 0.31582E+00_JPRB, 0.31246E+00_JPRB ,&
&  0.30746E+00_JPRB, 0.30505E+00_JPRB, 0.30343E+00_JPRB, 0.30013E+00_JPRB, 0.29477E+00_JPRB, 0.27795E+00_JPRB, 0.26868E+00_JPRB ,&
&  0.26866E+00_JPRB, 0.27760E+00_JPRB, 0.29259E+00_JPRB, 0.29925E+00_JPRB, 0.30493E+00_JPRB, 0.31000E+00_JPRB, 0.31083E+00_JPRB ,&
&  0.31223E+00_JPRB, 0.29194E+00_JPRB, 0.28085E+00_JPRB, 0.27216E+00_JPRB, 0.26448E+00_JPRB, 0.25964E+00_JPRB, 0.25829E+00_JPRB ,&
&  0.26120E+00_JPRB, 0.26717E+00_JPRB, 0.27245E+00_JPRB, 0.27501E+00_JPRB, 0.27471E+00_JPRB, 0.27158E+00_JPRB, 0.26829E+00_JPRB ,&
&  0.26799E+00_JPRB, 0.27460E+00_JPRB, 0.28752E+00_JPRB, 0.30048E+00_JPRB, 0.30944E+00_JPRB, 0.31901E+00_JPRB, 0.32710E+00_JPRB ,&
&  0.33272E+00_JPRB, 0.33799E+00_JPRB, 0.34450E+00_JPRB, 0.34696E+00_JPRB, 0.34811E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.33746E+00_JPRB, 0.33393E+00_JPRB, 0.33401E+00_JPRB, 0.33434E+00_JPRB, 0.33318E+00_JPRB, 0.33306E+00_JPRB, 0.33236E+00_JPRB ,&
&  0.33204E+00_JPRB, 0.33014E+00_JPRB, 0.32822E+00_JPRB, 0.32858E+00_JPRB, 0.33063E+00_JPRB, 0.33299E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33344E+00_JPRB, 0.33481E+00_JPRB, 0.33815E+00_JPRB, 0.34169E+00_JPRB, 0.34240E+00_JPRB, 0.34266E+00_JPRB, 0.34445E+00_JPRB ,&
&  0.34673E+00_JPRB, 0.34752E+00_JPRB, 0.35023E+00_JPRB, 0.34800E+00_JPRB, 0.34346E+00_JPRB, 0.34153E+00_JPRB, 0.33984E+00_JPRB ,&
&  0.33353E+00_JPRB, 0.31038E+00_JPRB, 0.29524E+00_JPRB, 0.29950E+00_JPRB, 0.31529E+00_JPRB, 0.31482E+00_JPRB, 0.31271E+00_JPRB ,&
&  0.31051E+00_JPRB, 0.30755E+00_JPRB, 0.30501E+00_JPRB, 0.30404E+00_JPRB, 0.30410E+00_JPRB, 0.30403E+00_JPRB, 0.30257E+00_JPRB ,&
&  0.30294E+00_JPRB, 0.30759E+00_JPRB, 0.31353E+00_JPRB, 0.31095E+00_JPRB, 0.30679E+00_JPRB, 0.30001E+00_JPRB, 0.29570E+00_JPRB ,&
&  0.29165E+00_JPRB, 0.28741E+00_JPRB, 0.28485E+00_JPRB, 0.28402E+00_JPRB, 0.28374E+00_JPRB, 0.28177E+00_JPRB, 0.27844E+00_JPRB ,&
&  0.27093E+00_JPRB, 0.26194E+00_JPRB, 0.25258E+00_JPRB, 0.24338E+00_JPRB, 0.23655E+00_JPRB, 0.23178E+00_JPRB, 0.22814E+00_JPRB ,&
&  0.22982E+00_JPRB, 0.24092E+00_JPRB, 0.25935E+00_JPRB, 0.27914E+00_JPRB, 0.29922E+00_JPRB, 0.27692E+00_JPRB, 0.26608E+00_JPRB ,&
&  0.25488E+00_JPRB, 0.24529E+00_JPRB, 0.23866E+00_JPRB, 0.23410E+00_JPRB, 0.23088E+00_JPRB, 0.22917E+00_JPRB, 0.22922E+00_JPRB ,&
&  0.23219E+00_JPRB, 0.23626E+00_JPRB, 0.23998E+00_JPRB, 0.24093E+00_JPRB, 0.24025E+00_JPRB, 0.23848E+00_JPRB, 0.23749E+00_JPRB ,&
&  0.23863E+00_JPRB, 0.24393E+00_JPRB, 0.25280E+00_JPRB, 0.26567E+00_JPRB, 0.27740E+00_JPRB, 0.28954E+00_JPRB, 0.30371E+00_JPRB ,&
&  0.31888E+00_JPRB, 0.33326E+00_JPRB, 0.33695E+00_JPRB, 0.34014E+00_JPRB, 0.34032E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.33297E+00_JPRB, 0.32972E+00_JPRB, 0.32781E+00_JPRB, 0.32815E+00_JPRB, 0.32837E+00_JPRB, 0.33099E+00_JPRB, 0.33014E+00_JPRB ,&
&  0.32571E+00_JPRB, 0.32107E+00_JPRB, 0.31966E+00_JPRB, 0.31953E+00_JPRB, 0.31841E+00_JPRB, 0.31718E+00_JPRB, 0.31790E+00_JPRB ,&
&  0.31641E+00_JPRB, 0.31829E+00_JPRB, 0.32114E+00_JPRB, 0.32263E+00_JPRB, 0.32336E+00_JPRB, 0.32352E+00_JPRB, 0.32369E+00_JPRB ,&
&  0.31753E+00_JPRB, 0.31580E+00_JPRB, 0.32139E+00_JPRB, 0.31865E+00_JPRB, 0.31474E+00_JPRB, 0.31134E+00_JPRB, 0.31011E+00_JPRB ,&
&  0.30169E+00_JPRB, 0.29006E+00_JPRB, 0.27733E+00_JPRB, 0.28282E+00_JPRB, 0.29967E+00_JPRB, 0.29725E+00_JPRB, 0.29498E+00_JPRB ,&
&  0.29467E+00_JPRB, 0.29396E+00_JPRB, 0.29479E+00_JPRB, 0.29530E+00_JPRB, 0.29628E+00_JPRB, 0.29645E+00_JPRB, 0.29765E+00_JPRB ,&
&  0.29913E+00_JPRB, 0.30499E+00_JPRB, 0.30853E+00_JPRB, 0.28927E+00_JPRB, 0.27252E+00_JPRB, 0.25972E+00_JPRB, 0.24752E+00_JPRB ,&
&  0.23668E+00_JPRB, 0.23402E+00_JPRB, 0.23050E+00_JPRB, 0.22440E+00_JPRB, 0.22493E+00_JPRB, 0.22554E+00_JPRB, 0.22479E+00_JPRB ,&
&  0.22286E+00_JPRB, 0.22168E+00_JPRB, 0.22028E+00_JPRB, 0.21795E+00_JPRB, 0.21707E+00_JPRB, 0.21834E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22743E+00_JPRB, 0.23377E+00_JPRB, 0.24810E+00_JPRB, 0.26040E+00_JPRB, 0.25694E+00_JPRB, 0.24930E+00_JPRB, 0.24189E+00_JPRB ,&
&  0.23146E+00_JPRB, 0.22635E+00_JPRB, 0.22133E+00_JPRB, 0.21726E+00_JPRB, 0.21545E+00_JPRB, 0.21578E+00_JPRB, 0.21671E+00_JPRB ,&
&  0.21802E+00_JPRB, 0.22005E+00_JPRB, 0.22088E+00_JPRB, 0.22051E+00_JPRB, 0.21977E+00_JPRB, 0.22029E+00_JPRB, 0.22192E+00_JPRB ,&
&  0.22491E+00_JPRB, 0.23005E+00_JPRB, 0.23630E+00_JPRB, 0.24401E+00_JPRB, 0.25077E+00_JPRB, 0.25708E+00_JPRB, 0.26682E+00_JPRB ,&
&  0.27479E+00_JPRB, 0.29308E+00_JPRB, 0.31901E+00_JPRB, 0.34453E+00_JPRB, 0.33927E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.31500E+00_JPRB, 0.31245E+00_JPRB, 0.30992E+00_JPRB, 0.30965E+00_JPRB, 0.31033E+00_JPRB, 0.31309E+00_JPRB, 0.31065E+00_JPRB ,&
&  0.30605E+00_JPRB, 0.30432E+00_JPRB, 0.30566E+00_JPRB, 0.30573E+00_JPRB, 0.30515E+00_JPRB, 0.29472E+00_JPRB, 0.28882E+00_JPRB ,&
&  0.28202E+00_JPRB, 0.27975E+00_JPRB, 0.27796E+00_JPRB, 0.27879E+00_JPRB, 0.28104E+00_JPRB, 0.28242E+00_JPRB, 0.27827E+00_JPRB ,&
&  0.27384E+00_JPRB, 0.26961E+00_JPRB, 0.26522E+00_JPRB, 0.26336E+00_JPRB, 0.26467E+00_JPRB, 0.26607E+00_JPRB, 0.26625E+00_JPRB ,&
&  0.26696E+00_JPRB, 0.26971E+00_JPRB, 0.27231E+00_JPRB, 0.27584E+00_JPRB, 0.28855E+00_JPRB, 0.28672E+00_JPRB, 0.28542E+00_JPRB ,&
&  0.28570E+00_JPRB, 0.28612E+00_JPRB, 0.28719E+00_JPRB, 0.28788E+00_JPRB, 0.29190E+00_JPRB, 0.29491E+00_JPRB, 0.29705E+00_JPRB ,&
&  0.28334E+00_JPRB, 0.27541E+00_JPRB, 0.26760E+00_JPRB, 0.25840E+00_JPRB, 0.24767E+00_JPRB, 0.23902E+00_JPRB, 0.23103E+00_JPRB ,&
&  0.22425E+00_JPRB, 0.21790E+00_JPRB, 0.21217E+00_JPRB, 0.20992E+00_JPRB, 0.20911E+00_JPRB, 0.20792E+00_JPRB, 0.20784E+00_JPRB ,&
&  0.20841E+00_JPRB, 0.20919E+00_JPRB, 0.20970E+00_JPRB, 0.21123E+00_JPRB, 0.21360E+00_JPRB, 0.21732E+00_JPRB, 0.22130E+00_JPRB ,&
&  0.22725E+00_JPRB, 0.23169E+00_JPRB, 0.23688E+00_JPRB, 0.24119E+00_JPRB, 0.23971E+00_JPRB, 0.23465E+00_JPRB, 0.22921E+00_JPRB ,&
&  0.22269E+00_JPRB, 0.21897E+00_JPRB, 0.21524E+00_JPRB, 0.21256E+00_JPRB, 0.21107E+00_JPRB, 0.21022E+00_JPRB, 0.21057E+00_JPRB ,&
&  0.21075E+00_JPRB, 0.21124E+00_JPRB, 0.21083E+00_JPRB, 0.21111E+00_JPRB, 0.21197E+00_JPRB, 0.21381E+00_JPRB, 0.21621E+00_JPRB ,&
&  0.21991E+00_JPRB, 0.22420E+00_JPRB, 0.22934E+00_JPRB, 0.23499E+00_JPRB, 0.24324E+00_JPRB, 0.25411E+00_JPRB, 0.26960E+00_JPRB ,&
&  0.28622E+00_JPRB, 0.31751E+00_JPRB, 0.33002E+00_JPRB, 0.32124E+00_JPRB, 0.31836E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.29987E+00_JPRB, 0.30111E+00_JPRB, 0.30246E+00_JPRB, 0.30477E+00_JPRB, 0.30757E+00_JPRB, 0.30853E+00_JPRB, 0.30870E+00_JPRB ,&
&  0.29942E+00_JPRB, 0.29179E+00_JPRB, 0.28496E+00_JPRB, 0.27465E+00_JPRB, 0.26645E+00_JPRB, 0.25905E+00_JPRB, 0.25370E+00_JPRB ,&
&  0.24886E+00_JPRB, 0.24639E+00_JPRB, 0.24373E+00_JPRB, 0.24296E+00_JPRB, 0.24283E+00_JPRB, 0.24349E+00_JPRB, 0.24204E+00_JPRB ,&
&  0.24036E+00_JPRB, 0.24145E+00_JPRB, 0.24214E+00_JPRB, 0.24662E+00_JPRB, 0.25319E+00_JPRB, 0.26063E+00_JPRB, 0.26788E+00_JPRB ,&
&  0.27715E+00_JPRB, 0.28111E+00_JPRB, 0.28424E+00_JPRB, 0.28498E+00_JPRB, 0.28365E+00_JPRB, 0.28303E+00_JPRB, 0.28094E+00_JPRB ,&
&  0.28052E+00_JPRB, 0.28005E+00_JPRB, 0.28134E+00_JPRB, 0.28241E+00_JPRB, 0.28100E+00_JPRB, 0.27852E+00_JPRB, 0.27291E+00_JPRB ,&
&  0.26470E+00_JPRB, 0.25803E+00_JPRB, 0.25177E+00_JPRB, 0.24688E+00_JPRB, 0.24016E+00_JPRB, 0.23424E+00_JPRB, 0.22840E+00_JPRB ,&
&  0.22331E+00_JPRB, 0.21947E+00_JPRB, 0.21495E+00_JPRB, 0.21151E+00_JPRB, 0.20989E+00_JPRB, 0.20798E+00_JPRB, 0.20715E+00_JPRB ,&
&  0.20711E+00_JPRB, 0.20750E+00_JPRB, 0.20958E+00_JPRB, 0.21130E+00_JPRB, 0.21400E+00_JPRB, 0.21712E+00_JPRB, 0.21905E+00_JPRB ,&
&  0.22236E+00_JPRB, 0.22521E+00_JPRB, 0.22676E+00_JPRB, 0.22787E+00_JPRB, 0.22690E+00_JPRB, 0.22402E+00_JPRB, 0.22107E+00_JPRB ,&
&  0.21727E+00_JPRB, 0.21459E+00_JPRB, 0.21289E+00_JPRB, 0.21106E+00_JPRB, 0.21015E+00_JPRB, 0.20937E+00_JPRB, 0.20854E+00_JPRB ,&
&  0.20792E+00_JPRB, 0.20766E+00_JPRB, 0.20740E+00_JPRB, 0.20844E+00_JPRB, 0.20958E+00_JPRB, 0.21163E+00_JPRB, 0.21411E+00_JPRB ,&
&  0.21643E+00_JPRB, 0.21998E+00_JPRB, 0.22387E+00_JPRB, 0.22915E+00_JPRB, 0.23860E+00_JPRB, 0.24646E+00_JPRB, 0.26345E+00_JPRB ,&
&  0.27624E+00_JPRB, 0.30254E+00_JPRB, 0.31328E+00_JPRB, 0.30147E+00_JPRB, 0.30011E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.28963E+00_JPRB, 0.29173E+00_JPRB, 0.29275E+00_JPRB, 0.29474E+00_JPRB, 0.29677E+00_JPRB, 0.29747E+00_JPRB, 0.29789E+00_JPRB ,&
&  0.29539E+00_JPRB, 0.28903E+00_JPRB, 0.28423E+00_JPRB, 0.27774E+00_JPRB, 0.27141E+00_JPRB, 0.26476E+00_JPRB, 0.25874E+00_JPRB ,&
&  0.25250E+00_JPRB, 0.24685E+00_JPRB, 0.24115E+00_JPRB, 0.23634E+00_JPRB, 0.23295E+00_JPRB, 0.23015E+00_JPRB, 0.22715E+00_JPRB ,&
&  0.22456E+00_JPRB, 0.22561E+00_JPRB, 0.22818E+00_JPRB, 0.23314E+00_JPRB, 0.24091E+00_JPRB, 0.24814E+00_JPRB, 0.25602E+00_JPRB ,&
&  0.26983E+00_JPRB, 0.27348E+00_JPRB, 0.27341E+00_JPRB, 0.27541E+00_JPRB, 0.27482E+00_JPRB, 0.27535E+00_JPRB, 0.26538E+00_JPRB ,&
&  0.26759E+00_JPRB, 0.26967E+00_JPRB, 0.27725E+00_JPRB, 0.28039E+00_JPRB, 0.27523E+00_JPRB, 0.27082E+00_JPRB, 0.26287E+00_JPRB ,&
&  0.25630E+00_JPRB, 0.25111E+00_JPRB, 0.24766E+00_JPRB, 0.24557E+00_JPRB, 0.24099E+00_JPRB, 0.23759E+00_JPRB, 0.23192E+00_JPRB ,&
&  0.22720E+00_JPRB, 0.22403E+00_JPRB, 0.22009E+00_JPRB, 0.21763E+00_JPRB, 0.21563E+00_JPRB, 0.21372E+00_JPRB, 0.21226E+00_JPRB ,&
&  0.21147E+00_JPRB, 0.21082E+00_JPRB, 0.21148E+00_JPRB, 0.21227E+00_JPRB, 0.21377E+00_JPRB, 0.21498E+00_JPRB, 0.21526E+00_JPRB ,&
&  0.21553E+00_JPRB, 0.21595E+00_JPRB, 0.21608E+00_JPRB, 0.21666E+00_JPRB, 0.21612E+00_JPRB, 0.21521E+00_JPRB, 0.21410E+00_JPRB ,&
&  0.21292E+00_JPRB, 0.21248E+00_JPRB, 0.21170E+00_JPRB, 0.21121E+00_JPRB, 0.20955E+00_JPRB, 0.20843E+00_JPRB, 0.20786E+00_JPRB ,&
&  0.20732E+00_JPRB, 0.20780E+00_JPRB, 0.20799E+00_JPRB, 0.20888E+00_JPRB, 0.21024E+00_JPRB, 0.21149E+00_JPRB, 0.21364E+00_JPRB ,&
&  0.21632E+00_JPRB, 0.21942E+00_JPRB, 0.22464E+00_JPRB, 0.23016E+00_JPRB, 0.23624E+00_JPRB, 0.24394E+00_JPRB, 0.25105E+00_JPRB ,&
&  0.26104E+00_JPRB, 0.27646E+00_JPRB, 0.27866E+00_JPRB, 0.28221E+00_JPRB, 0.28683E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.27505E+00_JPRB, 0.27857E+00_JPRB, 0.28092E+00_JPRB, 0.28247E+00_JPRB, 0.28398E+00_JPRB, 0.28327E+00_JPRB, 0.28171E+00_JPRB ,&
&  0.27753E+00_JPRB, 0.27608E+00_JPRB, 0.27331E+00_JPRB, 0.27035E+00_JPRB, 0.26689E+00_JPRB, 0.26298E+00_JPRB, 0.25895E+00_JPRB ,&
&  0.25444E+00_JPRB, 0.24954E+00_JPRB, 0.24371E+00_JPRB, 0.23734E+00_JPRB, 0.23152E+00_JPRB, 0.22607E+00_JPRB, 0.22289E+00_JPRB ,&
&  0.22143E+00_JPRB, 0.22222E+00_JPRB, 0.22509E+00_JPRB, 0.22864E+00_JPRB, 0.23295E+00_JPRB, 0.23783E+00_JPRB, 0.24389E+00_JPRB ,&
&  0.24909E+00_JPRB, 0.25529E+00_JPRB, 0.25906E+00_JPRB, 0.26066E+00_JPRB, 0.25926E+00_JPRB, 0.25916E+00_JPRB, 0.25820E+00_JPRB ,&
&  0.25690E+00_JPRB, 0.25485E+00_JPRB, 0.25398E+00_JPRB, 0.25402E+00_JPRB, 0.25311E+00_JPRB, 0.25115E+00_JPRB, 0.24844E+00_JPRB ,&
&  0.24592E+00_JPRB, 0.24376E+00_JPRB, 0.24108E+00_JPRB, 0.23879E+00_JPRB, 0.23707E+00_JPRB, 0.23494E+00_JPRB, 0.23211E+00_JPRB ,&
&  0.22957E+00_JPRB, 0.22703E+00_JPRB, 0.22404E+00_JPRB, 0.22150E+00_JPRB, 0.21900E+00_JPRB, 0.21718E+00_JPRB, 0.21555E+00_JPRB ,&
&  0.21485E+00_JPRB, 0.21411E+00_JPRB, 0.21389E+00_JPRB, 0.21378E+00_JPRB, 0.21421E+00_JPRB, 0.21395E+00_JPRB, 0.21354E+00_JPRB ,&
&  0.21304E+00_JPRB, 0.21290E+00_JPRB, 0.21237E+00_JPRB, 0.21217E+00_JPRB, 0.21198E+00_JPRB, 0.21131E+00_JPRB, 0.21098E+00_JPRB ,&
&  0.21105E+00_JPRB, 0.21097E+00_JPRB, 0.21044E+00_JPRB, 0.21039E+00_JPRB, 0.21006E+00_JPRB, 0.20957E+00_JPRB, 0.20987E+00_JPRB ,&
&  0.20973E+00_JPRB, 0.20998E+00_JPRB, 0.21035E+00_JPRB, 0.21092E+00_JPRB, 0.21209E+00_JPRB, 0.21354E+00_JPRB, 0.21547E+00_JPRB ,&
&  0.21838E+00_JPRB, 0.22101E+00_JPRB, 0.22365E+00_JPRB, 0.22674E+00_JPRB, 0.23250E+00_JPRB, 0.23537E+00_JPRB, 0.24042E+00_JPRB ,&
&  0.24515E+00_JPRB, 0.24413E+00_JPRB, 0.26714E+00_JPRB, 0.26826E+00_JPRB, 0.27040E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.23155E+00_JPRB, 0.23666E+00_JPRB, 0.24043E+00_JPRB, 0.24386E+00_JPRB, 0.24158E+00_JPRB, 0.26145E+00_JPRB, 0.25953E+00_JPRB ,&
&  0.25949E+00_JPRB, 0.25321E+00_JPRB, 0.25438E+00_JPRB, 0.25311E+00_JPRB, 0.25074E+00_JPRB, 0.24770E+00_JPRB, 0.24448E+00_JPRB ,&
&  0.24115E+00_JPRB, 0.23720E+00_JPRB, 0.23357E+00_JPRB, 0.23008E+00_JPRB, 0.22720E+00_JPRB, 0.22468E+00_JPRB, 0.22330E+00_JPRB ,&
&  0.22335E+00_JPRB, 0.22485E+00_JPRB, 0.22743E+00_JPRB, 0.22886E+00_JPRB, 0.23039E+00_JPRB, 0.23200E+00_JPRB, 0.23372E+00_JPRB ,&
&  0.23516E+00_JPRB, 0.23707E+00_JPRB, 0.23808E+00_JPRB, 0.23772E+00_JPRB, 0.23666E+00_JPRB, 0.23621E+00_JPRB, 0.23606E+00_JPRB ,&
&  0.23653E+00_JPRB, 0.23674E+00_JPRB, 0.23680E+00_JPRB, 0.23701E+00_JPRB, 0.23720E+00_JPRB, 0.23666E+00_JPRB, 0.23544E+00_JPRB ,&
&  0.23441E+00_JPRB, 0.23356E+00_JPRB, 0.23193E+00_JPRB, 0.23038E+00_JPRB, 0.22892E+00_JPRB, 0.22740E+00_JPRB, 0.22582E+00_JPRB ,&
&  0.22468E+00_JPRB, 0.22354E+00_JPRB, 0.22211E+00_JPRB, 0.22045E+00_JPRB, 0.21890E+00_JPRB, 0.21752E+00_JPRB, 0.21644E+00_JPRB ,&
&  0.21619E+00_JPRB, 0.21561E+00_JPRB, 0.21510E+00_JPRB, 0.21460E+00_JPRB, 0.21446E+00_JPRB, 0.21403E+00_JPRB, 0.21372E+00_JPRB ,&
&  0.21349E+00_JPRB, 0.21319E+00_JPRB, 0.21287E+00_JPRB, 0.21255E+00_JPRB, 0.21233E+00_JPRB, 0.21202E+00_JPRB, 0.21174E+00_JPRB ,&
&  0.21155E+00_JPRB, 0.21141E+00_JPRB, 0.21123E+00_JPRB, 0.21087E+00_JPRB, 0.21049E+00_JPRB, 0.21004E+00_JPRB, 0.21004E+00_JPRB ,&
&  0.21020E+00_JPRB, 0.21049E+00_JPRB, 0.21100E+00_JPRB, 0.21123E+00_JPRB, 0.21210E+00_JPRB, 0.21294E+00_JPRB, 0.21393E+00_JPRB ,&
&  0.21499E+00_JPRB, 0.21658E+00_JPRB, 0.21833E+00_JPRB, 0.22021E+00_JPRB, 0.22316E+00_JPRB, 0.22419E+00_JPRB, 0.22504E+00_JPRB ,&
&  0.22607E+00_JPRB, 0.22956E+00_JPRB, 0.23055E+00_JPRB, 0.23046E+00_JPRB, 0.23119E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.21974E+00_JPRB, 0.22002E+00_JPRB, 0.22006E+00_JPRB, 0.21998E+00_JPRB, 0.21825E+00_JPRB, 0.21757E+00_JPRB, 0.21695E+00_JPRB ,&
&  0.21649E+00_JPRB, 0.21626E+00_JPRB, 0.21667E+00_JPRB, 0.21720E+00_JPRB, 0.21773E+00_JPRB, 0.21827E+00_JPRB, 0.21872E+00_JPRB ,&
&  0.21893E+00_JPRB, 0.21916E+00_JPRB, 0.21811E+00_JPRB, 0.21762E+00_JPRB, 0.21748E+00_JPRB, 0.21756E+00_JPRB, 0.21756E+00_JPRB ,&
&  0.21768E+00_JPRB, 0.21778E+00_JPRB, 0.21777E+00_JPRB, 0.21810E+00_JPRB, 0.21869E+00_JPRB, 0.21918E+00_JPRB, 0.21968E+00_JPRB ,&
&  0.22018E+00_JPRB, 0.22089E+00_JPRB, 0.22153E+00_JPRB, 0.22213E+00_JPRB, 0.22290E+00_JPRB, 0.22364E+00_JPRB, 0.22405E+00_JPRB ,&
&  0.22436E+00_JPRB, 0.22441E+00_JPRB, 0.22453E+00_JPRB, 0.22462E+00_JPRB, 0.22483E+00_JPRB, 0.22509E+00_JPRB, 0.22512E+00_JPRB ,&
&  0.22503E+00_JPRB, 0.22485E+00_JPRB, 0.22459E+00_JPRB, 0.22404E+00_JPRB, 0.22346E+00_JPRB, 0.22290E+00_JPRB, 0.22235E+00_JPRB ,&
&  0.22188E+00_JPRB, 0.22134E+00_JPRB, 0.22074E+00_JPRB, 0.22012E+00_JPRB, 0.21946E+00_JPRB, 0.21881E+00_JPRB, 0.21816E+00_JPRB ,&
&  0.21751E+00_JPRB, 0.21708E+00_JPRB, 0.21661E+00_JPRB, 0.21621E+00_JPRB, 0.21589E+00_JPRB, 0.21558E+00_JPRB, 0.21527E+00_JPRB ,&
&  0.21497E+00_JPRB, 0.21460E+00_JPRB, 0.21426E+00_JPRB, 0.21389E+00_JPRB, 0.21356E+00_JPRB, 0.21319E+00_JPRB, 0.21290E+00_JPRB ,&
&  0.21261E+00_JPRB, 0.21236E+00_JPRB, 0.21215E+00_JPRB, 0.21206E+00_JPRB, 0.21201E+00_JPRB, 0.21205E+00_JPRB, 0.21210E+00_JPRB ,&
&  0.21217E+00_JPRB, 0.21222E+00_JPRB, 0.21226E+00_JPRB, 0.21216E+00_JPRB, 0.21237E+00_JPRB, 0.21259E+00_JPRB, 0.21291E+00_JPRB ,&
&  0.21322E+00_JPRB, 0.21370E+00_JPRB, 0.21418E+00_JPRB, 0.21472E+00_JPRB, 0.21555E+00_JPRB, 0.21585E+00_JPRB, 0.21611E+00_JPRB ,&
&  0.21638E+00_JPRB, 0.21695E+00_JPRB, 0.21744E+00_JPRB, 0.21771E+00_JPRB, 0.21829E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB, 0.21576E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB, 0.52491E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.50721E+00_JPRB, 0.50447E+00_JPRB, 0.50210E+00_JPRB, 0.49960E+00_JPRB, 0.49698E+00_JPRB, 0.49382E+00_JPRB, 0.49046E+00_JPRB ,&
&  0.48739E+00_JPRB, 0.48465E+00_JPRB, 0.48220E+00_JPRB, 0.47997E+00_JPRB, 0.47815E+00_JPRB, 0.47668E+00_JPRB, 0.47593E+00_JPRB ,&
&  0.47550E+00_JPRB, 0.47505E+00_JPRB, 0.47482E+00_JPRB, 0.47469E+00_JPRB, 0.47436E+00_JPRB, 0.47390E+00_JPRB, 0.47332E+00_JPRB ,&
&  0.47300E+00_JPRB, 0.47278E+00_JPRB, 0.47262E+00_JPRB, 0.47285E+00_JPRB, 0.47330E+00_JPRB, 0.47372E+00_JPRB, 0.47413E+00_JPRB ,&
&  0.47495E+00_JPRB, 0.47596E+00_JPRB, 0.47735E+00_JPRB, 0.47907E+00_JPRB, 0.48256E+00_JPRB, 0.48627E+00_JPRB, 0.49004E+00_JPRB ,&
&  0.49354E+00_JPRB, 0.49703E+00_JPRB, 0.49974E+00_JPRB, 0.50191E+00_JPRB, 0.50385E+00_JPRB, 0.50636E+00_JPRB, 0.50919E+00_JPRB ,&
&  0.51225E+00_JPRB, 0.51573E+00_JPRB, 0.52000E+00_JPRB, 0.52419E+00_JPRB, 0.52803E+00_JPRB, 0.53201E+00_JPRB, 0.53644E+00_JPRB ,&
&  0.54079E+00_JPRB, 0.54524E+00_JPRB, 0.54978E+00_JPRB, 0.55455E+00_JPRB, 0.55911E+00_JPRB, 0.56465E+00_JPRB, 0.57131E+00_JPRB ,&
&  0.57925E+00_JPRB, 0.58531E+00_JPRB, 0.59250E+00_JPRB, 0.60031E+00_JPRB, 0.61262E+00_JPRB, 0.61886E+00_JPRB, 0.62487E+00_JPRB ,&
&  0.63004E+00_JPRB, 0.63854E+00_JPRB, 0.64385E+00_JPRB, 0.64893E+00_JPRB, 0.65351E+00_JPRB, 0.65763E+00_JPRB, 0.66030E+00_JPRB ,&
&  0.66259E+00_JPRB, 0.66455E+00_JPRB, 0.66495E+00_JPRB, 0.66479E+00_JPRB, 0.66372E+00_JPRB, 0.66224E+00_JPRB, 0.65796E+00_JPRB ,&
&  0.65415E+00_JPRB, 0.65045E+00_JPRB, 0.64700E+00_JPRB, 0.64179E+00_JPRB, 0.63642E+00_JPRB, 0.63083E+00_JPRB, 0.62509E+00_JPRB ,&
&  0.61756E+00_JPRB, 0.60980E+00_JPRB, 0.60135E+00_JPRB, 0.59233E+00_JPRB, 0.58370E+00_JPRB, 0.57466E+00_JPRB, 0.56474E+00_JPRB ,&
&  0.55355E+00_JPRB, 0.54309E+00_JPRB, 0.53119E+00_JPRB, 0.51940E+00_JPRB, 0.51080E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.49472E+00_JPRB, 0.48282E+00_JPRB, 0.47788E+00_JPRB, 0.47549E+00_JPRB, 0.47428E+00_JPRB, 0.47256E+00_JPRB, 0.47041E+00_JPRB ,&
&  0.46881E+00_JPRB, 0.46699E+00_JPRB, 0.46537E+00_JPRB, 0.46492E+00_JPRB, 0.46372E+00_JPRB, 0.46301E+00_JPRB, 0.46219E+00_JPRB ,&
&  0.46148E+00_JPRB, 0.46050E+00_JPRB, 0.45992E+00_JPRB, 0.45873E+00_JPRB, 0.45710E+00_JPRB, 0.45483E+00_JPRB, 0.45312E+00_JPRB ,&
&  0.45168E+00_JPRB, 0.45073E+00_JPRB, 0.44939E+00_JPRB, 0.44850E+00_JPRB, 0.44880E+00_JPRB, 0.44856E+00_JPRB, 0.44777E+00_JPRB ,&
&  0.44939E+00_JPRB, 0.45022E+00_JPRB, 0.44970E+00_JPRB, 0.44808E+00_JPRB, 0.44640E+00_JPRB, 0.44635E+00_JPRB, 0.44866E+00_JPRB ,&
&  0.45218E+00_JPRB, 0.46134E+00_JPRB, 0.46811E+00_JPRB, 0.47453E+00_JPRB, 0.48108E+00_JPRB, 0.48922E+00_JPRB, 0.49884E+00_JPRB ,&
&  0.50839E+00_JPRB, 0.51461E+00_JPRB, 0.52133E+00_JPRB, 0.53017E+00_JPRB, 0.54075E+00_JPRB, 0.55230E+00_JPRB, 0.56569E+00_JPRB ,&
&  0.57627E+00_JPRB, 0.58738E+00_JPRB, 0.60056E+00_JPRB, 0.61593E+00_JPRB, 0.62854E+00_JPRB, 0.64678E+00_JPRB, 0.66973E+00_JPRB ,&
&  0.70020E+00_JPRB, 0.71442E+00_JPRB, 0.73012E+00_JPRB, 0.74531E+00_JPRB, 0.76553E+00_JPRB, 0.77623E+00_JPRB, 0.78616E+00_JPRB ,&
&  0.79301E+00_JPRB, 0.80017E+00_JPRB, 0.79907E+00_JPRB, 0.79621E+00_JPRB, 0.79347E+00_JPRB, 0.78906E+00_JPRB, 0.78479E+00_JPRB ,&
&  0.78073E+00_JPRB, 0.77753E+00_JPRB, 0.76997E+00_JPRB, 0.76371E+00_JPRB, 0.75738E+00_JPRB, 0.75182E+00_JPRB, 0.73951E+00_JPRB ,&
&  0.72860E+00_JPRB, 0.72061E+00_JPRB, 0.71570E+00_JPRB, 0.70747E+00_JPRB, 0.69915E+00_JPRB, 0.69153E+00_JPRB, 0.68437E+00_JPRB ,&
&  0.67693E+00_JPRB, 0.66799E+00_JPRB, 0.65842E+00_JPRB, 0.64805E+00_JPRB, 0.63910E+00_JPRB, 0.62613E+00_JPRB, 0.61107E+00_JPRB ,&
&  0.59291E+00_JPRB, 0.57663E+00_JPRB, 0.55683E+00_JPRB, 0.53323E+00_JPRB, 0.50952E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.51442E+00_JPRB, 0.48601E+00_JPRB, 0.47142E+00_JPRB, 0.46837E+00_JPRB, 0.46760E+00_JPRB, 0.46696E+00_JPRB, 0.46618E+00_JPRB ,&
&  0.46487E+00_JPRB, 0.46464E+00_JPRB, 0.46610E+00_JPRB, 0.46725E+00_JPRB, 0.46731E+00_JPRB, 0.46736E+00_JPRB, 0.46643E+00_JPRB ,&
&  0.46494E+00_JPRB, 0.46190E+00_JPRB, 0.45904E+00_JPRB, 0.45451E+00_JPRB, 0.44982E+00_JPRB, 0.44392E+00_JPRB, 0.43640E+00_JPRB ,&
&  0.42933E+00_JPRB, 0.42284E+00_JPRB, 0.41414E+00_JPRB, 0.41129E+00_JPRB, 0.40953E+00_JPRB, 0.39089E+00_JPRB, 0.38636E+00_JPRB ,&
&  0.39092E+00_JPRB, 0.39212E+00_JPRB, 0.39336E+00_JPRB, 0.39473E+00_JPRB, 0.39571E+00_JPRB, 0.38963E+00_JPRB, 0.38865E+00_JPRB ,&
&  0.37525E+00_JPRB, 0.38521E+00_JPRB, 0.39300E+00_JPRB, 0.40063E+00_JPRB, 0.40907E+00_JPRB, 0.42180E+00_JPRB, 0.44368E+00_JPRB ,&
&  0.48375E+00_JPRB, 0.50232E+00_JPRB, 0.52300E+00_JPRB, 0.54167E+00_JPRB, 0.56754E+00_JPRB, 0.59600E+00_JPRB, 0.62252E+00_JPRB ,&
&  0.64394E+00_JPRB, 0.66705E+00_JPRB, 0.69377E+00_JPRB, 0.73085E+00_JPRB, 0.75877E+00_JPRB, 0.78483E+00_JPRB, 0.80288E+00_JPRB ,&
&  0.82668E+00_JPRB, 0.84445E+00_JPRB, 0.85679E+00_JPRB, 0.86514E+00_JPRB, 0.87342E+00_JPRB, 0.87565E+00_JPRB, 0.87838E+00_JPRB ,&
&  0.87780E+00_JPRB, 0.87658E+00_JPRB, 0.86791E+00_JPRB, 0.85415E+00_JPRB, 0.84232E+00_JPRB, 0.83447E+00_JPRB, 0.82421E+00_JPRB ,&
&  0.81405E+00_JPRB, 0.80341E+00_JPRB, 0.79168E+00_JPRB, 0.77836E+00_JPRB, 0.76475E+00_JPRB, 0.75338E+00_JPRB, 0.74001E+00_JPRB ,&
&  0.72898E+00_JPRB, 0.72042E+00_JPRB, 0.71368E+00_JPRB, 0.70436E+00_JPRB, 0.69478E+00_JPRB, 0.68541E+00_JPRB, 0.67484E+00_JPRB ,&
&  0.66460E+00_JPRB, 0.65586E+00_JPRB, 0.64874E+00_JPRB, 0.64054E+00_JPRB, 0.63510E+00_JPRB, 0.62678E+00_JPRB, 0.61904E+00_JPRB ,&
&  0.60972E+00_JPRB, 0.60140E+00_JPRB, 0.58743E+00_JPRB, 0.56696E+00_JPRB, 0.53908E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.55057E+00_JPRB, 0.51744E+00_JPRB, 0.48472E+00_JPRB, 0.46710E+00_JPRB, 0.46445E+00_JPRB, 0.46338E+00_JPRB, 0.46317E+00_JPRB ,&
&  0.46368E+00_JPRB, 0.46672E+00_JPRB, 0.46971E+00_JPRB, 0.47188E+00_JPRB, 0.47237E+00_JPRB, 0.47114E+00_JPRB, 0.46872E+00_JPRB ,&
&  0.46432E+00_JPRB, 0.45906E+00_JPRB, 0.45382E+00_JPRB, 0.44920E+00_JPRB, 0.44403E+00_JPRB, 0.43844E+00_JPRB, 0.43081E+00_JPRB ,&
&  0.42204E+00_JPRB, 0.41361E+00_JPRB, 0.38434E+00_JPRB, 0.38168E+00_JPRB, 0.37558E+00_JPRB, 0.37432E+00_JPRB, 0.37761E+00_JPRB ,&
&  0.37979E+00_JPRB, 0.37417E+00_JPRB, 0.36714E+00_JPRB, 0.35463E+00_JPRB, 0.34361E+00_JPRB, 0.33197E+00_JPRB, 0.32501E+00_JPRB ,&
&  0.32234E+00_JPRB, 0.32399E+00_JPRB, 0.32991E+00_JPRB, 0.33622E+00_JPRB, 0.34487E+00_JPRB, 0.35431E+00_JPRB, 0.36929E+00_JPRB ,&
&  0.41012E+00_JPRB, 0.46856E+00_JPRB, 0.53706E+00_JPRB, 0.57096E+00_JPRB, 0.61013E+00_JPRB, 0.65301E+00_JPRB, 0.68453E+00_JPRB ,&
&  0.71119E+00_JPRB, 0.73917E+00_JPRB, 0.76672E+00_JPRB, 0.79904E+00_JPRB, 0.82646E+00_JPRB, 0.84552E+00_JPRB, 0.87368E+00_JPRB ,&
&  0.89654E+00_JPRB, 0.91180E+00_JPRB, 0.91404E+00_JPRB, 0.91431E+00_JPRB, 0.91214E+00_JPRB, 0.90140E+00_JPRB, 0.89023E+00_JPRB ,&
&  0.87892E+00_JPRB, 0.86710E+00_JPRB, 0.85073E+00_JPRB, 0.82339E+00_JPRB, 0.80327E+00_JPRB, 0.79140E+00_JPRB, 0.77229E+00_JPRB ,&
&  0.75440E+00_JPRB, 0.73460E+00_JPRB, 0.71781E+00_JPRB, 0.69536E+00_JPRB, 0.68206E+00_JPRB, 0.67911E+00_JPRB, 0.67942E+00_JPRB ,&
&  0.67942E+00_JPRB, 0.67864E+00_JPRB, 0.67679E+00_JPRB, 0.67081E+00_JPRB, 0.65921E+00_JPRB, 0.64634E+00_JPRB, 0.63492E+00_JPRB ,&
&  0.62307E+00_JPRB, 0.61090E+00_JPRB, 0.59934E+00_JPRB, 0.58752E+00_JPRB, 0.58139E+00_JPRB, 0.57733E+00_JPRB, 0.57874E+00_JPRB ,&
&  0.58275E+00_JPRB, 0.58820E+00_JPRB, 0.59028E+00_JPRB, 0.58743E+00_JPRB, 0.57357E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.62119E+00_JPRB, 0.60833E+00_JPRB, 0.58281E+00_JPRB, 0.54790E+00_JPRB, 0.52212E+00_JPRB, 0.50550E+00_JPRB, 0.49783E+00_JPRB ,&
&  0.49355E+00_JPRB, 0.49136E+00_JPRB, 0.48988E+00_JPRB, 0.48744E+00_JPRB, 0.48385E+00_JPRB, 0.47826E+00_JPRB, 0.47330E+00_JPRB ,&
&  0.46393E+00_JPRB, 0.45582E+00_JPRB, 0.44655E+00_JPRB, 0.44164E+00_JPRB, 0.43764E+00_JPRB, 0.43448E+00_JPRB, 0.43213E+00_JPRB ,&
&  0.42871E+00_JPRB, 0.42428E+00_JPRB, 0.40845E+00_JPRB, 0.40174E+00_JPRB, 0.39555E+00_JPRB, 0.39672E+00_JPRB, 0.39657E+00_JPRB ,&
&  0.41255E+00_JPRB, 0.37127E+00_JPRB, 0.36210E+00_JPRB, 0.34810E+00_JPRB, 0.33721E+00_JPRB, 0.32475E+00_JPRB, 0.31368E+00_JPRB ,&
&  0.30683E+00_JPRB, 0.30123E+00_JPRB, 0.30014E+00_JPRB, 0.30658E+00_JPRB, 0.31349E+00_JPRB, 0.32582E+00_JPRB, 0.33978E+00_JPRB ,&
&  0.37307E+00_JPRB, 0.48711E+00_JPRB, 0.54010E+00_JPRB, 0.58147E+00_JPRB, 0.62163E+00_JPRB, 0.66362E+00_JPRB, 0.68816E+00_JPRB ,&
&  0.71078E+00_JPRB, 0.74160E+00_JPRB, 0.76097E+00_JPRB, 0.76300E+00_JPRB, 0.81749E+00_JPRB, 0.85709E+00_JPRB, 0.89336E+00_JPRB ,&
&  0.92491E+00_JPRB, 0.94681E+00_JPRB, 0.96301E+00_JPRB, 0.97052E+00_JPRB, 0.98123E+00_JPRB, 0.97028E+00_JPRB, 0.95105E+00_JPRB ,&
&  0.93177E+00_JPRB, 0.91684E+00_JPRB, 0.90096E+00_JPRB, 0.86272E+00_JPRB, 0.83233E+00_JPRB, 0.81282E+00_JPRB, 0.77925E+00_JPRB ,&
&  0.74657E+00_JPRB, 0.71579E+00_JPRB, 0.68652E+00_JPRB, 0.65890E+00_JPRB, 0.63764E+00_JPRB, 0.62695E+00_JPRB, 0.62846E+00_JPRB ,&
&  0.63476E+00_JPRB, 0.63978E+00_JPRB, 0.63632E+00_JPRB, 0.62755E+00_JPRB, 0.61539E+00_JPRB, 0.60766E+00_JPRB, 0.59423E+00_JPRB ,&
&  0.57835E+00_JPRB, 0.55800E+00_JPRB, 0.54493E+00_JPRB, 0.53585E+00_JPRB, 0.53379E+00_JPRB, 0.53768E+00_JPRB, 0.54593E+00_JPRB ,&
&  0.55699E+00_JPRB, 0.57178E+00_JPRB, 0.59003E+00_JPRB, 0.60689E+00_JPRB, 0.61841E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.60372E+00_JPRB, 0.61360E+00_JPRB, 0.61669E+00_JPRB, 0.61087E+00_JPRB, 0.59846E+00_JPRB, 0.58164E+00_JPRB, 0.56400E+00_JPRB ,&
&  0.54827E+00_JPRB, 0.53604E+00_JPRB, 0.52502E+00_JPRB, 0.51699E+00_JPRB, 0.50763E+00_JPRB, 0.50186E+00_JPRB, 0.49523E+00_JPRB ,&
&  0.49203E+00_JPRB, 0.49125E+00_JPRB, 0.48595E+00_JPRB, 0.47932E+00_JPRB, 0.46545E+00_JPRB, 0.45275E+00_JPRB, 0.44320E+00_JPRB ,&
&  0.43473E+00_JPRB, 0.42825E+00_JPRB, 0.41285E+00_JPRB, 0.40134E+00_JPRB, 0.39637E+00_JPRB, 0.39571E+00_JPRB, 0.40489E+00_JPRB ,&
&  0.40849E+00_JPRB, 0.41725E+00_JPRB, 0.42423E+00_JPRB, 0.41870E+00_JPRB, 0.41255E+00_JPRB, 0.32656E+00_JPRB, 0.29873E+00_JPRB ,&
&  0.28989E+00_JPRB, 0.28376E+00_JPRB, 0.28229E+00_JPRB, 0.28747E+00_JPRB, 0.29592E+00_JPRB, 0.31679E+00_JPRB, 0.33357E+00_JPRB ,&
&  0.35231E+00_JPRB, 0.47327E+00_JPRB, 0.52232E+00_JPRB, 0.56835E+00_JPRB, 0.60045E+00_JPRB, 0.63034E+00_JPRB, 0.65220E+00_JPRB ,&
&  0.67040E+00_JPRB, 0.69629E+00_JPRB, 0.70917E+00_JPRB, 0.73596E+00_JPRB, 0.78903E+00_JPRB, 0.88517E+00_JPRB, 0.96002E+00_JPRB ,&
&  0.10444E+01_JPRB, 0.10593E+01_JPRB, 0.10641E+01_JPRB, 0.10365E+01_JPRB, 0.10340E+01_JPRB, 0.10263E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10007E+01_JPRB, 0.97002E+00_JPRB, 0.96017E+00_JPRB, 0.93612E+00_JPRB, 0.91588E+00_JPRB, 0.89586E+00_JPRB, 0.84122E+00_JPRB ,&
&  0.77872E+00_JPRB, 0.73596E+00_JPRB, 0.69757E+00_JPRB, 0.65384E+00_JPRB, 0.62176E+00_JPRB, 0.59953E+00_JPRB, 0.59516E+00_JPRB ,&
&  0.59703E+00_JPRB, 0.60766E+00_JPRB, 0.60201E+00_JPRB, 0.59054E+00_JPRB, 0.58174E+00_JPRB, 0.57584E+00_JPRB, 0.55888E+00_JPRB ,&
&  0.54403E+00_JPRB, 0.52932E+00_JPRB, 0.52272E+00_JPRB, 0.51572E+00_JPRB, 0.51209E+00_JPRB, 0.51297E+00_JPRB, 0.51771E+00_JPRB ,&
&  0.52754E+00_JPRB, 0.54060E+00_JPRB, 0.55831E+00_JPRB, 0.57600E+00_JPRB, 0.59463E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.58596E+00_JPRB, 0.60266E+00_JPRB, 0.61176E+00_JPRB, 0.61017E+00_JPRB, 0.59667E+00_JPRB, 0.57831E+00_JPRB, 0.56228E+00_JPRB ,&
&  0.54772E+00_JPRB, 0.53391E+00_JPRB, 0.52140E+00_JPRB, 0.51214E+00_JPRB, 0.50427E+00_JPRB, 0.49961E+00_JPRB, 0.49651E+00_JPRB ,&
&  0.49558E+00_JPRB, 0.48612E+00_JPRB, 0.48286E+00_JPRB, 0.48397E+00_JPRB, 0.48724E+00_JPRB, 0.48634E+00_JPRB, 0.48144E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.43942E+00_JPRB, 0.41668E+00_JPRB, 0.39772E+00_JPRB, 0.38408E+00_JPRB, 0.38106E+00_JPRB, 0.38878E+00_JPRB ,&
&  0.39903E+00_JPRB, 0.39922E+00_JPRB, 0.40235E+00_JPRB, 0.42131E+00_JPRB, 0.43601E+00_JPRB, 0.37819E+00_JPRB, 0.29965E+00_JPRB ,&
&  0.28655E+00_JPRB, 0.27379E+00_JPRB, 0.27279E+00_JPRB, 0.27822E+00_JPRB, 0.28841E+00_JPRB, 0.30398E+00_JPRB, 0.33156E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.46155E+00_JPRB, 0.50238E+00_JPRB, 0.53852E+00_JPRB, 0.57480E+00_JPRB, 0.60789E+00_JPRB, 0.62672E+00_JPRB ,&
&  0.63692E+00_JPRB, 0.65435E+00_JPRB, 0.69090E+00_JPRB, 0.73282E+00_JPRB, 0.80338E+00_JPRB, 0.94408E+00_JPRB, 0.10521E+01_JPRB ,&
&  0.12010E+01_JPRB, 0.12496E+01_JPRB, 0.11930E+01_JPRB, 0.10747E+01_JPRB, 0.10521E+01_JPRB, 0.10666E+01_JPRB, 0.10412E+01_JPRB ,&
&  0.10539E+01_JPRB, 0.10796E+01_JPRB, 0.10594E+01_JPRB, 0.10106E+01_JPRB, 0.97834E+00_JPRB, 0.95810E+00_JPRB, 0.88508E+00_JPRB ,&
&  0.80509E+00_JPRB, 0.77221E+00_JPRB, 0.77288E+00_JPRB, 0.70166E+00_JPRB, 0.65299E+00_JPRB, 0.60821E+00_JPRB, 0.58592E+00_JPRB ,&
&  0.57375E+00_JPRB, 0.56302E+00_JPRB, 0.54739E+00_JPRB, 0.53436E+00_JPRB, 0.52908E+00_JPRB, 0.52588E+00_JPRB, 0.51993E+00_JPRB ,&
&  0.51400E+00_JPRB, 0.50842E+00_JPRB, 0.50299E+00_JPRB, 0.49948E+00_JPRB, 0.49668E+00_JPRB, 0.49671E+00_JPRB, 0.50018E+00_JPRB ,&
&  0.50839E+00_JPRB, 0.52117E+00_JPRB, 0.53746E+00_JPRB, 0.55191E+00_JPRB, 0.56978E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.58663E+00_JPRB, 0.59961E+00_JPRB, 0.59350E+00_JPRB, 0.56984E+00_JPRB, 0.53665E+00_JPRB, 0.49672E+00_JPRB, 0.47147E+00_JPRB ,&
&  0.45863E+00_JPRB, 0.45364E+00_JPRB, 0.45060E+00_JPRB, 0.44520E+00_JPRB, 0.44376E+00_JPRB, 0.44388E+00_JPRB, 0.44555E+00_JPRB ,&
&  0.44948E+00_JPRB, 0.45486E+00_JPRB, 0.45939E+00_JPRB, 0.45900E+00_JPRB, 0.46367E+00_JPRB, 0.46757E+00_JPRB, 0.47119E+00_JPRB ,&
&  0.46814E+00_JPRB, 0.45064E+00_JPRB, 0.42715E+00_JPRB, 0.41184E+00_JPRB, 0.39615E+00_JPRB, 0.39783E+00_JPRB, 0.39034E+00_JPRB ,&
&  0.36607E+00_JPRB, 0.36302E+00_JPRB, 0.39342E+00_JPRB, 0.42262E+00_JPRB, 0.42094E+00_JPRB, 0.44493E+00_JPRB, 0.33219E+00_JPRB ,&
&  0.29563E+00_JPRB, 0.27809E+00_JPRB, 0.27469E+00_JPRB, 0.28246E+00_JPRB, 0.30289E+00_JPRB, 0.33046E+00_JPRB, 0.36073E+00_JPRB ,&
&  0.41657E+00_JPRB, 0.45952E+00_JPRB, 0.48381E+00_JPRB, 0.51441E+00_JPRB, 0.55091E+00_JPRB, 0.58832E+00_JPRB, 0.61718E+00_JPRB ,&
&  0.63600E+00_JPRB, 0.66691E+00_JPRB, 0.72137E+00_JPRB, 0.80739E+00_JPRB, 0.10011E+01_JPRB, 0.96936E+00_JPRB, 0.99366E+00_JPRB ,&
&  0.12641E+01_JPRB, 0.14236E+01_JPRB, 0.12800E+01_JPRB, 0.11340E+01_JPRB, 0.10639E+01_JPRB, 0.10561E+01_JPRB, 0.10555E+01_JPRB ,&
&  0.11170E+01_JPRB, 0.11764E+01_JPRB, 0.11692E+01_JPRB, 0.10672E+01_JPRB, 0.10147E+01_JPRB, 0.98404E+00_JPRB, 0.89238E+00_JPRB ,&
&  0.79256E+00_JPRB, 0.80699E+00_JPRB, 0.85269E+00_JPRB, 0.72296E+00_JPRB, 0.63690E+00_JPRB, 0.58414E+00_JPRB, 0.56527E+00_JPRB ,&
&  0.55587E+00_JPRB, 0.54651E+00_JPRB, 0.53396E+00_JPRB, 0.52064E+00_JPRB, 0.50282E+00_JPRB, 0.48836E+00_JPRB, 0.47475E+00_JPRB ,&
&  0.46695E+00_JPRB, 0.46163E+00_JPRB, 0.46229E+00_JPRB, 0.46581E+00_JPRB, 0.46859E+00_JPRB, 0.47155E+00_JPRB, 0.48125E+00_JPRB ,&
&  0.49639E+00_JPRB, 0.50589E+00_JPRB, 0.52118E+00_JPRB, 0.53861E+00_JPRB, 0.56189E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.52587E+00_JPRB, 0.52971E+00_JPRB, 0.51826E+00_JPRB, 0.49042E+00_JPRB, 0.46355E+00_JPRB, 0.42706E+00_JPRB, 0.40108E+00_JPRB ,&
&  0.38700E+00_JPRB, 0.38115E+00_JPRB, 0.38087E+00_JPRB, 0.38254E+00_JPRB, 0.38638E+00_JPRB, 0.39248E+00_JPRB, 0.39364E+00_JPRB ,&
&  0.39540E+00_JPRB, 0.39517E+00_JPRB, 0.39788E+00_JPRB, 0.40420E+00_JPRB, 0.41186E+00_JPRB, 0.42066E+00_JPRB, 0.42995E+00_JPRB ,&
&  0.43916E+00_JPRB, 0.43801E+00_JPRB, 0.42230E+00_JPRB, 0.41032E+00_JPRB, 0.41619E+00_JPRB, 0.43711E+00_JPRB, 0.42702E+00_JPRB ,&
&  0.38894E+00_JPRB, 0.36417E+00_JPRB, 0.36618E+00_JPRB, 0.42706E+00_JPRB, 0.46348E+00_JPRB, 0.48354E+00_JPRB, 0.37202E+00_JPRB ,&
&  0.31391E+00_JPRB, 0.29148E+00_JPRB, 0.29173E+00_JPRB, 0.31410E+00_JPRB, 0.35818E+00_JPRB, 0.42496E+00_JPRB, 0.47733E+00_JPRB ,&
&  0.52191E+00_JPRB, 0.56001E+00_JPRB, 0.57143E+00_JPRB, 0.59440E+00_JPRB, 0.63168E+00_JPRB, 0.66565E+00_JPRB, 0.70265E+00_JPRB ,&
&  0.72527E+00_JPRB, 0.76350E+00_JPRB, 0.88337E+00_JPRB, 0.75999E+00_JPRB, 0.80855E+00_JPRB, 0.86692E+00_JPRB, 0.97425E+00_JPRB ,&
&  0.11179E+01_JPRB, 0.11580E+01_JPRB, 0.11519E+01_JPRB, 0.12109E+01_JPRB, 0.11013E+01_JPRB, 0.10080E+01_JPRB, 0.98269E+00_JPRB ,&
&  0.11057E+01_JPRB, 0.12207E+01_JPRB, 0.11948E+01_JPRB, 0.10972E+01_JPRB, 0.10632E+01_JPRB, 0.10423E+01_JPRB, 0.93431E+00_JPRB ,&
&  0.80082E+00_JPRB, 0.72337E+00_JPRB, 0.65811E+00_JPRB, 0.57752E+00_JPRB, 0.54443E+00_JPRB, 0.52571E+00_JPRB, 0.51640E+00_JPRB ,&
&  0.51216E+00_JPRB, 0.50483E+00_JPRB, 0.49946E+00_JPRB, 0.47006E+00_JPRB, 0.43850E+00_JPRB, 0.40918E+00_JPRB, 0.38973E+00_JPRB ,&
&  0.37907E+00_JPRB, 0.37849E+00_JPRB, 0.38464E+00_JPRB, 0.40304E+00_JPRB, 0.43500E+00_JPRB, 0.46456E+00_JPRB, 0.48492E+00_JPRB ,&
&  0.48812E+00_JPRB, 0.47587E+00_JPRB, 0.47444E+00_JPRB, 0.48069E+00_JPRB, 0.50832E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.51350E+00_JPRB, 0.48316E+00_JPRB, 0.46360E+00_JPRB, 0.46894E+00_JPRB, 0.43292E+00_JPRB, 0.39903E+00_JPRB, 0.38092E+00_JPRB ,&
&  0.38233E+00_JPRB, 0.37960E+00_JPRB, 0.38234E+00_JPRB, 0.36496E+00_JPRB, 0.34996E+00_JPRB, 0.34091E+00_JPRB, 0.33732E+00_JPRB ,&
&  0.34655E+00_JPRB, 0.35775E+00_JPRB, 0.36577E+00_JPRB, 0.36387E+00_JPRB, 0.36202E+00_JPRB, 0.36603E+00_JPRB, 0.37730E+00_JPRB ,&
&  0.39396E+00_JPRB, 0.40842E+00_JPRB, 0.41006E+00_JPRB, 0.41052E+00_JPRB, 0.43557E+00_JPRB, 0.46539E+00_JPRB, 0.44242E+00_JPRB ,&
&  0.41315E+00_JPRB, 0.39101E+00_JPRB, 0.38743E+00_JPRB, 0.43202E+00_JPRB, 0.49813E+00_JPRB, 0.53128E+00_JPRB, 0.41510E+00_JPRB ,&
&  0.33728E+00_JPRB, 0.32718E+00_JPRB, 0.36425E+00_JPRB, 0.48596E+00_JPRB, 0.52122E+00_JPRB, 0.56722E+00_JPRB, 0.63764E+00_JPRB ,&
&  0.58852E+00_JPRB, 0.59091E+00_JPRB, 0.61545E+00_JPRB, 0.75793E+00_JPRB, 0.79450E+00_JPRB, 0.80951E+00_JPRB, 0.82125E+00_JPRB ,&
&  0.82914E+00_JPRB, 0.86368E+00_JPRB, 0.10066E+01_JPRB, 0.83815E+00_JPRB, 0.88174E+00_JPRB, 0.10845E+01_JPRB, 0.10748E+01_JPRB ,&
&  0.11044E+01_JPRB, 0.11055E+01_JPRB, 0.12122E+01_JPRB, 0.11418E+01_JPRB, 0.10155E+01_JPRB, 0.95083E+00_JPRB, 0.99696E+00_JPRB ,&
&  0.11722E+01_JPRB, 0.13109E+01_JPRB, 0.12754E+01_JPRB, 0.12256E+01_JPRB, 0.11974E+01_JPRB, 0.11455E+01_JPRB, 0.10262E+01_JPRB ,&
&  0.85529E+00_JPRB, 0.72584E+00_JPRB, 0.62624E+00_JPRB, 0.53943E+00_JPRB, 0.49794E+00_JPRB, 0.48222E+00_JPRB, 0.46778E+00_JPRB ,&
&  0.46599E+00_JPRB, 0.46322E+00_JPRB, 0.45747E+00_JPRB, 0.42350E+00_JPRB, 0.39207E+00_JPRB, 0.37292E+00_JPRB, 0.35994E+00_JPRB ,&
&  0.35408E+00_JPRB, 0.34152E+00_JPRB, 0.33268E+00_JPRB, 0.34489E+00_JPRB, 0.38281E+00_JPRB, 0.43032E+00_JPRB, 0.45140E+00_JPRB ,&
&  0.44185E+00_JPRB, 0.43829E+00_JPRB, 0.46433E+00_JPRB, 0.50341E+00_JPRB, 0.52166E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.52958E+00_JPRB, 0.51957E+00_JPRB, 0.51432E+00_JPRB, 0.50454E+00_JPRB, 0.48960E+00_JPRB, 0.44523E+00_JPRB, 0.41281E+00_JPRB ,&
&  0.42151E+00_JPRB, 0.39834E+00_JPRB, 0.39303E+00_JPRB, 0.36829E+00_JPRB, 0.35280E+00_JPRB, 0.33479E+00_JPRB, 0.32222E+00_JPRB ,&
&  0.31342E+00_JPRB, 0.32134E+00_JPRB, 0.34051E+00_JPRB, 0.34536E+00_JPRB, 0.34508E+00_JPRB, 0.34469E+00_JPRB, 0.35360E+00_JPRB ,&
&  0.36761E+00_JPRB, 0.38831E+00_JPRB, 0.40321E+00_JPRB, 0.41623E+00_JPRB, 0.43992E+00_JPRB, 0.47065E+00_JPRB, 0.47316E+00_JPRB ,&
&  0.47180E+00_JPRB, 0.45280E+00_JPRB, 0.43287E+00_JPRB, 0.46658E+00_JPRB, 0.53203E+00_JPRB, 0.59409E+00_JPRB, 0.57489E+00_JPRB ,&
&  0.40599E+00_JPRB, 0.40856E+00_JPRB, 0.57814E+00_JPRB, 0.62598E+00_JPRB, 0.67982E+00_JPRB, 0.71550E+00_JPRB, 0.76198E+00_JPRB ,&
&  0.80393E+00_JPRB, 0.84654E+00_JPRB, 0.87744E+00_JPRB, 0.90281E+00_JPRB, 0.92106E+00_JPRB, 0.94230E+00_JPRB, 0.93682E+00_JPRB ,&
&  0.10042E+01_JPRB, 0.80648E+00_JPRB, 0.76242E+00_JPRB, 0.81531E+00_JPRB, 0.10905E+01_JPRB, 0.11386E+01_JPRB, 0.10938E+01_JPRB ,&
&  0.10901E+01_JPRB, 0.10505E+01_JPRB, 0.10712E+01_JPRB, 0.11226E+01_JPRB, 0.11026E+01_JPRB, 0.11325E+01_JPRB, 0.12003E+01_JPRB ,&
&  0.13306E+01_JPRB, 0.14677E+01_JPRB, 0.14699E+01_JPRB, 0.14420E+01_JPRB, 0.13772E+01_JPRB, 0.12633E+01_JPRB, 0.10880E+01_JPRB ,&
&  0.87431E+00_JPRB, 0.72367E+00_JPRB, 0.59943E+00_JPRB, 0.51255E+00_JPRB, 0.46342E+00_JPRB, 0.44266E+00_JPRB, 0.42805E+00_JPRB ,&
&  0.42902E+00_JPRB, 0.42793E+00_JPRB, 0.41876E+00_JPRB, 0.39143E+00_JPRB, 0.37392E+00_JPRB, 0.38046E+00_JPRB, 0.38750E+00_JPRB ,&
&  0.38713E+00_JPRB, 0.36915E+00_JPRB, 0.34725E+00_JPRB, 0.35869E+00_JPRB, 0.39374E+00_JPRB, 0.43055E+00_JPRB, 0.45112E+00_JPRB ,&
&  0.45109E+00_JPRB, 0.45564E+00_JPRB, 0.46938E+00_JPRB, 0.49475E+00_JPRB, 0.51019E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.55675E+00_JPRB, 0.55959E+00_JPRB, 0.54591E+00_JPRB, 0.53286E+00_JPRB, 0.51643E+00_JPRB, 0.46130E+00_JPRB, 0.45021E+00_JPRB ,&
&  0.51413E+00_JPRB, 0.46000E+00_JPRB, 0.51534E+00_JPRB, 0.41298E+00_JPRB, 0.39105E+00_JPRB, 0.37212E+00_JPRB, 0.35210E+00_JPRB ,&
&  0.32977E+00_JPRB, 0.32469E+00_JPRB, 0.33210E+00_JPRB, 0.33820E+00_JPRB, 0.33897E+00_JPRB, 0.33594E+00_JPRB, 0.33866E+00_JPRB ,&
&  0.34688E+00_JPRB, 0.36875E+00_JPRB, 0.39917E+00_JPRB, 0.41605E+00_JPRB, 0.43610E+00_JPRB, 0.46925E+00_JPRB, 0.49161E+00_JPRB ,&
&  0.47614E+00_JPRB, 0.47742E+00_JPRB, 0.48814E+00_JPRB, 0.51457E+00_JPRB, 0.55882E+00_JPRB, 0.62208E+00_JPRB, 0.66279E+00_JPRB ,&
&  0.68140E+00_JPRB, 0.56213E+00_JPRB, 0.67064E+00_JPRB, 0.69350E+00_JPRB, 0.75082E+00_JPRB, 0.78300E+00_JPRB, 0.80593E+00_JPRB ,&
&  0.83881E+00_JPRB, 0.87674E+00_JPRB, 0.91958E+00_JPRB, 0.95875E+00_JPRB, 0.99878E+00_JPRB, 0.10364E+01_JPRB, 0.10370E+01_JPRB ,&
&  0.11216E+01_JPRB, 0.78501E+00_JPRB, 0.84509E+00_JPRB, 0.88018E+00_JPRB, 0.11947E+01_JPRB, 0.13239E+01_JPRB, 0.12440E+01_JPRB ,&
&  0.13365E+01_JPRB, 0.12396E+01_JPRB, 0.12832E+01_JPRB, 0.13244E+01_JPRB, 0.13865E+01_JPRB, 0.14893E+01_JPRB, 0.15478E+01_JPRB ,&
&  0.15611E+01_JPRB, 0.16124E+01_JPRB, 0.16843E+01_JPRB, 0.15774E+01_JPRB, 0.14137E+01_JPRB, 0.12183E+01_JPRB, 0.99674E+00_JPRB ,&
&  0.84395E+00_JPRB, 0.71170E+00_JPRB, 0.59932E+00_JPRB, 0.52381E+00_JPRB, 0.46537E+00_JPRB, 0.42802E+00_JPRB, 0.40797E+00_JPRB ,&
&  0.39825E+00_JPRB, 0.38540E+00_JPRB, 0.37580E+00_JPRB, 0.35971E+00_JPRB, 0.35746E+00_JPRB, 0.36648E+00_JPRB, 0.39551E+00_JPRB ,&
&  0.40719E+00_JPRB, 0.40208E+00_JPRB, 0.44608E+00_JPRB, 0.48949E+00_JPRB, 0.49485E+00_JPRB, 0.51196E+00_JPRB, 0.57666E+00_JPRB ,&
&  0.49918E+00_JPRB, 0.52343E+00_JPRB, 0.57348E+00_JPRB, 0.57498E+00_JPRB, 0.55882E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.60455E+00_JPRB, 0.60423E+00_JPRB, 0.58106E+00_JPRB, 0.55769E+00_JPRB, 0.54401E+00_JPRB, 0.51745E+00_JPRB, 0.48956E+00_JPRB ,&
&  0.53629E+00_JPRB, 0.52592E+00_JPRB, 0.52807E+00_JPRB, 0.53385E+00_JPRB, 0.52935E+00_JPRB, 0.46869E+00_JPRB, 0.41222E+00_JPRB ,&
&  0.37558E+00_JPRB, 0.35664E+00_JPRB, 0.36153E+00_JPRB, 0.36088E+00_JPRB, 0.35818E+00_JPRB, 0.35063E+00_JPRB, 0.34732E+00_JPRB ,&
&  0.34625E+00_JPRB, 0.35771E+00_JPRB, 0.38526E+00_JPRB, 0.40991E+00_JPRB, 0.42210E+00_JPRB, 0.43995E+00_JPRB, 0.47268E+00_JPRB ,&
&  0.46665E+00_JPRB, 0.47690E+00_JPRB, 0.49971E+00_JPRB, 0.47298E+00_JPRB, 0.57492E+00_JPRB, 0.61739E+00_JPRB, 0.64153E+00_JPRB ,&
&  0.68894E+00_JPRB, 0.72897E+00_JPRB, 0.74433E+00_JPRB, 0.74646E+00_JPRB, 0.76343E+00_JPRB, 0.77354E+00_JPRB, 0.79670E+00_JPRB ,&
&  0.82014E+00_JPRB, 0.85595E+00_JPRB, 0.90548E+00_JPRB, 0.96466E+00_JPRB, 0.10816E+01_JPRB, 0.99005E+00_JPRB, 0.10882E+01_JPRB ,&
&  0.11493E+01_JPRB, 0.11790E+01_JPRB, 0.12534E+01_JPRB, 0.11059E+01_JPRB, 0.13903E+01_JPRB, 0.15355E+01_JPRB, 0.13815E+01_JPRB ,&
&  0.13581E+01_JPRB, 0.12855E+01_JPRB, 0.14845E+01_JPRB, 0.16332E+01_JPRB, 0.17548E+01_JPRB, 0.18637E+01_JPRB, 0.19025E+01_JPRB ,&
&  0.18738E+01_JPRB, 0.19629E+01_JPRB, 0.19212E+01_JPRB, 0.17430E+01_JPRB, 0.14974E+01_JPRB, 0.12899E+01_JPRB, 0.10486E+01_JPRB ,&
&  0.84731E+00_JPRB, 0.68484E+00_JPRB, 0.58476E+00_JPRB, 0.53542E+00_JPRB, 0.45840E+00_JPRB, 0.43182E+00_JPRB, 0.40375E+00_JPRB ,&
&  0.37615E+00_JPRB, 0.35355E+00_JPRB, 0.34682E+00_JPRB, 0.34036E+00_JPRB, 0.34163E+00_JPRB, 0.35756E+00_JPRB, 0.37750E+00_JPRB ,&
&  0.43126E+00_JPRB, 0.52366E+00_JPRB, 0.71505E+00_JPRB, 0.68620E+00_JPRB, 0.66902E+00_JPRB, 0.64952E+00_JPRB, 0.56383E+00_JPRB ,&
&  0.52778E+00_JPRB, 0.61650E+00_JPRB, 0.60748E+00_JPRB, 0.60174E+00_JPRB, 0.59339E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.64763E+00_JPRB, 0.63476E+00_JPRB, 0.61820E+00_JPRB, 0.59566E+00_JPRB, 0.56899E+00_JPRB, 0.54783E+00_JPRB, 0.53963E+00_JPRB ,&
&  0.54136E+00_JPRB, 0.54179E+00_JPRB, 0.54421E+00_JPRB, 0.54120E+00_JPRB, 0.53082E+00_JPRB, 0.54847E+00_JPRB, 0.48365E+00_JPRB ,&
&  0.41123E+00_JPRB, 0.38919E+00_JPRB, 0.37295E+00_JPRB, 0.37776E+00_JPRB, 0.38876E+00_JPRB, 0.38185E+00_JPRB, 0.37338E+00_JPRB ,&
&  0.36610E+00_JPRB, 0.36212E+00_JPRB, 0.37133E+00_JPRB, 0.38874E+00_JPRB, 0.40259E+00_JPRB, 0.41904E+00_JPRB, 0.44260E+00_JPRB ,&
&  0.43517E+00_JPRB, 0.41991E+00_JPRB, 0.41904E+00_JPRB, 0.43357E+00_JPRB, 0.50784E+00_JPRB, 0.60144E+00_JPRB, 0.65326E+00_JPRB ,&
&  0.67368E+00_JPRB, 0.68906E+00_JPRB, 0.72370E+00_JPRB, 0.73690E+00_JPRB, 0.74378E+00_JPRB, 0.76426E+00_JPRB, 0.79016E+00_JPRB ,&
&  0.81813E+00_JPRB, 0.85616E+00_JPRB, 0.90751E+00_JPRB, 0.99135E+00_JPRB, 0.10103E+01_JPRB, 0.10036E+01_JPRB, 0.11165E+01_JPRB ,&
&  0.12072E+01_JPRB, 0.13373E+01_JPRB, 0.13771E+01_JPRB, 0.14421E+01_JPRB, 0.15877E+01_JPRB, 0.15351E+01_JPRB, 0.14527E+01_JPRB ,&
&  0.14909E+01_JPRB, 0.14962E+01_JPRB, 0.18159E+01_JPRB, 0.19247E+01_JPRB, 0.19549E+01_JPRB, 0.20582E+01_JPRB, 0.20581E+01_JPRB ,&
&  0.20796E+01_JPRB, 0.20776E+01_JPRB, 0.20388E+01_JPRB, 0.18478E+01_JPRB, 0.15608E+01_JPRB, 0.13269E+01_JPRB, 0.10390E+01_JPRB ,&
&  0.84178E+00_JPRB, 0.67873E+00_JPRB, 0.53050E+00_JPRB, 0.46981E+00_JPRB, 0.41677E+00_JPRB, 0.40316E+00_JPRB, 0.38648E+00_JPRB ,&
&  0.36215E+00_JPRB, 0.35343E+00_JPRB, 0.34392E+00_JPRB, 0.35557E+00_JPRB, 0.36509E+00_JPRB, 0.36995E+00_JPRB, 0.41614E+00_JPRB ,&
&  0.50909E+00_JPRB, 0.73493E+00_JPRB, 0.71799E+00_JPRB, 0.76339E+00_JPRB, 0.75683E+00_JPRB, 0.72886E+00_JPRB, 0.55891E+00_JPRB ,&
&  0.60340E+00_JPRB, 0.67493E+00_JPRB, 0.64474E+00_JPRB, 0.64317E+00_JPRB, 0.64788E+00_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.66859E+00_JPRB, 0.65070E+00_JPRB, 0.63751E+00_JPRB, 0.62302E+00_JPRB, 0.59986E+00_JPRB, 0.57638E+00_JPRB, 0.57708E+00_JPRB ,&
&  0.57280E+00_JPRB, 0.56665E+00_JPRB, 0.56796E+00_JPRB, 0.56639E+00_JPRB, 0.55963E+00_JPRB, 0.56218E+00_JPRB, 0.57963E+00_JPRB ,&
&  0.47804E+00_JPRB, 0.44326E+00_JPRB, 0.39278E+00_JPRB, 0.38180E+00_JPRB, 0.43031E+00_JPRB, 0.41261E+00_JPRB, 0.40584E+00_JPRB ,&
&  0.38917E+00_JPRB, 0.38046E+00_JPRB, 0.36758E+00_JPRB, 0.37289E+00_JPRB, 0.38779E+00_JPRB, 0.40439E+00_JPRB, 0.41845E+00_JPRB ,&
&  0.40235E+00_JPRB, 0.38196E+00_JPRB, 0.38166E+00_JPRB, 0.39942E+00_JPRB, 0.45367E+00_JPRB, 0.53412E+00_JPRB, 0.62509E+00_JPRB ,&
&  0.66545E+00_JPRB, 0.67891E+00_JPRB, 0.70129E+00_JPRB, 0.72700E+00_JPRB, 0.76017E+00_JPRB, 0.81235E+00_JPRB, 0.84681E+00_JPRB ,&
&  0.87112E+00_JPRB, 0.89232E+00_JPRB, 0.93862E+00_JPRB, 0.10091E+01_JPRB, 0.97138E+00_JPRB, 0.97986E+00_JPRB, 0.11324E+01_JPRB ,&
&  0.13067E+01_JPRB, 0.12876E+01_JPRB, 0.13562E+01_JPRB, 0.17135E+01_JPRB, 0.19115E+01_JPRB, 0.19074E+01_JPRB, 0.18034E+01_JPRB ,&
&  0.18145E+01_JPRB, 0.18748E+01_JPRB, 0.19538E+01_JPRB, 0.20590E+01_JPRB, 0.21162E+01_JPRB, 0.21922E+01_JPRB, 0.21827E+01_JPRB ,&
&  0.21670E+01_JPRB, 0.21638E+01_JPRB, 0.20783E+01_JPRB, 0.18808E+01_JPRB, 0.15377E+01_JPRB, 0.12668E+01_JPRB, 0.97977E+00_JPRB ,&
&  0.80928E+00_JPRB, 0.62134E+00_JPRB, 0.45151E+00_JPRB, 0.38691E+00_JPRB, 0.36387E+00_JPRB, 0.38626E+00_JPRB, 0.48247E+00_JPRB ,&
&  0.38471E+00_JPRB, 0.38807E+00_JPRB, 0.38994E+00_JPRB, 0.41547E+00_JPRB, 0.44978E+00_JPRB, 0.47270E+00_JPRB, 0.44890E+00_JPRB ,&
&  0.47371E+00_JPRB, 0.62818E+00_JPRB, 0.78328E+00_JPRB, 0.84081E+00_JPRB, 0.83775E+00_JPRB, 0.81182E+00_JPRB, 0.72193E+00_JPRB ,&
&  0.80696E+00_JPRB, 0.72062E+00_JPRB, 0.67986E+00_JPRB, 0.67240E+00_JPRB, 0.68418E+00_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.66370E+00_JPRB, 0.65952E+00_JPRB, 0.65619E+00_JPRB, 0.64432E+00_JPRB, 0.63202E+00_JPRB, 0.60834E+00_JPRB, 0.60777E+00_JPRB ,&
&  0.60216E+00_JPRB, 0.59634E+00_JPRB, 0.58851E+00_JPRB, 0.58362E+00_JPRB, 0.57930E+00_JPRB, 0.57367E+00_JPRB, 0.56327E+00_JPRB ,&
&  0.58442E+00_JPRB, 0.52099E+00_JPRB, 0.45518E+00_JPRB, 0.38824E+00_JPRB, 0.40430E+00_JPRB, 0.40032E+00_JPRB, 0.40406E+00_JPRB ,&
&  0.39567E+00_JPRB, 0.39857E+00_JPRB, 0.38664E+00_JPRB, 0.37793E+00_JPRB, 0.39115E+00_JPRB, 0.41187E+00_JPRB, 0.42292E+00_JPRB ,&
&  0.41323E+00_JPRB, 0.39276E+00_JPRB, 0.38999E+00_JPRB, 0.41827E+00_JPRB, 0.50439E+00_JPRB, 0.51491E+00_JPRB, 0.61367E+00_JPRB ,&
&  0.66764E+00_JPRB, 0.70126E+00_JPRB, 0.73112E+00_JPRB, 0.77901E+00_JPRB, 0.83479E+00_JPRB, 0.90758E+00_JPRB, 0.95066E+00_JPRB ,&
&  0.97563E+00_JPRB, 0.97016E+00_JPRB, 0.93405E+00_JPRB, 0.99899E+00_JPRB, 0.10596E+01_JPRB, 0.10979E+01_JPRB, 0.11560E+01_JPRB ,&
&  0.12380E+01_JPRB, 0.12727E+01_JPRB, 0.14363E+01_JPRB, 0.18378E+01_JPRB, 0.20332E+01_JPRB, 0.20180E+01_JPRB, 0.20873E+01_JPRB ,&
&  0.21904E+01_JPRB, 0.22961E+01_JPRB, 0.23906E+01_JPRB, 0.23937E+01_JPRB, 0.23963E+01_JPRB, 0.23076E+01_JPRB, 0.22645E+01_JPRB ,&
&  0.21944E+01_JPRB, 0.22003E+01_JPRB, 0.21543E+01_JPRB, 0.18602E+01_JPRB, 0.14966E+01_JPRB, 0.11979E+01_JPRB, 0.10214E+01_JPRB ,&
&  0.94682E+00_JPRB, 0.52852E+00_JPRB, 0.38649E+00_JPRB, 0.32335E+00_JPRB, 0.31445E+00_JPRB, 0.32740E+00_JPRB, 0.39188E+00_JPRB ,&
&  0.36240E+00_JPRB, 0.40399E+00_JPRB, 0.47828E+00_JPRB, 0.51841E+00_JPRB, 0.60972E+00_JPRB, 0.71442E+00_JPRB, 0.59150E+00_JPRB ,&
&  0.54528E+00_JPRB, 0.61800E+00_JPRB, 0.87606E+00_JPRB, 0.92317E+00_JPRB, 0.90483E+00_JPRB, 0.87446E+00_JPRB, 0.87365E+00_JPRB ,&
&  0.83327E+00_JPRB, 0.80145E+00_JPRB, 0.73948E+00_JPRB, 0.69208E+00_JPRB, 0.67574E+00_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.70152E+00_JPRB, 0.68698E+00_JPRB, 0.68454E+00_JPRB, 0.67720E+00_JPRB, 0.66746E+00_JPRB, 0.65284E+00_JPRB, 0.65000E+00_JPRB ,&
&  0.63101E+00_JPRB, 0.61470E+00_JPRB, 0.59949E+00_JPRB, 0.59016E+00_JPRB, 0.58541E+00_JPRB, 0.57845E+00_JPRB, 0.56116E+00_JPRB ,&
&  0.54099E+00_JPRB, 0.54129E+00_JPRB, 0.46023E+00_JPRB, 0.40620E+00_JPRB, 0.36064E+00_JPRB, 0.37167E+00_JPRB, 0.38845E+00_JPRB ,&
&  0.41166E+00_JPRB, 0.42091E+00_JPRB, 0.43479E+00_JPRB, 0.45212E+00_JPRB, 0.47256E+00_JPRB, 0.46365E+00_JPRB, 0.48422E+00_JPRB ,&
&  0.47601E+00_JPRB, 0.45835E+00_JPRB, 0.43359E+00_JPRB, 0.49097E+00_JPRB, 0.55478E+00_JPRB, 0.55920E+00_JPRB, 0.69129E+00_JPRB ,&
&  0.75986E+00_JPRB, 0.80790E+00_JPRB, 0.84577E+00_JPRB, 0.89311E+00_JPRB, 0.94330E+00_JPRB, 0.94406E+00_JPRB, 0.91098E+00_JPRB ,&
&  0.85381E+00_JPRB, 0.85104E+00_JPRB, 0.90384E+00_JPRB, 0.93384E+00_JPRB, 0.10564E+01_JPRB, 0.11058E+01_JPRB, 0.11910E+01_JPRB ,&
&  0.13215E+01_JPRB, 0.13236E+01_JPRB, 0.13762E+01_JPRB, 0.18637E+01_JPRB, 0.23947E+01_JPRB, 0.23949E+01_JPRB, 0.24257E+01_JPRB ,&
&  0.25668E+01_JPRB, 0.25892E+01_JPRB, 0.25307E+01_JPRB, 0.24011E+01_JPRB, 0.23713E+01_JPRB, 0.23743E+01_JPRB, 0.23815E+01_JPRB ,&
&  0.22616E+01_JPRB, 0.22030E+01_JPRB, 0.21084E+01_JPRB, 0.18756E+01_JPRB, 0.15540E+01_JPRB, 0.13635E+01_JPRB, 0.11763E+01_JPRB ,&
&  0.10005E+01_JPRB, 0.65996E+00_JPRB, 0.44349E+00_JPRB, 0.37115E+00_JPRB, 0.35189E+00_JPRB, 0.35202E+00_JPRB, 0.40017E+00_JPRB ,&
&  0.40797E+00_JPRB, 0.47927E+00_JPRB, 0.58295E+00_JPRB, 0.63254E+00_JPRB, 0.89626E+00_JPRB, 0.10670E+01_JPRB, 0.10048E+01_JPRB ,&
&  0.85309E+00_JPRB, 0.78037E+00_JPRB, 0.10142E+01_JPRB, 0.10314E+01_JPRB, 0.10100E+01_JPRB, 0.92845E+00_JPRB, 0.90735E+00_JPRB ,&
&  0.86899E+00_JPRB, 0.84778E+00_JPRB, 0.81578E+00_JPRB, 0.76196E+00_JPRB, 0.72181E+00_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.71996E+00_JPRB, 0.68760E+00_JPRB, 0.66886E+00_JPRB, 0.66027E+00_JPRB, 0.63935E+00_JPRB, 0.62491E+00_JPRB, 0.63756E+00_JPRB ,&
&  0.64406E+00_JPRB, 0.62613E+00_JPRB, 0.61034E+00_JPRB, 0.60505E+00_JPRB, 0.60573E+00_JPRB, 0.59926E+00_JPRB, 0.56608E+00_JPRB ,&
&  0.52613E+00_JPRB, 0.50330E+00_JPRB, 0.42051E+00_JPRB, 0.37048E+00_JPRB, 0.35032E+00_JPRB, 0.37137E+00_JPRB, 0.41125E+00_JPRB ,&
&  0.46241E+00_JPRB, 0.51090E+00_JPRB, 0.56907E+00_JPRB, 0.59560E+00_JPRB, 0.62067E+00_JPRB, 0.62413E+00_JPRB, 0.63220E+00_JPRB ,&
&  0.67851E+00_JPRB, 0.57003E+00_JPRB, 0.58305E+00_JPRB, 0.60144E+00_JPRB, 0.67373E+00_JPRB, 0.72528E+00_JPRB, 0.77535E+00_JPRB ,&
&  0.78394E+00_JPRB, 0.81540E+00_JPRB, 0.84192E+00_JPRB, 0.86729E+00_JPRB, 0.88598E+00_JPRB, 0.88383E+00_JPRB, 0.86310E+00_JPRB ,&
&  0.88140E+00_JPRB, 0.86230E+00_JPRB, 0.85963E+00_JPRB, 0.10458E+01_JPRB, 0.14465E+01_JPRB, 0.16105E+01_JPRB, 0.16399E+01_JPRB ,&
&  0.15478E+01_JPRB, 0.16170E+01_JPRB, 0.23562E+01_JPRB, 0.26350E+01_JPRB, 0.27164E+01_JPRB, 0.29776E+01_JPRB, 0.30341E+01_JPRB ,&
&  0.29009E+01_JPRB, 0.26283E+01_JPRB, 0.23816E+01_JPRB, 0.21369E+01_JPRB, 0.25006E+01_JPRB, 0.26240E+01_JPRB, 0.25725E+01_JPRB ,&
&  0.22794E+01_JPRB, 0.21962E+01_JPRB, 0.19759E+01_JPRB, 0.17671E+01_JPRB, 0.16177E+01_JPRB, 0.16086E+01_JPRB, 0.12790E+01_JPRB ,&
&  0.97607E+00_JPRB, 0.87988E+00_JPRB, 0.67220E+00_JPRB, 0.57033E+00_JPRB, 0.53018E+00_JPRB, 0.52430E+00_JPRB, 0.54122E+00_JPRB ,&
&  0.57522E+00_JPRB, 0.64299E+00_JPRB, 0.74385E+00_JPRB, 0.86334E+00_JPRB, 0.11570E+01_JPRB, 0.13760E+01_JPRB, 0.14003E+01_JPRB ,&
&  0.11755E+01_JPRB, 0.12201E+01_JPRB, 0.11736E+01_JPRB, 0.11361E+01_JPRB, 0.11252E+01_JPRB, 0.10178E+01_JPRB, 0.96572E+00_JPRB ,&
&  0.89231E+00_JPRB, 0.86104E+00_JPRB, 0.84425E+00_JPRB, 0.80124E+00_JPRB, 0.76563E+00_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.77462E+00_JPRB, 0.73987E+00_JPRB, 0.69225E+00_JPRB, 0.65890E+00_JPRB, 0.63201E+00_JPRB, 0.61835E+00_JPRB, 0.61698E+00_JPRB ,&
&  0.61850E+00_JPRB, 0.61620E+00_JPRB, 0.62562E+00_JPRB, 0.63750E+00_JPRB, 0.64199E+00_JPRB, 0.62227E+00_JPRB, 0.56591E+00_JPRB ,&
&  0.52044E+00_JPRB, 0.45347E+00_JPRB, 0.39337E+00_JPRB, 0.39007E+00_JPRB, 0.41581E+00_JPRB, 0.44072E+00_JPRB, 0.48118E+00_JPRB ,&
&  0.54566E+00_JPRB, 0.66338E+00_JPRB, 0.74347E+00_JPRB, 0.84112E+00_JPRB, 0.88802E+00_JPRB, 0.89114E+00_JPRB, 0.86574E+00_JPRB ,&
&  0.79672E+00_JPRB, 0.80265E+00_JPRB, 0.84627E+00_JPRB, 0.83452E+00_JPRB, 0.80895E+00_JPRB, 0.80598E+00_JPRB, 0.81045E+00_JPRB ,&
&  0.80635E+00_JPRB, 0.80614E+00_JPRB, 0.84479E+00_JPRB, 0.86573E+00_JPRB, 0.91575E+00_JPRB, 0.94326E+00_JPRB, 0.95785E+00_JPRB ,&
&  0.91380E+00_JPRB, 0.89951E+00_JPRB, 0.10260E+01_JPRB, 0.15085E+01_JPRB, 0.19194E+01_JPRB, 0.22310E+01_JPRB, 0.21561E+01_JPRB ,&
&  0.23927E+01_JPRB, 0.25847E+01_JPRB, 0.27118E+01_JPRB, 0.30679E+01_JPRB, 0.24773E+01_JPRB, 0.26684E+01_JPRB, 0.29877E+01_JPRB ,&
&  0.28904E+01_JPRB, 0.26178E+01_JPRB, 0.24593E+01_JPRB, 0.23352E+01_JPRB, 0.25250E+01_JPRB, 0.33044E+01_JPRB, 0.30403E+01_JPRB ,&
&  0.25277E+01_JPRB, 0.22581E+01_JPRB, 0.19585E+01_JPRB, 0.18727E+01_JPRB, 0.16120E+01_JPRB, 0.14052E+01_JPRB, 0.10094E+01_JPRB ,&
&  0.97831E+00_JPRB, 0.86573E+00_JPRB, 0.85564E+00_JPRB, 0.78905E+00_JPRB, 0.78622E+00_JPRB, 0.75073E+00_JPRB, 0.73743E+00_JPRB ,&
&  0.79475E+00_JPRB, 0.85715E+00_JPRB, 0.10813E+01_JPRB, 0.12977E+01_JPRB, 0.17844E+01_JPRB, 0.17111E+01_JPRB, 0.17489E+01_JPRB ,&
&  0.16196E+01_JPRB, 0.15460E+01_JPRB, 0.13055E+01_JPRB, 0.13034E+01_JPRB, 0.11909E+01_JPRB, 0.11055E+01_JPRB, 0.10401E+01_JPRB ,&
&  0.93956E+00_JPRB, 0.86644E+00_JPRB, 0.82067E+00_JPRB, 0.78826E+00_JPRB, 0.78577E+00_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.74371E+00_JPRB, 0.71405E+00_JPRB, 0.68058E+00_JPRB, 0.64787E+00_JPRB, 0.61445E+00_JPRB, 0.60189E+00_JPRB, 0.60324E+00_JPRB ,&
&  0.60963E+00_JPRB, 0.62379E+00_JPRB, 0.62947E+00_JPRB, 0.63035E+00_JPRB, 0.62939E+00_JPRB, 0.63173E+00_JPRB, 0.57415E+00_JPRB ,&
&  0.51989E+00_JPRB, 0.46270E+00_JPRB, 0.40111E+00_JPRB, 0.39366E+00_JPRB, 0.46370E+00_JPRB, 0.52528E+00_JPRB, 0.51909E+00_JPRB ,&
&  0.68695E+00_JPRB, 0.86844E+00_JPRB, 0.95399E+00_JPRB, 0.10794E+01_JPRB, 0.12099E+01_JPRB, 0.12715E+01_JPRB, 0.11766E+01_JPRB ,&
&  0.11210E+01_JPRB, 0.10704E+01_JPRB, 0.96388E+00_JPRB, 0.88020E+00_JPRB, 0.82268E+00_JPRB, 0.78958E+00_JPRB, 0.75799E+00_JPRB ,&
&  0.74516E+00_JPRB, 0.75535E+00_JPRB, 0.77170E+00_JPRB, 0.80160E+00_JPRB, 0.86197E+00_JPRB, 0.83486E+00_JPRB, 0.87475E+00_JPRB ,&
&  0.92485E+00_JPRB, 0.10018E+01_JPRB, 0.12984E+01_JPRB, 0.18786E+01_JPRB, 0.25531E+01_JPRB, 0.29722E+01_JPRB, 0.30385E+01_JPRB ,&
&  0.28779E+01_JPRB, 0.28725E+01_JPRB, 0.30207E+01_JPRB, 0.30565E+01_JPRB, 0.30124E+01_JPRB, 0.29577E+01_JPRB, 0.31313E+01_JPRB ,&
&  0.24548E+01_JPRB, 0.20252E+01_JPRB, 0.17746E+01_JPRB, 0.20144E+01_JPRB, 0.30673E+01_JPRB, 0.39246E+01_JPRB, 0.38840E+01_JPRB ,&
&  0.32531E+01_JPRB, 0.27248E+01_JPRB, 0.23019E+01_JPRB, 0.19633E+01_JPRB, 0.18744E+01_JPRB, 0.12991E+01_JPRB, 0.13636E+01_JPRB ,&
&  0.13574E+01_JPRB, 0.13480E+01_JPRB, 0.12224E+01_JPRB, 0.10273E+01_JPRB, 0.91620E+00_JPRB, 0.99316E+00_JPRB, 0.10331E+01_JPRB ,&
&  0.11206E+01_JPRB, 0.12175E+01_JPRB, 0.18194E+01_JPRB, 0.21275E+01_JPRB, 0.23594E+01_JPRB, 0.20957E+01_JPRB, 0.19711E+01_JPRB ,&
&  0.16629E+01_JPRB, 0.16227E+01_JPRB, 0.15640E+01_JPRB, 0.14560E+01_JPRB, 0.13280E+01_JPRB, 0.12161E+01_JPRB, 0.11097E+01_JPRB ,&
&  0.10118E+01_JPRB, 0.91580E+00_JPRB, 0.83131E+00_JPRB, 0.77520E+00_JPRB, 0.75809E+00_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.75706E+00_JPRB, 0.73092E+00_JPRB, 0.69310E+00_JPRB, 0.63864E+00_JPRB, 0.61473E+00_JPRB, 0.61932E+00_JPRB, 0.64054E+00_JPRB ,&
&  0.65083E+00_JPRB, 0.65282E+00_JPRB, 0.63965E+00_JPRB, 0.62194E+00_JPRB, 0.62118E+00_JPRB, 0.61005E+00_JPRB, 0.57311E+00_JPRB ,&
&  0.54371E+00_JPRB, 0.51311E+00_JPRB, 0.43446E+00_JPRB, 0.44204E+00_JPRB, 0.53109E+00_JPRB, 0.62803E+00_JPRB, 0.62857E+00_JPRB ,&
&  0.91325E+00_JPRB, 0.10339E+01_JPRB, 0.10208E+01_JPRB, 0.11315E+01_JPRB, 0.14701E+01_JPRB, 0.13966E+01_JPRB, 0.12723E+01_JPRB ,&
&  0.12273E+01_JPRB, 0.11240E+01_JPRB, 0.98248E+00_JPRB, 0.87492E+00_JPRB, 0.80590E+00_JPRB, 0.76682E+00_JPRB, 0.73889E+00_JPRB ,&
&  0.73147E+00_JPRB, 0.73901E+00_JPRB, 0.78684E+00_JPRB, 0.84213E+00_JPRB, 0.81960E+00_JPRB, 0.80037E+00_JPRB, 0.88516E+00_JPRB ,&
&  0.97483E+00_JPRB, 0.10827E+01_JPRB, 0.13967E+01_JPRB, 0.19663E+01_JPRB, 0.27351E+01_JPRB, 0.33995E+01_JPRB, 0.42137E+01_JPRB ,&
&  0.37347E+01_JPRB, 0.37757E+01_JPRB, 0.34325E+01_JPRB, 0.40456E+01_JPRB, 0.36377E+01_JPRB, 0.33277E+01_JPRB, 0.31700E+01_JPRB ,&
&  0.22813E+01_JPRB, 0.19142E+01_JPRB, 0.21007E+01_JPRB, 0.30068E+01_JPRB, 0.32571E+01_JPRB, 0.36458E+01_JPRB, 0.44275E+01_JPRB ,&
&  0.35154E+01_JPRB, 0.29246E+01_JPRB, 0.26064E+01_JPRB, 0.25151E+01_JPRB, 0.16312E+01_JPRB, 0.12311E+01_JPRB, 0.17605E+01_JPRB ,&
&  0.16481E+01_JPRB, 0.99889E+00_JPRB, 0.76916E+00_JPRB, 0.87313E+00_JPRB, 0.69526E+00_JPRB, 0.80829E+00_JPRB, 0.11462E+01_JPRB ,&
&  0.13082E+01_JPRB, 0.15143E+01_JPRB, 0.23411E+01_JPRB, 0.22789E+01_JPRB, 0.23429E+01_JPRB, 0.23940E+01_JPRB, 0.20275E+01_JPRB ,&
&  0.18279E+01_JPRB, 0.15496E+01_JPRB, 0.15928E+01_JPRB, 0.14676E+01_JPRB, 0.13659E+01_JPRB, 0.12855E+01_JPRB, 0.11677E+01_JPRB ,&
&  0.10619E+01_JPRB, 0.95163E+00_JPRB, 0.85742E+00_JPRB, 0.79488E+00_JPRB, 0.77307E+00_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.75932E+00_JPRB, 0.73413E+00_JPRB, 0.72108E+00_JPRB, 0.69127E+00_JPRB, 0.66785E+00_JPRB, 0.66398E+00_JPRB, 0.68000E+00_JPRB ,&
&  0.66437E+00_JPRB, 0.63283E+00_JPRB, 0.59956E+00_JPRB, 0.58809E+00_JPRB, 0.59618E+00_JPRB, 0.59650E+00_JPRB, 0.56648E+00_JPRB ,&
&  0.54135E+00_JPRB, 0.52021E+00_JPRB, 0.52148E+00_JPRB, 0.55187E+00_JPRB, 0.68864E+00_JPRB, 0.73624E+00_JPRB, 0.87809E+00_JPRB ,&
&  0.11120E+01_JPRB, 0.11189E+01_JPRB, 0.10999E+01_JPRB, 0.11956E+01_JPRB, 0.12201E+01_JPRB, 0.11721E+01_JPRB, 0.12158E+01_JPRB ,&
&  0.11502E+01_JPRB, 0.10672E+01_JPRB, 0.96027E+00_JPRB, 0.88201E+00_JPRB, 0.80682E+00_JPRB, 0.75443E+00_JPRB, 0.73811E+00_JPRB ,&
&  0.73722E+00_JPRB, 0.77822E+00_JPRB, 0.86446E+00_JPRB, 0.81960E+00_JPRB, 0.80447E+00_JPRB, 0.81121E+00_JPRB, 0.91187E+00_JPRB ,&
&  0.10271E+01_JPRB, 0.11623E+01_JPRB, 0.14409E+01_JPRB, 0.20803E+01_JPRB, 0.33835E+01_JPRB, 0.43692E+01_JPRB, 0.43393E+01_JPRB ,&
&  0.39339E+01_JPRB, 0.37659E+01_JPRB, 0.39637E+01_JPRB, 0.42186E+01_JPRB, 0.36271E+01_JPRB, 0.34104E+01_JPRB, 0.30941E+01_JPRB ,&
&  0.26563E+01_JPRB, 0.24161E+01_JPRB, 0.23795E+01_JPRB, 0.32167E+01_JPRB, 0.38054E+01_JPRB, 0.38261E+01_JPRB, 0.43256E+01_JPRB ,&
&  0.38634E+01_JPRB, 0.31387E+01_JPRB, 0.23748E+01_JPRB, 0.18876E+01_JPRB, 0.18706E+01_JPRB, 0.15964E+01_JPRB, 0.11773E+01_JPRB ,&
&  0.10103E+01_JPRB, 0.84695E+00_JPRB, 0.71464E+00_JPRB, 0.70181E+00_JPRB, 0.61096E+00_JPRB, 0.69272E+00_JPRB, 0.10140E+01_JPRB ,&
&  0.14337E+01_JPRB, 0.17144E+01_JPRB, 0.21752E+01_JPRB, 0.22398E+01_JPRB, 0.22936E+01_JPRB, 0.25919E+01_JPRB, 0.25548E+01_JPRB ,&
&  0.16629E+01_JPRB, 0.15579E+01_JPRB, 0.15971E+01_JPRB, 0.15050E+01_JPRB, 0.13989E+01_JPRB, 0.12878E+01_JPRB, 0.11525E+01_JPRB ,&
&  0.10272E+01_JPRB, 0.91579E+00_JPRB, 0.84770E+00_JPRB, 0.82406E+00_JPRB, 0.79928E+00_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.71188E+00_JPRB, 0.70883E+00_JPRB, 0.71036E+00_JPRB, 0.70105E+00_JPRB, 0.68667E+00_JPRB, 0.68990E+00_JPRB, 0.66674E+00_JPRB ,&
&  0.62427E+00_JPRB, 0.58786E+00_JPRB, 0.56476E+00_JPRB, 0.56786E+00_JPRB, 0.57831E+00_JPRB, 0.58465E+00_JPRB, 0.56464E+00_JPRB ,&
&  0.53665E+00_JPRB, 0.53156E+00_JPRB, 0.57131E+00_JPRB, 0.65516E+00_JPRB, 0.90507E+00_JPRB, 0.90715E+00_JPRB, 0.11495E+01_JPRB ,&
&  0.12544E+01_JPRB, 0.11474E+01_JPRB, 0.11443E+01_JPRB, 0.11241E+01_JPRB, 0.11480E+01_JPRB, 0.10687E+01_JPRB, 0.10566E+01_JPRB ,&
&  0.95808E+00_JPRB, 0.96424E+00_JPRB, 0.87066E+00_JPRB, 0.81406E+00_JPRB, 0.76583E+00_JPRB, 0.74684E+00_JPRB, 0.76240E+00_JPRB ,&
&  0.80743E+00_JPRB, 0.87623E+00_JPRB, 0.85023E+00_JPRB, 0.82481E+00_JPRB, 0.84770E+00_JPRB, 0.87728E+00_JPRB, 0.97808E+00_JPRB ,&
&  0.11070E+01_JPRB, 0.12628E+01_JPRB, 0.15704E+01_JPRB, 0.25946E+01_JPRB, 0.35200E+01_JPRB, 0.37803E+01_JPRB, 0.42486E+01_JPRB ,&
&  0.44147E+01_JPRB, 0.44823E+01_JPRB, 0.38864E+01_JPRB, 0.44093E+01_JPRB, 0.36410E+01_JPRB, 0.32462E+01_JPRB, 0.30281E+01_JPRB ,&
&  0.26922E+01_JPRB, 0.28555E+01_JPRB, 0.27004E+01_JPRB, 0.28657E+01_JPRB, 0.38572E+01_JPRB, 0.43667E+01_JPRB, 0.42609E+01_JPRB ,&
&  0.40615E+01_JPRB, 0.33673E+01_JPRB, 0.26515E+01_JPRB, 0.22688E+01_JPRB, 0.27842E+01_JPRB, 0.28580E+01_JPRB, 0.13895E+01_JPRB ,&
&  0.98415E+00_JPRB, 0.90517E+00_JPRB, 0.69239E+00_JPRB, 0.59452E+00_JPRB, 0.56188E+00_JPRB, 0.66388E+00_JPRB, 0.11081E+01_JPRB ,&
&  0.12375E+01_JPRB, 0.19272E+01_JPRB, 0.21162E+01_JPRB, 0.22736E+01_JPRB, 0.21876E+01_JPRB, 0.21891E+01_JPRB, 0.25217E+01_JPRB ,&
&  0.25397E+01_JPRB, 0.21979E+01_JPRB, 0.17937E+01_JPRB, 0.15680E+01_JPRB, 0.13683E+01_JPRB, 0.11948E+01_JPRB, 0.10674E+01_JPRB ,&
&  0.98850E+00_JPRB, 0.90424E+00_JPRB, 0.85371E+00_JPRB, 0.82302E+00_JPRB, 0.75701E+00_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.64992E+00_JPRB, 0.66658E+00_JPRB, 0.68514E+00_JPRB, 0.67542E+00_JPRB, 0.65618E+00_JPRB, 0.65692E+00_JPRB, 0.61659E+00_JPRB ,&
&  0.58089E+00_JPRB, 0.56158E+00_JPRB, 0.55337E+00_JPRB, 0.56421E+00_JPRB, 0.59056E+00_JPRB, 0.60974E+00_JPRB, 0.59304E+00_JPRB ,&
&  0.56061E+00_JPRB, 0.56056E+00_JPRB, 0.59969E+00_JPRB, 0.84695E+00_JPRB, 0.10768E+01_JPRB, 0.12446E+01_JPRB, 0.13755E+01_JPRB ,&
&  0.13278E+01_JPRB, 0.12197E+01_JPRB, 0.11717E+01_JPRB, 0.11795E+01_JPRB, 0.11505E+01_JPRB, 0.11196E+01_JPRB, 0.10182E+01_JPRB ,&
&  0.90662E+00_JPRB, 0.91251E+00_JPRB, 0.80526E+00_JPRB, 0.77149E+00_JPRB, 0.76034E+00_JPRB, 0.77305E+00_JPRB, 0.82193E+00_JPRB ,&
&  0.88543E+00_JPRB, 0.86664E+00_JPRB, 0.87985E+00_JPRB, 0.90830E+00_JPRB, 0.95916E+00_JPRB, 0.99274E+00_JPRB, 0.10784E+01_JPRB ,&
&  0.12287E+01_JPRB, 0.14568E+01_JPRB, 0.19268E+01_JPRB, 0.30688E+01_JPRB, 0.31971E+01_JPRB, 0.41238E+01_JPRB, 0.43026E+01_JPRB ,&
&  0.43251E+01_JPRB, 0.42293E+01_JPRB, 0.37688E+01_JPRB, 0.39474E+01_JPRB, 0.35311E+01_JPRB, 0.32018E+01_JPRB, 0.28989E+01_JPRB ,&
&  0.29679E+01_JPRB, 0.28172E+01_JPRB, 0.33312E+01_JPRB, 0.32085E+01_JPRB, 0.35820E+01_JPRB, 0.56084E+01_JPRB, 0.43377E+01_JPRB ,&
&  0.36094E+01_JPRB, 0.36994E+01_JPRB, 0.29854E+01_JPRB, 0.26192E+01_JPRB, 0.30886E+01_JPRB, 0.34936E+01_JPRB, 0.27406E+01_JPRB ,&
&  0.11495E+01_JPRB, 0.80948E+00_JPRB, 0.69694E+00_JPRB, 0.61073E+00_JPRB, 0.52860E+00_JPRB, 0.62779E+00_JPRB, 0.18446E+01_JPRB ,&
&  0.13858E+01_JPRB, 0.19113E+01_JPRB, 0.19763E+01_JPRB, 0.19905E+01_JPRB, 0.21152E+01_JPRB, 0.19560E+01_JPRB, 0.21783E+01_JPRB ,&
&  0.18712E+01_JPRB, 0.15238E+01_JPRB, 0.12712E+01_JPRB, 0.11012E+01_JPRB, 0.10154E+01_JPRB, 0.97308E+00_JPRB, 0.94918E+00_JPRB ,&
&  0.91994E+00_JPRB, 0.85527E+00_JPRB, 0.79567E+00_JPRB, 0.71903E+00_JPRB, 0.65189E+00_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.65385E+00_JPRB, 0.65176E+00_JPRB, 0.65467E+00_JPRB, 0.67037E+00_JPRB, 0.68357E+00_JPRB, 0.66356E+00_JPRB, 0.62671E+00_JPRB ,&
&  0.62981E+00_JPRB, 0.62099E+00_JPRB, 0.64206E+00_JPRB, 0.65974E+00_JPRB, 0.68377E+00_JPRB, 0.67605E+00_JPRB, 0.65253E+00_JPRB ,&
&  0.61360E+00_JPRB, 0.61912E+00_JPRB, 0.76382E+00_JPRB, 0.10506E+01_JPRB, 0.12187E+01_JPRB, 0.13123E+01_JPRB, 0.14719E+01_JPRB ,&
&  0.14954E+01_JPRB, 0.12839E+01_JPRB, 0.11143E+01_JPRB, 0.11784E+01_JPRB, 0.12077E+01_JPRB, 0.12358E+01_JPRB, 0.10718E+01_JPRB ,&
&  0.95508E+00_JPRB, 0.92053E+00_JPRB, 0.81840E+00_JPRB, 0.80693E+00_JPRB, 0.81660E+00_JPRB, 0.84021E+00_JPRB, 0.88285E+00_JPRB ,&
&  0.90784E+00_JPRB, 0.96020E+00_JPRB, 0.10316E+01_JPRB, 0.10910E+01_JPRB, 0.11360E+01_JPRB, 0.11643E+01_JPRB, 0.12528E+01_JPRB ,&
&  0.14408E+01_JPRB, 0.17441E+01_JPRB, 0.22705E+01_JPRB, 0.30080E+01_JPRB, 0.36132E+01_JPRB, 0.38317E+01_JPRB, 0.41225E+01_JPRB ,&
&  0.41428E+01_JPRB, 0.40999E+01_JPRB, 0.38809E+01_JPRB, 0.36027E+01_JPRB, 0.32456E+01_JPRB, 0.31660E+01_JPRB, 0.29162E+01_JPRB ,&
&  0.31129E+01_JPRB, 0.33657E+01_JPRB, 0.37939E+01_JPRB, 0.36559E+01_JPRB, 0.38186E+01_JPRB, 0.52936E+01_JPRB, 0.55594E+01_JPRB ,&
&  0.44686E+01_JPRB, 0.32596E+01_JPRB, 0.27571E+01_JPRB, 0.28530E+01_JPRB, 0.30450E+01_JPRB, 0.31157E+01_JPRB, 0.38414E+01_JPRB ,&
&  0.29560E+01_JPRB, 0.13582E+01_JPRB, 0.79911E+00_JPRB, 0.76894E+00_JPRB, 0.48788E+00_JPRB, 0.46414E+00_JPRB, 0.85069E+00_JPRB ,&
&  0.11595E+01_JPRB, 0.14905E+01_JPRB, 0.15677E+01_JPRB, 0.17361E+01_JPRB, 0.17843E+01_JPRB, 0.15357E+01_JPRB, 0.12472E+01_JPRB ,&
&  0.99219E+00_JPRB, 0.85931E+00_JPRB, 0.79647E+00_JPRB, 0.76677E+00_JPRB, 0.79739E+00_JPRB, 0.84345E+00_JPRB, 0.86875E+00_JPRB ,&
&  0.84900E+00_JPRB, 0.78865E+00_JPRB, 0.70287E+00_JPRB, 0.64138E+00_JPRB, 0.63267E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.72238E+00_JPRB, 0.71083E+00_JPRB, 0.71245E+00_JPRB, 0.73787E+00_JPRB, 0.74893E+00_JPRB, 0.75410E+00_JPRB, 0.80538E+00_JPRB ,&
&  0.82454E+00_JPRB, 0.82175E+00_JPRB, 0.83141E+00_JPRB, 0.82875E+00_JPRB, 0.80922E+00_JPRB, 0.76327E+00_JPRB, 0.72880E+00_JPRB ,&
&  0.72663E+00_JPRB, 0.84086E+00_JPRB, 0.10889E+01_JPRB, 0.13803E+01_JPRB, 0.14857E+01_JPRB, 0.18089E+01_JPRB, 0.14195E+01_JPRB ,&
&  0.13908E+01_JPRB, 0.15229E+01_JPRB, 0.12257E+01_JPRB, 0.12736E+01_JPRB, 0.12869E+01_JPRB, 0.13492E+01_JPRB, 0.11338E+01_JPRB ,&
&  0.10355E+01_JPRB, 0.99105E+00_JPRB, 0.89598E+00_JPRB, 0.88055E+00_JPRB, 0.90338E+00_JPRB, 0.95146E+00_JPRB, 0.99027E+00_JPRB ,&
&  0.10659E+01_JPRB, 0.11803E+01_JPRB, 0.12811E+01_JPRB, 0.13325E+01_JPRB, 0.13646E+01_JPRB, 0.13990E+01_JPRB, 0.15258E+01_JPRB ,&
&  0.17267E+01_JPRB, 0.20501E+01_JPRB, 0.25796E+01_JPRB, 0.31574E+01_JPRB, 0.35228E+01_JPRB, 0.35415E+01_JPRB, 0.37562E+01_JPRB ,&
&  0.36149E+01_JPRB, 0.33893E+01_JPRB, 0.34602E+01_JPRB, 0.33671E+01_JPRB, 0.31277E+01_JPRB, 0.29751E+01_JPRB, 0.27874E+01_JPRB ,&
&  0.33406E+01_JPRB, 0.42328E+01_JPRB, 0.52711E+01_JPRB, 0.45618E+01_JPRB, 0.44876E+01_JPRB, 0.54252E+01_JPRB, 0.55831E+01_JPRB ,&
&  0.48163E+01_JPRB, 0.36026E+01_JPRB, 0.31956E+01_JPRB, 0.28347E+01_JPRB, 0.26779E+01_JPRB, 0.25786E+01_JPRB, 0.29191E+01_JPRB ,&
&  0.29140E+01_JPRB, 0.27537E+01_JPRB, 0.15023E+01_JPRB, 0.71075E+00_JPRB, 0.52543E+00_JPRB, 0.57036E+00_JPRB, 0.70712E+00_JPRB ,&
&  0.93678E+00_JPRB, 0.11144E+01_JPRB, 0.12197E+01_JPRB, 0.15306E+01_JPRB, 0.13753E+01_JPRB, 0.11334E+01_JPRB, 0.92986E+00_JPRB ,&
&  0.81131E+00_JPRB, 0.72231E+00_JPRB, 0.68240E+00_JPRB, 0.68614E+00_JPRB, 0.74170E+00_JPRB, 0.80348E+00_JPRB, 0.82754E+00_JPRB ,&
&  0.78288E+00_JPRB, 0.71481E+00_JPRB, 0.68537E+00_JPRB, 0.68418E+00_JPRB, 0.70121E+00_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.82640E+00_JPRB, 0.84191E+00_JPRB, 0.87931E+00_JPRB, 0.93849E+00_JPRB, 0.99458E+00_JPRB, 0.10519E+01_JPRB, 0.10821E+01_JPRB ,&
&  0.10805E+01_JPRB, 0.10542E+01_JPRB, 0.10388E+01_JPRB, 0.99766E+00_JPRB, 0.97076E+00_JPRB, 0.95976E+00_JPRB, 0.10090E+01_JPRB ,&
&  0.11637E+01_JPRB, 0.13924E+01_JPRB, 0.16083E+01_JPRB, 0.17128E+01_JPRB, 0.17159E+01_JPRB, 0.18710E+01_JPRB, 0.17954E+01_JPRB ,&
&  0.13317E+01_JPRB, 0.19060E+01_JPRB, 0.13781E+01_JPRB, 0.14146E+01_JPRB, 0.14463E+01_JPRB, 0.15641E+01_JPRB, 0.13413E+01_JPRB ,&
&  0.11821E+01_JPRB, 0.11388E+01_JPRB, 0.10987E+01_JPRB, 0.10839E+01_JPRB, 0.10963E+01_JPRB, 0.11475E+01_JPRB, 0.12447E+01_JPRB ,&
&  0.13510E+01_JPRB, 0.14389E+01_JPRB, 0.15070E+01_JPRB, 0.15501E+01_JPRB, 0.16040E+01_JPRB, 0.16717E+01_JPRB, 0.18488E+01_JPRB ,&
&  0.20767E+01_JPRB, 0.24419E+01_JPRB, 0.28710E+01_JPRB, 0.33242E+01_JPRB, 0.33089E+01_JPRB, 0.32820E+01_JPRB, 0.34000E+01_JPRB ,&
&  0.33073E+01_JPRB, 0.31052E+01_JPRB, 0.31513E+01_JPRB, 0.29726E+01_JPRB, 0.27664E+01_JPRB, 0.28721E+01_JPRB, 0.28449E+01_JPRB ,&
&  0.38148E+01_JPRB, 0.44738E+01_JPRB, 0.59817E+01_JPRB, 0.54779E+01_JPRB, 0.48038E+01_JPRB, 0.50185E+01_JPRB, 0.43345E+01_JPRB ,&
&  0.33590E+01_JPRB, 0.30478E+01_JPRB, 0.27991E+01_JPRB, 0.25312E+01_JPRB, 0.23590E+01_JPRB, 0.21958E+01_JPRB, 0.22907E+01_JPRB ,&
&  0.22855E+01_JPRB, 0.21468E+01_JPRB, 0.15990E+01_JPRB, 0.82941E+00_JPRB, 0.62546E+00_JPRB, 0.59317E+00_JPRB, 0.69570E+00_JPRB ,&
&  0.91513E+00_JPRB, 0.10596E+01_JPRB, 0.11237E+01_JPRB, 0.10424E+01_JPRB, 0.89267E+00_JPRB, 0.76684E+00_JPRB, 0.67924E+00_JPRB ,&
&  0.65547E+00_JPRB, 0.65450E+00_JPRB, 0.66323E+00_JPRB, 0.71531E+00_JPRB, 0.77583E+00_JPRB, 0.81155E+00_JPRB, 0.76170E+00_JPRB ,&
&  0.72366E+00_JPRB, 0.72193E+00_JPRB, 0.75424E+00_JPRB, 0.78474E+00_JPRB, 0.81179E+00_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.98245E+00_JPRB, 0.10375E+01_JPRB, 0.11177E+01_JPRB, 0.12405E+01_JPRB, 0.14085E+01_JPRB, 0.15749E+01_JPRB, 0.14907E+01_JPRB ,&
&  0.13836E+01_JPRB, 0.12491E+01_JPRB, 0.12890E+01_JPRB, 0.13287E+01_JPRB, 0.13665E+01_JPRB, 0.14401E+01_JPRB, 0.15501E+01_JPRB ,&
&  0.16754E+01_JPRB, 0.17920E+01_JPRB, 0.19024E+01_JPRB, 0.20004E+01_JPRB, 0.20997E+01_JPRB, 0.23920E+01_JPRB, 0.24387E+01_JPRB ,&
&  0.19467E+01_JPRB, 0.14619E+01_JPRB, 0.17131E+01_JPRB, 0.15120E+01_JPRB, 0.15353E+01_JPRB, 0.16497E+01_JPRB, 0.16096E+01_JPRB ,&
&  0.14279E+01_JPRB, 0.13632E+01_JPRB, 0.13300E+01_JPRB, 0.13295E+01_JPRB, 0.13261E+01_JPRB, 0.14063E+01_JPRB, 0.14959E+01_JPRB ,&
&  0.15677E+01_JPRB, 0.16199E+01_JPRB, 0.16809E+01_JPRB, 0.17369E+01_JPRB, 0.18064E+01_JPRB, 0.19196E+01_JPRB, 0.21367E+01_JPRB ,&
&  0.23677E+01_JPRB, 0.26681E+01_JPRB, 0.29785E+01_JPRB, 0.31357E+01_JPRB, 0.30805E+01_JPRB, 0.30384E+01_JPRB, 0.30610E+01_JPRB ,&
&  0.30378E+01_JPRB, 0.30221E+01_JPRB, 0.29527E+01_JPRB, 0.28125E+01_JPRB, 0.26414E+01_JPRB, 0.29999E+01_JPRB, 0.33522E+01_JPRB ,&
&  0.42143E+01_JPRB, 0.47861E+01_JPRB, 0.60596E+01_JPRB, 0.52795E+01_JPRB, 0.45738E+01_JPRB, 0.41142E+01_JPRB, 0.34941E+01_JPRB ,&
&  0.28262E+01_JPRB, 0.27683E+01_JPRB, 0.26210E+01_JPRB, 0.22258E+01_JPRB, 0.18936E+01_JPRB, 0.15453E+01_JPRB, 0.17304E+01_JPRB ,&
&  0.17946E+01_JPRB, 0.18432E+01_JPRB, 0.12738E+01_JPRB, 0.91714E+00_JPRB, 0.69956E+00_JPRB, 0.63888E+00_JPRB, 0.74123E+00_JPRB ,&
&  0.95143E+00_JPRB, 0.10004E+01_JPRB, 0.90139E+00_JPRB, 0.86160E+00_JPRB, 0.77802E+00_JPRB, 0.66935E+00_JPRB, 0.58529E+00_JPRB ,&
&  0.60728E+00_JPRB, 0.62759E+00_JPRB, 0.66201E+00_JPRB, 0.71986E+00_JPRB, 0.75050E+00_JPRB, 0.79164E+00_JPRB, 0.79431E+00_JPRB ,&
&  0.83049E+00_JPRB, 0.86536E+00_JPRB, 0.88570E+00_JPRB, 0.90897E+00_JPRB, 0.93852E+00_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.10553E+01_JPRB, 0.11068E+01_JPRB, 0.11734E+01_JPRB, 0.12594E+01_JPRB, 0.13499E+01_JPRB, 0.13900E+01_JPRB, 0.13983E+01_JPRB ,&
&  0.14470E+01_JPRB, 0.15190E+01_JPRB, 0.16033E+01_JPRB, 0.16658E+01_JPRB, 0.16938E+01_JPRB, 0.17520E+01_JPRB, 0.18198E+01_JPRB ,&
&  0.18905E+01_JPRB, 0.19764E+01_JPRB, 0.20884E+01_JPRB, 0.22235E+01_JPRB, 0.24161E+01_JPRB, 0.26602E+01_JPRB, 0.27833E+01_JPRB ,&
&  0.21815E+01_JPRB, 0.14757E+01_JPRB, 0.18233E+01_JPRB, 0.17695E+01_JPRB, 0.16483E+01_JPRB, 0.16388E+01_JPRB, 0.16074E+01_JPRB ,&
&  0.15348E+01_JPRB, 0.15291E+01_JPRB, 0.15552E+01_JPRB, 0.15638E+01_JPRB, 0.15568E+01_JPRB, 0.16103E+01_JPRB, 0.16677E+01_JPRB ,&
&  0.17201E+01_JPRB, 0.17710E+01_JPRB, 0.18233E+01_JPRB, 0.18647E+01_JPRB, 0.19380E+01_JPRB, 0.20620E+01_JPRB, 0.22603E+01_JPRB ,&
&  0.24919E+01_JPRB, 0.26966E+01_JPRB, 0.28960E+01_JPRB, 0.30033E+01_JPRB, 0.29494E+01_JPRB, 0.29058E+01_JPRB, 0.29225E+01_JPRB ,&
&  0.29582E+01_JPRB, 0.29161E+01_JPRB, 0.29638E+01_JPRB, 0.31199E+01_JPRB, 0.33475E+01_JPRB, 0.33707E+01_JPRB, 0.40351E+01_JPRB ,&
&  0.46506E+01_JPRB, 0.49164E+01_JPRB, 0.52587E+01_JPRB, 0.61446E+01_JPRB, 0.39198E+01_JPRB, 0.34357E+01_JPRB, 0.27951E+01_JPRB ,&
&  0.28926E+01_JPRB, 0.27893E+01_JPRB, 0.24581E+01_JPRB, 0.19652E+01_JPRB, 0.16108E+01_JPRB, 0.11905E+01_JPRB, 0.12730E+01_JPRB ,&
&  0.14831E+01_JPRB, 0.15107E+01_JPRB, 0.11700E+01_JPRB, 0.10224E+01_JPRB, 0.85776E+00_JPRB, 0.86220E+00_JPRB, 0.93564E+00_JPRB ,&
&  0.98935E+00_JPRB, 0.91282E+00_JPRB, 0.88165E+00_JPRB, 0.94069E+00_JPRB, 0.83673E+00_JPRB, 0.60083E+00_JPRB, 0.51548E+00_JPRB ,&
&  0.53030E+00_JPRB, 0.57279E+00_JPRB, 0.63243E+00_JPRB, 0.69888E+00_JPRB, 0.73471E+00_JPRB, 0.77440E+00_JPRB, 0.81503E+00_JPRB ,&
&  0.88904E+00_JPRB, 0.94286E+00_JPRB, 0.98109E+00_JPRB, 0.99109E+00_JPRB, 0.10245E+01_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.10472E+01_JPRB, 0.10929E+01_JPRB, 0.11242E+01_JPRB, 0.11730E+01_JPRB, 0.12242E+01_JPRB, 0.12807E+01_JPRB, 0.13548E+01_JPRB ,&
&  0.14509E+01_JPRB, 0.15560E+01_JPRB, 0.16539E+01_JPRB, 0.17257E+01_JPRB, 0.17919E+01_JPRB, 0.18403E+01_JPRB, 0.18851E+01_JPRB ,&
&  0.19197E+01_JPRB, 0.19895E+01_JPRB, 0.20640E+01_JPRB, 0.21609E+01_JPRB, 0.22883E+01_JPRB, 0.24395E+01_JPRB, 0.25821E+01_JPRB ,&
&  0.26604E+01_JPRB, 0.25192E+01_JPRB, 0.21624E+01_JPRB, 0.15482E+01_JPRB, 0.13315E+01_JPRB, 0.15469E+01_JPRB, 0.15789E+01_JPRB ,&
&  0.16017E+01_JPRB, 0.16022E+01_JPRB, 0.15945E+01_JPRB, 0.16439E+01_JPRB, 0.16796E+01_JPRB, 0.17118E+01_JPRB, 0.17282E+01_JPRB ,&
&  0.17554E+01_JPRB, 0.18029E+01_JPRB, 0.18329E+01_JPRB, 0.18621E+01_JPRB, 0.19194E+01_JPRB, 0.20354E+01_JPRB, 0.22141E+01_JPRB ,&
&  0.24337E+01_JPRB, 0.26284E+01_JPRB, 0.27631E+01_JPRB, 0.28538E+01_JPRB, 0.28155E+01_JPRB, 0.28407E+01_JPRB, 0.29507E+01_JPRB ,&
&  0.31134E+01_JPRB, 0.31995E+01_JPRB, 0.33705E+01_JPRB, 0.36919E+01_JPRB, 0.39178E+01_JPRB, 0.37818E+01_JPRB, 0.41837E+01_JPRB ,&
&  0.46555E+01_JPRB, 0.47929E+01_JPRB, 0.47108E+01_JPRB, 0.59524E+01_JPRB, 0.37097E+01_JPRB, 0.35843E+01_JPRB, 0.32693E+01_JPRB ,&
&  0.30200E+01_JPRB, 0.29690E+01_JPRB, 0.23124E+01_JPRB, 0.16740E+01_JPRB, 0.14193E+01_JPRB, 0.10879E+01_JPRB, 0.11721E+01_JPRB ,&
&  0.13580E+01_JPRB, 0.12306E+01_JPRB, 0.10358E+01_JPRB, 0.97423E+00_JPRB, 0.97019E+00_JPRB, 0.10312E+01_JPRB, 0.10712E+01_JPRB ,&
&  0.89410E+00_JPRB, 0.86354E+00_JPRB, 0.88083E+00_JPRB, 0.11014E+01_JPRB, 0.11225E+01_JPRB, 0.56927E+00_JPRB, 0.48051E+00_JPRB ,&
&  0.49918E+00_JPRB, 0.54501E+00_JPRB, 0.59912E+00_JPRB, 0.63951E+00_JPRB, 0.67760E+00_JPRB, 0.72595E+00_JPRB, 0.77868E+00_JPRB ,&
&  0.83769E+00_JPRB, 0.90658E+00_JPRB, 0.95508E+00_JPRB, 0.98792E+00_JPRB, 0.10127E+01_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.86690E+00_JPRB, 0.91361E+00_JPRB, 0.95795E+00_JPRB, 0.98499E+00_JPRB, 0.99067E+00_JPRB, 0.10008E+01_JPRB, 0.10029E+01_JPRB ,&
&  0.10134E+01_JPRB, 0.10134E+01_JPRB, 0.10450E+01_JPRB, 0.11177E+01_JPRB, 0.12605E+01_JPRB, 0.13412E+01_JPRB, 0.14413E+01_JPRB ,&
&  0.14516E+01_JPRB, 0.14715E+01_JPRB, 0.14978E+01_JPRB, 0.15380E+01_JPRB, 0.16147E+01_JPRB, 0.17276E+01_JPRB, 0.19714E+01_JPRB ,&
&  0.24227E+01_JPRB, 0.27060E+01_JPRB, 0.26613E+01_JPRB, 0.23101E+01_JPRB, 0.18127E+01_JPRB, 0.15846E+01_JPRB, 0.17757E+01_JPRB ,&
&  0.18191E+01_JPRB, 0.17171E+01_JPRB, 0.16650E+01_JPRB, 0.16598E+01_JPRB, 0.16614E+01_JPRB, 0.16725E+01_JPRB, 0.16781E+01_JPRB ,&
&  0.16987E+01_JPRB, 0.17317E+01_JPRB, 0.17558E+01_JPRB, 0.17806E+01_JPRB, 0.18330E+01_JPRB, 0.19186E+01_JPRB, 0.20365E+01_JPRB ,&
&  0.21985E+01_JPRB, 0.23340E+01_JPRB, 0.24143E+01_JPRB, 0.24517E+01_JPRB, 0.25290E+01_JPRB, 0.27194E+01_JPRB, 0.30083E+01_JPRB ,&
&  0.32744E+01_JPRB, 0.33941E+01_JPRB, 0.37006E+01_JPRB, 0.40189E+01_JPRB, 0.40360E+01_JPRB, 0.39824E+01_JPRB, 0.41629E+01_JPRB ,&
&  0.42835E+01_JPRB, 0.42974E+01_JPRB, 0.37179E+01_JPRB, 0.51061E+01_JPRB, 0.51975E+01_JPRB, 0.43526E+01_JPRB, 0.34431E+01_JPRB ,&
&  0.30633E+01_JPRB, 0.28214E+01_JPRB, 0.23610E+01_JPRB, 0.18012E+01_JPRB, 0.15089E+01_JPRB, 0.14374E+01_JPRB, 0.12556E+01_JPRB ,&
&  0.12744E+01_JPRB, 0.10694E+01_JPRB, 0.98629E+00_JPRB, 0.98568E+00_JPRB, 0.92141E+00_JPRB, 0.10083E+01_JPRB, 0.92023E+00_JPRB ,&
&  0.74278E+00_JPRB, 0.76285E+00_JPRB, 0.83796E+00_JPRB, 0.12379E+01_JPRB, 0.19798E+01_JPRB, 0.54145E+00_JPRB, 0.44068E+00_JPRB ,&
&  0.46261E+00_JPRB, 0.49537E+00_JPRB, 0.53419E+00_JPRB, 0.55199E+00_JPRB, 0.55799E+00_JPRB, 0.59363E+00_JPRB, 0.64702E+00_JPRB ,&
&  0.68058E+00_JPRB, 0.70603E+00_JPRB, 0.73775E+00_JPRB, 0.79610E+00_JPRB, 0.82855E+00_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.60668E+00_JPRB, 0.63713E+00_JPRB, 0.65599E+00_JPRB, 0.67783E+00_JPRB, 0.68676E+00_JPRB, 0.67887E+00_JPRB, 0.64883E+00_JPRB ,&
&  0.62391E+00_JPRB, 0.59840E+00_JPRB, 0.58822E+00_JPRB, 0.57361E+00_JPRB, 0.59337E+00_JPRB, 0.63302E+00_JPRB, 0.66627E+00_JPRB ,&
&  0.74548E+00_JPRB, 0.70069E+00_JPRB, 0.70277E+00_JPRB, 0.73483E+00_JPRB, 0.74031E+00_JPRB, 0.77840E+00_JPRB, 0.91142E+00_JPRB ,&
&  0.11207E+01_JPRB, 0.13558E+01_JPRB, 0.17046E+01_JPRB, 0.22507E+01_JPRB, 0.23729E+01_JPRB, 0.18382E+01_JPRB, 0.18497E+01_JPRB ,&
&  0.16450E+01_JPRB, 0.14325E+01_JPRB, 0.14725E+01_JPRB, 0.14867E+01_JPRB, 0.15327E+01_JPRB, 0.15307E+01_JPRB, 0.15587E+01_JPRB ,&
&  0.15946E+01_JPRB, 0.16397E+01_JPRB, 0.16824E+01_JPRB, 0.17144E+01_JPRB, 0.17546E+01_JPRB, 0.18076E+01_JPRB, 0.18880E+01_JPRB ,&
&  0.19402E+01_JPRB, 0.19798E+01_JPRB, 0.20599E+01_JPRB, 0.20909E+01_JPRB, 0.23245E+01_JPRB, 0.26536E+01_JPRB, 0.29767E+01_JPRB ,&
&  0.31903E+01_JPRB, 0.33358E+01_JPRB, 0.36199E+01_JPRB, 0.37004E+01_JPRB, 0.37296E+01_JPRB, 0.37593E+01_JPRB, 0.36982E+01_JPRB ,&
&  0.35092E+01_JPRB, 0.28746E+01_JPRB, 0.26176E+01_JPRB, 0.40241E+01_JPRB, 0.38839E+01_JPRB, 0.30819E+01_JPRB, 0.30861E+01_JPRB ,&
&  0.28437E+01_JPRB, 0.27026E+01_JPRB, 0.23776E+01_JPRB, 0.19388E+01_JPRB, 0.16056E+01_JPRB, 0.13579E+01_JPRB, 0.12180E+01_JPRB ,&
&  0.11092E+01_JPRB, 0.95753E+00_JPRB, 0.96228E+00_JPRB, 0.99190E+00_JPRB, 0.91781E+00_JPRB, 0.91233E+00_JPRB, 0.81298E+00_JPRB ,&
&  0.76769E+00_JPRB, 0.64222E+00_JPRB, 0.69084E+00_JPRB, 0.86937E+00_JPRB, 0.72380E+00_JPRB, 0.44947E+00_JPRB, 0.40762E+00_JPRB ,&
&  0.42168E+00_JPRB, 0.43529E+00_JPRB, 0.44806E+00_JPRB, 0.45908E+00_JPRB, 0.46778E+00_JPRB, 0.48785E+00_JPRB, 0.50566E+00_JPRB ,&
&  0.52482E+00_JPRB, 0.55504E+00_JPRB, 0.56830E+00_JPRB, 0.58293E+00_JPRB, 0.60052E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.54794E+00_JPRB, 0.54503E+00_JPRB, 0.53788E+00_JPRB, 0.53875E+00_JPRB, 0.54412E+00_JPRB, 0.55798E+00_JPRB, 0.56684E+00_JPRB ,&
&  0.56534E+00_JPRB, 0.57018E+00_JPRB, 0.55358E+00_JPRB, 0.55473E+00_JPRB, 0.57022E+00_JPRB, 0.58016E+00_JPRB, 0.58966E+00_JPRB ,&
&  0.62506E+00_JPRB, 0.63859E+00_JPRB, 0.63639E+00_JPRB, 0.64083E+00_JPRB, 0.63539E+00_JPRB, 0.68020E+00_JPRB, 0.68438E+00_JPRB ,&
&  0.66822E+00_JPRB, 0.69371E+00_JPRB, 0.81712E+00_JPRB, 0.11290E+01_JPRB, 0.14137E+01_JPRB, 0.14021E+01_JPRB, 0.15906E+01_JPRB ,&
&  0.14495E+01_JPRB, 0.11607E+01_JPRB, 0.12583E+01_JPRB, 0.11235E+01_JPRB, 0.11529E+01_JPRB, 0.12045E+01_JPRB, 0.12071E+01_JPRB ,&
&  0.12284E+01_JPRB, 0.12236E+01_JPRB, 0.12402E+01_JPRB, 0.12663E+01_JPRB, 0.12794E+01_JPRB, 0.12996E+01_JPRB, 0.13293E+01_JPRB ,&
&  0.12710E+01_JPRB, 0.13108E+01_JPRB, 0.14651E+01_JPRB, 0.17366E+01_JPRB, 0.20434E+01_JPRB, 0.24597E+01_JPRB, 0.26374E+01_JPRB ,&
&  0.27616E+01_JPRB, 0.29050E+01_JPRB, 0.27417E+01_JPRB, 0.28201E+01_JPRB, 0.30462E+01_JPRB, 0.28500E+01_JPRB, 0.27213E+01_JPRB ,&
&  0.25843E+01_JPRB, 0.20348E+01_JPRB, 0.19418E+01_JPRB, 0.28216E+01_JPRB, 0.29435E+01_JPRB, 0.27483E+01_JPRB, 0.27492E+01_JPRB ,&
&  0.26116E+01_JPRB, 0.24113E+01_JPRB, 0.20597E+01_JPRB, 0.17753E+01_JPRB, 0.15300E+01_JPRB, 0.12738E+01_JPRB, 0.11692E+01_JPRB ,&
&  0.96275E+00_JPRB, 0.92770E+00_JPRB, 0.10238E+01_JPRB, 0.10473E+01_JPRB, 0.10019E+01_JPRB, 0.10244E+01_JPRB, 0.81991E+00_JPRB ,&
&  0.68915E+00_JPRB, 0.59989E+00_JPRB, 0.78258E+00_JPRB, 0.10170E+01_JPRB, 0.56741E+00_JPRB, 0.44580E+00_JPRB, 0.39008E+00_JPRB ,&
&  0.40676E+00_JPRB, 0.42537E+00_JPRB, 0.43194E+00_JPRB, 0.44078E+00_JPRB, 0.45810E+00_JPRB, 0.47333E+00_JPRB, 0.49665E+00_JPRB ,&
&  0.51140E+00_JPRB, 0.52315E+00_JPRB, 0.53536E+00_JPRB, 0.54915E+00_JPRB, 0.54524E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.59361E+00_JPRB, 0.60110E+00_JPRB, 0.60677E+00_JPRB, 0.62330E+00_JPRB, 0.64822E+00_JPRB, 0.65353E+00_JPRB, 0.66783E+00_JPRB ,&
&  0.67599E+00_JPRB, 0.69244E+00_JPRB, 0.71009E+00_JPRB, 0.73102E+00_JPRB, 0.74305E+00_JPRB, 0.74759E+00_JPRB, 0.76625E+00_JPRB ,&
&  0.78122E+00_JPRB, 0.77555E+00_JPRB, 0.78442E+00_JPRB, 0.80307E+00_JPRB, 0.80784E+00_JPRB, 0.83651E+00_JPRB, 0.83383E+00_JPRB ,&
&  0.86182E+00_JPRB, 0.86578E+00_JPRB, 0.89333E+00_JPRB, 0.97553E+00_JPRB, 0.10714E+01_JPRB, 0.12652E+01_JPRB, 0.16183E+01_JPRB ,&
&  0.13516E+01_JPRB, 0.10040E+01_JPRB, 0.92953E+00_JPRB, 0.79953E+00_JPRB, 0.79906E+00_JPRB, 0.82487E+00_JPRB, 0.82279E+00_JPRB ,&
&  0.80875E+00_JPRB, 0.77571E+00_JPRB, 0.77139E+00_JPRB, 0.77980E+00_JPRB, 0.82982E+00_JPRB, 0.94127E+00_JPRB, 0.10225E+01_JPRB ,&
&  0.10377E+01_JPRB, 0.11929E+01_JPRB, 0.13856E+01_JPRB, 0.16536E+01_JPRB, 0.18279E+01_JPRB, 0.22995E+01_JPRB, 0.23290E+01_JPRB ,&
&  0.22628E+01_JPRB, 0.21946E+01_JPRB, 0.18745E+01_JPRB, 0.19647E+01_JPRB, 0.21747E+01_JPRB, 0.20996E+01_JPRB, 0.18711E+01_JPRB ,&
&  0.18559E+01_JPRB, 0.17502E+01_JPRB, 0.18012E+01_JPRB, 0.23699E+01_JPRB, 0.25321E+01_JPRB, 0.25536E+01_JPRB, 0.24091E+01_JPRB ,&
&  0.22259E+01_JPRB, 0.19742E+01_JPRB, 0.17347E+01_JPRB, 0.16051E+01_JPRB, 0.15286E+01_JPRB, 0.13405E+01_JPRB, 0.11226E+01_JPRB ,&
&  0.96477E+00_JPRB, 0.10102E+01_JPRB, 0.10870E+01_JPRB, 0.10735E+01_JPRB, 0.11544E+01_JPRB, 0.91670E+00_JPRB, 0.10160E+01_JPRB ,&
&  0.62644E+00_JPRB, 0.58213E+00_JPRB, 0.50495E+00_JPRB, 0.64572E+00_JPRB, 0.51798E+00_JPRB, 0.44446E+00_JPRB, 0.45047E+00_JPRB ,&
&  0.46365E+00_JPRB, 0.47320E+00_JPRB, 0.48787E+00_JPRB, 0.49939E+00_JPRB, 0.51372E+00_JPRB, 0.53795E+00_JPRB, 0.56412E+00_JPRB ,&
&  0.58554E+00_JPRB, 0.57394E+00_JPRB, 0.57466E+00_JPRB, 0.57927E+00_JPRB, 0.59224E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.62960E+00_JPRB, 0.64736E+00_JPRB, 0.66531E+00_JPRB, 0.68362E+00_JPRB, 0.69961E+00_JPRB, 0.71470E+00_JPRB, 0.73043E+00_JPRB ,&
&  0.74611E+00_JPRB, 0.76431E+00_JPRB, 0.78345E+00_JPRB, 0.80407E+00_JPRB, 0.82069E+00_JPRB, 0.82748E+00_JPRB, 0.83869E+00_JPRB ,&
&  0.84637E+00_JPRB, 0.84806E+00_JPRB, 0.84039E+00_JPRB, 0.86157E+00_JPRB, 0.88425E+00_JPRB, 0.89991E+00_JPRB, 0.92424E+00_JPRB ,&
&  0.95240E+00_JPRB, 0.94050E+00_JPRB, 0.93798E+00_JPRB, 0.96274E+00_JPRB, 0.97866E+00_JPRB, 0.93191E+00_JPRB, 0.10215E+01_JPRB ,&
&  0.78189E+00_JPRB, 0.89357E+00_JPRB, 0.82784E+00_JPRB, 0.73034E+00_JPRB, 0.70930E+00_JPRB, 0.63601E+00_JPRB, 0.60135E+00_JPRB ,&
&  0.65464E+00_JPRB, 0.67377E+00_JPRB, 0.69484E+00_JPRB, 0.74668E+00_JPRB, 0.85732E+00_JPRB, 0.10342E+01_JPRB, 0.11673E+01_JPRB ,&
&  0.13440E+01_JPRB, 0.15155E+01_JPRB, 0.17147E+01_JPRB, 0.19353E+01_JPRB, 0.21820E+01_JPRB, 0.23144E+01_JPRB, 0.22766E+01_JPRB ,&
&  0.21657E+01_JPRB, 0.20363E+01_JPRB, 0.15467E+01_JPRB, 0.17417E+01_JPRB, 0.20561E+01_JPRB, 0.20024E+01_JPRB, 0.17041E+01_JPRB ,&
&  0.15316E+01_JPRB, 0.15167E+01_JPRB, 0.17172E+01_JPRB, 0.19227E+01_JPRB, 0.20433E+01_JPRB, 0.20730E+01_JPRB, 0.19540E+01_JPRB ,&
&  0.18200E+01_JPRB, 0.15875E+01_JPRB, 0.14710E+01_JPRB, 0.14964E+01_JPRB, 0.14630E+01_JPRB, 0.13417E+01_JPRB, 0.11005E+01_JPRB ,&
&  0.10148E+01_JPRB, 0.10449E+01_JPRB, 0.10824E+01_JPRB, 0.10557E+01_JPRB, 0.99012E+00_JPRB, 0.12329E+01_JPRB, 0.12830E+01_JPRB ,&
&  0.62786E+00_JPRB, 0.53712E+00_JPRB, 0.41724E+00_JPRB, 0.51616E+00_JPRB, 0.55750E+00_JPRB, 0.56537E+00_JPRB, 0.61707E+00_JPRB ,&
&  0.64973E+00_JPRB, 0.61185E+00_JPRB, 0.60320E+00_JPRB, 0.58053E+00_JPRB, 0.57979E+00_JPRB, 0.57549E+00_JPRB, 0.58836E+00_JPRB ,&
&  0.59583E+00_JPRB, 0.59553E+00_JPRB, 0.60220E+00_JPRB, 0.61171E+00_JPRB, 0.61800E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.65368E+00_JPRB, 0.67263E+00_JPRB, 0.69176E+00_JPRB, 0.70932E+00_JPRB, 0.72626E+00_JPRB, 0.74163E+00_JPRB, 0.75823E+00_JPRB ,&
&  0.77556E+00_JPRB, 0.79394E+00_JPRB, 0.81139E+00_JPRB, 0.83333E+00_JPRB, 0.85462E+00_JPRB, 0.86546E+00_JPRB, 0.88151E+00_JPRB ,&
&  0.88761E+00_JPRB, 0.91824E+00_JPRB, 0.93010E+00_JPRB, 0.95897E+00_JPRB, 0.98405E+00_JPRB, 0.99876E+00_JPRB, 0.10301E+01_JPRB ,&
&  0.10457E+01_JPRB, 0.10562E+01_JPRB, 0.10682E+01_JPRB, 0.10947E+01_JPRB, 0.11047E+01_JPRB, 0.11702E+01_JPRB, 0.95772E+00_JPRB ,&
&  0.95307E+00_JPRB, 0.88539E+00_JPRB, 0.82551E+00_JPRB, 0.77678E+00_JPRB, 0.73646E+00_JPRB, 0.69698E+00_JPRB, 0.68130E+00_JPRB ,&
&  0.70798E+00_JPRB, 0.74323E+00_JPRB, 0.79577E+00_JPRB, 0.87127E+00_JPRB, 0.99907E+00_JPRB, 0.11852E+01_JPRB, 0.13360E+01_JPRB ,&
&  0.15103E+01_JPRB, 0.17105E+01_JPRB, 0.19008E+01_JPRB, 0.21019E+01_JPRB, 0.23287E+01_JPRB, 0.25578E+01_JPRB, 0.27450E+01_JPRB ,&
&  0.27672E+01_JPRB, 0.27143E+01_JPRB, 0.23087E+01_JPRB, 0.25503E+01_JPRB, 0.27252E+01_JPRB, 0.24480E+01_JPRB, 0.18636E+01_JPRB ,&
&  0.14638E+01_JPRB, 0.12320E+01_JPRB, 0.12860E+01_JPRB, 0.15101E+01_JPRB, 0.15807E+01_JPRB, 0.15772E+01_JPRB, 0.14911E+01_JPRB ,&
&  0.13893E+01_JPRB, 0.12491E+01_JPRB, 0.11925E+01_JPRB, 0.12215E+01_JPRB, 0.12805E+01_JPRB, 0.12930E+01_JPRB, 0.11871E+01_JPRB ,&
&  0.11268E+01_JPRB, 0.10813E+01_JPRB, 0.10130E+01_JPRB, 0.89930E+00_JPRB, 0.83234E+00_JPRB, 0.74454E+00_JPRB, 0.81670E+00_JPRB ,&
&  0.56357E+00_JPRB, 0.47693E+00_JPRB, 0.50361E+00_JPRB, 0.55183E+00_JPRB, 0.62233E+00_JPRB, 0.68800E+00_JPRB, 0.85726E+00_JPRB ,&
&  0.99674E+00_JPRB, 0.10469E+01_JPRB, 0.97971E+00_JPRB, 0.80247E+00_JPRB, 0.66391E+00_JPRB, 0.60969E+00_JPRB, 0.60856E+00_JPRB ,&
&  0.60899E+00_JPRB, 0.60515E+00_JPRB, 0.61203E+00_JPRB, 0.62572E+00_JPRB, 0.63751E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.62459E+00_JPRB, 0.64100E+00_JPRB, 0.65984E+00_JPRB, 0.67547E+00_JPRB, 0.69632E+00_JPRB, 0.72657E+00_JPRB, 0.75629E+00_JPRB ,&
&  0.78107E+00_JPRB, 0.79861E+00_JPRB, 0.81022E+00_JPRB, 0.82904E+00_JPRB, 0.85360E+00_JPRB, 0.88675E+00_JPRB, 0.92999E+00_JPRB ,&
&  0.94454E+00_JPRB, 0.98080E+00_JPRB, 0.99682E+00_JPRB, 0.10327E+01_JPRB, 0.10447E+01_JPRB, 0.10890E+01_JPRB, 0.11195E+01_JPRB ,&
&  0.11360E+01_JPRB, 0.11442E+01_JPRB, 0.11522E+01_JPRB, 0.11730E+01_JPRB, 0.11557E+01_JPRB, 0.11884E+01_JPRB, 0.93964E+00_JPRB ,&
&  0.10265E+01_JPRB, 0.98989E+00_JPRB, 0.91800E+00_JPRB, 0.88905E+00_JPRB, 0.86400E+00_JPRB, 0.84703E+00_JPRB, 0.84285E+00_JPRB ,&
&  0.84487E+00_JPRB, 0.86882E+00_JPRB, 0.87040E+00_JPRB, 0.92513E+00_JPRB, 0.10436E+01_JPRB, 0.12233E+01_JPRB, 0.13809E+01_JPRB ,&
&  0.15669E+01_JPRB, 0.17554E+01_JPRB, 0.19519E+01_JPRB, 0.22058E+01_JPRB, 0.25388E+01_JPRB, 0.28902E+01_JPRB, 0.32384E+01_JPRB ,&
&  0.33696E+01_JPRB, 0.34327E+01_JPRB, 0.31429E+01_JPRB, 0.31757E+01_JPRB, 0.32466E+01_JPRB, 0.26576E+01_JPRB, 0.18580E+01_JPRB ,&
&  0.11602E+01_JPRB, 0.10716E+01_JPRB, 0.11749E+01_JPRB, 0.12713E+01_JPRB, 0.11777E+01_JPRB, 0.11039E+01_JPRB, 0.10067E+01_JPRB ,&
&  0.91762E+00_JPRB, 0.84407E+00_JPRB, 0.84995E+00_JPRB, 0.92247E+00_JPRB, 0.10125E+01_JPRB, 0.11105E+01_JPRB, 0.11352E+01_JPRB ,&
&  0.10996E+01_JPRB, 0.10034E+01_JPRB, 0.89869E+00_JPRB, 0.79956E+00_JPRB, 0.70705E+00_JPRB, 0.63156E+00_JPRB, 0.69584E+00_JPRB ,&
&  0.59881E+00_JPRB, 0.57182E+00_JPRB, 0.60079E+00_JPRB, 0.50757E+00_JPRB, 0.69084E+00_JPRB, 0.85430E+00_JPRB, 0.11012E+01_JPRB ,&
&  0.12304E+01_JPRB, 0.16713E+01_JPRB, 0.20917E+01_JPRB, 0.14962E+01_JPRB, 0.11087E+01_JPRB, 0.68250E+00_JPRB, 0.61040E+00_JPRB ,&
&  0.60863E+00_JPRB, 0.59379E+00_JPRB, 0.59282E+00_JPRB, 0.60150E+00_JPRB, 0.61123E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.58982E+00_JPRB, 0.60032E+00_JPRB, 0.61848E+00_JPRB, 0.61982E+00_JPRB, 0.62765E+00_JPRB, 0.64545E+00_JPRB, 0.66336E+00_JPRB ,&
&  0.68228E+00_JPRB, 0.69163E+00_JPRB, 0.70362E+00_JPRB, 0.71823E+00_JPRB, 0.73598E+00_JPRB, 0.77906E+00_JPRB, 0.84426E+00_JPRB ,&
&  0.89255E+00_JPRB, 0.93801E+00_JPRB, 0.99700E+00_JPRB, 0.10144E+01_JPRB, 0.10507E+01_JPRB, 0.11026E+01_JPRB, 0.11321E+01_JPRB ,&
&  0.11808E+01_JPRB, 0.12053E+01_JPRB, 0.12300E+01_JPRB, 0.12906E+01_JPRB, 0.12538E+01_JPRB, 0.12764E+01_JPRB, 0.99232E+00_JPRB ,&
&  0.10564E+01_JPRB, 0.10397E+01_JPRB, 0.99813E+00_JPRB, 0.98063E+00_JPRB, 0.95826E+00_JPRB, 0.92135E+00_JPRB, 0.89400E+00_JPRB ,&
&  0.88295E+00_JPRB, 0.85020E+00_JPRB, 0.81782E+00_JPRB, 0.87512E+00_JPRB, 0.96089E+00_JPRB, 0.11071E+01_JPRB, 0.12977E+01_JPRB ,&
&  0.14981E+01_JPRB, 0.16871E+01_JPRB, 0.18824E+01_JPRB, 0.21671E+01_JPRB, 0.25428E+01_JPRB, 0.29409E+01_JPRB, 0.33601E+01_JPRB ,&
&  0.36459E+01_JPRB, 0.38608E+01_JPRB, 0.39186E+01_JPRB, 0.33210E+01_JPRB, 0.31396E+01_JPRB, 0.23930E+01_JPRB, 0.15743E+01_JPRB ,&
&  0.11578E+01_JPRB, 0.10351E+01_JPRB, 0.10540E+01_JPRB, 0.10642E+01_JPRB, 0.91936E+00_JPRB, 0.82566E+00_JPRB, 0.72835E+00_JPRB ,&
&  0.66040E+00_JPRB, 0.64466E+00_JPRB, 0.67023E+00_JPRB, 0.74550E+00_JPRB, 0.83211E+00_JPRB, 0.93230E+00_JPRB, 0.97115E+00_JPRB ,&
&  0.93477E+00_JPRB, 0.84640E+00_JPRB, 0.76719E+00_JPRB, 0.69944E+00_JPRB, 0.67641E+00_JPRB, 0.69749E+00_JPRB, 0.64497E+00_JPRB ,&
&  0.63088E+00_JPRB, 0.57734E+00_JPRB, 0.57563E+00_JPRB, 0.63746E+00_JPRB, 0.72916E+00_JPRB, 0.95014E+00_JPRB, 0.13553E+01_JPRB ,&
&  0.19490E+01_JPRB, 0.20919E+01_JPRB, 0.25795E+01_JPRB, 0.34470E+01_JPRB, 0.25380E+01_JPRB, 0.17852E+01_JPRB, 0.55440E+00_JPRB ,&
&  0.56869E+00_JPRB, 0.56451E+00_JPRB, 0.56398E+00_JPRB, 0.56341E+00_JPRB, 0.57118E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.49751E+00_JPRB, 0.51779E+00_JPRB, 0.50808E+00_JPRB, 0.51069E+00_JPRB, 0.51705E+00_JPRB, 0.52747E+00_JPRB, 0.54967E+00_JPRB ,&
&  0.55459E+00_JPRB, 0.58369E+00_JPRB, 0.61382E+00_JPRB, 0.63749E+00_JPRB, 0.67123E+00_JPRB, 0.71872E+00_JPRB, 0.77782E+00_JPRB ,&
&  0.83800E+00_JPRB, 0.87266E+00_JPRB, 0.90118E+00_JPRB, 0.92077E+00_JPRB, 0.95869E+00_JPRB, 0.10120E+01_JPRB, 0.10390E+01_JPRB ,&
&  0.11039E+01_JPRB, 0.11491E+01_JPRB, 0.12052E+01_JPRB, 0.12869E+01_JPRB, 0.12306E+01_JPRB, 0.12244E+01_JPRB, 0.89554E+00_JPRB ,&
&  0.10190E+01_JPRB, 0.10309E+01_JPRB, 0.10218E+01_JPRB, 0.10062E+01_JPRB, 0.97148E+00_JPRB, 0.91302E+00_JPRB, 0.87748E+00_JPRB ,&
&  0.85051E+00_JPRB, 0.82188E+00_JPRB, 0.78814E+00_JPRB, 0.77702E+00_JPRB, 0.87798E+00_JPRB, 0.96067E+00_JPRB, 0.11331E+01_JPRB ,&
&  0.13408E+01_JPRB, 0.15734E+01_JPRB, 0.17971E+01_JPRB, 0.20707E+01_JPRB, 0.24280E+01_JPRB, 0.28374E+01_JPRB, 0.32555E+01_JPRB ,&
&  0.35671E+01_JPRB, 0.38286E+01_JPRB, 0.37890E+01_JPRB, 0.31342E+01_JPRB, 0.27628E+01_JPRB, 0.20692E+01_JPRB, 0.13552E+01_JPRB ,&
&  0.10820E+01_JPRB, 0.10062E+01_JPRB, 0.10228E+01_JPRB, 0.97567E+00_JPRB, 0.81407E+00_JPRB, 0.70006E+00_JPRB, 0.61002E+00_JPRB ,&
&  0.54667E+00_JPRB, 0.54312E+00_JPRB, 0.59665E+00_JPRB, 0.66597E+00_JPRB, 0.73615E+00_JPRB, 0.80437E+00_JPRB, 0.80402E+00_JPRB ,&
&  0.76116E+00_JPRB, 0.68897E+00_JPRB, 0.63979E+00_JPRB, 0.63907E+00_JPRB, 0.71212E+00_JPRB, 0.79969E+00_JPRB, 0.87036E+00_JPRB ,&
&  0.67903E+00_JPRB, 0.56011E+00_JPRB, 0.57368E+00_JPRB, 0.60876E+00_JPRB, 0.66061E+00_JPRB, 0.73737E+00_JPRB, 0.85741E+00_JPRB ,&
&  0.10326E+01_JPRB, 0.13911E+01_JPRB, 0.13625E+01_JPRB, 0.11272E+01_JPRB, 0.16392E+01_JPRB, 0.13615E+01_JPRB, 0.50444E+00_JPRB ,&
&  0.48459E+00_JPRB, 0.47631E+00_JPRB, 0.48077E+00_JPRB, 0.48366E+00_JPRB, 0.49472E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.46422E+00_JPRB, 0.45246E+00_JPRB, 0.43470E+00_JPRB, 0.44255E+00_JPRB, 0.44390E+00_JPRB, 0.45999E+00_JPRB, 0.47840E+00_JPRB ,&
&  0.50330E+00_JPRB, 0.55064E+00_JPRB, 0.59479E+00_JPRB, 0.62985E+00_JPRB, 0.64983E+00_JPRB, 0.67524E+00_JPRB, 0.70022E+00_JPRB ,&
&  0.71537E+00_JPRB, 0.73132E+00_JPRB, 0.75349E+00_JPRB, 0.78678E+00_JPRB, 0.83026E+00_JPRB, 0.87966E+00_JPRB, 0.94042E+00_JPRB ,&
&  0.10076E+01_JPRB, 0.10804E+01_JPRB, 0.11735E+01_JPRB, 0.12025E+01_JPRB, 0.12295E+01_JPRB, 0.12632E+01_JPRB, 0.12416E+01_JPRB ,&
&  0.89467E+00_JPRB, 0.10554E+01_JPRB, 0.10324E+01_JPRB, 0.99650E+00_JPRB, 0.96869E+00_JPRB, 0.92311E+00_JPRB, 0.88619E+00_JPRB ,&
&  0.85996E+00_JPRB, 0.80642E+00_JPRB, 0.78483E+00_JPRB, 0.75071E+00_JPRB, 0.79557E+00_JPRB, 0.83975E+00_JPRB, 0.95330E+00_JPRB ,&
&  0.11243E+01_JPRB, 0.13718E+01_JPRB, 0.16714E+01_JPRB, 0.19575E+01_JPRB, 0.22403E+01_JPRB, 0.25522E+01_JPRB, 0.29967E+01_JPRB ,&
&  0.34742E+01_JPRB, 0.38009E+01_JPRB, 0.39973E+01_JPRB, 0.34433E+01_JPRB, 0.27167E+01_JPRB, 0.23330E+01_JPRB, 0.17130E+01_JPRB ,&
&  0.11183E+01_JPRB, 0.91591E+00_JPRB, 0.94247E+00_JPRB, 0.93674E+00_JPRB, 0.79599E+00_JPRB, 0.69484E+00_JPRB, 0.60301E+00_JPRB ,&
&  0.53083E+00_JPRB, 0.49052E+00_JPRB, 0.51201E+00_JPRB, 0.59508E+00_JPRB, 0.64232E+00_JPRB, 0.66687E+00_JPRB, 0.65051E+00_JPRB ,&
&  0.62813E+00_JPRB, 0.57934E+00_JPRB, 0.53221E+00_JPRB, 0.53161E+00_JPRB, 0.56611E+00_JPRB, 0.60173E+00_JPRB, 0.60176E+00_JPRB ,&
&  0.58606E+00_JPRB, 0.58059E+00_JPRB, 0.58787E+00_JPRB, 0.60827E+00_JPRB, 0.62649E+00_JPRB, 0.65133E+00_JPRB, 0.67673E+00_JPRB ,&
&  0.71034E+00_JPRB, 0.75639E+00_JPRB, 0.74548E+00_JPRB, 0.73577E+00_JPRB, 0.82007E+00_JPRB, 0.87283E+00_JPRB, 0.56322E+00_JPRB ,&
&  0.48075E+00_JPRB, 0.44814E+00_JPRB, 0.44682E+00_JPRB, 0.44063E+00_JPRB, 0.47266E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.46697E+00_JPRB, 0.44914E+00_JPRB, 0.43833E+00_JPRB, 0.42994E+00_JPRB, 0.41827E+00_JPRB, 0.43995E+00_JPRB, 0.44428E+00_JPRB ,&
&  0.46602E+00_JPRB, 0.50302E+00_JPRB, 0.54728E+00_JPRB, 0.57073E+00_JPRB, 0.57602E+00_JPRB, 0.59831E+00_JPRB, 0.61697E+00_JPRB ,&
&  0.62741E+00_JPRB, 0.63411E+00_JPRB, 0.64525E+00_JPRB, 0.66045E+00_JPRB, 0.68847E+00_JPRB, 0.73177E+00_JPRB, 0.79677E+00_JPRB ,&
&  0.87067E+00_JPRB, 0.94888E+00_JPRB, 0.10318E+01_JPRB, 0.11081E+01_JPRB, 0.11570E+01_JPRB, 0.11961E+01_JPRB, 0.11630E+01_JPRB ,&
&  0.72122E+00_JPRB, 0.95707E+00_JPRB, 0.10927E+01_JPRB, 0.10329E+01_JPRB, 0.97938E+00_JPRB, 0.93476E+00_JPRB, 0.89220E+00_JPRB ,&
&  0.84633E+00_JPRB, 0.79498E+00_JPRB, 0.77908E+00_JPRB, 0.79648E+00_JPRB, 0.73644E+00_JPRB, 0.74998E+00_JPRB, 0.81043E+00_JPRB ,&
&  0.91927E+00_JPRB, 0.11018E+01_JPRB, 0.14339E+01_JPRB, 0.17917E+01_JPRB, 0.19916E+01_JPRB, 0.21877E+01_JPRB, 0.25545E+01_JPRB ,&
&  0.31128E+01_JPRB, 0.35401E+01_JPRB, 0.38459E+01_JPRB, 0.28891E+01_JPRB, 0.24428E+01_JPRB, 0.23597E+01_JPRB, 0.21218E+01_JPRB ,&
&  0.12953E+01_JPRB, 0.92893E+00_JPRB, 0.94226E+00_JPRB, 0.94952E+00_JPRB, 0.81798E+00_JPRB, 0.74320E+00_JPRB, 0.66130E+00_JPRB ,&
&  0.56971E+00_JPRB, 0.50767E+00_JPRB, 0.46897E+00_JPRB, 0.49663E+00_JPRB, 0.52846E+00_JPRB, 0.54443E+00_JPRB, 0.52240E+00_JPRB ,&
&  0.54290E+00_JPRB, 0.52133E+00_JPRB, 0.50879E+00_JPRB, 0.51483E+00_JPRB, 0.56435E+00_JPRB, 0.58515E+00_JPRB, 0.59326E+00_JPRB ,&
&  0.60586E+00_JPRB, 0.61545E+00_JPRB, 0.61838E+00_JPRB, 0.62909E+00_JPRB, 0.63747E+00_JPRB, 0.63917E+00_JPRB, 0.63836E+00_JPRB ,&
&  0.63316E+00_JPRB, 0.63746E+00_JPRB, 0.62712E+00_JPRB, 0.61334E+00_JPRB, 0.58861E+00_JPRB, 0.60478E+00_JPRB, 0.54105E+00_JPRB ,&
&  0.49743E+00_JPRB, 0.46599E+00_JPRB, 0.44356E+00_JPRB, 0.43844E+00_JPRB, 0.44622E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.47501E+00_JPRB, 0.46974E+00_JPRB, 0.46266E+00_JPRB, 0.45265E+00_JPRB, 0.44179E+00_JPRB, 0.44143E+00_JPRB, 0.42906E+00_JPRB ,&
&  0.42565E+00_JPRB, 0.44558E+00_JPRB, 0.47596E+00_JPRB, 0.50096E+00_JPRB, 0.52281E+00_JPRB, 0.53375E+00_JPRB, 0.55136E+00_JPRB ,&
&  0.55287E+00_JPRB, 0.55055E+00_JPRB, 0.54722E+00_JPRB, 0.54529E+00_JPRB, 0.55147E+00_JPRB, 0.57536E+00_JPRB, 0.61831E+00_JPRB ,&
&  0.67872E+00_JPRB, 0.75783E+00_JPRB, 0.85324E+00_JPRB, 0.95358E+00_JPRB, 0.10472E+01_JPRB, 0.11357E+01_JPRB, 0.11693E+01_JPRB ,&
&  0.81800E+00_JPRB, 0.49244E+00_JPRB, 0.98263E+00_JPRB, 0.11043E+01_JPRB, 0.10348E+01_JPRB, 0.99724E+00_JPRB, 0.91967E+00_JPRB ,&
&  0.84170E+00_JPRB, 0.81969E+00_JPRB, 0.78444E+00_JPRB, 0.76493E+00_JPRB, 0.70674E+00_JPRB, 0.70091E+00_JPRB, 0.71549E+00_JPRB ,&
&  0.77099E+00_JPRB, 0.88948E+00_JPRB, 0.11262E+01_JPRB, 0.15193E+01_JPRB, 0.17832E+01_JPRB, 0.18886E+01_JPRB, 0.21259E+01_JPRB ,&
&  0.26014E+01_JPRB, 0.30092E+01_JPRB, 0.31710E+01_JPRB, 0.21357E+01_JPRB, 0.14868E+01_JPRB, 0.13231E+01_JPRB, 0.12294E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.97444E+00_JPRB, 0.95952E+00_JPRB, 0.97929E+00_JPRB, 0.88688E+00_JPRB, 0.77611E+00_JPRB, 0.75973E+00_JPRB ,&
&  0.67622E+00_JPRB, 0.60113E+00_JPRB, 0.53980E+00_JPRB, 0.50885E+00_JPRB, 0.49636E+00_JPRB, 0.50524E+00_JPRB, 0.50143E+00_JPRB ,&
&  0.49954E+00_JPRB, 0.50897E+00_JPRB, 0.50869E+00_JPRB, 0.51671E+00_JPRB, 0.54672E+00_JPRB, 0.57609E+00_JPRB, 0.59489E+00_JPRB ,&
&  0.60643E+00_JPRB, 0.62238E+00_JPRB, 0.63387E+00_JPRB, 0.64022E+00_JPRB, 0.64276E+00_JPRB, 0.63132E+00_JPRB, 0.63724E+00_JPRB ,&
&  0.64695E+00_JPRB, 0.65477E+00_JPRB, 0.64465E+00_JPRB, 0.63812E+00_JPRB, 0.60762E+00_JPRB, 0.58736E+00_JPRB, 0.53106E+00_JPRB ,&
&  0.50874E+00_JPRB, 0.47224E+00_JPRB, 0.44597E+00_JPRB, 0.43893E+00_JPRB, 0.44879E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.47568E+00_JPRB, 0.47509E+00_JPRB, 0.47439E+00_JPRB, 0.47750E+00_JPRB, 0.47373E+00_JPRB, 0.47596E+00_JPRB, 0.46082E+00_JPRB ,&
&  0.43602E+00_JPRB, 0.42769E+00_JPRB, 0.44056E+00_JPRB, 0.44534E+00_JPRB, 0.44756E+00_JPRB, 0.46395E+00_JPRB, 0.48151E+00_JPRB ,&
&  0.48834E+00_JPRB, 0.49000E+00_JPRB, 0.48769E+00_JPRB, 0.48623E+00_JPRB, 0.47964E+00_JPRB, 0.48168E+00_JPRB, 0.49876E+00_JPRB ,&
&  0.52595E+00_JPRB, 0.57539E+00_JPRB, 0.64897E+00_JPRB, 0.73890E+00_JPRB, 0.83490E+00_JPRB, 0.96155E+00_JPRB, 0.10691E+01_JPRB ,&
&  0.12707E+01_JPRB, 0.88958E+00_JPRB, 0.58826E+00_JPRB, 0.98213E+00_JPRB, 0.11143E+01_JPRB, 0.11159E+01_JPRB, 0.97155E+00_JPRB ,&
&  0.92772E+00_JPRB, 0.89791E+00_JPRB, 0.87547E+00_JPRB, 0.82460E+00_JPRB, 0.73341E+00_JPRB, 0.69482E+00_JPRB, 0.66244E+00_JPRB ,&
&  0.66038E+00_JPRB, 0.71510E+00_JPRB, 0.85007E+00_JPRB, 0.11255E+01_JPRB, 0.14734E+01_JPRB, 0.16553E+01_JPRB, 0.17625E+01_JPRB ,&
&  0.20881E+01_JPRB, 0.25058E+01_JPRB, 0.26000E+01_JPRB, 0.17328E+01_JPRB, 0.12733E+01_JPRB, 0.11533E+01_JPRB, 0.11286E+01_JPRB ,&
&  0.10807E+01_JPRB, 0.10891E+01_JPRB, 0.11124E+01_JPRB, 0.10215E+01_JPRB, 0.91594E+00_JPRB, 0.83706E+00_JPRB, 0.83927E+00_JPRB ,&
&  0.75972E+00_JPRB, 0.69164E+00_JPRB, 0.63226E+00_JPRB, 0.57030E+00_JPRB, 0.51940E+00_JPRB, 0.51267E+00_JPRB, 0.50429E+00_JPRB ,&
&  0.50594E+00_JPRB, 0.51919E+00_JPRB, 0.51791E+00_JPRB, 0.53155E+00_JPRB, 0.54603E+00_JPRB, 0.56235E+00_JPRB, 0.57765E+00_JPRB ,&
&  0.58989E+00_JPRB, 0.60545E+00_JPRB, 0.62138E+00_JPRB, 0.62521E+00_JPRB, 0.62061E+00_JPRB, 0.61251E+00_JPRB, 0.62662E+00_JPRB ,&
&  0.63719E+00_JPRB, 0.65537E+00_JPRB, 0.65757E+00_JPRB, 0.66704E+00_JPRB, 0.64587E+00_JPRB, 0.62652E+00_JPRB, 0.58810E+00_JPRB ,&
&  0.58413E+00_JPRB, 0.55052E+00_JPRB, 0.51787E+00_JPRB, 0.47235E+00_JPRB, 0.47093E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.49177E+00_JPRB, 0.48348E+00_JPRB, 0.48163E+00_JPRB, 0.48338E+00_JPRB, 0.48272E+00_JPRB, 0.48534E+00_JPRB, 0.48766E+00_JPRB ,&
&  0.46505E+00_JPRB, 0.44279E+00_JPRB, 0.42658E+00_JPRB, 0.41901E+00_JPRB, 0.41640E+00_JPRB, 0.42831E+00_JPRB, 0.43286E+00_JPRB ,&
&  0.42990E+00_JPRB, 0.42980E+00_JPRB, 0.44533E+00_JPRB, 0.44879E+00_JPRB, 0.44797E+00_JPRB, 0.44445E+00_JPRB, 0.44440E+00_JPRB ,&
&  0.45398E+00_JPRB, 0.47403E+00_JPRB, 0.51355E+00_JPRB, 0.56961E+00_JPRB, 0.62696E+00_JPRB, 0.71778E+00_JPRB, 0.82300E+00_JPRB ,&
&  0.10028E+01_JPRB, 0.10159E+01_JPRB, 0.54001E+00_JPRB, 0.86894E+00_JPRB, 0.11463E+01_JPRB, 0.11800E+01_JPRB, 0.11084E+01_JPRB ,&
&  0.10419E+01_JPRB, 0.98687E+00_JPRB, 0.10418E+01_JPRB, 0.98899E+00_JPRB, 0.82353E+00_JPRB, 0.71976E+00_JPRB, 0.64481E+00_JPRB ,&
&  0.60560E+00_JPRB, 0.60720E+00_JPRB, 0.65887E+00_JPRB, 0.79732E+00_JPRB, 0.10709E+01_JPRB, 0.13271E+01_JPRB, 0.14559E+01_JPRB ,&
&  0.16117E+01_JPRB, 0.20080E+01_JPRB, 0.22780E+01_JPRB, 0.15850E+01_JPRB, 0.12747E+01_JPRB, 0.12942E+01_JPRB, 0.12774E+01_JPRB ,&
&  0.12186E+01_JPRB, 0.11955E+01_JPRB, 0.11709E+01_JPRB, 0.10751E+01_JPRB, 0.96042E+00_JPRB, 0.90916E+00_JPRB, 0.91327E+00_JPRB ,&
&  0.83719E+00_JPRB, 0.76905E+00_JPRB, 0.70825E+00_JPRB, 0.64836E+00_JPRB, 0.58268E+00_JPRB, 0.53497E+00_JPRB, 0.53297E+00_JPRB ,&
&  0.53337E+00_JPRB, 0.54548E+00_JPRB, 0.54384E+00_JPRB, 0.55144E+00_JPRB, 0.55901E+00_JPRB, 0.56679E+00_JPRB, 0.57183E+00_JPRB ,&
&  0.57955E+00_JPRB, 0.58988E+00_JPRB, 0.60237E+00_JPRB, 0.60199E+00_JPRB, 0.60101E+00_JPRB, 0.60003E+00_JPRB, 0.60437E+00_JPRB ,&
&  0.61499E+00_JPRB, 0.62753E+00_JPRB, 0.63315E+00_JPRB, 0.64134E+00_JPRB, 0.66749E+00_JPRB, 0.67115E+00_JPRB, 0.65968E+00_JPRB ,&
&  0.70625E+00_JPRB, 0.74651E+00_JPRB, 0.68956E+00_JPRB, 0.54105E+00_JPRB, 0.51006E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.51857E+00_JPRB, 0.50650E+00_JPRB, 0.49776E+00_JPRB, 0.49627E+00_JPRB, 0.49346E+00_JPRB, 0.49256E+00_JPRB, 0.50040E+00_JPRB ,&
&  0.49376E+00_JPRB, 0.47990E+00_JPRB, 0.45963E+00_JPRB, 0.43872E+00_JPRB, 0.42208E+00_JPRB, 0.42306E+00_JPRB, 0.42790E+00_JPRB ,&
&  0.42825E+00_JPRB, 0.42591E+00_JPRB, 0.42589E+00_JPRB, 0.42959E+00_JPRB, 0.42821E+00_JPRB, 0.42569E+00_JPRB, 0.42291E+00_JPRB ,&
&  0.42192E+00_JPRB, 0.42645E+00_JPRB, 0.44309E+00_JPRB, 0.47184E+00_JPRB, 0.50503E+00_JPRB, 0.55451E+00_JPRB, 0.62172E+00_JPRB ,&
&  0.76849E+00_JPRB, 0.85351E+00_JPRB, 0.92897E+00_JPRB, 0.13855E+01_JPRB, 0.14560E+01_JPRB, 0.13512E+01_JPRB, 0.12972E+01_JPRB ,&
&  0.12409E+01_JPRB, 0.11864E+01_JPRB, 0.12916E+01_JPRB, 0.10899E+01_JPRB, 0.91038E+00_JPRB, 0.76002E+00_JPRB, 0.66004E+00_JPRB ,&
&  0.60384E+00_JPRB, 0.56880E+00_JPRB, 0.56892E+00_JPRB, 0.60689E+00_JPRB, 0.74443E+00_JPRB, 0.98463E+00_JPRB, 0.11759E+01_JPRB ,&
&  0.13170E+01_JPRB, 0.15831E+01_JPRB, 0.19729E+01_JPRB, 0.16662E+01_JPRB, 0.13541E+01_JPRB, 0.15413E+01_JPRB, 0.17286E+01_JPRB ,&
&  0.19470E+01_JPRB, 0.14673E+01_JPRB, 0.12803E+01_JPRB, 0.11555E+01_JPRB, 0.10098E+01_JPRB, 0.10881E+01_JPRB, 0.97821E+00_JPRB ,&
&  0.90166E+00_JPRB, 0.84219E+00_JPRB, 0.78041E+00_JPRB, 0.72267E+00_JPRB, 0.66693E+00_JPRB, 0.61307E+00_JPRB, 0.58358E+00_JPRB ,&
&  0.57949E+00_JPRB, 0.58283E+00_JPRB, 0.57953E+00_JPRB, 0.58224E+00_JPRB, 0.57959E+00_JPRB, 0.58404E+00_JPRB, 0.58785E+00_JPRB ,&
&  0.59277E+00_JPRB, 0.59225E+00_JPRB, 0.57649E+00_JPRB, 0.57727E+00_JPRB, 0.57962E+00_JPRB, 0.57841E+00_JPRB, 0.57507E+00_JPRB ,&
&  0.58342E+00_JPRB, 0.59987E+00_JPRB, 0.61094E+00_JPRB, 0.62189E+00_JPRB, 0.63778E+00_JPRB, 0.64319E+00_JPRB, 0.63006E+00_JPRB ,&
&  0.62538E+00_JPRB, 0.63612E+00_JPRB, 0.63634E+00_JPRB, 0.57028E+00_JPRB, 0.54044E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.52833E+00_JPRB, 0.51777E+00_JPRB, 0.51090E+00_JPRB, 0.50391E+00_JPRB, 0.49653E+00_JPRB, 0.49573E+00_JPRB, 0.50167E+00_JPRB ,&
&  0.50228E+00_JPRB, 0.49372E+00_JPRB, 0.47989E+00_JPRB, 0.46599E+00_JPRB, 0.44655E+00_JPRB, 0.43546E+00_JPRB, 0.42936E+00_JPRB ,&
&  0.42497E+00_JPRB, 0.42453E+00_JPRB, 0.42068E+00_JPRB, 0.41883E+00_JPRB, 0.41681E+00_JPRB, 0.40929E+00_JPRB, 0.40565E+00_JPRB ,&
&  0.40271E+00_JPRB, 0.40583E+00_JPRB, 0.41399E+00_JPRB, 0.42977E+00_JPRB, 0.45300E+00_JPRB, 0.48673E+00_JPRB, 0.52731E+00_JPRB ,&
&  0.62548E+00_JPRB, 0.77250E+00_JPRB, 0.77870E+00_JPRB, 0.14864E+01_JPRB, 0.14615E+01_JPRB, 0.14104E+01_JPRB, 0.13079E+01_JPRB ,&
&  0.12925E+01_JPRB, 0.14166E+01_JPRB, 0.13074E+01_JPRB, 0.11236E+01_JPRB, 0.92830E+00_JPRB, 0.78752E+00_JPRB, 0.68123E+00_JPRB ,&
&  0.61517E+00_JPRB, 0.57446E+00_JPRB, 0.55543E+00_JPRB, 0.54939E+00_JPRB, 0.56649E+00_JPRB, 0.70071E+00_JPRB, 0.91455E+00_JPRB ,&
&  0.10777E+01_JPRB, 0.12734E+01_JPRB, 0.16349E+01_JPRB, 0.15638E+01_JPRB, 0.14601E+01_JPRB, 0.17204E+01_JPRB, 0.22212E+01_JPRB ,&
&  0.26775E+01_JPRB, 0.20515E+01_JPRB, 0.15250E+01_JPRB, 0.13023E+01_JPRB, 0.11804E+01_JPRB, 0.11515E+01_JPRB, 0.10546E+01_JPRB ,&
&  0.94956E+00_JPRB, 0.89178E+00_JPRB, 0.84137E+00_JPRB, 0.79255E+00_JPRB, 0.73952E+00_JPRB, 0.69372E+00_JPRB, 0.64949E+00_JPRB ,&
&  0.62855E+00_JPRB, 0.61978E+00_JPRB, 0.61438E+00_JPRB, 0.61894E+00_JPRB, 0.61874E+00_JPRB, 0.62351E+00_JPRB, 0.62252E+00_JPRB ,&
&  0.62412E+00_JPRB, 0.61366E+00_JPRB, 0.57658E+00_JPRB, 0.56476E+00_JPRB, 0.56519E+00_JPRB, 0.56250E+00_JPRB, 0.55758E+00_JPRB ,&
&  0.56948E+00_JPRB, 0.58220E+00_JPRB, 0.58651E+00_JPRB, 0.58973E+00_JPRB, 0.61242E+00_JPRB, 0.62791E+00_JPRB, 0.62299E+00_JPRB ,&
&  0.60818E+00_JPRB, 0.59492E+00_JPRB, 0.57993E+00_JPRB, 0.55325E+00_JPRB, 0.54168E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.53029E+00_JPRB, 0.52281E+00_JPRB, 0.51571E+00_JPRB, 0.50664E+00_JPRB, 0.49462E+00_JPRB, 0.48898E+00_JPRB, 0.49335E+00_JPRB ,&
&  0.49990E+00_JPRB, 0.49533E+00_JPRB, 0.48620E+00_JPRB, 0.47212E+00_JPRB, 0.45276E+00_JPRB, 0.44302E+00_JPRB, 0.43666E+00_JPRB ,&
&  0.42659E+00_JPRB, 0.41878E+00_JPRB, 0.40766E+00_JPRB, 0.40274E+00_JPRB, 0.40189E+00_JPRB, 0.39705E+00_JPRB, 0.38753E+00_JPRB ,&
&  0.38972E+00_JPRB, 0.39723E+00_JPRB, 0.40653E+00_JPRB, 0.41576E+00_JPRB, 0.43380E+00_JPRB, 0.46094E+00_JPRB, 0.50008E+00_JPRB ,&
&  0.57379E+00_JPRB, 0.83934E+00_JPRB, 0.71919E+00_JPRB, 0.11619E+01_JPRB, 0.11899E+01_JPRB, 0.12618E+01_JPRB, 0.12731E+01_JPRB ,&
&  0.12968E+01_JPRB, 0.13326E+01_JPRB, 0.12629E+01_JPRB, 0.10650E+01_JPRB, 0.87412E+00_JPRB, 0.75642E+00_JPRB, 0.68289E+00_JPRB ,&
&  0.63446E+00_JPRB, 0.59692E+00_JPRB, 0.56966E+00_JPRB, 0.55061E+00_JPRB, 0.53703E+00_JPRB, 0.55398E+00_JPRB, 0.66525E+00_JPRB ,&
&  0.85158E+00_JPRB, 0.10239E+01_JPRB, 0.12993E+01_JPRB, 0.15907E+01_JPRB, 0.14658E+01_JPRB, 0.16479E+01_JPRB, 0.23906E+01_JPRB ,&
&  0.28772E+01_JPRB, 0.28877E+01_JPRB, 0.19554E+01_JPRB, 0.15481E+01_JPRB, 0.13803E+01_JPRB, 0.12753E+01_JPRB, 0.11188E+01_JPRB ,&
&  0.97767E+00_JPRB, 0.91079E+00_JPRB, 0.87332E+00_JPRB, 0.83983E+00_JPRB, 0.79834E+00_JPRB, 0.74232E+00_JPRB, 0.68920E+00_JPRB ,&
&  0.64561E+00_JPRB, 0.63366E+00_JPRB, 0.63228E+00_JPRB, 0.63592E+00_JPRB, 0.64420E+00_JPRB, 0.65245E+00_JPRB, 0.65714E+00_JPRB ,&
&  0.65361E+00_JPRB, 0.64473E+00_JPRB, 0.59963E+00_JPRB, 0.57629E+00_JPRB, 0.57005E+00_JPRB, 0.56511E+00_JPRB, 0.55891E+00_JPRB ,&
&  0.56821E+00_JPRB, 0.57451E+00_JPRB, 0.57464E+00_JPRB, 0.58292E+00_JPRB, 0.59943E+00_JPRB, 0.62376E+00_JPRB, 0.59919E+00_JPRB ,&
&  0.60065E+00_JPRB, 0.58751E+00_JPRB, 0.57185E+00_JPRB, 0.55128E+00_JPRB, 0.54147E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.53004E+00_JPRB, 0.52367E+00_JPRB, 0.51679E+00_JPRB, 0.50920E+00_JPRB, 0.49988E+00_JPRB, 0.49264E+00_JPRB, 0.49004E+00_JPRB ,&
&  0.49056E+00_JPRB, 0.49296E+00_JPRB, 0.48784E+00_JPRB, 0.47292E+00_JPRB, 0.45609E+00_JPRB, 0.44097E+00_JPRB, 0.43292E+00_JPRB ,&
&  0.42642E+00_JPRB, 0.41777E+00_JPRB, 0.40695E+00_JPRB, 0.39613E+00_JPRB, 0.39308E+00_JPRB, 0.39471E+00_JPRB, 0.38981E+00_JPRB ,&
&  0.39538E+00_JPRB, 0.40274E+00_JPRB, 0.40974E+00_JPRB, 0.41643E+00_JPRB, 0.43229E+00_JPRB, 0.45760E+00_JPRB, 0.49463E+00_JPRB ,&
&  0.56276E+00_JPRB, 0.48614E+00_JPRB, 0.78372E+00_JPRB, 0.90738E+00_JPRB, 0.98055E+00_JPRB, 0.10470E+01_JPRB, 0.11812E+01_JPRB ,&
&  0.12903E+01_JPRB, 0.12532E+01_JPRB, 0.11491E+01_JPRB, 0.99065E+00_JPRB, 0.84721E+00_JPRB, 0.74368E+00_JPRB, 0.67748E+00_JPRB ,&
&  0.63646E+00_JPRB, 0.60531E+00_JPRB, 0.57727E+00_JPRB, 0.55301E+00_JPRB, 0.53625E+00_JPRB, 0.52804E+00_JPRB, 0.54309E+00_JPRB ,&
&  0.61748E+00_JPRB, 0.75592E+00_JPRB, 0.94410E+00_JPRB, 0.12776E+01_JPRB, 0.14501E+01_JPRB, 0.15691E+01_JPRB, 0.18520E+01_JPRB ,&
&  0.24727E+01_JPRB, 0.28439E+01_JPRB, 0.22703E+01_JPRB, 0.18089E+01_JPRB, 0.15699E+01_JPRB, 0.13479E+01_JPRB, 0.11535E+01_JPRB ,&
&  0.10067E+01_JPRB, 0.92099E+00_JPRB, 0.86959E+00_JPRB, 0.83789E+00_JPRB, 0.81722E+00_JPRB, 0.78609E+00_JPRB, 0.74314E+00_JPRB ,&
&  0.70500E+00_JPRB, 0.67989E+00_JPRB, 0.67002E+00_JPRB, 0.66987E+00_JPRB, 0.67140E+00_JPRB, 0.67879E+00_JPRB, 0.67872E+00_JPRB ,&
&  0.66839E+00_JPRB, 0.65216E+00_JPRB, 0.62191E+00_JPRB, 0.59643E+00_JPRB, 0.58808E+00_JPRB, 0.57918E+00_JPRB, 0.56913E+00_JPRB ,&
&  0.56650E+00_JPRB, 0.56934E+00_JPRB, 0.57246E+00_JPRB, 0.58412E+00_JPRB, 0.58625E+00_JPRB, 0.61040E+00_JPRB, 0.58598E+00_JPRB ,&
&  0.58241E+00_JPRB, 0.56960E+00_JPRB, 0.55763E+00_JPRB, 0.54419E+00_JPRB, 0.54027E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.53485E+00_JPRB, 0.52859E+00_JPRB, 0.52030E+00_JPRB, 0.51182E+00_JPRB, 0.50181E+00_JPRB, 0.49276E+00_JPRB, 0.48756E+00_JPRB ,&
&  0.48479E+00_JPRB, 0.48506E+00_JPRB, 0.48577E+00_JPRB, 0.47496E+00_JPRB, 0.46030E+00_JPRB, 0.44693E+00_JPRB, 0.43890E+00_JPRB ,&
&  0.43334E+00_JPRB, 0.42735E+00_JPRB, 0.41966E+00_JPRB, 0.40969E+00_JPRB, 0.40484E+00_JPRB, 0.40237E+00_JPRB, 0.40097E+00_JPRB ,&
&  0.40605E+00_JPRB, 0.41059E+00_JPRB, 0.41658E+00_JPRB, 0.42515E+00_JPRB, 0.44031E+00_JPRB, 0.46357E+00_JPRB, 0.49952E+00_JPRB ,&
&  0.56579E+00_JPRB, 0.68791E+00_JPRB, 0.84097E+00_JPRB, 0.79102E+00_JPRB, 0.80633E+00_JPRB, 0.84098E+00_JPRB, 0.96768E+00_JPRB ,&
&  0.10672E+01_JPRB, 0.10977E+01_JPRB, 0.10189E+01_JPRB, 0.90793E+00_JPRB, 0.80798E+00_JPRB, 0.73015E+00_JPRB, 0.67518E+00_JPRB ,&
&  0.62501E+00_JPRB, 0.58069E+00_JPRB, 0.54942E+00_JPRB, 0.52773E+00_JPRB, 0.51341E+00_JPRB, 0.50472E+00_JPRB, 0.50234E+00_JPRB ,&
&  0.51490E+00_JPRB, 0.56147E+00_JPRB, 0.67040E+00_JPRB, 0.89583E+00_JPRB, 0.13793E+01_JPRB, 0.13897E+01_JPRB, 0.17213E+01_JPRB ,&
&  0.23421E+01_JPRB, 0.22916E+01_JPRB, 0.20737E+01_JPRB, 0.18018E+01_JPRB, 0.15659E+01_JPRB, 0.13587E+01_JPRB, 0.11831E+01_JPRB ,&
&  0.10627E+01_JPRB, 0.97601E+00_JPRB, 0.89966E+00_JPRB, 0.84262E+00_JPRB, 0.80525E+00_JPRB, 0.78773E+00_JPRB, 0.77211E+00_JPRB ,&
&  0.75237E+00_JPRB, 0.73041E+00_JPRB, 0.71430E+00_JPRB, 0.70708E+00_JPRB, 0.70030E+00_JPRB, 0.69549E+00_JPRB, 0.67716E+00_JPRB ,&
&  0.65566E+00_JPRB, 0.63581E+00_JPRB, 0.61797E+00_JPRB, 0.59620E+00_JPRB, 0.59220E+00_JPRB, 0.58719E+00_JPRB, 0.57425E+00_JPRB ,&
&  0.55986E+00_JPRB, 0.55779E+00_JPRB, 0.56968E+00_JPRB, 0.57620E+00_JPRB, 0.58837E+00_JPRB, 0.61048E+00_JPRB, 0.58368E+00_JPRB ,&
&  0.57479E+00_JPRB, 0.55684E+00_JPRB, 0.54119E+00_JPRB, 0.53581E+00_JPRB, 0.54309E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.53553E+00_JPRB, 0.52679E+00_JPRB, 0.51802E+00_JPRB, 0.50963E+00_JPRB, 0.49844E+00_JPRB, 0.48930E+00_JPRB, 0.48084E+00_JPRB ,&
&  0.47494E+00_JPRB, 0.47198E+00_JPRB, 0.47275E+00_JPRB, 0.47343E+00_JPRB, 0.47088E+00_JPRB, 0.46313E+00_JPRB, 0.44968E+00_JPRB ,&
&  0.44104E+00_JPRB, 0.43391E+00_JPRB, 0.42516E+00_JPRB, 0.41448E+00_JPRB, 0.40408E+00_JPRB, 0.39945E+00_JPRB, 0.39952E+00_JPRB ,&
&  0.40395E+00_JPRB, 0.40961E+00_JPRB, 0.41342E+00_JPRB, 0.42495E+00_JPRB, 0.44172E+00_JPRB, 0.46497E+00_JPRB, 0.49750E+00_JPRB ,&
&  0.54741E+00_JPRB, 0.67581E+00_JPRB, 0.71154E+00_JPRB, 0.70672E+00_JPRB, 0.71348E+00_JPRB, 0.73442E+00_JPRB, 0.80533E+00_JPRB ,&
&  0.87411E+00_JPRB, 0.91954E+00_JPRB, 0.89674E+00_JPRB, 0.81356E+00_JPRB, 0.74425E+00_JPRB, 0.69519E+00_JPRB, 0.64742E+00_JPRB ,&
&  0.60155E+00_JPRB, 0.56569E+00_JPRB, 0.53729E+00_JPRB, 0.51796E+00_JPRB, 0.50340E+00_JPRB, 0.49284E+00_JPRB, 0.48747E+00_JPRB ,&
&  0.48778E+00_JPRB, 0.49518E+00_JPRB, 0.52614E+00_JPRB, 0.60320E+00_JPRB, 0.92651E+00_JPRB, 0.12143E+01_JPRB, 0.14682E+01_JPRB ,&
&  0.15636E+01_JPRB, 0.16165E+01_JPRB, 0.16417E+01_JPRB, 0.15507E+01_JPRB, 0.14157E+01_JPRB, 0.12553E+01_JPRB, 0.11358E+01_JPRB ,&
&  0.10580E+01_JPRB, 0.97645E+00_JPRB, 0.89827E+00_JPRB, 0.82481E+00_JPRB, 0.77260E+00_JPRB, 0.74399E+00_JPRB, 0.72699E+00_JPRB ,&
&  0.70938E+00_JPRB, 0.69732E+00_JPRB, 0.68762E+00_JPRB, 0.67851E+00_JPRB, 0.66824E+00_JPRB, 0.65472E+00_JPRB, 0.63664E+00_JPRB ,&
&  0.61526E+00_JPRB, 0.59440E+00_JPRB, 0.58247E+00_JPRB, 0.57524E+00_JPRB, 0.57038E+00_JPRB, 0.56229E+00_JPRB, 0.55228E+00_JPRB ,&
&  0.55124E+00_JPRB, 0.55682E+00_JPRB, 0.56260E+00_JPRB, 0.56974E+00_JPRB, 0.57405E+00_JPRB, 0.59144E+00_JPRB, 0.57305E+00_JPRB ,&
&  0.55805E+00_JPRB, 0.54055E+00_JPRB, 0.52664E+00_JPRB, 0.52038E+00_JPRB, 0.53911E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.54235E+00_JPRB, 0.52979E+00_JPRB, 0.51519E+00_JPRB, 0.49753E+00_JPRB, 0.48248E+00_JPRB, 0.47291E+00_JPRB, 0.46632E+00_JPRB ,&
&  0.46278E+00_JPRB, 0.46131E+00_JPRB, 0.46015E+00_JPRB, 0.45849E+00_JPRB, 0.45585E+00_JPRB, 0.45252E+00_JPRB, 0.44555E+00_JPRB ,&
&  0.43707E+00_JPRB, 0.42701E+00_JPRB, 0.41883E+00_JPRB, 0.41166E+00_JPRB, 0.39899E+00_JPRB, 0.39153E+00_JPRB, 0.39307E+00_JPRB ,&
&  0.39780E+00_JPRB, 0.40348E+00_JPRB, 0.40787E+00_JPRB, 0.41853E+00_JPRB, 0.43877E+00_JPRB, 0.46165E+00_JPRB, 0.48961E+00_JPRB ,&
&  0.54234E+00_JPRB, 0.54363E+00_JPRB, 0.58863E+00_JPRB, 0.60151E+00_JPRB, 0.60638E+00_JPRB, 0.63835E+00_JPRB, 0.68287E+00_JPRB ,&
&  0.71295E+00_JPRB, 0.76119E+00_JPRB, 0.75400E+00_JPRB, 0.69754E+00_JPRB, 0.64068E+00_JPRB, 0.60030E+00_JPRB, 0.57492E+00_JPRB ,&
&  0.55541E+00_JPRB, 0.53847E+00_JPRB, 0.52381E+00_JPRB, 0.51459E+00_JPRB, 0.50565E+00_JPRB, 0.49689E+00_JPRB, 0.48817E+00_JPRB ,&
&  0.48346E+00_JPRB, 0.48064E+00_JPRB, 0.48289E+00_JPRB, 0.50137E+00_JPRB, 0.57637E+00_JPRB, 0.72070E+00_JPRB, 0.92227E+00_JPRB ,&
&  0.10895E+01_JPRB, 0.11916E+01_JPRB, 0.12809E+01_JPRB, 0.12866E+01_JPRB, 0.12349E+01_JPRB, 0.11701E+01_JPRB, 0.10967E+01_JPRB ,&
&  0.99741E+00_JPRB, 0.88371E+00_JPRB, 0.79825E+00_JPRB, 0.73718E+00_JPRB, 0.69562E+00_JPRB, 0.66991E+00_JPRB, 0.65471E+00_JPRB ,&
&  0.64313E+00_JPRB, 0.64035E+00_JPRB, 0.65424E+00_JPRB, 0.66893E+00_JPRB, 0.66814E+00_JPRB, 0.64626E+00_JPRB, 0.61494E+00_JPRB ,&
&  0.58668E+00_JPRB, 0.56475E+00_JPRB, 0.54858E+00_JPRB, 0.53701E+00_JPRB, 0.52840E+00_JPRB, 0.52244E+00_JPRB, 0.51603E+00_JPRB ,&
&  0.51455E+00_JPRB, 0.52689E+00_JPRB, 0.52753E+00_JPRB, 0.54248E+00_JPRB, 0.57082E+00_JPRB, 0.55743E+00_JPRB, 0.53832E+00_JPRB ,&
&  0.52427E+00_JPRB, 0.51642E+00_JPRB, 0.50756E+00_JPRB, 0.50236E+00_JPRB, 0.55542E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.47613E+00_JPRB, 0.47643E+00_JPRB, 0.46848E+00_JPRB, 0.45876E+00_JPRB, 0.45052E+00_JPRB, 0.44632E+00_JPRB, 0.44300E+00_JPRB ,&
&  0.44129E+00_JPRB, 0.44041E+00_JPRB, 0.43972E+00_JPRB, 0.43768E+00_JPRB, 0.43370E+00_JPRB, 0.42749E+00_JPRB, 0.41968E+00_JPRB ,&
&  0.41267E+00_JPRB, 0.40560E+00_JPRB, 0.39957E+00_JPRB, 0.39287E+00_JPRB, 0.38453E+00_JPRB, 0.38025E+00_JPRB, 0.38273E+00_JPRB ,&
&  0.38839E+00_JPRB, 0.39524E+00_JPRB, 0.40142E+00_JPRB, 0.41146E+00_JPRB, 0.43397E+00_JPRB, 0.45716E+00_JPRB, 0.47897E+00_JPRB ,&
&  0.52300E+00_JPRB, 0.48737E+00_JPRB, 0.50834E+00_JPRB, 0.53609E+00_JPRB, 0.55938E+00_JPRB, 0.57397E+00_JPRB, 0.59203E+00_JPRB ,&
&  0.60983E+00_JPRB, 0.62455E+00_JPRB, 0.62486E+00_JPRB, 0.60152E+00_JPRB, 0.55740E+00_JPRB, 0.51758E+00_JPRB, 0.49437E+00_JPRB ,&
&  0.48242E+00_JPRB, 0.47722E+00_JPRB, 0.47244E+00_JPRB, 0.46823E+00_JPRB, 0.46556E+00_JPRB, 0.46319E+00_JPRB, 0.46054E+00_JPRB ,&
&  0.45809E+00_JPRB, 0.45616E+00_JPRB, 0.45529E+00_JPRB, 0.45731E+00_JPRB, 0.47252E+00_JPRB, 0.52070E+00_JPRB, 0.62559E+00_JPRB ,&
&  0.77511E+00_JPRB, 0.91653E+00_JPRB, 0.99430E+00_JPRB, 0.10424E+01_JPRB, 0.10780E+01_JPRB, 0.10399E+01_JPRB, 0.92819E+00_JPRB ,&
&  0.82126E+00_JPRB, 0.73054E+00_JPRB, 0.66494E+00_JPRB, 0.61849E+00_JPRB, 0.58712E+00_JPRB, 0.56841E+00_JPRB, 0.55889E+00_JPRB ,&
&  0.55568E+00_JPRB, 0.55620E+00_JPRB, 0.56129E+00_JPRB, 0.56626E+00_JPRB, 0.56965E+00_JPRB, 0.56548E+00_JPRB, 0.55442E+00_JPRB ,&
&  0.53803E+00_JPRB, 0.52582E+00_JPRB, 0.51414E+00_JPRB, 0.50181E+00_JPRB, 0.49140E+00_JPRB, 0.48142E+00_JPRB, 0.47405E+00_JPRB ,&
&  0.47128E+00_JPRB, 0.47802E+00_JPRB, 0.49055E+00_JPRB, 0.51465E+00_JPRB, 0.50728E+00_JPRB, 0.49498E+00_JPRB, 0.48387E+00_JPRB ,&
&  0.47958E+00_JPRB, 0.48018E+00_JPRB, 0.47362E+00_JPRB, 0.46917E+00_JPRB, 0.47258E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.44291E+00_JPRB, 0.44186E+00_JPRB, 0.43543E+00_JPRB, 0.42819E+00_JPRB, 0.42484E+00_JPRB, 0.42396E+00_JPRB, 0.42413E+00_JPRB ,&
&  0.42281E+00_JPRB, 0.41990E+00_JPRB, 0.41602E+00_JPRB, 0.41341E+00_JPRB, 0.40975E+00_JPRB, 0.40433E+00_JPRB, 0.39768E+00_JPRB ,&
&  0.39145E+00_JPRB, 0.38558E+00_JPRB, 0.38109E+00_JPRB, 0.37616E+00_JPRB, 0.37234E+00_JPRB, 0.37246E+00_JPRB, 0.37412E+00_JPRB ,&
&  0.37988E+00_JPRB, 0.38751E+00_JPRB, 0.39395E+00_JPRB, 0.40766E+00_JPRB, 0.42325E+00_JPRB, 0.44530E+00_JPRB, 0.46655E+00_JPRB ,&
&  0.50560E+00_JPRB, 0.43743E+00_JPRB, 0.45389E+00_JPRB, 0.49620E+00_JPRB, 0.51875E+00_JPRB, 0.53535E+00_JPRB, 0.53351E+00_JPRB ,&
&  0.53183E+00_JPRB, 0.53309E+00_JPRB, 0.53100E+00_JPRB, 0.52403E+00_JPRB, 0.50744E+00_JPRB, 0.47753E+00_JPRB, 0.44848E+00_JPRB ,&
&  0.42806E+00_JPRB, 0.42098E+00_JPRB, 0.41799E+00_JPRB, 0.41371E+00_JPRB, 0.40943E+00_JPRB, 0.40777E+00_JPRB, 0.40872E+00_JPRB ,&
&  0.40921E+00_JPRB, 0.40911E+00_JPRB, 0.40828E+00_JPRB, 0.40748E+00_JPRB, 0.40987E+00_JPRB, 0.42413E+00_JPRB, 0.46618E+00_JPRB ,&
&  0.54951E+00_JPRB, 0.64517E+00_JPRB, 0.73348E+00_JPRB, 0.80344E+00_JPRB, 0.82065E+00_JPRB, 0.79867E+00_JPRB, 0.75282E+00_JPRB ,&
&  0.69709E+00_JPRB, 0.64423E+00_JPRB, 0.60303E+00_JPRB, 0.57054E+00_JPRB, 0.54472E+00_JPRB, 0.52464E+00_JPRB, 0.51020E+00_JPRB ,&
&  0.50237E+00_JPRB, 0.49704E+00_JPRB, 0.49322E+00_JPRB, 0.48797E+00_JPRB, 0.48301E+00_JPRB, 0.47954E+00_JPRB, 0.47808E+00_JPRB ,&
&  0.47506E+00_JPRB, 0.47109E+00_JPRB, 0.46772E+00_JPRB, 0.46416E+00_JPRB, 0.46010E+00_JPRB, 0.45585E+00_JPRB, 0.45249E+00_JPRB ,&
&  0.45219E+00_JPRB, 0.45393E+00_JPRB, 0.45915E+00_JPRB, 0.46251E+00_JPRB, 0.47124E+00_JPRB, 0.46303E+00_JPRB, 0.45685E+00_JPRB ,&
&  0.45469E+00_JPRB, 0.45189E+00_JPRB, 0.45086E+00_JPRB, 0.41529E+00_JPRB, 0.43878E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.41260E+00_JPRB, 0.41419E+00_JPRB, 0.41263E+00_JPRB, 0.40714E+00_JPRB, 0.40336E+00_JPRB, 0.40092E+00_JPRB, 0.39979E+00_JPRB ,&
&  0.39941E+00_JPRB, 0.39852E+00_JPRB, 0.39632E+00_JPRB, 0.39173E+00_JPRB, 0.38775E+00_JPRB, 0.38254E+00_JPRB, 0.37708E+00_JPRB ,&
&  0.37249E+00_JPRB, 0.36845E+00_JPRB, 0.36628E+00_JPRB, 0.36344E+00_JPRB, 0.36230E+00_JPRB, 0.36372E+00_JPRB, 0.36581E+00_JPRB ,&
&  0.36875E+00_JPRB, 0.37143E+00_JPRB, 0.37951E+00_JPRB, 0.38930E+00_JPRB, 0.40699E+00_JPRB, 0.41924E+00_JPRB, 0.43647E+00_JPRB ,&
&  0.47407E+00_JPRB, 0.40113E+00_JPRB, 0.42511E+00_JPRB, 0.45615E+00_JPRB, 0.47491E+00_JPRB, 0.49266E+00_JPRB, 0.48157E+00_JPRB ,&
&  0.47304E+00_JPRB, 0.46687E+00_JPRB, 0.45903E+00_JPRB, 0.45206E+00_JPRB, 0.44759E+00_JPRB, 0.43816E+00_JPRB, 0.42610E+00_JPRB ,&
&  0.41251E+00_JPRB, 0.40265E+00_JPRB, 0.39494E+00_JPRB, 0.38770E+00_JPRB, 0.38147E+00_JPRB, 0.37764E+00_JPRB, 0.37593E+00_JPRB ,&
&  0.37591E+00_JPRB, 0.37618E+00_JPRB, 0.37607E+00_JPRB, 0.37666E+00_JPRB, 0.37877E+00_JPRB, 0.38326E+00_JPRB, 0.39231E+00_JPRB ,&
&  0.41763E+00_JPRB, 0.46220E+00_JPRB, 0.51243E+00_JPRB, 0.56100E+00_JPRB, 0.59727E+00_JPRB, 0.61315E+00_JPRB, 0.61729E+00_JPRB ,&
&  0.60605E+00_JPRB, 0.58441E+00_JPRB, 0.55916E+00_JPRB, 0.53477E+00_JPRB, 0.51342E+00_JPRB, 0.49561E+00_JPRB, 0.47923E+00_JPRB ,&
&  0.46463E+00_JPRB, 0.45085E+00_JPRB, 0.43992E+00_JPRB, 0.43211E+00_JPRB, 0.42493E+00_JPRB, 0.41858E+00_JPRB, 0.41523E+00_JPRB ,&
&  0.41370E+00_JPRB, 0.41332E+00_JPRB, 0.41316E+00_JPRB, 0.41440E+00_JPRB, 0.41607E+00_JPRB, 0.41745E+00_JPRB, 0.41799E+00_JPRB ,&
&  0.41733E+00_JPRB, 0.41854E+00_JPRB, 0.42008E+00_JPRB, 0.42945E+00_JPRB, 0.43033E+00_JPRB, 0.42974E+00_JPRB, 0.42814E+00_JPRB ,&
&  0.42707E+00_JPRB, 0.42381E+00_JPRB, 0.37681E+00_JPRB, 0.39858E+00_JPRB, 0.40715E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.39225E+00_JPRB, 0.39125E+00_JPRB, 0.39152E+00_JPRB, 0.39089E+00_JPRB, 0.38992E+00_JPRB, 0.38839E+00_JPRB, 0.38594E+00_JPRB ,&
&  0.38236E+00_JPRB, 0.37992E+00_JPRB, 0.37763E+00_JPRB, 0.37461E+00_JPRB, 0.37131E+00_JPRB, 0.36844E+00_JPRB, 0.36562E+00_JPRB ,&
&  0.36270E+00_JPRB, 0.35965E+00_JPRB, 0.35773E+00_JPRB, 0.35651E+00_JPRB, 0.35502E+00_JPRB, 0.35462E+00_JPRB, 0.35582E+00_JPRB ,&
&  0.35608E+00_JPRB, 0.35859E+00_JPRB, 0.36800E+00_JPRB, 0.37862E+00_JPRB, 0.39111E+00_JPRB, 0.40204E+00_JPRB, 0.41613E+00_JPRB ,&
&  0.40209E+00_JPRB, 0.36137E+00_JPRB, 0.38317E+00_JPRB, 0.40768E+00_JPRB, 0.43255E+00_JPRB, 0.45294E+00_JPRB, 0.44550E+00_JPRB ,&
&  0.43456E+00_JPRB, 0.42144E+00_JPRB, 0.40966E+00_JPRB, 0.40065E+00_JPRB, 0.39783E+00_JPRB, 0.39559E+00_JPRB, 0.39276E+00_JPRB ,&
&  0.38891E+00_JPRB, 0.38263E+00_JPRB, 0.37634E+00_JPRB, 0.37073E+00_JPRB, 0.36694E+00_JPRB, 0.36389E+00_JPRB, 0.36123E+00_JPRB ,&
&  0.36023E+00_JPRB, 0.36028E+00_JPRB, 0.36046E+00_JPRB, 0.36083E+00_JPRB, 0.36183E+00_JPRB, 0.36486E+00_JPRB, 0.36869E+00_JPRB ,&
&  0.37348E+00_JPRB, 0.38428E+00_JPRB, 0.40614E+00_JPRB, 0.43317E+00_JPRB, 0.46104E+00_JPRB, 0.48251E+00_JPRB, 0.49669E+00_JPRB ,&
&  0.50572E+00_JPRB, 0.50830E+00_JPRB, 0.50304E+00_JPRB, 0.49036E+00_JPRB, 0.47407E+00_JPRB, 0.45829E+00_JPRB, 0.44324E+00_JPRB ,&
&  0.42850E+00_JPRB, 0.41260E+00_JPRB, 0.39770E+00_JPRB, 0.38850E+00_JPRB, 0.38349E+00_JPRB, 0.37898E+00_JPRB, 0.37553E+00_JPRB ,&
&  0.37417E+00_JPRB, 0.37424E+00_JPRB, 0.37523E+00_JPRB, 0.37645E+00_JPRB, 0.37794E+00_JPRB, 0.38037E+00_JPRB, 0.38273E+00_JPRB ,&
&  0.38473E+00_JPRB, 0.38849E+00_JPRB, 0.39412E+00_JPRB, 0.40474E+00_JPRB, 0.41176E+00_JPRB, 0.41488E+00_JPRB, 0.41413E+00_JPRB ,&
&  0.40930E+00_JPRB, 0.40320E+00_JPRB, 0.39709E+00_JPRB, 0.39232E+00_JPRB, 0.39169E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.37553E+00_JPRB, 0.37462E+00_JPRB, 0.37405E+00_JPRB, 0.37272E+00_JPRB, 0.37163E+00_JPRB, 0.37060E+00_JPRB, 0.36761E+00_JPRB ,&
&  0.36664E+00_JPRB, 0.36345E+00_JPRB, 0.36031E+00_JPRB, 0.35902E+00_JPRB, 0.35730E+00_JPRB, 0.35492E+00_JPRB, 0.35307E+00_JPRB ,&
&  0.35097E+00_JPRB, 0.34864E+00_JPRB, 0.34712E+00_JPRB, 0.34502E+00_JPRB, 0.34181E+00_JPRB, 0.33951E+00_JPRB, 0.34014E+00_JPRB ,&
&  0.34112E+00_JPRB, 0.34372E+00_JPRB, 0.35150E+00_JPRB, 0.35919E+00_JPRB, 0.36725E+00_JPRB, 0.37473E+00_JPRB, 0.38179E+00_JPRB ,&
&  0.39203E+00_JPRB, 0.35148E+00_JPRB, 0.37120E+00_JPRB, 0.38475E+00_JPRB, 0.40521E+00_JPRB, 0.42190E+00_JPRB, 0.41723E+00_JPRB ,&
&  0.40391E+00_JPRB, 0.38782E+00_JPRB, 0.37501E+00_JPRB, 0.36670E+00_JPRB, 0.36566E+00_JPRB, 0.36622E+00_JPRB, 0.36489E+00_JPRB ,&
&  0.36272E+00_JPRB, 0.35766E+00_JPRB, 0.35194E+00_JPRB, 0.34815E+00_JPRB, 0.34616E+00_JPRB, 0.34472E+00_JPRB, 0.34310E+00_JPRB ,&
&  0.34226E+00_JPRB, 0.34229E+00_JPRB, 0.34238E+00_JPRB, 0.34239E+00_JPRB, 0.34306E+00_JPRB, 0.34480E+00_JPRB, 0.34711E+00_JPRB ,&
&  0.34900E+00_JPRB, 0.34996E+00_JPRB, 0.35332E+00_JPRB, 0.36167E+00_JPRB, 0.37406E+00_JPRB, 0.38664E+00_JPRB, 0.39663E+00_JPRB ,&
&  0.40395E+00_JPRB, 0.40986E+00_JPRB, 0.41337E+00_JPRB, 0.41233E+00_JPRB, 0.40670E+00_JPRB, 0.39807E+00_JPRB, 0.38890E+00_JPRB ,&
&  0.37957E+00_JPRB, 0.37015E+00_JPRB, 0.35942E+00_JPRB, 0.35320E+00_JPRB, 0.35056E+00_JPRB, 0.34915E+00_JPRB, 0.34835E+00_JPRB ,&
&  0.34803E+00_JPRB, 0.34783E+00_JPRB, 0.34833E+00_JPRB, 0.34921E+00_JPRB, 0.34976E+00_JPRB, 0.35046E+00_JPRB, 0.35214E+00_JPRB ,&
&  0.35540E+00_JPRB, 0.36088E+00_JPRB, 0.36801E+00_JPRB, 0.37797E+00_JPRB, 0.38659E+00_JPRB, 0.39182E+00_JPRB, 0.39442E+00_JPRB ,&
&  0.39245E+00_JPRB, 0.38784E+00_JPRB, 0.38213E+00_JPRB, 0.37712E+00_JPRB, 0.37627E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.36326E+00_JPRB, 0.36036E+00_JPRB, 0.35837E+00_JPRB, 0.35495E+00_JPRB, 0.35250E+00_JPRB, 0.35072E+00_JPRB, 0.34863E+00_JPRB ,&
&  0.34712E+00_JPRB, 0.34744E+00_JPRB, 0.34626E+00_JPRB, 0.34557E+00_JPRB, 0.34460E+00_JPRB, 0.34192E+00_JPRB, 0.33866E+00_JPRB ,&
&  0.33691E+00_JPRB, 0.33529E+00_JPRB, 0.33386E+00_JPRB, 0.33222E+00_JPRB, 0.33063E+00_JPRB, 0.32837E+00_JPRB, 0.32855E+00_JPRB ,&
&  0.32979E+00_JPRB, 0.33250E+00_JPRB, 0.33558E+00_JPRB, 0.33920E+00_JPRB, 0.34448E+00_JPRB, 0.35148E+00_JPRB, 0.35805E+00_JPRB ,&
&  0.36490E+00_JPRB, 0.35346E+00_JPRB, 0.36541E+00_JPRB, 0.37215E+00_JPRB, 0.38388E+00_JPRB, 0.39293E+00_JPRB, 0.39209E+00_JPRB ,&
&  0.38643E+00_JPRB, 0.37441E+00_JPRB, 0.36308E+00_JPRB, 0.35538E+00_JPRB, 0.35293E+00_JPRB, 0.35142E+00_JPRB, 0.34733E+00_JPRB ,&
&  0.34223E+00_JPRB, 0.33595E+00_JPRB, 0.33099E+00_JPRB, 0.32830E+00_JPRB, 0.32746E+00_JPRB, 0.32753E+00_JPRB, 0.32645E+00_JPRB ,&
&  0.32526E+00_JPRB, 0.32398E+00_JPRB, 0.32339E+00_JPRB, 0.32312E+00_JPRB, 0.32276E+00_JPRB, 0.32302E+00_JPRB, 0.32391E+00_JPRB ,&
&  0.32490E+00_JPRB, 0.32558E+00_JPRB, 0.32603E+00_JPRB, 0.32753E+00_JPRB, 0.33081E+00_JPRB, 0.33533E+00_JPRB, 0.33972E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34519E+00_JPRB, 0.34754E+00_JPRB, 0.34927E+00_JPRB, 0.34829E+00_JPRB, 0.34440E+00_JPRB, 0.33932E+00_JPRB ,&
&  0.33508E+00_JPRB, 0.33151E+00_JPRB, 0.32823E+00_JPRB, 0.32568E+00_JPRB, 0.32441E+00_JPRB, 0.32449E+00_JPRB, 0.32509E+00_JPRB ,&
&  0.32582E+00_JPRB, 0.32666E+00_JPRB, 0.32749E+00_JPRB, 0.32864E+00_JPRB, 0.32919E+00_JPRB, 0.32995E+00_JPRB, 0.33122E+00_JPRB ,&
&  0.33468E+00_JPRB, 0.33908E+00_JPRB, 0.34535E+00_JPRB, 0.35469E+00_JPRB, 0.36305E+00_JPRB, 0.36882E+00_JPRB, 0.37339E+00_JPRB ,&
&  0.37486E+00_JPRB, 0.37347E+00_JPRB, 0.37094E+00_JPRB, 0.36892E+00_JPRB, 0.36674E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.34150E+00_JPRB, 0.33779E+00_JPRB, 0.33502E+00_JPRB, 0.33273E+00_JPRB, 0.33265E+00_JPRB, 0.33140E+00_JPRB, 0.32845E+00_JPRB ,&
&  0.32617E+00_JPRB, 0.32724E+00_JPRB, 0.32963E+00_JPRB, 0.33048E+00_JPRB, 0.32962E+00_JPRB, 0.32813E+00_JPRB, 0.32533E+00_JPRB ,&
&  0.32155E+00_JPRB, 0.31973E+00_JPRB, 0.31957E+00_JPRB, 0.32038E+00_JPRB, 0.32034E+00_JPRB, 0.31876E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31851E+00_JPRB, 0.31997E+00_JPRB, 0.32024E+00_JPRB, 0.32228E+00_JPRB, 0.32604E+00_JPRB, 0.33273E+00_JPRB, 0.33895E+00_JPRB ,&
&  0.34090E+00_JPRB, 0.34171E+00_JPRB, 0.33572E+00_JPRB, 0.34642E+00_JPRB, 0.35260E+00_JPRB, 0.36078E+00_JPRB, 0.36512E+00_JPRB ,&
&  0.36571E+00_JPRB, 0.36137E+00_JPRB, 0.35097E+00_JPRB, 0.34294E+00_JPRB, 0.33815E+00_JPRB, 0.33364E+00_JPRB, 0.32672E+00_JPRB ,&
&  0.31950E+00_JPRB, 0.31316E+00_JPRB, 0.30944E+00_JPRB, 0.30782E+00_JPRB, 0.30821E+00_JPRB, 0.30894E+00_JPRB, 0.30742E+00_JPRB ,&
&  0.30564E+00_JPRB, 0.30388E+00_JPRB, 0.30320E+00_JPRB, 0.30245E+00_JPRB, 0.30129E+00_JPRB, 0.30036E+00_JPRB, 0.30011E+00_JPRB ,&
&  0.30030E+00_JPRB, 0.30049E+00_JPRB, 0.30097E+00_JPRB, 0.30187E+00_JPRB, 0.30249E+00_JPRB, 0.30287E+00_JPRB, 0.30372E+00_JPRB ,&
&  0.30458E+00_JPRB, 0.30532E+00_JPRB, 0.30598E+00_JPRB, 0.30640E+00_JPRB, 0.30635E+00_JPRB, 0.30576E+00_JPRB, 0.30413E+00_JPRB ,&
&  0.30211E+00_JPRB, 0.30152E+00_JPRB, 0.30163E+00_JPRB, 0.30100E+00_JPRB, 0.30014E+00_JPRB, 0.30030E+00_JPRB, 0.30157E+00_JPRB ,&
&  0.30296E+00_JPRB, 0.30476E+00_JPRB, 0.30625E+00_JPRB, 0.30738E+00_JPRB, 0.30758E+00_JPRB, 0.30834E+00_JPRB, 0.30943E+00_JPRB ,&
&  0.31133E+00_JPRB, 0.31428E+00_JPRB, 0.31853E+00_JPRB, 0.32409E+00_JPRB, 0.32987E+00_JPRB, 0.33424E+00_JPRB, 0.33877E+00_JPRB ,&
&  0.34274E+00_JPRB, 0.34458E+00_JPRB, 0.34485E+00_JPRB, 0.34546E+00_JPRB, 0.34464E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.31690E+00_JPRB, 0.31404E+00_JPRB, 0.31189E+00_JPRB, 0.31145E+00_JPRB, 0.31297E+00_JPRB, 0.31288E+00_JPRB, 0.31078E+00_JPRB ,&
&  0.30878E+00_JPRB, 0.30979E+00_JPRB, 0.31272E+00_JPRB, 0.31428E+00_JPRB, 0.31487E+00_JPRB, 0.31561E+00_JPRB, 0.31452E+00_JPRB ,&
&  0.31126E+00_JPRB, 0.30803E+00_JPRB, 0.30860E+00_JPRB, 0.31019E+00_JPRB, 0.31133E+00_JPRB, 0.31001E+00_JPRB, 0.30842E+00_JPRB ,&
&  0.30899E+00_JPRB, 0.30909E+00_JPRB, 0.31073E+00_JPRB, 0.31250E+00_JPRB, 0.31594E+00_JPRB, 0.32137E+00_JPRB, 0.32528E+00_JPRB ,&
&  0.32616E+00_JPRB, 0.32434E+00_JPRB, 0.32364E+00_JPRB, 0.32599E+00_JPRB, 0.32867E+00_JPRB, 0.33493E+00_JPRB, 0.34113E+00_JPRB ,&
&  0.34300E+00_JPRB, 0.33961E+00_JPRB, 0.32940E+00_JPRB, 0.32095E+00_JPRB, 0.31582E+00_JPRB, 0.31068E+00_JPRB, 0.30567E+00_JPRB ,&
&  0.30030E+00_JPRB, 0.29635E+00_JPRB, 0.29407E+00_JPRB, 0.29314E+00_JPRB, 0.29294E+00_JPRB, 0.29243E+00_JPRB, 0.29033E+00_JPRB ,&
&  0.28759E+00_JPRB, 0.28547E+00_JPRB, 0.28384E+00_JPRB, 0.28257E+00_JPRB, 0.28138E+00_JPRB, 0.28083E+00_JPRB, 0.28019E+00_JPRB ,&
&  0.27989E+00_JPRB, 0.28031E+00_JPRB, 0.28112E+00_JPRB, 0.28118E+00_JPRB, 0.28104E+00_JPRB, 0.28043E+00_JPRB, 0.28015E+00_JPRB ,&
&  0.28017E+00_JPRB, 0.27915E+00_JPRB, 0.27830E+00_JPRB, 0.27879E+00_JPRB, 0.27927E+00_JPRB, 0.28005E+00_JPRB, 0.28049E+00_JPRB ,&
&  0.28040E+00_JPRB, 0.28060E+00_JPRB, 0.28120E+00_JPRB, 0.28161E+00_JPRB, 0.28176E+00_JPRB, 0.28179E+00_JPRB, 0.28210E+00_JPRB ,&
&  0.28351E+00_JPRB, 0.28525E+00_JPRB, 0.28653E+00_JPRB, 0.28764E+00_JPRB, 0.28873E+00_JPRB, 0.29060E+00_JPRB, 0.29126E+00_JPRB ,&
&  0.29174E+00_JPRB, 0.29305E+00_JPRB, 0.29597E+00_JPRB, 0.29998E+00_JPRB, 0.30378E+00_JPRB, 0.30641E+00_JPRB, 0.30924E+00_JPRB ,&
&  0.31183E+00_JPRB, 0.31405E+00_JPRB, 0.31543E+00_JPRB, 0.31611E+00_JPRB, 0.31692E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.29327E+00_JPRB, 0.29223E+00_JPRB, 0.29184E+00_JPRB, 0.29218E+00_JPRB, 0.29384E+00_JPRB, 0.29382E+00_JPRB, 0.29323E+00_JPRB ,&
&  0.29260E+00_JPRB, 0.29425E+00_JPRB, 0.29772E+00_JPRB, 0.29990E+00_JPRB, 0.30028E+00_JPRB, 0.30026E+00_JPRB, 0.29997E+00_JPRB ,&
&  0.29987E+00_JPRB, 0.29981E+00_JPRB, 0.29900E+00_JPRB, 0.29946E+00_JPRB, 0.29960E+00_JPRB, 0.29877E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29988E+00_JPRB, 0.30040E+00_JPRB, 0.30259E+00_JPRB, 0.30431E+00_JPRB, 0.30797E+00_JPRB, 0.31366E+00_JPRB, 0.31627E+00_JPRB ,&
&  0.31545E+00_JPRB, 0.31129E+00_JPRB, 0.30781E+00_JPRB, 0.30834E+00_JPRB, 0.31004E+00_JPRB, 0.31351E+00_JPRB, 0.31713E+00_JPRB ,&
&  0.31771E+00_JPRB, 0.31146E+00_JPRB, 0.30213E+00_JPRB, 0.29574E+00_JPRB, 0.29183E+00_JPRB, 0.28919E+00_JPRB, 0.28689E+00_JPRB ,&
&  0.28417E+00_JPRB, 0.28228E+00_JPRB, 0.28159E+00_JPRB, 0.28170E+00_JPRB, 0.28151E+00_JPRB, 0.27883E+00_JPRB, 0.27572E+00_JPRB ,&
&  0.27168E+00_JPRB, 0.26886E+00_JPRB, 0.26651E+00_JPRB, 0.26571E+00_JPRB, 0.26583E+00_JPRB, 0.26560E+00_JPRB, 0.26459E+00_JPRB ,&
&  0.26329E+00_JPRB, 0.26285E+00_JPRB, 0.26247E+00_JPRB, 0.26131E+00_JPRB, 0.25986E+00_JPRB, 0.25943E+00_JPRB, 0.25854E+00_JPRB ,&
&  0.25685E+00_JPRB, 0.25630E+00_JPRB, 0.25693E+00_JPRB, 0.25842E+00_JPRB, 0.25877E+00_JPRB, 0.26001E+00_JPRB, 0.26196E+00_JPRB ,&
&  0.26153E+00_JPRB, 0.25929E+00_JPRB, 0.25791E+00_JPRB, 0.25816E+00_JPRB, 0.25973E+00_JPRB, 0.26198E+00_JPRB, 0.26300E+00_JPRB ,&
&  0.26390E+00_JPRB, 0.26382E+00_JPRB, 0.26466E+00_JPRB, 0.26600E+00_JPRB, 0.26783E+00_JPRB, 0.26935E+00_JPRB, 0.27023E+00_JPRB ,&
&  0.27087E+00_JPRB, 0.27146E+00_JPRB, 0.27353E+00_JPRB, 0.27672E+00_JPRB, 0.27946E+00_JPRB, 0.28255E+00_JPRB, 0.28493E+00_JPRB ,&
&  0.28658E+00_JPRB, 0.28834E+00_JPRB, 0.28953E+00_JPRB, 0.29049E+00_JPRB, 0.29186E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.27806E+00_JPRB, 0.27873E+00_JPRB, 0.27986E+00_JPRB, 0.28016E+00_JPRB, 0.28098E+00_JPRB, 0.28140E+00_JPRB, 0.28242E+00_JPRB ,&
&  0.28275E+00_JPRB, 0.28409E+00_JPRB, 0.28613E+00_JPRB, 0.28613E+00_JPRB, 0.28585E+00_JPRB, 0.28579E+00_JPRB, 0.28657E+00_JPRB ,&
&  0.28902E+00_JPRB, 0.29043E+00_JPRB, 0.29022E+00_JPRB, 0.29032E+00_JPRB, 0.28966E+00_JPRB, 0.28900E+00_JPRB, 0.29104E+00_JPRB ,&
&  0.29161E+00_JPRB, 0.29241E+00_JPRB, 0.29430E+00_JPRB, 0.29586E+00_JPRB, 0.29949E+00_JPRB, 0.30387E+00_JPRB, 0.30679E+00_JPRB ,&
&  0.30657E+00_JPRB, 0.30345E+00_JPRB, 0.29831E+00_JPRB, 0.29430E+00_JPRB, 0.29351E+00_JPRB, 0.29439E+00_JPRB, 0.29656E+00_JPRB ,&
&  0.29561E+00_JPRB, 0.28990E+00_JPRB, 0.28267E+00_JPRB, 0.27870E+00_JPRB, 0.27721E+00_JPRB, 0.27522E+00_JPRB, 0.27204E+00_JPRB ,&
&  0.26956E+00_JPRB, 0.26903E+00_JPRB, 0.26932E+00_JPRB, 0.26869E+00_JPRB, 0.26712E+00_JPRB, 0.26430E+00_JPRB, 0.26065E+00_JPRB ,&
&  0.25744E+00_JPRB, 0.25424E+00_JPRB, 0.25222E+00_JPRB, 0.25084E+00_JPRB, 0.25074E+00_JPRB, 0.25093E+00_JPRB, 0.25009E+00_JPRB ,&
&  0.24817E+00_JPRB, 0.24691E+00_JPRB, 0.24527E+00_JPRB, 0.24217E+00_JPRB, 0.23998E+00_JPRB, 0.23934E+00_JPRB, 0.23840E+00_JPRB ,&
&  0.23708E+00_JPRB, 0.23684E+00_JPRB, 0.23688E+00_JPRB, 0.23669E+00_JPRB, 0.23788E+00_JPRB, 0.23990E+00_JPRB, 0.23768E+00_JPRB ,&
&  0.23571E+00_JPRB, 0.23438E+00_JPRB, 0.23444E+00_JPRB, 0.23592E+00_JPRB, 0.23841E+00_JPRB, 0.24117E+00_JPRB, 0.24346E+00_JPRB ,&
&  0.24443E+00_JPRB, 0.24467E+00_JPRB, 0.24498E+00_JPRB, 0.24513E+00_JPRB, 0.24636E+00_JPRB, 0.24886E+00_JPRB, 0.25178E+00_JPRB ,&
&  0.25313E+00_JPRB, 0.25560E+00_JPRB, 0.25829E+00_JPRB, 0.26167E+00_JPRB, 0.26364E+00_JPRB, 0.26549E+00_JPRB, 0.26721E+00_JPRB ,&
&  0.26853E+00_JPRB, 0.27022E+00_JPRB, 0.27251E+00_JPRB, 0.27441E+00_JPRB, 0.27640E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.26676E+00_JPRB, 0.26824E+00_JPRB, 0.26905E+00_JPRB, 0.27029E+00_JPRB, 0.27253E+00_JPRB, 0.27387E+00_JPRB, 0.27383E+00_JPRB ,&
&  0.27384E+00_JPRB, 0.27501E+00_JPRB, 0.27589E+00_JPRB, 0.27630E+00_JPRB, 0.27405E+00_JPRB, 0.27383E+00_JPRB, 0.27563E+00_JPRB ,&
&  0.27798E+00_JPRB, 0.28035E+00_JPRB, 0.28074E+00_JPRB, 0.28090E+00_JPRB, 0.28044E+00_JPRB, 0.27939E+00_JPRB, 0.28097E+00_JPRB ,&
&  0.28235E+00_JPRB, 0.28300E+00_JPRB, 0.28437E+00_JPRB, 0.28571E+00_JPRB, 0.28860E+00_JPRB, 0.29196E+00_JPRB, 0.29390E+00_JPRB ,&
&  0.29397E+00_JPRB, 0.29225E+00_JPRB, 0.28567E+00_JPRB, 0.27890E+00_JPRB, 0.27918E+00_JPRB, 0.28020E+00_JPRB, 0.28006E+00_JPRB ,&
&  0.27793E+00_JPRB, 0.27511E+00_JPRB, 0.27318E+00_JPRB, 0.27167E+00_JPRB, 0.26985E+00_JPRB, 0.26663E+00_JPRB, 0.26311E+00_JPRB ,&
&  0.26011E+00_JPRB, 0.25914E+00_JPRB, 0.25789E+00_JPRB, 0.25566E+00_JPRB, 0.25231E+00_JPRB, 0.24959E+00_JPRB, 0.24678E+00_JPRB ,&
&  0.24181E+00_JPRB, 0.23696E+00_JPRB, 0.23436E+00_JPRB, 0.23377E+00_JPRB, 0.23419E+00_JPRB, 0.23379E+00_JPRB, 0.23206E+00_JPRB ,&
&  0.23005E+00_JPRB, 0.22752E+00_JPRB, 0.22600E+00_JPRB, 0.22561E+00_JPRB, 0.22680E+00_JPRB, 0.22572E+00_JPRB, 0.22154E+00_JPRB ,&
&  0.21924E+00_JPRB, 0.21789E+00_JPRB, 0.21822E+00_JPRB, 0.21747E+00_JPRB, 0.21682E+00_JPRB, 0.21575E+00_JPRB, 0.21615E+00_JPRB ,&
&  0.21722E+00_JPRB, 0.21750E+00_JPRB, 0.22003E+00_JPRB, 0.22449E+00_JPRB, 0.22852E+00_JPRB, 0.23088E+00_JPRB, 0.23233E+00_JPRB ,&
&  0.23292E+00_JPRB, 0.23241E+00_JPRB, 0.23186E+00_JPRB, 0.23237E+00_JPRB, 0.23443E+00_JPRB, 0.23732E+00_JPRB, 0.23993E+00_JPRB ,&
&  0.24048E+00_JPRB, 0.24151E+00_JPRB, 0.24412E+00_JPRB, 0.24806E+00_JPRB, 0.25038E+00_JPRB, 0.25230E+00_JPRB, 0.25390E+00_JPRB ,&
&  0.25464E+00_JPRB, 0.25604E+00_JPRB, 0.25950E+00_JPRB, 0.26165E+00_JPRB, 0.26396E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.25490E+00_JPRB, 0.25588E+00_JPRB, 0.25777E+00_JPRB, 0.25989E+00_JPRB, 0.26074E+00_JPRB, 0.26014E+00_JPRB, 0.25929E+00_JPRB ,&
&  0.25958E+00_JPRB, 0.26155E+00_JPRB, 0.26366E+00_JPRB, 0.26418E+00_JPRB, 0.26448E+00_JPRB, 0.26491E+00_JPRB, 0.26671E+00_JPRB ,&
&  0.26938E+00_JPRB, 0.27250E+00_JPRB, 0.27263E+00_JPRB, 0.27249E+00_JPRB, 0.27224E+00_JPRB, 0.27137E+00_JPRB, 0.27263E+00_JPRB ,&
&  0.27571E+00_JPRB, 0.27599E+00_JPRB, 0.27584E+00_JPRB, 0.27510E+00_JPRB, 0.27664E+00_JPRB, 0.27967E+00_JPRB, 0.28134E+00_JPRB ,&
&  0.27982E+00_JPRB, 0.27294E+00_JPRB, 0.26481E+00_JPRB, 0.26431E+00_JPRB, 0.26595E+00_JPRB, 0.26762E+00_JPRB, 0.26778E+00_JPRB ,&
&  0.26731E+00_JPRB, 0.26647E+00_JPRB, 0.26543E+00_JPRB, 0.26442E+00_JPRB, 0.26322E+00_JPRB, 0.25960E+00_JPRB, 0.25537E+00_JPRB ,&
&  0.25200E+00_JPRB, 0.24887E+00_JPRB, 0.24576E+00_JPRB, 0.24147E+00_JPRB, 0.23708E+00_JPRB, 0.23392E+00_JPRB, 0.23032E+00_JPRB ,&
&  0.22585E+00_JPRB, 0.22267E+00_JPRB, 0.22173E+00_JPRB, 0.22173E+00_JPRB, 0.22190E+00_JPRB, 0.22115E+00_JPRB, 0.22020E+00_JPRB ,&
&  0.21850E+00_JPRB, 0.21720E+00_JPRB, 0.21573E+00_JPRB, 0.21497E+00_JPRB, 0.21003E+00_JPRB, 0.20187E+00_JPRB, 0.19615E+00_JPRB ,&
&  0.19548E+00_JPRB, 0.19829E+00_JPRB, 0.20221E+00_JPRB, 0.20310E+00_JPRB, 0.20505E+00_JPRB, 0.20610E+00_JPRB, 0.20577E+00_JPRB ,&
&  0.20323E+00_JPRB, 0.19958E+00_JPRB, 0.19970E+00_JPRB, 0.20071E+00_JPRB, 0.20110E+00_JPRB, 0.20218E+00_JPRB, 0.20362E+00_JPRB ,&
&  0.20644E+00_JPRB, 0.20883E+00_JPRB, 0.21110E+00_JPRB, 0.21343E+00_JPRB, 0.21499E+00_JPRB, 0.21482E+00_JPRB, 0.21266E+00_JPRB ,&
&  0.21247E+00_JPRB, 0.21783E+00_JPRB, 0.22746E+00_JPRB, 0.23788E+00_JPRB, 0.24037E+00_JPRB, 0.24265E+00_JPRB, 0.24471E+00_JPRB ,&
&  0.24571E+00_JPRB, 0.24883E+00_JPRB, 0.25145E+00_JPRB, 0.25281E+00_JPRB, 0.25377E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.24742E+00_JPRB, 0.24698E+00_JPRB, 0.24556E+00_JPRB, 0.24430E+00_JPRB, 0.24269E+00_JPRB, 0.24118E+00_JPRB, 0.24175E+00_JPRB ,&
&  0.24417E+00_JPRB, 0.24736E+00_JPRB, 0.25044E+00_JPRB, 0.25281E+00_JPRB, 0.25468E+00_JPRB, 0.25563E+00_JPRB, 0.25712E+00_JPRB ,&
&  0.25887E+00_JPRB, 0.26188E+00_JPRB, 0.26299E+00_JPRB, 0.26397E+00_JPRB, 0.26341E+00_JPRB, 0.26284E+00_JPRB, 0.26401E+00_JPRB ,&
&  0.26726E+00_JPRB, 0.26880E+00_JPRB, 0.26798E+00_JPRB, 0.26591E+00_JPRB, 0.26639E+00_JPRB, 0.26821E+00_JPRB, 0.26750E+00_JPRB ,&
&  0.26093E+00_JPRB, 0.24762E+00_JPRB, 0.23840E+00_JPRB, 0.24105E+00_JPRB, 0.24952E+00_JPRB, 0.25338E+00_JPRB, 0.25606E+00_JPRB ,&
&  0.25776E+00_JPRB, 0.25802E+00_JPRB, 0.25725E+00_JPRB, 0.25583E+00_JPRB, 0.25334E+00_JPRB, 0.24896E+00_JPRB, 0.24371E+00_JPRB ,&
&  0.23896E+00_JPRB, 0.23657E+00_JPRB, 0.23368E+00_JPRB, 0.22854E+00_JPRB, 0.22347E+00_JPRB, 0.22028E+00_JPRB, 0.21652E+00_JPRB ,&
&  0.21269E+00_JPRB, 0.20979E+00_JPRB, 0.20914E+00_JPRB, 0.20879E+00_JPRB, 0.20872E+00_JPRB, 0.20819E+00_JPRB, 0.20602E+00_JPRB ,&
&  0.20264E+00_JPRB, 0.19842E+00_JPRB, 0.19280E+00_JPRB, 0.18774E+00_JPRB, 0.18416E+00_JPRB, 0.18065E+00_JPRB, 0.17646E+00_JPRB ,&
&  0.17438E+00_JPRB, 0.17677E+00_JPRB, 0.18524E+00_JPRB, 0.19476E+00_JPRB, 0.20018E+00_JPRB, 0.18863E+00_JPRB, 0.18446E+00_JPRB ,&
&  0.18354E+00_JPRB, 0.18325E+00_JPRB, 0.18355E+00_JPRB, 0.18488E+00_JPRB, 0.18553E+00_JPRB, 0.18529E+00_JPRB, 0.18585E+00_JPRB ,&
&  0.18784E+00_JPRB, 0.18990E+00_JPRB, 0.19099E+00_JPRB, 0.19218E+00_JPRB, 0.19290E+00_JPRB, 0.19268E+00_JPRB, 0.19229E+00_JPRB ,&
&  0.19291E+00_JPRB, 0.19585E+00_JPRB, 0.20072E+00_JPRB, 0.20760E+00_JPRB, 0.21261E+00_JPRB, 0.21718E+00_JPRB, 0.22303E+00_JPRB ,&
&  0.23045E+00_JPRB, 0.23775E+00_JPRB, 0.24173E+00_JPRB, 0.24520E+00_JPRB, 0.24673E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.24050E+00_JPRB, 0.24086E+00_JPRB, 0.23699E+00_JPRB, 0.23357E+00_JPRB, 0.23191E+00_JPRB, 0.23143E+00_JPRB, 0.23269E+00_JPRB ,&
&  0.23565E+00_JPRB, 0.23796E+00_JPRB, 0.24024E+00_JPRB, 0.24223E+00_JPRB, 0.24308E+00_JPRB, 0.24298E+00_JPRB, 0.24400E+00_JPRB ,&
&  0.24502E+00_JPRB, 0.24761E+00_JPRB, 0.24915E+00_JPRB, 0.25181E+00_JPRB, 0.25212E+00_JPRB, 0.25292E+00_JPRB, 0.25417E+00_JPRB ,&
&  0.25376E+00_JPRB, 0.25432E+00_JPRB, 0.25578E+00_JPRB, 0.25358E+00_JPRB, 0.25209E+00_JPRB, 0.24984E+00_JPRB, 0.24797E+00_JPRB ,&
&  0.24160E+00_JPRB, 0.23498E+00_JPRB, 0.22836E+00_JPRB, 0.23172E+00_JPRB, 0.23953E+00_JPRB, 0.24125E+00_JPRB, 0.24505E+00_JPRB ,&
&  0.24610E+00_JPRB, 0.24449E+00_JPRB, 0.24285E+00_JPRB, 0.23973E+00_JPRB, 0.23672E+00_JPRB, 0.23335E+00_JPRB, 0.23066E+00_JPRB ,&
&  0.22768E+00_JPRB, 0.22829E+00_JPRB, 0.22442E+00_JPRB, 0.21114E+00_JPRB, 0.20420E+00_JPRB, 0.19988E+00_JPRB, 0.19307E+00_JPRB ,&
&  0.18628E+00_JPRB, 0.18434E+00_JPRB, 0.18391E+00_JPRB, 0.18169E+00_JPRB, 0.18163E+00_JPRB, 0.18195E+00_JPRB, 0.18188E+00_JPRB ,&
&  0.18060E+00_JPRB, 0.18035E+00_JPRB, 0.17903E+00_JPRB, 0.17707E+00_JPRB, 0.17520E+00_JPRB, 0.17444E+00_JPRB, 0.17481E+00_JPRB ,&
&  0.17628E+00_JPRB, 0.17792E+00_JPRB, 0.18346E+00_JPRB, 0.18700E+00_JPRB, 0.18247E+00_JPRB, 0.17976E+00_JPRB, 0.17997E+00_JPRB ,&
&  0.17614E+00_JPRB, 0.17539E+00_JPRB, 0.17537E+00_JPRB, 0.17536E+00_JPRB, 0.17576E+00_JPRB, 0.17668E+00_JPRB, 0.17665E+00_JPRB ,&
&  0.17732E+00_JPRB, 0.17862E+00_JPRB, 0.17895E+00_JPRB, 0.17896E+00_JPRB, 0.17868E+00_JPRB, 0.17909E+00_JPRB, 0.18027E+00_JPRB ,&
&  0.18167E+00_JPRB, 0.18429E+00_JPRB, 0.18704E+00_JPRB, 0.19106E+00_JPRB, 0.19429E+00_JPRB, 0.19685E+00_JPRB, 0.20146E+00_JPRB ,&
&  0.20548E+00_JPRB, 0.21460E+00_JPRB, 0.23000E+00_JPRB, 0.24017E+00_JPRB, 0.23656E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.23122E+00_JPRB, 0.22989E+00_JPRB, 0.22851E+00_JPRB, 0.22713E+00_JPRB, 0.22795E+00_JPRB, 0.22909E+00_JPRB, 0.23063E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23283E+00_JPRB, 0.23263E+00_JPRB, 0.23047E+00_JPRB, 0.22924E+00_JPRB, 0.22316E+00_JPRB, 0.22097E+00_JPRB ,&
&  0.21937E+00_JPRB, 0.22127E+00_JPRB, 0.22390E+00_JPRB, 0.22785E+00_JPRB, 0.22956E+00_JPRB, 0.23135E+00_JPRB, 0.23044E+00_JPRB ,&
&  0.22830E+00_JPRB, 0.22569E+00_JPRB, 0.22318E+00_JPRB, 0.22120E+00_JPRB, 0.22062E+00_JPRB, 0.21898E+00_JPRB, 0.21723E+00_JPRB ,&
&  0.21715E+00_JPRB, 0.21852E+00_JPRB, 0.21926E+00_JPRB, 0.22002E+00_JPRB, 0.22590E+00_JPRB, 0.22646E+00_JPRB, 0.22851E+00_JPRB ,&
&  0.22909E+00_JPRB, 0.22972E+00_JPRB, 0.22911E+00_JPRB, 0.22781E+00_JPRB, 0.22661E+00_JPRB, 0.22135E+00_JPRB, 0.21908E+00_JPRB ,&
&  0.21193E+00_JPRB, 0.20773E+00_JPRB, 0.20100E+00_JPRB, 0.19495E+00_JPRB, 0.18835E+00_JPRB, 0.18341E+00_JPRB, 0.17926E+00_JPRB ,&
&  0.17596E+00_JPRB, 0.17350E+00_JPRB, 0.17149E+00_JPRB, 0.17122E+00_JPRB, 0.17130E+00_JPRB, 0.17093E+00_JPRB, 0.17109E+00_JPRB ,&
&  0.17191E+00_JPRB, 0.17280E+00_JPRB, 0.17273E+00_JPRB, 0.17330E+00_JPRB, 0.17381E+00_JPRB, 0.17457E+00_JPRB, 0.17603E+00_JPRB ,&
&  0.17828E+00_JPRB, 0.18023E+00_JPRB, 0.18099E+00_JPRB, 0.18281E+00_JPRB, 0.18216E+00_JPRB, 0.17955E+00_JPRB, 0.17696E+00_JPRB ,&
&  0.17408E+00_JPRB, 0.17282E+00_JPRB, 0.17144E+00_JPRB, 0.17068E+00_JPRB, 0.17060E+00_JPRB, 0.17081E+00_JPRB, 0.17160E+00_JPRB ,&
&  0.17244E+00_JPRB, 0.17331E+00_JPRB, 0.17364E+00_JPRB, 0.17407E+00_JPRB, 0.17446E+00_JPRB, 0.17529E+00_JPRB, 0.17686E+00_JPRB ,&
&  0.17901E+00_JPRB, 0.18149E+00_JPRB, 0.18393E+00_JPRB, 0.18689E+00_JPRB, 0.19175E+00_JPRB, 0.19828E+00_JPRB, 0.20819E+00_JPRB ,&
&  0.21605E+00_JPRB, 0.22858E+00_JPRB, 0.22866E+00_JPRB, 0.23046E+00_JPRB, 0.23165E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.22440E+00_JPRB, 0.22424E+00_JPRB, 0.22346E+00_JPRB, 0.22293E+00_JPRB, 0.22337E+00_JPRB, 0.22415E+00_JPRB, 0.22519E+00_JPRB ,&
&  0.22178E+00_JPRB, 0.21922E+00_JPRB, 0.21489E+00_JPRB, 0.21010E+00_JPRB, 0.20591E+00_JPRB, 0.20259E+00_JPRB, 0.20067E+00_JPRB ,&
&  0.20048E+00_JPRB, 0.20256E+00_JPRB, 0.20421E+00_JPRB, 0.20621E+00_JPRB, 0.20636E+00_JPRB, 0.20655E+00_JPRB, 0.20518E+00_JPRB ,&
&  0.20369E+00_JPRB, 0.20310E+00_JPRB, 0.20335E+00_JPRB, 0.20503E+00_JPRB, 0.20770E+00_JPRB, 0.20947E+00_JPRB, 0.21159E+00_JPRB ,&
&  0.21442E+00_JPRB, 0.21371E+00_JPRB, 0.21445E+00_JPRB, 0.21386E+00_JPRB, 0.21419E+00_JPRB, 0.21512E+00_JPRB, 0.21558E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21746E+00_JPRB, 0.21742E+00_JPRB, 0.21674E+00_JPRB, 0.21257E+00_JPRB, 0.21000E+00_JPRB, 0.20540E+00_JPRB ,&
&  0.19989E+00_JPRB, 0.19497E+00_JPRB, 0.18934E+00_JPRB, 0.18540E+00_JPRB, 0.18078E+00_JPRB, 0.17805E+00_JPRB, 0.17593E+00_JPRB ,&
&  0.17394E+00_JPRB, 0.17318E+00_JPRB, 0.17210E+00_JPRB, 0.17094E+00_JPRB, 0.17068E+00_JPRB, 0.16953E+00_JPRB, 0.16944E+00_JPRB ,&
&  0.16972E+00_JPRB, 0.16998E+00_JPRB, 0.17094E+00_JPRB, 0.17140E+00_JPRB, 0.17219E+00_JPRB, 0.17293E+00_JPRB, 0.17350E+00_JPRB ,&
&  0.17464E+00_JPRB, 0.17610E+00_JPRB, 0.17613E+00_JPRB, 0.17645E+00_JPRB, 0.17563E+00_JPRB, 0.17386E+00_JPRB, 0.17194E+00_JPRB ,&
&  0.16989E+00_JPRB, 0.16829E+00_JPRB, 0.16780E+00_JPRB, 0.16710E+00_JPRB, 0.16725E+00_JPRB, 0.16763E+00_JPRB, 0.16781E+00_JPRB ,&
&  0.16847E+00_JPRB, 0.16906E+00_JPRB, 0.16989E+00_JPRB, 0.17173E+00_JPRB, 0.17302E+00_JPRB, 0.17468E+00_JPRB, 0.17643E+00_JPRB ,&
&  0.17782E+00_JPRB, 0.18035E+00_JPRB, 0.18258E+00_JPRB, 0.18563E+00_JPRB, 0.19263E+00_JPRB, 0.19877E+00_JPRB, 0.20973E+00_JPRB ,&
&  0.21477E+00_JPRB, 0.22616E+00_JPRB, 0.22409E+00_JPRB, 0.22387E+00_JPRB, 0.22483E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.21927E+00_JPRB, 0.22011E+00_JPRB, 0.22006E+00_JPRB, 0.22101E+00_JPRB, 0.22180E+00_JPRB, 0.22284E+00_JPRB, 0.22297E+00_JPRB ,&
&  0.22242E+00_JPRB, 0.21825E+00_JPRB, 0.21593E+00_JPRB, 0.21261E+00_JPRB, 0.20946E+00_JPRB, 0.20614E+00_JPRB, 0.20343E+00_JPRB ,&
&  0.20168E+00_JPRB, 0.20033E+00_JPRB, 0.19912E+00_JPRB, 0.19742E+00_JPRB, 0.19533E+00_JPRB, 0.19323E+00_JPRB, 0.19131E+00_JPRB ,&
&  0.18958E+00_JPRB, 0.18965E+00_JPRB, 0.19089E+00_JPRB, 0.19307E+00_JPRB, 0.19707E+00_JPRB, 0.20014E+00_JPRB, 0.20409E+00_JPRB ,&
&  0.21022E+00_JPRB, 0.20704E+00_JPRB, 0.20722E+00_JPRB, 0.20751E+00_JPRB, 0.20728E+00_JPRB, 0.20713E+00_JPRB, 0.20431E+00_JPRB ,&
&  0.20796E+00_JPRB, 0.20820E+00_JPRB, 0.21174E+00_JPRB, 0.21088E+00_JPRB, 0.20632E+00_JPRB, 0.20271E+00_JPRB, 0.19773E+00_JPRB ,&
&  0.19414E+00_JPRB, 0.18963E+00_JPRB, 0.18659E+00_JPRB, 0.18383E+00_JPRB, 0.18059E+00_JPRB, 0.17926E+00_JPRB, 0.17702E+00_JPRB ,&
&  0.17566E+00_JPRB, 0.17536E+00_JPRB, 0.17440E+00_JPRB, 0.17435E+00_JPRB, 0.17352E+00_JPRB, 0.17236E+00_JPRB, 0.17159E+00_JPRB ,&
&  0.17091E+00_JPRB, 0.17021E+00_JPRB, 0.16997E+00_JPRB, 0.16972E+00_JPRB, 0.16986E+00_JPRB, 0.16988E+00_JPRB, 0.16995E+00_JPRB ,&
&  0.17017E+00_JPRB, 0.17034E+00_JPRB, 0.16995E+00_JPRB, 0.16982E+00_JPRB, 0.16893E+00_JPRB, 0.16815E+00_JPRB, 0.16693E+00_JPRB ,&
&  0.16618E+00_JPRB, 0.16574E+00_JPRB, 0.16505E+00_JPRB, 0.16459E+00_JPRB, 0.16445E+00_JPRB, 0.16494E+00_JPRB, 0.16555E+00_JPRB ,&
&  0.16626E+00_JPRB, 0.16756E+00_JPRB, 0.16863E+00_JPRB, 0.17006E+00_JPRB, 0.17166E+00_JPRB, 0.17291E+00_JPRB, 0.17472E+00_JPRB ,&
&  0.17651E+00_JPRB, 0.17835E+00_JPRB, 0.18146E+00_JPRB, 0.18491E+00_JPRB, 0.18949E+00_JPRB, 0.19454E+00_JPRB, 0.19727E+00_JPRB ,&
&  0.20034E+00_JPRB, 0.20716E+00_JPRB, 0.20978E+00_JPRB, 0.21729E+00_JPRB, 0.21846E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.20997E+00_JPRB, 0.21150E+00_JPRB, 0.21298E+00_JPRB, 0.21439E+00_JPRB, 0.21598E+00_JPRB, 0.21662E+00_JPRB, 0.21656E+00_JPRB ,&
&  0.21421E+00_JPRB, 0.21365E+00_JPRB, 0.21211E+00_JPRB, 0.21020E+00_JPRB, 0.20865E+00_JPRB, 0.20570E+00_JPRB, 0.20291E+00_JPRB ,&
&  0.20010E+00_JPRB, 0.19740E+00_JPRB, 0.19502E+00_JPRB, 0.19173E+00_JPRB, 0.18859E+00_JPRB, 0.18548E+00_JPRB, 0.18460E+00_JPRB ,&
&  0.18426E+00_JPRB, 0.18481E+00_JPRB, 0.18656E+00_JPRB, 0.18796E+00_JPRB, 0.19002E+00_JPRB, 0.19220E+00_JPRB, 0.19549E+00_JPRB ,&
&  0.19864E+00_JPRB, 0.20096E+00_JPRB, 0.20130E+00_JPRB, 0.20015E+00_JPRB, 0.19921E+00_JPRB, 0.19896E+00_JPRB, 0.19860E+00_JPRB ,&
&  0.19789E+00_JPRB, 0.19751E+00_JPRB, 0.19695E+00_JPRB, 0.19648E+00_JPRB, 0.19508E+00_JPRB, 0.19254E+00_JPRB, 0.18997E+00_JPRB ,&
&  0.18757E+00_JPRB, 0.18544E+00_JPRB, 0.18341E+00_JPRB, 0.18164E+00_JPRB, 0.18060E+00_JPRB, 0.17955E+00_JPRB, 0.17823E+00_JPRB ,&
&  0.17746E+00_JPRB, 0.17665E+00_JPRB, 0.17572E+00_JPRB, 0.17503E+00_JPRB, 0.17426E+00_JPRB, 0.17371E+00_JPRB, 0.17309E+00_JPRB ,&
&  0.17262E+00_JPRB, 0.17190E+00_JPRB, 0.17128E+00_JPRB, 0.17060E+00_JPRB, 0.17030E+00_JPRB, 0.16954E+00_JPRB, 0.16882E+00_JPRB ,&
&  0.16791E+00_JPRB, 0.16736E+00_JPRB, 0.16632E+00_JPRB, 0.16571E+00_JPRB, 0.16499E+00_JPRB, 0.16432E+00_JPRB, 0.16386E+00_JPRB ,&
&  0.16390E+00_JPRB, 0.16388E+00_JPRB, 0.16341E+00_JPRB, 0.16330E+00_JPRB, 0.16331E+00_JPRB, 0.16374E+00_JPRB, 0.16503E+00_JPRB ,&
&  0.16587E+00_JPRB, 0.16679E+00_JPRB, 0.16767E+00_JPRB, 0.16837E+00_JPRB, 0.16954E+00_JPRB, 0.17067E+00_JPRB, 0.17182E+00_JPRB ,&
&  0.17319E+00_JPRB, 0.17435E+00_JPRB, 0.17572E+00_JPRB, 0.17731E+00_JPRB, 0.18124E+00_JPRB, 0.18397E+00_JPRB, 0.18684E+00_JPRB ,&
&  0.18889E+00_JPRB, 0.18763E+00_JPRB, 0.20171E+00_JPRB, 0.20439E+00_JPRB, 0.20642E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.18507E+00_JPRB, 0.18796E+00_JPRB, 0.18977E+00_JPRB, 0.19130E+00_JPRB, 0.18542E+00_JPRB, 0.19679E+00_JPRB, 0.19355E+00_JPRB ,&
&  0.19302E+00_JPRB, 0.19172E+00_JPRB, 0.19336E+00_JPRB, 0.19340E+00_JPRB, 0.19277E+00_JPRB, 0.19185E+00_JPRB, 0.19066E+00_JPRB ,&
&  0.18953E+00_JPRB, 0.18811E+00_JPRB, 0.18720E+00_JPRB, 0.18621E+00_JPRB, 0.18543E+00_JPRB, 0.18466E+00_JPRB, 0.18414E+00_JPRB ,&
&  0.18415E+00_JPRB, 0.18496E+00_JPRB, 0.18641E+00_JPRB, 0.18581E+00_JPRB, 0.18583E+00_JPRB, 0.18624E+00_JPRB, 0.18685E+00_JPRB ,&
&  0.18712E+00_JPRB, 0.18725E+00_JPRB, 0.18699E+00_JPRB, 0.18594E+00_JPRB, 0.18596E+00_JPRB, 0.18601E+00_JPRB, 0.18615E+00_JPRB ,&
&  0.18611E+00_JPRB, 0.18585E+00_JPRB, 0.18499E+00_JPRB, 0.18420E+00_JPRB, 0.18338E+00_JPRB, 0.18240E+00_JPRB, 0.18121E+00_JPRB ,&
&  0.18023E+00_JPRB, 0.17947E+00_JPRB, 0.17856E+00_JPRB, 0.17765E+00_JPRB, 0.17686E+00_JPRB, 0.17610E+00_JPRB, 0.17538E+00_JPRB ,&
&  0.17491E+00_JPRB, 0.17457E+00_JPRB, 0.17419E+00_JPRB, 0.17364E+00_JPRB, 0.17302E+00_JPRB, 0.17243E+00_JPRB, 0.17190E+00_JPRB ,&
&  0.17155E+00_JPRB, 0.17098E+00_JPRB, 0.17041E+00_JPRB, 0.16979E+00_JPRB, 0.16933E+00_JPRB, 0.16856E+00_JPRB, 0.16789E+00_JPRB ,&
&  0.16723E+00_JPRB, 0.16664E+00_JPRB, 0.16619E+00_JPRB, 0.16586E+00_JPRB, 0.16555E+00_JPRB, 0.16533E+00_JPRB, 0.16517E+00_JPRB ,&
&  0.16506E+00_JPRB, 0.16497E+00_JPRB, 0.16497E+00_JPRB, 0.16490E+00_JPRB, 0.16480E+00_JPRB, 0.16478E+00_JPRB, 0.16505E+00_JPRB ,&
&  0.16530E+00_JPRB, 0.16547E+00_JPRB, 0.16558E+00_JPRB, 0.16529E+00_JPRB, 0.16569E+00_JPRB, 0.16603E+00_JPRB, 0.16644E+00_JPRB ,&
&  0.16682E+00_JPRB, 0.16761E+00_JPRB, 0.16855E+00_JPRB, 0.16968E+00_JPRB, 0.17185E+00_JPRB, 0.17378E+00_JPRB, 0.17582E+00_JPRB ,&
&  0.17811E+00_JPRB, 0.18142E+00_JPRB, 0.18280E+00_JPRB, 0.18370E+00_JPRB, 0.18441E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.17419E+00_JPRB, 0.17414E+00_JPRB, 0.17395E+00_JPRB, 0.17321E+00_JPRB, 0.17100E+00_JPRB, 0.17061E+00_JPRB, 0.17058E+00_JPRB ,&
&  0.17076E+00_JPRB, 0.17088E+00_JPRB, 0.17174E+00_JPRB, 0.17268E+00_JPRB, 0.17360E+00_JPRB, 0.17430E+00_JPRB, 0.17491E+00_JPRB ,&
&  0.17524E+00_JPRB, 0.17557E+00_JPRB, 0.17447E+00_JPRB, 0.17395E+00_JPRB, 0.17380E+00_JPRB, 0.17380E+00_JPRB, 0.17380E+00_JPRB ,&
&  0.17375E+00_JPRB, 0.17374E+00_JPRB, 0.17362E+00_JPRB, 0.17382E+00_JPRB, 0.17413E+00_JPRB, 0.17439E+00_JPRB, 0.17469E+00_JPRB ,&
&  0.17492E+00_JPRB, 0.17512E+00_JPRB, 0.17525E+00_JPRB, 0.17535E+00_JPRB, 0.17567E+00_JPRB, 0.17583E+00_JPRB, 0.17575E+00_JPRB ,&
&  0.17558E+00_JPRB, 0.17523E+00_JPRB, 0.17485E+00_JPRB, 0.17444E+00_JPRB, 0.17416E+00_JPRB, 0.17388E+00_JPRB, 0.17360E+00_JPRB ,&
&  0.17332E+00_JPRB, 0.17302E+00_JPRB, 0.17269E+00_JPRB, 0.17233E+00_JPRB, 0.17198E+00_JPRB, 0.17172E+00_JPRB, 0.17144E+00_JPRB ,&
&  0.17121E+00_JPRB, 0.17099E+00_JPRB, 0.17075E+00_JPRB, 0.17044E+00_JPRB, 0.17012E+00_JPRB, 0.16979E+00_JPRB, 0.16944E+00_JPRB ,&
&  0.16899E+00_JPRB, 0.16866E+00_JPRB, 0.16834E+00_JPRB, 0.16806E+00_JPRB, 0.16782E+00_JPRB, 0.16756E+00_JPRB, 0.16734E+00_JPRB ,&
&  0.16709E+00_JPRB, 0.16686E+00_JPRB, 0.16659E+00_JPRB, 0.16635E+00_JPRB, 0.16612E+00_JPRB, 0.16592E+00_JPRB, 0.16576E+00_JPRB ,&
&  0.16560E+00_JPRB, 0.16544E+00_JPRB, 0.16531E+00_JPRB, 0.16531E+00_JPRB, 0.16528E+00_JPRB, 0.16531E+00_JPRB, 0.16533E+00_JPRB ,&
&  0.16536E+00_JPRB, 0.16537E+00_JPRB, 0.16538E+00_JPRB, 0.16527E+00_JPRB, 0.16541E+00_JPRB, 0.16561E+00_JPRB, 0.16592E+00_JPRB ,&
&  0.16628E+00_JPRB, 0.16675E+00_JPRB, 0.16724E+00_JPRB, 0.16776E+00_JPRB, 0.16837E+00_JPRB, 0.16888E+00_JPRB, 0.16943E+00_JPRB ,&
&  0.17007E+00_JPRB, 0.17072E+00_JPRB, 0.17120E+00_JPRB, 0.17175E+00_JPRB, 0.17261E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB, 0.16842E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB, 0.48295E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.49062E+00_JPRB, 0.49008E+00_JPRB, 0.48965E+00_JPRB, 0.48950E+00_JPRB, 0.48954E+00_JPRB, 0.48952E+00_JPRB, 0.48884E+00_JPRB ,&
&  0.48802E+00_JPRB, 0.48689E+00_JPRB, 0.48563E+00_JPRB, 0.48414E+00_JPRB, 0.48233E+00_JPRB, 0.48017E+00_JPRB, 0.47818E+00_JPRB ,&
&  0.47632E+00_JPRB, 0.47481E+00_JPRB, 0.47321E+00_JPRB, 0.47207E+00_JPRB, 0.47074E+00_JPRB, 0.46931E+00_JPRB, 0.46784E+00_JPRB ,&
&  0.46667E+00_JPRB, 0.46558E+00_JPRB, 0.46453E+00_JPRB, 0.46358E+00_JPRB, 0.46309E+00_JPRB, 0.46251E+00_JPRB, 0.46191E+00_JPRB ,&
&  0.46140E+00_JPRB, 0.46108E+00_JPRB, 0.46080E+00_JPRB, 0.46052E+00_JPRB, 0.46030E+00_JPRB, 0.46039E+00_JPRB, 0.46049E+00_JPRB ,&
&  0.46072E+00_JPRB, 0.46139E+00_JPRB, 0.46246E+00_JPRB, 0.46344E+00_JPRB, 0.46451E+00_JPRB, 0.46601E+00_JPRB, 0.46777E+00_JPRB ,&
&  0.46929E+00_JPRB, 0.47068E+00_JPRB, 0.47229E+00_JPRB, 0.47405E+00_JPRB, 0.47565E+00_JPRB, 0.47736E+00_JPRB, 0.47898E+00_JPRB ,&
&  0.48061E+00_JPRB, 0.48195E+00_JPRB, 0.48335E+00_JPRB, 0.48393E+00_JPRB, 0.48484E+00_JPRB, 0.48559E+00_JPRB, 0.48649E+00_JPRB ,&
&  0.48664E+00_JPRB, 0.48669E+00_JPRB, 0.48683E+00_JPRB, 0.48717E+00_JPRB, 0.48735E+00_JPRB, 0.48762E+00_JPRB, 0.48797E+00_JPRB ,&
&  0.48851E+00_JPRB, 0.48878E+00_JPRB, 0.48927E+00_JPRB, 0.48961E+00_JPRB, 0.49008E+00_JPRB, 0.49015E+00_JPRB, 0.49043E+00_JPRB ,&
&  0.49070E+00_JPRB, 0.49118E+00_JPRB, 0.49128E+00_JPRB, 0.49143E+00_JPRB, 0.49150E+00_JPRB, 0.49167E+00_JPRB, 0.49180E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49219E+00_JPRB, 0.49235E+00_JPRB, 0.49228E+00_JPRB, 0.49197E+00_JPRB, 0.49167E+00_JPRB, 0.49147E+00_JPRB ,&
&  0.49143E+00_JPRB, 0.49141E+00_JPRB, 0.49166E+00_JPRB, 0.49202E+00_JPRB, 0.49268E+00_JPRB, 0.49284E+00_JPRB, 0.49296E+00_JPRB ,&
&  0.49299E+00_JPRB, 0.49298E+00_JPRB, 0.49254E+00_JPRB, 0.49192E+00_JPRB, 0.49120E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.49215E+00_JPRB, 0.49100E+00_JPRB, 0.49031E+00_JPRB, 0.49025E+00_JPRB, 0.49046E+00_JPRB, 0.49053E+00_JPRB, 0.49055E+00_JPRB ,&
&  0.48958E+00_JPRB, 0.48714E+00_JPRB, 0.48418E+00_JPRB, 0.48090E+00_JPRB, 0.47682E+00_JPRB, 0.47303E+00_JPRB, 0.47041E+00_JPRB ,&
&  0.46761E+00_JPRB, 0.46481E+00_JPRB, 0.46208E+00_JPRB, 0.45978E+00_JPRB, 0.45720E+00_JPRB, 0.45448E+00_JPRB, 0.45183E+00_JPRB ,&
&  0.44966E+00_JPRB, 0.44718E+00_JPRB, 0.44442E+00_JPRB, 0.44097E+00_JPRB, 0.43859E+00_JPRB, 0.43609E+00_JPRB, 0.43389E+00_JPRB ,&
&  0.43115E+00_JPRB, 0.42982E+00_JPRB, 0.42900E+00_JPRB, 0.42888E+00_JPRB, 0.43076E+00_JPRB, 0.43233E+00_JPRB, 0.43254E+00_JPRB ,&
&  0.43207E+00_JPRB, 0.42980E+00_JPRB, 0.43017E+00_JPRB, 0.43105E+00_JPRB, 0.43270E+00_JPRB, 0.43611E+00_JPRB, 0.44061E+00_JPRB ,&
&  0.44696E+00_JPRB, 0.45457E+00_JPRB, 0.46229E+00_JPRB, 0.46829E+00_JPRB, 0.47415E+00_JPRB, 0.47949E+00_JPRB, 0.48328E+00_JPRB ,&
&  0.48781E+00_JPRB, 0.49157E+00_JPRB, 0.49550E+00_JPRB, 0.49904E+00_JPRB, 0.50299E+00_JPRB, 0.50607E+00_JPRB, 0.50837E+00_JPRB ,&
&  0.50889E+00_JPRB, 0.50850E+00_JPRB, 0.50814E+00_JPRB, 0.50796E+00_JPRB, 0.50684E+00_JPRB, 0.50603E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50595E+00_JPRB, 0.50578E+00_JPRB, 0.50641E+00_JPRB, 0.50723E+00_JPRB, 0.50820E+00_JPRB, 0.50777E+00_JPRB, 0.50790E+00_JPRB ,&
&  0.50783E+00_JPRB, 0.50787E+00_JPRB, 0.50698E+00_JPRB, 0.50539E+00_JPRB, 0.50400E+00_JPRB, 0.50332E+00_JPRB, 0.50233E+00_JPRB ,&
&  0.50137E+00_JPRB, 0.50067E+00_JPRB, 0.50044E+00_JPRB, 0.49998E+00_JPRB, 0.49899E+00_JPRB, 0.49809E+00_JPRB, 0.49776E+00_JPRB ,&
&  0.49746E+00_JPRB, 0.49688E+00_JPRB, 0.49725E+00_JPRB, 0.49773E+00_JPRB, 0.49950E+00_JPRB, 0.49991E+00_JPRB, 0.50008E+00_JPRB ,&
&  0.49946E+00_JPRB, 0.49903E+00_JPRB, 0.49771E+00_JPRB, 0.49583E+00_JPRB, 0.49362E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.49985E+00_JPRB, 0.49942E+00_JPRB, 0.50038E+00_JPRB, 0.49981E+00_JPRB, 0.49894E+00_JPRB, 0.49697E+00_JPRB, 0.49604E+00_JPRB ,&
&  0.49363E+00_JPRB, 0.48940E+00_JPRB, 0.48257E+00_JPRB, 0.47647E+00_JPRB, 0.47031E+00_JPRB, 0.46633E+00_JPRB, 0.46244E+00_JPRB ,&
&  0.45854E+00_JPRB, 0.45530E+00_JPRB, 0.45181E+00_JPRB, 0.44761E+00_JPRB, 0.44249E+00_JPRB, 0.43636E+00_JPRB, 0.42848E+00_JPRB ,&
&  0.42240E+00_JPRB, 0.41657E+00_JPRB, 0.41145E+00_JPRB, 0.40792E+00_JPRB, 0.40678E+00_JPRB, 0.39260E+00_JPRB, 0.38981E+00_JPRB ,&
&  0.39562E+00_JPRB, 0.38818E+00_JPRB, 0.38139E+00_JPRB, 0.37043E+00_JPRB, 0.36869E+00_JPRB, 0.37517E+00_JPRB, 0.37980E+00_JPRB ,&
&  0.37422E+00_JPRB, 0.39225E+00_JPRB, 0.39365E+00_JPRB, 0.39445E+00_JPRB, 0.39331E+00_JPRB, 0.39087E+00_JPRB, 0.39801E+00_JPRB ,&
&  0.41429E+00_JPRB, 0.42136E+00_JPRB, 0.43661E+00_JPRB, 0.45280E+00_JPRB, 0.46338E+00_JPRB, 0.47509E+00_JPRB, 0.48720E+00_JPRB ,&
&  0.49879E+00_JPRB, 0.50702E+00_JPRB, 0.51322E+00_JPRB, 0.51925E+00_JPRB, 0.52808E+00_JPRB, 0.53674E+00_JPRB, 0.54429E+00_JPRB ,&
&  0.54961E+00_JPRB, 0.55268E+00_JPRB, 0.55645E+00_JPRB, 0.55996E+00_JPRB, 0.56505E+00_JPRB, 0.56943E+00_JPRB, 0.57387E+00_JPRB ,&
&  0.57727E+00_JPRB, 0.57986E+00_JPRB, 0.58123E+00_JPRB, 0.58122E+00_JPRB, 0.58060E+00_JPRB, 0.57759E+00_JPRB, 0.57626E+00_JPRB ,&
&  0.57198E+00_JPRB, 0.56762E+00_JPRB, 0.56205E+00_JPRB, 0.55516E+00_JPRB, 0.54993E+00_JPRB, 0.54593E+00_JPRB, 0.54155E+00_JPRB ,&
&  0.53712E+00_JPRB, 0.53276E+00_JPRB, 0.52899E+00_JPRB, 0.52412E+00_JPRB, 0.51961E+00_JPRB, 0.51490E+00_JPRB, 0.51187E+00_JPRB ,&
&  0.50994E+00_JPRB, 0.50855E+00_JPRB, 0.50869E+00_JPRB, 0.50854E+00_JPRB, 0.50984E+00_JPRB, 0.51019E+00_JPRB, 0.51136E+00_JPRB ,&
&  0.51142E+00_JPRB, 0.50920E+00_JPRB, 0.50556E+00_JPRB, 0.50259E+00_JPRB, 0.50005E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.50566E+00_JPRB, 0.50570E+00_JPRB, 0.50863E+00_JPRB, 0.50751E+00_JPRB, 0.50599E+00_JPRB, 0.50116E+00_JPRB, 0.49932E+00_JPRB ,&
&  0.49403E+00_JPRB, 0.48883E+00_JPRB, 0.47766E+00_JPRB, 0.47035E+00_JPRB, 0.46347E+00_JPRB, 0.45938E+00_JPRB, 0.45577E+00_JPRB ,&
&  0.45287E+00_JPRB, 0.44931E+00_JPRB, 0.44500E+00_JPRB, 0.43877E+00_JPRB, 0.43110E+00_JPRB, 0.42228E+00_JPRB, 0.41018E+00_JPRB ,&
&  0.39930E+00_JPRB, 0.39155E+00_JPRB, 0.38061E+00_JPRB, 0.38379E+00_JPRB, 0.37912E+00_JPRB, 0.37564E+00_JPRB, 0.37385E+00_JPRB ,&
&  0.36890E+00_JPRB, 0.36072E+00_JPRB, 0.34985E+00_JPRB, 0.34201E+00_JPRB, 0.33559E+00_JPRB, 0.32880E+00_JPRB, 0.33537E+00_JPRB ,&
&  0.34041E+00_JPRB, 0.35623E+00_JPRB, 0.36259E+00_JPRB, 0.36421E+00_JPRB, 0.36678E+00_JPRB, 0.36771E+00_JPRB, 0.37426E+00_JPRB ,&
&  0.38324E+00_JPRB, 0.39365E+00_JPRB, 0.42698E+00_JPRB, 0.44396E+00_JPRB, 0.45355E+00_JPRB, 0.46602E+00_JPRB, 0.47652E+00_JPRB ,&
&  0.48945E+00_JPRB, 0.49882E+00_JPRB, 0.50620E+00_JPRB, 0.51577E+00_JPRB, 0.52945E+00_JPRB, 0.54525E+00_JPRB, 0.55891E+00_JPRB ,&
&  0.57136E+00_JPRB, 0.57867E+00_JPRB, 0.58821E+00_JPRB, 0.59873E+00_JPRB, 0.61376E+00_JPRB, 0.62977E+00_JPRB, 0.64695E+00_JPRB ,&
&  0.66464E+00_JPRB, 0.68286E+00_JPRB, 0.69709E+00_JPRB, 0.70944E+00_JPRB, 0.71447E+00_JPRB, 0.71292E+00_JPRB, 0.70787E+00_JPRB ,&
&  0.69463E+00_JPRB, 0.68089E+00_JPRB, 0.66358E+00_JPRB, 0.63905E+00_JPRB, 0.62105E+00_JPRB, 0.61225E+00_JPRB, 0.59563E+00_JPRB ,&
&  0.58217E+00_JPRB, 0.56972E+00_JPRB, 0.55858E+00_JPRB, 0.54568E+00_JPRB, 0.53395E+00_JPRB, 0.52271E+00_JPRB, 0.51515E+00_JPRB ,&
&  0.50937E+00_JPRB, 0.50600E+00_JPRB, 0.50452E+00_JPRB, 0.50426E+00_JPRB, 0.50621E+00_JPRB, 0.50801E+00_JPRB, 0.51215E+00_JPRB ,&
&  0.51420E+00_JPRB, 0.51363E+00_JPRB, 0.51029E+00_JPRB, 0.50765E+00_JPRB, 0.50517E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.51570E+00_JPRB, 0.51472E+00_JPRB, 0.51312E+00_JPRB, 0.50657E+00_JPRB, 0.50222E+00_JPRB, 0.49665E+00_JPRB, 0.49459E+00_JPRB ,&
&  0.48860E+00_JPRB, 0.48335E+00_JPRB, 0.47178E+00_JPRB, 0.46479E+00_JPRB, 0.45679E+00_JPRB, 0.45238E+00_JPRB, 0.44817E+00_JPRB ,&
&  0.44555E+00_JPRB, 0.44208E+00_JPRB, 0.43995E+00_JPRB, 0.43433E+00_JPRB, 0.42892E+00_JPRB, 0.42003E+00_JPRB, 0.41113E+00_JPRB ,&
&  0.40116E+00_JPRB, 0.39133E+00_JPRB, 0.38254E+00_JPRB, 0.38409E+00_JPRB, 0.37978E+00_JPRB, 0.37675E+00_JPRB, 0.37542E+00_JPRB ,&
&  0.38119E+00_JPRB, 0.34967E+00_JPRB, 0.35008E+00_JPRB, 0.33975E+00_JPRB, 0.32802E+00_JPRB, 0.31428E+00_JPRB, 0.30416E+00_JPRB ,&
&  0.30602E+00_JPRB, 0.30745E+00_JPRB, 0.31427E+00_JPRB, 0.32478E+00_JPRB, 0.32941E+00_JPRB, 0.33924E+00_JPRB, 0.34742E+00_JPRB ,&
&  0.34809E+00_JPRB, 0.40030E+00_JPRB, 0.42523E+00_JPRB, 0.43908E+00_JPRB, 0.45437E+00_JPRB, 0.46954E+00_JPRB, 0.48364E+00_JPRB ,&
&  0.50193E+00_JPRB, 0.52283E+00_JPRB, 0.54080E+00_JPRB, 0.55772E+00_JPRB, 0.58375E+00_JPRB, 0.60162E+00_JPRB, 0.61688E+00_JPRB ,&
&  0.63044E+00_JPRB, 0.63477E+00_JPRB, 0.64429E+00_JPRB, 0.65740E+00_JPRB, 0.67635E+00_JPRB, 0.69712E+00_JPRB, 0.72116E+00_JPRB ,&
&  0.74722E+00_JPRB, 0.77169E+00_JPRB, 0.79672E+00_JPRB, 0.81719E+00_JPRB, 0.83119E+00_JPRB, 0.83772E+00_JPRB, 0.84466E+00_JPRB ,&
&  0.83039E+00_JPRB, 0.81285E+00_JPRB, 0.78654E+00_JPRB, 0.75464E+00_JPRB, 0.71811E+00_JPRB, 0.67854E+00_JPRB, 0.63939E+00_JPRB ,&
&  0.61995E+00_JPRB, 0.59874E+00_JPRB, 0.57869E+00_JPRB, 0.55889E+00_JPRB, 0.54117E+00_JPRB, 0.52617E+00_JPRB, 0.51745E+00_JPRB ,&
&  0.50937E+00_JPRB, 0.50436E+00_JPRB, 0.50145E+00_JPRB, 0.49977E+00_JPRB, 0.49950E+00_JPRB, 0.49850E+00_JPRB, 0.50010E+00_JPRB ,&
&  0.50324E+00_JPRB, 0.50798E+00_JPRB, 0.51022E+00_JPRB, 0.51245E+00_JPRB, 0.51398E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.52658E+00_JPRB, 0.52927E+00_JPRB, 0.52850E+00_JPRB, 0.52455E+00_JPRB, 0.52089E+00_JPRB, 0.51218E+00_JPRB, 0.50543E+00_JPRB ,&
&  0.49998E+00_JPRB, 0.49496E+00_JPRB, 0.48692E+00_JPRB, 0.47839E+00_JPRB, 0.46741E+00_JPRB, 0.45957E+00_JPRB, 0.45123E+00_JPRB ,&
&  0.44492E+00_JPRB, 0.43767E+00_JPRB, 0.42998E+00_JPRB, 0.42301E+00_JPRB, 0.41775E+00_JPRB, 0.41113E+00_JPRB, 0.40671E+00_JPRB ,&
&  0.40124E+00_JPRB, 0.39372E+00_JPRB, 0.38822E+00_JPRB, 0.38834E+00_JPRB, 0.38374E+00_JPRB, 0.38162E+00_JPRB, 0.38375E+00_JPRB ,&
&  0.38997E+00_JPRB, 0.38648E+00_JPRB, 0.38245E+00_JPRB, 0.36939E+00_JPRB, 0.35027E+00_JPRB, 0.30157E+00_JPRB, 0.30585E+00_JPRB ,&
&  0.30203E+00_JPRB, 0.29604E+00_JPRB, 0.29800E+00_JPRB, 0.30341E+00_JPRB, 0.30936E+00_JPRB, 0.32619E+00_JPRB, 0.32886E+00_JPRB ,&
&  0.32577E+00_JPRB, 0.39846E+00_JPRB, 0.41530E+00_JPRB, 0.43580E+00_JPRB, 0.45734E+00_JPRB, 0.48148E+00_JPRB, 0.50705E+00_JPRB ,&
&  0.52032E+00_JPRB, 0.54195E+00_JPRB, 0.56062E+00_JPRB, 0.59160E+00_JPRB, 0.61671E+00_JPRB, 0.64506E+00_JPRB, 0.66910E+00_JPRB ,&
&  0.67958E+00_JPRB, 0.68466E+00_JPRB, 0.69236E+00_JPRB, 0.69642E+00_JPRB, 0.70515E+00_JPRB, 0.72436E+00_JPRB, 0.75293E+00_JPRB ,&
&  0.79191E+00_JPRB, 0.80708E+00_JPRB, 0.85375E+00_JPRB, 0.88556E+00_JPRB, 0.89770E+00_JPRB, 0.92016E+00_JPRB, 0.94836E+00_JPRB ,&
&  0.93748E+00_JPRB, 0.93021E+00_JPRB, 0.90874E+00_JPRB, 0.84468E+00_JPRB, 0.77811E+00_JPRB, 0.73408E+00_JPRB, 0.69396E+00_JPRB ,&
&  0.66365E+00_JPRB, 0.63388E+00_JPRB, 0.60090E+00_JPRB, 0.57547E+00_JPRB, 0.55559E+00_JPRB, 0.54093E+00_JPRB, 0.53323E+00_JPRB ,&
&  0.52421E+00_JPRB, 0.51603E+00_JPRB, 0.50890E+00_JPRB, 0.50651E+00_JPRB, 0.50006E+00_JPRB, 0.49462E+00_JPRB, 0.48890E+00_JPRB ,&
&  0.48912E+00_JPRB, 0.49219E+00_JPRB, 0.50135E+00_JPRB, 0.51095E+00_JPRB, 0.51768E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.53036E+00_JPRB, 0.54530E+00_JPRB, 0.55492E+00_JPRB, 0.55914E+00_JPRB, 0.56452E+00_JPRB, 0.55123E+00_JPRB, 0.53881E+00_JPRB ,&
&  0.52876E+00_JPRB, 0.51729E+00_JPRB, 0.50711E+00_JPRB, 0.49744E+00_JPRB, 0.48771E+00_JPRB, 0.48069E+00_JPRB, 0.47447E+00_JPRB ,&
&  0.46936E+00_JPRB, 0.44102E+00_JPRB, 0.42698E+00_JPRB, 0.41875E+00_JPRB, 0.41134E+00_JPRB, 0.40828E+00_JPRB, 0.40751E+00_JPRB ,&
&  0.40253E+00_JPRB, 0.39896E+00_JPRB, 0.39862E+00_JPRB, 0.39577E+00_JPRB, 0.39016E+00_JPRB, 0.38281E+00_JPRB, 0.38468E+00_JPRB ,&
&  0.39427E+00_JPRB, 0.38550E+00_JPRB, 0.37575E+00_JPRB, 0.36702E+00_JPRB, 0.35324E+00_JPRB, 0.32977E+00_JPRB, 0.31383E+00_JPRB ,&
&  0.30865E+00_JPRB, 0.29388E+00_JPRB, 0.29475E+00_JPRB, 0.29695E+00_JPRB, 0.29811E+00_JPRB, 0.30602E+00_JPRB, 0.32343E+00_JPRB ,&
&  0.34527E+00_JPRB, 0.40693E+00_JPRB, 0.43103E+00_JPRB, 0.46330E+00_JPRB, 0.49444E+00_JPRB, 0.52086E+00_JPRB, 0.54510E+00_JPRB ,&
&  0.56009E+00_JPRB, 0.58177E+00_JPRB, 0.60708E+00_JPRB, 0.63821E+00_JPRB, 0.66703E+00_JPRB, 0.68722E+00_JPRB, 0.71280E+00_JPRB ,&
&  0.74503E+00_JPRB, 0.73740E+00_JPRB, 0.74255E+00_JPRB, 0.73154E+00_JPRB, 0.73338E+00_JPRB, 0.75614E+00_JPRB, 0.74517E+00_JPRB ,&
&  0.76384E+00_JPRB, 0.80995E+00_JPRB, 0.88469E+00_JPRB, 0.96369E+00_JPRB, 0.99940E+00_JPRB, 0.10404E+01_JPRB, 0.10586E+01_JPRB ,&
&  0.10098E+01_JPRB, 0.98014E+00_JPRB, 0.10013E+01_JPRB, 0.92726E+00_JPRB, 0.84659E+00_JPRB, 0.78012E+00_JPRB, 0.73577E+00_JPRB ,&
&  0.69066E+00_JPRB, 0.64559E+00_JPRB, 0.61688E+00_JPRB, 0.59436E+00_JPRB, 0.57123E+00_JPRB, 0.55236E+00_JPRB, 0.55067E+00_JPRB ,&
&  0.54940E+00_JPRB, 0.54694E+00_JPRB, 0.54152E+00_JPRB, 0.53382E+00_JPRB, 0.51805E+00_JPRB, 0.50033E+00_JPRB, 0.48459E+00_JPRB ,&
&  0.48059E+00_JPRB, 0.48316E+00_JPRB, 0.49419E+00_JPRB, 0.50509E+00_JPRB, 0.51667E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.52307E+00_JPRB, 0.53534E+00_JPRB, 0.54683E+00_JPRB, 0.55889E+00_JPRB, 0.58112E+00_JPRB, 0.54801E+00_JPRB, 0.52514E+00_JPRB ,&
&  0.51297E+00_JPRB, 0.51396E+00_JPRB, 0.51436E+00_JPRB, 0.50874E+00_JPRB, 0.49879E+00_JPRB, 0.48947E+00_JPRB, 0.48229E+00_JPRB ,&
&  0.47647E+00_JPRB, 0.46282E+00_JPRB, 0.44084E+00_JPRB, 0.40786E+00_JPRB, 0.39947E+00_JPRB, 0.39847E+00_JPRB, 0.40189E+00_JPRB ,&
&  0.40362E+00_JPRB, 0.39962E+00_JPRB, 0.39879E+00_JPRB, 0.39295E+00_JPRB, 0.38810E+00_JPRB, 0.37613E+00_JPRB, 0.36552E+00_JPRB ,&
&  0.36752E+00_JPRB, 0.36883E+00_JPRB, 0.38417E+00_JPRB, 0.37449E+00_JPRB, 0.37085E+00_JPRB, 0.37208E+00_JPRB, 0.31408E+00_JPRB ,&
&  0.30309E+00_JPRB, 0.29136E+00_JPRB, 0.28802E+00_JPRB, 0.28771E+00_JPRB, 0.29882E+00_JPRB, 0.31034E+00_JPRB, 0.32899E+00_JPRB ,&
&  0.35781E+00_JPRB, 0.41919E+00_JPRB, 0.44092E+00_JPRB, 0.46874E+00_JPRB, 0.49739E+00_JPRB, 0.52817E+00_JPRB, 0.56742E+00_JPRB ,&
&  0.60228E+00_JPRB, 0.63509E+00_JPRB, 0.65982E+00_JPRB, 0.68331E+00_JPRB, 0.70755E+00_JPRB, 0.67501E+00_JPRB, 0.70541E+00_JPRB ,&
&  0.89200E+00_JPRB, 0.86274E+00_JPRB, 0.82034E+00_JPRB, 0.79409E+00_JPRB, 0.78964E+00_JPRB, 0.78280E+00_JPRB, 0.77404E+00_JPRB ,&
&  0.81413E+00_JPRB, 0.89958E+00_JPRB, 0.10417E+01_JPRB, 0.11470E+01_JPRB, 0.11511E+01_JPRB, 0.11667E+01_JPRB, 0.11540E+01_JPRB ,&
&  0.10776E+01_JPRB, 0.10262E+01_JPRB, 0.10754E+01_JPRB, 0.89339E+00_JPRB, 0.77036E+00_JPRB, 0.70852E+00_JPRB, 0.68447E+00_JPRB ,&
&  0.65661E+00_JPRB, 0.62607E+00_JPRB, 0.60935E+00_JPRB, 0.58763E+00_JPRB, 0.55758E+00_JPRB, 0.54052E+00_JPRB, 0.53848E+00_JPRB ,&
&  0.53730E+00_JPRB, 0.53361E+00_JPRB, 0.53117E+00_JPRB, 0.52580E+00_JPRB, 0.51403E+00_JPRB, 0.49995E+00_JPRB, 0.49655E+00_JPRB ,&
&  0.49887E+00_JPRB, 0.50085E+00_JPRB, 0.50312E+00_JPRB, 0.50546E+00_JPRB, 0.51172E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.50847E+00_JPRB, 0.52285E+00_JPRB, 0.54144E+00_JPRB, 0.54108E+00_JPRB, 0.54255E+00_JPRB, 0.49310E+00_JPRB, 0.44039E+00_JPRB ,&
&  0.40470E+00_JPRB, 0.40807E+00_JPRB, 0.42363E+00_JPRB, 0.43647E+00_JPRB, 0.43663E+00_JPRB, 0.42884E+00_JPRB, 0.42839E+00_JPRB ,&
&  0.43445E+00_JPRB, 0.43270E+00_JPRB, 0.41547E+00_JPRB, 0.40659E+00_JPRB, 0.39105E+00_JPRB, 0.39008E+00_JPRB, 0.39321E+00_JPRB ,&
&  0.39660E+00_JPRB, 0.39535E+00_JPRB, 0.39282E+00_JPRB, 0.38936E+00_JPRB, 0.38798E+00_JPRB, 0.38786E+00_JPRB, 0.38483E+00_JPRB ,&
&  0.37630E+00_JPRB, 0.37175E+00_JPRB, 0.37834E+00_JPRB, 0.39647E+00_JPRB, 0.39468E+00_JPRB, 0.39588E+00_JPRB, 0.33202E+00_JPRB ,&
&  0.30634E+00_JPRB, 0.28999E+00_JPRB, 0.28978E+00_JPRB, 0.30096E+00_JPRB, 0.32785E+00_JPRB, 0.35268E+00_JPRB, 0.40329E+00_JPRB ,&
&  0.45024E+00_JPRB, 0.47483E+00_JPRB, 0.48798E+00_JPRB, 0.49861E+00_JPRB, 0.52583E+00_JPRB, 0.58153E+00_JPRB, 0.64896E+00_JPRB ,&
&  0.69116E+00_JPRB, 0.72022E+00_JPRB, 0.72474E+00_JPRB, 0.59631E+00_JPRB, 0.58092E+00_JPRB, 0.62109E+00_JPRB, 0.67404E+00_JPRB ,&
&  0.75213E+00_JPRB, 0.77041E+00_JPRB, 0.78120E+00_JPRB, 0.81556E+00_JPRB, 0.79900E+00_JPRB, 0.78141E+00_JPRB, 0.75684E+00_JPRB ,&
&  0.82089E+00_JPRB, 0.93691E+00_JPRB, 0.11304E+01_JPRB, 0.12340E+01_JPRB, 0.12063E+01_JPRB, 0.12207E+01_JPRB, 0.11799E+01_JPRB ,&
&  0.11374E+01_JPRB, 0.10226E+01_JPRB, 0.91903E+00_JPRB, 0.79667E+00_JPRB, 0.70305E+00_JPRB, 0.65100E+00_JPRB, 0.62241E+00_JPRB ,&
&  0.59651E+00_JPRB, 0.57308E+00_JPRB, 0.57204E+00_JPRB, 0.55939E+00_JPRB, 0.52966E+00_JPRB, 0.50223E+00_JPRB, 0.48083E+00_JPRB ,&
&  0.46641E+00_JPRB, 0.45898E+00_JPRB, 0.45777E+00_JPRB, 0.45058E+00_JPRB, 0.45301E+00_JPRB, 0.46212E+00_JPRB, 0.46795E+00_JPRB ,&
&  0.46888E+00_JPRB, 0.46858E+00_JPRB, 0.47416E+00_JPRB, 0.48069E+00_JPRB, 0.48974E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.56873E+00_JPRB, 0.55080E+00_JPRB, 0.52385E+00_JPRB, 0.52339E+00_JPRB, 0.50215E+00_JPRB, 0.48913E+00_JPRB, 0.44678E+00_JPRB ,&
&  0.39290E+00_JPRB, 0.37458E+00_JPRB, 0.39088E+00_JPRB, 0.39315E+00_JPRB, 0.37835E+00_JPRB, 0.37298E+00_JPRB, 0.37750E+00_JPRB ,&
&  0.38541E+00_JPRB, 0.40527E+00_JPRB, 0.40454E+00_JPRB, 0.39217E+00_JPRB, 0.38558E+00_JPRB, 0.38422E+00_JPRB, 0.38917E+00_JPRB ,&
&  0.39240E+00_JPRB, 0.39382E+00_JPRB, 0.38912E+00_JPRB, 0.38094E+00_JPRB, 0.38087E+00_JPRB, 0.38360E+00_JPRB, 0.38554E+00_JPRB ,&
&  0.38914E+00_JPRB, 0.38999E+00_JPRB, 0.40260E+00_JPRB, 0.41544E+00_JPRB, 0.41000E+00_JPRB, 0.40029E+00_JPRB, 0.34715E+00_JPRB ,&
&  0.31147E+00_JPRB, 0.30193E+00_JPRB, 0.32756E+00_JPRB, 0.40252E+00_JPRB, 0.43563E+00_JPRB, 0.47435E+00_JPRB, 0.50628E+00_JPRB ,&
&  0.47796E+00_JPRB, 0.48090E+00_JPRB, 0.51425E+00_JPRB, 0.63022E+00_JPRB, 0.67270E+00_JPRB, 0.73181E+00_JPRB, 0.77438E+00_JPRB ,&
&  0.80731E+00_JPRB, 0.83319E+00_JPRB, 0.84061E+00_JPRB, 0.69337E+00_JPRB, 0.69175E+00_JPRB, 0.84044E+00_JPRB, 0.76078E+00_JPRB ,&
&  0.79460E+00_JPRB, 0.83810E+00_JPRB, 0.88015E+00_JPRB, 0.84704E+00_JPRB, 0.78227E+00_JPRB, 0.77194E+00_JPRB, 0.80739E+00_JPRB ,&
&  0.88496E+00_JPRB, 0.10169E+01_JPRB, 0.12809E+01_JPRB, 0.14265E+01_JPRB, 0.14027E+01_JPRB, 0.13880E+01_JPRB, 0.13614E+01_JPRB ,&
&  0.13374E+01_JPRB, 0.12137E+01_JPRB, 0.10527E+01_JPRB, 0.90212E+00_JPRB, 0.77335E+00_JPRB, 0.67423E+00_JPRB, 0.61718E+00_JPRB ,&
&  0.59570E+00_JPRB, 0.58096E+00_JPRB, 0.57971E+00_JPRB, 0.56457E+00_JPRB, 0.52876E+00_JPRB, 0.48952E+00_JPRB, 0.45824E+00_JPRB ,&
&  0.43860E+00_JPRB, 0.41582E+00_JPRB, 0.39379E+00_JPRB, 0.38804E+00_JPRB, 0.41261E+00_JPRB, 0.44855E+00_JPRB, 0.46213E+00_JPRB ,&
&  0.45154E+00_JPRB, 0.45491E+00_JPRB, 0.48311E+00_JPRB, 0.51417E+00_JPRB, 0.53053E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.57067E+00_JPRB, 0.54553E+00_JPRB, 0.52407E+00_JPRB, 0.50184E+00_JPRB, 0.49241E+00_JPRB, 0.47942E+00_JPRB, 0.43455E+00_JPRB ,&
&  0.38749E+00_JPRB, 0.35638E+00_JPRB, 0.36394E+00_JPRB, 0.35427E+00_JPRB, 0.34307E+00_JPRB, 0.33611E+00_JPRB, 0.33536E+00_JPRB ,&
&  0.34158E+00_JPRB, 0.36169E+00_JPRB, 0.37703E+00_JPRB, 0.37985E+00_JPRB, 0.37684E+00_JPRB, 0.37457E+00_JPRB, 0.38126E+00_JPRB ,&
&  0.39088E+00_JPRB, 0.40209E+00_JPRB, 0.40803E+00_JPRB, 0.40788E+00_JPRB, 0.40466E+00_JPRB, 0.40568E+00_JPRB, 0.41141E+00_JPRB ,&
&  0.42254E+00_JPRB, 0.43026E+00_JPRB, 0.42173E+00_JPRB, 0.42758E+00_JPRB, 0.40873E+00_JPRB, 0.41080E+00_JPRB, 0.40043E+00_JPRB ,&
&  0.33837E+00_JPRB, 0.35058E+00_JPRB, 0.47162E+00_JPRB, 0.50303E+00_JPRB, 0.51894E+00_JPRB, 0.55384E+00_JPRB, 0.59977E+00_JPRB ,&
&  0.64265E+00_JPRB, 0.68312E+00_JPRB, 0.71931E+00_JPRB, 0.75898E+00_JPRB, 0.79638E+00_JPRB, 0.84648E+00_JPRB, 0.88938E+00_JPRB ,&
&  0.96322E+00_JPRB, 0.78901E+00_JPRB, 0.73485E+00_JPRB, 0.77667E+00_JPRB, 0.10340E+01_JPRB, 0.10582E+01_JPRB, 0.10182E+01_JPRB ,&
&  0.10049E+01_JPRB, 0.96329E+00_JPRB, 0.92917E+00_JPRB, 0.91308E+00_JPRB, 0.93389E+00_JPRB, 0.98642E+00_JPRB, 0.10821E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.13439E+01_JPRB, 0.16177E+01_JPRB, 0.16831E+01_JPRB, 0.16606E+01_JPRB, 0.16487E+01_JPRB, 0.15984E+01_JPRB ,&
&  0.15318E+01_JPRB, 0.13983E+01_JPRB, 0.11783E+01_JPRB, 0.99446E+00_JPRB, 0.84215E+00_JPRB, 0.72698E+00_JPRB, 0.63711E+00_JPRB ,&
&  0.59253E+00_JPRB, 0.57742E+00_JPRB, 0.56792E+00_JPRB, 0.55345E+00_JPRB, 0.53084E+00_JPRB, 0.50538E+00_JPRB, 0.47671E+00_JPRB ,&
&  0.44262E+00_JPRB, 0.41287E+00_JPRB, 0.38814E+00_JPRB, 0.39838E+00_JPRB, 0.43421E+00_JPRB, 0.47187E+00_JPRB, 0.48929E+00_JPRB ,&
&  0.49858E+00_JPRB, 0.50631E+00_JPRB, 0.50835E+00_JPRB, 0.51974E+00_JPRB, 0.54229E+00_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.57037E+00_JPRB, 0.53527E+00_JPRB, 0.50480E+00_JPRB, 0.48145E+00_JPRB, 0.47384E+00_JPRB, 0.46432E+00_JPRB, 0.45612E+00_JPRB ,&
&  0.50276E+00_JPRB, 0.42170E+00_JPRB, 0.50372E+00_JPRB, 0.40251E+00_JPRB, 0.37021E+00_JPRB, 0.36462E+00_JPRB, 0.34941E+00_JPRB ,&
&  0.34030E+00_JPRB, 0.34194E+00_JPRB, 0.34642E+00_JPRB, 0.35132E+00_JPRB, 0.34999E+00_JPRB, 0.34802E+00_JPRB, 0.35452E+00_JPRB ,&
&  0.36862E+00_JPRB, 0.38912E+00_JPRB, 0.41394E+00_JPRB, 0.42572E+00_JPRB, 0.43106E+00_JPRB, 0.43198E+00_JPRB, 0.44297E+00_JPRB ,&
&  0.41862E+00_JPRB, 0.41436E+00_JPRB, 0.42433E+00_JPRB, 0.42646E+00_JPRB, 0.42277E+00_JPRB, 0.41343E+00_JPRB, 0.43200E+00_JPRB ,&
&  0.43281E+00_JPRB, 0.40877E+00_JPRB, 0.51582E+00_JPRB, 0.52902E+00_JPRB, 0.55721E+00_JPRB, 0.58848E+00_JPRB, 0.62293E+00_JPRB ,&
&  0.65922E+00_JPRB, 0.70353E+00_JPRB, 0.75417E+00_JPRB, 0.79173E+00_JPRB, 0.85163E+00_JPRB, 0.92054E+00_JPRB, 0.97681E+00_JPRB ,&
&  0.10559E+01_JPRB, 0.80664E+00_JPRB, 0.88999E+00_JPRB, 0.93410E+00_JPRB, 0.11412E+01_JPRB, 0.13777E+01_JPRB, 0.13636E+01_JPRB ,&
&  0.14084E+01_JPRB, 0.13305E+01_JPRB, 0.12517E+01_JPRB, 0.11944E+01_JPRB, 0.12388E+01_JPRB, 0.13185E+01_JPRB, 0.14200E+01_JPRB ,&
&  0.15562E+01_JPRB, 0.17374E+01_JPRB, 0.20240E+01_JPRB, 0.20338E+01_JPRB, 0.19988E+01_JPRB, 0.19412E+01_JPRB, 0.18374E+01_JPRB ,&
&  0.16929E+01_JPRB, 0.15178E+01_JPRB, 0.12961E+01_JPRB, 0.10617E+01_JPRB, 0.90233E+00_JPRB, 0.76733E+00_JPRB, 0.65903E+00_JPRB ,&
&  0.58023E+00_JPRB, 0.53377E+00_JPRB, 0.52633E+00_JPRB, 0.51189E+00_JPRB, 0.50233E+00_JPRB, 0.48668E+00_JPRB, 0.45694E+00_JPRB ,&
&  0.43498E+00_JPRB, 0.43925E+00_JPRB, 0.48168E+00_JPRB, 0.53025E+00_JPRB, 0.54868E+00_JPRB, 0.56128E+00_JPRB, 0.61579E+00_JPRB ,&
&  0.55855E+00_JPRB, 0.59910E+00_JPRB, 0.64381E+00_JPRB, 0.62124E+00_JPRB, 0.60217E+00_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.57641E+00_JPRB, 0.53119E+00_JPRB, 0.48342E+00_JPRB, 0.46511E+00_JPRB, 0.47465E+00_JPRB, 0.48775E+00_JPRB, 0.47585E+00_JPRB ,&
&  0.51330E+00_JPRB, 0.49998E+00_JPRB, 0.52741E+00_JPRB, 0.56622E+00_JPRB, 0.60561E+00_JPRB, 0.52441E+00_JPRB, 0.39673E+00_JPRB ,&
&  0.35688E+00_JPRB, 0.35750E+00_JPRB, 0.36381E+00_JPRB, 0.36111E+00_JPRB, 0.36033E+00_JPRB, 0.35633E+00_JPRB, 0.35635E+00_JPRB ,&
&  0.36147E+00_JPRB, 0.37253E+00_JPRB, 0.40386E+00_JPRB, 0.41622E+00_JPRB, 0.42260E+00_JPRB, 0.42481E+00_JPRB, 0.43871E+00_JPRB ,&
&  0.41600E+00_JPRB, 0.40996E+00_JPRB, 0.41141E+00_JPRB, 0.38771E+00_JPRB, 0.42369E+00_JPRB, 0.43728E+00_JPRB, 0.44849E+00_JPRB ,&
&  0.46277E+00_JPRB, 0.49095E+00_JPRB, 0.52391E+00_JPRB, 0.55286E+00_JPRB, 0.57904E+00_JPRB, 0.60620E+00_JPRB, 0.63588E+00_JPRB ,&
&  0.66730E+00_JPRB, 0.70759E+00_JPRB, 0.75546E+00_JPRB, 0.81306E+00_JPRB, 0.94167E+00_JPRB, 0.92407E+00_JPRB, 0.10190E+01_JPRB ,&
&  0.10662E+01_JPRB, 0.11844E+01_JPRB, 0.13065E+01_JPRB, 0.12129E+01_JPRB, 0.13060E+01_JPRB, 0.14630E+01_JPRB, 0.14233E+01_JPRB ,&
&  0.14038E+01_JPRB, 0.13372E+01_JPRB, 0.13584E+01_JPRB, 0.14243E+01_JPRB, 0.15390E+01_JPRB, 0.16376E+01_JPRB, 0.17399E+01_JPRB ,&
&  0.19201E+01_JPRB, 0.22542E+01_JPRB, 0.23908E+01_JPRB, 0.23245E+01_JPRB, 0.22354E+01_JPRB, 0.21264E+01_JPRB, 0.19639E+01_JPRB ,&
&  0.17790E+01_JPRB, 0.15690E+01_JPRB, 0.13357E+01_JPRB, 0.11015E+01_JPRB, 0.90409E+00_JPRB, 0.79591E+00_JPRB, 0.68071E+00_JPRB ,&
&  0.56405E+00_JPRB, 0.48693E+00_JPRB, 0.48408E+00_JPRB, 0.48356E+00_JPRB, 0.47672E+00_JPRB, 0.45468E+00_JPRB, 0.43505E+00_JPRB ,&
&  0.46671E+00_JPRB, 0.53681E+00_JPRB, 0.72370E+00_JPRB, 0.71579E+00_JPRB, 0.71776E+00_JPRB, 0.70527E+00_JPRB, 0.58157E+00_JPRB ,&
&  0.54076E+00_JPRB, 0.67502E+00_JPRB, 0.63559E+00_JPRB, 0.62986E+00_JPRB, 0.61285E+00_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.59533E+00_JPRB, 0.53463E+00_JPRB, 0.47209E+00_JPRB, 0.45824E+00_JPRB, 0.47633E+00_JPRB, 0.48725E+00_JPRB, 0.49535E+00_JPRB ,&
&  0.51241E+00_JPRB, 0.52061E+00_JPRB, 0.53335E+00_JPRB, 0.54877E+00_JPRB, 0.55385E+00_JPRB, 0.58379E+00_JPRB, 0.52077E+00_JPRB ,&
&  0.42914E+00_JPRB, 0.40414E+00_JPRB, 0.38365E+00_JPRB, 0.38138E+00_JPRB, 0.39989E+00_JPRB, 0.39984E+00_JPRB, 0.39818E+00_JPRB ,&
&  0.40441E+00_JPRB, 0.40296E+00_JPRB, 0.41291E+00_JPRB, 0.41322E+00_JPRB, 0.41231E+00_JPRB, 0.41832E+00_JPRB, 0.42886E+00_JPRB ,&
&  0.40149E+00_JPRB, 0.38058E+00_JPRB, 0.36943E+00_JPRB, 0.36380E+00_JPRB, 0.39347E+00_JPRB, 0.43588E+00_JPRB, 0.46227E+00_JPRB ,&
&  0.48073E+00_JPRB, 0.50136E+00_JPRB, 0.52496E+00_JPRB, 0.55383E+00_JPRB, 0.58495E+00_JPRB, 0.61637E+00_JPRB, 0.64153E+00_JPRB ,&
&  0.66919E+00_JPRB, 0.70210E+00_JPRB, 0.74868E+00_JPRB, 0.82886E+00_JPRB, 0.87627E+00_JPRB, 0.94324E+00_JPRB, 0.10842E+01_JPRB ,&
&  0.11792E+01_JPRB, 0.12953E+01_JPRB, 0.14018E+01_JPRB, 0.15430E+01_JPRB, 0.16458E+01_JPRB, 0.15887E+01_JPRB, 0.15189E+01_JPRB ,&
&  0.15077E+01_JPRB, 0.14866E+01_JPRB, 0.16544E+01_JPRB, 0.17730E+01_JPRB, 0.18459E+01_JPRB, 0.19588E+01_JPRB, 0.20589E+01_JPRB ,&
&  0.22448E+01_JPRB, 0.24412E+01_JPRB, 0.24857E+01_JPRB, 0.24058E+01_JPRB, 0.22984E+01_JPRB, 0.21899E+01_JPRB, 0.20078E+01_JPRB ,&
&  0.17985E+01_JPRB, 0.15528E+01_JPRB, 0.12758E+01_JPRB, 0.10433E+01_JPRB, 0.84352E+00_JPRB, 0.77774E+00_JPRB, 0.69818E+00_JPRB ,&
&  0.55881E+00_JPRB, 0.49237E+00_JPRB, 0.48538E+00_JPRB, 0.50695E+00_JPRB, 0.50271E+00_JPRB, 0.47482E+00_JPRB, 0.46595E+00_JPRB ,&
&  0.51606E+00_JPRB, 0.66786E+00_JPRB, 0.71098E+00_JPRB, 0.74160E+00_JPRB, 0.75446E+00_JPRB, 0.75083E+00_JPRB, 0.53622E+00_JPRB ,&
&  0.56610E+00_JPRB, 0.67825E+00_JPRB, 0.65981E+00_JPRB, 0.64643E+00_JPRB, 0.63018E+00_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.61733E+00_JPRB, 0.55110E+00_JPRB, 0.49369E+00_JPRB, 0.47201E+00_JPRB, 0.48858E+00_JPRB, 0.50578E+00_JPRB, 0.51718E+00_JPRB ,&
&  0.53286E+00_JPRB, 0.54430E+00_JPRB, 0.55328E+00_JPRB, 0.56161E+00_JPRB, 0.56622E+00_JPRB, 0.55840E+00_JPRB, 0.60580E+00_JPRB ,&
&  0.46693E+00_JPRB, 0.42158E+00_JPRB, 0.39063E+00_JPRB, 0.39429E+00_JPRB, 0.46447E+00_JPRB, 0.47483E+00_JPRB, 0.49099E+00_JPRB ,&
&  0.48548E+00_JPRB, 0.48724E+00_JPRB, 0.47921E+00_JPRB, 0.46569E+00_JPRB, 0.45241E+00_JPRB, 0.44888E+00_JPRB, 0.43796E+00_JPRB ,&
&  0.40252E+00_JPRB, 0.37753E+00_JPRB, 0.37356E+00_JPRB, 0.37885E+00_JPRB, 0.41117E+00_JPRB, 0.43921E+00_JPRB, 0.48923E+00_JPRB ,&
&  0.51848E+00_JPRB, 0.54306E+00_JPRB, 0.56825E+00_JPRB, 0.59550E+00_JPRB, 0.61923E+00_JPRB, 0.65008E+00_JPRB, 0.68601E+00_JPRB ,&
&  0.72330E+00_JPRB, 0.76200E+00_JPRB, 0.83276E+00_JPRB, 0.91810E+00_JPRB, 0.92193E+00_JPRB, 0.99543E+00_JPRB, 0.11731E+01_JPRB ,&
&  0.13350E+01_JPRB, 0.13981E+01_JPRB, 0.16081E+01_JPRB, 0.20890E+01_JPRB, 0.21884E+01_JPRB, 0.21104E+01_JPRB, 0.19941E+01_JPRB ,&
&  0.19256E+01_JPRB, 0.18703E+01_JPRB, 0.19151E+01_JPRB, 0.20048E+01_JPRB, 0.20816E+01_JPRB, 0.22352E+01_JPRB, 0.23756E+01_JPRB ,&
&  0.24456E+01_JPRB, 0.24640E+01_JPRB, 0.24227E+01_JPRB, 0.23270E+01_JPRB, 0.21762E+01_JPRB, 0.20553E+01_JPRB, 0.19231E+01_JPRB ,&
&  0.17437E+01_JPRB, 0.15201E+01_JPRB, 0.12006E+01_JPRB, 0.99149E+00_JPRB, 0.76143E+00_JPRB, 0.69266E+00_JPRB, 0.77606E+00_JPRB ,&
&  0.59531E+00_JPRB, 0.52147E+00_JPRB, 0.52680E+00_JPRB, 0.56390E+00_JPRB, 0.57201E+00_JPRB, 0.56596E+00_JPRB, 0.51549E+00_JPRB ,&
&  0.47130E+00_JPRB, 0.56993E+00_JPRB, 0.77978E+00_JPRB, 0.82350E+00_JPRB, 0.82255E+00_JPRB, 0.79442E+00_JPRB, 0.63352E+00_JPRB ,&
&  0.72297E+00_JPRB, 0.68204E+00_JPRB, 0.67614E+00_JPRB, 0.66595E+00_JPRB, 0.65525E+00_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.63226E+00_JPRB, 0.56928E+00_JPRB, 0.52337E+00_JPRB, 0.50244E+00_JPRB, 0.51402E+00_JPRB, 0.53432E+00_JPRB, 0.54649E+00_JPRB ,&
&  0.55522E+00_JPRB, 0.56059E+00_JPRB, 0.56523E+00_JPRB, 0.57018E+00_JPRB, 0.57427E+00_JPRB, 0.55751E+00_JPRB, 0.54322E+00_JPRB ,&
&  0.53761E+00_JPRB, 0.44678E+00_JPRB, 0.41825E+00_JPRB, 0.40911E+00_JPRB, 0.46857E+00_JPRB, 0.51351E+00_JPRB, 0.56328E+00_JPRB ,&
&  0.60108E+00_JPRB, 0.63494E+00_JPRB, 0.62510E+00_JPRB, 0.59429E+00_JPRB, 0.56255E+00_JPRB, 0.53711E+00_JPRB, 0.49303E+00_JPRB ,&
&  0.44310E+00_JPRB, 0.42159E+00_JPRB, 0.42547E+00_JPRB, 0.45912E+00_JPRB, 0.51895E+00_JPRB, 0.48480E+00_JPRB, 0.54784E+00_JPRB ,&
&  0.58413E+00_JPRB, 0.61911E+00_JPRB, 0.63426E+00_JPRB, 0.65684E+00_JPRB, 0.71627E+00_JPRB, 0.79121E+00_JPRB, 0.85849E+00_JPRB ,&
&  0.91956E+00_JPRB, 0.93770E+00_JPRB, 0.92785E+00_JPRB, 0.99940E+00_JPRB, 0.10728E+01_JPRB, 0.11189E+01_JPRB, 0.12154E+01_JPRB ,&
&  0.14064E+01_JPRB, 0.15257E+01_JPRB, 0.18029E+01_JPRB, 0.23787E+01_JPRB, 0.25040E+01_JPRB, 0.24303E+01_JPRB, 0.24255E+01_JPRB ,&
&  0.23990E+01_JPRB, 0.23286E+01_JPRB, 0.22774E+01_JPRB, 0.22274E+01_JPRB, 0.23353E+01_JPRB, 0.25289E+01_JPRB, 0.26721E+01_JPRB ,&
&  0.25637E+01_JPRB, 0.23740E+01_JPRB, 0.22867E+01_JPRB, 0.21214E+01_JPRB, 0.19686E+01_JPRB, 0.17954E+01_JPRB, 0.17981E+01_JPRB ,&
&  0.17881E+01_JPRB, 0.13058E+01_JPRB, 0.10716E+01_JPRB, 0.99877E+00_JPRB, 0.75432E+00_JPRB, 0.64640E+00_JPRB, 0.66168E+00_JPRB ,&
&  0.54654E+00_JPRB, 0.50213E+00_JPRB, 0.54724E+00_JPRB, 0.59704E+00_JPRB, 0.65689E+00_JPRB, 0.69403E+00_JPRB, 0.66576E+00_JPRB ,&
&  0.62014E+00_JPRB, 0.66569E+00_JPRB, 0.92028E+00_JPRB, 0.96064E+00_JPRB, 0.93189E+00_JPRB, 0.85525E+00_JPRB, 0.78624E+00_JPRB ,&
&  0.72279E+00_JPRB, 0.72365E+00_JPRB, 0.71711E+00_JPRB, 0.70662E+00_JPRB, 0.69138E+00_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.69037E+00_JPRB, 0.62426E+00_JPRB, 0.57300E+00_JPRB, 0.55720E+00_JPRB, 0.57329E+00_JPRB, 0.58501E+00_JPRB, 0.59129E+00_JPRB ,&
&  0.57731E+00_JPRB, 0.56971E+00_JPRB, 0.57866E+00_JPRB, 0.57886E+00_JPRB, 0.57846E+00_JPRB, 0.56224E+00_JPRB, 0.52719E+00_JPRB ,&
&  0.50937E+00_JPRB, 0.49005E+00_JPRB, 0.44475E+00_JPRB, 0.44732E+00_JPRB, 0.47218E+00_JPRB, 0.55907E+00_JPRB, 0.63344E+00_JPRB ,&
&  0.70803E+00_JPRB, 0.76636E+00_JPRB, 0.79145E+00_JPRB, 0.80166E+00_JPRB, 0.77601E+00_JPRB, 0.71815E+00_JPRB, 0.66029E+00_JPRB ,&
&  0.60907E+00_JPRB, 0.58835E+00_JPRB, 0.56852E+00_JPRB, 0.64458E+00_JPRB, 0.67449E+00_JPRB, 0.63593E+00_JPRB, 0.72663E+00_JPRB ,&
&  0.76094E+00_JPRB, 0.79308E+00_JPRB, 0.80904E+00_JPRB, 0.87270E+00_JPRB, 0.94252E+00_JPRB, 0.96735E+00_JPRB, 0.95859E+00_JPRB ,&
&  0.94416E+00_JPRB, 0.95726E+00_JPRB, 0.99968E+00_JPRB, 0.98932E+00_JPRB, 0.10421E+01_JPRB, 0.11614E+01_JPRB, 0.13854E+01_JPRB ,&
&  0.16756E+01_JPRB, 0.16719E+01_JPRB, 0.16952E+01_JPRB, 0.23314E+01_JPRB, 0.28766E+01_JPRB, 0.28359E+01_JPRB, 0.27330E+01_JPRB ,&
&  0.27370E+01_JPRB, 0.25738E+01_JPRB, 0.24124E+01_JPRB, 0.23629E+01_JPRB, 0.26619E+01_JPRB, 0.28775E+01_JPRB, 0.29223E+01_JPRB ,&
&  0.27169E+01_JPRB, 0.24401E+01_JPRB, 0.22951E+01_JPRB, 0.21171E+01_JPRB, 0.18914E+01_JPRB, 0.18003E+01_JPRB, 0.18263E+01_JPRB ,&
&  0.17792E+01_JPRB, 0.15752E+01_JPRB, 0.12129E+01_JPRB, 0.10860E+01_JPRB, 0.86762E+00_JPRB, 0.75236E+00_JPRB, 0.72655E+00_JPRB ,&
&  0.62033E+00_JPRB, 0.58627E+00_JPRB, 0.59527E+00_JPRB, 0.62977E+00_JPRB, 0.86454E+00_JPRB, 0.93288E+00_JPRB, 0.99941E+00_JPRB ,&
&  0.99132E+00_JPRB, 0.96044E+00_JPRB, 0.12094E+01_JPRB, 0.12074E+01_JPRB, 0.11091E+01_JPRB, 0.96108E+00_JPRB, 0.83353E+00_JPRB ,&
&  0.78166E+00_JPRB, 0.79374E+00_JPRB, 0.78183E+00_JPRB, 0.76796E+00_JPRB, 0.74797E+00_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.69829E+00_JPRB, 0.63083E+00_JPRB, 0.58012E+00_JPRB, 0.56717E+00_JPRB, 0.57805E+00_JPRB, 0.58817E+00_JPRB, 0.59036E+00_JPRB ,&
&  0.59011E+00_JPRB, 0.59422E+00_JPRB, 0.59534E+00_JPRB, 0.57986E+00_JPRB, 0.56363E+00_JPRB, 0.54473E+00_JPRB, 0.51420E+00_JPRB ,&
&  0.48966E+00_JPRB, 0.48393E+00_JPRB, 0.45138E+00_JPRB, 0.45992E+00_JPRB, 0.50806E+00_JPRB, 0.63117E+00_JPRB, 0.76656E+00_JPRB ,&
&  0.88054E+00_JPRB, 0.95296E+00_JPRB, 0.10049E+01_JPRB, 0.10391E+01_JPRB, 0.10672E+01_JPRB, 0.10435E+01_JPRB, 0.98263E+00_JPRB ,&
&  0.10004E+01_JPRB, 0.85936E+00_JPRB, 0.93798E+00_JPRB, 0.93216E+00_JPRB, 0.98946E+00_JPRB, 0.98724E+00_JPRB, 0.95552E+00_JPRB ,&
&  0.92417E+00_JPRB, 0.92834E+00_JPRB, 0.92819E+00_JPRB, 0.95507E+00_JPRB, 0.99049E+00_JPRB, 0.10090E+01_JPRB, 0.10326E+01_JPRB ,&
&  0.10581E+01_JPRB, 0.10191E+01_JPRB, 0.94915E+00_JPRB, 0.94507E+00_JPRB, 0.12659E+01_JPRB, 0.16872E+01_JPRB, 0.20708E+01_JPRB ,&
&  0.20178E+01_JPRB, 0.18525E+01_JPRB, 0.26578E+01_JPRB, 0.29661E+01_JPRB, 0.28856E+01_JPRB, 0.30843E+01_JPRB, 0.30146E+01_JPRB ,&
&  0.27726E+01_JPRB, 0.24438E+01_JPRB, 0.21847E+01_JPRB, 0.22572E+01_JPRB, 0.31103E+01_JPRB, 0.33516E+01_JPRB, 0.33364E+01_JPRB ,&
&  0.29963E+01_JPRB, 0.26451E+01_JPRB, 0.23223E+01_JPRB, 0.20956E+01_JPRB, 0.19659E+01_JPRB, 0.20286E+01_JPRB, 0.18977E+01_JPRB ,&
&  0.17534E+01_JPRB, 0.18294E+01_JPRB, 0.15362E+01_JPRB, 0.12194E+01_JPRB, 0.10210E+01_JPRB, 0.92148E+00_JPRB, 0.85645E+00_JPRB ,&
&  0.76095E+00_JPRB, 0.70944E+00_JPRB, 0.70875E+00_JPRB, 0.84554E+00_JPRB, 0.11906E+01_JPRB, 0.12196E+01_JPRB, 0.12440E+01_JPRB ,&
&  0.12920E+01_JPRB, 0.16289E+01_JPRB, 0.15075E+01_JPRB, 0.13712E+01_JPRB, 0.12347E+01_JPRB, 0.10805E+01_JPRB, 0.96073E+00_JPRB ,&
&  0.87990E+00_JPRB, 0.87780E+00_JPRB, 0.86616E+00_JPRB, 0.82579E+00_JPRB, 0.77320E+00_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.74372E+00_JPRB, 0.66779E+00_JPRB, 0.63040E+00_JPRB, 0.61107E+00_JPRB, 0.60948E+00_JPRB, 0.60485E+00_JPRB, 0.58279E+00_JPRB ,&
&  0.57102E+00_JPRB, 0.58169E+00_JPRB, 0.57052E+00_JPRB, 0.56568E+00_JPRB, 0.56119E+00_JPRB, 0.53407E+00_JPRB, 0.50477E+00_JPRB ,&
&  0.49343E+00_JPRB, 0.48325E+00_JPRB, 0.48069E+00_JPRB, 0.53488E+00_JPRB, 0.60619E+00_JPRB, 0.70795E+00_JPRB, 0.86312E+00_JPRB ,&
&  0.99912E+00_JPRB, 0.11111E+01_JPRB, 0.11815E+01_JPRB, 0.13091E+01_JPRB, 0.14196E+01_JPRB, 0.14873E+01_JPRB, 0.14391E+01_JPRB ,&
&  0.12936E+01_JPRB, 0.13117E+01_JPRB, 0.13940E+01_JPRB, 0.13375E+01_JPRB, 0.12938E+01_JPRB, 0.12299E+01_JPRB, 0.11632E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.10404E+01_JPRB, 0.10308E+01_JPRB, 0.10466E+01_JPRB, 0.10768E+01_JPRB, 0.11394E+01_JPRB, 0.11323E+01_JPRB ,&
&  0.10546E+01_JPRB, 0.10075E+01_JPRB, 0.95560E+00_JPRB, 0.11737E+01_JPRB, 0.16982E+01_JPRB, 0.25163E+01_JPRB, 0.27585E+01_JPRB ,&
&  0.29193E+01_JPRB, 0.26609E+01_JPRB, 0.27442E+01_JPRB, 0.30921E+01_JPRB, 0.24819E+01_JPRB, 0.26452E+01_JPRB, 0.27256E+01_JPRB ,&
&  0.24808E+01_JPRB, 0.23397E+01_JPRB, 0.25916E+01_JPRB, 0.30500E+01_JPRB, 0.36803E+01_JPRB, 0.44922E+01_JPRB, 0.44013E+01_JPRB ,&
&  0.36454E+01_JPRB, 0.30045E+01_JPRB, 0.24045E+01_JPRB, 0.22048E+01_JPRB, 0.19433E+01_JPRB, 0.18814E+01_JPRB, 0.15228E+01_JPRB ,&
&  0.15299E+01_JPRB, 0.13193E+01_JPRB, 0.14008E+01_JPRB, 0.12335E+01_JPRB, 0.11242E+01_JPRB, 0.10084E+01_JPRB, 0.91825E+00_JPRB ,&
&  0.86979E+00_JPRB, 0.84611E+00_JPRB, 0.92008E+00_JPRB, 0.12308E+01_JPRB, 0.16197E+01_JPRB, 0.13991E+01_JPRB, 0.17018E+01_JPRB ,&
&  0.21809E+01_JPRB, 0.21200E+01_JPRB, 0.16283E+01_JPRB, 0.15117E+01_JPRB, 0.13056E+01_JPRB, 0.11561E+01_JPRB, 0.10871E+01_JPRB ,&
&  0.99945E+00_JPRB, 0.95227E+00_JPRB, 0.91909E+00_JPRB, 0.87500E+00_JPRB, 0.82051E+00_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.79494E+00_JPRB, 0.71316E+00_JPRB, 0.67751E+00_JPRB, 0.67170E+00_JPRB, 0.66017E+00_JPRB, 0.60963E+00_JPRB, 0.59236E+00_JPRB ,&
&  0.61105E+00_JPRB, 0.58880E+00_JPRB, 0.55203E+00_JPRB, 0.53350E+00_JPRB, 0.53564E+00_JPRB, 0.54107E+00_JPRB, 0.51637E+00_JPRB ,&
&  0.52304E+00_JPRB, 0.56280E+00_JPRB, 0.58908E+00_JPRB, 0.63114E+00_JPRB, 0.71393E+00_JPRB, 0.79943E+00_JPRB, 0.86670E+00_JPRB ,&
&  0.11617E+01_JPRB, 0.12903E+01_JPRB, 0.13271E+01_JPRB, 0.14936E+01_JPRB, 0.16899E+01_JPRB, 0.18363E+01_JPRB, 0.18212E+01_JPRB ,&
&  0.18508E+01_JPRB, 0.18649E+01_JPRB, 0.15857E+01_JPRB, 0.13909E+01_JPRB, 0.12745E+01_JPRB, 0.12101E+01_JPRB, 0.11393E+01_JPRB ,&
&  0.10485E+01_JPRB, 0.98462E+00_JPRB, 0.97067E+00_JPRB, 0.96539E+00_JPRB, 0.10047E+01_JPRB, 0.94640E+00_JPRB, 0.92839E+00_JPRB ,&
&  0.93762E+00_JPRB, 0.98485E+00_JPRB, 0.10693E+01_JPRB, 0.14600E+01_JPRB, 0.22807E+01_JPRB, 0.32720E+01_JPRB, 0.35352E+01_JPRB ,&
&  0.31484E+01_JPRB, 0.27529E+01_JPRB, 0.29368E+01_JPRB, 0.32385E+01_JPRB, 0.27877E+01_JPRB, 0.25389E+01_JPRB, 0.23802E+01_JPRB ,&
&  0.20028E+01_JPRB, 0.20411E+01_JPRB, 0.25173E+01_JPRB, 0.33453E+01_JPRB, 0.45554E+01_JPRB, 0.57856E+01_JPRB, 0.56271E+01_JPRB ,&
&  0.44695E+01_JPRB, 0.35314E+01_JPRB, 0.28370E+01_JPRB, 0.22466E+01_JPRB, 0.21357E+01_JPRB, 0.16327E+01_JPRB, 0.18096E+01_JPRB ,&
&  0.18103E+01_JPRB, 0.17214E+01_JPRB, 0.14274E+01_JPRB, 0.11704E+01_JPRB, 0.94909E+00_JPRB, 0.10128E+01_JPRB, 0.96797E+00_JPRB ,&
&  0.99082E+00_JPRB, 0.10371E+01_JPRB, 0.16279E+01_JPRB, 0.18870E+01_JPRB, 0.19872E+01_JPRB, 0.19387E+01_JPRB, 0.26229E+01_JPRB ,&
&  0.24939E+01_JPRB, 0.22669E+01_JPRB, 0.19116E+01_JPRB, 0.16532E+01_JPRB, 0.14298E+01_JPRB, 0.12836E+01_JPRB, 0.12029E+01_JPRB ,&
&  0.11027E+01_JPRB, 0.10489E+01_JPRB, 0.10049E+01_JPRB, 0.95005E+00_JPRB, 0.88383E+00_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.81839E+00_JPRB, 0.77628E+00_JPRB, 0.75721E+00_JPRB, 0.71131E+00_JPRB, 0.64313E+00_JPRB, 0.60745E+00_JPRB, 0.61160E+00_JPRB ,&
&  0.60053E+00_JPRB, 0.57573E+00_JPRB, 0.55064E+00_JPRB, 0.53174E+00_JPRB, 0.53473E+00_JPRB, 0.54553E+00_JPRB, 0.55259E+00_JPRB ,&
&  0.61294E+00_JPRB, 0.74655E+00_JPRB, 0.74830E+00_JPRB, 0.78638E+00_JPRB, 0.83090E+00_JPRB, 0.89371E+00_JPRB, 0.97178E+00_JPRB ,&
&  0.13160E+01_JPRB, 0.13443E+01_JPRB, 0.12843E+01_JPRB, 0.14570E+01_JPRB, 0.18355E+01_JPRB, 0.18491E+01_JPRB, 0.18403E+01_JPRB ,&
&  0.18653E+01_JPRB, 0.17587E+01_JPRB, 0.15060E+01_JPRB, 0.12982E+01_JPRB, 0.11583E+01_JPRB, 0.10781E+01_JPRB, 0.99102E+00_JPRB ,&
&  0.93670E+00_JPRB, 0.93339E+00_JPRB, 0.94812E+00_JPRB, 0.98073E+00_JPRB, 0.95038E+00_JPRB, 0.91708E+00_JPRB, 0.92501E+00_JPRB ,&
&  0.94478E+00_JPRB, 0.10085E+01_JPRB, 0.11689E+01_JPRB, 0.17243E+01_JPRB, 0.26244E+01_JPRB, 0.37225E+01_JPRB, 0.42474E+01_JPRB ,&
&  0.34223E+01_JPRB, 0.32374E+01_JPRB, 0.34094E+01_JPRB, 0.43734E+01_JPRB, 0.27848E+01_JPRB, 0.24556E+01_JPRB, 0.23627E+01_JPRB ,&
&  0.21963E+01_JPRB, 0.25519E+01_JPRB, 0.34453E+01_JPRB, 0.44767E+01_JPRB, 0.47611E+01_JPRB, 0.55695E+01_JPRB, 0.64219E+01_JPRB ,&
&  0.47123E+01_JPRB, 0.36422E+01_JPRB, 0.30734E+01_JPRB, 0.28101E+01_JPRB, 0.18380E+01_JPRB, 0.12481E+01_JPRB, 0.17476E+01_JPRB ,&
&  0.17111E+01_JPRB, 0.98821E+00_JPRB, 0.73447E+00_JPRB, 0.82399E+00_JPRB, 0.61192E+00_JPRB, 0.68801E+00_JPRB, 0.93751E+00_JPRB ,&
&  0.11106E+01_JPRB, 0.13072E+01_JPRB, 0.24376E+01_JPRB, 0.23744E+01_JPRB, 0.24097E+01_JPRB, 0.29204E+01_JPRB, 0.33454E+01_JPRB ,&
&  0.29639E+01_JPRB, 0.19788E+01_JPRB, 0.19672E+01_JPRB, 0.18062E+01_JPRB, 0.15426E+01_JPRB, 0.13364E+01_JPRB, 0.11867E+01_JPRB ,&
&  0.10581E+01_JPRB, 0.96663E+00_JPRB, 0.91162E+00_JPRB, 0.88038E+00_JPRB, 0.86649E+00_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.73285E+00_JPRB, 0.73122E+00_JPRB, 0.71288E+00_JPRB, 0.65057E+00_JPRB, 0.61053E+00_JPRB, 0.60585E+00_JPRB, 0.59428E+00_JPRB ,&
&  0.57583E+00_JPRB, 0.54907E+00_JPRB, 0.53401E+00_JPRB, 0.53962E+00_JPRB, 0.55025E+00_JPRB, 0.56644E+00_JPRB, 0.59873E+00_JPRB ,&
&  0.65560E+00_JPRB, 0.75822E+00_JPRB, 0.89264E+00_JPRB, 0.98378E+00_JPRB, 0.10319E+01_JPRB, 0.93687E+00_JPRB, 0.11176E+01_JPRB ,&
&  0.13459E+01_JPRB, 0.12838E+01_JPRB, 0.12670E+01_JPRB, 0.14926E+01_JPRB, 0.16565E+01_JPRB, 0.15817E+01_JPRB, 0.15958E+01_JPRB ,&
&  0.15920E+01_JPRB, 0.15066E+01_JPRB, 0.13577E+01_JPRB, 0.12583E+01_JPRB, 0.11302E+01_JPRB, 0.99679E+00_JPRB, 0.96140E+00_JPRB ,&
&  0.96270E+00_JPRB, 0.10099E+01_JPRB, 0.10658E+01_JPRB, 0.10233E+01_JPRB, 0.10048E+01_JPRB, 0.98107E+00_JPRB, 0.99385E+00_JPRB ,&
&  0.10246E+01_JPRB, 0.11198E+01_JPRB, 0.13861E+01_JPRB, 0.21821E+01_JPRB, 0.35248E+01_JPRB, 0.49579E+01_JPRB, 0.40856E+01_JPRB ,&
&  0.30688E+01_JPRB, 0.28101E+01_JPRB, 0.37281E+01_JPRB, 0.40965E+01_JPRB, 0.27043E+01_JPRB, 0.25597E+01_JPRB, 0.24637E+01_JPRB ,&
&  0.27738E+01_JPRB, 0.32684E+01_JPRB, 0.35609E+01_JPRB, 0.44437E+01_JPRB, 0.55214E+01_JPRB, 0.56809E+01_JPRB, 0.57886E+01_JPRB ,&
&  0.47085E+01_JPRB, 0.35793E+01_JPRB, 0.25705E+01_JPRB, 0.20086E+01_JPRB, 0.21562E+01_JPRB, 0.15480E+01_JPRB, 0.98537E+00_JPRB ,&
&  0.77137E+00_JPRB, 0.60806E+00_JPRB, 0.49056E+00_JPRB, 0.47728E+00_JPRB, 0.41884E+00_JPRB, 0.54420E+00_JPRB, 0.84831E+00_JPRB ,&
&  0.14904E+01_JPRB, 0.20703E+01_JPRB, 0.26684E+01_JPRB, 0.27719E+01_JPRB, 0.25153E+01_JPRB, 0.31469E+01_JPRB, 0.40354E+01_JPRB ,&
&  0.25216E+01_JPRB, 0.20466E+01_JPRB, 0.20343E+01_JPRB, 0.17172E+01_JPRB, 0.13519E+01_JPRB, 0.11179E+01_JPRB, 0.95243E+00_JPRB ,&
&  0.84681E+00_JPRB, 0.78559E+00_JPRB, 0.75391E+00_JPRB, 0.74613E+00_JPRB, 0.74836E+00_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.68350E+00_JPRB, 0.67286E+00_JPRB, 0.65580E+00_JPRB, 0.61954E+00_JPRB, 0.59641E+00_JPRB, 0.59009E+00_JPRB, 0.58150E+00_JPRB ,&
&  0.56155E+00_JPRB, 0.54054E+00_JPRB, 0.54150E+00_JPRB, 0.55998E+00_JPRB, 0.58733E+00_JPRB, 0.62818E+00_JPRB, 0.66808E+00_JPRB ,&
&  0.71447E+00_JPRB, 0.83602E+00_JPRB, 0.10300E+01_JPRB, 0.11957E+01_JPRB, 0.13178E+01_JPRB, 0.11006E+01_JPRB, 0.12487E+01_JPRB ,&
&  0.13198E+01_JPRB, 0.12954E+01_JPRB, 0.13523E+01_JPRB, 0.15261E+01_JPRB, 0.15986E+01_JPRB, 0.14156E+01_JPRB, 0.14513E+01_JPRB ,&
&  0.14228E+01_JPRB, 0.13694E+01_JPRB, 0.12701E+01_JPRB, 0.11972E+01_JPRB, 0.10906E+01_JPRB, 0.10904E+01_JPRB, 0.11569E+01_JPRB ,&
&  0.11955E+01_JPRB, 0.12511E+01_JPRB, 0.12082E+01_JPRB, 0.11630E+01_JPRB, 0.11376E+01_JPRB, 0.11220E+01_JPRB, 0.11449E+01_JPRB ,&
&  0.12157E+01_JPRB, 0.14137E+01_JPRB, 0.19525E+01_JPRB, 0.31923E+01_JPRB, 0.41872E+01_JPRB, 0.42032E+01_JPRB, 0.37663E+01_JPRB ,&
&  0.33994E+01_JPRB, 0.35081E+01_JPRB, 0.36716E+01_JPRB, 0.42207E+01_JPRB, 0.28669E+01_JPRB, 0.25997E+01_JPRB, 0.25605E+01_JPRB ,&
&  0.29218E+01_JPRB, 0.38557E+01_JPRB, 0.41536E+01_JPRB, 0.47111E+01_JPRB, 0.67621E+01_JPRB, 0.69827E+01_JPRB, 0.53968E+01_JPRB ,&
&  0.46017E+01_JPRB, 0.34522E+01_JPRB, 0.27011E+01_JPRB, 0.22341E+01_JPRB, 0.29082E+01_JPRB, 0.27575E+01_JPRB, 0.10469E+01_JPRB ,&
&  0.62732E+00_JPRB, 0.49839E+00_JPRB, 0.37025E+00_JPRB, 0.35646E+00_JPRB, 0.36971E+00_JPRB, 0.52263E+00_JPRB, 0.11857E+01_JPRB ,&
&  0.13824E+01_JPRB, 0.21784E+01_JPRB, 0.24373E+01_JPRB, 0.26302E+01_JPRB, 0.24790E+01_JPRB, 0.24447E+01_JPRB, 0.31642E+01_JPRB ,&
&  0.28870E+01_JPRB, 0.22824E+01_JPRB, 0.17108E+01_JPRB, 0.13253E+01_JPRB, 0.10448E+01_JPRB, 0.90094E+00_JPRB, 0.81335E+00_JPRB ,&
&  0.76533E+00_JPRB, 0.73964E+00_JPRB, 0.72117E+00_JPRB, 0.70370E+00_JPRB, 0.68319E+00_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.63199E+00_JPRB, 0.63227E+00_JPRB, 0.64253E+00_JPRB, 0.63373E+00_JPRB, 0.62148E+00_JPRB, 0.61707E+00_JPRB, 0.60532E+00_JPRB ,&
&  0.58319E+00_JPRB, 0.56703E+00_JPRB, 0.58036E+00_JPRB, 0.60927E+00_JPRB, 0.65515E+00_JPRB, 0.70682E+00_JPRB, 0.74735E+00_JPRB ,&
&  0.82026E+00_JPRB, 0.98048E+00_JPRB, 0.11961E+01_JPRB, 0.14803E+01_JPRB, 0.15633E+01_JPRB, 0.13678E+01_JPRB, 0.13696E+01_JPRB ,&
&  0.13525E+01_JPRB, 0.13958E+01_JPRB, 0.14151E+01_JPRB, 0.15609E+01_JPRB, 0.15780E+01_JPRB, 0.14900E+01_JPRB, 0.14527E+01_JPRB ,&
&  0.14330E+01_JPRB, 0.14270E+01_JPRB, 0.13625E+01_JPRB, 0.13682E+01_JPRB, 0.14220E+01_JPRB, 0.14900E+01_JPRB, 0.15554E+01_JPRB ,&
&  0.15821E+01_JPRB, 0.15251E+01_JPRB, 0.14309E+01_JPRB, 0.13642E+01_JPRB, 0.13389E+01_JPRB, 0.13565E+01_JPRB, 0.14284E+01_JPRB ,&
&  0.15906E+01_JPRB, 0.19942E+01_JPRB, 0.28806E+01_JPRB, 0.40129E+01_JPRB, 0.38659E+01_JPRB, 0.41779E+01_JPRB, 0.38042E+01_JPRB ,&
&  0.34996E+01_JPRB, 0.35457E+01_JPRB, 0.34035E+01_JPRB, 0.36895E+01_JPRB, 0.30790E+01_JPRB, 0.28661E+01_JPRB, 0.28035E+01_JPRB ,&
&  0.37580E+01_JPRB, 0.44302E+01_JPRB, 0.53378E+01_JPRB, 0.57263E+01_JPRB, 0.70027E+01_JPRB, 0.89569E+01_JPRB, 0.55605E+01_JPRB ,&
&  0.39494E+01_JPRB, 0.35893E+01_JPRB, 0.28115E+01_JPRB, 0.23594E+01_JPRB, 0.27118E+01_JPRB, 0.32534E+01_JPRB, 0.22851E+01_JPRB ,&
&  0.77515E+00_JPRB, 0.49722E+00_JPRB, 0.42156E+00_JPRB, 0.41329E+00_JPRB, 0.35349E+00_JPRB, 0.52230E+00_JPRB, 0.16241E+01_JPRB ,&
&  0.14692E+01_JPRB, 0.19414E+01_JPRB, 0.21545E+01_JPRB, 0.23073E+01_JPRB, 0.22977E+01_JPRB, 0.17920E+01_JPRB, 0.17776E+01_JPRB ,&
&  0.12788E+01_JPRB, 0.10020E+01_JPRB, 0.89844E+00_JPRB, 0.84199E+00_JPRB, 0.76003E+00_JPRB, 0.71604E+00_JPRB, 0.67504E+00_JPRB ,&
&  0.65682E+00_JPRB, 0.65151E+00_JPRB, 0.64400E+00_JPRB, 0.63407E+00_JPRB, 0.63590E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.61273E+00_JPRB, 0.63077E+00_JPRB, 0.65215E+00_JPRB, 0.67312E+00_JPRB, 0.68134E+00_JPRB, 0.66879E+00_JPRB, 0.64956E+00_JPRB ,&
&  0.64009E+00_JPRB, 0.63555E+00_JPRB, 0.65594E+00_JPRB, 0.68928E+00_JPRB, 0.75124E+00_JPRB, 0.81658E+00_JPRB, 0.90803E+00_JPRB ,&
&  0.10481E+01_JPRB, 0.12361E+01_JPRB, 0.14580E+01_JPRB, 0.16812E+01_JPRB, 0.16878E+01_JPRB, 0.14246E+01_JPRB, 0.13624E+01_JPRB ,&
&  0.14083E+01_JPRB, 0.15412E+01_JPRB, 0.15039E+01_JPRB, 0.16713E+01_JPRB, 0.17743E+01_JPRB, 0.17709E+01_JPRB, 0.17236E+01_JPRB ,&
&  0.17054E+01_JPRB, 0.17060E+01_JPRB, 0.17102E+01_JPRB, 0.17707E+01_JPRB, 0.18351E+01_JPRB, 0.18813E+01_JPRB, 0.18910E+01_JPRB ,&
&  0.18146E+01_JPRB, 0.17078E+01_JPRB, 0.16431E+01_JPRB, 0.16273E+01_JPRB, 0.16569E+01_JPRB, 0.17151E+01_JPRB, 0.18658E+01_JPRB ,&
&  0.21982E+01_JPRB, 0.27984E+01_JPRB, 0.36161E+01_JPRB, 0.39650E+01_JPRB, 0.38855E+01_JPRB, 0.37657E+01_JPRB, 0.35729E+01_JPRB ,&
&  0.34272E+01_JPRB, 0.35163E+01_JPRB, 0.34768E+01_JPRB, 0.34085E+01_JPRB, 0.31352E+01_JPRB, 0.31872E+01_JPRB, 0.33707E+01_JPRB ,&
&  0.45688E+01_JPRB, 0.55466E+01_JPRB, 0.57543E+01_JPRB, 0.57599E+01_JPRB, 0.60629E+01_JPRB, 0.69917E+01_JPRB, 0.64101E+01_JPRB ,&
&  0.46702E+01_JPRB, 0.32901E+01_JPRB, 0.26391E+01_JPRB, 0.25783E+01_JPRB, 0.25409E+01_JPRB, 0.26046E+01_JPRB, 0.28782E+01_JPRB ,&
&  0.23288E+01_JPRB, 0.92095E+00_JPRB, 0.58457E+00_JPRB, 0.75897E+00_JPRB, 0.48962E+00_JPRB, 0.53669E+00_JPRB, 0.90832E+00_JPRB ,&
&  0.11683E+01_JPRB, 0.14728E+01_JPRB, 0.16784E+01_JPRB, 0.18950E+01_JPRB, 0.16848E+01_JPRB, 0.13160E+01_JPRB, 0.10143E+01_JPRB ,&
&  0.81307E+00_JPRB, 0.75730E+00_JPRB, 0.75801E+00_JPRB, 0.71263E+00_JPRB, 0.65679E+00_JPRB, 0.63492E+00_JPRB, 0.60985E+00_JPRB ,&
&  0.59807E+00_JPRB, 0.61282E+00_JPRB, 0.62447E+00_JPRB, 0.61814E+00_JPRB, 0.60544E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.65288E+00_JPRB, 0.67850E+00_JPRB, 0.72512E+00_JPRB, 0.73899E+00_JPRB, 0.72677E+00_JPRB, 0.72273E+00_JPRB, 0.73314E+00_JPRB ,&
&  0.74789E+00_JPRB, 0.76581E+00_JPRB, 0.80431E+00_JPRB, 0.87032E+00_JPRB, 0.97738E+00_JPRB, 0.10862E+01_JPRB, 0.12206E+01_JPRB ,&
&  0.13700E+01_JPRB, 0.15268E+01_JPRB, 0.16919E+01_JPRB, 0.18438E+01_JPRB, 0.19246E+01_JPRB, 0.19685E+01_JPRB, 0.13103E+01_JPRB ,&
&  0.12128E+01_JPRB, 0.17173E+01_JPRB, 0.16295E+01_JPRB, 0.18114E+01_JPRB, 0.19294E+01_JPRB, 0.20041E+01_JPRB, 0.19433E+01_JPRB ,&
&  0.19387E+01_JPRB, 0.19601E+01_JPRB, 0.20008E+01_JPRB, 0.20139E+01_JPRB, 0.20176E+01_JPRB, 0.20184E+01_JPRB, 0.19778E+01_JPRB ,&
&  0.19219E+01_JPRB, 0.18904E+01_JPRB, 0.19057E+01_JPRB, 0.19544E+01_JPRB, 0.20427E+01_JPRB, 0.21888E+01_JPRB, 0.24410E+01_JPRB ,&
&  0.28113E+01_JPRB, 0.32701E+01_JPRB, 0.37043E+01_JPRB, 0.35945E+01_JPRB, 0.35016E+01_JPRB, 0.33930E+01_JPRB, 0.33930E+01_JPRB ,&
&  0.32531E+01_JPRB, 0.30373E+01_JPRB, 0.32608E+01_JPRB, 0.34418E+01_JPRB, 0.33291E+01_JPRB, 0.34058E+01_JPRB, 0.38347E+01_JPRB ,&
&  0.51241E+01_JPRB, 0.58793E+01_JPRB, 0.61512E+01_JPRB, 0.52233E+01_JPRB, 0.48788E+01_JPRB, 0.53734E+01_JPRB, 0.55339E+01_JPRB ,&
&  0.46911E+01_JPRB, 0.36141E+01_JPRB, 0.29622E+01_JPRB, 0.25096E+01_JPRB, 0.22114E+01_JPRB, 0.20856E+01_JPRB, 0.21485E+01_JPRB ,&
&  0.21162E+01_JPRB, 0.22204E+01_JPRB, 0.16977E+01_JPRB, 0.97812E+00_JPRB, 0.71266E+00_JPRB, 0.71895E+00_JPRB, 0.88412E+00_JPRB ,&
&  0.10463E+01_JPRB, 0.11839E+01_JPRB, 0.11985E+01_JPRB, 0.16227E+01_JPRB, 0.10736E+01_JPRB, 0.85040E+00_JPRB, 0.70460E+00_JPRB ,&
&  0.68088E+00_JPRB, 0.69801E+00_JPRB, 0.70340E+00_JPRB, 0.66692E+00_JPRB, 0.62818E+00_JPRB, 0.62938E+00_JPRB, 0.62340E+00_JPRB ,&
&  0.63649E+00_JPRB, 0.64155E+00_JPRB, 0.63441E+00_JPRB, 0.63693E+00_JPRB, 0.64785E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.74745E+00_JPRB, 0.78273E+00_JPRB, 0.81377E+00_JPRB, 0.82017E+00_JPRB, 0.83072E+00_JPRB, 0.85275E+00_JPRB, 0.89802E+00_JPRB ,&
&  0.95032E+00_JPRB, 0.10079E+01_JPRB, 0.10835E+01_JPRB, 0.11779E+01_JPRB, 0.12998E+01_JPRB, 0.14079E+01_JPRB, 0.15246E+01_JPRB ,&
&  0.16270E+01_JPRB, 0.17127E+01_JPRB, 0.17963E+01_JPRB, 0.18723E+01_JPRB, 0.19183E+01_JPRB, 0.19860E+01_JPRB, 0.16740E+01_JPRB ,&
&  0.11448E+01_JPRB, 0.19416E+01_JPRB, 0.17050E+01_JPRB, 0.18819E+01_JPRB, 0.19211E+01_JPRB, 0.20572E+01_JPRB, 0.20395E+01_JPRB ,&
&  0.20539E+01_JPRB, 0.20517E+01_JPRB, 0.20411E+01_JPRB, 0.20486E+01_JPRB, 0.20482E+01_JPRB, 0.20454E+01_JPRB, 0.20421E+01_JPRB ,&
&  0.20545E+01_JPRB, 0.21013E+01_JPRB, 0.21826E+01_JPRB, 0.22774E+01_JPRB, 0.24328E+01_JPRB, 0.26428E+01_JPRB, 0.28509E+01_JPRB ,&
&  0.30815E+01_JPRB, 0.33290E+01_JPRB, 0.34001E+01_JPRB, 0.32732E+01_JPRB, 0.31875E+01_JPRB, 0.32397E+01_JPRB, 0.33914E+01_JPRB ,&
&  0.32921E+01_JPRB, 0.31279E+01_JPRB, 0.32834E+01_JPRB, 0.33031E+01_JPRB, 0.32626E+01_JPRB, 0.37037E+01_JPRB, 0.41035E+01_JPRB ,&
&  0.50602E+01_JPRB, 0.52181E+01_JPRB, 0.59992E+01_JPRB, 0.49394E+01_JPRB, 0.40656E+01_JPRB, 0.42244E+01_JPRB, 0.38369E+01_JPRB ,&
&  0.31250E+01_JPRB, 0.28953E+01_JPRB, 0.25515E+01_JPRB, 0.20889E+01_JPRB, 0.19321E+01_JPRB, 0.17506E+01_JPRB, 0.17842E+01_JPRB ,&
&  0.18098E+01_JPRB, 0.17735E+01_JPRB, 0.13400E+01_JPRB, 0.83183E+00_JPRB, 0.73850E+00_JPRB, 0.76910E+00_JPRB, 0.90509E+00_JPRB ,&
&  0.97331E+00_JPRB, 0.10703E+01_JPRB, 0.11428E+01_JPRB, 0.10551E+01_JPRB, 0.75374E+00_JPRB, 0.63707E+00_JPRB, 0.60947E+00_JPRB ,&
&  0.64639E+00_JPRB, 0.68216E+00_JPRB, 0.68087E+00_JPRB, 0.66828E+00_JPRB, 0.64298E+00_JPRB, 0.66999E+00_JPRB, 0.68267E+00_JPRB ,&
&  0.71270E+00_JPRB, 0.70827E+00_JPRB, 0.72330E+00_JPRB, 0.73518E+00_JPRB, 0.73947E+00_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.92578E+00_JPRB, 0.99035E+00_JPRB, 0.10685E+01_JPRB, 0.11430E+01_JPRB, 0.11848E+01_JPRB, 0.12220E+01_JPRB, 0.12404E+01_JPRB ,&
&  0.12207E+01_JPRB, 0.11996E+01_JPRB, 0.12860E+01_JPRB, 0.13713E+01_JPRB, 0.14783E+01_JPRB, 0.15752E+01_JPRB, 0.16680E+01_JPRB ,&
&  0.17400E+01_JPRB, 0.18076E+01_JPRB, 0.18822E+01_JPRB, 0.19851E+01_JPRB, 0.20955E+01_JPRB, 0.22850E+01_JPRB, 0.21353E+01_JPRB ,&
&  0.15498E+01_JPRB, 0.14499E+01_JPRB, 0.18152E+01_JPRB, 0.18350E+01_JPRB, 0.17759E+01_JPRB, 0.19267E+01_JPRB, 0.20487E+01_JPRB ,&
&  0.20393E+01_JPRB, 0.20636E+01_JPRB, 0.20555E+01_JPRB, 0.20648E+01_JPRB, 0.20456E+01_JPRB, 0.20759E+01_JPRB, 0.21238E+01_JPRB ,&
&  0.22043E+01_JPRB, 0.23047E+01_JPRB, 0.24168E+01_JPRB, 0.25562E+01_JPRB, 0.27089E+01_JPRB, 0.28801E+01_JPRB, 0.30322E+01_JPRB ,&
&  0.31601E+01_JPRB, 0.32829E+01_JPRB, 0.32801E+01_JPRB, 0.31293E+01_JPRB, 0.31303E+01_JPRB, 0.32309E+01_JPRB, 0.32717E+01_JPRB ,&
&  0.32481E+01_JPRB, 0.33191E+01_JPRB, 0.34563E+01_JPRB, 0.34836E+01_JPRB, 0.34103E+01_JPRB, 0.38938E+01_JPRB, 0.41900E+01_JPRB ,&
&  0.46290E+01_JPRB, 0.46994E+01_JPRB, 0.54853E+01_JPRB, 0.45806E+01_JPRB, 0.38416E+01_JPRB, 0.35539E+01_JPRB, 0.30841E+01_JPRB ,&
&  0.26721E+01_JPRB, 0.25822E+01_JPRB, 0.24288E+01_JPRB, 0.18441E+01_JPRB, 0.17246E+01_JPRB, 0.13652E+01_JPRB, 0.16059E+01_JPRB ,&
&  0.17184E+01_JPRB, 0.18633E+01_JPRB, 0.11960E+01_JPRB, 0.89482E+00_JPRB, 0.76173E+00_JPRB, 0.80777E+00_JPRB, 0.85930E+00_JPRB ,&
&  0.86985E+00_JPRB, 0.88487E+00_JPRB, 0.83761E+00_JPRB, 0.81382E+00_JPRB, 0.63081E+00_JPRB, 0.58049E+00_JPRB, 0.57706E+00_JPRB ,&
&  0.62558E+00_JPRB, 0.66691E+00_JPRB, 0.67391E+00_JPRB, 0.68020E+00_JPRB, 0.69903E+00_JPRB, 0.74168E+00_JPRB, 0.77029E+00_JPRB ,&
&  0.80598E+00_JPRB, 0.82360E+00_JPRB, 0.84832E+00_JPRB, 0.86009E+00_JPRB, 0.87818E+00_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.95293E+00_JPRB, 0.10042E+01_JPRB, 0.10865E+01_JPRB, 0.11534E+01_JPRB, 0.11951E+01_JPRB, 0.12052E+01_JPRB, 0.11902E+01_JPRB ,&
&  0.11965E+01_JPRB, 0.12325E+01_JPRB, 0.13088E+01_JPRB, 0.13724E+01_JPRB, 0.14443E+01_JPRB, 0.15134E+01_JPRB, 0.15830E+01_JPRB ,&
&  0.16455E+01_JPRB, 0.17020E+01_JPRB, 0.17823E+01_JPRB, 0.18885E+01_JPRB, 0.20154E+01_JPRB, 0.22240E+01_JPRB, 0.22930E+01_JPRB ,&
&  0.18458E+01_JPRB, 0.13114E+01_JPRB, 0.15931E+01_JPRB, 0.16719E+01_JPRB, 0.15980E+01_JPRB, 0.17485E+01_JPRB, 0.18566E+01_JPRB ,&
&  0.18217E+01_JPRB, 0.18686E+01_JPRB, 0.19413E+01_JPRB, 0.19883E+01_JPRB, 0.20380E+01_JPRB, 0.21370E+01_JPRB, 0.22271E+01_JPRB ,&
&  0.23332E+01_JPRB, 0.24338E+01_JPRB, 0.25295E+01_JPRB, 0.26342E+01_JPRB, 0.27322E+01_JPRB, 0.28459E+01_JPRB, 0.29851E+01_JPRB ,&
&  0.31389E+01_JPRB, 0.32440E+01_JPRB, 0.32290E+01_JPRB, 0.31826E+01_JPRB, 0.31765E+01_JPRB, 0.32650E+01_JPRB, 0.33502E+01_JPRB ,&
&  0.33981E+01_JPRB, 0.33845E+01_JPRB, 0.36158E+01_JPRB, 0.39269E+01_JPRB, 0.40680E+01_JPRB, 0.39208E+01_JPRB, 0.41615E+01_JPRB ,&
&  0.43410E+01_JPRB, 0.42753E+01_JPRB, 0.45343E+01_JPRB, 0.50273E+01_JPRB, 0.34415E+01_JPRB, 0.32544E+01_JPRB, 0.25940E+01_JPRB ,&
&  0.27454E+01_JPRB, 0.26082E+01_JPRB, 0.23772E+01_JPRB, 0.20418E+01_JPRB, 0.15494E+01_JPRB, 0.11684E+01_JPRB, 0.13216E+01_JPRB ,&
&  0.15469E+01_JPRB, 0.14227E+01_JPRB, 0.10525E+01_JPRB, 0.88135E+00_JPRB, 0.80213E+00_JPRB, 0.91704E+00_JPRB, 0.92855E+00_JPRB ,&
&  0.85949E+00_JPRB, 0.85439E+00_JPRB, 0.84084E+00_JPRB, 0.93779E+00_JPRB, 0.77223E+00_JPRB, 0.57861E+00_JPRB, 0.54910E+00_JPRB ,&
&  0.59832E+00_JPRB, 0.64752E+00_JPRB, 0.69084E+00_JPRB, 0.72372E+00_JPRB, 0.75443E+00_JPRB, 0.78812E+00_JPRB, 0.81126E+00_JPRB ,&
&  0.85989E+00_JPRB, 0.85510E+00_JPRB, 0.89344E+00_JPRB, 0.91062E+00_JPRB, 0.93507E+00_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.94510E+00_JPRB, 0.97371E+00_JPRB, 0.10029E+01_JPRB, 0.10440E+01_JPRB, 0.10814E+01_JPRB, 0.10942E+01_JPRB, 0.10913E+01_JPRB ,&
&  0.11066E+01_JPRB, 0.11466E+01_JPRB, 0.11914E+01_JPRB, 0.12310E+01_JPRB, 0.12876E+01_JPRB, 0.13373E+01_JPRB, 0.13907E+01_JPRB ,&
&  0.14473E+01_JPRB, 0.15231E+01_JPRB, 0.16354E+01_JPRB, 0.17627E+01_JPRB, 0.18778E+01_JPRB, 0.20300E+01_JPRB, 0.21842E+01_JPRB ,&
&  0.23295E+01_JPRB, 0.21147E+01_JPRB, 0.20097E+01_JPRB, 0.16035E+01_JPRB, 0.11770E+01_JPRB, 0.14754E+01_JPRB, 0.15228E+01_JPRB ,&
&  0.15836E+01_JPRB, 0.16362E+01_JPRB, 0.16840E+01_JPRB, 0.17641E+01_JPRB, 0.19091E+01_JPRB, 0.20346E+01_JPRB, 0.21275E+01_JPRB ,&
&  0.22068E+01_JPRB, 0.22758E+01_JPRB, 0.23312E+01_JPRB, 0.23961E+01_JPRB, 0.24727E+01_JPRB, 0.25470E+01_JPRB, 0.26274E+01_JPRB ,&
&  0.27456E+01_JPRB, 0.28793E+01_JPRB, 0.29689E+01_JPRB, 0.30666E+01_JPRB, 0.30957E+01_JPRB, 0.32448E+01_JPRB, 0.34152E+01_JPRB ,&
&  0.35197E+01_JPRB, 0.35758E+01_JPRB, 0.37976E+01_JPRB, 0.40652E+01_JPRB, 0.40943E+01_JPRB, 0.36512E+01_JPRB, 0.37852E+01_JPRB ,&
&  0.39890E+01_JPRB, 0.38628E+01_JPRB, 0.37646E+01_JPRB, 0.48376E+01_JPRB, 0.29790E+01_JPRB, 0.31337E+01_JPRB, 0.30602E+01_JPRB ,&
&  0.26836E+01_JPRB, 0.25688E+01_JPRB, 0.22872E+01_JPRB, 0.18143E+01_JPRB, 0.14729E+01_JPRB, 0.10798E+01_JPRB, 0.12025E+01_JPRB ,&
&  0.13309E+01_JPRB, 0.11241E+01_JPRB, 0.88111E+00_JPRB, 0.75047E+00_JPRB, 0.77394E+00_JPRB, 0.93728E+00_JPRB, 0.95317E+00_JPRB ,&
&  0.79531E+00_JPRB, 0.85520E+00_JPRB, 0.79885E+00_JPRB, 0.11255E+01_JPRB, 0.11576E+01_JPRB, 0.56668E+00_JPRB, 0.48729E+00_JPRB ,&
&  0.52085E+00_JPRB, 0.58749E+00_JPRB, 0.66255E+00_JPRB, 0.71180E+00_JPRB, 0.72811E+00_JPRB, 0.75042E+00_JPRB, 0.77281E+00_JPRB ,&
&  0.79812E+00_JPRB, 0.83805E+00_JPRB, 0.88990E+00_JPRB, 0.91279E+00_JPRB, 0.93079E+00_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.80901E+00_JPRB, 0.83282E+00_JPRB, 0.85077E+00_JPRB, 0.85539E+00_JPRB, 0.84710E+00_JPRB, 0.83542E+00_JPRB, 0.81038E+00_JPRB ,&
&  0.78253E+00_JPRB, 0.79292E+00_JPRB, 0.83410E+00_JPRB, 0.92766E+00_JPRB, 0.99427E+00_JPRB, 0.10346E+01_JPRB, 0.10932E+01_JPRB ,&
&  0.11533E+01_JPRB, 0.11806E+01_JPRB, 0.11804E+01_JPRB, 0.12181E+01_JPRB, 0.12543E+01_JPRB, 0.12925E+01_JPRB, 0.14769E+01_JPRB ,&
&  0.17497E+01_JPRB, 0.20780E+01_JPRB, 0.23935E+01_JPRB, 0.26295E+01_JPRB, 0.21626E+01_JPRB, 0.14896E+01_JPRB, 0.15706E+01_JPRB ,&
&  0.17455E+01_JPRB, 0.16650E+01_JPRB, 0.14933E+01_JPRB, 0.14994E+01_JPRB, 0.15357E+01_JPRB, 0.16227E+01_JPRB, 0.17334E+01_JPRB ,&
&  0.18104E+01_JPRB, 0.18634E+01_JPRB, 0.19158E+01_JPRB, 0.19603E+01_JPRB, 0.20064E+01_JPRB, 0.20530E+01_JPRB, 0.21128E+01_JPRB ,&
&  0.21587E+01_JPRB, 0.21968E+01_JPRB, 0.23203E+01_JPRB, 0.25011E+01_JPRB, 0.26831E+01_JPRB, 0.29262E+01_JPRB, 0.31349E+01_JPRB ,&
&  0.32582E+01_JPRB, 0.33557E+01_JPRB, 0.37261E+01_JPRB, 0.39663E+01_JPRB, 0.37507E+01_JPRB, 0.34071E+01_JPRB, 0.35333E+01_JPRB ,&
&  0.35505E+01_JPRB, 0.33197E+01_JPRB, 0.26327E+01_JPRB, 0.38063E+01_JPRB, 0.38332E+01_JPRB, 0.32905E+01_JPRB, 0.27083E+01_JPRB ,&
&  0.24214E+01_JPRB, 0.23069E+01_JPRB, 0.20896E+01_JPRB, 0.17908E+01_JPRB, 0.14744E+01_JPRB, 0.14382E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.12241E+01_JPRB, 0.99936E+00_JPRB, 0.77815E+00_JPRB, 0.67791E+00_JPRB, 0.66153E+00_JPRB, 0.83581E+00_JPRB, 0.81454E+00_JPRB ,&
&  0.67724E+00_JPRB, 0.71781E+00_JPRB, 0.69830E+00_JPRB, 0.11256E+01_JPRB, 0.21980E+01_JPRB, 0.58863E+00_JPRB, 0.44107E+00_JPRB ,&
&  0.46635E+00_JPRB, 0.50726E+00_JPRB, 0.54655E+00_JPRB, 0.56050E+00_JPRB, 0.56238E+00_JPRB, 0.58611E+00_JPRB, 0.62680E+00_JPRB ,&
&  0.67162E+00_JPRB, 0.70341E+00_JPRB, 0.71910E+00_JPRB, 0.75525E+00_JPRB, 0.78438E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.62720E+00_JPRB, 0.64117E+00_JPRB, 0.63936E+00_JPRB, 0.64354E+00_JPRB, 0.63637E+00_JPRB, 0.63585E+00_JPRB, 0.61902E+00_JPRB ,&
&  0.60519E+00_JPRB, 0.61225E+00_JPRB, 0.64191E+00_JPRB, 0.69975E+00_JPRB, 0.71356E+00_JPRB, 0.72892E+00_JPRB, 0.73179E+00_JPRB ,&
&  0.75218E+00_JPRB, 0.74223E+00_JPRB, 0.73887E+00_JPRB, 0.72149E+00_JPRB, 0.73179E+00_JPRB, 0.77184E+00_JPRB, 0.85874E+00_JPRB ,&
&  0.97981E+00_JPRB, 0.11319E+01_JPRB, 0.13475E+01_JPRB, 0.15931E+01_JPRB, 0.21237E+01_JPRB, 0.17975E+01_JPRB, 0.15212E+01_JPRB ,&
&  0.16245E+01_JPRB, 0.13838E+01_JPRB, 0.12888E+01_JPRB, 0.12251E+01_JPRB, 0.11738E+01_JPRB, 0.11369E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11912E+01_JPRB, 0.12365E+01_JPRB, 0.12899E+01_JPRB, 0.13121E+01_JPRB, 0.13370E+01_JPRB, 0.13233E+01_JPRB, 0.13021E+01_JPRB ,&
&  0.12607E+01_JPRB, 0.13300E+01_JPRB, 0.14812E+01_JPRB, 0.15894E+01_JPRB, 0.18777E+01_JPRB, 0.21133E+01_JPRB, 0.23096E+01_JPRB ,&
&  0.24140E+01_JPRB, 0.25711E+01_JPRB, 0.29082E+01_JPRB, 0.29996E+01_JPRB, 0.30608E+01_JPRB, 0.29204E+01_JPRB, 0.28314E+01_JPRB ,&
&  0.26665E+01_JPRB, 0.21544E+01_JPRB, 0.17791E+01_JPRB, 0.26875E+01_JPRB, 0.27840E+01_JPRB, 0.22296E+01_JPRB, 0.23293E+01_JPRB ,&
&  0.21560E+01_JPRB, 0.20358E+01_JPRB, 0.19704E+01_JPRB, 0.17132E+01_JPRB, 0.13876E+01_JPRB, 0.13278E+01_JPRB, 0.12679E+01_JPRB ,&
&  0.11236E+01_JPRB, 0.85895E+00_JPRB, 0.67401E+00_JPRB, 0.61432E+00_JPRB, 0.61129E+00_JPRB, 0.72515E+00_JPRB, 0.73778E+00_JPRB ,&
&  0.74584E+00_JPRB, 0.59836E+00_JPRB, 0.60998E+00_JPRB, 0.85648E+00_JPRB, 0.78245E+00_JPRB, 0.48375E+00_JPRB, 0.39897E+00_JPRB ,&
&  0.41544E+00_JPRB, 0.43672E+00_JPRB, 0.45750E+00_JPRB, 0.46856E+00_JPRB, 0.47275E+00_JPRB, 0.48021E+00_JPRB, 0.51065E+00_JPRB ,&
&  0.53579E+00_JPRB, 0.55889E+00_JPRB, 0.57816E+00_JPRB, 0.60500E+00_JPRB, 0.61995E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.63259E+00_JPRB, 0.64409E+00_JPRB, 0.66092E+00_JPRB, 0.67124E+00_JPRB, 0.68274E+00_JPRB, 0.68960E+00_JPRB, 0.71672E+00_JPRB ,&
&  0.73555E+00_JPRB, 0.75279E+00_JPRB, 0.77770E+00_JPRB, 0.80431E+00_JPRB, 0.81488E+00_JPRB, 0.82630E+00_JPRB, 0.83045E+00_JPRB ,&
&  0.84509E+00_JPRB, 0.86161E+00_JPRB, 0.85847E+00_JPRB, 0.86368E+00_JPRB, 0.86899E+00_JPRB, 0.93294E+00_JPRB, 0.93293E+00_JPRB ,&
&  0.92423E+00_JPRB, 0.93868E+00_JPRB, 0.95330E+00_JPRB, 0.10066E+01_JPRB, 0.11634E+01_JPRB, 0.12832E+01_JPRB, 0.12870E+01_JPRB ,&
&  0.13044E+01_JPRB, 0.11046E+01_JPRB, 0.11172E+01_JPRB, 0.90305E+00_JPRB, 0.86203E+00_JPRB, 0.87765E+00_JPRB, 0.84944E+00_JPRB ,&
&  0.84315E+00_JPRB, 0.86718E+00_JPRB, 0.92085E+00_JPRB, 0.92499E+00_JPRB, 0.97103E+00_JPRB, 0.10072E+01_JPRB, 0.10262E+01_JPRB ,&
&  0.10645E+01_JPRB, 0.11187E+01_JPRB, 0.11782E+01_JPRB, 0.12838E+01_JPRB, 0.13615E+01_JPRB, 0.15987E+01_JPRB, 0.16957E+01_JPRB ,&
&  0.17304E+01_JPRB, 0.18094E+01_JPRB, 0.16847E+01_JPRB, 0.17146E+01_JPRB, 0.19572E+01_JPRB, 0.19597E+01_JPRB, 0.19096E+01_JPRB ,&
&  0.19673E+01_JPRB, 0.17689E+01_JPRB, 0.13133E+01_JPRB, 0.19668E+01_JPRB, 0.21976E+01_JPRB, 0.20341E+01_JPRB, 0.20011E+01_JPRB ,&
&  0.18648E+01_JPRB, 0.16750E+01_JPRB, 0.15434E+01_JPRB, 0.14861E+01_JPRB, 0.13218E+01_JPRB, 0.11908E+01_JPRB, 0.11315E+01_JPRB ,&
&  0.93229E+00_JPRB, 0.74727E+00_JPRB, 0.63101E+00_JPRB, 0.64091E+00_JPRB, 0.73389E+00_JPRB, 0.95738E+00_JPRB, 0.88195E+00_JPRB ,&
&  0.71069E+00_JPRB, 0.56723E+00_JPRB, 0.71888E+00_JPRB, 0.99845E+00_JPRB, 0.62039E+00_JPRB, 0.47270E+00_JPRB, 0.39937E+00_JPRB ,&
&  0.41064E+00_JPRB, 0.43637E+00_JPRB, 0.46167E+00_JPRB, 0.48120E+00_JPRB, 0.50496E+00_JPRB, 0.52588E+00_JPRB, 0.54192E+00_JPRB ,&
&  0.55779E+00_JPRB, 0.57746E+00_JPRB, 0.59419E+00_JPRB, 0.61344E+00_JPRB, 0.62926E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.74701E+00_JPRB, 0.77601E+00_JPRB, 0.79553E+00_JPRB, 0.81129E+00_JPRB, 0.81979E+00_JPRB, 0.82847E+00_JPRB, 0.84584E+00_JPRB ,&
&  0.86005E+00_JPRB, 0.87503E+00_JPRB, 0.89116E+00_JPRB, 0.91416E+00_JPRB, 0.94371E+00_JPRB, 0.96710E+00_JPRB, 0.97673E+00_JPRB ,&
&  0.98289E+00_JPRB, 0.10163E+01_JPRB, 0.10376E+01_JPRB, 0.10792E+01_JPRB, 0.10955E+01_JPRB, 0.11238E+01_JPRB, 0.11309E+01_JPRB ,&
&  0.11282E+01_JPRB, 0.11326E+01_JPRB, 0.11414E+01_JPRB, 0.11623E+01_JPRB, 0.11801E+01_JPRB, 0.12933E+01_JPRB, 0.14613E+01_JPRB ,&
&  0.12257E+01_JPRB, 0.11031E+01_JPRB, 0.90961E+00_JPRB, 0.75926E+00_JPRB, 0.76072E+00_JPRB, 0.78649E+00_JPRB, 0.80485E+00_JPRB ,&
&  0.83021E+00_JPRB, 0.86164E+00_JPRB, 0.89766E+00_JPRB, 0.91299E+00_JPRB, 0.97837E+00_JPRB, 0.10946E+01_JPRB, 0.11332E+01_JPRB ,&
&  0.12021E+01_JPRB, 0.12647E+01_JPRB, 0.12982E+01_JPRB, 0.14266E+01_JPRB, 0.14228E+01_JPRB, 0.17047E+01_JPRB, 0.18415E+01_JPRB ,&
&  0.18752E+01_JPRB, 0.17738E+01_JPRB, 0.13662E+01_JPRB, 0.13689E+01_JPRB, 0.15122E+01_JPRB, 0.14784E+01_JPRB, 0.14310E+01_JPRB ,&
&  0.16660E+01_JPRB, 0.16471E+01_JPRB, 0.14431E+01_JPRB, 0.18220E+01_JPRB, 0.19286E+01_JPRB, 0.18827E+01_JPRB, 0.16682E+01_JPRB ,&
&  0.14661E+01_JPRB, 0.11624E+01_JPRB, 0.11536E+01_JPRB, 0.11706E+01_JPRB, 0.11062E+01_JPRB, 0.10260E+01_JPRB, 0.89595E+00_JPRB ,&
&  0.76726E+00_JPRB, 0.67462E+00_JPRB, 0.63430E+00_JPRB, 0.65918E+00_JPRB, 0.89866E+00_JPRB, 0.88791E+00_JPRB, 0.11114E+01_JPRB ,&
&  0.59792E+00_JPRB, 0.56266E+00_JPRB, 0.48050E+00_JPRB, 0.59476E+00_JPRB, 0.52471E+00_JPRB, 0.42066E+00_JPRB, 0.42974E+00_JPRB ,&
&  0.48347E+00_JPRB, 0.52936E+00_JPRB, 0.56228E+00_JPRB, 0.58584E+00_JPRB, 0.60622E+00_JPRB, 0.62981E+00_JPRB, 0.64068E+00_JPRB ,&
&  0.65661E+00_JPRB, 0.67407E+00_JPRB, 0.69099E+00_JPRB, 0.71564E+00_JPRB, 0.72776E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.78778E+00_JPRB, 0.81084E+00_JPRB, 0.83106E+00_JPRB, 0.84994E+00_JPRB, 0.86972E+00_JPRB, 0.88923E+00_JPRB, 0.90579E+00_JPRB ,&
&  0.92085E+00_JPRB, 0.93853E+00_JPRB, 0.95343E+00_JPRB, 0.96811E+00_JPRB, 0.98974E+00_JPRB, 0.10092E+01_JPRB, 0.10212E+01_JPRB ,&
&  0.10401E+01_JPRB, 0.10662E+01_JPRB, 0.10979E+01_JPRB, 0.11252E+01_JPRB, 0.11386E+01_JPRB, 0.11435E+01_JPRB, 0.11424E+01_JPRB ,&
&  0.11312E+01_JPRB, 0.11252E+01_JPRB, 0.11308E+01_JPRB, 0.11470E+01_JPRB, 0.11784E+01_JPRB, 0.11637E+01_JPRB, 0.12441E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.11719E+01_JPRB, 0.98559E+00_JPRB, 0.87583E+00_JPRB, 0.88886E+00_JPRB, 0.85295E+00_JPRB, 0.86721E+00_JPRB ,&
&  0.91958E+00_JPRB, 0.94584E+00_JPRB, 0.97463E+00_JPRB, 0.99931E+00_JPRB, 0.10565E+01_JPRB, 0.11456E+01_JPRB, 0.12073E+01_JPRB ,&
&  0.13128E+01_JPRB, 0.14338E+01_JPRB, 0.15275E+01_JPRB, 0.16425E+01_JPRB, 0.18824E+01_JPRB, 0.20828E+01_JPRB, 0.21820E+01_JPRB ,&
&  0.21600E+01_JPRB, 0.20610E+01_JPRB, 0.15662E+01_JPRB, 0.16223E+01_JPRB, 0.17278E+01_JPRB, 0.16130E+01_JPRB, 0.14758E+01_JPRB ,&
&  0.15332E+01_JPRB, 0.13154E+01_JPRB, 0.15127E+01_JPRB, 0.15533E+01_JPRB, 0.14904E+01_JPRB, 0.13314E+01_JPRB, 0.11121E+01_JPRB ,&
&  0.91647E+00_JPRB, 0.76886E+00_JPRB, 0.78854E+00_JPRB, 0.90345E+00_JPRB, 0.87211E+00_JPRB, 0.72748E+00_JPRB, 0.66272E+00_JPRB ,&
&  0.59318E+00_JPRB, 0.53221E+00_JPRB, 0.51621E+00_JPRB, 0.56545E+00_JPRB, 0.67492E+00_JPRB, 0.10520E+01_JPRB, 0.11011E+01_JPRB ,&
&  0.57356E+00_JPRB, 0.52275E+00_JPRB, 0.40542E+00_JPRB, 0.47167E+00_JPRB, 0.48425E+00_JPRB, 0.45420E+00_JPRB, 0.50368E+00_JPRB ,&
&  0.61711E+00_JPRB, 0.71517E+00_JPRB, 0.72499E+00_JPRB, 0.66307E+00_JPRB, 0.64137E+00_JPRB, 0.64930E+00_JPRB, 0.66871E+00_JPRB ,&
&  0.68876E+00_JPRB, 0.70721E+00_JPRB, 0.72562E+00_JPRB, 0.74541E+00_JPRB, 0.76552E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.80909E+00_JPRB, 0.83297E+00_JPRB, 0.85294E+00_JPRB, 0.86926E+00_JPRB, 0.88976E+00_JPRB, 0.91123E+00_JPRB, 0.92972E+00_JPRB ,&
&  0.94653E+00_JPRB, 0.96175E+00_JPRB, 0.97892E+00_JPRB, 0.99467E+00_JPRB, 0.10128E+01_JPRB, 0.10345E+01_JPRB, 0.10567E+01_JPRB ,&
&  0.10780E+01_JPRB, 0.10916E+01_JPRB, 0.11227E+01_JPRB, 0.11605E+01_JPRB, 0.11854E+01_JPRB, 0.11928E+01_JPRB, 0.11937E+01_JPRB ,&
&  0.11953E+01_JPRB, 0.12174E+01_JPRB, 0.12329E+01_JPRB, 0.12511E+01_JPRB, 0.12798E+01_JPRB, 0.13494E+01_JPRB, 0.11496E+01_JPRB ,&
&  0.12552E+01_JPRB, 0.11683E+01_JPRB, 0.10220E+01_JPRB, 0.97347E+00_JPRB, 0.96633E+00_JPRB, 0.96577E+00_JPRB, 0.97672E+00_JPRB ,&
&  0.10115E+01_JPRB, 0.10118E+01_JPRB, 0.10298E+01_JPRB, 0.10519E+01_JPRB, 0.11086E+01_JPRB, 0.11852E+01_JPRB, 0.12704E+01_JPRB ,&
&  0.14093E+01_JPRB, 0.15877E+01_JPRB, 0.17962E+01_JPRB, 0.20241E+01_JPRB, 0.22954E+01_JPRB, 0.25209E+01_JPRB, 0.26673E+01_JPRB ,&
&  0.26986E+01_JPRB, 0.26689E+01_JPRB, 0.22709E+01_JPRB, 0.23987E+01_JPRB, 0.23825E+01_JPRB, 0.19942E+01_JPRB, 0.16493E+01_JPRB ,&
&  0.13724E+01_JPRB, 0.10880E+01_JPRB, 0.11413E+01_JPRB, 0.12010E+01_JPRB, 0.11024E+01_JPRB, 0.93973E+00_JPRB, 0.78946E+00_JPRB ,&
&  0.61718E+00_JPRB, 0.56173E+00_JPRB, 0.55852E+00_JPRB, 0.65065E+00_JPRB, 0.64115E+00_JPRB, 0.56220E+00_JPRB, 0.54383E+00_JPRB ,&
&  0.51410E+00_JPRB, 0.47729E+00_JPRB, 0.47239E+00_JPRB, 0.52904E+00_JPRB, 0.63846E+00_JPRB, 0.63655E+00_JPRB, 0.69811E+00_JPRB ,&
&  0.55244E+00_JPRB, 0.48775E+00_JPRB, 0.50233E+00_JPRB, 0.52891E+00_JPRB, 0.57161E+00_JPRB, 0.60892E+00_JPRB, 0.71166E+00_JPRB ,&
&  0.87451E+00_JPRB, 0.10845E+01_JPRB, 0.10562E+01_JPRB, 0.84688E+00_JPRB, 0.68711E+00_JPRB, 0.65633E+00_JPRB, 0.67265E+00_JPRB ,&
&  0.69452E+00_JPRB, 0.71237E+00_JPRB, 0.73379E+00_JPRB, 0.75694E+00_JPRB, 0.78191E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.81599E+00_JPRB, 0.83774E+00_JPRB, 0.85435E+00_JPRB, 0.86668E+00_JPRB, 0.88513E+00_JPRB, 0.90426E+00_JPRB, 0.92244E+00_JPRB ,&
&  0.93841E+00_JPRB, 0.95132E+00_JPRB, 0.96801E+00_JPRB, 0.98254E+00_JPRB, 0.99989E+00_JPRB, 0.10201E+01_JPRB, 0.10435E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.10847E+01_JPRB, 0.10970E+01_JPRB, 0.11351E+01_JPRB, 0.11835E+01_JPRB, 0.12098E+01_JPRB, 0.12234E+01_JPRB ,&
&  0.12212E+01_JPRB, 0.12407E+01_JPRB, 0.12621E+01_JPRB, 0.12876E+01_JPRB, 0.12951E+01_JPRB, 0.13488E+01_JPRB, 0.11743E+01_JPRB ,&
&  0.12258E+01_JPRB, 0.11590E+01_JPRB, 0.10573E+01_JPRB, 0.10431E+01_JPRB, 0.10361E+01_JPRB, 0.10401E+01_JPRB, 0.10435E+01_JPRB ,&
&  0.10480E+01_JPRB, 0.10699E+01_JPRB, 0.10747E+01_JPRB, 0.10927E+01_JPRB, 0.11366E+01_JPRB, 0.12129E+01_JPRB, 0.13131E+01_JPRB ,&
&  0.14553E+01_JPRB, 0.16507E+01_JPRB, 0.18868E+01_JPRB, 0.21473E+01_JPRB, 0.24438E+01_JPRB, 0.27099E+01_JPRB, 0.29622E+01_JPRB ,&
&  0.31521E+01_JPRB, 0.32633E+01_JPRB, 0.30019E+01_JPRB, 0.29778E+01_JPRB, 0.28273E+01_JPRB, 0.19878E+01_JPRB, 0.14624E+01_JPRB ,&
&  0.10547E+01_JPRB, 0.10104E+01_JPRB, 0.10524E+01_JPRB, 0.10314E+01_JPRB, 0.84049E+00_JPRB, 0.73021E+00_JPRB, 0.60242E+00_JPRB ,&
&  0.50222E+00_JPRB, 0.46314E+00_JPRB, 0.46584E+00_JPRB, 0.51471E+00_JPRB, 0.49584E+00_JPRB, 0.46362E+00_JPRB, 0.47535E+00_JPRB ,&
&  0.46787E+00_JPRB, 0.47550E+00_JPRB, 0.49786E+00_JPRB, 0.54217E+00_JPRB, 0.58856E+00_JPRB, 0.58599E+00_JPRB, 0.64009E+00_JPRB ,&
&  0.60731E+00_JPRB, 0.59804E+00_JPRB, 0.59348E+00_JPRB, 0.50823E+00_JPRB, 0.68615E+00_JPRB, 0.78243E+00_JPRB, 0.91360E+00_JPRB ,&
&  0.10504E+01_JPRB, 0.14476E+01_JPRB, 0.16983E+01_JPRB, 0.13240E+01_JPRB, 0.10122E+01_JPRB, 0.71986E+00_JPRB, 0.68667E+00_JPRB ,&
&  0.70515E+00_JPRB, 0.71112E+00_JPRB, 0.73220E+00_JPRB, 0.75906E+00_JPRB, 0.78860E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.80513E+00_JPRB, 0.83039E+00_JPRB, 0.84214E+00_JPRB, 0.84228E+00_JPRB, 0.85772E+00_JPRB, 0.88483E+00_JPRB, 0.89564E+00_JPRB ,&
&  0.90738E+00_JPRB, 0.90746E+00_JPRB, 0.92103E+00_JPRB, 0.93460E+00_JPRB, 0.94839E+00_JPRB, 0.96402E+00_JPRB, 0.98020E+00_JPRB ,&
&  0.99091E+00_JPRB, 0.10252E+01_JPRB, 0.10427E+01_JPRB, 0.10815E+01_JPRB, 0.11224E+01_JPRB, 0.11802E+01_JPRB, 0.12109E+01_JPRB ,&
&  0.12126E+01_JPRB, 0.12426E+01_JPRB, 0.12759E+01_JPRB, 0.13186E+01_JPRB, 0.13101E+01_JPRB, 0.13161E+01_JPRB, 0.10464E+01_JPRB ,&
&  0.11475E+01_JPRB, 0.11205E+01_JPRB, 0.10778E+01_JPRB, 0.10674E+01_JPRB, 0.10540E+01_JPRB, 0.10371E+01_JPRB, 0.10196E+01_JPRB ,&
&  0.10268E+01_JPRB, 0.10297E+01_JPRB, 0.10139E+01_JPRB, 0.10834E+01_JPRB, 0.11343E+01_JPRB, 0.11999E+01_JPRB, 0.13035E+01_JPRB ,&
&  0.14189E+01_JPRB, 0.15891E+01_JPRB, 0.18099E+01_JPRB, 0.20796E+01_JPRB, 0.23749E+01_JPRB, 0.26689E+01_JPRB, 0.29614E+01_JPRB ,&
&  0.32321E+01_JPRB, 0.34471E+01_JPRB, 0.34884E+01_JPRB, 0.28736E+01_JPRB, 0.24195E+01_JPRB, 0.17354E+01_JPRB, 0.13356E+01_JPRB ,&
&  0.11074E+01_JPRB, 0.10086E+01_JPRB, 0.98661E+00_JPRB, 0.93713E+00_JPRB, 0.74156E+00_JPRB, 0.62529E+00_JPRB, 0.51888E+00_JPRB ,&
&  0.47624E+00_JPRB, 0.44060E+00_JPRB, 0.43689E+00_JPRB, 0.43276E+00_JPRB, 0.42974E+00_JPRB, 0.42664E+00_JPRB, 0.43980E+00_JPRB ,&
&  0.44805E+00_JPRB, 0.47334E+00_JPRB, 0.52918E+00_JPRB, 0.59833E+00_JPRB, 0.66018E+00_JPRB, 0.69167E+00_JPRB, 0.64156E+00_JPRB ,&
&  0.65255E+00_JPRB, 0.62522E+00_JPRB, 0.61913E+00_JPRB, 0.66990E+00_JPRB, 0.75915E+00_JPRB, 0.92828E+00_JPRB, 0.11350E+01_JPRB ,&
&  0.13892E+01_JPRB, 0.14449E+01_JPRB, 0.17495E+01_JPRB, 0.23038E+01_JPRB, 0.20801E+01_JPRB, 0.17657E+01_JPRB, 0.65338E+00_JPRB ,&
&  0.68792E+00_JPRB, 0.69696E+00_JPRB, 0.72614E+00_JPRB, 0.75472E+00_JPRB, 0.79048E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.66514E+00_JPRB, 0.69135E+00_JPRB, 0.71005E+00_JPRB, 0.71202E+00_JPRB, 0.73961E+00_JPRB, 0.75999E+00_JPRB, 0.77118E+00_JPRB ,&
&  0.78182E+00_JPRB, 0.80047E+00_JPRB, 0.81061E+00_JPRB, 0.83289E+00_JPRB, 0.84834E+00_JPRB, 0.86381E+00_JPRB, 0.87517E+00_JPRB ,&
&  0.88765E+00_JPRB, 0.91261E+00_JPRB, 0.94181E+00_JPRB, 0.98540E+00_JPRB, 0.10280E+01_JPRB, 0.10823E+01_JPRB, 0.11296E+01_JPRB ,&
&  0.11630E+01_JPRB, 0.12046E+01_JPRB, 0.12540E+01_JPRB, 0.12829E+01_JPRB, 0.12661E+01_JPRB, 0.12424E+01_JPRB, 0.94276E+00_JPRB ,&
&  0.10653E+01_JPRB, 0.10803E+01_JPRB, 0.10741E+01_JPRB, 0.10563E+01_JPRB, 0.10306E+01_JPRB, 0.99704E+00_JPRB, 0.98351E+00_JPRB ,&
&  0.98089E+00_JPRB, 0.97120E+00_JPRB, 0.96221E+00_JPRB, 0.96856E+00_JPRB, 0.10988E+01_JPRB, 0.11657E+01_JPRB, 0.12543E+01_JPRB ,&
&  0.13751E+01_JPRB, 0.15232E+01_JPRB, 0.17042E+01_JPRB, 0.19672E+01_JPRB, 0.22591E+01_JPRB, 0.25472E+01_JPRB, 0.28444E+01_JPRB ,&
&  0.30735E+01_JPRB, 0.31936E+01_JPRB, 0.32327E+01_JPRB, 0.25811E+01_JPRB, 0.21409E+01_JPRB, 0.16948E+01_JPRB, 0.12443E+01_JPRB ,&
&  0.10465E+01_JPRB, 0.97583E+00_JPRB, 0.96548E+00_JPRB, 0.88078E+00_JPRB, 0.71428E+00_JPRB, 0.58236E+00_JPRB, 0.49185E+00_JPRB ,&
&  0.45477E+00_JPRB, 0.41683E+00_JPRB, 0.41203E+00_JPRB, 0.41259E+00_JPRB, 0.42217E+00_JPRB, 0.42924E+00_JPRB, 0.44180E+00_JPRB ,&
&  0.45948E+00_JPRB, 0.48951E+00_JPRB, 0.54343E+00_JPRB, 0.62632E+00_JPRB, 0.73636E+00_JPRB, 0.85675E+00_JPRB, 0.82485E+00_JPRB ,&
&  0.67702E+00_JPRB, 0.59621E+00_JPRB, 0.59200E+00_JPRB, 0.60142E+00_JPRB, 0.63342E+00_JPRB, 0.69429E+00_JPRB, 0.76927E+00_JPRB ,&
&  0.89631E+00_JPRB, 0.97087E+00_JPRB, 0.88192E+00_JPRB, 0.73871E+00_JPRB, 0.10127E+01_JPRB, 0.99589E+00_JPRB, 0.53019E+00_JPRB ,&
&  0.57747E+00_JPRB, 0.56783E+00_JPRB, 0.59075E+00_JPRB, 0.61795E+00_JPRB, 0.65167E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.51017E+00_JPRB, 0.55443E+00_JPRB, 0.57291E+00_JPRB, 0.59585E+00_JPRB, 0.61226E+00_JPRB, 0.63508E+00_JPRB, 0.65465E+00_JPRB ,&
&  0.67297E+00_JPRB, 0.69564E+00_JPRB, 0.70678E+00_JPRB, 0.72956E+00_JPRB, 0.74638E+00_JPRB, 0.75972E+00_JPRB, 0.77057E+00_JPRB ,&
&  0.78230E+00_JPRB, 0.80614E+00_JPRB, 0.83160E+00_JPRB, 0.86249E+00_JPRB, 0.89317E+00_JPRB, 0.93500E+00_JPRB, 0.99441E+00_JPRB ,&
&  0.10666E+01_JPRB, 0.11344E+01_JPRB, 0.11908E+01_JPRB, 0.12343E+01_JPRB, 0.12527E+01_JPRB, 0.12947E+01_JPRB, 0.12778E+01_JPRB ,&
&  0.93758E+00_JPRB, 0.11014E+01_JPRB, 0.10700E+01_JPRB, 0.10269E+01_JPRB, 0.99543E+00_JPRB, 0.96966E+00_JPRB, 0.96817E+00_JPRB ,&
&  0.96497E+00_JPRB, 0.91943E+00_JPRB, 0.92382E+00_JPRB, 0.94502E+00_JPRB, 0.10189E+01_JPRB, 0.10879E+01_JPRB, 0.11806E+01_JPRB ,&
&  0.13027E+01_JPRB, 0.14560E+01_JPRB, 0.16231E+01_JPRB, 0.18041E+01_JPRB, 0.20505E+01_JPRB, 0.23578E+01_JPRB, 0.26350E+01_JPRB ,&
&  0.29251E+01_JPRB, 0.32217E+01_JPRB, 0.33988E+01_JPRB, 0.28615E+01_JPRB, 0.23514E+01_JPRB, 0.20950E+01_JPRB, 0.15471E+01_JPRB ,&
&  0.10626E+01_JPRB, 0.88195E+00_JPRB, 0.88618E+00_JPRB, 0.85335E+00_JPRB, 0.69528E+00_JPRB, 0.58240E+00_JPRB, 0.50402E+00_JPRB ,&
&  0.47111E+00_JPRB, 0.43494E+00_JPRB, 0.42152E+00_JPRB, 0.42237E+00_JPRB, 0.42959E+00_JPRB, 0.43925E+00_JPRB, 0.46323E+00_JPRB ,&
&  0.47961E+00_JPRB, 0.49525E+00_JPRB, 0.51854E+00_JPRB, 0.54469E+00_JPRB, 0.59197E+00_JPRB, 0.66481E+00_JPRB, 0.67187E+00_JPRB ,&
&  0.64217E+00_JPRB, 0.60902E+00_JPRB, 0.58179E+00_JPRB, 0.58121E+00_JPRB, 0.57897E+00_JPRB, 0.58152E+00_JPRB, 0.59952E+00_JPRB ,&
&  0.62678E+00_JPRB, 0.64731E+00_JPRB, 0.61444E+00_JPRB, 0.59129E+00_JPRB, 0.59221E+00_JPRB, 0.56324E+00_JPRB, 0.50916E+00_JPRB ,&
&  0.48867E+00_JPRB, 0.47091E+00_JPRB, 0.45403E+00_JPRB, 0.46094E+00_JPRB, 0.48253E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.42781E+00_JPRB, 0.44615E+00_JPRB, 0.46665E+00_JPRB, 0.48581E+00_JPRB, 0.50315E+00_JPRB, 0.51823E+00_JPRB, 0.54239E+00_JPRB ,&
&  0.56901E+00_JPRB, 0.58671E+00_JPRB, 0.59298E+00_JPRB, 0.60901E+00_JPRB, 0.62726E+00_JPRB, 0.65052E+00_JPRB, 0.66743E+00_JPRB ,&
&  0.68733E+00_JPRB, 0.71643E+00_JPRB, 0.74141E+00_JPRB, 0.76859E+00_JPRB, 0.79564E+00_JPRB, 0.82353E+00_JPRB, 0.86428E+00_JPRB ,&
&  0.91745E+00_JPRB, 0.98724E+00_JPRB, 0.10669E+01_JPRB, 0.11394E+01_JPRB, 0.12011E+01_JPRB, 0.12666E+01_JPRB, 0.12711E+01_JPRB ,&
&  0.82325E+00_JPRB, 0.10301E+01_JPRB, 0.11518E+01_JPRB, 0.10587E+01_JPRB, 0.98098E+00_JPRB, 0.94258E+00_JPRB, 0.93390E+00_JPRB ,&
&  0.90790E+00_JPRB, 0.86593E+00_JPRB, 0.86587E+00_JPRB, 0.94330E+00_JPRB, 0.93610E+00_JPRB, 0.98315E+00_JPRB, 0.10496E+01_JPRB ,&
&  0.11475E+01_JPRB, 0.12694E+01_JPRB, 0.13830E+01_JPRB, 0.15182E+01_JPRB, 0.17138E+01_JPRB, 0.19603E+01_JPRB, 0.22661E+01_JPRB ,&
&  0.26505E+01_JPRB, 0.30896E+01_JPRB, 0.34976E+01_JPRB, 0.28082E+01_JPRB, 0.25493E+01_JPRB, 0.23972E+01_JPRB, 0.20520E+01_JPRB ,&
&  0.12479E+01_JPRB, 0.88401E+00_JPRB, 0.87237E+00_JPRB, 0.84085E+00_JPRB, 0.71630E+00_JPRB, 0.62877E+00_JPRB, 0.56375E+00_JPRB ,&
&  0.52307E+00_JPRB, 0.49078E+00_JPRB, 0.47430E+00_JPRB, 0.46688E+00_JPRB, 0.46762E+00_JPRB, 0.48374E+00_JPRB, 0.50184E+00_JPRB ,&
&  0.51710E+00_JPRB, 0.51881E+00_JPRB, 0.53924E+00_JPRB, 0.54897E+00_JPRB, 0.58209E+00_JPRB, 0.61007E+00_JPRB, 0.62658E+00_JPRB ,&
&  0.63287E+00_JPRB, 0.62526E+00_JPRB, 0.62072E+00_JPRB, 0.61773E+00_JPRB, 0.61773E+00_JPRB, 0.61690E+00_JPRB, 0.61527E+00_JPRB ,&
&  0.61285E+00_JPRB, 0.62728E+00_JPRB, 0.62066E+00_JPRB, 0.61360E+00_JPRB, 0.61961E+00_JPRB, 0.59441E+00_JPRB, 0.57973E+00_JPRB ,&
&  0.55857E+00_JPRB, 0.51140E+00_JPRB, 0.47649E+00_JPRB, 0.45048E+00_JPRB, 0.43309E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.44607E+00_JPRB, 0.42333E+00_JPRB, 0.42575E+00_JPRB, 0.42954E+00_JPRB, 0.44416E+00_JPRB, 0.45818E+00_JPRB, 0.45326E+00_JPRB ,&
&  0.45920E+00_JPRB, 0.46750E+00_JPRB, 0.47914E+00_JPRB, 0.49406E+00_JPRB, 0.51325E+00_JPRB, 0.53315E+00_JPRB, 0.54779E+00_JPRB ,&
&  0.57004E+00_JPRB, 0.59636E+00_JPRB, 0.62680E+00_JPRB, 0.65690E+00_JPRB, 0.68952E+00_JPRB, 0.71507E+00_JPRB, 0.73883E+00_JPRB ,&
&  0.76738E+00_JPRB, 0.81544E+00_JPRB, 0.88170E+00_JPRB, 0.95967E+00_JPRB, 0.10569E+01_JPRB, 0.11706E+01_JPRB, 0.12344E+01_JPRB ,&
&  0.92883E+00_JPRB, 0.55804E+00_JPRB, 0.10490E+01_JPRB, 0.11656E+01_JPRB, 0.10325E+01_JPRB, 0.97274E+00_JPRB, 0.91250E+00_JPRB ,&
&  0.87130E+00_JPRB, 0.87786E+00_JPRB, 0.85276E+00_JPRB, 0.88387E+00_JPRB, 0.87118E+00_JPRB, 0.88860E+00_JPRB, 0.91983E+00_JPRB ,&
&  0.95656E+00_JPRB, 0.10321E+01_JPRB, 0.11473E+01_JPRB, 0.12189E+01_JPRB, 0.13242E+01_JPRB, 0.15099E+01_JPRB, 0.17876E+01_JPRB ,&
&  0.21786E+01_JPRB, 0.26554E+01_JPRB, 0.31359E+01_JPRB, 0.24473E+01_JPRB, 0.19178E+01_JPRB, 0.15344E+01_JPRB, 0.12710E+01_JPRB ,&
&  0.11142E+01_JPRB, 0.90012E+00_JPRB, 0.85641E+00_JPRB, 0.82735E+00_JPRB, 0.74721E+00_JPRB, 0.64780E+00_JPRB, 0.62820E+00_JPRB ,&
&  0.58650E+00_JPRB, 0.55739E+00_JPRB, 0.54138E+00_JPRB, 0.52997E+00_JPRB, 0.52764E+00_JPRB, 0.53853E+00_JPRB, 0.54891E+00_JPRB ,&
&  0.56233E+00_JPRB, 0.56931E+00_JPRB, 0.58468E+00_JPRB, 0.60282E+00_JPRB, 0.60951E+00_JPRB, 0.61318E+00_JPRB, 0.62349E+00_JPRB ,&
&  0.62349E+00_JPRB, 0.63315E+00_JPRB, 0.64406E+00_JPRB, 0.64632E+00_JPRB, 0.64963E+00_JPRB, 0.64249E+00_JPRB, 0.64836E+00_JPRB ,&
&  0.65222E+00_JPRB, 0.65644E+00_JPRB, 0.65212E+00_JPRB, 0.65933E+00_JPRB, 0.66796E+00_JPRB, 0.66812E+00_JPRB, 0.68576E+00_JPRB ,&
&  0.67857E+00_JPRB, 0.65022E+00_JPRB, 0.58882E+00_JPRB, 0.51904E+00_JPRB, 0.48416E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.53522E+00_JPRB, 0.49914E+00_JPRB, 0.46707E+00_JPRB, 0.45019E+00_JPRB, 0.43867E+00_JPRB, 0.43392E+00_JPRB, 0.43245E+00_JPRB ,&
&  0.43584E+00_JPRB, 0.43986E+00_JPRB, 0.42731E+00_JPRB, 0.43148E+00_JPRB, 0.44371E+00_JPRB, 0.45766E+00_JPRB, 0.47602E+00_JPRB ,&
&  0.48681E+00_JPRB, 0.49541E+00_JPRB, 0.50585E+00_JPRB, 0.52516E+00_JPRB, 0.55125E+00_JPRB, 0.58028E+00_JPRB, 0.60789E+00_JPRB ,&
&  0.63532E+00_JPRB, 0.66472E+00_JPRB, 0.70496E+00_JPRB, 0.75925E+00_JPRB, 0.83555E+00_JPRB, 0.94156E+00_JPRB, 0.10536E+01_JPRB ,&
&  0.11861E+01_JPRB, 0.89927E+00_JPRB, 0.66869E+00_JPRB, 0.10498E+01_JPRB, 0.11189E+01_JPRB, 0.10635E+01_JPRB, 0.93052E+00_JPRB ,&
&  0.92367E+00_JPRB, 0.93094E+00_JPRB, 0.88991E+00_JPRB, 0.86702E+00_JPRB, 0.81103E+00_JPRB, 0.81192E+00_JPRB, 0.81979E+00_JPRB ,&
&  0.81727E+00_JPRB, 0.82908E+00_JPRB, 0.90254E+00_JPRB, 0.95686E+00_JPRB, 0.10015E+01_JPRB, 0.11206E+01_JPRB, 0.13672E+01_JPRB ,&
&  0.17065E+01_JPRB, 0.21509E+01_JPRB, 0.26558E+01_JPRB, 0.22444E+01_JPRB, 0.16914E+01_JPRB, 0.12840E+01_JPRB, 0.11319E+01_JPRB ,&
&  0.10443E+01_JPRB, 0.99215E+00_JPRB, 0.91433E+00_JPRB, 0.83376E+00_JPRB, 0.75323E+00_JPRB, 0.68076E+00_JPRB, 0.66587E+00_JPRB ,&
&  0.64203E+00_JPRB, 0.60598E+00_JPRB, 0.58183E+00_JPRB, 0.56084E+00_JPRB, 0.55798E+00_JPRB, 0.56724E+00_JPRB, 0.57632E+00_JPRB ,&
&  0.58461E+00_JPRB, 0.59406E+00_JPRB, 0.61108E+00_JPRB, 0.63212E+00_JPRB, 0.63776E+00_JPRB, 0.63344E+00_JPRB, 0.63239E+00_JPRB ,&
&  0.63033E+00_JPRB, 0.63178E+00_JPRB, 0.64696E+00_JPRB, 0.65326E+00_JPRB, 0.65178E+00_JPRB, 0.64011E+00_JPRB, 0.65103E+00_JPRB ,&
&  0.65295E+00_JPRB, 0.66316E+00_JPRB, 0.66188E+00_JPRB, 0.67367E+00_JPRB, 0.68082E+00_JPRB, 0.70528E+00_JPRB, 0.73151E+00_JPRB ,&
&  0.75422E+00_JPRB, 0.75152E+00_JPRB, 0.71901E+00_JPRB, 0.60974E+00_JPRB, 0.56837E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.57278E+00_JPRB, 0.54909E+00_JPRB, 0.53126E+00_JPRB, 0.51396E+00_JPRB, 0.49576E+00_JPRB, 0.47691E+00_JPRB, 0.46212E+00_JPRB ,&
&  0.44468E+00_JPRB, 0.44202E+00_JPRB, 0.43772E+00_JPRB, 0.43233E+00_JPRB, 0.42602E+00_JPRB, 0.42711E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.44081E+00_JPRB, 0.44384E+00_JPRB, 0.44743E+00_JPRB, 0.44883E+00_JPRB, 0.45253E+00_JPRB, 0.46306E+00_JPRB, 0.48066E+00_JPRB ,&
&  0.50588E+00_JPRB, 0.53168E+00_JPRB, 0.56389E+00_JPRB, 0.60094E+00_JPRB, 0.63730E+00_JPRB, 0.69340E+00_JPRB, 0.76547E+00_JPRB ,&
&  0.91791E+00_JPRB, 0.92302E+00_JPRB, 0.59190E+00_JPRB, 0.95440E+00_JPRB, 0.11863E+01_JPRB, 0.11264E+01_JPRB, 0.10597E+01_JPRB ,&
&  0.10475E+01_JPRB, 0.98819E+00_JPRB, 0.94588E+00_JPRB, 0.86325E+00_JPRB, 0.77445E+00_JPRB, 0.75581E+00_JPRB, 0.75492E+00_JPRB ,&
&  0.73182E+00_JPRB, 0.71624E+00_JPRB, 0.72963E+00_JPRB, 0.74839E+00_JPRB, 0.76759E+00_JPRB, 0.81849E+00_JPRB, 0.98907E+00_JPRB ,&
&  0.12663E+01_JPRB, 0.16874E+01_JPRB, 0.21635E+01_JPRB, 0.20536E+01_JPRB, 0.15571E+01_JPRB, 0.13279E+01_JPRB, 0.12216E+01_JPRB ,&
&  0.11880E+01_JPRB, 0.11205E+01_JPRB, 0.95030E+00_JPRB, 0.83753E+00_JPRB, 0.75247E+00_JPRB, 0.69021E+00_JPRB, 0.68858E+00_JPRB ,&
&  0.66961E+00_JPRB, 0.63779E+00_JPRB, 0.61848E+00_JPRB, 0.59147E+00_JPRB, 0.57899E+00_JPRB, 0.57494E+00_JPRB, 0.58141E+00_JPRB ,&
&  0.58987E+00_JPRB, 0.59717E+00_JPRB, 0.60817E+00_JPRB, 0.62254E+00_JPRB, 0.63142E+00_JPRB, 0.63527E+00_JPRB, 0.63318E+00_JPRB ,&
&  0.63280E+00_JPRB, 0.63801E+00_JPRB, 0.64834E+00_JPRB, 0.65305E+00_JPRB, 0.65103E+00_JPRB, 0.64673E+00_JPRB, 0.63893E+00_JPRB ,&
&  0.63555E+00_JPRB, 0.63582E+00_JPRB, 0.63044E+00_JPRB, 0.62576E+00_JPRB, 0.65562E+00_JPRB, 0.66630E+00_JPRB, 0.67392E+00_JPRB ,&
&  0.71478E+00_JPRB, 0.74917E+00_JPRB, 0.75102E+00_JPRB, 0.64235E+00_JPRB, 0.60202E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.57865E+00_JPRB, 0.56007E+00_JPRB, 0.54545E+00_JPRB, 0.53422E+00_JPRB, 0.52009E+00_JPRB, 0.50570E+00_JPRB, 0.49414E+00_JPRB ,&
&  0.47735E+00_JPRB, 0.46695E+00_JPRB, 0.45945E+00_JPRB, 0.45153E+00_JPRB, 0.44389E+00_JPRB, 0.43676E+00_JPRB, 0.43087E+00_JPRB ,&
&  0.42613E+00_JPRB, 0.42246E+00_JPRB, 0.42012E+00_JPRB, 0.41503E+00_JPRB, 0.41005E+00_JPRB, 0.40826E+00_JPRB, 0.41159E+00_JPRB ,&
&  0.42708E+00_JPRB, 0.44888E+00_JPRB, 0.47354E+00_JPRB, 0.49780E+00_JPRB, 0.51788E+00_JPRB, 0.54504E+00_JPRB, 0.57881E+00_JPRB ,&
&  0.71018E+00_JPRB, 0.77143E+00_JPRB, 0.92570E+00_JPRB, 0.14263E+01_JPRB, 0.14503E+01_JPRB, 0.12957E+01_JPRB, 0.12639E+01_JPRB ,&
&  0.12388E+01_JPRB, 0.11472E+01_JPRB, 0.11000E+01_JPRB, 0.93328E+00_JPRB, 0.79928E+00_JPRB, 0.73342E+00_JPRB, 0.69993E+00_JPRB ,&
&  0.66370E+00_JPRB, 0.63367E+00_JPRB, 0.63179E+00_JPRB, 0.62735E+00_JPRB, 0.63376E+00_JPRB, 0.69708E+00_JPRB, 0.78530E+00_JPRB ,&
&  0.96516E+00_JPRB, 0.12717E+01_JPRB, 0.17049E+01_JPRB, 0.19862E+01_JPRB, 0.15586E+01_JPRB, 0.14500E+01_JPRB, 0.14758E+01_JPRB ,&
&  0.17682E+01_JPRB, 0.14110E+01_JPRB, 0.10596E+01_JPRB, 0.86353E+00_JPRB, 0.75156E+00_JPRB, 0.71396E+00_JPRB, 0.67756E+00_JPRB ,&
&  0.66039E+00_JPRB, 0.64227E+00_JPRB, 0.62799E+00_JPRB, 0.61001E+00_JPRB, 0.59818E+00_JPRB, 0.58812E+00_JPRB, 0.58454E+00_JPRB ,&
&  0.58661E+00_JPRB, 0.59038E+00_JPRB, 0.59426E+00_JPRB, 0.60203E+00_JPRB, 0.61503E+00_JPRB, 0.62480E+00_JPRB, 0.62798E+00_JPRB ,&
&  0.62919E+00_JPRB, 0.63617E+00_JPRB, 0.62421E+00_JPRB, 0.62675E+00_JPRB, 0.63436E+00_JPRB, 0.63181E+00_JPRB, 0.60952E+00_JPRB ,&
&  0.59994E+00_JPRB, 0.61106E+00_JPRB, 0.61373E+00_JPRB, 0.60951E+00_JPRB, 0.61942E+00_JPRB, 0.63009E+00_JPRB, 0.61027E+00_JPRB ,&
&  0.61432E+00_JPRB, 0.62169E+00_JPRB, 0.63004E+00_JPRB, 0.61520E+00_JPRB, 0.59865E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.58183E+00_JPRB, 0.56718E+00_JPRB, 0.55164E+00_JPRB, 0.53579E+00_JPRB, 0.51840E+00_JPRB, 0.50542E+00_JPRB, 0.49859E+00_JPRB ,&
&  0.48727E+00_JPRB, 0.47728E+00_JPRB, 0.46996E+00_JPRB, 0.46673E+00_JPRB, 0.46425E+00_JPRB, 0.45616E+00_JPRB, 0.44074E+00_JPRB ,&
&  0.42514E+00_JPRB, 0.41503E+00_JPRB, 0.40980E+00_JPRB, 0.40426E+00_JPRB, 0.39776E+00_JPRB, 0.39386E+00_JPRB, 0.39452E+00_JPRB ,&
&  0.40065E+00_JPRB, 0.41068E+00_JPRB, 0.43044E+00_JPRB, 0.45194E+00_JPRB, 0.46784E+00_JPRB, 0.48095E+00_JPRB, 0.48961E+00_JPRB ,&
&  0.56713E+00_JPRB, 0.68601E+00_JPRB, 0.75882E+00_JPRB, 0.12310E+01_JPRB, 0.13027E+01_JPRB, 0.12672E+01_JPRB, 0.12564E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.13105E+01_JPRB, 0.11614E+01_JPRB, 0.10041E+01_JPRB, 0.86207E+00_JPRB, 0.76407E+00_JPRB, 0.69499E+00_JPRB ,&
&  0.64710E+00_JPRB, 0.60683E+00_JPRB, 0.58532E+00_JPRB, 0.57505E+00_JPRB, 0.57289E+00_JPRB, 0.60502E+00_JPRB, 0.68952E+00_JPRB ,&
&  0.81730E+00_JPRB, 0.10351E+01_JPRB, 0.13498E+01_JPRB, 0.15663E+01_JPRB, 0.15046E+01_JPRB, 0.15308E+01_JPRB, 0.17137E+01_JPRB ,&
&  0.21822E+01_JPRB, 0.18980E+01_JPRB, 0.12667E+01_JPRB, 0.92800E+00_JPRB, 0.77631E+00_JPRB, 0.71424E+00_JPRB, 0.66877E+00_JPRB ,&
&  0.64799E+00_JPRB, 0.63144E+00_JPRB, 0.61867E+00_JPRB, 0.60778E+00_JPRB, 0.59888E+00_JPRB, 0.59631E+00_JPRB, 0.59344E+00_JPRB ,&
&  0.59051E+00_JPRB, 0.58927E+00_JPRB, 0.58811E+00_JPRB, 0.58800E+00_JPRB, 0.59267E+00_JPRB, 0.60235E+00_JPRB, 0.61289E+00_JPRB ,&
&  0.61957E+00_JPRB, 0.62701E+00_JPRB, 0.61516E+00_JPRB, 0.60491E+00_JPRB, 0.60828E+00_JPRB, 0.60511E+00_JPRB, 0.58893E+00_JPRB ,&
&  0.59172E+00_JPRB, 0.60180E+00_JPRB, 0.60127E+00_JPRB, 0.58796E+00_JPRB, 0.59601E+00_JPRB, 0.61680E+00_JPRB, 0.59334E+00_JPRB ,&
&  0.58931E+00_JPRB, 0.59306E+00_JPRB, 0.60016E+00_JPRB, 0.59539E+00_JPRB, 0.58913E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.57214E+00_JPRB, 0.56470E+00_JPRB, 0.55070E+00_JPRB, 0.53413E+00_JPRB, 0.51726E+00_JPRB, 0.50302E+00_JPRB, 0.49598E+00_JPRB ,&
&  0.48774E+00_JPRB, 0.47922E+00_JPRB, 0.47151E+00_JPRB, 0.46745E+00_JPRB, 0.46590E+00_JPRB, 0.45830E+00_JPRB, 0.44429E+00_JPRB ,&
&  0.42945E+00_JPRB, 0.41579E+00_JPRB, 0.40486E+00_JPRB, 0.39911E+00_JPRB, 0.39273E+00_JPRB, 0.38946E+00_JPRB, 0.38903E+00_JPRB ,&
&  0.39008E+00_JPRB, 0.39596E+00_JPRB, 0.40875E+00_JPRB, 0.42975E+00_JPRB, 0.44641E+00_JPRB, 0.45388E+00_JPRB, 0.45908E+00_JPRB ,&
&  0.50473E+00_JPRB, 0.73371E+00_JPRB, 0.69556E+00_JPRB, 0.96249E+00_JPRB, 0.10284E+01_JPRB, 0.10738E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.11577E+01_JPRB, 0.11882E+01_JPRB, 0.11265E+01_JPRB, 0.10231E+01_JPRB, 0.90025E+00_JPRB, 0.80941E+00_JPRB, 0.73430E+00_JPRB ,&
&  0.67970E+00_JPRB, 0.63924E+00_JPRB, 0.60754E+00_JPRB, 0.58142E+00_JPRB, 0.56466E+00_JPRB, 0.56901E+00_JPRB, 0.63408E+00_JPRB ,&
&  0.78158E+00_JPRB, 0.96348E+00_JPRB, 0.11684E+01_JPRB, 0.13936E+01_JPRB, 0.13390E+01_JPRB, 0.15115E+01_JPRB, 0.20125E+01_JPRB ,&
&  0.24034E+01_JPRB, 0.24047E+01_JPRB, 0.16107E+01_JPRB, 0.10710E+01_JPRB, 0.83949E+00_JPRB, 0.72956E+00_JPRB, 0.66396E+00_JPRB ,&
&  0.62711E+00_JPRB, 0.60782E+00_JPRB, 0.59728E+00_JPRB, 0.59199E+00_JPRB, 0.59124E+00_JPRB, 0.59529E+00_JPRB, 0.59516E+00_JPRB ,&
&  0.59527E+00_JPRB, 0.59454E+00_JPRB, 0.59379E+00_JPRB, 0.59057E+00_JPRB, 0.58507E+00_JPRB, 0.58516E+00_JPRB, 0.59127E+00_JPRB ,&
&  0.60124E+00_JPRB, 0.60995E+00_JPRB, 0.60404E+00_JPRB, 0.59654E+00_JPRB, 0.59464E+00_JPRB, 0.59119E+00_JPRB, 0.58191E+00_JPRB ,&
&  0.58828E+00_JPRB, 0.59149E+00_JPRB, 0.58996E+00_JPRB, 0.58959E+00_JPRB, 0.58581E+00_JPRB, 0.60992E+00_JPRB, 0.58388E+00_JPRB ,&
&  0.57726E+00_JPRB, 0.57984E+00_JPRB, 0.58271E+00_JPRB, 0.57671E+00_JPRB, 0.57396E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.55498E+00_JPRB, 0.54838E+00_JPRB, 0.53476E+00_JPRB, 0.51933E+00_JPRB, 0.50813E+00_JPRB, 0.49622E+00_JPRB, 0.48715E+00_JPRB ,&
&  0.48020E+00_JPRB, 0.47442E+00_JPRB, 0.47013E+00_JPRB, 0.46495E+00_JPRB, 0.45788E+00_JPRB, 0.44803E+00_JPRB, 0.43828E+00_JPRB ,&
&  0.42798E+00_JPRB, 0.41793E+00_JPRB, 0.40802E+00_JPRB, 0.39777E+00_JPRB, 0.38990E+00_JPRB, 0.38443E+00_JPRB, 0.38065E+00_JPRB ,&
&  0.38016E+00_JPRB, 0.38295E+00_JPRB, 0.39085E+00_JPRB, 0.40767E+00_JPRB, 0.42702E+00_JPRB, 0.43849E+00_JPRB, 0.45028E+00_JPRB ,&
&  0.50422E+00_JPRB, 0.44906E+00_JPRB, 0.75726E+00_JPRB, 0.83405E+00_JPRB, 0.81290E+00_JPRB, 0.82750E+00_JPRB, 0.95020E+00_JPRB ,&
&  0.10403E+01_JPRB, 0.10690E+01_JPRB, 0.10678E+01_JPRB, 0.10192E+01_JPRB, 0.92439E+00_JPRB, 0.85239E+00_JPRB, 0.79637E+00_JPRB ,&
&  0.75400E+00_JPRB, 0.71882E+00_JPRB, 0.67652E+00_JPRB, 0.63343E+00_JPRB, 0.60110E+00_JPRB, 0.59491E+00_JPRB, 0.63040E+00_JPRB ,&
&  0.78889E+00_JPRB, 0.93556E+00_JPRB, 0.10504E+01_JPRB, 0.11947E+01_JPRB, 0.12320E+01_JPRB, 0.14461E+01_JPRB, 0.17212E+01_JPRB ,&
&  0.23563E+01_JPRB, 0.24442E+01_JPRB, 0.18303E+01_JPRB, 0.13118E+01_JPRB, 0.98875E+00_JPRB, 0.81756E+00_JPRB, 0.70139E+00_JPRB ,&
&  0.63833E+00_JPRB, 0.60216E+00_JPRB, 0.58034E+00_JPRB, 0.57434E+00_JPRB, 0.58186E+00_JPRB, 0.59602E+00_JPRB, 0.60771E+00_JPRB ,&
&  0.62233E+00_JPRB, 0.63298E+00_JPRB, 0.63382E+00_JPRB, 0.62816E+00_JPRB, 0.61026E+00_JPRB, 0.58786E+00_JPRB, 0.57547E+00_JPRB ,&
&  0.57591E+00_JPRB, 0.58416E+00_JPRB, 0.58507E+00_JPRB, 0.58087E+00_JPRB, 0.58144E+00_JPRB, 0.58127E+00_JPRB, 0.57918E+00_JPRB ,&
&  0.58043E+00_JPRB, 0.58326E+00_JPRB, 0.58247E+00_JPRB, 0.58776E+00_JPRB, 0.57392E+00_JPRB, 0.59951E+00_JPRB, 0.58284E+00_JPRB ,&
&  0.57359E+00_JPRB, 0.56970E+00_JPRB, 0.56747E+00_JPRB, 0.56077E+00_JPRB, 0.55521E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.53748E+00_JPRB, 0.53003E+00_JPRB, 0.51568E+00_JPRB, 0.50022E+00_JPRB, 0.49005E+00_JPRB, 0.48230E+00_JPRB, 0.47678E+00_JPRB ,&
&  0.47354E+00_JPRB, 0.46666E+00_JPRB, 0.46022E+00_JPRB, 0.45114E+00_JPRB, 0.43958E+00_JPRB, 0.42980E+00_JPRB, 0.42382E+00_JPRB ,&
&  0.41615E+00_JPRB, 0.41226E+00_JPRB, 0.40763E+00_JPRB, 0.39828E+00_JPRB, 0.38802E+00_JPRB, 0.37949E+00_JPRB, 0.37397E+00_JPRB ,&
&  0.37149E+00_JPRB, 0.37381E+00_JPRB, 0.37954E+00_JPRB, 0.39251E+00_JPRB, 0.40451E+00_JPRB, 0.41682E+00_JPRB, 0.43684E+00_JPRB ,&
&  0.50451E+00_JPRB, 0.69082E+00_JPRB, 0.82934E+00_JPRB, 0.75065E+00_JPRB, 0.70236E+00_JPRB, 0.67022E+00_JPRB, 0.75227E+00_JPRB ,&
&  0.82799E+00_JPRB, 0.90632E+00_JPRB, 0.94959E+00_JPRB, 0.96190E+00_JPRB, 0.91375E+00_JPRB, 0.87659E+00_JPRB, 0.83867E+00_JPRB ,&
&  0.79869E+00_JPRB, 0.75469E+00_JPRB, 0.69759E+00_JPRB, 0.64079E+00_JPRB, 0.60958E+00_JPRB, 0.61295E+00_JPRB, 0.66049E+00_JPRB ,&
&  0.77916E+00_JPRB, 0.89901E+00_JPRB, 0.95656E+00_JPRB, 0.10765E+01_JPRB, 0.12644E+01_JPRB, 0.12923E+01_JPRB, 0.16982E+01_JPRB ,&
&  0.22076E+01_JPRB, 0.19966E+01_JPRB, 0.16789E+01_JPRB, 0.13473E+01_JPRB, 0.10858E+01_JPRB, 0.87510E+00_JPRB, 0.75386E+00_JPRB ,&
&  0.67302E+00_JPRB, 0.62060E+00_JPRB, 0.58419E+00_JPRB, 0.56609E+00_JPRB, 0.57637E+00_JPRB, 0.60625E+00_JPRB, 0.63153E+00_JPRB ,&
&  0.66590E+00_JPRB, 0.70247E+00_JPRB, 0.72277E+00_JPRB, 0.72034E+00_JPRB, 0.69343E+00_JPRB, 0.65133E+00_JPRB, 0.61089E+00_JPRB ,&
&  0.58574E+00_JPRB, 0.57291E+00_JPRB, 0.57138E+00_JPRB, 0.56554E+00_JPRB, 0.57013E+00_JPRB, 0.58081E+00_JPRB, 0.57979E+00_JPRB ,&
&  0.57549E+00_JPRB, 0.57725E+00_JPRB, 0.58829E+00_JPRB, 0.58228E+00_JPRB, 0.57163E+00_JPRB, 0.60052E+00_JPRB, 0.58546E+00_JPRB ,&
&  0.57208E+00_JPRB, 0.56053E+00_JPRB, 0.55439E+00_JPRB, 0.54056E+00_JPRB, 0.53689E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.53148E+00_JPRB, 0.51993E+00_JPRB, 0.49868E+00_JPRB, 0.47949E+00_JPRB, 0.46776E+00_JPRB, 0.45976E+00_JPRB, 0.45601E+00_JPRB ,&
&  0.45434E+00_JPRB, 0.44753E+00_JPRB, 0.43999E+00_JPRB, 0.42872E+00_JPRB, 0.41876E+00_JPRB, 0.41013E+00_JPRB, 0.40239E+00_JPRB ,&
&  0.39600E+00_JPRB, 0.39248E+00_JPRB, 0.38928E+00_JPRB, 0.38573E+00_JPRB, 0.38008E+00_JPRB, 0.37182E+00_JPRB, 0.36661E+00_JPRB ,&
&  0.36417E+00_JPRB, 0.36472E+00_JPRB, 0.36870E+00_JPRB, 0.37726E+00_JPRB, 0.38660E+00_JPRB, 0.39801E+00_JPRB, 0.41621E+00_JPRB ,&
&  0.46253E+00_JPRB, 0.59330E+00_JPRB, 0.62353E+00_JPRB, 0.62115E+00_JPRB, 0.61809E+00_JPRB, 0.61264E+00_JPRB, 0.64514E+00_JPRB ,&
&  0.70460E+00_JPRB, 0.75941E+00_JPRB, 0.80307E+00_JPRB, 0.81653E+00_JPRB, 0.79138E+00_JPRB, 0.76457E+00_JPRB, 0.73073E+00_JPRB ,&
&  0.69614E+00_JPRB, 0.65921E+00_JPRB, 0.62336E+00_JPRB, 0.59130E+00_JPRB, 0.56880E+00_JPRB, 0.58013E+00_JPRB, 0.61747E+00_JPRB ,&
&  0.68971E+00_JPRB, 0.75444E+00_JPRB, 0.80106E+00_JPRB, 0.89877E+00_JPRB, 0.11056E+01_JPRB, 0.12367E+01_JPRB, 0.14147E+01_JPRB ,&
&  0.14585E+01_JPRB, 0.14167E+01_JPRB, 0.13414E+01_JPRB, 0.12336E+01_JPRB, 0.10744E+01_JPRB, 0.91447E+00_JPRB, 0.78502E+00_JPRB ,&
&  0.69179E+00_JPRB, 0.63042E+00_JPRB, 0.59276E+00_JPRB, 0.57667E+00_JPRB, 0.58488E+00_JPRB, 0.61442E+00_JPRB, 0.65279E+00_JPRB ,&
&  0.69998E+00_JPRB, 0.74839E+00_JPRB, 0.77971E+00_JPRB, 0.79248E+00_JPRB, 0.78132E+00_JPRB, 0.75172E+00_JPRB, 0.70546E+00_JPRB ,&
&  0.66250E+00_JPRB, 0.62659E+00_JPRB, 0.60314E+00_JPRB, 0.59438E+00_JPRB, 0.59430E+00_JPRB, 0.59552E+00_JPRB, 0.59604E+00_JPRB ,&
&  0.60109E+00_JPRB, 0.60325E+00_JPRB, 0.60035E+00_JPRB, 0.58721E+00_JPRB, 0.57099E+00_JPRB, 0.58782E+00_JPRB, 0.57489E+00_JPRB ,&
&  0.56211E+00_JPRB, 0.54766E+00_JPRB, 0.53698E+00_JPRB, 0.52089E+00_JPRB, 0.53058E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.56652E+00_JPRB, 0.53941E+00_JPRB, 0.50318E+00_JPRB, 0.47511E+00_JPRB, 0.45560E+00_JPRB, 0.44135E+00_JPRB, 0.43348E+00_JPRB ,&
&  0.43092E+00_JPRB, 0.42190E+00_JPRB, 0.40850E+00_JPRB, 0.39715E+00_JPRB, 0.39134E+00_JPRB, 0.38678E+00_JPRB, 0.38076E+00_JPRB ,&
&  0.37586E+00_JPRB, 0.37147E+00_JPRB, 0.36843E+00_JPRB, 0.36719E+00_JPRB, 0.36649E+00_JPRB, 0.36514E+00_JPRB, 0.36127E+00_JPRB ,&
&  0.35552E+00_JPRB, 0.35416E+00_JPRB, 0.35551E+00_JPRB, 0.36202E+00_JPRB, 0.37097E+00_JPRB, 0.38178E+00_JPRB, 0.40077E+00_JPRB ,&
&  0.45629E+00_JPRB, 0.45513E+00_JPRB, 0.49004E+00_JPRB, 0.49956E+00_JPRB, 0.50916E+00_JPRB, 0.54066E+00_JPRB, 0.57140E+00_JPRB ,&
&  0.60306E+00_JPRB, 0.64414E+00_JPRB, 0.68890E+00_JPRB, 0.70567E+00_JPRB, 0.68601E+00_JPRB, 0.65190E+00_JPRB, 0.62133E+00_JPRB ,&
&  0.59324E+00_JPRB, 0.57390E+00_JPRB, 0.55641E+00_JPRB, 0.54235E+00_JPRB, 0.54050E+00_JPRB, 0.55403E+00_JPRB, 0.57676E+00_JPRB ,&
&  0.59438E+00_JPRB, 0.60109E+00_JPRB, 0.66218E+00_JPRB, 0.73960E+00_JPRB, 0.85290E+00_JPRB, 0.93712E+00_JPRB, 0.10216E+01_JPRB ,&
&  0.10932E+01_JPRB, 0.11350E+01_JPRB, 0.11583E+01_JPRB, 0.11430E+01_JPRB, 0.10681E+01_JPRB, 0.97097E+00_JPRB, 0.86561E+00_JPRB ,&
&  0.76614E+00_JPRB, 0.69836E+00_JPRB, 0.66245E+00_JPRB, 0.64560E+00_JPRB, 0.64364E+00_JPRB, 0.65067E+00_JPRB, 0.66828E+00_JPRB ,&
&  0.69564E+00_JPRB, 0.72207E+00_JPRB, 0.74357E+00_JPRB, 0.75339E+00_JPRB, 0.75035E+00_JPRB, 0.74504E+00_JPRB, 0.73630E+00_JPRB ,&
&  0.72445E+00_JPRB, 0.70725E+00_JPRB, 0.68451E+00_JPRB, 0.66050E+00_JPRB, 0.63972E+00_JPRB, 0.62570E+00_JPRB, 0.61516E+00_JPRB ,&
&  0.60509E+00_JPRB, 0.60032E+00_JPRB, 0.58044E+00_JPRB, 0.57096E+00_JPRB, 0.58565E+00_JPRB, 0.57386E+00_JPRB, 0.55917E+00_JPRB ,&
&  0.54061E+00_JPRB, 0.52496E+00_JPRB, 0.51054E+00_JPRB, 0.49443E+00_JPRB, 0.54972E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.47900E+00_JPRB, 0.47827E+00_JPRB, 0.46515E+00_JPRB, 0.44662E+00_JPRB, 0.43269E+00_JPRB, 0.41990E+00_JPRB, 0.40903E+00_JPRB ,&
&  0.40188E+00_JPRB, 0.39330E+00_JPRB, 0.38290E+00_JPRB, 0.37501E+00_JPRB, 0.37367E+00_JPRB, 0.37116E+00_JPRB, 0.36735E+00_JPRB ,&
&  0.36044E+00_JPRB, 0.35384E+00_JPRB, 0.34933E+00_JPRB, 0.34778E+00_JPRB, 0.34841E+00_JPRB, 0.35028E+00_JPRB, 0.35116E+00_JPRB ,&
&  0.34917E+00_JPRB, 0.34521E+00_JPRB, 0.34504E+00_JPRB, 0.34732E+00_JPRB, 0.35250E+00_JPRB, 0.36076E+00_JPRB, 0.37990E+00_JPRB ,&
&  0.42532E+00_JPRB, 0.40473E+00_JPRB, 0.42350E+00_JPRB, 0.43507E+00_JPRB, 0.44936E+00_JPRB, 0.48106E+00_JPRB, 0.51343E+00_JPRB ,&
&  0.53441E+00_JPRB, 0.54941E+00_JPRB, 0.57793E+00_JPRB, 0.60178E+00_JPRB, 0.59864E+00_JPRB, 0.57886E+00_JPRB, 0.55279E+00_JPRB ,&
&  0.52664E+00_JPRB, 0.50725E+00_JPRB, 0.49157E+00_JPRB, 0.48091E+00_JPRB, 0.48041E+00_JPRB, 0.48832E+00_JPRB, 0.49529E+00_JPRB ,&
&  0.48836E+00_JPRB, 0.49425E+00_JPRB, 0.53344E+00_JPRB, 0.59219E+00_JPRB, 0.66933E+00_JPRB, 0.74718E+00_JPRB, 0.82683E+00_JPRB ,&
&  0.89866E+00_JPRB, 0.94879E+00_JPRB, 0.98680E+00_JPRB, 0.10135E+01_JPRB, 0.10067E+01_JPRB, 0.98608E+00_JPRB, 0.92534E+00_JPRB ,&
&  0.85143E+00_JPRB, 0.78899E+00_JPRB, 0.74687E+00_JPRB, 0.71359E+00_JPRB, 0.68876E+00_JPRB, 0.67243E+00_JPRB, 0.66377E+00_JPRB ,&
&  0.66274E+00_JPRB, 0.66572E+00_JPRB, 0.66553E+00_JPRB, 0.66180E+00_JPRB, 0.65730E+00_JPRB, 0.65467E+00_JPRB, 0.65186E+00_JPRB ,&
&  0.64986E+00_JPRB, 0.64778E+00_JPRB, 0.64117E+00_JPRB, 0.62758E+00_JPRB, 0.60852E+00_JPRB, 0.58985E+00_JPRB, 0.57253E+00_JPRB ,&
&  0.55910E+00_JPRB, 0.55534E+00_JPRB, 0.54856E+00_JPRB, 0.54925E+00_JPRB, 0.53785E+00_JPRB, 0.52708E+00_JPRB, 0.51650E+00_JPRB ,&
&  0.50434E+00_JPRB, 0.49168E+00_JPRB, 0.47477E+00_JPRB, 0.45807E+00_JPRB, 0.45757E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.42608E+00_JPRB, 0.43293E+00_JPRB, 0.43304E+00_JPRB, 0.42176E+00_JPRB, 0.40889E+00_JPRB, 0.40127E+00_JPRB, 0.39201E+00_JPRB ,&
&  0.38170E+00_JPRB, 0.37317E+00_JPRB, 0.36282E+00_JPRB, 0.35863E+00_JPRB, 0.35625E+00_JPRB, 0.35527E+00_JPRB, 0.35347E+00_JPRB ,&
&  0.34898E+00_JPRB, 0.34194E+00_JPRB, 0.33646E+00_JPRB, 0.33411E+00_JPRB, 0.33470E+00_JPRB, 0.33625E+00_JPRB, 0.33791E+00_JPRB ,&
&  0.33699E+00_JPRB, 0.33626E+00_JPRB, 0.33523E+00_JPRB, 0.33493E+00_JPRB, 0.33644E+00_JPRB, 0.34414E+00_JPRB, 0.35949E+00_JPRB ,&
&  0.39857E+00_JPRB, 0.36441E+00_JPRB, 0.37483E+00_JPRB, 0.39635E+00_JPRB, 0.41816E+00_JPRB, 0.44760E+00_JPRB, 0.47607E+00_JPRB ,&
&  0.48939E+00_JPRB, 0.48760E+00_JPRB, 0.49511E+00_JPRB, 0.51133E+00_JPRB, 0.51024E+00_JPRB, 0.50361E+00_JPRB, 0.49160E+00_JPRB ,&
&  0.47351E+00_JPRB, 0.45270E+00_JPRB, 0.43241E+00_JPRB, 0.42046E+00_JPRB, 0.42053E+00_JPRB, 0.42585E+00_JPRB, 0.42743E+00_JPRB ,&
&  0.41562E+00_JPRB, 0.41146E+00_JPRB, 0.42827E+00_JPRB, 0.45478E+00_JPRB, 0.49061E+00_JPRB, 0.54697E+00_JPRB, 0.61898E+00_JPRB ,&
&  0.67665E+00_JPRB, 0.73536E+00_JPRB, 0.79497E+00_JPRB, 0.84284E+00_JPRB, 0.88504E+00_JPRB, 0.89790E+00_JPRB, 0.87275E+00_JPRB ,&
&  0.83261E+00_JPRB, 0.78451E+00_JPRB, 0.73894E+00_JPRB, 0.69685E+00_JPRB, 0.66197E+00_JPRB, 0.63971E+00_JPRB, 0.62533E+00_JPRB ,&
&  0.61507E+00_JPRB, 0.60690E+00_JPRB, 0.59434E+00_JPRB, 0.57722E+00_JPRB, 0.55995E+00_JPRB, 0.54562E+00_JPRB, 0.53308E+00_JPRB ,&
&  0.52207E+00_JPRB, 0.51090E+00_JPRB, 0.50249E+00_JPRB, 0.49734E+00_JPRB, 0.49221E+00_JPRB, 0.48845E+00_JPRB, 0.48584E+00_JPRB ,&
&  0.48287E+00_JPRB, 0.48447E+00_JPRB, 0.48236E+00_JPRB, 0.47957E+00_JPRB, 0.48501E+00_JPRB, 0.47949E+00_JPRB, 0.47513E+00_JPRB ,&
&  0.46586E+00_JPRB, 0.45532E+00_JPRB, 0.44406E+00_JPRB, 0.40415E+00_JPRB, 0.42004E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.39056E+00_JPRB, 0.39631E+00_JPRB, 0.39701E+00_JPRB, 0.39215E+00_JPRB, 0.38389E+00_JPRB, 0.37815E+00_JPRB, 0.36914E+00_JPRB ,&
&  0.35918E+00_JPRB, 0.35352E+00_JPRB, 0.34692E+00_JPRB, 0.34417E+00_JPRB, 0.34427E+00_JPRB, 0.34076E+00_JPRB, 0.33777E+00_JPRB ,&
&  0.33513E+00_JPRB, 0.33034E+00_JPRB, 0.32727E+00_JPRB, 0.32448E+00_JPRB, 0.32422E+00_JPRB, 0.32443E+00_JPRB, 0.32438E+00_JPRB ,&
&  0.32299E+00_JPRB, 0.32049E+00_JPRB, 0.31840E+00_JPRB, 0.31719E+00_JPRB, 0.31942E+00_JPRB, 0.32631E+00_JPRB, 0.33952E+00_JPRB ,&
&  0.36850E+00_JPRB, 0.33191E+00_JPRB, 0.34498E+00_JPRB, 0.36107E+00_JPRB, 0.38708E+00_JPRB, 0.40528E+00_JPRB, 0.41307E+00_JPRB ,&
&  0.42205E+00_JPRB, 0.43160E+00_JPRB, 0.43750E+00_JPRB, 0.44240E+00_JPRB, 0.43790E+00_JPRB, 0.43023E+00_JPRB, 0.42429E+00_JPRB ,&
&  0.41630E+00_JPRB, 0.40642E+00_JPRB, 0.39654E+00_JPRB, 0.38689E+00_JPRB, 0.38356E+00_JPRB, 0.38515E+00_JPRB, 0.38264E+00_JPRB ,&
&  0.37378E+00_JPRB, 0.36591E+00_JPRB, 0.36209E+00_JPRB, 0.36539E+00_JPRB, 0.37689E+00_JPRB, 0.39730E+00_JPRB, 0.43759E+00_JPRB ,&
&  0.48372E+00_JPRB, 0.53554E+00_JPRB, 0.59311E+00_JPRB, 0.64277E+00_JPRB, 0.68237E+00_JPRB, 0.70333E+00_JPRB, 0.70961E+00_JPRB ,&
&  0.69958E+00_JPRB, 0.68264E+00_JPRB, 0.66496E+00_JPRB, 0.64204E+00_JPRB, 0.61229E+00_JPRB, 0.58433E+00_JPRB, 0.56170E+00_JPRB ,&
&  0.54399E+00_JPRB, 0.52586E+00_JPRB, 0.50603E+00_JPRB, 0.48465E+00_JPRB, 0.46247E+00_JPRB, 0.44246E+00_JPRB, 0.42586E+00_JPRB ,&
&  0.41201E+00_JPRB, 0.40009E+00_JPRB, 0.39283E+00_JPRB, 0.39061E+00_JPRB, 0.39379E+00_JPRB, 0.39879E+00_JPRB, 0.40267E+00_JPRB ,&
&  0.40725E+00_JPRB, 0.41406E+00_JPRB, 0.41919E+00_JPRB, 0.42857E+00_JPRB, 0.43000E+00_JPRB, 0.42905E+00_JPRB, 0.42837E+00_JPRB ,&
&  0.42426E+00_JPRB, 0.41614E+00_JPRB, 0.36570E+00_JPRB, 0.38372E+00_JPRB, 0.38638E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.36535E+00_JPRB, 0.36275E+00_JPRB, 0.35991E+00_JPRB, 0.36165E+00_JPRB, 0.36052E+00_JPRB, 0.35694E+00_JPRB, 0.35079E+00_JPRB ,&
&  0.34102E+00_JPRB, 0.33428E+00_JPRB, 0.32918E+00_JPRB, 0.32775E+00_JPRB, 0.33032E+00_JPRB, 0.32870E+00_JPRB, 0.32465E+00_JPRB ,&
&  0.32152E+00_JPRB, 0.31795E+00_JPRB, 0.31537E+00_JPRB, 0.31230E+00_JPRB, 0.30981E+00_JPRB, 0.30879E+00_JPRB, 0.30820E+00_JPRB ,&
&  0.30673E+00_JPRB, 0.30426E+00_JPRB, 0.30222E+00_JPRB, 0.30144E+00_JPRB, 0.30429E+00_JPRB, 0.31066E+00_JPRB, 0.32065E+00_JPRB ,&
&  0.31800E+00_JPRB, 0.30016E+00_JPRB, 0.31722E+00_JPRB, 0.33835E+00_JPRB, 0.35195E+00_JPRB, 0.36419E+00_JPRB, 0.37331E+00_JPRB ,&
&  0.38157E+00_JPRB, 0.38403E+00_JPRB, 0.38603E+00_JPRB, 0.38699E+00_JPRB, 0.38568E+00_JPRB, 0.38119E+00_JPRB, 0.37633E+00_JPRB ,&
&  0.37291E+00_JPRB, 0.37049E+00_JPRB, 0.36769E+00_JPRB, 0.36242E+00_JPRB, 0.35727E+00_JPRB, 0.35272E+00_JPRB, 0.34835E+00_JPRB ,&
&  0.34375E+00_JPRB, 0.33902E+00_JPRB, 0.33572E+00_JPRB, 0.33512E+00_JPRB, 0.33781E+00_JPRB, 0.34316E+00_JPRB, 0.35306E+00_JPRB ,&
&  0.36528E+00_JPRB, 0.38661E+00_JPRB, 0.42489E+00_JPRB, 0.46468E+00_JPRB, 0.49691E+00_JPRB, 0.51954E+00_JPRB, 0.53805E+00_JPRB ,&
&  0.54431E+00_JPRB, 0.54345E+00_JPRB, 0.54011E+00_JPRB, 0.53248E+00_JPRB, 0.52017E+00_JPRB, 0.50298E+00_JPRB, 0.48213E+00_JPRB ,&
&  0.46318E+00_JPRB, 0.44637E+00_JPRB, 0.43016E+00_JPRB, 0.41455E+00_JPRB, 0.39916E+00_JPRB, 0.38471E+00_JPRB, 0.37224E+00_JPRB ,&
&  0.36138E+00_JPRB, 0.35155E+00_JPRB, 0.34446E+00_JPRB, 0.34112E+00_JPRB, 0.34246E+00_JPRB, 0.34675E+00_JPRB, 0.35281E+00_JPRB ,&
&  0.35860E+00_JPRB, 0.36661E+00_JPRB, 0.37449E+00_JPRB, 0.38277E+00_JPRB, 0.38881E+00_JPRB, 0.38920E+00_JPRB, 0.38898E+00_JPRB ,&
&  0.38743E+00_JPRB, 0.38185E+00_JPRB, 0.37716E+00_JPRB, 0.37083E+00_JPRB, 0.36603E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.33723E+00_JPRB, 0.33303E+00_JPRB, 0.33290E+00_JPRB, 0.33645E+00_JPRB, 0.34008E+00_JPRB, 0.33938E+00_JPRB, 0.33531E+00_JPRB ,&
&  0.32628E+00_JPRB, 0.31835E+00_JPRB, 0.31194E+00_JPRB, 0.30900E+00_JPRB, 0.31060E+00_JPRB, 0.31088E+00_JPRB, 0.30847E+00_JPRB ,&
&  0.30583E+00_JPRB, 0.30103E+00_JPRB, 0.29700E+00_JPRB, 0.29298E+00_JPRB, 0.29006E+00_JPRB, 0.28877E+00_JPRB, 0.28829E+00_JPRB ,&
&  0.28699E+00_JPRB, 0.28553E+00_JPRB, 0.28548E+00_JPRB, 0.28583E+00_JPRB, 0.28787E+00_JPRB, 0.29233E+00_JPRB, 0.29768E+00_JPRB ,&
&  0.30665E+00_JPRB, 0.29167E+00_JPRB, 0.30852E+00_JPRB, 0.31347E+00_JPRB, 0.31783E+00_JPRB, 0.32497E+00_JPRB, 0.33326E+00_JPRB ,&
&  0.34079E+00_JPRB, 0.34534E+00_JPRB, 0.34765E+00_JPRB, 0.34770E+00_JPRB, 0.34577E+00_JPRB, 0.34401E+00_JPRB, 0.34015E+00_JPRB ,&
&  0.33525E+00_JPRB, 0.33330E+00_JPRB, 0.33242E+00_JPRB, 0.33040E+00_JPRB, 0.32830E+00_JPRB, 0.32534E+00_JPRB, 0.32245E+00_JPRB ,&
&  0.32005E+00_JPRB, 0.31744E+00_JPRB, 0.31527E+00_JPRB, 0.31306E+00_JPRB, 0.31168E+00_JPRB, 0.31239E+00_JPRB, 0.31532E+00_JPRB ,&
&  0.31863E+00_JPRB, 0.32393E+00_JPRB, 0.33569E+00_JPRB, 0.35282E+00_JPRB, 0.37213E+00_JPRB, 0.38996E+00_JPRB, 0.40479E+00_JPRB ,&
&  0.41414E+00_JPRB, 0.41921E+00_JPRB, 0.42017E+00_JPRB, 0.41854E+00_JPRB, 0.41376E+00_JPRB, 0.40733E+00_JPRB, 0.39794E+00_JPRB ,&
&  0.38862E+00_JPRB, 0.37979E+00_JPRB, 0.37055E+00_JPRB, 0.36152E+00_JPRB, 0.35172E+00_JPRB, 0.34127E+00_JPRB, 0.33173E+00_JPRB ,&
&  0.32413E+00_JPRB, 0.31864E+00_JPRB, 0.31474E+00_JPRB, 0.31251E+00_JPRB, 0.31148E+00_JPRB, 0.31287E+00_JPRB, 0.31546E+00_JPRB ,&
&  0.31935E+00_JPRB, 0.32473E+00_JPRB, 0.33142E+00_JPRB, 0.33796E+00_JPRB, 0.34445E+00_JPRB, 0.34762E+00_JPRB, 0.35006E+00_JPRB ,&
&  0.35005E+00_JPRB, 0.34727E+00_JPRB, 0.34517E+00_JPRB, 0.34265E+00_JPRB, 0.34046E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.31076E+00_JPRB, 0.30929E+00_JPRB, 0.31186E+00_JPRB, 0.31641E+00_JPRB, 0.32254E+00_JPRB, 0.32149E+00_JPRB, 0.31916E+00_JPRB ,&
&  0.31386E+00_JPRB, 0.30695E+00_JPRB, 0.30027E+00_JPRB, 0.29408E+00_JPRB, 0.29137E+00_JPRB, 0.29183E+00_JPRB, 0.29163E+00_JPRB ,&
&  0.29037E+00_JPRB, 0.28707E+00_JPRB, 0.28241E+00_JPRB, 0.27789E+00_JPRB, 0.27462E+00_JPRB, 0.27286E+00_JPRB, 0.27137E+00_JPRB ,&
&  0.27054E+00_JPRB, 0.26937E+00_JPRB, 0.27031E+00_JPRB, 0.27040E+00_JPRB, 0.27127E+00_JPRB, 0.27512E+00_JPRB, 0.27823E+00_JPRB ,&
&  0.27978E+00_JPRB, 0.28268E+00_JPRB, 0.28226E+00_JPRB, 0.28277E+00_JPRB, 0.28612E+00_JPRB, 0.29363E+00_JPRB, 0.30276E+00_JPRB ,&
&  0.31077E+00_JPRB, 0.31608E+00_JPRB, 0.31837E+00_JPRB, 0.31720E+00_JPRB, 0.31456E+00_JPRB, 0.31206E+00_JPRB, 0.31036E+00_JPRB ,&
&  0.30849E+00_JPRB, 0.30486E+00_JPRB, 0.30091E+00_JPRB, 0.29760E+00_JPRB, 0.29616E+00_JPRB, 0.29516E+00_JPRB, 0.29388E+00_JPRB ,&
&  0.29240E+00_JPRB, 0.29095E+00_JPRB, 0.28847E+00_JPRB, 0.28543E+00_JPRB, 0.28268E+00_JPRB, 0.28185E+00_JPRB, 0.28320E+00_JPRB ,&
&  0.28618E+00_JPRB, 0.29017E+00_JPRB, 0.29595E+00_JPRB, 0.30338E+00_JPRB, 0.31076E+00_JPRB, 0.31938E+00_JPRB, 0.32689E+00_JPRB ,&
&  0.33240E+00_JPRB, 0.33530E+00_JPRB, 0.33609E+00_JPRB, 0.33567E+00_JPRB, 0.33417E+00_JPRB, 0.33164E+00_JPRB, 0.32873E+00_JPRB ,&
&  0.32581E+00_JPRB, 0.32258E+00_JPRB, 0.31898E+00_JPRB, 0.31490E+00_JPRB, 0.30933E+00_JPRB, 0.30336E+00_JPRB, 0.29855E+00_JPRB ,&
&  0.29531E+00_JPRB, 0.29310E+00_JPRB, 0.29154E+00_JPRB, 0.29010E+00_JPRB, 0.28928E+00_JPRB, 0.28950E+00_JPRB, 0.29101E+00_JPRB ,&
&  0.29389E+00_JPRB, 0.29789E+00_JPRB, 0.30279E+00_JPRB, 0.30795E+00_JPRB, 0.31173E+00_JPRB, 0.31493E+00_JPRB, 0.31805E+00_JPRB ,&
&  0.31917E+00_JPRB, 0.31750E+00_JPRB, 0.31629E+00_JPRB, 0.31565E+00_JPRB, 0.31362E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.28809E+00_JPRB, 0.28882E+00_JPRB, 0.29340E+00_JPRB, 0.29833E+00_JPRB, 0.30337E+00_JPRB, 0.30467E+00_JPRB, 0.30233E+00_JPRB ,&
&  0.29959E+00_JPRB, 0.29586E+00_JPRB, 0.29034E+00_JPRB, 0.28351E+00_JPRB, 0.27732E+00_JPRB, 0.27565E+00_JPRB, 0.27574E+00_JPRB ,&
&  0.27599E+00_JPRB, 0.27443E+00_JPRB, 0.27117E+00_JPRB, 0.26751E+00_JPRB, 0.26328E+00_JPRB, 0.25959E+00_JPRB, 0.25682E+00_JPRB ,&
&  0.25602E+00_JPRB, 0.25564E+00_JPRB, 0.25623E+00_JPRB, 0.25663E+00_JPRB, 0.25758E+00_JPRB, 0.26047E+00_JPRB, 0.26580E+00_JPRB ,&
&  0.26352E+00_JPRB, 0.26437E+00_JPRB, 0.25786E+00_JPRB, 0.26045E+00_JPRB, 0.26296E+00_JPRB, 0.26769E+00_JPRB, 0.27292E+00_JPRB ,&
&  0.27810E+00_JPRB, 0.28273E+00_JPRB, 0.28566E+00_JPRB, 0.28591E+00_JPRB, 0.28466E+00_JPRB, 0.28458E+00_JPRB, 0.28445E+00_JPRB ,&
&  0.28373E+00_JPRB, 0.28271E+00_JPRB, 0.28081E+00_JPRB, 0.27707E+00_JPRB, 0.27191E+00_JPRB, 0.26747E+00_JPRB, 0.26439E+00_JPRB ,&
&  0.26393E+00_JPRB, 0.26389E+00_JPRB, 0.26283E+00_JPRB, 0.26156E+00_JPRB, 0.26015E+00_JPRB, 0.25963E+00_JPRB, 0.26079E+00_JPRB ,&
&  0.26286E+00_JPRB, 0.26484E+00_JPRB, 0.26653E+00_JPRB, 0.26945E+00_JPRB, 0.27370E+00_JPRB, 0.27781E+00_JPRB, 0.28149E+00_JPRB ,&
&  0.28425E+00_JPRB, 0.28491E+00_JPRB, 0.28451E+00_JPRB, 0.28350E+00_JPRB, 0.28287E+00_JPRB, 0.28230E+00_JPRB, 0.28229E+00_JPRB ,&
&  0.28214E+00_JPRB, 0.28150E+00_JPRB, 0.27992E+00_JPRB, 0.27750E+00_JPRB, 0.27506E+00_JPRB, 0.27273E+00_JPRB, 0.27116E+00_JPRB ,&
&  0.27041E+00_JPRB, 0.26995E+00_JPRB, 0.26918E+00_JPRB, 0.26831E+00_JPRB, 0.26835E+00_JPRB, 0.26869E+00_JPRB, 0.26975E+00_JPRB ,&
&  0.27151E+00_JPRB, 0.27406E+00_JPRB, 0.27753E+00_JPRB, 0.28124E+00_JPRB, 0.28203E+00_JPRB, 0.28270E+00_JPRB, 0.28496E+00_JPRB ,&
&  0.28715E+00_JPRB, 0.28788E+00_JPRB, 0.28783E+00_JPRB, 0.28783E+00_JPRB, 0.28849E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.26925E+00_JPRB, 0.27163E+00_JPRB, 0.27567E+00_JPRB, 0.28162E+00_JPRB, 0.28487E+00_JPRB, 0.28806E+00_JPRB, 0.28780E+00_JPRB ,&
&  0.28533E+00_JPRB, 0.28191E+00_JPRB, 0.27781E+00_JPRB, 0.27274E+00_JPRB, 0.26739E+00_JPRB, 0.26332E+00_JPRB, 0.26188E+00_JPRB ,&
&  0.26179E+00_JPRB, 0.26086E+00_JPRB, 0.25835E+00_JPRB, 0.25494E+00_JPRB, 0.25113E+00_JPRB, 0.24808E+00_JPRB, 0.24573E+00_JPRB ,&
&  0.24499E+00_JPRB, 0.24534E+00_JPRB, 0.24628E+00_JPRB, 0.24681E+00_JPRB, 0.24710E+00_JPRB, 0.24943E+00_JPRB, 0.25328E+00_JPRB ,&
&  0.25119E+00_JPRB, 0.24885E+00_JPRB, 0.24381E+00_JPRB, 0.24337E+00_JPRB, 0.24473E+00_JPRB, 0.24700E+00_JPRB, 0.24924E+00_JPRB ,&
&  0.25119E+00_JPRB, 0.25387E+00_JPRB, 0.25675E+00_JPRB, 0.25893E+00_JPRB, 0.26041E+00_JPRB, 0.26180E+00_JPRB, 0.26269E+00_JPRB ,&
&  0.26292E+00_JPRB, 0.26183E+00_JPRB, 0.25997E+00_JPRB, 0.25662E+00_JPRB, 0.25230E+00_JPRB, 0.24937E+00_JPRB, 0.24801E+00_JPRB ,&
&  0.24669E+00_JPRB, 0.24669E+00_JPRB, 0.24686E+00_JPRB, 0.24644E+00_JPRB, 0.24567E+00_JPRB, 0.24565E+00_JPRB, 0.24653E+00_JPRB ,&
&  0.24730E+00_JPRB, 0.24728E+00_JPRB, 0.24671E+00_JPRB, 0.24704E+00_JPRB, 0.24864E+00_JPRB, 0.25100E+00_JPRB, 0.25297E+00_JPRB ,&
&  0.25438E+00_JPRB, 0.25427E+00_JPRB, 0.25335E+00_JPRB, 0.25141E+00_JPRB, 0.24997E+00_JPRB, 0.25010E+00_JPRB, 0.25152E+00_JPRB ,&
&  0.25267E+00_JPRB, 0.25294E+00_JPRB, 0.25320E+00_JPRB, 0.25199E+00_JPRB, 0.25067E+00_JPRB, 0.25025E+00_JPRB, 0.25014E+00_JPRB ,&
&  0.24966E+00_JPRB, 0.24894E+00_JPRB, 0.24818E+00_JPRB, 0.24773E+00_JPRB, 0.24796E+00_JPRB, 0.24786E+00_JPRB, 0.24855E+00_JPRB ,&
&  0.25088E+00_JPRB, 0.25373E+00_JPRB, 0.25658E+00_JPRB, 0.25868E+00_JPRB, 0.25981E+00_JPRB, 0.25976E+00_JPRB, 0.26151E+00_JPRB ,&
&  0.26437E+00_JPRB, 0.26641E+00_JPRB, 0.26621E+00_JPRB, 0.26571E+00_JPRB, 0.26725E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.25298E+00_JPRB, 0.25547E+00_JPRB, 0.25971E+00_JPRB, 0.26460E+00_JPRB, 0.26789E+00_JPRB, 0.27189E+00_JPRB, 0.27250E+00_JPRB ,&
&  0.27170E+00_JPRB, 0.26988E+00_JPRB, 0.26587E+00_JPRB, 0.26102E+00_JPRB, 0.25648E+00_JPRB, 0.25271E+00_JPRB, 0.25012E+00_JPRB ,&
&  0.24873E+00_JPRB, 0.24753E+00_JPRB, 0.24610E+00_JPRB, 0.24376E+00_JPRB, 0.24089E+00_JPRB, 0.23831E+00_JPRB, 0.23628E+00_JPRB ,&
&  0.23550E+00_JPRB, 0.23535E+00_JPRB, 0.23562E+00_JPRB, 0.23547E+00_JPRB, 0.23530E+00_JPRB, 0.23650E+00_JPRB, 0.23811E+00_JPRB ,&
&  0.23719E+00_JPRB, 0.23313E+00_JPRB, 0.23057E+00_JPRB, 0.23038E+00_JPRB, 0.23052E+00_JPRB, 0.23106E+00_JPRB, 0.23179E+00_JPRB ,&
&  0.23202E+00_JPRB, 0.23273E+00_JPRB, 0.23409E+00_JPRB, 0.23561E+00_JPRB, 0.23670E+00_JPRB, 0.23908E+00_JPRB, 0.24029E+00_JPRB ,&
&  0.24040E+00_JPRB, 0.23970E+00_JPRB, 0.23840E+00_JPRB, 0.23587E+00_JPRB, 0.23411E+00_JPRB, 0.23359E+00_JPRB, 0.23301E+00_JPRB ,&
&  0.23376E+00_JPRB, 0.23456E+00_JPRB, 0.23534E+00_JPRB, 0.23468E+00_JPRB, 0.23317E+00_JPRB, 0.23307E+00_JPRB, 0.23365E+00_JPRB ,&
&  0.23377E+00_JPRB, 0.23282E+00_JPRB, 0.23125E+00_JPRB, 0.23044E+00_JPRB, 0.22993E+00_JPRB, 0.23027E+00_JPRB, 0.23049E+00_JPRB ,&
&  0.23031E+00_JPRB, 0.22919E+00_JPRB, 0.22843E+00_JPRB, 0.22810E+00_JPRB, 0.22794E+00_JPRB, 0.22840E+00_JPRB, 0.22835E+00_JPRB ,&
&  0.22906E+00_JPRB, 0.22982E+00_JPRB, 0.22907E+00_JPRB, 0.22861E+00_JPRB, 0.22829E+00_JPRB, 0.22833E+00_JPRB, 0.22814E+00_JPRB ,&
&  0.22753E+00_JPRB, 0.22680E+00_JPRB, 0.22628E+00_JPRB, 0.22619E+00_JPRB, 0.22550E+00_JPRB, 0.22465E+00_JPRB, 0.22533E+00_JPRB ,&
&  0.22838E+00_JPRB, 0.23168E+00_JPRB, 0.23455E+00_JPRB, 0.23648E+00_JPRB, 0.23716E+00_JPRB, 0.23854E+00_JPRB, 0.24112E+00_JPRB ,&
&  0.24332E+00_JPRB, 0.24420E+00_JPRB, 0.24611E+00_JPRB, 0.24788E+00_JPRB, 0.24993E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.23882E+00_JPRB, 0.24203E+00_JPRB, 0.24445E+00_JPRB, 0.24777E+00_JPRB, 0.25088E+00_JPRB, 0.25553E+00_JPRB, 0.25740E+00_JPRB ,&
&  0.25718E+00_JPRB, 0.25676E+00_JPRB, 0.25524E+00_JPRB, 0.25278E+00_JPRB, 0.24897E+00_JPRB, 0.24510E+00_JPRB, 0.24148E+00_JPRB ,&
&  0.23872E+00_JPRB, 0.23651E+00_JPRB, 0.23475E+00_JPRB, 0.23317E+00_JPRB, 0.23094E+00_JPRB, 0.22884E+00_JPRB, 0.22724E+00_JPRB ,&
&  0.22646E+00_JPRB, 0.22568E+00_JPRB, 0.22527E+00_JPRB, 0.22507E+00_JPRB, 0.22479E+00_JPRB, 0.22450E+00_JPRB, 0.22450E+00_JPRB ,&
&  0.22346E+00_JPRB, 0.22013E+00_JPRB, 0.21824E+00_JPRB, 0.21765E+00_JPRB, 0.21711E+00_JPRB, 0.21756E+00_JPRB, 0.21723E+00_JPRB ,&
&  0.21702E+00_JPRB, 0.21759E+00_JPRB, 0.21903E+00_JPRB, 0.22068E+00_JPRB, 0.22083E+00_JPRB, 0.22140E+00_JPRB, 0.22254E+00_JPRB ,&
&  0.22296E+00_JPRB, 0.22234E+00_JPRB, 0.22067E+00_JPRB, 0.22015E+00_JPRB, 0.22079E+00_JPRB, 0.22087E+00_JPRB, 0.22131E+00_JPRB ,&
&  0.22203E+00_JPRB, 0.22318E+00_JPRB, 0.22382E+00_JPRB, 0.22237E+00_JPRB, 0.22091E+00_JPRB, 0.22039E+00_JPRB, 0.22042E+00_JPRB ,&
&  0.22019E+00_JPRB, 0.21932E+00_JPRB, 0.21744E+00_JPRB, 0.21578E+00_JPRB, 0.21330E+00_JPRB, 0.21131E+00_JPRB, 0.21032E+00_JPRB ,&
&  0.21084E+00_JPRB, 0.21134E+00_JPRB, 0.21272E+00_JPRB, 0.21394E+00_JPRB, 0.21464E+00_JPRB, 0.21477E+00_JPRB, 0.21340E+00_JPRB ,&
&  0.21176E+00_JPRB, 0.21108E+00_JPRB, 0.21011E+00_JPRB, 0.20911E+00_JPRB, 0.20931E+00_JPRB, 0.20920E+00_JPRB, 0.20870E+00_JPRB ,&
&  0.20810E+00_JPRB, 0.20836E+00_JPRB, 0.20861E+00_JPRB, 0.20825E+00_JPRB, 0.20768E+00_JPRB, 0.20684E+00_JPRB, 0.20730E+00_JPRB ,&
&  0.20866E+00_JPRB, 0.21063E+00_JPRB, 0.21295E+00_JPRB, 0.21553E+00_JPRB, 0.21827E+00_JPRB, 0.22072E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22602E+00_JPRB, 0.22707E+00_JPRB, 0.22893E+00_JPRB, 0.23187E+00_JPRB, 0.23479E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.22175E+00_JPRB, 0.22635E+00_JPRB, 0.22888E+00_JPRB, 0.23108E+00_JPRB, 0.23366E+00_JPRB, 0.23821E+00_JPRB, 0.24142E+00_JPRB ,&
&  0.24211E+00_JPRB, 0.24215E+00_JPRB, 0.24177E+00_JPRB, 0.24073E+00_JPRB, 0.23905E+00_JPRB, 0.23751E+00_JPRB, 0.23486E+00_JPRB ,&
&  0.23156E+00_JPRB, 0.22855E+00_JPRB, 0.22585E+00_JPRB, 0.22329E+00_JPRB, 0.22090E+00_JPRB, 0.21872E+00_JPRB, 0.21698E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21614E+00_JPRB, 0.21519E+00_JPRB, 0.21493E+00_JPRB, 0.21494E+00_JPRB, 0.21427E+00_JPRB, 0.21294E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.20844E+00_JPRB, 0.20554E+00_JPRB, 0.20452E+00_JPRB, 0.20459E+00_JPRB, 0.20384E+00_JPRB, 0.20276E+00_JPRB ,&
&  0.20396E+00_JPRB, 0.20600E+00_JPRB, 0.20820E+00_JPRB, 0.20927E+00_JPRB, 0.20998E+00_JPRB, 0.21109E+00_JPRB, 0.21248E+00_JPRB ,&
&  0.21265E+00_JPRB, 0.21025E+00_JPRB, 0.20815E+00_JPRB, 0.20773E+00_JPRB, 0.20767E+00_JPRB, 0.20740E+00_JPRB, 0.20780E+00_JPRB ,&
&  0.20845E+00_JPRB, 0.20985E+00_JPRB, 0.21080E+00_JPRB, 0.20988E+00_JPRB, 0.20817E+00_JPRB, 0.20690E+00_JPRB, 0.20611E+00_JPRB ,&
&  0.20575E+00_JPRB, 0.20531E+00_JPRB, 0.20449E+00_JPRB, 0.20313E+00_JPRB, 0.20161E+00_JPRB, 0.20023E+00_JPRB, 0.19881E+00_JPRB ,&
&  0.19761E+00_JPRB, 0.19790E+00_JPRB, 0.20014E+00_JPRB, 0.20163E+00_JPRB, 0.20138E+00_JPRB, 0.20012E+00_JPRB, 0.19845E+00_JPRB ,&
&  0.19739E+00_JPRB, 0.19525E+00_JPRB, 0.19343E+00_JPRB, 0.19204E+00_JPRB, 0.19100E+00_JPRB, 0.18957E+00_JPRB, 0.18903E+00_JPRB ,&
&  0.18931E+00_JPRB, 0.19079E+00_JPRB, 0.19122E+00_JPRB, 0.19125E+00_JPRB, 0.19087E+00_JPRB, 0.19056E+00_JPRB, 0.19141E+00_JPRB ,&
&  0.19261E+00_JPRB, 0.19451E+00_JPRB, 0.19658E+00_JPRB, 0.19932E+00_JPRB, 0.20198E+00_JPRB, 0.20534E+00_JPRB, 0.20810E+00_JPRB ,&
&  0.20984E+00_JPRB, 0.21036E+00_JPRB, 0.21156E+00_JPRB, 0.21341E+00_JPRB, 0.21659E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.20342E+00_JPRB, 0.20869E+00_JPRB, 0.21326E+00_JPRB, 0.21567E+00_JPRB, 0.21720E+00_JPRB, 0.21957E+00_JPRB, 0.22209E+00_JPRB ,&
&  0.22391E+00_JPRB, 0.22497E+00_JPRB, 0.22547E+00_JPRB, 0.22540E+00_JPRB, 0.22456E+00_JPRB, 0.22314E+00_JPRB, 0.22147E+00_JPRB ,&
&  0.21962E+00_JPRB, 0.21699E+00_JPRB, 0.21457E+00_JPRB, 0.21274E+00_JPRB, 0.21120E+00_JPRB, 0.20940E+00_JPRB, 0.20796E+00_JPRB ,&
&  0.20643E+00_JPRB, 0.20587E+00_JPRB, 0.20518E+00_JPRB, 0.20450E+00_JPRB, 0.20396E+00_JPRB, 0.20307E+00_JPRB, 0.20228E+00_JPRB ,&
&  0.20013E+00_JPRB, 0.19658E+00_JPRB, 0.19495E+00_JPRB, 0.19422E+00_JPRB, 0.19438E+00_JPRB, 0.19378E+00_JPRB, 0.19301E+00_JPRB ,&
&  0.19461E+00_JPRB, 0.19740E+00_JPRB, 0.19944E+00_JPRB, 0.20053E+00_JPRB, 0.20128E+00_JPRB, 0.20232E+00_JPRB, 0.20353E+00_JPRB ,&
&  0.20134E+00_JPRB, 0.19870E+00_JPRB, 0.19680E+00_JPRB, 0.19586E+00_JPRB, 0.19579E+00_JPRB, 0.19661E+00_JPRB, 0.19820E+00_JPRB ,&
&  0.19963E+00_JPRB, 0.20047E+00_JPRB, 0.20018E+00_JPRB, 0.19891E+00_JPRB, 0.19734E+00_JPRB, 0.19657E+00_JPRB, 0.19613E+00_JPRB ,&
&  0.19594E+00_JPRB, 0.19567E+00_JPRB, 0.19534E+00_JPRB, 0.19435E+00_JPRB, 0.18911E+00_JPRB, 0.18537E+00_JPRB, 0.18275E+00_JPRB ,&
&  0.18333E+00_JPRB, 0.18348E+00_JPRB, 0.18251E+00_JPRB, 0.18771E+00_JPRB, 0.19322E+00_JPRB, 0.18890E+00_JPRB, 0.18374E+00_JPRB ,&
&  0.17671E+00_JPRB, 0.17732E+00_JPRB, 0.17465E+00_JPRB, 0.17010E+00_JPRB, 0.16883E+00_JPRB, 0.16888E+00_JPRB, 0.16846E+00_JPRB ,&
&  0.16912E+00_JPRB, 0.16916E+00_JPRB, 0.17023E+00_JPRB, 0.17073E+00_JPRB, 0.17100E+00_JPRB, 0.17173E+00_JPRB, 0.17250E+00_JPRB ,&
&  0.17447E+00_JPRB, 0.17815E+00_JPRB, 0.18243E+00_JPRB, 0.18403E+00_JPRB, 0.18663E+00_JPRB, 0.18899E+00_JPRB, 0.19053E+00_JPRB ,&
&  0.19178E+00_JPRB, 0.19360E+00_JPRB, 0.19534E+00_JPRB, 0.19677E+00_JPRB, 0.19922E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.19078E+00_JPRB, 0.19310E+00_JPRB, 0.19596E+00_JPRB, 0.19882E+00_JPRB, 0.20088E+00_JPRB, 0.20288E+00_JPRB, 0.20466E+00_JPRB ,&
&  0.20624E+00_JPRB, 0.20734E+00_JPRB, 0.20806E+00_JPRB, 0.20846E+00_JPRB, 0.20874E+00_JPRB, 0.20849E+00_JPRB, 0.20779E+00_JPRB ,&
&  0.20705E+00_JPRB, 0.20575E+00_JPRB, 0.20405E+00_JPRB, 0.20268E+00_JPRB, 0.20180E+00_JPRB, 0.20070E+00_JPRB, 0.19909E+00_JPRB ,&
&  0.19734E+00_JPRB, 0.19578E+00_JPRB, 0.19457E+00_JPRB, 0.19344E+00_JPRB, 0.19340E+00_JPRB, 0.19398E+00_JPRB, 0.19447E+00_JPRB ,&
&  0.19051E+00_JPRB, 0.18540E+00_JPRB, 0.18621E+00_JPRB, 0.18723E+00_JPRB, 0.18557E+00_JPRB, 0.18684E+00_JPRB, 0.18757E+00_JPRB ,&
&  0.18861E+00_JPRB, 0.18996E+00_JPRB, 0.19128E+00_JPRB, 0.19195E+00_JPRB, 0.19237E+00_JPRB, 0.19303E+00_JPRB, 0.19240E+00_JPRB ,&
&  0.19041E+00_JPRB, 0.18872E+00_JPRB, 0.18725E+00_JPRB, 0.18579E+00_JPRB, 0.18548E+00_JPRB, 0.18703E+00_JPRB, 0.18796E+00_JPRB ,&
&  0.18711E+00_JPRB, 0.18578E+00_JPRB, 0.18455E+00_JPRB, 0.18487E+00_JPRB, 0.18564E+00_JPRB, 0.18605E+00_JPRB, 0.18409E+00_JPRB ,&
&  0.18280E+00_JPRB, 0.18009E+00_JPRB, 0.17756E+00_JPRB, 0.17508E+00_JPRB, 0.17379E+00_JPRB, 0.17039E+00_JPRB, 0.16678E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16981E+00_JPRB, 0.17575E+00_JPRB, 0.17807E+00_JPRB, 0.16857E+00_JPRB, 0.16881E+00_JPRB, 0.16875E+00_JPRB ,&
&  0.16434E+00_JPRB, 0.15939E+00_JPRB, 0.15580E+00_JPRB, 0.15428E+00_JPRB, 0.15369E+00_JPRB, 0.15314E+00_JPRB, 0.15353E+00_JPRB ,&
&  0.15437E+00_JPRB, 0.15482E+00_JPRB, 0.15539E+00_JPRB, 0.15652E+00_JPRB, 0.15763E+00_JPRB, 0.15876E+00_JPRB, 0.15976E+00_JPRB ,&
&  0.16196E+00_JPRB, 0.16466E+00_JPRB, 0.16681E+00_JPRB, 0.16964E+00_JPRB, 0.17268E+00_JPRB, 0.17569E+00_JPRB, 0.17867E+00_JPRB ,&
&  0.18198E+00_JPRB, 0.18400E+00_JPRB, 0.18551E+00_JPRB, 0.18818E+00_JPRB, 0.18946E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.18492E+00_JPRB, 0.18746E+00_JPRB, 0.18929E+00_JPRB, 0.19100E+00_JPRB, 0.19232E+00_JPRB, 0.19373E+00_JPRB, 0.19434E+00_JPRB ,&
&  0.19489E+00_JPRB, 0.19486E+00_JPRB, 0.19527E+00_JPRB, 0.19521E+00_JPRB, 0.19538E+00_JPRB, 0.19430E+00_JPRB, 0.19374E+00_JPRB ,&
&  0.19332E+00_JPRB, 0.19365E+00_JPRB, 0.19363E+00_JPRB, 0.19350E+00_JPRB, 0.19280E+00_JPRB, 0.19187E+00_JPRB, 0.19039E+00_JPRB ,&
&  0.18734E+00_JPRB, 0.18522E+00_JPRB, 0.18547E+00_JPRB, 0.18496E+00_JPRB, 0.18535E+00_JPRB, 0.18536E+00_JPRB, 0.18582E+00_JPRB ,&
&  0.18171E+00_JPRB, 0.18025E+00_JPRB, 0.18049E+00_JPRB, 0.17950E+00_JPRB, 0.18165E+00_JPRB, 0.18259E+00_JPRB, 0.18433E+00_JPRB ,&
&  0.18477E+00_JPRB, 0.18508E+00_JPRB, 0.18580E+00_JPRB, 0.18534E+00_JPRB, 0.18565E+00_JPRB, 0.18607E+00_JPRB, 0.18493E+00_JPRB ,&
&  0.18200E+00_JPRB, 0.17974E+00_JPRB, 0.17656E+00_JPRB, 0.16872E+00_JPRB, 0.16683E+00_JPRB, 0.16716E+00_JPRB, 0.16584E+00_JPRB ,&
&  0.16322E+00_JPRB, 0.16359E+00_JPRB, 0.16317E+00_JPRB, 0.16175E+00_JPRB, 0.16361E+00_JPRB, 0.16626E+00_JPRB, 0.16642E+00_JPRB ,&
&  0.16526E+00_JPRB, 0.16497E+00_JPRB, 0.16435E+00_JPRB, 0.16310E+00_JPRB, 0.16163E+00_JPRB, 0.16103E+00_JPRB, 0.16047E+00_JPRB ,&
&  0.16019E+00_JPRB, 0.15895E+00_JPRB, 0.16233E+00_JPRB, 0.16318E+00_JPRB, 0.15849E+00_JPRB, 0.16035E+00_JPRB, 0.15806E+00_JPRB ,&
&  0.15147E+00_JPRB, 0.14853E+00_JPRB, 0.14685E+00_JPRB, 0.14544E+00_JPRB, 0.14514E+00_JPRB, 0.14521E+00_JPRB, 0.14504E+00_JPRB ,&
&  0.14585E+00_JPRB, 0.14688E+00_JPRB, 0.14738E+00_JPRB, 0.14806E+00_JPRB, 0.14898E+00_JPRB, 0.15032E+00_JPRB, 0.15208E+00_JPRB ,&
&  0.15363E+00_JPRB, 0.15580E+00_JPRB, 0.15689E+00_JPRB, 0.15869E+00_JPRB, 0.16002E+00_JPRB, 0.16157E+00_JPRB, 0.16526E+00_JPRB ,&
&  0.16830E+00_JPRB, 0.17424E+00_JPRB, 0.18243E+00_JPRB, 0.18510E+00_JPRB, 0.18252E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.17975E+00_JPRB, 0.18169E+00_JPRB, 0.18372E+00_JPRB, 0.18531E+00_JPRB, 0.18690E+00_JPRB, 0.18823E+00_JPRB, 0.18893E+00_JPRB ,&
&  0.18905E+00_JPRB, 0.18841E+00_JPRB, 0.18843E+00_JPRB, 0.18714E+00_JPRB, 0.18593E+00_JPRB, 0.18174E+00_JPRB, 0.18118E+00_JPRB ,&
&  0.18018E+00_JPRB, 0.18078E+00_JPRB, 0.18035E+00_JPRB, 0.18062E+00_JPRB, 0.17994E+00_JPRB, 0.17960E+00_JPRB, 0.17803E+00_JPRB ,&
&  0.17572E+00_JPRB, 0.17396E+00_JPRB, 0.17225E+00_JPRB, 0.17175E+00_JPRB, 0.17251E+00_JPRB, 0.17307E+00_JPRB, 0.17277E+00_JPRB ,&
&  0.17280E+00_JPRB, 0.17309E+00_JPRB, 0.17354E+00_JPRB, 0.17359E+00_JPRB, 0.17876E+00_JPRB, 0.18091E+00_JPRB, 0.18200E+00_JPRB ,&
&  0.18182E+00_JPRB, 0.18214E+00_JPRB, 0.18223E+00_JPRB, 0.18136E+00_JPRB, 0.18048E+00_JPRB, 0.17644E+00_JPRB, 0.17383E+00_JPRB ,&
&  0.16610E+00_JPRB, 0.16056E+00_JPRB, 0.15635E+00_JPRB, 0.15377E+00_JPRB, 0.15274E+00_JPRB, 0.15276E+00_JPRB, 0.15231E+00_JPRB ,&
&  0.15083E+00_JPRB, 0.14946E+00_JPRB, 0.14861E+00_JPRB, 0.14932E+00_JPRB, 0.15043E+00_JPRB, 0.15167E+00_JPRB, 0.15308E+00_JPRB ,&
&  0.15430E+00_JPRB, 0.15464E+00_JPRB, 0.15463E+00_JPRB, 0.15508E+00_JPRB, 0.15542E+00_JPRB, 0.15594E+00_JPRB, 0.15634E+00_JPRB ,&
&  0.15735E+00_JPRB, 0.15656E+00_JPRB, 0.15718E+00_JPRB, 0.15827E+00_JPRB, 0.15598E+00_JPRB, 0.15233E+00_JPRB, 0.14870E+00_JPRB ,&
&  0.14527E+00_JPRB, 0.14334E+00_JPRB, 0.14175E+00_JPRB, 0.14072E+00_JPRB, 0.14046E+00_JPRB, 0.14051E+00_JPRB, 0.14100E+00_JPRB ,&
&  0.14165E+00_JPRB, 0.14258E+00_JPRB, 0.14319E+00_JPRB, 0.14401E+00_JPRB, 0.14501E+00_JPRB, 0.14609E+00_JPRB, 0.14738E+00_JPRB ,&
&  0.14897E+00_JPRB, 0.15049E+00_JPRB, 0.15156E+00_JPRB, 0.15308E+00_JPRB, 0.15593E+00_JPRB, 0.15973E+00_JPRB, 0.16548E+00_JPRB ,&
&  0.17189E+00_JPRB, 0.17722E+00_JPRB, 0.17466E+00_JPRB, 0.17699E+00_JPRB, 0.17884E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.17972E+00_JPRB, 0.18123E+00_JPRB, 0.18272E+00_JPRB, 0.18421E+00_JPRB, 0.18548E+00_JPRB, 0.18669E+00_JPRB, 0.18748E+00_JPRB ,&
&  0.18500E+00_JPRB, 0.18357E+00_JPRB, 0.18091E+00_JPRB, 0.17855E+00_JPRB, 0.17577E+00_JPRB, 0.17272E+00_JPRB, 0.17168E+00_JPRB ,&
&  0.17016E+00_JPRB, 0.16986E+00_JPRB, 0.16861E+00_JPRB, 0.16808E+00_JPRB, 0.16705E+00_JPRB, 0.16643E+00_JPRB, 0.16535E+00_JPRB ,&
&  0.16328E+00_JPRB, 0.16223E+00_JPRB, 0.16123E+00_JPRB, 0.16160E+00_JPRB, 0.16307E+00_JPRB, 0.16465E+00_JPRB, 0.16632E+00_JPRB ,&
&  0.16875E+00_JPRB, 0.16984E+00_JPRB, 0.17216E+00_JPRB, 0.17390E+00_JPRB, 0.17684E+00_JPRB, 0.17928E+00_JPRB, 0.17978E+00_JPRB ,&
&  0.17974E+00_JPRB, 0.17793E+00_JPRB, 0.17737E+00_JPRB, 0.17525E+00_JPRB, 0.16931E+00_JPRB, 0.16597E+00_JPRB, 0.16124E+00_JPRB ,&
&  0.15601E+00_JPRB, 0.15209E+00_JPRB, 0.14939E+00_JPRB, 0.14855E+00_JPRB, 0.14769E+00_JPRB, 0.14748E+00_JPRB, 0.14732E+00_JPRB ,&
&  0.14659E+00_JPRB, 0.14572E+00_JPRB, 0.14470E+00_JPRB, 0.14416E+00_JPRB, 0.14456E+00_JPRB, 0.14533E+00_JPRB, 0.14695E+00_JPRB ,&
&  0.14856E+00_JPRB, 0.14947E+00_JPRB, 0.15045E+00_JPRB, 0.15079E+00_JPRB, 0.15092E+00_JPRB, 0.15117E+00_JPRB, 0.15126E+00_JPRB ,&
&  0.15193E+00_JPRB, 0.15209E+00_JPRB, 0.15160E+00_JPRB, 0.15075E+00_JPRB, 0.14861E+00_JPRB, 0.14578E+00_JPRB, 0.14356E+00_JPRB ,&
&  0.14166E+00_JPRB, 0.14041E+00_JPRB, 0.13995E+00_JPRB, 0.13948E+00_JPRB, 0.13970E+00_JPRB, 0.14005E+00_JPRB, 0.14037E+00_JPRB ,&
&  0.14092E+00_JPRB, 0.14142E+00_JPRB, 0.14190E+00_JPRB, 0.14304E+00_JPRB, 0.14398E+00_JPRB, 0.14538E+00_JPRB, 0.14685E+00_JPRB ,&
&  0.14787E+00_JPRB, 0.14942E+00_JPRB, 0.15074E+00_JPRB, 0.15219E+00_JPRB, 0.15650E+00_JPRB, 0.15993E+00_JPRB, 0.16689E+00_JPRB ,&
&  0.17136E+00_JPRB, 0.17307E+00_JPRB, 0.17372E+00_JPRB, 0.17901E+00_JPRB, 0.17921E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.18047E+00_JPRB, 0.18094E+00_JPRB, 0.18090E+00_JPRB, 0.18185E+00_JPRB, 0.18262E+00_JPRB, 0.18345E+00_JPRB, 0.18287E+00_JPRB ,&
&  0.18235E+00_JPRB, 0.18087E+00_JPRB, 0.17970E+00_JPRB, 0.17670E+00_JPRB, 0.17431E+00_JPRB, 0.17144E+00_JPRB, 0.16896E+00_JPRB ,&
&  0.16677E+00_JPRB, 0.16467E+00_JPRB, 0.16272E+00_JPRB, 0.16103E+00_JPRB, 0.15947E+00_JPRB, 0.15809E+00_JPRB, 0.15676E+00_JPRB ,&
&  0.15479E+00_JPRB, 0.15426E+00_JPRB, 0.15431E+00_JPRB, 0.15557E+00_JPRB, 0.15812E+00_JPRB, 0.15979E+00_JPRB, 0.16180E+00_JPRB ,&
&  0.16410E+00_JPRB, 0.16348E+00_JPRB, 0.16642E+00_JPRB, 0.16926E+00_JPRB, 0.17160E+00_JPRB, 0.17350E+00_JPRB, 0.17451E+00_JPRB ,&
&  0.17509E+00_JPRB, 0.17029E+00_JPRB, 0.17144E+00_JPRB, 0.16733E+00_JPRB, 0.16273E+00_JPRB, 0.16065E+00_JPRB, 0.15673E+00_JPRB ,&
&  0.15494E+00_JPRB, 0.15190E+00_JPRB, 0.14993E+00_JPRB, 0.14827E+00_JPRB, 0.14633E+00_JPRB, 0.14589E+00_JPRB, 0.14512E+00_JPRB ,&
&  0.14470E+00_JPRB, 0.14479E+00_JPRB, 0.14406E+00_JPRB, 0.14384E+00_JPRB, 0.14342E+00_JPRB, 0.14304E+00_JPRB, 0.14344E+00_JPRB ,&
&  0.14404E+00_JPRB, 0.14476E+00_JPRB, 0.14543E+00_JPRB, 0.14553E+00_JPRB, 0.14546E+00_JPRB, 0.14545E+00_JPRB, 0.14537E+00_JPRB ,&
&  0.14564E+00_JPRB, 0.14566E+00_JPRB, 0.14527E+00_JPRB, 0.14483E+00_JPRB, 0.14379E+00_JPRB, 0.14276E+00_JPRB, 0.14175E+00_JPRB ,&
&  0.14085E+00_JPRB, 0.14036E+00_JPRB, 0.13994E+00_JPRB, 0.13982E+00_JPRB, 0.13984E+00_JPRB, 0.14015E+00_JPRB, 0.14047E+00_JPRB ,&
&  0.14078E+00_JPRB, 0.14149E+00_JPRB, 0.14201E+00_JPRB, 0.14287E+00_JPRB, 0.14391E+00_JPRB, 0.14486E+00_JPRB, 0.14628E+00_JPRB ,&
&  0.14783E+00_JPRB, 0.14952E+00_JPRB, 0.15192E+00_JPRB, 0.15403E+00_JPRB, 0.15668E+00_JPRB, 0.16012E+00_JPRB, 0.16268E+00_JPRB ,&
&  0.16543E+00_JPRB, 0.16817E+00_JPRB, 0.16844E+00_JPRB, 0.17543E+00_JPRB, 0.17933E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.17331E+00_JPRB, 0.17407E+00_JPRB, 0.17445E+00_JPRB, 0.17484E+00_JPRB, 0.17586E+00_JPRB, 0.17586E+00_JPRB, 0.17565E+00_JPRB ,&
&  0.17357E+00_JPRB, 0.17276E+00_JPRB, 0.17146E+00_JPRB, 0.16964E+00_JPRB, 0.16779E+00_JPRB, 0.16476E+00_JPRB, 0.16285E+00_JPRB ,&
&  0.16098E+00_JPRB, 0.15901E+00_JPRB, 0.15770E+00_JPRB, 0.15601E+00_JPRB, 0.15456E+00_JPRB, 0.15293E+00_JPRB, 0.15237E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15150E+00_JPRB, 0.15230E+00_JPRB, 0.15293E+00_JPRB, 0.15410E+00_JPRB, 0.15535E+00_JPRB, 0.15696E+00_JPRB ,&
&  0.15737E+00_JPRB, 0.15815E+00_JPRB, 0.15890E+00_JPRB, 0.15939E+00_JPRB, 0.16109E+00_JPRB, 0.16231E+00_JPRB, 0.16273E+00_JPRB ,&
&  0.16226E+00_JPRB, 0.16139E+00_JPRB, 0.15969E+00_JPRB, 0.15775E+00_JPRB, 0.15554E+00_JPRB, 0.15312E+00_JPRB, 0.15081E+00_JPRB ,&
&  0.14902E+00_JPRB, 0.14737E+00_JPRB, 0.14612E+00_JPRB, 0.14511E+00_JPRB, 0.14450E+00_JPRB, 0.14403E+00_JPRB, 0.14356E+00_JPRB ,&
&  0.14331E+00_JPRB, 0.14310E+00_JPRB, 0.14277E+00_JPRB, 0.14276E+00_JPRB, 0.14267E+00_JPRB, 0.14272E+00_JPRB, 0.14278E+00_JPRB ,&
&  0.14289E+00_JPRB, 0.14287E+00_JPRB, 0.14277E+00_JPRB, 0.14247E+00_JPRB, 0.14217E+00_JPRB, 0.14184E+00_JPRB, 0.14151E+00_JPRB ,&
&  0.14116E+00_JPRB, 0.14113E+00_JPRB, 0.14086E+00_JPRB, 0.14067E+00_JPRB, 0.14042E+00_JPRB, 0.13999E+00_JPRB, 0.13987E+00_JPRB ,&
&  0.13996E+00_JPRB, 0.14009E+00_JPRB, 0.13996E+00_JPRB, 0.14007E+00_JPRB, 0.14026E+00_JPRB, 0.14055E+00_JPRB, 0.14133E+00_JPRB ,&
&  0.14181E+00_JPRB, 0.14232E+00_JPRB, 0.14279E+00_JPRB, 0.14346E+00_JPRB, 0.14459E+00_JPRB, 0.14573E+00_JPRB, 0.14701E+00_JPRB ,&
&  0.14854E+00_JPRB, 0.14998E+00_JPRB, 0.15141E+00_JPRB, 0.15283E+00_JPRB, 0.15518E+00_JPRB, 0.15688E+00_JPRB, 0.15912E+00_JPRB ,&
&  0.16109E+00_JPRB, 0.16011E+00_JPRB, 0.16820E+00_JPRB, 0.16898E+00_JPRB, 0.17026E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.15776E+00_JPRB, 0.15795E+00_JPRB, 0.15728E+00_JPRB, 0.15656E+00_JPRB, 0.15045E+00_JPRB, 0.15662E+00_JPRB, 0.15594E+00_JPRB ,&
&  0.15639E+00_JPRB, 0.15634E+00_JPRB, 0.15760E+00_JPRB, 0.15774E+00_JPRB, 0.15741E+00_JPRB, 0.15722E+00_JPRB, 0.15694E+00_JPRB ,&
&  0.15660E+00_JPRB, 0.15585E+00_JPRB, 0.15546E+00_JPRB, 0.15490E+00_JPRB, 0.15434E+00_JPRB, 0.15342E+00_JPRB, 0.15245E+00_JPRB ,&
&  0.15201E+00_JPRB, 0.15208E+00_JPRB, 0.15244E+00_JPRB, 0.15090E+00_JPRB, 0.15023E+00_JPRB, 0.14980E+00_JPRB, 0.14959E+00_JPRB ,&
&  0.14922E+00_JPRB, 0.14912E+00_JPRB, 0.14913E+00_JPRB, 0.14906E+00_JPRB, 0.15027E+00_JPRB, 0.15063E+00_JPRB, 0.15091E+00_JPRB ,&
&  0.15095E+00_JPRB, 0.15079E+00_JPRB, 0.15013E+00_JPRB, 0.14953E+00_JPRB, 0.14892E+00_JPRB, 0.14786E+00_JPRB, 0.14676E+00_JPRB ,&
&  0.14582E+00_JPRB, 0.14510E+00_JPRB, 0.14428E+00_JPRB, 0.14366E+00_JPRB, 0.14314E+00_JPRB, 0.14266E+00_JPRB, 0.14235E+00_JPRB ,&
&  0.14227E+00_JPRB, 0.14227E+00_JPRB, 0.14226E+00_JPRB, 0.14224E+00_JPRB, 0.14218E+00_JPRB, 0.14215E+00_JPRB, 0.14212E+00_JPRB ,&
&  0.14240E+00_JPRB, 0.14238E+00_JPRB, 0.14222E+00_JPRB, 0.14194E+00_JPRB, 0.14182E+00_JPRB, 0.14151E+00_JPRB, 0.14129E+00_JPRB ,&
&  0.14112E+00_JPRB, 0.14090E+00_JPRB, 0.14081E+00_JPRB, 0.14078E+00_JPRB, 0.14078E+00_JPRB, 0.14077E+00_JPRB, 0.14086E+00_JPRB ,&
&  0.14100E+00_JPRB, 0.14119E+00_JPRB, 0.14154E+00_JPRB, 0.14182E+00_JPRB, 0.14208E+00_JPRB, 0.14231E+00_JPRB, 0.14278E+00_JPRB ,&
&  0.14328E+00_JPRB, 0.14378E+00_JPRB, 0.14427E+00_JPRB, 0.14460E+00_JPRB, 0.14531E+00_JPRB, 0.14603E+00_JPRB, 0.14681E+00_JPRB ,&
&  0.14765E+00_JPRB, 0.14870E+00_JPRB, 0.14980E+00_JPRB, 0.15089E+00_JPRB, 0.15233E+00_JPRB, 0.15350E+00_JPRB, 0.15464E+00_JPRB ,&
&  0.15592E+00_JPRB, 0.15826E+00_JPRB, 0.15891E+00_JPRB, 0.15907E+00_JPRB, 0.15862E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.15291E+00_JPRB, 0.15288E+00_JPRB, 0.15277E+00_JPRB, 0.15243E+00_JPRB, 0.15176E+00_JPRB, 0.15160E+00_JPRB, 0.15139E+00_JPRB ,&
&  0.15110E+00_JPRB, 0.15047E+00_JPRB, 0.15046E+00_JPRB, 0.15039E+00_JPRB, 0.15029E+00_JPRB, 0.15018E+00_JPRB, 0.14995E+00_JPRB ,&
&  0.14949E+00_JPRB, 0.14878E+00_JPRB, 0.14666E+00_JPRB, 0.14564E+00_JPRB, 0.14499E+00_JPRB, 0.14464E+00_JPRB, 0.14434E+00_JPRB ,&
&  0.14417E+00_JPRB, 0.14407E+00_JPRB, 0.14397E+00_JPRB, 0.14436E+00_JPRB, 0.14453E+00_JPRB, 0.14459E+00_JPRB, 0.14463E+00_JPRB ,&
&  0.14463E+00_JPRB, 0.14471E+00_JPRB, 0.14472E+00_JPRB, 0.14471E+00_JPRB, 0.14482E+00_JPRB, 0.14481E+00_JPRB, 0.14463E+00_JPRB ,&
&  0.14442E+00_JPRB, 0.14417E+00_JPRB, 0.14393E+00_JPRB, 0.14362E+00_JPRB, 0.14340E+00_JPRB, 0.14322E+00_JPRB, 0.14315E+00_JPRB ,&
&  0.14299E+00_JPRB, 0.14287E+00_JPRB, 0.14275E+00_JPRB, 0.14268E+00_JPRB, 0.14258E+00_JPRB, 0.14253E+00_JPRB, 0.14241E+00_JPRB ,&
&  0.14248E+00_JPRB, 0.14250E+00_JPRB, 0.14255E+00_JPRB, 0.14257E+00_JPRB, 0.14259E+00_JPRB, 0.14256E+00_JPRB, 0.14249E+00_JPRB ,&
&  0.14234E+00_JPRB, 0.14224E+00_JPRB, 0.14213E+00_JPRB, 0.14205E+00_JPRB, 0.14203E+00_JPRB, 0.14201E+00_JPRB, 0.14201E+00_JPRB ,&
&  0.14202E+00_JPRB, 0.14211E+00_JPRB, 0.14214E+00_JPRB, 0.14219E+00_JPRB, 0.14225E+00_JPRB, 0.14233E+00_JPRB, 0.14238E+00_JPRB ,&
&  0.14246E+00_JPRB, 0.14254E+00_JPRB, 0.14264E+00_JPRB, 0.14278E+00_JPRB, 0.14294E+00_JPRB, 0.14316E+00_JPRB, 0.14343E+00_JPRB ,&
&  0.14372E+00_JPRB, 0.14401E+00_JPRB, 0.14430E+00_JPRB, 0.14458E+00_JPRB, 0.14503E+00_JPRB, 0.14548E+00_JPRB, 0.14602E+00_JPRB ,&
&  0.14655E+00_JPRB, 0.14707E+00_JPRB, 0.14755E+00_JPRB, 0.14807E+00_JPRB, 0.14873E+00_JPRB, 0.14907E+00_JPRB, 0.14937E+00_JPRB ,&
&  0.14981E+00_JPRB, 0.15026E+00_JPRB, 0.15072E+00_JPRB, 0.15113E+00_JPRB, 0.15178E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB, 0.14402E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB, 0.46113E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.46197E+00_JPRB, 0.46120E+00_JPRB, 0.46058E+00_JPRB, 0.45978E+00_JPRB, 0.45866E+00_JPRB, 0.45770E+00_JPRB, 0.45677E+00_JPRB ,&
&  0.45600E+00_JPRB, 0.45555E+00_JPRB, 0.45534E+00_JPRB, 0.45528E+00_JPRB, 0.45543E+00_JPRB, 0.45557E+00_JPRB, 0.45572E+00_JPRB ,&
&  0.45580E+00_JPRB, 0.45592E+00_JPRB, 0.45598E+00_JPRB, 0.45585E+00_JPRB, 0.45559E+00_JPRB, 0.45525E+00_JPRB, 0.45473E+00_JPRB ,&
&  0.45392E+00_JPRB, 0.45317E+00_JPRB, 0.45254E+00_JPRB, 0.45187E+00_JPRB, 0.45133E+00_JPRB, 0.45073E+00_JPRB, 0.45008E+00_JPRB ,&
&  0.44956E+00_JPRB, 0.44941E+00_JPRB, 0.44939E+00_JPRB, 0.44938E+00_JPRB, 0.44960E+00_JPRB, 0.45054E+00_JPRB, 0.45143E+00_JPRB ,&
&  0.45242E+00_JPRB, 0.45359E+00_JPRB, 0.45544E+00_JPRB, 0.45726E+00_JPRB, 0.45896E+00_JPRB, 0.46042E+00_JPRB, 0.46141E+00_JPRB ,&
&  0.46151E+00_JPRB, 0.46116E+00_JPRB, 0.46047E+00_JPRB, 0.45968E+00_JPRB, 0.45842E+00_JPRB, 0.45728E+00_JPRB, 0.45634E+00_JPRB ,&
&  0.45523E+00_JPRB, 0.45430E+00_JPRB, 0.45341E+00_JPRB, 0.45219E+00_JPRB, 0.44996E+00_JPRB, 0.44831E+00_JPRB, 0.44733E+00_JPRB ,&
&  0.44691E+00_JPRB, 0.44654E+00_JPRB, 0.44647E+00_JPRB, 0.44664E+00_JPRB, 0.44729E+00_JPRB, 0.44812E+00_JPRB, 0.44920E+00_JPRB ,&
&  0.45041E+00_JPRB, 0.45195E+00_JPRB, 0.45404E+00_JPRB, 0.45624E+00_JPRB, 0.45858E+00_JPRB, 0.46121E+00_JPRB, 0.46362E+00_JPRB ,&
&  0.46596E+00_JPRB, 0.46842E+00_JPRB, 0.47052E+00_JPRB, 0.47275E+00_JPRB, 0.47507E+00_JPRB, 0.47758E+00_JPRB, 0.47956E+00_JPRB ,&
&  0.48074E+00_JPRB, 0.48164E+00_JPRB, 0.48235E+00_JPRB, 0.48224E+00_JPRB, 0.48160E+00_JPRB, 0.48107E+00_JPRB, 0.48049E+00_JPRB ,&
&  0.47970E+00_JPRB, 0.47864E+00_JPRB, 0.47761E+00_JPRB, 0.47642E+00_JPRB, 0.47476E+00_JPRB, 0.47299E+00_JPRB, 0.47131E+00_JPRB ,&
&  0.46964E+00_JPRB, 0.46781E+00_JPRB, 0.46619E+00_JPRB, 0.46466E+00_JPRB, 0.46328E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.47100E+00_JPRB, 0.46713E+00_JPRB, 0.46377E+00_JPRB, 0.46074E+00_JPRB, 0.45833E+00_JPRB, 0.45664E+00_JPRB, 0.45547E+00_JPRB ,&
&  0.45463E+00_JPRB, 0.45456E+00_JPRB, 0.45432E+00_JPRB, 0.45400E+00_JPRB, 0.45361E+00_JPRB, 0.45249E+00_JPRB, 0.45095E+00_JPRB ,&
&  0.44951E+00_JPRB, 0.44853E+00_JPRB, 0.44739E+00_JPRB, 0.44613E+00_JPRB, 0.44440E+00_JPRB, 0.44194E+00_JPRB, 0.43913E+00_JPRB ,&
&  0.43601E+00_JPRB, 0.43210E+00_JPRB, 0.42754E+00_JPRB, 0.42301E+00_JPRB, 0.42085E+00_JPRB, 0.41830E+00_JPRB, 0.41590E+00_JPRB ,&
&  0.41400E+00_JPRB, 0.41393E+00_JPRB, 0.41393E+00_JPRB, 0.41381E+00_JPRB, 0.41398E+00_JPRB, 0.41586E+00_JPRB, 0.41732E+00_JPRB ,&
&  0.41900E+00_JPRB, 0.42180E+00_JPRB, 0.42714E+00_JPRB, 0.43290E+00_JPRB, 0.43948E+00_JPRB, 0.44724E+00_JPRB, 0.45614E+00_JPRB ,&
&  0.46253E+00_JPRB, 0.46542E+00_JPRB, 0.46698E+00_JPRB, 0.46817E+00_JPRB, 0.46794E+00_JPRB, 0.46694E+00_JPRB, 0.46512E+00_JPRB ,&
&  0.46425E+00_JPRB, 0.46312E+00_JPRB, 0.46233E+00_JPRB, 0.46170E+00_JPRB, 0.45957E+00_JPRB, 0.45774E+00_JPRB, 0.45607E+00_JPRB ,&
&  0.45392E+00_JPRB, 0.45240E+00_JPRB, 0.45176E+00_JPRB, 0.45131E+00_JPRB, 0.45188E+00_JPRB, 0.45354E+00_JPRB, 0.45608E+00_JPRB ,&
&  0.46001E+00_JPRB, 0.46555E+00_JPRB, 0.47260E+00_JPRB, 0.48050E+00_JPRB, 0.48957E+00_JPRB, 0.50099E+00_JPRB, 0.51011E+00_JPRB ,&
&  0.51853E+00_JPRB, 0.52602E+00_JPRB, 0.53285E+00_JPRB, 0.53825E+00_JPRB, 0.54245E+00_JPRB, 0.54669E+00_JPRB, 0.54731E+00_JPRB ,&
&  0.54719E+00_JPRB, 0.54648E+00_JPRB, 0.54593E+00_JPRB, 0.54343E+00_JPRB, 0.54034E+00_JPRB, 0.53703E+00_JPRB, 0.53392E+00_JPRB ,&
&  0.52933E+00_JPRB, 0.52457E+00_JPRB, 0.51983E+00_JPRB, 0.51513E+00_JPRB, 0.50999E+00_JPRB, 0.50504E+00_JPRB, 0.50013E+00_JPRB ,&
&  0.49513E+00_JPRB, 0.49000E+00_JPRB, 0.48515E+00_JPRB, 0.48044E+00_JPRB, 0.47569E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.48039E+00_JPRB, 0.47366E+00_JPRB, 0.46767E+00_JPRB, 0.46250E+00_JPRB, 0.45825E+00_JPRB, 0.45524E+00_JPRB, 0.45327E+00_JPRB ,&
&  0.45203E+00_JPRB, 0.45201E+00_JPRB, 0.45191E+00_JPRB, 0.45132E+00_JPRB, 0.45029E+00_JPRB, 0.45027E+00_JPRB, 0.44898E+00_JPRB ,&
&  0.44660E+00_JPRB, 0.44324E+00_JPRB, 0.43970E+00_JPRB, 0.43554E+00_JPRB, 0.43028E+00_JPRB, 0.42450E+00_JPRB, 0.41754E+00_JPRB ,&
&  0.41254E+00_JPRB, 0.40548E+00_JPRB, 0.39750E+00_JPRB, 0.38861E+00_JPRB, 0.38246E+00_JPRB, 0.36818E+00_JPRB, 0.36260E+00_JPRB ,&
&  0.35867E+00_JPRB, 0.35270E+00_JPRB, 0.35073E+00_JPRB, 0.35185E+00_JPRB, 0.36666E+00_JPRB, 0.37712E+00_JPRB, 0.38178E+00_JPRB ,&
&  0.37023E+00_JPRB, 0.37032E+00_JPRB, 0.36797E+00_JPRB, 0.36683E+00_JPRB, 0.36944E+00_JPRB, 0.37607E+00_JPRB, 0.39810E+00_JPRB ,&
&  0.43665E+00_JPRB, 0.45106E+00_JPRB, 0.45807E+00_JPRB, 0.46316E+00_JPRB, 0.46432E+00_JPRB, 0.46552E+00_JPRB, 0.46410E+00_JPRB ,&
&  0.46335E+00_JPRB, 0.46171E+00_JPRB, 0.46235E+00_JPRB, 0.46292E+00_JPRB, 0.46442E+00_JPRB, 0.46570E+00_JPRB, 0.46692E+00_JPRB ,&
&  0.46402E+00_JPRB, 0.46191E+00_JPRB, 0.46027E+00_JPRB, 0.46056E+00_JPRB, 0.46238E+00_JPRB, 0.46699E+00_JPRB, 0.47287E+00_JPRB ,&
&  0.47976E+00_JPRB, 0.48515E+00_JPRB, 0.49496E+00_JPRB, 0.50873E+00_JPRB, 0.52528E+00_JPRB, 0.54620E+00_JPRB, 0.56766E+00_JPRB ,&
&  0.58933E+00_JPRB, 0.60771E+00_JPRB, 0.62589E+00_JPRB, 0.63617E+00_JPRB, 0.64285E+00_JPRB, 0.64786E+00_JPRB, 0.64890E+00_JPRB ,&
&  0.64855E+00_JPRB, 0.64606E+00_JPRB, 0.64281E+00_JPRB, 0.63790E+00_JPRB, 0.63133E+00_JPRB, 0.62207E+00_JPRB, 0.61166E+00_JPRB ,&
&  0.59945E+00_JPRB, 0.58741E+00_JPRB, 0.57531E+00_JPRB, 0.56326E+00_JPRB, 0.55141E+00_JPRB, 0.54053E+00_JPRB, 0.53084E+00_JPRB ,&
&  0.52160E+00_JPRB, 0.51252E+00_JPRB, 0.50423E+00_JPRB, 0.49631E+00_JPRB, 0.48834E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.49115E+00_JPRB, 0.48325E+00_JPRB, 0.47492E+00_JPRB, 0.46767E+00_JPRB, 0.46181E+00_JPRB, 0.45773E+00_JPRB, 0.45529E+00_JPRB ,&
&  0.45380E+00_JPRB, 0.45185E+00_JPRB, 0.44976E+00_JPRB, 0.44751E+00_JPRB, 0.44501E+00_JPRB, 0.44496E+00_JPRB, 0.44469E+00_JPRB ,&
&  0.44423E+00_JPRB, 0.44076E+00_JPRB, 0.43537E+00_JPRB, 0.42792E+00_JPRB, 0.42027E+00_JPRB, 0.41410E+00_JPRB, 0.40839E+00_JPRB ,&
&  0.40059E+00_JPRB, 0.39181E+00_JPRB, 0.37638E+00_JPRB, 0.36533E+00_JPRB, 0.35047E+00_JPRB, 0.34164E+00_JPRB, 0.33726E+00_JPRB ,&
&  0.33527E+00_JPRB, 0.33728E+00_JPRB, 0.34201E+00_JPRB, 0.34583E+00_JPRB, 0.34279E+00_JPRB, 0.33764E+00_JPRB, 0.33453E+00_JPRB ,&
&  0.32891E+00_JPRB, 0.32542E+00_JPRB, 0.32301E+00_JPRB, 0.32264E+00_JPRB, 0.32474E+00_JPRB, 0.33147E+00_JPRB, 0.34518E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.41831E+00_JPRB, 0.45654E+00_JPRB, 0.47255E+00_JPRB, 0.46451E+00_JPRB, 0.46545E+00_JPRB, 0.46118E+00_JPRB ,&
&  0.45868E+00_JPRB, 0.45452E+00_JPRB, 0.45264E+00_JPRB, 0.45580E+00_JPRB, 0.46039E+00_JPRB, 0.46501E+00_JPRB, 0.46975E+00_JPRB ,&
&  0.47077E+00_JPRB, 0.47249E+00_JPRB, 0.46805E+00_JPRB, 0.46839E+00_JPRB, 0.47321E+00_JPRB, 0.47825E+00_JPRB, 0.47966E+00_JPRB ,&
&  0.48459E+00_JPRB, 0.49235E+00_JPRB, 0.50935E+00_JPRB, 0.53573E+00_JPRB, 0.56401E+00_JPRB, 0.59058E+00_JPRB, 0.61467E+00_JPRB ,&
&  0.63862E+00_JPRB, 0.66856E+00_JPRB, 0.69762E+00_JPRB, 0.71010E+00_JPRB, 0.71182E+00_JPRB, 0.71719E+00_JPRB, 0.71404E+00_JPRB ,&
&  0.71184E+00_JPRB, 0.70702E+00_JPRB, 0.70562E+00_JPRB, 0.70398E+00_JPRB, 0.69615E+00_JPRB, 0.68205E+00_JPRB, 0.66716E+00_JPRB ,&
&  0.65084E+00_JPRB, 0.63218E+00_JPRB, 0.61428E+00_JPRB, 0.59581E+00_JPRB, 0.57856E+00_JPRB, 0.56163E+00_JPRB, 0.54747E+00_JPRB ,&
&  0.53384E+00_JPRB, 0.52338E+00_JPRB, 0.51379E+00_JPRB, 0.50599E+00_JPRB, 0.49864E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.50391E+00_JPRB, 0.49867E+00_JPRB, 0.49335E+00_JPRB, 0.48655E+00_JPRB, 0.47960E+00_JPRB, 0.47344E+00_JPRB, 0.46885E+00_JPRB ,&
&  0.46542E+00_JPRB, 0.46183E+00_JPRB, 0.45848E+00_JPRB, 0.45416E+00_JPRB, 0.45016E+00_JPRB, 0.44692E+00_JPRB, 0.44434E+00_JPRB ,&
&  0.44323E+00_JPRB, 0.43847E+00_JPRB, 0.43315E+00_JPRB, 0.42629E+00_JPRB, 0.41979E+00_JPRB, 0.41257E+00_JPRB, 0.40532E+00_JPRB ,&
&  0.39906E+00_JPRB, 0.38752E+00_JPRB, 0.37285E+00_JPRB, 0.36336E+00_JPRB, 0.34934E+00_JPRB, 0.34412E+00_JPRB, 0.34418E+00_JPRB ,&
&  0.36115E+00_JPRB, 0.35441E+00_JPRB, 0.35757E+00_JPRB, 0.34545E+00_JPRB, 0.33446E+00_JPRB, 0.32414E+00_JPRB, 0.31718E+00_JPRB ,&
&  0.31160E+00_JPRB, 0.30730E+00_JPRB, 0.30676E+00_JPRB, 0.30853E+00_JPRB, 0.31020E+00_JPRB, 0.31791E+00_JPRB, 0.33095E+00_JPRB ,&
&  0.36640E+00_JPRB, 0.45404E+00_JPRB, 0.46532E+00_JPRB, 0.47917E+00_JPRB, 0.47582E+00_JPRB, 0.47530E+00_JPRB, 0.47527E+00_JPRB ,&
&  0.47367E+00_JPRB, 0.47100E+00_JPRB, 0.46505E+00_JPRB, 0.45959E+00_JPRB, 0.46582E+00_JPRB, 0.47085E+00_JPRB, 0.47848E+00_JPRB ,&
&  0.48140E+00_JPRB, 0.48474E+00_JPRB, 0.48356E+00_JPRB, 0.48431E+00_JPRB, 0.49053E+00_JPRB, 0.49119E+00_JPRB, 0.48792E+00_JPRB ,&
&  0.49581E+00_JPRB, 0.50945E+00_JPRB, 0.53164E+00_JPRB, 0.56066E+00_JPRB, 0.58211E+00_JPRB, 0.60091E+00_JPRB, 0.62889E+00_JPRB ,&
&  0.64903E+00_JPRB, 0.68192E+00_JPRB, 0.71704E+00_JPRB, 0.73377E+00_JPRB, 0.73805E+00_JPRB, 0.73578E+00_JPRB, 0.73140E+00_JPRB ,&
&  0.73200E+00_JPRB, 0.72289E+00_JPRB, 0.71921E+00_JPRB, 0.71774E+00_JPRB, 0.70833E+00_JPRB, 0.69424E+00_JPRB, 0.67951E+00_JPRB ,&
&  0.66164E+00_JPRB, 0.64506E+00_JPRB, 0.63055E+00_JPRB, 0.61377E+00_JPRB, 0.59635E+00_JPRB, 0.57943E+00_JPRB, 0.56304E+00_JPRB ,&
&  0.54707E+00_JPRB, 0.53413E+00_JPRB, 0.52223E+00_JPRB, 0.51463E+00_JPRB, 0.50801E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.51738E+00_JPRB, 0.51237E+00_JPRB, 0.50790E+00_JPRB, 0.50353E+00_JPRB, 0.49841E+00_JPRB, 0.49302E+00_JPRB, 0.48741E+00_JPRB ,&
&  0.48275E+00_JPRB, 0.47835E+00_JPRB, 0.47413E+00_JPRB, 0.46817E+00_JPRB, 0.46228E+00_JPRB, 0.45563E+00_JPRB, 0.45096E+00_JPRB ,&
&  0.44641E+00_JPRB, 0.44038E+00_JPRB, 0.43064E+00_JPRB, 0.42180E+00_JPRB, 0.41556E+00_JPRB, 0.41066E+00_JPRB, 0.40401E+00_JPRB ,&
&  0.39835E+00_JPRB, 0.38981E+00_JPRB, 0.37316E+00_JPRB, 0.35705E+00_JPRB, 0.34445E+00_JPRB, 0.34462E+00_JPRB, 0.37010E+00_JPRB ,&
&  0.38286E+00_JPRB, 0.38627E+00_JPRB, 0.38942E+00_JPRB, 0.38574E+00_JPRB, 0.38045E+00_JPRB, 0.33385E+00_JPRB, 0.32947E+00_JPRB ,&
&  0.31601E+00_JPRB, 0.30631E+00_JPRB, 0.30333E+00_JPRB, 0.30299E+00_JPRB, 0.30698E+00_JPRB, 0.31723E+00_JPRB, 0.33349E+00_JPRB ,&
&  0.36748E+00_JPRB, 0.46450E+00_JPRB, 0.47845E+00_JPRB, 0.49203E+00_JPRB, 0.49268E+00_JPRB, 0.49264E+00_JPRB, 0.49532E+00_JPRB ,&
&  0.49413E+00_JPRB, 0.49509E+00_JPRB, 0.49582E+00_JPRB, 0.49835E+00_JPRB, 0.50104E+00_JPRB, 0.49855E+00_JPRB, 0.50285E+00_JPRB ,&
&  0.50585E+00_JPRB, 0.50974E+00_JPRB, 0.50863E+00_JPRB, 0.50324E+00_JPRB, 0.50556E+00_JPRB, 0.49812E+00_JPRB, 0.49371E+00_JPRB ,&
&  0.51036E+00_JPRB, 0.53991E+00_JPRB, 0.56579E+00_JPRB, 0.59114E+00_JPRB, 0.61267E+00_JPRB, 0.63788E+00_JPRB, 0.66912E+00_JPRB ,&
&  0.69042E+00_JPRB, 0.72079E+00_JPRB, 0.75677E+00_JPRB, 0.76954E+00_JPRB, 0.76899E+00_JPRB, 0.76832E+00_JPRB, 0.77230E+00_JPRB ,&
&  0.77755E+00_JPRB, 0.78429E+00_JPRB, 0.76485E+00_JPRB, 0.75377E+00_JPRB, 0.73036E+00_JPRB, 0.69728E+00_JPRB, 0.67016E+00_JPRB ,&
&  0.63746E+00_JPRB, 0.60913E+00_JPRB, 0.59154E+00_JPRB, 0.57891E+00_JPRB, 0.56939E+00_JPRB, 0.56281E+00_JPRB, 0.55697E+00_JPRB ,&
&  0.55142E+00_JPRB, 0.54430E+00_JPRB, 0.53658E+00_JPRB, 0.52896E+00_JPRB, 0.52257E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.52709E+00_JPRB, 0.52471E+00_JPRB, 0.52063E+00_JPRB, 0.51654E+00_JPRB, 0.51043E+00_JPRB, 0.50565E+00_JPRB, 0.50098E+00_JPRB ,&
&  0.49770E+00_JPRB, 0.49542E+00_JPRB, 0.49330E+00_JPRB, 0.49063E+00_JPRB, 0.48702E+00_JPRB, 0.48368E+00_JPRB, 0.47867E+00_JPRB ,&
&  0.47024E+00_JPRB, 0.44445E+00_JPRB, 0.42544E+00_JPRB, 0.41838E+00_JPRB, 0.41243E+00_JPRB, 0.40809E+00_JPRB, 0.40514E+00_JPRB ,&
&  0.39784E+00_JPRB, 0.38961E+00_JPRB, 0.37771E+00_JPRB, 0.35931E+00_JPRB, 0.35115E+00_JPRB, 0.35546E+00_JPRB, 0.36397E+00_JPRB ,&
&  0.37876E+00_JPRB, 0.38757E+00_JPRB, 0.38196E+00_JPRB, 0.38298E+00_JPRB, 0.40014E+00_JPRB, 0.39433E+00_JPRB, 0.34563E+00_JPRB ,&
&  0.32228E+00_JPRB, 0.30783E+00_JPRB, 0.30256E+00_JPRB, 0.30123E+00_JPRB, 0.30561E+00_JPRB, 0.32006E+00_JPRB, 0.35854E+00_JPRB ,&
&  0.43290E+00_JPRB, 0.51410E+00_JPRB, 0.50451E+00_JPRB, 0.50179E+00_JPRB, 0.50710E+00_JPRB, 0.51046E+00_JPRB, 0.51401E+00_JPRB ,&
&  0.51701E+00_JPRB, 0.52412E+00_JPRB, 0.53369E+00_JPRB, 0.54229E+00_JPRB, 0.55288E+00_JPRB, 0.56228E+00_JPRB, 0.56453E+00_JPRB ,&
&  0.56809E+00_JPRB, 0.55233E+00_JPRB, 0.54419E+00_JPRB, 0.53169E+00_JPRB, 0.52572E+00_JPRB, 0.51234E+00_JPRB, 0.51407E+00_JPRB ,&
&  0.53574E+00_JPRB, 0.56424E+00_JPRB, 0.59424E+00_JPRB, 0.61800E+00_JPRB, 0.64330E+00_JPRB, 0.67847E+00_JPRB, 0.71672E+00_JPRB ,&
&  0.75039E+00_JPRB, 0.79592E+00_JPRB, 0.86275E+00_JPRB, 0.85214E+00_JPRB, 0.83911E+00_JPRB, 0.83326E+00_JPRB, 0.83725E+00_JPRB ,&
&  0.83887E+00_JPRB, 0.84995E+00_JPRB, 0.81622E+00_JPRB, 0.76380E+00_JPRB, 0.71868E+00_JPRB, 0.66263E+00_JPRB, 0.62799E+00_JPRB ,&
&  0.60590E+00_JPRB, 0.57284E+00_JPRB, 0.55647E+00_JPRB, 0.54769E+00_JPRB, 0.54259E+00_JPRB, 0.53603E+00_JPRB, 0.53043E+00_JPRB ,&
&  0.52854E+00_JPRB, 0.52824E+00_JPRB, 0.52937E+00_JPRB, 0.52890E+00_JPRB, 0.52835E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.51937E+00_JPRB, 0.52403E+00_JPRB, 0.52753E+00_JPRB, 0.52660E+00_JPRB, 0.51526E+00_JPRB, 0.48997E+00_JPRB, 0.47692E+00_JPRB ,&
&  0.46949E+00_JPRB, 0.46984E+00_JPRB, 0.46836E+00_JPRB, 0.46431E+00_JPRB, 0.46343E+00_JPRB, 0.46786E+00_JPRB, 0.47313E+00_JPRB ,&
&  0.47492E+00_JPRB, 0.46985E+00_JPRB, 0.45991E+00_JPRB, 0.44088E+00_JPRB, 0.42470E+00_JPRB, 0.41167E+00_JPRB, 0.40545E+00_JPRB ,&
&  0.40439E+00_JPRB, 0.39978E+00_JPRB, 0.39026E+00_JPRB, 0.37707E+00_JPRB, 0.36660E+00_JPRB, 0.35710E+00_JPRB, 0.35891E+00_JPRB ,&
&  0.36851E+00_JPRB, 0.37386E+00_JPRB, 0.38270E+00_JPRB, 0.38817E+00_JPRB, 0.39691E+00_JPRB, 0.42078E+00_JPRB, 0.37128E+00_JPRB ,&
&  0.33753E+00_JPRB, 0.32281E+00_JPRB, 0.31146E+00_JPRB, 0.30831E+00_JPRB, 0.31443E+00_JPRB, 0.33508E+00_JPRB, 0.38034E+00_JPRB ,&
&  0.49491E+00_JPRB, 0.50896E+00_JPRB, 0.51390E+00_JPRB, 0.52542E+00_JPRB, 0.53449E+00_JPRB, 0.53733E+00_JPRB, 0.54408E+00_JPRB ,&
&  0.55769E+00_JPRB, 0.58048E+00_JPRB, 0.60303E+00_JPRB, 0.60116E+00_JPRB, 0.59392E+00_JPRB, 0.57460E+00_JPRB, 0.62226E+00_JPRB ,&
&  0.77458E+00_JPRB, 0.72198E+00_JPRB, 0.64166E+00_JPRB, 0.58315E+00_JPRB, 0.55223E+00_JPRB, 0.53751E+00_JPRB, 0.54911E+00_JPRB ,&
&  0.56936E+00_JPRB, 0.59555E+00_JPRB, 0.63402E+00_JPRB, 0.66296E+00_JPRB, 0.69103E+00_JPRB, 0.73307E+00_JPRB, 0.79351E+00_JPRB ,&
&  0.85345E+00_JPRB, 0.93535E+00_JPRB, 0.11331E+01_JPRB, 0.98924E+00_JPRB, 0.91385E+00_JPRB, 0.89114E+00_JPRB, 0.88520E+00_JPRB ,&
&  0.86085E+00_JPRB, 0.86455E+00_JPRB, 0.80972E+00_JPRB, 0.73333E+00_JPRB, 0.69254E+00_JPRB, 0.64759E+00_JPRB, 0.61565E+00_JPRB ,&
&  0.58517E+00_JPRB, 0.55913E+00_JPRB, 0.54770E+00_JPRB, 0.53757E+00_JPRB, 0.53541E+00_JPRB, 0.53227E+00_JPRB, 0.52746E+00_JPRB ,&
&  0.52126E+00_JPRB, 0.51623E+00_JPRB, 0.51260E+00_JPRB, 0.51121E+00_JPRB, 0.51289E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.49211E+00_JPRB, 0.51915E+00_JPRB, 0.54635E+00_JPRB, 0.53456E+00_JPRB, 0.51676E+00_JPRB, 0.46800E+00_JPRB, 0.43303E+00_JPRB ,&
&  0.41143E+00_JPRB, 0.40843E+00_JPRB, 0.40832E+00_JPRB, 0.41066E+00_JPRB, 0.41580E+00_JPRB, 0.42455E+00_JPRB, 0.42998E+00_JPRB ,&
&  0.43518E+00_JPRB, 0.43214E+00_JPRB, 0.43085E+00_JPRB, 0.42553E+00_JPRB, 0.41591E+00_JPRB, 0.40336E+00_JPRB, 0.39448E+00_JPRB ,&
&  0.39061E+00_JPRB, 0.39007E+00_JPRB, 0.38699E+00_JPRB, 0.38140E+00_JPRB, 0.37703E+00_JPRB, 0.37039E+00_JPRB, 0.37508E+00_JPRB ,&
&  0.36960E+00_JPRB, 0.36795E+00_JPRB, 0.37980E+00_JPRB, 0.39441E+00_JPRB, 0.40511E+00_JPRB, 0.42804E+00_JPRB, 0.39343E+00_JPRB ,&
&  0.35406E+00_JPRB, 0.33561E+00_JPRB, 0.32810E+00_JPRB, 0.33347E+00_JPRB, 0.37212E+00_JPRB, 0.47798E+00_JPRB, 0.51017E+00_JPRB ,&
&  0.52627E+00_JPRB, 0.53930E+00_JPRB, 0.55431E+00_JPRB, 0.57421E+00_JPRB, 0.57832E+00_JPRB, 0.58286E+00_JPRB, 0.59837E+00_JPRB ,&
&  0.62222E+00_JPRB, 0.65996E+00_JPRB, 0.69784E+00_JPRB, 0.60385E+00_JPRB, 0.56364E+00_JPRB, 0.57947E+00_JPRB, 0.60478E+00_JPRB ,&
&  0.64027E+00_JPRB, 0.63116E+00_JPRB, 0.62111E+00_JPRB, 0.63659E+00_JPRB, 0.63569E+00_JPRB, 0.63578E+00_JPRB, 0.62617E+00_JPRB ,&
&  0.62623E+00_JPRB, 0.64598E+00_JPRB, 0.70216E+00_JPRB, 0.73222E+00_JPRB, 0.78068E+00_JPRB, 0.85516E+00_JPRB, 0.92129E+00_JPRB ,&
&  0.99580E+00_JPRB, 0.10502E+01_JPRB, 0.10747E+01_JPRB, 0.10348E+01_JPRB, 0.10051E+01_JPRB, 0.10039E+01_JPRB, 0.98549E+00_JPRB ,&
&  0.91489E+00_JPRB, 0.86557E+00_JPRB, 0.82427E+00_JPRB, 0.78125E+00_JPRB, 0.71873E+00_JPRB, 0.66740E+00_JPRB, 0.61428E+00_JPRB ,&
&  0.56249E+00_JPRB, 0.52489E+00_JPRB, 0.50744E+00_JPRB, 0.49863E+00_JPRB, 0.49778E+00_JPRB, 0.49719E+00_JPRB, 0.49030E+00_JPRB ,&
&  0.47811E+00_JPRB, 0.46445E+00_JPRB, 0.46004E+00_JPRB, 0.46580E+00_JPRB, 0.48238E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.54537E+00_JPRB, 0.54463E+00_JPRB, 0.51580E+00_JPRB, 0.52956E+00_JPRB, 0.50585E+00_JPRB, 0.48922E+00_JPRB, 0.46150E+00_JPRB ,&
&  0.44903E+00_JPRB, 0.42344E+00_JPRB, 0.40716E+00_JPRB, 0.38247E+00_JPRB, 0.36743E+00_JPRB, 0.35738E+00_JPRB, 0.36774E+00_JPRB ,&
&  0.38341E+00_JPRB, 0.39398E+00_JPRB, 0.39917E+00_JPRB, 0.39811E+00_JPRB, 0.39553E+00_JPRB, 0.38973E+00_JPRB, 0.38390E+00_JPRB ,&
&  0.37946E+00_JPRB, 0.38176E+00_JPRB, 0.38079E+00_JPRB, 0.37969E+00_JPRB, 0.37764E+00_JPRB, 0.37785E+00_JPRB, 0.38063E+00_JPRB ,&
&  0.37402E+00_JPRB, 0.36748E+00_JPRB, 0.37140E+00_JPRB, 0.39158E+00_JPRB, 0.40738E+00_JPRB, 0.42747E+00_JPRB, 0.40091E+00_JPRB ,&
&  0.36420E+00_JPRB, 0.35646E+00_JPRB, 0.36978E+00_JPRB, 0.46988E+00_JPRB, 0.48896E+00_JPRB, 0.49496E+00_JPRB, 0.52033E+00_JPRB ,&
&  0.49956E+00_JPRB, 0.49539E+00_JPRB, 0.51138E+00_JPRB, 0.58154E+00_JPRB, 0.60443E+00_JPRB, 0.61724E+00_JPRB, 0.63974E+00_JPRB ,&
&  0.67114E+00_JPRB, 0.71678E+00_JPRB, 0.76986E+00_JPRB, 0.68053E+00_JPRB, 0.71234E+00_JPRB, 0.84224E+00_JPRB, 0.74881E+00_JPRB ,&
&  0.70397E+00_JPRB, 0.67770E+00_JPRB, 0.69504E+00_JPRB, 0.69927E+00_JPRB, 0.69043E+00_JPRB, 0.70666E+00_JPRB, 0.71701E+00_JPRB ,&
&  0.69828E+00_JPRB, 0.72362E+00_JPRB, 0.79643E+00_JPRB, 0.87191E+00_JPRB, 0.93968E+00_JPRB, 0.10292E+01_JPRB, 0.11228E+01_JPRB ,&
&  0.12397E+01_JPRB, 0.13465E+01_JPRB, 0.14234E+01_JPRB, 0.14124E+01_JPRB, 0.13771E+01_JPRB, 0.13102E+01_JPRB, 0.12141E+01_JPRB ,&
&  0.10811E+01_JPRB, 0.98545E+00_JPRB, 0.92024E+00_JPRB, 0.86852E+00_JPRB, 0.80438E+00_JPRB, 0.72093E+00_JPRB, 0.61368E+00_JPRB ,&
&  0.53246E+00_JPRB, 0.47978E+00_JPRB, 0.45895E+00_JPRB, 0.45440E+00_JPRB, 0.45883E+00_JPRB, 0.46256E+00_JPRB, 0.45855E+00_JPRB ,&
&  0.45047E+00_JPRB, 0.44599E+00_JPRB, 0.44679E+00_JPRB, 0.46419E+00_JPRB, 0.51125E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.55674E+00_JPRB, 0.54623E+00_JPRB, 0.53286E+00_JPRB, 0.52972E+00_JPRB, 0.53754E+00_JPRB, 0.51086E+00_JPRB, 0.45556E+00_JPRB ,&
&  0.47204E+00_JPRB, 0.42437E+00_JPRB, 0.41130E+00_JPRB, 0.39894E+00_JPRB, 0.38836E+00_JPRB, 0.36537E+00_JPRB, 0.34483E+00_JPRB ,&
&  0.34470E+00_JPRB, 0.35704E+00_JPRB, 0.37191E+00_JPRB, 0.38351E+00_JPRB, 0.38631E+00_JPRB, 0.38239E+00_JPRB, 0.37748E+00_JPRB ,&
&  0.37431E+00_JPRB, 0.37597E+00_JPRB, 0.38035E+00_JPRB, 0.38658E+00_JPRB, 0.38837E+00_JPRB, 0.38986E+00_JPRB, 0.39445E+00_JPRB ,&
&  0.39769E+00_JPRB, 0.39875E+00_JPRB, 0.39100E+00_JPRB, 0.40029E+00_JPRB, 0.40670E+00_JPRB, 0.42145E+00_JPRB, 0.44998E+00_JPRB ,&
&  0.38584E+00_JPRB, 0.39064E+00_JPRB, 0.48321E+00_JPRB, 0.48616E+00_JPRB, 0.49810E+00_JPRB, 0.51144E+00_JPRB, 0.52137E+00_JPRB ,&
&  0.53663E+00_JPRB, 0.56149E+00_JPRB, 0.59203E+00_JPRB, 0.61924E+00_JPRB, 0.64239E+00_JPRB, 0.67216E+00_JPRB, 0.69591E+00_JPRB ,&
&  0.76273E+00_JPRB, 0.68394E+00_JPRB, 0.64194E+00_JPRB, 0.68250E+00_JPRB, 0.94708E+00_JPRB, 0.95107E+00_JPRB, 0.91547E+00_JPRB ,&
&  0.88240E+00_JPRB, 0.82523E+00_JPRB, 0.80062E+00_JPRB, 0.80945E+00_JPRB, 0.81596E+00_JPRB, 0.85255E+00_JPRB, 0.90354E+00_JPRB ,&
&  0.89767E+00_JPRB, 0.94978E+00_JPRB, 0.10229E+01_JPRB, 0.10907E+01_JPRB, 0.11645E+01_JPRB, 0.12756E+01_JPRB, 0.14021E+01_JPRB ,&
&  0.15414E+01_JPRB, 0.16628E+01_JPRB, 0.17187E+01_JPRB, 0.16950E+01_JPRB, 0.16192E+01_JPRB, 0.15184E+01_JPRB, 0.13922E+01_JPRB ,&
&  0.12282E+01_JPRB, 0.11106E+01_JPRB, 0.10308E+01_JPRB, 0.95321E+00_JPRB, 0.88317E+00_JPRB, 0.79123E+00_JPRB, 0.65864E+00_JPRB ,&
&  0.52219E+00_JPRB, 0.46262E+00_JPRB, 0.45309E+00_JPRB, 0.45214E+00_JPRB, 0.45576E+00_JPRB, 0.46274E+00_JPRB, 0.47048E+00_JPRB ,&
&  0.47396E+00_JPRB, 0.46764E+00_JPRB, 0.46958E+00_JPRB, 0.49214E+00_JPRB, 0.52073E+00_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.53034E+00_JPRB, 0.52395E+00_JPRB, 0.52223E+00_JPRB, 0.54080E+00_JPRB, 0.54438E+00_JPRB, 0.51613E+00_JPRB, 0.47850E+00_JPRB ,&
&  0.58470E+00_JPRB, 0.53612E+00_JPRB, 0.59760E+00_JPRB, 0.46311E+00_JPRB, 0.40842E+00_JPRB, 0.39323E+00_JPRB, 0.37562E+00_JPRB ,&
&  0.36194E+00_JPRB, 0.35640E+00_JPRB, 0.35586E+00_JPRB, 0.36468E+00_JPRB, 0.36990E+00_JPRB, 0.36803E+00_JPRB, 0.36562E+00_JPRB ,&
&  0.36449E+00_JPRB, 0.36817E+00_JPRB, 0.37797E+00_JPRB, 0.38446E+00_JPRB, 0.38977E+00_JPRB, 0.39533E+00_JPRB, 0.40530E+00_JPRB ,&
&  0.39400E+00_JPRB, 0.39195E+00_JPRB, 0.39892E+00_JPRB, 0.40305E+00_JPRB, 0.41256E+00_JPRB, 0.41804E+00_JPRB, 0.43411E+00_JPRB ,&
&  0.48028E+00_JPRB, 0.44908E+00_JPRB, 0.52316E+00_JPRB, 0.52688E+00_JPRB, 0.53506E+00_JPRB, 0.54417E+00_JPRB, 0.55254E+00_JPRB ,&
&  0.56110E+00_JPRB, 0.57889E+00_JPRB, 0.60632E+00_JPRB, 0.63080E+00_JPRB, 0.66734E+00_JPRB, 0.71231E+00_JPRB, 0.74387E+00_JPRB ,&
&  0.83616E+00_JPRB, 0.67726E+00_JPRB, 0.73644E+00_JPRB, 0.76555E+00_JPRB, 0.96526E+00_JPRB, 0.11876E+01_JPRB, 0.11238E+01_JPRB ,&
&  0.12130E+01_JPRB, 0.11059E+01_JPRB, 0.10192E+01_JPRB, 0.95393E+00_JPRB, 0.96399E+00_JPRB, 0.10049E+01_JPRB, 0.10598E+01_JPRB ,&
&  0.11074E+01_JPRB, 0.11618E+01_JPRB, 0.12628E+01_JPRB, 0.13528E+01_JPRB, 0.14471E+01_JPRB, 0.15579E+01_JPRB, 0.16655E+01_JPRB ,&
&  0.17642E+01_JPRB, 0.18267E+01_JPRB, 0.18600E+01_JPRB, 0.17580E+01_JPRB, 0.16676E+01_JPRB, 0.15697E+01_JPRB, 0.13806E+01_JPRB ,&
&  0.12334E+01_JPRB, 0.11202E+01_JPRB, 0.10277E+01_JPRB, 0.95820E+00_JPRB, 0.83906E+00_JPRB, 0.74005E+00_JPRB, 0.64619E+00_JPRB ,&
&  0.52467E+00_JPRB, 0.48132E+00_JPRB, 0.49366E+00_JPRB, 0.49903E+00_JPRB, 0.49415E+00_JPRB, 0.49746E+00_JPRB, 0.50585E+00_JPRB ,&
&  0.49778E+00_JPRB, 0.49935E+00_JPRB, 0.52107E+00_JPRB, 0.52835E+00_JPRB, 0.53057E+00_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.54834E+00_JPRB, 0.55455E+00_JPRB, 0.56084E+00_JPRB, 0.56767E+00_JPRB, 0.55361E+00_JPRB, 0.57318E+00_JPRB, 0.51727E+00_JPRB ,&
&  0.57141E+00_JPRB, 0.56941E+00_JPRB, 0.55037E+00_JPRB, 0.56546E+00_JPRB, 0.60959E+00_JPRB, 0.57409E+00_JPRB, 0.44402E+00_JPRB ,&
&  0.38844E+00_JPRB, 0.38192E+00_JPRB, 0.38469E+00_JPRB, 0.38606E+00_JPRB, 0.38742E+00_JPRB, 0.38619E+00_JPRB, 0.38628E+00_JPRB ,&
&  0.38678E+00_JPRB, 0.38635E+00_JPRB, 0.39231E+00_JPRB, 0.39867E+00_JPRB, 0.40174E+00_JPRB, 0.40537E+00_JPRB, 0.41652E+00_JPRB ,&
&  0.40165E+00_JPRB, 0.39163E+00_JPRB, 0.38739E+00_JPRB, 0.37707E+00_JPRB, 0.39619E+00_JPRB, 0.41197E+00_JPRB, 0.43860E+00_JPRB ,&
&  0.47795E+00_JPRB, 0.51819E+00_JPRB, 0.53789E+00_JPRB, 0.55399E+00_JPRB, 0.56805E+00_JPRB, 0.58348E+00_JPRB, 0.60012E+00_JPRB ,&
&  0.61590E+00_JPRB, 0.63517E+00_JPRB, 0.66276E+00_JPRB, 0.68368E+00_JPRB, 0.74445E+00_JPRB, 0.73848E+00_JPRB, 0.83602E+00_JPRB ,&
&  0.89357E+00_JPRB, 0.10135E+01_JPRB, 0.10894E+01_JPRB, 0.94750E+00_JPRB, 0.10938E+01_JPRB, 0.12523E+01_JPRB, 0.12150E+01_JPRB ,&
&  0.12063E+01_JPRB, 0.11600E+01_JPRB, 0.11660E+01_JPRB, 0.11963E+01_JPRB, 0.13039E+01_JPRB, 0.13832E+01_JPRB, 0.14180E+01_JPRB ,&
&  0.14500E+01_JPRB, 0.16008E+01_JPRB, 0.17063E+01_JPRB, 0.17791E+01_JPRB, 0.18158E+01_JPRB, 0.18468E+01_JPRB, 0.18956E+01_JPRB ,&
&  0.19312E+01_JPRB, 0.19603E+01_JPRB, 0.19490E+01_JPRB, 0.18817E+01_JPRB, 0.17828E+01_JPRB, 0.16470E+01_JPRB, 0.14647E+01_JPRB ,&
&  0.13012E+01_JPRB, 0.11001E+01_JPRB, 0.93812E+00_JPRB, 0.83750E+00_JPRB, 0.74376E+00_JPRB, 0.65272E+00_JPRB, 0.58889E+00_JPRB ,&
&  0.52476E+00_JPRB, 0.52002E+00_JPRB, 0.57064E+00_JPRB, 0.56602E+00_JPRB, 0.56598E+00_JPRB, 0.56652E+00_JPRB, 0.52569E+00_JPRB ,&
&  0.51193E+00_JPRB, 0.54072E+00_JPRB, 0.54781E+00_JPRB, 0.54747E+00_JPRB, 0.54654E+00_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.60026E+00_JPRB, 0.59919E+00_JPRB, 0.60248E+00_JPRB, 0.59442E+00_JPRB, 0.58370E+00_JPRB, 0.58178E+00_JPRB, 0.58185E+00_JPRB ,&
&  0.57579E+00_JPRB, 0.57651E+00_JPRB, 0.57465E+00_JPRB, 0.56688E+00_JPRB, 0.56611E+00_JPRB, 0.58999E+00_JPRB, 0.53420E+00_JPRB ,&
&  0.44420E+00_JPRB, 0.42286E+00_JPRB, 0.41443E+00_JPRB, 0.42266E+00_JPRB, 0.43676E+00_JPRB, 0.44158E+00_JPRB, 0.44372E+00_JPRB ,&
&  0.44792E+00_JPRB, 0.44165E+00_JPRB, 0.44478E+00_JPRB, 0.44632E+00_JPRB, 0.44395E+00_JPRB, 0.44584E+00_JPRB, 0.44797E+00_JPRB ,&
&  0.42029E+00_JPRB, 0.40064E+00_JPRB, 0.39022E+00_JPRB, 0.38420E+00_JPRB, 0.39845E+00_JPRB, 0.42508E+00_JPRB, 0.46517E+00_JPRB ,&
&  0.51265E+00_JPRB, 0.54726E+00_JPRB, 0.57745E+00_JPRB, 0.60796E+00_JPRB, 0.63279E+00_JPRB, 0.65988E+00_JPRB, 0.68481E+00_JPRB ,&
&  0.70483E+00_JPRB, 0.72201E+00_JPRB, 0.73258E+00_JPRB, 0.75074E+00_JPRB, 0.75301E+00_JPRB, 0.84579E+00_JPRB, 0.10304E+01_JPRB ,&
&  0.10836E+01_JPRB, 0.11768E+01_JPRB, 0.12152E+01_JPRB, 0.13139E+01_JPRB, 0.13934E+01_JPRB, 0.13523E+01_JPRB, 0.13519E+01_JPRB ,&
&  0.14260E+01_JPRB, 0.15109E+01_JPRB, 0.17215E+01_JPRB, 0.17285E+01_JPRB, 0.17604E+01_JPRB, 0.18128E+01_JPRB, 0.17978E+01_JPRB ,&
&  0.18411E+01_JPRB, 0.19038E+01_JPRB, 0.20094E+01_JPRB, 0.20198E+01_JPRB, 0.19943E+01_JPRB, 0.20121E+01_JPRB, 0.20106E+01_JPRB ,&
&  0.20291E+01_JPRB, 0.20455E+01_JPRB, 0.19526E+01_JPRB, 0.18660E+01_JPRB, 0.17045E+01_JPRB, 0.16961E+01_JPRB, 0.15708E+01_JPRB ,&
&  0.13176E+01_JPRB, 0.98895E+00_JPRB, 0.89479E+00_JPRB, 0.79101E+00_JPRB, 0.71984E+00_JPRB, 0.63708E+00_JPRB, 0.57204E+00_JPRB ,&
&  0.54445E+00_JPRB, 0.58236E+00_JPRB, 0.58607E+00_JPRB, 0.59576E+00_JPRB, 0.60604E+00_JPRB, 0.61612E+00_JPRB, 0.54516E+00_JPRB ,&
&  0.55182E+00_JPRB, 0.59846E+00_JPRB, 0.59777E+00_JPRB, 0.60076E+00_JPRB, 0.60022E+00_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.67003E+00_JPRB, 0.66107E+00_JPRB, 0.65331E+00_JPRB, 0.64378E+00_JPRB, 0.63139E+00_JPRB, 0.62103E+00_JPRB, 0.61062E+00_JPRB ,&
&  0.60380E+00_JPRB, 0.60751E+00_JPRB, 0.61088E+00_JPRB, 0.59719E+00_JPRB, 0.58423E+00_JPRB, 0.58140E+00_JPRB, 0.61845E+00_JPRB ,&
&  0.49383E+00_JPRB, 0.47572E+00_JPRB, 0.47619E+00_JPRB, 0.50475E+00_JPRB, 0.57542E+00_JPRB, 0.58299E+00_JPRB, 0.60027E+00_JPRB ,&
&  0.59179E+00_JPRB, 0.57913E+00_JPRB, 0.57184E+00_JPRB, 0.56349E+00_JPRB, 0.54660E+00_JPRB, 0.53708E+00_JPRB, 0.52129E+00_JPRB ,&
&  0.47448E+00_JPRB, 0.43790E+00_JPRB, 0.42513E+00_JPRB, 0.42099E+00_JPRB, 0.44557E+00_JPRB, 0.47425E+00_JPRB, 0.52800E+00_JPRB ,&
&  0.57715E+00_JPRB, 0.62243E+00_JPRB, 0.66622E+00_JPRB, 0.71951E+00_JPRB, 0.76302E+00_JPRB, 0.80459E+00_JPRB, 0.83073E+00_JPRB ,&
&  0.84239E+00_JPRB, 0.84267E+00_JPRB, 0.85464E+00_JPRB, 0.83799E+00_JPRB, 0.84019E+00_JPRB, 0.10558E+01_JPRB, 0.13281E+01_JPRB ,&
&  0.14003E+01_JPRB, 0.14635E+01_JPRB, 0.17835E+01_JPRB, 0.21761E+01_JPRB, 0.20545E+01_JPRB, 0.18846E+01_JPRB, 0.18176E+01_JPRB ,&
&  0.19075E+01_JPRB, 0.20486E+01_JPRB, 0.21974E+01_JPRB, 0.22372E+01_JPRB, 0.22162E+01_JPRB, 0.22621E+01_JPRB, 0.22698E+01_JPRB ,&
&  0.22526E+01_JPRB, 0.22739E+01_JPRB, 0.22411E+01_JPRB, 0.21715E+01_JPRB, 0.21142E+01_JPRB, 0.21519E+01_JPRB, 0.21467E+01_JPRB ,&
&  0.21116E+01_JPRB, 0.20207E+01_JPRB, 0.18659E+01_JPRB, 0.17811E+01_JPRB, 0.15032E+01_JPRB, 0.14560E+01_JPRB, 0.16122E+01_JPRB ,&
&  0.13325E+01_JPRB, 0.10540E+01_JPRB, 0.90291E+00_JPRB, 0.80553E+00_JPRB, 0.75997E+00_JPRB, 0.72500E+00_JPRB, 0.66157E+00_JPRB ,&
&  0.59302E+00_JPRB, 0.61409E+00_JPRB, 0.66977E+00_JPRB, 0.70305E+00_JPRB, 0.71245E+00_JPRB, 0.72415E+00_JPRB, 0.66305E+00_JPRB ,&
&  0.70001E+00_JPRB, 0.68139E+00_JPRB, 0.67481E+00_JPRB, 0.67739E+00_JPRB, 0.67478E+00_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.74053E+00_JPRB, 0.71716E+00_JPRB, 0.70200E+00_JPRB, 0.67733E+00_JPRB, 0.65672E+00_JPRB, 0.64548E+00_JPRB, 0.63812E+00_JPRB ,&
&  0.63829E+00_JPRB, 0.63630E+00_JPRB, 0.63284E+00_JPRB, 0.61634E+00_JPRB, 0.60499E+00_JPRB, 0.59669E+00_JPRB, 0.61358E+00_JPRB ,&
&  0.61416E+00_JPRB, 0.55097E+00_JPRB, 0.54436E+00_JPRB, 0.55426E+00_JPRB, 0.66313E+00_JPRB, 0.75187E+00_JPRB, 0.82547E+00_JPRB ,&
&  0.84745E+00_JPRB, 0.85882E+00_JPRB, 0.84735E+00_JPRB, 0.80653E+00_JPRB, 0.76134E+00_JPRB, 0.72050E+00_JPRB, 0.67165E+00_JPRB ,&
&  0.61227E+00_JPRB, 0.55373E+00_JPRB, 0.52592E+00_JPRB, 0.53162E+00_JPRB, 0.58264E+00_JPRB, 0.56210E+00_JPRB, 0.62024E+00_JPRB ,&
&  0.67777E+00_JPRB, 0.74281E+00_JPRB, 0.79961E+00_JPRB, 0.87213E+00_JPRB, 0.92225E+00_JPRB, 0.95925E+00_JPRB, 0.96550E+00_JPRB ,&
&  0.98055E+00_JPRB, 0.95175E+00_JPRB, 0.84568E+00_JPRB, 0.88057E+00_JPRB, 0.97399E+00_JPRB, 0.13603E+01_JPRB, 0.17537E+01_JPRB ,&
&  0.19853E+01_JPRB, 0.23404E+01_JPRB, 0.27772E+01_JPRB, 0.29776E+01_JPRB, 0.26580E+01_JPRB, 0.24514E+01_JPRB, 0.24579E+01_JPRB ,&
&  0.25847E+01_JPRB, 0.27362E+01_JPRB, 0.27829E+01_JPRB, 0.26547E+01_JPRB, 0.26720E+01_JPRB, 0.27182E+01_JPRB, 0.27557E+01_JPRB ,&
&  0.27230E+01_JPRB, 0.26149E+01_JPRB, 0.24794E+01_JPRB, 0.23513E+01_JPRB, 0.22666E+01_JPRB, 0.21003E+01_JPRB, 0.20483E+01_JPRB ,&
&  0.20695E+01_JPRB, 0.17649E+01_JPRB, 0.15912E+01_JPRB, 0.16474E+01_JPRB, 0.13641E+01_JPRB, 0.13337E+01_JPRB, 0.14256E+01_JPRB ,&
&  0.12175E+01_JPRB, 0.10211E+01_JPRB, 0.91452E+00_JPRB, 0.81380E+00_JPRB, 0.78265E+00_JPRB, 0.82772E+00_JPRB, 0.83549E+00_JPRB ,&
&  0.78953E+00_JPRB, 0.77008E+00_JPRB, 0.89400E+00_JPRB, 0.95432E+00_JPRB, 0.98424E+00_JPRB, 0.95369E+00_JPRB, 0.90910E+00_JPRB ,&
&  0.86634E+00_JPRB, 0.83350E+00_JPRB, 0.81298E+00_JPRB, 0.79226E+00_JPRB, 0.77011E+00_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.80935E+00_JPRB, 0.80055E+00_JPRB, 0.77173E+00_JPRB, 0.73015E+00_JPRB, 0.70230E+00_JPRB, 0.69082E+00_JPRB, 0.67159E+00_JPRB ,&
&  0.64893E+00_JPRB, 0.63274E+00_JPRB, 0.62792E+00_JPRB, 0.61559E+00_JPRB, 0.61153E+00_JPRB, 0.61492E+00_JPRB, 0.61561E+00_JPRB ,&
&  0.60878E+00_JPRB, 0.61369E+00_JPRB, 0.58319E+00_JPRB, 0.59805E+00_JPRB, 0.69449E+00_JPRB, 0.96358E+00_JPRB, 0.11539E+01_JPRB ,&
&  0.12266E+01_JPRB, 0.12338E+01_JPRB, 0.12104E+01_JPRB, 0.11962E+01_JPRB, 0.11419E+01_JPRB, 0.10390E+01_JPRB, 0.97575E+00_JPRB ,&
&  0.90808E+00_JPRB, 0.82468E+00_JPRB, 0.72859E+00_JPRB, 0.74163E+00_JPRB, 0.76349E+00_JPRB, 0.72279E+00_JPRB, 0.81212E+00_JPRB ,&
&  0.88014E+00_JPRB, 0.92907E+00_JPRB, 0.97878E+00_JPRB, 0.10245E+01_JPRB, 0.10453E+01_JPRB, 0.98044E+00_JPRB, 0.89407E+00_JPRB ,&
&  0.85597E+00_JPRB, 0.88742E+00_JPRB, 0.91555E+00_JPRB, 0.10330E+01_JPRB, 0.13865E+01_JPRB, 0.19958E+01_JPRB, 0.24915E+01_JPRB ,&
&  0.28086E+01_JPRB, 0.29103E+01_JPRB, 0.26934E+01_JPRB, 0.28890E+01_JPRB, 0.32263E+01_JPRB, 0.31110E+01_JPRB, 0.29463E+01_JPRB ,&
&  0.29550E+01_JPRB, 0.29415E+01_JPRB, 0.28797E+01_JPRB, 0.29105E+01_JPRB, 0.31337E+01_JPRB, 0.32536E+01_JPRB, 0.32440E+01_JPRB ,&
&  0.28336E+01_JPRB, 0.26777E+01_JPRB, 0.25969E+01_JPRB, 0.24796E+01_JPRB, 0.22111E+01_JPRB, 0.20985E+01_JPRB, 0.21332E+01_JPRB ,&
&  0.20674E+01_JPRB, 0.20183E+01_JPRB, 0.16039E+01_JPRB, 0.15018E+01_JPRB, 0.11701E+01_JPRB, 0.11308E+01_JPRB, 0.12583E+01_JPRB ,&
&  0.11538E+01_JPRB, 0.10528E+01_JPRB, 0.91431E+00_JPRB, 0.85109E+00_JPRB, 0.95583E+00_JPRB, 0.10637E+01_JPRB, 0.12221E+01_JPRB ,&
&  0.12070E+01_JPRB, 0.10736E+01_JPRB, 0.12179E+01_JPRB, 0.12545E+01_JPRB, 0.13091E+01_JPRB, 0.12454E+01_JPRB, 0.11321E+01_JPRB ,&
&  0.10450E+01_JPRB, 0.98313E+00_JPRB, 0.92042E+00_JPRB, 0.86186E+00_JPRB, 0.82717E+00_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.81828E+00_JPRB, 0.79952E+00_JPRB, 0.76461E+00_JPRB, 0.72529E+00_JPRB, 0.70094E+00_JPRB, 0.68229E+00_JPRB, 0.66758E+00_JPRB ,&
&  0.65527E+00_JPRB, 0.64123E+00_JPRB, 0.63193E+00_JPRB, 0.62099E+00_JPRB, 0.61077E+00_JPRB, 0.60261E+00_JPRB, 0.59598E+00_JPRB ,&
&  0.59740E+00_JPRB, 0.63719E+00_JPRB, 0.62257E+00_JPRB, 0.64269E+00_JPRB, 0.79037E+00_JPRB, 0.11077E+01_JPRB, 0.14044E+01_JPRB ,&
&  0.15321E+01_JPRB, 0.15165E+01_JPRB, 0.15101E+01_JPRB, 0.15049E+01_JPRB, 0.15131E+01_JPRB, 0.14751E+01_JPRB, 0.14120E+01_JPRB ,&
&  0.13226E+01_JPRB, 0.10677E+01_JPRB, 0.10178E+01_JPRB, 0.96186E+00_JPRB, 0.10044E+01_JPRB, 0.10318E+01_JPRB, 0.10173E+01_JPRB ,&
&  0.99589E+00_JPRB, 0.99869E+00_JPRB, 0.10059E+01_JPRB, 0.97021E+00_JPRB, 0.94313E+00_JPRB, 0.88408E+00_JPRB, 0.87558E+00_JPRB ,&
&  0.94529E+00_JPRB, 0.99085E+00_JPRB, 0.12191E+01_JPRB, 0.16290E+01_JPRB, 0.23665E+01_JPRB, 0.32588E+01_JPRB, 0.33369E+01_JPRB ,&
&  0.27238E+01_JPRB, 0.25798E+01_JPRB, 0.35175E+01_JPRB, 0.33020E+01_JPRB, 0.33473E+01_JPRB, 0.36729E+01_JPRB, 0.34475E+01_JPRB ,&
&  0.31521E+01_JPRB, 0.29251E+01_JPRB, 0.28431E+01_JPRB, 0.30056E+01_JPRB, 0.39253E+01_JPRB, 0.41004E+01_JPRB, 0.38224E+01_JPRB ,&
&  0.31988E+01_JPRB, 0.30325E+01_JPRB, 0.27826E+01_JPRB, 0.24977E+01_JPRB, 0.23203E+01_JPRB, 0.24119E+01_JPRB, 0.22729E+01_JPRB ,&
&  0.20925E+01_JPRB, 0.22185E+01_JPRB, 0.17671E+01_JPRB, 0.13590E+01_JPRB, 0.11357E+01_JPRB, 0.11538E+01_JPRB, 0.12191E+01_JPRB ,&
&  0.11933E+01_JPRB, 0.11158E+01_JPRB, 0.10357E+01_JPRB, 0.11491E+01_JPRB, 0.14578E+01_JPRB, 0.16580E+01_JPRB, 0.16292E+01_JPRB ,&
&  0.16110E+01_JPRB, 0.17983E+01_JPRB, 0.17222E+01_JPRB, 0.17131E+01_JPRB, 0.17630E+01_JPRB, 0.15915E+01_JPRB, 0.13511E+01_JPRB ,&
&  0.11476E+01_JPRB, 0.10211E+01_JPRB, 0.93831E+00_JPRB, 0.88133E+00_JPRB, 0.84051E+00_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.84896E+00_JPRB, 0.81244E+00_JPRB, 0.77978E+00_JPRB, 0.74916E+00_JPRB, 0.71652E+00_JPRB, 0.69344E+00_JPRB, 0.66972E+00_JPRB ,&
&  0.64354E+00_JPRB, 0.62572E+00_JPRB, 0.62968E+00_JPRB, 0.63985E+00_JPRB, 0.63591E+00_JPRB, 0.60657E+00_JPRB, 0.60027E+00_JPRB ,&
&  0.63503E+00_JPRB, 0.66843E+00_JPRB, 0.66204E+00_JPRB, 0.72932E+00_JPRB, 0.87509E+00_JPRB, 0.10874E+01_JPRB, 0.13849E+01_JPRB ,&
&  0.15775E+01_JPRB, 0.15416E+01_JPRB, 0.15197E+01_JPRB, 0.16695E+01_JPRB, 0.18623E+01_JPRB, 0.19441E+01_JPRB, 0.19033E+01_JPRB ,&
&  0.16582E+01_JPRB, 0.15000E+01_JPRB, 0.14491E+01_JPRB, 0.13043E+01_JPRB, 0.12371E+01_JPRB, 0.11546E+01_JPRB, 0.10503E+01_JPRB ,&
&  0.96989E+00_JPRB, 0.94399E+00_JPRB, 0.93044E+00_JPRB, 0.91880E+00_JPRB, 0.95399E+00_JPRB, 0.99971E+00_JPRB, 0.10324E+01_JPRB ,&
&  0.11133E+01_JPRB, 0.13231E+01_JPRB, 0.17184E+01_JPRB, 0.23315E+01_JPRB, 0.27694E+01_JPRB, 0.39332E+01_JPRB, 0.36621E+01_JPRB ,&
&  0.33835E+01_JPRB, 0.33182E+01_JPRB, 0.32804E+01_JPRB, 0.35545E+01_JPRB, 0.30477E+01_JPRB, 0.32619E+01_JPRB, 0.33919E+01_JPRB ,&
&  0.32687E+01_JPRB, 0.31697E+01_JPRB, 0.33293E+01_JPRB, 0.34054E+01_JPRB, 0.40901E+01_JPRB, 0.51837E+01_JPRB, 0.47380E+01_JPRB ,&
&  0.38884E+01_JPRB, 0.34399E+01_JPRB, 0.29612E+01_JPRB, 0.26649E+01_JPRB, 0.22346E+01_JPRB, 0.21961E+01_JPRB, 0.19143E+01_JPRB ,&
&  0.17363E+01_JPRB, 0.14061E+01_JPRB, 0.15029E+01_JPRB, 0.12815E+01_JPRB, 0.11978E+01_JPRB, 0.11696E+01_JPRB, 0.11620E+01_JPRB ,&
&  0.11957E+01_JPRB, 0.12175E+01_JPRB, 0.13664E+01_JPRB, 0.17202E+01_JPRB, 0.21856E+01_JPRB, 0.19295E+01_JPRB, 0.20921E+01_JPRB ,&
&  0.22726E+01_JPRB, 0.22855E+01_JPRB, 0.20025E+01_JPRB, 0.19856E+01_JPRB, 0.18546E+01_JPRB, 0.17051E+01_JPRB, 0.15116E+01_JPRB ,&
&  0.12979E+01_JPRB, 0.11086E+01_JPRB, 0.10013E+01_JPRB, 0.93520E+00_JPRB, 0.88674E+00_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.82142E+00_JPRB, 0.78674E+00_JPRB, 0.76464E+00_JPRB, 0.74587E+00_JPRB, 0.72692E+00_JPRB, 0.69314E+00_JPRB, 0.65140E+00_JPRB ,&
&  0.61924E+00_JPRB, 0.60674E+00_JPRB, 0.60650E+00_JPRB, 0.61662E+00_JPRB, 0.61437E+00_JPRB, 0.62419E+00_JPRB, 0.63537E+00_JPRB ,&
&  0.68733E+00_JPRB, 0.73607E+00_JPRB, 0.72985E+00_JPRB, 0.76393E+00_JPRB, 0.89583E+00_JPRB, 0.10644E+01_JPRB, 0.12101E+01_JPRB ,&
&  0.15268E+01_JPRB, 0.13945E+01_JPRB, 0.13693E+01_JPRB, 0.16159E+01_JPRB, 0.21433E+01_JPRB, 0.24595E+01_JPRB, 0.23741E+01_JPRB ,&
&  0.23379E+01_JPRB, 0.20814E+01_JPRB, 0.16752E+01_JPRB, 0.13721E+01_JPRB, 0.11541E+01_JPRB, 0.10021E+01_JPRB, 0.87573E+00_JPRB ,&
&  0.81858E+00_JPRB, 0.80977E+00_JPRB, 0.82065E+00_JPRB, 0.83602E+00_JPRB, 0.89724E+00_JPRB, 0.90591E+00_JPRB, 0.10691E+01_JPRB ,&
&  0.12583E+01_JPRB, 0.15204E+01_JPRB, 0.18911E+01_JPRB, 0.22550E+01_JPRB, 0.27400E+01_JPRB, 0.41759E+01_JPRB, 0.47022E+01_JPRB ,&
&  0.36601E+01_JPRB, 0.34361E+01_JPRB, 0.36889E+01_JPRB, 0.38488E+01_JPRB, 0.37010E+01_JPRB, 0.33612E+01_JPRB, 0.32951E+01_JPRB ,&
&  0.26350E+01_JPRB, 0.23435E+01_JPRB, 0.24234E+01_JPRB, 0.30293E+01_JPRB, 0.48342E+01_JPRB, 0.63830E+01_JPRB, 0.59031E+01_JPRB ,&
&  0.48903E+01_JPRB, 0.41261E+01_JPRB, 0.34728E+01_JPRB, 0.26192E+01_JPRB, 0.22500E+01_JPRB, 0.19365E+01_JPRB, 0.22195E+01_JPRB ,&
&  0.20658E+01_JPRB, 0.18424E+01_JPRB, 0.15091E+01_JPRB, 0.11637E+01_JPRB, 0.92856E+00_JPRB, 0.10638E+01_JPRB, 0.11375E+01_JPRB ,&
&  0.12551E+01_JPRB, 0.13887E+01_JPRB, 0.20894E+01_JPRB, 0.23507E+01_JPRB, 0.24334E+01_JPRB, 0.21660E+01_JPRB, 0.27092E+01_JPRB ,&
&  0.26027E+01_JPRB, 0.26363E+01_JPRB, 0.24244E+01_JPRB, 0.19935E+01_JPRB, 0.16526E+01_JPRB, 0.14004E+01_JPRB, 0.12543E+01_JPRB ,&
&  0.11846E+01_JPRB, 0.11243E+01_JPRB, 0.10398E+01_JPRB, 0.95678E+00_JPRB, 0.88878E+00_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.73113E+00_JPRB, 0.70381E+00_JPRB, 0.69031E+00_JPRB, 0.67802E+00_JPRB, 0.66111E+00_JPRB, 0.63536E+00_JPRB, 0.60458E+00_JPRB ,&
&  0.59032E+00_JPRB, 0.58502E+00_JPRB, 0.58728E+00_JPRB, 0.60744E+00_JPRB, 0.62386E+00_JPRB, 0.65355E+00_JPRB, 0.67755E+00_JPRB ,&
&  0.73812E+00_JPRB, 0.83901E+00_JPRB, 0.82250E+00_JPRB, 0.84868E+00_JPRB, 0.95028E+00_JPRB, 0.10438E+01_JPRB, 0.11681E+01_JPRB ,&
&  0.14466E+01_JPRB, 0.13129E+01_JPRB, 0.13304E+01_JPRB, 0.16003E+01_JPRB, 0.23682E+01_JPRB, 0.25170E+01_JPRB, 0.23218E+01_JPRB ,&
&  0.23702E+01_JPRB, 0.20022E+01_JPRB, 0.14647E+01_JPRB, 0.11700E+01_JPRB, 0.98959E+00_JPRB, 0.85100E+00_JPRB, 0.78284E+00_JPRB ,&
&  0.78127E+00_JPRB, 0.81419E+00_JPRB, 0.85509E+00_JPRB, 0.88754E+00_JPRB, 0.90174E+00_JPRB, 0.98826E+00_JPRB, 0.11899E+01_JPRB ,&
&  0.13956E+01_JPRB, 0.16234E+01_JPRB, 0.18196E+01_JPRB, 0.20666E+01_JPRB, 0.26290E+01_JPRB, 0.41369E+01_JPRB, 0.57953E+01_JPRB ,&
&  0.47718E+01_JPRB, 0.45751E+01_JPRB, 0.45955E+01_JPRB, 0.48297E+01_JPRB, 0.38566E+01_JPRB, 0.34514E+01_JPRB, 0.32461E+01_JPRB ,&
&  0.25672E+01_JPRB, 0.24599E+01_JPRB, 0.30857E+01_JPRB, 0.42343E+01_JPRB, 0.51653E+01_JPRB, 0.62009E+01_JPRB, 0.72066E+01_JPRB ,&
&  0.53664E+01_JPRB, 0.43911E+01_JPRB, 0.37783E+01_JPRB, 0.32082E+01_JPRB, 0.18630E+01_JPRB, 0.13127E+01_JPRB, 0.17697E+01_JPRB ,&
&  0.16135E+01_JPRB, 0.92264E+00_JPRB, 0.71674E+00_JPRB, 0.88531E+00_JPRB, 0.64755E+00_JPRB, 0.73690E+00_JPRB, 0.11045E+01_JPRB ,&
&  0.13635E+01_JPRB, 0.16527E+01_JPRB, 0.27613E+01_JPRB, 0.25677E+01_JPRB, 0.25959E+01_JPRB, 0.32175E+01_JPRB, 0.35104E+01_JPRB ,&
&  0.33412E+01_JPRB, 0.23821E+01_JPRB, 0.19137E+01_JPRB, 0.14635E+01_JPRB, 0.11804E+01_JPRB, 0.10783E+01_JPRB, 0.10262E+01_JPRB ,&
&  0.98561E+00_JPRB, 0.93269E+00_JPRB, 0.88318E+00_JPRB, 0.81641E+00_JPRB, 0.77335E+00_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.63340E+00_JPRB, 0.61318E+00_JPRB, 0.61118E+00_JPRB, 0.60459E+00_JPRB, 0.58045E+00_JPRB, 0.56598E+00_JPRB, 0.55925E+00_JPRB ,&
&  0.56316E+00_JPRB, 0.57033E+00_JPRB, 0.58559E+00_JPRB, 0.61830E+00_JPRB, 0.66067E+00_JPRB, 0.70594E+00_JPRB, 0.73190E+00_JPRB ,&
&  0.77744E+00_JPRB, 0.87515E+00_JPRB, 0.95200E+00_JPRB, 0.10237E+01_JPRB, 0.11366E+01_JPRB, 0.10801E+01_JPRB, 0.13426E+01_JPRB ,&
&  0.14141E+01_JPRB, 0.12888E+01_JPRB, 0.13831E+01_JPRB, 0.16962E+01_JPRB, 0.20439E+01_JPRB, 0.20017E+01_JPRB, 0.18696E+01_JPRB ,&
&  0.17309E+01_JPRB, 0.14388E+01_JPRB, 0.11565E+01_JPRB, 0.97683E+00_JPRB, 0.85944E+00_JPRB, 0.80215E+00_JPRB, 0.82223E+00_JPRB ,&
&  0.85995E+00_JPRB, 0.94439E+00_JPRB, 0.10051E+01_JPRB, 0.94158E+00_JPRB, 0.10057E+01_JPRB, 0.11163E+01_JPRB, 0.12971E+01_JPRB ,&
&  0.14630E+01_JPRB, 0.16234E+01_JPRB, 0.17834E+01_JPRB, 0.21364E+01_JPRB, 0.32984E+01_JPRB, 0.48782E+01_JPRB, 0.52392E+01_JPRB ,&
&  0.47643E+01_JPRB, 0.44416E+01_JPRB, 0.50779E+01_JPRB, 0.49084E+01_JPRB, 0.41123E+01_JPRB, 0.39854E+01_JPRB, 0.36122E+01_JPRB ,&
&  0.35271E+01_JPRB, 0.37513E+01_JPRB, 0.40843E+01_JPRB, 0.50002E+01_JPRB, 0.63086E+01_JPRB, 0.65222E+01_JPRB, 0.66221E+01_JPRB ,&
&  0.58193E+01_JPRB, 0.44684E+01_JPRB, 0.30582E+01_JPRB, 0.21265E+01_JPRB, 0.19766E+01_JPRB, 0.15552E+01_JPRB, 0.10160E+01_JPRB ,&
&  0.81811E+00_JPRB, 0.67950E+00_JPRB, 0.49358E+00_JPRB, 0.54275E+00_JPRB, 0.49283E+00_JPRB, 0.58660E+00_JPRB, 0.98850E+00_JPRB ,&
&  0.16817E+01_JPRB, 0.21745E+01_JPRB, 0.27162E+01_JPRB, 0.25270E+01_JPRB, 0.25285E+01_JPRB, 0.32579E+01_JPRB, 0.44879E+01_JPRB ,&
&  0.23928E+01_JPRB, 0.17459E+01_JPRB, 0.13266E+01_JPRB, 0.10641E+01_JPRB, 0.96861E+00_JPRB, 0.91446E+00_JPRB, 0.87406E+00_JPRB ,&
&  0.82302E+00_JPRB, 0.77860E+00_JPRB, 0.75663E+00_JPRB, 0.71957E+00_JPRB, 0.67011E+00_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.59069E+00_JPRB, 0.59314E+00_JPRB, 0.59429E+00_JPRB, 0.58314E+00_JPRB, 0.54859E+00_JPRB, 0.54576E+00_JPRB, 0.55930E+00_JPRB ,&
&  0.57703E+00_JPRB, 0.59913E+00_JPRB, 0.63411E+00_JPRB, 0.68371E+00_JPRB, 0.75130E+00_JPRB, 0.79406E+00_JPRB, 0.81417E+00_JPRB ,&
&  0.84976E+00_JPRB, 0.96213E+00_JPRB, 0.11015E+01_JPRB, 0.12458E+01_JPRB, 0.13859E+01_JPRB, 0.12209E+01_JPRB, 0.15191E+01_JPRB ,&
&  0.13678E+01_JPRB, 0.13057E+01_JPRB, 0.14033E+01_JPRB, 0.15963E+01_JPRB, 0.16956E+01_JPRB, 0.15864E+01_JPRB, 0.14422E+01_JPRB ,&
&  0.12568E+01_JPRB, 0.11632E+01_JPRB, 0.97997E+00_JPRB, 0.88225E+00_JPRB, 0.82507E+00_JPRB, 0.84959E+00_JPRB, 0.90441E+00_JPRB ,&
&  0.97068E+00_JPRB, 0.10706E+01_JPRB, 0.10341E+01_JPRB, 0.10828E+01_JPRB, 0.12312E+01_JPRB, 0.13579E+01_JPRB, 0.14940E+01_JPRB ,&
&  0.15796E+01_JPRB, 0.17367E+01_JPRB, 0.20453E+01_JPRB, 0.29288E+01_JPRB, 0.37420E+01_JPRB, 0.40496E+01_JPRB, 0.47299E+01_JPRB ,&
&  0.52149E+01_JPRB, 0.53349E+01_JPRB, 0.47614E+01_JPRB, 0.53471E+01_JPRB, 0.47002E+01_JPRB, 0.41033E+01_JPRB, 0.39597E+01_JPRB ,&
&  0.41272E+01_JPRB, 0.49900E+01_JPRB, 0.52543E+01_JPRB, 0.56339E+01_JPRB, 0.78727E+01_JPRB, 0.84967E+01_JPRB, 0.67719E+01_JPRB ,&
&  0.59822E+01_JPRB, 0.43074E+01_JPRB, 0.30220E+01_JPRB, 0.21822E+01_JPRB, 0.26206E+01_JPRB, 0.26567E+01_JPRB, 0.13209E+01_JPRB ,&
&  0.86312E+00_JPRB, 0.64511E+00_JPRB, 0.45935E+00_JPRB, 0.45077E+00_JPRB, 0.45565E+00_JPRB, 0.57386E+00_JPRB, 0.12261E+01_JPRB ,&
&  0.15191E+01_JPRB, 0.23987E+01_JPRB, 0.27154E+01_JPRB, 0.26432E+01_JPRB, 0.26422E+01_JPRB, 0.25273E+01_JPRB, 0.28100E+01_JPRB ,&
&  0.19022E+01_JPRB, 0.12051E+01_JPRB, 0.92150E+00_JPRB, 0.90587E+00_JPRB, 0.90422E+00_JPRB, 0.85518E+00_JPRB, 0.78999E+00_JPRB ,&
&  0.74938E+00_JPRB, 0.74419E+00_JPRB, 0.71751E+00_JPRB, 0.68463E+00_JPRB, 0.63665E+00_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.60263E+00_JPRB, 0.62146E+00_JPRB, 0.63282E+00_JPRB, 0.59960E+00_JPRB, 0.58634E+00_JPRB, 0.59419E+00_JPRB, 0.59188E+00_JPRB ,&
&  0.62431E+00_JPRB, 0.66592E+00_JPRB, 0.71225E+00_JPRB, 0.79123E+00_JPRB, 0.86470E+00_JPRB, 0.88882E+00_JPRB, 0.91256E+00_JPRB ,&
&  0.95220E+00_JPRB, 0.10772E+01_JPRB, 0.12725E+01_JPRB, 0.14999E+01_JPRB, 0.15439E+01_JPRB, 0.14211E+01_JPRB, 0.15626E+01_JPRB ,&
&  0.13649E+01_JPRB, 0.13636E+01_JPRB, 0.13715E+01_JPRB, 0.15118E+01_JPRB, 0.15252E+01_JPRB, 0.14720E+01_JPRB, 0.13278E+01_JPRB ,&
&  0.11526E+01_JPRB, 0.11556E+01_JPRB, 0.94658E+00_JPRB, 0.97248E+00_JPRB, 0.94907E+00_JPRB, 0.10054E+01_JPRB, 0.10480E+01_JPRB ,&
&  0.11334E+01_JPRB, 0.11393E+01_JPRB, 0.11669E+01_JPRB, 0.12897E+01_JPRB, 0.14620E+01_JPRB, 0.16137E+01_JPRB, 0.17624E+01_JPRB ,&
&  0.18626E+01_JPRB, 0.21485E+01_JPRB, 0.27119E+01_JPRB, 0.35617E+01_JPRB, 0.33655E+01_JPRB, 0.44348E+01_JPRB, 0.46920E+01_JPRB ,&
&  0.50554E+01_JPRB, 0.50707E+01_JPRB, 0.45171E+01_JPRB, 0.47801E+01_JPRB, 0.47716E+01_JPRB, 0.44171E+01_JPRB, 0.43508E+01_JPRB ,&
&  0.53099E+01_JPRB, 0.57529E+01_JPRB, 0.63696E+01_JPRB, 0.68117E+01_JPRB, 0.88329E+01_JPRB, 0.11940E+02_JPRB, 0.71010E+01_JPRB ,&
&  0.48855E+01_JPRB, 0.43289E+01_JPRB, 0.30510E+01_JPRB, 0.23326E+01_JPRB, 0.28282E+01_JPRB, 0.32358E+01_JPRB, 0.27038E+01_JPRB ,&
&  0.11171E+01_JPRB, 0.62464E+00_JPRB, 0.53123E+00_JPRB, 0.53826E+00_JPRB, 0.46725E+00_JPRB, 0.61023E+00_JPRB, 0.19244E+01_JPRB ,&
&  0.15624E+01_JPRB, 0.23856E+01_JPRB, 0.25597E+01_JPRB, 0.24846E+01_JPRB, 0.26643E+01_JPRB, 0.21068E+01_JPRB, 0.17279E+01_JPRB ,&
&  0.84309E+00_JPRB, 0.77123E+00_JPRB, 0.81528E+00_JPRB, 0.79764E+00_JPRB, 0.79344E+00_JPRB, 0.74990E+00_JPRB, 0.68674E+00_JPRB ,&
&  0.68270E+00_JPRB, 0.68640E+00_JPRB, 0.64869E+00_JPRB, 0.61818E+00_JPRB, 0.60138E+00_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.63665E+00_JPRB, 0.65609E+00_JPRB, 0.65676E+00_JPRB, 0.65381E+00_JPRB, 0.65817E+00_JPRB, 0.66708E+00_JPRB, 0.66341E+00_JPRB ,&
&  0.69997E+00_JPRB, 0.75924E+00_JPRB, 0.84164E+00_JPRB, 0.94139E+00_JPRB, 0.10054E+01_JPRB, 0.10171E+01_JPRB, 0.10468E+01_JPRB ,&
&  0.11225E+01_JPRB, 0.12812E+01_JPRB, 0.14836E+01_JPRB, 0.16260E+01_JPRB, 0.15867E+01_JPRB, 0.14327E+01_JPRB, 0.14601E+01_JPRB ,&
&  0.14028E+01_JPRB, 0.14375E+01_JPRB, 0.12882E+01_JPRB, 0.14061E+01_JPRB, 0.15391E+01_JPRB, 0.16026E+01_JPRB, 0.14658E+01_JPRB ,&
&  0.13560E+01_JPRB, 0.13381E+01_JPRB, 0.12200E+01_JPRB, 0.12236E+01_JPRB, 0.12470E+01_JPRB, 0.13051E+01_JPRB, 0.13496E+01_JPRB ,&
&  0.13046E+01_JPRB, 0.13352E+01_JPRB, 0.14357E+01_JPRB, 0.15984E+01_JPRB, 0.17900E+01_JPRB, 0.19621E+01_JPRB, 0.21552E+01_JPRB ,&
&  0.24102E+01_JPRB, 0.28410E+01_JPRB, 0.33816E+01_JPRB, 0.35316E+01_JPRB, 0.37477E+01_JPRB, 0.41187E+01_JPRB, 0.44705E+01_JPRB ,&
&  0.47615E+01_JPRB, 0.49133E+01_JPRB, 0.47555E+01_JPRB, 0.46147E+01_JPRB, 0.46773E+01_JPRB, 0.48044E+01_JPRB, 0.52326E+01_JPRB ,&
&  0.63212E+01_JPRB, 0.69862E+01_JPRB, 0.70561E+01_JPRB, 0.74122E+01_JPRB, 0.81869E+01_JPRB, 0.87658E+01_JPRB, 0.72762E+01_JPRB ,&
&  0.52858E+01_JPRB, 0.36764E+01_JPRB, 0.28147E+01_JPRB, 0.28185E+01_JPRB, 0.31675E+01_JPRB, 0.31940E+01_JPRB, 0.37722E+01_JPRB ,&
&  0.32993E+01_JPRB, 0.12711E+01_JPRB, 0.85844E+00_JPRB, 0.10765E+01_JPRB, 0.60211E+00_JPRB, 0.57202E+00_JPRB, 0.10259E+01_JPRB ,&
&  0.16135E+01_JPRB, 0.21950E+01_JPRB, 0.22485E+01_JPRB, 0.22092E+01_JPRB, 0.23799E+01_JPRB, 0.15699E+01_JPRB, 0.86034E+00_JPRB ,&
&  0.69051E+00_JPRB, 0.73402E+00_JPRB, 0.73241E+00_JPRB, 0.68509E+00_JPRB, 0.66991E+00_JPRB, 0.69099E+00_JPRB, 0.63501E+00_JPRB ,&
&  0.64583E+00_JPRB, 0.65097E+00_JPRB, 0.65202E+00_JPRB, 0.64433E+00_JPRB, 0.63142E+00_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.71668E+00_JPRB, 0.73551E+00_JPRB, 0.76845E+00_JPRB, 0.78985E+00_JPRB, 0.78905E+00_JPRB, 0.79926E+00_JPRB, 0.80728E+00_JPRB ,&
&  0.85353E+00_JPRB, 0.93778E+00_JPRB, 0.10320E+01_JPRB, 0.11207E+01_JPRB, 0.11543E+01_JPRB, 0.11683E+01_JPRB, 0.12402E+01_JPRB ,&
&  0.13442E+01_JPRB, 0.14978E+01_JPRB, 0.16430E+01_JPRB, 0.17453E+01_JPRB, 0.17547E+01_JPRB, 0.17709E+01_JPRB, 0.13971E+01_JPRB ,&
&  0.12558E+01_JPRB, 0.15311E+01_JPRB, 0.13426E+01_JPRB, 0.14479E+01_JPRB, 0.15936E+01_JPRB, 0.17143E+01_JPRB, 0.16905E+01_JPRB ,&
&  0.16473E+01_JPRB, 0.16157E+01_JPRB, 0.15502E+01_JPRB, 0.15443E+01_JPRB, 0.15987E+01_JPRB, 0.16433E+01_JPRB, 0.16559E+01_JPRB ,&
&  0.16949E+01_JPRB, 0.17386E+01_JPRB, 0.18516E+01_JPRB, 0.20106E+01_JPRB, 0.21855E+01_JPRB, 0.23850E+01_JPRB, 0.26692E+01_JPRB ,&
&  0.29816E+01_JPRB, 0.32601E+01_JPRB, 0.35246E+01_JPRB, 0.34627E+01_JPRB, 0.36635E+01_JPRB, 0.39607E+01_JPRB, 0.42381E+01_JPRB ,&
&  0.42047E+01_JPRB, 0.41138E+01_JPRB, 0.44849E+01_JPRB, 0.49638E+01_JPRB, 0.52350E+01_JPRB, 0.53261E+01_JPRB, 0.57536E+01_JPRB ,&
&  0.68191E+01_JPRB, 0.74844E+01_JPRB, 0.76319E+01_JPRB, 0.66464E+01_JPRB, 0.60942E+01_JPRB, 0.57213E+01_JPRB, 0.56269E+01_JPRB ,&
&  0.47644E+01_JPRB, 0.37495E+01_JPRB, 0.31827E+01_JPRB, 0.30410E+01_JPRB, 0.30301E+01_JPRB, 0.30451E+01_JPRB, 0.33016E+01_JPRB ,&
&  0.33244E+01_JPRB, 0.28210E+01_JPRB, 0.21491E+01_JPRB, 0.13428E+01_JPRB, 0.87557E+00_JPRB, 0.78798E+00_JPRB, 0.10071E+01_JPRB ,&
&  0.15506E+01_JPRB, 0.17507E+01_JPRB, 0.17795E+01_JPRB, 0.22412E+01_JPRB, 0.17962E+01_JPRB, 0.11389E+01_JPRB, 0.69950E+00_JPRB ,&
&  0.61906E+00_JPRB, 0.65511E+00_JPRB, 0.66754E+00_JPRB, 0.65819E+00_JPRB, 0.68309E+00_JPRB, 0.70909E+00_JPRB, 0.69624E+00_JPRB ,&
&  0.70958E+00_JPRB, 0.72232E+00_JPRB, 0.71485E+00_JPRB, 0.70056E+00_JPRB, 0.70700E+00_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.88531E+00_JPRB, 0.92569E+00_JPRB, 0.95675E+00_JPRB, 0.97583E+00_JPRB, 0.98512E+00_JPRB, 0.95911E+00_JPRB, 0.96618E+00_JPRB ,&
&  0.10336E+01_JPRB, 0.11183E+01_JPRB, 0.11927E+01_JPRB, 0.12406E+01_JPRB, 0.12636E+01_JPRB, 0.13124E+01_JPRB, 0.14063E+01_JPRB ,&
&  0.14881E+01_JPRB, 0.15877E+01_JPRB, 0.16660E+01_JPRB, 0.17516E+01_JPRB, 0.18048E+01_JPRB, 0.19477E+01_JPRB, 0.16590E+01_JPRB ,&
&  0.11903E+01_JPRB, 0.17917E+01_JPRB, 0.14224E+01_JPRB, 0.15502E+01_JPRB, 0.16520E+01_JPRB, 0.17864E+01_JPRB, 0.17903E+01_JPRB ,&
&  0.18002E+01_JPRB, 0.18305E+01_JPRB, 0.18326E+01_JPRB, 0.18035E+01_JPRB, 0.17982E+01_JPRB, 0.18656E+01_JPRB, 0.19521E+01_JPRB ,&
&  0.20349E+01_JPRB, 0.21005E+01_JPRB, 0.22034E+01_JPRB, 0.23407E+01_JPRB, 0.25026E+01_JPRB, 0.27070E+01_JPRB, 0.29541E+01_JPRB ,&
&  0.31546E+01_JPRB, 0.33458E+01_JPRB, 0.33754E+01_JPRB, 0.33903E+01_JPRB, 0.34668E+01_JPRB, 0.36543E+01_JPRB, 0.38770E+01_JPRB ,&
&  0.39709E+01_JPRB, 0.40287E+01_JPRB, 0.44481E+01_JPRB, 0.47180E+01_JPRB, 0.48286E+01_JPRB, 0.51927E+01_JPRB, 0.53724E+01_JPRB ,&
&  0.62632E+01_JPRB, 0.62160E+01_JPRB, 0.62683E+01_JPRB, 0.51990E+01_JPRB, 0.40886E+01_JPRB, 0.35756E+01_JPRB, 0.34178E+01_JPRB ,&
&  0.29692E+01_JPRB, 0.28139E+01_JPRB, 0.25575E+01_JPRB, 0.24896E+01_JPRB, 0.26630E+01_JPRB, 0.28579E+01_JPRB, 0.26931E+01_JPRB ,&
&  0.24376E+01_JPRB, 0.17407E+01_JPRB, 0.13491E+01_JPRB, 0.11312E+01_JPRB, 0.85533E+00_JPRB, 0.78736E+00_JPRB, 0.10772E+01_JPRB ,&
&  0.15091E+01_JPRB, 0.16769E+01_JPRB, 0.18279E+01_JPRB, 0.17541E+01_JPRB, 0.11627E+01_JPRB, 0.76699E+00_JPRB, 0.60217E+00_JPRB ,&
&  0.58079E+00_JPRB, 0.62508E+00_JPRB, 0.65065E+00_JPRB, 0.67287E+00_JPRB, 0.71341E+00_JPRB, 0.71804E+00_JPRB, 0.76454E+00_JPRB ,&
&  0.77459E+00_JPRB, 0.79545E+00_JPRB, 0.81762E+00_JPRB, 0.82273E+00_JPRB, 0.84643E+00_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.11238E+01_JPRB, 0.12295E+01_JPRB, 0.13291E+01_JPRB, 0.13917E+01_JPRB, 0.13426E+01_JPRB, 0.12833E+01_JPRB, 0.12344E+01_JPRB ,&
&  0.11930E+01_JPRB, 0.11891E+01_JPRB, 0.12467E+01_JPRB, 0.13000E+01_JPRB, 0.13469E+01_JPRB, 0.14231E+01_JPRB, 0.15315E+01_JPRB ,&
&  0.16358E+01_JPRB, 0.17357E+01_JPRB, 0.18325E+01_JPRB, 0.19499E+01_JPRB, 0.20933E+01_JPRB, 0.23244E+01_JPRB, 0.22145E+01_JPRB ,&
&  0.15726E+01_JPRB, 0.14038E+01_JPRB, 0.16453E+01_JPRB, 0.16043E+01_JPRB, 0.16099E+01_JPRB, 0.17338E+01_JPRB, 0.18136E+01_JPRB ,&
&  0.18567E+01_JPRB, 0.18919E+01_JPRB, 0.19195E+01_JPRB, 0.19059E+01_JPRB, 0.19053E+01_JPRB, 0.20071E+01_JPRB, 0.21181E+01_JPRB ,&
&  0.22222E+01_JPRB, 0.22904E+01_JPRB, 0.23708E+01_JPRB, 0.24802E+01_JPRB, 0.26087E+01_JPRB, 0.27573E+01_JPRB, 0.29312E+01_JPRB ,&
&  0.30388E+01_JPRB, 0.31659E+01_JPRB, 0.32073E+01_JPRB, 0.32085E+01_JPRB, 0.32653E+01_JPRB, 0.33162E+01_JPRB, 0.33897E+01_JPRB ,&
&  0.35613E+01_JPRB, 0.38369E+01_JPRB, 0.41169E+01_JPRB, 0.42533E+01_JPRB, 0.42152E+01_JPRB, 0.45886E+01_JPRB, 0.47506E+01_JPRB ,&
&  0.49265E+01_JPRB, 0.45626E+01_JPRB, 0.45421E+01_JPRB, 0.38262E+01_JPRB, 0.31708E+01_JPRB, 0.25706E+01_JPRB, 0.23729E+01_JPRB ,&
&  0.22107E+01_JPRB, 0.20496E+01_JPRB, 0.18563E+01_JPRB, 0.18072E+01_JPRB, 0.20691E+01_JPRB, 0.21560E+01_JPRB, 0.21450E+01_JPRB ,&
&  0.17495E+01_JPRB, 0.13888E+01_JPRB, 0.10290E+01_JPRB, 0.10155E+01_JPRB, 0.82518E+00_JPRB, 0.91059E+00_JPRB, 0.11181E+01_JPRB ,&
&  0.13648E+01_JPRB, 0.12828E+01_JPRB, 0.12732E+01_JPRB, 0.12199E+01_JPRB, 0.82818E+00_JPRB, 0.61630E+00_JPRB, 0.53389E+00_JPRB ,&
&  0.51719E+00_JPRB, 0.55947E+00_JPRB, 0.59260E+00_JPRB, 0.61519E+00_JPRB, 0.66939E+00_JPRB, 0.72254E+00_JPRB, 0.78577E+00_JPRB ,&
&  0.81527E+00_JPRB, 0.85667E+00_JPRB, 0.90432E+00_JPRB, 0.95410E+00_JPRB, 0.10290E+01_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.11339E+01_JPRB, 0.11929E+01_JPRB, 0.12468E+01_JPRB, 0.12222E+01_JPRB, 0.12095E+01_JPRB, 0.11674E+01_JPRB, 0.11253E+01_JPRB ,&
&  0.11503E+01_JPRB, 0.11853E+01_JPRB, 0.12485E+01_JPRB, 0.13039E+01_JPRB, 0.13533E+01_JPRB, 0.14378E+01_JPRB, 0.15368E+01_JPRB ,&
&  0.15962E+01_JPRB, 0.16521E+01_JPRB, 0.17161E+01_JPRB, 0.18077E+01_JPRB, 0.19052E+01_JPRB, 0.20444E+01_JPRB, 0.21294E+01_JPRB ,&
&  0.16699E+01_JPRB, 0.11470E+01_JPRB, 0.13528E+01_JPRB, 0.15193E+01_JPRB, 0.15185E+01_JPRB, 0.16510E+01_JPRB, 0.17476E+01_JPRB ,&
&  0.17238E+01_JPRB, 0.17671E+01_JPRB, 0.18320E+01_JPRB, 0.18376E+01_JPRB, 0.18367E+01_JPRB, 0.19584E+01_JPRB, 0.20817E+01_JPRB ,&
&  0.21976E+01_JPRB, 0.22985E+01_JPRB, 0.23835E+01_JPRB, 0.24791E+01_JPRB, 0.25628E+01_JPRB, 0.26466E+01_JPRB, 0.27225E+01_JPRB ,&
&  0.28028E+01_JPRB, 0.28379E+01_JPRB, 0.28953E+01_JPRB, 0.30034E+01_JPRB, 0.30158E+01_JPRB, 0.29973E+01_JPRB, 0.29652E+01_JPRB ,&
&  0.30555E+01_JPRB, 0.32155E+01_JPRB, 0.34935E+01_JPRB, 0.37348E+01_JPRB, 0.38989E+01_JPRB, 0.38200E+01_JPRB, 0.38376E+01_JPRB ,&
&  0.35885E+01_JPRB, 0.31604E+01_JPRB, 0.31869E+01_JPRB, 0.33332E+01_JPRB, 0.23295E+01_JPRB, 0.19620E+01_JPRB, 0.16965E+01_JPRB ,&
&  0.17887E+01_JPRB, 0.15808E+01_JPRB, 0.13878E+01_JPRB, 0.13639E+01_JPRB, 0.15812E+01_JPRB, 0.15128E+01_JPRB, 0.15333E+01_JPRB ,&
&  0.13343E+01_JPRB, 0.98838E+00_JPRB, 0.89070E+00_JPRB, 0.10032E+01_JPRB, 0.97525E+00_JPRB, 0.12943E+01_JPRB, 0.11743E+01_JPRB ,&
&  0.10406E+01_JPRB, 0.10084E+01_JPRB, 0.10649E+01_JPRB, 0.10375E+01_JPRB, 0.79187E+00_JPRB, 0.59630E+00_JPRB, 0.53103E+00_JPRB ,&
&  0.50499E+00_JPRB, 0.51571E+00_JPRB, 0.54708E+00_JPRB, 0.56550E+00_JPRB, 0.61015E+00_JPRB, 0.65816E+00_JPRB, 0.74789E+00_JPRB ,&
&  0.81864E+00_JPRB, 0.87958E+00_JPRB, 0.92907E+00_JPRB, 0.98841E+00_JPRB, 0.10567E+01_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.85332E+00_JPRB, 0.88363E+00_JPRB, 0.88378E+00_JPRB, 0.94439E+00_JPRB, 0.10031E+01_JPRB, 0.10234E+01_JPRB, 0.10384E+01_JPRB ,&
&  0.10831E+01_JPRB, 0.11584E+01_JPRB, 0.12218E+01_JPRB, 0.12706E+01_JPRB, 0.13242E+01_JPRB, 0.14001E+01_JPRB, 0.14257E+01_JPRB ,&
&  0.14145E+01_JPRB, 0.14102E+01_JPRB, 0.14280E+01_JPRB, 0.14576E+01_JPRB, 0.15109E+01_JPRB, 0.15729E+01_JPRB, 0.17503E+01_JPRB ,&
&  0.18760E+01_JPRB, 0.18491E+01_JPRB, 0.18065E+01_JPRB, 0.14808E+01_JPRB, 0.10656E+01_JPRB, 0.13583E+01_JPRB, 0.13523E+01_JPRB ,&
&  0.14727E+01_JPRB, 0.15560E+01_JPRB, 0.15835E+01_JPRB, 0.16660E+01_JPRB, 0.17520E+01_JPRB, 0.18460E+01_JPRB, 0.19559E+01_JPRB ,&
&  0.20385E+01_JPRB, 0.21313E+01_JPRB, 0.22202E+01_JPRB, 0.23029E+01_JPRB, 0.23678E+01_JPRB, 0.24124E+01_JPRB, 0.24274E+01_JPRB ,&
&  0.24507E+01_JPRB, 0.24985E+01_JPRB, 0.25514E+01_JPRB, 0.27101E+01_JPRB, 0.27375E+01_JPRB, 0.27594E+01_JPRB, 0.26716E+01_JPRB ,&
&  0.26748E+01_JPRB, 0.27296E+01_JPRB, 0.29151E+01_JPRB, 0.30613E+01_JPRB, 0.30647E+01_JPRB, 0.28510E+01_JPRB, 0.27134E+01_JPRB ,&
&  0.25273E+01_JPRB, 0.23025E+01_JPRB, 0.23094E+01_JPRB, 0.25297E+01_JPRB, 0.16726E+01_JPRB, 0.16823E+01_JPRB, 0.17715E+01_JPRB ,&
&  0.15534E+01_JPRB, 0.13978E+01_JPRB, 0.12806E+01_JPRB, 0.11429E+01_JPRB, 0.11911E+01_JPRB, 0.11436E+01_JPRB, 0.12818E+01_JPRB ,&
&  0.11414E+01_JPRB, 0.85273E+00_JPRB, 0.81114E+00_JPRB, 0.94846E+00_JPRB, 0.10810E+01_JPRB, 0.13300E+01_JPRB, 0.10411E+01_JPRB ,&
&  0.84754E+00_JPRB, 0.10371E+01_JPRB, 0.10952E+01_JPRB, 0.12694E+01_JPRB, 0.11167E+01_JPRB, 0.61214E+00_JPRB, 0.51476E+00_JPRB ,&
&  0.49603E+00_JPRB, 0.50726E+00_JPRB, 0.52646E+00_JPRB, 0.54036E+00_JPRB, 0.56374E+00_JPRB, 0.59675E+00_JPRB, 0.62598E+00_JPRB ,&
&  0.64971E+00_JPRB, 0.66530E+00_JPRB, 0.69874E+00_JPRB, 0.74601E+00_JPRB, 0.80874E+00_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.61025E+00_JPRB, 0.61046E+00_JPRB, 0.59746E+00_JPRB, 0.61088E+00_JPRB, 0.63529E+00_JPRB, 0.69136E+00_JPRB, 0.74471E+00_JPRB ,&
&  0.80735E+00_JPRB, 0.88857E+00_JPRB, 0.10055E+01_JPRB, 0.11139E+01_JPRB, 0.11470E+01_JPRB, 0.11375E+01_JPRB, 0.10464E+01_JPRB ,&
&  0.92860E+00_JPRB, 0.80638E+00_JPRB, 0.74295E+00_JPRB, 0.73174E+00_JPRB, 0.78531E+00_JPRB, 0.81729E+00_JPRB, 0.88847E+00_JPRB ,&
&  0.10025E+01_JPRB, 0.11931E+01_JPRB, 0.14041E+01_JPRB, 0.18714E+01_JPRB, 0.21721E+01_JPRB, 0.14809E+01_JPRB, 0.13238E+01_JPRB ,&
&  0.14791E+01_JPRB, 0.15304E+01_JPRB, 0.13128E+01_JPRB, 0.13087E+01_JPRB, 0.14272E+01_JPRB, 0.15250E+01_JPRB, 0.16483E+01_JPRB ,&
&  0.17100E+01_JPRB, 0.17773E+01_JPRB, 0.18381E+01_JPRB, 0.18736E+01_JPRB, 0.18895E+01_JPRB, 0.18605E+01_JPRB, 0.18334E+01_JPRB ,&
&  0.18166E+01_JPRB, 0.17991E+01_JPRB, 0.18910E+01_JPRB, 0.20150E+01_JPRB, 0.21524E+01_JPRB, 0.22462E+01_JPRB, 0.22462E+01_JPRB ,&
&  0.22441E+01_JPRB, 0.22666E+01_JPRB, 0.24742E+01_JPRB, 0.25145E+01_JPRB, 0.23503E+01_JPRB, 0.21858E+01_JPRB, 0.19647E+01_JPRB ,&
&  0.17514E+01_JPRB, 0.14824E+01_JPRB, 0.11498E+01_JPRB, 0.16975E+01_JPRB, 0.19533E+01_JPRB, 0.17125E+01_JPRB, 0.14442E+01_JPRB ,&
&  0.13725E+01_JPRB, 0.12858E+01_JPRB, 0.11874E+01_JPRB, 0.11157E+01_JPRB, 0.11015E+01_JPRB, 0.13208E+01_JPRB, 0.11832E+01_JPRB ,&
&  0.10434E+01_JPRB, 0.76417E+00_JPRB, 0.67723E+00_JPRB, 0.73135E+00_JPRB, 0.82211E+00_JPRB, 0.95927E+00_JPRB, 0.77858E+00_JPRB ,&
&  0.73977E+00_JPRB, 0.10346E+01_JPRB, 0.11599E+01_JPRB, 0.15835E+01_JPRB, 0.19085E+01_JPRB, 0.53639E+00_JPRB, 0.50754E+00_JPRB ,&
&  0.50168E+00_JPRB, 0.51009E+00_JPRB, 0.53239E+00_JPRB, 0.53274E+00_JPRB, 0.53449E+00_JPRB, 0.53817E+00_JPRB, 0.53505E+00_JPRB ,&
&  0.53914E+00_JPRB, 0.55631E+00_JPRB, 0.57457E+00_JPRB, 0.58247E+00_JPRB, 0.59978E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.59281E+00_JPRB, 0.59245E+00_JPRB, 0.59628E+00_JPRB, 0.60312E+00_JPRB, 0.60307E+00_JPRB, 0.60036E+00_JPRB, 0.61101E+00_JPRB ,&
&  0.63086E+00_JPRB, 0.67092E+00_JPRB, 0.73297E+00_JPRB, 0.76933E+00_JPRB, 0.75380E+00_JPRB, 0.70970E+00_JPRB, 0.69542E+00_JPRB ,&
&  0.69748E+00_JPRB, 0.69780E+00_JPRB, 0.72823E+00_JPRB, 0.77825E+00_JPRB, 0.84356E+00_JPRB, 0.86546E+00_JPRB, 0.92691E+00_JPRB ,&
&  0.92692E+00_JPRB, 0.95011E+00_JPRB, 0.10374E+01_JPRB, 0.11207E+01_JPRB, 0.17117E+01_JPRB, 0.16131E+01_JPRB, 0.12772E+01_JPRB ,&
&  0.11630E+01_JPRB, 0.11459E+01_JPRB, 0.11606E+01_JPRB, 0.10720E+01_JPRB, 0.10710E+01_JPRB, 0.10430E+01_JPRB, 0.10641E+01_JPRB ,&
&  0.10595E+01_JPRB, 0.11327E+01_JPRB, 0.11966E+01_JPRB, 0.12708E+01_JPRB, 0.13232E+01_JPRB, 0.13408E+01_JPRB, 0.13926E+01_JPRB ,&
&  0.14303E+01_JPRB, 0.14694E+01_JPRB, 0.15187E+01_JPRB, 0.13543E+01_JPRB, 0.14626E+01_JPRB, 0.15944E+01_JPRB, 0.16824E+01_JPRB ,&
&  0.17119E+01_JPRB, 0.17228E+01_JPRB, 0.18612E+01_JPRB, 0.18053E+01_JPRB, 0.17178E+01_JPRB, 0.15605E+01_JPRB, 0.14159E+01_JPRB ,&
&  0.13107E+01_JPRB, 0.97965E+00_JPRB, 0.64798E+00_JPRB, 0.93968E+00_JPRB, 0.11620E+01_JPRB, 0.10546E+01_JPRB, 0.11619E+01_JPRB ,&
&  0.11422E+01_JPRB, 0.11018E+01_JPRB, 0.10551E+01_JPRB, 0.10335E+01_JPRB, 0.98035E+00_JPRB, 0.10850E+01_JPRB, 0.10580E+01_JPRB ,&
&  0.91471E+00_JPRB, 0.70107E+00_JPRB, 0.59775E+00_JPRB, 0.59125E+00_JPRB, 0.62664E+00_JPRB, 0.70477E+00_JPRB, 0.78131E+00_JPRB ,&
&  0.86896E+00_JPRB, 0.73610E+00_JPRB, 0.79288E+00_JPRB, 0.93913E+00_JPRB, 0.63460E+00_JPRB, 0.46655E+00_JPRB, 0.46328E+00_JPRB ,&
&  0.46880E+00_JPRB, 0.48391E+00_JPRB, 0.50776E+00_JPRB, 0.52225E+00_JPRB, 0.53332E+00_JPRB, 0.54348E+00_JPRB, 0.55727E+00_JPRB ,&
&  0.56562E+00_JPRB, 0.57643E+00_JPRB, 0.58092E+00_JPRB, 0.58588E+00_JPRB, 0.59082E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.76633E+00_JPRB, 0.78041E+00_JPRB, 0.79971E+00_JPRB, 0.81954E+00_JPRB, 0.83027E+00_JPRB, 0.84747E+00_JPRB, 0.86310E+00_JPRB ,&
&  0.88504E+00_JPRB, 0.91722E+00_JPRB, 0.93735E+00_JPRB, 0.94467E+00_JPRB, 0.96034E+00_JPRB, 0.98171E+00_JPRB, 0.99340E+00_JPRB ,&
&  0.10189E+01_JPRB, 0.10309E+01_JPRB, 0.10871E+01_JPRB, 0.11149E+01_JPRB, 0.11542E+01_JPRB, 0.12060E+01_JPRB, 0.12150E+01_JPRB ,&
&  0.12086E+01_JPRB, 0.11979E+01_JPRB, 0.12128E+01_JPRB, 0.12141E+01_JPRB, 0.13015E+01_JPRB, 0.13161E+01_JPRB, 0.12510E+01_JPRB ,&
&  0.10361E+01_JPRB, 0.94673E+00_JPRB, 0.11253E+01_JPRB, 0.97340E+00_JPRB, 0.97217E+00_JPRB, 0.97905E+00_JPRB, 0.90918E+00_JPRB ,&
&  0.91524E+00_JPRB, 0.96577E+00_JPRB, 0.10274E+01_JPRB, 0.10859E+01_JPRB, 0.11654E+01_JPRB, 0.12055E+01_JPRB, 0.12311E+01_JPRB ,&
&  0.12635E+01_JPRB, 0.13051E+01_JPRB, 0.13277E+01_JPRB, 0.12862E+01_JPRB, 0.12603E+01_JPRB, 0.14541E+01_JPRB, 0.15334E+01_JPRB ,&
&  0.15615E+01_JPRB, 0.15918E+01_JPRB, 0.13323E+01_JPRB, 0.12356E+01_JPRB, 0.12966E+01_JPRB, 0.11540E+01_JPRB, 0.11004E+01_JPRB ,&
&  0.12005E+01_JPRB, 0.10448E+01_JPRB, 0.58171E+00_JPRB, 0.77968E+00_JPRB, 0.93331E+00_JPRB, 0.10011E+01_JPRB, 0.10299E+01_JPRB ,&
&  0.98116E+00_JPRB, 0.87175E+00_JPRB, 0.84506E+00_JPRB, 0.90335E+00_JPRB, 0.94691E+00_JPRB, 0.89617E+00_JPRB, 0.87495E+00_JPRB ,&
&  0.76861E+00_JPRB, 0.66590E+00_JPRB, 0.60569E+00_JPRB, 0.67355E+00_JPRB, 0.80148E+00_JPRB, 0.10318E+01_JPRB, 0.10214E+01_JPRB ,&
&  0.85697E+00_JPRB, 0.69228E+00_JPRB, 0.94721E+00_JPRB, 0.12570E+01_JPRB, 0.60233E+00_JPRB, 0.51221E+00_JPRB, 0.49361E+00_JPRB ,&
&  0.52391E+00_JPRB, 0.54041E+00_JPRB, 0.55247E+00_JPRB, 0.57550E+00_JPRB, 0.60946E+00_JPRB, 0.62289E+00_JPRB, 0.62722E+00_JPRB ,&
&  0.64505E+00_JPRB, 0.67395E+00_JPRB, 0.69545E+00_JPRB, 0.72179E+00_JPRB, 0.74507E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.89727E+00_JPRB, 0.91699E+00_JPRB, 0.92981E+00_JPRB, 0.94261E+00_JPRB, 0.96003E+00_JPRB, 0.97734E+00_JPRB, 0.98580E+00_JPRB ,&
&  0.10040E+01_JPRB, 0.10203E+01_JPRB, 0.10281E+01_JPRB, 0.10476E+01_JPRB, 0.10556E+01_JPRB, 0.11086E+01_JPRB, 0.11297E+01_JPRB ,&
&  0.11725E+01_JPRB, 0.11965E+01_JPRB, 0.12045E+01_JPRB, 0.12082E+01_JPRB, 0.12227E+01_JPRB, 0.12214E+01_JPRB, 0.12171E+01_JPRB ,&
&  0.12393E+01_JPRB, 0.12824E+01_JPRB, 0.13357E+01_JPRB, 0.13893E+01_JPRB, 0.14222E+01_JPRB, 0.15211E+01_JPRB, 0.16504E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.11012E+01_JPRB, 0.11636E+01_JPRB, 0.10309E+01_JPRB, 0.10475E+01_JPRB, 0.10717E+01_JPRB, 0.10379E+01_JPRB ,&
&  0.10251E+01_JPRB, 0.10677E+01_JPRB, 0.11040E+01_JPRB, 0.11335E+01_JPRB, 0.11704E+01_JPRB, 0.12162E+01_JPRB, 0.12438E+01_JPRB ,&
&  0.12951E+01_JPRB, 0.13374E+01_JPRB, 0.13781E+01_JPRB, 0.14478E+01_JPRB, 0.14000E+01_JPRB, 0.16214E+01_JPRB, 0.16885E+01_JPRB ,&
&  0.17005E+01_JPRB, 0.16314E+01_JPRB, 0.12512E+01_JPRB, 0.11879E+01_JPRB, 0.12322E+01_JPRB, 0.11764E+01_JPRB, 0.11113E+01_JPRB ,&
&  0.12087E+01_JPRB, 0.11650E+01_JPRB, 0.85435E+00_JPRB, 0.94735E+00_JPRB, 0.98148E+00_JPRB, 0.98832E+00_JPRB, 0.92673E+00_JPRB ,&
&  0.84914E+00_JPRB, 0.73630E+00_JPRB, 0.70545E+00_JPRB, 0.73093E+00_JPRB, 0.73977E+00_JPRB, 0.72029E+00_JPRB, 0.73405E+00_JPRB ,&
&  0.71682E+00_JPRB, 0.68234E+00_JPRB, 0.69340E+00_JPRB, 0.66032E+00_JPRB, 0.87974E+00_JPRB, 0.89936E+00_JPRB, 0.13172E+01_JPRB ,&
&  0.71193E+00_JPRB, 0.65029E+00_JPRB, 0.60162E+00_JPRB, 0.84149E+00_JPRB, 0.64134E+00_JPRB, 0.50782E+00_JPRB, 0.54721E+00_JPRB ,&
&  0.60910E+00_JPRB, 0.63719E+00_JPRB, 0.64677E+00_JPRB, 0.66211E+00_JPRB, 0.70499E+00_JPRB, 0.72774E+00_JPRB, 0.74759E+00_JPRB ,&
&  0.74966E+00_JPRB, 0.78125E+00_JPRB, 0.82472E+00_JPRB, 0.85705E+00_JPRB, 0.86993E+00_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.95203E+00_JPRB, 0.97604E+00_JPRB, 0.99551E+00_JPRB, 0.10112E+01_JPRB, 0.10257E+01_JPRB, 0.10433E+01_JPRB, 0.10598E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.10890E+01_JPRB, 0.10976E+01_JPRB, 0.11134E+01_JPRB, 0.11293E+01_JPRB, 0.11665E+01_JPRB, 0.11822E+01_JPRB ,&
&  0.12174E+01_JPRB, 0.12312E+01_JPRB, 0.12305E+01_JPRB, 0.12492E+01_JPRB, 0.12406E+01_JPRB, 0.12196E+01_JPRB, 0.12144E+01_JPRB ,&
&  0.12184E+01_JPRB, 0.12154E+01_JPRB, 0.12151E+01_JPRB, 0.12207E+01_JPRB, 0.12279E+01_JPRB, 0.12028E+01_JPRB, 0.12737E+01_JPRB ,&
&  0.93615E+00_JPRB, 0.12651E+01_JPRB, 0.13482E+01_JPRB, 0.12249E+01_JPRB, 0.11727E+01_JPRB, 0.10858E+01_JPRB, 0.10804E+01_JPRB ,&
&  0.10975E+01_JPRB, 0.11019E+01_JPRB, 0.11202E+01_JPRB, 0.11557E+01_JPRB, 0.11999E+01_JPRB, 0.12533E+01_JPRB, 0.12957E+01_JPRB ,&
&  0.13609E+01_JPRB, 0.14175E+01_JPRB, 0.14684E+01_JPRB, 0.15185E+01_JPRB, 0.16126E+01_JPRB, 0.16667E+01_JPRB, 0.16849E+01_JPRB ,&
&  0.16877E+01_JPRB, 0.17077E+01_JPRB, 0.13474E+01_JPRB, 0.13160E+01_JPRB, 0.14157E+01_JPRB, 0.13579E+01_JPRB, 0.12131E+01_JPRB ,&
&  0.12239E+01_JPRB, 0.11609E+01_JPRB, 0.11063E+01_JPRB, 0.10302E+01_JPRB, 0.98194E+00_JPRB, 0.92294E+00_JPRB, 0.85250E+00_JPRB ,&
&  0.71682E+00_JPRB, 0.60696E+00_JPRB, 0.57936E+00_JPRB, 0.61736E+00_JPRB, 0.63021E+00_JPRB, 0.62290E+00_JPRB, 0.58680E+00_JPRB ,&
&  0.57071E+00_JPRB, 0.57086E+00_JPRB, 0.59374E+00_JPRB, 0.64226E+00_JPRB, 0.66566E+00_JPRB, 0.11204E+01_JPRB, 0.13457E+01_JPRB ,&
&  0.72185E+00_JPRB, 0.62962E+00_JPRB, 0.52101E+00_JPRB, 0.68258E+00_JPRB, 0.69318E+00_JPRB, 0.63613E+00_JPRB, 0.67175E+00_JPRB ,&
&  0.77523E+00_JPRB, 0.82699E+00_JPRB, 0.84732E+00_JPRB, 0.78027E+00_JPRB, 0.79247E+00_JPRB, 0.77402E+00_JPRB, 0.77628E+00_JPRB ,&
&  0.79381E+00_JPRB, 0.82309E+00_JPRB, 0.85958E+00_JPRB, 0.89440E+00_JPRB, 0.92423E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.97257E+00_JPRB, 0.99541E+00_JPRB, 0.10179E+01_JPRB, 0.10422E+01_JPRB, 0.10665E+01_JPRB, 0.10839E+01_JPRB, 0.11041E+01_JPRB ,&
&  0.11259E+01_JPRB, 0.11429E+01_JPRB, 0.11506E+01_JPRB, 0.11600E+01_JPRB, 0.11752E+01_JPRB, 0.12099E+01_JPRB, 0.12391E+01_JPRB ,&
&  0.12247E+01_JPRB, 0.12399E+01_JPRB, 0.12492E+01_JPRB, 0.12552E+01_JPRB, 0.12438E+01_JPRB, 0.12134E+01_JPRB, 0.12221E+01_JPRB ,&
&  0.12151E+01_JPRB, 0.12151E+01_JPRB, 0.12175E+01_JPRB, 0.12431E+01_JPRB, 0.12837E+01_JPRB, 0.13690E+01_JPRB, 0.11432E+01_JPRB ,&
&  0.12531E+01_JPRB, 0.13696E+01_JPRB, 0.13949E+01_JPRB, 0.13019E+01_JPRB, 0.11868E+01_JPRB, 0.11558E+01_JPRB, 0.11529E+01_JPRB ,&
&  0.11665E+01_JPRB, 0.11529E+01_JPRB, 0.11648E+01_JPRB, 0.11800E+01_JPRB, 0.12074E+01_JPRB, 0.12594E+01_JPRB, 0.12973E+01_JPRB ,&
&  0.13595E+01_JPRB, 0.14579E+01_JPRB, 0.15563E+01_JPRB, 0.16508E+01_JPRB, 0.17701E+01_JPRB, 0.18847E+01_JPRB, 0.19507E+01_JPRB ,&
&  0.19871E+01_JPRB, 0.20285E+01_JPRB, 0.18550E+01_JPRB, 0.18705E+01_JPRB, 0.19582E+01_JPRB, 0.17750E+01_JPRB, 0.14915E+01_JPRB ,&
&  0.13663E+01_JPRB, 0.11256E+01_JPRB, 0.99060E+00_JPRB, 0.11012E+01_JPRB, 0.10040E+01_JPRB, 0.89291E+00_JPRB, 0.77169E+00_JPRB ,&
&  0.63289E+00_JPRB, 0.55727E+00_JPRB, 0.55482E+00_JPRB, 0.59524E+00_JPRB, 0.58819E+00_JPRB, 0.58715E+00_JPRB, 0.59535E+00_JPRB ,&
&  0.61422E+00_JPRB, 0.63248E+00_JPRB, 0.66562E+00_JPRB, 0.73541E+00_JPRB, 0.81819E+00_JPRB, 0.79511E+00_JPRB, 0.94154E+00_JPRB ,&
&  0.76487E+00_JPRB, 0.62936E+00_JPRB, 0.69934E+00_JPRB, 0.79987E+00_JPRB, 0.83300E+00_JPRB, 0.83440E+00_JPRB, 0.93698E+00_JPRB ,&
&  0.10820E+01_JPRB, 0.12169E+01_JPRB, 0.12254E+01_JPRB, 0.10060E+01_JPRB, 0.88925E+00_JPRB, 0.83848E+00_JPRB, 0.83043E+00_JPRB ,&
&  0.83824E+00_JPRB, 0.85551E+00_JPRB, 0.88476E+00_JPRB, 0.91856E+00_JPRB, 0.94808E+00_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.97506E+00_JPRB, 0.99920E+00_JPRB, 0.10232E+01_JPRB, 0.10454E+01_JPRB, 0.10615E+01_JPRB, 0.10736E+01_JPRB, 0.10819E+01_JPRB ,&
&  0.10991E+01_JPRB, 0.11165E+01_JPRB, 0.11293E+01_JPRB, 0.11361E+01_JPRB, 0.11423E+01_JPRB, 0.11600E+01_JPRB, 0.11849E+01_JPRB ,&
&  0.12157E+01_JPRB, 0.12310E+01_JPRB, 0.12664E+01_JPRB, 0.12496E+01_JPRB, 0.12317E+01_JPRB, 0.12364E+01_JPRB, 0.12629E+01_JPRB ,&
&  0.12698E+01_JPRB, 0.12618E+01_JPRB, 0.12577E+01_JPRB, 0.12895E+01_JPRB, 0.13247E+01_JPRB, 0.13740E+01_JPRB, 0.11861E+01_JPRB ,&
&  0.13473E+01_JPRB, 0.14556E+01_JPRB, 0.14323E+01_JPRB, 0.13750E+01_JPRB, 0.12666E+01_JPRB, 0.12359E+01_JPRB, 0.12299E+01_JPRB ,&
&  0.12250E+01_JPRB, 0.12250E+01_JPRB, 0.12209E+01_JPRB, 0.12259E+01_JPRB, 0.12425E+01_JPRB, 0.12681E+01_JPRB, 0.13132E+01_JPRB ,&
&  0.13653E+01_JPRB, 0.14419E+01_JPRB, 0.15627E+01_JPRB, 0.16886E+01_JPRB, 0.18589E+01_JPRB, 0.20248E+01_JPRB, 0.21432E+01_JPRB ,&
&  0.22658E+01_JPRB, 0.23900E+01_JPRB, 0.22628E+01_JPRB, 0.23234E+01_JPRB, 0.24590E+01_JPRB, 0.21880E+01_JPRB, 0.17033E+01_JPRB ,&
&  0.12197E+01_JPRB, 0.10733E+01_JPRB, 0.10833E+01_JPRB, 0.11399E+01_JPRB, 0.10004E+01_JPRB, 0.87029E+00_JPRB, 0.78444E+00_JPRB ,&
&  0.67269E+00_JPRB, 0.58518E+00_JPRB, 0.59270E+00_JPRB, 0.58628E+00_JPRB, 0.57555E+00_JPRB, 0.59826E+00_JPRB, 0.62516E+00_JPRB ,&
&  0.65620E+00_JPRB, 0.71343E+00_JPRB, 0.77272E+00_JPRB, 0.83633E+00_JPRB, 0.87576E+00_JPRB, 0.84046E+00_JPRB, 0.92544E+00_JPRB ,&
&  0.89418E+00_JPRB, 0.87688E+00_JPRB, 0.90731E+00_JPRB, 0.75888E+00_JPRB, 0.10407E+01_JPRB, 0.11096E+01_JPRB, 0.12265E+01_JPRB ,&
&  0.11971E+01_JPRB, 0.16105E+01_JPRB, 0.19883E+01_JPRB, 0.15509E+01_JPRB, 0.12193E+01_JPRB, 0.89390E+00_JPRB, 0.84003E+00_JPRB ,&
&  0.84532E+00_JPRB, 0.85914E+00_JPRB, 0.88506E+00_JPRB, 0.91793E+00_JPRB, 0.94753E+00_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.97278E+00_JPRB, 0.10020E+01_JPRB, 0.10336E+01_JPRB, 0.10525E+01_JPRB, 0.10621E+01_JPRB, 0.10470E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.10329E+01_JPRB, 0.10404E+01_JPRB, 0.10545E+01_JPRB, 0.10700E+01_JPRB, 0.10785E+01_JPRB, 0.10863E+01_JPRB, 0.11037E+01_JPRB ,&
&  0.11257E+01_JPRB, 0.11544E+01_JPRB, 0.11916E+01_JPRB, 0.12193E+01_JPRB, 0.12483E+01_JPRB, 0.12703E+01_JPRB, 0.13032E+01_JPRB ,&
&  0.13303E+01_JPRB, 0.13762E+01_JPRB, 0.13852E+01_JPRB, 0.14152E+01_JPRB, 0.14317E+01_JPRB, 0.14742E+01_JPRB, 0.11932E+01_JPRB ,&
&  0.13776E+01_JPRB, 0.14474E+01_JPRB, 0.14386E+01_JPRB, 0.13781E+01_JPRB, 0.12786E+01_JPRB, 0.12325E+01_JPRB, 0.12043E+01_JPRB ,&
&  0.11994E+01_JPRB, 0.11809E+01_JPRB, 0.11390E+01_JPRB, 0.12067E+01_JPRB, 0.12611E+01_JPRB, 0.12502E+01_JPRB, 0.13030E+01_JPRB ,&
&  0.13597E+01_JPRB, 0.14388E+01_JPRB, 0.15411E+01_JPRB, 0.16949E+01_JPRB, 0.18907E+01_JPRB, 0.20728E+01_JPRB, 0.22538E+01_JPRB ,&
&  0.24415E+01_JPRB, 0.26429E+01_JPRB, 0.28075E+01_JPRB, 0.25594E+01_JPRB, 0.25566E+01_JPRB, 0.22783E+01_JPRB, 0.17168E+01_JPRB ,&
&  0.11865E+01_JPRB, 0.10432E+01_JPRB, 0.10812E+01_JPRB, 0.11448E+01_JPRB, 0.10114E+01_JPRB, 0.85586E+00_JPRB, 0.77036E+00_JPRB ,&
&  0.67950E+00_JPRB, 0.62728E+00_JPRB, 0.61293E+00_JPRB, 0.58633E+00_JPRB, 0.57467E+00_JPRB, 0.60453E+00_JPRB, 0.63908E+00_JPRB ,&
&  0.68328E+00_JPRB, 0.74345E+00_JPRB, 0.81373E+00_JPRB, 0.88623E+00_JPRB, 0.96369E+00_JPRB, 0.98968E+00_JPRB, 0.92533E+00_JPRB ,&
&  0.95321E+00_JPRB, 0.96844E+00_JPRB, 0.10245E+01_JPRB, 0.11310E+01_JPRB, 0.11974E+01_JPRB, 0.14199E+01_JPRB, 0.16570E+01_JPRB ,&
&  0.18785E+01_JPRB, 0.17567E+01_JPRB, 0.20835E+01_JPRB, 0.28922E+01_JPRB, 0.26636E+01_JPRB, 0.21549E+01_JPRB, 0.82622E+00_JPRB ,&
&  0.81908E+00_JPRB, 0.81869E+00_JPRB, 0.85217E+00_JPRB, 0.89461E+00_JPRB, 0.94424E+00_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.82186E+00_JPRB, 0.87220E+00_JPRB, 0.92255E+00_JPRB, 0.98551E+00_JPRB, 0.10363E+01_JPRB, 0.10259E+01_JPRB, 0.99832E+00_JPRB ,&
&  0.97890E+00_JPRB, 0.97032E+00_JPRB, 0.97647E+00_JPRB, 0.99013E+00_JPRB, 0.10005E+01_JPRB, 0.10052E+01_JPRB, 0.10156E+01_JPRB ,&
&  0.10322E+01_JPRB, 0.10551E+01_JPRB, 0.10803E+01_JPRB, 0.11098E+01_JPRB, 0.11343E+01_JPRB, 0.11703E+01_JPRB, 0.12127E+01_JPRB ,&
&  0.12570E+01_JPRB, 0.13114E+01_JPRB, 0.13590E+01_JPRB, 0.14087E+01_JPRB, 0.14553E+01_JPRB, 0.14530E+01_JPRB, 0.10800E+01_JPRB ,&
&  0.13341E+01_JPRB, 0.13726E+01_JPRB, 0.13971E+01_JPRB, 0.13409E+01_JPRB, 0.12387E+01_JPRB, 0.11746E+01_JPRB, 0.11581E+01_JPRB ,&
&  0.11450E+01_JPRB, 0.11220E+01_JPRB, 0.10919E+01_JPRB, 0.10780E+01_JPRB, 0.11875E+01_JPRB, 0.11887E+01_JPRB, 0.12576E+01_JPRB ,&
&  0.13321E+01_JPRB, 0.14123E+01_JPRB, 0.15027E+01_JPRB, 0.16482E+01_JPRB, 0.18490E+01_JPRB, 0.20529E+01_JPRB, 0.22776E+01_JPRB ,&
&  0.24990E+01_JPRB, 0.26964E+01_JPRB, 0.28142E+01_JPRB, 0.24715E+01_JPRB, 0.23496E+01_JPRB, 0.21731E+01_JPRB, 0.14909E+01_JPRB ,&
&  0.11823E+01_JPRB, 0.11251E+01_JPRB, 0.11932E+01_JPRB, 0.11718E+01_JPRB, 0.10416E+01_JPRB, 0.91021E+00_JPRB, 0.80550E+00_JPRB ,&
&  0.68256E+00_JPRB, 0.62187E+00_JPRB, 0.60760E+00_JPRB, 0.59033E+00_JPRB, 0.58464E+00_JPRB, 0.61336E+00_JPRB, 0.65298E+00_JPRB ,&
&  0.69874E+00_JPRB, 0.75485E+00_JPRB, 0.80092E+00_JPRB, 0.84032E+00_JPRB, 0.88922E+00_JPRB, 0.97588E+00_JPRB, 0.10277E+01_JPRB ,&
&  0.93958E+00_JPRB, 0.89379E+00_JPRB, 0.93919E+00_JPRB, 0.98062E+00_JPRB, 0.10264E+01_JPRB, 0.11033E+01_JPRB, 0.11838E+01_JPRB ,&
&  0.13105E+01_JPRB, 0.13800E+01_JPRB, 0.11713E+01_JPRB, 0.98299E+00_JPRB, 0.12999E+01_JPRB, 0.13846E+01_JPRB, 0.71749E+00_JPRB ,&
&  0.68480E+00_JPRB, 0.70271E+00_JPRB, 0.71379E+00_JPRB, 0.74738E+00_JPRB, 0.78292E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.65930E+00_JPRB, 0.67535E+00_JPRB, 0.74556E+00_JPRB, 0.76639E+00_JPRB, 0.83473E+00_JPRB, 0.86161E+00_JPRB, 0.88898E+00_JPRB ,&
&  0.89176E+00_JPRB, 0.89246E+00_JPRB, 0.89156E+00_JPRB, 0.89406E+00_JPRB, 0.89713E+00_JPRB, 0.90495E+00_JPRB, 0.91147E+00_JPRB ,&
&  0.92319E+00_JPRB, 0.93936E+00_JPRB, 0.95899E+00_JPRB, 0.98016E+00_JPRB, 0.10086E+01_JPRB, 0.10429E+01_JPRB, 0.10870E+01_JPRB ,&
&  0.11426E+01_JPRB, 0.12084E+01_JPRB, 0.12826E+01_JPRB, 0.13675E+01_JPRB, 0.14581E+01_JPRB, 0.15248E+01_JPRB, 0.15309E+01_JPRB ,&
&  0.11099E+01_JPRB, 0.13385E+01_JPRB, 0.13219E+01_JPRB, 0.12701E+01_JPRB, 0.11993E+01_JPRB, 0.11524E+01_JPRB, 0.11448E+01_JPRB ,&
&  0.11363E+01_JPRB, 0.10807E+01_JPRB, 0.10758E+01_JPRB, 0.10864E+01_JPRB, 0.11025E+01_JPRB, 0.10996E+01_JPRB, 0.11651E+01_JPRB ,&
&  0.12361E+01_JPRB, 0.13292E+01_JPRB, 0.14416E+01_JPRB, 0.15573E+01_JPRB, 0.17451E+01_JPRB, 0.19916E+01_JPRB, 0.22508E+01_JPRB ,&
&  0.25227E+01_JPRB, 0.27974E+01_JPRB, 0.29698E+01_JPRB, 0.26383E+01_JPRB, 0.23084E+01_JPRB, 0.21737E+01_JPRB, 0.16057E+01_JPRB ,&
&  0.13112E+01_JPRB, 0.11559E+01_JPRB, 0.12460E+01_JPRB, 0.12600E+01_JPRB, 0.11093E+01_JPRB, 0.99678E+00_JPRB, 0.89690E+00_JPRB ,&
&  0.77290E+00_JPRB, 0.66626E+00_JPRB, 0.62882E+00_JPRB, 0.61798E+00_JPRB, 0.61816E+00_JPRB, 0.63900E+00_JPRB, 0.65934E+00_JPRB ,&
&  0.68632E+00_JPRB, 0.71761E+00_JPRB, 0.74403E+00_JPRB, 0.77281E+00_JPRB, 0.78918E+00_JPRB, 0.80717E+00_JPRB, 0.81472E+00_JPRB ,&
&  0.82951E+00_JPRB, 0.86562E+00_JPRB, 0.87162E+00_JPRB, 0.89564E+00_JPRB, 0.89936E+00_JPRB, 0.92174E+00_JPRB, 0.95078E+00_JPRB ,&
&  0.97645E+00_JPRB, 0.99853E+00_JPRB, 0.99583E+00_JPRB, 0.97006E+00_JPRB, 0.90895E+00_JPRB, 0.83035E+00_JPRB, 0.67080E+00_JPRB ,&
&  0.64375E+00_JPRB, 0.62499E+00_JPRB, 0.63037E+00_JPRB, 0.63472E+00_JPRB, 0.65186E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.56802E+00_JPRB, 0.58314E+00_JPRB, 0.59969E+00_JPRB, 0.62057E+00_JPRB, 0.65220E+00_JPRB, 0.68860E+00_JPRB, 0.73553E+00_JPRB ,&
&  0.78243E+00_JPRB, 0.79480E+00_JPRB, 0.79691E+00_JPRB, 0.79739E+00_JPRB, 0.79869E+00_JPRB, 0.80680E+00_JPRB, 0.81846E+00_JPRB ,&
&  0.82997E+00_JPRB, 0.84110E+00_JPRB, 0.85172E+00_JPRB, 0.86364E+00_JPRB, 0.87970E+00_JPRB, 0.90433E+00_JPRB, 0.93845E+00_JPRB ,&
&  0.97641E+00_JPRB, 0.10461E+01_JPRB, 0.11421E+01_JPRB, 0.12506E+01_JPRB, 0.13659E+01_JPRB, 0.14563E+01_JPRB, 0.14715E+01_JPRB ,&
&  0.89769E+00_JPRB, 0.11320E+01_JPRB, 0.13349E+01_JPRB, 0.12778E+01_JPRB, 0.12001E+01_JPRB, 0.11622E+01_JPRB, 0.11622E+01_JPRB ,&
&  0.11253E+01_JPRB, 0.10638E+01_JPRB, 0.10512E+01_JPRB, 0.11099E+01_JPRB, 0.10387E+01_JPRB, 0.10217E+01_JPRB, 0.10528E+01_JPRB ,&
&  0.11028E+01_JPRB, 0.11618E+01_JPRB, 0.12549E+01_JPRB, 0.13778E+01_JPRB, 0.15527E+01_JPRB, 0.18124E+01_JPRB, 0.21069E+01_JPRB ,&
&  0.24304E+01_JPRB, 0.27933E+01_JPRB, 0.31289E+01_JPRB, 0.28266E+01_JPRB, 0.27828E+01_JPRB, 0.28484E+01_JPRB, 0.25762E+01_JPRB ,&
&  0.17406E+01_JPRB, 0.13495E+01_JPRB, 0.13978E+01_JPRB, 0.14044E+01_JPRB, 0.12110E+01_JPRB, 0.11071E+01_JPRB, 0.10136E+01_JPRB ,&
&  0.91612E+00_JPRB, 0.80890E+00_JPRB, 0.69595E+00_JPRB, 0.66192E+00_JPRB, 0.66012E+00_JPRB, 0.67458E+00_JPRB, 0.69322E+00_JPRB ,&
&  0.70259E+00_JPRB, 0.70484E+00_JPRB, 0.71502E+00_JPRB, 0.72775E+00_JPRB, 0.75638E+00_JPRB, 0.77041E+00_JPRB, 0.77623E+00_JPRB ,&
&  0.80153E+00_JPRB, 0.81659E+00_JPRB, 0.83959E+00_JPRB, 0.87002E+00_JPRB, 0.89799E+00_JPRB, 0.91622E+00_JPRB, 0.91716E+00_JPRB ,&
&  0.91219E+00_JPRB, 0.93247E+00_JPRB, 0.95985E+00_JPRB, 0.96001E+00_JPRB, 0.92638E+00_JPRB, 0.83860E+00_JPRB, 0.76168E+00_JPRB ,&
&  0.67827E+00_JPRB, 0.65720E+00_JPRB, 0.62280E+00_JPRB, 0.59614E+00_JPRB, 0.57616E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.55278E+00_JPRB, 0.54750E+00_JPRB, 0.55083E+00_JPRB, 0.55576E+00_JPRB, 0.56754E+00_JPRB, 0.59349E+00_JPRB, 0.62277E+00_JPRB ,&
&  0.64634E+00_JPRB, 0.68633E+00_JPRB, 0.71352E+00_JPRB, 0.71428E+00_JPRB, 0.70852E+00_JPRB, 0.71191E+00_JPRB, 0.72118E+00_JPRB ,&
&  0.73627E+00_JPRB, 0.74782E+00_JPRB, 0.75403E+00_JPRB, 0.76006E+00_JPRB, 0.77037E+00_JPRB, 0.78208E+00_JPRB, 0.79823E+00_JPRB ,&
&  0.82454E+00_JPRB, 0.87087E+00_JPRB, 0.94083E+00_JPRB, 0.10460E+01_JPRB, 0.11865E+01_JPRB, 0.13404E+01_JPRB, 0.14217E+01_JPRB ,&
&  0.10577E+01_JPRB, 0.59982E+00_JPRB, 0.11410E+01_JPRB, 0.13572E+01_JPRB, 0.12518E+01_JPRB, 0.12147E+01_JPRB, 0.11754E+01_JPRB ,&
&  0.11252E+01_JPRB, 0.11258E+01_JPRB, 0.10939E+01_JPRB, 0.10965E+01_JPRB, 0.99247E+00_JPRB, 0.95673E+00_JPRB, 0.94965E+00_JPRB ,&
&  0.96593E+00_JPRB, 0.99731E+00_JPRB, 0.10620E+01_JPRB, 0.11568E+01_JPRB, 0.12953E+01_JPRB, 0.15122E+01_JPRB, 0.18386E+01_JPRB ,&
&  0.21916E+01_JPRB, 0.26081E+01_JPRB, 0.31591E+01_JPRB, 0.30641E+01_JPRB, 0.31945E+01_JPRB, 0.29186E+01_JPRB, 0.24547E+01_JPRB ,&
&  0.19441E+01_JPRB, 0.15845E+01_JPRB, 0.15895E+01_JPRB, 0.16089E+01_JPRB, 0.13727E+01_JPRB, 0.11651E+01_JPRB, 0.11607E+01_JPRB ,&
&  0.10195E+01_JPRB, 0.95477E+00_JPRB, 0.83520E+00_JPRB, 0.74546E+00_JPRB, 0.71295E+00_JPRB, 0.71954E+00_JPRB, 0.73178E+00_JPRB ,&
&  0.73271E+00_JPRB, 0.73015E+00_JPRB, 0.73461E+00_JPRB, 0.74263E+00_JPRB, 0.75808E+00_JPRB, 0.76845E+00_JPRB, 0.77276E+00_JPRB ,&
&  0.79586E+00_JPRB, 0.82145E+00_JPRB, 0.84663E+00_JPRB, 0.87225E+00_JPRB, 0.89283E+00_JPRB, 0.88245E+00_JPRB, 0.88783E+00_JPRB ,&
&  0.90086E+00_JPRB, 0.90952E+00_JPRB, 0.91624E+00_JPRB, 0.93211E+00_JPRB, 0.92714E+00_JPRB, 0.88237E+00_JPRB, 0.86632E+00_JPRB ,&
&  0.80389E+00_JPRB, 0.73245E+00_JPRB, 0.64744E+00_JPRB, 0.59595E+00_JPRB, 0.57647E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.62964E+00_JPRB, 0.60372E+00_JPRB, 0.56628E+00_JPRB, 0.55640E+00_JPRB, 0.55476E+00_JPRB, 0.55340E+00_JPRB, 0.56953E+00_JPRB ,&
&  0.58443E+00_JPRB, 0.60609E+00_JPRB, 0.60917E+00_JPRB, 0.62933E+00_JPRB, 0.64020E+00_JPRB, 0.64088E+00_JPRB, 0.64838E+00_JPRB ,&
&  0.65216E+00_JPRB, 0.65613E+00_JPRB, 0.66012E+00_JPRB, 0.66775E+00_JPRB, 0.67705E+00_JPRB, 0.68030E+00_JPRB, 0.68649E+00_JPRB ,&
&  0.70188E+00_JPRB, 0.72102E+00_JPRB, 0.74990E+00_JPRB, 0.81841E+00_JPRB, 0.93372E+00_JPRB, 0.10777E+01_JPRB, 0.12306E+01_JPRB ,&
&  0.13351E+01_JPRB, 0.98255E+00_JPRB, 0.72972E+00_JPRB, 0.12126E+01_JPRB, 0.13565E+01_JPRB, 0.13344E+01_JPRB, 0.11876E+01_JPRB ,&
&  0.12008E+01_JPRB, 0.11823E+01_JPRB, 0.11592E+01_JPRB, 0.11230E+01_JPRB, 0.99729E+00_JPRB, 0.93141E+00_JPRB, 0.88371E+00_JPRB ,&
&  0.86015E+00_JPRB, 0.85607E+00_JPRB, 0.89300E+00_JPRB, 0.98010E+00_JPRB, 0.10827E+01_JPRB, 0.12283E+01_JPRB, 0.15029E+01_JPRB ,&
&  0.18511E+01_JPRB, 0.22759E+01_JPRB, 0.29307E+01_JPRB, 0.32442E+01_JPRB, 0.37148E+01_JPRB, 0.31860E+01_JPRB, 0.26178E+01_JPRB ,&
&  0.20679E+01_JPRB, 0.19827E+01_JPRB, 0.20473E+01_JPRB, 0.18383E+01_JPRB, 0.15014E+01_JPRB, 0.13425E+01_JPRB, 0.13350E+01_JPRB ,&
&  0.11636E+01_JPRB, 0.10741E+01_JPRB, 0.99761E+00_JPRB, 0.88894E+00_JPRB, 0.81713E+00_JPRB, 0.79018E+00_JPRB, 0.78268E+00_JPRB ,&
&  0.77601E+00_JPRB, 0.76133E+00_JPRB, 0.75409E+00_JPRB, 0.75897E+00_JPRB, 0.76727E+00_JPRB, 0.77094E+00_JPRB, 0.78182E+00_JPRB ,&
&  0.79538E+00_JPRB, 0.81509E+00_JPRB, 0.83867E+00_JPRB, 0.85383E+00_JPRB, 0.84944E+00_JPRB, 0.83553E+00_JPRB, 0.85642E+00_JPRB ,&
&  0.86714E+00_JPRB, 0.89192E+00_JPRB, 0.89288E+00_JPRB, 0.90374E+00_JPRB, 0.90642E+00_JPRB, 0.91535E+00_JPRB, 0.91580E+00_JPRB ,&
&  0.91275E+00_JPRB, 0.88479E+00_JPRB, 0.79903E+00_JPRB, 0.68901E+00_JPRB, 0.66078E+00_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.69728E+00_JPRB, 0.67468E+00_JPRB, 0.64445E+00_JPRB, 0.61357E+00_JPRB, 0.59661E+00_JPRB, 0.58039E+00_JPRB, 0.56112E+00_JPRB ,&
&  0.55172E+00_JPRB, 0.54726E+00_JPRB, 0.53741E+00_JPRB, 0.54753E+00_JPRB, 0.57019E+00_JPRB, 0.58178E+00_JPRB, 0.58844E+00_JPRB ,&
&  0.59558E+00_JPRB, 0.60150E+00_JPRB, 0.59236E+00_JPRB, 0.58997E+00_JPRB, 0.59612E+00_JPRB, 0.60103E+00_JPRB, 0.60151E+00_JPRB ,&
&  0.60504E+00_JPRB, 0.61473E+00_JPRB, 0.62844E+00_JPRB, 0.65617E+00_JPRB, 0.71791E+00_JPRB, 0.80998E+00_JPRB, 0.93180E+00_JPRB ,&
&  0.10952E+01_JPRB, 0.10759E+01_JPRB, 0.63596E+00_JPRB, 0.10310E+01_JPRB, 0.13904E+01_JPRB, 0.14033E+01_JPRB, 0.13303E+01_JPRB ,&
&  0.13208E+01_JPRB, 0.12847E+01_JPRB, 0.12747E+01_JPRB, 0.11717E+01_JPRB, 0.10566E+01_JPRB, 0.96351E+00_JPRB, 0.88393E+00_JPRB ,&
&  0.81567E+00_JPRB, 0.77356E+00_JPRB, 0.76582E+00_JPRB, 0.81256E+00_JPRB, 0.89645E+00_JPRB, 0.97949E+00_JPRB, 0.11370E+01_JPRB ,&
&  0.14194E+01_JPRB, 0.17790E+01_JPRB, 0.23545E+01_JPRB, 0.29264E+01_JPRB, 0.38326E+01_JPRB, 0.38348E+01_JPRB, 0.31254E+01_JPRB ,&
&  0.24760E+01_JPRB, 0.24516E+01_JPRB, 0.24315E+01_JPRB, 0.21342E+01_JPRB, 0.16636E+01_JPRB, 0.15185E+01_JPRB, 0.14939E+01_JPRB ,&
&  0.13208E+01_JPRB, 0.11858E+01_JPRB, 0.11242E+01_JPRB, 0.10560E+01_JPRB, 0.95778E+00_JPRB, 0.87174E+00_JPRB, 0.82914E+00_JPRB ,&
&  0.81461E+00_JPRB, 0.80045E+00_JPRB, 0.79012E+00_JPRB, 0.78137E+00_JPRB, 0.77848E+00_JPRB, 0.77885E+00_JPRB, 0.78376E+00_JPRB ,&
&  0.79468E+00_JPRB, 0.80835E+00_JPRB, 0.82701E+00_JPRB, 0.82968E+00_JPRB, 0.82991E+00_JPRB, 0.83104E+00_JPRB, 0.83002E+00_JPRB ,&
&  0.82862E+00_JPRB, 0.83435E+00_JPRB, 0.83660E+00_JPRB, 0.81625E+00_JPRB, 0.86069E+00_JPRB, 0.87003E+00_JPRB, 0.89801E+00_JPRB ,&
&  0.93019E+00_JPRB, 0.93461E+00_JPRB, 0.87750E+00_JPRB, 0.74669E+00_JPRB, 0.72401E+00_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.70827E+00_JPRB, 0.69007E+00_JPRB, 0.67057E+00_JPRB, 0.65498E+00_JPRB, 0.64301E+00_JPRB, 0.63034E+00_JPRB, 0.61016E+00_JPRB ,&
&  0.58479E+00_JPRB, 0.56948E+00_JPRB, 0.54328E+00_JPRB, 0.53638E+00_JPRB, 0.54097E+00_JPRB, 0.54529E+00_JPRB, 0.55460E+00_JPRB ,&
&  0.55944E+00_JPRB, 0.56326E+00_JPRB, 0.56378E+00_JPRB, 0.56215E+00_JPRB, 0.56212E+00_JPRB, 0.56000E+00_JPRB, 0.55901E+00_JPRB ,&
&  0.55495E+00_JPRB, 0.55592E+00_JPRB, 0.56277E+00_JPRB, 0.57319E+00_JPRB, 0.60034E+00_JPRB, 0.64821E+00_JPRB, 0.71309E+00_JPRB ,&
&  0.85961E+00_JPRB, 0.91752E+00_JPRB, 0.99899E+00_JPRB, 0.14095E+01_JPRB, 0.16508E+01_JPRB, 0.15786E+01_JPRB, 0.15498E+01_JPRB ,&
&  0.15644E+01_JPRB, 0.15353E+01_JPRB, 0.14850E+01_JPRB, 0.12381E+01_JPRB, 0.10514E+01_JPRB, 0.95133E+00_JPRB, 0.87664E+00_JPRB ,&
&  0.80594E+00_JPRB, 0.75292E+00_JPRB, 0.71358E+00_JPRB, 0.70334E+00_JPRB, 0.73533E+00_JPRB, 0.80364E+00_JPRB, 0.88437E+00_JPRB ,&
&  0.10484E+01_JPRB, 0.13251E+01_JPRB, 0.17227E+01_JPRB, 0.22121E+01_JPRB, 0.31127E+01_JPRB, 0.37747E+01_JPRB, 0.36087E+01_JPRB ,&
&  0.33919E+01_JPRB, 0.31200E+01_JPRB, 0.27595E+01_JPRB, 0.24544E+01_JPRB, 0.18567E+01_JPRB, 0.18022E+01_JPRB, 0.15967E+01_JPRB ,&
&  0.14025E+01_JPRB, 0.12617E+01_JPRB, 0.11832E+01_JPRB, 0.11365E+01_JPRB, 0.10693E+01_JPRB, 0.97435E+00_JPRB, 0.89499E+00_JPRB ,&
&  0.84933E+00_JPRB, 0.83322E+00_JPRB, 0.82620E+00_JPRB, 0.81444E+00_JPRB, 0.80563E+00_JPRB, 0.80245E+00_JPRB, 0.80212E+00_JPRB ,&
&  0.79880E+00_JPRB, 0.79862E+00_JPRB, 0.78696E+00_JPRB, 0.79900E+00_JPRB, 0.79943E+00_JPRB, 0.79485E+00_JPRB, 0.78422E+00_JPRB ,&
&  0.78794E+00_JPRB, 0.80608E+00_JPRB, 0.81469E+00_JPRB, 0.81367E+00_JPRB, 0.81082E+00_JPRB, 0.81106E+00_JPRB, 0.80343E+00_JPRB ,&
&  0.80165E+00_JPRB, 0.77774E+00_JPRB, 0.74519E+00_JPRB, 0.73101E+00_JPRB, 0.72049E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.70366E+00_JPRB, 0.68763E+00_JPRB, 0.67571E+00_JPRB, 0.66163E+00_JPRB, 0.64875E+00_JPRB, 0.64221E+00_JPRB, 0.63600E+00_JPRB ,&
&  0.62433E+00_JPRB, 0.59711E+00_JPRB, 0.56850E+00_JPRB, 0.55498E+00_JPRB, 0.55030E+00_JPRB, 0.53963E+00_JPRB, 0.54106E+00_JPRB ,&
&  0.54298E+00_JPRB, 0.54793E+00_JPRB, 0.54393E+00_JPRB, 0.54138E+00_JPRB, 0.54006E+00_JPRB, 0.53880E+00_JPRB, 0.53553E+00_JPRB ,&
&  0.52914E+00_JPRB, 0.52584E+00_JPRB, 0.52971E+00_JPRB, 0.53505E+00_JPRB, 0.54485E+00_JPRB, 0.56679E+00_JPRB, 0.59091E+00_JPRB ,&
&  0.68560E+00_JPRB, 0.77508E+00_JPRB, 0.85094E+00_JPRB, 0.14036E+01_JPRB, 0.16293E+01_JPRB, 0.17092E+01_JPRB, 0.16091E+01_JPRB ,&
&  0.16240E+01_JPRB, 0.17935E+01_JPRB, 0.15549E+01_JPRB, 0.12238E+01_JPRB, 0.98893E+00_JPRB, 0.88183E+00_JPRB, 0.81408E+00_JPRB ,&
&  0.75774E+00_JPRB, 0.70658E+00_JPRB, 0.67402E+00_JPRB, 0.65523E+00_JPRB, 0.66214E+00_JPRB, 0.68182E+00_JPRB, 0.72407E+00_JPRB ,&
&  0.80890E+00_JPRB, 0.99599E+00_JPRB, 0.12776E+01_JPRB, 0.15212E+01_JPRB, 0.21809E+01_JPRB, 0.29447E+01_JPRB, 0.33910E+01_JPRB ,&
&  0.36174E+01_JPRB, 0.36113E+01_JPRB, 0.29837E+01_JPRB, 0.25814E+01_JPRB, 0.21145E+01_JPRB, 0.18672E+01_JPRB, 0.16189E+01_JPRB ,&
&  0.14459E+01_JPRB, 0.13097E+01_JPRB, 0.12111E+01_JPRB, 0.11545E+01_JPRB, 0.10999E+01_JPRB, 0.10344E+01_JPRB, 0.95303E+00_JPRB ,&
&  0.88142E+00_JPRB, 0.84945E+00_JPRB, 0.83634E+00_JPRB, 0.82543E+00_JPRB, 0.82107E+00_JPRB, 0.81529E+00_JPRB, 0.80875E+00_JPRB ,&
&  0.80187E+00_JPRB, 0.79309E+00_JPRB, 0.77144E+00_JPRB, 0.77517E+00_JPRB, 0.76570E+00_JPRB, 0.74220E+00_JPRB, 0.73996E+00_JPRB ,&
&  0.76938E+00_JPRB, 0.79256E+00_JPRB, 0.79635E+00_JPRB, 0.77944E+00_JPRB, 0.79371E+00_JPRB, 0.80977E+00_JPRB, 0.77535E+00_JPRB ,&
&  0.76493E+00_JPRB, 0.75331E+00_JPRB, 0.73253E+00_JPRB, 0.71982E+00_JPRB, 0.71354E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.69628E+00_JPRB, 0.67324E+00_JPRB, 0.65717E+00_JPRB, 0.64386E+00_JPRB, 0.63204E+00_JPRB, 0.62159E+00_JPRB, 0.61879E+00_JPRB ,&
&  0.61544E+00_JPRB, 0.60789E+00_JPRB, 0.58915E+00_JPRB, 0.56732E+00_JPRB, 0.55714E+00_JPRB, 0.55371E+00_JPRB, 0.54255E+00_JPRB ,&
&  0.53805E+00_JPRB, 0.53837E+00_JPRB, 0.53612E+00_JPRB, 0.52901E+00_JPRB, 0.52289E+00_JPRB, 0.51888E+00_JPRB, 0.51604E+00_JPRB ,&
&  0.50501E+00_JPRB, 0.49945E+00_JPRB, 0.50019E+00_JPRB, 0.50103E+00_JPRB, 0.50408E+00_JPRB, 0.52358E+00_JPRB, 0.54796E+00_JPRB ,&
&  0.60100E+00_JPRB, 0.79254E+00_JPRB, 0.82925E+00_JPRB, 0.11671E+01_JPRB, 0.13168E+01_JPRB, 0.14770E+01_JPRB, 0.15594E+01_JPRB ,&
&  0.16627E+01_JPRB, 0.17793E+01_JPRB, 0.14766E+01_JPRB, 0.11709E+01_JPRB, 0.91322E+00_JPRB, 0.77544E+00_JPRB, 0.70959E+00_JPRB ,&
&  0.67579E+00_JPRB, 0.64473E+00_JPRB, 0.62247E+00_JPRB, 0.61622E+00_JPRB, 0.62087E+00_JPRB, 0.62966E+00_JPRB, 0.64121E+00_JPRB ,&
&  0.68611E+00_JPRB, 0.79580E+00_JPRB, 0.10509E+01_JPRB, 0.13698E+01_JPRB, 0.15470E+01_JPRB, 0.20676E+01_JPRB, 0.28401E+01_JPRB ,&
&  0.34723E+01_JPRB, 0.37824E+01_JPRB, 0.31396E+01_JPRB, 0.26117E+01_JPRB, 0.22187E+01_JPRB, 0.19223E+01_JPRB, 0.16699E+01_JPRB ,&
&  0.14753E+01_JPRB, 0.13301E+01_JPRB, 0.12257E+01_JPRB, 0.11523E+01_JPRB, 0.11014E+01_JPRB, 0.10633E+01_JPRB, 0.10081E+01_JPRB ,&
&  0.94370E+00_JPRB, 0.89552E+00_JPRB, 0.86593E+00_JPRB, 0.84982E+00_JPRB, 0.84280E+00_JPRB, 0.82933E+00_JPRB, 0.80660E+00_JPRB ,&
&  0.78846E+00_JPRB, 0.78122E+00_JPRB, 0.76188E+00_JPRB, 0.76297E+00_JPRB, 0.75617E+00_JPRB, 0.72952E+00_JPRB, 0.72036E+00_JPRB ,&
&  0.75217E+00_JPRB, 0.76890E+00_JPRB, 0.77490E+00_JPRB, 0.77918E+00_JPRB, 0.79156E+00_JPRB, 0.83287E+00_JPRB, 0.78452E+00_JPRB ,&
&  0.75294E+00_JPRB, 0.73922E+00_JPRB, 0.72509E+00_JPRB, 0.71788E+00_JPRB, 0.70981E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.68632E+00_JPRB, 0.66196E+00_JPRB, 0.64232E+00_JPRB, 0.62307E+00_JPRB, 0.60886E+00_JPRB, 0.59419E+00_JPRB, 0.58451E+00_JPRB ,&
&  0.58040E+00_JPRB, 0.58033E+00_JPRB, 0.57804E+00_JPRB, 0.56911E+00_JPRB, 0.55405E+00_JPRB, 0.54254E+00_JPRB, 0.53489E+00_JPRB ,&
&  0.52714E+00_JPRB, 0.52563E+00_JPRB, 0.52482E+00_JPRB, 0.51697E+00_JPRB, 0.50830E+00_JPRB, 0.50117E+00_JPRB, 0.49350E+00_JPRB ,&
&  0.48647E+00_JPRB, 0.48243E+00_JPRB, 0.47929E+00_JPRB, 0.47657E+00_JPRB, 0.47520E+00_JPRB, 0.48492E+00_JPRB, 0.52865E+00_JPRB ,&
&  0.61862E+00_JPRB, 0.53615E+00_JPRB, 0.96405E+00_JPRB, 0.10763E+01_JPRB, 0.11202E+01_JPRB, 0.11922E+01_JPRB, 0.13716E+01_JPRB ,&
&  0.15633E+01_JPRB, 0.15368E+01_JPRB, 0.13446E+01_JPRB, 0.11073E+01_JPRB, 0.89616E+00_JPRB, 0.75402E+00_JPRB, 0.67925E+00_JPRB ,&
&  0.62643E+00_JPRB, 0.59175E+00_JPRB, 0.57665E+00_JPRB, 0.56815E+00_JPRB, 0.57141E+00_JPRB, 0.58258E+00_JPRB, 0.59209E+00_JPRB ,&
&  0.61783E+00_JPRB, 0.68088E+00_JPRB, 0.85480E+00_JPRB, 0.11876E+01_JPRB, 0.15124E+01_JPRB, 0.17657E+01_JPRB, 0.22716E+01_JPRB ,&
&  0.31132E+01_JPRB, 0.33202E+01_JPRB, 0.27543E+01_JPRB, 0.23361E+01_JPRB, 0.20668E+01_JPRB, 0.18712E+01_JPRB, 0.16795E+01_JPRB ,&
&  0.14957E+01_JPRB, 0.13199E+01_JPRB, 0.11931E+01_JPRB, 0.11059E+01_JPRB, 0.10557E+01_JPRB, 0.10357E+01_JPRB, 0.10186E+01_JPRB ,&
&  0.99601E+00_JPRB, 0.96339E+00_JPRB, 0.93125E+00_JPRB, 0.90080E+00_JPRB, 0.87647E+00_JPRB, 0.84977E+00_JPRB, 0.81152E+00_JPRB ,&
&  0.78097E+00_JPRB, 0.76577E+00_JPRB, 0.74568E+00_JPRB, 0.73420E+00_JPRB, 0.73357E+00_JPRB, 0.72862E+00_JPRB, 0.72861E+00_JPRB ,&
&  0.73967E+00_JPRB, 0.74444E+00_JPRB, 0.74697E+00_JPRB, 0.75705E+00_JPRB, 0.75161E+00_JPRB, 0.79324E+00_JPRB, 0.77526E+00_JPRB ,&
&  0.76638E+00_JPRB, 0.74739E+00_JPRB, 0.72354E+00_JPRB, 0.71063E+00_JPRB, 0.70229E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.67580E+00_JPRB, 0.65752E+00_JPRB, 0.63075E+00_JPRB, 0.60723E+00_JPRB, 0.58909E+00_JPRB, 0.57032E+00_JPRB, 0.55722E+00_JPRB ,&
&  0.54759E+00_JPRB, 0.54809E+00_JPRB, 0.55162E+00_JPRB, 0.55321E+00_JPRB, 0.54772E+00_JPRB, 0.53394E+00_JPRB, 0.52016E+00_JPRB ,&
&  0.51083E+00_JPRB, 0.50859E+00_JPRB, 0.50851E+00_JPRB, 0.50503E+00_JPRB, 0.49913E+00_JPRB, 0.49123E+00_JPRB, 0.48369E+00_JPRB ,&
&  0.47659E+00_JPRB, 0.47083E+00_JPRB, 0.46521E+00_JPRB, 0.46137E+00_JPRB, 0.45683E+00_JPRB, 0.46132E+00_JPRB, 0.50778E+00_JPRB ,&
&  0.63510E+00_JPRB, 0.75090E+00_JPRB, 0.90584E+00_JPRB, 0.88082E+00_JPRB, 0.89001E+00_JPRB, 0.93733E+00_JPRB, 0.10777E+01_JPRB ,&
&  0.11590E+01_JPRB, 0.11828E+01_JPRB, 0.11235E+01_JPRB, 0.10023E+01_JPRB, 0.85701E+00_JPRB, 0.74877E+00_JPRB, 0.66835E+00_JPRB ,&
&  0.60230E+00_JPRB, 0.56027E+00_JPRB, 0.53648E+00_JPRB, 0.53013E+00_JPRB, 0.53473E+00_JPRB, 0.54553E+00_JPRB, 0.55148E+00_JPRB ,&
&  0.56634E+00_JPRB, 0.60188E+00_JPRB, 0.73314E+00_JPRB, 0.10378E+01_JPRB, 0.15473E+01_JPRB, 0.16192E+01_JPRB, 0.21950E+01_JPRB ,&
&  0.28512E+01_JPRB, 0.25516E+01_JPRB, 0.22968E+01_JPRB, 0.20862E+01_JPRB, 0.19212E+01_JPRB, 0.17730E+01_JPRB, 0.15463E+01_JPRB ,&
&  0.13669E+01_JPRB, 0.12048E+01_JPRB, 0.10855E+01_JPRB, 0.10190E+01_JPRB, 0.97818E+00_JPRB, 0.96126E+00_JPRB, 0.95776E+00_JPRB ,&
&  0.95362E+00_JPRB, 0.94441E+00_JPRB, 0.93684E+00_JPRB, 0.91646E+00_JPRB, 0.88537E+00_JPRB, 0.85093E+00_JPRB, 0.81378E+00_JPRB ,&
&  0.78151E+00_JPRB, 0.75335E+00_JPRB, 0.72658E+00_JPRB, 0.70313E+00_JPRB, 0.70336E+00_JPRB, 0.71101E+00_JPRB, 0.71134E+00_JPRB ,&
&  0.70330E+00_JPRB, 0.70035E+00_JPRB, 0.71334E+00_JPRB, 0.71392E+00_JPRB, 0.71768E+00_JPRB, 0.76118E+00_JPRB, 0.75974E+00_JPRB ,&
&  0.75593E+00_JPRB, 0.73441E+00_JPRB, 0.71240E+00_JPRB, 0.69399E+00_JPRB, 0.68689E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.66979E+00_JPRB, 0.64630E+00_JPRB, 0.61893E+00_JPRB, 0.59351E+00_JPRB, 0.57067E+00_JPRB, 0.55313E+00_JPRB, 0.54244E+00_JPRB ,&
&  0.53346E+00_JPRB, 0.52775E+00_JPRB, 0.52915E+00_JPRB, 0.53355E+00_JPRB, 0.53284E+00_JPRB, 0.52424E+00_JPRB, 0.51175E+00_JPRB ,&
&  0.49924E+00_JPRB, 0.49009E+00_JPRB, 0.48381E+00_JPRB, 0.48173E+00_JPRB, 0.47772E+00_JPRB, 0.47349E+00_JPRB, 0.46647E+00_JPRB ,&
&  0.46127E+00_JPRB, 0.45650E+00_JPRB, 0.44920E+00_JPRB, 0.44162E+00_JPRB, 0.43652E+00_JPRB, 0.43795E+00_JPRB, 0.47657E+00_JPRB ,&
&  0.58152E+00_JPRB, 0.67489E+00_JPRB, 0.68669E+00_JPRB, 0.71772E+00_JPRB, 0.76203E+00_JPRB, 0.78693E+00_JPRB, 0.84856E+00_JPRB ,&
&  0.88415E+00_JPRB, 0.90148E+00_JPRB, 0.90727E+00_JPRB, 0.86467E+00_JPRB, 0.78448E+00_JPRB, 0.72125E+00_JPRB, 0.65714E+00_JPRB ,&
&  0.59797E+00_JPRB, 0.55164E+00_JPRB, 0.52218E+00_JPRB, 0.51065E+00_JPRB, 0.51366E+00_JPRB, 0.52113E+00_JPRB, 0.52406E+00_JPRB ,&
&  0.52943E+00_JPRB, 0.53787E+00_JPRB, 0.58866E+00_JPRB, 0.71270E+00_JPRB, 0.10300E+01_JPRB, 0.12410E+01_JPRB, 0.15823E+01_JPRB ,&
&  0.16737E+01_JPRB, 0.16470E+01_JPRB, 0.15943E+01_JPRB, 0.15317E+01_JPRB, 0.14598E+01_JPRB, 0.13555E+01_JPRB, 0.12652E+01_JPRB ,&
&  0.11594E+01_JPRB, 0.10530E+01_JPRB, 0.96993E+00_JPRB, 0.91441E+00_JPRB, 0.88027E+00_JPRB, 0.86055E+00_JPRB, 0.85407E+00_JPRB ,&
&  0.84228E+00_JPRB, 0.83877E+00_JPRB, 0.82837E+00_JPRB, 0.82103E+00_JPRB, 0.81651E+00_JPRB, 0.79655E+00_JPRB, 0.76389E+00_JPRB ,&
&  0.72921E+00_JPRB, 0.69976E+00_JPRB, 0.67968E+00_JPRB, 0.66991E+00_JPRB, 0.66583E+00_JPRB, 0.66189E+00_JPRB, 0.65782E+00_JPRB ,&
&  0.66390E+00_JPRB, 0.66826E+00_JPRB, 0.67224E+00_JPRB, 0.66989E+00_JPRB, 0.66533E+00_JPRB, 0.69788E+00_JPRB, 0.69870E+00_JPRB ,&
&  0.69979E+00_JPRB, 0.69756E+00_JPRB, 0.68653E+00_JPRB, 0.66997E+00_JPRB, 0.68037E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.67961E+00_JPRB, 0.65800E+00_JPRB, 0.62786E+00_JPRB, 0.59788E+00_JPRB, 0.56969E+00_JPRB, 0.54755E+00_JPRB, 0.53425E+00_JPRB ,&
&  0.52559E+00_JPRB, 0.51999E+00_JPRB, 0.51692E+00_JPRB, 0.51492E+00_JPRB, 0.51187E+00_JPRB, 0.50857E+00_JPRB, 0.50298E+00_JPRB ,&
&  0.49587E+00_JPRB, 0.48720E+00_JPRB, 0.47934E+00_JPRB, 0.47403E+00_JPRB, 0.46955E+00_JPRB, 0.46696E+00_JPRB, 0.46367E+00_JPRB ,&
&  0.45670E+00_JPRB, 0.44806E+00_JPRB, 0.43703E+00_JPRB, 0.42581E+00_JPRB, 0.41913E+00_JPRB, 0.41876E+00_JPRB, 0.44421E+00_JPRB ,&
&  0.52503E+00_JPRB, 0.53146E+00_JPRB, 0.56886E+00_JPRB, 0.60326E+00_JPRB, 0.64073E+00_JPRB, 0.67185E+00_JPRB, 0.69768E+00_JPRB ,&
&  0.71452E+00_JPRB, 0.73160E+00_JPRB, 0.75695E+00_JPRB, 0.75071E+00_JPRB, 0.70400E+00_JPRB, 0.66152E+00_JPRB, 0.62068E+00_JPRB ,&
&  0.58206E+00_JPRB, 0.54714E+00_JPRB, 0.51770E+00_JPRB, 0.50592E+00_JPRB, 0.50418E+00_JPRB, 0.50705E+00_JPRB, 0.50687E+00_JPRB ,&
&  0.50868E+00_JPRB, 0.50696E+00_JPRB, 0.51301E+00_JPRB, 0.53652E+00_JPRB, 0.58789E+00_JPRB, 0.68614E+00_JPRB, 0.85355E+00_JPRB ,&
&  0.93473E+00_JPRB, 0.94910E+00_JPRB, 0.97060E+00_JPRB, 0.98842E+00_JPRB, 0.96789E+00_JPRB, 0.93936E+00_JPRB, 0.91069E+00_JPRB ,&
&  0.88178E+00_JPRB, 0.85156E+00_JPRB, 0.82556E+00_JPRB, 0.79345E+00_JPRB, 0.76658E+00_JPRB, 0.74803E+00_JPRB, 0.73569E+00_JPRB ,&
&  0.72622E+00_JPRB, 0.72248E+00_JPRB, 0.71501E+00_JPRB, 0.69824E+00_JPRB, 0.68670E+00_JPRB, 0.67463E+00_JPRB, 0.65482E+00_JPRB ,&
&  0.62791E+00_JPRB, 0.61151E+00_JPRB, 0.60356E+00_JPRB, 0.60345E+00_JPRB, 0.60754E+00_JPRB, 0.61268E+00_JPRB, 0.61393E+00_JPRB ,&
&  0.61786E+00_JPRB, 0.63047E+00_JPRB, 0.62300E+00_JPRB, 0.61747E+00_JPRB, 0.64655E+00_JPRB, 0.64766E+00_JPRB, 0.65182E+00_JPRB ,&
&  0.65613E+00_JPRB, 0.65644E+00_JPRB, 0.64997E+00_JPRB, 0.63340E+00_JPRB, 0.67786E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.58482E+00_JPRB, 0.58897E+00_JPRB, 0.57550E+00_JPRB, 0.55798E+00_JPRB, 0.54115E+00_JPRB, 0.52799E+00_JPRB, 0.51697E+00_JPRB ,&
&  0.50938E+00_JPRB, 0.50400E+00_JPRB, 0.50060E+00_JPRB, 0.49851E+00_JPRB, 0.49530E+00_JPRB, 0.49007E+00_JPRB, 0.48430E+00_JPRB ,&
&  0.47950E+00_JPRB, 0.47373E+00_JPRB, 0.46473E+00_JPRB, 0.45855E+00_JPRB, 0.45498E+00_JPRB, 0.45039E+00_JPRB, 0.44569E+00_JPRB ,&
&  0.43988E+00_JPRB, 0.42827E+00_JPRB, 0.41667E+00_JPRB, 0.40917E+00_JPRB, 0.40344E+00_JPRB, 0.40107E+00_JPRB, 0.41666E+00_JPRB ,&
&  0.47030E+00_JPRB, 0.44595E+00_JPRB, 0.48043E+00_JPRB, 0.51831E+00_JPRB, 0.55976E+00_JPRB, 0.58600E+00_JPRB, 0.60512E+00_JPRB ,&
&  0.61782E+00_JPRB, 0.63458E+00_JPRB, 0.64743E+00_JPRB, 0.66385E+00_JPRB, 0.64117E+00_JPRB, 0.60652E+00_JPRB, 0.57784E+00_JPRB ,&
&  0.55147E+00_JPRB, 0.52708E+00_JPRB, 0.50709E+00_JPRB, 0.49372E+00_JPRB, 0.49008E+00_JPRB, 0.48875E+00_JPRB, 0.48482E+00_JPRB ,&
&  0.48466E+00_JPRB, 0.48165E+00_JPRB, 0.47981E+00_JPRB, 0.48575E+00_JPRB, 0.48549E+00_JPRB, 0.51211E+00_JPRB, 0.55477E+00_JPRB ,&
&  0.59260E+00_JPRB, 0.64400E+00_JPRB, 0.67114E+00_JPRB, 0.68954E+00_JPRB, 0.68677E+00_JPRB, 0.68171E+00_JPRB, 0.67531E+00_JPRB ,&
&  0.66909E+00_JPRB, 0.66209E+00_JPRB, 0.65421E+00_JPRB, 0.64318E+00_JPRB, 0.63045E+00_JPRB, 0.61997E+00_JPRB, 0.61094E+00_JPRB ,&
&  0.60359E+00_JPRB, 0.59732E+00_JPRB, 0.59322E+00_JPRB, 0.58101E+00_JPRB, 0.56192E+00_JPRB, 0.54602E+00_JPRB, 0.53379E+00_JPRB ,&
&  0.52387E+00_JPRB, 0.52074E+00_JPRB, 0.52190E+00_JPRB, 0.52919E+00_JPRB, 0.53657E+00_JPRB, 0.54489E+00_JPRB, 0.55010E+00_JPRB ,&
&  0.55477E+00_JPRB, 0.56596E+00_JPRB, 0.58217E+00_JPRB, 0.60151E+00_JPRB, 0.60937E+00_JPRB, 0.61388E+00_JPRB, 0.61697E+00_JPRB ,&
&  0.61777E+00_JPRB, 0.61703E+00_JPRB, 0.60816E+00_JPRB, 0.59722E+00_JPRB, 0.57343E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.53119E+00_JPRB, 0.53572E+00_JPRB, 0.53051E+00_JPRB, 0.51935E+00_JPRB, 0.50916E+00_JPRB, 0.49969E+00_JPRB, 0.49217E+00_JPRB ,&
&  0.48147E+00_JPRB, 0.47249E+00_JPRB, 0.46938E+00_JPRB, 0.46929E+00_JPRB, 0.46858E+00_JPRB, 0.46467E+00_JPRB, 0.45923E+00_JPRB ,&
&  0.45329E+00_JPRB, 0.44650E+00_JPRB, 0.43854E+00_JPRB, 0.43061E+00_JPRB, 0.42409E+00_JPRB, 0.41660E+00_JPRB, 0.41080E+00_JPRB ,&
&  0.40545E+00_JPRB, 0.40085E+00_JPRB, 0.39341E+00_JPRB, 0.38408E+00_JPRB, 0.38006E+00_JPRB, 0.37510E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.43705E+00_JPRB, 0.37959E+00_JPRB, 0.40397E+00_JPRB, 0.44753E+00_JPRB, 0.48763E+00_JPRB, 0.53231E+00_JPRB, 0.55441E+00_JPRB ,&
&  0.56880E+00_JPRB, 0.57786E+00_JPRB, 0.58263E+00_JPRB, 0.59122E+00_JPRB, 0.58941E+00_JPRB, 0.56787E+00_JPRB, 0.53961E+00_JPRB ,&
&  0.51887E+00_JPRB, 0.49854E+00_JPRB, 0.48537E+00_JPRB, 0.47271E+00_JPRB, 0.46723E+00_JPRB, 0.46561E+00_JPRB, 0.45946E+00_JPRB ,&
&  0.45369E+00_JPRB, 0.45535E+00_JPRB, 0.45855E+00_JPRB, 0.46497E+00_JPRB, 0.46968E+00_JPRB, 0.47453E+00_JPRB, 0.46874E+00_JPRB ,&
&  0.46388E+00_JPRB, 0.48181E+00_JPRB, 0.51166E+00_JPRB, 0.52854E+00_JPRB, 0.53614E+00_JPRB, 0.53343E+00_JPRB, 0.52617E+00_JPRB ,&
&  0.52146E+00_JPRB, 0.51846E+00_JPRB, 0.51918E+00_JPRB, 0.51681E+00_JPRB, 0.51245E+00_JPRB, 0.50495E+00_JPRB, 0.49623E+00_JPRB ,&
&  0.48811E+00_JPRB, 0.48122E+00_JPRB, 0.47582E+00_JPRB, 0.47065E+00_JPRB, 0.46417E+00_JPRB, 0.45482E+00_JPRB, 0.44980E+00_JPRB ,&
&  0.44785E+00_JPRB, 0.44917E+00_JPRB, 0.45354E+00_JPRB, 0.46053E+00_JPRB, 0.47016E+00_JPRB, 0.47896E+00_JPRB, 0.48957E+00_JPRB ,&
&  0.49981E+00_JPRB, 0.51113E+00_JPRB, 0.52559E+00_JPRB, 0.53362E+00_JPRB, 0.55923E+00_JPRB, 0.56677E+00_JPRB, 0.57415E+00_JPRB ,&
&  0.57668E+00_JPRB, 0.57667E+00_JPRB, 0.57619E+00_JPRB, 0.50599E+00_JPRB, 0.52588E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.48726E+00_JPRB, 0.48803E+00_JPRB, 0.48369E+00_JPRB, 0.47767E+00_JPRB, 0.47236E+00_JPRB, 0.46652E+00_JPRB, 0.46156E+00_JPRB ,&
&  0.45410E+00_JPRB, 0.44489E+00_JPRB, 0.43836E+00_JPRB, 0.43572E+00_JPRB, 0.43449E+00_JPRB, 0.43168E+00_JPRB, 0.42645E+00_JPRB ,&
&  0.41974E+00_JPRB, 0.41265E+00_JPRB, 0.40599E+00_JPRB, 0.39802E+00_JPRB, 0.39060E+00_JPRB, 0.38382E+00_JPRB, 0.37837E+00_JPRB ,&
&  0.37288E+00_JPRB, 0.36693E+00_JPRB, 0.36103E+00_JPRB, 0.35701E+00_JPRB, 0.35377E+00_JPRB, 0.35238E+00_JPRB, 0.36399E+00_JPRB ,&
&  0.40001E+00_JPRB, 0.34451E+00_JPRB, 0.36320E+00_JPRB, 0.39013E+00_JPRB, 0.42933E+00_JPRB, 0.47860E+00_JPRB, 0.50666E+00_JPRB ,&
&  0.52137E+00_JPRB, 0.53005E+00_JPRB, 0.53178E+00_JPRB, 0.53235E+00_JPRB, 0.53497E+00_JPRB, 0.52795E+00_JPRB, 0.51114E+00_JPRB ,&
&  0.49352E+00_JPRB, 0.47632E+00_JPRB, 0.46291E+00_JPRB, 0.45302E+00_JPRB, 0.44501E+00_JPRB, 0.43958E+00_JPRB, 0.43155E+00_JPRB ,&
&  0.42117E+00_JPRB, 0.41880E+00_JPRB, 0.41971E+00_JPRB, 0.41977E+00_JPRB, 0.42007E+00_JPRB, 0.41755E+00_JPRB, 0.40629E+00_JPRB ,&
&  0.39689E+00_JPRB, 0.39244E+00_JPRB, 0.39366E+00_JPRB, 0.39989E+00_JPRB, 0.40617E+00_JPRB, 0.40928E+00_JPRB, 0.40724E+00_JPRB ,&
&  0.40341E+00_JPRB, 0.39770E+00_JPRB, 0.39524E+00_JPRB, 0.39483E+00_JPRB, 0.39464E+00_JPRB, 0.39577E+00_JPRB, 0.39559E+00_JPRB ,&
&  0.39377E+00_JPRB, 0.39356E+00_JPRB, 0.39574E+00_JPRB, 0.39799E+00_JPRB, 0.39859E+00_JPRB, 0.39752E+00_JPRB, 0.39578E+00_JPRB ,&
&  0.39323E+00_JPRB, 0.39170E+00_JPRB, 0.39321E+00_JPRB, 0.39882E+00_JPRB, 0.40662E+00_JPRB, 0.41542E+00_JPRB, 0.42512E+00_JPRB ,&
&  0.43599E+00_JPRB, 0.44750E+00_JPRB, 0.46048E+00_JPRB, 0.48286E+00_JPRB, 0.49457E+00_JPRB, 0.50661E+00_JPRB, 0.51777E+00_JPRB ,&
&  0.51929E+00_JPRB, 0.51678E+00_JPRB, 0.43891E+00_JPRB, 0.47102E+00_JPRB, 0.47997E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.46061E+00_JPRB, 0.45219E+00_JPRB, 0.44208E+00_JPRB, 0.43579E+00_JPRB, 0.43373E+00_JPRB, 0.43111E+00_JPRB, 0.42825E+00_JPRB ,&
&  0.42328E+00_JPRB, 0.41752E+00_JPRB, 0.41188E+00_JPRB, 0.40682E+00_JPRB, 0.40265E+00_JPRB, 0.39964E+00_JPRB, 0.39573E+00_JPRB ,&
&  0.39000E+00_JPRB, 0.38347E+00_JPRB, 0.37690E+00_JPRB, 0.36951E+00_JPRB, 0.35978E+00_JPRB, 0.35204E+00_JPRB, 0.34593E+00_JPRB ,&
&  0.34111E+00_JPRB, 0.33730E+00_JPRB, 0.33597E+00_JPRB, 0.33428E+00_JPRB, 0.33243E+00_JPRB, 0.33266E+00_JPRB, 0.34316E+00_JPRB ,&
&  0.33913E+00_JPRB, 0.31402E+00_JPRB, 0.33076E+00_JPRB, 0.35339E+00_JPRB, 0.38745E+00_JPRB, 0.43873E+00_JPRB, 0.47379E+00_JPRB ,&
&  0.49314E+00_JPRB, 0.50248E+00_JPRB, 0.50550E+00_JPRB, 0.50232E+00_JPRB, 0.49756E+00_JPRB, 0.49674E+00_JPRB, 0.49092E+00_JPRB ,&
&  0.47814E+00_JPRB, 0.46246E+00_JPRB, 0.44703E+00_JPRB, 0.43560E+00_JPRB, 0.42363E+00_JPRB, 0.41440E+00_JPRB, 0.40458E+00_JPRB ,&
&  0.39550E+00_JPRB, 0.38979E+00_JPRB, 0.38789E+00_JPRB, 0.38615E+00_JPRB, 0.38360E+00_JPRB, 0.37708E+00_JPRB, 0.36887E+00_JPRB ,&
&  0.36303E+00_JPRB, 0.35810E+00_JPRB, 0.35299E+00_JPRB, 0.34843E+00_JPRB, 0.34568E+00_JPRB, 0.34381E+00_JPRB, 0.34183E+00_JPRB ,&
&  0.34147E+00_JPRB, 0.34215E+00_JPRB, 0.34378E+00_JPRB, 0.34563E+00_JPRB, 0.34660E+00_JPRB, 0.34608E+00_JPRB, 0.34629E+00_JPRB ,&
&  0.34713E+00_JPRB, 0.34544E+00_JPRB, 0.34532E+00_JPRB, 0.34907E+00_JPRB, 0.35379E+00_JPRB, 0.35695E+00_JPRB, 0.35809E+00_JPRB ,&
&  0.35699E+00_JPRB, 0.35634E+00_JPRB, 0.35727E+00_JPRB, 0.36028E+00_JPRB, 0.36423E+00_JPRB, 0.37009E+00_JPRB, 0.37726E+00_JPRB ,&
&  0.38477E+00_JPRB, 0.39415E+00_JPRB, 0.40696E+00_JPRB, 0.42544E+00_JPRB, 0.43974E+00_JPRB, 0.45074E+00_JPRB, 0.45941E+00_JPRB ,&
&  0.46083E+00_JPRB, 0.46061E+00_JPRB, 0.46484E+00_JPRB, 0.46459E+00_JPRB, 0.46404E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.40991E+00_JPRB, 0.40468E+00_JPRB, 0.39833E+00_JPRB, 0.39321E+00_JPRB, 0.39105E+00_JPRB, 0.38946E+00_JPRB, 0.38603E+00_JPRB ,&
&  0.38291E+00_JPRB, 0.38005E+00_JPRB, 0.37699E+00_JPRB, 0.37435E+00_JPRB, 0.37205E+00_JPRB, 0.37027E+00_JPRB, 0.36695E+00_JPRB ,&
&  0.36156E+00_JPRB, 0.35474E+00_JPRB, 0.34716E+00_JPRB, 0.34092E+00_JPRB, 0.33323E+00_JPRB, 0.32531E+00_JPRB, 0.31948E+00_JPRB ,&
&  0.31465E+00_JPRB, 0.31141E+00_JPRB, 0.31034E+00_JPRB, 0.30877E+00_JPRB, 0.30976E+00_JPRB, 0.31303E+00_JPRB, 0.32158E+00_JPRB ,&
&  0.33266E+00_JPRB, 0.31339E+00_JPRB, 0.32909E+00_JPRB, 0.34283E+00_JPRB, 0.37126E+00_JPRB, 0.41055E+00_JPRB, 0.43767E+00_JPRB ,&
&  0.45571E+00_JPRB, 0.46985E+00_JPRB, 0.47992E+00_JPRB, 0.48238E+00_JPRB, 0.47601E+00_JPRB, 0.47165E+00_JPRB, 0.46850E+00_JPRB ,&
&  0.46038E+00_JPRB, 0.44856E+00_JPRB, 0.43265E+00_JPRB, 0.41588E+00_JPRB, 0.40057E+00_JPRB, 0.38922E+00_JPRB, 0.38018E+00_JPRB ,&
&  0.37294E+00_JPRB, 0.36677E+00_JPRB, 0.36189E+00_JPRB, 0.35458E+00_JPRB, 0.34623E+00_JPRB, 0.33780E+00_JPRB, 0.33005E+00_JPRB ,&
&  0.32247E+00_JPRB, 0.31499E+00_JPRB, 0.30837E+00_JPRB, 0.30279E+00_JPRB, 0.29974E+00_JPRB, 0.29833E+00_JPRB, 0.29787E+00_JPRB ,&
&  0.29831E+00_JPRB, 0.30016E+00_JPRB, 0.30248E+00_JPRB, 0.30486E+00_JPRB, 0.30683E+00_JPRB, 0.30711E+00_JPRB, 0.30516E+00_JPRB ,&
&  0.30476E+00_JPRB, 0.30459E+00_JPRB, 0.30500E+00_JPRB, 0.30683E+00_JPRB, 0.30800E+00_JPRB, 0.30953E+00_JPRB, 0.31254E+00_JPRB ,&
&  0.31519E+00_JPRB, 0.31662E+00_JPRB, 0.31790E+00_JPRB, 0.31943E+00_JPRB, 0.32142E+00_JPRB, 0.32473E+00_JPRB, 0.32966E+00_JPRB ,&
&  0.33544E+00_JPRB, 0.34196E+00_JPRB, 0.35144E+00_JPRB, 0.36405E+00_JPRB, 0.37621E+00_JPRB, 0.38665E+00_JPRB, 0.39527E+00_JPRB ,&
&  0.39876E+00_JPRB, 0.40160E+00_JPRB, 0.40666E+00_JPRB, 0.40953E+00_JPRB, 0.41193E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.36857E+00_JPRB, 0.36594E+00_JPRB, 0.36291E+00_JPRB, 0.36055E+00_JPRB, 0.35903E+00_JPRB, 0.35587E+00_JPRB, 0.35194E+00_JPRB ,&
&  0.34946E+00_JPRB, 0.34720E+00_JPRB, 0.34567E+00_JPRB, 0.34298E+00_JPRB, 0.34173E+00_JPRB, 0.34111E+00_JPRB, 0.33752E+00_JPRB ,&
&  0.33307E+00_JPRB, 0.32651E+00_JPRB, 0.31998E+00_JPRB, 0.31404E+00_JPRB, 0.30836E+00_JPRB, 0.30308E+00_JPRB, 0.29724E+00_JPRB ,&
&  0.29182E+00_JPRB, 0.28782E+00_JPRB, 0.28574E+00_JPRB, 0.28398E+00_JPRB, 0.28543E+00_JPRB, 0.29142E+00_JPRB, 0.30105E+00_JPRB ,&
&  0.30724E+00_JPRB, 0.30963E+00_JPRB, 0.32095E+00_JPRB, 0.33234E+00_JPRB, 0.35372E+00_JPRB, 0.38238E+00_JPRB, 0.40291E+00_JPRB ,&
&  0.41631E+00_JPRB, 0.43112E+00_JPRB, 0.44713E+00_JPRB, 0.45666E+00_JPRB, 0.45743E+00_JPRB, 0.45169E+00_JPRB, 0.44736E+00_JPRB ,&
&  0.44178E+00_JPRB, 0.43130E+00_JPRB, 0.41999E+00_JPRB, 0.40589E+00_JPRB, 0.39086E+00_JPRB, 0.37845E+00_JPRB, 0.36858E+00_JPRB ,&
&  0.35950E+00_JPRB, 0.35072E+00_JPRB, 0.34258E+00_JPRB, 0.33420E+00_JPRB, 0.32503E+00_JPRB, 0.31528E+00_JPRB, 0.30497E+00_JPRB ,&
&  0.29493E+00_JPRB, 0.28641E+00_JPRB, 0.27981E+00_JPRB, 0.27646E+00_JPRB, 0.27472E+00_JPRB, 0.27326E+00_JPRB, 0.27225E+00_JPRB ,&
&  0.27255E+00_JPRB, 0.27378E+00_JPRB, 0.27566E+00_JPRB, 0.27778E+00_JPRB, 0.27978E+00_JPRB, 0.28050E+00_JPRB, 0.27878E+00_JPRB ,&
&  0.27682E+00_JPRB, 0.27651E+00_JPRB, 0.27693E+00_JPRB, 0.27499E+00_JPRB, 0.27271E+00_JPRB, 0.27341E+00_JPRB, 0.27583E+00_JPRB ,&
&  0.28015E+00_JPRB, 0.28404E+00_JPRB, 0.28722E+00_JPRB, 0.28976E+00_JPRB, 0.29161E+00_JPRB, 0.29480E+00_JPRB, 0.29873E+00_JPRB ,&
&  0.30375E+00_JPRB, 0.31013E+00_JPRB, 0.31910E+00_JPRB, 0.33040E+00_JPRB, 0.34075E+00_JPRB, 0.34925E+00_JPRB, 0.35429E+00_JPRB ,&
&  0.35733E+00_JPRB, 0.35947E+00_JPRB, 0.36167E+00_JPRB, 0.36490E+00_JPRB, 0.36795E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.32749E+00_JPRB, 0.32751E+00_JPRB, 0.32820E+00_JPRB, 0.32849E+00_JPRB, 0.32587E+00_JPRB, 0.32315E+00_JPRB, 0.31995E+00_JPRB ,&
&  0.31863E+00_JPRB, 0.31773E+00_JPRB, 0.31754E+00_JPRB, 0.31494E+00_JPRB, 0.31306E+00_JPRB, 0.31213E+00_JPRB, 0.30898E+00_JPRB ,&
&  0.30583E+00_JPRB, 0.29846E+00_JPRB, 0.29047E+00_JPRB, 0.28451E+00_JPRB, 0.28072E+00_JPRB, 0.27814E+00_JPRB, 0.27301E+00_JPRB ,&
&  0.26938E+00_JPRB, 0.26652E+00_JPRB, 0.26523E+00_JPRB, 0.26544E+00_JPRB, 0.26765E+00_JPRB, 0.27359E+00_JPRB, 0.28344E+00_JPRB ,&
&  0.29060E+00_JPRB, 0.29961E+00_JPRB, 0.30010E+00_JPRB, 0.31419E+00_JPRB, 0.32818E+00_JPRB, 0.34723E+00_JPRB, 0.36347E+00_JPRB ,&
&  0.37476E+00_JPRB, 0.38369E+00_JPRB, 0.39552E+00_JPRB, 0.40731E+00_JPRB, 0.41267E+00_JPRB, 0.41448E+00_JPRB, 0.41336E+00_JPRB ,&
&  0.41001E+00_JPRB, 0.40470E+00_JPRB, 0.39851E+00_JPRB, 0.39082E+00_JPRB, 0.38031E+00_JPRB, 0.36813E+00_JPRB, 0.35603E+00_JPRB ,&
&  0.34498E+00_JPRB, 0.33413E+00_JPRB, 0.32340E+00_JPRB, 0.31179E+00_JPRB, 0.30019E+00_JPRB, 0.28949E+00_JPRB, 0.27875E+00_JPRB ,&
&  0.26979E+00_JPRB, 0.26289E+00_JPRB, 0.25783E+00_JPRB, 0.25427E+00_JPRB, 0.25164E+00_JPRB, 0.24907E+00_JPRB, 0.24792E+00_JPRB ,&
&  0.24811E+00_JPRB, 0.24971E+00_JPRB, 0.25210E+00_JPRB, 0.25405E+00_JPRB, 0.25642E+00_JPRB, 0.25692E+00_JPRB, 0.25428E+00_JPRB ,&
&  0.25145E+00_JPRB, 0.25044E+00_JPRB, 0.24666E+00_JPRB, 0.24447E+00_JPRB, 0.24413E+00_JPRB, 0.24466E+00_JPRB, 0.24659E+00_JPRB ,&
&  0.25037E+00_JPRB, 0.25561E+00_JPRB, 0.26047E+00_JPRB, 0.26473E+00_JPRB, 0.26841E+00_JPRB, 0.27211E+00_JPRB, 0.27641E+00_JPRB ,&
&  0.28099E+00_JPRB, 0.28543E+00_JPRB, 0.29236E+00_JPRB, 0.30020E+00_JPRB, 0.30691E+00_JPRB, 0.31407E+00_JPRB, 0.31840E+00_JPRB ,&
&  0.32002E+00_JPRB, 0.32063E+00_JPRB, 0.32259E+00_JPRB, 0.32554E+00_JPRB, 0.32737E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.29362E+00_JPRB, 0.29514E+00_JPRB, 0.29706E+00_JPRB, 0.29737E+00_JPRB, 0.29680E+00_JPRB, 0.29675E+00_JPRB, 0.29573E+00_JPRB ,&
&  0.29353E+00_JPRB, 0.29240E+00_JPRB, 0.29272E+00_JPRB, 0.29148E+00_JPRB, 0.28891E+00_JPRB, 0.28575E+00_JPRB, 0.28230E+00_JPRB ,&
&  0.27945E+00_JPRB, 0.27644E+00_JPRB, 0.27127E+00_JPRB, 0.26443E+00_JPRB, 0.25852E+00_JPRB, 0.25471E+00_JPRB, 0.25272E+00_JPRB ,&
&  0.25188E+00_JPRB, 0.25191E+00_JPRB, 0.25159E+00_JPRB, 0.25227E+00_JPRB, 0.25442E+00_JPRB, 0.25929E+00_JPRB, 0.26651E+00_JPRB ,&
&  0.27378E+00_JPRB, 0.28053E+00_JPRB, 0.28741E+00_JPRB, 0.29492E+00_JPRB, 0.30388E+00_JPRB, 0.31459E+00_JPRB, 0.32442E+00_JPRB ,&
&  0.33325E+00_JPRB, 0.33639E+00_JPRB, 0.33790E+00_JPRB, 0.34267E+00_JPRB, 0.35130E+00_JPRB, 0.35742E+00_JPRB, 0.36129E+00_JPRB ,&
&  0.36327E+00_JPRB, 0.36208E+00_JPRB, 0.35970E+00_JPRB, 0.35453E+00_JPRB, 0.34698E+00_JPRB, 0.33724E+00_JPRB, 0.32678E+00_JPRB ,&
&  0.31645E+00_JPRB, 0.30579E+00_JPRB, 0.29429E+00_JPRB, 0.28330E+00_JPRB, 0.27385E+00_JPRB, 0.26542E+00_JPRB, 0.25704E+00_JPRB ,&
&  0.24937E+00_JPRB, 0.24327E+00_JPRB, 0.23846E+00_JPRB, 0.23521E+00_JPRB, 0.23258E+00_JPRB, 0.23054E+00_JPRB, 0.23002E+00_JPRB ,&
&  0.23037E+00_JPRB, 0.23173E+00_JPRB, 0.23247E+00_JPRB, 0.23398E+00_JPRB, 0.23646E+00_JPRB, 0.23748E+00_JPRB, 0.23633E+00_JPRB ,&
&  0.23239E+00_JPRB, 0.22848E+00_JPRB, 0.22483E+00_JPRB, 0.22430E+00_JPRB, 0.22437E+00_JPRB, 0.22569E+00_JPRB, 0.22757E+00_JPRB ,&
&  0.23123E+00_JPRB, 0.23640E+00_JPRB, 0.24107E+00_JPRB, 0.24448E+00_JPRB, 0.24840E+00_JPRB, 0.25185E+00_JPRB, 0.25529E+00_JPRB ,&
&  0.25911E+00_JPRB, 0.26320E+00_JPRB, 0.26863E+00_JPRB, 0.27411E+00_JPRB, 0.27907E+00_JPRB, 0.28298E+00_JPRB, 0.28580E+00_JPRB ,&
&  0.28758E+00_JPRB, 0.28801E+00_JPRB, 0.28824E+00_JPRB, 0.28972E+00_JPRB, 0.29197E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.26347E+00_JPRB, 0.26556E+00_JPRB, 0.26831E+00_JPRB, 0.27059E+00_JPRB, 0.27141E+00_JPRB, 0.27091E+00_JPRB, 0.27011E+00_JPRB ,&
&  0.26995E+00_JPRB, 0.26988E+00_JPRB, 0.26849E+00_JPRB, 0.26610E+00_JPRB, 0.26369E+00_JPRB, 0.26115E+00_JPRB, 0.25934E+00_JPRB ,&
&  0.25754E+00_JPRB, 0.25547E+00_JPRB, 0.25178E+00_JPRB, 0.24632E+00_JPRB, 0.24067E+00_JPRB, 0.23740E+00_JPRB, 0.23587E+00_JPRB ,&
&  0.23580E+00_JPRB, 0.23635E+00_JPRB, 0.23888E+00_JPRB, 0.24069E+00_JPRB, 0.24317E+00_JPRB, 0.24730E+00_JPRB, 0.25296E+00_JPRB ,&
&  0.25832E+00_JPRB, 0.26344E+00_JPRB, 0.26869E+00_JPRB, 0.27317E+00_JPRB, 0.27764E+00_JPRB, 0.28312E+00_JPRB, 0.28898E+00_JPRB ,&
&  0.29547E+00_JPRB, 0.29976E+00_JPRB, 0.30032E+00_JPRB, 0.30120E+00_JPRB, 0.30381E+00_JPRB, 0.30695E+00_JPRB, 0.30893E+00_JPRB ,&
&  0.31035E+00_JPRB, 0.30881E+00_JPRB, 0.30621E+00_JPRB, 0.30232E+00_JPRB, 0.29802E+00_JPRB, 0.29320E+00_JPRB, 0.28672E+00_JPRB ,&
&  0.27896E+00_JPRB, 0.27055E+00_JPRB, 0.26251E+00_JPRB, 0.25487E+00_JPRB, 0.24757E+00_JPRB, 0.24081E+00_JPRB, 0.23470E+00_JPRB ,&
&  0.22885E+00_JPRB, 0.22436E+00_JPRB, 0.22000E+00_JPRB, 0.21740E+00_JPRB, 0.21587E+00_JPRB, 0.21404E+00_JPRB, 0.21371E+00_JPRB ,&
&  0.21287E+00_JPRB, 0.21244E+00_JPRB, 0.21241E+00_JPRB, 0.21404E+00_JPRB, 0.21713E+00_JPRB, 0.21898E+00_JPRB, 0.21780E+00_JPRB ,&
&  0.21452E+00_JPRB, 0.21253E+00_JPRB, 0.21143E+00_JPRB, 0.21060E+00_JPRB, 0.20833E+00_JPRB, 0.20806E+00_JPRB, 0.21000E+00_JPRB ,&
&  0.21426E+00_JPRB, 0.21887E+00_JPRB, 0.22273E+00_JPRB, 0.22509E+00_JPRB, 0.22760E+00_JPRB, 0.22969E+00_JPRB, 0.23154E+00_JPRB ,&
&  0.23357E+00_JPRB, 0.23505E+00_JPRB, 0.23616E+00_JPRB, 0.23765E+00_JPRB, 0.24006E+00_JPRB, 0.24242E+00_JPRB, 0.24473E+00_JPRB ,&
&  0.24724E+00_JPRB, 0.25114E+00_JPRB, 0.25510E+00_JPRB, 0.25801E+00_JPRB, 0.26141E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.24231E+00_JPRB, 0.24492E+00_JPRB, 0.24747E+00_JPRB, 0.24763E+00_JPRB, 0.24590E+00_JPRB, 0.24504E+00_JPRB, 0.24516E+00_JPRB ,&
&  0.24549E+00_JPRB, 0.24552E+00_JPRB, 0.24428E+00_JPRB, 0.24419E+00_JPRB, 0.24426E+00_JPRB, 0.24340E+00_JPRB, 0.24284E+00_JPRB ,&
&  0.24169E+00_JPRB, 0.23880E+00_JPRB, 0.23465E+00_JPRB, 0.22953E+00_JPRB, 0.22573E+00_JPRB, 0.22377E+00_JPRB, 0.22353E+00_JPRB ,&
&  0.22430E+00_JPRB, 0.22488E+00_JPRB, 0.22742E+00_JPRB, 0.22961E+00_JPRB, 0.23237E+00_JPRB, 0.23603E+00_JPRB, 0.23988E+00_JPRB ,&
&  0.24274E+00_JPRB, 0.24574E+00_JPRB, 0.24874E+00_JPRB, 0.25155E+00_JPRB, 0.25519E+00_JPRB, 0.26059E+00_JPRB, 0.26509E+00_JPRB ,&
&  0.26990E+00_JPRB, 0.27222E+00_JPRB, 0.26954E+00_JPRB, 0.26653E+00_JPRB, 0.26636E+00_JPRB, 0.26795E+00_JPRB, 0.26960E+00_JPRB ,&
&  0.27097E+00_JPRB, 0.27243E+00_JPRB, 0.27361E+00_JPRB, 0.27317E+00_JPRB, 0.27043E+00_JPRB, 0.26562E+00_JPRB, 0.25928E+00_JPRB ,&
&  0.25237E+00_JPRB, 0.24546E+00_JPRB, 0.23886E+00_JPRB, 0.23233E+00_JPRB, 0.22676E+00_JPRB, 0.22134E+00_JPRB, 0.21635E+00_JPRB ,&
&  0.21166E+00_JPRB, 0.20718E+00_JPRB, 0.20378E+00_JPRB, 0.20166E+00_JPRB, 0.20107E+00_JPRB, 0.19991E+00_JPRB, 0.19984E+00_JPRB ,&
&  0.19938E+00_JPRB, 0.19927E+00_JPRB, 0.19956E+00_JPRB, 0.20051E+00_JPRB, 0.20192E+00_JPRB, 0.20236E+00_JPRB, 0.20147E+00_JPRB ,&
&  0.20097E+00_JPRB, 0.20215E+00_JPRB, 0.20195E+00_JPRB, 0.19962E+00_JPRB, 0.19673E+00_JPRB, 0.19602E+00_JPRB, 0.19736E+00_JPRB ,&
&  0.19999E+00_JPRB, 0.20274E+00_JPRB, 0.20516E+00_JPRB, 0.20731E+00_JPRB, 0.20910E+00_JPRB, 0.21132E+00_JPRB, 0.21333E+00_JPRB ,&
&  0.21503E+00_JPRB, 0.21589E+00_JPRB, 0.21743E+00_JPRB, 0.21907E+00_JPRB, 0.22078E+00_JPRB, 0.22200E+00_JPRB, 0.22187E+00_JPRB ,&
&  0.22142E+00_JPRB, 0.22294E+00_JPRB, 0.22773E+00_JPRB, 0.23492E+00_JPRB, 0.23945E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.22463E+00_JPRB, 0.22691E+00_JPRB, 0.22815E+00_JPRB, 0.22800E+00_JPRB, 0.22747E+00_JPRB, 0.22814E+00_JPRB, 0.22864E+00_JPRB ,&
&  0.22920E+00_JPRB, 0.22930E+00_JPRB, 0.22806E+00_JPRB, 0.22698E+00_JPRB, 0.22859E+00_JPRB, 0.22815E+00_JPRB, 0.22769E+00_JPRB ,&
&  0.22621E+00_JPRB, 0.22440E+00_JPRB, 0.22161E+00_JPRB, 0.21840E+00_JPRB, 0.21448E+00_JPRB, 0.21212E+00_JPRB, 0.21250E+00_JPRB ,&
&  0.21408E+00_JPRB, 0.21471E+00_JPRB, 0.21555E+00_JPRB, 0.21665E+00_JPRB, 0.21891E+00_JPRB, 0.22183E+00_JPRB, 0.22403E+00_JPRB ,&
&  0.22578E+00_JPRB, 0.22890E+00_JPRB, 0.23088E+00_JPRB, 0.22982E+00_JPRB, 0.23409E+00_JPRB, 0.23808E+00_JPRB, 0.24159E+00_JPRB ,&
&  0.24464E+00_JPRB, 0.24646E+00_JPRB, 0.24648E+00_JPRB, 0.24547E+00_JPRB, 0.24465E+00_JPRB, 0.24557E+00_JPRB, 0.24601E+00_JPRB ,&
&  0.24652E+00_JPRB, 0.24879E+00_JPRB, 0.25143E+00_JPRB, 0.25228E+00_JPRB, 0.25057E+00_JPRB, 0.24628E+00_JPRB, 0.23964E+00_JPRB ,&
&  0.23114E+00_JPRB, 0.22224E+00_JPRB, 0.21501E+00_JPRB, 0.20927E+00_JPRB, 0.20486E+00_JPRB, 0.20126E+00_JPRB, 0.19832E+00_JPRB ,&
&  0.19509E+00_JPRB, 0.19279E+00_JPRB, 0.19271E+00_JPRB, 0.19330E+00_JPRB, 0.19342E+00_JPRB, 0.19192E+00_JPRB, 0.19086E+00_JPRB ,&
&  0.18990E+00_JPRB, 0.19034E+00_JPRB, 0.19125E+00_JPRB, 0.19161E+00_JPRB, 0.19174E+00_JPRB, 0.19148E+00_JPRB, 0.19126E+00_JPRB ,&
&  0.19209E+00_JPRB, 0.19226E+00_JPRB, 0.19129E+00_JPRB, 0.18951E+00_JPRB, 0.18799E+00_JPRB, 0.18774E+00_JPRB, 0.18802E+00_JPRB ,&
&  0.18867E+00_JPRB, 0.18997E+00_JPRB, 0.19182E+00_JPRB, 0.19374E+00_JPRB, 0.19524E+00_JPRB, 0.19593E+00_JPRB, 0.19617E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19627E+00_JPRB, 0.19727E+00_JPRB, 0.20032E+00_JPRB, 0.20351E+00_JPRB, 0.20607E+00_JPRB, 0.20750E+00_JPRB ,&
&  0.20713E+00_JPRB, 0.20783E+00_JPRB, 0.21096E+00_JPRB, 0.21678E+00_JPRB, 0.22116E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.21151E+00_JPRB, 0.21362E+00_JPRB, 0.21432E+00_JPRB, 0.21391E+00_JPRB, 0.21341E+00_JPRB, 0.21429E+00_JPRB, 0.21556E+00_JPRB ,&
&  0.21619E+00_JPRB, 0.21623E+00_JPRB, 0.21528E+00_JPRB, 0.21344E+00_JPRB, 0.21206E+00_JPRB, 0.21226E+00_JPRB, 0.21073E+00_JPRB ,&
&  0.20939E+00_JPRB, 0.20975E+00_JPRB, 0.20866E+00_JPRB, 0.20732E+00_JPRB, 0.20511E+00_JPRB, 0.20385E+00_JPRB, 0.20449E+00_JPRB ,&
&  0.20663E+00_JPRB, 0.20728E+00_JPRB, 0.20798E+00_JPRB, 0.20892E+00_JPRB, 0.21067E+00_JPRB, 0.21278E+00_JPRB, 0.21418E+00_JPRB ,&
&  0.21519E+00_JPRB, 0.21442E+00_JPRB, 0.21281E+00_JPRB, 0.21552E+00_JPRB, 0.21864E+00_JPRB, 0.22267E+00_JPRB, 0.22533E+00_JPRB ,&
&  0.22766E+00_JPRB, 0.22942E+00_JPRB, 0.23028E+00_JPRB, 0.23022E+00_JPRB, 0.23055E+00_JPRB, 0.23226E+00_JPRB, 0.23350E+00_JPRB ,&
&  0.23392E+00_JPRB, 0.23352E+00_JPRB, 0.23206E+00_JPRB, 0.23061E+00_JPRB, 0.22717E+00_JPRB, 0.22207E+00_JPRB, 0.21452E+00_JPRB ,&
&  0.20669E+00_JPRB, 0.20042E+00_JPRB, 0.19569E+00_JPRB, 0.19187E+00_JPRB, 0.18921E+00_JPRB, 0.18891E+00_JPRB, 0.18887E+00_JPRB ,&
&  0.18845E+00_JPRB, 0.18804E+00_JPRB, 0.18939E+00_JPRB, 0.19013E+00_JPRB, 0.18624E+00_JPRB, 0.18106E+00_JPRB, 0.17661E+00_JPRB ,&
&  0.17558E+00_JPRB, 0.17668E+00_JPRB, 0.17842E+00_JPRB, 0.18041E+00_JPRB, 0.18368E+00_JPRB, 0.18444E+00_JPRB, 0.18346E+00_JPRB ,&
&  0.17822E+00_JPRB, 0.17286E+00_JPRB, 0.16973E+00_JPRB, 0.16734E+00_JPRB, 0.16589E+00_JPRB, 0.16586E+00_JPRB, 0.16658E+00_JPRB ,&
&  0.16870E+00_JPRB, 0.17057E+00_JPRB, 0.17299E+00_JPRB, 0.17427E+00_JPRB, 0.17407E+00_JPRB, 0.17376E+00_JPRB, 0.17305E+00_JPRB ,&
&  0.17432E+00_JPRB, 0.17859E+00_JPRB, 0.18508E+00_JPRB, 0.18834E+00_JPRB, 0.19232E+00_JPRB, 0.19505E+00_JPRB, 0.19640E+00_JPRB ,&
&  0.19730E+00_JPRB, 0.19782E+00_JPRB, 0.19833E+00_JPRB, 0.20247E+00_JPRB, 0.20805E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.20046E+00_JPRB, 0.20221E+00_JPRB, 0.20279E+00_JPRB, 0.20277E+00_JPRB, 0.20255E+00_JPRB, 0.20297E+00_JPRB, 0.20344E+00_JPRB ,&
&  0.20338E+00_JPRB, 0.20264E+00_JPRB, 0.20143E+00_JPRB, 0.19938E+00_JPRB, 0.19689E+00_JPRB, 0.19541E+00_JPRB, 0.19641E+00_JPRB ,&
&  0.19762E+00_JPRB, 0.19801E+00_JPRB, 0.19851E+00_JPRB, 0.19861E+00_JPRB, 0.19828E+00_JPRB, 0.19856E+00_JPRB, 0.19977E+00_JPRB ,&
&  0.20111E+00_JPRB, 0.20195E+00_JPRB, 0.20326E+00_JPRB, 0.20435E+00_JPRB, 0.20635E+00_JPRB, 0.20815E+00_JPRB, 0.20881E+00_JPRB ,&
&  0.20625E+00_JPRB, 0.19849E+00_JPRB, 0.19314E+00_JPRB, 0.19561E+00_JPRB, 0.20370E+00_JPRB, 0.20871E+00_JPRB, 0.21180E+00_JPRB ,&
&  0.21390E+00_JPRB, 0.21498E+00_JPRB, 0.21523E+00_JPRB, 0.21651E+00_JPRB, 0.21845E+00_JPRB, 0.22031E+00_JPRB, 0.21993E+00_JPRB ,&
&  0.21797E+00_JPRB, 0.21625E+00_JPRB, 0.21375E+00_JPRB, 0.20943E+00_JPRB, 0.20359E+00_JPRB, 0.19786E+00_JPRB, 0.19297E+00_JPRB ,&
&  0.18863E+00_JPRB, 0.18443E+00_JPRB, 0.18090E+00_JPRB, 0.17862E+00_JPRB, 0.17794E+00_JPRB, 0.17921E+00_JPRB, 0.17898E+00_JPRB ,&
&  0.17893E+00_JPRB, 0.17716E+00_JPRB, 0.17498E+00_JPRB, 0.17242E+00_JPRB, 0.17022E+00_JPRB, 0.16626E+00_JPRB, 0.16138E+00_JPRB ,&
&  0.15887E+00_JPRB, 0.16120E+00_JPRB, 0.16773E+00_JPRB, 0.17306E+00_JPRB, 0.17327E+00_JPRB, 0.16619E+00_JPRB, 0.16224E+00_JPRB ,&
&  0.15821E+00_JPRB, 0.15423E+00_JPRB, 0.15168E+00_JPRB, 0.15109E+00_JPRB, 0.15127E+00_JPRB, 0.15153E+00_JPRB, 0.15253E+00_JPRB ,&
&  0.15410E+00_JPRB, 0.15547E+00_JPRB, 0.15625E+00_JPRB, 0.15712E+00_JPRB, 0.15749E+00_JPRB, 0.15758E+00_JPRB, 0.15771E+00_JPRB ,&
&  0.15926E+00_JPRB, 0.16250E+00_JPRB, 0.16686E+00_JPRB, 0.17174E+00_JPRB, 0.17552E+00_JPRB, 0.17898E+00_JPRB, 0.18214E+00_JPRB ,&
&  0.18539E+00_JPRB, 0.18740E+00_JPRB, 0.19034E+00_JPRB, 0.19428E+00_JPRB, 0.19761E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.18788E+00_JPRB, 0.19157E+00_JPRB, 0.19310E+00_JPRB, 0.19312E+00_JPRB, 0.19206E+00_JPRB, 0.19114E+00_JPRB, 0.19092E+00_JPRB ,&
&  0.19076E+00_JPRB, 0.18966E+00_JPRB, 0.18883E+00_JPRB, 0.18818E+00_JPRB, 0.18748E+00_JPRB, 0.18670E+00_JPRB, 0.18695E+00_JPRB ,&
&  0.18818E+00_JPRB, 0.18988E+00_JPRB, 0.19119E+00_JPRB, 0.19181E+00_JPRB, 0.19145E+00_JPRB, 0.19183E+00_JPRB, 0.19271E+00_JPRB ,&
&  0.19269E+00_JPRB, 0.19290E+00_JPRB, 0.19608E+00_JPRB, 0.19643E+00_JPRB, 0.19732E+00_JPRB, 0.19737E+00_JPRB, 0.19739E+00_JPRB ,&
&  0.19268E+00_JPRB, 0.18745E+00_JPRB, 0.18453E+00_JPRB, 0.18663E+00_JPRB, 0.19468E+00_JPRB, 0.19618E+00_JPRB, 0.19952E+00_JPRB ,&
&  0.20163E+00_JPRB, 0.20269E+00_JPRB, 0.20404E+00_JPRB, 0.20526E+00_JPRB, 0.20670E+00_JPRB, 0.20589E+00_JPRB, 0.20373E+00_JPRB ,&
&  0.20099E+00_JPRB, 0.19999E+00_JPRB, 0.19674E+00_JPRB, 0.18283E+00_JPRB, 0.17428E+00_JPRB, 0.16944E+00_JPRB, 0.16437E+00_JPRB ,&
&  0.15976E+00_JPRB, 0.15720E+00_JPRB, 0.15654E+00_JPRB, 0.15526E+00_JPRB, 0.15612E+00_JPRB, 0.15812E+00_JPRB, 0.15900E+00_JPRB ,&
&  0.15874E+00_JPRB, 0.15917E+00_JPRB, 0.15889E+00_JPRB, 0.15796E+00_JPRB, 0.15630E+00_JPRB, 0.15526E+00_JPRB, 0.15469E+00_JPRB ,&
&  0.15520E+00_JPRB, 0.15497E+00_JPRB, 0.15886E+00_JPRB, 0.16017E+00_JPRB, 0.15544E+00_JPRB, 0.15397E+00_JPRB, 0.15209E+00_JPRB ,&
&  0.14711E+00_JPRB, 0.14430E+00_JPRB, 0.14275E+00_JPRB, 0.14215E+00_JPRB, 0.14311E+00_JPRB, 0.14450E+00_JPRB, 0.14523E+00_JPRB ,&
&  0.14551E+00_JPRB, 0.14654E+00_JPRB, 0.14702E+00_JPRB, 0.14756E+00_JPRB, 0.14794E+00_JPRB, 0.14874E+00_JPRB, 0.14994E+00_JPRB ,&
&  0.15144E+00_JPRB, 0.15415E+00_JPRB, 0.15682E+00_JPRB, 0.15982E+00_JPRB, 0.16161E+00_JPRB, 0.16301E+00_JPRB, 0.16623E+00_JPRB ,&
&  0.16821E+00_JPRB, 0.17510E+00_JPRB, 0.18516E+00_JPRB, 0.18838E+00_JPRB, 0.18352E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.18091E+00_JPRB, 0.18278E+00_JPRB, 0.18442E+00_JPRB, 0.18351E+00_JPRB, 0.18368E+00_JPRB, 0.18365E+00_JPRB, 0.18372E+00_JPRB ,&
&  0.18365E+00_JPRB, 0.18307E+00_JPRB, 0.18332E+00_JPRB, 0.18161E+00_JPRB, 0.18063E+00_JPRB, 0.17719E+00_JPRB, 0.17604E+00_JPRB ,&
&  0.17420E+00_JPRB, 0.17384E+00_JPRB, 0.17411E+00_JPRB, 0.17532E+00_JPRB, 0.17575E+00_JPRB, 0.17705E+00_JPRB, 0.17634E+00_JPRB ,&
&  0.17657E+00_JPRB, 0.17656E+00_JPRB, 0.17529E+00_JPRB, 0.17491E+00_JPRB, 0.17531E+00_JPRB, 0.17574E+00_JPRB, 0.17604E+00_JPRB ,&
&  0.17790E+00_JPRB, 0.18011E+00_JPRB, 0.18224E+00_JPRB, 0.18355E+00_JPRB, 0.18850E+00_JPRB, 0.18992E+00_JPRB, 0.19188E+00_JPRB ,&
&  0.19300E+00_JPRB, 0.19441E+00_JPRB, 0.19542E+00_JPRB, 0.19481E+00_JPRB, 0.19361E+00_JPRB, 0.19003E+00_JPRB, 0.18869E+00_JPRB ,&
&  0.17868E+00_JPRB, 0.17187E+00_JPRB, 0.16512E+00_JPRB, 0.15912E+00_JPRB, 0.15473E+00_JPRB, 0.15170E+00_JPRB, 0.14923E+00_JPRB ,&
&  0.14721E+00_JPRB, 0.14526E+00_JPRB, 0.14313E+00_JPRB, 0.14318E+00_JPRB, 0.14394E+00_JPRB, 0.14512E+00_JPRB, 0.14630E+00_JPRB ,&
&  0.14766E+00_JPRB, 0.14845E+00_JPRB, 0.14882E+00_JPRB, 0.14959E+00_JPRB, 0.14980E+00_JPRB, 0.15018E+00_JPRB, 0.15044E+00_JPRB ,&
&  0.15174E+00_JPRB, 0.15219E+00_JPRB, 0.15225E+00_JPRB, 0.15362E+00_JPRB, 0.15193E+00_JPRB, 0.14927E+00_JPRB, 0.14638E+00_JPRB ,&
&  0.14351E+00_JPRB, 0.14171E+00_JPRB, 0.14021E+00_JPRB, 0.13963E+00_JPRB, 0.14001E+00_JPRB, 0.14073E+00_JPRB, 0.14169E+00_JPRB ,&
&  0.14192E+00_JPRB, 0.14253E+00_JPRB, 0.14299E+00_JPRB, 0.14383E+00_JPRB, 0.14474E+00_JPRB, 0.14576E+00_JPRB, 0.14688E+00_JPRB ,&
&  0.14858E+00_JPRB, 0.15055E+00_JPRB, 0.15252E+00_JPRB, 0.15454E+00_JPRB, 0.15775E+00_JPRB, 0.16165E+00_JPRB, 0.16782E+00_JPRB ,&
&  0.17336E+00_JPRB, 0.18075E+00_JPRB, 0.17944E+00_JPRB, 0.18191E+00_JPRB, 0.18213E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.17642E+00_JPRB, 0.17721E+00_JPRB, 0.17540E+00_JPRB, 0.17580E+00_JPRB, 0.17657E+00_JPRB, 0.17699E+00_JPRB, 0.17745E+00_JPRB ,&
&  0.17504E+00_JPRB, 0.17418E+00_JPRB, 0.17190E+00_JPRB, 0.16985E+00_JPRB, 0.16738E+00_JPRB, 0.16481E+00_JPRB, 0.16283E+00_JPRB ,&
&  0.16099E+00_JPRB, 0.16028E+00_JPRB, 0.15941E+00_JPRB, 0.16016E+00_JPRB, 0.16018E+00_JPRB, 0.16179E+00_JPRB, 0.16311E+00_JPRB ,&
&  0.16385E+00_JPRB, 0.16484E+00_JPRB, 0.16357E+00_JPRB, 0.16337E+00_JPRB, 0.16508E+00_JPRB, 0.16667E+00_JPRB, 0.16927E+00_JPRB ,&
&  0.17203E+00_JPRB, 0.17508E+00_JPRB, 0.18003E+00_JPRB, 0.18183E+00_JPRB, 0.18402E+00_JPRB, 0.18540E+00_JPRB, 0.18540E+00_JPRB ,&
&  0.18514E+00_JPRB, 0.18514E+00_JPRB, 0.18600E+00_JPRB, 0.18573E+00_JPRB, 0.17896E+00_JPRB, 0.17375E+00_JPRB, 0.16827E+00_JPRB ,&
&  0.16317E+00_JPRB, 0.15831E+00_JPRB, 0.15399E+00_JPRB, 0.15109E+00_JPRB, 0.14808E+00_JPRB, 0.14590E+00_JPRB, 0.14421E+00_JPRB ,&
&  0.14249E+00_JPRB, 0.14148E+00_JPRB, 0.14019E+00_JPRB, 0.13981E+00_JPRB, 0.14009E+00_JPRB, 0.14086E+00_JPRB, 0.14191E+00_JPRB ,&
&  0.14315E+00_JPRB, 0.14402E+00_JPRB, 0.14512E+00_JPRB, 0.14617E+00_JPRB, 0.14693E+00_JPRB, 0.14743E+00_JPRB, 0.14722E+00_JPRB ,&
&  0.14746E+00_JPRB, 0.14773E+00_JPRB, 0.14731E+00_JPRB, 0.14725E+00_JPRB, 0.14615E+00_JPRB, 0.14487E+00_JPRB, 0.14355E+00_JPRB ,&
&  0.14179E+00_JPRB, 0.14043E+00_JPRB, 0.13993E+00_JPRB, 0.13965E+00_JPRB, 0.14033E+00_JPRB, 0.14135E+00_JPRB, 0.14204E+00_JPRB ,&
&  0.14240E+00_JPRB, 0.14236E+00_JPRB, 0.14241E+00_JPRB, 0.14352E+00_JPRB, 0.14467E+00_JPRB, 0.14613E+00_JPRB, 0.14764E+00_JPRB ,&
&  0.14814E+00_JPRB, 0.14940E+00_JPRB, 0.15050E+00_JPRB, 0.15186E+00_JPRB, 0.15644E+00_JPRB, 0.16009E+00_JPRB, 0.16706E+00_JPRB ,&
&  0.17058E+00_JPRB, 0.17212E+00_JPRB, 0.17178E+00_JPRB, 0.17583E+00_JPRB, 0.17605E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.17458E+00_JPRB, 0.17702E+00_JPRB, 0.17576E+00_JPRB, 0.17564E+00_JPRB, 0.17510E+00_JPRB, 0.17499E+00_JPRB, 0.17398E+00_JPRB ,&
&  0.17378E+00_JPRB, 0.17178E+00_JPRB, 0.17135E+00_JPRB, 0.16966E+00_JPRB, 0.16789E+00_JPRB, 0.16549E+00_JPRB, 0.16311E+00_JPRB ,&
&  0.16074E+00_JPRB, 0.15863E+00_JPRB, 0.15719E+00_JPRB, 0.15650E+00_JPRB, 0.15531E+00_JPRB, 0.15523E+00_JPRB, 0.15700E+00_JPRB ,&
&  0.15739E+00_JPRB, 0.15844E+00_JPRB, 0.15859E+00_JPRB, 0.15984E+00_JPRB, 0.16146E+00_JPRB, 0.16224E+00_JPRB, 0.16456E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16743E+00_JPRB, 0.17301E+00_JPRB, 0.17613E+00_JPRB, 0.17790E+00_JPRB, 0.18053E+00_JPRB, 0.18056E+00_JPRB ,&
&  0.17923E+00_JPRB, 0.17541E+00_JPRB, 0.17858E+00_JPRB, 0.17636E+00_JPRB, 0.16999E+00_JPRB, 0.16708E+00_JPRB, 0.16185E+00_JPRB ,&
&  0.15872E+00_JPRB, 0.15506E+00_JPRB, 0.15230E+00_JPRB, 0.15018E+00_JPRB, 0.14732E+00_JPRB, 0.14547E+00_JPRB, 0.14340E+00_JPRB ,&
&  0.14146E+00_JPRB, 0.14061E+00_JPRB, 0.13927E+00_JPRB, 0.13908E+00_JPRB, 0.13919E+00_JPRB, 0.13957E+00_JPRB, 0.14022E+00_JPRB ,&
&  0.14109E+00_JPRB, 0.14191E+00_JPRB, 0.14268E+00_JPRB, 0.14335E+00_JPRB, 0.14391E+00_JPRB, 0.14425E+00_JPRB, 0.14415E+00_JPRB ,&
&  0.14425E+00_JPRB, 0.14417E+00_JPRB, 0.14373E+00_JPRB, 0.14349E+00_JPRB, 0.14260E+00_JPRB, 0.14219E+00_JPRB, 0.14177E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14160E+00_JPRB, 0.14153E+00_JPRB, 0.14173E+00_JPRB, 0.14202E+00_JPRB, 0.14270E+00_JPRB, 0.14326E+00_JPRB ,&
&  0.14371E+00_JPRB, 0.14457E+00_JPRB, 0.14509E+00_JPRB, 0.14581E+00_JPRB, 0.14681E+00_JPRB, 0.14727E+00_JPRB, 0.14841E+00_JPRB ,&
&  0.14940E+00_JPRB, 0.15056E+00_JPRB, 0.15270E+00_JPRB, 0.15486E+00_JPRB, 0.15795E+00_JPRB, 0.16187E+00_JPRB, 0.16354E+00_JPRB ,&
&  0.16506E+00_JPRB, 0.16643E+00_JPRB, 0.16526E+00_JPRB, 0.17003E+00_JPRB, 0.17224E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.16841E+00_JPRB, 0.17023E+00_JPRB, 0.17139E+00_JPRB, 0.17220E+00_JPRB, 0.17338E+00_JPRB, 0.17389E+00_JPRB, 0.17437E+00_JPRB ,&
&  0.17253E+00_JPRB, 0.17191E+00_JPRB, 0.17085E+00_JPRB, 0.16955E+00_JPRB, 0.16861E+00_JPRB, 0.16637E+00_JPRB, 0.16412E+00_JPRB ,&
&  0.16163E+00_JPRB, 0.15939E+00_JPRB, 0.15800E+00_JPRB, 0.15665E+00_JPRB, 0.15528E+00_JPRB, 0.15415E+00_JPRB, 0.15582E+00_JPRB ,&
&  0.15679E+00_JPRB, 0.15743E+00_JPRB, 0.15866E+00_JPRB, 0.15862E+00_JPRB, 0.15993E+00_JPRB, 0.16198E+00_JPRB, 0.16470E+00_JPRB ,&
&  0.16653E+00_JPRB, 0.16935E+00_JPRB, 0.17102E+00_JPRB, 0.17186E+00_JPRB, 0.17300E+00_JPRB, 0.17378E+00_JPRB, 0.17348E+00_JPRB ,&
&  0.17221E+00_JPRB, 0.17053E+00_JPRB, 0.16856E+00_JPRB, 0.16605E+00_JPRB, 0.16324E+00_JPRB, 0.16052E+00_JPRB, 0.15780E+00_JPRB ,&
&  0.15579E+00_JPRB, 0.15397E+00_JPRB, 0.15206E+00_JPRB, 0.15032E+00_JPRB, 0.14868E+00_JPRB, 0.14719E+00_JPRB, 0.14569E+00_JPRB ,&
&  0.14450E+00_JPRB, 0.14357E+00_JPRB, 0.14242E+00_JPRB, 0.14179E+00_JPRB, 0.14116E+00_JPRB, 0.14087E+00_JPRB, 0.14073E+00_JPRB ,&
&  0.14109E+00_JPRB, 0.14157E+00_JPRB, 0.14206E+00_JPRB, 0.14233E+00_JPRB, 0.14259E+00_JPRB, 0.14271E+00_JPRB, 0.14294E+00_JPRB ,&
&  0.14316E+00_JPRB, 0.14342E+00_JPRB, 0.14306E+00_JPRB, 0.14272E+00_JPRB, 0.14226E+00_JPRB, 0.14175E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14187E+00_JPRB, 0.14210E+00_JPRB, 0.14197E+00_JPRB, 0.14225E+00_JPRB, 0.14273E+00_JPRB, 0.14323E+00_JPRB, 0.14447E+00_JPRB ,&
&  0.14515E+00_JPRB, 0.14581E+00_JPRB, 0.14642E+00_JPRB, 0.14691E+00_JPRB, 0.14764E+00_JPRB, 0.14824E+00_JPRB, 0.14898E+00_JPRB ,&
&  0.14987E+00_JPRB, 0.15075E+00_JPRB, 0.15197E+00_JPRB, 0.15350E+00_JPRB, 0.15713E+00_JPRB, 0.15958E+00_JPRB, 0.16174E+00_JPRB ,&
&  0.16284E+00_JPRB, 0.15982E+00_JPRB, 0.16700E+00_JPRB, 0.16627E+00_JPRB, 0.16610E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.15947E+00_JPRB, 0.16127E+00_JPRB, 0.16271E+00_JPRB, 0.16462E+00_JPRB, 0.16079E+00_JPRB, 0.16776E+00_JPRB, 0.16514E+00_JPRB ,&
&  0.16241E+00_JPRB, 0.15764E+00_JPRB, 0.15769E+00_JPRB, 0.15684E+00_JPRB, 0.15568E+00_JPRB, 0.15504E+00_JPRB, 0.15484E+00_JPRB ,&
&  0.15483E+00_JPRB, 0.15459E+00_JPRB, 0.15497E+00_JPRB, 0.15524E+00_JPRB, 0.15545E+00_JPRB, 0.15561E+00_JPRB, 0.15587E+00_JPRB ,&
&  0.15635E+00_JPRB, 0.15721E+00_JPRB, 0.15811E+00_JPRB, 0.15645E+00_JPRB, 0.15630E+00_JPRB, 0.15660E+00_JPRB, 0.15722E+00_JPRB ,&
&  0.15771E+00_JPRB, 0.15879E+00_JPRB, 0.15959E+00_JPRB, 0.16014E+00_JPRB, 0.16166E+00_JPRB, 0.16169E+00_JPRB, 0.16152E+00_JPRB ,&
&  0.16113E+00_JPRB, 0.16034E+00_JPRB, 0.15921E+00_JPRB, 0.15817E+00_JPRB, 0.15709E+00_JPRB, 0.15568E+00_JPRB, 0.15432E+00_JPRB ,&
&  0.15320E+00_JPRB, 0.15229E+00_JPRB, 0.15112E+00_JPRB, 0.14999E+00_JPRB, 0.14895E+00_JPRB, 0.14795E+00_JPRB, 0.14709E+00_JPRB ,&
&  0.14647E+00_JPRB, 0.14594E+00_JPRB, 0.14540E+00_JPRB, 0.14498E+00_JPRB, 0.14454E+00_JPRB, 0.14413E+00_JPRB, 0.14376E+00_JPRB ,&
&  0.14391E+00_JPRB, 0.14404E+00_JPRB, 0.14417E+00_JPRB, 0.14429E+00_JPRB, 0.14467E+00_JPRB, 0.14475E+00_JPRB, 0.14485E+00_JPRB ,&
&  0.14490E+00_JPRB, 0.14482E+00_JPRB, 0.14472E+00_JPRB, 0.14453E+00_JPRB, 0.14431E+00_JPRB, 0.14402E+00_JPRB, 0.14389E+00_JPRB ,&
&  0.14389E+00_JPRB, 0.14399E+00_JPRB, 0.14430E+00_JPRB, 0.14452E+00_JPRB, 0.14473E+00_JPRB, 0.14491E+00_JPRB, 0.14534E+00_JPRB ,&
&  0.14578E+00_JPRB, 0.14623E+00_JPRB, 0.14665E+00_JPRB, 0.14658E+00_JPRB, 0.14681E+00_JPRB, 0.14703E+00_JPRB, 0.14724E+00_JPRB ,&
&  0.14738E+00_JPRB, 0.14783E+00_JPRB, 0.14854E+00_JPRB, 0.14950E+00_JPRB, 0.15123E+00_JPRB, 0.15272E+00_JPRB, 0.15411E+00_JPRB ,&
&  0.15559E+00_JPRB, 0.15854E+00_JPRB, 0.15974E+00_JPRB, 0.16016E+00_JPRB, 0.16021E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.15039E+00_JPRB, 0.15114E+00_JPRB, 0.15167E+00_JPRB, 0.15210E+00_JPRB, 0.15193E+00_JPRB, 0.15199E+00_JPRB, 0.15205E+00_JPRB ,&
&  0.15204E+00_JPRB, 0.15201E+00_JPRB, 0.15226E+00_JPRB, 0.15249E+00_JPRB, 0.15275E+00_JPRB, 0.15302E+00_JPRB, 0.15333E+00_JPRB ,&
&  0.15346E+00_JPRB, 0.15373E+00_JPRB, 0.15264E+00_JPRB, 0.15174E+00_JPRB, 0.15118E+00_JPRB, 0.15083E+00_JPRB, 0.15065E+00_JPRB ,&
&  0.15049E+00_JPRB, 0.15038E+00_JPRB, 0.15025E+00_JPRB, 0.15103E+00_JPRB, 0.15153E+00_JPRB, 0.15199E+00_JPRB, 0.15234E+00_JPRB ,&
&  0.15248E+00_JPRB, 0.15268E+00_JPRB, 0.15286E+00_JPRB, 0.15304E+00_JPRB, 0.15325E+00_JPRB, 0.15322E+00_JPRB, 0.15309E+00_JPRB ,&
&  0.15290E+00_JPRB, 0.15259E+00_JPRB, 0.15223E+00_JPRB, 0.15190E+00_JPRB, 0.15162E+00_JPRB, 0.15130E+00_JPRB, 0.15096E+00_JPRB ,&
&  0.15062E+00_JPRB, 0.15023E+00_JPRB, 0.14986E+00_JPRB, 0.14950E+00_JPRB, 0.14916E+00_JPRB, 0.14888E+00_JPRB, 0.14856E+00_JPRB ,&
&  0.14835E+00_JPRB, 0.14815E+00_JPRB, 0.14795E+00_JPRB, 0.14781E+00_JPRB, 0.14760E+00_JPRB, 0.14752E+00_JPRB, 0.14733E+00_JPRB ,&
&  0.14727E+00_JPRB, 0.14721E+00_JPRB, 0.14716E+00_JPRB, 0.14709E+00_JPRB, 0.14710E+00_JPRB, 0.14708E+00_JPRB, 0.14707E+00_JPRB ,&
&  0.14705E+00_JPRB, 0.14717E+00_JPRB, 0.14706E+00_JPRB, 0.14691E+00_JPRB, 0.14675E+00_JPRB, 0.14658E+00_JPRB, 0.14643E+00_JPRB ,&
&  0.14629E+00_JPRB, 0.14615E+00_JPRB, 0.14606E+00_JPRB, 0.14607E+00_JPRB, 0.14610E+00_JPRB, 0.14615E+00_JPRB, 0.14622E+00_JPRB ,&
&  0.14630E+00_JPRB, 0.14640E+00_JPRB, 0.14651E+00_JPRB, 0.14649E+00_JPRB, 0.14651E+00_JPRB, 0.14654E+00_JPRB, 0.14656E+00_JPRB ,&
&  0.14658E+00_JPRB, 0.14663E+00_JPRB, 0.14672E+00_JPRB, 0.14683E+00_JPRB, 0.14707E+00_JPRB, 0.14717E+00_JPRB, 0.14733E+00_JPRB ,&
&  0.14754E+00_JPRB, 0.14791E+00_JPRB, 0.14824E+00_JPRB, 0.14860E+00_JPRB, 0.14902E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB, 0.14804E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB, 0.42865E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.42347E+00_JPRB, 0.42277E+00_JPRB, 0.42211E+00_JPRB, 0.42148E+00_JPRB, 0.42069E+00_JPRB, 0.42015E+00_JPRB, 0.41970E+00_JPRB ,&
&  0.41929E+00_JPRB, 0.41889E+00_JPRB, 0.41847E+00_JPRB, 0.41817E+00_JPRB, 0.41800E+00_JPRB, 0.41775E+00_JPRB, 0.41746E+00_JPRB ,&
&  0.41718E+00_JPRB, 0.41700E+00_JPRB, 0.41695E+00_JPRB, 0.41667E+00_JPRB, 0.41644E+00_JPRB, 0.41622E+00_JPRB, 0.41580E+00_JPRB ,&
&  0.41518E+00_JPRB, 0.41464E+00_JPRB, 0.41406E+00_JPRB, 0.41363E+00_JPRB, 0.41335E+00_JPRB, 0.41281E+00_JPRB, 0.41212E+00_JPRB ,&
&  0.41157E+00_JPRB, 0.41127E+00_JPRB, 0.41122E+00_JPRB, 0.41140E+00_JPRB, 0.41174E+00_JPRB, 0.41202E+00_JPRB, 0.41225E+00_JPRB ,&
&  0.41252E+00_JPRB, 0.41292E+00_JPRB, 0.41345E+00_JPRB, 0.41425E+00_JPRB, 0.41524E+00_JPRB, 0.41641E+00_JPRB, 0.41800E+00_JPRB ,&
&  0.41985E+00_JPRB, 0.42175E+00_JPRB, 0.42363E+00_JPRB, 0.42529E+00_JPRB, 0.42677E+00_JPRB, 0.42800E+00_JPRB, 0.42877E+00_JPRB ,&
&  0.42965E+00_JPRB, 0.43065E+00_JPRB, 0.43172E+00_JPRB, 0.43241E+00_JPRB, 0.43300E+00_JPRB, 0.43357E+00_JPRB, 0.43424E+00_JPRB ,&
&  0.43450E+00_JPRB, 0.43473E+00_JPRB, 0.43502E+00_JPRB, 0.43545E+00_JPRB, 0.43563E+00_JPRB, 0.43569E+00_JPRB, 0.43579E+00_JPRB ,&
&  0.43606E+00_JPRB, 0.43623E+00_JPRB, 0.43619E+00_JPRB, 0.43620E+00_JPRB, 0.43638E+00_JPRB, 0.43671E+00_JPRB, 0.43700E+00_JPRB ,&
&  0.43726E+00_JPRB, 0.43746E+00_JPRB, 0.43751E+00_JPRB, 0.43741E+00_JPRB, 0.43721E+00_JPRB, 0.43684E+00_JPRB, 0.43614E+00_JPRB ,&
&  0.43534E+00_JPRB, 0.43447E+00_JPRB, 0.43371E+00_JPRB, 0.43315E+00_JPRB, 0.43255E+00_JPRB, 0.43200E+00_JPRB, 0.43156E+00_JPRB ,&
&  0.43130E+00_JPRB, 0.43110E+00_JPRB, 0.43081E+00_JPRB, 0.43054E+00_JPRB, 0.43020E+00_JPRB, 0.42980E+00_JPRB, 0.42926E+00_JPRB ,&
&  0.42856E+00_JPRB, 0.42766E+00_JPRB, 0.42672E+00_JPRB, 0.42565E+00_JPRB, 0.42453E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.41324E+00_JPRB, 0.41190E+00_JPRB, 0.41093E+00_JPRB, 0.41033E+00_JPRB, 0.40970E+00_JPRB, 0.40915E+00_JPRB, 0.40869E+00_JPRB ,&
&  0.40819E+00_JPRB, 0.40817E+00_JPRB, 0.40786E+00_JPRB, 0.40764E+00_JPRB, 0.40734E+00_JPRB, 0.40742E+00_JPRB, 0.40722E+00_JPRB ,&
&  0.40694E+00_JPRB, 0.40620E+00_JPRB, 0.40589E+00_JPRB, 0.40512E+00_JPRB, 0.40529E+00_JPRB, 0.40562E+00_JPRB, 0.40706E+00_JPRB ,&
&  0.40737E+00_JPRB, 0.40750E+00_JPRB, 0.40698E+00_JPRB, 0.40451E+00_JPRB, 0.40240E+00_JPRB, 0.40037E+00_JPRB, 0.39876E+00_JPRB ,&
&  0.39737E+00_JPRB, 0.39652E+00_JPRB, 0.39660E+00_JPRB, 0.39714E+00_JPRB, 0.39936E+00_JPRB, 0.39999E+00_JPRB, 0.40015E+00_JPRB ,&
&  0.39962E+00_JPRB, 0.39755E+00_JPRB, 0.39799E+00_JPRB, 0.39868E+00_JPRB, 0.39989E+00_JPRB, 0.40275E+00_JPRB, 0.40533E+00_JPRB ,&
&  0.40862E+00_JPRB, 0.41249E+00_JPRB, 0.41832E+00_JPRB, 0.42258E+00_JPRB, 0.42703E+00_JPRB, 0.43086E+00_JPRB, 0.43282E+00_JPRB ,&
&  0.43457E+00_JPRB, 0.43614E+00_JPRB, 0.43772E+00_JPRB, 0.43894E+00_JPRB, 0.43999E+00_JPRB, 0.44107E+00_JPRB, 0.44239E+00_JPRB ,&
&  0.44347E+00_JPRB, 0.44472E+00_JPRB, 0.44590E+00_JPRB, 0.44702E+00_JPRB, 0.44771E+00_JPRB, 0.44812E+00_JPRB, 0.44879E+00_JPRB ,&
&  0.44951E+00_JPRB, 0.44939E+00_JPRB, 0.44912E+00_JPRB, 0.44957E+00_JPRB, 0.45009E+00_JPRB, 0.45061E+00_JPRB, 0.45016E+00_JPRB ,&
&  0.44990E+00_JPRB, 0.44935E+00_JPRB, 0.44894E+00_JPRB, 0.44809E+00_JPRB, 0.44712E+00_JPRB, 0.44614E+00_JPRB, 0.44494E+00_JPRB ,&
&  0.44324E+00_JPRB, 0.44147E+00_JPRB, 0.44006E+00_JPRB, 0.43910E+00_JPRB, 0.43782E+00_JPRB, 0.43708E+00_JPRB, 0.43652E+00_JPRB ,&
&  0.43598E+00_JPRB, 0.43428E+00_JPRB, 0.43247E+00_JPRB, 0.43047E+00_JPRB, 0.42852E+00_JPRB, 0.42659E+00_JPRB, 0.42482E+00_JPRB ,&
&  0.42304E+00_JPRB, 0.42108E+00_JPRB, 0.41909E+00_JPRB, 0.41705E+00_JPRB, 0.41510E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.41471E+00_JPRB, 0.41184E+00_JPRB, 0.40985E+00_JPRB, 0.40847E+00_JPRB, 0.40707E+00_JPRB, 0.40575E+00_JPRB, 0.40527E+00_JPRB ,&
&  0.40499E+00_JPRB, 0.40412E+00_JPRB, 0.40356E+00_JPRB, 0.40313E+00_JPRB, 0.40238E+00_JPRB, 0.40264E+00_JPRB, 0.40236E+00_JPRB ,&
&  0.40249E+00_JPRB, 0.40184E+00_JPRB, 0.40256E+00_JPRB, 0.40169E+00_JPRB, 0.40125E+00_JPRB, 0.40095E+00_JPRB, 0.40013E+00_JPRB ,&
&  0.40109E+00_JPRB, 0.40126E+00_JPRB, 0.39886E+00_JPRB, 0.39555E+00_JPRB, 0.39339E+00_JPRB, 0.38181E+00_JPRB, 0.38263E+00_JPRB ,&
&  0.38711E+00_JPRB, 0.38183E+00_JPRB, 0.38020E+00_JPRB, 0.37827E+00_JPRB, 0.37719E+00_JPRB, 0.37750E+00_JPRB, 0.38044E+00_JPRB ,&
&  0.37560E+00_JPRB, 0.38343E+00_JPRB, 0.38412E+00_JPRB, 0.38448E+00_JPRB, 0.38540E+00_JPRB, 0.38451E+00_JPRB, 0.38837E+00_JPRB ,&
&  0.40375E+00_JPRB, 0.40697E+00_JPRB, 0.41352E+00_JPRB, 0.41944E+00_JPRB, 0.42670E+00_JPRB, 0.43324E+00_JPRB, 0.43976E+00_JPRB ,&
&  0.44460E+00_JPRB, 0.44774E+00_JPRB, 0.44973E+00_JPRB, 0.45008E+00_JPRB, 0.45055E+00_JPRB, 0.45048E+00_JPRB, 0.45108E+00_JPRB ,&
&  0.45089E+00_JPRB, 0.45179E+00_JPRB, 0.45347E+00_JPRB, 0.45575E+00_JPRB, 0.45878E+00_JPRB, 0.46088E+00_JPRB, 0.46276E+00_JPRB ,&
&  0.46397E+00_JPRB, 0.46460E+00_JPRB, 0.46415E+00_JPRB, 0.46419E+00_JPRB, 0.46338E+00_JPRB, 0.46378E+00_JPRB, 0.46319E+00_JPRB ,&
&  0.46336E+00_JPRB, 0.46257E+00_JPRB, 0.46243E+00_JPRB, 0.46142E+00_JPRB, 0.46066E+00_JPRB, 0.46026E+00_JPRB, 0.45967E+00_JPRB ,&
&  0.45920E+00_JPRB, 0.45838E+00_JPRB, 0.45744E+00_JPRB, 0.45658E+00_JPRB, 0.45522E+00_JPRB, 0.45441E+00_JPRB, 0.45349E+00_JPRB ,&
&  0.45289E+00_JPRB, 0.44998E+00_JPRB, 0.44684E+00_JPRB, 0.44352E+00_JPRB, 0.44050E+00_JPRB, 0.43780E+00_JPRB, 0.43504E+00_JPRB ,&
&  0.43253E+00_JPRB, 0.42922E+00_JPRB, 0.42571E+00_JPRB, 0.42182E+00_JPRB, 0.41804E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.42490E+00_JPRB, 0.42089E+00_JPRB, 0.41737E+00_JPRB, 0.41484E+00_JPRB, 0.41265E+00_JPRB, 0.41073E+00_JPRB, 0.40934E+00_JPRB ,&
&  0.40812E+00_JPRB, 0.40736E+00_JPRB, 0.40674E+00_JPRB, 0.40635E+00_JPRB, 0.40589E+00_JPRB, 0.40526E+00_JPRB, 0.40437E+00_JPRB ,&
&  0.40494E+00_JPRB, 0.40410E+00_JPRB, 0.40464E+00_JPRB, 0.40300E+00_JPRB, 0.40185E+00_JPRB, 0.39978E+00_JPRB, 0.39579E+00_JPRB ,&
&  0.39474E+00_JPRB, 0.39288E+00_JPRB, 0.38173E+00_JPRB, 0.38645E+00_JPRB, 0.38322E+00_JPRB, 0.38203E+00_JPRB, 0.38059E+00_JPRB ,&
&  0.37555E+00_JPRB, 0.37291E+00_JPRB, 0.36907E+00_JPRB, 0.36845E+00_JPRB, 0.36965E+00_JPRB, 0.36716E+00_JPRB, 0.36793E+00_JPRB ,&
&  0.36464E+00_JPRB, 0.36350E+00_JPRB, 0.36380E+00_JPRB, 0.36396E+00_JPRB, 0.36655E+00_JPRB, 0.36727E+00_JPRB, 0.37028E+00_JPRB ,&
&  0.37778E+00_JPRB, 0.38840E+00_JPRB, 0.40619E+00_JPRB, 0.41564E+00_JPRB, 0.42594E+00_JPRB, 0.43389E+00_JPRB, 0.44165E+00_JPRB ,&
&  0.44648E+00_JPRB, 0.44983E+00_JPRB, 0.45219E+00_JPRB, 0.45356E+00_JPRB, 0.45338E+00_JPRB, 0.45084E+00_JPRB, 0.45006E+00_JPRB ,&
&  0.44963E+00_JPRB, 0.45003E+00_JPRB, 0.45095E+00_JPRB, 0.45201E+00_JPRB, 0.45469E+00_JPRB, 0.45738E+00_JPRB, 0.46104E+00_JPRB ,&
&  0.46473E+00_JPRB, 0.46887E+00_JPRB, 0.46969E+00_JPRB, 0.47159E+00_JPRB, 0.47165E+00_JPRB, 0.47190E+00_JPRB, 0.47172E+00_JPRB ,&
&  0.47163E+00_JPRB, 0.47155E+00_JPRB, 0.47217E+00_JPRB, 0.46765E+00_JPRB, 0.46639E+00_JPRB, 0.47137E+00_JPRB, 0.47105E+00_JPRB ,&
&  0.47116E+00_JPRB, 0.46957E+00_JPRB, 0.46735E+00_JPRB, 0.46531E+00_JPRB, 0.46365E+00_JPRB, 0.46257E+00_JPRB, 0.46181E+00_JPRB ,&
&  0.46083E+00_JPRB, 0.45733E+00_JPRB, 0.45376E+00_JPRB, 0.45052E+00_JPRB, 0.44705E+00_JPRB, 0.44455E+00_JPRB, 0.44216E+00_JPRB ,&
&  0.44053E+00_JPRB, 0.43927E+00_JPRB, 0.43767E+00_JPRB, 0.43449E+00_JPRB, 0.43006E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.43711E+00_JPRB, 0.43363E+00_JPRB, 0.43086E+00_JPRB, 0.42827E+00_JPRB, 0.42670E+00_JPRB, 0.42506E+00_JPRB, 0.42361E+00_JPRB ,&
&  0.42182E+00_JPRB, 0.42020E+00_JPRB, 0.41809E+00_JPRB, 0.41671E+00_JPRB, 0.41505E+00_JPRB, 0.41361E+00_JPRB, 0.41169E+00_JPRB ,&
&  0.41065E+00_JPRB, 0.40899E+00_JPRB, 0.40850E+00_JPRB, 0.40606E+00_JPRB, 0.40593E+00_JPRB, 0.40342E+00_JPRB, 0.39811E+00_JPRB ,&
&  0.39359E+00_JPRB, 0.38860E+00_JPRB, 0.37954E+00_JPRB, 0.38310E+00_JPRB, 0.38242E+00_JPRB, 0.38361E+00_JPRB, 0.37798E+00_JPRB ,&
&  0.37501E+00_JPRB, 0.36428E+00_JPRB, 0.37291E+00_JPRB, 0.36795E+00_JPRB, 0.36426E+00_JPRB, 0.35834E+00_JPRB, 0.35021E+00_JPRB ,&
&  0.34796E+00_JPRB, 0.34338E+00_JPRB, 0.34317E+00_JPRB, 0.34569E+00_JPRB, 0.34901E+00_JPRB, 0.35375E+00_JPRB, 0.35902E+00_JPRB ,&
&  0.36118E+00_JPRB, 0.38848E+00_JPRB, 0.40498E+00_JPRB, 0.41714E+00_JPRB, 0.42716E+00_JPRB, 0.43503E+00_JPRB, 0.44234E+00_JPRB ,&
&  0.44878E+00_JPRB, 0.45446E+00_JPRB, 0.45974E+00_JPRB, 0.46056E+00_JPRB, 0.46635E+00_JPRB, 0.46798E+00_JPRB, 0.46977E+00_JPRB ,&
&  0.47146E+00_JPRB, 0.47075E+00_JPRB, 0.47005E+00_JPRB, 0.46791E+00_JPRB, 0.46720E+00_JPRB, 0.46526E+00_JPRB, 0.46661E+00_JPRB ,&
&  0.46614E+00_JPRB, 0.46668E+00_JPRB, 0.46840E+00_JPRB, 0.46995E+00_JPRB, 0.47094E+00_JPRB, 0.47198E+00_JPRB, 0.47331E+00_JPRB ,&
&  0.47453E+00_JPRB, 0.47759E+00_JPRB, 0.48005E+00_JPRB, 0.48431E+00_JPRB, 0.48749E+00_JPRB, 0.48729E+00_JPRB, 0.48704E+00_JPRB ,&
&  0.48796E+00_JPRB, 0.48359E+00_JPRB, 0.47970E+00_JPRB, 0.47657E+00_JPRB, 0.47459E+00_JPRB, 0.47333E+00_JPRB, 0.47317E+00_JPRB ,&
&  0.47245E+00_JPRB, 0.46925E+00_JPRB, 0.46561E+00_JPRB, 0.46142E+00_JPRB, 0.45721E+00_JPRB, 0.45452E+00_JPRB, 0.45220E+00_JPRB ,&
&  0.45081E+00_JPRB, 0.44968E+00_JPRB, 0.44750E+00_JPRB, 0.44410E+00_JPRB, 0.44054E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.45353E+00_JPRB, 0.45115E+00_JPRB, 0.44945E+00_JPRB, 0.44718E+00_JPRB, 0.44591E+00_JPRB, 0.44422E+00_JPRB, 0.44275E+00_JPRB ,&
&  0.44147E+00_JPRB, 0.43893E+00_JPRB, 0.43575E+00_JPRB, 0.43191E+00_JPRB, 0.42899E+00_JPRB, 0.42619E+00_JPRB, 0.42307E+00_JPRB ,&
&  0.41821E+00_JPRB, 0.41268E+00_JPRB, 0.40699E+00_JPRB, 0.40226E+00_JPRB, 0.40172E+00_JPRB, 0.39975E+00_JPRB, 0.39742E+00_JPRB ,&
&  0.39252E+00_JPRB, 0.38714E+00_JPRB, 0.38061E+00_JPRB, 0.37991E+00_JPRB, 0.38070E+00_JPRB, 0.37629E+00_JPRB, 0.37768E+00_JPRB ,&
&  0.38501E+00_JPRB, 0.38668E+00_JPRB, 0.38691E+00_JPRB, 0.38681E+00_JPRB, 0.37126E+00_JPRB, 0.34245E+00_JPRB, 0.35163E+00_JPRB ,&
&  0.34968E+00_JPRB, 0.34101E+00_JPRB, 0.33765E+00_JPRB, 0.33568E+00_JPRB, 0.33909E+00_JPRB, 0.34773E+00_JPRB, 0.35713E+00_JPRB ,&
&  0.36250E+00_JPRB, 0.40819E+00_JPRB, 0.42673E+00_JPRB, 0.43924E+00_JPRB, 0.44664E+00_JPRB, 0.45338E+00_JPRB, 0.46077E+00_JPRB ,&
&  0.46702E+00_JPRB, 0.47618E+00_JPRB, 0.48782E+00_JPRB, 0.49831E+00_JPRB, 0.50735E+00_JPRB, 0.51420E+00_JPRB, 0.52032E+00_JPRB ,&
&  0.52783E+00_JPRB, 0.52693E+00_JPRB, 0.51992E+00_JPRB, 0.51134E+00_JPRB, 0.50343E+00_JPRB, 0.49503E+00_JPRB, 0.48762E+00_JPRB ,&
&  0.48313E+00_JPRB, 0.47825E+00_JPRB, 0.47962E+00_JPRB, 0.47854E+00_JPRB, 0.47807E+00_JPRB, 0.47902E+00_JPRB, 0.48408E+00_JPRB ,&
&  0.48750E+00_JPRB, 0.49227E+00_JPRB, 0.49607E+00_JPRB, 0.50134E+00_JPRB, 0.50482E+00_JPRB, 0.50428E+00_JPRB, 0.50372E+00_JPRB ,&
&  0.50181E+00_JPRB, 0.50421E+00_JPRB, 0.50366E+00_JPRB, 0.50467E+00_JPRB, 0.50791E+00_JPRB, 0.50904E+00_JPRB, 0.51104E+00_JPRB ,&
&  0.51202E+00_JPRB, 0.51210E+00_JPRB, 0.50945E+00_JPRB, 0.50465E+00_JPRB, 0.49811E+00_JPRB, 0.49080E+00_JPRB, 0.48247E+00_JPRB ,&
&  0.47491E+00_JPRB, 0.46723E+00_JPRB, 0.46097E+00_JPRB, 0.45752E+00_JPRB, 0.45506E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.47848E+00_JPRB, 0.47382E+00_JPRB, 0.47018E+00_JPRB, 0.46819E+00_JPRB, 0.46650E+00_JPRB, 0.46374E+00_JPRB, 0.46128E+00_JPRB ,&
&  0.45904E+00_JPRB, 0.45675E+00_JPRB, 0.45337E+00_JPRB, 0.44960E+00_JPRB, 0.44621E+00_JPRB, 0.44344E+00_JPRB, 0.44117E+00_JPRB ,&
&  0.43554E+00_JPRB, 0.42045E+00_JPRB, 0.41450E+00_JPRB, 0.40753E+00_JPRB, 0.39831E+00_JPRB, 0.39329E+00_JPRB, 0.38989E+00_JPRB ,&
&  0.38547E+00_JPRB, 0.38322E+00_JPRB, 0.38341E+00_JPRB, 0.37925E+00_JPRB, 0.37708E+00_JPRB, 0.38134E+00_JPRB, 0.38414E+00_JPRB ,&
&  0.37431E+00_JPRB, 0.37539E+00_JPRB, 0.39146E+00_JPRB, 0.39253E+00_JPRB, 0.37777E+00_JPRB, 0.37373E+00_JPRB, 0.36435E+00_JPRB ,&
&  0.35935E+00_JPRB, 0.34566E+00_JPRB, 0.34116E+00_JPRB, 0.33835E+00_JPRB, 0.34417E+00_JPRB, 0.35294E+00_JPRB, 0.37481E+00_JPRB ,&
&  0.39842E+00_JPRB, 0.44147E+00_JPRB, 0.45717E+00_JPRB, 0.46802E+00_JPRB, 0.47893E+00_JPRB, 0.49036E+00_JPRB, 0.50288E+00_JPRB ,&
&  0.51949E+00_JPRB, 0.53763E+00_JPRB, 0.55960E+00_JPRB, 0.57832E+00_JPRB, 0.59618E+00_JPRB, 0.60509E+00_JPRB, 0.60547E+00_JPRB ,&
&  0.61500E+00_JPRB, 0.61278E+00_JPRB, 0.59457E+00_JPRB, 0.57815E+00_JPRB, 0.55960E+00_JPRB, 0.54111E+00_JPRB, 0.51906E+00_JPRB ,&
&  0.51156E+00_JPRB, 0.51648E+00_JPRB, 0.50947E+00_JPRB, 0.50343E+00_JPRB, 0.49909E+00_JPRB, 0.49736E+00_JPRB, 0.49793E+00_JPRB ,&
&  0.49639E+00_JPRB, 0.50884E+00_JPRB, 0.56013E+00_JPRB, 0.56731E+00_JPRB, 0.56546E+00_JPRB, 0.56099E+00_JPRB, 0.55653E+00_JPRB ,&
&  0.55635E+00_JPRB, 0.56083E+00_JPRB, 0.56769E+00_JPRB, 0.57105E+00_JPRB, 0.57545E+00_JPRB, 0.57240E+00_JPRB, 0.57686E+00_JPRB ,&
&  0.57384E+00_JPRB, 0.57033E+00_JPRB, 0.56401E+00_JPRB, 0.55983E+00_JPRB, 0.55468E+00_JPRB, 0.54938E+00_JPRB, 0.54186E+00_JPRB ,&
&  0.53276E+00_JPRB, 0.52125E+00_JPRB, 0.50927E+00_JPRB, 0.49758E+00_JPRB, 0.48671E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.51044E+00_JPRB, 0.49621E+00_JPRB, 0.48725E+00_JPRB, 0.48440E+00_JPRB, 0.48374E+00_JPRB, 0.47028E+00_JPRB, 0.46049E+00_JPRB ,&
&  0.45487E+00_JPRB, 0.45451E+00_JPRB, 0.45370E+00_JPRB, 0.45044E+00_JPRB, 0.44909E+00_JPRB, 0.44780E+00_JPRB, 0.44739E+00_JPRB ,&
&  0.44550E+00_JPRB, 0.44022E+00_JPRB, 0.42911E+00_JPRB, 0.41153E+00_JPRB, 0.39823E+00_JPRB, 0.38912E+00_JPRB, 0.38381E+00_JPRB ,&
&  0.38035E+00_JPRB, 0.37971E+00_JPRB, 0.38430E+00_JPRB, 0.38544E+00_JPRB, 0.38467E+00_JPRB, 0.38148E+00_JPRB, 0.37740E+00_JPRB ,&
&  0.37748E+00_JPRB, 0.38483E+00_JPRB, 0.37982E+00_JPRB, 0.38196E+00_JPRB, 0.39101E+00_JPRB, 0.38897E+00_JPRB, 0.37581E+00_JPRB ,&
&  0.36546E+00_JPRB, 0.35518E+00_JPRB, 0.34981E+00_JPRB, 0.34874E+00_JPRB, 0.35853E+00_JPRB, 0.37478E+00_JPRB, 0.40434E+00_JPRB ,&
&  0.45235E+00_JPRB, 0.47371E+00_JPRB, 0.48486E+00_JPRB, 0.49732E+00_JPRB, 0.51072E+00_JPRB, 0.52541E+00_JPRB, 0.54806E+00_JPRB ,&
&  0.58198E+00_JPRB, 0.61668E+00_JPRB, 0.65006E+00_JPRB, 0.67236E+00_JPRB, 0.69742E+00_JPRB, 0.67317E+00_JPRB, 0.66813E+00_JPRB ,&
&  0.73185E+00_JPRB, 0.75792E+00_JPRB, 0.72272E+00_JPRB, 0.68793E+00_JPRB, 0.65125E+00_JPRB, 0.61735E+00_JPRB, 0.58806E+00_JPRB ,&
&  0.56793E+00_JPRB, 0.55453E+00_JPRB, 0.54325E+00_JPRB, 0.52927E+00_JPRB, 0.51906E+00_JPRB, 0.51846E+00_JPRB, 0.52969E+00_JPRB ,&
&  0.55609E+00_JPRB, 0.63304E+00_JPRB, 0.80929E+00_JPRB, 0.68277E+00_JPRB, 0.63592E+00_JPRB, 0.62568E+00_JPRB, 0.62700E+00_JPRB ,&
&  0.64094E+00_JPRB, 0.65310E+00_JPRB, 0.66015E+00_JPRB, 0.65067E+00_JPRB, 0.63095E+00_JPRB, 0.60737E+00_JPRB, 0.59176E+00_JPRB ,&
&  0.57876E+00_JPRB, 0.56588E+00_JPRB, 0.55673E+00_JPRB, 0.55428E+00_JPRB, 0.54894E+00_JPRB, 0.54288E+00_JPRB, 0.53821E+00_JPRB ,&
&  0.53473E+00_JPRB, 0.53367E+00_JPRB, 0.53483E+00_JPRB, 0.53195E+00_JPRB, 0.52616E+00_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.50522E+00_JPRB, 0.51311E+00_JPRB, 0.51690E+00_JPRB, 0.51269E+00_JPRB, 0.50733E+00_JPRB, 0.47690E+00_JPRB, 0.45326E+00_JPRB ,&
&  0.44243E+00_JPRB, 0.43913E+00_JPRB, 0.44119E+00_JPRB, 0.44278E+00_JPRB, 0.44521E+00_JPRB, 0.44705E+00_JPRB, 0.44587E+00_JPRB ,&
&  0.44004E+00_JPRB, 0.43401E+00_JPRB, 0.42785E+00_JPRB, 0.41735E+00_JPRB, 0.40403E+00_JPRB, 0.39019E+00_JPRB, 0.37985E+00_JPRB ,&
&  0.37342E+00_JPRB, 0.37165E+00_JPRB, 0.37671E+00_JPRB, 0.38265E+00_JPRB, 0.38862E+00_JPRB, 0.39109E+00_JPRB, 0.39110E+00_JPRB ,&
&  0.38441E+00_JPRB, 0.37922E+00_JPRB, 0.38489E+00_JPRB, 0.38834E+00_JPRB, 0.39195E+00_JPRB, 0.39867E+00_JPRB, 0.38756E+00_JPRB ,&
&  0.37570E+00_JPRB, 0.36561E+00_JPRB, 0.36503E+00_JPRB, 0.37512E+00_JPRB, 0.40700E+00_JPRB, 0.46091E+00_JPRB, 0.47633E+00_JPRB ,&
&  0.48914E+00_JPRB, 0.49930E+00_JPRB, 0.51092E+00_JPRB, 0.53020E+00_JPRB, 0.55346E+00_JPRB, 0.57974E+00_JPRB, 0.62441E+00_JPRB ,&
&  0.68326E+00_JPRB, 0.72918E+00_JPRB, 0.77699E+00_JPRB, 0.71527E+00_JPRB, 0.69030E+00_JPRB, 0.71400E+00_JPRB, 0.74454E+00_JPRB ,&
&  0.77623E+00_JPRB, 0.78864E+00_JPRB, 0.78332E+00_JPRB, 0.79143E+00_JPRB, 0.75758E+00_JPRB, 0.71224E+00_JPRB, 0.65007E+00_JPRB ,&
&  0.60359E+00_JPRB, 0.57001E+00_JPRB, 0.55225E+00_JPRB, 0.54163E+00_JPRB, 0.54787E+00_JPRB, 0.56361E+00_JPRB, 0.59023E+00_JPRB ,&
&  0.63257E+00_JPRB, 0.68320E+00_JPRB, 0.70870E+00_JPRB, 0.68204E+00_JPRB, 0.69281E+00_JPRB, 0.72129E+00_JPRB, 0.74334E+00_JPRB ,&
&  0.74403E+00_JPRB, 0.74235E+00_JPRB, 0.74635E+00_JPRB, 0.73890E+00_JPRB, 0.69171E+00_JPRB, 0.64905E+00_JPRB, 0.60088E+00_JPRB ,&
&  0.55504E+00_JPRB, 0.51431E+00_JPRB, 0.49255E+00_JPRB, 0.48680E+00_JPRB, 0.48227E+00_JPRB, 0.47714E+00_JPRB, 0.46834E+00_JPRB ,&
&  0.46102E+00_JPRB, 0.46101E+00_JPRB, 0.46973E+00_JPRB, 0.48016E+00_JPRB, 0.49497E+00_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.53611E+00_JPRB, 0.54764E+00_JPRB, 0.54976E+00_JPRB, 0.55560E+00_JPRB, 0.53272E+00_JPRB, 0.51187E+00_JPRB, 0.49112E+00_JPRB ,&
&  0.48468E+00_JPRB, 0.47040E+00_JPRB, 0.46220E+00_JPRB, 0.44853E+00_JPRB, 0.43612E+00_JPRB, 0.42922E+00_JPRB, 0.43093E+00_JPRB ,&
&  0.43473E+00_JPRB, 0.43303E+00_JPRB, 0.42755E+00_JPRB, 0.41490E+00_JPRB, 0.40307E+00_JPRB, 0.39284E+00_JPRB, 0.38442E+00_JPRB ,&
&  0.37830E+00_JPRB, 0.37821E+00_JPRB, 0.38165E+00_JPRB, 0.38859E+00_JPRB, 0.39382E+00_JPRB, 0.39807E+00_JPRB, 0.39946E+00_JPRB ,&
&  0.39576E+00_JPRB, 0.38896E+00_JPRB, 0.39070E+00_JPRB, 0.39932E+00_JPRB, 0.40280E+00_JPRB, 0.41256E+00_JPRB, 0.39844E+00_JPRB ,&
&  0.38466E+00_JPRB, 0.38435E+00_JPRB, 0.40016E+00_JPRB, 0.46036E+00_JPRB, 0.47984E+00_JPRB, 0.48819E+00_JPRB, 0.50045E+00_JPRB ,&
&  0.49270E+00_JPRB, 0.49298E+00_JPRB, 0.51148E+00_JPRB, 0.55665E+00_JPRB, 0.60055E+00_JPRB, 0.66839E+00_JPRB, 0.74429E+00_JPRB ,&
&  0.80595E+00_JPRB, 0.83251E+00_JPRB, 0.85332E+00_JPRB, 0.78964E+00_JPRB, 0.78785E+00_JPRB, 0.91170E+00_JPRB, 0.93594E+00_JPRB ,&
&  0.95022E+00_JPRB, 0.94375E+00_JPRB, 0.95933E+00_JPRB, 0.91883E+00_JPRB, 0.82435E+00_JPRB, 0.75407E+00_JPRB, 0.69982E+00_JPRB ,&
&  0.64419E+00_JPRB, 0.61044E+00_JPRB, 0.60203E+00_JPRB, 0.61663E+00_JPRB, 0.65064E+00_JPRB, 0.71762E+00_JPRB, 0.80232E+00_JPRB ,&
&  0.88345E+00_JPRB, 0.93212E+00_JPRB, 0.96025E+00_JPRB, 0.95079E+00_JPRB, 0.93585E+00_JPRB, 0.91608E+00_JPRB, 0.90196E+00_JPRB ,&
&  0.88572E+00_JPRB, 0.88108E+00_JPRB, 0.87354E+00_JPRB, 0.86297E+00_JPRB, 0.80351E+00_JPRB, 0.70558E+00_JPRB, 0.60779E+00_JPRB ,&
&  0.53858E+00_JPRB, 0.49699E+00_JPRB, 0.46822E+00_JPRB, 0.46179E+00_JPRB, 0.46452E+00_JPRB, 0.46689E+00_JPRB, 0.45669E+00_JPRB ,&
&  0.45329E+00_JPRB, 0.46259E+00_JPRB, 0.47644E+00_JPRB, 0.49167E+00_JPRB, 0.51226E+00_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.57232E+00_JPRB, 0.58077E+00_JPRB, 0.58582E+00_JPRB, 0.58310E+00_JPRB, 0.57366E+00_JPRB, 0.54617E+00_JPRB, 0.50719E+00_JPRB ,&
&  0.50078E+00_JPRB, 0.47388E+00_JPRB, 0.46213E+00_JPRB, 0.45059E+00_JPRB, 0.44244E+00_JPRB, 0.42996E+00_JPRB, 0.41541E+00_JPRB ,&
&  0.41399E+00_JPRB, 0.41982E+00_JPRB, 0.42513E+00_JPRB, 0.42272E+00_JPRB, 0.41514E+00_JPRB, 0.40538E+00_JPRB, 0.39655E+00_JPRB ,&
&  0.39092E+00_JPRB, 0.39166E+00_JPRB, 0.39863E+00_JPRB, 0.40798E+00_JPRB, 0.41178E+00_JPRB, 0.41414E+00_JPRB, 0.41858E+00_JPRB ,&
&  0.41892E+00_JPRB, 0.41489E+00_JPRB, 0.41160E+00_JPRB, 0.41536E+00_JPRB, 0.41707E+00_JPRB, 0.42990E+00_JPRB, 0.43768E+00_JPRB ,&
&  0.40709E+00_JPRB, 0.41577E+00_JPRB, 0.46714E+00_JPRB, 0.48350E+00_JPRB, 0.49669E+00_JPRB, 0.50850E+00_JPRB, 0.52061E+00_JPRB ,&
&  0.53394E+00_JPRB, 0.55268E+00_JPRB, 0.57316E+00_JPRB, 0.59170E+00_JPRB, 0.65931E+00_JPRB, 0.80701E+00_JPRB, 0.86459E+00_JPRB ,&
&  0.92692E+00_JPRB, 0.82716E+00_JPRB, 0.77312E+00_JPRB, 0.78052E+00_JPRB, 0.91962E+00_JPRB, 0.99718E+00_JPRB, 0.10373E+01_JPRB ,&
&  0.10815E+01_JPRB, 0.10650E+01_JPRB, 0.10363E+01_JPRB, 0.97770E+00_JPRB, 0.89540E+00_JPRB, 0.83360E+00_JPRB, 0.80467E+00_JPRB ,&
&  0.76871E+00_JPRB, 0.76074E+00_JPRB, 0.79524E+00_JPRB, 0.87786E+00_JPRB, 0.97345E+00_JPRB, 0.10552E+01_JPRB, 0.11133E+01_JPRB ,&
&  0.11698E+01_JPRB, 0.11966E+01_JPRB, 0.11816E+01_JPRB, 0.11533E+01_JPRB, 0.11051E+01_JPRB, 0.10769E+01_JPRB, 0.10663E+01_JPRB ,&
&  0.10528E+01_JPRB, 0.10406E+01_JPRB, 0.10144E+01_JPRB, 0.97640E+00_JPRB, 0.91427E+00_JPRB, 0.85316E+00_JPRB, 0.76730E+00_JPRB ,&
&  0.63101E+00_JPRB, 0.55095E+00_JPRB, 0.49504E+00_JPRB, 0.47989E+00_JPRB, 0.48311E+00_JPRB, 0.48987E+00_JPRB, 0.49243E+00_JPRB ,&
&  0.50616E+00_JPRB, 0.51495E+00_JPRB, 0.51707E+00_JPRB, 0.53008E+00_JPRB, 0.54878E+00_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.60996E+00_JPRB, 0.61207E+00_JPRB, 0.61010E+00_JPRB, 0.60930E+00_JPRB, 0.60575E+00_JPRB, 0.57888E+00_JPRB, 0.55659E+00_JPRB ,&
&  0.61358E+00_JPRB, 0.57426E+00_JPRB, 0.59256E+00_JPRB, 0.50437E+00_JPRB, 0.46731E+00_JPRB, 0.45807E+00_JPRB, 0.44042E+00_JPRB ,&
&  0.43255E+00_JPRB, 0.43405E+00_JPRB, 0.43218E+00_JPRB, 0.43200E+00_JPRB, 0.43020E+00_JPRB, 0.42118E+00_JPRB, 0.41385E+00_JPRB ,&
&  0.41135E+00_JPRB, 0.41535E+00_JPRB, 0.42510E+00_JPRB, 0.43187E+00_JPRB, 0.43766E+00_JPRB, 0.44239E+00_JPRB, 0.44695E+00_JPRB ,&
&  0.43626E+00_JPRB, 0.43151E+00_JPRB, 0.43229E+00_JPRB, 0.42998E+00_JPRB, 0.43355E+00_JPRB, 0.44317E+00_JPRB, 0.46041E+00_JPRB ,&
&  0.47844E+00_JPRB, 0.46464E+00_JPRB, 0.49954E+00_JPRB, 0.51586E+00_JPRB, 0.52944E+00_JPRB, 0.54143E+00_JPRB, 0.55205E+00_JPRB ,&
&  0.56369E+00_JPRB, 0.58109E+00_JPRB, 0.60133E+00_JPRB, 0.62079E+00_JPRB, 0.74079E+00_JPRB, 0.97037E+00_JPRB, 0.98255E+00_JPRB ,&
&  0.98499E+00_JPRB, 0.82495E+00_JPRB, 0.91259E+00_JPRB, 0.95437E+00_JPRB, 0.10668E+01_JPRB, 0.11795E+01_JPRB, 0.11721E+01_JPRB ,&
&  0.12826E+01_JPRB, 0.13237E+01_JPRB, 0.12619E+01_JPRB, 0.11430E+01_JPRB, 0.10410E+01_JPRB, 0.98754E+00_JPRB, 0.96053E+00_JPRB ,&
&  0.96454E+00_JPRB, 0.94849E+00_JPRB, 0.10555E+01_JPRB, 0.11403E+01_JPRB, 0.11827E+01_JPRB, 0.12258E+01_JPRB, 0.12809E+01_JPRB ,&
&  0.13185E+01_JPRB, 0.13228E+01_JPRB, 0.13161E+01_JPRB, 0.12460E+01_JPRB, 0.12104E+01_JPRB, 0.11837E+01_JPRB, 0.11365E+01_JPRB ,&
&  0.11054E+01_JPRB, 0.10787E+01_JPRB, 0.10400E+01_JPRB, 0.10427E+01_JPRB, 0.95224E+00_JPRB, 0.88828E+00_JPRB, 0.84699E+00_JPRB ,&
&  0.74007E+00_JPRB, 0.61939E+00_JPRB, 0.58107E+00_JPRB, 0.56228E+00_JPRB, 0.54885E+00_JPRB, 0.54923E+00_JPRB, 0.56362E+00_JPRB ,&
&  0.55026E+00_JPRB, 0.55957E+00_JPRB, 0.59270E+00_JPRB, 0.60109E+00_JPRB, 0.60327E+00_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.64630E+00_JPRB, 0.65119E+00_JPRB, 0.65025E+00_JPRB, 0.64968E+00_JPRB, 0.64697E+00_JPRB, 0.64001E+00_JPRB, 0.60723E+00_JPRB ,&
&  0.63326E+00_JPRB, 0.61765E+00_JPRB, 0.59011E+00_JPRB, 0.57647E+00_JPRB, 0.58213E+00_JPRB, 0.54353E+00_JPRB, 0.46454E+00_JPRB ,&
&  0.43862E+00_JPRB, 0.44189E+00_JPRB, 0.45216E+00_JPRB, 0.45555E+00_JPRB, 0.45579E+00_JPRB, 0.45255E+00_JPRB, 0.45000E+00_JPRB ,&
&  0.45052E+00_JPRB, 0.45356E+00_JPRB, 0.46318E+00_JPRB, 0.47562E+00_JPRB, 0.47833E+00_JPRB, 0.48131E+00_JPRB, 0.48723E+00_JPRB ,&
&  0.47041E+00_JPRB, 0.45448E+00_JPRB, 0.44575E+00_JPRB, 0.43135E+00_JPRB, 0.45277E+00_JPRB, 0.46912E+00_JPRB, 0.49118E+00_JPRB ,&
&  0.51349E+00_JPRB, 0.53508E+00_JPRB, 0.55550E+00_JPRB, 0.57157E+00_JPRB, 0.58156E+00_JPRB, 0.59188E+00_JPRB, 0.60182E+00_JPRB ,&
&  0.61357E+00_JPRB, 0.63047E+00_JPRB, 0.65240E+00_JPRB, 0.67214E+00_JPRB, 0.90432E+00_JPRB, 0.11302E+01_JPRB, 0.11912E+01_JPRB ,&
&  0.11565E+01_JPRB, 0.12637E+01_JPRB, 0.13225E+01_JPRB, 0.12507E+01_JPRB, 0.13467E+01_JPRB, 0.14022E+01_JPRB, 0.13639E+01_JPRB ,&
&  0.14218E+01_JPRB, 0.14036E+01_JPRB, 0.13601E+01_JPRB, 0.13100E+01_JPRB, 0.12760E+01_JPRB, 0.12293E+01_JPRB, 0.11936E+01_JPRB ,&
&  0.12071E+01_JPRB, 0.13106E+01_JPRB, 0.13175E+01_JPRB, 0.13398E+01_JPRB, 0.13446E+01_JPRB, 0.13629E+01_JPRB, 0.13836E+01_JPRB ,&
&  0.13978E+01_JPRB, 0.14153E+01_JPRB, 0.14141E+01_JPRB, 0.13751E+01_JPRB, 0.13250E+01_JPRB, 0.12725E+01_JPRB, 0.12084E+01_JPRB ,&
&  0.11565E+01_JPRB, 0.10439E+01_JPRB, 0.95481E+00_JPRB, 0.92387E+00_JPRB, 0.89324E+00_JPRB, 0.82410E+00_JPRB, 0.80043E+00_JPRB ,&
&  0.77310E+00_JPRB, 0.71567E+00_JPRB, 0.72918E+00_JPRB, 0.68734E+00_JPRB, 0.66308E+00_JPRB, 0.63565E+00_JPRB, 0.57934E+00_JPRB ,&
&  0.55611E+00_JPRB, 0.60243E+00_JPRB, 0.62475E+00_JPRB, 0.63684E+00_JPRB, 0.64104E+00_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.69535E+00_JPRB, 0.70479E+00_JPRB, 0.71385E+00_JPRB, 0.71113E+00_JPRB, 0.69203E+00_JPRB, 0.66711E+00_JPRB, 0.64630E+00_JPRB ,&
&  0.63461E+00_JPRB, 0.61853E+00_JPRB, 0.60278E+00_JPRB, 0.58940E+00_JPRB, 0.58346E+00_JPRB, 0.58593E+00_JPRB, 0.54648E+00_JPRB ,&
&  0.48782E+00_JPRB, 0.47749E+00_JPRB, 0.46726E+00_JPRB, 0.46971E+00_JPRB, 0.48121E+00_JPRB, 0.48655E+00_JPRB, 0.49168E+00_JPRB ,&
&  0.50722E+00_JPRB, 0.51710E+00_JPRB, 0.52176E+00_JPRB, 0.52739E+00_JPRB, 0.53138E+00_JPRB, 0.54008E+00_JPRB, 0.54859E+00_JPRB ,&
&  0.52358E+00_JPRB, 0.49066E+00_JPRB, 0.47300E+00_JPRB, 0.46630E+00_JPRB, 0.48880E+00_JPRB, 0.52004E+00_JPRB, 0.55492E+00_JPRB ,&
&  0.58475E+00_JPRB, 0.60900E+00_JPRB, 0.63123E+00_JPRB, 0.64822E+00_JPRB, 0.65561E+00_JPRB, 0.66343E+00_JPRB, 0.67592E+00_JPRB ,&
&  0.69699E+00_JPRB, 0.70886E+00_JPRB, 0.72071E+00_JPRB, 0.75024E+00_JPRB, 0.10974E+01_JPRB, 0.14992E+01_JPRB, 0.16720E+01_JPRB ,&
&  0.16505E+01_JPRB, 0.16794E+01_JPRB, 0.17042E+01_JPRB, 0.17811E+01_JPRB, 0.17764E+01_JPRB, 0.16761E+01_JPRB, 0.15787E+01_JPRB ,&
&  0.15855E+01_JPRB, 0.14999E+01_JPRB, 0.15660E+01_JPRB, 0.15417E+01_JPRB, 0.15341E+01_JPRB, 0.15400E+01_JPRB, 0.15105E+01_JPRB ,&
&  0.15311E+01_JPRB, 0.15480E+01_JPRB, 0.15254E+01_JPRB, 0.14750E+01_JPRB, 0.14505E+01_JPRB, 0.14635E+01_JPRB, 0.14748E+01_JPRB ,&
&  0.15164E+01_JPRB, 0.15466E+01_JPRB, 0.14796E+01_JPRB, 0.14270E+01_JPRB, 0.13620E+01_JPRB, 0.14245E+01_JPRB, 0.13170E+01_JPRB ,&
&  0.11374E+01_JPRB, 0.92078E+00_JPRB, 0.90652E+00_JPRB, 0.89293E+00_JPRB, 0.89557E+00_JPRB, 0.86772E+00_JPRB, 0.81237E+00_JPRB ,&
&  0.81256E+00_JPRB, 0.86448E+00_JPRB, 0.84432E+00_JPRB, 0.77727E+00_JPRB, 0.74981E+00_JPRB, 0.70403E+00_JPRB, 0.60938E+00_JPRB ,&
&  0.60679E+00_JPRB, 0.66135E+00_JPRB, 0.67006E+00_JPRB, 0.67454E+00_JPRB, 0.68104E+00_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.76011E+00_JPRB, 0.75522E+00_JPRB, 0.76141E+00_JPRB, 0.75411E+00_JPRB, 0.73177E+00_JPRB, 0.70401E+00_JPRB, 0.67661E+00_JPRB ,&
&  0.65780E+00_JPRB, 0.64551E+00_JPRB, 0.63518E+00_JPRB, 0.62652E+00_JPRB, 0.62155E+00_JPRB, 0.61167E+00_JPRB, 0.61794E+00_JPRB ,&
&  0.53698E+00_JPRB, 0.51255E+00_JPRB, 0.49705E+00_JPRB, 0.49944E+00_JPRB, 0.53966E+00_JPRB, 0.53826E+00_JPRB, 0.54422E+00_JPRB ,&
&  0.55295E+00_JPRB, 0.56116E+00_JPRB, 0.55641E+00_JPRB, 0.55748E+00_JPRB, 0.57039E+00_JPRB, 0.60274E+00_JPRB, 0.64091E+00_JPRB ,&
&  0.62097E+00_JPRB, 0.57236E+00_JPRB, 0.55397E+00_JPRB, 0.54990E+00_JPRB, 0.58596E+00_JPRB, 0.61422E+00_JPRB, 0.67297E+00_JPRB ,&
&  0.70186E+00_JPRB, 0.71832E+00_JPRB, 0.72776E+00_JPRB, 0.73473E+00_JPRB, 0.74626E+00_JPRB, 0.76802E+00_JPRB, 0.79699E+00_JPRB ,&
&  0.82257E+00_JPRB, 0.81505E+00_JPRB, 0.82241E+00_JPRB, 0.88450E+00_JPRB, 0.13880E+01_JPRB, 0.19473E+01_JPRB, 0.22731E+01_JPRB ,&
&  0.23995E+01_JPRB, 0.22991E+01_JPRB, 0.23099E+01_JPRB, 0.25464E+01_JPRB, 0.24450E+01_JPRB, 0.22035E+01_JPRB, 0.19748E+01_JPRB ,&
&  0.18599E+01_JPRB, 0.17512E+01_JPRB, 0.17839E+01_JPRB, 0.18146E+01_JPRB, 0.17436E+01_JPRB, 0.17239E+01_JPRB, 0.17062E+01_JPRB ,&
&  0.16998E+01_JPRB, 0.17016E+01_JPRB, 0.16347E+01_JPRB, 0.15646E+01_JPRB, 0.15115E+01_JPRB, 0.15538E+01_JPRB, 0.15834E+01_JPRB ,&
&  0.16642E+01_JPRB, 0.16368E+01_JPRB, 0.14663E+01_JPRB, 0.14576E+01_JPRB, 0.11960E+01_JPRB, 0.11379E+01_JPRB, 0.12782E+01_JPRB ,&
&  0.10974E+01_JPRB, 0.94530E+00_JPRB, 0.89420E+00_JPRB, 0.90551E+00_JPRB, 0.92402E+00_JPRB, 0.96522E+00_JPRB, 0.92600E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.87331E+00_JPRB, 0.95664E+00_JPRB, 0.88794E+00_JPRB, 0.85421E+00_JPRB, 0.80871E+00_JPRB, 0.73056E+00_JPRB ,&
&  0.75584E+00_JPRB, 0.74413E+00_JPRB, 0.73937E+00_JPRB, 0.74177E+00_JPRB, 0.75229E+00_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.79301E+00_JPRB, 0.77007E+00_JPRB, 0.78656E+00_JPRB, 0.74743E+00_JPRB, 0.72746E+00_JPRB, 0.70970E+00_JPRB, 0.69681E+00_JPRB ,&
&  0.68895E+00_JPRB, 0.67981E+00_JPRB, 0.67066E+00_JPRB, 0.66123E+00_JPRB, 0.65456E+00_JPRB, 0.64376E+00_JPRB, 0.63586E+00_JPRB ,&
&  0.63364E+00_JPRB, 0.58500E+00_JPRB, 0.57071E+00_JPRB, 0.54558E+00_JPRB, 0.57826E+00_JPRB, 0.57962E+00_JPRB, 0.59030E+00_JPRB ,&
&  0.60533E+00_JPRB, 0.62625E+00_JPRB, 0.62614E+00_JPRB, 0.63009E+00_JPRB, 0.64906E+00_JPRB, 0.70354E+00_JPRB, 0.77312E+00_JPRB ,&
&  0.79559E+00_JPRB, 0.75599E+00_JPRB, 0.73437E+00_JPRB, 0.75548E+00_JPRB, 0.83466E+00_JPRB, 0.78006E+00_JPRB, 0.81079E+00_JPRB ,&
&  0.81829E+00_JPRB, 0.81421E+00_JPRB, 0.81252E+00_JPRB, 0.82499E+00_JPRB, 0.86571E+00_JPRB, 0.91963E+00_JPRB, 0.94434E+00_JPRB ,&
&  0.95372E+00_JPRB, 0.89860E+00_JPRB, 0.86334E+00_JPRB, 0.11131E+01_JPRB, 0.17416E+01_JPRB, 0.24939E+01_JPRB, 0.27950E+01_JPRB ,&
&  0.28482E+01_JPRB, 0.27895E+01_JPRB, 0.27687E+01_JPRB, 0.29757E+01_JPRB, 0.27675E+01_JPRB, 0.24830E+01_JPRB, 0.23148E+01_JPRB ,&
&  0.21866E+01_JPRB, 0.21034E+01_JPRB, 0.20976E+01_JPRB, 0.19654E+01_JPRB, 0.18222E+01_JPRB, 0.17374E+01_JPRB, 0.16803E+01_JPRB ,&
&  0.16034E+01_JPRB, 0.15841E+01_JPRB, 0.15669E+01_JPRB, 0.14771E+01_JPRB, 0.14365E+01_JPRB, 0.14128E+01_JPRB, 0.15453E+01_JPRB ,&
&  0.17171E+01_JPRB, 0.13482E+01_JPRB, 0.11424E+01_JPRB, 0.12406E+01_JPRB, 0.97210E+00_JPRB, 0.93746E+00_JPRB, 0.10308E+01_JPRB ,&
&  0.96333E+00_JPRB, 0.92845E+00_JPRB, 0.94967E+00_JPRB, 0.91674E+00_JPRB, 0.94583E+00_JPRB, 0.10183E+01_JPRB, 0.10085E+01_JPRB ,&
&  0.96021E+00_JPRB, 0.93064E+00_JPRB, 0.10829E+01_JPRB, 0.10186E+01_JPRB, 0.97693E+00_JPRB, 0.92719E+00_JPRB, 0.88490E+00_JPRB ,&
&  0.86374E+00_JPRB, 0.84311E+00_JPRB, 0.82084E+00_JPRB, 0.80880E+00_JPRB, 0.80821E+00_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.81041E+00_JPRB, 0.78507E+00_JPRB, 0.81532E+00_JPRB, 0.79200E+00_JPRB, 0.78385E+00_JPRB, 0.77528E+00_JPRB, 0.76553E+00_JPRB ,&
&  0.74103E+00_JPRB, 0.73082E+00_JPRB, 0.72004E+00_JPRB, 0.69841E+00_JPRB, 0.68182E+00_JPRB, 0.66820E+00_JPRB, 0.66173E+00_JPRB ,&
&  0.65102E+00_JPRB, 0.66007E+00_JPRB, 0.61998E+00_JPRB, 0.60898E+00_JPRB, 0.58997E+00_JPRB, 0.61428E+00_JPRB, 0.63785E+00_JPRB ,&
&  0.67741E+00_JPRB, 0.71646E+00_JPRB, 0.75837E+00_JPRB, 0.81553E+00_JPRB, 0.87591E+00_JPRB, 0.93500E+00_JPRB, 0.10371E+01_JPRB ,&
&  0.10817E+01_JPRB, 0.10317E+01_JPRB, 0.97299E+00_JPRB, 0.99885E+00_JPRB, 0.10246E+01_JPRB, 0.95522E+00_JPRB, 0.96737E+00_JPRB ,&
&  0.95159E+00_JPRB, 0.94237E+00_JPRB, 0.94500E+00_JPRB, 0.98421E+00_JPRB, 0.99214E+00_JPRB, 0.93122E+00_JPRB, 0.87067E+00_JPRB ,&
&  0.84772E+00_JPRB, 0.87532E+00_JPRB, 0.10603E+01_JPRB, 0.14948E+01_JPRB, 0.22471E+01_JPRB, 0.31066E+01_JPRB, 0.32939E+01_JPRB ,&
&  0.33045E+01_JPRB, 0.29915E+01_JPRB, 0.23413E+01_JPRB, 0.25400E+01_JPRB, 0.30046E+01_JPRB, 0.27832E+01_JPRB, 0.24863E+01_JPRB ,&
&  0.24396E+01_JPRB, 0.22497E+01_JPRB, 0.21429E+01_JPRB, 0.19410E+01_JPRB, 0.18003E+01_JPRB, 0.16697E+01_JPRB, 0.15501E+01_JPRB ,&
&  0.14493E+01_JPRB, 0.14116E+01_JPRB, 0.14025E+01_JPRB, 0.14108E+01_JPRB, 0.14089E+01_JPRB, 0.15134E+01_JPRB, 0.17025E+01_JPRB ,&
&  0.17032E+01_JPRB, 0.15724E+01_JPRB, 0.12336E+01_JPRB, 0.11842E+01_JPRB, 0.91418E+00_JPRB, 0.87902E+00_JPRB, 0.95557E+00_JPRB ,&
&  0.93084E+00_JPRB, 0.10032E+01_JPRB, 0.10965E+01_JPRB, 0.10821E+01_JPRB, 0.11968E+01_JPRB, 0.12247E+01_JPRB, 0.12206E+01_JPRB ,&
&  0.11765E+01_JPRB, 0.10823E+01_JPRB, 0.12581E+01_JPRB, 0.11670E+01_JPRB, 0.10974E+01_JPRB, 0.10257E+01_JPRB, 0.96764E+00_JPRB ,&
&  0.94391E+00_JPRB, 0.91145E+00_JPRB, 0.87671E+00_JPRB, 0.86012E+00_JPRB, 0.84208E+00_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.80472E+00_JPRB, 0.79031E+00_JPRB, 0.82644E+00_JPRB, 0.80213E+00_JPRB, 0.80805E+00_JPRB, 0.77714E+00_JPRB, 0.76399E+00_JPRB ,&
&  0.75739E+00_JPRB, 0.74507E+00_JPRB, 0.72423E+00_JPRB, 0.70162E+00_JPRB, 0.68481E+00_JPRB, 0.67162E+00_JPRB, 0.67013E+00_JPRB ,&
&  0.66252E+00_JPRB, 0.65168E+00_JPRB, 0.59168E+00_JPRB, 0.56194E+00_JPRB, 0.55228E+00_JPRB, 0.59012E+00_JPRB, 0.65886E+00_JPRB ,&
&  0.74134E+00_JPRB, 0.81915E+00_JPRB, 0.93653E+00_JPRB, 0.10347E+01_JPRB, 0.11293E+01_JPRB, 0.12160E+01_JPRB, 0.13317E+01_JPRB ,&
&  0.14367E+01_JPRB, 0.12446E+01_JPRB, 0.12024E+01_JPRB, 0.11075E+01_JPRB, 0.10858E+01_JPRB, 0.10662E+01_JPRB, 0.10259E+01_JPRB ,&
&  0.99380E+00_JPRB, 0.99074E+00_JPRB, 0.98587E+00_JPRB, 0.94332E+00_JPRB, 0.88892E+00_JPRB, 0.85500E+00_JPRB, 0.85638E+00_JPRB ,&
&  0.90402E+00_JPRB, 0.10275E+01_JPRB, 0.14570E+01_JPRB, 0.21026E+01_JPRB, 0.31545E+01_JPRB, 0.39725E+01_JPRB, 0.38771E+01_JPRB ,&
&  0.30087E+01_JPRB, 0.26759E+01_JPRB, 0.34494E+01_JPRB, 0.29805E+01_JPRB, 0.29462E+01_JPRB, 0.30171E+01_JPRB, 0.27567E+01_JPRB ,&
&  0.24212E+01_JPRB, 0.20330E+01_JPRB, 0.19540E+01_JPRB, 0.18389E+01_JPRB, 0.22871E+01_JPRB, 0.19705E+01_JPRB, 0.17341E+01_JPRB ,&
&  0.15854E+01_JPRB, 0.15550E+01_JPRB, 0.15183E+01_JPRB, 0.15560E+01_JPRB, 0.17093E+01_JPRB, 0.20017E+01_JPRB, 0.20718E+01_JPRB ,&
&  0.18587E+01_JPRB, 0.18463E+01_JPRB, 0.15431E+01_JPRB, 0.12601E+01_JPRB, 0.10665E+01_JPRB, 0.10400E+01_JPRB, 0.10423E+01_JPRB ,&
&  0.10652E+01_JPRB, 0.11041E+01_JPRB, 0.12701E+01_JPRB, 0.14610E+01_JPRB, 0.16405E+01_JPRB, 0.16265E+01_JPRB, 0.14330E+01_JPRB ,&
&  0.13970E+01_JPRB, 0.15656E+01_JPRB, 0.15701E+01_JPRB, 0.13225E+01_JPRB, 0.11999E+01_JPRB, 0.11092E+01_JPRB, 0.10438E+01_JPRB ,&
&  0.98443E+00_JPRB, 0.93410E+00_JPRB, 0.89939E+00_JPRB, 0.86964E+00_JPRB, 0.84517E+00_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.83462E+00_JPRB, 0.82591E+00_JPRB, 0.80961E+00_JPRB, 0.79973E+00_JPRB, 0.79249E+00_JPRB, 0.74800E+00_JPRB, 0.71998E+00_JPRB ,&
&  0.69958E+00_JPRB, 0.68214E+00_JPRB, 0.69142E+00_JPRB, 0.69951E+00_JPRB, 0.69452E+00_JPRB, 0.67955E+00_JPRB, 0.67612E+00_JPRB ,&
&  0.66796E+00_JPRB, 0.61784E+00_JPRB, 0.56710E+00_JPRB, 0.57331E+00_JPRB, 0.57663E+00_JPRB, 0.58555E+00_JPRB, 0.67351E+00_JPRB ,&
&  0.80058E+00_JPRB, 0.95645E+00_JPRB, 0.11201E+01_JPRB, 0.12459E+01_JPRB, 0.13182E+01_JPRB, 0.13939E+01_JPRB, 0.14912E+01_JPRB ,&
&  0.14134E+01_JPRB, 0.14051E+01_JPRB, 0.13766E+01_JPRB, 0.12529E+01_JPRB, 0.11623E+01_JPRB, 0.10772E+01_JPRB, 0.10343E+01_JPRB ,&
&  0.10129E+01_JPRB, 0.10229E+01_JPRB, 0.98896E+00_JPRB, 0.93498E+00_JPRB, 0.95000E+00_JPRB, 0.96551E+00_JPRB, 0.96897E+00_JPRB ,&
&  0.97991E+00_JPRB, 0.12632E+01_JPRB, 0.18230E+01_JPRB, 0.25780E+01_JPRB, 0.27828E+01_JPRB, 0.35395E+01_JPRB, 0.35230E+01_JPRB ,&
&  0.39781E+01_JPRB, 0.37888E+01_JPRB, 0.35322E+01_JPRB, 0.38045E+01_JPRB, 0.30527E+01_JPRB, 0.29123E+01_JPRB, 0.27718E+01_JPRB ,&
&  0.24739E+01_JPRB, 0.23072E+01_JPRB, 0.24843E+01_JPRB, 0.22768E+01_JPRB, 0.24275E+01_JPRB, 0.26997E+01_JPRB, 0.24130E+01_JPRB ,&
&  0.22728E+01_JPRB, 0.21501E+01_JPRB, 0.20099E+01_JPRB, 0.20923E+01_JPRB, 0.20891E+01_JPRB, 0.21337E+01_JPRB, 0.18197E+01_JPRB ,&
&  0.17168E+01_JPRB, 0.14529E+01_JPRB, 0.15365E+01_JPRB, 0.13702E+01_JPRB, 0.13040E+01_JPRB, 0.12361E+01_JPRB, 0.11845E+01_JPRB ,&
&  0.12415E+01_JPRB, 0.12730E+01_JPRB, 0.15667E+01_JPRB, 0.18749E+01_JPRB, 0.21585E+01_JPRB, 0.19491E+01_JPRB, 0.17689E+01_JPRB ,&
&  0.18973E+01_JPRB, 0.19759E+01_JPRB, 0.16941E+01_JPRB, 0.14256E+01_JPRB, 0.12471E+01_JPRB, 0.11631E+01_JPRB, 0.11010E+01_JPRB ,&
&  0.10358E+01_JPRB, 0.97425E+00_JPRB, 0.93517E+00_JPRB, 0.90350E+00_JPRB, 0.87474E+00_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.78266E+00_JPRB, 0.77875E+00_JPRB, 0.76182E+00_JPRB, 0.76032E+00_JPRB, 0.76368E+00_JPRB, 0.72755E+00_JPRB, 0.71064E+00_JPRB ,&
&  0.68511E+00_JPRB, 0.67339E+00_JPRB, 0.67912E+00_JPRB, 0.69464E+00_JPRB, 0.68574E+00_JPRB, 0.68365E+00_JPRB, 0.67525E+00_JPRB ,&
&  0.66679E+00_JPRB, 0.63262E+00_JPRB, 0.58911E+00_JPRB, 0.58481E+00_JPRB, 0.61173E+00_JPRB, 0.62849E+00_JPRB, 0.69073E+00_JPRB ,&
&  0.91313E+00_JPRB, 0.11307E+01_JPRB, 0.12750E+01_JPRB, 0.13713E+01_JPRB, 0.14071E+01_JPRB, 0.14347E+01_JPRB, 0.14299E+01_JPRB ,&
&  0.14079E+01_JPRB, 0.13097E+01_JPRB, 0.12300E+01_JPRB, 0.11683E+01_JPRB, 0.11126E+01_JPRB, 0.10646E+01_JPRB, 0.10225E+01_JPRB ,&
&  0.99122E+00_JPRB, 0.92315E+00_JPRB, 0.85550E+00_JPRB, 0.81206E+00_JPRB, 0.84220E+00_JPRB, 0.82810E+00_JPRB, 0.89237E+00_JPRB ,&
&  0.10829E+01_JPRB, 0.14083E+01_JPRB, 0.18572E+01_JPRB, 0.24323E+01_JPRB, 0.27781E+01_JPRB, 0.35300E+01_JPRB, 0.40424E+01_JPRB ,&
&  0.38900E+01_JPRB, 0.35286E+01_JPRB, 0.39667E+01_JPRB, 0.44508E+01_JPRB, 0.40063E+01_JPRB, 0.33518E+01_JPRB, 0.30726E+01_JPRB ,&
&  0.23511E+01_JPRB, 0.19029E+01_JPRB, 0.17848E+01_JPRB, 0.20423E+01_JPRB, 0.30202E+01_JPRB, 0.38362E+01_JPRB, 0.36539E+01_JPRB ,&
&  0.35460E+01_JPRB, 0.33129E+01_JPRB, 0.30384E+01_JPRB, 0.26271E+01_JPRB, 0.23666E+01_JPRB, 0.21510E+01_JPRB, 0.26286E+01_JPRB ,&
&  0.25374E+01_JPRB, 0.24709E+01_JPRB, 0.20647E+01_JPRB, 0.16462E+01_JPRB, 0.13387E+01_JPRB, 0.14058E+01_JPRB, 0.13794E+01_JPRB ,&
&  0.14311E+01_JPRB, 0.15318E+01_JPRB, 0.24075E+01_JPRB, 0.27371E+01_JPRB, 0.26534E+01_JPRB, 0.25488E+01_JPRB, 0.25272E+01_JPRB ,&
&  0.23064E+01_JPRB, 0.23211E+01_JPRB, 0.19231E+01_JPRB, 0.15019E+01_JPRB, 0.12537E+01_JPRB, 0.11001E+01_JPRB, 0.10077E+01_JPRB ,&
&  0.97861E+00_JPRB, 0.95230E+00_JPRB, 0.91276E+00_JPRB, 0.86038E+00_JPRB, 0.82359E+00_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.72955E+00_JPRB, 0.72451E+00_JPRB, 0.70262E+00_JPRB, 0.70361E+00_JPRB, 0.71276E+00_JPRB, 0.71655E+00_JPRB, 0.70175E+00_JPRB ,&
&  0.68214E+00_JPRB, 0.69527E+00_JPRB, 0.69962E+00_JPRB, 0.69488E+00_JPRB, 0.68047E+00_JPRB, 0.68686E+00_JPRB, 0.68572E+00_JPRB ,&
&  0.69707E+00_JPRB, 0.72150E+00_JPRB, 0.65260E+00_JPRB, 0.65427E+00_JPRB, 0.68924E+00_JPRB, 0.73392E+00_JPRB, 0.80371E+00_JPRB ,&
&  0.10925E+01_JPRB, 0.12496E+01_JPRB, 0.12776E+01_JPRB, 0.13588E+01_JPRB, 0.14533E+01_JPRB, 0.13144E+01_JPRB, 0.12817E+01_JPRB ,&
&  0.12862E+01_JPRB, 0.11970E+01_JPRB, 0.11364E+01_JPRB, 0.10895E+01_JPRB, 0.10702E+01_JPRB, 0.10586E+01_JPRB, 0.10117E+01_JPRB ,&
&  0.93284E+00_JPRB, 0.85249E+00_JPRB, 0.81868E+00_JPRB, 0.81284E+00_JPRB, 0.75853E+00_JPRB, 0.76741E+00_JPRB, 0.84641E+00_JPRB ,&
&  0.11070E+01_JPRB, 0.14470E+01_JPRB, 0.17030E+01_JPRB, 0.21005E+01_JPRB, 0.25087E+01_JPRB, 0.33435E+01_JPRB, 0.47354E+01_JPRB ,&
&  0.44012E+01_JPRB, 0.43663E+01_JPRB, 0.43422E+01_JPRB, 0.50576E+01_JPRB, 0.41818E+01_JPRB, 0.35224E+01_JPRB, 0.31059E+01_JPRB ,&
&  0.23011E+01_JPRB, 0.19425E+01_JPRB, 0.21879E+01_JPRB, 0.27881E+01_JPRB, 0.30871E+01_JPRB, 0.35504E+01_JPRB, 0.43709E+01_JPRB ,&
&  0.39232E+01_JPRB, 0.37727E+01_JPRB, 0.36065E+01_JPRB, 0.33732E+01_JPRB, 0.20432E+01_JPRB, 0.16571E+01_JPRB, 0.23470E+01_JPRB ,&
&  0.24167E+01_JPRB, 0.15700E+01_JPRB, 0.10369E+01_JPRB, 0.12664E+01_JPRB, 0.92170E+00_JPRB, 0.99100E+00_JPRB, 0.13130E+01_JPRB ,&
&  0.15961E+01_JPRB, 0.20344E+01_JPRB, 0.33344E+01_JPRB, 0.31709E+01_JPRB, 0.30008E+01_JPRB, 0.35392E+01_JPRB, 0.31939E+01_JPRB ,&
&  0.29677E+01_JPRB, 0.23077E+01_JPRB, 0.18920E+01_JPRB, 0.13922E+01_JPRB, 0.10699E+01_JPRB, 0.89880E+00_JPRB, 0.83216E+00_JPRB ,&
&  0.80770E+00_JPRB, 0.80983E+00_JPRB, 0.80141E+00_JPRB, 0.77487E+00_JPRB, 0.75170E+00_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.74403E+00_JPRB, 0.69994E+00_JPRB, 0.64734E+00_JPRB, 0.65330E+00_JPRB, 0.66836E+00_JPRB, 0.68989E+00_JPRB, 0.67931E+00_JPRB ,&
&  0.68651E+00_JPRB, 0.70773E+00_JPRB, 0.73366E+00_JPRB, 0.73252E+00_JPRB, 0.70616E+00_JPRB, 0.71400E+00_JPRB, 0.70215E+00_JPRB ,&
&  0.70511E+00_JPRB, 0.74863E+00_JPRB, 0.78021E+00_JPRB, 0.81897E+00_JPRB, 0.87346E+00_JPRB, 0.90281E+00_JPRB, 0.10656E+01_JPRB ,&
&  0.12549E+01_JPRB, 0.12768E+01_JPRB, 0.12921E+01_JPRB, 0.13454E+01_JPRB, 0.13066E+01_JPRB, 0.11437E+01_JPRB, 0.12024E+01_JPRB ,&
&  0.11990E+01_JPRB, 0.11426E+01_JPRB, 0.10742E+01_JPRB, 0.10331E+01_JPRB, 0.10290E+01_JPRB, 0.10248E+01_JPRB, 0.98601E+00_JPRB ,&
&  0.90684E+00_JPRB, 0.88238E+00_JPRB, 0.85283E+00_JPRB, 0.76016E+00_JPRB, 0.72618E+00_JPRB, 0.75445E+00_JPRB, 0.85297E+00_JPRB ,&
&  0.10982E+01_JPRB, 0.13757E+01_JPRB, 0.15369E+01_JPRB, 0.18026E+01_JPRB, 0.27547E+01_JPRB, 0.34355E+01_JPRB, 0.36157E+01_JPRB ,&
&  0.36007E+01_JPRB, 0.36567E+01_JPRB, 0.47125E+01_JPRB, 0.48471E+01_JPRB, 0.44354E+01_JPRB, 0.41803E+01_JPRB, 0.34604E+01_JPRB ,&
&  0.29789E+01_JPRB, 0.26803E+01_JPRB, 0.28046E+01_JPRB, 0.32453E+01_JPRB, 0.37107E+01_JPRB, 0.36768E+01_JPRB, 0.44525E+01_JPRB ,&
&  0.45058E+01_JPRB, 0.42375E+01_JPRB, 0.33310E+01_JPRB, 0.25205E+01_JPRB, 0.22854E+01_JPRB, 0.16794E+01_JPRB, 0.11628E+01_JPRB ,&
&  0.10175E+01_JPRB, 0.96362E+00_JPRB, 0.84897E+00_JPRB, 0.85186E+00_JPRB, 0.75723E+00_JPRB, 0.83200E+00_JPRB, 0.11481E+01_JPRB ,&
&  0.19527E+01_JPRB, 0.28334E+01_JPRB, 0.33039E+01_JPRB, 0.29402E+01_JPRB, 0.28101E+01_JPRB, 0.34989E+01_JPRB, 0.40527E+01_JPRB ,&
&  0.23459E+01_JPRB, 0.19811E+01_JPRB, 0.13963E+01_JPRB, 0.10092E+01_JPRB, 0.84658E+00_JPRB, 0.75224E+00_JPRB, 0.71841E+00_JPRB ,&
&  0.71158E+00_JPRB, 0.70533E+00_JPRB, 0.71121E+00_JPRB, 0.72385E+00_JPRB, 0.72823E+00_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.72793E+00_JPRB, 0.65680E+00_JPRB, 0.60541E+00_JPRB, 0.59984E+00_JPRB, 0.62928E+00_JPRB, 0.64593E+00_JPRB, 0.65721E+00_JPRB ,&
&  0.68842E+00_JPRB, 0.73375E+00_JPRB, 0.75927E+00_JPRB, 0.74563E+00_JPRB, 0.73302E+00_JPRB, 0.73220E+00_JPRB, 0.72093E+00_JPRB ,&
&  0.73728E+00_JPRB, 0.81034E+00_JPRB, 0.93732E+00_JPRB, 0.10501E+01_JPRB, 0.11708E+01_JPRB, 0.11380E+01_JPRB, 0.13018E+01_JPRB ,&
&  0.12874E+01_JPRB, 0.12723E+01_JPRB, 0.13685E+01_JPRB, 0.13553E+01_JPRB, 0.12994E+01_JPRB, 0.11928E+01_JPRB, 0.12039E+01_JPRB ,&
&  0.11354E+01_JPRB, 0.11048E+01_JPRB, 0.10282E+01_JPRB, 0.10304E+01_JPRB, 0.10301E+01_JPRB, 0.10092E+01_JPRB, 0.96134E+00_JPRB ,&
&  0.91282E+00_JPRB, 0.92385E+00_JPRB, 0.80190E+00_JPRB, 0.77823E+00_JPRB, 0.78338E+00_JPRB, 0.84050E+00_JPRB, 0.94657E+00_JPRB ,&
&  0.11498E+01_JPRB, 0.13678E+01_JPRB, 0.15578E+01_JPRB, 0.20672E+01_JPRB, 0.25126E+01_JPRB, 0.24875E+01_JPRB, 0.32436E+01_JPRB ,&
&  0.38884E+01_JPRB, 0.43200E+01_JPRB, 0.43426E+01_JPRB, 0.49820E+01_JPRB, 0.48241E+01_JPRB, 0.42564E+01_JPRB, 0.38022E+01_JPRB ,&
&  0.32510E+01_JPRB, 0.33534E+01_JPRB, 0.39768E+01_JPRB, 0.42597E+01_JPRB, 0.51430E+01_JPRB, 0.48616E+01_JPRB, 0.50941E+01_JPRB ,&
&  0.52542E+01_JPRB, 0.47876E+01_JPRB, 0.38582E+01_JPRB, 0.30496E+01_JPRB, 0.32906E+01_JPRB, 0.28748E+01_JPRB, 0.15352E+01_JPRB ,&
&  0.12605E+01_JPRB, 0.10937E+01_JPRB, 0.87465E+00_JPRB, 0.75774E+00_JPRB, 0.71040E+00_JPRB, 0.82880E+00_JPRB, 0.13552E+01_JPRB ,&
&  0.17236E+01_JPRB, 0.29748E+01_JPRB, 0.34418E+01_JPRB, 0.31061E+01_JPRB, 0.27945E+01_JPRB, 0.34651E+01_JPRB, 0.32190E+01_JPRB ,&
&  0.23696E+01_JPRB, 0.17764E+01_JPRB, 0.10281E+01_JPRB, 0.80636E+00_JPRB, 0.75609E+00_JPRB, 0.71888E+00_JPRB, 0.67967E+00_JPRB ,&
&  0.64168E+00_JPRB, 0.63777E+00_JPRB, 0.65608E+00_JPRB, 0.67783E+00_JPRB, 0.69205E+00_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.63511E+00_JPRB, 0.58522E+00_JPRB, 0.57012E+00_JPRB, 0.56207E+00_JPRB, 0.60574E+00_JPRB, 0.65346E+00_JPRB, 0.66213E+00_JPRB ,&
&  0.71040E+00_JPRB, 0.75728E+00_JPRB, 0.78118E+00_JPRB, 0.77161E+00_JPRB, 0.75071E+00_JPRB, 0.75236E+00_JPRB, 0.77693E+00_JPRB ,&
&  0.83860E+00_JPRB, 0.95967E+00_JPRB, 0.11372E+01_JPRB, 0.13240E+01_JPRB, 0.13959E+01_JPRB, 0.13880E+01_JPRB, 0.13759E+01_JPRB ,&
&  0.12401E+01_JPRB, 0.12707E+01_JPRB, 0.14108E+01_JPRB, 0.14521E+01_JPRB, 0.13232E+01_JPRB, 0.12833E+01_JPRB, 0.11975E+01_JPRB ,&
&  0.11190E+01_JPRB, 0.11027E+01_JPRB, 0.10719E+01_JPRB, 0.10772E+01_JPRB, 0.10543E+01_JPRB, 0.10218E+01_JPRB, 0.99891E+00_JPRB ,&
&  0.97227E+00_JPRB, 0.86395E+00_JPRB, 0.80634E+00_JPRB, 0.80482E+00_JPRB, 0.89317E+00_JPRB, 0.10286E+01_JPRB, 0.11758E+01_JPRB ,&
&  0.13608E+01_JPRB, 0.15675E+01_JPRB, 0.18542E+01_JPRB, 0.23840E+01_JPRB, 0.21410E+01_JPRB, 0.27843E+01_JPRB, 0.32988E+01_JPRB ,&
&  0.35696E+01_JPRB, 0.38044E+01_JPRB, 0.37568E+01_JPRB, 0.43039E+01_JPRB, 0.46043E+01_JPRB, 0.42012E+01_JPRB, 0.36569E+01_JPRB ,&
&  0.36368E+01_JPRB, 0.37189E+01_JPRB, 0.50548E+01_JPRB, 0.57196E+01_JPRB, 0.63087E+01_JPRB, 0.80864E+01_JPRB, 0.59547E+01_JPRB ,&
&  0.50087E+01_JPRB, 0.51800E+01_JPRB, 0.43792E+01_JPRB, 0.34716E+01_JPRB, 0.34774E+01_JPRB, 0.36921E+01_JPRB, 0.34087E+01_JPRB ,&
&  0.16194E+01_JPRB, 0.10368E+01_JPRB, 0.96199E+00_JPRB, 0.86204E+00_JPRB, 0.72824E+00_JPRB, 0.83006E+00_JPRB, 0.19716E+01_JPRB ,&
&  0.16865E+01_JPRB, 0.27543E+01_JPRB, 0.31467E+01_JPRB, 0.27975E+01_JPRB, 0.27240E+01_JPRB, 0.29068E+01_JPRB, 0.30506E+01_JPRB ,&
&  0.17591E+01_JPRB, 0.12415E+01_JPRB, 0.88388E+00_JPRB, 0.74087E+00_JPRB, 0.71790E+00_JPRB, 0.70024E+00_JPRB, 0.66325E+00_JPRB ,&
&  0.62559E+00_JPRB, 0.60489E+00_JPRB, 0.61687E+00_JPRB, 0.61240E+00_JPRB, 0.64089E+00_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.58432E+00_JPRB, 0.57661E+00_JPRB, 0.61328E+00_JPRB, 0.63979E+00_JPRB, 0.66411E+00_JPRB, 0.67913E+00_JPRB, 0.70621E+00_JPRB ,&
&  0.76177E+00_JPRB, 0.80800E+00_JPRB, 0.84031E+00_JPRB, 0.83482E+00_JPRB, 0.82228E+00_JPRB, 0.86055E+00_JPRB, 0.93529E+00_JPRB ,&
&  0.10370E+01_JPRB, 0.11775E+01_JPRB, 0.13433E+01_JPRB, 0.14556E+01_JPRB, 0.15010E+01_JPRB, 0.13694E+01_JPRB, 0.12504E+01_JPRB ,&
&  0.11672E+01_JPRB, 0.12282E+01_JPRB, 0.12698E+01_JPRB, 0.13358E+01_JPRB, 0.13476E+01_JPRB, 0.12660E+01_JPRB, 0.11767E+01_JPRB ,&
&  0.11334E+01_JPRB, 0.11137E+01_JPRB, 0.10801E+01_JPRB, 0.10863E+01_JPRB, 0.10981E+01_JPRB, 0.11046E+01_JPRB, 0.10663E+01_JPRB ,&
&  0.95827E+00_JPRB, 0.90964E+00_JPRB, 0.88880E+00_JPRB, 0.95668E+00_JPRB, 0.11389E+01_JPRB, 0.13389E+01_JPRB, 0.15337E+01_JPRB ,&
&  0.17356E+01_JPRB, 0.19316E+01_JPRB, 0.21817E+01_JPRB, 0.23475E+01_JPRB, 0.26309E+01_JPRB, 0.29293E+01_JPRB, 0.31709E+01_JPRB ,&
&  0.32697E+01_JPRB, 0.35676E+01_JPRB, 0.38596E+01_JPRB, 0.40797E+01_JPRB, 0.42258E+01_JPRB, 0.40115E+01_JPRB, 0.36852E+01_JPRB ,&
&  0.39840E+01_JPRB, 0.47678E+01_JPRB, 0.58449E+01_JPRB, 0.63439E+01_JPRB, 0.66868E+01_JPRB, 0.81116E+01_JPRB, 0.79783E+01_JPRB ,&
&  0.62283E+01_JPRB, 0.45724E+01_JPRB, 0.38983E+01_JPRB, 0.37376E+01_JPRB, 0.38019E+01_JPRB, 0.39575E+01_JPRB, 0.49780E+01_JPRB ,&
&  0.45888E+01_JPRB, 0.21925E+01_JPRB, 0.14049E+01_JPRB, 0.14610E+01_JPRB, 0.85469E+00_JPRB, 0.76052E+00_JPRB, 0.12252E+01_JPRB ,&
&  0.16736E+01_JPRB, 0.24654E+01_JPRB, 0.28748E+01_JPRB, 0.23788E+01_JPRB, 0.24106E+01_JPRB, 0.21690E+01_JPRB, 0.19188E+01_JPRB ,&
&  0.12504E+01_JPRB, 0.94561E+00_JPRB, 0.75175E+00_JPRB, 0.69435E+00_JPRB, 0.66705E+00_JPRB, 0.67519E+00_JPRB, 0.67695E+00_JPRB ,&
&  0.63054E+00_JPRB, 0.57707E+00_JPRB, 0.57844E+00_JPRB, 0.58542E+00_JPRB, 0.58079E+00_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.62501E+00_JPRB, 0.65124E+00_JPRB, 0.71343E+00_JPRB, 0.76768E+00_JPRB, 0.79344E+00_JPRB, 0.80216E+00_JPRB, 0.80157E+00_JPRB ,&
&  0.83769E+00_JPRB, 0.89621E+00_JPRB, 0.93045E+00_JPRB, 0.95311E+00_JPRB, 0.99250E+00_JPRB, 0.10630E+01_JPRB, 0.11448E+01_JPRB ,&
&  0.12374E+01_JPRB, 0.13408E+01_JPRB, 0.14505E+01_JPRB, 0.15627E+01_JPRB, 0.17053E+01_JPRB, 0.17614E+01_JPRB, 0.11119E+01_JPRB ,&
&  0.10213E+01_JPRB, 0.12626E+01_JPRB, 0.12149E+01_JPRB, 0.12837E+01_JPRB, 0.13174E+01_JPRB, 0.13330E+01_JPRB, 0.11952E+01_JPRB ,&
&  0.11198E+01_JPRB, 0.10880E+01_JPRB, 0.10699E+01_JPRB, 0.11004E+01_JPRB, 0.11603E+01_JPRB, 0.12223E+01_JPRB, 0.12023E+01_JPRB ,&
&  0.11880E+01_JPRB, 0.12146E+01_JPRB, 0.12640E+01_JPRB, 0.14351E+01_JPRB, 0.16761E+01_JPRB, 0.18704E+01_JPRB, 0.20070E+01_JPRB ,&
&  0.21178E+01_JPRB, 0.22490E+01_JPRB, 0.24355E+01_JPRB, 0.25707E+01_JPRB, 0.28639E+01_JPRB, 0.30249E+01_JPRB, 0.31811E+01_JPRB ,&
&  0.31734E+01_JPRB, 0.31055E+01_JPRB, 0.35125E+01_JPRB, 0.38136E+01_JPRB, 0.38524E+01_JPRB, 0.36737E+01_JPRB, 0.37007E+01_JPRB ,&
&  0.46919E+01_JPRB, 0.59436E+01_JPRB, 0.73445E+01_JPRB, 0.65691E+01_JPRB, 0.62990E+01_JPRB, 0.69966E+01_JPRB, 0.75704E+01_JPRB ,&
&  0.62316E+01_JPRB, 0.48174E+01_JPRB, 0.43646E+01_JPRB, 0.39616E+01_JPRB, 0.39517E+01_JPRB, 0.39108E+01_JPRB, 0.43597E+01_JPRB ,&
&  0.44663E+01_JPRB, 0.33582E+01_JPRB, 0.19699E+01_JPRB, 0.11216E+01_JPRB, 0.82246E+00_JPRB, 0.86717E+00_JPRB, 0.11040E+01_JPRB ,&
&  0.17553E+01_JPRB, 0.22231E+01_JPRB, 0.25859E+01_JPRB, 0.27276E+01_JPRB, 0.17985E+01_JPRB, 0.14387E+01_JPRB, 0.11688E+01_JPRB ,&
&  0.79312E+00_JPRB, 0.66903E+00_JPRB, 0.66030E+00_JPRB, 0.65167E+00_JPRB, 0.63054E+00_JPRB, 0.65555E+00_JPRB, 0.69054E+00_JPRB ,&
&  0.64995E+00_JPRB, 0.58117E+00_JPRB, 0.58363E+00_JPRB, 0.58749E+00_JPRB, 0.59891E+00_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.69147E+00_JPRB, 0.73852E+00_JPRB, 0.80160E+00_JPRB, 0.89238E+00_JPRB, 0.98300E+00_JPRB, 0.10453E+01_JPRB, 0.98683E+00_JPRB ,&
&  0.93028E+00_JPRB, 0.96218E+00_JPRB, 0.10083E+01_JPRB, 0.10619E+01_JPRB, 0.11224E+01_JPRB, 0.11902E+01_JPRB, 0.12555E+01_JPRB ,&
&  0.13240E+01_JPRB, 0.13985E+01_JPRB, 0.14749E+01_JPRB, 0.15750E+01_JPRB, 0.16843E+01_JPRB, 0.17334E+01_JPRB, 0.12802E+01_JPRB ,&
&  0.97775E+00_JPRB, 0.15421E+01_JPRB, 0.12367E+01_JPRB, 0.13074E+01_JPRB, 0.13407E+01_JPRB, 0.14368E+01_JPRB, 0.12591E+01_JPRB ,&
&  0.10977E+01_JPRB, 0.10924E+01_JPRB, 0.11133E+01_JPRB, 0.11076E+01_JPRB, 0.11616E+01_JPRB, 0.12720E+01_JPRB, 0.14067E+01_JPRB ,&
&  0.14749E+01_JPRB, 0.15747E+01_JPRB, 0.17217E+01_JPRB, 0.19273E+01_JPRB, 0.20857E+01_JPRB, 0.22179E+01_JPRB, 0.23209E+01_JPRB ,&
&  0.24194E+01_JPRB, 0.25516E+01_JPRB, 0.26529E+01_JPRB, 0.28333E+01_JPRB, 0.29332E+01_JPRB, 0.31096E+01_JPRB, 0.32541E+01_JPRB ,&
&  0.32118E+01_JPRB, 0.31265E+01_JPRB, 0.33148E+01_JPRB, 0.33123E+01_JPRB, 0.33983E+01_JPRB, 0.39441E+01_JPRB, 0.42711E+01_JPRB ,&
&  0.54601E+01_JPRB, 0.60613E+01_JPRB, 0.72798E+01_JPRB, 0.58279E+01_JPRB, 0.52559E+01_JPRB, 0.54742E+01_JPRB, 0.50371E+01_JPRB ,&
&  0.41951E+01_JPRB, 0.40313E+01_JPRB, 0.38608E+01_JPRB, 0.38327E+01_JPRB, 0.38430E+01_JPRB, 0.38025E+01_JPRB, 0.37349E+01_JPRB ,&
&  0.29355E+01_JPRB, 0.16300E+01_JPRB, 0.11614E+01_JPRB, 0.88322E+00_JPRB, 0.73646E+00_JPRB, 0.81608E+00_JPRB, 0.12586E+01_JPRB ,&
&  0.20077E+01_JPRB, 0.23251E+01_JPRB, 0.25339E+01_JPRB, 0.18933E+01_JPRB, 0.11728E+01_JPRB, 0.88636E+00_JPRB, 0.70111E+00_JPRB ,&
&  0.55288E+00_JPRB, 0.53781E+00_JPRB, 0.58135E+00_JPRB, 0.59477E+00_JPRB, 0.58375E+00_JPRB, 0.60298E+00_JPRB, 0.61997E+00_JPRB ,&
&  0.60759E+00_JPRB, 0.60215E+00_JPRB, 0.63358E+00_JPRB, 0.63330E+00_JPRB, 0.65563E+00_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.75887E+00_JPRB, 0.82018E+00_JPRB, 0.92372E+00_JPRB, 0.10630E+01_JPRB, 0.12509E+01_JPRB, 0.14276E+01_JPRB, 0.14652E+01_JPRB ,&
&  0.12163E+01_JPRB, 0.97964E+00_JPRB, 0.10620E+01_JPRB, 0.11448E+01_JPRB, 0.12184E+01_JPRB, 0.12987E+01_JPRB, 0.13703E+01_JPRB ,&
&  0.14625E+01_JPRB, 0.15581E+01_JPRB, 0.16604E+01_JPRB, 0.17855E+01_JPRB, 0.19203E+01_JPRB, 0.20397E+01_JPRB, 0.19087E+01_JPRB ,&
&  0.13764E+01_JPRB, 0.11901E+01_JPRB, 0.14601E+01_JPRB, 0.13459E+01_JPRB, 0.12812E+01_JPRB, 0.13703E+01_JPRB, 0.13717E+01_JPRB ,&
&  0.12368E+01_JPRB, 0.11380E+01_JPRB, 0.11775E+01_JPRB, 0.11406E+01_JPRB, 0.10272E+01_JPRB, 0.11724E+01_JPRB, 0.14070E+01_JPRB ,&
&  0.15544E+01_JPRB, 0.16947E+01_JPRB, 0.18709E+01_JPRB, 0.20654E+01_JPRB, 0.22084E+01_JPRB, 0.23833E+01_JPRB, 0.25357E+01_JPRB ,&
&  0.26572E+01_JPRB, 0.27959E+01_JPRB, 0.28526E+01_JPRB, 0.28718E+01_JPRB, 0.29948E+01_JPRB, 0.31464E+01_JPRB, 0.32191E+01_JPRB ,&
&  0.32365E+01_JPRB, 0.33900E+01_JPRB, 0.35330E+01_JPRB, 0.36480E+01_JPRB, 0.37038E+01_JPRB, 0.43762E+01_JPRB, 0.48177E+01_JPRB ,&
&  0.53755E+01_JPRB, 0.54402E+01_JPRB, 0.59319E+01_JPRB, 0.46170E+01_JPRB, 0.41254E+01_JPRB, 0.38317E+01_JPRB, 0.35642E+01_JPRB ,&
&  0.33344E+01_JPRB, 0.33914E+01_JPRB, 0.33440E+01_JPRB, 0.34178E+01_JPRB, 0.35805E+01_JPRB, 0.33187E+01_JPRB, 0.30850E+01_JPRB ,&
&  0.18476E+01_JPRB, 0.12822E+01_JPRB, 0.91976E+00_JPRB, 0.83284E+00_JPRB, 0.77358E+00_JPRB, 0.89751E+00_JPRB, 0.12042E+01_JPRB ,&
&  0.16746E+01_JPRB, 0.15424E+01_JPRB, 0.14063E+01_JPRB, 0.12799E+01_JPRB, 0.82280E+00_JPRB, 0.62102E+00_JPRB, 0.52919E+00_JPRB ,&
&  0.49414E+00_JPRB, 0.50838E+00_JPRB, 0.53698E+00_JPRB, 0.54266E+00_JPRB, 0.53069E+00_JPRB, 0.54870E+00_JPRB, 0.58202E+00_JPRB ,&
&  0.60283E+00_JPRB, 0.62399E+00_JPRB, 0.66350E+00_JPRB, 0.67875E+00_JPRB, 0.72134E+00_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.73756E+00_JPRB, 0.82283E+00_JPRB, 0.88759E+00_JPRB, 0.94871E+00_JPRB, 0.97770E+00_JPRB, 0.10308E+01_JPRB, 0.10180E+01_JPRB ,&
&  0.94271E+00_JPRB, 0.88898E+00_JPRB, 0.95754E+00_JPRB, 0.10470E+01_JPRB, 0.11283E+01_JPRB, 0.11782E+01_JPRB, 0.12297E+01_JPRB ,&
&  0.12916E+01_JPRB, 0.13550E+01_JPRB, 0.14272E+01_JPRB, 0.14959E+01_JPRB, 0.15930E+01_JPRB, 0.17013E+01_JPRB, 0.17541E+01_JPRB ,&
&  0.14806E+01_JPRB, 0.10794E+01_JPRB, 0.13163E+01_JPRB, 0.14333E+01_JPRB, 0.12368E+01_JPRB, 0.12692E+01_JPRB, 0.13439E+01_JPRB ,&
&  0.11983E+01_JPRB, 0.11287E+01_JPRB, 0.11486E+01_JPRB, 0.11447E+01_JPRB, 0.96453E+00_JPRB, 0.10640E+01_JPRB, 0.12676E+01_JPRB ,&
&  0.14571E+01_JPRB, 0.16436E+01_JPRB, 0.18324E+01_JPRB, 0.19950E+01_JPRB, 0.21603E+01_JPRB, 0.23719E+01_JPRB, 0.25664E+01_JPRB ,&
&  0.27312E+01_JPRB, 0.28508E+01_JPRB, 0.29307E+01_JPRB, 0.30060E+01_JPRB, 0.30921E+01_JPRB, 0.32345E+01_JPRB, 0.32809E+01_JPRB ,&
&  0.33742E+01_JPRB, 0.34387E+01_JPRB, 0.37575E+01_JPRB, 0.41541E+01_JPRB, 0.44497E+01_JPRB, 0.43356E+01_JPRB, 0.45656E+01_JPRB ,&
&  0.45893E+01_JPRB, 0.43703E+01_JPRB, 0.43859E+01_JPRB, 0.43280E+01_JPRB, 0.28862E+01_JPRB, 0.26468E+01_JPRB, 0.26495E+01_JPRB ,&
&  0.29810E+01_JPRB, 0.28929E+01_JPRB, 0.28430E+01_JPRB, 0.28820E+01_JPRB, 0.28147E+01_JPRB, 0.26032E+01_JPRB, 0.21857E+01_JPRB ,&
&  0.15444E+01_JPRB, 0.11114E+01_JPRB, 0.91818E+00_JPRB, 0.89916E+00_JPRB, 0.89549E+00_JPRB, 0.11565E+01_JPRB, 0.10591E+01_JPRB ,&
&  0.12834E+01_JPRB, 0.10879E+01_JPRB, 0.10105E+01_JPRB, 0.10910E+01_JPRB, 0.83314E+00_JPRB, 0.61524E+00_JPRB, 0.52860E+00_JPRB ,&
&  0.49829E+00_JPRB, 0.51057E+00_JPRB, 0.51468E+00_JPRB, 0.50774E+00_JPRB, 0.50280E+00_JPRB, 0.52549E+00_JPRB, 0.55249E+00_JPRB ,&
&  0.57163E+00_JPRB, 0.59962E+00_JPRB, 0.62902E+00_JPRB, 0.66212E+00_JPRB, 0.70813E+00_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.61130E+00_JPRB, 0.63560E+00_JPRB, 0.67017E+00_JPRB, 0.71503E+00_JPRB, 0.74387E+00_JPRB, 0.76478E+00_JPRB, 0.76115E+00_JPRB ,&
&  0.78018E+00_JPRB, 0.80265E+00_JPRB, 0.84129E+00_JPRB, 0.87586E+00_JPRB, 0.89621E+00_JPRB, 0.94543E+00_JPRB, 0.98244E+00_JPRB ,&
&  0.99369E+00_JPRB, 0.99216E+00_JPRB, 0.10451E+01_JPRB, 0.11298E+01_JPRB, 0.12862E+01_JPRB, 0.15332E+01_JPRB, 0.17864E+01_JPRB ,&
&  0.20126E+01_JPRB, 0.20018E+01_JPRB, 0.19373E+01_JPRB, 0.12811E+01_JPRB, 0.90491E+00_JPRB, 0.10302E+01_JPRB, 0.10994E+01_JPRB ,&
&  0.11528E+01_JPRB, 0.11665E+01_JPRB, 0.10731E+01_JPRB, 0.10098E+01_JPRB, 0.98270E+00_JPRB, 0.10119E+01_JPRB, 0.11162E+01_JPRB ,&
&  0.12586E+01_JPRB, 0.14268E+01_JPRB, 0.16187E+01_JPRB, 0.17726E+01_JPRB, 0.19397E+01_JPRB, 0.21541E+01_JPRB, 0.23830E+01_JPRB ,&
&  0.25768E+01_JPRB, 0.27355E+01_JPRB, 0.28557E+01_JPRB, 0.30469E+01_JPRB, 0.31698E+01_JPRB, 0.33519E+01_JPRB, 0.34836E+01_JPRB ,&
&  0.36331E+01_JPRB, 0.36937E+01_JPRB, 0.39571E+01_JPRB, 0.42559E+01_JPRB, 0.42844E+01_JPRB, 0.38760E+01_JPRB, 0.37238E+01_JPRB ,&
&  0.36803E+01_JPRB, 0.33991E+01_JPRB, 0.32957E+01_JPRB, 0.34054E+01_JPRB, 0.22427E+01_JPRB, 0.22768E+01_JPRB, 0.26031E+01_JPRB ,&
&  0.25435E+01_JPRB, 0.24956E+01_JPRB, 0.22931E+01_JPRB, 0.19907E+01_JPRB, 0.18783E+01_JPRB, 0.17366E+01_JPRB, 0.15739E+01_JPRB ,&
&  0.13898E+01_JPRB, 0.11070E+01_JPRB, 0.92284E+00_JPRB, 0.78907E+00_JPRB, 0.87236E+00_JPRB, 0.11856E+01_JPRB, 0.93826E+00_JPRB ,&
&  0.86210E+00_JPRB, 0.98891E+00_JPRB, 0.89850E+00_JPRB, 0.11740E+01_JPRB, 0.11609E+01_JPRB, 0.77035E+00_JPRB, 0.58990E+00_JPRB ,&
&  0.50260E+00_JPRB, 0.49293E+00_JPRB, 0.50269E+00_JPRB, 0.49476E+00_JPRB, 0.49985E+00_JPRB, 0.52282E+00_JPRB, 0.52670E+00_JPRB ,&
&  0.55935E+00_JPRB, 0.57332E+00_JPRB, 0.59066E+00_JPRB, 0.60707E+00_JPRB, 0.59888E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.49133E+00_JPRB, 0.49274E+00_JPRB, 0.50253E+00_JPRB, 0.54293E+00_JPRB, 0.56362E+00_JPRB, 0.57421E+00_JPRB, 0.60154E+00_JPRB ,&
&  0.63444E+00_JPRB, 0.69262E+00_JPRB, 0.73340E+00_JPRB, 0.74908E+00_JPRB, 0.72415E+00_JPRB, 0.69853E+00_JPRB, 0.68905E+00_JPRB ,&
&  0.68968E+00_JPRB, 0.64604E+00_JPRB, 0.62226E+00_JPRB, 0.65324E+00_JPRB, 0.71959E+00_JPRB, 0.83810E+00_JPRB, 0.97625E+00_JPRB ,&
&  0.11635E+01_JPRB, 0.13577E+01_JPRB, 0.15995E+01_JPRB, 0.21788E+01_JPRB, 0.23371E+01_JPRB, 0.11430E+01_JPRB, 0.11729E+01_JPRB ,&
&  0.14760E+01_JPRB, 0.15009E+01_JPRB, 0.11923E+01_JPRB, 0.10568E+01_JPRB, 0.99576E+00_JPRB, 0.96829E+00_JPRB, 0.99395E+00_JPRB ,&
&  0.10879E+01_JPRB, 0.11884E+01_JPRB, 0.13285E+01_JPRB, 0.15074E+01_JPRB, 0.16963E+01_JPRB, 0.18696E+01_JPRB, 0.20820E+01_JPRB ,&
&  0.22657E+01_JPRB, 0.23609E+01_JPRB, 0.24816E+01_JPRB, 0.26471E+01_JPRB, 0.28869E+01_JPRB, 0.31956E+01_JPRB, 0.34780E+01_JPRB ,&
&  0.37008E+01_JPRB, 0.38032E+01_JPRB, 0.40586E+01_JPRB, 0.42064E+01_JPRB, 0.38474E+01_JPRB, 0.34752E+01_JPRB, 0.31454E+01_JPRB ,&
&  0.28393E+01_JPRB, 0.24284E+01_JPRB, 0.21227E+01_JPRB, 0.25371E+01_JPRB, 0.26301E+01_JPRB, 0.23055E+01_JPRB, 0.21929E+01_JPRB ,&
&  0.22302E+01_JPRB, 0.21591E+01_JPRB, 0.19463E+01_JPRB, 0.15665E+01_JPRB, 0.13314E+01_JPRB, 0.14150E+01_JPRB, 0.13012E+01_JPRB ,&
&  0.12079E+01_JPRB, 0.98593E+00_JPRB, 0.84607E+00_JPRB, 0.76356E+00_JPRB, 0.72521E+00_JPRB, 0.91581E+00_JPRB, 0.74389E+00_JPRB ,&
&  0.63274E+00_JPRB, 0.81931E+00_JPRB, 0.77420E+00_JPRB, 0.11609E+01_JPRB, 0.20650E+01_JPRB, 0.10590E+01_JPRB, 0.64896E+00_JPRB ,&
&  0.50055E+00_JPRB, 0.49535E+00_JPRB, 0.51048E+00_JPRB, 0.51457E+00_JPRB, 0.51217E+00_JPRB, 0.52396E+00_JPRB, 0.52526E+00_JPRB ,&
&  0.53271E+00_JPRB, 0.52647E+00_JPRB, 0.52881E+00_JPRB, 0.51366E+00_JPRB, 0.48660E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.56256E+00_JPRB, 0.56311E+00_JPRB, 0.55135E+00_JPRB, 0.54344E+00_JPRB, 0.54368E+00_JPRB, 0.54748E+00_JPRB, 0.55519E+00_JPRB ,&
&  0.56928E+00_JPRB, 0.59121E+00_JPRB, 0.61381E+00_JPRB, 0.64600E+00_JPRB, 0.65534E+00_JPRB, 0.63025E+00_JPRB, 0.63989E+00_JPRB ,&
&  0.63884E+00_JPRB, 0.65470E+00_JPRB, 0.64913E+00_JPRB, 0.72796E+00_JPRB, 0.75457E+00_JPRB, 0.76583E+00_JPRB, 0.85759E+00_JPRB ,&
&  0.84800E+00_JPRB, 0.93823E+00_JPRB, 0.10095E+01_JPRB, 0.11350E+01_JPRB, 0.18798E+01_JPRB, 0.17193E+01_JPRB, 0.11961E+01_JPRB ,&
&  0.12600E+01_JPRB, 0.12850E+01_JPRB, 0.11340E+01_JPRB, 0.99661E+00_JPRB, 0.95183E+00_JPRB, 0.88132E+00_JPRB, 0.86195E+00_JPRB ,&
&  0.91136E+00_JPRB, 0.98267E+00_JPRB, 0.11255E+01_JPRB, 0.12689E+01_JPRB, 0.13893E+01_JPRB, 0.15045E+01_JPRB, 0.16395E+01_JPRB ,&
&  0.17349E+01_JPRB, 0.18286E+01_JPRB, 0.18665E+01_JPRB, 0.19024E+01_JPRB, 0.22017E+01_JPRB, 0.25042E+01_JPRB, 0.27407E+01_JPRB ,&
&  0.29262E+01_JPRB, 0.31018E+01_JPRB, 0.33937E+01_JPRB, 0.32675E+01_JPRB, 0.29593E+01_JPRB, 0.27719E+01_JPRB, 0.23257E+01_JPRB ,&
&  0.18914E+01_JPRB, 0.14784E+01_JPRB, 0.13088E+01_JPRB, 0.16125E+01_JPRB, 0.17028E+01_JPRB, 0.15155E+01_JPRB, 0.17417E+01_JPRB ,&
&  0.18016E+01_JPRB, 0.18231E+01_JPRB, 0.16181E+01_JPRB, 0.14433E+01_JPRB, 0.12441E+01_JPRB, 0.12193E+01_JPRB, 0.11447E+01_JPRB ,&
&  0.10073E+01_JPRB, 0.83659E+00_JPRB, 0.73877E+00_JPRB, 0.67590E+00_JPRB, 0.63462E+00_JPRB, 0.72457E+00_JPRB, 0.77401E+00_JPRB ,&
&  0.80402E+00_JPRB, 0.72040E+00_JPRB, 0.72856E+00_JPRB, 0.10671E+01_JPRB, 0.10442E+01_JPRB, 0.65128E+00_JPRB, 0.50742E+00_JPRB ,&
&  0.48939E+00_JPRB, 0.49887E+00_JPRB, 0.50300E+00_JPRB, 0.50458E+00_JPRB, 0.51686E+00_JPRB, 0.52968E+00_JPRB, 0.52002E+00_JPRB ,&
&  0.52142E+00_JPRB, 0.51606E+00_JPRB, 0.51262E+00_JPRB, 0.51802E+00_JPRB, 0.53402E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.70275E+00_JPRB, 0.70654E+00_JPRB, 0.71132E+00_JPRB, 0.71934E+00_JPRB, 0.73538E+00_JPRB, 0.75259E+00_JPRB, 0.76400E+00_JPRB ,&
&  0.78991E+00_JPRB, 0.81177E+00_JPRB, 0.83546E+00_JPRB, 0.84477E+00_JPRB, 0.86234E+00_JPRB, 0.87836E+00_JPRB, 0.88482E+00_JPRB ,&
&  0.90565E+00_JPRB, 0.93039E+00_JPRB, 0.93216E+00_JPRB, 0.94326E+00_JPRB, 0.97183E+00_JPRB, 0.99080E+00_JPRB, 0.97335E+00_JPRB ,&
&  0.95146E+00_JPRB, 0.92477E+00_JPRB, 0.94488E+00_JPRB, 0.98318E+00_JPRB, 0.11756E+01_JPRB, 0.12940E+01_JPRB, 0.12775E+01_JPRB ,&
&  0.11379E+01_JPRB, 0.11339E+01_JPRB, 0.12608E+01_JPRB, 0.10417E+01_JPRB, 0.95808E+00_JPRB, 0.91425E+00_JPRB, 0.86719E+00_JPRB ,&
&  0.88272E+00_JPRB, 0.91955E+00_JPRB, 0.95875E+00_JPRB, 0.10051E+01_JPRB, 0.10696E+01_JPRB, 0.10993E+01_JPRB, 0.11541E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.11977E+01_JPRB, 0.12475E+01_JPRB, 0.13451E+01_JPRB, 0.15465E+01_JPRB, 0.18398E+01_JPRB, 0.18799E+01_JPRB ,&
&  0.19066E+01_JPRB, 0.20670E+01_JPRB, 0.19125E+01_JPRB, 0.18317E+01_JPRB, 0.17681E+01_JPRB, 0.15202E+01_JPRB, 0.12796E+01_JPRB ,&
&  0.12462E+01_JPRB, 0.11511E+01_JPRB, 0.88447E+00_JPRB, 0.11146E+01_JPRB, 0.12746E+01_JPRB, 0.13259E+01_JPRB, 0.14669E+01_JPRB ,&
&  0.15240E+01_JPRB, 0.13741E+01_JPRB, 0.11880E+01_JPRB, 0.11808E+01_JPRB, 0.11787E+01_JPRB, 0.10269E+01_JPRB, 0.97851E+00_JPRB ,&
&  0.84605E+00_JPRB, 0.75529E+00_JPRB, 0.70211E+00_JPRB, 0.69649E+00_JPRB, 0.73001E+00_JPRB, 0.93898E+00_JPRB, 0.10045E+01_JPRB ,&
&  0.87409E+00_JPRB, 0.73187E+00_JPRB, 0.81602E+00_JPRB, 0.12048E+01_JPRB, 0.73104E+00_JPRB, 0.59276E+00_JPRB, 0.54981E+00_JPRB ,&
&  0.56667E+00_JPRB, 0.57115E+00_JPRB, 0.57392E+00_JPRB, 0.57597E+00_JPRB, 0.57998E+00_JPRB, 0.59676E+00_JPRB, 0.59165E+00_JPRB ,&
&  0.58870E+00_JPRB, 0.58862E+00_JPRB, 0.60684E+00_JPRB, 0.63082E+00_JPRB, 0.67214E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.79398E+00_JPRB, 0.78578E+00_JPRB, 0.79612E+00_JPRB, 0.81179E+00_JPRB, 0.82586E+00_JPRB, 0.84160E+00_JPRB, 0.86897E+00_JPRB ,&
&  0.90275E+00_JPRB, 0.93077E+00_JPRB, 0.95886E+00_JPRB, 0.97941E+00_JPRB, 0.98668E+00_JPRB, 0.10004E+01_JPRB, 0.10131E+01_JPRB ,&
&  0.10271E+01_JPRB, 0.10227E+01_JPRB, 0.10412E+01_JPRB, 0.10723E+01_JPRB, 0.11069E+01_JPRB, 0.11210E+01_JPRB, 0.11327E+01_JPRB ,&
&  0.11475E+01_JPRB, 0.11873E+01_JPRB, 0.12341E+01_JPRB, 0.12907E+01_JPRB, 0.13862E+01_JPRB, 0.15546E+01_JPRB, 0.17258E+01_JPRB ,&
&  0.12484E+01_JPRB, 0.12746E+01_JPRB, 0.12605E+01_JPRB, 0.10632E+01_JPRB, 0.99593E+00_JPRB, 0.10317E+01_JPRB, 0.10118E+01_JPRB ,&
&  0.99227E+00_JPRB, 0.10196E+01_JPRB, 0.10411E+01_JPRB, 0.10656E+01_JPRB, 0.10958E+01_JPRB, 0.11414E+01_JPRB, 0.11442E+01_JPRB ,&
&  0.11719E+01_JPRB, 0.11926E+01_JPRB, 0.12127E+01_JPRB, 0.12716E+01_JPRB, 0.12468E+01_JPRB, 0.15299E+01_JPRB, 0.14899E+01_JPRB ,&
&  0.14091E+01_JPRB, 0.14037E+01_JPRB, 0.10776E+01_JPRB, 0.10864E+01_JPRB, 0.10946E+01_JPRB, 0.95266E+00_JPRB, 0.86209E+00_JPRB ,&
&  0.11002E+01_JPRB, 0.12124E+01_JPRB, 0.96200E+00_JPRB, 0.11306E+01_JPRB, 0.12081E+01_JPRB, 0.12421E+01_JPRB, 0.12505E+01_JPRB ,&
&  0.12202E+01_JPRB, 0.10883E+01_JPRB, 0.97913E+00_JPRB, 0.91822E+00_JPRB, 0.89317E+00_JPRB, 0.86196E+00_JPRB, 0.81307E+00_JPRB ,&
&  0.75344E+00_JPRB, 0.71714E+00_JPRB, 0.67193E+00_JPRB, 0.64469E+00_JPRB, 0.84366E+00_JPRB, 0.85687E+00_JPRB, 0.12937E+01_JPRB ,&
&  0.82160E+00_JPRB, 0.71071E+00_JPRB, 0.60174E+00_JPRB, 0.86126E+00_JPRB, 0.72538E+00_JPRB, 0.60312E+00_JPRB, 0.66278E+00_JPRB ,&
&  0.71682E+00_JPRB, 0.75136E+00_JPRB, 0.73617E+00_JPRB, 0.70588E+00_JPRB, 0.69336E+00_JPRB, 0.69395E+00_JPRB, 0.67345E+00_JPRB ,&
&  0.66656E+00_JPRB, 0.68563E+00_JPRB, 0.71965E+00_JPRB, 0.73192E+00_JPRB, 0.76417E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.83435E+00_JPRB, 0.85048E+00_JPRB, 0.86844E+00_JPRB, 0.89004E+00_JPRB, 0.91079E+00_JPRB, 0.92975E+00_JPRB, 0.95199E+00_JPRB ,&
&  0.98079E+00_JPRB, 0.10029E+01_JPRB, 0.10253E+01_JPRB, 0.10392E+01_JPRB, 0.10461E+01_JPRB, 0.10600E+01_JPRB, 0.10647E+01_JPRB ,&
&  0.10674E+01_JPRB, 0.10700E+01_JPRB, 0.10873E+01_JPRB, 0.10985E+01_JPRB, 0.10996E+01_JPRB, 0.10979E+01_JPRB, 0.11134E+01_JPRB ,&
&  0.11154E+01_JPRB, 0.11473E+01_JPRB, 0.11742E+01_JPRB, 0.12080E+01_JPRB, 0.12367E+01_JPRB, 0.12465E+01_JPRB, 0.13217E+01_JPRB ,&
&  0.10162E+01_JPRB, 0.14946E+01_JPRB, 0.14975E+01_JPRB, 0.12220E+01_JPRB, 0.11671E+01_JPRB, 0.11104E+01_JPRB, 0.11044E+01_JPRB ,&
&  0.11150E+01_JPRB, 0.11329E+01_JPRB, 0.11647E+01_JPRB, 0.12166E+01_JPRB, 0.12654E+01_JPRB, 0.13114E+01_JPRB, 0.13330E+01_JPRB ,&
&  0.13669E+01_JPRB, 0.13999E+01_JPRB, 0.14339E+01_JPRB, 0.14598E+01_JPRB, 0.15017E+01_JPRB, 0.15243E+01_JPRB, 0.14920E+01_JPRB ,&
&  0.13933E+01_JPRB, 0.13069E+01_JPRB, 0.95894E+00_JPRB, 0.92221E+00_JPRB, 0.97333E+00_JPRB, 0.92409E+00_JPRB, 0.82865E+00_JPRB ,&
&  0.10569E+01_JPRB, 0.11765E+01_JPRB, 0.11475E+01_JPRB, 0.11471E+01_JPRB, 0.11289E+01_JPRB, 0.10516E+01_JPRB, 0.10119E+01_JPRB ,&
&  0.92687E+00_JPRB, 0.81632E+00_JPRB, 0.75444E+00_JPRB, 0.71694E+00_JPRB, 0.72205E+00_JPRB, 0.71147E+00_JPRB, 0.68204E+00_JPRB ,&
&  0.63628E+00_JPRB, 0.62257E+00_JPRB, 0.62912E+00_JPRB, 0.63086E+00_JPRB, 0.63332E+00_JPRB, 0.10746E+01_JPRB, 0.13308E+01_JPRB ,&
&  0.73341E+00_JPRB, 0.64755E+00_JPRB, 0.56056E+00_JPRB, 0.73769E+00_JPRB, 0.77213E+00_JPRB, 0.79186E+00_JPRB, 0.90835E+00_JPRB ,&
&  0.10029E+01_JPRB, 0.10148E+01_JPRB, 0.92167E+00_JPRB, 0.81240E+00_JPRB, 0.77017E+00_JPRB, 0.75528E+00_JPRB, 0.73349E+00_JPRB ,&
&  0.73431E+00_JPRB, 0.75881E+00_JPRB, 0.78155E+00_JPRB, 0.79863E+00_JPRB, 0.81662E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.87810E+00_JPRB, 0.89234E+00_JPRB, 0.90657E+00_JPRB, 0.93169E+00_JPRB, 0.95927E+00_JPRB, 0.98538E+00_JPRB, 0.10064E+01_JPRB ,&
&  0.10281E+01_JPRB, 0.10442E+01_JPRB, 0.10638E+01_JPRB, 0.10813E+01_JPRB, 0.10939E+01_JPRB, 0.11014E+01_JPRB, 0.11024E+01_JPRB ,&
&  0.11054E+01_JPRB, 0.11099E+01_JPRB, 0.11233E+01_JPRB, 0.11337E+01_JPRB, 0.11320E+01_JPRB, 0.11399E+01_JPRB, 0.11578E+01_JPRB ,&
&  0.11701E+01_JPRB, 0.11819E+01_JPRB, 0.12137E+01_JPRB, 0.12585E+01_JPRB, 0.13194E+01_JPRB, 0.14411E+01_JPRB, 0.13037E+01_JPRB ,&
&  0.14156E+01_JPRB, 0.15931E+01_JPRB, 0.16133E+01_JPRB, 0.13269E+01_JPRB, 0.12151E+01_JPRB, 0.11883E+01_JPRB, 0.11912E+01_JPRB ,&
&  0.12053E+01_JPRB, 0.12168E+01_JPRB, 0.12423E+01_JPRB, 0.12899E+01_JPRB, 0.13627E+01_JPRB, 0.14377E+01_JPRB, 0.14770E+01_JPRB ,&
&  0.15306E+01_JPRB, 0.15929E+01_JPRB, 0.16601E+01_JPRB, 0.17238E+01_JPRB, 0.18275E+01_JPRB, 0.18800E+01_JPRB, 0.18550E+01_JPRB ,&
&  0.17716E+01_JPRB, 0.17146E+01_JPRB, 0.14719E+01_JPRB, 0.13993E+01_JPRB, 0.14394E+01_JPRB, 0.12719E+01_JPRB, 0.11316E+01_JPRB ,&
&  0.12278E+01_JPRB, 0.10946E+01_JPRB, 0.98924E+00_JPRB, 0.11081E+01_JPRB, 0.10552E+01_JPRB, 0.92557E+00_JPRB, 0.87883E+00_JPRB ,&
&  0.76422E+00_JPRB, 0.66397E+00_JPRB, 0.60605E+00_JPRB, 0.59783E+00_JPRB, 0.60808E+00_JPRB, 0.63787E+00_JPRB, 0.63445E+00_JPRB ,&
&  0.64206E+00_JPRB, 0.66676E+00_JPRB, 0.68509E+00_JPRB, 0.70718E+00_JPRB, 0.81672E+00_JPRB, 0.77491E+00_JPRB, 0.95976E+00_JPRB ,&
&  0.78773E+00_JPRB, 0.66097E+00_JPRB, 0.75561E+00_JPRB, 0.87910E+00_JPRB, 0.94212E+00_JPRB, 0.10102E+01_JPRB, 0.11635E+01_JPRB ,&
&  0.13418E+01_JPRB, 0.14819E+01_JPRB, 0.13363E+01_JPRB, 0.11329E+01_JPRB, 0.92198E+00_JPRB, 0.83259E+00_JPRB, 0.79669E+00_JPRB ,&
&  0.79247E+00_JPRB, 0.81833E+00_JPRB, 0.83805E+00_JPRB, 0.84823E+00_JPRB, 0.86280E+00_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.88638E+00_JPRB, 0.90160E+00_JPRB, 0.91249E+00_JPRB, 0.92953E+00_JPRB, 0.95096E+00_JPRB, 0.97546E+00_JPRB, 0.99384E+00_JPRB ,&
&  0.10141E+01_JPRB, 0.10299E+01_JPRB, 0.10445E+01_JPRB, 0.10622E+01_JPRB, 0.10749E+01_JPRB, 0.10931E+01_JPRB, 0.10981E+01_JPRB ,&
&  0.11168E+01_JPRB, 0.11127E+01_JPRB, 0.11212E+01_JPRB, 0.11295E+01_JPRB, 0.11366E+01_JPRB, 0.11424E+01_JPRB, 0.11557E+01_JPRB ,&
&  0.11641E+01_JPRB, 0.11601E+01_JPRB, 0.11684E+01_JPRB, 0.11817E+01_JPRB, 0.11859E+01_JPRB, 0.12590E+01_JPRB, 0.11258E+01_JPRB ,&
&  0.15479E+01_JPRB, 0.17457E+01_JPRB, 0.16896E+01_JPRB, 0.14121E+01_JPRB, 0.13011E+01_JPRB, 0.12771E+01_JPRB, 0.12790E+01_JPRB ,&
&  0.12803E+01_JPRB, 0.12887E+01_JPRB, 0.12928E+01_JPRB, 0.13264E+01_JPRB, 0.14025E+01_JPRB, 0.15132E+01_JPRB, 0.16092E+01_JPRB ,&
&  0.16826E+01_JPRB, 0.17738E+01_JPRB, 0.18881E+01_JPRB, 0.20163E+01_JPRB, 0.21832E+01_JPRB, 0.22835E+01_JPRB, 0.23155E+01_JPRB ,&
&  0.23064E+01_JPRB, 0.22861E+01_JPRB, 0.20924E+01_JPRB, 0.21682E+01_JPRB, 0.22258E+01_JPRB, 0.18052E+01_JPRB, 0.14650E+01_JPRB ,&
&  0.12163E+01_JPRB, 0.10664E+01_JPRB, 0.10666E+01_JPRB, 0.10783E+01_JPRB, 0.96948E+00_JPRB, 0.88571E+00_JPRB, 0.81953E+00_JPRB ,&
&  0.69530E+00_JPRB, 0.63154E+00_JPRB, 0.59921E+00_JPRB, 0.56512E+00_JPRB, 0.56550E+00_JPRB, 0.58694E+00_JPRB, 0.63867E+00_JPRB ,&
&  0.67000E+00_JPRB, 0.70498E+00_JPRB, 0.74935E+00_JPRB, 0.78168E+00_JPRB, 0.84687E+00_JPRB, 0.83009E+00_JPRB, 0.97056E+00_JPRB ,&
&  0.92446E+00_JPRB, 0.90580E+00_JPRB, 0.94703E+00_JPRB, 0.79105E+00_JPRB, 0.10412E+01_JPRB, 0.11240E+01_JPRB, 0.12703E+01_JPRB ,&
&  0.13169E+01_JPRB, 0.17794E+01_JPRB, 0.21499E+01_JPRB, 0.18055E+01_JPRB, 0.13130E+01_JPRB, 0.90745E+00_JPRB, 0.81429E+00_JPRB ,&
&  0.81144E+00_JPRB, 0.83170E+00_JPRB, 0.85223E+00_JPRB, 0.86213E+00_JPRB, 0.87384E+00_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.85763E+00_JPRB, 0.87136E+00_JPRB, 0.88388E+00_JPRB, 0.89264E+00_JPRB, 0.89365E+00_JPRB, 0.90428E+00_JPRB, 0.92462E+00_JPRB ,&
&  0.94325E+00_JPRB, 0.96141E+00_JPRB, 0.97882E+00_JPRB, 0.99310E+00_JPRB, 0.10091E+01_JPRB, 0.10185E+01_JPRB, 0.10412E+01_JPRB ,&
&  0.10741E+01_JPRB, 0.11055E+01_JPRB, 0.11216E+01_JPRB, 0.11546E+01_JPRB, 0.11738E+01_JPRB, 0.11807E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12612E+01_JPRB, 0.13151E+01_JPRB, 0.13534E+01_JPRB, 0.13750E+01_JPRB, 0.13681E+01_JPRB, 0.14441E+01_JPRB, 0.12926E+01_JPRB ,&
&  0.16406E+01_JPRB, 0.18429E+01_JPRB, 0.17412E+01_JPRB, 0.14704E+01_JPRB, 0.13706E+01_JPRB, 0.13259E+01_JPRB, 0.12912E+01_JPRB ,&
&  0.12871E+01_JPRB, 0.12693E+01_JPRB, 0.12256E+01_JPRB, 0.13018E+01_JPRB, 0.13845E+01_JPRB, 0.15074E+01_JPRB, 0.16524E+01_JPRB ,&
&  0.17721E+01_JPRB, 0.19010E+01_JPRB, 0.20386E+01_JPRB, 0.21832E+01_JPRB, 0.23735E+01_JPRB, 0.25352E+01_JPRB, 0.26580E+01_JPRB ,&
&  0.27537E+01_JPRB, 0.28240E+01_JPRB, 0.28879E+01_JPRB, 0.27175E+01_JPRB, 0.27037E+01_JPRB, 0.23917E+01_JPRB, 0.18035E+01_JPRB ,&
&  0.11924E+01_JPRB, 0.10171E+01_JPRB, 0.10393E+01_JPRB, 0.10410E+01_JPRB, 0.96664E+00_JPRB, 0.83246E+00_JPRB, 0.78337E+00_JPRB ,&
&  0.67166E+00_JPRB, 0.60470E+00_JPRB, 0.58756E+00_JPRB, 0.56987E+00_JPRB, 0.56073E+00_JPRB, 0.59581E+00_JPRB, 0.62576E+00_JPRB ,&
&  0.67411E+00_JPRB, 0.74339E+00_JPRB, 0.80027E+00_JPRB, 0.84788E+00_JPRB, 0.94943E+00_JPRB, 0.10448E+01_JPRB, 0.97854E+00_JPRB ,&
&  0.10072E+01_JPRB, 0.99984E+00_JPRB, 0.10239E+01_JPRB, 0.11383E+01_JPRB, 0.11123E+01_JPRB, 0.13127E+01_JPRB, 0.15290E+01_JPRB ,&
&  0.18295E+01_JPRB, 0.17681E+01_JPRB, 0.20728E+01_JPRB, 0.31038E+01_JPRB, 0.26577E+01_JPRB, 0.22413E+01_JPRB, 0.83709E+00_JPRB ,&
&  0.82595E+00_JPRB, 0.83972E+00_JPRB, 0.86169E+00_JPRB, 0.86226E+00_JPRB, 0.86299E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.77745E+00_JPRB, 0.80973E+00_JPRB, 0.81377E+00_JPRB, 0.83818E+00_JPRB, 0.83505E+00_JPRB, 0.82354E+00_JPRB, 0.83218E+00_JPRB ,&
&  0.84641E+00_JPRB, 0.85716E+00_JPRB, 0.87942E+00_JPRB, 0.90176E+00_JPRB, 0.91847E+00_JPRB, 0.92352E+00_JPRB, 0.93733E+00_JPRB ,&
&  0.96183E+00_JPRB, 0.99510E+00_JPRB, 0.10356E+01_JPRB, 0.10901E+01_JPRB, 0.11283E+01_JPRB, 0.11557E+01_JPRB, 0.12016E+01_JPRB ,&
&  0.12381E+01_JPRB, 0.13137E+01_JPRB, 0.13974E+01_JPRB, 0.14797E+01_JPRB, 0.15306E+01_JPRB, 0.15525E+01_JPRB, 0.12520E+01_JPRB ,&
&  0.17014E+01_JPRB, 0.17611E+01_JPRB, 0.16964E+01_JPRB, 0.14862E+01_JPRB, 0.13774E+01_JPRB, 0.13094E+01_JPRB, 0.12678E+01_JPRB ,&
&  0.12463E+01_JPRB, 0.12190E+01_JPRB, 0.11852E+01_JPRB, 0.11729E+01_JPRB, 0.13418E+01_JPRB, 0.14254E+01_JPRB, 0.15604E+01_JPRB ,&
&  0.17045E+01_JPRB, 0.18381E+01_JPRB, 0.19745E+01_JPRB, 0.21216E+01_JPRB, 0.23217E+01_JPRB, 0.25503E+01_JPRB, 0.27676E+01_JPRB ,&
&  0.29582E+01_JPRB, 0.30683E+01_JPRB, 0.32350E+01_JPRB, 0.28993E+01_JPRB, 0.28190E+01_JPRB, 0.26590E+01_JPRB, 0.17219E+01_JPRB ,&
&  0.10945E+01_JPRB, 0.10195E+01_JPRB, 0.10801E+01_JPRB, 0.10609E+01_JPRB, 0.96025E+00_JPRB, 0.82971E+00_JPRB, 0.73934E+00_JPRB ,&
&  0.64861E+00_JPRB, 0.62305E+00_JPRB, 0.60211E+00_JPRB, 0.58345E+00_JPRB, 0.57443E+00_JPRB, 0.60259E+00_JPRB, 0.62906E+00_JPRB ,&
&  0.68704E+00_JPRB, 0.73213E+00_JPRB, 0.78901E+00_JPRB, 0.84458E+00_JPRB, 0.92343E+00_JPRB, 0.10222E+01_JPRB, 0.11192E+01_JPRB ,&
&  0.97234E+00_JPRB, 0.89226E+00_JPRB, 0.92142E+00_JPRB, 0.97787E+00_JPRB, 0.10149E+01_JPRB, 0.11020E+01_JPRB, 0.12065E+01_JPRB ,&
&  0.13339E+01_JPRB, 0.14287E+01_JPRB, 0.13054E+01_JPRB, 0.10956E+01_JPRB, 0.14555E+01_JPRB, 0.15021E+01_JPRB, 0.70435E+00_JPRB ,&
&  0.69427E+00_JPRB, 0.72675E+00_JPRB, 0.74520E+00_JPRB, 0.76673E+00_JPRB, 0.78703E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.63811E+00_JPRB, 0.64538E+00_JPRB, 0.64589E+00_JPRB, 0.68144E+00_JPRB, 0.69003E+00_JPRB, 0.70548E+00_JPRB, 0.71870E+00_JPRB ,&
&  0.73028E+00_JPRB, 0.75164E+00_JPRB, 0.77658E+00_JPRB, 0.80322E+00_JPRB, 0.82433E+00_JPRB, 0.83990E+00_JPRB, 0.85211E+00_JPRB ,&
&  0.86541E+00_JPRB, 0.88204E+00_JPRB, 0.90405E+00_JPRB, 0.93297E+00_JPRB, 0.98019E+00_JPRB, 0.10332E+01_JPRB, 0.10889E+01_JPRB ,&
&  0.11421E+01_JPRB, 0.12134E+01_JPRB, 0.13026E+01_JPRB, 0.14106E+01_JPRB, 0.15199E+01_JPRB, 0.15976E+01_JPRB, 0.16392E+01_JPRB ,&
&  0.13256E+01_JPRB, 0.16065E+01_JPRB, 0.15826E+01_JPRB, 0.14607E+01_JPRB, 0.13521E+01_JPRB, 0.12956E+01_JPRB, 0.12551E+01_JPRB ,&
&  0.12293E+01_JPRB, 0.11693E+01_JPRB, 0.11662E+01_JPRB, 0.11932E+01_JPRB, 0.12714E+01_JPRB, 0.13256E+01_JPRB, 0.14189E+01_JPRB ,&
&  0.15512E+01_JPRB, 0.16577E+01_JPRB, 0.17871E+01_JPRB, 0.19187E+01_JPRB, 0.20998E+01_JPRB, 0.23781E+01_JPRB, 0.27013E+01_JPRB ,&
&  0.30440E+01_JPRB, 0.33139E+01_JPRB, 0.35168E+01_JPRB, 0.31155E+01_JPRB, 0.28656E+01_JPRB, 0.26194E+01_JPRB, 0.16612E+01_JPRB ,&
&  0.11423E+01_JPRB, 0.98339E+00_JPRB, 0.10501E+01_JPRB, 0.10957E+01_JPRB, 0.97088E+00_JPRB, 0.85761E+00_JPRB, 0.75274E+00_JPRB ,&
&  0.67120E+00_JPRB, 0.62182E+00_JPRB, 0.60878E+00_JPRB, 0.58820E+00_JPRB, 0.58502E+00_JPRB, 0.60281E+00_JPRB, 0.62169E+00_JPRB ,&
&  0.66111E+00_JPRB, 0.67785E+00_JPRB, 0.72238E+00_JPRB, 0.76201E+00_JPRB, 0.80466E+00_JPRB, 0.84054E+00_JPRB, 0.87374E+00_JPRB ,&
&  0.89563E+00_JPRB, 0.90505E+00_JPRB, 0.90938E+00_JPRB, 0.93522E+00_JPRB, 0.94593E+00_JPRB, 0.96339E+00_JPRB, 0.99072E+00_JPRB ,&
&  0.10211E+01_JPRB, 0.10396E+01_JPRB, 0.10177E+01_JPRB, 0.99250E+00_JPRB, 0.94471E+00_JPRB, 0.89790E+00_JPRB, 0.68968E+00_JPRB ,&
&  0.65370E+00_JPRB, 0.61557E+00_JPRB, 0.63132E+00_JPRB, 0.65131E+00_JPRB, 0.64299E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.56400E+00_JPRB, 0.55138E+00_JPRB, 0.53772E+00_JPRB, 0.54122E+00_JPRB, 0.54595E+00_JPRB, 0.56713E+00_JPRB, 0.56846E+00_JPRB ,&
&  0.58314E+00_JPRB, 0.62784E+00_JPRB, 0.68144E+00_JPRB, 0.71016E+00_JPRB, 0.73340E+00_JPRB, 0.75610E+00_JPRB, 0.77464E+00_JPRB ,&
&  0.78526E+00_JPRB, 0.79335E+00_JPRB, 0.80167E+00_JPRB, 0.81300E+00_JPRB, 0.83415E+00_JPRB, 0.86676E+00_JPRB, 0.91853E+00_JPRB ,&
&  0.97063E+00_JPRB, 0.10269E+01_JPRB, 0.10980E+01_JPRB, 0.11889E+01_JPRB, 0.12859E+01_JPRB, 0.13361E+01_JPRB, 0.13633E+01_JPRB ,&
&  0.86049E+00_JPRB, 0.13154E+01_JPRB, 0.15610E+01_JPRB, 0.14752E+01_JPRB, 0.13547E+01_JPRB, 0.12896E+01_JPRB, 0.12420E+01_JPRB ,&
&  0.11677E+01_JPRB, 0.10926E+01_JPRB, 0.10867E+01_JPRB, 0.12038E+01_JPRB, 0.11866E+01_JPRB, 0.12144E+01_JPRB, 0.12480E+01_JPRB ,&
&  0.13458E+01_JPRB, 0.14177E+01_JPRB, 0.14583E+01_JPRB, 0.15191E+01_JPRB, 0.16235E+01_JPRB, 0.18581E+01_JPRB, 0.22558E+01_JPRB ,&
&  0.27903E+01_JPRB, 0.33508E+01_JPRB, 0.36683E+01_JPRB, 0.31529E+01_JPRB, 0.32427E+01_JPRB, 0.29580E+01_JPRB, 0.24005E+01_JPRB ,&
&  0.14230E+01_JPRB, 0.10460E+01_JPRB, 0.10754E+01_JPRB, 0.11040E+01_JPRB, 0.99043E+00_JPRB, 0.90467E+00_JPRB, 0.81428E+00_JPRB ,&
&  0.72046E+00_JPRB, 0.66625E+00_JPRB, 0.63814E+00_JPRB, 0.61807E+00_JPRB, 0.61047E+00_JPRB, 0.61224E+00_JPRB, 0.62541E+00_JPRB ,&
&  0.63209E+00_JPRB, 0.64799E+00_JPRB, 0.67350E+00_JPRB, 0.70308E+00_JPRB, 0.74192E+00_JPRB, 0.78281E+00_JPRB, 0.82645E+00_JPRB ,&
&  0.85840E+00_JPRB, 0.88721E+00_JPRB, 0.91128E+00_JPRB, 0.93040E+00_JPRB, 0.93935E+00_JPRB, 0.94873E+00_JPRB, 0.93760E+00_JPRB ,&
&  0.92915E+00_JPRB, 0.94422E+00_JPRB, 0.95233E+00_JPRB, 0.94236E+00_JPRB, 0.90564E+00_JPRB, 0.86800E+00_JPRB, 0.83485E+00_JPRB ,&
&  0.77268E+00_JPRB, 0.68203E+00_JPRB, 0.60241E+00_JPRB, 0.58042E+00_JPRB, 0.56795E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.52677E+00_JPRB, 0.50320E+00_JPRB, 0.49458E+00_JPRB, 0.49466E+00_JPRB, 0.50438E+00_JPRB, 0.51042E+00_JPRB, 0.50998E+00_JPRB ,&
&  0.51275E+00_JPRB, 0.52397E+00_JPRB, 0.53807E+00_JPRB, 0.58387E+00_JPRB, 0.63089E+00_JPRB, 0.66551E+00_JPRB, 0.68700E+00_JPRB ,&
&  0.70444E+00_JPRB, 0.70920E+00_JPRB, 0.70633E+00_JPRB, 0.70773E+00_JPRB, 0.71023E+00_JPRB, 0.72227E+00_JPRB, 0.74979E+00_JPRB ,&
&  0.79611E+00_JPRB, 0.84540E+00_JPRB, 0.90685E+00_JPRB, 0.98665E+00_JPRB, 0.10858E+01_JPRB, 0.11797E+01_JPRB, 0.12489E+01_JPRB ,&
&  0.96116E+00_JPRB, 0.56039E+00_JPRB, 0.13407E+01_JPRB, 0.15882E+01_JPRB, 0.14134E+01_JPRB, 0.13419E+01_JPRB, 0.12382E+01_JPRB ,&
&  0.11020E+01_JPRB, 0.10599E+01_JPRB, 0.10256E+01_JPRB, 0.10828E+01_JPRB, 0.10864E+01_JPRB, 0.11155E+01_JPRB, 0.11437E+01_JPRB ,&
&  0.11985E+01_JPRB, 0.12769E+01_JPRB, 0.12922E+01_JPRB, 0.12755E+01_JPRB, 0.12815E+01_JPRB, 0.13675E+01_JPRB, 0.16297E+01_JPRB ,&
&  0.21573E+01_JPRB, 0.29362E+01_JPRB, 0.36657E+01_JPRB, 0.32929E+01_JPRB, 0.34058E+01_JPRB, 0.26767E+01_JPRB, 0.19324E+01_JPRB ,&
&  0.14403E+01_JPRB, 0.11565E+01_JPRB, 0.11161E+01_JPRB, 0.11315E+01_JPRB, 0.10380E+01_JPRB, 0.92186E+00_JPRB, 0.89938E+00_JPRB ,&
&  0.79023E+00_JPRB, 0.72001E+00_JPRB, 0.67284E+00_JPRB, 0.64797E+00_JPRB, 0.63700E+00_JPRB, 0.63607E+00_JPRB, 0.63983E+00_JPRB ,&
&  0.63984E+00_JPRB, 0.64711E+00_JPRB, 0.65720E+00_JPRB, 0.68208E+00_JPRB, 0.71631E+00_JPRB, 0.76551E+00_JPRB, 0.80648E+00_JPRB ,&
&  0.83072E+00_JPRB, 0.85031E+00_JPRB, 0.86937E+00_JPRB, 0.88099E+00_JPRB, 0.89006E+00_JPRB, 0.86992E+00_JPRB, 0.86323E+00_JPRB ,&
&  0.87285E+00_JPRB, 0.88828E+00_JPRB, 0.91002E+00_JPRB, 0.93770E+00_JPRB, 0.90743E+00_JPRB, 0.88069E+00_JPRB, 0.89920E+00_JPRB ,&
&  0.88876E+00_JPRB, 0.82651E+00_JPRB, 0.70796E+00_JPRB, 0.59259E+00_JPRB, 0.55081E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.56319E+00_JPRB, 0.54077E+00_JPRB, 0.51333E+00_JPRB, 0.50586E+00_JPRB, 0.51452E+00_JPRB, 0.50944E+00_JPRB, 0.50619E+00_JPRB ,&
&  0.50194E+00_JPRB, 0.49826E+00_JPRB, 0.48392E+00_JPRB, 0.49225E+00_JPRB, 0.52063E+00_JPRB, 0.56240E+00_JPRB, 0.59725E+00_JPRB ,&
&  0.62288E+00_JPRB, 0.63249E+00_JPRB, 0.63258E+00_JPRB, 0.63172E+00_JPRB, 0.63140E+00_JPRB, 0.63169E+00_JPRB, 0.63542E+00_JPRB ,&
&  0.65374E+00_JPRB, 0.68439E+00_JPRB, 0.72495E+00_JPRB, 0.76913E+00_JPRB, 0.82765E+00_JPRB, 0.91941E+00_JPRB, 0.10671E+01_JPRB ,&
&  0.12976E+01_JPRB, 0.97727E+00_JPRB, 0.67942E+00_JPRB, 0.13569E+01_JPRB, 0.15086E+01_JPRB, 0.14771E+01_JPRB, 0.12695E+01_JPRB ,&
&  0.11796E+01_JPRB, 0.10676E+01_JPRB, 0.10006E+01_JPRB, 0.10342E+01_JPRB, 0.98244E+00_JPRB, 0.10029E+01_JPRB, 0.10514E+01_JPRB ,&
&  0.10938E+01_JPRB, 0.11392E+01_JPRB, 0.11768E+01_JPRB, 0.11664E+01_JPRB, 0.11249E+01_JPRB, 0.11106E+01_JPRB, 0.12017E+01_JPRB ,&
&  0.15269E+01_JPRB, 0.21939E+01_JPRB, 0.32669E+01_JPRB, 0.33553E+01_JPRB, 0.34646E+01_JPRB, 0.24443E+01_JPRB, 0.17891E+01_JPRB ,&
&  0.14678E+01_JPRB, 0.14049E+01_JPRB, 0.13310E+01_JPRB, 0.12006E+01_JPRB, 0.10580E+01_JPRB, 0.96936E+00_JPRB, 0.97722E+00_JPRB ,&
&  0.87806E+00_JPRB, 0.80527E+00_JPRB, 0.74446E+00_JPRB, 0.70647E+00_JPRB, 0.68355E+00_JPRB, 0.66888E+00_JPRB, 0.65778E+00_JPRB ,&
&  0.66112E+00_JPRB, 0.66288E+00_JPRB, 0.67384E+00_JPRB, 0.68743E+00_JPRB, 0.70614E+00_JPRB, 0.73439E+00_JPRB, 0.77302E+00_JPRB ,&
&  0.80274E+00_JPRB, 0.81749E+00_JPRB, 0.82405E+00_JPRB, 0.82178E+00_JPRB, 0.79931E+00_JPRB, 0.78470E+00_JPRB, 0.80473E+00_JPRB ,&
&  0.80400E+00_JPRB, 0.81781E+00_JPRB, 0.82158E+00_JPRB, 0.84727E+00_JPRB, 0.86791E+00_JPRB, 0.87987E+00_JPRB, 0.90731E+00_JPRB ,&
&  0.94433E+00_JPRB, 0.93179E+00_JPRB, 0.83584E+00_JPRB, 0.63979E+00_JPRB, 0.58828E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.62606E+00_JPRB, 0.60917E+00_JPRB, 0.59522E+00_JPRB, 0.59367E+00_JPRB, 0.59136E+00_JPRB, 0.57611E+00_JPRB, 0.55616E+00_JPRB ,&
&  0.53083E+00_JPRB, 0.51192E+00_JPRB, 0.48538E+00_JPRB, 0.46090E+00_JPRB, 0.45685E+00_JPRB, 0.46943E+00_JPRB, 0.49597E+00_JPRB ,&
&  0.52186E+00_JPRB, 0.53568E+00_JPRB, 0.54578E+00_JPRB, 0.55553E+00_JPRB, 0.56412E+00_JPRB, 0.56765E+00_JPRB, 0.56602E+00_JPRB ,&
&  0.56963E+00_JPRB, 0.58092E+00_JPRB, 0.59889E+00_JPRB, 0.61894E+00_JPRB, 0.64560E+00_JPRB, 0.69003E+00_JPRB, 0.77279E+00_JPRB ,&
&  0.94707E+00_JPRB, 0.10377E+01_JPRB, 0.58459E+00_JPRB, 0.10853E+01_JPRB, 0.15053E+01_JPRB, 0.14884E+01_JPRB, 0.14039E+01_JPRB ,&
&  0.12890E+01_JPRB, 0.11058E+01_JPRB, 0.10661E+01_JPRB, 0.10142E+01_JPRB, 0.93025E+00_JPRB, 0.91136E+00_JPRB, 0.92813E+00_JPRB ,&
&  0.94203E+00_JPRB, 0.93100E+00_JPRB, 0.94446E+00_JPRB, 0.98243E+00_JPRB, 0.97682E+00_JPRB, 0.94581E+00_JPRB, 0.96245E+00_JPRB ,&
&  0.11199E+01_JPRB, 0.15794E+01_JPRB, 0.25847E+01_JPRB, 0.31979E+01_JPRB, 0.34079E+01_JPRB, 0.26626E+01_JPRB, 0.19929E+01_JPRB ,&
&  0.17283E+01_JPRB, 0.17586E+01_JPRB, 0.16206E+01_JPRB, 0.13539E+01_JPRB, 0.11323E+01_JPRB, 0.10591E+01_JPRB, 0.10428E+01_JPRB ,&
&  0.95751E+00_JPRB, 0.88002E+00_JPRB, 0.83103E+00_JPRB, 0.79170E+00_JPRB, 0.76097E+00_JPRB, 0.73024E+00_JPRB, 0.70498E+00_JPRB ,&
&  0.69749E+00_JPRB, 0.69318E+00_JPRB, 0.69363E+00_JPRB, 0.69615E+00_JPRB, 0.69905E+00_JPRB, 0.70811E+00_JPRB, 0.73057E+00_JPRB ,&
&  0.76046E+00_JPRB, 0.77643E+00_JPRB, 0.78420E+00_JPRB, 0.76651E+00_JPRB, 0.75045E+00_JPRB, 0.75025E+00_JPRB, 0.76746E+00_JPRB ,&
&  0.77179E+00_JPRB, 0.78250E+00_JPRB, 0.77992E+00_JPRB, 0.77901E+00_JPRB, 0.82217E+00_JPRB, 0.83744E+00_JPRB, 0.87363E+00_JPRB ,&
&  0.92242E+00_JPRB, 0.98785E+00_JPRB, 0.98840E+00_JPRB, 0.71006E+00_JPRB, 0.66132E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.67330E+00_JPRB, 0.65239E+00_JPRB, 0.64866E+00_JPRB, 0.64904E+00_JPRB, 0.64652E+00_JPRB, 0.63751E+00_JPRB, 0.61916E+00_JPRB ,&
&  0.59620E+00_JPRB, 0.57889E+00_JPRB, 0.55857E+00_JPRB, 0.53236E+00_JPRB, 0.49471E+00_JPRB, 0.46040E+00_JPRB, 0.45638E+00_JPRB ,&
&  0.47299E+00_JPRB, 0.48103E+00_JPRB, 0.48828E+00_JPRB, 0.49737E+00_JPRB, 0.50693E+00_JPRB, 0.51840E+00_JPRB, 0.52596E+00_JPRB ,&
&  0.52856E+00_JPRB, 0.53332E+00_JPRB, 0.54243E+00_JPRB, 0.55107E+00_JPRB, 0.56339E+00_JPRB, 0.58674E+00_JPRB, 0.62734E+00_JPRB ,&
&  0.74396E+00_JPRB, 0.90255E+00_JPRB, 0.93776E+00_JPRB, 0.15285E+01_JPRB, 0.17709E+01_JPRB, 0.16570E+01_JPRB, 0.16133E+01_JPRB ,&
&  0.15396E+01_JPRB, 0.13553E+01_JPRB, 0.12422E+01_JPRB, 0.10506E+01_JPRB, 0.93849E+00_JPRB, 0.88561E+00_JPRB, 0.86144E+00_JPRB ,&
&  0.84213E+00_JPRB, 0.81556E+00_JPRB, 0.78612E+00_JPRB, 0.78898E+00_JPRB, 0.79750E+00_JPRB, 0.79337E+00_JPRB, 0.80551E+00_JPRB ,&
&  0.88121E+00_JPRB, 0.11589E+01_JPRB, 0.18775E+01_JPRB, 0.30085E+01_JPRB, 0.33739E+01_JPRB, 0.30324E+01_JPRB, 0.24966E+01_JPRB ,&
&  0.24821E+01_JPRB, 0.23366E+01_JPRB, 0.20871E+01_JPRB, 0.16546E+01_JPRB, 0.13178E+01_JPRB, 0.12885E+01_JPRB, 0.11149E+01_JPRB ,&
&  0.10084E+01_JPRB, 0.93607E+00_JPRB, 0.89323E+00_JPRB, 0.86149E+00_JPRB, 0.83597E+00_JPRB, 0.80311E+00_JPRB, 0.76410E+00_JPRB ,&
&  0.73317E+00_JPRB, 0.71697E+00_JPRB, 0.70422E+00_JPRB, 0.69915E+00_JPRB, 0.69438E+00_JPRB, 0.69165E+00_JPRB, 0.69704E+00_JPRB ,&
&  0.71925E+00_JPRB, 0.73686E+00_JPRB, 0.71818E+00_JPRB, 0.71300E+00_JPRB, 0.71486E+00_JPRB, 0.71633E+00_JPRB, 0.71912E+00_JPRB ,&
&  0.72016E+00_JPRB, 0.74176E+00_JPRB, 0.76387E+00_JPRB, 0.76730E+00_JPRB, 0.78320E+00_JPRB, 0.80366E+00_JPRB, 0.81495E+00_JPRB ,&
&  0.82584E+00_JPRB, 0.81524E+00_JPRB, 0.79960E+00_JPRB, 0.73317E+00_JPRB, 0.70413E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.69413E+00_JPRB, 0.67488E+00_JPRB, 0.66254E+00_JPRB, 0.65630E+00_JPRB, 0.65139E+00_JPRB, 0.64650E+00_JPRB, 0.63170E+00_JPRB ,&
&  0.61511E+00_JPRB, 0.60496E+00_JPRB, 0.58887E+00_JPRB, 0.57249E+00_JPRB, 0.54013E+00_JPRB, 0.49921E+00_JPRB, 0.48072E+00_JPRB ,&
&  0.47008E+00_JPRB, 0.46884E+00_JPRB, 0.47740E+00_JPRB, 0.48169E+00_JPRB, 0.48483E+00_JPRB, 0.49261E+00_JPRB, 0.49949E+00_JPRB ,&
&  0.50612E+00_JPRB, 0.50796E+00_JPRB, 0.51387E+00_JPRB, 0.51938E+00_JPRB, 0.52316E+00_JPRB, 0.53796E+00_JPRB, 0.56205E+00_JPRB ,&
&  0.64155E+00_JPRB, 0.85925E+00_JPRB, 0.76816E+00_JPRB, 0.14783E+01_JPRB, 0.16951E+01_JPRB, 0.16849E+01_JPRB, 0.15509E+01_JPRB ,&
&  0.15223E+01_JPRB, 0.15108E+01_JPRB, 0.12423E+01_JPRB, 0.10655E+01_JPRB, 0.94216E+00_JPRB, 0.87707E+00_JPRB, 0.83760E+00_JPRB ,&
&  0.80812E+00_JPRB, 0.77662E+00_JPRB, 0.73558E+00_JPRB, 0.70405E+00_JPRB, 0.69314E+00_JPRB, 0.68848E+00_JPRB, 0.69846E+00_JPRB ,&
&  0.73200E+00_JPRB, 0.88223E+00_JPRB, 0.13554E+01_JPRB, 0.23487E+01_JPRB, 0.32322E+01_JPRB, 0.33454E+01_JPRB, 0.30017E+01_JPRB ,&
&  0.30206E+01_JPRB, 0.30620E+01_JPRB, 0.27569E+01_JPRB, 0.21303E+01_JPRB, 0.16913E+01_JPRB, 0.14720E+01_JPRB, 0.12510E+01_JPRB ,&
&  0.11075E+01_JPRB, 0.10167E+01_JPRB, 0.96217E+00_JPRB, 0.91669E+00_JPRB, 0.88018E+00_JPRB, 0.84377E+00_JPRB, 0.79906E+00_JPRB ,&
&  0.75785E+00_JPRB, 0.72381E+00_JPRB, 0.70612E+00_JPRB, 0.69436E+00_JPRB, 0.69144E+00_JPRB, 0.68873E+00_JPRB, 0.68719E+00_JPRB ,&
&  0.69481E+00_JPRB, 0.71260E+00_JPRB, 0.68499E+00_JPRB, 0.67660E+00_JPRB, 0.67988E+00_JPRB, 0.68278E+00_JPRB, 0.69250E+00_JPRB ,&
&  0.71307E+00_JPRB, 0.72139E+00_JPRB, 0.73161E+00_JPRB, 0.72783E+00_JPRB, 0.74667E+00_JPRB, 0.78698E+00_JPRB, 0.79131E+00_JPRB ,&
&  0.79494E+00_JPRB, 0.78202E+00_JPRB, 0.76347E+00_JPRB, 0.73451E+00_JPRB, 0.71359E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.69591E+00_JPRB, 0.68109E+00_JPRB, 0.66208E+00_JPRB, 0.64428E+00_JPRB, 0.63899E+00_JPRB, 0.63636E+00_JPRB, 0.62390E+00_JPRB ,&
&  0.61175E+00_JPRB, 0.59897E+00_JPRB, 0.58148E+00_JPRB, 0.56078E+00_JPRB, 0.53748E+00_JPRB, 0.50878E+00_JPRB, 0.48241E+00_JPRB ,&
&  0.46997E+00_JPRB, 0.46008E+00_JPRB, 0.46214E+00_JPRB, 0.46766E+00_JPRB, 0.47591E+00_JPRB, 0.48168E+00_JPRB, 0.48852E+00_JPRB ,&
&  0.49461E+00_JPRB, 0.49786E+00_JPRB, 0.50183E+00_JPRB, 0.50509E+00_JPRB, 0.50585E+00_JPRB, 0.51139E+00_JPRB, 0.53790E+00_JPRB ,&
&  0.59743E+00_JPRB, 0.90784E+00_JPRB, 0.76367E+00_JPRB, 0.12232E+01_JPRB, 0.14134E+01_JPRB, 0.14978E+01_JPRB, 0.14006E+01_JPRB ,&
&  0.13897E+01_JPRB, 0.13252E+01_JPRB, 0.11553E+01_JPRB, 0.10257E+01_JPRB, 0.92816E+00_JPRB, 0.86387E+00_JPRB, 0.81846E+00_JPRB ,&
&  0.78651E+00_JPRB, 0.74785E+00_JPRB, 0.70333E+00_JPRB, 0.66355E+00_JPRB, 0.64373E+00_JPRB, 0.63415E+00_JPRB, 0.63578E+00_JPRB ,&
&  0.64737E+00_JPRB, 0.71892E+00_JPRB, 0.98562E+00_JPRB, 0.18017E+01_JPRB, 0.27390E+01_JPRB, 0.33065E+01_JPRB, 0.34302E+01_JPRB ,&
&  0.35254E+01_JPRB, 0.37576E+01_JPRB, 0.34084E+01_JPRB, 0.27002E+01_JPRB, 0.21497E+01_JPRB, 0.17502E+01_JPRB, 0.14475E+01_JPRB ,&
&  0.12625E+01_JPRB, 0.11511E+01_JPRB, 0.10745E+01_JPRB, 0.10105E+01_JPRB, 0.94836E+00_JPRB, 0.88010E+00_JPRB, 0.81096E+00_JPRB ,&
&  0.75838E+00_JPRB, 0.72220E+00_JPRB, 0.69955E+00_JPRB, 0.69350E+00_JPRB, 0.69169E+00_JPRB, 0.69108E+00_JPRB, 0.69230E+00_JPRB ,&
&  0.69587E+00_JPRB, 0.70482E+00_JPRB, 0.68202E+00_JPRB, 0.67227E+00_JPRB, 0.67012E+00_JPRB, 0.67607E+00_JPRB, 0.68132E+00_JPRB ,&
&  0.70974E+00_JPRB, 0.71490E+00_JPRB, 0.70771E+00_JPRB, 0.70893E+00_JPRB, 0.72815E+00_JPRB, 0.78123E+00_JPRB, 0.78703E+00_JPRB ,&
&  0.78252E+00_JPRB, 0.76421E+00_JPRB, 0.74489E+00_JPRB, 0.72963E+00_JPRB, 0.71461E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.69520E+00_JPRB, 0.67615E+00_JPRB, 0.66322E+00_JPRB, 0.65047E+00_JPRB, 0.64436E+00_JPRB, 0.63789E+00_JPRB, 0.62339E+00_JPRB ,&
&  0.60418E+00_JPRB, 0.58194E+00_JPRB, 0.55974E+00_JPRB, 0.54075E+00_JPRB, 0.52602E+00_JPRB, 0.50956E+00_JPRB, 0.49615E+00_JPRB ,&
&  0.48424E+00_JPRB, 0.47015E+00_JPRB, 0.46093E+00_JPRB, 0.45751E+00_JPRB, 0.46062E+00_JPRB, 0.46549E+00_JPRB, 0.47233E+00_JPRB ,&
&  0.48065E+00_JPRB, 0.48624E+00_JPRB, 0.48725E+00_JPRB, 0.48941E+00_JPRB, 0.48749E+00_JPRB, 0.49179E+00_JPRB, 0.52042E+00_JPRB ,&
&  0.58900E+00_JPRB, 0.55263E+00_JPRB, 0.84206E+00_JPRB, 0.99437E+00_JPRB, 0.11232E+01_JPRB, 0.12426E+01_JPRB, 0.13012E+01_JPRB ,&
&  0.13279E+01_JPRB, 0.11999E+01_JPRB, 0.10532E+01_JPRB, 0.95705E+00_JPRB, 0.88127E+00_JPRB, 0.82340E+00_JPRB, 0.77696E+00_JPRB ,&
&  0.73133E+00_JPRB, 0.69638E+00_JPRB, 0.66833E+00_JPRB, 0.63214E+00_JPRB, 0.60965E+00_JPRB, 0.60103E+00_JPRB, 0.59920E+00_JPRB ,&
&  0.59599E+00_JPRB, 0.62143E+00_JPRB, 0.75241E+00_JPRB, 0.11926E+01_JPRB, 0.22517E+01_JPRB, 0.30755E+01_JPRB, 0.36620E+01_JPRB ,&
&  0.43748E+01_JPRB, 0.45649E+01_JPRB, 0.40472E+01_JPRB, 0.32572E+01_JPRB, 0.25047E+01_JPRB, 0.19986E+01_JPRB, 0.15903E+01_JPRB ,&
&  0.13548E+01_JPRB, 0.12235E+01_JPRB, 0.11465E+01_JPRB, 0.10848E+01_JPRB, 0.10196E+01_JPRB, 0.93170E+00_JPRB, 0.83347E+00_JPRB ,&
&  0.75652E+00_JPRB, 0.71302E+00_JPRB, 0.68628E+00_JPRB, 0.67159E+00_JPRB, 0.66438E+00_JPRB, 0.66514E+00_JPRB, 0.67244E+00_JPRB ,&
&  0.68376E+00_JPRB, 0.69819E+00_JPRB, 0.68065E+00_JPRB, 0.66954E+00_JPRB, 0.67120E+00_JPRB, 0.68345E+00_JPRB, 0.69553E+00_JPRB ,&
&  0.69771E+00_JPRB, 0.68839E+00_JPRB, 0.67557E+00_JPRB, 0.67930E+00_JPRB, 0.69870E+00_JPRB, 0.77192E+00_JPRB, 0.78196E+00_JPRB ,&
&  0.77760E+00_JPRB, 0.76025E+00_JPRB, 0.73333E+00_JPRB, 0.72297E+00_JPRB, 0.71724E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.68942E+00_JPRB, 0.67208E+00_JPRB, 0.66832E+00_JPRB, 0.67245E+00_JPRB, 0.67570E+00_JPRB, 0.66289E+00_JPRB, 0.62996E+00_JPRB ,&
&  0.60018E+00_JPRB, 0.57457E+00_JPRB, 0.55014E+00_JPRB, 0.53047E+00_JPRB, 0.51762E+00_JPRB, 0.50847E+00_JPRB, 0.49874E+00_JPRB ,&
&  0.49187E+00_JPRB, 0.48597E+00_JPRB, 0.47249E+00_JPRB, 0.45994E+00_JPRB, 0.45442E+00_JPRB, 0.45358E+00_JPRB, 0.45734E+00_JPRB ,&
&  0.46168E+00_JPRB, 0.46703E+00_JPRB, 0.47087E+00_JPRB, 0.47360E+00_JPRB, 0.47324E+00_JPRB, 0.47898E+00_JPRB, 0.50447E+00_JPRB ,&
&  0.58238E+00_JPRB, 0.71975E+00_JPRB, 0.82709E+00_JPRB, 0.84590E+00_JPRB, 0.92216E+00_JPRB, 0.10539E+01_JPRB, 0.11912E+01_JPRB ,&
&  0.11847E+01_JPRB, 0.10865E+01_JPRB, 0.95833E+00_JPRB, 0.87020E+00_JPRB, 0.80393E+00_JPRB, 0.75698E+00_JPRB, 0.70848E+00_JPRB ,&
&  0.66768E+00_JPRB, 0.64011E+00_JPRB, 0.61253E+00_JPRB, 0.58791E+00_JPRB, 0.57952E+00_JPRB, 0.58075E+00_JPRB, 0.58142E+00_JPRB ,&
&  0.57900E+00_JPRB, 0.57995E+00_JPRB, 0.63708E+00_JPRB, 0.93465E+00_JPRB, 0.18362E+01_JPRB, 0.24421E+01_JPRB, 0.35367E+01_JPRB ,&
&  0.45608E+01_JPRB, 0.44776E+01_JPRB, 0.40017E+01_JPRB, 0.33334E+01_JPRB, 0.26122E+01_JPRB, 0.20100E+01_JPRB, 0.16339E+01_JPRB ,&
&  0.13977E+01_JPRB, 0.12124E+01_JPRB, 0.11006E+01_JPRB, 0.10298E+01_JPRB, 0.97244E+00_JPRB, 0.91844E+00_JPRB, 0.85279E+00_JPRB ,&
&  0.78434E+00_JPRB, 0.72818E+00_JPRB, 0.68948E+00_JPRB, 0.66254E+00_JPRB, 0.64819E+00_JPRB, 0.64429E+00_JPRB, 0.64539E+00_JPRB ,&
&  0.64533E+00_JPRB, 0.65544E+00_JPRB, 0.66327E+00_JPRB, 0.65935E+00_JPRB, 0.66761E+00_JPRB, 0.68311E+00_JPRB, 0.69207E+00_JPRB ,&
&  0.67367E+00_JPRB, 0.65892E+00_JPRB, 0.65508E+00_JPRB, 0.65396E+00_JPRB, 0.69295E+00_JPRB, 0.77302E+00_JPRB, 0.77051E+00_JPRB ,&
&  0.76408E+00_JPRB, 0.74981E+00_JPRB, 0.73355E+00_JPRB, 0.71382E+00_JPRB, 0.71349E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.68086E+00_JPRB, 0.66121E+00_JPRB, 0.65151E+00_JPRB, 0.66112E+00_JPRB, 0.67472E+00_JPRB, 0.66698E+00_JPRB, 0.63573E+00_JPRB ,&
&  0.59562E+00_JPRB, 0.56305E+00_JPRB, 0.54012E+00_JPRB, 0.52398E+00_JPRB, 0.51209E+00_JPRB, 0.50188E+00_JPRB, 0.49065E+00_JPRB ,&
&  0.48108E+00_JPRB, 0.47597E+00_JPRB, 0.47228E+00_JPRB, 0.46714E+00_JPRB, 0.45723E+00_JPRB, 0.44937E+00_JPRB, 0.44672E+00_JPRB ,&
&  0.44525E+00_JPRB, 0.44577E+00_JPRB, 0.45097E+00_JPRB, 0.45681E+00_JPRB, 0.45904E+00_JPRB, 0.46229E+00_JPRB, 0.47978E+00_JPRB ,&
&  0.53824E+00_JPRB, 0.64926E+00_JPRB, 0.65716E+00_JPRB, 0.71496E+00_JPRB, 0.80953E+00_JPRB, 0.92491E+00_JPRB, 0.10264E+01_JPRB ,&
&  0.10350E+01_JPRB, 0.98443E+00_JPRB, 0.89330E+00_JPRB, 0.80093E+00_JPRB, 0.72722E+00_JPRB, 0.68461E+00_JPRB, 0.65410E+00_JPRB ,&
&  0.62510E+00_JPRB, 0.59525E+00_JPRB, 0.56932E+00_JPRB, 0.55001E+00_JPRB, 0.53718E+00_JPRB, 0.53304E+00_JPRB, 0.53470E+00_JPRB ,&
&  0.53675E+00_JPRB, 0.53705E+00_JPRB, 0.54937E+00_JPRB, 0.65591E+00_JPRB, 0.11397E+01_JPRB, 0.17898E+01_JPRB, 0.25202E+01_JPRB ,&
&  0.27475E+01_JPRB, 0.28566E+01_JPRB, 0.28191E+01_JPRB, 0.26371E+01_JPRB, 0.23187E+01_JPRB, 0.20048E+01_JPRB, 0.16664E+01_JPRB ,&
&  0.14179E+01_JPRB, 0.12489E+01_JPRB, 0.11272E+01_JPRB, 0.10249E+01_JPRB, 0.93356E+00_JPRB, 0.86057E+00_JPRB, 0.80973E+00_JPRB ,&
&  0.77285E+00_JPRB, 0.74114E+00_JPRB, 0.71142E+00_JPRB, 0.68420E+00_JPRB, 0.66339E+00_JPRB, 0.65106E+00_JPRB, 0.64960E+00_JPRB ,&
&  0.64616E+00_JPRB, 0.63598E+00_JPRB, 0.63591E+00_JPRB, 0.64043E+00_JPRB, 0.64779E+00_JPRB, 0.65015E+00_JPRB, 0.64964E+00_JPRB ,&
&  0.65391E+00_JPRB, 0.65803E+00_JPRB, 0.65539E+00_JPRB, 0.65522E+00_JPRB, 0.68450E+00_JPRB, 0.74397E+00_JPRB, 0.75935E+00_JPRB ,&
&  0.75856E+00_JPRB, 0.74276E+00_JPRB, 0.72999E+00_JPRB, 0.71932E+00_JPRB, 0.71248E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.77531E+00_JPRB, 0.69600E+00_JPRB, 0.64962E+00_JPRB, 0.63247E+00_JPRB, 0.63192E+00_JPRB, 0.61949E+00_JPRB, 0.59864E+00_JPRB ,&
&  0.57672E+00_JPRB, 0.55866E+00_JPRB, 0.54387E+00_JPRB, 0.52725E+00_JPRB, 0.51076E+00_JPRB, 0.49680E+00_JPRB, 0.48336E+00_JPRB ,&
&  0.47047E+00_JPRB, 0.46290E+00_JPRB, 0.45733E+00_JPRB, 0.45363E+00_JPRB, 0.45087E+00_JPRB, 0.44523E+00_JPRB, 0.43893E+00_JPRB ,&
&  0.43410E+00_JPRB, 0.42752E+00_JPRB, 0.42594E+00_JPRB, 0.43225E+00_JPRB, 0.43805E+00_JPRB, 0.44199E+00_JPRB, 0.45806E+00_JPRB ,&
&  0.50931E+00_JPRB, 0.53590E+00_JPRB, 0.58917E+00_JPRB, 0.63705E+00_JPRB, 0.68241E+00_JPRB, 0.74576E+00_JPRB, 0.77489E+00_JPRB ,&
&  0.76447E+00_JPRB, 0.77226E+00_JPRB, 0.75894E+00_JPRB, 0.70958E+00_JPRB, 0.65445E+00_JPRB, 0.61795E+00_JPRB, 0.59244E+00_JPRB ,&
&  0.57120E+00_JPRB, 0.54719E+00_JPRB, 0.52226E+00_JPRB, 0.50253E+00_JPRB, 0.49124E+00_JPRB, 0.48761E+00_JPRB, 0.48807E+00_JPRB ,&
&  0.49015E+00_JPRB, 0.49453E+00_JPRB, 0.49954E+00_JPRB, 0.51707E+00_JPRB, 0.62928E+00_JPRB, 0.91999E+00_JPRB, 0.13768E+01_JPRB ,&
&  0.16001E+01_JPRB, 0.16732E+01_JPRB, 0.17299E+01_JPRB, 0.17383E+01_JPRB, 0.16263E+01_JPRB, 0.15024E+01_JPRB, 0.13680E+01_JPRB ,&
&  0.12338E+01_JPRB, 0.10929E+01_JPRB, 0.97386E+00_JPRB, 0.88092E+00_JPRB, 0.80893E+00_JPRB, 0.75956E+00_JPRB, 0.73282E+00_JPRB ,&
&  0.72114E+00_JPRB, 0.71480E+00_JPRB, 0.70889E+00_JPRB, 0.69623E+00_JPRB, 0.67943E+00_JPRB, 0.66749E+00_JPRB, 0.65982E+00_JPRB ,&
&  0.65015E+00_JPRB, 0.64111E+00_JPRB, 0.63446E+00_JPRB, 0.63021E+00_JPRB, 0.62940E+00_JPRB, 0.62743E+00_JPRB, 0.62846E+00_JPRB ,&
&  0.63288E+00_JPRB, 0.64326E+00_JPRB, 0.63560E+00_JPRB, 0.63947E+00_JPRB, 0.68909E+00_JPRB, 0.70818E+00_JPRB, 0.72910E+00_JPRB ,&
&  0.74848E+00_JPRB, 0.74266E+00_JPRB, 0.72300E+00_JPRB, 0.71839E+00_JPRB, 0.83149E+00_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.69426E+00_JPRB, 0.66321E+00_JPRB, 0.62090E+00_JPRB, 0.59461E+00_JPRB, 0.57650E+00_JPRB, 0.55485E+00_JPRB, 0.53916E+00_JPRB ,&
&  0.52701E+00_JPRB, 0.51738E+00_JPRB, 0.50622E+00_JPRB, 0.49264E+00_JPRB, 0.47966E+00_JPRB, 0.47076E+00_JPRB, 0.46284E+00_JPRB ,&
&  0.45506E+00_JPRB, 0.44760E+00_JPRB, 0.44042E+00_JPRB, 0.43497E+00_JPRB, 0.42856E+00_JPRB, 0.42347E+00_JPRB, 0.41888E+00_JPRB ,&
&  0.41333E+00_JPRB, 0.40774E+00_JPRB, 0.40087E+00_JPRB, 0.40463E+00_JPRB, 0.41334E+00_JPRB, 0.42115E+00_JPRB, 0.43150E+00_JPRB ,&
&  0.46685E+00_JPRB, 0.46688E+00_JPRB, 0.51130E+00_JPRB, 0.56072E+00_JPRB, 0.59428E+00_JPRB, 0.62388E+00_JPRB, 0.63303E+00_JPRB ,&
&  0.62179E+00_JPRB, 0.63165E+00_JPRB, 0.62406E+00_JPRB, 0.60215E+00_JPRB, 0.58498E+00_JPRB, 0.55064E+00_JPRB, 0.52012E+00_JPRB ,&
&  0.49896E+00_JPRB, 0.48546E+00_JPRB, 0.47328E+00_JPRB, 0.45980E+00_JPRB, 0.45162E+00_JPRB, 0.45014E+00_JPRB, 0.45320E+00_JPRB ,&
&  0.45635E+00_JPRB, 0.46146E+00_JPRB, 0.46690E+00_JPRB, 0.47416E+00_JPRB, 0.49260E+00_JPRB, 0.58041E+00_JPRB, 0.80676E+00_JPRB ,&
&  0.97462E+00_JPRB, 0.10747E+01_JPRB, 0.11410E+01_JPRB, 0.11693E+01_JPRB, 0.11355E+01_JPRB, 0.10699E+01_JPRB, 0.99109E+00_JPRB ,&
&  0.90860E+00_JPRB, 0.82220E+00_JPRB, 0.74382E+00_JPRB, 0.68332E+00_JPRB, 0.63678E+00_JPRB, 0.60647E+00_JPRB, 0.59169E+00_JPRB ,&
&  0.59156E+00_JPRB, 0.60174E+00_JPRB, 0.61625E+00_JPRB, 0.62994E+00_JPRB, 0.63732E+00_JPRB, 0.63700E+00_JPRB, 0.63273E+00_JPRB ,&
&  0.62512E+00_JPRB, 0.61538E+00_JPRB, 0.60540E+00_JPRB, 0.59769E+00_JPRB, 0.59229E+00_JPRB, 0.58900E+00_JPRB, 0.58798E+00_JPRB ,&
&  0.58888E+00_JPRB, 0.59260E+00_JPRB, 0.60094E+00_JPRB, 0.62175E+00_JPRB, 0.63076E+00_JPRB, 0.64680E+00_JPRB, 0.65873E+00_JPRB ,&
&  0.67283E+00_JPRB, 0.67742E+00_JPRB, 0.66930E+00_JPRB, 0.64722E+00_JPRB, 0.66149E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.60291E+00_JPRB, 0.59510E+00_JPRB, 0.57443E+00_JPRB, 0.55146E+00_JPRB, 0.53161E+00_JPRB, 0.51339E+00_JPRB, 0.49833E+00_JPRB ,&
&  0.48693E+00_JPRB, 0.47720E+00_JPRB, 0.46684E+00_JPRB, 0.45718E+00_JPRB, 0.44737E+00_JPRB, 0.43945E+00_JPRB, 0.43142E+00_JPRB ,&
&  0.42597E+00_JPRB, 0.42224E+00_JPRB, 0.41860E+00_JPRB, 0.41393E+00_JPRB, 0.40886E+00_JPRB, 0.40365E+00_JPRB, 0.39758E+00_JPRB ,&
&  0.39262E+00_JPRB, 0.38993E+00_JPRB, 0.38426E+00_JPRB, 0.38256E+00_JPRB, 0.38891E+00_JPRB, 0.39723E+00_JPRB, 0.40663E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.40103E+00_JPRB, 0.41365E+00_JPRB, 0.46557E+00_JPRB, 0.50179E+00_JPRB, 0.54573E+00_JPRB, 0.57112E+00_JPRB ,&
&  0.58012E+00_JPRB, 0.58868E+00_JPRB, 0.57824E+00_JPRB, 0.56075E+00_JPRB, 0.53169E+00_JPRB, 0.50598E+00_JPRB, 0.48599E+00_JPRB ,&
&  0.46876E+00_JPRB, 0.45413E+00_JPRB, 0.44235E+00_JPRB, 0.43254E+00_JPRB, 0.42385E+00_JPRB, 0.42020E+00_JPRB, 0.42120E+00_JPRB ,&
&  0.42319E+00_JPRB, 0.42590E+00_JPRB, 0.42994E+00_JPRB, 0.43723E+00_JPRB, 0.44674E+00_JPRB, 0.46708E+00_JPRB, 0.54237E+00_JPRB ,&
&  0.66269E+00_JPRB, 0.75004E+00_JPRB, 0.82469E+00_JPRB, 0.86665E+00_JPRB, 0.86356E+00_JPRB, 0.82380E+00_JPRB, 0.75155E+00_JPRB ,&
&  0.67540E+00_JPRB, 0.61701E+00_JPRB, 0.58149E+00_JPRB, 0.56040E+00_JPRB, 0.54938E+00_JPRB, 0.54151E+00_JPRB, 0.53536E+00_JPRB ,&
&  0.52985E+00_JPRB, 0.52700E+00_JPRB, 0.52937E+00_JPRB, 0.53665E+00_JPRB, 0.54699E+00_JPRB, 0.55539E+00_JPRB, 0.55859E+00_JPRB ,&
&  0.55754E+00_JPRB, 0.55414E+00_JPRB, 0.55145E+00_JPRB, 0.54852E+00_JPRB, 0.54494E+00_JPRB, 0.54000E+00_JPRB, 0.53775E+00_JPRB ,&
&  0.53921E+00_JPRB, 0.54410E+00_JPRB, 0.55311E+00_JPRB, 0.55625E+00_JPRB, 0.57822E+00_JPRB, 0.58787E+00_JPRB, 0.59453E+00_JPRB ,&
&  0.60058E+00_JPRB, 0.59986E+00_JPRB, 0.59855E+00_JPRB, 0.52857E+00_JPRB, 0.58659E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.51999E+00_JPRB, 0.51864E+00_JPRB, 0.50555E+00_JPRB, 0.49098E+00_JPRB, 0.47625E+00_JPRB, 0.46217E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.44466E+00_JPRB, 0.43511E+00_JPRB, 0.42679E+00_JPRB, 0.42134E+00_JPRB, 0.41540E+00_JPRB, 0.41004E+00_JPRB, 0.40378E+00_JPRB ,&
&  0.39889E+00_JPRB, 0.39593E+00_JPRB, 0.39383E+00_JPRB, 0.39116E+00_JPRB, 0.38792E+00_JPRB, 0.38379E+00_JPRB, 0.37958E+00_JPRB ,&
&  0.37548E+00_JPRB, 0.37263E+00_JPRB, 0.36890E+00_JPRB, 0.36678E+00_JPRB, 0.36933E+00_JPRB, 0.37540E+00_JPRB, 0.38123E+00_JPRB ,&
&  0.40300E+00_JPRB, 0.35490E+00_JPRB, 0.36840E+00_JPRB, 0.39406E+00_JPRB, 0.42260E+00_JPRB, 0.46078E+00_JPRB, 0.49268E+00_JPRB ,&
&  0.50897E+00_JPRB, 0.52528E+00_JPRB, 0.52075E+00_JPRB, 0.50950E+00_JPRB, 0.48887E+00_JPRB, 0.46744E+00_JPRB, 0.44973E+00_JPRB ,&
&  0.43826E+00_JPRB, 0.43007E+00_JPRB, 0.42028E+00_JPRB, 0.40928E+00_JPRB, 0.39889E+00_JPRB, 0.39162E+00_JPRB, 0.38866E+00_JPRB ,&
&  0.38922E+00_JPRB, 0.39025E+00_JPRB, 0.39041E+00_JPRB, 0.39316E+00_JPRB, 0.39958E+00_JPRB, 0.40917E+00_JPRB, 0.42998E+00_JPRB ,&
&  0.47506E+00_JPRB, 0.53584E+00_JPRB, 0.58779E+00_JPRB, 0.63603E+00_JPRB, 0.66596E+00_JPRB, 0.67352E+00_JPRB, 0.64210E+00_JPRB ,&
&  0.59892E+00_JPRB, 0.55942E+00_JPRB, 0.53376E+00_JPRB, 0.52034E+00_JPRB, 0.51211E+00_JPRB, 0.50633E+00_JPRB, 0.49806E+00_JPRB ,&
&  0.48691E+00_JPRB, 0.47387E+00_JPRB, 0.46231E+00_JPRB, 0.45170E+00_JPRB, 0.44445E+00_JPRB, 0.44277E+00_JPRB, 0.44526E+00_JPRB ,&
&  0.44895E+00_JPRB, 0.45186E+00_JPRB, 0.45460E+00_JPRB, 0.45823E+00_JPRB, 0.46171E+00_JPRB, 0.46536E+00_JPRB, 0.46823E+00_JPRB ,&
&  0.47295E+00_JPRB, 0.47907E+00_JPRB, 0.48590E+00_JPRB, 0.49939E+00_JPRB, 0.51023E+00_JPRB, 0.51751E+00_JPRB, 0.52303E+00_JPRB ,&
&  0.52506E+00_JPRB, 0.53108E+00_JPRB, 0.45722E+00_JPRB, 0.49237E+00_JPRB, 0.51064E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.45918E+00_JPRB, 0.45826E+00_JPRB, 0.45370E+00_JPRB, 0.44299E+00_JPRB, 0.43079E+00_JPRB, 0.42036E+00_JPRB, 0.41391E+00_JPRB ,&
&  0.40998E+00_JPRB, 0.40426E+00_JPRB, 0.39937E+00_JPRB, 0.39612E+00_JPRB, 0.39198E+00_JPRB, 0.38732E+00_JPRB, 0.38066E+00_JPRB ,&
&  0.37518E+00_JPRB, 0.37094E+00_JPRB, 0.36777E+00_JPRB, 0.36589E+00_JPRB, 0.36480E+00_JPRB, 0.36355E+00_JPRB, 0.36150E+00_JPRB ,&
&  0.35931E+00_JPRB, 0.35790E+00_JPRB, 0.35668E+00_JPRB, 0.35643E+00_JPRB, 0.35694E+00_JPRB, 0.36049E+00_JPRB, 0.36635E+00_JPRB ,&
&  0.35344E+00_JPRB, 0.32812E+00_JPRB, 0.34023E+00_JPRB, 0.35534E+00_JPRB, 0.37307E+00_JPRB, 0.40185E+00_JPRB, 0.43240E+00_JPRB ,&
&  0.45009E+00_JPRB, 0.46105E+00_JPRB, 0.45496E+00_JPRB, 0.44460E+00_JPRB, 0.43314E+00_JPRB, 0.42151E+00_JPRB, 0.41223E+00_JPRB ,&
&  0.40325E+00_JPRB, 0.39705E+00_JPRB, 0.39162E+00_JPRB, 0.38369E+00_JPRB, 0.37405E+00_JPRB, 0.36612E+00_JPRB, 0.36136E+00_JPRB ,&
&  0.35968E+00_JPRB, 0.36033E+00_JPRB, 0.36142E+00_JPRB, 0.36284E+00_JPRB, 0.36576E+00_JPRB, 0.37155E+00_JPRB, 0.38026E+00_JPRB ,&
&  0.39128E+00_JPRB, 0.41201E+00_JPRB, 0.44136E+00_JPRB, 0.46968E+00_JPRB, 0.49651E+00_JPRB, 0.51502E+00_JPRB, 0.52549E+00_JPRB ,&
&  0.52485E+00_JPRB, 0.51335E+00_JPRB, 0.49778E+00_JPRB, 0.48156E+00_JPRB, 0.46259E+00_JPRB, 0.44006E+00_JPRB, 0.41637E+00_JPRB ,&
&  0.39672E+00_JPRB, 0.38180E+00_JPRB, 0.37207E+00_JPRB, 0.36785E+00_JPRB, 0.36669E+00_JPRB, 0.36877E+00_JPRB, 0.37262E+00_JPRB ,&
&  0.37739E+00_JPRB, 0.38255E+00_JPRB, 0.38648E+00_JPRB, 0.39137E+00_JPRB, 0.39614E+00_JPRB, 0.40158E+00_JPRB, 0.40743E+00_JPRB ,&
&  0.41405E+00_JPRB, 0.42039E+00_JPRB, 0.42724E+00_JPRB, 0.43849E+00_JPRB, 0.44797E+00_JPRB, 0.45668E+00_JPRB, 0.46196E+00_JPRB ,&
&  0.46120E+00_JPRB, 0.46088E+00_JPRB, 0.46012E+00_JPRB, 0.45784E+00_JPRB, 0.46028E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.41484E+00_JPRB, 0.41266E+00_JPRB, 0.41063E+00_JPRB, 0.40778E+00_JPRB, 0.40318E+00_JPRB, 0.39647E+00_JPRB, 0.39124E+00_JPRB ,&
&  0.38788E+00_JPRB, 0.38415E+00_JPRB, 0.38104E+00_JPRB, 0.37716E+00_JPRB, 0.37247E+00_JPRB, 0.36673E+00_JPRB, 0.35906E+00_JPRB ,&
&  0.35204E+00_JPRB, 0.34556E+00_JPRB, 0.34140E+00_JPRB, 0.33952E+00_JPRB, 0.33966E+00_JPRB, 0.33989E+00_JPRB, 0.33847E+00_JPRB ,&
&  0.33761E+00_JPRB, 0.33759E+00_JPRB, 0.33747E+00_JPRB, 0.33753E+00_JPRB, 0.33795E+00_JPRB, 0.34104E+00_JPRB, 0.34464E+00_JPRB ,&
&  0.34934E+00_JPRB, 0.32578E+00_JPRB, 0.33594E+00_JPRB, 0.33962E+00_JPRB, 0.34977E+00_JPRB, 0.36684E+00_JPRB, 0.38669E+00_JPRB ,&
&  0.40164E+00_JPRB, 0.40248E+00_JPRB, 0.39694E+00_JPRB, 0.38848E+00_JPRB, 0.38232E+00_JPRB, 0.37660E+00_JPRB, 0.37038E+00_JPRB ,&
&  0.36435E+00_JPRB, 0.36032E+00_JPRB, 0.35677E+00_JPRB, 0.35321E+00_JPRB, 0.34812E+00_JPRB, 0.34160E+00_JPRB, 0.33520E+00_JPRB ,&
&  0.33060E+00_JPRB, 0.32888E+00_JPRB, 0.32957E+00_JPRB, 0.33139E+00_JPRB, 0.33378E+00_JPRB, 0.33793E+00_JPRB, 0.34320E+00_JPRB ,&
&  0.34906E+00_JPRB, 0.35279E+00_JPRB, 0.35763E+00_JPRB, 0.36701E+00_JPRB, 0.37755E+00_JPRB, 0.38578E+00_JPRB, 0.39077E+00_JPRB ,&
&  0.39280E+00_JPRB, 0.39019E+00_JPRB, 0.38455E+00_JPRB, 0.37654E+00_JPRB, 0.36748E+00_JPRB, 0.35821E+00_JPRB, 0.35013E+00_JPRB ,&
&  0.34358E+00_JPRB, 0.33781E+00_JPRB, 0.33271E+00_JPRB, 0.32911E+00_JPRB, 0.32762E+00_JPRB, 0.32763E+00_JPRB, 0.32895E+00_JPRB ,&
&  0.33238E+00_JPRB, 0.33664E+00_JPRB, 0.34024E+00_JPRB, 0.34389E+00_JPRB, 0.34734E+00_JPRB, 0.35051E+00_JPRB, 0.35486E+00_JPRB ,&
&  0.36084E+00_JPRB, 0.36554E+00_JPRB, 0.36985E+00_JPRB, 0.37592E+00_JPRB, 0.38332E+00_JPRB, 0.39146E+00_JPRB, 0.39911E+00_JPRB ,&
&  0.40438E+00_JPRB, 0.40710E+00_JPRB, 0.41039E+00_JPRB, 0.41216E+00_JPRB, 0.41543E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.38617E+00_JPRB, 0.38471E+00_JPRB, 0.38213E+00_JPRB, 0.38053E+00_JPRB, 0.37823E+00_JPRB, 0.37486E+00_JPRB, 0.37105E+00_JPRB ,&
&  0.36796E+00_JPRB, 0.36505E+00_JPRB, 0.36205E+00_JPRB, 0.35788E+00_JPRB, 0.35213E+00_JPRB, 0.34652E+00_JPRB, 0.33986E+00_JPRB ,&
&  0.33353E+00_JPRB, 0.32801E+00_JPRB, 0.32427E+00_JPRB, 0.32218E+00_JPRB, 0.32042E+00_JPRB, 0.31903E+00_JPRB, 0.31732E+00_JPRB ,&
&  0.31640E+00_JPRB, 0.31618E+00_JPRB, 0.31626E+00_JPRB, 0.31678E+00_JPRB, 0.31750E+00_JPRB, 0.32017E+00_JPRB, 0.32193E+00_JPRB ,&
&  0.32411E+00_JPRB, 0.32181E+00_JPRB, 0.32863E+00_JPRB, 0.33315E+00_JPRB, 0.34030E+00_JPRB, 0.34629E+00_JPRB, 0.35134E+00_JPRB ,&
&  0.35561E+00_JPRB, 0.35777E+00_JPRB, 0.35417E+00_JPRB, 0.34884E+00_JPRB, 0.34480E+00_JPRB, 0.34253E+00_JPRB, 0.33780E+00_JPRB ,&
&  0.33292E+00_JPRB, 0.32939E+00_JPRB, 0.32658E+00_JPRB, 0.32460E+00_JPRB, 0.32251E+00_JPRB, 0.31940E+00_JPRB, 0.31595E+00_JPRB ,&
&  0.31155E+00_JPRB, 0.30829E+00_JPRB, 0.30571E+00_JPRB, 0.30472E+00_JPRB, 0.30495E+00_JPRB, 0.30616E+00_JPRB, 0.30805E+00_JPRB ,&
&  0.31110E+00_JPRB, 0.31421E+00_JPRB, 0.31590E+00_JPRB, 0.31694E+00_JPRB, 0.31823E+00_JPRB, 0.32040E+00_JPRB, 0.32205E+00_JPRB ,&
&  0.32223E+00_JPRB, 0.32127E+00_JPRB, 0.31934E+00_JPRB, 0.31654E+00_JPRB, 0.31315E+00_JPRB, 0.31041E+00_JPRB, 0.30808E+00_JPRB ,&
&  0.30576E+00_JPRB, 0.30321E+00_JPRB, 0.30058E+00_JPRB, 0.29913E+00_JPRB, 0.29828E+00_JPRB, 0.29796E+00_JPRB, 0.29896E+00_JPRB ,&
&  0.30131E+00_JPRB, 0.30418E+00_JPRB, 0.30740E+00_JPRB, 0.31041E+00_JPRB, 0.31326E+00_JPRB, 0.31573E+00_JPRB, 0.31889E+00_JPRB ,&
&  0.32407E+00_JPRB, 0.32978E+00_JPRB, 0.33546E+00_JPRB, 0.34091E+00_JPRB, 0.34710E+00_JPRB, 0.35456E+00_JPRB, 0.36248E+00_JPRB ,&
&  0.36798E+00_JPRB, 0.37169E+00_JPRB, 0.37599E+00_JPRB, 0.38069E+00_JPRB, 0.38484E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.34953E+00_JPRB, 0.35126E+00_JPRB, 0.35168E+00_JPRB, 0.35241E+00_JPRB, 0.35225E+00_JPRB, 0.35080E+00_JPRB, 0.34790E+00_JPRB ,&
&  0.34467E+00_JPRB, 0.34096E+00_JPRB, 0.33722E+00_JPRB, 0.33233E+00_JPRB, 0.32719E+00_JPRB, 0.32296E+00_JPRB, 0.31825E+00_JPRB ,&
&  0.31339E+00_JPRB, 0.30888E+00_JPRB, 0.30578E+00_JPRB, 0.30330E+00_JPRB, 0.30089E+00_JPRB, 0.29863E+00_JPRB, 0.29559E+00_JPRB ,&
&  0.29298E+00_JPRB, 0.29050E+00_JPRB, 0.29088E+00_JPRB, 0.29335E+00_JPRB, 0.29537E+00_JPRB, 0.29683E+00_JPRB, 0.29807E+00_JPRB ,&
&  0.29841E+00_JPRB, 0.30464E+00_JPRB, 0.29885E+00_JPRB, 0.30814E+00_JPRB, 0.31054E+00_JPRB, 0.31116E+00_JPRB, 0.31142E+00_JPRB ,&
&  0.31344E+00_JPRB, 0.31577E+00_JPRB, 0.31725E+00_JPRB, 0.31433E+00_JPRB, 0.31023E+00_JPRB, 0.30703E+00_JPRB, 0.30341E+00_JPRB ,&
&  0.29847E+00_JPRB, 0.29650E+00_JPRB, 0.29660E+00_JPRB, 0.29700E+00_JPRB, 0.29714E+00_JPRB, 0.29672E+00_JPRB, 0.29563E+00_JPRB ,&
&  0.29412E+00_JPRB, 0.29190E+00_JPRB, 0.28926E+00_JPRB, 0.28661E+00_JPRB, 0.28494E+00_JPRB, 0.28296E+00_JPRB, 0.28115E+00_JPRB ,&
&  0.28042E+00_JPRB, 0.28095E+00_JPRB, 0.28152E+00_JPRB, 0.28170E+00_JPRB, 0.28118E+00_JPRB, 0.28087E+00_JPRB, 0.28098E+00_JPRB ,&
&  0.28066E+00_JPRB, 0.28006E+00_JPRB, 0.27917E+00_JPRB, 0.27779E+00_JPRB, 0.27621E+00_JPRB, 0.27534E+00_JPRB, 0.27521E+00_JPRB ,&
&  0.27477E+00_JPRB, 0.27319E+00_JPRB, 0.27087E+00_JPRB, 0.27038E+00_JPRB, 0.27034E+00_JPRB, 0.27066E+00_JPRB, 0.27122E+00_JPRB ,&
&  0.27247E+00_JPRB, 0.27474E+00_JPRB, 0.27729E+00_JPRB, 0.27981E+00_JPRB, 0.28231E+00_JPRB, 0.28583E+00_JPRB, 0.29001E+00_JPRB ,&
&  0.29469E+00_JPRB, 0.29978E+00_JPRB, 0.30481E+00_JPRB, 0.30969E+00_JPRB, 0.31390E+00_JPRB, 0.31870E+00_JPRB, 0.32473E+00_JPRB ,&
&  0.32976E+00_JPRB, 0.33369E+00_JPRB, 0.33706E+00_JPRB, 0.34129E+00_JPRB, 0.34594E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.31877E+00_JPRB, 0.32346E+00_JPRB, 0.32614E+00_JPRB, 0.32742E+00_JPRB, 0.32777E+00_JPRB, 0.32634E+00_JPRB, 0.32392E+00_JPRB ,&
&  0.32020E+00_JPRB, 0.31651E+00_JPRB, 0.31246E+00_JPRB, 0.30784E+00_JPRB, 0.30366E+00_JPRB, 0.30015E+00_JPRB, 0.29709E+00_JPRB ,&
&  0.29399E+00_JPRB, 0.29114E+00_JPRB, 0.28880E+00_JPRB, 0.28641E+00_JPRB, 0.28360E+00_JPRB, 0.28003E+00_JPRB, 0.27534E+00_JPRB ,&
&  0.27258E+00_JPRB, 0.27043E+00_JPRB, 0.27138E+00_JPRB, 0.27420E+00_JPRB, 0.27641E+00_JPRB, 0.27776E+00_JPRB, 0.27879E+00_JPRB ,&
&  0.28024E+00_JPRB, 0.28160E+00_JPRB, 0.28280E+00_JPRB, 0.28529E+00_JPRB, 0.28762E+00_JPRB, 0.28734E+00_JPRB, 0.28571E+00_JPRB ,&
&  0.28431E+00_JPRB, 0.28461E+00_JPRB, 0.28537E+00_JPRB, 0.28570E+00_JPRB, 0.28499E+00_JPRB, 0.28277E+00_JPRB, 0.27916E+00_JPRB ,&
&  0.27285E+00_JPRB, 0.26817E+00_JPRB, 0.26713E+00_JPRB, 0.26823E+00_JPRB, 0.27009E+00_JPRB, 0.27153E+00_JPRB, 0.27242E+00_JPRB ,&
&  0.27272E+00_JPRB, 0.27217E+00_JPRB, 0.27109E+00_JPRB, 0.26980E+00_JPRB, 0.26806E+00_JPRB, 0.26604E+00_JPRB, 0.26486E+00_JPRB ,&
&  0.26353E+00_JPRB, 0.26238E+00_JPRB, 0.26164E+00_JPRB, 0.26124E+00_JPRB, 0.26015E+00_JPRB, 0.25858E+00_JPRB, 0.25770E+00_JPRB ,&
&  0.25650E+00_JPRB, 0.25585E+00_JPRB, 0.25486E+00_JPRB, 0.25390E+00_JPRB, 0.25263E+00_JPRB, 0.25139E+00_JPRB, 0.25103E+00_JPRB ,&
&  0.25114E+00_JPRB, 0.25003E+00_JPRB, 0.24868E+00_JPRB, 0.24861E+00_JPRB, 0.24920E+00_JPRB, 0.25013E+00_JPRB, 0.25089E+00_JPRB ,&
&  0.25163E+00_JPRB, 0.25259E+00_JPRB, 0.25465E+00_JPRB, 0.25698E+00_JPRB, 0.25964E+00_JPRB, 0.26286E+00_JPRB, 0.26654E+00_JPRB ,&
&  0.27091E+00_JPRB, 0.27601E+00_JPRB, 0.28175E+00_JPRB, 0.28709E+00_JPRB, 0.29166E+00_JPRB, 0.29470E+00_JPRB, 0.29850E+00_JPRB ,&
&  0.30298E+00_JPRB, 0.30597E+00_JPRB, 0.30837E+00_JPRB, 0.31119E+00_JPRB, 0.31426E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.28845E+00_JPRB, 0.29324E+00_JPRB, 0.29696E+00_JPRB, 0.29910E+00_JPRB, 0.29923E+00_JPRB, 0.29806E+00_JPRB, 0.29555E+00_JPRB ,&
&  0.29188E+00_JPRB, 0.28820E+00_JPRB, 0.28450E+00_JPRB, 0.28151E+00_JPRB, 0.27968E+00_JPRB, 0.27823E+00_JPRB, 0.27709E+00_JPRB ,&
&  0.27541E+00_JPRB, 0.27333E+00_JPRB, 0.27069E+00_JPRB, 0.26836E+00_JPRB, 0.26555E+00_JPRB, 0.26156E+00_JPRB, 0.25864E+00_JPRB ,&
&  0.25662E+00_JPRB, 0.25466E+00_JPRB, 0.25407E+00_JPRB, 0.25530E+00_JPRB, 0.25765E+00_JPRB, 0.25996E+00_JPRB, 0.26250E+00_JPRB ,&
&  0.26427E+00_JPRB, 0.26512E+00_JPRB, 0.26575E+00_JPRB, 0.26558E+00_JPRB, 0.26555E+00_JPRB, 0.26525E+00_JPRB, 0.26454E+00_JPRB ,&
&  0.26371E+00_JPRB, 0.26292E+00_JPRB, 0.26156E+00_JPRB, 0.26125E+00_JPRB, 0.26100E+00_JPRB, 0.25974E+00_JPRB, 0.25758E+00_JPRB ,&
&  0.25414E+00_JPRB, 0.24946E+00_JPRB, 0.24574E+00_JPRB, 0.24388E+00_JPRB, 0.24330E+00_JPRB, 0.24363E+00_JPRB, 0.24453E+00_JPRB ,&
&  0.24570E+00_JPRB, 0.24735E+00_JPRB, 0.24886E+00_JPRB, 0.24961E+00_JPRB, 0.24947E+00_JPRB, 0.24931E+00_JPRB, 0.24812E+00_JPRB ,&
&  0.24695E+00_JPRB, 0.24620E+00_JPRB, 0.24565E+00_JPRB, 0.24505E+00_JPRB, 0.24300E+00_JPRB, 0.24046E+00_JPRB, 0.23993E+00_JPRB ,&
&  0.23856E+00_JPRB, 0.23766E+00_JPRB, 0.23632E+00_JPRB, 0.23569E+00_JPRB, 0.23489E+00_JPRB, 0.23368E+00_JPRB, 0.23238E+00_JPRB ,&
&  0.23133E+00_JPRB, 0.23078E+00_JPRB, 0.23046E+00_JPRB, 0.22968E+00_JPRB, 0.23016E+00_JPRB, 0.23097E+00_JPRB, 0.23243E+00_JPRB ,&
&  0.23364E+00_JPRB, 0.23481E+00_JPRB, 0.23641E+00_JPRB, 0.23800E+00_JPRB, 0.24044E+00_JPRB, 0.24335E+00_JPRB, 0.24666E+00_JPRB ,&
&  0.24946E+00_JPRB, 0.25327E+00_JPRB, 0.25762E+00_JPRB, 0.26169E+00_JPRB, 0.26491E+00_JPRB, 0.26718E+00_JPRB, 0.27020E+00_JPRB ,&
&  0.27507E+00_JPRB, 0.27890E+00_JPRB, 0.28062E+00_JPRB, 0.28272E+00_JPRB, 0.28467E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.26643E+00_JPRB, 0.26795E+00_JPRB, 0.26957E+00_JPRB, 0.27145E+00_JPRB, 0.27274E+00_JPRB, 0.27322E+00_JPRB, 0.27189E+00_JPRB ,&
&  0.26936E+00_JPRB, 0.26650E+00_JPRB, 0.26365E+00_JPRB, 0.26124E+00_JPRB, 0.25988E+00_JPRB, 0.25914E+00_JPRB, 0.25875E+00_JPRB ,&
&  0.25844E+00_JPRB, 0.25808E+00_JPRB, 0.25628E+00_JPRB, 0.25355E+00_JPRB, 0.24995E+00_JPRB, 0.24686E+00_JPRB, 0.24601E+00_JPRB ,&
&  0.24449E+00_JPRB, 0.24340E+00_JPRB, 0.24265E+00_JPRB, 0.24186E+00_JPRB, 0.24156E+00_JPRB, 0.24252E+00_JPRB, 0.24429E+00_JPRB ,&
&  0.24619E+00_JPRB, 0.24771E+00_JPRB, 0.24890E+00_JPRB, 0.24976E+00_JPRB, 0.24924E+00_JPRB, 0.24814E+00_JPRB, 0.24683E+00_JPRB ,&
&  0.24523E+00_JPRB, 0.24368E+00_JPRB, 0.24230E+00_JPRB, 0.24048E+00_JPRB, 0.23785E+00_JPRB, 0.23471E+00_JPRB, 0.23245E+00_JPRB ,&
&  0.23071E+00_JPRB, 0.22911E+00_JPRB, 0.22745E+00_JPRB, 0.22660E+00_JPRB, 0.22600E+00_JPRB, 0.22577E+00_JPRB, 0.22587E+00_JPRB ,&
&  0.22631E+00_JPRB, 0.22757E+00_JPRB, 0.22930E+00_JPRB, 0.23118E+00_JPRB, 0.23173E+00_JPRB, 0.23191E+00_JPRB, 0.23217E+00_JPRB ,&
&  0.23155E+00_JPRB, 0.23028E+00_JPRB, 0.22855E+00_JPRB, 0.22664E+00_JPRB, 0.22542E+00_JPRB, 0.22451E+00_JPRB, 0.22348E+00_JPRB ,&
&  0.22162E+00_JPRB, 0.22045E+00_JPRB, 0.21979E+00_JPRB, 0.21964E+00_JPRB, 0.21982E+00_JPRB, 0.21985E+00_JPRB, 0.21916E+00_JPRB ,&
&  0.21831E+00_JPRB, 0.21758E+00_JPRB, 0.21666E+00_JPRB, 0.21576E+00_JPRB, 0.21422E+00_JPRB, 0.21473E+00_JPRB, 0.21574E+00_JPRB ,&
&  0.21616E+00_JPRB, 0.21632E+00_JPRB, 0.21656E+00_JPRB, 0.21777E+00_JPRB, 0.21914E+00_JPRB, 0.22096E+00_JPRB, 0.22351E+00_JPRB ,&
&  0.22564E+00_JPRB, 0.22843E+00_JPRB, 0.23248E+00_JPRB, 0.23624E+00_JPRB, 0.23898E+00_JPRB, 0.24030E+00_JPRB, 0.24292E+00_JPRB ,&
&  0.24862E+00_JPRB, 0.25465E+00_JPRB, 0.25838E+00_JPRB, 0.26201E+00_JPRB, 0.26521E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.24490E+00_JPRB, 0.24557E+00_JPRB, 0.24630E+00_JPRB, 0.24736E+00_JPRB, 0.24892E+00_JPRB, 0.24983E+00_JPRB, 0.25055E+00_JPRB ,&
&  0.24996E+00_JPRB, 0.24830E+00_JPRB, 0.24637E+00_JPRB, 0.24431E+00_JPRB, 0.24347E+00_JPRB, 0.24352E+00_JPRB, 0.24380E+00_JPRB ,&
&  0.24348E+00_JPRB, 0.24347E+00_JPRB, 0.24184E+00_JPRB, 0.23898E+00_JPRB, 0.23558E+00_JPRB, 0.23328E+00_JPRB, 0.23266E+00_JPRB ,&
&  0.23213E+00_JPRB, 0.23125E+00_JPRB, 0.23086E+00_JPRB, 0.23038E+00_JPRB, 0.22830E+00_JPRB, 0.22721E+00_JPRB, 0.22757E+00_JPRB ,&
&  0.22835E+00_JPRB, 0.22973E+00_JPRB, 0.22994E+00_JPRB, 0.22744E+00_JPRB, 0.22967E+00_JPRB, 0.22938E+00_JPRB, 0.22750E+00_JPRB ,&
&  0.22510E+00_JPRB, 0.22261E+00_JPRB, 0.22034E+00_JPRB, 0.21813E+00_JPRB, 0.21588E+00_JPRB, 0.21417E+00_JPRB, 0.21303E+00_JPRB ,&
&  0.21269E+00_JPRB, 0.21282E+00_JPRB, 0.21324E+00_JPRB, 0.21414E+00_JPRB, 0.21479E+00_JPRB, 0.21515E+00_JPRB, 0.21521E+00_JPRB ,&
&  0.21533E+00_JPRB, 0.21553E+00_JPRB, 0.21592E+00_JPRB, 0.21616E+00_JPRB, 0.21642E+00_JPRB, 0.21668E+00_JPRB, 0.21660E+00_JPRB ,&
&  0.21559E+00_JPRB, 0.21387E+00_JPRB, 0.21153E+00_JPRB, 0.21067E+00_JPRB, 0.21063E+00_JPRB, 0.20919E+00_JPRB, 0.20685E+00_JPRB ,&
&  0.20459E+00_JPRB, 0.20407E+00_JPRB, 0.20516E+00_JPRB, 0.20637E+00_JPRB, 0.20733E+00_JPRB, 0.20765E+00_JPRB, 0.20736E+00_JPRB ,&
&  0.20639E+00_JPRB, 0.20429E+00_JPRB, 0.20179E+00_JPRB, 0.19986E+00_JPRB, 0.19883E+00_JPRB, 0.19888E+00_JPRB, 0.19981E+00_JPRB ,&
&  0.19995E+00_JPRB, 0.20001E+00_JPRB, 0.20110E+00_JPRB, 0.20207E+00_JPRB, 0.20277E+00_JPRB, 0.20363E+00_JPRB, 0.20512E+00_JPRB ,&
&  0.20715E+00_JPRB, 0.21040E+00_JPRB, 0.21436E+00_JPRB, 0.21865E+00_JPRB, 0.22163E+00_JPRB, 0.22088E+00_JPRB, 0.22160E+00_JPRB ,&
&  0.22617E+00_JPRB, 0.23160E+00_JPRB, 0.23601E+00_JPRB, 0.23944E+00_JPRB, 0.24259E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.22644E+00_JPRB, 0.22744E+00_JPRB, 0.22835E+00_JPRB, 0.22919E+00_JPRB, 0.23079E+00_JPRB, 0.23205E+00_JPRB, 0.23361E+00_JPRB ,&
&  0.23474E+00_JPRB, 0.23349E+00_JPRB, 0.23186E+00_JPRB, 0.23056E+00_JPRB, 0.23010E+00_JPRB, 0.22955E+00_JPRB, 0.22916E+00_JPRB ,&
&  0.22999E+00_JPRB, 0.22944E+00_JPRB, 0.22693E+00_JPRB, 0.22510E+00_JPRB, 0.22197E+00_JPRB, 0.21991E+00_JPRB, 0.21952E+00_JPRB ,&
&  0.22109E+00_JPRB, 0.22082E+00_JPRB, 0.22020E+00_JPRB, 0.21919E+00_JPRB, 0.21782E+00_JPRB, 0.21711E+00_JPRB, 0.21598E+00_JPRB ,&
&  0.21533E+00_JPRB, 0.21434E+00_JPRB, 0.20965E+00_JPRB, 0.21022E+00_JPRB, 0.20980E+00_JPRB, 0.20971E+00_JPRB, 0.20913E+00_JPRB ,&
&  0.20847E+00_JPRB, 0.20746E+00_JPRB, 0.20568E+00_JPRB, 0.20373E+00_JPRB, 0.20208E+00_JPRB, 0.20117E+00_JPRB, 0.20104E+00_JPRB ,&
&  0.20138E+00_JPRB, 0.20231E+00_JPRB, 0.20391E+00_JPRB, 0.20597E+00_JPRB, 0.20686E+00_JPRB, 0.20751E+00_JPRB, 0.20848E+00_JPRB ,&
&  0.20888E+00_JPRB, 0.20894E+00_JPRB, 0.20893E+00_JPRB, 0.20886E+00_JPRB, 0.20856E+00_JPRB, 0.20775E+00_JPRB, 0.20711E+00_JPRB ,&
&  0.20554E+00_JPRB, 0.20320E+00_JPRB, 0.20161E+00_JPRB, 0.19964E+00_JPRB, 0.19423E+00_JPRB, 0.18406E+00_JPRB, 0.17838E+00_JPRB ,&
&  0.17851E+00_JPRB, 0.18315E+00_JPRB, 0.18886E+00_JPRB, 0.19203E+00_JPRB, 0.19710E+00_JPRB, 0.19729E+00_JPRB, 0.19374E+00_JPRB ,&
&  0.18567E+00_JPRB, 0.18038E+00_JPRB, 0.17664E+00_JPRB, 0.17258E+00_JPRB, 0.17012E+00_JPRB, 0.16968E+00_JPRB, 0.17015E+00_JPRB ,&
&  0.17216E+00_JPRB, 0.17501E+00_JPRB, 0.17786E+00_JPRB, 0.17946E+00_JPRB, 0.17962E+00_JPRB, 0.17834E+00_JPRB, 0.17682E+00_JPRB ,&
&  0.17648E+00_JPRB, 0.18039E+00_JPRB, 0.18852E+00_JPRB, 0.19752E+00_JPRB, 0.20176E+00_JPRB, 0.20347E+00_JPRB, 0.20521E+00_JPRB ,&
&  0.20877E+00_JPRB, 0.21355E+00_JPRB, 0.21833E+00_JPRB, 0.22164E+00_JPRB, 0.22434E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.20787E+00_JPRB, 0.20890E+00_JPRB, 0.21002E+00_JPRB, 0.21119E+00_JPRB, 0.21252E+00_JPRB, 0.21402E+00_JPRB, 0.21532E+00_JPRB ,&
&  0.21678E+00_JPRB, 0.21774E+00_JPRB, 0.21797E+00_JPRB, 0.21695E+00_JPRB, 0.21621E+00_JPRB, 0.21637E+00_JPRB, 0.21731E+00_JPRB ,&
&  0.21552E+00_JPRB, 0.21273E+00_JPRB, 0.21158E+00_JPRB, 0.21145E+00_JPRB, 0.21091E+00_JPRB, 0.21063E+00_JPRB, 0.21138E+00_JPRB ,&
&  0.21230E+00_JPRB, 0.21190E+00_JPRB, 0.21151E+00_JPRB, 0.21036E+00_JPRB, 0.20941E+00_JPRB, 0.20866E+00_JPRB, 0.20760E+00_JPRB ,&
&  0.20432E+00_JPRB, 0.19443E+00_JPRB, 0.18795E+00_JPRB, 0.19124E+00_JPRB, 0.19808E+00_JPRB, 0.19740E+00_JPRB, 0.19743E+00_JPRB ,&
&  0.19697E+00_JPRB, 0.19565E+00_JPRB, 0.19419E+00_JPRB, 0.19281E+00_JPRB, 0.19223E+00_JPRB, 0.19251E+00_JPRB, 0.19334E+00_JPRB ,&
&  0.19431E+00_JPRB, 0.19645E+00_JPRB, 0.19888E+00_JPRB, 0.19997E+00_JPRB, 0.20105E+00_JPRB, 0.20158E+00_JPRB, 0.20175E+00_JPRB ,&
&  0.20133E+00_JPRB, 0.20028E+00_JPRB, 0.19951E+00_JPRB, 0.19874E+00_JPRB, 0.19801E+00_JPRB, 0.19688E+00_JPRB, 0.19325E+00_JPRB ,&
&  0.18798E+00_JPRB, 0.18281E+00_JPRB, 0.17652E+00_JPRB, 0.17024E+00_JPRB, 0.16459E+00_JPRB, 0.16049E+00_JPRB, 0.15712E+00_JPRB ,&
&  0.15665E+00_JPRB, 0.16036E+00_JPRB, 0.16804E+00_JPRB, 0.17745E+00_JPRB, 0.18324E+00_JPRB, 0.17338E+00_JPRB, 0.16729E+00_JPRB ,&
&  0.16156E+00_JPRB, 0.15705E+00_JPRB, 0.15436E+00_JPRB, 0.15297E+00_JPRB, 0.15205E+00_JPRB, 0.15213E+00_JPRB, 0.15278E+00_JPRB ,&
&  0.15409E+00_JPRB, 0.15551E+00_JPRB, 0.15683E+00_JPRB, 0.15766E+00_JPRB, 0.15797E+00_JPRB, 0.15781E+00_JPRB, 0.15743E+00_JPRB ,&
&  0.15790E+00_JPRB, 0.16017E+00_JPRB, 0.16419E+00_JPRB, 0.16893E+00_JPRB, 0.17304E+00_JPRB, 0.17698E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18918E+00_JPRB, 0.19465E+00_JPRB, 0.19911E+00_JPRB, 0.20371E+00_JPRB, 0.20628E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.19406E+00_JPRB, 0.19589E+00_JPRB, 0.19651E+00_JPRB, 0.19770E+00_JPRB, 0.19806E+00_JPRB, 0.19931E+00_JPRB, 0.20029E+00_JPRB ,&
&  0.20265E+00_JPRB, 0.20482E+00_JPRB, 0.20530E+00_JPRB, 0.20350E+00_JPRB, 0.20241E+00_JPRB, 0.19966E+00_JPRB, 0.19861E+00_JPRB ,&
&  0.19838E+00_JPRB, 0.19943E+00_JPRB, 0.19963E+00_JPRB, 0.19973E+00_JPRB, 0.19971E+00_JPRB, 0.19968E+00_JPRB, 0.20017E+00_JPRB ,&
&  0.19817E+00_JPRB, 0.19735E+00_JPRB, 0.19938E+00_JPRB, 0.19851E+00_JPRB, 0.19779E+00_JPRB, 0.19642E+00_JPRB, 0.19567E+00_JPRB ,&
&  0.19166E+00_JPRB, 0.18595E+00_JPRB, 0.17936E+00_JPRB, 0.18231E+00_JPRB, 0.18942E+00_JPRB, 0.18713E+00_JPRB, 0.18641E+00_JPRB ,&
&  0.18587E+00_JPRB, 0.18473E+00_JPRB, 0.18431E+00_JPRB, 0.18387E+00_JPRB, 0.18450E+00_JPRB, 0.18515E+00_JPRB, 0.18685E+00_JPRB ,&
&  0.18923E+00_JPRB, 0.19257E+00_JPRB, 0.19246E+00_JPRB, 0.18625E+00_JPRB, 0.18315E+00_JPRB, 0.18203E+00_JPRB, 0.17873E+00_JPRB ,&
&  0.17486E+00_JPRB, 0.17339E+00_JPRB, 0.17234E+00_JPRB, 0.16985E+00_JPRB, 0.16908E+00_JPRB, 0.16861E+00_JPRB, 0.16622E+00_JPRB ,&
&  0.16309E+00_JPRB, 0.16076E+00_JPRB, 0.15746E+00_JPRB, 0.15408E+00_JPRB, 0.15175E+00_JPRB, 0.15067E+00_JPRB, 0.15076E+00_JPRB ,&
&  0.15255E+00_JPRB, 0.15441E+00_JPRB, 0.16014E+00_JPRB, 0.16385E+00_JPRB, 0.16086E+00_JPRB, 0.15762E+00_JPRB, 0.15413E+00_JPRB ,&
&  0.14894E+00_JPRB, 0.14652E+00_JPRB, 0.14464E+00_JPRB, 0.14329E+00_JPRB, 0.14321E+00_JPRB, 0.14399E+00_JPRB, 0.14457E+00_JPRB ,&
&  0.14532E+00_JPRB, 0.14633E+00_JPRB, 0.14681E+00_JPRB, 0.14704E+00_JPRB, 0.14710E+00_JPRB, 0.14736E+00_JPRB, 0.14799E+00_JPRB ,&
&  0.14879E+00_JPRB, 0.15064E+00_JPRB, 0.15313E+00_JPRB, 0.15574E+00_JPRB, 0.15768E+00_JPRB, 0.15926E+00_JPRB, 0.16252E+00_JPRB ,&
&  0.16626E+00_JPRB, 0.17492E+00_JPRB, 0.18716E+00_JPRB, 0.19618E+00_JPRB, 0.19275E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.19137E+00_JPRB, 0.19257E+00_JPRB, 0.19229E+00_JPRB, 0.19244E+00_JPRB, 0.19198E+00_JPRB, 0.19302E+00_JPRB, 0.19220E+00_JPRB ,&
&  0.19207E+00_JPRB, 0.19093E+00_JPRB, 0.18981E+00_JPRB, 0.18716E+00_JPRB, 0.18572E+00_JPRB, 0.17973E+00_JPRB, 0.17812E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17757E+00_JPRB, 0.17780E+00_JPRB, 0.17828E+00_JPRB, 0.17914E+00_JPRB, 0.18072E+00_JPRB, 0.18067E+00_JPRB ,&
&  0.17955E+00_JPRB, 0.17751E+00_JPRB, 0.17512E+00_JPRB, 0.17293E+00_JPRB, 0.17326E+00_JPRB, 0.17289E+00_JPRB, 0.17192E+00_JPRB ,&
&  0.17093E+00_JPRB, 0.17149E+00_JPRB, 0.17196E+00_JPRB, 0.17327E+00_JPRB, 0.17814E+00_JPRB, 0.17652E+00_JPRB, 0.17533E+00_JPRB ,&
&  0.17518E+00_JPRB, 0.17513E+00_JPRB, 0.17569E+00_JPRB, 0.17613E+00_JPRB, 0.17818E+00_JPRB, 0.17772E+00_JPRB, 0.18041E+00_JPRB ,&
&  0.17804E+00_JPRB, 0.17651E+00_JPRB, 0.17437E+00_JPRB, 0.17149E+00_JPRB, 0.16771E+00_JPRB, 0.16528E+00_JPRB, 0.16300E+00_JPRB ,&
&  0.16047E+00_JPRB, 0.15837E+00_JPRB, 0.15567E+00_JPRB, 0.15469E+00_JPRB, 0.15330E+00_JPRB, 0.15206E+00_JPRB, 0.15094E+00_JPRB ,&
&  0.15012E+00_JPRB, 0.14895E+00_JPRB, 0.14796E+00_JPRB, 0.14730E+00_JPRB, 0.14713E+00_JPRB, 0.14729E+00_JPRB, 0.14833E+00_JPRB ,&
&  0.15030E+00_JPRB, 0.15190E+00_JPRB, 0.15317E+00_JPRB, 0.15425E+00_JPRB, 0.15285E+00_JPRB, 0.15029E+00_JPRB, 0.14773E+00_JPRB ,&
&  0.14431E+00_JPRB, 0.14246E+00_JPRB, 0.14071E+00_JPRB, 0.13969E+00_JPRB, 0.13943E+00_JPRB, 0.13939E+00_JPRB, 0.13967E+00_JPRB ,&
&  0.13996E+00_JPRB, 0.14043E+00_JPRB, 0.14077E+00_JPRB, 0.14123E+00_JPRB, 0.14183E+00_JPRB, 0.14243E+00_JPRB, 0.14331E+00_JPRB ,&
&  0.14466E+00_JPRB, 0.14636E+00_JPRB, 0.14858E+00_JPRB, 0.15111E+00_JPRB, 0.15426E+00_JPRB, 0.15840E+00_JPRB, 0.16445E+00_JPRB ,&
&  0.17067E+00_JPRB, 0.18092E+00_JPRB, 0.18199E+00_JPRB, 0.18741E+00_JPRB, 0.19050E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.18902E+00_JPRB, 0.19081E+00_JPRB, 0.19150E+00_JPRB, 0.19242E+00_JPRB, 0.19211E+00_JPRB, 0.18977E+00_JPRB, 0.18736E+00_JPRB ,&
&  0.18199E+00_JPRB, 0.17765E+00_JPRB, 0.17239E+00_JPRB, 0.16749E+00_JPRB, 0.16286E+00_JPRB, 0.15989E+00_JPRB, 0.15816E+00_JPRB ,&
&  0.15815E+00_JPRB, 0.15956E+00_JPRB, 0.16060E+00_JPRB, 0.16120E+00_JPRB, 0.16159E+00_JPRB, 0.16256E+00_JPRB, 0.16265E+00_JPRB ,&
&  0.16221E+00_JPRB, 0.16113E+00_JPRB, 0.16033E+00_JPRB, 0.16116E+00_JPRB, 0.16311E+00_JPRB, 0.16514E+00_JPRB, 0.16682E+00_JPRB ,&
&  0.16830E+00_JPRB, 0.16762E+00_JPRB, 0.16760E+00_JPRB, 0.16680E+00_JPRB, 0.16627E+00_JPRB, 0.16562E+00_JPRB, 0.16543E+00_JPRB ,&
&  0.16621E+00_JPRB, 0.16684E+00_JPRB, 0.16862E+00_JPRB, 0.16958E+00_JPRB, 0.16843E+00_JPRB, 0.17035E+00_JPRB, 0.17057E+00_JPRB ,&
&  0.16982E+00_JPRB, 0.16682E+00_JPRB, 0.16357E+00_JPRB, 0.16103E+00_JPRB, 0.15836E+00_JPRB, 0.15618E+00_JPRB, 0.15438E+00_JPRB ,&
&  0.15273E+00_JPRB, 0.15183E+00_JPRB, 0.15001E+00_JPRB, 0.14847E+00_JPRB, 0.14751E+00_JPRB, 0.14649E+00_JPRB, 0.14586E+00_JPRB ,&
&  0.14545E+00_JPRB, 0.14475E+00_JPRB, 0.14462E+00_JPRB, 0.14426E+00_JPRB, 0.14438E+00_JPRB, 0.14454E+00_JPRB, 0.14489E+00_JPRB ,&
&  0.14566E+00_JPRB, 0.14659E+00_JPRB, 0.14650E+00_JPRB, 0.14636E+00_JPRB, 0.14493E+00_JPRB, 0.14334E+00_JPRB, 0.14197E+00_JPRB ,&
&  0.14020E+00_JPRB, 0.13891E+00_JPRB, 0.13811E+00_JPRB, 0.13734E+00_JPRB, 0.13723E+00_JPRB, 0.13728E+00_JPRB, 0.13726E+00_JPRB ,&
&  0.13749E+00_JPRB, 0.13782E+00_JPRB, 0.13831E+00_JPRB, 0.13928E+00_JPRB, 0.14015E+00_JPRB, 0.14138E+00_JPRB, 0.14273E+00_JPRB ,&
&  0.14382E+00_JPRB, 0.14575E+00_JPRB, 0.14776E+00_JPRB, 0.15007E+00_JPRB, 0.15473E+00_JPRB, 0.15884E+00_JPRB, 0.16622E+00_JPRB ,&
&  0.17201E+00_JPRB, 0.18126E+00_JPRB, 0.17935E+00_JPRB, 0.18337E+00_JPRB, 0.18702E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.18830E+00_JPRB, 0.18946E+00_JPRB, 0.18925E+00_JPRB, 0.18954E+00_JPRB, 0.18849E+00_JPRB, 0.18683E+00_JPRB, 0.18445E+00_JPRB ,&
&  0.18183E+00_JPRB, 0.17617E+00_JPRB, 0.17218E+00_JPRB, 0.16776E+00_JPRB, 0.16405E+00_JPRB, 0.16080E+00_JPRB, 0.15851E+00_JPRB ,&
&  0.15749E+00_JPRB, 0.15716E+00_JPRB, 0.15697E+00_JPRB, 0.15612E+00_JPRB, 0.15456E+00_JPRB, 0.15352E+00_JPRB, 0.15218E+00_JPRB ,&
&  0.15121E+00_JPRB, 0.15112E+00_JPRB, 0.15192E+00_JPRB, 0.15370E+00_JPRB, 0.15686E+00_JPRB, 0.15916E+00_JPRB, 0.16124E+00_JPRB ,&
&  0.16334E+00_JPRB, 0.15989E+00_JPRB, 0.15967E+00_JPRB, 0.15997E+00_JPRB, 0.15986E+00_JPRB, 0.15915E+00_JPRB, 0.15783E+00_JPRB ,&
&  0.16069E+00_JPRB, 0.16142E+00_JPRB, 0.16627E+00_JPRB, 0.16609E+00_JPRB, 0.16508E+00_JPRB, 0.16615E+00_JPRB, 0.16477E+00_JPRB ,&
&  0.16438E+00_JPRB, 0.16136E+00_JPRB, 0.15912E+00_JPRB, 0.15599E+00_JPRB, 0.15291E+00_JPRB, 0.15135E+00_JPRB, 0.14956E+00_JPRB ,&
&  0.14897E+00_JPRB, 0.14882E+00_JPRB, 0.14749E+00_JPRB, 0.14666E+00_JPRB, 0.14515E+00_JPRB, 0.14345E+00_JPRB, 0.14238E+00_JPRB ,&
&  0.14129E+00_JPRB, 0.14043E+00_JPRB, 0.13996E+00_JPRB, 0.13952E+00_JPRB, 0.13935E+00_JPRB, 0.13926E+00_JPRB, 0.13918E+00_JPRB ,&
&  0.13921E+00_JPRB, 0.13913E+00_JPRB, 0.13862E+00_JPRB, 0.13833E+00_JPRB, 0.13738E+00_JPRB, 0.13673E+00_JPRB, 0.13619E+00_JPRB ,&
&  0.13573E+00_JPRB, 0.13557E+00_JPRB, 0.13550E+00_JPRB, 0.13556E+00_JPRB, 0.13570E+00_JPRB, 0.13616E+00_JPRB, 0.13668E+00_JPRB ,&
&  0.13723E+00_JPRB, 0.13814E+00_JPRB, 0.13885E+00_JPRB, 0.13984E+00_JPRB, 0.14103E+00_JPRB, 0.14205E+00_JPRB, 0.14365E+00_JPRB ,&
&  0.14541E+00_JPRB, 0.14745E+00_JPRB, 0.15042E+00_JPRB, 0.15309E+00_JPRB, 0.15632E+00_JPRB, 0.16078E+00_JPRB, 0.16564E+00_JPRB ,&
&  0.17097E+00_JPRB, 0.17717E+00_JPRB, 0.17784E+00_JPRB, 0.18309E+00_JPRB, 0.18646E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.18133E+00_JPRB, 0.18200E+00_JPRB, 0.18262E+00_JPRB, 0.18286E+00_JPRB, 0.18292E+00_JPRB, 0.18220E+00_JPRB, 0.18127E+00_JPRB ,&
&  0.17752E+00_JPRB, 0.17478E+00_JPRB, 0.17174E+00_JPRB, 0.16872E+00_JPRB, 0.16607E+00_JPRB, 0.16338E+00_JPRB, 0.16096E+00_JPRB ,&
&  0.15877E+00_JPRB, 0.15713E+00_JPRB, 0.15561E+00_JPRB, 0.15354E+00_JPRB, 0.15091E+00_JPRB, 0.14851E+00_JPRB, 0.14697E+00_JPRB ,&
&  0.14651E+00_JPRB, 0.14679E+00_JPRB, 0.14814E+00_JPRB, 0.14956E+00_JPRB, 0.15107E+00_JPRB, 0.15220E+00_JPRB, 0.15335E+00_JPRB ,&
&  0.15277E+00_JPRB, 0.15239E+00_JPRB, 0.15205E+00_JPRB, 0.15145E+00_JPRB, 0.15202E+00_JPRB, 0.15279E+00_JPRB, 0.15325E+00_JPRB ,&
&  0.15380E+00_JPRB, 0.15521E+00_JPRB, 0.15622E+00_JPRB, 0.15716E+00_JPRB, 0.15707E+00_JPRB, 0.15634E+00_JPRB, 0.15494E+00_JPRB ,&
&  0.15319E+00_JPRB, 0.15133E+00_JPRB, 0.14967E+00_JPRB, 0.14826E+00_JPRB, 0.14717E+00_JPRB, 0.14596E+00_JPRB, 0.14493E+00_JPRB ,&
&  0.14442E+00_JPRB, 0.14395E+00_JPRB, 0.14329E+00_JPRB, 0.14262E+00_JPRB, 0.14160E+00_JPRB, 0.14070E+00_JPRB, 0.13978E+00_JPRB ,&
&  0.13893E+00_JPRB, 0.13814E+00_JPRB, 0.13741E+00_JPRB, 0.13666E+00_JPRB, 0.13614E+00_JPRB, 0.13568E+00_JPRB, 0.13523E+00_JPRB ,&
&  0.13476E+00_JPRB, 0.13459E+00_JPRB, 0.13425E+00_JPRB, 0.13414E+00_JPRB, 0.13394E+00_JPRB, 0.13381E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13449E+00_JPRB, 0.13500E+00_JPRB, 0.13527E+00_JPRB, 0.13561E+00_JPRB, 0.13595E+00_JPRB, 0.13639E+00_JPRB, 0.13709E+00_JPRB ,&
&  0.13764E+00_JPRB, 0.13831E+00_JPRB, 0.13887E+00_JPRB, 0.13953E+00_JPRB, 0.14075E+00_JPRB, 0.14201E+00_JPRB, 0.14345E+00_JPRB ,&
&  0.14524E+00_JPRB, 0.14711E+00_JPRB, 0.14928E+00_JPRB, 0.15138E+00_JPRB, 0.15420E+00_JPRB, 0.15593E+00_JPRB, 0.15917E+00_JPRB ,&
&  0.16275E+00_JPRB, 0.16407E+00_JPRB, 0.17605E+00_JPRB, 0.17721E+00_JPRB, 0.17884E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.15580E+00_JPRB, 0.15780E+00_JPRB, 0.15920E+00_JPRB, 0.16089E+00_JPRB, 0.16000E+00_JPRB, 0.17171E+00_JPRB, 0.16972E+00_JPRB ,&
&  0.16858E+00_JPRB, 0.16386E+00_JPRB, 0.16351E+00_JPRB, 0.16188E+00_JPRB, 0.15988E+00_JPRB, 0.15766E+00_JPRB, 0.15562E+00_JPRB ,&
&  0.15373E+00_JPRB, 0.15163E+00_JPRB, 0.15012E+00_JPRB, 0.14890E+00_JPRB, 0.14778E+00_JPRB, 0.14677E+00_JPRB, 0.14597E+00_JPRB ,&
&  0.14592E+00_JPRB, 0.14646E+00_JPRB, 0.14747E+00_JPRB, 0.14735E+00_JPRB, 0.14705E+00_JPRB, 0.14665E+00_JPRB, 0.14643E+00_JPRB ,&
&  0.14587E+00_JPRB, 0.14546E+00_JPRB, 0.14510E+00_JPRB, 0.14455E+00_JPRB, 0.14515E+00_JPRB, 0.14575E+00_JPRB, 0.14636E+00_JPRB ,&
&  0.14693E+00_JPRB, 0.14748E+00_JPRB, 0.14745E+00_JPRB, 0.14731E+00_JPRB, 0.14692E+00_JPRB, 0.14641E+00_JPRB, 0.14561E+00_JPRB ,&
&  0.14485E+00_JPRB, 0.14404E+00_JPRB, 0.14314E+00_JPRB, 0.14252E+00_JPRB, 0.14198E+00_JPRB, 0.14147E+00_JPRB, 0.14103E+00_JPRB ,&
&  0.14081E+00_JPRB, 0.14066E+00_JPRB, 0.14048E+00_JPRB, 0.14019E+00_JPRB, 0.13984E+00_JPRB, 0.13947E+00_JPRB, 0.13915E+00_JPRB ,&
&  0.13890E+00_JPRB, 0.13844E+00_JPRB, 0.13802E+00_JPRB, 0.13758E+00_JPRB, 0.13725E+00_JPRB, 0.13682E+00_JPRB, 0.13645E+00_JPRB ,&
&  0.13611E+00_JPRB, 0.13585E+00_JPRB, 0.13577E+00_JPRB, 0.13572E+00_JPRB, 0.13568E+00_JPRB, 0.13572E+00_JPRB, 0.13584E+00_JPRB ,&
&  0.13602E+00_JPRB, 0.13621E+00_JPRB, 0.13648E+00_JPRB, 0.13668E+00_JPRB, 0.13687E+00_JPRB, 0.13704E+00_JPRB, 0.13743E+00_JPRB ,&
&  0.13790E+00_JPRB, 0.13841E+00_JPRB, 0.13888E+00_JPRB, 0.13909E+00_JPRB, 0.13986E+00_JPRB, 0.14066E+00_JPRB, 0.14154E+00_JPRB ,&
&  0.14246E+00_JPRB, 0.14364E+00_JPRB, 0.14493E+00_JPRB, 0.14632E+00_JPRB, 0.14795E+00_JPRB, 0.14901E+00_JPRB, 0.15018E+00_JPRB ,&
&  0.15177E+00_JPRB, 0.15438E+00_JPRB, 0.15530E+00_JPRB, 0.15566E+00_JPRB, 0.15586E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.14529E+00_JPRB, 0.14511E+00_JPRB, 0.14490E+00_JPRB, 0.14466E+00_JPRB, 0.14391E+00_JPRB, 0.14356E+00_JPRB, 0.14325E+00_JPRB ,&
&  0.14308E+00_JPRB, 0.14297E+00_JPRB, 0.14304E+00_JPRB, 0.14317E+00_JPRB, 0.14330E+00_JPRB, 0.14342E+00_JPRB, 0.14352E+00_JPRB ,&
&  0.14348E+00_JPRB, 0.14347E+00_JPRB, 0.14283E+00_JPRB, 0.14232E+00_JPRB, 0.14201E+00_JPRB, 0.14178E+00_JPRB, 0.14146E+00_JPRB ,&
&  0.14121E+00_JPRB, 0.14090E+00_JPRB, 0.14054E+00_JPRB, 0.14028E+00_JPRB, 0.14029E+00_JPRB, 0.14015E+00_JPRB, 0.14015E+00_JPRB ,&
&  0.14013E+00_JPRB, 0.14028E+00_JPRB, 0.14034E+00_JPRB, 0.14041E+00_JPRB, 0.14074E+00_JPRB, 0.14103E+00_JPRB, 0.14109E+00_JPRB ,&
&  0.14111E+00_JPRB, 0.14093E+00_JPRB, 0.14075E+00_JPRB, 0.14062E+00_JPRB, 0.14056E+00_JPRB, 0.14054E+00_JPRB, 0.14051E+00_JPRB ,&
&  0.14039E+00_JPRB, 0.14021E+00_JPRB, 0.13998E+00_JPRB, 0.13971E+00_JPRB, 0.13947E+00_JPRB, 0.13936E+00_JPRB, 0.13933E+00_JPRB ,&
&  0.13932E+00_JPRB, 0.13920E+00_JPRB, 0.13901E+00_JPRB, 0.13871E+00_JPRB, 0.13843E+00_JPRB, 0.13815E+00_JPRB, 0.13789E+00_JPRB ,&
&  0.13765E+00_JPRB, 0.13753E+00_JPRB, 0.13740E+00_JPRB, 0.13730E+00_JPRB, 0.13723E+00_JPRB, 0.13712E+00_JPRB, 0.13702E+00_JPRB ,&
&  0.13693E+00_JPRB, 0.13685E+00_JPRB, 0.13677E+00_JPRB, 0.13670E+00_JPRB, 0.13665E+00_JPRB, 0.13662E+00_JPRB, 0.13662E+00_JPRB ,&
&  0.13664E+00_JPRB, 0.13666E+00_JPRB, 0.13669E+00_JPRB, 0.13683E+00_JPRB, 0.13699E+00_JPRB, 0.13719E+00_JPRB, 0.13742E+00_JPRB ,&
&  0.13767E+00_JPRB, 0.13796E+00_JPRB, 0.13826E+00_JPRB, 0.13848E+00_JPRB, 0.13882E+00_JPRB, 0.13916E+00_JPRB, 0.13953E+00_JPRB ,&
&  0.13988E+00_JPRB, 0.14032E+00_JPRB, 0.14080E+00_JPRB, 0.14135E+00_JPRB, 0.14195E+00_JPRB, 0.14229E+00_JPRB, 0.14270E+00_JPRB ,&
&  0.14319E+00_JPRB, 0.14383E+00_JPRB, 0.14419E+00_JPRB, 0.14445E+00_JPRB, 0.14474E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB, 0.13899E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB, 0.41090E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.41225E+00_JPRB, 0.41219E+00_JPRB, 0.41213E+00_JPRB, 0.41212E+00_JPRB, 0.41211E+00_JPRB, 0.41218E+00_JPRB, 0.41222E+00_JPRB ,&
&  0.41240E+00_JPRB, 0.41256E+00_JPRB, 0.41266E+00_JPRB, 0.41270E+00_JPRB, 0.41281E+00_JPRB, 0.41289E+00_JPRB, 0.41291E+00_JPRB ,&
&  0.41285E+00_JPRB, 0.41287E+00_JPRB, 0.41270E+00_JPRB, 0.41248E+00_JPRB, 0.41218E+00_JPRB, 0.41199E+00_JPRB, 0.41168E+00_JPRB ,&
&  0.41143E+00_JPRB, 0.41100E+00_JPRB, 0.41060E+00_JPRB, 0.40995E+00_JPRB, 0.40947E+00_JPRB, 0.40893E+00_JPRB, 0.40846E+00_JPRB ,&
&  0.40803E+00_JPRB, 0.40777E+00_JPRB, 0.40750E+00_JPRB, 0.40714E+00_JPRB, 0.40686E+00_JPRB, 0.40694E+00_JPRB, 0.40712E+00_JPRB ,&
&  0.40727E+00_JPRB, 0.40748E+00_JPRB, 0.40778E+00_JPRB, 0.40790E+00_JPRB, 0.40783E+00_JPRB, 0.40783E+00_JPRB, 0.40778E+00_JPRB ,&
&  0.40767E+00_JPRB, 0.40746E+00_JPRB, 0.40725E+00_JPRB, 0.40721E+00_JPRB, 0.40719E+00_JPRB, 0.40704E+00_JPRB, 0.40693E+00_JPRB ,&
&  0.40684E+00_JPRB, 0.40672E+00_JPRB, 0.40655E+00_JPRB, 0.40624E+00_JPRB, 0.40602E+00_JPRB, 0.40578E+00_JPRB, 0.40554E+00_JPRB ,&
&  0.40526E+00_JPRB, 0.40494E+00_JPRB, 0.40462E+00_JPRB, 0.40426E+00_JPRB, 0.40386E+00_JPRB, 0.40351E+00_JPRB, 0.40324E+00_JPRB ,&
&  0.40303E+00_JPRB, 0.40294E+00_JPRB, 0.40289E+00_JPRB, 0.40286E+00_JPRB, 0.40290E+00_JPRB, 0.40305E+00_JPRB, 0.40327E+00_JPRB ,&
&  0.40356E+00_JPRB, 0.40394E+00_JPRB, 0.40446E+00_JPRB, 0.40511E+00_JPRB, 0.40572E+00_JPRB, 0.40634E+00_JPRB, 0.40700E+00_JPRB ,&
&  0.40760E+00_JPRB, 0.40807E+00_JPRB, 0.40839E+00_JPRB, 0.40871E+00_JPRB, 0.40929E+00_JPRB, 0.40971E+00_JPRB, 0.41001E+00_JPRB ,&
&  0.41026E+00_JPRB, 0.41059E+00_JPRB, 0.41088E+00_JPRB, 0.41120E+00_JPRB, 0.41132E+00_JPRB, 0.41150E+00_JPRB, 0.41169E+00_JPRB ,&
&  0.41188E+00_JPRB, 0.41200E+00_JPRB, 0.41210E+00_JPRB, 0.41218E+00_JPRB, 0.41222E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.40830E+00_JPRB, 0.40830E+00_JPRB, 0.40850E+00_JPRB, 0.40885E+00_JPRB, 0.40930E+00_JPRB, 0.40973E+00_JPRB, 0.41022E+00_JPRB ,&
&  0.41082E+00_JPRB, 0.41125E+00_JPRB, 0.41127E+00_JPRB, 0.41159E+00_JPRB, 0.41223E+00_JPRB, 0.41297E+00_JPRB, 0.41294E+00_JPRB ,&
&  0.41299E+00_JPRB, 0.41301E+00_JPRB, 0.41321E+00_JPRB, 0.41290E+00_JPRB, 0.41244E+00_JPRB, 0.41174E+00_JPRB, 0.41132E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41021E+00_JPRB, 0.40932E+00_JPRB, 0.40771E+00_JPRB, 0.40629E+00_JPRB, 0.40481E+00_JPRB, 0.40331E+00_JPRB ,&
&  0.40135E+00_JPRB, 0.39967E+00_JPRB, 0.39833E+00_JPRB, 0.39717E+00_JPRB, 0.39873E+00_JPRB, 0.39976E+00_JPRB, 0.40036E+00_JPRB ,&
&  0.40045E+00_JPRB, 0.39876E+00_JPRB, 0.39820E+00_JPRB, 0.39784E+00_JPRB, 0.39758E+00_JPRB, 0.39892E+00_JPRB, 0.39981E+00_JPRB ,&
&  0.40054E+00_JPRB, 0.40117E+00_JPRB, 0.40283E+00_JPRB, 0.40366E+00_JPRB, 0.40429E+00_JPRB, 0.40468E+00_JPRB, 0.40461E+00_JPRB ,&
&  0.40500E+00_JPRB, 0.40540E+00_JPRB, 0.40577E+00_JPRB, 0.40572E+00_JPRB, 0.40561E+00_JPRB, 0.40544E+00_JPRB, 0.40501E+00_JPRB ,&
&  0.40427E+00_JPRB, 0.40372E+00_JPRB, 0.40304E+00_JPRB, 0.40225E+00_JPRB, 0.40132E+00_JPRB, 0.40071E+00_JPRB, 0.40018E+00_JPRB ,&
&  0.39969E+00_JPRB, 0.39937E+00_JPRB, 0.39915E+00_JPRB, 0.39895E+00_JPRB, 0.39881E+00_JPRB, 0.39892E+00_JPRB, 0.39933E+00_JPRB ,&
&  0.39996E+00_JPRB, 0.40069E+00_JPRB, 0.40151E+00_JPRB, 0.40251E+00_JPRB, 0.40341E+00_JPRB, 0.40427E+00_JPRB, 0.40551E+00_JPRB ,&
&  0.40697E+00_JPRB, 0.40807E+00_JPRB, 0.40890E+00_JPRB, 0.40959E+00_JPRB, 0.41056E+00_JPRB, 0.41104E+00_JPRB, 0.41117E+00_JPRB ,&
&  0.41115E+00_JPRB, 0.41153E+00_JPRB, 0.41190E+00_JPRB, 0.41230E+00_JPRB, 0.41229E+00_JPRB, 0.41196E+00_JPRB, 0.41143E+00_JPRB ,&
&  0.41084E+00_JPRB, 0.41034E+00_JPRB, 0.40973E+00_JPRB, 0.40910E+00_JPRB, 0.40860E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.40576E+00_JPRB, 0.40722E+00_JPRB, 0.40913E+00_JPRB, 0.41116E+00_JPRB, 0.41348E+00_JPRB, 0.41506E+00_JPRB, 0.41681E+00_JPRB ,&
&  0.41799E+00_JPRB, 0.41893E+00_JPRB, 0.41903E+00_JPRB, 0.41978E+00_JPRB, 0.42009E+00_JPRB, 0.42061E+00_JPRB, 0.42029E+00_JPRB ,&
&  0.42036E+00_JPRB, 0.41978E+00_JPRB, 0.42048E+00_JPRB, 0.42071E+00_JPRB, 0.42100E+00_JPRB, 0.42109E+00_JPRB, 0.41848E+00_JPRB ,&
&  0.41590E+00_JPRB, 0.41427E+00_JPRB, 0.41082E+00_JPRB, 0.40756E+00_JPRB, 0.40536E+00_JPRB, 0.38886E+00_JPRB, 0.38382E+00_JPRB ,&
&  0.38909E+00_JPRB, 0.39275E+00_JPRB, 0.39625E+00_JPRB, 0.39891E+00_JPRB, 0.39330E+00_JPRB, 0.38520E+00_JPRB, 0.38051E+00_JPRB ,&
&  0.36654E+00_JPRB, 0.36941E+00_JPRB, 0.37142E+00_JPRB, 0.37310E+00_JPRB, 0.37651E+00_JPRB, 0.37774E+00_JPRB, 0.38136E+00_JPRB ,&
&  0.39067E+00_JPRB, 0.39304E+00_JPRB, 0.39620E+00_JPRB, 0.39882E+00_JPRB, 0.40036E+00_JPRB, 0.40219E+00_JPRB, 0.40625E+00_JPRB ,&
&  0.40814E+00_JPRB, 0.40980E+00_JPRB, 0.41099E+00_JPRB, 0.41098E+00_JPRB, 0.41107E+00_JPRB, 0.41047E+00_JPRB, 0.40967E+00_JPRB ,&
&  0.40795E+00_JPRB, 0.40710E+00_JPRB, 0.40652E+00_JPRB, 0.40587E+00_JPRB, 0.40557E+00_JPRB, 0.40418E+00_JPRB, 0.40312E+00_JPRB ,&
&  0.40243E+00_JPRB, 0.40209E+00_JPRB, 0.40217E+00_JPRB, 0.40277E+00_JPRB, 0.40388E+00_JPRB, 0.40517E+00_JPRB, 0.40629E+00_JPRB ,&
&  0.40764E+00_JPRB, 0.40872E+00_JPRB, 0.41008E+00_JPRB, 0.41137E+00_JPRB, 0.41223E+00_JPRB, 0.41286E+00_JPRB, 0.41349E+00_JPRB ,&
&  0.41464E+00_JPRB, 0.41593E+00_JPRB, 0.41652E+00_JPRB, 0.41628E+00_JPRB, 0.41711E+00_JPRB, 0.41761E+00_JPRB, 0.41765E+00_JPRB ,&
&  0.41733E+00_JPRB, 0.41723E+00_JPRB, 0.41623E+00_JPRB, 0.41480E+00_JPRB, 0.41309E+00_JPRB, 0.41159E+00_JPRB, 0.41004E+00_JPRB ,&
&  0.40854E+00_JPRB, 0.40719E+00_JPRB, 0.40617E+00_JPRB, 0.40537E+00_JPRB, 0.40510E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.41066E+00_JPRB, 0.41405E+00_JPRB, 0.41830E+00_JPRB, 0.42083E+00_JPRB, 0.42404E+00_JPRB, 0.42529E+00_JPRB, 0.42722E+00_JPRB ,&
&  0.42781E+00_JPRB, 0.42879E+00_JPRB, 0.42941E+00_JPRB, 0.42975E+00_JPRB, 0.42970E+00_JPRB, 0.42934E+00_JPRB, 0.42905E+00_JPRB ,&
&  0.42918E+00_JPRB, 0.42889E+00_JPRB, 0.42965E+00_JPRB, 0.42998E+00_JPRB, 0.43035E+00_JPRB, 0.43029E+00_JPRB, 0.42735E+00_JPRB ,&
&  0.42361E+00_JPRB, 0.42030E+00_JPRB, 0.39936E+00_JPRB, 0.39674E+00_JPRB, 0.39271E+00_JPRB, 0.39058E+00_JPRB, 0.39143E+00_JPRB ,&
&  0.38952E+00_JPRB, 0.38689E+00_JPRB, 0.37779E+00_JPRB, 0.36823E+00_JPRB, 0.36034E+00_JPRB, 0.35154E+00_JPRB, 0.35015E+00_JPRB ,&
&  0.34871E+00_JPRB, 0.35099E+00_JPRB, 0.35345E+00_JPRB, 0.35535E+00_JPRB, 0.36081E+00_JPRB, 0.36448E+00_JPRB, 0.37098E+00_JPRB ,&
&  0.37907E+00_JPRB, 0.38483E+00_JPRB, 0.38559E+00_JPRB, 0.38451E+00_JPRB, 0.39480E+00_JPRB, 0.40153E+00_JPRB, 0.40758E+00_JPRB ,&
&  0.41131E+00_JPRB, 0.41217E+00_JPRB, 0.41433E+00_JPRB, 0.41467E+00_JPRB, 0.41526E+00_JPRB, 0.41696E+00_JPRB, 0.41729E+00_JPRB ,&
&  0.41843E+00_JPRB, 0.41821E+00_JPRB, 0.41728E+00_JPRB, 0.41580E+00_JPRB, 0.41308E+00_JPRB, 0.41132E+00_JPRB, 0.41007E+00_JPRB ,&
&  0.41000E+00_JPRB, 0.41102E+00_JPRB, 0.41246E+00_JPRB, 0.41476E+00_JPRB, 0.41724E+00_JPRB, 0.41889E+00_JPRB, 0.41966E+00_JPRB ,&
&  0.41989E+00_JPRB, 0.41967E+00_JPRB, 0.42140E+00_JPRB, 0.42058E+00_JPRB, 0.42089E+00_JPRB, 0.42398E+00_JPRB, 0.42278E+00_JPRB ,&
&  0.42304E+00_JPRB, 0.42476E+00_JPRB, 0.42581E+00_JPRB, 0.42673E+00_JPRB, 0.42743E+00_JPRB, 0.42764E+00_JPRB, 0.42810E+00_JPRB ,&
&  0.42790E+00_JPRB, 0.42701E+00_JPRB, 0.42508E+00_JPRB, 0.42260E+00_JPRB, 0.41983E+00_JPRB, 0.41705E+00_JPRB, 0.41354E+00_JPRB ,&
&  0.41045E+00_JPRB, 0.40769E+00_JPRB, 0.40633E+00_JPRB, 0.40600E+00_JPRB, 0.40776E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.42197E+00_JPRB, 0.42555E+00_JPRB, 0.43038E+00_JPRB, 0.43222E+00_JPRB, 0.43497E+00_JPRB, 0.43606E+00_JPRB, 0.43717E+00_JPRB ,&
&  0.43768E+00_JPRB, 0.43830E+00_JPRB, 0.43821E+00_JPRB, 0.43829E+00_JPRB, 0.43787E+00_JPRB, 0.43783E+00_JPRB, 0.43740E+00_JPRB ,&
&  0.43721E+00_JPRB, 0.43681E+00_JPRB, 0.43686E+00_JPRB, 0.43674E+00_JPRB, 0.43737E+00_JPRB, 0.43760E+00_JPRB, 0.43623E+00_JPRB ,&
&  0.43317E+00_JPRB, 0.42619E+00_JPRB, 0.41031E+00_JPRB, 0.40603E+00_JPRB, 0.40214E+00_JPRB, 0.40191E+00_JPRB, 0.40072E+00_JPRB ,&
&  0.39857E+00_JPRB, 0.36957E+00_JPRB, 0.37057E+00_JPRB, 0.36340E+00_JPRB, 0.35963E+00_JPRB, 0.35601E+00_JPRB, 0.34885E+00_JPRB ,&
&  0.34623E+00_JPRB, 0.34359E+00_JPRB, 0.34322E+00_JPRB, 0.34707E+00_JPRB, 0.35107E+00_JPRB, 0.35729E+00_JPRB, 0.36490E+00_JPRB ,&
&  0.36584E+00_JPRB, 0.37788E+00_JPRB, 0.38419E+00_JPRB, 0.38809E+00_JPRB, 0.39839E+00_JPRB, 0.40662E+00_JPRB, 0.40902E+00_JPRB ,&
&  0.41177E+00_JPRB, 0.41169E+00_JPRB, 0.41447E+00_JPRB, 0.41893E+00_JPRB, 0.42397E+00_JPRB, 0.42441E+00_JPRB, 0.42522E+00_JPRB ,&
&  0.42545E+00_JPRB, 0.42575E+00_JPRB, 0.42586E+00_JPRB, 0.42546E+00_JPRB, 0.42495E+00_JPRB, 0.42545E+00_JPRB, 0.42642E+00_JPRB ,&
&  0.42907E+00_JPRB, 0.43179E+00_JPRB, 0.43488E+00_JPRB, 0.43725E+00_JPRB, 0.43892E+00_JPRB, 0.44017E+00_JPRB, 0.44150E+00_JPRB ,&
&  0.44255E+00_JPRB, 0.44291E+00_JPRB, 0.44154E+00_JPRB, 0.44035E+00_JPRB, 0.43836E+00_JPRB, 0.43561E+00_JPRB, 0.43484E+00_JPRB ,&
&  0.43614E+00_JPRB, 0.43668E+00_JPRB, 0.43686E+00_JPRB, 0.43727E+00_JPRB, 0.43824E+00_JPRB, 0.43859E+00_JPRB, 0.43890E+00_JPRB ,&
&  0.43881E+00_JPRB, 0.43796E+00_JPRB, 0.43625E+00_JPRB, 0.43236E+00_JPRB, 0.42820E+00_JPRB, 0.42563E+00_JPRB, 0.42133E+00_JPRB ,&
&  0.41742E+00_JPRB, 0.41342E+00_JPRB, 0.41263E+00_JPRB, 0.41294E+00_JPRB, 0.41693E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.43602E+00_JPRB, 0.44138E+00_JPRB, 0.44679E+00_JPRB, 0.44949E+00_JPRB, 0.45218E+00_JPRB, 0.45259E+00_JPRB, 0.45344E+00_JPRB ,&
&  0.45378E+00_JPRB, 0.45451E+00_JPRB, 0.45456E+00_JPRB, 0.45453E+00_JPRB, 0.45392E+00_JPRB, 0.45318E+00_JPRB, 0.45185E+00_JPRB ,&
&  0.44987E+00_JPRB, 0.44796E+00_JPRB, 0.44543E+00_JPRB, 0.44359E+00_JPRB, 0.44255E+00_JPRB, 0.44348E+00_JPRB, 0.44289E+00_JPRB ,&
&  0.43991E+00_JPRB, 0.43340E+00_JPRB, 0.42119E+00_JPRB, 0.41457E+00_JPRB, 0.41283E+00_JPRB, 0.40624E+00_JPRB, 0.40569E+00_JPRB ,&
&  0.40371E+00_JPRB, 0.40337E+00_JPRB, 0.39954E+00_JPRB, 0.39739E+00_JPRB, 0.38923E+00_JPRB, 0.34897E+00_JPRB, 0.34468E+00_JPRB ,&
&  0.34152E+00_JPRB, 0.33667E+00_JPRB, 0.33604E+00_JPRB, 0.33728E+00_JPRB, 0.34132E+00_JPRB, 0.35407E+00_JPRB, 0.36567E+00_JPRB ,&
&  0.36608E+00_JPRB, 0.38040E+00_JPRB, 0.38551E+00_JPRB, 0.39065E+00_JPRB, 0.39997E+00_JPRB, 0.40851E+00_JPRB, 0.41130E+00_JPRB ,&
&  0.41513E+00_JPRB, 0.41923E+00_JPRB, 0.42587E+00_JPRB, 0.43266E+00_JPRB, 0.43743E+00_JPRB, 0.43974E+00_JPRB, 0.44263E+00_JPRB ,&
&  0.44515E+00_JPRB, 0.44822E+00_JPRB, 0.45196E+00_JPRB, 0.45449E+00_JPRB, 0.45474E+00_JPRB, 0.45333E+00_JPRB, 0.45270E+00_JPRB ,&
&  0.45463E+00_JPRB, 0.45555E+00_JPRB, 0.46023E+00_JPRB, 0.45858E+00_JPRB, 0.45873E+00_JPRB, 0.45832E+00_JPRB, 0.45981E+00_JPRB ,&
&  0.45955E+00_JPRB, 0.46397E+00_JPRB, 0.46697E+00_JPRB, 0.46852E+00_JPRB, 0.46280E+00_JPRB, 0.45732E+00_JPRB, 0.45365E+00_JPRB ,&
&  0.45211E+00_JPRB, 0.45198E+00_JPRB, 0.44834E+00_JPRB, 0.44744E+00_JPRB, 0.44743E+00_JPRB, 0.44788E+00_JPRB, 0.44866E+00_JPRB ,&
&  0.44882E+00_JPRB, 0.44720E+00_JPRB, 0.44410E+00_JPRB, 0.43910E+00_JPRB, 0.43447E+00_JPRB, 0.43039E+00_JPRB, 0.42328E+00_JPRB ,&
&  0.41857E+00_JPRB, 0.41536E+00_JPRB, 0.41666E+00_JPRB, 0.42015E+00_JPRB, 0.42748E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.44593E+00_JPRB, 0.45348E+00_JPRB, 0.46113E+00_JPRB, 0.46747E+00_JPRB, 0.47038E+00_JPRB, 0.47033E+00_JPRB, 0.46849E+00_JPRB ,&
&  0.46652E+00_JPRB, 0.46564E+00_JPRB, 0.46474E+00_JPRB, 0.46416E+00_JPRB, 0.46360E+00_JPRB, 0.46252E+00_JPRB, 0.46073E+00_JPRB ,&
&  0.45960E+00_JPRB, 0.44962E+00_JPRB, 0.44284E+00_JPRB, 0.44128E+00_JPRB, 0.44058E+00_JPRB, 0.44139E+00_JPRB, 0.44309E+00_JPRB ,&
&  0.44044E+00_JPRB, 0.43379E+00_JPRB, 0.42819E+00_JPRB, 0.42043E+00_JPRB, 0.41266E+00_JPRB, 0.41281E+00_JPRB, 0.41174E+00_JPRB ,&
&  0.40309E+00_JPRB, 0.39606E+00_JPRB, 0.39977E+00_JPRB, 0.40100E+00_JPRB, 0.38737E+00_JPRB, 0.37014E+00_JPRB, 0.35065E+00_JPRB ,&
&  0.34858E+00_JPRB, 0.34037E+00_JPRB, 0.33873E+00_JPRB, 0.33985E+00_JPRB, 0.34568E+00_JPRB, 0.35262E+00_JPRB, 0.36499E+00_JPRB ,&
&  0.36993E+00_JPRB, 0.37465E+00_JPRB, 0.38467E+00_JPRB, 0.39570E+00_JPRB, 0.40438E+00_JPRB, 0.41356E+00_JPRB, 0.42044E+00_JPRB ,&
&  0.42999E+00_JPRB, 0.44004E+00_JPRB, 0.44889E+00_JPRB, 0.45610E+00_JPRB, 0.46312E+00_JPRB, 0.46507E+00_JPRB, 0.46777E+00_JPRB ,&
&  0.48670E+00_JPRB, 0.50313E+00_JPRB, 0.51243E+00_JPRB, 0.51840E+00_JPRB, 0.51896E+00_JPRB, 0.51590E+00_JPRB, 0.50535E+00_JPRB ,&
&  0.49969E+00_JPRB, 0.50014E+00_JPRB, 0.49824E+00_JPRB, 0.49745E+00_JPRB, 0.49518E+00_JPRB, 0.49153E+00_JPRB, 0.48464E+00_JPRB ,&
&  0.47546E+00_JPRB, 0.47659E+00_JPRB, 0.51921E+00_JPRB, 0.52531E+00_JPRB, 0.51518E+00_JPRB, 0.49926E+00_JPRB, 0.48332E+00_JPRB ,&
&  0.47217E+00_JPRB, 0.46552E+00_JPRB, 0.46167E+00_JPRB, 0.45862E+00_JPRB, 0.45656E+00_JPRB, 0.45309E+00_JPRB, 0.45361E+00_JPRB ,&
&  0.45214E+00_JPRB, 0.44871E+00_JPRB, 0.44496E+00_JPRB, 0.44020E+00_JPRB, 0.43530E+00_JPRB, 0.43128E+00_JPRB, 0.42571E+00_JPRB ,&
&  0.42161E+00_JPRB, 0.41939E+00_JPRB, 0.42243E+00_JPRB, 0.42873E+00_JPRB, 0.43785E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.45321E+00_JPRB, 0.46180E+00_JPRB, 0.47241E+00_JPRB, 0.48007E+00_JPRB, 0.48176E+00_JPRB, 0.47166E+00_JPRB, 0.46265E+00_JPRB ,&
&  0.45691E+00_JPRB, 0.45482E+00_JPRB, 0.45442E+00_JPRB, 0.45346E+00_JPRB, 0.45323E+00_JPRB, 0.45298E+00_JPRB, 0.45267E+00_JPRB ,&
&  0.45259E+00_JPRB, 0.45109E+00_JPRB, 0.44725E+00_JPRB, 0.44032E+00_JPRB, 0.43582E+00_JPRB, 0.43497E+00_JPRB, 0.43613E+00_JPRB ,&
&  0.43631E+00_JPRB, 0.43433E+00_JPRB, 0.43255E+00_JPRB, 0.42503E+00_JPRB, 0.41670E+00_JPRB, 0.40716E+00_JPRB, 0.40074E+00_JPRB ,&
&  0.40060E+00_JPRB, 0.40195E+00_JPRB, 0.39790E+00_JPRB, 0.39864E+00_JPRB, 0.39980E+00_JPRB, 0.38343E+00_JPRB, 0.36385E+00_JPRB ,&
&  0.35299E+00_JPRB, 0.34571E+00_JPRB, 0.34241E+00_JPRB, 0.34336E+00_JPRB, 0.35622E+00_JPRB, 0.37528E+00_JPRB, 0.38228E+00_JPRB ,&
&  0.36889E+00_JPRB, 0.38016E+00_JPRB, 0.39495E+00_JPRB, 0.40486E+00_JPRB, 0.41491E+00_JPRB, 0.42823E+00_JPRB, 0.44179E+00_JPRB ,&
&  0.45359E+00_JPRB, 0.46353E+00_JPRB, 0.46939E+00_JPRB, 0.48082E+00_JPRB, 0.48626E+00_JPRB, 0.48002E+00_JPRB, 0.48694E+00_JPRB ,&
&  0.53989E+00_JPRB, 0.58658E+00_JPRB, 0.60209E+00_JPRB, 0.60097E+00_JPRB, 0.59395E+00_JPRB, 0.58165E+00_JPRB, 0.57247E+00_JPRB ,&
&  0.56388E+00_JPRB, 0.55448E+00_JPRB, 0.54556E+00_JPRB, 0.53705E+00_JPRB, 0.52925E+00_JPRB, 0.52013E+00_JPRB, 0.51380E+00_JPRB ,&
&  0.50804E+00_JPRB, 0.51719E+00_JPRB, 0.64919E+00_JPRB, 0.61887E+00_JPRB, 0.57890E+00_JPRB, 0.55281E+00_JPRB, 0.52872E+00_JPRB ,&
&  0.51146E+00_JPRB, 0.50010E+00_JPRB, 0.49315E+00_JPRB, 0.48489E+00_JPRB, 0.46814E+00_JPRB, 0.45561E+00_JPRB, 0.45009E+00_JPRB ,&
&  0.44725E+00_JPRB, 0.44180E+00_JPRB, 0.43703E+00_JPRB, 0.43253E+00_JPRB, 0.42919E+00_JPRB, 0.42788E+00_JPRB, 0.42533E+00_JPRB ,&
&  0.42389E+00_JPRB, 0.42708E+00_JPRB, 0.43254E+00_JPRB, 0.44091E+00_JPRB, 0.44749E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.45836E+00_JPRB, 0.47591E+00_JPRB, 0.48968E+00_JPRB, 0.49494E+00_JPRB, 0.49287E+00_JPRB, 0.46734E+00_JPRB, 0.44419E+00_JPRB ,&
&  0.43123E+00_JPRB, 0.42728E+00_JPRB, 0.42835E+00_JPRB, 0.42979E+00_JPRB, 0.43223E+00_JPRB, 0.43550E+00_JPRB, 0.43809E+00_JPRB ,&
&  0.43842E+00_JPRB, 0.43802E+00_JPRB, 0.43769E+00_JPRB, 0.43628E+00_JPRB, 0.43359E+00_JPRB, 0.43127E+00_JPRB, 0.43078E+00_JPRB ,&
&  0.43029E+00_JPRB, 0.42812E+00_JPRB, 0.42569E+00_JPRB, 0.42079E+00_JPRB, 0.41758E+00_JPRB, 0.41507E+00_JPRB, 0.41083E+00_JPRB ,&
&  0.40378E+00_JPRB, 0.40297E+00_JPRB, 0.40836E+00_JPRB, 0.41077E+00_JPRB, 0.41103E+00_JPRB, 0.39993E+00_JPRB, 0.37749E+00_JPRB ,&
&  0.36260E+00_JPRB, 0.35614E+00_JPRB, 0.36146E+00_JPRB, 0.38216E+00_JPRB, 0.39707E+00_JPRB, 0.38468E+00_JPRB, 0.39273E+00_JPRB ,&
&  0.39656E+00_JPRB, 0.39964E+00_JPRB, 0.40547E+00_JPRB, 0.41860E+00_JPRB, 0.43480E+00_JPRB, 0.45054E+00_JPRB, 0.46306E+00_JPRB ,&
&  0.47474E+00_JPRB, 0.48464E+00_JPRB, 0.49544E+00_JPRB, 0.48373E+00_JPRB, 0.48219E+00_JPRB, 0.51285E+00_JPRB, 0.55053E+00_JPRB ,&
&  0.58514E+00_JPRB, 0.60508E+00_JPRB, 0.61131E+00_JPRB, 0.63207E+00_JPRB, 0.64145E+00_JPRB, 0.63964E+00_JPRB, 0.63012E+00_JPRB ,&
&  0.61753E+00_JPRB, 0.59521E+00_JPRB, 0.57550E+00_JPRB, 0.56333E+00_JPRB, 0.56046E+00_JPRB, 0.56344E+00_JPRB, 0.56260E+00_JPRB ,&
&  0.55949E+00_JPRB, 0.54470E+00_JPRB, 0.53119E+00_JPRB, 0.50373E+00_JPRB, 0.50147E+00_JPRB, 0.51154E+00_JPRB, 0.52046E+00_JPRB ,&
&  0.51741E+00_JPRB, 0.51117E+00_JPRB, 0.50611E+00_JPRB, 0.49704E+00_JPRB, 0.46877E+00_JPRB, 0.44953E+00_JPRB, 0.43807E+00_JPRB ,&
&  0.43126E+00_JPRB, 0.42172E+00_JPRB, 0.42046E+00_JPRB, 0.41792E+00_JPRB, 0.42003E+00_JPRB, 0.42335E+00_JPRB, 0.42186E+00_JPRB ,&
&  0.42195E+00_JPRB, 0.42411E+00_JPRB, 0.43035E+00_JPRB, 0.43693E+00_JPRB, 0.44634E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.50410E+00_JPRB, 0.51112E+00_JPRB, 0.50852E+00_JPRB, 0.50936E+00_JPRB, 0.49773E+00_JPRB, 0.48379E+00_JPRB, 0.46915E+00_JPRB ,&
&  0.46474E+00_JPRB, 0.45150E+00_JPRB, 0.43708E+00_JPRB, 0.42244E+00_JPRB, 0.41399E+00_JPRB, 0.41022E+00_JPRB, 0.41286E+00_JPRB ,&
&  0.41921E+00_JPRB, 0.42707E+00_JPRB, 0.43235E+00_JPRB, 0.43138E+00_JPRB, 0.42815E+00_JPRB, 0.42654E+00_JPRB, 0.42708E+00_JPRB ,&
&  0.42641E+00_JPRB, 0.42475E+00_JPRB, 0.42221E+00_JPRB, 0.41921E+00_JPRB, 0.41701E+00_JPRB, 0.41583E+00_JPRB, 0.41610E+00_JPRB ,&
&  0.41675E+00_JPRB, 0.41627E+00_JPRB, 0.41893E+00_JPRB, 0.42058E+00_JPRB, 0.42173E+00_JPRB, 0.41229E+00_JPRB, 0.39252E+00_JPRB ,&
&  0.37602E+00_JPRB, 0.37437E+00_JPRB, 0.39923E+00_JPRB, 0.39473E+00_JPRB, 0.40593E+00_JPRB, 0.41536E+00_JPRB, 0.41575E+00_JPRB ,&
&  0.41526E+00_JPRB, 0.41786E+00_JPRB, 0.43043E+00_JPRB, 0.44866E+00_JPRB, 0.46289E+00_JPRB, 0.47812E+00_JPRB, 0.49147E+00_JPRB ,&
&  0.50804E+00_JPRB, 0.52508E+00_JPRB, 0.53568E+00_JPRB, 0.52341E+00_JPRB, 0.56998E+00_JPRB, 0.72347E+00_JPRB, 0.73855E+00_JPRB ,&
&  0.74420E+00_JPRB, 0.74780E+00_JPRB, 0.76567E+00_JPRB, 0.77540E+00_JPRB, 0.74871E+00_JPRB, 0.72611E+00_JPRB, 0.70754E+00_JPRB ,&
&  0.67676E+00_JPRB, 0.65437E+00_JPRB, 0.63998E+00_JPRB, 0.64553E+00_JPRB, 0.64663E+00_JPRB, 0.64968E+00_JPRB, 0.65575E+00_JPRB ,&
&  0.66014E+00_JPRB, 0.64809E+00_JPRB, 0.63213E+00_JPRB, 0.60453E+00_JPRB, 0.58931E+00_JPRB, 0.57874E+00_JPRB, 0.57246E+00_JPRB ,&
&  0.56214E+00_JPRB, 0.55508E+00_JPRB, 0.54666E+00_JPRB, 0.53166E+00_JPRB, 0.49650E+00_JPRB, 0.46300E+00_JPRB, 0.44263E+00_JPRB ,&
&  0.42742E+00_JPRB, 0.41370E+00_JPRB, 0.41158E+00_JPRB, 0.41380E+00_JPRB, 0.41976E+00_JPRB, 0.42668E+00_JPRB, 0.42618E+00_JPRB ,&
&  0.42992E+00_JPRB, 0.43740E+00_JPRB, 0.45194E+00_JPRB, 0.47025E+00_JPRB, 0.48690E+00_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.54102E+00_JPRB, 0.54208E+00_JPRB, 0.53527E+00_JPRB, 0.52813E+00_JPRB, 0.52050E+00_JPRB, 0.50372E+00_JPRB, 0.47706E+00_JPRB ,&
&  0.47346E+00_JPRB, 0.44438E+00_JPRB, 0.42596E+00_JPRB, 0.42020E+00_JPRB, 0.41753E+00_JPRB, 0.41157E+00_JPRB, 0.40314E+00_JPRB ,&
&  0.40411E+00_JPRB, 0.41310E+00_JPRB, 0.42395E+00_JPRB, 0.42872E+00_JPRB, 0.43092E+00_JPRB, 0.42958E+00_JPRB, 0.42937E+00_JPRB ,&
&  0.42932E+00_JPRB, 0.42932E+00_JPRB, 0.42951E+00_JPRB, 0.42595E+00_JPRB, 0.42299E+00_JPRB, 0.42253E+00_JPRB, 0.42171E+00_JPRB ,&
&  0.42623E+00_JPRB, 0.42851E+00_JPRB, 0.42988E+00_JPRB, 0.42898E+00_JPRB, 0.42591E+00_JPRB, 0.42517E+00_JPRB, 0.40581E+00_JPRB ,&
&  0.38995E+00_JPRB, 0.40050E+00_JPRB, 0.40152E+00_JPRB, 0.40952E+00_JPRB, 0.42007E+00_JPRB, 0.42827E+00_JPRB, 0.43642E+00_JPRB ,&
&  0.44307E+00_JPRB, 0.45169E+00_JPRB, 0.46259E+00_JPRB, 0.47615E+00_JPRB, 0.49365E+00_JPRB, 0.51157E+00_JPRB, 0.52712E+00_JPRB ,&
&  0.55819E+00_JPRB, 0.53834E+00_JPRB, 0.54020E+00_JPRB, 0.59185E+00_JPRB, 0.77724E+00_JPRB, 0.82810E+00_JPRB, 0.86616E+00_JPRB ,&
&  0.90510E+00_JPRB, 0.91741E+00_JPRB, 0.91277E+00_JPRB, 0.91159E+00_JPRB, 0.88715E+00_JPRB, 0.86159E+00_JPRB, 0.85206E+00_JPRB ,&
&  0.82695E+00_JPRB, 0.81859E+00_JPRB, 0.81082E+00_JPRB, 0.81904E+00_JPRB, 0.81800E+00_JPRB, 0.82865E+00_JPRB, 0.83767E+00_JPRB ,&
&  0.83253E+00_JPRB, 0.81184E+00_JPRB, 0.77020E+00_JPRB, 0.72612E+00_JPRB, 0.68563E+00_JPRB, 0.66216E+00_JPRB, 0.64271E+00_JPRB ,&
&  0.62166E+00_JPRB, 0.60607E+00_JPRB, 0.58729E+00_JPRB, 0.56140E+00_JPRB, 0.52840E+00_JPRB, 0.50124E+00_JPRB, 0.47868E+00_JPRB ,&
&  0.44778E+00_JPRB, 0.43290E+00_JPRB, 0.43169E+00_JPRB, 0.43304E+00_JPRB, 0.43913E+00_JPRB, 0.45234E+00_JPRB, 0.45303E+00_JPRB ,&
&  0.46745E+00_JPRB, 0.48115E+00_JPRB, 0.48872E+00_JPRB, 0.50076E+00_JPRB, 0.51827E+00_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.57743E+00_JPRB, 0.57558E+00_JPRB, 0.56705E+00_JPRB, 0.55820E+00_JPRB, 0.54944E+00_JPRB, 0.52438E+00_JPRB, 0.50690E+00_JPRB ,&
&  0.53742E+00_JPRB, 0.49945E+00_JPRB, 0.49310E+00_JPRB, 0.44980E+00_JPRB, 0.42817E+00_JPRB, 0.41791E+00_JPRB, 0.40920E+00_JPRB ,&
&  0.40899E+00_JPRB, 0.41641E+00_JPRB, 0.42380E+00_JPRB, 0.43067E+00_JPRB, 0.43508E+00_JPRB, 0.43459E+00_JPRB, 0.43265E+00_JPRB ,&
&  0.43231E+00_JPRB, 0.43509E+00_JPRB, 0.43858E+00_JPRB, 0.43906E+00_JPRB, 0.43941E+00_JPRB, 0.43927E+00_JPRB, 0.43656E+00_JPRB ,&
&  0.43259E+00_JPRB, 0.43337E+00_JPRB, 0.44039E+00_JPRB, 0.43979E+00_JPRB, 0.43925E+00_JPRB, 0.43799E+00_JPRB, 0.43673E+00_JPRB ,&
&  0.42596E+00_JPRB, 0.41785E+00_JPRB, 0.42383E+00_JPRB, 0.43315E+00_JPRB, 0.44446E+00_JPRB, 0.45449E+00_JPRB, 0.46414E+00_JPRB ,&
&  0.47597E+00_JPRB, 0.49022E+00_JPRB, 0.50668E+00_JPRB, 0.52357E+00_JPRB, 0.54311E+00_JPRB, 0.56748E+00_JPRB, 0.58523E+00_JPRB ,&
&  0.63980E+00_JPRB, 0.59883E+00_JPRB, 0.70881E+00_JPRB, 0.79969E+00_JPRB, 0.95747E+00_JPRB, 0.10712E+01_JPRB, 0.10794E+01_JPRB ,&
&  0.11521E+01_JPRB, 0.11563E+01_JPRB, 0.11501E+01_JPRB, 0.11165E+01_JPRB, 0.10812E+01_JPRB, 0.10580E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.10530E+01_JPRB, 0.10321E+01_JPRB, 0.10942E+01_JPRB, 0.10879E+01_JPRB, 0.10752E+01_JPRB, 0.10773E+01_JPRB, 0.10623E+01_JPRB ,&
&  0.10310E+01_JPRB, 0.98080E+00_JPRB, 0.91398E+00_JPRB, 0.81172E+00_JPRB, 0.76555E+00_JPRB, 0.72734E+00_JPRB, 0.67310E+00_JPRB ,&
&  0.64280E+00_JPRB, 0.61005E+00_JPRB, 0.58080E+00_JPRB, 0.57416E+00_JPRB, 0.53378E+00_JPRB, 0.50355E+00_JPRB, 0.48400E+00_JPRB ,&
&  0.45971E+00_JPRB, 0.45215E+00_JPRB, 0.45989E+00_JPRB, 0.47095E+00_JPRB, 0.47859E+00_JPRB, 0.48390E+00_JPRB, 0.50339E+00_JPRB ,&
&  0.49718E+00_JPRB, 0.51786E+00_JPRB, 0.55306E+00_JPRB, 0.56435E+00_JPRB, 0.57188E+00_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.62860E+00_JPRB, 0.62297E+00_JPRB, 0.61233E+00_JPRB, 0.60098E+00_JPRB, 0.58079E+00_JPRB, 0.55533E+00_JPRB, 0.52692E+00_JPRB ,&
&  0.53904E+00_JPRB, 0.52740E+00_JPRB, 0.51391E+00_JPRB, 0.49576E+00_JPRB, 0.47718E+00_JPRB, 0.45193E+00_JPRB, 0.41948E+00_JPRB ,&
&  0.41329E+00_JPRB, 0.42471E+00_JPRB, 0.43799E+00_JPRB, 0.44681E+00_JPRB, 0.44737E+00_JPRB, 0.44641E+00_JPRB, 0.44706E+00_JPRB ,&
&  0.44883E+00_JPRB, 0.45118E+00_JPRB, 0.45496E+00_JPRB, 0.45781E+00_JPRB, 0.45749E+00_JPRB, 0.45757E+00_JPRB, 0.45933E+00_JPRB ,&
&  0.44713E+00_JPRB, 0.43973E+00_JPRB, 0.43865E+00_JPRB, 0.43572E+00_JPRB, 0.45928E+00_JPRB, 0.47168E+00_JPRB, 0.48269E+00_JPRB ,&
&  0.48539E+00_JPRB, 0.48295E+00_JPRB, 0.48392E+00_JPRB, 0.49577E+00_JPRB, 0.50775E+00_JPRB, 0.51372E+00_JPRB, 0.52315E+00_JPRB ,&
&  0.53767E+00_JPRB, 0.55542E+00_JPRB, 0.57586E+00_JPRB, 0.60309E+00_JPRB, 0.63835E+00_JPRB, 0.64332E+00_JPRB, 0.68826E+00_JPRB ,&
&  0.75816E+00_JPRB, 0.88290E+00_JPRB, 0.10233E+01_JPRB, 0.10416E+01_JPRB, 0.11826E+01_JPRB, 0.12765E+01_JPRB, 0.12757E+01_JPRB ,&
&  0.13190E+01_JPRB, 0.13146E+01_JPRB, 0.13631E+01_JPRB, 0.13612E+01_JPRB, 0.13676E+01_JPRB, 0.13698E+01_JPRB, 0.13608E+01_JPRB ,&
&  0.13662E+01_JPRB, 0.14052E+01_JPRB, 0.14379E+01_JPRB, 0.14386E+01_JPRB, 0.13978E+01_JPRB, 0.13380E+01_JPRB, 0.12575E+01_JPRB ,&
&  0.11811E+01_JPRB, 0.10975E+01_JPRB, 0.10140E+01_JPRB, 0.94684E+00_JPRB, 0.87227E+00_JPRB, 0.79694E+00_JPRB, 0.72554E+00_JPRB ,&
&  0.65756E+00_JPRB, 0.58103E+00_JPRB, 0.54774E+00_JPRB, 0.54548E+00_JPRB, 0.53411E+00_JPRB, 0.49761E+00_JPRB, 0.47802E+00_JPRB ,&
&  0.47222E+00_JPRB, 0.48436E+00_JPRB, 0.50201E+00_JPRB, 0.51238E+00_JPRB, 0.53096E+00_JPRB, 0.55233E+00_JPRB, 0.53015E+00_JPRB ,&
&  0.51759E+00_JPRB, 0.57045E+00_JPRB, 0.60129E+00_JPRB, 0.61797E+00_JPRB, 0.62683E+00_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.67484E+00_JPRB, 0.66261E+00_JPRB, 0.65035E+00_JPRB, 0.63821E+00_JPRB, 0.61721E+00_JPRB, 0.58825E+00_JPRB, 0.56725E+00_JPRB ,&
&  0.56055E+00_JPRB, 0.54471E+00_JPRB, 0.52785E+00_JPRB, 0.51104E+00_JPRB, 0.49829E+00_JPRB, 0.48605E+00_JPRB, 0.46978E+00_JPRB ,&
&  0.45390E+00_JPRB, 0.46224E+00_JPRB, 0.45895E+00_JPRB, 0.46236E+00_JPRB, 0.46925E+00_JPRB, 0.46539E+00_JPRB, 0.46510E+00_JPRB ,&
&  0.47346E+00_JPRB, 0.47611E+00_JPRB, 0.47506E+00_JPRB, 0.47188E+00_JPRB, 0.47013E+00_JPRB, 0.47386E+00_JPRB, 0.47756E+00_JPRB ,&
&  0.46162E+00_JPRB, 0.44800E+00_JPRB, 0.43830E+00_JPRB, 0.44004E+00_JPRB, 0.46572E+00_JPRB, 0.50343E+00_JPRB, 0.53363E+00_JPRB ,&
&  0.55829E+00_JPRB, 0.57679E+00_JPRB, 0.58434E+00_JPRB, 0.59076E+00_JPRB, 0.59213E+00_JPRB, 0.59058E+00_JPRB, 0.59127E+00_JPRB ,&
&  0.59671E+00_JPRB, 0.61011E+00_JPRB, 0.63102E+00_JPRB, 0.66974E+00_JPRB, 0.69855E+00_JPRB, 0.73274E+00_JPRB, 0.81935E+00_JPRB ,&
&  0.95253E+00_JPRB, 0.11251E+01_JPRB, 0.12197E+01_JPRB, 0.13590E+01_JPRB, 0.14722E+01_JPRB, 0.15019E+01_JPRB, 0.14935E+01_JPRB ,&
&  0.15246E+01_JPRB, 0.14917E+01_JPRB, 0.15855E+01_JPRB, 0.15653E+01_JPRB, 0.15666E+01_JPRB, 0.16247E+01_JPRB, 0.16150E+01_JPRB ,&
&  0.15906E+01_JPRB, 0.15699E+01_JPRB, 0.16287E+01_JPRB, 0.16232E+01_JPRB, 0.15869E+01_JPRB, 0.15284E+01_JPRB, 0.14290E+01_JPRB ,&
&  0.13318E+01_JPRB, 0.12248E+01_JPRB, 0.10361E+01_JPRB, 0.89616E+00_JPRB, 0.76577E+00_JPRB, 0.75267E+00_JPRB, 0.69260E+00_JPRB ,&
&  0.62917E+00_JPRB, 0.55127E+00_JPRB, 0.54542E+00_JPRB, 0.54563E+00_JPRB, 0.55461E+00_JPRB, 0.53756E+00_JPRB, 0.50686E+00_JPRB ,&
&  0.50353E+00_JPRB, 0.52414E+00_JPRB, 0.53152E+00_JPRB, 0.54556E+00_JPRB, 0.57336E+00_JPRB, 0.61870E+00_JPRB, 0.55827E+00_JPRB ,&
&  0.57581E+00_JPRB, 0.63929E+00_JPRB, 0.66687E+00_JPRB, 0.67878E+00_JPRB, 0.68081E+00_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.69478E+00_JPRB, 0.67734E+00_JPRB, 0.66116E+00_JPRB, 0.64860E+00_JPRB, 0.63042E+00_JPRB, 0.61552E+00_JPRB, 0.59702E+00_JPRB ,&
&  0.58026E+00_JPRB, 0.56014E+00_JPRB, 0.54476E+00_JPRB, 0.53404E+00_JPRB, 0.52658E+00_JPRB, 0.51411E+00_JPRB, 0.51920E+00_JPRB ,&
&  0.49101E+00_JPRB, 0.49255E+00_JPRB, 0.46610E+00_JPRB, 0.47365E+00_JPRB, 0.51039E+00_JPRB, 0.48984E+00_JPRB, 0.48871E+00_JPRB ,&
&  0.49023E+00_JPRB, 0.49669E+00_JPRB, 0.49085E+00_JPRB, 0.48366E+00_JPRB, 0.48047E+00_JPRB, 0.48399E+00_JPRB, 0.49412E+00_JPRB ,&
&  0.48442E+00_JPRB, 0.45653E+00_JPRB, 0.44283E+00_JPRB, 0.44815E+00_JPRB, 0.48255E+00_JPRB, 0.52714E+00_JPRB, 0.57805E+00_JPRB ,&
&  0.60790E+00_JPRB, 0.63365E+00_JPRB, 0.65221E+00_JPRB, 0.66370E+00_JPRB, 0.66550E+00_JPRB, 0.66612E+00_JPRB, 0.66937E+00_JPRB ,&
&  0.67319E+00_JPRB, 0.68842E+00_JPRB, 0.71941E+00_JPRB, 0.75933E+00_JPRB, 0.76945E+00_JPRB, 0.83786E+00_JPRB, 0.10167E+01_JPRB ,&
&  0.12986E+01_JPRB, 0.13899E+01_JPRB, 0.14963E+01_JPRB, 0.17686E+01_JPRB, 0.19025E+01_JPRB, 0.19260E+01_JPRB, 0.19079E+01_JPRB ,&
&  0.18916E+01_JPRB, 0.18519E+01_JPRB, 0.18351E+01_JPRB, 0.18809E+01_JPRB, 0.18426E+01_JPRB, 0.18169E+01_JPRB, 0.17513E+01_JPRB ,&
&  0.16785E+01_JPRB, 0.16836E+01_JPRB, 0.17032E+01_JPRB, 0.16540E+01_JPRB, 0.15770E+01_JPRB, 0.15410E+01_JPRB, 0.14874E+01_JPRB ,&
&  0.13954E+01_JPRB, 0.12143E+01_JPRB, 0.90022E+00_JPRB, 0.79401E+00_JPRB, 0.62921E+00_JPRB, 0.62920E+00_JPRB, 0.71509E+00_JPRB ,&
&  0.68850E+00_JPRB, 0.63902E+00_JPRB, 0.60917E+00_JPRB, 0.59836E+00_JPRB, 0.59917E+00_JPRB, 0.61380E+00_JPRB, 0.58409E+00_JPRB ,&
&  0.54881E+00_JPRB, 0.55520E+00_JPRB, 0.57080E+00_JPRB, 0.60341E+00_JPRB, 0.65793E+00_JPRB, 0.70323E+00_JPRB, 0.66284E+00_JPRB ,&
&  0.70844E+00_JPRB, 0.72770E+00_JPRB, 0.73369E+00_JPRB, 0.73128E+00_JPRB, 0.71642E+00_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.68591E+00_JPRB, 0.66848E+00_JPRB, 0.65093E+00_JPRB, 0.64020E+00_JPRB, 0.63326E+00_JPRB, 0.63137E+00_JPRB, 0.61807E+00_JPRB ,&
&  0.59441E+00_JPRB, 0.57517E+00_JPRB, 0.56312E+00_JPRB, 0.56080E+00_JPRB, 0.55587E+00_JPRB, 0.54252E+00_JPRB, 0.53982E+00_JPRB ,&
&  0.56501E+00_JPRB, 0.55481E+00_JPRB, 0.50586E+00_JPRB, 0.47272E+00_JPRB, 0.49774E+00_JPRB, 0.49240E+00_JPRB, 0.49909E+00_JPRB ,&
&  0.49291E+00_JPRB, 0.51356E+00_JPRB, 0.50985E+00_JPRB, 0.50897E+00_JPRB, 0.51206E+00_JPRB, 0.52681E+00_JPRB, 0.56100E+00_JPRB ,&
&  0.56510E+00_JPRB, 0.52136E+00_JPRB, 0.49879E+00_JPRB, 0.51447E+00_JPRB, 0.57679E+00_JPRB, 0.57161E+00_JPRB, 0.61543E+00_JPRB ,&
&  0.64868E+00_JPRB, 0.68167E+00_JPRB, 0.70988E+00_JPRB, 0.73525E+00_JPRB, 0.75287E+00_JPRB, 0.76570E+00_JPRB, 0.77965E+00_JPRB ,&
&  0.79849E+00_JPRB, 0.81476E+00_JPRB, 0.81243E+00_JPRB, 0.84195E+00_JPRB, 0.92620E+00_JPRB, 0.11237E+01_JPRB, 0.13462E+01_JPRB ,&
&  0.15120E+01_JPRB, 0.15423E+01_JPRB, 0.15942E+01_JPRB, 0.18608E+01_JPRB, 0.20628E+01_JPRB, 0.21462E+01_JPRB, 0.22312E+01_JPRB ,&
&  0.22308E+01_JPRB, 0.22038E+01_JPRB, 0.21462E+01_JPRB, 0.19549E+01_JPRB, 0.17051E+01_JPRB, 0.15557E+01_JPRB, 0.15051E+01_JPRB ,&
&  0.14640E+01_JPRB, 0.14832E+01_JPRB, 0.15481E+01_JPRB, 0.15167E+01_JPRB, 0.14986E+01_JPRB, 0.14289E+01_JPRB, 0.14898E+01_JPRB ,&
&  0.14308E+01_JPRB, 0.88607E+00_JPRB, 0.69271E+00_JPRB, 0.72268E+00_JPRB, 0.61213E+00_JPRB, 0.61568E+00_JPRB, 0.67583E+00_JPRB ,&
&  0.65533E+00_JPRB, 0.65024E+00_JPRB, 0.67242E+00_JPRB, 0.67321E+00_JPRB, 0.67268E+00_JPRB, 0.70656E+00_JPRB, 0.70600E+00_JPRB ,&
&  0.67492E+00_JPRB, 0.64010E+00_JPRB, 0.66202E+00_JPRB, 0.71142E+00_JPRB, 0.77716E+00_JPRB, 0.80464E+00_JPRB, 0.80453E+00_JPRB ,&
&  0.80304E+00_JPRB, 0.79494E+00_JPRB, 0.77869E+00_JPRB, 0.74714E+00_JPRB, 0.70950E+00_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.70691E+00_JPRB, 0.69060E+00_JPRB, 0.67158E+00_JPRB, 0.66754E+00_JPRB, 0.66723E+00_JPRB, 0.66229E+00_JPRB, 0.64425E+00_JPRB ,&
&  0.60515E+00_JPRB, 0.58742E+00_JPRB, 0.58186E+00_JPRB, 0.58180E+00_JPRB, 0.58196E+00_JPRB, 0.57733E+00_JPRB, 0.57298E+00_JPRB ,&
&  0.56019E+00_JPRB, 0.58245E+00_JPRB, 0.52902E+00_JPRB, 0.49977E+00_JPRB, 0.46452E+00_JPRB, 0.48066E+00_JPRB, 0.50130E+00_JPRB ,&
&  0.51879E+00_JPRB, 0.53012E+00_JPRB, 0.55844E+00_JPRB, 0.59959E+00_JPRB, 0.64058E+00_JPRB, 0.68037E+00_JPRB, 0.76381E+00_JPRB ,&
&  0.75413E+00_JPRB, 0.68512E+00_JPRB, 0.61928E+00_JPRB, 0.64076E+00_JPRB, 0.67777E+00_JPRB, 0.64173E+00_JPRB, 0.70487E+00_JPRB ,&
&  0.74706E+00_JPRB, 0.78440E+00_JPRB, 0.82056E+00_JPRB, 0.84169E+00_JPRB, 0.84717E+00_JPRB, 0.82100E+00_JPRB, 0.80866E+00_JPRB ,&
&  0.80823E+00_JPRB, 0.82876E+00_JPRB, 0.86455E+00_JPRB, 0.89680E+00_JPRB, 0.11000E+01_JPRB, 0.13499E+01_JPRB, 0.15296E+01_JPRB ,&
&  0.16905E+01_JPRB, 0.14642E+01_JPRB, 0.13403E+01_JPRB, 0.17815E+01_JPRB, 0.24763E+01_JPRB, 0.26603E+01_JPRB, 0.24500E+01_JPRB ,&
&  0.25355E+01_JPRB, 0.22566E+01_JPRB, 0.19346E+01_JPRB, 0.17254E+01_JPRB, 0.15857E+01_JPRB, 0.14905E+01_JPRB, 0.14477E+01_JPRB ,&
&  0.13627E+01_JPRB, 0.13582E+01_JPRB, 0.14304E+01_JPRB, 0.14705E+01_JPRB, 0.14608E+01_JPRB, 0.14400E+01_JPRB, 0.14654E+01_JPRB ,&
&  0.13624E+01_JPRB, 0.11050E+01_JPRB, 0.79048E+00_JPRB, 0.72797E+00_JPRB, 0.60359E+00_JPRB, 0.59517E+00_JPRB, 0.65418E+00_JPRB ,&
&  0.64750E+00_JPRB, 0.68147E+00_JPRB, 0.69806E+00_JPRB, 0.69298E+00_JPRB, 0.75081E+00_JPRB, 0.78291E+00_JPRB, 0.84657E+00_JPRB ,&
&  0.88180E+00_JPRB, 0.77502E+00_JPRB, 0.82236E+00_JPRB, 0.88074E+00_JPRB, 0.92443E+00_JPRB, 0.91576E+00_JPRB, 0.88558E+00_JPRB ,&
&  0.86147E+00_JPRB, 0.83679E+00_JPRB, 0.80200E+00_JPRB, 0.75674E+00_JPRB, 0.72567E+00_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.71325E+00_JPRB, 0.69836E+00_JPRB, 0.67549E+00_JPRB, 0.67234E+00_JPRB, 0.66868E+00_JPRB, 0.66214E+00_JPRB, 0.65426E+00_JPRB ,&
&  0.64437E+00_JPRB, 0.64251E+00_JPRB, 0.63421E+00_JPRB, 0.62591E+00_JPRB, 0.62051E+00_JPRB, 0.60789E+00_JPRB, 0.59586E+00_JPRB ,&
&  0.57668E+00_JPRB, 0.57028E+00_JPRB, 0.50884E+00_JPRB, 0.47133E+00_JPRB, 0.45113E+00_JPRB, 0.46297E+00_JPRB, 0.49274E+00_JPRB ,&
&  0.53798E+00_JPRB, 0.59695E+00_JPRB, 0.69899E+00_JPRB, 0.79717E+00_JPRB, 0.93753E+00_JPRB, 0.10617E+01_JPRB, 0.11563E+01_JPRB ,&
&  0.11715E+01_JPRB, 0.90397E+00_JPRB, 0.87093E+00_JPRB, 0.81273E+00_JPRB, 0.82306E+00_JPRB, 0.80372E+00_JPRB, 0.81055E+00_JPRB ,&
&  0.81571E+00_JPRB, 0.84295E+00_JPRB, 0.86566E+00_JPRB, 0.85450E+00_JPRB, 0.83512E+00_JPRB, 0.82211E+00_JPRB, 0.85179E+00_JPRB ,&
&  0.88988E+00_JPRB, 0.89178E+00_JPRB, 0.92162E+00_JPRB, 0.10794E+01_JPRB, 0.14210E+01_JPRB, 0.16847E+01_JPRB, 0.18503E+01_JPRB ,&
&  0.17562E+01_JPRB, 0.16940E+01_JPRB, 0.24162E+01_JPRB, 0.27084E+01_JPRB, 0.25118E+01_JPRB, 0.27615E+01_JPRB, 0.26826E+01_JPRB ,&
&  0.23878E+01_JPRB, 0.18997E+01_JPRB, 0.16618E+01_JPRB, 0.15067E+01_JPRB, 0.17433E+01_JPRB, 0.16522E+01_JPRB, 0.14788E+01_JPRB ,&
&  0.13242E+01_JPRB, 0.13954E+01_JPRB, 0.14294E+01_JPRB, 0.14907E+01_JPRB, 0.15648E+01_JPRB, 0.15831E+01_JPRB, 0.14313E+01_JPRB ,&
&  0.12508E+01_JPRB, 0.13033E+01_JPRB, 0.10471E+01_JPRB, 0.80220E+00_JPRB, 0.68924E+00_JPRB, 0.68583E+00_JPRB, 0.70149E+00_JPRB ,&
&  0.71710E+00_JPRB, 0.72708E+00_JPRB, 0.72913E+00_JPRB, 0.77391E+00_JPRB, 0.86780E+00_JPRB, 0.92741E+00_JPRB, 0.97136E+00_JPRB ,&
&  0.95826E+00_JPRB, 0.10278E+01_JPRB, 0.10410E+01_JPRB, 0.10450E+01_JPRB, 0.10170E+01_JPRB, 0.94818E+00_JPRB, 0.89746E+00_JPRB ,&
&  0.86994E+00_JPRB, 0.85461E+00_JPRB, 0.81081E+00_JPRB, 0.76228E+00_JPRB, 0.72415E+00_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.72671E+00_JPRB, 0.72054E+00_JPRB, 0.70967E+00_JPRB, 0.69571E+00_JPRB, 0.68569E+00_JPRB, 0.68410E+00_JPRB, 0.69115E+00_JPRB ,&
&  0.69012E+00_JPRB, 0.67615E+00_JPRB, 0.67865E+00_JPRB, 0.68894E+00_JPRB, 0.68447E+00_JPRB, 0.65779E+00_JPRB, 0.63041E+00_JPRB ,&
&  0.61137E+00_JPRB, 0.55615E+00_JPRB, 0.48731E+00_JPRB, 0.48698E+00_JPRB, 0.48728E+00_JPRB, 0.47875E+00_JPRB, 0.51318E+00_JPRB ,&
&  0.57462E+00_JPRB, 0.70452E+00_JPRB, 0.86702E+00_JPRB, 0.10680E+01_JPRB, 0.12178E+01_JPRB, 0.12917E+01_JPRB, 0.13296E+01_JPRB ,&
&  0.12167E+01_JPRB, 0.11432E+01_JPRB, 0.10883E+01_JPRB, 0.10189E+01_JPRB, 0.98922E+00_JPRB, 0.96974E+00_JPRB, 0.93209E+00_JPRB ,&
&  0.89468E+00_JPRB, 0.88749E+00_JPRB, 0.89041E+00_JPRB, 0.86498E+00_JPRB, 0.86629E+00_JPRB, 0.89055E+00_JPRB, 0.92855E+00_JPRB ,&
&  0.92338E+00_JPRB, 0.92613E+00_JPRB, 0.10149E+01_JPRB, 0.12784E+01_JPRB, 0.13660E+01_JPRB, 0.15744E+01_JPRB, 0.17956E+01_JPRB ,&
&  0.24665E+01_JPRB, 0.27349E+01_JPRB, 0.26456E+01_JPRB, 0.33003E+01_JPRB, 0.20442E+01_JPRB, 0.22287E+01_JPRB, 0.24279E+01_JPRB ,&
&  0.22155E+01_JPRB, 0.20484E+01_JPRB, 0.21298E+01_JPRB, 0.19021E+01_JPRB, 0.16455E+01_JPRB, 0.21867E+01_JPRB, 0.18094E+01_JPRB ,&
&  0.15399E+01_JPRB, 0.15398E+01_JPRB, 0.16197E+01_JPRB, 0.17859E+01_JPRB, 0.15385E+01_JPRB, 0.13579E+01_JPRB, 0.11045E+01_JPRB ,&
&  0.10840E+01_JPRB, 0.97748E+00_JPRB, 0.10389E+01_JPRB, 0.91245E+00_JPRB, 0.89181E+00_JPRB, 0.82484E+00_JPRB, 0.76380E+00_JPRB ,&
&  0.76596E+00_JPRB, 0.76224E+00_JPRB, 0.79073E+00_JPRB, 0.90916E+00_JPRB, 0.10989E+01_JPRB, 0.10429E+01_JPRB, 0.11092E+01_JPRB ,&
&  0.11623E+01_JPRB, 0.11940E+01_JPRB, 0.11389E+01_JPRB, 0.10989E+01_JPRB, 0.99407E+00_JPRB, 0.92888E+00_JPRB, 0.89597E+00_JPRB ,&
&  0.87114E+00_JPRB, 0.82496E+00_JPRB, 0.77406E+00_JPRB, 0.75547E+00_JPRB, 0.72579E+00_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.73676E+00_JPRB, 0.72384E+00_JPRB, 0.72320E+00_JPRB, 0.71641E+00_JPRB, 0.72161E+00_JPRB, 0.72410E+00_JPRB, 0.72832E+00_JPRB ,&
&  0.72020E+00_JPRB, 0.70947E+00_JPRB, 0.71605E+00_JPRB, 0.71888E+00_JPRB, 0.70745E+00_JPRB, 0.68782E+00_JPRB, 0.64972E+00_JPRB ,&
&  0.62602E+00_JPRB, 0.58447E+00_JPRB, 0.50600E+00_JPRB, 0.48722E+00_JPRB, 0.50417E+00_JPRB, 0.50491E+00_JPRB, 0.53259E+00_JPRB ,&
&  0.68069E+00_JPRB, 0.88644E+00_JPRB, 0.10469E+01_JPRB, 0.12107E+01_JPRB, 0.13296E+01_JPRB, 0.13161E+01_JPRB, 0.12889E+01_JPRB ,&
&  0.12876E+01_JPRB, 0.11305E+01_JPRB, 0.10322E+01_JPRB, 0.10184E+01_JPRB, 0.98495E+00_JPRB, 0.94557E+00_JPRB, 0.91106E+00_JPRB ,&
&  0.87680E+00_JPRB, 0.84444E+00_JPRB, 0.82233E+00_JPRB, 0.80855E+00_JPRB, 0.82927E+00_JPRB, 0.85704E+00_JPRB, 0.89173E+00_JPRB ,&
&  0.91668E+00_JPRB, 0.97819E+00_JPRB, 0.11740E+01_JPRB, 0.14329E+01_JPRB, 0.15942E+01_JPRB, 0.17941E+01_JPRB, 0.23031E+01_JPRB ,&
&  0.26011E+01_JPRB, 0.26125E+01_JPRB, 0.28876E+01_JPRB, 0.32192E+01_JPRB, 0.27972E+01_JPRB, 0.22758E+01_JPRB, 0.23531E+01_JPRB ,&
&  0.18132E+01_JPRB, 0.16027E+01_JPRB, 0.15635E+01_JPRB, 0.16048E+01_JPRB, 0.20735E+01_JPRB, 0.29969E+01_JPRB, 0.26431E+01_JPRB ,&
&  0.23501E+01_JPRB, 0.20458E+01_JPRB, 0.20678E+01_JPRB, 0.18880E+01_JPRB, 0.16246E+01_JPRB, 0.11958E+01_JPRB, 0.14232E+01_JPRB ,&
&  0.14514E+01_JPRB, 0.14649E+01_JPRB, 0.13175E+01_JPRB, 0.10850E+01_JPRB, 0.90472E+00_JPRB, 0.95298E+00_JPRB, 0.91393E+00_JPRB ,&
&  0.88198E+00_JPRB, 0.87198E+00_JPRB, 0.10917E+01_JPRB, 0.12044E+01_JPRB, 0.12763E+01_JPRB, 0.13138E+01_JPRB, 0.14218E+01_JPRB ,&
&  0.13335E+01_JPRB, 0.13338E+01_JPRB, 0.12417E+01_JPRB, 0.10795E+01_JPRB, 0.97462E+00_JPRB, 0.92002E+00_JPRB, 0.87957E+00_JPRB ,&
&  0.84061E+00_JPRB, 0.78230E+00_JPRB, 0.75874E+00_JPRB, 0.73770E+00_JPRB, 0.72403E+00_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.74223E+00_JPRB, 0.74606E+00_JPRB, 0.74550E+00_JPRB, 0.74301E+00_JPRB, 0.73183E+00_JPRB, 0.71995E+00_JPRB, 0.72765E+00_JPRB ,&
&  0.72681E+00_JPRB, 0.71737E+00_JPRB, 0.70171E+00_JPRB, 0.69143E+00_JPRB, 0.67651E+00_JPRB, 0.66843E+00_JPRB, 0.65030E+00_JPRB ,&
&  0.64069E+00_JPRB, 0.64115E+00_JPRB, 0.54945E+00_JPRB, 0.52907E+00_JPRB, 0.53776E+00_JPRB, 0.55325E+00_JPRB, 0.58438E+00_JPRB ,&
&  0.79680E+00_JPRB, 0.10224E+01_JPRB, 0.11181E+01_JPRB, 0.12211E+01_JPRB, 0.13065E+01_JPRB, 0.12849E+01_JPRB, 0.12773E+01_JPRB ,&
&  0.12785E+01_JPRB, 0.11876E+01_JPRB, 0.10834E+01_JPRB, 0.10156E+01_JPRB, 0.94972E+00_JPRB, 0.90944E+00_JPRB, 0.87717E+00_JPRB ,&
&  0.84364E+00_JPRB, 0.81513E+00_JPRB, 0.80559E+00_JPRB, 0.81518E+00_JPRB, 0.80611E+00_JPRB, 0.86289E+00_JPRB, 0.92950E+00_JPRB ,&
&  0.99614E+00_JPRB, 0.11442E+01_JPRB, 0.13340E+01_JPRB, 0.16108E+01_JPRB, 0.18543E+01_JPRB, 0.20988E+01_JPRB, 0.29047E+01_JPRB ,&
&  0.28370E+01_JPRB, 0.28735E+01_JPRB, 0.30392E+01_JPRB, 0.39470E+01_JPRB, 0.27415E+01_JPRB, 0.22115E+01_JPRB, 0.22385E+01_JPRB ,&
&  0.17083E+01_JPRB, 0.15651E+01_JPRB, 0.18313E+01_JPRB, 0.23569E+01_JPRB, 0.24522E+01_JPRB, 0.27142E+01_JPRB, 0.34579E+01_JPRB ,&
&  0.30531E+01_JPRB, 0.26550E+01_JPRB, 0.24813E+01_JPRB, 0.23853E+01_JPRB, 0.12573E+01_JPRB, 0.97473E+00_JPRB, 0.14165E+01_JPRB ,&
&  0.14603E+01_JPRB, 0.10049E+01_JPRB, 0.78450E+00_JPRB, 0.92942E+00_JPRB, 0.73881E+00_JPRB, 0.81350E+00_JPRB, 0.10138E+01_JPRB ,&
&  0.10207E+01_JPRB, 0.11022E+01_JPRB, 0.15207E+01_JPRB, 0.14768E+01_JPRB, 0.14303E+01_JPRB, 0.18017E+01_JPRB, 0.17134E+01_JPRB ,&
&  0.15528E+01_JPRB, 0.13105E+01_JPRB, 0.12030E+01_JPRB, 0.10335E+01_JPRB, 0.94440E+00_JPRB, 0.87939E+00_JPRB, 0.82923E+00_JPRB ,&
&  0.77271E+00_JPRB, 0.74693E+00_JPRB, 0.75249E+00_JPRB, 0.75317E+00_JPRB, 0.72889E+00_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.74818E+00_JPRB, 0.74238E+00_JPRB, 0.73328E+00_JPRB, 0.73972E+00_JPRB, 0.72453E+00_JPRB, 0.70520E+00_JPRB, 0.73353E+00_JPRB ,&
&  0.73241E+00_JPRB, 0.70304E+00_JPRB, 0.67178E+00_JPRB, 0.66122E+00_JPRB, 0.65482E+00_JPRB, 0.67153E+00_JPRB, 0.67522E+00_JPRB ,&
&  0.67927E+00_JPRB, 0.67768E+00_JPRB, 0.65935E+00_JPRB, 0.64519E+00_JPRB, 0.64689E+00_JPRB, 0.65404E+00_JPRB, 0.73424E+00_JPRB ,&
&  0.97189E+00_JPRB, 0.11081E+01_JPRB, 0.11347E+01_JPRB, 0.11865E+01_JPRB, 0.12395E+01_JPRB, 0.12307E+01_JPRB, 0.13042E+01_JPRB ,&
&  0.12637E+01_JPRB, 0.11559E+01_JPRB, 0.10655E+01_JPRB, 0.98560E+00_JPRB, 0.94060E+00_JPRB, 0.90966E+00_JPRB, 0.88311E+00_JPRB ,&
&  0.84229E+00_JPRB, 0.81601E+00_JPRB, 0.82911E+00_JPRB, 0.80862E+00_JPRB, 0.83582E+00_JPRB, 0.91577E+00_JPRB, 0.10057E+01_JPRB ,&
&  0.11259E+01_JPRB, 0.13068E+01_JPRB, 0.14897E+01_JPRB, 0.18151E+01_JPRB, 0.22757E+01_JPRB, 0.24897E+01_JPRB, 0.23347E+01_JPRB ,&
&  0.21771E+01_JPRB, 0.21679E+01_JPRB, 0.28978E+01_JPRB, 0.32949E+01_JPRB, 0.28514E+01_JPRB, 0.25127E+01_JPRB, 0.22798E+01_JPRB ,&
&  0.19991E+01_JPRB, 0.19100E+01_JPRB, 0.21174E+01_JPRB, 0.27178E+01_JPRB, 0.30273E+01_JPRB, 0.28353E+01_JPRB, 0.35676E+01_JPRB ,&
&  0.35687E+01_JPRB, 0.31124E+01_JPRB, 0.21701E+01_JPRB, 0.16131E+01_JPRB, 0.14157E+01_JPRB, 0.11508E+01_JPRB, 0.84704E+00_JPRB ,&
&  0.77630E+00_JPRB, 0.76770E+00_JPRB, 0.71918E+00_JPRB, 0.72208E+00_JPRB, 0.69625E+00_JPRB, 0.81641E+00_JPRB, 0.99803E+00_JPRB ,&
&  0.12627E+01_JPRB, 0.15624E+01_JPRB, 0.17090E+01_JPRB, 0.16624E+01_JPRB, 0.15729E+01_JPRB, 0.20133E+01_JPRB, 0.19028E+01_JPRB ,&
&  0.13670E+01_JPRB, 0.11995E+01_JPRB, 0.10802E+01_JPRB, 0.96267E+00_JPRB, 0.88651E+00_JPRB, 0.81741E+00_JPRB, 0.76788E+00_JPRB ,&
&  0.74406E+00_JPRB, 0.74463E+00_JPRB, 0.74716E+00_JPRB, 0.74288E+00_JPRB, 0.72620E+00_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.75114E+00_JPRB, 0.74039E+00_JPRB, 0.72556E+00_JPRB, 0.71545E+00_JPRB, 0.68697E+00_JPRB, 0.69502E+00_JPRB, 0.73666E+00_JPRB ,&
&  0.73353E+00_JPRB, 0.68068E+00_JPRB, 0.65009E+00_JPRB, 0.65015E+00_JPRB, 0.67544E+00_JPRB, 0.69982E+00_JPRB, 0.70311E+00_JPRB ,&
&  0.71799E+00_JPRB, 0.72454E+00_JPRB, 0.77276E+00_JPRB, 0.79311E+00_JPRB, 0.84672E+00_JPRB, 0.84036E+00_JPRB, 0.98681E+00_JPRB ,&
&  0.11455E+01_JPRB, 0.11458E+01_JPRB, 0.11618E+01_JPRB, 0.12078E+01_JPRB, 0.12618E+01_JPRB, 0.12935E+01_JPRB, 0.13124E+01_JPRB ,&
&  0.12042E+01_JPRB, 0.11046E+01_JPRB, 0.10087E+01_JPRB, 0.95444E+00_JPRB, 0.92763E+00_JPRB, 0.88365E+00_JPRB, 0.85879E+00_JPRB ,&
&  0.83605E+00_JPRB, 0.83950E+00_JPRB, 0.81878E+00_JPRB, 0.83962E+00_JPRB, 0.92919E+00_JPRB, 0.10375E+01_JPRB, 0.11545E+01_JPRB ,&
&  0.12921E+01_JPRB, 0.14982E+01_JPRB, 0.17271E+01_JPRB, 0.20976E+01_JPRB, 0.20527E+01_JPRB, 0.17841E+01_JPRB, 0.20431E+01_JPRB ,&
&  0.22633E+01_JPRB, 0.24908E+01_JPRB, 0.24710E+01_JPRB, 0.31836E+01_JPRB, 0.30265E+01_JPRB, 0.25004E+01_JPRB, 0.22402E+01_JPRB ,&
&  0.19988E+01_JPRB, 0.23060E+01_JPRB, 0.26059E+01_JPRB, 0.29596E+01_JPRB, 0.38637E+01_JPRB, 0.36515E+01_JPRB, 0.39156E+01_JPRB ,&
&  0.40536E+01_JPRB, 0.33603E+01_JPRB, 0.24367E+01_JPRB, 0.18303E+01_JPRB, 0.20296E+01_JPRB, 0.20011E+01_JPRB, 0.98757E+00_JPRB ,&
&  0.82716E+00_JPRB, 0.77855E+00_JPRB, 0.75378E+00_JPRB, 0.70701E+00_JPRB, 0.69582E+00_JPRB, 0.83544E+00_JPRB, 0.12929E+01_JPRB ,&
&  0.12449E+01_JPRB, 0.19586E+01_JPRB, 0.18479E+01_JPRB, 0.17894E+01_JPRB, 0.16704E+01_JPRB, 0.21047E+01_JPRB, 0.18475E+01_JPRB ,&
&  0.14541E+01_JPRB, 0.12598E+01_JPRB, 0.10581E+01_JPRB, 0.95436E+00_JPRB, 0.85814E+00_JPRB, 0.78528E+00_JPRB, 0.76507E+00_JPRB ,&
&  0.74311E+00_JPRB, 0.71581E+00_JPRB, 0.70890E+00_JPRB, 0.71704E+00_JPRB, 0.73087E+00_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.73527E+00_JPRB, 0.72405E+00_JPRB, 0.72275E+00_JPRB, 0.72493E+00_JPRB, 0.70667E+00_JPRB, 0.73509E+00_JPRB, 0.74525E+00_JPRB ,&
&  0.72171E+00_JPRB, 0.68622E+00_JPRB, 0.67291E+00_JPRB, 0.68503E+00_JPRB, 0.71028E+00_JPRB, 0.73721E+00_JPRB, 0.76313E+00_JPRB ,&
&  0.79376E+00_JPRB, 0.83211E+00_JPRB, 0.89578E+00_JPRB, 0.10213E+01_JPRB, 0.10921E+01_JPRB, 0.11062E+01_JPRB, 0.12419E+01_JPRB ,&
&  0.12148E+01_JPRB, 0.11223E+01_JPRB, 0.11865E+01_JPRB, 0.12656E+01_JPRB, 0.12535E+01_JPRB, 0.12411E+01_JPRB, 0.12027E+01_JPRB ,&
&  0.11162E+01_JPRB, 0.10579E+01_JPRB, 0.98594E+00_JPRB, 0.97095E+00_JPRB, 0.96748E+00_JPRB, 0.92699E+00_JPRB, 0.91003E+00_JPRB ,&
&  0.89979E+00_JPRB, 0.87961E+00_JPRB, 0.88521E+00_JPRB, 0.92753E+00_JPRB, 0.10512E+01_JPRB, 0.11606E+01_JPRB, 0.13021E+01_JPRB ,&
&  0.14837E+01_JPRB, 0.16752E+01_JPRB, 0.18930E+01_JPRB, 0.21647E+01_JPRB, 0.18246E+01_JPRB, 0.21724E+01_JPRB, 0.22927E+01_JPRB ,&
&  0.24220E+01_JPRB, 0.23790E+01_JPRB, 0.21877E+01_JPRB, 0.26744E+01_JPRB, 0.27857E+01_JPRB, 0.23589E+01_JPRB, 0.20791E+01_JPRB ,&
&  0.22700E+01_JPRB, 0.25302E+01_JPRB, 0.31041E+01_JPRB, 0.35864E+01_JPRB, 0.40927E+01_JPRB, 0.56113E+01_JPRB, 0.39663E+01_JPRB ,&
&  0.33190E+01_JPRB, 0.32493E+01_JPRB, 0.26728E+01_JPRB, 0.21934E+01_JPRB, 0.23329E+01_JPRB, 0.28264E+01_JPRB, 0.20642E+01_JPRB ,&
&  0.90213E+00_JPRB, 0.72397E+00_JPRB, 0.78472E+00_JPRB, 0.78832E+00_JPRB, 0.71228E+00_JPRB, 0.83099E+00_JPRB, 0.19521E+01_JPRB ,&
&  0.15130E+01_JPRB, 0.21002E+01_JPRB, 0.18536E+01_JPRB, 0.17596E+01_JPRB, 0.17828E+01_JPRB, 0.20388E+01_JPRB, 0.20503E+01_JPRB ,&
&  0.14042E+01_JPRB, 0.11888E+01_JPRB, 0.10161E+01_JPRB, 0.91094E+00_JPRB, 0.82242E+00_JPRB, 0.77086E+00_JPRB, 0.74109E+00_JPRB ,&
&  0.71812E+00_JPRB, 0.70614E+00_JPRB, 0.70247E+00_JPRB, 0.70908E+00_JPRB, 0.72349E+00_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.74152E+00_JPRB, 0.71786E+00_JPRB, 0.71437E+00_JPRB, 0.70801E+00_JPRB, 0.74945E+00_JPRB, 0.78722E+00_JPRB, 0.74625E+00_JPRB ,&
&  0.74616E+00_JPRB, 0.74324E+00_JPRB, 0.75441E+00_JPRB, 0.74816E+00_JPRB, 0.76417E+00_JPRB, 0.82521E+00_JPRB, 0.87896E+00_JPRB ,&
&  0.94020E+00_JPRB, 0.10063E+01_JPRB, 0.10940E+01_JPRB, 0.12288E+01_JPRB, 0.13387E+01_JPRB, 0.12982E+01_JPRB, 0.12635E+01_JPRB ,&
&  0.11959E+01_JPRB, 0.11438E+01_JPRB, 0.11480E+01_JPRB, 0.12732E+01_JPRB, 0.12812E+01_JPRB, 0.12489E+01_JPRB, 0.12264E+01_JPRB ,&
&  0.11514E+01_JPRB, 0.11125E+01_JPRB, 0.10514E+01_JPRB, 0.10469E+01_JPRB, 0.10206E+01_JPRB, 0.10337E+01_JPRB, 0.10951E+01_JPRB ,&
&  0.10609E+01_JPRB, 0.10447E+01_JPRB, 0.10675E+01_JPRB, 0.11174E+01_JPRB, 0.12405E+01_JPRB, 0.13536E+01_JPRB, 0.15009E+01_JPRB ,&
&  0.16706E+01_JPRB, 0.18241E+01_JPRB, 0.20154E+01_JPRB, 0.20685E+01_JPRB, 0.20955E+01_JPRB, 0.21912E+01_JPRB, 0.24017E+01_JPRB ,&
&  0.24682E+01_JPRB, 0.24111E+01_JPRB, 0.23286E+01_JPRB, 0.24042E+01_JPRB, 0.23795E+01_JPRB, 0.22075E+01_JPRB, 0.21352E+01_JPRB ,&
&  0.25953E+01_JPRB, 0.32664E+01_JPRB, 0.37464E+01_JPRB, 0.41809E+01_JPRB, 0.44582E+01_JPRB, 0.56053E+01_JPRB, 0.51022E+01_JPRB ,&
&  0.38689E+01_JPRB, 0.29657E+01_JPRB, 0.26352E+01_JPRB, 0.26987E+01_JPRB, 0.27719E+01_JPRB, 0.27301E+01_JPRB, 0.29291E+01_JPRB ,&
&  0.28260E+01_JPRB, 0.16196E+01_JPRB, 0.12515E+01_JPRB, 0.16342E+01_JPRB, 0.11424E+01_JPRB, 0.85771E+00_JPRB, 0.11627E+01_JPRB ,&
&  0.16213E+01_JPRB, 0.19495E+01_JPRB, 0.19002E+01_JPRB, 0.17452E+01_JPRB, 0.18806E+01_JPRB, 0.18946E+01_JPRB, 0.17105E+01_JPRB ,&
&  0.12392E+01_JPRB, 0.10336E+01_JPRB, 0.91054E+00_JPRB, 0.82874E+00_JPRB, 0.76421E+00_JPRB, 0.74080E+00_JPRB, 0.73584E+00_JPRB ,&
&  0.71932E+00_JPRB, 0.72228E+00_JPRB, 0.72172E+00_JPRB, 0.71971E+00_JPRB, 0.73572E+00_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.73004E+00_JPRB, 0.71177E+00_JPRB, 0.69870E+00_JPRB, 0.70646E+00_JPRB, 0.77376E+00_JPRB, 0.80850E+00_JPRB, 0.79615E+00_JPRB ,&
&  0.78847E+00_JPRB, 0.79171E+00_JPRB, 0.82532E+00_JPRB, 0.81470E+00_JPRB, 0.86965E+00_JPRB, 0.99121E+00_JPRB, 0.10734E+01_JPRB ,&
&  0.11374E+01_JPRB, 0.12061E+01_JPRB, 0.12970E+01_JPRB, 0.14386E+01_JPRB, 0.16242E+01_JPRB, 0.18524E+01_JPRB, 0.11666E+01_JPRB ,&
&  0.10617E+01_JPRB, 0.11902E+01_JPRB, 0.12085E+01_JPRB, 0.13523E+01_JPRB, 0.13293E+01_JPRB, 0.13125E+01_JPRB, 0.12563E+01_JPRB ,&
&  0.12376E+01_JPRB, 0.12147E+01_JPRB, 0.12138E+01_JPRB, 0.12397E+01_JPRB, 0.12640E+01_JPRB, 0.13764E+01_JPRB, 0.14205E+01_JPRB ,&
&  0.13868E+01_JPRB, 0.13878E+01_JPRB, 0.14537E+01_JPRB, 0.15176E+01_JPRB, 0.15950E+01_JPRB, 0.16673E+01_JPRB, 0.17605E+01_JPRB ,&
&  0.18681E+01_JPRB, 0.20006E+01_JPRB, 0.21738E+01_JPRB, 0.20834E+01_JPRB, 0.20816E+01_JPRB, 0.21443E+01_JPRB, 0.23012E+01_JPRB ,&
&  0.22096E+01_JPRB, 0.20081E+01_JPRB, 0.21017E+01_JPRB, 0.22302E+01_JPRB, 0.21858E+01_JPRB, 0.21101E+01_JPRB, 0.24234E+01_JPRB ,&
&  0.32713E+01_JPRB, 0.41205E+01_JPRB, 0.50537E+01_JPRB, 0.45400E+01_JPRB, 0.45681E+01_JPRB, 0.49393E+01_JPRB, 0.46680E+01_JPRB ,&
&  0.41885E+01_JPRB, 0.34257E+01_JPRB, 0.30759E+01_JPRB, 0.29422E+01_JPRB, 0.27100E+01_JPRB, 0.23516E+01_JPRB, 0.23025E+01_JPRB ,&
&  0.26532E+01_JPRB, 0.31794E+01_JPRB, 0.26790E+01_JPRB, 0.17656E+01_JPRB, 0.12996E+01_JPRB, 0.97846E+00_JPRB, 0.11194E+01_JPRB ,&
&  0.17259E+01_JPRB, 0.17573E+01_JPRB, 0.17247E+01_JPRB, 0.19665E+01_JPRB, 0.17317E+01_JPRB, 0.15798E+01_JPRB, 0.12813E+01_JPRB ,&
&  0.98802E+00_JPRB, 0.82379E+00_JPRB, 0.76015E+00_JPRB, 0.71012E+00_JPRB, 0.70706E+00_JPRB, 0.71363E+00_JPRB, 0.73325E+00_JPRB ,&
&  0.75031E+00_JPRB, 0.75121E+00_JPRB, 0.73852E+00_JPRB, 0.72918E+00_JPRB, 0.73696E+00_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.74228E+00_JPRB, 0.72245E+00_JPRB, 0.70236E+00_JPRB, 0.72805E+00_JPRB, 0.79477E+00_JPRB, 0.82181E+00_JPRB, 0.83180E+00_JPRB ,&
&  0.83890E+00_JPRB, 0.83449E+00_JPRB, 0.88249E+00_JPRB, 0.90910E+00_JPRB, 0.10169E+01_JPRB, 0.11331E+01_JPRB, 0.12220E+01_JPRB ,&
&  0.12976E+01_JPRB, 0.13616E+01_JPRB, 0.14562E+01_JPRB, 0.15917E+01_JPRB, 0.17203E+01_JPRB, 0.18197E+01_JPRB, 0.14009E+01_JPRB ,&
&  0.97877E+00_JPRB, 0.14462E+01_JPRB, 0.13295E+01_JPRB, 0.14634E+01_JPRB, 0.14735E+01_JPRB, 0.15461E+01_JPRB, 0.14741E+01_JPRB ,&
&  0.14576E+01_JPRB, 0.14326E+01_JPRB, 0.15113E+01_JPRB, 0.15506E+01_JPRB, 0.15890E+01_JPRB, 0.16684E+01_JPRB, 0.17450E+01_JPRB ,&
&  0.18002E+01_JPRB, 0.18676E+01_JPRB, 0.18938E+01_JPRB, 0.18935E+01_JPRB, 0.19095E+01_JPRB, 0.19350E+01_JPRB, 0.19854E+01_JPRB ,&
&  0.20926E+01_JPRB, 0.22273E+01_JPRB, 0.22554E+01_JPRB, 0.22308E+01_JPRB, 0.21183E+01_JPRB, 0.21692E+01_JPRB, 0.22425E+01_JPRB ,&
&  0.20869E+01_JPRB, 0.18907E+01_JPRB, 0.19841E+01_JPRB, 0.20338E+01_JPRB, 0.21291E+01_JPRB, 0.26292E+01_JPRB, 0.31478E+01_JPRB ,&
&  0.40475E+01_JPRB, 0.44333E+01_JPRB, 0.55920E+01_JPRB, 0.41169E+01_JPRB, 0.39148E+01_JPRB, 0.41334E+01_JPRB, 0.36073E+01_JPRB ,&
&  0.31077E+01_JPRB, 0.30579E+01_JPRB, 0.28481E+01_JPRB, 0.27890E+01_JPRB, 0.25766E+01_JPRB, 0.22267E+01_JPRB, 0.19679E+01_JPRB ,&
&  0.19747E+01_JPRB, 0.20477E+01_JPRB, 0.17981E+01_JPRB, 0.12693E+01_JPRB, 0.11044E+01_JPRB, 0.10729E+01_JPRB, 0.14496E+01_JPRB ,&
&  0.17602E+01_JPRB, 0.18013E+01_JPRB, 0.18128E+01_JPRB, 0.17472E+01_JPRB, 0.13357E+01_JPRB, 0.10999E+01_JPRB, 0.84855E+00_JPRB ,&
&  0.68216E+00_JPRB, 0.62274E+00_JPRB, 0.66979E+00_JPRB, 0.66463E+00_JPRB, 0.68450E+00_JPRB, 0.70960E+00_JPRB, 0.72492E+00_JPRB ,&
&  0.73753E+00_JPRB, 0.74094E+00_JPRB, 0.74401E+00_JPRB, 0.74275E+00_JPRB, 0.74830E+00_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.69517E+00_JPRB, 0.68944E+00_JPRB, 0.69745E+00_JPRB, 0.77966E+00_JPRB, 0.90427E+00_JPRB, 0.10029E+01_JPRB, 0.10232E+01_JPRB ,&
&  0.10305E+01_JPRB, 0.86677E+00_JPRB, 0.91914E+00_JPRB, 0.95932E+00_JPRB, 0.10626E+01_JPRB, 0.11780E+01_JPRB, 0.13066E+01_JPRB ,&
&  0.14286E+01_JPRB, 0.15489E+01_JPRB, 0.16554E+01_JPRB, 0.17718E+01_JPRB, 0.19181E+01_JPRB, 0.20627E+01_JPRB, 0.18297E+01_JPRB ,&
&  0.13497E+01_JPRB, 0.11921E+01_JPRB, 0.14760E+01_JPRB, 0.14934E+01_JPRB, 0.14960E+01_JPRB, 0.15797E+01_JPRB, 0.16159E+01_JPRB ,&
&  0.16501E+01_JPRB, 0.16051E+01_JPRB, 0.16592E+01_JPRB, 0.17105E+01_JPRB, 0.17098E+01_JPRB, 0.18077E+01_JPRB, 0.19323E+01_JPRB ,&
&  0.20163E+01_JPRB, 0.20861E+01_JPRB, 0.21068E+01_JPRB, 0.20981E+01_JPRB, 0.21094E+01_JPRB, 0.21590E+01_JPRB, 0.22390E+01_JPRB ,&
&  0.23671E+01_JPRB, 0.25164E+01_JPRB, 0.25865E+01_JPRB, 0.24524E+01_JPRB, 0.23990E+01_JPRB, 0.24462E+01_JPRB, 0.24165E+01_JPRB ,&
&  0.22964E+01_JPRB, 0.22906E+01_JPRB, 0.24244E+01_JPRB, 0.26640E+01_JPRB, 0.27942E+01_JPRB, 0.34510E+01_JPRB, 0.38145E+01_JPRB ,&
&  0.42599E+01_JPRB, 0.44824E+01_JPRB, 0.49540E+01_JPRB, 0.34801E+01_JPRB, 0.34286E+01_JPRB, 0.33609E+01_JPRB, 0.30200E+01_JPRB ,&
&  0.27863E+01_JPRB, 0.27655E+01_JPRB, 0.26359E+01_JPRB, 0.24892E+01_JPRB, 0.23524E+01_JPRB, 0.19462E+01_JPRB, 0.18367E+01_JPRB ,&
&  0.16718E+01_JPRB, 0.17839E+01_JPRB, 0.14453E+01_JPRB, 0.12752E+01_JPRB, 0.11280E+01_JPRB, 0.12044E+01_JPRB, 0.13809E+01_JPRB ,&
&  0.16316E+01_JPRB, 0.16527E+01_JPRB, 0.15007E+01_JPRB, 0.12195E+01_JPRB, 0.84207E+00_JPRB, 0.67380E+00_JPRB, 0.59245E+00_JPRB ,&
&  0.59064E+00_JPRB, 0.60763E+00_JPRB, 0.63211E+00_JPRB, 0.64920E+00_JPRB, 0.66680E+00_JPRB, 0.68505E+00_JPRB, 0.68797E+00_JPRB ,&
&  0.68440E+00_JPRB, 0.67440E+00_JPRB, 0.66872E+00_JPRB, 0.69741E+00_JPRB, 0.71063E+00_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.61079E+00_JPRB, 0.60096E+00_JPRB, 0.61483E+00_JPRB, 0.70315E+00_JPRB, 0.79569E+00_JPRB, 0.85318E+00_JPRB, 0.83126E+00_JPRB ,&
&  0.78939E+00_JPRB, 0.75313E+00_JPRB, 0.80330E+00_JPRB, 0.86137E+00_JPRB, 0.94718E+00_JPRB, 0.10663E+01_JPRB, 0.11926E+01_JPRB ,&
&  0.13322E+01_JPRB, 0.14429E+01_JPRB, 0.15265E+01_JPRB, 0.16322E+01_JPRB, 0.17670E+01_JPRB, 0.19031E+01_JPRB, 0.19643E+01_JPRB ,&
&  0.15671E+01_JPRB, 0.11064E+01_JPRB, 0.13043E+01_JPRB, 0.14470E+01_JPRB, 0.15470E+01_JPRB, 0.16237E+01_JPRB, 0.16403E+01_JPRB ,&
&  0.15436E+01_JPRB, 0.15500E+01_JPRB, 0.16206E+01_JPRB, 0.16812E+01_JPRB, 0.16894E+01_JPRB, 0.18025E+01_JPRB, 0.19232E+01_JPRB ,&
&  0.20567E+01_JPRB, 0.21548E+01_JPRB, 0.22115E+01_JPRB, 0.22673E+01_JPRB, 0.23228E+01_JPRB, 0.24117E+01_JPRB, 0.25559E+01_JPRB ,&
&  0.27335E+01_JPRB, 0.28970E+01_JPRB, 0.30117E+01_JPRB, 0.29964E+01_JPRB, 0.29726E+01_JPRB, 0.30408E+01_JPRB, 0.30108E+01_JPRB ,&
&  0.30001E+01_JPRB, 0.29863E+01_JPRB, 0.32927E+01_JPRB, 0.37249E+01_JPRB, 0.39775E+01_JPRB, 0.36352E+01_JPRB, 0.39664E+01_JPRB ,&
&  0.40450E+01_JPRB, 0.39343E+01_JPRB, 0.36386E+01_JPRB, 0.37198E+01_JPRB, 0.25605E+01_JPRB, 0.26094E+01_JPRB, 0.24848E+01_JPRB ,&
&  0.26264E+01_JPRB, 0.25164E+01_JPRB, 0.23799E+01_JPRB, 0.21901E+01_JPRB, 0.21532E+01_JPRB, 0.17586E+01_JPRB, 0.15890E+01_JPRB ,&
&  0.15292E+01_JPRB, 0.15481E+01_JPRB, 0.13969E+01_JPRB, 0.13503E+01_JPRB, 0.12451E+01_JPRB, 0.13212E+01_JPRB, 0.13157E+01_JPRB ,&
&  0.14387E+01_JPRB, 0.12702E+01_JPRB, 0.10767E+01_JPRB, 0.86914E+00_JPRB, 0.60557E+00_JPRB, 0.52415E+00_JPRB, 0.53644E+00_JPRB ,&
&  0.55847E+00_JPRB, 0.58117E+00_JPRB, 0.58568E+00_JPRB, 0.60755E+00_JPRB, 0.62150E+00_JPRB, 0.61392E+00_JPRB, 0.59950E+00_JPRB ,&
&  0.59697E+00_JPRB, 0.61227E+00_JPRB, 0.61042E+00_JPRB, 0.61424E+00_JPRB, 0.61758E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.53218E+00_JPRB, 0.53869E+00_JPRB, 0.56198E+00_JPRB, 0.60692E+00_JPRB, 0.67376E+00_JPRB, 0.71000E+00_JPRB, 0.71068E+00_JPRB ,&
&  0.68750E+00_JPRB, 0.69333E+00_JPRB, 0.72083E+00_JPRB, 0.72620E+00_JPRB, 0.77472E+00_JPRB, 0.87291E+00_JPRB, 0.96911E+00_JPRB ,&
&  0.10980E+01_JPRB, 0.12170E+01_JPRB, 0.13276E+01_JPRB, 0.14980E+01_JPRB, 0.16693E+01_JPRB, 0.18669E+01_JPRB, 0.19796E+01_JPRB ,&
&  0.19446E+01_JPRB, 0.16745E+01_JPRB, 0.16179E+01_JPRB, 0.13504E+01_JPRB, 0.11985E+01_JPRB, 0.14238E+01_JPRB, 0.14015E+01_JPRB ,&
&  0.14107E+01_JPRB, 0.14558E+01_JPRB, 0.14721E+01_JPRB, 0.15403E+01_JPRB, 0.16466E+01_JPRB, 0.17564E+01_JPRB, 0.18894E+01_JPRB ,&
&  0.20435E+01_JPRB, 0.21924E+01_JPRB, 0.23269E+01_JPRB, 0.24508E+01_JPRB, 0.25393E+01_JPRB, 0.26593E+01_JPRB, 0.28395E+01_JPRB ,&
&  0.30392E+01_JPRB, 0.31984E+01_JPRB, 0.33731E+01_JPRB, 0.35942E+01_JPRB, 0.35675E+01_JPRB, 0.36776E+01_JPRB, 0.37223E+01_JPRB ,&
&  0.37473E+01_JPRB, 0.36967E+01_JPRB, 0.37220E+01_JPRB, 0.38643E+01_JPRB, 0.38286E+01_JPRB, 0.33895E+01_JPRB, 0.33525E+01_JPRB ,&
&  0.34336E+01_JPRB, 0.31455E+01_JPRB, 0.26901E+01_JPRB, 0.29848E+01_JPRB, 0.19827E+01_JPRB, 0.22086E+01_JPRB, 0.23859E+01_JPRB ,&
&  0.22701E+01_JPRB, 0.21529E+01_JPRB, 0.19595E+01_JPRB, 0.18891E+01_JPRB, 0.18158E+01_JPRB, 0.15662E+01_JPRB, 0.14898E+01_JPRB ,&
&  0.14614E+01_JPRB, 0.12327E+01_JPRB, 0.11487E+01_JPRB, 0.12003E+01_JPRB, 0.12780E+01_JPRB, 0.11905E+01_JPRB, 0.10314E+01_JPRB ,&
&  0.93384E+00_JPRB, 0.10792E+01_JPRB, 0.96585E+00_JPRB, 0.10636E+01_JPRB, 0.89574E+00_JPRB, 0.50663E+00_JPRB, 0.50534E+00_JPRB ,&
&  0.51887E+00_JPRB, 0.53344E+00_JPRB, 0.53605E+00_JPRB, 0.55791E+00_JPRB, 0.56084E+00_JPRB, 0.55274E+00_JPRB, 0.55520E+00_JPRB ,&
&  0.56081E+00_JPRB, 0.55811E+00_JPRB, 0.55908E+00_JPRB, 0.55513E+00_JPRB, 0.54901E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.55279E+00_JPRB, 0.54972E+00_JPRB, 0.55660E+00_JPRB, 0.55448E+00_JPRB, 0.56510E+00_JPRB, 0.56471E+00_JPRB, 0.56837E+00_JPRB ,&
&  0.58036E+00_JPRB, 0.59935E+00_JPRB, 0.59143E+00_JPRB, 0.56830E+00_JPRB, 0.56337E+00_JPRB, 0.57373E+00_JPRB, 0.61137E+00_JPRB ,&
&  0.69818E+00_JPRB, 0.78749E+00_JPRB, 0.86628E+00_JPRB, 0.98049E+00_JPRB, 0.11913E+01_JPRB, 0.15496E+01_JPRB, 0.18964E+01_JPRB ,&
&  0.21503E+01_JPRB, 0.23270E+01_JPRB, 0.23294E+01_JPRB, 0.24392E+01_JPRB, 0.19403E+01_JPRB, 0.13840E+01_JPRB, 0.15427E+01_JPRB ,&
&  0.16661E+01_JPRB, 0.16518E+01_JPRB, 0.14931E+01_JPRB, 0.14705E+01_JPRB, 0.14912E+01_JPRB, 0.15866E+01_JPRB, 0.17750E+01_JPRB ,&
&  0.19677E+01_JPRB, 0.21712E+01_JPRB, 0.23426E+01_JPRB, 0.24944E+01_JPRB, 0.26234E+01_JPRB, 0.27739E+01_JPRB, 0.29158E+01_JPRB ,&
&  0.30719E+01_JPRB, 0.31883E+01_JPRB, 0.33092E+01_JPRB, 0.35239E+01_JPRB, 0.37332E+01_JPRB, 0.39320E+01_JPRB, 0.40335E+01_JPRB ,&
&  0.40400E+01_JPRB, 0.37971E+01_JPRB, 0.37958E+01_JPRB, 0.38090E+01_JPRB, 0.35536E+01_JPRB, 0.32044E+01_JPRB, 0.30464E+01_JPRB ,&
&  0.28678E+01_JPRB, 0.25915E+01_JPRB, 0.19553E+01_JPRB, 0.23070E+01_JPRB, 0.25570E+01_JPRB, 0.22273E+01_JPRB, 0.20790E+01_JPRB ,&
&  0.19087E+01_JPRB, 0.17936E+01_JPRB, 0.16469E+01_JPRB, 0.16262E+01_JPRB, 0.16959E+01_JPRB, 0.19537E+01_JPRB, 0.14825E+01_JPRB ,&
&  0.13405E+01_JPRB, 0.97913E+00_JPRB, 0.87665E+00_JPRB, 0.93766E+00_JPRB, 0.10001E+01_JPRB, 0.94104E+00_JPRB, 0.77517E+00_JPRB ,&
&  0.77095E+00_JPRB, 0.96354E+00_JPRB, 0.10179E+01_JPRB, 0.14442E+01_JPRB, 0.17805E+01_JPRB, 0.52732E+00_JPRB, 0.48205E+00_JPRB ,&
&  0.48965E+00_JPRB, 0.49395E+00_JPRB, 0.49782E+00_JPRB, 0.50728E+00_JPRB, 0.50198E+00_JPRB, 0.50885E+00_JPRB, 0.52422E+00_JPRB ,&
&  0.53458E+00_JPRB, 0.54084E+00_JPRB, 0.54729E+00_JPRB, 0.54192E+00_JPRB, 0.55014E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.63448E+00_JPRB, 0.63489E+00_JPRB, 0.63800E+00_JPRB, 0.63322E+00_JPRB, 0.62833E+00_JPRB, 0.62593E+00_JPRB, 0.62696E+00_JPRB ,&
&  0.62652E+00_JPRB, 0.61678E+00_JPRB, 0.60694E+00_JPRB, 0.59864E+00_JPRB, 0.58612E+00_JPRB, 0.56002E+00_JPRB, 0.52250E+00_JPRB ,&
&  0.52903E+00_JPRB, 0.54052E+00_JPRB, 0.55732E+00_JPRB, 0.59789E+00_JPRB, 0.68191E+00_JPRB, 0.81001E+00_JPRB, 0.98068E+00_JPRB ,&
&  0.13128E+01_JPRB, 0.16805E+01_JPRB, 0.19414E+01_JPRB, 0.20326E+01_JPRB, 0.22522E+01_JPRB, 0.16458E+01_JPRB, 0.15084E+01_JPRB ,&
&  0.16305E+01_JPRB, 0.14340E+01_JPRB, 0.13768E+01_JPRB, 0.13451E+01_JPRB, 0.13666E+01_JPRB, 0.14342E+01_JPRB, 0.15663E+01_JPRB ,&
&  0.17673E+01_JPRB, 0.19732E+01_JPRB, 0.21575E+01_JPRB, 0.23448E+01_JPRB, 0.25210E+01_JPRB, 0.27059E+01_JPRB, 0.28543E+01_JPRB ,&
&  0.29452E+01_JPRB, 0.30525E+01_JPRB, 0.32219E+01_JPRB, 0.32830E+01_JPRB, 0.35666E+01_JPRB, 0.37551E+01_JPRB, 0.38365E+01_JPRB ,&
&  0.37324E+01_JPRB, 0.35302E+01_JPRB, 0.35075E+01_JPRB, 0.32953E+01_JPRB, 0.30678E+01_JPRB, 0.28591E+01_JPRB, 0.26270E+01_JPRB ,&
&  0.24006E+01_JPRB, 0.20833E+01_JPRB, 0.16401E+01_JPRB, 0.17888E+01_JPRB, 0.18362E+01_JPRB, 0.16254E+01_JPRB, 0.18487E+01_JPRB ,&
&  0.17071E+01_JPRB, 0.16573E+01_JPRB, 0.15601E+01_JPRB, 0.14674E+01_JPRB, 0.14006E+01_JPRB, 0.15529E+01_JPRB, 0.13944E+01_JPRB ,&
&  0.10901E+01_JPRB, 0.85616E+00_JPRB, 0.71781E+00_JPRB, 0.70223E+00_JPRB, 0.74706E+00_JPRB, 0.79711E+00_JPRB, 0.77904E+00_JPRB ,&
&  0.83476E+00_JPRB, 0.78347E+00_JPRB, 0.81459E+00_JPRB, 0.98360E+00_JPRB, 0.71644E+00_JPRB, 0.49980E+00_JPRB, 0.47161E+00_JPRB ,&
&  0.46742E+00_JPRB, 0.47113E+00_JPRB, 0.47866E+00_JPRB, 0.50032E+00_JPRB, 0.52113E+00_JPRB, 0.53956E+00_JPRB, 0.55407E+00_JPRB ,&
&  0.56284E+00_JPRB, 0.58327E+00_JPRB, 0.60314E+00_JPRB, 0.62022E+00_JPRB, 0.63337E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.72980E+00_JPRB, 0.74281E+00_JPRB, 0.73676E+00_JPRB, 0.74429E+00_JPRB, 0.74711E+00_JPRB, 0.75948E+00_JPRB, 0.75684E+00_JPRB ,&
&  0.75146E+00_JPRB, 0.75697E+00_JPRB, 0.73713E+00_JPRB, 0.74172E+00_JPRB, 0.73808E+00_JPRB, 0.74174E+00_JPRB, 0.73193E+00_JPRB ,&
&  0.73522E+00_JPRB, 0.75168E+00_JPRB, 0.76057E+00_JPRB, 0.76834E+00_JPRB, 0.76367E+00_JPRB, 0.74517E+00_JPRB, 0.72522E+00_JPRB ,&
&  0.76563E+00_JPRB, 0.87026E+00_JPRB, 0.99188E+00_JPRB, 0.10814E+01_JPRB, 0.12875E+01_JPRB, 0.12862E+01_JPRB, 0.13921E+01_JPRB ,&
&  0.14153E+01_JPRB, 0.12768E+01_JPRB, 0.13806E+01_JPRB, 0.12314E+01_JPRB, 0.12251E+01_JPRB, 0.12025E+01_JPRB, 0.12046E+01_JPRB ,&
&  0.12534E+01_JPRB, 0.13453E+01_JPRB, 0.14523E+01_JPRB, 0.15734E+01_JPRB, 0.16774E+01_JPRB, 0.17225E+01_JPRB, 0.17540E+01_JPRB ,&
&  0.17682E+01_JPRB, 0.19246E+01_JPRB, 0.23481E+01_JPRB, 0.26664E+01_JPRB, 0.30277E+01_JPRB, 0.32085E+01_JPRB, 0.32242E+01_JPRB ,&
&  0.31719E+01_JPRB, 0.31482E+01_JPRB, 0.26816E+01_JPRB, 0.23254E+01_JPRB, 0.22280E+01_JPRB, 0.19762E+01_JPRB, 0.18834E+01_JPRB ,&
&  0.19257E+01_JPRB, 0.17734E+01_JPRB, 0.13654E+01_JPRB, 0.14424E+01_JPRB, 0.15131E+01_JPRB, 0.14713E+01_JPRB, 0.16103E+01_JPRB ,&
&  0.14203E+01_JPRB, 0.13056E+01_JPRB, 0.12570E+01_JPRB, 0.12362E+01_JPRB, 0.12030E+01_JPRB, 0.11268E+01_JPRB, 0.10889E+01_JPRB ,&
&  0.92147E+00_JPRB, 0.81900E+00_JPRB, 0.76635E+00_JPRB, 0.74644E+00_JPRB, 0.80205E+00_JPRB, 0.93441E+00_JPRB, 0.80111E+00_JPRB ,&
&  0.81388E+00_JPRB, 0.73760E+00_JPRB, 0.88327E+00_JPRB, 0.11702E+01_JPRB, 0.63646E+00_JPRB, 0.53147E+00_JPRB, 0.48714E+00_JPRB ,&
&  0.50648E+00_JPRB, 0.53844E+00_JPRB, 0.56351E+00_JPRB, 0.59566E+00_JPRB, 0.61197E+00_JPRB, 0.61371E+00_JPRB, 0.62330E+00_JPRB ,&
&  0.64946E+00_JPRB, 0.67580E+00_JPRB, 0.71234E+00_JPRB, 0.71657E+00_JPRB, 0.71445E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.78777E+00_JPRB, 0.80296E+00_JPRB, 0.80367E+00_JPRB, 0.81906E+00_JPRB, 0.83807E+00_JPRB, 0.84232E+00_JPRB, 0.84328E+00_JPRB ,&
&  0.83297E+00_JPRB, 0.83520E+00_JPRB, 0.83148E+00_JPRB, 0.84915E+00_JPRB, 0.85146E+00_JPRB, 0.85415E+00_JPRB, 0.85692E+00_JPRB ,&
&  0.87124E+00_JPRB, 0.88335E+00_JPRB, 0.91443E+00_JPRB, 0.91970E+00_JPRB, 0.91625E+00_JPRB, 0.92330E+00_JPRB, 0.94369E+00_JPRB ,&
&  0.96817E+00_JPRB, 0.99693E+00_JPRB, 0.10345E+01_JPRB, 0.10674E+01_JPRB, 0.11773E+01_JPRB, 0.13352E+01_JPRB, 0.16119E+01_JPRB ,&
&  0.14026E+01_JPRB, 0.13326E+01_JPRB, 0.12746E+01_JPRB, 0.11030E+01_JPRB, 0.10907E+01_JPRB, 0.11117E+01_JPRB, 0.10668E+01_JPRB ,&
&  0.10195E+01_JPRB, 0.10018E+01_JPRB, 0.98477E+00_JPRB, 0.98362E+00_JPRB, 0.99564E+00_JPRB, 0.98323E+00_JPRB, 0.97528E+00_JPRB ,&
&  0.93927E+00_JPRB, 0.96040E+00_JPRB, 0.12014E+01_JPRB, 0.15227E+01_JPRB, 0.18091E+01_JPRB, 0.20821E+01_JPRB, 0.18886E+01_JPRB ,&
&  0.18132E+01_JPRB, 0.19028E+01_JPRB, 0.17039E+01_JPRB, 0.15568E+01_JPRB, 0.14067E+01_JPRB, 0.12585E+01_JPRB, 0.11811E+01_JPRB ,&
&  0.12841E+01_JPRB, 0.13617E+01_JPRB, 0.12334E+01_JPRB, 0.14121E+01_JPRB, 0.14261E+01_JPRB, 0.14460E+01_JPRB, 0.12799E+01_JPRB ,&
&  0.11198E+01_JPRB, 0.10309E+01_JPRB, 0.10161E+01_JPRB, 0.99927E+00_JPRB, 0.10052E+01_JPRB, 0.95903E+00_JPRB, 0.88297E+00_JPRB ,&
&  0.85100E+00_JPRB, 0.78989E+00_JPRB, 0.77022E+00_JPRB, 0.72094E+00_JPRB, 0.84735E+00_JPRB, 0.80843E+00_JPRB, 0.10431E+01_JPRB ,&
&  0.79358E+00_JPRB, 0.71471E+00_JPRB, 0.62768E+00_JPRB, 0.88975E+00_JPRB, 0.62747E+00_JPRB, 0.51189E+00_JPRB, 0.53813E+00_JPRB ,&
&  0.61077E+00_JPRB, 0.65053E+00_JPRB, 0.66687E+00_JPRB, 0.69403E+00_JPRB, 0.70146E+00_JPRB, 0.69429E+00_JPRB, 0.70123E+00_JPRB ,&
&  0.73730E+00_JPRB, 0.74437E+00_JPRB, 0.75105E+00_JPRB, 0.75263E+00_JPRB, 0.76957E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.81264E+00_JPRB, 0.83627E+00_JPRB, 0.85034E+00_JPRB, 0.86436E+00_JPRB, 0.88263E+00_JPRB, 0.89113E+00_JPRB, 0.89762E+00_JPRB ,&
&  0.89945E+00_JPRB, 0.89626E+00_JPRB, 0.90351E+00_JPRB, 0.91763E+00_JPRB, 0.92559E+00_JPRB, 0.92400E+00_JPRB, 0.93051E+00_JPRB ,&
&  0.94536E+00_JPRB, 0.96933E+00_JPRB, 0.99421E+00_JPRB, 0.98806E+00_JPRB, 0.98375E+00_JPRB, 0.99755E+00_JPRB, 0.10168E+01_JPRB ,&
&  0.10182E+01_JPRB, 0.10147E+01_JPRB, 0.10224E+01_JPRB, 0.10326E+01_JPRB, 0.10522E+01_JPRB, 0.10557E+01_JPRB, 0.11518E+01_JPRB ,&
&  0.10483E+01_JPRB, 0.14476E+01_JPRB, 0.13955E+01_JPRB, 0.12112E+01_JPRB, 0.11951E+01_JPRB, 0.11162E+01_JPRB, 0.11076E+01_JPRB ,&
&  0.11093E+01_JPRB, 0.10833E+01_JPRB, 0.10601E+01_JPRB, 0.10392E+01_JPRB, 0.10465E+01_JPRB, 0.10635E+01_JPRB, 0.10409E+01_JPRB ,&
&  0.10544E+01_JPRB, 0.10826E+01_JPRB, 0.11613E+01_JPRB, 0.12673E+01_JPRB, 0.13637E+01_JPRB, 0.13876E+01_JPRB, 0.12905E+01_JPRB ,&
&  0.12224E+01_JPRB, 0.11164E+01_JPRB, 0.90771E+00_JPRB, 0.89570E+00_JPRB, 0.87023E+00_JPRB, 0.72272E+00_JPRB, 0.65988E+00_JPRB ,&
&  0.87356E+00_JPRB, 0.99922E+00_JPRB, 0.12676E+01_JPRB, 0.13088E+01_JPRB, 0.12600E+01_JPRB, 0.12574E+01_JPRB, 0.11094E+01_JPRB ,&
&  0.99017E+00_JPRB, 0.90212E+00_JPRB, 0.88742E+00_JPRB, 0.85136E+00_JPRB, 0.85475E+00_JPRB, 0.84082E+00_JPRB, 0.78359E+00_JPRB ,&
&  0.73613E+00_JPRB, 0.70891E+00_JPRB, 0.69988E+00_JPRB, 0.71606E+00_JPRB, 0.67797E+00_JPRB, 0.10121E+01_JPRB, 0.15717E+01_JPRB ,&
&  0.86329E+00_JPRB, 0.64614E+00_JPRB, 0.48653E+00_JPRB, 0.61729E+00_JPRB, 0.57687E+00_JPRB, 0.55524E+00_JPRB, 0.61392E+00_JPRB ,&
&  0.72751E+00_JPRB, 0.77408E+00_JPRB, 0.78171E+00_JPRB, 0.77274E+00_JPRB, 0.74916E+00_JPRB, 0.72721E+00_JPRB, 0.71782E+00_JPRB ,&
&  0.73297E+00_JPRB, 0.74791E+00_JPRB, 0.76491E+00_JPRB, 0.77426E+00_JPRB, 0.78884E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.82799E+00_JPRB, 0.85039E+00_JPRB, 0.86764E+00_JPRB, 0.88735E+00_JPRB, 0.90795E+00_JPRB, 0.92673E+00_JPRB, 0.94412E+00_JPRB ,&
&  0.95846E+00_JPRB, 0.97039E+00_JPRB, 0.98139E+00_JPRB, 0.99646E+00_JPRB, 0.10161E+01_JPRB, 0.10318E+01_JPRB, 0.10518E+01_JPRB ,&
&  0.10826E+01_JPRB, 0.11073E+01_JPRB, 0.11339E+01_JPRB, 0.11311E+01_JPRB, 0.11312E+01_JPRB, 0.11520E+01_JPRB, 0.11721E+01_JPRB ,&
&  0.11791E+01_JPRB, 0.11937E+01_JPRB, 0.12179E+01_JPRB, 0.12461E+01_JPRB, 0.12813E+01_JPRB, 0.13473E+01_JPRB, 0.12840E+01_JPRB ,&
&  0.13738E+01_JPRB, 0.15214E+01_JPRB, 0.14747E+01_JPRB, 0.13301E+01_JPRB, 0.12666E+01_JPRB, 0.12524E+01_JPRB, 0.12736E+01_JPRB ,&
&  0.13150E+01_JPRB, 0.13079E+01_JPRB, 0.13079E+01_JPRB, 0.12993E+01_JPRB, 0.13057E+01_JPRB, 0.13336E+01_JPRB, 0.13609E+01_JPRB ,&
&  0.13996E+01_JPRB, 0.14481E+01_JPRB, 0.15179E+01_JPRB, 0.16089E+01_JPRB, 0.17088E+01_JPRB, 0.16996E+01_JPRB, 0.16464E+01_JPRB ,&
&  0.15312E+01_JPRB, 0.14137E+01_JPRB, 0.11061E+01_JPRB, 0.10226E+01_JPRB, 0.95057E+00_JPRB, 0.82172E+00_JPRB, 0.71517E+00_JPRB ,&
&  0.88839E+00_JPRB, 0.94812E+00_JPRB, 0.10849E+01_JPRB, 0.12585E+01_JPRB, 0.11751E+01_JPRB, 0.10875E+01_JPRB, 0.97417E+00_JPRB ,&
&  0.88374E+00_JPRB, 0.81850E+00_JPRB, 0.79231E+00_JPRB, 0.75909E+00_JPRB, 0.75338E+00_JPRB, 0.75936E+00_JPRB, 0.70529E+00_JPRB ,&
&  0.68279E+00_JPRB, 0.67819E+00_JPRB, 0.68216E+00_JPRB, 0.69134E+00_JPRB, 0.74592E+00_JPRB, 0.74525E+00_JPRB, 0.10452E+01_JPRB ,&
&  0.80876E+00_JPRB, 0.55234E+00_JPRB, 0.58582E+00_JPRB, 0.63345E+00_JPRB, 0.61747E+00_JPRB, 0.61686E+00_JPRB, 0.69313E+00_JPRB ,&
&  0.84393E+00_JPRB, 0.96165E+00_JPRB, 0.98862E+00_JPRB, 0.93363E+00_JPRB, 0.84097E+00_JPRB, 0.78647E+00_JPRB, 0.75793E+00_JPRB ,&
&  0.76724E+00_JPRB, 0.77629E+00_JPRB, 0.78820E+00_JPRB, 0.79191E+00_JPRB, 0.80403E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.82262E+00_JPRB, 0.84795E+00_JPRB, 0.86869E+00_JPRB, 0.88883E+00_JPRB, 0.90714E+00_JPRB, 0.92429E+00_JPRB, 0.94379E+00_JPRB ,&
&  0.95667E+00_JPRB, 0.97564E+00_JPRB, 0.99879E+00_JPRB, 0.10223E+01_JPRB, 0.10422E+01_JPRB, 0.10697E+01_JPRB, 0.10931E+01_JPRB ,&
&  0.11264E+01_JPRB, 0.11488E+01_JPRB, 0.11700E+01_JPRB, 0.11719E+01_JPRB, 0.11705E+01_JPRB, 0.11814E+01_JPRB, 0.11879E+01_JPRB ,&
&  0.11818E+01_JPRB, 0.11836E+01_JPRB, 0.11941E+01_JPRB, 0.12157E+01_JPRB, 0.12246E+01_JPRB, 0.12743E+01_JPRB, 0.11227E+01_JPRB ,&
&  0.14693E+01_JPRB, 0.16536E+01_JPRB, 0.15890E+01_JPRB, 0.14558E+01_JPRB, 0.13830E+01_JPRB, 0.14109E+01_JPRB, 0.14525E+01_JPRB ,&
&  0.15104E+01_JPRB, 0.15953E+01_JPRB, 0.15591E+01_JPRB, 0.15472E+01_JPRB, 0.15660E+01_JPRB, 0.16134E+01_JPRB, 0.16898E+01_JPRB ,&
&  0.17706E+01_JPRB, 0.18600E+01_JPRB, 0.19569E+01_JPRB, 0.20514E+01_JPRB, 0.21354E+01_JPRB, 0.21879E+01_JPRB, 0.21854E+01_JPRB ,&
&  0.21192E+01_JPRB, 0.20279E+01_JPRB, 0.18206E+01_JPRB, 0.17792E+01_JPRB, 0.16769E+01_JPRB, 0.13821E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.10366E+01_JPRB, 0.10623E+01_JPRB, 0.10653E+01_JPRB, 0.11753E+01_JPRB, 0.11128E+01_JPRB, 0.10193E+01_JPRB, 0.92172E+00_JPRB ,&
&  0.83813E+00_JPRB, 0.77049E+00_JPRB, 0.75204E+00_JPRB, 0.71031E+00_JPRB, 0.68112E+00_JPRB, 0.66082E+00_JPRB, 0.64280E+00_JPRB ,&
&  0.65124E+00_JPRB, 0.66468E+00_JPRB, 0.68487E+00_JPRB, 0.70016E+00_JPRB, 0.70667E+00_JPRB, 0.65971E+00_JPRB, 0.90062E+00_JPRB ,&
&  0.77150E+00_JPRB, 0.66213E+00_JPRB, 0.77940E+00_JPRB, 0.55116E+00_JPRB, 0.65296E+00_JPRB, 0.71838E+00_JPRB, 0.84461E+00_JPRB ,&
&  0.93144E+00_JPRB, 0.13056E+01_JPRB, 0.17936E+01_JPRB, 0.14318E+01_JPRB, 0.11509E+01_JPRB, 0.87150E+00_JPRB, 0.77945E+00_JPRB ,&
&  0.77678E+00_JPRB, 0.77905E+00_JPRB, 0.78693E+00_JPRB, 0.78911E+00_JPRB, 0.79984E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.81221E+00_JPRB, 0.83154E+00_JPRB, 0.85373E+00_JPRB, 0.87369E+00_JPRB, 0.88606E+00_JPRB, 0.89687E+00_JPRB, 0.91094E+00_JPRB ,&
&  0.91775E+00_JPRB, 0.91846E+00_JPRB, 0.92910E+00_JPRB, 0.94719E+00_JPRB, 0.96998E+00_JPRB, 0.99472E+00_JPRB, 0.10252E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.11030E+01_JPRB, 0.11503E+01_JPRB, 0.11825E+01_JPRB, 0.12016E+01_JPRB, 0.12203E+01_JPRB, 0.12361E+01_JPRB ,&
&  0.12664E+01_JPRB, 0.12849E+01_JPRB, 0.13049E+01_JPRB, 0.13515E+01_JPRB, 0.13846E+01_JPRB, 0.14217E+01_JPRB, 0.11946E+01_JPRB ,&
&  0.15745E+01_JPRB, 0.18525E+01_JPRB, 0.17391E+01_JPRB, 0.15658E+01_JPRB, 0.14752E+01_JPRB, 0.14853E+01_JPRB, 0.15020E+01_JPRB ,&
&  0.15412E+01_JPRB, 0.15681E+01_JPRB, 0.15312E+01_JPRB, 0.16582E+01_JPRB, 0.17563E+01_JPRB, 0.18041E+01_JPRB, 0.19257E+01_JPRB ,&
&  0.20500E+01_JPRB, 0.21680E+01_JPRB, 0.22829E+01_JPRB, 0.23915E+01_JPRB, 0.24812E+01_JPRB, 0.25802E+01_JPRB, 0.26048E+01_JPRB ,&
&  0.26049E+01_JPRB, 0.26040E+01_JPRB, 0.25826E+01_JPRB, 0.23426E+01_JPRB, 0.22356E+01_JPRB, 0.20951E+01_JPRB, 0.16878E+01_JPRB ,&
&  0.13507E+01_JPRB, 0.10972E+01_JPRB, 0.10171E+01_JPRB, 0.10544E+01_JPRB, 0.99402E+00_JPRB, 0.89478E+00_JPRB, 0.81799E+00_JPRB ,&
&  0.75083E+00_JPRB, 0.70678E+00_JPRB, 0.70896E+00_JPRB, 0.67556E+00_JPRB, 0.60198E+00_JPRB, 0.61484E+00_JPRB, 0.63636E+00_JPRB ,&
&  0.62488E+00_JPRB, 0.63974E+00_JPRB, 0.66340E+00_JPRB, 0.66409E+00_JPRB, 0.68751E+00_JPRB, 0.74722E+00_JPRB, 0.76424E+00_JPRB ,&
&  0.82256E+00_JPRB, 0.70849E+00_JPRB, 0.70871E+00_JPRB, 0.75979E+00_JPRB, 0.74118E+00_JPRB, 0.95975E+00_JPRB, 0.13066E+01_JPRB ,&
&  0.18921E+01_JPRB, 0.20521E+01_JPRB, 0.26701E+01_JPRB, 0.41618E+01_JPRB, 0.31629E+01_JPRB, 0.21029E+01_JPRB, 0.81113E+00_JPRB ,&
&  0.81156E+00_JPRB, 0.80388E+00_JPRB, 0.81762E+00_JPRB, 0.81314E+00_JPRB, 0.81456E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.71026E+00_JPRB, 0.70798E+00_JPRB, 0.73306E+00_JPRB, 0.75907E+00_JPRB, 0.78245E+00_JPRB, 0.80446E+00_JPRB, 0.82071E+00_JPRB ,&
&  0.84459E+00_JPRB, 0.86306E+00_JPRB, 0.86641E+00_JPRB, 0.86184E+00_JPRB, 0.87338E+00_JPRB, 0.87454E+00_JPRB, 0.89609E+00_JPRB ,&
&  0.92982E+00_JPRB, 0.96852E+00_JPRB, 0.10201E+01_JPRB, 0.10518E+01_JPRB, 0.10815E+01_JPRB, 0.11304E+01_JPRB, 0.11751E+01_JPRB ,&
&  0.12249E+01_JPRB, 0.12874E+01_JPRB, 0.13445E+01_JPRB, 0.14087E+01_JPRB, 0.14677E+01_JPRB, 0.14561E+01_JPRB, 0.11359E+01_JPRB ,&
&  0.16332E+01_JPRB, 0.18999E+01_JPRB, 0.18415E+01_JPRB, 0.16488E+01_JPRB, 0.15261E+01_JPRB, 0.14839E+01_JPRB, 0.14886E+01_JPRB ,&
&  0.14960E+01_JPRB, 0.14728E+01_JPRB, 0.14565E+01_JPRB, 0.14950E+01_JPRB, 0.17696E+01_JPRB, 0.18057E+01_JPRB, 0.19389E+01_JPRB ,&
&  0.20745E+01_JPRB, 0.21979E+01_JPRB, 0.23045E+01_JPRB, 0.23960E+01_JPRB, 0.24644E+01_JPRB, 0.25499E+01_JPRB, 0.26410E+01_JPRB ,&
&  0.27230E+01_JPRB, 0.27714E+01_JPRB, 0.28604E+01_JPRB, 0.26049E+01_JPRB, 0.24870E+01_JPRB, 0.26683E+01_JPRB, 0.23800E+01_JPRB ,&
&  0.15672E+01_JPRB, 0.11113E+01_JPRB, 0.11022E+01_JPRB, 0.10479E+01_JPRB, 0.93791E+00_JPRB, 0.82707E+00_JPRB, 0.76043E+00_JPRB ,&
&  0.71508E+00_JPRB, 0.67918E+00_JPRB, 0.66057E+00_JPRB, 0.62791E+00_JPRB, 0.61973E+00_JPRB, 0.63299E+00_JPRB, 0.63459E+00_JPRB ,&
&  0.65435E+00_JPRB, 0.62872E+00_JPRB, 0.61986E+00_JPRB, 0.62600E+00_JPRB, 0.66903E+00_JPRB, 0.75896E+00_JPRB, 0.94601E+00_JPRB ,&
&  0.10800E+01_JPRB, 0.67640E+00_JPRB, 0.71181E+00_JPRB, 0.75252E+00_JPRB, 0.81988E+00_JPRB, 0.94555E+00_JPRB, 0.11444E+01_JPRB ,&
&  0.14835E+01_JPRB, 0.19538E+01_JPRB, 0.18928E+01_JPRB, 0.15059E+01_JPRB, 0.19042E+01_JPRB, 0.15993E+01_JPRB, 0.71760E+00_JPRB ,&
&  0.72157E+00_JPRB, 0.70158E+00_JPRB, 0.70722E+00_JPRB, 0.71371E+00_JPRB, 0.71570E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.58342E+00_JPRB, 0.57118E+00_JPRB, 0.57905E+00_JPRB, 0.59721E+00_JPRB, 0.61247E+00_JPRB, 0.64269E+00_JPRB, 0.68017E+00_JPRB ,&
&  0.72761E+00_JPRB, 0.75800E+00_JPRB, 0.76836E+00_JPRB, 0.76999E+00_JPRB, 0.77779E+00_JPRB, 0.78041E+00_JPRB, 0.79326E+00_JPRB ,&
&  0.81811E+00_JPRB, 0.84353E+00_JPRB, 0.87700E+00_JPRB, 0.90565E+00_JPRB, 0.92769E+00_JPRB, 0.97949E+00_JPRB, 0.10375E+01_JPRB ,&
&  0.11091E+01_JPRB, 0.11850E+01_JPRB, 0.12775E+01_JPRB, 0.13779E+01_JPRB, 0.14801E+01_JPRB, 0.15179E+01_JPRB, 0.15195E+01_JPRB ,&
&  0.11976E+01_JPRB, 0.17408E+01_JPRB, 0.18009E+01_JPRB, 0.16710E+01_JPRB, 0.15412E+01_JPRB, 0.15028E+01_JPRB, 0.14643E+01_JPRB ,&
&  0.14271E+01_JPRB, 0.13339E+01_JPRB, 0.13562E+01_JPRB, 0.14394E+01_JPRB, 0.16088E+01_JPRB, 0.16927E+01_JPRB, 0.18135E+01_JPRB ,&
&  0.19296E+01_JPRB, 0.20242E+01_JPRB, 0.21067E+01_JPRB, 0.21882E+01_JPRB, 0.22942E+01_JPRB, 0.24389E+01_JPRB, 0.26270E+01_JPRB ,&
&  0.27880E+01_JPRB, 0.29137E+01_JPRB, 0.30034E+01_JPRB, 0.28179E+01_JPRB, 0.26260E+01_JPRB, 0.29077E+01_JPRB, 0.27757E+01_JPRB ,&
&  0.17578E+01_JPRB, 0.11705E+01_JPRB, 0.11952E+01_JPRB, 0.11930E+01_JPRB, 0.10178E+01_JPRB, 0.90108E+00_JPRB, 0.81133E+00_JPRB ,&
&  0.71296E+00_JPRB, 0.64698E+00_JPRB, 0.65049E+00_JPRB, 0.64731E+00_JPRB, 0.65307E+00_JPRB, 0.65420E+00_JPRB, 0.66573E+00_JPRB ,&
&  0.68291E+00_JPRB, 0.65676E+00_JPRB, 0.63792E+00_JPRB, 0.62725E+00_JPRB, 0.63955E+00_JPRB, 0.66144E+00_JPRB, 0.68731E+00_JPRB ,&
&  0.74476E+00_JPRB, 0.70225E+00_JPRB, 0.71152E+00_JPRB, 0.75465E+00_JPRB, 0.79757E+00_JPRB, 0.84930E+00_JPRB, 0.90258E+00_JPRB ,&
&  0.94639E+00_JPRB, 0.99135E+00_JPRB, 0.10317E+01_JPRB, 0.10346E+01_JPRB, 0.95778E+00_JPRB, 0.87631E+00_JPRB, 0.63770E+00_JPRB ,&
&  0.62339E+00_JPRB, 0.60879E+00_JPRB, 0.60980E+00_JPRB, 0.60700E+00_JPRB, 0.60432E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.51277E+00_JPRB, 0.50506E+00_JPRB, 0.51022E+00_JPRB, 0.51931E+00_JPRB, 0.53603E+00_JPRB, 0.56387E+00_JPRB, 0.60545E+00_JPRB ,&
&  0.65247E+00_JPRB, 0.67827E+00_JPRB, 0.69938E+00_JPRB, 0.68944E+00_JPRB, 0.69093E+00_JPRB, 0.70680E+00_JPRB, 0.71616E+00_JPRB ,&
&  0.74330E+00_JPRB, 0.77195E+00_JPRB, 0.78777E+00_JPRB, 0.79100E+00_JPRB, 0.80095E+00_JPRB, 0.82985E+00_JPRB, 0.87084E+00_JPRB ,&
&  0.92074E+00_JPRB, 0.98826E+00_JPRB, 0.10812E+01_JPRB, 0.12077E+01_JPRB, 0.13468E+01_JPRB, 0.13914E+01_JPRB, 0.13746E+01_JPRB ,&
&  0.71163E+00_JPRB, 0.12562E+01_JPRB, 0.16909E+01_JPRB, 0.16777E+01_JPRB, 0.16061E+01_JPRB, 0.15396E+01_JPRB, 0.14953E+01_JPRB ,&
&  0.13495E+01_JPRB, 0.11961E+01_JPRB, 0.11750E+01_JPRB, 0.13810E+01_JPRB, 0.14096E+01_JPRB, 0.15230E+01_JPRB, 0.16256E+01_JPRB ,&
&  0.17369E+01_JPRB, 0.18310E+01_JPRB, 0.18911E+01_JPRB, 0.19439E+01_JPRB, 0.20352E+01_JPRB, 0.21741E+01_JPRB, 0.23491E+01_JPRB ,&
&  0.25719E+01_JPRB, 0.28449E+01_JPRB, 0.30426E+01_JPRB, 0.29105E+01_JPRB, 0.28435E+01_JPRB, 0.30403E+01_JPRB, 0.37464E+01_JPRB ,&
&  0.23420E+01_JPRB, 0.13818E+01_JPRB, 0.13707E+01_JPRB, 0.13875E+01_JPRB, 0.11514E+01_JPRB, 0.10219E+01_JPRB, 0.92137E+00_JPRB ,&
&  0.79019E+00_JPRB, 0.72693E+00_JPRB, 0.69957E+00_JPRB, 0.68092E+00_JPRB, 0.69350E+00_JPRB, 0.71584E+00_JPRB, 0.75468E+00_JPRB ,&
&  0.76875E+00_JPRB, 0.75465E+00_JPRB, 0.73149E+00_JPRB, 0.71959E+00_JPRB, 0.71390E+00_JPRB, 0.71747E+00_JPRB, 0.71195E+00_JPRB ,&
&  0.72747E+00_JPRB, 0.72491E+00_JPRB, 0.73767E+00_JPRB, 0.77339E+00_JPRB, 0.82049E+00_JPRB, 0.83581E+00_JPRB, 0.83719E+00_JPRB ,&
&  0.80468E+00_JPRB, 0.81639E+00_JPRB, 0.82524E+00_JPRB, 0.80734E+00_JPRB, 0.78318E+00_JPRB, 0.73183E+00_JPRB, 0.63925E+00_JPRB ,&
&  0.59485E+00_JPRB, 0.54844E+00_JPRB, 0.52505E+00_JPRB, 0.51486E+00_JPRB, 0.52451E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.49221E+00_JPRB, 0.48515E+00_JPRB, 0.48406E+00_JPRB, 0.50180E+00_JPRB, 0.49887E+00_JPRB, 0.51277E+00_JPRB, 0.53138E+00_JPRB ,&
&  0.57815E+00_JPRB, 0.62040E+00_JPRB, 0.65708E+00_JPRB, 0.65221E+00_JPRB, 0.65075E+00_JPRB, 0.63683E+00_JPRB, 0.64040E+00_JPRB ,&
&  0.65175E+00_JPRB, 0.67222E+00_JPRB, 0.69475E+00_JPRB, 0.70391E+00_JPRB, 0.71013E+00_JPRB, 0.71822E+00_JPRB, 0.73253E+00_JPRB ,&
&  0.75579E+00_JPRB, 0.78998E+00_JPRB, 0.85158E+00_JPRB, 0.96399E+00_JPRB, 0.11400E+01_JPRB, 0.13154E+01_JPRB, 0.14371E+01_JPRB ,&
&  0.99216E+00_JPRB, 0.46497E+00_JPRB, 0.12366E+01_JPRB, 0.16430E+01_JPRB, 0.16210E+01_JPRB, 0.15400E+01_JPRB, 0.14703E+01_JPRB ,&
&  0.12807E+01_JPRB, 0.11832E+01_JPRB, 0.10621E+01_JPRB, 0.11736E+01_JPRB, 0.11386E+01_JPRB, 0.12194E+01_JPRB, 0.13420E+01_JPRB ,&
&  0.14725E+01_JPRB, 0.15921E+01_JPRB, 0.17020E+01_JPRB, 0.17489E+01_JPRB, 0.17704E+01_JPRB, 0.18278E+01_JPRB, 0.19467E+01_JPRB ,&
&  0.21604E+01_JPRB, 0.25109E+01_JPRB, 0.29599E+01_JPRB, 0.31472E+01_JPRB, 0.36475E+01_JPRB, 0.34536E+01_JPRB, 0.37303E+01_JPRB ,&
&  0.24558E+01_JPRB, 0.16730E+01_JPRB, 0.15682E+01_JPRB, 0.15895E+01_JPRB, 0.13253E+01_JPRB, 0.10901E+01_JPRB, 0.10749E+01_JPRB ,&
&  0.89129E+00_JPRB, 0.80810E+00_JPRB, 0.78447E+00_JPRB, 0.75732E+00_JPRB, 0.73610E+00_JPRB, 0.76133E+00_JPRB, 0.80396E+00_JPRB ,&
&  0.83749E+00_JPRB, 0.84305E+00_JPRB, 0.82659E+00_JPRB, 0.81124E+00_JPRB, 0.80023E+00_JPRB, 0.78160E+00_JPRB, 0.76604E+00_JPRB ,&
&  0.77637E+00_JPRB, 0.79031E+00_JPRB, 0.80926E+00_JPRB, 0.82889E+00_JPRB, 0.85170E+00_JPRB, 0.83970E+00_JPRB, 0.82974E+00_JPRB ,&
&  0.82281E+00_JPRB, 0.83141E+00_JPRB, 0.81245E+00_JPRB, 0.80856E+00_JPRB, 0.79354E+00_JPRB, 0.75619E+00_JPRB, 0.72682E+00_JPRB ,&
&  0.67205E+00_JPRB, 0.58677E+00_JPRB, 0.52990E+00_JPRB, 0.51318E+00_JPRB, 0.49805E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.51628E+00_JPRB, 0.50497E+00_JPRB, 0.50254E+00_JPRB, 0.51196E+00_JPRB, 0.52012E+00_JPRB, 0.52832E+00_JPRB, 0.51471E+00_JPRB ,&
&  0.52893E+00_JPRB, 0.56431E+00_JPRB, 0.58457E+00_JPRB, 0.60430E+00_JPRB, 0.59942E+00_JPRB, 0.58782E+00_JPRB, 0.58109E+00_JPRB ,&
&  0.57823E+00_JPRB, 0.58166E+00_JPRB, 0.59586E+00_JPRB, 0.60307E+00_JPRB, 0.60779E+00_JPRB, 0.61519E+00_JPRB, 0.61970E+00_JPRB ,&
&  0.62569E+00_JPRB, 0.63939E+00_JPRB, 0.66818E+00_JPRB, 0.72467E+00_JPRB, 0.82328E+00_JPRB, 0.99036E+00_JPRB, 0.12227E+01_JPRB ,&
&  0.16131E+01_JPRB, 0.11631E+01_JPRB, 0.58540E+00_JPRB, 0.12985E+01_JPRB, 0.16660E+01_JPRB, 0.16588E+01_JPRB, 0.14197E+01_JPRB ,&
&  0.13240E+01_JPRB, 0.11608E+01_JPRB, 0.10603E+01_JPRB, 0.11055E+01_JPRB, 0.96442E+00_JPRB, 0.93068E+00_JPRB, 0.97262E+00_JPRB ,&
&  0.10705E+01_JPRB, 0.12199E+01_JPRB, 0.13811E+01_JPRB, 0.14938E+01_JPRB, 0.15557E+01_JPRB, 0.15843E+01_JPRB, 0.16312E+01_JPRB ,&
&  0.17724E+01_JPRB, 0.19934E+01_JPRB, 0.23879E+01_JPRB, 0.30659E+01_JPRB, 0.44448E+01_JPRB, 0.41690E+01_JPRB, 0.41475E+01_JPRB ,&
&  0.26102E+01_JPRB, 0.21971E+01_JPRB, 0.20322E+01_JPRB, 0.18508E+01_JPRB, 0.14857E+01_JPRB, 0.12866E+01_JPRB, 0.12898E+01_JPRB ,&
&  0.10498E+01_JPRB, 0.90190E+00_JPRB, 0.85102E+00_JPRB, 0.80804E+00_JPRB, 0.78384E+00_JPRB, 0.79263E+00_JPRB, 0.82419E+00_JPRB ,&
&  0.86167E+00_JPRB, 0.86334E+00_JPRB, 0.84917E+00_JPRB, 0.83495E+00_JPRB, 0.82621E+00_JPRB, 0.81703E+00_JPRB, 0.80590E+00_JPRB ,&
&  0.80487E+00_JPRB, 0.81998E+00_JPRB, 0.84393E+00_JPRB, 0.84183E+00_JPRB, 0.82088E+00_JPRB, 0.81076E+00_JPRB, 0.83875E+00_JPRB ,&
&  0.82691E+00_JPRB, 0.84419E+00_JPRB, 0.82840E+00_JPRB, 0.83222E+00_JPRB, 0.80823E+00_JPRB, 0.79261E+00_JPRB, 0.77921E+00_JPRB ,&
&  0.75146E+00_JPRB, 0.68193E+00_JPRB, 0.60492E+00_JPRB, 0.52624E+00_JPRB, 0.51572E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.57138E+00_JPRB, 0.55503E+00_JPRB, 0.55709E+00_JPRB, 0.55100E+00_JPRB, 0.54089E+00_JPRB, 0.53882E+00_JPRB, 0.53286E+00_JPRB ,&
&  0.52432E+00_JPRB, 0.52049E+00_JPRB, 0.51559E+00_JPRB, 0.54344E+00_JPRB, 0.55693E+00_JPRB, 0.54710E+00_JPRB, 0.53737E+00_JPRB ,&
&  0.53025E+00_JPRB, 0.53063E+00_JPRB, 0.53487E+00_JPRB, 0.53848E+00_JPRB, 0.54019E+00_JPRB, 0.54575E+00_JPRB, 0.55078E+00_JPRB ,&
&  0.54754E+00_JPRB, 0.54863E+00_JPRB, 0.55805E+00_JPRB, 0.58057E+00_JPRB, 0.62516E+00_JPRB, 0.70195E+00_JPRB, 0.83710E+00_JPRB ,&
&  0.11450E+01_JPRB, 0.11678E+01_JPRB, 0.57548E+00_JPRB, 0.10461E+01_JPRB, 0.16775E+01_JPRB, 0.17369E+01_JPRB, 0.16307E+01_JPRB ,&
&  0.14684E+01_JPRB, 0.11691E+01_JPRB, 0.11051E+01_JPRB, 0.11191E+01_JPRB, 0.97458E+00_JPRB, 0.86772E+00_JPRB, 0.80521E+00_JPRB ,&
&  0.78622E+00_JPRB, 0.83398E+00_JPRB, 0.95441E+00_JPRB, 0.10958E+01_JPRB, 0.12236E+01_JPRB, 0.13029E+01_JPRB, 0.13452E+01_JPRB ,&
&  0.14055E+01_JPRB, 0.15277E+01_JPRB, 0.17829E+01_JPRB, 0.21255E+01_JPRB, 0.36095E+01_JPRB, 0.44251E+01_JPRB, 0.46699E+01_JPRB ,&
&  0.34508E+01_JPRB, 0.29907E+01_JPRB, 0.25969E+01_JPRB, 0.22002E+01_JPRB, 0.17280E+01_JPRB, 0.15766E+01_JPRB, 0.15567E+01_JPRB ,&
&  0.12846E+01_JPRB, 0.10599E+01_JPRB, 0.96213E+00_JPRB, 0.89264E+00_JPRB, 0.84188E+00_JPRB, 0.82064E+00_JPRB, 0.81420E+00_JPRB ,&
&  0.83000E+00_JPRB, 0.83995E+00_JPRB, 0.82888E+00_JPRB, 0.81742E+00_JPRB, 0.81755E+00_JPRB, 0.81782E+00_JPRB, 0.81860E+00_JPRB ,&
&  0.81294E+00_JPRB, 0.82576E+00_JPRB, 0.84714E+00_JPRB, 0.84235E+00_JPRB, 0.84913E+00_JPRB, 0.86528E+00_JPRB, 0.83892E+00_JPRB ,&
&  0.81138E+00_JPRB, 0.81960E+00_JPRB, 0.81069E+00_JPRB, 0.78693E+00_JPRB, 0.82617E+00_JPRB, 0.82263E+00_JPRB, 0.81951E+00_JPRB ,&
&  0.86674E+00_JPRB, 0.98271E+00_JPRB, 0.94457E+00_JPRB, 0.63776E+00_JPRB, 0.58610E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.60815E+00_JPRB, 0.58336E+00_JPRB, 0.57879E+00_JPRB, 0.57395E+00_JPRB, 0.56200E+00_JPRB, 0.55331E+00_JPRB, 0.54861E+00_JPRB ,&
&  0.53710E+00_JPRB, 0.51662E+00_JPRB, 0.49852E+00_JPRB, 0.49994E+00_JPRB, 0.51219E+00_JPRB, 0.52161E+00_JPRB, 0.51274E+00_JPRB ,&
&  0.50149E+00_JPRB, 0.49758E+00_JPRB, 0.50586E+00_JPRB, 0.51018E+00_JPRB, 0.50853E+00_JPRB, 0.51068E+00_JPRB, 0.50799E+00_JPRB ,&
&  0.50590E+00_JPRB, 0.50171E+00_JPRB, 0.50299E+00_JPRB, 0.51421E+00_JPRB, 0.53485E+00_JPRB, 0.57239E+00_JPRB, 0.62929E+00_JPRB ,&
&  0.81480E+00_JPRB, 0.96347E+00_JPRB, 0.11813E+01_JPRB, 0.18236E+01_JPRB, 0.22179E+01_JPRB, 0.20736E+01_JPRB, 0.18953E+01_JPRB ,&
&  0.17296E+01_JPRB, 0.14593E+01_JPRB, 0.13322E+01_JPRB, 0.11900E+01_JPRB, 0.10174E+01_JPRB, 0.88806E+00_JPRB, 0.80167E+00_JPRB ,&
&  0.75292E+00_JPRB, 0.72603E+00_JPRB, 0.73551E+00_JPRB, 0.78575E+00_JPRB, 0.86517E+00_JPRB, 0.95419E+00_JPRB, 0.10183E+01_JPRB ,&
&  0.10729E+01_JPRB, 0.11607E+01_JPRB, 0.13323E+01_JPRB, 0.15480E+01_JPRB, 0.23848E+01_JPRB, 0.37795E+01_JPRB, 0.45111E+01_JPRB ,&
&  0.45039E+01_JPRB, 0.40132E+01_JPRB, 0.33107E+01_JPRB, 0.26672E+01_JPRB, 0.19762E+01_JPRB, 0.20040E+01_JPRB, 0.17830E+01_JPRB ,&
&  0.15128E+01_JPRB, 0.12949E+01_JPRB, 0.11360E+01_JPRB, 0.10215E+01_JPRB, 0.93120E+00_JPRB, 0.86054E+00_JPRB, 0.82277E+00_JPRB ,&
&  0.81036E+00_JPRB, 0.81236E+00_JPRB, 0.80219E+00_JPRB, 0.78646E+00_JPRB, 0.79284E+00_JPRB, 0.80068E+00_JPRB, 0.80839E+00_JPRB ,&
&  0.80767E+00_JPRB, 0.81443E+00_JPRB, 0.79947E+00_JPRB, 0.81833E+00_JPRB, 0.84442E+00_JPRB, 0.84051E+00_JPRB, 0.81361E+00_JPRB ,&
&  0.77774E+00_JPRB, 0.78711E+00_JPRB, 0.79301E+00_JPRB, 0.78840E+00_JPRB, 0.80686E+00_JPRB, 0.82318E+00_JPRB, 0.81967E+00_JPRB ,&
&  0.83769E+00_JPRB, 0.81692E+00_JPRB, 0.76794E+00_JPRB, 0.67607E+00_JPRB, 0.63814E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.64991E+00_JPRB, 0.62321E+00_JPRB, 0.60507E+00_JPRB, 0.59234E+00_JPRB, 0.58637E+00_JPRB, 0.58635E+00_JPRB, 0.58087E+00_JPRB ,&
&  0.57136E+00_JPRB, 0.55174E+00_JPRB, 0.52594E+00_JPRB, 0.50358E+00_JPRB, 0.50338E+00_JPRB, 0.50220E+00_JPRB, 0.50140E+00_JPRB ,&
&  0.49186E+00_JPRB, 0.48790E+00_JPRB, 0.49022E+00_JPRB, 0.48960E+00_JPRB, 0.48680E+00_JPRB, 0.48298E+00_JPRB, 0.48091E+00_JPRB ,&
&  0.48429E+00_JPRB, 0.48472E+00_JPRB, 0.48377E+00_JPRB, 0.48669E+00_JPRB, 0.49626E+00_JPRB, 0.52006E+00_JPRB, 0.55630E+00_JPRB ,&
&  0.66205E+00_JPRB, 0.88467E+00_JPRB, 0.10701E+01_JPRB, 0.20562E+01_JPRB, 0.23808E+01_JPRB, 0.22878E+01_JPRB, 0.19927E+01_JPRB ,&
&  0.17359E+01_JPRB, 0.17057E+01_JPRB, 0.14543E+01_JPRB, 0.12137E+01_JPRB, 0.10012E+01_JPRB, 0.88036E+00_JPRB, 0.79302E+00_JPRB ,&
&  0.73176E+00_JPRB, 0.69822E+00_JPRB, 0.68791E+00_JPRB, 0.68888E+00_JPRB, 0.69575E+00_JPRB, 0.72851E+00_JPRB, 0.76604E+00_JPRB ,&
&  0.80914E+00_JPRB, 0.90537E+00_JPRB, 0.10809E+01_JPRB, 0.12402E+01_JPRB, 0.17428E+01_JPRB, 0.28336E+01_JPRB, 0.37887E+01_JPRB ,&
&  0.42581E+01_JPRB, 0.40327E+01_JPRB, 0.35548E+01_JPRB, 0.29076E+01_JPRB, 0.23515E+01_JPRB, 0.21363E+01_JPRB, 0.19026E+01_JPRB ,&
&  0.16670E+01_JPRB, 0.15082E+01_JPRB, 0.12991E+01_JPRB, 0.11494E+01_JPRB, 0.10694E+01_JPRB, 0.96675E+00_JPRB, 0.87954E+00_JPRB ,&
&  0.82668E+00_JPRB, 0.80206E+00_JPRB, 0.78695E+00_JPRB, 0.77096E+00_JPRB, 0.76731E+00_JPRB, 0.77669E+00_JPRB, 0.78650E+00_JPRB ,&
&  0.78681E+00_JPRB, 0.79431E+00_JPRB, 0.76435E+00_JPRB, 0.76326E+00_JPRB, 0.78993E+00_JPRB, 0.79407E+00_JPRB, 0.77396E+00_JPRB ,&
&  0.75757E+00_JPRB, 0.75744E+00_JPRB, 0.76290E+00_JPRB, 0.75639E+00_JPRB, 0.75609E+00_JPRB, 0.82738E+00_JPRB, 0.82662E+00_JPRB ,&
&  0.82989E+00_JPRB, 0.77102E+00_JPRB, 0.73771E+00_JPRB, 0.69846E+00_JPRB, 0.67580E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.66707E+00_JPRB, 0.64970E+00_JPRB, 0.63508E+00_JPRB, 0.61968E+00_JPRB, 0.61022E+00_JPRB, 0.60565E+00_JPRB, 0.59844E+00_JPRB ,&
&  0.58509E+00_JPRB, 0.56568E+00_JPRB, 0.54465E+00_JPRB, 0.51616E+00_JPRB, 0.49973E+00_JPRB, 0.49682E+00_JPRB, 0.50300E+00_JPRB ,&
&  0.49565E+00_JPRB, 0.48130E+00_JPRB, 0.48127E+00_JPRB, 0.48368E+00_JPRB, 0.48197E+00_JPRB, 0.47472E+00_JPRB, 0.46645E+00_JPRB ,&
&  0.46889E+00_JPRB, 0.47256E+00_JPRB, 0.47538E+00_JPRB, 0.47792E+00_JPRB, 0.48184E+00_JPRB, 0.49286E+00_JPRB, 0.52105E+00_JPRB ,&
&  0.60555E+00_JPRB, 0.10352E+01_JPRB, 0.10993E+01_JPRB, 0.18944E+01_JPRB, 0.20485E+01_JPRB, 0.20045E+01_JPRB, 0.18134E+01_JPRB ,&
&  0.16264E+01_JPRB, 0.15116E+01_JPRB, 0.12904E+01_JPRB, 0.11222E+01_JPRB, 0.95865E+00_JPRB, 0.83688E+00_JPRB, 0.75957E+00_JPRB ,&
&  0.70736E+00_JPRB, 0.66759E+00_JPRB, 0.64410E+00_JPRB, 0.63718E+00_JPRB, 0.63384E+00_JPRB, 0.63804E+00_JPRB, 0.65041E+00_JPRB ,&
&  0.67260E+00_JPRB, 0.73821E+00_JPRB, 0.90751E+00_JPRB, 0.12812E+01_JPRB, 0.15789E+01_JPRB, 0.23449E+01_JPRB, 0.34997E+01_JPRB ,&
&  0.41474E+01_JPRB, 0.40166E+01_JPRB, 0.33612E+01_JPRB, 0.27829E+01_JPRB, 0.23513E+01_JPRB, 0.20597E+01_JPRB, 0.18156E+01_JPRB ,&
&  0.16495E+01_JPRB, 0.15279E+01_JPRB, 0.14021E+01_JPRB, 0.12486E+01_JPRB, 0.11395E+01_JPRB, 0.10580E+01_JPRB, 0.97255E+00_JPRB ,&
&  0.88671E+00_JPRB, 0.81997E+00_JPRB, 0.78281E+00_JPRB, 0.75509E+00_JPRB, 0.73903E+00_JPRB, 0.74337E+00_JPRB, 0.75214E+00_JPRB ,&
&  0.75076E+00_JPRB, 0.75681E+00_JPRB, 0.73100E+00_JPRB, 0.72616E+00_JPRB, 0.74800E+00_JPRB, 0.75586E+00_JPRB, 0.73747E+00_JPRB ,&
&  0.73655E+00_JPRB, 0.74170E+00_JPRB, 0.74850E+00_JPRB, 0.76369E+00_JPRB, 0.75037E+00_JPRB, 0.82845E+00_JPRB, 0.81389E+00_JPRB ,&
&  0.81237E+00_JPRB, 0.76573E+00_JPRB, 0.72766E+00_JPRB, 0.71330E+00_JPRB, 0.69352E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.67093E+00_JPRB, 0.65511E+00_JPRB, 0.64307E+00_JPRB, 0.63592E+00_JPRB, 0.63065E+00_JPRB, 0.61917E+00_JPRB, 0.60666E+00_JPRB ,&
&  0.59454E+00_JPRB, 0.58037E+00_JPRB, 0.55798E+00_JPRB, 0.52590E+00_JPRB, 0.50130E+00_JPRB, 0.49098E+00_JPRB, 0.49429E+00_JPRB ,&
&  0.49511E+00_JPRB, 0.48675E+00_JPRB, 0.48091E+00_JPRB, 0.48009E+00_JPRB, 0.47909E+00_JPRB, 0.47194E+00_JPRB, 0.46267E+00_JPRB ,&
&  0.45559E+00_JPRB, 0.45638E+00_JPRB, 0.45982E+00_JPRB, 0.46505E+00_JPRB, 0.46969E+00_JPRB, 0.47658E+00_JPRB, 0.49611E+00_JPRB ,&
&  0.57359E+00_JPRB, 0.57973E+00_JPRB, 0.12423E+01_JPRB, 0.16194E+01_JPRB, 0.16506E+01_JPRB, 0.15984E+01_JPRB, 0.15743E+01_JPRB ,&
&  0.14630E+01_JPRB, 0.12983E+01_JPRB, 0.11116E+01_JPRB, 0.95641E+00_JPRB, 0.84097E+00_JPRB, 0.76492E+00_JPRB, 0.70869E+00_JPRB ,&
&  0.66545E+00_JPRB, 0.63212E+00_JPRB, 0.60479E+00_JPRB, 0.59111E+00_JPRB, 0.59377E+00_JPRB, 0.60018E+00_JPRB, 0.60725E+00_JPRB ,&
&  0.61944E+00_JPRB, 0.64762E+00_JPRB, 0.74076E+00_JPRB, 0.10346E+01_JPRB, 0.16848E+01_JPRB, 0.22632E+01_JPRB, 0.32187E+01_JPRB ,&
&  0.40552E+01_JPRB, 0.36357E+01_JPRB, 0.29218E+01_JPRB, 0.24277E+01_JPRB, 0.20750E+01_JPRB, 0.18304E+01_JPRB, 0.16462E+01_JPRB ,&
&  0.15064E+01_JPRB, 0.14068E+01_JPRB, 0.13252E+01_JPRB, 0.12336E+01_JPRB, 0.11563E+01_JPRB, 0.10963E+01_JPRB, 0.10335E+01_JPRB ,&
&  0.96131E+00_JPRB, 0.89058E+00_JPRB, 0.82660E+00_JPRB, 0.77149E+00_JPRB, 0.72633E+00_JPRB, 0.71389E+00_JPRB, 0.71330E+00_JPRB ,&
&  0.70875E+00_JPRB, 0.71217E+00_JPRB, 0.70524E+00_JPRB, 0.71037E+00_JPRB, 0.73592E+00_JPRB, 0.74132E+00_JPRB, 0.73493E+00_JPRB ,&
&  0.71552E+00_JPRB, 0.72590E+00_JPRB, 0.72951E+00_JPRB, 0.73255E+00_JPRB, 0.70531E+00_JPRB, 0.79191E+00_JPRB, 0.78189E+00_JPRB ,&
&  0.77788E+00_JPRB, 0.76449E+00_JPRB, 0.72713E+00_JPRB, 0.71228E+00_JPRB, 0.69459E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.67087E+00_JPRB, 0.65089E+00_JPRB, 0.64012E+00_JPRB, 0.63328E+00_JPRB, 0.62850E+00_JPRB, 0.62399E+00_JPRB, 0.61189E+00_JPRB ,&
&  0.59573E+00_JPRB, 0.58616E+00_JPRB, 0.57300E+00_JPRB, 0.55476E+00_JPRB, 0.52825E+00_JPRB, 0.50913E+00_JPRB, 0.50474E+00_JPRB ,&
&  0.50147E+00_JPRB, 0.49549E+00_JPRB, 0.48491E+00_JPRB, 0.47924E+00_JPRB, 0.47251E+00_JPRB, 0.46398E+00_JPRB, 0.45897E+00_JPRB ,&
&  0.44918E+00_JPRB, 0.44490E+00_JPRB, 0.44720E+00_JPRB, 0.45344E+00_JPRB, 0.45934E+00_JPRB, 0.46715E+00_JPRB, 0.47703E+00_JPRB ,&
&  0.53537E+00_JPRB, 0.78559E+00_JPRB, 0.11131E+01_JPRB, 0.13002E+01_JPRB, 0.13276E+01_JPRB, 0.12568E+01_JPRB, 0.12558E+01_JPRB ,&
&  0.11394E+01_JPRB, 0.10317E+01_JPRB, 0.91691E+00_JPRB, 0.80877E+00_JPRB, 0.72642E+00_JPRB, 0.68386E+00_JPRB, 0.65954E+00_JPRB ,&
&  0.63023E+00_JPRB, 0.60421E+00_JPRB, 0.58025E+00_JPRB, 0.55998E+00_JPRB, 0.55294E+00_JPRB, 0.56232E+00_JPRB, 0.57653E+00_JPRB ,&
&  0.58316E+00_JPRB, 0.59459E+00_JPRB, 0.64127E+00_JPRB, 0.82298E+00_JPRB, 0.13700E+01_JPRB, 0.16739E+01_JPRB, 0.25624E+01_JPRB ,&
&  0.30450E+01_JPRB, 0.27094E+01_JPRB, 0.22456E+01_JPRB, 0.19118E+01_JPRB, 0.16473E+01_JPRB, 0.14489E+01_JPRB, 0.13058E+01_JPRB ,&
&  0.12237E+01_JPRB, 0.11667E+01_JPRB, 0.11180E+01_JPRB, 0.10711E+01_JPRB, 0.10279E+01_JPRB, 0.99434E+00_JPRB, 0.96665E+00_JPRB ,&
&  0.93457E+00_JPRB, 0.89431E+00_JPRB, 0.84925E+00_JPRB, 0.79736E+00_JPRB, 0.74480E+00_JPRB, 0.71137E+00_JPRB, 0.69287E+00_JPRB ,&
&  0.67819E+00_JPRB, 0.67828E+00_JPRB, 0.68230E+00_JPRB, 0.69259E+00_JPRB, 0.71787E+00_JPRB, 0.73081E+00_JPRB, 0.73092E+00_JPRB ,&
&  0.70123E+00_JPRB, 0.69252E+00_JPRB, 0.69047E+00_JPRB, 0.67149E+00_JPRB, 0.67671E+00_JPRB, 0.74327E+00_JPRB, 0.72587E+00_JPRB ,&
&  0.71365E+00_JPRB, 0.70952E+00_JPRB, 0.70009E+00_JPRB, 0.68964E+00_JPRB, 0.69592E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.68583E+00_JPRB, 0.65637E+00_JPRB, 0.63894E+00_JPRB, 0.62852E+00_JPRB, 0.62372E+00_JPRB, 0.61910E+00_JPRB, 0.60798E+00_JPRB ,&
&  0.59347E+00_JPRB, 0.57929E+00_JPRB, 0.56763E+00_JPRB, 0.55925E+00_JPRB, 0.54140E+00_JPRB, 0.51223E+00_JPRB, 0.48972E+00_JPRB ,&
&  0.48337E+00_JPRB, 0.47795E+00_JPRB, 0.47350E+00_JPRB, 0.46512E+00_JPRB, 0.45779E+00_JPRB, 0.45474E+00_JPRB, 0.45137E+00_JPRB ,&
&  0.44724E+00_JPRB, 0.44303E+00_JPRB, 0.44425E+00_JPRB, 0.44869E+00_JPRB, 0.45307E+00_JPRB, 0.45716E+00_JPRB, 0.46287E+00_JPRB ,&
&  0.49136E+00_JPRB, 0.63410E+00_JPRB, 0.77919E+00_JPRB, 0.97803E+00_JPRB, 0.10338E+01_JPRB, 0.99021E+00_JPRB, 0.95975E+00_JPRB ,&
&  0.90604E+00_JPRB, 0.84394E+00_JPRB, 0.78370E+00_JPRB, 0.72511E+00_JPRB, 0.66039E+00_JPRB, 0.62439E+00_JPRB, 0.60791E+00_JPRB ,&
&  0.59543E+00_JPRB, 0.57757E+00_JPRB, 0.55779E+00_JPRB, 0.53714E+00_JPRB, 0.52318E+00_JPRB, 0.52653E+00_JPRB, 0.53682E+00_JPRB ,&
&  0.53547E+00_JPRB, 0.53724E+00_JPRB, 0.54913E+00_JPRB, 0.62066E+00_JPRB, 0.89943E+00_JPRB, 0.11747E+01_JPRB, 0.15648E+01_JPRB ,&
&  0.15987E+01_JPRB, 0.16791E+01_JPRB, 0.15314E+01_JPRB, 0.13694E+01_JPRB, 0.12328E+01_JPRB, 0.11277E+01_JPRB, 0.10482E+01_JPRB ,&
&  0.99366E+00_JPRB, 0.94972E+00_JPRB, 0.91714E+00_JPRB, 0.89277E+00_JPRB, 0.87279E+00_JPRB, 0.84898E+00_JPRB, 0.83666E+00_JPRB ,&
&  0.82984E+00_JPRB, 0.81227E+00_JPRB, 0.79079E+00_JPRB, 0.76618E+00_JPRB, 0.73561E+00_JPRB, 0.70924E+00_JPRB, 0.68947E+00_JPRB ,&
&  0.67148E+00_JPRB, 0.65397E+00_JPRB, 0.65191E+00_JPRB, 0.66281E+00_JPRB, 0.67340E+00_JPRB, 0.68148E+00_JPRB, 0.67933E+00_JPRB ,&
&  0.67268E+00_JPRB, 0.66771E+00_JPRB, 0.65800E+00_JPRB, 0.64058E+00_JPRB, 0.64177E+00_JPRB, 0.68705E+00_JPRB, 0.67694E+00_JPRB ,&
&  0.66204E+00_JPRB, 0.65228E+00_JPRB, 0.65353E+00_JPRB, 0.65900E+00_JPRB, 0.71203E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.71713E+00_JPRB, 0.68356E+00_JPRB, 0.64967E+00_JPRB, 0.62874E+00_JPRB, 0.61496E+00_JPRB, 0.60381E+00_JPRB, 0.58935E+00_JPRB ,&
&  0.57576E+00_JPRB, 0.56159E+00_JPRB, 0.54975E+00_JPRB, 0.54110E+00_JPRB, 0.53162E+00_JPRB, 0.51134E+00_JPRB, 0.48376E+00_JPRB ,&
&  0.46473E+00_JPRB, 0.45926E+00_JPRB, 0.45555E+00_JPRB, 0.44861E+00_JPRB, 0.44243E+00_JPRB, 0.44066E+00_JPRB, 0.43848E+00_JPRB ,&
&  0.43890E+00_JPRB, 0.43982E+00_JPRB, 0.44343E+00_JPRB, 0.44549E+00_JPRB, 0.44388E+00_JPRB, 0.45038E+00_JPRB, 0.45033E+00_JPRB ,&
&  0.47413E+00_JPRB, 0.50592E+00_JPRB, 0.60136E+00_JPRB, 0.70899E+00_JPRB, 0.78054E+00_JPRB, 0.80864E+00_JPRB, 0.80055E+00_JPRB ,&
&  0.78148E+00_JPRB, 0.75917E+00_JPRB, 0.72460E+00_JPRB, 0.68737E+00_JPRB, 0.64466E+00_JPRB, 0.59810E+00_JPRB, 0.57024E+00_JPRB ,&
&  0.56216E+00_JPRB, 0.55239E+00_JPRB, 0.54126E+00_JPRB, 0.52695E+00_JPRB, 0.51280E+00_JPRB, 0.50863E+00_JPRB, 0.51023E+00_JPRB ,&
&  0.50652E+00_JPRB, 0.50266E+00_JPRB, 0.49936E+00_JPRB, 0.50564E+00_JPRB, 0.55263E+00_JPRB, 0.68645E+00_JPRB, 0.86489E+00_JPRB ,&
&  0.93578E+00_JPRB, 0.98665E+00_JPRB, 0.10119E+01_JPRB, 0.10026E+01_JPRB, 0.96124E+00_JPRB, 0.90222E+00_JPRB, 0.84004E+00_JPRB ,&
&  0.79437E+00_JPRB, 0.76286E+00_JPRB, 0.74646E+00_JPRB, 0.74057E+00_JPRB, 0.73828E+00_JPRB, 0.73214E+00_JPRB, 0.72009E+00_JPRB ,&
&  0.71044E+00_JPRB, 0.70583E+00_JPRB, 0.70550E+00_JPRB, 0.70048E+00_JPRB, 0.69323E+00_JPRB, 0.68431E+00_JPRB, 0.67802E+00_JPRB ,&
&  0.67538E+00_JPRB, 0.66800E+00_JPRB, 0.65262E+00_JPRB, 0.63978E+00_JPRB, 0.64152E+00_JPRB, 0.64296E+00_JPRB, 0.64055E+00_JPRB ,&
&  0.63683E+00_JPRB, 0.64007E+00_JPRB, 0.62862E+00_JPRB, 0.63420E+00_JPRB, 0.66330E+00_JPRB, 0.64904E+00_JPRB, 0.64133E+00_JPRB ,&
&  0.62331E+00_JPRB, 0.61301E+00_JPRB, 0.61611E+00_JPRB, 0.63304E+00_JPRB, 0.70502E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.60401E+00_JPRB, 0.59940E+00_JPRB, 0.59165E+00_JPRB, 0.58464E+00_JPRB, 0.57390E+00_JPRB, 0.56391E+00_JPRB, 0.55083E+00_JPRB ,&
&  0.53873E+00_JPRB, 0.52680E+00_JPRB, 0.51634E+00_JPRB, 0.51287E+00_JPRB, 0.50598E+00_JPRB, 0.49570E+00_JPRB, 0.47622E+00_JPRB ,&
&  0.45980E+00_JPRB, 0.44703E+00_JPRB, 0.43923E+00_JPRB, 0.43589E+00_JPRB, 0.43378E+00_JPRB, 0.43081E+00_JPRB, 0.42977E+00_JPRB ,&
&  0.43096E+00_JPRB, 0.43104E+00_JPRB, 0.43381E+00_JPRB, 0.43382E+00_JPRB, 0.43715E+00_JPRB, 0.43327E+00_JPRB, 0.43073E+00_JPRB ,&
&  0.44928E+00_JPRB, 0.44871E+00_JPRB, 0.50289E+00_JPRB, 0.56403E+00_JPRB, 0.62291E+00_JPRB, 0.66335E+00_JPRB, 0.69982E+00_JPRB ,&
&  0.72430E+00_JPRB, 0.71327E+00_JPRB, 0.68773E+00_JPRB, 0.64805E+00_JPRB, 0.61698E+00_JPRB, 0.58280E+00_JPRB, 0.54844E+00_JPRB ,&
&  0.53130E+00_JPRB, 0.52307E+00_JPRB, 0.51612E+00_JPRB, 0.50980E+00_JPRB, 0.50293E+00_JPRB, 0.49909E+00_JPRB, 0.49759E+00_JPRB ,&
&  0.49181E+00_JPRB, 0.48455E+00_JPRB, 0.47574E+00_JPRB, 0.47292E+00_JPRB, 0.47493E+00_JPRB, 0.50688E+00_JPRB, 0.59023E+00_JPRB ,&
&  0.63311E+00_JPRB, 0.65649E+00_JPRB, 0.67697E+00_JPRB, 0.70774E+00_JPRB, 0.72848E+00_JPRB, 0.72836E+00_JPRB, 0.69511E+00_JPRB ,&
&  0.66313E+00_JPRB, 0.63922E+00_JPRB, 0.62604E+00_JPRB, 0.62070E+00_JPRB, 0.62162E+00_JPRB, 0.62269E+00_JPRB, 0.62429E+00_JPRB ,&
&  0.61867E+00_JPRB, 0.61249E+00_JPRB, 0.61105E+00_JPRB, 0.61242E+00_JPRB, 0.61638E+00_JPRB, 0.61986E+00_JPRB, 0.62549E+00_JPRB ,&
&  0.63623E+00_JPRB, 0.64982E+00_JPRB, 0.66057E+00_JPRB, 0.65089E+00_JPRB, 0.63232E+00_JPRB, 0.61897E+00_JPRB, 0.61068E+00_JPRB ,&
&  0.60499E+00_JPRB, 0.60676E+00_JPRB, 0.61218E+00_JPRB, 0.62831E+00_JPRB, 0.63159E+00_JPRB, 0.62168E+00_JPRB, 0.60584E+00_JPRB ,&
&  0.59137E+00_JPRB, 0.58195E+00_JPRB, 0.58212E+00_JPRB, 0.59107E+00_JPRB, 0.57994E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.54169E+00_JPRB, 0.54049E+00_JPRB, 0.53686E+00_JPRB, 0.53418E+00_JPRB, 0.52790E+00_JPRB, 0.52077E+00_JPRB, 0.51343E+00_JPRB ,&
&  0.50393E+00_JPRB, 0.49455E+00_JPRB, 0.48639E+00_JPRB, 0.48381E+00_JPRB, 0.48176E+00_JPRB, 0.47949E+00_JPRB, 0.47259E+00_JPRB ,&
&  0.46228E+00_JPRB, 0.45222E+00_JPRB, 0.44317E+00_JPRB, 0.43743E+00_JPRB, 0.43554E+00_JPRB, 0.43313E+00_JPRB, 0.42982E+00_JPRB ,&
&  0.42787E+00_JPRB, 0.42869E+00_JPRB, 0.42817E+00_JPRB, 0.42735E+00_JPRB, 0.42592E+00_JPRB, 0.42072E+00_JPRB, 0.42069E+00_JPRB ,&
&  0.43196E+00_JPRB, 0.39641E+00_JPRB, 0.42644E+00_JPRB, 0.45906E+00_JPRB, 0.48545E+00_JPRB, 0.52862E+00_JPRB, 0.58608E+00_JPRB ,&
&  0.63812E+00_JPRB, 0.65179E+00_JPRB, 0.64310E+00_JPRB, 0.61452E+00_JPRB, 0.59229E+00_JPRB, 0.57543E+00_JPRB, 0.55034E+00_JPRB ,&
&  0.52541E+00_JPRB, 0.50896E+00_JPRB, 0.49771E+00_JPRB, 0.48868E+00_JPRB, 0.48237E+00_JPRB, 0.47695E+00_JPRB, 0.47214E+00_JPRB ,&
&  0.46530E+00_JPRB, 0.45829E+00_JPRB, 0.45202E+00_JPRB, 0.44889E+00_JPRB, 0.45048E+00_JPRB, 0.45708E+00_JPRB, 0.48325E+00_JPRB ,&
&  0.50886E+00_JPRB, 0.52444E+00_JPRB, 0.52869E+00_JPRB, 0.54256E+00_JPRB, 0.56339E+00_JPRB, 0.58131E+00_JPRB, 0.58511E+00_JPRB ,&
&  0.57138E+00_JPRB, 0.55097E+00_JPRB, 0.53386E+00_JPRB, 0.52171E+00_JPRB, 0.51599E+00_JPRB, 0.51752E+00_JPRB, 0.52488E+00_JPRB ,&
&  0.52828E+00_JPRB, 0.52797E+00_JPRB, 0.52844E+00_JPRB, 0.53178E+00_JPRB, 0.53654E+00_JPRB, 0.54511E+00_JPRB, 0.55699E+00_JPRB ,&
&  0.57169E+00_JPRB, 0.58833E+00_JPRB, 0.60724E+00_JPRB, 0.62177E+00_JPRB, 0.62489E+00_JPRB, 0.62230E+00_JPRB, 0.61523E+00_JPRB ,&
&  0.60694E+00_JPRB, 0.60071E+00_JPRB, 0.59538E+00_JPRB, 0.58129E+00_JPRB, 0.60156E+00_JPRB, 0.60096E+00_JPRB, 0.58580E+00_JPRB ,&
&  0.57231E+00_JPRB, 0.55671E+00_JPRB, 0.55035E+00_JPRB, 0.50175E+00_JPRB, 0.53398E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.49161E+00_JPRB, 0.49085E+00_JPRB, 0.48716E+00_JPRB, 0.48335E+00_JPRB, 0.48067E+00_JPRB, 0.47651E+00_JPRB, 0.47324E+00_JPRB ,&
&  0.46855E+00_JPRB, 0.46538E+00_JPRB, 0.45955E+00_JPRB, 0.45593E+00_JPRB, 0.45408E+00_JPRB, 0.45374E+00_JPRB, 0.45294E+00_JPRB ,&
&  0.44918E+00_JPRB, 0.44246E+00_JPRB, 0.43539E+00_JPRB, 0.42969E+00_JPRB, 0.42542E+00_JPRB, 0.42088E+00_JPRB, 0.41666E+00_JPRB ,&
&  0.41297E+00_JPRB, 0.40969E+00_JPRB, 0.40676E+00_JPRB, 0.40682E+00_JPRB, 0.40713E+00_JPRB, 0.40649E+00_JPRB, 0.40456E+00_JPRB ,&
&  0.41121E+00_JPRB, 0.36158E+00_JPRB, 0.37459E+00_JPRB, 0.38921E+00_JPRB, 0.40453E+00_JPRB, 0.44119E+00_JPRB, 0.49296E+00_JPRB ,&
&  0.53801E+00_JPRB, 0.57146E+00_JPRB, 0.58154E+00_JPRB, 0.56949E+00_JPRB, 0.55331E+00_JPRB, 0.54244E+00_JPRB, 0.53313E+00_JPRB ,&
&  0.51552E+00_JPRB, 0.49986E+00_JPRB, 0.48542E+00_JPRB, 0.47150E+00_JPRB, 0.45824E+00_JPRB, 0.44702E+00_JPRB, 0.43710E+00_JPRB ,&
&  0.43084E+00_JPRB, 0.42832E+00_JPRB, 0.42661E+00_JPRB, 0.42532E+00_JPRB, 0.42608E+00_JPRB, 0.42883E+00_JPRB, 0.43186E+00_JPRB ,&
&  0.43791E+00_JPRB, 0.44900E+00_JPRB, 0.45400E+00_JPRB, 0.45823E+00_JPRB, 0.46709E+00_JPRB, 0.48091E+00_JPRB, 0.48759E+00_JPRB ,&
&  0.48705E+00_JPRB, 0.48094E+00_JPRB, 0.47259E+00_JPRB, 0.46412E+00_JPRB, 0.45663E+00_JPRB, 0.45012E+00_JPRB, 0.44894E+00_JPRB ,&
&  0.44873E+00_JPRB, 0.45063E+00_JPRB, 0.45339E+00_JPRB, 0.45904E+00_JPRB, 0.46574E+00_JPRB, 0.47283E+00_JPRB, 0.48033E+00_JPRB ,&
&  0.48834E+00_JPRB, 0.49724E+00_JPRB, 0.50950E+00_JPRB, 0.52323E+00_JPRB, 0.53415E+00_JPRB, 0.53955E+00_JPRB, 0.54090E+00_JPRB ,&
&  0.53990E+00_JPRB, 0.53874E+00_JPRB, 0.53420E+00_JPRB, 0.53772E+00_JPRB, 0.53894E+00_JPRB, 0.54128E+00_JPRB, 0.53591E+00_JPRB ,&
&  0.52836E+00_JPRB, 0.52094E+00_JPRB, 0.45103E+00_JPRB, 0.48023E+00_JPRB, 0.48742E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.46222E+00_JPRB, 0.45779E+00_JPRB, 0.45316E+00_JPRB, 0.44776E+00_JPRB, 0.44318E+00_JPRB, 0.44001E+00_JPRB, 0.43866E+00_JPRB ,&
&  0.43954E+00_JPRB, 0.44087E+00_JPRB, 0.43980E+00_JPRB, 0.43505E+00_JPRB, 0.43064E+00_JPRB, 0.42791E+00_JPRB, 0.42684E+00_JPRB ,&
&  0.42508E+00_JPRB, 0.42161E+00_JPRB, 0.41667E+00_JPRB, 0.41189E+00_JPRB, 0.40626E+00_JPRB, 0.40144E+00_JPRB, 0.39738E+00_JPRB ,&
&  0.39404E+00_JPRB, 0.39254E+00_JPRB, 0.39143E+00_JPRB, 0.39075E+00_JPRB, 0.39130E+00_JPRB, 0.39105E+00_JPRB, 0.39552E+00_JPRB ,&
&  0.36973E+00_JPRB, 0.33809E+00_JPRB, 0.35077E+00_JPRB, 0.36304E+00_JPRB, 0.37490E+00_JPRB, 0.40093E+00_JPRB, 0.43580E+00_JPRB ,&
&  0.47407E+00_JPRB, 0.50632E+00_JPRB, 0.52458E+00_JPRB, 0.52188E+00_JPRB, 0.50985E+00_JPRB, 0.50131E+00_JPRB, 0.49686E+00_JPRB ,&
&  0.48768E+00_JPRB, 0.47673E+00_JPRB, 0.46762E+00_JPRB, 0.45680E+00_JPRB, 0.44524E+00_JPRB, 0.43451E+00_JPRB, 0.42584E+00_JPRB ,&
&  0.41903E+00_JPRB, 0.41496E+00_JPRB, 0.41396E+00_JPRB, 0.41258E+00_JPRB, 0.41148E+00_JPRB, 0.41201E+00_JPRB, 0.41162E+00_JPRB ,&
&  0.41207E+00_JPRB, 0.41267E+00_JPRB, 0.41555E+00_JPRB, 0.41793E+00_JPRB, 0.42129E+00_JPRB, 0.42369E+00_JPRB, 0.42426E+00_JPRB ,&
&  0.42260E+00_JPRB, 0.42134E+00_JPRB, 0.42149E+00_JPRB, 0.41930E+00_JPRB, 0.41237E+00_JPRB, 0.40503E+00_JPRB, 0.40147E+00_JPRB ,&
&  0.40117E+00_JPRB, 0.40188E+00_JPRB, 0.40299E+00_JPRB, 0.40576E+00_JPRB, 0.40979E+00_JPRB, 0.41448E+00_JPRB, 0.41973E+00_JPRB ,&
&  0.42558E+00_JPRB, 0.43121E+00_JPRB, 0.43636E+00_JPRB, 0.44226E+00_JPRB, 0.45093E+00_JPRB, 0.45873E+00_JPRB, 0.46557E+00_JPRB ,&
&  0.47128E+00_JPRB, 0.47627E+00_JPRB, 0.48061E+00_JPRB, 0.48706E+00_JPRB, 0.49310E+00_JPRB, 0.49697E+00_JPRB, 0.49539E+00_JPRB ,&
&  0.48959E+00_JPRB, 0.48050E+00_JPRB, 0.47714E+00_JPRB, 0.47021E+00_JPRB, 0.46502E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.41779E+00_JPRB, 0.41513E+00_JPRB, 0.41133E+00_JPRB, 0.40707E+00_JPRB, 0.40256E+00_JPRB, 0.40034E+00_JPRB, 0.39975E+00_JPRB ,&
&  0.40168E+00_JPRB, 0.40223E+00_JPRB, 0.40399E+00_JPRB, 0.40233E+00_JPRB, 0.39995E+00_JPRB, 0.39814E+00_JPRB, 0.39615E+00_JPRB ,&
&  0.39377E+00_JPRB, 0.39061E+00_JPRB, 0.38748E+00_JPRB, 0.38497E+00_JPRB, 0.38195E+00_JPRB, 0.37770E+00_JPRB, 0.37372E+00_JPRB ,&
&  0.37069E+00_JPRB, 0.36933E+00_JPRB, 0.36850E+00_JPRB, 0.36788E+00_JPRB, 0.36728E+00_JPRB, 0.36820E+00_JPRB, 0.36881E+00_JPRB ,&
&  0.37117E+00_JPRB, 0.34151E+00_JPRB, 0.35210E+00_JPRB, 0.35634E+00_JPRB, 0.36433E+00_JPRB, 0.38142E+00_JPRB, 0.40189E+00_JPRB ,&
&  0.42326E+00_JPRB, 0.44269E+00_JPRB, 0.45606E+00_JPRB, 0.45864E+00_JPRB, 0.45310E+00_JPRB, 0.44926E+00_JPRB, 0.44965E+00_JPRB ,&
&  0.44800E+00_JPRB, 0.44386E+00_JPRB, 0.44020E+00_JPRB, 0.43583E+00_JPRB, 0.42948E+00_JPRB, 0.42038E+00_JPRB, 0.41137E+00_JPRB ,&
&  0.40628E+00_JPRB, 0.40348E+00_JPRB, 0.40101E+00_JPRB, 0.39829E+00_JPRB, 0.39345E+00_JPRB, 0.38892E+00_JPRB, 0.38497E+00_JPRB ,&
&  0.38131E+00_JPRB, 0.37630E+00_JPRB, 0.37367E+00_JPRB, 0.37288E+00_JPRB, 0.37151E+00_JPRB, 0.37156E+00_JPRB, 0.37160E+00_JPRB ,&
&  0.37027E+00_JPRB, 0.36778E+00_JPRB, 0.36654E+00_JPRB, 0.36667E+00_JPRB, 0.36547E+00_JPRB, 0.36394E+00_JPRB, 0.36195E+00_JPRB ,&
&  0.36167E+00_JPRB, 0.36178E+00_JPRB, 0.36132E+00_JPRB, 0.36164E+00_JPRB, 0.36316E+00_JPRB, 0.36637E+00_JPRB, 0.37025E+00_JPRB ,&
&  0.37454E+00_JPRB, 0.37932E+00_JPRB, 0.38367E+00_JPRB, 0.38751E+00_JPRB, 0.39297E+00_JPRB, 0.39811E+00_JPRB, 0.40473E+00_JPRB ,&
&  0.41232E+00_JPRB, 0.41939E+00_JPRB, 0.42581E+00_JPRB, 0.43205E+00_JPRB, 0.43661E+00_JPRB, 0.43857E+00_JPRB, 0.43909E+00_JPRB ,&
&  0.43841E+00_JPRB, 0.43287E+00_JPRB, 0.42844E+00_JPRB, 0.42342E+00_JPRB, 0.41921E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.38392E+00_JPRB, 0.38040E+00_JPRB, 0.37622E+00_JPRB, 0.37296E+00_JPRB, 0.36990E+00_JPRB, 0.36665E+00_JPRB, 0.36528E+00_JPRB ,&
&  0.36551E+00_JPRB, 0.36684E+00_JPRB, 0.36909E+00_JPRB, 0.37021E+00_JPRB, 0.37066E+00_JPRB, 0.37149E+00_JPRB, 0.37125E+00_JPRB ,&
&  0.37042E+00_JPRB, 0.36898E+00_JPRB, 0.36703E+00_JPRB, 0.36427E+00_JPRB, 0.36094E+00_JPRB, 0.35782E+00_JPRB, 0.35370E+00_JPRB ,&
&  0.35084E+00_JPRB, 0.34917E+00_JPRB, 0.34823E+00_JPRB, 0.34794E+00_JPRB, 0.34897E+00_JPRB, 0.35042E+00_JPRB, 0.35178E+00_JPRB ,&
&  0.35142E+00_JPRB, 0.34156E+00_JPRB, 0.34704E+00_JPRB, 0.34994E+00_JPRB, 0.35816E+00_JPRB, 0.37227E+00_JPRB, 0.38506E+00_JPRB ,&
&  0.39348E+00_JPRB, 0.40297E+00_JPRB, 0.41169E+00_JPRB, 0.41669E+00_JPRB, 0.41524E+00_JPRB, 0.41313E+00_JPRB, 0.41448E+00_JPRB ,&
&  0.41537E+00_JPRB, 0.41440E+00_JPRB, 0.41327E+00_JPRB, 0.41137E+00_JPRB, 0.40827E+00_JPRB, 0.40485E+00_JPRB, 0.39953E+00_JPRB ,&
&  0.39427E+00_JPRB, 0.39045E+00_JPRB, 0.38656E+00_JPRB, 0.38254E+00_JPRB, 0.37656E+00_JPRB, 0.36950E+00_JPRB, 0.36111E+00_JPRB ,&
&  0.35210E+00_JPRB, 0.34569E+00_JPRB, 0.34168E+00_JPRB, 0.33817E+00_JPRB, 0.33504E+00_JPRB, 0.33351E+00_JPRB, 0.33168E+00_JPRB ,&
&  0.33105E+00_JPRB, 0.33168E+00_JPRB, 0.33088E+00_JPRB, 0.33051E+00_JPRB, 0.33144E+00_JPRB, 0.33178E+00_JPRB, 0.33301E+00_JPRB ,&
&  0.33283E+00_JPRB, 0.33237E+00_JPRB, 0.33218E+00_JPRB, 0.33183E+00_JPRB, 0.33123E+00_JPRB, 0.33256E+00_JPRB, 0.33515E+00_JPRB ,&
&  0.33794E+00_JPRB, 0.34207E+00_JPRB, 0.34652E+00_JPRB, 0.35011E+00_JPRB, 0.35341E+00_JPRB, 0.35869E+00_JPRB, 0.36550E+00_JPRB ,&
&  0.37331E+00_JPRB, 0.38178E+00_JPRB, 0.38960E+00_JPRB, 0.39592E+00_JPRB, 0.39837E+00_JPRB, 0.39922E+00_JPRB, 0.39976E+00_JPRB ,&
&  0.39859E+00_JPRB, 0.39369E+00_JPRB, 0.39124E+00_JPRB, 0.38945E+00_JPRB, 0.38685E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.33997E+00_JPRB, 0.33874E+00_JPRB, 0.33777E+00_JPRB, 0.33750E+00_JPRB, 0.33730E+00_JPRB, 0.33721E+00_JPRB, 0.33770E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.34002E+00_JPRB, 0.34352E+00_JPRB, 0.34522E+00_JPRB, 0.34579E+00_JPRB, 0.34699E+00_JPRB, 0.34783E+00_JPRB ,&
&  0.34673E+00_JPRB, 0.34577E+00_JPRB, 0.34464E+00_JPRB, 0.34238E+00_JPRB, 0.33869E+00_JPRB, 0.33667E+00_JPRB, 0.33489E+00_JPRB ,&
&  0.33205E+00_JPRB, 0.32863E+00_JPRB, 0.32564E+00_JPRB, 0.32409E+00_JPRB, 0.32422E+00_JPRB, 0.32389E+00_JPRB, 0.32450E+00_JPRB ,&
&  0.32462E+00_JPRB, 0.32776E+00_JPRB, 0.31608E+00_JPRB, 0.32564E+00_JPRB, 0.33465E+00_JPRB, 0.34355E+00_JPRB, 0.35297E+00_JPRB ,&
&  0.35959E+00_JPRB, 0.36597E+00_JPRB, 0.37332E+00_JPRB, 0.37575E+00_JPRB, 0.37490E+00_JPRB, 0.37169E+00_JPRB, 0.37065E+00_JPRB ,&
&  0.37005E+00_JPRB, 0.36749E+00_JPRB, 0.36590E+00_JPRB, 0.36618E+00_JPRB, 0.36665E+00_JPRB, 0.36552E+00_JPRB, 0.36358E+00_JPRB ,&
&  0.36188E+00_JPRB, 0.35936E+00_JPRB, 0.35504E+00_JPRB, 0.35017E+00_JPRB, 0.34500E+00_JPRB, 0.33862E+00_JPRB, 0.33001E+00_JPRB ,&
&  0.32018E+00_JPRB, 0.31241E+00_JPRB, 0.30595E+00_JPRB, 0.30195E+00_JPRB, 0.30023E+00_JPRB, 0.29876E+00_JPRB, 0.29748E+00_JPRB ,&
&  0.29743E+00_JPRB, 0.29801E+00_JPRB, 0.29816E+00_JPRB, 0.29793E+00_JPRB, 0.29873E+00_JPRB, 0.29952E+00_JPRB, 0.30234E+00_JPRB ,&
&  0.30556E+00_JPRB, 0.30721E+00_JPRB, 0.30794E+00_JPRB, 0.30858E+00_JPRB, 0.30740E+00_JPRB, 0.30804E+00_JPRB, 0.30987E+00_JPRB ,&
&  0.31188E+00_JPRB, 0.31450E+00_JPRB, 0.31636E+00_JPRB, 0.31818E+00_JPRB, 0.32068E+00_JPRB, 0.32442E+00_JPRB, 0.33009E+00_JPRB ,&
&  0.33563E+00_JPRB, 0.34167E+00_JPRB, 0.34814E+00_JPRB, 0.35329E+00_JPRB, 0.35637E+00_JPRB, 0.35735E+00_JPRB, 0.35771E+00_JPRB ,&
&  0.35608E+00_JPRB, 0.35187E+00_JPRB, 0.34759E+00_JPRB, 0.34448E+00_JPRB, 0.34195E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.30850E+00_JPRB, 0.30891E+00_JPRB, 0.31022E+00_JPRB, 0.31085E+00_JPRB, 0.31093E+00_JPRB, 0.31180E+00_JPRB, 0.31477E+00_JPRB ,&
&  0.31749E+00_JPRB, 0.31936E+00_JPRB, 0.32108E+00_JPRB, 0.32208E+00_JPRB, 0.32258E+00_JPRB, 0.32440E+00_JPRB, 0.32581E+00_JPRB ,&
&  0.32614E+00_JPRB, 0.32516E+00_JPRB, 0.32395E+00_JPRB, 0.32238E+00_JPRB, 0.32019E+00_JPRB, 0.31780E+00_JPRB, 0.31583E+00_JPRB ,&
&  0.31321E+00_JPRB, 0.31027E+00_JPRB, 0.30738E+00_JPRB, 0.30611E+00_JPRB, 0.30589E+00_JPRB, 0.30588E+00_JPRB, 0.30663E+00_JPRB ,&
&  0.30738E+00_JPRB, 0.30979E+00_JPRB, 0.31268E+00_JPRB, 0.31612E+00_JPRB, 0.31955E+00_JPRB, 0.32412E+00_JPRB, 0.32916E+00_JPRB ,&
&  0.33392E+00_JPRB, 0.33640E+00_JPRB, 0.33839E+00_JPRB, 0.33680E+00_JPRB, 0.33675E+00_JPRB, 0.33649E+00_JPRB, 0.33616E+00_JPRB ,&
&  0.33499E+00_JPRB, 0.32994E+00_JPRB, 0.32591E+00_JPRB, 0.32565E+00_JPRB, 0.32663E+00_JPRB, 0.32735E+00_JPRB, 0.32764E+00_JPRB ,&
&  0.32681E+00_JPRB, 0.32536E+00_JPRB, 0.32236E+00_JPRB, 0.31820E+00_JPRB, 0.31377E+00_JPRB, 0.30843E+00_JPRB, 0.30085E+00_JPRB ,&
&  0.29378E+00_JPRB, 0.28804E+00_JPRB, 0.28360E+00_JPRB, 0.27972E+00_JPRB, 0.27721E+00_JPRB, 0.27456E+00_JPRB, 0.27407E+00_JPRB ,&
&  0.27478E+00_JPRB, 0.27689E+00_JPRB, 0.27802E+00_JPRB, 0.27724E+00_JPRB, 0.27607E+00_JPRB, 0.27619E+00_JPRB, 0.27841E+00_JPRB ,&
&  0.28254E+00_JPRB, 0.28583E+00_JPRB, 0.28704E+00_JPRB, 0.28694E+00_JPRB, 0.28636E+00_JPRB, 0.28743E+00_JPRB, 0.28951E+00_JPRB ,&
&  0.29059E+00_JPRB, 0.29115E+00_JPRB, 0.29222E+00_JPRB, 0.29205E+00_JPRB, 0.29393E+00_JPRB, 0.29719E+00_JPRB, 0.30083E+00_JPRB ,&
&  0.30436E+00_JPRB, 0.30876E+00_JPRB, 0.31311E+00_JPRB, 0.31634E+00_JPRB, 0.31903E+00_JPRB, 0.31881E+00_JPRB, 0.31871E+00_JPRB ,&
&  0.31838E+00_JPRB, 0.31675E+00_JPRB, 0.31517E+00_JPRB, 0.31292E+00_JPRB, 0.31014E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.28165E+00_JPRB, 0.28457E+00_JPRB, 0.28824E+00_JPRB, 0.28912E+00_JPRB, 0.28962E+00_JPRB, 0.29124E+00_JPRB, 0.29449E+00_JPRB ,&
&  0.29692E+00_JPRB, 0.29786E+00_JPRB, 0.29819E+00_JPRB, 0.29836E+00_JPRB, 0.29906E+00_JPRB, 0.30093E+00_JPRB, 0.30184E+00_JPRB ,&
&  0.30134E+00_JPRB, 0.30078E+00_JPRB, 0.29996E+00_JPRB, 0.29928E+00_JPRB, 0.29774E+00_JPRB, 0.29544E+00_JPRB, 0.29347E+00_JPRB ,&
&  0.29075E+00_JPRB, 0.28823E+00_JPRB, 0.28633E+00_JPRB, 0.28484E+00_JPRB, 0.28398E+00_JPRB, 0.28389E+00_JPRB, 0.28533E+00_JPRB ,&
&  0.28718E+00_JPRB, 0.28959E+00_JPRB, 0.29256E+00_JPRB, 0.29600E+00_JPRB, 0.29945E+00_JPRB, 0.30338E+00_JPRB, 0.30687E+00_JPRB ,&
&  0.30963E+00_JPRB, 0.31199E+00_JPRB, 0.31355E+00_JPRB, 0.31078E+00_JPRB, 0.30734E+00_JPRB, 0.30667E+00_JPRB, 0.30649E+00_JPRB ,&
&  0.30407E+00_JPRB, 0.30042E+00_JPRB, 0.29557E+00_JPRB, 0.29286E+00_JPRB, 0.29212E+00_JPRB, 0.29110E+00_JPRB, 0.28982E+00_JPRB ,&
&  0.28894E+00_JPRB, 0.28826E+00_JPRB, 0.28704E+00_JPRB, 0.28493E+00_JPRB, 0.28152E+00_JPRB, 0.27689E+00_JPRB, 0.27228E+00_JPRB ,&
&  0.26937E+00_JPRB, 0.26733E+00_JPRB, 0.26735E+00_JPRB, 0.26535E+00_JPRB, 0.26133E+00_JPRB, 0.25758E+00_JPRB, 0.25726E+00_JPRB ,&
&  0.25774E+00_JPRB, 0.25870E+00_JPRB, 0.25880E+00_JPRB, 0.25763E+00_JPRB, 0.25774E+00_JPRB, 0.25843E+00_JPRB, 0.25981E+00_JPRB ,&
&  0.26389E+00_JPRB, 0.26686E+00_JPRB, 0.26684E+00_JPRB, 0.26584E+00_JPRB, 0.26394E+00_JPRB, 0.26353E+00_JPRB, 0.26431E+00_JPRB ,&
&  0.26515E+00_JPRB, 0.26553E+00_JPRB, 0.26580E+00_JPRB, 0.26523E+00_JPRB, 0.26529E+00_JPRB, 0.26674E+00_JPRB, 0.26858E+00_JPRB ,&
&  0.27083E+00_JPRB, 0.27361E+00_JPRB, 0.27603E+00_JPRB, 0.27844E+00_JPRB, 0.28060E+00_JPRB, 0.28151E+00_JPRB, 0.28233E+00_JPRB ,&
&  0.28326E+00_JPRB, 0.28330E+00_JPRB, 0.28290E+00_JPRB, 0.28264E+00_JPRB, 0.28230E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.26116E+00_JPRB, 0.26531E+00_JPRB, 0.27072E+00_JPRB, 0.27203E+00_JPRB, 0.27449E+00_JPRB, 0.27705E+00_JPRB, 0.28060E+00_JPRB ,&
&  0.28303E+00_JPRB, 0.28345E+00_JPRB, 0.28222E+00_JPRB, 0.28094E+00_JPRB, 0.28032E+00_JPRB, 0.28024E+00_JPRB, 0.27984E+00_JPRB ,&
&  0.27971E+00_JPRB, 0.27990E+00_JPRB, 0.27983E+00_JPRB, 0.27960E+00_JPRB, 0.27833E+00_JPRB, 0.27643E+00_JPRB, 0.27430E+00_JPRB ,&
&  0.27084E+00_JPRB, 0.26852E+00_JPRB, 0.26773E+00_JPRB, 0.26743E+00_JPRB, 0.26665E+00_JPRB, 0.26712E+00_JPRB, 0.26883E+00_JPRB ,&
&  0.27255E+00_JPRB, 0.27550E+00_JPRB, 0.27928E+00_JPRB, 0.28313E+00_JPRB, 0.28537E+00_JPRB, 0.28700E+00_JPRB, 0.28893E+00_JPRB ,&
&  0.29012E+00_JPRB, 0.29039E+00_JPRB, 0.29040E+00_JPRB, 0.29034E+00_JPRB, 0.28756E+00_JPRB, 0.28674E+00_JPRB, 0.28650E+00_JPRB ,&
&  0.28550E+00_JPRB, 0.28291E+00_JPRB, 0.27920E+00_JPRB, 0.27601E+00_JPRB, 0.27283E+00_JPRB, 0.27051E+00_JPRB, 0.26857E+00_JPRB ,&
&  0.26714E+00_JPRB, 0.26646E+00_JPRB, 0.26643E+00_JPRB, 0.26497E+00_JPRB, 0.25982E+00_JPRB, 0.25478E+00_JPRB, 0.25199E+00_JPRB ,&
&  0.25072E+00_JPRB, 0.25046E+00_JPRB, 0.25188E+00_JPRB, 0.25069E+00_JPRB, 0.24698E+00_JPRB, 0.24401E+00_JPRB, 0.24296E+00_JPRB ,&
&  0.24165E+00_JPRB, 0.24116E+00_JPRB, 0.24198E+00_JPRB, 0.24252E+00_JPRB, 0.24295E+00_JPRB, 0.24295E+00_JPRB, 0.24402E+00_JPRB ,&
&  0.24519E+00_JPRB, 0.24684E+00_JPRB, 0.24676E+00_JPRB, 0.24406E+00_JPRB, 0.24031E+00_JPRB, 0.23765E+00_JPRB, 0.23735E+00_JPRB ,&
&  0.23910E+00_JPRB, 0.24077E+00_JPRB, 0.24133E+00_JPRB, 0.24157E+00_JPRB, 0.24265E+00_JPRB, 0.24509E+00_JPRB, 0.24826E+00_JPRB ,&
&  0.24961E+00_JPRB, 0.25092E+00_JPRB, 0.25276E+00_JPRB, 0.25462E+00_JPRB, 0.25579E+00_JPRB, 0.25634E+00_JPRB, 0.25638E+00_JPRB ,&
&  0.25569E+00_JPRB, 0.25635E+00_JPRB, 0.25786E+00_JPRB, 0.25968E+00_JPRB, 0.26088E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.24746E+00_JPRB, 0.25057E+00_JPRB, 0.25367E+00_JPRB, 0.25541E+00_JPRB, 0.25722E+00_JPRB, 0.25913E+00_JPRB, 0.26253E+00_JPRB ,&
&  0.26656E+00_JPRB, 0.26800E+00_JPRB, 0.26646E+00_JPRB, 0.26381E+00_JPRB, 0.26214E+00_JPRB, 0.26147E+00_JPRB, 0.26136E+00_JPRB ,&
&  0.26126E+00_JPRB, 0.26115E+00_JPRB, 0.26034E+00_JPRB, 0.25845E+00_JPRB, 0.25578E+00_JPRB, 0.25309E+00_JPRB, 0.25103E+00_JPRB ,&
&  0.25063E+00_JPRB, 0.25197E+00_JPRB, 0.25418E+00_JPRB, 0.25437E+00_JPRB, 0.25452E+00_JPRB, 0.25578E+00_JPRB, 0.25782E+00_JPRB ,&
&  0.26171E+00_JPRB, 0.26359E+00_JPRB, 0.26549E+00_JPRB, 0.26143E+00_JPRB, 0.26576E+00_JPRB, 0.26727E+00_JPRB, 0.26767E+00_JPRB ,&
&  0.26884E+00_JPRB, 0.26980E+00_JPRB, 0.27079E+00_JPRB, 0.27013E+00_JPRB, 0.26589E+00_JPRB, 0.26415E+00_JPRB, 0.26497E+00_JPRB ,&
&  0.26572E+00_JPRB, 0.26415E+00_JPRB, 0.26128E+00_JPRB, 0.25858E+00_JPRB, 0.25628E+00_JPRB, 0.25465E+00_JPRB, 0.25328E+00_JPRB ,&
&  0.25263E+00_JPRB, 0.25188E+00_JPRB, 0.24995E+00_JPRB, 0.24477E+00_JPRB, 0.23887E+00_JPRB, 0.23498E+00_JPRB, 0.23227E+00_JPRB ,&
&  0.23230E+00_JPRB, 0.23426E+00_JPRB, 0.23600E+00_JPRB, 0.23620E+00_JPRB, 0.23497E+00_JPRB, 0.23265E+00_JPRB, 0.22932E+00_JPRB ,&
&  0.22452E+00_JPRB, 0.22259E+00_JPRB, 0.22212E+00_JPRB, 0.22124E+00_JPRB, 0.22163E+00_JPRB, 0.22337E+00_JPRB, 0.22514E+00_JPRB ,&
&  0.22731E+00_JPRB, 0.22730E+00_JPRB, 0.22587E+00_JPRB, 0.22344E+00_JPRB, 0.22092E+00_JPRB, 0.22075E+00_JPRB, 0.22147E+00_JPRB ,&
&  0.22218E+00_JPRB, 0.22282E+00_JPRB, 0.22381E+00_JPRB, 0.22510E+00_JPRB, 0.22674E+00_JPRB, 0.22908E+00_JPRB, 0.23099E+00_JPRB ,&
&  0.23093E+00_JPRB, 0.22977E+00_JPRB, 0.23081E+00_JPRB, 0.23258E+00_JPRB, 0.23346E+00_JPRB, 0.23399E+00_JPRB, 0.23434E+00_JPRB ,&
&  0.23397E+00_JPRB, 0.23402E+00_JPRB, 0.23593E+00_JPRB, 0.24052E+00_JPRB, 0.24470E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.23517E+00_JPRB, 0.23650E+00_JPRB, 0.23669E+00_JPRB, 0.23715E+00_JPRB, 0.23778E+00_JPRB, 0.23968E+00_JPRB, 0.24387E+00_JPRB ,&
&  0.24802E+00_JPRB, 0.24998E+00_JPRB, 0.24852E+00_JPRB, 0.24630E+00_JPRB, 0.24517E+00_JPRB, 0.24461E+00_JPRB, 0.24445E+00_JPRB ,&
&  0.24430E+00_JPRB, 0.24326E+00_JPRB, 0.24033E+00_JPRB, 0.23626E+00_JPRB, 0.23322E+00_JPRB, 0.23235E+00_JPRB, 0.23268E+00_JPRB ,&
&  0.23637E+00_JPRB, 0.23891E+00_JPRB, 0.24131E+00_JPRB, 0.24347E+00_JPRB, 0.24404E+00_JPRB, 0.24373E+00_JPRB, 0.24516E+00_JPRB ,&
&  0.24611E+00_JPRB, 0.24362E+00_JPRB, 0.23820E+00_JPRB, 0.24215E+00_JPRB, 0.24424E+00_JPRB, 0.24550E+00_JPRB, 0.24474E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24747E+00_JPRB, 0.24955E+00_JPRB, 0.25120E+00_JPRB, 0.25045E+00_JPRB, 0.24739E+00_JPRB, 0.24644E+00_JPRB ,&
&  0.24671E+00_JPRB, 0.24564E+00_JPRB, 0.24294E+00_JPRB, 0.24047E+00_JPRB, 0.23770E+00_JPRB, 0.23645E+00_JPRB, 0.23627E+00_JPRB ,&
&  0.23545E+00_JPRB, 0.23333E+00_JPRB, 0.22903E+00_JPRB, 0.22481E+00_JPRB, 0.22169E+00_JPRB, 0.21865E+00_JPRB, 0.21734E+00_JPRB ,&
&  0.21776E+00_JPRB, 0.21897E+00_JPRB, 0.21904E+00_JPRB, 0.21760E+00_JPRB, 0.21273E+00_JPRB, 0.20284E+00_JPRB, 0.19571E+00_JPRB ,&
&  0.19445E+00_JPRB, 0.19901E+00_JPRB, 0.20705E+00_JPRB, 0.20864E+00_JPRB, 0.21093E+00_JPRB, 0.21243E+00_JPRB, 0.21368E+00_JPRB ,&
&  0.21204E+00_JPRB, 0.19905E+00_JPRB, 0.19331E+00_JPRB, 0.19002E+00_JPRB, 0.18690E+00_JPRB, 0.18655E+00_JPRB, 0.18816E+00_JPRB ,&
&  0.19144E+00_JPRB, 0.19557E+00_JPRB, 0.19849E+00_JPRB, 0.20019E+00_JPRB, 0.20071E+00_JPRB, 0.20044E+00_JPRB, 0.19973E+00_JPRB ,&
&  0.19995E+00_JPRB, 0.20384E+00_JPRB, 0.21169E+00_JPRB, 0.21975E+00_JPRB, 0.22077E+00_JPRB, 0.22234E+00_JPRB, 0.22275E+00_JPRB ,&
&  0.22315E+00_JPRB, 0.22507E+00_JPRB, 0.22625E+00_JPRB, 0.22899E+00_JPRB, 0.23241E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.22446E+00_JPRB, 0.22570E+00_JPRB, 0.22591E+00_JPRB, 0.22506E+00_JPRB, 0.22382E+00_JPRB, 0.22401E+00_JPRB, 0.22541E+00_JPRB ,&
&  0.22834E+00_JPRB, 0.23046E+00_JPRB, 0.23096E+00_JPRB, 0.23097E+00_JPRB, 0.23004E+00_JPRB, 0.22799E+00_JPRB, 0.22627E+00_JPRB ,&
&  0.22495E+00_JPRB, 0.22417E+00_JPRB, 0.22356E+00_JPRB, 0.22335E+00_JPRB, 0.22326E+00_JPRB, 0.22416E+00_JPRB, 0.22504E+00_JPRB ,&
&  0.22863E+00_JPRB, 0.22862E+00_JPRB, 0.22793E+00_JPRB, 0.22769E+00_JPRB, 0.22746E+00_JPRB, 0.22742E+00_JPRB, 0.22726E+00_JPRB ,&
&  0.22450E+00_JPRB, 0.21677E+00_JPRB, 0.20999E+00_JPRB, 0.21552E+00_JPRB, 0.22730E+00_JPRB, 0.22763E+00_JPRB, 0.22773E+00_JPRB ,&
&  0.22908E+00_JPRB, 0.23194E+00_JPRB, 0.23398E+00_JPRB, 0.23499E+00_JPRB, 0.23377E+00_JPRB, 0.23197E+00_JPRB, 0.23139E+00_JPRB ,&
&  0.23160E+00_JPRB, 0.22986E+00_JPRB, 0.22470E+00_JPRB, 0.21876E+00_JPRB, 0.21283E+00_JPRB, 0.21051E+00_JPRB, 0.20892E+00_JPRB ,&
&  0.20648E+00_JPRB, 0.20506E+00_JPRB, 0.20454E+00_JPRB, 0.20407E+00_JPRB, 0.20449E+00_JPRB, 0.20244E+00_JPRB, 0.19946E+00_JPRB ,&
&  0.19604E+00_JPRB, 0.19197E+00_JPRB, 0.18709E+00_JPRB, 0.18194E+00_JPRB, 0.17754E+00_JPRB, 0.17277E+00_JPRB, 0.16948E+00_JPRB ,&
&  0.16892E+00_JPRB, 0.17290E+00_JPRB, 0.18256E+00_JPRB, 0.19265E+00_JPRB, 0.19969E+00_JPRB, 0.18910E+00_JPRB, 0.18239E+00_JPRB ,&
&  0.17506E+00_JPRB, 0.16907E+00_JPRB, 0.16565E+00_JPRB, 0.16489E+00_JPRB, 0.16506E+00_JPRB, 0.16590E+00_JPRB, 0.16754E+00_JPRB ,&
&  0.16967E+00_JPRB, 0.17236E+00_JPRB, 0.17440E+00_JPRB, 0.17589E+00_JPRB, 0.17693E+00_JPRB, 0.17723E+00_JPRB, 0.17701E+00_JPRB ,&
&  0.17676E+00_JPRB, 0.17801E+00_JPRB, 0.18042E+00_JPRB, 0.18469E+00_JPRB, 0.18764E+00_JPRB, 0.19147E+00_JPRB, 0.19671E+00_JPRB ,&
&  0.20437E+00_JPRB, 0.21336E+00_JPRB, 0.21675E+00_JPRB, 0.21904E+00_JPRB, 0.22203E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.21307E+00_JPRB, 0.21755E+00_JPRB, 0.21996E+00_JPRB, 0.21945E+00_JPRB, 0.21903E+00_JPRB, 0.21957E+00_JPRB, 0.22027E+00_JPRB ,&
&  0.22124E+00_JPRB, 0.22071E+00_JPRB, 0.21922E+00_JPRB, 0.21776E+00_JPRB, 0.21621E+00_JPRB, 0.21423E+00_JPRB, 0.21292E+00_JPRB ,&
&  0.21192E+00_JPRB, 0.21200E+00_JPRB, 0.21181E+00_JPRB, 0.21280E+00_JPRB, 0.21341E+00_JPRB, 0.21569E+00_JPRB, 0.21414E+00_JPRB ,&
&  0.21052E+00_JPRB, 0.20849E+00_JPRB, 0.21074E+00_JPRB, 0.21105E+00_JPRB, 0.21155E+00_JPRB, 0.21226E+00_JPRB, 0.21360E+00_JPRB ,&
&  0.21106E+00_JPRB, 0.20806E+00_JPRB, 0.20419E+00_JPRB, 0.20968E+00_JPRB, 0.21808E+00_JPRB, 0.21756E+00_JPRB, 0.21698E+00_JPRB ,&
&  0.21847E+00_JPRB, 0.22113E+00_JPRB, 0.22240E+00_JPRB, 0.22265E+00_JPRB, 0.22116E+00_JPRB, 0.22043E+00_JPRB, 0.21927E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21374E+00_JPRB, 0.20502E+00_JPRB, 0.18899E+00_JPRB, 0.18272E+00_JPRB, 0.17983E+00_JPRB, 0.17595E+00_JPRB ,&
&  0.17089E+00_JPRB, 0.16865E+00_JPRB, 0.16846E+00_JPRB, 0.16679E+00_JPRB, 0.16697E+00_JPRB, 0.16657E+00_JPRB, 0.16553E+00_JPRB ,&
&  0.16421E+00_JPRB, 0.16330E+00_JPRB, 0.16181E+00_JPRB, 0.15941E+00_JPRB, 0.15762E+00_JPRB, 0.15727E+00_JPRB, 0.15810E+00_JPRB ,&
&  0.16007E+00_JPRB, 0.16304E+00_JPRB, 0.17146E+00_JPRB, 0.17637E+00_JPRB, 0.17444E+00_JPRB, 0.17056E+00_JPRB, 0.16433E+00_JPRB ,&
&  0.15873E+00_JPRB, 0.15699E+00_JPRB, 0.15576E+00_JPRB, 0.15473E+00_JPRB, 0.15507E+00_JPRB, 0.15612E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15847E+00_JPRB, 0.15994E+00_JPRB, 0.16085E+00_JPRB, 0.16136E+00_JPRB, 0.16183E+00_JPRB, 0.16209E+00_JPRB, 0.16259E+00_JPRB ,&
&  0.16272E+00_JPRB, 0.16406E+00_JPRB, 0.16522E+00_JPRB, 0.16756E+00_JPRB, 0.16916E+00_JPRB, 0.17122E+00_JPRB, 0.17557E+00_JPRB ,&
&  0.17922E+00_JPRB, 0.18998E+00_JPRB, 0.20481E+00_JPRB, 0.21220E+00_JPRB, 0.20927E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.20506E+00_JPRB, 0.20742E+00_JPRB, 0.20957E+00_JPRB, 0.20824E+00_JPRB, 0.20824E+00_JPRB, 0.20848E+00_JPRB, 0.20959E+00_JPRB ,&
&  0.21009E+00_JPRB, 0.20960E+00_JPRB, 0.20922E+00_JPRB, 0.20768E+00_JPRB, 0.20616E+00_JPRB, 0.19936E+00_JPRB, 0.19648E+00_JPRB ,&
&  0.19422E+00_JPRB, 0.19478E+00_JPRB, 0.19527E+00_JPRB, 0.19724E+00_JPRB, 0.19737E+00_JPRB, 0.19643E+00_JPRB, 0.19334E+00_JPRB ,&
&  0.19092E+00_JPRB, 0.18947E+00_JPRB, 0.18789E+00_JPRB, 0.18815E+00_JPRB, 0.19019E+00_JPRB, 0.19238E+00_JPRB, 0.19373E+00_JPRB ,&
&  0.19436E+00_JPRB, 0.19639E+00_JPRB, 0.19815E+00_JPRB, 0.20092E+00_JPRB, 0.20764E+00_JPRB, 0.20857E+00_JPRB, 0.20941E+00_JPRB ,&
&  0.21033E+00_JPRB, 0.21152E+00_JPRB, 0.21221E+00_JPRB, 0.21191E+00_JPRB, 0.21019E+00_JPRB, 0.20523E+00_JPRB, 0.20388E+00_JPRB ,&
&  0.19152E+00_JPRB, 0.18424E+00_JPRB, 0.17738E+00_JPRB, 0.17234E+00_JPRB, 0.16701E+00_JPRB, 0.16415E+00_JPRB, 0.16093E+00_JPRB ,&
&  0.15821E+00_JPRB, 0.15524E+00_JPRB, 0.15304E+00_JPRB, 0.15206E+00_JPRB, 0.15176E+00_JPRB, 0.15151E+00_JPRB, 0.15087E+00_JPRB ,&
&  0.15098E+00_JPRB, 0.15055E+00_JPRB, 0.15024E+00_JPRB, 0.14992E+00_JPRB, 0.14971E+00_JPRB, 0.14984E+00_JPRB, 0.15062E+00_JPRB ,&
&  0.15259E+00_JPRB, 0.15441E+00_JPRB, 0.15629E+00_JPRB, 0.15805E+00_JPRB, 0.15760E+00_JPRB, 0.15589E+00_JPRB, 0.15330E+00_JPRB ,&
&  0.15107E+00_JPRB, 0.14963E+00_JPRB, 0.14808E+00_JPRB, 0.14768E+00_JPRB, 0.14830E+00_JPRB, 0.14906E+00_JPRB, 0.15038E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15250E+00_JPRB, 0.15313E+00_JPRB, 0.15356E+00_JPRB, 0.15425E+00_JPRB, 0.15490E+00_JPRB, 0.15598E+00_JPRB ,&
&  0.15720E+00_JPRB, 0.15870E+00_JPRB, 0.16007E+00_JPRB, 0.16195E+00_JPRB, 0.16474E+00_JPRB, 0.16896E+00_JPRB, 0.17618E+00_JPRB ,&
&  0.18327E+00_JPRB, 0.20183E+00_JPRB, 0.20408E+00_JPRB, 0.20344E+00_JPRB, 0.20491E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.19616E+00_JPRB, 0.19808E+00_JPRB, 0.19945E+00_JPRB, 0.20074E+00_JPRB, 0.20087E+00_JPRB, 0.20107E+00_JPRB, 0.20175E+00_JPRB ,&
&  0.19790E+00_JPRB, 0.19515E+00_JPRB, 0.19057E+00_JPRB, 0.18629E+00_JPRB, 0.18221E+00_JPRB, 0.17878E+00_JPRB, 0.17670E+00_JPRB ,&
&  0.17560E+00_JPRB, 0.17566E+00_JPRB, 0.17584E+00_JPRB, 0.17644E+00_JPRB, 0.17566E+00_JPRB, 0.17472E+00_JPRB, 0.17383E+00_JPRB ,&
&  0.17274E+00_JPRB, 0.17313E+00_JPRB, 0.17369E+00_JPRB, 0.17703E+00_JPRB, 0.18172E+00_JPRB, 0.18700E+00_JPRB, 0.19082E+00_JPRB ,&
&  0.19350E+00_JPRB, 0.19554E+00_JPRB, 0.19806E+00_JPRB, 0.19913E+00_JPRB, 0.20019E+00_JPRB, 0.20146E+00_JPRB, 0.20196E+00_JPRB ,&
&  0.20326E+00_JPRB, 0.20330E+00_JPRB, 0.20317E+00_JPRB, 0.20340E+00_JPRB, 0.19504E+00_JPRB, 0.18886E+00_JPRB, 0.18257E+00_JPRB ,&
&  0.17655E+00_JPRB, 0.17043E+00_JPRB, 0.16491E+00_JPRB, 0.16184E+00_JPRB, 0.15860E+00_JPRB, 0.15672E+00_JPRB, 0.15507E+00_JPRB ,&
&  0.15384E+00_JPRB, 0.15257E+00_JPRB, 0.15076E+00_JPRB, 0.14899E+00_JPRB, 0.14795E+00_JPRB, 0.14708E+00_JPRB, 0.14599E+00_JPRB ,&
&  0.14589E+00_JPRB, 0.14526E+00_JPRB, 0.14521E+00_JPRB, 0.14485E+00_JPRB, 0.14459E+00_JPRB, 0.14438E+00_JPRB, 0.14440E+00_JPRB ,&
&  0.14481E+00_JPRB, 0.14560E+00_JPRB, 0.14590E+00_JPRB, 0.14629E+00_JPRB, 0.14618E+00_JPRB, 0.14556E+00_JPRB, 0.14503E+00_JPRB ,&
&  0.14447E+00_JPRB, 0.14403E+00_JPRB, 0.14406E+00_JPRB, 0.14398E+00_JPRB, 0.14449E+00_JPRB, 0.14518E+00_JPRB, 0.14586E+00_JPRB ,&
&  0.14681E+00_JPRB, 0.14751E+00_JPRB, 0.14823E+00_JPRB, 0.14913E+00_JPRB, 0.14990E+00_JPRB, 0.15090E+00_JPRB, 0.15204E+00_JPRB ,&
&  0.15291E+00_JPRB, 0.15464E+00_JPRB, 0.15607E+00_JPRB, 0.15785E+00_JPRB, 0.16121E+00_JPRB, 0.16480E+00_JPRB, 0.17322E+00_JPRB ,&
&  0.18118E+00_JPRB, 0.19779E+00_JPRB, 0.19915E+00_JPRB, 0.19432E+00_JPRB, 0.19496E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.19071E+00_JPRB, 0.19211E+00_JPRB, 0.19279E+00_JPRB, 0.19379E+00_JPRB, 0.19421E+00_JPRB, 0.19567E+00_JPRB, 0.19581E+00_JPRB ,&
&  0.19592E+00_JPRB, 0.19208E+00_JPRB, 0.18975E+00_JPRB, 0.18708E+00_JPRB, 0.18476E+00_JPRB, 0.18205E+00_JPRB, 0.17941E+00_JPRB ,&
&  0.17736E+00_JPRB, 0.17554E+00_JPRB, 0.17360E+00_JPRB, 0.17093E+00_JPRB, 0.16824E+00_JPRB, 0.16630E+00_JPRB, 0.16491E+00_JPRB ,&
&  0.16357E+00_JPRB, 0.16434E+00_JPRB, 0.16593E+00_JPRB, 0.16912E+00_JPRB, 0.17345E+00_JPRB, 0.17706E+00_JPRB, 0.18185E+00_JPRB ,&
&  0.18787E+00_JPRB, 0.18969E+00_JPRB, 0.19197E+00_JPRB, 0.19301E+00_JPRB, 0.19269E+00_JPRB, 0.19321E+00_JPRB, 0.19094E+00_JPRB ,&
&  0.19111E+00_JPRB, 0.18860E+00_JPRB, 0.19306E+00_JPRB, 0.19011E+00_JPRB, 0.18351E+00_JPRB, 0.18047E+00_JPRB, 0.17441E+00_JPRB ,&
&  0.16998E+00_JPRB, 0.16522E+00_JPRB, 0.16202E+00_JPRB, 0.16019E+00_JPRB, 0.15719E+00_JPRB, 0.15556E+00_JPRB, 0.15306E+00_JPRB ,&
&  0.15077E+00_JPRB, 0.15000E+00_JPRB, 0.14856E+00_JPRB, 0.14795E+00_JPRB, 0.14717E+00_JPRB, 0.14608E+00_JPRB, 0.14490E+00_JPRB ,&
&  0.14411E+00_JPRB, 0.14342E+00_JPRB, 0.14307E+00_JPRB, 0.14257E+00_JPRB, 0.14186E+00_JPRB, 0.14115E+00_JPRB, 0.14034E+00_JPRB ,&
&  0.13981E+00_JPRB, 0.13953E+00_JPRB, 0.13932E+00_JPRB, 0.13927E+00_JPRB, 0.13921E+00_JPRB, 0.13900E+00_JPRB, 0.13913E+00_JPRB ,&
&  0.13939E+00_JPRB, 0.14001E+00_JPRB, 0.14053E+00_JPRB, 0.14122E+00_JPRB, 0.14185E+00_JPRB, 0.14268E+00_JPRB, 0.14331E+00_JPRB ,&
&  0.14399E+00_JPRB, 0.14493E+00_JPRB, 0.14575E+00_JPRB, 0.14672E+00_JPRB, 0.14777E+00_JPRB, 0.14866E+00_JPRB, 0.15007E+00_JPRB ,&
&  0.15150E+00_JPRB, 0.15292E+00_JPRB, 0.15490E+00_JPRB, 0.15694E+00_JPRB, 0.15955E+00_JPRB, 0.16395E+00_JPRB, 0.16899E+00_JPRB ,&
&  0.17529E+00_JPRB, 0.18466E+00_JPRB, 0.18501E+00_JPRB, 0.18783E+00_JPRB, 0.18942E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.18998E+00_JPRB, 0.19122E+00_JPRB, 0.19225E+00_JPRB, 0.19276E+00_JPRB, 0.19333E+00_JPRB, 0.19335E+00_JPRB, 0.19325E+00_JPRB ,&
&  0.19045E+00_JPRB, 0.18924E+00_JPRB, 0.18748E+00_JPRB, 0.18548E+00_JPRB, 0.18372E+00_JPRB, 0.18144E+00_JPRB, 0.17923E+00_JPRB ,&
&  0.17717E+00_JPRB, 0.17476E+00_JPRB, 0.17228E+00_JPRB, 0.16937E+00_JPRB, 0.16659E+00_JPRB, 0.16397E+00_JPRB, 0.16256E+00_JPRB ,&
&  0.16189E+00_JPRB, 0.16225E+00_JPRB, 0.16386E+00_JPRB, 0.16514E+00_JPRB, 0.16696E+00_JPRB, 0.16928E+00_JPRB, 0.17292E+00_JPRB ,&
&  0.17566E+00_JPRB, 0.17905E+00_JPRB, 0.18129E+00_JPRB, 0.18197E+00_JPRB, 0.18128E+00_JPRB, 0.18092E+00_JPRB, 0.18004E+00_JPRB ,&
&  0.17821E+00_JPRB, 0.17631E+00_JPRB, 0.17468E+00_JPRB, 0.17291E+00_JPRB, 0.17098E+00_JPRB, 0.16863E+00_JPRB, 0.16572E+00_JPRB ,&
&  0.16309E+00_JPRB, 0.16077E+00_JPRB, 0.15859E+00_JPRB, 0.15689E+00_JPRB, 0.15523E+00_JPRB, 0.15373E+00_JPRB, 0.15211E+00_JPRB ,&
&  0.15086E+00_JPRB, 0.14977E+00_JPRB, 0.14840E+00_JPRB, 0.14728E+00_JPRB, 0.14601E+00_JPRB, 0.14495E+00_JPRB, 0.14380E+00_JPRB ,&
&  0.14292E+00_JPRB, 0.14215E+00_JPRB, 0.14151E+00_JPRB, 0.14092E+00_JPRB, 0.14055E+00_JPRB, 0.14020E+00_JPRB, 0.13980E+00_JPRB ,&
&  0.13945E+00_JPRB, 0.13943E+00_JPRB, 0.13946E+00_JPRB, 0.13950E+00_JPRB, 0.13960E+00_JPRB, 0.13931E+00_JPRB, 0.13938E+00_JPRB ,&
&  0.13970E+00_JPRB, 0.14003E+00_JPRB, 0.14013E+00_JPRB, 0.14056E+00_JPRB, 0.14124E+00_JPRB, 0.14207E+00_JPRB, 0.14328E+00_JPRB ,&
&  0.14423E+00_JPRB, 0.14519E+00_JPRB, 0.14609E+00_JPRB, 0.14702E+00_JPRB, 0.14822E+00_JPRB, 0.14951E+00_JPRB, 0.15086E+00_JPRB ,&
&  0.15245E+00_JPRB, 0.15394E+00_JPRB, 0.15549E+00_JPRB, 0.15709E+00_JPRB, 0.15965E+00_JPRB, 0.16214E+00_JPRB, 0.16596E+00_JPRB ,&
&  0.16982E+00_JPRB, 0.17172E+00_JPRB, 0.18484E+00_JPRB, 0.18588E+00_JPRB, 0.18743E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.16874E+00_JPRB, 0.17102E+00_JPRB, 0.17259E+00_JPRB, 0.17430E+00_JPRB, 0.17256E+00_JPRB, 0.18411E+00_JPRB, 0.18211E+00_JPRB ,&
&  0.18178E+00_JPRB, 0.17671E+00_JPRB, 0.17705E+00_JPRB, 0.17627E+00_JPRB, 0.17483E+00_JPRB, 0.17321E+00_JPRB, 0.17142E+00_JPRB ,&
&  0.16968E+00_JPRB, 0.16749E+00_JPRB, 0.16567E+00_JPRB, 0.16417E+00_JPRB, 0.16279E+00_JPRB, 0.16159E+00_JPRB, 0.16039E+00_JPRB ,&
&  0.16024E+00_JPRB, 0.16090E+00_JPRB, 0.16194E+00_JPRB, 0.16130E+00_JPRB, 0.16169E+00_JPRB, 0.16254E+00_JPRB, 0.16369E+00_JPRB ,&
&  0.16466E+00_JPRB, 0.16600E+00_JPRB, 0.16686E+00_JPRB, 0.16709E+00_JPRB, 0.16742E+00_JPRB, 0.16672E+00_JPRB, 0.16623E+00_JPRB ,&
&  0.16555E+00_JPRB, 0.16460E+00_JPRB, 0.16339E+00_JPRB, 0.16236E+00_JPRB, 0.16121E+00_JPRB, 0.15973E+00_JPRB, 0.15820E+00_JPRB ,&
&  0.15683E+00_JPRB, 0.15561E+00_JPRB, 0.15432E+00_JPRB, 0.15324E+00_JPRB, 0.15221E+00_JPRB, 0.15119E+00_JPRB, 0.15025E+00_JPRB ,&
&  0.14950E+00_JPRB, 0.14883E+00_JPRB, 0.14810E+00_JPRB, 0.14734E+00_JPRB, 0.14651E+00_JPRB, 0.14571E+00_JPRB, 0.14494E+00_JPRB ,&
&  0.14457E+00_JPRB, 0.14424E+00_JPRB, 0.14391E+00_JPRB, 0.14357E+00_JPRB, 0.14345E+00_JPRB, 0.14333E+00_JPRB, 0.14318E+00_JPRB ,&
&  0.14301E+00_JPRB, 0.14276E+00_JPRB, 0.14268E+00_JPRB, 0.14260E+00_JPRB, 0.14252E+00_JPRB, 0.14246E+00_JPRB, 0.14257E+00_JPRB ,&
&  0.14274E+00_JPRB, 0.14298E+00_JPRB, 0.14347E+00_JPRB, 0.14398E+00_JPRB, 0.14453E+00_JPRB, 0.14506E+00_JPRB, 0.14583E+00_JPRB ,&
&  0.14666E+00_JPRB, 0.14746E+00_JPRB, 0.14815E+00_JPRB, 0.14872E+00_JPRB, 0.14963E+00_JPRB, 0.15049E+00_JPRB, 0.15133E+00_JPRB ,&
&  0.15225E+00_JPRB, 0.15343E+00_JPRB, 0.15465E+00_JPRB, 0.15590E+00_JPRB, 0.15736E+00_JPRB, 0.15859E+00_JPRB, 0.15983E+00_JPRB ,&
&  0.16130E+00_JPRB, 0.16399E+00_JPRB, 0.16581E+00_JPRB, 0.16700E+00_JPRB, 0.16830E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.15459E+00_JPRB, 0.15475E+00_JPRB, 0.15482E+00_JPRB, 0.15483E+00_JPRB, 0.15466E+00_JPRB, 0.15463E+00_JPRB, 0.15460E+00_JPRB ,&
&  0.15458E+00_JPRB, 0.15442E+00_JPRB, 0.15461E+00_JPRB, 0.15471E+00_JPRB, 0.15475E+00_JPRB, 0.15475E+00_JPRB, 0.15463E+00_JPRB ,&
&  0.15435E+00_JPRB, 0.15400E+00_JPRB, 0.15309E+00_JPRB, 0.15293E+00_JPRB, 0.15296E+00_JPRB, 0.15316E+00_JPRB, 0.15330E+00_JPRB ,&
&  0.15354E+00_JPRB, 0.15377E+00_JPRB, 0.15396E+00_JPRB, 0.15453E+00_JPRB, 0.15478E+00_JPRB, 0.15506E+00_JPRB, 0.15534E+00_JPRB ,&
&  0.15560E+00_JPRB, 0.15591E+00_JPRB, 0.15613E+00_JPRB, 0.15638E+00_JPRB, 0.15659E+00_JPRB, 0.15653E+00_JPRB, 0.15632E+00_JPRB ,&
&  0.15601E+00_JPRB, 0.15551E+00_JPRB, 0.15500E+00_JPRB, 0.15453E+00_JPRB, 0.15410E+00_JPRB, 0.15366E+00_JPRB, 0.15324E+00_JPRB ,&
&  0.15279E+00_JPRB, 0.15228E+00_JPRB, 0.15173E+00_JPRB, 0.15124E+00_JPRB, 0.15074E+00_JPRB, 0.15030E+00_JPRB, 0.14981E+00_JPRB ,&
&  0.14942E+00_JPRB, 0.14903E+00_JPRB, 0.14866E+00_JPRB, 0.14828E+00_JPRB, 0.14794E+00_JPRB, 0.14761E+00_JPRB, 0.14728E+00_JPRB ,&
&  0.14693E+00_JPRB, 0.14675E+00_JPRB, 0.14651E+00_JPRB, 0.14629E+00_JPRB, 0.14609E+00_JPRB, 0.14595E+00_JPRB, 0.14575E+00_JPRB ,&
&  0.14557E+00_JPRB, 0.14539E+00_JPRB, 0.14532E+00_JPRB, 0.14519E+00_JPRB, 0.14514E+00_JPRB, 0.14512E+00_JPRB, 0.14518E+00_JPRB ,&
&  0.14524E+00_JPRB, 0.14534E+00_JPRB, 0.14550E+00_JPRB, 0.14576E+00_JPRB, 0.14600E+00_JPRB, 0.14627E+00_JPRB, 0.14653E+00_JPRB ,&
&  0.14683E+00_JPRB, 0.14713E+00_JPRB, 0.14745E+00_JPRB, 0.14775E+00_JPRB, 0.14813E+00_JPRB, 0.14851E+00_JPRB, 0.14894E+00_JPRB ,&
&  0.14933E+00_JPRB, 0.14976E+00_JPRB, 0.15012E+00_JPRB, 0.15055E+00_JPRB, 0.15089E+00_JPRB, 0.15112E+00_JPRB, 0.15132E+00_JPRB ,&
&  0.15168E+00_JPRB, 0.15204E+00_JPRB, 0.15246E+00_JPRB, 0.15285E+00_JPRB, 0.15353E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB, 0.14843E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB, 0.40359E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.39867E+00_JPRB, 0.39874E+00_JPRB, 0.39877E+00_JPRB, 0.39881E+00_JPRB, 0.39884E+00_JPRB, 0.39887E+00_JPRB, 0.39892E+00_JPRB ,&
&  0.39892E+00_JPRB, 0.39905E+00_JPRB, 0.39924E+00_JPRB, 0.39947E+00_JPRB, 0.39968E+00_JPRB, 0.39989E+00_JPRB, 0.40009E+00_JPRB ,&
&  0.40027E+00_JPRB, 0.40037E+00_JPRB, 0.40054E+00_JPRB, 0.40086E+00_JPRB, 0.40129E+00_JPRB, 0.40180E+00_JPRB, 0.40249E+00_JPRB ,&
&  0.40278E+00_JPRB, 0.40278E+00_JPRB, 0.40280E+00_JPRB, 0.40320E+00_JPRB, 0.40317E+00_JPRB, 0.40319E+00_JPRB, 0.40326E+00_JPRB ,&
&  0.40331E+00_JPRB, 0.40330E+00_JPRB, 0.40318E+00_JPRB, 0.40305E+00_JPRB, 0.40302E+00_JPRB, 0.40311E+00_JPRB, 0.40323E+00_JPRB ,&
&  0.40340E+00_JPRB, 0.40351E+00_JPRB, 0.40389E+00_JPRB, 0.40433E+00_JPRB, 0.40483E+00_JPRB, 0.40513E+00_JPRB, 0.40547E+00_JPRB ,&
&  0.40566E+00_JPRB, 0.40588E+00_JPRB, 0.40624E+00_JPRB, 0.40678E+00_JPRB, 0.40739E+00_JPRB, 0.40796E+00_JPRB, 0.40839E+00_JPRB ,&
&  0.40910E+00_JPRB, 0.41002E+00_JPRB, 0.41099E+00_JPRB, 0.41207E+00_JPRB, 0.41292E+00_JPRB, 0.41380E+00_JPRB, 0.41473E+00_JPRB ,&
&  0.41555E+00_JPRB, 0.41605E+00_JPRB, 0.41666E+00_JPRB, 0.41730E+00_JPRB, 0.41780E+00_JPRB, 0.41754E+00_JPRB, 0.41736E+00_JPRB ,&
&  0.41721E+00_JPRB, 0.41686E+00_JPRB, 0.41578E+00_JPRB, 0.41488E+00_JPRB, 0.41402E+00_JPRB, 0.41311E+00_JPRB, 0.41212E+00_JPRB ,&
&  0.41124E+00_JPRB, 0.41045E+00_JPRB, 0.40961E+00_JPRB, 0.40872E+00_JPRB, 0.40790E+00_JPRB, 0.40712E+00_JPRB, 0.40635E+00_JPRB ,&
&  0.40565E+00_JPRB, 0.40504E+00_JPRB, 0.40446E+00_JPRB, 0.40392E+00_JPRB, 0.40333E+00_JPRB, 0.40275E+00_JPRB, 0.40216E+00_JPRB ,&
&  0.40164E+00_JPRB, 0.40110E+00_JPRB, 0.40061E+00_JPRB, 0.40012E+00_JPRB, 0.39977E+00_JPRB, 0.39951E+00_JPRB, 0.39928E+00_JPRB ,&
&  0.39907E+00_JPRB, 0.39894E+00_JPRB, 0.39887E+00_JPRB, 0.39877E+00_JPRB, 0.39868E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.39601E+00_JPRB, 0.39631E+00_JPRB, 0.39657E+00_JPRB, 0.39678E+00_JPRB, 0.39708E+00_JPRB, 0.39745E+00_JPRB, 0.39777E+00_JPRB ,&
&  0.39801E+00_JPRB, 0.39842E+00_JPRB, 0.39892E+00_JPRB, 0.39942E+00_JPRB, 0.39988E+00_JPRB, 0.40042E+00_JPRB, 0.40091E+00_JPRB ,&
&  0.40144E+00_JPRB, 0.40198E+00_JPRB, 0.40256E+00_JPRB, 0.40308E+00_JPRB, 0.40365E+00_JPRB, 0.40410E+00_JPRB, 0.40554E+00_JPRB ,&
&  0.40682E+00_JPRB, 0.40756E+00_JPRB, 0.40741E+00_JPRB, 0.40443E+00_JPRB, 0.40226E+00_JPRB, 0.40035E+00_JPRB, 0.39880E+00_JPRB ,&
&  0.39721E+00_JPRB, 0.39736E+00_JPRB, 0.39777E+00_JPRB, 0.39819E+00_JPRB, 0.39990E+00_JPRB, 0.39964E+00_JPRB, 0.39924E+00_JPRB ,&
&  0.39820E+00_JPRB, 0.39541E+00_JPRB, 0.39569E+00_JPRB, 0.39608E+00_JPRB, 0.39650E+00_JPRB, 0.39782E+00_JPRB, 0.39869E+00_JPRB ,&
&  0.39974E+00_JPRB, 0.40092E+00_JPRB, 0.40329E+00_JPRB, 0.40596E+00_JPRB, 0.40840E+00_JPRB, 0.41044E+00_JPRB, 0.41238E+00_JPRB ,&
&  0.41407E+00_JPRB, 0.41578E+00_JPRB, 0.41762E+00_JPRB, 0.41972E+00_JPRB, 0.42131E+00_JPRB, 0.42307E+00_JPRB, 0.42459E+00_JPRB ,&
&  0.42561E+00_JPRB, 0.42609E+00_JPRB, 0.42741E+00_JPRB, 0.42933E+00_JPRB, 0.43128E+00_JPRB, 0.43167E+00_JPRB, 0.43135E+00_JPRB ,&
&  0.43036E+00_JPRB, 0.42922E+00_JPRB, 0.42729E+00_JPRB, 0.42553E+00_JPRB, 0.42368E+00_JPRB, 0.42182E+00_JPRB, 0.41962E+00_JPRB ,&
&  0.41734E+00_JPRB, 0.41515E+00_JPRB, 0.41346E+00_JPRB, 0.41200E+00_JPRB, 0.41045E+00_JPRB, 0.40898E+00_JPRB, 0.40796E+00_JPRB ,&
&  0.40696E+00_JPRB, 0.40608E+00_JPRB, 0.40513E+00_JPRB, 0.40422E+00_JPRB, 0.40281E+00_JPRB, 0.40141E+00_JPRB, 0.39993E+00_JPRB ,&
&  0.39871E+00_JPRB, 0.39757E+00_JPRB, 0.39661E+00_JPRB, 0.39582E+00_JPRB, 0.39546E+00_JPRB, 0.39529E+00_JPRB, 0.39520E+00_JPRB ,&
&  0.39515E+00_JPRB, 0.39530E+00_JPRB, 0.39550E+00_JPRB, 0.39565E+00_JPRB, 0.39580E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.39878E+00_JPRB, 0.39939E+00_JPRB, 0.40003E+00_JPRB, 0.40043E+00_JPRB, 0.40118E+00_JPRB, 0.40185E+00_JPRB, 0.40248E+00_JPRB ,&
&  0.40308E+00_JPRB, 0.40343E+00_JPRB, 0.40370E+00_JPRB, 0.40399E+00_JPRB, 0.40419E+00_JPRB, 0.40480E+00_JPRB, 0.40542E+00_JPRB ,&
&  0.40601E+00_JPRB, 0.40641E+00_JPRB, 0.40753E+00_JPRB, 0.40866E+00_JPRB, 0.40947E+00_JPRB, 0.41028E+00_JPRB, 0.40807E+00_JPRB ,&
&  0.40670E+00_JPRB, 0.40602E+00_JPRB, 0.40506E+00_JPRB, 0.40237E+00_JPRB, 0.40112E+00_JPRB, 0.39131E+00_JPRB, 0.39434E+00_JPRB ,&
&  0.40621E+00_JPRB, 0.40543E+00_JPRB, 0.40298E+00_JPRB, 0.39872E+00_JPRB, 0.38373E+00_JPRB, 0.37715E+00_JPRB, 0.37613E+00_JPRB ,&
&  0.36624E+00_JPRB, 0.37468E+00_JPRB, 0.37694E+00_JPRB, 0.37784E+00_JPRB, 0.37994E+00_JPRB, 0.37843E+00_JPRB, 0.37871E+00_JPRB ,&
&  0.38516E+00_JPRB, 0.38519E+00_JPRB, 0.38822E+00_JPRB, 0.39193E+00_JPRB, 0.39602E+00_JPRB, 0.39989E+00_JPRB, 0.40444E+00_JPRB ,&
&  0.40824E+00_JPRB, 0.41105E+00_JPRB, 0.41298E+00_JPRB, 0.41372E+00_JPRB, 0.41464E+00_JPRB, 0.41567E+00_JPRB, 0.41775E+00_JPRB ,&
&  0.42121E+00_JPRB, 0.42482E+00_JPRB, 0.42685E+00_JPRB, 0.42637E+00_JPRB, 0.42704E+00_JPRB, 0.42750E+00_JPRB, 0.42789E+00_JPRB ,&
&  0.42762E+00_JPRB, 0.42688E+00_JPRB, 0.42470E+00_JPRB, 0.42407E+00_JPRB, 0.42386E+00_JPRB, 0.42303E+00_JPRB, 0.42172E+00_JPRB ,&
&  0.41969E+00_JPRB, 0.41679E+00_JPRB, 0.41432E+00_JPRB, 0.41289E+00_JPRB, 0.41129E+00_JPRB, 0.40987E+00_JPRB, 0.40910E+00_JPRB ,&
&  0.40789E+00_JPRB, 0.40688E+00_JPRB, 0.40527E+00_JPRB, 0.40305E+00_JPRB, 0.40088E+00_JPRB, 0.39909E+00_JPRB, 0.39730E+00_JPRB ,&
&  0.39636E+00_JPRB, 0.39589E+00_JPRB, 0.39570E+00_JPRB, 0.39536E+00_JPRB, 0.39543E+00_JPRB, 0.39576E+00_JPRB, 0.39619E+00_JPRB ,&
&  0.39665E+00_JPRB, 0.39713E+00_JPRB, 0.39747E+00_JPRB, 0.39790E+00_JPRB, 0.39811E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.40035E+00_JPRB, 0.40057E+00_JPRB, 0.40099E+00_JPRB, 0.40137E+00_JPRB, 0.40210E+00_JPRB, 0.40274E+00_JPRB, 0.40411E+00_JPRB ,&
&  0.40545E+00_JPRB, 0.40672E+00_JPRB, 0.40794E+00_JPRB, 0.40895E+00_JPRB, 0.40998E+00_JPRB, 0.41040E+00_JPRB, 0.41059E+00_JPRB ,&
&  0.41069E+00_JPRB, 0.41062E+00_JPRB, 0.41120E+00_JPRB, 0.41201E+00_JPRB, 0.41239E+00_JPRB, 0.41313E+00_JPRB, 0.41052E+00_JPRB ,&
&  0.40867E+00_JPRB, 0.40772E+00_JPRB, 0.39789E+00_JPRB, 0.40293E+00_JPRB, 0.40208E+00_JPRB, 0.40283E+00_JPRB, 0.40135E+00_JPRB ,&
&  0.39096E+00_JPRB, 0.38656E+00_JPRB, 0.37503E+00_JPRB, 0.36508E+00_JPRB, 0.36530E+00_JPRB, 0.35778E+00_JPRB, 0.35804E+00_JPRB ,&
&  0.35622E+00_JPRB, 0.35332E+00_JPRB, 0.35210E+00_JPRB, 0.35139E+00_JPRB, 0.35333E+00_JPRB, 0.35497E+00_JPRB, 0.35684E+00_JPRB ,&
&  0.36121E+00_JPRB, 0.36495E+00_JPRB, 0.37045E+00_JPRB, 0.37085E+00_JPRB, 0.37693E+00_JPRB, 0.38364E+00_JPRB, 0.39061E+00_JPRB ,&
&  0.39503E+00_JPRB, 0.39432E+00_JPRB, 0.39688E+00_JPRB, 0.40097E+00_JPRB, 0.40439E+00_JPRB, 0.40786E+00_JPRB, 0.41251E+00_JPRB ,&
&  0.41816E+00_JPRB, 0.42341E+00_JPRB, 0.42518E+00_JPRB, 0.42407E+00_JPRB, 0.42204E+00_JPRB, 0.42022E+00_JPRB, 0.41967E+00_JPRB ,&
&  0.41912E+00_JPRB, 0.42007E+00_JPRB, 0.41995E+00_JPRB, 0.42083E+00_JPRB, 0.42160E+00_JPRB, 0.42383E+00_JPRB, 0.42552E+00_JPRB ,&
&  0.42614E+00_JPRB, 0.42571E+00_JPRB, 0.42607E+00_JPRB, 0.42286E+00_JPRB, 0.42164E+00_JPRB, 0.42060E+00_JPRB, 0.41693E+00_JPRB ,&
&  0.41374E+00_JPRB, 0.41104E+00_JPRB, 0.40830E+00_JPRB, 0.40589E+00_JPRB, 0.40408E+00_JPRB, 0.40278E+00_JPRB, 0.40212E+00_JPRB ,&
&  0.40198E+00_JPRB, 0.40168E+00_JPRB, 0.40116E+00_JPRB, 0.39999E+00_JPRB, 0.39984E+00_JPRB, 0.39886E+00_JPRB, 0.39839E+00_JPRB ,&
&  0.39816E+00_JPRB, 0.39873E+00_JPRB, 0.39879E+00_JPRB, 0.39940E+00_JPRB, 0.40010E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.39913E+00_JPRB, 0.40114E+00_JPRB, 0.40437E+00_JPRB, 0.40719E+00_JPRB, 0.40974E+00_JPRB, 0.41186E+00_JPRB, 0.41301E+00_JPRB ,&
&  0.41383E+00_JPRB, 0.41414E+00_JPRB, 0.41447E+00_JPRB, 0.41494E+00_JPRB, 0.41576E+00_JPRB, 0.41642E+00_JPRB, 0.41731E+00_JPRB ,&
&  0.41765E+00_JPRB, 0.41777E+00_JPRB, 0.41761E+00_JPRB, 0.41743E+00_JPRB, 0.41740E+00_JPRB, 0.41799E+00_JPRB, 0.41725E+00_JPRB ,&
&  0.41753E+00_JPRB, 0.41518E+00_JPRB, 0.40791E+00_JPRB, 0.41034E+00_JPRB, 0.41125E+00_JPRB, 0.41231E+00_JPRB, 0.40855E+00_JPRB ,&
&  0.39865E+00_JPRB, 0.37661E+00_JPRB, 0.38688E+00_JPRB, 0.37845E+00_JPRB, 0.36967E+00_JPRB, 0.36083E+00_JPRB, 0.34872E+00_JPRB ,&
&  0.34141E+00_JPRB, 0.33219E+00_JPRB, 0.32541E+00_JPRB, 0.32318E+00_JPRB, 0.32316E+00_JPRB, 0.32707E+00_JPRB, 0.33240E+00_JPRB ,&
&  0.33522E+00_JPRB, 0.34313E+00_JPRB, 0.34725E+00_JPRB, 0.35227E+00_JPRB, 0.35852E+00_JPRB, 0.36657E+00_JPRB, 0.36990E+00_JPRB ,&
&  0.37576E+00_JPRB, 0.37889E+00_JPRB, 0.38510E+00_JPRB, 0.39275E+00_JPRB, 0.40348E+00_JPRB, 0.41000E+00_JPRB, 0.41398E+00_JPRB ,&
&  0.41773E+00_JPRB, 0.41914E+00_JPRB, 0.41952E+00_JPRB, 0.41963E+00_JPRB, 0.42045E+00_JPRB, 0.42159E+00_JPRB, 0.42312E+00_JPRB ,&
&  0.42487E+00_JPRB, 0.42741E+00_JPRB, 0.43021E+00_JPRB, 0.43344E+00_JPRB, 0.43551E+00_JPRB, 0.43983E+00_JPRB, 0.44469E+00_JPRB ,&
&  0.44856E+00_JPRB, 0.45018E+00_JPRB, 0.45342E+00_JPRB, 0.44905E+00_JPRB, 0.44705E+00_JPRB, 0.43944E+00_JPRB, 0.43391E+00_JPRB ,&
&  0.42999E+00_JPRB, 0.42475E+00_JPRB, 0.42044E+00_JPRB, 0.41694E+00_JPRB, 0.41451E+00_JPRB, 0.41208E+00_JPRB, 0.41047E+00_JPRB ,&
&  0.40874E+00_JPRB, 0.40655E+00_JPRB, 0.40488E+00_JPRB, 0.40268E+00_JPRB, 0.40129E+00_JPRB, 0.39970E+00_JPRB, 0.39905E+00_JPRB ,&
&  0.39870E+00_JPRB, 0.39870E+00_JPRB, 0.39844E+00_JPRB, 0.39891E+00_JPRB, 0.39917E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.40339E+00_JPRB, 0.40687E+00_JPRB, 0.41108E+00_JPRB, 0.41476E+00_JPRB, 0.41664E+00_JPRB, 0.41818E+00_JPRB, 0.41757E+00_JPRB ,&
&  0.41715E+00_JPRB, 0.41621E+00_JPRB, 0.41549E+00_JPRB, 0.41534E+00_JPRB, 0.41594E+00_JPRB, 0.41689E+00_JPRB, 0.41823E+00_JPRB ,&
&  0.41943E+00_JPRB, 0.42097E+00_JPRB, 0.42150E+00_JPRB, 0.42223E+00_JPRB, 0.42306E+00_JPRB, 0.42526E+00_JPRB, 0.42598E+00_JPRB ,&
&  0.42708E+00_JPRB, 0.42380E+00_JPRB, 0.41622E+00_JPRB, 0.41376E+00_JPRB, 0.41743E+00_JPRB, 0.41093E+00_JPRB, 0.41270E+00_JPRB ,&
&  0.41760E+00_JPRB, 0.41859E+00_JPRB, 0.41177E+00_JPRB, 0.40646E+00_JPRB, 0.39332E+00_JPRB, 0.34827E+00_JPRB, 0.34611E+00_JPRB ,&
&  0.33765E+00_JPRB, 0.32379E+00_JPRB, 0.31665E+00_JPRB, 0.31055E+00_JPRB, 0.30802E+00_JPRB, 0.31347E+00_JPRB, 0.32215E+00_JPRB ,&
&  0.32501E+00_JPRB, 0.33662E+00_JPRB, 0.34076E+00_JPRB, 0.34687E+00_JPRB, 0.35368E+00_JPRB, 0.36301E+00_JPRB, 0.36859E+00_JPRB ,&
&  0.37475E+00_JPRB, 0.38119E+00_JPRB, 0.38976E+00_JPRB, 0.40074E+00_JPRB, 0.40976E+00_JPRB, 0.41400E+00_JPRB, 0.41274E+00_JPRB ,&
&  0.41574E+00_JPRB, 0.41181E+00_JPRB, 0.41361E+00_JPRB, 0.41626E+00_JPRB, 0.41765E+00_JPRB, 0.42059E+00_JPRB, 0.42156E+00_JPRB ,&
&  0.42631E+00_JPRB, 0.43434E+00_JPRB, 0.44361E+00_JPRB, 0.44851E+00_JPRB, 0.45263E+00_JPRB, 0.45755E+00_JPRB, 0.46536E+00_JPRB ,&
&  0.47239E+00_JPRB, 0.48095E+00_JPRB, 0.48781E+00_JPRB, 0.47868E+00_JPRB, 0.47484E+00_JPRB, 0.46487E+00_JPRB, 0.45759E+00_JPRB ,&
&  0.44906E+00_JPRB, 0.44301E+00_JPRB, 0.43303E+00_JPRB, 0.42788E+00_JPRB, 0.42286E+00_JPRB, 0.41842E+00_JPRB, 0.41496E+00_JPRB ,&
&  0.41227E+00_JPRB, 0.40974E+00_JPRB, 0.40782E+00_JPRB, 0.40644E+00_JPRB, 0.40476E+00_JPRB, 0.40289E+00_JPRB, 0.40189E+00_JPRB ,&
&  0.39992E+00_JPRB, 0.39886E+00_JPRB, 0.39805E+00_JPRB, 0.39856E+00_JPRB, 0.39995E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.40204E+00_JPRB, 0.40690E+00_JPRB, 0.41128E+00_JPRB, 0.41713E+00_JPRB, 0.41831E+00_JPRB, 0.42054E+00_JPRB, 0.41753E+00_JPRB ,&
&  0.41535E+00_JPRB, 0.41427E+00_JPRB, 0.41378E+00_JPRB, 0.41487E+00_JPRB, 0.41637E+00_JPRB, 0.41808E+00_JPRB, 0.42056E+00_JPRB ,&
&  0.42280E+00_JPRB, 0.42098E+00_JPRB, 0.42194E+00_JPRB, 0.42536E+00_JPRB, 0.42657E+00_JPRB, 0.42909E+00_JPRB, 0.43049E+00_JPRB ,&
&  0.43038E+00_JPRB, 0.42793E+00_JPRB, 0.42549E+00_JPRB, 0.41988E+00_JPRB, 0.41794E+00_JPRB, 0.42010E+00_JPRB, 0.42276E+00_JPRB ,&
&  0.41477E+00_JPRB, 0.41091E+00_JPRB, 0.42070E+00_JPRB, 0.42278E+00_JPRB, 0.39972E+00_JPRB, 0.37793E+00_JPRB, 0.36176E+00_JPRB ,&
&  0.34543E+00_JPRB, 0.32398E+00_JPRB, 0.31614E+00_JPRB, 0.31096E+00_JPRB, 0.31280E+00_JPRB, 0.31602E+00_JPRB, 0.32368E+00_JPRB ,&
&  0.32980E+00_JPRB, 0.33756E+00_JPRB, 0.34601E+00_JPRB, 0.35660E+00_JPRB, 0.36409E+00_JPRB, 0.37210E+00_JPRB, 0.37904E+00_JPRB ,&
&  0.38647E+00_JPRB, 0.39514E+00_JPRB, 0.40569E+00_JPRB, 0.41852E+00_JPRB, 0.42683E+00_JPRB, 0.43255E+00_JPRB, 0.43373E+00_JPRB ,&
&  0.44645E+00_JPRB, 0.43706E+00_JPRB, 0.43893E+00_JPRB, 0.43975E+00_JPRB, 0.43974E+00_JPRB, 0.44140E+00_JPRB, 0.44248E+00_JPRB ,&
&  0.44974E+00_JPRB, 0.46109E+00_JPRB, 0.46461E+00_JPRB, 0.46698E+00_JPRB, 0.46902E+00_JPRB, 0.47189E+00_JPRB, 0.48153E+00_JPRB ,&
&  0.49106E+00_JPRB, 0.51049E+00_JPRB, 0.55794E+00_JPRB, 0.53317E+00_JPRB, 0.51697E+00_JPRB, 0.50073E+00_JPRB, 0.48465E+00_JPRB ,&
&  0.47009E+00_JPRB, 0.45699E+00_JPRB, 0.44405E+00_JPRB, 0.43243E+00_JPRB, 0.42444E+00_JPRB, 0.41666E+00_JPRB, 0.41560E+00_JPRB ,&
&  0.41439E+00_JPRB, 0.41230E+00_JPRB, 0.41034E+00_JPRB, 0.40845E+00_JPRB, 0.40489E+00_JPRB, 0.40201E+00_JPRB, 0.40052E+00_JPRB ,&
&  0.39779E+00_JPRB, 0.39764E+00_JPRB, 0.39700E+00_JPRB, 0.39790E+00_JPRB, 0.39895E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.40092E+00_JPRB, 0.41098E+00_JPRB, 0.41939E+00_JPRB, 0.42533E+00_JPRB, 0.42519E+00_JPRB, 0.42605E+00_JPRB, 0.42086E+00_JPRB ,&
&  0.41818E+00_JPRB, 0.41877E+00_JPRB, 0.41975E+00_JPRB, 0.42058E+00_JPRB, 0.42122E+00_JPRB, 0.42154E+00_JPRB, 0.42233E+00_JPRB ,&
&  0.42381E+00_JPRB, 0.42566E+00_JPRB, 0.42657E+00_JPRB, 0.42789E+00_JPRB, 0.42990E+00_JPRB, 0.43267E+00_JPRB, 0.43543E+00_JPRB ,&
&  0.43605E+00_JPRB, 0.43376E+00_JPRB, 0.43398E+00_JPRB, 0.43054E+00_JPRB, 0.42695E+00_JPRB, 0.42324E+00_JPRB, 0.42121E+00_JPRB ,&
&  0.42556E+00_JPRB, 0.43172E+00_JPRB, 0.42739E+00_JPRB, 0.42359E+00_JPRB, 0.42144E+00_JPRB, 0.40192E+00_JPRB, 0.37818E+00_JPRB ,&
&  0.35458E+00_JPRB, 0.33430E+00_JPRB, 0.32466E+00_JPRB, 0.32025E+00_JPRB, 0.32417E+00_JPRB, 0.33338E+00_JPRB, 0.34324E+00_JPRB ,&
&  0.34732E+00_JPRB, 0.36047E+00_JPRB, 0.37379E+00_JPRB, 0.38222E+00_JPRB, 0.38913E+00_JPRB, 0.39574E+00_JPRB, 0.40217E+00_JPRB ,&
&  0.41240E+00_JPRB, 0.42599E+00_JPRB, 0.44770E+00_JPRB, 0.46913E+00_JPRB, 0.48485E+00_JPRB, 0.48889E+00_JPRB, 0.51334E+00_JPRB ,&
&  0.57739E+00_JPRB, 0.53435E+00_JPRB, 0.51118E+00_JPRB, 0.50071E+00_JPRB, 0.49080E+00_JPRB, 0.48695E+00_JPRB, 0.48941E+00_JPRB ,&
&  0.49591E+00_JPRB, 0.50221E+00_JPRB, 0.50249E+00_JPRB, 0.50048E+00_JPRB, 0.50107E+00_JPRB, 0.50509E+00_JPRB, 0.51542E+00_JPRB ,&
&  0.52790E+00_JPRB, 0.56424E+00_JPRB, 0.65722E+00_JPRB, 0.57618E+00_JPRB, 0.53347E+00_JPRB, 0.51272E+00_JPRB, 0.49438E+00_JPRB ,&
&  0.47928E+00_JPRB, 0.46632E+00_JPRB, 0.45283E+00_JPRB, 0.43605E+00_JPRB, 0.42366E+00_JPRB, 0.41313E+00_JPRB, 0.40967E+00_JPRB ,&
&  0.40623E+00_JPRB, 0.40274E+00_JPRB, 0.40226E+00_JPRB, 0.40038E+00_JPRB, 0.39793E+00_JPRB, 0.39892E+00_JPRB, 0.39924E+00_JPRB ,&
&  0.39617E+00_JPRB, 0.39441E+00_JPRB, 0.39357E+00_JPRB, 0.39364E+00_JPRB, 0.39504E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.41126E+00_JPRB, 0.41973E+00_JPRB, 0.43176E+00_JPRB, 0.45003E+00_JPRB, 0.45926E+00_JPRB, 0.45330E+00_JPRB, 0.43825E+00_JPRB ,&
&  0.42949E+00_JPRB, 0.42552E+00_JPRB, 0.42858E+00_JPRB, 0.43261E+00_JPRB, 0.43436E+00_JPRB, 0.43409E+00_JPRB, 0.43250E+00_JPRB ,&
&  0.42911E+00_JPRB, 0.42747E+00_JPRB, 0.42761E+00_JPRB, 0.42907E+00_JPRB, 0.43137E+00_JPRB, 0.43375E+00_JPRB, 0.43720E+00_JPRB ,&
&  0.43988E+00_JPRB, 0.43945E+00_JPRB, 0.43814E+00_JPRB, 0.43523E+00_JPRB, 0.43458E+00_JPRB, 0.43656E+00_JPRB, 0.43586E+00_JPRB ,&
&  0.43082E+00_JPRB, 0.43372E+00_JPRB, 0.43800E+00_JPRB, 0.43183E+00_JPRB, 0.42931E+00_JPRB, 0.41563E+00_JPRB, 0.39247E+00_JPRB ,&
&  0.37178E+00_JPRB, 0.35243E+00_JPRB, 0.34658E+00_JPRB, 0.34905E+00_JPRB, 0.35713E+00_JPRB, 0.36604E+00_JPRB, 0.37862E+00_JPRB ,&
&  0.38972E+00_JPRB, 0.39919E+00_JPRB, 0.40631E+00_JPRB, 0.40993E+00_JPRB, 0.41445E+00_JPRB, 0.42236E+00_JPRB, 0.43817E+00_JPRB ,&
&  0.46701E+00_JPRB, 0.50315E+00_JPRB, 0.54001E+00_JPRB, 0.51931E+00_JPRB, 0.51944E+00_JPRB, 0.53404E+00_JPRB, 0.55496E+00_JPRB ,&
&  0.57230E+00_JPRB, 0.56635E+00_JPRB, 0.55960E+00_JPRB, 0.57570E+00_JPRB, 0.58405E+00_JPRB, 0.59407E+00_JPRB, 0.58961E+00_JPRB ,&
&  0.57819E+00_JPRB, 0.56525E+00_JPRB, 0.55710E+00_JPRB, 0.55674E+00_JPRB, 0.56601E+00_JPRB, 0.57488E+00_JPRB, 0.57535E+00_JPRB ,&
&  0.56286E+00_JPRB, 0.54103E+00_JPRB, 0.51568E+00_JPRB, 0.48926E+00_JPRB, 0.48334E+00_JPRB, 0.48527E+00_JPRB, 0.48529E+00_JPRB ,&
&  0.47673E+00_JPRB, 0.46553E+00_JPRB, 0.45428E+00_JPRB, 0.43773E+00_JPRB, 0.42312E+00_JPRB, 0.40811E+00_JPRB, 0.39964E+00_JPRB ,&
&  0.39400E+00_JPRB, 0.39104E+00_JPRB, 0.39331E+00_JPRB, 0.38829E+00_JPRB, 0.39007E+00_JPRB, 0.39416E+00_JPRB, 0.39217E+00_JPRB ,&
&  0.39043E+00_JPRB, 0.39065E+00_JPRB, 0.38839E+00_JPRB, 0.39112E+00_JPRB, 0.39722E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.43536E+00_JPRB, 0.45936E+00_JPRB, 0.48094E+00_JPRB, 0.49693E+00_JPRB, 0.50041E+00_JPRB, 0.49962E+00_JPRB, 0.48651E+00_JPRB ,&
&  0.48666E+00_JPRB, 0.46406E+00_JPRB, 0.44980E+00_JPRB, 0.43360E+00_JPRB, 0.42105E+00_JPRB, 0.41289E+00_JPRB, 0.41360E+00_JPRB ,&
&  0.41767E+00_JPRB, 0.42248E+00_JPRB, 0.42545E+00_JPRB, 0.42613E+00_JPRB, 0.42828E+00_JPRB, 0.43178E+00_JPRB, 0.43658E+00_JPRB ,&
&  0.44178E+00_JPRB, 0.44450E+00_JPRB, 0.44633E+00_JPRB, 0.44701E+00_JPRB, 0.44721E+00_JPRB, 0.44669E+00_JPRB, 0.44273E+00_JPRB ,&
&  0.44262E+00_JPRB, 0.44039E+00_JPRB, 0.44092E+00_JPRB, 0.43785E+00_JPRB, 0.43619E+00_JPRB, 0.42626E+00_JPRB, 0.40099E+00_JPRB ,&
&  0.38555E+00_JPRB, 0.37813E+00_JPRB, 0.38190E+00_JPRB, 0.39254E+00_JPRB, 0.40120E+00_JPRB, 0.41115E+00_JPRB, 0.41716E+00_JPRB ,&
&  0.41594E+00_JPRB, 0.41662E+00_JPRB, 0.42579E+00_JPRB, 0.43484E+00_JPRB, 0.44215E+00_JPRB, 0.45388E+00_JPRB, 0.46740E+00_JPRB ,&
&  0.49237E+00_JPRB, 0.53122E+00_JPRB, 0.59372E+00_JPRB, 0.60608E+00_JPRB, 0.66074E+00_JPRB, 0.78570E+00_JPRB, 0.69127E+00_JPRB ,&
&  0.68987E+00_JPRB, 0.69213E+00_JPRB, 0.70681E+00_JPRB, 0.73992E+00_JPRB, 0.71869E+00_JPRB, 0.70580E+00_JPRB, 0.69589E+00_JPRB ,&
&  0.66596E+00_JPRB, 0.65308E+00_JPRB, 0.64336E+00_JPRB, 0.65591E+00_JPRB, 0.65282E+00_JPRB, 0.64946E+00_JPRB, 0.64272E+00_JPRB ,&
&  0.61612E+00_JPRB, 0.58215E+00_JPRB, 0.54718E+00_JPRB, 0.51439E+00_JPRB, 0.49787E+00_JPRB, 0.48826E+00_JPRB, 0.48187E+00_JPRB ,&
&  0.47346E+00_JPRB, 0.46671E+00_JPRB, 0.46069E+00_JPRB, 0.44998E+00_JPRB, 0.43315E+00_JPRB, 0.41415E+00_JPRB, 0.40585E+00_JPRB ,&
&  0.39834E+00_JPRB, 0.39274E+00_JPRB, 0.39011E+00_JPRB, 0.38803E+00_JPRB, 0.39142E+00_JPRB, 0.39366E+00_JPRB, 0.39389E+00_JPRB ,&
&  0.39643E+00_JPRB, 0.39363E+00_JPRB, 0.39108E+00_JPRB, 0.39567E+00_JPRB, 0.41339E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.47438E+00_JPRB, 0.50607E+00_JPRB, 0.52042E+00_JPRB, 0.52938E+00_JPRB, 0.53725E+00_JPRB, 0.53572E+00_JPRB, 0.50946E+00_JPRB ,&
&  0.50768E+00_JPRB, 0.45914E+00_JPRB, 0.44196E+00_JPRB, 0.42719E+00_JPRB, 0.42041E+00_JPRB, 0.41330E+00_JPRB, 0.40453E+00_JPRB ,&
&  0.40603E+00_JPRB, 0.41443E+00_JPRB, 0.42411E+00_JPRB, 0.42845E+00_JPRB, 0.43197E+00_JPRB, 0.43454E+00_JPRB, 0.43718E+00_JPRB ,&
&  0.44170E+00_JPRB, 0.44736E+00_JPRB, 0.45306E+00_JPRB, 0.45485E+00_JPRB, 0.45443E+00_JPRB, 0.45359E+00_JPRB, 0.45193E+00_JPRB ,&
&  0.45057E+00_JPRB, 0.44406E+00_JPRB, 0.44393E+00_JPRB, 0.44334E+00_JPRB, 0.43943E+00_JPRB, 0.43847E+00_JPRB, 0.42586E+00_JPRB ,&
&  0.40326E+00_JPRB, 0.41139E+00_JPRB, 0.42152E+00_JPRB, 0.43173E+00_JPRB, 0.44040E+00_JPRB, 0.44752E+00_JPRB, 0.45149E+00_JPRB ,&
&  0.45265E+00_JPRB, 0.45281E+00_JPRB, 0.45553E+00_JPRB, 0.46030E+00_JPRB, 0.46604E+00_JPRB, 0.47353E+00_JPRB, 0.48030E+00_JPRB ,&
&  0.49043E+00_JPRB, 0.47920E+00_JPRB, 0.52306E+00_JPRB, 0.64138E+00_JPRB, 0.90283E+00_JPRB, 0.87342E+00_JPRB, 0.87663E+00_JPRB ,&
&  0.91303E+00_JPRB, 0.87994E+00_JPRB, 0.87048E+00_JPRB, 0.86448E+00_JPRB, 0.84760E+00_JPRB, 0.82944E+00_JPRB, 0.82761E+00_JPRB ,&
&  0.79938E+00_JPRB, 0.80799E+00_JPRB, 0.79163E+00_JPRB, 0.78284E+00_JPRB, 0.76087E+00_JPRB, 0.75063E+00_JPRB, 0.73275E+00_JPRB ,&
&  0.70370E+00_JPRB, 0.67687E+00_JPRB, 0.63182E+00_JPRB, 0.58046E+00_JPRB, 0.54032E+00_JPRB, 0.51907E+00_JPRB, 0.50029E+00_JPRB ,&
&  0.48675E+00_JPRB, 0.47905E+00_JPRB, 0.47021E+00_JPRB, 0.46494E+00_JPRB, 0.45416E+00_JPRB, 0.44248E+00_JPRB, 0.43729E+00_JPRB ,&
&  0.42231E+00_JPRB, 0.41342E+00_JPRB, 0.40608E+00_JPRB, 0.40302E+00_JPRB, 0.40876E+00_JPRB, 0.41623E+00_JPRB, 0.42222E+00_JPRB ,&
&  0.42964E+00_JPRB, 0.42676E+00_JPRB, 0.42669E+00_JPRB, 0.43281E+00_JPRB, 0.44727E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.50812E+00_JPRB, 0.53161E+00_JPRB, 0.54269E+00_JPRB, 0.55464E+00_JPRB, 0.56866E+00_JPRB, 0.56683E+00_JPRB, 0.56531E+00_JPRB ,&
&  0.59084E+00_JPRB, 0.55625E+00_JPRB, 0.59316E+00_JPRB, 0.48404E+00_JPRB, 0.44195E+00_JPRB, 0.43242E+00_JPRB, 0.42048E+00_JPRB ,&
&  0.41876E+00_JPRB, 0.42392E+00_JPRB, 0.42893E+00_JPRB, 0.43314E+00_JPRB, 0.43876E+00_JPRB, 0.43884E+00_JPRB, 0.43908E+00_JPRB ,&
&  0.44173E+00_JPRB, 0.44823E+00_JPRB, 0.45960E+00_JPRB, 0.46298E+00_JPRB, 0.46611E+00_JPRB, 0.47000E+00_JPRB, 0.46675E+00_JPRB ,&
&  0.45809E+00_JPRB, 0.45705E+00_JPRB, 0.45729E+00_JPRB, 0.45098E+00_JPRB, 0.44460E+00_JPRB, 0.44509E+00_JPRB, 0.45603E+00_JPRB ,&
&  0.45751E+00_JPRB, 0.44659E+00_JPRB, 0.46020E+00_JPRB, 0.46738E+00_JPRB, 0.47179E+00_JPRB, 0.47373E+00_JPRB, 0.47429E+00_JPRB ,&
&  0.47589E+00_JPRB, 0.47639E+00_JPRB, 0.48027E+00_JPRB, 0.48279E+00_JPRB, 0.48617E+00_JPRB, 0.49233E+00_JPRB, 0.49379E+00_JPRB ,&
&  0.51003E+00_JPRB, 0.50133E+00_JPRB, 0.61299E+00_JPRB, 0.73071E+00_JPRB, 0.97665E+00_JPRB, 0.10963E+01_JPRB, 0.10694E+01_JPRB ,&
&  0.11640E+01_JPRB, 0.10822E+01_JPRB, 0.11071E+01_JPRB, 0.10693E+01_JPRB, 0.10320E+01_JPRB, 0.98677E+00_JPRB, 0.93909E+00_JPRB ,&
&  0.92887E+00_JPRB, 0.93240E+00_JPRB, 0.96193E+00_JPRB, 0.93357E+00_JPRB, 0.88024E+00_JPRB, 0.86215E+00_JPRB, 0.85127E+00_JPRB ,&
&  0.84533E+00_JPRB, 0.83254E+00_JPRB, 0.79093E+00_JPRB, 0.68256E+00_JPRB, 0.62907E+00_JPRB, 0.58653E+00_JPRB, 0.53550E+00_JPRB ,&
&  0.51066E+00_JPRB, 0.48575E+00_JPRB, 0.46765E+00_JPRB, 0.46353E+00_JPRB, 0.45559E+00_JPRB, 0.44882E+00_JPRB, 0.44709E+00_JPRB ,&
&  0.43411E+00_JPRB, 0.43086E+00_JPRB, 0.42991E+00_JPRB, 0.43389E+00_JPRB, 0.43830E+00_JPRB, 0.44507E+00_JPRB, 0.45424E+00_JPRB ,&
&  0.45361E+00_JPRB, 0.45204E+00_JPRB, 0.46173E+00_JPRB, 0.46908E+00_JPRB, 0.48487E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.52572E+00_JPRB, 0.54610E+00_JPRB, 0.56324E+00_JPRB, 0.57935E+00_JPRB, 0.60079E+00_JPRB, 0.62378E+00_JPRB, 0.62340E+00_JPRB ,&
&  0.63712E+00_JPRB, 0.64145E+00_JPRB, 0.63825E+00_JPRB, 0.62588E+00_JPRB, 0.60105E+00_JPRB, 0.52978E+00_JPRB, 0.44176E+00_JPRB ,&
&  0.41660E+00_JPRB, 0.42132E+00_JPRB, 0.43136E+00_JPRB, 0.43550E+00_JPRB, 0.43746E+00_JPRB, 0.44026E+00_JPRB, 0.44011E+00_JPRB ,&
&  0.44361E+00_JPRB, 0.44926E+00_JPRB, 0.45983E+00_JPRB, 0.47348E+00_JPRB, 0.48560E+00_JPRB, 0.48977E+00_JPRB, 0.48713E+00_JPRB ,&
&  0.47551E+00_JPRB, 0.47015E+00_JPRB, 0.46242E+00_JPRB, 0.45119E+00_JPRB, 0.45652E+00_JPRB, 0.46448E+00_JPRB, 0.48166E+00_JPRB ,&
&  0.49705E+00_JPRB, 0.50836E+00_JPRB, 0.51318E+00_JPRB, 0.51858E+00_JPRB, 0.52152E+00_JPRB, 0.51873E+00_JPRB, 0.51249E+00_JPRB ,&
&  0.50713E+00_JPRB, 0.50339E+00_JPRB, 0.50110E+00_JPRB, 0.50329E+00_JPRB, 0.50376E+00_JPRB, 0.51683E+00_JPRB, 0.53588E+00_JPRB ,&
&  0.56111E+00_JPRB, 0.65438E+00_JPRB, 0.79444E+00_JPRB, 0.82309E+00_JPRB, 0.10641E+01_JPRB, 0.11613E+01_JPRB, 0.11968E+01_JPRB ,&
&  0.12031E+01_JPRB, 0.11240E+01_JPRB, 0.12348E+01_JPRB, 0.12499E+01_JPRB, 0.12455E+01_JPRB, 0.11955E+01_JPRB, 0.11028E+01_JPRB ,&
&  0.10562E+01_JPRB, 0.11254E+01_JPRB, 0.11465E+01_JPRB, 0.11281E+01_JPRB, 0.10759E+01_JPRB, 0.10661E+01_JPRB, 0.10600E+01_JPRB ,&
&  0.10789E+01_JPRB, 0.10890E+01_JPRB, 0.10293E+01_JPRB, 0.91539E+00_JPRB, 0.79026E+00_JPRB, 0.68732E+00_JPRB, 0.60731E+00_JPRB ,&
&  0.54534E+00_JPRB, 0.48714E+00_JPRB, 0.46621E+00_JPRB, 0.46037E+00_JPRB, 0.45984E+00_JPRB, 0.45058E+00_JPRB, 0.44798E+00_JPRB ,&
&  0.44795E+00_JPRB, 0.45010E+00_JPRB, 0.45568E+00_JPRB, 0.45971E+00_JPRB, 0.47031E+00_JPRB, 0.47751E+00_JPRB, 0.46684E+00_JPRB ,&
&  0.45890E+00_JPRB, 0.46621E+00_JPRB, 0.47431E+00_JPRB, 0.48573E+00_JPRB, 0.50442E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.54918E+00_JPRB, 0.57190E+00_JPRB, 0.58936E+00_JPRB, 0.60467E+00_JPRB, 0.62972E+00_JPRB, 0.66659E+00_JPRB, 0.68314E+00_JPRB ,&
&  0.68347E+00_JPRB, 0.68110E+00_JPRB, 0.67671E+00_JPRB, 0.66574E+00_JPRB, 0.64446E+00_JPRB, 0.61070E+00_JPRB, 0.53364E+00_JPRB ,&
&  0.45037E+00_JPRB, 0.44596E+00_JPRB, 0.43416E+00_JPRB, 0.43970E+00_JPRB, 0.44473E+00_JPRB, 0.44255E+00_JPRB, 0.43878E+00_JPRB ,&
&  0.44380E+00_JPRB, 0.44951E+00_JPRB, 0.46128E+00_JPRB, 0.47572E+00_JPRB, 0.47883E+00_JPRB, 0.48309E+00_JPRB, 0.49100E+00_JPRB ,&
&  0.48104E+00_JPRB, 0.46753E+00_JPRB, 0.45791E+00_JPRB, 0.45488E+00_JPRB, 0.46950E+00_JPRB, 0.49344E+00_JPRB, 0.51566E+00_JPRB ,&
&  0.53210E+00_JPRB, 0.54284E+00_JPRB, 0.55148E+00_JPRB, 0.55837E+00_JPRB, 0.56216E+00_JPRB, 0.55909E+00_JPRB, 0.54914E+00_JPRB ,&
&  0.53668E+00_JPRB, 0.52878E+00_JPRB, 0.52443E+00_JPRB, 0.52876E+00_JPRB, 0.53569E+00_JPRB, 0.57237E+00_JPRB, 0.61877E+00_JPRB ,&
&  0.68530E+00_JPRB, 0.79590E+00_JPRB, 0.88487E+00_JPRB, 0.10079E+01_JPRB, 0.11891E+01_JPRB, 0.12989E+01_JPRB, 0.13313E+01_JPRB ,&
&  0.13592E+01_JPRB, 0.13034E+01_JPRB, 0.14861E+01_JPRB, 0.14814E+01_JPRB, 0.13934E+01_JPRB, 0.13618E+01_JPRB, 0.12561E+01_JPRB ,&
&  0.11748E+01_JPRB, 0.11216E+01_JPRB, 0.11883E+01_JPRB, 0.12336E+01_JPRB, 0.12611E+01_JPRB, 0.13014E+01_JPRB, 0.13047E+01_JPRB ,&
&  0.13135E+01_JPRB, 0.12997E+01_JPRB, 0.11149E+01_JPRB, 0.92915E+00_JPRB, 0.75237E+00_JPRB, 0.72074E+00_JPRB, 0.64253E+00_JPRB ,&
&  0.56121E+00_JPRB, 0.48673E+00_JPRB, 0.47791E+00_JPRB, 0.47200E+00_JPRB, 0.47913E+00_JPRB, 0.47729E+00_JPRB, 0.46973E+00_JPRB ,&
&  0.47438E+00_JPRB, 0.48478E+00_JPRB, 0.48404E+00_JPRB, 0.48960E+00_JPRB, 0.50206E+00_JPRB, 0.50517E+00_JPRB, 0.48856E+00_JPRB ,&
&  0.48475E+00_JPRB, 0.48901E+00_JPRB, 0.48629E+00_JPRB, 0.49752E+00_JPRB, 0.52023E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.57580E+00_JPRB, 0.59576E+00_JPRB, 0.61168E+00_JPRB, 0.63441E+00_JPRB, 0.66919E+00_JPRB, 0.70803E+00_JPRB, 0.72124E+00_JPRB ,&
&  0.71881E+00_JPRB, 0.71284E+00_JPRB, 0.70315E+00_JPRB, 0.68692E+00_JPRB, 0.66918E+00_JPRB, 0.63504E+00_JPRB, 0.59925E+00_JPRB ,&
&  0.49295E+00_JPRB, 0.46593E+00_JPRB, 0.42708E+00_JPRB, 0.42950E+00_JPRB, 0.47048E+00_JPRB, 0.46057E+00_JPRB, 0.45867E+00_JPRB ,&
&  0.45347E+00_JPRB, 0.45984E+00_JPRB, 0.46627E+00_JPRB, 0.47501E+00_JPRB, 0.46977E+00_JPRB, 0.47790E+00_JPRB, 0.49128E+00_JPRB ,&
&  0.48460E+00_JPRB, 0.47452E+00_JPRB, 0.47455E+00_JPRB, 0.48113E+00_JPRB, 0.50065E+00_JPRB, 0.53007E+00_JPRB, 0.56601E+00_JPRB ,&
&  0.58455E+00_JPRB, 0.60086E+00_JPRB, 0.60999E+00_JPRB, 0.61629E+00_JPRB, 0.61786E+00_JPRB, 0.61282E+00_JPRB, 0.60194E+00_JPRB ,&
&  0.58685E+00_JPRB, 0.57422E+00_JPRB, 0.57527E+00_JPRB, 0.58747E+00_JPRB, 0.58867E+00_JPRB, 0.64424E+00_JPRB, 0.76095E+00_JPRB ,&
&  0.90985E+00_JPRB, 0.97374E+00_JPRB, 0.10719E+01_JPRB, 0.13080E+01_JPRB, 0.14778E+01_JPRB, 0.15646E+01_JPRB, 0.15527E+01_JPRB ,&
&  0.15292E+01_JPRB, 0.14931E+01_JPRB, 0.14805E+01_JPRB, 0.14969E+01_JPRB, 0.14364E+01_JPRB, 0.14477E+01_JPRB, 0.13680E+01_JPRB ,&
&  0.12480E+01_JPRB, 0.12265E+01_JPRB, 0.12960E+01_JPRB, 0.13531E+01_JPRB, 0.13787E+01_JPRB, 0.14268E+01_JPRB, 0.14618E+01_JPRB ,&
&  0.14334E+01_JPRB, 0.12605E+01_JPRB, 0.94535E+00_JPRB, 0.84033E+00_JPRB, 0.64475E+00_JPRB, 0.62574E+00_JPRB, 0.67978E+00_JPRB ,&
&  0.65827E+00_JPRB, 0.58373E+00_JPRB, 0.53250E+00_JPRB, 0.50953E+00_JPRB, 0.50935E+00_JPRB, 0.54834E+00_JPRB, 0.53568E+00_JPRB ,&
&  0.51124E+00_JPRB, 0.52062E+00_JPRB, 0.53140E+00_JPRB, 0.54252E+00_JPRB, 0.55710E+00_JPRB, 0.56131E+00_JPRB, 0.54400E+00_JPRB ,&
&  0.54915E+00_JPRB, 0.53815E+00_JPRB, 0.52944E+00_JPRB, 0.53420E+00_JPRB, 0.55161E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.61942E+00_JPRB, 0.63063E+00_JPRB, 0.64815E+00_JPRB, 0.67430E+00_JPRB, 0.71476E+00_JPRB, 0.74636E+00_JPRB, 0.74955E+00_JPRB ,&
&  0.74253E+00_JPRB, 0.73419E+00_JPRB, 0.71993E+00_JPRB, 0.70359E+00_JPRB, 0.67615E+00_JPRB, 0.63355E+00_JPRB, 0.59452E+00_JPRB ,&
&  0.58794E+00_JPRB, 0.52408E+00_JPRB, 0.47049E+00_JPRB, 0.43394E+00_JPRB, 0.45887E+00_JPRB, 0.45929E+00_JPRB, 0.47206E+00_JPRB ,&
&  0.46924E+00_JPRB, 0.48512E+00_JPRB, 0.48785E+00_JPRB, 0.48347E+00_JPRB, 0.47711E+00_JPRB, 0.49641E+00_JPRB, 0.52115E+00_JPRB ,&
&  0.53705E+00_JPRB, 0.54208E+00_JPRB, 0.54565E+00_JPRB, 0.56527E+00_JPRB, 0.60491E+00_JPRB, 0.58740E+00_JPRB, 0.62563E+00_JPRB ,&
&  0.64538E+00_JPRB, 0.65949E+00_JPRB, 0.66992E+00_JPRB, 0.68398E+00_JPRB, 0.68862E+00_JPRB, 0.69414E+00_JPRB, 0.70029E+00_JPRB ,&
&  0.70798E+00_JPRB, 0.70695E+00_JPRB, 0.67953E+00_JPRB, 0.67291E+00_JPRB, 0.68470E+00_JPRB, 0.78928E+00_JPRB, 0.85799E+00_JPRB ,&
&  0.94202E+00_JPRB, 0.95231E+00_JPRB, 0.10025E+01_JPRB, 0.12546E+01_JPRB, 0.14498E+01_JPRB, 0.15176E+01_JPRB, 0.15738E+01_JPRB ,&
&  0.15937E+01_JPRB, 0.15965E+01_JPRB, 0.16465E+01_JPRB, 0.16630E+01_JPRB, 0.16583E+01_JPRB, 0.16355E+01_JPRB, 0.15934E+01_JPRB ,&
&  0.14945E+01_JPRB, 0.14616E+01_JPRB, 0.15108E+01_JPRB, 0.14843E+01_JPRB, 0.14643E+01_JPRB, 0.14008E+01_JPRB, 0.14931E+01_JPRB ,&
&  0.14813E+01_JPRB, 0.90172E+00_JPRB, 0.72461E+00_JPRB, 0.80932E+00_JPRB, 0.67817E+00_JPRB, 0.65070E+00_JPRB, 0.67993E+00_JPRB ,&
&  0.64283E+00_JPRB, 0.60567E+00_JPRB, 0.58974E+00_JPRB, 0.56205E+00_JPRB, 0.56581E+00_JPRB, 0.65071E+00_JPRB, 0.64185E+00_JPRB ,&
&  0.61619E+00_JPRB, 0.60874E+00_JPRB, 0.63210E+00_JPRB, 0.63511E+00_JPRB, 0.63666E+00_JPRB, 0.64623E+00_JPRB, 0.63799E+00_JPRB ,&
&  0.61538E+00_JPRB, 0.59904E+00_JPRB, 0.58955E+00_JPRB, 0.59287E+00_JPRB, 0.60672E+00_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.69457E+00_JPRB, 0.69881E+00_JPRB, 0.70766E+00_JPRB, 0.72932E+00_JPRB, 0.76423E+00_JPRB, 0.78671E+00_JPRB, 0.77941E+00_JPRB ,&
&  0.76963E+00_JPRB, 0.75337E+00_JPRB, 0.73722E+00_JPRB, 0.71397E+00_JPRB, 0.67651E+00_JPRB, 0.63310E+00_JPRB, 0.59349E+00_JPRB ,&
&  0.56912E+00_JPRB, 0.54560E+00_JPRB, 0.47742E+00_JPRB, 0.45047E+00_JPRB, 0.42944E+00_JPRB, 0.44454E+00_JPRB, 0.46411E+00_JPRB ,&
&  0.48514E+00_JPRB, 0.49439E+00_JPRB, 0.49994E+00_JPRB, 0.50486E+00_JPRB, 0.51270E+00_JPRB, 0.54875E+00_JPRB, 0.60229E+00_JPRB ,&
&  0.63447E+00_JPRB, 0.65153E+00_JPRB, 0.63175E+00_JPRB, 0.67991E+00_JPRB, 0.68777E+00_JPRB, 0.66383E+00_JPRB, 0.71652E+00_JPRB ,&
&  0.74354E+00_JPRB, 0.76297E+00_JPRB, 0.76624E+00_JPRB, 0.78225E+00_JPRB, 0.79140E+00_JPRB, 0.77260E+00_JPRB, 0.74906E+00_JPRB ,&
&  0.71905E+00_JPRB, 0.70240E+00_JPRB, 0.70000E+00_JPRB, 0.69456E+00_JPRB, 0.70625E+00_JPRB, 0.77046E+00_JPRB, 0.78222E+00_JPRB ,&
&  0.78372E+00_JPRB, 0.66784E+00_JPRB, 0.62443E+00_JPRB, 0.86989E+00_JPRB, 0.12532E+01_JPRB, 0.14587E+01_JPRB, 0.14349E+01_JPRB ,&
&  0.15753E+01_JPRB, 0.16765E+01_JPRB, 0.18237E+01_JPRB, 0.19055E+01_JPRB, 0.19046E+01_JPRB, 0.18387E+01_JPRB, 0.17721E+01_JPRB ,&
&  0.17100E+01_JPRB, 0.16813E+01_JPRB, 0.16662E+01_JPRB, 0.16437E+01_JPRB, 0.16085E+01_JPRB, 0.16327E+01_JPRB, 0.17174E+01_JPRB ,&
&  0.15882E+01_JPRB, 0.13503E+01_JPRB, 0.91497E+00_JPRB, 0.89763E+00_JPRB, 0.70343E+00_JPRB, 0.67013E+00_JPRB, 0.70589E+00_JPRB ,&
&  0.66697E+00_JPRB, 0.66730E+00_JPRB, 0.65011E+00_JPRB, 0.62311E+00_JPRB, 0.66964E+00_JPRB, 0.76782E+00_JPRB, 0.80108E+00_JPRB ,&
&  0.81897E+00_JPRB, 0.76279E+00_JPRB, 0.80716E+00_JPRB, 0.80925E+00_JPRB, 0.80508E+00_JPRB, 0.79632E+00_JPRB, 0.77698E+00_JPRB ,&
&  0.74176E+00_JPRB, 0.70735E+00_JPRB, 0.68730E+00_JPRB, 0.68267E+00_JPRB, 0.68594E+00_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.74962E+00_JPRB, 0.73926E+00_JPRB, 0.73718E+00_JPRB, 0.75187E+00_JPRB, 0.77853E+00_JPRB, 0.79147E+00_JPRB, 0.77882E+00_JPRB ,&
&  0.76858E+00_JPRB, 0.75427E+00_JPRB, 0.74404E+00_JPRB, 0.70858E+00_JPRB, 0.67340E+00_JPRB, 0.62841E+00_JPRB, 0.58708E+00_JPRB ,&
&  0.56147E+00_JPRB, 0.51816E+00_JPRB, 0.45799E+00_JPRB, 0.43294E+00_JPRB, 0.41394E+00_JPRB, 0.41991E+00_JPRB, 0.44475E+00_JPRB ,&
&  0.47578E+00_JPRB, 0.49840E+00_JPRB, 0.53774E+00_JPRB, 0.55615E+00_JPRB, 0.58811E+00_JPRB, 0.65133E+00_JPRB, 0.72570E+00_JPRB ,&
&  0.82940E+00_JPRB, 0.72696E+00_JPRB, 0.75189E+00_JPRB, 0.74921E+00_JPRB, 0.77329E+00_JPRB, 0.77879E+00_JPRB, 0.77744E+00_JPRB ,&
&  0.78758E+00_JPRB, 0.79580E+00_JPRB, 0.81360E+00_JPRB, 0.81049E+00_JPRB, 0.79387E+00_JPRB, 0.77456E+00_JPRB, 0.76050E+00_JPRB ,&
&  0.75740E+00_JPRB, 0.74699E+00_JPRB, 0.72197E+00_JPRB, 0.69347E+00_JPRB, 0.75965E+00_JPRB, 0.81162E+00_JPRB, 0.76879E+00_JPRB ,&
&  0.63710E+00_JPRB, 0.65050E+00_JPRB, 0.84892E+00_JPRB, 0.11000E+01_JPRB, 0.13600E+01_JPRB, 0.15998E+01_JPRB, 0.17465E+01_JPRB ,&
&  0.18934E+01_JPRB, 0.19893E+01_JPRB, 0.20821E+01_JPRB, 0.18670E+01_JPRB, 0.20022E+01_JPRB, 0.19447E+01_JPRB, 0.18540E+01_JPRB ,&
&  0.17595E+01_JPRB, 0.17437E+01_JPRB, 0.16832E+01_JPRB, 0.17234E+01_JPRB, 0.18300E+01_JPRB, 0.19865E+01_JPRB, 0.17610E+01_JPRB ,&
&  0.14729E+01_JPRB, 0.15447E+01_JPRB, 0.13186E+01_JPRB, 0.10449E+01_JPRB, 0.82965E+00_JPRB, 0.79969E+00_JPRB, 0.78034E+00_JPRB ,&
&  0.75173E+00_JPRB, 0.72866E+00_JPRB, 0.69789E+00_JPRB, 0.73239E+00_JPRB, 0.84117E+00_JPRB, 0.93198E+00_JPRB, 0.94271E+00_JPRB ,&
&  0.92276E+00_JPRB, 0.98856E+00_JPRB, 0.10091E+01_JPRB, 0.10054E+01_JPRB, 0.10180E+01_JPRB, 0.99532E+00_JPRB, 0.94523E+00_JPRB ,&
&  0.88556E+00_JPRB, 0.83964E+00_JPRB, 0.81011E+00_JPRB, 0.78531E+00_JPRB, 0.76472E+00_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.79191E+00_JPRB, 0.77654E+00_JPRB, 0.77344E+00_JPRB, 0.78345E+00_JPRB, 0.80183E+00_JPRB, 0.80082E+00_JPRB, 0.78672E+00_JPRB ,&
&  0.76630E+00_JPRB, 0.74337E+00_JPRB, 0.71518E+00_JPRB, 0.68657E+00_JPRB, 0.66257E+00_JPRB, 0.62516E+00_JPRB, 0.59012E+00_JPRB ,&
&  0.55770E+00_JPRB, 0.50222E+00_JPRB, 0.45775E+00_JPRB, 0.45606E+00_JPRB, 0.44685E+00_JPRB, 0.42601E+00_JPRB, 0.44072E+00_JPRB ,&
&  0.47158E+00_JPRB, 0.51765E+00_JPRB, 0.57787E+00_JPRB, 0.65551E+00_JPRB, 0.72700E+00_JPRB, 0.82106E+00_JPRB, 0.89099E+00_JPRB ,&
&  0.85795E+00_JPRB, 0.84250E+00_JPRB, 0.85245E+00_JPRB, 0.84837E+00_JPRB, 0.81453E+00_JPRB, 0.80573E+00_JPRB, 0.81424E+00_JPRB ,&
&  0.82780E+00_JPRB, 0.84046E+00_JPRB, 0.84552E+00_JPRB, 0.83632E+00_JPRB, 0.83407E+00_JPRB, 0.83176E+00_JPRB, 0.83468E+00_JPRB ,&
&  0.79321E+00_JPRB, 0.75486E+00_JPRB, 0.71122E+00_JPRB, 0.73127E+00_JPRB, 0.67488E+00_JPRB, 0.65162E+00_JPRB, 0.64519E+00_JPRB ,&
&  0.72673E+00_JPRB, 0.86255E+00_JPRB, 0.94851E+00_JPRB, 0.13816E+01_JPRB, 0.14226E+01_JPRB, 0.15235E+01_JPRB, 0.18329E+01_JPRB ,&
&  0.20234E+01_JPRB, 0.20926E+01_JPRB, 0.20956E+01_JPRB, 0.17080E+01_JPRB, 0.14869E+01_JPRB, 0.19291E+01_JPRB, 0.19252E+01_JPRB ,&
&  0.18212E+01_JPRB, 0.17787E+01_JPRB, 0.18087E+01_JPRB, 0.20706E+01_JPRB, 0.17287E+01_JPRB, 0.15207E+01_JPRB, 0.12049E+01_JPRB ,&
&  0.12532E+01_JPRB, 0.11004E+01_JPRB, 0.12303E+01_JPRB, 0.10807E+01_JPRB, 0.10198E+01_JPRB, 0.93930E+00_JPRB, 0.83804E+00_JPRB ,&
&  0.79074E+00_JPRB, 0.76367E+00_JPRB, 0.77075E+00_JPRB, 0.90672E+00_JPRB, 0.10696E+01_JPRB, 0.11057E+01_JPRB, 0.11529E+01_JPRB ,&
&  0.11848E+01_JPRB, 0.12074E+01_JPRB, 0.11724E+01_JPRB, 0.11774E+01_JPRB, 0.11452E+01_JPRB, 0.11053E+01_JPRB, 0.10576E+01_JPRB ,&
&  0.10071E+01_JPRB, 0.95701E+00_JPRB, 0.90424E+00_JPRB, 0.85594E+00_JPRB, 0.81771E+00_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.82700E+00_JPRB, 0.81298E+00_JPRB, 0.81309E+00_JPRB, 0.81883E+00_JPRB, 0.81777E+00_JPRB, 0.80287E+00_JPRB, 0.78211E+00_JPRB ,&
&  0.76109E+00_JPRB, 0.72333E+00_JPRB, 0.68487E+00_JPRB, 0.65473E+00_JPRB, 0.63747E+00_JPRB, 0.62271E+00_JPRB, 0.59759E+00_JPRB ,&
&  0.56560E+00_JPRB, 0.53517E+00_JPRB, 0.49067E+00_JPRB, 0.47073E+00_JPRB, 0.49359E+00_JPRB, 0.46364E+00_JPRB, 0.44438E+00_JPRB ,&
&  0.48873E+00_JPRB, 0.54833E+00_JPRB, 0.63948E+00_JPRB, 0.76385E+00_JPRB, 0.87736E+00_JPRB, 0.98706E+00_JPRB, 0.10454E+01_JPRB ,&
&  0.99930E+00_JPRB, 0.92251E+00_JPRB, 0.86174E+00_JPRB, 0.80963E+00_JPRB, 0.76901E+00_JPRB, 0.76524E+00_JPRB, 0.77442E+00_JPRB ,&
&  0.80385E+00_JPRB, 0.83187E+00_JPRB, 0.82059E+00_JPRB, 0.78523E+00_JPRB, 0.77841E+00_JPRB, 0.73747E+00_JPRB, 0.71089E+00_JPRB ,&
&  0.71789E+00_JPRB, 0.71441E+00_JPRB, 0.70726E+00_JPRB, 0.76512E+00_JPRB, 0.69058E+00_JPRB, 0.64308E+00_JPRB, 0.66189E+00_JPRB ,&
&  0.74554E+00_JPRB, 0.88622E+00_JPRB, 0.10190E+01_JPRB, 0.13893E+01_JPRB, 0.18898E+01_JPRB, 0.19536E+01_JPRB, 0.21315E+01_JPRB ,&
&  0.18267E+01_JPRB, 0.15352E+01_JPRB, 0.13218E+01_JPRB, 0.11410E+01_JPRB, 0.14164E+01_JPRB, 0.20362E+01_JPRB, 0.22087E+01_JPRB ,&
&  0.21549E+01_JPRB, 0.21380E+01_JPRB, 0.23120E+01_JPRB, 0.21494E+01_JPRB, 0.16097E+01_JPRB, 0.11435E+01_JPRB, 0.14523E+01_JPRB ,&
&  0.15514E+01_JPRB, 0.15261E+01_JPRB, 0.13753E+01_JPRB, 0.11341E+01_JPRB, 0.92255E+00_JPRB, 0.10242E+01_JPRB, 0.95221E+00_JPRB ,&
&  0.85919E+00_JPRB, 0.81803E+00_JPRB, 0.95738E+00_JPRB, 0.12015E+01_JPRB, 0.12988E+01_JPRB, 0.14599E+01_JPRB, 0.15006E+01_JPRB ,&
&  0.14358E+01_JPRB, 0.13898E+01_JPRB, 0.13233E+01_JPRB, 0.12443E+01_JPRB, 0.11780E+01_JPRB, 0.11054E+01_JPRB, 0.10509E+01_JPRB ,&
&  0.10041E+01_JPRB, 0.96525E+00_JPRB, 0.92243E+00_JPRB, 0.88249E+00_JPRB, 0.84980E+00_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.84878E+00_JPRB, 0.84149E+00_JPRB, 0.83561E+00_JPRB, 0.82716E+00_JPRB, 0.81411E+00_JPRB, 0.79513E+00_JPRB, 0.77211E+00_JPRB ,&
&  0.73571E+00_JPRB, 0.69986E+00_JPRB, 0.66170E+00_JPRB, 0.64047E+00_JPRB, 0.61817E+00_JPRB, 0.59873E+00_JPRB, 0.58696E+00_JPRB ,&
&  0.58366E+00_JPRB, 0.59420E+00_JPRB, 0.53729E+00_JPRB, 0.51652E+00_JPRB, 0.52504E+00_JPRB, 0.51954E+00_JPRB, 0.46334E+00_JPRB ,&
&  0.52658E+00_JPRB, 0.59272E+00_JPRB, 0.67395E+00_JPRB, 0.81768E+00_JPRB, 0.10479E+01_JPRB, 0.10365E+01_JPRB, 0.10142E+01_JPRB ,&
&  0.98950E+00_JPRB, 0.93381E+00_JPRB, 0.88113E+00_JPRB, 0.81567E+00_JPRB, 0.77210E+00_JPRB, 0.77234E+00_JPRB, 0.78396E+00_JPRB ,&
&  0.81676E+00_JPRB, 0.83219E+00_JPRB, 0.80460E+00_JPRB, 0.76353E+00_JPRB, 0.72589E+00_JPRB, 0.70400E+00_JPRB, 0.68884E+00_JPRB ,&
&  0.69316E+00_JPRB, 0.69489E+00_JPRB, 0.70088E+00_JPRB, 0.77565E+00_JPRB, 0.70907E+00_JPRB, 0.67398E+00_JPRB, 0.79876E+00_JPRB ,&
&  0.90775E+00_JPRB, 0.11482E+01_JPRB, 0.12408E+01_JPRB, 0.15497E+01_JPRB, 0.18861E+01_JPRB, 0.20239E+01_JPRB, 0.21026E+01_JPRB ,&
&  0.16471E+01_JPRB, 0.14198E+01_JPRB, 0.15472E+01_JPRB, 0.17376E+01_JPRB, 0.16969E+01_JPRB, 0.18615E+01_JPRB, 0.29929E+01_JPRB ,&
&  0.22006E+01_JPRB, 0.23113E+01_JPRB, 0.25622E+01_JPRB, 0.25267E+01_JPRB, 0.12031E+01_JPRB, 0.83219E+00_JPRB, 0.12451E+01_JPRB ,&
&  0.12680E+01_JPRB, 0.79869E+00_JPRB, 0.63431E+00_JPRB, 0.78462E+00_JPRB, 0.63659E+00_JPRB, 0.72478E+00_JPRB, 0.89074E+00_JPRB ,&
&  0.94141E+00_JPRB, 0.96504E+00_JPRB, 0.11588E+01_JPRB, 0.12606E+01_JPRB, 0.13729E+01_JPRB, 0.18596E+01_JPRB, 0.17961E+01_JPRB ,&
&  0.16871E+01_JPRB, 0.14755E+01_JPRB, 0.14130E+01_JPRB, 0.12642E+01_JPRB, 0.11607E+01_JPRB, 0.10693E+01_JPRB, 0.99752E+00_JPRB ,&
&  0.94621E+00_JPRB, 0.90986E+00_JPRB, 0.88469E+00_JPRB, 0.86485E+00_JPRB, 0.85822E+00_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.86031E+00_JPRB, 0.85202E+00_JPRB, 0.84278E+00_JPRB, 0.83001E+00_JPRB, 0.81194E+00_JPRB, 0.78655E+00_JPRB, 0.75287E+00_JPRB ,&
&  0.71173E+00_JPRB, 0.67339E+00_JPRB, 0.64175E+00_JPRB, 0.61930E+00_JPRB, 0.59717E+00_JPRB, 0.58015E+00_JPRB, 0.57919E+00_JPRB ,&
&  0.58687E+00_JPRB, 0.60625E+00_JPRB, 0.60312E+00_JPRB, 0.58537E+00_JPRB, 0.58473E+00_JPRB, 0.57741E+00_JPRB, 0.54924E+00_JPRB ,&
&  0.61634E+00_JPRB, 0.66237E+00_JPRB, 0.72277E+00_JPRB, 0.83261E+00_JPRB, 0.94818E+00_JPRB, 0.94286E+00_JPRB, 0.97657E+00_JPRB ,&
&  0.95695E+00_JPRB, 0.93796E+00_JPRB, 0.87866E+00_JPRB, 0.82559E+00_JPRB, 0.80100E+00_JPRB, 0.79663E+00_JPRB, 0.82358E+00_JPRB ,&
&  0.83619E+00_JPRB, 0.84241E+00_JPRB, 0.80317E+00_JPRB, 0.74054E+00_JPRB, 0.71336E+00_JPRB, 0.69351E+00_JPRB, 0.68296E+00_JPRB ,&
&  0.68330E+00_JPRB, 0.70136E+00_JPRB, 0.72268E+00_JPRB, 0.80149E+00_JPRB, 0.93748E+00_JPRB, 0.10116E+01_JPRB, 0.93863E+00_JPRB ,&
&  0.90368E+00_JPRB, 0.10670E+01_JPRB, 0.13815E+01_JPRB, 0.15200E+01_JPRB, 0.17311E+01_JPRB, 0.19200E+01_JPRB, 0.18937E+01_JPRB ,&
&  0.18297E+01_JPRB, 0.19774E+01_JPRB, 0.19662E+01_JPRB, 0.22005E+01_JPRB, 0.23289E+01_JPRB, 0.18990E+01_JPRB, 0.23930E+01_JPRB ,&
&  0.24420E+01_JPRB, 0.23535E+01_JPRB, 0.18664E+01_JPRB, 0.13267E+01_JPRB, 0.14017E+01_JPRB, 0.10107E+01_JPRB, 0.65982E+00_JPRB ,&
&  0.59111E+00_JPRB, 0.55615E+00_JPRB, 0.52942E+00_JPRB, 0.55118E+00_JPRB, 0.55609E+00_JPRB, 0.67644E+00_JPRB, 0.85288E+00_JPRB ,&
&  0.11049E+01_JPRB, 0.12204E+01_JPRB, 0.12364E+01_JPRB, 0.12627E+01_JPRB, 0.12965E+01_JPRB, 0.16649E+01_JPRB, 0.17634E+01_JPRB ,&
&  0.15527E+01_JPRB, 0.14702E+01_JPRB, 0.13228E+01_JPRB, 0.11937E+01_JPRB, 0.11073E+01_JPRB, 0.10378E+01_JPRB, 0.98408E+00_JPRB ,&
&  0.94938E+00_JPRB, 0.92361E+00_JPRB, 0.90344E+00_JPRB, 0.88743E+00_JPRB, 0.87040E+00_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.84789E+00_JPRB, 0.83698E+00_JPRB, 0.82736E+00_JPRB, 0.80893E+00_JPRB, 0.78570E+00_JPRB, 0.75833E+00_JPRB, 0.72122E+00_JPRB ,&
&  0.67300E+00_JPRB, 0.63332E+00_JPRB, 0.60659E+00_JPRB, 0.58484E+00_JPRB, 0.57466E+00_JPRB, 0.56895E+00_JPRB, 0.57836E+00_JPRB ,&
&  0.59294E+00_JPRB, 0.61494E+00_JPRB, 0.65402E+00_JPRB, 0.66092E+00_JPRB, 0.71710E+00_JPRB, 0.70963E+00_JPRB, 0.71500E+00_JPRB ,&
&  0.75968E+00_JPRB, 0.78110E+00_JPRB, 0.82861E+00_JPRB, 0.88866E+00_JPRB, 0.96554E+00_JPRB, 0.96293E+00_JPRB, 0.95778E+00_JPRB ,&
&  0.94182E+00_JPRB, 0.94715E+00_JPRB, 0.89795E+00_JPRB, 0.85431E+00_JPRB, 0.82549E+00_JPRB, 0.83424E+00_JPRB, 0.83776E+00_JPRB ,&
&  0.85045E+00_JPRB, 0.84573E+00_JPRB, 0.81003E+00_JPRB, 0.75423E+00_JPRB, 0.72481E+00_JPRB, 0.71079E+00_JPRB, 0.71606E+00_JPRB ,&
&  0.72320E+00_JPRB, 0.76213E+00_JPRB, 0.80495E+00_JPRB, 0.90931E+00_JPRB, 0.87502E+00_JPRB, 0.75897E+00_JPRB, 0.86877E+00_JPRB ,&
&  0.97050E+00_JPRB, 0.12224E+01_JPRB, 0.13213E+01_JPRB, 0.15395E+01_JPRB, 0.16451E+01_JPRB, 0.16448E+01_JPRB, 0.17838E+01_JPRB ,&
&  0.19554E+01_JPRB, 0.26793E+01_JPRB, 0.20922E+01_JPRB, 0.21054E+01_JPRB, 0.31661E+01_JPRB, 0.25624E+01_JPRB, 0.24158E+01_JPRB ,&
&  0.27562E+01_JPRB, 0.23307E+01_JPRB, 0.20818E+01_JPRB, 0.17015E+01_JPRB, 0.24951E+01_JPRB, 0.24765E+01_JPRB, 0.87204E+00_JPRB ,&
&  0.64685E+00_JPRB, 0.58551E+00_JPRB, 0.54227E+00_JPRB, 0.51251E+00_JPRB, 0.54568E+00_JPRB, 0.70189E+00_JPRB, 0.10076E+01_JPRB ,&
&  0.11630E+01_JPRB, 0.13212E+01_JPRB, 0.13016E+01_JPRB, 0.13304E+01_JPRB, 0.12830E+01_JPRB, 0.14819E+01_JPRB, 0.15350E+01_JPRB ,&
&  0.14155E+01_JPRB, 0.13502E+01_JPRB, 0.11960E+01_JPRB, 0.10942E+01_JPRB, 0.10376E+01_JPRB, 0.99577E+00_JPRB, 0.95630E+00_JPRB ,&
&  0.92594E+00_JPRB, 0.90846E+00_JPRB, 0.89275E+00_JPRB, 0.87528E+00_JPRB, 0.86046E+00_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.81869E+00_JPRB, 0.80832E+00_JPRB, 0.79189E+00_JPRB, 0.76571E+00_JPRB, 0.75267E+00_JPRB, 0.73518E+00_JPRB, 0.69512E+00_JPRB ,&
&  0.64889E+00_JPRB, 0.62033E+00_JPRB, 0.59221E+00_JPRB, 0.56995E+00_JPRB, 0.56819E+00_JPRB, 0.57260E+00_JPRB, 0.59175E+00_JPRB ,&
&  0.61649E+00_JPRB, 0.64692E+00_JPRB, 0.67887E+00_JPRB, 0.76198E+00_JPRB, 0.86451E+00_JPRB, 0.87091E+00_JPRB, 0.98297E+00_JPRB ,&
&  0.97854E+00_JPRB, 0.95593E+00_JPRB, 0.99253E+00_JPRB, 0.10165E+01_JPRB, 0.10361E+01_JPRB, 0.10002E+01_JPRB, 0.98047E+00_JPRB ,&
&  0.94334E+00_JPRB, 0.96353E+00_JPRB, 0.91906E+00_JPRB, 0.88990E+00_JPRB, 0.88325E+00_JPRB, 0.87910E+00_JPRB, 0.87473E+00_JPRB ,&
&  0.87272E+00_JPRB, 0.86045E+00_JPRB, 0.83336E+00_JPRB, 0.79242E+00_JPRB, 0.76417E+00_JPRB, 0.74978E+00_JPRB, 0.75989E+00_JPRB ,&
&  0.79530E+00_JPRB, 0.85613E+00_JPRB, 0.91455E+00_JPRB, 0.98796E+00_JPRB, 0.81940E+00_JPRB, 0.94944E+00_JPRB, 0.10115E+01_JPRB ,&
&  0.11432E+01_JPRB, 0.12591E+01_JPRB, 0.12645E+01_JPRB, 0.13768E+01_JPRB, 0.15542E+01_JPRB, 0.16030E+01_JPRB, 0.17663E+01_JPRB ,&
&  0.22341E+01_JPRB, 0.22157E+01_JPRB, 0.19654E+01_JPRB, 0.21553E+01_JPRB, 0.28404E+01_JPRB, 0.40670E+01_JPRB, 0.22212E+01_JPRB ,&
&  0.18006E+01_JPRB, 0.22197E+01_JPRB, 0.21124E+01_JPRB, 0.19474E+01_JPRB, 0.26331E+01_JPRB, 0.32175E+01_JPRB, 0.21593E+01_JPRB ,&
&  0.75586E+00_JPRB, 0.59163E+00_JPRB, 0.63098E+00_JPRB, 0.59418E+00_JPRB, 0.58429E+00_JPRB, 0.73911E+00_JPRB, 0.15071E+01_JPRB ,&
&  0.12799E+01_JPRB, 0.13857E+01_JPRB, 0.13675E+01_JPRB, 0.13413E+01_JPRB, 0.13182E+01_JPRB, 0.14163E+01_JPRB, 0.14684E+01_JPRB ,&
&  0.11901E+01_JPRB, 0.11248E+01_JPRB, 0.10219E+01_JPRB, 0.97142E+00_JPRB, 0.94939E+00_JPRB, 0.93387E+00_JPRB, 0.90259E+00_JPRB ,&
&  0.87936E+00_JPRB, 0.86509E+00_JPRB, 0.85227E+00_JPRB, 0.84121E+00_JPRB, 0.83140E+00_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.78348E+00_JPRB, 0.76872E+00_JPRB, 0.74615E+00_JPRB, 0.73920E+00_JPRB, 0.75297E+00_JPRB, 0.73276E+00_JPRB, 0.70187E+00_JPRB ,&
&  0.65470E+00_JPRB, 0.63099E+00_JPRB, 0.60538E+00_JPRB, 0.59488E+00_JPRB, 0.59129E+00_JPRB, 0.60432E+00_JPRB, 0.62896E+00_JPRB ,&
&  0.66131E+00_JPRB, 0.69479E+00_JPRB, 0.73676E+00_JPRB, 0.86849E+00_JPRB, 0.10530E+01_JPRB, 0.10394E+01_JPRB, 0.11407E+01_JPRB ,&
&  0.12844E+01_JPRB, 0.12411E+01_JPRB, 0.11661E+01_JPRB, 0.11445E+01_JPRB, 0.11403E+01_JPRB, 0.11063E+01_JPRB, 0.10435E+01_JPRB ,&
&  0.97739E+00_JPRB, 0.10147E+01_JPRB, 0.97100E+00_JPRB, 0.93878E+00_JPRB, 0.90967E+00_JPRB, 0.90965E+00_JPRB, 0.92732E+00_JPRB ,&
&  0.90956E+00_JPRB, 0.91363E+00_JPRB, 0.88109E+00_JPRB, 0.85522E+00_JPRB, 0.83378E+00_JPRB, 0.82466E+00_JPRB, 0.86321E+00_JPRB ,&
&  0.93529E+00_JPRB, 0.10086E+01_JPRB, 0.10528E+01_JPRB, 0.10406E+01_JPRB, 0.10564E+01_JPRB, 0.10843E+01_JPRB, 0.12027E+01_JPRB ,&
&  0.12959E+01_JPRB, 0.13561E+01_JPRB, 0.13937E+01_JPRB, 0.14185E+01_JPRB, 0.15584E+01_JPRB, 0.17339E+01_JPRB, 0.19584E+01_JPRB ,&
&  0.23034E+01_JPRB, 0.23828E+01_JPRB, 0.20980E+01_JPRB, 0.22075E+01_JPRB, 0.26438E+01_JPRB, 0.31667E+01_JPRB, 0.27880E+01_JPRB ,&
&  0.21814E+01_JPRB, 0.17799E+01_JPRB, 0.18687E+01_JPRB, 0.22264E+01_JPRB, 0.24603E+01_JPRB, 0.23813E+01_JPRB, 0.27500E+01_JPRB ,&
&  0.23411E+01_JPRB, 0.12244E+01_JPRB, 0.10000E+01_JPRB, 0.11648E+01_JPRB, 0.91475E+00_JPRB, 0.93136E+00_JPRB, 0.12120E+01_JPRB ,&
&  0.12877E+01_JPRB, 0.13959E+01_JPRB, 0.14042E+01_JPRB, 0.12581E+01_JPRB, 0.13018E+01_JPRB, 0.13497E+01_JPRB, 0.12690E+01_JPRB ,&
&  0.10820E+01_JPRB, 0.99940E+00_JPRB, 0.93626E+00_JPRB, 0.89544E+00_JPRB, 0.87100E+00_JPRB, 0.84406E+00_JPRB, 0.82745E+00_JPRB ,&
&  0.80812E+00_JPRB, 0.79542E+00_JPRB, 0.78985E+00_JPRB, 0.78603E+00_JPRB, 0.78879E+00_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.75453E+00_JPRB, 0.73935E+00_JPRB, 0.75004E+00_JPRB, 0.77813E+00_JPRB, 0.78359E+00_JPRB, 0.78167E+00_JPRB, 0.75399E+00_JPRB ,&
&  0.68888E+00_JPRB, 0.67385E+00_JPRB, 0.67345E+00_JPRB, 0.66692E+00_JPRB, 0.65609E+00_JPRB, 0.66836E+00_JPRB, 0.69098E+00_JPRB ,&
&  0.72534E+00_JPRB, 0.78931E+00_JPRB, 0.90224E+00_JPRB, 0.11466E+01_JPRB, 0.13585E+01_JPRB, 0.15950E+01_JPRB, 0.11698E+01_JPRB ,&
&  0.12705E+01_JPRB, 0.14224E+01_JPRB, 0.12815E+01_JPRB, 0.12520E+01_JPRB, 0.12394E+01_JPRB, 0.12188E+01_JPRB, 0.11147E+01_JPRB ,&
&  0.10798E+01_JPRB, 0.10886E+01_JPRB, 0.10415E+01_JPRB, 0.97133E+00_JPRB, 0.93996E+00_JPRB, 0.99212E+00_JPRB, 0.98534E+00_JPRB ,&
&  0.98825E+00_JPRB, 0.10075E+01_JPRB, 0.96108E+00_JPRB, 0.95260E+00_JPRB, 0.93867E+00_JPRB, 0.93905E+00_JPRB, 0.10007E+01_JPRB ,&
&  0.10815E+01_JPRB, 0.11465E+01_JPRB, 0.11900E+01_JPRB, 0.11463E+01_JPRB, 0.12062E+01_JPRB, 0.12878E+01_JPRB, 0.14321E+01_JPRB ,&
&  0.14527E+01_JPRB, 0.13356E+01_JPRB, 0.13965E+01_JPRB, 0.15515E+01_JPRB, 0.17292E+01_JPRB, 0.18132E+01_JPRB, 0.20145E+01_JPRB ,&
&  0.24868E+01_JPRB, 0.28249E+01_JPRB, 0.28871E+01_JPRB, 0.24286E+01_JPRB, 0.24515E+01_JPRB, 0.28204E+01_JPRB, 0.29442E+01_JPRB ,&
&  0.26352E+01_JPRB, 0.22238E+01_JPRB, 0.22303E+01_JPRB, 0.21488E+01_JPRB, 0.20555E+01_JPRB, 0.18972E+01_JPRB, 0.19656E+01_JPRB ,&
&  0.21638E+01_JPRB, 0.23060E+01_JPRB, 0.17971E+01_JPRB, 0.14638E+01_JPRB, 0.14029E+01_JPRB, 0.12545E+01_JPRB, 0.12166E+01_JPRB ,&
&  0.13532E+01_JPRB, 0.13534E+01_JPRB, 0.13483E+01_JPRB, 0.15068E+01_JPRB, 0.13746E+01_JPRB, 0.12703E+01_JPRB, 0.10615E+01_JPRB ,&
&  0.88905E+00_JPRB, 0.83626E+00_JPRB, 0.80156E+00_JPRB, 0.78106E+00_JPRB, 0.78719E+00_JPRB, 0.78417E+00_JPRB, 0.78270E+00_JPRB ,&
&  0.77143E+00_JPRB, 0.76997E+00_JPRB, 0.77385E+00_JPRB, 0.77712E+00_JPRB, 0.77158E+00_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.75097E+00_JPRB, 0.76373E+00_JPRB, 0.79195E+00_JPRB, 0.82292E+00_JPRB, 0.84455E+00_JPRB, 0.87506E+00_JPRB, 0.85934E+00_JPRB ,&
&  0.77401E+00_JPRB, 0.71752E+00_JPRB, 0.72640E+00_JPRB, 0.71291E+00_JPRB, 0.70362E+00_JPRB, 0.71711E+00_JPRB, 0.75113E+00_JPRB ,&
&  0.81422E+00_JPRB, 0.95264E+00_JPRB, 0.11654E+01_JPRB, 0.14393E+01_JPRB, 0.15529E+01_JPRB, 0.16033E+01_JPRB, 0.15450E+01_JPRB ,&
&  0.11806E+01_JPRB, 0.15592E+01_JPRB, 0.12987E+01_JPRB, 0.13225E+01_JPRB, 0.13075E+01_JPRB, 0.13889E+01_JPRB, 0.13101E+01_JPRB ,&
&  0.11898E+01_JPRB, 0.11195E+01_JPRB, 0.11265E+01_JPRB, 0.10699E+01_JPRB, 0.10577E+01_JPRB, 0.10524E+01_JPRB, 0.10629E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.10424E+01_JPRB, 0.10495E+01_JPRB, 0.10632E+01_JPRB, 0.10579E+01_JPRB, 0.10683E+01_JPRB, 0.11034E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.12740E+01_JPRB, 0.12989E+01_JPRB, 0.13386E+01_JPRB, 0.13775E+01_JPRB, 0.15439E+01_JPRB, 0.16985E+01_JPRB ,&
&  0.16650E+01_JPRB, 0.15067E+01_JPRB, 0.15410E+01_JPRB, 0.16405E+01_JPRB, 0.17300E+01_JPRB, 0.19908E+01_JPRB, 0.21435E+01_JPRB ,&
&  0.26454E+01_JPRB, 0.25984E+01_JPRB, 0.30265E+01_JPRB, 0.23812E+01_JPRB, 0.22850E+01_JPRB, 0.25865E+01_JPRB, 0.23230E+01_JPRB ,&
&  0.19458E+01_JPRB, 0.20677E+01_JPRB, 0.20333E+01_JPRB, 0.20331E+01_JPRB, 0.19490E+01_JPRB, 0.17631E+01_JPRB, 0.16558E+01_JPRB ,&
&  0.16773E+01_JPRB, 0.17609E+01_JPRB, 0.17071E+01_JPRB, 0.14566E+01_JPRB, 0.13656E+01_JPRB, 0.12545E+01_JPRB, 0.13167E+01_JPRB ,&
&  0.14741E+01_JPRB, 0.15844E+01_JPRB, 0.16590E+01_JPRB, 0.15920E+01_JPRB, 0.11445E+01_JPRB, 0.96928E+00_JPRB, 0.84981E+00_JPRB ,&
&  0.78865E+00_JPRB, 0.75406E+00_JPRB, 0.75086E+00_JPRB, 0.76213E+00_JPRB, 0.77401E+00_JPRB, 0.77965E+00_JPRB, 0.77144E+00_JPRB ,&
&  0.76740E+00_JPRB, 0.76738E+00_JPRB, 0.76565E+00_JPRB, 0.76094E+00_JPRB, 0.75580E+00_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.79432E+00_JPRB, 0.82290E+00_JPRB, 0.86437E+00_JPRB, 0.92818E+00_JPRB, 0.98877E+00_JPRB, 0.10819E+01_JPRB, 0.11616E+01_JPRB ,&
&  0.10363E+01_JPRB, 0.70222E+00_JPRB, 0.71651E+00_JPRB, 0.73264E+00_JPRB, 0.75930E+00_JPRB, 0.80335E+00_JPRB, 0.87122E+00_JPRB ,&
&  0.98863E+00_JPRB, 0.11461E+01_JPRB, 0.13243E+01_JPRB, 0.14802E+01_JPRB, 0.16178E+01_JPRB, 0.16775E+01_JPRB, 0.16126E+01_JPRB ,&
&  0.12886E+01_JPRB, 0.12145E+01_JPRB, 0.14711E+01_JPRB, 0.14495E+01_JPRB, 0.14526E+01_JPRB, 0.14419E+01_JPRB, 0.14309E+01_JPRB ,&
&  0.12895E+01_JPRB, 0.12223E+01_JPRB, 0.12305E+01_JPRB, 0.12262E+01_JPRB, 0.11529E+01_JPRB, 0.11775E+01_JPRB, 0.12001E+01_JPRB ,&
&  0.11982E+01_JPRB, 0.11928E+01_JPRB, 0.12129E+01_JPRB, 0.12453E+01_JPRB, 0.12299E+01_JPRB, 0.12365E+01_JPRB, 0.12852E+01_JPRB ,&
&  0.14049E+01_JPRB, 0.14982E+01_JPRB, 0.15540E+01_JPRB, 0.15642E+01_JPRB, 0.16647E+01_JPRB, 0.18328E+01_JPRB, 0.18857E+01_JPRB ,&
&  0.18030E+01_JPRB, 0.17787E+01_JPRB, 0.18116E+01_JPRB, 0.18811E+01_JPRB, 0.18419E+01_JPRB, 0.21827E+01_JPRB, 0.22968E+01_JPRB ,&
&  0.24682E+01_JPRB, 0.24629E+01_JPRB, 0.28435E+01_JPRB, 0.21080E+01_JPRB, 0.22155E+01_JPRB, 0.22830E+01_JPRB, 0.19902E+01_JPRB ,&
&  0.17720E+01_JPRB, 0.18393E+01_JPRB, 0.17805E+01_JPRB, 0.17899E+01_JPRB, 0.17602E+01_JPRB, 0.15501E+01_JPRB, 0.16000E+01_JPRB ,&
&  0.15281E+01_JPRB, 0.16639E+01_JPRB, 0.15888E+01_JPRB, 0.15258E+01_JPRB, 0.13313E+01_JPRB, 0.12749E+01_JPRB, 0.13294E+01_JPRB ,&
&  0.15374E+01_JPRB, 0.16799E+01_JPRB, 0.15352E+01_JPRB, 0.12500E+01_JPRB, 0.88145E+00_JPRB, 0.76248E+00_JPRB, 0.75196E+00_JPRB ,&
&  0.74501E+00_JPRB, 0.74923E+00_JPRB, 0.76013E+00_JPRB, 0.77592E+00_JPRB, 0.78429E+00_JPRB, 0.78118E+00_JPRB, 0.77586E+00_JPRB ,&
&  0.77737E+00_JPRB, 0.77791E+00_JPRB, 0.77416E+00_JPRB, 0.77542E+00_JPRB, 0.78205E+00_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.81399E+00_JPRB, 0.82717E+00_JPRB, 0.84434E+00_JPRB, 0.84698E+00_JPRB, 0.83818E+00_JPRB, 0.81208E+00_JPRB, 0.75334E+00_JPRB ,&
&  0.69732E+00_JPRB, 0.67193E+00_JPRB, 0.70355E+00_JPRB, 0.74561E+00_JPRB, 0.79383E+00_JPRB, 0.85494E+00_JPRB, 0.94798E+00_JPRB ,&
&  0.10648E+01_JPRB, 0.11658E+01_JPRB, 0.12680E+01_JPRB, 0.14348E+01_JPRB, 0.16051E+01_JPRB, 0.15996E+01_JPRB, 0.15848E+01_JPRB ,&
&  0.13790E+01_JPRB, 0.11127E+01_JPRB, 0.14085E+01_JPRB, 0.16290E+01_JPRB, 0.16034E+01_JPRB, 0.15623E+01_JPRB, 0.15087E+01_JPRB ,&
&  0.13442E+01_JPRB, 0.13752E+01_JPRB, 0.14300E+01_JPRB, 0.14472E+01_JPRB, 0.13424E+01_JPRB, 0.13665E+01_JPRB, 0.13908E+01_JPRB ,&
&  0.14188E+01_JPRB, 0.14308E+01_JPRB, 0.14874E+01_JPRB, 0.15224E+01_JPRB, 0.15259E+01_JPRB, 0.15717E+01_JPRB, 0.16475E+01_JPRB ,&
&  0.17263E+01_JPRB, 0.17888E+01_JPRB, 0.18803E+01_JPRB, 0.19417E+01_JPRB, 0.20085E+01_JPRB, 0.20970E+01_JPRB, 0.20698E+01_JPRB ,&
&  0.19869E+01_JPRB, 0.19074E+01_JPRB, 0.20081E+01_JPRB, 0.22392E+01_JPRB, 0.24106E+01_JPRB, 0.21784E+01_JPRB, 0.23616E+01_JPRB ,&
&  0.23296E+01_JPRB, 0.23079E+01_JPRB, 0.22405E+01_JPRB, 0.24996E+01_JPRB, 0.17425E+01_JPRB, 0.18757E+01_JPRB, 0.16327E+01_JPRB ,&
&  0.17085E+01_JPRB, 0.16347E+01_JPRB, 0.15599E+01_JPRB, 0.15333E+01_JPRB, 0.15269E+01_JPRB, 0.13768E+01_JPRB, 0.14372E+01_JPRB ,&
&  0.15088E+01_JPRB, 0.15534E+01_JPRB, 0.15475E+01_JPRB, 0.15659E+01_JPRB, 0.14785E+01_JPRB, 0.14810E+01_JPRB, 0.15135E+01_JPRB ,&
&  0.16756E+01_JPRB, 0.14771E+01_JPRB, 0.12144E+01_JPRB, 0.97406E+00_JPRB, 0.80741E+00_JPRB, 0.70775E+00_JPRB, 0.74188E+00_JPRB ,&
&  0.75764E+00_JPRB, 0.76711E+00_JPRB, 0.76802E+00_JPRB, 0.77624E+00_JPRB, 0.78121E+00_JPRB, 0.77991E+00_JPRB, 0.78978E+00_JPRB ,&
&  0.79401E+00_JPRB, 0.79708E+00_JPRB, 0.79713E+00_JPRB, 0.79919E+00_JPRB, 0.80452E+00_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.81093E+00_JPRB, 0.80458E+00_JPRB, 0.78302E+00_JPRB, 0.74241E+00_JPRB, 0.69449E+00_JPRB, 0.64526E+00_JPRB, 0.61442E+00_JPRB ,&
&  0.58560E+00_JPRB, 0.60305E+00_JPRB, 0.64720E+00_JPRB, 0.70872E+00_JPRB, 0.78379E+00_JPRB, 0.86572E+00_JPRB, 0.94947E+00_JPRB ,&
&  0.10044E+01_JPRB, 0.10565E+01_JPRB, 0.11423E+01_JPRB, 0.13361E+01_JPRB, 0.15394E+01_JPRB, 0.16650E+01_JPRB, 0.17100E+01_JPRB ,&
&  0.17244E+01_JPRB, 0.17251E+01_JPRB, 0.16562E+01_JPRB, 0.13928E+01_JPRB, 0.12418E+01_JPRB, 0.15478E+01_JPRB, 0.14706E+01_JPRB ,&
&  0.14683E+01_JPRB, 0.14642E+01_JPRB, 0.14960E+01_JPRB, 0.15337E+01_JPRB, 0.15263E+01_JPRB, 0.15344E+01_JPRB, 0.15875E+01_JPRB ,&
&  0.16367E+01_JPRB, 0.16976E+01_JPRB, 0.17570E+01_JPRB, 0.17925E+01_JPRB, 0.18478E+01_JPRB, 0.19235E+01_JPRB, 0.19925E+01_JPRB ,&
&  0.20481E+01_JPRB, 0.21401E+01_JPRB, 0.22483E+01_JPRB, 0.23372E+01_JPRB, 0.23106E+01_JPRB, 0.23508E+01_JPRB, 0.23012E+01_JPRB ,&
&  0.22261E+01_JPRB, 0.21691E+01_JPRB, 0.21763E+01_JPRB, 0.23070E+01_JPRB, 0.23408E+01_JPRB, 0.20470E+01_JPRB, 0.20445E+01_JPRB ,&
&  0.21404E+01_JPRB, 0.21369E+01_JPRB, 0.18600E+01_JPRB, 0.23206E+01_JPRB, 0.13905E+01_JPRB, 0.16215E+01_JPRB, 0.17208E+01_JPRB ,&
&  0.16379E+01_JPRB, 0.14904E+01_JPRB, 0.14293E+01_JPRB, 0.14201E+01_JPRB, 0.13779E+01_JPRB, 0.13194E+01_JPRB, 0.14356E+01_JPRB ,&
&  0.14956E+01_JPRB, 0.14828E+01_JPRB, 0.15216E+01_JPRB, 0.15205E+01_JPRB, 0.15210E+01_JPRB, 0.14428E+01_JPRB, 0.13185E+01_JPRB ,&
&  0.11842E+01_JPRB, 0.12958E+01_JPRB, 0.11616E+01_JPRB, 0.10940E+01_JPRB, 0.94331E+00_JPRB, 0.72035E+00_JPRB, 0.75533E+00_JPRB ,&
&  0.76895E+00_JPRB, 0.76856E+00_JPRB, 0.78035E+00_JPRB, 0.78568E+00_JPRB, 0.77622E+00_JPRB, 0.77780E+00_JPRB, 0.79064E+00_JPRB ,&
&  0.78995E+00_JPRB, 0.79037E+00_JPRB, 0.79416E+00_JPRB, 0.79667E+00_JPRB, 0.80500E+00_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.69584E+00_JPRB, 0.68129E+00_JPRB, 0.65843E+00_JPRB, 0.63345E+00_JPRB, 0.60932E+00_JPRB, 0.57784E+00_JPRB, 0.57637E+00_JPRB ,&
&  0.56200E+00_JPRB, 0.56774E+00_JPRB, 0.58876E+00_JPRB, 0.60890E+00_JPRB, 0.64724E+00_JPRB, 0.70929E+00_JPRB, 0.77118E+00_JPRB ,&
&  0.81469E+00_JPRB, 0.83451E+00_JPRB, 0.89842E+00_JPRB, 0.99766E+00_JPRB, 0.11647E+01_JPRB, 0.14333E+01_JPRB, 0.18626E+01_JPRB ,&
&  0.20565E+01_JPRB, 0.21799E+01_JPRB, 0.21557E+01_JPRB, 0.20176E+01_JPRB, 0.17461E+01_JPRB, 0.17077E+01_JPRB, 0.18783E+01_JPRB ,&
&  0.19773E+01_JPRB, 0.18571E+01_JPRB, 0.16606E+01_JPRB, 0.16455E+01_JPRB, 0.16571E+01_JPRB, 0.16866E+01_JPRB, 0.17414E+01_JPRB ,&
&  0.18102E+01_JPRB, 0.18616E+01_JPRB, 0.19102E+01_JPRB, 0.19822E+01_JPRB, 0.20433E+01_JPRB, 0.21058E+01_JPRB, 0.21884E+01_JPRB ,&
&  0.22734E+01_JPRB, 0.23465E+01_JPRB, 0.24097E+01_JPRB, 0.24606E+01_JPRB, 0.25653E+01_JPRB, 0.26665E+01_JPRB, 0.26632E+01_JPRB ,&
&  0.26023E+01_JPRB, 0.24251E+01_JPRB, 0.24543E+01_JPRB, 0.25462E+01_JPRB, 0.24234E+01_JPRB, 0.21803E+01_JPRB, 0.20967E+01_JPRB ,&
&  0.21521E+01_JPRB, 0.21821E+01_JPRB, 0.15092E+01_JPRB, 0.18511E+01_JPRB, 0.20316E+01_JPRB, 0.17745E+01_JPRB, 0.15782E+01_JPRB ,&
&  0.14927E+01_JPRB, 0.13862E+01_JPRB, 0.13733E+01_JPRB, 0.13350E+01_JPRB, 0.14058E+01_JPRB, 0.13816E+01_JPRB, 0.12169E+01_JPRB ,&
&  0.12846E+01_JPRB, 0.12421E+01_JPRB, 0.13127E+01_JPRB, 0.13520E+01_JPRB, 0.13075E+01_JPRB, 0.12025E+01_JPRB, 0.10459E+01_JPRB ,&
&  0.95181E+00_JPRB, 0.11848E+01_JPRB, 0.11207E+01_JPRB, 0.12594E+01_JPRB, 0.11837E+01_JPRB, 0.72056E+00_JPRB, 0.75327E+00_JPRB ,&
&  0.75684E+00_JPRB, 0.76133E+00_JPRB, 0.76837E+00_JPRB, 0.76579E+00_JPRB, 0.76251E+00_JPRB, 0.76264E+00_JPRB, 0.76773E+00_JPRB ,&
&  0.75984E+00_JPRB, 0.74822E+00_JPRB, 0.73684E+00_JPRB, 0.72895E+00_JPRB, 0.71560E+00_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.64497E+00_JPRB, 0.64336E+00_JPRB, 0.64123E+00_JPRB, 0.64504E+00_JPRB, 0.65487E+00_JPRB, 0.65584E+00_JPRB, 0.64629E+00_JPRB ,&
&  0.62708E+00_JPRB, 0.61735E+00_JPRB, 0.59494E+00_JPRB, 0.56965E+00_JPRB, 0.57035E+00_JPRB, 0.61689E+00_JPRB, 0.66611E+00_JPRB ,&
&  0.69731E+00_JPRB, 0.71341E+00_JPRB, 0.71664E+00_JPRB, 0.72210E+00_JPRB, 0.76283E+00_JPRB, 0.79415E+00_JPRB, 0.99448E+00_JPRB ,&
&  0.13644E+01_JPRB, 0.18997E+01_JPRB, 0.23008E+01_JPRB, 0.25071E+01_JPRB, 0.23531E+01_JPRB, 0.19528E+01_JPRB, 0.21683E+01_JPRB ,&
&  0.19698E+01_JPRB, 0.16081E+01_JPRB, 0.15203E+01_JPRB, 0.15267E+01_JPRB, 0.15669E+01_JPRB, 0.16046E+01_JPRB, 0.16711E+01_JPRB ,&
&  0.17417E+01_JPRB, 0.18177E+01_JPRB, 0.18984E+01_JPRB, 0.19937E+01_JPRB, 0.20944E+01_JPRB, 0.22075E+01_JPRB, 0.23225E+01_JPRB ,&
&  0.24516E+01_JPRB, 0.25271E+01_JPRB, 0.26516E+01_JPRB, 0.25410E+01_JPRB, 0.27479E+01_JPRB, 0.29272E+01_JPRB, 0.30115E+01_JPRB ,&
&  0.29598E+01_JPRB, 0.28289E+01_JPRB, 0.28778E+01_JPRB, 0.28539E+01_JPRB, 0.28164E+01_JPRB, 0.26553E+01_JPRB, 0.24451E+01_JPRB ,&
&  0.24904E+01_JPRB, 0.23169E+01_JPRB, 0.15527E+01_JPRB, 0.15343E+01_JPRB, 0.16528E+01_JPRB, 0.13615E+01_JPRB, 0.15091E+01_JPRB ,&
&  0.13839E+01_JPRB, 0.12917E+01_JPRB, 0.11992E+01_JPRB, 0.11338E+01_JPRB, 0.12063E+01_JPRB, 0.12870E+01_JPRB, 0.11152E+01_JPRB ,&
&  0.10527E+01_JPRB, 0.10145E+01_JPRB, 0.10406E+01_JPRB, 0.10786E+01_JPRB, 0.10568E+01_JPRB, 0.10054E+01_JPRB, 0.94727E+00_JPRB ,&
&  0.97570E+00_JPRB, 0.93948E+00_JPRB, 0.82456E+00_JPRB, 0.83502E+00_JPRB, 0.69865E+00_JPRB, 0.69069E+00_JPRB, 0.70723E+00_JPRB ,&
&  0.70980E+00_JPRB, 0.69473E+00_JPRB, 0.69407E+00_JPRB, 0.69178E+00_JPRB, 0.69532E+00_JPRB, 0.68420E+00_JPRB, 0.69123E+00_JPRB ,&
&  0.69068E+00_JPRB, 0.68484E+00_JPRB, 0.67637E+00_JPRB, 0.67526E+00_JPRB, 0.65983E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.74111E+00_JPRB, 0.75034E+00_JPRB, 0.76645E+00_JPRB, 0.77931E+00_JPRB, 0.80089E+00_JPRB, 0.79205E+00_JPRB, 0.78238E+00_JPRB ,&
&  0.75264E+00_JPRB, 0.74699E+00_JPRB, 0.74039E+00_JPRB, 0.73735E+00_JPRB, 0.74284E+00_JPRB, 0.76672E+00_JPRB, 0.81169E+00_JPRB ,&
&  0.82589E+00_JPRB, 0.83154E+00_JPRB, 0.82527E+00_JPRB, 0.81519E+00_JPRB, 0.78326E+00_JPRB, 0.75406E+00_JPRB, 0.73051E+00_JPRB ,&
&  0.75201E+00_JPRB, 0.94791E+00_JPRB, 0.12078E+01_JPRB, 0.14530E+01_JPRB, 0.17321E+01_JPRB, 0.16175E+01_JPRB, 0.18465E+01_JPRB ,&
&  0.19248E+01_JPRB, 0.14862E+01_JPRB, 0.15386E+01_JPRB, 0.13810E+01_JPRB, 0.14003E+01_JPRB, 0.14118E+01_JPRB, 0.14741E+01_JPRB ,&
&  0.15662E+01_JPRB, 0.16342E+01_JPRB, 0.16847E+01_JPRB, 0.17435E+01_JPRB, 0.18257E+01_JPRB, 0.19071E+01_JPRB, 0.19617E+01_JPRB ,&
&  0.20029E+01_JPRB, 0.19555E+01_JPRB, 0.20099E+01_JPRB, 0.23545E+01_JPRB, 0.26568E+01_JPRB, 0.28224E+01_JPRB, 0.29503E+01_JPRB ,&
&  0.31193E+01_JPRB, 0.32089E+01_JPRB, 0.29035E+01_JPRB, 0.28494E+01_JPRB, 0.29955E+01_JPRB, 0.26902E+01_JPRB, 0.26751E+01_JPRB ,&
&  0.27652E+01_JPRB, 0.23860E+01_JPRB, 0.14376E+01_JPRB, 0.15128E+01_JPRB, 0.15494E+01_JPRB, 0.14750E+01_JPRB, 0.15060E+01_JPRB ,&
&  0.13139E+01_JPRB, 0.12018E+01_JPRB, 0.10580E+01_JPRB, 0.10052E+01_JPRB, 0.10107E+01_JPRB, 0.10782E+01_JPRB, 0.10369E+01_JPRB ,&
&  0.94234E+00_JPRB, 0.90522E+00_JPRB, 0.97314E+00_JPRB, 0.10342E+01_JPRB, 0.10177E+01_JPRB, 0.95813E+00_JPRB, 0.88791E+00_JPRB ,&
&  0.92542E+00_JPRB, 0.86065E+00_JPRB, 0.94111E+00_JPRB, 0.95972E+00_JPRB, 0.68081E+00_JPRB, 0.70351E+00_JPRB, 0.70227E+00_JPRB ,&
&  0.71045E+00_JPRB, 0.70307E+00_JPRB, 0.69058E+00_JPRB, 0.69234E+00_JPRB, 0.68603E+00_JPRB, 0.67922E+00_JPRB, 0.69631E+00_JPRB ,&
&  0.72033E+00_JPRB, 0.71943E+00_JPRB, 0.71805E+00_JPRB, 0.72026E+00_JPRB, 0.72882E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.84781E+00_JPRB, 0.86738E+00_JPRB, 0.89079E+00_JPRB, 0.89456E+00_JPRB, 0.90112E+00_JPRB, 0.89539E+00_JPRB, 0.89248E+00_JPRB ,&
&  0.88053E+00_JPRB, 0.87776E+00_JPRB, 0.88051E+00_JPRB, 0.88570E+00_JPRB, 0.89569E+00_JPRB, 0.90821E+00_JPRB, 0.93865E+00_JPRB ,&
&  0.93902E+00_JPRB, 0.94449E+00_JPRB, 0.95216E+00_JPRB, 0.96724E+00_JPRB, 0.98261E+00_JPRB, 0.98142E+00_JPRB, 0.95675E+00_JPRB ,&
&  0.93994E+00_JPRB, 0.93912E+00_JPRB, 0.95349E+00_JPRB, 0.98217E+00_JPRB, 0.11452E+01_JPRB, 0.14530E+01_JPRB, 0.19423E+01_JPRB ,&
&  0.17954E+01_JPRB, 0.15020E+01_JPRB, 0.13657E+01_JPRB, 0.11841E+01_JPRB, 0.11730E+01_JPRB, 0.12045E+01_JPRB, 0.11594E+01_JPRB ,&
&  0.11518E+01_JPRB, 0.11297E+01_JPRB, 0.11455E+01_JPRB, 0.11912E+01_JPRB, 0.12313E+01_JPRB, 0.12223E+01_JPRB, 0.12028E+01_JPRB ,&
&  0.11094E+01_JPRB, 0.10823E+01_JPRB, 0.11606E+01_JPRB, 0.13707E+01_JPRB, 0.18025E+01_JPRB, 0.21089E+01_JPRB, 0.20193E+01_JPRB ,&
&  0.22279E+01_JPRB, 0.25812E+01_JPRB, 0.26505E+01_JPRB, 0.26392E+01_JPRB, 0.25303E+01_JPRB, 0.23680E+01_JPRB, 0.22623E+01_JPRB ,&
&  0.22143E+01_JPRB, 0.19638E+01_JPRB, 0.14689E+01_JPRB, 0.15491E+01_JPRB, 0.14973E+01_JPRB, 0.15524E+01_JPRB, 0.14105E+01_JPRB ,&
&  0.12450E+01_JPRB, 0.11386E+01_JPRB, 0.10449E+01_JPRB, 0.95348E+00_JPRB, 0.90998E+00_JPRB, 0.94510E+00_JPRB, 0.94234E+00_JPRB ,&
&  0.89067E+00_JPRB, 0.90433E+00_JPRB, 0.90203E+00_JPRB, 0.90032E+00_JPRB, 0.10248E+01_JPRB, 0.88675E+00_JPRB, 0.99667E+00_JPRB ,&
&  0.87902E+00_JPRB, 0.83317E+00_JPRB, 0.68458E+00_JPRB, 0.96148E+00_JPRB, 0.71772E+00_JPRB, 0.66764E+00_JPRB, 0.69699E+00_JPRB ,&
&  0.73522E+00_JPRB, 0.75310E+00_JPRB, 0.77725E+00_JPRB, 0.79676E+00_JPRB, 0.79185E+00_JPRB, 0.78474E+00_JPRB, 0.79655E+00_JPRB ,&
&  0.80796E+00_JPRB, 0.81073E+00_JPRB, 0.81519E+00_JPRB, 0.81849E+00_JPRB, 0.83263E+00_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.91926E+00_JPRB, 0.93634E+00_JPRB, 0.95021E+00_JPRB, 0.95964E+00_JPRB, 0.96385E+00_JPRB, 0.96745E+00_JPRB, 0.97199E+00_JPRB ,&
&  0.97334E+00_JPRB, 0.97135E+00_JPRB, 0.96983E+00_JPRB, 0.98498E+00_JPRB, 0.10057E+01_JPRB, 0.10149E+01_JPRB, 0.10380E+01_JPRB ,&
&  0.10332E+01_JPRB, 0.10461E+01_JPRB, 0.10659E+01_JPRB, 0.10801E+01_JPRB, 0.11044E+01_JPRB, 0.11045E+01_JPRB, 0.10894E+01_JPRB ,&
&  0.10855E+01_JPRB, 0.10767E+01_JPRB, 0.10778E+01_JPRB, 0.10806E+01_JPRB, 0.10937E+01_JPRB, 0.11113E+01_JPRB, 0.12793E+01_JPRB ,&
&  0.13020E+01_JPRB, 0.16028E+01_JPRB, 0.14982E+01_JPRB, 0.13442E+01_JPRB, 0.13437E+01_JPRB, 0.12300E+01_JPRB, 0.11726E+01_JPRB ,&
&  0.10888E+01_JPRB, 0.10069E+01_JPRB, 0.97406E+00_JPRB, 0.96635E+00_JPRB, 0.10132E+01_JPRB, 0.10725E+01_JPRB, 0.11064E+01_JPRB ,&
&  0.11858E+01_JPRB, 0.12442E+01_JPRB, 0.13149E+01_JPRB, 0.13688E+01_JPRB, 0.14139E+01_JPRB, 0.15596E+01_JPRB, 0.14854E+01_JPRB ,&
&  0.15263E+01_JPRB, 0.17002E+01_JPRB, 0.17639E+01_JPRB, 0.18170E+01_JPRB, 0.17030E+01_JPRB, 0.15737E+01_JPRB, 0.14084E+01_JPRB ,&
&  0.12449E+01_JPRB, 0.12012E+01_JPRB, 0.13224E+01_JPRB, 0.13630E+01_JPRB, 0.13729E+01_JPRB, 0.14087E+01_JPRB, 0.12876E+01_JPRB ,&
&  0.11715E+01_JPRB, 0.10884E+01_JPRB, 0.99959E+00_JPRB, 0.91542E+00_JPRB, 0.87701E+00_JPRB, 0.87012E+00_JPRB, 0.84945E+00_JPRB ,&
&  0.81677E+00_JPRB, 0.80241E+00_JPRB, 0.80406E+00_JPRB, 0.81873E+00_JPRB, 0.80679E+00_JPRB, 0.11390E+01_JPRB, 0.14059E+01_JPRB ,&
&  0.92826E+00_JPRB, 0.73270E+00_JPRB, 0.52744E+00_JPRB, 0.74106E+00_JPRB, 0.63731E+00_JPRB, 0.65151E+00_JPRB, 0.72409E+00_JPRB ,&
&  0.82055E+00_JPRB, 0.89328E+00_JPRB, 0.91972E+00_JPRB, 0.94803E+00_JPRB, 0.84770E+00_JPRB, 0.82953E+00_JPRB, 0.82554E+00_JPRB ,&
&  0.83355E+00_JPRB, 0.84845E+00_JPRB, 0.86407E+00_JPRB, 0.88038E+00_JPRB, 0.90012E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.95947E+00_JPRB, 0.97198E+00_JPRB, 0.98336E+00_JPRB, 0.99851E+00_JPRB, 0.10123E+01_JPRB, 0.10298E+01_JPRB, 0.10414E+01_JPRB ,&
&  0.10525E+01_JPRB, 0.10600E+01_JPRB, 0.10604E+01_JPRB, 0.10738E+01_JPRB, 0.10866E+01_JPRB, 0.11110E+01_JPRB, 0.11391E+01_JPRB ,&
&  0.11708E+01_JPRB, 0.12041E+01_JPRB, 0.12415E+01_JPRB, 0.12624E+01_JPRB, 0.12787E+01_JPRB, 0.12975E+01_JPRB, 0.13151E+01_JPRB ,&
&  0.13260E+01_JPRB, 0.13306E+01_JPRB, 0.13490E+01_JPRB, 0.13822E+01_JPRB, 0.14224E+01_JPRB, 0.14739E+01_JPRB, 0.12810E+01_JPRB ,&
&  0.15056E+01_JPRB, 0.16813E+01_JPRB, 0.16782E+01_JPRB, 0.15666E+01_JPRB, 0.15435E+01_JPRB, 0.15337E+01_JPRB, 0.15566E+01_JPRB ,&
&  0.16113E+01_JPRB, 0.15650E+01_JPRB, 0.15004E+01_JPRB, 0.13919E+01_JPRB, 0.13370E+01_JPRB, 0.13684E+01_JPRB, 0.14365E+01_JPRB ,&
&  0.14808E+01_JPRB, 0.15283E+01_JPRB, 0.15376E+01_JPRB, 0.15322E+01_JPRB, 0.15354E+01_JPRB, 0.15216E+01_JPRB, 0.14362E+01_JPRB ,&
&  0.13623E+01_JPRB, 0.13415E+01_JPRB, 0.11665E+01_JPRB, 0.11652E+01_JPRB, 0.11569E+01_JPRB, 0.10549E+01_JPRB, 0.90433E+00_JPRB ,&
&  0.85815E+00_JPRB, 0.81934E+00_JPRB, 0.89025E+00_JPRB, 0.12092E+01_JPRB, 0.12879E+01_JPRB, 0.12550E+01_JPRB, 0.11815E+01_JPRB ,&
&  0.10791E+01_JPRB, 0.98502E+00_JPRB, 0.87676E+00_JPRB, 0.81100E+00_JPRB, 0.78778E+00_JPRB, 0.80900E+00_JPRB, 0.81630E+00_JPRB ,&
&  0.80511E+00_JPRB, 0.78535E+00_JPRB, 0.77407E+00_JPRB, 0.76294E+00_JPRB, 0.81954E+00_JPRB, 0.89739E+00_JPRB, 0.12665E+01_JPRB ,&
&  0.95192E+00_JPRB, 0.59352E+00_JPRB, 0.57745E+00_JPRB, 0.65529E+00_JPRB, 0.58387E+00_JPRB, 0.61062E+00_JPRB, 0.71920E+00_JPRB ,&
&  0.92463E+00_JPRB, 0.10465E+01_JPRB, 0.10330E+01_JPRB, 0.10056E+01_JPRB, 0.95173E+00_JPRB, 0.95158E+00_JPRB, 0.93192E+00_JPRB ,&
&  0.91826E+00_JPRB, 0.91949E+00_JPRB, 0.92523E+00_JPRB, 0.93228E+00_JPRB, 0.94478E+00_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.98203E+00_JPRB, 0.99693E+00_JPRB, 0.10102E+01_JPRB, 0.10230E+01_JPRB, 0.10329E+01_JPRB, 0.10518E+01_JPRB, 0.10591E+01_JPRB ,&
&  0.10706E+01_JPRB, 0.10767E+01_JPRB, 0.10924E+01_JPRB, 0.11127E+01_JPRB, 0.11311E+01_JPRB, 0.11567E+01_JPRB, 0.11883E+01_JPRB ,&
&  0.12266E+01_JPRB, 0.12543E+01_JPRB, 0.12864E+01_JPRB, 0.13021E+01_JPRB, 0.12946E+01_JPRB, 0.13004E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.12926E+01_JPRB, 0.12853E+01_JPRB, 0.13006E+01_JPRB, 0.13232E+01_JPRB, 0.13246E+01_JPRB, 0.13396E+01_JPRB, 0.10576E+01_JPRB ,&
&  0.15799E+01_JPRB, 0.18166E+01_JPRB, 0.18955E+01_JPRB, 0.17902E+01_JPRB, 0.17413E+01_JPRB, 0.17924E+01_JPRB, 0.18628E+01_JPRB ,&
&  0.19607E+01_JPRB, 0.20536E+01_JPRB, 0.20134E+01_JPRB, 0.19055E+01_JPRB, 0.18080E+01_JPRB, 0.18198E+01_JPRB, 0.18476E+01_JPRB ,&
&  0.18778E+01_JPRB, 0.19017E+01_JPRB, 0.18944E+01_JPRB, 0.18750E+01_JPRB, 0.18566E+01_JPRB, 0.18272E+01_JPRB, 0.17635E+01_JPRB ,&
&  0.16998E+01_JPRB, 0.16343E+01_JPRB, 0.14449E+01_JPRB, 0.15537E+01_JPRB, 0.14486E+01_JPRB, 0.12120E+01_JPRB, 0.98013E+00_JPRB ,&
&  0.85813E+00_JPRB, 0.84621E+00_JPRB, 0.10263E+01_JPRB, 0.11665E+01_JPRB, 0.11384E+01_JPRB, 0.10954E+01_JPRB, 0.10385E+01_JPRB ,&
&  0.93205E+00_JPRB, 0.83490E+00_JPRB, 0.76975E+00_JPRB, 0.74552E+00_JPRB, 0.74951E+00_JPRB, 0.77260E+00_JPRB, 0.78443E+00_JPRB ,&
&  0.78254E+00_JPRB, 0.78021E+00_JPRB, 0.76575E+00_JPRB, 0.75058E+00_JPRB, 0.75490E+00_JPRB, 0.74611E+00_JPRB, 0.10576E+01_JPRB ,&
&  0.89782E+00_JPRB, 0.66681E+00_JPRB, 0.80890E+00_JPRB, 0.44727E+00_JPRB, 0.53721E+00_JPRB, 0.54744E+00_JPRB, 0.64430E+00_JPRB ,&
&  0.77779E+00_JPRB, 0.11034E+01_JPRB, 0.15232E+01_JPRB, 0.13360E+01_JPRB, 0.12084E+01_JPRB, 0.10990E+01_JPRB, 0.10078E+01_JPRB ,&
&  0.97175E+00_JPRB, 0.96050E+00_JPRB, 0.95848E+00_JPRB, 0.95627E+00_JPRB, 0.96625E+00_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.10151E+01_JPRB, 0.10328E+01_JPRB, 0.10335E+01_JPRB, 0.10489E+01_JPRB, 0.10286E+01_JPRB, 0.10458E+01_JPRB, 0.10467E+01_JPRB ,&
&  0.10649E+01_JPRB, 0.10756E+01_JPRB, 0.10947E+01_JPRB, 0.11124E+01_JPRB, 0.11402E+01_JPRB, 0.11655E+01_JPRB, 0.11866E+01_JPRB ,&
&  0.12152E+01_JPRB, 0.12464E+01_JPRB, 0.12720E+01_JPRB, 0.12955E+01_JPRB, 0.12833E+01_JPRB, 0.12923E+01_JPRB, 0.12939E+01_JPRB ,&
&  0.13206E+01_JPRB, 0.13144E+01_JPRB, 0.13276E+01_JPRB, 0.13366E+01_JPRB, 0.13263E+01_JPRB, 0.13523E+01_JPRB, 0.10121E+01_JPRB ,&
&  0.16598E+01_JPRB, 0.19083E+01_JPRB, 0.20445E+01_JPRB, 0.19735E+01_JPRB, 0.18960E+01_JPRB, 0.18948E+01_JPRB, 0.19270E+01_JPRB ,&
&  0.20146E+01_JPRB, 0.20865E+01_JPRB, 0.20757E+01_JPRB, 0.22118E+01_JPRB, 0.22758E+01_JPRB, 0.22327E+01_JPRB, 0.22954E+01_JPRB ,&
&  0.23471E+01_JPRB, 0.23943E+01_JPRB, 0.24324E+01_JPRB, 0.24668E+01_JPRB, 0.24898E+01_JPRB, 0.25559E+01_JPRB, 0.25912E+01_JPRB ,&
&  0.25700E+01_JPRB, 0.25689E+01_JPRB, 0.25598E+01_JPRB, 0.24223E+01_JPRB, 0.22717E+01_JPRB, 0.19027E+01_JPRB, 0.15557E+01_JPRB ,&
&  0.11011E+01_JPRB, 0.96260E+00_JPRB, 0.10305E+01_JPRB, 0.11217E+01_JPRB, 0.10714E+01_JPRB, 0.10069E+01_JPRB, 0.91407E+00_JPRB ,&
&  0.80599E+00_JPRB, 0.77204E+00_JPRB, 0.74509E+00_JPRB, 0.74299E+00_JPRB, 0.75458E+00_JPRB, 0.77219E+00_JPRB, 0.78278E+00_JPRB ,&
&  0.78902E+00_JPRB, 0.79203E+00_JPRB, 0.76663E+00_JPRB, 0.73847E+00_JPRB, 0.73920E+00_JPRB, 0.80021E+00_JPRB, 0.78145E+00_JPRB ,&
&  0.90745E+00_JPRB, 0.73934E+00_JPRB, 0.68698E+00_JPRB, 0.64211E+00_JPRB, 0.56559E+00_JPRB, 0.58538E+00_JPRB, 0.75454E+00_JPRB ,&
&  0.12439E+01_JPRB, 0.16195E+01_JPRB, 0.23311E+01_JPRB, 0.39589E+01_JPRB, 0.27978E+01_JPRB, 0.20720E+01_JPRB, 0.10729E+01_JPRB ,&
&  0.10076E+01_JPRB, 0.98612E+00_JPRB, 0.97542E+00_JPRB, 0.96813E+00_JPRB, 0.98961E+00_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.94398E+00_JPRB, 0.96904E+00_JPRB, 0.99643E+00_JPRB, 0.10207E+01_JPRB, 0.10293E+01_JPRB, 0.10317E+01_JPRB, 0.10309E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10349E+01_JPRB, 0.10586E+01_JPRB, 0.10367E+01_JPRB, 0.10577E+01_JPRB, 0.10451E+01_JPRB, 0.10539E+01_JPRB ,&
&  0.10709E+01_JPRB, 0.11023E+01_JPRB, 0.11406E+01_JPRB, 0.11602E+01_JPRB, 0.11679E+01_JPRB, 0.12068E+01_JPRB, 0.12607E+01_JPRB ,&
&  0.13266E+01_JPRB, 0.13617E+01_JPRB, 0.14067E+01_JPRB, 0.14324E+01_JPRB, 0.14152E+01_JPRB, 0.13963E+01_JPRB, 0.10270E+01_JPRB ,&
&  0.15821E+01_JPRB, 0.19206E+01_JPRB, 0.21561E+01_JPRB, 0.21528E+01_JPRB, 0.20388E+01_JPRB, 0.19343E+01_JPRB, 0.19556E+01_JPRB ,&
&  0.19992E+01_JPRB, 0.20084E+01_JPRB, 0.20104E+01_JPRB, 0.20825E+01_JPRB, 0.23709E+01_JPRB, 0.23325E+01_JPRB, 0.24003E+01_JPRB ,&
&  0.24751E+01_JPRB, 0.25627E+01_JPRB, 0.26755E+01_JPRB, 0.28114E+01_JPRB, 0.29411E+01_JPRB, 0.30694E+01_JPRB, 0.32100E+01_JPRB ,&
&  0.33145E+01_JPRB, 0.33516E+01_JPRB, 0.33858E+01_JPRB, 0.30137E+01_JPRB, 0.27685E+01_JPRB, 0.27697E+01_JPRB, 0.20784E+01_JPRB ,&
&  0.12586E+01_JPRB, 0.10605E+01_JPRB, 0.11086E+01_JPRB, 0.11397E+01_JPRB, 0.10895E+01_JPRB, 0.96726E+00_JPRB, 0.87851E+00_JPRB ,&
&  0.81624E+00_JPRB, 0.78947E+00_JPRB, 0.76990E+00_JPRB, 0.78700E+00_JPRB, 0.79482E+00_JPRB, 0.80925E+00_JPRB, 0.81898E+00_JPRB ,&
&  0.83092E+00_JPRB, 0.80919E+00_JPRB, 0.77621E+00_JPRB, 0.75827E+00_JPRB, 0.77447E+00_JPRB, 0.86856E+00_JPRB, 0.10275E+01_JPRB ,&
&  0.11099E+01_JPRB, 0.69272E+00_JPRB, 0.71308E+00_JPRB, 0.67514E+00_JPRB, 0.67142E+00_JPRB, 0.72855E+00_JPRB, 0.90392E+00_JPRB ,&
&  0.13586E+01_JPRB, 0.20888E+01_JPRB, 0.23563E+01_JPRB, 0.20243E+01_JPRB, 0.24922E+01_JPRB, 0.17898E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.98310E+00_JPRB, 0.95446E+00_JPRB, 0.91744E+00_JPRB, 0.90700E+00_JPRB, 0.91663E+00_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.78908E+00_JPRB, 0.80526E+00_JPRB, 0.82803E+00_JPRB, 0.86378E+00_JPRB, 0.90324E+00_JPRB, 0.93028E+00_JPRB, 0.95740E+00_JPRB ,&
&  0.96735E+00_JPRB, 0.98250E+00_JPRB, 0.99585E+00_JPRB, 0.95628E+00_JPRB, 0.96806E+00_JPRB, 0.95379E+00_JPRB, 0.94933E+00_JPRB ,&
&  0.94866E+00_JPRB, 0.96720E+00_JPRB, 0.99098E+00_JPRB, 0.10154E+01_JPRB, 0.10528E+01_JPRB, 0.11000E+01_JPRB, 0.11620E+01_JPRB ,&
&  0.12391E+01_JPRB, 0.13084E+01_JPRB, 0.13889E+01_JPRB, 0.14385E+01_JPRB, 0.14861E+01_JPRB, 0.15110E+01_JPRB, 0.14898E+01_JPRB ,&
&  0.96129E+00_JPRB, 0.17068E+01_JPRB, 0.19263E+01_JPRB, 0.20486E+01_JPRB, 0.21329E+01_JPRB, 0.20276E+01_JPRB, 0.20326E+01_JPRB ,&
&  0.19681E+01_JPRB, 0.18537E+01_JPRB, 0.19008E+01_JPRB, 0.20208E+01_JPRB, 0.21688E+01_JPRB, 0.21503E+01_JPRB, 0.21745E+01_JPRB ,&
&  0.22195E+01_JPRB, 0.23138E+01_JPRB, 0.24407E+01_JPRB, 0.25883E+01_JPRB, 0.27689E+01_JPRB, 0.29809E+01_JPRB, 0.32441E+01_JPRB ,&
&  0.35135E+01_JPRB, 0.37588E+01_JPRB, 0.39708E+01_JPRB, 0.35346E+01_JPRB, 0.31218E+01_JPRB, 0.31861E+01_JPRB, 0.26691E+01_JPRB ,&
&  0.15630E+01_JPRB, 0.11055E+01_JPRB, 0.11522E+01_JPRB, 0.12609E+01_JPRB, 0.11694E+01_JPRB, 0.10656E+01_JPRB, 0.96026E+00_JPRB ,&
&  0.89827E+00_JPRB, 0.85509E+00_JPRB, 0.87447E+00_JPRB, 0.84725E+00_JPRB, 0.87581E+00_JPRB, 0.87506E+00_JPRB, 0.88606E+00_JPRB ,&
&  0.89556E+00_JPRB, 0.86954E+00_JPRB, 0.85089E+00_JPRB, 0.83875E+00_JPRB, 0.81986E+00_JPRB, 0.80353E+00_JPRB, 0.79471E+00_JPRB ,&
&  0.80458E+00_JPRB, 0.76632E+00_JPRB, 0.74576E+00_JPRB, 0.76197E+00_JPRB, 0.78742E+00_JPRB, 0.82374E+00_JPRB, 0.91797E+00_JPRB ,&
&  0.10741E+01_JPRB, 0.12411E+01_JPRB, 0.15249E+01_JPRB, 0.16167E+01_JPRB, 0.15623E+01_JPRB, 0.11819E+01_JPRB, 0.87281E+00_JPRB ,&
&  0.86816E+00_JPRB, 0.82548E+00_JPRB, 0.79107E+00_JPRB, 0.77180E+00_JPRB, 0.77533E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.71472E+00_JPRB, 0.72629E+00_JPRB, 0.71968E+00_JPRB, 0.74338E+00_JPRB, 0.76325E+00_JPRB, 0.79279E+00_JPRB, 0.82217E+00_JPRB ,&
&  0.84697E+00_JPRB, 0.87006E+00_JPRB, 0.90094E+00_JPRB, 0.88174E+00_JPRB, 0.89305E+00_JPRB, 0.88417E+00_JPRB, 0.86215E+00_JPRB ,&
&  0.85904E+00_JPRB, 0.87217E+00_JPRB, 0.88715E+00_JPRB, 0.90285E+00_JPRB, 0.92166E+00_JPRB, 0.95787E+00_JPRB, 0.10027E+01_JPRB ,&
&  0.10593E+01_JPRB, 0.11312E+01_JPRB, 0.12267E+01_JPRB, 0.13092E+01_JPRB, 0.13875E+01_JPRB, 0.14063E+01_JPRB, 0.13294E+01_JPRB ,&
&  0.66283E+00_JPRB, 0.93679E+00_JPRB, 0.15763E+01_JPRB, 0.18086E+01_JPRB, 0.19229E+01_JPRB, 0.19757E+01_JPRB, 0.19702E+01_JPRB ,&
&  0.18518E+01_JPRB, 0.16815E+01_JPRB, 0.17084E+01_JPRB, 0.19606E+01_JPRB, 0.19294E+01_JPRB, 0.19289E+01_JPRB, 0.19271E+01_JPRB ,&
&  0.19456E+01_JPRB, 0.20015E+01_JPRB, 0.20911E+01_JPRB, 0.22058E+01_JPRB, 0.23912E+01_JPRB, 0.26320E+01_JPRB, 0.29464E+01_JPRB ,&
&  0.33309E+01_JPRB, 0.37285E+01_JPRB, 0.40859E+01_JPRB, 0.36142E+01_JPRB, 0.35276E+01_JPRB, 0.34899E+01_JPRB, 0.36081E+01_JPRB ,&
&  0.19785E+01_JPRB, 0.12313E+01_JPRB, 0.12799E+01_JPRB, 0.13861E+01_JPRB, 0.12531E+01_JPRB, 0.11786E+01_JPRB, 0.11218E+01_JPRB ,&
&  0.10568E+01_JPRB, 0.99293E+00_JPRB, 0.97422E+00_JPRB, 0.94656E+00_JPRB, 0.93390E+00_JPRB, 0.94561E+00_JPRB, 0.96913E+00_JPRB ,&
&  0.96616E+00_JPRB, 0.95847E+00_JPRB, 0.95368E+00_JPRB, 0.95289E+00_JPRB, 0.92199E+00_JPRB, 0.89201E+00_JPRB, 0.88119E+00_JPRB ,&
&  0.88218E+00_JPRB, 0.86183E+00_JPRB, 0.86899E+00_JPRB, 0.90232E+00_JPRB, 0.93847E+00_JPRB, 0.95891E+00_JPRB, 0.97396E+00_JPRB ,&
&  0.96430E+00_JPRB, 0.98043E+00_JPRB, 0.11066E+01_JPRB, 0.11555E+01_JPRB, 0.11485E+01_JPRB, 0.97032E+00_JPRB, 0.80418E+00_JPRB ,&
&  0.78520E+00_JPRB, 0.75779E+00_JPRB, 0.73190E+00_JPRB, 0.72301E+00_JPRB, 0.71399E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.65575E+00_JPRB, 0.63789E+00_JPRB, 0.62351E+00_JPRB, 0.62895E+00_JPRB, 0.63841E+00_JPRB, 0.65773E+00_JPRB, 0.70801E+00_JPRB ,&
&  0.74836E+00_JPRB, 0.76465E+00_JPRB, 0.78611E+00_JPRB, 0.78800E+00_JPRB, 0.79877E+00_JPRB, 0.78613E+00_JPRB, 0.78563E+00_JPRB ,&
&  0.77515E+00_JPRB, 0.78449E+00_JPRB, 0.80235E+00_JPRB, 0.81980E+00_JPRB, 0.83307E+00_JPRB, 0.84622E+00_JPRB, 0.86543E+00_JPRB ,&
&  0.89097E+00_JPRB, 0.92879E+00_JPRB, 0.99108E+00_JPRB, 0.10849E+01_JPRB, 0.12291E+01_JPRB, 0.13975E+01_JPRB, 0.15007E+01_JPRB ,&
&  0.11255E+01_JPRB, 0.53375E+00_JPRB, 0.10228E+01_JPRB, 0.15744E+01_JPRB, 0.16588E+01_JPRB, 0.17319E+01_JPRB, 0.16958E+01_JPRB ,&
&  0.15800E+01_JPRB, 0.15032E+01_JPRB, 0.14790E+01_JPRB, 0.17129E+01_JPRB, 0.17415E+01_JPRB, 0.17493E+01_JPRB, 0.17458E+01_JPRB ,&
&  0.17335E+01_JPRB, 0.17483E+01_JPRB, 0.17809E+01_JPRB, 0.18545E+01_JPRB, 0.19975E+01_JPRB, 0.22247E+01_JPRB, 0.25224E+01_JPRB ,&
&  0.29244E+01_JPRB, 0.34733E+01_JPRB, 0.40118E+01_JPRB, 0.37212E+01_JPRB, 0.38216E+01_JPRB, 0.33913E+01_JPRB, 0.29290E+01_JPRB ,&
&  0.18304E+01_JPRB, 0.13379E+01_JPRB, 0.13537E+01_JPRB, 0.14920E+01_JPRB, 0.13530E+01_JPRB, 0.11688E+01_JPRB, 0.12238E+01_JPRB ,&
&  0.11465E+01_JPRB, 0.10888E+01_JPRB, 0.10543E+01_JPRB, 0.10179E+01_JPRB, 0.10024E+01_JPRB, 0.99715E+00_JPRB, 0.99763E+00_JPRB ,&
&  0.99207E+00_JPRB, 0.99834E+00_JPRB, 0.10085E+01_JPRB, 0.10032E+01_JPRB, 0.97909E+00_JPRB, 0.96927E+00_JPRB, 0.97898E+00_JPRB ,&
&  0.99003E+00_JPRB, 0.98264E+00_JPRB, 0.99734E+00_JPRB, 0.10152E+01_JPRB, 0.10318E+01_JPRB, 0.10086E+01_JPRB, 0.10070E+01_JPRB ,&
&  0.99437E+00_JPRB, 0.99199E+00_JPRB, 0.98675E+00_JPRB, 0.96961E+00_JPRB, 0.95814E+00_JPRB, 0.89062E+00_JPRB, 0.80644E+00_JPRB ,&
&  0.77873E+00_JPRB, 0.74731E+00_JPRB, 0.72354E+00_JPRB, 0.70318E+00_JPRB, 0.68112E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.69849E+00_JPRB, 0.68154E+00_JPRB, 0.65072E+00_JPRB, 0.61044E+00_JPRB, 0.60473E+00_JPRB, 0.60170E+00_JPRB, 0.61404E+00_JPRB ,&
&  0.65521E+00_JPRB, 0.70752E+00_JPRB, 0.71604E+00_JPRB, 0.71217E+00_JPRB, 0.69292E+00_JPRB, 0.69096E+00_JPRB, 0.69276E+00_JPRB ,&
&  0.70090E+00_JPRB, 0.70586E+00_JPRB, 0.72279E+00_JPRB, 0.74584E+00_JPRB, 0.77041E+00_JPRB, 0.78886E+00_JPRB, 0.79240E+00_JPRB ,&
&  0.79779E+00_JPRB, 0.80064E+00_JPRB, 0.81913E+00_JPRB, 0.84882E+00_JPRB, 0.90990E+00_JPRB, 0.10288E+01_JPRB, 0.12193E+01_JPRB ,&
&  0.16826E+01_JPRB, 0.13641E+01_JPRB, 0.70134E+00_JPRB, 0.13001E+01_JPRB, 0.15206E+01_JPRB, 0.16464E+01_JPRB, 0.14400E+01_JPRB ,&
&  0.13895E+01_JPRB, 0.12680E+01_JPRB, 0.12256E+01_JPRB, 0.13740E+01_JPRB, 0.14170E+01_JPRB, 0.14649E+01_JPRB, 0.14766E+01_JPRB ,&
&  0.14344E+01_JPRB, 0.14485E+01_JPRB, 0.14720E+01_JPRB, 0.15145E+01_JPRB, 0.15913E+01_JPRB, 0.17608E+01_JPRB, 0.20636E+01_JPRB ,&
&  0.24441E+01_JPRB, 0.29525E+01_JPRB, 0.37083E+01_JPRB, 0.40392E+01_JPRB, 0.43953E+01_JPRB, 0.33028E+01_JPRB, 0.25699E+01_JPRB ,&
&  0.17199E+01_JPRB, 0.15695E+01_JPRB, 0.16379E+01_JPRB, 0.16291E+01_JPRB, 0.14759E+01_JPRB, 0.13213E+01_JPRB, 0.13714E+01_JPRB ,&
&  0.12652E+01_JPRB, 0.11889E+01_JPRB, 0.11501E+01_JPRB, 0.11085E+01_JPRB, 0.10771E+01_JPRB, 0.10585E+01_JPRB, 0.10457E+01_JPRB ,&
&  0.10323E+01_JPRB, 0.10373E+01_JPRB, 0.10440E+01_JPRB, 0.10499E+01_JPRB, 0.10508E+01_JPRB, 0.10419E+01_JPRB, 0.10518E+01_JPRB ,&
&  0.10591E+01_JPRB, 0.10641E+01_JPRB, 0.10722E+01_JPRB, 0.10879E+01_JPRB, 0.10597E+01_JPRB, 0.10133E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10127E+01_JPRB, 0.10229E+01_JPRB, 0.10064E+01_JPRB, 0.10189E+01_JPRB, 0.97269E+00_JPRB, 0.90223E+00_JPRB, 0.83310E+00_JPRB ,&
&  0.82782E+00_JPRB, 0.82010E+00_JPRB, 0.77886E+00_JPRB, 0.72267E+00_JPRB, 0.70158E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.75600E+00_JPRB, 0.76095E+00_JPRB, 0.75246E+00_JPRB, 0.71059E+00_JPRB, 0.67086E+00_JPRB, 0.63239E+00_JPRB, 0.60486E+00_JPRB ,&
&  0.60001E+00_JPRB, 0.60772E+00_JPRB, 0.63675E+00_JPRB, 0.65850E+00_JPRB, 0.65006E+00_JPRB, 0.62965E+00_JPRB, 0.62218E+00_JPRB ,&
&  0.62734E+00_JPRB, 0.64467E+00_JPRB, 0.65676E+00_JPRB, 0.67791E+00_JPRB, 0.70004E+00_JPRB, 0.71862E+00_JPRB, 0.73894E+00_JPRB ,&
&  0.72741E+00_JPRB, 0.72909E+00_JPRB, 0.74006E+00_JPRB, 0.74941E+00_JPRB, 0.76844E+00_JPRB, 0.81196E+00_JPRB, 0.85382E+00_JPRB ,&
&  0.10135E+01_JPRB, 0.15830E+01_JPRB, 0.97056E+00_JPRB, 0.13387E+01_JPRB, 0.15599E+01_JPRB, 0.15771E+01_JPRB, 0.14242E+01_JPRB ,&
&  0.12909E+01_JPRB, 0.11306E+01_JPRB, 0.11253E+01_JPRB, 0.11553E+01_JPRB, 0.10826E+01_JPRB, 0.11038E+01_JPRB, 0.11292E+01_JPRB ,&
&  0.11365E+01_JPRB, 0.11700E+01_JPRB, 0.11979E+01_JPRB, 0.12272E+01_JPRB, 0.12483E+01_JPRB, 0.13101E+01_JPRB, 0.14897E+01_JPRB ,&
&  0.18301E+01_JPRB, 0.23034E+01_JPRB, 0.31078E+01_JPRB, 0.38224E+01_JPRB, 0.46555E+01_JPRB, 0.36371E+01_JPRB, 0.27049E+01_JPRB ,&
&  0.19989E+01_JPRB, 0.19089E+01_JPRB, 0.19172E+01_JPRB, 0.18834E+01_JPRB, 0.16837E+01_JPRB, 0.16385E+01_JPRB, 0.16684E+01_JPRB ,&
&  0.14917E+01_JPRB, 0.13482E+01_JPRB, 0.12775E+01_JPRB, 0.12268E+01_JPRB, 0.11725E+01_JPRB, 0.11328E+01_JPRB, 0.11071E+01_JPRB ,&
&  0.10861E+01_JPRB, 0.10801E+01_JPRB, 0.10922E+01_JPRB, 0.11001E+01_JPRB, 0.11104E+01_JPRB, 0.11086E+01_JPRB, 0.11038E+01_JPRB ,&
&  0.10879E+01_JPRB, 0.10805E+01_JPRB, 0.10805E+01_JPRB, 0.10751E+01_JPRB, 0.10503E+01_JPRB, 0.10172E+01_JPRB, 0.10112E+01_JPRB ,&
&  0.10136E+01_JPRB, 0.10371E+01_JPRB, 0.10323E+01_JPRB, 0.10083E+01_JPRB, 0.10399E+01_JPRB, 0.98597E+00_JPRB, 0.92076E+00_JPRB ,&
&  0.95172E+00_JPRB, 0.10959E+01_JPRB, 0.11454E+01_JPRB, 0.79306E+00_JPRB, 0.76378E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.77834E+00_JPRB, 0.77589E+00_JPRB, 0.76035E+00_JPRB, 0.75497E+00_JPRB, 0.72081E+00_JPRB, 0.68417E+00_JPRB, 0.65137E+00_JPRB ,&
&  0.62996E+00_JPRB, 0.61883E+00_JPRB, 0.62341E+00_JPRB, 0.62388E+00_JPRB, 0.62450E+00_JPRB, 0.61661E+00_JPRB, 0.61140E+00_JPRB ,&
&  0.59847E+00_JPRB, 0.60249E+00_JPRB, 0.61673E+00_JPRB, 0.64028E+00_JPRB, 0.66781E+00_JPRB, 0.67745E+00_JPRB, 0.68748E+00_JPRB ,&
&  0.68847E+00_JPRB, 0.68432E+00_JPRB, 0.68448E+00_JPRB, 0.69237E+00_JPRB, 0.70926E+00_JPRB, 0.72858E+00_JPRB, 0.75455E+00_JPRB ,&
&  0.80350E+00_JPRB, 0.13845E+01_JPRB, 0.20797E+01_JPRB, 0.23587E+01_JPRB, 0.20362E+01_JPRB, 0.16895E+01_JPRB, 0.15009E+01_JPRB ,&
&  0.13912E+01_JPRB, 0.12643E+01_JPRB, 0.12892E+01_JPRB, 0.11368E+01_JPRB, 0.98175E+00_JPRB, 0.92636E+00_JPRB, 0.94074E+00_JPRB ,&
&  0.99690E+00_JPRB, 0.10183E+01_JPRB, 0.10307E+01_JPRB, 0.10329E+01_JPRB, 0.10148E+01_JPRB, 0.10174E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.12229E+01_JPRB, 0.15531E+01_JPRB, 0.21492E+01_JPRB, 0.31794E+01_JPRB, 0.42928E+01_JPRB, 0.38555E+01_JPRB, 0.31276E+01_JPRB ,&
&  0.28882E+01_JPRB, 0.26393E+01_JPRB, 0.24020E+01_JPRB, 0.22653E+01_JPRB, 0.18336E+01_JPRB, 0.21729E+01_JPRB, 0.20407E+01_JPRB ,&
&  0.18818E+01_JPRB, 0.17693E+01_JPRB, 0.16128E+01_JPRB, 0.14527E+01_JPRB, 0.13338E+01_JPRB, 0.12263E+01_JPRB, 0.11478E+01_JPRB ,&
&  0.11222E+01_JPRB, 0.11276E+01_JPRB, 0.11419E+01_JPRB, 0.11670E+01_JPRB, 0.11945E+01_JPRB, 0.12037E+01_JPRB, 0.11831E+01_JPRB ,&
&  0.11289E+01_JPRB, 0.10833E+01_JPRB, 0.10146E+01_JPRB, 0.10027E+01_JPRB, 0.99223E+00_JPRB, 0.96114E+00_JPRB, 0.94235E+00_JPRB ,&
&  0.96125E+00_JPRB, 0.10034E+01_JPRB, 0.10197E+01_JPRB, 0.10128E+01_JPRB, 0.10042E+01_JPRB, 0.10063E+01_JPRB, 0.93612E+00_JPRB ,&
&  0.90711E+00_JPRB, 0.95726E+00_JPRB, 0.91464E+00_JPRB, 0.83747E+00_JPRB, 0.80415E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.79715E+00_JPRB, 0.77057E+00_JPRB, 0.75334E+00_JPRB, 0.75443E+00_JPRB, 0.73247E+00_JPRB, 0.71711E+00_JPRB, 0.71057E+00_JPRB ,&
&  0.67325E+00_JPRB, 0.64654E+00_JPRB, 0.62593E+00_JPRB, 0.62060E+00_JPRB, 0.62194E+00_JPRB, 0.61546E+00_JPRB, 0.61726E+00_JPRB ,&
&  0.60835E+00_JPRB, 0.59755E+00_JPRB, 0.59812E+00_JPRB, 0.61750E+00_JPRB, 0.64518E+00_JPRB, 0.65314E+00_JPRB, 0.66754E+00_JPRB ,&
&  0.68198E+00_JPRB, 0.67735E+00_JPRB, 0.66672E+00_JPRB, 0.66469E+00_JPRB, 0.66829E+00_JPRB, 0.66449E+00_JPRB, 0.68000E+00_JPRB ,&
&  0.73344E+00_JPRB, 0.14327E+01_JPRB, 0.15586E+01_JPRB, 0.26054E+01_JPRB, 0.22572E+01_JPRB, 0.19042E+01_JPRB, 0.14734E+01_JPRB ,&
&  0.14259E+01_JPRB, 0.16022E+01_JPRB, 0.14007E+01_JPRB, 0.12289E+01_JPRB, 0.10494E+01_JPRB, 0.91367E+00_JPRB, 0.88179E+00_JPRB ,&
&  0.91267E+00_JPRB, 0.94399E+00_JPRB, 0.95986E+00_JPRB, 0.92821E+00_JPRB, 0.89114E+00_JPRB, 0.87943E+00_JPRB, 0.89329E+00_JPRB ,&
&  0.93461E+00_JPRB, 0.10444E+01_JPRB, 0.13520E+01_JPRB, 0.19758E+01_JPRB, 0.33529E+01_JPRB, 0.39978E+01_JPRB, 0.36481E+01_JPRB ,&
&  0.35024E+01_JPRB, 0.36240E+01_JPRB, 0.32805E+01_JPRB, 0.28992E+01_JPRB, 0.24163E+01_JPRB, 0.24006E+01_JPRB, 0.22670E+01_JPRB ,&
&  0.21273E+01_JPRB, 0.20827E+01_JPRB, 0.20037E+01_JPRB, 0.18491E+01_JPRB, 0.16673E+01_JPRB, 0.14801E+01_JPRB, 0.13084E+01_JPRB ,&
&  0.11918E+01_JPRB, 0.11401E+01_JPRB, 0.11200E+01_JPRB, 0.11509E+01_JPRB, 0.12098E+01_JPRB, 0.12502E+01_JPRB, 0.12585E+01_JPRB ,&
&  0.12134E+01_JPRB, 0.11447E+01_JPRB, 0.10169E+01_JPRB, 0.95142E+00_JPRB, 0.94346E+00_JPRB, 0.91425E+00_JPRB, 0.90043E+00_JPRB ,&
&  0.93719E+00_JPRB, 0.97168E+00_JPRB, 0.98156E+00_JPRB, 0.97043E+00_JPRB, 0.97284E+00_JPRB, 0.10135E+01_JPRB, 0.95797E+00_JPRB ,&
&  0.92026E+00_JPRB, 0.93808E+00_JPRB, 0.94405E+00_JPRB, 0.88020E+00_JPRB, 0.84053E+00_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.83858E+00_JPRB, 0.79591E+00_JPRB, 0.76470E+00_JPRB, 0.75140E+00_JPRB, 0.74338E+00_JPRB, 0.73174E+00_JPRB, 0.72126E+00_JPRB ,&
&  0.69936E+00_JPRB, 0.67263E+00_JPRB, 0.65188E+00_JPRB, 0.63424E+00_JPRB, 0.63351E+00_JPRB, 0.62740E+00_JPRB, 0.61915E+00_JPRB ,&
&  0.60735E+00_JPRB, 0.60047E+00_JPRB, 0.59800E+00_JPRB, 0.60545E+00_JPRB, 0.62255E+00_JPRB, 0.63376E+00_JPRB, 0.64785E+00_JPRB ,&
&  0.65488E+00_JPRB, 0.66363E+00_JPRB, 0.66427E+00_JPRB, 0.65913E+00_JPRB, 0.65058E+00_JPRB, 0.63803E+00_JPRB, 0.62293E+00_JPRB ,&
&  0.71021E+00_JPRB, 0.15979E+01_JPRB, 0.13509E+01_JPRB, 0.23437E+01_JPRB, 0.20490E+01_JPRB, 0.18418E+01_JPRB, 0.14562E+01_JPRB ,&
&  0.13677E+01_JPRB, 0.14913E+01_JPRB, 0.13870E+01_JPRB, 0.12671E+01_JPRB, 0.11513E+01_JPRB, 0.10321E+01_JPRB, 0.95271E+00_JPRB ,&
&  0.92449E+00_JPRB, 0.91892E+00_JPRB, 0.90612E+00_JPRB, 0.87746E+00_JPRB, 0.85928E+00_JPRB, 0.84246E+00_JPRB, 0.84311E+00_JPRB ,&
&  0.85480E+00_JPRB, 0.88723E+00_JPRB, 0.10036E+01_JPRB, 0.14024E+01_JPRB, 0.22493E+01_JPRB, 0.33857E+01_JPRB, 0.41564E+01_JPRB ,&
&  0.43060E+01_JPRB, 0.46899E+01_JPRB, 0.41838E+01_JPRB, 0.35738E+01_JPRB, 0.31019E+01_JPRB, 0.28148E+01_JPRB, 0.25632E+01_JPRB ,&
&  0.23094E+01_JPRB, 0.21575E+01_JPRB, 0.21091E+01_JPRB, 0.20733E+01_JPRB, 0.19970E+01_JPRB, 0.18563E+01_JPRB, 0.16727E+01_JPRB ,&
&  0.14815E+01_JPRB, 0.13481E+01_JPRB, 0.12561E+01_JPRB, 0.12170E+01_JPRB, 0.12195E+01_JPRB, 0.12451E+01_JPRB, 0.12581E+01_JPRB ,&
&  0.12170E+01_JPRB, 0.11653E+01_JPRB, 0.10568E+01_JPRB, 0.97008E+00_JPRB, 0.95154E+00_JPRB, 0.92284E+00_JPRB, 0.90027E+00_JPRB ,&
&  0.93610E+00_JPRB, 0.96010E+00_JPRB, 0.95452E+00_JPRB, 0.95067E+00_JPRB, 0.93775E+00_JPRB, 0.10097E+01_JPRB, 0.95348E+00_JPRB ,&
&  0.92195E+00_JPRB, 0.91750E+00_JPRB, 0.91807E+00_JPRB, 0.89677E+00_JPRB, 0.87781E+00_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.87220E+00_JPRB, 0.83990E+00_JPRB, 0.80822E+00_JPRB, 0.78535E+00_JPRB, 0.76600E+00_JPRB, 0.75353E+00_JPRB, 0.74091E+00_JPRB ,&
&  0.71631E+00_JPRB, 0.69180E+00_JPRB, 0.67866E+00_JPRB, 0.66830E+00_JPRB, 0.65926E+00_JPRB, 0.65244E+00_JPRB, 0.63591E+00_JPRB ,&
&  0.61834E+00_JPRB, 0.59836E+00_JPRB, 0.60577E+00_JPRB, 0.61713E+00_JPRB, 0.62701E+00_JPRB, 0.63547E+00_JPRB, 0.65337E+00_JPRB ,&
&  0.64826E+00_JPRB, 0.64756E+00_JPRB, 0.64963E+00_JPRB, 0.65049E+00_JPRB, 0.64033E+00_JPRB, 0.63974E+00_JPRB, 0.61310E+00_JPRB ,&
&  0.68408E+00_JPRB, 0.75422E+00_JPRB, 0.14554E+01_JPRB, 0.19955E+01_JPRB, 0.18435E+01_JPRB, 0.16592E+01_JPRB, 0.14932E+01_JPRB ,&
&  0.14729E+01_JPRB, 0.13974E+01_JPRB, 0.12734E+01_JPRB, 0.11956E+01_JPRB, 0.11221E+01_JPRB, 0.10625E+01_JPRB, 0.10146E+01_JPRB ,&
&  0.97876E+00_JPRB, 0.93390E+00_JPRB, 0.89862E+00_JPRB, 0.86166E+00_JPRB, 0.83963E+00_JPRB, 0.82529E+00_JPRB, 0.82297E+00_JPRB ,&
&  0.82365E+00_JPRB, 0.82644E+00_JPRB, 0.86072E+00_JPRB, 0.10642E+01_JPRB, 0.17748E+01_JPRB, 0.27424E+01_JPRB, 0.38570E+01_JPRB ,&
&  0.47494E+01_JPRB, 0.50597E+01_JPRB, 0.45985E+01_JPRB, 0.40417E+01_JPRB, 0.35111E+01_JPRB, 0.31096E+01_JPRB, 0.28008E+01_JPRB ,&
&  0.25201E+01_JPRB, 0.22805E+01_JPRB, 0.20967E+01_JPRB, 0.20001E+01_JPRB, 0.19418E+01_JPRB, 0.18946E+01_JPRB, 0.18310E+01_JPRB ,&
&  0.17367E+01_JPRB, 0.16065E+01_JPRB, 0.14744E+01_JPRB, 0.13667E+01_JPRB, 0.12974E+01_JPRB, 0.12718E+01_JPRB, 0.12519E+01_JPRB ,&
&  0.11956E+01_JPRB, 0.11427E+01_JPRB, 0.10654E+01_JPRB, 0.10101E+01_JPRB, 0.10079E+01_JPRB, 0.99148E+00_JPRB, 0.97276E+00_JPRB ,&
&  0.96744E+00_JPRB, 0.97320E+00_JPRB, 0.95993E+00_JPRB, 0.95966E+00_JPRB, 0.92863E+00_JPRB, 0.99912E+00_JPRB, 0.96462E+00_JPRB ,&
&  0.92909E+00_JPRB, 0.92020E+00_JPRB, 0.90345E+00_JPRB, 0.89333E+00_JPRB, 0.88967E+00_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.90168E+00_JPRB, 0.88667E+00_JPRB, 0.84983E+00_JPRB, 0.81400E+00_JPRB, 0.78902E+00_JPRB, 0.77149E+00_JPRB, 0.75616E+00_JPRB ,&
&  0.73636E+00_JPRB, 0.71048E+00_JPRB, 0.69186E+00_JPRB, 0.68466E+00_JPRB, 0.68393E+00_JPRB, 0.68047E+00_JPRB, 0.66350E+00_JPRB ,&
&  0.64168E+00_JPRB, 0.62490E+00_JPRB, 0.62625E+00_JPRB, 0.63296E+00_JPRB, 0.64315E+00_JPRB, 0.63740E+00_JPRB, 0.64718E+00_JPRB ,&
&  0.65213E+00_JPRB, 0.64859E+00_JPRB, 0.64551E+00_JPRB, 0.64293E+00_JPRB, 0.63794E+00_JPRB, 0.63388E+00_JPRB, 0.61442E+00_JPRB ,&
&  0.65074E+00_JPRB, 0.85435E+00_JPRB, 0.13293E+01_JPRB, 0.16879E+01_JPRB, 0.16910E+01_JPRB, 0.15134E+01_JPRB, 0.14065E+01_JPRB ,&
&  0.13191E+01_JPRB, 0.12565E+01_JPRB, 0.11537E+01_JPRB, 0.10861E+01_JPRB, 0.10272E+01_JPRB, 0.98532E+00_JPRB, 0.95466E+00_JPRB ,&
&  0.93531E+00_JPRB, 0.91402E+00_JPRB, 0.88341E+00_JPRB, 0.85384E+00_JPRB, 0.83213E+00_JPRB, 0.81440E+00_JPRB, 0.80672E+00_JPRB ,&
&  0.80587E+00_JPRB, 0.80189E+00_JPRB, 0.80873E+00_JPRB, 0.89201E+00_JPRB, 0.14045E+01_JPRB, 0.17934E+01_JPRB, 0.30817E+01_JPRB ,&
&  0.41374E+01_JPRB, 0.39026E+01_JPRB, 0.37139E+01_JPRB, 0.34088E+01_JPRB, 0.30334E+01_JPRB, 0.27260E+01_JPRB, 0.24116E+01_JPRB ,&
&  0.21414E+01_JPRB, 0.19280E+01_JPRB, 0.17542E+01_JPRB, 0.16364E+01_JPRB, 0.15628E+01_JPRB, 0.15247E+01_JPRB, 0.15115E+01_JPRB ,&
&  0.15002E+01_JPRB, 0.14857E+01_JPRB, 0.14575E+01_JPRB, 0.14115E+01_JPRB, 0.13433E+01_JPRB, 0.12732E+01_JPRB, 0.12059E+01_JPRB ,&
&  0.11352E+01_JPRB, 0.10921E+01_JPRB, 0.10440E+01_JPRB, 0.10150E+01_JPRB, 0.10385E+01_JPRB, 0.10683E+01_JPRB, 0.10499E+01_JPRB ,&
&  0.10038E+01_JPRB, 0.98786E+00_JPRB, 0.98594E+00_JPRB, 0.95512E+00_JPRB, 0.92369E+00_JPRB, 0.10032E+01_JPRB, 0.98401E+00_JPRB ,&
&  0.95190E+00_JPRB, 0.93170E+00_JPRB, 0.91458E+00_JPRB, 0.89467E+00_JPRB, 0.90563E+00_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.92854E+00_JPRB, 0.89613E+00_JPRB, 0.86505E+00_JPRB, 0.83050E+00_JPRB, 0.80633E+00_JPRB, 0.78035E+00_JPRB, 0.75519E+00_JPRB ,&
&  0.73364E+00_JPRB, 0.72128E+00_JPRB, 0.71025E+00_JPRB, 0.69753E+00_JPRB, 0.69762E+00_JPRB, 0.69191E+00_JPRB, 0.67765E+00_JPRB ,&
&  0.65617E+00_JPRB, 0.64003E+00_JPRB, 0.63394E+00_JPRB, 0.63466E+00_JPRB, 0.64424E+00_JPRB, 0.64977E+00_JPRB, 0.65708E+00_JPRB ,&
&  0.66129E+00_JPRB, 0.64210E+00_JPRB, 0.63166E+00_JPRB, 0.63279E+00_JPRB, 0.63311E+00_JPRB, 0.63579E+00_JPRB, 0.62114E+00_JPRB ,&
&  0.63702E+00_JPRB, 0.74171E+00_JPRB, 0.10187E+01_JPRB, 0.13821E+01_JPRB, 0.14312E+01_JPRB, 0.13525E+01_JPRB, 0.12711E+01_JPRB ,&
&  0.11979E+01_JPRB, 0.11330E+01_JPRB, 0.10491E+01_JPRB, 0.95776E+00_JPRB, 0.88831E+00_JPRB, 0.84765E+00_JPRB, 0.82695E+00_JPRB ,&
&  0.82281E+00_JPRB, 0.82002E+00_JPRB, 0.80990E+00_JPRB, 0.80124E+00_JPRB, 0.79077E+00_JPRB, 0.78164E+00_JPRB, 0.77782E+00_JPRB ,&
&  0.77304E+00_JPRB, 0.76663E+00_JPRB, 0.76391E+00_JPRB, 0.78270E+00_JPRB, 0.10165E+01_JPRB, 0.12604E+01_JPRB, 0.18317E+01_JPRB ,&
&  0.20795E+01_JPRB, 0.21804E+01_JPRB, 0.22757E+01_JPRB, 0.22489E+01_JPRB, 0.21206E+01_JPRB, 0.19916E+01_JPRB, 0.18590E+01_JPRB ,&
&  0.17135E+01_JPRB, 0.15629E+01_JPRB, 0.14586E+01_JPRB, 0.13821E+01_JPRB, 0.13107E+01_JPRB, 0.12575E+01_JPRB, 0.12243E+01_JPRB ,&
&  0.12185E+01_JPRB, 0.12390E+01_JPRB, 0.12603E+01_JPRB, 0.12731E+01_JPRB, 0.12559E+01_JPRB, 0.12208E+01_JPRB, 0.11692E+01_JPRB ,&
&  0.10987E+01_JPRB, 0.10435E+01_JPRB, 0.10196E+01_JPRB, 0.10208E+01_JPRB, 0.10332E+01_JPRB, 0.10378E+01_JPRB, 0.10435E+01_JPRB ,&
&  0.10455E+01_JPRB, 0.10307E+01_JPRB, 0.10123E+01_JPRB, 0.98676E+00_JPRB, 0.95771E+00_JPRB, 0.10194E+01_JPRB, 0.10062E+01_JPRB ,&
&  0.97965E+00_JPRB, 0.95224E+00_JPRB, 0.93389E+00_JPRB, 0.91171E+00_JPRB, 0.94913E+00_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.10010E+01_JPRB, 0.94167E+00_JPRB, 0.89386E+00_JPRB, 0.85936E+00_JPRB, 0.83705E+00_JPRB, 0.80749E+00_JPRB, 0.77147E+00_JPRB ,&
&  0.74481E+00_JPRB, 0.73530E+00_JPRB, 0.72498E+00_JPRB, 0.70756E+00_JPRB, 0.69512E+00_JPRB, 0.68189E+00_JPRB, 0.66997E+00_JPRB ,&
&  0.65250E+00_JPRB, 0.64510E+00_JPRB, 0.63829E+00_JPRB, 0.63942E+00_JPRB, 0.63800E+00_JPRB, 0.64095E+00_JPRB, 0.65114E+00_JPRB ,&
&  0.65363E+00_JPRB, 0.63447E+00_JPRB, 0.63165E+00_JPRB, 0.63229E+00_JPRB, 0.63174E+00_JPRB, 0.63165E+00_JPRB, 0.62113E+00_JPRB ,&
&  0.64313E+00_JPRB, 0.66533E+00_JPRB, 0.83719E+00_JPRB, 0.10030E+01_JPRB, 0.10826E+01_JPRB, 0.11332E+01_JPRB, 0.11167E+01_JPRB ,&
&  0.10804E+01_JPRB, 0.10210E+01_JPRB, 0.95415E+00_JPRB, 0.88464E+00_JPRB, 0.82137E+00_JPRB, 0.78033E+00_JPRB, 0.75845E+00_JPRB ,&
&  0.74693E+00_JPRB, 0.74375E+00_JPRB, 0.73908E+00_JPRB, 0.73658E+00_JPRB, 0.73381E+00_JPRB, 0.72844E+00_JPRB, 0.72957E+00_JPRB ,&
&  0.73567E+00_JPRB, 0.72597E+00_JPRB, 0.72230E+00_JPRB, 0.73661E+00_JPRB, 0.78038E+00_JPRB, 0.87641E+00_JPRB, 0.11342E+01_JPRB ,&
&  0.12974E+01_JPRB, 0.14177E+01_JPRB, 0.15469E+01_JPRB, 0.15958E+01_JPRB, 0.15607E+01_JPRB, 0.14959E+01_JPRB, 0.14027E+01_JPRB ,&
&  0.13305E+01_JPRB, 0.12771E+01_JPRB, 0.12274E+01_JPRB, 0.11770E+01_JPRB, 0.11325E+01_JPRB, 0.10998E+01_JPRB, 0.10776E+01_JPRB ,&
&  0.10619E+01_JPRB, 0.10680E+01_JPRB, 0.10828E+01_JPRB, 0.10966E+01_JPRB, 0.11003E+01_JPRB, 0.10963E+01_JPRB, 0.10838E+01_JPRB ,&
&  0.10485E+01_JPRB, 0.99989E+00_JPRB, 0.97177E+00_JPRB, 0.97609E+00_JPRB, 0.10026E+01_JPRB, 0.10155E+01_JPRB, 0.10204E+01_JPRB ,&
&  0.10300E+01_JPRB, 0.10339E+01_JPRB, 0.10035E+01_JPRB, 0.98829E+00_JPRB, 0.10460E+01_JPRB, 0.10292E+01_JPRB, 0.10060E+01_JPRB ,&
&  0.97821E+00_JPRB, 0.96107E+00_JPRB, 0.93887E+00_JPRB, 0.91181E+00_JPRB, 0.98549E+00_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.85600E+00_JPRB, 0.85660E+00_JPRB, 0.83812E+00_JPRB, 0.82067E+00_JPRB, 0.81169E+00_JPRB, 0.80230E+00_JPRB, 0.78323E+00_JPRB ,&
&  0.76124E+00_JPRB, 0.74082E+00_JPRB, 0.72830E+00_JPRB, 0.71086E+00_JPRB, 0.69348E+00_JPRB, 0.68268E+00_JPRB, 0.67117E+00_JPRB ,&
&  0.65426E+00_JPRB, 0.64831E+00_JPRB, 0.64678E+00_JPRB, 0.64353E+00_JPRB, 0.63778E+00_JPRB, 0.63402E+00_JPRB, 0.63994E+00_JPRB ,&
&  0.64046E+00_JPRB, 0.63932E+00_JPRB, 0.64022E+00_JPRB, 0.63598E+00_JPRB, 0.62978E+00_JPRB, 0.62788E+00_JPRB, 0.61652E+00_JPRB ,&
&  0.62227E+00_JPRB, 0.58951E+00_JPRB, 0.67496E+00_JPRB, 0.77771E+00_JPRB, 0.86261E+00_JPRB, 0.92164E+00_JPRB, 0.97190E+00_JPRB ,&
&  0.97609E+00_JPRB, 0.93199E+00_JPRB, 0.88134E+00_JPRB, 0.84496E+00_JPRB, 0.80475E+00_JPRB, 0.77289E+00_JPRB, 0.74702E+00_JPRB ,&
&  0.73018E+00_JPRB, 0.72192E+00_JPRB, 0.71589E+00_JPRB, 0.71026E+00_JPRB, 0.70434E+00_JPRB, 0.69954E+00_JPRB, 0.69840E+00_JPRB ,&
&  0.69602E+00_JPRB, 0.69733E+00_JPRB, 0.70028E+00_JPRB, 0.69973E+00_JPRB, 0.71129E+00_JPRB, 0.76742E+00_JPRB, 0.88284E+00_JPRB ,&
&  0.96032E+00_JPRB, 0.10456E+01_JPRB, 0.11581E+01_JPRB, 0.12198E+01_JPRB, 0.12067E+01_JPRB, 0.11743E+01_JPRB, 0.11287E+01_JPRB ,&
&  0.10905E+01_JPRB, 0.10463E+01_JPRB, 0.10200E+01_JPRB, 0.99230E+00_JPRB, 0.96207E+00_JPRB, 0.93786E+00_JPRB, 0.92468E+00_JPRB ,&
&  0.92042E+00_JPRB, 0.91991E+00_JPRB, 0.92921E+00_JPRB, 0.94243E+00_JPRB, 0.94789E+00_JPRB, 0.95020E+00_JPRB, 0.95600E+00_JPRB ,&
&  0.96078E+00_JPRB, 0.94801E+00_JPRB, 0.92485E+00_JPRB, 0.92851E+00_JPRB, 0.95301E+00_JPRB, 0.97168E+00_JPRB, 0.98279E+00_JPRB ,&
&  0.99273E+00_JPRB, 0.10026E+01_JPRB, 0.10090E+01_JPRB, 0.10248E+01_JPRB, 0.10150E+01_JPRB, 0.98551E+00_JPRB, 0.95872E+00_JPRB ,&
&  0.93595E+00_JPRB, 0.91404E+00_JPRB, 0.89785E+00_JPRB, 0.87772E+00_JPRB, 0.80701E+00_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.80477E+00_JPRB, 0.80029E+00_JPRB, 0.78644E+00_JPRB, 0.77147E+00_JPRB, 0.76477E+00_JPRB, 0.76274E+00_JPRB, 0.76264E+00_JPRB ,&
&  0.75721E+00_JPRB, 0.74821E+00_JPRB, 0.73740E+00_JPRB, 0.72385E+00_JPRB, 0.70913E+00_JPRB, 0.69672E+00_JPRB, 0.68498E+00_JPRB ,&
&  0.66615E+00_JPRB, 0.65520E+00_JPRB, 0.65454E+00_JPRB, 0.65183E+00_JPRB, 0.64892E+00_JPRB, 0.64409E+00_JPRB, 0.64379E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64898E+00_JPRB, 0.65031E+00_JPRB, 0.64413E+00_JPRB, 0.63858E+00_JPRB, 0.63772E+00_JPRB, 0.63305E+00_JPRB ,&
&  0.63244E+00_JPRB, 0.53135E+00_JPRB, 0.59337E+00_JPRB, 0.67959E+00_JPRB, 0.72226E+00_JPRB, 0.78430E+00_JPRB, 0.86450E+00_JPRB ,&
&  0.90369E+00_JPRB, 0.88773E+00_JPRB, 0.85448E+00_JPRB, 0.82412E+00_JPRB, 0.79032E+00_JPRB, 0.75971E+00_JPRB, 0.73430E+00_JPRB ,&
&  0.71843E+00_JPRB, 0.70940E+00_JPRB, 0.70253E+00_JPRB, 0.69381E+00_JPRB, 0.68465E+00_JPRB, 0.68115E+00_JPRB, 0.67942E+00_JPRB ,&
&  0.68043E+00_JPRB, 0.68526E+00_JPRB, 0.68834E+00_JPRB, 0.69632E+00_JPRB, 0.70393E+00_JPRB, 0.72591E+00_JPRB, 0.78108E+00_JPRB ,&
&  0.81996E+00_JPRB, 0.86745E+00_JPRB, 0.93561E+00_JPRB, 0.96520E+00_JPRB, 0.95990E+00_JPRB, 0.94847E+00_JPRB, 0.92946E+00_JPRB ,&
&  0.90465E+00_JPRB, 0.86861E+00_JPRB, 0.84904E+00_JPRB, 0.83012E+00_JPRB, 0.81516E+00_JPRB, 0.80756E+00_JPRB, 0.81282E+00_JPRB ,&
&  0.81929E+00_JPRB, 0.82022E+00_JPRB, 0.82378E+00_JPRB, 0.83141E+00_JPRB, 0.83585E+00_JPRB, 0.83571E+00_JPRB, 0.84382E+00_JPRB ,&
&  0.86254E+00_JPRB, 0.88411E+00_JPRB, 0.89016E+00_JPRB, 0.88496E+00_JPRB, 0.88849E+00_JPRB, 0.90453E+00_JPRB, 0.91887E+00_JPRB ,&
&  0.93019E+00_JPRB, 0.93686E+00_JPRB, 0.94761E+00_JPRB, 0.91263E+00_JPRB, 0.95407E+00_JPRB, 0.93908E+00_JPRB, 0.91698E+00_JPRB ,&
&  0.89433E+00_JPRB, 0.87262E+00_JPRB, 0.86068E+00_JPRB, 0.73661E+00_JPRB, 0.77872E+00_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.74785E+00_JPRB, 0.74159E+00_JPRB, 0.72900E+00_JPRB, 0.71597E+00_JPRB, 0.70684E+00_JPRB, 0.70356E+00_JPRB, 0.70256E+00_JPRB ,&
&  0.70663E+00_JPRB, 0.71042E+00_JPRB, 0.71120E+00_JPRB, 0.70811E+00_JPRB, 0.70282E+00_JPRB, 0.69424E+00_JPRB, 0.68703E+00_JPRB ,&
&  0.67418E+00_JPRB, 0.66040E+00_JPRB, 0.65597E+00_JPRB, 0.65648E+00_JPRB, 0.65480E+00_JPRB, 0.64831E+00_JPRB, 0.64458E+00_JPRB ,&
&  0.64354E+00_JPRB, 0.64656E+00_JPRB, 0.64115E+00_JPRB, 0.63753E+00_JPRB, 0.63250E+00_JPRB, 0.63414E+00_JPRB, 0.63485E+00_JPRB ,&
&  0.63380E+00_JPRB, 0.50910E+00_JPRB, 0.57333E+00_JPRB, 0.62917E+00_JPRB, 0.65566E+00_JPRB, 0.70784E+00_JPRB, 0.76350E+00_JPRB ,&
&  0.80268E+00_JPRB, 0.82046E+00_JPRB, 0.81361E+00_JPRB, 0.79699E+00_JPRB, 0.77151E+00_JPRB, 0.74636E+00_JPRB, 0.72355E+00_JPRB ,&
&  0.70202E+00_JPRB, 0.68677E+00_JPRB, 0.67487E+00_JPRB, 0.66451E+00_JPRB, 0.65900E+00_JPRB, 0.65357E+00_JPRB, 0.64860E+00_JPRB ,&
&  0.64998E+00_JPRB, 0.65747E+00_JPRB, 0.65881E+00_JPRB, 0.66223E+00_JPRB, 0.66663E+00_JPRB, 0.67079E+00_JPRB, 0.69230E+00_JPRB ,&
&  0.71320E+00_JPRB, 0.73094E+00_JPRB, 0.75032E+00_JPRB, 0.76263E+00_JPRB, 0.76350E+00_JPRB, 0.76764E+00_JPRB, 0.76529E+00_JPRB ,&
&  0.76082E+00_JPRB, 0.74907E+00_JPRB, 0.73618E+00_JPRB, 0.72497E+00_JPRB, 0.71638E+00_JPRB, 0.71041E+00_JPRB, 0.71225E+00_JPRB ,&
&  0.71882E+00_JPRB, 0.71592E+00_JPRB, 0.71238E+00_JPRB, 0.71318E+00_JPRB, 0.71567E+00_JPRB, 0.72062E+00_JPRB, 0.73160E+00_JPRB ,&
&  0.75100E+00_JPRB, 0.77697E+00_JPRB, 0.79876E+00_JPRB, 0.80735E+00_JPRB, 0.80887E+00_JPRB, 0.81607E+00_JPRB, 0.82714E+00_JPRB ,&
&  0.83952E+00_JPRB, 0.84645E+00_JPRB, 0.85270E+00_JPRB, 0.87584E+00_JPRB, 0.87900E+00_JPRB, 0.86853E+00_JPRB, 0.85138E+00_JPRB ,&
&  0.82521E+00_JPRB, 0.81233E+00_JPRB, 0.65998E+00_JPRB, 0.72005E+00_JPRB, 0.73460E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.70595E+00_JPRB, 0.69505E+00_JPRB, 0.67943E+00_JPRB, 0.66706E+00_JPRB, 0.65748E+00_JPRB, 0.65428E+00_JPRB, 0.65246E+00_JPRB ,&
&  0.65679E+00_JPRB, 0.66442E+00_JPRB, 0.67209E+00_JPRB, 0.67666E+00_JPRB, 0.68006E+00_JPRB, 0.68038E+00_JPRB, 0.67692E+00_JPRB ,&
&  0.67416E+00_JPRB, 0.66341E+00_JPRB, 0.65519E+00_JPRB, 0.65573E+00_JPRB, 0.65338E+00_JPRB, 0.65043E+00_JPRB, 0.64639E+00_JPRB ,&
&  0.64216E+00_JPRB, 0.63862E+00_JPRB, 0.62923E+00_JPRB, 0.62473E+00_JPRB, 0.62361E+00_JPRB, 0.62605E+00_JPRB, 0.63593E+00_JPRB ,&
&  0.56124E+00_JPRB, 0.48176E+00_JPRB, 0.53898E+00_JPRB, 0.59796E+00_JPRB, 0.62839E+00_JPRB, 0.66771E+00_JPRB, 0.70349E+00_JPRB ,&
&  0.75012E+00_JPRB, 0.77923E+00_JPRB, 0.79425E+00_JPRB, 0.78854E+00_JPRB, 0.77115E+00_JPRB, 0.75181E+00_JPRB, 0.73181E+00_JPRB ,&
&  0.70710E+00_JPRB, 0.67864E+00_JPRB, 0.66005E+00_JPRB, 0.65124E+00_JPRB, 0.64378E+00_JPRB, 0.63569E+00_JPRB, 0.63089E+00_JPRB ,&
&  0.62935E+00_JPRB, 0.62839E+00_JPRB, 0.62679E+00_JPRB, 0.62597E+00_JPRB, 0.62811E+00_JPRB, 0.63092E+00_JPRB, 0.63627E+00_JPRB ,&
&  0.64390E+00_JPRB, 0.65144E+00_JPRB, 0.65717E+00_JPRB, 0.65866E+00_JPRB, 0.66062E+00_JPRB, 0.66841E+00_JPRB, 0.67312E+00_JPRB ,&
&  0.67743E+00_JPRB, 0.67706E+00_JPRB, 0.67456E+00_JPRB, 0.67066E+00_JPRB, 0.66411E+00_JPRB, 0.65775E+00_JPRB, 0.65406E+00_JPRB ,&
&  0.65355E+00_JPRB, 0.64915E+00_JPRB, 0.64846E+00_JPRB, 0.65173E+00_JPRB, 0.65661E+00_JPRB, 0.66210E+00_JPRB, 0.66735E+00_JPRB ,&
&  0.67751E+00_JPRB, 0.69598E+00_JPRB, 0.71073E+00_JPRB, 0.72042E+00_JPRB, 0.72678E+00_JPRB, 0.73244E+00_JPRB, 0.74137E+00_JPRB ,&
&  0.74991E+00_JPRB, 0.75859E+00_JPRB, 0.76965E+00_JPRB, 0.78254E+00_JPRB, 0.78651E+00_JPRB, 0.78233E+00_JPRB, 0.77232E+00_JPRB ,&
&  0.75648E+00_JPRB, 0.73571E+00_JPRB, 0.72922E+00_JPRB, 0.71959E+00_JPRB, 0.71182E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.62810E+00_JPRB, 0.62022E+00_JPRB, 0.60875E+00_JPRB, 0.60217E+00_JPRB, 0.59968E+00_JPRB, 0.60289E+00_JPRB, 0.60736E+00_JPRB ,&
&  0.61200E+00_JPRB, 0.61376E+00_JPRB, 0.62118E+00_JPRB, 0.62910E+00_JPRB, 0.63411E+00_JPRB, 0.63775E+00_JPRB, 0.63733E+00_JPRB ,&
&  0.63754E+00_JPRB, 0.63802E+00_JPRB, 0.63414E+00_JPRB, 0.63096E+00_JPRB, 0.62617E+00_JPRB, 0.62728E+00_JPRB, 0.62744E+00_JPRB ,&
&  0.62597E+00_JPRB, 0.62230E+00_JPRB, 0.61531E+00_JPRB, 0.60619E+00_JPRB, 0.60207E+00_JPRB, 0.60137E+00_JPRB, 0.59909E+00_JPRB ,&
&  0.60514E+00_JPRB, 0.52436E+00_JPRB, 0.56163E+00_JPRB, 0.57758E+00_JPRB, 0.59935E+00_JPRB, 0.62684E+00_JPRB, 0.65781E+00_JPRB ,&
&  0.68936E+00_JPRB, 0.71498E+00_JPRB, 0.72407E+00_JPRB, 0.72422E+00_JPRB, 0.71152E+00_JPRB, 0.69122E+00_JPRB, 0.66817E+00_JPRB ,&
&  0.64846E+00_JPRB, 0.63334E+00_JPRB, 0.61533E+00_JPRB, 0.60493E+00_JPRB, 0.59646E+00_JPRB, 0.58566E+00_JPRB, 0.57648E+00_JPRB ,&
&  0.57252E+00_JPRB, 0.57013E+00_JPRB, 0.56994E+00_JPRB, 0.57185E+00_JPRB, 0.57493E+00_JPRB, 0.57727E+00_JPRB, 0.57741E+00_JPRB ,&
&  0.57798E+00_JPRB, 0.57938E+00_JPRB, 0.58007E+00_JPRB, 0.58269E+00_JPRB, 0.58666E+00_JPRB, 0.59315E+00_JPRB, 0.59365E+00_JPRB ,&
&  0.58906E+00_JPRB, 0.59087E+00_JPRB, 0.59344E+00_JPRB, 0.59350E+00_JPRB, 0.59362E+00_JPRB, 0.59336E+00_JPRB, 0.59085E+00_JPRB ,&
&  0.58865E+00_JPRB, 0.59052E+00_JPRB, 0.59305E+00_JPRB, 0.59886E+00_JPRB, 0.60299E+00_JPRB, 0.61200E+00_JPRB, 0.62011E+00_JPRB ,&
&  0.62863E+00_JPRB, 0.64232E+00_JPRB, 0.65641E+00_JPRB, 0.66713E+00_JPRB, 0.67359E+00_JPRB, 0.67854E+00_JPRB, 0.68360E+00_JPRB ,&
&  0.68787E+00_JPRB, 0.69212E+00_JPRB, 0.69678E+00_JPRB, 0.69804E+00_JPRB, 0.69429E+00_JPRB, 0.68876E+00_JPRB, 0.68314E+00_JPRB ,&
&  0.67085E+00_JPRB, 0.65487E+00_JPRB, 0.64625E+00_JPRB, 0.63884E+00_JPRB, 0.63227E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.57713E+00_JPRB, 0.57129E+00_JPRB, 0.56570E+00_JPRB, 0.56476E+00_JPRB, 0.56381E+00_JPRB, 0.56596E+00_JPRB, 0.56887E+00_JPRB ,&
&  0.57569E+00_JPRB, 0.58013E+00_JPRB, 0.58941E+00_JPRB, 0.59706E+00_JPRB, 0.60087E+00_JPRB, 0.60541E+00_JPRB, 0.60842E+00_JPRB ,&
&  0.60977E+00_JPRB, 0.61163E+00_JPRB, 0.61282E+00_JPRB, 0.61746E+00_JPRB, 0.61845E+00_JPRB, 0.61798E+00_JPRB, 0.61714E+00_JPRB ,&
&  0.61524E+00_JPRB, 0.61286E+00_JPRB, 0.60740E+00_JPRB, 0.59996E+00_JPRB, 0.59074E+00_JPRB, 0.58665E+00_JPRB, 0.58049E+00_JPRB ,&
&  0.58155E+00_JPRB, 0.55087E+00_JPRB, 0.56261E+00_JPRB, 0.56097E+00_JPRB, 0.57587E+00_JPRB, 0.59950E+00_JPRB, 0.63159E+00_JPRB ,&
&  0.65377E+00_JPRB, 0.66552E+00_JPRB, 0.66710E+00_JPRB, 0.66474E+00_JPRB, 0.65035E+00_JPRB, 0.63539E+00_JPRB, 0.61854E+00_JPRB ,&
&  0.59942E+00_JPRB, 0.58875E+00_JPRB, 0.57520E+00_JPRB, 0.56529E+00_JPRB, 0.56121E+00_JPRB, 0.55830E+00_JPRB, 0.55348E+00_JPRB ,&
&  0.54613E+00_JPRB, 0.53980E+00_JPRB, 0.54042E+00_JPRB, 0.54395E+00_JPRB, 0.54769E+00_JPRB, 0.55010E+00_JPRB, 0.54910E+00_JPRB ,&
&  0.54295E+00_JPRB, 0.53822E+00_JPRB, 0.53749E+00_JPRB, 0.53888E+00_JPRB, 0.53665E+00_JPRB, 0.53110E+00_JPRB, 0.52513E+00_JPRB ,&
&  0.52431E+00_JPRB, 0.52726E+00_JPRB, 0.53075E+00_JPRB, 0.53522E+00_JPRB, 0.53819E+00_JPRB, 0.54174E+00_JPRB, 0.54322E+00_JPRB ,&
&  0.54758E+00_JPRB, 0.55178E+00_JPRB, 0.56054E+00_JPRB, 0.56894E+00_JPRB, 0.57069E+00_JPRB, 0.57910E+00_JPRB, 0.58804E+00_JPRB ,&
&  0.59731E+00_JPRB, 0.60838E+00_JPRB, 0.61758E+00_JPRB, 0.62459E+00_JPRB, 0.63179E+00_JPRB, 0.63587E+00_JPRB, 0.64167E+00_JPRB ,&
&  0.64933E+00_JPRB, 0.65109E+00_JPRB, 0.65672E+00_JPRB, 0.65674E+00_JPRB, 0.65134E+00_JPRB, 0.64268E+00_JPRB, 0.63071E+00_JPRB ,&
&  0.61702E+00_JPRB, 0.59974E+00_JPRB, 0.58997E+00_JPRB, 0.58371E+00_JPRB, 0.57991E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.51317E+00_JPRB, 0.51053E+00_JPRB, 0.51050E+00_JPRB, 0.51353E+00_JPRB, 0.51700E+00_JPRB, 0.51973E+00_JPRB, 0.52537E+00_JPRB ,&
&  0.53529E+00_JPRB, 0.54317E+00_JPRB, 0.55277E+00_JPRB, 0.56256E+00_JPRB, 0.57004E+00_JPRB, 0.57512E+00_JPRB, 0.57790E+00_JPRB ,&
&  0.57637E+00_JPRB, 0.57395E+00_JPRB, 0.57707E+00_JPRB, 0.58187E+00_JPRB, 0.58341E+00_JPRB, 0.58310E+00_JPRB, 0.58317E+00_JPRB ,&
&  0.58269E+00_JPRB, 0.58038E+00_JPRB, 0.57735E+00_JPRB, 0.57279E+00_JPRB, 0.56742E+00_JPRB, 0.56476E+00_JPRB, 0.56238E+00_JPRB ,&
&  0.55572E+00_JPRB, 0.55549E+00_JPRB, 0.50511E+00_JPRB, 0.52020E+00_JPRB, 0.53304E+00_JPRB, 0.55356E+00_JPRB, 0.57363E+00_JPRB ,&
&  0.58786E+00_JPRB, 0.59680E+00_JPRB, 0.60122E+00_JPRB, 0.59947E+00_JPRB, 0.58706E+00_JPRB, 0.57917E+00_JPRB, 0.57019E+00_JPRB ,&
&  0.55893E+00_JPRB, 0.55091E+00_JPRB, 0.53777E+00_JPRB, 0.52377E+00_JPRB, 0.51903E+00_JPRB, 0.51953E+00_JPRB, 0.51953E+00_JPRB ,&
&  0.51586E+00_JPRB, 0.51243E+00_JPRB, 0.51385E+00_JPRB, 0.51487E+00_JPRB, 0.51470E+00_JPRB, 0.50947E+00_JPRB, 0.50283E+00_JPRB ,&
&  0.49246E+00_JPRB, 0.48239E+00_JPRB, 0.47551E+00_JPRB, 0.47076E+00_JPRB, 0.46689E+00_JPRB, 0.46260E+00_JPRB, 0.45829E+00_JPRB ,&
&  0.46260E+00_JPRB, 0.47454E+00_JPRB, 0.48556E+00_JPRB, 0.49448E+00_JPRB, 0.50031E+00_JPRB, 0.50414E+00_JPRB, 0.50923E+00_JPRB ,&
&  0.51512E+00_JPRB, 0.52530E+00_JPRB, 0.53246E+00_JPRB, 0.53873E+00_JPRB, 0.54320E+00_JPRB, 0.54750E+00_JPRB, 0.55250E+00_JPRB ,&
&  0.55680E+00_JPRB, 0.56407E+00_JPRB, 0.56979E+00_JPRB, 0.57665E+00_JPRB, 0.58093E+00_JPRB, 0.58646E+00_JPRB, 0.59258E+00_JPRB ,&
&  0.59927E+00_JPRB, 0.60506E+00_JPRB, 0.60730E+00_JPRB, 0.60540E+00_JPRB, 0.59788E+00_JPRB, 0.58797E+00_JPRB, 0.57772E+00_JPRB ,&
&  0.56461E+00_JPRB, 0.54941E+00_JPRB, 0.53431E+00_JPRB, 0.52462E+00_JPRB, 0.51641E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.46500E+00_JPRB, 0.46528E+00_JPRB, 0.46964E+00_JPRB, 0.47409E+00_JPRB, 0.47737E+00_JPRB, 0.48219E+00_JPRB, 0.49172E+00_JPRB ,&
&  0.50137E+00_JPRB, 0.51069E+00_JPRB, 0.51712E+00_JPRB, 0.52492E+00_JPRB, 0.53587E+00_JPRB, 0.54329E+00_JPRB, 0.54925E+00_JPRB ,&
&  0.55176E+00_JPRB, 0.55218E+00_JPRB, 0.55328E+00_JPRB, 0.55648E+00_JPRB, 0.55611E+00_JPRB, 0.55256E+00_JPRB, 0.55134E+00_JPRB ,&
&  0.55129E+00_JPRB, 0.54962E+00_JPRB, 0.54938E+00_JPRB, 0.55039E+00_JPRB, 0.54800E+00_JPRB, 0.54411E+00_JPRB, 0.54353E+00_JPRB ,&
&  0.54369E+00_JPRB, 0.53001E+00_JPRB, 0.52237E+00_JPRB, 0.51541E+00_JPRB, 0.51911E+00_JPRB, 0.53063E+00_JPRB, 0.54330E+00_JPRB ,&
&  0.55097E+00_JPRB, 0.55614E+00_JPRB, 0.55588E+00_JPRB, 0.55498E+00_JPRB, 0.55036E+00_JPRB, 0.53894E+00_JPRB, 0.52917E+00_JPRB ,&
&  0.52253E+00_JPRB, 0.51307E+00_JPRB, 0.49054E+00_JPRB, 0.47755E+00_JPRB, 0.47419E+00_JPRB, 0.47684E+00_JPRB, 0.48084E+00_JPRB ,&
&  0.48253E+00_JPRB, 0.48002E+00_JPRB, 0.47398E+00_JPRB, 0.46678E+00_JPRB, 0.45693E+00_JPRB, 0.44556E+00_JPRB, 0.43563E+00_JPRB ,&
&  0.43004E+00_JPRB, 0.42601E+00_JPRB, 0.42271E+00_JPRB, 0.42543E+00_JPRB, 0.42488E+00_JPRB, 0.42135E+00_JPRB, 0.42375E+00_JPRB ,&
&  0.43572E+00_JPRB, 0.45075E+00_JPRB, 0.46461E+00_JPRB, 0.47221E+00_JPRB, 0.47535E+00_JPRB, 0.47896E+00_JPRB, 0.48309E+00_JPRB ,&
&  0.48887E+00_JPRB, 0.49449E+00_JPRB, 0.49960E+00_JPRB, 0.50621E+00_JPRB, 0.51438E+00_JPRB, 0.52165E+00_JPRB, 0.52466E+00_JPRB ,&
&  0.52723E+00_JPRB, 0.52892E+00_JPRB, 0.53042E+00_JPRB, 0.53596E+00_JPRB, 0.54412E+00_JPRB, 0.54980E+00_JPRB, 0.55431E+00_JPRB ,&
&  0.55892E+00_JPRB, 0.56100E+00_JPRB, 0.55950E+00_JPRB, 0.55521E+00_JPRB, 0.54957E+00_JPRB, 0.53851E+00_JPRB, 0.52469E+00_JPRB ,&
&  0.50874E+00_JPRB, 0.49404E+00_JPRB, 0.48311E+00_JPRB, 0.47368E+00_JPRB, 0.46707E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.42869E+00_JPRB, 0.42822E+00_JPRB, 0.43084E+00_JPRB, 0.43695E+00_JPRB, 0.44114E+00_JPRB, 0.44504E+00_JPRB, 0.45277E+00_JPRB ,&
&  0.45982E+00_JPRB, 0.46607E+00_JPRB, 0.47354E+00_JPRB, 0.48172E+00_JPRB, 0.48802E+00_JPRB, 0.49711E+00_JPRB, 0.50572E+00_JPRB ,&
&  0.51126E+00_JPRB, 0.51351E+00_JPRB, 0.51481E+00_JPRB, 0.51594E+00_JPRB, 0.51580E+00_JPRB, 0.51328E+00_JPRB, 0.51286E+00_JPRB ,&
&  0.51107E+00_JPRB, 0.50906E+00_JPRB, 0.51126E+00_JPRB, 0.51165E+00_JPRB, 0.51335E+00_JPRB, 0.51611E+00_JPRB, 0.51621E+00_JPRB ,&
&  0.51409E+00_JPRB, 0.50105E+00_JPRB, 0.49627E+00_JPRB, 0.49336E+00_JPRB, 0.49764E+00_JPRB, 0.49880E+00_JPRB, 0.49748E+00_JPRB ,&
&  0.49124E+00_JPRB, 0.48743E+00_JPRB, 0.48934E+00_JPRB, 0.49173E+00_JPRB, 0.48602E+00_JPRB, 0.47302E+00_JPRB, 0.46268E+00_JPRB ,&
&  0.45449E+00_JPRB, 0.44215E+00_JPRB, 0.42428E+00_JPRB, 0.41370E+00_JPRB, 0.40920E+00_JPRB, 0.40429E+00_JPRB, 0.39979E+00_JPRB ,&
&  0.39741E+00_JPRB, 0.39666E+00_JPRB, 0.39809E+00_JPRB, 0.39726E+00_JPRB, 0.39362E+00_JPRB, 0.38918E+00_JPRB, 0.38748E+00_JPRB ,&
&  0.39024E+00_JPRB, 0.39287E+00_JPRB, 0.39659E+00_JPRB, 0.39927E+00_JPRB, 0.39880E+00_JPRB, 0.39940E+00_JPRB, 0.40346E+00_JPRB ,&
&  0.40985E+00_JPRB, 0.41605E+00_JPRB, 0.42195E+00_JPRB, 0.42443E+00_JPRB, 0.42773E+00_JPRB, 0.43464E+00_JPRB, 0.43520E+00_JPRB ,&
&  0.43364E+00_JPRB, 0.43538E+00_JPRB, 0.43940E+00_JPRB, 0.44729E+00_JPRB, 0.45949E+00_JPRB, 0.46741E+00_JPRB, 0.46853E+00_JPRB ,&
&  0.46853E+00_JPRB, 0.47077E+00_JPRB, 0.47478E+00_JPRB, 0.48443E+00_JPRB, 0.49479E+00_JPRB, 0.50214E+00_JPRB, 0.50514E+00_JPRB ,&
&  0.50191E+00_JPRB, 0.49503E+00_JPRB, 0.48273E+00_JPRB, 0.46837E+00_JPRB, 0.45573E+00_JPRB, 0.44587E+00_JPRB, 0.43783E+00_JPRB ,&
&  0.42991E+00_JPRB, 0.42645E+00_JPRB, 0.42608E+00_JPRB, 0.42788E+00_JPRB, 0.42871E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.40075E+00_JPRB, 0.40243E+00_JPRB, 0.40457E+00_JPRB, 0.40743E+00_JPRB, 0.41056E+00_JPRB, 0.41251E+00_JPRB, 0.41637E+00_JPRB ,&
&  0.41855E+00_JPRB, 0.42500E+00_JPRB, 0.43565E+00_JPRB, 0.44637E+00_JPRB, 0.45418E+00_JPRB, 0.46084E+00_JPRB, 0.46734E+00_JPRB ,&
&  0.47263E+00_JPRB, 0.47964E+00_JPRB, 0.48052E+00_JPRB, 0.47807E+00_JPRB, 0.47539E+00_JPRB, 0.47335E+00_JPRB, 0.47153E+00_JPRB ,&
&  0.46904E+00_JPRB, 0.46895E+00_JPRB, 0.47274E+00_JPRB, 0.47701E+00_JPRB, 0.48140E+00_JPRB, 0.48265E+00_JPRB, 0.48135E+00_JPRB ,&
&  0.48500E+00_JPRB, 0.47552E+00_JPRB, 0.46936E+00_JPRB, 0.46214E+00_JPRB, 0.45578E+00_JPRB, 0.44436E+00_JPRB, 0.43597E+00_JPRB ,&
&  0.42600E+00_JPRB, 0.41726E+00_JPRB, 0.41355E+00_JPRB, 0.40958E+00_JPRB, 0.39991E+00_JPRB, 0.38859E+00_JPRB, 0.38017E+00_JPRB ,&
&  0.37455E+00_JPRB, 0.36887E+00_JPRB, 0.36209E+00_JPRB, 0.35655E+00_JPRB, 0.35205E+00_JPRB, 0.35052E+00_JPRB, 0.35231E+00_JPRB ,&
&  0.35645E+00_JPRB, 0.36032E+00_JPRB, 0.36257E+00_JPRB, 0.36309E+00_JPRB, 0.36151E+00_JPRB, 0.35895E+00_JPRB, 0.35748E+00_JPRB ,&
&  0.35822E+00_JPRB, 0.35985E+00_JPRB, 0.36208E+00_JPRB, 0.36729E+00_JPRB, 0.36821E+00_JPRB, 0.36690E+00_JPRB, 0.36165E+00_JPRB ,&
&  0.35958E+00_JPRB, 0.35870E+00_JPRB, 0.36064E+00_JPRB, 0.36589E+00_JPRB, 0.37435E+00_JPRB, 0.38245E+00_JPRB, 0.38816E+00_JPRB ,&
&  0.39094E+00_JPRB, 0.39256E+00_JPRB, 0.39555E+00_JPRB, 0.39914E+00_JPRB, 0.40056E+00_JPRB, 0.39984E+00_JPRB, 0.39830E+00_JPRB ,&
&  0.40106E+00_JPRB, 0.40851E+00_JPRB, 0.41650E+00_JPRB, 0.42180E+00_JPRB, 0.42445E+00_JPRB, 0.42345E+00_JPRB, 0.41652E+00_JPRB ,&
&  0.40394E+00_JPRB, 0.38951E+00_JPRB, 0.37546E+00_JPRB, 0.36358E+00_JPRB, 0.35542E+00_JPRB, 0.35154E+00_JPRB, 0.35445E+00_JPRB ,&
&  0.36227E+00_JPRB, 0.37228E+00_JPRB, 0.38150E+00_JPRB, 0.38836E+00_JPRB, 0.39574E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.37325E+00_JPRB, 0.37307E+00_JPRB, 0.37454E+00_JPRB, 0.37341E+00_JPRB, 0.37072E+00_JPRB, 0.36867E+00_JPRB, 0.37215E+00_JPRB ,&
&  0.37897E+00_JPRB, 0.39053E+00_JPRB, 0.40592E+00_JPRB, 0.41946E+00_JPRB, 0.42765E+00_JPRB, 0.43267E+00_JPRB, 0.44033E+00_JPRB ,&
&  0.44745E+00_JPRB, 0.45539E+00_JPRB, 0.45558E+00_JPRB, 0.44675E+00_JPRB, 0.43925E+00_JPRB, 0.43530E+00_JPRB, 0.43478E+00_JPRB ,&
&  0.43413E+00_JPRB, 0.43593E+00_JPRB, 0.43912E+00_JPRB, 0.44252E+00_JPRB, 0.44186E+00_JPRB, 0.44003E+00_JPRB, 0.44363E+00_JPRB ,&
&  0.44161E+00_JPRB, 0.43192E+00_JPRB, 0.42208E+00_JPRB, 0.39716E+00_JPRB, 0.39647E+00_JPRB, 0.38614E+00_JPRB, 0.37818E+00_JPRB ,&
&  0.36749E+00_JPRB, 0.35432E+00_JPRB, 0.34476E+00_JPRB, 0.33865E+00_JPRB, 0.33595E+00_JPRB, 0.33339E+00_JPRB, 0.33130E+00_JPRB ,&
&  0.32986E+00_JPRB, 0.32948E+00_JPRB, 0.33070E+00_JPRB, 0.32835E+00_JPRB, 0.32718E+00_JPRB, 0.32797E+00_JPRB, 0.32970E+00_JPRB ,&
&  0.32951E+00_JPRB, 0.32931E+00_JPRB, 0.32915E+00_JPRB, 0.32819E+00_JPRB, 0.32656E+00_JPRB, 0.32413E+00_JPRB, 0.32185E+00_JPRB ,&
&  0.32100E+00_JPRB, 0.32224E+00_JPRB, 0.32280E+00_JPRB, 0.31919E+00_JPRB, 0.31172E+00_JPRB, 0.30261E+00_JPRB, 0.29683E+00_JPRB ,&
&  0.29463E+00_JPRB, 0.29705E+00_JPRB, 0.30512E+00_JPRB, 0.31644E+00_JPRB, 0.32691E+00_JPRB, 0.33302E+00_JPRB, 0.33665E+00_JPRB ,&
&  0.34086E+00_JPRB, 0.33890E+00_JPRB, 0.33503E+00_JPRB, 0.33156E+00_JPRB, 0.32843E+00_JPRB, 0.32435E+00_JPRB, 0.32425E+00_JPRB ,&
&  0.32684E+00_JPRB, 0.32943E+00_JPRB, 0.33022E+00_JPRB, 0.32917E+00_JPRB, 0.32717E+00_JPRB, 0.32294E+00_JPRB, 0.31698E+00_JPRB ,&
&  0.30958E+00_JPRB, 0.30095E+00_JPRB, 0.29895E+00_JPRB, 0.29950E+00_JPRB, 0.30323E+00_JPRB, 0.31100E+00_JPRB, 0.32165E+00_JPRB ,&
&  0.33284E+00_JPRB, 0.34207E+00_JPRB, 0.35207E+00_JPRB, 0.36137E+00_JPRB, 0.36967E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.35593E+00_JPRB, 0.35741E+00_JPRB, 0.35482E+00_JPRB, 0.35195E+00_JPRB, 0.35158E+00_JPRB, 0.35317E+00_JPRB, 0.35884E+00_JPRB ,&
&  0.36855E+00_JPRB, 0.38113E+00_JPRB, 0.39319E+00_JPRB, 0.40319E+00_JPRB, 0.41222E+00_JPRB, 0.41619E+00_JPRB, 0.42215E+00_JPRB ,&
&  0.42858E+00_JPRB, 0.43447E+00_JPRB, 0.43523E+00_JPRB, 0.42494E+00_JPRB, 0.41494E+00_JPRB, 0.40961E+00_JPRB, 0.40959E+00_JPRB ,&
&  0.41136E+00_JPRB, 0.41399E+00_JPRB, 0.42013E+00_JPRB, 0.42239E+00_JPRB, 0.41648E+00_JPRB, 0.41502E+00_JPRB, 0.41282E+00_JPRB ,&
&  0.40718E+00_JPRB, 0.39571E+00_JPRB, 0.36330E+00_JPRB, 0.35762E+00_JPRB, 0.34935E+00_JPRB, 0.34027E+00_JPRB, 0.33123E+00_JPRB ,&
&  0.32094E+00_JPRB, 0.31065E+00_JPRB, 0.30365E+00_JPRB, 0.29956E+00_JPRB, 0.30029E+00_JPRB, 0.30151E+00_JPRB, 0.30099E+00_JPRB ,&
&  0.30297E+00_JPRB, 0.30550E+00_JPRB, 0.30766E+00_JPRB, 0.30784E+00_JPRB, 0.30735E+00_JPRB, 0.30617E+00_JPRB, 0.30162E+00_JPRB ,&
&  0.29752E+00_JPRB, 0.29534E+00_JPRB, 0.29364E+00_JPRB, 0.29220E+00_JPRB, 0.29201E+00_JPRB, 0.29246E+00_JPRB, 0.29263E+00_JPRB ,&
&  0.29056E+00_JPRB, 0.28711E+00_JPRB, 0.28325E+00_JPRB, 0.27646E+00_JPRB, 0.26468E+00_JPRB, 0.24950E+00_JPRB, 0.24020E+00_JPRB ,&
&  0.23879E+00_JPRB, 0.24972E+00_JPRB, 0.27122E+00_JPRB, 0.28049E+00_JPRB, 0.28783E+00_JPRB, 0.29172E+00_JPRB, 0.29605E+00_JPRB ,&
&  0.29128E+00_JPRB, 0.26720E+00_JPRB, 0.25382E+00_JPRB, 0.24669E+00_JPRB, 0.24115E+00_JPRB, 0.23693E+00_JPRB, 0.23666E+00_JPRB ,&
&  0.23981E+00_JPRB, 0.24614E+00_JPRB, 0.25250E+00_JPRB, 0.25563E+00_JPRB, 0.25492E+00_JPRB, 0.25169E+00_JPRB, 0.24836E+00_JPRB ,&
&  0.24565E+00_JPRB, 0.25026E+00_JPRB, 0.26494E+00_JPRB, 0.28700E+00_JPRB, 0.29514E+00_JPRB, 0.30143E+00_JPRB, 0.30733E+00_JPRB ,&
&  0.31366E+00_JPRB, 0.32321E+00_JPRB, 0.33478E+00_JPRB, 0.34397E+00_JPRB, 0.35210E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.32953E+00_JPRB, 0.33371E+00_JPRB, 0.33637E+00_JPRB, 0.33978E+00_JPRB, 0.34735E+00_JPRB, 0.35793E+00_JPRB, 0.37001E+00_JPRB ,&
&  0.38122E+00_JPRB, 0.39037E+00_JPRB, 0.39488E+00_JPRB, 0.40004E+00_JPRB, 0.40498E+00_JPRB, 0.40732E+00_JPRB, 0.41245E+00_JPRB ,&
&  0.41317E+00_JPRB, 0.40975E+00_JPRB, 0.40364E+00_JPRB, 0.39556E+00_JPRB, 0.39127E+00_JPRB, 0.39249E+00_JPRB, 0.39534E+00_JPRB ,&
&  0.39719E+00_JPRB, 0.39863E+00_JPRB, 0.39866E+00_JPRB, 0.39521E+00_JPRB, 0.39191E+00_JPRB, 0.38996E+00_JPRB, 0.38588E+00_JPRB ,&
&  0.37662E+00_JPRB, 0.32708E+00_JPRB, 0.28948E+00_JPRB, 0.28521E+00_JPRB, 0.30165E+00_JPRB, 0.30385E+00_JPRB, 0.29870E+00_JPRB ,&
&  0.29296E+00_JPRB, 0.28824E+00_JPRB, 0.28554E+00_JPRB, 0.28340E+00_JPRB, 0.28069E+00_JPRB, 0.27825E+00_JPRB, 0.27891E+00_JPRB ,&
&  0.28013E+00_JPRB, 0.27785E+00_JPRB, 0.27477E+00_JPRB, 0.26847E+00_JPRB, 0.26393E+00_JPRB, 0.26201E+00_JPRB, 0.26044E+00_JPRB ,&
&  0.25622E+00_JPRB, 0.25210E+00_JPRB, 0.25197E+00_JPRB, 0.25313E+00_JPRB, 0.25496E+00_JPRB, 0.25623E+00_JPRB, 0.25365E+00_JPRB ,&
&  0.24780E+00_JPRB, 0.24139E+00_JPRB, 0.23327E+00_JPRB, 0.22356E+00_JPRB, 0.21428E+00_JPRB, 0.20763E+00_JPRB, 0.20244E+00_JPRB ,&
&  0.20160E+00_JPRB, 0.20724E+00_JPRB, 0.22025E+00_JPRB, 0.23585E+00_JPRB, 0.25468E+00_JPRB, 0.24436E+00_JPRB, 0.23339E+00_JPRB ,&
&  0.22274E+00_JPRB, 0.21413E+00_JPRB, 0.20777E+00_JPRB, 0.20485E+00_JPRB, 0.20369E+00_JPRB, 0.20368E+00_JPRB, 0.20481E+00_JPRB ,&
&  0.20686E+00_JPRB, 0.20952E+00_JPRB, 0.21186E+00_JPRB, 0.21295E+00_JPRB, 0.21355E+00_JPRB, 0.21408E+00_JPRB, 0.21396E+00_JPRB ,&
&  0.21333E+00_JPRB, 0.21569E+00_JPRB, 0.22246E+00_JPRB, 0.23097E+00_JPRB, 0.23921E+00_JPRB, 0.24699E+00_JPRB, 0.25996E+00_JPRB ,&
&  0.27588E+00_JPRB, 0.29583E+00_JPRB, 0.30823E+00_JPRB, 0.31769E+00_JPRB, 0.32586E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.32360E+00_JPRB, 0.33249E+00_JPRB, 0.34447E+00_JPRB, 0.35482E+00_JPRB, 0.36384E+00_JPRB, 0.37240E+00_JPRB, 0.37991E+00_JPRB ,&
&  0.38395E+00_JPRB, 0.38629E+00_JPRB, 0.38718E+00_JPRB, 0.39068E+00_JPRB, 0.38765E+00_JPRB, 0.38081E+00_JPRB, 0.37527E+00_JPRB ,&
&  0.37067E+00_JPRB, 0.36708E+00_JPRB, 0.36179E+00_JPRB, 0.35901E+00_JPRB, 0.35871E+00_JPRB, 0.35875E+00_JPRB, 0.35308E+00_JPRB ,&
&  0.34259E+00_JPRB, 0.34161E+00_JPRB, 0.34629E+00_JPRB, 0.34091E+00_JPRB, 0.33522E+00_JPRB, 0.32977E+00_JPRB, 0.32388E+00_JPRB ,&
&  0.30799E+00_JPRB, 0.28800E+00_JPRB, 0.26107E+00_JPRB, 0.25694E+00_JPRB, 0.27275E+00_JPRB, 0.26644E+00_JPRB, 0.26953E+00_JPRB ,&
&  0.27006E+00_JPRB, 0.27034E+00_JPRB, 0.26886E+00_JPRB, 0.26730E+00_JPRB, 0.26373E+00_JPRB, 0.26245E+00_JPRB, 0.26088E+00_JPRB ,&
&  0.25527E+00_JPRB, 0.25032E+00_JPRB, 0.24244E+00_JPRB, 0.22483E+00_JPRB, 0.21974E+00_JPRB, 0.21801E+00_JPRB, 0.21315E+00_JPRB ,&
&  0.20550E+00_JPRB, 0.19983E+00_JPRB, 0.19752E+00_JPRB, 0.19390E+00_JPRB, 0.19604E+00_JPRB, 0.19792E+00_JPRB, 0.19791E+00_JPRB ,&
&  0.19787E+00_JPRB, 0.19750E+00_JPRB, 0.19520E+00_JPRB, 0.19137E+00_JPRB, 0.18840E+00_JPRB, 0.18740E+00_JPRB, 0.18823E+00_JPRB ,&
&  0.19062E+00_JPRB, 0.19518E+00_JPRB, 0.20604E+00_JPRB, 0.21454E+00_JPRB, 0.21359E+00_JPRB, 0.20916E+00_JPRB, 0.20023E+00_JPRB ,&
&  0.19248E+00_JPRB, 0.18956E+00_JPRB, 0.18703E+00_JPRB, 0.18568E+00_JPRB, 0.18599E+00_JPRB, 0.18741E+00_JPRB, 0.18819E+00_JPRB ,&
&  0.18925E+00_JPRB, 0.19029E+00_JPRB, 0.19111E+00_JPRB, 0.19161E+00_JPRB, 0.19244E+00_JPRB, 0.19341E+00_JPRB, 0.19521E+00_JPRB ,&
&  0.19785E+00_JPRB, 0.20226E+00_JPRB, 0.20801E+00_JPRB, 0.21420E+00_JPRB, 0.21870E+00_JPRB, 0.22091E+00_JPRB, 0.22483E+00_JPRB ,&
&  0.23068E+00_JPRB, 0.24563E+00_JPRB, 0.27416E+00_JPRB, 0.31924E+00_JPRB, 0.32311E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.32408E+00_JPRB, 0.33616E+00_JPRB, 0.35146E+00_JPRB, 0.36097E+00_JPRB, 0.36636E+00_JPRB, 0.36907E+00_JPRB, 0.36803E+00_JPRB ,&
&  0.36467E+00_JPRB, 0.36414E+00_JPRB, 0.37523E+00_JPRB, 0.35726E+00_JPRB, 0.33963E+00_JPRB, 0.30643E+00_JPRB, 0.28994E+00_JPRB ,&
&  0.27753E+00_JPRB, 0.27537E+00_JPRB, 0.27507E+00_JPRB, 0.27973E+00_JPRB, 0.28357E+00_JPRB, 0.28659E+00_JPRB, 0.28284E+00_JPRB ,&
&  0.27750E+00_JPRB, 0.26827E+00_JPRB, 0.26053E+00_JPRB, 0.25329E+00_JPRB, 0.25338E+00_JPRB, 0.25226E+00_JPRB, 0.24966E+00_JPRB ,&
&  0.24668E+00_JPRB, 0.24480E+00_JPRB, 0.24470E+00_JPRB, 0.24497E+00_JPRB, 0.25656E+00_JPRB, 0.25577E+00_JPRB, 0.25581E+00_JPRB ,&
&  0.25546E+00_JPRB, 0.25541E+00_JPRB, 0.25441E+00_JPRB, 0.25265E+00_JPRB, 0.24786E+00_JPRB, 0.23479E+00_JPRB, 0.23409E+00_JPRB ,&
&  0.22484E+00_JPRB, 0.21428E+00_JPRB, 0.20406E+00_JPRB, 0.19695E+00_JPRB, 0.19263E+00_JPRB, 0.19077E+00_JPRB, 0.18614E+00_JPRB ,&
&  0.18117E+00_JPRB, 0.17547E+00_JPRB, 0.17283E+00_JPRB, 0.17264E+00_JPRB, 0.17323E+00_JPRB, 0.17381E+00_JPRB, 0.17430E+00_JPRB ,&
&  0.17556E+00_JPRB, 0.17616E+00_JPRB, 0.17654E+00_JPRB, 0.17726E+00_JPRB, 0.17796E+00_JPRB, 0.17903E+00_JPRB, 0.18084E+00_JPRB ,&
&  0.18402E+00_JPRB, 0.18744E+00_JPRB, 0.19062E+00_JPRB, 0.19328E+00_JPRB, 0.19229E+00_JPRB, 0.18893E+00_JPRB, 0.18448E+00_JPRB ,&
&  0.18124E+00_JPRB, 0.18014E+00_JPRB, 0.17912E+00_JPRB, 0.17902E+00_JPRB, 0.17918E+00_JPRB, 0.17959E+00_JPRB, 0.18021E+00_JPRB ,&
&  0.18103E+00_JPRB, 0.18173E+00_JPRB, 0.18259E+00_JPRB, 0.18370E+00_JPRB, 0.18526E+00_JPRB, 0.18668E+00_JPRB, 0.18866E+00_JPRB ,&
&  0.19162E+00_JPRB, 0.19488E+00_JPRB, 0.19951E+00_JPRB, 0.20371E+00_JPRB, 0.20939E+00_JPRB, 0.21757E+00_JPRB, 0.23099E+00_JPRB ,&
&  0.24541E+00_JPRB, 0.27966E+00_JPRB, 0.29537E+00_JPRB, 0.30633E+00_JPRB, 0.31382E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.32453E+00_JPRB, 0.33412E+00_JPRB, 0.34820E+00_JPRB, 0.35732E+00_JPRB, 0.36456E+00_JPRB, 0.36044E+00_JPRB, 0.35652E+00_JPRB ,&
&  0.35243E+00_JPRB, 0.33842E+00_JPRB, 0.32125E+00_JPRB, 0.29209E+00_JPRB, 0.26754E+00_JPRB, 0.24876E+00_JPRB, 0.23831E+00_JPRB ,&
&  0.23202E+00_JPRB, 0.22990E+00_JPRB, 0.22785E+00_JPRB, 0.22729E+00_JPRB, 0.22601E+00_JPRB, 0.22753E+00_JPRB, 0.22879E+00_JPRB ,&
&  0.22832E+00_JPRB, 0.22724E+00_JPRB, 0.22667E+00_JPRB, 0.22978E+00_JPRB, 0.23582E+00_JPRB, 0.24056E+00_JPRB, 0.24512E+00_JPRB ,&
&  0.24841E+00_JPRB, 0.24578E+00_JPRB, 0.24699E+00_JPRB, 0.24503E+00_JPRB, 0.24303E+00_JPRB, 0.24330E+00_JPRB, 0.24433E+00_JPRB ,&
&  0.24475E+00_JPRB, 0.24518E+00_JPRB, 0.24440E+00_JPRB, 0.24129E+00_JPRB, 0.22950E+00_JPRB, 0.22340E+00_JPRB, 0.21559E+00_JPRB ,&
&  0.20759E+00_JPRB, 0.19823E+00_JPRB, 0.19065E+00_JPRB, 0.18701E+00_JPRB, 0.18264E+00_JPRB, 0.18095E+00_JPRB, 0.17778E+00_JPRB ,&
&  0.17416E+00_JPRB, 0.17074E+00_JPRB, 0.16851E+00_JPRB, 0.16740E+00_JPRB, 0.16701E+00_JPRB, 0.16594E+00_JPRB, 0.16558E+00_JPRB ,&
&  0.16588E+00_JPRB, 0.16622E+00_JPRB, 0.16707E+00_JPRB, 0.16792E+00_JPRB, 0.16884E+00_JPRB, 0.16968E+00_JPRB, 0.17070E+00_JPRB ,&
&  0.17232E+00_JPRB, 0.17431E+00_JPRB, 0.17520E+00_JPRB, 0.17596E+00_JPRB, 0.17567E+00_JPRB, 0.17515E+00_JPRB, 0.17436E+00_JPRB ,&
&  0.17373E+00_JPRB, 0.17353E+00_JPRB, 0.17353E+00_JPRB, 0.17346E+00_JPRB, 0.17393E+00_JPRB, 0.17481E+00_JPRB, 0.17574E+00_JPRB ,&
&  0.17729E+00_JPRB, 0.17869E+00_JPRB, 0.18029E+00_JPRB, 0.18218E+00_JPRB, 0.18433E+00_JPRB, 0.18688E+00_JPRB, 0.18974E+00_JPRB ,&
&  0.19231E+00_JPRB, 0.19561E+00_JPRB, 0.19841E+00_JPRB, 0.20066E+00_JPRB, 0.20597E+00_JPRB, 0.21314E+00_JPRB, 0.22745E+00_JPRB ,&
&  0.24386E+00_JPRB, 0.28370E+00_JPRB, 0.32581E+00_JPRB, 0.31132E+00_JPRB, 0.31643E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.29760E+00_JPRB, 0.30852E+00_JPRB, 0.31729E+00_JPRB, 0.32278E+00_JPRB, 0.32836E+00_JPRB, 0.33051E+00_JPRB, 0.33869E+00_JPRB ,&
&  0.33324E+00_JPRB, 0.31520E+00_JPRB, 0.30405E+00_JPRB, 0.28762E+00_JPRB, 0.26735E+00_JPRB, 0.25078E+00_JPRB, 0.24241E+00_JPRB ,&
&  0.23744E+00_JPRB, 0.23347E+00_JPRB, 0.22870E+00_JPRB, 0.22345E+00_JPRB, 0.21818E+00_JPRB, 0.21426E+00_JPRB, 0.21173E+00_JPRB ,&
&  0.20943E+00_JPRB, 0.21005E+00_JPRB, 0.21264E+00_JPRB, 0.21697E+00_JPRB, 0.22315E+00_JPRB, 0.22741E+00_JPRB, 0.23075E+00_JPRB ,&
&  0.23287E+00_JPRB, 0.22391E+00_JPRB, 0.22561E+00_JPRB, 0.22760E+00_JPRB, 0.22633E+00_JPRB, 0.22941E+00_JPRB, 0.23432E+00_JPRB ,&
&  0.23186E+00_JPRB, 0.22494E+00_JPRB, 0.22754E+00_JPRB, 0.22258E+00_JPRB, 0.21418E+00_JPRB, 0.21232E+00_JPRB, 0.20651E+00_JPRB ,&
&  0.20360E+00_JPRB, 0.19642E+00_JPRB, 0.19131E+00_JPRB, 0.18793E+00_JPRB, 0.18240E+00_JPRB, 0.18013E+00_JPRB, 0.17671E+00_JPRB ,&
&  0.17321E+00_JPRB, 0.17077E+00_JPRB, 0.16788E+00_JPRB, 0.16647E+00_JPRB, 0.16527E+00_JPRB, 0.16404E+00_JPRB, 0.16307E+00_JPRB ,&
&  0.16253E+00_JPRB, 0.16203E+00_JPRB, 0.16206E+00_JPRB, 0.16208E+00_JPRB, 0.16207E+00_JPRB, 0.16239E+00_JPRB, 0.16257E+00_JPRB ,&
&  0.16318E+00_JPRB, 0.16390E+00_JPRB, 0.16448E+00_JPRB, 0.16542E+00_JPRB, 0.16583E+00_JPRB, 0.16637E+00_JPRB, 0.16677E+00_JPRB ,&
&  0.16721E+00_JPRB, 0.16788E+00_JPRB, 0.16823E+00_JPRB, 0.16891E+00_JPRB, 0.16968E+00_JPRB, 0.17117E+00_JPRB, 0.17265E+00_JPRB ,&
&  0.17445E+00_JPRB, 0.17655E+00_JPRB, 0.17878E+00_JPRB, 0.18115E+00_JPRB, 0.18409E+00_JPRB, 0.18678E+00_JPRB, 0.19033E+00_JPRB ,&
&  0.19416E+00_JPRB, 0.19783E+00_JPRB, 0.20213E+00_JPRB, 0.20756E+00_JPRB, 0.21472E+00_JPRB, 0.22438E+00_JPRB, 0.23330E+00_JPRB ,&
&  0.24352E+00_JPRB, 0.26963E+00_JPRB, 0.27605E+00_JPRB, 0.28034E+00_JPRB, 0.28896E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.28960E+00_JPRB, 0.29451E+00_JPRB, 0.29828E+00_JPRB, 0.30070E+00_JPRB, 0.30301E+00_JPRB, 0.30464E+00_JPRB, 0.30524E+00_JPRB ,&
&  0.29879E+00_JPRB, 0.29382E+00_JPRB, 0.28686E+00_JPRB, 0.27848E+00_JPRB, 0.27082E+00_JPRB, 0.26373E+00_JPRB, 0.25569E+00_JPRB ,&
&  0.24909E+00_JPRB, 0.24286E+00_JPRB, 0.23636E+00_JPRB, 0.22954E+00_JPRB, 0.22240E+00_JPRB, 0.21520E+00_JPRB, 0.21175E+00_JPRB ,&
&  0.21074E+00_JPRB, 0.21187E+00_JPRB, 0.21389E+00_JPRB, 0.21544E+00_JPRB, 0.21635E+00_JPRB, 0.21693E+00_JPRB, 0.21577E+00_JPRB ,&
&  0.21206E+00_JPRB, 0.21273E+00_JPRB, 0.21368E+00_JPRB, 0.21429E+00_JPRB, 0.21586E+00_JPRB, 0.21718E+00_JPRB, 0.21699E+00_JPRB ,&
&  0.21514E+00_JPRB, 0.21268E+00_JPRB, 0.20933E+00_JPRB, 0.20620E+00_JPRB, 0.20502E+00_JPRB, 0.20285E+00_JPRB, 0.20038E+00_JPRB ,&
&  0.19841E+00_JPRB, 0.19452E+00_JPRB, 0.19054E+00_JPRB, 0.18772E+00_JPRB, 0.18404E+00_JPRB, 0.18153E+00_JPRB, 0.17826E+00_JPRB ,&
&  0.17563E+00_JPRB, 0.17325E+00_JPRB, 0.17063E+00_JPRB, 0.16861E+00_JPRB, 0.16672E+00_JPRB, 0.16532E+00_JPRB, 0.16392E+00_JPRB ,&
&  0.16311E+00_JPRB, 0.16234E+00_JPRB, 0.16184E+00_JPRB, 0.16128E+00_JPRB, 0.16095E+00_JPRB, 0.16061E+00_JPRB, 0.16035E+00_JPRB ,&
&  0.16020E+00_JPRB, 0.16051E+00_JPRB, 0.16073E+00_JPRB, 0.16113E+00_JPRB, 0.16160E+00_JPRB, 0.16188E+00_JPRB, 0.16242E+00_JPRB ,&
&  0.16344E+00_JPRB, 0.16455E+00_JPRB, 0.16529E+00_JPRB, 0.16624E+00_JPRB, 0.16731E+00_JPRB, 0.16862E+00_JPRB, 0.17040E+00_JPRB ,&
&  0.17228E+00_JPRB, 0.17437E+00_JPRB, 0.17673E+00_JPRB, 0.17906E+00_JPRB, 0.18207E+00_JPRB, 0.18515E+00_JPRB, 0.18839E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19635E+00_JPRB, 0.20058E+00_JPRB, 0.20523E+00_JPRB, 0.21192E+00_JPRB, 0.21793E+00_JPRB, 0.22538E+00_JPRB ,&
&  0.23265E+00_JPRB, 0.23829E+00_JPRB, 0.27045E+00_JPRB, 0.27528E+00_JPRB, 0.28203E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.21582E+00_JPRB, 0.22258E+00_JPRB, 0.22934E+00_JPRB, 0.23894E+00_JPRB, 0.24557E+00_JPRB, 0.28115E+00_JPRB, 0.27609E+00_JPRB ,&
&  0.27278E+00_JPRB, 0.25925E+00_JPRB, 0.25779E+00_JPRB, 0.25419E+00_JPRB, 0.24967E+00_JPRB, 0.24476E+00_JPRB, 0.23981E+00_JPRB ,&
&  0.23517E+00_JPRB, 0.22972E+00_JPRB, 0.22507E+00_JPRB, 0.22139E+00_JPRB, 0.21810E+00_JPRB, 0.21487E+00_JPRB, 0.21206E+00_JPRB ,&
&  0.21086E+00_JPRB, 0.21072E+00_JPRB, 0.20994E+00_JPRB, 0.20583E+00_JPRB, 0.20239E+00_JPRB, 0.20028E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19957E+00_JPRB, 0.20125E+00_JPRB, 0.20231E+00_JPRB, 0.20316E+00_JPRB, 0.20441E+00_JPRB, 0.20355E+00_JPRB, 0.20270E+00_JPRB ,&
&  0.20148E+00_JPRB, 0.20013E+00_JPRB, 0.19872E+00_JPRB, 0.19750E+00_JPRB, 0.19628E+00_JPRB, 0.19447E+00_JPRB, 0.19291E+00_JPRB ,&
&  0.19138E+00_JPRB, 0.18940E+00_JPRB, 0.18675E+00_JPRB, 0.18499E+00_JPRB, 0.18294E+00_JPRB, 0.18119E+00_JPRB, 0.17944E+00_JPRB ,&
&  0.17814E+00_JPRB, 0.17695E+00_JPRB, 0.17556E+00_JPRB, 0.17387E+00_JPRB, 0.17220E+00_JPRB, 0.17072E+00_JPRB, 0.16931E+00_JPRB ,&
&  0.16854E+00_JPRB, 0.16774E+00_JPRB, 0.16695E+00_JPRB, 0.16611E+00_JPRB, 0.16569E+00_JPRB, 0.16525E+00_JPRB, 0.16485E+00_JPRB ,&
&  0.16452E+00_JPRB, 0.16433E+00_JPRB, 0.16435E+00_JPRB, 0.16443E+00_JPRB, 0.16453E+00_JPRB, 0.16489E+00_JPRB, 0.16533E+00_JPRB ,&
&  0.16593E+00_JPRB, 0.16667E+00_JPRB, 0.16768E+00_JPRB, 0.16874E+00_JPRB, 0.16969E+00_JPRB, 0.17045E+00_JPRB, 0.17145E+00_JPRB ,&
&  0.17304E+00_JPRB, 0.17488E+00_JPRB, 0.17689E+00_JPRB, 0.17844E+00_JPRB, 0.18062E+00_JPRB, 0.18260E+00_JPRB, 0.18457E+00_JPRB ,&
&  0.18652E+00_JPRB, 0.18884E+00_JPRB, 0.19142E+00_JPRB, 0.19418E+00_JPRB, 0.19736E+00_JPRB, 0.20012E+00_JPRB, 0.20268E+00_JPRB ,&
&  0.20530E+00_JPRB, 0.20900E+00_JPRB, 0.21101E+00_JPRB, 0.21218E+00_JPRB, 0.21369E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.19454E+00_JPRB, 0.19470E+00_JPRB, 0.19514E+00_JPRB, 0.19565E+00_JPRB, 0.19616E+00_JPRB, 0.19689E+00_JPRB, 0.19711E+00_JPRB ,&
&  0.19714E+00_JPRB, 0.19706E+00_JPRB, 0.19741E+00_JPRB, 0.19780E+00_JPRB, 0.19805E+00_JPRB, 0.19805E+00_JPRB, 0.19766E+00_JPRB ,&
&  0.19699E+00_JPRB, 0.19618E+00_JPRB, 0.19496E+00_JPRB, 0.19441E+00_JPRB, 0.19401E+00_JPRB, 0.19361E+00_JPRB, 0.19284E+00_JPRB ,&
&  0.19217E+00_JPRB, 0.19097E+00_JPRB, 0.18932E+00_JPRB, 0.18802E+00_JPRB, 0.18779E+00_JPRB, 0.18803E+00_JPRB, 0.18862E+00_JPRB ,&
&  0.18908E+00_JPRB, 0.18937E+00_JPRB, 0.18947E+00_JPRB, 0.18963E+00_JPRB, 0.18986E+00_JPRB, 0.18971E+00_JPRB, 0.18944E+00_JPRB ,&
&  0.18910E+00_JPRB, 0.18856E+00_JPRB, 0.18804E+00_JPRB, 0.18757E+00_JPRB, 0.18714E+00_JPRB, 0.18664E+00_JPRB, 0.18605E+00_JPRB ,&
&  0.18537E+00_JPRB, 0.18452E+00_JPRB, 0.18352E+00_JPRB, 0.18272E+00_JPRB, 0.18188E+00_JPRB, 0.18121E+00_JPRB, 0.18040E+00_JPRB ,&
&  0.17972E+00_JPRB, 0.17910E+00_JPRB, 0.17846E+00_JPRB, 0.17787E+00_JPRB, 0.17732E+00_JPRB, 0.17678E+00_JPRB, 0.17625E+00_JPRB ,&
&  0.17575E+00_JPRB, 0.17534E+00_JPRB, 0.17487E+00_JPRB, 0.17444E+00_JPRB, 0.17414E+00_JPRB, 0.17390E+00_JPRB, 0.17369E+00_JPRB ,&
&  0.17351E+00_JPRB, 0.17345E+00_JPRB, 0.17343E+00_JPRB, 0.17340E+00_JPRB, 0.17340E+00_JPRB, 0.17351E+00_JPRB, 0.17362E+00_JPRB ,&
&  0.17379E+00_JPRB, 0.17390E+00_JPRB, 0.17403E+00_JPRB, 0.17422E+00_JPRB, 0.17443E+00_JPRB, 0.17467E+00_JPRB, 0.17500E+00_JPRB ,&
&  0.17540E+00_JPRB, 0.17588E+00_JPRB, 0.17638E+00_JPRB, 0.17670E+00_JPRB, 0.17737E+00_JPRB, 0.17812E+00_JPRB, 0.17900E+00_JPRB ,&
&  0.17996E+00_JPRB, 0.18111E+00_JPRB, 0.18235E+00_JPRB, 0.18379E+00_JPRB, 0.18527E+00_JPRB, 0.18622E+00_JPRB, 0.18717E+00_JPRB ,&
&  0.18817E+00_JPRB, 0.18933E+00_JPRB, 0.19044E+00_JPRB, 0.19157E+00_JPRB, 0.19295E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB, 0.18000E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB, 0.35758E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.36380E+00_JPRB, 0.36352E+00_JPRB, 0.36324E+00_JPRB, 0.36293E+00_JPRB, 0.36252E+00_JPRB, 0.36211E+00_JPRB, 0.36176E+00_JPRB ,&
&  0.36141E+00_JPRB, 0.36117E+00_JPRB, 0.36074E+00_JPRB, 0.36030E+00_JPRB, 0.35986E+00_JPRB, 0.35948E+00_JPRB, 0.35906E+00_JPRB ,&
&  0.35864E+00_JPRB, 0.35822E+00_JPRB, 0.35794E+00_JPRB, 0.35758E+00_JPRB, 0.35719E+00_JPRB, 0.35683E+00_JPRB, 0.35652E+00_JPRB ,&
&  0.35623E+00_JPRB, 0.35588E+00_JPRB, 0.35552E+00_JPRB, 0.35506E+00_JPRB, 0.35477E+00_JPRB, 0.35434E+00_JPRB, 0.35399E+00_JPRB ,&
&  0.35359E+00_JPRB, 0.35330E+00_JPRB, 0.35300E+00_JPRB, 0.35270E+00_JPRB, 0.35234E+00_JPRB, 0.35228E+00_JPRB, 0.35206E+00_JPRB ,&
&  0.35196E+00_JPRB, 0.35183E+00_JPRB, 0.35184E+00_JPRB, 0.35180E+00_JPRB, 0.35180E+00_JPRB, 0.35182E+00_JPRB, 0.35207E+00_JPRB ,&
&  0.35226E+00_JPRB, 0.35250E+00_JPRB, 0.35288E+00_JPRB, 0.35322E+00_JPRB, 0.35352E+00_JPRB, 0.35383E+00_JPRB, 0.35418E+00_JPRB ,&
&  0.35468E+00_JPRB, 0.35516E+00_JPRB, 0.35563E+00_JPRB, 0.35610E+00_JPRB, 0.35656E+00_JPRB, 0.35691E+00_JPRB, 0.35721E+00_JPRB ,&
&  0.35735E+00_JPRB, 0.35758E+00_JPRB, 0.35779E+00_JPRB, 0.35802E+00_JPRB, 0.35806E+00_JPRB, 0.35819E+00_JPRB, 0.35823E+00_JPRB ,&
&  0.35826E+00_JPRB, 0.35808E+00_JPRB, 0.35809E+00_JPRB, 0.35799E+00_JPRB, 0.35801E+00_JPRB, 0.35796E+00_JPRB, 0.35812E+00_JPRB ,&
&  0.35822E+00_JPRB, 0.35850E+00_JPRB, 0.35881E+00_JPRB, 0.35926E+00_JPRB, 0.35959E+00_JPRB, 0.35999E+00_JPRB, 0.36033E+00_JPRB ,&
&  0.36071E+00_JPRB, 0.36110E+00_JPRB, 0.36153E+00_JPRB, 0.36180E+00_JPRB, 0.36204E+00_JPRB, 0.36234E+00_JPRB, 0.36267E+00_JPRB ,&
&  0.36311E+00_JPRB, 0.36341E+00_JPRB, 0.36372E+00_JPRB, 0.36398E+00_JPRB, 0.36406E+00_JPRB, 0.36417E+00_JPRB, 0.36418E+00_JPRB ,&
&  0.36416E+00_JPRB, 0.36413E+00_JPRB, 0.36407E+00_JPRB, 0.36403E+00_JPRB, 0.36397E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.36926E+00_JPRB, 0.36917E+00_JPRB, 0.36895E+00_JPRB, 0.36851E+00_JPRB, 0.36795E+00_JPRB, 0.36727E+00_JPRB, 0.36659E+00_JPRB ,&
&  0.36581E+00_JPRB, 0.36504E+00_JPRB, 0.36411E+00_JPRB, 0.36319E+00_JPRB, 0.36224E+00_JPRB, 0.36139E+00_JPRB, 0.36050E+00_JPRB ,&
&  0.35966E+00_JPRB, 0.35882E+00_JPRB, 0.35828E+00_JPRB, 0.35770E+00_JPRB, 0.35706E+00_JPRB, 0.35623E+00_JPRB, 0.35559E+00_JPRB ,&
&  0.35488E+00_JPRB, 0.35400E+00_JPRB, 0.35304E+00_JPRB, 0.35124E+00_JPRB, 0.34974E+00_JPRB, 0.34836E+00_JPRB, 0.34718E+00_JPRB ,&
&  0.34611E+00_JPRB, 0.34509E+00_JPRB, 0.34430E+00_JPRB, 0.34381E+00_JPRB, 0.34449E+00_JPRB, 0.34462E+00_JPRB, 0.34446E+00_JPRB ,&
&  0.34417E+00_JPRB, 0.34296E+00_JPRB, 0.34256E+00_JPRB, 0.34241E+00_JPRB, 0.34239E+00_JPRB, 0.34312E+00_JPRB, 0.34371E+00_JPRB ,&
&  0.34435E+00_JPRB, 0.34486E+00_JPRB, 0.34596E+00_JPRB, 0.34700E+00_JPRB, 0.34796E+00_JPRB, 0.34902E+00_JPRB, 0.34959E+00_JPRB ,&
&  0.35023E+00_JPRB, 0.35122E+00_JPRB, 0.35218E+00_JPRB, 0.35317E+00_JPRB, 0.35375E+00_JPRB, 0.35428E+00_JPRB, 0.35468E+00_JPRB ,&
&  0.35501E+00_JPRB, 0.35549E+00_JPRB, 0.35604E+00_JPRB, 0.35657E+00_JPRB, 0.35694E+00_JPRB, 0.35722E+00_JPRB, 0.35737E+00_JPRB ,&
&  0.35740E+00_JPRB, 0.35710E+00_JPRB, 0.35692E+00_JPRB, 0.35671E+00_JPRB, 0.35638E+00_JPRB, 0.35600E+00_JPRB, 0.35585E+00_JPRB ,&
&  0.35571E+00_JPRB, 0.35546E+00_JPRB, 0.35510E+00_JPRB, 0.35524E+00_JPRB, 0.35545E+00_JPRB, 0.35553E+00_JPRB, 0.35549E+00_JPRB ,&
&  0.35651E+00_JPRB, 0.35793E+00_JPRB, 0.35960E+00_JPRB, 0.36179E+00_JPRB, 0.36344E+00_JPRB, 0.36451E+00_JPRB, 0.36526E+00_JPRB ,&
&  0.36551E+00_JPRB, 0.36580E+00_JPRB, 0.36611E+00_JPRB, 0.36665E+00_JPRB, 0.36705E+00_JPRB, 0.36775E+00_JPRB, 0.36868E+00_JPRB ,&
&  0.36933E+00_JPRB, 0.36957E+00_JPRB, 0.36961E+00_JPRB, 0.36951E+00_JPRB, 0.36937E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.37151E+00_JPRB, 0.37091E+00_JPRB, 0.37034E+00_JPRB, 0.36935E+00_JPRB, 0.36874E+00_JPRB, 0.36786E+00_JPRB, 0.36707E+00_JPRB ,&
&  0.36610E+00_JPRB, 0.36498E+00_JPRB, 0.36419E+00_JPRB, 0.36356E+00_JPRB, 0.36289E+00_JPRB, 0.36228E+00_JPRB, 0.36157E+00_JPRB ,&
&  0.36083E+00_JPRB, 0.35995E+00_JPRB, 0.35965E+00_JPRB, 0.35909E+00_JPRB, 0.35843E+00_JPRB, 0.35767E+00_JPRB, 0.35555E+00_JPRB ,&
&  0.35360E+00_JPRB, 0.35127E+00_JPRB, 0.34895E+00_JPRB, 0.34703E+00_JPRB, 0.34578E+00_JPRB, 0.33553E+00_JPRB, 0.33389E+00_JPRB ,&
&  0.33802E+00_JPRB, 0.34028E+00_JPRB, 0.34287E+00_JPRB, 0.34555E+00_JPRB, 0.34023E+00_JPRB, 0.33360E+00_JPRB, 0.32975E+00_JPRB ,&
&  0.32027E+00_JPRB, 0.32400E+00_JPRB, 0.32585E+00_JPRB, 0.32759E+00_JPRB, 0.33027E+00_JPRB, 0.33027E+00_JPRB, 0.33197E+00_JPRB ,&
&  0.33828E+00_JPRB, 0.33824E+00_JPRB, 0.33898E+00_JPRB, 0.33982E+00_JPRB, 0.34064E+00_JPRB, 0.34145E+00_JPRB, 0.34308E+00_JPRB ,&
&  0.34433E+00_JPRB, 0.34580E+00_JPRB, 0.34740E+00_JPRB, 0.34815E+00_JPRB, 0.34899E+00_JPRB, 0.35014E+00_JPRB, 0.35121E+00_JPRB ,&
&  0.35173E+00_JPRB, 0.35242E+00_JPRB, 0.35322E+00_JPRB, 0.35394E+00_JPRB, 0.35494E+00_JPRB, 0.35554E+00_JPRB, 0.35598E+00_JPRB ,&
&  0.35626E+00_JPRB, 0.35610E+00_JPRB, 0.35597E+00_JPRB, 0.35599E+00_JPRB, 0.35582E+00_JPRB, 0.35553E+00_JPRB, 0.35557E+00_JPRB ,&
&  0.35583E+00_JPRB, 0.35599E+00_JPRB, 0.35639E+00_JPRB, 0.35708E+00_JPRB, 0.35889E+00_JPRB, 0.36141E+00_JPRB, 0.36638E+00_JPRB ,&
&  0.37042E+00_JPRB, 0.37391E+00_JPRB, 0.37517E+00_JPRB, 0.37632E+00_JPRB, 0.37715E+00_JPRB, 0.37725E+00_JPRB, 0.37715E+00_JPRB ,&
&  0.37645E+00_JPRB, 0.37529E+00_JPRB, 0.37416E+00_JPRB, 0.37304E+00_JPRB, 0.37240E+00_JPRB, 0.37195E+00_JPRB, 0.37203E+00_JPRB ,&
&  0.37249E+00_JPRB, 0.37324E+00_JPRB, 0.37346E+00_JPRB, 0.37284E+00_JPRB, 0.37206E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.37627E+00_JPRB, 0.37472E+00_JPRB, 0.37415E+00_JPRB, 0.37281E+00_JPRB, 0.37140E+00_JPRB, 0.36963E+00_JPRB, 0.36821E+00_JPRB ,&
&  0.36662E+00_JPRB, 0.36531E+00_JPRB, 0.36425E+00_JPRB, 0.36334E+00_JPRB, 0.36300E+00_JPRB, 0.36258E+00_JPRB, 0.36238E+00_JPRB ,&
&  0.36200E+00_JPRB, 0.36140E+00_JPRB, 0.36114E+00_JPRB, 0.36067E+00_JPRB, 0.36013E+00_JPRB, 0.35945E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35465E+00_JPRB, 0.35090E+00_JPRB, 0.34189E+00_JPRB, 0.34378E+00_JPRB, 0.34255E+00_JPRB, 0.34135E+00_JPRB, 0.34185E+00_JPRB ,&
&  0.33726E+00_JPRB, 0.33606E+00_JPRB, 0.33094E+00_JPRB, 0.32616E+00_JPRB, 0.32428E+00_JPRB, 0.31935E+00_JPRB, 0.31920E+00_JPRB ,&
&  0.31741E+00_JPRB, 0.31656E+00_JPRB, 0.31712E+00_JPRB, 0.31692E+00_JPRB, 0.31987E+00_JPRB, 0.32143E+00_JPRB, 0.32374E+00_JPRB ,&
&  0.32767E+00_JPRB, 0.33082E+00_JPRB, 0.33291E+00_JPRB, 0.32954E+00_JPRB, 0.33119E+00_JPRB, 0.33280E+00_JPRB, 0.33484E+00_JPRB ,&
&  0.33669E+00_JPRB, 0.33789E+00_JPRB, 0.33905E+00_JPRB, 0.33988E+00_JPRB, 0.34110E+00_JPRB, 0.34295E+00_JPRB, 0.34491E+00_JPRB ,&
&  0.34720E+00_JPRB, 0.34882E+00_JPRB, 0.34992E+00_JPRB, 0.35102E+00_JPRB, 0.35218E+00_JPRB, 0.35321E+00_JPRB, 0.35443E+00_JPRB ,&
&  0.35579E+00_JPRB, 0.35751E+00_JPRB, 0.35932E+00_JPRB, 0.36124E+00_JPRB, 0.36301E+00_JPRB, 0.36537E+00_JPRB, 0.36823E+00_JPRB ,&
&  0.37249E+00_JPRB, 0.37646E+00_JPRB, 0.38277E+00_JPRB, 0.38654E+00_JPRB, 0.39438E+00_JPRB, 0.40220E+00_JPRB, 0.40700E+00_JPRB ,&
&  0.40616E+00_JPRB, 0.40390E+00_JPRB, 0.40135E+00_JPRB, 0.39848E+00_JPRB, 0.39606E+00_JPRB, 0.39327E+00_JPRB, 0.39136E+00_JPRB ,&
&  0.38964E+00_JPRB, 0.38794E+00_JPRB, 0.38629E+00_JPRB, 0.38458E+00_JPRB, 0.38315E+00_JPRB, 0.38147E+00_JPRB, 0.38029E+00_JPRB ,&
&  0.37977E+00_JPRB, 0.37998E+00_JPRB, 0.37991E+00_JPRB, 0.37925E+00_JPRB, 0.37785E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.38153E+00_JPRB, 0.38114E+00_JPRB, 0.38188E+00_JPRB, 0.38122E+00_JPRB, 0.37964E+00_JPRB, 0.37805E+00_JPRB, 0.37491E+00_JPRB ,&
&  0.37186E+00_JPRB, 0.36893E+00_JPRB, 0.36642E+00_JPRB, 0.36466E+00_JPRB, 0.36325E+00_JPRB, 0.36218E+00_JPRB, 0.36188E+00_JPRB ,&
&  0.36129E+00_JPRB, 0.36142E+00_JPRB, 0.36159E+00_JPRB, 0.36181E+00_JPRB, 0.36245E+00_JPRB, 0.36276E+00_JPRB, 0.36271E+00_JPRB ,&
&  0.36082E+00_JPRB, 0.35612E+00_JPRB, 0.34973E+00_JPRB, 0.35133E+00_JPRB, 0.35219E+00_JPRB, 0.35373E+00_JPRB, 0.35537E+00_JPRB ,&
&  0.35452E+00_JPRB, 0.34024E+00_JPRB, 0.34446E+00_JPRB, 0.33766E+00_JPRB, 0.33392E+00_JPRB, 0.32961E+00_JPRB, 0.31943E+00_JPRB ,&
&  0.31585E+00_JPRB, 0.30984E+00_JPRB, 0.30648E+00_JPRB, 0.30653E+00_JPRB, 0.30800E+00_JPRB, 0.31205E+00_JPRB, 0.31711E+00_JPRB ,&
&  0.31810E+00_JPRB, 0.32568E+00_JPRB, 0.32672E+00_JPRB, 0.32656E+00_JPRB, 0.32844E+00_JPRB, 0.33055E+00_JPRB, 0.32987E+00_JPRB ,&
&  0.33070E+00_JPRB, 0.33046E+00_JPRB, 0.32978E+00_JPRB, 0.33134E+00_JPRB, 0.33539E+00_JPRB, 0.33747E+00_JPRB, 0.33958E+00_JPRB ,&
&  0.34227E+00_JPRB, 0.34390E+00_JPRB, 0.34495E+00_JPRB, 0.34642E+00_JPRB, 0.34833E+00_JPRB, 0.35024E+00_JPRB, 0.35306E+00_JPRB ,&
&  0.35618E+00_JPRB, 0.36140E+00_JPRB, 0.36625E+00_JPRB, 0.37340E+00_JPRB, 0.38006E+00_JPRB, 0.38884E+00_JPRB, 0.39661E+00_JPRB ,&
&  0.40472E+00_JPRB, 0.41138E+00_JPRB, 0.41906E+00_JPRB, 0.42632E+00_JPRB, 0.43241E+00_JPRB, 0.43086E+00_JPRB, 0.42425E+00_JPRB ,&
&  0.41825E+00_JPRB, 0.40971E+00_JPRB, 0.40473E+00_JPRB, 0.40087E+00_JPRB, 0.39830E+00_JPRB, 0.39723E+00_JPRB, 0.39609E+00_JPRB ,&
&  0.39577E+00_JPRB, 0.39478E+00_JPRB, 0.39352E+00_JPRB, 0.39154E+00_JPRB, 0.38991E+00_JPRB, 0.38807E+00_JPRB, 0.38684E+00_JPRB ,&
&  0.38495E+00_JPRB, 0.38422E+00_JPRB, 0.38309E+00_JPRB, 0.38239E+00_JPRB, 0.38183E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.38663E+00_JPRB, 0.38861E+00_JPRB, 0.38850E+00_JPRB, 0.38751E+00_JPRB, 0.38410E+00_JPRB, 0.38127E+00_JPRB, 0.37681E+00_JPRB ,&
&  0.37258E+00_JPRB, 0.36841E+00_JPRB, 0.36473E+00_JPRB, 0.36241E+00_JPRB, 0.36077E+00_JPRB, 0.35970E+00_JPRB, 0.35930E+00_JPRB ,&
&  0.35935E+00_JPRB, 0.35988E+00_JPRB, 0.36037E+00_JPRB, 0.36093E+00_JPRB, 0.36220E+00_JPRB, 0.36374E+00_JPRB, 0.36532E+00_JPRB ,&
&  0.36575E+00_JPRB, 0.36375E+00_JPRB, 0.35779E+00_JPRB, 0.35764E+00_JPRB, 0.36146E+00_JPRB, 0.35952E+00_JPRB, 0.36689E+00_JPRB ,&
&  0.37176E+00_JPRB, 0.37413E+00_JPRB, 0.36913E+00_JPRB, 0.36589E+00_JPRB, 0.35931E+00_JPRB, 0.32876E+00_JPRB, 0.32605E+00_JPRB ,&
&  0.32026E+00_JPRB, 0.31213E+00_JPRB, 0.30848E+00_JPRB, 0.30555E+00_JPRB, 0.30508E+00_JPRB, 0.30994E+00_JPRB, 0.31697E+00_JPRB ,&
&  0.31905E+00_JPRB, 0.33098E+00_JPRB, 0.33126E+00_JPRB, 0.33109E+00_JPRB, 0.33240E+00_JPRB, 0.33542E+00_JPRB, 0.33526E+00_JPRB ,&
&  0.33637E+00_JPRB, 0.33678E+00_JPRB, 0.33647E+00_JPRB, 0.33880E+00_JPRB, 0.34116E+00_JPRB, 0.34299E+00_JPRB, 0.34607E+00_JPRB ,&
&  0.35222E+00_JPRB, 0.35649E+00_JPRB, 0.35922E+00_JPRB, 0.36188E+00_JPRB, 0.36616E+00_JPRB, 0.37074E+00_JPRB, 0.37640E+00_JPRB ,&
&  0.38094E+00_JPRB, 0.38569E+00_JPRB, 0.39150E+00_JPRB, 0.39648E+00_JPRB, 0.40168E+00_JPRB, 0.40698E+00_JPRB, 0.41328E+00_JPRB ,&
&  0.41617E+00_JPRB, 0.42103E+00_JPRB, 0.42690E+00_JPRB, 0.43838E+00_JPRB, 0.43957E+00_JPRB, 0.43104E+00_JPRB, 0.42409E+00_JPRB ,&
&  0.41939E+00_JPRB, 0.41934E+00_JPRB, 0.41500E+00_JPRB, 0.41270E+00_JPRB, 0.40997E+00_JPRB, 0.40935E+00_JPRB, 0.40760E+00_JPRB ,&
&  0.40658E+00_JPRB, 0.40443E+00_JPRB, 0.40133E+00_JPRB, 0.39774E+00_JPRB, 0.39492E+00_JPRB, 0.39189E+00_JPRB, 0.38953E+00_JPRB ,&
&  0.38677E+00_JPRB, 0.38558E+00_JPRB, 0.38472E+00_JPRB, 0.38475E+00_JPRB, 0.38453E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.39086E+00_JPRB, 0.39275E+00_JPRB, 0.38996E+00_JPRB, 0.38867E+00_JPRB, 0.38344E+00_JPRB, 0.37871E+00_JPRB, 0.37256E+00_JPRB ,&
&  0.36815E+00_JPRB, 0.36500E+00_JPRB, 0.36181E+00_JPRB, 0.36084E+00_JPRB, 0.36069E+00_JPRB, 0.36093E+00_JPRB, 0.36223E+00_JPRB ,&
&  0.36362E+00_JPRB, 0.36216E+00_JPRB, 0.36330E+00_JPRB, 0.36555E+00_JPRB, 0.36725E+00_JPRB, 0.36880E+00_JPRB, 0.37027E+00_JPRB ,&
&  0.36944E+00_JPRB, 0.36866E+00_JPRB, 0.36789E+00_JPRB, 0.36452E+00_JPRB, 0.36343E+00_JPRB, 0.36901E+00_JPRB, 0.37673E+00_JPRB ,&
&  0.37364E+00_JPRB, 0.37448E+00_JPRB, 0.37864E+00_JPRB, 0.37944E+00_JPRB, 0.36999E+00_JPRB, 0.35529E+00_JPRB, 0.33759E+00_JPRB ,&
&  0.32543E+00_JPRB, 0.31124E+00_JPRB, 0.30507E+00_JPRB, 0.30200E+00_JPRB, 0.30478E+00_JPRB, 0.31059E+00_JPRB, 0.32127E+00_JPRB ,&
&  0.32843E+00_JPRB, 0.33214E+00_JPRB, 0.33589E+00_JPRB, 0.33982E+00_JPRB, 0.34189E+00_JPRB, 0.34526E+00_JPRB, 0.34700E+00_JPRB ,&
&  0.35086E+00_JPRB, 0.35281E+00_JPRB, 0.35396E+00_JPRB, 0.35732E+00_JPRB, 0.36307E+00_JPRB, 0.36885E+00_JPRB, 0.37868E+00_JPRB ,&
&  0.40280E+00_JPRB, 0.41182E+00_JPRB, 0.41849E+00_JPRB, 0.42175E+00_JPRB, 0.42580E+00_JPRB, 0.42727E+00_JPRB, 0.42218E+00_JPRB ,&
&  0.42102E+00_JPRB, 0.42571E+00_JPRB, 0.42531E+00_JPRB, 0.42542E+00_JPRB, 0.42660E+00_JPRB, 0.42755E+00_JPRB, 0.42943E+00_JPRB ,&
&  0.42538E+00_JPRB, 0.43160E+00_JPRB, 0.46818E+00_JPRB, 0.47965E+00_JPRB, 0.47186E+00_JPRB, 0.45867E+00_JPRB, 0.44770E+00_JPRB ,&
&  0.43977E+00_JPRB, 0.43316E+00_JPRB, 0.42765E+00_JPRB, 0.42325E+00_JPRB, 0.42012E+00_JPRB, 0.41831E+00_JPRB, 0.41809E+00_JPRB ,&
&  0.41441E+00_JPRB, 0.41011E+00_JPRB, 0.40543E+00_JPRB, 0.40190E+00_JPRB, 0.39862E+00_JPRB, 0.39504E+00_JPRB, 0.39267E+00_JPRB ,&
&  0.39030E+00_JPRB, 0.38986E+00_JPRB, 0.38972E+00_JPRB, 0.39035E+00_JPRB, 0.39070E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.39118E+00_JPRB, 0.39482E+00_JPRB, 0.39663E+00_JPRB, 0.39196E+00_JPRB, 0.38183E+00_JPRB, 0.37277E+00_JPRB, 0.36629E+00_JPRB ,&
&  0.36446E+00_JPRB, 0.36387E+00_JPRB, 0.36324E+00_JPRB, 0.36308E+00_JPRB, 0.36350E+00_JPRB, 0.36457E+00_JPRB, 0.36628E+00_JPRB ,&
&  0.36862E+00_JPRB, 0.37065E+00_JPRB, 0.37134E+00_JPRB, 0.37105E+00_JPRB, 0.37205E+00_JPRB, 0.37463E+00_JPRB, 0.37756E+00_JPRB ,&
&  0.37885E+00_JPRB, 0.37808E+00_JPRB, 0.37874E+00_JPRB, 0.37785E+00_JPRB, 0.37735E+00_JPRB, 0.37632E+00_JPRB, 0.37666E+00_JPRB ,&
&  0.38141E+00_JPRB, 0.38562E+00_JPRB, 0.38167E+00_JPRB, 0.38315E+00_JPRB, 0.38724E+00_JPRB, 0.37400E+00_JPRB, 0.35248E+00_JPRB ,&
&  0.32972E+00_JPRB, 0.31248E+00_JPRB, 0.30515E+00_JPRB, 0.30357E+00_JPRB, 0.30862E+00_JPRB, 0.32011E+00_JPRB, 0.33427E+00_JPRB ,&
&  0.33821E+00_JPRB, 0.34233E+00_JPRB, 0.34889E+00_JPRB, 0.35372E+00_JPRB, 0.35652E+00_JPRB, 0.35798E+00_JPRB, 0.35950E+00_JPRB ,&
&  0.36247E+00_JPRB, 0.36818E+00_JPRB, 0.37813E+00_JPRB, 0.39142E+00_JPRB, 0.41197E+00_JPRB, 0.42574E+00_JPRB, 0.45032E+00_JPRB ,&
&  0.48810E+00_JPRB, 0.49046E+00_JPRB, 0.49230E+00_JPRB, 0.48890E+00_JPRB, 0.48501E+00_JPRB, 0.47854E+00_JPRB, 0.47375E+00_JPRB ,&
&  0.46946E+00_JPRB, 0.46473E+00_JPRB, 0.45805E+00_JPRB, 0.45159E+00_JPRB, 0.45066E+00_JPRB, 0.45349E+00_JPRB, 0.46227E+00_JPRB ,&
&  0.47769E+00_JPRB, 0.50705E+00_JPRB, 0.58696E+00_JPRB, 0.53390E+00_JPRB, 0.50558E+00_JPRB, 0.49246E+00_JPRB, 0.47737E+00_JPRB ,&
&  0.46586E+00_JPRB, 0.45870E+00_JPRB, 0.45420E+00_JPRB, 0.44808E+00_JPRB, 0.43712E+00_JPRB, 0.42899E+00_JPRB, 0.42309E+00_JPRB ,&
&  0.41760E+00_JPRB, 0.41229E+00_JPRB, 0.40863E+00_JPRB, 0.40594E+00_JPRB, 0.40271E+00_JPRB, 0.39981E+00_JPRB, 0.39791E+00_JPRB ,&
&  0.39484E+00_JPRB, 0.39281E+00_JPRB, 0.39212E+00_JPRB, 0.39097E+00_JPRB, 0.39070E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.39412E+00_JPRB, 0.39813E+00_JPRB, 0.39607E+00_JPRB, 0.39179E+00_JPRB, 0.38136E+00_JPRB, 0.37263E+00_JPRB, 0.36639E+00_JPRB ,&
&  0.36593E+00_JPRB, 0.36368E+00_JPRB, 0.36288E+00_JPRB, 0.36320E+00_JPRB, 0.36439E+00_JPRB, 0.36675E+00_JPRB, 0.36849E+00_JPRB ,&
&  0.36967E+00_JPRB, 0.37171E+00_JPRB, 0.37413E+00_JPRB, 0.37589E+00_JPRB, 0.37755E+00_JPRB, 0.37969E+00_JPRB, 0.38217E+00_JPRB ,&
&  0.38499E+00_JPRB, 0.38628E+00_JPRB, 0.38709E+00_JPRB, 0.38823E+00_JPRB, 0.39182E+00_JPRB, 0.39680E+00_JPRB, 0.39870E+00_JPRB ,&
&  0.39386E+00_JPRB, 0.39085E+00_JPRB, 0.39420E+00_JPRB, 0.39559E+00_JPRB, 0.39875E+00_JPRB, 0.38976E+00_JPRB, 0.36449E+00_JPRB ,&
&  0.33825E+00_JPRB, 0.32067E+00_JPRB, 0.31798E+00_JPRB, 0.32502E+00_JPRB, 0.33814E+00_JPRB, 0.34717E+00_JPRB, 0.35431E+00_JPRB ,&
&  0.36017E+00_JPRB, 0.36257E+00_JPRB, 0.36501E+00_JPRB, 0.37033E+00_JPRB, 0.37414E+00_JPRB, 0.37559E+00_JPRB, 0.37798E+00_JPRB ,&
&  0.38410E+00_JPRB, 0.39348E+00_JPRB, 0.39835E+00_JPRB, 0.39459E+00_JPRB, 0.41195E+00_JPRB, 0.47060E+00_JPRB, 0.51581E+00_JPRB ,&
&  0.52757E+00_JPRB, 0.52976E+00_JPRB, 0.53125E+00_JPRB, 0.54379E+00_JPRB, 0.54455E+00_JPRB, 0.54246E+00_JPRB, 0.53461E+00_JPRB ,&
&  0.52138E+00_JPRB, 0.50663E+00_JPRB, 0.49516E+00_JPRB, 0.49237E+00_JPRB, 0.50265E+00_JPRB, 0.52439E+00_JPRB, 0.54677E+00_JPRB ,&
&  0.56002E+00_JPRB, 0.54798E+00_JPRB, 0.50874E+00_JPRB, 0.47392E+00_JPRB, 0.46371E+00_JPRB, 0.46682E+00_JPRB, 0.47176E+00_JPRB ,&
&  0.46904E+00_JPRB, 0.46521E+00_JPRB, 0.46146E+00_JPRB, 0.45651E+00_JPRB, 0.43842E+00_JPRB, 0.42670E+00_JPRB, 0.41720E+00_JPRB ,&
&  0.41013E+00_JPRB, 0.40526E+00_JPRB, 0.40632E+00_JPRB, 0.40371E+00_JPRB, 0.40126E+00_JPRB, 0.39820E+00_JPRB, 0.39320E+00_JPRB ,&
&  0.38969E+00_JPRB, 0.38780E+00_JPRB, 0.38648E+00_JPRB, 0.38663E+00_JPRB, 0.38856E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.40105E+00_JPRB, 0.40118E+00_JPRB, 0.40126E+00_JPRB, 0.39746E+00_JPRB, 0.39427E+00_JPRB, 0.39305E+00_JPRB, 0.39239E+00_JPRB ,&
&  0.40125E+00_JPRB, 0.38763E+00_JPRB, 0.37547E+00_JPRB, 0.36630E+00_JPRB, 0.36113E+00_JPRB, 0.35938E+00_JPRB, 0.36250E+00_JPRB ,&
&  0.36767E+00_JPRB, 0.37277E+00_JPRB, 0.37685E+00_JPRB, 0.38079E+00_JPRB, 0.38452E+00_JPRB, 0.38686E+00_JPRB, 0.38916E+00_JPRB ,&
&  0.39235E+00_JPRB, 0.39563E+00_JPRB, 0.39905E+00_JPRB, 0.40503E+00_JPRB, 0.41028E+00_JPRB, 0.41508E+00_JPRB, 0.41495E+00_JPRB ,&
&  0.41085E+00_JPRB, 0.40616E+00_JPRB, 0.40571E+00_JPRB, 0.40757E+00_JPRB, 0.40837E+00_JPRB, 0.40071E+00_JPRB, 0.37464E+00_JPRB ,&
&  0.34976E+00_JPRB, 0.34032E+00_JPRB, 0.35037E+00_JPRB, 0.36837E+00_JPRB, 0.37395E+00_JPRB, 0.37875E+00_JPRB, 0.38012E+00_JPRB ,&
&  0.37760E+00_JPRB, 0.37693E+00_JPRB, 0.38293E+00_JPRB, 0.39169E+00_JPRB, 0.39438E+00_JPRB, 0.39661E+00_JPRB, 0.40064E+00_JPRB ,&
&  0.41118E+00_JPRB, 0.42240E+00_JPRB, 0.42914E+00_JPRB, 0.42418E+00_JPRB, 0.47769E+00_JPRB, 0.67209E+00_JPRB, 0.66364E+00_JPRB ,&
&  0.66170E+00_JPRB, 0.65614E+00_JPRB, 0.65874E+00_JPRB, 0.65890E+00_JPRB, 0.63731E+00_JPRB, 0.62178E+00_JPRB, 0.61048E+00_JPRB ,&
&  0.58751E+00_JPRB, 0.57485E+00_JPRB, 0.57184E+00_JPRB, 0.59133E+00_JPRB, 0.60755E+00_JPRB, 0.61764E+00_JPRB, 0.62608E+00_JPRB ,&
&  0.62004E+00_JPRB, 0.58983E+00_JPRB, 0.55291E+00_JPRB, 0.51951E+00_JPRB, 0.50264E+00_JPRB, 0.49213E+00_JPRB, 0.48572E+00_JPRB ,&
&  0.47830E+00_JPRB, 0.47531E+00_JPRB, 0.47493E+00_JPRB, 0.47223E+00_JPRB, 0.45551E+00_JPRB, 0.43512E+00_JPRB, 0.42305E+00_JPRB ,&
&  0.41532E+00_JPRB, 0.40814E+00_JPRB, 0.40211E+00_JPRB, 0.39725E+00_JPRB, 0.39768E+00_JPRB, 0.39774E+00_JPRB, 0.39392E+00_JPRB ,&
&  0.39404E+00_JPRB, 0.39106E+00_JPRB, 0.39115E+00_JPRB, 0.39314E+00_JPRB, 0.39861E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.40510E+00_JPRB, 0.40844E+00_JPRB, 0.41025E+00_JPRB, 0.41004E+00_JPRB, 0.41219E+00_JPRB, 0.41273E+00_JPRB, 0.40682E+00_JPRB ,&
&  0.42167E+00_JPRB, 0.39184E+00_JPRB, 0.37900E+00_JPRB, 0.37277E+00_JPRB, 0.37193E+00_JPRB, 0.36862E+00_JPRB, 0.36369E+00_JPRB ,&
&  0.36535E+00_JPRB, 0.37255E+00_JPRB, 0.38113E+00_JPRB, 0.38711E+00_JPRB, 0.39212E+00_JPRB, 0.39497E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.40077E+00_JPRB, 0.40601E+00_JPRB, 0.41364E+00_JPRB, 0.42028E+00_JPRB, 0.42471E+00_JPRB, 0.42931E+00_JPRB, 0.42929E+00_JPRB ,&
&  0.42761E+00_JPRB, 0.42251E+00_JPRB, 0.41753E+00_JPRB, 0.41525E+00_JPRB, 0.41313E+00_JPRB, 0.41311E+00_JPRB, 0.40459E+00_JPRB ,&
&  0.37715E+00_JPRB, 0.37637E+00_JPRB, 0.39440E+00_JPRB, 0.40062E+00_JPRB, 0.40081E+00_JPRB, 0.39853E+00_JPRB, 0.39903E+00_JPRB ,&
&  0.40114E+00_JPRB, 0.40325E+00_JPRB, 0.40787E+00_JPRB, 0.41012E+00_JPRB, 0.41361E+00_JPRB, 0.41726E+00_JPRB, 0.42890E+00_JPRB ,&
&  0.45573E+00_JPRB, 0.44107E+00_JPRB, 0.44697E+00_JPRB, 0.48323E+00_JPRB, 0.64919E+00_JPRB, 0.71659E+00_JPRB, 0.76145E+00_JPRB ,&
&  0.81754E+00_JPRB, 0.80555E+00_JPRB, 0.77985E+00_JPRB, 0.76208E+00_JPRB, 0.74886E+00_JPRB, 0.73410E+00_JPRB, 0.72784E+00_JPRB ,&
&  0.70373E+00_JPRB, 0.70373E+00_JPRB, 0.71336E+00_JPRB, 0.73245E+00_JPRB, 0.73440E+00_JPRB, 0.73223E+00_JPRB, 0.72330E+00_JPRB ,&
&  0.70340E+00_JPRB, 0.67327E+00_JPRB, 0.62962E+00_JPRB, 0.58746E+00_JPRB, 0.55251E+00_JPRB, 0.53473E+00_JPRB, 0.52236E+00_JPRB ,&
&  0.51194E+00_JPRB, 0.50584E+00_JPRB, 0.49949E+00_JPRB, 0.49170E+00_JPRB, 0.48205E+00_JPRB, 0.47215E+00_JPRB, 0.46854E+00_JPRB ,&
&  0.44962E+00_JPRB, 0.43688E+00_JPRB, 0.42301E+00_JPRB, 0.41331E+00_JPRB, 0.41139E+00_JPRB, 0.41148E+00_JPRB, 0.41129E+00_JPRB ,&
&  0.41376E+00_JPRB, 0.40674E+00_JPRB, 0.40535E+00_JPRB, 0.40534E+00_JPRB, 0.40634E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.41077E+00_JPRB, 0.41786E+00_JPRB, 0.42071E+00_JPRB, 0.42296E+00_JPRB, 0.42892E+00_JPRB, 0.43160E+00_JPRB, 0.43411E+00_JPRB ,&
&  0.47601E+00_JPRB, 0.45231E+00_JPRB, 0.46306E+00_JPRB, 0.41314E+00_JPRB, 0.39262E+00_JPRB, 0.38409E+00_JPRB, 0.37262E+00_JPRB ,&
&  0.37118E+00_JPRB, 0.37561E+00_JPRB, 0.38101E+00_JPRB, 0.38746E+00_JPRB, 0.39508E+00_JPRB, 0.39896E+00_JPRB, 0.40216E+00_JPRB ,&
&  0.40650E+00_JPRB, 0.41262E+00_JPRB, 0.41963E+00_JPRB, 0.42473E+00_JPRB, 0.42911E+00_JPRB, 0.43446E+00_JPRB, 0.43360E+00_JPRB ,&
&  0.42438E+00_JPRB, 0.42012E+00_JPRB, 0.41919E+00_JPRB, 0.41642E+00_JPRB, 0.41809E+00_JPRB, 0.41943E+00_JPRB, 0.42717E+00_JPRB ,&
&  0.42893E+00_JPRB, 0.42000E+00_JPRB, 0.42827E+00_JPRB, 0.42563E+00_JPRB, 0.42392E+00_JPRB, 0.42132E+00_JPRB, 0.41867E+00_JPRB ,&
&  0.41808E+00_JPRB, 0.42079E+00_JPRB, 0.42681E+00_JPRB, 0.43182E+00_JPRB, 0.43645E+00_JPRB, 0.44704E+00_JPRB, 0.47091E+00_JPRB ,&
&  0.51282E+00_JPRB, 0.48210E+00_JPRB, 0.55373E+00_JPRB, 0.58876E+00_JPRB, 0.77336E+00_JPRB, 0.95813E+00_JPRB, 0.96163E+00_JPRB ,&
&  0.10517E+01_JPRB, 0.10121E+01_JPRB, 0.96053E+00_JPRB, 0.91940E+00_JPRB, 0.90100E+00_JPRB, 0.86099E+00_JPRB, 0.81525E+00_JPRB ,&
&  0.80537E+00_JPRB, 0.80027E+00_JPRB, 0.84512E+00_JPRB, 0.84436E+00_JPRB, 0.81917E+00_JPRB, 0.80448E+00_JPRB, 0.77588E+00_JPRB ,&
&  0.75198E+00_JPRB, 0.72391E+00_JPRB, 0.69393E+00_JPRB, 0.63716E+00_JPRB, 0.61395E+00_JPRB, 0.59188E+00_JPRB, 0.55456E+00_JPRB ,&
&  0.53767E+00_JPRB, 0.51731E+00_JPRB, 0.49869E+00_JPRB, 0.50221E+00_JPRB, 0.48834E+00_JPRB, 0.48096E+00_JPRB, 0.47696E+00_JPRB ,&
&  0.46372E+00_JPRB, 0.45137E+00_JPRB, 0.45233E+00_JPRB, 0.44734E+00_JPRB, 0.43743E+00_JPRB, 0.42912E+00_JPRB, 0.42648E+00_JPRB ,&
&  0.42153E+00_JPRB, 0.41888E+00_JPRB, 0.42216E+00_JPRB, 0.41558E+00_JPRB, 0.41137E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.42460E+00_JPRB, 0.42934E+00_JPRB, 0.42993E+00_JPRB, 0.43494E+00_JPRB, 0.44489E+00_JPRB, 0.45484E+00_JPRB, 0.46279E+00_JPRB ,&
&  0.48260E+00_JPRB, 0.49038E+00_JPRB, 0.49086E+00_JPRB, 0.48498E+00_JPRB, 0.47288E+00_JPRB, 0.43467E+00_JPRB, 0.38312E+00_JPRB ,&
&  0.37133E+00_JPRB, 0.37892E+00_JPRB, 0.38859E+00_JPRB, 0.39340E+00_JPRB, 0.39935E+00_JPRB, 0.40523E+00_JPRB, 0.41008E+00_JPRB ,&
&  0.41548E+00_JPRB, 0.41942E+00_JPRB, 0.42477E+00_JPRB, 0.43001E+00_JPRB, 0.43347E+00_JPRB, 0.43860E+00_JPRB, 0.44169E+00_JPRB ,&
&  0.42739E+00_JPRB, 0.41954E+00_JPRB, 0.41575E+00_JPRB, 0.41619E+00_JPRB, 0.42981E+00_JPRB, 0.43749E+00_JPRB, 0.44695E+00_JPRB ,&
&  0.45555E+00_JPRB, 0.46007E+00_JPRB, 0.45913E+00_JPRB, 0.45739E+00_JPRB, 0.45444E+00_JPRB, 0.45316E+00_JPRB, 0.45039E+00_JPRB ,&
&  0.44929E+00_JPRB, 0.45143E+00_JPRB, 0.45872E+00_JPRB, 0.46470E+00_JPRB, 0.47348E+00_JPRB, 0.49292E+00_JPRB, 0.54833E+00_JPRB ,&
&  0.61057E+00_JPRB, 0.68826E+00_JPRB, 0.73338E+00_JPRB, 0.73412E+00_JPRB, 0.88647E+00_JPRB, 0.99877E+00_JPRB, 0.10189E+01_JPRB ,&
&  0.10519E+01_JPRB, 0.99006E+00_JPRB, 0.10274E+01_JPRB, 0.10642E+01_JPRB, 0.10804E+01_JPRB, 0.10355E+01_JPRB, 0.95943E+00_JPRB ,&
&  0.94508E+00_JPRB, 0.10104E+01_JPRB, 0.10097E+01_JPRB, 0.10018E+01_JPRB, 0.95696E+00_JPRB, 0.92397E+00_JPRB, 0.88198E+00_JPRB ,&
&  0.87072E+00_JPRB, 0.85611E+00_JPRB, 0.81393E+00_JPRB, 0.76745E+00_JPRB, 0.71712E+00_JPRB, 0.66717E+00_JPRB, 0.61729E+00_JPRB ,&
&  0.57051E+00_JPRB, 0.51660E+00_JPRB, 0.49645E+00_JPRB, 0.49471E+00_JPRB, 0.49734E+00_JPRB, 0.48369E+00_JPRB, 0.47904E+00_JPRB ,&
&  0.47613E+00_JPRB, 0.47745E+00_JPRB, 0.48543E+00_JPRB, 0.47391E+00_JPRB, 0.45669E+00_JPRB, 0.43778E+00_JPRB, 0.42623E+00_JPRB ,&
&  0.41761E+00_JPRB, 0.42212E+00_JPRB, 0.42607E+00_JPRB, 0.42383E+00_JPRB, 0.42180E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.43645E+00_JPRB, 0.43523E+00_JPRB, 0.43668E+00_JPRB, 0.44612E+00_JPRB, 0.46005E+00_JPRB, 0.47502E+00_JPRB, 0.49321E+00_JPRB ,&
&  0.51037E+00_JPRB, 0.51941E+00_JPRB, 0.51364E+00_JPRB, 0.50239E+00_JPRB, 0.49300E+00_JPRB, 0.48807E+00_JPRB, 0.45904E+00_JPRB ,&
&  0.41358E+00_JPRB, 0.41075E+00_JPRB, 0.40264E+00_JPRB, 0.40505E+00_JPRB, 0.41492E+00_JPRB, 0.42076E+00_JPRB, 0.42634E+00_JPRB ,&
&  0.43503E+00_JPRB, 0.43795E+00_JPRB, 0.43980E+00_JPRB, 0.44144E+00_JPRB, 0.44533E+00_JPRB, 0.45786E+00_JPRB, 0.46731E+00_JPRB ,&
&  0.44712E+00_JPRB, 0.43399E+00_JPRB, 0.43225E+00_JPRB, 0.43919E+00_JPRB, 0.45360E+00_JPRB, 0.46962E+00_JPRB, 0.48082E+00_JPRB ,&
&  0.49065E+00_JPRB, 0.49646E+00_JPRB, 0.49785E+00_JPRB, 0.49658E+00_JPRB, 0.49398E+00_JPRB, 0.49021E+00_JPRB, 0.48898E+00_JPRB ,&
&  0.48906E+00_JPRB, 0.49262E+00_JPRB, 0.50115E+00_JPRB, 0.50989E+00_JPRB, 0.51918E+00_JPRB, 0.56656E+00_JPRB, 0.67192E+00_JPRB ,&
&  0.79525E+00_JPRB, 0.91807E+00_JPRB, 0.94866E+00_JPRB, 0.10261E+01_JPRB, 0.10931E+01_JPRB, 0.11502E+01_JPRB, 0.11636E+01_JPRB ,&
&  0.11684E+01_JPRB, 0.10891E+01_JPRB, 0.12039E+01_JPRB, 0.12467E+01_JPRB, 0.12084E+01_JPRB, 0.12113E+01_JPRB, 0.11396E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.10673E+01_JPRB, 0.11063E+01_JPRB, 0.11084E+01_JPRB, 0.10800E+01_JPRB, 0.10848E+01_JPRB, 0.10386E+01_JPRB ,&
&  0.10098E+01_JPRB, 0.96464E+00_JPRB, 0.83906E+00_JPRB, 0.74555E+00_JPRB, 0.65727E+00_JPRB, 0.67549E+00_JPRB, 0.63721E+00_JPRB ,&
&  0.58614E+00_JPRB, 0.52264E+00_JPRB, 0.52182E+00_JPRB, 0.51941E+00_JPRB, 0.52863E+00_JPRB, 0.53135E+00_JPRB, 0.51136E+00_JPRB ,&
&  0.51056E+00_JPRB, 0.52123E+00_JPRB, 0.50832E+00_JPRB, 0.49535E+00_JPRB, 0.47928E+00_JPRB, 0.45784E+00_JPRB, 0.44524E+00_JPRB ,&
&  0.44459E+00_JPRB, 0.44781E+00_JPRB, 0.44538E+00_JPRB, 0.43962E+00_JPRB, 0.43711E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.46294E+00_JPRB, 0.46193E+00_JPRB, 0.46477E+00_JPRB, 0.47396E+00_JPRB, 0.48694E+00_JPRB, 0.50453E+00_JPRB, 0.52299E+00_JPRB ,&
&  0.53732E+00_JPRB, 0.54061E+00_JPRB, 0.52952E+00_JPRB, 0.52112E+00_JPRB, 0.51575E+00_JPRB, 0.50930E+00_JPRB, 0.51189E+00_JPRB ,&
&  0.45691E+00_JPRB, 0.43597E+00_JPRB, 0.41026E+00_JPRB, 0.41066E+00_JPRB, 0.42979E+00_JPRB, 0.43625E+00_JPRB, 0.44258E+00_JPRB ,&
&  0.44749E+00_JPRB, 0.45558E+00_JPRB, 0.45668E+00_JPRB, 0.45945E+00_JPRB, 0.46620E+00_JPRB, 0.48529E+00_JPRB, 0.50677E+00_JPRB ,&
&  0.49105E+00_JPRB, 0.47551E+00_JPRB, 0.47884E+00_JPRB, 0.49117E+00_JPRB, 0.50097E+00_JPRB, 0.52001E+00_JPRB, 0.54273E+00_JPRB ,&
&  0.54989E+00_JPRB, 0.54819E+00_JPRB, 0.54135E+00_JPRB, 0.53761E+00_JPRB, 0.53624E+00_JPRB, 0.53504E+00_JPRB, 0.53969E+00_JPRB ,&
&  0.54680E+00_JPRB, 0.55353E+00_JPRB, 0.56299E+00_JPRB, 0.57003E+00_JPRB, 0.57335E+00_JPRB, 0.64917E+00_JPRB, 0.78757E+00_JPRB ,&
&  0.96694E+00_JPRB, 0.10024E+01_JPRB, 0.10737E+01_JPRB, 0.13141E+01_JPRB, 0.14200E+01_JPRB, 0.14521E+01_JPRB, 0.13927E+01_JPRB ,&
&  0.13319E+01_JPRB, 0.12405E+01_JPRB, 0.12247E+01_JPRB, 0.12340E+01_JPRB, 0.12097E+01_JPRB, 0.12661E+01_JPRB, 0.12317E+01_JPRB ,&
&  0.11563E+01_JPRB, 0.11343E+01_JPRB, 0.11314E+01_JPRB, 0.11208E+01_JPRB, 0.10846E+01_JPRB, 0.10841E+01_JPRB, 0.10493E+01_JPRB ,&
&  0.99399E+00_JPRB, 0.86463E+00_JPRB, 0.69015E+00_JPRB, 0.65425E+00_JPRB, 0.56511E+00_JPRB, 0.58811E+00_JPRB, 0.67224E+00_JPRB ,&
&  0.67935E+00_JPRB, 0.64100E+00_JPRB, 0.60456E+00_JPRB, 0.58618E+00_JPRB, 0.58181E+00_JPRB, 0.61012E+00_JPRB, 0.58145E+00_JPRB ,&
&  0.55113E+00_JPRB, 0.55883E+00_JPRB, 0.55625E+00_JPRB, 0.54204E+00_JPRB, 0.52167E+00_JPRB, 0.50361E+00_JPRB, 0.49200E+00_JPRB ,&
&  0.49015E+00_JPRB, 0.48897E+00_JPRB, 0.48473E+00_JPRB, 0.47174E+00_JPRB, 0.46387E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.50575E+00_JPRB, 0.50711E+00_JPRB, 0.50745E+00_JPRB, 0.51534E+00_JPRB, 0.52663E+00_JPRB, 0.54132E+00_JPRB, 0.55624E+00_JPRB ,&
&  0.56350E+00_JPRB, 0.55804E+00_JPRB, 0.54519E+00_JPRB, 0.54219E+00_JPRB, 0.53702E+00_JPRB, 0.52767E+00_JPRB, 0.52604E+00_JPRB ,&
&  0.53692E+00_JPRB, 0.48919E+00_JPRB, 0.45006E+00_JPRB, 0.42109E+00_JPRB, 0.43011E+00_JPRB, 0.43534E+00_JPRB, 0.44753E+00_JPRB ,&
&  0.45512E+00_JPRB, 0.47243E+00_JPRB, 0.47944E+00_JPRB, 0.48420E+00_JPRB, 0.49690E+00_JPRB, 0.52723E+00_JPRB, 0.55661E+00_JPRB ,&
&  0.56105E+00_JPRB, 0.56384E+00_JPRB, 0.57571E+00_JPRB, 0.59884E+00_JPRB, 0.61840E+00_JPRB, 0.59347E+00_JPRB, 0.61492E+00_JPRB ,&
&  0.61410E+00_JPRB, 0.60616E+00_JPRB, 0.59249E+00_JPRB, 0.59383E+00_JPRB, 0.60093E+00_JPRB, 0.61341E+00_JPRB, 0.62871E+00_JPRB ,&
&  0.64676E+00_JPRB, 0.65024E+00_JPRB, 0.63067E+00_JPRB, 0.62799E+00_JPRB, 0.64554E+00_JPRB, 0.76765E+00_JPRB, 0.84665E+00_JPRB ,&
&  0.97878E+00_JPRB, 0.97348E+00_JPRB, 0.98355E+00_JPRB, 0.12004E+01_JPRB, 0.14000E+01_JPRB, 0.14803E+01_JPRB, 0.14859E+01_JPRB ,&
&  0.14473E+01_JPRB, 0.13895E+01_JPRB, 0.13593E+01_JPRB, 0.12953E+01_JPRB, 0.12904E+01_JPRB, 0.13157E+01_JPRB, 0.12775E+01_JPRB ,&
&  0.11925E+01_JPRB, 0.11244E+01_JPRB, 0.11022E+01_JPRB, 0.10603E+01_JPRB, 0.10237E+01_JPRB, 0.95629E+00_JPRB, 0.96961E+00_JPRB ,&
&  0.97207E+00_JPRB, 0.64325E+00_JPRB, 0.55852E+00_JPRB, 0.61769E+00_JPRB, 0.55704E+00_JPRB, 0.57454E+00_JPRB, 0.63768E+00_JPRB ,&
&  0.64185E+00_JPRB, 0.64028E+00_JPRB, 0.66007E+00_JPRB, 0.64287E+00_JPRB, 0.65347E+00_JPRB, 0.67770E+00_JPRB, 0.66140E+00_JPRB ,&
&  0.64059E+00_JPRB, 0.61584E+00_JPRB, 0.61785E+00_JPRB, 0.60065E+00_JPRB, 0.57715E+00_JPRB, 0.55956E+00_JPRB, 0.54596E+00_JPRB ,&
&  0.53413E+00_JPRB, 0.52768E+00_JPRB, 0.52493E+00_JPRB, 0.51338E+00_JPRB, 0.50487E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.56812E+00_JPRB, 0.56992E+00_JPRB, 0.57153E+00_JPRB, 0.57603E+00_JPRB, 0.58037E+00_JPRB, 0.58972E+00_JPRB, 0.59534E+00_JPRB ,&
&  0.59168E+00_JPRB, 0.57550E+00_JPRB, 0.56332E+00_JPRB, 0.56578E+00_JPRB, 0.55553E+00_JPRB, 0.54349E+00_JPRB, 0.54389E+00_JPRB ,&
&  0.55740E+00_JPRB, 0.54717E+00_JPRB, 0.47606E+00_JPRB, 0.44179E+00_JPRB, 0.41687E+00_JPRB, 0.42656E+00_JPRB, 0.43789E+00_JPRB ,&
&  0.46027E+00_JPRB, 0.47442E+00_JPRB, 0.48359E+00_JPRB, 0.49877E+00_JPRB, 0.53194E+00_JPRB, 0.57554E+00_JPRB, 0.62497E+00_JPRB ,&
&  0.64935E+00_JPRB, 0.67561E+00_JPRB, 0.66412E+00_JPRB, 0.70795E+00_JPRB, 0.71846E+00_JPRB, 0.68286E+00_JPRB, 0.70724E+00_JPRB ,&
&  0.70121E+00_JPRB, 0.69434E+00_JPRB, 0.68203E+00_JPRB, 0.68749E+00_JPRB, 0.69354E+00_JPRB, 0.68421E+00_JPRB, 0.66917E+00_JPRB ,&
&  0.65847E+00_JPRB, 0.65524E+00_JPRB, 0.64998E+00_JPRB, 0.64290E+00_JPRB, 0.67573E+00_JPRB, 0.75877E+00_JPRB, 0.79315E+00_JPRB ,&
&  0.86673E+00_JPRB, 0.74672E+00_JPRB, 0.66874E+00_JPRB, 0.90033E+00_JPRB, 0.11898E+01_JPRB, 0.12865E+01_JPRB, 0.12507E+01_JPRB ,&
&  0.13545E+01_JPRB, 0.13825E+01_JPRB, 0.14147E+01_JPRB, 0.14425E+01_JPRB, 0.14350E+01_JPRB, 0.14120E+01_JPRB, 0.13498E+01_JPRB ,&
&  0.12336E+01_JPRB, 0.11506E+01_JPRB, 0.11100E+01_JPRB, 0.11259E+01_JPRB, 0.10815E+01_JPRB, 0.10428E+01_JPRB, 0.10089E+01_JPRB ,&
&  0.89092E+00_JPRB, 0.79314E+00_JPRB, 0.62766E+00_JPRB, 0.63530E+00_JPRB, 0.54400E+00_JPRB, 0.54948E+00_JPRB, 0.60769E+00_JPRB ,&
&  0.61191E+00_JPRB, 0.64763E+00_JPRB, 0.66309E+00_JPRB, 0.65303E+00_JPRB, 0.73201E+00_JPRB, 0.75774E+00_JPRB, 0.74191E+00_JPRB ,&
&  0.72715E+00_JPRB, 0.66947E+00_JPRB, 0.68390E+00_JPRB, 0.68676E+00_JPRB, 0.67175E+00_JPRB, 0.65531E+00_JPRB, 0.63729E+00_JPRB ,&
&  0.61736E+00_JPRB, 0.60074E+00_JPRB, 0.59066E+00_JPRB, 0.57617E+00_JPRB, 0.56809E+00_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.62771E+00_JPRB, 0.61639E+00_JPRB, 0.61032E+00_JPRB, 0.61049E+00_JPRB, 0.61322E+00_JPRB, 0.61659E+00_JPRB, 0.61727E+00_JPRB ,&
&  0.61155E+00_JPRB, 0.59381E+00_JPRB, 0.58972E+00_JPRB, 0.58738E+00_JPRB, 0.56874E+00_JPRB, 0.55466E+00_JPRB, 0.55858E+00_JPRB ,&
&  0.57616E+00_JPRB, 0.54123E+00_JPRB, 0.46026E+00_JPRB, 0.41910E+00_JPRB, 0.39872E+00_JPRB, 0.40482E+00_JPRB, 0.42222E+00_JPRB ,&
&  0.44521E+00_JPRB, 0.46292E+00_JPRB, 0.48440E+00_JPRB, 0.51489E+00_JPRB, 0.56241E+00_JPRB, 0.61451E+00_JPRB, 0.71235E+00_JPRB ,&
&  0.83670E+00_JPRB, 0.76155E+00_JPRB, 0.78418E+00_JPRB, 0.77862E+00_JPRB, 0.80829E+00_JPRB, 0.79486E+00_JPRB, 0.77220E+00_JPRB ,&
&  0.73863E+00_JPRB, 0.71864E+00_JPRB, 0.70151E+00_JPRB, 0.68111E+00_JPRB, 0.66236E+00_JPRB, 0.65315E+00_JPRB, 0.65964E+00_JPRB ,&
&  0.68685E+00_JPRB, 0.69595E+00_JPRB, 0.68354E+00_JPRB, 0.69461E+00_JPRB, 0.77403E+00_JPRB, 0.79422E+00_JPRB, 0.78551E+00_JPRB ,&
&  0.72909E+00_JPRB, 0.75445E+00_JPRB, 0.96760E+00_JPRB, 0.11118E+01_JPRB, 0.12104E+01_JPRB, 0.12910E+01_JPRB, 0.13180E+01_JPRB ,&
&  0.14007E+01_JPRB, 0.13730E+01_JPRB, 0.13982E+01_JPRB, 0.12495E+01_JPRB, 0.13808E+01_JPRB, 0.13749E+01_JPRB, 0.13319E+01_JPRB ,&
&  0.12701E+01_JPRB, 0.12468E+01_JPRB, 0.12051E+01_JPRB, 0.12349E+01_JPRB, 0.12229E+01_JPRB, 0.12005E+01_JPRB, 0.95550E+00_JPRB ,&
&  0.79065E+00_JPRB, 0.82053E+00_JPRB, 0.74972E+00_JPRB, 0.67365E+00_JPRB, 0.60446E+00_JPRB, 0.61170E+00_JPRB, 0.62341E+00_JPRB ,&
&  0.63637E+00_JPRB, 0.64594E+00_JPRB, 0.64187E+00_JPRB, 0.67307E+00_JPRB, 0.76431E+00_JPRB, 0.86151E+00_JPRB, 0.85613E+00_JPRB ,&
&  0.79967E+00_JPRB, 0.81729E+00_JPRB, 0.82386E+00_JPRB, 0.80476E+00_JPRB, 0.83040E+00_JPRB, 0.80980E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.73620E+00_JPRB, 0.71188E+00_JPRB, 0.69282E+00_JPRB, 0.67057E+00_JPRB, 0.64536E+00_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.68949E+00_JPRB, 0.67448E+00_JPRB, 0.66146E+00_JPRB, 0.65322E+00_JPRB, 0.64600E+00_JPRB, 0.64210E+00_JPRB, 0.63727E+00_JPRB ,&
&  0.62395E+00_JPRB, 0.60106E+00_JPRB, 0.60496E+00_JPRB, 0.59743E+00_JPRB, 0.58263E+00_JPRB, 0.57017E+00_JPRB, 0.57637E+00_JPRB ,&
&  0.59282E+00_JPRB, 0.53213E+00_JPRB, 0.45315E+00_JPRB, 0.44289E+00_JPRB, 0.42996E+00_JPRB, 0.40936E+00_JPRB, 0.41629E+00_JPRB ,&
&  0.43535E+00_JPRB, 0.45755E+00_JPRB, 0.47952E+00_JPRB, 0.53363E+00_JPRB, 0.60696E+00_JPRB, 0.70042E+00_JPRB, 0.83074E+00_JPRB ,&
&  0.85690E+00_JPRB, 0.88759E+00_JPRB, 0.94403E+00_JPRB, 0.90263E+00_JPRB, 0.85733E+00_JPRB, 0.81065E+00_JPRB, 0.76912E+00_JPRB ,&
&  0.73980E+00_JPRB, 0.71853E+00_JPRB, 0.70162E+00_JPRB, 0.68784E+00_JPRB, 0.68922E+00_JPRB, 0.69249E+00_JPRB, 0.69910E+00_JPRB ,&
&  0.70928E+00_JPRB, 0.70075E+00_JPRB, 0.67865E+00_JPRB, 0.74977E+00_JPRB, 0.68177E+00_JPRB, 0.65978E+00_JPRB, 0.66511E+00_JPRB ,&
&  0.83482E+00_JPRB, 0.95242E+00_JPRB, 0.10323E+01_JPRB, 0.12516E+01_JPRB, 0.11152E+01_JPRB, 0.12001E+01_JPRB, 0.14352E+01_JPRB ,&
&  0.14941E+01_JPRB, 0.14913E+01_JPRB, 0.15788E+01_JPRB, 0.12911E+01_JPRB, 0.11453E+01_JPRB, 0.14901E+01_JPRB, 0.14935E+01_JPRB ,&
&  0.14407E+01_JPRB, 0.14150E+01_JPRB, 0.13993E+01_JPRB, 0.15313E+01_JPRB, 0.10811E+01_JPRB, 0.88380E+00_JPRB, 0.71058E+00_JPRB ,&
&  0.70185E+00_JPRB, 0.67380E+00_JPRB, 0.74057E+00_JPRB, 0.70887E+00_JPRB, 0.69980E+00_JPRB, 0.67936E+00_JPRB, 0.64555E+00_JPRB ,&
&  0.64126E+00_JPRB, 0.63952E+00_JPRB, 0.64664E+00_JPRB, 0.73128E+00_JPRB, 0.87610E+00_JPRB, 0.90965E+00_JPRB, 0.98309E+00_JPRB ,&
&  0.10012E+01_JPRB, 0.10498E+01_JPRB, 0.10091E+01_JPRB, 0.10046E+01_JPRB, 0.98326E+00_JPRB, 0.94379E+00_JPRB, 0.88862E+00_JPRB ,&
&  0.84745E+00_JPRB, 0.81192E+00_JPRB, 0.77525E+00_JPRB, 0.74196E+00_JPRB, 0.71273E+00_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.73954E+00_JPRB, 0.71603E+00_JPRB, 0.69462E+00_JPRB, 0.67682E+00_JPRB, 0.66785E+00_JPRB, 0.66016E+00_JPRB, 0.64941E+00_JPRB ,&
&  0.62703E+00_JPRB, 0.60809E+00_JPRB, 0.61182E+00_JPRB, 0.58901E+00_JPRB, 0.57318E+00_JPRB, 0.57449E+00_JPRB, 0.59472E+00_JPRB ,&
&  0.60498E+00_JPRB, 0.57892E+00_JPRB, 0.49418E+00_JPRB, 0.45616E+00_JPRB, 0.46091E+00_JPRB, 0.43378E+00_JPRB, 0.42431E+00_JPRB ,&
&  0.44705E+00_JPRB, 0.47163E+00_JPRB, 0.51677E+00_JPRB, 0.61531E+00_JPRB, 0.72722E+00_JPRB, 0.85599E+00_JPRB, 0.99922E+00_JPRB ,&
&  0.10192E+01_JPRB, 0.10019E+01_JPRB, 0.94836E+00_JPRB, 0.88106E+00_JPRB, 0.80996E+00_JPRB, 0.76271E+00_JPRB, 0.72785E+00_JPRB ,&
&  0.70364E+00_JPRB, 0.69262E+00_JPRB, 0.69408E+00_JPRB, 0.68600E+00_JPRB, 0.69353E+00_JPRB, 0.66620E+00_JPRB, 0.65897E+00_JPRB ,&
&  0.66968E+00_JPRB, 0.68037E+00_JPRB, 0.69301E+00_JPRB, 0.80162E+00_JPRB, 0.75216E+00_JPRB, 0.69124E+00_JPRB, 0.71837E+00_JPRB ,&
&  0.82171E+00_JPRB, 0.89318E+00_JPRB, 0.10221E+01_JPRB, 0.12449E+01_JPRB, 0.15355E+01_JPRB, 0.16498E+01_JPRB, 0.17614E+01_JPRB ,&
&  0.14380E+01_JPRB, 0.12721E+01_JPRB, 0.12859E+01_JPRB, 0.12189E+01_JPRB, 0.15434E+01_JPRB, 0.20647E+01_JPRB, 0.20762E+01_JPRB ,&
&  0.19789E+01_JPRB, 0.18666E+01_JPRB, 0.18531E+01_JPRB, 0.15077E+01_JPRB, 0.10597E+01_JPRB, 0.78616E+00_JPRB, 0.88157E+00_JPRB ,&
&  0.91237E+00_JPRB, 0.92797E+00_JPRB, 0.84551E+00_JPRB, 0.74047E+00_JPRB, 0.66249E+00_JPRB, 0.74490E+00_JPRB, 0.73029E+00_JPRB ,&
&  0.69421E+00_JPRB, 0.67982E+00_JPRB, 0.79764E+00_JPRB, 0.96882E+00_JPRB, 0.10660E+01_JPRB, 0.11288E+01_JPRB, 0.12864E+01_JPRB ,&
&  0.13438E+01_JPRB, 0.14062E+01_JPRB, 0.13490E+01_JPRB, 0.12634E+01_JPRB, 0.11749E+01_JPRB, 0.10750E+01_JPRB, 0.99199E+00_JPRB ,&
&  0.92815E+00_JPRB, 0.87526E+00_JPRB, 0.83007E+00_JPRB, 0.79188E+00_JPRB, 0.76351E+00_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.78072E+00_JPRB, 0.74765E+00_JPRB, 0.71851E+00_JPRB, 0.70068E+00_JPRB, 0.68655E+00_JPRB, 0.67308E+00_JPRB, 0.65285E+00_JPRB ,&
&  0.62328E+00_JPRB, 0.62263E+00_JPRB, 0.61741E+00_JPRB, 0.59036E+00_JPRB, 0.58016E+00_JPRB, 0.58393E+00_JPRB, 0.59721E+00_JPRB ,&
&  0.60918E+00_JPRB, 0.63557E+00_JPRB, 0.54475E+00_JPRB, 0.50343E+00_JPRB, 0.49551E+00_JPRB, 0.47988E+00_JPRB, 0.44765E+00_JPRB ,&
&  0.47258E+00_JPRB, 0.50527E+00_JPRB, 0.56448E+00_JPRB, 0.68954E+00_JPRB, 0.88132E+00_JPRB, 0.96708E+00_JPRB, 0.99266E+00_JPRB ,&
&  0.10217E+01_JPRB, 0.97551E+00_JPRB, 0.91111E+00_JPRB, 0.85077E+00_JPRB, 0.78487E+00_JPRB, 0.75093E+00_JPRB, 0.73051E+00_JPRB ,&
&  0.72128E+00_JPRB, 0.73366E+00_JPRB, 0.73784E+00_JPRB, 0.75649E+00_JPRB, 0.72858E+00_JPRB, 0.69147E+00_JPRB, 0.67711E+00_JPRB ,&
&  0.67082E+00_JPRB, 0.68120E+00_JPRB, 0.70513E+00_JPRB, 0.82050E+00_JPRB, 0.85877E+00_JPRB, 0.80566E+00_JPRB, 0.89875E+00_JPRB ,&
&  0.95676E+00_JPRB, 0.10735E+01_JPRB, 0.11327E+01_JPRB, 0.13818E+01_JPRB, 0.15912E+01_JPRB, 0.17010E+01_JPRB, 0.18169E+01_JPRB ,&
&  0.17071E+01_JPRB, 0.17970E+01_JPRB, 0.21663E+01_JPRB, 0.26072E+01_JPRB, 0.23003E+01_JPRB, 0.22789E+01_JPRB, 0.30180E+01_JPRB ,&
&  0.22174E+01_JPRB, 0.21595E+01_JPRB, 0.22244E+01_JPRB, 0.21347E+01_JPRB, 0.89504E+00_JPRB, 0.69309E+00_JPRB, 0.91177E+00_JPRB ,&
&  0.93088E+00_JPRB, 0.66279E+00_JPRB, 0.54873E+00_JPRB, 0.61726E+00_JPRB, 0.53451E+00_JPRB, 0.60756E+00_JPRB, 0.78901E+00_JPRB ,&
&  0.82327E+00_JPRB, 0.84472E+00_JPRB, 0.10791E+01_JPRB, 0.12044E+01_JPRB, 0.12049E+01_JPRB, 0.14681E+01_JPRB, 0.16800E+01_JPRB ,&
&  0.17412E+01_JPRB, 0.16837E+01_JPRB, 0.16934E+01_JPRB, 0.15362E+01_JPRB, 0.13547E+01_JPRB, 0.11995E+01_JPRB, 0.10799E+01_JPRB ,&
&  0.98102E+00_JPRB, 0.91478E+00_JPRB, 0.87196E+00_JPRB, 0.83369E+00_JPRB, 0.80482E+00_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.77740E+00_JPRB, 0.74649E+00_JPRB, 0.71616E+00_JPRB, 0.69662E+00_JPRB, 0.68531E+00_JPRB, 0.67171E+00_JPRB, 0.65103E+00_JPRB ,&
&  0.63236E+00_JPRB, 0.64470E+00_JPRB, 0.61996E+00_JPRB, 0.59864E+00_JPRB, 0.58364E+00_JPRB, 0.58219E+00_JPRB, 0.59584E+00_JPRB ,&
&  0.60674E+00_JPRB, 0.62236E+00_JPRB, 0.60430E+00_JPRB, 0.56224E+00_JPRB, 0.54626E+00_JPRB, 0.52172E+00_JPRB, 0.50374E+00_JPRB ,&
&  0.52940E+00_JPRB, 0.57437E+00_JPRB, 0.64847E+00_JPRB, 0.76994E+00_JPRB, 0.87726E+00_JPRB, 0.93158E+00_JPRB, 0.10007E+01_JPRB ,&
&  0.98798E+00_JPRB, 0.93553E+00_JPRB, 0.88073E+00_JPRB, 0.82815E+00_JPRB, 0.78514E+00_JPRB, 0.76296E+00_JPRB, 0.74416E+00_JPRB ,&
&  0.73882E+00_JPRB, 0.74581E+00_JPRB, 0.75960E+00_JPRB, 0.72662E+00_JPRB, 0.69640E+00_JPRB, 0.67703E+00_JPRB, 0.66086E+00_JPRB ,&
&  0.65964E+00_JPRB, 0.69613E+00_JPRB, 0.74761E+00_JPRB, 0.82309E+00_JPRB, 0.95058E+00_JPRB, 0.99327E+00_JPRB, 0.92191E+00_JPRB ,&
&  0.86703E+00_JPRB, 0.95707E+00_JPRB, 0.12444E+01_JPRB, 0.13042E+01_JPRB, 0.12976E+01_JPRB, 0.13897E+01_JPRB, 0.14128E+01_JPRB ,&
&  0.16057E+01_JPRB, 0.20205E+01_JPRB, 0.20965E+01_JPRB, 0.25632E+01_JPRB, 0.27500E+01_JPRB, 0.21145E+01_JPRB, 0.23698E+01_JPRB ,&
&  0.23346E+01_JPRB, 0.20305E+01_JPRB, 0.15677E+01_JPRB, 0.11517E+01_JPRB, 0.11675E+01_JPRB, 0.96674E+00_JPRB, 0.63728E+00_JPRB ,&
&  0.57592E+00_JPRB, 0.54946E+00_JPRB, 0.52465E+00_JPRB, 0.52715E+00_JPRB, 0.52338E+00_JPRB, 0.61846E+00_JPRB, 0.78803E+00_JPRB ,&
&  0.10241E+01_JPRB, 0.11683E+01_JPRB, 0.13052E+01_JPRB, 0.13431E+01_JPRB, 0.12726E+01_JPRB, 0.15036E+01_JPRB, 0.19389E+01_JPRB ,&
&  0.16719E+01_JPRB, 0.15663E+01_JPRB, 0.15427E+01_JPRB, 0.14062E+01_JPRB, 0.12658E+01_JPRB, 0.11449E+01_JPRB, 0.10415E+01_JPRB ,&
&  0.96373E+00_JPRB, 0.91326E+00_JPRB, 0.87695E+00_JPRB, 0.84242E+00_JPRB, 0.81026E+00_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.76119E+00_JPRB, 0.73773E+00_JPRB, 0.72100E+00_JPRB, 0.70610E+00_JPRB, 0.69068E+00_JPRB, 0.68217E+00_JPRB, 0.65616E+00_JPRB ,&
&  0.65823E+00_JPRB, 0.65101E+00_JPRB, 0.61174E+00_JPRB, 0.58286E+00_JPRB, 0.56705E+00_JPRB, 0.56847E+00_JPRB, 0.58383E+00_JPRB ,&
&  0.59305E+00_JPRB, 0.60980E+00_JPRB, 0.64845E+00_JPRB, 0.63425E+00_JPRB, 0.66027E+00_JPRB, 0.61154E+00_JPRB, 0.61622E+00_JPRB ,&
&  0.63074E+00_JPRB, 0.68970E+00_JPRB, 0.78294E+00_JPRB, 0.85581E+00_JPRB, 0.91612E+00_JPRB, 0.98114E+00_JPRB, 0.98131E+00_JPRB ,&
&  0.95494E+00_JPRB, 0.93764E+00_JPRB, 0.86527E+00_JPRB, 0.81360E+00_JPRB, 0.79006E+00_JPRB, 0.76321E+00_JPRB, 0.74603E+00_JPRB ,&
&  0.74807E+00_JPRB, 0.76363E+00_JPRB, 0.73212E+00_JPRB, 0.71537E+00_JPRB, 0.70868E+00_JPRB, 0.69777E+00_JPRB, 0.68572E+00_JPRB ,&
&  0.70335E+00_JPRB, 0.74810E+00_JPRB, 0.80800E+00_JPRB, 0.86588E+00_JPRB, 0.85012E+00_JPRB, 0.75107E+00_JPRB, 0.85365E+00_JPRB ,&
&  0.94948E+00_JPRB, 0.11293E+01_JPRB, 0.11832E+01_JPRB, 0.13823E+01_JPRB, 0.12968E+01_JPRB, 0.12481E+01_JPRB, 0.13811E+01_JPRB ,&
&  0.16940E+01_JPRB, 0.23281E+01_JPRB, 0.20400E+01_JPRB, 0.23116E+01_JPRB, 0.30597E+01_JPRB, 0.23705E+01_JPRB, 0.20220E+01_JPRB ,&
&  0.21763E+01_JPRB, 0.18271E+01_JPRB, 0.15102E+01_JPRB, 0.12949E+01_JPRB, 0.16931E+01_JPRB, 0.17577E+01_JPRB, 0.84943E+00_JPRB ,&
&  0.66575E+00_JPRB, 0.63054E+00_JPRB, 0.60000E+00_JPRB, 0.56562E+00_JPRB, 0.56891E+00_JPRB, 0.70524E+00_JPRB, 0.10023E+01_JPRB ,&
&  0.11419E+01_JPRB, 0.14195E+01_JPRB, 0.15053E+01_JPRB, 0.14750E+01_JPRB, 0.12933E+01_JPRB, 0.15068E+01_JPRB, 0.15783E+01_JPRB ,&
&  0.17263E+01_JPRB, 0.16517E+01_JPRB, 0.14959E+01_JPRB, 0.13484E+01_JPRB, 0.12061E+01_JPRB, 0.10901E+01_JPRB, 0.99967E+00_JPRB ,&
&  0.92622E+00_JPRB, 0.87885E+00_JPRB, 0.84554E+00_JPRB, 0.81874E+00_JPRB, 0.78809E+00_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.76671E+00_JPRB, 0.74473E+00_JPRB, 0.72483E+00_JPRB, 0.70520E+00_JPRB, 0.69820E+00_JPRB, 0.68712E+00_JPRB, 0.66547E+00_JPRB ,&
&  0.69377E+00_JPRB, 0.64644E+00_JPRB, 0.60047E+00_JPRB, 0.57013E+00_JPRB, 0.56027E+00_JPRB, 0.56214E+00_JPRB, 0.57252E+00_JPRB ,&
&  0.58927E+00_JPRB, 0.60302E+00_JPRB, 0.64838E+00_JPRB, 0.75431E+00_JPRB, 0.79642E+00_JPRB, 0.74435E+00_JPRB, 0.78813E+00_JPRB ,&
&  0.78282E+00_JPRB, 0.83859E+00_JPRB, 0.93752E+00_JPRB, 0.96382E+00_JPRB, 0.99880E+00_JPRB, 0.10379E+01_JPRB, 0.98745E+00_JPRB ,&
&  0.93674E+00_JPRB, 0.92801E+00_JPRB, 0.86266E+00_JPRB, 0.82836E+00_JPRB, 0.80284E+00_JPRB, 0.79514E+00_JPRB, 0.78189E+00_JPRB ,&
&  0.78843E+00_JPRB, 0.76160E+00_JPRB, 0.74629E+00_JPRB, 0.74386E+00_JPRB, 0.74475E+00_JPRB, 0.74167E+00_JPRB, 0.75065E+00_JPRB ,&
&  0.79074E+00_JPRB, 0.83220E+00_JPRB, 0.89151E+00_JPRB, 0.92755E+00_JPRB, 0.77365E+00_JPRB, 0.88556E+00_JPRB, 0.92460E+00_JPRB ,&
&  0.97003E+00_JPRB, 0.10340E+01_JPRB, 0.10336E+01_JPRB, 0.11686E+01_JPRB, 0.12656E+01_JPRB, 0.12467E+01_JPRB, 0.13616E+01_JPRB ,&
&  0.18592E+01_JPRB, 0.18908E+01_JPRB, 0.19123E+01_JPRB, 0.22592E+01_JPRB, 0.25474E+01_JPRB, 0.33951E+01_JPRB, 0.18073E+01_JPRB ,&
&  0.14126E+01_JPRB, 0.16623E+01_JPRB, 0.15557E+01_JPRB, 0.14682E+01_JPRB, 0.17751E+01_JPRB, 0.22027E+01_JPRB, 0.18550E+01_JPRB ,&
&  0.81269E+00_JPRB, 0.65497E+00_JPRB, 0.72199E+00_JPRB, 0.72648E+00_JPRB, 0.67380E+00_JPRB, 0.84316E+00_JPRB, 0.15517E+01_JPRB ,&
&  0.14786E+01_JPRB, 0.16525E+01_JPRB, 0.16160E+01_JPRB, 0.14246E+01_JPRB, 0.13070E+01_JPRB, 0.14613E+01_JPRB, 0.18002E+01_JPRB ,&
&  0.16700E+01_JPRB, 0.14738E+01_JPRB, 0.13410E+01_JPRB, 0.12366E+01_JPRB, 0.11151E+01_JPRB, 0.10074E+01_JPRB, 0.92270E+00_JPRB ,&
&  0.86591E+00_JPRB, 0.83563E+00_JPRB, 0.81998E+00_JPRB, 0.80526E+00_JPRB, 0.78737E+00_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.74085E+00_JPRB, 0.72570E+00_JPRB, 0.71597E+00_JPRB, 0.71102E+00_JPRB, 0.71849E+00_JPRB, 0.70742E+00_JPRB, 0.69507E+00_JPRB ,&
&  0.72815E+00_JPRB, 0.64324E+00_JPRB, 0.59815E+00_JPRB, 0.56529E+00_JPRB, 0.56081E+00_JPRB, 0.57038E+00_JPRB, 0.57698E+00_JPRB ,&
&  0.58360E+00_JPRB, 0.60818E+00_JPRB, 0.67798E+00_JPRB, 0.82738E+00_JPRB, 0.91910E+00_JPRB, 0.87712E+00_JPRB, 0.89958E+00_JPRB ,&
&  0.97205E+00_JPRB, 0.99643E+00_JPRB, 0.98791E+00_JPRB, 0.10047E+01_JPRB, 0.10900E+01_JPRB, 0.10887E+01_JPRB, 0.10165E+01_JPRB ,&
&  0.95178E+00_JPRB, 0.94504E+00_JPRB, 0.90116E+00_JPRB, 0.87264E+00_JPRB, 0.86480E+00_JPRB, 0.85681E+00_JPRB, 0.85429E+00_JPRB ,&
&  0.81858E+00_JPRB, 0.80412E+00_JPRB, 0.80050E+00_JPRB, 0.79824E+00_JPRB, 0.81293E+00_JPRB, 0.80786E+00_JPRB, 0.84014E+00_JPRB ,&
&  0.89351E+00_JPRB, 0.93047E+00_JPRB, 0.96829E+00_JPRB, 0.91980E+00_JPRB, 0.92736E+00_JPRB, 0.93840E+00_JPRB, 0.98407E+00_JPRB ,&
&  0.99564E+00_JPRB, 0.10380E+01_JPRB, 0.11035E+01_JPRB, 0.11684E+01_JPRB, 0.12870E+01_JPRB, 0.13802E+01_JPRB, 0.15429E+01_JPRB ,&
&  0.18607E+01_JPRB, 0.19848E+01_JPRB, 0.19238E+01_JPRB, 0.20598E+01_JPRB, 0.23420E+01_JPRB, 0.28881E+01_JPRB, 0.22679E+01_JPRB ,&
&  0.17057E+01_JPRB, 0.14227E+01_JPRB, 0.14824E+01_JPRB, 0.17694E+01_JPRB, 0.20082E+01_JPRB, 0.22760E+01_JPRB, 0.28051E+01_JPRB ,&
&  0.30176E+01_JPRB, 0.17665E+01_JPRB, 0.14461E+01_JPRB, 0.16664E+01_JPRB, 0.13198E+01_JPRB, 0.11607E+01_JPRB, 0.14316E+01_JPRB ,&
&  0.16063E+01_JPRB, 0.16779E+01_JPRB, 0.16117E+01_JPRB, 0.13570E+01_JPRB, 0.13591E+01_JPRB, 0.15821E+01_JPRB, 0.17043E+01_JPRB ,&
&  0.14428E+01_JPRB, 0.13092E+01_JPRB, 0.11809E+01_JPRB, 0.10532E+01_JPRB, 0.95278E+00_JPRB, 0.88420E+00_JPRB, 0.84293E+00_JPRB ,&
&  0.82561E+00_JPRB, 0.81429E+00_JPRB, 0.80163E+00_JPRB, 0.78190E+00_JPRB, 0.76043E+00_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.72678E+00_JPRB, 0.73062E+00_JPRB, 0.74266E+00_JPRB, 0.74834E+00_JPRB, 0.74359E+00_JPRB, 0.72305E+00_JPRB, 0.73958E+00_JPRB ,&
&  0.76314E+00_JPRB, 0.65655E+00_JPRB, 0.60019E+00_JPRB, 0.57671E+00_JPRB, 0.57959E+00_JPRB, 0.59436E+00_JPRB, 0.60128E+00_JPRB ,&
&  0.61485E+00_JPRB, 0.66149E+00_JPRB, 0.80351E+00_JPRB, 0.93424E+00_JPRB, 0.10731E+01_JPRB, 0.12771E+01_JPRB, 0.95642E+00_JPRB ,&
&  0.10006E+01_JPRB, 0.10473E+01_JPRB, 0.10209E+01_JPRB, 0.10870E+01_JPRB, 0.11404E+01_JPRB, 0.11247E+01_JPRB, 0.10564E+01_JPRB ,&
&  0.99536E+00_JPRB, 0.10029E+01_JPRB, 0.97884E+00_JPRB, 0.95465E+00_JPRB, 0.95210E+00_JPRB, 0.95857E+00_JPRB, 0.91985E+00_JPRB ,&
&  0.91413E+00_JPRB, 0.90322E+00_JPRB, 0.87947E+00_JPRB, 0.89185E+00_JPRB, 0.90590E+00_JPRB, 0.90970E+00_JPRB, 0.95444E+00_JPRB ,&
&  0.99493E+00_JPRB, 0.10161E+01_JPRB, 0.10218E+01_JPRB, 0.97396E+00_JPRB, 0.10044E+01_JPRB, 0.10305E+01_JPRB, 0.10730E+01_JPRB ,&
&  0.10336E+01_JPRB, 0.97360E+00_JPRB, 0.11119E+01_JPRB, 0.12890E+01_JPRB, 0.14240E+01_JPRB, 0.14596E+01_JPRB, 0.16182E+01_JPRB ,&
&  0.19848E+01_JPRB, 0.22578E+01_JPRB, 0.23519E+01_JPRB, 0.19612E+01_JPRB, 0.21411E+01_JPRB, 0.25298E+01_JPRB, 0.25807E+01_JPRB ,&
&  0.23405E+01_JPRB, 0.18998E+01_JPRB, 0.19025E+01_JPRB, 0.19608E+01_JPRB, 0.20271E+01_JPRB, 0.22441E+01_JPRB, 0.22849E+01_JPRB ,&
&  0.26180E+01_JPRB, 0.31457E+01_JPRB, 0.27019E+01_JPRB, 0.21638E+01_JPRB, 0.18465E+01_JPRB, 0.16017E+01_JPRB, 0.15830E+01_JPRB ,&
&  0.17470E+01_JPRB, 0.16516E+01_JPRB, 0.15065E+01_JPRB, 0.15466E+01_JPRB, 0.14350E+01_JPRB, 0.15154E+01_JPRB, 0.14448E+01_JPRB ,&
&  0.12037E+01_JPRB, 0.10589E+01_JPRB, 0.91883E+00_JPRB, 0.84324E+00_JPRB, 0.80141E+00_JPRB, 0.79222E+00_JPRB, 0.78527E+00_JPRB ,&
&  0.78274E+00_JPRB, 0.77429E+00_JPRB, 0.76136E+00_JPRB, 0.75146E+00_JPRB, 0.73666E+00_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.74915E+00_JPRB, 0.75912E+00_JPRB, 0.78072E+00_JPRB, 0.78820E+00_JPRB, 0.77508E+00_JPRB, 0.77901E+00_JPRB, 0.80447E+00_JPRB ,&
&  0.80389E+00_JPRB, 0.65685E+00_JPRB, 0.63147E+00_JPRB, 0.63218E+00_JPRB, 0.64610E+00_JPRB, 0.65053E+00_JPRB, 0.65259E+00_JPRB ,&
&  0.68835E+00_JPRB, 0.78864E+00_JPRB, 0.94404E+00_JPRB, 0.10431E+01_JPRB, 0.11376E+01_JPRB, 0.12627E+01_JPRB, 0.12037E+01_JPRB ,&
&  0.96842E+00_JPRB, 0.11780E+01_JPRB, 0.10970E+01_JPRB, 0.12155E+01_JPRB, 0.12173E+01_JPRB, 0.12503E+01_JPRB, 0.11776E+01_JPRB ,&
&  0.11107E+01_JPRB, 0.11119E+01_JPRB, 0.11466E+01_JPRB, 0.11032E+01_JPRB, 0.10762E+01_JPRB, 0.10201E+01_JPRB, 0.10273E+01_JPRB ,&
&  0.10273E+01_JPRB, 0.10112E+01_JPRB, 0.10041E+01_JPRB, 0.10258E+01_JPRB, 0.10389E+01_JPRB, 0.10401E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10622E+01_JPRB, 0.10681E+01_JPRB, 0.10489E+01_JPRB, 0.10652E+01_JPRB, 0.10848E+01_JPRB, 0.11408E+01_JPRB, 0.11906E+01_JPRB ,&
&  0.11707E+01_JPRB, 0.11262E+01_JPRB, 0.12225E+01_JPRB, 0.13295E+01_JPRB, 0.14227E+01_JPRB, 0.16061E+01_JPRB, 0.17234E+01_JPRB ,&
&  0.20826E+01_JPRB, 0.21203E+01_JPRB, 0.24668E+01_JPRB, 0.18266E+01_JPRB, 0.19230E+01_JPRB, 0.22295E+01_JPRB, 0.21385E+01_JPRB ,&
&  0.17409E+01_JPRB, 0.17540E+01_JPRB, 0.17964E+01_JPRB, 0.19110E+01_JPRB, 0.19947E+01_JPRB, 0.20847E+01_JPRB, 0.19718E+01_JPRB ,&
&  0.20044E+01_JPRB, 0.21474E+01_JPRB, 0.20968E+01_JPRB, 0.17769E+01_JPRB, 0.16494E+01_JPRB, 0.14931E+01_JPRB, 0.16237E+01_JPRB ,&
&  0.17587E+01_JPRB, 0.17120E+01_JPRB, 0.17128E+01_JPRB, 0.16571E+01_JPRB, 0.13409E+01_JPRB, 0.12611E+01_JPRB, 0.11045E+01_JPRB ,&
&  0.94066E+00_JPRB, 0.85005E+00_JPRB, 0.76328E+00_JPRB, 0.75347E+00_JPRB, 0.75638E+00_JPRB, 0.75331E+00_JPRB, 0.76457E+00_JPRB ,&
&  0.76066E+00_JPRB, 0.75400E+00_JPRB, 0.75588E+00_JPRB, 0.75304E+00_JPRB, 0.74524E+00_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.78454E+00_JPRB, 0.79531E+00_JPRB, 0.81907E+00_JPRB, 0.84800E+00_JPRB, 0.87844E+00_JPRB, 0.92640E+00_JPRB, 0.89052E+00_JPRB ,&
&  0.85479E+00_JPRB, 0.68694E+00_JPRB, 0.69798E+00_JPRB, 0.70614E+00_JPRB, 0.71799E+00_JPRB, 0.71287E+00_JPRB, 0.73223E+00_JPRB ,&
&  0.80995E+00_JPRB, 0.92896E+00_JPRB, 0.10444E+01_JPRB, 0.11537E+01_JPRB, 0.12590E+01_JPRB, 0.13938E+01_JPRB, 0.13830E+01_JPRB ,&
&  0.11145E+01_JPRB, 0.11093E+01_JPRB, 0.12507E+01_JPRB, 0.13147E+01_JPRB, 0.12898E+01_JPRB, 0.13419E+01_JPRB, 0.13602E+01_JPRB ,&
&  0.13267E+01_JPRB, 0.13055E+01_JPRB, 0.13589E+01_JPRB, 0.12378E+01_JPRB, 0.10948E+01_JPRB, 0.11009E+01_JPRB, 0.11292E+01_JPRB ,&
&  0.11406E+01_JPRB, 0.11459E+01_JPRB, 0.11565E+01_JPRB, 0.11826E+01_JPRB, 0.11945E+01_JPRB, 0.11768E+01_JPRB, 0.11550E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.11629E+01_JPRB, 0.11623E+01_JPRB, 0.11514E+01_JPRB, 0.11972E+01_JPRB, 0.12554E+01_JPRB, 0.12997E+01_JPRB ,&
&  0.13318E+01_JPRB, 0.13878E+01_JPRB, 0.14462E+01_JPRB, 0.15221E+01_JPRB, 0.15235E+01_JPRB, 0.17606E+01_JPRB, 0.18820E+01_JPRB ,&
&  0.20514E+01_JPRB, 0.20790E+01_JPRB, 0.22620E+01_JPRB, 0.16514E+01_JPRB, 0.18629E+01_JPRB, 0.19692E+01_JPRB, 0.18888E+01_JPRB ,&
&  0.16764E+01_JPRB, 0.17234E+01_JPRB, 0.17291E+01_JPRB, 0.17651E+01_JPRB, 0.18432E+01_JPRB, 0.17552E+01_JPRB, 0.17791E+01_JPRB ,&
&  0.16894E+01_JPRB, 0.18527E+01_JPRB, 0.17797E+01_JPRB, 0.16772E+01_JPRB, 0.14975E+01_JPRB, 0.14336E+01_JPRB, 0.15100E+01_JPRB ,&
&  0.16484E+01_JPRB, 0.16290E+01_JPRB, 0.15576E+01_JPRB, 0.13814E+01_JPRB, 0.11050E+01_JPRB, 0.99467E+00_JPRB, 0.87072E+00_JPRB ,&
&  0.82933E+00_JPRB, 0.77326E+00_JPRB, 0.75321E+00_JPRB, 0.76115E+00_JPRB, 0.75833E+00_JPRB, 0.78367E+00_JPRB, 0.78124E+00_JPRB ,&
&  0.76811E+00_JPRB, 0.77403E+00_JPRB, 0.78053E+00_JPRB, 0.78106E+00_JPRB, 0.78095E+00_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.77385E+00_JPRB, 0.76854E+00_JPRB, 0.76748E+00_JPRB, 0.77696E+00_JPRB, 0.79749E+00_JPRB, 0.81130E+00_JPRB, 0.76604E+00_JPRB ,&
&  0.74942E+00_JPRB, 0.74356E+00_JPRB, 0.77135E+00_JPRB, 0.78256E+00_JPRB, 0.78762E+00_JPRB, 0.79354E+00_JPRB, 0.83623E+00_JPRB ,&
&  0.91196E+00_JPRB, 0.10243E+01_JPRB, 0.11350E+01_JPRB, 0.12044E+01_JPRB, 0.12692E+01_JPRB, 0.13625E+01_JPRB, 0.14646E+01_JPRB ,&
&  0.13106E+01_JPRB, 0.11055E+01_JPRB, 0.13096E+01_JPRB, 0.14087E+01_JPRB, 0.13527E+01_JPRB, 0.14425E+01_JPRB, 0.16228E+01_JPRB ,&
&  0.14069E+01_JPRB, 0.13538E+01_JPRB, 0.13622E+01_JPRB, 0.12910E+01_JPRB, 0.11767E+01_JPRB, 0.12008E+01_JPRB, 0.12380E+01_JPRB ,&
&  0.12642E+01_JPRB, 0.12772E+01_JPRB, 0.13152E+01_JPRB, 0.13507E+01_JPRB, 0.13639E+01_JPRB, 0.13420E+01_JPRB, 0.13101E+01_JPRB ,&
&  0.13204E+01_JPRB, 0.13431E+01_JPRB, 0.13742E+01_JPRB, 0.13744E+01_JPRB, 0.14030E+01_JPRB, 0.14639E+01_JPRB, 0.15202E+01_JPRB ,&
&  0.15706E+01_JPRB, 0.15800E+01_JPRB, 0.16615E+01_JPRB, 0.18195E+01_JPRB, 0.19588E+01_JPRB, 0.17876E+01_JPRB, 0.19368E+01_JPRB ,&
&  0.19030E+01_JPRB, 0.18291E+01_JPRB, 0.17387E+01_JPRB, 0.20060E+01_JPRB, 0.14400E+01_JPRB, 0.16319E+01_JPRB, 0.15177E+01_JPRB ,&
&  0.17002E+01_JPRB, 0.16572E+01_JPRB, 0.16391E+01_JPRB, 0.16802E+01_JPRB, 0.17335E+01_JPRB, 0.16040E+01_JPRB, 0.15902E+01_JPRB ,&
&  0.16231E+01_JPRB, 0.16493E+01_JPRB, 0.16332E+01_JPRB, 0.16265E+01_JPRB, 0.15218E+01_JPRB, 0.15059E+01_JPRB, 0.14929E+01_JPRB ,&
&  0.15470E+01_JPRB, 0.14195E+01_JPRB, 0.12603E+01_JPRB, 0.10808E+01_JPRB, 0.89969E+00_JPRB, 0.84090E+00_JPRB, 0.83047E+00_JPRB ,&
&  0.83845E+00_JPRB, 0.85128E+00_JPRB, 0.80739E+00_JPRB, 0.78465E+00_JPRB, 0.78634E+00_JPRB, 0.81090E+00_JPRB, 0.79985E+00_JPRB ,&
&  0.80190E+00_JPRB, 0.79714E+00_JPRB, 0.80130E+00_JPRB, 0.79725E+00_JPRB, 0.78957E+00_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.77241E+00_JPRB, 0.76429E+00_JPRB, 0.75574E+00_JPRB, 0.76646E+00_JPRB, 0.77241E+00_JPRB, 0.76868E+00_JPRB, 0.75668E+00_JPRB ,&
&  0.76066E+00_JPRB, 0.78668E+00_JPRB, 0.81791E+00_JPRB, 0.83363E+00_JPRB, 0.83712E+00_JPRB, 0.84617E+00_JPRB, 0.88961E+00_JPRB ,&
&  0.96005E+00_JPRB, 0.10674E+01_JPRB, 0.11981E+01_JPRB, 0.12908E+01_JPRB, 0.13704E+01_JPRB, 0.14183E+01_JPRB, 0.15185E+01_JPRB ,&
&  0.15124E+01_JPRB, 0.15391E+01_JPRB, 0.14975E+01_JPRB, 0.12519E+01_JPRB, 0.11685E+01_JPRB, 0.14308E+01_JPRB, 0.14380E+01_JPRB ,&
&  0.13734E+01_JPRB, 0.13396E+01_JPRB, 0.13139E+01_JPRB, 0.12894E+01_JPRB, 0.12768E+01_JPRB, 0.13191E+01_JPRB, 0.13630E+01_JPRB ,&
&  0.13854E+01_JPRB, 0.14289E+01_JPRB, 0.14733E+01_JPRB, 0.15057E+01_JPRB, 0.15255E+01_JPRB, 0.15525E+01_JPRB, 0.15571E+01_JPRB ,&
&  0.16107E+01_JPRB, 0.16640E+01_JPRB, 0.17237E+01_JPRB, 0.17450E+01_JPRB, 0.17228E+01_JPRB, 0.17725E+01_JPRB, 0.18113E+01_JPRB ,&
&  0.18528E+01_JPRB, 0.18435E+01_JPRB, 0.18270E+01_JPRB, 0.18941E+01_JPRB, 0.19165E+01_JPRB, 0.16674E+01_JPRB, 0.16514E+01_JPRB ,&
&  0.17282E+01_JPRB, 0.16677E+01_JPRB, 0.14773E+01_JPRB, 0.18170E+01_JPRB, 0.11833E+01_JPRB, 0.14560E+01_JPRB, 0.16403E+01_JPRB ,&
&  0.15986E+01_JPRB, 0.15713E+01_JPRB, 0.15778E+01_JPRB, 0.15975E+01_JPRB, 0.15919E+01_JPRB, 0.14845E+01_JPRB, 0.15462E+01_JPRB ,&
&  0.16022E+01_JPRB, 0.15694E+01_JPRB, 0.15409E+01_JPRB, 0.15232E+01_JPRB, 0.14822E+01_JPRB, 0.13787E+01_JPRB, 0.12329E+01_JPRB ,&
&  0.10742E+01_JPRB, 0.12318E+01_JPRB, 0.11150E+01_JPRB, 0.10399E+01_JPRB, 0.92074E+00_JPRB, 0.80297E+00_JPRB, 0.83324E+00_JPRB ,&
&  0.85641E+00_JPRB, 0.84105E+00_JPRB, 0.81570E+00_JPRB, 0.81723E+00_JPRB, 0.81728E+00_JPRB, 0.82151E+00_JPRB, 0.81002E+00_JPRB ,&
&  0.81143E+00_JPRB, 0.81187E+00_JPRB, 0.80970E+00_JPRB, 0.79851E+00_JPRB, 0.78560E+00_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.74845E+00_JPRB, 0.74156E+00_JPRB, 0.73646E+00_JPRB, 0.76587E+00_JPRB, 0.76083E+00_JPRB, 0.74817E+00_JPRB, 0.75463E+00_JPRB ,&
&  0.76543E+00_JPRB, 0.78145E+00_JPRB, 0.80326E+00_JPRB, 0.82583E+00_JPRB, 0.85774E+00_JPRB, 0.90783E+00_JPRB, 0.96148E+00_JPRB ,&
&  0.10239E+01_JPRB, 0.10955E+01_JPRB, 0.12137E+01_JPRB, 0.13636E+01_JPRB, 0.15509E+01_JPRB, 0.16451E+01_JPRB, 0.16389E+01_JPRB ,&
&  0.15261E+01_JPRB, 0.15727E+01_JPRB, 0.16498E+01_JPRB, 0.17003E+01_JPRB, 0.14750E+01_JPRB, 0.15595E+01_JPRB, 0.16840E+01_JPRB ,&
&  0.16708E+01_JPRB, 0.15077E+01_JPRB, 0.13936E+01_JPRB, 0.13750E+01_JPRB, 0.13684E+01_JPRB, 0.14209E+01_JPRB, 0.14760E+01_JPRB ,&
&  0.15168E+01_JPRB, 0.15705E+01_JPRB, 0.16088E+01_JPRB, 0.16501E+01_JPRB, 0.17129E+01_JPRB, 0.18090E+01_JPRB, 0.18553E+01_JPRB ,&
&  0.18961E+01_JPRB, 0.19830E+01_JPRB, 0.20190E+01_JPRB, 0.19779E+01_JPRB, 0.20274E+01_JPRB, 0.20564E+01_JPRB, 0.20765E+01_JPRB ,&
&  0.21238E+01_JPRB, 0.20176E+01_JPRB, 0.20363E+01_JPRB, 0.20731E+01_JPRB, 0.19685E+01_JPRB, 0.17480E+01_JPRB, 0.16804E+01_JPRB ,&
&  0.17008E+01_JPRB, 0.16767E+01_JPRB, 0.12334E+01_JPRB, 0.16266E+01_JPRB, 0.19211E+01_JPRB, 0.16663E+01_JPRB, 0.16135E+01_JPRB ,&
&  0.15306E+01_JPRB, 0.15348E+01_JPRB, 0.15706E+01_JPRB, 0.16018E+01_JPRB, 0.16156E+01_JPRB, 0.16166E+01_JPRB, 0.15277E+01_JPRB ,&
&  0.16603E+01_JPRB, 0.15752E+01_JPRB, 0.14758E+01_JPRB, 0.14042E+01_JPRB, 0.13088E+01_JPRB, 0.11530E+01_JPRB, 0.10058E+01_JPRB ,&
&  0.92297E+00_JPRB, 0.11578E+01_JPRB, 0.11190E+01_JPRB, 0.11936E+01_JPRB, 0.11906E+01_JPRB, 0.81651E+00_JPRB, 0.85652E+00_JPRB ,&
&  0.85693E+00_JPRB, 0.85078E+00_JPRB, 0.83127E+00_JPRB, 0.80534E+00_JPRB, 0.80343E+00_JPRB, 0.80711E+00_JPRB, 0.80828E+00_JPRB ,&
&  0.80442E+00_JPRB, 0.79256E+00_JPRB, 0.78604E+00_JPRB, 0.76825E+00_JPRB, 0.75815E+00_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.76137E+00_JPRB, 0.76292E+00_JPRB, 0.75902E+00_JPRB, 0.77231E+00_JPRB, 0.76628E+00_JPRB, 0.77988E+00_JPRB, 0.76441E+00_JPRB ,&
&  0.74878E+00_JPRB, 0.74497E+00_JPRB, 0.74906E+00_JPRB, 0.75383E+00_JPRB, 0.77599E+00_JPRB, 0.82392E+00_JPRB, 0.88445E+00_JPRB ,&
&  0.94421E+00_JPRB, 0.95997E+00_JPRB, 0.10022E+01_JPRB, 0.11100E+01_JPRB, 0.13335E+01_JPRB, 0.15496E+01_JPRB, 0.15943E+01_JPRB ,&
&  0.16833E+01_JPRB, 0.18243E+01_JPRB, 0.19464E+01_JPRB, 0.20663E+01_JPRB, 0.20531E+01_JPRB, 0.20200E+01_JPRB, 0.20395E+01_JPRB ,&
&  0.17612E+01_JPRB, 0.14452E+01_JPRB, 0.13981E+01_JPRB, 0.14162E+01_JPRB, 0.14584E+01_JPRB, 0.14988E+01_JPRB, 0.15326E+01_JPRB ,&
&  0.15740E+01_JPRB, 0.16323E+01_JPRB, 0.16858E+01_JPRB, 0.17650E+01_JPRB, 0.18794E+01_JPRB, 0.20123E+01_JPRB, 0.20924E+01_JPRB ,&
&  0.21332E+01_JPRB, 0.21965E+01_JPRB, 0.22874E+01_JPRB, 0.21931E+01_JPRB, 0.24311E+01_JPRB, 0.25128E+01_JPRB, 0.25412E+01_JPRB ,&
&  0.24954E+01_JPRB, 0.23465E+01_JPRB, 0.23406E+01_JPRB, 0.23190E+01_JPRB, 0.22821E+01_JPRB, 0.20560E+01_JPRB, 0.18972E+01_JPRB ,&
&  0.18615E+01_JPRB, 0.15927E+01_JPRB, 0.11672E+01_JPRB, 0.12602E+01_JPRB, 0.14016E+01_JPRB, 0.12936E+01_JPRB, 0.16083E+01_JPRB ,&
&  0.15441E+01_JPRB, 0.15020E+01_JPRB, 0.14989E+01_JPRB, 0.15125E+01_JPRB, 0.15811E+01_JPRB, 0.15650E+01_JPRB, 0.15349E+01_JPRB ,&
&  0.15677E+01_JPRB, 0.14393E+01_JPRB, 0.13114E+01_JPRB, 0.11893E+01_JPRB, 0.10644E+01_JPRB, 0.96443E+00_JPRB, 0.91585E+00_JPRB ,&
&  0.92174E+00_JPRB, 0.93655E+00_JPRB, 0.88215E+00_JPRB, 0.84520E+00_JPRB, 0.76733E+00_JPRB, 0.79528E+00_JPRB, 0.80278E+00_JPRB ,&
&  0.83166E+00_JPRB, 0.82763E+00_JPRB, 0.81365E+00_JPRB, 0.79235E+00_JPRB, 0.78052E+00_JPRB, 0.77254E+00_JPRB, 0.77082E+00_JPRB ,&
&  0.77963E+00_JPRB, 0.79016E+00_JPRB, 0.78485E+00_JPRB, 0.77745E+00_JPRB, 0.76834E+00_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.77006E+00_JPRB, 0.76634E+00_JPRB, 0.76547E+00_JPRB, 0.78076E+00_JPRB, 0.79366E+00_JPRB, 0.78360E+00_JPRB, 0.79109E+00_JPRB ,&
&  0.76940E+00_JPRB, 0.75217E+00_JPRB, 0.76240E+00_JPRB, 0.76782E+00_JPRB, 0.78371E+00_JPRB, 0.79750E+00_JPRB, 0.80551E+00_JPRB ,&
&  0.80279E+00_JPRB, 0.79541E+00_JPRB, 0.76377E+00_JPRB, 0.77454E+00_JPRB, 0.85790E+00_JPRB, 0.99669E+00_JPRB, 0.12595E+01_JPRB ,&
&  0.15506E+01_JPRB, 0.16967E+01_JPRB, 0.18070E+01_JPRB, 0.18294E+01_JPRB, 0.17382E+01_JPRB, 0.16027E+01_JPRB, 0.20737E+01_JPRB ,&
&  0.19775E+01_JPRB, 0.14299E+01_JPRB, 0.14787E+01_JPRB, 0.13925E+01_JPRB, 0.14342E+01_JPRB, 0.14462E+01_JPRB, 0.14815E+01_JPRB ,&
&  0.15056E+01_JPRB, 0.15675E+01_JPRB, 0.16611E+01_JPRB, 0.17897E+01_JPRB, 0.19214E+01_JPRB, 0.20611E+01_JPRB, 0.21688E+01_JPRB ,&
&  0.22329E+01_JPRB, 0.22970E+01_JPRB, 0.24301E+01_JPRB, 0.24635E+01_JPRB, 0.24794E+01_JPRB, 0.26067E+01_JPRB, 0.27385E+01_JPRB ,&
&  0.28298E+01_JPRB, 0.29780E+01_JPRB, 0.25994E+01_JPRB, 0.25534E+01_JPRB, 0.26290E+01_JPRB, 0.22309E+01_JPRB, 0.20996E+01_JPRB ,&
&  0.20284E+01_JPRB, 0.15830E+01_JPRB, 0.10718E+01_JPRB, 0.11416E+01_JPRB, 0.12446E+01_JPRB, 0.13359E+01_JPRB, 0.14487E+01_JPRB ,&
&  0.13970E+01_JPRB, 0.13322E+01_JPRB, 0.13362E+01_JPRB, 0.14096E+01_JPRB, 0.14289E+01_JPRB, 0.13783E+01_JPRB, 0.13208E+01_JPRB ,&
&  0.12847E+01_JPRB, 0.11593E+01_JPRB, 0.10925E+01_JPRB, 0.99509E+00_JPRB, 0.92688E+00_JPRB, 0.88379E+00_JPRB, 0.83136E+00_JPRB ,&
&  0.87204E+00_JPRB, 0.90887E+00_JPRB, 0.10094E+01_JPRB, 0.94132E+00_JPRB, 0.74072E+00_JPRB, 0.79318E+00_JPRB, 0.79395E+00_JPRB ,&
&  0.80078E+00_JPRB, 0.80500E+00_JPRB, 0.81935E+00_JPRB, 0.81065E+00_JPRB, 0.79187E+00_JPRB, 0.77358E+00_JPRB, 0.75713E+00_JPRB ,&
&  0.77037E+00_JPRB, 0.78657E+00_JPRB, 0.79381E+00_JPRB, 0.78124E+00_JPRB, 0.77101E+00_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.83790E+00_JPRB, 0.84020E+00_JPRB, 0.85638E+00_JPRB, 0.85893E+00_JPRB, 0.85866E+00_JPRB, 0.86624E+00_JPRB, 0.86820E+00_JPRB ,&
&  0.86200E+00_JPRB, 0.86925E+00_JPRB, 0.87932E+00_JPRB, 0.89069E+00_JPRB, 0.90510E+00_JPRB, 0.91786E+00_JPRB, 0.92520E+00_JPRB ,&
&  0.91609E+00_JPRB, 0.90704E+00_JPRB, 0.93045E+00_JPRB, 0.94941E+00_JPRB, 0.94732E+00_JPRB, 0.94223E+00_JPRB, 0.96174E+00_JPRB ,&
&  0.10122E+01_JPRB, 0.10432E+01_JPRB, 0.10532E+01_JPRB, 0.10411E+01_JPRB, 0.10754E+01_JPRB, 0.12851E+01_JPRB, 0.19989E+01_JPRB ,&
&  0.19739E+01_JPRB, 0.14480E+01_JPRB, 0.13359E+01_JPRB, 0.12166E+01_JPRB, 0.12441E+01_JPRB, 0.13206E+01_JPRB, 0.13271E+01_JPRB ,&
&  0.13099E+01_JPRB, 0.13439E+01_JPRB, 0.13819E+01_JPRB, 0.14179E+01_JPRB, 0.14774E+01_JPRB, 0.15247E+01_JPRB, 0.15500E+01_JPRB ,&
&  0.15213E+01_JPRB, 0.15192E+01_JPRB, 0.15695E+01_JPRB, 0.16615E+01_JPRB, 0.18912E+01_JPRB, 0.22625E+01_JPRB, 0.21955E+01_JPRB ,&
&  0.21984E+01_JPRB, 0.23578E+01_JPRB, 0.24731E+01_JPRB, 0.25233E+01_JPRB, 0.24055E+01_JPRB, 0.21499E+01_JPRB, 0.19318E+01_JPRB ,&
&  0.17125E+01_JPRB, 0.13944E+01_JPRB, 0.11404E+01_JPRB, 0.12808E+01_JPRB, 0.13286E+01_JPRB, 0.13843E+01_JPRB, 0.12904E+01_JPRB ,&
&  0.12278E+01_JPRB, 0.11132E+01_JPRB, 0.11020E+01_JPRB, 0.11593E+01_JPRB, 0.12183E+01_JPRB, 0.11970E+01_JPRB, 0.11374E+01_JPRB ,&
&  0.10472E+01_JPRB, 0.10040E+01_JPRB, 0.98678E+00_JPRB, 0.92607E+00_JPRB, 0.89990E+00_JPRB, 0.77448E+00_JPRB, 0.80657E+00_JPRB ,&
&  0.82235E+00_JPRB, 0.86110E+00_JPRB, 0.71927E+00_JPRB, 0.92251E+00_JPRB, 0.75812E+00_JPRB, 0.73480E+00_JPRB, 0.75066E+00_JPRB ,&
&  0.77137E+00_JPRB, 0.79975E+00_JPRB, 0.80592E+00_JPRB, 0.88553E+00_JPRB, 0.87086E+00_JPRB, 0.85925E+00_JPRB, 0.86257E+00_JPRB ,&
&  0.83195E+00_JPRB, 0.82452E+00_JPRB, 0.82348E+00_JPRB, 0.83288E+00_JPRB, 0.83563E+00_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.90173E+00_JPRB, 0.90452E+00_JPRB, 0.91049E+00_JPRB, 0.92092E+00_JPRB, 0.93096E+00_JPRB, 0.94124E+00_JPRB, 0.94564E+00_JPRB ,&
&  0.94086E+00_JPRB, 0.94349E+00_JPRB, 0.95342E+00_JPRB, 0.96906E+00_JPRB, 0.98347E+00_JPRB, 0.99592E+00_JPRB, 0.10104E+01_JPRB ,&
&  0.10093E+01_JPRB, 0.10122E+01_JPRB, 0.10307E+01_JPRB, 0.10520E+01_JPRB, 0.10633E+01_JPRB, 0.10625E+01_JPRB, 0.10652E+01_JPRB ,&
&  0.10602E+01_JPRB, 0.10705E+01_JPRB, 0.11000E+01_JPRB, 0.11216E+01_JPRB, 0.11497E+01_JPRB, 0.12200E+01_JPRB, 0.14348E+01_JPRB ,&
&  0.14241E+01_JPRB, 0.15823E+01_JPRB, 0.14202E+01_JPRB, 0.12726E+01_JPRB, 0.12679E+01_JPRB, 0.12146E+01_JPRB, 0.11777E+01_JPRB ,&
&  0.11382E+01_JPRB, 0.11169E+01_JPRB, 0.10713E+01_JPRB, 0.10783E+01_JPRB, 0.10718E+01_JPRB, 0.11083E+01_JPRB, 0.11029E+01_JPRB ,&
&  0.11542E+01_JPRB, 0.12078E+01_JPRB, 0.13111E+01_JPRB, 0.14060E+01_JPRB, 0.15373E+01_JPRB, 0.16153E+01_JPRB, 0.14754E+01_JPRB ,&
&  0.14453E+01_JPRB, 0.15881E+01_JPRB, 0.17872E+01_JPRB, 0.19283E+01_JPRB, 0.18151E+01_JPRB, 0.16194E+01_JPRB, 0.14544E+01_JPRB ,&
&  0.12897E+01_JPRB, 0.11866E+01_JPRB, 0.13287E+01_JPRB, 0.13102E+01_JPRB, 0.13317E+01_JPRB, 0.12809E+01_JPRB, 0.11492E+01_JPRB ,&
&  0.10261E+01_JPRB, 0.88752E+00_JPRB, 0.87395E+00_JPRB, 0.87571E+00_JPRB, 0.90254E+00_JPRB, 0.88859E+00_JPRB, 0.87288E+00_JPRB ,&
&  0.81491E+00_JPRB, 0.76183E+00_JPRB, 0.79754E+00_JPRB, 0.83271E+00_JPRB, 0.86250E+00_JPRB, 0.10437E+01_JPRB, 0.10360E+01_JPRB ,&
&  0.76152E+00_JPRB, 0.66611E+00_JPRB, 0.50667E+00_JPRB, 0.70001E+00_JPRB, 0.64999E+00_JPRB, 0.69315E+00_JPRB, 0.77048E+00_JPRB ,&
&  0.87884E+00_JPRB, 0.97243E+00_JPRB, 0.10355E+01_JPRB, 0.99554E+00_JPRB, 0.92017E+00_JPRB, 0.90383E+00_JPRB, 0.89328E+00_JPRB ,&
&  0.89696E+00_JPRB, 0.89899E+00_JPRB, 0.89872E+00_JPRB, 0.89738E+00_JPRB, 0.89256E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.93822E+00_JPRB, 0.94743E+00_JPRB, 0.96084E+00_JPRB, 0.97850E+00_JPRB, 0.99595E+00_JPRB, 0.10120E+01_JPRB, 0.10252E+01_JPRB ,&
&  0.10348E+01_JPRB, 0.10497E+01_JPRB, 0.10730E+01_JPRB, 0.11006E+01_JPRB, 0.11125E+01_JPRB, 0.11294E+01_JPRB, 0.11343E+01_JPRB ,&
&  0.11586E+01_JPRB, 0.11867E+01_JPRB, 0.11997E+01_JPRB, 0.12147E+01_JPRB, 0.12255E+01_JPRB, 0.12413E+01_JPRB, 0.12612E+01_JPRB ,&
&  0.12798E+01_JPRB, 0.13219E+01_JPRB, 0.13839E+01_JPRB, 0.14350E+01_JPRB, 0.15051E+01_JPRB, 0.16302E+01_JPRB, 0.14141E+01_JPRB ,&
&  0.15686E+01_JPRB, 0.16048E+01_JPRB, 0.15208E+01_JPRB, 0.13498E+01_JPRB, 0.12933E+01_JPRB, 0.12446E+01_JPRB, 0.12112E+01_JPRB ,&
&  0.11944E+01_JPRB, 0.11565E+01_JPRB, 0.11770E+01_JPRB, 0.11769E+01_JPRB, 0.11806E+01_JPRB, 0.12548E+01_JPRB, 0.12962E+01_JPRB ,&
&  0.13256E+01_JPRB, 0.13583E+01_JPRB, 0.14050E+01_JPRB, 0.14344E+01_JPRB, 0.14875E+01_JPRB, 0.15069E+01_JPRB, 0.14357E+01_JPRB ,&
&  0.13356E+01_JPRB, 0.12834E+01_JPRB, 0.12268E+01_JPRB, 0.12418E+01_JPRB, 0.12096E+01_JPRB, 0.10606E+01_JPRB, 0.90333E+00_JPRB ,&
&  0.99808E+00_JPRB, 0.97915E+00_JPRB, 0.10215E+01_JPRB, 0.13788E+01_JPRB, 0.13178E+01_JPRB, 0.12313E+01_JPRB, 0.10297E+01_JPRB ,&
&  0.91225E+00_JPRB, 0.76215E+00_JPRB, 0.70508E+00_JPRB, 0.71249E+00_JPRB, 0.70358E+00_JPRB, 0.69768E+00_JPRB, 0.67079E+00_JPRB ,&
&  0.62419E+00_JPRB, 0.59844E+00_JPRB, 0.63841E+00_JPRB, 0.71430E+00_JPRB, 0.83637E+00_JPRB, 0.88239E+00_JPRB, 0.11528E+01_JPRB ,&
&  0.79669E+00_JPRB, 0.49891E+00_JPRB, 0.56969E+00_JPRB, 0.62100E+00_JPRB, 0.60824E+00_JPRB, 0.63720E+00_JPRB, 0.76562E+00_JPRB ,&
&  0.97240E+00_JPRB, 0.10332E+01_JPRB, 0.10515E+01_JPRB, 0.99871E+00_JPRB, 0.98975E+00_JPRB, 0.98651E+00_JPRB, 0.96820E+00_JPRB ,&
&  0.96016E+00_JPRB, 0.95613E+00_JPRB, 0.95252E+00_JPRB, 0.94217E+00_JPRB, 0.93562E+00_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.98519E+00_JPRB, 0.99822E+00_JPRB, 0.10187E+01_JPRB, 0.10471E+01_JPRB, 0.10624E+01_JPRB, 0.10867E+01_JPRB, 0.10973E+01_JPRB ,&
&  0.11102E+01_JPRB, 0.11407E+01_JPRB, 0.11690E+01_JPRB, 0.11999E+01_JPRB, 0.12286E+01_JPRB, 0.12562E+01_JPRB, 0.12828E+01_JPRB ,&
&  0.13050E+01_JPRB, 0.13267E+01_JPRB, 0.13360E+01_JPRB, 0.13332E+01_JPRB, 0.13165E+01_JPRB, 0.13082E+01_JPRB, 0.12892E+01_JPRB ,&
&  0.12725E+01_JPRB, 0.12752E+01_JPRB, 0.12869E+01_JPRB, 0.12954E+01_JPRB, 0.13096E+01_JPRB, 0.13857E+01_JPRB, 0.10311E+01_JPRB ,&
&  0.14969E+01_JPRB, 0.15802E+01_JPRB, 0.15726E+01_JPRB, 0.14473E+01_JPRB, 0.13871E+01_JPRB, 0.14020E+01_JPRB, 0.14513E+01_JPRB ,&
&  0.15313E+01_JPRB, 0.16409E+01_JPRB, 0.17448E+01_JPRB, 0.16685E+01_JPRB, 0.15666E+01_JPRB, 0.16035E+01_JPRB, 0.16484E+01_JPRB ,&
&  0.16794E+01_JPRB, 0.16887E+01_JPRB, 0.16985E+01_JPRB, 0.16817E+01_JPRB, 0.16686E+01_JPRB, 0.16466E+01_JPRB, 0.15924E+01_JPRB ,&
&  0.15116E+01_JPRB, 0.14065E+01_JPRB, 0.12678E+01_JPRB, 0.13480E+01_JPRB, 0.13471E+01_JPRB, 0.11328E+01_JPRB, 0.81668E+00_JPRB ,&
&  0.74246E+00_JPRB, 0.87619E+00_JPRB, 0.10625E+01_JPRB, 0.14128E+01_JPRB, 0.12652E+01_JPRB, 0.11338E+01_JPRB, 0.10229E+01_JPRB ,&
&  0.85468E+00_JPRB, 0.74389E+00_JPRB, 0.67397E+00_JPRB, 0.66629E+00_JPRB, 0.66608E+00_JPRB, 0.67716E+00_JPRB, 0.66166E+00_JPRB ,&
&  0.63808E+00_JPRB, 0.61654E+00_JPRB, 0.63685E+00_JPRB, 0.68123E+00_JPRB, 0.76640E+00_JPRB, 0.75028E+00_JPRB, 0.10969E+01_JPRB ,&
&  0.83190E+00_JPRB, 0.68481E+00_JPRB, 0.89507E+00_JPRB, 0.54585E+00_JPRB, 0.68797E+00_JPRB, 0.77382E+00_JPRB, 0.88940E+00_JPRB ,&
&  0.92194E+00_JPRB, 0.12166E+01_JPRB, 0.15154E+01_JPRB, 0.13426E+01_JPRB, 0.12063E+01_JPRB, 0.10917E+01_JPRB, 0.10451E+01_JPRB ,&
&  0.10317E+01_JPRB, 0.10194E+01_JPRB, 0.10135E+01_JPRB, 0.10028E+01_JPRB, 0.98835E+00_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.10650E+01_JPRB, 0.10687E+01_JPRB, 0.10631E+01_JPRB, 0.10952E+01_JPRB, 0.10939E+01_JPRB, 0.11105E+01_JPRB, 0.11128E+01_JPRB ,&
&  0.11368E+01_JPRB, 0.11568E+01_JPRB, 0.11805E+01_JPRB, 0.12190E+01_JPRB, 0.12319E+01_JPRB, 0.12494E+01_JPRB, 0.12803E+01_JPRB ,&
&  0.13101E+01_JPRB, 0.13478E+01_JPRB, 0.13476E+01_JPRB, 0.13344E+01_JPRB, 0.13015E+01_JPRB, 0.12797E+01_JPRB, 0.12535E+01_JPRB ,&
&  0.12310E+01_JPRB, 0.12272E+01_JPRB, 0.12351E+01_JPRB, 0.12518E+01_JPRB, 0.12657E+01_JPRB, 0.13053E+01_JPRB, 0.90408E+00_JPRB ,&
&  0.14466E+01_JPRB, 0.16260E+01_JPRB, 0.15905E+01_JPRB, 0.15663E+01_JPRB, 0.14973E+01_JPRB, 0.15418E+01_JPRB, 0.16245E+01_JPRB ,&
&  0.17466E+01_JPRB, 0.18675E+01_JPRB, 0.19291E+01_JPRB, 0.19523E+01_JPRB, 0.19465E+01_JPRB, 0.20129E+01_JPRB, 0.20743E+01_JPRB ,&
&  0.21167E+01_JPRB, 0.21429E+01_JPRB, 0.21717E+01_JPRB, 0.21835E+01_JPRB, 0.22058E+01_JPRB, 0.22399E+01_JPRB, 0.22296E+01_JPRB ,&
&  0.21793E+01_JPRB, 0.21318E+01_JPRB, 0.21488E+01_JPRB, 0.22646E+01_JPRB, 0.22372E+01_JPRB, 0.19140E+01_JPRB, 0.14556E+01_JPRB ,&
&  0.92354E+00_JPRB, 0.75568E+00_JPRB, 0.10223E+01_JPRB, 0.13648E+01_JPRB, 0.12029E+01_JPRB, 0.10669E+01_JPRB, 0.95109E+00_JPRB ,&
&  0.88088E+00_JPRB, 0.78323E+00_JPRB, 0.70036E+00_JPRB, 0.69372E+00_JPRB, 0.71749E+00_JPRB, 0.77111E+00_JPRB, 0.76098E+00_JPRB ,&
&  0.77449E+00_JPRB, 0.78092E+00_JPRB, 0.77213E+00_JPRB, 0.78623E+00_JPRB, 0.87360E+00_JPRB, 0.92678E+00_JPRB, 0.85363E+00_JPRB ,&
&  0.97497E+00_JPRB, 0.85252E+00_JPRB, 0.85431E+00_JPRB, 0.94148E+00_JPRB, 0.97593E+00_JPRB, 0.11612E+01_JPRB, 0.14525E+01_JPRB ,&
&  0.18978E+01_JPRB, 0.18526E+01_JPRB, 0.24607E+01_JPRB, 0.37554E+01_JPRB, 0.29505E+01_JPRB, 0.21059E+01_JPRB, 0.11390E+01_JPRB ,&
&  0.11245E+01_JPRB, 0.10969E+01_JPRB, 0.11052E+01_JPRB, 0.10872E+01_JPRB, 0.10727E+01_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.10284E+01_JPRB, 0.10139E+01_JPRB, 0.10141E+01_JPRB, 0.10135E+01_JPRB, 0.10114E+01_JPRB, 0.10210E+01_JPRB, 0.10443E+01_JPRB ,&
&  0.10561E+01_JPRB, 0.10925E+01_JPRB, 0.11484E+01_JPRB, 0.11846E+01_JPRB, 0.12324E+01_JPRB, 0.12371E+01_JPRB, 0.12051E+01_JPRB ,&
&  0.11981E+01_JPRB, 0.12316E+01_JPRB, 0.12621E+01_JPRB, 0.12741E+01_JPRB, 0.12377E+01_JPRB, 0.12172E+01_JPRB, 0.12312E+01_JPRB ,&
&  0.12674E+01_JPRB, 0.12997E+01_JPRB, 0.13327E+01_JPRB, 0.13829E+01_JPRB, 0.13894E+01_JPRB, 0.13165E+01_JPRB, 0.91639E+00_JPRB ,&
&  0.13380E+01_JPRB, 0.15901E+01_JPRB, 0.16636E+01_JPRB, 0.16397E+01_JPRB, 0.16458E+01_JPRB, 0.15951E+01_JPRB, 0.16668E+01_JPRB ,&
&  0.17950E+01_JPRB, 0.19308E+01_JPRB, 0.19610E+01_JPRB, 0.19065E+01_JPRB, 0.21843E+01_JPRB, 0.22547E+01_JPRB, 0.23593E+01_JPRB ,&
&  0.24447E+01_JPRB, 0.25229E+01_JPRB, 0.26007E+01_JPRB, 0.26915E+01_JPRB, 0.27678E+01_JPRB, 0.28437E+01_JPRB, 0.29009E+01_JPRB ,&
&  0.29077E+01_JPRB, 0.28990E+01_JPRB, 0.29617E+01_JPRB, 0.28399E+01_JPRB, 0.27672E+01_JPRB, 0.28488E+01_JPRB, 0.24041E+01_JPRB ,&
&  0.13719E+01_JPRB, 0.89806E+00_JPRB, 0.11297E+01_JPRB, 0.13055E+01_JPRB, 0.11311E+01_JPRB, 0.10292E+01_JPRB, 0.10003E+01_JPRB ,&
&  0.90284E+00_JPRB, 0.80476E+00_JPRB, 0.77614E+00_JPRB, 0.82306E+00_JPRB, 0.90236E+00_JPRB, 0.97244E+00_JPRB, 0.98861E+00_JPRB ,&
&  0.95416E+00_JPRB, 0.93254E+00_JPRB, 0.93377E+00_JPRB, 0.95621E+00_JPRB, 0.10154E+01_JPRB, 0.11453E+01_JPRB, 0.13295E+01_JPRB ,&
&  0.12676E+01_JPRB, 0.95553E+00_JPRB, 0.10493E+01_JPRB, 0.11059E+01_JPRB, 0.12122E+01_JPRB, 0.13686E+01_JPRB, 0.16660E+01_JPRB ,&
&  0.21290E+01_JPRB, 0.26189E+01_JPRB, 0.25636E+01_JPRB, 0.21480E+01_JPRB, 0.27096E+01_JPRB, 0.18191E+01_JPRB, 0.97604E+00_JPRB ,&
&  0.10069E+01_JPRB, 0.10261E+01_JPRB, 0.10453E+01_JPRB, 0.10421E+01_JPRB, 0.10380E+01_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.88867E+00_JPRB, 0.87239E+00_JPRB, 0.86136E+00_JPRB, 0.85132E+00_JPRB, 0.86327E+00_JPRB, 0.89830E+00_JPRB, 0.93876E+00_JPRB ,&
&  0.98010E+00_JPRB, 0.10271E+01_JPRB, 0.10894E+01_JPRB, 0.11118E+01_JPRB, 0.11507E+01_JPRB, 0.11492E+01_JPRB, 0.11379E+01_JPRB ,&
&  0.11153E+01_JPRB, 0.11270E+01_JPRB, 0.11510E+01_JPRB, 0.11763E+01_JPRB, 0.11859E+01_JPRB, 0.12136E+01_JPRB, 0.12504E+01_JPRB ,&
&  0.12996E+01_JPRB, 0.13477E+01_JPRB, 0.14117E+01_JPRB, 0.15171E+01_JPRB, 0.15516E+01_JPRB, 0.15558E+01_JPRB, 0.15739E+01_JPRB ,&
&  0.87140E+00_JPRB, 0.14173E+01_JPRB, 0.15579E+01_JPRB, 0.15588E+01_JPRB, 0.16874E+01_JPRB, 0.17259E+01_JPRB, 0.17727E+01_JPRB ,&
&  0.18863E+01_JPRB, 0.18908E+01_JPRB, 0.19221E+01_JPRB, 0.20269E+01_JPRB, 0.22161E+01_JPRB, 0.23019E+01_JPRB, 0.23912E+01_JPRB ,&
&  0.24325E+01_JPRB, 0.24659E+01_JPRB, 0.25560E+01_JPRB, 0.26782E+01_JPRB, 0.28010E+01_JPRB, 0.29245E+01_JPRB, 0.30825E+01_JPRB ,&
&  0.31880E+01_JPRB, 0.32778E+01_JPRB, 0.34548E+01_JPRB, 0.31628E+01_JPRB, 0.29543E+01_JPRB, 0.31387E+01_JPRB, 0.28792E+01_JPRB ,&
&  0.19797E+01_JPRB, 0.11349E+01_JPRB, 0.11598E+01_JPRB, 0.13580E+01_JPRB, 0.11665E+01_JPRB, 0.11124E+01_JPRB, 0.10343E+01_JPRB ,&
&  0.90821E+00_JPRB, 0.84337E+00_JPRB, 0.90912E+00_JPRB, 0.98431E+00_JPRB, 0.10640E+01_JPRB, 0.11300E+01_JPRB, 0.11325E+01_JPRB ,&
&  0.10666E+01_JPRB, 0.10255E+01_JPRB, 0.97691E+00_JPRB, 0.96212E+00_JPRB, 0.97743E+00_JPRB, 0.99494E+00_JPRB, 0.98707E+00_JPRB ,&
&  0.98497E+00_JPRB, 0.97285E+00_JPRB, 0.10131E+01_JPRB, 0.10449E+01_JPRB, 0.10841E+01_JPRB, 0.11252E+01_JPRB, 0.12646E+01_JPRB ,&
&  0.14974E+01_JPRB, 0.17077E+01_JPRB, 0.19592E+01_JPRB, 0.19140E+01_JPRB, 0.17356E+01_JPRB, 0.12502E+01_JPRB, 0.84726E+00_JPRB ,&
&  0.86825E+00_JPRB, 0.89016E+00_JPRB, 0.89449E+00_JPRB, 0.90582E+00_JPRB, 0.89740E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.79876E+00_JPRB, 0.79144E+00_JPRB, 0.78425E+00_JPRB, 0.78878E+00_JPRB, 0.79798E+00_JPRB, 0.84714E+00_JPRB, 0.92646E+00_JPRB ,&
&  0.10035E+01_JPRB, 0.10505E+01_JPRB, 0.10924E+01_JPRB, 0.11133E+01_JPRB, 0.11159E+01_JPRB, 0.10757E+01_JPRB, 0.10717E+01_JPRB ,&
&  0.10657E+01_JPRB, 0.10639E+01_JPRB, 0.10859E+01_JPRB, 0.11210E+01_JPRB, 0.11520E+01_JPRB, 0.11864E+01_JPRB, 0.12248E+01_JPRB ,&
&  0.12732E+01_JPRB, 0.13451E+01_JPRB, 0.14200E+01_JPRB, 0.15338E+01_JPRB, 0.16198E+01_JPRB, 0.16714E+01_JPRB, 0.16289E+01_JPRB ,&
&  0.76010E+00_JPRB, 0.90015E+00_JPRB, 0.13991E+01_JPRB, 0.13994E+01_JPRB, 0.15085E+01_JPRB, 0.16678E+01_JPRB, 0.18891E+01_JPRB ,&
&  0.18688E+01_JPRB, 0.17097E+01_JPRB, 0.17760E+01_JPRB, 0.20879E+01_JPRB, 0.20968E+01_JPRB, 0.21801E+01_JPRB, 0.22489E+01_JPRB ,&
&  0.22655E+01_JPRB, 0.22594E+01_JPRB, 0.22824E+01_JPRB, 0.23756E+01_JPRB, 0.25105E+01_JPRB, 0.26571E+01_JPRB, 0.28334E+01_JPRB ,&
&  0.30127E+01_JPRB, 0.32151E+01_JPRB, 0.34210E+01_JPRB, 0.29067E+01_JPRB, 0.30930E+01_JPRB, 0.33878E+01_JPRB, 0.37459E+01_JPRB ,&
&  0.28838E+01_JPRB, 0.16731E+01_JPRB, 0.13967E+01_JPRB, 0.15472E+01_JPRB, 0.13497E+01_JPRB, 0.13304E+01_JPRB, 0.11291E+01_JPRB ,&
&  0.10905E+01_JPRB, 0.10186E+01_JPRB, 0.10489E+01_JPRB, 0.11165E+01_JPRB, 0.11846E+01_JPRB, 0.12318E+01_JPRB, 0.12240E+01_JPRB ,&
&  0.11538E+01_JPRB, 0.11080E+01_JPRB, 0.10822E+01_JPRB, 0.10777E+01_JPRB, 0.10872E+01_JPRB, 0.10942E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.10700E+01_JPRB, 0.10692E+01_JPRB, 0.11103E+01_JPRB, 0.11372E+01_JPRB, 0.11645E+01_JPRB, 0.11923E+01_JPRB, 0.12171E+01_JPRB ,&
&  0.11719E+01_JPRB, 0.11461E+01_JPRB, 0.12326E+01_JPRB, 0.11374E+01_JPRB, 0.10127E+01_JPRB, 0.86963E+00_JPRB, 0.73806E+00_JPRB ,&
&  0.75848E+00_JPRB, 0.76883E+00_JPRB, 0.76705E+00_JPRB, 0.77592E+00_JPRB, 0.79430E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.71307E+00_JPRB, 0.71900E+00_JPRB, 0.71263E+00_JPRB, 0.73003E+00_JPRB, 0.73215E+00_JPRB, 0.80009E+00_JPRB, 0.87023E+00_JPRB ,&
&  0.97934E+00_JPRB, 0.10599E+01_JPRB, 0.11111E+01_JPRB, 0.11243E+01_JPRB, 0.11057E+01_JPRB, 0.10519E+01_JPRB, 0.10013E+01_JPRB ,&
&  0.10014E+01_JPRB, 0.10052E+01_JPRB, 0.10175E+01_JPRB, 0.10394E+01_JPRB, 0.10619E+01_JPRB, 0.10971E+01_JPRB, 0.11329E+01_JPRB ,&
&  0.11794E+01_JPRB, 0.12492E+01_JPRB, 0.13359E+01_JPRB, 0.14629E+01_JPRB, 0.16259E+01_JPRB, 0.18494E+01_JPRB, 0.20749E+01_JPRB ,&
&  0.16149E+01_JPRB, 0.82509E+00_JPRB, 0.11971E+01_JPRB, 0.12382E+01_JPRB, 0.11733E+01_JPRB, 0.13859E+01_JPRB, 0.16817E+01_JPRB ,&
&  0.16583E+01_JPRB, 0.16282E+01_JPRB, 0.16474E+01_JPRB, 0.19413E+01_JPRB, 0.19712E+01_JPRB, 0.20286E+01_JPRB, 0.20756E+01_JPRB ,&
&  0.21055E+01_JPRB, 0.21027E+01_JPRB, 0.20992E+01_JPRB, 0.21787E+01_JPRB, 0.23088E+01_JPRB, 0.24777E+01_JPRB, 0.26538E+01_JPRB ,&
&  0.28609E+01_JPRB, 0.30864E+01_JPRB, 0.33709E+01_JPRB, 0.29000E+01_JPRB, 0.32260E+01_JPRB, 0.32774E+01_JPRB, 0.29279E+01_JPRB ,&
&  0.27963E+01_JPRB, 0.19034E+01_JPRB, 0.15187E+01_JPRB, 0.16760E+01_JPRB, 0.15188E+01_JPRB, 0.11919E+01_JPRB, 0.11990E+01_JPRB ,&
&  0.11405E+01_JPRB, 0.11484E+01_JPRB, 0.11317E+01_JPRB, 0.12067E+01_JPRB, 0.12482E+01_JPRB, 0.12759E+01_JPRB, 0.12527E+01_JPRB ,&
&  0.12281E+01_JPRB, 0.11821E+01_JPRB, 0.11619E+01_JPRB, 0.11617E+01_JPRB, 0.11631E+01_JPRB, 0.11885E+01_JPRB, 0.11882E+01_JPRB ,&
&  0.11816E+01_JPRB, 0.11973E+01_JPRB, 0.12419E+01_JPRB, 0.12916E+01_JPRB, 0.13124E+01_JPRB, 0.12435E+01_JPRB, 0.11830E+01_JPRB ,&
&  0.11413E+01_JPRB, 0.10987E+01_JPRB, 0.99698E+00_JPRB, 0.88137E+00_JPRB, 0.84753E+00_JPRB, 0.77610E+00_JPRB, 0.71667E+00_JPRB ,&
&  0.72669E+00_JPRB, 0.72463E+00_JPRB, 0.71086E+00_JPRB, 0.71042E+00_JPRB, 0.71500E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.75342E+00_JPRB, 0.75010E+00_JPRB, 0.72117E+00_JPRB, 0.70349E+00_JPRB, 0.69938E+00_JPRB, 0.72694E+00_JPRB, 0.77120E+00_JPRB ,&
&  0.84071E+00_JPRB, 0.93229E+00_JPRB, 0.10061E+01_JPRB, 0.10408E+01_JPRB, 0.10330E+01_JPRB, 0.10387E+01_JPRB, 0.99502E+00_JPRB ,&
&  0.94723E+00_JPRB, 0.93956E+00_JPRB, 0.95167E+00_JPRB, 0.95998E+00_JPRB, 0.97497E+00_JPRB, 0.99628E+00_JPRB, 0.10209E+01_JPRB ,&
&  0.10343E+01_JPRB, 0.10453E+01_JPRB, 0.10586E+01_JPRB, 0.11323E+01_JPRB, 0.12555E+01_JPRB, 0.14470E+01_JPRB, 0.17964E+01_JPRB ,&
&  0.25706E+01_JPRB, 0.25417E+01_JPRB, 0.13384E+01_JPRB, 0.14191E+01_JPRB, 0.12504E+01_JPRB, 0.13297E+01_JPRB, 0.13319E+01_JPRB ,&
&  0.14746E+01_JPRB, 0.14461E+01_JPRB, 0.14967E+01_JPRB, 0.17305E+01_JPRB, 0.17735E+01_JPRB, 0.18324E+01_JPRB, 0.18733E+01_JPRB ,&
&  0.19398E+01_JPRB, 0.19641E+01_JPRB, 0.19377E+01_JPRB, 0.19518E+01_JPRB, 0.20659E+01_JPRB, 0.22270E+01_JPRB, 0.24128E+01_JPRB ,&
&  0.26439E+01_JPRB, 0.28495E+01_JPRB, 0.31398E+01_JPRB, 0.29034E+01_JPRB, 0.31993E+01_JPRB, 0.29708E+01_JPRB, 0.24098E+01_JPRB ,&
&  0.20585E+01_JPRB, 0.18516E+01_JPRB, 0.17899E+01_JPRB, 0.17986E+01_JPRB, 0.16435E+01_JPRB, 0.12828E+01_JPRB, 0.12784E+01_JPRB ,&
&  0.12129E+01_JPRB, 0.11894E+01_JPRB, 0.11805E+01_JPRB, 0.12027E+01_JPRB, 0.12502E+01_JPRB, 0.12646E+01_JPRB, 0.12727E+01_JPRB ,&
&  0.12512E+01_JPRB, 0.12180E+01_JPRB, 0.12012E+01_JPRB, 0.11956E+01_JPRB, 0.11967E+01_JPRB, 0.12230E+01_JPRB, 0.12354E+01_JPRB ,&
&  0.12489E+01_JPRB, 0.12768E+01_JPRB, 0.13201E+01_JPRB, 0.13657E+01_JPRB, 0.13405E+01_JPRB, 0.12583E+01_JPRB, 0.11690E+01_JPRB ,&
&  0.11413E+01_JPRB, 0.11442E+01_JPRB, 0.10641E+01_JPRB, 0.10039E+01_JPRB, 0.94651E+00_JPRB, 0.93250E+00_JPRB, 0.90634E+00_JPRB ,&
&  0.90128E+00_JPRB, 0.89969E+00_JPRB, 0.86125E+00_JPRB, 0.75379E+00_JPRB, 0.75413E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.84873E+00_JPRB, 0.80776E+00_JPRB, 0.80102E+00_JPRB, 0.79524E+00_JPRB, 0.78134E+00_JPRB, 0.77916E+00_JPRB, 0.74482E+00_JPRB ,&
&  0.74709E+00_JPRB, 0.79875E+00_JPRB, 0.86426E+00_JPRB, 0.89926E+00_JPRB, 0.92005E+00_JPRB, 0.92077E+00_JPRB, 0.89734E+00_JPRB ,&
&  0.86959E+00_JPRB, 0.87886E+00_JPRB, 0.89503E+00_JPRB, 0.90465E+00_JPRB, 0.92207E+00_JPRB, 0.93754E+00_JPRB, 0.96170E+00_JPRB ,&
&  0.98057E+00_JPRB, 0.95867E+00_JPRB, 0.93077E+00_JPRB, 0.92253E+00_JPRB, 0.97367E+00_JPRB, 0.10525E+01_JPRB, 0.11607E+01_JPRB ,&
&  0.16891E+01_JPRB, 0.34059E+01_JPRB, 0.17955E+01_JPRB, 0.16189E+01_JPRB, 0.15063E+01_JPRB, 0.13923E+01_JPRB, 0.13108E+01_JPRB ,&
&  0.13648E+01_JPRB, 0.12391E+01_JPRB, 0.12739E+01_JPRB, 0.14683E+01_JPRB, 0.15092E+01_JPRB, 0.15449E+01_JPRB, 0.16055E+01_JPRB ,&
&  0.16568E+01_JPRB, 0.16923E+01_JPRB, 0.17074E+01_JPRB, 0.16762E+01_JPRB, 0.17558E+01_JPRB, 0.18791E+01_JPRB, 0.20521E+01_JPRB ,&
&  0.22488E+01_JPRB, 0.24992E+01_JPRB, 0.28354E+01_JPRB, 0.27944E+01_JPRB, 0.30188E+01_JPRB, 0.28422E+01_JPRB, 0.23407E+01_JPRB ,&
&  0.20522E+01_JPRB, 0.20203E+01_JPRB, 0.19007E+01_JPRB, 0.18366E+01_JPRB, 0.16943E+01_JPRB, 0.14336E+01_JPRB, 0.14379E+01_JPRB ,&
&  0.12948E+01_JPRB, 0.12422E+01_JPRB, 0.12383E+01_JPRB, 0.12346E+01_JPRB, 0.12567E+01_JPRB, 0.12604E+01_JPRB, 0.12652E+01_JPRB ,&
&  0.12493E+01_JPRB, 0.12235E+01_JPRB, 0.12070E+01_JPRB, 0.12065E+01_JPRB, 0.12059E+01_JPRB, 0.12085E+01_JPRB, 0.12277E+01_JPRB ,&
&  0.12454E+01_JPRB, 0.12831E+01_JPRB, 0.13382E+01_JPRB, 0.13424E+01_JPRB, 0.13008E+01_JPRB, 0.12164E+01_JPRB, 0.11299E+01_JPRB ,&
&  0.11012E+01_JPRB, 0.11088E+01_JPRB, 0.10744E+01_JPRB, 0.10288E+01_JPRB, 0.10284E+01_JPRB, 0.10071E+01_JPRB, 0.10404E+01_JPRB ,&
&  0.11216E+01_JPRB, 0.12392E+01_JPRB, 0.13293E+01_JPRB, 0.88296E+00_JPRB, 0.87958E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.89592E+00_JPRB, 0.85201E+00_JPRB, 0.84696E+00_JPRB, 0.86817E+00_JPRB, 0.86515E+00_JPRB, 0.84686E+00_JPRB, 0.80900E+00_JPRB ,&
&  0.76381E+00_JPRB, 0.76918E+00_JPRB, 0.79195E+00_JPRB, 0.81159E+00_JPRB, 0.82422E+00_JPRB, 0.83575E+00_JPRB, 0.82380E+00_JPRB ,&
&  0.80082E+00_JPRB, 0.80668E+00_JPRB, 0.82242E+00_JPRB, 0.85020E+00_JPRB, 0.87080E+00_JPRB, 0.89162E+00_JPRB, 0.92149E+00_JPRB ,&
&  0.92926E+00_JPRB, 0.92262E+00_JPRB, 0.90792E+00_JPRB, 0.89520E+00_JPRB, 0.89283E+00_JPRB, 0.91356E+00_JPRB, 0.95900E+00_JPRB ,&
&  0.12078E+01_JPRB, 0.29878E+01_JPRB, 0.29266E+01_JPRB, 0.20514E+01_JPRB, 0.15871E+01_JPRB, 0.13511E+01_JPRB, 0.12526E+01_JPRB ,&
&  0.13450E+01_JPRB, 0.13699E+01_JPRB, 0.14918E+01_JPRB, 0.13991E+01_JPRB, 0.13453E+01_JPRB, 0.13572E+01_JPRB, 0.14029E+01_JPRB ,&
&  0.14026E+01_JPRB, 0.14119E+01_JPRB, 0.14098E+01_JPRB, 0.13940E+01_JPRB, 0.14077E+01_JPRB, 0.15112E+01_JPRB, 0.16451E+01_JPRB ,&
&  0.17788E+01_JPRB, 0.20139E+01_JPRB, 0.24247E+01_JPRB, 0.28221E+01_JPRB, 0.28332E+01_JPRB, 0.29050E+01_JPRB, 0.27613E+01_JPRB ,&
&  0.28830E+01_JPRB, 0.25903E+01_JPRB, 0.21336E+01_JPRB, 0.18033E+01_JPRB, 0.15783E+01_JPRB, 0.16249E+01_JPRB, 0.15393E+01_JPRB ,&
&  0.13662E+01_JPRB, 0.12709E+01_JPRB, 0.12401E+01_JPRB, 0.12412E+01_JPRB, 0.12496E+01_JPRB, 0.12516E+01_JPRB, 0.12254E+01_JPRB ,&
&  0.12038E+01_JPRB, 0.11949E+01_JPRB, 0.11918E+01_JPRB, 0.11970E+01_JPRB, 0.11946E+01_JPRB, 0.11936E+01_JPRB, 0.12092E+01_JPRB ,&
&  0.12125E+01_JPRB, 0.12344E+01_JPRB, 0.12431E+01_JPRB, 0.12804E+01_JPRB, 0.12424E+01_JPRB, 0.11531E+01_JPRB, 0.10737E+01_JPRB ,&
&  0.10488E+01_JPRB, 0.10703E+01_JPRB, 0.10754E+01_JPRB, 0.10624E+01_JPRB, 0.10445E+01_JPRB, 0.10201E+01_JPRB, 0.99762E+00_JPRB ,&
&  0.10085E+01_JPRB, 0.10230E+01_JPRB, 0.99118E+00_JPRB, 0.93060E+00_JPRB, 0.93936E+00_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.94396E+00_JPRB, 0.91356E+00_JPRB, 0.88980E+00_JPRB, 0.89471E+00_JPRB, 0.90039E+00_JPRB, 0.88874E+00_JPRB, 0.86647E+00_JPRB ,&
&  0.84502E+00_JPRB, 0.82155E+00_JPRB, 0.82792E+00_JPRB, 0.81608E+00_JPRB, 0.80762E+00_JPRB, 0.79944E+00_JPRB, 0.79471E+00_JPRB ,&
&  0.79764E+00_JPRB, 0.80624E+00_JPRB, 0.79490E+00_JPRB, 0.79804E+00_JPRB, 0.82295E+00_JPRB, 0.84828E+00_JPRB, 0.85846E+00_JPRB ,&
&  0.86804E+00_JPRB, 0.87906E+00_JPRB, 0.87877E+00_JPRB, 0.87690E+00_JPRB, 0.86348E+00_JPRB, 0.86640E+00_JPRB, 0.88281E+00_JPRB ,&
&  0.10217E+01_JPRB, 0.23574E+01_JPRB, 0.17334E+01_JPRB, 0.20456E+01_JPRB, 0.15602E+01_JPRB, 0.13382E+01_JPRB, 0.11193E+01_JPRB ,&
&  0.12160E+01_JPRB, 0.15082E+01_JPRB, 0.14129E+01_JPRB, 0.13443E+01_JPRB, 0.12743E+01_JPRB, 0.12513E+01_JPRB, 0.12409E+01_JPRB ,&
&  0.12358E+01_JPRB, 0.12488E+01_JPRB, 0.12359E+01_JPRB, 0.12373E+01_JPRB, 0.12182E+01_JPRB, 0.12660E+01_JPRB, 0.13472E+01_JPRB ,&
&  0.14219E+01_JPRB, 0.15461E+01_JPRB, 0.18254E+01_JPRB, 0.21671E+01_JPRB, 0.28466E+01_JPRB, 0.32178E+01_JPRB, 0.33943E+01_JPRB ,&
&  0.35430E+01_JPRB, 0.31174E+01_JPRB, 0.25223E+01_JPRB, 0.20053E+01_JPRB, 0.17127E+01_JPRB, 0.16481E+01_JPRB, 0.16255E+01_JPRB ,&
&  0.14729E+01_JPRB, 0.13602E+01_JPRB, 0.12915E+01_JPRB, 0.12600E+01_JPRB, 0.12677E+01_JPRB, 0.12670E+01_JPRB, 0.12185E+01_JPRB ,&
&  0.11807E+01_JPRB, 0.11797E+01_JPRB, 0.11886E+01_JPRB, 0.11971E+01_JPRB, 0.11858E+01_JPRB, 0.11624E+01_JPRB, 0.11635E+01_JPRB ,&
&  0.11746E+01_JPRB, 0.11855E+01_JPRB, 0.11880E+01_JPRB, 0.12506E+01_JPRB, 0.12103E+01_JPRB, 0.11395E+01_JPRB, 0.10693E+01_JPRB ,&
&  0.10572E+01_JPRB, 0.10577E+01_JPRB, 0.10525E+01_JPRB, 0.10371E+01_JPRB, 0.10561E+01_JPRB, 0.10684E+01_JPRB, 0.10043E+01_JPRB ,&
&  0.99661E+00_JPRB, 0.99164E+00_JPRB, 0.98317E+00_JPRB, 0.95420E+00_JPRB, 0.96561E+00_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.94497E+00_JPRB, 0.92251E+00_JPRB, 0.92413E+00_JPRB, 0.92499E+00_JPRB, 0.91899E+00_JPRB, 0.89779E+00_JPRB, 0.88444E+00_JPRB ,&
&  0.89506E+00_JPRB, 0.88965E+00_JPRB, 0.87896E+00_JPRB, 0.84985E+00_JPRB, 0.83289E+00_JPRB, 0.81256E+00_JPRB, 0.79989E+00_JPRB ,&
&  0.80259E+00_JPRB, 0.80595E+00_JPRB, 0.79844E+00_JPRB, 0.79795E+00_JPRB, 0.79614E+00_JPRB, 0.81071E+00_JPRB, 0.82428E+00_JPRB ,&
&  0.83677E+00_JPRB, 0.84643E+00_JPRB, 0.86205E+00_JPRB, 0.86888E+00_JPRB, 0.86131E+00_JPRB, 0.84740E+00_JPRB, 0.84920E+00_JPRB ,&
&  0.95790E+00_JPRB, 0.22928E+01_JPRB, 0.14430E+01_JPRB, 0.20243E+01_JPRB, 0.15430E+01_JPRB, 0.13502E+01_JPRB, 0.11324E+01_JPRB ,&
&  0.12215E+01_JPRB, 0.14668E+01_JPRB, 0.13971E+01_JPRB, 0.12723E+01_JPRB, 0.12329E+01_JPRB, 0.12027E+01_JPRB, 0.11626E+01_JPRB ,&
&  0.11483E+01_JPRB, 0.11666E+01_JPRB, 0.11612E+01_JPRB, 0.11674E+01_JPRB, 0.11657E+01_JPRB, 0.11488E+01_JPRB, 0.11713E+01_JPRB ,&
&  0.11987E+01_JPRB, 0.12486E+01_JPRB, 0.14188E+01_JPRB, 0.20088E+01_JPRB, 0.25286E+01_JPRB, 0.33998E+01_JPRB, 0.43393E+01_JPRB ,&
&  0.42616E+01_JPRB, 0.36946E+01_JPRB, 0.27005E+01_JPRB, 0.22304E+01_JPRB, 0.19144E+01_JPRB, 0.17665E+01_JPRB, 0.16828E+01_JPRB ,&
&  0.15923E+01_JPRB, 0.15222E+01_JPRB, 0.14039E+01_JPRB, 0.13440E+01_JPRB, 0.13299E+01_JPRB, 0.13139E+01_JPRB, 0.12775E+01_JPRB ,&
&  0.12362E+01_JPRB, 0.12307E+01_JPRB, 0.12376E+01_JPRB, 0.12318E+01_JPRB, 0.12146E+01_JPRB, 0.11782E+01_JPRB, 0.11617E+01_JPRB ,&
&  0.11738E+01_JPRB, 0.11750E+01_JPRB, 0.11916E+01_JPRB, 0.12826E+01_JPRB, 0.12447E+01_JPRB, 0.11562E+01_JPRB, 0.10932E+01_JPRB ,&
&  0.10909E+01_JPRB, 0.10651E+01_JPRB, 0.10550E+01_JPRB, 0.10607E+01_JPRB, 0.10762E+01_JPRB, 0.11348E+01_JPRB, 0.10575E+01_JPRB ,&
&  0.10138E+01_JPRB, 0.97793E+00_JPRB, 0.95817E+00_JPRB, 0.94820E+00_JPRB, 0.95200E+00_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.97132E+00_JPRB, 0.95762E+00_JPRB, 0.95609E+00_JPRB, 0.96806E+00_JPRB, 0.95289E+00_JPRB, 0.92883E+00_JPRB, 0.90522E+00_JPRB ,&
&  0.92003E+00_JPRB, 0.92034E+00_JPRB, 0.91356E+00_JPRB, 0.87778E+00_JPRB, 0.85696E+00_JPRB, 0.83832E+00_JPRB, 0.83029E+00_JPRB ,&
&  0.83287E+00_JPRB, 0.82473E+00_JPRB, 0.80997E+00_JPRB, 0.80260E+00_JPRB, 0.80375E+00_JPRB, 0.80652E+00_JPRB, 0.82344E+00_JPRB ,&
&  0.81704E+00_JPRB, 0.82356E+00_JPRB, 0.84597E+00_JPRB, 0.85579E+00_JPRB, 0.86280E+00_JPRB, 0.84368E+00_JPRB, 0.83702E+00_JPRB ,&
&  0.88631E+00_JPRB, 0.99327E+00_JPRB, 0.15677E+01_JPRB, 0.18531E+01_JPRB, 0.15535E+01_JPRB, 0.13912E+01_JPRB, 0.12975E+01_JPRB ,&
&  0.13445E+01_JPRB, 0.13848E+01_JPRB, 0.13348E+01_JPRB, 0.12560E+01_JPRB, 0.12122E+01_JPRB, 0.11612E+01_JPRB, 0.11230E+01_JPRB ,&
&  0.10870E+01_JPRB, 0.10696E+01_JPRB, 0.10763E+01_JPRB, 0.11006E+01_JPRB, 0.11134E+01_JPRB, 0.11046E+01_JPRB, 0.10755E+01_JPRB ,&
&  0.10629E+01_JPRB, 0.10642E+01_JPRB, 0.11356E+01_JPRB, 0.15302E+01_JPRB, 0.24303E+01_JPRB, 0.35086E+01_JPRB, 0.43033E+01_JPRB ,&
&  0.44397E+01_JPRB, 0.37038E+01_JPRB, 0.27395E+01_JPRB, 0.23474E+01_JPRB, 0.20725E+01_JPRB, 0.18677E+01_JPRB, 0.16931E+01_JPRB ,&
&  0.15942E+01_JPRB, 0.15728E+01_JPRB, 0.15240E+01_JPRB, 0.14454E+01_JPRB, 0.13697E+01_JPRB, 0.13271E+01_JPRB, 0.12936E+01_JPRB ,&
&  0.12719E+01_JPRB, 0.12809E+01_JPRB, 0.12911E+01_JPRB, 0.12616E+01_JPRB, 0.12276E+01_JPRB, 0.11915E+01_JPRB, 0.11749E+01_JPRB ,&
&  0.11851E+01_JPRB, 0.12002E+01_JPRB, 0.12292E+01_JPRB, 0.12876E+01_JPRB, 0.12625E+01_JPRB, 0.12132E+01_JPRB, 0.11549E+01_JPRB ,&
&  0.11036E+01_JPRB, 0.10633E+01_JPRB, 0.10450E+01_JPRB, 0.10830E+01_JPRB, 0.10845E+01_JPRB, 0.11640E+01_JPRB, 0.11246E+01_JPRB ,&
&  0.10578E+01_JPRB, 0.10118E+01_JPRB, 0.98763E+00_JPRB, 0.97830E+00_JPRB, 0.98195E+00_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.10079E+01_JPRB, 0.10031E+01_JPRB, 0.10127E+01_JPRB, 0.10217E+01_JPRB, 0.10100E+01_JPRB, 0.98068E+00_JPRB, 0.95209E+00_JPRB ,&
&  0.93617E+00_JPRB, 0.92213E+00_JPRB, 0.90471E+00_JPRB, 0.87529E+00_JPRB, 0.85873E+00_JPRB, 0.85661E+00_JPRB, 0.85922E+00_JPRB ,&
&  0.86362E+00_JPRB, 0.85339E+00_JPRB, 0.83705E+00_JPRB, 0.81983E+00_JPRB, 0.81886E+00_JPRB, 0.82395E+00_JPRB, 0.83738E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83285E+00_JPRB, 0.83862E+00_JPRB, 0.84556E+00_JPRB, 0.85347E+00_JPRB, 0.84343E+00_JPRB, 0.83084E+00_JPRB ,&
&  0.84311E+00_JPRB, 0.11207E+01_JPRB, 0.14074E+01_JPRB, 0.15366E+01_JPRB, 0.14988E+01_JPRB, 0.13908E+01_JPRB, 0.13435E+01_JPRB ,&
&  0.12806E+01_JPRB, 0.12881E+01_JPRB, 0.12918E+01_JPRB, 0.12507E+01_JPRB, 0.11976E+01_JPRB, 0.11538E+01_JPRB, 0.11219E+01_JPRB ,&
&  0.10865E+01_JPRB, 0.10558E+01_JPRB, 0.10329E+01_JPRB, 0.10365E+01_JPRB, 0.10499E+01_JPRB, 0.10665E+01_JPRB, 0.10490E+01_JPRB ,&
&  0.10182E+01_JPRB, 0.99118E+00_JPRB, 0.99499E+00_JPRB, 0.11932E+01_JPRB, 0.19243E+01_JPRB, 0.23787E+01_JPRB, 0.34696E+01_JPRB ,&
&  0.38705E+01_JPRB, 0.29287E+01_JPRB, 0.23651E+01_JPRB, 0.21047E+01_JPRB, 0.19206E+01_JPRB, 0.17349E+01_JPRB, 0.15953E+01_JPRB ,&
&  0.15309E+01_JPRB, 0.14996E+01_JPRB, 0.15034E+01_JPRB, 0.14674E+01_JPRB, 0.13769E+01_JPRB, 0.12849E+01_JPRB, 0.12394E+01_JPRB ,&
&  0.12166E+01_JPRB, 0.11993E+01_JPRB, 0.11966E+01_JPRB, 0.11976E+01_JPRB, 0.11856E+01_JPRB, 0.11486E+01_JPRB, 0.11313E+01_JPRB ,&
&  0.11345E+01_JPRB, 0.11627E+01_JPRB, 0.12150E+01_JPRB, 0.12368E+01_JPRB, 0.12534E+01_JPRB, 0.12611E+01_JPRB, 0.11982E+01_JPRB ,&
&  0.11109E+01_JPRB, 0.10592E+01_JPRB, 0.10490E+01_JPRB, 0.10651E+01_JPRB, 0.11049E+01_JPRB, 0.11996E+01_JPRB, 0.11591E+01_JPRB ,&
&  0.10986E+01_JPRB, 0.10461E+01_JPRB, 0.10198E+01_JPRB, 0.10033E+01_JPRB, 0.10201E+01_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.10389E+01_JPRB, 0.10290E+01_JPRB, 0.10377E+01_JPRB, 0.10425E+01_JPRB, 0.10315E+01_JPRB, 0.10006E+01_JPRB, 0.97525E+00_JPRB ,&
&  0.96272E+00_JPRB, 0.92809E+00_JPRB, 0.89361E+00_JPRB, 0.86885E+00_JPRB, 0.86900E+00_JPRB, 0.87422E+00_JPRB, 0.87600E+00_JPRB ,&
&  0.86767E+00_JPRB, 0.86633E+00_JPRB, 0.85330E+00_JPRB, 0.84278E+00_JPRB, 0.83865E+00_JPRB, 0.83621E+00_JPRB, 0.82667E+00_JPRB ,&
&  0.83013E+00_JPRB, 0.83077E+00_JPRB, 0.83545E+00_JPRB, 0.84347E+00_JPRB, 0.84100E+00_JPRB, 0.84415E+00_JPRB, 0.83688E+00_JPRB ,&
&  0.82004E+00_JPRB, 0.90455E+00_JPRB, 0.10493E+01_JPRB, 0.12664E+01_JPRB, 0.13403E+01_JPRB, 0.13237E+01_JPRB, 0.12798E+01_JPRB ,&
&  0.12292E+01_JPRB, 0.12079E+01_JPRB, 0.12113E+01_JPRB, 0.11873E+01_JPRB, 0.11495E+01_JPRB, 0.11208E+01_JPRB, 0.10945E+01_JPRB ,&
&  0.10727E+01_JPRB, 0.10525E+01_JPRB, 0.10468E+01_JPRB, 0.10353E+01_JPRB, 0.10233E+01_JPRB, 0.10288E+01_JPRB, 0.10397E+01_JPRB ,&
&  0.10011E+01_JPRB, 0.96861E+00_JPRB, 0.94365E+00_JPRB, 0.97731E+00_JPRB, 0.13203E+01_JPRB, 0.17424E+01_JPRB, 0.23219E+01_JPRB ,&
&  0.22034E+01_JPRB, 0.19584E+01_JPRB, 0.18006E+01_JPRB, 0.16826E+01_JPRB, 0.16055E+01_JPRB, 0.14918E+01_JPRB, 0.13960E+01_JPRB ,&
&  0.13068E+01_JPRB, 0.12710E+01_JPRB, 0.12430E+01_JPRB, 0.12309E+01_JPRB, 0.12080E+01_JPRB, 0.11905E+01_JPRB, 0.11416E+01_JPRB ,&
&  0.11119E+01_JPRB, 0.10917E+01_JPRB, 0.10870E+01_JPRB, 0.10963E+01_JPRB, 0.11147E+01_JPRB, 0.11135E+01_JPRB, 0.11103E+01_JPRB ,&
&  0.11224E+01_JPRB, 0.11353E+01_JPRB, 0.11776E+01_JPRB, 0.12308E+01_JPRB, 0.12191E+01_JPRB, 0.11847E+01_JPRB, 0.11583E+01_JPRB ,&
&  0.11363E+01_JPRB, 0.10971E+01_JPRB, 0.10610E+01_JPRB, 0.10803E+01_JPRB, 0.10965E+01_JPRB, 0.11812E+01_JPRB, 0.11651E+01_JPRB ,&
&  0.11211E+01_JPRB, 0.10636E+01_JPRB, 0.10365E+01_JPRB, 0.10196E+01_JPRB, 0.10389E+01_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.10749E+01_JPRB, 0.10783E+01_JPRB, 0.10722E+01_JPRB, 0.10667E+01_JPRB, 0.10524E+01_JPRB, 0.10225E+01_JPRB, 0.99107E+00_JPRB ,&
&  0.96894E+00_JPRB, 0.94301E+00_JPRB, 0.90501E+00_JPRB, 0.88411E+00_JPRB, 0.87806E+00_JPRB, 0.87812E+00_JPRB, 0.87454E+00_JPRB ,&
&  0.87067E+00_JPRB, 0.87545E+00_JPRB, 0.87004E+00_JPRB, 0.87107E+00_JPRB, 0.86457E+00_JPRB, 0.86127E+00_JPRB, 0.84263E+00_JPRB ,&
&  0.83057E+00_JPRB, 0.82231E+00_JPRB, 0.82059E+00_JPRB, 0.82796E+00_JPRB, 0.83465E+00_JPRB, 0.83942E+00_JPRB, 0.84493E+00_JPRB ,&
&  0.83836E+00_JPRB, 0.76884E+00_JPRB, 0.85599E+00_JPRB, 0.99617E+00_JPRB, 0.10935E+01_JPRB, 0.11718E+01_JPRB, 0.11590E+01_JPRB ,&
&  0.11427E+01_JPRB, 0.11247E+01_JPRB, 0.11288E+01_JPRB, 0.11101E+01_JPRB, 0.10745E+01_JPRB, 0.10427E+01_JPRB, 0.10230E+01_JPRB ,&
&  0.10214E+01_JPRB, 0.10160E+01_JPRB, 0.10130E+01_JPRB, 0.10080E+01_JPRB, 0.10030E+01_JPRB, 0.10013E+01_JPRB, 0.10109E+01_JPRB ,&
&  0.10039E+01_JPRB, 0.97141E+00_JPRB, 0.93586E+00_JPRB, 0.91276E+00_JPRB, 0.96215E+00_JPRB, 0.11120E+01_JPRB, 0.13769E+01_JPRB ,&
&  0.15212E+01_JPRB, 0.14492E+01_JPRB, 0.13881E+01_JPRB, 0.13347E+01_JPRB, 0.12922E+01_JPRB, 0.12679E+01_JPRB, 0.12396E+01_JPRB ,&
&  0.12190E+01_JPRB, 0.12010E+01_JPRB, 0.11901E+01_JPRB, 0.11760E+01_JPRB, 0.11605E+01_JPRB, 0.11457E+01_JPRB, 0.11247E+01_JPRB ,&
&  0.11059E+01_JPRB, 0.10973E+01_JPRB, 0.10855E+01_JPRB, 0.10909E+01_JPRB, 0.10943E+01_JPRB, 0.10946E+01_JPRB, 0.10958E+01_JPRB ,&
&  0.11092E+01_JPRB, 0.11410E+01_JPRB, 0.11634E+01_JPRB, 0.11866E+01_JPRB, 0.11798E+01_JPRB, 0.11558E+01_JPRB, 0.11430E+01_JPRB ,&
&  0.11231E+01_JPRB, 0.11050E+01_JPRB, 0.10492E+01_JPRB, 0.10548E+01_JPRB, 0.11528E+01_JPRB, 0.11430E+01_JPRB, 0.11298E+01_JPRB ,&
&  0.11210E+01_JPRB, 0.10945E+01_JPRB, 0.10502E+01_JPRB, 0.10048E+01_JPRB, 0.10408E+01_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.97459E+00_JPRB, 0.10101E+01_JPRB, 0.10123E+01_JPRB, 0.10125E+01_JPRB, 0.10101E+01_JPRB, 0.99494E+00_JPRB, 0.97449E+00_JPRB ,&
&  0.95321E+00_JPRB, 0.94122E+00_JPRB, 0.92420E+00_JPRB, 0.90141E+00_JPRB, 0.88678E+00_JPRB, 0.87821E+00_JPRB, 0.87027E+00_JPRB ,&
&  0.86719E+00_JPRB, 0.86394E+00_JPRB, 0.86143E+00_JPRB, 0.85811E+00_JPRB, 0.84944E+00_JPRB, 0.84215E+00_JPRB, 0.82937E+00_JPRB ,&
&  0.82091E+00_JPRB, 0.82041E+00_JPRB, 0.82140E+00_JPRB, 0.82613E+00_JPRB, 0.83441E+00_JPRB, 0.84374E+00_JPRB, 0.85018E+00_JPRB ,&
&  0.85040E+00_JPRB, 0.74389E+00_JPRB, 0.77349E+00_JPRB, 0.85341E+00_JPRB, 0.94669E+00_JPRB, 0.10327E+01_JPRB, 0.10636E+01_JPRB ,&
&  0.10721E+01_JPRB, 0.10714E+01_JPRB, 0.10598E+01_JPRB, 0.10425E+01_JPRB, 0.10226E+01_JPRB, 0.10095E+01_JPRB, 0.98106E+00_JPRB ,&
&  0.96393E+00_JPRB, 0.95897E+00_JPRB, 0.96422E+00_JPRB, 0.96568E+00_JPRB, 0.96192E+00_JPRB, 0.95681E+00_JPRB, 0.95893E+00_JPRB ,&
&  0.95724E+00_JPRB, 0.93851E+00_JPRB, 0.91460E+00_JPRB, 0.89713E+00_JPRB, 0.90258E+00_JPRB, 0.96759E+00_JPRB, 0.10796E+01_JPRB ,&
&  0.11852E+01_JPRB, 0.11809E+01_JPRB, 0.11501E+01_JPRB, 0.11176E+01_JPRB, 0.11039E+01_JPRB, 0.11034E+01_JPRB, 0.11101E+01_JPRB ,&
&  0.11062E+01_JPRB, 0.11013E+01_JPRB, 0.11129E+01_JPRB, 0.11162E+01_JPRB, 0.11197E+01_JPRB, 0.11179E+01_JPRB, 0.11146E+01_JPRB ,&
&  0.11051E+01_JPRB, 0.10925E+01_JPRB, 0.10862E+01_JPRB, 0.10840E+01_JPRB, 0.10893E+01_JPRB, 0.10889E+01_JPRB, 0.10810E+01_JPRB ,&
&  0.10704E+01_JPRB, 0.10816E+01_JPRB, 0.11185E+01_JPRB, 0.11500E+01_JPRB, 0.11429E+01_JPRB, 0.11258E+01_JPRB, 0.11111E+01_JPRB ,&
&  0.11076E+01_JPRB, 0.10939E+01_JPRB, 0.10832E+01_JPRB, 0.11150E+01_JPRB, 0.11110E+01_JPRB, 0.10975E+01_JPRB, 0.10851E+01_JPRB ,&
&  0.10766E+01_JPRB, 0.10562E+01_JPRB, 0.10353E+01_JPRB, 0.10013E+01_JPRB, 0.91669E+00_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.89613E+00_JPRB, 0.93246E+00_JPRB, 0.95317E+00_JPRB, 0.95918E+00_JPRB, 0.96706E+00_JPRB, 0.97052E+00_JPRB, 0.96764E+00_JPRB ,&
&  0.96547E+00_JPRB, 0.96028E+00_JPRB, 0.94751E+00_JPRB, 0.92765E+00_JPRB, 0.90746E+00_JPRB, 0.88529E+00_JPRB, 0.86816E+00_JPRB ,&
&  0.86340E+00_JPRB, 0.86079E+00_JPRB, 0.84852E+00_JPRB, 0.83700E+00_JPRB, 0.82589E+00_JPRB, 0.81701E+00_JPRB, 0.81351E+00_JPRB ,&
&  0.80895E+00_JPRB, 0.80724E+00_JPRB, 0.81302E+00_JPRB, 0.82075E+00_JPRB, 0.83093E+00_JPRB, 0.84222E+00_JPRB, 0.85708E+00_JPRB ,&
&  0.86034E+00_JPRB, 0.67557E+00_JPRB, 0.70228E+00_JPRB, 0.78242E+00_JPRB, 0.83878E+00_JPRB, 0.90988E+00_JPRB, 0.97643E+00_JPRB ,&
&  0.10023E+01_JPRB, 0.10277E+01_JPRB, 0.10221E+01_JPRB, 0.10084E+01_JPRB, 0.10061E+01_JPRB, 0.98873E+00_JPRB, 0.95084E+00_JPRB ,&
&  0.92214E+00_JPRB, 0.91987E+00_JPRB, 0.93183E+00_JPRB, 0.93973E+00_JPRB, 0.93817E+00_JPRB, 0.93153E+00_JPRB, 0.92866E+00_JPRB ,&
&  0.92626E+00_JPRB, 0.92390E+00_JPRB, 0.92007E+00_JPRB, 0.91301E+00_JPRB, 0.92342E+00_JPRB, 0.95576E+00_JPRB, 0.10023E+01_JPRB ,&
&  0.10268E+01_JPRB, 0.10316E+01_JPRB, 0.10254E+01_JPRB, 0.10111E+01_JPRB, 0.10060E+01_JPRB, 0.10184E+01_JPRB, 0.10292E+01_JPRB ,&
&  0.10318E+01_JPRB, 0.10326E+01_JPRB, 0.10409E+01_JPRB, 0.10459E+01_JPRB, 0.10564E+01_JPRB, 0.10670E+01_JPRB, 0.10850E+01_JPRB ,&
&  0.10975E+01_JPRB, 0.10958E+01_JPRB, 0.10915E+01_JPRB, 0.10795E+01_JPRB, 0.10595E+01_JPRB, 0.10460E+01_JPRB, 0.10410E+01_JPRB ,&
&  0.10372E+01_JPRB, 0.10443E+01_JPRB, 0.10639E+01_JPRB, 0.10861E+01_JPRB, 0.10907E+01_JPRB, 0.10896E+01_JPRB, 0.10894E+01_JPRB ,&
&  0.10934E+01_JPRB, 0.10938E+01_JPRB, 0.10981E+01_JPRB, 0.10384E+01_JPRB, 0.10761E+01_JPRB, 0.10706E+01_JPRB, 0.10466E+01_JPRB ,&
&  0.10185E+01_JPRB, 0.98501E+00_JPRB, 0.96486E+00_JPRB, 0.80497E+00_JPRB, 0.86810E+00_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.85189E+00_JPRB, 0.87318E+00_JPRB, 0.87934E+00_JPRB, 0.89373E+00_JPRB, 0.90373E+00_JPRB, 0.91079E+00_JPRB, 0.92080E+00_JPRB ,&
&  0.92749E+00_JPRB, 0.92419E+00_JPRB, 0.91789E+00_JPRB, 0.90455E+00_JPRB, 0.88217E+00_JPRB, 0.85819E+00_JPRB, 0.83872E+00_JPRB ,&
&  0.82499E+00_JPRB, 0.82338E+00_JPRB, 0.81611E+00_JPRB, 0.80637E+00_JPRB, 0.80044E+00_JPRB, 0.79729E+00_JPRB, 0.79952E+00_JPRB ,&
&  0.80743E+00_JPRB, 0.81166E+00_JPRB, 0.80692E+00_JPRB, 0.80603E+00_JPRB, 0.81525E+00_JPRB, 0.83169E+00_JPRB, 0.83960E+00_JPRB ,&
&  0.84180E+00_JPRB, 0.64650E+00_JPRB, 0.68817E+00_JPRB, 0.73712E+00_JPRB, 0.77798E+00_JPRB, 0.83768E+00_JPRB, 0.90346E+00_JPRB ,&
&  0.94617E+00_JPRB, 0.97257E+00_JPRB, 0.97462E+00_JPRB, 0.96565E+00_JPRB, 0.94747E+00_JPRB, 0.93060E+00_JPRB, 0.91108E+00_JPRB ,&
&  0.89457E+00_JPRB, 0.89009E+00_JPRB, 0.89507E+00_JPRB, 0.90033E+00_JPRB, 0.90280E+00_JPRB, 0.90722E+00_JPRB, 0.90213E+00_JPRB ,&
&  0.90085E+00_JPRB, 0.89861E+00_JPRB, 0.89800E+00_JPRB, 0.89949E+00_JPRB, 0.90111E+00_JPRB, 0.90676E+00_JPRB, 0.91647E+00_JPRB ,&
&  0.91676E+00_JPRB, 0.92005E+00_JPRB, 0.92073E+00_JPRB, 0.91714E+00_JPRB, 0.92116E+00_JPRB, 0.93224E+00_JPRB, 0.94887E+00_JPRB ,&
&  0.96901E+00_JPRB, 0.97328E+00_JPRB, 0.98022E+00_JPRB, 0.98913E+00_JPRB, 0.10027E+01_JPRB, 0.10178E+01_JPRB, 0.10362E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10444E+01_JPRB, 0.10425E+01_JPRB, 0.10330E+01_JPRB, 0.10309E+01_JPRB, 0.10132E+01_JPRB, 0.99275E+00_JPRB ,&
&  0.98933E+00_JPRB, 0.99019E+00_JPRB, 0.99332E+00_JPRB, 0.10052E+01_JPRB, 0.10115E+01_JPRB, 0.10092E+01_JPRB, 0.10142E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.10276E+01_JPRB, 0.10212E+01_JPRB, 0.10171E+01_JPRB, 0.10019E+01_JPRB, 0.98845E+00_JPRB, 0.96403E+00_JPRB ,&
&  0.93447E+00_JPRB, 0.91184E+00_JPRB, 0.71733E+00_JPRB, 0.80200E+00_JPRB, 0.82416E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.84744E+00_JPRB, 0.85326E+00_JPRB, 0.85667E+00_JPRB, 0.86149E+00_JPRB, 0.86318E+00_JPRB, 0.86286E+00_JPRB, 0.86832E+00_JPRB ,&
&  0.87430E+00_JPRB, 0.87063E+00_JPRB, 0.86327E+00_JPRB, 0.86093E+00_JPRB, 0.85212E+00_JPRB, 0.83904E+00_JPRB, 0.82861E+00_JPRB ,&
&  0.81536E+00_JPRB, 0.81051E+00_JPRB, 0.80763E+00_JPRB, 0.80443E+00_JPRB, 0.79807E+00_JPRB, 0.79005E+00_JPRB, 0.79387E+00_JPRB ,&
&  0.80085E+00_JPRB, 0.80826E+00_JPRB, 0.81050E+00_JPRB, 0.81487E+00_JPRB, 0.82127E+00_JPRB, 0.82842E+00_JPRB, 0.84107E+00_JPRB ,&
&  0.73031E+00_JPRB, 0.60494E+00_JPRB, 0.65696E+00_JPRB, 0.70988E+00_JPRB, 0.75536E+00_JPRB, 0.82413E+00_JPRB, 0.87413E+00_JPRB ,&
&  0.91242E+00_JPRB, 0.94447E+00_JPRB, 0.95163E+00_JPRB, 0.92990E+00_JPRB, 0.90741E+00_JPRB, 0.89985E+00_JPRB, 0.89892E+00_JPRB ,&
&  0.89571E+00_JPRB, 0.89517E+00_JPRB, 0.89996E+00_JPRB, 0.90500E+00_JPRB, 0.91006E+00_JPRB, 0.91658E+00_JPRB, 0.91894E+00_JPRB ,&
&  0.91699E+00_JPRB, 0.90476E+00_JPRB, 0.89233E+00_JPRB, 0.87321E+00_JPRB, 0.85717E+00_JPRB, 0.85422E+00_JPRB, 0.85141E+00_JPRB ,&
&  0.84272E+00_JPRB, 0.84267E+00_JPRB, 0.84910E+00_JPRB, 0.85683E+00_JPRB, 0.86912E+00_JPRB, 0.88311E+00_JPRB, 0.90302E+00_JPRB ,&
&  0.92595E+00_JPRB, 0.94437E+00_JPRB, 0.95818E+00_JPRB, 0.97325E+00_JPRB, 0.98041E+00_JPRB, 0.98916E+00_JPRB, 0.10054E+01_JPRB ,&
&  0.10183E+01_JPRB, 0.10072E+01_JPRB, 0.99647E+00_JPRB, 0.98599E+00_JPRB, 0.98283E+00_JPRB, 0.98105E+00_JPRB, 0.97185E+00_JPRB ,&
&  0.96276E+00_JPRB, 0.95690E+00_JPRB, 0.95447E+00_JPRB, 0.95908E+00_JPRB, 0.95679E+00_JPRB, 0.94972E+00_JPRB, 0.94527E+00_JPRB ,&
&  0.94131E+00_JPRB, 0.93026E+00_JPRB, 0.91787E+00_JPRB, 0.91063E+00_JPRB, 0.89898E+00_JPRB, 0.89022E+00_JPRB, 0.88091E+00_JPRB ,&
&  0.87281E+00_JPRB, 0.85548E+00_JPRB, 0.85727E+00_JPRB, 0.84885E+00_JPRB, 0.84414E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.82562E+00_JPRB, 0.82862E+00_JPRB, 0.83264E+00_JPRB, 0.83327E+00_JPRB, 0.82997E+00_JPRB, 0.82845E+00_JPRB, 0.83117E+00_JPRB ,&
&  0.83363E+00_JPRB, 0.82687E+00_JPRB, 0.81985E+00_JPRB, 0.81463E+00_JPRB, 0.81286E+00_JPRB, 0.80484E+00_JPRB, 0.79228E+00_JPRB ,&
&  0.78346E+00_JPRB, 0.78288E+00_JPRB, 0.78855E+00_JPRB, 0.79183E+00_JPRB, 0.78359E+00_JPRB, 0.77482E+00_JPRB, 0.77211E+00_JPRB ,&
&  0.77353E+00_JPRB, 0.77993E+00_JPRB, 0.79048E+00_JPRB, 0.79684E+00_JPRB, 0.80592E+00_JPRB, 0.82040E+00_JPRB, 0.82194E+00_JPRB ,&
&  0.82022E+00_JPRB, 0.68314E+00_JPRB, 0.72131E+00_JPRB, 0.73670E+00_JPRB, 0.76689E+00_JPRB, 0.81034E+00_JPRB, 0.84600E+00_JPRB ,&
&  0.87996E+00_JPRB, 0.89926E+00_JPRB, 0.90779E+00_JPRB, 0.90137E+00_JPRB, 0.88631E+00_JPRB, 0.87252E+00_JPRB, 0.86886E+00_JPRB ,&
&  0.86789E+00_JPRB, 0.87507E+00_JPRB, 0.88921E+00_JPRB, 0.89915E+00_JPRB, 0.90285E+00_JPRB, 0.90408E+00_JPRB, 0.90671E+00_JPRB ,&
&  0.90371E+00_JPRB, 0.88779E+00_JPRB, 0.86510E+00_JPRB, 0.83740E+00_JPRB, 0.81562E+00_JPRB, 0.80584E+00_JPRB, 0.79553E+00_JPRB ,&
&  0.78094E+00_JPRB, 0.77481E+00_JPRB, 0.77805E+00_JPRB, 0.78510E+00_JPRB, 0.79758E+00_JPRB, 0.81146E+00_JPRB, 0.82311E+00_JPRB ,&
&  0.83554E+00_JPRB, 0.85399E+00_JPRB, 0.87376E+00_JPRB, 0.89231E+00_JPRB, 0.90936E+00_JPRB, 0.92363E+00_JPRB, 0.93326E+00_JPRB ,&
&  0.93837E+00_JPRB, 0.93407E+00_JPRB, 0.92088E+00_JPRB, 0.91104E+00_JPRB, 0.90737E+00_JPRB, 0.91208E+00_JPRB, 0.91328E+00_JPRB ,&
&  0.90525E+00_JPRB, 0.89840E+00_JPRB, 0.89474E+00_JPRB, 0.88347E+00_JPRB, 0.87074E+00_JPRB, 0.85868E+00_JPRB, 0.84627E+00_JPRB ,&
&  0.83676E+00_JPRB, 0.83279E+00_JPRB, 0.82917E+00_JPRB, 0.82266E+00_JPRB, 0.81214E+00_JPRB, 0.80649E+00_JPRB, 0.80191E+00_JPRB ,&
&  0.79744E+00_JPRB, 0.79216E+00_JPRB, 0.79278E+00_JPRB, 0.80216E+00_JPRB, 0.81609E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.78913E+00_JPRB, 0.79436E+00_JPRB, 0.80303E+00_JPRB, 0.81324E+00_JPRB, 0.81674E+00_JPRB, 0.81662E+00_JPRB, 0.81492E+00_JPRB ,&
&  0.81634E+00_JPRB, 0.80885E+00_JPRB, 0.80348E+00_JPRB, 0.80035E+00_JPRB, 0.79994E+00_JPRB, 0.79686E+00_JPRB, 0.78786E+00_JPRB ,&
&  0.78032E+00_JPRB, 0.77747E+00_JPRB, 0.78206E+00_JPRB, 0.78362E+00_JPRB, 0.76851E+00_JPRB, 0.76372E+00_JPRB, 0.75607E+00_JPRB ,&
&  0.75504E+00_JPRB, 0.75532E+00_JPRB, 0.75840E+00_JPRB, 0.76470E+00_JPRB, 0.78130E+00_JPRB, 0.79313E+00_JPRB, 0.79958E+00_JPRB ,&
&  0.79800E+00_JPRB, 0.71880E+00_JPRB, 0.74654E+00_JPRB, 0.75907E+00_JPRB, 0.78184E+00_JPRB, 0.80457E+00_JPRB, 0.82439E+00_JPRB ,&
&  0.85017E+00_JPRB, 0.86422E+00_JPRB, 0.86985E+00_JPRB, 0.87078E+00_JPRB, 0.87234E+00_JPRB, 0.87239E+00_JPRB, 0.86575E+00_JPRB ,&
&  0.86149E+00_JPRB, 0.86885E+00_JPRB, 0.88557E+00_JPRB, 0.89427E+00_JPRB, 0.89859E+00_JPRB, 0.90090E+00_JPRB, 0.89764E+00_JPRB ,&
&  0.89058E+00_JPRB, 0.87611E+00_JPRB, 0.85186E+00_JPRB, 0.82269E+00_JPRB, 0.78974E+00_JPRB, 0.76684E+00_JPRB, 0.75437E+00_JPRB ,&
&  0.74729E+00_JPRB, 0.74902E+00_JPRB, 0.75252E+00_JPRB, 0.75659E+00_JPRB, 0.76229E+00_JPRB, 0.77326E+00_JPRB, 0.77948E+00_JPRB ,&
&  0.78892E+00_JPRB, 0.79765E+00_JPRB, 0.81022E+00_JPRB, 0.82990E+00_JPRB, 0.84926E+00_JPRB, 0.85970E+00_JPRB, 0.85966E+00_JPRB ,&
&  0.86474E+00_JPRB, 0.86720E+00_JPRB, 0.86307E+00_JPRB, 0.86226E+00_JPRB, 0.85780E+00_JPRB, 0.85871E+00_JPRB, 0.85595E+00_JPRB ,&
&  0.84706E+00_JPRB, 0.84032E+00_JPRB, 0.83213E+00_JPRB, 0.81650E+00_JPRB, 0.80267E+00_JPRB, 0.79251E+00_JPRB, 0.78405E+00_JPRB ,&
&  0.78272E+00_JPRB, 0.78632E+00_JPRB, 0.79151E+00_JPRB, 0.78631E+00_JPRB, 0.77173E+00_JPRB, 0.76232E+00_JPRB, 0.75148E+00_JPRB ,&
&  0.74409E+00_JPRB, 0.74605E+00_JPRB, 0.74606E+00_JPRB, 0.75346E+00_JPRB, 0.77239E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.74644E+00_JPRB, 0.75870E+00_JPRB, 0.77407E+00_JPRB, 0.78995E+00_JPRB, 0.80003E+00_JPRB, 0.80637E+00_JPRB, 0.80456E+00_JPRB ,&
&  0.79425E+00_JPRB, 0.78399E+00_JPRB, 0.78607E+00_JPRB, 0.78632E+00_JPRB, 0.78555E+00_JPRB, 0.78391E+00_JPRB, 0.77736E+00_JPRB ,&
&  0.76785E+00_JPRB, 0.75970E+00_JPRB, 0.75629E+00_JPRB, 0.75336E+00_JPRB, 0.74356E+00_JPRB, 0.73602E+00_JPRB, 0.72791E+00_JPRB ,&
&  0.72783E+00_JPRB, 0.73054E+00_JPRB, 0.73352E+00_JPRB, 0.73387E+00_JPRB, 0.74200E+00_JPRB, 0.75242E+00_JPRB, 0.75684E+00_JPRB ,&
&  0.75132E+00_JPRB, 0.75773E+00_JPRB, 0.67544E+00_JPRB, 0.74532E+00_JPRB, 0.77060E+00_JPRB, 0.78580E+00_JPRB, 0.80119E+00_JPRB ,&
&  0.81138E+00_JPRB, 0.82273E+00_JPRB, 0.83081E+00_JPRB, 0.83528E+00_JPRB, 0.85231E+00_JPRB, 0.86652E+00_JPRB, 0.87030E+00_JPRB ,&
&  0.87183E+00_JPRB, 0.87100E+00_JPRB, 0.87240E+00_JPRB, 0.87267E+00_JPRB, 0.87730E+00_JPRB, 0.87736E+00_JPRB, 0.87146E+00_JPRB ,&
&  0.86338E+00_JPRB, 0.85158E+00_JPRB, 0.83405E+00_JPRB, 0.81365E+00_JPRB, 0.78156E+00_JPRB, 0.75351E+00_JPRB, 0.73561E+00_JPRB ,&
&  0.72122E+00_JPRB, 0.71749E+00_JPRB, 0.71303E+00_JPRB, 0.70868E+00_JPRB, 0.70798E+00_JPRB, 0.71164E+00_JPRB, 0.71219E+00_JPRB ,&
&  0.71984E+00_JPRB, 0.72904E+00_JPRB, 0.73338E+00_JPRB, 0.74087E+00_JPRB, 0.75072E+00_JPRB, 0.76495E+00_JPRB, 0.77841E+00_JPRB ,&
&  0.79091E+00_JPRB, 0.79951E+00_JPRB, 0.80079E+00_JPRB, 0.80064E+00_JPRB, 0.79791E+00_JPRB, 0.79636E+00_JPRB, 0.78901E+00_JPRB ,&
&  0.78314E+00_JPRB, 0.77993E+00_JPRB, 0.77332E+00_JPRB, 0.76112E+00_JPRB, 0.75348E+00_JPRB, 0.74703E+00_JPRB, 0.74530E+00_JPRB ,&
&  0.74605E+00_JPRB, 0.74732E+00_JPRB, 0.74488E+00_JPRB, 0.74565E+00_JPRB, 0.73931E+00_JPRB, 0.71877E+00_JPRB, 0.70255E+00_JPRB ,&
&  0.70138E+00_JPRB, 0.70645E+00_JPRB, 0.71114E+00_JPRB, 0.71997E+00_JPRB, 0.73453E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.76517E+00_JPRB, 0.77115E+00_JPRB, 0.78008E+00_JPRB, 0.79323E+00_JPRB, 0.80708E+00_JPRB, 0.81612E+00_JPRB, 0.81582E+00_JPRB ,&
&  0.80507E+00_JPRB, 0.79211E+00_JPRB, 0.78808E+00_JPRB, 0.78780E+00_JPRB, 0.78511E+00_JPRB, 0.78389E+00_JPRB, 0.77948E+00_JPRB ,&
&  0.76846E+00_JPRB, 0.75158E+00_JPRB, 0.74362E+00_JPRB, 0.72982E+00_JPRB, 0.71681E+00_JPRB, 0.70619E+00_JPRB, 0.70632E+00_JPRB ,&
&  0.71509E+00_JPRB, 0.72520E+00_JPRB, 0.73766E+00_JPRB, 0.75185E+00_JPRB, 0.75952E+00_JPRB, 0.75819E+00_JPRB, 0.75503E+00_JPRB ,&
&  0.75248E+00_JPRB, 0.74971E+00_JPRB, 0.76060E+00_JPRB, 0.76965E+00_JPRB, 0.77578E+00_JPRB, 0.78714E+00_JPRB, 0.79644E+00_JPRB ,&
&  0.80424E+00_JPRB, 0.81551E+00_JPRB, 0.83408E+00_JPRB, 0.84775E+00_JPRB, 0.85764E+00_JPRB, 0.86823E+00_JPRB, 0.87197E+00_JPRB ,&
&  0.87828E+00_JPRB, 0.88954E+00_JPRB, 0.89181E+00_JPRB, 0.88282E+00_JPRB, 0.86847E+00_JPRB, 0.85926E+00_JPRB, 0.85406E+00_JPRB ,&
&  0.84474E+00_JPRB, 0.82533E+00_JPRB, 0.81062E+00_JPRB, 0.79883E+00_JPRB, 0.77976E+00_JPRB, 0.75937E+00_JPRB, 0.73550E+00_JPRB ,&
&  0.70844E+00_JPRB, 0.68833E+00_JPRB, 0.67280E+00_JPRB, 0.65883E+00_JPRB, 0.65569E+00_JPRB, 0.64752E+00_JPRB, 0.64161E+00_JPRB ,&
&  0.64684E+00_JPRB, 0.66574E+00_JPRB, 0.67995E+00_JPRB, 0.68513E+00_JPRB, 0.68792E+00_JPRB, 0.70396E+00_JPRB, 0.72336E+00_JPRB ,&
&  0.73850E+00_JPRB, 0.74486E+00_JPRB, 0.74091E+00_JPRB, 0.74316E+00_JPRB, 0.74811E+00_JPRB, 0.75218E+00_JPRB, 0.74587E+00_JPRB ,&
&  0.73693E+00_JPRB, 0.73105E+00_JPRB, 0.72956E+00_JPRB, 0.72569E+00_JPRB, 0.72702E+00_JPRB, 0.72680E+00_JPRB, 0.72401E+00_JPRB ,&
&  0.71835E+00_JPRB, 0.70918E+00_JPRB, 0.69732E+00_JPRB, 0.68993E+00_JPRB, 0.67911E+00_JPRB, 0.66303E+00_JPRB, 0.66365E+00_JPRB ,&
&  0.67484E+00_JPRB, 0.69300E+00_JPRB, 0.71032E+00_JPRB, 0.73068E+00_JPRB, 0.75432E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.79160E+00_JPRB, 0.79943E+00_JPRB, 0.80486E+00_JPRB, 0.80878E+00_JPRB, 0.81730E+00_JPRB, 0.82811E+00_JPRB, 0.82795E+00_JPRB ,&
&  0.81336E+00_JPRB, 0.79979E+00_JPRB, 0.79044E+00_JPRB, 0.78799E+00_JPRB, 0.78981E+00_JPRB, 0.79217E+00_JPRB, 0.78642E+00_JPRB ,&
&  0.76535E+00_JPRB, 0.74283E+00_JPRB, 0.72902E+00_JPRB, 0.71533E+00_JPRB, 0.70684E+00_JPRB, 0.70147E+00_JPRB, 0.70839E+00_JPRB ,&
&  0.71862E+00_JPRB, 0.73468E+00_JPRB, 0.74775E+00_JPRB, 0.75722E+00_JPRB, 0.76107E+00_JPRB, 0.74974E+00_JPRB, 0.75111E+00_JPRB ,&
&  0.75829E+00_JPRB, 0.76058E+00_JPRB, 0.77569E+00_JPRB, 0.77523E+00_JPRB, 0.76211E+00_JPRB, 0.75154E+00_JPRB, 0.74111E+00_JPRB ,&
&  0.73846E+00_JPRB, 0.74978E+00_JPRB, 0.77507E+00_JPRB, 0.81164E+00_JPRB, 0.83275E+00_JPRB, 0.84075E+00_JPRB, 0.83777E+00_JPRB ,&
&  0.84057E+00_JPRB, 0.85252E+00_JPRB, 0.85924E+00_JPRB, 0.85097E+00_JPRB, 0.83263E+00_JPRB, 0.81763E+00_JPRB, 0.80439E+00_JPRB ,&
&  0.79453E+00_JPRB, 0.78268E+00_JPRB, 0.77463E+00_JPRB, 0.76458E+00_JPRB, 0.74269E+00_JPRB, 0.71677E+00_JPRB, 0.69414E+00_JPRB ,&
&  0.67733E+00_JPRB, 0.66076E+00_JPRB, 0.64062E+00_JPRB, 0.63432E+00_JPRB, 0.63843E+00_JPRB, 0.61565E+00_JPRB, 0.59121E+00_JPRB ,&
&  0.59527E+00_JPRB, 0.62706E+00_JPRB, 0.65200E+00_JPRB, 0.66231E+00_JPRB, 0.66346E+00_JPRB, 0.66793E+00_JPRB, 0.67144E+00_JPRB ,&
&  0.67799E+00_JPRB, 0.68665E+00_JPRB, 0.69322E+00_JPRB, 0.70434E+00_JPRB, 0.71581E+00_JPRB, 0.71407E+00_JPRB, 0.69956E+00_JPRB ,&
&  0.68364E+00_JPRB, 0.67440E+00_JPRB, 0.67039E+00_JPRB, 0.67238E+00_JPRB, 0.67563E+00_JPRB, 0.67361E+00_JPRB, 0.66778E+00_JPRB ,&
&  0.65600E+00_JPRB, 0.63756E+00_JPRB, 0.62191E+00_JPRB, 0.61302E+00_JPRB, 0.61241E+00_JPRB, 0.61891E+00_JPRB, 0.63511E+00_JPRB ,&
&  0.66639E+00_JPRB, 0.71402E+00_JPRB, 0.74937E+00_JPRB, 0.76927E+00_JPRB, 0.77451E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.82017E+00_JPRB, 0.82821E+00_JPRB, 0.83782E+00_JPRB, 0.85083E+00_JPRB, 0.86273E+00_JPRB, 0.85946E+00_JPRB, 0.84688E+00_JPRB ,&
&  0.82318E+00_JPRB, 0.81427E+00_JPRB, 0.81901E+00_JPRB, 0.82147E+00_JPRB, 0.82323E+00_JPRB, 0.82105E+00_JPRB, 0.80493E+00_JPRB ,&
&  0.76742E+00_JPRB, 0.73745E+00_JPRB, 0.72312E+00_JPRB, 0.71569E+00_JPRB, 0.71336E+00_JPRB, 0.72038E+00_JPRB, 0.72879E+00_JPRB ,&
&  0.74562E+00_JPRB, 0.75564E+00_JPRB, 0.76772E+00_JPRB, 0.77722E+00_JPRB, 0.78002E+00_JPRB, 0.78400E+00_JPRB, 0.78513E+00_JPRB ,&
&  0.77106E+00_JPRB, 0.77131E+00_JPRB, 0.77283E+00_JPRB, 0.76322E+00_JPRB, 0.74186E+00_JPRB, 0.72041E+00_JPRB, 0.69403E+00_JPRB ,&
&  0.67655E+00_JPRB, 0.66776E+00_JPRB, 0.69329E+00_JPRB, 0.75507E+00_JPRB, 0.79855E+00_JPRB, 0.80665E+00_JPRB, 0.79874E+00_JPRB ,&
&  0.80069E+00_JPRB, 0.80813E+00_JPRB, 0.81164E+00_JPRB, 0.80014E+00_JPRB, 0.78042E+00_JPRB, 0.76307E+00_JPRB, 0.74646E+00_JPRB ,&
&  0.74079E+00_JPRB, 0.73990E+00_JPRB, 0.72243E+00_JPRB, 0.69017E+00_JPRB, 0.66253E+00_JPRB, 0.64300E+00_JPRB, 0.63274E+00_JPRB ,&
&  0.62651E+00_JPRB, 0.61687E+00_JPRB, 0.61089E+00_JPRB, 0.61571E+00_JPRB, 0.59587E+00_JPRB, 0.57816E+00_JPRB, 0.58230E+00_JPRB ,&
&  0.59912E+00_JPRB, 0.60498E+00_JPRB, 0.61448E+00_JPRB, 0.61783E+00_JPRB, 0.61503E+00_JPRB, 0.61149E+00_JPRB, 0.61276E+00_JPRB ,&
&  0.62149E+00_JPRB, 0.63148E+00_JPRB, 0.63880E+00_JPRB, 0.64365E+00_JPRB, 0.64852E+00_JPRB, 0.63643E+00_JPRB, 0.61496E+00_JPRB ,&
&  0.59709E+00_JPRB, 0.59348E+00_JPRB, 0.59766E+00_JPRB, 0.59674E+00_JPRB, 0.59504E+00_JPRB, 0.59518E+00_JPRB, 0.59389E+00_JPRB ,&
&  0.59112E+00_JPRB, 0.58379E+00_JPRB, 0.57506E+00_JPRB, 0.57516E+00_JPRB, 0.58633E+00_JPRB, 0.60889E+00_JPRB, 0.64405E+00_JPRB ,&
&  0.69612E+00_JPRB, 0.73542E+00_JPRB, 0.77349E+00_JPRB, 0.78912E+00_JPRB, 0.79274E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.85596E+00_JPRB, 0.87762E+00_JPRB, 0.89625E+00_JPRB, 0.90886E+00_JPRB, 0.91056E+00_JPRB, 0.90360E+00_JPRB, 0.88757E+00_JPRB ,&
&  0.85740E+00_JPRB, 0.84023E+00_JPRB, 0.84916E+00_JPRB, 0.86389E+00_JPRB, 0.87284E+00_JPRB, 0.87282E+00_JPRB, 0.84119E+00_JPRB ,&
&  0.78663E+00_JPRB, 0.76603E+00_JPRB, 0.76309E+00_JPRB, 0.77272E+00_JPRB, 0.79093E+00_JPRB, 0.79782E+00_JPRB, 0.78558E+00_JPRB ,&
&  0.77315E+00_JPRB, 0.77469E+00_JPRB, 0.77750E+00_JPRB, 0.78448E+00_JPRB, 0.77391E+00_JPRB, 0.75295E+00_JPRB, 0.74390E+00_JPRB ,&
&  0.74117E+00_JPRB, 0.73351E+00_JPRB, 0.72557E+00_JPRB, 0.67674E+00_JPRB, 0.70052E+00_JPRB, 0.68897E+00_JPRB, 0.67067E+00_JPRB ,&
&  0.65376E+00_JPRB, 0.65629E+00_JPRB, 0.69812E+00_JPRB, 0.74472E+00_JPRB, 0.75624E+00_JPRB, 0.75892E+00_JPRB, 0.75394E+00_JPRB ,&
&  0.75048E+00_JPRB, 0.75088E+00_JPRB, 0.75225E+00_JPRB, 0.74407E+00_JPRB, 0.72366E+00_JPRB, 0.69732E+00_JPRB, 0.66847E+00_JPRB ,&
&  0.65101E+00_JPRB, 0.62605E+00_JPRB, 0.58256E+00_JPRB, 0.56227E+00_JPRB, 0.56038E+00_JPRB, 0.56835E+00_JPRB, 0.57218E+00_JPRB ,&
&  0.56917E+00_JPRB, 0.56182E+00_JPRB, 0.56573E+00_JPRB, 0.56281E+00_JPRB, 0.54115E+00_JPRB, 0.52803E+00_JPRB, 0.52217E+00_JPRB ,&
&  0.51101E+00_JPRB, 0.51025E+00_JPRB, 0.51526E+00_JPRB, 0.52713E+00_JPRB, 0.53762E+00_JPRB, 0.55187E+00_JPRB, 0.55783E+00_JPRB ,&
&  0.56011E+00_JPRB, 0.55651E+00_JPRB, 0.55044E+00_JPRB, 0.53822E+00_JPRB, 0.51954E+00_JPRB, 0.51235E+00_JPRB, 0.51372E+00_JPRB ,&
&  0.52250E+00_JPRB, 0.52829E+00_JPRB, 0.52761E+00_JPRB, 0.52638E+00_JPRB, 0.51474E+00_JPRB, 0.50204E+00_JPRB, 0.49240E+00_JPRB ,&
&  0.48791E+00_JPRB, 0.49087E+00_JPRB, 0.50634E+00_JPRB, 0.53127E+00_JPRB, 0.56328E+00_JPRB, 0.60212E+00_JPRB, 0.64693E+00_JPRB ,&
&  0.67851E+00_JPRB, 0.69700E+00_JPRB, 0.72133E+00_JPRB, 0.76901E+00_JPRB, 0.81514E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.85897E+00_JPRB, 0.88564E+00_JPRB, 0.91488E+00_JPRB, 0.93167E+00_JPRB, 0.93240E+00_JPRB, 0.92907E+00_JPRB, 0.91385E+00_JPRB ,&
&  0.88281E+00_JPRB, 0.85739E+00_JPRB, 0.85313E+00_JPRB, 0.85700E+00_JPRB, 0.86879E+00_JPRB, 0.88750E+00_JPRB, 0.88900E+00_JPRB ,&
&  0.86230E+00_JPRB, 0.84293E+00_JPRB, 0.83330E+00_JPRB, 0.82625E+00_JPRB, 0.80924E+00_JPRB, 0.78457E+00_JPRB, 0.76849E+00_JPRB ,&
&  0.76395E+00_JPRB, 0.76391E+00_JPRB, 0.75506E+00_JPRB, 0.74117E+00_JPRB, 0.70965E+00_JPRB, 0.67075E+00_JPRB, 0.64021E+00_JPRB ,&
&  0.62958E+00_JPRB, 0.62037E+00_JPRB, 0.57985E+00_JPRB, 0.59607E+00_JPRB, 0.59999E+00_JPRB, 0.60038E+00_JPRB, 0.59790E+00_JPRB ,&
&  0.61011E+00_JPRB, 0.64352E+00_JPRB, 0.67666E+00_JPRB, 0.68646E+00_JPRB, 0.69592E+00_JPRB, 0.69767E+00_JPRB, 0.69920E+00_JPRB ,&
&  0.70377E+00_JPRB, 0.69577E+00_JPRB, 0.66613E+00_JPRB, 0.62217E+00_JPRB, 0.57881E+00_JPRB, 0.54506E+00_JPRB, 0.51631E+00_JPRB ,&
&  0.49655E+00_JPRB, 0.48414E+00_JPRB, 0.47443E+00_JPRB, 0.47110E+00_JPRB, 0.47884E+00_JPRB, 0.49397E+00_JPRB, 0.48897E+00_JPRB ,&
&  0.47084E+00_JPRB, 0.45411E+00_JPRB, 0.43200E+00_JPRB, 0.40196E+00_JPRB, 0.37122E+00_JPRB, 0.33283E+00_JPRB, 0.31303E+00_JPRB ,&
&  0.31769E+00_JPRB, 0.35717E+00_JPRB, 0.40911E+00_JPRB, 0.42886E+00_JPRB, 0.45620E+00_JPRB, 0.48299E+00_JPRB, 0.47256E+00_JPRB ,&
&  0.42488E+00_JPRB, 0.35655E+00_JPRB, 0.33353E+00_JPRB, 0.32073E+00_JPRB, 0.31376E+00_JPRB, 0.30883E+00_JPRB, 0.30771E+00_JPRB ,&
&  0.31188E+00_JPRB, 0.32304E+00_JPRB, 0.33254E+00_JPRB, 0.32944E+00_JPRB, 0.31911E+00_JPRB, 0.31297E+00_JPRB, 0.31377E+00_JPRB ,&
&  0.32348E+00_JPRB, 0.35191E+00_JPRB, 0.41522E+00_JPRB, 0.53039E+00_JPRB, 0.56700E+00_JPRB, 0.60038E+00_JPRB, 0.63885E+00_JPRB ,&
&  0.67569E+00_JPRB, 0.70908E+00_JPRB, 0.73750E+00_JPRB, 0.77251E+00_JPRB, 0.81954E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.79780E+00_JPRB, 0.81615E+00_JPRB, 0.83749E+00_JPRB, 0.84877E+00_JPRB, 0.84988E+00_JPRB, 0.84693E+00_JPRB, 0.83823E+00_JPRB ,&
&  0.82330E+00_JPRB, 0.80536E+00_JPRB, 0.79213E+00_JPRB, 0.78311E+00_JPRB, 0.77906E+00_JPRB, 0.78189E+00_JPRB, 0.78714E+00_JPRB ,&
&  0.78622E+00_JPRB, 0.78208E+00_JPRB, 0.76985E+00_JPRB, 0.75394E+00_JPRB, 0.73712E+00_JPRB, 0.72413E+00_JPRB, 0.71424E+00_JPRB ,&
&  0.70457E+00_JPRB, 0.68993E+00_JPRB, 0.66706E+00_JPRB, 0.63173E+00_JPRB, 0.58991E+00_JPRB, 0.55169E+00_JPRB, 0.52204E+00_JPRB ,&
&  0.49632E+00_JPRB, 0.42959E+00_JPRB, 0.39271E+00_JPRB, 0.40420E+00_JPRB, 0.49567E+00_JPRB, 0.52722E+00_JPRB, 0.55168E+00_JPRB ,&
&  0.58855E+00_JPRB, 0.62775E+00_JPRB, 0.64659E+00_JPRB, 0.64710E+00_JPRB, 0.64158E+00_JPRB, 0.62896E+00_JPRB, 0.60929E+00_JPRB ,&
&  0.58429E+00_JPRB, 0.55372E+00_JPRB, 0.52767E+00_JPRB, 0.49852E+00_JPRB, 0.46862E+00_JPRB, 0.44660E+00_JPRB, 0.43128E+00_JPRB ,&
&  0.42067E+00_JPRB, 0.41098E+00_JPRB, 0.40326E+00_JPRB, 0.39352E+00_JPRB, 0.38442E+00_JPRB, 0.36342E+00_JPRB, 0.34137E+00_JPRB ,&
&  0.31843E+00_JPRB, 0.30079E+00_JPRB, 0.28221E+00_JPRB, 0.26397E+00_JPRB, 0.24902E+00_JPRB, 0.23927E+00_JPRB, 0.23206E+00_JPRB ,&
&  0.22992E+00_JPRB, 0.23615E+00_JPRB, 0.25521E+00_JPRB, 0.28822E+00_JPRB, 0.32576E+00_JPRB, 0.29775E+00_JPRB, 0.27742E+00_JPRB ,&
&  0.25632E+00_JPRB, 0.24413E+00_JPRB, 0.23942E+00_JPRB, 0.23874E+00_JPRB, 0.23990E+00_JPRB, 0.24210E+00_JPRB, 0.24440E+00_JPRB ,&
&  0.24766E+00_JPRB, 0.25071E+00_JPRB, 0.25388E+00_JPRB, 0.25617E+00_JPRB, 0.25758E+00_JPRB, 0.26022E+00_JPRB, 0.26486E+00_JPRB ,&
&  0.27225E+00_JPRB, 0.28445E+00_JPRB, 0.30436E+00_JPRB, 0.33029E+00_JPRB, 0.35744E+00_JPRB, 0.38202E+00_JPRB, 0.43512E+00_JPRB ,&
&  0.52258E+00_JPRB, 0.63442E+00_JPRB, 0.69180E+00_JPRB, 0.74802E+00_JPRB, 0.77606E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.73491E+00_JPRB, 0.74706E+00_JPRB, 0.75723E+00_JPRB, 0.75932E+00_JPRB, 0.75863E+00_JPRB, 0.75359E+00_JPRB, 0.74150E+00_JPRB ,&
&  0.72176E+00_JPRB, 0.69543E+00_JPRB, 0.66982E+00_JPRB, 0.64943E+00_JPRB, 0.63666E+00_JPRB, 0.63970E+00_JPRB, 0.64826E+00_JPRB ,&
&  0.66564E+00_JPRB, 0.69110E+00_JPRB, 0.69625E+00_JPRB, 0.68952E+00_JPRB, 0.65376E+00_JPRB, 0.61654E+00_JPRB, 0.58218E+00_JPRB ,&
&  0.55159E+00_JPRB, 0.52261E+00_JPRB, 0.51172E+00_JPRB, 0.49281E+00_JPRB, 0.47947E+00_JPRB, 0.46266E+00_JPRB, 0.43887E+00_JPRB ,&
&  0.40012E+00_JPRB, 0.36971E+00_JPRB, 0.35294E+00_JPRB, 0.36536E+00_JPRB, 0.45335E+00_JPRB, 0.51430E+00_JPRB, 0.52450E+00_JPRB ,&
&  0.55891E+00_JPRB, 0.58350E+00_JPRB, 0.57864E+00_JPRB, 0.55877E+00_JPRB, 0.53723E+00_JPRB, 0.51383E+00_JPRB, 0.49443E+00_JPRB ,&
&  0.47513E+00_JPRB, 0.44874E+00_JPRB, 0.39913E+00_JPRB, 0.32887E+00_JPRB, 0.29347E+00_JPRB, 0.28009E+00_JPRB, 0.26570E+00_JPRB ,&
&  0.25486E+00_JPRB, 0.24621E+00_JPRB, 0.24277E+00_JPRB, 0.23929E+00_JPRB, 0.23863E+00_JPRB, 0.23475E+00_JPRB, 0.23133E+00_JPRB ,&
&  0.22681E+00_JPRB, 0.22350E+00_JPRB, 0.21873E+00_JPRB, 0.21405E+00_JPRB, 0.21054E+00_JPRB, 0.20998E+00_JPRB, 0.20979E+00_JPRB ,&
&  0.21087E+00_JPRB, 0.21501E+00_JPRB, 0.22677E+00_JPRB, 0.23324E+00_JPRB, 0.23373E+00_JPRB, 0.22925E+00_JPRB, 0.22202E+00_JPRB ,&
&  0.21529E+00_JPRB, 0.21352E+00_JPRB, 0.21205E+00_JPRB, 0.21257E+00_JPRB, 0.21534E+00_JPRB, 0.21900E+00_JPRB, 0.22194E+00_JPRB ,&
&  0.22425E+00_JPRB, 0.22662E+00_JPRB, 0.22880E+00_JPRB, 0.23120E+00_JPRB, 0.23415E+00_JPRB, 0.23766E+00_JPRB, 0.24249E+00_JPRB ,&
&  0.24764E+00_JPRB, 0.25491E+00_JPRB, 0.26522E+00_JPRB, 0.28016E+00_JPRB, 0.29360E+00_JPRB, 0.30228E+00_JPRB, 0.31404E+00_JPRB ,&
&  0.33015E+00_JPRB, 0.37752E+00_JPRB, 0.51205E+00_JPRB, 0.73719E+00_JPRB, 0.76952E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.65792E+00_JPRB, 0.66952E+00_JPRB, 0.68378E+00_JPRB, 0.68694E+00_JPRB, 0.69708E+00_JPRB, 0.69590E+00_JPRB, 0.67861E+00_JPRB ,&
&  0.65038E+00_JPRB, 0.62377E+00_JPRB, 0.60921E+00_JPRB, 0.59127E+00_JPRB, 0.56347E+00_JPRB, 0.50332E+00_JPRB, 0.48030E+00_JPRB ,&
&  0.46176E+00_JPRB, 0.46134E+00_JPRB, 0.45205E+00_JPRB, 0.45212E+00_JPRB, 0.44105E+00_JPRB, 0.42969E+00_JPRB, 0.41113E+00_JPRB ,&
&  0.39397E+00_JPRB, 0.37697E+00_JPRB, 0.36516E+00_JPRB, 0.35949E+00_JPRB, 0.35722E+00_JPRB, 0.35250E+00_JPRB, 0.34014E+00_JPRB ,&
&  0.33144E+00_JPRB, 0.32970E+00_JPRB, 0.33229E+00_JPRB, 0.34146E+00_JPRB, 0.38464E+00_JPRB, 0.44101E+00_JPRB, 0.44168E+00_JPRB ,&
&  0.46422E+00_JPRB, 0.47767E+00_JPRB, 0.49077E+00_JPRB, 0.47985E+00_JPRB, 0.44720E+00_JPRB, 0.40826E+00_JPRB, 0.35455E+00_JPRB ,&
&  0.30339E+00_JPRB, 0.27449E+00_JPRB, 0.25105E+00_JPRB, 0.23461E+00_JPRB, 0.22337E+00_JPRB, 0.21880E+00_JPRB, 0.21293E+00_JPRB ,&
&  0.20910E+00_JPRB, 0.20617E+00_JPRB, 0.20497E+00_JPRB, 0.20437E+00_JPRB, 0.20364E+00_JPRB, 0.20160E+00_JPRB, 0.20105E+00_JPRB ,&
&  0.20162E+00_JPRB, 0.20182E+00_JPRB, 0.20163E+00_JPRB, 0.20250E+00_JPRB, 0.20351E+00_JPRB, 0.20498E+00_JPRB, 0.20633E+00_JPRB ,&
&  0.20836E+00_JPRB, 0.20948E+00_JPRB, 0.21122E+00_JPRB, 0.21310E+00_JPRB, 0.21211E+00_JPRB, 0.21096E+00_JPRB, 0.20817E+00_JPRB ,&
&  0.20504E+00_JPRB, 0.20476E+00_JPRB, 0.20421E+00_JPRB, 0.20478E+00_JPRB, 0.20640E+00_JPRB, 0.20857E+00_JPRB, 0.21142E+00_JPRB ,&
&  0.21414E+00_JPRB, 0.21697E+00_JPRB, 0.21912E+00_JPRB, 0.22130E+00_JPRB, 0.22496E+00_JPRB, 0.22885E+00_JPRB, 0.23380E+00_JPRB ,&
&  0.23893E+00_JPRB, 0.24478E+00_JPRB, 0.25286E+00_JPRB, 0.26332E+00_JPRB, 0.27841E+00_JPRB, 0.29538E+00_JPRB, 0.32164E+00_JPRB ,&
&  0.34575E+00_JPRB, 0.44540E+00_JPRB, 0.68124E+00_JPRB, 0.61022E+00_JPRB, 0.62985E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.62131E+00_JPRB, 0.63795E+00_JPRB, 0.66290E+00_JPRB, 0.65567E+00_JPRB, 0.67576E+00_JPRB, 0.66231E+00_JPRB, 0.63943E+00_JPRB ,&
&  0.59210E+00_JPRB, 0.53539E+00_JPRB, 0.48875E+00_JPRB, 0.43663E+00_JPRB, 0.39598E+00_JPRB, 0.36643E+00_JPRB, 0.35193E+00_JPRB ,&
&  0.34473E+00_JPRB, 0.34247E+00_JPRB, 0.34101E+00_JPRB, 0.34008E+00_JPRB, 0.33524E+00_JPRB, 0.33183E+00_JPRB, 0.32605E+00_JPRB ,&
&  0.32017E+00_JPRB, 0.31597E+00_JPRB, 0.31275E+00_JPRB, 0.31772E+00_JPRB, 0.32383E+00_JPRB, 0.33021E+00_JPRB, 0.33356E+00_JPRB ,&
&  0.33652E+00_JPRB, 0.33793E+00_JPRB, 0.34071E+00_JPRB, 0.33873E+00_JPRB, 0.34988E+00_JPRB, 0.37106E+00_JPRB, 0.38910E+00_JPRB ,&
&  0.40659E+00_JPRB, 0.42116E+00_JPRB, 0.41710E+00_JPRB, 0.39335E+00_JPRB, 0.32761E+00_JPRB, 0.29345E+00_JPRB, 0.26343E+00_JPRB ,&
&  0.24451E+00_JPRB, 0.23047E+00_JPRB, 0.22104E+00_JPRB, 0.21565E+00_JPRB, 0.21133E+00_JPRB, 0.20894E+00_JPRB, 0.20640E+00_JPRB ,&
&  0.20396E+00_JPRB, 0.20249E+00_JPRB, 0.20081E+00_JPRB, 0.20000E+00_JPRB, 0.19906E+00_JPRB, 0.19781E+00_JPRB, 0.19691E+00_JPRB ,&
&  0.19711E+00_JPRB, 0.19695E+00_JPRB, 0.19812E+00_JPRB, 0.19913E+00_JPRB, 0.20109E+00_JPRB, 0.20271E+00_JPRB, 0.20496E+00_JPRB ,&
&  0.20686E+00_JPRB, 0.20843E+00_JPRB, 0.20828E+00_JPRB, 0.20781E+00_JPRB, 0.20691E+00_JPRB, 0.20640E+00_JPRB, 0.20553E+00_JPRB ,&
&  0.20440E+00_JPRB, 0.20459E+00_JPRB, 0.20487E+00_JPRB, 0.20523E+00_JPRB, 0.20635E+00_JPRB, 0.20794E+00_JPRB, 0.21035E+00_JPRB ,&
&  0.21354E+00_JPRB, 0.21687E+00_JPRB, 0.21967E+00_JPRB, 0.22266E+00_JPRB, 0.22652E+00_JPRB, 0.23092E+00_JPRB, 0.23640E+00_JPRB ,&
&  0.24123E+00_JPRB, 0.24955E+00_JPRB, 0.26111E+00_JPRB, 0.27317E+00_JPRB, 0.29159E+00_JPRB, 0.30924E+00_JPRB, 0.33870E+00_JPRB ,&
&  0.38726E+00_JPRB, 0.55855E+00_JPRB, 0.84193E+00_JPRB, 0.61749E+00_JPRB, 0.60674E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.54224E+00_JPRB, 0.55137E+00_JPRB, 0.56556E+00_JPRB, 0.57709E+00_JPRB, 0.60117E+00_JPRB, 0.61472E+00_JPRB, 0.66046E+00_JPRB ,&
&  0.61474E+00_JPRB, 0.53595E+00_JPRB, 0.50392E+00_JPRB, 0.46909E+00_JPRB, 0.42940E+00_JPRB, 0.40083E+00_JPRB, 0.38071E+00_JPRB ,&
&  0.36860E+00_JPRB, 0.35826E+00_JPRB, 0.34826E+00_JPRB, 0.33582E+00_JPRB, 0.32308E+00_JPRB, 0.30895E+00_JPRB, 0.29840E+00_JPRB ,&
&  0.29213E+00_JPRB, 0.29050E+00_JPRB, 0.29008E+00_JPRB, 0.29423E+00_JPRB, 0.29993E+00_JPRB, 0.30753E+00_JPRB, 0.31633E+00_JPRB ,&
&  0.33658E+00_JPRB, 0.34313E+00_JPRB, 0.34381E+00_JPRB, 0.34368E+00_JPRB, 0.34477E+00_JPRB, 0.34715E+00_JPRB, 0.33759E+00_JPRB ,&
&  0.34597E+00_JPRB, 0.36708E+00_JPRB, 0.37294E+00_JPRB, 0.35388E+00_JPRB, 0.29939E+00_JPRB, 0.26679E+00_JPRB, 0.24420E+00_JPRB ,&
&  0.23576E+00_JPRB, 0.22595E+00_JPRB, 0.21984E+00_JPRB, 0.21401E+00_JPRB, 0.21019E+00_JPRB, 0.20832E+00_JPRB, 0.20645E+00_JPRB ,&
&  0.20512E+00_JPRB, 0.20418E+00_JPRB, 0.20222E+00_JPRB, 0.20135E+00_JPRB, 0.20026E+00_JPRB, 0.19968E+00_JPRB, 0.19937E+00_JPRB ,&
&  0.19921E+00_JPRB, 0.19891E+00_JPRB, 0.19912E+00_JPRB, 0.19966E+00_JPRB, 0.20058E+00_JPRB, 0.20131E+00_JPRB, 0.20198E+00_JPRB ,&
&  0.20286E+00_JPRB, 0.20367E+00_JPRB, 0.20388E+00_JPRB, 0.20437E+00_JPRB, 0.20480E+00_JPRB, 0.20596E+00_JPRB, 0.20649E+00_JPRB ,&
&  0.20705E+00_JPRB, 0.20752E+00_JPRB, 0.20752E+00_JPRB, 0.20813E+00_JPRB, 0.20906E+00_JPRB, 0.21091E+00_JPRB, 0.21300E+00_JPRB ,&
&  0.21568E+00_JPRB, 0.21883E+00_JPRB, 0.22231E+00_JPRB, 0.22581E+00_JPRB, 0.23040E+00_JPRB, 0.23545E+00_JPRB, 0.24200E+00_JPRB ,&
&  0.24913E+00_JPRB, 0.25821E+00_JPRB, 0.27051E+00_JPRB, 0.28547E+00_JPRB, 0.30373E+00_JPRB, 0.32197E+00_JPRB, 0.34256E+00_JPRB ,&
&  0.37736E+00_JPRB, 0.47664E+00_JPRB, 0.53687E+00_JPRB, 0.52531E+00_JPRB, 0.53608E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.52595E+00_JPRB, 0.52836E+00_JPRB, 0.53097E+00_JPRB, 0.53508E+00_JPRB, 0.53948E+00_JPRB, 0.54379E+00_JPRB, 0.54752E+00_JPRB ,&
&  0.52921E+00_JPRB, 0.50611E+00_JPRB, 0.48711E+00_JPRB, 0.46450E+00_JPRB, 0.44387E+00_JPRB, 0.42678E+00_JPRB, 0.40662E+00_JPRB ,&
&  0.38650E+00_JPRB, 0.36640E+00_JPRB, 0.34375E+00_JPRB, 0.32516E+00_JPRB, 0.30686E+00_JPRB, 0.29206E+00_JPRB, 0.28311E+00_JPRB ,&
&  0.27822E+00_JPRB, 0.27690E+00_JPRB, 0.27836E+00_JPRB, 0.28132E+00_JPRB, 0.28413E+00_JPRB, 0.28880E+00_JPRB, 0.29528E+00_JPRB ,&
&  0.30426E+00_JPRB, 0.31887E+00_JPRB, 0.33202E+00_JPRB, 0.33675E+00_JPRB, 0.33320E+00_JPRB, 0.32474E+00_JPRB, 0.31746E+00_JPRB ,&
&  0.30866E+00_JPRB, 0.29649E+00_JPRB, 0.28167E+00_JPRB, 0.26701E+00_JPRB, 0.25336E+00_JPRB, 0.24342E+00_JPRB, 0.23516E+00_JPRB ,&
&  0.22816E+00_JPRB, 0.22199E+00_JPRB, 0.21723E+00_JPRB, 0.21404E+00_JPRB, 0.21131E+00_JPRB, 0.20911E+00_JPRB, 0.20702E+00_JPRB ,&
&  0.20599E+00_JPRB, 0.20492E+00_JPRB, 0.20428E+00_JPRB, 0.20392E+00_JPRB, 0.20344E+00_JPRB, 0.20331E+00_JPRB, 0.20321E+00_JPRB ,&
&  0.20288E+00_JPRB, 0.20227E+00_JPRB, 0.20193E+00_JPRB, 0.20170E+00_JPRB, 0.20212E+00_JPRB, 0.20246E+00_JPRB, 0.20302E+00_JPRB ,&
&  0.20390E+00_JPRB, 0.20571E+00_JPRB, 0.20685E+00_JPRB, 0.20792E+00_JPRB, 0.20848E+00_JPRB, 0.20896E+00_JPRB, 0.20960E+00_JPRB ,&
&  0.21065E+00_JPRB, 0.21193E+00_JPRB, 0.21308E+00_JPRB, 0.21461E+00_JPRB, 0.21617E+00_JPRB, 0.21833E+00_JPRB, 0.22063E+00_JPRB ,&
&  0.22327E+00_JPRB, 0.22653E+00_JPRB, 0.23060E+00_JPRB, 0.23430E+00_JPRB, 0.24006E+00_JPRB, 0.24584E+00_JPRB, 0.25233E+00_JPRB ,&
&  0.25900E+00_JPRB, 0.26620E+00_JPRB, 0.27564E+00_JPRB, 0.28683E+00_JPRB, 0.30075E+00_JPRB, 0.30900E+00_JPRB, 0.32023E+00_JPRB ,&
&  0.33743E+00_JPRB, 0.37371E+00_JPRB, 0.50943E+00_JPRB, 0.52300E+00_JPRB, 0.52767E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.30722E+00_JPRB, 0.31301E+00_JPRB, 0.31702E+00_JPRB, 0.32608E+00_JPRB, 0.33903E+00_JPRB, 0.43596E+00_JPRB, 0.43096E+00_JPRB ,&
&  0.43003E+00_JPRB, 0.38706E+00_JPRB, 0.37902E+00_JPRB, 0.36685E+00_JPRB, 0.35151E+00_JPRB, 0.33254E+00_JPRB, 0.32080E+00_JPRB ,&
&  0.31101E+00_JPRB, 0.29964E+00_JPRB, 0.28898E+00_JPRB, 0.28274E+00_JPRB, 0.27779E+00_JPRB, 0.27342E+00_JPRB, 0.27089E+00_JPRB ,&
&  0.26993E+00_JPRB, 0.27038E+00_JPRB, 0.27115E+00_JPRB, 0.27230E+00_JPRB, 0.27566E+00_JPRB, 0.27932E+00_JPRB, 0.28300E+00_JPRB ,&
&  0.28482E+00_JPRB, 0.28900E+00_JPRB, 0.28982E+00_JPRB, 0.28665E+00_JPRB, 0.28146E+00_JPRB, 0.27428E+00_JPRB, 0.26702E+00_JPRB ,&
&  0.25954E+00_JPRB, 0.25258E+00_JPRB, 0.24683E+00_JPRB, 0.24163E+00_JPRB, 0.23573E+00_JPRB, 0.23016E+00_JPRB, 0.22659E+00_JPRB ,&
&  0.22336E+00_JPRB, 0.22008E+00_JPRB, 0.21749E+00_JPRB, 0.21620E+00_JPRB, 0.21474E+00_JPRB, 0.21376E+00_JPRB, 0.21305E+00_JPRB ,&
&  0.21286E+00_JPRB, 0.21275E+00_JPRB, 0.21283E+00_JPRB, 0.21313E+00_JPRB, 0.21311E+00_JPRB, 0.21304E+00_JPRB, 0.21301E+00_JPRB ,&
&  0.21357E+00_JPRB, 0.21360E+00_JPRB, 0.21353E+00_JPRB, 0.21330E+00_JPRB, 0.21395E+00_JPRB, 0.21416E+00_JPRB, 0.21454E+00_JPRB ,&
&  0.21516E+00_JPRB, 0.21602E+00_JPRB, 0.21667E+00_JPRB, 0.21741E+00_JPRB, 0.21813E+00_JPRB, 0.21899E+00_JPRB, 0.22000E+00_JPRB ,&
&  0.22101E+00_JPRB, 0.22251E+00_JPRB, 0.22404E+00_JPRB, 0.22540E+00_JPRB, 0.22660E+00_JPRB, 0.22825E+00_JPRB, 0.23009E+00_JPRB ,&
&  0.23285E+00_JPRB, 0.23553E+00_JPRB, 0.23828E+00_JPRB, 0.23995E+00_JPRB, 0.24264E+00_JPRB, 0.24489E+00_JPRB, 0.24727E+00_JPRB ,&
&  0.25004E+00_JPRB, 0.25448E+00_JPRB, 0.26093E+00_JPRB, 0.26795E+00_JPRB, 0.27575E+00_JPRB, 0.28237E+00_JPRB, 0.28675E+00_JPRB ,&
&  0.29056E+00_JPRB, 0.29470E+00_JPRB, 0.30036E+00_JPRB, 0.30338E+00_JPRB, 0.30579E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.27223E+00_JPRB, 0.27225E+00_JPRB, 0.27231E+00_JPRB, 0.27195E+00_JPRB, 0.27039E+00_JPRB, 0.26918E+00_JPRB, 0.26810E+00_JPRB ,&
&  0.26754E+00_JPRB, 0.26666E+00_JPRB, 0.26622E+00_JPRB, 0.26603E+00_JPRB, 0.26563E+00_JPRB, 0.26505E+00_JPRB, 0.26429E+00_JPRB ,&
&  0.26330E+00_JPRB, 0.26202E+00_JPRB, 0.26068E+00_JPRB, 0.26040E+00_JPRB, 0.26048E+00_JPRB, 0.26058E+00_JPRB, 0.26061E+00_JPRB ,&
&  0.26054E+00_JPRB, 0.26044E+00_JPRB, 0.25991E+00_JPRB, 0.25996E+00_JPRB, 0.25995E+00_JPRB, 0.25995E+00_JPRB, 0.26006E+00_JPRB ,&
&  0.26027E+00_JPRB, 0.26047E+00_JPRB, 0.26024E+00_JPRB, 0.25976E+00_JPRB, 0.25887E+00_JPRB, 0.25719E+00_JPRB, 0.25456E+00_JPRB ,&
&  0.25156E+00_JPRB, 0.24857E+00_JPRB, 0.24617E+00_JPRB, 0.24397E+00_JPRB, 0.24195E+00_JPRB, 0.24011E+00_JPRB, 0.23859E+00_JPRB ,&
&  0.23677E+00_JPRB, 0.23497E+00_JPRB, 0.23332E+00_JPRB, 0.23216E+00_JPRB, 0.23087E+00_JPRB, 0.22976E+00_JPRB, 0.22874E+00_JPRB ,&
&  0.22804E+00_JPRB, 0.22743E+00_JPRB, 0.22690E+00_JPRB, 0.22663E+00_JPRB, 0.22642E+00_JPRB, 0.22622E+00_JPRB, 0.22600E+00_JPRB ,&
&  0.22580E+00_JPRB, 0.22566E+00_JPRB, 0.22554E+00_JPRB, 0.22552E+00_JPRB, 0.22562E+00_JPRB, 0.22552E+00_JPRB, 0.22539E+00_JPRB ,&
&  0.22535E+00_JPRB, 0.22563E+00_JPRB, 0.22582E+00_JPRB, 0.22614E+00_JPRB, 0.22662E+00_JPRB, 0.22729E+00_JPRB, 0.22802E+00_JPRB ,&
&  0.22876E+00_JPRB, 0.22961E+00_JPRB, 0.23054E+00_JPRB, 0.23167E+00_JPRB, 0.23289E+00_JPRB, 0.23409E+00_JPRB, 0.23526E+00_JPRB ,&
&  0.23650E+00_JPRB, 0.23765E+00_JPRB, 0.23872E+00_JPRB, 0.23950E+00_JPRB, 0.24057E+00_JPRB, 0.24166E+00_JPRB, 0.24293E+00_JPRB ,&
&  0.24438E+00_JPRB, 0.24630E+00_JPRB, 0.24896E+00_JPRB, 0.25213E+00_JPRB, 0.25611E+00_JPRB, 0.25904E+00_JPRB, 0.26202E+00_JPRB ,&
&  0.26417E+00_JPRB, 0.26584E+00_JPRB, 0.26708E+00_JPRB, 0.26866E+00_JPRB, 0.27041E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB ,&
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB ,&
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB ,&
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB ,&
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB ,&
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB ,&
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB ,&
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB ,&
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB ,&
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB ,&
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB ,&
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB ,&
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB ,&
&  0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB, 0.24278E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2090:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_A1B2090
