!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_A1B2100 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS2100* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB, 0.23197E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.23977E+00_JPRB, 0.23960E+00_JPRB, 0.23947E+00_JPRB, 0.23936E+00_JPRB, 0.23924E+00_JPRB, 0.23919E+00_JPRB, 0.23917E+00_JPRB ,&
&  0.23910E+00_JPRB, 0.23901E+00_JPRB, 0.23882E+00_JPRB, 0.23863E+00_JPRB, 0.23848E+00_JPRB, 0.23827E+00_JPRB, 0.23800E+00_JPRB ,&
&  0.23766E+00_JPRB, 0.23739E+00_JPRB, 0.23699E+00_JPRB, 0.23650E+00_JPRB, 0.23599E+00_JPRB, 0.23547E+00_JPRB, 0.23485E+00_JPRB ,&
&  0.23412E+00_JPRB, 0.23340E+00_JPRB, 0.23264E+00_JPRB, 0.23180E+00_JPRB, 0.23093E+00_JPRB, 0.23000E+00_JPRB, 0.22916E+00_JPRB ,&
&  0.22852E+00_JPRB, 0.22778E+00_JPRB, 0.22704E+00_JPRB, 0.22629E+00_JPRB, 0.22564E+00_JPRB, 0.22512E+00_JPRB, 0.22457E+00_JPRB ,&
&  0.22403E+00_JPRB, 0.22357E+00_JPRB, 0.22345E+00_JPRB, 0.22345E+00_JPRB, 0.22353E+00_JPRB, 0.22387E+00_JPRB, 0.22450E+00_JPRB ,&
&  0.22500E+00_JPRB, 0.22545E+00_JPRB, 0.22602E+00_JPRB, 0.22667E+00_JPRB, 0.22736E+00_JPRB, 0.22806E+00_JPRB, 0.22867E+00_JPRB ,&
&  0.22915E+00_JPRB, 0.22954E+00_JPRB, 0.22997E+00_JPRB, 0.23032E+00_JPRB, 0.23054E+00_JPRB, 0.23073E+00_JPRB, 0.23095E+00_JPRB ,&
&  0.23112E+00_JPRB, 0.23123E+00_JPRB, 0.23134E+00_JPRB, 0.23156E+00_JPRB, 0.23175E+00_JPRB, 0.23199E+00_JPRB, 0.23232E+00_JPRB ,&
&  0.23279E+00_JPRB, 0.23313E+00_JPRB, 0.23355E+00_JPRB, 0.23408E+00_JPRB, 0.23477E+00_JPRB, 0.23561E+00_JPRB, 0.23627E+00_JPRB ,&
&  0.23697E+00_JPRB, 0.23769E+00_JPRB, 0.23853E+00_JPRB, 0.23914E+00_JPRB, 0.23979E+00_JPRB, 0.24035E+00_JPRB, 0.24090E+00_JPRB ,&
&  0.24106E+00_JPRB, 0.24131E+00_JPRB, 0.24162E+00_JPRB, 0.24189E+00_JPRB, 0.24173E+00_JPRB, 0.24166E+00_JPRB, 0.24162E+00_JPRB ,&
&  0.24183E+00_JPRB, 0.24147E+00_JPRB, 0.24136E+00_JPRB, 0.24120E+00_JPRB, 0.24116E+00_JPRB, 0.24086E+00_JPRB, 0.24068E+00_JPRB ,&
&  0.24048E+00_JPRB, 0.24035E+00_JPRB, 0.24017E+00_JPRB, 0.24002E+00_JPRB, 0.23987E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.24720E+00_JPRB, 0.24693E+00_JPRB, 0.24660E+00_JPRB, 0.24622E+00_JPRB, 0.24581E+00_JPRB, 0.24556E+00_JPRB, 0.24529E+00_JPRB ,&
&  0.24494E+00_JPRB, 0.24465E+00_JPRB, 0.24413E+00_JPRB, 0.24372E+00_JPRB, 0.24337E+00_JPRB, 0.24327E+00_JPRB, 0.24305E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24228E+00_JPRB, 0.24184E+00_JPRB, 0.24099E+00_JPRB, 0.24002E+00_JPRB, 0.23869E+00_JPRB, 0.23741E+00_JPRB ,&
&  0.23586E+00_JPRB, 0.23408E+00_JPRB, 0.23248E+00_JPRB, 0.23072E+00_JPRB, 0.22906E+00_JPRB, 0.22740E+00_JPRB, 0.22621E+00_JPRB ,&
&  0.22544E+00_JPRB, 0.22435E+00_JPRB, 0.22338E+00_JPRB, 0.22246E+00_JPRB, 0.22142E+00_JPRB, 0.22056E+00_JPRB, 0.21973E+00_JPRB ,&
&  0.21827E+00_JPRB, 0.21671E+00_JPRB, 0.21641E+00_JPRB, 0.21629E+00_JPRB, 0.21657E+00_JPRB, 0.21756E+00_JPRB, 0.21959E+00_JPRB ,&
&  0.22229E+00_JPRB, 0.22486E+00_JPRB, 0.22762E+00_JPRB, 0.23004E+00_JPRB, 0.23199E+00_JPRB, 0.23341E+00_JPRB, 0.23454E+00_JPRB ,&
&  0.23518E+00_JPRB, 0.23561E+00_JPRB, 0.23582E+00_JPRB, 0.23579E+00_JPRB, 0.23545E+00_JPRB, 0.23536E+00_JPRB, 0.23543E+00_JPRB ,&
&  0.23563E+00_JPRB, 0.23504E+00_JPRB, 0.23491E+00_JPRB, 0.23511E+00_JPRB, 0.23562E+00_JPRB, 0.23607E+00_JPRB, 0.23698E+00_JPRB ,&
&  0.23832E+00_JPRB, 0.23998E+00_JPRB, 0.24151E+00_JPRB, 0.24356E+00_JPRB, 0.24595E+00_JPRB, 0.24874E+00_JPRB, 0.25042E+00_JPRB ,&
&  0.25248E+00_JPRB, 0.25463E+00_JPRB, 0.25690E+00_JPRB, 0.25744E+00_JPRB, 0.25804E+00_JPRB, 0.25847E+00_JPRB, 0.25905E+00_JPRB ,&
&  0.25818E+00_JPRB, 0.25787E+00_JPRB, 0.25767E+00_JPRB, 0.25799E+00_JPRB, 0.25692E+00_JPRB, 0.25642E+00_JPRB, 0.25591E+00_JPRB ,&
&  0.25625E+00_JPRB, 0.25472E+00_JPRB, 0.25398E+00_JPRB, 0.25302E+00_JPRB, 0.25256E+00_JPRB, 0.25117E+00_JPRB, 0.25015E+00_JPRB ,&
&  0.24935E+00_JPRB, 0.24896E+00_JPRB, 0.24839E+00_JPRB, 0.24792E+00_JPRB, 0.24760E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.25318E+00_JPRB, 0.25200E+00_JPRB, 0.25011E+00_JPRB, 0.24763E+00_JPRB, 0.24545E+00_JPRB, 0.24383E+00_JPRB, 0.24214E+00_JPRB ,&
&  0.24079E+00_JPRB, 0.23964E+00_JPRB, 0.23888E+00_JPRB, 0.23840E+00_JPRB, 0.23794E+00_JPRB, 0.23860E+00_JPRB, 0.23934E+00_JPRB ,&
&  0.23995E+00_JPRB, 0.24010E+00_JPRB, 0.24117E+00_JPRB, 0.24124E+00_JPRB, 0.24089E+00_JPRB, 0.23969E+00_JPRB, 0.23766E+00_JPRB ,&
&  0.23528E+00_JPRB, 0.23241E+00_JPRB, 0.22901E+00_JPRB, 0.22505E+00_JPRB, 0.22144E+00_JPRB, 0.20818E+00_JPRB, 0.20532E+00_JPRB ,&
&  0.20645E+00_JPRB, 0.20438E+00_JPRB, 0.20281E+00_JPRB, 0.20197E+00_JPRB, 0.20111E+00_JPRB, 0.19980E+00_JPRB, 0.19855E+00_JPRB ,&
&  0.18814E+00_JPRB, 0.18730E+00_JPRB, 0.18567E+00_JPRB, 0.18455E+00_JPRB, 0.18575E+00_JPRB, 0.19028E+00_JPRB, 0.19823E+00_JPRB ,&
&  0.21338E+00_JPRB, 0.21943E+00_JPRB, 0.22482E+00_JPRB, 0.23103E+00_JPRB, 0.23593E+00_JPRB, 0.23951E+00_JPRB, 0.24291E+00_JPRB ,&
&  0.24551E+00_JPRB, 0.24794E+00_JPRB, 0.24954E+00_JPRB, 0.25016E+00_JPRB, 0.24984E+00_JPRB, 0.24951E+00_JPRB, 0.24919E+00_JPRB ,&
&  0.24906E+00_JPRB, 0.24900E+00_JPRB, 0.24920E+00_JPRB, 0.24921E+00_JPRB, 0.24905E+00_JPRB, 0.24913E+00_JPRB, 0.25052E+00_JPRB ,&
&  0.25261E+00_JPRB, 0.25597E+00_JPRB, 0.25891E+00_JPRB, 0.26321E+00_JPRB, 0.26711E+00_JPRB, 0.27217E+00_JPRB, 0.27418E+00_JPRB ,&
&  0.27665E+00_JPRB, 0.27824E+00_JPRB, 0.28035E+00_JPRB, 0.28030E+00_JPRB, 0.28015E+00_JPRB, 0.27962E+00_JPRB, 0.27961E+00_JPRB ,&
&  0.27829E+00_JPRB, 0.27738E+00_JPRB, 0.27610E+00_JPRB, 0.27583E+00_JPRB, 0.27423E+00_JPRB, 0.27375E+00_JPRB, 0.27266E+00_JPRB ,&
&  0.27308E+00_JPRB, 0.27138E+00_JPRB, 0.27083E+00_JPRB, 0.26878E+00_JPRB, 0.26702E+00_JPRB, 0.26386E+00_JPRB, 0.26120E+00_JPRB ,&
&  0.25902E+00_JPRB, 0.25742E+00_JPRB, 0.25613E+00_JPRB, 0.25507E+00_JPRB, 0.25402E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.25610E+00_JPRB, 0.25239E+00_JPRB, 0.24824E+00_JPRB, 0.24472E+00_JPRB, 0.24158E+00_JPRB, 0.23929E+00_JPRB, 0.23719E+00_JPRB ,&
&  0.23561E+00_JPRB, 0.23420E+00_JPRB, 0.23323E+00_JPRB, 0.23246E+00_JPRB, 0.23198E+00_JPRB, 0.23196E+00_JPRB, 0.23199E+00_JPRB ,&
&  0.23247E+00_JPRB, 0.23282E+00_JPRB, 0.23373E+00_JPRB, 0.23434E+00_JPRB, 0.23516E+00_JPRB, 0.23482E+00_JPRB, 0.23299E+00_JPRB ,&
&  0.23079E+00_JPRB, 0.22813E+00_JPRB, 0.21754E+00_JPRB, 0.21398E+00_JPRB, 0.20519E+00_JPRB, 0.19926E+00_JPRB, 0.19598E+00_JPRB ,&
&  0.19000E+00_JPRB, 0.18716E+00_JPRB, 0.18116E+00_JPRB, 0.17734E+00_JPRB, 0.17584E+00_JPRB, 0.17043E+00_JPRB, 0.16794E+00_JPRB ,&
&  0.16427E+00_JPRB, 0.15818E+00_JPRB, 0.15571E+00_JPRB, 0.15452E+00_JPRB, 0.15644E+00_JPRB, 0.16135E+00_JPRB, 0.16998E+00_JPRB ,&
&  0.18586E+00_JPRB, 0.20348E+00_JPRB, 0.22751E+00_JPRB, 0.23425E+00_JPRB, 0.23605E+00_JPRB, 0.24169E+00_JPRB, 0.24589E+00_JPRB ,&
&  0.24838E+00_JPRB, 0.25386E+00_JPRB, 0.25989E+00_JPRB, 0.26179E+00_JPRB, 0.26318E+00_JPRB, 0.26346E+00_JPRB, 0.26334E+00_JPRB ,&
&  0.26355E+00_JPRB, 0.26555E+00_JPRB, 0.26667E+00_JPRB, 0.26787E+00_JPRB, 0.26907E+00_JPRB, 0.27032E+00_JPRB, 0.27328E+00_JPRB ,&
&  0.27576E+00_JPRB, 0.28180E+00_JPRB, 0.28680E+00_JPRB, 0.29495E+00_JPRB, 0.29834E+00_JPRB, 0.30251E+00_JPRB, 0.30356E+00_JPRB ,&
&  0.30539E+00_JPRB, 0.30641E+00_JPRB, 0.30727E+00_JPRB, 0.30471E+00_JPRB, 0.30417E+00_JPRB, 0.30595E+00_JPRB, 0.30517E+00_JPRB ,&
&  0.30404E+00_JPRB, 0.30303E+00_JPRB, 0.30159E+00_JPRB, 0.30022E+00_JPRB, 0.29911E+00_JPRB, 0.29876E+00_JPRB, 0.29794E+00_JPRB ,&
&  0.29853E+00_JPRB, 0.29727E+00_JPRB, 0.29590E+00_JPRB, 0.29318E+00_JPRB, 0.28952E+00_JPRB, 0.28461E+00_JPRB, 0.27863E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.26995E+00_JPRB, 0.26569E+00_JPRB, 0.26261E+00_JPRB, 0.25926E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.24789E+00_JPRB, 0.24302E+00_JPRB, 0.23909E+00_JPRB, 0.23643E+00_JPRB, 0.23424E+00_JPRB, 0.23296E+00_JPRB, 0.23162E+00_JPRB ,&
&  0.23100E+00_JPRB, 0.23005E+00_JPRB, 0.22961E+00_JPRB, 0.22905E+00_JPRB, 0.22899E+00_JPRB, 0.22921E+00_JPRB, 0.22980E+00_JPRB ,&
&  0.23063E+00_JPRB, 0.23121E+00_JPRB, 0.23204E+00_JPRB, 0.23173E+00_JPRB, 0.23172E+00_JPRB, 0.23126E+00_JPRB, 0.23077E+00_JPRB ,&
&  0.22829E+00_JPRB, 0.22474E+00_JPRB, 0.21739E+00_JPRB, 0.21340E+00_JPRB, 0.20577E+00_JPRB, 0.20089E+00_JPRB, 0.19614E+00_JPRB ,&
&  0.19339E+00_JPRB, 0.18271E+00_JPRB, 0.18511E+00_JPRB, 0.17767E+00_JPRB, 0.17027E+00_JPRB, 0.16315E+00_JPRB, 0.15544E+00_JPRB ,&
&  0.15228E+00_JPRB, 0.14467E+00_JPRB, 0.14185E+00_JPRB, 0.14181E+00_JPRB, 0.14311E+00_JPRB, 0.14877E+00_JPRB, 0.15740E+00_JPRB ,&
&  0.17327E+00_JPRB, 0.21341E+00_JPRB, 0.22651E+00_JPRB, 0.23343E+00_JPRB, 0.23331E+00_JPRB, 0.23476E+00_JPRB, 0.23376E+00_JPRB ,&
&  0.24009E+00_JPRB, 0.24763E+00_JPRB, 0.25559E+00_JPRB, 0.26026E+00_JPRB, 0.26737E+00_JPRB, 0.27109E+00_JPRB, 0.27347E+00_JPRB ,&
&  0.27640E+00_JPRB, 0.28129E+00_JPRB, 0.28530E+00_JPRB, 0.28951E+00_JPRB, 0.29547E+00_JPRB, 0.30231E+00_JPRB, 0.31072E+00_JPRB ,&
&  0.31584E+00_JPRB, 0.32624E+00_JPRB, 0.32983E+00_JPRB, 0.33479E+00_JPRB, 0.33543E+00_JPRB, 0.33448E+00_JPRB, 0.33437E+00_JPRB ,&
&  0.33550E+00_JPRB, 0.33703E+00_JPRB, 0.33829E+00_JPRB, 0.33777E+00_JPRB, 0.33753E+00_JPRB, 0.33311E+00_JPRB, 0.33106E+00_JPRB ,&
&  0.33001E+00_JPRB, 0.32798E+00_JPRB, 0.32556E+00_JPRB, 0.32252E+00_JPRB, 0.31907E+00_JPRB, 0.31530E+00_JPRB, 0.31305E+00_JPRB ,&
&  0.31021E+00_JPRB, 0.30714E+00_JPRB, 0.30350E+00_JPRB, 0.30010E+00_JPRB, 0.29646E+00_JPRB, 0.29281E+00_JPRB, 0.28649E+00_JPRB ,&
&  0.27822E+00_JPRB, 0.27128E+00_JPRB, 0.26446E+00_JPRB, 0.25904E+00_JPRB, 0.25326E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.24121E+00_JPRB, 0.23735E+00_JPRB, 0.23441E+00_JPRB, 0.23334E+00_JPRB, 0.23211E+00_JPRB, 0.23187E+00_JPRB, 0.23092E+00_JPRB ,&
&  0.23040E+00_JPRB, 0.22954E+00_JPRB, 0.22904E+00_JPRB, 0.22828E+00_JPRB, 0.22794E+00_JPRB, 0.22764E+00_JPRB, 0.22787E+00_JPRB ,&
&  0.22827E+00_JPRB, 0.22930E+00_JPRB, 0.23040E+00_JPRB, 0.23058E+00_JPRB, 0.23105E+00_JPRB, 0.23099E+00_JPRB, 0.23067E+00_JPRB ,&
&  0.22876E+00_JPRB, 0.22522E+00_JPRB, 0.21805E+00_JPRB, 0.21289E+00_JPRB, 0.20852E+00_JPRB, 0.20234E+00_JPRB, 0.20223E+00_JPRB ,&
&  0.20109E+00_JPRB, 0.19710E+00_JPRB, 0.19019E+00_JPRB, 0.18789E+00_JPRB, 0.18343E+00_JPRB, 0.16446E+00_JPRB, 0.16342E+00_JPRB ,&
&  0.15736E+00_JPRB, 0.14567E+00_JPRB, 0.14206E+00_JPRB, 0.13962E+00_JPRB, 0.14063E+00_JPRB, 0.14576E+00_JPRB, 0.15390E+00_JPRB ,&
&  0.16993E+00_JPRB, 0.21864E+00_JPRB, 0.23066E+00_JPRB, 0.23658E+00_JPRB, 0.23459E+00_JPRB, 0.23830E+00_JPRB, 0.23982E+00_JPRB ,&
&  0.24498E+00_JPRB, 0.24862E+00_JPRB, 0.25332E+00_JPRB, 0.25695E+00_JPRB, 0.26131E+00_JPRB, 0.26258E+00_JPRB, 0.26652E+00_JPRB ,&
&  0.27436E+00_JPRB, 0.28303E+00_JPRB, 0.28972E+00_JPRB, 0.29919E+00_JPRB, 0.31342E+00_JPRB, 0.34133E+00_JPRB, 0.37311E+00_JPRB ,&
&  0.37628E+00_JPRB, 0.37371E+00_JPRB, 0.37219E+00_JPRB, 0.36875E+00_JPRB, 0.36774E+00_JPRB, 0.36700E+00_JPRB, 0.36880E+00_JPRB ,&
&  0.37353E+00_JPRB, 0.37832E+00_JPRB, 0.37292E+00_JPRB, 0.36060E+00_JPRB, 0.35519E+00_JPRB, 0.34595E+00_JPRB, 0.34358E+00_JPRB ,&
&  0.34023E+00_JPRB, 0.34321E+00_JPRB, 0.33922E+00_JPRB, 0.33357E+00_JPRB, 0.32847E+00_JPRB, 0.32320E+00_JPRB, 0.31976E+00_JPRB ,&
&  0.31552E+00_JPRB, 0.31064E+00_JPRB, 0.30568E+00_JPRB, 0.30208E+00_JPRB, 0.29906E+00_JPRB, 0.29675E+00_JPRB, 0.29273E+00_JPRB ,&
&  0.28382E+00_JPRB, 0.27204E+00_JPRB, 0.26289E+00_JPRB, 0.25387E+00_JPRB, 0.24656E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.23930E+00_JPRB, 0.23791E+00_JPRB, 0.23757E+00_JPRB, 0.23902E+00_JPRB, 0.23981E+00_JPRB, 0.24064E+00_JPRB, 0.23939E+00_JPRB ,&
&  0.23800E+00_JPRB, 0.23637E+00_JPRB, 0.23485E+00_JPRB, 0.23322E+00_JPRB, 0.23200E+00_JPRB, 0.23059E+00_JPRB, 0.22948E+00_JPRB ,&
&  0.22837E+00_JPRB, 0.22479E+00_JPRB, 0.22336E+00_JPRB, 0.22432E+00_JPRB, 0.22583E+00_JPRB, 0.22730E+00_JPRB, 0.22923E+00_JPRB ,&
&  0.22821E+00_JPRB, 0.22509E+00_JPRB, 0.22067E+00_JPRB, 0.21471E+00_JPRB, 0.20997E+00_JPRB, 0.20812E+00_JPRB, 0.20554E+00_JPRB ,&
&  0.20038E+00_JPRB, 0.19615E+00_JPRB, 0.19967E+00_JPRB, 0.19913E+00_JPRB, 0.18938E+00_JPRB, 0.18567E+00_JPRB, 0.17456E+00_JPRB ,&
&  0.16230E+00_JPRB, 0.14775E+00_JPRB, 0.14412E+00_JPRB, 0.14257E+00_JPRB, 0.14595E+00_JPRB, 0.15203E+00_JPRB, 0.16506E+00_JPRB ,&
&  0.18620E+00_JPRB, 0.22470E+00_JPRB, 0.23660E+00_JPRB, 0.24426E+00_JPRB, 0.24883E+00_JPRB, 0.25149E+00_JPRB, 0.25365E+00_JPRB ,&
&  0.25817E+00_JPRB, 0.26114E+00_JPRB, 0.26537E+00_JPRB, 0.27029E+00_JPRB, 0.27663E+00_JPRB, 0.28121E+00_JPRB, 0.28267E+00_JPRB ,&
&  0.29583E+00_JPRB, 0.29913E+00_JPRB, 0.29990E+00_JPRB, 0.31609E+00_JPRB, 0.35427E+00_JPRB, 0.39716E+00_JPRB, 0.41840E+00_JPRB ,&
&  0.41756E+00_JPRB, 0.41353E+00_JPRB, 0.39916E+00_JPRB, 0.39087E+00_JPRB, 0.38673E+00_JPRB, 0.38587E+00_JPRB, 0.38772E+00_JPRB ,&
&  0.39229E+00_JPRB, 0.40124E+00_JPRB, 0.42940E+00_JPRB, 0.40845E+00_JPRB, 0.39512E+00_JPRB, 0.38194E+00_JPRB, 0.36694E+00_JPRB ,&
&  0.35504E+00_JPRB, 0.34798E+00_JPRB, 0.34299E+00_JPRB, 0.33663E+00_JPRB, 0.33095E+00_JPRB, 0.32629E+00_JPRB, 0.32485E+00_JPRB ,&
&  0.32034E+00_JPRB, 0.31470E+00_JPRB, 0.30751E+00_JPRB, 0.30438E+00_JPRB, 0.30144E+00_JPRB, 0.29828E+00_JPRB, 0.29474E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.27211E+00_JPRB, 0.26121E+00_JPRB, 0.25150E+00_JPRB, 0.24408E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.24293E+00_JPRB, 0.24350E+00_JPRB, 0.24627E+00_JPRB, 0.25129E+00_JPRB, 0.25566E+00_JPRB, 0.25239E+00_JPRB, 0.24824E+00_JPRB ,&
&  0.24547E+00_JPRB, 0.24471E+00_JPRB, 0.24453E+00_JPRB, 0.24278E+00_JPRB, 0.24121E+00_JPRB, 0.23940E+00_JPRB, 0.23736E+00_JPRB ,&
&  0.23509E+00_JPRB, 0.23257E+00_JPRB, 0.22958E+00_JPRB, 0.22554E+00_JPRB, 0.22327E+00_JPRB, 0.22377E+00_JPRB, 0.22599E+00_JPRB ,&
&  0.22812E+00_JPRB, 0.22768E+00_JPRB, 0.22578E+00_JPRB, 0.22044E+00_JPRB, 0.21437E+00_JPRB, 0.20964E+00_JPRB, 0.20956E+00_JPRB ,&
&  0.20732E+00_JPRB, 0.20279E+00_JPRB, 0.19859E+00_JPRB, 0.20279E+00_JPRB, 0.20525E+00_JPRB, 0.19575E+00_JPRB, 0.18284E+00_JPRB ,&
&  0.16968E+00_JPRB, 0.15818E+00_JPRB, 0.15280E+00_JPRB, 0.15249E+00_JPRB, 0.15846E+00_JPRB, 0.17003E+00_JPRB, 0.18639E+00_JPRB ,&
&  0.21166E+00_JPRB, 0.23189E+00_JPRB, 0.24269E+00_JPRB, 0.25129E+00_JPRB, 0.25408E+00_JPRB, 0.25474E+00_JPRB, 0.25538E+00_JPRB ,&
&  0.25765E+00_JPRB, 0.25998E+00_JPRB, 0.26384E+00_JPRB, 0.26739E+00_JPRB, 0.27627E+00_JPRB, 0.28864E+00_JPRB, 0.31119E+00_JPRB ,&
&  0.37473E+00_JPRB, 0.37270E+00_JPRB, 0.33305E+00_JPRB, 0.34988E+00_JPRB, 0.38962E+00_JPRB, 0.42737E+00_JPRB, 0.45708E+00_JPRB ,&
&  0.46346E+00_JPRB, 0.44196E+00_JPRB, 0.42283E+00_JPRB, 0.41294E+00_JPRB, 0.40635E+00_JPRB, 0.40195E+00_JPRB, 0.40181E+00_JPRB ,&
&  0.40601E+00_JPRB, 0.41869E+00_JPRB, 0.47578E+00_JPRB, 0.41773E+00_JPRB, 0.38398E+00_JPRB, 0.36971E+00_JPRB, 0.35507E+00_JPRB ,&
&  0.34316E+00_JPRB, 0.33684E+00_JPRB, 0.33615E+00_JPRB, 0.33686E+00_JPRB, 0.32465E+00_JPRB, 0.31351E+00_JPRB, 0.30528E+00_JPRB ,&
&  0.30152E+00_JPRB, 0.29715E+00_JPRB, 0.29498E+00_JPRB, 0.29696E+00_JPRB, 0.29594E+00_JPRB, 0.29020E+00_JPRB, 0.28703E+00_JPRB ,&
&  0.28107E+00_JPRB, 0.26958E+00_JPRB, 0.26162E+00_JPRB, 0.25405E+00_JPRB, 0.24667E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.24231E+00_JPRB, 0.24998E+00_JPRB, 0.26062E+00_JPRB, 0.26856E+00_JPRB, 0.27175E+00_JPRB, 0.26009E+00_JPRB, 0.24805E+00_JPRB ,&
&  0.24191E+00_JPRB, 0.23947E+00_JPRB, 0.24059E+00_JPRB, 0.24122E+00_JPRB, 0.24128E+00_JPRB, 0.24131E+00_JPRB, 0.24092E+00_JPRB ,&
&  0.23839E+00_JPRB, 0.23465E+00_JPRB, 0.23110E+00_JPRB, 0.22764E+00_JPRB, 0.22481E+00_JPRB, 0.22374E+00_JPRB, 0.22425E+00_JPRB ,&
&  0.22570E+00_JPRB, 0.22614E+00_JPRB, 0.22542E+00_JPRB, 0.22201E+00_JPRB, 0.21910E+00_JPRB, 0.21690E+00_JPRB, 0.21334E+00_JPRB ,&
&  0.20741E+00_JPRB, 0.20777E+00_JPRB, 0.20721E+00_JPRB, 0.20704E+00_JPRB, 0.21029E+00_JPRB, 0.20482E+00_JPRB, 0.19185E+00_JPRB ,&
&  0.17798E+00_JPRB, 0.16805E+00_JPRB, 0.16587E+00_JPRB, 0.17215E+00_JPRB, 0.18698E+00_JPRB, 0.21041E+00_JPRB, 0.22193E+00_JPRB ,&
&  0.23189E+00_JPRB, 0.24048E+00_JPRB, 0.24565E+00_JPRB, 0.25091E+00_JPRB, 0.25512E+00_JPRB, 0.25728E+00_JPRB, 0.25825E+00_JPRB ,&
&  0.25833E+00_JPRB, 0.25876E+00_JPRB, 0.26026E+00_JPRB, 0.24904E+00_JPRB, 0.25866E+00_JPRB, 0.30969E+00_JPRB, 0.33991E+00_JPRB ,&
&  0.37228E+00_JPRB, 0.37317E+00_JPRB, 0.37457E+00_JPRB, 0.39844E+00_JPRB, 0.42442E+00_JPRB, 0.44669E+00_JPRB, 0.46754E+00_JPRB ,&
&  0.47646E+00_JPRB, 0.45255E+00_JPRB, 0.42865E+00_JPRB, 0.42057E+00_JPRB, 0.41807E+00_JPRB, 0.41464E+00_JPRB, 0.40174E+00_JPRB ,&
&  0.39185E+00_JPRB, 0.37422E+00_JPRB, 0.35166E+00_JPRB, 0.33269E+00_JPRB, 0.33013E+00_JPRB, 0.33823E+00_JPRB, 0.34898E+00_JPRB ,&
&  0.34804E+00_JPRB, 0.34555E+00_JPRB, 0.34430E+00_JPRB, 0.34494E+00_JPRB, 0.31900E+00_JPRB, 0.30145E+00_JPRB, 0.28880E+00_JPRB ,&
&  0.28213E+00_JPRB, 0.27902E+00_JPRB, 0.27764E+00_JPRB, 0.28172E+00_JPRB, 0.28199E+00_JPRB, 0.27988E+00_JPRB, 0.27144E+00_JPRB ,&
&  0.25979E+00_JPRB, 0.25060E+00_JPRB, 0.24427E+00_JPRB, 0.24076E+00_JPRB, 0.24033E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.26925E+00_JPRB, 0.27648E+00_JPRB, 0.28110E+00_JPRB, 0.28511E+00_JPRB, 0.28331E+00_JPRB, 0.27873E+00_JPRB, 0.27324E+00_JPRB ,&
&  0.27535E+00_JPRB, 0.26862E+00_JPRB, 0.26256E+00_JPRB, 0.25290E+00_JPRB, 0.24479E+00_JPRB, 0.23758E+00_JPRB, 0.23612E+00_JPRB ,&
&  0.23755E+00_JPRB, 0.23921E+00_JPRB, 0.23789E+00_JPRB, 0.23369E+00_JPRB, 0.22922E+00_JPRB, 0.22642E+00_JPRB, 0.22580E+00_JPRB ,&
&  0.22629E+00_JPRB, 0.22737E+00_JPRB, 0.22734E+00_JPRB, 0.22591E+00_JPRB, 0.22425E+00_JPRB, 0.22221E+00_JPRB, 0.21964E+00_JPRB ,&
&  0.21568E+00_JPRB, 0.21410E+00_JPRB, 0.21380E+00_JPRB, 0.21474E+00_JPRB, 0.21852E+00_JPRB, 0.21368E+00_JPRB, 0.20145E+00_JPRB ,&
&  0.18732E+00_JPRB, 0.18320E+00_JPRB, 0.19127E+00_JPRB, 0.21875E+00_JPRB, 0.23365E+00_JPRB, 0.24567E+00_JPRB, 0.25635E+00_JPRB ,&
&  0.25546E+00_JPRB, 0.25292E+00_JPRB, 0.25644E+00_JPRB, 0.26865E+00_JPRB, 0.27089E+00_JPRB, 0.27134E+00_JPRB, 0.26950E+00_JPRB ,&
&  0.26699E+00_JPRB, 0.26691E+00_JPRB, 0.26775E+00_JPRB, 0.26162E+00_JPRB, 0.31740E+00_JPRB, 0.40553E+00_JPRB, 0.40018E+00_JPRB ,&
&  0.41675E+00_JPRB, 0.42668E+00_JPRB, 0.45062E+00_JPRB, 0.45076E+00_JPRB, 0.43494E+00_JPRB, 0.44109E+00_JPRB, 0.47300E+00_JPRB ,&
&  0.48347E+00_JPRB, 0.46991E+00_JPRB, 0.45034E+00_JPRB, 0.44289E+00_JPRB, 0.42599E+00_JPRB, 0.40851E+00_JPRB, 0.39219E+00_JPRB ,&
&  0.38742E+00_JPRB, 0.37854E+00_JPRB, 0.36236E+00_JPRB, 0.34384E+00_JPRB, 0.33735E+00_JPRB, 0.33740E+00_JPRB, 0.34314E+00_JPRB ,&
&  0.34493E+00_JPRB, 0.34892E+00_JPRB, 0.35151E+00_JPRB, 0.35292E+00_JPRB, 0.32855E+00_JPRB, 0.30082E+00_JPRB, 0.28273E+00_JPRB ,&
&  0.26974E+00_JPRB, 0.26055E+00_JPRB, 0.25292E+00_JPRB, 0.25604E+00_JPRB, 0.26256E+00_JPRB, 0.26380E+00_JPRB, 0.25410E+00_JPRB ,&
&  0.24675E+00_JPRB, 0.24398E+00_JPRB, 0.24395E+00_JPRB, 0.25185E+00_JPRB, 0.26099E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.28656E+00_JPRB, 0.29543E+00_JPRB, 0.29757E+00_JPRB, 0.29798E+00_JPRB, 0.30114E+00_JPRB, 0.29751E+00_JPRB, 0.28727E+00_JPRB ,&
&  0.28911E+00_JPRB, 0.27245E+00_JPRB, 0.26537E+00_JPRB, 0.25766E+00_JPRB, 0.25259E+00_JPRB, 0.24654E+00_JPRB, 0.23756E+00_JPRB ,&
&  0.23495E+00_JPRB, 0.23742E+00_JPRB, 0.24177E+00_JPRB, 0.24154E+00_JPRB, 0.23823E+00_JPRB, 0.23415E+00_JPRB, 0.23162E+00_JPRB ,&
&  0.23076E+00_JPRB, 0.23152E+00_JPRB, 0.23314E+00_JPRB, 0.23158E+00_JPRB, 0.22861E+00_JPRB, 0.22614E+00_JPRB, 0.22589E+00_JPRB ,&
&  0.22348E+00_JPRB, 0.22044E+00_JPRB, 0.22188E+00_JPRB, 0.22285E+00_JPRB, 0.22713E+00_JPRB, 0.23092E+00_JPRB, 0.22497E+00_JPRB ,&
&  0.20801E+00_JPRB, 0.21286E+00_JPRB, 0.24235E+00_JPRB, 0.25936E+00_JPRB, 0.27503E+00_JPRB, 0.28199E+00_JPRB, 0.28696E+00_JPRB ,&
&  0.29114E+00_JPRB, 0.29611E+00_JPRB, 0.29868E+00_JPRB, 0.30094E+00_JPRB, 0.30090E+00_JPRB, 0.29904E+00_JPRB, 0.29557E+00_JPRB ,&
&  0.29407E+00_JPRB, 0.27107E+00_JPRB, 0.26527E+00_JPRB, 0.28494E+00_JPRB, 0.42704E+00_JPRB, 0.43641E+00_JPRB, 0.43322E+00_JPRB ,&
&  0.45038E+00_JPRB, 0.45344E+00_JPRB, 0.47370E+00_JPRB, 0.48430E+00_JPRB, 0.47578E+00_JPRB, 0.47196E+00_JPRB, 0.49684E+00_JPRB ,&
&  0.51786E+00_JPRB, 0.51417E+00_JPRB, 0.49399E+00_JPRB, 0.47078E+00_JPRB, 0.43995E+00_JPRB, 0.42614E+00_JPRB, 0.41760E+00_JPRB ,&
&  0.41841E+00_JPRB, 0.41541E+00_JPRB, 0.39824E+00_JPRB, 0.37750E+00_JPRB, 0.36314E+00_JPRB, 0.36413E+00_JPRB, 0.36686E+00_JPRB ,&
&  0.36765E+00_JPRB, 0.37167E+00_JPRB, 0.36891E+00_JPRB, 0.36301E+00_JPRB, 0.34699E+00_JPRB, 0.33053E+00_JPRB, 0.32028E+00_JPRB ,&
&  0.29577E+00_JPRB, 0.27107E+00_JPRB, 0.25446E+00_JPRB, 0.25420E+00_JPRB, 0.26061E+00_JPRB, 0.26193E+00_JPRB, 0.25644E+00_JPRB ,&
&  0.25609E+00_JPRB, 0.25532E+00_JPRB, 0.25574E+00_JPRB, 0.26459E+00_JPRB, 0.27376E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.29359E+00_JPRB, 0.30127E+00_JPRB, 0.30396E+00_JPRB, 0.30488E+00_JPRB, 0.31150E+00_JPRB, 0.30851E+00_JPRB, 0.30985E+00_JPRB ,&
&  0.32934E+00_JPRB, 0.31950E+00_JPRB, 0.33518E+00_JPRB, 0.28862E+00_JPRB, 0.26480E+00_JPRB, 0.25651E+00_JPRB, 0.24679E+00_JPRB ,&
&  0.24402E+00_JPRB, 0.24724E+00_JPRB, 0.24911E+00_JPRB, 0.24913E+00_JPRB, 0.24828E+00_JPRB, 0.24228E+00_JPRB, 0.23854E+00_JPRB ,&
&  0.23749E+00_JPRB, 0.23858E+00_JPRB, 0.24080E+00_JPRB, 0.23885E+00_JPRB, 0.23621E+00_JPRB, 0.23416E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.22918E+00_JPRB, 0.23058E+00_JPRB, 0.23349E+00_JPRB, 0.23264E+00_JPRB, 0.23567E+00_JPRB, 0.24275E+00_JPRB, 0.24574E+00_JPRB ,&
&  0.25557E+00_JPRB, 0.25645E+00_JPRB, 0.28251E+00_JPRB, 0.30127E+00_JPRB, 0.31210E+00_JPRB, 0.31539E+00_JPRB, 0.31796E+00_JPRB ,&
&  0.32381E+00_JPRB, 0.33190E+00_JPRB, 0.34069E+00_JPRB, 0.34767E+00_JPRB, 0.34854E+00_JPRB, 0.34798E+00_JPRB, 0.34417E+00_JPRB ,&
&  0.34253E+00_JPRB, 0.29892E+00_JPRB, 0.34154E+00_JPRB, 0.36485E+00_JPRB, 0.46760E+00_JPRB, 0.57192E+00_JPRB, 0.54609E+00_JPRB ,&
&  0.60280E+00_JPRB, 0.55265E+00_JPRB, 0.54758E+00_JPRB, 0.53112E+00_JPRB, 0.51841E+00_JPRB, 0.52215E+00_JPRB, 0.54308E+00_JPRB ,&
&  0.59182E+00_JPRB, 0.57429E+00_JPRB, 0.56350E+00_JPRB, 0.51457E+00_JPRB, 0.48341E+00_JPRB, 0.47820E+00_JPRB, 0.47054E+00_JPRB ,&
&  0.46948E+00_JPRB, 0.46822E+00_JPRB, 0.45930E+00_JPRB, 0.41728E+00_JPRB, 0.40740E+00_JPRB, 0.40412E+00_JPRB, 0.38431E+00_JPRB ,&
&  0.39006E+00_JPRB, 0.38607E+00_JPRB, 0.36941E+00_JPRB, 0.38038E+00_JPRB, 0.35614E+00_JPRB, 0.34053E+00_JPRB, 0.33624E+00_JPRB ,&
&  0.32231E+00_JPRB, 0.28974E+00_JPRB, 0.27769E+00_JPRB, 0.27539E+00_JPRB, 0.26995E+00_JPRB, 0.26611E+00_JPRB, 0.26778E+00_JPRB ,&
&  0.26280E+00_JPRB, 0.26542E+00_JPRB, 0.27543E+00_JPRB, 0.27916E+00_JPRB, 0.28473E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.30165E+00_JPRB, 0.30466E+00_JPRB, 0.30829E+00_JPRB, 0.31231E+00_JPRB, 0.32475E+00_JPRB, 0.33130E+00_JPRB, 0.33149E+00_JPRB ,&
&  0.34605E+00_JPRB, 0.34483E+00_JPRB, 0.34554E+00_JPRB, 0.34295E+00_JPRB, 0.33808E+00_JPRB, 0.30703E+00_JPRB, 0.26219E+00_JPRB ,&
&  0.24773E+00_JPRB, 0.24972E+00_JPRB, 0.25606E+00_JPRB, 0.25832E+00_JPRB, 0.25479E+00_JPRB, 0.24890E+00_JPRB, 0.24492E+00_JPRB ,&
&  0.24519E+00_JPRB, 0.24579E+00_JPRB, 0.24774E+00_JPRB, 0.24787E+00_JPRB, 0.24583E+00_JPRB, 0.24482E+00_JPRB, 0.24419E+00_JPRB ,&
&  0.24049E+00_JPRB, 0.24265E+00_JPRB, 0.24648E+00_JPRB, 0.24210E+00_JPRB, 0.25423E+00_JPRB, 0.26695E+00_JPRB, 0.27349E+00_JPRB ,&
&  0.29278E+00_JPRB, 0.31619E+00_JPRB, 0.33240E+00_JPRB, 0.34792E+00_JPRB, 0.35150E+00_JPRB, 0.35343E+00_JPRB, 0.36114E+00_JPRB ,&
&  0.37461E+00_JPRB, 0.38761E+00_JPRB, 0.39999E+00_JPRB, 0.41021E+00_JPRB, 0.41523E+00_JPRB, 0.41039E+00_JPRB, 0.42755E+00_JPRB ,&
&  0.43338E+00_JPRB, 0.46142E+00_JPRB, 0.51721E+00_JPRB, 0.50718E+00_JPRB, 0.59151E+00_JPRB, 0.64052E+00_JPRB, 0.60782E+00_JPRB ,&
&  0.61438E+00_JPRB, 0.56950E+00_JPRB, 0.61242E+00_JPRB, 0.63533E+00_JPRB, 0.65734E+00_JPRB, 0.65835E+00_JPRB, 0.67677E+00_JPRB ,&
&  0.73665E+00_JPRB, 0.79151E+00_JPRB, 0.67252E+00_JPRB, 0.61771E+00_JPRB, 0.57395E+00_JPRB, 0.56758E+00_JPRB, 0.53936E+00_JPRB ,&
&  0.53922E+00_JPRB, 0.54908E+00_JPRB, 0.53807E+00_JPRB, 0.52288E+00_JPRB, 0.49196E+00_JPRB, 0.46180E+00_JPRB, 0.43339E+00_JPRB ,&
&  0.41544E+00_JPRB, 0.37909E+00_JPRB, 0.36119E+00_JPRB, 0.36068E+00_JPRB, 0.35915E+00_JPRB, 0.33317E+00_JPRB, 0.33644E+00_JPRB ,&
&  0.33984E+00_JPRB, 0.31855E+00_JPRB, 0.30252E+00_JPRB, 0.28990E+00_JPRB, 0.28591E+00_JPRB, 0.27975E+00_JPRB, 0.26783E+00_JPRB ,&
&  0.26046E+00_JPRB, 0.27459E+00_JPRB, 0.28466E+00_JPRB, 0.29038E+00_JPRB, 0.29711E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.31400E+00_JPRB, 0.31404E+00_JPRB, 0.31432E+00_JPRB, 0.32140E+00_JPRB, 0.33530E+00_JPRB, 0.34104E+00_JPRB, 0.35039E+00_JPRB ,&
&  0.35638E+00_JPRB, 0.35652E+00_JPRB, 0.35980E+00_JPRB, 0.35440E+00_JPRB, 0.35141E+00_JPRB, 0.34874E+00_JPRB, 0.32340E+00_JPRB ,&
&  0.28812E+00_JPRB, 0.27908E+00_JPRB, 0.26979E+00_JPRB, 0.26870E+00_JPRB, 0.26813E+00_JPRB, 0.26135E+00_JPRB, 0.25680E+00_JPRB ,&
&  0.25861E+00_JPRB, 0.25649E+00_JPRB, 0.25540E+00_JPRB, 0.25494E+00_JPRB, 0.25473E+00_JPRB, 0.25641E+00_JPRB, 0.25746E+00_JPRB ,&
&  0.25385E+00_JPRB, 0.25883E+00_JPRB, 0.26564E+00_JPRB, 0.26378E+00_JPRB, 0.28189E+00_JPRB, 0.30593E+00_JPRB, 0.32495E+00_JPRB ,&
&  0.34974E+00_JPRB, 0.36998E+00_JPRB, 0.38232E+00_JPRB, 0.39202E+00_JPRB, 0.39443E+00_JPRB, 0.40538E+00_JPRB, 0.42350E+00_JPRB ,&
&  0.44562E+00_JPRB, 0.46206E+00_JPRB, 0.47620E+00_JPRB, 0.49261E+00_JPRB, 0.48958E+00_JPRB, 0.50608E+00_JPRB, 0.55764E+00_JPRB ,&
&  0.62046E+00_JPRB, 0.69728E+00_JPRB, 0.73129E+00_JPRB, 0.75515E+00_JPRB, 0.80880E+00_JPRB, 0.78545E+00_JPRB, 0.70189E+00_JPRB ,&
&  0.67902E+00_JPRB, 0.62514E+00_JPRB, 0.80782E+00_JPRB, 0.79703E+00_JPRB, 0.76593E+00_JPRB, 0.79263E+00_JPRB, 0.77140E+00_JPRB ,&
&  0.77283E+00_JPRB, 0.74556E+00_JPRB, 0.69357E+00_JPRB, 0.66846E+00_JPRB, 0.64858E+00_JPRB, 0.67840E+00_JPRB, 0.64085E+00_JPRB ,&
&  0.64455E+00_JPRB, 0.63741E+00_JPRB, 0.55166E+00_JPRB, 0.48629E+00_JPRB, 0.42437E+00_JPRB, 0.44103E+00_JPRB, 0.42676E+00_JPRB ,&
&  0.40888E+00_JPRB, 0.35944E+00_JPRB, 0.37227E+00_JPRB, 0.37414E+00_JPRB, 0.37848E+00_JPRB, 0.38558E+00_JPRB, 0.38006E+00_JPRB ,&
&  0.37773E+00_JPRB, 0.37636E+00_JPRB, 0.34017E+00_JPRB, 0.31699E+00_JPRB, 0.30697E+00_JPRB, 0.29471E+00_JPRB, 0.27706E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.29065E+00_JPRB, 0.30240E+00_JPRB, 0.30945E+00_JPRB, 0.31390E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.33109E+00_JPRB, 0.32963E+00_JPRB, 0.32750E+00_JPRB, 0.33631E+00_JPRB, 0.34688E+00_JPRB, 0.35105E+00_JPRB, 0.35974E+00_JPRB ,&
&  0.36875E+00_JPRB, 0.37064E+00_JPRB, 0.36777E+00_JPRB, 0.36326E+00_JPRB, 0.36109E+00_JPRB, 0.35979E+00_JPRB, 0.36198E+00_JPRB ,&
&  0.32231E+00_JPRB, 0.30273E+00_JPRB, 0.27961E+00_JPRB, 0.27926E+00_JPRB, 0.29847E+00_JPRB, 0.28565E+00_JPRB, 0.28273E+00_JPRB ,&
&  0.27733E+00_JPRB, 0.27615E+00_JPRB, 0.26977E+00_JPRB, 0.26539E+00_JPRB, 0.26391E+00_JPRB, 0.26680E+00_JPRB, 0.27355E+00_JPRB ,&
&  0.27580E+00_JPRB, 0.29112E+00_JPRB, 0.30781E+00_JPRB, 0.30949E+00_JPRB, 0.33281E+00_JPRB, 0.35746E+00_JPRB, 0.38843E+00_JPRB ,&
&  0.41524E+00_JPRB, 0.42369E+00_JPRB, 0.42843E+00_JPRB, 0.44107E+00_JPRB, 0.45595E+00_JPRB, 0.48268E+00_JPRB, 0.51318E+00_JPRB ,&
&  0.54529E+00_JPRB, 0.57063E+00_JPRB, 0.58811E+00_JPRB, 0.59898E+00_JPRB, 0.59217E+00_JPRB, 0.63278E+00_JPRB, 0.71636E+00_JPRB ,&
&  0.85393E+00_JPRB, 0.91059E+00_JPRB, 0.93868E+00_JPRB, 0.10733E+01_JPRB, 0.11294E+01_JPRB, 0.10528E+01_JPRB, 0.89141E+00_JPRB ,&
&  0.78945E+00_JPRB, 0.70657E+00_JPRB, 0.75961E+00_JPRB, 0.78425E+00_JPRB, 0.75103E+00_JPRB, 0.81349E+00_JPRB, 0.77719E+00_JPRB ,&
&  0.71057E+00_JPRB, 0.65762E+00_JPRB, 0.63497E+00_JPRB, 0.66682E+00_JPRB, 0.65884E+00_JPRB, 0.70008E+00_JPRB, 0.69278E+00_JPRB ,&
&  0.68888E+00_JPRB, 0.59214E+00_JPRB, 0.45148E+00_JPRB, 0.41778E+00_JPRB, 0.34344E+00_JPRB, 0.34997E+00_JPRB, 0.41719E+00_JPRB ,&
&  0.41889E+00_JPRB, 0.42108E+00_JPRB, 0.41377E+00_JPRB, 0.41265E+00_JPRB, 0.41412E+00_JPRB, 0.46835E+00_JPRB, 0.47836E+00_JPRB ,&
&  0.42639E+00_JPRB, 0.42348E+00_JPRB, 0.40389E+00_JPRB, 0.38057E+00_JPRB, 0.36063E+00_JPRB, 0.33458E+00_JPRB, 0.31569E+00_JPRB ,&
&  0.31483E+00_JPRB, 0.31739E+00_JPRB, 0.32441E+00_JPRB, 0.32740E+00_JPRB, 0.32896E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.35486E+00_JPRB, 0.35397E+00_JPRB, 0.34988E+00_JPRB, 0.35345E+00_JPRB, 0.35967E+00_JPRB, 0.36532E+00_JPRB, 0.37387E+00_JPRB ,&
&  0.38373E+00_JPRB, 0.38359E+00_JPRB, 0.37694E+00_JPRB, 0.37448E+00_JPRB, 0.37242E+00_JPRB, 0.36853E+00_JPRB, 0.36990E+00_JPRB ,&
&  0.37993E+00_JPRB, 0.36098E+00_JPRB, 0.32265E+00_JPRB, 0.29178E+00_JPRB, 0.30272E+00_JPRB, 0.30118E+00_JPRB, 0.31102E+00_JPRB ,&
&  0.30504E+00_JPRB, 0.31483E+00_JPRB, 0.30379E+00_JPRB, 0.29717E+00_JPRB, 0.29249E+00_JPRB, 0.29764E+00_JPRB, 0.31381E+00_JPRB ,&
&  0.33778E+00_JPRB, 0.37704E+00_JPRB, 0.41300E+00_JPRB, 0.41896E+00_JPRB, 0.44586E+00_JPRB, 0.43336E+00_JPRB, 0.46218E+00_JPRB ,&
&  0.48302E+00_JPRB, 0.48582E+00_JPRB, 0.49066E+00_JPRB, 0.51173E+00_JPRB, 0.54529E+00_JPRB, 0.58384E+00_JPRB, 0.63271E+00_JPRB ,&
&  0.67793E+00_JPRB, 0.69416E+00_JPRB, 0.67295E+00_JPRB, 0.66601E+00_JPRB, 0.70251E+00_JPRB, 0.77991E+00_JPRB, 0.79972E+00_JPRB ,&
&  0.88362E+00_JPRB, 0.92326E+00_JPRB, 0.90494E+00_JPRB, 0.10871E+01_JPRB, 0.12007E+01_JPRB, 0.11709E+01_JPRB, 0.10493E+01_JPRB ,&
&  0.94888E+00_JPRB, 0.86257E+00_JPRB, 0.89367E+00_JPRB, 0.88155E+00_JPRB, 0.85257E+00_JPRB, 0.83234E+00_JPRB, 0.77480E+00_JPRB ,&
&  0.68880E+00_JPRB, 0.64772E+00_JPRB, 0.66113E+00_JPRB, 0.67242E+00_JPRB, 0.68232E+00_JPRB, 0.65551E+00_JPRB, 0.66856E+00_JPRB ,&
&  0.69552E+00_JPRB, 0.41426E+00_JPRB, 0.34359E+00_JPRB, 0.38513E+00_JPRB, 0.33744E+00_JPRB, 0.34374E+00_JPRB, 0.38216E+00_JPRB ,&
&  0.37999E+00_JPRB, 0.38327E+00_JPRB, 0.42014E+00_JPRB, 0.44098E+00_JPRB, 0.48197E+00_JPRB, 0.55728E+00_JPRB, 0.55400E+00_JPRB ,&
&  0.52796E+00_JPRB, 0.49066E+00_JPRB, 0.49220E+00_JPRB, 0.47235E+00_JPRB, 0.44134E+00_JPRB, 0.39796E+00_JPRB, 0.36645E+00_JPRB ,&
&  0.35341E+00_JPRB, 0.35221E+00_JPRB, 0.35268E+00_JPRB, 0.35191E+00_JPRB, 0.35073E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.39312E+00_JPRB, 0.39128E+00_JPRB, 0.37907E+00_JPRB, 0.37490E+00_JPRB, 0.38268E+00_JPRB, 0.39334E+00_JPRB, 0.40069E+00_JPRB ,&
&  0.40218E+00_JPRB, 0.39945E+00_JPRB, 0.39436E+00_JPRB, 0.39271E+00_JPRB, 0.38706E+00_JPRB, 0.37590E+00_JPRB, 0.37073E+00_JPRB ,&
&  0.37005E+00_JPRB, 0.37302E+00_JPRB, 0.33496E+00_JPRB, 0.32020E+00_JPRB, 0.30140E+00_JPRB, 0.31060E+00_JPRB, 0.32412E+00_JPRB ,&
&  0.34097E+00_JPRB, 0.33729E+00_JPRB, 0.32955E+00_JPRB, 0.33030E+00_JPRB, 0.33176E+00_JPRB, 0.34631E+00_JPRB, 0.40114E+00_JPRB ,&
&  0.46654E+00_JPRB, 0.53663E+00_JPRB, 0.54546E+00_JPRB, 0.56978E+00_JPRB, 0.55074E+00_JPRB, 0.51694E+00_JPRB, 0.55821E+00_JPRB ,&
&  0.56840E+00_JPRB, 0.56677E+00_JPRB, 0.57043E+00_JPRB, 0.59716E+00_JPRB, 0.63397E+00_JPRB, 0.65171E+00_JPRB, 0.68141E+00_JPRB ,&
&  0.69625E+00_JPRB, 0.69858E+00_JPRB, 0.68858E+00_JPRB, 0.66922E+00_JPRB, 0.72076E+00_JPRB, 0.75230E+00_JPRB, 0.70998E+00_JPRB ,&
&  0.75210E+00_JPRB, 0.68301E+00_JPRB, 0.56860E+00_JPRB, 0.80061E+00_JPRB, 0.11882E+01_JPRB, 0.11972E+01_JPRB, 0.96926E+00_JPRB ,&
&  0.97480E+00_JPRB, 0.95362E+00_JPRB, 0.10189E+01_JPRB, 0.10531E+01_JPRB, 0.99233E+00_JPRB, 0.88852E+00_JPRB, 0.80395E+00_JPRB ,&
&  0.72852E+00_JPRB, 0.70975E+00_JPRB, 0.74055E+00_JPRB, 0.86749E+00_JPRB, 0.87840E+00_JPRB, 0.85405E+00_JPRB, 0.80178E+00_JPRB ,&
&  0.65998E+00_JPRB, 0.55258E+00_JPRB, 0.40593E+00_JPRB, 0.41335E+00_JPRB, 0.34060E+00_JPRB, 0.34237E+00_JPRB, 0.37748E+00_JPRB ,&
&  0.37247E+00_JPRB, 0.39663E+00_JPRB, 0.43685E+00_JPRB, 0.46807E+00_JPRB, 0.60052E+00_JPRB, 0.67479E+00_JPRB, 0.69482E+00_JPRB ,&
&  0.66535E+00_JPRB, 0.58742E+00_JPRB, 0.59149E+00_JPRB, 0.55154E+00_JPRB, 0.52556E+00_JPRB, 0.47538E+00_JPRB, 0.43377E+00_JPRB ,&
&  0.40964E+00_JPRB, 0.40265E+00_JPRB, 0.39660E+00_JPRB, 0.39234E+00_JPRB, 0.38929E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.41155E+00_JPRB, 0.39937E+00_JPRB, 0.37601E+00_JPRB, 0.37476E+00_JPRB, 0.38535E+00_JPRB, 0.40420E+00_JPRB, 0.41854E+00_JPRB ,&
&  0.42297E+00_JPRB, 0.42028E+00_JPRB, 0.41182E+00_JPRB, 0.40667E+00_JPRB, 0.39543E+00_JPRB, 0.37796E+00_JPRB, 0.37167E+00_JPRB ,&
&  0.36585E+00_JPRB, 0.35580E+00_JPRB, 0.32645E+00_JPRB, 0.31063E+00_JPRB, 0.29800E+00_JPRB, 0.30344E+00_JPRB, 0.31962E+00_JPRB ,&
&  0.33494E+00_JPRB, 0.34694E+00_JPRB, 0.36324E+00_JPRB, 0.37555E+00_JPRB, 0.38190E+00_JPRB, 0.42916E+00_JPRB, 0.53200E+00_JPRB ,&
&  0.71257E+00_JPRB, 0.66984E+00_JPRB, 0.68435E+00_JPRB, 0.64378E+00_JPRB, 0.62361E+00_JPRB, 0.61661E+00_JPRB, 0.60454E+00_JPRB ,&
&  0.57659E+00_JPRB, 0.56189E+00_JPRB, 0.57247E+00_JPRB, 0.59394E+00_JPRB, 0.64183E+00_JPRB, 0.67862E+00_JPRB, 0.74065E+00_JPRB ,&
&  0.76136E+00_JPRB, 0.73485E+00_JPRB, 0.68548E+00_JPRB, 0.68571E+00_JPRB, 0.74436E+00_JPRB, 0.74703E+00_JPRB, 0.72487E+00_JPRB ,&
&  0.62953E+00_JPRB, 0.56792E+00_JPRB, 0.66677E+00_JPRB, 0.79645E+00_JPRB, 0.10303E+01_JPRB, 0.11771E+01_JPRB, 0.10823E+01_JPRB ,&
&  0.10681E+01_JPRB, 0.10395E+01_JPRB, 0.10405E+01_JPRB, 0.82665E+00_JPRB, 0.88856E+00_JPRB, 0.84001E+00_JPRB, 0.79276E+00_JPRB ,&
&  0.76030E+00_JPRB, 0.78009E+00_JPRB, 0.86257E+00_JPRB, 0.10152E+01_JPRB, 0.11174E+01_JPRB, 0.11383E+01_JPRB, 0.83296E+00_JPRB ,&
&  0.62518E+00_JPRB, 0.65550E+00_JPRB, 0.59045E+00_JPRB, 0.49850E+00_JPRB, 0.42786E+00_JPRB, 0.43429E+00_JPRB, 0.43179E+00_JPRB ,&
&  0.43266E+00_JPRB, 0.44015E+00_JPRB, 0.48987E+00_JPRB, 0.58845E+00_JPRB, 0.76908E+00_JPRB, 0.88114E+00_JPRB, 0.85671E+00_JPRB ,&
&  0.75781E+00_JPRB, 0.75863E+00_JPRB, 0.70098E+00_JPRB, 0.62426E+00_JPRB, 0.60083E+00_JPRB, 0.55772E+00_JPRB, 0.51252E+00_JPRB ,&
&  0.47840E+00_JPRB, 0.46552E+00_JPRB, 0.45509E+00_JPRB, 0.44237E+00_JPRB, 0.42322E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.43271E+00_JPRB, 0.41911E+00_JPRB, 0.39777E+00_JPRB, 0.39739E+00_JPRB, 0.41306E+00_JPRB, 0.43139E+00_JPRB, 0.43997E+00_JPRB ,&
&  0.43521E+00_JPRB, 0.42853E+00_JPRB, 0.41825E+00_JPRB, 0.41805E+00_JPRB, 0.41179E+00_JPRB, 0.38952E+00_JPRB, 0.37648E+00_JPRB ,&
&  0.37439E+00_JPRB, 0.34864E+00_JPRB, 0.32066E+00_JPRB, 0.32277E+00_JPRB, 0.32214E+00_JPRB, 0.30534E+00_JPRB, 0.31516E+00_JPRB ,&
&  0.33231E+00_JPRB, 0.35458E+00_JPRB, 0.37795E+00_JPRB, 0.41604E+00_JPRB, 0.47942E+00_JPRB, 0.58671E+00_JPRB, 0.72367E+00_JPRB ,&
&  0.75415E+00_JPRB, 0.75621E+00_JPRB, 0.77296E+00_JPRB, 0.72848E+00_JPRB, 0.65035E+00_JPRB, 0.60953E+00_JPRB, 0.59403E+00_JPRB ,&
&  0.57114E+00_JPRB, 0.56077E+00_JPRB, 0.59089E+00_JPRB, 0.64615E+00_JPRB, 0.71966E+00_JPRB, 0.76093E+00_JPRB, 0.79776E+00_JPRB ,&
&  0.75550E+00_JPRB, 0.74592E+00_JPRB, 0.73876E+00_JPRB, 0.81263E+00_JPRB, 0.62428E+00_JPRB, 0.60632E+00_JPRB, 0.59236E+00_JPRB ,&
&  0.69595E+00_JPRB, 0.67587E+00_JPRB, 0.67171E+00_JPRB, 0.81345E+00_JPRB, 0.77283E+00_JPRB, 0.89210E+00_JPRB, 0.10574E+01_JPRB ,&
&  0.11447E+01_JPRB, 0.11454E+01_JPRB, 0.11220E+01_JPRB, 0.70066E+00_JPRB, 0.62601E+00_JPRB, 0.79976E+00_JPRB, 0.85378E+00_JPRB ,&
&  0.86345E+00_JPRB, 0.95803E+00_JPRB, 0.10867E+01_JPRB, 0.12932E+01_JPRB, 0.10198E+01_JPRB, 0.83517E+00_JPRB, 0.62843E+00_JPRB ,&
&  0.60581E+00_JPRB, 0.56175E+00_JPRB, 0.64400E+00_JPRB, 0.59063E+00_JPRB, 0.58306E+00_JPRB, 0.55963E+00_JPRB, 0.51943E+00_JPRB ,&
&  0.52122E+00_JPRB, 0.53826E+00_JPRB, 0.62561E+00_JPRB, 0.82562E+00_JPRB, 0.10428E+01_JPRB, 0.10285E+01_JPRB, 0.10639E+01_JPRB ,&
&  0.10011E+01_JPRB, 0.88660E+00_JPRB, 0.76488E+00_JPRB, 0.70098E+00_JPRB, 0.66286E+00_JPRB, 0.63229E+00_JPRB, 0.59738E+00_JPRB ,&
&  0.55598E+00_JPRB, 0.52290E+00_JPRB, 0.49737E+00_JPRB, 0.47298E+00_JPRB, 0.44850E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.46583E+00_JPRB, 0.45124E+00_JPRB, 0.42747E+00_JPRB, 0.43055E+00_JPRB, 0.45303E+00_JPRB, 0.46200E+00_JPRB, 0.45578E+00_JPRB ,&
&  0.45126E+00_JPRB, 0.43739E+00_JPRB, 0.42905E+00_JPRB, 0.42034E+00_JPRB, 0.41094E+00_JPRB, 0.40941E+00_JPRB, 0.39518E+00_JPRB ,&
&  0.39451E+00_JPRB, 0.38076E+00_JPRB, 0.34463E+00_JPRB, 0.32624E+00_JPRB, 0.33616E+00_JPRB, 0.31880E+00_JPRB, 0.31050E+00_JPRB ,&
&  0.34134E+00_JPRB, 0.37770E+00_JPRB, 0.42410E+00_JPRB, 0.49876E+00_JPRB, 0.60957E+00_JPRB, 0.80506E+00_JPRB, 0.91026E+00_JPRB ,&
&  0.86102E+00_JPRB, 0.81302E+00_JPRB, 0.71530E+00_JPRB, 0.63208E+00_JPRB, 0.57119E+00_JPRB, 0.54599E+00_JPRB, 0.53669E+00_JPRB ,&
&  0.53566E+00_JPRB, 0.55073E+00_JPRB, 0.60256E+00_JPRB, 0.67304E+00_JPRB, 0.73587E+00_JPRB, 0.74785E+00_JPRB, 0.74406E+00_JPRB ,&
&  0.77083E+00_JPRB, 0.83699E+00_JPRB, 0.85599E+00_JPRB, 0.79627E+00_JPRB, 0.72131E+00_JPRB, 0.62489E+00_JPRB, 0.64043E+00_JPRB ,&
&  0.69065E+00_JPRB, 0.63976E+00_JPRB, 0.67200E+00_JPRB, 0.83534E+00_JPRB, 0.91220E+00_JPRB, 0.92649E+00_JPRB, 0.11049E+01_JPRB ,&
&  0.90309E+00_JPRB, 0.72530E+00_JPRB, 0.62028E+00_JPRB, 0.52806E+00_JPRB, 0.73476E+00_JPRB, 0.99940E+00_JPRB, 0.10978E+01_JPRB ,&
&  0.11448E+01_JPRB, 0.12438E+01_JPRB, 0.13770E+01_JPRB, 0.12726E+01_JPRB, 0.95465E+00_JPRB, 0.74892E+00_JPRB, 0.88365E+00_JPRB ,&
&  0.91242E+00_JPRB, 0.92277E+00_JPRB, 0.87640E+00_JPRB, 0.75411E+00_JPRB, 0.62037E+00_JPRB, 0.69144E+00_JPRB, 0.68131E+00_JPRB ,&
&  0.68038E+00_JPRB, 0.69522E+00_JPRB, 0.87543E+00_JPRB, 0.11361E+01_JPRB, 0.12717E+01_JPRB, 0.14031E+01_JPRB, 0.13957E+01_JPRB ,&
&  0.11674E+01_JPRB, 0.10785E+01_JPRB, 0.91935E+00_JPRB, 0.79542E+00_JPRB, 0.73123E+00_JPRB, 0.68942E+00_JPRB, 0.65720E+00_JPRB ,&
&  0.61457E+00_JPRB, 0.58198E+00_JPRB, 0.55042E+00_JPRB, 0.51264E+00_JPRB, 0.48292E+00_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.51060E+00_JPRB, 0.48910E+00_JPRB, 0.46060E+00_JPRB, 0.46493E+00_JPRB, 0.49328E+00_JPRB, 0.48667E+00_JPRB, 0.47160E+00_JPRB ,&
&  0.47054E+00_JPRB, 0.45366E+00_JPRB, 0.44128E+00_JPRB, 0.42557E+00_JPRB, 0.41170E+00_JPRB, 0.40554E+00_JPRB, 0.40698E+00_JPRB ,&
&  0.42253E+00_JPRB, 0.44296E+00_JPRB, 0.39734E+00_JPRB, 0.36553E+00_JPRB, 0.35472E+00_JPRB, 0.34212E+00_JPRB, 0.31628E+00_JPRB ,&
&  0.36134E+00_JPRB, 0.42327E+00_JPRB, 0.49382E+00_JPRB, 0.60765E+00_JPRB, 0.79212E+00_JPRB, 0.86017E+00_JPRB, 0.86034E+00_JPRB ,&
&  0.82342E+00_JPRB, 0.74503E+00_JPRB, 0.65618E+00_JPRB, 0.58250E+00_JPRB, 0.54485E+00_JPRB, 0.53413E+00_JPRB, 0.53055E+00_JPRB ,&
&  0.54475E+00_JPRB, 0.59558E+00_JPRB, 0.65327E+00_JPRB, 0.71211E+00_JPRB, 0.76278E+00_JPRB, 0.79884E+00_JPRB, 0.82196E+00_JPRB ,&
&  0.86950E+00_JPRB, 0.92459E+00_JPRB, 0.94224E+00_JPRB, 0.89515E+00_JPRB, 0.86032E+00_JPRB, 0.75786E+00_JPRB, 0.89266E+00_JPRB ,&
&  0.83880E+00_JPRB, 0.85945E+00_JPRB, 0.84339E+00_JPRB, 0.11207E+01_JPRB, 0.99634E+00_JPRB, 0.94645E+00_JPRB, 0.10315E+01_JPRB ,&
&  0.80523E+00_JPRB, 0.74881E+00_JPRB, 0.84375E+00_JPRB, 0.95301E+00_JPRB, 0.10281E+01_JPRB, 0.11166E+01_JPRB, 0.15531E+01_JPRB ,&
&  0.13141E+01_JPRB, 0.14833E+01_JPRB, 0.15974E+01_JPRB, 0.15355E+01_JPRB, 0.73006E+00_JPRB, 0.55286E+00_JPRB, 0.82121E+00_JPRB ,&
&  0.88699E+00_JPRB, 0.62160E+00_JPRB, 0.47961E+00_JPRB, 0.58156E+00_JPRB, 0.46312E+00_JPRB, 0.53615E+00_JPRB, 0.72942E+00_JPRB ,&
&  0.81115E+00_JPRB, 0.93060E+00_JPRB, 0.12618E+01_JPRB, 0.13790E+01_JPRB, 0.13355E+01_JPRB, 0.16918E+01_JPRB, 0.14955E+01_JPRB ,&
&  0.13235E+01_JPRB, 0.11577E+01_JPRB, 0.10541E+01_JPRB, 0.91557E+00_JPRB, 0.82235E+00_JPRB, 0.77056E+00_JPRB, 0.72906E+00_JPRB ,&
&  0.68274E+00_JPRB, 0.64216E+00_JPRB, 0.60514E+00_JPRB, 0.56708E+00_JPRB, 0.53573E+00_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.54482E+00_JPRB, 0.52590E+00_JPRB, 0.50357E+00_JPRB, 0.50745E+00_JPRB, 0.52873E+00_JPRB, 0.50312E+00_JPRB, 0.49379E+00_JPRB ,&
&  0.47729E+00_JPRB, 0.46466E+00_JPRB, 0.44561E+00_JPRB, 0.42520E+00_JPRB, 0.40929E+00_JPRB, 0.41056E+00_JPRB, 0.41894E+00_JPRB ,&
&  0.43106E+00_JPRB, 0.46698E+00_JPRB, 0.49423E+00_JPRB, 0.46023E+00_JPRB, 0.43197E+00_JPRB, 0.39400E+00_JPRB, 0.37036E+00_JPRB ,&
&  0.41711E+00_JPRB, 0.50555E+00_JPRB, 0.58535E+00_JPRB, 0.66163E+00_JPRB, 0.74280E+00_JPRB, 0.74430E+00_JPRB, 0.76839E+00_JPRB ,&
&  0.75050E+00_JPRB, 0.67768E+00_JPRB, 0.61497E+00_JPRB, 0.56643E+00_JPRB, 0.54628E+00_JPRB, 0.54080E+00_JPRB, 0.56284E+00_JPRB ,&
&  0.60129E+00_JPRB, 0.64322E+00_JPRB, 0.67599E+00_JPRB, 0.74188E+00_JPRB, 0.82550E+00_JPRB, 0.87135E+00_JPRB, 0.91270E+00_JPRB ,&
&  0.94801E+00_JPRB, 0.96396E+00_JPRB, 0.96699E+00_JPRB, 0.97251E+00_JPRB, 0.10329E+01_JPRB, 0.94103E+00_JPRB, 0.81137E+00_JPRB ,&
&  0.74758E+00_JPRB, 0.85355E+00_JPRB, 0.12359E+01_JPRB, 0.12959E+01_JPRB, 0.10325E+01_JPRB, 0.10256E+01_JPRB, 0.98711E+00_JPRB ,&
&  0.96205E+00_JPRB, 0.10958E+01_JPRB, 0.11984E+01_JPRB, 0.13035E+01_JPRB, 0.13584E+01_JPRB, 0.11325E+01_JPRB, 0.15344E+01_JPRB ,&
&  0.16135E+01_JPRB, 0.15706E+01_JPRB, 0.11836E+01_JPRB, 0.83229E+00_JPRB, 0.82693E+00_JPRB, 0.62285E+00_JPRB, 0.45359E+00_JPRB ,&
&  0.42796E+00_JPRB, 0.40942E+00_JPRB, 0.37877E+00_JPRB, 0.39358E+00_JPRB, 0.39588E+00_JPRB, 0.50742E+00_JPRB, 0.73665E+00_JPRB ,&
&  0.10311E+01_JPRB, 0.12732E+01_JPRB, 0.14401E+01_JPRB, 0.14831E+01_JPRB, 0.14176E+01_JPRB, 0.16691E+01_JPRB, 0.16332E+01_JPRB ,&
&  0.12060E+01_JPRB, 0.11125E+01_JPRB, 0.98159E+00_JPRB, 0.93280E+00_JPRB, 0.88752E+00_JPRB, 0.84466E+00_JPRB, 0.79040E+00_JPRB ,&
&  0.73346E+00_JPRB, 0.69160E+00_JPRB, 0.65515E+00_JPRB, 0.60970E+00_JPRB, 0.57248E+00_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.57663E+00_JPRB, 0.56179E+00_JPRB, 0.56289E+00_JPRB, 0.56011E+00_JPRB, 0.56001E+00_JPRB, 0.52356E+00_JPRB, 0.51456E+00_JPRB ,&
&  0.48147E+00_JPRB, 0.46574E+00_JPRB, 0.43670E+00_JPRB, 0.41985E+00_JPRB, 0.40980E+00_JPRB, 0.41926E+00_JPRB, 0.43867E+00_JPRB ,&
&  0.45857E+00_JPRB, 0.51422E+00_JPRB, 0.60136E+00_JPRB, 0.62227E+00_JPRB, 0.62276E+00_JPRB, 0.56447E+00_JPRB, 0.52696E+00_JPRB ,&
&  0.53362E+00_JPRB, 0.64299E+00_JPRB, 0.68314E+00_JPRB, 0.70798E+00_JPRB, 0.74145E+00_JPRB, 0.73513E+00_JPRB, 0.71245E+00_JPRB ,&
&  0.69507E+00_JPRB, 0.64338E+00_JPRB, 0.58633E+00_JPRB, 0.55914E+00_JPRB, 0.55290E+00_JPRB, 0.56651E+00_JPRB, 0.59758E+00_JPRB ,&
&  0.65380E+00_JPRB, 0.67234E+00_JPRB, 0.70761E+00_JPRB, 0.79897E+00_JPRB, 0.87871E+00_JPRB, 0.91416E+00_JPRB, 0.96520E+00_JPRB ,&
&  0.10088E+01_JPRB, 0.10092E+01_JPRB, 0.99899E+00_JPRB, 0.97386E+00_JPRB, 0.83940E+00_JPRB, 0.71739E+00_JPRB, 0.86100E+00_JPRB ,&
&  0.99153E+00_JPRB, 0.12377E+01_JPRB, 0.14144E+01_JPRB, 0.15938E+01_JPRB, 0.11737E+01_JPRB, 0.10626E+01_JPRB, 0.10477E+01_JPRB ,&
&  0.11277E+01_JPRB, 0.16825E+01_JPRB, 0.14997E+01_JPRB, 0.13401E+01_JPRB, 0.15569E+01_JPRB, 0.12926E+01_JPRB, 0.13628E+01_JPRB ,&
&  0.16260E+01_JPRB, 0.15232E+01_JPRB, 0.11786E+01_JPRB, 0.92487E+00_JPRB, 0.11039E+01_JPRB, 0.10277E+01_JPRB, 0.56612E+00_JPRB ,&
&  0.47004E+00_JPRB, 0.43964E+00_JPRB, 0.41459E+00_JPRB, 0.39632E+00_JPRB, 0.40265E+00_JPRB, 0.54121E+00_JPRB, 0.92970E+00_JPRB ,&
&  0.11949E+01_JPRB, 0.15348E+01_JPRB, 0.15938E+01_JPRB, 0.15642E+01_JPRB, 0.14522E+01_JPRB, 0.15941E+01_JPRB, 0.16075E+01_JPRB ,&
&  0.12964E+01_JPRB, 0.11610E+01_JPRB, 0.10345E+01_JPRB, 0.97273E+00_JPRB, 0.92518E+00_JPRB, 0.88227E+00_JPRB, 0.83271E+00_JPRB ,&
&  0.77756E+00_JPRB, 0.73121E+00_JPRB, 0.68831E+00_JPRB, 0.63942E+00_JPRB, 0.60051E+00_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.60295E+00_JPRB, 0.59669E+00_JPRB, 0.61101E+00_JPRB, 0.61666E+00_JPRB, 0.59910E+00_JPRB, 0.56825E+00_JPRB, 0.52936E+00_JPRB ,&
&  0.49247E+00_JPRB, 0.46146E+00_JPRB, 0.43799E+00_JPRB, 0.42644E+00_JPRB, 0.42395E+00_JPRB, 0.43764E+00_JPRB, 0.45769E+00_JPRB ,&
&  0.50042E+00_JPRB, 0.57637E+00_JPRB, 0.66714E+00_JPRB, 0.80916E+00_JPRB, 0.80586E+00_JPRB, 0.72432E+00_JPRB, 0.75464E+00_JPRB ,&
&  0.76699E+00_JPRB, 0.83989E+00_JPRB, 0.84930E+00_JPRB, 0.80478E+00_JPRB, 0.76836E+00_JPRB, 0.75217E+00_JPRB, 0.72412E+00_JPRB ,&
&  0.66574E+00_JPRB, 0.63273E+00_JPRB, 0.58692E+00_JPRB, 0.58341E+00_JPRB, 0.59156E+00_JPRB, 0.61247E+00_JPRB, 0.65474E+00_JPRB ,&
&  0.67113E+00_JPRB, 0.66874E+00_JPRB, 0.73930E+00_JPRB, 0.83420E+00_JPRB, 0.90004E+00_JPRB, 0.95916E+00_JPRB, 0.10262E+01_JPRB ,&
&  0.10467E+01_JPRB, 0.10451E+01_JPRB, 0.10232E+01_JPRB, 0.99161E+00_JPRB, 0.82051E+00_JPRB, 0.93759E+00_JPRB, 0.98110E+00_JPRB ,&
&  0.10939E+01_JPRB, 0.12438E+01_JPRB, 0.13128E+01_JPRB, 0.14647E+01_JPRB, 0.12325E+01_JPRB, 0.10986E+01_JPRB, 0.10509E+01_JPRB ,&
&  0.13134E+01_JPRB, 0.14000E+01_JPRB, 0.12479E+01_JPRB, 0.12966E+01_JPRB, 0.15236E+01_JPRB, 0.18526E+01_JPRB, 0.12437E+01_JPRB ,&
&  0.11176E+01_JPRB, 0.13963E+01_JPRB, 0.11811E+01_JPRB, 0.10352E+01_JPRB, 0.11768E+01_JPRB, 0.12657E+01_JPRB, 0.97958E+00_JPRB ,&
&  0.56559E+00_JPRB, 0.47607E+00_JPRB, 0.48490E+00_JPRB, 0.50822E+00_JPRB, 0.50076E+00_JPRB, 0.67322E+00_JPRB, 0.15463E+01_JPRB ,&
&  0.16270E+01_JPRB, 0.18110E+01_JPRB, 0.17223E+01_JPRB, 0.15591E+01_JPRB, 0.14685E+01_JPRB, 0.15172E+01_JPRB, 0.14984E+01_JPRB ,&
&  0.12724E+01_JPRB, 0.11494E+01_JPRB, 0.10472E+01_JPRB, 0.97981E+00_JPRB, 0.92251E+00_JPRB, 0.86914E+00_JPRB, 0.81831E+00_JPRB ,&
&  0.77822E+00_JPRB, 0.73501E+00_JPRB, 0.69572E+00_JPRB, 0.65382E+00_JPRB, 0.61953E+00_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.61860E+00_JPRB, 0.61954E+00_JPRB, 0.63436E+00_JPRB, 0.64144E+00_JPRB, 0.64840E+00_JPRB, 0.64576E+00_JPRB, 0.58881E+00_JPRB ,&
&  0.51885E+00_JPRB, 0.47000E+00_JPRB, 0.44793E+00_JPRB, 0.44219E+00_JPRB, 0.44837E+00_JPRB, 0.46672E+00_JPRB, 0.48785E+00_JPRB ,&
&  0.55954E+00_JPRB, 0.63195E+00_JPRB, 0.75577E+00_JPRB, 0.91317E+00_JPRB, 0.97714E+00_JPRB, 0.87287E+00_JPRB, 0.88251E+00_JPRB ,&
&  0.10145E+01_JPRB, 0.10315E+01_JPRB, 0.96371E+00_JPRB, 0.92649E+00_JPRB, 0.89406E+00_JPRB, 0.85855E+00_JPRB, 0.78305E+00_JPRB ,&
&  0.70981E+00_JPRB, 0.70457E+00_JPRB, 0.65709E+00_JPRB, 0.65071E+00_JPRB, 0.65227E+00_JPRB, 0.68752E+00_JPRB, 0.68007E+00_JPRB ,&
&  0.66105E+00_JPRB, 0.69993E+00_JPRB, 0.77773E+00_JPRB, 0.85158E+00_JPRB, 0.92358E+00_JPRB, 0.99048E+00_JPRB, 0.10685E+01_JPRB ,&
&  0.11054E+01_JPRB, 0.10913E+01_JPRB, 0.10498E+01_JPRB, 0.97271E+00_JPRB, 0.96462E+00_JPRB, 0.98627E+00_JPRB, 0.10648E+01_JPRB ,&
&  0.11109E+01_JPRB, 0.12052E+01_JPRB, 0.12562E+01_JPRB, 0.12757E+01_JPRB, 0.11639E+01_JPRB, 0.10959E+01_JPRB, 0.10658E+01_JPRB ,&
&  0.12378E+01_JPRB, 0.12601E+01_JPRB, 0.11152E+01_JPRB, 0.13094E+01_JPRB, 0.15914E+01_JPRB, 0.18087E+01_JPRB, 0.16751E+01_JPRB ,&
&  0.14358E+01_JPRB, 0.12087E+01_JPRB, 0.11108E+01_JPRB, 0.11876E+01_JPRB, 0.12610E+01_JPRB, 0.12776E+01_JPRB, 0.14559E+01_JPRB ,&
&  0.14464E+01_JPRB, 0.98871E+00_JPRB, 0.87009E+00_JPRB, 0.10500E+01_JPRB, 0.92240E+00_JPRB, 0.95522E+00_JPRB, 0.13097E+01_JPRB ,&
&  0.16292E+01_JPRB, 0.18302E+01_JPRB, 0.17445E+01_JPRB, 0.15609E+01_JPRB, 0.15314E+01_JPRB, 0.14963E+01_JPRB, 0.14506E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.10939E+01_JPRB, 0.99871E+00_JPRB, 0.93371E+00_JPRB, 0.87228E+00_JPRB, 0.81942E+00_JPRB, 0.78550E+00_JPRB ,&
&  0.75451E+00_JPRB, 0.71917E+00_JPRB, 0.68743E+00_JPRB, 0.65386E+00_JPRB, 0.62672E+00_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.63069E+00_JPRB, 0.63878E+00_JPRB, 0.65184E+00_JPRB, 0.67083E+00_JPRB, 0.72848E+00_JPRB, 0.74387E+00_JPRB, 0.68500E+00_JPRB ,&
&  0.61347E+00_JPRB, 0.49937E+00_JPRB, 0.47748E+00_JPRB, 0.47782E+00_JPRB, 0.48902E+00_JPRB, 0.51107E+00_JPRB, 0.56032E+00_JPRB ,&
&  0.63297E+00_JPRB, 0.70863E+00_JPRB, 0.86678E+00_JPRB, 0.10425E+01_JPRB, 0.12191E+01_JPRB, 0.13699E+01_JPRB, 0.95841E+00_JPRB ,&
&  0.92260E+00_JPRB, 0.10320E+01_JPRB, 0.10057E+01_JPRB, 0.10134E+01_JPRB, 0.99407E+00_JPRB, 0.97192E+00_JPRB, 0.85933E+00_JPRB ,&
&  0.78853E+00_JPRB, 0.78333E+00_JPRB, 0.73730E+00_JPRB, 0.70328E+00_JPRB, 0.71920E+00_JPRB, 0.72320E+00_JPRB, 0.67734E+00_JPRB ,&
&  0.68932E+00_JPRB, 0.73596E+00_JPRB, 0.80340E+00_JPRB, 0.87211E+00_JPRB, 0.94603E+00_JPRB, 0.10264E+01_JPRB, 0.10792E+01_JPRB ,&
&  0.11348E+01_JPRB, 0.11378E+01_JPRB, 0.10975E+01_JPRB, 0.10053E+01_JPRB, 0.10095E+01_JPRB, 0.10222E+01_JPRB, 0.10391E+01_JPRB ,&
&  0.98706E+00_JPRB, 0.95613E+00_JPRB, 0.10880E+01_JPRB, 0.11549E+01_JPRB, 0.10973E+01_JPRB, 0.10117E+01_JPRB, 0.99811E+00_JPRB ,&
&  0.11884E+01_JPRB, 0.13209E+01_JPRB, 0.14338E+01_JPRB, 0.14892E+01_JPRB, 0.16819E+01_JPRB, 0.18176E+01_JPRB, 0.18619E+01_JPRB ,&
&  0.18512E+01_JPRB, 0.15963E+01_JPRB, 0.14355E+01_JPRB, 0.14032E+01_JPRB, 0.13508E+01_JPRB, 0.13397E+01_JPRB, 0.14341E+01_JPRB ,&
&  0.15837E+01_JPRB, 0.17956E+01_JPRB, 0.18350E+01_JPRB, 0.15679E+01_JPRB, 0.14268E+01_JPRB, 0.12609E+01_JPRB, 0.13537E+01_JPRB ,&
&  0.15911E+01_JPRB, 0.16345E+01_JPRB, 0.16067E+01_JPRB, 0.17070E+01_JPRB, 0.14512E+01_JPRB, 0.13592E+01_JPRB, 0.12706E+01_JPRB ,&
&  0.11537E+01_JPRB, 0.10379E+01_JPRB, 0.96258E+00_JPRB, 0.90313E+00_JPRB, 0.83063E+00_JPRB, 0.77853E+00_JPRB, 0.75240E+00_JPRB ,&
&  0.72344E+00_JPRB, 0.70252E+00_JPRB, 0.67690E+00_JPRB, 0.65190E+00_JPRB, 0.63568E+00_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.64404E+00_JPRB, 0.64974E+00_JPRB, 0.66463E+00_JPRB, 0.68975E+00_JPRB, 0.79018E+00_JPRB, 0.82785E+00_JPRB, 0.82466E+00_JPRB ,&
&  0.80166E+00_JPRB, 0.55336E+00_JPRB, 0.51754E+00_JPRB, 0.51869E+00_JPRB, 0.53886E+00_JPRB, 0.57500E+00_JPRB, 0.64005E+00_JPRB ,&
&  0.70484E+00_JPRB, 0.80549E+00_JPRB, 0.96773E+00_JPRB, 0.11726E+01_JPRB, 0.13241E+01_JPRB, 0.13686E+01_JPRB, 0.12374E+01_JPRB ,&
&  0.84616E+00_JPRB, 0.10901E+01_JPRB, 0.10606E+01_JPRB, 0.11100E+01_JPRB, 0.10868E+01_JPRB, 0.10311E+01_JPRB, 0.92356E+00_JPRB ,&
&  0.86234E+00_JPRB, 0.85443E+00_JPRB, 0.83363E+00_JPRB, 0.82336E+00_JPRB, 0.80847E+00_JPRB, 0.74181E+00_JPRB, 0.72351E+00_JPRB ,&
&  0.74978E+00_JPRB, 0.80893E+00_JPRB, 0.87912E+00_JPRB, 0.94038E+00_JPRB, 0.10153E+01_JPRB, 0.10815E+01_JPRB, 0.11232E+01_JPRB ,&
&  0.11737E+01_JPRB, 0.11800E+01_JPRB, 0.11078E+01_JPRB, 0.10722E+01_JPRB, 0.10561E+01_JPRB, 0.10664E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.99611E+00_JPRB, 0.94763E+00_JPRB, 0.10189E+01_JPRB, 0.10091E+01_JPRB, 0.95792E+00_JPRB, 0.99661E+00_JPRB, 0.10353E+01_JPRB ,&
&  0.12749E+01_JPRB, 0.13840E+01_JPRB, 0.17834E+01_JPRB, 0.16274E+01_JPRB, 0.16451E+01_JPRB, 0.18091E+01_JPRB, 0.17540E+01_JPRB ,&
&  0.15770E+01_JPRB, 0.15887E+01_JPRB, 0.14968E+01_JPRB, 0.15044E+01_JPRB, 0.14639E+01_JPRB, 0.14749E+01_JPRB, 0.14854E+01_JPRB ,&
&  0.15496E+01_JPRB, 0.16700E+01_JPRB, 0.18309E+01_JPRB, 0.15669E+01_JPRB, 0.14383E+01_JPRB, 0.13470E+01_JPRB, 0.14308E+01_JPRB ,&
&  0.15702E+01_JPRB, 0.16058E+01_JPRB, 0.17027E+01_JPRB, 0.16220E+01_JPRB, 0.13103E+01_JPRB, 0.12192E+01_JPRB, 0.11494E+01_JPRB ,&
&  0.10797E+01_JPRB, 0.99720E+00_JPRB, 0.93656E+00_JPRB, 0.86577E+00_JPRB, 0.79998E+00_JPRB, 0.76487E+00_JPRB, 0.74011E+00_JPRB ,&
&  0.71610E+00_JPRB, 0.69918E+00_JPRB, 0.68215E+00_JPRB, 0.66141E+00_JPRB, 0.64756E+00_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.66867E+00_JPRB, 0.67880E+00_JPRB, 0.69880E+00_JPRB, 0.71829E+00_JPRB, 0.82089E+00_JPRB, 0.94085E+00_JPRB, 0.10089E+01_JPRB ,&
&  0.95539E+00_JPRB, 0.58331E+00_JPRB, 0.55992E+00_JPRB, 0.56647E+00_JPRB, 0.58984E+00_JPRB, 0.64677E+00_JPRB, 0.71869E+00_JPRB ,&
&  0.81939E+00_JPRB, 0.93683E+00_JPRB, 0.10990E+01_JPRB, 0.12985E+01_JPRB, 0.14165E+01_JPRB, 0.15275E+01_JPRB, 0.14575E+01_JPRB ,&
&  0.10285E+01_JPRB, 0.92537E+00_JPRB, 0.11062E+01_JPRB, 0.11353E+01_JPRB, 0.10881E+01_JPRB, 0.10751E+01_JPRB, 0.10541E+01_JPRB ,&
&  0.10067E+01_JPRB, 0.96605E+00_JPRB, 0.95718E+00_JPRB, 0.92588E+00_JPRB, 0.82995E+00_JPRB, 0.80172E+00_JPRB, 0.80960E+00_JPRB ,&
&  0.84921E+00_JPRB, 0.90469E+00_JPRB, 0.96838E+00_JPRB, 0.10232E+01_JPRB, 0.10929E+01_JPRB, 0.11400E+01_JPRB, 0.11838E+01_JPRB ,&
&  0.12056E+01_JPRB, 0.12211E+01_JPRB, 0.11704E+01_JPRB, 0.11076E+01_JPRB, 0.10929E+01_JPRB, 0.10778E+01_JPRB, 0.10443E+01_JPRB ,&
&  0.10152E+01_JPRB, 0.10362E+01_JPRB, 0.10459E+01_JPRB, 0.10050E+01_JPRB, 0.97121E+00_JPRB, 0.11030E+01_JPRB, 0.12206E+01_JPRB ,&
&  0.13896E+01_JPRB, 0.15844E+01_JPRB, 0.20345E+01_JPRB, 0.16084E+01_JPRB, 0.16102E+01_JPRB, 0.17105E+01_JPRB, 0.16863E+01_JPRB ,&
&  0.15757E+01_JPRB, 0.16034E+01_JPRB, 0.15561E+01_JPRB, 0.15247E+01_JPRB, 0.15144E+01_JPRB, 0.14464E+01_JPRB, 0.16123E+01_JPRB ,&
&  0.15495E+01_JPRB, 0.17458E+01_JPRB, 0.16699E+01_JPRB, 0.15944E+01_JPRB, 0.13972E+01_JPRB, 0.13688E+01_JPRB, 0.13780E+01_JPRB ,&
&  0.14851E+01_JPRB, 0.14939E+01_JPRB, 0.14379E+01_JPRB, 0.13759E+01_JPRB, 0.12053E+01_JPRB, 0.11393E+01_JPRB, 0.10759E+01_JPRB ,&
&  0.10013E+01_JPRB, 0.93112E+00_JPRB, 0.88739E+00_JPRB, 0.84340E+00_JPRB, 0.81352E+00_JPRB, 0.78363E+00_JPRB, 0.75082E+00_JPRB ,&
&  0.72471E+00_JPRB, 0.70707E+00_JPRB, 0.69251E+00_JPRB, 0.67578E+00_JPRB, 0.66680E+00_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.68087E+00_JPRB, 0.67795E+00_JPRB, 0.68490E+00_JPRB, 0.67458E+00_JPRB, 0.67560E+00_JPRB, 0.67994E+00_JPRB, 0.66070E+00_JPRB ,&
&  0.62949E+00_JPRB, 0.59672E+00_JPRB, 0.59879E+00_JPRB, 0.62401E+00_JPRB, 0.66480E+00_JPRB, 0.73305E+00_JPRB, 0.82457E+00_JPRB ,&
&  0.94072E+00_JPRB, 0.10414E+01_JPRB, 0.11942E+01_JPRB, 0.13257E+01_JPRB, 0.14154E+01_JPRB, 0.14677E+01_JPRB, 0.14417E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.91225E+00_JPRB, 0.11347E+01_JPRB, 0.12309E+01_JPRB, 0.11726E+01_JPRB, 0.11664E+01_JPRB, 0.11374E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.10410E+01_JPRB, 0.10318E+01_JPRB, 0.96582E+00_JPRB, 0.88270E+00_JPRB, 0.88646E+00_JPRB, 0.91817E+00_JPRB ,&
&  0.96732E+00_JPRB, 0.10090E+01_JPRB, 0.10587E+01_JPRB, 0.11106E+01_JPRB, 0.11590E+01_JPRB, 0.12265E+01_JPRB, 0.12669E+01_JPRB ,&
&  0.12753E+01_JPRB, 0.12718E+01_JPRB, 0.12529E+01_JPRB, 0.11806E+01_JPRB, 0.11288E+01_JPRB, 0.10973E+01_JPRB, 0.10629E+01_JPRB ,&
&  0.10569E+01_JPRB, 0.10492E+01_JPRB, 0.10880E+01_JPRB, 0.11687E+01_JPRB, 0.13057E+01_JPRB, 0.12928E+01_JPRB, 0.14433E+01_JPRB ,&
&  0.15510E+01_JPRB, 0.17846E+01_JPRB, 0.18518E+01_JPRB, 0.20532E+01_JPRB, 0.13345E+01_JPRB, 0.15132E+01_JPRB, 0.14271E+01_JPRB ,&
&  0.16239E+01_JPRB, 0.15453E+01_JPRB, 0.15202E+01_JPRB, 0.15291E+01_JPRB, 0.15949E+01_JPRB, 0.15298E+01_JPRB, 0.15881E+01_JPRB ,&
&  0.15983E+01_JPRB, 0.15964E+01_JPRB, 0.15072E+01_JPRB, 0.14856E+01_JPRB, 0.14016E+01_JPRB, 0.14482E+01_JPRB, 0.13441E+01_JPRB ,&
&  0.13818E+01_JPRB, 0.13687E+01_JPRB, 0.13470E+01_JPRB, 0.13334E+01_JPRB, 0.11823E+01_JPRB, 0.10238E+01_JPRB, 0.95244E+00_JPRB ,&
&  0.91737E+00_JPRB, 0.88460E+00_JPRB, 0.84399E+00_JPRB, 0.81419E+00_JPRB, 0.79547E+00_JPRB, 0.77737E+00_JPRB, 0.75775E+00_JPRB ,&
&  0.73750E+00_JPRB, 0.72071E+00_JPRB, 0.70773E+00_JPRB, 0.69442E+00_JPRB, 0.68574E+00_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.67043E+00_JPRB, 0.66224E+00_JPRB, 0.66567E+00_JPRB, 0.66592E+00_JPRB, 0.66170E+00_JPRB, 0.64497E+00_JPRB, 0.63838E+00_JPRB ,&
&  0.63222E+00_JPRB, 0.63884E+00_JPRB, 0.64355E+00_JPRB, 0.66813E+00_JPRB, 0.70320E+00_JPRB, 0.76389E+00_JPRB, 0.82471E+00_JPRB ,&
&  0.89551E+00_JPRB, 0.98711E+00_JPRB, 0.11589E+01_JPRB, 0.13656E+01_JPRB, 0.15087E+01_JPRB, 0.15714E+01_JPRB, 0.14607E+01_JPRB ,&
&  0.13378E+01_JPRB, 0.13076E+01_JPRB, 0.12662E+01_JPRB, 0.10591E+01_JPRB, 0.99119E+00_JPRB, 0.11851E+01_JPRB, 0.11405E+01_JPRB ,&
&  0.10862E+01_JPRB, 0.10638E+01_JPRB, 0.10266E+01_JPRB, 0.98912E+00_JPRB, 0.96175E+00_JPRB, 0.97329E+00_JPRB, 0.10048E+01_JPRB ,&
&  0.10470E+01_JPRB, 0.10873E+01_JPRB, 0.11401E+01_JPRB, 0.11933E+01_JPRB, 0.12491E+01_JPRB, 0.13356E+01_JPRB, 0.13766E+01_JPRB ,&
&  0.13676E+01_JPRB, 0.13563E+01_JPRB, 0.13261E+01_JPRB, 0.12583E+01_JPRB, 0.11760E+01_JPRB, 0.11612E+01_JPRB, 0.11719E+01_JPRB ,&
&  0.11907E+01_JPRB, 0.12196E+01_JPRB, 0.12686E+01_JPRB, 0.14001E+01_JPRB, 0.15200E+01_JPRB, 0.14384E+01_JPRB, 0.14528E+01_JPRB ,&
&  0.16353E+01_JPRB, 0.18478E+01_JPRB, 0.16528E+01_JPRB, 0.19047E+01_JPRB, 0.11610E+01_JPRB, 0.13733E+01_JPRB, 0.15523E+01_JPRB ,&
&  0.14711E+01_JPRB, 0.14501E+01_JPRB, 0.14932E+01_JPRB, 0.15826E+01_JPRB, 0.16558E+01_JPRB, 0.15292E+01_JPRB, 0.15415E+01_JPRB ,&
&  0.15035E+01_JPRB, 0.13865E+01_JPRB, 0.13593E+01_JPRB, 0.13427E+01_JPRB, 0.13711E+01_JPRB, 0.13669E+01_JPRB, 0.12033E+01_JPRB ,&
&  0.11036E+01_JPRB, 0.12673E+01_JPRB, 0.13266E+01_JPRB, 0.13465E+01_JPRB, 0.11744E+01_JPRB, 0.90652E+00_JPRB, 0.88372E+00_JPRB ,&
&  0.87409E+00_JPRB, 0.83500E+00_JPRB, 0.81189E+00_JPRB, 0.79616E+00_JPRB, 0.78714E+00_JPRB, 0.77424E+00_JPRB, 0.76259E+00_JPRB ,&
&  0.75128E+00_JPRB, 0.73335E+00_JPRB, 0.71780E+00_JPRB, 0.70155E+00_JPRB, 0.68458E+00_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.64849E+00_JPRB, 0.64879E+00_JPRB, 0.65715E+00_JPRB, 0.65806E+00_JPRB, 0.63069E+00_JPRB, 0.61276E+00_JPRB, 0.60720E+00_JPRB ,&
&  0.61544E+00_JPRB, 0.62966E+00_JPRB, 0.64082E+00_JPRB, 0.64974E+00_JPRB, 0.67747E+00_JPRB, 0.69494E+00_JPRB, 0.72550E+00_JPRB ,&
&  0.78053E+00_JPRB, 0.86876E+00_JPRB, 0.10325E+01_JPRB, 0.12899E+01_JPRB, 0.15224E+01_JPRB, 0.17263E+01_JPRB, 0.15822E+01_JPRB ,&
&  0.14415E+01_JPRB, 0.13679E+01_JPRB, 0.13515E+01_JPRB, 0.13429E+01_JPRB, 0.11886E+01_JPRB, 0.12506E+01_JPRB, 0.13093E+01_JPRB ,&
&  0.13103E+01_JPRB, 0.12184E+01_JPRB, 0.11301E+01_JPRB, 0.10865E+01_JPRB, 0.10534E+01_JPRB, 0.10557E+01_JPRB, 0.10834E+01_JPRB ,&
&  0.11197E+01_JPRB, 0.11577E+01_JPRB, 0.12185E+01_JPRB, 0.12953E+01_JPRB, 0.13880E+01_JPRB, 0.14743E+01_JPRB, 0.15065E+01_JPRB ,&
&  0.14945E+01_JPRB, 0.14819E+01_JPRB, 0.14568E+01_JPRB, 0.13762E+01_JPRB, 0.13462E+01_JPRB, 0.13662E+01_JPRB, 0.14084E+01_JPRB ,&
&  0.14666E+01_JPRB, 0.14471E+01_JPRB, 0.15171E+01_JPRB, 0.16646E+01_JPRB, 0.16921E+01_JPRB, 0.16375E+01_JPRB, 0.16280E+01_JPRB ,&
&  0.17408E+01_JPRB, 0.19008E+01_JPRB, 0.13309E+01_JPRB, 0.15563E+01_JPRB, 0.18084E+01_JPRB, 0.15406E+01_JPRB, 0.14380E+01_JPRB ,&
&  0.14148E+01_JPRB, 0.14291E+01_JPRB, 0.14980E+01_JPRB, 0.15893E+01_JPRB, 0.16753E+01_JPRB, 0.17304E+01_JPRB, 0.13892E+01_JPRB ,&
&  0.13495E+01_JPRB, 0.12425E+01_JPRB, 0.12244E+01_JPRB, 0.12345E+01_JPRB, 0.12606E+01_JPRB, 0.12335E+01_JPRB, 0.11012E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.12058E+01_JPRB, 0.12463E+01_JPRB, 0.14013E+01_JPRB, 0.14090E+01_JPRB, 0.85216E+00_JPRB, 0.86591E+00_JPRB ,&
&  0.84557E+00_JPRB, 0.81436E+00_JPRB, 0.79966E+00_JPRB, 0.78677E+00_JPRB, 0.77900E+00_JPRB, 0.77015E+00_JPRB, 0.76340E+00_JPRB ,&
&  0.74901E+00_JPRB, 0.72748E+00_JPRB, 0.70514E+00_JPRB, 0.68319E+00_JPRB, 0.66327E+00_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.63846E+00_JPRB, 0.64466E+00_JPRB, 0.65343E+00_JPRB, 0.63846E+00_JPRB, 0.61135E+00_JPRB, 0.59801E+00_JPRB, 0.58975E+00_JPRB ,&
&  0.59138E+00_JPRB, 0.60669E+00_JPRB, 0.61613E+00_JPRB, 0.61408E+00_JPRB, 0.61188E+00_JPRB, 0.62666E+00_JPRB, 0.66865E+00_JPRB ,&
&  0.72357E+00_JPRB, 0.79386E+00_JPRB, 0.92133E+00_JPRB, 0.11798E+01_JPRB, 0.14972E+01_JPRB, 0.17366E+01_JPRB, 0.16627E+01_JPRB ,&
&  0.15223E+01_JPRB, 0.15133E+01_JPRB, 0.15155E+01_JPRB, 0.15339E+01_JPRB, 0.15382E+01_JPRB, 0.15713E+01_JPRB, 0.15891E+01_JPRB ,&
&  0.13777E+01_JPRB, 0.11345E+01_JPRB, 0.11040E+01_JPRB, 0.11023E+01_JPRB, 0.11141E+01_JPRB, 0.11199E+01_JPRB, 0.11453E+01_JPRB ,&
&  0.11789E+01_JPRB, 0.12325E+01_JPRB, 0.13098E+01_JPRB, 0.14331E+01_JPRB, 0.15640E+01_JPRB, 0.16394E+01_JPRB, 0.16752E+01_JPRB ,&
&  0.16924E+01_JPRB, 0.17019E+01_JPRB, 0.17434E+01_JPRB, 0.15793E+01_JPRB, 0.16329E+01_JPRB, 0.16813E+01_JPRB, 0.17264E+01_JPRB ,&
&  0.17156E+01_JPRB, 0.16674E+01_JPRB, 0.17856E+01_JPRB, 0.18659E+01_JPRB, 0.19416E+01_JPRB, 0.19268E+01_JPRB, 0.18790E+01_JPRB ,&
&  0.19424E+01_JPRB, 0.17779E+01_JPRB, 0.12046E+01_JPRB, 0.11857E+01_JPRB, 0.12675E+01_JPRB, 0.11754E+01_JPRB, 0.13923E+01_JPRB ,&
&  0.13606E+01_JPRB, 0.13800E+01_JPRB, 0.13979E+01_JPRB, 0.14329E+01_JPRB, 0.14743E+01_JPRB, 0.14983E+01_JPRB, 0.12920E+01_JPRB ,&
&  0.11908E+01_JPRB, 0.11183E+01_JPRB, 0.11009E+01_JPRB, 0.10823E+01_JPRB, 0.10735E+01_JPRB, 0.10498E+01_JPRB, 0.10346E+01_JPRB ,&
&  0.10419E+01_JPRB, 0.10346E+01_JPRB, 0.99082E+00_JPRB, 0.98895E+00_JPRB, 0.91483E+00_JPRB, 0.85425E+00_JPRB, 0.84741E+00_JPRB ,&
&  0.83620E+00_JPRB, 0.80188E+00_JPRB, 0.79152E+00_JPRB, 0.77772E+00_JPRB, 0.76797E+00_JPRB, 0.75764E+00_JPRB, 0.74478E+00_JPRB ,&
&  0.72417E+00_JPRB, 0.70404E+00_JPRB, 0.68329E+00_JPRB, 0.66060E+00_JPRB, 0.64296E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.64239E+00_JPRB, 0.65196E+00_JPRB, 0.65398E+00_JPRB, 0.64035E+00_JPRB, 0.64298E+00_JPRB, 0.64842E+00_JPRB, 0.63472E+00_JPRB ,&
&  0.62520E+00_JPRB, 0.61860E+00_JPRB, 0.63210E+00_JPRB, 0.63309E+00_JPRB, 0.63664E+00_JPRB, 0.65059E+00_JPRB, 0.66733E+00_JPRB ,&
&  0.68570E+00_JPRB, 0.70099E+00_JPRB, 0.76622E+00_JPRB, 0.89903E+00_JPRB, 0.11449E+01_JPRB, 0.14629E+01_JPRB, 0.16792E+01_JPRB ,&
&  0.17919E+01_JPRB, 0.18956E+01_JPRB, 0.19843E+01_JPRB, 0.20522E+01_JPRB, 0.18895E+01_JPRB, 0.18925E+01_JPRB, 0.20023E+01_JPRB ,&
&  0.14516E+01_JPRB, 0.10733E+01_JPRB, 0.11618E+01_JPRB, 0.10936E+01_JPRB, 0.11128E+01_JPRB, 0.11577E+01_JPRB, 0.11939E+01_JPRB ,&
&  0.12465E+01_JPRB, 0.13156E+01_JPRB, 0.14064E+01_JPRB, 0.15352E+01_JPRB, 0.16655E+01_JPRB, 0.17552E+01_JPRB, 0.18212E+01_JPRB ,&
&  0.18773E+01_JPRB, 0.19413E+01_JPRB, 0.20434E+01_JPRB, 0.20235E+01_JPRB, 0.19796E+01_JPRB, 0.20552E+01_JPRB, 0.20900E+01_JPRB ,&
&  0.21331E+01_JPRB, 0.21269E+01_JPRB, 0.18806E+01_JPRB, 0.19410E+01_JPRB, 0.21640E+01_JPRB, 0.20549E+01_JPRB, 0.20259E+01_JPRB ,&
&  0.21556E+01_JPRB, 0.18207E+01_JPRB, 0.11161E+01_JPRB, 0.10693E+01_JPRB, 0.11542E+01_JPRB, 0.12494E+01_JPRB, 0.13434E+01_JPRB ,&
&  0.13579E+01_JPRB, 0.13435E+01_JPRB, 0.13030E+01_JPRB, 0.12894E+01_JPRB, 0.12443E+01_JPRB, 0.12056E+01_JPRB, 0.11610E+01_JPRB ,&
&  0.10578E+01_JPRB, 0.10307E+01_JPRB, 0.99823E+00_JPRB, 0.95919E+00_JPRB, 0.94219E+00_JPRB, 0.91827E+00_JPRB, 0.92509E+00_JPRB ,&
&  0.98486E+00_JPRB, 0.96684E+00_JPRB, 0.10121E+01_JPRB, 0.94169E+00_JPRB, 0.85528E+00_JPRB, 0.87763E+00_JPRB, 0.86129E+00_JPRB ,&
&  0.87629E+00_JPRB, 0.85221E+00_JPRB, 0.82869E+00_JPRB, 0.80559E+00_JPRB, 0.78739E+00_JPRB, 0.76356E+00_JPRB, 0.73263E+00_JPRB ,&
&  0.70174E+00_JPRB, 0.67816E+00_JPRB, 0.66318E+00_JPRB, 0.64944E+00_JPRB, 0.63668E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.64576E+00_JPRB, 0.63948E+00_JPRB, 0.65302E+00_JPRB, 0.69118E+00_JPRB, 0.66962E+00_JPRB, 0.68937E+00_JPRB, 0.68429E+00_JPRB ,&
&  0.68326E+00_JPRB, 0.67828E+00_JPRB, 0.69088E+00_JPRB, 0.70586E+00_JPRB, 0.72861E+00_JPRB, 0.74898E+00_JPRB, 0.75496E+00_JPRB ,&
&  0.76580E+00_JPRB, 0.77756E+00_JPRB, 0.79697E+00_JPRB, 0.83064E+00_JPRB, 0.89023E+00_JPRB, 0.98764E+00_JPRB, 0.11775E+01_JPRB ,&
&  0.13670E+01_JPRB, 0.14563E+01_JPRB, 0.14807E+01_JPRB, 0.14671E+01_JPRB, 0.16141E+01_JPRB, 0.20108E+01_JPRB, 0.22892E+01_JPRB ,&
&  0.13617E+01_JPRB, 0.11534E+01_JPRB, 0.10690E+01_JPRB, 0.98017E+00_JPRB, 0.10144E+01_JPRB, 0.11286E+01_JPRB, 0.11881E+01_JPRB ,&
&  0.12201E+01_JPRB, 0.12860E+01_JPRB, 0.13742E+01_JPRB, 0.14837E+01_JPRB, 0.15778E+01_JPRB, 0.16663E+01_JPRB, 0.17607E+01_JPRB ,&
&  0.18649E+01_JPRB, 0.20041E+01_JPRB, 0.21531E+01_JPRB, 0.23178E+01_JPRB, 0.22997E+01_JPRB, 0.25600E+01_JPRB, 0.26143E+01_JPRB ,&
&  0.26485E+01_JPRB, 0.25235E+01_JPRB, 0.21978E+01_JPRB, 0.22338E+01_JPRB, 0.23921E+01_JPRB, 0.23712E+01_JPRB, 0.21934E+01_JPRB ,&
&  0.21873E+01_JPRB, 0.17007E+01_JPRB, 0.11960E+01_JPRB, 0.11792E+01_JPRB, 0.11924E+01_JPRB, 0.12434E+01_JPRB, 0.12069E+01_JPRB ,&
&  0.12345E+01_JPRB, 0.11910E+01_JPRB, 0.11508E+01_JPRB, 0.11161E+01_JPRB, 0.10936E+01_JPRB, 0.11124E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.10234E+01_JPRB, 0.99661E+00_JPRB, 0.93141E+00_JPRB, 0.88468E+00_JPRB, 0.88430E+00_JPRB, 0.77078E+00_JPRB, 0.86611E+00_JPRB ,&
&  0.89130E+00_JPRB, 0.89503E+00_JPRB, 0.81032E+00_JPRB, 0.98918E+00_JPRB, 0.89522E+00_JPRB, 0.86188E+00_JPRB, 0.87059E+00_JPRB ,&
&  0.89546E+00_JPRB, 0.85506E+00_JPRB, 0.83316E+00_JPRB, 0.81422E+00_JPRB, 0.80406E+00_JPRB, 0.78060E+00_JPRB, 0.73161E+00_JPRB ,&
&  0.69507E+00_JPRB, 0.67128E+00_JPRB, 0.65862E+00_JPRB, 0.64875E+00_JPRB, 0.64844E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.63915E+00_JPRB, 0.65065E+00_JPRB, 0.71392E+00_JPRB, 0.67440E+00_JPRB, 0.67340E+00_JPRB, 0.68895E+00_JPRB, 0.70657E+00_JPRB ,&
&  0.70855E+00_JPRB, 0.71925E+00_JPRB, 0.74071E+00_JPRB, 0.77658E+00_JPRB, 0.81133E+00_JPRB, 0.83834E+00_JPRB, 0.86806E+00_JPRB ,&
&  0.89107E+00_JPRB, 0.90514E+00_JPRB, 0.91290E+00_JPRB, 0.92173E+00_JPRB, 0.94939E+00_JPRB, 0.98076E+00_JPRB, 0.10262E+01_JPRB ,&
&  0.10826E+01_JPRB, 0.10956E+01_JPRB, 0.10933E+01_JPRB, 0.11195E+01_JPRB, 0.11454E+01_JPRB, 0.12428E+01_JPRB, 0.14156E+01_JPRB ,&
&  0.10798E+01_JPRB, 0.11317E+01_JPRB, 0.10802E+01_JPRB, 0.10175E+01_JPRB, 0.10455E+01_JPRB, 0.10302E+01_JPRB, 0.10741E+01_JPRB ,&
&  0.11478E+01_JPRB, 0.11612E+01_JPRB, 0.12126E+01_JPRB, 0.12591E+01_JPRB, 0.13133E+01_JPRB, 0.13895E+01_JPRB, 0.14666E+01_JPRB ,&
&  0.15678E+01_JPRB, 0.16988E+01_JPRB, 0.18037E+01_JPRB, 0.19704E+01_JPRB, 0.21272E+01_JPRB, 0.22891E+01_JPRB, 0.23766E+01_JPRB ,&
&  0.24336E+01_JPRB, 0.24729E+01_JPRB, 0.22851E+01_JPRB, 0.22905E+01_JPRB, 0.22954E+01_JPRB, 0.21888E+01_JPRB, 0.19299E+01_JPRB ,&
&  0.16669E+01_JPRB, 0.14540E+01_JPRB, 0.13126E+01_JPRB, 0.12228E+01_JPRB, 0.11941E+01_JPRB, 0.11552E+01_JPRB, 0.11117E+01_JPRB ,&
&  0.10919E+01_JPRB, 0.10342E+01_JPRB, 0.98410E+00_JPRB, 0.94080E+00_JPRB, 0.93246E+00_JPRB, 0.95612E+00_JPRB, 0.95362E+00_JPRB ,&
&  0.90745E+00_JPRB, 0.86675E+00_JPRB, 0.81598E+00_JPRB, 0.78507E+00_JPRB, 0.77471E+00_JPRB, 0.95963E+00_JPRB, 0.88538E+00_JPRB ,&
&  0.74380E+00_JPRB, 0.66643E+00_JPRB, 0.59993E+00_JPRB, 0.81815E+00_JPRB, 0.82648E+00_JPRB, 0.82151E+00_JPRB, 0.85351E+00_JPRB ,&
&  0.86456E+00_JPRB, 0.85659E+00_JPRB, 0.88157E+00_JPRB, 0.83921E+00_JPRB, 0.80360E+00_JPRB, 0.78336E+00_JPRB, 0.77245E+00_JPRB ,&
&  0.71910E+00_JPRB, 0.66754E+00_JPRB, 0.61909E+00_JPRB, 0.62244E+00_JPRB, 0.59224E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.58334E+00_JPRB, 0.63712E+00_JPRB, 0.72846E+00_JPRB, 0.65949E+00_JPRB, 0.65750E+00_JPRB, 0.67251E+00_JPRB, 0.69562E+00_JPRB ,&
&  0.71991E+00_JPRB, 0.75119E+00_JPRB, 0.78462E+00_JPRB, 0.83094E+00_JPRB, 0.87139E+00_JPRB, 0.90702E+00_JPRB, 0.94065E+00_JPRB ,&
&  0.97118E+00_JPRB, 0.99729E+00_JPRB, 0.10223E+01_JPRB, 0.10339E+01_JPRB, 0.10459E+01_JPRB, 0.10677E+01_JPRB, 0.10957E+01_JPRB ,&
&  0.11464E+01_JPRB, 0.11706E+01_JPRB, 0.12009E+01_JPRB, 0.12511E+01_JPRB, 0.12844E+01_JPRB, 0.13136E+01_JPRB, 0.11186E+01_JPRB ,&
&  0.12087E+01_JPRB, 0.11089E+01_JPRB, 0.10597E+01_JPRB, 0.10383E+01_JPRB, 0.10145E+01_JPRB, 0.10041E+01_JPRB, 0.10141E+01_JPRB ,&
&  0.10582E+01_JPRB, 0.10310E+01_JPRB, 0.10504E+01_JPRB, 0.10421E+01_JPRB, 0.10726E+01_JPRB, 0.11457E+01_JPRB, 0.11868E+01_JPRB ,&
&  0.12818E+01_JPRB, 0.14007E+01_JPRB, 0.15206E+01_JPRB, 0.16112E+01_JPRB, 0.16961E+01_JPRB, 0.16979E+01_JPRB, 0.17013E+01_JPRB ,&
&  0.17489E+01_JPRB, 0.18415E+01_JPRB, 0.17995E+01_JPRB, 0.18312E+01_JPRB, 0.18164E+01_JPRB, 0.16719E+01_JPRB, 0.13346E+01_JPRB ,&
&  0.11657E+01_JPRB, 0.12345E+01_JPRB, 0.11365E+01_JPRB, 0.12973E+01_JPRB, 0.11835E+01_JPRB, 0.10670E+01_JPRB, 0.10263E+01_JPRB ,&
&  0.95362E+00_JPRB, 0.89053E+00_JPRB, 0.84615E+00_JPRB, 0.77745E+00_JPRB, 0.78235E+00_JPRB, 0.79060E+00_JPRB, 0.79095E+00_JPRB ,&
&  0.74375E+00_JPRB, 0.69494E+00_JPRB, 0.64910E+00_JPRB, 0.64719E+00_JPRB, 0.75608E+00_JPRB, 0.78732E+00_JPRB, 0.91719E+00_JPRB ,&
&  0.61407E+00_JPRB, 0.41489E+00_JPRB, 0.52467E+00_JPRB, 0.64720E+00_JPRB, 0.67532E+00_JPRB, 0.71146E+00_JPRB, 0.77312E+00_JPRB ,&
&  0.84343E+00_JPRB, 0.84171E+00_JPRB, 0.81274E+00_JPRB, 0.80926E+00_JPRB, 0.79877E+00_JPRB, 0.79424E+00_JPRB, 0.71246E+00_JPRB ,&
&  0.68357E+00_JPRB, 0.62824E+00_JPRB, 0.59756E+00_JPRB, 0.58379E+00_JPRB, 0.55701E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.57309E+00_JPRB, 0.65353E+00_JPRB, 0.72215E+00_JPRB, 0.65238E+00_JPRB, 0.66010E+00_JPRB, 0.67912E+00_JPRB, 0.70862E+00_JPRB ,&
&  0.74262E+00_JPRB, 0.78305E+00_JPRB, 0.82546E+00_JPRB, 0.87562E+00_JPRB, 0.91510E+00_JPRB, 0.95145E+00_JPRB, 0.99068E+00_JPRB ,&
&  0.10317E+01_JPRB, 0.10453E+01_JPRB, 0.10781E+01_JPRB, 0.10827E+01_JPRB, 0.11064E+01_JPRB, 0.11069E+01_JPRB, 0.11260E+01_JPRB ,&
&  0.11660E+01_JPRB, 0.11902E+01_JPRB, 0.12157E+01_JPRB, 0.12333E+01_JPRB, 0.12214E+01_JPRB, 0.12542E+01_JPRB, 0.97289E+00_JPRB ,&
&  0.11656E+01_JPRB, 0.10896E+01_JPRB, 0.10266E+01_JPRB, 0.10321E+01_JPRB, 0.10143E+01_JPRB, 0.10190E+01_JPRB, 0.10179E+01_JPRB ,&
&  0.10155E+01_JPRB, 0.10249E+01_JPRB, 0.10462E+01_JPRB, 0.10348E+01_JPRB, 0.99082E+00_JPRB, 0.10319E+01_JPRB, 0.10739E+01_JPRB ,&
&  0.11370E+01_JPRB, 0.12129E+01_JPRB, 0.12721E+01_JPRB, 0.13101E+01_JPRB, 0.13306E+01_JPRB, 0.13096E+01_JPRB, 0.12851E+01_JPRB ,&
&  0.12583E+01_JPRB, 0.12711E+01_JPRB, 0.11984E+01_JPRB, 0.12573E+01_JPRB, 0.13651E+01_JPRB, 0.11673E+01_JPRB, 0.90481E+00_JPRB ,&
&  0.78214E+00_JPRB, 0.97690E+00_JPRB, 0.11627E+01_JPRB, 0.13042E+01_JPRB, 0.11253E+01_JPRB, 0.10311E+01_JPRB, 0.94667E+00_JPRB ,&
&  0.84713E+00_JPRB, 0.77025E+00_JPRB, 0.72913E+00_JPRB, 0.66551E+00_JPRB, 0.63790E+00_JPRB, 0.63402E+00_JPRB, 0.62928E+00_JPRB ,&
&  0.60894E+00_JPRB, 0.58954E+00_JPRB, 0.57696E+00_JPRB, 0.58274E+00_JPRB, 0.59554E+00_JPRB, 0.59714E+00_JPRB, 0.97018E+00_JPRB ,&
&  0.62527E+00_JPRB, 0.46069E+00_JPRB, 0.68480E+00_JPRB, 0.41809E+00_JPRB, 0.53716E+00_JPRB, 0.59204E+00_JPRB, 0.63757E+00_JPRB ,&
&  0.60216E+00_JPRB, 0.76616E+00_JPRB, 0.11147E+01_JPRB, 0.11485E+01_JPRB, 0.12084E+01_JPRB, 0.12833E+01_JPRB, 0.76172E+00_JPRB ,&
&  0.61223E+00_JPRB, 0.60050E+00_JPRB, 0.60254E+00_JPRB, 0.55520E+00_JPRB, 0.55125E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.55410E+00_JPRB, 0.61597E+00_JPRB, 0.68549E+00_JPRB, 0.73043E+00_JPRB, 0.70730E+00_JPRB, 0.70342E+00_JPRB, 0.71843E+00_JPRB ,&
&  0.74152E+00_JPRB, 0.77283E+00_JPRB, 0.80593E+00_JPRB, 0.84993E+00_JPRB, 0.88002E+00_JPRB, 0.91814E+00_JPRB, 0.97027E+00_JPRB ,&
&  0.10312E+01_JPRB, 0.10419E+01_JPRB, 0.10610E+01_JPRB, 0.10826E+01_JPRB, 0.11010E+01_JPRB, 0.11226E+01_JPRB, 0.11403E+01_JPRB ,&
&  0.11456E+01_JPRB, 0.11595E+01_JPRB, 0.11634E+01_JPRB, 0.11935E+01_JPRB, 0.12041E+01_JPRB, 0.13041E+01_JPRB, 0.94170E+00_JPRB ,&
&  0.11109E+01_JPRB, 0.10590E+01_JPRB, 0.99259E+00_JPRB, 0.10093E+01_JPRB, 0.10058E+01_JPRB, 0.99399E+00_JPRB, 0.98128E+00_JPRB ,&
&  0.10078E+01_JPRB, 0.10208E+01_JPRB, 0.10308E+01_JPRB, 0.11218E+01_JPRB, 0.11233E+01_JPRB, 0.11198E+01_JPRB, 0.12170E+01_JPRB ,&
&  0.13080E+01_JPRB, 0.13832E+01_JPRB, 0.14086E+01_JPRB, 0.14502E+01_JPRB, 0.14526E+01_JPRB, 0.14297E+01_JPRB, 0.13938E+01_JPRB ,&
&  0.13500E+01_JPRB, 0.13396E+01_JPRB, 0.13708E+01_JPRB, 0.14583E+01_JPRB, 0.14834E+01_JPRB, 0.13066E+01_JPRB, 0.10658E+01_JPRB ,&
&  0.79798E+00_JPRB, 0.73538E+00_JPRB, 0.10327E+01_JPRB, 0.12077E+01_JPRB, 0.10622E+01_JPRB, 0.99973E+00_JPRB, 0.86173E+00_JPRB ,&
&  0.78430E+00_JPRB, 0.70257E+00_JPRB, 0.66756E+00_JPRB, 0.61481E+00_JPRB, 0.58539E+00_JPRB, 0.58458E+00_JPRB, 0.61305E+00_JPRB ,&
&  0.62486E+00_JPRB, 0.61016E+00_JPRB, 0.60196E+00_JPRB, 0.59109E+00_JPRB, 0.62910E+00_JPRB, 0.72124E+00_JPRB, 0.71521E+00_JPRB ,&
&  0.77393E+00_JPRB, 0.60321E+00_JPRB, 0.55762E+00_JPRB, 0.59393E+00_JPRB, 0.52015E+00_JPRB, 0.55967E+00_JPRB, 0.62067E+00_JPRB ,&
&  0.80047E+00_JPRB, 0.88354E+00_JPRB, 0.12214E+01_JPRB, 0.34213E+01_JPRB, 0.39465E+01_JPRB, 0.49488E+01_JPRB, 0.19734E+01_JPRB ,&
&  0.91790E+00_JPRB, 0.61049E+00_JPRB, 0.54273E+00_JPRB, 0.54140E+00_JPRB, 0.53421E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.52571E+00_JPRB, 0.57373E+00_JPRB, 0.61260E+00_JPRB, 0.66742E+00_JPRB, 0.68613E+00_JPRB, 0.69046E+00_JPRB, 0.68916E+00_JPRB ,&
&  0.67833E+00_JPRB, 0.68487E+00_JPRB, 0.70693E+00_JPRB, 0.76020E+00_JPRB, 0.81630E+00_JPRB, 0.87070E+00_JPRB, 0.92879E+00_JPRB ,&
&  0.10023E+01_JPRB, 0.10468E+01_JPRB, 0.10309E+01_JPRB, 0.10340E+01_JPRB, 0.10607E+01_JPRB, 0.10906E+01_JPRB, 0.10992E+01_JPRB ,&
&  0.10951E+01_JPRB, 0.11096E+01_JPRB, 0.11277E+01_JPRB, 0.11674E+01_JPRB, 0.12166E+01_JPRB, 0.12482E+01_JPRB, 0.88695E+00_JPRB ,&
&  0.10718E+01_JPRB, 0.10236E+01_JPRB, 0.10009E+01_JPRB, 0.10149E+01_JPRB, 0.97710E+00_JPRB, 0.91827E+00_JPRB, 0.90888E+00_JPRB ,&
&  0.95249E+00_JPRB, 0.10221E+01_JPRB, 0.11044E+01_JPRB, 0.11765E+01_JPRB, 0.13077E+01_JPRB, 0.13339E+01_JPRB, 0.15072E+01_JPRB ,&
&  0.16544E+01_JPRB, 0.17369E+01_JPRB, 0.17778E+01_JPRB, 0.18154E+01_JPRB, 0.18636E+01_JPRB, 0.19394E+01_JPRB, 0.20259E+01_JPRB ,&
&  0.20762E+01_JPRB, 0.20996E+01_JPRB, 0.21297E+01_JPRB, 0.21557E+01_JPRB, 0.19227E+01_JPRB, 0.20026E+01_JPRB, 0.16008E+01_JPRB ,&
&  0.10802E+01_JPRB, 0.88284E+00_JPRB, 0.10675E+01_JPRB, 0.11471E+01_JPRB, 0.10574E+01_JPRB, 0.97720E+00_JPRB, 0.87161E+00_JPRB ,&
&  0.75428E+00_JPRB, 0.68344E+00_JPRB, 0.64161E+00_JPRB, 0.60373E+00_JPRB, 0.56993E+00_JPRB, 0.57242E+00_JPRB, 0.58823E+00_JPRB ,&
&  0.59548E+00_JPRB, 0.59256E+00_JPRB, 0.58905E+00_JPRB, 0.60936E+00_JPRB, 0.66125E+00_JPRB, 0.81488E+00_JPRB, 0.10827E+01_JPRB ,&
&  0.12985E+01_JPRB, 0.64677E+00_JPRB, 0.57669E+00_JPRB, 0.58424E+00_JPRB, 0.60907E+00_JPRB, 0.64635E+00_JPRB, 0.70616E+00_JPRB ,&
&  0.86305E+00_JPRB, 0.11004E+01_JPRB, 0.10259E+01_JPRB, 0.12274E+01_JPRB, 0.25080E+01_JPRB, 0.27426E+01_JPRB, 0.15807E+01_JPRB ,&
&  0.99375E+00_JPRB, 0.70271E+00_JPRB, 0.56954E+00_JPRB, 0.52677E+00_JPRB, 0.51562E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.53058E+00_JPRB, 0.54535E+00_JPRB, 0.56559E+00_JPRB, 0.58950E+00_JPRB, 0.58613E+00_JPRB, 0.58667E+00_JPRB, 0.57413E+00_JPRB ,&
&  0.57158E+00_JPRB, 0.58953E+00_JPRB, 0.63470E+00_JPRB, 0.71087E+00_JPRB, 0.76666E+00_JPRB, 0.84396E+00_JPRB, 0.94791E+00_JPRB ,&
&  0.97803E+00_JPRB, 0.95701E+00_JPRB, 0.94361E+00_JPRB, 0.94375E+00_JPRB, 0.96048E+00_JPRB, 0.98652E+00_JPRB, 0.10093E+01_JPRB ,&
&  0.10321E+01_JPRB, 0.10847E+01_JPRB, 0.11282E+01_JPRB, 0.11915E+01_JPRB, 0.12511E+01_JPRB, 0.12697E+01_JPRB, 0.12976E+01_JPRB ,&
&  0.76793E+00_JPRB, 0.10640E+01_JPRB, 0.10348E+01_JPRB, 0.10108E+01_JPRB, 0.88238E+00_JPRB, 0.79622E+00_JPRB, 0.81040E+00_JPRB ,&
&  0.93231E+00_JPRB, 0.10703E+01_JPRB, 0.12685E+01_JPRB, 0.13202E+01_JPRB, 0.14760E+01_JPRB, 0.15805E+01_JPRB, 0.17792E+01_JPRB ,&
&  0.18628E+01_JPRB, 0.19431E+01_JPRB, 0.20440E+01_JPRB, 0.21548E+01_JPRB, 0.22703E+01_JPRB, 0.24180E+01_JPRB, 0.25561E+01_JPRB ,&
&  0.26626E+01_JPRB, 0.27047E+01_JPRB, 0.27921E+01_JPRB, 0.25072E+01_JPRB, 0.21795E+01_JPRB, 0.22615E+01_JPRB, 0.21845E+01_JPRB ,&
&  0.18172E+01_JPRB, 0.12560E+01_JPRB, 0.12037E+01_JPRB, 0.12435E+01_JPRB, 0.11313E+01_JPRB, 0.10461E+01_JPRB, 0.93890E+00_JPRB ,&
&  0.80673E+00_JPRB, 0.68313E+00_JPRB, 0.62827E+00_JPRB, 0.58523E+00_JPRB, 0.57035E+00_JPRB, 0.59969E+00_JPRB, 0.63919E+00_JPRB ,&
&  0.63704E+00_JPRB, 0.62623E+00_JPRB, 0.61695E+00_JPRB, 0.63179E+00_JPRB, 0.67171E+00_JPRB, 0.70834E+00_JPRB, 0.73822E+00_JPRB ,&
&  0.70828E+00_JPRB, 0.66878E+00_JPRB, 0.66203E+00_JPRB, 0.67359E+00_JPRB, 0.69561E+00_JPRB, 0.71732E+00_JPRB, 0.75193E+00_JPRB ,&
&  0.77161E+00_JPRB, 0.77110E+00_JPRB, 0.86195E+00_JPRB, 0.10547E+01_JPRB, 0.11705E+01_JPRB, 0.12552E+01_JPRB, 0.10363E+01_JPRB ,&
&  0.82765E+00_JPRB, 0.69614E+00_JPRB, 0.61529E+00_JPRB, 0.56179E+00_JPRB, 0.51980E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.52096E+00_JPRB, 0.52238E+00_JPRB, 0.51968E+00_JPRB, 0.52930E+00_JPRB, 0.52765E+00_JPRB, 0.52779E+00_JPRB, 0.52428E+00_JPRB ,&
&  0.54824E+00_JPRB, 0.59520E+00_JPRB, 0.66687E+00_JPRB, 0.74771E+00_JPRB, 0.80616E+00_JPRB, 0.87850E+00_JPRB, 0.91133E+00_JPRB ,&
&  0.90293E+00_JPRB, 0.88021E+00_JPRB, 0.87320E+00_JPRB, 0.89194E+00_JPRB, 0.90435E+00_JPRB, 0.92989E+00_JPRB, 0.95825E+00_JPRB ,&
&  0.98845E+00_JPRB, 0.10344E+01_JPRB, 0.10902E+01_JPRB, 0.11557E+01_JPRB, 0.12360E+01_JPRB, 0.12780E+01_JPRB, 0.12709E+01_JPRB ,&
&  0.48729E+00_JPRB, 0.68198E+00_JPRB, 0.97297E+00_JPRB, 0.10207E+01_JPRB, 0.79547E+00_JPRB, 0.70708E+00_JPRB, 0.64855E+00_JPRB ,&
&  0.77307E+00_JPRB, 0.10102E+01_JPRB, 0.12287E+01_JPRB, 0.15077E+01_JPRB, 0.15697E+01_JPRB, 0.17306E+01_JPRB, 0.18593E+01_JPRB ,&
&  0.19060E+01_JPRB, 0.19671E+01_JPRB, 0.20810E+01_JPRB, 0.22211E+01_JPRB, 0.23525E+01_JPRB, 0.24834E+01_JPRB, 0.26217E+01_JPRB ,&
&  0.27726E+01_JPRB, 0.28584E+01_JPRB, 0.29649E+01_JPRB, 0.25183E+01_JPRB, 0.26004E+01_JPRB, 0.27434E+01_JPRB, 0.31216E+01_JPRB ,&
&  0.26170E+01_JPRB, 0.16076E+01_JPRB, 0.12590E+01_JPRB, 0.13172E+01_JPRB, 0.12024E+01_JPRB, 0.11242E+01_JPRB, 0.98740E+00_JPRB ,&
&  0.84104E+00_JPRB, 0.72839E+00_JPRB, 0.66656E+00_JPRB, 0.64654E+00_JPRB, 0.65264E+00_JPRB, 0.69056E+00_JPRB, 0.72057E+00_JPRB ,&
&  0.70854E+00_JPRB, 0.69713E+00_JPRB, 0.68775E+00_JPRB, 0.68374E+00_JPRB, 0.71929E+00_JPRB, 0.75543E+00_JPRB, 0.77059E+00_JPRB ,&
&  0.77045E+00_JPRB, 0.77009E+00_JPRB, 0.78218E+00_JPRB, 0.80260E+00_JPRB, 0.82744E+00_JPRB, 0.84136E+00_JPRB, 0.84741E+00_JPRB ,&
&  0.76020E+00_JPRB, 0.71592E+00_JPRB, 0.71783E+00_JPRB, 0.71502E+00_JPRB, 0.73720E+00_JPRB, 0.77106E+00_JPRB, 0.72579E+00_JPRB ,&
&  0.69709E+00_JPRB, 0.68547E+00_JPRB, 0.65737E+00_JPRB, 0.59466E+00_JPRB, 0.53303E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.53589E+00_JPRB, 0.53650E+00_JPRB, 0.52686E+00_JPRB, 0.52504E+00_JPRB, 0.51718E+00_JPRB, 0.52303E+00_JPRB, 0.53751E+00_JPRB ,&
&  0.56823E+00_JPRB, 0.62108E+00_JPRB, 0.68581E+00_JPRB, 0.74359E+00_JPRB, 0.81136E+00_JPRB, 0.86220E+00_JPRB, 0.86199E+00_JPRB ,&
&  0.84300E+00_JPRB, 0.83731E+00_JPRB, 0.82297E+00_JPRB, 0.84392E+00_JPRB, 0.86250E+00_JPRB, 0.89263E+00_JPRB, 0.92766E+00_JPRB ,&
&  0.96980E+00_JPRB, 0.10231E+01_JPRB, 0.10892E+01_JPRB, 0.11664E+01_JPRB, 0.12766E+01_JPRB, 0.13973E+01_JPRB, 0.14989E+01_JPRB ,&
&  0.99196E+00_JPRB, 0.51683E+00_JPRB, 0.89688E+00_JPRB, 0.10488E+01_JPRB, 0.82137E+00_JPRB, 0.72213E+00_JPRB, 0.61311E+00_JPRB ,&
&  0.60911E+00_JPRB, 0.90526E+00_JPRB, 0.11759E+01_JPRB, 0.14363E+01_JPRB, 0.15332E+01_JPRB, 0.16861E+01_JPRB, 0.17997E+01_JPRB ,&
&  0.18586E+01_JPRB, 0.19209E+01_JPRB, 0.19938E+01_JPRB, 0.20775E+01_JPRB, 0.21879E+01_JPRB, 0.23047E+01_JPRB, 0.24545E+01_JPRB ,&
&  0.26388E+01_JPRB, 0.27636E+01_JPRB, 0.28536E+01_JPRB, 0.24604E+01_JPRB, 0.26205E+01_JPRB, 0.24976E+01_JPRB, 0.23941E+01_JPRB ,&
&  0.21875E+01_JPRB, 0.13633E+01_JPRB, 0.10716E+01_JPRB, 0.13025E+01_JPRB, 0.12143E+01_JPRB, 0.94418E+00_JPRB, 0.10110E+01_JPRB ,&
&  0.89176E+00_JPRB, 0.78517E+00_JPRB, 0.75888E+00_JPRB, 0.75769E+00_JPRB, 0.73357E+00_JPRB, 0.74793E+00_JPRB, 0.75379E+00_JPRB ,&
&  0.77724E+00_JPRB, 0.76192E+00_JPRB, 0.73945E+00_JPRB, 0.74930E+00_JPRB, 0.77296E+00_JPRB, 0.80444E+00_JPRB, 0.82607E+00_JPRB ,&
&  0.85004E+00_JPRB, 0.86691E+00_JPRB, 0.88179E+00_JPRB, 0.90521E+00_JPRB, 0.92513E+00_JPRB, 0.87321E+00_JPRB, 0.85112E+00_JPRB ,&
&  0.78455E+00_JPRB, 0.76208E+00_JPRB, 0.74664E+00_JPRB, 0.73668E+00_JPRB, 0.72874E+00_JPRB, 0.74131E+00_JPRB, 0.77270E+00_JPRB ,&
&  0.80374E+00_JPRB, 0.80972E+00_JPRB, 0.74186E+00_JPRB, 0.62296E+00_JPRB, 0.55616E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.53086E+00_JPRB, 0.52434E+00_JPRB, 0.51368E+00_JPRB, 0.52621E+00_JPRB, 0.52532E+00_JPRB, 0.53383E+00_JPRB, 0.55249E+00_JPRB ,&
&  0.58271E+00_JPRB, 0.62058E+00_JPRB, 0.64957E+00_JPRB, 0.69330E+00_JPRB, 0.75300E+00_JPRB, 0.80604E+00_JPRB, 0.80774E+00_JPRB ,&
&  0.79311E+00_JPRB, 0.78567E+00_JPRB, 0.78742E+00_JPRB, 0.79344E+00_JPRB, 0.81728E+00_JPRB, 0.84934E+00_JPRB, 0.88065E+00_JPRB ,&
&  0.92372E+00_JPRB, 0.98323E+00_JPRB, 0.10447E+01_JPRB, 0.11213E+01_JPRB, 0.12140E+01_JPRB, 0.13621E+01_JPRB, 0.15547E+01_JPRB ,&
&  0.20893E+01_JPRB, 0.19651E+01_JPRB, 0.81387E+00_JPRB, 0.10601E+01_JPRB, 0.94368E+00_JPRB, 0.79841E+00_JPRB, 0.64840E+00_JPRB ,&
&  0.60022E+00_JPRB, 0.76181E+00_JPRB, 0.10770E+01_JPRB, 0.13566E+01_JPRB, 0.13772E+01_JPRB, 0.15024E+01_JPRB, 0.16118E+01_JPRB ,&
&  0.17313E+01_JPRB, 0.18088E+01_JPRB, 0.18423E+01_JPRB, 0.18556E+01_JPRB, 0.19025E+01_JPRB, 0.19865E+01_JPRB, 0.21410E+01_JPRB ,&
&  0.23710E+01_JPRB, 0.26150E+01_JPRB, 0.27943E+01_JPRB, 0.25733E+01_JPRB, 0.26757E+01_JPRB, 0.23438E+01_JPRB, 0.20878E+01_JPRB ,&
&  0.16339E+01_JPRB, 0.13958E+01_JPRB, 0.13458E+01_JPRB, 0.13476E+01_JPRB, 0.11649E+01_JPRB, 0.10364E+01_JPRB, 0.11076E+01_JPRB ,&
&  0.10249E+01_JPRB, 0.91187E+00_JPRB, 0.84865E+00_JPRB, 0.80832E+00_JPRB, 0.79579E+00_JPRB, 0.79595E+00_JPRB, 0.80416E+00_JPRB ,&
&  0.82632E+00_JPRB, 0.82589E+00_JPRB, 0.80057E+00_JPRB, 0.82170E+00_JPRB, 0.84499E+00_JPRB, 0.86874E+00_JPRB, 0.88626E+00_JPRB ,&
&  0.90912E+00_JPRB, 0.93416E+00_JPRB, 0.94875E+00_JPRB, 0.95383E+00_JPRB, 0.90131E+00_JPRB, 0.84503E+00_JPRB, 0.85176E+00_JPRB ,&
&  0.81064E+00_JPRB, 0.81102E+00_JPRB, 0.81722E+00_JPRB, 0.82085E+00_JPRB, 0.80757E+00_JPRB, 0.83258E+00_JPRB, 0.84822E+00_JPRB ,&
&  0.88792E+00_JPRB, 0.92231E+00_JPRB, 0.84081E+00_JPRB, 0.64113E+00_JPRB, 0.56397E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.56580E+00_JPRB, 0.55706E+00_JPRB, 0.55053E+00_JPRB, 0.55561E+00_JPRB, 0.55949E+00_JPRB, 0.55196E+00_JPRB, 0.55452E+00_JPRB ,&
&  0.56438E+00_JPRB, 0.57762E+00_JPRB, 0.57604E+00_JPRB, 0.60781E+00_JPRB, 0.67925E+00_JPRB, 0.75271E+00_JPRB, 0.75442E+00_JPRB ,&
&  0.74074E+00_JPRB, 0.72611E+00_JPRB, 0.71889E+00_JPRB, 0.73111E+00_JPRB, 0.74288E+00_JPRB, 0.78198E+00_JPRB, 0.82124E+00_JPRB ,&
&  0.85949E+00_JPRB, 0.89846E+00_JPRB, 0.93519E+00_JPRB, 0.97476E+00_JPRB, 0.10422E+01_JPRB, 0.11394E+01_JPRB, 0.13948E+01_JPRB ,&
&  0.18567E+01_JPRB, 0.33141E+01_JPRB, 0.10503E+01_JPRB, 0.10673E+01_JPRB, 0.10083E+01_JPRB, 0.88375E+00_JPRB, 0.77938E+00_JPRB ,&
&  0.73879E+00_JPRB, 0.70153E+00_JPRB, 0.97822E+00_JPRB, 0.12579E+01_JPRB, 0.12206E+01_JPRB, 0.12599E+01_JPRB, 0.13322E+01_JPRB ,&
&  0.14888E+01_JPRB, 0.15980E+01_JPRB, 0.16627E+01_JPRB, 0.16480E+01_JPRB, 0.16258E+01_JPRB, 0.16151E+01_JPRB, 0.16638E+01_JPRB ,&
&  0.18475E+01_JPRB, 0.21933E+01_JPRB, 0.25533E+01_JPRB, 0.25903E+01_JPRB, 0.27703E+01_JPRB, 0.24479E+01_JPRB, 0.22316E+01_JPRB ,&
&  0.18550E+01_JPRB, 0.17105E+01_JPRB, 0.17115E+01_JPRB, 0.15794E+01_JPRB, 0.13028E+01_JPRB, 0.11753E+01_JPRB, 0.11899E+01_JPRB ,&
&  0.11320E+01_JPRB, 0.10257E+01_JPRB, 0.92398E+00_JPRB, 0.84546E+00_JPRB, 0.83592E+00_JPRB, 0.86338E+00_JPRB, 0.84608E+00_JPRB ,&
&  0.87395E+00_JPRB, 0.91097E+00_JPRB, 0.88709E+00_JPRB, 0.91837E+00_JPRB, 0.93554E+00_JPRB, 0.94119E+00_JPRB, 0.94095E+00_JPRB ,&
&  0.94326E+00_JPRB, 0.96686E+00_JPRB, 0.97363E+00_JPRB, 0.93500E+00_JPRB, 0.85616E+00_JPRB, 0.80746E+00_JPRB, 0.81234E+00_JPRB ,&
&  0.81743E+00_JPRB, 0.84498E+00_JPRB, 0.84675E+00_JPRB, 0.81999E+00_JPRB, 0.85228E+00_JPRB, 0.85035E+00_JPRB, 0.86077E+00_JPRB ,&
&  0.89362E+00_JPRB, 0.96783E+00_JPRB, 0.10134E+01_JPRB, 0.72267E+00_JPRB, 0.61779E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.63163E+00_JPRB, 0.61249E+00_JPRB, 0.59320E+00_JPRB, 0.57626E+00_JPRB, 0.56652E+00_JPRB, 0.57215E+00_JPRB, 0.57647E+00_JPRB ,&
&  0.57069E+00_JPRB, 0.56997E+00_JPRB, 0.56429E+00_JPRB, 0.56932E+00_JPRB, 0.59360E+00_JPRB, 0.64027E+00_JPRB, 0.68669E+00_JPRB ,&
&  0.68460E+00_JPRB, 0.65142E+00_JPRB, 0.64607E+00_JPRB, 0.64083E+00_JPRB, 0.65827E+00_JPRB, 0.68115E+00_JPRB, 0.72378E+00_JPRB ,&
&  0.77969E+00_JPRB, 0.82106E+00_JPRB, 0.84297E+00_JPRB, 0.85613E+00_JPRB, 0.88309E+00_JPRB, 0.92052E+00_JPRB, 0.11610E+01_JPRB ,&
&  0.16712E+01_JPRB, 0.33682E+01_JPRB, 0.26417E+01_JPRB, 0.16520E+01_JPRB, 0.10909E+01_JPRB, 0.90694E+00_JPRB, 0.87891E+00_JPRB ,&
&  0.93933E+00_JPRB, 0.90205E+00_JPRB, 0.10970E+01_JPRB, 0.11565E+01_JPRB, 0.10945E+01_JPRB, 0.10418E+01_JPRB, 0.10853E+01_JPRB ,&
&  0.12107E+01_JPRB, 0.13309E+01_JPRB, 0.14145E+01_JPRB, 0.14302E+01_JPRB, 0.13963E+01_JPRB, 0.13704E+01_JPRB, 0.13582E+01_JPRB ,&
&  0.13592E+01_JPRB, 0.14976E+01_JPRB, 0.19995E+01_JPRB, 0.25199E+01_JPRB, 0.28279E+01_JPRB, 0.26351E+01_JPRB, 0.25007E+01_JPRB ,&
&  0.24490E+01_JPRB, 0.22053E+01_JPRB, 0.20515E+01_JPRB, 0.17773E+01_JPRB, 0.14289E+01_JPRB, 0.14541E+01_JPRB, 0.12556E+01_JPRB ,&
&  0.11783E+01_JPRB, 0.10999E+01_JPRB, 0.10086E+01_JPRB, 0.90614E+00_JPRB, 0.90322E+00_JPRB, 0.91156E+00_JPRB, 0.88756E+00_JPRB ,&
&  0.91686E+00_JPRB, 0.95315E+00_JPRB, 0.93529E+00_JPRB, 0.94209E+00_JPRB, 0.94556E+00_JPRB, 0.95695E+00_JPRB, 0.96259E+00_JPRB ,&
&  0.96451E+00_JPRB, 0.98761E+00_JPRB, 0.92425E+00_JPRB, 0.85965E+00_JPRB, 0.82675E+00_JPRB, 0.78624E+00_JPRB, 0.77375E+00_JPRB ,&
&  0.80177E+00_JPRB, 0.86045E+00_JPRB, 0.86357E+00_JPRB, 0.86980E+00_JPRB, 0.85571E+00_JPRB, 0.85678E+00_JPRB, 0.82850E+00_JPRB ,&
&  0.82469E+00_JPRB, 0.81574E+00_JPRB, 0.84993E+00_JPRB, 0.78146E+00_JPRB, 0.69046E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.67805E+00_JPRB, 0.65553E+00_JPRB, 0.60842E+00_JPRB, 0.56922E+00_JPRB, 0.56441E+00_JPRB, 0.56972E+00_JPRB, 0.57889E+00_JPRB ,&
&  0.58073E+00_JPRB, 0.57965E+00_JPRB, 0.57567E+00_JPRB, 0.56936E+00_JPRB, 0.57524E+00_JPRB, 0.56646E+00_JPRB, 0.59065E+00_JPRB ,&
&  0.61350E+00_JPRB, 0.61072E+00_JPRB, 0.61827E+00_JPRB, 0.59731E+00_JPRB, 0.59592E+00_JPRB, 0.59187E+00_JPRB, 0.61587E+00_JPRB ,&
&  0.67232E+00_JPRB, 0.73672E+00_JPRB, 0.77447E+00_JPRB, 0.78750E+00_JPRB, 0.80519E+00_JPRB, 0.84768E+00_JPRB, 0.10063E+01_JPRB ,&
&  0.14255E+01_JPRB, 0.29057E+01_JPRB, 0.14653E+01_JPRB, 0.19762E+01_JPRB, 0.13020E+01_JPRB, 0.98452E+00_JPRB, 0.83561E+00_JPRB ,&
&  0.90008E+00_JPRB, 0.10892E+01_JPRB, 0.10301E+01_JPRB, 0.10406E+01_JPRB, 0.98692E+00_JPRB, 0.94916E+00_JPRB, 0.94070E+00_JPRB ,&
&  0.97749E+00_JPRB, 0.10821E+01_JPRB, 0.11843E+01_JPRB, 0.12301E+01_JPRB, 0.12020E+01_JPRB, 0.11429E+01_JPRB, 0.11303E+01_JPRB ,&
&  0.11265E+01_JPRB, 0.11410E+01_JPRB, 0.13552E+01_JPRB, 0.16742E+01_JPRB, 0.24806E+01_JPRB, 0.27830E+01_JPRB, 0.28770E+01_JPRB ,&
&  0.29616E+01_JPRB, 0.27824E+01_JPRB, 0.23739E+01_JPRB, 0.20003E+01_JPRB, 0.16765E+01_JPRB, 0.15863E+01_JPRB, 0.13742E+01_JPRB ,&
&  0.12392E+01_JPRB, 0.11465E+01_JPRB, 0.10617E+01_JPRB, 0.97915E+00_JPRB, 0.93741E+00_JPRB, 0.92881E+00_JPRB, 0.90739E+00_JPRB ,&
&  0.94003E+00_JPRB, 0.98028E+00_JPRB, 0.97007E+00_JPRB, 0.94878E+00_JPRB, 0.94054E+00_JPRB, 0.94650E+00_JPRB, 0.94877E+00_JPRB ,&
&  0.96327E+00_JPRB, 0.99208E+00_JPRB, 0.92661E+00_JPRB, 0.82261E+00_JPRB, 0.80149E+00_JPRB, 0.80083E+00_JPRB, 0.79321E+00_JPRB ,&
&  0.84065E+00_JPRB, 0.88584E+00_JPRB, 0.88445E+00_JPRB, 0.85523E+00_JPRB, 0.86005E+00_JPRB, 0.89511E+00_JPRB, 0.84308E+00_JPRB ,&
&  0.81196E+00_JPRB, 0.79481E+00_JPRB, 0.78698E+00_JPRB, 0.77112E+00_JPRB, 0.73300E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.68009E+00_JPRB, 0.63893E+00_JPRB, 0.58989E+00_JPRB, 0.56743E+00_JPRB, 0.57305E+00_JPRB, 0.55981E+00_JPRB, 0.56647E+00_JPRB ,&
&  0.58471E+00_JPRB, 0.58500E+00_JPRB, 0.58423E+00_JPRB, 0.57465E+00_JPRB, 0.56711E+00_JPRB, 0.55317E+00_JPRB, 0.54486E+00_JPRB ,&
&  0.56051E+00_JPRB, 0.60086E+00_JPRB, 0.61444E+00_JPRB, 0.60321E+00_JPRB, 0.59363E+00_JPRB, 0.57701E+00_JPRB, 0.57212E+00_JPRB ,&
&  0.59657E+00_JPRB, 0.64393E+00_JPRB, 0.69302E+00_JPRB, 0.71874E+00_JPRB, 0.73758E+00_JPRB, 0.76073E+00_JPRB, 0.86394E+00_JPRB ,&
&  0.11460E+01_JPRB, 0.27049E+01_JPRB, 0.12036E+01_JPRB, 0.20148E+01_JPRB, 0.14682E+01_JPRB, 0.11389E+01_JPRB, 0.89703E+00_JPRB ,&
&  0.89068E+00_JPRB, 0.98840E+00_JPRB, 0.95773E+00_JPRB, 0.96880E+00_JPRB, 0.94102E+00_JPRB, 0.91068E+00_JPRB, 0.90721E+00_JPRB ,&
&  0.90558E+00_JPRB, 0.91458E+00_JPRB, 0.95733E+00_JPRB, 0.99494E+00_JPRB, 0.99969E+00_JPRB, 0.96891E+00_JPRB, 0.93969E+00_JPRB ,&
&  0.93008E+00_JPRB, 0.93621E+00_JPRB, 0.10470E+01_JPRB, 0.13068E+01_JPRB, 0.18391E+01_JPRB, 0.25794E+01_JPRB, 0.33957E+01_JPRB ,&
&  0.37507E+01_JPRB, 0.34928E+01_JPRB, 0.28412E+01_JPRB, 0.23675E+01_JPRB, 0.20296E+01_JPRB, 0.18704E+01_JPRB, 0.16204E+01_JPRB ,&
&  0.14180E+01_JPRB, 0.12828E+01_JPRB, 0.11723E+01_JPRB, 0.11003E+01_JPRB, 0.10313E+01_JPRB, 0.10089E+01_JPRB, 0.98240E+00_JPRB ,&
&  0.98826E+00_JPRB, 0.10371E+01_JPRB, 0.10330E+01_JPRB, 0.99813E+00_JPRB, 0.94924E+00_JPRB, 0.93351E+00_JPRB, 0.91887E+00_JPRB ,&
&  0.92340E+00_JPRB, 0.95774E+00_JPRB, 0.93257E+00_JPRB, 0.85050E+00_JPRB, 0.81306E+00_JPRB, 0.81477E+00_JPRB, 0.82077E+00_JPRB ,&
&  0.87551E+00_JPRB, 0.91453E+00_JPRB, 0.90853E+00_JPRB, 0.89817E+00_JPRB, 0.88174E+00_JPRB, 0.92702E+00_JPRB, 0.86852E+00_JPRB ,&
&  0.81279E+00_JPRB, 0.78825E+00_JPRB, 0.76994E+00_JPRB, 0.73812E+00_JPRB, 0.72846E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.68353E+00_JPRB, 0.63223E+00_JPRB, 0.59808E+00_JPRB, 0.57353E+00_JPRB, 0.57612E+00_JPRB, 0.57041E+00_JPRB, 0.56636E+00_JPRB ,&
&  0.58290E+00_JPRB, 0.59267E+00_JPRB, 0.60496E+00_JPRB, 0.59421E+00_JPRB, 0.57272E+00_JPRB, 0.56700E+00_JPRB, 0.55783E+00_JPRB ,&
&  0.55098E+00_JPRB, 0.58000E+00_JPRB, 0.60833E+00_JPRB, 0.61822E+00_JPRB, 0.60680E+00_JPRB, 0.57846E+00_JPRB, 0.56395E+00_JPRB ,&
&  0.57030E+00_JPRB, 0.58799E+00_JPRB, 0.61571E+00_JPRB, 0.64546E+00_JPRB, 0.67082E+00_JPRB, 0.69440E+00_JPRB, 0.72948E+00_JPRB ,&
&  0.84000E+00_JPRB, 0.10488E+01_JPRB, 0.15310E+01_JPRB, 0.18985E+01_JPRB, 0.15423E+01_JPRB, 0.12126E+01_JPRB, 0.10027E+01_JPRB ,&
&  0.97029E+00_JPRB, 0.92954E+00_JPRB, 0.90370E+00_JPRB, 0.88513E+00_JPRB, 0.85671E+00_JPRB, 0.83726E+00_JPRB, 0.83861E+00_JPRB ,&
&  0.82414E+00_JPRB, 0.81026E+00_JPRB, 0.79836E+00_JPRB, 0.79787E+00_JPRB, 0.80033E+00_JPRB, 0.79668E+00_JPRB, 0.78829E+00_JPRB ,&
&  0.79073E+00_JPRB, 0.79944E+00_JPRB, 0.84676E+00_JPRB, 0.10372E+01_JPRB, 0.15824E+01_JPRB, 0.23188E+01_JPRB, 0.34537E+01_JPRB ,&
&  0.42722E+01_JPRB, 0.38667E+01_JPRB, 0.32306E+01_JPRB, 0.27202E+01_JPRB, 0.23544E+01_JPRB, 0.21503E+01_JPRB, 0.19373E+01_JPRB ,&
&  0.17430E+01_JPRB, 0.15436E+01_JPRB, 0.13789E+01_JPRB, 0.12744E+01_JPRB, 0.11598E+01_JPRB, 0.11048E+01_JPRB, 0.11017E+01_JPRB ,&
&  0.10742E+01_JPRB, 0.10704E+01_JPRB, 0.10705E+01_JPRB, 0.10983E+01_JPRB, 0.10463E+01_JPRB, 0.98864E+00_JPRB, 0.93676E+00_JPRB ,&
&  0.90435E+00_JPRB, 0.92132E+00_JPRB, 0.91272E+00_JPRB, 0.88095E+00_JPRB, 0.86781E+00_JPRB, 0.86749E+00_JPRB, 0.87218E+00_JPRB ,&
&  0.88661E+00_JPRB, 0.91299E+00_JPRB, 0.92198E+00_JPRB, 0.92750E+00_JPRB, 0.87206E+00_JPRB, 0.93710E+00_JPRB, 0.88247E+00_JPRB ,&
&  0.82923E+00_JPRB, 0.79570E+00_JPRB, 0.77468E+00_JPRB, 0.74927E+00_JPRB, 0.73843E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.74077E+00_JPRB, 0.68780E+00_JPRB, 0.64053E+00_JPRB, 0.61177E+00_JPRB, 0.58811E+00_JPRB, 0.57893E+00_JPRB, 0.57324E+00_JPRB ,&
&  0.58130E+00_JPRB, 0.58867E+00_JPRB, 0.60322E+00_JPRB, 0.60060E+00_JPRB, 0.58111E+00_JPRB, 0.57808E+00_JPRB, 0.59076E+00_JPRB ,&
&  0.58651E+00_JPRB, 0.59160E+00_JPRB, 0.60222E+00_JPRB, 0.59485E+00_JPRB, 0.58212E+00_JPRB, 0.56754E+00_JPRB, 0.55348E+00_JPRB ,&
&  0.55690E+00_JPRB, 0.56317E+00_JPRB, 0.57666E+00_JPRB, 0.59773E+00_JPRB, 0.62550E+00_JPRB, 0.65345E+00_JPRB, 0.67414E+00_JPRB ,&
&  0.69268E+00_JPRB, 0.96964E+00_JPRB, 0.13630E+01_JPRB, 0.16303E+01_JPRB, 0.15111E+01_JPRB, 0.12217E+01_JPRB, 0.10791E+01_JPRB ,&
&  0.96375E+00_JPRB, 0.90202E+00_JPRB, 0.86239E+00_JPRB, 0.83656E+00_JPRB, 0.79919E+00_JPRB, 0.78039E+00_JPRB, 0.77915E+00_JPRB ,&
&  0.76449E+00_JPRB, 0.73575E+00_JPRB, 0.70503E+00_JPRB, 0.68197E+00_JPRB, 0.67558E+00_JPRB, 0.68099E+00_JPRB, 0.68768E+00_JPRB ,&
&  0.69486E+00_JPRB, 0.70565E+00_JPRB, 0.73719E+00_JPRB, 0.81724E+00_JPRB, 0.12468E+01_JPRB, 0.15111E+01_JPRB, 0.27030E+01_JPRB ,&
&  0.33446E+01_JPRB, 0.29044E+01_JPRB, 0.26198E+01_JPRB, 0.24205E+01_JPRB, 0.22116E+01_JPRB, 0.20218E+01_JPRB, 0.19050E+01_JPRB ,&
&  0.17841E+01_JPRB, 0.16416E+01_JPRB, 0.15208E+01_JPRB, 0.14352E+01_JPRB, 0.13511E+01_JPRB, 0.12884E+01_JPRB, 0.12060E+01_JPRB ,&
&  0.11708E+01_JPRB, 0.11274E+01_JPRB, 0.11077E+01_JPRB, 0.11251E+01_JPRB, 0.11363E+01_JPRB, 0.10826E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.95254E+00_JPRB, 0.92241E+00_JPRB, 0.90835E+00_JPRB, 0.89224E+00_JPRB, 0.90282E+00_JPRB, 0.92075E+00_JPRB, 0.92416E+00_JPRB ,&
&  0.90041E+00_JPRB, 0.90020E+00_JPRB, 0.92668E+00_JPRB, 0.91939E+00_JPRB, 0.87819E+00_JPRB, 0.94848E+00_JPRB, 0.90293E+00_JPRB ,&
&  0.84964E+00_JPRB, 0.80069E+00_JPRB, 0.76969E+00_JPRB, 0.74017E+00_JPRB, 0.76423E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.78022E+00_JPRB, 0.74206E+00_JPRB, 0.69973E+00_JPRB, 0.66968E+00_JPRB, 0.64375E+00_JPRB, 0.62467E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.61714E+00_JPRB, 0.60873E+00_JPRB, 0.61517E+00_JPRB, 0.61423E+00_JPRB, 0.60971E+00_JPRB, 0.60061E+00_JPRB, 0.61006E+00_JPRB ,&
&  0.61917E+00_JPRB, 0.61765E+00_JPRB, 0.61149E+00_JPRB, 0.60116E+00_JPRB, 0.59003E+00_JPRB, 0.57587E+00_JPRB, 0.56720E+00_JPRB ,&
&  0.56157E+00_JPRB, 0.56283E+00_JPRB, 0.57050E+00_JPRB, 0.58430E+00_JPRB, 0.60783E+00_JPRB, 0.63686E+00_JPRB, 0.66507E+00_JPRB ,&
&  0.65660E+00_JPRB, 0.72865E+00_JPRB, 0.95833E+00_JPRB, 0.13235E+01_JPRB, 0.14094E+01_JPRB, 0.12317E+01_JPRB, 0.10857E+01_JPRB ,&
&  0.98254E+00_JPRB, 0.90774E+00_JPRB, 0.85407E+00_JPRB, 0.81954E+00_JPRB, 0.78356E+00_JPRB, 0.76585E+00_JPRB, 0.74825E+00_JPRB ,&
&  0.72695E+00_JPRB, 0.70034E+00_JPRB, 0.66785E+00_JPRB, 0.64826E+00_JPRB, 0.64409E+00_JPRB, 0.64946E+00_JPRB, 0.64942E+00_JPRB ,&
&  0.64743E+00_JPRB, 0.65743E+00_JPRB, 0.66857E+00_JPRB, 0.69404E+00_JPRB, 0.90146E+00_JPRB, 0.11305E+01_JPRB, 0.16285E+01_JPRB ,&
&  0.16450E+01_JPRB, 0.15631E+01_JPRB, 0.15515E+01_JPRB, 0.15378E+01_JPRB, 0.15600E+01_JPRB, 0.15727E+01_JPRB, 0.16010E+01_JPRB ,&
&  0.16218E+01_JPRB, 0.16012E+01_JPRB, 0.15513E+01_JPRB, 0.15124E+01_JPRB, 0.14730E+01_JPRB, 0.14362E+01_JPRB, 0.13593E+01_JPRB ,&
&  0.12959E+01_JPRB, 0.12435E+01_JPRB, 0.12082E+01_JPRB, 0.11673E+01_JPRB, 0.11549E+01_JPRB, 0.11252E+01_JPRB, 0.10747E+01_JPRB ,&
&  0.10061E+01_JPRB, 0.94532E+00_JPRB, 0.93361E+00_JPRB, 0.92953E+00_JPRB, 0.92226E+00_JPRB, 0.91130E+00_JPRB, 0.91546E+00_JPRB ,&
&  0.92724E+00_JPRB, 0.93250E+00_JPRB, 0.92799E+00_JPRB, 0.92216E+00_JPRB, 0.88523E+00_JPRB, 0.94326E+00_JPRB, 0.91335E+00_JPRB ,&
&  0.87268E+00_JPRB, 0.82061E+00_JPRB, 0.77064E+00_JPRB, 0.74360E+00_JPRB, 0.79164E+00_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.86857E+00_JPRB, 0.82500E+00_JPRB, 0.78221E+00_JPRB, 0.73057E+00_JPRB, 0.69075E+00_JPRB, 0.67405E+00_JPRB, 0.66413E+00_JPRB ,&
&  0.64817E+00_JPRB, 0.63259E+00_JPRB, 0.63207E+00_JPRB, 0.63683E+00_JPRB, 0.63898E+00_JPRB, 0.63765E+00_JPRB, 0.62923E+00_JPRB ,&
&  0.62672E+00_JPRB, 0.63102E+00_JPRB, 0.62539E+00_JPRB, 0.61524E+00_JPRB, 0.60395E+00_JPRB, 0.59458E+00_JPRB, 0.58757E+00_JPRB ,&
&  0.58411E+00_JPRB, 0.58780E+00_JPRB, 0.59042E+00_JPRB, 0.60160E+00_JPRB, 0.62122E+00_JPRB, 0.64364E+00_JPRB, 0.67127E+00_JPRB ,&
&  0.66934E+00_JPRB, 0.59994E+00_JPRB, 0.75099E+00_JPRB, 0.95145E+00_JPRB, 0.11046E+01_JPRB, 0.11259E+01_JPRB, 0.10569E+01_JPRB ,&
&  0.10042E+01_JPRB, 0.93359E+00_JPRB, 0.87681E+00_JPRB, 0.83193E+00_JPRB, 0.79140E+00_JPRB, 0.75843E+00_JPRB, 0.73413E+00_JPRB ,&
&  0.70641E+00_JPRB, 0.68380E+00_JPRB, 0.66188E+00_JPRB, 0.64190E+00_JPRB, 0.63900E+00_JPRB, 0.63621E+00_JPRB, 0.63192E+00_JPRB ,&
&  0.62976E+00_JPRB, 0.63213E+00_JPRB, 0.63655E+00_JPRB, 0.64572E+00_JPRB, 0.67698E+00_JPRB, 0.75377E+00_JPRB, 0.89670E+00_JPRB ,&
&  0.97161E+00_JPRB, 0.10160E+01_JPRB, 0.10283E+01_JPRB, 0.10279E+01_JPRB, 0.10487E+01_JPRB, 0.10958E+01_JPRB, 0.11720E+01_JPRB ,&
&  0.12646E+01_JPRB, 0.13401E+01_JPRB, 0.13942E+01_JPRB, 0.14281E+01_JPRB, 0.14274E+01_JPRB, 0.14054E+01_JPRB, 0.13667E+01_JPRB ,&
&  0.13252E+01_JPRB, 0.12624E+01_JPRB, 0.12021E+01_JPRB, 0.11544E+01_JPRB, 0.11233E+01_JPRB, 0.11009E+01_JPRB, 0.10670E+01_JPRB ,&
&  0.10202E+01_JPRB, 0.96795E+00_JPRB, 0.93859E+00_JPRB, 0.92749E+00_JPRB, 0.92256E+00_JPRB, 0.91202E+00_JPRB, 0.91336E+00_JPRB ,&
&  0.91343E+00_JPRB, 0.92009E+00_JPRB, 0.88911E+00_JPRB, 0.90127E+00_JPRB, 0.94515E+00_JPRB, 0.91827E+00_JPRB, 0.89427E+00_JPRB ,&
&  0.86052E+00_JPRB, 0.82477E+00_JPRB, 0.78940E+00_JPRB, 0.75959E+00_JPRB, 0.85821E+00_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.80804E+00_JPRB, 0.80576E+00_JPRB, 0.77717E+00_JPRB, 0.73693E+00_JPRB, 0.69942E+00_JPRB, 0.66898E+00_JPRB, 0.65387E+00_JPRB ,&
&  0.64203E+00_JPRB, 0.63366E+00_JPRB, 0.63850E+00_JPRB, 0.64999E+00_JPRB, 0.66357E+00_JPRB, 0.66729E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.65189E+00_JPRB, 0.63553E+00_JPRB, 0.62585E+00_JPRB, 0.62050E+00_JPRB, 0.61197E+00_JPRB, 0.60877E+00_JPRB, 0.60776E+00_JPRB ,&
&  0.61165E+00_JPRB, 0.61741E+00_JPRB, 0.62091E+00_JPRB, 0.63028E+00_JPRB, 0.64533E+00_JPRB, 0.66781E+00_JPRB, 0.69061E+00_JPRB ,&
&  0.69075E+00_JPRB, 0.58742E+00_JPRB, 0.64629E+00_JPRB, 0.77694E+00_JPRB, 0.90651E+00_JPRB, 0.97616E+00_JPRB, 0.98971E+00_JPRB ,&
&  0.96993E+00_JPRB, 0.93802E+00_JPRB, 0.89578E+00_JPRB, 0.84766E+00_JPRB, 0.79937E+00_JPRB, 0.75641E+00_JPRB, 0.71564E+00_JPRB ,&
&  0.68530E+00_JPRB, 0.66370E+00_JPRB, 0.65174E+00_JPRB, 0.64359E+00_JPRB, 0.63764E+00_JPRB, 0.62420E+00_JPRB, 0.61695E+00_JPRB ,&
&  0.61869E+00_JPRB, 0.62193E+00_JPRB, 0.62687E+00_JPRB, 0.63309E+00_JPRB, 0.65263E+00_JPRB, 0.68124E+00_JPRB, 0.70948E+00_JPRB ,&
&  0.73727E+00_JPRB, 0.76645E+00_JPRB, 0.78364E+00_JPRB, 0.78810E+00_JPRB, 0.79173E+00_JPRB, 0.83308E+00_JPRB, 0.87228E+00_JPRB ,&
&  0.92561E+00_JPRB, 0.10073E+01_JPRB, 0.11037E+01_JPRB, 0.11623E+01_JPRB, 0.11711E+01_JPRB, 0.11442E+01_JPRB, 0.10981E+01_JPRB ,&
&  0.10672E+01_JPRB, 0.10324E+01_JPRB, 0.99767E+00_JPRB, 0.97039E+00_JPRB, 0.95472E+00_JPRB, 0.95215E+00_JPRB, 0.95018E+00_JPRB ,&
&  0.94509E+00_JPRB, 0.93054E+00_JPRB, 0.91754E+00_JPRB, 0.92174E+00_JPRB, 0.92954E+00_JPRB, 0.92725E+00_JPRB, 0.91632E+00_JPRB ,&
&  0.90098E+00_JPRB, 0.88778E+00_JPRB, 0.87860E+00_JPRB, 0.90128E+00_JPRB, 0.87969E+00_JPRB, 0.86688E+00_JPRB, 0.84557E+00_JPRB ,&
&  0.82539E+00_JPRB, 0.80165E+00_JPRB, 0.76949E+00_JPRB, 0.74186E+00_JPRB, 0.74042E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.72060E+00_JPRB, 0.75699E+00_JPRB, 0.76258E+00_JPRB, 0.73742E+00_JPRB, 0.70795E+00_JPRB, 0.68887E+00_JPRB, 0.67636E+00_JPRB ,&
&  0.67349E+00_JPRB, 0.66877E+00_JPRB, 0.67203E+00_JPRB, 0.68384E+00_JPRB, 0.69679E+00_JPRB, 0.70063E+00_JPRB, 0.68557E+00_JPRB ,&
&  0.66542E+00_JPRB, 0.64964E+00_JPRB, 0.64234E+00_JPRB, 0.63947E+00_JPRB, 0.63544E+00_JPRB, 0.63376E+00_JPRB, 0.64264E+00_JPRB ,&
&  0.65339E+00_JPRB, 0.66628E+00_JPRB, 0.67997E+00_JPRB, 0.69115E+00_JPRB, 0.69940E+00_JPRB, 0.70713E+00_JPRB, 0.72608E+00_JPRB ,&
&  0.73518E+00_JPRB, 0.56374E+00_JPRB, 0.58795E+00_JPRB, 0.69179E+00_JPRB, 0.77753E+00_JPRB, 0.87988E+00_JPRB, 0.93415E+00_JPRB ,&
&  0.93699E+00_JPRB, 0.91739E+00_JPRB, 0.88469E+00_JPRB, 0.84593E+00_JPRB, 0.80165E+00_JPRB, 0.76018E+00_JPRB, 0.71966E+00_JPRB ,&
&  0.69217E+00_JPRB, 0.67157E+00_JPRB, 0.65964E+00_JPRB, 0.64521E+00_JPRB, 0.63677E+00_JPRB, 0.63193E+00_JPRB, 0.62781E+00_JPRB ,&
&  0.62522E+00_JPRB, 0.62111E+00_JPRB, 0.62420E+00_JPRB, 0.63146E+00_JPRB, 0.63412E+00_JPRB, 0.64297E+00_JPRB, 0.64424E+00_JPRB ,&
&  0.66084E+00_JPRB, 0.68335E+00_JPRB, 0.69326E+00_JPRB, 0.69532E+00_JPRB, 0.70148E+00_JPRB, 0.71662E+00_JPRB, 0.73724E+00_JPRB ,&
&  0.75868E+00_JPRB, 0.80285E+00_JPRB, 0.84841E+00_JPRB, 0.89260E+00_JPRB, 0.91867E+00_JPRB, 0.92070E+00_JPRB, 0.90786E+00_JPRB ,&
&  0.90863E+00_JPRB, 0.90136E+00_JPRB, 0.88958E+00_JPRB, 0.87322E+00_JPRB, 0.85335E+00_JPRB, 0.84169E+00_JPRB, 0.84312E+00_JPRB ,&
&  0.85289E+00_JPRB, 0.86975E+00_JPRB, 0.88084E+00_JPRB, 0.89901E+00_JPRB, 0.90287E+00_JPRB, 0.89238E+00_JPRB, 0.88096E+00_JPRB ,&
&  0.86923E+00_JPRB, 0.85224E+00_JPRB, 0.83808E+00_JPRB, 0.78357E+00_JPRB, 0.82977E+00_JPRB, 0.83057E+00_JPRB, 0.81591E+00_JPRB ,&
&  0.80155E+00_JPRB, 0.77039E+00_JPRB, 0.75417E+00_JPRB, 0.60638E+00_JPRB, 0.67902E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.69499E+00_JPRB, 0.72673E+00_JPRB, 0.73703E+00_JPRB, 0.72909E+00_JPRB, 0.71084E+00_JPRB, 0.70119E+00_JPRB, 0.69535E+00_JPRB ,&
&  0.69196E+00_JPRB, 0.68890E+00_JPRB, 0.69864E+00_JPRB, 0.70662E+00_JPRB, 0.71044E+00_JPRB, 0.70438E+00_JPRB, 0.69747E+00_JPRB ,&
&  0.68537E+00_JPRB, 0.66972E+00_JPRB, 0.65864E+00_JPRB, 0.64915E+00_JPRB, 0.64871E+00_JPRB, 0.65726E+00_JPRB, 0.67399E+00_JPRB ,&
&  0.69537E+00_JPRB, 0.71047E+00_JPRB, 0.72275E+00_JPRB, 0.73549E+00_JPRB, 0.73808E+00_JPRB, 0.73846E+00_JPRB, 0.74179E+00_JPRB ,&
&  0.74817E+00_JPRB, 0.56362E+00_JPRB, 0.60309E+00_JPRB, 0.65664E+00_JPRB, 0.70431E+00_JPRB, 0.77435E+00_JPRB, 0.83417E+00_JPRB ,&
&  0.85663E+00_JPRB, 0.83440E+00_JPRB, 0.81678E+00_JPRB, 0.79398E+00_JPRB, 0.76576E+00_JPRB, 0.74013E+00_JPRB, 0.71045E+00_JPRB ,&
&  0.68821E+00_JPRB, 0.67192E+00_JPRB, 0.66489E+00_JPRB, 0.65901E+00_JPRB, 0.65242E+00_JPRB, 0.64345E+00_JPRB, 0.63259E+00_JPRB ,&
&  0.62170E+00_JPRB, 0.61697E+00_JPRB, 0.61487E+00_JPRB, 0.61298E+00_JPRB, 0.61619E+00_JPRB, 0.62179E+00_JPRB, 0.62593E+00_JPRB ,&
&  0.63567E+00_JPRB, 0.64459E+00_JPRB, 0.64869E+00_JPRB, 0.64877E+00_JPRB, 0.64868E+00_JPRB, 0.65174E+00_JPRB, 0.66837E+00_JPRB ,&
&  0.67937E+00_JPRB, 0.69817E+00_JPRB, 0.72198E+00_JPRB, 0.75914E+00_JPRB, 0.79031E+00_JPRB, 0.80200E+00_JPRB, 0.80520E+00_JPRB ,&
&  0.80563E+00_JPRB, 0.79684E+00_JPRB, 0.78051E+00_JPRB, 0.76355E+00_JPRB, 0.75122E+00_JPRB, 0.74772E+00_JPRB, 0.74371E+00_JPRB ,&
&  0.74795E+00_JPRB, 0.75790E+00_JPRB, 0.76578E+00_JPRB, 0.78033E+00_JPRB, 0.79767E+00_JPRB, 0.80334E+00_JPRB, 0.78820E+00_JPRB ,&
&  0.77474E+00_JPRB, 0.76916E+00_JPRB, 0.76409E+00_JPRB, 0.77996E+00_JPRB, 0.78229E+00_JPRB, 0.77699E+00_JPRB, 0.76027E+00_JPRB ,&
&  0.74720E+00_JPRB, 0.73700E+00_JPRB, 0.55642E+00_JPRB, 0.63966E+00_JPRB, 0.66171E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.70811E+00_JPRB, 0.70451E+00_JPRB, 0.69814E+00_JPRB, 0.70057E+00_JPRB, 0.70423E+00_JPRB, 0.70402E+00_JPRB, 0.70458E+00_JPRB ,&
&  0.70352E+00_JPRB, 0.70135E+00_JPRB, 0.71066E+00_JPRB, 0.71915E+00_JPRB, 0.72137E+00_JPRB, 0.71682E+00_JPRB, 0.71247E+00_JPRB ,&
&  0.70955E+00_JPRB, 0.70315E+00_JPRB, 0.69636E+00_JPRB, 0.69447E+00_JPRB, 0.69818E+00_JPRB, 0.71294E+00_JPRB, 0.73483E+00_JPRB ,&
&  0.75118E+00_JPRB, 0.75089E+00_JPRB, 0.75469E+00_JPRB, 0.76576E+00_JPRB, 0.76817E+00_JPRB, 0.76960E+00_JPRB, 0.77650E+00_JPRB ,&
&  0.65847E+00_JPRB, 0.53296E+00_JPRB, 0.59486E+00_JPRB, 0.64984E+00_JPRB, 0.69065E+00_JPRB, 0.73718E+00_JPRB, 0.78906E+00_JPRB ,&
&  0.80491E+00_JPRB, 0.79304E+00_JPRB, 0.78157E+00_JPRB, 0.76655E+00_JPRB, 0.74670E+00_JPRB, 0.73377E+00_JPRB, 0.72357E+00_JPRB ,&
&  0.72195E+00_JPRB, 0.71193E+00_JPRB, 0.69610E+00_JPRB, 0.68336E+00_JPRB, 0.67322E+00_JPRB, 0.66014E+00_JPRB, 0.64733E+00_JPRB ,&
&  0.63396E+00_JPRB, 0.62259E+00_JPRB, 0.62006E+00_JPRB, 0.61956E+00_JPRB, 0.62316E+00_JPRB, 0.62409E+00_JPRB, 0.61993E+00_JPRB ,&
&  0.61699E+00_JPRB, 0.61934E+00_JPRB, 0.62897E+00_JPRB, 0.63759E+00_JPRB, 0.64061E+00_JPRB, 0.63950E+00_JPRB, 0.64822E+00_JPRB ,&
&  0.65436E+00_JPRB, 0.66617E+00_JPRB, 0.68135E+00_JPRB, 0.70667E+00_JPRB, 0.72609E+00_JPRB, 0.73182E+00_JPRB, 0.73451E+00_JPRB ,&
&  0.74016E+00_JPRB, 0.73449E+00_JPRB, 0.71869E+00_JPRB, 0.70753E+00_JPRB, 0.70226E+00_JPRB, 0.69503E+00_JPRB, 0.69245E+00_JPRB ,&
&  0.69829E+00_JPRB, 0.70265E+00_JPRB, 0.70067E+00_JPRB, 0.70576E+00_JPRB, 0.71619E+00_JPRB, 0.72831E+00_JPRB, 0.73769E+00_JPRB ,&
&  0.72585E+00_JPRB, 0.73027E+00_JPRB, 0.72976E+00_JPRB, 0.73791E+00_JPRB, 0.74714E+00_JPRB, 0.73624E+00_JPRB, 0.72639E+00_JPRB ,&
&  0.71720E+00_JPRB, 0.69602E+00_JPRB, 0.69642E+00_JPRB, 0.69706E+00_JPRB, 0.69922E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.68118E+00_JPRB, 0.67271E+00_JPRB, 0.66665E+00_JPRB, 0.67462E+00_JPRB, 0.68928E+00_JPRB, 0.70089E+00_JPRB, 0.70200E+00_JPRB ,&
&  0.70299E+00_JPRB, 0.70857E+00_JPRB, 0.71760E+00_JPRB, 0.72062E+00_JPRB, 0.71465E+00_JPRB, 0.69924E+00_JPRB, 0.69207E+00_JPRB ,&
&  0.69147E+00_JPRB, 0.69401E+00_JPRB, 0.69519E+00_JPRB, 0.70061E+00_JPRB, 0.72119E+00_JPRB, 0.75265E+00_JPRB, 0.78236E+00_JPRB ,&
&  0.79715E+00_JPRB, 0.79200E+00_JPRB, 0.78594E+00_JPRB, 0.78081E+00_JPRB, 0.77943E+00_JPRB, 0.78409E+00_JPRB, 0.78499E+00_JPRB ,&
&  0.77649E+00_JPRB, 0.63016E+00_JPRB, 0.68425E+00_JPRB, 0.70359E+00_JPRB, 0.72572E+00_JPRB, 0.75229E+00_JPRB, 0.78375E+00_JPRB ,&
&  0.78674E+00_JPRB, 0.77805E+00_JPRB, 0.76062E+00_JPRB, 0.74862E+00_JPRB, 0.73428E+00_JPRB, 0.73092E+00_JPRB, 0.72923E+00_JPRB ,&
&  0.72788E+00_JPRB, 0.72516E+00_JPRB, 0.70333E+00_JPRB, 0.66825E+00_JPRB, 0.64944E+00_JPRB, 0.63878E+00_JPRB, 0.63022E+00_JPRB ,&
&  0.62238E+00_JPRB, 0.61135E+00_JPRB, 0.60044E+00_JPRB, 0.59130E+00_JPRB, 0.59352E+00_JPRB, 0.59245E+00_JPRB, 0.58058E+00_JPRB ,&
&  0.57609E+00_JPRB, 0.57462E+00_JPRB, 0.58268E+00_JPRB, 0.59329E+00_JPRB, 0.59755E+00_JPRB, 0.59273E+00_JPRB, 0.60150E+00_JPRB ,&
&  0.62197E+00_JPRB, 0.63514E+00_JPRB, 0.64658E+00_JPRB, 0.66125E+00_JPRB, 0.66854E+00_JPRB, 0.66786E+00_JPRB, 0.66891E+00_JPRB ,&
&  0.67535E+00_JPRB, 0.67391E+00_JPRB, 0.66620E+00_JPRB, 0.66015E+00_JPRB, 0.66162E+00_JPRB, 0.66269E+00_JPRB, 0.67273E+00_JPRB ,&
&  0.68215E+00_JPRB, 0.68557E+00_JPRB, 0.68202E+00_JPRB, 0.68497E+00_JPRB, 0.69102E+00_JPRB, 0.70078E+00_JPRB, 0.70695E+00_JPRB ,&
&  0.71323E+00_JPRB, 0.70999E+00_JPRB, 0.69732E+00_JPRB, 0.69698E+00_JPRB, 0.70117E+00_JPRB, 0.69776E+00_JPRB, 0.68557E+00_JPRB ,&
&  0.67541E+00_JPRB, 0.66718E+00_JPRB, 0.66983E+00_JPRB, 0.67189E+00_JPRB, 0.67722E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.68097E+00_JPRB, 0.67717E+00_JPRB, 0.67588E+00_JPRB, 0.68647E+00_JPRB, 0.70366E+00_JPRB, 0.70783E+00_JPRB, 0.70934E+00_JPRB ,&
&  0.72168E+00_JPRB, 0.73107E+00_JPRB, 0.73082E+00_JPRB, 0.72961E+00_JPRB, 0.72400E+00_JPRB, 0.71073E+00_JPRB, 0.70079E+00_JPRB ,&
&  0.70675E+00_JPRB, 0.71308E+00_JPRB, 0.71624E+00_JPRB, 0.74110E+00_JPRB, 0.77362E+00_JPRB, 0.81195E+00_JPRB, 0.83804E+00_JPRB ,&
&  0.84747E+00_JPRB, 0.84034E+00_JPRB, 0.83666E+00_JPRB, 0.83125E+00_JPRB, 0.82581E+00_JPRB, 0.82740E+00_JPRB, 0.82183E+00_JPRB ,&
&  0.81187E+00_JPRB, 0.71710E+00_JPRB, 0.74575E+00_JPRB, 0.75470E+00_JPRB, 0.76963E+00_JPRB, 0.79261E+00_JPRB, 0.81764E+00_JPRB ,&
&  0.81610E+00_JPRB, 0.81200E+00_JPRB, 0.80334E+00_JPRB, 0.78461E+00_JPRB, 0.76772E+00_JPRB, 0.75790E+00_JPRB, 0.74151E+00_JPRB ,&
&  0.71449E+00_JPRB, 0.67777E+00_JPRB, 0.64512E+00_JPRB, 0.62307E+00_JPRB, 0.61329E+00_JPRB, 0.61135E+00_JPRB, 0.60789E+00_JPRB ,&
&  0.60705E+00_JPRB, 0.60654E+00_JPRB, 0.59688E+00_JPRB, 0.58815E+00_JPRB, 0.57928E+00_JPRB, 0.57377E+00_JPRB, 0.56156E+00_JPRB ,&
&  0.55369E+00_JPRB, 0.54320E+00_JPRB, 0.54283E+00_JPRB, 0.56277E+00_JPRB, 0.58338E+00_JPRB, 0.58752E+00_JPRB, 0.59984E+00_JPRB ,&
&  0.61630E+00_JPRB, 0.62287E+00_JPRB, 0.62392E+00_JPRB, 0.62083E+00_JPRB, 0.61334E+00_JPRB, 0.61277E+00_JPRB, 0.62055E+00_JPRB ,&
&  0.62746E+00_JPRB, 0.63728E+00_JPRB, 0.64061E+00_JPRB, 0.64520E+00_JPRB, 0.65189E+00_JPRB, 0.65700E+00_JPRB, 0.66802E+00_JPRB ,&
&  0.67366E+00_JPRB, 0.67774E+00_JPRB, 0.68917E+00_JPRB, 0.70063E+00_JPRB, 0.70538E+00_JPRB, 0.70753E+00_JPRB, 0.71636E+00_JPRB ,&
&  0.71947E+00_JPRB, 0.71719E+00_JPRB, 0.70855E+00_JPRB, 0.69602E+00_JPRB, 0.68679E+00_JPRB, 0.67324E+00_JPRB, 0.65354E+00_JPRB ,&
&  0.63708E+00_JPRB, 0.63542E+00_JPRB, 0.64489E+00_JPRB, 0.65494E+00_JPRB, 0.67190E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.67539E+00_JPRB, 0.68681E+00_JPRB, 0.69983E+00_JPRB, 0.71256E+00_JPRB, 0.72585E+00_JPRB, 0.72569E+00_JPRB, 0.72710E+00_JPRB ,&
&  0.74630E+00_JPRB, 0.74929E+00_JPRB, 0.74862E+00_JPRB, 0.74010E+00_JPRB, 0.73393E+00_JPRB, 0.73401E+00_JPRB, 0.72536E+00_JPRB ,&
&  0.72120E+00_JPRB, 0.72791E+00_JPRB, 0.74681E+00_JPRB, 0.77551E+00_JPRB, 0.80350E+00_JPRB, 0.84002E+00_JPRB, 0.86702E+00_JPRB ,&
&  0.87151E+00_JPRB, 0.86093E+00_JPRB, 0.86104E+00_JPRB, 0.86418E+00_JPRB, 0.86431E+00_JPRB, 0.84729E+00_JPRB, 0.84261E+00_JPRB ,&
&  0.82251E+00_JPRB, 0.82758E+00_JPRB, 0.69769E+00_JPRB, 0.76680E+00_JPRB, 0.81018E+00_JPRB, 0.84336E+00_JPRB, 0.85117E+00_JPRB ,&
&  0.84292E+00_JPRB, 0.83048E+00_JPRB, 0.80848E+00_JPRB, 0.78340E+00_JPRB, 0.77428E+00_JPRB, 0.75399E+00_JPRB, 0.73107E+00_JPRB ,&
&  0.68261E+00_JPRB, 0.62080E+00_JPRB, 0.58930E+00_JPRB, 0.57796E+00_JPRB, 0.57263E+00_JPRB, 0.57443E+00_JPRB, 0.58039E+00_JPRB ,&
&  0.58468E+00_JPRB, 0.58578E+00_JPRB, 0.57535E+00_JPRB, 0.55959E+00_JPRB, 0.54112E+00_JPRB, 0.53147E+00_JPRB, 0.52898E+00_JPRB ,&
&  0.52552E+00_JPRB, 0.51297E+00_JPRB, 0.51636E+00_JPRB, 0.54629E+00_JPRB, 0.57132E+00_JPRB, 0.58851E+00_JPRB, 0.59545E+00_JPRB ,&
&  0.60015E+00_JPRB, 0.59347E+00_JPRB, 0.58937E+00_JPRB, 0.58044E+00_JPRB, 0.55750E+00_JPRB, 0.55149E+00_JPRB, 0.57257E+00_JPRB ,&
&  0.59636E+00_JPRB, 0.60992E+00_JPRB, 0.61793E+00_JPRB, 0.63005E+00_JPRB, 0.64277E+00_JPRB, 0.64990E+00_JPRB, 0.65615E+00_JPRB ,&
&  0.66086E+00_JPRB, 0.66545E+00_JPRB, 0.67887E+00_JPRB, 0.69368E+00_JPRB, 0.70048E+00_JPRB, 0.70147E+00_JPRB, 0.70123E+00_JPRB ,&
&  0.69810E+00_JPRB, 0.69032E+00_JPRB, 0.68134E+00_JPRB, 0.67268E+00_JPRB, 0.65247E+00_JPRB, 0.62821E+00_JPRB, 0.61045E+00_JPRB ,&
&  0.60177E+00_JPRB, 0.61205E+00_JPRB, 0.61975E+00_JPRB, 0.63223E+00_JPRB, 0.65874E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.71855E+00_JPRB, 0.73242E+00_JPRB, 0.74386E+00_JPRB, 0.76499E+00_JPRB, 0.78444E+00_JPRB, 0.78199E+00_JPRB, 0.77503E+00_JPRB ,&
&  0.76999E+00_JPRB, 0.76536E+00_JPRB, 0.75816E+00_JPRB, 0.74856E+00_JPRB, 0.74211E+00_JPRB, 0.74157E+00_JPRB, 0.74236E+00_JPRB ,&
&  0.74406E+00_JPRB, 0.76428E+00_JPRB, 0.79752E+00_JPRB, 0.82371E+00_JPRB, 0.85230E+00_JPRB, 0.89597E+00_JPRB, 0.91678E+00_JPRB ,&
&  0.90063E+00_JPRB, 0.89998E+00_JPRB, 0.88914E+00_JPRB, 0.87450E+00_JPRB, 0.87620E+00_JPRB, 0.86673E+00_JPRB, 0.83830E+00_JPRB ,&
&  0.83160E+00_JPRB, 0.82368E+00_JPRB, 0.81955E+00_JPRB, 0.84711E+00_JPRB, 0.86823E+00_JPRB, 0.86718E+00_JPRB, 0.86245E+00_JPRB ,&
&  0.84078E+00_JPRB, 0.80393E+00_JPRB, 0.77009E+00_JPRB, 0.75424E+00_JPRB, 0.75262E+00_JPRB, 0.73249E+00_JPRB, 0.70441E+00_JPRB ,&
&  0.64696E+00_JPRB, 0.59986E+00_JPRB, 0.57393E+00_JPRB, 0.55865E+00_JPRB, 0.55394E+00_JPRB, 0.55161E+00_JPRB, 0.54762E+00_JPRB ,&
&  0.55335E+00_JPRB, 0.54870E+00_JPRB, 0.52178E+00_JPRB, 0.49372E+00_JPRB, 0.48952E+00_JPRB, 0.49858E+00_JPRB, 0.51828E+00_JPRB ,&
&  0.53340E+00_JPRB, 0.52914E+00_JPRB, 0.53619E+00_JPRB, 0.54393E+00_JPRB, 0.56103E+00_JPRB, 0.57749E+00_JPRB, 0.57813E+00_JPRB ,&
&  0.57763E+00_JPRB, 0.57210E+00_JPRB, 0.55881E+00_JPRB, 0.55028E+00_JPRB, 0.53915E+00_JPRB, 0.53494E+00_JPRB, 0.54660E+00_JPRB ,&
&  0.56413E+00_JPRB, 0.57391E+00_JPRB, 0.58488E+00_JPRB, 0.59844E+00_JPRB, 0.60923E+00_JPRB, 0.61746E+00_JPRB, 0.62408E+00_JPRB ,&
&  0.63428E+00_JPRB, 0.64888E+00_JPRB, 0.66369E+00_JPRB, 0.67659E+00_JPRB, 0.67649E+00_JPRB, 0.66225E+00_JPRB, 0.63680E+00_JPRB ,&
&  0.61977E+00_JPRB, 0.60639E+00_JPRB, 0.60306E+00_JPRB, 0.60834E+00_JPRB, 0.60343E+00_JPRB, 0.58396E+00_JPRB, 0.57427E+00_JPRB ,&
&  0.58515E+00_JPRB, 0.60904E+00_JPRB, 0.63184E+00_JPRB, 0.66277E+00_JPRB, 0.69521E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.75812E+00_JPRB, 0.78965E+00_JPRB, 0.81405E+00_JPRB, 0.82314E+00_JPRB, 0.82130E+00_JPRB, 0.81274E+00_JPRB, 0.79029E+00_JPRB ,&
&  0.76587E+00_JPRB, 0.74856E+00_JPRB, 0.73015E+00_JPRB, 0.70365E+00_JPRB, 0.68169E+00_JPRB, 0.67712E+00_JPRB, 0.68969E+00_JPRB ,&
&  0.73594E+00_JPRB, 0.77705E+00_JPRB, 0.79383E+00_JPRB, 0.80776E+00_JPRB, 0.85793E+00_JPRB, 0.90010E+00_JPRB, 0.90958E+00_JPRB ,&
&  0.90501E+00_JPRB, 0.90694E+00_JPRB, 0.87046E+00_JPRB, 0.83716E+00_JPRB, 0.83701E+00_JPRB, 0.84204E+00_JPRB, 0.82336E+00_JPRB ,&
&  0.84871E+00_JPRB, 0.86544E+00_JPRB, 0.85872E+00_JPRB, 0.83282E+00_JPRB, 0.82468E+00_JPRB, 0.81410E+00_JPRB, 0.80234E+00_JPRB ,&
&  0.78052E+00_JPRB, 0.73247E+00_JPRB, 0.70312E+00_JPRB, 0.70196E+00_JPRB, 0.69490E+00_JPRB, 0.68745E+00_JPRB, 0.67371E+00_JPRB ,&
&  0.63363E+00_JPRB, 0.59784E+00_JPRB, 0.57106E+00_JPRB, 0.55268E+00_JPRB, 0.54125E+00_JPRB, 0.53244E+00_JPRB, 0.52334E+00_JPRB ,&
&  0.51553E+00_JPRB, 0.48894E+00_JPRB, 0.46161E+00_JPRB, 0.45625E+00_JPRB, 0.46261E+00_JPRB, 0.46988E+00_JPRB, 0.49709E+00_JPRB ,&
&  0.52208E+00_JPRB, 0.52046E+00_JPRB, 0.51682E+00_JPRB, 0.51917E+00_JPRB, 0.52784E+00_JPRB, 0.52782E+00_JPRB, 0.51735E+00_JPRB ,&
&  0.51200E+00_JPRB, 0.50936E+00_JPRB, 0.49982E+00_JPRB, 0.49276E+00_JPRB, 0.49205E+00_JPRB, 0.49610E+00_JPRB, 0.51066E+00_JPRB ,&
&  0.51969E+00_JPRB, 0.52110E+00_JPRB, 0.53474E+00_JPRB, 0.55196E+00_JPRB, 0.56224E+00_JPRB, 0.57027E+00_JPRB, 0.57598E+00_JPRB ,&
&  0.58940E+00_JPRB, 0.60886E+00_JPRB, 0.62387E+00_JPRB, 0.62588E+00_JPRB, 0.60281E+00_JPRB, 0.57845E+00_JPRB, 0.55930E+00_JPRB ,&
&  0.55104E+00_JPRB, 0.55556E+00_JPRB, 0.56316E+00_JPRB, 0.57044E+00_JPRB, 0.57710E+00_JPRB, 0.58151E+00_JPRB, 0.58078E+00_JPRB ,&
&  0.60031E+00_JPRB, 0.63145E+00_JPRB, 0.65664E+00_JPRB, 0.68669E+00_JPRB, 0.71798E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.82593E+00_JPRB, 0.84357E+00_JPRB, 0.85555E+00_JPRB, 0.85648E+00_JPRB, 0.84109E+00_JPRB, 0.81442E+00_JPRB, 0.79156E+00_JPRB ,&
&  0.76551E+00_JPRB, 0.74021E+00_JPRB, 0.71616E+00_JPRB, 0.68347E+00_JPRB, 0.66588E+00_JPRB, 0.66935E+00_JPRB, 0.71842E+00_JPRB ,&
&  0.78377E+00_JPRB, 0.78872E+00_JPRB, 0.78285E+00_JPRB, 0.80835E+00_JPRB, 0.86115E+00_JPRB, 0.89320E+00_JPRB, 0.91431E+00_JPRB ,&
&  0.91743E+00_JPRB, 0.88459E+00_JPRB, 0.85138E+00_JPRB, 0.83777E+00_JPRB, 0.86088E+00_JPRB, 0.87827E+00_JPRB, 0.88305E+00_JPRB ,&
&  0.87835E+00_JPRB, 0.85683E+00_JPRB, 0.83730E+00_JPRB, 0.82067E+00_JPRB, 0.82437E+00_JPRB, 0.79923E+00_JPRB, 0.74689E+00_JPRB ,&
&  0.68055E+00_JPRB, 0.65421E+00_JPRB, 0.64313E+00_JPRB, 0.62891E+00_JPRB, 0.61808E+00_JPRB, 0.61821E+00_JPRB, 0.60900E+00_JPRB ,&
&  0.59548E+00_JPRB, 0.57994E+00_JPRB, 0.56516E+00_JPRB, 0.54763E+00_JPRB, 0.53392E+00_JPRB, 0.51448E+00_JPRB, 0.49629E+00_JPRB ,&
&  0.47941E+00_JPRB, 0.45868E+00_JPRB, 0.43972E+00_JPRB, 0.42665E+00_JPRB, 0.42109E+00_JPRB, 0.43161E+00_JPRB, 0.47162E+00_JPRB ,&
&  0.50292E+00_JPRB, 0.49989E+00_JPRB, 0.49743E+00_JPRB, 0.50124E+00_JPRB, 0.49389E+00_JPRB, 0.48316E+00_JPRB, 0.47724E+00_JPRB ,&
&  0.47199E+00_JPRB, 0.46782E+00_JPRB, 0.46587E+00_JPRB, 0.46758E+00_JPRB, 0.46970E+00_JPRB, 0.46790E+00_JPRB, 0.47041E+00_JPRB ,&
&  0.47521E+00_JPRB, 0.48052E+00_JPRB, 0.49757E+00_JPRB, 0.52201E+00_JPRB, 0.54257E+00_JPRB, 0.55705E+00_JPRB, 0.57045E+00_JPRB ,&
&  0.59127E+00_JPRB, 0.60402E+00_JPRB, 0.59134E+00_JPRB, 0.57922E+00_JPRB, 0.57287E+00_JPRB, 0.57792E+00_JPRB, 0.58843E+00_JPRB ,&
&  0.60910E+00_JPRB, 0.62206E+00_JPRB, 0.62821E+00_JPRB, 0.62977E+00_JPRB, 0.63304E+00_JPRB, 0.63394E+00_JPRB, 0.64646E+00_JPRB ,&
&  0.68015E+00_JPRB, 0.70924E+00_JPRB, 0.71663E+00_JPRB, 0.74927E+00_JPRB, 0.79103E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.86600E+00_JPRB, 0.87036E+00_JPRB, 0.86069E+00_JPRB, 0.85641E+00_JPRB, 0.84586E+00_JPRB, 0.83232E+00_JPRB, 0.80999E+00_JPRB ,&
&  0.78369E+00_JPRB, 0.75557E+00_JPRB, 0.72906E+00_JPRB, 0.70739E+00_JPRB, 0.71149E+00_JPRB, 0.74427E+00_JPRB, 0.77190E+00_JPRB ,&
&  0.76103E+00_JPRB, 0.75724E+00_JPRB, 0.77451E+00_JPRB, 0.80532E+00_JPRB, 0.83881E+00_JPRB, 0.86970E+00_JPRB, 0.89195E+00_JPRB ,&
&  0.91000E+00_JPRB, 0.89179E+00_JPRB, 0.89537E+00_JPRB, 0.89726E+00_JPRB, 0.89095E+00_JPRB, 0.87277E+00_JPRB, 0.84383E+00_JPRB ,&
&  0.81515E+00_JPRB, 0.78796E+00_JPRB, 0.75431E+00_JPRB, 0.69519E+00_JPRB, 0.76831E+00_JPRB, 0.69127E+00_JPRB, 0.66255E+00_JPRB ,&
&  0.63529E+00_JPRB, 0.61718E+00_JPRB, 0.59693E+00_JPRB, 0.57496E+00_JPRB, 0.56742E+00_JPRB, 0.56284E+00_JPRB, 0.55498E+00_JPRB ,&
&  0.54850E+00_JPRB, 0.54156E+00_JPRB, 0.53103E+00_JPRB, 0.51749E+00_JPRB, 0.50472E+00_JPRB, 0.49100E+00_JPRB, 0.47840E+00_JPRB ,&
&  0.46020E+00_JPRB, 0.43358E+00_JPRB, 0.40559E+00_JPRB, 0.38512E+00_JPRB, 0.37459E+00_JPRB, 0.38390E+00_JPRB, 0.41651E+00_JPRB ,&
&  0.45003E+00_JPRB, 0.45558E+00_JPRB, 0.46512E+00_JPRB, 0.46763E+00_JPRB, 0.45888E+00_JPRB, 0.44872E+00_JPRB, 0.43097E+00_JPRB ,&
&  0.41733E+00_JPRB, 0.41487E+00_JPRB, 0.41561E+00_JPRB, 0.41917E+00_JPRB, 0.42438E+00_JPRB, 0.43172E+00_JPRB, 0.44198E+00_JPRB ,&
&  0.45146E+00_JPRB, 0.45476E+00_JPRB, 0.46701E+00_JPRB, 0.49374E+00_JPRB, 0.52453E+00_JPRB, 0.55569E+00_JPRB, 0.58225E+00_JPRB ,&
&  0.60016E+00_JPRB, 0.61258E+00_JPRB, 0.62078E+00_JPRB, 0.63593E+00_JPRB, 0.65155E+00_JPRB, 0.66735E+00_JPRB, 0.67561E+00_JPRB ,&
&  0.67330E+00_JPRB, 0.67688E+00_JPRB, 0.69385E+00_JPRB, 0.72192E+00_JPRB, 0.74162E+00_JPRB, 0.77540E+00_JPRB, 0.79646E+00_JPRB ,&
&  0.81884E+00_JPRB, 0.82456E+00_JPRB, 0.81913E+00_JPRB, 0.83163E+00_JPRB, 0.84841E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.85233E+00_JPRB, 0.86287E+00_JPRB, 0.86068E+00_JPRB, 0.85959E+00_JPRB, 0.86138E+00_JPRB, 0.85441E+00_JPRB, 0.84033E+00_JPRB ,&
&  0.82619E+00_JPRB, 0.79975E+00_JPRB, 0.76948E+00_JPRB, 0.75598E+00_JPRB, 0.76363E+00_JPRB, 0.76363E+00_JPRB, 0.74503E+00_JPRB ,&
&  0.74341E+00_JPRB, 0.75738E+00_JPRB, 0.78052E+00_JPRB, 0.80627E+00_JPRB, 0.84093E+00_JPRB, 0.87683E+00_JPRB, 0.90368E+00_JPRB ,&
&  0.91423E+00_JPRB, 0.91220E+00_JPRB, 0.90447E+00_JPRB, 0.87771E+00_JPRB, 0.84196E+00_JPRB, 0.81444E+00_JPRB, 0.79358E+00_JPRB ,&
&  0.76786E+00_JPRB, 0.72028E+00_JPRB, 0.65070E+00_JPRB, 0.64512E+00_JPRB, 0.58102E+00_JPRB, 0.56064E+00_JPRB, 0.58277E+00_JPRB ,&
&  0.59809E+00_JPRB, 0.58013E+00_JPRB, 0.55640E+00_JPRB, 0.53673E+00_JPRB, 0.53306E+00_JPRB, 0.53176E+00_JPRB, 0.52672E+00_JPRB ,&
&  0.52290E+00_JPRB, 0.51621E+00_JPRB, 0.50219E+00_JPRB, 0.48927E+00_JPRB, 0.47709E+00_JPRB, 0.46491E+00_JPRB, 0.45664E+00_JPRB ,&
&  0.42870E+00_JPRB, 0.38592E+00_JPRB, 0.35411E+00_JPRB, 0.34040E+00_JPRB, 0.33286E+00_JPRB, 0.33657E+00_JPRB, 0.34921E+00_JPRB ,&
&  0.35932E+00_JPRB, 0.35971E+00_JPRB, 0.35400E+00_JPRB, 0.34151E+00_JPRB, 0.32256E+00_JPRB, 0.27687E+00_JPRB, 0.25077E+00_JPRB ,&
&  0.26631E+00_JPRB, 0.30006E+00_JPRB, 0.33519E+00_JPRB, 0.34386E+00_JPRB, 0.35332E+00_JPRB, 0.35582E+00_JPRB, 0.34134E+00_JPRB ,&
&  0.31336E+00_JPRB, 0.28250E+00_JPRB, 0.27624E+00_JPRB, 0.27833E+00_JPRB, 0.28265E+00_JPRB, 0.29078E+00_JPRB, 0.30647E+00_JPRB ,&
&  0.33029E+00_JPRB, 0.36900E+00_JPRB, 0.39108E+00_JPRB, 0.39376E+00_JPRB, 0.38647E+00_JPRB, 0.37493E+00_JPRB, 0.37295E+00_JPRB ,&
&  0.38677E+00_JPRB, 0.44761E+00_JPRB, 0.56959E+00_JPRB, 0.71722E+00_JPRB, 0.71627E+00_JPRB, 0.76019E+00_JPRB, 0.79493E+00_JPRB ,&
&  0.82459E+00_JPRB, 0.83784E+00_JPRB, 0.82444E+00_JPRB, 0.82571E+00_JPRB, 0.83233E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.81309E+00_JPRB, 0.83307E+00_JPRB, 0.84408E+00_JPRB, 0.85814E+00_JPRB, 0.85841E+00_JPRB, 0.82827E+00_JPRB, 0.81370E+00_JPRB ,&
&  0.81843E+00_JPRB, 0.81366E+00_JPRB, 0.80295E+00_JPRB, 0.79635E+00_JPRB, 0.78750E+00_JPRB, 0.77769E+00_JPRB, 0.76937E+00_JPRB ,&
&  0.76114E+00_JPRB, 0.75629E+00_JPRB, 0.76340E+00_JPRB, 0.77745E+00_JPRB, 0.78902E+00_JPRB, 0.79539E+00_JPRB, 0.80039E+00_JPRB ,&
&  0.79987E+00_JPRB, 0.76676E+00_JPRB, 0.72170E+00_JPRB, 0.68886E+00_JPRB, 0.67225E+00_JPRB, 0.65835E+00_JPRB, 0.62534E+00_JPRB ,&
&  0.55816E+00_JPRB, 0.43673E+00_JPRB, 0.38013E+00_JPRB, 0.42110E+00_JPRB, 0.53976E+00_JPRB, 0.50437E+00_JPRB, 0.52488E+00_JPRB ,&
&  0.55532E+00_JPRB, 0.57109E+00_JPRB, 0.56692E+00_JPRB, 0.55650E+00_JPRB, 0.55312E+00_JPRB, 0.55678E+00_JPRB, 0.55343E+00_JPRB ,&
&  0.55003E+00_JPRB, 0.53411E+00_JPRB, 0.52466E+00_JPRB, 0.51404E+00_JPRB, 0.49664E+00_JPRB, 0.47100E+00_JPRB, 0.45556E+00_JPRB ,&
&  0.41757E+00_JPRB, 0.38013E+00_JPRB, 0.36932E+00_JPRB, 0.37020E+00_JPRB, 0.33946E+00_JPRB, 0.32698E+00_JPRB, 0.31227E+00_JPRB ,&
&  0.27209E+00_JPRB, 0.25061E+00_JPRB, 0.21829E+00_JPRB, 0.19794E+00_JPRB, 0.18283E+00_JPRB, 0.16889E+00_JPRB, 0.16689E+00_JPRB ,&
&  0.17143E+00_JPRB, 0.18354E+00_JPRB, 0.20498E+00_JPRB, 0.23958E+00_JPRB, 0.27126E+00_JPRB, 0.22707E+00_JPRB, 0.20867E+00_JPRB ,&
&  0.19474E+00_JPRB, 0.19172E+00_JPRB, 0.19700E+00_JPRB, 0.20327E+00_JPRB, 0.20975E+00_JPRB, 0.21741E+00_JPRB, 0.22468E+00_JPRB ,&
&  0.22991E+00_JPRB, 0.23295E+00_JPRB, 0.23395E+00_JPRB, 0.23332E+00_JPRB, 0.23246E+00_JPRB, 0.23341E+00_JPRB, 0.23692E+00_JPRB ,&
&  0.24296E+00_JPRB, 0.25508E+00_JPRB, 0.27887E+00_JPRB, 0.31406E+00_JPRB, 0.36021E+00_JPRB, 0.42046E+00_JPRB, 0.51277E+00_JPRB ,&
&  0.63773E+00_JPRB, 0.80222E+00_JPRB, 0.84631E+00_JPRB, 0.85526E+00_JPRB, 0.82461E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.81929E+00_JPRB, 0.81710E+00_JPRB, 0.82509E+00_JPRB, 0.81483E+00_JPRB, 0.79581E+00_JPRB, 0.78683E+00_JPRB, 0.80388E+00_JPRB ,&
&  0.82372E+00_JPRB, 0.79440E+00_JPRB, 0.77284E+00_JPRB, 0.75368E+00_JPRB, 0.74017E+00_JPRB, 0.72646E+00_JPRB, 0.70434E+00_JPRB ,&
&  0.68433E+00_JPRB, 0.67708E+00_JPRB, 0.67528E+00_JPRB, 0.67031E+00_JPRB, 0.66010E+00_JPRB, 0.65006E+00_JPRB, 0.63234E+00_JPRB ,&
&  0.59976E+00_JPRB, 0.56564E+00_JPRB, 0.54769E+00_JPRB, 0.52416E+00_JPRB, 0.50738E+00_JPRB, 0.48798E+00_JPRB, 0.46037E+00_JPRB ,&
&  0.41559E+00_JPRB, 0.35763E+00_JPRB, 0.32133E+00_JPRB, 0.35760E+00_JPRB, 0.45295E+00_JPRB, 0.44691E+00_JPRB, 0.46597E+00_JPRB ,&
&  0.50058E+00_JPRB, 0.52184E+00_JPRB, 0.53233E+00_JPRB, 0.53577E+00_JPRB, 0.54502E+00_JPRB, 0.55057E+00_JPRB, 0.54715E+00_JPRB ,&
&  0.53275E+00_JPRB, 0.48065E+00_JPRB, 0.43840E+00_JPRB, 0.33577E+00_JPRB, 0.26426E+00_JPRB, 0.22662E+00_JPRB, 0.19763E+00_JPRB ,&
&  0.17889E+00_JPRB, 0.16769E+00_JPRB, 0.16515E+00_JPRB, 0.16504E+00_JPRB, 0.17025E+00_JPRB, 0.17550E+00_JPRB, 0.17536E+00_JPRB ,&
&  0.17159E+00_JPRB, 0.16390E+00_JPRB, 0.15275E+00_JPRB, 0.14589E+00_JPRB, 0.14236E+00_JPRB, 0.14244E+00_JPRB, 0.14432E+00_JPRB ,&
&  0.14592E+00_JPRB, 0.15018E+00_JPRB, 0.16260E+00_JPRB, 0.17358E+00_JPRB, 0.18702E+00_JPRB, 0.16872E+00_JPRB, 0.15395E+00_JPRB ,&
&  0.14884E+00_JPRB, 0.15296E+00_JPRB, 0.15943E+00_JPRB, 0.16431E+00_JPRB, 0.16949E+00_JPRB, 0.17396E+00_JPRB, 0.17848E+00_JPRB ,&
&  0.18279E+00_JPRB, 0.18580E+00_JPRB, 0.18739E+00_JPRB, 0.18911E+00_JPRB, 0.19069E+00_JPRB, 0.19301E+00_JPRB, 0.19595E+00_JPRB ,&
&  0.20096E+00_JPRB, 0.21019E+00_JPRB, 0.22224E+00_JPRB, 0.24214E+00_JPRB, 0.26082E+00_JPRB, 0.27948E+00_JPRB, 0.30080E+00_JPRB ,&
&  0.32183E+00_JPRB, 0.40191E+00_JPRB, 0.59575E+00_JPRB, 0.91736E+00_JPRB, 0.91075E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.76907E+00_JPRB, 0.78161E+00_JPRB, 0.79872E+00_JPRB, 0.77527E+00_JPRB, 0.75999E+00_JPRB, 0.76340E+00_JPRB, 0.76425E+00_JPRB ,&
&  0.74121E+00_JPRB, 0.70625E+00_JPRB, 0.65729E+00_JPRB, 0.61716E+00_JPRB, 0.55164E+00_JPRB, 0.46745E+00_JPRB, 0.42987E+00_JPRB ,&
&  0.39577E+00_JPRB, 0.39230E+00_JPRB, 0.39465E+00_JPRB, 0.41072E+00_JPRB, 0.41962E+00_JPRB, 0.41621E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.36767E+00_JPRB, 0.34054E+00_JPRB, 0.32307E+00_JPRB, 0.30917E+00_JPRB, 0.30897E+00_JPRB, 0.30719E+00_JPRB, 0.30070E+00_JPRB ,&
&  0.29208E+00_JPRB, 0.29630E+00_JPRB, 0.31363E+00_JPRB, 0.34208E+00_JPRB, 0.39177E+00_JPRB, 0.37173E+00_JPRB, 0.37796E+00_JPRB ,&
&  0.40795E+00_JPRB, 0.42956E+00_JPRB, 0.43820E+00_JPRB, 0.44137E+00_JPRB, 0.43476E+00_JPRB, 0.45984E+00_JPRB, 0.40346E+00_JPRB ,&
&  0.29635E+00_JPRB, 0.23745E+00_JPRB, 0.19550E+00_JPRB, 0.16866E+00_JPRB, 0.15094E+00_JPRB, 0.14066E+00_JPRB, 0.12941E+00_JPRB ,&
&  0.12424E+00_JPRB, 0.12045E+00_JPRB, 0.11949E+00_JPRB, 0.12084E+00_JPRB, 0.12317E+00_JPRB, 0.12611E+00_JPRB, 0.12708E+00_JPRB ,&
&  0.12872E+00_JPRB, 0.12872E+00_JPRB, 0.12784E+00_JPRB, 0.12753E+00_JPRB, 0.12647E+00_JPRB, 0.12700E+00_JPRB, 0.12687E+00_JPRB ,&
&  0.12684E+00_JPRB, 0.12685E+00_JPRB, 0.12837E+00_JPRB, 0.12883E+00_JPRB, 0.12959E+00_JPRB, 0.12663E+00_JPRB, 0.12484E+00_JPRB ,&
&  0.12491E+00_JPRB, 0.12681E+00_JPRB, 0.13001E+00_JPRB, 0.13374E+00_JPRB, 0.13886E+00_JPRB, 0.14300E+00_JPRB, 0.14772E+00_JPRB ,&
&  0.15185E+00_JPRB, 0.15586E+00_JPRB, 0.15979E+00_JPRB, 0.16380E+00_JPRB, 0.16807E+00_JPRB, 0.17237E+00_JPRB, 0.17689E+00_JPRB ,&
&  0.18153E+00_JPRB, 0.18961E+00_JPRB, 0.19963E+00_JPRB, 0.21707E+00_JPRB, 0.24060E+00_JPRB, 0.26768E+00_JPRB, 0.31498E+00_JPRB ,&
&  0.37574E+00_JPRB, 0.60541E+00_JPRB, 0.90469E+00_JPRB, 0.73239E+00_JPRB, 0.75036E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.76057E+00_JPRB, 0.77896E+00_JPRB, 0.79533E+00_JPRB, 0.79276E+00_JPRB, 0.77556E+00_JPRB, 0.71796E+00_JPRB, 0.65464E+00_JPRB ,&
&  0.57606E+00_JPRB, 0.50992E+00_JPRB, 0.44070E+00_JPRB, 0.38067E+00_JPRB, 0.33040E+00_JPRB, 0.29182E+00_JPRB, 0.26892E+00_JPRB ,&
&  0.25288E+00_JPRB, 0.24591E+00_JPRB, 0.24345E+00_JPRB, 0.24808E+00_JPRB, 0.25062E+00_JPRB, 0.25609E+00_JPRB, 0.25054E+00_JPRB ,&
&  0.24366E+00_JPRB, 0.23560E+00_JPRB, 0.23363E+00_JPRB, 0.24239E+00_JPRB, 0.25551E+00_JPRB, 0.26830E+00_JPRB, 0.28489E+00_JPRB ,&
&  0.30318E+00_JPRB, 0.30906E+00_JPRB, 0.32303E+00_JPRB, 0.32677E+00_JPRB, 0.31205E+00_JPRB, 0.31372E+00_JPRB, 0.31957E+00_JPRB ,&
&  0.33274E+00_JPRB, 0.35134E+00_JPRB, 0.36204E+00_JPRB, 0.38228E+00_JPRB, 0.32990E+00_JPRB, 0.26983E+00_JPRB, 0.21831E+00_JPRB ,&
&  0.18003E+00_JPRB, 0.15831E+00_JPRB, 0.14328E+00_JPRB, 0.13562E+00_JPRB, 0.12837E+00_JPRB, 0.12440E+00_JPRB, 0.12014E+00_JPRB ,&
&  0.11821E+00_JPRB, 0.11803E+00_JPRB, 0.11743E+00_JPRB, 0.11699E+00_JPRB, 0.11722E+00_JPRB, 0.11769E+00_JPRB, 0.11703E+00_JPRB ,&
&  0.11620E+00_JPRB, 0.11584E+00_JPRB, 0.11530E+00_JPRB, 0.11490E+00_JPRB, 0.11405E+00_JPRB, 0.11381E+00_JPRB, 0.11227E+00_JPRB ,&
&  0.11141E+00_JPRB, 0.11005E+00_JPRB, 0.10924E+00_JPRB, 0.10907E+00_JPRB, 0.10933E+00_JPRB, 0.10962E+00_JPRB, 0.11013E+00_JPRB ,&
&  0.11120E+00_JPRB, 0.11217E+00_JPRB, 0.11382E+00_JPRB, 0.11550E+00_JPRB, 0.11817E+00_JPRB, 0.12065E+00_JPRB, 0.12414E+00_JPRB ,&
&  0.12784E+00_JPRB, 0.13232E+00_JPRB, 0.13740E+00_JPRB, 0.14309E+00_JPRB, 0.14807E+00_JPRB, 0.15298E+00_JPRB, 0.15836E+00_JPRB ,&
&  0.16267E+00_JPRB, 0.16958E+00_JPRB, 0.17742E+00_JPRB, 0.19035E+00_JPRB, 0.21727E+00_JPRB, 0.24109E+00_JPRB, 0.28993E+00_JPRB ,&
&  0.37354E+00_JPRB, 0.62832E+00_JPRB, 0.85960E+00_JPRB, 0.71099E+00_JPRB, 0.74212E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.63994E+00_JPRB, 0.67104E+00_JPRB, 0.69555E+00_JPRB, 0.69784E+00_JPRB, 0.69684E+00_JPRB, 0.65968E+00_JPRB, 0.60199E+00_JPRB ,&
&  0.49518E+00_JPRB, 0.42552E+00_JPRB, 0.38176E+00_JPRB, 0.34052E+00_JPRB, 0.30308E+00_JPRB, 0.27801E+00_JPRB, 0.26083E+00_JPRB ,&
&  0.24780E+00_JPRB, 0.24065E+00_JPRB, 0.23519E+00_JPRB, 0.23059E+00_JPRB, 0.22567E+00_JPRB, 0.22091E+00_JPRB, 0.21222E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20348E+00_JPRB, 0.20213E+00_JPRB, 0.20458E+00_JPRB, 0.21148E+00_JPRB, 0.21870E+00_JPRB, 0.22946E+00_JPRB ,&
&  0.24306E+00_JPRB, 0.25689E+00_JPRB, 0.26389E+00_JPRB, 0.26712E+00_JPRB, 0.27005E+00_JPRB, 0.26816E+00_JPRB, 0.24549E+00_JPRB ,&
&  0.25053E+00_JPRB, 0.27370E+00_JPRB, 0.29210E+00_JPRB, 0.30272E+00_JPRB, 0.25883E+00_JPRB, 0.21765E+00_JPRB, 0.17711E+00_JPRB ,&
&  0.15752E+00_JPRB, 0.14239E+00_JPRB, 0.13458E+00_JPRB, 0.12873E+00_JPRB, 0.12455E+00_JPRB, 0.12199E+00_JPRB, 0.11971E+00_JPRB ,&
&  0.11760E+00_JPRB, 0.11662E+00_JPRB, 0.11515E+00_JPRB, 0.11443E+00_JPRB, 0.11339E+00_JPRB, 0.11313E+00_JPRB, 0.11189E+00_JPRB ,&
&  0.11061E+00_JPRB, 0.10959E+00_JPRB, 0.10891E+00_JPRB, 0.10854E+00_JPRB, 0.10775E+00_JPRB, 0.10693E+00_JPRB, 0.10517E+00_JPRB ,&
&  0.10387E+00_JPRB, 0.10259E+00_JPRB, 0.10159E+00_JPRB, 0.10154E+00_JPRB, 0.10190E+00_JPRB, 0.10344E+00_JPRB, 0.10427E+00_JPRB ,&
&  0.10441E+00_JPRB, 0.10442E+00_JPRB, 0.10442E+00_JPRB, 0.10505E+00_JPRB, 0.10603E+00_JPRB, 0.10779E+00_JPRB, 0.11024E+00_JPRB ,&
&  0.11320E+00_JPRB, 0.11773E+00_JPRB, 0.12235E+00_JPRB, 0.12758E+00_JPRB, 0.13255E+00_JPRB, 0.13756E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.15069E+00_JPRB, 0.15836E+00_JPRB, 0.16956E+00_JPRB, 0.18165E+00_JPRB, 0.20333E+00_JPRB, 0.22285E+00_JPRB, 0.25479E+00_JPRB ,&
&  0.32563E+00_JPRB, 0.47302E+00_JPRB, 0.55846E+00_JPRB, 0.56934E+00_JPRB, 0.60227E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.46168E+00_JPRB, 0.47315E+00_JPRB, 0.48824E+00_JPRB, 0.49739E+00_JPRB, 0.50724E+00_JPRB, 0.47461E+00_JPRB, 0.44889E+00_JPRB ,&
&  0.39639E+00_JPRB, 0.36644E+00_JPRB, 0.34155E+00_JPRB, 0.31869E+00_JPRB, 0.29838E+00_JPRB, 0.28069E+00_JPRB, 0.26353E+00_JPRB ,&
&  0.25251E+00_JPRB, 0.24579E+00_JPRB, 0.23663E+00_JPRB, 0.22654E+00_JPRB, 0.21580E+00_JPRB, 0.20643E+00_JPRB, 0.20135E+00_JPRB ,&
&  0.19684E+00_JPRB, 0.19360E+00_JPRB, 0.19230E+00_JPRB, 0.19267E+00_JPRB, 0.19465E+00_JPRB, 0.19508E+00_JPRB, 0.19594E+00_JPRB ,&
&  0.19330E+00_JPRB, 0.20039E+00_JPRB, 0.20965E+00_JPRB, 0.21388E+00_JPRB, 0.22156E+00_JPRB, 0.22009E+00_JPRB, 0.21533E+00_JPRB ,&
&  0.20850E+00_JPRB, 0.20024E+00_JPRB, 0.19106E+00_JPRB, 0.17970E+00_JPRB, 0.16924E+00_JPRB, 0.16141E+00_JPRB, 0.15375E+00_JPRB ,&
&  0.14907E+00_JPRB, 0.14363E+00_JPRB, 0.13744E+00_JPRB, 0.13215E+00_JPRB, 0.12756E+00_JPRB, 0.12372E+00_JPRB, 0.12025E+00_JPRB ,&
&  0.11761E+00_JPRB, 0.11599E+00_JPRB, 0.11440E+00_JPRB, 0.11379E+00_JPRB, 0.11260E+00_JPRB, 0.11146E+00_JPRB, 0.11034E+00_JPRB ,&
&  0.10967E+00_JPRB, 0.10845E+00_JPRB, 0.10750E+00_JPRB, 0.10663E+00_JPRB, 0.10537E+00_JPRB, 0.10427E+00_JPRB, 0.10304E+00_JPRB ,&
&  0.10195E+00_JPRB, 0.10134E+00_JPRB, 0.10098E+00_JPRB, 0.10084E+00_JPRB, 0.10102E+00_JPRB, 0.10091E+00_JPRB, 0.10108E+00_JPRB ,&
&  0.10130E+00_JPRB, 0.10166E+00_JPRB, 0.10163E+00_JPRB, 0.10187E+00_JPRB, 0.10264E+00_JPRB, 0.10371E+00_JPRB, 0.10634E+00_JPRB ,&
&  0.10939E+00_JPRB, 0.11333E+00_JPRB, 0.11767E+00_JPRB, 0.12279E+00_JPRB, 0.12794E+00_JPRB, 0.13317E+00_JPRB, 0.13887E+00_JPRB ,&
&  0.14632E+00_JPRB, 0.15299E+00_JPRB, 0.16064E+00_JPRB, 0.17219E+00_JPRB, 0.19639E+00_JPRB, 0.21016E+00_JPRB, 0.22552E+00_JPRB ,&
&  0.25273E+00_JPRB, 0.29064E+00_JPRB, 0.39386E+00_JPRB, 0.42682E+00_JPRB, 0.44412E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.20492E+00_JPRB, 0.20521E+00_JPRB, 0.20626E+00_JPRB, 0.21312E+00_JPRB, 0.22669E+00_JPRB, 0.30378E+00_JPRB, 0.29531E+00_JPRB ,&
&  0.28838E+00_JPRB, 0.25996E+00_JPRB, 0.25614E+00_JPRB, 0.24889E+00_JPRB, 0.23924E+00_JPRB, 0.23119E+00_JPRB, 0.22632E+00_JPRB ,&
&  0.22109E+00_JPRB, 0.21542E+00_JPRB, 0.21201E+00_JPRB, 0.20871E+00_JPRB, 0.20550E+00_JPRB, 0.20195E+00_JPRB, 0.19925E+00_JPRB ,&
&  0.19691E+00_JPRB, 0.19531E+00_JPRB, 0.19353E+00_JPRB, 0.19010E+00_JPRB, 0.18857E+00_JPRB, 0.18548E+00_JPRB, 0.18083E+00_JPRB ,&
&  0.17479E+00_JPRB, 0.17144E+00_JPRB, 0.16772E+00_JPRB, 0.16437E+00_JPRB, 0.16681E+00_JPRB, 0.16567E+00_JPRB, 0.16341E+00_JPRB ,&
&  0.15993E+00_JPRB, 0.15635E+00_JPRB, 0.15172E+00_JPRB, 0.14731E+00_JPRB, 0.14359E+00_JPRB, 0.14053E+00_JPRB, 0.13785E+00_JPRB ,&
&  0.13572E+00_JPRB, 0.13391E+00_JPRB, 0.13155E+00_JPRB, 0.12938E+00_JPRB, 0.12709E+00_JPRB, 0.12507E+00_JPRB, 0.12305E+00_JPRB ,&
&  0.12143E+00_JPRB, 0.11991E+00_JPRB, 0.11832E+00_JPRB, 0.11668E+00_JPRB, 0.11538E+00_JPRB, 0.11369E+00_JPRB, 0.11242E+00_JPRB ,&
&  0.11128E+00_JPRB, 0.10989E+00_JPRB, 0.10886E+00_JPRB, 0.10760E+00_JPRB, 0.10666E+00_JPRB, 0.10589E+00_JPRB, 0.10514E+00_JPRB ,&
&  0.10413E+00_JPRB, 0.10346E+00_JPRB, 0.10318E+00_JPRB, 0.10295E+00_JPRB, 0.10260E+00_JPRB, 0.10259E+00_JPRB, 0.10285E+00_JPRB ,&
&  0.10324E+00_JPRB, 0.10369E+00_JPRB, 0.10449E+00_JPRB, 0.10546E+00_JPRB, 0.10690E+00_JPRB, 0.10880E+00_JPRB, 0.11184E+00_JPRB ,&
&  0.11516E+00_JPRB, 0.11885E+00_JPRB, 0.12271E+00_JPRB, 0.12718E+00_JPRB, 0.13167E+00_JPRB, 0.13627E+00_JPRB, 0.14152E+00_JPRB ,&
&  0.14724E+00_JPRB, 0.15161E+00_JPRB, 0.15559E+00_JPRB, 0.15982E+00_JPRB, 0.16830E+00_JPRB, 0.17472E+00_JPRB, 0.17973E+00_JPRB ,&
&  0.18533E+00_JPRB, 0.19284E+00_JPRB, 0.19773E+00_JPRB, 0.20116E+00_JPRB, 0.20332E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.17823E+00_JPRB, 0.18071E+00_JPRB, 0.18237E+00_JPRB, 0.18343E+00_JPRB, 0.18322E+00_JPRB, 0.18414E+00_JPRB, 0.18512E+00_JPRB ,&
&  0.18640E+00_JPRB, 0.18797E+00_JPRB, 0.18905E+00_JPRB, 0.19006E+00_JPRB, 0.19056E+00_JPRB, 0.19057E+00_JPRB, 0.19057E+00_JPRB ,&
&  0.19039E+00_JPRB, 0.19016E+00_JPRB, 0.18717E+00_JPRB, 0.18448E+00_JPRB, 0.18259E+00_JPRB, 0.18157E+00_JPRB, 0.18062E+00_JPRB ,&
&  0.17981E+00_JPRB, 0.17866E+00_JPRB, 0.17725E+00_JPRB, 0.17509E+00_JPRB, 0.17189E+00_JPRB, 0.16878E+00_JPRB, 0.16559E+00_JPRB ,&
&  0.16272E+00_JPRB, 0.16034E+00_JPRB, 0.15814E+00_JPRB, 0.15592E+00_JPRB, 0.15452E+00_JPRB, 0.15295E+00_JPRB, 0.15135E+00_JPRB ,&
&  0.14937E+00_JPRB, 0.14726E+00_JPRB, 0.14524E+00_JPRB, 0.14318E+00_JPRB, 0.14120E+00_JPRB, 0.13923E+00_JPRB, 0.13780E+00_JPRB ,&
&  0.13629E+00_JPRB, 0.13513E+00_JPRB, 0.13386E+00_JPRB, 0.13270E+00_JPRB, 0.13147E+00_JPRB, 0.13032E+00_JPRB, 0.12904E+00_JPRB ,&
&  0.12816E+00_JPRB, 0.12725E+00_JPRB, 0.12632E+00_JPRB, 0.12546E+00_JPRB, 0.12460E+00_JPRB, 0.12377E+00_JPRB, 0.12283E+00_JPRB ,&
&  0.12197E+00_JPRB, 0.12141E+00_JPRB, 0.12083E+00_JPRB, 0.12018E+00_JPRB, 0.11957E+00_JPRB, 0.11915E+00_JPRB, 0.11878E+00_JPRB ,&
&  0.11843E+00_JPRB, 0.11839E+00_JPRB, 0.11854E+00_JPRB, 0.11869E+00_JPRB, 0.11884E+00_JPRB, 0.11920E+00_JPRB, 0.11976E+00_JPRB ,&
&  0.12041E+00_JPRB, 0.12111E+00_JPRB, 0.12203E+00_JPRB, 0.12333E+00_JPRB, 0.12477E+00_JPRB, 0.12642E+00_JPRB, 0.12824E+00_JPRB ,&
&  0.13020E+00_JPRB, 0.13231E+00_JPRB, 0.13455E+00_JPRB, 0.13680E+00_JPRB, 0.13920E+00_JPRB, 0.14160E+00_JPRB, 0.14390E+00_JPRB ,&
&  0.14629E+00_JPRB, 0.14865E+00_JPRB, 0.15103E+00_JPRB, 0.15320E+00_JPRB, 0.15645E+00_JPRB, 0.15857E+00_JPRB, 0.16080E+00_JPRB ,&
&  0.16293E+00_JPRB, 0.16537E+00_JPRB, 0.16784E+00_JPRB, 0.17092E+00_JPRB, 0.17395E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB, 0.14506E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB, 0.26952E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.27404E+00_JPRB, 0.27354E+00_JPRB, 0.27274E+00_JPRB, 0.27211E+00_JPRB, 0.27123E+00_JPRB, 0.27066E+00_JPRB, 0.27001E+00_JPRB ,&
&  0.26951E+00_JPRB, 0.26898E+00_JPRB, 0.26872E+00_JPRB, 0.26815E+00_JPRB, 0.26769E+00_JPRB, 0.26725E+00_JPRB, 0.26696E+00_JPRB ,&
&  0.26660E+00_JPRB, 0.26636E+00_JPRB, 0.26613E+00_JPRB, 0.26595E+00_JPRB, 0.26580E+00_JPRB, 0.26553E+00_JPRB, 0.26501E+00_JPRB ,&
&  0.26389E+00_JPRB, 0.26253E+00_JPRB, 0.26141E+00_JPRB, 0.26082E+00_JPRB, 0.26057E+00_JPRB, 0.26040E+00_JPRB, 0.26030E+00_JPRB ,&
&  0.26044E+00_JPRB, 0.26090E+00_JPRB, 0.26148E+00_JPRB, 0.26203E+00_JPRB, 0.26266E+00_JPRB, 0.26368E+00_JPRB, 0.26443E+00_JPRB ,&
&  0.26501E+00_JPRB, 0.26553E+00_JPRB, 0.26643E+00_JPRB, 0.26718E+00_JPRB, 0.26804E+00_JPRB, 0.26886E+00_JPRB, 0.27000E+00_JPRB ,&
&  0.27115E+00_JPRB, 0.27225E+00_JPRB, 0.27333E+00_JPRB, 0.27436E+00_JPRB, 0.27535E+00_JPRB, 0.27641E+00_JPRB, 0.27759E+00_JPRB ,&
&  0.27876E+00_JPRB, 0.27979E+00_JPRB, 0.28072E+00_JPRB, 0.28134E+00_JPRB, 0.28180E+00_JPRB, 0.28184E+00_JPRB, 0.28180E+00_JPRB ,&
&  0.28138E+00_JPRB, 0.28125E+00_JPRB, 0.28082E+00_JPRB, 0.28042E+00_JPRB, 0.27973E+00_JPRB, 0.27943E+00_JPRB, 0.27896E+00_JPRB ,&
&  0.27885E+00_JPRB, 0.27856E+00_JPRB, 0.27848E+00_JPRB, 0.27834E+00_JPRB, 0.27850E+00_JPRB, 0.27884E+00_JPRB, 0.27922E+00_JPRB ,&
&  0.27970E+00_JPRB, 0.28017E+00_JPRB, 0.28078E+00_JPRB, 0.28076E+00_JPRB, 0.28073E+00_JPRB, 0.28074E+00_JPRB, 0.28096E+00_JPRB ,&
&  0.28102E+00_JPRB, 0.28110E+00_JPRB, 0.28127E+00_JPRB, 0.28137E+00_JPRB, 0.28127E+00_JPRB, 0.28107E+00_JPRB, 0.28089E+00_JPRB ,&
&  0.28056E+00_JPRB, 0.28030E+00_JPRB, 0.27999E+00_JPRB, 0.27976E+00_JPRB, 0.27924E+00_JPRB, 0.27870E+00_JPRB, 0.27797E+00_JPRB ,&
&  0.27742E+00_JPRB, 0.27670E+00_JPRB, 0.27613E+00_JPRB, 0.27544E+00_JPRB, 0.27480E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.29392E+00_JPRB, 0.29283E+00_JPRB, 0.29167E+00_JPRB, 0.29056E+00_JPRB, 0.28901E+00_JPRB, 0.28744E+00_JPRB, 0.28580E+00_JPRB ,&
&  0.28412E+00_JPRB, 0.28220E+00_JPRB, 0.28023E+00_JPRB, 0.27814E+00_JPRB, 0.27587E+00_JPRB, 0.27370E+00_JPRB, 0.27199E+00_JPRB ,&
&  0.27056E+00_JPRB, 0.26900E+00_JPRB, 0.26794E+00_JPRB, 0.26641E+00_JPRB, 0.26482E+00_JPRB, 0.26355E+00_JPRB, 0.26315E+00_JPRB ,&
&  0.26181E+00_JPRB, 0.25994E+00_JPRB, 0.25798E+00_JPRB, 0.25581E+00_JPRB, 0.25388E+00_JPRB, 0.25185E+00_JPRB, 0.24964E+00_JPRB ,&
&  0.24738E+00_JPRB, 0.24592E+00_JPRB, 0.24483E+00_JPRB, 0.24373E+00_JPRB, 0.24261E+00_JPRB, 0.24299E+00_JPRB, 0.24319E+00_JPRB ,&
&  0.24353E+00_JPRB, 0.24429E+00_JPRB, 0.24657E+00_JPRB, 0.24870E+00_JPRB, 0.25088E+00_JPRB, 0.25303E+00_JPRB, 0.25630E+00_JPRB ,&
&  0.25938E+00_JPRB, 0.26302E+00_JPRB, 0.26722E+00_JPRB, 0.27114E+00_JPRB, 0.27416E+00_JPRB, 0.27600E+00_JPRB, 0.27821E+00_JPRB ,&
&  0.27949E+00_JPRB, 0.28129E+00_JPRB, 0.28240E+00_JPRB, 0.28266E+00_JPRB, 0.28200E+00_JPRB, 0.28135E+00_JPRB, 0.28110E+00_JPRB ,&
&  0.28053E+00_JPRB, 0.28101E+00_JPRB, 0.28137E+00_JPRB, 0.28171E+00_JPRB, 0.28120E+00_JPRB, 0.28159E+00_JPRB, 0.28178E+00_JPRB ,&
&  0.28297E+00_JPRB, 0.28525E+00_JPRB, 0.28729E+00_JPRB, 0.28965E+00_JPRB, 0.29277E+00_JPRB, 0.29672E+00_JPRB, 0.29963E+00_JPRB ,&
&  0.30263E+00_JPRB, 0.30444E+00_JPRB, 0.30534E+00_JPRB, 0.30496E+00_JPRB, 0.30467E+00_JPRB, 0.30452E+00_JPRB, 0.30520E+00_JPRB ,&
&  0.30514E+00_JPRB, 0.30534E+00_JPRB, 0.30558E+00_JPRB, 0.30558E+00_JPRB, 0.30475E+00_JPRB, 0.30393E+00_JPRB, 0.30348E+00_JPRB ,&
&  0.30286E+00_JPRB, 0.30223E+00_JPRB, 0.30166E+00_JPRB, 0.30126E+00_JPRB, 0.30061E+00_JPRB, 0.29997E+00_JPRB, 0.29936E+00_JPRB ,&
&  0.29878E+00_JPRB, 0.29791E+00_JPRB, 0.29694E+00_JPRB, 0.29598E+00_JPRB, 0.29509E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.30250E+00_JPRB, 0.30252E+00_JPRB, 0.30246E+00_JPRB, 0.30224E+00_JPRB, 0.30145E+00_JPRB, 0.30041E+00_JPRB, 0.29900E+00_JPRB ,&
&  0.29725E+00_JPRB, 0.29467E+00_JPRB, 0.29198E+00_JPRB, 0.28917E+00_JPRB, 0.28656E+00_JPRB, 0.28451E+00_JPRB, 0.28253E+00_JPRB ,&
&  0.28030E+00_JPRB, 0.27728E+00_JPRB, 0.27402E+00_JPRB, 0.26957E+00_JPRB, 0.26541E+00_JPRB, 0.26184E+00_JPRB, 0.25783E+00_JPRB ,&
&  0.25212E+00_JPRB, 0.24616E+00_JPRB, 0.24100E+00_JPRB, 0.23562E+00_JPRB, 0.23161E+00_JPRB, 0.21685E+00_JPRB, 0.21284E+00_JPRB ,&
&  0.21045E+00_JPRB, 0.20809E+00_JPRB, 0.20768E+00_JPRB, 0.20781E+00_JPRB, 0.21031E+00_JPRB, 0.20856E+00_JPRB, 0.20767E+00_JPRB ,&
&  0.19950E+00_JPRB, 0.20010E+00_JPRB, 0.20177E+00_JPRB, 0.20403E+00_JPRB, 0.20902E+00_JPRB, 0.21485E+00_JPRB, 0.22466E+00_JPRB ,&
&  0.24272E+00_JPRB, 0.25037E+00_JPRB, 0.26168E+00_JPRB, 0.27019E+00_JPRB, 0.27472E+00_JPRB, 0.27677E+00_JPRB, 0.27865E+00_JPRB ,&
&  0.28078E+00_JPRB, 0.28375E+00_JPRB, 0.28521E+00_JPRB, 0.28607E+00_JPRB, 0.28549E+00_JPRB, 0.28428E+00_JPRB, 0.28346E+00_JPRB ,&
&  0.28217E+00_JPRB, 0.28223E+00_JPRB, 0.28296E+00_JPRB, 0.28568E+00_JPRB, 0.28959E+00_JPRB, 0.29455E+00_JPRB, 0.30030E+00_JPRB ,&
&  0.30642E+00_JPRB, 0.31362E+00_JPRB, 0.31903E+00_JPRB, 0.32452E+00_JPRB, 0.33040E+00_JPRB, 0.33538E+00_JPRB, 0.33911E+00_JPRB ,&
&  0.34177E+00_JPRB, 0.34089E+00_JPRB, 0.33704E+00_JPRB, 0.33297E+00_JPRB, 0.32889E+00_JPRB, 0.32581E+00_JPRB, 0.32447E+00_JPRB ,&
&  0.32329E+00_JPRB, 0.32256E+00_JPRB, 0.32199E+00_JPRB, 0.32033E+00_JPRB, 0.31883E+00_JPRB, 0.31707E+00_JPRB, 0.31533E+00_JPRB ,&
&  0.31332E+00_JPRB, 0.31163E+00_JPRB, 0.31022E+00_JPRB, 0.30889E+00_JPRB, 0.30790E+00_JPRB, 0.30644E+00_JPRB, 0.30530E+00_JPRB ,&
&  0.30405E+00_JPRB, 0.30386E+00_JPRB, 0.30328E+00_JPRB, 0.30291E+00_JPRB, 0.30251E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.28726E+00_JPRB, 0.28780E+00_JPRB, 0.28867E+00_JPRB, 0.28928E+00_JPRB, 0.29028E+00_JPRB, 0.29098E+00_JPRB, 0.29185E+00_JPRB ,&
&  0.29238E+00_JPRB, 0.29294E+00_JPRB, 0.29288E+00_JPRB, 0.29247E+00_JPRB, 0.29150E+00_JPRB, 0.29062E+00_JPRB, 0.28951E+00_JPRB ,&
&  0.28774E+00_JPRB, 0.28387E+00_JPRB, 0.27866E+00_JPRB, 0.27272E+00_JPRB, 0.26593E+00_JPRB, 0.25862E+00_JPRB, 0.25016E+00_JPRB ,&
&  0.24145E+00_JPRB, 0.23352E+00_JPRB, 0.22149E+00_JPRB, 0.21768E+00_JPRB, 0.20966E+00_JPRB, 0.20382E+00_JPRB, 0.19994E+00_JPRB ,&
&  0.19508E+00_JPRB, 0.19171E+00_JPRB, 0.18811E+00_JPRB, 0.18405E+00_JPRB, 0.18063E+00_JPRB, 0.17502E+00_JPRB, 0.17191E+00_JPRB ,&
&  0.16973E+00_JPRB, 0.16873E+00_JPRB, 0.16882E+00_JPRB, 0.17023E+00_JPRB, 0.17363E+00_JPRB, 0.17805E+00_JPRB, 0.18678E+00_JPRB ,&
&  0.20522E+00_JPRB, 0.23206E+00_JPRB, 0.27189E+00_JPRB, 0.27882E+00_JPRB, 0.27753E+00_JPRB, 0.28229E+00_JPRB, 0.28707E+00_JPRB ,&
&  0.29152E+00_JPRB, 0.29398E+00_JPRB, 0.29567E+00_JPRB, 0.29681E+00_JPRB, 0.29840E+00_JPRB, 0.29977E+00_JPRB, 0.30101E+00_JPRB ,&
&  0.30219E+00_JPRB, 0.30536E+00_JPRB, 0.30982E+00_JPRB, 0.31542E+00_JPRB, 0.32059E+00_JPRB, 0.32398E+00_JPRB, 0.32798E+00_JPRB ,&
&  0.33122E+00_JPRB, 0.33525E+00_JPRB, 0.34087E+00_JPRB, 0.34768E+00_JPRB, 0.35510E+00_JPRB, 0.35945E+00_JPRB, 0.36241E+00_JPRB ,&
&  0.36356E+00_JPRB, 0.35977E+00_JPRB, 0.35387E+00_JPRB, 0.34746E+00_JPRB, 0.34305E+00_JPRB, 0.34347E+00_JPRB, 0.34116E+00_JPRB ,&
&  0.33973E+00_JPRB, 0.33608E+00_JPRB, 0.33112E+00_JPRB, 0.32542E+00_JPRB, 0.32108E+00_JPRB, 0.31701E+00_JPRB, 0.31419E+00_JPRB ,&
&  0.31163E+00_JPRB, 0.30911E+00_JPRB, 0.30623E+00_JPRB, 0.30249E+00_JPRB, 0.29893E+00_JPRB, 0.29522E+00_JPRB, 0.29237E+00_JPRB ,&
&  0.28977E+00_JPRB, 0.28837E+00_JPRB, 0.28724E+00_JPRB, 0.28694E+00_JPRB, 0.28671E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.27888E+00_JPRB, 0.28182E+00_JPRB, 0.28459E+00_JPRB, 0.28665E+00_JPRB, 0.28827E+00_JPRB, 0.28951E+00_JPRB, 0.28989E+00_JPRB ,&
&  0.29004E+00_JPRB, 0.28992E+00_JPRB, 0.29006E+00_JPRB, 0.29012E+00_JPRB, 0.29046E+00_JPRB, 0.29111E+00_JPRB, 0.29156E+00_JPRB ,&
&  0.29193E+00_JPRB, 0.28884E+00_JPRB, 0.28271E+00_JPRB, 0.27591E+00_JPRB, 0.26890E+00_JPRB, 0.26015E+00_JPRB, 0.25002E+00_JPRB ,&
&  0.24132E+00_JPRB, 0.23462E+00_JPRB, 0.22632E+00_JPRB, 0.22100E+00_JPRB, 0.21460E+00_JPRB, 0.21130E+00_JPRB, 0.20644E+00_JPRB ,&
&  0.20603E+00_JPRB, 0.19126E+00_JPRB, 0.18914E+00_JPRB, 0.17985E+00_JPRB, 0.17224E+00_JPRB, 0.16608E+00_JPRB, 0.16159E+00_JPRB ,&
&  0.16013E+00_JPRB, 0.15852E+00_JPRB, 0.15861E+00_JPRB, 0.16068E+00_JPRB, 0.16360E+00_JPRB, 0.16971E+00_JPRB, 0.17703E+00_JPRB ,&
&  0.19430E+00_JPRB, 0.25123E+00_JPRB, 0.27004E+00_JPRB, 0.27553E+00_JPRB, 0.27921E+00_JPRB, 0.28886E+00_JPRB, 0.29749E+00_JPRB ,&
&  0.30735E+00_JPRB, 0.31197E+00_JPRB, 0.31477E+00_JPRB, 0.31506E+00_JPRB, 0.32273E+00_JPRB, 0.32963E+00_JPRB, 0.33465E+00_JPRB ,&
&  0.33968E+00_JPRB, 0.34419E+00_JPRB, 0.34949E+00_JPRB, 0.35130E+00_JPRB, 0.34942E+00_JPRB, 0.34617E+00_JPRB, 0.34336E+00_JPRB ,&
&  0.34243E+00_JPRB, 0.34210E+00_JPRB, 0.34818E+00_JPRB, 0.35785E+00_JPRB, 0.37015E+00_JPRB, 0.37921E+00_JPRB, 0.38267E+00_JPRB ,&
&  0.38278E+00_JPRB, 0.38038E+00_JPRB, 0.38088E+00_JPRB, 0.38359E+00_JPRB, 0.37874E+00_JPRB, 0.37000E+00_JPRB, 0.36451E+00_JPRB ,&
&  0.35919E+00_JPRB, 0.34891E+00_JPRB, 0.34085E+00_JPRB, 0.33393E+00_JPRB, 0.32834E+00_JPRB, 0.32295E+00_JPRB, 0.31786E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.30595E+00_JPRB, 0.29962E+00_JPRB, 0.29153E+00_JPRB, 0.28501E+00_JPRB, 0.27826E+00_JPRB, 0.27434E+00_JPRB ,&
&  0.27114E+00_JPRB, 0.27080E+00_JPRB, 0.27085E+00_JPRB, 0.27315E+00_JPRB, 0.27514E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.27651E+00_JPRB, 0.28319E+00_JPRB, 0.28860E+00_JPRB, 0.29243E+00_JPRB, 0.29475E+00_JPRB, 0.29590E+00_JPRB, 0.29548E+00_JPRB ,&
&  0.29463E+00_JPRB, 0.29321E+00_JPRB, 0.29233E+00_JPRB, 0.29118E+00_JPRB, 0.29087E+00_JPRB, 0.29001E+00_JPRB, 0.28965E+00_JPRB ,&
&  0.28960E+00_JPRB, 0.28893E+00_JPRB, 0.28547E+00_JPRB, 0.27896E+00_JPRB, 0.27164E+00_JPRB, 0.26366E+00_JPRB, 0.25316E+00_JPRB ,&
&  0.24328E+00_JPRB, 0.23717E+00_JPRB, 0.23076E+00_JPRB, 0.22499E+00_JPRB, 0.22185E+00_JPRB, 0.21736E+00_JPRB, 0.21741E+00_JPRB ,&
&  0.21606E+00_JPRB, 0.21178E+00_JPRB, 0.20636E+00_JPRB, 0.20112E+00_JPRB, 0.19456E+00_JPRB, 0.17292E+00_JPRB, 0.16912E+00_JPRB ,&
&  0.16306E+00_JPRB, 0.15814E+00_JPRB, 0.15694E+00_JPRB, 0.15769E+00_JPRB, 0.16007E+00_JPRB, 0.16671E+00_JPRB, 0.17601E+00_JPRB ,&
&  0.19057E+00_JPRB, 0.25678E+00_JPRB, 0.26918E+00_JPRB, 0.27856E+00_JPRB, 0.29265E+00_JPRB, 0.30602E+00_JPRB, 0.31737E+00_JPRB ,&
&  0.32705E+00_JPRB, 0.33369E+00_JPRB, 0.34177E+00_JPRB, 0.34789E+00_JPRB, 0.35506E+00_JPRB, 0.36063E+00_JPRB, 0.36325E+00_JPRB ,&
&  0.36425E+00_JPRB, 0.36173E+00_JPRB, 0.36941E+00_JPRB, 0.37178E+00_JPRB, 0.36863E+00_JPRB, 0.37386E+00_JPRB, 0.37504E+00_JPRB ,&
&  0.37066E+00_JPRB, 0.36324E+00_JPRB, 0.36670E+00_JPRB, 0.37989E+00_JPRB, 0.40063E+00_JPRB, 0.41635E+00_JPRB, 0.41875E+00_JPRB ,&
&  0.41777E+00_JPRB, 0.41362E+00_JPRB, 0.40929E+00_JPRB, 0.40732E+00_JPRB, 0.39590E+00_JPRB, 0.37989E+00_JPRB, 0.36837E+00_JPRB ,&
&  0.35905E+00_JPRB, 0.35727E+00_JPRB, 0.35207E+00_JPRB, 0.34593E+00_JPRB, 0.33889E+00_JPRB, 0.33108E+00_JPRB, 0.32105E+00_JPRB ,&
&  0.30796E+00_JPRB, 0.29346E+00_JPRB, 0.28226E+00_JPRB, 0.27233E+00_JPRB, 0.26728E+00_JPRB, 0.26139E+00_JPRB, 0.25878E+00_JPRB ,&
&  0.25662E+00_JPRB, 0.25787E+00_JPRB, 0.25998E+00_JPRB, 0.26457E+00_JPRB, 0.26962E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.28348E+00_JPRB, 0.29450E+00_JPRB, 0.30336E+00_JPRB, 0.30845E+00_JPRB, 0.30949E+00_JPRB, 0.30951E+00_JPRB, 0.30801E+00_JPRB ,&
&  0.30692E+00_JPRB, 0.30551E+00_JPRB, 0.30362E+00_JPRB, 0.30061E+00_JPRB, 0.29740E+00_JPRB, 0.29366E+00_JPRB, 0.29104E+00_JPRB ,&
&  0.28817E+00_JPRB, 0.28311E+00_JPRB, 0.27973E+00_JPRB, 0.27644E+00_JPRB, 0.27093E+00_JPRB, 0.26407E+00_JPRB, 0.25535E+00_JPRB ,&
&  0.24598E+00_JPRB, 0.23920E+00_JPRB, 0.23487E+00_JPRB, 0.22956E+00_JPRB, 0.22482E+00_JPRB, 0.22134E+00_JPRB, 0.21986E+00_JPRB ,&
&  0.22001E+00_JPRB, 0.21759E+00_JPRB, 0.21627E+00_JPRB, 0.20943E+00_JPRB, 0.20004E+00_JPRB, 0.19668E+00_JPRB, 0.18204E+00_JPRB ,&
&  0.16985E+00_JPRB, 0.16146E+00_JPRB, 0.15942E+00_JPRB, 0.15970E+00_JPRB, 0.16362E+00_JPRB, 0.17217E+00_JPRB, 0.19116E+00_JPRB ,&
&  0.22400E+00_JPRB, 0.27482E+00_JPRB, 0.28503E+00_JPRB, 0.30006E+00_JPRB, 0.31707E+00_JPRB, 0.33306E+00_JPRB, 0.34456E+00_JPRB ,&
&  0.35212E+00_JPRB, 0.35832E+00_JPRB, 0.36788E+00_JPRB, 0.37604E+00_JPRB, 0.38997E+00_JPRB, 0.40105E+00_JPRB, 0.40891E+00_JPRB ,&
&  0.41631E+00_JPRB, 0.39859E+00_JPRB, 0.39895E+00_JPRB, 0.39938E+00_JPRB, 0.40800E+00_JPRB, 0.41339E+00_JPRB, 0.40442E+00_JPRB ,&
&  0.39918E+00_JPRB, 0.41024E+00_JPRB, 0.40965E+00_JPRB, 0.41600E+00_JPRB, 0.43239E+00_JPRB, 0.45029E+00_JPRB, 0.45513E+00_JPRB ,&
&  0.44459E+00_JPRB, 0.44550E+00_JPRB, 0.46908E+00_JPRB, 0.44709E+00_JPRB, 0.43307E+00_JPRB, 0.41223E+00_JPRB, 0.38770E+00_JPRB ,&
&  0.37444E+00_JPRB, 0.36700E+00_JPRB, 0.36317E+00_JPRB, 0.35759E+00_JPRB, 0.34939E+00_JPRB, 0.33735E+00_JPRB, 0.31905E+00_JPRB ,&
&  0.29596E+00_JPRB, 0.27893E+00_JPRB, 0.26727E+00_JPRB, 0.25993E+00_JPRB, 0.25586E+00_JPRB, 0.25125E+00_JPRB, 0.24811E+00_JPRB ,&
&  0.24706E+00_JPRB, 0.25025E+00_JPRB, 0.25636E+00_JPRB, 0.26557E+00_JPRB, 0.27496E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.29872E+00_JPRB, 0.31079E+00_JPRB, 0.31829E+00_JPRB, 0.31979E+00_JPRB, 0.31671E+00_JPRB, 0.30888E+00_JPRB, 0.30207E+00_JPRB ,&
&  0.29978E+00_JPRB, 0.29952E+00_JPRB, 0.29998E+00_JPRB, 0.29767E+00_JPRB, 0.29488E+00_JPRB, 0.29207E+00_JPRB, 0.28997E+00_JPRB ,&
&  0.28794E+00_JPRB, 0.28617E+00_JPRB, 0.28423E+00_JPRB, 0.27923E+00_JPRB, 0.27276E+00_JPRB, 0.26518E+00_JPRB, 0.25722E+00_JPRB ,&
&  0.24970E+00_JPRB, 0.24421E+00_JPRB, 0.24003E+00_JPRB, 0.23666E+00_JPRB, 0.23199E+00_JPRB, 0.22917E+00_JPRB, 0.22670E+00_JPRB ,&
&  0.22459E+00_JPRB, 0.22498E+00_JPRB, 0.22440E+00_JPRB, 0.22135E+00_JPRB, 0.21417E+00_JPRB, 0.20500E+00_JPRB, 0.19179E+00_JPRB ,&
&  0.18063E+00_JPRB, 0.17339E+00_JPRB, 0.16930E+00_JPRB, 0.16920E+00_JPRB, 0.17545E+00_JPRB, 0.18950E+00_JPRB, 0.21869E+00_JPRB ,&
&  0.27566E+00_JPRB, 0.29350E+00_JPRB, 0.30483E+00_JPRB, 0.32070E+00_JPRB, 0.33570E+00_JPRB, 0.34882E+00_JPRB, 0.36096E+00_JPRB ,&
&  0.37079E+00_JPRB, 0.38028E+00_JPRB, 0.39098E+00_JPRB, 0.39778E+00_JPRB, 0.40941E+00_JPRB, 0.40620E+00_JPRB, 0.42615E+00_JPRB ,&
&  0.48800E+00_JPRB, 0.48385E+00_JPRB, 0.45087E+00_JPRB, 0.44946E+00_JPRB, 0.45594E+00_JPRB, 0.46106E+00_JPRB, 0.46836E+00_JPRB ,&
&  0.47308E+00_JPRB, 0.47243E+00_JPRB, 0.47111E+00_JPRB, 0.46677E+00_JPRB, 0.46950E+00_JPRB, 0.47463E+00_JPRB, 0.47327E+00_JPRB ,&
&  0.46757E+00_JPRB, 0.48348E+00_JPRB, 0.52683E+00_JPRB, 0.44155E+00_JPRB, 0.40457E+00_JPRB, 0.39334E+00_JPRB, 0.37799E+00_JPRB ,&
&  0.36820E+00_JPRB, 0.36531E+00_JPRB, 0.36802E+00_JPRB, 0.36578E+00_JPRB, 0.34569E+00_JPRB, 0.31889E+00_JPRB, 0.29146E+00_JPRB ,&
&  0.27636E+00_JPRB, 0.26794E+00_JPRB, 0.25927E+00_JPRB, 0.25332E+00_JPRB, 0.24999E+00_JPRB, 0.24645E+00_JPRB, 0.24324E+00_JPRB ,&
&  0.24606E+00_JPRB, 0.25240E+00_JPRB, 0.26199E+00_JPRB, 0.27668E+00_JPRB, 0.28859E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.29932E+00_JPRB, 0.31606E+00_JPRB, 0.32417E+00_JPRB, 0.32088E+00_JPRB, 0.31623E+00_JPRB, 0.29909E+00_JPRB, 0.28218E+00_JPRB ,&
&  0.27617E+00_JPRB, 0.27201E+00_JPRB, 0.27111E+00_JPRB, 0.27047E+00_JPRB, 0.27096E+00_JPRB, 0.27282E+00_JPRB, 0.27416E+00_JPRB ,&
&  0.27284E+00_JPRB, 0.27131E+00_JPRB, 0.27090E+00_JPRB, 0.27070E+00_JPRB, 0.26769E+00_JPRB, 0.26157E+00_JPRB, 0.25535E+00_JPRB ,&
&  0.24784E+00_JPRB, 0.24292E+00_JPRB, 0.23976E+00_JPRB, 0.23781E+00_JPRB, 0.23710E+00_JPRB, 0.23796E+00_JPRB, 0.23882E+00_JPRB ,&
&  0.23704E+00_JPRB, 0.23535E+00_JPRB, 0.23587E+00_JPRB, 0.23525E+00_JPRB, 0.22895E+00_JPRB, 0.21856E+00_JPRB, 0.20338E+00_JPRB ,&
&  0.19182E+00_JPRB, 0.18672E+00_JPRB, 0.18792E+00_JPRB, 0.19749E+00_JPRB, 0.22346E+00_JPRB, 0.27433E+00_JPRB, 0.29308E+00_JPRB ,&
&  0.30889E+00_JPRB, 0.31949E+00_JPRB, 0.33110E+00_JPRB, 0.34702E+00_JPRB, 0.36041E+00_JPRB, 0.37188E+00_JPRB, 0.38307E+00_JPRB ,&
&  0.39861E+00_JPRB, 0.41424E+00_JPRB, 0.43451E+00_JPRB, 0.39887E+00_JPRB, 0.40600E+00_JPRB, 0.47049E+00_JPRB, 0.47830E+00_JPRB ,&
&  0.50683E+00_JPRB, 0.50905E+00_JPRB, 0.50558E+00_JPRB, 0.53177E+00_JPRB, 0.53552E+00_JPRB, 0.54389E+00_JPRB, 0.53905E+00_JPRB ,&
&  0.52743E+00_JPRB, 0.51109E+00_JPRB, 0.49676E+00_JPRB, 0.48522E+00_JPRB, 0.48674E+00_JPRB, 0.49578E+00_JPRB, 0.48661E+00_JPRB ,&
&  0.47659E+00_JPRB, 0.47170E+00_JPRB, 0.44069E+00_JPRB, 0.38934E+00_JPRB, 0.37104E+00_JPRB, 0.36794E+00_JPRB, 0.37114E+00_JPRB ,&
&  0.36883E+00_JPRB, 0.36926E+00_JPRB, 0.36907E+00_JPRB, 0.36284E+00_JPRB, 0.32836E+00_JPRB, 0.29621E+00_JPRB, 0.27084E+00_JPRB ,&
&  0.25731E+00_JPRB, 0.25342E+00_JPRB, 0.24865E+00_JPRB, 0.24797E+00_JPRB, 0.24647E+00_JPRB, 0.24722E+00_JPRB, 0.24570E+00_JPRB ,&
&  0.24771E+00_JPRB, 0.25314E+00_JPRB, 0.26463E+00_JPRB, 0.27798E+00_JPRB, 0.28854E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.32253E+00_JPRB, 0.32933E+00_JPRB, 0.32697E+00_JPRB, 0.32615E+00_JPRB, 0.32009E+00_JPRB, 0.31201E+00_JPRB, 0.30091E+00_JPRB ,&
&  0.30351E+00_JPRB, 0.28611E+00_JPRB, 0.27345E+00_JPRB, 0.26185E+00_JPRB, 0.25465E+00_JPRB, 0.24999E+00_JPRB, 0.25174E+00_JPRB ,&
&  0.25778E+00_JPRB, 0.26523E+00_JPRB, 0.26891E+00_JPRB, 0.26726E+00_JPRB, 0.26304E+00_JPRB, 0.25556E+00_JPRB, 0.25066E+00_JPRB ,&
&  0.24472E+00_JPRB, 0.24193E+00_JPRB, 0.24294E+00_JPRB, 0.24647E+00_JPRB, 0.25018E+00_JPRB, 0.25086E+00_JPRB, 0.24817E+00_JPRB ,&
&  0.24382E+00_JPRB, 0.24140E+00_JPRB, 0.24286E+00_JPRB, 0.24554E+00_JPRB, 0.24289E+00_JPRB, 0.22983E+00_JPRB, 0.21514E+00_JPRB ,&
&  0.20309E+00_JPRB, 0.20289E+00_JPRB, 0.22351E+00_JPRB, 0.27672E+00_JPRB, 0.29572E+00_JPRB, 0.31018E+00_JPRB, 0.32249E+00_JPRB ,&
&  0.31928E+00_JPRB, 0.32415E+00_JPRB, 0.34630E+00_JPRB, 0.38847E+00_JPRB, 0.39841E+00_JPRB, 0.40910E+00_JPRB, 0.42067E+00_JPRB ,&
&  0.44169E+00_JPRB, 0.44283E+00_JPRB, 0.45816E+00_JPRB, 0.44605E+00_JPRB, 0.53722E+00_JPRB, 0.71075E+00_JPRB, 0.63088E+00_JPRB ,&
&  0.61736E+00_JPRB, 0.60537E+00_JPRB, 0.62768E+00_JPRB, 0.62568E+00_JPRB, 0.60634E+00_JPRB, 0.59394E+00_JPRB, 0.58132E+00_JPRB ,&
&  0.55141E+00_JPRB, 0.53312E+00_JPRB, 0.51224E+00_JPRB, 0.50717E+00_JPRB, 0.50149E+00_JPRB, 0.50104E+00_JPRB, 0.49553E+00_JPRB ,&
&  0.48714E+00_JPRB, 0.47931E+00_JPRB, 0.44862E+00_JPRB, 0.40827E+00_JPRB, 0.38736E+00_JPRB, 0.36997E+00_JPRB, 0.35928E+00_JPRB ,&
&  0.35360E+00_JPRB, 0.35946E+00_JPRB, 0.36713E+00_JPRB, 0.36364E+00_JPRB, 0.33217E+00_JPRB, 0.29700E+00_JPRB, 0.26760E+00_JPRB ,&
&  0.24880E+00_JPRB, 0.24214E+00_JPRB, 0.23280E+00_JPRB, 0.22953E+00_JPRB, 0.23577E+00_JPRB, 0.24452E+00_JPRB, 0.24915E+00_JPRB ,&
&  0.25372E+00_JPRB, 0.26132E+00_JPRB, 0.27850E+00_JPRB, 0.29773E+00_JPRB, 0.31176E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.32583E+00_JPRB, 0.33476E+00_JPRB, 0.34062E+00_JPRB, 0.34629E+00_JPRB, 0.34950E+00_JPRB, 0.34128E+00_JPRB, 0.32294E+00_JPRB ,&
&  0.33083E+00_JPRB, 0.28820E+00_JPRB, 0.27471E+00_JPRB, 0.26647E+00_JPRB, 0.26515E+00_JPRB, 0.26046E+00_JPRB, 0.25218E+00_JPRB ,&
&  0.25168E+00_JPRB, 0.25783E+00_JPRB, 0.26829E+00_JPRB, 0.27182E+00_JPRB, 0.26940E+00_JPRB, 0.26214E+00_JPRB, 0.25308E+00_JPRB ,&
&  0.24528E+00_JPRB, 0.24333E+00_JPRB, 0.24780E+00_JPRB, 0.25526E+00_JPRB, 0.25925E+00_JPRB, 0.25833E+00_JPRB, 0.25570E+00_JPRB ,&
&  0.25266E+00_JPRB, 0.25004E+00_JPRB, 0.24965E+00_JPRB, 0.25094E+00_JPRB, 0.25174E+00_JPRB, 0.24897E+00_JPRB, 0.23935E+00_JPRB ,&
&  0.22589E+00_JPRB, 0.23429E+00_JPRB, 0.28866E+00_JPRB, 0.31281E+00_JPRB, 0.33288E+00_JPRB, 0.34402E+00_JPRB, 0.35522E+00_JPRB ,&
&  0.37014E+00_JPRB, 0.38735E+00_JPRB, 0.40789E+00_JPRB, 0.43315E+00_JPRB, 0.44996E+00_JPRB, 0.46569E+00_JPRB, 0.48210E+00_JPRB ,&
&  0.52646E+00_JPRB, 0.44640E+00_JPRB, 0.42644E+00_JPRB, 0.45954E+00_JPRB, 0.69633E+00_JPRB, 0.76107E+00_JPRB, 0.75160E+00_JPRB ,&
&  0.74872E+00_JPRB, 0.69519E+00_JPRB, 0.68048E+00_JPRB, 0.68064E+00_JPRB, 0.67251E+00_JPRB, 0.66029E+00_JPRB, 0.64784E+00_JPRB ,&
&  0.62576E+00_JPRB, 0.60098E+00_JPRB, 0.56458E+00_JPRB, 0.53837E+00_JPRB, 0.52028E+00_JPRB, 0.52042E+00_JPRB, 0.50876E+00_JPRB ,&
&  0.49274E+00_JPRB, 0.48649E+00_JPRB, 0.45730E+00_JPRB, 0.42495E+00_JPRB, 0.40071E+00_JPRB, 0.38471E+00_JPRB, 0.37254E+00_JPRB ,&
&  0.36461E+00_JPRB, 0.36646E+00_JPRB, 0.37132E+00_JPRB, 0.37211E+00_JPRB, 0.35193E+00_JPRB, 0.32325E+00_JPRB, 0.29730E+00_JPRB ,&
&  0.26895E+00_JPRB, 0.25465E+00_JPRB, 0.23857E+00_JPRB, 0.23568E+00_JPRB, 0.24498E+00_JPRB, 0.25637E+00_JPRB, 0.26639E+00_JPRB ,&
&  0.27766E+00_JPRB, 0.28712E+00_JPRB, 0.29691E+00_JPRB, 0.30920E+00_JPRB, 0.31686E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.32912E+00_JPRB, 0.34175E+00_JPRB, 0.35405E+00_JPRB, 0.36620E+00_JPRB, 0.37193E+00_JPRB, 0.35702E+00_JPRB, 0.35366E+00_JPRB ,&
&  0.39406E+00_JPRB, 0.34487E+00_JPRB, 0.36407E+00_JPRB, 0.30552E+00_JPRB, 0.28300E+00_JPRB, 0.27675E+00_JPRB, 0.26600E+00_JPRB ,&
&  0.26340E+00_JPRB, 0.26837E+00_JPRB, 0.27370E+00_JPRB, 0.27767E+00_JPRB, 0.27803E+00_JPRB, 0.26720E+00_JPRB, 0.25791E+00_JPRB ,&
&  0.24923E+00_JPRB, 0.24723E+00_JPRB, 0.25292E+00_JPRB, 0.25826E+00_JPRB, 0.26266E+00_JPRB, 0.26354E+00_JPRB, 0.26146E+00_JPRB ,&
&  0.25535E+00_JPRB, 0.25412E+00_JPRB, 0.25665E+00_JPRB, 0.25721E+00_JPRB, 0.25951E+00_JPRB, 0.25920E+00_JPRB, 0.26117E+00_JPRB ,&
&  0.26745E+00_JPRB, 0.27582E+00_JPRB, 0.31453E+00_JPRB, 0.34153E+00_JPRB, 0.36070E+00_JPRB, 0.37084E+00_JPRB, 0.38512E+00_JPRB ,&
&  0.40000E+00_JPRB, 0.41867E+00_JPRB, 0.44646E+00_JPRB, 0.48152E+00_JPRB, 0.51155E+00_JPRB, 0.53605E+00_JPRB, 0.56575E+00_JPRB ,&
&  0.61796E+00_JPRB, 0.49588E+00_JPRB, 0.58032E+00_JPRB, 0.58697E+00_JPRB, 0.72699E+00_JPRB, 0.88541E+00_JPRB, 0.83521E+00_JPRB ,&
&  0.87708E+00_JPRB, 0.80790E+00_JPRB, 0.78319E+00_JPRB, 0.75119E+00_JPRB, 0.73822E+00_JPRB, 0.72706E+00_JPRB, 0.71581E+00_JPRB ,&
&  0.72793E+00_JPRB, 0.65529E+00_JPRB, 0.63585E+00_JPRB, 0.59401E+00_JPRB, 0.56415E+00_JPRB, 0.55724E+00_JPRB, 0.53185E+00_JPRB ,&
&  0.51332E+00_JPRB, 0.50408E+00_JPRB, 0.48424E+00_JPRB, 0.43435E+00_JPRB, 0.41899E+00_JPRB, 0.40541E+00_JPRB, 0.37765E+00_JPRB ,&
&  0.36713E+00_JPRB, 0.35591E+00_JPRB, 0.34552E+00_JPRB, 0.36121E+00_JPRB, 0.34950E+00_JPRB, 0.32442E+00_JPRB, 0.29893E+00_JPRB ,&
&  0.27669E+00_JPRB, 0.26181E+00_JPRB, 0.26616E+00_JPRB, 0.26944E+00_JPRB, 0.26926E+00_JPRB, 0.27321E+00_JPRB, 0.29170E+00_JPRB ,&
&  0.29378E+00_JPRB, 0.30350E+00_JPRB, 0.32091E+00_JPRB, 0.32597E+00_JPRB, 0.32587E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.33707E+00_JPRB, 0.35200E+00_JPRB, 0.36370E+00_JPRB, 0.37923E+00_JPRB, 0.38751E+00_JPRB, 0.38113E+00_JPRB, 0.36414E+00_JPRB ,&
&  0.37323E+00_JPRB, 0.37151E+00_JPRB, 0.37015E+00_JPRB, 0.37213E+00_JPRB, 0.37428E+00_JPRB, 0.33749E+00_JPRB, 0.27624E+00_JPRB ,&
&  0.25741E+00_JPRB, 0.26240E+00_JPRB, 0.27499E+00_JPRB, 0.28256E+00_JPRB, 0.28024E+00_JPRB, 0.27171E+00_JPRB, 0.26520E+00_JPRB ,&
&  0.26148E+00_JPRB, 0.25943E+00_JPRB, 0.26308E+00_JPRB, 0.26537E+00_JPRB, 0.26618E+00_JPRB, 0.26877E+00_JPRB, 0.27016E+00_JPRB ,&
&  0.26487E+00_JPRB, 0.26474E+00_JPRB, 0.26463E+00_JPRB, 0.25974E+00_JPRB, 0.27078E+00_JPRB, 0.27141E+00_JPRB, 0.27315E+00_JPRB ,&
&  0.28934E+00_JPRB, 0.31390E+00_JPRB, 0.33754E+00_JPRB, 0.36636E+00_JPRB, 0.39077E+00_JPRB, 0.40653E+00_JPRB, 0.42255E+00_JPRB ,&
&  0.44259E+00_JPRB, 0.47170E+00_JPRB, 0.50761E+00_JPRB, 0.54972E+00_JPRB, 0.59716E+00_JPRB, 0.59891E+00_JPRB, 0.67380E+00_JPRB ,&
&  0.68670E+00_JPRB, 0.72861E+00_JPRB, 0.75154E+00_JPRB, 0.68487E+00_JPRB, 0.82476E+00_JPRB, 0.94269E+00_JPRB, 0.87614E+00_JPRB ,&
&  0.85561E+00_JPRB, 0.77076E+00_JPRB, 0.80840E+00_JPRB, 0.84212E+00_JPRB, 0.86446E+00_JPRB, 0.82524E+00_JPRB, 0.79220E+00_JPRB ,&
&  0.82316E+00_JPRB, 0.83613E+00_JPRB, 0.71924E+00_JPRB, 0.67447E+00_JPRB, 0.63181E+00_JPRB, 0.61657E+00_JPRB, 0.57673E+00_JPRB ,&
&  0.55759E+00_JPRB, 0.54943E+00_JPRB, 0.52667E+00_JPRB, 0.50001E+00_JPRB, 0.46890E+00_JPRB, 0.44184E+00_JPRB, 0.41607E+00_JPRB ,&
&  0.38588E+00_JPRB, 0.34812E+00_JPRB, 0.33131E+00_JPRB, 0.33320E+00_JPRB, 0.34028E+00_JPRB, 0.31062E+00_JPRB, 0.28502E+00_JPRB ,&
&  0.27610E+00_JPRB, 0.27841E+00_JPRB, 0.29763E+00_JPRB, 0.29836E+00_JPRB, 0.29640E+00_JPRB, 0.29412E+00_JPRB, 0.28781E+00_JPRB ,&
&  0.28599E+00_JPRB, 0.31171E+00_JPRB, 0.32457E+00_JPRB, 0.32790E+00_JPRB, 0.32948E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.35213E+00_JPRB, 0.36933E+00_JPRB, 0.37899E+00_JPRB, 0.39522E+00_JPRB, 0.40665E+00_JPRB, 0.39743E+00_JPRB, 0.39319E+00_JPRB ,&
&  0.39081E+00_JPRB, 0.38735E+00_JPRB, 0.38213E+00_JPRB, 0.38018E+00_JPRB, 0.37950E+00_JPRB, 0.37317E+00_JPRB, 0.33271E+00_JPRB ,&
&  0.28509E+00_JPRB, 0.28458E+00_JPRB, 0.28057E+00_JPRB, 0.28852E+00_JPRB, 0.29604E+00_JPRB, 0.28577E+00_JPRB, 0.27790E+00_JPRB ,&
&  0.27772E+00_JPRB, 0.27417E+00_JPRB, 0.27263E+00_JPRB, 0.27418E+00_JPRB, 0.27843E+00_JPRB, 0.28564E+00_JPRB, 0.28924E+00_JPRB ,&
&  0.28111E+00_JPRB, 0.27668E+00_JPRB, 0.27300E+00_JPRB, 0.27234E+00_JPRB, 0.28223E+00_JPRB, 0.28741E+00_JPRB, 0.29263E+00_JPRB ,&
&  0.30989E+00_JPRB, 0.33415E+00_JPRB, 0.35900E+00_JPRB, 0.39497E+00_JPRB, 0.42971E+00_JPRB, 0.45819E+00_JPRB, 0.48689E+00_JPRB ,&
&  0.53255E+00_JPRB, 0.57901E+00_JPRB, 0.61163E+00_JPRB, 0.65710E+00_JPRB, 0.69198E+00_JPRB, 0.74534E+00_JPRB, 0.79170E+00_JPRB ,&
&  0.82799E+00_JPRB, 0.84818E+00_JPRB, 0.82467E+00_JPRB, 0.83449E+00_JPRB, 0.95161E+00_JPRB, 0.99062E+00_JPRB, 0.94237E+00_JPRB ,&
&  0.93357E+00_JPRB, 0.84341E+00_JPRB, 0.10137E+01_JPRB, 0.10247E+01_JPRB, 0.94615E+00_JPRB, 0.93437E+00_JPRB, 0.86703E+00_JPRB ,&
&  0.85587E+00_JPRB, 0.82070E+00_JPRB, 0.76140E+00_JPRB, 0.71988E+00_JPRB, 0.68398E+00_JPRB, 0.69033E+00_JPRB, 0.64969E+00_JPRB ,&
&  0.64157E+00_JPRB, 0.62121E+00_JPRB, 0.53936E+00_JPRB, 0.48404E+00_JPRB, 0.42917E+00_JPRB, 0.44723E+00_JPRB, 0.43153E+00_JPRB ,&
&  0.39597E+00_JPRB, 0.34474E+00_JPRB, 0.35044E+00_JPRB, 0.35363E+00_JPRB, 0.36755E+00_JPRB, 0.35568E+00_JPRB, 0.31419E+00_JPRB ,&
&  0.29880E+00_JPRB, 0.31152E+00_JPRB, 0.31360E+00_JPRB, 0.31522E+00_JPRB, 0.31655E+00_JPRB, 0.31420E+00_JPRB, 0.29482E+00_JPRB ,&
&  0.29372E+00_JPRB, 0.31804E+00_JPRB, 0.33132E+00_JPRB, 0.33396E+00_JPRB, 0.34115E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.37166E+00_JPRB, 0.39287E+00_JPRB, 0.40421E+00_JPRB, 0.41531E+00_JPRB, 0.42560E+00_JPRB, 0.41577E+00_JPRB, 0.41199E+00_JPRB ,&
&  0.41285E+00_JPRB, 0.40672E+00_JPRB, 0.39385E+00_JPRB, 0.39135E+00_JPRB, 0.39044E+00_JPRB, 0.38426E+00_JPRB, 0.37477E+00_JPRB ,&
&  0.31198E+00_JPRB, 0.30180E+00_JPRB, 0.28209E+00_JPRB, 0.28762E+00_JPRB, 0.32313E+00_JPRB, 0.31103E+00_JPRB, 0.30376E+00_JPRB ,&
&  0.29766E+00_JPRB, 0.29438E+00_JPRB, 0.28752E+00_JPRB, 0.28653E+00_JPRB, 0.29127E+00_JPRB, 0.30479E+00_JPRB, 0.31945E+00_JPRB ,&
&  0.31360E+00_JPRB, 0.30318E+00_JPRB, 0.29568E+00_JPRB, 0.29380E+00_JPRB, 0.30125E+00_JPRB, 0.31134E+00_JPRB, 0.32559E+00_JPRB ,&
&  0.34588E+00_JPRB, 0.36736E+00_JPRB, 0.40210E+00_JPRB, 0.45329E+00_JPRB, 0.49718E+00_JPRB, 0.53476E+00_JPRB, 0.59029E+00_JPRB ,&
&  0.66033E+00_JPRB, 0.71642E+00_JPRB, 0.75397E+00_JPRB, 0.79517E+00_JPRB, 0.81311E+00_JPRB, 0.86609E+00_JPRB, 0.89039E+00_JPRB ,&
&  0.97147E+00_JPRB, 0.95161E+00_JPRB, 0.95190E+00_JPRB, 0.10453E+01_JPRB, 0.11449E+01_JPRB, 0.11460E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.10222E+01_JPRB, 0.94340E+00_JPRB, 0.98184E+00_JPRB, 0.10123E+01_JPRB, 0.95265E+00_JPRB, 0.97403E+00_JPRB, 0.89418E+00_JPRB ,&
&  0.80217E+00_JPRB, 0.74534E+00_JPRB, 0.72153E+00_JPRB, 0.72577E+00_JPRB, 0.69815E+00_JPRB, 0.72773E+00_JPRB, 0.71056E+00_JPRB ,&
&  0.71503E+00_JPRB, 0.62972E+00_JPRB, 0.48926E+00_JPRB, 0.45785E+00_JPRB, 0.38212E+00_JPRB, 0.39132E+00_JPRB, 0.46515E+00_JPRB ,&
&  0.45147E+00_JPRB, 0.41421E+00_JPRB, 0.40498E+00_JPRB, 0.40603E+00_JPRB, 0.42231E+00_JPRB, 0.45374E+00_JPRB, 0.40207E+00_JPRB ,&
&  0.34690E+00_JPRB, 0.34451E+00_JPRB, 0.34915E+00_JPRB, 0.34619E+00_JPRB, 0.34346E+00_JPRB, 0.34556E+00_JPRB, 0.34198E+00_JPRB ,&
&  0.34126E+00_JPRB, 0.34315E+00_JPRB, 0.34877E+00_JPRB, 0.35146E+00_JPRB, 0.36182E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.39457E+00_JPRB, 0.41281E+00_JPRB, 0.42635E+00_JPRB, 0.43330E+00_JPRB, 0.44621E+00_JPRB, 0.44148E+00_JPRB, 0.43485E+00_JPRB ,&
&  0.43315E+00_JPRB, 0.42366E+00_JPRB, 0.40712E+00_JPRB, 0.40414E+00_JPRB, 0.40023E+00_JPRB, 0.39224E+00_JPRB, 0.38394E+00_JPRB ,&
&  0.38728E+00_JPRB, 0.35270E+00_JPRB, 0.31452E+00_JPRB, 0.29263E+00_JPRB, 0.31933E+00_JPRB, 0.31566E+00_JPRB, 0.32208E+00_JPRB ,&
&  0.31479E+00_JPRB, 0.32292E+00_JPRB, 0.32053E+00_JPRB, 0.31865E+00_JPRB, 0.32906E+00_JPRB, 0.34835E+00_JPRB, 0.36614E+00_JPRB ,&
&  0.36791E+00_JPRB, 0.36525E+00_JPRB, 0.35373E+00_JPRB, 0.35182E+00_JPRB, 0.37044E+00_JPRB, 0.36684E+00_JPRB, 0.38914E+00_JPRB ,&
&  0.40942E+00_JPRB, 0.43113E+00_JPRB, 0.46585E+00_JPRB, 0.52286E+00_JPRB, 0.57591E+00_JPRB, 0.62632E+00_JPRB, 0.69551E+00_JPRB ,&
&  0.75695E+00_JPRB, 0.78132E+00_JPRB, 0.78387E+00_JPRB, 0.80601E+00_JPRB, 0.87330E+00_JPRB, 0.96066E+00_JPRB, 0.96208E+00_JPRB ,&
&  0.96786E+00_JPRB, 0.91676E+00_JPRB, 0.87735E+00_JPRB, 0.10327E+01_JPRB, 0.11594E+01_JPRB, 0.11417E+01_JPRB, 0.10871E+01_JPRB ,&
&  0.10703E+01_JPRB, 0.10520E+01_JPRB, 0.11068E+01_JPRB, 0.10968E+01_JPRB, 0.10625E+01_JPRB, 0.10157E+01_JPRB, 0.91064E+00_JPRB ,&
&  0.79819E+00_JPRB, 0.74733E+00_JPRB, 0.76937E+00_JPRB, 0.77632E+00_JPRB, 0.77686E+00_JPRB, 0.73894E+00_JPRB, 0.76953E+00_JPRB ,&
&  0.82279E+00_JPRB, 0.48076E+00_JPRB, 0.39762E+00_JPRB, 0.45172E+00_JPRB, 0.39893E+00_JPRB, 0.40449E+00_JPRB, 0.44632E+00_JPRB ,&
&  0.43113E+00_JPRB, 0.42102E+00_JPRB, 0.45261E+00_JPRB, 0.45685E+00_JPRB, 0.49057E+00_JPRB, 0.53511E+00_JPRB, 0.48445E+00_JPRB ,&
&  0.43435E+00_JPRB, 0.40929E+00_JPRB, 0.41078E+00_JPRB, 0.39910E+00_JPRB, 0.39935E+00_JPRB, 0.39517E+00_JPRB, 0.39400E+00_JPRB ,&
&  0.38701E+00_JPRB, 0.38368E+00_JPRB, 0.38053E+00_JPRB, 0.38329E+00_JPRB, 0.38946E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.43495E+00_JPRB, 0.45040E+00_JPRB, 0.45985E+00_JPRB, 0.46726E+00_JPRB, 0.47836E+00_JPRB, 0.47683E+00_JPRB, 0.46547E+00_JPRB ,&
&  0.45240E+00_JPRB, 0.44002E+00_JPRB, 0.42226E+00_JPRB, 0.41725E+00_JPRB, 0.41002E+00_JPRB, 0.40032E+00_JPRB, 0.39265E+00_JPRB ,&
&  0.39323E+00_JPRB, 0.38342E+00_JPRB, 0.32662E+00_JPRB, 0.30608E+00_JPRB, 0.29093E+00_JPRB, 0.30414E+00_JPRB, 0.31689E+00_JPRB ,&
&  0.33577E+00_JPRB, 0.34209E+00_JPRB, 0.34877E+00_JPRB, 0.36282E+00_JPRB, 0.38967E+00_JPRB, 0.41034E+00_JPRB, 0.44163E+00_JPRB ,&
&  0.45021E+00_JPRB, 0.45909E+00_JPRB, 0.43014E+00_JPRB, 0.43837E+00_JPRB, 0.45996E+00_JPRB, 0.45486E+00_JPRB, 0.49720E+00_JPRB ,&
&  0.52307E+00_JPRB, 0.55606E+00_JPRB, 0.59036E+00_JPRB, 0.61849E+00_JPRB, 0.65240E+00_JPRB, 0.67092E+00_JPRB, 0.71194E+00_JPRB ,&
&  0.73924E+00_JPRB, 0.75474E+00_JPRB, 0.75141E+00_JPRB, 0.75670E+00_JPRB, 0.79513E+00_JPRB, 0.84021E+00_JPRB, 0.83786E+00_JPRB ,&
&  0.86691E+00_JPRB, 0.74148E+00_JPRB, 0.57395E+00_JPRB, 0.82585E+00_JPRB, 0.11370E+01_JPRB, 0.10778E+01_JPRB, 0.91479E+00_JPRB ,&
&  0.97601E+00_JPRB, 0.10275E+01_JPRB, 0.11493E+01_JPRB, 0.11634E+01_JPRB, 0.11253E+01_JPRB, 0.10584E+01_JPRB, 0.98812E+00_JPRB ,&
&  0.91591E+00_JPRB, 0.92775E+00_JPRB, 0.99759E+00_JPRB, 0.11108E+01_JPRB, 0.11224E+01_JPRB, 0.10921E+01_JPRB, 0.10256E+01_JPRB ,&
&  0.85550E+00_JPRB, 0.70817E+00_JPRB, 0.50228E+00_JPRB, 0.51696E+00_JPRB, 0.42116E+00_JPRB, 0.41917E+00_JPRB, 0.45478E+00_JPRB ,&
&  0.44036E+00_JPRB, 0.46198E+00_JPRB, 0.50202E+00_JPRB, 0.52245E+00_JPRB, 0.62645E+00_JPRB, 0.61425E+00_JPRB, 0.58425E+00_JPRB ,&
&  0.52980E+00_JPRB, 0.47169E+00_JPRB, 0.48012E+00_JPRB, 0.47167E+00_JPRB, 0.47536E+00_JPRB, 0.46605E+00_JPRB, 0.45659E+00_JPRB ,&
&  0.44332E+00_JPRB, 0.43401E+00_JPRB, 0.42898E+00_JPRB, 0.42645E+00_JPRB, 0.42692E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.46892E+00_JPRB, 0.47048E+00_JPRB, 0.46828E+00_JPRB, 0.46874E+00_JPRB, 0.47353E+00_JPRB, 0.47171E+00_JPRB, 0.46426E+00_JPRB ,&
&  0.46180E+00_JPRB, 0.45190E+00_JPRB, 0.43812E+00_JPRB, 0.42643E+00_JPRB, 0.42127E+00_JPRB, 0.41231E+00_JPRB, 0.40593E+00_JPRB ,&
&  0.40424E+00_JPRB, 0.36830E+00_JPRB, 0.30822E+00_JPRB, 0.28313E+00_JPRB, 0.27093E+00_JPRB, 0.28342E+00_JPRB, 0.30419E+00_JPRB ,&
&  0.33007E+00_JPRB, 0.34705E+00_JPRB, 0.37791E+00_JPRB, 0.42117E+00_JPRB, 0.47335E+00_JPRB, 0.51827E+00_JPRB, 0.55277E+00_JPRB ,&
&  0.64870E+00_JPRB, 0.57987E+00_JPRB, 0.58561E+00_JPRB, 0.56460E+00_JPRB, 0.59274E+00_JPRB, 0.60309E+00_JPRB, 0.60253E+00_JPRB ,&
&  0.60103E+00_JPRB, 0.60362E+00_JPRB, 0.61013E+00_JPRB, 0.60950E+00_JPRB, 0.61951E+00_JPRB, 0.64797E+00_JPRB, 0.70669E+00_JPRB ,&
&  0.76286E+00_JPRB, 0.75511E+00_JPRB, 0.72716E+00_JPRB, 0.74709E+00_JPRB, 0.79688E+00_JPRB, 0.82455E+00_JPRB, 0.81971E+00_JPRB ,&
&  0.78045E+00_JPRB, 0.73361E+00_JPRB, 0.89440E+00_JPRB, 0.99590E+00_JPRB, 0.11285E+01_JPRB, 0.11224E+01_JPRB, 0.10335E+01_JPRB ,&
&  0.10743E+01_JPRB, 0.10948E+01_JPRB, 0.11597E+01_JPRB, 0.93184E+00_JPRB, 0.10290E+01_JPRB, 0.10257E+01_JPRB, 0.10178E+01_JPRB ,&
&  0.10172E+01_JPRB, 0.10850E+01_JPRB, 0.11683E+01_JPRB, 0.13287E+01_JPRB, 0.14463E+01_JPRB, 0.15105E+01_JPRB, 0.11362E+01_JPRB ,&
&  0.85487E+00_JPRB, 0.88323E+00_JPRB, 0.78795E+00_JPRB, 0.64114E+00_JPRB, 0.53164E+00_JPRB, 0.52972E+00_JPRB, 0.52101E+00_JPRB ,&
&  0.51640E+00_JPRB, 0.53557E+00_JPRB, 0.56894E+00_JPRB, 0.68819E+00_JPRB, 0.80551E+00_JPRB, 0.78691E+00_JPRB, 0.70794E+00_JPRB ,&
&  0.60317E+00_JPRB, 0.60768E+00_JPRB, 0.58249E+00_JPRB, 0.55896E+00_JPRB, 0.57742E+00_JPRB, 0.56004E+00_JPRB, 0.52894E+00_JPRB ,&
&  0.49977E+00_JPRB, 0.48732E+00_JPRB, 0.48053E+00_JPRB, 0.47411E+00_JPRB, 0.46845E+00_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.49608E+00_JPRB, 0.49487E+00_JPRB, 0.48963E+00_JPRB, 0.48599E+00_JPRB, 0.48143E+00_JPRB, 0.47424E+00_JPRB, 0.46711E+00_JPRB ,&
&  0.46848E+00_JPRB, 0.44838E+00_JPRB, 0.44116E+00_JPRB, 0.43543E+00_JPRB, 0.43233E+00_JPRB, 0.42704E+00_JPRB, 0.42399E+00_JPRB ,&
&  0.42073E+00_JPRB, 0.35427E+00_JPRB, 0.29448E+00_JPRB, 0.29054E+00_JPRB, 0.29313E+00_JPRB, 0.28866E+00_JPRB, 0.30554E+00_JPRB ,&
&  0.32971E+00_JPRB, 0.35835E+00_JPRB, 0.40026E+00_JPRB, 0.49081E+00_JPRB, 0.58720E+00_JPRB, 0.66945E+00_JPRB, 0.74647E+00_JPRB ,&
&  0.74493E+00_JPRB, 0.75416E+00_JPRB, 0.76919E+00_JPRB, 0.71924E+00_JPRB, 0.67333E+00_JPRB, 0.64779E+00_JPRB, 0.64117E+00_JPRB ,&
&  0.63791E+00_JPRB, 0.62612E+00_JPRB, 0.62918E+00_JPRB, 0.63854E+00_JPRB, 0.65707E+00_JPRB, 0.69660E+00_JPRB, 0.74781E+00_JPRB ,&
&  0.77051E+00_JPRB, 0.75333E+00_JPRB, 0.73883E+00_JPRB, 0.78971E+00_JPRB, 0.72199E+00_JPRB, 0.69062E+00_JPRB, 0.66893E+00_JPRB ,&
&  0.84843E+00_JPRB, 0.95602E+00_JPRB, 0.95759E+00_JPRB, 0.10491E+01_JPRB, 0.96086E+00_JPRB, 0.98379E+00_JPRB, 0.11162E+01_JPRB ,&
&  0.12090E+01_JPRB, 0.12176E+01_JPRB, 0.12631E+01_JPRB, 0.87401E+00_JPRB, 0.76346E+00_JPRB, 0.10206E+01_JPRB, 0.10838E+01_JPRB ,&
&  0.11053E+01_JPRB, 0.12554E+01_JPRB, 0.14739E+01_JPRB, 0.17330E+01_JPRB, 0.13261E+01_JPRB, 0.11195E+01_JPRB, 0.83389E+00_JPRB ,&
&  0.79451E+00_JPRB, 0.71858E+00_JPRB, 0.79201E+00_JPRB, 0.72709E+00_JPRB, 0.69757E+00_JPRB, 0.66699E+00_JPRB, 0.62041E+00_JPRB ,&
&  0.61464E+00_JPRB, 0.61016E+00_JPRB, 0.70247E+00_JPRB, 0.89095E+00_JPRB, 0.10426E+01_JPRB, 0.92767E+00_JPRB, 0.88791E+00_JPRB ,&
&  0.84553E+00_JPRB, 0.79056E+00_JPRB, 0.71595E+00_JPRB, 0.69160E+00_JPRB, 0.66075E+00_JPRB, 0.62400E+00_JPRB, 0.58176E+00_JPRB ,&
&  0.54755E+00_JPRB, 0.53000E+00_JPRB, 0.51404E+00_JPRB, 0.50304E+00_JPRB, 0.49775E+00_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.52552E+00_JPRB, 0.51943E+00_JPRB, 0.51308E+00_JPRB, 0.50600E+00_JPRB, 0.49759E+00_JPRB, 0.48799E+00_JPRB, 0.48168E+00_JPRB ,&
&  0.48724E+00_JPRB, 0.46387E+00_JPRB, 0.44523E+00_JPRB, 0.43998E+00_JPRB, 0.43988E+00_JPRB, 0.44178E+00_JPRB, 0.44448E+00_JPRB ,&
&  0.44232E+00_JPRB, 0.39737E+00_JPRB, 0.31758E+00_JPRB, 0.29656E+00_JPRB, 0.31533E+00_JPRB, 0.30799E+00_JPRB, 0.31068E+00_JPRB ,&
&  0.34293E+00_JPRB, 0.38618E+00_JPRB, 0.46087E+00_JPRB, 0.60327E+00_JPRB, 0.73823E+00_JPRB, 0.86285E+00_JPRB, 0.94078E+00_JPRB ,&
&  0.94393E+00_JPRB, 0.86002E+00_JPRB, 0.75475E+00_JPRB, 0.66159E+00_JPRB, 0.61458E+00_JPRB, 0.59608E+00_JPRB, 0.60149E+00_JPRB ,&
&  0.61443E+00_JPRB, 0.62218E+00_JPRB, 0.63141E+00_JPRB, 0.63525E+00_JPRB, 0.67119E+00_JPRB, 0.69050E+00_JPRB, 0.72472E+00_JPRB ,&
&  0.74317E+00_JPRB, 0.74301E+00_JPRB, 0.76700E+00_JPRB, 0.80381E+00_JPRB, 0.77434E+00_JPRB, 0.72059E+00_JPRB, 0.71707E+00_JPRB ,&
&  0.80098E+00_JPRB, 0.89082E+00_JPRB, 0.96291E+00_JPRB, 0.10686E+01_JPRB, 0.11640E+01_JPRB, 0.11890E+01_JPRB, 0.13245E+01_JPRB ,&
&  0.11372E+01_JPRB, 0.92223E+00_JPRB, 0.77607E+00_JPRB, 0.66807E+00_JPRB, 0.88997E+00_JPRB, 0.12360E+01_JPRB, 0.13856E+01_JPRB ,&
&  0.15405E+01_JPRB, 0.17084E+01_JPRB, 0.18825E+01_JPRB, 0.16729E+01_JPRB, 0.12750E+01_JPRB, 0.94465E+00_JPRB, 0.10888E+01_JPRB ,&
&  0.11063E+01_JPRB, 0.10654E+01_JPRB, 0.97246E+00_JPRB, 0.85227E+00_JPRB, 0.68267E+00_JPRB, 0.76363E+00_JPRB, 0.73460E+00_JPRB ,&
&  0.70622E+00_JPRB, 0.70031E+00_JPRB, 0.94762E+00_JPRB, 0.11892E+01_JPRB, 0.12257E+01_JPRB, 0.12082E+01_JPRB, 0.11525E+01_JPRB ,&
&  0.10492E+01_JPRB, 0.98910E+00_JPRB, 0.88821E+00_JPRB, 0.79702E+00_JPRB, 0.72323E+00_JPRB, 0.65604E+00_JPRB, 0.60410E+00_JPRB ,&
&  0.57520E+00_JPRB, 0.55754E+00_JPRB, 0.53804E+00_JPRB, 0.53155E+00_JPRB, 0.52767E+00_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.57259E+00_JPRB, 0.56049E+00_JPRB, 0.55018E+00_JPRB, 0.54104E+00_JPRB, 0.52999E+00_JPRB, 0.51242E+00_JPRB, 0.50759E+00_JPRB ,&
&  0.50697E+00_JPRB, 0.48519E+00_JPRB, 0.46412E+00_JPRB, 0.45804E+00_JPRB, 0.46458E+00_JPRB, 0.46844E+00_JPRB, 0.46738E+00_JPRB ,&
&  0.46605E+00_JPRB, 0.47513E+00_JPRB, 0.37172E+00_JPRB, 0.33030E+00_JPRB, 0.33221E+00_JPRB, 0.32903E+00_JPRB, 0.31649E+00_JPRB ,&
&  0.36645E+00_JPRB, 0.43636E+00_JPRB, 0.53066E+00_JPRB, 0.69133E+00_JPRB, 0.92159E+00_JPRB, 0.93674E+00_JPRB, 0.93759E+00_JPRB ,&
&  0.93092E+00_JPRB, 0.85143E+00_JPRB, 0.75059E+00_JPRB, 0.67859E+00_JPRB, 0.64139E+00_JPRB, 0.62802E+00_JPRB, 0.63324E+00_JPRB ,&
&  0.64848E+00_JPRB, 0.65918E+00_JPRB, 0.66543E+00_JPRB, 0.68718E+00_JPRB, 0.69016E+00_JPRB, 0.69908E+00_JPRB, 0.73293E+00_JPRB ,&
&  0.74398E+00_JPRB, 0.73364E+00_JPRB, 0.75457E+00_JPRB, 0.82481E+00_JPRB, 0.85151E+00_JPRB, 0.78001E+00_JPRB, 0.92782E+00_JPRB ,&
&  0.86879E+00_JPRB, 0.97822E+00_JPRB, 0.10467E+01_JPRB, 0.13675E+01_JPRB, 0.14243E+01_JPRB, 0.13179E+01_JPRB, 0.13429E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.99443E+00_JPRB, 0.10803E+01_JPRB, 0.11764E+01_JPRB, 0.12477E+01_JPRB, 0.13265E+01_JPRB, 0.19655E+01_JPRB ,&
&  0.16917E+01_JPRB, 0.18678E+01_JPRB, 0.19764E+01_JPRB, 0.18800E+01_JPRB, 0.91744E+00_JPRB, 0.66580E+00_JPRB, 0.90639E+00_JPRB ,&
&  0.96251E+00_JPRB, 0.68375E+00_JPRB, 0.51192E+00_JPRB, 0.60855E+00_JPRB, 0.47828E+00_JPRB, 0.55606E+00_JPRB, 0.70824E+00_JPRB ,&
&  0.74911E+00_JPRB, 0.82161E+00_JPRB, 0.11482E+01_JPRB, 0.13306E+01_JPRB, 0.13697E+01_JPRB, 0.15911E+01_JPRB, 0.13610E+01_JPRB ,&
&  0.12436E+01_JPRB, 0.11275E+01_JPRB, 0.10746E+01_JPRB, 0.93273E+00_JPRB, 0.81931E+00_JPRB, 0.74435E+00_JPRB, 0.68997E+00_JPRB ,&
&  0.65249E+00_JPRB, 0.62066E+00_JPRB, 0.59968E+00_JPRB, 0.59020E+00_JPRB, 0.58248E+00_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.62999E+00_JPRB, 0.61017E+00_JPRB, 0.59334E+00_JPRB, 0.58108E+00_JPRB, 0.56315E+00_JPRB, 0.54351E+00_JPRB, 0.54066E+00_JPRB ,&
&  0.52951E+00_JPRB, 0.51036E+00_JPRB, 0.49683E+00_JPRB, 0.48288E+00_JPRB, 0.48359E+00_JPRB, 0.48630E+00_JPRB, 0.47424E+00_JPRB ,&
&  0.46051E+00_JPRB, 0.45457E+00_JPRB, 0.43037E+00_JPRB, 0.39177E+00_JPRB, 0.37090E+00_JPRB, 0.35556E+00_JPRB, 0.36010E+00_JPRB ,&
&  0.43060E+00_JPRB, 0.54035E+00_JPRB, 0.64642E+00_JPRB, 0.77467E+00_JPRB, 0.89920E+00_JPRB, 0.89170E+00_JPRB, 0.94827E+00_JPRB ,&
&  0.95584E+00_JPRB, 0.90360E+00_JPRB, 0.81984E+00_JPRB, 0.74600E+00_JPRB, 0.70758E+00_JPRB, 0.68481E+00_JPRB, 0.66980E+00_JPRB ,&
&  0.67292E+00_JPRB, 0.68847E+00_JPRB, 0.70226E+00_JPRB, 0.69769E+00_JPRB, 0.68878E+00_JPRB, 0.70125E+00_JPRB, 0.73582E+00_JPRB ,&
&  0.74244E+00_JPRB, 0.72807E+00_JPRB, 0.75714E+00_JPRB, 0.80121E+00_JPRB, 0.90530E+00_JPRB, 0.92382E+00_JPRB, 0.79383E+00_JPRB ,&
&  0.71702E+00_JPRB, 0.79392E+00_JPRB, 0.10953E+01_JPRB, 0.13166E+01_JPRB, 0.13144E+01_JPRB, 0.12790E+01_JPRB, 0.11813E+01_JPRB ,&
&  0.11499E+01_JPRB, 0.13187E+01_JPRB, 0.14197E+01_JPRB, 0.15729E+01_JPRB, 0.16417E+01_JPRB, 0.13353E+01_JPRB, 0.17665E+01_JPRB ,&
&  0.18720E+01_JPRB, 0.18442E+01_JPRB, 0.14783E+01_JPRB, 0.10275E+01_JPRB, 0.10007E+01_JPRB, 0.71837E+00_JPRB, 0.49994E+00_JPRB ,&
&  0.47070E+00_JPRB, 0.45165E+00_JPRB, 0.41418E+00_JPRB, 0.41948E+00_JPRB, 0.40976E+00_JPRB, 0.50710E+00_JPRB, 0.67800E+00_JPRB ,&
&  0.86394E+00_JPRB, 0.10586E+01_JPRB, 0.13076E+01_JPRB, 0.14709E+01_JPRB, 0.14058E+01_JPRB, 0.15648E+01_JPRB, 0.15307E+01_JPRB ,&
&  0.12521E+01_JPRB, 0.11718E+01_JPRB, 0.10625E+01_JPRB, 0.94448E+00_JPRB, 0.85743E+00_JPRB, 0.80297E+00_JPRB, 0.76213E+00_JPRB ,&
&  0.72256E+00_JPRB, 0.68923E+00_JPRB, 0.66855E+00_JPRB, 0.65701E+00_JPRB, 0.64593E+00_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.67925E+00_JPRB, 0.65775E+00_JPRB, 0.64033E+00_JPRB, 0.62688E+00_JPRB, 0.60612E+00_JPRB, 0.58703E+00_JPRB, 0.58002E+00_JPRB ,&
&  0.56138E+00_JPRB, 0.54025E+00_JPRB, 0.52157E+00_JPRB, 0.50640E+00_JPRB, 0.50078E+00_JPRB, 0.49789E+00_JPRB, 0.48609E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.45906E+00_JPRB, 0.45926E+00_JPRB, 0.44319E+00_JPRB, 0.43746E+00_JPRB, 0.43093E+00_JPRB, 0.46173E+00_JPRB ,&
&  0.55565E+00_JPRB, 0.71347E+00_JPRB, 0.82023E+00_JPRB, 0.91837E+00_JPRB, 0.10211E+01_JPRB, 0.10447E+01_JPRB, 0.10497E+01_JPRB ,&
&  0.10265E+01_JPRB, 0.98237E+00_JPRB, 0.89422E+00_JPRB, 0.80698E+00_JPRB, 0.74903E+00_JPRB, 0.71920E+00_JPRB, 0.70649E+00_JPRB ,&
&  0.70362E+00_JPRB, 0.72775E+00_JPRB, 0.72436E+00_JPRB, 0.70249E+00_JPRB, 0.68921E+00_JPRB, 0.71340E+00_JPRB, 0.74317E+00_JPRB ,&
&  0.74850E+00_JPRB, 0.76575E+00_JPRB, 0.80674E+00_JPRB, 0.81723E+00_JPRB, 0.76231E+00_JPRB, 0.65550E+00_JPRB, 0.71978E+00_JPRB ,&
&  0.80504E+00_JPRB, 0.98630E+00_JPRB, 0.10475E+01_JPRB, 0.13826E+01_JPRB, 0.14075E+01_JPRB, 0.12763E+01_JPRB, 0.12607E+01_JPRB ,&
&  0.13090E+01_JPRB, 0.17022E+01_JPRB, 0.15017E+01_JPRB, 0.14902E+01_JPRB, 0.17738E+01_JPRB, 0.14557E+01_JPRB, 0.15454E+01_JPRB ,&
&  0.17922E+01_JPRB, 0.16654E+01_JPRB, 0.13902E+01_JPRB, 0.10491E+01_JPRB, 0.12269E+01_JPRB, 0.10771E+01_JPRB, 0.58200E+00_JPRB ,&
&  0.48780E+00_JPRB, 0.45741E+00_JPRB, 0.43410E+00_JPRB, 0.41889E+00_JPRB, 0.41768E+00_JPRB, 0.54951E+00_JPRB, 0.84819E+00_JPRB ,&
&  0.95068E+00_JPRB, 0.12721E+01_JPRB, 0.14916E+01_JPRB, 0.15784E+01_JPRB, 0.14053E+01_JPRB, 0.14014E+01_JPRB, 0.14856E+01_JPRB ,&
&  0.12806E+01_JPRB, 0.12427E+01_JPRB, 0.11023E+01_JPRB, 0.10055E+01_JPRB, 0.93010E+00_JPRB, 0.87252E+00_JPRB, 0.82182E+00_JPRB ,&
&  0.77546E+00_JPRB, 0.74603E+00_JPRB, 0.72468E+00_JPRB, 0.71600E+00_JPRB, 0.69851E+00_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.71187E+00_JPRB, 0.69794E+00_JPRB, 0.67821E+00_JPRB, 0.66287E+00_JPRB, 0.64292E+00_JPRB, 0.62149E+00_JPRB, 0.62256E+00_JPRB ,&
&  0.60679E+00_JPRB, 0.58019E+00_JPRB, 0.55182E+00_JPRB, 0.53845E+00_JPRB, 0.52573E+00_JPRB, 0.50945E+00_JPRB, 0.49456E+00_JPRB ,&
&  0.47892E+00_JPRB, 0.47006E+00_JPRB, 0.47604E+00_JPRB, 0.51366E+00_JPRB, 0.52689E+00_JPRB, 0.52846E+00_JPRB, 0.57601E+00_JPRB ,&
&  0.74358E+00_JPRB, 0.87752E+00_JPRB, 0.94834E+00_JPRB, 0.10129E+01_JPRB, 0.10335E+01_JPRB, 0.10310E+01_JPRB, 0.10282E+01_JPRB ,&
&  0.99601E+00_JPRB, 0.96210E+00_JPRB, 0.89539E+00_JPRB, 0.82845E+00_JPRB, 0.77634E+00_JPRB, 0.75439E+00_JPRB, 0.74800E+00_JPRB ,&
&  0.76587E+00_JPRB, 0.75348E+00_JPRB, 0.73233E+00_JPRB, 0.70175E+00_JPRB, 0.69455E+00_JPRB, 0.72363E+00_JPRB, 0.75386E+00_JPRB ,&
&  0.76849E+00_JPRB, 0.81973E+00_JPRB, 0.85115E+00_JPRB, 0.82686E+00_JPRB, 0.68330E+00_JPRB, 0.75847E+00_JPRB, 0.79080E+00_JPRB ,&
&  0.83869E+00_JPRB, 0.90101E+00_JPRB, 0.91262E+00_JPRB, 0.11709E+01_JPRB, 0.13466E+01_JPRB, 0.12692E+01_JPRB, 0.12104E+01_JPRB ,&
&  0.14418E+01_JPRB, 0.14610E+01_JPRB, 0.13373E+01_JPRB, 0.13958E+01_JPRB, 0.15423E+01_JPRB, 0.18764E+01_JPRB, 0.12837E+01_JPRB ,&
&  0.11792E+01_JPRB, 0.14014E+01_JPRB, 0.12367E+01_JPRB, 0.97847E+00_JPRB, 0.11071E+01_JPRB, 0.11710E+01_JPRB, 0.96588E+00_JPRB ,&
&  0.55625E+00_JPRB, 0.47046E+00_JPRB, 0.49003E+00_JPRB, 0.51896E+00_JPRB, 0.48795E+00_JPRB, 0.64483E+00_JPRB, 0.13385E+01_JPRB ,&
&  0.12671E+01_JPRB, 0.15193E+01_JPRB, 0.15833E+01_JPRB, 0.14942E+01_JPRB, 0.13191E+01_JPRB, 0.12417E+01_JPRB, 0.13798E+01_JPRB ,&
&  0.12249E+01_JPRB, 0.11507E+01_JPRB, 0.10918E+01_JPRB, 0.10397E+01_JPRB, 0.97708E+00_JPRB, 0.90575E+00_JPRB, 0.84719E+00_JPRB ,&
&  0.80489E+00_JPRB, 0.78173E+00_JPRB, 0.76010E+00_JPRB, 0.74351E+00_JPRB, 0.72334E+00_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.75208E+00_JPRB, 0.72959E+00_JPRB, 0.70907E+00_JPRB, 0.69597E+00_JPRB, 0.67331E+00_JPRB, 0.65439E+00_JPRB, 0.67744E+00_JPRB ,&
&  0.66903E+00_JPRB, 0.62966E+00_JPRB, 0.57909E+00_JPRB, 0.55837E+00_JPRB, 0.54016E+00_JPRB, 0.52038E+00_JPRB, 0.50740E+00_JPRB ,&
&  0.49634E+00_JPRB, 0.49343E+00_JPRB, 0.49525E+00_JPRB, 0.53530E+00_JPRB, 0.56733E+00_JPRB, 0.55327E+00_JPRB, 0.62250E+00_JPRB ,&
&  0.85608E+00_JPRB, 0.98457E+00_JPRB, 0.10181E+01_JPRB, 0.10639E+01_JPRB, 0.10760E+01_JPRB, 0.10637E+01_JPRB, 0.10412E+01_JPRB ,&
&  0.96399E+00_JPRB, 0.92750E+00_JPRB, 0.85856E+00_JPRB, 0.83092E+00_JPRB, 0.80804E+00_JPRB, 0.78955E+00_JPRB, 0.78847E+00_JPRB ,&
&  0.76064E+00_JPRB, 0.73803E+00_JPRB, 0.71194E+00_JPRB, 0.68398E+00_JPRB, 0.70013E+00_JPRB, 0.74337E+00_JPRB, 0.78300E+00_JPRB ,&
&  0.82007E+00_JPRB, 0.87215E+00_JPRB, 0.87868E+00_JPRB, 0.80650E+00_JPRB, 0.78012E+00_JPRB, 0.77959E+00_JPRB, 0.81909E+00_JPRB ,&
&  0.82754E+00_JPRB, 0.88904E+00_JPRB, 0.98158E+00_JPRB, 0.11029E+01_JPRB, 0.12252E+01_JPRB, 0.12300E+01_JPRB, 0.12004E+01_JPRB ,&
&  0.13170E+01_JPRB, 0.13040E+01_JPRB, 0.11883E+01_JPRB, 0.12906E+01_JPRB, 0.14271E+01_JPRB, 0.17280E+01_JPRB, 0.16869E+01_JPRB ,&
&  0.13992E+01_JPRB, 0.11293E+01_JPRB, 0.10311E+01_JPRB, 0.10053E+01_JPRB, 0.10610E+01_JPRB, 0.11063E+01_JPRB, 0.13428E+01_JPRB ,&
&  0.13433E+01_JPRB, 0.97031E+00_JPRB, 0.84451E+00_JPRB, 0.10406E+01_JPRB, 0.92308E+00_JPRB, 0.87865E+00_JPRB, 0.11395E+01_JPRB ,&
&  0.13856E+01_JPRB, 0.15780E+01_JPRB, 0.15401E+01_JPRB, 0.13943E+01_JPRB, 0.13155E+01_JPRB, 0.12404E+01_JPRB, 0.13020E+01_JPRB ,&
&  0.12062E+01_JPRB, 0.11294E+01_JPRB, 0.11011E+01_JPRB, 0.10678E+01_JPRB, 0.10001E+01_JPRB, 0.92029E+00_JPRB, 0.85362E+00_JPRB ,&
&  0.81863E+00_JPRB, 0.79949E+00_JPRB, 0.78491E+00_JPRB, 0.77519E+00_JPRB, 0.76380E+00_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.78182E+00_JPRB, 0.75689E+00_JPRB, 0.74152E+00_JPRB, 0.72554E+00_JPRB, 0.70051E+00_JPRB, 0.69876E+00_JPRB, 0.75193E+00_JPRB ,&
&  0.75601E+00_JPRB, 0.66769E+00_JPRB, 0.61920E+00_JPRB, 0.58021E+00_JPRB, 0.55276E+00_JPRB, 0.53505E+00_JPRB, 0.52674E+00_JPRB ,&
&  0.52082E+00_JPRB, 0.52350E+00_JPRB, 0.52257E+00_JPRB, 0.56648E+00_JPRB, 0.65027E+00_JPRB, 0.79591E+00_JPRB, 0.66247E+00_JPRB ,&
&  0.90102E+00_JPRB, 0.11259E+01_JPRB, 0.11502E+01_JPRB, 0.11157E+01_JPRB, 0.10539E+01_JPRB, 0.10572E+01_JPRB, 0.10087E+01_JPRB ,&
&  0.92149E+00_JPRB, 0.88149E+00_JPRB, 0.82864E+00_JPRB, 0.79984E+00_JPRB, 0.79313E+00_JPRB, 0.77647E+00_JPRB, 0.74623E+00_JPRB ,&
&  0.72388E+00_JPRB, 0.70492E+00_JPRB, 0.68886E+00_JPRB, 0.69151E+00_JPRB, 0.71696E+00_JPRB, 0.76212E+00_JPRB, 0.81925E+00_JPRB ,&
&  0.88763E+00_JPRB, 0.91943E+00_JPRB, 0.90473E+00_JPRB, 0.82761E+00_JPRB, 0.82278E+00_JPRB, 0.81998E+00_JPRB, 0.83171E+00_JPRB ,&
&  0.80610E+00_JPRB, 0.78576E+00_JPRB, 0.92579E+00_JPRB, 0.10808E+01_JPRB, 0.11560E+01_JPRB, 0.11074E+01_JPRB, 0.10735E+01_JPRB ,&
&  0.12292E+01_JPRB, 0.13615E+01_JPRB, 0.14507E+01_JPRB, 0.13893E+01_JPRB, 0.15333E+01_JPRB, 0.17079E+01_JPRB, 0.18321E+01_JPRB ,&
&  0.17528E+01_JPRB, 0.13717E+01_JPRB, 0.11691E+01_JPRB, 0.10550E+01_JPRB, 0.10196E+01_JPRB, 0.10703E+01_JPRB, 0.12758E+01_JPRB ,&
&  0.15208E+01_JPRB, 0.18509E+01_JPRB, 0.18585E+01_JPRB, 0.15249E+01_JPRB, 0.13686E+01_JPRB, 0.12480E+01_JPRB, 0.12014E+01_JPRB ,&
&  0.15181E+01_JPRB, 0.15823E+01_JPRB, 0.15113E+01_JPRB, 0.16520E+01_JPRB, 0.13977E+01_JPRB, 0.12889E+01_JPRB, 0.12506E+01_JPRB ,&
&  0.12044E+01_JPRB, 0.11501E+01_JPRB, 0.10984E+01_JPRB, 0.10501E+01_JPRB, 0.97071E+00_JPRB, 0.89889E+00_JPRB, 0.85756E+00_JPRB ,&
&  0.83469E+00_JPRB, 0.82369E+00_JPRB, 0.81132E+00_JPRB, 0.80467E+00_JPRB, 0.79738E+00_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.79517E+00_JPRB, 0.77356E+00_JPRB, 0.74694E+00_JPRB, 0.74189E+00_JPRB, 0.73711E+00_JPRB, 0.74922E+00_JPRB, 0.78028E+00_JPRB ,&
&  0.86766E+00_JPRB, 0.72087E+00_JPRB, 0.64163E+00_JPRB, 0.58654E+00_JPRB, 0.56192E+00_JPRB, 0.55593E+00_JPRB, 0.55093E+00_JPRB ,&
&  0.55468E+00_JPRB, 0.56468E+00_JPRB, 0.58503E+00_JPRB, 0.61164E+00_JPRB, 0.71885E+00_JPRB, 0.89580E+00_JPRB, 0.92633E+00_JPRB ,&
&  0.10288E+01_JPRB, 0.14345E+01_JPRB, 0.12610E+01_JPRB, 0.11396E+01_JPRB, 0.10670E+01_JPRB, 0.11224E+01_JPRB, 0.10221E+01_JPRB ,&
&  0.91804E+00_JPRB, 0.89403E+00_JPRB, 0.87707E+00_JPRB, 0.80652E+00_JPRB, 0.77603E+00_JPRB, 0.73889E+00_JPRB, 0.72834E+00_JPRB ,&
&  0.71630E+00_JPRB, 0.70441E+00_JPRB, 0.69172E+00_JPRB, 0.70870E+00_JPRB, 0.74725E+00_JPRB, 0.80218E+00_JPRB, 0.88570E+00_JPRB ,&
&  0.96585E+00_JPRB, 0.94850E+00_JPRB, 0.88511E+00_JPRB, 0.85742E+00_JPRB, 0.85028E+00_JPRB, 0.86048E+00_JPRB, 0.86879E+00_JPRB ,&
&  0.85036E+00_JPRB, 0.82561E+00_JPRB, 0.90844E+00_JPRB, 0.94986E+00_JPRB, 0.96446E+00_JPRB, 0.10273E+01_JPRB, 0.10121E+01_JPRB ,&
&  0.12074E+01_JPRB, 0.12580E+01_JPRB, 0.15592E+01_JPRB, 0.14244E+01_JPRB, 0.15901E+01_JPRB, 0.17301E+01_JPRB, 0.16377E+01_JPRB ,&
&  0.13877E+01_JPRB, 0.12613E+01_JPRB, 0.10934E+01_JPRB, 0.10221E+01_JPRB, 0.10519E+01_JPRB, 0.11694E+01_JPRB, 0.13308E+01_JPRB ,&
&  0.15129E+01_JPRB, 0.17160E+01_JPRB, 0.18856E+01_JPRB, 0.15636E+01_JPRB, 0.14318E+01_JPRB, 0.13485E+01_JPRB, 0.13759E+01_JPRB ,&
&  0.16063E+01_JPRB, 0.16790E+01_JPRB, 0.17260E+01_JPRB, 0.16877E+01_JPRB, 0.13680E+01_JPRB, 0.12456E+01_JPRB, 0.11870E+01_JPRB ,&
&  0.11442E+01_JPRB, 0.11001E+01_JPRB, 0.10254E+01_JPRB, 0.96211E+00_JPRB, 0.91574E+00_JPRB, 0.88689E+00_JPRB, 0.85486E+00_JPRB ,&
&  0.83713E+00_JPRB, 0.81782E+00_JPRB, 0.80717E+00_JPRB, 0.80854E+00_JPRB, 0.80538E+00_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.78852E+00_JPRB, 0.77531E+00_JPRB, 0.77530E+00_JPRB, 0.78634E+00_JPRB, 0.79753E+00_JPRB, 0.84455E+00_JPRB, 0.89529E+00_JPRB ,&
&  0.10248E+01_JPRB, 0.72352E+00_JPRB, 0.62922E+00_JPRB, 0.59636E+00_JPRB, 0.58704E+00_JPRB, 0.58282E+00_JPRB, 0.58770E+00_JPRB ,&
&  0.59595E+00_JPRB, 0.62019E+00_JPRB, 0.65621E+00_JPRB, 0.72267E+00_JPRB, 0.88851E+00_JPRB, 0.11177E+01_JPRB, 0.12585E+01_JPRB ,&
&  0.12783E+01_JPRB, 0.13011E+01_JPRB, 0.13139E+01_JPRB, 0.11498E+01_JPRB, 0.10721E+01_JPRB, 0.11011E+01_JPRB, 0.10730E+01_JPRB ,&
&  0.99078E+00_JPRB, 0.92014E+00_JPRB, 0.86598E+00_JPRB, 0.79481E+00_JPRB, 0.75976E+00_JPRB, 0.74033E+00_JPRB, 0.72717E+00_JPRB ,&
&  0.70563E+00_JPRB, 0.69062E+00_JPRB, 0.69243E+00_JPRB, 0.73175E+00_JPRB, 0.78490E+00_JPRB, 0.86629E+00_JPRB, 0.99003E+00_JPRB ,&
&  0.10214E+01_JPRB, 0.96439E+00_JPRB, 0.91604E+00_JPRB, 0.86470E+00_JPRB, 0.86947E+00_JPRB, 0.87960E+00_JPRB, 0.88381E+00_JPRB ,&
&  0.87875E+00_JPRB, 0.90483E+00_JPRB, 0.93056E+00_JPRB, 0.93021E+00_JPRB, 0.90016E+00_JPRB, 0.10078E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.11866E+01_JPRB, 0.13304E+01_JPRB, 0.16787E+01_JPRB, 0.13814E+01_JPRB, 0.16210E+01_JPRB, 0.16644E+01_JPRB, 0.15206E+01_JPRB ,&
&  0.13036E+01_JPRB, 0.11777E+01_JPRB, 0.10754E+01_JPRB, 0.10506E+01_JPRB, 0.11573E+01_JPRB, 0.12733E+01_JPRB, 0.15984E+01_JPRB ,&
&  0.16135E+01_JPRB, 0.17909E+01_JPRB, 0.17050E+01_JPRB, 0.16555E+01_JPRB, 0.14799E+01_JPRB, 0.14148E+01_JPRB, 0.14255E+01_JPRB ,&
&  0.15788E+01_JPRB, 0.15993E+01_JPRB, 0.15212E+01_JPRB, 0.14371E+01_JPRB, 0.12389E+01_JPRB, 0.11294E+01_JPRB, 0.10762E+01_JPRB ,&
&  0.10229E+01_JPRB, 0.96842E+00_JPRB, 0.92937E+00_JPRB, 0.91702E+00_JPRB, 0.89492E+00_JPRB, 0.87267E+00_JPRB, 0.84785E+00_JPRB ,&
&  0.83307E+00_JPRB, 0.82062E+00_JPRB, 0.81357E+00_JPRB, 0.81100E+00_JPRB, 0.80538E+00_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.76384E+00_JPRB, 0.75561E+00_JPRB, 0.75919E+00_JPRB, 0.75372E+00_JPRB, 0.75356E+00_JPRB, 0.75911E+00_JPRB, 0.74621E+00_JPRB ,&
&  0.69063E+00_JPRB, 0.64424E+00_JPRB, 0.62836E+00_JPRB, 0.62681E+00_JPRB, 0.62254E+00_JPRB, 0.61495E+00_JPRB, 0.61899E+00_JPRB ,&
&  0.63734E+00_JPRB, 0.67829E+00_JPRB, 0.74768E+00_JPRB, 0.88720E+00_JPRB, 0.10698E+01_JPRB, 0.12714E+01_JPRB, 0.14376E+01_JPRB ,&
&  0.14174E+01_JPRB, 0.10420E+01_JPRB, 0.12018E+01_JPRB, 0.12181E+01_JPRB, 0.10744E+01_JPRB, 0.10518E+01_JPRB, 0.10364E+01_JPRB ,&
&  0.93263E+00_JPRB, 0.89288E+00_JPRB, 0.86475E+00_JPRB, 0.82319E+00_JPRB, 0.76517E+00_JPRB, 0.74127E+00_JPRB, 0.71709E+00_JPRB ,&
&  0.70314E+00_JPRB, 0.70022E+00_JPRB, 0.72157E+00_JPRB, 0.79105E+00_JPRB, 0.87844E+00_JPRB, 0.10036E+01_JPRB, 0.10768E+01_JPRB ,&
&  0.10516E+01_JPRB, 0.10127E+01_JPRB, 0.99347E+00_JPRB, 0.93821E+00_JPRB, 0.93306E+00_JPRB, 0.94498E+00_JPRB, 0.94643E+00_JPRB ,&
&  0.94323E+00_JPRB, 0.93718E+00_JPRB, 0.96932E+00_JPRB, 0.10259E+01_JPRB, 0.10745E+01_JPRB, 0.10264E+01_JPRB, 0.11380E+01_JPRB ,&
&  0.12553E+01_JPRB, 0.14820E+01_JPRB, 0.16360E+01_JPRB, 0.18574E+01_JPRB, 0.13696E+01_JPRB, 0.14920E+01_JPRB, 0.12720E+01_JPRB ,&
&  0.12630E+01_JPRB, 0.11566E+01_JPRB, 0.11684E+01_JPRB, 0.12181E+01_JPRB, 0.13668E+01_JPRB, 0.14742E+01_JPRB, 0.17717E+01_JPRB ,&
&  0.16696E+01_JPRB, 0.16286E+01_JPRB, 0.15745E+01_JPRB, 0.16366E+01_JPRB, 0.15323E+01_JPRB, 0.15277E+01_JPRB, 0.14492E+01_JPRB ,&
&  0.14843E+01_JPRB, 0.14086E+01_JPRB, 0.13060E+01_JPRB, 0.12492E+01_JPRB, 0.11469E+01_JPRB, 0.98458E+00_JPRB, 0.92471E+00_JPRB ,&
&  0.88529E+00_JPRB, 0.88660E+00_JPRB, 0.88321E+00_JPRB, 0.87798E+00_JPRB, 0.86856E+00_JPRB, 0.85483E+00_JPRB, 0.84076E+00_JPRB ,&
&  0.83063E+00_JPRB, 0.81933E+00_JPRB, 0.81062E+00_JPRB, 0.80202E+00_JPRB, 0.78592E+00_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.73525E+00_JPRB, 0.72522E+00_JPRB, 0.72624E+00_JPRB, 0.72587E+00_JPRB, 0.72622E+00_JPRB, 0.72019E+00_JPRB, 0.70668E+00_JPRB ,&
&  0.68142E+00_JPRB, 0.66980E+00_JPRB, 0.66729E+00_JPRB, 0.66504E+00_JPRB, 0.64895E+00_JPRB, 0.64141E+00_JPRB, 0.65972E+00_JPRB ,&
&  0.70879E+00_JPRB, 0.79348E+00_JPRB, 0.90856E+00_JPRB, 0.10440E+01_JPRB, 0.12175E+01_JPRB, 0.13432E+01_JPRB, 0.13755E+01_JPRB ,&
&  0.14724E+01_JPRB, 0.15159E+01_JPRB, 0.13634E+01_JPRB, 0.10091E+01_JPRB, 0.89236E+00_JPRB, 0.10242E+01_JPRB, 0.96610E+00_JPRB ,&
&  0.89797E+00_JPRB, 0.86824E+00_JPRB, 0.83960E+00_JPRB, 0.80875E+00_JPRB, 0.77557E+00_JPRB, 0.75231E+00_JPRB, 0.73650E+00_JPRB ,&
&  0.73533E+00_JPRB, 0.75721E+00_JPRB, 0.82136E+00_JPRB, 0.94243E+00_JPRB, 0.10597E+01_JPRB, 0.11385E+01_JPRB, 0.11429E+01_JPRB ,&
&  0.11398E+01_JPRB, 0.11459E+01_JPRB, 0.11372E+01_JPRB, 0.10880E+01_JPRB, 0.10496E+01_JPRB, 0.10613E+01_JPRB, 0.10691E+01_JPRB ,&
&  0.10721E+01_JPRB, 0.10714E+01_JPRB, 0.10748E+01_JPRB, 0.11222E+01_JPRB, 0.11559E+01_JPRB, 0.10904E+01_JPRB, 0.11548E+01_JPRB ,&
&  0.13673E+01_JPRB, 0.16762E+01_JPRB, 0.16688E+01_JPRB, 0.19707E+01_JPRB, 0.11956E+01_JPRB, 0.13291E+01_JPRB, 0.13442E+01_JPRB ,&
&  0.11911E+01_JPRB, 0.12168E+01_JPRB, 0.12729E+01_JPRB, 0.13833E+01_JPRB, 0.15313E+01_JPRB, 0.15634E+01_JPRB, 0.17013E+01_JPRB ,&
&  0.15676E+01_JPRB, 0.14560E+01_JPRB, 0.14450E+01_JPRB, 0.14904E+01_JPRB, 0.14862E+01_JPRB, 0.14452E+01_JPRB, 0.13041E+01_JPRB ,&
&  0.11697E+01_JPRB, 0.12509E+01_JPRB, 0.11761E+01_JPRB, 0.11835E+01_JPRB, 0.10953E+01_JPRB, 0.88425E+00_JPRB, 0.88737E+00_JPRB ,&
&  0.88473E+00_JPRB, 0.87244E+00_JPRB, 0.86909E+00_JPRB, 0.85086E+00_JPRB, 0.83806E+00_JPRB, 0.82656E+00_JPRB, 0.81653E+00_JPRB ,&
&  0.81092E+00_JPRB, 0.80249E+00_JPRB, 0.79016E+00_JPRB, 0.77406E+00_JPRB, 0.75440E+00_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.71353E+00_JPRB, 0.70583E+00_JPRB, 0.71218E+00_JPRB, 0.70477E+00_JPRB, 0.69403E+00_JPRB, 0.69451E+00_JPRB, 0.69664E+00_JPRB ,&
&  0.69022E+00_JPRB, 0.68694E+00_JPRB, 0.67950E+00_JPRB, 0.67421E+00_JPRB, 0.67589E+00_JPRB, 0.70061E+00_JPRB, 0.75028E+00_JPRB ,&
&  0.81776E+00_JPRB, 0.92335E+00_JPRB, 0.10634E+01_JPRB, 0.11928E+01_JPRB, 0.13402E+01_JPRB, 0.14068E+01_JPRB, 0.13757E+01_JPRB ,&
&  0.14237E+01_JPRB, 0.14837E+01_JPRB, 0.14728E+01_JPRB, 0.14486E+01_JPRB, 0.11653E+01_JPRB, 0.11874E+01_JPRB, 0.12192E+01_JPRB ,&
&  0.11500E+01_JPRB, 0.97048E+00_JPRB, 0.87212E+00_JPRB, 0.83792E+00_JPRB, 0.80171E+00_JPRB, 0.79260E+00_JPRB, 0.80754E+00_JPRB ,&
&  0.84732E+00_JPRB, 0.91544E+00_JPRB, 0.10224E+01_JPRB, 0.11430E+01_JPRB, 0.12209E+01_JPRB, 0.12529E+01_JPRB, 0.12661E+01_JPRB ,&
&  0.12881E+01_JPRB, 0.13077E+01_JPRB, 0.13313E+01_JPRB, 0.12921E+01_JPRB, 0.12832E+01_JPRB, 0.12835E+01_JPRB, 0.13148E+01_JPRB ,&
&  0.13469E+01_JPRB, 0.12917E+01_JPRB, 0.12972E+01_JPRB, 0.13660E+01_JPRB, 0.13677E+01_JPRB, 0.13469E+01_JPRB, 0.13972E+01_JPRB ,&
&  0.16032E+01_JPRB, 0.19513E+01_JPRB, 0.15617E+01_JPRB, 0.16274E+01_JPRB, 0.18123E+01_JPRB, 0.14804E+01_JPRB, 0.12594E+01_JPRB ,&
&  0.12358E+01_JPRB, 0.12709E+01_JPRB, 0.13236E+01_JPRB, 0.14270E+01_JPRB, 0.15850E+01_JPRB, 0.17570E+01_JPRB, 0.15211E+01_JPRB ,&
&  0.14122E+01_JPRB, 0.13101E+01_JPRB, 0.13112E+01_JPRB, 0.13186E+01_JPRB, 0.13067E+01_JPRB, 0.12865E+01_JPRB, 0.11855E+01_JPRB ,&
&  0.11040E+01_JPRB, 0.11753E+01_JPRB, 0.11773E+01_JPRB, 0.12333E+01_JPRB, 0.13491E+01_JPRB, 0.92969E+00_JPRB, 0.91187E+00_JPRB ,&
&  0.86479E+00_JPRB, 0.85468E+00_JPRB, 0.84449E+00_JPRB, 0.82841E+00_JPRB, 0.81296E+00_JPRB, 0.79856E+00_JPRB, 0.79062E+00_JPRB ,&
&  0.78215E+00_JPRB, 0.76749E+00_JPRB, 0.75644E+00_JPRB, 0.74865E+00_JPRB, 0.72724E+00_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.70223E+00_JPRB, 0.70193E+00_JPRB, 0.70734E+00_JPRB, 0.70387E+00_JPRB, 0.69484E+00_JPRB, 0.69756E+00_JPRB, 0.69229E+00_JPRB ,&
&  0.67739E+00_JPRB, 0.66327E+00_JPRB, 0.64678E+00_JPRB, 0.65652E+00_JPRB, 0.67659E+00_JPRB, 0.71794E+00_JPRB, 0.77079E+00_JPRB ,&
&  0.84462E+00_JPRB, 0.95661E+00_JPRB, 0.11051E+01_JPRB, 0.12190E+01_JPRB, 0.13720E+01_JPRB, 0.14320E+01_JPRB, 0.14577E+01_JPRB ,&
&  0.15108E+01_JPRB, 0.15732E+01_JPRB, 0.16155E+01_JPRB, 0.16221E+01_JPRB, 0.15601E+01_JPRB, 0.15190E+01_JPRB, 0.15258E+01_JPRB ,&
&  0.12453E+01_JPRB, 0.96515E+00_JPRB, 0.90915E+00_JPRB, 0.88766E+00_JPRB, 0.88393E+00_JPRB, 0.89231E+00_JPRB, 0.94023E+00_JPRB ,&
&  0.10174E+01_JPRB, 0.11015E+01_JPRB, 0.12091E+01_JPRB, 0.12909E+01_JPRB, 0.13590E+01_JPRB, 0.14085E+01_JPRB, 0.14410E+01_JPRB ,&
&  0.14962E+01_JPRB, 0.15562E+01_JPRB, 0.16556E+01_JPRB, 0.15798E+01_JPRB, 0.17038E+01_JPRB, 0.17052E+01_JPRB, 0.17147E+01_JPRB ,&
&  0.16735E+01_JPRB, 0.15641E+01_JPRB, 0.16136E+01_JPRB, 0.16791E+01_JPRB, 0.17482E+01_JPRB, 0.17604E+01_JPRB, 0.17447E+01_JPRB ,&
&  0.19435E+01_JPRB, 0.19970E+01_JPRB, 0.15039E+01_JPRB, 0.13300E+01_JPRB, 0.13391E+01_JPRB, 0.11192E+01_JPRB, 0.12342E+01_JPRB ,&
&  0.12233E+01_JPRB, 0.12545E+01_JPRB, 0.12973E+01_JPRB, 0.13990E+01_JPRB, 0.14958E+01_JPRB, 0.15529E+01_JPRB, 0.13893E+01_JPRB ,&
&  0.12210E+01_JPRB, 0.11446E+01_JPRB, 0.11221E+01_JPRB, 0.11077E+01_JPRB, 0.11227E+01_JPRB, 0.11272E+01_JPRB, 0.11035E+01_JPRB ,&
&  0.10723E+01_JPRB, 0.10165E+01_JPRB, 0.98977E+00_JPRB, 0.91984E+00_JPRB, 0.86133E+00_JPRB, 0.87656E+00_JPRB, 0.83350E+00_JPRB ,&
&  0.82573E+00_JPRB, 0.83217E+00_JPRB, 0.82610E+00_JPRB, 0.81144E+00_JPRB, 0.80319E+00_JPRB, 0.78437E+00_JPRB, 0.77541E+00_JPRB ,&
&  0.77024E+00_JPRB, 0.75562E+00_JPRB, 0.74180E+00_JPRB, 0.73435E+00_JPRB, 0.71409E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.68568E+00_JPRB, 0.67883E+00_JPRB, 0.68047E+00_JPRB, 0.68504E+00_JPRB, 0.67672E+00_JPRB, 0.66887E+00_JPRB, 0.66125E+00_JPRB ,&
&  0.63166E+00_JPRB, 0.62832E+00_JPRB, 0.61649E+00_JPRB, 0.62287E+00_JPRB, 0.64735E+00_JPRB, 0.67075E+00_JPRB, 0.71043E+00_JPRB ,&
&  0.75127E+00_JPRB, 0.80637E+00_JPRB, 0.89303E+00_JPRB, 0.10727E+01_JPRB, 0.12711E+01_JPRB, 0.14115E+01_JPRB, 0.15768E+01_JPRB ,&
&  0.16315E+01_JPRB, 0.16771E+01_JPRB, 0.17833E+01_JPRB, 0.19194E+01_JPRB, 0.19601E+01_JPRB, 0.19597E+01_JPRB, 0.18684E+01_JPRB ,&
&  0.13482E+01_JPRB, 0.94910E+00_JPRB, 0.97445E+00_JPRB, 0.92455E+00_JPRB, 0.95496E+00_JPRB, 0.99584E+00_JPRB, 0.10476E+01_JPRB ,&
&  0.11298E+01_JPRB, 0.12121E+01_JPRB, 0.12967E+01_JPRB, 0.13651E+01_JPRB, 0.14530E+01_JPRB, 0.15500E+01_JPRB, 0.16022E+01_JPRB ,&
&  0.16462E+01_JPRB, 0.16998E+01_JPRB, 0.17520E+01_JPRB, 0.17222E+01_JPRB, 0.18037E+01_JPRB, 0.19244E+01_JPRB, 0.20204E+01_JPRB ,&
&  0.21106E+01_JPRB, 0.20982E+01_JPRB, 0.17689E+01_JPRB, 0.18248E+01_JPRB, 0.20689E+01_JPRB, 0.19433E+01_JPRB, 0.19916E+01_JPRB ,&
&  0.22948E+01_JPRB, 0.23522E+01_JPRB, 0.13875E+01_JPRB, 0.11983E+01_JPRB, 0.11984E+01_JPRB, 0.11607E+01_JPRB, 0.11828E+01_JPRB ,&
&  0.11748E+01_JPRB, 0.12190E+01_JPRB, 0.12871E+01_JPRB, 0.13164E+01_JPRB, 0.13215E+01_JPRB, 0.12492E+01_JPRB, 0.11784E+01_JPRB ,&
&  0.10406E+01_JPRB, 0.97752E+00_JPRB, 0.97897E+00_JPRB, 0.99261E+00_JPRB, 0.99363E+00_JPRB, 0.10044E+01_JPRB, 0.95850E+00_JPRB ,&
&  0.93853E+00_JPRB, 0.90755E+00_JPRB, 0.97497E+00_JPRB, 0.89454E+00_JPRB, 0.78957E+00_JPRB, 0.81312E+00_JPRB, 0.79532E+00_JPRB ,&
&  0.82490E+00_JPRB, 0.83528E+00_JPRB, 0.83690E+00_JPRB, 0.83408E+00_JPRB, 0.82967E+00_JPRB, 0.81656E+00_JPRB, 0.80161E+00_JPRB ,&
&  0.78345E+00_JPRB, 0.76316E+00_JPRB, 0.74068E+00_JPRB, 0.71976E+00_JPRB, 0.70057E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.62675E+00_JPRB, 0.61873E+00_JPRB, 0.62661E+00_JPRB, 0.60758E+00_JPRB, 0.62737E+00_JPRB, 0.61377E+00_JPRB, 0.64153E+00_JPRB ,&
&  0.61769E+00_JPRB, 0.61208E+00_JPRB, 0.61141E+00_JPRB, 0.62144E+00_JPRB, 0.63238E+00_JPRB, 0.64910E+00_JPRB, 0.66197E+00_JPRB ,&
&  0.68456E+00_JPRB, 0.72764E+00_JPRB, 0.78695E+00_JPRB, 0.85412E+00_JPRB, 0.94157E+00_JPRB, 0.10267E+01_JPRB, 0.10989E+01_JPRB ,&
&  0.11961E+01_JPRB, 0.12776E+01_JPRB, 0.13183E+01_JPRB, 0.14394E+01_JPRB, 0.18042E+01_JPRB, 0.22512E+01_JPRB, 0.22736E+01_JPRB ,&
&  0.14255E+01_JPRB, 0.10029E+01_JPRB, 0.92396E+00_JPRB, 0.86625E+00_JPRB, 0.91331E+00_JPRB, 0.10272E+01_JPRB, 0.10770E+01_JPRB ,&
&  0.11224E+01_JPRB, 0.11787E+01_JPRB, 0.12512E+01_JPRB, 0.13294E+01_JPRB, 0.14279E+01_JPRB, 0.15355E+01_JPRB, 0.15919E+01_JPRB ,&
&  0.15975E+01_JPRB, 0.16365E+01_JPRB, 0.16526E+01_JPRB, 0.17825E+01_JPRB, 0.19847E+01_JPRB, 0.22263E+01_JPRB, 0.24059E+01_JPRB ,&
&  0.25036E+01_JPRB, 0.24537E+01_JPRB, 0.20768E+01_JPRB, 0.20886E+01_JPRB, 0.22030E+01_JPRB, 0.21654E+01_JPRB, 0.20855E+01_JPRB ,&
&  0.22837E+01_JPRB, 0.21592E+01_JPRB, 0.14346E+01_JPRB, 0.12957E+01_JPRB, 0.12180E+01_JPRB, 0.11709E+01_JPRB, 0.11079E+01_JPRB ,&
&  0.11327E+01_JPRB, 0.11711E+01_JPRB, 0.11979E+01_JPRB, 0.11831E+01_JPRB, 0.11452E+01_JPRB, 0.10994E+01_JPRB, 0.10879E+01_JPRB ,&
&  0.10020E+01_JPRB, 0.99032E+00_JPRB, 0.98161E+00_JPRB, 0.93193E+00_JPRB, 0.99095E+00_JPRB, 0.87106E+00_JPRB, 0.89695E+00_JPRB ,&
&  0.80699E+00_JPRB, 0.78645E+00_JPRB, 0.69391E+00_JPRB, 0.84742E+00_JPRB, 0.79553E+00_JPRB, 0.74125E+00_JPRB, 0.76348E+00_JPRB ,&
&  0.77748E+00_JPRB, 0.79838E+00_JPRB, 0.78943E+00_JPRB, 0.79016E+00_JPRB, 0.79531E+00_JPRB, 0.78511E+00_JPRB, 0.77367E+00_JPRB ,&
&  0.75884E+00_JPRB, 0.74157E+00_JPRB, 0.71000E+00_JPRB, 0.68040E+00_JPRB, 0.65203E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.61813E+00_JPRB, 0.62424E+00_JPRB, 0.61254E+00_JPRB, 0.59071E+00_JPRB, 0.57468E+00_JPRB, 0.56736E+00_JPRB, 0.57533E+00_JPRB ,&
&  0.58483E+00_JPRB, 0.60480E+00_JPRB, 0.62039E+00_JPRB, 0.63769E+00_JPRB, 0.64635E+00_JPRB, 0.65313E+00_JPRB, 0.66775E+00_JPRB ,&
&  0.68709E+00_JPRB, 0.70729E+00_JPRB, 0.73233E+00_JPRB, 0.76683E+00_JPRB, 0.79757E+00_JPRB, 0.83569E+00_JPRB, 0.84984E+00_JPRB ,&
&  0.84648E+00_JPRB, 0.86850E+00_JPRB, 0.88244E+00_JPRB, 0.91814E+00_JPRB, 0.10953E+01_JPRB, 0.12741E+01_JPRB, 0.14589E+01_JPRB ,&
&  0.10973E+01_JPRB, 0.10968E+01_JPRB, 0.98670E+00_JPRB, 0.91153E+00_JPRB, 0.93451E+00_JPRB, 0.92666E+00_JPRB, 0.94941E+00_JPRB ,&
&  0.98489E+00_JPRB, 0.10012E+01_JPRB, 0.10192E+01_JPRB, 0.10473E+01_JPRB, 0.10807E+01_JPRB, 0.11490E+01_JPRB, 0.11234E+01_JPRB ,&
&  0.11162E+01_JPRB, 0.11710E+01_JPRB, 0.12379E+01_JPRB, 0.14211E+01_JPRB, 0.16809E+01_JPRB, 0.19110E+01_JPRB, 0.19370E+01_JPRB ,&
&  0.19591E+01_JPRB, 0.20930E+01_JPRB, 0.20351E+01_JPRB, 0.20371E+01_JPRB, 0.20013E+01_JPRB, 0.19311E+01_JPRB, 0.19332E+01_JPRB ,&
&  0.18336E+01_JPRB, 0.16682E+01_JPRB, 0.15108E+01_JPRB, 0.13145E+01_JPRB, 0.11837E+01_JPRB, 0.10901E+01_JPRB, 0.10557E+01_JPRB ,&
&  0.10331E+01_JPRB, 0.10546E+01_JPRB, 0.10664E+01_JPRB, 0.10274E+01_JPRB, 0.10066E+01_JPRB, 0.97814E+00_JPRB, 0.97626E+00_JPRB ,&
&  0.96612E+00_JPRB, 0.89120E+00_JPRB, 0.83898E+00_JPRB, 0.80435E+00_JPRB, 0.78223E+00_JPRB, 0.10807E+01_JPRB, 0.10322E+01_JPRB ,&
&  0.70080E+00_JPRB, 0.58235E+00_JPRB, 0.51671E+00_JPRB, 0.73871E+00_JPRB, 0.67820E+00_JPRB, 0.67863E+00_JPRB, 0.69835E+00_JPRB ,&
&  0.72014E+00_JPRB, 0.73459E+00_JPRB, 0.74343E+00_JPRB, 0.73259E+00_JPRB, 0.72886E+00_JPRB, 0.72542E+00_JPRB, 0.70830E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.67657E+00_JPRB, 0.67170E+00_JPRB, 0.63949E+00_JPRB, 0.62267E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.60025E+00_JPRB, 0.60005E+00_JPRB, 0.57982E+00_JPRB, 0.55163E+00_JPRB, 0.53852E+00_JPRB, 0.54771E+00_JPRB, 0.56335E+00_JPRB ,&
&  0.57790E+00_JPRB, 0.59759E+00_JPRB, 0.62041E+00_JPRB, 0.64259E+00_JPRB, 0.66221E+00_JPRB, 0.68330E+00_JPRB, 0.70815E+00_JPRB ,&
&  0.73170E+00_JPRB, 0.76080E+00_JPRB, 0.78991E+00_JPRB, 0.81856E+00_JPRB, 0.84266E+00_JPRB, 0.88580E+00_JPRB, 0.91464E+00_JPRB ,&
&  0.96201E+00_JPRB, 0.10096E+01_JPRB, 0.10666E+01_JPRB, 0.11199E+01_JPRB, 0.11849E+01_JPRB, 0.12269E+01_JPRB, 0.10354E+01_JPRB ,&
&  0.11619E+01_JPRB, 0.11122E+01_JPRB, 0.10123E+01_JPRB, 0.93318E+00_JPRB, 0.88112E+00_JPRB, 0.84365E+00_JPRB, 0.80656E+00_JPRB ,&
&  0.79262E+00_JPRB, 0.77574E+00_JPRB, 0.74607E+00_JPRB, 0.72803E+00_JPRB, 0.72909E+00_JPRB, 0.78688E+00_JPRB, 0.78470E+00_JPRB ,&
&  0.85502E+00_JPRB, 0.97500E+00_JPRB, 0.10747E+01_JPRB, 0.12336E+01_JPRB, 0.13738E+01_JPRB, 0.14673E+01_JPRB, 0.14198E+01_JPRB ,&
&  0.14359E+01_JPRB, 0.15569E+01_JPRB, 0.15265E+01_JPRB, 0.15551E+01_JPRB, 0.15337E+01_JPRB, 0.14623E+01_JPRB, 0.14152E+01_JPRB ,&
&  0.12578E+01_JPRB, 0.12290E+01_JPRB, 0.12786E+01_JPRB, 0.13752E+01_JPRB, 0.11841E+01_JPRB, 0.10609E+01_JPRB, 0.10378E+01_JPRB ,&
&  0.96174E+00_JPRB, 0.96630E+00_JPRB, 0.97041E+00_JPRB, 0.92243E+00_JPRB, 0.88883E+00_JPRB, 0.86083E+00_JPRB, 0.85753E+00_JPRB ,&
&  0.86203E+00_JPRB, 0.77176E+00_JPRB, 0.73003E+00_JPRB, 0.73069E+00_JPRB, 0.77832E+00_JPRB, 0.80273E+00_JPRB, 0.94941E+00_JPRB ,&
&  0.63707E+00_JPRB, 0.42360E+00_JPRB, 0.48467E+00_JPRB, 0.58763E+00_JPRB, 0.57854E+00_JPRB, 0.59216E+00_JPRB, 0.63683E+00_JPRB ,&
&  0.68091E+00_JPRB, 0.70573E+00_JPRB, 0.69724E+00_JPRB, 0.71941E+00_JPRB, 0.75111E+00_JPRB, 0.73736E+00_JPRB, 0.67988E+00_JPRB ,&
&  0.68554E+00_JPRB, 0.64950E+00_JPRB, 0.62969E+00_JPRB, 0.61909E+00_JPRB, 0.61145E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.57688E+00_JPRB, 0.56863E+00_JPRB, 0.54925E+00_JPRB, 0.51986E+00_JPRB, 0.50914E+00_JPRB, 0.52727E+00_JPRB, 0.55232E+00_JPRB ,&
&  0.57082E+00_JPRB, 0.59378E+00_JPRB, 0.61774E+00_JPRB, 0.64399E+00_JPRB, 0.67302E+00_JPRB, 0.69984E+00_JPRB, 0.72048E+00_JPRB ,&
&  0.73461E+00_JPRB, 0.75374E+00_JPRB, 0.77009E+00_JPRB, 0.78267E+00_JPRB, 0.78530E+00_JPRB, 0.78610E+00_JPRB, 0.78441E+00_JPRB ,&
&  0.81446E+00_JPRB, 0.84343E+00_JPRB, 0.88118E+00_JPRB, 0.91742E+00_JPRB, 0.93579E+00_JPRB, 0.91183E+00_JPRB, 0.72915E+00_JPRB ,&
&  0.10580E+01_JPRB, 0.10721E+01_JPRB, 0.99201E+00_JPRB, 0.91204E+00_JPRB, 0.84044E+00_JPRB, 0.78984E+00_JPRB, 0.74051E+00_JPRB ,&
&  0.69923E+00_JPRB, 0.72437E+00_JPRB, 0.71226E+00_JPRB, 0.72214E+00_JPRB, 0.70693E+00_JPRB, 0.75556E+00_JPRB, 0.80557E+00_JPRB ,&
&  0.90867E+00_JPRB, 0.10196E+01_JPRB, 0.11193E+01_JPRB, 0.12032E+01_JPRB, 0.12466E+01_JPRB, 0.12800E+01_JPRB, 0.12270E+01_JPRB ,&
&  0.12302E+01_JPRB, 0.12294E+01_JPRB, 0.11432E+01_JPRB, 0.11615E+01_JPRB, 0.11920E+01_JPRB, 0.10397E+01_JPRB, 0.93119E+00_JPRB ,&
&  0.84272E+00_JPRB, 0.95495E+00_JPRB, 0.12599E+01_JPRB, 0.13886E+01_JPRB, 0.11939E+01_JPRB, 0.11164E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.93716E+00_JPRB, 0.91901E+00_JPRB, 0.89804E+00_JPRB, 0.85769E+00_JPRB, 0.79109E+00_JPRB, 0.75348E+00_JPRB, 0.75091E+00_JPRB ,&
&  0.75605E+00_JPRB, 0.68537E+00_JPRB, 0.63180E+00_JPRB, 0.64813E+00_JPRB, 0.65495E+00_JPRB, 0.59998E+00_JPRB, 0.88329E+00_JPRB ,&
&  0.64234E+00_JPRB, 0.48537E+00_JPRB, 0.62607E+00_JPRB, 0.39218E+00_JPRB, 0.49179E+00_JPRB, 0.48984E+00_JPRB, 0.51416E+00_JPRB ,&
&  0.44607E+00_JPRB, 0.52032E+00_JPRB, 0.87664E+00_JPRB, 0.12100E+01_JPRB, 0.13939E+01_JPRB, 0.13808E+01_JPRB, 0.77634E+00_JPRB ,&
&  0.67828E+00_JPRB, 0.65012E+00_JPRB, 0.62927E+00_JPRB, 0.60575E+00_JPRB, 0.58095E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.55459E+00_JPRB, 0.52376E+00_JPRB, 0.51446E+00_JPRB, 0.51036E+00_JPRB, 0.50160E+00_JPRB, 0.52051E+00_JPRB, 0.53543E+00_JPRB ,&
&  0.55653E+00_JPRB, 0.58338E+00_JPRB, 0.61624E+00_JPRB, 0.64505E+00_JPRB, 0.68662E+00_JPRB, 0.72641E+00_JPRB, 0.75657E+00_JPRB ,&
&  0.77952E+00_JPRB, 0.79563E+00_JPRB, 0.80536E+00_JPRB, 0.80769E+00_JPRB, 0.80415E+00_JPRB, 0.79984E+00_JPRB, 0.79238E+00_JPRB ,&
&  0.78507E+00_JPRB, 0.79444E+00_JPRB, 0.82111E+00_JPRB, 0.85207E+00_JPRB, 0.86159E+00_JPRB, 0.86667E+00_JPRB, 0.56284E+00_JPRB ,&
&  0.92380E+00_JPRB, 0.95940E+00_JPRB, 0.93745E+00_JPRB, 0.87206E+00_JPRB, 0.80755E+00_JPRB, 0.76107E+00_JPRB, 0.72436E+00_JPRB ,&
&  0.70180E+00_JPRB, 0.71682E+00_JPRB, 0.71661E+00_JPRB, 0.81423E+00_JPRB, 0.89121E+00_JPRB, 0.89416E+00_JPRB, 0.98270E+00_JPRB ,&
&  0.10551E+01_JPRB, 0.11569E+01_JPRB, 0.12258E+01_JPRB, 0.12802E+01_JPRB, 0.13214E+01_JPRB, 0.13624E+01_JPRB, 0.13790E+01_JPRB ,&
&  0.13848E+01_JPRB, 0.13487E+01_JPRB, 0.13024E+01_JPRB, 0.13318E+01_JPRB, 0.13388E+01_JPRB, 0.11484E+01_JPRB, 0.10473E+01_JPRB ,&
&  0.78963E+00_JPRB, 0.74551E+00_JPRB, 0.11005E+01_JPRB, 0.13436E+01_JPRB, 0.12263E+01_JPRB, 0.11717E+01_JPRB, 0.10458E+01_JPRB ,&
&  0.92572E+00_JPRB, 0.86065E+00_JPRB, 0.82213E+00_JPRB, 0.78086E+00_JPRB, 0.72630E+00_JPRB, 0.68842E+00_JPRB, 0.66759E+00_JPRB ,&
&  0.65042E+00_JPRB, 0.58571E+00_JPRB, 0.55041E+00_JPRB, 0.55362E+00_JPRB, 0.55075E+00_JPRB, 0.60852E+00_JPRB, 0.60103E+00_JPRB ,&
&  0.72881E+00_JPRB, 0.67266E+00_JPRB, 0.55959E+00_JPRB, 0.53968E+00_JPRB, 0.46034E+00_JPRB, 0.44438E+00_JPRB, 0.44685E+00_JPRB ,&
&  0.46990E+00_JPRB, 0.43112E+00_JPRB, 0.92579E+00_JPRB, 0.37060E+01_JPRB, 0.36701E+01_JPRB, 0.46627E+01_JPRB, 0.24209E+01_JPRB ,&
&  0.11357E+01_JPRB, 0.69364E+00_JPRB, 0.59703E+00_JPRB, 0.58664E+00_JPRB, 0.56786E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.50486E+00_JPRB, 0.48705E+00_JPRB, 0.47086E+00_JPRB, 0.48297E+00_JPRB, 0.49194E+00_JPRB, 0.49805E+00_JPRB, 0.52430E+00_JPRB ,&
&  0.57884E+00_JPRB, 0.59255E+00_JPRB, 0.61926E+00_JPRB, 0.66167E+00_JPRB, 0.72319E+00_JPRB, 0.76889E+00_JPRB, 0.81027E+00_JPRB ,&
&  0.85153E+00_JPRB, 0.87307E+00_JPRB, 0.86048E+00_JPRB, 0.84010E+00_JPRB, 0.83152E+00_JPRB, 0.82991E+00_JPRB, 0.83636E+00_JPRB ,&
&  0.83222E+00_JPRB, 0.84143E+00_JPRB, 0.86421E+00_JPRB, 0.89253E+00_JPRB, 0.91367E+00_JPRB, 0.88931E+00_JPRB, 0.56093E+00_JPRB ,&
&  0.74699E+00_JPRB, 0.80315E+00_JPRB, 0.82949E+00_JPRB, 0.80849E+00_JPRB, 0.75413E+00_JPRB, 0.69672E+00_JPRB, 0.68935E+00_JPRB ,&
&  0.73393E+00_JPRB, 0.80096E+00_JPRB, 0.86877E+00_JPRB, 0.92442E+00_JPRB, 0.10819E+01_JPRB, 0.11071E+01_JPRB, 0.12569E+01_JPRB ,&
&  0.13375E+01_JPRB, 0.14027E+01_JPRB, 0.14849E+01_JPRB, 0.15363E+01_JPRB, 0.16058E+01_JPRB, 0.16906E+01_JPRB, 0.17691E+01_JPRB ,&
&  0.18035E+01_JPRB, 0.17956E+01_JPRB, 0.18453E+01_JPRB, 0.19962E+01_JPRB, 0.18310E+01_JPRB, 0.20154E+01_JPRB, 0.16396E+01_JPRB ,&
&  0.11183E+01_JPRB, 0.95328E+00_JPRB, 0.11615E+01_JPRB, 0.13040E+01_JPRB, 0.12379E+01_JPRB, 0.12107E+01_JPRB, 0.11025E+01_JPRB ,&
&  0.94836E+00_JPRB, 0.84025E+00_JPRB, 0.79097E+00_JPRB, 0.75326E+00_JPRB, 0.71047E+00_JPRB, 0.66234E+00_JPRB, 0.62257E+00_JPRB ,&
&  0.57681E+00_JPRB, 0.54556E+00_JPRB, 0.55667E+00_JPRB, 0.54938E+00_JPRB, 0.56855E+00_JPRB, 0.65226E+00_JPRB, 0.85887E+00_JPRB ,&
&  0.13178E+01_JPRB, 0.71183E+00_JPRB, 0.59409E+00_JPRB, 0.54850E+00_JPRB, 0.49944E+00_JPRB, 0.47064E+00_JPRB, 0.45455E+00_JPRB ,&
&  0.46265E+00_JPRB, 0.51742E+00_JPRB, 0.64849E+00_JPRB, 0.13723E+01_JPRB, 0.25268E+01_JPRB, 0.25918E+01_JPRB, 0.17387E+01_JPRB ,&
&  0.10464E+01_JPRB, 0.71762E+00_JPRB, 0.57331E+00_JPRB, 0.53677E+00_JPRB, 0.51082E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.48090E+00_JPRB, 0.46909E+00_JPRB, 0.47001E+00_JPRB, 0.46999E+00_JPRB, 0.50547E+00_JPRB, 0.52909E+00_JPRB, 0.55031E+00_JPRB ,&
&  0.58187E+00_JPRB, 0.60154E+00_JPRB, 0.63313E+00_JPRB, 0.69025E+00_JPRB, 0.75049E+00_JPRB, 0.79727E+00_JPRB, 0.81632E+00_JPRB ,&
&  0.82404E+00_JPRB, 0.81644E+00_JPRB, 0.79714E+00_JPRB, 0.77353E+00_JPRB, 0.77867E+00_JPRB, 0.77482E+00_JPRB, 0.79379E+00_JPRB ,&
&  0.82577E+00_JPRB, 0.85941E+00_JPRB, 0.90147E+00_JPRB, 0.94050E+00_JPRB, 0.97707E+00_JPRB, 0.99298E+00_JPRB, 0.95437E+00_JPRB ,&
&  0.46890E+00_JPRB, 0.66508E+00_JPRB, 0.70443E+00_JPRB, 0.69621E+00_JPRB, 0.65405E+00_JPRB, 0.63631E+00_JPRB, 0.69571E+00_JPRB ,&
&  0.78964E+00_JPRB, 0.84655E+00_JPRB, 0.96831E+00_JPRB, 0.10452E+01_JPRB, 0.12637E+01_JPRB, 0.13436E+01_JPRB, 0.14761E+01_JPRB ,&
&  0.15731E+01_JPRB, 0.16437E+01_JPRB, 0.17423E+01_JPRB, 0.18469E+01_JPRB, 0.19648E+01_JPRB, 0.20947E+01_JPRB, 0.22568E+01_JPRB ,&
&  0.24092E+01_JPRB, 0.25994E+01_JPRB, 0.28556E+01_JPRB, 0.28332E+01_JPRB, 0.26206E+01_JPRB, 0.27958E+01_JPRB, 0.26563E+01_JPRB ,&
&  0.22371E+01_JPRB, 0.16082E+01_JPRB, 0.15469E+01_JPRB, 0.14395E+01_JPRB, 0.13184E+01_JPRB, 0.12642E+01_JPRB, 0.11619E+01_JPRB ,&
&  0.10030E+01_JPRB, 0.91229E+00_JPRB, 0.83062E+00_JPRB, 0.77767E+00_JPRB, 0.71464E+00_JPRB, 0.67056E+00_JPRB, 0.63863E+00_JPRB ,&
&  0.61719E+00_JPRB, 0.63110E+00_JPRB, 0.64611E+00_JPRB, 0.63066E+00_JPRB, 0.60052E+00_JPRB, 0.61613E+00_JPRB, 0.67398E+00_JPRB ,&
&  0.75954E+00_JPRB, 0.67951E+00_JPRB, 0.62568E+00_JPRB, 0.62508E+00_JPRB, 0.61620E+00_JPRB, 0.60778E+00_JPRB, 0.60340E+00_JPRB ,&
&  0.61631E+00_JPRB, 0.60719E+00_JPRB, 0.70189E+00_JPRB, 0.10813E+01_JPRB, 0.14625E+01_JPRB, 0.17096E+01_JPRB, 0.14097E+01_JPRB ,&
&  0.99572E+00_JPRB, 0.73404E+00_JPRB, 0.58439E+00_JPRB, 0.54367E+00_JPRB, 0.52151E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.52605E+00_JPRB, 0.51636E+00_JPRB, 0.51663E+00_JPRB, 0.51393E+00_JPRB, 0.54644E+00_JPRB, 0.58247E+00_JPRB, 0.59273E+00_JPRB ,&
&  0.58695E+00_JPRB, 0.60348E+00_JPRB, 0.61929E+00_JPRB, 0.64890E+00_JPRB, 0.71122E+00_JPRB, 0.73843E+00_JPRB, 0.77060E+00_JPRB ,&
&  0.80194E+00_JPRB, 0.79510E+00_JPRB, 0.76046E+00_JPRB, 0.73962E+00_JPRB, 0.75451E+00_JPRB, 0.76369E+00_JPRB, 0.78947E+00_JPRB ,&
&  0.82997E+00_JPRB, 0.87601E+00_JPRB, 0.93209E+00_JPRB, 0.10039E+01_JPRB, 0.10857E+01_JPRB, 0.11470E+01_JPRB, 0.11220E+01_JPRB ,&
&  0.48965E+00_JPRB, 0.47585E+00_JPRB, 0.60027E+00_JPRB, 0.60813E+00_JPRB, 0.57497E+00_JPRB, 0.60190E+00_JPRB, 0.66851E+00_JPRB ,&
&  0.74153E+00_JPRB, 0.76971E+00_JPRB, 0.89669E+00_JPRB, 0.12420E+01_JPRB, 0.13770E+01_JPRB, 0.15012E+01_JPRB, 0.16113E+01_JPRB ,&
&  0.17063E+01_JPRB, 0.18077E+01_JPRB, 0.18964E+01_JPRB, 0.19759E+01_JPRB, 0.20770E+01_JPRB, 0.22075E+01_JPRB, 0.23849E+01_JPRB ,&
&  0.26122E+01_JPRB, 0.29496E+01_JPRB, 0.32803E+01_JPRB, 0.30140E+01_JPRB, 0.30970E+01_JPRB, 0.32846E+01_JPRB, 0.37503E+01_JPRB ,&
&  0.33750E+01_JPRB, 0.23632E+01_JPRB, 0.18067E+01_JPRB, 0.15486E+01_JPRB, 0.12749E+01_JPRB, 0.11949E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.10134E+01_JPRB, 0.95117E+00_JPRB, 0.89052E+00_JPRB, 0.81715E+00_JPRB, 0.72913E+00_JPRB, 0.70117E+00_JPRB, 0.68641E+00_JPRB ,&
&  0.71217E+00_JPRB, 0.72921E+00_JPRB, 0.73036E+00_JPRB, 0.73081E+00_JPRB, 0.69074E+00_JPRB, 0.69891E+00_JPRB, 0.73648E+00_JPRB ,&
&  0.77280E+00_JPRB, 0.79835E+00_JPRB, 0.80936E+00_JPRB, 0.82051E+00_JPRB, 0.83785E+00_JPRB, 0.84630E+00_JPRB, 0.81980E+00_JPRB ,&
&  0.78346E+00_JPRB, 0.75332E+00_JPRB, 0.76034E+00_JPRB, 0.75604E+00_JPRB, 0.91359E+00_JPRB, 0.12145E+01_JPRB, 0.12551E+01_JPRB ,&
&  0.10687E+01_JPRB, 0.81780E+00_JPRB, 0.65666E+00_JPRB, 0.60606E+00_JPRB, 0.58295E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.62932E+00_JPRB, 0.59870E+00_JPRB, 0.59030E+00_JPRB, 0.57385E+00_JPRB, 0.58699E+00_JPRB, 0.61244E+00_JPRB, 0.63184E+00_JPRB ,&
&  0.63550E+00_JPRB, 0.63383E+00_JPRB, 0.63342E+00_JPRB, 0.65093E+00_JPRB, 0.67594E+00_JPRB, 0.70251E+00_JPRB, 0.73584E+00_JPRB ,&
&  0.75693E+00_JPRB, 0.76672E+00_JPRB, 0.73728E+00_JPRB, 0.69693E+00_JPRB, 0.70038E+00_JPRB, 0.71448E+00_JPRB, 0.73954E+00_JPRB ,&
&  0.77705E+00_JPRB, 0.81774E+00_JPRB, 0.87454E+00_JPRB, 0.97704E+00_JPRB, 0.11347E+01_JPRB, 0.13220E+01_JPRB, 0.15118E+01_JPRB ,&
&  0.11719E+01_JPRB, 0.56500E+00_JPRB, 0.57658E+00_JPRB, 0.64980E+00_JPRB, 0.60009E+00_JPRB, 0.62800E+00_JPRB, 0.65204E+00_JPRB ,&
&  0.68207E+00_JPRB, 0.75572E+00_JPRB, 0.82441E+00_JPRB, 0.10331E+01_JPRB, 0.12977E+01_JPRB, 0.14470E+01_JPRB, 0.15799E+01_JPRB ,&
&  0.16843E+01_JPRB, 0.17588E+01_JPRB, 0.18027E+01_JPRB, 0.18480E+01_JPRB, 0.19330E+01_JPRB, 0.20628E+01_JPRB, 0.22390E+01_JPRB ,&
&  0.24856E+01_JPRB, 0.27957E+01_JPRB, 0.31772E+01_JPRB, 0.29099E+01_JPRB, 0.32788E+01_JPRB, 0.32882E+01_JPRB, 0.33378E+01_JPRB ,&
&  0.30888E+01_JPRB, 0.21266E+01_JPRB, 0.16328E+01_JPRB, 0.15253E+01_JPRB, 0.12165E+01_JPRB, 0.92192E+00_JPRB, 0.99281E+00_JPRB ,&
&  0.97877E+00_JPRB, 0.95862E+00_JPRB, 0.93847E+00_JPRB, 0.84851E+00_JPRB, 0.76767E+00_JPRB, 0.74246E+00_JPRB, 0.78909E+00_JPRB ,&
&  0.82190E+00_JPRB, 0.84504E+00_JPRB, 0.83706E+00_JPRB, 0.83394E+00_JPRB, 0.80851E+00_JPRB, 0.82241E+00_JPRB, 0.85730E+00_JPRB ,&
&  0.89092E+00_JPRB, 0.92378E+00_JPRB, 0.93995E+00_JPRB, 0.96758E+00_JPRB, 0.10045E+01_JPRB, 0.95130E+00_JPRB, 0.87950E+00_JPRB ,&
&  0.86544E+00_JPRB, 0.85731E+00_JPRB, 0.81823E+00_JPRB, 0.80242E+00_JPRB, 0.79466E+00_JPRB, 0.83294E+00_JPRB, 0.93963E+00_JPRB ,&
&  0.92648E+00_JPRB, 0.81744E+00_JPRB, 0.70536E+00_JPRB, 0.67423E+00_JPRB, 0.68008E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.70996E+00_JPRB, 0.64482E+00_JPRB, 0.60470E+00_JPRB, 0.59983E+00_JPRB, 0.58745E+00_JPRB, 0.59344E+00_JPRB, 0.60265E+00_JPRB ,&
&  0.60086E+00_JPRB, 0.61984E+00_JPRB, 0.61330E+00_JPRB, 0.62546E+00_JPRB, 0.64502E+00_JPRB, 0.65016E+00_JPRB, 0.67189E+00_JPRB ,&
&  0.69454E+00_JPRB, 0.72062E+00_JPRB, 0.74086E+00_JPRB, 0.70233E+00_JPRB, 0.69158E+00_JPRB, 0.68986E+00_JPRB, 0.69933E+00_JPRB ,&
&  0.72476E+00_JPRB, 0.76125E+00_JPRB, 0.80324E+00_JPRB, 0.85830E+00_JPRB, 0.94075E+00_JPRB, 0.10916E+01_JPRB, 0.13784E+01_JPRB ,&
&  0.20989E+01_JPRB, 0.16594E+01_JPRB, 0.59098E+00_JPRB, 0.73627E+00_JPRB, 0.72590E+00_JPRB, 0.71671E+00_JPRB, 0.63318E+00_JPRB ,&
&  0.66502E+00_JPRB, 0.74614E+00_JPRB, 0.78464E+00_JPRB, 0.86616E+00_JPRB, 0.10136E+01_JPRB, 0.12207E+01_JPRB, 0.13933E+01_JPRB ,&
&  0.15049E+01_JPRB, 0.15815E+01_JPRB, 0.16098E+01_JPRB, 0.16400E+01_JPRB, 0.16857E+01_JPRB, 0.17755E+01_JPRB, 0.19198E+01_JPRB ,&
&  0.21373E+01_JPRB, 0.24498E+01_JPRB, 0.28598E+01_JPRB, 0.28119E+01_JPRB, 0.34385E+01_JPRB, 0.33069E+01_JPRB, 0.30895E+01_JPRB ,&
&  0.25668E+01_JPRB, 0.20947E+01_JPRB, 0.17949E+01_JPRB, 0.14867E+01_JPRB, 0.10856E+01_JPRB, 0.89078E+00_JPRB, 0.93396E+00_JPRB ,&
&  0.94726E+00_JPRB, 0.86192E+00_JPRB, 0.84052E+00_JPRB, 0.82619E+00_JPRB, 0.78006E+00_JPRB, 0.82743E+00_JPRB, 0.90070E+00_JPRB ,&
&  0.92058E+00_JPRB, 0.92151E+00_JPRB, 0.93173E+00_JPRB, 0.91695E+00_JPRB, 0.90661E+00_JPRB, 0.91432E+00_JPRB, 0.94515E+00_JPRB ,&
&  0.96218E+00_JPRB, 0.97948E+00_JPRB, 0.98104E+00_JPRB, 0.10080E+01_JPRB, 0.10339E+01_JPRB, 0.97070E+00_JPRB, 0.88911E+00_JPRB ,&
&  0.87432E+00_JPRB, 0.88735E+00_JPRB, 0.86757E+00_JPRB, 0.85631E+00_JPRB, 0.82752E+00_JPRB, 0.80385E+00_JPRB, 0.80665E+00_JPRB ,&
&  0.89195E+00_JPRB, 0.83387E+00_JPRB, 0.78063E+00_JPRB, 0.71378E+00_JPRB, 0.73459E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.73472E+00_JPRB, 0.69770E+00_JPRB, 0.63120E+00_JPRB, 0.60443E+00_JPRB, 0.60076E+00_JPRB, 0.59201E+00_JPRB, 0.57813E+00_JPRB ,&
&  0.55705E+00_JPRB, 0.55053E+00_JPRB, 0.57627E+00_JPRB, 0.60640E+00_JPRB, 0.63315E+00_JPRB, 0.63420E+00_JPRB, 0.63937E+00_JPRB ,&
&  0.64345E+00_JPRB, 0.66467E+00_JPRB, 0.70526E+00_JPRB, 0.71309E+00_JPRB, 0.69796E+00_JPRB, 0.68920E+00_JPRB, 0.68782E+00_JPRB ,&
&  0.69324E+00_JPRB, 0.70162E+00_JPRB, 0.72526E+00_JPRB, 0.75280E+00_JPRB, 0.80134E+00_JPRB, 0.88475E+00_JPRB, 0.10672E+01_JPRB ,&
&  0.14885E+01_JPRB, 0.25174E+01_JPRB, 0.73048E+00_JPRB, 0.92490E+00_JPRB, 0.87479E+00_JPRB, 0.78812E+00_JPRB, 0.72840E+00_JPRB ,&
&  0.71125E+00_JPRB, 0.73043E+00_JPRB, 0.81247E+00_JPRB, 0.83789E+00_JPRB, 0.77753E+00_JPRB, 0.90386E+00_JPRB, 0.10964E+01_JPRB ,&
&  0.12702E+01_JPRB, 0.13475E+01_JPRB, 0.13730E+01_JPRB, 0.13645E+01_JPRB, 0.13802E+01_JPRB, 0.14141E+01_JPRB, 0.14943E+01_JPRB ,&
&  0.16470E+01_JPRB, 0.19011E+01_JPRB, 0.22634E+01_JPRB, 0.23992E+01_JPRB, 0.31528E+01_JPRB, 0.33578E+01_JPRB, 0.32180E+01_JPRB ,&
&  0.26131E+01_JPRB, 0.23710E+01_JPRB, 0.21129E+01_JPRB, 0.16211E+01_JPRB, 0.11897E+01_JPRB, 0.96985E+00_JPRB, 0.96985E+00_JPRB ,&
&  0.94127E+00_JPRB, 0.86957E+00_JPRB, 0.85391E+00_JPRB, 0.83503E+00_JPRB, 0.83855E+00_JPRB, 0.88916E+00_JPRB, 0.93899E+00_JPRB ,&
&  0.94459E+00_JPRB, 0.95804E+00_JPRB, 0.95296E+00_JPRB, 0.93997E+00_JPRB, 0.94173E+00_JPRB, 0.94952E+00_JPRB, 0.98152E+00_JPRB ,&
&  0.99228E+00_JPRB, 0.10006E+01_JPRB, 0.99896E+00_JPRB, 0.97226E+00_JPRB, 0.97386E+00_JPRB, 0.93686E+00_JPRB, 0.88720E+00_JPRB ,&
&  0.88889E+00_JPRB, 0.90714E+00_JPRB, 0.89121E+00_JPRB, 0.85647E+00_JPRB, 0.88215E+00_JPRB, 0.86441E+00_JPRB, 0.84251E+00_JPRB ,&
&  0.88313E+00_JPRB, 0.90264E+00_JPRB, 0.88550E+00_JPRB, 0.76452E+00_JPRB, 0.77427E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.75435E+00_JPRB, 0.71857E+00_JPRB, 0.67740E+00_JPRB, 0.62479E+00_JPRB, 0.62962E+00_JPRB, 0.64051E+00_JPRB, 0.61801E+00_JPRB ,&
&  0.57317E+00_JPRB, 0.56874E+00_JPRB, 0.57973E+00_JPRB, 0.60268E+00_JPRB, 0.62177E+00_JPRB, 0.62179E+00_JPRB, 0.63841E+00_JPRB ,&
&  0.62961E+00_JPRB, 0.62543E+00_JPRB, 0.64654E+00_JPRB, 0.67963E+00_JPRB, 0.68192E+00_JPRB, 0.67918E+00_JPRB, 0.69029E+00_JPRB ,&
&  0.69576E+00_JPRB, 0.68319E+00_JPRB, 0.67646E+00_JPRB, 0.68294E+00_JPRB, 0.71582E+00_JPRB, 0.77263E+00_JPRB, 0.87142E+00_JPRB ,&
&  0.12799E+01_JPRB, 0.28492E+01_JPRB, 0.21359E+01_JPRB, 0.16276E+01_JPRB, 0.11454E+01_JPRB, 0.90154E+00_JPRB, 0.80279E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83342E+00_JPRB, 0.93396E+00_JPRB, 0.83589E+00_JPRB, 0.75022E+00_JPRB, 0.73237E+00_JPRB, 0.84022E+00_JPRB ,&
&  0.95183E+00_JPRB, 0.10357E+01_JPRB, 0.11009E+01_JPRB, 0.11096E+01_JPRB, 0.11193E+01_JPRB, 0.11430E+01_JPRB, 0.11760E+01_JPRB ,&
&  0.12560E+01_JPRB, 0.14240E+01_JPRB, 0.17062E+01_JPRB, 0.20559E+01_JPRB, 0.24908E+01_JPRB, 0.29643E+01_JPRB, 0.34415E+01_JPRB ,&
&  0.32948E+01_JPRB, 0.29188E+01_JPRB, 0.24082E+01_JPRB, 0.17408E+01_JPRB, 0.12432E+01_JPRB, 0.11783E+01_JPRB, 0.10010E+01_JPRB ,&
&  0.94784E+00_JPRB, 0.89102E+00_JPRB, 0.88934E+00_JPRB, 0.90712E+00_JPRB, 0.93218E+00_JPRB, 0.96367E+00_JPRB, 0.98276E+00_JPRB ,&
&  0.99080E+00_JPRB, 0.98597E+00_JPRB, 0.98195E+00_JPRB, 0.96517E+00_JPRB, 0.95835E+00_JPRB, 0.95929E+00_JPRB, 0.97023E+00_JPRB ,&
&  0.97354E+00_JPRB, 0.98514E+00_JPRB, 0.94137E+00_JPRB, 0.92971E+00_JPRB, 0.92441E+00_JPRB, 0.90613E+00_JPRB, 0.86113E+00_JPRB ,&
&  0.87127E+00_JPRB, 0.91053E+00_JPRB, 0.92147E+00_JPRB, 0.90844E+00_JPRB, 0.90720E+00_JPRB, 0.91882E+00_JPRB, 0.89500E+00_JPRB ,&
&  0.90349E+00_JPRB, 0.89327E+00_JPRB, 0.83923E+00_JPRB, 0.80413E+00_JPRB, 0.80293E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.79273E+00_JPRB, 0.76349E+00_JPRB, 0.73032E+00_JPRB, 0.69023E+00_JPRB, 0.66795E+00_JPRB, 0.67391E+00_JPRB, 0.69269E+00_JPRB ,&
&  0.67231E+00_JPRB, 0.64981E+00_JPRB, 0.64266E+00_JPRB, 0.65743E+00_JPRB, 0.67817E+00_JPRB, 0.67914E+00_JPRB, 0.67669E+00_JPRB ,&
&  0.66154E+00_JPRB, 0.65482E+00_JPRB, 0.64250E+00_JPRB, 0.64992E+00_JPRB, 0.69880E+00_JPRB, 0.69539E+00_JPRB, 0.68417E+00_JPRB ,&
&  0.70176E+00_JPRB, 0.69788E+00_JPRB, 0.67639E+00_JPRB, 0.67293E+00_JPRB, 0.67880E+00_JPRB, 0.71171E+00_JPRB, 0.77930E+00_JPRB ,&
&  0.11954E+01_JPRB, 0.26435E+01_JPRB, 0.14666E+01_JPRB, 0.18149E+01_JPRB, 0.12513E+01_JPRB, 0.10066E+01_JPRB, 0.81621E+00_JPRB ,&
&  0.83262E+00_JPRB, 0.10119E+01_JPRB, 0.91591E+00_JPRB, 0.84968E+00_JPRB, 0.76211E+00_JPRB, 0.72713E+00_JPRB, 0.77630E+00_JPRB ,&
&  0.83395E+00_JPRB, 0.85235E+00_JPRB, 0.88421E+00_JPRB, 0.91097E+00_JPRB, 0.90342E+00_JPRB, 0.90455E+00_JPRB, 0.93341E+00_JPRB ,&
&  0.97140E+00_JPRB, 0.10606E+01_JPRB, 0.12418E+01_JPRB, 0.15057E+01_JPRB, 0.21333E+01_JPRB, 0.28484E+01_JPRB, 0.36374E+01_JPRB ,&
&  0.36860E+01_JPRB, 0.36664E+01_JPRB, 0.27711E+01_JPRB, 0.19022E+01_JPRB, 0.13830E+01_JPRB, 0.12159E+01_JPRB, 0.10735E+01_JPRB ,&
&  0.99632E+00_JPRB, 0.96429E+00_JPRB, 0.93490E+00_JPRB, 0.94331E+00_JPRB, 0.10059E+01_JPRB, 0.10401E+01_JPRB, 0.10226E+01_JPRB ,&
&  0.10230E+01_JPRB, 0.10063E+01_JPRB, 0.99744E+00_JPRB, 0.97373E+00_JPRB, 0.97377E+00_JPRB, 0.96791E+00_JPRB, 0.95608E+00_JPRB ,&
&  0.94769E+00_JPRB, 0.95226E+00_JPRB, 0.92158E+00_JPRB, 0.91987E+00_JPRB, 0.91747E+00_JPRB, 0.90497E+00_JPRB, 0.86632E+00_JPRB ,&
&  0.88334E+00_JPRB, 0.90221E+00_JPRB, 0.90983E+00_JPRB, 0.89722E+00_JPRB, 0.89800E+00_JPRB, 0.94068E+00_JPRB, 0.92156E+00_JPRB ,&
&  0.92435E+00_JPRB, 0.90849E+00_JPRB, 0.85763E+00_JPRB, 0.83990E+00_JPRB, 0.83607E+00_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.83668E+00_JPRB, 0.80106E+00_JPRB, 0.77501E+00_JPRB, 0.74551E+00_JPRB, 0.72262E+00_JPRB, 0.70864E+00_JPRB, 0.71616E+00_JPRB ,&
&  0.71276E+00_JPRB, 0.68882E+00_JPRB, 0.67195E+00_JPRB, 0.67438E+00_JPRB, 0.67254E+00_JPRB, 0.67539E+00_JPRB, 0.67734E+00_JPRB ,&
&  0.67046E+00_JPRB, 0.65794E+00_JPRB, 0.65361E+00_JPRB, 0.65850E+00_JPRB, 0.68626E+00_JPRB, 0.72295E+00_JPRB, 0.72958E+00_JPRB ,&
&  0.72042E+00_JPRB, 0.71391E+00_JPRB, 0.69407E+00_JPRB, 0.67476E+00_JPRB, 0.67019E+00_JPRB, 0.68331E+00_JPRB, 0.73130E+00_JPRB ,&
&  0.11056E+01_JPRB, 0.26487E+01_JPRB, 0.12945E+01_JPRB, 0.18437E+01_JPRB, 0.13120E+01_JPRB, 0.10622E+01_JPRB, 0.86686E+00_JPRB ,&
&  0.88180E+00_JPRB, 0.98280E+00_JPRB, 0.93514E+00_JPRB, 0.87537E+00_JPRB, 0.80394E+00_JPRB, 0.76567E+00_JPRB, 0.76015E+00_JPRB ,&
&  0.80152E+00_JPRB, 0.80263E+00_JPRB, 0.79933E+00_JPRB, 0.80813E+00_JPRB, 0.79400E+00_JPRB, 0.78694E+00_JPRB, 0.79799E+00_JPRB ,&
&  0.80070E+00_JPRB, 0.81880E+00_JPRB, 0.90844E+00_JPRB, 0.12568E+01_JPRB, 0.17224E+01_JPRB, 0.26981E+01_JPRB, 0.41613E+01_JPRB ,&
&  0.41714E+01_JPRB, 0.42979E+01_JPRB, 0.30905E+01_JPRB, 0.21005E+01_JPRB, 0.14887E+01_JPRB, 0.12553E+01_JPRB, 0.11670E+01_JPRB ,&
&  0.10854E+01_JPRB, 0.10274E+01_JPRB, 0.99237E+00_JPRB, 0.99587E+00_JPRB, 0.10333E+01_JPRB, 0.10331E+01_JPRB, 0.10399E+01_JPRB ,&
&  0.10392E+01_JPRB, 0.99114E+00_JPRB, 0.97086E+00_JPRB, 0.95755E+00_JPRB, 0.96279E+00_JPRB, 0.96099E+00_JPRB, 0.94861E+00_JPRB ,&
&  0.93686E+00_JPRB, 0.92831E+00_JPRB, 0.91294E+00_JPRB, 0.92221E+00_JPRB, 0.93060E+00_JPRB, 0.92227E+00_JPRB, 0.88366E+00_JPRB ,&
&  0.89298E+00_JPRB, 0.90133E+00_JPRB, 0.89397E+00_JPRB, 0.90797E+00_JPRB, 0.91249E+00_JPRB, 0.96268E+00_JPRB, 0.92011E+00_JPRB ,&
&  0.91675E+00_JPRB, 0.90172E+00_JPRB, 0.86528E+00_JPRB, 0.85752E+00_JPRB, 0.86528E+00_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.87122E+00_JPRB, 0.85139E+00_JPRB, 0.83435E+00_JPRB, 0.80865E+00_JPRB, 0.77218E+00_JPRB, 0.75237E+00_JPRB, 0.74039E+00_JPRB ,&
&  0.73506E+00_JPRB, 0.71053E+00_JPRB, 0.69211E+00_JPRB, 0.69163E+00_JPRB, 0.69636E+00_JPRB, 0.68745E+00_JPRB, 0.68395E+00_JPRB ,&
&  0.68889E+00_JPRB, 0.69104E+00_JPRB, 0.68204E+00_JPRB, 0.67328E+00_JPRB, 0.67503E+00_JPRB, 0.70892E+00_JPRB, 0.72316E+00_JPRB ,&
&  0.72882E+00_JPRB, 0.72006E+00_JPRB, 0.69840E+00_JPRB, 0.66994E+00_JPRB, 0.66189E+00_JPRB, 0.66679E+00_JPRB, 0.69852E+00_JPRB ,&
&  0.96845E+00_JPRB, 0.11385E+01_JPRB, 0.18568E+01_JPRB, 0.18520E+01_JPRB, 0.13520E+01_JPRB, 0.11165E+01_JPRB, 0.98967E+00_JPRB ,&
&  0.10113E+01_JPRB, 0.98827E+00_JPRB, 0.93796E+00_JPRB, 0.88779E+00_JPRB, 0.84111E+00_JPRB, 0.80215E+00_JPRB, 0.77648E+00_JPRB ,&
&  0.77463E+00_JPRB, 0.77128E+00_JPRB, 0.76670E+00_JPRB, 0.76854E+00_JPRB, 0.76369E+00_JPRB, 0.76254E+00_JPRB, 0.76207E+00_JPRB ,&
&  0.76357E+00_JPRB, 0.76285E+00_JPRB, 0.78326E+00_JPRB, 0.92754E+00_JPRB, 0.14920E+01_JPRB, 0.23878E+01_JPRB, 0.37411E+01_JPRB ,&
&  0.44914E+01_JPRB, 0.41655E+01_JPRB, 0.29410E+01_JPRB, 0.21466E+01_JPRB, 0.15591E+01_JPRB, 0.12517E+01_JPRB, 0.11664E+01_JPRB ,&
&  0.11318E+01_JPRB, 0.11022E+01_JPRB, 0.10479E+01_JPRB, 0.10368E+01_JPRB, 0.10377E+01_JPRB, 0.10164E+01_JPRB, 0.10105E+01_JPRB ,&
&  0.10011E+01_JPRB, 0.99468E+00_JPRB, 0.98070E+00_JPRB, 0.96401E+00_JPRB, 0.94454E+00_JPRB, 0.92880E+00_JPRB, 0.92035E+00_JPRB ,&
&  0.91412E+00_JPRB, 0.89988E+00_JPRB, 0.89733E+00_JPRB, 0.92170E+00_JPRB, 0.96775E+00_JPRB, 0.97405E+00_JPRB, 0.95466E+00_JPRB ,&
&  0.91744E+00_JPRB, 0.89843E+00_JPRB, 0.88937E+00_JPRB, 0.91262E+00_JPRB, 0.90302E+00_JPRB, 0.98513E+00_JPRB, 0.94425E+00_JPRB ,&
&  0.92908E+00_JPRB, 0.91590E+00_JPRB, 0.88688E+00_JPRB, 0.86712E+00_JPRB, 0.86880E+00_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.88053E+00_JPRB, 0.85505E+00_JPRB, 0.83998E+00_JPRB, 0.82906E+00_JPRB, 0.80603E+00_JPRB, 0.78111E+00_JPRB, 0.77529E+00_JPRB ,&
&  0.77286E+00_JPRB, 0.75486E+00_JPRB, 0.73319E+00_JPRB, 0.70524E+00_JPRB, 0.69817E+00_JPRB, 0.70217E+00_JPRB, 0.70688E+00_JPRB ,&
&  0.70549E+00_JPRB, 0.70598E+00_JPRB, 0.70484E+00_JPRB, 0.70351E+00_JPRB, 0.70252E+00_JPRB, 0.72476E+00_JPRB, 0.73386E+00_JPRB ,&
&  0.72615E+00_JPRB, 0.72746E+00_JPRB, 0.70642E+00_JPRB, 0.68422E+00_JPRB, 0.67357E+00_JPRB, 0.66003E+00_JPRB, 0.65272E+00_JPRB ,&
&  0.82000E+00_JPRB, 0.11795E+01_JPRB, 0.17021E+01_JPRB, 0.16227E+01_JPRB, 0.13343E+01_JPRB, 0.11098E+01_JPRB, 0.10552E+01_JPRB ,&
&  0.10300E+01_JPRB, 0.99533E+00_JPRB, 0.94269E+00_JPRB, 0.89719E+00_JPRB, 0.85660E+00_JPRB, 0.83042E+00_JPRB, 0.80350E+00_JPRB ,&
&  0.78996E+00_JPRB, 0.78560E+00_JPRB, 0.76558E+00_JPRB, 0.75449E+00_JPRB, 0.75120E+00_JPRB, 0.75446E+00_JPRB, 0.75714E+00_JPRB ,&
&  0.76229E+00_JPRB, 0.76491E+00_JPRB, 0.77257E+00_JPRB, 0.82497E+00_JPRB, 0.11654E+01_JPRB, 0.14058E+01_JPRB, 0.25371E+01_JPRB ,&
&  0.36426E+01_JPRB, 0.29584E+01_JPRB, 0.22753E+01_JPRB, 0.18857E+01_JPRB, 0.15591E+01_JPRB, 0.12710E+01_JPRB, 0.11414E+01_JPRB ,&
&  0.10915E+01_JPRB, 0.10674E+01_JPRB, 0.10239E+01_JPRB, 0.10064E+01_JPRB, 0.99033E+00_JPRB, 0.97848E+00_JPRB, 0.96882E+00_JPRB ,&
&  0.94824E+00_JPRB, 0.94627E+00_JPRB, 0.94407E+00_JPRB, 0.94166E+00_JPRB, 0.92535E+00_JPRB, 0.91651E+00_JPRB, 0.90751E+00_JPRB ,&
&  0.90037E+00_JPRB, 0.88152E+00_JPRB, 0.87780E+00_JPRB, 0.90184E+00_JPRB, 0.97463E+00_JPRB, 0.10242E+01_JPRB, 0.10122E+01_JPRB ,&
&  0.95888E+00_JPRB, 0.91809E+00_JPRB, 0.91717E+00_JPRB, 0.91680E+00_JPRB, 0.93057E+00_JPRB, 0.10362E+01_JPRB, 0.98306E+00_JPRB ,&
&  0.95098E+00_JPRB, 0.94053E+00_JPRB, 0.92034E+00_JPRB, 0.89405E+00_JPRB, 0.89253E+00_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.91656E+00_JPRB, 0.88637E+00_JPRB, 0.86372E+00_JPRB, 0.83797E+00_JPRB, 0.82996E+00_JPRB, 0.81368E+00_JPRB, 0.79186E+00_JPRB ,&
&  0.77978E+00_JPRB, 0.77925E+00_JPRB, 0.77304E+00_JPRB, 0.75746E+00_JPRB, 0.74363E+00_JPRB, 0.73832E+00_JPRB, 0.73829E+00_JPRB ,&
&  0.73678E+00_JPRB, 0.73540E+00_JPRB, 0.73290E+00_JPRB, 0.72564E+00_JPRB, 0.72930E+00_JPRB, 0.73884E+00_JPRB, 0.74841E+00_JPRB ,&
&  0.73179E+00_JPRB, 0.71834E+00_JPRB, 0.70660E+00_JPRB, 0.69474E+00_JPRB, 0.69145E+00_JPRB, 0.67316E+00_JPRB, 0.64854E+00_JPRB ,&
&  0.70975E+00_JPRB, 0.89619E+00_JPRB, 0.12198E+01_JPRB, 0.14892E+01_JPRB, 0.13358E+01_JPRB, 0.11715E+01_JPRB, 0.10999E+01_JPRB ,&
&  0.10519E+01_JPRB, 0.98591E+00_JPRB, 0.93721E+00_JPRB, 0.89903E+00_JPRB, 0.88184E+00_JPRB, 0.86037E+00_JPRB, 0.81763E+00_JPRB ,&
&  0.78675E+00_JPRB, 0.76934E+00_JPRB, 0.76170E+00_JPRB, 0.76096E+00_JPRB, 0.75481E+00_JPRB, 0.76693E+00_JPRB, 0.77626E+00_JPRB ,&
&  0.78257E+00_JPRB, 0.78036E+00_JPRB, 0.77517E+00_JPRB, 0.78249E+00_JPRB, 0.93452E+00_JPRB, 0.10705E+01_JPRB, 0.14530E+01_JPRB ,&
&  0.16407E+01_JPRB, 0.17249E+01_JPRB, 0.16527E+01_JPRB, 0.15829E+01_JPRB, 0.14702E+01_JPRB, 0.12740E+01_JPRB, 0.11262E+01_JPRB ,&
&  0.10435E+01_JPRB, 0.10117E+01_JPRB, 0.97483E+00_JPRB, 0.96071E+00_JPRB, 0.94742E+00_JPRB, 0.93209E+00_JPRB, 0.92621E+00_JPRB ,&
&  0.91784E+00_JPRB, 0.91051E+00_JPRB, 0.90417E+00_JPRB, 0.91163E+00_JPRB, 0.90473E+00_JPRB, 0.88496E+00_JPRB, 0.87030E+00_JPRB ,&
&  0.86609E+00_JPRB, 0.85433E+00_JPRB, 0.86466E+00_JPRB, 0.90073E+00_JPRB, 0.93433E+00_JPRB, 0.95282E+00_JPRB, 0.97199E+00_JPRB ,&
&  0.98759E+00_JPRB, 0.97876E+00_JPRB, 0.95440E+00_JPRB, 0.95792E+00_JPRB, 0.96781E+00_JPRB, 0.10558E+01_JPRB, 0.10106E+01_JPRB ,&
&  0.98242E+00_JPRB, 0.96443E+00_JPRB, 0.94652E+00_JPRB, 0.90688E+00_JPRB, 0.93777E+00_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.92601E+00_JPRB, 0.91604E+00_JPRB, 0.90082E+00_JPRB, 0.87127E+00_JPRB, 0.85766E+00_JPRB, 0.84295E+00_JPRB, 0.82647E+00_JPRB ,&
&  0.81835E+00_JPRB, 0.81084E+00_JPRB, 0.80545E+00_JPRB, 0.79252E+00_JPRB, 0.77957E+00_JPRB, 0.77196E+00_JPRB, 0.76205E+00_JPRB ,&
&  0.76304E+00_JPRB, 0.76377E+00_JPRB, 0.75621E+00_JPRB, 0.74842E+00_JPRB, 0.74562E+00_JPRB, 0.74347E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.73948E+00_JPRB, 0.72926E+00_JPRB, 0.72839E+00_JPRB, 0.72071E+00_JPRB, 0.70832E+00_JPRB, 0.69771E+00_JPRB, 0.66713E+00_JPRB ,&
&  0.66067E+00_JPRB, 0.67540E+00_JPRB, 0.91058E+00_JPRB, 0.11772E+01_JPRB, 0.11760E+01_JPRB, 0.11638E+01_JPRB, 0.11032E+01_JPRB ,&
&  0.10492E+01_JPRB, 0.98856E+00_JPRB, 0.92015E+00_JPRB, 0.87627E+00_JPRB, 0.86647E+00_JPRB, 0.85330E+00_JPRB, 0.83471E+00_JPRB ,&
&  0.81103E+00_JPRB, 0.78905E+00_JPRB, 0.77387E+00_JPRB, 0.77888E+00_JPRB, 0.77560E+00_JPRB, 0.78019E+00_JPRB, 0.78968E+00_JPRB ,&
&  0.79221E+00_JPRB, 0.79217E+00_JPRB, 0.78072E+00_JPRB, 0.77462E+00_JPRB, 0.77544E+00_JPRB, 0.79802E+00_JPRB, 0.90211E+00_JPRB ,&
&  0.10496E+01_JPRB, 0.11183E+01_JPRB, 0.11581E+01_JPRB, 0.12005E+01_JPRB, 0.12250E+01_JPRB, 0.11966E+01_JPRB, 0.11164E+01_JPRB ,&
&  0.10491E+01_JPRB, 0.10082E+01_JPRB, 0.96104E+00_JPRB, 0.96265E+00_JPRB, 0.94613E+00_JPRB, 0.92350E+00_JPRB, 0.90372E+00_JPRB ,&
&  0.88200E+00_JPRB, 0.87904E+00_JPRB, 0.86605E+00_JPRB, 0.86059E+00_JPRB, 0.85593E+00_JPRB, 0.84550E+00_JPRB, 0.83256E+00_JPRB ,&
&  0.83099E+00_JPRB, 0.82665E+00_JPRB, 0.83428E+00_JPRB, 0.86078E+00_JPRB, 0.89358E+00_JPRB, 0.91361E+00_JPRB, 0.93443E+00_JPRB ,&
&  0.95236E+00_JPRB, 0.97603E+00_JPRB, 0.94622E+00_JPRB, 0.94559E+00_JPRB, 0.10460E+01_JPRB, 0.10469E+01_JPRB, 0.10100E+01_JPRB ,&
&  0.98553E+00_JPRB, 0.96228E+00_JPRB, 0.94857E+00_JPRB, 0.90034E+00_JPRB, 0.89899E+00_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.82862E+00_JPRB, 0.85853E+00_JPRB, 0.85691E+00_JPRB, 0.83519E+00_JPRB, 0.82329E+00_JPRB, 0.81882E+00_JPRB, 0.81817E+00_JPRB ,&
&  0.81483E+00_JPRB, 0.80517E+00_JPRB, 0.81057E+00_JPRB, 0.81789E+00_JPRB, 0.81676E+00_JPRB, 0.80662E+00_JPRB, 0.79950E+00_JPRB ,&
&  0.79807E+00_JPRB, 0.80224E+00_JPRB, 0.80753E+00_JPRB, 0.79780E+00_JPRB, 0.77750E+00_JPRB, 0.76421E+00_JPRB, 0.75233E+00_JPRB ,&
&  0.75056E+00_JPRB, 0.75015E+00_JPRB, 0.74776E+00_JPRB, 0.74914E+00_JPRB, 0.72876E+00_JPRB, 0.70615E+00_JPRB, 0.68735E+00_JPRB ,&
&  0.67036E+00_JPRB, 0.58827E+00_JPRB, 0.70220E+00_JPRB, 0.89384E+00_JPRB, 0.99365E+00_JPRB, 0.10044E+01_JPRB, 0.96693E+00_JPRB ,&
&  0.94274E+00_JPRB, 0.91074E+00_JPRB, 0.87392E+00_JPRB, 0.83815E+00_JPRB, 0.81915E+00_JPRB, 0.80838E+00_JPRB, 0.80132E+00_JPRB ,&
&  0.79435E+00_JPRB, 0.78619E+00_JPRB, 0.78156E+00_JPRB, 0.76625E+00_JPRB, 0.75390E+00_JPRB, 0.75332E+00_JPRB, 0.75715E+00_JPRB ,&
&  0.76412E+00_JPRB, 0.76254E+00_JPRB, 0.76474E+00_JPRB, 0.76772E+00_JPRB, 0.75164E+00_JPRB, 0.75341E+00_JPRB, 0.76930E+00_JPRB ,&
&  0.81470E+00_JPRB, 0.85570E+00_JPRB, 0.87984E+00_JPRB, 0.94088E+00_JPRB, 0.98803E+00_JPRB, 0.10274E+01_JPRB, 0.10260E+01_JPRB ,&
&  0.10011E+01_JPRB, 0.98110E+00_JPRB, 0.95806E+00_JPRB, 0.94790E+00_JPRB, 0.93197E+00_JPRB, 0.89983E+00_JPRB, 0.86793E+00_JPRB ,&
&  0.85451E+00_JPRB, 0.84702E+00_JPRB, 0.83061E+00_JPRB, 0.82103E+00_JPRB, 0.81155E+00_JPRB, 0.79538E+00_JPRB, 0.78207E+00_JPRB ,&
&  0.77329E+00_JPRB, 0.77277E+00_JPRB, 0.77640E+00_JPRB, 0.79158E+00_JPRB, 0.81571E+00_JPRB, 0.84398E+00_JPRB, 0.87232E+00_JPRB ,&
&  0.89865E+00_JPRB, 0.91763E+00_JPRB, 0.94375E+00_JPRB, 0.99268E+00_JPRB, 0.99332E+00_JPRB, 0.98652E+00_JPRB, 0.99183E+00_JPRB ,&
&  0.97895E+00_JPRB, 0.94710E+00_JPRB, 0.91298E+00_JPRB, 0.85595E+00_JPRB, 0.77691E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.80151E+00_JPRB, 0.81885E+00_JPRB, 0.81932E+00_JPRB, 0.81168E+00_JPRB, 0.80561E+00_JPRB, 0.80247E+00_JPRB, 0.80694E+00_JPRB ,&
&  0.80584E+00_JPRB, 0.80041E+00_JPRB, 0.80383E+00_JPRB, 0.80709E+00_JPRB, 0.81028E+00_JPRB, 0.80909E+00_JPRB, 0.81516E+00_JPRB ,&
&  0.81702E+00_JPRB, 0.81948E+00_JPRB, 0.82338E+00_JPRB, 0.81773E+00_JPRB, 0.79855E+00_JPRB, 0.78495E+00_JPRB, 0.77104E+00_JPRB ,&
&  0.76967E+00_JPRB, 0.76901E+00_JPRB, 0.76936E+00_JPRB, 0.76301E+00_JPRB, 0.74357E+00_JPRB, 0.72707E+00_JPRB, 0.71668E+00_JPRB ,&
&  0.71601E+00_JPRB, 0.57015E+00_JPRB, 0.62421E+00_JPRB, 0.79698E+00_JPRB, 0.87652E+00_JPRB, 0.86523E+00_JPRB, 0.86373E+00_JPRB ,&
&  0.87471E+00_JPRB, 0.87303E+00_JPRB, 0.85304E+00_JPRB, 0.82313E+00_JPRB, 0.78547E+00_JPRB, 0.77536E+00_JPRB, 0.75888E+00_JPRB ,&
&  0.74173E+00_JPRB, 0.73533E+00_JPRB, 0.73007E+00_JPRB, 0.73074E+00_JPRB, 0.72252E+00_JPRB, 0.71258E+00_JPRB, 0.71511E+00_JPRB ,&
&  0.72509E+00_JPRB, 0.72728E+00_JPRB, 0.73411E+00_JPRB, 0.74114E+00_JPRB, 0.73830E+00_JPRB, 0.75565E+00_JPRB, 0.76684E+00_JPRB ,&
&  0.78085E+00_JPRB, 0.79161E+00_JPRB, 0.80000E+00_JPRB, 0.81307E+00_JPRB, 0.83993E+00_JPRB, 0.89218E+00_JPRB, 0.92504E+00_JPRB ,&
&  0.92104E+00_JPRB, 0.90752E+00_JPRB, 0.90809E+00_JPRB, 0.90568E+00_JPRB, 0.89414E+00_JPRB, 0.87205E+00_JPRB, 0.84763E+00_JPRB ,&
&  0.83314E+00_JPRB, 0.82012E+00_JPRB, 0.80858E+00_JPRB, 0.79421E+00_JPRB, 0.77828E+00_JPRB, 0.76422E+00_JPRB, 0.75361E+00_JPRB ,&
&  0.74693E+00_JPRB, 0.74446E+00_JPRB, 0.75188E+00_JPRB, 0.76929E+00_JPRB, 0.79174E+00_JPRB, 0.81733E+00_JPRB, 0.84410E+00_JPRB ,&
&  0.87542E+00_JPRB, 0.90933E+00_JPRB, 0.94322E+00_JPRB, 0.90426E+00_JPRB, 0.96369E+00_JPRB, 0.97490E+00_JPRB, 0.98071E+00_JPRB ,&
&  0.97059E+00_JPRB, 0.93245E+00_JPRB, 0.87815E+00_JPRB, 0.68892E+00_JPRB, 0.76480E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.79777E+00_JPRB, 0.80330E+00_JPRB, 0.80654E+00_JPRB, 0.79576E+00_JPRB, 0.78233E+00_JPRB, 0.77805E+00_JPRB, 0.78573E+00_JPRB ,&
&  0.78234E+00_JPRB, 0.76954E+00_JPRB, 0.77466E+00_JPRB, 0.77883E+00_JPRB, 0.78655E+00_JPRB, 0.79072E+00_JPRB, 0.79550E+00_JPRB ,&
&  0.80043E+00_JPRB, 0.80504E+00_JPRB, 0.80375E+00_JPRB, 0.81244E+00_JPRB, 0.80524E+00_JPRB, 0.79973E+00_JPRB, 0.79677E+00_JPRB ,&
&  0.78814E+00_JPRB, 0.77905E+00_JPRB, 0.77159E+00_JPRB, 0.76144E+00_JPRB, 0.75410E+00_JPRB, 0.74567E+00_JPRB, 0.73068E+00_JPRB ,&
&  0.74222E+00_JPRB, 0.58767E+00_JPRB, 0.65818E+00_JPRB, 0.77996E+00_JPRB, 0.78545E+00_JPRB, 0.78547E+00_JPRB, 0.79942E+00_JPRB ,&
&  0.81934E+00_JPRB, 0.83032E+00_JPRB, 0.82134E+00_JPRB, 0.79741E+00_JPRB, 0.75445E+00_JPRB, 0.73240E+00_JPRB, 0.71892E+00_JPRB ,&
&  0.70406E+00_JPRB, 0.69352E+00_JPRB, 0.68744E+00_JPRB, 0.68935E+00_JPRB, 0.68951E+00_JPRB, 0.68654E+00_JPRB, 0.68409E+00_JPRB ,&
&  0.68863E+00_JPRB, 0.69949E+00_JPRB, 0.70743E+00_JPRB, 0.71554E+00_JPRB, 0.72307E+00_JPRB, 0.72116E+00_JPRB, 0.72244E+00_JPRB ,&
&  0.72470E+00_JPRB, 0.72927E+00_JPRB, 0.73155E+00_JPRB, 0.73810E+00_JPRB, 0.74976E+00_JPRB, 0.78349E+00_JPRB, 0.81312E+00_JPRB ,&
&  0.82205E+00_JPRB, 0.82287E+00_JPRB, 0.82537E+00_JPRB, 0.82834E+00_JPRB, 0.83170E+00_JPRB, 0.83105E+00_JPRB, 0.82086E+00_JPRB ,&
&  0.80890E+00_JPRB, 0.79316E+00_JPRB, 0.78391E+00_JPRB, 0.77667E+00_JPRB, 0.76678E+00_JPRB, 0.76058E+00_JPRB, 0.76035E+00_JPRB ,&
&  0.75332E+00_JPRB, 0.74165E+00_JPRB, 0.74442E+00_JPRB, 0.75905E+00_JPRB, 0.77216E+00_JPRB, 0.78110E+00_JPRB, 0.79888E+00_JPRB ,&
&  0.83198E+00_JPRB, 0.86756E+00_JPRB, 0.89662E+00_JPRB, 0.92829E+00_JPRB, 0.94462E+00_JPRB, 0.95115E+00_JPRB, 0.94656E+00_JPRB ,&
&  0.92915E+00_JPRB, 0.90808E+00_JPRB, 0.66106E+00_JPRB, 0.76109E+00_JPRB, 0.78644E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.81222E+00_JPRB, 0.80240E+00_JPRB, 0.79029E+00_JPRB, 0.77369E+00_JPRB, 0.75817E+00_JPRB, 0.75014E+00_JPRB, 0.75682E+00_JPRB ,&
&  0.75638E+00_JPRB, 0.74505E+00_JPRB, 0.75505E+00_JPRB, 0.76836E+00_JPRB, 0.77929E+00_JPRB, 0.78950E+00_JPRB, 0.79356E+00_JPRB ,&
&  0.79815E+00_JPRB, 0.79605E+00_JPRB, 0.79985E+00_JPRB, 0.81223E+00_JPRB, 0.83073E+00_JPRB, 0.83218E+00_JPRB, 0.82904E+00_JPRB ,&
&  0.80640E+00_JPRB, 0.79495E+00_JPRB, 0.78449E+00_JPRB, 0.77188E+00_JPRB, 0.76066E+00_JPRB, 0.75708E+00_JPRB, 0.76022E+00_JPRB ,&
&  0.64682E+00_JPRB, 0.55005E+00_JPRB, 0.65386E+00_JPRB, 0.74282E+00_JPRB, 0.74881E+00_JPRB, 0.75233E+00_JPRB, 0.76979E+00_JPRB ,&
&  0.77598E+00_JPRB, 0.77786E+00_JPRB, 0.77525E+00_JPRB, 0.76198E+00_JPRB, 0.73015E+00_JPRB, 0.69692E+00_JPRB, 0.67830E+00_JPRB ,&
&  0.66693E+00_JPRB, 0.67162E+00_JPRB, 0.67349E+00_JPRB, 0.68082E+00_JPRB, 0.68115E+00_JPRB, 0.67434E+00_JPRB, 0.67332E+00_JPRB ,&
&  0.67473E+00_JPRB, 0.68548E+00_JPRB, 0.69596E+00_JPRB, 0.70403E+00_JPRB, 0.70765E+00_JPRB, 0.70914E+00_JPRB, 0.70700E+00_JPRB ,&
&  0.70605E+00_JPRB, 0.70592E+00_JPRB, 0.70425E+00_JPRB, 0.70835E+00_JPRB, 0.71620E+00_JPRB, 0.73189E+00_JPRB, 0.75547E+00_JPRB ,&
&  0.76940E+00_JPRB, 0.77383E+00_JPRB, 0.77628E+00_JPRB, 0.77610E+00_JPRB, 0.78512E+00_JPRB, 0.79096E+00_JPRB, 0.78418E+00_JPRB ,&
&  0.77412E+00_JPRB, 0.76654E+00_JPRB, 0.76853E+00_JPRB, 0.76935E+00_JPRB, 0.77199E+00_JPRB, 0.77180E+00_JPRB, 0.76562E+00_JPRB ,&
&  0.75655E+00_JPRB, 0.74285E+00_JPRB, 0.73295E+00_JPRB, 0.73310E+00_JPRB, 0.74370E+00_JPRB, 0.75221E+00_JPRB, 0.77022E+00_JPRB ,&
&  0.80230E+00_JPRB, 0.84096E+00_JPRB, 0.87331E+00_JPRB, 0.90907E+00_JPRB, 0.93005E+00_JPRB, 0.93963E+00_JPRB, 0.93239E+00_JPRB ,&
&  0.91321E+00_JPRB, 0.87420E+00_JPRB, 0.86010E+00_JPRB, 0.83585E+00_JPRB, 0.82313E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.77949E+00_JPRB, 0.77235E+00_JPRB, 0.75792E+00_JPRB, 0.73220E+00_JPRB, 0.69921E+00_JPRB, 0.70811E+00_JPRB, 0.72060E+00_JPRB ,&
&  0.71479E+00_JPRB, 0.70574E+00_JPRB, 0.71641E+00_JPRB, 0.74161E+00_JPRB, 0.76642E+00_JPRB, 0.78079E+00_JPRB, 0.78348E+00_JPRB ,&
&  0.77894E+00_JPRB, 0.78136E+00_JPRB, 0.78991E+00_JPRB, 0.80709E+00_JPRB, 0.80613E+00_JPRB, 0.78807E+00_JPRB, 0.77685E+00_JPRB ,&
&  0.77354E+00_JPRB, 0.77160E+00_JPRB, 0.76890E+00_JPRB, 0.75215E+00_JPRB, 0.74113E+00_JPRB, 0.73839E+00_JPRB, 0.73920E+00_JPRB ,&
&  0.73480E+00_JPRB, 0.59535E+00_JPRB, 0.69534E+00_JPRB, 0.70626E+00_JPRB, 0.71781E+00_JPRB, 0.71442E+00_JPRB, 0.71590E+00_JPRB ,&
&  0.72115E+00_JPRB, 0.73687E+00_JPRB, 0.72924E+00_JPRB, 0.70334E+00_JPRB, 0.67619E+00_JPRB, 0.64812E+00_JPRB, 0.62917E+00_JPRB ,&
&  0.61983E+00_JPRB, 0.62272E+00_JPRB, 0.63268E+00_JPRB, 0.64308E+00_JPRB, 0.65425E+00_JPRB, 0.65612E+00_JPRB, 0.66021E+00_JPRB ,&
&  0.66323E+00_JPRB, 0.66886E+00_JPRB, 0.67195E+00_JPRB, 0.67832E+00_JPRB, 0.68121E+00_JPRB, 0.67852E+00_JPRB, 0.67159E+00_JPRB ,&
&  0.66172E+00_JPRB, 0.65633E+00_JPRB, 0.65675E+00_JPRB, 0.66016E+00_JPRB, 0.66960E+00_JPRB, 0.68639E+00_JPRB, 0.70457E+00_JPRB ,&
&  0.72399E+00_JPRB, 0.73661E+00_JPRB, 0.74353E+00_JPRB, 0.74575E+00_JPRB, 0.75369E+00_JPRB, 0.76204E+00_JPRB, 0.76005E+00_JPRB ,&
&  0.75115E+00_JPRB, 0.74143E+00_JPRB, 0.73706E+00_JPRB, 0.73180E+00_JPRB, 0.73058E+00_JPRB, 0.72993E+00_JPRB, 0.72507E+00_JPRB ,&
&  0.71707E+00_JPRB, 0.71573E+00_JPRB, 0.71752E+00_JPRB, 0.70741E+00_JPRB, 0.70975E+00_JPRB, 0.72065E+00_JPRB, 0.74012E+00_JPRB ,&
&  0.76414E+00_JPRB, 0.79555E+00_JPRB, 0.82652E+00_JPRB, 0.85221E+00_JPRB, 0.86703E+00_JPRB, 0.87105E+00_JPRB, 0.86446E+00_JPRB ,&
&  0.84662E+00_JPRB, 0.82204E+00_JPRB, 0.80371E+00_JPRB, 0.79372E+00_JPRB, 0.79449E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.74264E+00_JPRB, 0.74291E+00_JPRB, 0.73253E+00_JPRB, 0.69473E+00_JPRB, 0.66622E+00_JPRB, 0.67608E+00_JPRB, 0.68515E+00_JPRB ,&
&  0.68459E+00_JPRB, 0.69941E+00_JPRB, 0.72456E+00_JPRB, 0.75014E+00_JPRB, 0.76726E+00_JPRB, 0.76727E+00_JPRB, 0.77149E+00_JPRB ,&
&  0.76921E+00_JPRB, 0.76723E+00_JPRB, 0.77723E+00_JPRB, 0.78212E+00_JPRB, 0.77142E+00_JPRB, 0.75033E+00_JPRB, 0.74770E+00_JPRB ,&
&  0.75360E+00_JPRB, 0.74934E+00_JPRB, 0.73115E+00_JPRB, 0.71885E+00_JPRB, 0.70895E+00_JPRB, 0.71422E+00_JPRB, 0.72085E+00_JPRB ,&
&  0.71482E+00_JPRB, 0.66216E+00_JPRB, 0.69989E+00_JPRB, 0.71127E+00_JPRB, 0.71906E+00_JPRB, 0.70856E+00_JPRB, 0.71348E+00_JPRB ,&
&  0.72479E+00_JPRB, 0.73170E+00_JPRB, 0.71150E+00_JPRB, 0.68788E+00_JPRB, 0.66678E+00_JPRB, 0.64942E+00_JPRB, 0.62166E+00_JPRB ,&
&  0.61113E+00_JPRB, 0.60803E+00_JPRB, 0.62128E+00_JPRB, 0.63536E+00_JPRB, 0.65166E+00_JPRB, 0.65829E+00_JPRB, 0.66868E+00_JPRB ,&
&  0.67100E+00_JPRB, 0.66569E+00_JPRB, 0.66089E+00_JPRB, 0.65740E+00_JPRB, 0.65705E+00_JPRB, 0.65146E+00_JPRB, 0.64321E+00_JPRB ,&
&  0.63685E+00_JPRB, 0.63567E+00_JPRB, 0.64072E+00_JPRB, 0.65044E+00_JPRB, 0.66295E+00_JPRB, 0.68343E+00_JPRB, 0.70000E+00_JPRB ,&
&  0.71992E+00_JPRB, 0.72707E+00_JPRB, 0.73296E+00_JPRB, 0.74592E+00_JPRB, 0.75365E+00_JPRB, 0.75139E+00_JPRB, 0.74780E+00_JPRB ,&
&  0.74355E+00_JPRB, 0.73644E+00_JPRB, 0.73978E+00_JPRB, 0.73767E+00_JPRB, 0.73722E+00_JPRB, 0.72818E+00_JPRB, 0.71277E+00_JPRB ,&
&  0.70703E+00_JPRB, 0.70287E+00_JPRB, 0.69468E+00_JPRB, 0.69224E+00_JPRB, 0.69029E+00_JPRB, 0.70848E+00_JPRB, 0.74022E+00_JPRB ,&
&  0.75737E+00_JPRB, 0.78021E+00_JPRB, 0.81292E+00_JPRB, 0.83400E+00_JPRB, 0.83267E+00_JPRB, 0.83029E+00_JPRB, 0.81744E+00_JPRB ,&
&  0.79759E+00_JPRB, 0.78376E+00_JPRB, 0.77132E+00_JPRB, 0.76684E+00_JPRB, 0.76296E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.70253E+00_JPRB, 0.72388E+00_JPRB, 0.69127E+00_JPRB, 0.67376E+00_JPRB, 0.65372E+00_JPRB, 0.64892E+00_JPRB, 0.63976E+00_JPRB ,&
&  0.65017E+00_JPRB, 0.66708E+00_JPRB, 0.69078E+00_JPRB, 0.69874E+00_JPRB, 0.70561E+00_JPRB, 0.70846E+00_JPRB, 0.72552E+00_JPRB ,&
&  0.72104E+00_JPRB, 0.71291E+00_JPRB, 0.71612E+00_JPRB, 0.70585E+00_JPRB, 0.69707E+00_JPRB, 0.70178E+00_JPRB, 0.71040E+00_JPRB ,&
&  0.72026E+00_JPRB, 0.71150E+00_JPRB, 0.69425E+00_JPRB, 0.66781E+00_JPRB, 0.65535E+00_JPRB, 0.65276E+00_JPRB, 0.66023E+00_JPRB ,&
&  0.66530E+00_JPRB, 0.68810E+00_JPRB, 0.62583E+00_JPRB, 0.68148E+00_JPRB, 0.69028E+00_JPRB, 0.68915E+00_JPRB, 0.68775E+00_JPRB ,&
&  0.69242E+00_JPRB, 0.68201E+00_JPRB, 0.67307E+00_JPRB, 0.66430E+00_JPRB, 0.64181E+00_JPRB, 0.62367E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.58712E+00_JPRB, 0.57661E+00_JPRB, 0.58168E+00_JPRB, 0.59908E+00_JPRB, 0.61697E+00_JPRB, 0.62695E+00_JPRB, 0.63307E+00_JPRB ,&
&  0.63630E+00_JPRB, 0.62896E+00_JPRB, 0.61905E+00_JPRB, 0.61925E+00_JPRB, 0.62588E+00_JPRB, 0.63095E+00_JPRB, 0.63443E+00_JPRB ,&
&  0.63520E+00_JPRB, 0.61271E+00_JPRB, 0.60034E+00_JPRB, 0.61688E+00_JPRB, 0.63493E+00_JPRB, 0.65521E+00_JPRB, 0.68170E+00_JPRB ,&
&  0.70206E+00_JPRB, 0.71021E+00_JPRB, 0.70992E+00_JPRB, 0.71648E+00_JPRB, 0.72271E+00_JPRB, 0.73069E+00_JPRB, 0.73704E+00_JPRB ,&
&  0.72607E+00_JPRB, 0.72322E+00_JPRB, 0.72383E+00_JPRB, 0.72361E+00_JPRB, 0.71439E+00_JPRB, 0.69120E+00_JPRB, 0.65894E+00_JPRB ,&
&  0.64419E+00_JPRB, 0.62930E+00_JPRB, 0.61835E+00_JPRB, 0.61545E+00_JPRB, 0.61753E+00_JPRB, 0.64373E+00_JPRB, 0.68131E+00_JPRB ,&
&  0.71859E+00_JPRB, 0.74858E+00_JPRB, 0.77430E+00_JPRB, 0.79778E+00_JPRB, 0.79562E+00_JPRB, 0.77307E+00_JPRB, 0.76265E+00_JPRB ,&
&  0.74692E+00_JPRB, 0.73044E+00_JPRB, 0.71933E+00_JPRB, 0.71046E+00_JPRB, 0.70091E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.70615E+00_JPRB, 0.70749E+00_JPRB, 0.69282E+00_JPRB, 0.68264E+00_JPRB, 0.66623E+00_JPRB, 0.64460E+00_JPRB, 0.62556E+00_JPRB ,&
&  0.62698E+00_JPRB, 0.64206E+00_JPRB, 0.65664E+00_JPRB, 0.66725E+00_JPRB, 0.68270E+00_JPRB, 0.68585E+00_JPRB, 0.69241E+00_JPRB ,&
&  0.68835E+00_JPRB, 0.65140E+00_JPRB, 0.64561E+00_JPRB, 0.65331E+00_JPRB, 0.65516E+00_JPRB, 0.66105E+00_JPRB, 0.67164E+00_JPRB ,&
&  0.68138E+00_JPRB, 0.67657E+00_JPRB, 0.66844E+00_JPRB, 0.63899E+00_JPRB, 0.63233E+00_JPRB, 0.63723E+00_JPRB, 0.64208E+00_JPRB ,&
&  0.65209E+00_JPRB, 0.66809E+00_JPRB, 0.67182E+00_JPRB, 0.67966E+00_JPRB, 0.68872E+00_JPRB, 0.69308E+00_JPRB, 0.68973E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.63817E+00_JPRB, 0.62754E+00_JPRB, 0.61022E+00_JPRB, 0.58867E+00_JPRB, 0.57352E+00_JPRB, 0.56567E+00_JPRB ,&
&  0.56127E+00_JPRB, 0.56621E+00_JPRB, 0.57236E+00_JPRB, 0.58628E+00_JPRB, 0.59149E+00_JPRB, 0.59559E+00_JPRB, 0.60076E+00_JPRB ,&
&  0.60012E+00_JPRB, 0.59583E+00_JPRB, 0.59586E+00_JPRB, 0.60574E+00_JPRB, 0.61185E+00_JPRB, 0.60978E+00_JPRB, 0.60559E+00_JPRB ,&
&  0.59323E+00_JPRB, 0.58015E+00_JPRB, 0.58797E+00_JPRB, 0.60322E+00_JPRB, 0.61045E+00_JPRB, 0.63643E+00_JPRB, 0.67006E+00_JPRB ,&
&  0.69868E+00_JPRB, 0.71718E+00_JPRB, 0.71702E+00_JPRB, 0.71304E+00_JPRB, 0.71086E+00_JPRB, 0.71782E+00_JPRB, 0.72984E+00_JPRB ,&
&  0.71379E+00_JPRB, 0.70456E+00_JPRB, 0.70258E+00_JPRB, 0.68339E+00_JPRB, 0.64700E+00_JPRB, 0.61555E+00_JPRB, 0.59241E+00_JPRB ,&
&  0.57397E+00_JPRB, 0.56350E+00_JPRB, 0.56712E+00_JPRB, 0.57062E+00_JPRB, 0.58343E+00_JPRB, 0.61915E+00_JPRB, 0.65647E+00_JPRB ,&
&  0.69699E+00_JPRB, 0.71772E+00_JPRB, 0.72420E+00_JPRB, 0.73203E+00_JPRB, 0.73732E+00_JPRB, 0.72579E+00_JPRB, 0.71615E+00_JPRB ,&
&  0.71306E+00_JPRB, 0.70456E+00_JPRB, 0.69519E+00_JPRB, 0.68557E+00_JPRB, 0.69297E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.70140E+00_JPRB, 0.69393E+00_JPRB, 0.68051E+00_JPRB, 0.65459E+00_JPRB, 0.63724E+00_JPRB, 0.61012E+00_JPRB, 0.59471E+00_JPRB ,&
&  0.59690E+00_JPRB, 0.59814E+00_JPRB, 0.60320E+00_JPRB, 0.61246E+00_JPRB, 0.61035E+00_JPRB, 0.60566E+00_JPRB, 0.60111E+00_JPRB ,&
&  0.60424E+00_JPRB, 0.60473E+00_JPRB, 0.61321E+00_JPRB, 0.62875E+00_JPRB, 0.64516E+00_JPRB, 0.65442E+00_JPRB, 0.66410E+00_JPRB ,&
&  0.67148E+00_JPRB, 0.66525E+00_JPRB, 0.66091E+00_JPRB, 0.63067E+00_JPRB, 0.61310E+00_JPRB, 0.61829E+00_JPRB, 0.62563E+00_JPRB ,&
&  0.63555E+00_JPRB, 0.64580E+00_JPRB, 0.65582E+00_JPRB, 0.66122E+00_JPRB, 0.64727E+00_JPRB, 0.64356E+00_JPRB, 0.63284E+00_JPRB ,&
&  0.59027E+00_JPRB, 0.54863E+00_JPRB, 0.52614E+00_JPRB, 0.51885E+00_JPRB, 0.51412E+00_JPRB, 0.52019E+00_JPRB, 0.52753E+00_JPRB ,&
&  0.54036E+00_JPRB, 0.55213E+00_JPRB, 0.55514E+00_JPRB, 0.55836E+00_JPRB, 0.55985E+00_JPRB, 0.55891E+00_JPRB, 0.55913E+00_JPRB ,&
&  0.55617E+00_JPRB, 0.55435E+00_JPRB, 0.56162E+00_JPRB, 0.56236E+00_JPRB, 0.55379E+00_JPRB, 0.55150E+00_JPRB, 0.54986E+00_JPRB ,&
&  0.54792E+00_JPRB, 0.54600E+00_JPRB, 0.54318E+00_JPRB, 0.55336E+00_JPRB, 0.57889E+00_JPRB, 0.60835E+00_JPRB, 0.63396E+00_JPRB ,&
&  0.64869E+00_JPRB, 0.66065E+00_JPRB, 0.66371E+00_JPRB, 0.66555E+00_JPRB, 0.66493E+00_JPRB, 0.65612E+00_JPRB, 0.65280E+00_JPRB ,&
&  0.64510E+00_JPRB, 0.62609E+00_JPRB, 0.60059E+00_JPRB, 0.56088E+00_JPRB, 0.52216E+00_JPRB, 0.49987E+00_JPRB, 0.49918E+00_JPRB ,&
&  0.51120E+00_JPRB, 0.52356E+00_JPRB, 0.52991E+00_JPRB, 0.53796E+00_JPRB, 0.55271E+00_JPRB, 0.57044E+00_JPRB, 0.58909E+00_JPRB ,&
&  0.60076E+00_JPRB, 0.60070E+00_JPRB, 0.59805E+00_JPRB, 0.60563E+00_JPRB, 0.61601E+00_JPRB, 0.63332E+00_JPRB, 0.65244E+00_JPRB ,&
&  0.66929E+00_JPRB, 0.66887E+00_JPRB, 0.66933E+00_JPRB, 0.67068E+00_JPRB, 0.69058E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.69561E+00_JPRB, 0.69447E+00_JPRB, 0.66644E+00_JPRB, 0.63513E+00_JPRB, 0.61639E+00_JPRB, 0.59547E+00_JPRB, 0.57854E+00_JPRB ,&
&  0.57696E+00_JPRB, 0.57417E+00_JPRB, 0.57190E+00_JPRB, 0.56913E+00_JPRB, 0.56715E+00_JPRB, 0.56388E+00_JPRB, 0.55171E+00_JPRB ,&
&  0.54730E+00_JPRB, 0.55389E+00_JPRB, 0.57628E+00_JPRB, 0.60156E+00_JPRB, 0.62192E+00_JPRB, 0.64835E+00_JPRB, 0.65951E+00_JPRB ,&
&  0.67117E+00_JPRB, 0.68359E+00_JPRB, 0.68591E+00_JPRB, 0.65335E+00_JPRB, 0.62654E+00_JPRB, 0.62772E+00_JPRB, 0.63880E+00_JPRB ,&
&  0.63910E+00_JPRB, 0.64211E+00_JPRB, 0.64557E+00_JPRB, 0.64167E+00_JPRB, 0.62066E+00_JPRB, 0.58239E+00_JPRB, 0.55230E+00_JPRB ,&
&  0.52751E+00_JPRB, 0.50832E+00_JPRB, 0.50255E+00_JPRB, 0.49140E+00_JPRB, 0.48995E+00_JPRB, 0.50006E+00_JPRB, 0.51317E+00_JPRB ,&
&  0.52670E+00_JPRB, 0.53272E+00_JPRB, 0.53965E+00_JPRB, 0.53620E+00_JPRB, 0.53181E+00_JPRB, 0.53442E+00_JPRB, 0.53414E+00_JPRB ,&
&  0.53335E+00_JPRB, 0.52421E+00_JPRB, 0.52162E+00_JPRB, 0.51481E+00_JPRB, 0.51102E+00_JPRB, 0.51169E+00_JPRB, 0.51496E+00_JPRB ,&
&  0.51648E+00_JPRB, 0.52226E+00_JPRB, 0.52410E+00_JPRB, 0.52636E+00_JPRB, 0.53581E+00_JPRB, 0.55234E+00_JPRB, 0.56894E+00_JPRB ,&
&  0.58137E+00_JPRB, 0.59117E+00_JPRB, 0.59887E+00_JPRB, 0.60469E+00_JPRB, 0.60455E+00_JPRB, 0.60003E+00_JPRB, 0.58629E+00_JPRB ,&
&  0.55590E+00_JPRB, 0.52321E+00_JPRB, 0.49832E+00_JPRB, 0.47836E+00_JPRB, 0.47446E+00_JPRB, 0.48505E+00_JPRB, 0.50242E+00_JPRB ,&
&  0.51559E+00_JPRB, 0.51953E+00_JPRB, 0.51352E+00_JPRB, 0.51178E+00_JPRB, 0.51676E+00_JPRB, 0.52627E+00_JPRB, 0.53367E+00_JPRB ,&
&  0.53291E+00_JPRB, 0.52969E+00_JPRB, 0.53337E+00_JPRB, 0.54008E+00_JPRB, 0.55375E+00_JPRB, 0.57107E+00_JPRB, 0.58979E+00_JPRB ,&
&  0.60851E+00_JPRB, 0.62315E+00_JPRB, 0.64228E+00_JPRB, 0.66173E+00_JPRB, 0.68727E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.67751E+00_JPRB, 0.65060E+00_JPRB, 0.62210E+00_JPRB, 0.59750E+00_JPRB, 0.58556E+00_JPRB, 0.56795E+00_JPRB, 0.55318E+00_JPRB ,&
&  0.55226E+00_JPRB, 0.56002E+00_JPRB, 0.55517E+00_JPRB, 0.54902E+00_JPRB, 0.53948E+00_JPRB, 0.53164E+00_JPRB, 0.52245E+00_JPRB ,&
&  0.51849E+00_JPRB, 0.53016E+00_JPRB, 0.55591E+00_JPRB, 0.56883E+00_JPRB, 0.57516E+00_JPRB, 0.60578E+00_JPRB, 0.63716E+00_JPRB ,&
&  0.64632E+00_JPRB, 0.66024E+00_JPRB, 0.66453E+00_JPRB, 0.63956E+00_JPRB, 0.62324E+00_JPRB, 0.61694E+00_JPRB, 0.62078E+00_JPRB ,&
&  0.61598E+00_JPRB, 0.62009E+00_JPRB, 0.61201E+00_JPRB, 0.54180E+00_JPRB, 0.54077E+00_JPRB, 0.51007E+00_JPRB, 0.49951E+00_JPRB ,&
&  0.49425E+00_JPRB, 0.49961E+00_JPRB, 0.49597E+00_JPRB, 0.48712E+00_JPRB, 0.48366E+00_JPRB, 0.48693E+00_JPRB, 0.48828E+00_JPRB ,&
&  0.48371E+00_JPRB, 0.47300E+00_JPRB, 0.47745E+00_JPRB, 0.48388E+00_JPRB, 0.48765E+00_JPRB, 0.49476E+00_JPRB, 0.50527E+00_JPRB ,&
&  0.49505E+00_JPRB, 0.48595E+00_JPRB, 0.47740E+00_JPRB, 0.47772E+00_JPRB, 0.48437E+00_JPRB, 0.49894E+00_JPRB, 0.50417E+00_JPRB ,&
&  0.50232E+00_JPRB, 0.50125E+00_JPRB, 0.50238E+00_JPRB, 0.50872E+00_JPRB, 0.52358E+00_JPRB, 0.53845E+00_JPRB, 0.53971E+00_JPRB ,&
&  0.54079E+00_JPRB, 0.54531E+00_JPRB, 0.55011E+00_JPRB, 0.55200E+00_JPRB, 0.54811E+00_JPRB, 0.53304E+00_JPRB, 0.51551E+00_JPRB ,&
&  0.49890E+00_JPRB, 0.48215E+00_JPRB, 0.46789E+00_JPRB, 0.46326E+00_JPRB, 0.47625E+00_JPRB, 0.50535E+00_JPRB, 0.52154E+00_JPRB ,&
&  0.52535E+00_JPRB, 0.52735E+00_JPRB, 0.53079E+00_JPRB, 0.53192E+00_JPRB, 0.53396E+00_JPRB, 0.53204E+00_JPRB, 0.52555E+00_JPRB ,&
&  0.51433E+00_JPRB, 0.50311E+00_JPRB, 0.50017E+00_JPRB, 0.50414E+00_JPRB, 0.50896E+00_JPRB, 0.52061E+00_JPRB, 0.53676E+00_JPRB ,&
&  0.55834E+00_JPRB, 0.59622E+00_JPRB, 0.62831E+00_JPRB, 0.65330E+00_JPRB, 0.67160E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.65423E+00_JPRB, 0.62896E+00_JPRB, 0.61211E+00_JPRB, 0.59883E+00_JPRB, 0.58067E+00_JPRB, 0.56683E+00_JPRB, 0.56127E+00_JPRB ,&
&  0.56947E+00_JPRB, 0.56806E+00_JPRB, 0.55811E+00_JPRB, 0.54970E+00_JPRB, 0.53358E+00_JPRB, 0.51918E+00_JPRB, 0.51329E+00_JPRB ,&
&  0.52183E+00_JPRB, 0.53813E+00_JPRB, 0.55811E+00_JPRB, 0.56644E+00_JPRB, 0.56592E+00_JPRB, 0.57442E+00_JPRB, 0.59369E+00_JPRB ,&
&  0.62229E+00_JPRB, 0.64222E+00_JPRB, 0.64933E+00_JPRB, 0.63629E+00_JPRB, 0.61885E+00_JPRB, 0.61074E+00_JPRB, 0.59637E+00_JPRB ,&
&  0.57963E+00_JPRB, 0.58318E+00_JPRB, 0.51554E+00_JPRB, 0.50801E+00_JPRB, 0.48466E+00_JPRB, 0.47024E+00_JPRB, 0.46649E+00_JPRB ,&
&  0.47956E+00_JPRB, 0.48362E+00_JPRB, 0.47540E+00_JPRB, 0.46345E+00_JPRB, 0.45521E+00_JPRB, 0.44409E+00_JPRB, 0.43376E+00_JPRB ,&
&  0.42648E+00_JPRB, 0.42896E+00_JPRB, 0.43798E+00_JPRB, 0.44731E+00_JPRB, 0.45840E+00_JPRB, 0.46897E+00_JPRB, 0.46547E+00_JPRB ,&
&  0.45380E+00_JPRB, 0.45175E+00_JPRB, 0.45861E+00_JPRB, 0.47306E+00_JPRB, 0.48894E+00_JPRB, 0.49644E+00_JPRB, 0.50038E+00_JPRB ,&
&  0.49279E+00_JPRB, 0.47610E+00_JPRB, 0.46174E+00_JPRB, 0.44988E+00_JPRB, 0.42445E+00_JPRB, 0.36108E+00_JPRB, 0.33821E+00_JPRB ,&
&  0.34888E+00_JPRB, 0.40773E+00_JPRB, 0.47834E+00_JPRB, 0.46979E+00_JPRB, 0.46504E+00_JPRB, 0.45082E+00_JPRB, 0.41479E+00_JPRB ,&
&  0.37368E+00_JPRB, 0.31210E+00_JPRB, 0.29032E+00_JPRB, 0.28661E+00_JPRB, 0.28560E+00_JPRB, 0.29099E+00_JPRB, 0.30305E+00_JPRB ,&
&  0.31899E+00_JPRB, 0.34605E+00_JPRB, 0.36355E+00_JPRB, 0.37025E+00_JPRB, 0.36877E+00_JPRB, 0.35264E+00_JPRB, 0.33880E+00_JPRB ,&
&  0.32948E+00_JPRB, 0.35409E+00_JPRB, 0.41237E+00_JPRB, 0.49511E+00_JPRB, 0.49383E+00_JPRB, 0.51804E+00_JPRB, 0.54262E+00_JPRB ,&
&  0.56604E+00_JPRB, 0.60547E+00_JPRB, 0.63150E+00_JPRB, 0.63732E+00_JPRB, 0.65577E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.61445E+00_JPRB, 0.60289E+00_JPRB, 0.59513E+00_JPRB, 0.58970E+00_JPRB, 0.58311E+00_JPRB, 0.58205E+00_JPRB, 0.58147E+00_JPRB ,&
&  0.57570E+00_JPRB, 0.56719E+00_JPRB, 0.56784E+00_JPRB, 0.55036E+00_JPRB, 0.52560E+00_JPRB, 0.50902E+00_JPRB, 0.50671E+00_JPRB ,&
&  0.51063E+00_JPRB, 0.51781E+00_JPRB, 0.52698E+00_JPRB, 0.53588E+00_JPRB, 0.54094E+00_JPRB, 0.55275E+00_JPRB, 0.57378E+00_JPRB ,&
&  0.59828E+00_JPRB, 0.61207E+00_JPRB, 0.61633E+00_JPRB, 0.60842E+00_JPRB, 0.59610E+00_JPRB, 0.58331E+00_JPRB, 0.57248E+00_JPRB ,&
&  0.56434E+00_JPRB, 0.46250E+00_JPRB, 0.39410E+00_JPRB, 0.40036E+00_JPRB, 0.47045E+00_JPRB, 0.45443E+00_JPRB, 0.45223E+00_JPRB ,&
&  0.46459E+00_JPRB, 0.46493E+00_JPRB, 0.45182E+00_JPRB, 0.43487E+00_JPRB, 0.41900E+00_JPRB, 0.40753E+00_JPRB, 0.39906E+00_JPRB ,&
&  0.39508E+00_JPRB, 0.40162E+00_JPRB, 0.40979E+00_JPRB, 0.42251E+00_JPRB, 0.42585E+00_JPRB, 0.41332E+00_JPRB, 0.39824E+00_JPRB ,&
&  0.38364E+00_JPRB, 0.37847E+00_JPRB, 0.38078E+00_JPRB, 0.38702E+00_JPRB, 0.39526E+00_JPRB, 0.40173E+00_JPRB, 0.39984E+00_JPRB ,&
&  0.37087E+00_JPRB, 0.34798E+00_JPRB, 0.31987E+00_JPRB, 0.29338E+00_JPRB, 0.27030E+00_JPRB, 0.25477E+00_JPRB, 0.24689E+00_JPRB ,&
&  0.24493E+00_JPRB, 0.25204E+00_JPRB, 0.27328E+00_JPRB, 0.32227E+00_JPRB, 0.39042E+00_JPRB, 0.31156E+00_JPRB, 0.27231E+00_JPRB ,&
&  0.25058E+00_JPRB, 0.23752E+00_JPRB, 0.23261E+00_JPRB, 0.23323E+00_JPRB, 0.23714E+00_JPRB, 0.24303E+00_JPRB, 0.24763E+00_JPRB ,&
&  0.25129E+00_JPRB, 0.25469E+00_JPRB, 0.25675E+00_JPRB, 0.25778E+00_JPRB, 0.25933E+00_JPRB, 0.26043E+00_JPRB, 0.25936E+00_JPRB ,&
&  0.25774E+00_JPRB, 0.26359E+00_JPRB, 0.28069E+00_JPRB, 0.30783E+00_JPRB, 0.34003E+00_JPRB, 0.37478E+00_JPRB, 0.42657E+00_JPRB ,&
&  0.49210E+00_JPRB, 0.56762E+00_JPRB, 0.58521E+00_JPRB, 0.59750E+00_JPRB, 0.60788E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.55762E+00_JPRB, 0.56145E+00_JPRB, 0.56587E+00_JPRB, 0.56841E+00_JPRB, 0.56310E+00_JPRB, 0.55590E+00_JPRB, 0.55128E+00_JPRB ,&
&  0.55636E+00_JPRB, 0.55616E+00_JPRB, 0.53258E+00_JPRB, 0.51406E+00_JPRB, 0.50337E+00_JPRB, 0.49973E+00_JPRB, 0.49656E+00_JPRB ,&
&  0.49041E+00_JPRB, 0.48831E+00_JPRB, 0.48961E+00_JPRB, 0.50099E+00_JPRB, 0.51046E+00_JPRB, 0.51874E+00_JPRB, 0.52621E+00_JPRB ,&
&  0.52184E+00_JPRB, 0.53479E+00_JPRB, 0.55651E+00_JPRB, 0.55589E+00_JPRB, 0.55331E+00_JPRB, 0.54628E+00_JPRB, 0.53199E+00_JPRB ,&
&  0.48426E+00_JPRB, 0.42034E+00_JPRB, 0.35848E+00_JPRB, 0.36970E+00_JPRB, 0.44755E+00_JPRB, 0.45722E+00_JPRB, 0.44144E+00_JPRB ,&
&  0.45176E+00_JPRB, 0.44775E+00_JPRB, 0.42995E+00_JPRB, 0.41688E+00_JPRB, 0.40147E+00_JPRB, 0.39142E+00_JPRB, 0.38442E+00_JPRB ,&
&  0.38158E+00_JPRB, 0.38026E+00_JPRB, 0.36570E+00_JPRB, 0.31759E+00_JPRB, 0.28918E+00_JPRB, 0.27194E+00_JPRB, 0.25432E+00_JPRB ,&
&  0.24052E+00_JPRB, 0.23159E+00_JPRB, 0.22988E+00_JPRB, 0.23007E+00_JPRB, 0.23456E+00_JPRB, 0.23638E+00_JPRB, 0.23808E+00_JPRB ,&
&  0.23774E+00_JPRB, 0.23657E+00_JPRB, 0.23248E+00_JPRB, 0.22791E+00_JPRB, 0.22475E+00_JPRB, 0.22421E+00_JPRB, 0.22394E+00_JPRB ,&
&  0.22654E+00_JPRB, 0.23057E+00_JPRB, 0.24695E+00_JPRB, 0.26461E+00_JPRB, 0.26412E+00_JPRB, 0.24892E+00_JPRB, 0.23396E+00_JPRB ,&
&  0.22431E+00_JPRB, 0.21942E+00_JPRB, 0.21631E+00_JPRB, 0.21575E+00_JPRB, 0.21848E+00_JPRB, 0.22235E+00_JPRB, 0.22340E+00_JPRB ,&
&  0.22329E+00_JPRB, 0.22398E+00_JPRB, 0.22409E+00_JPRB, 0.22460E+00_JPRB, 0.22529E+00_JPRB, 0.22778E+00_JPRB, 0.23246E+00_JPRB ,&
&  0.23801E+00_JPRB, 0.24797E+00_JPRB, 0.25932E+00_JPRB, 0.27460E+00_JPRB, 0.28578E+00_JPRB, 0.29390E+00_JPRB, 0.30420E+00_JPRB ,&
&  0.32156E+00_JPRB, 0.35964E+00_JPRB, 0.42858E+00_JPRB, 0.55499E+00_JPRB, 0.55013E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.52633E+00_JPRB, 0.53790E+00_JPRB, 0.55054E+00_JPRB, 0.54260E+00_JPRB, 0.52897E+00_JPRB, 0.52131E+00_JPRB, 0.52101E+00_JPRB ,&
&  0.52054E+00_JPRB, 0.51759E+00_JPRB, 0.50531E+00_JPRB, 0.49284E+00_JPRB, 0.47490E+00_JPRB, 0.42658E+00_JPRB, 0.40809E+00_JPRB ,&
&  0.39053E+00_JPRB, 0.39300E+00_JPRB, 0.39774E+00_JPRB, 0.40453E+00_JPRB, 0.41880E+00_JPRB, 0.42984E+00_JPRB, 0.43213E+00_JPRB ,&
&  0.42241E+00_JPRB, 0.40423E+00_JPRB, 0.38683E+00_JPRB, 0.37326E+00_JPRB, 0.37293E+00_JPRB, 0.37316E+00_JPRB, 0.36640E+00_JPRB ,&
&  0.35286E+00_JPRB, 0.34668E+00_JPRB, 0.34635E+00_JPRB, 0.36635E+00_JPRB, 0.41919E+00_JPRB, 0.43899E+00_JPRB, 0.41355E+00_JPRB ,&
&  0.41790E+00_JPRB, 0.41321E+00_JPRB, 0.40930E+00_JPRB, 0.41083E+00_JPRB, 0.40096E+00_JPRB, 0.38248E+00_JPRB, 0.34873E+00_JPRB ,&
&  0.30455E+00_JPRB, 0.28163E+00_JPRB, 0.26052E+00_JPRB, 0.24094E+00_JPRB, 0.22617E+00_JPRB, 0.21748E+00_JPRB, 0.21001E+00_JPRB ,&
&  0.20512E+00_JPRB, 0.20104E+00_JPRB, 0.19830E+00_JPRB, 0.19800E+00_JPRB, 0.19923E+00_JPRB, 0.19979E+00_JPRB, 0.20055E+00_JPRB ,&
&  0.20176E+00_JPRB, 0.20234E+00_JPRB, 0.20260E+00_JPRB, 0.20366E+00_JPRB, 0.20495E+00_JPRB, 0.20740E+00_JPRB, 0.21090E+00_JPRB ,&
&  0.21524E+00_JPRB, 0.21898E+00_JPRB, 0.22263E+00_JPRB, 0.22857E+00_JPRB, 0.22910E+00_JPRB, 0.22797E+00_JPRB, 0.22028E+00_JPRB ,&
&  0.21473E+00_JPRB, 0.21222E+00_JPRB, 0.20968E+00_JPRB, 0.20896E+00_JPRB, 0.20999E+00_JPRB, 0.21181E+00_JPRB, 0.21362E+00_JPRB ,&
&  0.21486E+00_JPRB, 0.21491E+00_JPRB, 0.21414E+00_JPRB, 0.21409E+00_JPRB, 0.21469E+00_JPRB, 0.21487E+00_JPRB, 0.21832E+00_JPRB ,&
&  0.22580E+00_JPRB, 0.23686E+00_JPRB, 0.24917E+00_JPRB, 0.26623E+00_JPRB, 0.28210E+00_JPRB, 0.29847E+00_JPRB, 0.32591E+00_JPRB ,&
&  0.36563E+00_JPRB, 0.47429E+00_JPRB, 0.54408E+00_JPRB, 0.48178E+00_JPRB, 0.50574E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.49828E+00_JPRB, 0.50588E+00_JPRB, 0.51714E+00_JPRB, 0.52193E+00_JPRB, 0.53017E+00_JPRB, 0.51859E+00_JPRB, 0.51166E+00_JPRB ,&
&  0.48146E+00_JPRB, 0.45903E+00_JPRB, 0.42788E+00_JPRB, 0.38874E+00_JPRB, 0.36250E+00_JPRB, 0.34215E+00_JPRB, 0.33508E+00_JPRB ,&
&  0.32767E+00_JPRB, 0.32292E+00_JPRB, 0.31844E+00_JPRB, 0.31672E+00_JPRB, 0.31577E+00_JPRB, 0.31698E+00_JPRB, 0.31690E+00_JPRB ,&
&  0.31338E+00_JPRB, 0.30776E+00_JPRB, 0.30117E+00_JPRB, 0.30424E+00_JPRB, 0.31718E+00_JPRB, 0.33220E+00_JPRB, 0.34980E+00_JPRB ,&
&  0.37005E+00_JPRB, 0.38092E+00_JPRB, 0.39113E+00_JPRB, 0.39527E+00_JPRB, 0.37989E+00_JPRB, 0.38243E+00_JPRB, 0.38513E+00_JPRB ,&
&  0.39317E+00_JPRB, 0.40032E+00_JPRB, 0.39756E+00_JPRB, 0.39354E+00_JPRB, 0.34296E+00_JPRB, 0.30973E+00_JPRB, 0.28115E+00_JPRB ,&
&  0.25969E+00_JPRB, 0.24584E+00_JPRB, 0.23445E+00_JPRB, 0.22480E+00_JPRB, 0.21524E+00_JPRB, 0.20902E+00_JPRB, 0.20433E+00_JPRB ,&
&  0.20049E+00_JPRB, 0.19872E+00_JPRB, 0.19700E+00_JPRB, 0.19642E+00_JPRB, 0.19664E+00_JPRB, 0.19599E+00_JPRB, 0.19580E+00_JPRB ,&
&  0.19545E+00_JPRB, 0.19462E+00_JPRB, 0.19447E+00_JPRB, 0.19547E+00_JPRB, 0.19728E+00_JPRB, 0.20035E+00_JPRB, 0.20148E+00_JPRB ,&
&  0.20235E+00_JPRB, 0.20289E+00_JPRB, 0.20292E+00_JPRB, 0.20471E+00_JPRB, 0.20565E+00_JPRB, 0.20583E+00_JPRB, 0.20424E+00_JPRB ,&
&  0.20261E+00_JPRB, 0.20139E+00_JPRB, 0.20119E+00_JPRB, 0.20161E+00_JPRB, 0.20294E+00_JPRB, 0.20506E+00_JPRB, 0.20557E+00_JPRB ,&
&  0.20644E+00_JPRB, 0.20612E+00_JPRB, 0.20584E+00_JPRB, 0.20753E+00_JPRB, 0.20954E+00_JPRB, 0.21176E+00_JPRB, 0.21480E+00_JPRB ,&
&  0.21662E+00_JPRB, 0.22349E+00_JPRB, 0.23606E+00_JPRB, 0.25529E+00_JPRB, 0.28451E+00_JPRB, 0.30091E+00_JPRB, 0.34003E+00_JPRB ,&
&  0.40330E+00_JPRB, 0.54769E+00_JPRB, 0.61202E+00_JPRB, 0.46823E+00_JPRB, 0.48296E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.44071E+00_JPRB, 0.45295E+00_JPRB, 0.47029E+00_JPRB, 0.48107E+00_JPRB, 0.49536E+00_JPRB, 0.49812E+00_JPRB, 0.49286E+00_JPRB ,&
&  0.47407E+00_JPRB, 0.44429E+00_JPRB, 0.42764E+00_JPRB, 0.40715E+00_JPRB, 0.38587E+00_JPRB, 0.36451E+00_JPRB, 0.35502E+00_JPRB ,&
&  0.34359E+00_JPRB, 0.33079E+00_JPRB, 0.31770E+00_JPRB, 0.30645E+00_JPRB, 0.29569E+00_JPRB, 0.28565E+00_JPRB, 0.27979E+00_JPRB ,&
&  0.27376E+00_JPRB, 0.27310E+00_JPRB, 0.27247E+00_JPRB, 0.27665E+00_JPRB, 0.28785E+00_JPRB, 0.30684E+00_JPRB, 0.33274E+00_JPRB ,&
&  0.37171E+00_JPRB, 0.37770E+00_JPRB, 0.36285E+00_JPRB, 0.35649E+00_JPRB, 0.34919E+00_JPRB, 0.34283E+00_JPRB, 0.32908E+00_JPRB ,&
&  0.33641E+00_JPRB, 0.34895E+00_JPRB, 0.36465E+00_JPRB, 0.35408E+00_JPRB, 0.30881E+00_JPRB, 0.28847E+00_JPRB, 0.26492E+00_JPRB ,&
&  0.25239E+00_JPRB, 0.24213E+00_JPRB, 0.23408E+00_JPRB, 0.22646E+00_JPRB, 0.21883E+00_JPRB, 0.21406E+00_JPRB, 0.20884E+00_JPRB ,&
&  0.20503E+00_JPRB, 0.20338E+00_JPRB, 0.20095E+00_JPRB, 0.20082E+00_JPRB, 0.20059E+00_JPRB, 0.19980E+00_JPRB, 0.19900E+00_JPRB ,&
&  0.19770E+00_JPRB, 0.19630E+00_JPRB, 0.19561E+00_JPRB, 0.19608E+00_JPRB, 0.19692E+00_JPRB, 0.19775E+00_JPRB, 0.19591E+00_JPRB ,&
&  0.19464E+00_JPRB, 0.19338E+00_JPRB, 0.19321E+00_JPRB, 0.19429E+00_JPRB, 0.19462E+00_JPRB, 0.19534E+00_JPRB, 0.19545E+00_JPRB ,&
&  0.19553E+00_JPRB, 0.19606E+00_JPRB, 0.19578E+00_JPRB, 0.19649E+00_JPRB, 0.19681E+00_JPRB, 0.19795E+00_JPRB, 0.19878E+00_JPRB ,&
&  0.19968E+00_JPRB, 0.20164E+00_JPRB, 0.20373E+00_JPRB, 0.20666E+00_JPRB, 0.21068E+00_JPRB, 0.21347E+00_JPRB, 0.21790E+00_JPRB ,&
&  0.22326E+00_JPRB, 0.23193E+00_JPRB, 0.24813E+00_JPRB, 0.26635E+00_JPRB, 0.28884E+00_JPRB, 0.30040E+00_JPRB, 0.30928E+00_JPRB ,&
&  0.32317E+00_JPRB, 0.35737E+00_JPRB, 0.40804E+00_JPRB, 0.42869E+00_JPRB, 0.43365E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.43964E+00_JPRB, 0.44084E+00_JPRB, 0.44153E+00_JPRB, 0.43818E+00_JPRB, 0.43696E+00_JPRB, 0.43695E+00_JPRB, 0.43420E+00_JPRB ,&
&  0.42628E+00_JPRB, 0.42384E+00_JPRB, 0.41893E+00_JPRB, 0.41183E+00_JPRB, 0.40487E+00_JPRB, 0.39366E+00_JPRB, 0.37736E+00_JPRB ,&
&  0.35549E+00_JPRB, 0.33625E+00_JPRB, 0.32173E+00_JPRB, 0.30836E+00_JPRB, 0.29527E+00_JPRB, 0.28249E+00_JPRB, 0.27727E+00_JPRB ,&
&  0.27273E+00_JPRB, 0.27082E+00_JPRB, 0.27025E+00_JPRB, 0.27378E+00_JPRB, 0.28138E+00_JPRB, 0.29087E+00_JPRB, 0.30124E+00_JPRB ,&
&  0.31308E+00_JPRB, 0.32410E+00_JPRB, 0.32549E+00_JPRB, 0.31834E+00_JPRB, 0.31145E+00_JPRB, 0.30710E+00_JPRB, 0.30376E+00_JPRB ,&
&  0.29983E+00_JPRB, 0.29150E+00_JPRB, 0.28205E+00_JPRB, 0.27217E+00_JPRB, 0.26220E+00_JPRB, 0.25492E+00_JPRB, 0.24638E+00_JPRB ,&
&  0.24014E+00_JPRB, 0.23446E+00_JPRB, 0.22846E+00_JPRB, 0.22331E+00_JPRB, 0.21980E+00_JPRB, 0.21687E+00_JPRB, 0.21394E+00_JPRB ,&
&  0.21243E+00_JPRB, 0.21094E+00_JPRB, 0.20928E+00_JPRB, 0.20764E+00_JPRB, 0.20562E+00_JPRB, 0.20405E+00_JPRB, 0.20262E+00_JPRB ,&
&  0.20153E+00_JPRB, 0.20058E+00_JPRB, 0.19999E+00_JPRB, 0.19939E+00_JPRB, 0.19889E+00_JPRB, 0.19792E+00_JPRB, 0.19678E+00_JPRB ,&
&  0.19602E+00_JPRB, 0.19566E+00_JPRB, 0.19489E+00_JPRB, 0.19445E+00_JPRB, 0.19410E+00_JPRB, 0.19338E+00_JPRB, 0.19382E+00_JPRB ,&
&  0.19493E+00_JPRB, 0.19618E+00_JPRB, 0.19651E+00_JPRB, 0.19759E+00_JPRB, 0.19889E+00_JPRB, 0.20052E+00_JPRB, 0.20303E+00_JPRB ,&
&  0.20496E+00_JPRB, 0.20697E+00_JPRB, 0.20947E+00_JPRB, 0.21169E+00_JPRB, 0.21508E+00_JPRB, 0.21927E+00_JPRB, 0.22441E+00_JPRB ,&
&  0.23144E+00_JPRB, 0.23992E+00_JPRB, 0.25175E+00_JPRB, 0.26618E+00_JPRB, 0.28674E+00_JPRB, 0.29502E+00_JPRB, 0.30317E+00_JPRB ,&
&  0.31017E+00_JPRB, 0.32401E+00_JPRB, 0.40639E+00_JPRB, 0.42135E+00_JPRB, 0.43627E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.31344E+00_JPRB, 0.32381E+00_JPRB, 0.33257E+00_JPRB, 0.34099E+00_JPRB, 0.34361E+00_JPRB, 0.40842E+00_JPRB, 0.38880E+00_JPRB ,&
&  0.37931E+00_JPRB, 0.35128E+00_JPRB, 0.35229E+00_JPRB, 0.34795E+00_JPRB, 0.34144E+00_JPRB, 0.33404E+00_JPRB, 0.32776E+00_JPRB ,&
&  0.32152E+00_JPRB, 0.31376E+00_JPRB, 0.30884E+00_JPRB, 0.30283E+00_JPRB, 0.29709E+00_JPRB, 0.29138E+00_JPRB, 0.28798E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28400E+00_JPRB, 0.28292E+00_JPRB, 0.28201E+00_JPRB, 0.28453E+00_JPRB, 0.28584E+00_JPRB, 0.28405E+00_JPRB ,&
&  0.28045E+00_JPRB, 0.27703E+00_JPRB, 0.27275E+00_JPRB, 0.26838E+00_JPRB, 0.26532E+00_JPRB, 0.26259E+00_JPRB, 0.25977E+00_JPRB ,&
&  0.25606E+00_JPRB, 0.25191E+00_JPRB, 0.24772E+00_JPRB, 0.24441E+00_JPRB, 0.24129E+00_JPRB, 0.23809E+00_JPRB, 0.23512E+00_JPRB ,&
&  0.23204E+00_JPRB, 0.22899E+00_JPRB, 0.22594E+00_JPRB, 0.22394E+00_JPRB, 0.22226E+00_JPRB, 0.22070E+00_JPRB, 0.21929E+00_JPRB ,&
&  0.21867E+00_JPRB, 0.21795E+00_JPRB, 0.21705E+00_JPRB, 0.21593E+00_JPRB, 0.21487E+00_JPRB, 0.21371E+00_JPRB, 0.21277E+00_JPRB ,&
&  0.21260E+00_JPRB, 0.21201E+00_JPRB, 0.21140E+00_JPRB, 0.21061E+00_JPRB, 0.21023E+00_JPRB, 0.20944E+00_JPRB, 0.20845E+00_JPRB ,&
&  0.20755E+00_JPRB, 0.20673E+00_JPRB, 0.20652E+00_JPRB, 0.20625E+00_JPRB, 0.20596E+00_JPRB, 0.20575E+00_JPRB, 0.20613E+00_JPRB ,&
&  0.20660E+00_JPRB, 0.20718E+00_JPRB, 0.20814E+00_JPRB, 0.20932E+00_JPRB, 0.21035E+00_JPRB, 0.21137E+00_JPRB, 0.21279E+00_JPRB ,&
&  0.21471E+00_JPRB, 0.21664E+00_JPRB, 0.21880E+00_JPRB, 0.22081E+00_JPRB, 0.22418E+00_JPRB, 0.22781E+00_JPRB, 0.23179E+00_JPRB ,&
&  0.23669E+00_JPRB, 0.24339E+00_JPRB, 0.25106E+00_JPRB, 0.25796E+00_JPRB, 0.26458E+00_JPRB, 0.27194E+00_JPRB, 0.27801E+00_JPRB ,&
&  0.28332E+00_JPRB, 0.28851E+00_JPRB, 0.29418E+00_JPRB, 0.30036E+00_JPRB, 0.30709E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.27102E+00_JPRB, 0.27369E+00_JPRB, 0.27597E+00_JPRB, 0.27764E+00_JPRB, 0.27796E+00_JPRB, 0.27970E+00_JPRB, 0.28014E+00_JPRB ,&
&  0.28038E+00_JPRB, 0.28030E+00_JPRB, 0.28108E+00_JPRB, 0.28189E+00_JPRB, 0.28244E+00_JPRB, 0.28272E+00_JPRB, 0.28277E+00_JPRB ,&
&  0.28230E+00_JPRB, 0.28136E+00_JPRB, 0.27770E+00_JPRB, 0.27567E+00_JPRB, 0.27450E+00_JPRB, 0.27398E+00_JPRB, 0.27373E+00_JPRB ,&
&  0.27303E+00_JPRB, 0.27216E+00_JPRB, 0.27074E+00_JPRB, 0.27050E+00_JPRB, 0.26968E+00_JPRB, 0.26861E+00_JPRB, 0.26712E+00_JPRB ,&
&  0.26566E+00_JPRB, 0.26449E+00_JPRB, 0.26305E+00_JPRB, 0.26157E+00_JPRB, 0.25976E+00_JPRB, 0.25755E+00_JPRB, 0.25482E+00_JPRB ,&
&  0.25198E+00_JPRB, 0.24908E+00_JPRB, 0.24668E+00_JPRB, 0.24446E+00_JPRB, 0.24253E+00_JPRB, 0.24099E+00_JPRB, 0.23990E+00_JPRB ,&
&  0.23870E+00_JPRB, 0.23734E+00_JPRB, 0.23596E+00_JPRB, 0.23479E+00_JPRB, 0.23365E+00_JPRB, 0.23266E+00_JPRB, 0.23217E+00_JPRB ,&
&  0.23188E+00_JPRB, 0.23154E+00_JPRB, 0.23111E+00_JPRB, 0.23073E+00_JPRB, 0.23025E+00_JPRB, 0.22981E+00_JPRB, 0.22936E+00_JPRB ,&
&  0.22899E+00_JPRB, 0.22876E+00_JPRB, 0.22840E+00_JPRB, 0.22802E+00_JPRB, 0.22779E+00_JPRB, 0.22740E+00_JPRB, 0.22696E+00_JPRB ,&
&  0.22644E+00_JPRB, 0.22614E+00_JPRB, 0.22580E+00_JPRB, 0.22538E+00_JPRB, 0.22499E+00_JPRB, 0.22473E+00_JPRB, 0.22472E+00_JPRB ,&
&  0.22468E+00_JPRB, 0.22457E+00_JPRB, 0.22462E+00_JPRB, 0.22534E+00_JPRB, 0.22609E+00_JPRB, 0.22700E+00_JPRB, 0.22803E+00_JPRB ,&
&  0.22938E+00_JPRB, 0.23079E+00_JPRB, 0.23236E+00_JPRB, 0.23387E+00_JPRB, 0.23587E+00_JPRB, 0.23785E+00_JPRB, 0.23984E+00_JPRB ,&
&  0.24174E+00_JPRB, 0.24370E+00_JPRB, 0.24567E+00_JPRB, 0.24766E+00_JPRB, 0.25025E+00_JPRB, 0.25249E+00_JPRB, 0.25502E+00_JPRB ,&
&  0.25742E+00_JPRB, 0.26007E+00_JPRB, 0.26278E+00_JPRB, 0.26516E+00_JPRB, 0.26762E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB, 0.24686E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB, 0.28545E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.29190E+00_JPRB, 0.29200E+00_JPRB, 0.29213E+00_JPRB, 0.29231E+00_JPRB, 0.29240E+00_JPRB, 0.29237E+00_JPRB, 0.29223E+00_JPRB ,&
&  0.29206E+00_JPRB, 0.29184E+00_JPRB, 0.29139E+00_JPRB, 0.29095E+00_JPRB, 0.29055E+00_JPRB, 0.29013E+00_JPRB, 0.28944E+00_JPRB ,&
&  0.28869E+00_JPRB, 0.28781E+00_JPRB, 0.28694E+00_JPRB, 0.28627E+00_JPRB, 0.28625E+00_JPRB, 0.28629E+00_JPRB, 0.28629E+00_JPRB ,&
&  0.28624E+00_JPRB, 0.28629E+00_JPRB, 0.28636E+00_JPRB, 0.28638E+00_JPRB, 0.28640E+00_JPRB, 0.28629E+00_JPRB, 0.28617E+00_JPRB ,&
&  0.28600E+00_JPRB, 0.28584E+00_JPRB, 0.28556E+00_JPRB, 0.28537E+00_JPRB, 0.28509E+00_JPRB, 0.28502E+00_JPRB, 0.28484E+00_JPRB ,&
&  0.28476E+00_JPRB, 0.28458E+00_JPRB, 0.28448E+00_JPRB, 0.28435E+00_JPRB, 0.28427E+00_JPRB, 0.28398E+00_JPRB, 0.28368E+00_JPRB ,&
&  0.28335E+00_JPRB, 0.28313E+00_JPRB, 0.28323E+00_JPRB, 0.28355E+00_JPRB, 0.28385E+00_JPRB, 0.28424E+00_JPRB, 0.28454E+00_JPRB ,&
&  0.28484E+00_JPRB, 0.28503E+00_JPRB, 0.28542E+00_JPRB, 0.28579E+00_JPRB, 0.28620E+00_JPRB, 0.28652E+00_JPRB, 0.28693E+00_JPRB ,&
&  0.28730E+00_JPRB, 0.28794E+00_JPRB, 0.28880E+00_JPRB, 0.28979E+00_JPRB, 0.29073E+00_JPRB, 0.29161E+00_JPRB, 0.29255E+00_JPRB ,&
&  0.29355E+00_JPRB, 0.29450E+00_JPRB, 0.29520E+00_JPRB, 0.29598E+00_JPRB, 0.29678E+00_JPRB, 0.29759E+00_JPRB, 0.29815E+00_JPRB ,&
&  0.29874E+00_JPRB, 0.29939E+00_JPRB, 0.30012E+00_JPRB, 0.30031E+00_JPRB, 0.30042E+00_JPRB, 0.30045E+00_JPRB, 0.30032E+00_JPRB ,&
&  0.29980E+00_JPRB, 0.29926E+00_JPRB, 0.29875E+00_JPRB, 0.29825E+00_JPRB, 0.29754E+00_JPRB, 0.29675E+00_JPRB, 0.29586E+00_JPRB ,&
&  0.29493E+00_JPRB, 0.29384E+00_JPRB, 0.29285E+00_JPRB, 0.29220E+00_JPRB, 0.29191E+00_JPRB, 0.29159E+00_JPRB, 0.29132E+00_JPRB ,&
&  0.29115E+00_JPRB, 0.29129E+00_JPRB, 0.29147E+00_JPRB, 0.29164E+00_JPRB, 0.29185E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.29700E+00_JPRB, 0.29723E+00_JPRB, 0.29765E+00_JPRB, 0.29801E+00_JPRB, 0.29790E+00_JPRB, 0.29788E+00_JPRB, 0.29789E+00_JPRB ,&
&  0.29817E+00_JPRB, 0.29850E+00_JPRB, 0.29829E+00_JPRB, 0.29774E+00_JPRB, 0.29718E+00_JPRB, 0.29661E+00_JPRB, 0.29587E+00_JPRB ,&
&  0.29524E+00_JPRB, 0.29457E+00_JPRB, 0.29363E+00_JPRB, 0.29176E+00_JPRB, 0.29053E+00_JPRB, 0.29014E+00_JPRB, 0.29066E+00_JPRB ,&
&  0.29078E+00_JPRB, 0.29040E+00_JPRB, 0.28934E+00_JPRB, 0.28813E+00_JPRB, 0.28671E+00_JPRB, 0.28528E+00_JPRB, 0.28372E+00_JPRB ,&
&  0.28231E+00_JPRB, 0.28090E+00_JPRB, 0.27984E+00_JPRB, 0.27888E+00_JPRB, 0.27894E+00_JPRB, 0.27894E+00_JPRB, 0.27896E+00_JPRB ,&
&  0.27894E+00_JPRB, 0.27871E+00_JPRB, 0.27898E+00_JPRB, 0.27926E+00_JPRB, 0.27958E+00_JPRB, 0.28002E+00_JPRB, 0.27995E+00_JPRB ,&
&  0.27937E+00_JPRB, 0.27891E+00_JPRB, 0.27968E+00_JPRB, 0.27993E+00_JPRB, 0.27997E+00_JPRB, 0.27982E+00_JPRB, 0.28064E+00_JPRB ,&
&  0.28141E+00_JPRB, 0.28201E+00_JPRB, 0.28278E+00_JPRB, 0.28431E+00_JPRB, 0.28536E+00_JPRB, 0.28649E+00_JPRB, 0.28788E+00_JPRB ,&
&  0.28946E+00_JPRB, 0.29124E+00_JPRB, 0.29322E+00_JPRB, 0.29523E+00_JPRB, 0.29704E+00_JPRB, 0.29860E+00_JPRB, 0.30047E+00_JPRB ,&
&  0.30247E+00_JPRB, 0.30481E+00_JPRB, 0.30621E+00_JPRB, 0.30771E+00_JPRB, 0.30938E+00_JPRB, 0.31188E+00_JPRB, 0.31273E+00_JPRB ,&
&  0.31347E+00_JPRB, 0.31412E+00_JPRB, 0.31488E+00_JPRB, 0.31519E+00_JPRB, 0.31544E+00_JPRB, 0.31569E+00_JPRB, 0.31584E+00_JPRB ,&
&  0.31567E+00_JPRB, 0.31506E+00_JPRB, 0.31406E+00_JPRB, 0.31283E+00_JPRB, 0.31136E+00_JPRB, 0.30999E+00_JPRB, 0.30849E+00_JPRB ,&
&  0.30732E+00_JPRB, 0.30573E+00_JPRB, 0.30424E+00_JPRB, 0.30348E+00_JPRB, 0.30337E+00_JPRB, 0.30221E+00_JPRB, 0.30075E+00_JPRB ,&
&  0.29944E+00_JPRB, 0.29853E+00_JPRB, 0.29796E+00_JPRB, 0.29745E+00_JPRB, 0.29718E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.30180E+00_JPRB, 0.30211E+00_JPRB, 0.30256E+00_JPRB, 0.30297E+00_JPRB, 0.30284E+00_JPRB, 0.30291E+00_JPRB, 0.30287E+00_JPRB ,&
&  0.30273E+00_JPRB, 0.30220E+00_JPRB, 0.30179E+00_JPRB, 0.30150E+00_JPRB, 0.30124E+00_JPRB, 0.30119E+00_JPRB, 0.30107E+00_JPRB ,&
&  0.30111E+00_JPRB, 0.30085E+00_JPRB, 0.30046E+00_JPRB, 0.29916E+00_JPRB, 0.29721E+00_JPRB, 0.29481E+00_JPRB, 0.29320E+00_JPRB ,&
&  0.29133E+00_JPRB, 0.28922E+00_JPRB, 0.28658E+00_JPRB, 0.28366E+00_JPRB, 0.28075E+00_JPRB, 0.26296E+00_JPRB, 0.25721E+00_JPRB ,&
&  0.25656E+00_JPRB, 0.25653E+00_JPRB, 0.25850E+00_JPRB, 0.26139E+00_JPRB, 0.26481E+00_JPRB, 0.26179E+00_JPRB, 0.26105E+00_JPRB ,&
&  0.25057E+00_JPRB, 0.25133E+00_JPRB, 0.25214E+00_JPRB, 0.25312E+00_JPRB, 0.25582E+00_JPRB, 0.25824E+00_JPRB, 0.26270E+00_JPRB ,&
&  0.27314E+00_JPRB, 0.27406E+00_JPRB, 0.27466E+00_JPRB, 0.27553E+00_JPRB, 0.27637E+00_JPRB, 0.27716E+00_JPRB, 0.27814E+00_JPRB ,&
&  0.27858E+00_JPRB, 0.27901E+00_JPRB, 0.27972E+00_JPRB, 0.28101E+00_JPRB, 0.28281E+00_JPRB, 0.28504E+00_JPRB, 0.28760E+00_JPRB ,&
&  0.29015E+00_JPRB, 0.29327E+00_JPRB, 0.29664E+00_JPRB, 0.30012E+00_JPRB, 0.30367E+00_JPRB, 0.30705E+00_JPRB, 0.31066E+00_JPRB ,&
&  0.31381E+00_JPRB, 0.31708E+00_JPRB, 0.31913E+00_JPRB, 0.32150E+00_JPRB, 0.32290E+00_JPRB, 0.32488E+00_JPRB, 0.32525E+00_JPRB ,&
&  0.32578E+00_JPRB, 0.32504E+00_JPRB, 0.32482E+00_JPRB, 0.32428E+00_JPRB, 0.32367E+00_JPRB, 0.32341E+00_JPRB, 0.32289E+00_JPRB ,&
&  0.32257E+00_JPRB, 0.32197E+00_JPRB, 0.32121E+00_JPRB, 0.32003E+00_JPRB, 0.31806E+00_JPRB, 0.31586E+00_JPRB, 0.31395E+00_JPRB ,&
&  0.31324E+00_JPRB, 0.31331E+00_JPRB, 0.31371E+00_JPRB, 0.31455E+00_JPRB, 0.31463E+00_JPRB, 0.31361E+00_JPRB, 0.31189E+00_JPRB ,&
&  0.31001E+00_JPRB, 0.30772E+00_JPRB, 0.30536E+00_JPRB, 0.30351E+00_JPRB, 0.30230E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.30771E+00_JPRB, 0.30848E+00_JPRB, 0.30913E+00_JPRB, 0.30973E+00_JPRB, 0.30982E+00_JPRB, 0.31000E+00_JPRB, 0.30983E+00_JPRB ,&
&  0.30953E+00_JPRB, 0.30909E+00_JPRB, 0.30867E+00_JPRB, 0.30833E+00_JPRB, 0.30789E+00_JPRB, 0.30766E+00_JPRB, 0.30713E+00_JPRB ,&
&  0.30703E+00_JPRB, 0.30612E+00_JPRB, 0.30485E+00_JPRB, 0.30203E+00_JPRB, 0.29931E+00_JPRB, 0.29611E+00_JPRB, 0.29193E+00_JPRB ,&
&  0.28778E+00_JPRB, 0.28396E+00_JPRB, 0.27052E+00_JPRB, 0.26603E+00_JPRB, 0.25840E+00_JPRB, 0.25408E+00_JPRB, 0.25103E+00_JPRB ,&
&  0.24844E+00_JPRB, 0.24703E+00_JPRB, 0.24263E+00_JPRB, 0.23843E+00_JPRB, 0.23438E+00_JPRB, 0.22797E+00_JPRB, 0.22391E+00_JPRB ,&
&  0.21957E+00_JPRB, 0.21751E+00_JPRB, 0.21804E+00_JPRB, 0.22043E+00_JPRB, 0.22630E+00_JPRB, 0.23351E+00_JPRB, 0.24310E+00_JPRB ,&
&  0.25538E+00_JPRB, 0.26583E+00_JPRB, 0.27579E+00_JPRB, 0.27479E+00_JPRB, 0.27639E+00_JPRB, 0.27771E+00_JPRB, 0.27814E+00_JPRB ,&
&  0.27471E+00_JPRB, 0.27313E+00_JPRB, 0.27654E+00_JPRB, 0.27768E+00_JPRB, 0.27982E+00_JPRB, 0.28281E+00_JPRB, 0.28614E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29464E+00_JPRB, 0.29969E+00_JPRB, 0.30531E+00_JPRB, 0.31224E+00_JPRB, 0.31817E+00_JPRB, 0.32249E+00_JPRB ,&
&  0.32569E+00_JPRB, 0.33003E+00_JPRB, 0.33239E+00_JPRB, 0.33456E+00_JPRB, 0.33650E+00_JPRB, 0.34010E+00_JPRB, 0.34289E+00_JPRB ,&
&  0.34484E+00_JPRB, 0.34386E+00_JPRB, 0.34172E+00_JPRB, 0.33608E+00_JPRB, 0.33348E+00_JPRB, 0.33628E+00_JPRB, 0.33514E+00_JPRB ,&
&  0.33455E+00_JPRB, 0.33305E+00_JPRB, 0.33158E+00_JPRB, 0.32952E+00_JPRB, 0.32748E+00_JPRB, 0.32439E+00_JPRB, 0.32187E+00_JPRB ,&
&  0.32050E+00_JPRB, 0.32126E+00_JPRB, 0.32231E+00_JPRB, 0.32345E+00_JPRB, 0.32315E+00_JPRB, 0.32237E+00_JPRB, 0.32093E+00_JPRB ,&
&  0.31868E+00_JPRB, 0.31566E+00_JPRB, 0.31240E+00_JPRB, 0.30966E+00_JPRB, 0.30794E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.31868E+00_JPRB, 0.31918E+00_JPRB, 0.31908E+00_JPRB, 0.31944E+00_JPRB, 0.31939E+00_JPRB, 0.31953E+00_JPRB, 0.31899E+00_JPRB ,&
&  0.31817E+00_JPRB, 0.31739E+00_JPRB, 0.31645E+00_JPRB, 0.31554E+00_JPRB, 0.31440E+00_JPRB, 0.31341E+00_JPRB, 0.31282E+00_JPRB ,&
&  0.31225E+00_JPRB, 0.31090E+00_JPRB, 0.30848E+00_JPRB, 0.30531E+00_JPRB, 0.30263E+00_JPRB, 0.29941E+00_JPRB, 0.29598E+00_JPRB ,&
&  0.29208E+00_JPRB, 0.28715E+00_JPRB, 0.27867E+00_JPRB, 0.27434E+00_JPRB, 0.26808E+00_JPRB, 0.26452E+00_JPRB, 0.26032E+00_JPRB ,&
&  0.26056E+00_JPRB, 0.24272E+00_JPRB, 0.24072E+00_JPRB, 0.23289E+00_JPRB, 0.22704E+00_JPRB, 0.22249E+00_JPRB, 0.21475E+00_JPRB ,&
&  0.21130E+00_JPRB, 0.20723E+00_JPRB, 0.20582E+00_JPRB, 0.20788E+00_JPRB, 0.21204E+00_JPRB, 0.22058E+00_JPRB, 0.23195E+00_JPRB ,&
&  0.24633E+00_JPRB, 0.27426E+00_JPRB, 0.28004E+00_JPRB, 0.28329E+00_JPRB, 0.28672E+00_JPRB, 0.28892E+00_JPRB, 0.28505E+00_JPRB ,&
&  0.28272E+00_JPRB, 0.27994E+00_JPRB, 0.27814E+00_JPRB, 0.27791E+00_JPRB, 0.28166E+00_JPRB, 0.28331E+00_JPRB, 0.28707E+00_JPRB ,&
&  0.29158E+00_JPRB, 0.29824E+00_JPRB, 0.30588E+00_JPRB, 0.31498E+00_JPRB, 0.32714E+00_JPRB, 0.33756E+00_JPRB, 0.34638E+00_JPRB ,&
&  0.35182E+00_JPRB, 0.35418E+00_JPRB, 0.35247E+00_JPRB, 0.35344E+00_JPRB, 0.35557E+00_JPRB, 0.36101E+00_JPRB, 0.36378E+00_JPRB ,&
&  0.36566E+00_JPRB, 0.36762E+00_JPRB, 0.36638E+00_JPRB, 0.36406E+00_JPRB, 0.35922E+00_JPRB, 0.35409E+00_JPRB, 0.35172E+00_JPRB ,&
&  0.35277E+00_JPRB, 0.35013E+00_JPRB, 0.34637E+00_JPRB, 0.34155E+00_JPRB, 0.33873E+00_JPRB, 0.33492E+00_JPRB, 0.33238E+00_JPRB ,&
&  0.33072E+00_JPRB, 0.33074E+00_JPRB, 0.32992E+00_JPRB, 0.33018E+00_JPRB, 0.32928E+00_JPRB, 0.32905E+00_JPRB, 0.32834E+00_JPRB ,&
&  0.32601E+00_JPRB, 0.32400E+00_JPRB, 0.32060E+00_JPRB, 0.31865E+00_JPRB, 0.31875E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.33279E+00_JPRB, 0.33314E+00_JPRB, 0.33353E+00_JPRB, 0.33351E+00_JPRB, 0.33303E+00_JPRB, 0.33295E+00_JPRB, 0.33329E+00_JPRB ,&
&  0.33224E+00_JPRB, 0.33170E+00_JPRB, 0.33067E+00_JPRB, 0.32979E+00_JPRB, 0.32863E+00_JPRB, 0.32810E+00_JPRB, 0.32745E+00_JPRB ,&
&  0.32608E+00_JPRB, 0.32347E+00_JPRB, 0.31977E+00_JPRB, 0.31608E+00_JPRB, 0.31239E+00_JPRB, 0.30977E+00_JPRB, 0.30600E+00_JPRB ,&
&  0.30216E+00_JPRB, 0.29712E+00_JPRB, 0.28957E+00_JPRB, 0.28334E+00_JPRB, 0.28064E+00_JPRB, 0.27597E+00_JPRB, 0.27628E+00_JPRB ,&
&  0.27421E+00_JPRB, 0.27156E+00_JPRB, 0.26471E+00_JPRB, 0.26274E+00_JPRB, 0.25865E+00_JPRB, 0.23197E+00_JPRB, 0.22482E+00_JPRB ,&
&  0.21552E+00_JPRB, 0.20640E+00_JPRB, 0.20297E+00_JPRB, 0.20197E+00_JPRB, 0.20429E+00_JPRB, 0.21273E+00_JPRB, 0.22487E+00_JPRB ,&
&  0.24038E+00_JPRB, 0.27927E+00_JPRB, 0.28967E+00_JPRB, 0.29843E+00_JPRB, 0.30595E+00_JPRB, 0.30994E+00_JPRB, 0.30994E+00_JPRB ,&
&  0.30875E+00_JPRB, 0.30790E+00_JPRB, 0.30843E+00_JPRB, 0.30931E+00_JPRB, 0.30793E+00_JPRB, 0.30391E+00_JPRB, 0.30474E+00_JPRB ,&
&  0.30219E+00_JPRB, 0.30614E+00_JPRB, 0.31514E+00_JPRB, 0.32729E+00_JPRB, 0.34117E+00_JPRB, 0.34940E+00_JPRB, 0.36031E+00_JPRB ,&
&  0.37090E+00_JPRB, 0.37699E+00_JPRB, 0.38556E+00_JPRB, 0.39153E+00_JPRB, 0.39280E+00_JPRB, 0.39308E+00_JPRB, 0.39031E+00_JPRB ,&
&  0.38667E+00_JPRB, 0.39270E+00_JPRB, 0.39444E+00_JPRB, 0.39073E+00_JPRB, 0.38459E+00_JPRB, 0.37559E+00_JPRB, 0.36879E+00_JPRB ,&
&  0.36589E+00_JPRB, 0.36721E+00_JPRB, 0.36414E+00_JPRB, 0.35653E+00_JPRB, 0.35118E+00_JPRB, 0.34655E+00_JPRB, 0.34271E+00_JPRB ,&
&  0.33989E+00_JPRB, 0.33876E+00_JPRB, 0.33692E+00_JPRB, 0.33624E+00_JPRB, 0.33621E+00_JPRB, 0.33640E+00_JPRB, 0.33665E+00_JPRB ,&
&  0.33624E+00_JPRB, 0.33646E+00_JPRB, 0.33575E+00_JPRB, 0.33393E+00_JPRB, 0.33377E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.34994E+00_JPRB, 0.35235E+00_JPRB, 0.35473E+00_JPRB, 0.35542E+00_JPRB, 0.35603E+00_JPRB, 0.35567E+00_JPRB, 0.35316E+00_JPRB ,&
&  0.34834E+00_JPRB, 0.34453E+00_JPRB, 0.34082E+00_JPRB, 0.33982E+00_JPRB, 0.33922E+00_JPRB, 0.33911E+00_JPRB, 0.33907E+00_JPRB ,&
&  0.33749E+00_JPRB, 0.33033E+00_JPRB, 0.32530E+00_JPRB, 0.32275E+00_JPRB, 0.32028E+00_JPRB, 0.31909E+00_JPRB, 0.31761E+00_JPRB ,&
&  0.31390E+00_JPRB, 0.30860E+00_JPRB, 0.30349E+00_JPRB, 0.29388E+00_JPRB, 0.28627E+00_JPRB, 0.28483E+00_JPRB, 0.28760E+00_JPRB ,&
&  0.28532E+00_JPRB, 0.27710E+00_JPRB, 0.27781E+00_JPRB, 0.27641E+00_JPRB, 0.26655E+00_JPRB, 0.26476E+00_JPRB, 0.24133E+00_JPRB ,&
&  0.22423E+00_JPRB, 0.21108E+00_JPRB, 0.20612E+00_JPRB, 0.20322E+00_JPRB, 0.20672E+00_JPRB, 0.21566E+00_JPRB, 0.23632E+00_JPRB ,&
&  0.25929E+00_JPRB, 0.28489E+00_JPRB, 0.29402E+00_JPRB, 0.30644E+00_JPRB, 0.31804E+00_JPRB, 0.33206E+00_JPRB, 0.34549E+00_JPRB ,&
&  0.35252E+00_JPRB, 0.35682E+00_JPRB, 0.36120E+00_JPRB, 0.36228E+00_JPRB, 0.36532E+00_JPRB, 0.36570E+00_JPRB, 0.36241E+00_JPRB ,&
&  0.36395E+00_JPRB, 0.36413E+00_JPRB, 0.36834E+00_JPRB, 0.37428E+00_JPRB, 0.38084E+00_JPRB, 0.38233E+00_JPRB, 0.38216E+00_JPRB ,&
&  0.38653E+00_JPRB, 0.40047E+00_JPRB, 0.41701E+00_JPRB, 0.43477E+00_JPRB, 0.43605E+00_JPRB, 0.42976E+00_JPRB, 0.41564E+00_JPRB ,&
&  0.40002E+00_JPRB, 0.40643E+00_JPRB, 0.45325E+00_JPRB, 0.44071E+00_JPRB, 0.43116E+00_JPRB, 0.41994E+00_JPRB, 0.40053E+00_JPRB ,&
&  0.38016E+00_JPRB, 0.37307E+00_JPRB, 0.36814E+00_JPRB, 0.35958E+00_JPRB, 0.35354E+00_JPRB, 0.34655E+00_JPRB, 0.34181E+00_JPRB ,&
&  0.33917E+00_JPRB, 0.33978E+00_JPRB, 0.33940E+00_JPRB, 0.34014E+00_JPRB, 0.34033E+00_JPRB, 0.34076E+00_JPRB, 0.34250E+00_JPRB ,&
&  0.34372E+00_JPRB, 0.34622E+00_JPRB, 0.34850E+00_JPRB, 0.34967E+00_JPRB, 0.34992E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.36701E+00_JPRB, 0.37405E+00_JPRB, 0.37942E+00_JPRB, 0.38768E+00_JPRB, 0.39461E+00_JPRB, 0.39284E+00_JPRB, 0.38723E+00_JPRB ,&
&  0.37628E+00_JPRB, 0.36295E+00_JPRB, 0.35468E+00_JPRB, 0.34841E+00_JPRB, 0.34529E+00_JPRB, 0.34396E+00_JPRB, 0.34269E+00_JPRB ,&
&  0.34148E+00_JPRB, 0.33975E+00_JPRB, 0.33526E+00_JPRB, 0.32854E+00_JPRB, 0.32381E+00_JPRB, 0.32225E+00_JPRB, 0.32138E+00_JPRB ,&
&  0.31981E+00_JPRB, 0.31663E+00_JPRB, 0.31210E+00_JPRB, 0.30307E+00_JPRB, 0.29499E+00_JPRB, 0.29545E+00_JPRB, 0.29156E+00_JPRB ,&
&  0.28762E+00_JPRB, 0.28566E+00_JPRB, 0.28484E+00_JPRB, 0.28298E+00_JPRB, 0.28033E+00_JPRB, 0.27358E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.23750E+00_JPRB, 0.22165E+00_JPRB, 0.21188E+00_JPRB, 0.20908E+00_JPRB, 0.21434E+00_JPRB, 0.22887E+00_JPRB, 0.25156E+00_JPRB ,&
&  0.28303E+00_JPRB, 0.29713E+00_JPRB, 0.31260E+00_JPRB, 0.33200E+00_JPRB, 0.34959E+00_JPRB, 0.36914E+00_JPRB, 0.38586E+00_JPRB ,&
&  0.40050E+00_JPRB, 0.41801E+00_JPRB, 0.43729E+00_JPRB, 0.45780E+00_JPRB, 0.48207E+00_JPRB, 0.45978E+00_JPRB, 0.46526E+00_JPRB ,&
&  0.51275E+00_JPRB, 0.51591E+00_JPRB, 0.49909E+00_JPRB, 0.47748E+00_JPRB, 0.46206E+00_JPRB, 0.45331E+00_JPRB, 0.44917E+00_JPRB ,&
&  0.44872E+00_JPRB, 0.45335E+00_JPRB, 0.46278E+00_JPRB, 0.47239E+00_JPRB, 0.47224E+00_JPRB, 0.46388E+00_JPRB, 0.44325E+00_JPRB ,&
&  0.42453E+00_JPRB, 0.43033E+00_JPRB, 0.54119E+00_JPRB, 0.48564E+00_JPRB, 0.44406E+00_JPRB, 0.42383E+00_JPRB, 0.40076E+00_JPRB ,&
&  0.38377E+00_JPRB, 0.37252E+00_JPRB, 0.36287E+00_JPRB, 0.35673E+00_JPRB, 0.34701E+00_JPRB, 0.33499E+00_JPRB, 0.32455E+00_JPRB ,&
&  0.32261E+00_JPRB, 0.32429E+00_JPRB, 0.32716E+00_JPRB, 0.33019E+00_JPRB, 0.33231E+00_JPRB, 0.33570E+00_JPRB, 0.33951E+00_JPRB ,&
&  0.34077E+00_JPRB, 0.34364E+00_JPRB, 0.34910E+00_JPRB, 0.35470E+00_JPRB, 0.36046E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.35928E+00_JPRB, 0.37716E+00_JPRB, 0.40068E+00_JPRB, 0.41447E+00_JPRB, 0.42142E+00_JPRB, 0.40815E+00_JPRB, 0.39559E+00_JPRB ,&
&  0.38237E+00_JPRB, 0.36191E+00_JPRB, 0.35194E+00_JPRB, 0.34541E+00_JPRB, 0.34147E+00_JPRB, 0.33853E+00_JPRB, 0.33613E+00_JPRB ,&
&  0.33392E+00_JPRB, 0.33090E+00_JPRB, 0.32881E+00_JPRB, 0.32944E+00_JPRB, 0.33065E+00_JPRB, 0.32878E+00_JPRB, 0.32527E+00_JPRB ,&
&  0.32221E+00_JPRB, 0.31914E+00_JPRB, 0.31252E+00_JPRB, 0.30274E+00_JPRB, 0.29891E+00_JPRB, 0.30318E+00_JPRB, 0.30181E+00_JPRB ,&
&  0.29794E+00_JPRB, 0.29249E+00_JPRB, 0.29143E+00_JPRB, 0.29258E+00_JPRB, 0.29050E+00_JPRB, 0.28533E+00_JPRB, 0.26973E+00_JPRB ,&
&  0.24852E+00_JPRB, 0.23156E+00_JPRB, 0.22538E+00_JPRB, 0.23473E+00_JPRB, 0.25476E+00_JPRB, 0.28743E+00_JPRB, 0.29897E+00_JPRB ,&
&  0.30989E+00_JPRB, 0.32420E+00_JPRB, 0.33939E+00_JPRB, 0.36090E+00_JPRB, 0.38541E+00_JPRB, 0.40683E+00_JPRB, 0.42516E+00_JPRB ,&
&  0.44870E+00_JPRB, 0.47041E+00_JPRB, 0.49775E+00_JPRB, 0.46445E+00_JPRB, 0.47201E+00_JPRB, 0.50754E+00_JPRB, 0.52281E+00_JPRB ,&
&  0.53259E+00_JPRB, 0.54617E+00_JPRB, 0.55904E+00_JPRB, 0.57540E+00_JPRB, 0.56337E+00_JPRB, 0.54430E+00_JPRB, 0.52458E+00_JPRB ,&
&  0.51425E+00_JPRB, 0.50149E+00_JPRB, 0.49403E+00_JPRB, 0.50207E+00_JPRB, 0.51078E+00_JPRB, 0.50457E+00_JPRB, 0.48649E+00_JPRB ,&
&  0.46149E+00_JPRB, 0.42747E+00_JPRB, 0.39633E+00_JPRB, 0.37640E+00_JPRB, 0.37486E+00_JPRB, 0.38067E+00_JPRB, 0.38124E+00_JPRB ,&
&  0.37759E+00_JPRB, 0.37444E+00_JPRB, 0.37002E+00_JPRB, 0.36634E+00_JPRB, 0.34423E+00_JPRB, 0.32758E+00_JPRB, 0.31460E+00_JPRB ,&
&  0.31157E+00_JPRB, 0.31153E+00_JPRB, 0.31369E+00_JPRB, 0.32044E+00_JPRB, 0.32298E+00_JPRB, 0.32677E+00_JPRB, 0.32964E+00_JPRB ,&
&  0.32991E+00_JPRB, 0.33009E+00_JPRB, 0.33470E+00_JPRB, 0.34032E+00_JPRB, 0.34812E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.40579E+00_JPRB, 0.41935E+00_JPRB, 0.43049E+00_JPRB, 0.43769E+00_JPRB, 0.42961E+00_JPRB, 0.42624E+00_JPRB, 0.42442E+00_JPRB ,&
&  0.42920E+00_JPRB, 0.38909E+00_JPRB, 0.36660E+00_JPRB, 0.34776E+00_JPRB, 0.33490E+00_JPRB, 0.32205E+00_JPRB, 0.31739E+00_JPRB ,&
&  0.31765E+00_JPRB, 0.31935E+00_JPRB, 0.32087E+00_JPRB, 0.32210E+00_JPRB, 0.32343E+00_JPRB, 0.32323E+00_JPRB, 0.32387E+00_JPRB ,&
&  0.32588E+00_JPRB, 0.32578E+00_JPRB, 0.31903E+00_JPRB, 0.31346E+00_JPRB, 0.31166E+00_JPRB, 0.31322E+00_JPRB, 0.31093E+00_JPRB ,&
&  0.30772E+00_JPRB, 0.30359E+00_JPRB, 0.30147E+00_JPRB, 0.30295E+00_JPRB, 0.30217E+00_JPRB, 0.29785E+00_JPRB, 0.27940E+00_JPRB ,&
&  0.25340E+00_JPRB, 0.24521E+00_JPRB, 0.25811E+00_JPRB, 0.30169E+00_JPRB, 0.31469E+00_JPRB, 0.32286E+00_JPRB, 0.33512E+00_JPRB ,&
&  0.33242E+00_JPRB, 0.33559E+00_JPRB, 0.35401E+00_JPRB, 0.39756E+00_JPRB, 0.41943E+00_JPRB, 0.43862E+00_JPRB, 0.45876E+00_JPRB ,&
&  0.48999E+00_JPRB, 0.51003E+00_JPRB, 0.53063E+00_JPRB, 0.49510E+00_JPRB, 0.54267E+00_JPRB, 0.66192E+00_JPRB, 0.63602E+00_JPRB ,&
&  0.63092E+00_JPRB, 0.62860E+00_JPRB, 0.64484E+00_JPRB, 0.64451E+00_JPRB, 0.62761E+00_JPRB, 0.60520E+00_JPRB, 0.58914E+00_JPRB ,&
&  0.56635E+00_JPRB, 0.55012E+00_JPRB, 0.54463E+00_JPRB, 0.55885E+00_JPRB, 0.54971E+00_JPRB, 0.53562E+00_JPRB, 0.51837E+00_JPRB ,&
&  0.49274E+00_JPRB, 0.45839E+00_JPRB, 0.42609E+00_JPRB, 0.40151E+00_JPRB, 0.39325E+00_JPRB, 0.39189E+00_JPRB, 0.38832E+00_JPRB ,&
&  0.38111E+00_JPRB, 0.38064E+00_JPRB, 0.38383E+00_JPRB, 0.38513E+00_JPRB, 0.36193E+00_JPRB, 0.33502E+00_JPRB, 0.31523E+00_JPRB ,&
&  0.30237E+00_JPRB, 0.29538E+00_JPRB, 0.29025E+00_JPRB, 0.29526E+00_JPRB, 0.30401E+00_JPRB, 0.31322E+00_JPRB, 0.31559E+00_JPRB ,&
&  0.31924E+00_JPRB, 0.32339E+00_JPRB, 0.33532E+00_JPRB, 0.35802E+00_JPRB, 0.38310E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.42646E+00_JPRB, 0.44576E+00_JPRB, 0.46092E+00_JPRB, 0.45825E+00_JPRB, 0.44916E+00_JPRB, 0.43935E+00_JPRB, 0.43481E+00_JPRB ,&
&  0.44246E+00_JPRB, 0.37963E+00_JPRB, 0.35711E+00_JPRB, 0.34196E+00_JPRB, 0.33368E+00_JPRB, 0.31904E+00_JPRB, 0.30534E+00_JPRB ,&
&  0.30256E+00_JPRB, 0.30771E+00_JPRB, 0.31696E+00_JPRB, 0.32297E+00_JPRB, 0.32783E+00_JPRB, 0.33091E+00_JPRB, 0.33171E+00_JPRB ,&
&  0.33316E+00_JPRB, 0.33472E+00_JPRB, 0.33129E+00_JPRB, 0.32677E+00_JPRB, 0.32452E+00_JPRB, 0.32544E+00_JPRB, 0.32707E+00_JPRB ,&
&  0.32780E+00_JPRB, 0.32681E+00_JPRB, 0.31958E+00_JPRB, 0.31756E+00_JPRB, 0.31662E+00_JPRB, 0.31823E+00_JPRB, 0.31939E+00_JPRB ,&
&  0.27161E+00_JPRB, 0.27292E+00_JPRB, 0.31343E+00_JPRB, 0.32028E+00_JPRB, 0.33057E+00_JPRB, 0.33653E+00_JPRB, 0.34231E+00_JPRB ,&
&  0.35295E+00_JPRB, 0.37302E+00_JPRB, 0.40054E+00_JPRB, 0.43362E+00_JPRB, 0.46058E+00_JPRB, 0.48148E+00_JPRB, 0.49954E+00_JPRB ,&
&  0.54192E+00_JPRB, 0.49503E+00_JPRB, 0.47419E+00_JPRB, 0.51266E+00_JPRB, 0.62301E+00_JPRB, 0.68000E+00_JPRB, 0.72815E+00_JPRB ,&
&  0.74950E+00_JPRB, 0.73009E+00_JPRB, 0.70925E+00_JPRB, 0.69719E+00_JPRB, 0.67985E+00_JPRB, 0.66085E+00_JPRB, 0.65425E+00_JPRB ,&
&  0.63483E+00_JPRB, 0.62796E+00_JPRB, 0.62764E+00_JPRB, 0.62477E+00_JPRB, 0.59709E+00_JPRB, 0.57689E+00_JPRB, 0.55514E+00_JPRB ,&
&  0.52886E+00_JPRB, 0.49919E+00_JPRB, 0.46548E+00_JPRB, 0.43797E+00_JPRB, 0.41836E+00_JPRB, 0.41472E+00_JPRB, 0.41079E+00_JPRB ,&
&  0.40294E+00_JPRB, 0.39937E+00_JPRB, 0.39583E+00_JPRB, 0.39290E+00_JPRB, 0.37857E+00_JPRB, 0.36325E+00_JPRB, 0.35391E+00_JPRB ,&
&  0.33030E+00_JPRB, 0.31493E+00_JPRB, 0.29674E+00_JPRB, 0.29655E+00_JPRB, 0.30782E+00_JPRB, 0.32034E+00_JPRB, 0.32981E+00_JPRB ,&
&  0.34349E+00_JPRB, 0.35243E+00_JPRB, 0.35891E+00_JPRB, 0.37130E+00_JPRB, 0.39502E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.43912E+00_JPRB, 0.45246E+00_JPRB, 0.46075E+00_JPRB, 0.45951E+00_JPRB, 0.46034E+00_JPRB, 0.45360E+00_JPRB, 0.46068E+00_JPRB ,&
&  0.49947E+00_JPRB, 0.45619E+00_JPRB, 0.49098E+00_JPRB, 0.39291E+00_JPRB, 0.34944E+00_JPRB, 0.32980E+00_JPRB, 0.31302E+00_JPRB ,&
&  0.30885E+00_JPRB, 0.31490E+00_JPRB, 0.32051E+00_JPRB, 0.32593E+00_JPRB, 0.33627E+00_JPRB, 0.33618E+00_JPRB, 0.33532E+00_JPRB ,&
&  0.33838E+00_JPRB, 0.34106E+00_JPRB, 0.34733E+00_JPRB, 0.33999E+00_JPRB, 0.33925E+00_JPRB, 0.34122E+00_JPRB, 0.35169E+00_JPRB ,&
&  0.33950E+00_JPRB, 0.33729E+00_JPRB, 0.34602E+00_JPRB, 0.34496E+00_JPRB, 0.34462E+00_JPRB, 0.34300E+00_JPRB, 0.34356E+00_JPRB ,&
&  0.34967E+00_JPRB, 0.32607E+00_JPRB, 0.34564E+00_JPRB, 0.35235E+00_JPRB, 0.35748E+00_JPRB, 0.36409E+00_JPRB, 0.36933E+00_JPRB ,&
&  0.37907E+00_JPRB, 0.39678E+00_JPRB, 0.42418E+00_JPRB, 0.45326E+00_JPRB, 0.48665E+00_JPRB, 0.51673E+00_JPRB, 0.53922E+00_JPRB ,&
&  0.59151E+00_JPRB, 0.51746E+00_JPRB, 0.61152E+00_JPRB, 0.65325E+00_JPRB, 0.75735E+00_JPRB, 0.89469E+00_JPRB, 0.92646E+00_JPRB ,&
&  0.97839E+00_JPRB, 0.91664E+00_JPRB, 0.86753E+00_JPRB, 0.81408E+00_JPRB, 0.76567E+00_JPRB, 0.73198E+00_JPRB, 0.71000E+00_JPRB ,&
&  0.71245E+00_JPRB, 0.71095E+00_JPRB, 0.74739E+00_JPRB, 0.71707E+00_JPRB, 0.66169E+00_JPRB, 0.62848E+00_JPRB, 0.59569E+00_JPRB ,&
&  0.56757E+00_JPRB, 0.54720E+00_JPRB, 0.52561E+00_JPRB, 0.48089E+00_JPRB, 0.46872E+00_JPRB, 0.46030E+00_JPRB, 0.43399E+00_JPRB ,&
&  0.42102E+00_JPRB, 0.40171E+00_JPRB, 0.38227E+00_JPRB, 0.38778E+00_JPRB, 0.37259E+00_JPRB, 0.36141E+00_JPRB, 0.35849E+00_JPRB ,&
&  0.34582E+00_JPRB, 0.32551E+00_JPRB, 0.32757E+00_JPRB, 0.33266E+00_JPRB, 0.33278E+00_JPRB, 0.33765E+00_JPRB, 0.35953E+00_JPRB ,&
&  0.36512E+00_JPRB, 0.37703E+00_JPRB, 0.40040E+00_JPRB, 0.41148E+00_JPRB, 0.42665E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.44591E+00_JPRB, 0.44628E+00_JPRB, 0.44862E+00_JPRB, 0.45168E+00_JPRB, 0.46359E+00_JPRB, 0.48588E+00_JPRB, 0.47582E+00_JPRB ,&
&  0.50390E+00_JPRB, 0.50556E+00_JPRB, 0.49654E+00_JPRB, 0.48913E+00_JPRB, 0.46968E+00_JPRB, 0.40470E+00_JPRB, 0.32762E+00_JPRB ,&
&  0.30567E+00_JPRB, 0.31176E+00_JPRB, 0.32318E+00_JPRB, 0.32827E+00_JPRB, 0.33557E+00_JPRB, 0.33421E+00_JPRB, 0.33453E+00_JPRB ,&
&  0.33866E+00_JPRB, 0.34391E+00_JPRB, 0.35058E+00_JPRB, 0.35485E+00_JPRB, 0.35218E+00_JPRB, 0.36159E+00_JPRB, 0.38054E+00_JPRB ,&
&  0.35913E+00_JPRB, 0.34621E+00_JPRB, 0.34402E+00_JPRB, 0.33527E+00_JPRB, 0.35261E+00_JPRB, 0.35985E+00_JPRB, 0.36622E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37956E+00_JPRB, 0.37938E+00_JPRB, 0.38357E+00_JPRB, 0.39173E+00_JPRB, 0.40722E+00_JPRB, 0.42303E+00_JPRB ,&
&  0.44191E+00_JPRB, 0.46279E+00_JPRB, 0.49089E+00_JPRB, 0.52924E+00_JPRB, 0.57680E+00_JPRB, 0.58488E+00_JPRB, 0.63492E+00_JPRB ,&
&  0.67973E+00_JPRB, 0.76413E+00_JPRB, 0.86655E+00_JPRB, 0.83000E+00_JPRB, 0.93854E+00_JPRB, 0.10278E+01_JPRB, 0.10112E+01_JPRB ,&
&  0.10185E+01_JPRB, 0.94555E+00_JPRB, 0.93752E+00_JPRB, 0.92379E+00_JPRB, 0.90954E+00_JPRB, 0.87470E+00_JPRB, 0.81905E+00_JPRB ,&
&  0.79840E+00_JPRB, 0.89117E+00_JPRB, 0.88783E+00_JPRB, 0.82750E+00_JPRB, 0.74782E+00_JPRB, 0.70095E+00_JPRB, 0.65105E+00_JPRB ,&
&  0.63158E+00_JPRB, 0.62778E+00_JPRB, 0.60398E+00_JPRB, 0.57945E+00_JPRB, 0.54138E+00_JPRB, 0.50234E+00_JPRB, 0.47072E+00_JPRB ,&
&  0.43517E+00_JPRB, 0.38907E+00_JPRB, 0.36905E+00_JPRB, 0.36676E+00_JPRB, 0.36474E+00_JPRB, 0.34937E+00_JPRB, 0.34541E+00_JPRB ,&
&  0.34627E+00_JPRB, 0.34507E+00_JPRB, 0.35829E+00_JPRB, 0.35824E+00_JPRB, 0.36039E+00_JPRB, 0.36140E+00_JPRB, 0.35498E+00_JPRB ,&
&  0.35430E+00_JPRB, 0.38502E+00_JPRB, 0.40736E+00_JPRB, 0.42497E+00_JPRB, 0.43646E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.44401E+00_JPRB, 0.44239E+00_JPRB, 0.43683E+00_JPRB, 0.44556E+00_JPRB, 0.46929E+00_JPRB, 0.49101E+00_JPRB, 0.50505E+00_JPRB ,&
&  0.50612E+00_JPRB, 0.50522E+00_JPRB, 0.50310E+00_JPRB, 0.49724E+00_JPRB, 0.48247E+00_JPRB, 0.46560E+00_JPRB, 0.41380E+00_JPRB ,&
&  0.34500E+00_JPRB, 0.34194E+00_JPRB, 0.32435E+00_JPRB, 0.32672E+00_JPRB, 0.34178E+00_JPRB, 0.33882E+00_JPRB, 0.33852E+00_JPRB ,&
&  0.34375E+00_JPRB, 0.34906E+00_JPRB, 0.35618E+00_JPRB, 0.35812E+00_JPRB, 0.36065E+00_JPRB, 0.39337E+00_JPRB, 0.39302E+00_JPRB ,&
&  0.36080E+00_JPRB, 0.34145E+00_JPRB, 0.33843E+00_JPRB, 0.34395E+00_JPRB, 0.35716E+00_JPRB, 0.37321E+00_JPRB, 0.38064E+00_JPRB ,&
&  0.38689E+00_JPRB, 0.39327E+00_JPRB, 0.39617E+00_JPRB, 0.40749E+00_JPRB, 0.42665E+00_JPRB, 0.45111E+00_JPRB, 0.47295E+00_JPRB ,&
&  0.49089E+00_JPRB, 0.51504E+00_JPRB, 0.54981E+00_JPRB, 0.59893E+00_JPRB, 0.63844E+00_JPRB, 0.68985E+00_JPRB, 0.76385E+00_JPRB ,&
&  0.82872E+00_JPRB, 0.92349E+00_JPRB, 0.98572E+00_JPRB, 0.10529E+01_JPRB, 0.11590E+01_JPRB, 0.11498E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.10579E+01_JPRB, 0.99000E+00_JPRB, 0.11072E+01_JPRB, 0.10972E+01_JPRB, 0.10396E+01_JPRB, 0.10231E+01_JPRB, 0.94941E+00_JPRB ,&
&  0.89953E+00_JPRB, 0.88462E+00_JPRB, 0.86326E+00_JPRB, 0.83529E+00_JPRB, 0.80214E+00_JPRB, 0.79974E+00_JPRB, 0.74805E+00_JPRB ,&
&  0.73887E+00_JPRB, 0.71198E+00_JPRB, 0.60891E+00_JPRB, 0.53735E+00_JPRB, 0.47445E+00_JPRB, 0.48127E+00_JPRB, 0.46946E+00_JPRB ,&
&  0.43306E+00_JPRB, 0.38052E+00_JPRB, 0.37794E+00_JPRB, 0.37369E+00_JPRB, 0.37587E+00_JPRB, 0.37689E+00_JPRB, 0.36450E+00_JPRB ,&
&  0.36885E+00_JPRB, 0.38573E+00_JPRB, 0.38549E+00_JPRB, 0.39121E+00_JPRB, 0.39261E+00_JPRB, 0.38710E+00_JPRB, 0.35422E+00_JPRB ,&
&  0.35596E+00_JPRB, 0.38129E+00_JPRB, 0.39986E+00_JPRB, 0.42070E+00_JPRB, 0.43608E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.45331E+00_JPRB, 0.45465E+00_JPRB, 0.45042E+00_JPRB, 0.45811E+00_JPRB, 0.48029E+00_JPRB, 0.50225E+00_JPRB, 0.51568E+00_JPRB ,&
&  0.51722E+00_JPRB, 0.51964E+00_JPRB, 0.51402E+00_JPRB, 0.50613E+00_JPRB, 0.49592E+00_JPRB, 0.47739E+00_JPRB, 0.47948E+00_JPRB ,&
&  0.38619E+00_JPRB, 0.36380E+00_JPRB, 0.32004E+00_JPRB, 0.31485E+00_JPRB, 0.35257E+00_JPRB, 0.34918E+00_JPRB, 0.35194E+00_JPRB ,&
&  0.35019E+00_JPRB, 0.35838E+00_JPRB, 0.35881E+00_JPRB, 0.36108E+00_JPRB, 0.38283E+00_JPRB, 0.41351E+00_JPRB, 0.40011E+00_JPRB ,&
&  0.37563E+00_JPRB, 0.35840E+00_JPRB, 0.35773E+00_JPRB, 0.35989E+00_JPRB, 0.36699E+00_JPRB, 0.38209E+00_JPRB, 0.39381E+00_JPRB ,&
&  0.40338E+00_JPRB, 0.42510E+00_JPRB, 0.44429E+00_JPRB, 0.46909E+00_JPRB, 0.49812E+00_JPRB, 0.53549E+00_JPRB, 0.56261E+00_JPRB ,&
&  0.58994E+00_JPRB, 0.62549E+00_JPRB, 0.66791E+00_JPRB, 0.72436E+00_JPRB, 0.75877E+00_JPRB, 0.84262E+00_JPRB, 0.94211E+00_JPRB ,&
&  0.11057E+01_JPRB, 0.11669E+01_JPRB, 0.12193E+01_JPRB, 0.13789E+01_JPRB, 0.14165E+01_JPRB, 0.13632E+01_JPRB, 0.12303E+01_JPRB ,&
&  0.11442E+01_JPRB, 0.10748E+01_JPRB, 0.10728E+01_JPRB, 0.10906E+01_JPRB, 0.10638E+01_JPRB, 0.10779E+01_JPRB, 0.10162E+01_JPRB ,&
&  0.91218E+00_JPRB, 0.86535E+00_JPRB, 0.84334E+00_JPRB, 0.84785E+00_JPRB, 0.81056E+00_JPRB, 0.81526E+00_JPRB, 0.78343E+00_JPRB ,&
&  0.77893E+00_JPRB, 0.67228E+00_JPRB, 0.51764E+00_JPRB, 0.47875E+00_JPRB, 0.40556E+00_JPRB, 0.41421E+00_JPRB, 0.49053E+00_JPRB ,&
&  0.47497E+00_JPRB, 0.43395E+00_JPRB, 0.40724E+00_JPRB, 0.39272E+00_JPRB, 0.39025E+00_JPRB, 0.42631E+00_JPRB, 0.41846E+00_JPRB ,&
&  0.39322E+00_JPRB, 0.41213E+00_JPRB, 0.43138E+00_JPRB, 0.44311E+00_JPRB, 0.44380E+00_JPRB, 0.44053E+00_JPRB, 0.41244E+00_JPRB ,&
&  0.41599E+00_JPRB, 0.41823E+00_JPRB, 0.42635E+00_JPRB, 0.43973E+00_JPRB, 0.44809E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.48095E+00_JPRB, 0.48156E+00_JPRB, 0.48045E+00_JPRB, 0.48778E+00_JPRB, 0.50298E+00_JPRB, 0.52095E+00_JPRB, 0.53352E+00_JPRB ,&
&  0.53214E+00_JPRB, 0.53182E+00_JPRB, 0.52501E+00_JPRB, 0.51585E+00_JPRB, 0.50591E+00_JPRB, 0.49111E+00_JPRB, 0.48253E+00_JPRB ,&
&  0.49052E+00_JPRB, 0.42099E+00_JPRB, 0.35142E+00_JPRB, 0.31306E+00_JPRB, 0.33294E+00_JPRB, 0.33915E+00_JPRB, 0.35321E+00_JPRB ,&
&  0.35028E+00_JPRB, 0.36923E+00_JPRB, 0.37017E+00_JPRB, 0.37665E+00_JPRB, 0.42024E+00_JPRB, 0.43178E+00_JPRB, 0.42325E+00_JPRB ,&
&  0.42137E+00_JPRB, 0.40452E+00_JPRB, 0.39842E+00_JPRB, 0.40554E+00_JPRB, 0.42088E+00_JPRB, 0.40762E+00_JPRB, 0.43112E+00_JPRB ,&
&  0.45907E+00_JPRB, 0.49000E+00_JPRB, 0.51743E+00_JPRB, 0.55947E+00_JPRB, 0.61078E+00_JPRB, 0.64801E+00_JPRB, 0.68511E+00_JPRB ,&
&  0.71924E+00_JPRB, 0.74237E+00_JPRB, 0.75395E+00_JPRB, 0.79891E+00_JPRB, 0.87529E+00_JPRB, 0.10091E+01_JPRB, 0.10193E+01_JPRB ,&
&  0.10943E+01_JPRB, 0.11634E+01_JPRB, 0.11625E+01_JPRB, 0.13558E+01_JPRB, 0.14329E+01_JPRB, 0.13908E+01_JPRB, 0.13083E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.11987E+01_JPRB, 0.12378E+01_JPRB, 0.12278E+01_JPRB, 0.12054E+01_JPRB, 0.11630E+01_JPRB, 0.10654E+01_JPRB ,&
&  0.95743E+00_JPRB, 0.89157E+00_JPRB, 0.85929E+00_JPRB, 0.81928E+00_JPRB, 0.78419E+00_JPRB, 0.75287E+00_JPRB, 0.76848E+00_JPRB ,&
&  0.81264E+00_JPRB, 0.49699E+00_JPRB, 0.41331E+00_JPRB, 0.45899E+00_JPRB, 0.40121E+00_JPRB, 0.40918E+00_JPRB, 0.44828E+00_JPRB ,&
&  0.44027E+00_JPRB, 0.43257E+00_JPRB, 0.44038E+00_JPRB, 0.42786E+00_JPRB, 0.43465E+00_JPRB, 0.50179E+00_JPRB, 0.50298E+00_JPRB ,&
&  0.47858E+00_JPRB, 0.46442E+00_JPRB, 0.49343E+00_JPRB, 0.49839E+00_JPRB, 0.50291E+00_JPRB, 0.49959E+00_JPRB, 0.48623E+00_JPRB ,&
&  0.47516E+00_JPRB, 0.47409E+00_JPRB, 0.47354E+00_JPRB, 0.47663E+00_JPRB, 0.47896E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.53720E+00_JPRB, 0.53999E+00_JPRB, 0.54210E+00_JPRB, 0.54233E+00_JPRB, 0.54641E+00_JPRB, 0.55899E+00_JPRB, 0.56600E+00_JPRB ,&
&  0.56122E+00_JPRB, 0.55584E+00_JPRB, 0.54416E+00_JPRB, 0.53223E+00_JPRB, 0.52038E+00_JPRB, 0.50948E+00_JPRB, 0.49986E+00_JPRB ,&
&  0.49029E+00_JPRB, 0.46269E+00_JPRB, 0.36971E+00_JPRB, 0.32856E+00_JPRB, 0.30498E+00_JPRB, 0.32223E+00_JPRB, 0.34153E+00_JPRB ,&
&  0.35247E+00_JPRB, 0.36747E+00_JPRB, 0.38066E+00_JPRB, 0.43233E+00_JPRB, 0.49936E+00_JPRB, 0.48381E+00_JPRB, 0.50585E+00_JPRB ,&
&  0.51029E+00_JPRB, 0.49358E+00_JPRB, 0.45881E+00_JPRB, 0.47899E+00_JPRB, 0.47057E+00_JPRB, 0.46274E+00_JPRB, 0.53303E+00_JPRB ,&
&  0.57887E+00_JPRB, 0.61401E+00_JPRB, 0.65003E+00_JPRB, 0.68635E+00_JPRB, 0.71722E+00_JPRB, 0.71451E+00_JPRB, 0.72732E+00_JPRB ,&
&  0.75085E+00_JPRB, 0.77618E+00_JPRB, 0.81554E+00_JPRB, 0.84035E+00_JPRB, 0.89403E+00_JPRB, 0.95651E+00_JPRB, 0.94541E+00_JPRB ,&
&  0.99067E+00_JPRB, 0.90921E+00_JPRB, 0.74357E+00_JPRB, 0.96805E+00_JPRB, 0.13527E+01_JPRB, 0.13971E+01_JPRB, 0.12675E+01_JPRB ,&
&  0.13392E+01_JPRB, 0.13493E+01_JPRB, 0.14447E+01_JPRB, 0.14987E+01_JPRB, 0.14140E+01_JPRB, 0.13036E+01_JPRB, 0.11848E+01_JPRB ,&
&  0.10540E+01_JPRB, 0.96987E+00_JPRB, 0.93079E+00_JPRB, 0.96297E+00_JPRB, 0.92809E+00_JPRB, 0.92057E+00_JPRB, 0.91155E+00_JPRB ,&
&  0.80172E+00_JPRB, 0.67463E+00_JPRB, 0.50210E+00_JPRB, 0.51165E+00_JPRB, 0.42020E+00_JPRB, 0.42167E+00_JPRB, 0.45633E+00_JPRB ,&
&  0.44311E+00_JPRB, 0.46246E+00_JPRB, 0.47275E+00_JPRB, 0.46774E+00_JPRB, 0.54054E+00_JPRB, 0.60986E+00_JPRB, 0.65612E+00_JPRB ,&
&  0.63949E+00_JPRB, 0.56351E+00_JPRB, 0.59354E+00_JPRB, 0.58407E+00_JPRB, 0.59476E+00_JPRB, 0.59040E+00_JPRB, 0.58301E+00_JPRB ,&
&  0.57003E+00_JPRB, 0.55572E+00_JPRB, 0.54398E+00_JPRB, 0.53721E+00_JPRB, 0.53575E+00_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.56528E+00_JPRB, 0.55803E+00_JPRB, 0.55419E+00_JPRB, 0.55330E+00_JPRB, 0.55743E+00_JPRB, 0.56704E+00_JPRB, 0.56805E+00_JPRB ,&
&  0.56717E+00_JPRB, 0.55973E+00_JPRB, 0.54894E+00_JPRB, 0.53617E+00_JPRB, 0.52461E+00_JPRB, 0.51633E+00_JPRB, 0.50896E+00_JPRB ,&
&  0.49532E+00_JPRB, 0.43474E+00_JPRB, 0.34514E+00_JPRB, 0.30390E+00_JPRB, 0.28805E+00_JPRB, 0.29768E+00_JPRB, 0.32769E+00_JPRB ,&
&  0.35528E+00_JPRB, 0.38010E+00_JPRB, 0.42814E+00_JPRB, 0.51018E+00_JPRB, 0.57705E+00_JPRB, 0.59526E+00_JPRB, 0.62360E+00_JPRB ,&
&  0.71219E+00_JPRB, 0.57432E+00_JPRB, 0.56738E+00_JPRB, 0.56267E+00_JPRB, 0.61490E+00_JPRB, 0.65741E+00_JPRB, 0.64803E+00_JPRB ,&
&  0.63438E+00_JPRB, 0.65761E+00_JPRB, 0.68197E+00_JPRB, 0.70733E+00_JPRB, 0.72394E+00_JPRB, 0.75185E+00_JPRB, 0.82372E+00_JPRB ,&
&  0.89063E+00_JPRB, 0.90937E+00_JPRB, 0.87020E+00_JPRB, 0.85484E+00_JPRB, 0.91997E+00_JPRB, 0.95700E+00_JPRB, 0.93907E+00_JPRB ,&
&  0.84417E+00_JPRB, 0.80375E+00_JPRB, 0.94049E+00_JPRB, 0.11147E+01_JPRB, 0.13166E+01_JPRB, 0.14591E+01_JPRB, 0.14591E+01_JPRB ,&
&  0.15182E+01_JPRB, 0.15180E+01_JPRB, 0.15645E+01_JPRB, 0.14128E+01_JPRB, 0.14756E+01_JPRB, 0.14038E+01_JPRB, 0.13229E+01_JPRB ,&
&  0.12739E+01_JPRB, 0.12498E+01_JPRB, 0.11951E+01_JPRB, 0.12120E+01_JPRB, 0.12775E+01_JPRB, 0.12862E+01_JPRB, 0.10197E+01_JPRB ,&
&  0.80828E+00_JPRB, 0.80665E+00_JPRB, 0.70405E+00_JPRB, 0.61447E+00_JPRB, 0.53569E+00_JPRB, 0.53528E+00_JPRB, 0.52952E+00_JPRB ,&
&  0.52171E+00_JPRB, 0.52130E+00_JPRB, 0.51345E+00_JPRB, 0.55196E+00_JPRB, 0.63045E+00_JPRB, 0.72016E+00_JPRB, 0.77083E+00_JPRB ,&
&  0.71035E+00_JPRB, 0.75223E+00_JPRB, 0.73432E+00_JPRB, 0.70746E+00_JPRB, 0.74005E+00_JPRB, 0.74047E+00_JPRB, 0.71362E+00_JPRB ,&
&  0.67642E+00_JPRB, 0.64924E+00_JPRB, 0.62442E+00_JPRB, 0.59961E+00_JPRB, 0.57920E+00_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.63645E+00_JPRB, 0.61786E+00_JPRB, 0.60410E+00_JPRB, 0.59685E+00_JPRB, 0.59815E+00_JPRB, 0.60475E+00_JPRB, 0.60012E+00_JPRB ,&
&  0.58428E+00_JPRB, 0.56636E+00_JPRB, 0.55521E+00_JPRB, 0.54721E+00_JPRB, 0.53577E+00_JPRB, 0.52473E+00_JPRB, 0.51650E+00_JPRB ,&
&  0.51130E+00_JPRB, 0.41638E+00_JPRB, 0.32457E+00_JPRB, 0.30871E+00_JPRB, 0.30455E+00_JPRB, 0.29367E+00_JPRB, 0.31507E+00_JPRB ,&
&  0.34919E+00_JPRB, 0.40308E+00_JPRB, 0.47351E+00_JPRB, 0.60485E+00_JPRB, 0.68092E+00_JPRB, 0.73643E+00_JPRB, 0.77051E+00_JPRB ,&
&  0.73356E+00_JPRB, 0.71243E+00_JPRB, 0.73441E+00_JPRB, 0.72375E+00_JPRB, 0.70079E+00_JPRB, 0.70188E+00_JPRB, 0.68224E+00_JPRB ,&
&  0.67681E+00_JPRB, 0.69044E+00_JPRB, 0.71687E+00_JPRB, 0.75734E+00_JPRB, 0.81275E+00_JPRB, 0.87468E+00_JPRB, 0.96128E+00_JPRB ,&
&  0.97243E+00_JPRB, 0.93941E+00_JPRB, 0.84317E+00_JPRB, 0.84273E+00_JPRB, 0.77496E+00_JPRB, 0.78143E+00_JPRB, 0.78137E+00_JPRB ,&
&  0.96291E+00_JPRB, 0.10153E+01_JPRB, 0.98692E+00_JPRB, 0.11799E+01_JPRB, 0.11554E+01_JPRB, 0.12604E+01_JPRB, 0.15574E+01_JPRB ,&
&  0.16534E+01_JPRB, 0.16690E+01_JPRB, 0.17000E+01_JPRB, 0.13605E+01_JPRB, 0.11601E+01_JPRB, 0.15509E+01_JPRB, 0.16148E+01_JPRB ,&
&  0.16611E+01_JPRB, 0.17221E+01_JPRB, 0.17100E+01_JPRB, 0.17777E+01_JPRB, 0.14779E+01_JPRB, 0.11789E+01_JPRB, 0.87225E+00_JPRB ,&
&  0.80689E+00_JPRB, 0.73841E+00_JPRB, 0.78143E+00_JPRB, 0.69963E+00_JPRB, 0.67255E+00_JPRB, 0.62990E+00_JPRB, 0.59543E+00_JPRB ,&
&  0.59298E+00_JPRB, 0.58520E+00_JPRB, 0.61585E+00_JPRB, 0.67414E+00_JPRB, 0.76803E+00_JPRB, 0.77566E+00_JPRB, 0.87763E+00_JPRB ,&
&  0.91443E+00_JPRB, 0.89454E+00_JPRB, 0.82393E+00_JPRB, 0.82762E+00_JPRB, 0.80535E+00_JPRB, 0.79523E+00_JPRB, 0.77854E+00_JPRB ,&
&  0.75229E+00_JPRB, 0.72447E+00_JPRB, 0.70188E+00_JPRB, 0.67860E+00_JPRB, 0.65698E+00_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.69041E+00_JPRB, 0.67051E+00_JPRB, 0.65411E+00_JPRB, 0.64564E+00_JPRB, 0.64155E+00_JPRB, 0.63454E+00_JPRB, 0.61560E+00_JPRB ,&
&  0.59010E+00_JPRB, 0.56706E+00_JPRB, 0.54644E+00_JPRB, 0.53552E+00_JPRB, 0.52179E+00_JPRB, 0.52106E+00_JPRB, 0.52577E+00_JPRB ,&
&  0.52082E+00_JPRB, 0.45900E+00_JPRB, 0.34947E+00_JPRB, 0.31787E+00_JPRB, 0.32549E+00_JPRB, 0.31099E+00_JPRB, 0.31226E+00_JPRB ,&
&  0.36430E+00_JPRB, 0.43947E+00_JPRB, 0.53625E+00_JPRB, 0.71056E+00_JPRB, 0.80157E+00_JPRB, 0.89506E+00_JPRB, 0.93115E+00_JPRB ,&
&  0.93526E+00_JPRB, 0.81877E+00_JPRB, 0.74913E+00_JPRB, 0.69414E+00_JPRB, 0.65230E+00_JPRB, 0.62862E+00_JPRB, 0.63983E+00_JPRB ,&
&  0.65684E+00_JPRB, 0.70213E+00_JPRB, 0.75490E+00_JPRB, 0.77475E+00_JPRB, 0.82882E+00_JPRB, 0.86408E+00_JPRB, 0.91140E+00_JPRB ,&
&  0.92529E+00_JPRB, 0.90304E+00_JPRB, 0.87664E+00_JPRB, 0.91230E+00_JPRB, 0.88691E+00_JPRB, 0.83455E+00_JPRB, 0.86009E+00_JPRB ,&
&  0.96130E+00_JPRB, 0.97094E+00_JPRB, 0.10076E+01_JPRB, 0.11565E+01_JPRB, 0.13604E+01_JPRB, 0.14071E+01_JPRB, 0.16059E+01_JPRB ,&
&  0.13638E+01_JPRB, 0.12081E+01_JPRB, 0.10522E+01_JPRB, 0.91915E+00_JPRB, 0.11681E+01_JPRB, 0.18803E+01_JPRB, 0.19816E+01_JPRB ,&
&  0.19835E+01_JPRB, 0.20992E+01_JPRB, 0.21384E+01_JPRB, 0.18445E+01_JPRB, 0.14572E+01_JPRB, 0.11245E+01_JPRB, 0.12861E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12899E+01_JPRB, 0.11335E+01_JPRB, 0.89164E+00_JPRB, 0.71763E+00_JPRB, 0.77795E+00_JPRB, 0.74950E+00_JPRB ,&
&  0.72534E+00_JPRB, 0.72320E+00_JPRB, 0.88427E+00_JPRB, 0.89716E+00_JPRB, 0.90988E+00_JPRB, 0.97797E+00_JPRB, 0.10487E+01_JPRB ,&
&  0.96902E+00_JPRB, 0.94970E+00_JPRB, 0.92815E+00_JPRB, 0.88429E+00_JPRB, 0.85765E+00_JPRB, 0.82628E+00_JPRB, 0.79592E+00_JPRB ,&
&  0.77562E+00_JPRB, 0.76016E+00_JPRB, 0.74096E+00_JPRB, 0.72682E+00_JPRB, 0.70902E+00_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.73000E+00_JPRB, 0.71037E+00_JPRB, 0.69564E+00_JPRB, 0.68230E+00_JPRB, 0.66858E+00_JPRB, 0.64843E+00_JPRB, 0.62242E+00_JPRB ,&
&  0.59378E+00_JPRB, 0.56532E+00_JPRB, 0.54512E+00_JPRB, 0.53208E+00_JPRB, 0.51817E+00_JPRB, 0.51572E+00_JPRB, 0.52103E+00_JPRB ,&
&  0.52345E+00_JPRB, 0.53816E+00_JPRB, 0.40348E+00_JPRB, 0.34519E+00_JPRB, 0.34314E+00_JPRB, 0.33771E+00_JPRB, 0.32796E+00_JPRB ,&
&  0.39618E+00_JPRB, 0.49298E+00_JPRB, 0.57462E+00_JPRB, 0.75119E+00_JPRB, 0.96794E+00_JPRB, 0.93940E+00_JPRB, 0.90627E+00_JPRB ,&
&  0.89917E+00_JPRB, 0.81775E+00_JPRB, 0.74037E+00_JPRB, 0.69266E+00_JPRB, 0.66011E+00_JPRB, 0.65666E+00_JPRB, 0.67458E+00_JPRB ,&
&  0.71945E+00_JPRB, 0.78327E+00_JPRB, 0.79206E+00_JPRB, 0.81508E+00_JPRB, 0.82950E+00_JPRB, 0.88473E+00_JPRB, 0.94411E+00_JPRB ,&
&  0.91652E+00_JPRB, 0.89248E+00_JPRB, 0.91822E+00_JPRB, 0.10083E+01_JPRB, 0.99348E+00_JPRB, 0.94114E+00_JPRB, 0.10628E+01_JPRB ,&
&  0.10518E+01_JPRB, 0.11351E+01_JPRB, 0.11175E+01_JPRB, 0.12991E+01_JPRB, 0.14067E+01_JPRB, 0.13856E+01_JPRB, 0.14477E+01_JPRB ,&
&  0.11584E+01_JPRB, 0.10367E+01_JPRB, 0.10722E+01_JPRB, 0.11736E+01_JPRB, 0.13172E+01_JPRB, 0.16128E+01_JPRB, 0.23627E+01_JPRB ,&
&  0.19954E+01_JPRB, 0.21166E+01_JPRB, 0.21222E+01_JPRB, 0.19911E+01_JPRB, 0.10873E+01_JPRB, 0.85392E+00_JPRB, 0.12966E+01_JPRB ,&
&  0.13906E+01_JPRB, 0.88521E+00_JPRB, 0.62537E+00_JPRB, 0.72380E+00_JPRB, 0.57091E+00_JPRB, 0.65576E+00_JPRB, 0.83437E+00_JPRB ,&
&  0.88652E+00_JPRB, 0.93514E+00_JPRB, 0.11432E+01_JPRB, 0.10407E+01_JPRB, 0.10156E+01_JPRB, 0.12996E+01_JPRB, 0.11925E+01_JPRB ,&
&  0.11062E+01_JPRB, 0.10594E+01_JPRB, 0.10489E+01_JPRB, 0.94137E+00_JPRB, 0.87833E+00_JPRB, 0.84127E+00_JPRB, 0.82311E+00_JPRB ,&
&  0.81690E+00_JPRB, 0.80841E+00_JPRB, 0.79684E+00_JPRB, 0.77742E+00_JPRB, 0.75184E+00_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.74832E+00_JPRB, 0.73441E+00_JPRB, 0.72401E+00_JPRB, 0.71143E+00_JPRB, 0.69099E+00_JPRB, 0.66162E+00_JPRB, 0.63077E+00_JPRB ,&
&  0.60444E+00_JPRB, 0.57150E+00_JPRB, 0.54823E+00_JPRB, 0.53264E+00_JPRB, 0.52059E+00_JPRB, 0.51550E+00_JPRB, 0.51620E+00_JPRB ,&
&  0.51073E+00_JPRB, 0.51118E+00_JPRB, 0.47964E+00_JPRB, 0.42145E+00_JPRB, 0.40689E+00_JPRB, 0.39668E+00_JPRB, 0.39892E+00_JPRB ,&
&  0.47389E+00_JPRB, 0.56780E+00_JPRB, 0.66340E+00_JPRB, 0.79036E+00_JPRB, 0.90246E+00_JPRB, 0.86311E+00_JPRB, 0.89150E+00_JPRB ,&
&  0.86979E+00_JPRB, 0.81954E+00_JPRB, 0.75276E+00_JPRB, 0.72326E+00_JPRB, 0.71772E+00_JPRB, 0.72675E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.78462E+00_JPRB, 0.81863E+00_JPRB, 0.81052E+00_JPRB, 0.78996E+00_JPRB, 0.83266E+00_JPRB, 0.91840E+00_JPRB, 0.96463E+00_JPRB ,&
&  0.93463E+00_JPRB, 0.93905E+00_JPRB, 0.96323E+00_JPRB, 0.10425E+01_JPRB, 0.11465E+01_JPRB, 0.11420E+01_JPRB, 0.99383E+00_JPRB ,&
&  0.90484E+00_JPRB, 0.97549E+00_JPRB, 0.12231E+01_JPRB, 0.12435E+01_JPRB, 0.12731E+01_JPRB, 0.13106E+01_JPRB, 0.13235E+01_JPRB ,&
&  0.12952E+01_JPRB, 0.13031E+01_JPRB, 0.12924E+01_JPRB, 0.14724E+01_JPRB, 0.16528E+01_JPRB, 0.16644E+01_JPRB, 0.22914E+01_JPRB ,&
&  0.22766E+01_JPRB, 0.20785E+01_JPRB, 0.16023E+01_JPRB, 0.11995E+01_JPRB, 0.11769E+01_JPRB, 0.93009E+00_JPRB, 0.63848E+00_JPRB ,&
&  0.60331E+00_JPRB, 0.56948E+00_JPRB, 0.50553E+00_JPRB, 0.50003E+00_JPRB, 0.47540E+00_JPRB, 0.59074E+00_JPRB, 0.82707E+00_JPRB ,&
&  0.11017E+01_JPRB, 0.12830E+01_JPRB, 0.13068E+01_JPRB, 0.12070E+01_JPRB, 0.11194E+01_JPRB, 0.14167E+01_JPRB, 0.13678E+01_JPRB ,&
&  0.11026E+01_JPRB, 0.10834E+01_JPRB, 0.10186E+01_JPRB, 0.93991E+00_JPRB, 0.90055E+00_JPRB, 0.86890E+00_JPRB, 0.85039E+00_JPRB ,&
&  0.83546E+00_JPRB, 0.82270E+00_JPRB, 0.80816E+00_JPRB, 0.78604E+00_JPRB, 0.76645E+00_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.76765E+00_JPRB, 0.75674E+00_JPRB, 0.74349E+00_JPRB, 0.72676E+00_JPRB, 0.70256E+00_JPRB, 0.67451E+00_JPRB, 0.64770E+00_JPRB ,&
&  0.62503E+00_JPRB, 0.59297E+00_JPRB, 0.56475E+00_JPRB, 0.54384E+00_JPRB, 0.52920E+00_JPRB, 0.52106E+00_JPRB, 0.51971E+00_JPRB ,&
&  0.51907E+00_JPRB, 0.51134E+00_JPRB, 0.52720E+00_JPRB, 0.50000E+00_JPRB, 0.52758E+00_JPRB, 0.53194E+00_JPRB, 0.55759E+00_JPRB ,&
&  0.62096E+00_JPRB, 0.71629E+00_JPRB, 0.84084E+00_JPRB, 0.90754E+00_JPRB, 0.97332E+00_JPRB, 0.96697E+00_JPRB, 0.93891E+00_JPRB ,&
&  0.89788E+00_JPRB, 0.87867E+00_JPRB, 0.83316E+00_JPRB, 0.80996E+00_JPRB, 0.79175E+00_JPRB, 0.78753E+00_JPRB, 0.79810E+00_JPRB ,&
&  0.81678E+00_JPRB, 0.83833E+00_JPRB, 0.79551E+00_JPRB, 0.79815E+00_JPRB, 0.86072E+00_JPRB, 0.93410E+00_JPRB, 0.98066E+00_JPRB ,&
&  0.99349E+00_JPRB, 0.10005E+01_JPRB, 0.10358E+01_JPRB, 0.11115E+01_JPRB, 0.10351E+01_JPRB, 0.89272E+00_JPRB, 0.97961E+00_JPRB ,&
&  0.10553E+01_JPRB, 0.12079E+01_JPRB, 0.11975E+01_JPRB, 0.14070E+01_JPRB, 0.13847E+01_JPRB, 0.12980E+01_JPRB, 0.13801E+01_JPRB ,&
&  0.14598E+01_JPRB, 0.17535E+01_JPRB, 0.14978E+01_JPRB, 0.15039E+01_JPRB, 0.20756E+01_JPRB, 0.19264E+01_JPRB, 0.20421E+01_JPRB ,&
&  0.22076E+01_JPRB, 0.18342E+01_JPRB, 0.14838E+01_JPRB, 0.12870E+01_JPRB, 0.16347E+01_JPRB, 0.15118E+01_JPRB, 0.79454E+00_JPRB ,&
&  0.64857E+00_JPRB, 0.57601E+00_JPRB, 0.52689E+00_JPRB, 0.48817E+00_JPRB, 0.47153E+00_JPRB, 0.62111E+00_JPRB, 0.10482E+01_JPRB ,&
&  0.10980E+01_JPRB, 0.15747E+01_JPRB, 0.15844E+01_JPRB, 0.14553E+01_JPRB, 0.13276E+01_JPRB, 0.14917E+01_JPRB, 0.14551E+01_JPRB ,&
&  0.12047E+01_JPRB, 0.11513E+01_JPRB, 0.10256E+01_JPRB, 0.94382E+00_JPRB, 0.88519E+00_JPRB, 0.85221E+00_JPRB, 0.84073E+00_JPRB ,&
&  0.82880E+00_JPRB, 0.81666E+00_JPRB, 0.80515E+00_JPRB, 0.79259E+00_JPRB, 0.78042E+00_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.77506E+00_JPRB, 0.75486E+00_JPRB, 0.74256E+00_JPRB, 0.73047E+00_JPRB, 0.71647E+00_JPRB, 0.69689E+00_JPRB, 0.67125E+00_JPRB ,&
&  0.64632E+00_JPRB, 0.61326E+00_JPRB, 0.58343E+00_JPRB, 0.55932E+00_JPRB, 0.54096E+00_JPRB, 0.52882E+00_JPRB, 0.52620E+00_JPRB ,&
&  0.52382E+00_JPRB, 0.52463E+00_JPRB, 0.53656E+00_JPRB, 0.57679E+00_JPRB, 0.63337E+00_JPRB, 0.64819E+00_JPRB, 0.73245E+00_JPRB ,&
&  0.85096E+00_JPRB, 0.98139E+00_JPRB, 0.11227E+01_JPRB, 0.11550E+01_JPRB, 0.11566E+01_JPRB, 0.11064E+01_JPRB, 0.10526E+01_JPRB ,&
&  0.10014E+01_JPRB, 0.99718E+00_JPRB, 0.94117E+00_JPRB, 0.89689E+00_JPRB, 0.86370E+00_JPRB, 0.85120E+00_JPRB, 0.84978E+00_JPRB ,&
&  0.87385E+00_JPRB, 0.83461E+00_JPRB, 0.80119E+00_JPRB, 0.82181E+00_JPRB, 0.88169E+00_JPRB, 0.96315E+00_JPRB, 0.10184E+01_JPRB ,&
&  0.10377E+01_JPRB, 0.10346E+01_JPRB, 0.10684E+01_JPRB, 0.11336E+01_JPRB, 0.94496E+00_JPRB, 0.10779E+01_JPRB, 0.10814E+01_JPRB ,&
&  0.11045E+01_JPRB, 0.11655E+01_JPRB, 0.11613E+01_JPRB, 0.13541E+01_JPRB, 0.13986E+01_JPRB, 0.13129E+01_JPRB, 0.13364E+01_JPRB ,&
&  0.15949E+01_JPRB, 0.15172E+01_JPRB, 0.14371E+01_JPRB, 0.16574E+01_JPRB, 0.20331E+01_JPRB, 0.27664E+01_JPRB, 0.18114E+01_JPRB ,&
&  0.15495E+01_JPRB, 0.16991E+01_JPRB, 0.14639E+01_JPRB, 0.13067E+01_JPRB, 0.15417E+01_JPRB, 0.16543E+01_JPRB, 0.13591E+01_JPRB ,&
&  0.72305E+00_JPRB, 0.57066E+00_JPRB, 0.58578E+00_JPRB, 0.59644E+00_JPRB, 0.57071E+00_JPRB, 0.72613E+00_JPRB, 0.15082E+01_JPRB ,&
&  0.13112E+01_JPRB, 0.16923E+01_JPRB, 0.16598E+01_JPRB, 0.14872E+01_JPRB, 0.13726E+01_JPRB, 0.14194E+01_JPRB, 0.14135E+01_JPRB ,&
&  0.11521E+01_JPRB, 0.10438E+01_JPRB, 0.94486E+00_JPRB, 0.88190E+00_JPRB, 0.85529E+00_JPRB, 0.84030E+00_JPRB, 0.83482E+00_JPRB ,&
&  0.82577E+00_JPRB, 0.81423E+00_JPRB, 0.80855E+00_JPRB, 0.80462E+00_JPRB, 0.79454E+00_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.75605E+00_JPRB, 0.74575E+00_JPRB, 0.74037E+00_JPRB, 0.74033E+00_JPRB, 0.73300E+00_JPRB, 0.70510E+00_JPRB, 0.67616E+00_JPRB ,&
&  0.64902E+00_JPRB, 0.61604E+00_JPRB, 0.59013E+00_JPRB, 0.56468E+00_JPRB, 0.54707E+00_JPRB, 0.53525E+00_JPRB, 0.53075E+00_JPRB ,&
&  0.53579E+00_JPRB, 0.53872E+00_JPRB, 0.55226E+00_JPRB, 0.62424E+00_JPRB, 0.72794E+00_JPRB, 0.75533E+00_JPRB, 0.84577E+00_JPRB ,&
&  0.11951E+01_JPRB, 0.14052E+01_JPRB, 0.13631E+01_JPRB, 0.13548E+01_JPRB, 0.13620E+01_JPRB, 0.13377E+01_JPRB, 0.12553E+01_JPRB ,&
&  0.11446E+01_JPRB, 0.10836E+01_JPRB, 0.98724E+00_JPRB, 0.92918E+00_JPRB, 0.89666E+00_JPRB, 0.88343E+00_JPRB, 0.88869E+00_JPRB ,&
&  0.86923E+00_JPRB, 0.83900E+00_JPRB, 0.81961E+00_JPRB, 0.84047E+00_JPRB, 0.90943E+00_JPRB, 0.98589E+00_JPRB, 0.10471E+01_JPRB ,&
&  0.10572E+01_JPRB, 0.10516E+01_JPRB, 0.10618E+01_JPRB, 0.10492E+01_JPRB, 0.10751E+01_JPRB, 0.11028E+01_JPRB, 0.11430E+01_JPRB ,&
&  0.11470E+01_JPRB, 0.12259E+01_JPRB, 0.13136E+01_JPRB, 0.13414E+01_JPRB, 0.13476E+01_JPRB, 0.13407E+01_JPRB, 0.13731E+01_JPRB ,&
&  0.15317E+01_JPRB, 0.15232E+01_JPRB, 0.15615E+01_JPRB, 0.19196E+01_JPRB, 0.21521E+01_JPRB, 0.24062E+01_JPRB, 0.21732E+01_JPRB ,&
&  0.17736E+01_JPRB, 0.13724E+01_JPRB, 0.12578E+01_JPRB, 0.13216E+01_JPRB, 0.14503E+01_JPRB, 0.14557E+01_JPRB, 0.18088E+01_JPRB ,&
&  0.17801E+01_JPRB, 0.12555E+01_JPRB, 0.10790E+01_JPRB, 0.13155E+01_JPRB, 0.11387E+01_JPRB, 0.10246E+01_JPRB, 0.12489E+01_JPRB ,&
&  0.14082E+01_JPRB, 0.16626E+01_JPRB, 0.15604E+01_JPRB, 0.13428E+01_JPRB, 0.13551E+01_JPRB, 0.13164E+01_JPRB, 0.12337E+01_JPRB ,&
&  0.10558E+01_JPRB, 0.98623E+00_JPRB, 0.93487E+00_JPRB, 0.88799E+00_JPRB, 0.85595E+00_JPRB, 0.84186E+00_JPRB, 0.83334E+00_JPRB ,&
&  0.81526E+00_JPRB, 0.80459E+00_JPRB, 0.79909E+00_JPRB, 0.78914E+00_JPRB, 0.77134E+00_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.74208E+00_JPRB, 0.74400E+00_JPRB, 0.75121E+00_JPRB, 0.72910E+00_JPRB, 0.69922E+00_JPRB, 0.67158E+00_JPRB, 0.65688E+00_JPRB ,&
&  0.63778E+00_JPRB, 0.60956E+00_JPRB, 0.58315E+00_JPRB, 0.56354E+00_JPRB, 0.54599E+00_JPRB, 0.53733E+00_JPRB, 0.53562E+00_JPRB ,&
&  0.54440E+00_JPRB, 0.55863E+00_JPRB, 0.61071E+00_JPRB, 0.71945E+00_JPRB, 0.84539E+00_JPRB, 0.10417E+01_JPRB, 0.94514E+00_JPRB ,&
&  0.12467E+01_JPRB, 0.16711E+01_JPRB, 0.16475E+01_JPRB, 0.15965E+01_JPRB, 0.15400E+01_JPRB, 0.14792E+01_JPRB, 0.13339E+01_JPRB ,&
&  0.11955E+01_JPRB, 0.11215E+01_JPRB, 0.10258E+01_JPRB, 0.96338E+00_JPRB, 0.94070E+00_JPRB, 0.92869E+00_JPRB, 0.89261E+00_JPRB ,&
&  0.86267E+00_JPRB, 0.83698E+00_JPRB, 0.82514E+00_JPRB, 0.84840E+00_JPRB, 0.89840E+00_JPRB, 0.97979E+00_JPRB, 0.10331E+01_JPRB ,&
&  0.10560E+01_JPRB, 0.10565E+01_JPRB, 0.10514E+01_JPRB, 0.10015E+01_JPRB, 0.10447E+01_JPRB, 0.10865E+01_JPRB, 0.11326E+01_JPRB ,&
&  0.11100E+01_JPRB, 0.10772E+01_JPRB, 0.12205E+01_JPRB, 0.13387E+01_JPRB, 0.13599E+01_JPRB, 0.12978E+01_JPRB, 0.13198E+01_JPRB ,&
&  0.15445E+01_JPRB, 0.17420E+01_JPRB, 0.21016E+01_JPRB, 0.21951E+01_JPRB, 0.22466E+01_JPRB, 0.22692E+01_JPRB, 0.21815E+01_JPRB ,&
&  0.20964E+01_JPRB, 0.16867E+01_JPRB, 0.14750E+01_JPRB, 0.13761E+01_JPRB, 0.13522E+01_JPRB, 0.13812E+01_JPRB, 0.16604E+01_JPRB ,&
&  0.19182E+01_JPRB, 0.20453E+01_JPRB, 0.20681E+01_JPRB, 0.17890E+01_JPRB, 0.15761E+01_JPRB, 0.13582E+01_JPRB, 0.12973E+01_JPRB ,&
&  0.15268E+01_JPRB, 0.15896E+01_JPRB, 0.14718E+01_JPRB, 0.16163E+01_JPRB, 0.13618E+01_JPRB, 0.13146E+01_JPRB, 0.12120E+01_JPRB ,&
&  0.10971E+01_JPRB, 0.10181E+01_JPRB, 0.95079E+00_JPRB, 0.90785E+00_JPRB, 0.88808E+00_JPRB, 0.84612E+00_JPRB, 0.81271E+00_JPRB ,&
&  0.78782E+00_JPRB, 0.77642E+00_JPRB, 0.76450E+00_JPRB, 0.75557E+00_JPRB, 0.74455E+00_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.76089E+00_JPRB, 0.76400E+00_JPRB, 0.75643E+00_JPRB, 0.72410E+00_JPRB, 0.70056E+00_JPRB, 0.68319E+00_JPRB, 0.64751E+00_JPRB ,&
&  0.62326E+00_JPRB, 0.60240E+00_JPRB, 0.57947E+00_JPRB, 0.56126E+00_JPRB, 0.55231E+00_JPRB, 0.54055E+00_JPRB, 0.54610E+00_JPRB ,&
&  0.58379E+00_JPRB, 0.64580E+00_JPRB, 0.75384E+00_JPRB, 0.85405E+00_JPRB, 0.98955E+00_JPRB, 0.12337E+01_JPRB, 0.13451E+01_JPRB ,&
&  0.13520E+01_JPRB, 0.19780E+01_JPRB, 0.17818E+01_JPRB, 0.16782E+01_JPRB, 0.15879E+01_JPRB, 0.14627E+01_JPRB, 0.13055E+01_JPRB ,&
&  0.11554E+01_JPRB, 0.11033E+01_JPRB, 0.10616E+01_JPRB, 0.10082E+01_JPRB, 0.98669E+00_JPRB, 0.91279E+00_JPRB, 0.89659E+00_JPRB ,&
&  0.87832E+00_JPRB, 0.86314E+00_JPRB, 0.84521E+00_JPRB, 0.87231E+00_JPRB, 0.92743E+00_JPRB, 0.98513E+00_JPRB, 0.10393E+01_JPRB ,&
&  0.10575E+01_JPRB, 0.10512E+01_JPRB, 0.10116E+01_JPRB, 0.99499E+00_JPRB, 0.10027E+01_JPRB, 0.10478E+01_JPRB, 0.11013E+01_JPRB ,&
&  0.11136E+01_JPRB, 0.10960E+01_JPRB, 0.11786E+01_JPRB, 0.11993E+01_JPRB, 0.11989E+01_JPRB, 0.12859E+01_JPRB, 0.13153E+01_JPRB ,&
&  0.15712E+01_JPRB, 0.16063E+01_JPRB, 0.22306E+01_JPRB, 0.20613E+01_JPRB, 0.20163E+01_JPRB, 0.20798E+01_JPRB, 0.19262E+01_JPRB ,&
&  0.16397E+01_JPRB, 0.15156E+01_JPRB, 0.13686E+01_JPRB, 0.13186E+01_JPRB, 0.13446E+01_JPRB, 0.14463E+01_JPRB, 0.16493E+01_JPRB ,&
&  0.16885E+01_JPRB, 0.16460E+01_JPRB, 0.16709E+01_JPRB, 0.14467E+01_JPRB, 0.13943E+01_JPRB, 0.13012E+01_JPRB, 0.12817E+01_JPRB ,&
&  0.15305E+01_JPRB, 0.15908E+01_JPRB, 0.16164E+01_JPRB, 0.15515E+01_JPRB, 0.13140E+01_JPRB, 0.12369E+01_JPRB, 0.11303E+01_JPRB ,&
&  0.10442E+01_JPRB, 0.97702E+00_JPRB, 0.93689E+00_JPRB, 0.89753E+00_JPRB, 0.85359E+00_JPRB, 0.81041E+00_JPRB, 0.78233E+00_JPRB ,&
&  0.75993E+00_JPRB, 0.74851E+00_JPRB, 0.74907E+00_JPRB, 0.74914E+00_JPRB, 0.75273E+00_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.81453E+00_JPRB, 0.82836E+00_JPRB, 0.83798E+00_JPRB, 0.87403E+00_JPRB, 0.92186E+00_JPRB, 0.98994E+00_JPRB, 0.10231E+01_JPRB ,&
&  0.88228E+00_JPRB, 0.59648E+00_JPRB, 0.58024E+00_JPRB, 0.56929E+00_JPRB, 0.56030E+00_JPRB, 0.56492E+00_JPRB, 0.60623E+00_JPRB ,&
&  0.66850E+00_JPRB, 0.74807E+00_JPRB, 0.83973E+00_JPRB, 0.96534E+00_JPRB, 0.11430E+01_JPRB, 0.14847E+01_JPRB, 0.17405E+01_JPRB ,&
&  0.15796E+01_JPRB, 0.16944E+01_JPRB, 0.18316E+01_JPRB, 0.16277E+01_JPRB, 0.14972E+01_JPRB, 0.13900E+01_JPRB, 0.13053E+01_JPRB ,&
&  0.11853E+01_JPRB, 0.11243E+01_JPRB, 0.11020E+01_JPRB, 0.10771E+01_JPRB, 0.94107E+00_JPRB, 0.92012E+00_JPRB, 0.91526E+00_JPRB ,&
&  0.91149E+00_JPRB, 0.91258E+00_JPRB, 0.91371E+00_JPRB, 0.95002E+00_JPRB, 0.99479E+00_JPRB, 0.10444E+01_JPRB, 0.10816E+01_JPRB ,&
&  0.10893E+01_JPRB, 0.10793E+01_JPRB, 0.10513E+01_JPRB, 0.99110E+00_JPRB, 0.99270E+00_JPRB, 0.10235E+01_JPRB, 0.10652E+01_JPRB ,&
&  0.10942E+01_JPRB, 0.11419E+01_JPRB, 0.11707E+01_JPRB, 0.11674E+01_JPRB, 0.11433E+01_JPRB, 0.13103E+01_JPRB, 0.13935E+01_JPRB ,&
&  0.15614E+01_JPRB, 0.17680E+01_JPRB, 0.23963E+01_JPRB, 0.17868E+01_JPRB, 0.18161E+01_JPRB, 0.18534E+01_JPRB, 0.17244E+01_JPRB ,&
&  0.14881E+01_JPRB, 0.13929E+01_JPRB, 0.12815E+01_JPRB, 0.12584E+01_JPRB, 0.13490E+01_JPRB, 0.14339E+01_JPRB, 0.16759E+01_JPRB ,&
&  0.14584E+01_JPRB, 0.14943E+01_JPRB, 0.13537E+01_JPRB, 0.13397E+01_JPRB, 0.12875E+01_JPRB, 0.12183E+01_JPRB, 0.11811E+01_JPRB ,&
&  0.14017E+01_JPRB, 0.14010E+01_JPRB, 0.13367E+01_JPRB, 0.12681E+01_JPRB, 0.11013E+01_JPRB, 0.10303E+01_JPRB, 0.95338E+00_JPRB ,&
&  0.91842E+00_JPRB, 0.90581E+00_JPRB, 0.90237E+00_JPRB, 0.87057E+00_JPRB, 0.84631E+00_JPRB, 0.81554E+00_JPRB, 0.79395E+00_JPRB ,&
&  0.78043E+00_JPRB, 0.77575E+00_JPRB, 0.77708E+00_JPRB, 0.78648E+00_JPRB, 0.79908E+00_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.89298E+00_JPRB, 0.88535E+00_JPRB, 0.88100E+00_JPRB, 0.87792E+00_JPRB, 0.86818E+00_JPRB, 0.83778E+00_JPRB, 0.77521E+00_JPRB ,&
&  0.68696E+00_JPRB, 0.61760E+00_JPRB, 0.60412E+00_JPRB, 0.59944E+00_JPRB, 0.60534E+00_JPRB, 0.63398E+00_JPRB, 0.68467E+00_JPRB ,&
&  0.73816E+00_JPRB, 0.82779E+00_JPRB, 0.95306E+00_JPRB, 0.11127E+01_JPRB, 0.13266E+01_JPRB, 0.16576E+01_JPRB, 0.19749E+01_JPRB ,&
&  0.19035E+01_JPRB, 0.13860E+01_JPRB, 0.15814E+01_JPRB, 0.15582E+01_JPRB, 0.14462E+01_JPRB, 0.13268E+01_JPRB, 0.12166E+01_JPRB ,&
&  0.11109E+01_JPRB, 0.10732E+01_JPRB, 0.10479E+01_JPRB, 0.10063E+01_JPRB, 0.95532E+00_JPRB, 0.96860E+00_JPRB, 0.98658E+00_JPRB ,&
&  0.10048E+01_JPRB, 0.10178E+01_JPRB, 0.10188E+01_JPRB, 0.10535E+01_JPRB, 0.10807E+01_JPRB, 0.11269E+01_JPRB, 0.11794E+01_JPRB ,&
&  0.11926E+01_JPRB, 0.11871E+01_JPRB, 0.11683E+01_JPRB, 0.11010E+01_JPRB, 0.10855E+01_JPRB, 0.11204E+01_JPRB, 0.11530E+01_JPRB ,&
&  0.11759E+01_JPRB, 0.11672E+01_JPRB, 0.12128E+01_JPRB, 0.13175E+01_JPRB, 0.14488E+01_JPRB, 0.13784E+01_JPRB, 0.15693E+01_JPRB ,&
&  0.17079E+01_JPRB, 0.19739E+01_JPRB, 0.19267E+01_JPRB, 0.21696E+01_JPRB, 0.14460E+01_JPRB, 0.15507E+01_JPRB, 0.13906E+01_JPRB ,&
&  0.13962E+01_JPRB, 0.12701E+01_JPRB, 0.12188E+01_JPRB, 0.12723E+01_JPRB, 0.13961E+01_JPRB, 0.15325E+01_JPRB, 0.15004E+01_JPRB ,&
&  0.12432E+01_JPRB, 0.11950E+01_JPRB, 0.12164E+01_JPRB, 0.12962E+01_JPRB, 0.12717E+01_JPRB, 0.12244E+01_JPRB, 0.11180E+01_JPRB ,&
&  0.11948E+01_JPRB, 0.11992E+01_JPRB, 0.11752E+01_JPRB, 0.11122E+01_JPRB, 0.95782E+00_JPRB, 0.86731E+00_JPRB, 0.87396E+00_JPRB ,&
&  0.89359E+00_JPRB, 0.91064E+00_JPRB, 0.91200E+00_JPRB, 0.90085E+00_JPRB, 0.87548E+00_JPRB, 0.85332E+00_JPRB, 0.84044E+00_JPRB ,&
&  0.83251E+00_JPRB, 0.84068E+00_JPRB, 0.85120E+00_JPRB, 0.86274E+00_JPRB, 0.88010E+00_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.88792E+00_JPRB, 0.86584E+00_JPRB, 0.84211E+00_JPRB, 0.81969E+00_JPRB, 0.79922E+00_JPRB, 0.75214E+00_JPRB, 0.70347E+00_JPRB ,&
&  0.67183E+00_JPRB, 0.65568E+00_JPRB, 0.65582E+00_JPRB, 0.66169E+00_JPRB, 0.68452E+00_JPRB, 0.71196E+00_JPRB, 0.76121E+00_JPRB ,&
&  0.83639E+00_JPRB, 0.94329E+00_JPRB, 0.10909E+01_JPRB, 0.12706E+01_JPRB, 0.14952E+01_JPRB, 0.17493E+01_JPRB, 0.18879E+01_JPRB ,&
&  0.20060E+01_JPRB, 0.19912E+01_JPRB, 0.17451E+01_JPRB, 0.12830E+01_JPRB, 0.11647E+01_JPRB, 0.12029E+01_JPRB, 0.11315E+01_JPRB ,&
&  0.11169E+01_JPRB, 0.10863E+01_JPRB, 0.10521E+01_JPRB, 0.10452E+01_JPRB, 0.10464E+01_JPRB, 0.10733E+01_JPRB, 0.10969E+01_JPRB ,&
&  0.11147E+01_JPRB, 0.11301E+01_JPRB, 0.11521E+01_JPRB, 0.12027E+01_JPRB, 0.12393E+01_JPRB, 0.12869E+01_JPRB, 0.13257E+01_JPRB ,&
&  0.13466E+01_JPRB, 0.13751E+01_JPRB, 0.13566E+01_JPRB, 0.13160E+01_JPRB, 0.12788E+01_JPRB, 0.13108E+01_JPRB, 0.13531E+01_JPRB ,&
&  0.13888E+01_JPRB, 0.13950E+01_JPRB, 0.14211E+01_JPRB, 0.15575E+01_JPRB, 0.16787E+01_JPRB, 0.15606E+01_JPRB, 0.16262E+01_JPRB ,&
&  0.18768E+01_JPRB, 0.20585E+01_JPRB, 0.16018E+01_JPRB, 0.18625E+01_JPRB, 0.11332E+01_JPRB, 0.13466E+01_JPRB, 0.13997E+01_JPRB ,&
&  0.12756E+01_JPRB, 0.12350E+01_JPRB, 0.12301E+01_JPRB, 0.13220E+01_JPRB, 0.14411E+01_JPRB, 0.14682E+01_JPRB, 0.13000E+01_JPRB ,&
&  0.11141E+01_JPRB, 0.10766E+01_JPRB, 0.11111E+01_JPRB, 0.11584E+01_JPRB, 0.11560E+01_JPRB, 0.10790E+01_JPRB, 0.91278E+00_JPRB ,&
&  0.83706E+00_JPRB, 0.10682E+01_JPRB, 0.11038E+01_JPRB, 0.11622E+01_JPRB, 0.10029E+01_JPRB, 0.85651E+00_JPRB, 0.90833E+00_JPRB ,&
&  0.94819E+00_JPRB, 0.96667E+00_JPRB, 0.96167E+00_JPRB, 0.94385E+00_JPRB, 0.91956E+00_JPRB, 0.89979E+00_JPRB, 0.89122E+00_JPRB ,&
&  0.89017E+00_JPRB, 0.88690E+00_JPRB, 0.89276E+00_JPRB, 0.90077E+00_JPRB, 0.90245E+00_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.81923E+00_JPRB, 0.79247E+00_JPRB, 0.77174E+00_JPRB, 0.75452E+00_JPRB, 0.74346E+00_JPRB, 0.72130E+00_JPRB, 0.69319E+00_JPRB ,&
&  0.67890E+00_JPRB, 0.67317E+00_JPRB, 0.68245E+00_JPRB, 0.70801E+00_JPRB, 0.74598E+00_JPRB, 0.78460E+00_JPRB, 0.85782E+00_JPRB ,&
&  0.94475E+00_JPRB, 0.10743E+01_JPRB, 0.12401E+01_JPRB, 0.14728E+01_JPRB, 0.16603E+01_JPRB, 0.18271E+01_JPRB, 0.18499E+01_JPRB ,&
&  0.19214E+01_JPRB, 0.19199E+01_JPRB, 0.18076E+01_JPRB, 0.16663E+01_JPRB, 0.13087E+01_JPRB, 0.12923E+01_JPRB, 0.14202E+01_JPRB ,&
&  0.14624E+01_JPRB, 0.12704E+01_JPRB, 0.11519E+01_JPRB, 0.11542E+01_JPRB, 0.11526E+01_JPRB, 0.11867E+01_JPRB, 0.12187E+01_JPRB ,&
&  0.12485E+01_JPRB, 0.12805E+01_JPRB, 0.13106E+01_JPRB, 0.13598E+01_JPRB, 0.14100E+01_JPRB, 0.14356E+01_JPRB, 0.14698E+01_JPRB ,&
&  0.14802E+01_JPRB, 0.15027E+01_JPRB, 0.15155E+01_JPRB, 0.14974E+01_JPRB, 0.15278E+01_JPRB, 0.15799E+01_JPRB, 0.16712E+01_JPRB ,&
&  0.17613E+01_JPRB, 0.17376E+01_JPRB, 0.18443E+01_JPRB, 0.20187E+01_JPRB, 0.20428E+01_JPRB, 0.19741E+01_JPRB, 0.19725E+01_JPRB ,&
&  0.21548E+01_JPRB, 0.23160E+01_JPRB, 0.13359E+01_JPRB, 0.14495E+01_JPRB, 0.16352E+01_JPRB, 0.13975E+01_JPRB, 0.12597E+01_JPRB ,&
&  0.12315E+01_JPRB, 0.12171E+01_JPRB, 0.12536E+01_JPRB, 0.13128E+01_JPRB, 0.14251E+01_JPRB, 0.16313E+01_JPRB, 0.11890E+01_JPRB ,&
&  0.97877E+00_JPRB, 0.92762E+00_JPRB, 0.93870E+00_JPRB, 0.96182E+00_JPRB, 0.96136E+00_JPRB, 0.88622E+00_JPRB, 0.79047E+00_JPRB ,&
&  0.79594E+00_JPRB, 0.11095E+01_JPRB, 0.12521E+01_JPRB, 0.14531E+01_JPRB, 0.13555E+01_JPRB, 0.87176E+00_JPRB, 0.94906E+00_JPRB ,&
&  0.97172E+00_JPRB, 0.97349E+00_JPRB, 0.93937E+00_JPRB, 0.92611E+00_JPRB, 0.91593E+00_JPRB, 0.90449E+00_JPRB, 0.90265E+00_JPRB ,&
&  0.89651E+00_JPRB, 0.89886E+00_JPRB, 0.90333E+00_JPRB, 0.88076E+00_JPRB, 0.84649E+00_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.75564E+00_JPRB, 0.73369E+00_JPRB, 0.72146E+00_JPRB, 0.70611E+00_JPRB, 0.69956E+00_JPRB, 0.69216E+00_JPRB, 0.68928E+00_JPRB ,&
&  0.68783E+00_JPRB, 0.69041E+00_JPRB, 0.70685E+00_JPRB, 0.73608E+00_JPRB, 0.77863E+00_JPRB, 0.83642E+00_JPRB, 0.93833E+00_JPRB ,&
&  0.10391E+01_JPRB, 0.11915E+01_JPRB, 0.13736E+01_JPRB, 0.15794E+01_JPRB, 0.17152E+01_JPRB, 0.17432E+01_JPRB, 0.18298E+01_JPRB ,&
&  0.18848E+01_JPRB, 0.18945E+01_JPRB, 0.18560E+01_JPRB, 0.17873E+01_JPRB, 0.16826E+01_JPRB, 0.16721E+01_JPRB, 0.17367E+01_JPRB ,&
&  0.15541E+01_JPRB, 0.12481E+01_JPRB, 0.12122E+01_JPRB, 0.12373E+01_JPRB, 0.12677E+01_JPRB, 0.12883E+01_JPRB, 0.13300E+01_JPRB ,&
&  0.13655E+01_JPRB, 0.13934E+01_JPRB, 0.14230E+01_JPRB, 0.14582E+01_JPRB, 0.15246E+01_JPRB, 0.15616E+01_JPRB, 0.16158E+01_JPRB ,&
&  0.16542E+01_JPRB, 0.16997E+01_JPRB, 0.17909E+01_JPRB, 0.17018E+01_JPRB, 0.18455E+01_JPRB, 0.19538E+01_JPRB, 0.20517E+01_JPRB ,&
&  0.20838E+01_JPRB, 0.20624E+01_JPRB, 0.22171E+01_JPRB, 0.23381E+01_JPRB, 0.24341E+01_JPRB, 0.24649E+01_JPRB, 0.24063E+01_JPRB ,&
&  0.24952E+01_JPRB, 0.23253E+01_JPRB, 0.13197E+01_JPRB, 0.11612E+01_JPRB, 0.12163E+01_JPRB, 0.10806E+01_JPRB, 0.12252E+01_JPRB ,&
&  0.12122E+01_JPRB, 0.12216E+01_JPRB, 0.12407E+01_JPRB, 0.12951E+01_JPRB, 0.13521E+01_JPRB, 0.15241E+01_JPRB, 0.11164E+01_JPRB ,&
&  0.82109E+00_JPRB, 0.76816E+00_JPRB, 0.76177E+00_JPRB, 0.77834E+00_JPRB, 0.81012E+00_JPRB, 0.80871E+00_JPRB, 0.81720E+00_JPRB ,&
&  0.87174E+00_JPRB, 0.91116E+00_JPRB, 0.95051E+00_JPRB, 0.93987E+00_JPRB, 0.83192E+00_JPRB, 0.84118E+00_JPRB, 0.87081E+00_JPRB ,&
&  0.88955E+00_JPRB, 0.89498E+00_JPRB, 0.88404E+00_JPRB, 0.88246E+00_JPRB, 0.86945E+00_JPRB, 0.86260E+00_JPRB, 0.85526E+00_JPRB ,&
&  0.84372E+00_JPRB, 0.83981E+00_JPRB, 0.82717E+00_JPRB, 0.79352E+00_JPRB, 0.77138E+00_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.68715E+00_JPRB, 0.68939E+00_JPRB, 0.68513E+00_JPRB, 0.68516E+00_JPRB, 0.68635E+00_JPRB, 0.67111E+00_JPRB, 0.69054E+00_JPRB ,&
&  0.68371E+00_JPRB, 0.70230E+00_JPRB, 0.72271E+00_JPRB, 0.74361E+00_JPRB, 0.78621E+00_JPRB, 0.82993E+00_JPRB, 0.89040E+00_JPRB ,&
&  0.99373E+00_JPRB, 0.11618E+01_JPRB, 0.13728E+01_JPRB, 0.16325E+01_JPRB, 0.17006E+01_JPRB, 0.17359E+01_JPRB, 0.18219E+01_JPRB ,&
&  0.19354E+01_JPRB, 0.20209E+01_JPRB, 0.20607E+01_JPRB, 0.21121E+01_JPRB, 0.21445E+01_JPRB, 0.20555E+01_JPRB, 0.19422E+01_JPRB ,&
&  0.15756E+01_JPRB, 0.12232E+01_JPRB, 0.13264E+01_JPRB, 0.12609E+01_JPRB, 0.13213E+01_JPRB, 0.13757E+01_JPRB, 0.14172E+01_JPRB ,&
&  0.14605E+01_JPRB, 0.14900E+01_JPRB, 0.15318E+01_JPRB, 0.15825E+01_JPRB, 0.16565E+01_JPRB, 0.17284E+01_JPRB, 0.18146E+01_JPRB ,&
&  0.18723E+01_JPRB, 0.19346E+01_JPRB, 0.19910E+01_JPRB, 0.19633E+01_JPRB, 0.20706E+01_JPRB, 0.22260E+01_JPRB, 0.23664E+01_JPRB ,&
&  0.24783E+01_JPRB, 0.24539E+01_JPRB, 0.22089E+01_JPRB, 0.23065E+01_JPRB, 0.24800E+01_JPRB, 0.23862E+01_JPRB, 0.24760E+01_JPRB ,&
&  0.27196E+01_JPRB, 0.24110E+01_JPRB, 0.12413E+01_JPRB, 0.10784E+01_JPRB, 0.11180E+01_JPRB, 0.11389E+01_JPRB, 0.11575E+01_JPRB ,&
&  0.11536E+01_JPRB, 0.11604E+01_JPRB, 0.11861E+01_JPRB, 0.12088E+01_JPRB, 0.11768E+01_JPRB, 0.11705E+01_JPRB, 0.10057E+01_JPRB ,&
&  0.73713E+00_JPRB, 0.73000E+00_JPRB, 0.75932E+00_JPRB, 0.78346E+00_JPRB, 0.81452E+00_JPRB, 0.79567E+00_JPRB, 0.76656E+00_JPRB ,&
&  0.82497E+00_JPRB, 0.88838E+00_JPRB, 0.99730E+00_JPRB, 0.91435E+00_JPRB, 0.74463E+00_JPRB, 0.77348E+00_JPRB, 0.77625E+00_JPRB ,&
&  0.79689E+00_JPRB, 0.79821E+00_JPRB, 0.80856E+00_JPRB, 0.81290E+00_JPRB, 0.82103E+00_JPRB, 0.80589E+00_JPRB, 0.78828E+00_JPRB ,&
&  0.76817E+00_JPRB, 0.75079E+00_JPRB, 0.73622E+00_JPRB, 0.71295E+00_JPRB, 0.69573E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.62739E+00_JPRB, 0.59946E+00_JPRB, 0.57853E+00_JPRB, 0.58211E+00_JPRB, 0.58785E+00_JPRB, 0.57458E+00_JPRB, 0.60172E+00_JPRB ,&
&  0.57609E+00_JPRB, 0.59559E+00_JPRB, 0.60599E+00_JPRB, 0.60311E+00_JPRB, 0.63854E+00_JPRB, 0.67553E+00_JPRB, 0.72281E+00_JPRB ,&
&  0.78843E+00_JPRB, 0.89344E+00_JPRB, 0.10233E+01_JPRB, 0.11833E+01_JPRB, 0.13742E+01_JPRB, 0.14574E+01_JPRB, 0.15445E+01_JPRB ,&
&  0.16725E+01_JPRB, 0.18434E+01_JPRB, 0.20910E+01_JPRB, 0.23669E+01_JPRB, 0.26913E+01_JPRB, 0.25066E+01_JPRB, 0.21147E+01_JPRB ,&
&  0.15714E+01_JPRB, 0.12420E+01_JPRB, 0.11970E+01_JPRB, 0.11252E+01_JPRB, 0.11997E+01_JPRB, 0.13557E+01_JPRB, 0.14338E+01_JPRB ,&
&  0.14834E+01_JPRB, 0.15393E+01_JPRB, 0.16028E+01_JPRB, 0.16747E+01_JPRB, 0.17565E+01_JPRB, 0.18497E+01_JPRB, 0.19090E+01_JPRB ,&
&  0.19423E+01_JPRB, 0.19802E+01_JPRB, 0.20197E+01_JPRB, 0.21451E+01_JPRB, 0.22946E+01_JPRB, 0.24798E+01_JPRB, 0.25248E+01_JPRB ,&
&  0.25371E+01_JPRB, 0.24673E+01_JPRB, 0.21412E+01_JPRB, 0.21197E+01_JPRB, 0.21895E+01_JPRB, 0.22165E+01_JPRB, 0.22479E+01_JPRB ,&
&  0.24383E+01_JPRB, 0.19719E+01_JPRB, 0.12491E+01_JPRB, 0.12119E+01_JPRB, 0.11894E+01_JPRB, 0.11952E+01_JPRB, 0.10658E+01_JPRB ,&
&  0.10557E+01_JPRB, 0.10477E+01_JPRB, 0.10474E+01_JPRB, 0.10142E+01_JPRB, 0.10139E+01_JPRB, 0.10143E+01_JPRB, 0.97393E+00_JPRB ,&
&  0.85437E+00_JPRB, 0.82685E+00_JPRB, 0.77126E+00_JPRB, 0.71651E+00_JPRB, 0.78839E+00_JPRB, 0.72822E+00_JPRB, 0.77492E+00_JPRB ,&
&  0.74303E+00_JPRB, 0.78974E+00_JPRB, 0.71091E+00_JPRB, 0.86837E+00_JPRB, 0.73657E+00_JPRB, 0.70235E+00_JPRB, 0.69327E+00_JPRB ,&
&  0.71209E+00_JPRB, 0.72166E+00_JPRB, 0.73220E+00_JPRB, 0.72086E+00_JPRB, 0.73566E+00_JPRB, 0.76138E+00_JPRB, 0.75991E+00_JPRB ,&
&  0.71641E+00_JPRB, 0.70000E+00_JPRB, 0.67462E+00_JPRB, 0.64376E+00_JPRB, 0.63782E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.54552E+00_JPRB, 0.53981E+00_JPRB, 0.53893E+00_JPRB, 0.53658E+00_JPRB, 0.54498E+00_JPRB, 0.54541E+00_JPRB, 0.54455E+00_JPRB ,&
&  0.54237E+00_JPRB, 0.55074E+00_JPRB, 0.55628E+00_JPRB, 0.56585E+00_JPRB, 0.57724E+00_JPRB, 0.60560E+00_JPRB, 0.63778E+00_JPRB ,&
&  0.67613E+00_JPRB, 0.72084E+00_JPRB, 0.76983E+00_JPRB, 0.84006E+00_JPRB, 0.93825E+00_JPRB, 0.10039E+01_JPRB, 0.10769E+01_JPRB ,&
&  0.11717E+01_JPRB, 0.13160E+01_JPRB, 0.14711E+01_JPRB, 0.16779E+01_JPRB, 0.19111E+01_JPRB, 0.19326E+01_JPRB, 0.17841E+01_JPRB ,&
&  0.13391E+01_JPRB, 0.13258E+01_JPRB, 0.12509E+01_JPRB, 0.11449E+01_JPRB, 0.11686E+01_JPRB, 0.11482E+01_JPRB, 0.11833E+01_JPRB ,&
&  0.12354E+01_JPRB, 0.12496E+01_JPRB, 0.12542E+01_JPRB, 0.12804E+01_JPRB, 0.13038E+01_JPRB, 0.13283E+01_JPRB, 0.13231E+01_JPRB ,&
&  0.13241E+01_JPRB, 0.13818E+01_JPRB, 0.16033E+01_JPRB, 0.18868E+01_JPRB, 0.20896E+01_JPRB, 0.21650E+01_JPRB, 0.20614E+01_JPRB ,&
&  0.19124E+01_JPRB, 0.18194E+01_JPRB, 0.16977E+01_JPRB, 0.16732E+01_JPRB, 0.17250E+01_JPRB, 0.17493E+01_JPRB, 0.18571E+01_JPRB ,&
&  0.17528E+01_JPRB, 0.14765E+01_JPRB, 0.13540E+01_JPRB, 0.12700E+01_JPRB, 0.12027E+01_JPRB, 0.11308E+01_JPRB, 0.10581E+01_JPRB ,&
&  0.10499E+01_JPRB, 0.10027E+01_JPRB, 0.96274E+00_JPRB, 0.89878E+00_JPRB, 0.85129E+00_JPRB, 0.83364E+00_JPRB, 0.81710E+00_JPRB ,&
&  0.74965E+00_JPRB, 0.69739E+00_JPRB, 0.66461E+00_JPRB, 0.66672E+00_JPRB, 0.69255E+00_JPRB, 0.92320E+00_JPRB, 0.91671E+00_JPRB ,&
&  0.65649E+00_JPRB, 0.57380E+00_JPRB, 0.50579E+00_JPRB, 0.67358E+00_JPRB, 0.64013E+00_JPRB, 0.63505E+00_JPRB, 0.63903E+00_JPRB ,&
&  0.64412E+00_JPRB, 0.64187E+00_JPRB, 0.62201E+00_JPRB, 0.61834E+00_JPRB, 0.62662E+00_JPRB, 0.63317E+00_JPRB, 0.62182E+00_JPRB ,&
&  0.58546E+00_JPRB, 0.56813E+00_JPRB, 0.59162E+00_JPRB, 0.55534E+00_JPRB, 0.54879E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.50006E+00_JPRB, 0.49937E+00_JPRB, 0.51214E+00_JPRB, 0.52450E+00_JPRB, 0.53096E+00_JPRB, 0.53521E+00_JPRB, 0.54291E+00_JPRB ,&
&  0.54618E+00_JPRB, 0.55210E+00_JPRB, 0.55950E+00_JPRB, 0.57452E+00_JPRB, 0.59726E+00_JPRB, 0.63231E+00_JPRB, 0.67209E+00_JPRB ,&
&  0.72039E+00_JPRB, 0.77253E+00_JPRB, 0.83029E+00_JPRB, 0.89308E+00_JPRB, 0.97126E+00_JPRB, 0.10798E+01_JPRB, 0.12280E+01_JPRB ,&
&  0.13081E+01_JPRB, 0.14082E+01_JPRB, 0.15272E+01_JPRB, 0.16239E+01_JPRB, 0.16277E+01_JPRB, 0.15698E+01_JPRB, 0.12549E+01_JPRB ,&
&  0.14012E+01_JPRB, 0.13356E+01_JPRB, 0.12198E+01_JPRB, 0.11074E+01_JPRB, 0.10305E+01_JPRB, 0.96946E+00_JPRB, 0.92417E+00_JPRB ,&
&  0.91429E+00_JPRB, 0.88377E+00_JPRB, 0.83877E+00_JPRB, 0.80476E+00_JPRB, 0.79027E+00_JPRB, 0.81444E+00_JPRB, 0.80531E+00_JPRB ,&
&  0.88936E+00_JPRB, 0.10680E+01_JPRB, 0.12913E+01_JPRB, 0.15603E+01_JPRB, 0.17098E+01_JPRB, 0.16822E+01_JPRB, 0.14215E+01_JPRB ,&
&  0.12661E+01_JPRB, 0.11978E+01_JPRB, 0.11682E+01_JPRB, 0.12269E+01_JPRB, 0.12391E+01_JPRB, 0.12953E+01_JPRB, 0.13180E+01_JPRB ,&
&  0.12923E+01_JPRB, 0.11634E+01_JPRB, 0.11624E+01_JPRB, 0.13756E+01_JPRB, 0.12434E+01_JPRB, 0.11080E+01_JPRB, 0.11032E+01_JPRB ,&
&  0.10259E+01_JPRB, 0.97288E+00_JPRB, 0.94641E+00_JPRB, 0.88770E+00_JPRB, 0.82650E+00_JPRB, 0.78605E+00_JPRB, 0.76087E+00_JPRB ,&
&  0.69925E+00_JPRB, 0.63430E+00_JPRB, 0.58680E+00_JPRB, 0.58079E+00_JPRB, 0.63757E+00_JPRB, 0.68107E+00_JPRB, 0.91542E+00_JPRB ,&
&  0.61849E+00_JPRB, 0.43513E+00_JPRB, 0.48475E+00_JPRB, 0.56190E+00_JPRB, 0.55009E+00_JPRB, 0.57106E+00_JPRB, 0.63303E+00_JPRB ,&
&  0.70295E+00_JPRB, 0.77096E+00_JPRB, 0.76874E+00_JPRB, 0.66060E+00_JPRB, 0.61730E+00_JPRB, 0.59455E+00_JPRB, 0.56903E+00_JPRB ,&
&  0.54888E+00_JPRB, 0.51245E+00_JPRB, 0.52429E+00_JPRB, 0.50836E+00_JPRB, 0.50502E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.47177E+00_JPRB, 0.46979E+00_JPRB, 0.47696E+00_JPRB, 0.48415E+00_JPRB, 0.48522E+00_JPRB, 0.48596E+00_JPRB, 0.48614E+00_JPRB ,&
&  0.48777E+00_JPRB, 0.48670E+00_JPRB, 0.48821E+00_JPRB, 0.49492E+00_JPRB, 0.50971E+00_JPRB, 0.54174E+00_JPRB, 0.57836E+00_JPRB ,&
&  0.60986E+00_JPRB, 0.63842E+00_JPRB, 0.66519E+00_JPRB, 0.68779E+00_JPRB, 0.71088E+00_JPRB, 0.72527E+00_JPRB, 0.77228E+00_JPRB ,&
&  0.83194E+00_JPRB, 0.90329E+00_JPRB, 0.97463E+00_JPRB, 0.10231E+01_JPRB, 0.10588E+01_JPRB, 0.10005E+01_JPRB, 0.79297E+00_JPRB ,&
&  0.12373E+01_JPRB, 0.12239E+01_JPRB, 0.11139E+01_JPRB, 0.10023E+01_JPRB, 0.92548E+00_JPRB, 0.85789E+00_JPRB, 0.78867E+00_JPRB ,&
&  0.72395E+00_JPRB, 0.74670E+00_JPRB, 0.72617E+00_JPRB, 0.76928E+00_JPRB, 0.73171E+00_JPRB, 0.78330E+00_JPRB, 0.92459E+00_JPRB ,&
&  0.10382E+01_JPRB, 0.11438E+01_JPRB, 0.12683E+01_JPRB, 0.13992E+01_JPRB, 0.14689E+01_JPRB, 0.14381E+01_JPRB, 0.12917E+01_JPRB ,&
&  0.12065E+01_JPRB, 0.11737E+01_JPRB, 0.11176E+01_JPRB, 0.11983E+01_JPRB, 0.11622E+01_JPRB, 0.10961E+01_JPRB, 0.10993E+01_JPRB ,&
&  0.97447E+00_JPRB, 0.98648E+00_JPRB, 0.11213E+01_JPRB, 0.13849E+01_JPRB, 0.12638E+01_JPRB, 0.11698E+01_JPRB, 0.11356E+01_JPRB ,&
&  0.10550E+01_JPRB, 0.98756E+00_JPRB, 0.94071E+00_JPRB, 0.88108E+00_JPRB, 0.82165E+00_JPRB, 0.77323E+00_JPRB, 0.72277E+00_JPRB ,&
&  0.67547E+00_JPRB, 0.61798E+00_JPRB, 0.57105E+00_JPRB, 0.54551E+00_JPRB, 0.54497E+00_JPRB, 0.52743E+00_JPRB, 0.78308E+00_JPRB ,&
&  0.63545E+00_JPRB, 0.52909E+00_JPRB, 0.56637E+00_JPRB, 0.37025E+00_JPRB, 0.45602E+00_JPRB, 0.50473E+00_JPRB, 0.62075E+00_JPRB ,&
&  0.78763E+00_JPRB, 0.12699E+01_JPRB, 0.19990E+01_JPRB, 0.18056E+01_JPRB, 0.13736E+01_JPRB, 0.87568E+00_JPRB, 0.51577E+00_JPRB ,&
&  0.49494E+00_JPRB, 0.49022E+00_JPRB, 0.49439E+00_JPRB, 0.48437E+00_JPRB, 0.48005E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.43102E+00_JPRB, 0.42163E+00_JPRB, 0.42892E+00_JPRB, 0.43800E+00_JPRB, 0.43365E+00_JPRB, 0.43264E+00_JPRB, 0.42741E+00_JPRB ,&
&  0.41788E+00_JPRB, 0.41178E+00_JPRB, 0.43315E+00_JPRB, 0.47353E+00_JPRB, 0.52898E+00_JPRB, 0.60029E+00_JPRB, 0.66909E+00_JPRB ,&
&  0.71026E+00_JPRB, 0.75592E+00_JPRB, 0.79288E+00_JPRB, 0.82132E+00_JPRB, 0.81595E+00_JPRB, 0.80867E+00_JPRB, 0.78748E+00_JPRB ,&
&  0.77126E+00_JPRB, 0.77535E+00_JPRB, 0.80561E+00_JPRB, 0.83017E+00_JPRB, 0.88225E+00_JPRB, 0.94287E+00_JPRB, 0.62855E+00_JPRB ,&
&  0.10226E+01_JPRB, 0.10320E+01_JPRB, 0.98181E+00_JPRB, 0.89180E+00_JPRB, 0.84571E+00_JPRB, 0.77388E+00_JPRB, 0.71261E+00_JPRB ,&
&  0.71011E+00_JPRB, 0.74669E+00_JPRB, 0.78265E+00_JPRB, 0.91201E+00_JPRB, 0.10559E+01_JPRB, 0.11286E+01_JPRB, 0.12807E+01_JPRB ,&
&  0.13975E+01_JPRB, 0.15432E+01_JPRB, 0.16934E+01_JPRB, 0.17985E+01_JPRB, 0.18888E+01_JPRB, 0.19271E+01_JPRB, 0.19138E+01_JPRB ,&
&  0.17924E+01_JPRB, 0.17098E+01_JPRB, 0.17503E+01_JPRB, 0.18624E+01_JPRB, 0.16449E+01_JPRB, 0.15452E+01_JPRB, 0.13097E+01_JPRB ,&
&  0.99938E+00_JPRB, 0.92937E+00_JPRB, 0.10576E+01_JPRB, 0.12985E+01_JPRB, 0.12805E+01_JPRB, 0.12086E+01_JPRB, 0.11096E+01_JPRB ,&
&  0.10260E+01_JPRB, 0.94012E+00_JPRB, 0.87813E+00_JPRB, 0.82261E+00_JPRB, 0.77629E+00_JPRB, 0.73331E+00_JPRB, 0.68794E+00_JPRB ,&
&  0.66064E+00_JPRB, 0.62123E+00_JPRB, 0.57669E+00_JPRB, 0.56377E+00_JPRB, 0.56574E+00_JPRB, 0.60773E+00_JPRB, 0.63418E+00_JPRB ,&
&  0.74206E+00_JPRB, 0.58752E+00_JPRB, 0.52632E+00_JPRB, 0.51450E+00_JPRB, 0.45810E+00_JPRB, 0.50247E+00_JPRB, 0.67294E+00_JPRB ,&
&  0.10804E+01_JPRB, 0.16216E+01_JPRB, 0.27695E+01_JPRB, 0.49996E+01_JPRB, 0.27606E+01_JPRB, 0.21251E+01_JPRB, 0.50056E+00_JPRB ,&
&  0.47848E+00_JPRB, 0.48041E+00_JPRB, 0.47723E+00_JPRB, 0.46007E+00_JPRB, 0.45486E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.42152E+00_JPRB, 0.41288E+00_JPRB, 0.42060E+00_JPRB, 0.41866E+00_JPRB, 0.42192E+00_JPRB, 0.42846E+00_JPRB, 0.43252E+00_JPRB ,&
&  0.43592E+00_JPRB, 0.45226E+00_JPRB, 0.53976E+00_JPRB, 0.65935E+00_JPRB, 0.77110E+00_JPRB, 0.85458E+00_JPRB, 0.88827E+00_JPRB ,&
&  0.91749E+00_JPRB, 0.92197E+00_JPRB, 0.89826E+00_JPRB, 0.86928E+00_JPRB, 0.86489E+00_JPRB, 0.89676E+00_JPRB, 0.92014E+00_JPRB ,&
&  0.93977E+00_JPRB, 0.95647E+00_JPRB, 0.97251E+00_JPRB, 0.10015E+01_JPRB, 0.10297E+01_JPRB, 0.96308E+00_JPRB, 0.60325E+00_JPRB ,&
&  0.86786E+00_JPRB, 0.91951E+00_JPRB, 0.92284E+00_JPRB, 0.85255E+00_JPRB, 0.79214E+00_JPRB, 0.75604E+00_JPRB, 0.80487E+00_JPRB ,&
&  0.88229E+00_JPRB, 0.93678E+00_JPRB, 0.97671E+00_JPRB, 0.10498E+01_JPRB, 0.14590E+01_JPRB, 0.16007E+01_JPRB, 0.18366E+01_JPRB ,&
&  0.20120E+01_JPRB, 0.21993E+01_JPRB, 0.23317E+01_JPRB, 0.24850E+01_JPRB, 0.26141E+01_JPRB, 0.27022E+01_JPRB, 0.28261E+01_JPRB ,&
&  0.28672E+01_JPRB, 0.29293E+01_JPRB, 0.29274E+01_JPRB, 0.28358E+01_JPRB, 0.23286E+01_JPRB, 0.24751E+01_JPRB, 0.17324E+01_JPRB ,&
&  0.12706E+01_JPRB, 0.11186E+01_JPRB, 0.12450E+01_JPRB, 0.13023E+01_JPRB, 0.12302E+01_JPRB, 0.11346E+01_JPRB, 0.10128E+01_JPRB ,&
&  0.93535E+00_JPRB, 0.87219E+00_JPRB, 0.81924E+00_JPRB, 0.77162E+00_JPRB, 0.75377E+00_JPRB, 0.73199E+00_JPRB, 0.70944E+00_JPRB ,&
&  0.70144E+00_JPRB, 0.66848E+00_JPRB, 0.62635E+00_JPRB, 0.62388E+00_JPRB, 0.64551E+00_JPRB, 0.74507E+00_JPRB, 0.97294E+00_JPRB ,&
&  0.10923E+01_JPRB, 0.65373E+00_JPRB, 0.61115E+00_JPRB, 0.57682E+00_JPRB, 0.56821E+00_JPRB, 0.60116E+00_JPRB, 0.68784E+00_JPRB ,&
&  0.90420E+00_JPRB, 0.16101E+01_JPRB, 0.25036E+01_JPRB, 0.19122E+01_JPRB, 0.21876E+01_JPRB, 0.15003E+01_JPRB, 0.60111E+00_JPRB ,&
&  0.46430E+00_JPRB, 0.45463E+00_JPRB, 0.45989E+00_JPRB, 0.44975E+00_JPRB, 0.43679E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.47274E+00_JPRB, 0.46160E+00_JPRB, 0.46071E+00_JPRB, 0.47077E+00_JPRB, 0.47705E+00_JPRB, 0.50094E+00_JPRB, 0.53099E+00_JPRB ,&
&  0.54594E+00_JPRB, 0.61435E+00_JPRB, 0.76703E+00_JPRB, 0.89659E+00_JPRB, 0.94767E+00_JPRB, 0.98370E+00_JPRB, 0.93173E+00_JPRB ,&
&  0.91400E+00_JPRB, 0.87766E+00_JPRB, 0.82292E+00_JPRB, 0.80884E+00_JPRB, 0.81761E+00_JPRB, 0.84220E+00_JPRB, 0.87558E+00_JPRB ,&
&  0.91385E+00_JPRB, 0.95427E+00_JPRB, 0.98617E+00_JPRB, 0.10158E+01_JPRB, 0.10561E+01_JPRB, 0.10313E+01_JPRB, 0.95850E+00_JPRB ,&
&  0.51536E+00_JPRB, 0.85751E+00_JPRB, 0.86436E+00_JPRB, 0.81673E+00_JPRB, 0.77650E+00_JPRB, 0.78878E+00_JPRB, 0.91416E+00_JPRB ,&
&  0.10506E+01_JPRB, 0.11014E+01_JPRB, 0.12582E+01_JPRB, 0.13724E+01_JPRB, 0.17565E+01_JPRB, 0.19812E+01_JPRB, 0.21842E+01_JPRB ,&
&  0.23696E+01_JPRB, 0.25090E+01_JPRB, 0.26609E+01_JPRB, 0.28132E+01_JPRB, 0.29638E+01_JPRB, 0.30889E+01_JPRB, 0.32441E+01_JPRB ,&
&  0.34226E+01_JPRB, 0.36715E+01_JPRB, 0.39201E+01_JPRB, 0.37659E+01_JPRB, 0.32022E+01_JPRB, 0.30109E+01_JPRB, 0.24404E+01_JPRB ,&
&  0.16971E+01_JPRB, 0.11423E+01_JPRB, 0.11879E+01_JPRB, 0.12685E+01_JPRB, 0.11895E+01_JPRB, 0.11012E+01_JPRB, 0.10165E+01_JPRB ,&
&  0.94853E+00_JPRB, 0.87247E+00_JPRB, 0.78107E+00_JPRB, 0.77337E+00_JPRB, 0.78867E+00_JPRB, 0.78121E+00_JPRB, 0.76737E+00_JPRB ,&
&  0.77019E+00_JPRB, 0.75152E+00_JPRB, 0.72792E+00_JPRB, 0.70332E+00_JPRB, 0.70229E+00_JPRB, 0.73229E+00_JPRB, 0.75658E+00_JPRB ,&
&  0.78447E+00_JPRB, 0.72427E+00_JPRB, 0.68190E+00_JPRB, 0.68793E+00_JPRB, 0.69259E+00_JPRB, 0.70354E+00_JPRB, 0.70375E+00_JPRB ,&
&  0.75833E+00_JPRB, 0.91719E+00_JPRB, 0.13075E+01_JPRB, 0.17034E+01_JPRB, 0.13865E+01_JPRB, 0.10928E+01_JPRB, 0.61877E+00_JPRB ,&
&  0.49164E+00_JPRB, 0.47501E+00_JPRB, 0.49294E+00_JPRB, 0.49639E+00_JPRB, 0.49507E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.57924E+00_JPRB, 0.56690E+00_JPRB, 0.55060E+00_JPRB, 0.56680E+00_JPRB, 0.60011E+00_JPRB, 0.64176E+00_JPRB, 0.65937E+00_JPRB ,&
&  0.68209E+00_JPRB, 0.78043E+00_JPRB, 0.91039E+00_JPRB, 0.10247E+01_JPRB, 0.10422E+01_JPRB, 0.10083E+01_JPRB, 0.90602E+00_JPRB ,&
&  0.87584E+00_JPRB, 0.84585E+00_JPRB, 0.82656E+00_JPRB, 0.80570E+00_JPRB, 0.80247E+00_JPRB, 0.82315E+00_JPRB, 0.86242E+00_JPRB ,&
&  0.91345E+00_JPRB, 0.96752E+00_JPRB, 0.10293E+01_JPRB, 0.10929E+01_JPRB, 0.11444E+01_JPRB, 0.11436E+01_JPRB, 0.10266E+01_JPRB ,&
&  0.44477E+00_JPRB, 0.55395E+00_JPRB, 0.82459E+00_JPRB, 0.83024E+00_JPRB, 0.81833E+00_JPRB, 0.85065E+00_JPRB, 0.96518E+00_JPRB ,&
&  0.10829E+01_JPRB, 0.11480E+01_JPRB, 0.13240E+01_JPRB, 0.16556E+01_JPRB, 0.18335E+01_JPRB, 0.20745E+01_JPRB, 0.22130E+01_JPRB ,&
&  0.23516E+01_JPRB, 0.24953E+01_JPRB, 0.26566E+01_JPRB, 0.28188E+01_JPRB, 0.29819E+01_JPRB, 0.31321E+01_JPRB, 0.33305E+01_JPRB ,&
&  0.36006E+01_JPRB, 0.38880E+01_JPRB, 0.41315E+01_JPRB, 0.37298E+01_JPRB, 0.39583E+01_JPRB, 0.34844E+01_JPRB, 0.30162E+01_JPRB ,&
&  0.18417E+01_JPRB, 0.11967E+01_JPRB, 0.12191E+01_JPRB, 0.13163E+01_JPRB, 0.11786E+01_JPRB, 0.10547E+01_JPRB, 0.93962E+00_JPRB ,&
&  0.87661E+00_JPRB, 0.80310E+00_JPRB, 0.80670E+00_JPRB, 0.81502E+00_JPRB, 0.80353E+00_JPRB, 0.83965E+00_JPRB, 0.80111E+00_JPRB ,&
&  0.81409E+00_JPRB, 0.82472E+00_JPRB, 0.81695E+00_JPRB, 0.82061E+00_JPRB, 0.81664E+00_JPRB, 0.81375E+00_JPRB, 0.79805E+00_JPRB ,&
&  0.80071E+00_JPRB, 0.79384E+00_JPRB, 0.79997E+00_JPRB, 0.81320E+00_JPRB, 0.81414E+00_JPRB, 0.80450E+00_JPRB, 0.76410E+00_JPRB ,&
&  0.72481E+00_JPRB, 0.74113E+00_JPRB, 0.84620E+00_JPRB, 0.98119E+00_JPRB, 0.94521E+00_JPRB, 0.81505E+00_JPRB, 0.64436E+00_JPRB ,&
&  0.62966E+00_JPRB, 0.65060E+00_JPRB, 0.65293E+00_JPRB, 0.63311E+00_JPRB, 0.60842E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.68919E+00_JPRB, 0.66331E+00_JPRB, 0.62532E+00_JPRB, 0.61950E+00_JPRB, 0.67536E+00_JPRB, 0.72306E+00_JPRB, 0.72513E+00_JPRB ,&
&  0.73208E+00_JPRB, 0.79507E+00_JPRB, 0.90954E+00_JPRB, 0.10002E+01_JPRB, 0.10286E+01_JPRB, 0.10195E+01_JPRB, 0.92355E+00_JPRB ,&
&  0.88547E+00_JPRB, 0.86050E+00_JPRB, 0.83249E+00_JPRB, 0.81840E+00_JPRB, 0.81147E+00_JPRB, 0.81345E+00_JPRB, 0.82785E+00_JPRB ,&
&  0.84930E+00_JPRB, 0.89348E+00_JPRB, 0.97269E+00_JPRB, 0.10817E+01_JPRB, 0.12459E+01_JPRB, 0.14316E+01_JPRB, 0.15899E+01_JPRB ,&
&  0.11784E+01_JPRB, 0.53057E+00_JPRB, 0.60554E+00_JPRB, 0.86870E+00_JPRB, 0.85061E+00_JPRB, 0.90456E+00_JPRB, 0.92079E+00_JPRB ,&
&  0.99685E+00_JPRB, 0.11554E+01_JPRB, 0.13245E+01_JPRB, 0.16228E+01_JPRB, 0.18137E+01_JPRB, 0.20068E+01_JPRB, 0.20955E+01_JPRB ,&
&  0.21903E+01_JPRB, 0.23037E+01_JPRB, 0.24173E+01_JPRB, 0.25352E+01_JPRB, 0.26669E+01_JPRB, 0.28168E+01_JPRB, 0.30439E+01_JPRB ,&
&  0.33304E+01_JPRB, 0.36006E+01_JPRB, 0.37955E+01_JPRB, 0.32674E+01_JPRB, 0.34474E+01_JPRB, 0.27604E+01_JPRB, 0.21203E+01_JPRB ,&
&  0.16975E+01_JPRB, 0.14032E+01_JPRB, 0.13110E+01_JPRB, 0.13643E+01_JPRB, 0.11886E+01_JPRB, 0.93805E+00_JPRB, 0.93743E+00_JPRB ,&
&  0.85619E+00_JPRB, 0.82914E+00_JPRB, 0.82764E+00_JPRB, 0.77310E+00_JPRB, 0.82846E+00_JPRB, 0.84930E+00_JPRB, 0.86227E+00_JPRB ,&
&  0.84424E+00_JPRB, 0.86231E+00_JPRB, 0.88615E+00_JPRB, 0.92419E+00_JPRB, 0.91084E+00_JPRB, 0.90543E+00_JPRB, 0.88578E+00_JPRB ,&
&  0.88731E+00_JPRB, 0.89210E+00_JPRB, 0.90416E+00_JPRB, 0.92398E+00_JPRB, 0.92806E+00_JPRB, 0.85708E+00_JPRB, 0.80872E+00_JPRB ,&
&  0.78662E+00_JPRB, 0.77958E+00_JPRB, 0.75440E+00_JPRB, 0.80052E+00_JPRB, 0.81420E+00_JPRB, 0.82555E+00_JPRB, 0.85866E+00_JPRB ,&
&  0.87238E+00_JPRB, 0.87312E+00_JPRB, 0.82385E+00_JPRB, 0.76786E+00_JPRB, 0.73157E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.74636E+00_JPRB, 0.70296E+00_JPRB, 0.67119E+00_JPRB, 0.68165E+00_JPRB, 0.73129E+00_JPRB, 0.78274E+00_JPRB, 0.78160E+00_JPRB ,&
&  0.76766E+00_JPRB, 0.78758E+00_JPRB, 0.85584E+00_JPRB, 0.91437E+00_JPRB, 0.92257E+00_JPRB, 0.93198E+00_JPRB, 0.95025E+00_JPRB ,&
&  0.89830E+00_JPRB, 0.86496E+00_JPRB, 0.84802E+00_JPRB, 0.82792E+00_JPRB, 0.82624E+00_JPRB, 0.81939E+00_JPRB, 0.81948E+00_JPRB ,&
&  0.83131E+00_JPRB, 0.84435E+00_JPRB, 0.88387E+00_JPRB, 0.94690E+00_JPRB, 0.10591E+01_JPRB, 0.13025E+01_JPRB, 0.16730E+01_JPRB ,&
&  0.22998E+01_JPRB, 0.15211E+01_JPRB, 0.60894E+00_JPRB, 0.81470E+00_JPRB, 0.90220E+00_JPRB, 0.95162E+00_JPRB, 0.87869E+00_JPRB ,&
&  0.90871E+00_JPRB, 0.99372E+00_JPRB, 0.11616E+01_JPRB, 0.14857E+01_JPRB, 0.16710E+01_JPRB, 0.18146E+01_JPRB, 0.19022E+01_JPRB ,&
&  0.19399E+01_JPRB, 0.20097E+01_JPRB, 0.20687E+01_JPRB, 0.21203E+01_JPRB, 0.21820E+01_JPRB, 0.22757E+01_JPRB, 0.24372E+01_JPRB ,&
&  0.27456E+01_JPRB, 0.31217E+01_JPRB, 0.34440E+01_JPRB, 0.29965E+01_JPRB, 0.30120E+01_JPRB, 0.23624E+01_JPRB, 0.19926E+01_JPRB ,&
&  0.17510E+01_JPRB, 0.17435E+01_JPRB, 0.16924E+01_JPRB, 0.14489E+01_JPRB, 0.11960E+01_JPRB, 0.10373E+01_JPRB, 0.10822E+01_JPRB ,&
&  0.93771E+00_JPRB, 0.86221E+00_JPRB, 0.83530E+00_JPRB, 0.85664E+00_JPRB, 0.87988E+00_JPRB, 0.88776E+00_JPRB, 0.88506E+00_JPRB ,&
&  0.89713E+00_JPRB, 0.89718E+00_JPRB, 0.93681E+00_JPRB, 0.96294E+00_JPRB, 0.98166E+00_JPRB, 0.10185E+01_JPRB, 0.10161E+01_JPRB ,&
&  0.10015E+01_JPRB, 0.99209E+00_JPRB, 0.99616E+00_JPRB, 0.10092E+01_JPRB, 0.95852E+00_JPRB, 0.88115E+00_JPRB, 0.87383E+00_JPRB ,&
&  0.86862E+00_JPRB, 0.86657E+00_JPRB, 0.83593E+00_JPRB, 0.84901E+00_JPRB, 0.85991E+00_JPRB, 0.92248E+00_JPRB, 0.99732E+00_JPRB ,&
&  0.10305E+01_JPRB, 0.10037E+01_JPRB, 0.93615E+00_JPRB, 0.80914E+00_JPRB, 0.79447E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.79821E+00_JPRB, 0.78742E+00_JPRB, 0.76191E+00_JPRB, 0.74466E+00_JPRB, 0.76726E+00_JPRB, 0.80166E+00_JPRB, 0.80082E+00_JPRB ,&
&  0.77681E+00_JPRB, 0.76379E+00_JPRB, 0.79875E+00_JPRB, 0.83524E+00_JPRB, 0.83262E+00_JPRB, 0.84916E+00_JPRB, 0.85816E+00_JPRB ,&
&  0.88990E+00_JPRB, 0.88270E+00_JPRB, 0.86243E+00_JPRB, 0.83517E+00_JPRB, 0.83210E+00_JPRB, 0.81882E+00_JPRB, 0.81754E+00_JPRB ,&
&  0.82614E+00_JPRB, 0.83642E+00_JPRB, 0.85736E+00_JPRB, 0.89139E+00_JPRB, 0.92992E+00_JPRB, 0.10248E+01_JPRB, 0.12231E+01_JPRB ,&
&  0.14942E+01_JPRB, 0.19442E+01_JPRB, 0.74013E+00_JPRB, 0.94639E+00_JPRB, 0.11260E+01_JPRB, 0.10755E+01_JPRB, 0.98181E+00_JPRB ,&
&  0.92215E+00_JPRB, 0.86714E+00_JPRB, 0.96114E+00_JPRB, 0.12095E+01_JPRB, 0.13953E+01_JPRB, 0.15216E+01_JPRB, 0.15324E+01_JPRB ,&
&  0.15744E+01_JPRB, 0.16485E+01_JPRB, 0.17720E+01_JPRB, 0.18048E+01_JPRB, 0.18052E+01_JPRB, 0.17955E+01_JPRB, 0.18516E+01_JPRB ,&
&  0.20653E+01_JPRB, 0.24316E+01_JPRB, 0.29750E+01_JPRB, 0.27719E+01_JPRB, 0.27742E+01_JPRB, 0.24509E+01_JPRB, 0.21827E+01_JPRB ,&
&  0.19740E+01_JPRB, 0.19936E+01_JPRB, 0.19149E+01_JPRB, 0.16161E+01_JPRB, 0.13065E+01_JPRB, 0.11625E+01_JPRB, 0.11667E+01_JPRB ,&
&  0.10281E+01_JPRB, 0.93824E+00_JPRB, 0.92090E+00_JPRB, 0.92353E+00_JPRB, 0.96703E+00_JPRB, 0.95481E+00_JPRB, 0.93111E+00_JPRB ,&
&  0.94207E+00_JPRB, 0.92517E+00_JPRB, 0.94191E+00_JPRB, 0.97381E+00_JPRB, 0.10065E+01_JPRB, 0.10603E+01_JPRB, 0.10846E+01_JPRB ,&
&  0.10775E+01_JPRB, 0.10794E+01_JPRB, 0.10661E+01_JPRB, 0.10169E+01_JPRB, 0.94939E+00_JPRB, 0.91013E+00_JPRB, 0.90239E+00_JPRB ,&
&  0.91357E+00_JPRB, 0.92242E+00_JPRB, 0.89066E+00_JPRB, 0.85009E+00_JPRB, 0.88520E+00_JPRB, 0.90371E+00_JPRB, 0.99746E+00_JPRB ,&
&  0.11077E+01_JPRB, 0.11802E+01_JPRB, 0.11376E+01_JPRB, 0.86483E+00_JPRB, 0.85234E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.87850E+00_JPRB, 0.84502E+00_JPRB, 0.82774E+00_JPRB, 0.81005E+00_JPRB, 0.79364E+00_JPRB, 0.82539E+00_JPRB, 0.81679E+00_JPRB ,&
&  0.79297E+00_JPRB, 0.76576E+00_JPRB, 0.78394E+00_JPRB, 0.82995E+00_JPRB, 0.83192E+00_JPRB, 0.81144E+00_JPRB, 0.79705E+00_JPRB ,&
&  0.79589E+00_JPRB, 0.81239E+00_JPRB, 0.81558E+00_JPRB, 0.82776E+00_JPRB, 0.82917E+00_JPRB, 0.81936E+00_JPRB, 0.81291E+00_JPRB ,&
&  0.82873E+00_JPRB, 0.83878E+00_JPRB, 0.84915E+00_JPRB, 0.85567E+00_JPRB, 0.87091E+00_JPRB, 0.87552E+00_JPRB, 0.96167E+00_JPRB ,&
&  0.10884E+01_JPRB, 0.16734E+01_JPRB, 0.13856E+01_JPRB, 0.16353E+01_JPRB, 0.15242E+01_JPRB, 0.13698E+01_JPRB, 0.11615E+01_JPRB ,&
&  0.11260E+01_JPRB, 0.10468E+01_JPRB, 0.11170E+01_JPRB, 0.10283E+01_JPRB, 0.10732E+01_JPRB, 0.11578E+01_JPRB, 0.11598E+01_JPRB ,&
&  0.11428E+01_JPRB, 0.11374E+01_JPRB, 0.13021E+01_JPRB, 0.13794E+01_JPRB, 0.14142E+01_JPRB, 0.14221E+01_JPRB, 0.14357E+01_JPRB ,&
&  0.15436E+01_JPRB, 0.17344E+01_JPRB, 0.22457E+01_JPRB, 0.26231E+01_JPRB, 0.25617E+01_JPRB, 0.25346E+01_JPRB, 0.24921E+01_JPRB ,&
&  0.25318E+01_JPRB, 0.23207E+01_JPRB, 0.20904E+01_JPRB, 0.17551E+01_JPRB, 0.14177E+01_JPRB, 0.13850E+01_JPRB, 0.12183E+01_JPRB ,&
&  0.10843E+01_JPRB, 0.10005E+01_JPRB, 0.99225E+00_JPRB, 0.10062E+01_JPRB, 0.10332E+01_JPRB, 0.10143E+01_JPRB, 0.97607E+00_JPRB ,&
&  0.95953E+00_JPRB, 0.96323E+00_JPRB, 0.94643E+00_JPRB, 0.96315E+00_JPRB, 0.97760E+00_JPRB, 0.10095E+01_JPRB, 0.10378E+01_JPRB ,&
&  0.10725E+01_JPRB, 0.10957E+01_JPRB, 0.10531E+01_JPRB, 0.10111E+01_JPRB, 0.95970E+00_JPRB, 0.92718E+00_JPRB, 0.89722E+00_JPRB ,&
&  0.90785E+00_JPRB, 0.93166E+00_JPRB, 0.92936E+00_JPRB, 0.89409E+00_JPRB, 0.87398E+00_JPRB, 0.86948E+00_JPRB, 0.88185E+00_JPRB ,&
&  0.90851E+00_JPRB, 0.96565E+00_JPRB, 0.96873E+00_JPRB, 0.91533E+00_JPRB, 0.90246E+00_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.92325E+00_JPRB, 0.90922E+00_JPRB, 0.87217E+00_JPRB, 0.86777E+00_JPRB, 0.84202E+00_JPRB, 0.86937E+00_JPRB, 0.87464E+00_JPRB ,&
&  0.84082E+00_JPRB, 0.80899E+00_JPRB, 0.78927E+00_JPRB, 0.80938E+00_JPRB, 0.83084E+00_JPRB, 0.81760E+00_JPRB, 0.77783E+00_JPRB ,&
&  0.76477E+00_JPRB, 0.76244E+00_JPRB, 0.76210E+00_JPRB, 0.76624E+00_JPRB, 0.78293E+00_JPRB, 0.80678E+00_JPRB, 0.80484E+00_JPRB ,&
&  0.80184E+00_JPRB, 0.83256E+00_JPRB, 0.86022E+00_JPRB, 0.85983E+00_JPRB, 0.85006E+00_JPRB, 0.83800E+00_JPRB, 0.87243E+00_JPRB ,&
&  0.92255E+00_JPRB, 0.17640E+01_JPRB, 0.13397E+01_JPRB, 0.21031E+01_JPRB, 0.16748E+01_JPRB, 0.14688E+01_JPRB, 0.11932E+01_JPRB ,&
&  0.10954E+01_JPRB, 0.11854E+01_JPRB, 0.10616E+01_JPRB, 0.95506E+00_JPRB, 0.90307E+00_JPRB, 0.90623E+00_JPRB, 0.89047E+00_JPRB ,&
&  0.84121E+00_JPRB, 0.85684E+00_JPRB, 0.91207E+00_JPRB, 0.99900E+00_JPRB, 0.10617E+01_JPRB, 0.10881E+01_JPRB, 0.11270E+01_JPRB ,&
&  0.11920E+01_JPRB, 0.12783E+01_JPRB, 0.16126E+01_JPRB, 0.20452E+01_JPRB, 0.24513E+01_JPRB, 0.26189E+01_JPRB, 0.28380E+01_JPRB ,&
&  0.29224E+01_JPRB, 0.27464E+01_JPRB, 0.23027E+01_JPRB, 0.18893E+01_JPRB, 0.15937E+01_JPRB, 0.14849E+01_JPRB, 0.13679E+01_JPRB ,&
&  0.12403E+01_JPRB, 0.11203E+01_JPRB, 0.10953E+01_JPRB, 0.10819E+01_JPRB, 0.11133E+01_JPRB, 0.10986E+01_JPRB, 0.10332E+01_JPRB ,&
&  0.97719E+00_JPRB, 0.96067E+00_JPRB, 0.96513E+00_JPRB, 0.97986E+00_JPRB, 0.99786E+00_JPRB, 0.99616E+00_JPRB, 0.99672E+00_JPRB ,&
&  0.10266E+01_JPRB, 0.10689E+01_JPRB, 0.10476E+01_JPRB, 0.10379E+01_JPRB, 0.99930E+00_JPRB, 0.96710E+00_JPRB, 0.94901E+00_JPRB ,&
&  0.97444E+00_JPRB, 0.96544E+00_JPRB, 0.92966E+00_JPRB, 0.88734E+00_JPRB, 0.87018E+00_JPRB, 0.87991E+00_JPRB, 0.85373E+00_JPRB ,&
&  0.86018E+00_JPRB, 0.90735E+00_JPRB, 0.96000E+00_JPRB, 0.92402E+00_JPRB, 0.91963E+00_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.93512E+00_JPRB, 0.93529E+00_JPRB, 0.88950E+00_JPRB, 0.87821E+00_JPRB, 0.85771E+00_JPRB, 0.85540E+00_JPRB, 0.86864E+00_JPRB ,&
&  0.86145E+00_JPRB, 0.83735E+00_JPRB, 0.80649E+00_JPRB, 0.79383E+00_JPRB, 0.80550E+00_JPRB, 0.81752E+00_JPRB, 0.78480E+00_JPRB ,&
&  0.74706E+00_JPRB, 0.73433E+00_JPRB, 0.72755E+00_JPRB, 0.73076E+00_JPRB, 0.74225E+00_JPRB, 0.75999E+00_JPRB, 0.76985E+00_JPRB ,&
&  0.76575E+00_JPRB, 0.80381E+00_JPRB, 0.84603E+00_JPRB, 0.86046E+00_JPRB, 0.84407E+00_JPRB, 0.83256E+00_JPRB, 0.84716E+00_JPRB ,&
&  0.93745E+00_JPRB, 0.20765E+01_JPRB, 0.14137E+01_JPRB, 0.21483E+01_JPRB, 0.17116E+01_JPRB, 0.14584E+01_JPRB, 0.11811E+01_JPRB ,&
&  0.11079E+01_JPRB, 0.11057E+01_JPRB, 0.10300E+01_JPRB, 0.94708E+00_JPRB, 0.88950E+00_JPRB, 0.85306E+00_JPRB, 0.82562E+00_JPRB ,&
&  0.80464E+00_JPRB, 0.79597E+00_JPRB, 0.79957E+00_JPRB, 0.85184E+00_JPRB, 0.89700E+00_JPRB, 0.91953E+00_JPRB, 0.93320E+00_JPRB ,&
&  0.96777E+00_JPRB, 0.10243E+01_JPRB, 0.12229E+01_JPRB, 0.17806E+01_JPRB, 0.21894E+01_JPRB, 0.26050E+01_JPRB, 0.37585E+01_JPRB ,&
&  0.36532E+01_JPRB, 0.33917E+01_JPRB, 0.25460E+01_JPRB, 0.20578E+01_JPRB, 0.17421E+01_JPRB, 0.15561E+01_JPRB, 0.14816E+01_JPRB ,&
&  0.13959E+01_JPRB, 0.12861E+01_JPRB, 0.12394E+01_JPRB, 0.12059E+01_JPRB, 0.11812E+01_JPRB, 0.11518E+01_JPRB, 0.10927E+01_JPRB ,&
&  0.10380E+01_JPRB, 0.99547E+00_JPRB, 0.99833E+00_JPRB, 0.10089E+01_JPRB, 0.10203E+01_JPRB, 0.10046E+01_JPRB, 0.99428E+00_JPRB ,&
&  0.10046E+01_JPRB, 0.10407E+01_JPRB, 0.10346E+01_JPRB, 0.10387E+01_JPRB, 0.10281E+01_JPRB, 0.10030E+01_JPRB, 0.98436E+00_JPRB ,&
&  0.10221E+01_JPRB, 0.10163E+01_JPRB, 0.95476E+00_JPRB, 0.92591E+00_JPRB, 0.89310E+00_JPRB, 0.91305E+00_JPRB, 0.86455E+00_JPRB ,&
&  0.84782E+00_JPRB, 0.84984E+00_JPRB, 0.92067E+00_JPRB, 0.92763E+00_JPRB, 0.94009E+00_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.94191E+00_JPRB, 0.92654E+00_JPRB, 0.90578E+00_JPRB, 0.87085E+00_JPRB, 0.85959E+00_JPRB, 0.84536E+00_JPRB, 0.83859E+00_JPRB ,&
&  0.83741E+00_JPRB, 0.82566E+00_JPRB, 0.81255E+00_JPRB, 0.79334E+00_JPRB, 0.78603E+00_JPRB, 0.79729E+00_JPRB, 0.79185E+00_JPRB ,&
&  0.77386E+00_JPRB, 0.75021E+00_JPRB, 0.74232E+00_JPRB, 0.73318E+00_JPRB, 0.72380E+00_JPRB, 0.72533E+00_JPRB, 0.73577E+00_JPRB ,&
&  0.74087E+00_JPRB, 0.75898E+00_JPRB, 0.78866E+00_JPRB, 0.81529E+00_JPRB, 0.81254E+00_JPRB, 0.80382E+00_JPRB, 0.82760E+00_JPRB ,&
&  0.94574E+00_JPRB, 0.93073E+00_JPRB, 0.18097E+01_JPRB, 0.20405E+01_JPRB, 0.17178E+01_JPRB, 0.14894E+01_JPRB, 0.12712E+01_JPRB ,&
&  0.11873E+01_JPRB, 0.10707E+01_JPRB, 0.97861E+00_JPRB, 0.90522E+00_JPRB, 0.86445E+00_JPRB, 0.81676E+00_JPRB, 0.80506E+00_JPRB ,&
&  0.80269E+00_JPRB, 0.78778E+00_JPRB, 0.78520E+00_JPRB, 0.80391E+00_JPRB, 0.81508E+00_JPRB, 0.82753E+00_JPRB, 0.83292E+00_JPRB ,&
&  0.85465E+00_JPRB, 0.88803E+00_JPRB, 0.99499E+00_JPRB, 0.13518E+01_JPRB, 0.20102E+01_JPRB, 0.26895E+01_JPRB, 0.37436E+01_JPRB ,&
&  0.42463E+01_JPRB, 0.39103E+01_JPRB, 0.28084E+01_JPRB, 0.22083E+01_JPRB, 0.18844E+01_JPRB, 0.16669E+01_JPRB, 0.15676E+01_JPRB ,&
&  0.14909E+01_JPRB, 0.14053E+01_JPRB, 0.13411E+01_JPRB, 0.13445E+01_JPRB, 0.12715E+01_JPRB, 0.12055E+01_JPRB, 0.11385E+01_JPRB ,&
&  0.10922E+01_JPRB, 0.10500E+01_JPRB, 0.10098E+01_JPRB, 0.98710E+00_JPRB, 0.99873E+00_JPRB, 0.10003E+01_JPRB, 0.98953E+00_JPRB ,&
&  0.98849E+00_JPRB, 0.10068E+01_JPRB, 0.10125E+01_JPRB, 0.10133E+01_JPRB, 0.10371E+01_JPRB, 0.10420E+01_JPRB, 0.10424E+01_JPRB ,&
&  0.10313E+01_JPRB, 0.10326E+01_JPRB, 0.98873E+00_JPRB, 0.96634E+00_JPRB, 0.91385E+00_JPRB, 0.95263E+00_JPRB, 0.90137E+00_JPRB ,&
&  0.86548E+00_JPRB, 0.83967E+00_JPRB, 0.85882E+00_JPRB, 0.91201E+00_JPRB, 0.94503E+00_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.91979E+00_JPRB, 0.90583E+00_JPRB, 0.89216E+00_JPRB, 0.87515E+00_JPRB, 0.85855E+00_JPRB, 0.85082E+00_JPRB, 0.84225E+00_JPRB ,&
&  0.84553E+00_JPRB, 0.84836E+00_JPRB, 0.84017E+00_JPRB, 0.81566E+00_JPRB, 0.79584E+00_JPRB, 0.79314E+00_JPRB, 0.79930E+00_JPRB ,&
&  0.79424E+00_JPRB, 0.78388E+00_JPRB, 0.78732E+00_JPRB, 0.77054E+00_JPRB, 0.74884E+00_JPRB, 0.73536E+00_JPRB, 0.73041E+00_JPRB ,&
&  0.73547E+00_JPRB, 0.74645E+00_JPRB, 0.77257E+00_JPRB, 0.79743E+00_JPRB, 0.80024E+00_JPRB, 0.79148E+00_JPRB, 0.77445E+00_JPRB ,&
&  0.87045E+00_JPRB, 0.11410E+01_JPRB, 0.18692E+01_JPRB, 0.18207E+01_JPRB, 0.16062E+01_JPRB, 0.13830E+01_JPRB, 0.12816E+01_JPRB ,&
&  0.11441E+01_JPRB, 0.10640E+01_JPRB, 0.98210E+00_JPRB, 0.90291E+00_JPRB, 0.84616E+00_JPRB, 0.81455E+00_JPRB, 0.79995E+00_JPRB ,&
&  0.77931E+00_JPRB, 0.76592E+00_JPRB, 0.76755E+00_JPRB, 0.77448E+00_JPRB, 0.78043E+00_JPRB, 0.78034E+00_JPRB, 0.77687E+00_JPRB ,&
&  0.79437E+00_JPRB, 0.82322E+00_JPRB, 0.88148E+00_JPRB, 0.10741E+01_JPRB, 0.17425E+01_JPRB, 0.21099E+01_JPRB, 0.32430E+01_JPRB ,&
&  0.43898E+01_JPRB, 0.36825E+01_JPRB, 0.28677E+01_JPRB, 0.23289E+01_JPRB, 0.19960E+01_JPRB, 0.17746E+01_JPRB, 0.16439E+01_JPRB ,&
&  0.15179E+01_JPRB, 0.14429E+01_JPRB, 0.13844E+01_JPRB, 0.13194E+01_JPRB, 0.12753E+01_JPRB, 0.12043E+01_JPRB, 0.11437E+01_JPRB ,&
&  0.11257E+01_JPRB, 0.10852E+01_JPRB, 0.10403E+01_JPRB, 0.99284E+00_JPRB, 0.98394E+00_JPRB, 0.98603E+00_JPRB, 0.98239E+00_JPRB ,&
&  0.98401E+00_JPRB, 0.98409E+00_JPRB, 0.98879E+00_JPRB, 0.99578E+00_JPRB, 0.10346E+01_JPRB, 0.10682E+01_JPRB, 0.10624E+01_JPRB ,&
&  0.10154E+01_JPRB, 0.10027E+01_JPRB, 0.10122E+01_JPRB, 0.97456E+00_JPRB, 0.95098E+00_JPRB, 0.10192E+01_JPRB, 0.95491E+00_JPRB ,&
&  0.89565E+00_JPRB, 0.86310E+00_JPRB, 0.84463E+00_JPRB, 0.85764E+00_JPRB, 0.91816E+00_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.91921E+00_JPRB, 0.91182E+00_JPRB, 0.89680E+00_JPRB, 0.88683E+00_JPRB, 0.88665E+00_JPRB, 0.87671E+00_JPRB, 0.86979E+00_JPRB ,&
&  0.86725E+00_JPRB, 0.85799E+00_JPRB, 0.85038E+00_JPRB, 0.83692E+00_JPRB, 0.82034E+00_JPRB, 0.80721E+00_JPRB, 0.80395E+00_JPRB ,&
&  0.80565E+00_JPRB, 0.79760E+00_JPRB, 0.80253E+00_JPRB, 0.80481E+00_JPRB, 0.80180E+00_JPRB, 0.78925E+00_JPRB, 0.77665E+00_JPRB ,&
&  0.76983E+00_JPRB, 0.75710E+00_JPRB, 0.75400E+00_JPRB, 0.77405E+00_JPRB, 0.79217E+00_JPRB, 0.79365E+00_JPRB, 0.77694E+00_JPRB ,&
&  0.80624E+00_JPRB, 0.99625E+00_JPRB, 0.14307E+01_JPRB, 0.15586E+01_JPRB, 0.14705E+01_JPRB, 0.13482E+01_JPRB, 0.12149E+01_JPRB ,&
&  0.11045E+01_JPRB, 0.10127E+01_JPRB, 0.96170E+00_JPRB, 0.91880E+00_JPRB, 0.87469E+00_JPRB, 0.83848E+00_JPRB, 0.80762E+00_JPRB ,&
&  0.77384E+00_JPRB, 0.76078E+00_JPRB, 0.75423E+00_JPRB, 0.75369E+00_JPRB, 0.75376E+00_JPRB, 0.75289E+00_JPRB, 0.75792E+00_JPRB ,&
&  0.77136E+00_JPRB, 0.79144E+00_JPRB, 0.82447E+00_JPRB, 0.89589E+00_JPRB, 0.12262E+01_JPRB, 0.17620E+01_JPRB, 0.24888E+01_JPRB ,&
&  0.26734E+01_JPRB, 0.27197E+01_JPRB, 0.24812E+01_JPRB, 0.21489E+01_JPRB, 0.18810E+01_JPRB, 0.16870E+01_JPRB, 0.15589E+01_JPRB ,&
&  0.14359E+01_JPRB, 0.13510E+01_JPRB, 0.12911E+01_JPRB, 0.12202E+01_JPRB, 0.11442E+01_JPRB, 0.11039E+01_JPRB, 0.11082E+01_JPRB ,&
&  0.11258E+01_JPRB, 0.11126E+01_JPRB, 0.10677E+01_JPRB, 0.10149E+01_JPRB, 0.96041E+00_JPRB, 0.93916E+00_JPRB, 0.93269E+00_JPRB ,&
&  0.92942E+00_JPRB, 0.94180E+00_JPRB, 0.97079E+00_JPRB, 0.10027E+01_JPRB, 0.10231E+01_JPRB, 0.10301E+01_JPRB, 0.10265E+01_JPRB ,&
&  0.10352E+01_JPRB, 0.10236E+01_JPRB, 0.10045E+01_JPRB, 0.99980E+00_JPRB, 0.97849E+00_JPRB, 0.10381E+01_JPRB, 0.99507E+00_JPRB ,&
&  0.93285E+00_JPRB, 0.88845E+00_JPRB, 0.86564E+00_JPRB, 0.85367E+00_JPRB, 0.90204E+00_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.93819E+00_JPRB, 0.93224E+00_JPRB, 0.91568E+00_JPRB, 0.90348E+00_JPRB, 0.88821E+00_JPRB, 0.87483E+00_JPRB, 0.86344E+00_JPRB ,&
&  0.85809E+00_JPRB, 0.84371E+00_JPRB, 0.83685E+00_JPRB, 0.82983E+00_JPRB, 0.82743E+00_JPRB, 0.82008E+00_JPRB, 0.80848E+00_JPRB ,&
&  0.79584E+00_JPRB, 0.78907E+00_JPRB, 0.78765E+00_JPRB, 0.79258E+00_JPRB, 0.80288E+00_JPRB, 0.80932E+00_JPRB, 0.80414E+00_JPRB ,&
&  0.79515E+00_JPRB, 0.78119E+00_JPRB, 0.76399E+00_JPRB, 0.77734E+00_JPRB, 0.79273E+00_JPRB, 0.79674E+00_JPRB, 0.78109E+00_JPRB ,&
&  0.78497E+00_JPRB, 0.82431E+00_JPRB, 0.10816E+01_JPRB, 0.11850E+01_JPRB, 0.11955E+01_JPRB, 0.12261E+01_JPRB, 0.11593E+01_JPRB ,&
&  0.10915E+01_JPRB, 0.10014E+01_JPRB, 0.93001E+00_JPRB, 0.89600E+00_JPRB, 0.87516E+00_JPRB, 0.84013E+00_JPRB, 0.81114E+00_JPRB ,&
&  0.77113E+00_JPRB, 0.75264E+00_JPRB, 0.74067E+00_JPRB, 0.73612E+00_JPRB, 0.73379E+00_JPRB, 0.73488E+00_JPRB, 0.74109E+00_JPRB ,&
&  0.75052E+00_JPRB, 0.76163E+00_JPRB, 0.77917E+00_JPRB, 0.81659E+00_JPRB, 0.89373E+00_JPRB, 0.11276E+01_JPRB, 0.14773E+01_JPRB ,&
&  0.17109E+01_JPRB, 0.18801E+01_JPRB, 0.20013E+01_JPRB, 0.19874E+01_JPRB, 0.18310E+01_JPRB, 0.16454E+01_JPRB, 0.15073E+01_JPRB ,&
&  0.13917E+01_JPRB, 0.12800E+01_JPRB, 0.11847E+01_JPRB, 0.11084E+01_JPRB, 0.10399E+01_JPRB, 0.10113E+01_JPRB, 0.10400E+01_JPRB ,&
&  0.10579E+01_JPRB, 0.10548E+01_JPRB, 0.10226E+01_JPRB, 0.98186E+00_JPRB, 0.93026E+00_JPRB, 0.89830E+00_JPRB, 0.89137E+00_JPRB ,&
&  0.89690E+00_JPRB, 0.91521E+00_JPRB, 0.94061E+00_JPRB, 0.96842E+00_JPRB, 0.99095E+00_JPRB, 0.10015E+01_JPRB, 0.10030E+01_JPRB ,&
&  0.10102E+01_JPRB, 0.10120E+01_JPRB, 0.95697E+00_JPRB, 0.94507E+00_JPRB, 0.10282E+01_JPRB, 0.10159E+01_JPRB, 0.99805E+00_JPRB ,&
&  0.96002E+00_JPRB, 0.90351E+00_JPRB, 0.86458E+00_JPRB, 0.83806E+00_JPRB, 0.89577E+00_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.81242E+00_JPRB, 0.84075E+00_JPRB, 0.85133E+00_JPRB, 0.85402E+00_JPRB, 0.84517E+00_JPRB, 0.83090E+00_JPRB, 0.81900E+00_JPRB ,&
&  0.81442E+00_JPRB, 0.80134E+00_JPRB, 0.79615E+00_JPRB, 0.79475E+00_JPRB, 0.79898E+00_JPRB, 0.80717E+00_JPRB, 0.80829E+00_JPRB ,&
&  0.80035E+00_JPRB, 0.78770E+00_JPRB, 0.77942E+00_JPRB, 0.77966E+00_JPRB, 0.78999E+00_JPRB, 0.80087E+00_JPRB, 0.80802E+00_JPRB ,&
&  0.81096E+00_JPRB, 0.80478E+00_JPRB, 0.80175E+00_JPRB, 0.81066E+00_JPRB, 0.81811E+00_JPRB, 0.80975E+00_JPRB, 0.78926E+00_JPRB ,&
&  0.78017E+00_JPRB, 0.72097E+00_JPRB, 0.82019E+00_JPRB, 0.92950E+00_JPRB, 0.10154E+01_JPRB, 0.10517E+01_JPRB, 0.10548E+01_JPRB ,&
&  0.10196E+01_JPRB, 0.97243E+00_JPRB, 0.91600E+00_JPRB, 0.87260E+00_JPRB, 0.84013E+00_JPRB, 0.81579E+00_JPRB, 0.79944E+00_JPRB ,&
&  0.77539E+00_JPRB, 0.75302E+00_JPRB, 0.73325E+00_JPRB, 0.72250E+00_JPRB, 0.71876E+00_JPRB, 0.71559E+00_JPRB, 0.71305E+00_JPRB ,&
&  0.71091E+00_JPRB, 0.71140E+00_JPRB, 0.72281E+00_JPRB, 0.74789E+00_JPRB, 0.78779E+00_JPRB, 0.88315E+00_JPRB, 0.10454E+01_JPRB ,&
&  0.11880E+01_JPRB, 0.13377E+01_JPRB, 0.14846E+01_JPRB, 0.16012E+01_JPRB, 0.16017E+01_JPRB, 0.15327E+01_JPRB, 0.14328E+01_JPRB ,&
&  0.13163E+01_JPRB, 0.11749E+01_JPRB, 0.10523E+01_JPRB, 0.97908E+00_JPRB, 0.95364E+00_JPRB, 0.95493E+00_JPRB, 0.98104E+00_JPRB ,&
&  0.99414E+00_JPRB, 0.98855E+00_JPRB, 0.95482E+00_JPRB, 0.92001E+00_JPRB, 0.89706E+00_JPRB, 0.87108E+00_JPRB, 0.86328E+00_JPRB ,&
&  0.86220E+00_JPRB, 0.87221E+00_JPRB, 0.89208E+00_JPRB, 0.91069E+00_JPRB, 0.93539E+00_JPRB, 0.95114E+00_JPRB, 0.95432E+00_JPRB ,&
&  0.95365E+00_JPRB, 0.94951E+00_JPRB, 0.94573E+00_JPRB, 0.96854E+00_JPRB, 0.95684E+00_JPRB, 0.95016E+00_JPRB, 0.93993E+00_JPRB ,&
&  0.91932E+00_JPRB, 0.88153E+00_JPRB, 0.83880E+00_JPRB, 0.81544E+00_JPRB, 0.75717E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.76220E+00_JPRB, 0.78902E+00_JPRB, 0.80197E+00_JPRB, 0.80578E+00_JPRB, 0.80514E+00_JPRB, 0.79418E+00_JPRB, 0.78067E+00_JPRB ,&
&  0.76402E+00_JPRB, 0.75753E+00_JPRB, 0.75552E+00_JPRB, 0.76052E+00_JPRB, 0.76693E+00_JPRB, 0.77616E+00_JPRB, 0.78895E+00_JPRB ,&
&  0.80114E+00_JPRB, 0.80743E+00_JPRB, 0.81246E+00_JPRB, 0.81114E+00_JPRB, 0.80980E+00_JPRB, 0.81238E+00_JPRB, 0.81686E+00_JPRB ,&
&  0.82592E+00_JPRB, 0.82906E+00_JPRB, 0.82604E+00_JPRB, 0.83430E+00_JPRB, 0.84686E+00_JPRB, 0.83622E+00_JPRB, 0.81664E+00_JPRB ,&
&  0.80004E+00_JPRB, 0.66831E+00_JPRB, 0.69156E+00_JPRB, 0.79617E+00_JPRB, 0.87494E+00_JPRB, 0.95978E+00_JPRB, 0.98549E+00_JPRB ,&
&  0.96763E+00_JPRB, 0.92844E+00_JPRB, 0.88710E+00_JPRB, 0.84930E+00_JPRB, 0.82329E+00_JPRB, 0.79434E+00_JPRB, 0.77449E+00_JPRB ,&
&  0.76113E+00_JPRB, 0.74683E+00_JPRB, 0.72550E+00_JPRB, 0.70848E+00_JPRB, 0.69482E+00_JPRB, 0.68124E+00_JPRB, 0.67414E+00_JPRB ,&
&  0.67643E+00_JPRB, 0.68865E+00_JPRB, 0.70383E+00_JPRB, 0.72462E+00_JPRB, 0.76182E+00_JPRB, 0.81034E+00_JPRB, 0.88419E+00_JPRB ,&
&  0.96198E+00_JPRB, 0.10463E+01_JPRB, 0.11305E+01_JPRB, 0.12042E+01_JPRB, 0.12673E+01_JPRB, 0.12775E+01_JPRB, 0.12223E+01_JPRB ,&
&  0.11209E+01_JPRB, 0.10389E+01_JPRB, 0.97869E+00_JPRB, 0.93894E+00_JPRB, 0.93218E+00_JPRB, 0.94475E+00_JPRB, 0.95243E+00_JPRB ,&
&  0.94173E+00_JPRB, 0.91561E+00_JPRB, 0.88108E+00_JPRB, 0.84990E+00_JPRB, 0.83463E+00_JPRB, 0.82546E+00_JPRB, 0.81364E+00_JPRB ,&
&  0.80323E+00_JPRB, 0.80107E+00_JPRB, 0.80844E+00_JPRB, 0.82741E+00_JPRB, 0.85159E+00_JPRB, 0.87768E+00_JPRB, 0.89134E+00_JPRB ,&
&  0.89967E+00_JPRB, 0.90776E+00_JPRB, 0.92396E+00_JPRB, 0.87352E+00_JPRB, 0.90646E+00_JPRB, 0.91296E+00_JPRB, 0.91108E+00_JPRB ,&
&  0.89326E+00_JPRB, 0.86604E+00_JPRB, 0.83207E+00_JPRB, 0.68574E+00_JPRB, 0.73471E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.74525E+00_JPRB, 0.75380E+00_JPRB, 0.76089E+00_JPRB, 0.75933E+00_JPRB, 0.75684E+00_JPRB, 0.75017E+00_JPRB, 0.73589E+00_JPRB ,&
&  0.72230E+00_JPRB, 0.71739E+00_JPRB, 0.71812E+00_JPRB, 0.72012E+00_JPRB, 0.72358E+00_JPRB, 0.73015E+00_JPRB, 0.74169E+00_JPRB ,&
&  0.75538E+00_JPRB, 0.76978E+00_JPRB, 0.78672E+00_JPRB, 0.80244E+00_JPRB, 0.80604E+00_JPRB, 0.80283E+00_JPRB, 0.80299E+00_JPRB ,&
&  0.80914E+00_JPRB, 0.80810E+00_JPRB, 0.80177E+00_JPRB, 0.80420E+00_JPRB, 0.81209E+00_JPRB, 0.80312E+00_JPRB, 0.78490E+00_JPRB ,&
&  0.77133E+00_JPRB, 0.62534E+00_JPRB, 0.65972E+00_JPRB, 0.72254E+00_JPRB, 0.78886E+00_JPRB, 0.85427E+00_JPRB, 0.88149E+00_JPRB ,&
&  0.87428E+00_JPRB, 0.84342E+00_JPRB, 0.81679E+00_JPRB, 0.78350E+00_JPRB, 0.76926E+00_JPRB, 0.75799E+00_JPRB, 0.74327E+00_JPRB ,&
&  0.72201E+00_JPRB, 0.69993E+00_JPRB, 0.67568E+00_JPRB, 0.65484E+00_JPRB, 0.63830E+00_JPRB, 0.63107E+00_JPRB, 0.62933E+00_JPRB ,&
&  0.63026E+00_JPRB, 0.64150E+00_JPRB, 0.64797E+00_JPRB, 0.65904E+00_JPRB, 0.67768E+00_JPRB, 0.70673E+00_JPRB, 0.74580E+00_JPRB ,&
&  0.79629E+00_JPRB, 0.85454E+00_JPRB, 0.91216E+00_JPRB, 0.95243E+00_JPRB, 0.97973E+00_JPRB, 0.10014E+01_JPRB, 0.99659E+00_JPRB ,&
&  0.96044E+00_JPRB, 0.92038E+00_JPRB, 0.91220E+00_JPRB, 0.91115E+00_JPRB, 0.90009E+00_JPRB, 0.88894E+00_JPRB, 0.87166E+00_JPRB ,&
&  0.84473E+00_JPRB, 0.81366E+00_JPRB, 0.78824E+00_JPRB, 0.76430E+00_JPRB, 0.74923E+00_JPRB, 0.74190E+00_JPRB, 0.72470E+00_JPRB ,&
&  0.71316E+00_JPRB, 0.71138E+00_JPRB, 0.71951E+00_JPRB, 0.73993E+00_JPRB, 0.76733E+00_JPRB, 0.79257E+00_JPRB, 0.80758E+00_JPRB ,&
&  0.81426E+00_JPRB, 0.82196E+00_JPRB, 0.82799E+00_JPRB, 0.85220E+00_JPRB, 0.85841E+00_JPRB, 0.86013E+00_JPRB, 0.85747E+00_JPRB ,&
&  0.85118E+00_JPRB, 0.84435E+00_JPRB, 0.64878E+00_JPRB, 0.71126E+00_JPRB, 0.72604E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.76454E+00_JPRB, 0.74804E+00_JPRB, 0.73983E+00_JPRB, 0.73514E+00_JPRB, 0.73143E+00_JPRB, 0.72506E+00_JPRB, 0.71232E+00_JPRB ,&
&  0.69964E+00_JPRB, 0.69352E+00_JPRB, 0.68792E+00_JPRB, 0.68028E+00_JPRB, 0.68195E+00_JPRB, 0.68809E+00_JPRB, 0.69912E+00_JPRB ,&
&  0.71122E+00_JPRB, 0.72670E+00_JPRB, 0.74158E+00_JPRB, 0.76041E+00_JPRB, 0.77294E+00_JPRB, 0.78175E+00_JPRB, 0.78644E+00_JPRB ,&
&  0.78960E+00_JPRB, 0.79003E+00_JPRB, 0.78275E+00_JPRB, 0.77340E+00_JPRB, 0.77068E+00_JPRB, 0.77137E+00_JPRB, 0.76624E+00_JPRB ,&
&  0.66796E+00_JPRB, 0.58032E+00_JPRB, 0.63258E+00_JPRB, 0.69291E+00_JPRB, 0.75014E+00_JPRB, 0.79794E+00_JPRB, 0.81879E+00_JPRB ,&
&  0.80833E+00_JPRB, 0.78163E+00_JPRB, 0.75624E+00_JPRB, 0.72589E+00_JPRB, 0.71427E+00_JPRB, 0.70982E+00_JPRB, 0.70149E+00_JPRB ,&
&  0.69182E+00_JPRB, 0.68040E+00_JPRB, 0.66644E+00_JPRB, 0.64474E+00_JPRB, 0.62970E+00_JPRB, 0.61878E+00_JPRB, 0.61734E+00_JPRB ,&
&  0.61993E+00_JPRB, 0.62399E+00_JPRB, 0.62756E+00_JPRB, 0.63039E+00_JPRB, 0.63778E+00_JPRB, 0.64963E+00_JPRB, 0.66482E+00_JPRB ,&
&  0.68821E+00_JPRB, 0.71514E+00_JPRB, 0.75158E+00_JPRB, 0.77693E+00_JPRB, 0.79313E+00_JPRB, 0.80765E+00_JPRB, 0.82613E+00_JPRB ,&
&  0.83601E+00_JPRB, 0.83226E+00_JPRB, 0.81064E+00_JPRB, 0.79910E+00_JPRB, 0.79658E+00_JPRB, 0.79266E+00_JPRB, 0.78034E+00_JPRB ,&
&  0.75810E+00_JPRB, 0.73139E+00_JPRB, 0.70440E+00_JPRB, 0.68868E+00_JPRB, 0.68594E+00_JPRB, 0.68397E+00_JPRB, 0.67808E+00_JPRB ,&
&  0.67295E+00_JPRB, 0.67208E+00_JPRB, 0.67751E+00_JPRB, 0.68883E+00_JPRB, 0.70394E+00_JPRB, 0.72225E+00_JPRB, 0.73682E+00_JPRB ,&
&  0.75108E+00_JPRB, 0.76523E+00_JPRB, 0.77591E+00_JPRB, 0.79337E+00_JPRB, 0.81092E+00_JPRB, 0.81935E+00_JPRB, 0.81470E+00_JPRB ,&
&  0.81168E+00_JPRB, 0.79913E+00_JPRB, 0.78993E+00_JPRB, 0.78573E+00_JPRB, 0.77815E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.73465E+00_JPRB, 0.71749E+00_JPRB, 0.70776E+00_JPRB, 0.70394E+00_JPRB, 0.69555E+00_JPRB, 0.67674E+00_JPRB, 0.66122E+00_JPRB ,&
&  0.64721E+00_JPRB, 0.64001E+00_JPRB, 0.63738E+00_JPRB, 0.63699E+00_JPRB, 0.64298E+00_JPRB, 0.64850E+00_JPRB, 0.65519E+00_JPRB ,&
&  0.66636E+00_JPRB, 0.68437E+00_JPRB, 0.69985E+00_JPRB, 0.71422E+00_JPRB, 0.72720E+00_JPRB, 0.74010E+00_JPRB, 0.74368E+00_JPRB ,&
&  0.74655E+00_JPRB, 0.75413E+00_JPRB, 0.75577E+00_JPRB, 0.74368E+00_JPRB, 0.73080E+00_JPRB, 0.73302E+00_JPRB, 0.72973E+00_JPRB ,&
&  0.72004E+00_JPRB, 0.60637E+00_JPRB, 0.65804E+00_JPRB, 0.68132E+00_JPRB, 0.70765E+00_JPRB, 0.73159E+00_JPRB, 0.74437E+00_JPRB ,&
&  0.74104E+00_JPRB, 0.72537E+00_JPRB, 0.70672E+00_JPRB, 0.68040E+00_JPRB, 0.67250E+00_JPRB, 0.66599E+00_JPRB, 0.65797E+00_JPRB ,&
&  0.65272E+00_JPRB, 0.64746E+00_JPRB, 0.64006E+00_JPRB, 0.62952E+00_JPRB, 0.61570E+00_JPRB, 0.60551E+00_JPRB, 0.59656E+00_JPRB ,&
&  0.59454E+00_JPRB, 0.58751E+00_JPRB, 0.58092E+00_JPRB, 0.58158E+00_JPRB, 0.59324E+00_JPRB, 0.59924E+00_JPRB, 0.60065E+00_JPRB ,&
&  0.60321E+00_JPRB, 0.60874E+00_JPRB, 0.62198E+00_JPRB, 0.64071E+00_JPRB, 0.65608E+00_JPRB, 0.66742E+00_JPRB, 0.68331E+00_JPRB ,&
&  0.70832E+00_JPRB, 0.72659E+00_JPRB, 0.72793E+00_JPRB, 0.71488E+00_JPRB, 0.70571E+00_JPRB, 0.70339E+00_JPRB, 0.69047E+00_JPRB ,&
&  0.66967E+00_JPRB, 0.65528E+00_JPRB, 0.65015E+00_JPRB, 0.64769E+00_JPRB, 0.64302E+00_JPRB, 0.63674E+00_JPRB, 0.63576E+00_JPRB ,&
&  0.63683E+00_JPRB, 0.63697E+00_JPRB, 0.63592E+00_JPRB, 0.63847E+00_JPRB, 0.64436E+00_JPRB, 0.65206E+00_JPRB, 0.66278E+00_JPRB ,&
&  0.67846E+00_JPRB, 0.69287E+00_JPRB, 0.70106E+00_JPRB, 0.71206E+00_JPRB, 0.72556E+00_JPRB, 0.73180E+00_JPRB, 0.74570E+00_JPRB ,&
&  0.74555E+00_JPRB, 0.74059E+00_JPRB, 0.73847E+00_JPRB, 0.74352E+00_JPRB, 0.74556E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.70313E+00_JPRB, 0.68379E+00_JPRB, 0.67232E+00_JPRB, 0.66106E+00_JPRB, 0.65303E+00_JPRB, 0.63542E+00_JPRB, 0.62551E+00_JPRB ,&
&  0.61928E+00_JPRB, 0.61551E+00_JPRB, 0.61529E+00_JPRB, 0.61594E+00_JPRB, 0.62613E+00_JPRB, 0.63259E+00_JPRB, 0.63497E+00_JPRB ,&
&  0.64248E+00_JPRB, 0.65986E+00_JPRB, 0.67585E+00_JPRB, 0.68929E+00_JPRB, 0.69997E+00_JPRB, 0.70781E+00_JPRB, 0.70784E+00_JPRB ,&
&  0.71173E+00_JPRB, 0.71987E+00_JPRB, 0.72390E+00_JPRB, 0.71415E+00_JPRB, 0.70174E+00_JPRB, 0.69517E+00_JPRB, 0.69525E+00_JPRB ,&
&  0.68482E+00_JPRB, 0.63865E+00_JPRB, 0.65257E+00_JPRB, 0.66418E+00_JPRB, 0.67741E+00_JPRB, 0.68360E+00_JPRB, 0.69009E+00_JPRB ,&
&  0.69142E+00_JPRB, 0.67517E+00_JPRB, 0.65491E+00_JPRB, 0.64006E+00_JPRB, 0.64332E+00_JPRB, 0.64093E+00_JPRB, 0.63237E+00_JPRB ,&
&  0.62919E+00_JPRB, 0.62666E+00_JPRB, 0.61988E+00_JPRB, 0.60738E+00_JPRB, 0.59606E+00_JPRB, 0.58860E+00_JPRB, 0.58378E+00_JPRB ,&
&  0.57299E+00_JPRB, 0.56401E+00_JPRB, 0.55914E+00_JPRB, 0.55368E+00_JPRB, 0.54904E+00_JPRB, 0.55333E+00_JPRB, 0.55707E+00_JPRB ,&
&  0.55733E+00_JPRB, 0.55960E+00_JPRB, 0.56905E+00_JPRB, 0.58504E+00_JPRB, 0.59914E+00_JPRB, 0.61464E+00_JPRB, 0.63230E+00_JPRB ,&
&  0.65026E+00_JPRB, 0.66709E+00_JPRB, 0.67381E+00_JPRB, 0.67870E+00_JPRB, 0.67326E+00_JPRB, 0.66381E+00_JPRB, 0.64463E+00_JPRB ,&
&  0.62345E+00_JPRB, 0.61837E+00_JPRB, 0.61835E+00_JPRB, 0.62328E+00_JPRB, 0.62101E+00_JPRB, 0.61785E+00_JPRB, 0.61614E+00_JPRB ,&
&  0.61211E+00_JPRB, 0.60856E+00_JPRB, 0.60354E+00_JPRB, 0.59883E+00_JPRB, 0.60192E+00_JPRB, 0.60748E+00_JPRB, 0.62058E+00_JPRB ,&
&  0.63824E+00_JPRB, 0.65711E+00_JPRB, 0.67666E+00_JPRB, 0.68987E+00_JPRB, 0.69427E+00_JPRB, 0.69713E+00_JPRB, 0.70033E+00_JPRB ,&
&  0.70124E+00_JPRB, 0.69925E+00_JPRB, 0.70470E+00_JPRB, 0.70867E+00_JPRB, 0.71295E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.65152E+00_JPRB, 0.62670E+00_JPRB, 0.60877E+00_JPRB, 0.60215E+00_JPRB, 0.59871E+00_JPRB, 0.58692E+00_JPRB, 0.58192E+00_JPRB ,&
&  0.58806E+00_JPRB, 0.58994E+00_JPRB, 0.59712E+00_JPRB, 0.59695E+00_JPRB, 0.59629E+00_JPRB, 0.59887E+00_JPRB, 0.59129E+00_JPRB ,&
&  0.59134E+00_JPRB, 0.60033E+00_JPRB, 0.61090E+00_JPRB, 0.62519E+00_JPRB, 0.63720E+00_JPRB, 0.64627E+00_JPRB, 0.65401E+00_JPRB ,&
&  0.65709E+00_JPRB, 0.66363E+00_JPRB, 0.66725E+00_JPRB, 0.66312E+00_JPRB, 0.65233E+00_JPRB, 0.64391E+00_JPRB, 0.64245E+00_JPRB ,&
&  0.64109E+00_JPRB, 0.63466E+00_JPRB, 0.59036E+00_JPRB, 0.61408E+00_JPRB, 0.62098E+00_JPRB, 0.62405E+00_JPRB, 0.62594E+00_JPRB ,&
&  0.62036E+00_JPRB, 0.60525E+00_JPRB, 0.59818E+00_JPRB, 0.58631E+00_JPRB, 0.58499E+00_JPRB, 0.59098E+00_JPRB, 0.58988E+00_JPRB ,&
&  0.58620E+00_JPRB, 0.57646E+00_JPRB, 0.56303E+00_JPRB, 0.55250E+00_JPRB, 0.54998E+00_JPRB, 0.54826E+00_JPRB, 0.54102E+00_JPRB ,&
&  0.53596E+00_JPRB, 0.53511E+00_JPRB, 0.53340E+00_JPRB, 0.52761E+00_JPRB, 0.51298E+00_JPRB, 0.49957E+00_JPRB, 0.49723E+00_JPRB ,&
&  0.50109E+00_JPRB, 0.50888E+00_JPRB, 0.52370E+00_JPRB, 0.54253E+00_JPRB, 0.56265E+00_JPRB, 0.58150E+00_JPRB, 0.59839E+00_JPRB ,&
&  0.60513E+00_JPRB, 0.60866E+00_JPRB, 0.61268E+00_JPRB, 0.61851E+00_JPRB, 0.62026E+00_JPRB, 0.61269E+00_JPRB, 0.59301E+00_JPRB ,&
&  0.57172E+00_JPRB, 0.56561E+00_JPRB, 0.56878E+00_JPRB, 0.57702E+00_JPRB, 0.58092E+00_JPRB, 0.58001E+00_JPRB, 0.57331E+00_JPRB ,&
&  0.56921E+00_JPRB, 0.56086E+00_JPRB, 0.55205E+00_JPRB, 0.54774E+00_JPRB, 0.54746E+00_JPRB, 0.55346E+00_JPRB, 0.57071E+00_JPRB ,&
&  0.59141E+00_JPRB, 0.61453E+00_JPRB, 0.63523E+00_JPRB, 0.64268E+00_JPRB, 0.64642E+00_JPRB, 0.64540E+00_JPRB, 0.63638E+00_JPRB ,&
&  0.64043E+00_JPRB, 0.65905E+00_JPRB, 0.66833E+00_JPRB, 0.66955E+00_JPRB, 0.66694E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.60221E+00_JPRB, 0.57372E+00_JPRB, 0.56249E+00_JPRB, 0.56425E+00_JPRB, 0.56770E+00_JPRB, 0.56367E+00_JPRB, 0.57110E+00_JPRB ,&
&  0.58236E+00_JPRB, 0.59089E+00_JPRB, 0.59558E+00_JPRB, 0.59191E+00_JPRB, 0.58946E+00_JPRB, 0.58655E+00_JPRB, 0.57190E+00_JPRB ,&
&  0.56710E+00_JPRB, 0.56537E+00_JPRB, 0.56947E+00_JPRB, 0.57648E+00_JPRB, 0.58410E+00_JPRB, 0.59617E+00_JPRB, 0.60511E+00_JPRB ,&
&  0.61101E+00_JPRB, 0.61470E+00_JPRB, 0.61885E+00_JPRB, 0.61905E+00_JPRB, 0.61551E+00_JPRB, 0.60647E+00_JPRB, 0.59952E+00_JPRB ,&
&  0.59539E+00_JPRB, 0.58371E+00_JPRB, 0.57343E+00_JPRB, 0.57202E+00_JPRB, 0.57501E+00_JPRB, 0.57787E+00_JPRB, 0.57710E+00_JPRB ,&
&  0.56408E+00_JPRB, 0.54736E+00_JPRB, 0.54557E+00_JPRB, 0.54537E+00_JPRB, 0.55056E+00_JPRB, 0.56256E+00_JPRB, 0.56597E+00_JPRB ,&
&  0.56294E+00_JPRB, 0.54912E+00_JPRB, 0.53026E+00_JPRB, 0.52682E+00_JPRB, 0.52924E+00_JPRB, 0.52301E+00_JPRB, 0.51678E+00_JPRB ,&
&  0.51569E+00_JPRB, 0.51269E+00_JPRB, 0.50670E+00_JPRB, 0.49849E+00_JPRB, 0.48573E+00_JPRB, 0.46924E+00_JPRB, 0.46086E+00_JPRB ,&
&  0.46077E+00_JPRB, 0.47659E+00_JPRB, 0.50424E+00_JPRB, 0.53508E+00_JPRB, 0.56431E+00_JPRB, 0.57854E+00_JPRB, 0.57773E+00_JPRB ,&
&  0.56510E+00_JPRB, 0.56154E+00_JPRB, 0.56639E+00_JPRB, 0.57109E+00_JPRB, 0.57271E+00_JPRB, 0.56264E+00_JPRB, 0.54576E+00_JPRB ,&
&  0.53334E+00_JPRB, 0.52406E+00_JPRB, 0.52273E+00_JPRB, 0.52728E+00_JPRB, 0.53180E+00_JPRB, 0.53274E+00_JPRB, 0.52665E+00_JPRB ,&
&  0.52044E+00_JPRB, 0.51388E+00_JPRB, 0.50996E+00_JPRB, 0.51271E+00_JPRB, 0.51803E+00_JPRB, 0.52331E+00_JPRB, 0.53609E+00_JPRB ,&
&  0.55451E+00_JPRB, 0.57303E+00_JPRB, 0.57725E+00_JPRB, 0.57289E+00_JPRB, 0.57226E+00_JPRB, 0.56941E+00_JPRB, 0.56668E+00_JPRB ,&
&  0.57671E+00_JPRB, 0.59878E+00_JPRB, 0.61454E+00_JPRB, 0.62317E+00_JPRB, 0.62315E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.52841E+00_JPRB, 0.51161E+00_JPRB, 0.51204E+00_JPRB, 0.51919E+00_JPRB, 0.52326E+00_JPRB, 0.53113E+00_JPRB, 0.55068E+00_JPRB ,&
&  0.56217E+00_JPRB, 0.56951E+00_JPRB, 0.57241E+00_JPRB, 0.56716E+00_JPRB, 0.56618E+00_JPRB, 0.55894E+00_JPRB, 0.54621E+00_JPRB ,&
&  0.54091E+00_JPRB, 0.53987E+00_JPRB, 0.54149E+00_JPRB, 0.54527E+00_JPRB, 0.54748E+00_JPRB, 0.55025E+00_JPRB, 0.55343E+00_JPRB ,&
&  0.55515E+00_JPRB, 0.55510E+00_JPRB, 0.55426E+00_JPRB, 0.55449E+00_JPRB, 0.55354E+00_JPRB, 0.54598E+00_JPRB, 0.53745E+00_JPRB ,&
&  0.52812E+00_JPRB, 0.51887E+00_JPRB, 0.51621E+00_JPRB, 0.52054E+00_JPRB, 0.52548E+00_JPRB, 0.52851E+00_JPRB, 0.52619E+00_JPRB ,&
&  0.52149E+00_JPRB, 0.51870E+00_JPRB, 0.52091E+00_JPRB, 0.52782E+00_JPRB, 0.53640E+00_JPRB, 0.54594E+00_JPRB, 0.55095E+00_JPRB ,&
&  0.54541E+00_JPRB, 0.52794E+00_JPRB, 0.51736E+00_JPRB, 0.51688E+00_JPRB, 0.51337E+00_JPRB, 0.49915E+00_JPRB, 0.48917E+00_JPRB ,&
&  0.48443E+00_JPRB, 0.47568E+00_JPRB, 0.46641E+00_JPRB, 0.45371E+00_JPRB, 0.44200E+00_JPRB, 0.43763E+00_JPRB, 0.43649E+00_JPRB ,&
&  0.44024E+00_JPRB, 0.45922E+00_JPRB, 0.49295E+00_JPRB, 0.52549E+00_JPRB, 0.53952E+00_JPRB, 0.53294E+00_JPRB, 0.51579E+00_JPRB ,&
&  0.50763E+00_JPRB, 0.50385E+00_JPRB, 0.50495E+00_JPRB, 0.50474E+00_JPRB, 0.50174E+00_JPRB, 0.49333E+00_JPRB, 0.48630E+00_JPRB ,&
&  0.47617E+00_JPRB, 0.46504E+00_JPRB, 0.46050E+00_JPRB, 0.45759E+00_JPRB, 0.45447E+00_JPRB, 0.45440E+00_JPRB, 0.45342E+00_JPRB ,&
&  0.45396E+00_JPRB, 0.45785E+00_JPRB, 0.46937E+00_JPRB, 0.47582E+00_JPRB, 0.47682E+00_JPRB, 0.46667E+00_JPRB, 0.46774E+00_JPRB ,&
&  0.47055E+00_JPRB, 0.46826E+00_JPRB, 0.46359E+00_JPRB, 0.46326E+00_JPRB, 0.46303E+00_JPRB, 0.46474E+00_JPRB, 0.47619E+00_JPRB ,&
&  0.50282E+00_JPRB, 0.52616E+00_JPRB, 0.54519E+00_JPRB, 0.55050E+00_JPRB, 0.54607E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.47373E+00_JPRB, 0.47884E+00_JPRB, 0.49180E+00_JPRB, 0.50252E+00_JPRB, 0.50605E+00_JPRB, 0.51567E+00_JPRB, 0.52696E+00_JPRB ,&
&  0.53617E+00_JPRB, 0.53796E+00_JPRB, 0.53427E+00_JPRB, 0.52404E+00_JPRB, 0.51191E+00_JPRB, 0.50190E+00_JPRB, 0.49790E+00_JPRB ,&
&  0.50007E+00_JPRB, 0.50312E+00_JPRB, 0.50864E+00_JPRB, 0.51171E+00_JPRB, 0.51538E+00_JPRB, 0.51421E+00_JPRB, 0.51465E+00_JPRB ,&
&  0.51646E+00_JPRB, 0.51750E+00_JPRB, 0.51803E+00_JPRB, 0.51439E+00_JPRB, 0.51147E+00_JPRB, 0.50895E+00_JPRB, 0.50040E+00_JPRB ,&
&  0.49086E+00_JPRB, 0.48442E+00_JPRB, 0.48829E+00_JPRB, 0.49968E+00_JPRB, 0.50695E+00_JPRB, 0.50411E+00_JPRB, 0.50277E+00_JPRB ,&
&  0.50302E+00_JPRB, 0.50784E+00_JPRB, 0.51562E+00_JPRB, 0.52729E+00_JPRB, 0.53557E+00_JPRB, 0.54657E+00_JPRB, 0.54717E+00_JPRB ,&
&  0.54076E+00_JPRB, 0.53156E+00_JPRB, 0.52865E+00_JPRB, 0.52747E+00_JPRB, 0.49878E+00_JPRB, 0.46735E+00_JPRB, 0.45195E+00_JPRB ,&
&  0.44370E+00_JPRB, 0.43339E+00_JPRB, 0.41763E+00_JPRB, 0.40654E+00_JPRB, 0.40533E+00_JPRB, 0.41010E+00_JPRB, 0.42165E+00_JPRB ,&
&  0.43081E+00_JPRB, 0.44405E+00_JPRB, 0.45927E+00_JPRB, 0.46898E+00_JPRB, 0.46898E+00_JPRB, 0.45661E+00_JPRB, 0.44512E+00_JPRB ,&
&  0.43823E+00_JPRB, 0.43619E+00_JPRB, 0.43461E+00_JPRB, 0.43257E+00_JPRB, 0.43074E+00_JPRB, 0.43026E+00_JPRB, 0.42922E+00_JPRB ,&
&  0.42500E+00_JPRB, 0.41779E+00_JPRB, 0.40900E+00_JPRB, 0.40395E+00_JPRB, 0.40283E+00_JPRB, 0.40268E+00_JPRB, 0.40428E+00_JPRB ,&
&  0.41328E+00_JPRB, 0.42459E+00_JPRB, 0.43368E+00_JPRB, 0.43773E+00_JPRB, 0.43472E+00_JPRB, 0.42684E+00_JPRB, 0.42209E+00_JPRB ,&
&  0.41848E+00_JPRB, 0.41591E+00_JPRB, 0.41251E+00_JPRB, 0.41088E+00_JPRB, 0.41056E+00_JPRB, 0.41767E+00_JPRB, 0.43315E+00_JPRB ,&
&  0.45248E+00_JPRB, 0.47774E+00_JPRB, 0.48911E+00_JPRB, 0.48748E+00_JPRB, 0.48009E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.44454E+00_JPRB, 0.45718E+00_JPRB, 0.47323E+00_JPRB, 0.48466E+00_JPRB, 0.48966E+00_JPRB, 0.49515E+00_JPRB, 0.49767E+00_JPRB ,&
&  0.49258E+00_JPRB, 0.49046E+00_JPRB, 0.48490E+00_JPRB, 0.47691E+00_JPRB, 0.46994E+00_JPRB, 0.46638E+00_JPRB, 0.46669E+00_JPRB ,&
&  0.46880E+00_JPRB, 0.47062E+00_JPRB, 0.46713E+00_JPRB, 0.46539E+00_JPRB, 0.46933E+00_JPRB, 0.47466E+00_JPRB, 0.47811E+00_JPRB ,&
&  0.47949E+00_JPRB, 0.48091E+00_JPRB, 0.48563E+00_JPRB, 0.48863E+00_JPRB, 0.48875E+00_JPRB, 0.48218E+00_JPRB, 0.47731E+00_JPRB ,&
&  0.47234E+00_JPRB, 0.47061E+00_JPRB, 0.47722E+00_JPRB, 0.46921E+00_JPRB, 0.48402E+00_JPRB, 0.48600E+00_JPRB, 0.49158E+00_JPRB ,&
&  0.49984E+00_JPRB, 0.51345E+00_JPRB, 0.52388E+00_JPRB, 0.53177E+00_JPRB, 0.53416E+00_JPRB, 0.53733E+00_JPRB, 0.52756E+00_JPRB ,&
&  0.51390E+00_JPRB, 0.49536E+00_JPRB, 0.47669E+00_JPRB, 0.45649E+00_JPRB, 0.43719E+00_JPRB, 0.42222E+00_JPRB, 0.41491E+00_JPRB ,&
&  0.40600E+00_JPRB, 0.39216E+00_JPRB, 0.37881E+00_JPRB, 0.37358E+00_JPRB, 0.37822E+00_JPRB, 0.38839E+00_JPRB, 0.40412E+00_JPRB ,&
&  0.41412E+00_JPRB, 0.41742E+00_JPRB, 0.41442E+00_JPRB, 0.41017E+00_JPRB, 0.40574E+00_JPRB, 0.39846E+00_JPRB, 0.39418E+00_JPRB ,&
&  0.39068E+00_JPRB, 0.38492E+00_JPRB, 0.38187E+00_JPRB, 0.38359E+00_JPRB, 0.38675E+00_JPRB, 0.38878E+00_JPRB, 0.38728E+00_JPRB ,&
&  0.38429E+00_JPRB, 0.37981E+00_JPRB, 0.37173E+00_JPRB, 0.36730E+00_JPRB, 0.36753E+00_JPRB, 0.37189E+00_JPRB, 0.37942E+00_JPRB ,&
&  0.38187E+00_JPRB, 0.38784E+00_JPRB, 0.39371E+00_JPRB, 0.39589E+00_JPRB, 0.39850E+00_JPRB, 0.40061E+00_JPRB, 0.40076E+00_JPRB ,&
&  0.39906E+00_JPRB, 0.39406E+00_JPRB, 0.38714E+00_JPRB, 0.38414E+00_JPRB, 0.38520E+00_JPRB, 0.39192E+00_JPRB, 0.40107E+00_JPRB ,&
&  0.41008E+00_JPRB, 0.42381E+00_JPRB, 0.43419E+00_JPRB, 0.43645E+00_JPRB, 0.43853E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.42483E+00_JPRB, 0.43624E+00_JPRB, 0.44964E+00_JPRB, 0.45904E+00_JPRB, 0.45816E+00_JPRB, 0.45819E+00_JPRB, 0.46437E+00_JPRB ,&
&  0.45833E+00_JPRB, 0.45114E+00_JPRB, 0.44522E+00_JPRB, 0.43915E+00_JPRB, 0.43697E+00_JPRB, 0.43559E+00_JPRB, 0.43850E+00_JPRB ,&
&  0.44062E+00_JPRB, 0.43953E+00_JPRB, 0.43793E+00_JPRB, 0.43870E+00_JPRB, 0.44530E+00_JPRB, 0.45352E+00_JPRB, 0.46037E+00_JPRB ,&
&  0.46162E+00_JPRB, 0.46509E+00_JPRB, 0.47073E+00_JPRB, 0.47552E+00_JPRB, 0.47794E+00_JPRB, 0.47942E+00_JPRB, 0.47820E+00_JPRB ,&
&  0.47566E+00_JPRB, 0.47358E+00_JPRB, 0.45662E+00_JPRB, 0.47035E+00_JPRB, 0.47481E+00_JPRB, 0.47569E+00_JPRB, 0.48130E+00_JPRB ,&
&  0.48917E+00_JPRB, 0.49802E+00_JPRB, 0.50212E+00_JPRB, 0.50333E+00_JPRB, 0.49858E+00_JPRB, 0.48920E+00_JPRB, 0.46500E+00_JPRB ,&
&  0.44470E+00_JPRB, 0.42358E+00_JPRB, 0.41038E+00_JPRB, 0.40237E+00_JPRB, 0.39607E+00_JPRB, 0.39063E+00_JPRB, 0.38542E+00_JPRB ,&
&  0.37824E+00_JPRB, 0.37182E+00_JPRB, 0.36790E+00_JPRB, 0.36794E+00_JPRB, 0.37212E+00_JPRB, 0.38305E+00_JPRB, 0.39656E+00_JPRB ,&
&  0.40274E+00_JPRB, 0.39559E+00_JPRB, 0.38775E+00_JPRB, 0.37912E+00_JPRB, 0.36519E+00_JPRB, 0.33769E+00_JPRB, 0.32085E+00_JPRB ,&
&  0.31672E+00_JPRB, 0.32929E+00_JPRB, 0.35065E+00_JPRB, 0.35657E+00_JPRB, 0.36168E+00_JPRB, 0.35929E+00_JPRB, 0.35391E+00_JPRB ,&
&  0.34522E+00_JPRB, 0.32167E+00_JPRB, 0.30880E+00_JPRB, 0.30496E+00_JPRB, 0.30264E+00_JPRB, 0.30282E+00_JPRB, 0.30535E+00_JPRB ,&
&  0.30953E+00_JPRB, 0.31689E+00_JPRB, 0.32204E+00_JPRB, 0.32416E+00_JPRB, 0.32727E+00_JPRB, 0.32744E+00_JPRB, 0.32638E+00_JPRB ,&
&  0.32276E+00_JPRB, 0.32937E+00_JPRB, 0.34768E+00_JPRB, 0.37621E+00_JPRB, 0.38262E+00_JPRB, 0.39021E+00_JPRB, 0.39911E+00_JPRB ,&
&  0.40449E+00_JPRB, 0.40953E+00_JPRB, 0.40990E+00_JPRB, 0.41075E+00_JPRB, 0.41592E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.40172E+00_JPRB, 0.40877E+00_JPRB, 0.41719E+00_JPRB, 0.42819E+00_JPRB, 0.43066E+00_JPRB, 0.43231E+00_JPRB, 0.44142E+00_JPRB ,&
&  0.44179E+00_JPRB, 0.43882E+00_JPRB, 0.43555E+00_JPRB, 0.43575E+00_JPRB, 0.43780E+00_JPRB, 0.44013E+00_JPRB, 0.43959E+00_JPRB ,&
&  0.43045E+00_JPRB, 0.42519E+00_JPRB, 0.42790E+00_JPRB, 0.43087E+00_JPRB, 0.43540E+00_JPRB, 0.44187E+00_JPRB, 0.44747E+00_JPRB ,&
&  0.45046E+00_JPRB, 0.45401E+00_JPRB, 0.45640E+00_JPRB, 0.46257E+00_JPRB, 0.47212E+00_JPRB, 0.47864E+00_JPRB, 0.48494E+00_JPRB ,&
&  0.47093E+00_JPRB, 0.42329E+00_JPRB, 0.39277E+00_JPRB, 0.39918E+00_JPRB, 0.44885E+00_JPRB, 0.45911E+00_JPRB, 0.45999E+00_JPRB ,&
&  0.46308E+00_JPRB, 0.47073E+00_JPRB, 0.47023E+00_JPRB, 0.46433E+00_JPRB, 0.45252E+00_JPRB, 0.43359E+00_JPRB, 0.41047E+00_JPRB ,&
&  0.39499E+00_JPRB, 0.38469E+00_JPRB, 0.37790E+00_JPRB, 0.37128E+00_JPRB, 0.36780E+00_JPRB, 0.36280E+00_JPRB, 0.35513E+00_JPRB ,&
&  0.35210E+00_JPRB, 0.35072E+00_JPRB, 0.35179E+00_JPRB, 0.35342E+00_JPRB, 0.35538E+00_JPRB, 0.35676E+00_JPRB, 0.35398E+00_JPRB ,&
&  0.34179E+00_JPRB, 0.32628E+00_JPRB, 0.31155E+00_JPRB, 0.29550E+00_JPRB, 0.28174E+00_JPRB, 0.27309E+00_JPRB, 0.26619E+00_JPRB ,&
&  0.26348E+00_JPRB, 0.26911E+00_JPRB, 0.28585E+00_JPRB, 0.30517E+00_JPRB, 0.33733E+00_JPRB, 0.31282E+00_JPRB, 0.28914E+00_JPRB ,&
&  0.27748E+00_JPRB, 0.27176E+00_JPRB, 0.26854E+00_JPRB, 0.26836E+00_JPRB, 0.26982E+00_JPRB, 0.27023E+00_JPRB, 0.27029E+00_JPRB ,&
&  0.27153E+00_JPRB, 0.27387E+00_JPRB, 0.27575E+00_JPRB, 0.27738E+00_JPRB, 0.27990E+00_JPRB, 0.28280E+00_JPRB, 0.28435E+00_JPRB ,&
&  0.28511E+00_JPRB, 0.28980E+00_JPRB, 0.29934E+00_JPRB, 0.31039E+00_JPRB, 0.32142E+00_JPRB, 0.33287E+00_JPRB, 0.35041E+00_JPRB ,&
&  0.37242E+00_JPRB, 0.39909E+00_JPRB, 0.39850E+00_JPRB, 0.39846E+00_JPRB, 0.39865E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.39026E+00_JPRB, 0.40047E+00_JPRB, 0.40584E+00_JPRB, 0.41117E+00_JPRB, 0.41340E+00_JPRB, 0.41487E+00_JPRB, 0.42280E+00_JPRB ,&
&  0.42759E+00_JPRB, 0.43134E+00_JPRB, 0.43365E+00_JPRB, 0.43456E+00_JPRB, 0.43033E+00_JPRB, 0.42376E+00_JPRB, 0.42164E+00_JPRB ,&
&  0.42045E+00_JPRB, 0.42093E+00_JPRB, 0.41989E+00_JPRB, 0.41990E+00_JPRB, 0.42043E+00_JPRB, 0.42520E+00_JPRB, 0.43195E+00_JPRB ,&
&  0.43351E+00_JPRB, 0.43800E+00_JPRB, 0.45062E+00_JPRB, 0.45831E+00_JPRB, 0.46316E+00_JPRB, 0.46274E+00_JPRB, 0.45701E+00_JPRB ,&
&  0.42969E+00_JPRB, 0.40182E+00_JPRB, 0.37441E+00_JPRB, 0.37591E+00_JPRB, 0.42923E+00_JPRB, 0.43562E+00_JPRB, 0.43487E+00_JPRB ,&
&  0.43332E+00_JPRB, 0.43765E+00_JPRB, 0.43785E+00_JPRB, 0.42332E+00_JPRB, 0.41272E+00_JPRB, 0.39801E+00_JPRB, 0.38540E+00_JPRB ,&
&  0.37467E+00_JPRB, 0.36698E+00_JPRB, 0.35382E+00_JPRB, 0.32658E+00_JPRB, 0.30893E+00_JPRB, 0.29987E+00_JPRB, 0.28608E+00_JPRB ,&
&  0.27571E+00_JPRB, 0.27150E+00_JPRB, 0.27046E+00_JPRB, 0.27001E+00_JPRB, 0.27409E+00_JPRB, 0.27601E+00_JPRB, 0.27473E+00_JPRB ,&
&  0.27156E+00_JPRB, 0.26890E+00_JPRB, 0.26527E+00_JPRB, 0.26019E+00_JPRB, 0.25750E+00_JPRB, 0.25666E+00_JPRB, 0.25711E+00_JPRB ,&
&  0.26093E+00_JPRB, 0.26691E+00_JPRB, 0.28016E+00_JPRB, 0.28960E+00_JPRB, 0.28621E+00_JPRB, 0.27496E+00_JPRB, 0.26450E+00_JPRB ,&
&  0.25776E+00_JPRB, 0.25496E+00_JPRB, 0.25285E+00_JPRB, 0.25069E+00_JPRB, 0.25124E+00_JPRB, 0.25319E+00_JPRB, 0.25465E+00_JPRB ,&
&  0.25497E+00_JPRB, 0.25662E+00_JPRB, 0.25757E+00_JPRB, 0.25874E+00_JPRB, 0.25977E+00_JPRB, 0.26146E+00_JPRB, 0.26376E+00_JPRB ,&
&  0.26601E+00_JPRB, 0.27123E+00_JPRB, 0.27705E+00_JPRB, 0.28357E+00_JPRB, 0.28767E+00_JPRB, 0.29075E+00_JPRB, 0.29678E+00_JPRB ,&
&  0.30307E+00_JPRB, 0.31875E+00_JPRB, 0.34951E+00_JPRB, 0.38660E+00_JPRB, 0.38119E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.37768E+00_JPRB, 0.38644E+00_JPRB, 0.39417E+00_JPRB, 0.40017E+00_JPRB, 0.40391E+00_JPRB, 0.40465E+00_JPRB, 0.40738E+00_JPRB ,&
&  0.41485E+00_JPRB, 0.41923E+00_JPRB, 0.42238E+00_JPRB, 0.42172E+00_JPRB, 0.42117E+00_JPRB, 0.40243E+00_JPRB, 0.39759E+00_JPRB ,&
&  0.39162E+00_JPRB, 0.38747E+00_JPRB, 0.38181E+00_JPRB, 0.38179E+00_JPRB, 0.38159E+00_JPRB, 0.38355E+00_JPRB, 0.38429E+00_JPRB ,&
&  0.38437E+00_JPRB, 0.38237E+00_JPRB, 0.37918E+00_JPRB, 0.37721E+00_JPRB, 0.37848E+00_JPRB, 0.37994E+00_JPRB, 0.37701E+00_JPRB ,&
&  0.37273E+00_JPRB, 0.37183E+00_JPRB, 0.37509E+00_JPRB, 0.38241E+00_JPRB, 0.42299E+00_JPRB, 0.41786E+00_JPRB, 0.41540E+00_JPRB ,&
&  0.41209E+00_JPRB, 0.40793E+00_JPRB, 0.40740E+00_JPRB, 0.40088E+00_JPRB, 0.39139E+00_JPRB, 0.37038E+00_JPRB, 0.35867E+00_JPRB ,&
&  0.32415E+00_JPRB, 0.30500E+00_JPRB, 0.28811E+00_JPRB, 0.27708E+00_JPRB, 0.26691E+00_JPRB, 0.26101E+00_JPRB, 0.25567E+00_JPRB ,&
&  0.25162E+00_JPRB, 0.24848E+00_JPRB, 0.24566E+00_JPRB, 0.24610E+00_JPRB, 0.24777E+00_JPRB, 0.24873E+00_JPRB, 0.24967E+00_JPRB ,&
&  0.25023E+00_JPRB, 0.25072E+00_JPRB, 0.25137E+00_JPRB, 0.25269E+00_JPRB, 0.25443E+00_JPRB, 0.25672E+00_JPRB, 0.25962E+00_JPRB ,&
&  0.26437E+00_JPRB, 0.26814E+00_JPRB, 0.26968E+00_JPRB, 0.27225E+00_JPRB, 0.27193E+00_JPRB, 0.26638E+00_JPRB, 0.25996E+00_JPRB ,&
&  0.25328E+00_JPRB, 0.24978E+00_JPRB, 0.24637E+00_JPRB, 0.24399E+00_JPRB, 0.24342E+00_JPRB, 0.24341E+00_JPRB, 0.24531E+00_JPRB ,&
&  0.24740E+00_JPRB, 0.24924E+00_JPRB, 0.25052E+00_JPRB, 0.25193E+00_JPRB, 0.25375E+00_JPRB, 0.25517E+00_JPRB, 0.25703E+00_JPRB ,&
&  0.25938E+00_JPRB, 0.26258E+00_JPRB, 0.26736E+00_JPRB, 0.27286E+00_JPRB, 0.27978E+00_JPRB, 0.29018E+00_JPRB, 0.30860E+00_JPRB ,&
&  0.33323E+00_JPRB, 0.38930E+00_JPRB, 0.39492E+00_JPRB, 0.36455E+00_JPRB, 0.37054E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.35456E+00_JPRB, 0.36131E+00_JPRB, 0.36839E+00_JPRB, 0.37597E+00_JPRB, 0.38543E+00_JPRB, 0.38951E+00_JPRB, 0.39152E+00_JPRB ,&
&  0.38748E+00_JPRB, 0.38279E+00_JPRB, 0.37191E+00_JPRB, 0.36177E+00_JPRB, 0.35224E+00_JPRB, 0.34466E+00_JPRB, 0.33888E+00_JPRB ,&
&  0.33565E+00_JPRB, 0.33318E+00_JPRB, 0.32943E+00_JPRB, 0.32616E+00_JPRB, 0.32334E+00_JPRB, 0.32333E+00_JPRB, 0.32351E+00_JPRB ,&
&  0.32276E+00_JPRB, 0.32287E+00_JPRB, 0.32159E+00_JPRB, 0.32678E+00_JPRB, 0.33592E+00_JPRB, 0.34712E+00_JPRB, 0.36173E+00_JPRB ,&
&  0.38222E+00_JPRB, 0.39344E+00_JPRB, 0.40900E+00_JPRB, 0.40951E+00_JPRB, 0.40768E+00_JPRB, 0.40387E+00_JPRB, 0.40078E+00_JPRB ,&
&  0.39616E+00_JPRB, 0.38949E+00_JPRB, 0.38464E+00_JPRB, 0.37861E+00_JPRB, 0.34950E+00_JPRB, 0.32655E+00_JPRB, 0.30894E+00_JPRB ,&
&  0.29363E+00_JPRB, 0.27920E+00_JPRB, 0.26656E+00_JPRB, 0.26015E+00_JPRB, 0.25505E+00_JPRB, 0.25203E+00_JPRB, 0.24887E+00_JPRB ,&
&  0.24553E+00_JPRB, 0.24424E+00_JPRB, 0.24228E+00_JPRB, 0.24156E+00_JPRB, 0.24251E+00_JPRB, 0.24246E+00_JPRB, 0.24321E+00_JPRB ,&
&  0.24390E+00_JPRB, 0.24493E+00_JPRB, 0.24732E+00_JPRB, 0.24912E+00_JPRB, 0.25100E+00_JPRB, 0.25278E+00_JPRB, 0.25371E+00_JPRB ,&
&  0.25550E+00_JPRB, 0.25702E+00_JPRB, 0.25699E+00_JPRB, 0.25627E+00_JPRB, 0.25452E+00_JPRB, 0.25067E+00_JPRB, 0.24681E+00_JPRB ,&
&  0.24270E+00_JPRB, 0.23973E+00_JPRB, 0.23851E+00_JPRB, 0.23753E+00_JPRB, 0.23831E+00_JPRB, 0.23909E+00_JPRB, 0.24003E+00_JPRB ,&
&  0.24190E+00_JPRB, 0.24367E+00_JPRB, 0.24570E+00_JPRB, 0.24854E+00_JPRB, 0.25075E+00_JPRB, 0.25266E+00_JPRB, 0.25510E+00_JPRB ,&
&  0.25552E+00_JPRB, 0.25978E+00_JPRB, 0.26517E+00_JPRB, 0.27203E+00_JPRB, 0.28370E+00_JPRB, 0.29360E+00_JPRB, 0.31613E+00_JPRB ,&
&  0.35206E+00_JPRB, 0.42028E+00_JPRB, 0.41437E+00_JPRB, 0.34573E+00_JPRB, 0.34864E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.34368E+00_JPRB, 0.34638E+00_JPRB, 0.34968E+00_JPRB, 0.35370E+00_JPRB, 0.35867E+00_JPRB, 0.36310E+00_JPRB, 0.36810E+00_JPRB ,&
&  0.37102E+00_JPRB, 0.36755E+00_JPRB, 0.36803E+00_JPRB, 0.36379E+00_JPRB, 0.35945E+00_JPRB, 0.35392E+00_JPRB, 0.34793E+00_JPRB ,&
&  0.34155E+00_JPRB, 0.33260E+00_JPRB, 0.32301E+00_JPRB, 0.31451E+00_JPRB, 0.30734E+00_JPRB, 0.30169E+00_JPRB, 0.29713E+00_JPRB ,&
&  0.29398E+00_JPRB, 0.29425E+00_JPRB, 0.29636E+00_JPRB, 0.30239E+00_JPRB, 0.31250E+00_JPRB, 0.32512E+00_JPRB, 0.34419E+00_JPRB ,&
&  0.37121E+00_JPRB, 0.37339E+00_JPRB, 0.37824E+00_JPRB, 0.37731E+00_JPRB, 0.37316E+00_JPRB, 0.37108E+00_JPRB, 0.36697E+00_JPRB ,&
&  0.36748E+00_JPRB, 0.36028E+00_JPRB, 0.37012E+00_JPRB, 0.35403E+00_JPRB, 0.32756E+00_JPRB, 0.31219E+00_JPRB, 0.29395E+00_JPRB ,&
&  0.28042E+00_JPRB, 0.26893E+00_JPRB, 0.26029E+00_JPRB, 0.25654E+00_JPRB, 0.25280E+00_JPRB, 0.25103E+00_JPRB, 0.24741E+00_JPRB ,&
&  0.24364E+00_JPRB, 0.24167E+00_JPRB, 0.23854E+00_JPRB, 0.23819E+00_JPRB, 0.23822E+00_JPRB, 0.23764E+00_JPRB, 0.23770E+00_JPRB ,&
&  0.23809E+00_JPRB, 0.23906E+00_JPRB, 0.24082E+00_JPRB, 0.24243E+00_JPRB, 0.24338E+00_JPRB, 0.24395E+00_JPRB, 0.24382E+00_JPRB ,&
&  0.24427E+00_JPRB, 0.24397E+00_JPRB, 0.24315E+00_JPRB, 0.24184E+00_JPRB, 0.23990E+00_JPRB, 0.23787E+00_JPRB, 0.23594E+00_JPRB ,&
&  0.23440E+00_JPRB, 0.23384E+00_JPRB, 0.23339E+00_JPRB, 0.23364E+00_JPRB, 0.23427E+00_JPRB, 0.23602E+00_JPRB, 0.23768E+00_JPRB ,&
&  0.23957E+00_JPRB, 0.24218E+00_JPRB, 0.24449E+00_JPRB, 0.24731E+00_JPRB, 0.24990E+00_JPRB, 0.25150E+00_JPRB, 0.25553E+00_JPRB ,&
&  0.26071E+00_JPRB, 0.26765E+00_JPRB, 0.27572E+00_JPRB, 0.28357E+00_JPRB, 0.29354E+00_JPRB, 0.30912E+00_JPRB, 0.33040E+00_JPRB ,&
&  0.35218E+00_JPRB, 0.37620E+00_JPRB, 0.37063E+00_JPRB, 0.34245E+00_JPRB, 0.34076E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.34598E+00_JPRB, 0.34395E+00_JPRB, 0.34437E+00_JPRB, 0.34526E+00_JPRB, 0.34866E+00_JPRB, 0.35378E+00_JPRB, 0.35934E+00_JPRB ,&
&  0.35886E+00_JPRB, 0.36192E+00_JPRB, 0.36113E+00_JPRB, 0.35861E+00_JPRB, 0.35436E+00_JPRB, 0.35046E+00_JPRB, 0.34568E+00_JPRB ,&
&  0.33887E+00_JPRB, 0.33015E+00_JPRB, 0.32162E+00_JPRB, 0.31297E+00_JPRB, 0.30472E+00_JPRB, 0.29613E+00_JPRB, 0.29091E+00_JPRB ,&
&  0.28766E+00_JPRB, 0.28678E+00_JPRB, 0.28878E+00_JPRB, 0.29161E+00_JPRB, 0.29700E+00_JPRB, 0.30434E+00_JPRB, 0.31407E+00_JPRB ,&
&  0.32365E+00_JPRB, 0.33428E+00_JPRB, 0.33979E+00_JPRB, 0.33968E+00_JPRB, 0.33944E+00_JPRB, 0.33734E+00_JPRB, 0.33387E+00_JPRB ,&
&  0.32742E+00_JPRB, 0.32003E+00_JPRB, 0.31153E+00_JPRB, 0.30255E+00_JPRB, 0.29339E+00_JPRB, 0.28466E+00_JPRB, 0.27654E+00_JPRB ,&
&  0.26994E+00_JPRB, 0.26434E+00_JPRB, 0.25923E+00_JPRB, 0.25496E+00_JPRB, 0.25164E+00_JPRB, 0.24916E+00_JPRB, 0.24620E+00_JPRB ,&
&  0.24371E+00_JPRB, 0.24150E+00_JPRB, 0.23890E+00_JPRB, 0.23708E+00_JPRB, 0.23546E+00_JPRB, 0.23438E+00_JPRB, 0.23348E+00_JPRB ,&
&  0.23334E+00_JPRB, 0.23321E+00_JPRB, 0.23374E+00_JPRB, 0.23427E+00_JPRB, 0.23492E+00_JPRB, 0.23483E+00_JPRB, 0.23458E+00_JPRB ,&
&  0.23415E+00_JPRB, 0.23373E+00_JPRB, 0.23243E+00_JPRB, 0.23143E+00_JPRB, 0.23074E+00_JPRB, 0.22998E+00_JPRB, 0.22997E+00_JPRB ,&
&  0.23083E+00_JPRB, 0.23171E+00_JPRB, 0.23246E+00_JPRB, 0.23405E+00_JPRB, 0.23565E+00_JPRB, 0.23795E+00_JPRB, 0.24070E+00_JPRB ,&
&  0.24274E+00_JPRB, 0.24515E+00_JPRB, 0.24782E+00_JPRB, 0.25093E+00_JPRB, 0.25462E+00_JPRB, 0.25847E+00_JPRB, 0.26295E+00_JPRB ,&
&  0.26800E+00_JPRB, 0.27319E+00_JPRB, 0.27905E+00_JPRB, 0.28549E+00_JPRB, 0.29328E+00_JPRB, 0.30022E+00_JPRB, 0.31044E+00_JPRB ,&
&  0.31864E+00_JPRB, 0.31862E+00_JPRB, 0.35898E+00_JPRB, 0.35268E+00_JPRB, 0.34846E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.28791E+00_JPRB, 0.29444E+00_JPRB, 0.29925E+00_JPRB, 0.30423E+00_JPRB, 0.30078E+00_JPRB, 0.33793E+00_JPRB, 0.33643E+00_JPRB ,&
&  0.33703E+00_JPRB, 0.32326E+00_JPRB, 0.32351E+00_JPRB, 0.32196E+00_JPRB, 0.31889E+00_JPRB, 0.31444E+00_JPRB, 0.31033E+00_JPRB ,&
&  0.30671E+00_JPRB, 0.30195E+00_JPRB, 0.29645E+00_JPRB, 0.29178E+00_JPRB, 0.28803E+00_JPRB, 0.28481E+00_JPRB, 0.28210E+00_JPRB ,&
&  0.28148E+00_JPRB, 0.28273E+00_JPRB, 0.28514E+00_JPRB, 0.28423E+00_JPRB, 0.28538E+00_JPRB, 0.28742E+00_JPRB, 0.28977E+00_JPRB ,&
&  0.29200E+00_JPRB, 0.29388E+00_JPRB, 0.29396E+00_JPRB, 0.29232E+00_JPRB, 0.29102E+00_JPRB, 0.28861E+00_JPRB, 0.28640E+00_JPRB ,&
&  0.28375E+00_JPRB, 0.28083E+00_JPRB, 0.27740E+00_JPRB, 0.27465E+00_JPRB, 0.27167E+00_JPRB, 0.26804E+00_JPRB, 0.26453E+00_JPRB ,&
&  0.26153E+00_JPRB, 0.25884E+00_JPRB, 0.25595E+00_JPRB, 0.25326E+00_JPRB, 0.25071E+00_JPRB, 0.24811E+00_JPRB, 0.24582E+00_JPRB ,&
&  0.24409E+00_JPRB, 0.24262E+00_JPRB, 0.24115E+00_JPRB, 0.23961E+00_JPRB, 0.23810E+00_JPRB, 0.23663E+00_JPRB, 0.23543E+00_JPRB ,&
&  0.23520E+00_JPRB, 0.23450E+00_JPRB, 0.23381E+00_JPRB, 0.23340E+00_JPRB, 0.23340E+00_JPRB, 0.23292E+00_JPRB, 0.23271E+00_JPRB ,&
&  0.23265E+00_JPRB, 0.23268E+00_JPRB, 0.23291E+00_JPRB, 0.23322E+00_JPRB, 0.23363E+00_JPRB, 0.23417E+00_JPRB, 0.23505E+00_JPRB ,&
&  0.23609E+00_JPRB, 0.23735E+00_JPRB, 0.23870E+00_JPRB, 0.24010E+00_JPRB, 0.24152E+00_JPRB, 0.24303E+00_JPRB, 0.24472E+00_JPRB ,&
&  0.24643E+00_JPRB, 0.24810E+00_JPRB, 0.24973E+00_JPRB, 0.25133E+00_JPRB, 0.25389E+00_JPRB, 0.25637E+00_JPRB, 0.25901E+00_JPRB ,&
&  0.26186E+00_JPRB, 0.26519E+00_JPRB, 0.26870E+00_JPRB, 0.27246E+00_JPRB, 0.27663E+00_JPRB, 0.27831E+00_JPRB, 0.27965E+00_JPRB ,&
&  0.28058E+00_JPRB, 0.28344E+00_JPRB, 0.28568E+00_JPRB, 0.28566E+00_JPRB, 0.28548E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.25963E+00_JPRB, 0.25985E+00_JPRB, 0.26002E+00_JPRB, 0.26018E+00_JPRB, 0.25921E+00_JPRB, 0.25980E+00_JPRB, 0.26030E+00_JPRB ,&
&  0.26104E+00_JPRB, 0.26231E+00_JPRB, 0.26379E+00_JPRB, 0.26491E+00_JPRB, 0.26574E+00_JPRB, 0.26623E+00_JPRB, 0.26657E+00_JPRB ,&
&  0.26659E+00_JPRB, 0.26676E+00_JPRB, 0.26373E+00_JPRB, 0.26266E+00_JPRB, 0.26235E+00_JPRB, 0.26242E+00_JPRB, 0.26264E+00_JPRB ,&
&  0.26281E+00_JPRB, 0.26284E+00_JPRB, 0.26241E+00_JPRB, 0.26270E+00_JPRB, 0.26301E+00_JPRB, 0.26322E+00_JPRB, 0.26348E+00_JPRB ,&
&  0.26370E+00_JPRB, 0.26388E+00_JPRB, 0.26382E+00_JPRB, 0.26361E+00_JPRB, 0.26344E+00_JPRB, 0.26291E+00_JPRB, 0.26204E+00_JPRB ,&
&  0.26098E+00_JPRB, 0.25966E+00_JPRB, 0.25836E+00_JPRB, 0.25716E+00_JPRB, 0.25612E+00_JPRB, 0.25523E+00_JPRB, 0.25438E+00_JPRB ,&
&  0.25337E+00_JPRB, 0.25207E+00_JPRB, 0.25070E+00_JPRB, 0.24936E+00_JPRB, 0.24809E+00_JPRB, 0.24692E+00_JPRB, 0.24571E+00_JPRB ,&
&  0.24477E+00_JPRB, 0.24397E+00_JPRB, 0.24318E+00_JPRB, 0.24249E+00_JPRB, 0.24183E+00_JPRB, 0.24121E+00_JPRB, 0.24061E+00_JPRB ,&
&  0.23981E+00_JPRB, 0.23935E+00_JPRB, 0.23884E+00_JPRB, 0.23859E+00_JPRB, 0.23862E+00_JPRB, 0.23861E+00_JPRB, 0.23870E+00_JPRB ,&
&  0.23884E+00_JPRB, 0.23904E+00_JPRB, 0.23916E+00_JPRB, 0.23936E+00_JPRB, 0.23967E+00_JPRB, 0.24007E+00_JPRB, 0.24056E+00_JPRB ,&
&  0.24109E+00_JPRB, 0.24167E+00_JPRB, 0.24230E+00_JPRB, 0.24302E+00_JPRB, 0.24376E+00_JPRB, 0.24459E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24616E+00_JPRB, 0.24689E+00_JPRB, 0.24762E+00_JPRB, 0.24818E+00_JPRB, 0.24897E+00_JPRB, 0.24974E+00_JPRB, 0.25065E+00_JPRB ,&
&  0.25158E+00_JPRB, 0.25256E+00_JPRB, 0.25341E+00_JPRB, 0.25433E+00_JPRB, 0.25533E+00_JPRB, 0.25520E+00_JPRB, 0.25512E+00_JPRB ,&
&  0.25511E+00_JPRB, 0.25536E+00_JPRB, 0.25540E+00_JPRB, 0.25574E+00_JPRB, 0.25657E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB, 0.24533E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB, 0.39587E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.39757E+00_JPRB, 0.39781E+00_JPRB, 0.39796E+00_JPRB, 0.39803E+00_JPRB, 0.39813E+00_JPRB, 0.39841E+00_JPRB, 0.39862E+00_JPRB ,&
&  0.39892E+00_JPRB, 0.39930E+00_JPRB, 0.39996E+00_JPRB, 0.40035E+00_JPRB, 0.40074E+00_JPRB, 0.40100E+00_JPRB, 0.40131E+00_JPRB ,&
&  0.40154E+00_JPRB, 0.40175E+00_JPRB, 0.40187E+00_JPRB, 0.40206E+00_JPRB, 0.40233E+00_JPRB, 0.40271E+00_JPRB, 0.40306E+00_JPRB ,&
&  0.40306E+00_JPRB, 0.40276E+00_JPRB, 0.40255E+00_JPRB, 0.40227E+00_JPRB, 0.40190E+00_JPRB, 0.40115E+00_JPRB, 0.40020E+00_JPRB ,&
&  0.39911E+00_JPRB, 0.39806E+00_JPRB, 0.39720E+00_JPRB, 0.39639E+00_JPRB, 0.39581E+00_JPRB, 0.39530E+00_JPRB, 0.39500E+00_JPRB ,&
&  0.39456E+00_JPRB, 0.39437E+00_JPRB, 0.39418E+00_JPRB, 0.39409E+00_JPRB, 0.39372E+00_JPRB, 0.39340E+00_JPRB, 0.39294E+00_JPRB ,&
&  0.39244E+00_JPRB, 0.39175E+00_JPRB, 0.39117E+00_JPRB, 0.39043E+00_JPRB, 0.38961E+00_JPRB, 0.38844E+00_JPRB, 0.38733E+00_JPRB ,&
&  0.38622E+00_JPRB, 0.38505E+00_JPRB, 0.38380E+00_JPRB, 0.38274E+00_JPRB, 0.38195E+00_JPRB, 0.38118E+00_JPRB, 0.38024E+00_JPRB ,&
&  0.37985E+00_JPRB, 0.37995E+00_JPRB, 0.38005E+00_JPRB, 0.37992E+00_JPRB, 0.38016E+00_JPRB, 0.38048E+00_JPRB, 0.38086E+00_JPRB ,&
&  0.38119E+00_JPRB, 0.38182E+00_JPRB, 0.38276E+00_JPRB, 0.38363E+00_JPRB, 0.38449E+00_JPRB, 0.38537E+00_JPRB, 0.38640E+00_JPRB ,&
&  0.38733E+00_JPRB, 0.38823E+00_JPRB, 0.38909E+00_JPRB, 0.39004E+00_JPRB, 0.39059E+00_JPRB, 0.39100E+00_JPRB, 0.39111E+00_JPRB ,&
&  0.39131E+00_JPRB, 0.39141E+00_JPRB, 0.39156E+00_JPRB, 0.39169E+00_JPRB, 0.39202E+00_JPRB, 0.39232E+00_JPRB, 0.39265E+00_JPRB ,&
&  0.39303E+00_JPRB, 0.39355E+00_JPRB, 0.39398E+00_JPRB, 0.39434E+00_JPRB, 0.39475E+00_JPRB, 0.39533E+00_JPRB, 0.39579E+00_JPRB ,&
&  0.39615E+00_JPRB, 0.39656E+00_JPRB, 0.39693E+00_JPRB, 0.39724E+00_JPRB, 0.39740E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.39276E+00_JPRB, 0.39436E+00_JPRB, 0.39536E+00_JPRB, 0.39626E+00_JPRB, 0.39717E+00_JPRB, 0.39854E+00_JPRB, 0.39966E+00_JPRB ,&
&  0.40088E+00_JPRB, 0.40250E+00_JPRB, 0.40460E+00_JPRB, 0.40601E+00_JPRB, 0.40711E+00_JPRB, 0.40786E+00_JPRB, 0.40857E+00_JPRB ,&
&  0.40883E+00_JPRB, 0.40876E+00_JPRB, 0.40827E+00_JPRB, 0.40796E+00_JPRB, 0.40768E+00_JPRB, 0.40747E+00_JPRB, 0.40709E+00_JPRB ,&
&  0.40609E+00_JPRB, 0.40476E+00_JPRB, 0.40326E+00_JPRB, 0.40102E+00_JPRB, 0.39836E+00_JPRB, 0.39546E+00_JPRB, 0.39225E+00_JPRB ,&
&  0.38906E+00_JPRB, 0.38647E+00_JPRB, 0.38393E+00_JPRB, 0.38148E+00_JPRB, 0.38030E+00_JPRB, 0.37934E+00_JPRB, 0.37841E+00_JPRB ,&
&  0.37746E+00_JPRB, 0.37622E+00_JPRB, 0.37603E+00_JPRB, 0.37609E+00_JPRB, 0.37648E+00_JPRB, 0.37750E+00_JPRB, 0.37848E+00_JPRB ,&
&  0.37910E+00_JPRB, 0.37912E+00_JPRB, 0.37866E+00_JPRB, 0.37843E+00_JPRB, 0.37815E+00_JPRB, 0.37761E+00_JPRB, 0.37666E+00_JPRB ,&
&  0.37591E+00_JPRB, 0.37468E+00_JPRB, 0.37315E+00_JPRB, 0.37144E+00_JPRB, 0.37023E+00_JPRB, 0.36886E+00_JPRB, 0.36743E+00_JPRB ,&
&  0.36640E+00_JPRB, 0.36586E+00_JPRB, 0.36550E+00_JPRB, 0.36524E+00_JPRB, 0.36552E+00_JPRB, 0.36609E+00_JPRB, 0.36668E+00_JPRB ,&
&  0.36731E+00_JPRB, 0.36864E+00_JPRB, 0.37037E+00_JPRB, 0.37177E+00_JPRB, 0.37298E+00_JPRB, 0.37413E+00_JPRB, 0.37565E+00_JPRB ,&
&  0.37664E+00_JPRB, 0.37725E+00_JPRB, 0.37708E+00_JPRB, 0.37756E+00_JPRB, 0.37782E+00_JPRB, 0.37846E+00_JPRB, 0.37857E+00_JPRB ,&
&  0.37942E+00_JPRB, 0.37969E+00_JPRB, 0.37976E+00_JPRB, 0.38005E+00_JPRB, 0.38049E+00_JPRB, 0.38085E+00_JPRB, 0.38092E+00_JPRB ,&
&  0.38103E+00_JPRB, 0.38130E+00_JPRB, 0.38156E+00_JPRB, 0.38179E+00_JPRB, 0.38249E+00_JPRB, 0.38339E+00_JPRB, 0.38440E+00_JPRB ,&
&  0.38553E+00_JPRB, 0.38708E+00_JPRB, 0.38876E+00_JPRB, 0.39009E+00_JPRB, 0.39130E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.38825E+00_JPRB, 0.38954E+00_JPRB, 0.39052E+00_JPRB, 0.39198E+00_JPRB, 0.39462E+00_JPRB, 0.39786E+00_JPRB, 0.39999E+00_JPRB ,&
&  0.40279E+00_JPRB, 0.40604E+00_JPRB, 0.40954E+00_JPRB, 0.41171E+00_JPRB, 0.41332E+00_JPRB, 0.41407E+00_JPRB, 0.41406E+00_JPRB ,&
&  0.41311E+00_JPRB, 0.41185E+00_JPRB, 0.41059E+00_JPRB, 0.40934E+00_JPRB, 0.40806E+00_JPRB, 0.40666E+00_JPRB, 0.40424E+00_JPRB ,&
&  0.40196E+00_JPRB, 0.40103E+00_JPRB, 0.39759E+00_JPRB, 0.39316E+00_JPRB, 0.38930E+00_JPRB, 0.36041E+00_JPRB, 0.35028E+00_JPRB ,&
&  0.34793E+00_JPRB, 0.34567E+00_JPRB, 0.34651E+00_JPRB, 0.34775E+00_JPRB, 0.35365E+00_JPRB, 0.35085E+00_JPRB, 0.34920E+00_JPRB ,&
&  0.33289E+00_JPRB, 0.33171E+00_JPRB, 0.32859E+00_JPRB, 0.32694E+00_JPRB, 0.32866E+00_JPRB, 0.33092E+00_JPRB, 0.33757E+00_JPRB ,&
&  0.35384E+00_JPRB, 0.35893E+00_JPRB, 0.36123E+00_JPRB, 0.36258E+00_JPRB, 0.36348E+00_JPRB, 0.36380E+00_JPRB, 0.36414E+00_JPRB ,&
&  0.36433E+00_JPRB, 0.36404E+00_JPRB, 0.36350E+00_JPRB, 0.36258E+00_JPRB, 0.36154E+00_JPRB, 0.36027E+00_JPRB, 0.35925E+00_JPRB ,&
&  0.35853E+00_JPRB, 0.35842E+00_JPRB, 0.35876E+00_JPRB, 0.35894E+00_JPRB, 0.35950E+00_JPRB, 0.35967E+00_JPRB, 0.35985E+00_JPRB ,&
&  0.35985E+00_JPRB, 0.35997E+00_JPRB, 0.36071E+00_JPRB, 0.36168E+00_JPRB, 0.36291E+00_JPRB, 0.36323E+00_JPRB, 0.36423E+00_JPRB ,&
&  0.36496E+00_JPRB, 0.36615E+00_JPRB, 0.36685E+00_JPRB, 0.36858E+00_JPRB, 0.36938E+00_JPRB, 0.37127E+00_JPRB, 0.37273E+00_JPRB ,&
&  0.37533E+00_JPRB, 0.37685E+00_JPRB, 0.37838E+00_JPRB, 0.37936E+00_JPRB, 0.38027E+00_JPRB, 0.38094E+00_JPRB, 0.38098E+00_JPRB ,&
&  0.38103E+00_JPRB, 0.38048E+00_JPRB, 0.37997E+00_JPRB, 0.37929E+00_JPRB, 0.37902E+00_JPRB, 0.37866E+00_JPRB, 0.37872E+00_JPRB ,&
&  0.37861E+00_JPRB, 0.37970E+00_JPRB, 0.38180E+00_JPRB, 0.38430E+00_JPRB, 0.38636E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.39944E+00_JPRB, 0.39952E+00_JPRB, 0.39897E+00_JPRB, 0.40041E+00_JPRB, 0.40239E+00_JPRB, 0.40627E+00_JPRB, 0.40691E+00_JPRB ,&
&  0.41048E+00_JPRB, 0.41396E+00_JPRB, 0.41753E+00_JPRB, 0.41883E+00_JPRB, 0.41959E+00_JPRB, 0.41913E+00_JPRB, 0.41739E+00_JPRB ,&
&  0.41540E+00_JPRB, 0.41232E+00_JPRB, 0.40963E+00_JPRB, 0.40635E+00_JPRB, 0.40295E+00_JPRB, 0.40135E+00_JPRB, 0.40033E+00_JPRB ,&
&  0.40021E+00_JPRB, 0.39923E+00_JPRB, 0.37240E+00_JPRB, 0.36239E+00_JPRB, 0.34800E+00_JPRB, 0.34003E+00_JPRB, 0.33569E+00_JPRB ,&
&  0.33166E+00_JPRB, 0.32762E+00_JPRB, 0.32240E+00_JPRB, 0.31410E+00_JPRB, 0.30345E+00_JPRB, 0.28922E+00_JPRB, 0.28194E+00_JPRB ,&
&  0.27575E+00_JPRB, 0.27404E+00_JPRB, 0.27486E+00_JPRB, 0.27851E+00_JPRB, 0.28589E+00_JPRB, 0.29482E+00_JPRB, 0.30609E+00_JPRB ,&
&  0.32193E+00_JPRB, 0.33755E+00_JPRB, 0.34672E+00_JPRB, 0.34523E+00_JPRB, 0.35325E+00_JPRB, 0.35710E+00_JPRB, 0.35903E+00_JPRB ,&
&  0.35975E+00_JPRB, 0.35809E+00_JPRB, 0.36003E+00_JPRB, 0.35909E+00_JPRB, 0.35863E+00_JPRB, 0.35890E+00_JPRB, 0.35956E+00_JPRB ,&
&  0.36114E+00_JPRB, 0.36264E+00_JPRB, 0.36354E+00_JPRB, 0.36412E+00_JPRB, 0.36398E+00_JPRB, 0.36329E+00_JPRB, 0.36254E+00_JPRB ,&
&  0.36184E+00_JPRB, 0.36127E+00_JPRB, 0.36086E+00_JPRB, 0.36092E+00_JPRB, 0.36177E+00_JPRB, 0.36225E+00_JPRB, 0.36213E+00_JPRB ,&
&  0.36274E+00_JPRB, 0.36387E+00_JPRB, 0.36519E+00_JPRB, 0.36460E+00_JPRB, 0.36641E+00_JPRB, 0.37188E+00_JPRB, 0.37462E+00_JPRB ,&
&  0.37977E+00_JPRB, 0.38381E+00_JPRB, 0.38880E+00_JPRB, 0.39249E+00_JPRB, 0.39547E+00_JPRB, 0.39666E+00_JPRB, 0.39618E+00_JPRB ,&
&  0.39539E+00_JPRB, 0.39420E+00_JPRB, 0.39354E+00_JPRB, 0.39200E+00_JPRB, 0.39110E+00_JPRB, 0.38965E+00_JPRB, 0.38941E+00_JPRB ,&
&  0.38929E+00_JPRB, 0.39039E+00_JPRB, 0.39054E+00_JPRB, 0.39375E+00_JPRB, 0.39657E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.41478E+00_JPRB, 0.41455E+00_JPRB, 0.41498E+00_JPRB, 0.41871E+00_JPRB, 0.42565E+00_JPRB, 0.43352E+00_JPRB, 0.43228E+00_JPRB ,&
&  0.43149E+00_JPRB, 0.42807E+00_JPRB, 0.42837E+00_JPRB, 0.42685E+00_JPRB, 0.42446E+00_JPRB, 0.42123E+00_JPRB, 0.41842E+00_JPRB ,&
&  0.41613E+00_JPRB, 0.41072E+00_JPRB, 0.40423E+00_JPRB, 0.40334E+00_JPRB, 0.40537E+00_JPRB, 0.40656E+00_JPRB, 0.40551E+00_JPRB ,&
&  0.40195E+00_JPRB, 0.39337E+00_JPRB, 0.37205E+00_JPRB, 0.35570E+00_JPRB, 0.34077E+00_JPRB, 0.33548E+00_JPRB, 0.32685E+00_JPRB ,&
&  0.33239E+00_JPRB, 0.30387E+00_JPRB, 0.30281E+00_JPRB, 0.29020E+00_JPRB, 0.27976E+00_JPRB, 0.26854E+00_JPRB, 0.25885E+00_JPRB ,&
&  0.25557E+00_JPRB, 0.25234E+00_JPRB, 0.25200E+00_JPRB, 0.25686E+00_JPRB, 0.26333E+00_JPRB, 0.27505E+00_JPRB, 0.28900E+00_JPRB ,&
&  0.30231E+00_JPRB, 0.33769E+00_JPRB, 0.34730E+00_JPRB, 0.35072E+00_JPRB, 0.35858E+00_JPRB, 0.36363E+00_JPRB, 0.36400E+00_JPRB ,&
&  0.36359E+00_JPRB, 0.36150E+00_JPRB, 0.36264E+00_JPRB, 0.36312E+00_JPRB, 0.36630E+00_JPRB, 0.36601E+00_JPRB, 0.36588E+00_JPRB ,&
&  0.36747E+00_JPRB, 0.36918E+00_JPRB, 0.37131E+00_JPRB, 0.37308E+00_JPRB, 0.37489E+00_JPRB, 0.37592E+00_JPRB, 0.37663E+00_JPRB ,&
&  0.37658E+00_JPRB, 0.37631E+00_JPRB, 0.37560E+00_JPRB, 0.37590E+00_JPRB, 0.37607E+00_JPRB, 0.37820E+00_JPRB, 0.37892E+00_JPRB ,&
&  0.37952E+00_JPRB, 0.37961E+00_JPRB, 0.37916E+00_JPRB, 0.37998E+00_JPRB, 0.38221E+00_JPRB, 0.38416E+00_JPRB, 0.38798E+00_JPRB ,&
&  0.39469E+00_JPRB, 0.39936E+00_JPRB, 0.40500E+00_JPRB, 0.40976E+00_JPRB, 0.41357E+00_JPRB, 0.41378E+00_JPRB, 0.41172E+00_JPRB ,&
&  0.41041E+00_JPRB, 0.40905E+00_JPRB, 0.40768E+00_JPRB, 0.40546E+00_JPRB, 0.40341E+00_JPRB, 0.40097E+00_JPRB, 0.39986E+00_JPRB ,&
&  0.39984E+00_JPRB, 0.40213E+00_JPRB, 0.40255E+00_JPRB, 0.40629E+00_JPRB, 0.41079E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.42557E+00_JPRB, 0.42779E+00_JPRB, 0.43296E+00_JPRB, 0.43903E+00_JPRB, 0.44517E+00_JPRB, 0.45142E+00_JPRB, 0.45376E+00_JPRB ,&
&  0.45496E+00_JPRB, 0.45105E+00_JPRB, 0.44578E+00_JPRB, 0.43341E+00_JPRB, 0.42506E+00_JPRB, 0.42096E+00_JPRB, 0.41621E+00_JPRB ,&
&  0.41191E+00_JPRB, 0.41030E+00_JPRB, 0.40962E+00_JPRB, 0.41020E+00_JPRB, 0.40862E+00_JPRB, 0.40801E+00_JPRB, 0.40241E+00_JPRB ,&
&  0.38986E+00_JPRB, 0.37902E+00_JPRB, 0.36155E+00_JPRB, 0.34540E+00_JPRB, 0.33446E+00_JPRB, 0.32605E+00_JPRB, 0.32589E+00_JPRB ,&
&  0.32431E+00_JPRB, 0.32362E+00_JPRB, 0.32301E+00_JPRB, 0.32207E+00_JPRB, 0.32461E+00_JPRB, 0.27244E+00_JPRB, 0.25881E+00_JPRB ,&
&  0.24971E+00_JPRB, 0.24383E+00_JPRB, 0.24244E+00_JPRB, 0.24322E+00_JPRB, 0.24817E+00_JPRB, 0.26240E+00_JPRB, 0.28253E+00_JPRB ,&
&  0.30040E+00_JPRB, 0.34884E+00_JPRB, 0.35731E+00_JPRB, 0.36526E+00_JPRB, 0.37662E+00_JPRB, 0.38376E+00_JPRB, 0.38711E+00_JPRB ,&
&  0.39220E+00_JPRB, 0.39966E+00_JPRB, 0.40665E+00_JPRB, 0.41186E+00_JPRB, 0.40991E+00_JPRB, 0.40886E+00_JPRB, 0.40569E+00_JPRB ,&
&  0.39891E+00_JPRB, 0.39238E+00_JPRB, 0.39070E+00_JPRB, 0.39005E+00_JPRB, 0.39054E+00_JPRB, 0.39095E+00_JPRB, 0.39111E+00_JPRB ,&
&  0.39292E+00_JPRB, 0.39414E+00_JPRB, 0.39777E+00_JPRB, 0.39858E+00_JPRB, 0.40057E+00_JPRB, 0.40356E+00_JPRB, 0.40560E+00_JPRB ,&
&  0.40544E+00_JPRB, 0.40553E+00_JPRB, 0.40296E+00_JPRB, 0.39975E+00_JPRB, 0.39846E+00_JPRB, 0.39716E+00_JPRB, 0.39895E+00_JPRB ,&
&  0.40188E+00_JPRB, 0.40840E+00_JPRB, 0.41171E+00_JPRB, 0.41563E+00_JPRB, 0.41914E+00_JPRB, 0.41940E+00_JPRB, 0.41946E+00_JPRB ,&
&  0.41909E+00_JPRB, 0.41945E+00_JPRB, 0.41831E+00_JPRB, 0.41641E+00_JPRB, 0.41284E+00_JPRB, 0.40935E+00_JPRB, 0.40643E+00_JPRB ,&
&  0.40695E+00_JPRB, 0.40998E+00_JPRB, 0.41112E+00_JPRB, 0.41462E+00_JPRB, 0.42027E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.43543E+00_JPRB, 0.43954E+00_JPRB, 0.44715E+00_JPRB, 0.45299E+00_JPRB, 0.46094E+00_JPRB, 0.46663E+00_JPRB, 0.46877E+00_JPRB ,&
&  0.47583E+00_JPRB, 0.47088E+00_JPRB, 0.45696E+00_JPRB, 0.43931E+00_JPRB, 0.42217E+00_JPRB, 0.41155E+00_JPRB, 0.40670E+00_JPRB ,&
&  0.40689E+00_JPRB, 0.40329E+00_JPRB, 0.40076E+00_JPRB, 0.40026E+00_JPRB, 0.39978E+00_JPRB, 0.39912E+00_JPRB, 0.39428E+00_JPRB ,&
&  0.37967E+00_JPRB, 0.36885E+00_JPRB, 0.35743E+00_JPRB, 0.34334E+00_JPRB, 0.33103E+00_JPRB, 0.32446E+00_JPRB, 0.32803E+00_JPRB ,&
&  0.33205E+00_JPRB, 0.32656E+00_JPRB, 0.32482E+00_JPRB, 0.32345E+00_JPRB, 0.32283E+00_JPRB, 0.31967E+00_JPRB, 0.27324E+00_JPRB ,&
&  0.25749E+00_JPRB, 0.24613E+00_JPRB, 0.24200E+00_JPRB, 0.24271E+00_JPRB, 0.25300E+00_JPRB, 0.27026E+00_JPRB, 0.30120E+00_JPRB ,&
&  0.33341E+00_JPRB, 0.36228E+00_JPRB, 0.37796E+00_JPRB, 0.39074E+00_JPRB, 0.40477E+00_JPRB, 0.41781E+00_JPRB, 0.42633E+00_JPRB ,&
&  0.43788E+00_JPRB, 0.45147E+00_JPRB, 0.46116E+00_JPRB, 0.46829E+00_JPRB, 0.47125E+00_JPRB, 0.46841E+00_JPRB, 0.46624E+00_JPRB ,&
&  0.45945E+00_JPRB, 0.42745E+00_JPRB, 0.41261E+00_JPRB, 0.40235E+00_JPRB, 0.39592E+00_JPRB, 0.39622E+00_JPRB, 0.39658E+00_JPRB ,&
&  0.40175E+00_JPRB, 0.40807E+00_JPRB, 0.41071E+00_JPRB, 0.41293E+00_JPRB, 0.41393E+00_JPRB, 0.41341E+00_JPRB, 0.41479E+00_JPRB ,&
&  0.41768E+00_JPRB, 0.42353E+00_JPRB, 0.43598E+00_JPRB, 0.42777E+00_JPRB, 0.41676E+00_JPRB, 0.40640E+00_JPRB, 0.40115E+00_JPRB ,&
&  0.39987E+00_JPRB, 0.40205E+00_JPRB, 0.40673E+00_JPRB, 0.40887E+00_JPRB, 0.41135E+00_JPRB, 0.41447E+00_JPRB, 0.42171E+00_JPRB ,&
&  0.42329E+00_JPRB, 0.42531E+00_JPRB, 0.42593E+00_JPRB, 0.42566E+00_JPRB, 0.42203E+00_JPRB, 0.41856E+00_JPRB, 0.41638E+00_JPRB ,&
&  0.41755E+00_JPRB, 0.42018E+00_JPRB, 0.42247E+00_JPRB, 0.42592E+00_JPRB, 0.43109E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.43747E+00_JPRB, 0.44650E+00_JPRB, 0.46007E+00_JPRB, 0.46932E+00_JPRB, 0.47912E+00_JPRB, 0.47212E+00_JPRB, 0.46437E+00_JPRB ,&
&  0.46195E+00_JPRB, 0.45856E+00_JPRB, 0.44776E+00_JPRB, 0.43397E+00_JPRB, 0.41683E+00_JPRB, 0.40410E+00_JPRB, 0.39938E+00_JPRB ,&
&  0.39921E+00_JPRB, 0.39747E+00_JPRB, 0.39532E+00_JPRB, 0.39024E+00_JPRB, 0.38503E+00_JPRB, 0.38443E+00_JPRB, 0.38032E+00_JPRB ,&
&  0.37080E+00_JPRB, 0.36298E+00_JPRB, 0.35467E+00_JPRB, 0.34620E+00_JPRB, 0.33729E+00_JPRB, 0.33524E+00_JPRB, 0.33460E+00_JPRB ,&
&  0.33332E+00_JPRB, 0.33424E+00_JPRB, 0.33748E+00_JPRB, 0.33146E+00_JPRB, 0.33235E+00_JPRB, 0.33688E+00_JPRB, 0.30357E+00_JPRB ,&
&  0.27442E+00_JPRB, 0.25567E+00_JPRB, 0.24591E+00_JPRB, 0.24664E+00_JPRB, 0.26061E+00_JPRB, 0.28962E+00_JPRB, 0.32622E+00_JPRB ,&
&  0.36526E+00_JPRB, 0.39294E+00_JPRB, 0.41406E+00_JPRB, 0.42898E+00_JPRB, 0.44044E+00_JPRB, 0.45364E+00_JPRB, 0.46699E+00_JPRB ,&
&  0.48210E+00_JPRB, 0.50206E+00_JPRB, 0.51553E+00_JPRB, 0.52795E+00_JPRB, 0.53143E+00_JPRB, 0.49740E+00_JPRB, 0.51556E+00_JPRB ,&
&  0.60303E+00_JPRB, 0.51383E+00_JPRB, 0.44391E+00_JPRB, 0.41613E+00_JPRB, 0.41272E+00_JPRB, 0.41877E+00_JPRB, 0.42787E+00_JPRB ,&
&  0.43783E+00_JPRB, 0.44104E+00_JPRB, 0.43881E+00_JPRB, 0.43503E+00_JPRB, 0.43356E+00_JPRB, 0.43423E+00_JPRB, 0.44218E+00_JPRB ,&
&  0.45503E+00_JPRB, 0.52255E+00_JPRB, 0.65886E+00_JPRB, 0.50405E+00_JPRB, 0.44702E+00_JPRB, 0.41433E+00_JPRB, 0.39207E+00_JPRB ,&
&  0.38367E+00_JPRB, 0.38253E+00_JPRB, 0.38704E+00_JPRB, 0.38941E+00_JPRB, 0.38813E+00_JPRB, 0.38967E+00_JPRB, 0.39347E+00_JPRB ,&
&  0.40303E+00_JPRB, 0.41316E+00_JPRB, 0.41991E+00_JPRB, 0.42594E+00_JPRB, 0.42598E+00_JPRB, 0.42218E+00_JPRB, 0.41949E+00_JPRB ,&
&  0.42331E+00_JPRB, 0.42807E+00_JPRB, 0.43191E+00_JPRB, 0.43362E+00_JPRB, 0.43433E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.43676E+00_JPRB, 0.46120E+00_JPRB, 0.48813E+00_JPRB, 0.48861E+00_JPRB, 0.48962E+00_JPRB, 0.46265E+00_JPRB, 0.43286E+00_JPRB ,&
&  0.41261E+00_JPRB, 0.40093E+00_JPRB, 0.39704E+00_JPRB, 0.39017E+00_JPRB, 0.38510E+00_JPRB, 0.38304E+00_JPRB, 0.38145E+00_JPRB ,&
&  0.38068E+00_JPRB, 0.37896E+00_JPRB, 0.37683E+00_JPRB, 0.37361E+00_JPRB, 0.37320E+00_JPRB, 0.37638E+00_JPRB, 0.37372E+00_JPRB ,&
&  0.36631E+00_JPRB, 0.36205E+00_JPRB, 0.35191E+00_JPRB, 0.33962E+00_JPRB, 0.33446E+00_JPRB, 0.33768E+00_JPRB, 0.34479E+00_JPRB ,&
&  0.34304E+00_JPRB, 0.33815E+00_JPRB, 0.34442E+00_JPRB, 0.35431E+00_JPRB, 0.34605E+00_JPRB, 0.34771E+00_JPRB, 0.31388E+00_JPRB ,&
&  0.28111E+00_JPRB, 0.26166E+00_JPRB, 0.25637E+00_JPRB, 0.27694E+00_JPRB, 0.31777E+00_JPRB, 0.37469E+00_JPRB, 0.40373E+00_JPRB ,&
&  0.42638E+00_JPRB, 0.44808E+00_JPRB, 0.46749E+00_JPRB, 0.48729E+00_JPRB, 0.51411E+00_JPRB, 0.53966E+00_JPRB, 0.56850E+00_JPRB ,&
&  0.58867E+00_JPRB, 0.59632E+00_JPRB, 0.58988E+00_JPRB, 0.52980E+00_JPRB, 0.50925E+00_JPRB, 0.48529E+00_JPRB, 0.50743E+00_JPRB ,&
&  0.53180E+00_JPRB, 0.49774E+00_JPRB, 0.47439E+00_JPRB, 0.47648E+00_JPRB, 0.47798E+00_JPRB, 0.48653E+00_JPRB, 0.48970E+00_JPRB ,&
&  0.48107E+00_JPRB, 0.46442E+00_JPRB, 0.44868E+00_JPRB, 0.43785E+00_JPRB, 0.43869E+00_JPRB, 0.44788E+00_JPRB, 0.45853E+00_JPRB ,&
&  0.47728E+00_JPRB, 0.52037E+00_JPRB, 0.51717E+00_JPRB, 0.41695E+00_JPRB, 0.40049E+00_JPRB, 0.39392E+00_JPRB, 0.38883E+00_JPRB ,&
&  0.37901E+00_JPRB, 0.37344E+00_JPRB, 0.37415E+00_JPRB, 0.37534E+00_JPRB, 0.37016E+00_JPRB, 0.36469E+00_JPRB, 0.36193E+00_JPRB ,&
&  0.36439E+00_JPRB, 0.37143E+00_JPRB, 0.38170E+00_JPRB, 0.40293E+00_JPRB, 0.41560E+00_JPRB, 0.42056E+00_JPRB, 0.41710E+00_JPRB ,&
&  0.41458E+00_JPRB, 0.41689E+00_JPRB, 0.41758E+00_JPRB, 0.41551E+00_JPRB, 0.42135E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.51260E+00_JPRB, 0.51420E+00_JPRB, 0.50176E+00_JPRB, 0.49913E+00_JPRB, 0.48402E+00_JPRB, 0.47339E+00_JPRB, 0.44896E+00_JPRB ,&
&  0.44207E+00_JPRB, 0.41603E+00_JPRB, 0.39201E+00_JPRB, 0.36704E+00_JPRB, 0.35562E+00_JPRB, 0.35167E+00_JPRB, 0.35385E+00_JPRB ,&
&  0.35917E+00_JPRB, 0.36522E+00_JPRB, 0.36993E+00_JPRB, 0.36587E+00_JPRB, 0.36924E+00_JPRB, 0.37202E+00_JPRB, 0.37039E+00_JPRB ,&
&  0.36697E+00_JPRB, 0.36581E+00_JPRB, 0.36011E+00_JPRB, 0.35121E+00_JPRB, 0.34671E+00_JPRB, 0.34578E+00_JPRB, 0.34248E+00_JPRB ,&
&  0.33596E+00_JPRB, 0.33669E+00_JPRB, 0.34930E+00_JPRB, 0.37428E+00_JPRB, 0.35656E+00_JPRB, 0.35383E+00_JPRB, 0.31290E+00_JPRB ,&
&  0.28167E+00_JPRB, 0.27818E+00_JPRB, 0.30202E+00_JPRB, 0.36813E+00_JPRB, 0.39010E+00_JPRB, 0.42063E+00_JPRB, 0.43932E+00_JPRB ,&
&  0.43846E+00_JPRB, 0.45211E+00_JPRB, 0.49224E+00_JPRB, 0.56857E+00_JPRB, 0.59538E+00_JPRB, 0.62977E+00_JPRB, 0.66308E+00_JPRB ,&
&  0.68614E+00_JPRB, 0.69738E+00_JPRB, 0.69652E+00_JPRB, 0.65667E+00_JPRB, 0.66373E+00_JPRB, 0.68458E+00_JPRB, 0.57177E+00_JPRB ,&
&  0.55350E+00_JPRB, 0.53061E+00_JPRB, 0.53201E+00_JPRB, 0.55529E+00_JPRB, 0.53450E+00_JPRB, 0.53530E+00_JPRB, 0.51947E+00_JPRB ,&
&  0.48624E+00_JPRB, 0.46711E+00_JPRB, 0.45419E+00_JPRB, 0.45134E+00_JPRB, 0.45333E+00_JPRB, 0.46330E+00_JPRB, 0.47911E+00_JPRB ,&
&  0.49806E+00_JPRB, 0.50539E+00_JPRB, 0.48094E+00_JPRB, 0.42483E+00_JPRB, 0.40390E+00_JPRB, 0.39400E+00_JPRB, 0.38645E+00_JPRB ,&
&  0.37875E+00_JPRB, 0.37503E+00_JPRB, 0.37672E+00_JPRB, 0.37675E+00_JPRB, 0.37047E+00_JPRB, 0.35755E+00_JPRB, 0.34629E+00_JPRB ,&
&  0.34175E+00_JPRB, 0.34230E+00_JPRB, 0.34285E+00_JPRB, 0.36116E+00_JPRB, 0.38415E+00_JPRB, 0.40397E+00_JPRB, 0.40765E+00_JPRB ,&
&  0.41006E+00_JPRB, 0.41975E+00_JPRB, 0.43094E+00_JPRB, 0.45683E+00_JPRB, 0.48184E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.52178E+00_JPRB, 0.52472E+00_JPRB, 0.51995E+00_JPRB, 0.50967E+00_JPRB, 0.50750E+00_JPRB, 0.48283E+00_JPRB, 0.42218E+00_JPRB ,&
&  0.42440E+00_JPRB, 0.37911E+00_JPRB, 0.36179E+00_JPRB, 0.35322E+00_JPRB, 0.35403E+00_JPRB, 0.34961E+00_JPRB, 0.34545E+00_JPRB ,&
&  0.34564E+00_JPRB, 0.35512E+00_JPRB, 0.36522E+00_JPRB, 0.36486E+00_JPRB, 0.36860E+00_JPRB, 0.37170E+00_JPRB, 0.37014E+00_JPRB ,&
&  0.37260E+00_JPRB, 0.37567E+00_JPRB, 0.37641E+00_JPRB, 0.37241E+00_JPRB, 0.36184E+00_JPRB, 0.35346E+00_JPRB, 0.34468E+00_JPRB ,&
&  0.33897E+00_JPRB, 0.34099E+00_JPRB, 0.35154E+00_JPRB, 0.37532E+00_JPRB, 0.36649E+00_JPRB, 0.35589E+00_JPRB, 0.34556E+00_JPRB ,&
&  0.29831E+00_JPRB, 0.31498E+00_JPRB, 0.38190E+00_JPRB, 0.40622E+00_JPRB, 0.43079E+00_JPRB, 0.46198E+00_JPRB, 0.50516E+00_JPRB ,&
&  0.54374E+00_JPRB, 0.58397E+00_JPRB, 0.62925E+00_JPRB, 0.68031E+00_JPRB, 0.73406E+00_JPRB, 0.78006E+00_JPRB, 0.80846E+00_JPRB ,&
&  0.84368E+00_JPRB, 0.68582E+00_JPRB, 0.65727E+00_JPRB, 0.76080E+00_JPRB, 0.92475E+00_JPRB, 0.73388E+00_JPRB, 0.65773E+00_JPRB ,&
&  0.62394E+00_JPRB, 0.58964E+00_JPRB, 0.57785E+00_JPRB, 0.59855E+00_JPRB, 0.59169E+00_JPRB, 0.59506E+00_JPRB, 0.59736E+00_JPRB ,&
&  0.54784E+00_JPRB, 0.52147E+00_JPRB, 0.50222E+00_JPRB, 0.50327E+00_JPRB, 0.51111E+00_JPRB, 0.52134E+00_JPRB, 0.52404E+00_JPRB ,&
&  0.51717E+00_JPRB, 0.50635E+00_JPRB, 0.48447E+00_JPRB, 0.45169E+00_JPRB, 0.42909E+00_JPRB, 0.41639E+00_JPRB, 0.40625E+00_JPRB ,&
&  0.39765E+00_JPRB, 0.38818E+00_JPRB, 0.37995E+00_JPRB, 0.37529E+00_JPRB, 0.37372E+00_JPRB, 0.37338E+00_JPRB, 0.37725E+00_JPRB ,&
&  0.36453E+00_JPRB, 0.35609E+00_JPRB, 0.34801E+00_JPRB, 0.35958E+00_JPRB, 0.38359E+00_JPRB, 0.40617E+00_JPRB, 0.42264E+00_JPRB ,&
&  0.44370E+00_JPRB, 0.45732E+00_JPRB, 0.45377E+00_JPRB, 0.46746E+00_JPRB, 0.48765E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.52769E+00_JPRB, 0.53472E+00_JPRB, 0.52968E+00_JPRB, 0.53001E+00_JPRB, 0.52661E+00_JPRB, 0.48204E+00_JPRB, 0.44199E+00_JPRB ,&
&  0.50178E+00_JPRB, 0.43204E+00_JPRB, 0.46402E+00_JPRB, 0.38848E+00_JPRB, 0.36409E+00_JPRB, 0.35925E+00_JPRB, 0.34406E+00_JPRB ,&
&  0.32821E+00_JPRB, 0.33854E+00_JPRB, 0.35044E+00_JPRB, 0.35276E+00_JPRB, 0.35886E+00_JPRB, 0.36501E+00_JPRB, 0.36473E+00_JPRB ,&
&  0.36910E+00_JPRB, 0.37803E+00_JPRB, 0.39006E+00_JPRB, 0.38273E+00_JPRB, 0.37088E+00_JPRB, 0.36424E+00_JPRB, 0.35695E+00_JPRB ,&
&  0.34326E+00_JPRB, 0.33790E+00_JPRB, 0.35034E+00_JPRB, 0.37085E+00_JPRB, 0.38156E+00_JPRB, 0.36536E+00_JPRB, 0.35786E+00_JPRB ,&
&  0.35986E+00_JPRB, 0.36192E+00_JPRB, 0.41375E+00_JPRB, 0.44005E+00_JPRB, 0.47570E+00_JPRB, 0.52270E+00_JPRB, 0.56920E+00_JPRB ,&
&  0.62374E+00_JPRB, 0.68104E+00_JPRB, 0.74289E+00_JPRB, 0.81335E+00_JPRB, 0.87633E+00_JPRB, 0.93854E+00_JPRB, 0.98944E+00_JPRB ,&
&  0.10602E+01_JPRB, 0.79405E+00_JPRB, 0.95756E+00_JPRB, 0.10110E+01_JPRB, 0.11490E+01_JPRB, 0.10738E+01_JPRB, 0.90972E+00_JPRB ,&
&  0.89041E+00_JPRB, 0.78170E+00_JPRB, 0.71262E+00_JPRB, 0.65218E+00_JPRB, 0.63141E+00_JPRB, 0.64805E+00_JPRB, 0.66715E+00_JPRB ,&
&  0.69815E+00_JPRB, 0.68179E+00_JPRB, 0.65022E+00_JPRB, 0.64421E+00_JPRB, 0.64322E+00_JPRB, 0.63912E+00_JPRB, 0.62286E+00_JPRB ,&
&  0.59518E+00_JPRB, 0.56341E+00_JPRB, 0.52944E+00_JPRB, 0.48761E+00_JPRB, 0.46662E+00_JPRB, 0.44909E+00_JPRB, 0.43092E+00_JPRB ,&
&  0.42106E+00_JPRB, 0.40046E+00_JPRB, 0.37742E+00_JPRB, 0.36472E+00_JPRB, 0.35552E+00_JPRB, 0.35341E+00_JPRB, 0.36077E+00_JPRB ,&
&  0.36253E+00_JPRB, 0.36107E+00_JPRB, 0.38752E+00_JPRB, 0.40754E+00_JPRB, 0.41948E+00_JPRB, 0.43512E+00_JPRB, 0.48200E+00_JPRB ,&
&  0.46486E+00_JPRB, 0.48319E+00_JPRB, 0.51671E+00_JPRB, 0.52108E+00_JPRB, 0.52327E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.54060E+00_JPRB, 0.55476E+00_JPRB, 0.54322E+00_JPRB, 0.54573E+00_JPRB, 0.52543E+00_JPRB, 0.51306E+00_JPRB, 0.45722E+00_JPRB ,&
&  0.48035E+00_JPRB, 0.46472E+00_JPRB, 0.45109E+00_JPRB, 0.44879E+00_JPRB, 0.46040E+00_JPRB, 0.39865E+00_JPRB, 0.34538E+00_JPRB ,&
&  0.32099E+00_JPRB, 0.31794E+00_JPRB, 0.33657E+00_JPRB, 0.34641E+00_JPRB, 0.34871E+00_JPRB, 0.35322E+00_JPRB, 0.34987E+00_JPRB ,&
&  0.36753E+00_JPRB, 0.37573E+00_JPRB, 0.38723E+00_JPRB, 0.38974E+00_JPRB, 0.38391E+00_JPRB, 0.38049E+00_JPRB, 0.37995E+00_JPRB ,&
&  0.36488E+00_JPRB, 0.35895E+00_JPRB, 0.35196E+00_JPRB, 0.35062E+00_JPRB, 0.39732E+00_JPRB, 0.39070E+00_JPRB, 0.38417E+00_JPRB ,&
&  0.39304E+00_JPRB, 0.41353E+00_JPRB, 0.44438E+00_JPRB, 0.48198E+00_JPRB, 0.52748E+00_JPRB, 0.57661E+00_JPRB, 0.63096E+00_JPRB ,&
&  0.68367E+00_JPRB, 0.75135E+00_JPRB, 0.80178E+00_JPRB, 0.89368E+00_JPRB, 0.96096E+00_JPRB, 0.91087E+00_JPRB, 0.10747E+01_JPRB ,&
&  0.11674E+01_JPRB, 0.12930E+01_JPRB, 0.14072E+01_JPRB, 0.12567E+01_JPRB, 0.13859E+01_JPRB, 0.12440E+01_JPRB, 0.10278E+01_JPRB ,&
&  0.97273E+00_JPRB, 0.88480E+00_JPRB, 0.85189E+00_JPRB, 0.82124E+00_JPRB, 0.84091E+00_JPRB, 0.80972E+00_JPRB, 0.79484E+00_JPRB ,&
&  0.83186E+00_JPRB, 0.95542E+00_JPRB, 0.90948E+00_JPRB, 0.92019E+00_JPRB, 0.89524E+00_JPRB, 0.86334E+00_JPRB, 0.79830E+00_JPRB ,&
&  0.74364E+00_JPRB, 0.68237E+00_JPRB, 0.62514E+00_JPRB, 0.57822E+00_JPRB, 0.52571E+00_JPRB, 0.48675E+00_JPRB, 0.45800E+00_JPRB ,&
&  0.42654E+00_JPRB, 0.38519E+00_JPRB, 0.36729E+00_JPRB, 0.35693E+00_JPRB, 0.35275E+00_JPRB, 0.34103E+00_JPRB, 0.34285E+00_JPRB ,&
&  0.35733E+00_JPRB, 0.38888E+00_JPRB, 0.43769E+00_JPRB, 0.45417E+00_JPRB, 0.48051E+00_JPRB, 0.51311E+00_JPRB, 0.48673E+00_JPRB ,&
&  0.46759E+00_JPRB, 0.53129E+00_JPRB, 0.53968E+00_JPRB, 0.54414E+00_JPRB, 0.53870E+00_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.57911E+00_JPRB, 0.59049E+00_JPRB, 0.57765E+00_JPRB, 0.56439E+00_JPRB, 0.53166E+00_JPRB, 0.50972E+00_JPRB, 0.47750E+00_JPRB ,&
&  0.48094E+00_JPRB, 0.47023E+00_JPRB, 0.45969E+00_JPRB, 0.45651E+00_JPRB, 0.46052E+00_JPRB, 0.44992E+00_JPRB, 0.39267E+00_JPRB ,&
&  0.34849E+00_JPRB, 0.33970E+00_JPRB, 0.32867E+00_JPRB, 0.34071E+00_JPRB, 0.35469E+00_JPRB, 0.35281E+00_JPRB, 0.36196E+00_JPRB ,&
&  0.38218E+00_JPRB, 0.38804E+00_JPRB, 0.38889E+00_JPRB, 0.38820E+00_JPRB, 0.38749E+00_JPRB, 0.39134E+00_JPRB, 0.39062E+00_JPRB ,&
&  0.37367E+00_JPRB, 0.36132E+00_JPRB, 0.35046E+00_JPRB, 0.35474E+00_JPRB, 0.39871E+00_JPRB, 0.41543E+00_JPRB, 0.41589E+00_JPRB ,&
&  0.42737E+00_JPRB, 0.44518E+00_JPRB, 0.46632E+00_JPRB, 0.50461E+00_JPRB, 0.55619E+00_JPRB, 0.61853E+00_JPRB, 0.68512E+00_JPRB ,&
&  0.75067E+00_JPRB, 0.82167E+00_JPRB, 0.87952E+00_JPRB, 0.95299E+00_JPRB, 0.93797E+00_JPRB, 0.10070E+01_JPRB, 0.11790E+01_JPRB ,&
&  0.12838E+01_JPRB, 0.14098E+01_JPRB, 0.14723E+01_JPRB, 0.15794E+01_JPRB, 0.16350E+01_JPRB, 0.14482E+01_JPRB, 0.12433E+01_JPRB ,&
&  0.11185E+01_JPRB, 0.10378E+01_JPRB, 0.12592E+01_JPRB, 0.11104E+01_JPRB, 0.10159E+01_JPRB, 0.10038E+01_JPRB, 0.97746E+00_JPRB ,&
&  0.10107E+01_JPRB, 0.10816E+01_JPRB, 0.11132E+01_JPRB, 0.11388E+01_JPRB, 0.11379E+01_JPRB, 0.11342E+01_JPRB, 0.10674E+01_JPRB ,&
&  0.10152E+01_JPRB, 0.91846E+00_JPRB, 0.73151E+00_JPRB, 0.59917E+00_JPRB, 0.50394E+00_JPRB, 0.49626E+00_JPRB, 0.46562E+00_JPRB ,&
&  0.42542E+00_JPRB, 0.38207E+00_JPRB, 0.37317E+00_JPRB, 0.36137E+00_JPRB, 0.36049E+00_JPRB, 0.35689E+00_JPRB, 0.34814E+00_JPRB ,&
&  0.35987E+00_JPRB, 0.40319E+00_JPRB, 0.42678E+00_JPRB, 0.46816E+00_JPRB, 0.51798E+00_JPRB, 0.58255E+00_JPRB, 0.50132E+00_JPRB ,&
&  0.52689E+00_JPRB, 0.59002E+00_JPRB, 0.60389E+00_JPRB, 0.60587E+00_JPRB, 0.58767E+00_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.66261E+00_JPRB, 0.65728E+00_JPRB, 0.63710E+00_JPRB, 0.60001E+00_JPRB, 0.55590E+00_JPRB, 0.51650E+00_JPRB, 0.48044E+00_JPRB ,&
&  0.47785E+00_JPRB, 0.46691E+00_JPRB, 0.45796E+00_JPRB, 0.45901E+00_JPRB, 0.46061E+00_JPRB, 0.45844E+00_JPRB, 0.45752E+00_JPRB ,&
&  0.38651E+00_JPRB, 0.37737E+00_JPRB, 0.34981E+00_JPRB, 0.36053E+00_JPRB, 0.41213E+00_JPRB, 0.37734E+00_JPRB, 0.38325E+00_JPRB ,&
&  0.38794E+00_JPRB, 0.39924E+00_JPRB, 0.39283E+00_JPRB, 0.39154E+00_JPRB, 0.40009E+00_JPRB, 0.40890E+00_JPRB, 0.40709E+00_JPRB ,&
&  0.39002E+00_JPRB, 0.37204E+00_JPRB, 0.36707E+00_JPRB, 0.37371E+00_JPRB, 0.41592E+00_JPRB, 0.44969E+00_JPRB, 0.46937E+00_JPRB ,&
&  0.48821E+00_JPRB, 0.50508E+00_JPRB, 0.51792E+00_JPRB, 0.54762E+00_JPRB, 0.59509E+00_JPRB, 0.65256E+00_JPRB, 0.73545E+00_JPRB ,&
&  0.81662E+00_JPRB, 0.89076E+00_JPRB, 0.97046E+00_JPRB, 0.10483E+01_JPRB, 0.10474E+01_JPRB, 0.11753E+01_JPRB, 0.13219E+01_JPRB ,&
&  0.14536E+01_JPRB, 0.14086E+01_JPRB, 0.14333E+01_JPRB, 0.16776E+01_JPRB, 0.18121E+01_JPRB, 0.17172E+01_JPRB, 0.15132E+01_JPRB ,&
&  0.13341E+01_JPRB, 0.12114E+01_JPRB, 0.12289E+01_JPRB, 0.11820E+01_JPRB, 0.10873E+01_JPRB, 0.11258E+01_JPRB, 0.11429E+01_JPRB ,&
&  0.11684E+01_JPRB, 0.12222E+01_JPRB, 0.12973E+01_JPRB, 0.13491E+01_JPRB, 0.13161E+01_JPRB, 0.13323E+01_JPRB, 0.12882E+01_JPRB ,&
&  0.12234E+01_JPRB, 0.10278E+01_JPRB, 0.74506E+00_JPRB, 0.60086E+00_JPRB, 0.46462E+00_JPRB, 0.44392E+00_JPRB, 0.49694E+00_JPRB ,&
&  0.48294E+00_JPRB, 0.45782E+00_JPRB, 0.42809E+00_JPRB, 0.40390E+00_JPRB, 0.39328E+00_JPRB, 0.40451E+00_JPRB, 0.39640E+00_JPRB ,&
&  0.38672E+00_JPRB, 0.42765E+00_JPRB, 0.49570E+00_JPRB, 0.57757E+00_JPRB, 0.63785E+00_JPRB, 0.68279E+00_JPRB, 0.63491E+00_JPRB ,&
&  0.69463E+00_JPRB, 0.69366E+00_JPRB, 0.69550E+00_JPRB, 0.70515E+00_JPRB, 0.69129E+00_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.74642E+00_JPRB, 0.72737E+00_JPRB, 0.69141E+00_JPRB, 0.63287E+00_JPRB, 0.56809E+00_JPRB, 0.51264E+00_JPRB, 0.48578E+00_JPRB ,&
&  0.48114E+00_JPRB, 0.47090E+00_JPRB, 0.45958E+00_JPRB, 0.45916E+00_JPRB, 0.46042E+00_JPRB, 0.45754E+00_JPRB, 0.46275E+00_JPRB ,&
&  0.49049E+00_JPRB, 0.46441E+00_JPRB, 0.40276E+00_JPRB, 0.36128E+00_JPRB, 0.38415E+00_JPRB, 0.38108E+00_JPRB, 0.38965E+00_JPRB ,&
&  0.39115E+00_JPRB, 0.41210E+00_JPRB, 0.39563E+00_JPRB, 0.39994E+00_JPRB, 0.42732E+00_JPRB, 0.44498E+00_JPRB, 0.45257E+00_JPRB ,&
&  0.44997E+00_JPRB, 0.44164E+00_JPRB, 0.44222E+00_JPRB, 0.46670E+00_JPRB, 0.51674E+00_JPRB, 0.50746E+00_JPRB, 0.54281E+00_JPRB ,&
&  0.55817E+00_JPRB, 0.58131E+00_JPRB, 0.58694E+00_JPRB, 0.61988E+00_JPRB, 0.67943E+00_JPRB, 0.77836E+00_JPRB, 0.85441E+00_JPRB ,&
&  0.91425E+00_JPRB, 0.96228E+00_JPRB, 0.98763E+00_JPRB, 0.10669E+01_JPRB, 0.11814E+01_JPRB, 0.13319E+01_JPRB, 0.13055E+01_JPRB ,&
&  0.13243E+01_JPRB, 0.12670E+01_JPRB, 0.12356E+01_JPRB, 0.14513E+01_JPRB, 0.16448E+01_JPRB, 0.16708E+01_JPRB, 0.16065E+01_JPRB ,&
&  0.15124E+01_JPRB, 0.14238E+01_JPRB, 0.14505E+01_JPRB, 0.13857E+01_JPRB, 0.13405E+01_JPRB, 0.13720E+01_JPRB, 0.14220E+01_JPRB ,&
&  0.14002E+01_JPRB, 0.14440E+01_JPRB, 0.15320E+01_JPRB, 0.15303E+01_JPRB, 0.15092E+01_JPRB, 0.14470E+01_JPRB, 0.14424E+01_JPRB ,&
&  0.13832E+01_JPRB, 0.85171E+00_JPRB, 0.67235E+00_JPRB, 0.68676E+00_JPRB, 0.52563E+00_JPRB, 0.45856E+00_JPRB, 0.47620E+00_JPRB ,&
&  0.45891E+00_JPRB, 0.45625E+00_JPRB, 0.47072E+00_JPRB, 0.45562E+00_JPRB, 0.44930E+00_JPRB, 0.47482E+00_JPRB, 0.48467E+00_JPRB ,&
&  0.50565E+00_JPRB, 0.53282E+00_JPRB, 0.66296E+00_JPRB, 0.74670E+00_JPRB, 0.78751E+00_JPRB, 0.80296E+00_JPRB, 0.80481E+00_JPRB ,&
&  0.79198E+00_JPRB, 0.78103E+00_JPRB, 0.77271E+00_JPRB, 0.77221E+00_JPRB, 0.77085E+00_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.84428E+00_JPRB, 0.81775E+00_JPRB, 0.76730E+00_JPRB, 0.69515E+00_JPRB, 0.61135E+00_JPRB, 0.54531E+00_JPRB, 0.52346E+00_JPRB ,&
&  0.50873E+00_JPRB, 0.50067E+00_JPRB, 0.48756E+00_JPRB, 0.48025E+00_JPRB, 0.47784E+00_JPRB, 0.47350E+00_JPRB, 0.47647E+00_JPRB ,&
&  0.48212E+00_JPRB, 0.49397E+00_JPRB, 0.41156E+00_JPRB, 0.35478E+00_JPRB, 0.32545E+00_JPRB, 0.34571E+00_JPRB, 0.36414E+00_JPRB ,&
&  0.38681E+00_JPRB, 0.40902E+00_JPRB, 0.41444E+00_JPRB, 0.46030E+00_JPRB, 0.50504E+00_JPRB, 0.52551E+00_JPRB, 0.56087E+00_JPRB ,&
&  0.56993E+00_JPRB, 0.56533E+00_JPRB, 0.55363E+00_JPRB, 0.60000E+00_JPRB, 0.61130E+00_JPRB, 0.57864E+00_JPRB, 0.63886E+00_JPRB ,&
&  0.66635E+00_JPRB, 0.69477E+00_JPRB, 0.71831E+00_JPRB, 0.77051E+00_JPRB, 0.83228E+00_JPRB, 0.86174E+00_JPRB, 0.85061E+00_JPRB ,&
&  0.87295E+00_JPRB, 0.92032E+00_JPRB, 0.98918E+00_JPRB, 0.10467E+01_JPRB, 0.11251E+01_JPRB, 0.12085E+01_JPRB, 0.11685E+01_JPRB ,&
&  0.11654E+01_JPRB, 0.10006E+01_JPRB, 0.86719E+00_JPRB, 0.10944E+01_JPRB, 0.15014E+01_JPRB, 0.16459E+01_JPRB, 0.15302E+01_JPRB ,&
&  0.16130E+01_JPRB, 0.16214E+01_JPRB, 0.16752E+01_JPRB, 0.16420E+01_JPRB, 0.15836E+01_JPRB, 0.16138E+01_JPRB, 0.16927E+01_JPRB ,&
&  0.16996E+01_JPRB, 0.17613E+01_JPRB, 0.18046E+01_JPRB, 0.17763E+01_JPRB, 0.17341E+01_JPRB, 0.17316E+01_JPRB, 0.17291E+01_JPRB ,&
&  0.15650E+01_JPRB, 0.13036E+01_JPRB, 0.91615E+00_JPRB, 0.86608E+00_JPRB, 0.61844E+00_JPRB, 0.51089E+00_JPRB, 0.50439E+00_JPRB ,&
&  0.46402E+00_JPRB, 0.47652E+00_JPRB, 0.50044E+00_JPRB, 0.49646E+00_JPRB, 0.54641E+00_JPRB, 0.58568E+00_JPRB, 0.64672E+00_JPRB ,&
&  0.70482E+00_JPRB, 0.72171E+00_JPRB, 0.86936E+00_JPRB, 0.90525E+00_JPRB, 0.93529E+00_JPRB, 0.95788E+00_JPRB, 0.96199E+00_JPRB ,&
&  0.93983E+00_JPRB, 0.91190E+00_JPRB, 0.88902E+00_JPRB, 0.86722E+00_JPRB, 0.85569E+00_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.86328E+00_JPRB, 0.82254E+00_JPRB, 0.75447E+00_JPRB, 0.68911E+00_JPRB, 0.61482E+00_JPRB, 0.54900E+00_JPRB, 0.53694E+00_JPRB ,&
&  0.55724E+00_JPRB, 0.55646E+00_JPRB, 0.54853E+00_JPRB, 0.52748E+00_JPRB, 0.51680E+00_JPRB, 0.51096E+00_JPRB, 0.50271E+00_JPRB ,&
&  0.49370E+00_JPRB, 0.45121E+00_JPRB, 0.36452E+00_JPRB, 0.31501E+00_JPRB, 0.30174E+00_JPRB, 0.30584E+00_JPRB, 0.33720E+00_JPRB ,&
&  0.37219E+00_JPRB, 0.42836E+00_JPRB, 0.48661E+00_JPRB, 0.55765E+00_JPRB, 0.64780E+00_JPRB, 0.74315E+00_JPRB, 0.78367E+00_JPRB ,&
&  0.81754E+00_JPRB, 0.70183E+00_JPRB, 0.73086E+00_JPRB, 0.71949E+00_JPRB, 0.74899E+00_JPRB, 0.74792E+00_JPRB, 0.75638E+00_JPRB ,&
&  0.74692E+00_JPRB, 0.77280E+00_JPRB, 0.80067E+00_JPRB, 0.82713E+00_JPRB, 0.84095E+00_JPRB, 0.84815E+00_JPRB, 0.85624E+00_JPRB ,&
&  0.91273E+00_JPRB, 0.97571E+00_JPRB, 0.10131E+01_JPRB, 0.10371E+01_JPRB, 0.10993E+01_JPRB, 0.11811E+01_JPRB, 0.11770E+01_JPRB ,&
&  0.10705E+01_JPRB, 0.10014E+01_JPRB, 0.11421E+01_JPRB, 0.12789E+01_JPRB, 0.14356E+01_JPRB, 0.16231E+01_JPRB, 0.16970E+01_JPRB ,&
&  0.17490E+01_JPRB, 0.16953E+01_JPRB, 0.17238E+01_JPRB, 0.16153E+01_JPRB, 0.17967E+01_JPRB, 0.18404E+01_JPRB, 0.19000E+01_JPRB ,&
&  0.19425E+01_JPRB, 0.20516E+01_JPRB, 0.19907E+01_JPRB, 0.19222E+01_JPRB, 0.19464E+01_JPRB, 0.20384E+01_JPRB, 0.18267E+01_JPRB ,&
&  0.15311E+01_JPRB, 0.16284E+01_JPRB, 0.14375E+01_JPRB, 0.11842E+01_JPRB, 0.90375E+00_JPRB, 0.77442E+00_JPRB, 0.66303E+00_JPRB ,&
&  0.58698E+00_JPRB, 0.55214E+00_JPRB, 0.55457E+00_JPRB, 0.62463E+00_JPRB, 0.73923E+00_JPRB, 0.83152E+00_JPRB, 0.88843E+00_JPRB ,&
&  0.92521E+00_JPRB, 0.11342E+01_JPRB, 0.11379E+01_JPRB, 0.11106E+01_JPRB, 0.11293E+01_JPRB, 0.11391E+01_JPRB, 0.11146E+01_JPRB ,&
&  0.10498E+01_JPRB, 0.10018E+01_JPRB, 0.96478E+00_JPRB, 0.93074E+00_JPRB, 0.89315E+00_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.87694E+00_JPRB, 0.83767E+00_JPRB, 0.76806E+00_JPRB, 0.69737E+00_JPRB, 0.62773E+00_JPRB, 0.56727E+00_JPRB, 0.55049E+00_JPRB ,&
&  0.55949E+00_JPRB, 0.57135E+00_JPRB, 0.58242E+00_JPRB, 0.58871E+00_JPRB, 0.58855E+00_JPRB, 0.57197E+00_JPRB, 0.55081E+00_JPRB ,&
&  0.53430E+00_JPRB, 0.43085E+00_JPRB, 0.34061E+00_JPRB, 0.31495E+00_JPRB, 0.30375E+00_JPRB, 0.30057E+00_JPRB, 0.32171E+00_JPRB ,&
&  0.37074E+00_JPRB, 0.47838E+00_JPRB, 0.60064E+00_JPRB, 0.76094E+00_JPRB, 0.90549E+00_JPRB, 0.10124E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.94548E+00_JPRB, 0.94046E+00_JPRB, 0.93371E+00_JPRB, 0.89271E+00_JPRB, 0.87715E+00_JPRB, 0.84193E+00_JPRB, 0.82088E+00_JPRB ,&
&  0.81045E+00_JPRB, 0.80924E+00_JPRB, 0.85297E+00_JPRB, 0.88628E+00_JPRB, 0.91264E+00_JPRB, 0.93659E+00_JPRB, 0.96126E+00_JPRB ,&
&  0.98146E+00_JPRB, 0.10040E+01_JPRB, 0.10123E+01_JPRB, 0.10805E+01_JPRB, 0.92534E+00_JPRB, 0.94787E+00_JPRB, 0.10328E+01_JPRB ,&
&  0.12246E+01_JPRB, 0.12192E+01_JPRB, 0.11801E+01_JPRB, 0.13107E+01_JPRB, 0.12472E+01_JPRB, 0.13276E+01_JPRB, 0.15791E+01_JPRB ,&
&  0.16865E+01_JPRB, 0.17131E+01_JPRB, 0.18151E+01_JPRB, 0.15922E+01_JPRB, 0.15451E+01_JPRB, 0.21536E+01_JPRB, 0.22707E+01_JPRB ,&
&  0.22820E+01_JPRB, 0.22798E+01_JPRB, 0.21743E+01_JPRB, 0.21579E+01_JPRB, 0.18663E+01_JPRB, 0.16256E+01_JPRB, 0.12609E+01_JPRB ,&
&  0.13327E+01_JPRB, 0.12856E+01_JPRB, 0.14182E+01_JPRB, 0.12981E+01_JPRB, 0.11991E+01_JPRB, 0.10606E+01_JPRB, 0.87299E+00_JPRB ,&
&  0.75408E+00_JPRB, 0.68814E+00_JPRB, 0.68147E+00_JPRB, 0.84127E+00_JPRB, 0.10474E+01_JPRB, 0.10842E+01_JPRB, 0.12904E+01_JPRB ,&
&  0.13978E+01_JPRB, 0.14512E+01_JPRB, 0.13047E+01_JPRB, 0.12916E+01_JPRB, 0.12435E+01_JPRB, 0.11946E+01_JPRB, 0.11298E+01_JPRB ,&
&  0.10740E+01_JPRB, 0.10250E+01_JPRB, 0.98042E+00_JPRB, 0.93817E+00_JPRB, 0.90035E+00_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.88321E+00_JPRB, 0.85166E+00_JPRB, 0.78452E+00_JPRB, 0.71454E+00_JPRB, 0.64796E+00_JPRB, 0.60021E+00_JPRB, 0.58680E+00_JPRB ,&
&  0.59691E+00_JPRB, 0.60890E+00_JPRB, 0.61501E+00_JPRB, 0.61278E+00_JPRB, 0.60610E+00_JPRB, 0.60709E+00_JPRB, 0.60159E+00_JPRB ,&
&  0.57800E+00_JPRB, 0.47876E+00_JPRB, 0.36196E+00_JPRB, 0.31905E+00_JPRB, 0.32821E+00_JPRB, 0.31860E+00_JPRB, 0.32745E+00_JPRB ,&
&  0.41438E+00_JPRB, 0.57052E+00_JPRB, 0.78015E+00_JPRB, 0.98532E+00_JPRB, 0.11648E+01_JPRB, 0.12149E+01_JPRB, 0.11651E+01_JPRB ,&
&  0.11645E+01_JPRB, 0.10953E+01_JPRB, 0.96691E+00_JPRB, 0.87203E+00_JPRB, 0.82996E+00_JPRB, 0.81277E+00_JPRB, 0.78647E+00_JPRB ,&
&  0.76640E+00_JPRB, 0.77369E+00_JPRB, 0.80206E+00_JPRB, 0.82413E+00_JPRB, 0.85851E+00_JPRB, 0.87071E+00_JPRB, 0.87278E+00_JPRB ,&
&  0.89715E+00_JPRB, 0.93719E+00_JPRB, 0.98892E+00_JPRB, 0.10786E+01_JPRB, 0.98338E+00_JPRB, 0.97915E+00_JPRB, 0.11084E+01_JPRB ,&
&  0.12144E+01_JPRB, 0.12169E+01_JPRB, 0.12190E+01_JPRB, 0.12651E+01_JPRB, 0.13685E+01_JPRB, 0.13582E+01_JPRB, 0.15603E+01_JPRB ,&
&  0.15111E+01_JPRB, 0.14475E+01_JPRB, 0.14517E+01_JPRB, 0.14111E+01_JPRB, 0.17332E+01_JPRB, 0.23533E+01_JPRB, 0.26277E+01_JPRB ,&
&  0.25753E+01_JPRB, 0.25009E+01_JPRB, 0.24062E+01_JPRB, 0.21000E+01_JPRB, 0.16493E+01_JPRB, 0.12464E+01_JPRB, 0.14573E+01_JPRB ,&
&  0.15750E+01_JPRB, 0.16063E+01_JPRB, 0.15232E+01_JPRB, 0.13009E+01_JPRB, 0.11372E+01_JPRB, 0.12546E+01_JPRB, 0.11359E+01_JPRB ,&
&  0.95012E+00_JPRB, 0.86512E+00_JPRB, 0.10011E+01_JPRB, 0.11974E+01_JPRB, 0.13160E+01_JPRB, 0.14254E+01_JPRB, 0.15934E+01_JPRB ,&
&  0.15404E+01_JPRB, 0.15234E+01_JPRB, 0.14432E+01_JPRB, 0.13334E+01_JPRB, 0.12395E+01_JPRB, 0.11671E+01_JPRB, 0.11034E+01_JPRB ,&
&  0.10654E+01_JPRB, 0.10337E+01_JPRB, 0.99067E+00_JPRB, 0.95116E+00_JPRB, 0.90983E+00_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.89491E+00_JPRB, 0.86572E+00_JPRB, 0.79705E+00_JPRB, 0.72079E+00_JPRB, 0.66953E+00_JPRB, 0.63753E+00_JPRB, 0.62239E+00_JPRB ,&
&  0.62419E+00_JPRB, 0.62621E+00_JPRB, 0.62920E+00_JPRB, 0.62798E+00_JPRB, 0.62275E+00_JPRB, 0.61547E+00_JPRB, 0.61976E+00_JPRB ,&
&  0.60398E+00_JPRB, 0.58728E+00_JPRB, 0.42947E+00_JPRB, 0.36516E+00_JPRB, 0.35904E+00_JPRB, 0.36209E+00_JPRB, 0.36374E+00_JPRB ,&
&  0.49637E+00_JPRB, 0.74855E+00_JPRB, 0.97410E+00_JPRB, 0.11616E+01_JPRB, 0.13282E+01_JPRB, 0.11895E+01_JPRB, 0.11421E+01_JPRB ,&
&  0.11683E+01_JPRB, 0.11183E+01_JPRB, 0.10033E+01_JPRB, 0.90340E+00_JPRB, 0.82148E+00_JPRB, 0.78636E+00_JPRB, 0.77251E+00_JPRB ,&
&  0.76150E+00_JPRB, 0.76641E+00_JPRB, 0.78529E+00_JPRB, 0.82273E+00_JPRB, 0.84327E+00_JPRB, 0.85301E+00_JPRB, 0.86533E+00_JPRB ,&
&  0.87675E+00_JPRB, 0.90026E+00_JPRB, 0.93767E+00_JPRB, 0.10689E+01_JPRB, 0.10497E+01_JPRB, 0.10906E+01_JPRB, 0.13135E+01_JPRB ,&
&  0.13118E+01_JPRB, 0.13387E+01_JPRB, 0.12971E+01_JPRB, 0.13457E+01_JPRB, 0.13856E+01_JPRB, 0.14114E+01_JPRB, 0.15773E+01_JPRB ,&
&  0.15437E+01_JPRB, 0.15630E+01_JPRB, 0.16996E+01_JPRB, 0.17992E+01_JPRB, 0.18796E+01_JPRB, 0.21743E+01_JPRB, 0.29315E+01_JPRB ,&
&  0.25397E+01_JPRB, 0.25010E+01_JPRB, 0.24344E+01_JPRB, 0.21693E+01_JPRB, 0.11740E+01_JPRB, 0.87189E+00_JPRB, 0.12453E+01_JPRB ,&
&  0.13147E+01_JPRB, 0.85947E+00_JPRB, 0.67190E+00_JPRB, 0.90150E+00_JPRB, 0.73373E+00_JPRB, 0.85882E+00_JPRB, 0.11356E+01_JPRB ,&
&  0.11522E+01_JPRB, 0.10585E+01_JPRB, 0.12312E+01_JPRB, 0.13138E+01_JPRB, 0.13637E+01_JPRB, 0.16147E+01_JPRB, 0.17071E+01_JPRB ,&
&  0.16672E+01_JPRB, 0.14880E+01_JPRB, 0.14908E+01_JPRB, 0.13517E+01_JPRB, 0.12515E+01_JPRB, 0.11960E+01_JPRB, 0.11522E+01_JPRB ,&
&  0.10945E+01_JPRB, 0.10303E+01_JPRB, 0.98117E+00_JPRB, 0.95051E+00_JPRB, 0.91885E+00_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.89192E+00_JPRB, 0.85998E+00_JPRB, 0.78919E+00_JPRB, 0.71406E+00_JPRB, 0.67587E+00_JPRB, 0.64877E+00_JPRB, 0.63273E+00_JPRB ,&
&  0.63377E+00_JPRB, 0.64064E+00_JPRB, 0.64644E+00_JPRB, 0.65063E+00_JPRB, 0.64827E+00_JPRB, 0.64310E+00_JPRB, 0.63910E+00_JPRB ,&
&  0.62142E+00_JPRB, 0.58603E+00_JPRB, 0.51044E+00_JPRB, 0.44710E+00_JPRB, 0.42941E+00_JPRB, 0.41755E+00_JPRB, 0.44828E+00_JPRB ,&
&  0.58844E+00_JPRB, 0.90661E+00_JPRB, 0.11725E+01_JPRB, 0.12827E+01_JPRB, 0.12226E+01_JPRB, 0.11197E+01_JPRB, 0.11945E+01_JPRB ,&
&  0.12045E+01_JPRB, 0.11364E+01_JPRB, 0.10039E+01_JPRB, 0.91817E+00_JPRB, 0.85055E+00_JPRB, 0.81039E+00_JPRB, 0.79695E+00_JPRB ,&
&  0.77833E+00_JPRB, 0.77885E+00_JPRB, 0.80928E+00_JPRB, 0.82789E+00_JPRB, 0.83509E+00_JPRB, 0.83942E+00_JPRB, 0.84253E+00_JPRB ,&
&  0.86051E+00_JPRB, 0.89722E+00_JPRB, 0.95714E+00_JPRB, 0.10622E+01_JPRB, 0.12484E+01_JPRB, 0.13357E+01_JPRB, 0.12780E+01_JPRB ,&
&  0.12060E+01_JPRB, 0.12363E+01_JPRB, 0.13924E+01_JPRB, 0.13464E+01_JPRB, 0.13329E+01_JPRB, 0.13782E+01_JPRB, 0.15119E+01_JPRB ,&
&  0.17430E+01_JPRB, 0.18956E+01_JPRB, 0.17897E+01_JPRB, 0.18904E+01_JPRB, 0.20084E+01_JPRB, 0.18648E+01_JPRB, 0.24094E+01_JPRB ,&
&  0.25936E+01_JPRB, 0.22443E+01_JPRB, 0.17275E+01_JPRB, 0.12630E+01_JPRB, 0.11945E+01_JPRB, 0.91393E+00_JPRB, 0.59779E+00_JPRB ,&
&  0.55189E+00_JPRB, 0.52816E+00_JPRB, 0.49531E+00_JPRB, 0.54798E+00_JPRB, 0.55448E+00_JPRB, 0.71417E+00_JPRB, 0.10422E+01_JPRB ,&
&  0.12879E+01_JPRB, 0.13104E+01_JPRB, 0.13712E+01_JPRB, 0.13645E+01_JPRB, 0.13424E+01_JPRB, 0.15684E+01_JPRB, 0.17732E+01_JPRB ,&
&  0.15523E+01_JPRB, 0.14532E+01_JPRB, 0.14144E+01_JPRB, 0.12820E+01_JPRB, 0.12327E+01_JPRB, 0.11991E+01_JPRB, 0.11376E+01_JPRB ,&
&  0.10549E+01_JPRB, 0.97494E+00_JPRB, 0.93607E+00_JPRB, 0.91989E+00_JPRB, 0.91183E+00_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.88768E+00_JPRB, 0.85076E+00_JPRB, 0.79212E+00_JPRB, 0.72999E+00_JPRB, 0.70426E+00_JPRB, 0.68507E+00_JPRB, 0.66954E+00_JPRB ,&
&  0.67155E+00_JPRB, 0.67885E+00_JPRB, 0.68892E+00_JPRB, 0.69497E+00_JPRB, 0.69212E+00_JPRB, 0.68189E+00_JPRB, 0.66924E+00_JPRB ,&
&  0.64921E+00_JPRB, 0.60360E+00_JPRB, 0.57204E+00_JPRB, 0.52218E+00_JPRB, 0.50705E+00_JPRB, 0.48527E+00_JPRB, 0.53742E+00_JPRB ,&
&  0.71458E+00_JPRB, 0.10839E+01_JPRB, 0.13658E+01_JPRB, 0.14044E+01_JPRB, 0.13380E+01_JPRB, 0.12873E+01_JPRB, 0.12798E+01_JPRB ,&
&  0.12141E+01_JPRB, 0.11408E+01_JPRB, 0.10185E+01_JPRB, 0.93392E+00_JPRB, 0.87119E+00_JPRB, 0.83246E+00_JPRB, 0.81024E+00_JPRB ,&
&  0.79468E+00_JPRB, 0.81971E+00_JPRB, 0.82382E+00_JPRB, 0.82988E+00_JPRB, 0.82872E+00_JPRB, 0.82890E+00_JPRB, 0.83854E+00_JPRB ,&
&  0.87448E+00_JPRB, 0.94049E+00_JPRB, 0.10484E+01_JPRB, 0.12175E+01_JPRB, 0.12244E+01_JPRB, 0.11160E+01_JPRB, 0.11968E+01_JPRB ,&
&  0.12455E+01_JPRB, 0.13421E+01_JPRB, 0.13177E+01_JPRB, 0.14037E+01_JPRB, 0.13571E+01_JPRB, 0.13300E+01_JPRB, 0.14906E+01_JPRB ,&
&  0.17571E+01_JPRB, 0.20800E+01_JPRB, 0.17870E+01_JPRB, 0.17461E+01_JPRB, 0.21713E+01_JPRB, 0.20113E+01_JPRB, 0.21574E+01_JPRB ,&
&  0.24063E+01_JPRB, 0.20588E+01_JPRB, 0.16776E+01_JPRB, 0.13159E+01_JPRB, 0.15235E+01_JPRB, 0.14467E+01_JPRB, 0.80215E+00_JPRB ,&
&  0.61434E+00_JPRB, 0.54272E+00_JPRB, 0.50402E+00_JPRB, 0.49670E+00_JPRB, 0.50557E+00_JPRB, 0.67924E+00_JPRB, 0.11406E+01_JPRB ,&
&  0.12648E+01_JPRB, 0.14502E+01_JPRB, 0.14906E+01_JPRB, 0.14582E+01_JPRB, 0.14232E+01_JPRB, 0.14380E+01_JPRB, 0.15853E+01_JPRB ,&
&  0.16793E+01_JPRB, 0.15576E+01_JPRB, 0.13729E+01_JPRB, 0.12506E+01_JPRB, 0.11784E+01_JPRB, 0.10886E+01_JPRB, 0.10006E+01_JPRB ,&
&  0.93102E+00_JPRB, 0.90072E+00_JPRB, 0.89023E+00_JPRB, 0.89552E+00_JPRB, 0.90847E+00_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.88652E+00_JPRB, 0.87158E+00_JPRB, 0.82386E+00_JPRB, 0.76818E+00_JPRB, 0.74115E+00_JPRB, 0.72296E+00_JPRB, 0.71171E+00_JPRB ,&
&  0.71976E+00_JPRB, 0.72985E+00_JPRB, 0.73000E+00_JPRB, 0.72107E+00_JPRB, 0.71569E+00_JPRB, 0.69927E+00_JPRB, 0.68287E+00_JPRB ,&
&  0.66111E+00_JPRB, 0.61898E+00_JPRB, 0.58553E+00_JPRB, 0.57553E+00_JPRB, 0.55682E+00_JPRB, 0.54194E+00_JPRB, 0.62704E+00_JPRB ,&
&  0.89592E+00_JPRB, 0.13282E+01_JPRB, 0.15322E+01_JPRB, 0.15528E+01_JPRB, 0.14683E+01_JPRB, 0.13907E+01_JPRB, 0.12429E+01_JPRB ,&
&  0.10857E+01_JPRB, 0.10577E+01_JPRB, 0.96021E+00_JPRB, 0.92158E+00_JPRB, 0.89407E+00_JPRB, 0.86796E+00_JPRB, 0.85108E+00_JPRB ,&
&  0.84815E+00_JPRB, 0.83193E+00_JPRB, 0.82469E+00_JPRB, 0.82149E+00_JPRB, 0.82211E+00_JPRB, 0.82951E+00_JPRB, 0.87076E+00_JPRB ,&
&  0.94776E+00_JPRB, 0.10316E+01_JPRB, 0.11718E+01_JPRB, 0.12988E+01_JPRB, 0.10709E+01_JPRB, 0.11646E+01_JPRB, 0.11893E+01_JPRB ,&
&  0.12019E+01_JPRB, 0.12344E+01_JPRB, 0.11867E+01_JPRB, 0.12426E+01_JPRB, 0.13215E+01_JPRB, 0.13299E+01_JPRB, 0.14357E+01_JPRB ,&
&  0.18272E+01_JPRB, 0.17230E+01_JPRB, 0.15865E+01_JPRB, 0.16494E+01_JPRB, 0.19837E+01_JPRB, 0.28016E+01_JPRB, 0.19818E+01_JPRB ,&
&  0.18022E+01_JPRB, 0.20154E+01_JPRB, 0.16762E+01_JPRB, 0.13891E+01_JPRB, 0.15481E+01_JPRB, 0.17675E+01_JPRB, 0.15823E+01_JPRB ,&
&  0.77978E+00_JPRB, 0.58178E+00_JPRB, 0.57875E+00_JPRB, 0.58429E+00_JPRB, 0.56247E+00_JPRB, 0.68769E+00_JPRB, 0.15386E+01_JPRB ,&
&  0.13704E+01_JPRB, 0.15449E+01_JPRB, 0.15362E+01_JPRB, 0.14399E+01_JPRB, 0.14480E+01_JPRB, 0.14917E+01_JPRB, 0.15985E+01_JPRB ,&
&  0.14137E+01_JPRB, 0.12777E+01_JPRB, 0.11737E+01_JPRB, 0.10620E+01_JPRB, 0.97669E+00_JPRB, 0.91791E+00_JPRB, 0.88457E+00_JPRB ,&
&  0.86419E+00_JPRB, 0.85694E+00_JPRB, 0.86330E+00_JPRB, 0.87619E+00_JPRB, 0.88731E+00_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.88865E+00_JPRB, 0.89206E+00_JPRB, 0.84756E+00_JPRB, 0.79470E+00_JPRB, 0.76369E+00_JPRB, 0.74887E+00_JPRB, 0.73845E+00_JPRB ,&
&  0.79965E+00_JPRB, 0.79390E+00_JPRB, 0.75882E+00_JPRB, 0.72702E+00_JPRB, 0.71749E+00_JPRB, 0.69768E+00_JPRB, 0.68268E+00_JPRB ,&
&  0.65926E+00_JPRB, 0.63124E+00_JPRB, 0.60303E+00_JPRB, 0.58344E+00_JPRB, 0.59293E+00_JPRB, 0.56450E+00_JPRB, 0.65583E+00_JPRB ,&
&  0.10660E+01_JPRB, 0.16247E+01_JPRB, 0.15880E+01_JPRB, 0.15747E+01_JPRB, 0.14933E+01_JPRB, 0.14152E+01_JPRB, 0.11604E+01_JPRB ,&
&  0.10335E+01_JPRB, 0.10227E+01_JPRB, 0.94551E+00_JPRB, 0.93143E+00_JPRB, 0.91951E+00_JPRB, 0.90631E+00_JPRB, 0.89380E+00_JPRB ,&
&  0.85461E+00_JPRB, 0.83325E+00_JPRB, 0.83859E+00_JPRB, 0.84270E+00_JPRB, 0.84295E+00_JPRB, 0.87669E+00_JPRB, 0.94566E+00_JPRB ,&
&  0.10211E+01_JPRB, 0.11162E+01_JPRB, 0.12604E+01_JPRB, 0.12414E+01_JPRB, 0.11721E+01_JPRB, 0.11639E+01_JPRB, 0.11921E+01_JPRB ,&
&  0.11785E+01_JPRB, 0.11973E+01_JPRB, 0.12034E+01_JPRB, 0.11934E+01_JPRB, 0.12599E+01_JPRB, 0.13505E+01_JPRB, 0.14336E+01_JPRB ,&
&  0.16201E+01_JPRB, 0.15290E+01_JPRB, 0.14214E+01_JPRB, 0.15580E+01_JPRB, 0.18940E+01_JPRB, 0.25262E+01_JPRB, 0.25471E+01_JPRB ,&
&  0.22310E+01_JPRB, 0.16666E+01_JPRB, 0.14341E+01_JPRB, 0.15246E+01_JPRB, 0.16376E+01_JPRB, 0.17184E+01_JPRB, 0.21319E+01_JPRB ,&
&  0.20654E+01_JPRB, 0.12484E+01_JPRB, 0.10403E+01_JPRB, 0.12606E+01_JPRB, 0.93953E+00_JPRB, 0.75213E+00_JPRB, 0.10826E+01_JPRB ,&
&  0.12925E+01_JPRB, 0.14612E+01_JPRB, 0.14946E+01_JPRB, 0.14317E+01_JPRB, 0.14673E+01_JPRB, 0.14355E+01_JPRB, 0.13209E+01_JPRB ,&
&  0.11775E+01_JPRB, 0.10690E+01_JPRB, 0.98676E+00_JPRB, 0.93716E+00_JPRB, 0.91623E+00_JPRB, 0.90030E+00_JPRB, 0.87363E+00_JPRB ,&
&  0.85704E+00_JPRB, 0.84807E+00_JPRB, 0.85498E+00_JPRB, 0.87618E+00_JPRB, 0.88870E+00_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.89365E+00_JPRB, 0.88702E+00_JPRB, 0.84605E+00_JPRB, 0.80461E+00_JPRB, 0.77711E+00_JPRB, 0.76952E+00_JPRB, 0.77732E+00_JPRB ,&
&  0.96819E+00_JPRB, 0.85301E+00_JPRB, 0.76884E+00_JPRB, 0.72945E+00_JPRB, 0.70766E+00_JPRB, 0.69199E+00_JPRB, 0.67450E+00_JPRB ,&
&  0.65978E+00_JPRB, 0.64146E+00_JPRB, 0.62982E+00_JPRB, 0.62810E+00_JPRB, 0.65982E+00_JPRB, 0.80815E+00_JPRB, 0.78998E+00_JPRB ,&
&  0.11334E+01_JPRB, 0.18327E+01_JPRB, 0.16584E+01_JPRB, 0.16027E+01_JPRB, 0.14868E+01_JPRB, 0.14336E+01_JPRB, 0.11606E+01_JPRB ,&
&  0.10501E+01_JPRB, 0.10174E+01_JPRB, 0.97694E+00_JPRB, 0.95723E+00_JPRB, 0.95798E+00_JPRB, 0.95052E+00_JPRB, 0.88555E+00_JPRB ,&
&  0.84937E+00_JPRB, 0.84444E+00_JPRB, 0.86213E+00_JPRB, 0.87377E+00_JPRB, 0.89787E+00_JPRB, 0.95686E+00_JPRB, 0.10069E+01_JPRB ,&
&  0.10823E+01_JPRB, 0.12357E+01_JPRB, 0.13307E+01_JPRB, 0.12035E+01_JPRB, 0.11807E+01_JPRB, 0.11972E+01_JPRB, 0.12146E+01_JPRB ,&
&  0.11472E+01_JPRB, 0.10765E+01_JPRB, 0.11513E+01_JPRB, 0.12210E+01_JPRB, 0.12699E+01_JPRB, 0.12970E+01_JPRB, 0.13251E+01_JPRB ,&
&  0.14913E+01_JPRB, 0.15645E+01_JPRB, 0.17075E+01_JPRB, 0.16937E+01_JPRB, 0.18899E+01_JPRB, 0.23455E+01_JPRB, 0.25310E+01_JPRB ,&
&  0.25879E+01_JPRB, 0.20754E+01_JPRB, 0.18686E+01_JPRB, 0.18187E+01_JPRB, 0.17451E+01_JPRB, 0.16566E+01_JPRB, 0.17792E+01_JPRB ,&
&  0.19269E+01_JPRB, 0.21685E+01_JPRB, 0.19532E+01_JPRB, 0.15731E+01_JPRB, 0.12408E+01_JPRB, 0.94231E+00_JPRB, 0.10593E+01_JPRB ,&
&  0.13018E+01_JPRB, 0.13649E+01_JPRB, 0.13654E+01_JPRB, 0.17019E+01_JPRB, 0.14229E+01_JPRB, 0.12389E+01_JPRB, 0.10653E+01_JPRB ,&
&  0.96681E+00_JPRB, 0.95192E+00_JPRB, 0.95248E+00_JPRB, 0.93444E+00_JPRB, 0.91200E+00_JPRB, 0.89214E+00_JPRB, 0.87192E+00_JPRB ,&
&  0.85847E+00_JPRB, 0.85793E+00_JPRB, 0.87198E+00_JPRB, 0.88868E+00_JPRB, 0.90019E+00_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.87003E+00_JPRB, 0.84745E+00_JPRB, 0.82072E+00_JPRB, 0.79641E+00_JPRB, 0.77937E+00_JPRB, 0.78862E+00_JPRB, 0.83920E+00_JPRB ,&
&  0.11155E+01_JPRB, 0.85259E+00_JPRB, 0.75634E+00_JPRB, 0.72471E+00_JPRB, 0.70841E+00_JPRB, 0.68556E+00_JPRB, 0.67703E+00_JPRB ,&
&  0.67632E+00_JPRB, 0.67522E+00_JPRB, 0.68889E+00_JPRB, 0.72719E+00_JPRB, 0.83842E+00_JPRB, 0.11153E+01_JPRB, 0.12778E+01_JPRB ,&
&  0.12884E+01_JPRB, 0.20985E+01_JPRB, 0.17369E+01_JPRB, 0.16327E+01_JPRB, 0.14769E+01_JPRB, 0.14004E+01_JPRB, 0.12379E+01_JPRB ,&
&  0.11039E+01_JPRB, 0.10421E+01_JPRB, 0.10159E+01_JPRB, 0.98143E+00_JPRB, 0.95651E+00_JPRB, 0.90677E+00_JPRB, 0.88216E+00_JPRB ,&
&  0.87748E+00_JPRB, 0.89114E+00_JPRB, 0.90726E+00_JPRB, 0.92788E+00_JPRB, 0.95975E+00_JPRB, 0.10289E+01_JPRB, 0.11073E+01_JPRB ,&
&  0.12350E+01_JPRB, 0.13740E+01_JPRB, 0.12940E+01_JPRB, 0.12082E+01_JPRB, 0.11778E+01_JPRB, 0.12068E+01_JPRB, 0.12261E+01_JPRB ,&
&  0.11765E+01_JPRB, 0.11159E+01_JPRB, 0.11694E+01_JPRB, 0.11879E+01_JPRB, 0.11864E+01_JPRB, 0.12636E+01_JPRB, 0.12761E+01_JPRB ,&
&  0.14929E+01_JPRB, 0.16024E+01_JPRB, 0.21107E+01_JPRB, 0.20165E+01_JPRB, 0.18447E+01_JPRB, 0.22156E+01_JPRB, 0.22050E+01_JPRB ,&
&  0.18833E+01_JPRB, 0.17812E+01_JPRB, 0.17678E+01_JPRB, 0.18772E+01_JPRB, 0.18496E+01_JPRB, 0.17733E+01_JPRB, 0.16492E+01_JPRB ,&
&  0.14917E+01_JPRB, 0.15489E+01_JPRB, 0.13823E+01_JPRB, 0.11771E+01_JPRB, 0.11484E+01_JPRB, 0.10165E+01_JPRB, 0.11544E+01_JPRB ,&
&  0.13889E+01_JPRB, 0.14570E+01_JPRB, 0.15531E+01_JPRB, 0.15571E+01_JPRB, 0.12082E+01_JPRB, 0.10182E+01_JPRB, 0.93835E+00_JPRB ,&
&  0.93988E+00_JPRB, 0.96003E+00_JPRB, 0.95032E+00_JPRB, 0.92329E+00_JPRB, 0.91057E+00_JPRB, 0.90097E+00_JPRB, 0.89016E+00_JPRB ,&
&  0.88718E+00_JPRB, 0.88898E+00_JPRB, 0.89180E+00_JPRB, 0.89036E+00_JPRB, 0.89024E+00_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.87184E+00_JPRB, 0.86592E+00_JPRB, 0.85580E+00_JPRB, 0.85858E+00_JPRB, 0.89131E+00_JPRB, 0.92648E+00_JPRB, 0.10516E+01_JPRB ,&
&  0.12666E+01_JPRB, 0.84284E+00_JPRB, 0.74159E+00_JPRB, 0.71930E+00_JPRB, 0.70937E+00_JPRB, 0.70649E+00_JPRB, 0.72557E+00_JPRB ,&
&  0.74323E+00_JPRB, 0.76237E+00_JPRB, 0.80684E+00_JPRB, 0.93036E+00_JPRB, 0.11856E+01_JPRB, 0.15947E+01_JPRB, 0.17541E+01_JPRB ,&
&  0.16647E+01_JPRB, 0.17204E+01_JPRB, 0.18751E+01_JPRB, 0.16057E+01_JPRB, 0.14853E+01_JPRB, 0.14028E+01_JPRB, 0.13728E+01_JPRB ,&
&  0.11979E+01_JPRB, 0.11056E+01_JPRB, 0.10533E+01_JPRB, 0.99405E+00_JPRB, 0.93664E+00_JPRB, 0.91582E+00_JPRB, 0.92820E+00_JPRB ,&
&  0.94596E+00_JPRB, 0.96416E+00_JPRB, 0.98742E+00_JPRB, 0.10236E+01_JPRB, 0.10899E+01_JPRB, 0.11960E+01_JPRB, 0.13083E+01_JPRB ,&
&  0.14144E+01_JPRB, 0.14399E+01_JPRB, 0.13269E+01_JPRB, 0.12321E+01_JPRB, 0.11978E+01_JPRB, 0.12254E+01_JPRB, 0.12345E+01_JPRB ,&
&  0.12154E+01_JPRB, 0.12142E+01_JPRB, 0.12264E+01_JPRB, 0.12189E+01_JPRB, 0.11667E+01_JPRB, 0.12846E+01_JPRB, 0.13454E+01_JPRB ,&
&  0.15598E+01_JPRB, 0.17941E+01_JPRB, 0.23524E+01_JPRB, 0.19350E+01_JPRB, 0.20001E+01_JPRB, 0.20925E+01_JPRB, 0.20345E+01_JPRB ,&
&  0.16820E+01_JPRB, 0.16764E+01_JPRB, 0.17862E+01_JPRB, 0.18413E+01_JPRB, 0.19283E+01_JPRB, 0.17732E+01_JPRB, 0.17089E+01_JPRB ,&
&  0.13361E+01_JPRB, 0.13260E+01_JPRB, 0.10937E+01_JPRB, 0.11150E+01_JPRB, 0.10927E+01_JPRB, 0.10521E+01_JPRB, 0.11389E+01_JPRB ,&
&  0.13567E+01_JPRB, 0.14212E+01_JPRB, 0.13297E+01_JPRB, 0.12085E+01_JPRB, 0.98127E+00_JPRB, 0.92443E+00_JPRB, 0.91821E+00_JPRB ,&
&  0.92721E+00_JPRB, 0.96840E+00_JPRB, 0.95208E+00_JPRB, 0.94263E+00_JPRB, 0.92821E+00_JPRB, 0.91615E+00_JPRB, 0.91286E+00_JPRB ,&
&  0.91234E+00_JPRB, 0.90887E+00_JPRB, 0.90424E+00_JPRB, 0.89437E+00_JPRB, 0.88570E+00_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.86642E+00_JPRB, 0.85996E+00_JPRB, 0.85463E+00_JPRB, 0.84136E+00_JPRB, 0.82867E+00_JPRB, 0.85077E+00_JPRB, 0.85733E+00_JPRB ,&
&  0.81673E+00_JPRB, 0.76198E+00_JPRB, 0.75987E+00_JPRB, 0.76033E+00_JPRB, 0.76501E+00_JPRB, 0.77825E+00_JPRB, 0.80828E+00_JPRB ,&
&  0.83694E+00_JPRB, 0.89169E+00_JPRB, 0.10278E+01_JPRB, 0.12715E+01_JPRB, 0.15522E+01_JPRB, 0.18516E+01_JPRB, 0.21726E+01_JPRB ,&
&  0.20595E+01_JPRB, 0.14399E+01_JPRB, 0.17552E+01_JPRB, 0.17085E+01_JPRB, 0.15088E+01_JPRB, 0.13630E+01_JPRB, 0.12914E+01_JPRB ,&
&  0.12338E+01_JPRB, 0.11837E+01_JPRB, 0.11365E+01_JPRB, 0.10834E+01_JPRB, 0.10376E+01_JPRB, 0.10562E+01_JPRB, 0.10836E+01_JPRB ,&
&  0.11052E+01_JPRB, 0.11355E+01_JPRB, 0.11776E+01_JPRB, 0.12449E+01_JPRB, 0.13507E+01_JPRB, 0.14571E+01_JPRB, 0.15308E+01_JPRB ,&
&  0.15517E+01_JPRB, 0.15571E+01_JPRB, 0.14852E+01_JPRB, 0.14005E+01_JPRB, 0.13330E+01_JPRB, 0.13251E+01_JPRB, 0.12967E+01_JPRB ,&
&  0.12770E+01_JPRB, 0.12522E+01_JPRB, 0.12856E+01_JPRB, 0.13574E+01_JPRB, 0.14018E+01_JPRB, 0.13462E+01_JPRB, 0.15131E+01_JPRB ,&
&  0.16987E+01_JPRB, 0.19643E+01_JPRB, 0.20914E+01_JPRB, 0.25517E+01_JPRB, 0.17726E+01_JPRB, 0.18529E+01_JPRB, 0.15970E+01_JPRB ,&
&  0.16139E+01_JPRB, 0.15773E+01_JPRB, 0.16683E+01_JPRB, 0.17731E+01_JPRB, 0.18889E+01_JPRB, 0.16934E+01_JPRB, 0.15198E+01_JPRB ,&
&  0.12557E+01_JPRB, 0.10853E+01_JPRB, 0.96240E+00_JPRB, 0.10205E+01_JPRB, 0.10579E+01_JPRB, 0.11253E+01_JPRB, 0.11371E+01_JPRB ,&
&  0.12164E+01_JPRB, 0.12041E+01_JPRB, 0.11173E+01_JPRB, 0.11172E+01_JPRB, 0.10351E+01_JPRB, 0.94857E+00_JPRB, 0.95075E+00_JPRB ,&
&  0.95961E+00_JPRB, 0.97885E+00_JPRB, 0.95618E+00_JPRB, 0.93191E+00_JPRB, 0.91203E+00_JPRB, 0.89861E+00_JPRB, 0.90030E+00_JPRB ,&
&  0.89433E+00_JPRB, 0.88917E+00_JPRB, 0.88584E+00_JPRB, 0.87755E+00_JPRB, 0.86980E+00_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.83422E+00_JPRB, 0.82076E+00_JPRB, 0.79222E+00_JPRB, 0.77487E+00_JPRB, 0.78592E+00_JPRB, 0.79431E+00_JPRB, 0.77718E+00_JPRB ,&
&  0.77711E+00_JPRB, 0.78743E+00_JPRB, 0.80184E+00_JPRB, 0.82334E+00_JPRB, 0.84564E+00_JPRB, 0.86408E+00_JPRB, 0.91516E+00_JPRB ,&
&  0.97973E+00_JPRB, 0.11054E+01_JPRB, 0.13036E+01_JPRB, 0.15048E+01_JPRB, 0.16886E+01_JPRB, 0.18970E+01_JPRB, 0.20936E+01_JPRB ,&
&  0.23294E+01_JPRB, 0.23857E+01_JPRB, 0.21824E+01_JPRB, 0.15004E+01_JPRB, 0.12863E+01_JPRB, 0.13797E+01_JPRB, 0.13566E+01_JPRB ,&
&  0.13309E+01_JPRB, 0.12856E+01_JPRB, 0.12510E+01_JPRB, 0.12359E+01_JPRB, 0.12505E+01_JPRB, 0.12856E+01_JPRB, 0.13109E+01_JPRB ,&
&  0.13528E+01_JPRB, 0.14020E+01_JPRB, 0.14580E+01_JPRB, 0.15211E+01_JPRB, 0.15744E+01_JPRB, 0.16291E+01_JPRB, 0.16849E+01_JPRB ,&
&  0.17213E+01_JPRB, 0.17822E+01_JPRB, 0.18069E+01_JPRB, 0.17491E+01_JPRB, 0.16381E+01_JPRB, 0.15855E+01_JPRB, 0.15144E+01_JPRB ,&
&  0.14842E+01_JPRB, 0.14579E+01_JPRB, 0.14692E+01_JPRB, 0.15561E+01_JPRB, 0.16164E+01_JPRB, 0.14998E+01_JPRB, 0.15692E+01_JPRB ,&
&  0.18549E+01_JPRB, 0.21262E+01_JPRB, 0.19151E+01_JPRB, 0.23826E+01_JPRB, 0.14286E+01_JPRB, 0.16021E+01_JPRB, 0.15882E+01_JPRB ,&
&  0.13695E+01_JPRB, 0.14064E+01_JPRB, 0.15102E+01_JPRB, 0.16025E+01_JPRB, 0.16206E+01_JPRB, 0.14429E+01_JPRB, 0.13131E+01_JPRB ,&
&  0.11681E+01_JPRB, 0.98109E+00_JPRB, 0.87516E+00_JPRB, 0.90926E+00_JPRB, 0.99800E+00_JPRB, 0.10705E+01_JPRB, 0.96823E+00_JPRB ,&
&  0.84907E+00_JPRB, 0.10820E+01_JPRB, 0.11527E+01_JPRB, 0.13033E+01_JPRB, 0.12225E+01_JPRB, 0.95881E+00_JPRB, 0.97674E+00_JPRB ,&
&  0.99213E+00_JPRB, 0.97181E+00_JPRB, 0.95490E+00_JPRB, 0.92438E+00_JPRB, 0.91117E+00_JPRB, 0.89762E+00_JPRB, 0.88484E+00_JPRB ,&
&  0.87870E+00_JPRB, 0.87482E+00_JPRB, 0.87020E+00_JPRB, 0.85476E+00_JPRB, 0.84079E+00_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.78800E+00_JPRB, 0.76159E+00_JPRB, 0.74186E+00_JPRB, 0.74108E+00_JPRB, 0.75082E+00_JPRB, 0.75468E+00_JPRB, 0.76130E+00_JPRB ,&
&  0.78048E+00_JPRB, 0.80365E+00_JPRB, 0.82927E+00_JPRB, 0.86627E+00_JPRB, 0.90423E+00_JPRB, 0.94498E+00_JPRB, 0.10099E+01_JPRB ,&
&  0.10901E+01_JPRB, 0.12033E+01_JPRB, 0.13575E+01_JPRB, 0.15234E+01_JPRB, 0.16729E+01_JPRB, 0.18413E+01_JPRB, 0.20095E+01_JPRB ,&
&  0.21986E+01_JPRB, 0.22696E+01_JPRB, 0.21996E+01_JPRB, 0.20455E+01_JPRB, 0.16104E+01_JPRB, 0.16386E+01_JPRB, 0.17488E+01_JPRB ,&
&  0.17791E+01_JPRB, 0.15228E+01_JPRB, 0.14450E+01_JPRB, 0.14253E+01_JPRB, 0.14419E+01_JPRB, 0.14863E+01_JPRB, 0.15202E+01_JPRB ,&
&  0.15614E+01_JPRB, 0.15997E+01_JPRB, 0.16360E+01_JPRB, 0.16659E+01_JPRB, 0.16979E+01_JPRB, 0.17603E+01_JPRB, 0.18790E+01_JPRB ,&
&  0.19205E+01_JPRB, 0.19658E+01_JPRB, 0.20104E+01_JPRB, 0.19940E+01_JPRB, 0.20663E+01_JPRB, 0.21145E+01_JPRB, 0.20406E+01_JPRB ,&
&  0.19665E+01_JPRB, 0.18242E+01_JPRB, 0.18656E+01_JPRB, 0.19981E+01_JPRB, 0.20076E+01_JPRB, 0.18898E+01_JPRB, 0.19022E+01_JPRB ,&
&  0.21348E+01_JPRB, 0.23743E+01_JPRB, 0.16700E+01_JPRB, 0.18350E+01_JPRB, 0.20689E+01_JPRB, 0.16180E+01_JPRB, 0.12830E+01_JPRB ,&
&  0.11189E+01_JPRB, 0.11409E+01_JPRB, 0.12737E+01_JPRB, 0.13242E+01_JPRB, 0.13269E+01_JPRB, 0.14813E+01_JPRB, 0.11521E+01_JPRB ,&
&  0.10476E+01_JPRB, 0.88420E+00_JPRB, 0.80451E+00_JPRB, 0.81911E+00_JPRB, 0.87848E+00_JPRB, 0.90705E+00_JPRB, 0.80865E+00_JPRB ,&
&  0.79419E+00_JPRB, 0.11017E+01_JPRB, 0.12626E+01_JPRB, 0.15337E+01_JPRB, 0.15430E+01_JPRB, 0.94819E+00_JPRB, 0.95432E+00_JPRB ,&
&  0.95404E+00_JPRB, 0.94295E+00_JPRB, 0.92191E+00_JPRB, 0.90119E+00_JPRB, 0.88466E+00_JPRB, 0.86987E+00_JPRB, 0.85994E+00_JPRB ,&
&  0.85230E+00_JPRB, 0.84555E+00_JPRB, 0.84039E+00_JPRB, 0.82677E+00_JPRB, 0.80855E+00_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.72159E+00_JPRB, 0.70630E+00_JPRB, 0.70062E+00_JPRB, 0.71293E+00_JPRB, 0.73337E+00_JPRB, 0.74676E+00_JPRB, 0.75099E+00_JPRB ,&
&  0.76592E+00_JPRB, 0.78688E+00_JPRB, 0.82004E+00_JPRB, 0.84951E+00_JPRB, 0.88746E+00_JPRB, 0.92328E+00_JPRB, 0.98240E+00_JPRB ,&
&  0.10407E+01_JPRB, 0.11100E+01_JPRB, 0.12305E+01_JPRB, 0.14107E+01_JPRB, 0.15609E+01_JPRB, 0.16950E+01_JPRB, 0.18966E+01_JPRB ,&
&  0.20919E+01_JPRB, 0.21982E+01_JPRB, 0.23170E+01_JPRB, 0.23518E+01_JPRB, 0.22207E+01_JPRB, 0.20599E+01_JPRB, 0.20813E+01_JPRB ,&
&  0.17229E+01_JPRB, 0.14436E+01_JPRB, 0.14798E+01_JPRB, 0.15344E+01_JPRB, 0.15936E+01_JPRB, 0.16111E+01_JPRB, 0.16329E+01_JPRB ,&
&  0.16649E+01_JPRB, 0.16869E+01_JPRB, 0.17075E+01_JPRB, 0.17352E+01_JPRB, 0.17865E+01_JPRB, 0.18801E+01_JPRB, 0.20143E+01_JPRB ,&
&  0.20551E+01_JPRB, 0.21172E+01_JPRB, 0.22302E+01_JPRB, 0.20687E+01_JPRB, 0.22351E+01_JPRB, 0.24641E+01_JPRB, 0.25945E+01_JPRB ,&
&  0.25582E+01_JPRB, 0.23644E+01_JPRB, 0.23857E+01_JPRB, 0.24421E+01_JPRB, 0.24603E+01_JPRB, 0.24025E+01_JPRB, 0.23641E+01_JPRB ,&
&  0.24813E+01_JPRB, 0.23538E+01_JPRB, 0.16755E+01_JPRB, 0.14830E+01_JPRB, 0.14824E+01_JPRB, 0.11480E+01_JPRB, 0.11026E+01_JPRB ,&
&  0.96574E+00_JPRB, 0.97239E+00_JPRB, 0.10858E+01_JPRB, 0.11717E+01_JPRB, 0.12452E+01_JPRB, 0.14303E+01_JPRB, 0.11525E+01_JPRB ,&
&  0.92652E+00_JPRB, 0.77556E+00_JPRB, 0.69183E+00_JPRB, 0.68427E+00_JPRB, 0.71022E+00_JPRB, 0.74408E+00_JPRB, 0.76232E+00_JPRB ,&
&  0.81380E+00_JPRB, 0.85490E+00_JPRB, 0.88544E+00_JPRB, 0.93472E+00_JPRB, 0.89130E+00_JPRB, 0.91960E+00_JPRB, 0.90633E+00_JPRB ,&
&  0.88179E+00_JPRB, 0.88104E+00_JPRB, 0.87089E+00_JPRB, 0.84902E+00_JPRB, 0.83816E+00_JPRB, 0.82500E+00_JPRB, 0.81960E+00_JPRB ,&
&  0.81658E+00_JPRB, 0.81040E+00_JPRB, 0.79803E+00_JPRB, 0.78245E+00_JPRB, 0.75292E+00_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.67006E+00_JPRB, 0.65468E+00_JPRB, 0.66396E+00_JPRB, 0.68394E+00_JPRB, 0.69461E+00_JPRB, 0.70483E+00_JPRB, 0.71546E+00_JPRB ,&
&  0.71802E+00_JPRB, 0.71857E+00_JPRB, 0.72249E+00_JPRB, 0.72297E+00_JPRB, 0.75437E+00_JPRB, 0.79006E+00_JPRB, 0.83624E+00_JPRB ,&
&  0.90585E+00_JPRB, 0.96825E+00_JPRB, 0.10254E+01_JPRB, 0.11314E+01_JPRB, 0.12069E+01_JPRB, 0.12782E+01_JPRB, 0.13961E+01_JPRB ,&
&  0.15463E+01_JPRB, 0.17079E+01_JPRB, 0.18622E+01_JPRB, 0.20115E+01_JPRB, 0.21945E+01_JPRB, 0.23637E+01_JPRB, 0.23730E+01_JPRB ,&
&  0.17979E+01_JPRB, 0.14031E+01_JPRB, 0.15574E+01_JPRB, 0.14928E+01_JPRB, 0.15598E+01_JPRB, 0.16079E+01_JPRB, 0.16253E+01_JPRB ,&
&  0.16609E+01_JPRB, 0.16860E+01_JPRB, 0.17084E+01_JPRB, 0.17356E+01_JPRB, 0.18010E+01_JPRB, 0.18878E+01_JPRB, 0.20072E+01_JPRB ,&
&  0.20736E+01_JPRB, 0.21356E+01_JPRB, 0.21661E+01_JPRB, 0.21735E+01_JPRB, 0.23197E+01_JPRB, 0.25680E+01_JPRB, 0.27569E+01_JPRB ,&
&  0.28558E+01_JPRB, 0.29251E+01_JPRB, 0.26336E+01_JPRB, 0.26244E+01_JPRB, 0.25820E+01_JPRB, 0.23278E+01_JPRB, 0.23508E+01_JPRB ,&
&  0.24893E+01_JPRB, 0.22950E+01_JPRB, 0.14646E+01_JPRB, 0.13516E+01_JPRB, 0.12746E+01_JPRB, 0.11390E+01_JPRB, 0.10249E+01_JPRB ,&
&  0.84888E+00_JPRB, 0.83294E+00_JPRB, 0.90251E+00_JPRB, 0.10575E+01_JPRB, 0.11616E+01_JPRB, 0.12212E+01_JPRB, 0.10895E+01_JPRB ,&
&  0.85890E+00_JPRB, 0.72193E+00_JPRB, 0.64646E+00_JPRB, 0.65654E+00_JPRB, 0.66280E+00_JPRB, 0.70725E+00_JPRB, 0.69110E+00_JPRB ,&
&  0.74425E+00_JPRB, 0.80048E+00_JPRB, 0.94787E+00_JPRB, 0.89978E+00_JPRB, 0.78953E+00_JPRB, 0.86815E+00_JPRB, 0.84946E+00_JPRB ,&
&  0.83423E+00_JPRB, 0.83103E+00_JPRB, 0.82018E+00_JPRB, 0.80972E+00_JPRB, 0.80110E+00_JPRB, 0.80182E+00_JPRB, 0.78712E+00_JPRB ,&
&  0.77730E+00_JPRB, 0.76109E+00_JPRB, 0.75119E+00_JPRB, 0.71627E+00_JPRB, 0.69178E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.62082E+00_JPRB, 0.62497E+00_JPRB, 0.63381E+00_JPRB, 0.65045E+00_JPRB, 0.64735E+00_JPRB, 0.64473E+00_JPRB, 0.66966E+00_JPRB ,&
&  0.65228E+00_JPRB, 0.63891E+00_JPRB, 0.63567E+00_JPRB, 0.60944E+00_JPRB, 0.62595E+00_JPRB, 0.64181E+00_JPRB, 0.67727E+00_JPRB ,&
&  0.71444E+00_JPRB, 0.73745E+00_JPRB, 0.75827E+00_JPRB, 0.78969E+00_JPRB, 0.84815E+00_JPRB, 0.89291E+00_JPRB, 0.93060E+00_JPRB ,&
&  0.99646E+00_JPRB, 0.10997E+01_JPRB, 0.12141E+01_JPRB, 0.14792E+01_JPRB, 0.20913E+01_JPRB, 0.25750E+01_JPRB, 0.25604E+01_JPRB ,&
&  0.19178E+01_JPRB, 0.14035E+01_JPRB, 0.13482E+01_JPRB, 0.12405E+01_JPRB, 0.13039E+01_JPRB, 0.14417E+01_JPRB, 0.14882E+01_JPRB ,&
&  0.14791E+01_JPRB, 0.14962E+01_JPRB, 0.15052E+01_JPRB, 0.15209E+01_JPRB, 0.15457E+01_JPRB, 0.15634E+01_JPRB, 0.15544E+01_JPRB ,&
&  0.15005E+01_JPRB, 0.14371E+01_JPRB, 0.14867E+01_JPRB, 0.17916E+01_JPRB, 0.20524E+01_JPRB, 0.24650E+01_JPRB, 0.24306E+01_JPRB ,&
&  0.23255E+01_JPRB, 0.22412E+01_JPRB, 0.22001E+01_JPRB, 0.21501E+01_JPRB, 0.20692E+01_JPRB, 0.19754E+01_JPRB, 0.20034E+01_JPRB ,&
&  0.21083E+01_JPRB, 0.19591E+01_JPRB, 0.13100E+01_JPRB, 0.13286E+01_JPRB, 0.12356E+01_JPRB, 0.11976E+01_JPRB, 0.96473E+00_JPRB ,&
&  0.78601E+00_JPRB, 0.74096E+00_JPRB, 0.75236E+00_JPRB, 0.80964E+00_JPRB, 0.93032E+00_JPRB, 0.10158E+01_JPRB, 0.94002E+00_JPRB ,&
&  0.82015E+00_JPRB, 0.71047E+00_JPRB, 0.64336E+00_JPRB, 0.60208E+00_JPRB, 0.64853E+00_JPRB, 0.63894E+00_JPRB, 0.76531E+00_JPRB ,&
&  0.63521E+00_JPRB, 0.69523E+00_JPRB, 0.65146E+00_JPRB, 0.84184E+00_JPRB, 0.76017E+00_JPRB, 0.73994E+00_JPRB, 0.72958E+00_JPRB ,&
&  0.72506E+00_JPRB, 0.70843E+00_JPRB, 0.69215E+00_JPRB, 0.67118E+00_JPRB, 0.65002E+00_JPRB, 0.67623E+00_JPRB, 0.71003E+00_JPRB ,&
&  0.67899E+00_JPRB, 0.69469E+00_JPRB, 0.67561E+00_JPRB, 0.64525E+00_JPRB, 0.64060E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.60478E+00_JPRB, 0.61696E+00_JPRB, 0.61788E+00_JPRB, 0.62410E+00_JPRB, 0.63715E+00_JPRB, 0.64578E+00_JPRB, 0.65878E+00_JPRB ,&
&  0.67262E+00_JPRB, 0.67831E+00_JPRB, 0.68389E+00_JPRB, 0.69150E+00_JPRB, 0.69594E+00_JPRB, 0.71449E+00_JPRB, 0.73300E+00_JPRB ,&
&  0.75770E+00_JPRB, 0.77705E+00_JPRB, 0.79761E+00_JPRB, 0.82944E+00_JPRB, 0.86989E+00_JPRB, 0.93391E+00_JPRB, 0.10010E+01_JPRB ,&
&  0.10877E+01_JPRB, 0.11892E+01_JPRB, 0.13093E+01_JPRB, 0.14058E+01_JPRB, 0.14545E+01_JPRB, 0.14759E+01_JPRB, 0.16091E+01_JPRB ,&
&  0.14026E+01_JPRB, 0.14880E+01_JPRB, 0.14159E+01_JPRB, 0.12689E+01_JPRB, 0.12568E+01_JPRB, 0.11712E+01_JPRB, 0.11479E+01_JPRB ,&
&  0.11583E+01_JPRB, 0.11110E+01_JPRB, 0.10881E+01_JPRB, 0.10687E+01_JPRB, 0.10475E+01_JPRB, 0.99851E+00_JPRB, 0.93970E+00_JPRB ,&
&  0.93242E+00_JPRB, 0.99237E+00_JPRB, 0.11218E+01_JPRB, 0.13888E+01_JPRB, 0.16489E+01_JPRB, 0.17401E+01_JPRB, 0.16288E+01_JPRB ,&
&  0.15546E+01_JPRB, 0.15381E+01_JPRB, 0.14519E+01_JPRB, 0.13809E+01_JPRB, 0.14402E+01_JPRB, 0.14874E+01_JPRB, 0.15342E+01_JPRB ,&
&  0.14694E+01_JPRB, 0.13160E+01_JPRB, 0.12492E+01_JPRB, 0.11896E+01_JPRB, 0.11402E+01_JPRB, 0.10770E+01_JPRB, 0.94101E+00_JPRB ,&
&  0.79373E+00_JPRB, 0.70884E+00_JPRB, 0.66935E+00_JPRB, 0.68489E+00_JPRB, 0.76659E+00_JPRB, 0.78470E+00_JPRB, 0.73687E+00_JPRB ,&
&  0.65059E+00_JPRB, 0.60146E+00_JPRB, 0.57105E+00_JPRB, 0.55018E+00_JPRB, 0.53717E+00_JPRB, 0.85180E+00_JPRB, 0.11460E+01_JPRB ,&
&  0.57628E+00_JPRB, 0.50761E+00_JPRB, 0.45828E+00_JPRB, 0.62994E+00_JPRB, 0.62128E+00_JPRB, 0.63115E+00_JPRB, 0.63116E+00_JPRB ,&
&  0.62264E+00_JPRB, 0.62169E+00_JPRB, 0.61910E+00_JPRB, 0.60286E+00_JPRB, 0.59089E+00_JPRB, 0.58752E+00_JPRB, 0.61147E+00_JPRB ,&
&  0.60750E+00_JPRB, 0.60741E+00_JPRB, 0.60230E+00_JPRB, 0.61298E+00_JPRB, 0.60925E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.60111E+00_JPRB, 0.61429E+00_JPRB, 0.63008E+00_JPRB, 0.64975E+00_JPRB, 0.66222E+00_JPRB, 0.68001E+00_JPRB, 0.69420E+00_JPRB ,&
&  0.71431E+00_JPRB, 0.73010E+00_JPRB, 0.74322E+00_JPRB, 0.75677E+00_JPRB, 0.77459E+00_JPRB, 0.79718E+00_JPRB, 0.82628E+00_JPRB ,&
&  0.85795E+00_JPRB, 0.88960E+00_JPRB, 0.92423E+00_JPRB, 0.96401E+00_JPRB, 0.10107E+01_JPRB, 0.10680E+01_JPRB, 0.11220E+01_JPRB ,&
&  0.11717E+01_JPRB, 0.12209E+01_JPRB, 0.12689E+01_JPRB, 0.13059E+01_JPRB, 0.13172E+01_JPRB, 0.13297E+01_JPRB, 0.11724E+01_JPRB ,&
&  0.14223E+01_JPRB, 0.14039E+01_JPRB, 0.13103E+01_JPRB, 0.11963E+01_JPRB, 0.10942E+01_JPRB, 0.10032E+01_JPRB, 0.92366E+00_JPRB ,&
&  0.93227E+00_JPRB, 0.89511E+00_JPRB, 0.88307E+00_JPRB, 0.88252E+00_JPRB, 0.83181E+00_JPRB, 0.83800E+00_JPRB, 0.88336E+00_JPRB ,&
&  0.96397E+00_JPRB, 0.10826E+01_JPRB, 0.11449E+01_JPRB, 0.12185E+01_JPRB, 0.13232E+01_JPRB, 0.12717E+01_JPRB, 0.12231E+01_JPRB ,&
&  0.11301E+01_JPRB, 0.10618E+01_JPRB, 0.85430E+00_JPRB, 0.95673E+00_JPRB, 0.10017E+01_JPRB, 0.10466E+01_JPRB, 0.11103E+01_JPRB ,&
&  0.10340E+01_JPRB, 0.87311E+00_JPRB, 0.91699E+00_JPRB, 0.11145E+01_JPRB, 0.10591E+01_JPRB, 0.96647E+00_JPRB, 0.88462E+00_JPRB ,&
&  0.79892E+00_JPRB, 0.73262E+00_JPRB, 0.67529E+00_JPRB, 0.65472E+00_JPRB, 0.68618E+00_JPRB, 0.68269E+00_JPRB, 0.63189E+00_JPRB ,&
&  0.56645E+00_JPRB, 0.53981E+00_JPRB, 0.52924E+00_JPRB, 0.52192E+00_JPRB, 0.55729E+00_JPRB, 0.57568E+00_JPRB, 0.81993E+00_JPRB ,&
&  0.56723E+00_JPRB, 0.37937E+00_JPRB, 0.45500E+00_JPRB, 0.55734E+00_JPRB, 0.54547E+00_JPRB, 0.58692E+00_JPRB, 0.62125E+00_JPRB ,&
&  0.65385E+00_JPRB, 0.66601E+00_JPRB, 0.62389E+00_JPRB, 0.62176E+00_JPRB, 0.61181E+00_JPRB, 0.60140E+00_JPRB, 0.59019E+00_JPRB ,&
&  0.57937E+00_JPRB, 0.58058E+00_JPRB, 0.59432E+00_JPRB, 0.59343E+00_JPRB, 0.59476E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.53002E+00_JPRB, 0.53899E+00_JPRB, 0.54971E+00_JPRB, 0.55887E+00_JPRB, 0.56434E+00_JPRB, 0.57497E+00_JPRB, 0.58259E+00_JPRB ,&
&  0.59183E+00_JPRB, 0.59522E+00_JPRB, 0.60232E+00_JPRB, 0.60817E+00_JPRB, 0.61675E+00_JPRB, 0.62984E+00_JPRB, 0.64280E+00_JPRB ,&
&  0.65210E+00_JPRB, 0.66678E+00_JPRB, 0.68460E+00_JPRB, 0.69571E+00_JPRB, 0.72633E+00_JPRB, 0.78504E+00_JPRB, 0.85080E+00_JPRB ,&
&  0.90150E+00_JPRB, 0.96579E+00_JPRB, 0.10245E+01_JPRB, 0.10447E+01_JPRB, 0.10356E+01_JPRB, 0.10459E+01_JPRB, 0.90233E+00_JPRB ,&
&  0.12674E+01_JPRB, 0.12628E+01_JPRB, 0.11470E+01_JPRB, 0.10784E+01_JPRB, 0.10126E+01_JPRB, 0.94825E+00_JPRB, 0.90346E+00_JPRB ,&
&  0.90511E+00_JPRB, 0.10520E+01_JPRB, 0.10791E+01_JPRB, 0.11235E+01_JPRB, 0.10606E+01_JPRB, 0.10665E+01_JPRB, 0.11782E+01_JPRB ,&
&  0.13097E+01_JPRB, 0.13412E+01_JPRB, 0.13216E+01_JPRB, 0.13746E+01_JPRB, 0.14434E+01_JPRB, 0.14772E+01_JPRB, 0.14873E+01_JPRB ,&
&  0.14298E+01_JPRB, 0.13734E+01_JPRB, 0.12744E+01_JPRB, 0.14802E+01_JPRB, 0.13412E+01_JPRB, 0.11959E+01_JPRB, 0.10986E+01_JPRB ,&
&  0.86648E+00_JPRB, 0.82614E+00_JPRB, 0.88558E+00_JPRB, 0.95838E+00_JPRB, 0.91136E+00_JPRB, 0.89350E+00_JPRB, 0.84272E+00_JPRB ,&
&  0.79023E+00_JPRB, 0.75787E+00_JPRB, 0.71788E+00_JPRB, 0.70314E+00_JPRB, 0.70232E+00_JPRB, 0.67111E+00_JPRB, 0.60819E+00_JPRB ,&
&  0.55845E+00_JPRB, 0.54643E+00_JPRB, 0.53069E+00_JPRB, 0.52324E+00_JPRB, 0.53299E+00_JPRB, 0.49775E+00_JPRB, 0.65507E+00_JPRB ,&
&  0.56219E+00_JPRB, 0.48278E+00_JPRB, 0.61137E+00_JPRB, 0.40181E+00_JPRB, 0.51126E+00_JPRB, 0.62754E+00_JPRB, 0.81634E+00_JPRB ,&
&  0.87110E+00_JPRB, 0.11283E+01_JPRB, 0.14777E+01_JPRB, 0.11700E+01_JPRB, 0.88579E+00_JPRB, 0.59233E+00_JPRB, 0.55742E+00_JPRB ,&
&  0.54712E+00_JPRB, 0.53718E+00_JPRB, 0.54008E+00_JPRB, 0.53294E+00_JPRB, 0.53145E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.46295E+00_JPRB, 0.46494E+00_JPRB, 0.47258E+00_JPRB, 0.47752E+00_JPRB, 0.48264E+00_JPRB, 0.48611E+00_JPRB, 0.48980E+00_JPRB ,&
&  0.50316E+00_JPRB, 0.51082E+00_JPRB, 0.53062E+00_JPRB, 0.54504E+00_JPRB, 0.55776E+00_JPRB, 0.58666E+00_JPRB, 0.60925E+00_JPRB ,&
&  0.64631E+00_JPRB, 0.69426E+00_JPRB, 0.71097E+00_JPRB, 0.73763E+00_JPRB, 0.75233E+00_JPRB, 0.76720E+00_JPRB, 0.77651E+00_JPRB ,&
&  0.83434E+00_JPRB, 0.94725E+00_JPRB, 0.10233E+01_JPRB, 0.11016E+01_JPRB, 0.10986E+01_JPRB, 0.11320E+01_JPRB, 0.91286E+00_JPRB ,&
&  0.12194E+01_JPRB, 0.12393E+01_JPRB, 0.11716E+01_JPRB, 0.11439E+01_JPRB, 0.11318E+01_JPRB, 0.11255E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.11597E+01_JPRB, 0.12446E+01_JPRB, 0.12591E+01_JPRB, 0.13508E+01_JPRB, 0.14407E+01_JPRB, 0.14577E+01_JPRB, 0.16073E+01_JPRB ,&
&  0.17278E+01_JPRB, 0.18231E+01_JPRB, 0.19489E+01_JPRB, 0.21101E+01_JPRB, 0.22273E+01_JPRB, 0.23012E+01_JPRB, 0.23260E+01_JPRB ,&
&  0.23285E+01_JPRB, 0.23919E+01_JPRB, 0.25798E+01_JPRB, 0.25573E+01_JPRB, 0.20674E+01_JPRB, 0.17199E+01_JPRB, 0.13559E+01_JPRB ,&
&  0.95361E+00_JPRB, 0.79557E+00_JPRB, 0.79727E+00_JPRB, 0.85278E+00_JPRB, 0.81993E+00_JPRB, 0.79527E+00_JPRB, 0.76813E+00_JPRB ,&
&  0.74539E+00_JPRB, 0.75367E+00_JPRB, 0.75760E+00_JPRB, 0.75283E+00_JPRB, 0.72716E+00_JPRB, 0.66830E+00_JPRB, 0.60258E+00_JPRB ,&
&  0.58175E+00_JPRB, 0.56606E+00_JPRB, 0.56035E+00_JPRB, 0.57017E+00_JPRB, 0.58577E+00_JPRB, 0.63407E+00_JPRB, 0.60865E+00_JPRB ,&
&  0.66680E+00_JPRB, 0.52974E+00_JPRB, 0.53510E+00_JPRB, 0.56696E+00_JPRB, 0.60089E+00_JPRB, 0.83168E+00_JPRB, 0.12265E+01_JPRB ,&
&  0.18176E+01_JPRB, 0.20879E+01_JPRB, 0.27752E+01_JPRB, 0.40784E+01_JPRB, 0.25671E+01_JPRB, 0.16367E+01_JPRB, 0.53101E+00_JPRB ,&
&  0.52025E+00_JPRB, 0.49540E+00_JPRB, 0.48820E+00_JPRB, 0.47319E+00_JPRB, 0.46842E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.38214E+00_JPRB, 0.37607E+00_JPRB, 0.38572E+00_JPRB, 0.40246E+00_JPRB, 0.41494E+00_JPRB, 0.44331E+00_JPRB, 0.47745E+00_JPRB ,&
&  0.51477E+00_JPRB, 0.56053E+00_JPRB, 0.59721E+00_JPRB, 0.62083E+00_JPRB, 0.66288E+00_JPRB, 0.72323E+00_JPRB, 0.74527E+00_JPRB ,&
&  0.79322E+00_JPRB, 0.81985E+00_JPRB, 0.82758E+00_JPRB, 0.83113E+00_JPRB, 0.85218E+00_JPRB, 0.89111E+00_JPRB, 0.93583E+00_JPRB ,&
&  0.97506E+00_JPRB, 0.10266E+01_JPRB, 0.11000E+01_JPRB, 0.11472E+01_JPRB, 0.11672E+01_JPRB, 0.11665E+01_JPRB, 0.92863E+00_JPRB ,&
&  0.12058E+01_JPRB, 0.12725E+01_JPRB, 0.13106E+01_JPRB, 0.13022E+01_JPRB, 0.12911E+01_JPRB, 0.12890E+01_JPRB, 0.13248E+01_JPRB ,&
&  0.13912E+01_JPRB, 0.14208E+01_JPRB, 0.14397E+01_JPRB, 0.14658E+01_JPRB, 0.18446E+01_JPRB, 0.19165E+01_JPRB, 0.20822E+01_JPRB ,&
&  0.22299E+01_JPRB, 0.23615E+01_JPRB, 0.25431E+01_JPRB, 0.27365E+01_JPRB, 0.29370E+01_JPRB, 0.31103E+01_JPRB, 0.32642E+01_JPRB ,&
&  0.34034E+01_JPRB, 0.35567E+01_JPRB, 0.37083E+01_JPRB, 0.31984E+01_JPRB, 0.25598E+01_JPRB, 0.21140E+01_JPRB, 0.13754E+01_JPRB ,&
&  0.91373E+00_JPRB, 0.80691E+00_JPRB, 0.83153E+00_JPRB, 0.83918E+00_JPRB, 0.76947E+00_JPRB, 0.72777E+00_JPRB, 0.69381E+00_JPRB ,&
&  0.69883E+00_JPRB, 0.72336E+00_JPRB, 0.74471E+00_JPRB, 0.74739E+00_JPRB, 0.71340E+00_JPRB, 0.66100E+00_JPRB, 0.61193E+00_JPRB ,&
&  0.62127E+00_JPRB, 0.61550E+00_JPRB, 0.60977E+00_JPRB, 0.64033E+00_JPRB, 0.66409E+00_JPRB, 0.76263E+00_JPRB, 0.90360E+00_JPRB ,&
&  0.89020E+00_JPRB, 0.56267E+00_JPRB, 0.56513E+00_JPRB, 0.58078E+00_JPRB, 0.60999E+00_JPRB, 0.68397E+00_JPRB, 0.83995E+00_JPRB ,&
&  0.11866E+01_JPRB, 0.19450E+01_JPRB, 0.22185E+01_JPRB, 0.14672E+01_JPRB, 0.19925E+01_JPRB, 0.12929E+01_JPRB, 0.51651E+00_JPRB ,&
&  0.48860E+00_JPRB, 0.47409E+00_JPRB, 0.44677E+00_JPRB, 0.41702E+00_JPRB, 0.39683E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.38780E+00_JPRB, 0.37045E+00_JPRB, 0.38025E+00_JPRB, 0.40401E+00_JPRB, 0.43245E+00_JPRB, 0.47530E+00_JPRB, 0.52675E+00_JPRB ,&
&  0.57095E+00_JPRB, 0.61310E+00_JPRB, 0.68553E+00_JPRB, 0.69940E+00_JPRB, 0.70966E+00_JPRB, 0.74441E+00_JPRB, 0.76433E+00_JPRB ,&
&  0.78597E+00_JPRB, 0.79899E+00_JPRB, 0.81662E+00_JPRB, 0.83447E+00_JPRB, 0.86776E+00_JPRB, 0.90574E+00_JPRB, 0.93615E+00_JPRB ,&
&  0.97773E+00_JPRB, 0.10250E+01_JPRB, 0.10985E+01_JPRB, 0.11767E+01_JPRB, 0.12489E+01_JPRB, 0.12609E+01_JPRB, 0.12780E+01_JPRB ,&
&  0.92545E+00_JPRB, 0.12586E+01_JPRB, 0.13042E+01_JPRB, 0.13292E+01_JPRB, 0.13261E+01_JPRB, 0.13221E+01_JPRB, 0.13698E+01_JPRB ,&
&  0.14411E+01_JPRB, 0.14329E+01_JPRB, 0.15265E+01_JPRB, 0.16909E+01_JPRB, 0.20848E+01_JPRB, 0.21807E+01_JPRB, 0.23042E+01_JPRB ,&
&  0.24446E+01_JPRB, 0.25791E+01_JPRB, 0.27347E+01_JPRB, 0.29232E+01_JPRB, 0.31168E+01_JPRB, 0.33003E+01_JPRB, 0.34877E+01_JPRB ,&
&  0.36501E+01_JPRB, 0.37991E+01_JPRB, 0.39736E+01_JPRB, 0.33146E+01_JPRB, 0.27601E+01_JPRB, 0.22289E+01_JPRB, 0.14903E+01_JPRB ,&
&  0.99181E+00_JPRB, 0.79632E+00_JPRB, 0.82444E+00_JPRB, 0.86573E+00_JPRB, 0.77947E+00_JPRB, 0.72422E+00_JPRB, 0.65631E+00_JPRB ,&
&  0.64915E+00_JPRB, 0.66234E+00_JPRB, 0.67623E+00_JPRB, 0.67403E+00_JPRB, 0.67663E+00_JPRB, 0.63533E+00_JPRB, 0.61225E+00_JPRB ,&
&  0.62423E+00_JPRB, 0.61547E+00_JPRB, 0.61308E+00_JPRB, 0.62712E+00_JPRB, 0.65976E+00_JPRB, 0.69650E+00_JPRB, 0.69997E+00_JPRB ,&
&  0.70390E+00_JPRB, 0.63681E+00_JPRB, 0.61422E+00_JPRB, 0.63161E+00_JPRB, 0.63472E+00_JPRB, 0.63633E+00_JPRB, 0.66017E+00_JPRB ,&
&  0.74589E+00_JPRB, 0.85008E+00_JPRB, 0.94028E+00_JPRB, 0.93144E+00_JPRB, 0.97981E+00_JPRB, 0.76970E+00_JPRB, 0.57140E+00_JPRB ,&
&  0.53598E+00_JPRB, 0.52456E+00_JPRB, 0.48142E+00_JPRB, 0.45155E+00_JPRB, 0.41750E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.47944E+00_JPRB, 0.42481E+00_JPRB, 0.41987E+00_JPRB, 0.46103E+00_JPRB, 0.50338E+00_JPRB, 0.54674E+00_JPRB, 0.58581E+00_JPRB ,&
&  0.62514E+00_JPRB, 0.64612E+00_JPRB, 0.68836E+00_JPRB, 0.70632E+00_JPRB, 0.71870E+00_JPRB, 0.71803E+00_JPRB, 0.71776E+00_JPRB ,&
&  0.73659E+00_JPRB, 0.75688E+00_JPRB, 0.77340E+00_JPRB, 0.80482E+00_JPRB, 0.83792E+00_JPRB, 0.87058E+00_JPRB, 0.91078E+00_JPRB ,&
&  0.94972E+00_JPRB, 0.99566E+00_JPRB, 0.10430E+01_JPRB, 0.11019E+01_JPRB, 0.11437E+01_JPRB, 0.11560E+01_JPRB, 0.10992E+01_JPRB ,&
&  0.60542E+00_JPRB, 0.97292E+00_JPRB, 0.12567E+01_JPRB, 0.12775E+01_JPRB, 0.12856E+01_JPRB, 0.12850E+01_JPRB, 0.13492E+01_JPRB ,&
&  0.13456E+01_JPRB, 0.13054E+01_JPRB, 0.14204E+01_JPRB, 0.19274E+01_JPRB, 0.21296E+01_JPRB, 0.22462E+01_JPRB, 0.23219E+01_JPRB ,&
&  0.24183E+01_JPRB, 0.24933E+01_JPRB, 0.25850E+01_JPRB, 0.26837E+01_JPRB, 0.28018E+01_JPRB, 0.29154E+01_JPRB, 0.30182E+01_JPRB ,&
&  0.31012E+01_JPRB, 0.31459E+01_JPRB, 0.31360E+01_JPRB, 0.24784E+01_JPRB, 0.24248E+01_JPRB, 0.20572E+01_JPRB, 0.17636E+01_JPRB ,&
&  0.11355E+01_JPRB, 0.83890E+00_JPRB, 0.83137E+00_JPRB, 0.86716E+00_JPRB, 0.77965E+00_JPRB, 0.72416E+00_JPRB, 0.64986E+00_JPRB ,&
&  0.64793E+00_JPRB, 0.64883E+00_JPRB, 0.65570E+00_JPRB, 0.64623E+00_JPRB, 0.64597E+00_JPRB, 0.60351E+00_JPRB, 0.62392E+00_JPRB ,&
&  0.63846E+00_JPRB, 0.62919E+00_JPRB, 0.63269E+00_JPRB, 0.65691E+00_JPRB, 0.69034E+00_JPRB, 0.72430E+00_JPRB, 0.73484E+00_JPRB ,&
&  0.72652E+00_JPRB, 0.71087E+00_JPRB, 0.71103E+00_JPRB, 0.71130E+00_JPRB, 0.71082E+00_JPRB, 0.70971E+00_JPRB, 0.69964E+00_JPRB ,&
&  0.68877E+00_JPRB, 0.70297E+00_JPRB, 0.73951E+00_JPRB, 0.68935E+00_JPRB, 0.73122E+00_JPRB, 0.66335E+00_JPRB, 0.64766E+00_JPRB ,&
&  0.63131E+00_JPRB, 0.61320E+00_JPRB, 0.56755E+00_JPRB, 0.54497E+00_JPRB, 0.51552E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.57387E+00_JPRB, 0.51199E+00_JPRB, 0.46028E+00_JPRB, 0.47548E+00_JPRB, 0.53785E+00_JPRB, 0.59096E+00_JPRB, 0.61844E+00_JPRB ,&
&  0.64995E+00_JPRB, 0.65709E+00_JPRB, 0.66202E+00_JPRB, 0.67003E+00_JPRB, 0.68965E+00_JPRB, 0.69280E+00_JPRB, 0.70910E+00_JPRB ,&
&  0.71457E+00_JPRB, 0.71514E+00_JPRB, 0.72187E+00_JPRB, 0.74883E+00_JPRB, 0.78216E+00_JPRB, 0.81642E+00_JPRB, 0.84948E+00_JPRB ,&
&  0.88456E+00_JPRB, 0.92273E+00_JPRB, 0.97898E+00_JPRB, 0.10492E+01_JPRB, 0.11422E+01_JPRB, 0.12439E+01_JPRB, 0.13183E+01_JPRB ,&
&  0.95846E+00_JPRB, 0.54649E+00_JPRB, 0.98672E+00_JPRB, 0.12391E+01_JPRB, 0.12166E+01_JPRB, 0.12117E+01_JPRB, 0.12067E+01_JPRB ,&
&  0.12010E+01_JPRB, 0.12343E+01_JPRB, 0.12919E+01_JPRB, 0.16761E+01_JPRB, 0.20252E+01_JPRB, 0.21556E+01_JPRB, 0.22030E+01_JPRB ,&
&  0.22492E+01_JPRB, 0.22675E+01_JPRB, 0.22991E+01_JPRB, 0.23503E+01_JPRB, 0.24150E+01_JPRB, 0.24585E+01_JPRB, 0.24881E+01_JPRB ,&
&  0.25125E+01_JPRB, 0.24972E+01_JPRB, 0.24023E+01_JPRB, 0.17510E+01_JPRB, 0.15837E+01_JPRB, 0.13957E+01_JPRB, 0.12821E+01_JPRB ,&
&  0.11441E+01_JPRB, 0.92201E+00_JPRB, 0.86284E+00_JPRB, 0.86225E+00_JPRB, 0.79186E+00_JPRB, 0.69284E+00_JPRB, 0.67512E+00_JPRB ,&
&  0.67780E+00_JPRB, 0.65987E+00_JPRB, 0.63647E+00_JPRB, 0.61967E+00_JPRB, 0.64655E+00_JPRB, 0.64435E+00_JPRB, 0.64316E+00_JPRB ,&
&  0.63778E+00_JPRB, 0.66605E+00_JPRB, 0.69469E+00_JPRB, 0.71810E+00_JPRB, 0.74284E+00_JPRB, 0.77257E+00_JPRB, 0.77451E+00_JPRB ,&
&  0.78246E+00_JPRB, 0.77656E+00_JPRB, 0.77746E+00_JPRB, 0.76240E+00_JPRB, 0.74966E+00_JPRB, 0.72325E+00_JPRB, 0.72159E+00_JPRB ,&
&  0.72728E+00_JPRB, 0.73550E+00_JPRB, 0.73344E+00_JPRB, 0.72437E+00_JPRB, 0.74764E+00_JPRB, 0.74241E+00_JPRB, 0.75091E+00_JPRB ,&
&  0.74844E+00_JPRB, 0.72826E+00_JPRB, 0.67171E+00_JPRB, 0.62730E+00_JPRB, 0.60906E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.61783E+00_JPRB, 0.58408E+00_JPRB, 0.52950E+00_JPRB, 0.49467E+00_JPRB, 0.51708E+00_JPRB, 0.58183E+00_JPRB, 0.63304E+00_JPRB ,&
&  0.65201E+00_JPRB, 0.64959E+00_JPRB, 0.63926E+00_JPRB, 0.63662E+00_JPRB, 0.64768E+00_JPRB, 0.64151E+00_JPRB, 0.64933E+00_JPRB ,&
&  0.66196E+00_JPRB, 0.66764E+00_JPRB, 0.67784E+00_JPRB, 0.69410E+00_JPRB, 0.71184E+00_JPRB, 0.74190E+00_JPRB, 0.77207E+00_JPRB ,&
&  0.79500E+00_JPRB, 0.81520E+00_JPRB, 0.84677E+00_JPRB, 0.88939E+00_JPRB, 0.95530E+00_JPRB, 0.10801E+01_JPRB, 0.12996E+01_JPRB ,&
&  0.17140E+01_JPRB, 0.11909E+01_JPRB, 0.67167E+00_JPRB, 0.11418E+01_JPRB, 0.12450E+01_JPRB, 0.12076E+01_JPRB, 0.10952E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.10999E+01_JPRB, 0.11468E+01_JPRB, 0.13288E+01_JPRB, 0.15358E+01_JPRB, 0.17931E+01_JPRB, 0.19478E+01_JPRB ,&
&  0.19835E+01_JPRB, 0.19976E+01_JPRB, 0.20094E+01_JPRB, 0.20418E+01_JPRB, 0.20980E+01_JPRB, 0.21559E+01_JPRB, 0.21687E+01_JPRB ,&
&  0.21883E+01_JPRB, 0.22261E+01_JPRB, 0.21629E+01_JPRB, 0.16211E+01_JPRB, 0.15014E+01_JPRB, 0.14527E+01_JPRB, 0.13871E+01_JPRB ,&
&  0.12303E+01_JPRB, 0.11274E+01_JPRB, 0.10437E+01_JPRB, 0.91258E+00_JPRB, 0.81037E+00_JPRB, 0.74847E+00_JPRB, 0.76247E+00_JPRB ,&
&  0.74903E+00_JPRB, 0.70148E+00_JPRB, 0.67563E+00_JPRB, 0.65576E+00_JPRB, 0.67481E+00_JPRB, 0.69691E+00_JPRB, 0.69426E+00_JPRB ,&
&  0.70851E+00_JPRB, 0.73228E+00_JPRB, 0.75003E+00_JPRB, 0.76718E+00_JPRB, 0.79011E+00_JPRB, 0.81128E+00_JPRB, 0.82195E+00_JPRB ,&
&  0.83280E+00_JPRB, 0.82993E+00_JPRB, 0.82572E+00_JPRB, 0.81408E+00_JPRB, 0.77875E+00_JPRB, 0.74589E+00_JPRB, 0.75368E+00_JPRB ,&
&  0.75174E+00_JPRB, 0.75893E+00_JPRB, 0.75083E+00_JPRB, 0.76248E+00_JPRB, 0.76840E+00_JPRB, 0.78923E+00_JPRB, 0.81951E+00_JPRB ,&
&  0.84696E+00_JPRB, 0.83159E+00_JPRB, 0.75043E+00_JPRB, 0.65699E+00_JPRB, 0.63494E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.64310E+00_JPRB, 0.63229E+00_JPRB, 0.60338E+00_JPRB, 0.54228E+00_JPRB, 0.52506E+00_JPRB, 0.54130E+00_JPRB, 0.60212E+00_JPRB ,&
&  0.62803E+00_JPRB, 0.63418E+00_JPRB, 0.61889E+00_JPRB, 0.60239E+00_JPRB, 0.59328E+00_JPRB, 0.59364E+00_JPRB, 0.60855E+00_JPRB ,&
&  0.62191E+00_JPRB, 0.61414E+00_JPRB, 0.61225E+00_JPRB, 0.62020E+00_JPRB, 0.63777E+00_JPRB, 0.66477E+00_JPRB, 0.69861E+00_JPRB ,&
&  0.72580E+00_JPRB, 0.73778E+00_JPRB, 0.74470E+00_JPRB, 0.77176E+00_JPRB, 0.80620E+00_JPRB, 0.81830E+00_JPRB, 0.87051E+00_JPRB ,&
&  0.11509E+01_JPRB, 0.15972E+01_JPRB, 0.77917E+00_JPRB, 0.11881E+01_JPRB, 0.13815E+01_JPRB, 0.12451E+01_JPRB, 0.11413E+01_JPRB ,&
&  0.10786E+01_JPRB, 0.96089E+00_JPRB, 0.93889E+00_JPRB, 0.10665E+01_JPRB, 0.11732E+01_JPRB, 0.12727E+01_JPRB, 0.14252E+01_JPRB ,&
&  0.15643E+01_JPRB, 0.16500E+01_JPRB, 0.17177E+01_JPRB, 0.17682E+01_JPRB, 0.18424E+01_JPRB, 0.19535E+01_JPRB, 0.20549E+01_JPRB ,&
&  0.21288E+01_JPRB, 0.22393E+01_JPRB, 0.22956E+01_JPRB, 0.18378E+01_JPRB, 0.17522E+01_JPRB, 0.18675E+01_JPRB, 0.17719E+01_JPRB ,&
&  0.15475E+01_JPRB, 0.13959E+01_JPRB, 0.12153E+01_JPRB, 0.10133E+01_JPRB, 0.87439E+00_JPRB, 0.81931E+00_JPRB, 0.81871E+00_JPRB ,&
&  0.78618E+00_JPRB, 0.75055E+00_JPRB, 0.74230E+00_JPRB, 0.74276E+00_JPRB, 0.74359E+00_JPRB, 0.74813E+00_JPRB, 0.75884E+00_JPRB ,&
&  0.78317E+00_JPRB, 0.80543E+00_JPRB, 0.82327E+00_JPRB, 0.83393E+00_JPRB, 0.84519E+00_JPRB, 0.85489E+00_JPRB, 0.84857E+00_JPRB ,&
&  0.85198E+00_JPRB, 0.84666E+00_JPRB, 0.84087E+00_JPRB, 0.82578E+00_JPRB, 0.80337E+00_JPRB, 0.79181E+00_JPRB, 0.78547E+00_JPRB ,&
&  0.77836E+00_JPRB, 0.76872E+00_JPRB, 0.74435E+00_JPRB, 0.73290E+00_JPRB, 0.76677E+00_JPRB, 0.75365E+00_JPRB, 0.79602E+00_JPRB ,&
&  0.86206E+00_JPRB, 0.91169E+00_JPRB, 0.86663E+00_JPRB, 0.67260E+00_JPRB, 0.65014E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.66400E+00_JPRB, 0.65788E+00_JPRB, 0.65050E+00_JPRB, 0.60926E+00_JPRB, 0.57303E+00_JPRB, 0.55812E+00_JPRB, 0.57390E+00_JPRB ,&
&  0.60773E+00_JPRB, 0.63068E+00_JPRB, 0.62847E+00_JPRB, 0.61649E+00_JPRB, 0.61227E+00_JPRB, 0.61697E+00_JPRB, 0.62537E+00_JPRB ,&
&  0.62527E+00_JPRB, 0.61532E+00_JPRB, 0.60707E+00_JPRB, 0.61313E+00_JPRB, 0.61584E+00_JPRB, 0.62539E+00_JPRB, 0.64587E+00_JPRB ,&
&  0.67381E+00_JPRB, 0.68731E+00_JPRB, 0.68835E+00_JPRB, 0.69320E+00_JPRB, 0.71761E+00_JPRB, 0.71767E+00_JPRB, 0.72532E+00_JPRB ,&
&  0.82030E+00_JPRB, 0.13453E+01_JPRB, 0.15350E+01_JPRB, 0.18620E+01_JPRB, 0.16792E+01_JPRB, 0.13831E+01_JPRB, 0.12744E+01_JPRB ,&
&  0.12090E+01_JPRB, 0.10610E+01_JPRB, 0.10293E+01_JPRB, 0.94438E+00_JPRB, 0.99669E+00_JPRB, 0.10444E+01_JPRB, 0.10697E+01_JPRB ,&
&  0.11453E+01_JPRB, 0.12927E+01_JPRB, 0.13812E+01_JPRB, 0.14644E+01_JPRB, 0.15425E+01_JPRB, 0.16366E+01_JPRB, 0.17797E+01_JPRB ,&
&  0.19505E+01_JPRB, 0.21735E+01_JPRB, 0.24366E+01_JPRB, 0.22806E+01_JPRB, 0.20073E+01_JPRB, 0.22958E+01_JPRB, 0.23830E+01_JPRB ,&
&  0.22787E+01_JPRB, 0.18084E+01_JPRB, 0.14384E+01_JPRB, 0.11641E+01_JPRB, 0.97629E+00_JPRB, 0.99608E+00_JPRB, 0.89652E+00_JPRB ,&
&  0.82449E+00_JPRB, 0.81086E+00_JPRB, 0.79937E+00_JPRB, 0.80645E+00_JPRB, 0.80783E+00_JPRB, 0.80025E+00_JPRB, 0.80575E+00_JPRB ,&
&  0.82492E+00_JPRB, 0.84070E+00_JPRB, 0.85397E+00_JPRB, 0.86355E+00_JPRB, 0.87732E+00_JPRB, 0.88549E+00_JPRB, 0.87577E+00_JPRB ,&
&  0.85687E+00_JPRB, 0.84693E+00_JPRB, 0.80389E+00_JPRB, 0.80228E+00_JPRB, 0.80123E+00_JPRB, 0.80196E+00_JPRB, 0.77794E+00_JPRB ,&
&  0.77465E+00_JPRB, 0.78183E+00_JPRB, 0.76353E+00_JPRB, 0.75031E+00_JPRB, 0.76056E+00_JPRB, 0.75387E+00_JPRB, 0.72242E+00_JPRB ,&
&  0.72408E+00_JPRB, 0.71408E+00_JPRB, 0.70765E+00_JPRB, 0.68696E+00_JPRB, 0.67346E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.69747E+00_JPRB, 0.69180E+00_JPRB, 0.69112E+00_JPRB, 0.67226E+00_JPRB, 0.64854E+00_JPRB, 0.63135E+00_JPRB, 0.62482E+00_JPRB ,&
&  0.62737E+00_JPRB, 0.63874E+00_JPRB, 0.64786E+00_JPRB, 0.63655E+00_JPRB, 0.63102E+00_JPRB, 0.63966E+00_JPRB, 0.64214E+00_JPRB ,&
&  0.64135E+00_JPRB, 0.63743E+00_JPRB, 0.63495E+00_JPRB, 0.63448E+00_JPRB, 0.63139E+00_JPRB, 0.62080E+00_JPRB, 0.62059E+00_JPRB ,&
&  0.63335E+00_JPRB, 0.64545E+00_JPRB, 0.65030E+00_JPRB, 0.65090E+00_JPRB, 0.66446E+00_JPRB, 0.66683E+00_JPRB, 0.67343E+00_JPRB ,&
&  0.71355E+00_JPRB, 0.12160E+01_JPRB, 0.12049E+01_JPRB, 0.21007E+01_JPRB, 0.17916E+01_JPRB, 0.15055E+01_JPRB, 0.12779E+01_JPRB ,&
&  0.12154E+01_JPRB, 0.12937E+01_JPRB, 0.11118E+01_JPRB, 0.97368E+00_JPRB, 0.92259E+00_JPRB, 0.93662E+00_JPRB, 0.93212E+00_JPRB ,&
&  0.92066E+00_JPRB, 0.95389E+00_JPRB, 0.10137E+01_JPRB, 0.11016E+01_JPRB, 0.12019E+01_JPRB, 0.12865E+01_JPRB, 0.13915E+01_JPRB ,&
&  0.15205E+01_JPRB, 0.17428E+01_JPRB, 0.20944E+01_JPRB, 0.21610E+01_JPRB, 0.21996E+01_JPRB, 0.25531E+01_JPRB, 0.29318E+01_JPRB ,&
&  0.29101E+01_JPRB, 0.23977E+01_JPRB, 0.17497E+01_JPRB, 0.13369E+01_JPRB, 0.11392E+01_JPRB, 0.10821E+01_JPRB, 0.97939E+00_JPRB ,&
&  0.89807E+00_JPRB, 0.86866E+00_JPRB, 0.85735E+00_JPRB, 0.84656E+00_JPRB, 0.84282E+00_JPRB, 0.83954E+00_JPRB, 0.83774E+00_JPRB ,&
&  0.84650E+00_JPRB, 0.87124E+00_JPRB, 0.87523E+00_JPRB, 0.87560E+00_JPRB, 0.88065E+00_JPRB, 0.88311E+00_JPRB, 0.87348E+00_JPRB ,&
&  0.86819E+00_JPRB, 0.85977E+00_JPRB, 0.80712E+00_JPRB, 0.78270E+00_JPRB, 0.78644E+00_JPRB, 0.78393E+00_JPRB, 0.78100E+00_JPRB ,&
&  0.80291E+00_JPRB, 0.80525E+00_JPRB, 0.78184E+00_JPRB, 0.74680E+00_JPRB, 0.75729E+00_JPRB, 0.77551E+00_JPRB, 0.72758E+00_JPRB ,&
&  0.71836E+00_JPRB, 0.71268E+00_JPRB, 0.71214E+00_JPRB, 0.70964E+00_JPRB, 0.70353E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.70241E+00_JPRB, 0.69659E+00_JPRB, 0.69730E+00_JPRB, 0.68957E+00_JPRB, 0.67202E+00_JPRB, 0.65578E+00_JPRB, 0.65183E+00_JPRB ,&
&  0.64817E+00_JPRB, 0.65101E+00_JPRB, 0.65841E+00_JPRB, 0.65066E+00_JPRB, 0.65164E+00_JPRB, 0.65668E+00_JPRB, 0.65704E+00_JPRB ,&
&  0.64935E+00_JPRB, 0.64524E+00_JPRB, 0.64018E+00_JPRB, 0.63639E+00_JPRB, 0.63497E+00_JPRB, 0.62552E+00_JPRB, 0.62055E+00_JPRB ,&
&  0.61977E+00_JPRB, 0.62479E+00_JPRB, 0.62979E+00_JPRB, 0.63132E+00_JPRB, 0.64002E+00_JPRB, 0.64212E+00_JPRB, 0.64365E+00_JPRB ,&
&  0.68389E+00_JPRB, 0.12248E+01_JPRB, 0.10973E+01_JPRB, 0.18751E+01_JPRB, 0.16698E+01_JPRB, 0.13969E+01_JPRB, 0.11699E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.11949E+01_JPRB, 0.11110E+01_JPRB, 0.10299E+01_JPRB, 0.96600E+00_JPRB, 0.91496E+00_JPRB, 0.88053E+00_JPRB ,&
&  0.85134E+00_JPRB, 0.83677E+00_JPRB, 0.84184E+00_JPRB, 0.86546E+00_JPRB, 0.92588E+00_JPRB, 0.10011E+01_JPRB, 0.10701E+01_JPRB ,&
&  0.11471E+01_JPRB, 0.12772E+01_JPRB, 0.15247E+01_JPRB, 0.19257E+01_JPRB, 0.20237E+01_JPRB, 0.23207E+01_JPRB, 0.33974E+01_JPRB ,&
&  0.35310E+01_JPRB, 0.31043E+01_JPRB, 0.21749E+01_JPRB, 0.15257E+01_JPRB, 0.12722E+01_JPRB, 0.11687E+01_JPRB, 0.10716E+01_JPRB ,&
&  0.98226E+00_JPRB, 0.92696E+00_JPRB, 0.91926E+00_JPRB, 0.88669E+00_JPRB, 0.85890E+00_JPRB, 0.86440E+00_JPRB, 0.88288E+00_JPRB ,&
&  0.89577E+00_JPRB, 0.88616E+00_JPRB, 0.89508E+00_JPRB, 0.88333E+00_JPRB, 0.87038E+00_JPRB, 0.85407E+00_JPRB, 0.84798E+00_JPRB ,&
&  0.84922E+00_JPRB, 0.85639E+00_JPRB, 0.82671E+00_JPRB, 0.79479E+00_JPRB, 0.79074E+00_JPRB, 0.79229E+00_JPRB, 0.78577E+00_JPRB ,&
&  0.82011E+00_JPRB, 0.82031E+00_JPRB, 0.79611E+00_JPRB, 0.77770E+00_JPRB, 0.75804E+00_JPRB, 0.77776E+00_JPRB, 0.72342E+00_JPRB ,&
&  0.70662E+00_JPRB, 0.70142E+00_JPRB, 0.70356E+00_JPRB, 0.71148E+00_JPRB, 0.71406E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.71479E+00_JPRB, 0.70476E+00_JPRB, 0.70348E+00_JPRB, 0.70053E+00_JPRB, 0.69141E+00_JPRB, 0.68013E+00_JPRB, 0.68230E+00_JPRB ,&
&  0.68537E+00_JPRB, 0.68673E+00_JPRB, 0.67768E+00_JPRB, 0.66192E+00_JPRB, 0.65981E+00_JPRB, 0.65682E+00_JPRB, 0.65244E+00_JPRB ,&
&  0.64797E+00_JPRB, 0.64701E+00_JPRB, 0.63855E+00_JPRB, 0.63390E+00_JPRB, 0.63629E+00_JPRB, 0.63396E+00_JPRB, 0.62584E+00_JPRB ,&
&  0.61705E+00_JPRB, 0.61181E+00_JPRB, 0.61739E+00_JPRB, 0.62315E+00_JPRB, 0.63599E+00_JPRB, 0.63807E+00_JPRB, 0.63837E+00_JPRB ,&
&  0.68948E+00_JPRB, 0.62928E+00_JPRB, 0.12647E+01_JPRB, 0.16095E+01_JPRB, 0.15700E+01_JPRB, 0.13599E+01_JPRB, 0.12093E+01_JPRB ,&
&  0.11856E+01_JPRB, 0.11242E+01_JPRB, 0.10811E+01_JPRB, 0.10118E+01_JPRB, 0.94234E+00_JPRB, 0.87100E+00_JPRB, 0.84354E+00_JPRB ,&
&  0.82987E+00_JPRB, 0.81384E+00_JPRB, 0.80995E+00_JPRB, 0.81017E+00_JPRB, 0.81995E+00_JPRB, 0.83922E+00_JPRB, 0.87890E+00_JPRB ,&
&  0.92005E+00_JPRB, 0.97109E+00_JPRB, 0.10804E+01_JPRB, 0.13876E+01_JPRB, 0.18469E+01_JPRB, 0.22240E+01_JPRB, 0.30295E+01_JPRB ,&
&  0.38035E+01_JPRB, 0.35843E+01_JPRB, 0.25664E+01_JPRB, 0.17606E+01_JPRB, 0.14018E+01_JPRB, 0.12623E+01_JPRB, 0.11560E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.98822E+00_JPRB, 0.94808E+00_JPRB, 0.91488E+00_JPRB, 0.88370E+00_JPRB, 0.86399E+00_JPRB, 0.88855E+00_JPRB ,&
&  0.91683E+00_JPRB, 0.92116E+00_JPRB, 0.89959E+00_JPRB, 0.87908E+00_JPRB, 0.86228E+00_JPRB, 0.84443E+00_JPRB, 0.83941E+00_JPRB ,&
&  0.84023E+00_JPRB, 0.85035E+00_JPRB, 0.83537E+00_JPRB, 0.81630E+00_JPRB, 0.81602E+00_JPRB, 0.82909E+00_JPRB, 0.83360E+00_JPRB ,&
&  0.82746E+00_JPRB, 0.81933E+00_JPRB, 0.79550E+00_JPRB, 0.78887E+00_JPRB, 0.74765E+00_JPRB, 0.77308E+00_JPRB, 0.72366E+00_JPRB ,&
&  0.70289E+00_JPRB, 0.70008E+00_JPRB, 0.70683E+00_JPRB, 0.71264E+00_JPRB, 0.73112E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.72626E+00_JPRB, 0.70726E+00_JPRB, 0.70210E+00_JPRB, 0.70714E+00_JPRB, 0.70541E+00_JPRB, 0.70980E+00_JPRB, 0.71467E+00_JPRB ,&
&  0.71185E+00_JPRB, 0.69678E+00_JPRB, 0.68287E+00_JPRB, 0.66723E+00_JPRB, 0.66196E+00_JPRB, 0.65463E+00_JPRB, 0.65321E+00_JPRB ,&
&  0.64663E+00_JPRB, 0.64313E+00_JPRB, 0.63688E+00_JPRB, 0.63549E+00_JPRB, 0.63599E+00_JPRB, 0.63150E+00_JPRB, 0.61254E+00_JPRB ,&
&  0.60818E+00_JPRB, 0.60753E+00_JPRB, 0.61559E+00_JPRB, 0.62689E+00_JPRB, 0.64050E+00_JPRB, 0.64727E+00_JPRB, 0.64920E+00_JPRB ,&
&  0.69063E+00_JPRB, 0.78625E+00_JPRB, 0.12061E+01_JPRB, 0.13120E+01_JPRB, 0.13308E+01_JPRB, 0.12334E+01_JPRB, 0.11541E+01_JPRB ,&
&  0.10916E+01_JPRB, 0.10635E+01_JPRB, 0.10581E+01_JPRB, 0.98660E+00_JPRB, 0.88658E+00_JPRB, 0.81017E+00_JPRB, 0.78667E+00_JPRB ,&
&  0.77603E+00_JPRB, 0.76718E+00_JPRB, 0.76407E+00_JPRB, 0.76612E+00_JPRB, 0.76386E+00_JPRB, 0.76473E+00_JPRB, 0.77821E+00_JPRB ,&
&  0.79549E+00_JPRB, 0.82073E+00_JPRB, 0.86411E+00_JPRB, 0.10232E+01_JPRB, 0.15821E+01_JPRB, 0.17696E+01_JPRB, 0.26674E+01_JPRB ,&
&  0.38855E+01_JPRB, 0.34880E+01_JPRB, 0.27070E+01_JPRB, 0.19993E+01_JPRB, 0.15993E+01_JPRB, 0.14095E+01_JPRB, 0.12674E+01_JPRB ,&
&  0.11566E+01_JPRB, 0.10632E+01_JPRB, 0.98883E+00_JPRB, 0.95287E+00_JPRB, 0.91916E+00_JPRB, 0.88875E+00_JPRB, 0.88287E+00_JPRB ,&
&  0.90084E+00_JPRB, 0.91435E+00_JPRB, 0.90313E+00_JPRB, 0.87727E+00_JPRB, 0.85779E+00_JPRB, 0.85537E+00_JPRB, 0.85303E+00_JPRB ,&
&  0.85307E+00_JPRB, 0.85382E+00_JPRB, 0.84494E+00_JPRB, 0.81868E+00_JPRB, 0.82503E+00_JPRB, 0.84952E+00_JPRB, 0.85100E+00_JPRB ,&
&  0.81985E+00_JPRB, 0.79095E+00_JPRB, 0.80125E+00_JPRB, 0.78098E+00_JPRB, 0.76534E+00_JPRB, 0.78728E+00_JPRB, 0.72871E+00_JPRB ,&
&  0.69780E+00_JPRB, 0.69273E+00_JPRB, 0.71010E+00_JPRB, 0.73020E+00_JPRB, 0.76337E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.75469E+00_JPRB, 0.71763E+00_JPRB, 0.69873E+00_JPRB, 0.69933E+00_JPRB, 0.70765E+00_JPRB, 0.70734E+00_JPRB, 0.71227E+00_JPRB ,&
&  0.70655E+00_JPRB, 0.69329E+00_JPRB, 0.67810E+00_JPRB, 0.66396E+00_JPRB, 0.65702E+00_JPRB, 0.65211E+00_JPRB, 0.64861E+00_JPRB ,&
&  0.64623E+00_JPRB, 0.64395E+00_JPRB, 0.63885E+00_JPRB, 0.64255E+00_JPRB, 0.64490E+00_JPRB, 0.64364E+00_JPRB, 0.62256E+00_JPRB ,&
&  0.61264E+00_JPRB, 0.61453E+00_JPRB, 0.61994E+00_JPRB, 0.62490E+00_JPRB, 0.63308E+00_JPRB, 0.64558E+00_JPRB, 0.65689E+00_JPRB ,&
&  0.67532E+00_JPRB, 0.76951E+00_JPRB, 0.10006E+01_JPRB, 0.11628E+01_JPRB, 0.12120E+01_JPRB, 0.11857E+01_JPRB, 0.11167E+01_JPRB ,&
&  0.10596E+01_JPRB, 0.10234E+01_JPRB, 0.98745E+00_JPRB, 0.90067E+00_JPRB, 0.81409E+00_JPRB, 0.76896E+00_JPRB, 0.74798E+00_JPRB ,&
&  0.74016E+00_JPRB, 0.73978E+00_JPRB, 0.73768E+00_JPRB, 0.73553E+00_JPRB, 0.73275E+00_JPRB, 0.73278E+00_JPRB, 0.73733E+00_JPRB ,&
&  0.74509E+00_JPRB, 0.75079E+00_JPRB, 0.76890E+00_JPRB, 0.79993E+00_JPRB, 0.10944E+01_JPRB, 0.14766E+01_JPRB, 0.20784E+01_JPRB ,&
&  0.23567E+01_JPRB, 0.24153E+01_JPRB, 0.22109E+01_JPRB, 0.19355E+01_JPRB, 0.16935E+01_JPRB, 0.15038E+01_JPRB, 0.13429E+01_JPRB ,&
&  0.12034E+01_JPRB, 0.10808E+01_JPRB, 0.98209E+00_JPRB, 0.92459E+00_JPRB, 0.89406E+00_JPRB, 0.86461E+00_JPRB, 0.85026E+00_JPRB ,&
&  0.85458E+00_JPRB, 0.86436E+00_JPRB, 0.86759E+00_JPRB, 0.86317E+00_JPRB, 0.85036E+00_JPRB, 0.83980E+00_JPRB, 0.83440E+00_JPRB ,&
&  0.83823E+00_JPRB, 0.83686E+00_JPRB, 0.84681E+00_JPRB, 0.83916E+00_JPRB, 0.83053E+00_JPRB, 0.82248E+00_JPRB, 0.82048E+00_JPRB ,&
&  0.82559E+00_JPRB, 0.80409E+00_JPRB, 0.79219E+00_JPRB, 0.78642E+00_JPRB, 0.76777E+00_JPRB, 0.78831E+00_JPRB, 0.73372E+00_JPRB ,&
&  0.70412E+00_JPRB, 0.69738E+00_JPRB, 0.70222E+00_JPRB, 0.71800E+00_JPRB, 0.75941E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.78145E+00_JPRB, 0.75261E+00_JPRB, 0.72375E+00_JPRB, 0.70624E+00_JPRB, 0.70021E+00_JPRB, 0.69925E+00_JPRB, 0.69953E+00_JPRB ,&
&  0.69719E+00_JPRB, 0.69235E+00_JPRB, 0.67593E+00_JPRB, 0.66375E+00_JPRB, 0.65547E+00_JPRB, 0.65331E+00_JPRB, 0.65139E+00_JPRB ,&
&  0.65194E+00_JPRB, 0.65129E+00_JPRB, 0.64699E+00_JPRB, 0.64617E+00_JPRB, 0.64787E+00_JPRB, 0.64315E+00_JPRB, 0.62711E+00_JPRB ,&
&  0.61085E+00_JPRB, 0.61274E+00_JPRB, 0.61821E+00_JPRB, 0.62181E+00_JPRB, 0.62889E+00_JPRB, 0.63890E+00_JPRB, 0.65509E+00_JPRB ,&
&  0.67660E+00_JPRB, 0.68805E+00_JPRB, 0.88260E+00_JPRB, 0.10098E+01_JPRB, 0.10700E+01_JPRB, 0.10968E+01_JPRB, 0.10569E+01_JPRB ,&
&  0.10043E+01_JPRB, 0.93351E+00_JPRB, 0.87546E+00_JPRB, 0.81424E+00_JPRB, 0.77441E+00_JPRB, 0.74685E+00_JPRB, 0.72921E+00_JPRB ,&
&  0.72104E+00_JPRB, 0.71841E+00_JPRB, 0.71616E+00_JPRB, 0.71268E+00_JPRB, 0.71094E+00_JPRB, 0.71073E+00_JPRB, 0.71197E+00_JPRB ,&
&  0.71529E+00_JPRB, 0.72244E+00_JPRB, 0.73074E+00_JPRB, 0.74338E+00_JPRB, 0.78575E+00_JPRB, 0.94116E+00_JPRB, 0.12681E+01_JPRB ,&
&  0.15640E+01_JPRB, 0.17359E+01_JPRB, 0.17061E+01_JPRB, 0.16448E+01_JPRB, 0.15891E+01_JPRB, 0.14718E+01_JPRB, 0.13480E+01_JPRB ,&
&  0.12248E+01_JPRB, 0.11090E+01_JPRB, 0.99099E+00_JPRB, 0.89879E+00_JPRB, 0.84187E+00_JPRB, 0.81099E+00_JPRB, 0.79555E+00_JPRB ,&
&  0.79681E+00_JPRB, 0.80993E+00_JPRB, 0.81683E+00_JPRB, 0.82587E+00_JPRB, 0.82389E+00_JPRB, 0.81500E+00_JPRB, 0.80513E+00_JPRB ,&
&  0.79324E+00_JPRB, 0.78055E+00_JPRB, 0.78045E+00_JPRB, 0.77718E+00_JPRB, 0.77406E+00_JPRB, 0.77595E+00_JPRB, 0.77746E+00_JPRB ,&
&  0.78045E+00_JPRB, 0.79290E+00_JPRB, 0.76304E+00_JPRB, 0.75844E+00_JPRB, 0.81922E+00_JPRB, 0.78541E+00_JPRB, 0.74779E+00_JPRB ,&
&  0.71826E+00_JPRB, 0.70713E+00_JPRB, 0.70678E+00_JPRB, 0.72477E+00_JPRB, 0.77045E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.69849E+00_JPRB, 0.69822E+00_JPRB, 0.69531E+00_JPRB, 0.69564E+00_JPRB, 0.69461E+00_JPRB, 0.68627E+00_JPRB, 0.68007E+00_JPRB ,&
&  0.67954E+00_JPRB, 0.67700E+00_JPRB, 0.67411E+00_JPRB, 0.66752E+00_JPRB, 0.65767E+00_JPRB, 0.65073E+00_JPRB, 0.65047E+00_JPRB ,&
&  0.64776E+00_JPRB, 0.64689E+00_JPRB, 0.64662E+00_JPRB, 0.64254E+00_JPRB, 0.63692E+00_JPRB, 0.62816E+00_JPRB, 0.61402E+00_JPRB ,&
&  0.59665E+00_JPRB, 0.59151E+00_JPRB, 0.60049E+00_JPRB, 0.60801E+00_JPRB, 0.61300E+00_JPRB, 0.61572E+00_JPRB, 0.62660E+00_JPRB ,&
&  0.65988E+00_JPRB, 0.64295E+00_JPRB, 0.72215E+00_JPRB, 0.84398E+00_JPRB, 0.91449E+00_JPRB, 0.90874E+00_JPRB, 0.90532E+00_JPRB ,&
&  0.88129E+00_JPRB, 0.85202E+00_JPRB, 0.81798E+00_JPRB, 0.77735E+00_JPRB, 0.74225E+00_JPRB, 0.71988E+00_JPRB, 0.71147E+00_JPRB ,&
&  0.70691E+00_JPRB, 0.70262E+00_JPRB, 0.70218E+00_JPRB, 0.69607E+00_JPRB, 0.69051E+00_JPRB, 0.68611E+00_JPRB, 0.68800E+00_JPRB ,&
&  0.69103E+00_JPRB, 0.69489E+00_JPRB, 0.69808E+00_JPRB, 0.70360E+00_JPRB, 0.71746E+00_JPRB, 0.76519E+00_JPRB, 0.91381E+00_JPRB ,&
&  0.11088E+01_JPRB, 0.12923E+01_JPRB, 0.13763E+01_JPRB, 0.13548E+01_JPRB, 0.13266E+01_JPRB, 0.12887E+01_JPRB, 0.11987E+01_JPRB ,&
&  0.10824E+01_JPRB, 0.10050E+01_JPRB, 0.94575E+00_JPRB, 0.89437E+00_JPRB, 0.85727E+00_JPRB, 0.82880E+00_JPRB, 0.80721E+00_JPRB ,&
&  0.79438E+00_JPRB, 0.79048E+00_JPRB, 0.78841E+00_JPRB, 0.78739E+00_JPRB, 0.78320E+00_JPRB, 0.76853E+00_JPRB, 0.74628E+00_JPRB ,&
&  0.72039E+00_JPRB, 0.71032E+00_JPRB, 0.71364E+00_JPRB, 0.71114E+00_JPRB, 0.70947E+00_JPRB, 0.71721E+00_JPRB, 0.73038E+00_JPRB ,&
&  0.73655E+00_JPRB, 0.74791E+00_JPRB, 0.75818E+00_JPRB, 0.78937E+00_JPRB, 0.78653E+00_JPRB, 0.76736E+00_JPRB, 0.75541E+00_JPRB ,&
&  0.73384E+00_JPRB, 0.71366E+00_JPRB, 0.70005E+00_JPRB, 0.70371E+00_JPRB, 0.66132E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.68650E+00_JPRB, 0.68479E+00_JPRB, 0.67969E+00_JPRB, 0.67918E+00_JPRB, 0.68085E+00_JPRB, 0.67036E+00_JPRB, 0.66120E+00_JPRB ,&
&  0.65675E+00_JPRB, 0.65874E+00_JPRB, 0.66314E+00_JPRB, 0.66524E+00_JPRB, 0.66063E+00_JPRB, 0.65109E+00_JPRB, 0.64565E+00_JPRB ,&
&  0.63718E+00_JPRB, 0.62951E+00_JPRB, 0.62789E+00_JPRB, 0.62746E+00_JPRB, 0.62200E+00_JPRB, 0.61445E+00_JPRB, 0.60810E+00_JPRB ,&
&  0.59491E+00_JPRB, 0.58643E+00_JPRB, 0.58484E+00_JPRB, 0.58822E+00_JPRB, 0.59197E+00_JPRB, 0.59093E+00_JPRB, 0.60776E+00_JPRB ,&
&  0.65020E+00_JPRB, 0.58336E+00_JPRB, 0.58932E+00_JPRB, 0.69049E+00_JPRB, 0.75466E+00_JPRB, 0.77027E+00_JPRB, 0.78869E+00_JPRB ,&
&  0.80885E+00_JPRB, 0.80175E+00_JPRB, 0.78364E+00_JPRB, 0.74475E+00_JPRB, 0.71543E+00_JPRB, 0.69661E+00_JPRB, 0.68118E+00_JPRB ,&
&  0.67090E+00_JPRB, 0.66282E+00_JPRB, 0.66055E+00_JPRB, 0.65413E+00_JPRB, 0.65177E+00_JPRB, 0.64349E+00_JPRB, 0.64111E+00_JPRB ,&
&  0.64229E+00_JPRB, 0.64827E+00_JPRB, 0.65486E+00_JPRB, 0.65991E+00_JPRB, 0.67199E+00_JPRB, 0.69667E+00_JPRB, 0.75853E+00_JPRB ,&
&  0.86568E+00_JPRB, 0.10132E+01_JPRB, 0.11157E+01_JPRB, 0.11477E+01_JPRB, 0.11321E+01_JPRB, 0.10903E+01_JPRB, 0.10261E+01_JPRB ,&
&  0.95438E+00_JPRB, 0.89390E+00_JPRB, 0.85096E+00_JPRB, 0.82783E+00_JPRB, 0.81148E+00_JPRB, 0.79831E+00_JPRB, 0.78520E+00_JPRB ,&
&  0.77379E+00_JPRB, 0.76703E+00_JPRB, 0.76257E+00_JPRB, 0.75609E+00_JPRB, 0.74691E+00_JPRB, 0.72560E+00_JPRB, 0.69468E+00_JPRB ,&
&  0.67080E+00_JPRB, 0.65712E+00_JPRB, 0.65603E+00_JPRB, 0.65540E+00_JPRB, 0.65823E+00_JPRB, 0.67260E+00_JPRB, 0.69039E+00_JPRB ,&
&  0.70020E+00_JPRB, 0.71208E+00_JPRB, 0.73508E+00_JPRB, 0.71773E+00_JPRB, 0.75252E+00_JPRB, 0.75261E+00_JPRB, 0.75343E+00_JPRB ,&
&  0.74279E+00_JPRB, 0.72708E+00_JPRB, 0.71633E+00_JPRB, 0.61006E+00_JPRB, 0.66836E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.67544E+00_JPRB, 0.67288E+00_JPRB, 0.66186E+00_JPRB, 0.65509E+00_JPRB, 0.65056E+00_JPRB, 0.64552E+00_JPRB, 0.63682E+00_JPRB ,&
&  0.63189E+00_JPRB, 0.63173E+00_JPRB, 0.63155E+00_JPRB, 0.63271E+00_JPRB, 0.63394E+00_JPRB, 0.62955E+00_JPRB, 0.62207E+00_JPRB ,&
&  0.61419E+00_JPRB, 0.60667E+00_JPRB, 0.60817E+00_JPRB, 0.60756E+00_JPRB, 0.60249E+00_JPRB, 0.59833E+00_JPRB, 0.59282E+00_JPRB ,&
&  0.57863E+00_JPRB, 0.56454E+00_JPRB, 0.55348E+00_JPRB, 0.55646E+00_JPRB, 0.56312E+00_JPRB, 0.55846E+00_JPRB, 0.57292E+00_JPRB ,&
&  0.61985E+00_JPRB, 0.54285E+00_JPRB, 0.56923E+00_JPRB, 0.62570E+00_JPRB, 0.65176E+00_JPRB, 0.67785E+00_JPRB, 0.69638E+00_JPRB ,&
&  0.70865E+00_JPRB, 0.71566E+00_JPRB, 0.68887E+00_JPRB, 0.65778E+00_JPRB, 0.63167E+00_JPRB, 0.61120E+00_JPRB, 0.59689E+00_JPRB ,&
&  0.59144E+00_JPRB, 0.59486E+00_JPRB, 0.59797E+00_JPRB, 0.59556E+00_JPRB, 0.59327E+00_JPRB, 0.58900E+00_JPRB, 0.58640E+00_JPRB ,&
&  0.58558E+00_JPRB, 0.58786E+00_JPRB, 0.59245E+00_JPRB, 0.60293E+00_JPRB, 0.61414E+00_JPRB, 0.62772E+00_JPRB, 0.65115E+00_JPRB ,&
&  0.69496E+00_JPRB, 0.77083E+00_JPRB, 0.85496E+00_JPRB, 0.90168E+00_JPRB, 0.90451E+00_JPRB, 0.88032E+00_JPRB, 0.83940E+00_JPRB ,&
&  0.79674E+00_JPRB, 0.76305E+00_JPRB, 0.74241E+00_JPRB, 0.72989E+00_JPRB, 0.71725E+00_JPRB, 0.70607E+00_JPRB, 0.69704E+00_JPRB ,&
&  0.68946E+00_JPRB, 0.68020E+00_JPRB, 0.67629E+00_JPRB, 0.67050E+00_JPRB, 0.66065E+00_JPRB, 0.64431E+00_JPRB, 0.62776E+00_JPRB ,&
&  0.61243E+00_JPRB, 0.60060E+00_JPRB, 0.59527E+00_JPRB, 0.59749E+00_JPRB, 0.60364E+00_JPRB, 0.61686E+00_JPRB, 0.63518E+00_JPRB ,&
&  0.64914E+00_JPRB, 0.66035E+00_JPRB, 0.67301E+00_JPRB, 0.69940E+00_JPRB, 0.71700E+00_JPRB, 0.73193E+00_JPRB, 0.73906E+00_JPRB ,&
&  0.73839E+00_JPRB, 0.73895E+00_JPRB, 0.58636E+00_JPRB, 0.64960E+00_JPRB, 0.66680E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.67577E+00_JPRB, 0.65970E+00_JPRB, 0.64018E+00_JPRB, 0.62694E+00_JPRB, 0.62059E+00_JPRB, 0.61873E+00_JPRB, 0.61606E+00_JPRB ,&
&  0.61143E+00_JPRB, 0.61184E+00_JPRB, 0.61293E+00_JPRB, 0.61393E+00_JPRB, 0.61463E+00_JPRB, 0.61188E+00_JPRB, 0.60461E+00_JPRB ,&
&  0.59614E+00_JPRB, 0.58502E+00_JPRB, 0.58343E+00_JPRB, 0.58288E+00_JPRB, 0.57665E+00_JPRB, 0.57231E+00_JPRB, 0.57184E+00_JPRB ,&
&  0.56544E+00_JPRB, 0.55557E+00_JPRB, 0.53776E+00_JPRB, 0.53211E+00_JPRB, 0.53489E+00_JPRB, 0.53509E+00_JPRB, 0.54156E+00_JPRB ,&
&  0.52033E+00_JPRB, 0.49381E+00_JPRB, 0.54410E+00_JPRB, 0.57524E+00_JPRB, 0.58739E+00_JPRB, 0.59837E+00_JPRB, 0.61063E+00_JPRB ,&
&  0.60612E+00_JPRB, 0.60777E+00_JPRB, 0.59369E+00_JPRB, 0.57054E+00_JPRB, 0.55493E+00_JPRB, 0.54906E+00_JPRB, 0.54853E+00_JPRB ,&
&  0.55475E+00_JPRB, 0.55962E+00_JPRB, 0.56134E+00_JPRB, 0.56177E+00_JPRB, 0.56334E+00_JPRB, 0.56068E+00_JPRB, 0.55889E+00_JPRB ,&
&  0.56058E+00_JPRB, 0.56413E+00_JPRB, 0.56609E+00_JPRB, 0.57028E+00_JPRB, 0.57896E+00_JPRB, 0.59195E+00_JPRB, 0.60605E+00_JPRB ,&
&  0.62166E+00_JPRB, 0.64507E+00_JPRB, 0.68503E+00_JPRB, 0.71912E+00_JPRB, 0.73077E+00_JPRB, 0.71809E+00_JPRB, 0.69536E+00_JPRB ,&
&  0.67505E+00_JPRB, 0.66180E+00_JPRB, 0.65261E+00_JPRB, 0.64475E+00_JPRB, 0.63319E+00_JPRB, 0.62216E+00_JPRB, 0.61096E+00_JPRB ,&
&  0.60393E+00_JPRB, 0.59467E+00_JPRB, 0.58712E+00_JPRB, 0.57966E+00_JPRB, 0.57466E+00_JPRB, 0.57275E+00_JPRB, 0.57257E+00_JPRB ,&
&  0.57131E+00_JPRB, 0.56572E+00_JPRB, 0.56255E+00_JPRB, 0.56520E+00_JPRB, 0.56915E+00_JPRB, 0.57704E+00_JPRB, 0.58844E+00_JPRB ,&
&  0.60237E+00_JPRB, 0.61544E+00_JPRB, 0.62949E+00_JPRB, 0.65667E+00_JPRB, 0.68401E+00_JPRB, 0.70713E+00_JPRB, 0.72032E+00_JPRB ,&
&  0.72776E+00_JPRB, 0.72032E+00_JPRB, 0.71699E+00_JPRB, 0.70444E+00_JPRB, 0.68866E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.62324E+00_JPRB, 0.60333E+00_JPRB, 0.58765E+00_JPRB, 0.57748E+00_JPRB, 0.57205E+00_JPRB, 0.57162E+00_JPRB, 0.56950E+00_JPRB ,&
&  0.56821E+00_JPRB, 0.56541E+00_JPRB, 0.56363E+00_JPRB, 0.56628E+00_JPRB, 0.56776E+00_JPRB, 0.56855E+00_JPRB, 0.57007E+00_JPRB ,&
&  0.56490E+00_JPRB, 0.55255E+00_JPRB, 0.54267E+00_JPRB, 0.54010E+00_JPRB, 0.54125E+00_JPRB, 0.53741E+00_JPRB, 0.53607E+00_JPRB ,&
&  0.53911E+00_JPRB, 0.53125E+00_JPRB, 0.51808E+00_JPRB, 0.50959E+00_JPRB, 0.51009E+00_JPRB, 0.50769E+00_JPRB, 0.50480E+00_JPRB ,&
&  0.52340E+00_JPRB, 0.47397E+00_JPRB, 0.51975E+00_JPRB, 0.52653E+00_JPRB, 0.53146E+00_JPRB, 0.53384E+00_JPRB, 0.53281E+00_JPRB ,&
&  0.52849E+00_JPRB, 0.52294E+00_JPRB, 0.51714E+00_JPRB, 0.51188E+00_JPRB, 0.50879E+00_JPRB, 0.50477E+00_JPRB, 0.50132E+00_JPRB ,&
&  0.50285E+00_JPRB, 0.50517E+00_JPRB, 0.51079E+00_JPRB, 0.51711E+00_JPRB, 0.52505E+00_JPRB, 0.52798E+00_JPRB, 0.53057E+00_JPRB ,&
&  0.53344E+00_JPRB, 0.53516E+00_JPRB, 0.53518E+00_JPRB, 0.53416E+00_JPRB, 0.53279E+00_JPRB, 0.53742E+00_JPRB, 0.54377E+00_JPRB ,&
&  0.54976E+00_JPRB, 0.56036E+00_JPRB, 0.57759E+00_JPRB, 0.59265E+00_JPRB, 0.59972E+00_JPRB, 0.59710E+00_JPRB, 0.58812E+00_JPRB ,&
&  0.58296E+00_JPRB, 0.58383E+00_JPRB, 0.58554E+00_JPRB, 0.58139E+00_JPRB, 0.57421E+00_JPRB, 0.56742E+00_JPRB, 0.55736E+00_JPRB ,&
&  0.54422E+00_JPRB, 0.53296E+00_JPRB, 0.52592E+00_JPRB, 0.52235E+00_JPRB, 0.52169E+00_JPRB, 0.52047E+00_JPRB, 0.51917E+00_JPRB ,&
&  0.52003E+00_JPRB, 0.51934E+00_JPRB, 0.52012E+00_JPRB, 0.52426E+00_JPRB, 0.52798E+00_JPRB, 0.53383E+00_JPRB, 0.54146E+00_JPRB ,&
&  0.55195E+00_JPRB, 0.56349E+00_JPRB, 0.57464E+00_JPRB, 0.59301E+00_JPRB, 0.61926E+00_JPRB, 0.63845E+00_JPRB, 0.65059E+00_JPRB ,&
&  0.66169E+00_JPRB, 0.66440E+00_JPRB, 0.65855E+00_JPRB, 0.65031E+00_JPRB, 0.63817E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.58028E+00_JPRB, 0.56749E+00_JPRB, 0.55766E+00_JPRB, 0.55049E+00_JPRB, 0.54618E+00_JPRB, 0.54564E+00_JPRB, 0.54382E+00_JPRB ,&
&  0.54208E+00_JPRB, 0.54076E+00_JPRB, 0.53494E+00_JPRB, 0.52887E+00_JPRB, 0.52871E+00_JPRB, 0.53451E+00_JPRB, 0.53918E+00_JPRB ,&
&  0.53661E+00_JPRB, 0.52654E+00_JPRB, 0.51706E+00_JPRB, 0.51522E+00_JPRB, 0.51828E+00_JPRB, 0.52082E+00_JPRB, 0.52056E+00_JPRB ,&
&  0.52227E+00_JPRB, 0.52329E+00_JPRB, 0.51511E+00_JPRB, 0.49856E+00_JPRB, 0.49083E+00_JPRB, 0.48634E+00_JPRB, 0.49058E+00_JPRB ,&
&  0.49671E+00_JPRB, 0.48908E+00_JPRB, 0.50125E+00_JPRB, 0.49904E+00_JPRB, 0.49792E+00_JPRB, 0.49392E+00_JPRB, 0.48711E+00_JPRB ,&
&  0.48179E+00_JPRB, 0.47645E+00_JPRB, 0.47320E+00_JPRB, 0.47181E+00_JPRB, 0.47233E+00_JPRB, 0.47378E+00_JPRB, 0.47851E+00_JPRB ,&
&  0.47989E+00_JPRB, 0.47839E+00_JPRB, 0.48113E+00_JPRB, 0.48827E+00_JPRB, 0.49711E+00_JPRB, 0.50220E+00_JPRB, 0.50682E+00_JPRB ,&
&  0.51060E+00_JPRB, 0.51273E+00_JPRB, 0.51563E+00_JPRB, 0.51651E+00_JPRB, 0.51295E+00_JPRB, 0.51220E+00_JPRB, 0.51207E+00_JPRB ,&
&  0.51431E+00_JPRB, 0.51887E+00_JPRB, 0.52728E+00_JPRB, 0.53749E+00_JPRB, 0.54196E+00_JPRB, 0.53846E+00_JPRB, 0.53403E+00_JPRB ,&
&  0.53472E+00_JPRB, 0.54003E+00_JPRB, 0.54597E+00_JPRB, 0.54961E+00_JPRB, 0.54938E+00_JPRB, 0.54228E+00_JPRB, 0.53328E+00_JPRB ,&
&  0.52358E+00_JPRB, 0.51350E+00_JPRB, 0.50439E+00_JPRB, 0.49858E+00_JPRB, 0.49400E+00_JPRB, 0.49069E+00_JPRB, 0.48856E+00_JPRB ,&
&  0.48769E+00_JPRB, 0.48758E+00_JPRB, 0.49047E+00_JPRB, 0.49640E+00_JPRB, 0.49950E+00_JPRB, 0.50746E+00_JPRB, 0.51769E+00_JPRB ,&
&  0.52815E+00_JPRB, 0.53790E+00_JPRB, 0.54943E+00_JPRB, 0.56612E+00_JPRB, 0.58507E+00_JPRB, 0.59916E+00_JPRB, 0.61025E+00_JPRB ,&
&  0.61749E+00_JPRB, 0.61690E+00_JPRB, 0.60960E+00_JPRB, 0.60130E+00_JPRB, 0.59220E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.53734E+00_JPRB, 0.52960E+00_JPRB, 0.52363E+00_JPRB, 0.51876E+00_JPRB, 0.51484E+00_JPRB, 0.51367E+00_JPRB, 0.51221E+00_JPRB ,&
&  0.51056E+00_JPRB, 0.50830E+00_JPRB, 0.50723E+00_JPRB, 0.50435E+00_JPRB, 0.50177E+00_JPRB, 0.50479E+00_JPRB, 0.51037E+00_JPRB ,&
&  0.50912E+00_JPRB, 0.50267E+00_JPRB, 0.49759E+00_JPRB, 0.49768E+00_JPRB, 0.50120E+00_JPRB, 0.50384E+00_JPRB, 0.50689E+00_JPRB ,&
&  0.50676E+00_JPRB, 0.50613E+00_JPRB, 0.50473E+00_JPRB, 0.48951E+00_JPRB, 0.47477E+00_JPRB, 0.46756E+00_JPRB, 0.46335E+00_JPRB ,&
&  0.46236E+00_JPRB, 0.46584E+00_JPRB, 0.45036E+00_JPRB, 0.45959E+00_JPRB, 0.45557E+00_JPRB, 0.44950E+00_JPRB, 0.44622E+00_JPRB ,&
&  0.44422E+00_JPRB, 0.44187E+00_JPRB, 0.44028E+00_JPRB, 0.44239E+00_JPRB, 0.44518E+00_JPRB, 0.44619E+00_JPRB, 0.44815E+00_JPRB ,&
&  0.44805E+00_JPRB, 0.45069E+00_JPRB, 0.45295E+00_JPRB, 0.45813E+00_JPRB, 0.46214E+00_JPRB, 0.46470E+00_JPRB, 0.46793E+00_JPRB ,&
&  0.47239E+00_JPRB, 0.47654E+00_JPRB, 0.48049E+00_JPRB, 0.48381E+00_JPRB, 0.48511E+00_JPRB, 0.48484E+00_JPRB, 0.48072E+00_JPRB ,&
&  0.47740E+00_JPRB, 0.47720E+00_JPRB, 0.48066E+00_JPRB, 0.48854E+00_JPRB, 0.49572E+00_JPRB, 0.49732E+00_JPRB, 0.49719E+00_JPRB ,&
&  0.49860E+00_JPRB, 0.50294E+00_JPRB, 0.50448E+00_JPRB, 0.50390E+00_JPRB, 0.50236E+00_JPRB, 0.49998E+00_JPRB, 0.49556E+00_JPRB ,&
&  0.48952E+00_JPRB, 0.48451E+00_JPRB, 0.47934E+00_JPRB, 0.46817E+00_JPRB, 0.45954E+00_JPRB, 0.45853E+00_JPRB, 0.45993E+00_JPRB ,&
&  0.45931E+00_JPRB, 0.45765E+00_JPRB, 0.45867E+00_JPRB, 0.46415E+00_JPRB, 0.46392E+00_JPRB, 0.47022E+00_JPRB, 0.48364E+00_JPRB ,&
&  0.49753E+00_JPRB, 0.50978E+00_JPRB, 0.51941E+00_JPRB, 0.52942E+00_JPRB, 0.53936E+00_JPRB, 0.54701E+00_JPRB, 0.55378E+00_JPRB ,&
&  0.55724E+00_JPRB, 0.55598E+00_JPRB, 0.55093E+00_JPRB, 0.54776E+00_JPRB, 0.54484E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.51261E+00_JPRB, 0.51094E+00_JPRB, 0.50923E+00_JPRB, 0.50716E+00_JPRB, 0.50430E+00_JPRB, 0.50179E+00_JPRB, 0.49746E+00_JPRB ,&
&  0.49279E+00_JPRB, 0.48902E+00_JPRB, 0.48749E+00_JPRB, 0.48715E+00_JPRB, 0.48872E+00_JPRB, 0.49292E+00_JPRB, 0.49975E+00_JPRB ,&
&  0.49996E+00_JPRB, 0.49346E+00_JPRB, 0.49054E+00_JPRB, 0.49073E+00_JPRB, 0.49323E+00_JPRB, 0.49502E+00_JPRB, 0.49782E+00_JPRB ,&
&  0.49966E+00_JPRB, 0.50027E+00_JPRB, 0.49958E+00_JPRB, 0.49278E+00_JPRB, 0.47635E+00_JPRB, 0.46639E+00_JPRB, 0.45754E+00_JPRB ,&
&  0.45220E+00_JPRB, 0.44742E+00_JPRB, 0.44467E+00_JPRB, 0.44359E+00_JPRB, 0.44255E+00_JPRB, 0.44072E+00_JPRB, 0.44060E+00_JPRB ,&
&  0.43867E+00_JPRB, 0.43611E+00_JPRB, 0.43468E+00_JPRB, 0.43522E+00_JPRB, 0.43693E+00_JPRB, 0.43857E+00_JPRB, 0.43857E+00_JPRB ,&
&  0.43968E+00_JPRB, 0.44352E+00_JPRB, 0.44473E+00_JPRB, 0.44509E+00_JPRB, 0.44506E+00_JPRB, 0.44471E+00_JPRB, 0.44490E+00_JPRB ,&
&  0.44727E+00_JPRB, 0.45233E+00_JPRB, 0.45659E+00_JPRB, 0.46082E+00_JPRB, 0.46259E+00_JPRB, 0.46342E+00_JPRB, 0.46340E+00_JPRB ,&
&  0.46036E+00_JPRB, 0.45644E+00_JPRB, 0.45510E+00_JPRB, 0.45787E+00_JPRB, 0.46351E+00_JPRB, 0.46965E+00_JPRB, 0.47315E+00_JPRB ,&
&  0.47593E+00_JPRB, 0.47721E+00_JPRB, 0.47537E+00_JPRB, 0.47450E+00_JPRB, 0.47208E+00_JPRB, 0.46772E+00_JPRB, 0.46524E+00_JPRB ,&
&  0.46280E+00_JPRB, 0.46062E+00_JPRB, 0.45866E+00_JPRB, 0.44980E+00_JPRB, 0.44216E+00_JPRB, 0.43745E+00_JPRB, 0.43433E+00_JPRB ,&
&  0.43309E+00_JPRB, 0.43210E+00_JPRB, 0.43174E+00_JPRB, 0.43298E+00_JPRB, 0.43497E+00_JPRB, 0.44133E+00_JPRB, 0.45414E+00_JPRB ,&
&  0.47327E+00_JPRB, 0.49109E+00_JPRB, 0.50085E+00_JPRB, 0.50784E+00_JPRB, 0.51232E+00_JPRB, 0.51462E+00_JPRB, 0.51694E+00_JPRB ,&
&  0.52049E+00_JPRB, 0.52167E+00_JPRB, 0.51945E+00_JPRB, 0.51664E+00_JPRB, 0.51494E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.48587E+00_JPRB, 0.48607E+00_JPRB, 0.48470E+00_JPRB, 0.48345E+00_JPRB, 0.48209E+00_JPRB, 0.48087E+00_JPRB, 0.47892E+00_JPRB ,&
&  0.47411E+00_JPRB, 0.46790E+00_JPRB, 0.46433E+00_JPRB, 0.46275E+00_JPRB, 0.46845E+00_JPRB, 0.47688E+00_JPRB, 0.48439E+00_JPRB ,&
&  0.48367E+00_JPRB, 0.47479E+00_JPRB, 0.47213E+00_JPRB, 0.47417E+00_JPRB, 0.47678E+00_JPRB, 0.47961E+00_JPRB, 0.48344E+00_JPRB ,&
&  0.48761E+00_JPRB, 0.49188E+00_JPRB, 0.49376E+00_JPRB, 0.49130E+00_JPRB, 0.48494E+00_JPRB, 0.47143E+00_JPRB, 0.45957E+00_JPRB ,&
&  0.45165E+00_JPRB, 0.44547E+00_JPRB, 0.44200E+00_JPRB, 0.44104E+00_JPRB, 0.43884E+00_JPRB, 0.43504E+00_JPRB, 0.42967E+00_JPRB ,&
&  0.42498E+00_JPRB, 0.42277E+00_JPRB, 0.42489E+00_JPRB, 0.42655E+00_JPRB, 0.42742E+00_JPRB, 0.42722E+00_JPRB, 0.42442E+00_JPRB ,&
&  0.42297E+00_JPRB, 0.42268E+00_JPRB, 0.42261E+00_JPRB, 0.42443E+00_JPRB, 0.42237E+00_JPRB, 0.42063E+00_JPRB, 0.42285E+00_JPRB ,&
&  0.42745E+00_JPRB, 0.43141E+00_JPRB, 0.43496E+00_JPRB, 0.43764E+00_JPRB, 0.43869E+00_JPRB, 0.44076E+00_JPRB, 0.44211E+00_JPRB ,&
&  0.44128E+00_JPRB, 0.43781E+00_JPRB, 0.43459E+00_JPRB, 0.43518E+00_JPRB, 0.43758E+00_JPRB, 0.44030E+00_JPRB, 0.44390E+00_JPRB ,&
&  0.44466E+00_JPRB, 0.43971E+00_JPRB, 0.43471E+00_JPRB, 0.43029E+00_JPRB, 0.42622E+00_JPRB, 0.42306E+00_JPRB, 0.42149E+00_JPRB ,&
&  0.42103E+00_JPRB, 0.41800E+00_JPRB, 0.41209E+00_JPRB, 0.40719E+00_JPRB, 0.40650E+00_JPRB, 0.40556E+00_JPRB, 0.40160E+00_JPRB ,&
&  0.40185E+00_JPRB, 0.40427E+00_JPRB, 0.40313E+00_JPRB, 0.40125E+00_JPRB, 0.40326E+00_JPRB, 0.40864E+00_JPRB, 0.41962E+00_JPRB ,&
&  0.43428E+00_JPRB, 0.45113E+00_JPRB, 0.46667E+00_JPRB, 0.48013E+00_JPRB, 0.48808E+00_JPRB, 0.48997E+00_JPRB, 0.48819E+00_JPRB ,&
&  0.48743E+00_JPRB, 0.48806E+00_JPRB, 0.48701E+00_JPRB, 0.48523E+00_JPRB, 0.48511E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.45890E+00_JPRB, 0.45879E+00_JPRB, 0.45902E+00_JPRB, 0.46016E+00_JPRB, 0.46091E+00_JPRB, 0.45992E+00_JPRB, 0.45966E+00_JPRB ,&
&  0.45654E+00_JPRB, 0.45199E+00_JPRB, 0.44917E+00_JPRB, 0.45063E+00_JPRB, 0.45968E+00_JPRB, 0.47133E+00_JPRB, 0.47842E+00_JPRB ,&
&  0.47420E+00_JPRB, 0.46200E+00_JPRB, 0.45907E+00_JPRB, 0.46139E+00_JPRB, 0.46361E+00_JPRB, 0.46777E+00_JPRB, 0.47389E+00_JPRB ,&
&  0.47948E+00_JPRB, 0.48331E+00_JPRB, 0.48752E+00_JPRB, 0.48925E+00_JPRB, 0.48955E+00_JPRB, 0.48255E+00_JPRB, 0.47145E+00_JPRB ,&
&  0.46074E+00_JPRB, 0.45303E+00_JPRB, 0.44838E+00_JPRB, 0.44488E+00_JPRB, 0.43984E+00_JPRB, 0.43470E+00_JPRB, 0.42803E+00_JPRB ,&
&  0.42473E+00_JPRB, 0.42439E+00_JPRB, 0.42524E+00_JPRB, 0.42337E+00_JPRB, 0.42087E+00_JPRB, 0.41587E+00_JPRB, 0.41178E+00_JPRB ,&
&  0.41032E+00_JPRB, 0.41175E+00_JPRB, 0.41326E+00_JPRB, 0.41263E+00_JPRB, 0.40948E+00_JPRB, 0.40732E+00_JPRB, 0.40885E+00_JPRB ,&
&  0.41479E+00_JPRB, 0.41466E+00_JPRB, 0.41368E+00_JPRB, 0.41441E+00_JPRB, 0.41652E+00_JPRB, 0.41845E+00_JPRB, 0.41791E+00_JPRB ,&
&  0.41731E+00_JPRB, 0.41736E+00_JPRB, 0.41775E+00_JPRB, 0.41974E+00_JPRB, 0.41992E+00_JPRB, 0.41787E+00_JPRB, 0.41242E+00_JPRB ,&
&  0.40410E+00_JPRB, 0.39359E+00_JPRB, 0.38925E+00_JPRB, 0.38617E+00_JPRB, 0.38319E+00_JPRB, 0.38253E+00_JPRB, 0.38388E+00_JPRB ,&
&  0.38248E+00_JPRB, 0.37631E+00_JPRB, 0.37348E+00_JPRB, 0.37392E+00_JPRB, 0.37280E+00_JPRB, 0.37483E+00_JPRB, 0.37845E+00_JPRB ,&
&  0.38844E+00_JPRB, 0.39398E+00_JPRB, 0.39561E+00_JPRB, 0.39353E+00_JPRB, 0.39389E+00_JPRB, 0.39567E+00_JPRB, 0.39998E+00_JPRB ,&
&  0.40647E+00_JPRB, 0.41344E+00_JPRB, 0.42060E+00_JPRB, 0.42850E+00_JPRB, 0.43846E+00_JPRB, 0.44814E+00_JPRB, 0.45181E+00_JPRB ,&
&  0.45196E+00_JPRB, 0.45221E+00_JPRB, 0.45338E+00_JPRB, 0.45483E+00_JPRB, 0.45771E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.42543E+00_JPRB, 0.42771E+00_JPRB, 0.42996E+00_JPRB, 0.43147E+00_JPRB, 0.43411E+00_JPRB, 0.43890E+00_JPRB, 0.44082E+00_JPRB ,&
&  0.43895E+00_JPRB, 0.43685E+00_JPRB, 0.43673E+00_JPRB, 0.44011E+00_JPRB, 0.44668E+00_JPRB, 0.45359E+00_JPRB, 0.46130E+00_JPRB ,&
&  0.45735E+00_JPRB, 0.44506E+00_JPRB, 0.44058E+00_JPRB, 0.44259E+00_JPRB, 0.44742E+00_JPRB, 0.45281E+00_JPRB, 0.45949E+00_JPRB ,&
&  0.46584E+00_JPRB, 0.47004E+00_JPRB, 0.47444E+00_JPRB, 0.47839E+00_JPRB, 0.48096E+00_JPRB, 0.47929E+00_JPRB, 0.47489E+00_JPRB ,&
&  0.46962E+00_JPRB, 0.46436E+00_JPRB, 0.45978E+00_JPRB, 0.44406E+00_JPRB, 0.44146E+00_JPRB, 0.43246E+00_JPRB, 0.42494E+00_JPRB ,&
&  0.41971E+00_JPRB, 0.41540E+00_JPRB, 0.41269E+00_JPRB, 0.40971E+00_JPRB, 0.40554E+00_JPRB, 0.39849E+00_JPRB, 0.39663E+00_JPRB ,&
&  0.39591E+00_JPRB, 0.39719E+00_JPRB, 0.39281E+00_JPRB, 0.39058E+00_JPRB, 0.38812E+00_JPRB, 0.38625E+00_JPRB, 0.39065E+00_JPRB ,&
&  0.39589E+00_JPRB, 0.39347E+00_JPRB, 0.38958E+00_JPRB, 0.38925E+00_JPRB, 0.39348E+00_JPRB, 0.39511E+00_JPRB, 0.39610E+00_JPRB ,&
&  0.39793E+00_JPRB, 0.39855E+00_JPRB, 0.39407E+00_JPRB, 0.38905E+00_JPRB, 0.38043E+00_JPRB, 0.36976E+00_JPRB, 0.36697E+00_JPRB ,&
&  0.36404E+00_JPRB, 0.35823E+00_JPRB, 0.35464E+00_JPRB, 0.35351E+00_JPRB, 0.35310E+00_JPRB, 0.35386E+00_JPRB, 0.35389E+00_JPRB ,&
&  0.35377E+00_JPRB, 0.35023E+00_JPRB, 0.34502E+00_JPRB, 0.34047E+00_JPRB, 0.33427E+00_JPRB, 0.33679E+00_JPRB, 0.34643E+00_JPRB ,&
&  0.35838E+00_JPRB, 0.36561E+00_JPRB, 0.37034E+00_JPRB, 0.37379E+00_JPRB, 0.37691E+00_JPRB, 0.37839E+00_JPRB, 0.38079E+00_JPRB ,&
&  0.38396E+00_JPRB, 0.38542E+00_JPRB, 0.38403E+00_JPRB, 0.38387E+00_JPRB, 0.38674E+00_JPRB, 0.39170E+00_JPRB, 0.39779E+00_JPRB ,&
&  0.40385E+00_JPRB, 0.40896E+00_JPRB, 0.41324E+00_JPRB, 0.41863E+00_JPRB, 0.42213E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.39843E+00_JPRB, 0.40273E+00_JPRB, 0.40675E+00_JPRB, 0.41116E+00_JPRB, 0.41744E+00_JPRB, 0.42062E+00_JPRB, 0.42160E+00_JPRB ,&
&  0.42289E+00_JPRB, 0.42344E+00_JPRB, 0.42499E+00_JPRB, 0.42933E+00_JPRB, 0.43535E+00_JPRB, 0.43963E+00_JPRB, 0.44531E+00_JPRB ,&
&  0.44539E+00_JPRB, 0.43682E+00_JPRB, 0.42898E+00_JPRB, 0.42868E+00_JPRB, 0.43415E+00_JPRB, 0.43986E+00_JPRB, 0.44709E+00_JPRB ,&
&  0.45427E+00_JPRB, 0.45953E+00_JPRB, 0.46254E+00_JPRB, 0.46406E+00_JPRB, 0.46605E+00_JPRB, 0.46572E+00_JPRB, 0.46270E+00_JPRB ,&
&  0.45859E+00_JPRB, 0.44978E+00_JPRB, 0.43032E+00_JPRB, 0.42752E+00_JPRB, 0.41879E+00_JPRB, 0.41117E+00_JPRB, 0.40588E+00_JPRB ,&
&  0.40159E+00_JPRB, 0.39812E+00_JPRB, 0.39421E+00_JPRB, 0.39090E+00_JPRB, 0.38711E+00_JPRB, 0.38029E+00_JPRB, 0.37689E+00_JPRB ,&
&  0.37383E+00_JPRB, 0.37127E+00_JPRB, 0.36881E+00_JPRB, 0.36918E+00_JPRB, 0.36975E+00_JPRB, 0.37271E+00_JPRB, 0.37421E+00_JPRB ,&
&  0.37119E+00_JPRB, 0.36759E+00_JPRB, 0.36483E+00_JPRB, 0.36648E+00_JPRB, 0.36871E+00_JPRB, 0.37025E+00_JPRB, 0.37083E+00_JPRB ,&
&  0.36843E+00_JPRB, 0.36396E+00_JPRB, 0.35934E+00_JPRB, 0.35398E+00_JPRB, 0.34885E+00_JPRB, 0.32316E+00_JPRB, 0.31186E+00_JPRB ,&
&  0.30844E+00_JPRB, 0.31735E+00_JPRB, 0.32985E+00_JPRB, 0.32857E+00_JPRB, 0.33053E+00_JPRB, 0.33058E+00_JPRB, 0.32806E+00_JPRB ,&
&  0.31927E+00_JPRB, 0.30569E+00_JPRB, 0.29646E+00_JPRB, 0.29232E+00_JPRB, 0.28935E+00_JPRB, 0.29059E+00_JPRB, 0.29655E+00_JPRB ,&
&  0.30650E+00_JPRB, 0.31814E+00_JPRB, 0.32698E+00_JPRB, 0.33104E+00_JPRB, 0.33001E+00_JPRB, 0.32691E+00_JPRB, 0.32279E+00_JPRB ,&
&  0.31823E+00_JPRB, 0.32380E+00_JPRB, 0.33786E+00_JPRB, 0.35787E+00_JPRB, 0.36050E+00_JPRB, 0.36542E+00_JPRB, 0.36874E+00_JPRB ,&
&  0.37388E+00_JPRB, 0.38032E+00_JPRB, 0.38593E+00_JPRB, 0.39189E+00_JPRB, 0.39555E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.37723E+00_JPRB, 0.38217E+00_JPRB, 0.38889E+00_JPRB, 0.39638E+00_JPRB, 0.40173E+00_JPRB, 0.40414E+00_JPRB, 0.40669E+00_JPRB ,&
&  0.40997E+00_JPRB, 0.41135E+00_JPRB, 0.41240E+00_JPRB, 0.41518E+00_JPRB, 0.42081E+00_JPRB, 0.42382E+00_JPRB, 0.42857E+00_JPRB ,&
&  0.43043E+00_JPRB, 0.42829E+00_JPRB, 0.42153E+00_JPRB, 0.41953E+00_JPRB, 0.42305E+00_JPRB, 0.42779E+00_JPRB, 0.43305E+00_JPRB ,&
&  0.43854E+00_JPRB, 0.44359E+00_JPRB, 0.44600E+00_JPRB, 0.44672E+00_JPRB, 0.44694E+00_JPRB, 0.44683E+00_JPRB, 0.44419E+00_JPRB ,&
&  0.43922E+00_JPRB, 0.39995E+00_JPRB, 0.37338E+00_JPRB, 0.37631E+00_JPRB, 0.38797E+00_JPRB, 0.38253E+00_JPRB, 0.38099E+00_JPRB ,&
&  0.37940E+00_JPRB, 0.37593E+00_JPRB, 0.37075E+00_JPRB, 0.36655E+00_JPRB, 0.36132E+00_JPRB, 0.35459E+00_JPRB, 0.34916E+00_JPRB ,&
&  0.34795E+00_JPRB, 0.35273E+00_JPRB, 0.35432E+00_JPRB, 0.35665E+00_JPRB, 0.35559E+00_JPRB, 0.35769E+00_JPRB, 0.35662E+00_JPRB ,&
&  0.35136E+00_JPRB, 0.34535E+00_JPRB, 0.34343E+00_JPRB, 0.34311E+00_JPRB, 0.34419E+00_JPRB, 0.34684E+00_JPRB, 0.34682E+00_JPRB ,&
&  0.33366E+00_JPRB, 0.32217E+00_JPRB, 0.30372E+00_JPRB, 0.28584E+00_JPRB, 0.27276E+00_JPRB, 0.26678E+00_JPRB, 0.26244E+00_JPRB ,&
&  0.26153E+00_JPRB, 0.26720E+00_JPRB, 0.28355E+00_JPRB, 0.30355E+00_JPRB, 0.31782E+00_JPRB, 0.29962E+00_JPRB, 0.28387E+00_JPRB ,&
&  0.27422E+00_JPRB, 0.26913E+00_JPRB, 0.26481E+00_JPRB, 0.26324E+00_JPRB, 0.26360E+00_JPRB, 0.26583E+00_JPRB, 0.26907E+00_JPRB ,&
&  0.27367E+00_JPRB, 0.27800E+00_JPRB, 0.28080E+00_JPRB, 0.28214E+00_JPRB, 0.28287E+00_JPRB, 0.28361E+00_JPRB, 0.28299E+00_JPRB ,&
&  0.28276E+00_JPRB, 0.28599E+00_JPRB, 0.29268E+00_JPRB, 0.30139E+00_JPRB, 0.30954E+00_JPRB, 0.31856E+00_JPRB, 0.33129E+00_JPRB ,&
&  0.34544E+00_JPRB, 0.35731E+00_JPRB, 0.36150E+00_JPRB, 0.36742E+00_JPRB, 0.37259E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.36888E+00_JPRB, 0.37549E+00_JPRB, 0.38048E+00_JPRB, 0.39038E+00_JPRB, 0.39318E+00_JPRB, 0.39598E+00_JPRB, 0.39829E+00_JPRB ,&
&  0.40440E+00_JPRB, 0.40799E+00_JPRB, 0.40578E+00_JPRB, 0.40519E+00_JPRB, 0.40434E+00_JPRB, 0.40322E+00_JPRB, 0.40420E+00_JPRB ,&
&  0.40427E+00_JPRB, 0.40635E+00_JPRB, 0.40403E+00_JPRB, 0.40126E+00_JPRB, 0.40317E+00_JPRB, 0.40796E+00_JPRB, 0.41118E+00_JPRB ,&
&  0.40608E+00_JPRB, 0.40476E+00_JPRB, 0.41331E+00_JPRB, 0.41548E+00_JPRB, 0.41676E+00_JPRB, 0.41878E+00_JPRB, 0.41856E+00_JPRB ,&
&  0.40484E+00_JPRB, 0.37964E+00_JPRB, 0.35208E+00_JPRB, 0.35338E+00_JPRB, 0.36798E+00_JPRB, 0.36265E+00_JPRB, 0.36103E+00_JPRB ,&
&  0.35813E+00_JPRB, 0.35364E+00_JPRB, 0.34934E+00_JPRB, 0.34735E+00_JPRB, 0.34458E+00_JPRB, 0.34000E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.34577E+00_JPRB, 0.34126E+00_JPRB, 0.31966E+00_JPRB, 0.30318E+00_JPRB, 0.29422E+00_JPRB, 0.28279E+00_JPRB ,&
&  0.26957E+00_JPRB, 0.26173E+00_JPRB, 0.26045E+00_JPRB, 0.25875E+00_JPRB, 0.26037E+00_JPRB, 0.26148E+00_JPRB, 0.26261E+00_JPRB ,&
&  0.26353E+00_JPRB, 0.26338E+00_JPRB, 0.25884E+00_JPRB, 0.25295E+00_JPRB, 0.24973E+00_JPRB, 0.24971E+00_JPRB, 0.25232E+00_JPRB ,&
&  0.25805E+00_JPRB, 0.26423E+00_JPRB, 0.27842E+00_JPRB, 0.28616E+00_JPRB, 0.27768E+00_JPRB, 0.27205E+00_JPRB, 0.26565E+00_JPRB ,&
&  0.25541E+00_JPRB, 0.25146E+00_JPRB, 0.24817E+00_JPRB, 0.24633E+00_JPRB, 0.24716E+00_JPRB, 0.25042E+00_JPRB, 0.25220E+00_JPRB ,&
&  0.25472E+00_JPRB, 0.25735E+00_JPRB, 0.25842E+00_JPRB, 0.25929E+00_JPRB, 0.26001E+00_JPRB, 0.26186E+00_JPRB, 0.26374E+00_JPRB ,&
&  0.26533E+00_JPRB, 0.26819E+00_JPRB, 0.27166E+00_JPRB, 0.27631E+00_JPRB, 0.28041E+00_JPRB, 0.28285E+00_JPRB, 0.28986E+00_JPRB ,&
&  0.29816E+00_JPRB, 0.31530E+00_JPRB, 0.34334E+00_JPRB, 0.36811E+00_JPRB, 0.36429E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.37093E+00_JPRB, 0.37526E+00_JPRB, 0.37601E+00_JPRB, 0.37930E+00_JPRB, 0.38401E+00_JPRB, 0.38952E+00_JPRB, 0.39409E+00_JPRB ,&
&  0.39923E+00_JPRB, 0.39896E+00_JPRB, 0.39385E+00_JPRB, 0.38680E+00_JPRB, 0.38089E+00_JPRB, 0.36412E+00_JPRB, 0.35526E+00_JPRB ,&
&  0.34803E+00_JPRB, 0.34764E+00_JPRB, 0.34819E+00_JPRB, 0.34987E+00_JPRB, 0.35096E+00_JPRB, 0.35599E+00_JPRB, 0.35750E+00_JPRB ,&
&  0.35497E+00_JPRB, 0.34983E+00_JPRB, 0.34462E+00_JPRB, 0.34369E+00_JPRB, 0.34627E+00_JPRB, 0.34822E+00_JPRB, 0.34535E+00_JPRB ,&
&  0.34169E+00_JPRB, 0.34141E+00_JPRB, 0.34201E+00_JPRB, 0.34345E+00_JPRB, 0.35309E+00_JPRB, 0.34836E+00_JPRB, 0.34389E+00_JPRB ,&
&  0.33676E+00_JPRB, 0.33060E+00_JPRB, 0.32632E+00_JPRB, 0.32661E+00_JPRB, 0.32954E+00_JPRB, 0.32927E+00_JPRB, 0.33187E+00_JPRB ,&
&  0.32013E+00_JPRB, 0.31032E+00_JPRB, 0.29535E+00_JPRB, 0.28353E+00_JPRB, 0.27194E+00_JPRB, 0.26401E+00_JPRB, 0.25662E+00_JPRB ,&
&  0.25032E+00_JPRB, 0.24510E+00_JPRB, 0.24016E+00_JPRB, 0.23853E+00_JPRB, 0.23940E+00_JPRB, 0.23950E+00_JPRB, 0.23913E+00_JPRB ,&
&  0.23970E+00_JPRB, 0.24067E+00_JPRB, 0.24143E+00_JPRB, 0.24242E+00_JPRB, 0.24399E+00_JPRB, 0.24658E+00_JPRB, 0.25008E+00_JPRB ,&
&  0.25554E+00_JPRB, 0.25992E+00_JPRB, 0.26230E+00_JPRB, 0.26341E+00_JPRB, 0.26028E+00_JPRB, 0.25499E+00_JPRB, 0.24958E+00_JPRB ,&
&  0.24420E+00_JPRB, 0.24167E+00_JPRB, 0.23942E+00_JPRB, 0.23834E+00_JPRB, 0.23815E+00_JPRB, 0.23905E+00_JPRB, 0.24090E+00_JPRB ,&
&  0.24273E+00_JPRB, 0.24504E+00_JPRB, 0.24656E+00_JPRB, 0.24837E+00_JPRB, 0.25056E+00_JPRB, 0.25246E+00_JPRB, 0.25444E+00_JPRB ,&
&  0.25665E+00_JPRB, 0.25837E+00_JPRB, 0.26133E+00_JPRB, 0.26496E+00_JPRB, 0.27186E+00_JPRB, 0.28007E+00_JPRB, 0.29472E+00_JPRB ,&
&  0.31201E+00_JPRB, 0.34460E+00_JPRB, 0.35526E+00_JPRB, 0.36205E+00_JPRB, 0.36904E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.36729E+00_JPRB, 0.36982E+00_JPRB, 0.37383E+00_JPRB, 0.37929E+00_JPRB, 0.38611E+00_JPRB, 0.39229E+00_JPRB, 0.39373E+00_JPRB ,&
&  0.38439E+00_JPRB, 0.37148E+00_JPRB, 0.35332E+00_JPRB, 0.33890E+00_JPRB, 0.32498E+00_JPRB, 0.31458E+00_JPRB, 0.30611E+00_JPRB ,&
&  0.30074E+00_JPRB, 0.30030E+00_JPRB, 0.30076E+00_JPRB, 0.30201E+00_JPRB, 0.30103E+00_JPRB, 0.30211E+00_JPRB, 0.30248E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30116E+00_JPRB, 0.30106E+00_JPRB, 0.30455E+00_JPRB, 0.31137E+00_JPRB, 0.31769E+00_JPRB, 0.32395E+00_JPRB ,&
&  0.33128E+00_JPRB, 0.33344E+00_JPRB, 0.33766E+00_JPRB, 0.33642E+00_JPRB, 0.33292E+00_JPRB, 0.32840E+00_JPRB, 0.32209E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31354E+00_JPRB, 0.31217E+00_JPRB, 0.31262E+00_JPRB, 0.31275E+00_JPRB, 0.31458E+00_JPRB, 0.30846E+00_JPRB ,&
&  0.30112E+00_JPRB, 0.29063E+00_JPRB, 0.27865E+00_JPRB, 0.26982E+00_JPRB, 0.26147E+00_JPRB, 0.25547E+00_JPRB, 0.25012E+00_JPRB ,&
&  0.24517E+00_JPRB, 0.24184E+00_JPRB, 0.23839E+00_JPRB, 0.23587E+00_JPRB, 0.23508E+00_JPRB, 0.23334E+00_JPRB, 0.23238E+00_JPRB ,&
&  0.23198E+00_JPRB, 0.23241E+00_JPRB, 0.23401E+00_JPRB, 0.23547E+00_JPRB, 0.23746E+00_JPRB, 0.23973E+00_JPRB, 0.24168E+00_JPRB ,&
&  0.24426E+00_JPRB, 0.24633E+00_JPRB, 0.24658E+00_JPRB, 0.24597E+00_JPRB, 0.24387E+00_JPRB, 0.24112E+00_JPRB, 0.23868E+00_JPRB ,&
&  0.23622E+00_JPRB, 0.23449E+00_JPRB, 0.23383E+00_JPRB, 0.23298E+00_JPRB, 0.23342E+00_JPRB, 0.23459E+00_JPRB, 0.23552E+00_JPRB ,&
&  0.23728E+00_JPRB, 0.23873E+00_JPRB, 0.24040E+00_JPRB, 0.24315E+00_JPRB, 0.24574E+00_JPRB, 0.24802E+00_JPRB, 0.25036E+00_JPRB ,&
&  0.25056E+00_JPRB, 0.25228E+00_JPRB, 0.25460E+00_JPRB, 0.25873E+00_JPRB, 0.27066E+00_JPRB, 0.27922E+00_JPRB, 0.29893E+00_JPRB ,&
&  0.31624E+00_JPRB, 0.35178E+00_JPRB, 0.35213E+00_JPRB, 0.35756E+00_JPRB, 0.36370E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.36201E+00_JPRB, 0.36869E+00_JPRB, 0.37552E+00_JPRB, 0.38067E+00_JPRB, 0.38434E+00_JPRB, 0.38743E+00_JPRB, 0.38597E+00_JPRB ,&
&  0.37990E+00_JPRB, 0.36481E+00_JPRB, 0.35344E+00_JPRB, 0.34232E+00_JPRB, 0.33295E+00_JPRB, 0.32405E+00_JPRB, 0.31574E+00_JPRB ,&
&  0.30775E+00_JPRB, 0.30113E+00_JPRB, 0.29565E+00_JPRB, 0.29088E+00_JPRB, 0.28436E+00_JPRB, 0.27863E+00_JPRB, 0.27402E+00_JPRB ,&
&  0.27021E+00_JPRB, 0.27007E+00_JPRB, 0.27235E+00_JPRB, 0.27673E+00_JPRB, 0.28490E+00_JPRB, 0.29320E+00_JPRB, 0.30249E+00_JPRB ,&
&  0.31675E+00_JPRB, 0.32057E+00_JPRB, 0.31952E+00_JPRB, 0.31883E+00_JPRB, 0.31387E+00_JPRB, 0.30774E+00_JPRB, 0.29847E+00_JPRB ,&
&  0.29926E+00_JPRB, 0.29731E+00_JPRB, 0.30449E+00_JPRB, 0.30472E+00_JPRB, 0.30349E+00_JPRB, 0.30349E+00_JPRB, 0.29602E+00_JPRB ,&
&  0.29044E+00_JPRB, 0.28125E+00_JPRB, 0.27415E+00_JPRB, 0.26684E+00_JPRB, 0.25970E+00_JPRB, 0.25544E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24580E+00_JPRB, 0.24269E+00_JPRB, 0.23870E+00_JPRB, 0.23614E+00_JPRB, 0.23394E+00_JPRB, 0.23169E+00_JPRB, 0.23051E+00_JPRB ,&
&  0.22962E+00_JPRB, 0.22929E+00_JPRB, 0.22940E+00_JPRB, 0.22958E+00_JPRB, 0.23005E+00_JPRB, 0.23094E+00_JPRB, 0.23162E+00_JPRB ,&
&  0.23238E+00_JPRB, 0.23264E+00_JPRB, 0.23226E+00_JPRB, 0.23183E+00_JPRB, 0.23071E+00_JPRB, 0.22997E+00_JPRB, 0.22899E+00_JPRB ,&
&  0.22842E+00_JPRB, 0.22875E+00_JPRB, 0.22906E+00_JPRB, 0.22976E+00_JPRB, 0.23049E+00_JPRB, 0.23206E+00_JPRB, 0.23353E+00_JPRB ,&
&  0.23511E+00_JPRB, 0.23704E+00_JPRB, 0.23864E+00_JPRB, 0.24066E+00_JPRB, 0.24308E+00_JPRB, 0.24474E+00_JPRB, 0.24758E+00_JPRB ,&
&  0.25038E+00_JPRB, 0.25386E+00_JPRB, 0.25975E+00_JPRB, 0.26630E+00_JPRB, 0.27512E+00_JPRB, 0.28608E+00_JPRB, 0.29995E+00_JPRB ,&
&  0.31769E+00_JPRB, 0.34268E+00_JPRB, 0.34447E+00_JPRB, 0.35169E+00_JPRB, 0.35676E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.35228E+00_JPRB, 0.35708E+00_JPRB, 0.36118E+00_JPRB, 0.36484E+00_JPRB, 0.36807E+00_JPRB, 0.36891E+00_JPRB, 0.36798E+00_JPRB ,&
&  0.35890E+00_JPRB, 0.35220E+00_JPRB, 0.34402E+00_JPRB, 0.33675E+00_JPRB, 0.33002E+00_JPRB, 0.32206E+00_JPRB, 0.31522E+00_JPRB ,&
&  0.30821E+00_JPRB, 0.30119E+00_JPRB, 0.29453E+00_JPRB, 0.28672E+00_JPRB, 0.27858E+00_JPRB, 0.27078E+00_JPRB, 0.26601E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26314E+00_JPRB, 0.26610E+00_JPRB, 0.26995E+00_JPRB, 0.27493E+00_JPRB, 0.28133E+00_JPRB, 0.28819E+00_JPRB ,&
&  0.29253E+00_JPRB, 0.29489E+00_JPRB, 0.29444E+00_JPRB, 0.29293E+00_JPRB, 0.29089E+00_JPRB, 0.28917E+00_JPRB, 0.28649E+00_JPRB ,&
&  0.28308E+00_JPRB, 0.28070E+00_JPRB, 0.27995E+00_JPRB, 0.28095E+00_JPRB, 0.28070E+00_JPRB, 0.27880E+00_JPRB, 0.27487E+00_JPRB ,&
&  0.27048E+00_JPRB, 0.26605E+00_JPRB, 0.26150E+00_JPRB, 0.25726E+00_JPRB, 0.25377E+00_JPRB, 0.25012E+00_JPRB, 0.24620E+00_JPRB ,&
&  0.24371E+00_JPRB, 0.24108E+00_JPRB, 0.23819E+00_JPRB, 0.23545E+00_JPRB, 0.23317E+00_JPRB, 0.23141E+00_JPRB, 0.22992E+00_JPRB ,&
&  0.22877E+00_JPRB, 0.22788E+00_JPRB, 0.22716E+00_JPRB, 0.22662E+00_JPRB, 0.22610E+00_JPRB, 0.22532E+00_JPRB, 0.22454E+00_JPRB ,&
&  0.22377E+00_JPRB, 0.22347E+00_JPRB, 0.22301E+00_JPRB, 0.22304E+00_JPRB, 0.22330E+00_JPRB, 0.22347E+00_JPRB, 0.22399E+00_JPRB ,&
&  0.22512E+00_JPRB, 0.22630E+00_JPRB, 0.22695E+00_JPRB, 0.22807E+00_JPRB, 0.22915E+00_JPRB, 0.23056E+00_JPRB, 0.23238E+00_JPRB ,&
&  0.23356E+00_JPRB, 0.23486E+00_JPRB, 0.23639E+00_JPRB, 0.23804E+00_JPRB, 0.24074E+00_JPRB, 0.24367E+00_JPRB, 0.24692E+00_JPRB ,&
&  0.25108E+00_JPRB, 0.25514E+00_JPRB, 0.25975E+00_JPRB, 0.26521E+00_JPRB, 0.27486E+00_JPRB, 0.28153E+00_JPRB, 0.29013E+00_JPRB ,&
&  0.29875E+00_JPRB, 0.30376E+00_JPRB, 0.33795E+00_JPRB, 0.34055E+00_JPRB, 0.34591E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.28002E+00_JPRB, 0.28658E+00_JPRB, 0.29098E+00_JPRB, 0.29569E+00_JPRB, 0.29424E+00_JPRB, 0.33265E+00_JPRB, 0.33172E+00_JPRB ,&
&  0.33061E+00_JPRB, 0.31634E+00_JPRB, 0.31401E+00_JPRB, 0.30925E+00_JPRB, 0.30366E+00_JPRB, 0.29754E+00_JPRB, 0.29225E+00_JPRB ,&
&  0.28742E+00_JPRB, 0.28204E+00_JPRB, 0.27765E+00_JPRB, 0.27359E+00_JPRB, 0.27026E+00_JPRB, 0.26731E+00_JPRB, 0.26523E+00_JPRB ,&
&  0.26471E+00_JPRB, 0.26618E+00_JPRB, 0.26930E+00_JPRB, 0.27105E+00_JPRB, 0.27180E+00_JPRB, 0.27242E+00_JPRB, 0.27305E+00_JPRB ,&
&  0.27307E+00_JPRB, 0.27304E+00_JPRB, 0.27190E+00_JPRB, 0.26976E+00_JPRB, 0.26784E+00_JPRB, 0.26632E+00_JPRB, 0.26512E+00_JPRB ,&
&  0.26398E+00_JPRB, 0.26285E+00_JPRB, 0.26164E+00_JPRB, 0.26089E+00_JPRB, 0.25981E+00_JPRB, 0.25832E+00_JPRB, 0.25623E+00_JPRB ,&
&  0.25413E+00_JPRB, 0.25198E+00_JPRB, 0.24943E+00_JPRB, 0.24739E+00_JPRB, 0.24540E+00_JPRB, 0.24339E+00_JPRB, 0.24140E+00_JPRB ,&
&  0.24026E+00_JPRB, 0.23915E+00_JPRB, 0.23777E+00_JPRB, 0.23602E+00_JPRB, 0.23469E+00_JPRB, 0.23328E+00_JPRB, 0.23204E+00_JPRB ,&
&  0.23122E+00_JPRB, 0.23038E+00_JPRB, 0.22951E+00_JPRB, 0.22862E+00_JPRB, 0.22796E+00_JPRB, 0.22721E+00_JPRB, 0.22652E+00_JPRB ,&
&  0.22592E+00_JPRB, 0.22524E+00_JPRB, 0.22507E+00_JPRB, 0.22500E+00_JPRB, 0.22508E+00_JPRB, 0.22514E+00_JPRB, 0.22546E+00_JPRB ,&
&  0.22594E+00_JPRB, 0.22663E+00_JPRB, 0.22747E+00_JPRB, 0.22828E+00_JPRB, 0.22905E+00_JPRB, 0.22986E+00_JPRB, 0.23105E+00_JPRB ,&
&  0.23234E+00_JPRB, 0.23372E+00_JPRB, 0.23515E+00_JPRB, 0.23627E+00_JPRB, 0.23858E+00_JPRB, 0.24076E+00_JPRB, 0.24309E+00_JPRB ,&
&  0.24575E+00_JPRB, 0.24889E+00_JPRB, 0.25226E+00_JPRB, 0.25585E+00_JPRB, 0.26057E+00_JPRB, 0.26357E+00_JPRB, 0.26576E+00_JPRB ,&
&  0.26817E+00_JPRB, 0.27220E+00_JPRB, 0.27379E+00_JPRB, 0.27517E+00_JPRB, 0.27724E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.25319E+00_JPRB, 0.25348E+00_JPRB, 0.25358E+00_JPRB, 0.25358E+00_JPRB, 0.25175E+00_JPRB, 0.25216E+00_JPRB, 0.25225E+00_JPRB ,&
&  0.25251E+00_JPRB, 0.25279E+00_JPRB, 0.25340E+00_JPRB, 0.25412E+00_JPRB, 0.25483E+00_JPRB, 0.25548E+00_JPRB, 0.25605E+00_JPRB ,&
&  0.25640E+00_JPRB, 0.25684E+00_JPRB, 0.25555E+00_JPRB, 0.25479E+00_JPRB, 0.25471E+00_JPRB, 0.25482E+00_JPRB, 0.25482E+00_JPRB ,&
&  0.25477E+00_JPRB, 0.25466E+00_JPRB, 0.25421E+00_JPRB, 0.25389E+00_JPRB, 0.25374E+00_JPRB, 0.25354E+00_JPRB, 0.25324E+00_JPRB ,&
&  0.25284E+00_JPRB, 0.25254E+00_JPRB, 0.25217E+00_JPRB, 0.25175E+00_JPRB, 0.25160E+00_JPRB, 0.25140E+00_JPRB, 0.25080E+00_JPRB ,&
&  0.25008E+00_JPRB, 0.24906E+00_JPRB, 0.24811E+00_JPRB, 0.24726E+00_JPRB, 0.24657E+00_JPRB, 0.24621E+00_JPRB, 0.24584E+00_JPRB ,&
&  0.24535E+00_JPRB, 0.24468E+00_JPRB, 0.24404E+00_JPRB, 0.24327E+00_JPRB, 0.24259E+00_JPRB, 0.24199E+00_JPRB, 0.24128E+00_JPRB ,&
&  0.24049E+00_JPRB, 0.23968E+00_JPRB, 0.23886E+00_JPRB, 0.23795E+00_JPRB, 0.23708E+00_JPRB, 0.23619E+00_JPRB, 0.23522E+00_JPRB ,&
&  0.23427E+00_JPRB, 0.23385E+00_JPRB, 0.23337E+00_JPRB, 0.23296E+00_JPRB, 0.23265E+00_JPRB, 0.23242E+00_JPRB, 0.23218E+00_JPRB ,&
&  0.23200E+00_JPRB, 0.23180E+00_JPRB, 0.23170E+00_JPRB, 0.23162E+00_JPRB, 0.23161E+00_JPRB, 0.23168E+00_JPRB, 0.23185E+00_JPRB ,&
&  0.23206E+00_JPRB, 0.23231E+00_JPRB, 0.23266E+00_JPRB, 0.23323E+00_JPRB, 0.23379E+00_JPRB, 0.23445E+00_JPRB, 0.23509E+00_JPRB ,&
&  0.23580E+00_JPRB, 0.23650E+00_JPRB, 0.23722E+00_JPRB, 0.23789E+00_JPRB, 0.23880E+00_JPRB, 0.23967E+00_JPRB, 0.24062E+00_JPRB ,&
&  0.24157E+00_JPRB, 0.24260E+00_JPRB, 0.24361E+00_JPRB, 0.24469E+00_JPRB, 0.24593E+00_JPRB, 0.24630E+00_JPRB, 0.24668E+00_JPRB ,&
&  0.24716E+00_JPRB, 0.24785E+00_JPRB, 0.24850E+00_JPRB, 0.24933E+00_JPRB, 0.25075E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB, 0.24087E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB, 0.36972E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.36947E+00_JPRB, 0.37034E+00_JPRB, 0.37118E+00_JPRB, 0.37183E+00_JPRB, 0.37228E+00_JPRB, 0.37256E+00_JPRB, 0.37270E+00_JPRB ,&
&  0.37262E+00_JPRB, 0.37271E+00_JPRB, 0.37291E+00_JPRB, 0.37310E+00_JPRB, 0.37325E+00_JPRB, 0.37333E+00_JPRB, 0.37329E+00_JPRB ,&
&  0.37353E+00_JPRB, 0.37435E+00_JPRB, 0.37482E+00_JPRB, 0.37513E+00_JPRB, 0.37541E+00_JPRB, 0.37565E+00_JPRB, 0.37587E+00_JPRB ,&
&  0.37610E+00_JPRB, 0.37636E+00_JPRB, 0.37663E+00_JPRB, 0.37695E+00_JPRB, 0.37744E+00_JPRB, 0.37788E+00_JPRB, 0.37837E+00_JPRB ,&
&  0.37912E+00_JPRB, 0.38027E+00_JPRB, 0.38166E+00_JPRB, 0.38326E+00_JPRB, 0.38523E+00_JPRB, 0.38749E+00_JPRB, 0.38946E+00_JPRB ,&
&  0.39113E+00_JPRB, 0.39230E+00_JPRB, 0.39239E+00_JPRB, 0.39169E+00_JPRB, 0.39021E+00_JPRB, 0.38826E+00_JPRB, 0.38590E+00_JPRB ,&
&  0.38316E+00_JPRB, 0.38072E+00_JPRB, 0.37852E+00_JPRB, 0.37646E+00_JPRB, 0.37449E+00_JPRB, 0.37272E+00_JPRB, 0.37096E+00_JPRB ,&
&  0.36945E+00_JPRB, 0.36799E+00_JPRB, 0.36662E+00_JPRB, 0.36520E+00_JPRB, 0.36377E+00_JPRB, 0.36232E+00_JPRB, 0.36091E+00_JPRB ,&
&  0.35977E+00_JPRB, 0.35877E+00_JPRB, 0.35802E+00_JPRB, 0.35732E+00_JPRB, 0.35684E+00_JPRB, 0.35682E+00_JPRB, 0.35698E+00_JPRB ,&
&  0.35709E+00_JPRB, 0.35732E+00_JPRB, 0.35766E+00_JPRB, 0.35781E+00_JPRB, 0.35793E+00_JPRB, 0.35800E+00_JPRB, 0.35800E+00_JPRB ,&
&  0.35801E+00_JPRB, 0.35821E+00_JPRB, 0.35837E+00_JPRB, 0.35866E+00_JPRB, 0.35893E+00_JPRB, 0.35926E+00_JPRB, 0.35968E+00_JPRB ,&
&  0.35997E+00_JPRB, 0.36043E+00_JPRB, 0.36084E+00_JPRB, 0.36145E+00_JPRB, 0.36200E+00_JPRB, 0.36248E+00_JPRB, 0.36288E+00_JPRB ,&
&  0.36325E+00_JPRB, 0.36349E+00_JPRB, 0.36380E+00_JPRB, 0.36403E+00_JPRB, 0.36441E+00_JPRB, 0.36481E+00_JPRB, 0.36527E+00_JPRB ,&
&  0.36575E+00_JPRB, 0.36638E+00_JPRB, 0.36694E+00_JPRB, 0.36774E+00_JPRB, 0.36858E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.37227E+00_JPRB, 0.37356E+00_JPRB, 0.37434E+00_JPRB, 0.37471E+00_JPRB, 0.37490E+00_JPRB, 0.37459E+00_JPRB, 0.37364E+00_JPRB ,&
&  0.37210E+00_JPRB, 0.37061E+00_JPRB, 0.36936E+00_JPRB, 0.36791E+00_JPRB, 0.36605E+00_JPRB, 0.36526E+00_JPRB, 0.36478E+00_JPRB ,&
&  0.36339E+00_JPRB, 0.36175E+00_JPRB, 0.36020E+00_JPRB, 0.35907E+00_JPRB, 0.35786E+00_JPRB, 0.35655E+00_JPRB, 0.35577E+00_JPRB ,&
&  0.35531E+00_JPRB, 0.35468E+00_JPRB, 0.35370E+00_JPRB, 0.35235E+00_JPRB, 0.35199E+00_JPRB, 0.35157E+00_JPRB, 0.35101E+00_JPRB ,&
&  0.35018E+00_JPRB, 0.34867E+00_JPRB, 0.34746E+00_JPRB, 0.34702E+00_JPRB, 0.34931E+00_JPRB, 0.35252E+00_JPRB, 0.35685E+00_JPRB ,&
&  0.36248E+00_JPRB, 0.36779E+00_JPRB, 0.37377E+00_JPRB, 0.37948E+00_JPRB, 0.38411E+00_JPRB, 0.38938E+00_JPRB, 0.39306E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39400E+00_JPRB, 0.39317E+00_JPRB, 0.39307E+00_JPRB, 0.39317E+00_JPRB, 0.39285E+00_JPRB, 0.39177E+00_JPRB ,&
&  0.39042E+00_JPRB, 0.38900E+00_JPRB, 0.38735E+00_JPRB, 0.38482E+00_JPRB, 0.38174E+00_JPRB, 0.37872E+00_JPRB, 0.37557E+00_JPRB ,&
&  0.37262E+00_JPRB, 0.36921E+00_JPRB, 0.36604E+00_JPRB, 0.36303E+00_JPRB, 0.36035E+00_JPRB, 0.35735E+00_JPRB, 0.35460E+00_JPRB ,&
&  0.35276E+00_JPRB, 0.35239E+00_JPRB, 0.35244E+00_JPRB, 0.35192E+00_JPRB, 0.35178E+00_JPRB, 0.35185E+00_JPRB, 0.35242E+00_JPRB ,&
&  0.35317E+00_JPRB, 0.35466E+00_JPRB, 0.35627E+00_JPRB, 0.35810E+00_JPRB, 0.35969E+00_JPRB, 0.36120E+00_JPRB, 0.36247E+00_JPRB ,&
&  0.36361E+00_JPRB, 0.36458E+00_JPRB, 0.36544E+00_JPRB, 0.36628E+00_JPRB, 0.36695E+00_JPRB, 0.36750E+00_JPRB, 0.36775E+00_JPRB ,&
&  0.36788E+00_JPRB, 0.36764E+00_JPRB, 0.36730E+00_JPRB, 0.36674E+00_JPRB, 0.36645E+00_JPRB, 0.36567E+00_JPRB, 0.36554E+00_JPRB ,&
&  0.36564E+00_JPRB, 0.36643E+00_JPRB, 0.36740E+00_JPRB, 0.36896E+00_JPRB, 0.37064E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.37246E+00_JPRB, 0.37495E+00_JPRB, 0.37663E+00_JPRB, 0.37749E+00_JPRB, 0.37783E+00_JPRB, 0.37810E+00_JPRB, 0.37783E+00_JPRB ,&
&  0.37721E+00_JPRB, 0.37609E+00_JPRB, 0.37513E+00_JPRB, 0.37383E+00_JPRB, 0.37223E+00_JPRB, 0.37195E+00_JPRB, 0.37054E+00_JPRB ,&
&  0.36850E+00_JPRB, 0.36622E+00_JPRB, 0.36428E+00_JPRB, 0.36197E+00_JPRB, 0.35928E+00_JPRB, 0.35591E+00_JPRB, 0.35055E+00_JPRB ,&
&  0.34723E+00_JPRB, 0.34333E+00_JPRB, 0.33872E+00_JPRB, 0.33741E+00_JPRB, 0.33615E+00_JPRB, 0.32310E+00_JPRB, 0.32310E+00_JPRB ,&
&  0.33385E+00_JPRB, 0.33812E+00_JPRB, 0.34009E+00_JPRB, 0.34231E+00_JPRB, 0.32914E+00_JPRB, 0.31381E+00_JPRB, 0.30416E+00_JPRB ,&
&  0.28922E+00_JPRB, 0.30484E+00_JPRB, 0.31473E+00_JPRB, 0.31944E+00_JPRB, 0.32167E+00_JPRB, 0.32451E+00_JPRB, 0.34399E+00_JPRB ,&
&  0.38349E+00_JPRB, 0.39242E+00_JPRB, 0.40201E+00_JPRB, 0.41237E+00_JPRB, 0.42224E+00_JPRB, 0.42876E+00_JPRB, 0.43529E+00_JPRB ,&
&  0.43801E+00_JPRB, 0.43924E+00_JPRB, 0.43636E+00_JPRB, 0.43133E+00_JPRB, 0.42193E+00_JPRB, 0.41410E+00_JPRB, 0.40719E+00_JPRB ,&
&  0.40223E+00_JPRB, 0.39694E+00_JPRB, 0.39128E+00_JPRB, 0.38489E+00_JPRB, 0.37895E+00_JPRB, 0.37248E+00_JPRB, 0.36604E+00_JPRB ,&
&  0.35928E+00_JPRB, 0.35243E+00_JPRB, 0.34730E+00_JPRB, 0.34341E+00_JPRB, 0.34277E+00_JPRB, 0.34323E+00_JPRB, 0.34484E+00_JPRB ,&
&  0.34647E+00_JPRB, 0.34835E+00_JPRB, 0.35012E+00_JPRB, 0.35295E+00_JPRB, 0.35553E+00_JPRB, 0.35754E+00_JPRB, 0.35937E+00_JPRB ,&
&  0.36087E+00_JPRB, 0.36216E+00_JPRB, 0.36325E+00_JPRB, 0.36402E+00_JPRB, 0.36515E+00_JPRB, 0.36642E+00_JPRB, 0.36724E+00_JPRB ,&
&  0.36835E+00_JPRB, 0.36943E+00_JPRB, 0.37043E+00_JPRB, 0.37075E+00_JPRB, 0.37042E+00_JPRB, 0.36838E+00_JPRB, 0.36633E+00_JPRB ,&
&  0.36448E+00_JPRB, 0.36441E+00_JPRB, 0.36495E+00_JPRB, 0.36694E+00_JPRB, 0.36944E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.38377E+00_JPRB, 0.38602E+00_JPRB, 0.38849E+00_JPRB, 0.39019E+00_JPRB, 0.39056E+00_JPRB, 0.39036E+00_JPRB, 0.38957E+00_JPRB ,&
&  0.38842E+00_JPRB, 0.38753E+00_JPRB, 0.38611E+00_JPRB, 0.38379E+00_JPRB, 0.38186E+00_JPRB, 0.37957E+00_JPRB, 0.37726E+00_JPRB ,&
&  0.37424E+00_JPRB, 0.37131E+00_JPRB, 0.36899E+00_JPRB, 0.36594E+00_JPRB, 0.36316E+00_JPRB, 0.36079E+00_JPRB, 0.35697E+00_JPRB ,&
&  0.35425E+00_JPRB, 0.34848E+00_JPRB, 0.32726E+00_JPRB, 0.32667E+00_JPRB, 0.32197E+00_JPRB, 0.32105E+00_JPRB, 0.32357E+00_JPRB ,&
&  0.31891E+00_JPRB, 0.31856E+00_JPRB, 0.31222E+00_JPRB, 0.30255E+00_JPRB, 0.29699E+00_JPRB, 0.28799E+00_JPRB, 0.28771E+00_JPRB ,&
&  0.28616E+00_JPRB, 0.28937E+00_JPRB, 0.29690E+00_JPRB, 0.30117E+00_JPRB, 0.30350E+00_JPRB, 0.30598E+00_JPRB, 0.31490E+00_JPRB ,&
&  0.33968E+00_JPRB, 0.37713E+00_JPRB, 0.42593E+00_JPRB, 0.44767E+00_JPRB, 0.46518E+00_JPRB, 0.48338E+00_JPRB, 0.49998E+00_JPRB ,&
&  0.51102E+00_JPRB, 0.51990E+00_JPRB, 0.51756E+00_JPRB, 0.51323E+00_JPRB, 0.49998E+00_JPRB, 0.48238E+00_JPRB, 0.46249E+00_JPRB ,&
&  0.44403E+00_JPRB, 0.43070E+00_JPRB, 0.42016E+00_JPRB, 0.41319E+00_JPRB, 0.40815E+00_JPRB, 0.40354E+00_JPRB, 0.40038E+00_JPRB ,&
&  0.39339E+00_JPRB, 0.38104E+00_JPRB, 0.36967E+00_JPRB, 0.36013E+00_JPRB, 0.35632E+00_JPRB, 0.35495E+00_JPRB, 0.35708E+00_JPRB ,&
&  0.35953E+00_JPRB, 0.36355E+00_JPRB, 0.36833E+00_JPRB, 0.36987E+00_JPRB, 0.37311E+00_JPRB, 0.37740E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37770E+00_JPRB, 0.37760E+00_JPRB, 0.37661E+00_JPRB, 0.37507E+00_JPRB, 0.37317E+00_JPRB, 0.37106E+00_JPRB, 0.36955E+00_JPRB ,&
&  0.36925E+00_JPRB, 0.37038E+00_JPRB, 0.37393E+00_JPRB, 0.37804E+00_JPRB, 0.38299E+00_JPRB, 0.38539E+00_JPRB, 0.38439E+00_JPRB ,&
&  0.37992E+00_JPRB, 0.37798E+00_JPRB, 0.37669E+00_JPRB, 0.37879E+00_JPRB, 0.38115E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.39477E+00_JPRB, 0.40035E+00_JPRB, 0.40838E+00_JPRB, 0.41481E+00_JPRB, 0.41959E+00_JPRB, 0.41952E+00_JPRB, 0.41404E+00_JPRB ,&
&  0.40836E+00_JPRB, 0.40275E+00_JPRB, 0.39795E+00_JPRB, 0.39355E+00_JPRB, 0.39008E+00_JPRB, 0.38589E+00_JPRB, 0.38171E+00_JPRB ,&
&  0.37823E+00_JPRB, 0.37588E+00_JPRB, 0.37446E+00_JPRB, 0.37353E+00_JPRB, 0.37362E+00_JPRB, 0.37275E+00_JPRB, 0.37273E+00_JPRB ,&
&  0.37291E+00_JPRB, 0.36584E+00_JPRB, 0.34853E+00_JPRB, 0.33949E+00_JPRB, 0.33158E+00_JPRB, 0.33071E+00_JPRB, 0.33099E+00_JPRB ,&
&  0.33031E+00_JPRB, 0.29675E+00_JPRB, 0.30056E+00_JPRB, 0.29215E+00_JPRB, 0.28644E+00_JPRB, 0.28576E+00_JPRB, 0.27938E+00_JPRB ,&
&  0.27520E+00_JPRB, 0.27069E+00_JPRB, 0.27206E+00_JPRB, 0.27772E+00_JPRB, 0.27978E+00_JPRB, 0.28811E+00_JPRB, 0.29948E+00_JPRB ,&
&  0.32875E+00_JPRB, 0.42232E+00_JPRB, 0.46009E+00_JPRB, 0.48319E+00_JPRB, 0.50159E+00_JPRB, 0.52571E+00_JPRB, 0.54953E+00_JPRB ,&
&  0.56563E+00_JPRB, 0.58690E+00_JPRB, 0.59200E+00_JPRB, 0.58392E+00_JPRB, 0.58109E+00_JPRB, 0.56828E+00_JPRB, 0.54323E+00_JPRB ,&
&  0.51298E+00_JPRB, 0.47930E+00_JPRB, 0.45789E+00_JPRB, 0.44477E+00_JPRB, 0.43352E+00_JPRB, 0.42207E+00_JPRB, 0.41348E+00_JPRB ,&
&  0.41373E+00_JPRB, 0.40658E+00_JPRB, 0.39910E+00_JPRB, 0.38714E+00_JPRB, 0.38041E+00_JPRB, 0.37676E+00_JPRB, 0.37661E+00_JPRB ,&
&  0.37836E+00_JPRB, 0.38186E+00_JPRB, 0.38642E+00_JPRB, 0.39085E+00_JPRB, 0.39330E+00_JPRB, 0.39317E+00_JPRB, 0.39592E+00_JPRB ,&
&  0.39736E+00_JPRB, 0.39722E+00_JPRB, 0.39636E+00_JPRB, 0.39463E+00_JPRB, 0.39250E+00_JPRB, 0.39098E+00_JPRB, 0.39209E+00_JPRB ,&
&  0.39387E+00_JPRB, 0.39525E+00_JPRB, 0.39635E+00_JPRB, 0.39869E+00_JPRB, 0.40294E+00_JPRB, 0.40683E+00_JPRB, 0.40817E+00_JPRB ,&
&  0.40216E+00_JPRB, 0.39639E+00_JPRB, 0.39131E+00_JPRB, 0.39055E+00_JPRB, 0.39118E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.38652E+00_JPRB, 0.40056E+00_JPRB, 0.41948E+00_JPRB, 0.43429E+00_JPRB, 0.44631E+00_JPRB, 0.44487E+00_JPRB, 0.43648E+00_JPRB ,&
&  0.42740E+00_JPRB, 0.42005E+00_JPRB, 0.41334E+00_JPRB, 0.40652E+00_JPRB, 0.39841E+00_JPRB, 0.39220E+00_JPRB, 0.38701E+00_JPRB ,&
&  0.38412E+00_JPRB, 0.38243E+00_JPRB, 0.38141E+00_JPRB, 0.38040E+00_JPRB, 0.37997E+00_JPRB, 0.38069E+00_JPRB, 0.38117E+00_JPRB ,&
&  0.38244E+00_JPRB, 0.37782E+00_JPRB, 0.36384E+00_JPRB, 0.34949E+00_JPRB, 0.34229E+00_JPRB, 0.33468E+00_JPRB, 0.33654E+00_JPRB ,&
&  0.33598E+00_JPRB, 0.33525E+00_JPRB, 0.33160E+00_JPRB, 0.33059E+00_JPRB, 0.32063E+00_JPRB, 0.27709E+00_JPRB, 0.27914E+00_JPRB ,&
&  0.27013E+00_JPRB, 0.26118E+00_JPRB, 0.26026E+00_JPRB, 0.26133E+00_JPRB, 0.26411E+00_JPRB, 0.27930E+00_JPRB, 0.29800E+00_JPRB ,&
&  0.32719E+00_JPRB, 0.43226E+00_JPRB, 0.45997E+00_JPRB, 0.49029E+00_JPRB, 0.51732E+00_JPRB, 0.55246E+00_JPRB, 0.59171E+00_JPRB ,&
&  0.61390E+00_JPRB, 0.63792E+00_JPRB, 0.65051E+00_JPRB, 0.65622E+00_JPRB, 0.64852E+00_JPRB, 0.63891E+00_JPRB, 0.62816E+00_JPRB ,&
&  0.60682E+00_JPRB, 0.55223E+00_JPRB, 0.51343E+00_JPRB, 0.49065E+00_JPRB, 0.47139E+00_JPRB, 0.45642E+00_JPRB, 0.44402E+00_JPRB ,&
&  0.43689E+00_JPRB, 0.41803E+00_JPRB, 0.41146E+00_JPRB, 0.40307E+00_JPRB, 0.39850E+00_JPRB, 0.39507E+00_JPRB, 0.39095E+00_JPRB ,&
&  0.38530E+00_JPRB, 0.38420E+00_JPRB, 0.38759E+00_JPRB, 0.38938E+00_JPRB, 0.38937E+00_JPRB, 0.38705E+00_JPRB, 0.38457E+00_JPRB ,&
&  0.38036E+00_JPRB, 0.38226E+00_JPRB, 0.38101E+00_JPRB, 0.38091E+00_JPRB, 0.38217E+00_JPRB, 0.38803E+00_JPRB, 0.39531E+00_JPRB ,&
&  0.40119E+00_JPRB, 0.40370E+00_JPRB, 0.40397E+00_JPRB, 0.40463E+00_JPRB, 0.40405E+00_JPRB, 0.40272E+00_JPRB, 0.39916E+00_JPRB ,&
&  0.39600E+00_JPRB, 0.39375E+00_JPRB, 0.38868E+00_JPRB, 0.38395E+00_JPRB, 0.38164E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.38295E+00_JPRB, 0.40716E+00_JPRB, 0.43430E+00_JPRB, 0.45341E+00_JPRB, 0.46325E+00_JPRB, 0.45187E+00_JPRB, 0.43864E+00_JPRB ,&
&  0.42780E+00_JPRB, 0.41902E+00_JPRB, 0.41311E+00_JPRB, 0.40842E+00_JPRB, 0.40266E+00_JPRB, 0.39722E+00_JPRB, 0.39204E+00_JPRB ,&
&  0.38813E+00_JPRB, 0.37992E+00_JPRB, 0.37555E+00_JPRB, 0.37531E+00_JPRB, 0.37577E+00_JPRB, 0.37854E+00_JPRB, 0.38173E+00_JPRB ,&
&  0.38172E+00_JPRB, 0.37579E+00_JPRB, 0.36996E+00_JPRB, 0.35786E+00_JPRB, 0.34479E+00_JPRB, 0.33755E+00_JPRB, 0.33786E+00_JPRB ,&
&  0.33269E+00_JPRB, 0.32005E+00_JPRB, 0.33095E+00_JPRB, 0.33956E+00_JPRB, 0.32819E+00_JPRB, 0.32302E+00_JPRB, 0.28882E+00_JPRB ,&
&  0.27556E+00_JPRB, 0.26100E+00_JPRB, 0.25766E+00_JPRB, 0.25866E+00_JPRB, 0.26629E+00_JPRB, 0.28154E+00_JPRB, 0.31526E+00_JPRB ,&
&  0.36770E+00_JPRB, 0.44339E+00_JPRB, 0.46345E+00_JPRB, 0.50205E+00_JPRB, 0.53787E+00_JPRB, 0.56962E+00_JPRB, 0.59291E+00_JPRB ,&
&  0.62779E+00_JPRB, 0.67200E+00_JPRB, 0.69316E+00_JPRB, 0.68758E+00_JPRB, 0.68374E+00_JPRB, 0.68436E+00_JPRB, 0.69455E+00_JPRB ,&
&  0.71533E+00_JPRB, 0.63240E+00_JPRB, 0.57366E+00_JPRB, 0.53115E+00_JPRB, 0.50240E+00_JPRB, 0.48955E+00_JPRB, 0.45394E+00_JPRB ,&
&  0.43766E+00_JPRB, 0.43149E+00_JPRB, 0.42231E+00_JPRB, 0.41628E+00_JPRB, 0.41118E+00_JPRB, 0.40640E+00_JPRB, 0.39899E+00_JPRB ,&
&  0.39024E+00_JPRB, 0.38852E+00_JPRB, 0.41171E+00_JPRB, 0.41481E+00_JPRB, 0.40596E+00_JPRB, 0.38611E+00_JPRB, 0.36745E+00_JPRB ,&
&  0.35538E+00_JPRB, 0.35201E+00_JPRB, 0.35201E+00_JPRB, 0.35212E+00_JPRB, 0.35640E+00_JPRB, 0.36319E+00_JPRB, 0.37410E+00_JPRB ,&
&  0.37818E+00_JPRB, 0.38687E+00_JPRB, 0.39479E+00_JPRB, 0.39875E+00_JPRB, 0.39796E+00_JPRB, 0.39504E+00_JPRB, 0.39245E+00_JPRB ,&
&  0.39018E+00_JPRB, 0.38776E+00_JPRB, 0.38451E+00_JPRB, 0.37931E+00_JPRB, 0.37541E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.37321E+00_JPRB, 0.40556E+00_JPRB, 0.44811E+00_JPRB, 0.47790E+00_JPRB, 0.47989E+00_JPRB, 0.45415E+00_JPRB, 0.43127E+00_JPRB ,&
&  0.41762E+00_JPRB, 0.41158E+00_JPRB, 0.40976E+00_JPRB, 0.40563E+00_JPRB, 0.40098E+00_JPRB, 0.39587E+00_JPRB, 0.39082E+00_JPRB ,&
&  0.38644E+00_JPRB, 0.38236E+00_JPRB, 0.37745E+00_JPRB, 0.37073E+00_JPRB, 0.36624E+00_JPRB, 0.36561E+00_JPRB, 0.36731E+00_JPRB ,&
&  0.37104E+00_JPRB, 0.37028E+00_JPRB, 0.36774E+00_JPRB, 0.36127E+00_JPRB, 0.35137E+00_JPRB, 0.34078E+00_JPRB, 0.33173E+00_JPRB ,&
&  0.32658E+00_JPRB, 0.32625E+00_JPRB, 0.32778E+00_JPRB, 0.34068E+00_JPRB, 0.33600E+00_JPRB, 0.33406E+00_JPRB, 0.31242E+00_JPRB ,&
&  0.29252E+00_JPRB, 0.27154E+00_JPRB, 0.26358E+00_JPRB, 0.26678E+00_JPRB, 0.28549E+00_JPRB, 0.31609E+00_JPRB, 0.37052E+00_JPRB ,&
&  0.46924E+00_JPRB, 0.51704E+00_JPRB, 0.55732E+00_JPRB, 0.59545E+00_JPRB, 0.62503E+00_JPRB, 0.65592E+00_JPRB, 0.69210E+00_JPRB ,&
&  0.73620E+00_JPRB, 0.77010E+00_JPRB, 0.78459E+00_JPRB, 0.75215E+00_JPRB, 0.73031E+00_JPRB, 0.65485E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.83856E+00_JPRB, 0.73236E+00_JPRB, 0.64720E+00_JPRB, 0.59917E+00_JPRB, 0.56096E+00_JPRB, 0.52577E+00_JPRB, 0.49225E+00_JPRB ,&
&  0.46311E+00_JPRB, 0.44646E+00_JPRB, 0.43590E+00_JPRB, 0.41328E+00_JPRB, 0.39170E+00_JPRB, 0.38762E+00_JPRB, 0.40244E+00_JPRB ,&
&  0.44283E+00_JPRB, 0.50960E+00_JPRB, 0.61440E+00_JPRB, 0.44630E+00_JPRB, 0.39379E+00_JPRB, 0.35800E+00_JPRB, 0.34347E+00_JPRB ,&
&  0.34082E+00_JPRB, 0.34157E+00_JPRB, 0.34335E+00_JPRB, 0.34180E+00_JPRB, 0.34005E+00_JPRB, 0.34257E+00_JPRB, 0.34570E+00_JPRB ,&
&  0.34845E+00_JPRB, 0.35720E+00_JPRB, 0.36933E+00_JPRB, 0.37902E+00_JPRB, 0.38515E+00_JPRB, 0.38822E+00_JPRB, 0.39145E+00_JPRB ,&
&  0.39153E+00_JPRB, 0.38839E+00_JPRB, 0.38382E+00_JPRB, 0.37346E+00_JPRB, 0.36616E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.36715E+00_JPRB, 0.40636E+00_JPRB, 0.46318E+00_JPRB, 0.49551E+00_JPRB, 0.49629E+00_JPRB, 0.45574E+00_JPRB, 0.41424E+00_JPRB ,&
&  0.39173E+00_JPRB, 0.37760E+00_JPRB, 0.37549E+00_JPRB, 0.37305E+00_JPRB, 0.37138E+00_JPRB, 0.37019E+00_JPRB, 0.36901E+00_JPRB ,&
&  0.36642E+00_JPRB, 0.36282E+00_JPRB, 0.35878E+00_JPRB, 0.35584E+00_JPRB, 0.35287E+00_JPRB, 0.35042E+00_JPRB, 0.34987E+00_JPRB ,&
&  0.35139E+00_JPRB, 0.35171E+00_JPRB, 0.35036E+00_JPRB, 0.34816E+00_JPRB, 0.34770E+00_JPRB, 0.34996E+00_JPRB, 0.34896E+00_JPRB ,&
&  0.33935E+00_JPRB, 0.32975E+00_JPRB, 0.33496E+00_JPRB, 0.34947E+00_JPRB, 0.34536E+00_JPRB, 0.34610E+00_JPRB, 0.32396E+00_JPRB ,&
&  0.29984E+00_JPRB, 0.28329E+00_JPRB, 0.29173E+00_JPRB, 0.32828E+00_JPRB, 0.39353E+00_JPRB, 0.52073E+00_JPRB, 0.59427E+00_JPRB ,&
&  0.66167E+00_JPRB, 0.70464E+00_JPRB, 0.73107E+00_JPRB, 0.75772E+00_JPRB, 0.80721E+00_JPRB, 0.85713E+00_JPRB, 0.89304E+00_JPRB ,&
&  0.91699E+00_JPRB, 0.92489E+00_JPRB, 0.92528E+00_JPRB, 0.69147E+00_JPRB, 0.64896E+00_JPRB, 0.67273E+00_JPRB, 0.70259E+00_JPRB ,&
&  0.71883E+00_JPRB, 0.67323E+00_JPRB, 0.65478E+00_JPRB, 0.66001E+00_JPRB, 0.61627E+00_JPRB, 0.57629E+00_JPRB, 0.53927E+00_JPRB ,&
&  0.50600E+00_JPRB, 0.48367E+00_JPRB, 0.44703E+00_JPRB, 0.41602E+00_JPRB, 0.41510E+00_JPRB, 0.41709E+00_JPRB, 0.41938E+00_JPRB ,&
&  0.43077E+00_JPRB, 0.43622E+00_JPRB, 0.40767E+00_JPRB, 0.34231E+00_JPRB, 0.32428E+00_JPRB, 0.31753E+00_JPRB, 0.31782E+00_JPRB ,&
&  0.32013E+00_JPRB, 0.32284E+00_JPRB, 0.32708E+00_JPRB, 0.32785E+00_JPRB, 0.32629E+00_JPRB, 0.32607E+00_JPRB, 0.32606E+00_JPRB ,&
&  0.32412E+00_JPRB, 0.32659E+00_JPRB, 0.33418E+00_JPRB, 0.34083E+00_JPRB, 0.35406E+00_JPRB, 0.36645E+00_JPRB, 0.37718E+00_JPRB ,&
&  0.37539E+00_JPRB, 0.37338E+00_JPRB, 0.36950E+00_JPRB, 0.35906E+00_JPRB, 0.35322E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.40569E+00_JPRB, 0.44481E+00_JPRB, 0.47698E+00_JPRB, 0.50135E+00_JPRB, 0.49423E+00_JPRB, 0.46883E+00_JPRB, 0.41644E+00_JPRB ,&
&  0.40596E+00_JPRB, 0.37773E+00_JPRB, 0.36975E+00_JPRB, 0.35642E+00_JPRB, 0.34568E+00_JPRB, 0.33688E+00_JPRB, 0.33309E+00_JPRB ,&
&  0.33541E+00_JPRB, 0.33942E+00_JPRB, 0.34211E+00_JPRB, 0.34297E+00_JPRB, 0.34104E+00_JPRB, 0.33986E+00_JPRB, 0.33957E+00_JPRB ,&
&  0.33955E+00_JPRB, 0.33963E+00_JPRB, 0.34077E+00_JPRB, 0.34223E+00_JPRB, 0.34470E+00_JPRB, 0.34719E+00_JPRB, 0.34519E+00_JPRB ,&
&  0.33814E+00_JPRB, 0.33193E+00_JPRB, 0.34255E+00_JPRB, 0.35865E+00_JPRB, 0.35492E+00_JPRB, 0.35979E+00_JPRB, 0.33478E+00_JPRB ,&
&  0.30824E+00_JPRB, 0.32030E+00_JPRB, 0.39282E+00_JPRB, 0.55493E+00_JPRB, 0.61199E+00_JPRB, 0.67093E+00_JPRB, 0.71704E+00_JPRB ,&
&  0.67931E+00_JPRB, 0.69183E+00_JPRB, 0.75286E+00_JPRB, 0.92894E+00_JPRB, 0.95886E+00_JPRB, 0.99838E+00_JPRB, 0.10410E+01_JPRB ,&
&  0.11027E+01_JPRB, 0.10888E+01_JPRB, 0.10432E+01_JPRB, 0.84007E+00_JPRB, 0.93063E+00_JPRB, 0.10690E+01_JPRB, 0.89217E+00_JPRB ,&
&  0.83351E+00_JPRB, 0.78913E+00_JPRB, 0.78957E+00_JPRB, 0.73871E+00_JPRB, 0.66579E+00_JPRB, 0.61580E+00_JPRB, 0.58410E+00_JPRB ,&
&  0.54155E+00_JPRB, 0.51230E+00_JPRB, 0.46402E+00_JPRB, 0.45065E+00_JPRB, 0.45682E+00_JPRB, 0.44447E+00_JPRB, 0.42971E+00_JPRB ,&
&  0.41434E+00_JPRB, 0.38738E+00_JPRB, 0.36369E+00_JPRB, 0.33966E+00_JPRB, 0.32482E+00_JPRB, 0.32135E+00_JPRB, 0.32214E+00_JPRB ,&
&  0.32307E+00_JPRB, 0.32244E+00_JPRB, 0.32467E+00_JPRB, 0.32690E+00_JPRB, 0.32411E+00_JPRB, 0.32148E+00_JPRB, 0.32348E+00_JPRB ,&
&  0.32261E+00_JPRB, 0.31851E+00_JPRB, 0.31609E+00_JPRB, 0.32287E+00_JPRB, 0.33258E+00_JPRB, 0.34547E+00_JPRB, 0.35831E+00_JPRB ,&
&  0.36430E+00_JPRB, 0.37262E+00_JPRB, 0.37097E+00_JPRB, 0.37211E+00_JPRB, 0.37644E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.43321E+00_JPRB, 0.45774E+00_JPRB, 0.48391E+00_JPRB, 0.50276E+00_JPRB, 0.51697E+00_JPRB, 0.46608E+00_JPRB, 0.42125E+00_JPRB ,&
&  0.44817E+00_JPRB, 0.38807E+00_JPRB, 0.36372E+00_JPRB, 0.34946E+00_JPRB, 0.34434E+00_JPRB, 0.33411E+00_JPRB, 0.32392E+00_JPRB ,&
&  0.32018E+00_JPRB, 0.32336E+00_JPRB, 0.33163E+00_JPRB, 0.33583E+00_JPRB, 0.33701E+00_JPRB, 0.33738E+00_JPRB, 0.33672E+00_JPRB ,&
&  0.33683E+00_JPRB, 0.33805E+00_JPRB, 0.33946E+00_JPRB, 0.34146E+00_JPRB, 0.34176E+00_JPRB, 0.34236E+00_JPRB, 0.34072E+00_JPRB ,&
&  0.33566E+00_JPRB, 0.33568E+00_JPRB, 0.34741E+00_JPRB, 0.36762E+00_JPRB, 0.37426E+00_JPRB, 0.38528E+00_JPRB, 0.39224E+00_JPRB ,&
&  0.33707E+00_JPRB, 0.39145E+00_JPRB, 0.59398E+00_JPRB, 0.65910E+00_JPRB, 0.70697E+00_JPRB, 0.77239E+00_JPRB, 0.83034E+00_JPRB ,&
&  0.87434E+00_JPRB, 0.91141E+00_JPRB, 0.94200E+00_JPRB, 0.98283E+00_JPRB, 0.10375E+01_JPRB, 0.10862E+01_JPRB, 0.11507E+01_JPRB ,&
&  0.13088E+01_JPRB, 0.98945E+00_JPRB, 0.88098E+00_JPRB, 0.91184E+00_JPRB, 0.12971E+01_JPRB, 0.13127E+01_JPRB, 0.10911E+01_JPRB ,&
&  0.95298E+00_JPRB, 0.85885E+00_JPRB, 0.80839E+00_JPRB, 0.74614E+00_JPRB, 0.68794E+00_JPRB, 0.65385E+00_JPRB, 0.62520E+00_JPRB ,&
&  0.57429E+00_JPRB, 0.54922E+00_JPRB, 0.49783E+00_JPRB, 0.48837E+00_JPRB, 0.48890E+00_JPRB, 0.46621E+00_JPRB, 0.43234E+00_JPRB ,&
&  0.40398E+00_JPRB, 0.38071E+00_JPRB, 0.36264E+00_JPRB, 0.35049E+00_JPRB, 0.34279E+00_JPRB, 0.33782E+00_JPRB, 0.33426E+00_JPRB ,&
&  0.33227E+00_JPRB, 0.32990E+00_JPRB, 0.32359E+00_JPRB, 0.32387E+00_JPRB, 0.32504E+00_JPRB, 0.32743E+00_JPRB, 0.33444E+00_JPRB ,&
&  0.33667E+00_JPRB, 0.33205E+00_JPRB, 0.32631E+00_JPRB, 0.33229E+00_JPRB, 0.34411E+00_JPRB, 0.36250E+00_JPRB, 0.38380E+00_JPRB ,&
&  0.40465E+00_JPRB, 0.41024E+00_JPRB, 0.39139E+00_JPRB, 0.38736E+00_JPRB, 0.39784E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.45456E+00_JPRB, 0.47569E+00_JPRB, 0.49120E+00_JPRB, 0.50922E+00_JPRB, 0.52467E+00_JPRB, 0.47918E+00_JPRB, 0.45134E+00_JPRB ,&
&  0.51493E+00_JPRB, 0.43927E+00_JPRB, 0.48821E+00_JPRB, 0.39918E+00_JPRB, 0.35991E+00_JPRB, 0.34489E+00_JPRB, 0.33326E+00_JPRB ,&
&  0.32244E+00_JPRB, 0.32880E+00_JPRB, 0.33585E+00_JPRB, 0.33838E+00_JPRB, 0.34622E+00_JPRB, 0.34232E+00_JPRB, 0.33877E+00_JPRB ,&
&  0.33648E+00_JPRB, 0.33614E+00_JPRB, 0.34136E+00_JPRB, 0.34082E+00_JPRB, 0.33964E+00_JPRB, 0.33800E+00_JPRB, 0.33343E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32623E+00_JPRB, 0.34758E+00_JPRB, 0.36773E+00_JPRB, 0.38547E+00_JPRB, 0.40380E+00_JPRB, 0.43813E+00_JPRB ,&
&  0.46927E+00_JPRB, 0.49055E+00_JPRB, 0.63361E+00_JPRB, 0.67507E+00_JPRB, 0.72427E+00_JPRB, 0.76017E+00_JPRB, 0.80425E+00_JPRB ,&
&  0.84934E+00_JPRB, 0.88864E+00_JPRB, 0.91313E+00_JPRB, 0.95876E+00_JPRB, 0.10456E+01_JPRB, 0.11092E+01_JPRB, 0.12143E+01_JPRB ,&
&  0.14041E+01_JPRB, 0.10105E+01_JPRB, 0.11900E+01_JPRB, 0.11514E+01_JPRB, 0.13507E+01_JPRB, 0.14742E+01_JPRB, 0.14016E+01_JPRB ,&
&  0.13481E+01_JPRB, 0.10974E+01_JPRB, 0.93618E+00_JPRB, 0.81117E+00_JPRB, 0.72333E+00_JPRB, 0.69026E+00_JPRB, 0.66416E+00_JPRB ,&
&  0.66259E+00_JPRB, 0.64081E+00_JPRB, 0.55585E+00_JPRB, 0.52090E+00_JPRB, 0.51010E+00_JPRB, 0.50130E+00_JPRB, 0.46584E+00_JPRB ,&
&  0.44341E+00_JPRB, 0.42862E+00_JPRB, 0.40970E+00_JPRB, 0.39223E+00_JPRB, 0.37842E+00_JPRB, 0.36902E+00_JPRB, 0.35785E+00_JPRB ,&
&  0.35423E+00_JPRB, 0.34466E+00_JPRB, 0.32344E+00_JPRB, 0.31331E+00_JPRB, 0.31185E+00_JPRB, 0.31118E+00_JPRB, 0.32164E+00_JPRB ,&
&  0.33270E+00_JPRB, 0.33476E+00_JPRB, 0.35806E+00_JPRB, 0.39040E+00_JPRB, 0.40063E+00_JPRB, 0.41064E+00_JPRB, 0.46012E+00_JPRB ,&
&  0.43311E+00_JPRB, 0.44729E+00_JPRB, 0.45125E+00_JPRB, 0.43522E+00_JPRB, 0.43548E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.47804E+00_JPRB, 0.48904E+00_JPRB, 0.49857E+00_JPRB, 0.51726E+00_JPRB, 0.52953E+00_JPRB, 0.53105E+00_JPRB, 0.49427E+00_JPRB ,&
&  0.53375E+00_JPRB, 0.51415E+00_JPRB, 0.49536E+00_JPRB, 0.49247E+00_JPRB, 0.48414E+00_JPRB, 0.41423E+00_JPRB, 0.34236E+00_JPRB ,&
&  0.32190E+00_JPRB, 0.32002E+00_JPRB, 0.33709E+00_JPRB, 0.34138E+00_JPRB, 0.34758E+00_JPRB, 0.34672E+00_JPRB, 0.34251E+00_JPRB ,&
&  0.34093E+00_JPRB, 0.33884E+00_JPRB, 0.34347E+00_JPRB, 0.34402E+00_JPRB, 0.33965E+00_JPRB, 0.33834E+00_JPRB, 0.33757E+00_JPRB ,&
&  0.32301E+00_JPRB, 0.31964E+00_JPRB, 0.33379E+00_JPRB, 0.33662E+00_JPRB, 0.38710E+00_JPRB, 0.41349E+00_JPRB, 0.45257E+00_JPRB ,&
&  0.49283E+00_JPRB, 0.53488E+00_JPRB, 0.60598E+00_JPRB, 0.66115E+00_JPRB, 0.69841E+00_JPRB, 0.72731E+00_JPRB, 0.77033E+00_JPRB ,&
&  0.80558E+00_JPRB, 0.85295E+00_JPRB, 0.90702E+00_JPRB, 0.99362E+00_JPRB, 0.11215E+01_JPRB, 0.10308E+01_JPRB, 0.12415E+01_JPRB ,&
&  0.13366E+01_JPRB, 0.14916E+01_JPRB, 0.16290E+01_JPRB, 0.14639E+01_JPRB, 0.15966E+01_JPRB, 0.16356E+01_JPRB, 0.15156E+01_JPRB ,&
&  0.13590E+01_JPRB, 0.11333E+01_JPRB, 0.10430E+01_JPRB, 0.97259E+00_JPRB, 0.93324E+00_JPRB, 0.85420E+00_JPRB, 0.78374E+00_JPRB ,&
&  0.80020E+00_JPRB, 0.89361E+00_JPRB, 0.70956E+00_JPRB, 0.65376E+00_JPRB, 0.60729E+00_JPRB, 0.59194E+00_JPRB, 0.56299E+00_JPRB ,&
&  0.54698E+00_JPRB, 0.54312E+00_JPRB, 0.52236E+00_JPRB, 0.49616E+00_JPRB, 0.44806E+00_JPRB, 0.42582E+00_JPRB, 0.40000E+00_JPRB ,&
&  0.38546E+00_JPRB, 0.36222E+00_JPRB, 0.32946E+00_JPRB, 0.31092E+00_JPRB, 0.30018E+00_JPRB, 0.29919E+00_JPRB, 0.31390E+00_JPRB ,&
&  0.34542E+00_JPRB, 0.37698E+00_JPRB, 0.44924E+00_JPRB, 0.46720E+00_JPRB, 0.48065E+00_JPRB, 0.51181E+00_JPRB, 0.45662E+00_JPRB ,&
&  0.42713E+00_JPRB, 0.49061E+00_JPRB, 0.47694E+00_JPRB, 0.46653E+00_JPRB, 0.46802E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.51614E+00_JPRB, 0.52217E+00_JPRB, 0.52689E+00_JPRB, 0.53556E+00_JPRB, 0.54333E+00_JPRB, 0.53407E+00_JPRB, 0.53142E+00_JPRB ,&
&  0.53567E+00_JPRB, 0.53097E+00_JPRB, 0.51467E+00_JPRB, 0.50336E+00_JPRB, 0.48859E+00_JPRB, 0.48558E+00_JPRB, 0.42240E+00_JPRB ,&
&  0.35538E+00_JPRB, 0.34403E+00_JPRB, 0.33212E+00_JPRB, 0.33953E+00_JPRB, 0.35871E+00_JPRB, 0.35573E+00_JPRB, 0.35371E+00_JPRB ,&
&  0.35134E+00_JPRB, 0.34673E+00_JPRB, 0.34626E+00_JPRB, 0.34668E+00_JPRB, 0.34893E+00_JPRB, 0.35927E+00_JPRB, 0.36385E+00_JPRB ,&
&  0.34040E+00_JPRB, 0.31643E+00_JPRB, 0.31363E+00_JPRB, 0.32695E+00_JPRB, 0.37313E+00_JPRB, 0.42195E+00_JPRB, 0.46261E+00_JPRB ,&
&  0.50122E+00_JPRB, 0.54236E+00_JPRB, 0.59064E+00_JPRB, 0.64774E+00_JPRB, 0.68443E+00_JPRB, 0.72003E+00_JPRB, 0.75918E+00_JPRB ,&
&  0.80382E+00_JPRB, 0.86639E+00_JPRB, 0.94787E+00_JPRB, 0.10687E+01_JPRB, 0.10686E+01_JPRB, 0.11096E+01_JPRB, 0.12618E+01_JPRB ,&
&  0.13692E+01_JPRB, 0.14902E+01_JPRB, 0.16021E+01_JPRB, 0.17873E+01_JPRB, 0.18692E+01_JPRB, 0.17284E+01_JPRB, 0.15417E+01_JPRB ,&
&  0.13870E+01_JPRB, 0.11861E+01_JPRB, 0.12591E+01_JPRB, 0.11202E+01_JPRB, 0.10121E+01_JPRB, 0.98278E+00_JPRB, 0.92130E+00_JPRB ,&
&  0.89610E+00_JPRB, 0.92594E+00_JPRB, 0.89351E+00_JPRB, 0.87971E+00_JPRB, 0.81975E+00_JPRB, 0.80386E+00_JPRB, 0.75316E+00_JPRB ,&
&  0.75122E+00_JPRB, 0.73838E+00_JPRB, 0.63925E+00_JPRB, 0.55526E+00_JPRB, 0.48045E+00_JPRB, 0.46207E+00_JPRB, 0.42857E+00_JPRB ,&
&  0.40352E+00_JPRB, 0.37227E+00_JPRB, 0.34529E+00_JPRB, 0.31885E+00_JPRB, 0.31341E+00_JPRB, 0.32086E+00_JPRB, 0.33607E+00_JPRB ,&
&  0.37786E+00_JPRB, 0.45298E+00_JPRB, 0.48104E+00_JPRB, 0.50512E+00_JPRB, 0.52175E+00_JPRB, 0.54947E+00_JPRB, 0.44816E+00_JPRB ,&
&  0.45198E+00_JPRB, 0.50817E+00_JPRB, 0.50561E+00_JPRB, 0.50157E+00_JPRB, 0.50393E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.57918E+00_JPRB, 0.58125E+00_JPRB, 0.57961E+00_JPRB, 0.57477E+00_JPRB, 0.56673E+00_JPRB, 0.55553E+00_JPRB, 0.55389E+00_JPRB ,&
&  0.55440E+00_JPRB, 0.54453E+00_JPRB, 0.53005E+00_JPRB, 0.51833E+00_JPRB, 0.50669E+00_JPRB, 0.50119E+00_JPRB, 0.51210E+00_JPRB ,&
&  0.41029E+00_JPRB, 0.37997E+00_JPRB, 0.33589E+00_JPRB, 0.33665E+00_JPRB, 0.39729E+00_JPRB, 0.37334E+00_JPRB, 0.36461E+00_JPRB ,&
&  0.35460E+00_JPRB, 0.35722E+00_JPRB, 0.35484E+00_JPRB, 0.37084E+00_JPRB, 0.38590E+00_JPRB, 0.40573E+00_JPRB, 0.40755E+00_JPRB ,&
&  0.37553E+00_JPRB, 0.33813E+00_JPRB, 0.32914E+00_JPRB, 0.33657E+00_JPRB, 0.37510E+00_JPRB, 0.41261E+00_JPRB, 0.48110E+00_JPRB ,&
&  0.52988E+00_JPRB, 0.57388E+00_JPRB, 0.61563E+00_JPRB, 0.66956E+00_JPRB, 0.71124E+00_JPRB, 0.75803E+00_JPRB, 0.81548E+00_JPRB ,&
&  0.87942E+00_JPRB, 0.95707E+00_JPRB, 0.10380E+01_JPRB, 0.11402E+01_JPRB, 0.11617E+01_JPRB, 0.12160E+01_JPRB, 0.12927E+01_JPRB ,&
&  0.13700E+01_JPRB, 0.14215E+01_JPRB, 0.15674E+01_JPRB, 0.19366E+01_JPRB, 0.21032E+01_JPRB, 0.20320E+01_JPRB, 0.17728E+01_JPRB ,&
&  0.15197E+01_JPRB, 0.12823E+01_JPRB, 0.12060E+01_JPRB, 0.11019E+01_JPRB, 0.10049E+01_JPRB, 0.10270E+01_JPRB, 0.98965E+00_JPRB ,&
&  0.94495E+00_JPRB, 0.95767E+00_JPRB, 0.96770E+00_JPRB, 0.99629E+00_JPRB, 0.95689E+00_JPRB, 0.96036E+00_JPRB, 0.93343E+00_JPRB ,&
&  0.93514E+00_JPRB, 0.85783E+00_JPRB, 0.66657E+00_JPRB, 0.57507E+00_JPRB, 0.48943E+00_JPRB, 0.46048E+00_JPRB, 0.51246E+00_JPRB ,&
&  0.46393E+00_JPRB, 0.42983E+00_JPRB, 0.39826E+00_JPRB, 0.37331E+00_JPRB, 0.37850E+00_JPRB, 0.41557E+00_JPRB, 0.39991E+00_JPRB ,&
&  0.40105E+00_JPRB, 0.47160E+00_JPRB, 0.55709E+00_JPRB, 0.62806E+00_JPRB, 0.66189E+00_JPRB, 0.66197E+00_JPRB, 0.56664E+00_JPRB ,&
&  0.58276E+00_JPRB, 0.57049E+00_JPRB, 0.57299E+00_JPRB, 0.56720E+00_JPRB, 0.57018E+00_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.66747E+00_JPRB, 0.66254E+00_JPRB, 0.64900E+00_JPRB, 0.62390E+00_JPRB, 0.60396E+00_JPRB, 0.59145E+00_JPRB, 0.58596E+00_JPRB ,&
&  0.57652E+00_JPRB, 0.55927E+00_JPRB, 0.54607E+00_JPRB, 0.53179E+00_JPRB, 0.52659E+00_JPRB, 0.52626E+00_JPRB, 0.52480E+00_JPRB ,&
&  0.54501E+00_JPRB, 0.47411E+00_JPRB, 0.39337E+00_JPRB, 0.34523E+00_JPRB, 0.36770E+00_JPRB, 0.35836E+00_JPRB, 0.36326E+00_JPRB ,&
&  0.36153E+00_JPRB, 0.38949E+00_JPRB, 0.39431E+00_JPRB, 0.42178E+00_JPRB, 0.45696E+00_JPRB, 0.49771E+00_JPRB, 0.50171E+00_JPRB ,&
&  0.46443E+00_JPRB, 0.41325E+00_JPRB, 0.39189E+00_JPRB, 0.40191E+00_JPRB, 0.45173E+00_JPRB, 0.44784E+00_JPRB, 0.51242E+00_JPRB ,&
&  0.55765E+00_JPRB, 0.60548E+00_JPRB, 0.65788E+00_JPRB, 0.70611E+00_JPRB, 0.76850E+00_JPRB, 0.84195E+00_JPRB, 0.91879E+00_JPRB ,&
&  0.99897E+00_JPRB, 0.10602E+01_JPRB, 0.10831E+01_JPRB, 0.11239E+01_JPRB, 0.12277E+01_JPRB, 0.13279E+01_JPRB, 0.12569E+01_JPRB ,&
&  0.12870E+01_JPRB, 0.13255E+01_JPRB, 0.14434E+01_JPRB, 0.17836E+01_JPRB, 0.20101E+01_JPRB, 0.20308E+01_JPRB, 0.18982E+01_JPRB ,&
&  0.16936E+01_JPRB, 0.14984E+01_JPRB, 0.14216E+01_JPRB, 0.12680E+01_JPRB, 0.11799E+01_JPRB, 0.11687E+01_JPRB, 0.11496E+01_JPRB ,&
&  0.11080E+01_JPRB, 0.11106E+01_JPRB, 0.11316E+01_JPRB, 0.11002E+01_JPRB, 0.10960E+01_JPRB, 0.10853E+01_JPRB, 0.11138E+01_JPRB ,&
&  0.11971E+01_JPRB, 0.84333E+00_JPRB, 0.68427E+00_JPRB, 0.70245E+00_JPRB, 0.58877E+00_JPRB, 0.52247E+00_JPRB, 0.52570E+00_JPRB ,&
&  0.46706E+00_JPRB, 0.46059E+00_JPRB, 0.48261E+00_JPRB, 0.46168E+00_JPRB, 0.49947E+00_JPRB, 0.56803E+00_JPRB, 0.53035E+00_JPRB ,&
&  0.52789E+00_JPRB, 0.55014E+00_JPRB, 0.69835E+00_JPRB, 0.79394E+00_JPRB, 0.83064E+00_JPRB, 0.80788E+00_JPRB, 0.76583E+00_JPRB ,&
&  0.70743E+00_JPRB, 0.68445E+00_JPRB, 0.68186E+00_JPRB, 0.67693E+00_JPRB, 0.67388E+00_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.77105E+00_JPRB, 0.75414E+00_JPRB, 0.73487E+00_JPRB, 0.70743E+00_JPRB, 0.68637E+00_JPRB, 0.66400E+00_JPRB, 0.63289E+00_JPRB ,&
&  0.60265E+00_JPRB, 0.57868E+00_JPRB, 0.56063E+00_JPRB, 0.55114E+00_JPRB, 0.55070E+00_JPRB, 0.55594E+00_JPRB, 0.55654E+00_JPRB ,&
&  0.56226E+00_JPRB, 0.54056E+00_JPRB, 0.43672E+00_JPRB, 0.37109E+00_JPRB, 0.33051E+00_JPRB, 0.33966E+00_JPRB, 0.35160E+00_JPRB ,&
&  0.37818E+00_JPRB, 0.40979E+00_JPRB, 0.46769E+00_JPRB, 0.55071E+00_JPRB, 0.60590E+00_JPRB, 0.63621E+00_JPRB, 0.64883E+00_JPRB ,&
&  0.60395E+00_JPRB, 0.55803E+00_JPRB, 0.50017E+00_JPRB, 0.52293E+00_JPRB, 0.53235E+00_JPRB, 0.52507E+00_JPRB, 0.58608E+00_JPRB ,&
&  0.63055E+00_JPRB, 0.68541E+00_JPRB, 0.73332E+00_JPRB, 0.80165E+00_JPRB, 0.86034E+00_JPRB, 0.90616E+00_JPRB, 0.94753E+00_JPRB ,&
&  0.98108E+00_JPRB, 0.10215E+01_JPRB, 0.10496E+01_JPRB, 0.10664E+01_JPRB, 0.11219E+01_JPRB, 0.11813E+01_JPRB, 0.11510E+01_JPRB ,&
&  0.12268E+01_JPRB, 0.11701E+01_JPRB, 0.11608E+01_JPRB, 0.16357E+01_JPRB, 0.22479E+01_JPRB, 0.22168E+01_JPRB, 0.19698E+01_JPRB ,&
&  0.18996E+01_JPRB, 0.18186E+01_JPRB, 0.18042E+01_JPRB, 0.17298E+01_JPRB, 0.15837E+01_JPRB, 0.15420E+01_JPRB, 0.14882E+01_JPRB ,&
&  0.14423E+01_JPRB, 0.14030E+01_JPRB, 0.13557E+01_JPRB, 0.13656E+01_JPRB, 0.13775E+01_JPRB, 0.14187E+01_JPRB, 0.14635E+01_JPRB ,&
&  0.14471E+01_JPRB, 0.12832E+01_JPRB, 0.91420E+00_JPRB, 0.86511E+00_JPRB, 0.69734E+00_JPRB, 0.62621E+00_JPRB, 0.61761E+00_JPRB ,&
&  0.54811E+00_JPRB, 0.55862E+00_JPRB, 0.57816E+00_JPRB, 0.55849E+00_JPRB, 0.67778E+00_JPRB, 0.73728E+00_JPRB, 0.75127E+00_JPRB ,&
&  0.74648E+00_JPRB, 0.73167E+00_JPRB, 0.92653E+00_JPRB, 0.99120E+00_JPRB, 0.99464E+00_JPRB, 0.96283E+00_JPRB, 0.92190E+00_JPRB ,&
&  0.86239E+00_JPRB, 0.82617E+00_JPRB, 0.81060E+00_JPRB, 0.80232E+00_JPRB, 0.78472E+00_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.83249E+00_JPRB, 0.79456E+00_JPRB, 0.75723E+00_JPRB, 0.72961E+00_JPRB, 0.70858E+00_JPRB, 0.68084E+00_JPRB, 0.64376E+00_JPRB ,&
&  0.61641E+00_JPRB, 0.60087E+00_JPRB, 0.58912E+00_JPRB, 0.57839E+00_JPRB, 0.57584E+00_JPRB, 0.57957E+00_JPRB, 0.58433E+00_JPRB ,&
&  0.58134E+00_JPRB, 0.52673E+00_JPRB, 0.41847E+00_JPRB, 0.35102E+00_JPRB, 0.31343E+00_JPRB, 0.31204E+00_JPRB, 0.33554E+00_JPRB ,&
&  0.37721E+00_JPRB, 0.47291E+00_JPRB, 0.61815E+00_JPRB, 0.71914E+00_JPRB, 0.79991E+00_JPRB, 0.86010E+00_JPRB, 0.86506E+00_JPRB ,&
&  0.90707E+00_JPRB, 0.74347E+00_JPRB, 0.72465E+00_JPRB, 0.66212E+00_JPRB, 0.66986E+00_JPRB, 0.70452E+00_JPRB, 0.70888E+00_JPRB ,&
&  0.69211E+00_JPRB, 0.72227E+00_JPRB, 0.75301E+00_JPRB, 0.80578E+00_JPRB, 0.84059E+00_JPRB, 0.88761E+00_JPRB, 0.96369E+00_JPRB ,&
&  0.10494E+01_JPRB, 0.10867E+01_JPRB, 0.10426E+01_JPRB, 0.99987E+00_JPRB, 0.11659E+01_JPRB, 0.13400E+01_JPRB, 0.13819E+01_JPRB ,&
&  0.13198E+01_JPRB, 0.13678E+01_JPRB, 0.19926E+01_JPRB, 0.22732E+01_JPRB, 0.23684E+01_JPRB, 0.25041E+01_JPRB, 0.23917E+01_JPRB ,&
&  0.21887E+01_JPRB, 0.19972E+01_JPRB, 0.18921E+01_JPRB, 0.16700E+01_JPRB, 0.18590E+01_JPRB, 0.19118E+01_JPRB, 0.19158E+01_JPRB ,&
&  0.18868E+01_JPRB, 0.17910E+01_JPRB, 0.17223E+01_JPRB, 0.17263E+01_JPRB, 0.17707E+01_JPRB, 0.18265E+01_JPRB, 0.17523E+01_JPRB ,&
&  0.15481E+01_JPRB, 0.15981E+01_JPRB, 0.13730E+01_JPRB, 0.11234E+01_JPRB, 0.92350E+00_JPRB, 0.85479E+00_JPRB, 0.78856E+00_JPRB ,&
&  0.71626E+00_JPRB, 0.68216E+00_JPRB, 0.71233E+00_JPRB, 0.77337E+00_JPRB, 0.88839E+00_JPRB, 0.98719E+00_JPRB, 0.99789E+00_JPRB ,&
&  0.96846E+00_JPRB, 0.11072E+01_JPRB, 0.11281E+01_JPRB, 0.10948E+01_JPRB, 0.10843E+01_JPRB, 0.10304E+01_JPRB, 0.99268E+00_JPRB ,&
&  0.95816E+00_JPRB, 0.94286E+00_JPRB, 0.93052E+00_JPRB, 0.90899E+00_JPRB, 0.87339E+00_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.88541E+00_JPRB, 0.84101E+00_JPRB, 0.79788E+00_JPRB, 0.76409E+00_JPRB, 0.73064E+00_JPRB, 0.69856E+00_JPRB, 0.66213E+00_JPRB ,&
&  0.63401E+00_JPRB, 0.61705E+00_JPRB, 0.60430E+00_JPRB, 0.60144E+00_JPRB, 0.60501E+00_JPRB, 0.59959E+00_JPRB, 0.59670E+00_JPRB ,&
&  0.58310E+00_JPRB, 0.48670E+00_JPRB, 0.39536E+00_JPRB, 0.36601E+00_JPRB, 0.34332E+00_JPRB, 0.32324E+00_JPRB, 0.33809E+00_JPRB ,&
&  0.40655E+00_JPRB, 0.58056E+00_JPRB, 0.75754E+00_JPRB, 0.91183E+00_JPRB, 0.10370E+01_JPRB, 0.11186E+01_JPRB, 0.11140E+01_JPRB ,&
&  0.10009E+01_JPRB, 0.10057E+01_JPRB, 0.98995E+00_JPRB, 0.93817E+00_JPRB, 0.87262E+00_JPRB, 0.84404E+00_JPRB, 0.82214E+00_JPRB ,&
&  0.80039E+00_JPRB, 0.81896E+00_JPRB, 0.85283E+00_JPRB, 0.89085E+00_JPRB, 0.95387E+00_JPRB, 0.10275E+01_JPRB, 0.10990E+01_JPRB ,&
&  0.11090E+01_JPRB, 0.11030E+01_JPRB, 0.10386E+01_JPRB, 0.10356E+01_JPRB, 0.10471E+01_JPRB, 0.13855E+01_JPRB, 0.15274E+01_JPRB ,&
&  0.18667E+01_JPRB, 0.21309E+01_JPRB, 0.22007E+01_JPRB, 0.26036E+01_JPRB, 0.22280E+01_JPRB, 0.24367E+01_JPRB, 0.25533E+01_JPRB ,&
&  0.22702E+01_JPRB, 0.20428E+01_JPRB, 0.19574E+01_JPRB, 0.16651E+01_JPRB, 0.15817E+01_JPRB, 0.22472E+01_JPRB, 0.23833E+01_JPRB ,&
&  0.23989E+01_JPRB, 0.24870E+01_JPRB, 0.23367E+01_JPRB, 0.22011E+01_JPRB, 0.20616E+01_JPRB, 0.18850E+01_JPRB, 0.14530E+01_JPRB ,&
&  0.14492E+01_JPRB, 0.13193E+01_JPRB, 0.13572E+01_JPRB, 0.12098E+01_JPRB, 0.11071E+01_JPRB, 0.10016E+01_JPRB, 0.89457E+00_JPRB ,&
&  0.82682E+00_JPRB, 0.79420E+00_JPRB, 0.93648E+00_JPRB, 0.11650E+01_JPRB, 0.13107E+01_JPRB, 0.11858E+01_JPRB, 0.13593E+01_JPRB ,&
&  0.13615E+01_JPRB, 0.12509E+01_JPRB, 0.10805E+01_JPRB, 0.10598E+01_JPRB, 0.10430E+01_JPRB, 0.10161E+01_JPRB, 0.10041E+01_JPRB ,&
&  0.98682E+00_JPRB, 0.96426E+00_JPRB, 0.95022E+00_JPRB, 0.94471E+00_JPRB, 0.92074E+00_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.90307E+00_JPRB, 0.85888E+00_JPRB, 0.81276E+00_JPRB, 0.77716E+00_JPRB, 0.74567E+00_JPRB, 0.71481E+00_JPRB, 0.68779E+00_JPRB ,&
&  0.66993E+00_JPRB, 0.65113E+00_JPRB, 0.62544E+00_JPRB, 0.60897E+00_JPRB, 0.59980E+00_JPRB, 0.61217E+00_JPRB, 0.60296E+00_JPRB ,&
&  0.57859E+00_JPRB, 0.50079E+00_JPRB, 0.41155E+00_JPRB, 0.38124E+00_JPRB, 0.38808E+00_JPRB, 0.36830E+00_JPRB, 0.37049E+00_JPRB ,&
&  0.49985E+00_JPRB, 0.77922E+00_JPRB, 0.95829E+00_JPRB, 0.10827E+01_JPRB, 0.12476E+01_JPRB, 0.13385E+01_JPRB, 0.12913E+01_JPRB ,&
&  0.12697E+01_JPRB, 0.11546E+01_JPRB, 0.10721E+01_JPRB, 0.10039E+01_JPRB, 0.95296E+00_JPRB, 0.89915E+00_JPRB, 0.85518E+00_JPRB ,&
&  0.85079E+00_JPRB, 0.85533E+00_JPRB, 0.88721E+00_JPRB, 0.89521E+00_JPRB, 0.96660E+00_JPRB, 0.10063E+01_JPRB, 0.10181E+01_JPRB ,&
&  0.10231E+01_JPRB, 0.10448E+01_JPRB, 0.10304E+01_JPRB, 0.10369E+01_JPRB, 0.11828E+01_JPRB, 0.15419E+01_JPRB, 0.18671E+01_JPRB ,&
&  0.20769E+01_JPRB, 0.22566E+01_JPRB, 0.25002E+01_JPRB, 0.27971E+01_JPRB, 0.32712E+01_JPRB, 0.31676E+01_JPRB, 0.29966E+01_JPRB ,&
&  0.22436E+01_JPRB, 0.17996E+01_JPRB, 0.14855E+01_JPRB, 0.12305E+01_JPRB, 0.18007E+01_JPRB, 0.28270E+01_JPRB, 0.32087E+01_JPRB ,&
&  0.31914E+01_JPRB, 0.32942E+01_JPRB, 0.31500E+01_JPRB, 0.26374E+01_JPRB, 0.22097E+01_JPRB, 0.17735E+01_JPRB, 0.19794E+01_JPRB ,&
&  0.19082E+01_JPRB, 0.18282E+01_JPRB, 0.16111E+01_JPRB, 0.13069E+01_JPRB, 0.10662E+01_JPRB, 0.11081E+01_JPRB, 0.10122E+01_JPRB ,&
&  0.92764E+00_JPRB, 0.94656E+00_JPRB, 0.13854E+01_JPRB, 0.16305E+01_JPRB, 0.16012E+01_JPRB, 0.15181E+01_JPRB, 0.17189E+01_JPRB ,&
&  0.14276E+01_JPRB, 0.12408E+01_JPRB, 0.11114E+01_JPRB, 0.10403E+01_JPRB, 0.10221E+01_JPRB, 0.10043E+01_JPRB, 0.99498E+00_JPRB ,&
&  0.98234E+00_JPRB, 0.97481E+00_JPRB, 0.96349E+00_JPRB, 0.94831E+00_JPRB, 0.92878E+00_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.91843E+00_JPRB, 0.87436E+00_JPRB, 0.83716E+00_JPRB, 0.80008E+00_JPRB, 0.76660E+00_JPRB, 0.74126E+00_JPRB, 0.72239E+00_JPRB ,&
&  0.69699E+00_JPRB, 0.67610E+00_JPRB, 0.65205E+00_JPRB, 0.62912E+00_JPRB, 0.61909E+00_JPRB, 0.62019E+00_JPRB, 0.60967E+00_JPRB ,&
&  0.59216E+00_JPRB, 0.55579E+00_JPRB, 0.45238E+00_JPRB, 0.42008E+00_JPRB, 0.42305E+00_JPRB, 0.42331E+00_JPRB, 0.44811E+00_JPRB ,&
&  0.69168E+00_JPRB, 0.10185E+01_JPRB, 0.10968E+01_JPRB, 0.11407E+01_JPRB, 0.13318E+01_JPRB, 0.12087E+01_JPRB, 0.11791E+01_JPRB ,&
&  0.12574E+01_JPRB, 0.11577E+01_JPRB, 0.10801E+01_JPRB, 0.10173E+01_JPRB, 0.97283E+00_JPRB, 0.93981E+00_JPRB, 0.91007E+00_JPRB ,&
&  0.89086E+00_JPRB, 0.88481E+00_JPRB, 0.91814E+00_JPRB, 0.99384E+00_JPRB, 0.10196E+01_JPRB, 0.10159E+01_JPRB, 0.10149E+01_JPRB ,&
&  0.10039E+01_JPRB, 0.99431E+00_JPRB, 0.99876E+00_JPRB, 0.11160E+01_JPRB, 0.13149E+01_JPRB, 0.17295E+01_JPRB, 0.24881E+01_JPRB ,&
&  0.24949E+01_JPRB, 0.27540E+01_JPRB, 0.27560E+01_JPRB, 0.37526E+01_JPRB, 0.42142E+01_JPRB, 0.38466E+01_JPRB, 0.33258E+01_JPRB ,&
&  0.23499E+01_JPRB, 0.18312E+01_JPRB, 0.16422E+01_JPRB, 0.17450E+01_JPRB, 0.21914E+01_JPRB, 0.27814E+01_JPRB, 0.35263E+01_JPRB ,&
&  0.31128E+01_JPRB, 0.30751E+01_JPRB, 0.30476E+01_JPRB, 0.27785E+01_JPRB, 0.17180E+01_JPRB, 0.13211E+01_JPRB, 0.18354E+01_JPRB ,&
&  0.18448E+01_JPRB, 0.11327E+01_JPRB, 0.82240E+00_JPRB, 0.95903E+00_JPRB, 0.73146E+00_JPRB, 0.80663E+00_JPRB, 0.10253E+01_JPRB ,&
&  0.10651E+01_JPRB, 0.11948E+01_JPRB, 0.16622E+01_JPRB, 0.16463E+01_JPRB, 0.16696E+01_JPRB, 0.17956E+01_JPRB, 0.16864E+01_JPRB ,&
&  0.13973E+01_JPRB, 0.12032E+01_JPRB, 0.11986E+01_JPRB, 0.11266E+01_JPRB, 0.10977E+01_JPRB, 0.10695E+01_JPRB, 0.10434E+01_JPRB ,&
&  0.10381E+01_JPRB, 0.10222E+01_JPRB, 0.99950E+00_JPRB, 0.97577E+00_JPRB, 0.95440E+00_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.93290E+00_JPRB, 0.89166E+00_JPRB, 0.85026E+00_JPRB, 0.81716E+00_JPRB, 0.79350E+00_JPRB, 0.76806E+00_JPRB, 0.73002E+00_JPRB ,&
&  0.68640E+00_JPRB, 0.66139E+00_JPRB, 0.64756E+00_JPRB, 0.63144E+00_JPRB, 0.62837E+00_JPRB, 0.63458E+00_JPRB, 0.62746E+00_JPRB ,&
&  0.60271E+00_JPRB, 0.57361E+00_JPRB, 0.53109E+00_JPRB, 0.49135E+00_JPRB, 0.47504E+00_JPRB, 0.46009E+00_JPRB, 0.56816E+00_JPRB ,&
&  0.94302E+00_JPRB, 0.12717E+01_JPRB, 0.12678E+01_JPRB, 0.11709E+01_JPRB, 0.11441E+01_JPRB, 0.10693E+01_JPRB, 0.11895E+01_JPRB ,&
&  0.11991E+01_JPRB, 0.11530E+01_JPRB, 0.10623E+01_JPRB, 0.10019E+01_JPRB, 0.95900E+00_JPRB, 0.92052E+00_JPRB, 0.90322E+00_JPRB ,&
&  0.89159E+00_JPRB, 0.89822E+00_JPRB, 0.96160E+00_JPRB, 0.98298E+00_JPRB, 0.99074E+00_JPRB, 0.97927E+00_JPRB, 0.10122E+01_JPRB ,&
&  0.99416E+00_JPRB, 0.99272E+00_JPRB, 0.10252E+01_JPRB, 0.11937E+01_JPRB, 0.16285E+01_JPRB, 0.21857E+01_JPRB, 0.23686E+01_JPRB ,&
&  0.24299E+01_JPRB, 0.26139E+01_JPRB, 0.32242E+01_JPRB, 0.38157E+01_JPRB, 0.41802E+01_JPRB, 0.40081E+01_JPRB, 0.31789E+01_JPRB ,&
&  0.26923E+01_JPRB, 0.23188E+01_JPRB, 0.19120E+01_JPRB, 0.20304E+01_JPRB, 0.26455E+01_JPRB, 0.27487E+01_JPRB, 0.32173E+01_JPRB ,&
&  0.31859E+01_JPRB, 0.28614E+01_JPRB, 0.22619E+01_JPRB, 0.17366E+01_JPRB, 0.17338E+01_JPRB, 0.13072E+01_JPRB, 0.86509E+00_JPRB ,&
&  0.78327E+00_JPRB, 0.71172E+00_JPRB, 0.63728E+00_JPRB, 0.64143E+00_JPRB, 0.60732E+00_JPRB, 0.71758E+00_JPRB, 0.99343E+00_JPRB ,&
&  0.12419E+01_JPRB, 0.14244E+01_JPRB, 0.15937E+01_JPRB, 0.15514E+01_JPRB, 0.14889E+01_JPRB, 0.17045E+01_JPRB, 0.19270E+01_JPRB ,&
&  0.12819E+01_JPRB, 0.11818E+01_JPRB, 0.12045E+01_JPRB, 0.11717E+01_JPRB, 0.11471E+01_JPRB, 0.11148E+01_JPRB, 0.10936E+01_JPRB ,&
&  0.10834E+01_JPRB, 0.10609E+01_JPRB, 0.10280E+01_JPRB, 0.99814E+00_JPRB, 0.96593E+00_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.91380E+00_JPRB, 0.86239E+00_JPRB, 0.83215E+00_JPRB, 0.81966E+00_JPRB, 0.80108E+00_JPRB, 0.75981E+00_JPRB, 0.70283E+00_JPRB ,&
&  0.66513E+00_JPRB, 0.64567E+00_JPRB, 0.63679E+00_JPRB, 0.63121E+00_JPRB, 0.63165E+00_JPRB, 0.63067E+00_JPRB, 0.62230E+00_JPRB ,&
&  0.59163E+00_JPRB, 0.57314E+00_JPRB, 0.57654E+00_JPRB, 0.55208E+00_JPRB, 0.54436E+00_JPRB, 0.53499E+00_JPRB, 0.72008E+00_JPRB ,&
&  0.12135E+01_JPRB, 0.14956E+01_JPRB, 0.13930E+01_JPRB, 0.11694E+01_JPRB, 0.11462E+01_JPRB, 0.11659E+01_JPRB, 0.12153E+01_JPRB ,&
&  0.11559E+01_JPRB, 0.10795E+01_JPRB, 0.95778E+00_JPRB, 0.91216E+00_JPRB, 0.90211E+00_JPRB, 0.89165E+00_JPRB, 0.88986E+00_JPRB ,&
&  0.88718E+00_JPRB, 0.92597E+00_JPRB, 0.93494E+00_JPRB, 0.94337E+00_JPRB, 0.96484E+00_JPRB, 0.99415E+00_JPRB, 0.10321E+01_JPRB ,&
&  0.10495E+01_JPRB, 0.10654E+01_JPRB, 0.11535E+01_JPRB, 0.14684E+01_JPRB, 0.18685E+01_JPRB, 0.20521E+01_JPRB, 0.24340E+01_JPRB ,&
&  0.28306E+01_JPRB, 0.31732E+01_JPRB, 0.31288E+01_JPRB, 0.40207E+01_JPRB, 0.41165E+01_JPRB, 0.35739E+01_JPRB, 0.30380E+01_JPRB ,&
&  0.26748E+01_JPRB, 0.27611E+01_JPRB, 0.21819E+01_JPRB, 0.19777E+01_JPRB, 0.28889E+01_JPRB, 0.30664E+01_JPRB, 0.32240E+01_JPRB ,&
&  0.34135E+01_JPRB, 0.29234E+01_JPRB, 0.23298E+01_JPRB, 0.19180E+01_JPRB, 0.23192E+01_JPRB, 0.20742E+01_JPRB, 0.98335E+00_JPRB ,&
&  0.76383E+00_JPRB, 0.69605E+00_JPRB, 0.60937E+00_JPRB, 0.56900E+00_JPRB, 0.55848E+00_JPRB, 0.66538E+00_JPRB, 0.10840E+01_JPRB ,&
&  0.11643E+01_JPRB, 0.14757E+01_JPRB, 0.15172E+01_JPRB, 0.14999E+01_JPRB, 0.14305E+01_JPRB, 0.14616E+01_JPRB, 0.18155E+01_JPRB ,&
&  0.15114E+01_JPRB, 0.13491E+01_JPRB, 0.11740E+01_JPRB, 0.10899E+01_JPRB, 0.10455E+01_JPRB, 0.10376E+01_JPRB, 0.10649E+01_JPRB ,&
&  0.10778E+01_JPRB, 0.10747E+01_JPRB, 0.10412E+01_JPRB, 0.99544E+00_JPRB, 0.95291E+00_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.86266E+00_JPRB, 0.84110E+00_JPRB, 0.82529E+00_JPRB, 0.79345E+00_JPRB, 0.76850E+00_JPRB, 0.73664E+00_JPRB, 0.69616E+00_JPRB ,&
&  0.67312E+00_JPRB, 0.66040E+00_JPRB, 0.65179E+00_JPRB, 0.63984E+00_JPRB, 0.62744E+00_JPRB, 0.61596E+00_JPRB, 0.61382E+00_JPRB ,&
&  0.59443E+00_JPRB, 0.56744E+00_JPRB, 0.57034E+00_JPRB, 0.58753E+00_JPRB, 0.61172E+00_JPRB, 0.67770E+00_JPRB, 0.90246E+00_JPRB ,&
&  0.14302E+01_JPRB, 0.16601E+01_JPRB, 0.15141E+01_JPRB, 0.13455E+01_JPRB, 0.12504E+01_JPRB, 0.12353E+01_JPRB, 0.11482E+01_JPRB ,&
&  0.10460E+01_JPRB, 0.10071E+01_JPRB, 0.89891E+00_JPRB, 0.89841E+00_JPRB, 0.91261E+00_JPRB, 0.93009E+00_JPRB, 0.94221E+00_JPRB ,&
&  0.97778E+00_JPRB, 0.94327E+00_JPRB, 0.94625E+00_JPRB, 0.96758E+00_JPRB, 0.10002E+01_JPRB, 0.10565E+01_JPRB, 0.10969E+01_JPRB ,&
&  0.11403E+01_JPRB, 0.11957E+01_JPRB, 0.13869E+01_JPRB, 0.17260E+01_JPRB, 0.17749E+01_JPRB, 0.22422E+01_JPRB, 0.25092E+01_JPRB ,&
&  0.27384E+01_JPRB, 0.28606E+01_JPRB, 0.28750E+01_JPRB, 0.34671E+01_JPRB, 0.37833E+01_JPRB, 0.33690E+01_JPRB, 0.28768E+01_JPRB ,&
&  0.28400E+01_JPRB, 0.24431E+01_JPRB, 0.21426E+01_JPRB, 0.19494E+01_JPRB, 0.24389E+01_JPRB, 0.38523E+01_JPRB, 0.30074E+01_JPRB ,&
&  0.28814E+01_JPRB, 0.32255E+01_JPRB, 0.26548E+01_JPRB, 0.22308E+01_JPRB, 0.23484E+01_JPRB, 0.24001E+01_JPRB, 0.18118E+01_JPRB ,&
&  0.86225E+00_JPRB, 0.66794E+00_JPRB, 0.66105E+00_JPRB, 0.64413E+00_JPRB, 0.57647E+00_JPRB, 0.66826E+00_JPRB, 0.14911E+01_JPRB ,&
&  0.12719E+01_JPRB, 0.15714E+01_JPRB, 0.15505E+01_JPRB, 0.14529E+01_JPRB, 0.14246E+01_JPRB, 0.13135E+01_JPRB, 0.14477E+01_JPRB ,&
&  0.12175E+01_JPRB, 0.11013E+01_JPRB, 0.10470E+01_JPRB, 0.99965E+00_JPRB, 0.99066E+00_JPRB, 0.99238E+00_JPRB, 0.10217E+01_JPRB ,&
&  0.10410E+01_JPRB, 0.10449E+01_JPRB, 0.10114E+01_JPRB, 0.95495E+00_JPRB, 0.90392E+00_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.82615E+00_JPRB, 0.81320E+00_JPRB, 0.78739E+00_JPRB, 0.76392E+00_JPRB, 0.75179E+00_JPRB, 0.72262E+00_JPRB, 0.69821E+00_JPRB ,&
&  0.69604E+00_JPRB, 0.67904E+00_JPRB, 0.66054E+00_JPRB, 0.64296E+00_JPRB, 0.62841E+00_JPRB, 0.61113E+00_JPRB, 0.60554E+00_JPRB ,&
&  0.60055E+00_JPRB, 0.57191E+00_JPRB, 0.56805E+00_JPRB, 0.60930E+00_JPRB, 0.73085E+00_JPRB, 0.89095E+00_JPRB, 0.10154E+01_JPRB ,&
&  0.15974E+01_JPRB, 0.17971E+01_JPRB, 0.13988E+01_JPRB, 0.13243E+01_JPRB, 0.13233E+01_JPRB, 0.12842E+01_JPRB, 0.11442E+01_JPRB ,&
&  0.10454E+01_JPRB, 0.10177E+01_JPRB, 0.92209E+00_JPRB, 0.91908E+00_JPRB, 0.94967E+00_JPRB, 0.99746E+00_JPRB, 0.10309E+01_JPRB ,&
&  0.98318E+00_JPRB, 0.98726E+00_JPRB, 0.10289E+01_JPRB, 0.10588E+01_JPRB, 0.10893E+01_JPRB, 0.11529E+01_JPRB, 0.11979E+01_JPRB ,&
&  0.12655E+01_JPRB, 0.13940E+01_JPRB, 0.16477E+01_JPRB, 0.18165E+01_JPRB, 0.20541E+01_JPRB, 0.22387E+01_JPRB, 0.24891E+01_JPRB ,&
&  0.25048E+01_JPRB, 0.26167E+01_JPRB, 0.28461E+01_JPRB, 0.31138E+01_JPRB, 0.32685E+01_JPRB, 0.31099E+01_JPRB, 0.28349E+01_JPRB ,&
&  0.27808E+01_JPRB, 0.23620E+01_JPRB, 0.20298E+01_JPRB, 0.19364E+01_JPRB, 0.22925E+01_JPRB, 0.34619E+01_JPRB, 0.37997E+01_JPRB ,&
&  0.37632E+01_JPRB, 0.27212E+01_JPRB, 0.23306E+01_JPRB, 0.24649E+01_JPRB, 0.22971E+01_JPRB, 0.21686E+01_JPRB, 0.24772E+01_JPRB ,&
&  0.21382E+01_JPRB, 0.12416E+01_JPRB, 0.10336E+01_JPRB, 0.12124E+01_JPRB, 0.82109E+00_JPRB, 0.67943E+00_JPRB, 0.10133E+01_JPRB ,&
&  0.11996E+01_JPRB, 0.14005E+01_JPRB, 0.14033E+01_JPRB, 0.13652E+01_JPRB, 0.13971E+01_JPRB, 0.12198E+01_JPRB, 0.11781E+01_JPRB ,&
&  0.10097E+01_JPRB, 0.98639E+00_JPRB, 0.97129E+00_JPRB, 0.91737E+00_JPRB, 0.90159E+00_JPRB, 0.93302E+00_JPRB, 0.95706E+00_JPRB ,&
&  0.96221E+00_JPRB, 0.95653E+00_JPRB, 0.93166E+00_JPRB, 0.89655E+00_JPRB, 0.85319E+00_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.80364E+00_JPRB, 0.79284E+00_JPRB, 0.78976E+00_JPRB, 0.76832E+00_JPRB, 0.74790E+00_JPRB, 0.72693E+00_JPRB, 0.72948E+00_JPRB ,&
&  0.70794E+00_JPRB, 0.68786E+00_JPRB, 0.67181E+00_JPRB, 0.64549E+00_JPRB, 0.63945E+00_JPRB, 0.62331E+00_JPRB, 0.63153E+00_JPRB ,&
&  0.64315E+00_JPRB, 0.62054E+00_JPRB, 0.64147E+00_JPRB, 0.78649E+00_JPRB, 0.10164E+01_JPRB, 0.14578E+01_JPRB, 0.11762E+01_JPRB ,&
&  0.15220E+01_JPRB, 0.19095E+01_JPRB, 0.14315E+01_JPRB, 0.13828E+01_JPRB, 0.13814E+01_JPRB, 0.13564E+01_JPRB, 0.12157E+01_JPRB ,&
&  0.11133E+01_JPRB, 0.10714E+01_JPRB, 0.10350E+01_JPRB, 0.10075E+01_JPRB, 0.10328E+01_JPRB, 0.10512E+01_JPRB, 0.10142E+01_JPRB ,&
&  0.10281E+01_JPRB, 0.10839E+01_JPRB, 0.11134E+01_JPRB, 0.11372E+01_JPRB, 0.11881E+01_JPRB, 0.12482E+01_JPRB, 0.13125E+01_JPRB ,&
&  0.14379E+01_JPRB, 0.16164E+01_JPRB, 0.18834E+01_JPRB, 0.19387E+01_JPRB, 0.20704E+01_JPRB, 0.22659E+01_JPRB, 0.23817E+01_JPRB ,&
&  0.23033E+01_JPRB, 0.22476E+01_JPRB, 0.25811E+01_JPRB, 0.28858E+01_JPRB, 0.29253E+01_JPRB, 0.28020E+01_JPRB, 0.27176E+01_JPRB ,&
&  0.28158E+01_JPRB, 0.26605E+01_JPRB, 0.26601E+01_JPRB, 0.23143E+01_JPRB, 0.26716E+01_JPRB, 0.34535E+01_JPRB, 0.34662E+01_JPRB ,&
&  0.40329E+01_JPRB, 0.34085E+01_JPRB, 0.30685E+01_JPRB, 0.26309E+01_JPRB, 0.22396E+01_JPRB, 0.20353E+01_JPRB, 0.20222E+01_JPRB ,&
&  0.19742E+01_JPRB, 0.20390E+01_JPRB, 0.18945E+01_JPRB, 0.12157E+01_JPRB, 0.87674E+00_JPRB, 0.74571E+00_JPRB, 0.84619E+00_JPRB ,&
&  0.10777E+01_JPRB, 0.11764E+01_JPRB, 0.11980E+01_JPRB, 0.16667E+01_JPRB, 0.12646E+01_JPRB, 0.10893E+01_JPRB, 0.98239E+00_JPRB ,&
&  0.90749E+00_JPRB, 0.90934E+00_JPRB, 0.88476E+00_JPRB, 0.84640E+00_JPRB, 0.85074E+00_JPRB, 0.86478E+00_JPRB, 0.88087E+00_JPRB ,&
&  0.88460E+00_JPRB, 0.89117E+00_JPRB, 0.89394E+00_JPRB, 0.87364E+00_JPRB, 0.83561E+00_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.78223E+00_JPRB, 0.78346E+00_JPRB, 0.77087E+00_JPRB, 0.75697E+00_JPRB, 0.75002E+00_JPRB, 0.74910E+00_JPRB, 0.76413E+00_JPRB ,&
&  0.70379E+00_JPRB, 0.68091E+00_JPRB, 0.67172E+00_JPRB, 0.66170E+00_JPRB, 0.67401E+00_JPRB, 0.68945E+00_JPRB, 0.71810E+00_JPRB ,&
&  0.75566E+00_JPRB, 0.79286E+00_JPRB, 0.91767E+00_JPRB, 0.11871E+01_JPRB, 0.14938E+01_JPRB, 0.18728E+01_JPRB, 0.17764E+01_JPRB ,&
&  0.14727E+01_JPRB, 0.20781E+01_JPRB, 0.14615E+01_JPRB, 0.14963E+01_JPRB, 0.14619E+01_JPRB, 0.15500E+01_JPRB, 0.14532E+01_JPRB ,&
&  0.13006E+01_JPRB, 0.12290E+01_JPRB, 0.11912E+01_JPRB, 0.11054E+01_JPRB, 0.10916E+01_JPRB, 0.10656E+01_JPRB, 0.10703E+01_JPRB ,&
&  0.11256E+01_JPRB, 0.11839E+01_JPRB, 0.12219E+01_JPRB, 0.12666E+01_JPRB, 0.13287E+01_JPRB, 0.13835E+01_JPRB, 0.14946E+01_JPRB ,&
&  0.16847E+01_JPRB, 0.18988E+01_JPRB, 0.19839E+01_JPRB, 0.20164E+01_JPRB, 0.20486E+01_JPRB, 0.22186E+01_JPRB, 0.23354E+01_JPRB ,&
&  0.23232E+01_JPRB, 0.22701E+01_JPRB, 0.24751E+01_JPRB, 0.25258E+01_JPRB, 0.25042E+01_JPRB, 0.26738E+01_JPRB, 0.26171E+01_JPRB ,&
&  0.28474E+01_JPRB, 0.26437E+01_JPRB, 0.29977E+01_JPRB, 0.27617E+01_JPRB, 0.28648E+01_JPRB, 0.31457E+01_JPRB, 0.27394E+01_JPRB ,&
&  0.26025E+01_JPRB, 0.26993E+01_JPRB, 0.26237E+01_JPRB, 0.24730E+01_JPRB, 0.22340E+01_JPRB, 0.20081E+01_JPRB, 0.17542E+01_JPRB ,&
&  0.15062E+01_JPRB, 0.14254E+01_JPRB, 0.13176E+01_JPRB, 0.90212E+00_JPRB, 0.74181E+00_JPRB, 0.69000E+00_JPRB, 0.77042E+00_JPRB ,&
&  0.97836E+00_JPRB, 0.11363E+01_JPRB, 0.13820E+01_JPRB, 0.13454E+01_JPRB, 0.10447E+01_JPRB, 0.89081E+00_JPRB, 0.86162E+00_JPRB ,&
&  0.83587E+00_JPRB, 0.82305E+00_JPRB, 0.77801E+00_JPRB, 0.75882E+00_JPRB, 0.77827E+00_JPRB, 0.80921E+00_JPRB, 0.81599E+00_JPRB ,&
&  0.83474E+00_JPRB, 0.84235E+00_JPRB, 0.83040E+00_JPRB, 0.80526E+00_JPRB, 0.79011E+00_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.83887E+00_JPRB, 0.85966E+00_JPRB, 0.88206E+00_JPRB, 0.92462E+00_JPRB, 0.98584E+00_JPRB, 0.10646E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.92427E+00_JPRB, 0.68273E+00_JPRB, 0.69912E+00_JPRB, 0.73203E+00_JPRB, 0.79372E+00_JPRB, 0.85220E+00_JPRB, 0.92017E+00_JPRB ,&
&  0.10200E+01_JPRB, 0.11546E+01_JPRB, 0.13857E+01_JPRB, 0.16630E+01_JPRB, 0.19362E+01_JPRB, 0.22811E+01_JPRB, 0.22310E+01_JPRB ,&
&  0.17782E+01_JPRB, 0.15590E+01_JPRB, 0.16871E+01_JPRB, 0.16754E+01_JPRB, 0.15430E+01_JPRB, 0.16541E+01_JPRB, 0.16259E+01_JPRB ,&
&  0.14476E+01_JPRB, 0.13539E+01_JPRB, 0.13048E+01_JPRB, 0.12300E+01_JPRB, 0.11868E+01_JPRB, 0.12169E+01_JPRB, 0.12709E+01_JPRB ,&
&  0.13417E+01_JPRB, 0.13889E+01_JPRB, 0.14111E+01_JPRB, 0.14356E+01_JPRB, 0.14934E+01_JPRB, 0.15579E+01_JPRB, 0.17277E+01_JPRB ,&
&  0.19281E+01_JPRB, 0.20827E+01_JPRB, 0.21022E+01_JPRB, 0.20150E+01_JPRB, 0.20616E+01_JPRB, 0.21668E+01_JPRB, 0.22298E+01_JPRB ,&
&  0.22702E+01_JPRB, 0.23501E+01_JPRB, 0.24077E+01_JPRB, 0.24320E+01_JPRB, 0.23201E+01_JPRB, 0.25870E+01_JPRB, 0.25604E+01_JPRB ,&
&  0.26705E+01_JPRB, 0.25560E+01_JPRB, 0.28982E+01_JPRB, 0.25626E+01_JPRB, 0.27881E+01_JPRB, 0.27073E+01_JPRB, 0.24146E+01_JPRB ,&
&  0.23092E+01_JPRB, 0.24482E+01_JPRB, 0.24688E+01_JPRB, 0.23267E+01_JPRB, 0.21943E+01_JPRB, 0.17952E+01_JPRB, 0.17133E+01_JPRB ,&
&  0.13438E+01_JPRB, 0.13641E+01_JPRB, 0.10242E+01_JPRB, 0.77801E+00_JPRB, 0.63351E+00_JPRB, 0.67239E+00_JPRB, 0.77698E+00_JPRB ,&
&  0.10006E+01_JPRB, 0.10803E+01_JPRB, 0.10749E+01_JPRB, 0.10835E+01_JPRB, 0.97055E+00_JPRB, 0.83369E+00_JPRB, 0.81755E+00_JPRB ,&
&  0.82366E+00_JPRB, 0.79249E+00_JPRB, 0.74367E+00_JPRB, 0.74247E+00_JPRB, 0.76372E+00_JPRB, 0.80852E+00_JPRB, 0.81439E+00_JPRB ,&
&  0.82716E+00_JPRB, 0.82223E+00_JPRB, 0.81069E+00_JPRB, 0.80458E+00_JPRB, 0.81468E+00_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.88427E+00_JPRB, 0.88473E+00_JPRB, 0.90174E+00_JPRB, 0.90981E+00_JPRB, 0.91056E+00_JPRB, 0.86129E+00_JPRB, 0.82290E+00_JPRB ,&
&  0.77418E+00_JPRB, 0.72763E+00_JPRB, 0.79234E+00_JPRB, 0.87470E+00_JPRB, 0.97290E+00_JPRB, 0.10784E+01_JPRB, 0.11883E+01_JPRB ,&
&  0.13337E+01_JPRB, 0.15146E+01_JPRB, 0.17399E+01_JPRB, 0.19165E+01_JPRB, 0.20771E+01_JPRB, 0.22991E+01_JPRB, 0.24777E+01_JPRB ,&
&  0.20829E+01_JPRB, 0.14330E+01_JPRB, 0.17585E+01_JPRB, 0.19177E+01_JPRB, 0.17094E+01_JPRB, 0.16940E+01_JPRB, 0.16481E+01_JPRB ,&
&  0.15376E+01_JPRB, 0.14817E+01_JPRB, 0.14575E+01_JPRB, 0.14354E+01_JPRB, 0.14137E+01_JPRB, 0.14581E+01_JPRB, 0.15031E+01_JPRB ,&
&  0.15243E+01_JPRB, 0.15141E+01_JPRB, 0.15055E+01_JPRB, 0.15739E+01_JPRB, 0.17140E+01_JPRB, 0.17895E+01_JPRB, 0.19319E+01_JPRB ,&
&  0.20574E+01_JPRB, 0.22122E+01_JPRB, 0.22519E+01_JPRB, 0.21956E+01_JPRB, 0.22532E+01_JPRB, 0.23026E+01_JPRB, 0.23008E+01_JPRB ,&
&  0.22941E+01_JPRB, 0.22918E+01_JPRB, 0.23606E+01_JPRB, 0.25767E+01_JPRB, 0.26983E+01_JPRB, 0.24485E+01_JPRB, 0.25329E+01_JPRB ,&
&  0.25794E+01_JPRB, 0.26210E+01_JPRB, 0.25359E+01_JPRB, 0.32204E+01_JPRB, 0.24224E+01_JPRB, 0.23708E+01_JPRB, 0.20220E+01_JPRB ,&
&  0.22125E+01_JPRB, 0.22174E+01_JPRB, 0.22656E+01_JPRB, 0.22592E+01_JPRB, 0.20955E+01_JPRB, 0.17277E+01_JPRB, 0.16517E+01_JPRB ,&
&  0.14374E+01_JPRB, 0.12910E+01_JPRB, 0.99930E+00_JPRB, 0.79271E+00_JPRB, 0.73288E+00_JPRB, 0.94770E+00_JPRB, 0.99377E+00_JPRB ,&
&  0.10253E+01_JPRB, 0.99789E+00_JPRB, 0.11097E+01_JPRB, 0.11660E+01_JPRB, 0.10487E+01_JPRB, 0.81291E+00_JPRB, 0.77279E+00_JPRB ,&
&  0.78019E+00_JPRB, 0.77923E+00_JPRB, 0.75256E+00_JPRB, 0.75873E+00_JPRB, 0.78625E+00_JPRB, 0.83378E+00_JPRB, 0.85772E+00_JPRB ,&
&  0.85983E+00_JPRB, 0.84348E+00_JPRB, 0.84172E+00_JPRB, 0.84998E+00_JPRB, 0.87768E+00_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.85519E+00_JPRB, 0.84422E+00_JPRB, 0.83172E+00_JPRB, 0.81138E+00_JPRB, 0.78230E+00_JPRB, 0.76249E+00_JPRB, 0.76506E+00_JPRB ,&
&  0.77247E+00_JPRB, 0.81172E+00_JPRB, 0.87933E+00_JPRB, 0.96280E+00_JPRB, 0.10741E+01_JPRB, 0.12018E+01_JPRB, 0.13375E+01_JPRB ,&
&  0.15160E+01_JPRB, 0.16958E+01_JPRB, 0.18149E+01_JPRB, 0.19113E+01_JPRB, 0.20270E+01_JPRB, 0.22154E+01_JPRB, 0.24221E+01_JPRB ,&
&  0.25785E+01_JPRB, 0.25472E+01_JPRB, 0.23652E+01_JPRB, 0.17303E+01_JPRB, 0.14490E+01_JPRB, 0.16852E+01_JPRB, 0.16813E+01_JPRB ,&
&  0.16600E+01_JPRB, 0.16081E+01_JPRB, 0.15774E+01_JPRB, 0.15479E+01_JPRB, 0.15560E+01_JPRB, 0.15900E+01_JPRB, 0.16119E+01_JPRB ,&
&  0.16081E+01_JPRB, 0.16126E+01_JPRB, 0.16499E+01_JPRB, 0.17468E+01_JPRB, 0.19028E+01_JPRB, 0.19959E+01_JPRB, 0.20968E+01_JPRB ,&
&  0.21829E+01_JPRB, 0.23300E+01_JPRB, 0.24165E+01_JPRB, 0.24650E+01_JPRB, 0.24509E+01_JPRB, 0.24791E+01_JPRB, 0.25490E+01_JPRB ,&
&  0.25676E+01_JPRB, 0.26041E+01_JPRB, 0.27038E+01_JPRB, 0.28810E+01_JPRB, 0.29531E+01_JPRB, 0.25996E+01_JPRB, 0.25113E+01_JPRB ,&
&  0.26734E+01_JPRB, 0.28571E+01_JPRB, 0.26381E+01_JPRB, 0.33312E+01_JPRB, 0.21185E+01_JPRB, 0.22332E+01_JPRB, 0.21521E+01_JPRB ,&
&  0.20300E+01_JPRB, 0.19630E+01_JPRB, 0.20383E+01_JPRB, 0.20781E+01_JPRB, 0.19933E+01_JPRB, 0.16265E+01_JPRB, 0.16330E+01_JPRB ,&
&  0.15491E+01_JPRB, 0.12160E+01_JPRB, 0.97803E+00_JPRB, 0.74649E+00_JPRB, 0.83626E+00_JPRB, 0.11186E+01_JPRB, 0.10147E+01_JPRB ,&
&  0.85336E+00_JPRB, 0.10337E+01_JPRB, 0.11609E+01_JPRB, 0.12602E+01_JPRB, 0.11884E+01_JPRB, 0.74450E+00_JPRB, 0.74102E+00_JPRB ,&
&  0.76508E+00_JPRB, 0.78074E+00_JPRB, 0.79871E+00_JPRB, 0.80067E+00_JPRB, 0.81679E+00_JPRB, 0.85299E+00_JPRB, 0.88516E+00_JPRB ,&
&  0.89343E+00_JPRB, 0.88520E+00_JPRB, 0.88857E+00_JPRB, 0.88055E+00_JPRB, 0.86727E+00_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.79087E+00_JPRB, 0.76106E+00_JPRB, 0.73720E+00_JPRB, 0.72658E+00_JPRB, 0.72638E+00_JPRB, 0.73813E+00_JPRB, 0.75969E+00_JPRB ,&
&  0.79648E+00_JPRB, 0.84734E+00_JPRB, 0.89978E+00_JPRB, 0.94627E+00_JPRB, 0.10139E+01_JPRB, 0.10986E+01_JPRB, 0.12098E+01_JPRB ,&
&  0.13345E+01_JPRB, 0.14680E+01_JPRB, 0.15399E+01_JPRB, 0.15741E+01_JPRB, 0.16426E+01_JPRB, 0.17509E+01_JPRB, 0.19887E+01_JPRB ,&
&  0.23182E+01_JPRB, 0.25899E+01_JPRB, 0.27213E+01_JPRB, 0.24698E+01_JPRB, 0.18714E+01_JPRB, 0.17864E+01_JPRB, 0.19392E+01_JPRB ,&
&  0.19903E+01_JPRB, 0.17975E+01_JPRB, 0.16693E+01_JPRB, 0.16494E+01_JPRB, 0.16497E+01_JPRB, 0.16831E+01_JPRB, 0.17050E+01_JPRB ,&
&  0.17105E+01_JPRB, 0.17274E+01_JPRB, 0.17835E+01_JPRB, 0.18579E+01_JPRB, 0.19750E+01_JPRB, 0.20993E+01_JPRB, 0.22387E+01_JPRB ,&
&  0.23018E+01_JPRB, 0.23838E+01_JPRB, 0.23857E+01_JPRB, 0.23259E+01_JPRB, 0.23486E+01_JPRB, 0.25043E+01_JPRB, 0.27475E+01_JPRB ,&
&  0.29007E+01_JPRB, 0.28697E+01_JPRB, 0.30459E+01_JPRB, 0.32536E+01_JPRB, 0.32039E+01_JPRB, 0.29473E+01_JPRB, 0.28980E+01_JPRB ,&
&  0.29994E+01_JPRB, 0.33093E+01_JPRB, 0.24345E+01_JPRB, 0.28038E+01_JPRB, 0.28893E+01_JPRB, 0.24540E+01_JPRB, 0.19919E+01_JPRB ,&
&  0.18121E+01_JPRB, 0.16087E+01_JPRB, 0.14634E+01_JPRB, 0.15009E+01_JPRB, 0.15312E+01_JPRB, 0.15353E+01_JPRB, 0.14597E+01_JPRB ,&
&  0.14235E+01_JPRB, 0.11558E+01_JPRB, 0.90084E+00_JPRB, 0.76013E+00_JPRB, 0.81653E+00_JPRB, 0.97758E+00_JPRB, 0.91764E+00_JPRB ,&
&  0.86246E+00_JPRB, 0.99747E+00_JPRB, 0.10760E+01_JPRB, 0.13053E+01_JPRB, 0.14197E+01_JPRB, 0.73247E+00_JPRB, 0.74145E+00_JPRB ,&
&  0.75766E+00_JPRB, 0.78913E+00_JPRB, 0.79871E+00_JPRB, 0.80218E+00_JPRB, 0.80550E+00_JPRB, 0.82888E+00_JPRB, 0.85697E+00_JPRB ,&
&  0.86813E+00_JPRB, 0.86948E+00_JPRB, 0.86757E+00_JPRB, 0.84916E+00_JPRB, 0.81625E+00_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.72111E+00_JPRB, 0.71997E+00_JPRB, 0.70908E+00_JPRB, 0.71992E+00_JPRB, 0.73345E+00_JPRB, 0.73035E+00_JPRB, 0.74468E+00_JPRB ,&
&  0.76573E+00_JPRB, 0.78642E+00_JPRB, 0.79824E+00_JPRB, 0.80523E+00_JPRB, 0.85324E+00_JPRB, 0.91283E+00_JPRB, 0.98517E+00_JPRB ,&
&  0.10268E+01_JPRB, 0.10665E+01_JPRB, 0.10762E+01_JPRB, 0.10643E+01_JPRB, 0.10577E+01_JPRB, 0.10230E+01_JPRB, 0.97689E+00_JPRB ,&
&  0.10410E+01_JPRB, 0.13210E+01_JPRB, 0.17697E+01_JPRB, 0.21699E+01_JPRB, 0.25118E+01_JPRB, 0.21025E+01_JPRB, 0.21769E+01_JPRB ,&
&  0.18882E+01_JPRB, 0.15283E+01_JPRB, 0.15633E+01_JPRB, 0.16568E+01_JPRB, 0.17401E+01_JPRB, 0.17347E+01_JPRB, 0.17677E+01_JPRB ,&
&  0.17894E+01_JPRB, 0.18123E+01_JPRB, 0.18526E+01_JPRB, 0.19413E+01_JPRB, 0.20205E+01_JPRB, 0.20835E+01_JPRB, 0.21672E+01_JPRB ,&
&  0.22275E+01_JPRB, 0.22524E+01_JPRB, 0.22886E+01_JPRB, 0.21359E+01_JPRB, 0.22749E+01_JPRB, 0.25619E+01_JPRB, 0.28118E+01_JPRB ,&
&  0.29258E+01_JPRB, 0.29775E+01_JPRB, 0.32291E+01_JPRB, 0.33373E+01_JPRB, 0.34056E+01_JPRB, 0.33559E+01_JPRB, 0.33539E+01_JPRB ,&
&  0.33682E+01_JPRB, 0.30857E+01_JPRB, 0.22548E+01_JPRB, 0.23961E+01_JPRB, 0.21495E+01_JPRB, 0.17306E+01_JPRB, 0.17206E+01_JPRB ,&
&  0.15591E+01_JPRB, 0.14074E+01_JPRB, 0.12505E+01_JPRB, 0.12151E+01_JPRB, 0.12316E+01_JPRB, 0.12628E+01_JPRB, 0.12963E+01_JPRB ,&
&  0.11630E+01_JPRB, 0.99374E+00_JPRB, 0.82860E+00_JPRB, 0.78664E+00_JPRB, 0.75882E+00_JPRB, 0.80384E+00_JPRB, 0.84302E+00_JPRB ,&
&  0.87875E+00_JPRB, 0.80263E+00_JPRB, 0.81665E+00_JPRB, 0.86328E+00_JPRB, 0.72510E+00_JPRB, 0.68820E+00_JPRB, 0.69190E+00_JPRB ,&
&  0.70188E+00_JPRB, 0.70959E+00_JPRB, 0.71461E+00_JPRB, 0.71383E+00_JPRB, 0.72362E+00_JPRB, 0.74062E+00_JPRB, 0.75021E+00_JPRB ,&
&  0.75326E+00_JPRB, 0.74050E+00_JPRB, 0.73082E+00_JPRB, 0.72411E+00_JPRB, 0.71764E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.60815E+00_JPRB, 0.60955E+00_JPRB, 0.61135E+00_JPRB, 0.61860E+00_JPRB, 0.61844E+00_JPRB, 0.61399E+00_JPRB, 0.60237E+00_JPRB ,&
&  0.59513E+00_JPRB, 0.58082E+00_JPRB, 0.57079E+00_JPRB, 0.56383E+00_JPRB, 0.57612E+00_JPRB, 0.60682E+00_JPRB, 0.65500E+00_JPRB ,&
&  0.66905E+00_JPRB, 0.66659E+00_JPRB, 0.64252E+00_JPRB, 0.62947E+00_JPRB, 0.65248E+00_JPRB, 0.65719E+00_JPRB, 0.63430E+00_JPRB ,&
&  0.61382E+00_JPRB, 0.67363E+00_JPRB, 0.83460E+00_JPRB, 0.10634E+01_JPRB, 0.14702E+01_JPRB, 0.16708E+01_JPRB, 0.20962E+01_JPRB ,&
&  0.18999E+01_JPRB, 0.13913E+01_JPRB, 0.14951E+01_JPRB, 0.14389E+01_JPRB, 0.15333E+01_JPRB, 0.15842E+01_JPRB, 0.16013E+01_JPRB ,&
&  0.16486E+01_JPRB, 0.16931E+01_JPRB, 0.17358E+01_JPRB, 0.17835E+01_JPRB, 0.18173E+01_JPRB, 0.18172E+01_JPRB, 0.18250E+01_JPRB ,&
&  0.18250E+01_JPRB, 0.18058E+01_JPRB, 0.18487E+01_JPRB, 0.19700E+01_JPRB, 0.22552E+01_JPRB, 0.25807E+01_JPRB, 0.26258E+01_JPRB ,&
&  0.26452E+01_JPRB, 0.28239E+01_JPRB, 0.27652E+01_JPRB, 0.29441E+01_JPRB, 0.30219E+01_JPRB, 0.29333E+01_JPRB, 0.31296E+01_JPRB ,&
&  0.32468E+01_JPRB, 0.27286E+01_JPRB, 0.18291E+01_JPRB, 0.19075E+01_JPRB, 0.17587E+01_JPRB, 0.15971E+01_JPRB, 0.15970E+01_JPRB ,&
&  0.14730E+01_JPRB, 0.13660E+01_JPRB, 0.12567E+01_JPRB, 0.11953E+01_JPRB, 0.11564E+01_JPRB, 0.10862E+01_JPRB, 0.10584E+01_JPRB ,&
&  0.96720E+00_JPRB, 0.91106E+00_JPRB, 0.87428E+00_JPRB, 0.83838E+00_JPRB, 0.81997E+00_JPRB, 0.82475E+00_JPRB, 0.76412E+00_JPRB ,&
&  0.78360E+00_JPRB, 0.73003E+00_JPRB, 0.78752E+00_JPRB, 0.82426E+00_JPRB, 0.62266E+00_JPRB, 0.60298E+00_JPRB, 0.58201E+00_JPRB ,&
&  0.58400E+00_JPRB, 0.57878E+00_JPRB, 0.57606E+00_JPRB, 0.56951E+00_JPRB, 0.59120E+00_JPRB, 0.62424E+00_JPRB, 0.62268E+00_JPRB ,&
&  0.63152E+00_JPRB, 0.61467E+00_JPRB, 0.61703E+00_JPRB, 0.62364E+00_JPRB, 0.62693E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.58104E+00_JPRB, 0.57037E+00_JPRB, 0.55978E+00_JPRB, 0.54893E+00_JPRB, 0.55444E+00_JPRB, 0.55480E+00_JPRB, 0.56384E+00_JPRB ,&
&  0.57430E+00_JPRB, 0.57332E+00_JPRB, 0.57715E+00_JPRB, 0.57815E+00_JPRB, 0.59192E+00_JPRB, 0.60407E+00_JPRB, 0.63429E+00_JPRB ,&
&  0.66264E+00_JPRB, 0.67711E+00_JPRB, 0.69736E+00_JPRB, 0.71681E+00_JPRB, 0.73108E+00_JPRB, 0.74581E+00_JPRB, 0.72997E+00_JPRB ,&
&  0.71133E+00_JPRB, 0.71106E+00_JPRB, 0.74398E+00_JPRB, 0.86347E+00_JPRB, 0.10919E+01_JPRB, 0.13913E+01_JPRB, 0.19786E+01_JPRB ,&
&  0.18876E+01_JPRB, 0.12774E+01_JPRB, 0.11538E+01_JPRB, 0.10215E+01_JPRB, 0.10701E+01_JPRB, 0.11511E+01_JPRB, 0.11544E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.11592E+01_JPRB, 0.11500E+01_JPRB, 0.11282E+01_JPRB, 0.10731E+01_JPRB, 0.10046E+01_JPRB, 0.96530E+00_JPRB ,&
&  0.96237E+00_JPRB, 0.95550E+00_JPRB, 0.10874E+01_JPRB, 0.14375E+01_JPRB, 0.16911E+01_JPRB, 0.21391E+01_JPRB, 0.20288E+01_JPRB ,&
&  0.20442E+01_JPRB, 0.21105E+01_JPRB, 0.20562E+01_JPRB, 0.21946E+01_JPRB, 0.22567E+01_JPRB, 0.23454E+01_JPRB, 0.23510E+01_JPRB ,&
&  0.24920E+01_JPRB, 0.21870E+01_JPRB, 0.15634E+01_JPRB, 0.16886E+01_JPRB, 0.15822E+01_JPRB, 0.15658E+01_JPRB, 0.14588E+01_JPRB ,&
&  0.13736E+01_JPRB, 0.12757E+01_JPRB, 0.11978E+01_JPRB, 0.11357E+01_JPRB, 0.11078E+01_JPRB, 0.10789E+01_JPRB, 0.98726E+00_JPRB ,&
&  0.95235E+00_JPRB, 0.91834E+00_JPRB, 0.89522E+00_JPRB, 0.84328E+00_JPRB, 0.87991E+00_JPRB, 0.74641E+00_JPRB, 0.91450E+00_JPRB ,&
&  0.69439E+00_JPRB, 0.63769E+00_JPRB, 0.54212E+00_JPRB, 0.66601E+00_JPRB, 0.58931E+00_JPRB, 0.54083E+00_JPRB, 0.54673E+00_JPRB ,&
&  0.52960E+00_JPRB, 0.51094E+00_JPRB, 0.49662E+00_JPRB, 0.49799E+00_JPRB, 0.51131E+00_JPRB, 0.53404E+00_JPRB, 0.53784E+00_JPRB ,&
&  0.54715E+00_JPRB, 0.56085E+00_JPRB, 0.57983E+00_JPRB, 0.60112E+00_JPRB, 0.59856E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.59644E+00_JPRB, 0.59607E+00_JPRB, 0.60350E+00_JPRB, 0.61778E+00_JPRB, 0.62443E+00_JPRB, 0.62903E+00_JPRB, 0.63200E+00_JPRB ,&
&  0.64871E+00_JPRB, 0.66806E+00_JPRB, 0.68605E+00_JPRB, 0.70395E+00_JPRB, 0.71960E+00_JPRB, 0.73316E+00_JPRB, 0.76766E+00_JPRB ,&
&  0.78496E+00_JPRB, 0.80945E+00_JPRB, 0.82636E+00_JPRB, 0.84424E+00_JPRB, 0.85972E+00_JPRB, 0.85277E+00_JPRB, 0.85680E+00_JPRB ,&
&  0.86563E+00_JPRB, 0.89172E+00_JPRB, 0.91861E+00_JPRB, 0.96600E+00_JPRB, 0.10278E+01_JPRB, 0.10388E+01_JPRB, 0.11600E+01_JPRB ,&
&  0.11123E+01_JPRB, 0.12667E+01_JPRB, 0.11100E+01_JPRB, 0.94906E+00_JPRB, 0.90978E+00_JPRB, 0.82107E+00_JPRB, 0.77723E+00_JPRB ,&
&  0.79037E+00_JPRB, 0.78921E+00_JPRB, 0.76900E+00_JPRB, 0.74201E+00_JPRB, 0.71074E+00_JPRB, 0.76176E+00_JPRB, 0.83975E+00_JPRB ,&
&  0.99887E+00_JPRB, 0.10906E+01_JPRB, 0.12127E+01_JPRB, 0.13181E+01_JPRB, 0.14523E+01_JPRB, 0.15166E+01_JPRB, 0.14909E+01_JPRB ,&
&  0.14848E+01_JPRB, 0.14631E+01_JPRB, 0.13675E+01_JPRB, 0.15588E+01_JPRB, 0.17112E+01_JPRB, 0.17836E+01_JPRB, 0.18011E+01_JPRB ,&
&  0.17824E+01_JPRB, 0.15929E+01_JPRB, 0.15575E+01_JPRB, 0.14814E+01_JPRB, 0.13891E+01_JPRB, 0.13727E+01_JPRB, 0.12908E+01_JPRB ,&
&  0.12085E+01_JPRB, 0.11505E+01_JPRB, 0.11285E+01_JPRB, 0.11278E+01_JPRB, 0.11311E+01_JPRB, 0.11230E+01_JPRB, 0.10330E+01_JPRB ,&
&  0.96685E+00_JPRB, 0.91027E+00_JPRB, 0.86676E+00_JPRB, 0.82609E+00_JPRB, 0.72182E+00_JPRB, 0.99806E+00_JPRB, 0.13190E+01_JPRB ,&
&  0.62000E+00_JPRB, 0.48941E+00_JPRB, 0.39800E+00_JPRB, 0.51197E+00_JPRB, 0.53177E+00_JPRB, 0.57151E+00_JPRB, 0.64245E+00_JPRB ,&
&  0.65813E+00_JPRB, 0.64672E+00_JPRB, 0.58423E+00_JPRB, 0.52349E+00_JPRB, 0.53706E+00_JPRB, 0.53914E+00_JPRB, 0.54172E+00_JPRB ,&
&  0.55611E+00_JPRB, 0.56525E+00_JPRB, 0.57671E+00_JPRB, 0.59658E+00_JPRB, 0.59204E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.62101E+00_JPRB, 0.63420E+00_JPRB, 0.65180E+00_JPRB, 0.67538E+00_JPRB, 0.69771E+00_JPRB, 0.71491E+00_JPRB, 0.72633E+00_JPRB ,&
&  0.74518E+00_JPRB, 0.77004E+00_JPRB, 0.78480E+00_JPRB, 0.79201E+00_JPRB, 0.80856E+00_JPRB, 0.82362E+00_JPRB, 0.85088E+00_JPRB ,&
&  0.86766E+00_JPRB, 0.90630E+00_JPRB, 0.93581E+00_JPRB, 0.96547E+00_JPRB, 0.98831E+00_JPRB, 0.10160E+01_JPRB, 0.10421E+01_JPRB ,&
&  0.10634E+01_JPRB, 0.11055E+01_JPRB, 0.11421E+01_JPRB, 0.11688E+01_JPRB, 0.12121E+01_JPRB, 0.12448E+01_JPRB, 0.10203E+01_JPRB ,&
&  0.12323E+01_JPRB, 0.11760E+01_JPRB, 0.10415E+01_JPRB, 0.93613E+00_JPRB, 0.86489E+00_JPRB, 0.82061E+00_JPRB, 0.80169E+00_JPRB ,&
&  0.82054E+00_JPRB, 0.89826E+00_JPRB, 0.95831E+00_JPRB, 0.96582E+00_JPRB, 0.95103E+00_JPRB, 0.10501E+01_JPRB, 0.12511E+01_JPRB ,&
&  0.13669E+01_JPRB, 0.14277E+01_JPRB, 0.14705E+01_JPRB, 0.14707E+01_JPRB, 0.15279E+01_JPRB, 0.16179E+01_JPRB, 0.16586E+01_JPRB ,&
&  0.16464E+01_JPRB, 0.15833E+01_JPRB, 0.14046E+01_JPRB, 0.16847E+01_JPRB, 0.16766E+01_JPRB, 0.16036E+01_JPRB, 0.14477E+01_JPRB ,&
&  0.12834E+01_JPRB, 0.11416E+01_JPRB, 0.11878E+01_JPRB, 0.12750E+01_JPRB, 0.12208E+01_JPRB, 0.11740E+01_JPRB, 0.10841E+01_JPRB ,&
&  0.10141E+01_JPRB, 0.99081E+00_JPRB, 0.10080E+01_JPRB, 0.10479E+01_JPRB, 0.10760E+01_JPRB, 0.10696E+01_JPRB, 0.99703E+00_JPRB ,&
&  0.92459E+00_JPRB, 0.84306E+00_JPRB, 0.78481E+00_JPRB, 0.73561E+00_JPRB, 0.67131E+00_JPRB, 0.62420E+00_JPRB, 0.83846E+00_JPRB ,&
&  0.54477E+00_JPRB, 0.38186E+00_JPRB, 0.42364E+00_JPRB, 0.47871E+00_JPRB, 0.52594E+00_JPRB, 0.59025E+00_JPRB, 0.75313E+00_JPRB ,&
&  0.96352E+00_JPRB, 0.11115E+01_JPRB, 0.10879E+01_JPRB, 0.82527E+00_JPRB, 0.64549E+00_JPRB, 0.58899E+00_JPRB, 0.58781E+00_JPRB ,&
&  0.59657E+00_JPRB, 0.59594E+00_JPRB, 0.59958E+00_JPRB, 0.60479E+00_JPRB, 0.61385E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.60613E+00_JPRB, 0.61561E+00_JPRB, 0.62830E+00_JPRB, 0.64198E+00_JPRB, 0.65858E+00_JPRB, 0.67538E+00_JPRB, 0.68847E+00_JPRB ,&
&  0.70192E+00_JPRB, 0.71782E+00_JPRB, 0.73271E+00_JPRB, 0.74285E+00_JPRB, 0.75406E+00_JPRB, 0.77591E+00_JPRB, 0.80164E+00_JPRB ,&
&  0.81193E+00_JPRB, 0.85260E+00_JPRB, 0.86949E+00_JPRB, 0.90650E+00_JPRB, 0.93952E+00_JPRB, 0.98063E+00_JPRB, 0.10108E+01_JPRB ,&
&  0.10386E+01_JPRB, 0.10801E+01_JPRB, 0.10907E+01_JPRB, 0.10776E+01_JPRB, 0.10853E+01_JPRB, 0.11468E+01_JPRB, 0.92695E+00_JPRB ,&
&  0.11804E+01_JPRB, 0.11659E+01_JPRB, 0.10622E+01_JPRB, 0.10170E+01_JPRB, 0.98439E+00_JPRB, 0.99116E+00_JPRB, 0.10169E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.11894E+01_JPRB, 0.12073E+01_JPRB, 0.12548E+01_JPRB, 0.13404E+01_JPRB, 0.15130E+01_JPRB, 0.16720E+01_JPRB ,&
&  0.17256E+01_JPRB, 0.18467E+01_JPRB, 0.18992E+01_JPRB, 0.19526E+01_JPRB, 0.20482E+01_JPRB, 0.21457E+01_JPRB, 0.22110E+01_JPRB ,&
&  0.21814E+01_JPRB, 0.21298E+01_JPRB, 0.19603E+01_JPRB, 0.20884E+01_JPRB, 0.19744E+01_JPRB, 0.16174E+01_JPRB, 0.12780E+01_JPRB ,&
&  0.94677E+00_JPRB, 0.97204E+00_JPRB, 0.10327E+01_JPRB, 0.10894E+01_JPRB, 0.10206E+01_JPRB, 0.96061E+00_JPRB, 0.83555E+00_JPRB ,&
&  0.78171E+00_JPRB, 0.76855E+00_JPRB, 0.80980E+00_JPRB, 0.86790E+00_JPRB, 0.90329E+00_JPRB, 0.89134E+00_JPRB, 0.83236E+00_JPRB ,&
&  0.77650E+00_JPRB, 0.70657E+00_JPRB, 0.66762E+00_JPRB, 0.64249E+00_JPRB, 0.59003E+00_JPRB, 0.51299E+00_JPRB, 0.61916E+00_JPRB ,&
&  0.52302E+00_JPRB, 0.47871E+00_JPRB, 0.56025E+00_JPRB, 0.40180E+00_JPRB, 0.58698E+00_JPRB, 0.74693E+00_JPRB, 0.10097E+01_JPRB ,&
&  0.12191E+01_JPRB, 0.17301E+01_JPRB, 0.21846E+01_JPRB, 0.16597E+01_JPRB, 0.11402E+01_JPRB, 0.64826E+00_JPRB, 0.55904E+00_JPRB ,&
&  0.56768E+00_JPRB, 0.57356E+00_JPRB, 0.58253E+00_JPRB, 0.58306E+00_JPRB, 0.59501E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.56378E+00_JPRB, 0.55862E+00_JPRB, 0.55832E+00_JPRB, 0.54933E+00_JPRB, 0.54475E+00_JPRB, 0.54578E+00_JPRB, 0.53407E+00_JPRB ,&
&  0.54291E+00_JPRB, 0.55865E+00_JPRB, 0.56407E+00_JPRB, 0.58119E+00_JPRB, 0.59164E+00_JPRB, 0.62717E+00_JPRB, 0.67882E+00_JPRB ,&
&  0.71348E+00_JPRB, 0.79053E+00_JPRB, 0.82607E+00_JPRB, 0.87046E+00_JPRB, 0.90714E+00_JPRB, 0.94490E+00_JPRB, 0.97725E+00_JPRB ,&
&  0.10249E+01_JPRB, 0.10813E+01_JPRB, 0.11421E+01_JPRB, 0.11826E+01_JPRB, 0.11840E+01_JPRB, 0.12812E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.12013E+01_JPRB, 0.12249E+01_JPRB, 0.11641E+01_JPRB, 0.11430E+01_JPRB, 0.11262E+01_JPRB, 0.11273E+01_JPRB, 0.11382E+01_JPRB ,&
&  0.11826E+01_JPRB, 0.12244E+01_JPRB, 0.12672E+01_JPRB, 0.14206E+01_JPRB, 0.15892E+01_JPRB, 0.17226E+01_JPRB, 0.18542E+01_JPRB ,&
&  0.19822E+01_JPRB, 0.21074E+01_JPRB, 0.22689E+01_JPRB, 0.24349E+01_JPRB, 0.26048E+01_JPRB, 0.27121E+01_JPRB, 0.28326E+01_JPRB ,&
&  0.28855E+01_JPRB, 0.29216E+01_JPRB, 0.27251E+01_JPRB, 0.23826E+01_JPRB, 0.20135E+01_JPRB, 0.16113E+01_JPRB, 0.12117E+01_JPRB ,&
&  0.92416E+00_JPRB, 0.86278E+00_JPRB, 0.86392E+00_JPRB, 0.87879E+00_JPRB, 0.79240E+00_JPRB, 0.73187E+00_JPRB, 0.69486E+00_JPRB ,&
&  0.65658E+00_JPRB, 0.60553E+00_JPRB, 0.63021E+00_JPRB, 0.67365E+00_JPRB, 0.68387E+00_JPRB, 0.67378E+00_JPRB, 0.65177E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.58155E+00_JPRB, 0.57414E+00_JPRB, 0.56805E+00_JPRB, 0.57114E+00_JPRB, 0.60019E+00_JPRB, 0.54016E+00_JPRB ,&
&  0.59508E+00_JPRB, 0.51640E+00_JPRB, 0.52604E+00_JPRB, 0.55574E+00_JPRB, 0.62325E+00_JPRB, 0.80103E+00_JPRB, 0.11257E+01_JPRB ,&
&  0.17090E+01_JPRB, 0.19682E+01_JPRB, 0.22228E+01_JPRB, 0.27763E+01_JPRB, 0.24202E+01_JPRB, 0.17843E+01_JPRB, 0.51993E+00_JPRB ,&
&  0.51830E+00_JPRB, 0.52314E+00_JPRB, 0.54536E+00_JPRB, 0.55418E+00_JPRB, 0.55897E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.48477E+00_JPRB, 0.47165E+00_JPRB, 0.46452E+00_JPRB, 0.45909E+00_JPRB, 0.44416E+00_JPRB, 0.44487E+00_JPRB, 0.44205E+00_JPRB ,&
&  0.45831E+00_JPRB, 0.47885E+00_JPRB, 0.51526E+00_JPRB, 0.55829E+00_JPRB, 0.61347E+00_JPRB, 0.65141E+00_JPRB, 0.69615E+00_JPRB ,&
&  0.72222E+00_JPRB, 0.75670E+00_JPRB, 0.79683E+00_JPRB, 0.83020E+00_JPRB, 0.87105E+00_JPRB, 0.89882E+00_JPRB, 0.94748E+00_JPRB ,&
&  0.99137E+00_JPRB, 0.10425E+01_JPRB, 0.11116E+01_JPRB, 0.11732E+01_JPRB, 0.12251E+01_JPRB, 0.12408E+01_JPRB, 0.98924E+00_JPRB ,&
&  0.12260E+01_JPRB, 0.12678E+01_JPRB, 0.12726E+01_JPRB, 0.12394E+01_JPRB, 0.11976E+01_JPRB, 0.11589E+01_JPRB, 0.11704E+01_JPRB ,&
&  0.11951E+01_JPRB, 0.12143E+01_JPRB, 0.12350E+01_JPRB, 0.12935E+01_JPRB, 0.16053E+01_JPRB, 0.17356E+01_JPRB, 0.18718E+01_JPRB ,&
&  0.20188E+01_JPRB, 0.21843E+01_JPRB, 0.23566E+01_JPRB, 0.25325E+01_JPRB, 0.26541E+01_JPRB, 0.27878E+01_JPRB, 0.29298E+01_JPRB ,&
&  0.30278E+01_JPRB, 0.31103E+01_JPRB, 0.29780E+01_JPRB, 0.23211E+01_JPRB, 0.19139E+01_JPRB, 0.15693E+01_JPRB, 0.10599E+01_JPRB ,&
&  0.82380E+00_JPRB, 0.76069E+00_JPRB, 0.76649E+00_JPRB, 0.75711E+00_JPRB, 0.68373E+00_JPRB, 0.62008E+00_JPRB, 0.58094E+00_JPRB ,&
&  0.52702E+00_JPRB, 0.51584E+00_JPRB, 0.51059E+00_JPRB, 0.54146E+00_JPRB, 0.55889E+00_JPRB, 0.53567E+00_JPRB, 0.51854E+00_JPRB ,&
&  0.48824E+00_JPRB, 0.46372E+00_JPRB, 0.48792E+00_JPRB, 0.52780E+00_JPRB, 0.61773E+00_JPRB, 0.70721E+00_JPRB, 0.75456E+00_JPRB ,&
&  0.79650E+00_JPRB, 0.52163E+00_JPRB, 0.53382E+00_JPRB, 0.54637E+00_JPRB, 0.57494E+00_JPRB, 0.62457E+00_JPRB, 0.71768E+00_JPRB ,&
&  0.91137E+00_JPRB, 0.12521E+01_JPRB, 0.11145E+01_JPRB, 0.84909E+00_JPRB, 0.92676E+00_JPRB, 0.85341E+00_JPRB, 0.46589E+00_JPRB ,&
&  0.46818E+00_JPRB, 0.46953E+00_JPRB, 0.48522E+00_JPRB, 0.49001E+00_JPRB, 0.49309E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.42905E+00_JPRB, 0.41543E+00_JPRB, 0.39906E+00_JPRB, 0.39003E+00_JPRB, 0.38205E+00_JPRB, 0.37320E+00_JPRB, 0.38316E+00_JPRB ,&
&  0.40979E+00_JPRB, 0.43314E+00_JPRB, 0.47243E+00_JPRB, 0.53076E+00_JPRB, 0.58368E+00_JPRB, 0.61184E+00_JPRB, 0.64380E+00_JPRB ,&
&  0.67057E+00_JPRB, 0.70088E+00_JPRB, 0.72608E+00_JPRB, 0.74885E+00_JPRB, 0.77842E+00_JPRB, 0.81472E+00_JPRB, 0.86378E+00_JPRB ,&
&  0.92341E+00_JPRB, 0.98530E+00_JPRB, 0.10634E+01_JPRB, 0.11620E+01_JPRB, 0.12570E+01_JPRB, 0.12986E+01_JPRB, 0.13543E+01_JPRB ,&
&  0.10296E+01_JPRB, 0.12630E+01_JPRB, 0.12725E+01_JPRB, 0.12476E+01_JPRB, 0.12122E+01_JPRB, 0.11811E+01_JPRB, 0.11762E+01_JPRB ,&
&  0.11833E+01_JPRB, 0.11370E+01_JPRB, 0.11591E+01_JPRB, 0.12398E+01_JPRB, 0.14331E+01_JPRB, 0.15703E+01_JPRB, 0.17094E+01_JPRB ,&
&  0.18558E+01_JPRB, 0.20186E+01_JPRB, 0.21882E+01_JPRB, 0.23511E+01_JPRB, 0.24820E+01_JPRB, 0.26286E+01_JPRB, 0.27367E+01_JPRB ,&
&  0.28669E+01_JPRB, 0.29713E+01_JPRB, 0.30023E+01_JPRB, 0.24444E+01_JPRB, 0.19434E+01_JPRB, 0.16570E+01_JPRB, 0.12039E+01_JPRB ,&
&  0.82597E+00_JPRB, 0.67455E+00_JPRB, 0.68741E+00_JPRB, 0.70941E+00_JPRB, 0.65759E+00_JPRB, 0.60765E+00_JPRB, 0.55590E+00_JPRB ,&
&  0.52425E+00_JPRB, 0.47573E+00_JPRB, 0.46206E+00_JPRB, 0.47529E+00_JPRB, 0.50528E+00_JPRB, 0.48260E+00_JPRB, 0.44906E+00_JPRB ,&
&  0.42365E+00_JPRB, 0.41338E+00_JPRB, 0.44928E+00_JPRB, 0.49141E+00_JPRB, 0.54998E+00_JPRB, 0.58226E+00_JPRB, 0.58661E+00_JPRB ,&
&  0.62291E+00_JPRB, 0.56151E+00_JPRB, 0.55591E+00_JPRB, 0.56215E+00_JPRB, 0.56345E+00_JPRB, 0.57032E+00_JPRB, 0.58631E+00_JPRB ,&
&  0.62339E+00_JPRB, 0.68762E+00_JPRB, 0.70897E+00_JPRB, 0.65727E+00_JPRB, 0.60852E+00_JPRB, 0.56601E+00_JPRB, 0.52262E+00_JPRB ,&
&  0.52598E+00_JPRB, 0.51436E+00_JPRB, 0.50164E+00_JPRB, 0.48260E+00_JPRB, 0.45671E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.43879E+00_JPRB, 0.41453E+00_JPRB, 0.37772E+00_JPRB, 0.34431E+00_JPRB, 0.36564E+00_JPRB, 0.36482E+00_JPRB, 0.37866E+00_JPRB ,&
&  0.38348E+00_JPRB, 0.40680E+00_JPRB, 0.42981E+00_JPRB, 0.46447E+00_JPRB, 0.50689E+00_JPRB, 0.54887E+00_JPRB, 0.58314E+00_JPRB ,&
&  0.61371E+00_JPRB, 0.64411E+00_JPRB, 0.66809E+00_JPRB, 0.67715E+00_JPRB, 0.69147E+00_JPRB, 0.71549E+00_JPRB, 0.74318E+00_JPRB ,&
&  0.77924E+00_JPRB, 0.82753E+00_JPRB, 0.89582E+00_JPRB, 0.98362E+00_JPRB, 0.10877E+01_JPRB, 0.11441E+01_JPRB, 0.11767E+01_JPRB ,&
&  0.70181E+00_JPRB, 0.10650E+01_JPRB, 0.12704E+01_JPRB, 0.12270E+01_JPRB, 0.12001E+01_JPRB, 0.11795E+01_JPRB, 0.11704E+01_JPRB ,&
&  0.11137E+01_JPRB, 0.10503E+01_JPRB, 0.10588E+01_JPRB, 0.12174E+01_JPRB, 0.12192E+01_JPRB, 0.13468E+01_JPRB, 0.14541E+01_JPRB ,&
&  0.15680E+01_JPRB, 0.16869E+01_JPRB, 0.18231E+01_JPRB, 0.19912E+01_JPRB, 0.21613E+01_JPRB, 0.23049E+01_JPRB, 0.24336E+01_JPRB ,&
&  0.26016E+01_JPRB, 0.27489E+01_JPRB, 0.27914E+01_JPRB, 0.21517E+01_JPRB, 0.19394E+01_JPRB, 0.18052E+01_JPRB, 0.15978E+01_JPRB ,&
&  0.96718E+00_JPRB, 0.68642E+00_JPRB, 0.68523E+00_JPRB, 0.71174E+00_JPRB, 0.67160E+00_JPRB, 0.63197E+00_JPRB, 0.59180E+00_JPRB ,&
&  0.54199E+00_JPRB, 0.50031E+00_JPRB, 0.47567E+00_JPRB, 0.46749E+00_JPRB, 0.48597E+00_JPRB, 0.48932E+00_JPRB, 0.46925E+00_JPRB ,&
&  0.44459E+00_JPRB, 0.44601E+00_JPRB, 0.47328E+00_JPRB, 0.49609E+00_JPRB, 0.53059E+00_JPRB, 0.55457E+00_JPRB, 0.57335E+00_JPRB ,&
&  0.58556E+00_JPRB, 0.56171E+00_JPRB, 0.56631E+00_JPRB, 0.57760E+00_JPRB, 0.57981E+00_JPRB, 0.58056E+00_JPRB, 0.57778E+00_JPRB ,&
&  0.57676E+00_JPRB, 0.59434E+00_JPRB, 0.60776E+00_JPRB, 0.61748E+00_JPRB, 0.61007E+00_JPRB, 0.59853E+00_JPRB, 0.59566E+00_JPRB ,&
&  0.58341E+00_JPRB, 0.56227E+00_JPRB, 0.53833E+00_JPRB, 0.50221E+00_JPRB, 0.46787E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.45946E+00_JPRB, 0.43256E+00_JPRB, 0.39521E+00_JPRB, 0.38025E+00_JPRB, 0.40176E+00_JPRB, 0.41920E+00_JPRB, 0.41345E+00_JPRB ,&
&  0.40705E+00_JPRB, 0.39785E+00_JPRB, 0.39255E+00_JPRB, 0.40088E+00_JPRB, 0.43359E+00_JPRB, 0.47977E+00_JPRB, 0.51392E+00_JPRB ,&
&  0.54726E+00_JPRB, 0.57418E+00_JPRB, 0.59638E+00_JPRB, 0.60422E+00_JPRB, 0.60127E+00_JPRB, 0.61119E+00_JPRB, 0.62552E+00_JPRB ,&
&  0.65010E+00_JPRB, 0.68608E+00_JPRB, 0.73292E+00_JPRB, 0.79909E+00_JPRB, 0.88930E+00_JPRB, 0.99940E+00_JPRB, 0.11113E+01_JPRB ,&
&  0.82540E+00_JPRB, 0.44357E+00_JPRB, 0.10683E+01_JPRB, 0.13154E+01_JPRB, 0.12445E+01_JPRB, 0.12225E+01_JPRB, 0.11590E+01_JPRB ,&
&  0.10782E+01_JPRB, 0.10836E+01_JPRB, 0.10460E+01_JPRB, 0.11316E+01_JPRB, 0.11020E+01_JPRB, 0.11402E+01_JPRB, 0.11806E+01_JPRB ,&
&  0.12393E+01_JPRB, 0.13073E+01_JPRB, 0.14087E+01_JPRB, 0.15306E+01_JPRB, 0.16664E+01_JPRB, 0.18689E+01_JPRB, 0.20167E+01_JPRB ,&
&  0.21535E+01_JPRB, 0.22775E+01_JPRB, 0.23048E+01_JPRB, 0.16830E+01_JPRB, 0.13689E+01_JPRB, 0.12095E+01_JPRB, 0.10913E+01_JPRB ,&
&  0.93230E+00_JPRB, 0.74603E+00_JPRB, 0.71756E+00_JPRB, 0.73588E+00_JPRB, 0.70874E+00_JPRB, 0.64789E+00_JPRB, 0.64814E+00_JPRB ,&
&  0.58545E+00_JPRB, 0.54152E+00_JPRB, 0.51148E+00_JPRB, 0.48742E+00_JPRB, 0.49232E+00_JPRB, 0.50166E+00_JPRB, 0.50456E+00_JPRB ,&
&  0.49160E+00_JPRB, 0.49180E+00_JPRB, 0.50397E+00_JPRB, 0.50582E+00_JPRB, 0.50916E+00_JPRB, 0.51677E+00_JPRB, 0.52162E+00_JPRB ,&
&  0.54921E+00_JPRB, 0.55246E+00_JPRB, 0.56164E+00_JPRB, 0.57813E+00_JPRB, 0.58722E+00_JPRB, 0.57569E+00_JPRB, 0.57781E+00_JPRB ,&
&  0.58919E+00_JPRB, 0.60095E+00_JPRB, 0.60019E+00_JPRB, 0.61175E+00_JPRB, 0.61226E+00_JPRB, 0.61323E+00_JPRB, 0.62599E+00_JPRB ,&
&  0.64436E+00_JPRB, 0.64654E+00_JPRB, 0.61286E+00_JPRB, 0.53395E+00_JPRB, 0.49361E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.48439E+00_JPRB, 0.44700E+00_JPRB, 0.41190E+00_JPRB, 0.41183E+00_JPRB, 0.43026E+00_JPRB, 0.43510E+00_JPRB, 0.43308E+00_JPRB ,&
&  0.42354E+00_JPRB, 0.41295E+00_JPRB, 0.39643E+00_JPRB, 0.38887E+00_JPRB, 0.39307E+00_JPRB, 0.40395E+00_JPRB, 0.43695E+00_JPRB ,&
&  0.46981E+00_JPRB, 0.50007E+00_JPRB, 0.52523E+00_JPRB, 0.52890E+00_JPRB, 0.51699E+00_JPRB, 0.51716E+00_JPRB, 0.52595E+00_JPRB ,&
&  0.53399E+00_JPRB, 0.55561E+00_JPRB, 0.58471E+00_JPRB, 0.62489E+00_JPRB, 0.68451E+00_JPRB, 0.78159E+00_JPRB, 0.93159E+00_JPRB ,&
&  0.11643E+01_JPRB, 0.81637E+00_JPRB, 0.58595E+00_JPRB, 0.11324E+01_JPRB, 0.13798E+01_JPRB, 0.14065E+01_JPRB, 0.11993E+01_JPRB ,&
&  0.11620E+01_JPRB, 0.11116E+01_JPRB, 0.10769E+01_JPRB, 0.11191E+01_JPRB, 0.10501E+01_JPRB, 0.10359E+01_JPRB, 0.10212E+01_JPRB ,&
&  0.10091E+01_JPRB, 0.10593E+01_JPRB, 0.11291E+01_JPRB, 0.11959E+01_JPRB, 0.12605E+01_JPRB, 0.14020E+01_JPRB, 0.15802E+01_JPRB ,&
&  0.17566E+01_JPRB, 0.18627E+01_JPRB, 0.18945E+01_JPRB, 0.14350E+01_JPRB, 0.12059E+01_JPRB, 0.10975E+01_JPRB, 0.10417E+01_JPRB ,&
&  0.92562E+00_JPRB, 0.85823E+00_JPRB, 0.83546E+00_JPRB, 0.78778E+00_JPRB, 0.74262E+00_JPRB, 0.69981E+00_JPRB, 0.70691E+00_JPRB ,&
&  0.65120E+00_JPRB, 0.60226E+00_JPRB, 0.57159E+00_JPRB, 0.55196E+00_JPRB, 0.54404E+00_JPRB, 0.54556E+00_JPRB, 0.54128E+00_JPRB ,&
&  0.54385E+00_JPRB, 0.54563E+00_JPRB, 0.54866E+00_JPRB, 0.53665E+00_JPRB, 0.52598E+00_JPRB, 0.51930E+00_JPRB, 0.50709E+00_JPRB ,&
&  0.52316E+00_JPRB, 0.54725E+00_JPRB, 0.55755E+00_JPRB, 0.57442E+00_JPRB, 0.57468E+00_JPRB, 0.56903E+00_JPRB, 0.58421E+00_JPRB ,&
&  0.58986E+00_JPRB, 0.60582E+00_JPRB, 0.60373E+00_JPRB, 0.61163E+00_JPRB, 0.60619E+00_JPRB, 0.60373E+00_JPRB, 0.61398E+00_JPRB ,&
&  0.65895E+00_JPRB, 0.71179E+00_JPRB, 0.70770E+00_JPRB, 0.55806E+00_JPRB, 0.50950E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.47324E+00_JPRB, 0.46241E+00_JPRB, 0.42453E+00_JPRB, 0.42762E+00_JPRB, 0.42933E+00_JPRB, 0.43496E+00_JPRB, 0.43242E+00_JPRB ,&
&  0.43076E+00_JPRB, 0.43239E+00_JPRB, 0.43485E+00_JPRB, 0.42736E+00_JPRB, 0.40543E+00_JPRB, 0.39580E+00_JPRB, 0.39899E+00_JPRB ,&
&  0.40994E+00_JPRB, 0.43348E+00_JPRB, 0.46177E+00_JPRB, 0.46620E+00_JPRB, 0.46750E+00_JPRB, 0.46412E+00_JPRB, 0.46086E+00_JPRB ,&
&  0.46417E+00_JPRB, 0.47083E+00_JPRB, 0.47992E+00_JPRB, 0.50191E+00_JPRB, 0.54095E+00_JPRB, 0.60027E+00_JPRB, 0.69366E+00_JPRB ,&
&  0.91993E+00_JPRB, 0.96058E+00_JPRB, 0.50003E+00_JPRB, 0.91242E+00_JPRB, 0.13710E+01_JPRB, 0.14444E+01_JPRB, 0.13752E+01_JPRB ,&
&  0.12999E+01_JPRB, 0.11650E+01_JPRB, 0.11151E+01_JPRB, 0.10758E+01_JPRB, 0.98110E+00_JPRB, 0.93100E+00_JPRB, 0.88198E+00_JPRB ,&
&  0.85095E+00_JPRB, 0.87798E+00_JPRB, 0.95034E+00_JPRB, 0.10069E+01_JPRB, 0.10243E+01_JPRB, 0.10654E+01_JPRB, 0.11981E+01_JPRB ,&
&  0.13949E+01_JPRB, 0.15954E+01_JPRB, 0.16833E+01_JPRB, 0.13494E+01_JPRB, 0.11947E+01_JPRB, 0.11988E+01_JPRB, 0.11582E+01_JPRB ,&
&  0.10767E+01_JPRB, 0.97892E+00_JPRB, 0.90474E+00_JPRB, 0.84433E+00_JPRB, 0.78395E+00_JPRB, 0.76020E+00_JPRB, 0.77006E+00_JPRB ,&
&  0.72667E+00_JPRB, 0.66871E+00_JPRB, 0.63292E+00_JPRB, 0.61076E+00_JPRB, 0.59459E+00_JPRB, 0.58685E+00_JPRB, 0.58019E+00_JPRB ,&
&  0.57624E+00_JPRB, 0.58216E+00_JPRB, 0.58979E+00_JPRB, 0.58311E+00_JPRB, 0.56672E+00_JPRB, 0.54642E+00_JPRB, 0.52774E+00_JPRB ,&
&  0.52565E+00_JPRB, 0.53545E+00_JPRB, 0.55561E+00_JPRB, 0.56321E+00_JPRB, 0.56693E+00_JPRB, 0.57492E+00_JPRB, 0.58236E+00_JPRB ,&
&  0.59303E+00_JPRB, 0.60298E+00_JPRB, 0.59426E+00_JPRB, 0.58449E+00_JPRB, 0.60246E+00_JPRB, 0.59230E+00_JPRB, 0.58347E+00_JPRB ,&
&  0.62172E+00_JPRB, 0.69844E+00_JPRB, 0.82424E+00_JPRB, 0.56462E+00_JPRB, 0.50821E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.46729E+00_JPRB, 0.44563E+00_JPRB, 0.43846E+00_JPRB, 0.43549E+00_JPRB, 0.43717E+00_JPRB, 0.44346E+00_JPRB, 0.44965E+00_JPRB ,&
&  0.45004E+00_JPRB, 0.44992E+00_JPRB, 0.45055E+00_JPRB, 0.44695E+00_JPRB, 0.43585E+00_JPRB, 0.42497E+00_JPRB, 0.41515E+00_JPRB ,&
&  0.41371E+00_JPRB, 0.41578E+00_JPRB, 0.42820E+00_JPRB, 0.43560E+00_JPRB, 0.43669E+00_JPRB, 0.43430E+00_JPRB, 0.43506E+00_JPRB ,&
&  0.43851E+00_JPRB, 0.43771E+00_JPRB, 0.43516E+00_JPRB, 0.44508E+00_JPRB, 0.46506E+00_JPRB, 0.50370E+00_JPRB, 0.56380E+00_JPRB ,&
&  0.72908E+00_JPRB, 0.86560E+00_JPRB, 0.92187E+00_JPRB, 0.13804E+01_JPRB, 0.16773E+01_JPRB, 0.16142E+01_JPRB, 0.15330E+01_JPRB ,&
&  0.14094E+01_JPRB, 0.12611E+01_JPRB, 0.12389E+01_JPRB, 0.10749E+01_JPRB, 0.95162E+00_JPRB, 0.85278E+00_JPRB, 0.77507E+00_JPRB ,&
&  0.71812E+00_JPRB, 0.70389E+00_JPRB, 0.74923E+00_JPRB, 0.82791E+00_JPRB, 0.87918E+00_JPRB, 0.89703E+00_JPRB, 0.94918E+00_JPRB ,&
&  0.10866E+01_JPRB, 0.13387E+01_JPRB, 0.15426E+01_JPRB, 0.14323E+01_JPRB, 0.11858E+01_JPRB, 0.13006E+01_JPRB, 0.14270E+01_JPRB ,&
&  0.16030E+01_JPRB, 0.12166E+01_JPRB, 0.99138E+00_JPRB, 0.89830E+00_JPRB, 0.82516E+00_JPRB, 0.87786E+00_JPRB, 0.83696E+00_JPRB ,&
&  0.78840E+00_JPRB, 0.73659E+00_JPRB, 0.69036E+00_JPRB, 0.66169E+00_JPRB, 0.64008E+00_JPRB, 0.62469E+00_JPRB, 0.61394E+00_JPRB ,&
&  0.60762E+00_JPRB, 0.60488E+00_JPRB, 0.61281E+00_JPRB, 0.61499E+00_JPRB, 0.60818E+00_JPRB, 0.59008E+00_JPRB, 0.56436E+00_JPRB ,&
&  0.54438E+00_JPRB, 0.54047E+00_JPRB, 0.53145E+00_JPRB, 0.54274E+00_JPRB, 0.55645E+00_JPRB, 0.56346E+00_JPRB, 0.56159E+00_JPRB ,&
&  0.57342E+00_JPRB, 0.58985E+00_JPRB, 0.59026E+00_JPRB, 0.58592E+00_JPRB, 0.58688E+00_JPRB, 0.58938E+00_JPRB, 0.56214E+00_JPRB ,&
&  0.55161E+00_JPRB, 0.54139E+00_JPRB, 0.53920E+00_JPRB, 0.52519E+00_JPRB, 0.49168E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.46643E+00_JPRB, 0.45277E+00_JPRB, 0.44765E+00_JPRB, 0.44973E+00_JPRB, 0.45177E+00_JPRB, 0.45430E+00_JPRB, 0.46255E+00_JPRB ,&
&  0.46654E+00_JPRB, 0.46696E+00_JPRB, 0.46509E+00_JPRB, 0.46227E+00_JPRB, 0.45467E+00_JPRB, 0.44641E+00_JPRB, 0.43710E+00_JPRB ,&
&  0.43039E+00_JPRB, 0.41246E+00_JPRB, 0.40341E+00_JPRB, 0.41122E+00_JPRB, 0.41515E+00_JPRB, 0.41460E+00_JPRB, 0.41533E+00_JPRB ,&
&  0.41556E+00_JPRB, 0.41970E+00_JPRB, 0.41979E+00_JPRB, 0.42554E+00_JPRB, 0.43664E+00_JPRB, 0.46330E+00_JPRB, 0.49937E+00_JPRB ,&
&  0.61557E+00_JPRB, 0.81455E+00_JPRB, 0.87517E+00_JPRB, 0.15779E+01_JPRB, 0.17389E+01_JPRB, 0.16802E+01_JPRB, 0.14725E+01_JPRB ,&
&  0.13441E+01_JPRB, 0.13716E+01_JPRB, 0.11474E+01_JPRB, 0.99290E+00_JPRB, 0.90519E+00_JPRB, 0.81706E+00_JPRB, 0.71613E+00_JPRB ,&
&  0.63970E+00_JPRB, 0.60586E+00_JPRB, 0.60115E+00_JPRB, 0.63696E+00_JPRB, 0.70122E+00_JPRB, 0.75957E+00_JPRB, 0.80607E+00_JPRB ,&
&  0.88576E+00_JPRB, 0.10711E+01_JPRB, 0.13402E+01_JPRB, 0.13194E+01_JPRB, 0.12460E+01_JPRB, 0.14144E+01_JPRB, 0.17411E+01_JPRB ,&
&  0.20910E+01_JPRB, 0.17019E+01_JPRB, 0.11771E+01_JPRB, 0.97594E+00_JPRB, 0.91973E+00_JPRB, 0.94175E+00_JPRB, 0.92389E+00_JPRB ,&
&  0.86629E+00_JPRB, 0.81754E+00_JPRB, 0.75768E+00_JPRB, 0.71796E+00_JPRB, 0.68970E+00_JPRB, 0.65737E+00_JPRB, 0.63511E+00_JPRB ,&
&  0.62457E+00_JPRB, 0.61870E+00_JPRB, 0.61915E+00_JPRB, 0.61607E+00_JPRB, 0.61558E+00_JPRB, 0.61230E+00_JPRB, 0.60337E+00_JPRB ,&
&  0.58409E+00_JPRB, 0.55689E+00_JPRB, 0.52682E+00_JPRB, 0.52331E+00_JPRB, 0.53770E+00_JPRB, 0.54557E+00_JPRB, 0.54966E+00_JPRB ,&
&  0.57441E+00_JPRB, 0.58470E+00_JPRB, 0.57782E+00_JPRB, 0.56105E+00_JPRB, 0.56822E+00_JPRB, 0.59732E+00_JPRB, 0.56598E+00_JPRB ,&
&  0.56168E+00_JPRB, 0.55231E+00_JPRB, 0.54201E+00_JPRB, 0.53068E+00_JPRB, 0.49962E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.48857E+00_JPRB, 0.47708E+00_JPRB, 0.46868E+00_JPRB, 0.46615E+00_JPRB, 0.46061E+00_JPRB, 0.46015E+00_JPRB, 0.46743E+00_JPRB ,&
&  0.47999E+00_JPRB, 0.48353E+00_JPRB, 0.48141E+00_JPRB, 0.47836E+00_JPRB, 0.47265E+00_JPRB, 0.46492E+00_JPRB, 0.45499E+00_JPRB ,&
&  0.44618E+00_JPRB, 0.43266E+00_JPRB, 0.41509E+00_JPRB, 0.40249E+00_JPRB, 0.40630E+00_JPRB, 0.41116E+00_JPRB, 0.41119E+00_JPRB ,&
&  0.41029E+00_JPRB, 0.41636E+00_JPRB, 0.41891E+00_JPRB, 0.42882E+00_JPRB, 0.43609E+00_JPRB, 0.45249E+00_JPRB, 0.48355E+00_JPRB ,&
&  0.57408E+00_JPRB, 0.86731E+00_JPRB, 0.89302E+00_JPRB, 0.14197E+01_JPRB, 0.16116E+01_JPRB, 0.15519E+01_JPRB, 0.13467E+01_JPRB ,&
&  0.12458E+01_JPRB, 0.11830E+01_JPRB, 0.10231E+01_JPRB, 0.89323E+00_JPRB, 0.80872E+00_JPRB, 0.74222E+00_JPRB, 0.67924E+00_JPRB ,&
&  0.62984E+00_JPRB, 0.60147E+00_JPRB, 0.58504E+00_JPRB, 0.57623E+00_JPRB, 0.59692E+00_JPRB, 0.64097E+00_JPRB, 0.69343E+00_JPRB ,&
&  0.75135E+00_JPRB, 0.84997E+00_JPRB, 0.10827E+01_JPRB, 0.13120E+01_JPRB, 0.12311E+01_JPRB, 0.14130E+01_JPRB, 0.20465E+01_JPRB ,&
&  0.24004E+01_JPRB, 0.23610E+01_JPRB, 0.15744E+01_JPRB, 0.11471E+01_JPRB, 0.10147E+01_JPRB, 0.10211E+01_JPRB, 0.10149E+01_JPRB ,&
&  0.96313E+00_JPRB, 0.92462E+00_JPRB, 0.85899E+00_JPRB, 0.78636E+00_JPRB, 0.73206E+00_JPRB, 0.69332E+00_JPRB, 0.65532E+00_JPRB ,&
&  0.62716E+00_JPRB, 0.61536E+00_JPRB, 0.60720E+00_JPRB, 0.59911E+00_JPRB, 0.59570E+00_JPRB, 0.59829E+00_JPRB, 0.60165E+00_JPRB ,&
&  0.60074E+00_JPRB, 0.58490E+00_JPRB, 0.54711E+00_JPRB, 0.52395E+00_JPRB, 0.52417E+00_JPRB, 0.53236E+00_JPRB, 0.53728E+00_JPRB ,&
&  0.56879E+00_JPRB, 0.57803E+00_JPRB, 0.56545E+00_JPRB, 0.56083E+00_JPRB, 0.56012E+00_JPRB, 0.60672E+00_JPRB, 0.57252E+00_JPRB ,&
&  0.56110E+00_JPRB, 0.55309E+00_JPRB, 0.55089E+00_JPRB, 0.54506E+00_JPRB, 0.51895E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.51196E+00_JPRB, 0.49641E+00_JPRB, 0.48743E+00_JPRB, 0.47552E+00_JPRB, 0.46578E+00_JPRB, 0.46218E+00_JPRB, 0.46822E+00_JPRB ,&
&  0.48347E+00_JPRB, 0.49604E+00_JPRB, 0.49024E+00_JPRB, 0.48708E+00_JPRB, 0.48549E+00_JPRB, 0.47863E+00_JPRB, 0.46841E+00_JPRB ,&
&  0.45812E+00_JPRB, 0.44814E+00_JPRB, 0.43617E+00_JPRB, 0.42336E+00_JPRB, 0.41432E+00_JPRB, 0.41455E+00_JPRB, 0.41282E+00_JPRB ,&
&  0.41107E+00_JPRB, 0.41482E+00_JPRB, 0.41951E+00_JPRB, 0.42792E+00_JPRB, 0.43626E+00_JPRB, 0.44971E+00_JPRB, 0.47643E+00_JPRB ,&
&  0.57858E+00_JPRB, 0.46921E+00_JPRB, 0.10218E+01_JPRB, 0.12490E+01_JPRB, 0.12928E+01_JPRB, 0.12140E+01_JPRB, 0.11598E+01_JPRB ,&
&  0.11228E+01_JPRB, 0.10156E+01_JPRB, 0.92620E+00_JPRB, 0.84784E+00_JPRB, 0.75389E+00_JPRB, 0.69318E+00_JPRB, 0.65641E+00_JPRB ,&
&  0.62727E+00_JPRB, 0.60151E+00_JPRB, 0.58064E+00_JPRB, 0.56342E+00_JPRB, 0.55419E+00_JPRB, 0.56585E+00_JPRB, 0.59750E+00_JPRB ,&
&  0.63468E+00_JPRB, 0.69141E+00_JPRB, 0.85236E+00_JPRB, 0.10779E+01_JPRB, 0.12092E+01_JPRB, 0.13689E+01_JPRB, 0.17005E+01_JPRB ,&
&  0.22125E+01_JPRB, 0.24370E+01_JPRB, 0.18839E+01_JPRB, 0.14293E+01_JPRB, 0.11820E+01_JPRB, 0.11203E+01_JPRB, 0.11212E+01_JPRB ,&
&  0.10756E+01_JPRB, 0.10173E+01_JPRB, 0.93958E+00_JPRB, 0.85079E+00_JPRB, 0.77378E+00_JPRB, 0.72285E+00_JPRB, 0.68704E+00_JPRB ,&
&  0.65405E+00_JPRB, 0.62380E+00_JPRB, 0.60444E+00_JPRB, 0.59263E+00_JPRB, 0.58547E+00_JPRB, 0.58323E+00_JPRB, 0.58733E+00_JPRB ,&
&  0.59652E+00_JPRB, 0.59560E+00_JPRB, 0.57424E+00_JPRB, 0.55118E+00_JPRB, 0.54740E+00_JPRB, 0.55486E+00_JPRB, 0.56362E+00_JPRB ,&
&  0.57877E+00_JPRB, 0.58230E+00_JPRB, 0.57052E+00_JPRB, 0.56723E+00_JPRB, 0.54668E+00_JPRB, 0.58271E+00_JPRB, 0.56886E+00_JPRB ,&
&  0.55788E+00_JPRB, 0.55524E+00_JPRB, 0.55983E+00_JPRB, 0.55459E+00_JPRB, 0.53631E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.53056E+00_JPRB, 0.51498E+00_JPRB, 0.50256E+00_JPRB, 0.48406E+00_JPRB, 0.47395E+00_JPRB, 0.47068E+00_JPRB, 0.47636E+00_JPRB ,&
&  0.48412E+00_JPRB, 0.49165E+00_JPRB, 0.48778E+00_JPRB, 0.48338E+00_JPRB, 0.47702E+00_JPRB, 0.47302E+00_JPRB, 0.46580E+00_JPRB ,&
&  0.46179E+00_JPRB, 0.45798E+00_JPRB, 0.45144E+00_JPRB, 0.44381E+00_JPRB, 0.43280E+00_JPRB, 0.42771E+00_JPRB, 0.42154E+00_JPRB ,&
&  0.41581E+00_JPRB, 0.41367E+00_JPRB, 0.41577E+00_JPRB, 0.42469E+00_JPRB, 0.43624E+00_JPRB, 0.44983E+00_JPRB, 0.47011E+00_JPRB ,&
&  0.58458E+00_JPRB, 0.66049E+00_JPRB, 0.10290E+01_JPRB, 0.10554E+01_JPRB, 0.97508E+00_JPRB, 0.88676E+00_JPRB, 0.88854E+00_JPRB ,&
&  0.87587E+00_JPRB, 0.83095E+00_JPRB, 0.78326E+00_JPRB, 0.75076E+00_JPRB, 0.71806E+00_JPRB, 0.69539E+00_JPRB, 0.66194E+00_JPRB ,&
&  0.63022E+00_JPRB, 0.59650E+00_JPRB, 0.56031E+00_JPRB, 0.53680E+00_JPRB, 0.52311E+00_JPRB, 0.52139E+00_JPRB, 0.53386E+00_JPRB ,&
&  0.55206E+00_JPRB, 0.57707E+00_JPRB, 0.66659E+00_JPRB, 0.85909E+00_JPRB, 0.11532E+01_JPRB, 0.12184E+01_JPRB, 0.16273E+01_JPRB ,&
&  0.21796E+01_JPRB, 0.21470E+01_JPRB, 0.19201E+01_JPRB, 0.17204E+01_JPRB, 0.14635E+01_JPRB, 0.13594E+01_JPRB, 0.13319E+01_JPRB ,&
&  0.12459E+01_JPRB, 0.11113E+01_JPRB, 0.97312E+00_JPRB, 0.86873E+00_JPRB, 0.79644E+00_JPRB, 0.75177E+00_JPRB, 0.72880E+00_JPRB ,&
&  0.70629E+00_JPRB, 0.67727E+00_JPRB, 0.64610E+00_JPRB, 0.62125E+00_JPRB, 0.60627E+00_JPRB, 0.60016E+00_JPRB, 0.59878E+00_JPRB ,&
&  0.60065E+00_JPRB, 0.60049E+00_JPRB, 0.59390E+00_JPRB, 0.57980E+00_JPRB, 0.59141E+00_JPRB, 0.61636E+00_JPRB, 0.62611E+00_JPRB ,&
&  0.61296E+00_JPRB, 0.60171E+00_JPRB, 0.59103E+00_JPRB, 0.57513E+00_JPRB, 0.56206E+00_JPRB, 0.58745E+00_JPRB, 0.56957E+00_JPRB ,&
&  0.54958E+00_JPRB, 0.54872E+00_JPRB, 0.56351E+00_JPRB, 0.56365E+00_JPRB, 0.55720E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.54853E+00_JPRB, 0.53001E+00_JPRB, 0.51317E+00_JPRB, 0.49707E+00_JPRB, 0.48338E+00_JPRB, 0.47832E+00_JPRB, 0.47938E+00_JPRB ,&
&  0.48325E+00_JPRB, 0.48422E+00_JPRB, 0.48077E+00_JPRB, 0.47348E+00_JPRB, 0.46858E+00_JPRB, 0.46438E+00_JPRB, 0.46116E+00_JPRB ,&
&  0.45498E+00_JPRB, 0.45589E+00_JPRB, 0.45242E+00_JPRB, 0.44617E+00_JPRB, 0.43910E+00_JPRB, 0.43056E+00_JPRB, 0.42236E+00_JPRB ,&
&  0.41721E+00_JPRB, 0.41570E+00_JPRB, 0.41685E+00_JPRB, 0.42146E+00_JPRB, 0.43376E+00_JPRB, 0.44589E+00_JPRB, 0.46052E+00_JPRB ,&
&  0.54758E+00_JPRB, 0.68377E+00_JPRB, 0.80436E+00_JPRB, 0.88787E+00_JPRB, 0.80396E+00_JPRB, 0.73704E+00_JPRB, 0.70761E+00_JPRB ,&
&  0.69506E+00_JPRB, 0.68687E+00_JPRB, 0.66147E+00_JPRB, 0.63522E+00_JPRB, 0.60599E+00_JPRB, 0.59322E+00_JPRB, 0.57754E+00_JPRB ,&
&  0.55906E+00_JPRB, 0.54254E+00_JPRB, 0.52519E+00_JPRB, 0.51876E+00_JPRB, 0.51433E+00_JPRB, 0.50850E+00_JPRB, 0.50511E+00_JPRB ,&
&  0.50857E+00_JPRB, 0.51641E+00_JPRB, 0.54683E+00_JPRB, 0.66398E+00_JPRB, 0.92167E+00_JPRB, 0.11259E+01_JPRB, 0.13935E+01_JPRB ,&
&  0.14668E+01_JPRB, 0.15343E+01_JPRB, 0.15866E+01_JPRB, 0.15801E+01_JPRB, 0.14848E+01_JPRB, 0.14200E+01_JPRB, 0.14112E+01_JPRB ,&
&  0.13332E+01_JPRB, 0.11459E+01_JPRB, 0.96596E+00_JPRB, 0.85610E+00_JPRB, 0.80061E+00_JPRB, 0.77391E+00_JPRB, 0.75216E+00_JPRB ,&
&  0.73864E+00_JPRB, 0.72525E+00_JPRB, 0.70252E+00_JPRB, 0.67367E+00_JPRB, 0.64651E+00_JPRB, 0.62461E+00_JPRB, 0.61139E+00_JPRB ,&
&  0.60060E+00_JPRB, 0.59585E+00_JPRB, 0.60191E+00_JPRB, 0.61308E+00_JPRB, 0.62442E+00_JPRB, 0.63603E+00_JPRB, 0.64580E+00_JPRB ,&
&  0.65594E+00_JPRB, 0.64735E+00_JPRB, 0.61375E+00_JPRB, 0.59296E+00_JPRB, 0.57256E+00_JPRB, 0.58044E+00_JPRB, 0.55992E+00_JPRB ,&
&  0.54183E+00_JPRB, 0.53696E+00_JPRB, 0.54878E+00_JPRB, 0.55677E+00_JPRB, 0.56328E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.62836E+00_JPRB, 0.58075E+00_JPRB, 0.54139E+00_JPRB, 0.51064E+00_JPRB, 0.49345E+00_JPRB, 0.48418E+00_JPRB, 0.47759E+00_JPRB ,&
&  0.47442E+00_JPRB, 0.47153E+00_JPRB, 0.46402E+00_JPRB, 0.45786E+00_JPRB, 0.45364E+00_JPRB, 0.45190E+00_JPRB, 0.45058E+00_JPRB ,&
&  0.44237E+00_JPRB, 0.44120E+00_JPRB, 0.44149E+00_JPRB, 0.43737E+00_JPRB, 0.43162E+00_JPRB, 0.42557E+00_JPRB, 0.41776E+00_JPRB ,&
&  0.41255E+00_JPRB, 0.41105E+00_JPRB, 0.41343E+00_JPRB, 0.41759E+00_JPRB, 0.42187E+00_JPRB, 0.43324E+00_JPRB, 0.45529E+00_JPRB ,&
&  0.53109E+00_JPRB, 0.58095E+00_JPRB, 0.65390E+00_JPRB, 0.69158E+00_JPRB, 0.63023E+00_JPRB, 0.61152E+00_JPRB, 0.60638E+00_JPRB ,&
&  0.61158E+00_JPRB, 0.61595E+00_JPRB, 0.59344E+00_JPRB, 0.55775E+00_JPRB, 0.52634E+00_JPRB, 0.50968E+00_JPRB, 0.49919E+00_JPRB ,&
&  0.48918E+00_JPRB, 0.48206E+00_JPRB, 0.47554E+00_JPRB, 0.47739E+00_JPRB, 0.47934E+00_JPRB, 0.47762E+00_JPRB, 0.47628E+00_JPRB ,&
&  0.47645E+00_JPRB, 0.47921E+00_JPRB, 0.48747E+00_JPRB, 0.53222E+00_JPRB, 0.65944E+00_JPRB, 0.80257E+00_JPRB, 0.94651E+00_JPRB ,&
&  0.10169E+01_JPRB, 0.10882E+01_JPRB, 0.11773E+01_JPRB, 0.12481E+01_JPRB, 0.12574E+01_JPRB, 0.12529E+01_JPRB, 0.12455E+01_JPRB ,&
&  0.12107E+01_JPRB, 0.10980E+01_JPRB, 0.95074E+00_JPRB, 0.84261E+00_JPRB, 0.78399E+00_JPRB, 0.75435E+00_JPRB, 0.73697E+00_JPRB ,&
&  0.72275E+00_JPRB, 0.70986E+00_JPRB, 0.69078E+00_JPRB, 0.66341E+00_JPRB, 0.63264E+00_JPRB, 0.61068E+00_JPRB, 0.59427E+00_JPRB ,&
&  0.58444E+00_JPRB, 0.58664E+00_JPRB, 0.59894E+00_JPRB, 0.61964E+00_JPRB, 0.63430E+00_JPRB, 0.64303E+00_JPRB, 0.64442E+00_JPRB ,&
&  0.64580E+00_JPRB, 0.65542E+00_JPRB, 0.61542E+00_JPRB, 0.59348E+00_JPRB, 0.61894E+00_JPRB, 0.58436E+00_JPRB, 0.55346E+00_JPRB ,&
&  0.53709E+00_JPRB, 0.52466E+00_JPRB, 0.52345E+00_JPRB, 0.54085E+00_JPRB, 0.65404E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.58033E+00_JPRB, 0.56349E+00_JPRB, 0.53665E+00_JPRB, 0.51091E+00_JPRB, 0.49396E+00_JPRB, 0.48371E+00_JPRB, 0.47780E+00_JPRB ,&
&  0.47222E+00_JPRB, 0.46577E+00_JPRB, 0.45816E+00_JPRB, 0.44759E+00_JPRB, 0.44103E+00_JPRB, 0.43817E+00_JPRB, 0.43630E+00_JPRB ,&
&  0.43265E+00_JPRB, 0.42406E+00_JPRB, 0.42139E+00_JPRB, 0.41890E+00_JPRB, 0.41774E+00_JPRB, 0.41641E+00_JPRB, 0.41207E+00_JPRB ,&
&  0.40655E+00_JPRB, 0.40410E+00_JPRB, 0.40805E+00_JPRB, 0.41057E+00_JPRB, 0.41309E+00_JPRB, 0.41848E+00_JPRB, 0.43303E+00_JPRB ,&
&  0.48982E+00_JPRB, 0.49195E+00_JPRB, 0.54771E+00_JPRB, 0.59423E+00_JPRB, 0.58231E+00_JPRB, 0.56708E+00_JPRB, 0.55821E+00_JPRB ,&
&  0.54516E+00_JPRB, 0.53577E+00_JPRB, 0.51532E+00_JPRB, 0.48691E+00_JPRB, 0.46350E+00_JPRB, 0.45829E+00_JPRB, 0.45357E+00_JPRB ,&
&  0.44969E+00_JPRB, 0.44851E+00_JPRB, 0.44829E+00_JPRB, 0.44893E+00_JPRB, 0.44957E+00_JPRB, 0.44930E+00_JPRB, 0.44885E+00_JPRB ,&
&  0.45004E+00_JPRB, 0.45351E+00_JPRB, 0.45755E+00_JPRB, 0.46935E+00_JPRB, 0.51794E+00_JPRB, 0.60995E+00_JPRB, 0.70991E+00_JPRB ,&
&  0.77410E+00_JPRB, 0.83050E+00_JPRB, 0.91025E+00_JPRB, 0.97664E+00_JPRB, 0.10166E+01_JPRB, 0.10292E+01_JPRB, 0.10198E+01_JPRB ,&
&  0.10028E+01_JPRB, 0.96287E+00_JPRB, 0.87859E+00_JPRB, 0.78841E+00_JPRB, 0.72973E+00_JPRB, 0.69348E+00_JPRB, 0.66825E+00_JPRB ,&
&  0.65008E+00_JPRB, 0.62769E+00_JPRB, 0.60454E+00_JPRB, 0.58475E+00_JPRB, 0.57221E+00_JPRB, 0.56074E+00_JPRB, 0.55142E+00_JPRB ,&
&  0.55446E+00_JPRB, 0.56984E+00_JPRB, 0.59172E+00_JPRB, 0.60571E+00_JPRB, 0.61040E+00_JPRB, 0.61328E+00_JPRB, 0.61111E+00_JPRB ,&
&  0.61170E+00_JPRB, 0.61762E+00_JPRB, 0.61652E+00_JPRB, 0.61895E+00_JPRB, 0.59781E+00_JPRB, 0.57953E+00_JPRB, 0.54888E+00_JPRB ,&
&  0.52970E+00_JPRB, 0.51721E+00_JPRB, 0.50463E+00_JPRB, 0.51858E+00_JPRB, 0.56926E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.52111E+00_JPRB, 0.51935E+00_JPRB, 0.51364E+00_JPRB, 0.50257E+00_JPRB, 0.49511E+00_JPRB, 0.48853E+00_JPRB, 0.47966E+00_JPRB ,&
&  0.47062E+00_JPRB, 0.46286E+00_JPRB, 0.45586E+00_JPRB, 0.44695E+00_JPRB, 0.43701E+00_JPRB, 0.43407E+00_JPRB, 0.43374E+00_JPRB ,&
&  0.43077E+00_JPRB, 0.42399E+00_JPRB, 0.41743E+00_JPRB, 0.41600E+00_JPRB, 0.41543E+00_JPRB, 0.41505E+00_JPRB, 0.41216E+00_JPRB ,&
&  0.40730E+00_JPRB, 0.40679E+00_JPRB, 0.41058E+00_JPRB, 0.41200E+00_JPRB, 0.40855E+00_JPRB, 0.41045E+00_JPRB, 0.42087E+00_JPRB ,&
&  0.47042E+00_JPRB, 0.41841E+00_JPRB, 0.45314E+00_JPRB, 0.52213E+00_JPRB, 0.51382E+00_JPRB, 0.51119E+00_JPRB, 0.49589E+00_JPRB ,&
&  0.47978E+00_JPRB, 0.47215E+00_JPRB, 0.46524E+00_JPRB, 0.45193E+00_JPRB, 0.43748E+00_JPRB, 0.42939E+00_JPRB, 0.42853E+00_JPRB ,&
&  0.43174E+00_JPRB, 0.43397E+00_JPRB, 0.43528E+00_JPRB, 0.43187E+00_JPRB, 0.43010E+00_JPRB, 0.42912E+00_JPRB, 0.42910E+00_JPRB ,&
&  0.42931E+00_JPRB, 0.43217E+00_JPRB, 0.43616E+00_JPRB, 0.44024E+00_JPRB, 0.45508E+00_JPRB, 0.50244E+00_JPRB, 0.57855E+00_JPRB ,&
&  0.65150E+00_JPRB, 0.71604E+00_JPRB, 0.76942E+00_JPRB, 0.82016E+00_JPRB, 0.85448E+00_JPRB, 0.88314E+00_JPRB, 0.89375E+00_JPRB ,&
&  0.87403E+00_JPRB, 0.83186E+00_JPRB, 0.78148E+00_JPRB, 0.72543E+00_JPRB, 0.67490E+00_JPRB, 0.63506E+00_JPRB, 0.60429E+00_JPRB ,&
&  0.58119E+00_JPRB, 0.56585E+00_JPRB, 0.55394E+00_JPRB, 0.54066E+00_JPRB, 0.53312E+00_JPRB, 0.53315E+00_JPRB, 0.53861E+00_JPRB ,&
&  0.54858E+00_JPRB, 0.55979E+00_JPRB, 0.56718E+00_JPRB, 0.56919E+00_JPRB, 0.56624E+00_JPRB, 0.56355E+00_JPRB, 0.56142E+00_JPRB ,&
&  0.56770E+00_JPRB, 0.57997E+00_JPRB, 0.59297E+00_JPRB, 0.58300E+00_JPRB, 0.59318E+00_JPRB, 0.58673E+00_JPRB, 0.56447E+00_JPRB ,&
&  0.53997E+00_JPRB, 0.52019E+00_JPRB, 0.50906E+00_JPRB, 0.44880E+00_JPRB, 0.51858E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.48885E+00_JPRB, 0.49003E+00_JPRB, 0.48901E+00_JPRB, 0.48687E+00_JPRB, 0.48400E+00_JPRB, 0.47817E+00_JPRB, 0.46917E+00_JPRB ,&
&  0.46015E+00_JPRB, 0.45135E+00_JPRB, 0.44545E+00_JPRB, 0.43964E+00_JPRB, 0.43079E+00_JPRB, 0.42549E+00_JPRB, 0.42523E+00_JPRB ,&
&  0.42408E+00_JPRB, 0.41960E+00_JPRB, 0.41427E+00_JPRB, 0.41366E+00_JPRB, 0.41202E+00_JPRB, 0.40954E+00_JPRB, 0.40594E+00_JPRB ,&
&  0.40213E+00_JPRB, 0.40108E+00_JPRB, 0.40193E+00_JPRB, 0.40347E+00_JPRB, 0.40073E+00_JPRB, 0.40218E+00_JPRB, 0.41222E+00_JPRB ,&
&  0.45156E+00_JPRB, 0.38710E+00_JPRB, 0.42217E+00_JPRB, 0.46171E+00_JPRB, 0.44875E+00_JPRB, 0.43710E+00_JPRB, 0.42681E+00_JPRB ,&
&  0.41960E+00_JPRB, 0.41767E+00_JPRB, 0.41688E+00_JPRB, 0.41363E+00_JPRB, 0.40987E+00_JPRB, 0.40529E+00_JPRB, 0.40361E+00_JPRB ,&
&  0.40388E+00_JPRB, 0.40821E+00_JPRB, 0.41086E+00_JPRB, 0.41118E+00_JPRB, 0.40863E+00_JPRB, 0.40598E+00_JPRB, 0.40390E+00_JPRB ,&
&  0.40341E+00_JPRB, 0.40509E+00_JPRB, 0.40765E+00_JPRB, 0.41176E+00_JPRB, 0.42048E+00_JPRB, 0.44216E+00_JPRB, 0.48526E+00_JPRB ,&
&  0.54373E+00_JPRB, 0.59540E+00_JPRB, 0.64095E+00_JPRB, 0.69085E+00_JPRB, 0.73704E+00_JPRB, 0.76687E+00_JPRB, 0.77164E+00_JPRB ,&
&  0.75274E+00_JPRB, 0.71439E+00_JPRB, 0.68212E+00_JPRB, 0.65089E+00_JPRB, 0.62080E+00_JPRB, 0.59577E+00_JPRB, 0.57821E+00_JPRB ,&
&  0.56688E+00_JPRB, 0.55948E+00_JPRB, 0.55506E+00_JPRB, 0.55160E+00_JPRB, 0.55184E+00_JPRB, 0.55317E+00_JPRB, 0.55621E+00_JPRB ,&
&  0.55459E+00_JPRB, 0.55004E+00_JPRB, 0.53772E+00_JPRB, 0.52077E+00_JPRB, 0.51002E+00_JPRB, 0.50522E+00_JPRB, 0.50407E+00_JPRB ,&
&  0.50829E+00_JPRB, 0.51887E+00_JPRB, 0.53282E+00_JPRB, 0.55672E+00_JPRB, 0.56502E+00_JPRB, 0.57119E+00_JPRB, 0.56503E+00_JPRB ,&
&  0.54591E+00_JPRB, 0.52420E+00_JPRB, 0.42675E+00_JPRB, 0.47679E+00_JPRB, 0.49077E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.48989E+00_JPRB, 0.48469E+00_JPRB, 0.47986E+00_JPRB, 0.47514E+00_JPRB, 0.46933E+00_JPRB, 0.46191E+00_JPRB, 0.45319E+00_JPRB ,&
&  0.44530E+00_JPRB, 0.43822E+00_JPRB, 0.43123E+00_JPRB, 0.42545E+00_JPRB, 0.42019E+00_JPRB, 0.41814E+00_JPRB, 0.41798E+00_JPRB ,&
&  0.41866E+00_JPRB, 0.41825E+00_JPRB, 0.41727E+00_JPRB, 0.41767E+00_JPRB, 0.41750E+00_JPRB, 0.41531E+00_JPRB, 0.41209E+00_JPRB ,&
&  0.40676E+00_JPRB, 0.40217E+00_JPRB, 0.40068E+00_JPRB, 0.40331E+00_JPRB, 0.40255E+00_JPRB, 0.40436E+00_JPRB, 0.41139E+00_JPRB ,&
&  0.39385E+00_JPRB, 0.36437E+00_JPRB, 0.39251E+00_JPRB, 0.42270E+00_JPRB, 0.41380E+00_JPRB, 0.40491E+00_JPRB, 0.40114E+00_JPRB ,&
&  0.39708E+00_JPRB, 0.39528E+00_JPRB, 0.39382E+00_JPRB, 0.39354E+00_JPRB, 0.39502E+00_JPRB, 0.39467E+00_JPRB, 0.39298E+00_JPRB ,&
&  0.39184E+00_JPRB, 0.39425E+00_JPRB, 0.39690E+00_JPRB, 0.39843E+00_JPRB, 0.39846E+00_JPRB, 0.39760E+00_JPRB, 0.39432E+00_JPRB ,&
&  0.39129E+00_JPRB, 0.38967E+00_JPRB, 0.39008E+00_JPRB, 0.39388E+00_JPRB, 0.40049E+00_JPRB, 0.41267E+00_JPRB, 0.43568E+00_JPRB ,&
&  0.47086E+00_JPRB, 0.50860E+00_JPRB, 0.54285E+00_JPRB, 0.57919E+00_JPRB, 0.62194E+00_JPRB, 0.66101E+00_JPRB, 0.68358E+00_JPRB ,&
&  0.67542E+00_JPRB, 0.64686E+00_JPRB, 0.61350E+00_JPRB, 0.58611E+00_JPRB, 0.56588E+00_JPRB, 0.55290E+00_JPRB, 0.54514E+00_JPRB ,&
&  0.54230E+00_JPRB, 0.54057E+00_JPRB, 0.53938E+00_JPRB, 0.53907E+00_JPRB, 0.53863E+00_JPRB, 0.53542E+00_JPRB, 0.52994E+00_JPRB ,&
&  0.52064E+00_JPRB, 0.50615E+00_JPRB, 0.48967E+00_JPRB, 0.47645E+00_JPRB, 0.46740E+00_JPRB, 0.46380E+00_JPRB, 0.46441E+00_JPRB ,&
&  0.46974E+00_JPRB, 0.48159E+00_JPRB, 0.49759E+00_JPRB, 0.52561E+00_JPRB, 0.54528E+00_JPRB, 0.55894E+00_JPRB, 0.56375E+00_JPRB ,&
&  0.55349E+00_JPRB, 0.53312E+00_JPRB, 0.51352E+00_JPRB, 0.50245E+00_JPRB, 0.49826E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.48485E+00_JPRB, 0.47244E+00_JPRB, 0.46013E+00_JPRB, 0.45194E+00_JPRB, 0.44600E+00_JPRB, 0.44012E+00_JPRB, 0.43348E+00_JPRB ,&
&  0.42564E+00_JPRB, 0.41813E+00_JPRB, 0.41115E+00_JPRB, 0.40458E+00_JPRB, 0.40149E+00_JPRB, 0.40018E+00_JPRB, 0.40098E+00_JPRB ,&
&  0.40380E+00_JPRB, 0.40854E+00_JPRB, 0.41073E+00_JPRB, 0.41077E+00_JPRB, 0.41156E+00_JPRB, 0.41146E+00_JPRB, 0.40986E+00_JPRB ,&
&  0.40592E+00_JPRB, 0.40109E+00_JPRB, 0.39855E+00_JPRB, 0.39944E+00_JPRB, 0.39973E+00_JPRB, 0.40025E+00_JPRB, 0.39994E+00_JPRB ,&
&  0.40132E+00_JPRB, 0.36608E+00_JPRB, 0.38887E+00_JPRB, 0.40270E+00_JPRB, 0.39948E+00_JPRB, 0.39749E+00_JPRB, 0.39378E+00_JPRB ,&
&  0.38775E+00_JPRB, 0.38451E+00_JPRB, 0.38300E+00_JPRB, 0.38466E+00_JPRB, 0.38766E+00_JPRB, 0.38891E+00_JPRB, 0.38899E+00_JPRB ,&
&  0.38620E+00_JPRB, 0.38631E+00_JPRB, 0.38605E+00_JPRB, 0.38645E+00_JPRB, 0.38749E+00_JPRB, 0.38860E+00_JPRB, 0.38884E+00_JPRB ,&
&  0.38771E+00_JPRB, 0.38499E+00_JPRB, 0.38228E+00_JPRB, 0.38228E+00_JPRB, 0.38408E+00_JPRB, 0.38846E+00_JPRB, 0.39740E+00_JPRB ,&
&  0.41274E+00_JPRB, 0.43231E+00_JPRB, 0.45516E+00_JPRB, 0.48240E+00_JPRB, 0.51870E+00_JPRB, 0.55470E+00_JPRB, 0.58248E+00_JPRB ,&
&  0.59001E+00_JPRB, 0.58259E+00_JPRB, 0.56545E+00_JPRB, 0.54442E+00_JPRB, 0.52565E+00_JPRB, 0.50780E+00_JPRB, 0.49180E+00_JPRB ,&
&  0.47991E+00_JPRB, 0.47043E+00_JPRB, 0.46384E+00_JPRB, 0.46046E+00_JPRB, 0.45749E+00_JPRB, 0.45270E+00_JPRB, 0.44810E+00_JPRB ,&
&  0.44333E+00_JPRB, 0.43905E+00_JPRB, 0.43592E+00_JPRB, 0.43304E+00_JPRB, 0.42964E+00_JPRB, 0.42672E+00_JPRB, 0.42677E+00_JPRB ,&
&  0.43091E+00_JPRB, 0.43912E+00_JPRB, 0.45329E+00_JPRB, 0.47325E+00_JPRB, 0.49553E+00_JPRB, 0.51292E+00_JPRB, 0.52248E+00_JPRB ,&
&  0.52310E+00_JPRB, 0.51916E+00_JPRB, 0.51044E+00_JPRB, 0.50234E+00_JPRB, 0.49537E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.46552E+00_JPRB, 0.45576E+00_JPRB, 0.44578E+00_JPRB, 0.43797E+00_JPRB, 0.43195E+00_JPRB, 0.42545E+00_JPRB, 0.41814E+00_JPRB ,&
&  0.41226E+00_JPRB, 0.40560E+00_JPRB, 0.40072E+00_JPRB, 0.39648E+00_JPRB, 0.39324E+00_JPRB, 0.39231E+00_JPRB, 0.39328E+00_JPRB ,&
&  0.39593E+00_JPRB, 0.40140E+00_JPRB, 0.40441E+00_JPRB, 0.40324E+00_JPRB, 0.40516E+00_JPRB, 0.40756E+00_JPRB, 0.40820E+00_JPRB ,&
&  0.40607E+00_JPRB, 0.40316E+00_JPRB, 0.39992E+00_JPRB, 0.39800E+00_JPRB, 0.39798E+00_JPRB, 0.39883E+00_JPRB, 0.39551E+00_JPRB ,&
&  0.38936E+00_JPRB, 0.38001E+00_JPRB, 0.38960E+00_JPRB, 0.39468E+00_JPRB, 0.39512E+00_JPRB, 0.39288E+00_JPRB, 0.38954E+00_JPRB ,&
&  0.38427E+00_JPRB, 0.38124E+00_JPRB, 0.38213E+00_JPRB, 0.38429E+00_JPRB, 0.38669E+00_JPRB, 0.38769E+00_JPRB, 0.38864E+00_JPRB ,&
&  0.38552E+00_JPRB, 0.38404E+00_JPRB, 0.38263E+00_JPRB, 0.38198E+00_JPRB, 0.38305E+00_JPRB, 0.38436E+00_JPRB, 0.38516E+00_JPRB ,&
&  0.38437E+00_JPRB, 0.38524E+00_JPRB, 0.38639E+00_JPRB, 0.38593E+00_JPRB, 0.38765E+00_JPRB, 0.39013E+00_JPRB, 0.39372E+00_JPRB ,&
&  0.39937E+00_JPRB, 0.40782E+00_JPRB, 0.41791E+00_JPRB, 0.43233E+00_JPRB, 0.45494E+00_JPRB, 0.47981E+00_JPRB, 0.49930E+00_JPRB ,&
&  0.50917E+00_JPRB, 0.50963E+00_JPRB, 0.50392E+00_JPRB, 0.49325E+00_JPRB, 0.48254E+00_JPRB, 0.47240E+00_JPRB, 0.46055E+00_JPRB ,&
&  0.44726E+00_JPRB, 0.43748E+00_JPRB, 0.43006E+00_JPRB, 0.42469E+00_JPRB, 0.41969E+00_JPRB, 0.41594E+00_JPRB, 0.41435E+00_JPRB ,&
&  0.41483E+00_JPRB, 0.41566E+00_JPRB, 0.41521E+00_JPRB, 0.41421E+00_JPRB, 0.41410E+00_JPRB, 0.41227E+00_JPRB, 0.41130E+00_JPRB ,&
&  0.41337E+00_JPRB, 0.41875E+00_JPRB, 0.42788E+00_JPRB, 0.44246E+00_JPRB, 0.45708E+00_JPRB, 0.46921E+00_JPRB, 0.47825E+00_JPRB ,&
&  0.48419E+00_JPRB, 0.48651E+00_JPRB, 0.48475E+00_JPRB, 0.48081E+00_JPRB, 0.47458E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.42881E+00_JPRB, 0.42397E+00_JPRB, 0.41828E+00_JPRB, 0.41273E+00_JPRB, 0.40756E+00_JPRB, 0.40229E+00_JPRB, 0.39747E+00_JPRB ,&
&  0.39323E+00_JPRB, 0.39077E+00_JPRB, 0.38927E+00_JPRB, 0.38810E+00_JPRB, 0.38499E+00_JPRB, 0.38367E+00_JPRB, 0.38452E+00_JPRB ,&
&  0.38702E+00_JPRB, 0.38954E+00_JPRB, 0.39142E+00_JPRB, 0.39115E+00_JPRB, 0.39359E+00_JPRB, 0.39594E+00_JPRB, 0.39629E+00_JPRB ,&
&  0.39565E+00_JPRB, 0.39504E+00_JPRB, 0.39316E+00_JPRB, 0.39062E+00_JPRB, 0.39063E+00_JPRB, 0.39082E+00_JPRB, 0.38689E+00_JPRB ,&
&  0.37974E+00_JPRB, 0.37976E+00_JPRB, 0.37203E+00_JPRB, 0.38389E+00_JPRB, 0.38640E+00_JPRB, 0.38170E+00_JPRB, 0.37602E+00_JPRB ,&
&  0.37465E+00_JPRB, 0.37615E+00_JPRB, 0.37963E+00_JPRB, 0.38185E+00_JPRB, 0.38251E+00_JPRB, 0.38260E+00_JPRB, 0.38255E+00_JPRB ,&
&  0.37549E+00_JPRB, 0.37159E+00_JPRB, 0.37162E+00_JPRB, 0.37280E+00_JPRB, 0.37310E+00_JPRB, 0.37394E+00_JPRB, 0.37596E+00_JPRB ,&
&  0.37751E+00_JPRB, 0.37858E+00_JPRB, 0.37982E+00_JPRB, 0.38255E+00_JPRB, 0.38453E+00_JPRB, 0.38557E+00_JPRB, 0.38754E+00_JPRB ,&
&  0.39066E+00_JPRB, 0.39491E+00_JPRB, 0.39932E+00_JPRB, 0.40399E+00_JPRB, 0.41375E+00_JPRB, 0.42792E+00_JPRB, 0.44005E+00_JPRB ,&
&  0.44613E+00_JPRB, 0.44748E+00_JPRB, 0.44584E+00_JPRB, 0.44025E+00_JPRB, 0.43192E+00_JPRB, 0.42483E+00_JPRB, 0.41873E+00_JPRB ,&
&  0.41043E+00_JPRB, 0.40523E+00_JPRB, 0.40100E+00_JPRB, 0.39568E+00_JPRB, 0.39105E+00_JPRB, 0.38818E+00_JPRB, 0.38687E+00_JPRB ,&
&  0.38745E+00_JPRB, 0.38979E+00_JPRB, 0.39093E+00_JPRB, 0.39271E+00_JPRB, 0.39596E+00_JPRB, 0.39950E+00_JPRB, 0.39971E+00_JPRB ,&
&  0.39856E+00_JPRB, 0.39832E+00_JPRB, 0.40043E+00_JPRB, 0.40538E+00_JPRB, 0.41097E+00_JPRB, 0.41770E+00_JPRB, 0.42479E+00_JPRB ,&
&  0.43102E+00_JPRB, 0.43497E+00_JPRB, 0.43597E+00_JPRB, 0.43578E+00_JPRB, 0.43304E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.39516E+00_JPRB, 0.39353E+00_JPRB, 0.39161E+00_JPRB, 0.38868E+00_JPRB, 0.38548E+00_JPRB, 0.38263E+00_JPRB, 0.38031E+00_JPRB ,&
&  0.37885E+00_JPRB, 0.37864E+00_JPRB, 0.37928E+00_JPRB, 0.37956E+00_JPRB, 0.37946E+00_JPRB, 0.38033E+00_JPRB, 0.38071E+00_JPRB ,&
&  0.38229E+00_JPRB, 0.38264E+00_JPRB, 0.38062E+00_JPRB, 0.38031E+00_JPRB, 0.38292E+00_JPRB, 0.38771E+00_JPRB, 0.38965E+00_JPRB ,&
&  0.38749E+00_JPRB, 0.38740E+00_JPRB, 0.38732E+00_JPRB, 0.38537E+00_JPRB, 0.38392E+00_JPRB, 0.38285E+00_JPRB, 0.37879E+00_JPRB ,&
&  0.37376E+00_JPRB, 0.37346E+00_JPRB, 0.37514E+00_JPRB, 0.37772E+00_JPRB, 0.37690E+00_JPRB, 0.37446E+00_JPRB, 0.37477E+00_JPRB ,&
&  0.37311E+00_JPRB, 0.37312E+00_JPRB, 0.37315E+00_JPRB, 0.37390E+00_JPRB, 0.37240E+00_JPRB, 0.37013E+00_JPRB, 0.36588E+00_JPRB ,&
&  0.35734E+00_JPRB, 0.35480E+00_JPRB, 0.35703E+00_JPRB, 0.36135E+00_JPRB, 0.36097E+00_JPRB, 0.36182E+00_JPRB, 0.36492E+00_JPRB ,&
&  0.36880E+00_JPRB, 0.37163E+00_JPRB, 0.37341E+00_JPRB, 0.37467E+00_JPRB, 0.37639E+00_JPRB, 0.37803E+00_JPRB, 0.37897E+00_JPRB ,&
&  0.38016E+00_JPRB, 0.38220E+00_JPRB, 0.38486E+00_JPRB, 0.38811E+00_JPRB, 0.39159E+00_JPRB, 0.39684E+00_JPRB, 0.40204E+00_JPRB ,&
&  0.40461E+00_JPRB, 0.40366E+00_JPRB, 0.40139E+00_JPRB, 0.39676E+00_JPRB, 0.39078E+00_JPRB, 0.38607E+00_JPRB, 0.38244E+00_JPRB ,&
&  0.37675E+00_JPRB, 0.37179E+00_JPRB, 0.36790E+00_JPRB, 0.36612E+00_JPRB, 0.36540E+00_JPRB, 0.36580E+00_JPRB, 0.36734E+00_JPRB ,&
&  0.36870E+00_JPRB, 0.37127E+00_JPRB, 0.37730E+00_JPRB, 0.38124E+00_JPRB, 0.38308E+00_JPRB, 0.38612E+00_JPRB, 0.38769E+00_JPRB ,&
&  0.38617E+00_JPRB, 0.38487E+00_JPRB, 0.38396E+00_JPRB, 0.38615E+00_JPRB, 0.38799E+00_JPRB, 0.38840E+00_JPRB, 0.38920E+00_JPRB ,&
&  0.39227E+00_JPRB, 0.39557E+00_JPRB, 0.39632E+00_JPRB, 0.39637E+00_JPRB, 0.39647E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.36495E+00_JPRB, 0.36510E+00_JPRB, 0.36462E+00_JPRB, 0.36368E+00_JPRB, 0.36277E+00_JPRB, 0.36205E+00_JPRB, 0.36227E+00_JPRB ,&
&  0.36260E+00_JPRB, 0.36338E+00_JPRB, 0.36437E+00_JPRB, 0.36519E+00_JPRB, 0.36693E+00_JPRB, 0.36890E+00_JPRB, 0.37048E+00_JPRB ,&
&  0.37218E+00_JPRB, 0.37163E+00_JPRB, 0.36923E+00_JPRB, 0.36683E+00_JPRB, 0.36756E+00_JPRB, 0.37287E+00_JPRB, 0.37878E+00_JPRB ,&
&  0.38027E+00_JPRB, 0.38135E+00_JPRB, 0.38184E+00_JPRB, 0.38073E+00_JPRB, 0.37922E+00_JPRB, 0.37866E+00_JPRB, 0.37493E+00_JPRB ,&
&  0.37031E+00_JPRB, 0.36868E+00_JPRB, 0.36823E+00_JPRB, 0.36543E+00_JPRB, 0.36476E+00_JPRB, 0.36675E+00_JPRB, 0.36551E+00_JPRB ,&
&  0.36455E+00_JPRB, 0.36299E+00_JPRB, 0.36019E+00_JPRB, 0.35779E+00_JPRB, 0.35490E+00_JPRB, 0.34923E+00_JPRB, 0.34263E+00_JPRB ,&
&  0.33733E+00_JPRB, 0.33757E+00_JPRB, 0.34284E+00_JPRB, 0.34844E+00_JPRB, 0.34767E+00_JPRB, 0.34687E+00_JPRB, 0.34846E+00_JPRB ,&
&  0.35202E+00_JPRB, 0.35645E+00_JPRB, 0.35928E+00_JPRB, 0.35851E+00_JPRB, 0.35745E+00_JPRB, 0.35934E+00_JPRB, 0.36199E+00_JPRB ,&
&  0.36329E+00_JPRB, 0.36476E+00_JPRB, 0.36586E+00_JPRB, 0.36753E+00_JPRB, 0.36860E+00_JPRB, 0.36844E+00_JPRB, 0.36938E+00_JPRB ,&
&  0.36864E+00_JPRB, 0.36639E+00_JPRB, 0.36236E+00_JPRB, 0.35695E+00_JPRB, 0.35249E+00_JPRB, 0.34926E+00_JPRB, 0.34848E+00_JPRB ,&
&  0.34716E+00_JPRB, 0.34805E+00_JPRB, 0.34794E+00_JPRB, 0.34544E+00_JPRB, 0.34225E+00_JPRB, 0.34204E+00_JPRB, 0.34542E+00_JPRB ,&
&  0.35026E+00_JPRB, 0.35373E+00_JPRB, 0.35935E+00_JPRB, 0.36202E+00_JPRB, 0.36386E+00_JPRB, 0.36626E+00_JPRB, 0.36697E+00_JPRB ,&
&  0.36446E+00_JPRB, 0.36246E+00_JPRB, 0.36210E+00_JPRB, 0.36500E+00_JPRB, 0.36584E+00_JPRB, 0.36484E+00_JPRB, 0.36211E+00_JPRB ,&
&  0.36125E+00_JPRB, 0.36295E+00_JPRB, 0.36353E+00_JPRB, 0.36315E+00_JPRB, 0.36366E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.34303E+00_JPRB, 0.34515E+00_JPRB, 0.34731E+00_JPRB, 0.34926E+00_JPRB, 0.35055E+00_JPRB, 0.35157E+00_JPRB, 0.35081E+00_JPRB ,&
&  0.35072E+00_JPRB, 0.35239E+00_JPRB, 0.35452E+00_JPRB, 0.35627E+00_JPRB, 0.35727E+00_JPRB, 0.35920E+00_JPRB, 0.36109E+00_JPRB ,&
&  0.36236E+00_JPRB, 0.36158E+00_JPRB, 0.35841E+00_JPRB, 0.35704E+00_JPRB, 0.35596E+00_JPRB, 0.36084E+00_JPRB, 0.36921E+00_JPRB ,&
&  0.37286E+00_JPRB, 0.37513E+00_JPRB, 0.37792E+00_JPRB, 0.37847E+00_JPRB, 0.37867E+00_JPRB, 0.37828E+00_JPRB, 0.37376E+00_JPRB ,&
&  0.36877E+00_JPRB, 0.36405E+00_JPRB, 0.36122E+00_JPRB, 0.35787E+00_JPRB, 0.35761E+00_JPRB, 0.35519E+00_JPRB, 0.35340E+00_JPRB ,&
&  0.35214E+00_JPRB, 0.34970E+00_JPRB, 0.34678E+00_JPRB, 0.34182E+00_JPRB, 0.33564E+00_JPRB, 0.32764E+00_JPRB, 0.32222E+00_JPRB ,&
&  0.32015E+00_JPRB, 0.32371E+00_JPRB, 0.33203E+00_JPRB, 0.33592E+00_JPRB, 0.33289E+00_JPRB, 0.33234E+00_JPRB, 0.33561E+00_JPRB ,&
&  0.34036E+00_JPRB, 0.34557E+00_JPRB, 0.34739E+00_JPRB, 0.34658E+00_JPRB, 0.34470E+00_JPRB, 0.34627E+00_JPRB, 0.35043E+00_JPRB ,&
&  0.35409E+00_JPRB, 0.35607E+00_JPRB, 0.35542E+00_JPRB, 0.35464E+00_JPRB, 0.35234E+00_JPRB, 0.34703E+00_JPRB, 0.33982E+00_JPRB ,&
&  0.33324E+00_JPRB, 0.32912E+00_JPRB, 0.32595E+00_JPRB, 0.32279E+00_JPRB, 0.32155E+00_JPRB, 0.32184E+00_JPRB, 0.32396E+00_JPRB ,&
&  0.32476E+00_JPRB, 0.32658E+00_JPRB, 0.32665E+00_JPRB, 0.32247E+00_JPRB, 0.31916E+00_JPRB, 0.32057E+00_JPRB, 0.32233E+00_JPRB ,&
&  0.32501E+00_JPRB, 0.32761E+00_JPRB, 0.33022E+00_JPRB, 0.33130E+00_JPRB, 0.33285E+00_JPRB, 0.33546E+00_JPRB, 0.33698E+00_JPRB ,&
&  0.33773E+00_JPRB, 0.33830E+00_JPRB, 0.34009E+00_JPRB, 0.34305E+00_JPRB, 0.34577E+00_JPRB, 0.34787E+00_JPRB, 0.34707E+00_JPRB ,&
&  0.34564E+00_JPRB, 0.34409E+00_JPRB, 0.34442E+00_JPRB, 0.34458E+00_JPRB, 0.34303E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.33798E+00_JPRB, 0.33784E+00_JPRB, 0.34101E+00_JPRB, 0.34447E+00_JPRB, 0.34531E+00_JPRB, 0.34514E+00_JPRB, 0.34272E+00_JPRB ,&
&  0.34109E+00_JPRB, 0.34226E+00_JPRB, 0.34403E+00_JPRB, 0.34527E+00_JPRB, 0.34730E+00_JPRB, 0.35001E+00_JPRB, 0.35216E+00_JPRB ,&
&  0.35224E+00_JPRB, 0.35051E+00_JPRB, 0.34750E+00_JPRB, 0.34820E+00_JPRB, 0.34818E+00_JPRB, 0.35015E+00_JPRB, 0.35746E+00_JPRB ,&
&  0.36267E+00_JPRB, 0.36623E+00_JPRB, 0.36923E+00_JPRB, 0.36955E+00_JPRB, 0.36892E+00_JPRB, 0.37028E+00_JPRB, 0.36693E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.35495E+00_JPRB, 0.34990E+00_JPRB, 0.34295E+00_JPRB, 0.34545E+00_JPRB, 0.34289E+00_JPRB, 0.33978E+00_JPRB ,&
&  0.33568E+00_JPRB, 0.33055E+00_JPRB, 0.32463E+00_JPRB, 0.31908E+00_JPRB, 0.31501E+00_JPRB, 0.31127E+00_JPRB, 0.30632E+00_JPRB ,&
&  0.30474E+00_JPRB, 0.30668E+00_JPRB, 0.31232E+00_JPRB, 0.31501E+00_JPRB, 0.31266E+00_JPRB, 0.31365E+00_JPRB, 0.32032E+00_JPRB ,&
&  0.32708E+00_JPRB, 0.33022E+00_JPRB, 0.33008E+00_JPRB, 0.33065E+00_JPRB, 0.33127E+00_JPRB, 0.33262E+00_JPRB, 0.33326E+00_JPRB ,&
&  0.33224E+00_JPRB, 0.33063E+00_JPRB, 0.32815E+00_JPRB, 0.32227E+00_JPRB, 0.31533E+00_JPRB, 0.30963E+00_JPRB, 0.30744E+00_JPRB ,&
&  0.31018E+00_JPRB, 0.31007E+00_JPRB, 0.31015E+00_JPRB, 0.31176E+00_JPRB, 0.31230E+00_JPRB, 0.31153E+00_JPRB, 0.31053E+00_JPRB ,&
&  0.31174E+00_JPRB, 0.31098E+00_JPRB, 0.31245E+00_JPRB, 0.30967E+00_JPRB, 0.30525E+00_JPRB, 0.30106E+00_JPRB, 0.29712E+00_JPRB ,&
&  0.29484E+00_JPRB, 0.29407E+00_JPRB, 0.29610E+00_JPRB, 0.29675E+00_JPRB, 0.29851E+00_JPRB, 0.30063E+00_JPRB, 0.30243E+00_JPRB ,&
&  0.30342E+00_JPRB, 0.30470E+00_JPRB, 0.30717E+00_JPRB, 0.31223E+00_JPRB, 0.31762E+00_JPRB, 0.32234E+00_JPRB, 0.32724E+00_JPRB ,&
&  0.33180E+00_JPRB, 0.33505E+00_JPRB, 0.34083E+00_JPRB, 0.34198E+00_JPRB, 0.34113E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.33716E+00_JPRB, 0.33483E+00_JPRB, 0.33492E+00_JPRB, 0.33578E+00_JPRB, 0.33538E+00_JPRB, 0.33467E+00_JPRB, 0.33215E+00_JPRB ,&
&  0.32989E+00_JPRB, 0.33026E+00_JPRB, 0.33161E+00_JPRB, 0.33409E+00_JPRB, 0.33842E+00_JPRB, 0.34303E+00_JPRB, 0.34241E+00_JPRB ,&
&  0.33932E+00_JPRB, 0.33798E+00_JPRB, 0.34013E+00_JPRB, 0.34229E+00_JPRB, 0.34268E+00_JPRB, 0.34330E+00_JPRB, 0.34685E+00_JPRB ,&
&  0.35137E+00_JPRB, 0.35336E+00_JPRB, 0.35619E+00_JPRB, 0.35570E+00_JPRB, 0.35197E+00_JPRB, 0.35202E+00_JPRB, 0.35018E+00_JPRB ,&
&  0.34531E+00_JPRB, 0.33946E+00_JPRB, 0.33049E+00_JPRB, 0.33145E+00_JPRB, 0.32784E+00_JPRB, 0.32491E+00_JPRB, 0.32058E+00_JPRB ,&
&  0.31584E+00_JPRB, 0.31194E+00_JPRB, 0.30755E+00_JPRB, 0.30399E+00_JPRB, 0.30215E+00_JPRB, 0.30206E+00_JPRB, 0.29994E+00_JPRB ,&
&  0.29857E+00_JPRB, 0.30007E+00_JPRB, 0.30141E+00_JPRB, 0.29890E+00_JPRB, 0.29902E+00_JPRB, 0.30550E+00_JPRB, 0.31064E+00_JPRB ,&
&  0.31251E+00_JPRB, 0.31243E+00_JPRB, 0.30989E+00_JPRB, 0.30926E+00_JPRB, 0.30820E+00_JPRB, 0.30622E+00_JPRB, 0.30290E+00_JPRB ,&
&  0.29800E+00_JPRB, 0.29564E+00_JPRB, 0.29406E+00_JPRB, 0.29081E+00_JPRB, 0.28571E+00_JPRB, 0.26875E+00_JPRB, 0.25939E+00_JPRB ,&
&  0.25913E+00_JPRB, 0.26799E+00_JPRB, 0.28310E+00_JPRB, 0.28953E+00_JPRB, 0.29487E+00_JPRB, 0.29984E+00_JPRB, 0.30076E+00_JPRB ,&
&  0.30248E+00_JPRB, 0.28209E+00_JPRB, 0.27098E+00_JPRB, 0.26246E+00_JPRB, 0.25487E+00_JPRB, 0.25001E+00_JPRB, 0.24870E+00_JPRB ,&
&  0.25155E+00_JPRB, 0.25753E+00_JPRB, 0.26268E+00_JPRB, 0.26513E+00_JPRB, 0.26477E+00_JPRB, 0.26154E+00_JPRB, 0.25824E+00_JPRB ,&
&  0.25778E+00_JPRB, 0.26415E+00_JPRB, 0.27678E+00_JPRB, 0.28978E+00_JPRB, 0.29860E+00_JPRB, 0.30799E+00_JPRB, 0.31595E+00_JPRB ,&
&  0.32148E+00_JPRB, 0.32665E+00_JPRB, 0.33308E+00_JPRB, 0.33554E+00_JPRB, 0.33674E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.32689E+00_JPRB, 0.32352E+00_JPRB, 0.32367E+00_JPRB, 0.32404E+00_JPRB, 0.32294E+00_JPRB, 0.32289E+00_JPRB, 0.32226E+00_JPRB ,&
&  0.32204E+00_JPRB, 0.32017E+00_JPRB, 0.31822E+00_JPRB, 0.31852E+00_JPRB, 0.32052E+00_JPRB, 0.32280E+00_JPRB, 0.32336E+00_JPRB ,&
&  0.32313E+00_JPRB, 0.32442E+00_JPRB, 0.32769E+00_JPRB, 0.33121E+00_JPRB, 0.33196E+00_JPRB, 0.33224E+00_JPRB, 0.33401E+00_JPRB ,&
&  0.33628E+00_JPRB, 0.33707E+00_JPRB, 0.33976E+00_JPRB, 0.33756E+00_JPRB, 0.33308E+00_JPRB, 0.33115E+00_JPRB, 0.32946E+00_JPRB ,&
&  0.32335E+00_JPRB, 0.30041E+00_JPRB, 0.28527E+00_JPRB, 0.28929E+00_JPRB, 0.30524E+00_JPRB, 0.30493E+00_JPRB, 0.30287E+00_JPRB ,&
&  0.30074E+00_JPRB, 0.29790E+00_JPRB, 0.29544E+00_JPRB, 0.29450E+00_JPRB, 0.29451E+00_JPRB, 0.29436E+00_JPRB, 0.29283E+00_JPRB ,&
&  0.29310E+00_JPRB, 0.29753E+00_JPRB, 0.30345E+00_JPRB, 0.30122E+00_JPRB, 0.29740E+00_JPRB, 0.29083E+00_JPRB, 0.28666E+00_JPRB ,&
&  0.28272E+00_JPRB, 0.27852E+00_JPRB, 0.27606E+00_JPRB, 0.27524E+00_JPRB, 0.27494E+00_JPRB, 0.27302E+00_JPRB, 0.26978E+00_JPRB ,&
&  0.26210E+00_JPRB, 0.25322E+00_JPRB, 0.24389E+00_JPRB, 0.23472E+00_JPRB, 0.22781E+00_JPRB, 0.22302E+00_JPRB, 0.21941E+00_JPRB ,&
&  0.22095E+00_JPRB, 0.23162E+00_JPRB, 0.24968E+00_JPRB, 0.26929E+00_JPRB, 0.29004E+00_JPRB, 0.26754E+00_JPRB, 0.25655E+00_JPRB ,&
&  0.24547E+00_JPRB, 0.23602E+00_JPRB, 0.22951E+00_JPRB, 0.22507E+00_JPRB, 0.22195E+00_JPRB, 0.22032E+00_JPRB, 0.22039E+00_JPRB ,&
&  0.22329E+00_JPRB, 0.22727E+00_JPRB, 0.23089E+00_JPRB, 0.23171E+00_JPRB, 0.23095E+00_JPRB, 0.22918E+00_JPRB, 0.22818E+00_JPRB ,&
&  0.22923E+00_JPRB, 0.23432E+00_JPRB, 0.24297E+00_JPRB, 0.25547E+00_JPRB, 0.26695E+00_JPRB, 0.27887E+00_JPRB, 0.29292E+00_JPRB ,&
&  0.30807E+00_JPRB, 0.32270E+00_JPRB, 0.32641E+00_JPRB, 0.32953E+00_JPRB, 0.32967E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.32327E+00_JPRB, 0.31992E+00_JPRB, 0.31804E+00_JPRB, 0.31838E+00_JPRB, 0.31861E+00_JPRB, 0.32128E+00_JPRB, 0.32056E+00_JPRB ,&
&  0.31620E+00_JPRB, 0.31157E+00_JPRB, 0.31013E+00_JPRB, 0.30993E+00_JPRB, 0.30879E+00_JPRB, 0.30759E+00_JPRB, 0.30825E+00_JPRB ,&
&  0.30675E+00_JPRB, 0.30853E+00_JPRB, 0.31129E+00_JPRB, 0.31273E+00_JPRB, 0.31347E+00_JPRB, 0.31365E+00_JPRB, 0.31378E+00_JPRB ,&
&  0.30767E+00_JPRB, 0.30601E+00_JPRB, 0.31157E+00_JPRB, 0.30890E+00_JPRB, 0.30505E+00_JPRB, 0.30172E+00_JPRB, 0.30054E+00_JPRB ,&
&  0.29230E+00_JPRB, 0.28088E+00_JPRB, 0.26808E+00_JPRB, 0.27322E+00_JPRB, 0.29008E+00_JPRB, 0.28794E+00_JPRB, 0.28569E+00_JPRB ,&
&  0.28537E+00_JPRB, 0.28467E+00_JPRB, 0.28548E+00_JPRB, 0.28598E+00_JPRB, 0.28686E+00_JPRB, 0.28695E+00_JPRB, 0.28803E+00_JPRB ,&
&  0.28941E+00_JPRB, 0.29504E+00_JPRB, 0.29867E+00_JPRB, 0.27986E+00_JPRB, 0.26332E+00_JPRB, 0.25066E+00_JPRB, 0.23864E+00_JPRB ,&
&  0.22796E+00_JPRB, 0.22511E+00_JPRB, 0.22166E+00_JPRB, 0.21586E+00_JPRB, 0.21642E+00_JPRB, 0.21699E+00_JPRB, 0.21627E+00_JPRB ,&
&  0.21444E+00_JPRB, 0.21332E+00_JPRB, 0.21192E+00_JPRB, 0.20961E+00_JPRB, 0.20872E+00_JPRB, 0.20993E+00_JPRB, 0.21331E+00_JPRB ,&
&  0.21870E+00_JPRB, 0.22493E+00_JPRB, 0.23902E+00_JPRB, 0.25118E+00_JPRB, 0.24797E+00_JPRB, 0.24012E+00_JPRB, 0.23264E+00_JPRB ,&
&  0.22253E+00_JPRB, 0.21754E+00_JPRB, 0.21266E+00_JPRB, 0.20871E+00_JPRB, 0.20697E+00_JPRB, 0.20732E+00_JPRB, 0.20828E+00_JPRB ,&
&  0.20959E+00_JPRB, 0.21157E+00_JPRB, 0.21238E+00_JPRB, 0.21200E+00_JPRB, 0.21124E+00_JPRB, 0.21171E+00_JPRB, 0.21325E+00_JPRB ,&
&  0.21614E+00_JPRB, 0.22112E+00_JPRB, 0.22720E+00_JPRB, 0.23470E+00_JPRB, 0.24124E+00_JPRB, 0.24733E+00_JPRB, 0.25668E+00_JPRB ,&
&  0.26456E+00_JPRB, 0.28249E+00_JPRB, 0.30827E+00_JPRB, 0.33447E+00_JPRB, 0.32977E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.30580E+00_JPRB, 0.30318E+00_JPRB, 0.30067E+00_JPRB, 0.30041E+00_JPRB, 0.30104E+00_JPRB, 0.30388E+00_JPRB, 0.30149E+00_JPRB ,&
&  0.29698E+00_JPRB, 0.29531E+00_JPRB, 0.29668E+00_JPRB, 0.29688E+00_JPRB, 0.29622E+00_JPRB, 0.28567E+00_JPRB, 0.27970E+00_JPRB ,&
&  0.27294E+00_JPRB, 0.27060E+00_JPRB, 0.26877E+00_JPRB, 0.26955E+00_JPRB, 0.27185E+00_JPRB, 0.27328E+00_JPRB, 0.26919E+00_JPRB ,&
&  0.26478E+00_JPRB, 0.26053E+00_JPRB, 0.25618E+00_JPRB, 0.25434E+00_JPRB, 0.25565E+00_JPRB, 0.25710E+00_JPRB, 0.25733E+00_JPRB ,&
&  0.25804E+00_JPRB, 0.26075E+00_JPRB, 0.26327E+00_JPRB, 0.26674E+00_JPRB, 0.27943E+00_JPRB, 0.27778E+00_JPRB, 0.27647E+00_JPRB ,&
&  0.27672E+00_JPRB, 0.27708E+00_JPRB, 0.27808E+00_JPRB, 0.27866E+00_JPRB, 0.28249E+00_JPRB, 0.28556E+00_JPRB, 0.28770E+00_JPRB ,&
&  0.27392E+00_JPRB, 0.26577E+00_JPRB, 0.25787E+00_JPRB, 0.24876E+00_JPRB, 0.23841E+00_JPRB, 0.23000E+00_JPRB, 0.22221E+00_JPRB ,&
&  0.21560E+00_JPRB, 0.20941E+00_JPRB, 0.20386E+00_JPRB, 0.20167E+00_JPRB, 0.20090E+00_JPRB, 0.19977E+00_JPRB, 0.19972E+00_JPRB ,&
&  0.20029E+00_JPRB, 0.20104E+00_JPRB, 0.20156E+00_JPRB, 0.20305E+00_JPRB, 0.20535E+00_JPRB, 0.20895E+00_JPRB, 0.21280E+00_JPRB ,&
&  0.21858E+00_JPRB, 0.22296E+00_JPRB, 0.22805E+00_JPRB, 0.23218E+00_JPRB, 0.23069E+00_JPRB, 0.22574E+00_JPRB, 0.22037E+00_JPRB ,&
&  0.21405E+00_JPRB, 0.21043E+00_JPRB, 0.20681E+00_JPRB, 0.20421E+00_JPRB, 0.20277E+00_JPRB, 0.20197E+00_JPRB, 0.20235E+00_JPRB ,&
&  0.20256E+00_JPRB, 0.20307E+00_JPRB, 0.20269E+00_JPRB, 0.20296E+00_JPRB, 0.20380E+00_JPRB, 0.20559E+00_JPRB, 0.20791E+00_JPRB ,&
&  0.21149E+00_JPRB, 0.21564E+00_JPRB, 0.22064E+00_JPRB, 0.22613E+00_JPRB, 0.23410E+00_JPRB, 0.24470E+00_JPRB, 0.25991E+00_JPRB ,&
&  0.27664E+00_JPRB, 0.30868E+00_JPRB, 0.32155E+00_JPRB, 0.31212E+00_JPRB, 0.30908E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.29095E+00_JPRB, 0.29214E+00_JPRB, 0.29353E+00_JPRB, 0.29585E+00_JPRB, 0.29867E+00_JPRB, 0.29967E+00_JPRB, 0.29988E+00_JPRB ,&
&  0.29066E+00_JPRB, 0.28298E+00_JPRB, 0.27616E+00_JPRB, 0.26583E+00_JPRB, 0.25762E+00_JPRB, 0.25018E+00_JPRB, 0.24481E+00_JPRB ,&
&  0.24008E+00_JPRB, 0.23762E+00_JPRB, 0.23507E+00_JPRB, 0.23431E+00_JPRB, 0.23428E+00_JPRB, 0.23499E+00_JPRB, 0.23361E+00_JPRB ,&
&  0.23196E+00_JPRB, 0.23299E+00_JPRB, 0.23365E+00_JPRB, 0.23812E+00_JPRB, 0.24465E+00_JPRB, 0.25207E+00_JPRB, 0.25928E+00_JPRB ,&
&  0.26850E+00_JPRB, 0.27249E+00_JPRB, 0.27559E+00_JPRB, 0.27634E+00_JPRB, 0.27495E+00_JPRB, 0.27425E+00_JPRB, 0.27211E+00_JPRB ,&
&  0.27158E+00_JPRB, 0.27104E+00_JPRB, 0.27221E+00_JPRB, 0.27322E+00_JPRB, 0.27186E+00_JPRB, 0.26925E+00_JPRB, 0.26352E+00_JPRB ,&
&  0.25526E+00_JPRB, 0.24853E+00_JPRB, 0.24234E+00_JPRB, 0.23754E+00_JPRB, 0.23107E+00_JPRB, 0.22535E+00_JPRB, 0.21969E+00_JPRB ,&
&  0.21473E+00_JPRB, 0.21097E+00_JPRB, 0.20656E+00_JPRB, 0.20323E+00_JPRB, 0.20167E+00_JPRB, 0.19984E+00_JPRB, 0.19906E+00_JPRB ,&
&  0.19901E+00_JPRB, 0.19938E+00_JPRB, 0.20138E+00_JPRB, 0.20305E+00_JPRB, 0.20567E+00_JPRB, 0.20870E+00_JPRB, 0.21059E+00_JPRB ,&
&  0.21381E+00_JPRB, 0.21658E+00_JPRB, 0.21808E+00_JPRB, 0.21914E+00_JPRB, 0.21817E+00_JPRB, 0.21539E+00_JPRB, 0.21251E+00_JPRB ,&
&  0.20885E+00_JPRB, 0.20625E+00_JPRB, 0.20459E+00_JPRB, 0.20282E+00_JPRB, 0.20195E+00_JPRB, 0.20121E+00_JPRB, 0.20043E+00_JPRB ,&
&  0.19985E+00_JPRB, 0.19962E+00_JPRB, 0.19938E+00_JPRB, 0.20038E+00_JPRB, 0.20149E+00_JPRB, 0.20347E+00_JPRB, 0.20587E+00_JPRB ,&
&  0.20814E+00_JPRB, 0.21159E+00_JPRB, 0.21537E+00_JPRB, 0.22050E+00_JPRB, 0.22959E+00_JPRB, 0.23729E+00_JPRB, 0.25390E+00_JPRB ,&
&  0.26700E+00_JPRB, 0.29421E+00_JPRB, 0.30521E+00_JPRB, 0.29286E+00_JPRB, 0.29130E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.28090E+00_JPRB, 0.28296E+00_JPRB, 0.28404E+00_JPRB, 0.28604E+00_JPRB, 0.28815E+00_JPRB, 0.28890E+00_JPRB, 0.28947E+00_JPRB ,&
&  0.28692E+00_JPRB, 0.28040E+00_JPRB, 0.27555E+00_JPRB, 0.26910E+00_JPRB, 0.26273E+00_JPRB, 0.25613E+00_JPRB, 0.25010E+00_JPRB ,&
&  0.24391E+00_JPRB, 0.23833E+00_JPRB, 0.23272E+00_JPRB, 0.22800E+00_JPRB, 0.22471E+00_JPRB, 0.22199E+00_JPRB, 0.21904E+00_JPRB ,&
&  0.21649E+00_JPRB, 0.21747E+00_JPRB, 0.21995E+00_JPRB, 0.22480E+00_JPRB, 0.23245E+00_JPRB, 0.23962E+00_JPRB, 0.24742E+00_JPRB ,&
&  0.26123E+00_JPRB, 0.26507E+00_JPRB, 0.26495E+00_JPRB, 0.26687E+00_JPRB, 0.26624E+00_JPRB, 0.26675E+00_JPRB, 0.25671E+00_JPRB ,&
&  0.25862E+00_JPRB, 0.26067E+00_JPRB, 0.26806E+00_JPRB, 0.27126E+00_JPRB, 0.26622E+00_JPRB, 0.26175E+00_JPRB, 0.25378E+00_JPRB ,&
&  0.24709E+00_JPRB, 0.24184E+00_JPRB, 0.23836E+00_JPRB, 0.23623E+00_JPRB, 0.23183E+00_JPRB, 0.22856E+00_JPRB, 0.22311E+00_JPRB ,&
&  0.21854E+00_JPRB, 0.21544E+00_JPRB, 0.21159E+00_JPRB, 0.20917E+00_JPRB, 0.20721E+00_JPRB, 0.20535E+00_JPRB, 0.20393E+00_JPRB ,&
&  0.20316E+00_JPRB, 0.20253E+00_JPRB, 0.20318E+00_JPRB, 0.20395E+00_JPRB, 0.20541E+00_JPRB, 0.20659E+00_JPRB, 0.20687E+00_JPRB ,&
&  0.20715E+00_JPRB, 0.20756E+00_JPRB, 0.20766E+00_JPRB, 0.20820E+00_JPRB, 0.20767E+00_JPRB, 0.20680E+00_JPRB, 0.20574E+00_JPRB ,&
&  0.20462E+00_JPRB, 0.20422E+00_JPRB, 0.20350E+00_JPRB, 0.20305E+00_JPRB, 0.20149E+00_JPRB, 0.20043E+00_JPRB, 0.19987E+00_JPRB ,&
&  0.19936E+00_JPRB, 0.19981E+00_JPRB, 0.19999E+00_JPRB, 0.20085E+00_JPRB, 0.20217E+00_JPRB, 0.20340E+00_JPRB, 0.20550E+00_JPRB ,&
&  0.20813E+00_JPRB, 0.21117E+00_JPRB, 0.21624E+00_JPRB, 0.22167E+00_JPRB, 0.22764E+00_JPRB, 0.23535E+00_JPRB, 0.24270E+00_JPRB ,&
&  0.25292E+00_JPRB, 0.26864E+00_JPRB, 0.27086E+00_JPRB, 0.27388E+00_JPRB, 0.27826E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.26678E+00_JPRB, 0.27011E+00_JPRB, 0.27234E+00_JPRB, 0.27382E+00_JPRB, 0.27525E+00_JPRB, 0.27456E+00_JPRB, 0.27304E+00_JPRB ,&
&  0.26891E+00_JPRB, 0.26747E+00_JPRB, 0.26473E+00_JPRB, 0.26183E+00_JPRB, 0.25844E+00_JPRB, 0.25469E+00_JPRB, 0.25067E+00_JPRB ,&
&  0.24619E+00_JPRB, 0.24135E+00_JPRB, 0.23554E+00_JPRB, 0.22925E+00_JPRB, 0.22350E+00_JPRB, 0.21813E+00_JPRB, 0.21491E+00_JPRB ,&
&  0.21341E+00_JPRB, 0.21414E+00_JPRB, 0.21693E+00_JPRB, 0.22042E+00_JPRB, 0.22466E+00_JPRB, 0.22947E+00_JPRB, 0.23545E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24677E+00_JPRB, 0.25052E+00_JPRB, 0.25214E+00_JPRB, 0.25074E+00_JPRB, 0.25062E+00_JPRB, 0.24963E+00_JPRB ,&
&  0.24831E+00_JPRB, 0.24617E+00_JPRB, 0.24521E+00_JPRB, 0.24514E+00_JPRB, 0.24414E+00_JPRB, 0.24210E+00_JPRB, 0.23935E+00_JPRB ,&
&  0.23684E+00_JPRB, 0.23467E+00_JPRB, 0.23205E+00_JPRB, 0.22980E+00_JPRB, 0.22811E+00_JPRB, 0.22604E+00_JPRB, 0.22332E+00_JPRB ,&
&  0.22087E+00_JPRB, 0.21841E+00_JPRB, 0.21551E+00_JPRB, 0.21303E+00_JPRB, 0.21057E+00_JPRB, 0.20879E+00_JPRB, 0.20719E+00_JPRB ,&
&  0.20649E+00_JPRB, 0.20576E+00_JPRB, 0.20554E+00_JPRB, 0.20543E+00_JPRB, 0.20584E+00_JPRB, 0.20559E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20472E+00_JPRB, 0.20459E+00_JPRB, 0.20408E+00_JPRB, 0.20390E+00_JPRB, 0.20371E+00_JPRB, 0.20308E+00_JPRB, 0.20278E+00_JPRB ,&
&  0.20289E+00_JPRB, 0.20283E+00_JPRB, 0.20234E+00_JPRB, 0.20233E+00_JPRB, 0.20203E+00_JPRB, 0.20158E+00_JPRB, 0.20188E+00_JPRB ,&
&  0.20174E+00_JPRB, 0.20198E+00_JPRB, 0.20235E+00_JPRB, 0.20291E+00_JPRB, 0.20405E+00_JPRB, 0.20547E+00_JPRB, 0.20736E+00_JPRB ,&
&  0.21021E+00_JPRB, 0.21279E+00_JPRB, 0.21540E+00_JPRB, 0.21845E+00_JPRB, 0.22407E+00_JPRB, 0.22704E+00_JPRB, 0.23221E+00_JPRB ,&
&  0.23713E+00_JPRB, 0.23667E+00_JPRB, 0.25951E+00_JPRB, 0.26059E+00_JPRB, 0.26256E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.22357E+00_JPRB, 0.22865E+00_JPRB, 0.23248E+00_JPRB, 0.23607E+00_JPRB, 0.23434E+00_JPRB, 0.25405E+00_JPRB, 0.25209E+00_JPRB ,&
&  0.25192E+00_JPRB, 0.24535E+00_JPRB, 0.24635E+00_JPRB, 0.24501E+00_JPRB, 0.24262E+00_JPRB, 0.23957E+00_JPRB, 0.23634E+00_JPRB ,&
&  0.23300E+00_JPRB, 0.22906E+00_JPRB, 0.22543E+00_JPRB, 0.22194E+00_JPRB, 0.21907E+00_JPRB, 0.21656E+00_JPRB, 0.21518E+00_JPRB ,&
&  0.21520E+00_JPRB, 0.21667E+00_JPRB, 0.21919E+00_JPRB, 0.22063E+00_JPRB, 0.22217E+00_JPRB, 0.22378E+00_JPRB, 0.22549E+00_JPRB ,&
&  0.22695E+00_JPRB, 0.22886E+00_JPRB, 0.22988E+00_JPRB, 0.22954E+00_JPRB, 0.22843E+00_JPRB, 0.22790E+00_JPRB, 0.22768E+00_JPRB ,&
&  0.22806E+00_JPRB, 0.22820E+00_JPRB, 0.22820E+00_JPRB, 0.22835E+00_JPRB, 0.22849E+00_JPRB, 0.22791E+00_JPRB, 0.22669E+00_JPRB ,&
&  0.22566E+00_JPRB, 0.22482E+00_JPRB, 0.22324E+00_JPRB, 0.22173E+00_JPRB, 0.22031E+00_JPRB, 0.21883E+00_JPRB, 0.21731E+00_JPRB ,&
&  0.21620E+00_JPRB, 0.21508E+00_JPRB, 0.21370E+00_JPRB, 0.21209E+00_JPRB, 0.21057E+00_JPRB, 0.20923E+00_JPRB, 0.20816E+00_JPRB ,&
&  0.20791E+00_JPRB, 0.20733E+00_JPRB, 0.20683E+00_JPRB, 0.20634E+00_JPRB, 0.20620E+00_JPRB, 0.20578E+00_JPRB, 0.20549E+00_JPRB ,&
&  0.20528E+00_JPRB, 0.20500E+00_JPRB, 0.20470E+00_JPRB, 0.20441E+00_JPRB, 0.20421E+00_JPRB, 0.20393E+00_JPRB, 0.20366E+00_JPRB ,&
&  0.20350E+00_JPRB, 0.20337E+00_JPRB, 0.20321E+00_JPRB, 0.20287E+00_JPRB, 0.20251E+00_JPRB, 0.20208E+00_JPRB, 0.20209E+00_JPRB ,&
&  0.20225E+00_JPRB, 0.20254E+00_JPRB, 0.20305E+00_JPRB, 0.20328E+00_JPRB, 0.20413E+00_JPRB, 0.20495E+00_JPRB, 0.20591E+00_JPRB ,&
&  0.20694E+00_JPRB, 0.20848E+00_JPRB, 0.21017E+00_JPRB, 0.21199E+00_JPRB, 0.21482E+00_JPRB, 0.21586E+00_JPRB, 0.21673E+00_JPRB ,&
&  0.21778E+00_JPRB, 0.22117E+00_JPRB, 0.22224E+00_JPRB, 0.22225E+00_JPRB, 0.22307E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.21147E+00_JPRB, 0.21175E+00_JPRB, 0.21182E+00_JPRB, 0.21177E+00_JPRB, 0.21017E+00_JPRB, 0.20957E+00_JPRB, 0.20900E+00_JPRB ,&
&  0.20859E+00_JPRB, 0.20837E+00_JPRB, 0.20877E+00_JPRB, 0.20928E+00_JPRB, 0.20980E+00_JPRB, 0.21032E+00_JPRB, 0.21074E+00_JPRB ,&
&  0.21092E+00_JPRB, 0.21113E+00_JPRB, 0.21012E+00_JPRB, 0.20965E+00_JPRB, 0.20952E+00_JPRB, 0.20960E+00_JPRB, 0.20961E+00_JPRB ,&
&  0.20973E+00_JPRB, 0.20983E+00_JPRB, 0.20983E+00_JPRB, 0.21015E+00_JPRB, 0.21072E+00_JPRB, 0.21119E+00_JPRB, 0.21169E+00_JPRB ,&
&  0.21216E+00_JPRB, 0.21284E+00_JPRB, 0.21346E+00_JPRB, 0.21404E+00_JPRB, 0.21476E+00_JPRB, 0.21545E+00_JPRB, 0.21582E+00_JPRB ,&
&  0.21609E+00_JPRB, 0.21612E+00_JPRB, 0.21621E+00_JPRB, 0.21628E+00_JPRB, 0.21646E+00_JPRB, 0.21670E+00_JPRB, 0.21672E+00_JPRB ,&
&  0.21662E+00_JPRB, 0.21644E+00_JPRB, 0.21618E+00_JPRB, 0.21564E+00_JPRB, 0.21507E+00_JPRB, 0.21453E+00_JPRB, 0.21400E+00_JPRB ,&
&  0.21353E+00_JPRB, 0.21301E+00_JPRB, 0.21244E+00_JPRB, 0.21183E+00_JPRB, 0.21119E+00_JPRB, 0.21057E+00_JPRB, 0.20994E+00_JPRB ,&
&  0.20931E+00_JPRB, 0.20890E+00_JPRB, 0.20844E+00_JPRB, 0.20805E+00_JPRB, 0.20775E+00_JPRB, 0.20745E+00_JPRB, 0.20716E+00_JPRB ,&
&  0.20687E+00_JPRB, 0.20651E+00_JPRB, 0.20619E+00_JPRB, 0.20583E+00_JPRB, 0.20552E+00_JPRB, 0.20516E+00_JPRB, 0.20488E+00_JPRB ,&
&  0.20461E+00_JPRB, 0.20437E+00_JPRB, 0.20417E+00_JPRB, 0.20409E+00_JPRB, 0.20404E+00_JPRB, 0.20408E+00_JPRB, 0.20413E+00_JPRB ,&
&  0.20420E+00_JPRB, 0.20426E+00_JPRB, 0.20430E+00_JPRB, 0.20420E+00_JPRB, 0.20440E+00_JPRB, 0.20462E+00_JPRB, 0.20493E+00_JPRB ,&
&  0.20522E+00_JPRB, 0.20568E+00_JPRB, 0.20614E+00_JPRB, 0.20666E+00_JPRB, 0.20744E+00_JPRB, 0.20772E+00_JPRB, 0.20797E+00_JPRB ,&
&  0.20823E+00_JPRB, 0.20878E+00_JPRB, 0.20925E+00_JPRB, 0.20951E+00_JPRB, 0.21008E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB, 0.20765E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB, 0.49204E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.47512E+00_JPRB, 0.47249E+00_JPRB, 0.47033E+00_JPRB, 0.46807E+00_JPRB, 0.46568E+00_JPRB, 0.46277E+00_JPRB, 0.45968E+00_JPRB ,&
&  0.45684E+00_JPRB, 0.45432E+00_JPRB, 0.45207E+00_JPRB, 0.45000E+00_JPRB, 0.44834E+00_JPRB, 0.44698E+00_JPRB, 0.44626E+00_JPRB ,&
&  0.44577E+00_JPRB, 0.44523E+00_JPRB, 0.44490E+00_JPRB, 0.44472E+00_JPRB, 0.44437E+00_JPRB, 0.44388E+00_JPRB, 0.44326E+00_JPRB ,&
&  0.44292E+00_JPRB, 0.44267E+00_JPRB, 0.44249E+00_JPRB, 0.44269E+00_JPRB, 0.44311E+00_JPRB, 0.44351E+00_JPRB, 0.44390E+00_JPRB ,&
&  0.44469E+00_JPRB, 0.44565E+00_JPRB, 0.44698E+00_JPRB, 0.44862E+00_JPRB, 0.45197E+00_JPRB, 0.45554E+00_JPRB, 0.45915E+00_JPRB ,&
&  0.46250E+00_JPRB, 0.46582E+00_JPRB, 0.46834E+00_JPRB, 0.47031E+00_JPRB, 0.47206E+00_JPRB, 0.47433E+00_JPRB, 0.47691E+00_JPRB ,&
&  0.47969E+00_JPRB, 0.48288E+00_JPRB, 0.48688E+00_JPRB, 0.49077E+00_JPRB, 0.49436E+00_JPRB, 0.49809E+00_JPRB, 0.50228E+00_JPRB ,&
&  0.50640E+00_JPRB, 0.51064E+00_JPRB, 0.51498E+00_JPRB, 0.51956E+00_JPRB, 0.52394E+00_JPRB, 0.52929E+00_JPRB, 0.53576E+00_JPRB ,&
&  0.54354E+00_JPRB, 0.54947E+00_JPRB, 0.55653E+00_JPRB, 0.56423E+00_JPRB, 0.57645E+00_JPRB, 0.58267E+00_JPRB, 0.58866E+00_JPRB ,&
&  0.59381E+00_JPRB, 0.60229E+00_JPRB, 0.60759E+00_JPRB, 0.61262E+00_JPRB, 0.61715E+00_JPRB, 0.62121E+00_JPRB, 0.62381E+00_JPRB ,&
&  0.62601E+00_JPRB, 0.62789E+00_JPRB, 0.62825E+00_JPRB, 0.62813E+00_JPRB, 0.62716E+00_JPRB, 0.62582E+00_JPRB, 0.62174E+00_JPRB ,&
&  0.61812E+00_JPRB, 0.61458E+00_JPRB, 0.61126E+00_JPRB, 0.60623E+00_JPRB, 0.60104E+00_JPRB, 0.59562E+00_JPRB, 0.59007E+00_JPRB ,&
&  0.58275E+00_JPRB, 0.57520E+00_JPRB, 0.56696E+00_JPRB, 0.55818E+00_JPRB, 0.54978E+00_JPRB, 0.54099E+00_JPRB, 0.53134E+00_JPRB ,&
&  0.52042E+00_JPRB, 0.51023E+00_JPRB, 0.49861E+00_JPRB, 0.48709E+00_JPRB, 0.47866E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.46343E+00_JPRB, 0.45154E+00_JPRB, 0.44659E+00_JPRB, 0.44436E+00_JPRB, 0.44341E+00_JPRB, 0.44199E+00_JPRB, 0.44017E+00_JPRB ,&
&  0.43882E+00_JPRB, 0.43730E+00_JPRB, 0.43583E+00_JPRB, 0.43542E+00_JPRB, 0.43427E+00_JPRB, 0.43358E+00_JPRB, 0.43273E+00_JPRB ,&
&  0.43195E+00_JPRB, 0.43089E+00_JPRB, 0.43018E+00_JPRB, 0.42895E+00_JPRB, 0.42730E+00_JPRB, 0.42503E+00_JPRB, 0.42334E+00_JPRB ,&
&  0.42194E+00_JPRB, 0.42102E+00_JPRB, 0.41976E+00_JPRB, 0.41896E+00_JPRB, 0.41920E+00_JPRB, 0.41893E+00_JPRB, 0.41813E+00_JPRB ,&
&  0.41964E+00_JPRB, 0.42038E+00_JPRB, 0.41986E+00_JPRB, 0.41834E+00_JPRB, 0.41679E+00_JPRB, 0.41687E+00_JPRB, 0.41917E+00_JPRB ,&
&  0.42261E+00_JPRB, 0.43144E+00_JPRB, 0.43789E+00_JPRB, 0.44399E+00_JPRB, 0.45029E+00_JPRB, 0.45813E+00_JPRB, 0.46737E+00_JPRB ,&
&  0.47645E+00_JPRB, 0.48215E+00_JPRB, 0.48825E+00_JPRB, 0.49641E+00_JPRB, 0.50641E+00_JPRB, 0.51736E+00_JPRB, 0.53019E+00_JPRB ,&
&  0.54021E+00_JPRB, 0.55083E+00_JPRB, 0.56349E+00_JPRB, 0.57837E+00_JPRB, 0.59053E+00_JPRB, 0.60836E+00_JPRB, 0.63094E+00_JPRB ,&
&  0.66110E+00_JPRB, 0.67505E+00_JPRB, 0.69044E+00_JPRB, 0.70525E+00_JPRB, 0.72504E+00_JPRB, 0.73535E+00_JPRB, 0.74492E+00_JPRB ,&
&  0.75154E+00_JPRB, 0.75868E+00_JPRB, 0.75782E+00_JPRB, 0.75529E+00_JPRB, 0.75268E+00_JPRB, 0.74830E+00_JPRB, 0.74403E+00_JPRB ,&
&  0.73984E+00_JPRB, 0.73635E+00_JPRB, 0.72862E+00_JPRB, 0.72244E+00_JPRB, 0.71647E+00_JPRB, 0.71150E+00_JPRB, 0.70013E+00_JPRB ,&
&  0.69012E+00_JPRB, 0.68286E+00_JPRB, 0.67847E+00_JPRB, 0.67072E+00_JPRB, 0.66283E+00_JPRB, 0.65560E+00_JPRB, 0.64882E+00_JPRB ,&
&  0.64176E+00_JPRB, 0.63322E+00_JPRB, 0.62408E+00_JPRB, 0.61416E+00_JPRB, 0.60552E+00_JPRB, 0.59295E+00_JPRB, 0.57829E+00_JPRB ,&
&  0.56052E+00_JPRB, 0.54454E+00_JPRB, 0.52499E+00_JPRB, 0.50167E+00_JPRB, 0.47817E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.48349E+00_JPRB, 0.45502E+00_JPRB, 0.44045E+00_JPRB, 0.43760E+00_JPRB, 0.43705E+00_JPRB, 0.43666E+00_JPRB, 0.43610E+00_JPRB ,&
&  0.43511E+00_JPRB, 0.43517E+00_JPRB, 0.43672E+00_JPRB, 0.43785E+00_JPRB, 0.43789E+00_JPRB, 0.43795E+00_JPRB, 0.43711E+00_JPRB ,&
&  0.43572E+00_JPRB, 0.43279E+00_JPRB, 0.43008E+00_JPRB, 0.42586E+00_JPRB, 0.42146E+00_JPRB, 0.41592E+00_JPRB, 0.40895E+00_JPRB ,&
&  0.40223E+00_JPRB, 0.39605E+00_JPRB, 0.38772E+00_JPRB, 0.38497E+00_JPRB, 0.38314E+00_JPRB, 0.36540E+00_JPRB, 0.36096E+00_JPRB ,&
&  0.36497E+00_JPRB, 0.36610E+00_JPRB, 0.36743E+00_JPRB, 0.36890E+00_JPRB, 0.37047E+00_JPRB, 0.36506E+00_JPRB, 0.36440E+00_JPRB ,&
&  0.35167E+00_JPRB, 0.36067E+00_JPRB, 0.36790E+00_JPRB, 0.37519E+00_JPRB, 0.38335E+00_JPRB, 0.39578E+00_JPRB, 0.41647E+00_JPRB ,&
&  0.45442E+00_JPRB, 0.47222E+00_JPRB, 0.49191E+00_JPRB, 0.50946E+00_JPRB, 0.53413E+00_JPRB, 0.56149E+00_JPRB, 0.58704E+00_JPRB ,&
&  0.60738E+00_JPRB, 0.62937E+00_JPRB, 0.65476E+00_JPRB, 0.69053E+00_JPRB, 0.71711E+00_JPRB, 0.74199E+00_JPRB, 0.75914E+00_JPRB ,&
&  0.78226E+00_JPRB, 0.79963E+00_JPRB, 0.81175E+00_JPRB, 0.82007E+00_JPRB, 0.82821E+00_JPRB, 0.83016E+00_JPRB, 0.83236E+00_JPRB ,&
&  0.83115E+00_JPRB, 0.82957E+00_JPRB, 0.82110E+00_JPRB, 0.80811E+00_JPRB, 0.79639E+00_JPRB, 0.78801E+00_JPRB, 0.77728E+00_JPRB ,&
&  0.76683E+00_JPRB, 0.75606E+00_JPRB, 0.74454E+00_JPRB, 0.73174E+00_JPRB, 0.71929E+00_JPRB, 0.70901E+00_JPRB, 0.69704E+00_JPRB ,&
&  0.68721E+00_JPRB, 0.67967E+00_JPRB, 0.67380E+00_JPRB, 0.66535E+00_JPRB, 0.65668E+00_JPRB, 0.64819E+00_JPRB, 0.63861E+00_JPRB ,&
&  0.62916E+00_JPRB, 0.62118E+00_JPRB, 0.61466E+00_JPRB, 0.60707E+00_JPRB, 0.60207E+00_JPRB, 0.59424E+00_JPRB, 0.58689E+00_JPRB ,&
&  0.57792E+00_JPRB, 0.56987E+00_JPRB, 0.55620E+00_JPRB, 0.53595E+00_JPRB, 0.50818E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.52002E+00_JPRB, 0.48654E+00_JPRB, 0.45374E+00_JPRB, 0.43620E+00_JPRB, 0.43378E+00_JPRB, 0.43306E+00_JPRB, 0.43323E+00_JPRB ,&
&  0.43395E+00_JPRB, 0.43705E+00_JPRB, 0.44013E+00_JPRB, 0.44232E+00_JPRB, 0.44296E+00_JPRB, 0.44195E+00_JPRB, 0.43983E+00_JPRB ,&
&  0.43575E+00_JPRB, 0.43092E+00_JPRB, 0.42595E+00_JPRB, 0.42166E+00_JPRB, 0.41683E+00_JPRB, 0.41162E+00_JPRB, 0.40461E+00_JPRB ,&
&  0.39630E+00_JPRB, 0.38827E+00_JPRB, 0.36016E+00_JPRB, 0.35735E+00_JPRB, 0.35138E+00_JPRB, 0.35017E+00_JPRB, 0.35337E+00_JPRB ,&
&  0.35605E+00_JPRB, 0.35111E+00_JPRB, 0.34479E+00_JPRB, 0.33287E+00_JPRB, 0.32206E+00_JPRB, 0.31066E+00_JPRB, 0.30361E+00_JPRB ,&
&  0.30080E+00_JPRB, 0.30211E+00_JPRB, 0.30745E+00_JPRB, 0.31330E+00_JPRB, 0.32161E+00_JPRB, 0.33082E+00_JPRB, 0.34513E+00_JPRB ,&
&  0.38477E+00_JPRB, 0.44199E+00_JPRB, 0.50834E+00_JPRB, 0.54091E+00_JPRB, 0.57839E+00_JPRB, 0.61991E+00_JPRB, 0.65023E+00_JPRB ,&
&  0.67537E+00_JPRB, 0.70172E+00_JPRB, 0.72726E+00_JPRB, 0.75769E+00_JPRB, 0.78279E+00_JPRB, 0.79954E+00_JPRB, 0.82535E+00_JPRB ,&
&  0.84623E+00_JPRB, 0.85948E+00_JPRB, 0.86046E+00_JPRB, 0.86014E+00_JPRB, 0.85798E+00_JPRB, 0.84747E+00_JPRB, 0.83641E+00_JPRB ,&
&  0.82469E+00_JPRB, 0.81266E+00_JPRB, 0.79609E+00_JPRB, 0.76976E+00_JPRB, 0.74903E+00_JPRB, 0.73606E+00_JPRB, 0.71691E+00_JPRB ,&
&  0.69972E+00_JPRB, 0.68082E+00_JPRB, 0.66537E+00_JPRB, 0.64494E+00_JPRB, 0.63408E+00_JPRB, 0.63223E+00_JPRB, 0.63366E+00_JPRB ,&
&  0.63458E+00_JPRB, 0.63489E+00_JPRB, 0.63392E+00_JPRB, 0.62922E+00_JPRB, 0.61940E+00_JPRB, 0.60820E+00_JPRB, 0.59814E+00_JPRB ,&
&  0.58727E+00_JPRB, 0.57610E+00_JPRB, 0.56535E+00_JPRB, 0.55427E+00_JPRB, 0.54883E+00_JPRB, 0.54540E+00_JPRB, 0.54736E+00_JPRB ,&
&  0.55182E+00_JPRB, 0.55761E+00_JPRB, 0.55991E+00_JPRB, 0.55710E+00_JPRB, 0.54323E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.58935E+00_JPRB, 0.57644E+00_JPRB, 0.55103E+00_JPRB, 0.51626E+00_JPRB, 0.49078E+00_JPRB, 0.47462E+00_JPRB, 0.46724E+00_JPRB ,&
&  0.46320E+00_JPRB, 0.46115E+00_JPRB, 0.45989E+00_JPRB, 0.45759E+00_JPRB, 0.45433E+00_JPRB, 0.44898E+00_JPRB, 0.44440E+00_JPRB ,&
&  0.43554E+00_JPRB, 0.42801E+00_JPRB, 0.41917E+00_JPRB, 0.41467E+00_JPRB, 0.41102E+00_JPRB, 0.40814E+00_JPRB, 0.40615E+00_JPRB ,&
&  0.40298E+00_JPRB, 0.39888E+00_JPRB, 0.38373E+00_JPRB, 0.37716E+00_JPRB, 0.37129E+00_JPRB, 0.37264E+00_JPRB, 0.37300E+00_JPRB ,&
&  0.38957E+00_JPRB, 0.35033E+00_JPRB, 0.34098E+00_JPRB, 0.32719E+00_JPRB, 0.31632E+00_JPRB, 0.30395E+00_JPRB, 0.29318E+00_JPRB ,&
&  0.28640E+00_JPRB, 0.28111E+00_JPRB, 0.28001E+00_JPRB, 0.28596E+00_JPRB, 0.29263E+00_JPRB, 0.30443E+00_JPRB, 0.31786E+00_JPRB ,&
&  0.35086E+00_JPRB, 0.46169E+00_JPRB, 0.51268E+00_JPRB, 0.55284E+00_JPRB, 0.59158E+00_JPRB, 0.63274E+00_JPRB, 0.65684E+00_JPRB ,&
&  0.67846E+00_JPRB, 0.70832E+00_JPRB, 0.72601E+00_JPRB, 0.72661E+00_JPRB, 0.77728E+00_JPRB, 0.81413E+00_JPRB, 0.84687E+00_JPRB ,&
&  0.87496E+00_JPRB, 0.89256E+00_JPRB, 0.90485E+00_JPRB, 0.90886E+00_JPRB, 0.91718E+00_JPRB, 0.90485E+00_JPRB, 0.88558E+00_JPRB ,&
&  0.86546E+00_JPRB, 0.84978E+00_JPRB, 0.83211E+00_JPRB, 0.79493E+00_JPRB, 0.76446E+00_JPRB, 0.74464E+00_JPRB, 0.71315E+00_JPRB ,&
&  0.68316E+00_JPRB, 0.65451E+00_JPRB, 0.62783E+00_JPRB, 0.60330E+00_JPRB, 0.58536E+00_JPRB, 0.57676E+00_JPRB, 0.57986E+00_JPRB ,&
&  0.58726E+00_JPRB, 0.59386E+00_JPRB, 0.59199E+00_JPRB, 0.58561E+00_JPRB, 0.57568E+00_JPRB, 0.56953E+00_JPRB, 0.55748E+00_JPRB ,&
&  0.54255E+00_JPRB, 0.52307E+00_JPRB, 0.51067E+00_JPRB, 0.50204E+00_JPRB, 0.50031E+00_JPRB, 0.50452E+00_JPRB, 0.51304E+00_JPRB ,&
&  0.52444E+00_JPRB, 0.53960E+00_JPRB, 0.55817E+00_JPRB, 0.57517E+00_JPRB, 0.58669E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.57080E+00_JPRB, 0.58086E+00_JPRB, 0.58414E+00_JPRB, 0.57850E+00_JPRB, 0.56639E+00_JPRB, 0.54993E+00_JPRB, 0.53256E+00_JPRB ,&
&  0.51720E+00_JPRB, 0.50529E+00_JPRB, 0.49463E+00_JPRB, 0.48694E+00_JPRB, 0.47800E+00_JPRB, 0.47259E+00_JPRB, 0.46639E+00_JPRB ,&
&  0.46359E+00_JPRB, 0.46308E+00_JPRB, 0.45817E+00_JPRB, 0.45198E+00_JPRB, 0.43860E+00_JPRB, 0.42630E+00_JPRB, 0.41710E+00_JPRB ,&
&  0.40897E+00_JPRB, 0.40287E+00_JPRB, 0.38814E+00_JPRB, 0.37704E+00_JPRB, 0.37246E+00_JPRB, 0.37270E+00_JPRB, 0.38237E+00_JPRB ,&
&  0.38680E+00_JPRB, 0.39613E+00_JPRB, 0.40351E+00_JPRB, 0.39815E+00_JPRB, 0.39223E+00_JPRB, 0.30784E+00_JPRB, 0.27988E+00_JPRB ,&
&  0.27085E+00_JPRB, 0.26477E+00_JPRB, 0.26317E+00_JPRB, 0.26808E+00_JPRB, 0.27616E+00_JPRB, 0.29587E+00_JPRB, 0.31202E+00_JPRB ,&
&  0.33085E+00_JPRB, 0.44864E+00_JPRB, 0.49631E+00_JPRB, 0.54156E+00_JPRB, 0.57272E+00_JPRB, 0.60234E+00_JPRB, 0.62432E+00_JPRB ,&
&  0.64216E+00_JPRB, 0.66771E+00_JPRB, 0.68010E+00_JPRB, 0.70577E+00_JPRB, 0.75668E+00_JPRB, 0.85027E+00_JPRB, 0.92151E+00_JPRB ,&
&  0.10019E+01_JPRB, 0.10111E+01_JPRB, 0.10101E+01_JPRB, 0.97605E+00_JPRB, 0.96749E+00_JPRB, 0.95484E+00_JPRB, 0.93638E+00_JPRB ,&
&  0.92220E+00_JPRB, 0.88945E+00_JPRB, 0.87644E+00_JPRB, 0.85211E+00_JPRB, 0.83187E+00_JPRB, 0.81205E+00_JPRB, 0.76228E+00_JPRB ,&
&  0.70557E+00_JPRB, 0.66640E+00_JPRB, 0.63132E+00_JPRB, 0.59294E+00_JPRB, 0.56490E+00_JPRB, 0.54578E+00_JPRB, 0.54325E+00_JPRB ,&
&  0.54696E+00_JPRB, 0.55917E+00_JPRB, 0.55619E+00_JPRB, 0.54755E+00_JPRB, 0.54076E+00_JPRB, 0.53625E+00_JPRB, 0.52055E+00_JPRB ,&
&  0.50686E+00_JPRB, 0.49322E+00_JPRB, 0.48748E+00_JPRB, 0.48121E+00_JPRB, 0.47825E+00_JPRB, 0.47965E+00_JPRB, 0.48476E+00_JPRB ,&
&  0.49492E+00_JPRB, 0.50817E+00_JPRB, 0.52593E+00_JPRB, 0.54335E+00_JPRB, 0.56166E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.55280E+00_JPRB, 0.56971E+00_JPRB, 0.57911E+00_JPRB, 0.57776E+00_JPRB, 0.56454E+00_JPRB, 0.54638E+00_JPRB, 0.53061E+00_JPRB ,&
&  0.51633E+00_JPRB, 0.50276E+00_JPRB, 0.49054E+00_JPRB, 0.48156E+00_JPRB, 0.47399E+00_JPRB, 0.46967E+00_JPRB, 0.46689E+00_JPRB ,&
&  0.46628E+00_JPRB, 0.45793E+00_JPRB, 0.45521E+00_JPRB, 0.45656E+00_JPRB, 0.46010E+00_JPRB, 0.45947E+00_JPRB, 0.45482E+00_JPRB ,&
&  0.43778E+00_JPRB, 0.41415E+00_JPRB, 0.39212E+00_JPRB, 0.37382E+00_JPRB, 0.36095E+00_JPRB, 0.35866E+00_JPRB, 0.36693E+00_JPRB ,&
&  0.37800E+00_JPRB, 0.37898E+00_JPRB, 0.38237E+00_JPRB, 0.40142E+00_JPRB, 0.41705E+00_JPRB, 0.35954E+00_JPRB, 0.28109E+00_JPRB ,&
&  0.26763E+00_JPRB, 0.25532E+00_JPRB, 0.25422E+00_JPRB, 0.25943E+00_JPRB, 0.26916E+00_JPRB, 0.28406E+00_JPRB, 0.31064E+00_JPRB ,&
&  0.35158E+00_JPRB, 0.43790E+00_JPRB, 0.47675E+00_JPRB, 0.51218E+00_JPRB, 0.54825E+00_JPRB, 0.58152E+00_JPRB, 0.60076E+00_JPRB ,&
&  0.61103E+00_JPRB, 0.62802E+00_JPRB, 0.66440E+00_JPRB, 0.70601E+00_JPRB, 0.77550E+00_JPRB, 0.91320E+00_JPRB, 0.10174E+01_JPRB ,&
&  0.11613E+01_JPRB, 0.12032E+01_JPRB, 0.11411E+01_JPRB, 0.10172E+01_JPRB, 0.98698E+00_JPRB, 0.99219E+00_JPRB, 0.96008E+00_JPRB ,&
&  0.96436E+00_JPRB, 0.98111E+00_JPRB, 0.95902E+00_JPRB, 0.91373E+00_JPRB, 0.88355E+00_JPRB, 0.86362E+00_JPRB, 0.79778E+00_JPRB ,&
&  0.72569E+00_JPRB, 0.69428E+00_JPRB, 0.69213E+00_JPRB, 0.63114E+00_JPRB, 0.58933E+00_JPRB, 0.55131E+00_JPRB, 0.53324E+00_JPRB ,&
&  0.52404E+00_JPRB, 0.51561E+00_JPRB, 0.50288E+00_JPRB, 0.49217E+00_JPRB, 0.48853E+00_JPRB, 0.48651E+00_JPRB, 0.48153E+00_JPRB ,&
&  0.47668E+00_JPRB, 0.47199E+00_JPRB, 0.46747E+00_JPRB, 0.46466E+00_JPRB, 0.46269E+00_JPRB, 0.46335E+00_JPRB, 0.46744E+00_JPRB ,&
&  0.47616E+00_JPRB, 0.48925E+00_JPRB, 0.50554E+00_JPRB, 0.51952E+00_JPRB, 0.53677E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.55291E+00_JPRB, 0.56621E+00_JPRB, 0.56057E+00_JPRB, 0.53732E+00_JPRB, 0.50458E+00_JPRB, 0.46526E+00_JPRB, 0.44083E+00_JPRB ,&
&  0.42853E+00_JPRB, 0.42379E+00_JPRB, 0.42103E+00_JPRB, 0.41587E+00_JPRB, 0.41459E+00_JPRB, 0.41482E+00_JPRB, 0.41654E+00_JPRB ,&
&  0.42052E+00_JPRB, 0.42608E+00_JPRB, 0.43105E+00_JPRB, 0.43144E+00_JPRB, 0.43669E+00_JPRB, 0.44091E+00_JPRB, 0.44464E+00_JPRB ,&
&  0.44206E+00_JPRB, 0.42541E+00_JPRB, 0.40280E+00_JPRB, 0.38833E+00_JPRB, 0.37350E+00_JPRB, 0.37632E+00_JPRB, 0.36952E+00_JPRB ,&
&  0.34513E+00_JPRB, 0.34234E+00_JPRB, 0.37389E+00_JPRB, 0.40300E+00_JPRB, 0.40164E+00_JPRB, 0.42611E+00_JPRB, 0.31345E+00_JPRB ,&
&  0.27678E+00_JPRB, 0.25941E+00_JPRB, 0.25611E+00_JPRB, 0.26340E+00_JPRB, 0.28266E+00_JPRB, 0.30910E+00_JPRB, 0.33886E+00_JPRB ,&
&  0.39577E+00_JPRB, 0.43655E+00_JPRB, 0.45971E+00_JPRB, 0.49021E+00_JPRB, 0.52656E+00_JPRB, 0.56384E+00_JPRB, 0.59280E+00_JPRB ,&
&  0.61167E+00_JPRB, 0.64232E+00_JPRB, 0.69634E+00_JPRB, 0.78170E+00_JPRB, 0.97457E+00_JPRB, 0.94077E+00_JPRB, 0.96116E+00_JPRB ,&
&  0.12111E+01_JPRB, 0.13661E+01_JPRB, 0.12232E+01_JPRB, 0.10741E+01_JPRB, 0.99946E+00_JPRB, 0.98377E+00_JPRB, 0.97027E+00_JPRB ,&
&  0.10159E+01_JPRB, 0.10607E+01_JPRB, 0.10505E+01_JPRB, 0.95948E+00_JPRB, 0.91249E+00_JPRB, 0.88401E+00_JPRB, 0.80330E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.72335E+00_JPRB, 0.76164E+00_JPRB, 0.64900E+00_JPRB, 0.57490E+00_JPRB, 0.52972E+00_JPRB, 0.51457E+00_JPRB ,&
&  0.50750E+00_JPRB, 0.50002E+00_JPRB, 0.48969E+00_JPRB, 0.47824E+00_JPRB, 0.46281E+00_JPRB, 0.45036E+00_JPRB, 0.43823E+00_JPRB ,&
&  0.43160E+00_JPRB, 0.42713E+00_JPRB, 0.42820E+00_JPRB, 0.43191E+00_JPRB, 0.43497E+00_JPRB, 0.43829E+00_JPRB, 0.44821E+00_JPRB ,&
&  0.46361E+00_JPRB, 0.47357E+00_JPRB, 0.48900E+00_JPRB, 0.50608E+00_JPRB, 0.52868E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.49127E+00_JPRB, 0.49556E+00_JPRB, 0.48480E+00_JPRB, 0.45738E+00_JPRB, 0.43155E+00_JPRB, 0.39625E+00_JPRB, 0.37145E+00_JPRB ,&
&  0.35814E+00_JPRB, 0.35289E+00_JPRB, 0.35325E+00_JPRB, 0.35537E+00_JPRB, 0.35950E+00_JPRB, 0.36571E+00_JPRB, 0.36693E+00_JPRB ,&
&  0.36862E+00_JPRB, 0.36813E+00_JPRB, 0.37066E+00_JPRB, 0.37686E+00_JPRB, 0.38475E+00_JPRB, 0.39402E+00_JPRB, 0.40385E+00_JPRB ,&
&  0.41361E+00_JPRB, 0.41329E+00_JPRB, 0.39849E+00_JPRB, 0.38722E+00_JPRB, 0.39355E+00_JPRB, 0.41510E+00_JPRB, 0.40587E+00_JPRB ,&
&  0.36839E+00_JPRB, 0.34335E+00_JPRB, 0.34570E+00_JPRB, 0.40706E+00_JPRB, 0.44325E+00_JPRB, 0.46379E+00_JPRB, 0.35262E+00_JPRB ,&
&  0.29480E+00_JPRB, 0.27240E+00_JPRB, 0.27227E+00_JPRB, 0.29373E+00_JPRB, 0.33696E+00_JPRB, 0.40464E+00_JPRB, 0.45554E+00_JPRB ,&
&  0.49981E+00_JPRB, 0.53765E+00_JPRB, 0.54919E+00_JPRB, 0.57228E+00_JPRB, 0.60941E+00_JPRB, 0.64323E+00_JPRB, 0.68001E+00_JPRB ,&
&  0.70231E+00_JPRB, 0.74033E+00_JPRB, 0.86077E+00_JPRB, 0.73797E+00_JPRB, 0.78579E+00_JPRB, 0.84300E+00_JPRB, 0.94519E+00_JPRB ,&
&  0.10728E+01_JPRB, 0.11065E+01_JPRB, 0.10977E+01_JPRB, 0.11481E+01_JPRB, 0.10352E+01_JPRB, 0.93652E+00_JPRB, 0.89991E+00_JPRB ,&
&  0.10014E+01_JPRB, 0.10951E+01_JPRB, 0.10693E+01_JPRB, 0.98243E+00_JPRB, 0.95191E+00_JPRB, 0.93209E+00_JPRB, 0.83755E+00_JPRB ,&
&  0.72157E+00_JPRB, 0.65295E+00_JPRB, 0.59516E+00_JPRB, 0.52470E+00_JPRB, 0.49521E+00_JPRB, 0.47857E+00_JPRB, 0.47091E+00_JPRB ,&
&  0.46813E+00_JPRB, 0.46279E+00_JPRB, 0.45903E+00_JPRB, 0.43286E+00_JPRB, 0.40360E+00_JPRB, 0.37687E+00_JPRB, 0.35894E+00_JPRB ,&
&  0.34921E+00_JPRB, 0.34869E+00_JPRB, 0.35429E+00_JPRB, 0.37221E+00_JPRB, 0.40297E+00_JPRB, 0.43153E+00_JPRB, 0.45136E+00_JPRB ,&
&  0.45424E+00_JPRB, 0.44245E+00_JPRB, 0.44123E+00_JPRB, 0.44717E+00_JPRB, 0.47395E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.48163E+00_JPRB, 0.45246E+00_JPRB, 0.43356E+00_JPRB, 0.43913E+00_JPRB, 0.40393E+00_JPRB, 0.37111E+00_JPRB, 0.35374E+00_JPRB ,&
&  0.35525E+00_JPRB, 0.35265E+00_JPRB, 0.35570E+00_JPRB, 0.33902E+00_JPRB, 0.32472E+00_JPRB, 0.31595E+00_JPRB, 0.31247E+00_JPRB ,&
&  0.32161E+00_JPRB, 0.33257E+00_JPRB, 0.34024E+00_JPRB, 0.33817E+00_JPRB, 0.33630E+00_JPRB, 0.34040E+00_JPRB, 0.35190E+00_JPRB ,&
&  0.36910E+00_JPRB, 0.38417E+00_JPRB, 0.38643E+00_JPRB, 0.38756E+00_JPRB, 0.41312E+00_JPRB, 0.44347E+00_JPRB, 0.42089E+00_JPRB ,&
&  0.39191E+00_JPRB, 0.37020E+00_JPRB, 0.36684E+00_JPRB, 0.41178E+00_JPRB, 0.47772E+00_JPRB, 0.51149E+00_JPRB, 0.39519E+00_JPRB ,&
&  0.31749E+00_JPRB, 0.30747E+00_JPRB, 0.34386E+00_JPRB, 0.46608E+00_JPRB, 0.50013E+00_JPRB, 0.54548E+00_JPRB, 0.61616E+00_JPRB ,&
&  0.56699E+00_JPRB, 0.56898E+00_JPRB, 0.59335E+00_JPRB, 0.73631E+00_JPRB, 0.77295E+00_JPRB, 0.78801E+00_JPRB, 0.79973E+00_JPRB ,&
&  0.80758E+00_JPRB, 0.84239E+00_JPRB, 0.98617E+00_JPRB, 0.81747E+00_JPRB, 0.86083E+00_JPRB, 0.10613E+01_JPRB, 0.10476E+01_JPRB ,&
&  0.10632E+01_JPRB, 0.10560E+01_JPRB, 0.11505E+01_JPRB, 0.10728E+01_JPRB, 0.94409E+00_JPRB, 0.87361E+00_JPRB, 0.90536E+00_JPRB ,&
&  0.10562E+01_JPRB, 0.11707E+01_JPRB, 0.11363E+01_JPRB, 0.10903E+01_JPRB, 0.10646E+01_JPRB, 0.10181E+01_JPRB, 0.91452E+00_JPRB ,&
&  0.76684E+00_JPRB, 0.65453E+00_JPRB, 0.56649E+00_JPRB, 0.48991E+00_JPRB, 0.45308E+00_JPRB, 0.43909E+00_JPRB, 0.42693E+00_JPRB ,&
&  0.42651E+00_JPRB, 0.42528E+00_JPRB, 0.42070E+00_JPRB, 0.38961E+00_JPRB, 0.36021E+00_JPRB, 0.34279E+00_JPRB, 0.33116E+00_JPRB ,&
&  0.32570E+00_JPRB, 0.31338E+00_JPRB, 0.30469E+00_JPRB, 0.31621E+00_JPRB, 0.35205E+00_JPRB, 0.39714E+00_JPRB, 0.41704E+00_JPRB ,&
&  0.40767E+00_JPRB, 0.40569E+00_JPRB, 0.43167E+00_JPRB, 0.47015E+00_JPRB, 0.48840E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.49746E+00_JPRB, 0.48805E+00_JPRB, 0.48332E+00_JPRB, 0.47390E+00_JPRB, 0.45963E+00_JPRB, 0.41635E+00_JPRB, 0.38433E+00_JPRB ,&
&  0.39309E+00_JPRB, 0.37028E+00_JPRB, 0.36534E+00_JPRB, 0.34147E+00_JPRB, 0.32742E+00_JPRB, 0.31063E+00_JPRB, 0.29828E+00_JPRB ,&
&  0.28988E+00_JPRB, 0.29764E+00_JPRB, 0.31630E+00_JPRB, 0.32087E+00_JPRB, 0.32040E+00_JPRB, 0.31997E+00_JPRB, 0.32873E+00_JPRB ,&
&  0.34296E+00_JPRB, 0.36406E+00_JPRB, 0.37937E+00_JPRB, 0.39300E+00_JPRB, 0.41730E+00_JPRB, 0.44865E+00_JPRB, 0.45154E+00_JPRB ,&
&  0.45041E+00_JPRB, 0.43182E+00_JPRB, 0.41187E+00_JPRB, 0.44627E+00_JPRB, 0.51195E+00_JPRB, 0.57414E+00_JPRB, 0.55606E+00_JPRB ,&
&  0.38587E+00_JPRB, 0.38798E+00_JPRB, 0.55790E+00_JPRB, 0.60454E+00_JPRB, 0.65801E+00_JPRB, 0.69353E+00_JPRB, 0.73990E+00_JPRB ,&
&  0.78179E+00_JPRB, 0.82430E+00_JPRB, 0.85519E+00_JPRB, 0.88058E+00_JPRB, 0.89913E+00_JPRB, 0.92084E+00_JPRB, 0.91595E+00_JPRB ,&
&  0.98458E+00_JPRB, 0.78684E+00_JPRB, 0.74251E+00_JPRB, 0.79504E+00_JPRB, 0.10694E+01_JPRB, 0.11156E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.10464E+01_JPRB, 0.99697E+00_JPRB, 0.10055E+01_JPRB, 0.10424E+01_JPRB, 0.10110E+01_JPRB, 0.10292E+01_JPRB, 0.10839E+01_JPRB ,&
&  0.11946E+01_JPRB, 0.13059E+01_JPRB, 0.13023E+01_JPRB, 0.12748E+01_JPRB, 0.12177E+01_JPRB, 0.11173E+01_JPRB, 0.96570E+00_JPRB ,&
&  0.78179E+00_JPRB, 0.65088E+00_JPRB, 0.54137E+00_JPRB, 0.46462E+00_JPRB, 0.42127E+00_JPRB, 0.40333E+00_JPRB, 0.39118E+00_JPRB ,&
&  0.39309E+00_JPRB, 0.39308E+00_JPRB, 0.38512E+00_JPRB, 0.35981E+00_JPRB, 0.34341E+00_JPRB, 0.34949E+00_JPRB, 0.35608E+00_JPRB ,&
&  0.35521E+00_JPRB, 0.33623E+00_JPRB, 0.31626E+00_JPRB, 0.32749E+00_JPRB, 0.36031E+00_JPRB, 0.39468E+00_JPRB, 0.41483E+00_JPRB ,&
&  0.41481E+00_JPRB, 0.42141E+00_JPRB, 0.43532E+00_JPRB, 0.46022E+00_JPRB, 0.47649E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.52471E+00_JPRB, 0.52762E+00_JPRB, 0.51446E+00_JPRB, 0.50158E+00_JPRB, 0.48556E+00_JPRB, 0.43168E+00_JPRB, 0.42148E+00_JPRB ,&
&  0.48498E+00_JPRB, 0.43121E+00_JPRB, 0.48692E+00_JPRB, 0.38494E+00_JPRB, 0.36354E+00_JPRB, 0.34607E+00_JPRB, 0.32676E+00_JPRB ,&
&  0.30571E+00_JPRB, 0.30180E+00_JPRB, 0.30901E+00_JPRB, 0.31445E+00_JPRB, 0.31527E+00_JPRB, 0.31209E+00_JPRB, 0.31468E+00_JPRB ,&
&  0.32299E+00_JPRB, 0.34452E+00_JPRB, 0.37475E+00_JPRB, 0.39172E+00_JPRB, 0.41232E+00_JPRB, 0.44611E+00_JPRB, 0.46951E+00_JPRB ,&
&  0.45414E+00_JPRB, 0.45534E+00_JPRB, 0.46684E+00_JPRB, 0.49400E+00_JPRB, 0.53867E+00_JPRB, 0.60210E+00_JPRB, 0.64274E+00_JPRB ,&
&  0.66166E+00_JPRB, 0.54131E+00_JPRB, 0.64969E+00_JPRB, 0.67201E+00_JPRB, 0.72900E+00_JPRB, 0.76098E+00_JPRB, 0.78378E+00_JPRB ,&
&  0.81658E+00_JPRB, 0.85447E+00_JPRB, 0.89735E+00_JPRB, 0.93669E+00_JPRB, 0.97699E+00_JPRB, 0.10157E+01_JPRB, 0.10174E+01_JPRB ,&
&  0.11046E+01_JPRB, 0.76721E+00_JPRB, 0.82865E+00_JPRB, 0.86369E+00_JPRB, 0.11760E+01_JPRB, 0.12995E+01_JPRB, 0.12135E+01_JPRB ,&
&  0.12697E+01_JPRB, 0.11666E+01_JPRB, 0.11909E+01_JPRB, 0.12140E+01_JPRB, 0.12590E+01_JPRB, 0.13446E+01_JPRB, 0.13926E+01_JPRB ,&
&  0.13964E+01_JPRB, 0.14305E+01_JPRB, 0.14870E+01_JPRB, 0.13912E+01_JPRB, 0.12493E+01_JPRB, 0.10780E+01_JPRB, 0.88614E+00_JPRB ,&
&  0.75352E+00_JPRB, 0.63801E+00_JPRB, 0.53919E+00_JPRB, 0.47317E+00_JPRB, 0.42199E+00_JPRB, 0.38962E+00_JPRB, 0.37258E+00_JPRB ,&
&  0.36465E+00_JPRB, 0.35351E+00_JPRB, 0.34474E+00_JPRB, 0.32955E+00_JPRB, 0.32776E+00_JPRB, 0.33611E+00_JPRB, 0.36398E+00_JPRB ,&
&  0.37547E+00_JPRB, 0.36883E+00_JPRB, 0.41234E+00_JPRB, 0.45168E+00_JPRB, 0.45627E+00_JPRB, 0.47305E+00_JPRB, 0.53528E+00_JPRB ,&
&  0.46053E+00_JPRB, 0.48644E+00_JPRB, 0.53664E+00_JPRB, 0.54027E+00_JPRB, 0.52624E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.57049E+00_JPRB, 0.57087E+00_JPRB, 0.54872E+00_JPRB, 0.52552E+00_JPRB, 0.51167E+00_JPRB, 0.48618E+00_JPRB, 0.45879E+00_JPRB ,&
&  0.50462E+00_JPRB, 0.49424E+00_JPRB, 0.49644E+00_JPRB, 0.50379E+00_JPRB, 0.50022E+00_JPRB, 0.44055E+00_JPRB, 0.38408E+00_JPRB ,&
&  0.34909E+00_JPRB, 0.33196E+00_JPRB, 0.33785E+00_JPRB, 0.33762E+00_JPRB, 0.33511E+00_JPRB, 0.32783E+00_JPRB, 0.32469E+00_JPRB ,&
&  0.32382E+00_JPRB, 0.33452E+00_JPRB, 0.36101E+00_JPRB, 0.38511E+00_JPRB, 0.39739E+00_JPRB, 0.41572E+00_JPRB, 0.44943E+00_JPRB ,&
&  0.44392E+00_JPRB, 0.45435E+00_JPRB, 0.47806E+00_JPRB, 0.45146E+00_JPRB, 0.55462E+00_JPRB, 0.59675E+00_JPRB, 0.62080E+00_JPRB ,&
&  0.66814E+00_JPRB, 0.70801E+00_JPRB, 0.72303E+00_JPRB, 0.72492E+00_JPRB, 0.74179E+00_JPRB, 0.75199E+00_JPRB, 0.77530E+00_JPRB ,&
&  0.79893E+00_JPRB, 0.83489E+00_JPRB, 0.88447E+00_JPRB, 0.94376E+00_JPRB, 0.10609E+01_JPRB, 0.97284E+00_JPRB, 0.10736E+01_JPRB ,&
&  0.11367E+01_JPRB, 0.11673E+01_JPRB, 0.12452E+01_JPRB, 0.10944E+01_JPRB, 0.13729E+01_JPRB, 0.15068E+01_JPRB, 0.13416E+01_JPRB ,&
&  0.12987E+01_JPRB, 0.12135E+01_JPRB, 0.13725E+01_JPRB, 0.14912E+01_JPRB, 0.15901E+01_JPRB, 0.16801E+01_JPRB, 0.17099E+01_JPRB ,&
&  0.16750E+01_JPRB, 0.17365E+01_JPRB, 0.16925E+01_JPRB, 0.15364E+01_JPRB, 0.13222E+01_JPRB, 0.11396E+01_JPRB, 0.93073E+00_JPRB ,&
&  0.75496E+00_JPRB, 0.61244E+00_JPRB, 0.52403E+00_JPRB, 0.48041E+00_JPRB, 0.41431E+00_JPRB, 0.39153E+00_JPRB, 0.36769E+00_JPRB ,&
&  0.34367E+00_JPRB, 0.32296E+00_JPRB, 0.31620E+00_JPRB, 0.30907E+00_JPRB, 0.31075E+00_JPRB, 0.32651E+00_JPRB, 0.34615E+00_JPRB ,&
&  0.39836E+00_JPRB, 0.48647E+00_JPRB, 0.67167E+00_JPRB, 0.63728E+00_JPRB, 0.62038E+00_JPRB, 0.60121E+00_JPRB, 0.51875E+00_JPRB ,&
&  0.48576E+00_JPRB, 0.57329E+00_JPRB, 0.56700E+00_JPRB, 0.56492E+00_JPRB, 0.55888E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.61230E+00_JPRB, 0.59993E+00_JPRB, 0.58414E+00_JPRB, 0.56201E+00_JPRB, 0.53520E+00_JPRB, 0.51445E+00_JPRB, 0.50679E+00_JPRB ,&
&  0.50797E+00_JPRB, 0.50893E+00_JPRB, 0.51168E+00_JPRB, 0.50908E+00_JPRB, 0.49939E+00_JPRB, 0.51776E+00_JPRB, 0.45479E+00_JPRB ,&
&  0.38343E+00_JPRB, 0.36449E+00_JPRB, 0.34959E+00_JPRB, 0.35588E+00_JPRB, 0.36750E+00_JPRB, 0.35998E+00_JPRB, 0.35188E+00_JPRB ,&
&  0.34523E+00_JPRB, 0.34042E+00_JPRB, 0.34866E+00_JPRB, 0.36496E+00_JPRB, 0.37823E+00_JPRB, 0.39492E+00_JPRB, 0.41942E+00_JPRB ,&
&  0.41230E+00_JPRB, 0.39708E+00_JPRB, 0.39640E+00_JPRB, 0.41138E+00_JPRB, 0.48635E+00_JPRB, 0.58008E+00_JPRB, 0.63200E+00_JPRB ,&
&  0.65240E+00_JPRB, 0.66781E+00_JPRB, 0.70262E+00_JPRB, 0.71615E+00_JPRB, 0.72339E+00_JPRB, 0.74433E+00_JPRB, 0.77079E+00_JPRB ,&
&  0.79919E+00_JPRB, 0.83751E+00_JPRB, 0.88889E+00_JPRB, 0.97284E+00_JPRB, 0.99148E+00_JPRB, 0.99028E+00_JPRB, 0.11105E+01_JPRB ,&
&  0.12043E+01_JPRB, 0.13326E+01_JPRB, 0.13754E+01_JPRB, 0.14336E+01_JPRB, 0.15651E+01_JPRB, 0.14984E+01_JPRB, 0.13985E+01_JPRB ,&
&  0.14130E+01_JPRB, 0.14027E+01_JPRB, 0.16678E+01_JPRB, 0.17537E+01_JPRB, 0.17728E+01_JPRB, 0.18552E+01_JPRB, 0.18485E+01_JPRB ,&
&  0.18573E+01_JPRB, 0.18434E+01_JPRB, 0.18033E+01_JPRB, 0.16307E+01_JPRB, 0.13775E+01_JPRB, 0.11701E+01_JPRB, 0.92134E+00_JPRB ,&
&  0.74583E+00_JPRB, 0.60194E+00_JPRB, 0.47412E+00_JPRB, 0.42164E+00_JPRB, 0.37647E+00_JPRB, 0.36408E+00_JPRB, 0.35106E+00_JPRB ,&
&  0.33008E+00_JPRB, 0.32158E+00_JPRB, 0.31091E+00_JPRB, 0.31839E+00_JPRB, 0.32714E+00_JPRB, 0.33632E+00_JPRB, 0.38126E+00_JPRB ,&
&  0.47229E+00_JPRB, 0.68965E+00_JPRB, 0.66292E+00_JPRB, 0.70150E+00_JPRB, 0.69853E+00_JPRB, 0.67084E+00_JPRB, 0.50687E+00_JPRB ,&
&  0.55435E+00_JPRB, 0.62463E+00_JPRB, 0.59988E+00_JPRB, 0.60329E+00_JPRB, 0.61084E+00_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.63130E+00_JPRB, 0.61439E+00_JPRB, 0.60136E+00_JPRB, 0.58639E+00_JPRB, 0.56232E+00_JPRB, 0.54002E+00_JPRB, 0.54099E+00_JPRB ,&
&  0.53697E+00_JPRB, 0.53196E+00_JPRB, 0.53405E+00_JPRB, 0.53282E+00_JPRB, 0.52692E+00_JPRB, 0.52936E+00_JPRB, 0.54675E+00_JPRB ,&
&  0.44586E+00_JPRB, 0.41570E+00_JPRB, 0.36665E+00_JPRB, 0.35835E+00_JPRB, 0.41108E+00_JPRB, 0.39187E+00_JPRB, 0.38551E+00_JPRB ,&
&  0.36912E+00_JPRB, 0.36093E+00_JPRB, 0.34739E+00_JPRB, 0.35159E+00_JPRB, 0.36549E+00_JPRB, 0.38199E+00_JPRB, 0.39647E+00_JPRB ,&
&  0.38033E+00_JPRB, 0.35965E+00_JPRB, 0.35923E+00_JPRB, 0.37707E+00_JPRB, 0.43173E+00_JPRB, 0.51253E+00_JPRB, 0.60403E+00_JPRB ,&
&  0.64463E+00_JPRB, 0.65854E+00_JPRB, 0.68142E+00_JPRB, 0.70796E+00_JPRB, 0.74196E+00_JPRB, 0.79494E+00_JPRB, 0.82999E+00_JPRB ,&
&  0.85457E+00_JPRB, 0.87586E+00_JPRB, 0.92184E+00_JPRB, 0.99305E+00_JPRB, 0.95520E+00_JPRB, 0.97066E+00_JPRB, 0.11400E+01_JPRB ,&
&  0.13238E+01_JPRB, 0.12982E+01_JPRB, 0.13731E+01_JPRB, 0.16908E+01_JPRB, 0.18495E+01_JPRB, 0.18216E+01_JPRB, 0.17066E+01_JPRB ,&
&  0.17017E+01_JPRB, 0.17461E+01_JPRB, 0.18037E+01_JPRB, 0.18845E+01_JPRB, 0.19242E+01_JPRB, 0.19810E+01_JPRB, 0.19634E+01_JPRB ,&
&  0.19371E+01_JPRB, 0.19268E+01_JPRB, 0.18439E+01_JPRB, 0.16587E+01_JPRB, 0.13582E+01_JPRB, 0.11190E+01_JPRB, 0.86768E+00_JPRB ,&
&  0.71388E+00_JPRB, 0.54888E+00_JPRB, 0.40408E+00_JPRB, 0.34869E+00_JPRB, 0.32906E+00_JPRB, 0.34708E+00_JPRB, 0.42791E+00_JPRB ,&
&  0.34658E+00_JPRB, 0.34758E+00_JPRB, 0.34603E+00_JPRB, 0.36398E+00_JPRB, 0.39274E+00_JPRB, 0.41637E+00_JPRB, 0.40341E+00_JPRB ,&
&  0.42946E+00_JPRB, 0.57612E+00_JPRB, 0.71315E+00_JPRB, 0.76307E+00_JPRB, 0.76956E+00_JPRB, 0.74277E+00_JPRB, 0.65508E+00_JPRB ,&
&  0.74213E+00_JPRB, 0.66190E+00_JPRB, 0.62943E+00_JPRB, 0.62927E+00_JPRB, 0.64443E+00_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.62488E+00_JPRB, 0.62100E+00_JPRB, 0.61690E+00_JPRB, 0.60320E+00_JPRB, 0.58905E+00_JPRB, 0.56776E+00_JPRB, 0.56819E+00_JPRB ,&
&  0.56342E+00_JPRB, 0.55905E+00_JPRB, 0.55220E+00_JPRB, 0.54775E+00_JPRB, 0.54423E+00_JPRB, 0.53892E+00_JPRB, 0.52763E+00_JPRB ,&
&  0.54998E+00_JPRB, 0.49301E+00_JPRB, 0.42855E+00_JPRB, 0.36236E+00_JPRB, 0.38045E+00_JPRB, 0.37690E+00_JPRB, 0.38278E+00_JPRB ,&
&  0.37564E+00_JPRB, 0.38056E+00_JPRB, 0.36943E+00_JPRB, 0.36047E+00_JPRB, 0.37412E+00_JPRB, 0.39569E+00_JPRB, 0.40694E+00_JPRB ,&
&  0.39687E+00_JPRB, 0.37516E+00_JPRB, 0.37124E+00_JPRB, 0.39911E+00_JPRB, 0.48559E+00_JPRB, 0.49552E+00_JPRB, 0.59505E+00_JPRB ,&
&  0.64942E+00_JPRB, 0.68363E+00_JPRB, 0.71424E+00_JPRB, 0.76291E+00_JPRB, 0.81928E+00_JPRB, 0.89237E+00_JPRB, 0.93546E+00_JPRB ,&
&  0.96021E+00_JPRB, 0.95436E+00_JPRB, 0.91776E+00_JPRB, 0.98264E+00_JPRB, 0.10435E+01_JPRB, 0.10851E+01_JPRB, 0.11559E+01_JPRB ,&
&  0.12482E+01_JPRB, 0.12821E+01_JPRB, 0.14441E+01_JPRB, 0.17955E+01_JPRB, 0.19399E+01_JPRB, 0.19092E+01_JPRB, 0.19569E+01_JPRB ,&
&  0.20372E+01_JPRB, 0.21221E+01_JPRB, 0.21927E+01_JPRB, 0.21874E+01_JPRB, 0.21781E+01_JPRB, 0.20875E+01_JPRB, 0.20360E+01_JPRB ,&
&  0.19601E+01_JPRB, 0.19586E+01_JPRB, 0.19095E+01_JPRB, 0.16429E+01_JPRB, 0.13216E+01_JPRB, 0.10595E+01_JPRB, 0.90073E+00_JPRB ,&
&  0.82654E+00_JPRB, 0.47182E+00_JPRB, 0.34921E+00_JPRB, 0.29338E+00_JPRB, 0.28564E+00_JPRB, 0.29553E+00_JPRB, 0.34947E+00_JPRB ,&
&  0.32512E+00_JPRB, 0.35776E+00_JPRB, 0.41380E+00_JPRB, 0.44355E+00_JPRB, 0.51665E+00_JPRB, 0.60067E+00_JPRB, 0.50642E+00_JPRB ,&
&  0.47662E+00_JPRB, 0.54660E+00_JPRB, 0.78331E+00_JPRB, 0.82468E+00_JPRB, 0.82182E+00_JPRB, 0.79204E+00_JPRB, 0.79001E+00_JPRB ,&
&  0.75733E+00_JPRB, 0.73217E+00_JPRB, 0.68327E+00_JPRB, 0.64711E+00_JPRB, 0.63509E+00_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.65757E+00_JPRB, 0.64290E+00_JPRB, 0.63876E+00_JPRB, 0.62919E+00_JPRB, 0.61934E+00_JPRB, 0.60731E+00_JPRB, 0.60596E+00_JPRB ,&
&  0.58841E+00_JPRB, 0.57403E+00_JPRB, 0.56031E+00_JPRB, 0.55174E+00_JPRB, 0.54780E+00_JPRB, 0.54127E+00_JPRB, 0.52424E+00_JPRB ,&
&  0.50408E+00_JPRB, 0.50863E+00_JPRB, 0.42841E+00_JPRB, 0.37726E+00_JPRB, 0.33459E+00_JPRB, 0.34683E+00_JPRB, 0.36522E+00_JPRB ,&
&  0.39124E+00_JPRB, 0.40145E+00_JPRB, 0.41766E+00_JPRB, 0.43856E+00_JPRB, 0.46317E+00_JPRB, 0.45564E+00_JPRB, 0.47847E+00_JPRB ,&
&  0.46993E+00_JPRB, 0.45077E+00_JPRB, 0.42173E+00_JPRB, 0.47964E+00_JPRB, 0.54175E+00_JPRB, 0.54427E+00_JPRB, 0.67811E+00_JPRB ,&
&  0.74643E+00_JPRB, 0.79442E+00_JPRB, 0.83234E+00_JPRB, 0.87968E+00_JPRB, 0.92941E+00_JPRB, 0.92939E+00_JPRB, 0.89528E+00_JPRB ,&
&  0.83716E+00_JPRB, 0.83356E+00_JPRB, 0.88573E+00_JPRB, 0.91554E+00_JPRB, 0.10357E+01_JPRB, 0.10844E+01_JPRB, 0.11775E+01_JPRB ,&
&  0.13162E+01_JPRB, 0.13130E+01_JPRB, 0.13528E+01_JPRB, 0.18020E+01_JPRB, 0.22647E+01_JPRB, 0.22494E+01_JPRB, 0.22597E+01_JPRB ,&
&  0.23775E+01_JPRB, 0.23864E+01_JPRB, 0.23204E+01_JPRB, 0.21931E+01_JPRB, 0.21520E+01_JPRB, 0.21416E+01_JPRB, 0.21309E+01_JPRB ,&
&  0.20121E+01_JPRB, 0.19560E+01_JPRB, 0.18674E+01_JPRB, 0.16544E+01_JPRB, 0.13711E+01_JPRB, 0.12022E+01_JPRB, 0.10344E+01_JPRB ,&
&  0.87645E+00_JPRB, 0.58435E+00_JPRB, 0.39945E+00_JPRB, 0.33584E+00_JPRB, 0.31938E+00_JPRB, 0.31843E+00_JPRB, 0.35722E+00_JPRB ,&
&  0.36322E+00_JPRB, 0.41893E+00_JPRB, 0.49674E+00_JPRB, 0.53368E+00_JPRB, 0.74236E+00_JPRB, 0.87265E+00_JPRB, 0.82689E+00_JPRB ,&
&  0.71741E+00_JPRB, 0.67133E+00_JPRB, 0.88878E+00_JPRB, 0.90921E+00_JPRB, 0.90429E+00_JPRB, 0.83252E+00_JPRB, 0.81388E+00_JPRB ,&
&  0.78331E+00_JPRB, 0.76928E+00_JPRB, 0.74814E+00_JPRB, 0.70845E+00_JPRB, 0.67549E+00_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.66074E+00_JPRB, 0.63072E+00_JPRB, 0.61318E+00_JPRB, 0.60523E+00_JPRB, 0.58774E+00_JPRB, 0.57564E+00_JPRB, 0.58890E+00_JPRB ,&
&  0.59685E+00_JPRB, 0.58162E+00_JPRB, 0.56771E+00_JPRB, 0.56310E+00_JPRB, 0.56468E+00_JPRB, 0.55856E+00_JPRB, 0.52662E+00_JPRB ,&
&  0.48837E+00_JPRB, 0.46936E+00_JPRB, 0.38864E+00_JPRB, 0.34134E+00_JPRB, 0.32338E+00_JPRB, 0.34627E+00_JPRB, 0.38714E+00_JPRB ,&
&  0.43878E+00_JPRB, 0.48793E+00_JPRB, 0.55110E+00_JPRB, 0.58567E+00_JPRB, 0.61772E+00_JPRB, 0.62671E+00_JPRB, 0.64024E+00_JPRB ,&
&  0.69423E+00_JPRB, 0.57540E+00_JPRB, 0.58458E+00_JPRB, 0.60046E+00_JPRB, 0.67009E+00_JPRB, 0.71932E+00_JPRB, 0.76792E+00_JPRB ,&
&  0.77477E+00_JPRB, 0.80463E+00_JPRB, 0.82986E+00_JPRB, 0.85372E+00_JPRB, 0.87066E+00_JPRB, 0.86683E+00_JPRB, 0.84439E+00_JPRB ,&
&  0.86141E+00_JPRB, 0.84148E+00_JPRB, 0.83843E+00_JPRB, 0.10201E+01_JPRB, 0.14071E+01_JPRB, 0.15593E+01_JPRB, 0.15889E+01_JPRB ,&
&  0.14932E+01_JPRB, 0.15559E+01_JPRB, 0.22740E+01_JPRB, 0.25224E+01_JPRB, 0.25761E+01_JPRB, 0.27878E+01_JPRB, 0.28159E+01_JPRB ,&
&  0.26918E+01_JPRB, 0.24361E+01_JPRB, 0.22009E+01_JPRB, 0.19635E+01_JPRB, 0.22563E+01_JPRB, 0.23430E+01_JPRB, 0.22790E+01_JPRB ,&
&  0.20159E+01_JPRB, 0.19435E+01_JPRB, 0.17489E+01_JPRB, 0.15614E+01_JPRB, 0.14267E+01_JPRB, 0.14126E+01_JPRB, 0.11243E+01_JPRB ,&
&  0.86006E+00_JPRB, 0.77289E+00_JPRB, 0.59593E+00_JPRB, 0.50928E+00_JPRB, 0.47501E+00_JPRB, 0.46922E+00_JPRB, 0.48153E+00_JPRB ,&
&  0.50607E+00_JPRB, 0.55625E+00_JPRB, 0.62855E+00_JPRB, 0.72019E+00_JPRB, 0.94897E+00_JPRB, 0.11119E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.97340E+00_JPRB, 0.10307E+01_JPRB, 0.10107E+01_JPRB, 0.98793E+00_JPRB, 0.99727E+00_JPRB, 0.90482E+00_JPRB, 0.86220E+00_JPRB ,&
&  0.79889E+00_JPRB, 0.77613E+00_JPRB, 0.76640E+00_JPRB, 0.73234E+00_JPRB, 0.70177E+00_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.69739E+00_JPRB, 0.66810E+00_JPRB, 0.62804E+00_JPRB, 0.60000E+00_JPRB, 0.57801E+00_JPRB, 0.56767E+00_JPRB, 0.56767E+00_JPRB ,&
&  0.57082E+00_JPRB, 0.57058E+00_JPRB, 0.58050E+00_JPRB, 0.59241E+00_JPRB, 0.59787E+00_JPRB, 0.57922E+00_JPRB, 0.52472E+00_JPRB ,&
&  0.48210E+00_JPRB, 0.41924E+00_JPRB, 0.36163E+00_JPRB, 0.35965E+00_JPRB, 0.38545E+00_JPRB, 0.41184E+00_JPRB, 0.45404E+00_JPRB ,&
&  0.51507E+00_JPRB, 0.62959E+00_JPRB, 0.71899E+00_JPRB, 0.83824E+00_JPRB, 0.90055E+00_JPRB, 0.91420E+00_JPRB, 0.89491E+00_JPRB ,&
&  0.82370E+00_JPRB, 0.82695E+00_JPRB, 0.86402E+00_JPRB, 0.84552E+00_JPRB, 0.81407E+00_JPRB, 0.80597E+00_JPRB, 0.80583E+00_JPRB ,&
&  0.79787E+00_JPRB, 0.79496E+00_JPRB, 0.83102E+00_JPRB, 0.84970E+00_JPRB, 0.89754E+00_JPRB, 0.92257E+00_JPRB, 0.93461E+00_JPRB ,&
&  0.88829E+00_JPRB, 0.87212E+00_JPRB, 0.99333E+00_JPRB, 0.14516E+01_JPRB, 0.18253E+01_JPRB, 0.21072E+01_JPRB, 0.20266E+01_JPRB ,&
&  0.22494E+01_JPRB, 0.24325E+01_JPRB, 0.25546E+01_JPRB, 0.29016E+01_JPRB, 0.23358E+01_JPRB, 0.24874E+01_JPRB, 0.27644E+01_JPRB ,&
&  0.26864E+01_JPRB, 0.24319E+01_JPRB, 0.22742E+01_JPRB, 0.21358E+01_JPRB, 0.22480E+01_JPRB, 0.28967E+01_JPRB, 0.26607E+01_JPRB ,&
&  0.22137E+01_JPRB, 0.19846E+01_JPRB, 0.17268E+01_JPRB, 0.16457E+01_JPRB, 0.14146E+01_JPRB, 0.12332E+01_JPRB, 0.89367E+00_JPRB ,&
&  0.86467E+00_JPRB, 0.76653E+00_JPRB, 0.75683E+00_JPRB, 0.69959E+00_JPRB, 0.69492E+00_JPRB, 0.66371E+00_JPRB, 0.64970E+00_JPRB ,&
&  0.69139E+00_JPRB, 0.73411E+00_JPRB, 0.89962E+00_JPRB, 0.10712E+01_JPRB, 0.14455E+01_JPRB, 0.13706E+01_JPRB, 0.14201E+01_JPRB ,&
&  0.13394E+01_JPRB, 0.12934E+01_JPRB, 0.11079E+01_JPRB, 0.11204E+01_JPRB, 0.10341E+01_JPRB, 0.97007E+00_JPRB, 0.92379E+00_JPRB ,&
&  0.83922E+00_JPRB, 0.77670E+00_JPRB, 0.73733E+00_JPRB, 0.70864E+00_JPRB, 0.70621E+00_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.66773E+00_JPRB, 0.64325E+00_JPRB, 0.61538E+00_JPRB, 0.58814E+00_JPRB, 0.56055E+00_JPRB, 0.55183E+00_JPRB, 0.55615E+00_JPRB ,&
&  0.56453E+00_JPRB, 0.57881E+00_JPRB, 0.58395E+00_JPRB, 0.58470E+00_JPRB, 0.58386E+00_JPRB, 0.58652E+00_JPRB, 0.53177E+00_JPRB ,&
&  0.48077E+00_JPRB, 0.42728E+00_JPRB, 0.36988E+00_JPRB, 0.36319E+00_JPRB, 0.42897E+00_JPRB, 0.49142E+00_JPRB, 0.48726E+00_JPRB ,&
&  0.63997E+00_JPRB, 0.81654E+00_JPRB, 0.91919E+00_JPRB, 0.10778E+01_JPRB, 0.12444E+01_JPRB, 0.13314E+01_JPRB, 0.12361E+01_JPRB ,&
&  0.11748E+01_JPRB, 0.11117E+01_JPRB, 0.99119E+00_JPRB, 0.89652E+00_JPRB, 0.82872E+00_JPRB, 0.78720E+00_JPRB, 0.75007E+00_JPRB ,&
&  0.73353E+00_JPRB, 0.74095E+00_JPRB, 0.75470E+00_JPRB, 0.78265E+00_JPRB, 0.84018E+00_JPRB, 0.80968E+00_JPRB, 0.84571E+00_JPRB ,&
&  0.89198E+00_JPRB, 0.96539E+00_JPRB, 0.12489E+01_JPRB, 0.17879E+01_JPRB, 0.23853E+01_JPRB, 0.27447E+01_JPRB, 0.27977E+01_JPRB ,&
&  0.26598E+01_JPRB, 0.26582E+01_JPRB, 0.28095E+01_JPRB, 0.28735E+01_JPRB, 0.28448E+01_JPRB, 0.27707E+01_JPRB, 0.29237E+01_JPRB ,&
&  0.22960E+01_JPRB, 0.18931E+01_JPRB, 0.16477E+01_JPRB, 0.18160E+01_JPRB, 0.26652E+01_JPRB, 0.33844E+01_JPRB, 0.33534E+01_JPRB ,&
&  0.28118E+01_JPRB, 0.23688E+01_JPRB, 0.20104E+01_JPRB, 0.17153E+01_JPRB, 0.16276E+01_JPRB, 0.11344E+01_JPRB, 0.11915E+01_JPRB ,&
&  0.11860E+01_JPRB, 0.11756E+01_JPRB, 0.10665E+01_JPRB, 0.90087E+00_JPRB, 0.80125E+00_JPRB, 0.86237E+00_JPRB, 0.88889E+00_JPRB ,&
&  0.95049E+00_JPRB, 0.10194E+01_JPRB, 0.14889E+01_JPRB, 0.17376E+01_JPRB, 0.19059E+01_JPRB, 0.16737E+01_JPRB, 0.16241E+01_JPRB ,&
&  0.13863E+01_JPRB, 0.13747E+01_JPRB, 0.13296E+01_JPRB, 0.12482E+01_JPRB, 0.11511E+01_JPRB, 0.10599E+01_JPRB, 0.97899E+00_JPRB ,&
&  0.90046E+00_JPRB, 0.81937E+00_JPRB, 0.74461E+00_JPRB, 0.69440E+00_JPRB, 0.67907E+00_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.68340E+00_JPRB, 0.66159E+00_JPRB, 0.62983E+00_JPRB, 0.58384E+00_JPRB, 0.56513E+00_JPRB, 0.57285E+00_JPRB, 0.59517E+00_JPRB ,&
&  0.60495E+00_JPRB, 0.60653E+00_JPRB, 0.59312E+00_JPRB, 0.57535E+00_JPRB, 0.57407E+00_JPRB, 0.56348E+00_JPRB, 0.52932E+00_JPRB ,&
&  0.50310E+00_JPRB, 0.47647E+00_JPRB, 0.40255E+00_JPRB, 0.40952E+00_JPRB, 0.48961E+00_JPRB, 0.58913E+00_JPRB, 0.58351E+00_JPRB ,&
&  0.83581E+00_JPRB, 0.97111E+00_JPRB, 0.98861E+00_JPRB, 0.11296E+01_JPRB, 0.15278E+01_JPRB, 0.14687E+01_JPRB, 0.13397E+01_JPRB ,&
&  0.12826E+01_JPRB, 0.11599E+01_JPRB, 0.10026E+01_JPRB, 0.88315E+00_JPRB, 0.80500E+00_JPRB, 0.76003E+00_JPRB, 0.72863E+00_JPRB ,&
&  0.71816E+00_JPRB, 0.72265E+00_JPRB, 0.76743E+00_JPRB, 0.82040E+00_JPRB, 0.79385E+00_JPRB, 0.77026E+00_JPRB, 0.85057E+00_JPRB ,&
&  0.93686E+00_JPRB, 0.10400E+01_JPRB, 0.13342E+01_JPRB, 0.18571E+01_JPRB, 0.25313E+01_JPRB, 0.31030E+01_JPRB, 0.38423E+01_JPRB ,&
&  0.34145E+01_JPRB, 0.34483E+01_JPRB, 0.31675E+01_JPRB, 0.38406E+01_JPRB, 0.34618E+01_JPRB, 0.31426E+01_JPRB, 0.29779E+01_JPRB ,&
&  0.21408E+01_JPRB, 0.17876E+01_JPRB, 0.19146E+01_JPRB, 0.26450E+01_JPRB, 0.28077E+01_JPRB, 0.31199E+01_JPRB, 0.37841E+01_JPRB ,&
&  0.30140E+01_JPRB, 0.25246E+01_JPRB, 0.22598E+01_JPRB, 0.21753E+01_JPRB, 0.14233E+01_JPRB, 0.10741E+01_JPRB, 0.15188E+01_JPRB ,&
&  0.14239E+01_JPRB, 0.87338E+00_JPRB, 0.67759E+00_JPRB, 0.76510E+00_JPRB, 0.61136E+00_JPRB, 0.70063E+00_JPRB, 0.96803E+00_JPRB ,&
&  0.10891E+01_JPRB, 0.12494E+01_JPRB, 0.19106E+01_JPRB, 0.18688E+01_JPRB, 0.19089E+01_JPRB, 0.19258E+01_JPRB, 0.16875E+01_JPRB ,&
&  0.15344E+01_JPRB, 0.13349E+01_JPRB, 0.13846E+01_JPRB, 0.12851E+01_JPRB, 0.11994E+01_JPRB, 0.11296E+01_JPRB, 0.10317E+01_JPRB ,&
&  0.94571E+00_JPRB, 0.85157E+00_JPRB, 0.76986E+00_JPRB, 0.71466E+00_JPRB, 0.69634E+00_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.69340E+00_JPRB, 0.67386E+00_JPRB, 0.66465E+00_JPRB, 0.64025E+00_JPRB, 0.62054E+00_JPRB, 0.61812E+00_JPRB, 0.63272E+00_JPRB ,&
&  0.61634E+00_JPRB, 0.58579E+00_JPRB, 0.55367E+00_JPRB, 0.54208E+00_JPRB, 0.54926E+00_JPRB, 0.55017E+00_JPRB, 0.52256E+00_JPRB ,&
&  0.49995E+00_JPRB, 0.48239E+00_JPRB, 0.48843E+00_JPRB, 0.51427E+00_JPRB, 0.63128E+00_JPRB, 0.68628E+00_JPRB, 0.80310E+00_JPRB ,&
&  0.10051E+01_JPRB, 0.10519E+01_JPRB, 0.10708E+01_JPRB, 0.11911E+01_JPRB, 0.12454E+01_JPRB, 0.12176E+01_JPRB, 0.12665E+01_JPRB ,&
&  0.11837E+01_JPRB, 0.10855E+01_JPRB, 0.96936E+00_JPRB, 0.88471E+00_JPRB, 0.80369E+00_JPRB, 0.74741E+00_JPRB, 0.72690E+00_JPRB ,&
&  0.72130E+00_JPRB, 0.75828E+00_JPRB, 0.84137E+00_JPRB, 0.79369E+00_JPRB, 0.77408E+00_JPRB, 0.77599E+00_JPRB, 0.87291E+00_JPRB ,&
&  0.98451E+00_JPRB, 0.11118E+01_JPRB, 0.13679E+01_JPRB, 0.19523E+01_JPRB, 0.31134E+01_JPRB, 0.39605E+01_JPRB, 0.39378E+01_JPRB ,&
&  0.35890E+01_JPRB, 0.34361E+01_JPRB, 0.36477E+01_JPRB, 0.39886E+01_JPRB, 0.34451E+01_JPRB, 0.32250E+01_JPRB, 0.29126E+01_JPRB ,&
&  0.24941E+01_JPRB, 0.22460E+01_JPRB, 0.21515E+01_JPRB, 0.28288E+01_JPRB, 0.32732E+01_JPRB, 0.32551E+01_JPRB, 0.36656E+01_JPRB ,&
&  0.32896E+01_JPRB, 0.26881E+01_JPRB, 0.20511E+01_JPRB, 0.16371E+01_JPRB, 0.16237E+01_JPRB, 0.13724E+01_JPRB, 0.10178E+01_JPRB ,&
&  0.88120E+00_JPRB, 0.74479E+00_JPRB, 0.63076E+00_JPRB, 0.61823E+00_JPRB, 0.53807E+00_JPRB, 0.59518E+00_JPRB, 0.85187E+00_JPRB ,&
&  0.11751E+01_JPRB, 0.13970E+01_JPRB, 0.17713E+01_JPRB, 0.18377E+01_JPRB, 0.18838E+01_JPRB, 0.20923E+01_JPRB, 0.21449E+01_JPRB ,&
&  0.14250E+01_JPRB, 0.13861E+01_JPRB, 0.14267E+01_JPRB, 0.13456E+01_JPRB, 0.12479E+01_JPRB, 0.11442E+01_JPRB, 0.10252E+01_JPRB ,&
&  0.91955E+00_JPRB, 0.82540E+00_JPRB, 0.76595E+00_JPRB, 0.74566E+00_JPRB, 0.72581E+00_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.66183E+00_JPRB, 0.66068E+00_JPRB, 0.66169E+00_JPRB, 0.65366E+00_JPRB, 0.64078E+00_JPRB, 0.64324E+00_JPRB, 0.61891E+00_JPRB ,&
&  0.57733E+00_JPRB, 0.54389E+00_JPRB, 0.51999E+00_JPRB, 0.52173E+00_JPRB, 0.53188E+00_JPRB, 0.53875E+00_JPRB, 0.52041E+00_JPRB ,&
&  0.49445E+00_JPRB, 0.49123E+00_JPRB, 0.53247E+00_JPRB, 0.60155E+00_JPRB, 0.81651E+00_JPRB, 0.83209E+00_JPRB, 0.10276E+01_JPRB ,&
&  0.11187E+01_JPRB, 0.10774E+01_JPRB, 0.11112E+01_JPRB, 0.11065E+01_JPRB, 0.11567E+01_JPRB, 0.10826E+01_JPRB, 0.10628E+01_JPRB ,&
&  0.95465E+00_JPRB, 0.95910E+00_JPRB, 0.86576E+00_JPRB, 0.80793E+00_JPRB, 0.75787E+00_JPRB, 0.73603E+00_JPRB, 0.74643E+00_JPRB ,&
&  0.78644E+00_JPRB, 0.85214E+00_JPRB, 0.82266E+00_JPRB, 0.79310E+00_JPRB, 0.81069E+00_JPRB, 0.83544E+00_JPRB, 0.93313E+00_JPRB ,&
&  0.10578E+01_JPRB, 0.12013E+01_JPRB, 0.14766E+01_JPRB, 0.24021E+01_JPRB, 0.31964E+01_JPRB, 0.34306E+01_JPRB, 0.38718E+01_JPRB ,&
&  0.40430E+01_JPRB, 0.41134E+01_JPRB, 0.35817E+01_JPRB, 0.41441E+01_JPRB, 0.34418E+01_JPRB, 0.30619E+01_JPRB, 0.28474E+01_JPRB ,&
&  0.25163E+01_JPRB, 0.26132E+01_JPRB, 0.24140E+01_JPRB, 0.25322E+01_JPRB, 0.33367E+01_JPRB, 0.37089E+01_JPRB, 0.35972E+01_JPRB ,&
&  0.34394E+01_JPRB, 0.28677E+01_JPRB, 0.22760E+01_JPRB, 0.19457E+01_JPRB, 0.23711E+01_JPRB, 0.23933E+01_JPRB, 0.11835E+01_JPRB ,&
&  0.85155E+00_JPRB, 0.78646E+00_JPRB, 0.60460E+00_JPRB, 0.52020E+00_JPRB, 0.48677E+00_JPRB, 0.56000E+00_JPRB, 0.91176E+00_JPRB ,&
&  0.10094E+01_JPRB, 0.15459E+01_JPRB, 0.17043E+01_JPRB, 0.18423E+01_JPRB, 0.17933E+01_JPRB, 0.18129E+01_JPRB, 0.22887E+01_JPRB ,&
&  0.23089E+01_JPRB, 0.19952E+01_JPRB, 0.16200E+01_JPRB, 0.14102E+01_JPRB, 0.12290E+01_JPRB, 0.10733E+01_JPRB, 0.95918E+00_JPRB ,&
&  0.89129E+00_JPRB, 0.81962E+00_JPRB, 0.77737E+00_JPRB, 0.75365E+00_JPRB, 0.69844E+00_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.61209E+00_JPRB, 0.62743E+00_JPRB, 0.64262E+00_JPRB, 0.63114E+00_JPRB, 0.61164E+00_JPRB, 0.61066E+00_JPRB, 0.57099E+00_JPRB ,&
&  0.53738E+00_JPRB, 0.51779E+00_JPRB, 0.50797E+00_JPRB, 0.51729E+00_JPRB, 0.54185E+00_JPRB, 0.55999E+00_JPRB, 0.54531E+00_JPRB ,&
&  0.51477E+00_JPRB, 0.51587E+00_JPRB, 0.55032E+00_JPRB, 0.76160E+00_JPRB, 0.95788E+00_JPRB, 0.10941E+01_JPRB, 0.11980E+01_JPRB ,&
&  0.11678E+01_JPRB, 0.11337E+01_JPRB, 0.11296E+01_JPRB, 0.11504E+01_JPRB, 0.11222E+01_JPRB, 0.10982E+01_JPRB, 0.99289E+00_JPRB ,&
&  0.88319E+00_JPRB, 0.89268E+00_JPRB, 0.78937E+00_JPRB, 0.75677E+00_JPRB, 0.74584E+00_JPRB, 0.75590E+00_JPRB, 0.79975E+00_JPRB ,&
&  0.85850E+00_JPRB, 0.83616E+00_JPRB, 0.84476E+00_JPRB, 0.86681E+00_JPRB, 0.91119E+00_JPRB, 0.94070E+00_JPRB, 0.10245E+01_JPRB ,&
&  0.11667E+01_JPRB, 0.13734E+01_JPRB, 0.17924E+01_JPRB, 0.28222E+01_JPRB, 0.29076E+01_JPRB, 0.37544E+01_JPRB, 0.39386E+01_JPRB ,&
&  0.39801E+01_JPRB, 0.39010E+01_JPRB, 0.34879E+01_JPRB, 0.36942E+01_JPRB, 0.33121E+01_JPRB, 0.30079E+01_JPRB, 0.27098E+01_JPRB ,&
&  0.27154E+01_JPRB, 0.25380E+01_JPRB, 0.29259E+01_JPRB, 0.28122E+01_JPRB, 0.31107E+01_JPRB, 0.47512E+01_JPRB, 0.36643E+01_JPRB ,&
&  0.30566E+01_JPRB, 0.31368E+01_JPRB, 0.25442E+01_JPRB, 0.22226E+01_JPRB, 0.26191E+01_JPRB, 0.29109E+01_JPRB, 0.22539E+01_JPRB ,&
&  0.97275E+00_JPRB, 0.69778E+00_JPRB, 0.59816E+00_JPRB, 0.52250E+00_JPRB, 0.45292E+00_JPRB, 0.52513E+00_JPRB, 0.14632E+01_JPRB ,&
&  0.11120E+01_JPRB, 0.15241E+01_JPRB, 0.15783E+01_JPRB, 0.15936E+01_JPRB, 0.17219E+01_JPRB, 0.16230E+01_JPRB, 0.19235E+01_JPRB ,&
&  0.16697E+01_JPRB, 0.13613E+01_JPRB, 0.11373E+01_JPRB, 0.98846E+00_JPRB, 0.91720E+00_JPRB, 0.88263E+00_JPRB, 0.86082E+00_JPRB ,&
&  0.83494E+00_JPRB, 0.77989E+00_JPRB, 0.73024E+00_JPRB, 0.66713E+00_JPRB, 0.61105E+00_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.61583E+00_JPRB, 0.61177E+00_JPRB, 0.61229E+00_JPRB, 0.62487E+00_JPRB, 0.63621E+00_JPRB, 0.61595E+00_JPRB, 0.57996E+00_JPRB ,&
&  0.58233E+00_JPRB, 0.57024E+00_JPRB, 0.58737E+00_JPRB, 0.60287E+00_JPRB, 0.62427E+00_JPRB, 0.61708E+00_JPRB, 0.59686E+00_JPRB ,&
&  0.56099E+00_JPRB, 0.56531E+00_JPRB, 0.69252E+00_JPRB, 0.93717E+00_JPRB, 0.10799E+01_JPRB, 0.11471E+01_JPRB, 0.12625E+01_JPRB ,&
&  0.12885E+01_JPRB, 0.11705E+01_JPRB, 0.10439E+01_JPRB, 0.11161E+01_JPRB, 0.11523E+01_JPRB, 0.11869E+01_JPRB, 0.10297E+01_JPRB ,&
&  0.92396E+00_JPRB, 0.89608E+00_JPRB, 0.79736E+00_JPRB, 0.78638E+00_JPRB, 0.79502E+00_JPRB, 0.81572E+00_JPRB, 0.85322E+00_JPRB ,&
&  0.87221E+00_JPRB, 0.91639E+00_JPRB, 0.97896E+00_JPRB, 0.10311E+01_JPRB, 0.10715E+01_JPRB, 0.10958E+01_JPRB, 0.11807E+01_JPRB ,&
&  0.13552E+01_JPRB, 0.16287E+01_JPRB, 0.20974E+01_JPRB, 0.27581E+01_JPRB, 0.32920E+01_JPRB, 0.35002E+01_JPRB, 0.37860E+01_JPRB ,&
&  0.38239E+01_JPRB, 0.37915E+01_JPRB, 0.35972E+01_JPRB, 0.33574E+01_JPRB, 0.30312E+01_JPRB, 0.29610E+01_JPRB, 0.27015E+01_JPRB ,&
&  0.28103E+01_JPRB, 0.29761E+01_JPRB, 0.32876E+01_JPRB, 0.31610E+01_JPRB, 0.32942E+01_JPRB, 0.44926E+01_JPRB, 0.46868E+01_JPRB ,&
&  0.37722E+01_JPRB, 0.27750E+01_JPRB, 0.23573E+01_JPRB, 0.24385E+01_JPRB, 0.25906E+01_JPRB, 0.26026E+01_JPRB, 0.30935E+01_JPRB ,&
&  0.23474E+01_JPRB, 0.10987E+01_JPRB, 0.65582E+00_JPRB, 0.62467E+00_JPRB, 0.40838E+00_JPRB, 0.39096E+00_JPRB, 0.69379E+00_JPRB ,&
&  0.93828E+00_JPRB, 0.12105E+01_JPRB, 0.12840E+01_JPRB, 0.14114E+01_JPRB, 0.14704E+01_JPRB, 0.12941E+01_JPRB, 0.10781E+01_JPRB ,&
&  0.87459E+00_JPRB, 0.76881E+00_JPRB, 0.72095E+00_JPRB, 0.69835E+00_JPRB, 0.72808E+00_JPRB, 0.77120E+00_JPRB, 0.79316E+00_JPRB ,&
&  0.77442E+00_JPRB, 0.72309E+00_JPRB, 0.65256E+00_JPRB, 0.60227E+00_JPRB, 0.59618E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.67651E+00_JPRB, 0.66433E+00_JPRB, 0.66433E+00_JPRB, 0.68728E+00_JPRB, 0.69702E+00_JPRB, 0.70094E+00_JPRB, 0.74695E+00_JPRB ,&
&  0.76019E+00_JPRB, 0.75188E+00_JPRB, 0.75873E+00_JPRB, 0.75353E+00_JPRB, 0.73427E+00_JPRB, 0.69268E+00_JPRB, 0.66210E+00_JPRB ,&
&  0.65925E+00_JPRB, 0.75874E+00_JPRB, 0.97485E+00_JPRB, 0.12208E+01_JPRB, 0.13074E+01_JPRB, 0.15737E+01_JPRB, 0.12144E+01_JPRB ,&
&  0.11818E+01_JPRB, 0.13655E+01_JPRB, 0.11351E+01_JPRB, 0.11825E+01_JPRB, 0.11989E+01_JPRB, 0.12605E+01_JPRB, 0.10807E+01_JPRB ,&
&  0.99815E+00_JPRB, 0.96021E+00_JPRB, 0.86851E+00_JPRB, 0.85314E+00_JPRB, 0.87364E+00_JPRB, 0.91588E+00_JPRB, 0.94583E+00_JPRB ,&
&  0.10103E+01_JPRB, 0.11110E+01_JPRB, 0.11998E+01_JPRB, 0.12458E+01_JPRB, 0.12751E+01_JPRB, 0.13046E+01_JPRB, 0.14242E+01_JPRB ,&
&  0.16081E+01_JPRB, 0.18973E+01_JPRB, 0.23701E+01_JPRB, 0.28882E+01_JPRB, 0.32131E+01_JPRB, 0.32417E+01_JPRB, 0.34521E+01_JPRB ,&
&  0.33323E+01_JPRB, 0.31328E+01_JPRB, 0.32078E+01_JPRB, 0.31343E+01_JPRB, 0.29152E+01_JPRB, 0.27691E+01_JPRB, 0.25574E+01_JPRB ,&
&  0.29762E+01_JPRB, 0.36792E+01_JPRB, 0.45026E+01_JPRB, 0.38951E+01_JPRB, 0.38279E+01_JPRB, 0.45854E+01_JPRB, 0.47105E+01_JPRB ,&
&  0.40768E+01_JPRB, 0.30973E+01_JPRB, 0.27668E+01_JPRB, 0.24541E+01_JPRB, 0.22950E+01_JPRB, 0.21563E+01_JPRB, 0.23837E+01_JPRB ,&
&  0.23354E+01_JPRB, 0.21396E+01_JPRB, 0.12060E+01_JPRB, 0.59445E+00_JPRB, 0.44884E+00_JPRB, 0.48060E+00_JPRB, 0.58956E+00_JPRB ,&
&  0.77564E+00_JPRB, 0.93485E+00_JPRB, 0.10373E+01_JPRB, 0.12824E+01_JPRB, 0.11603E+01_JPRB, 0.97762E+00_JPRB, 0.81815E+00_JPRB ,&
&  0.72756E+00_JPRB, 0.65862E+00_JPRB, 0.62842E+00_JPRB, 0.63271E+00_JPRB, 0.68295E+00_JPRB, 0.73918E+00_JPRB, 0.76042E+00_JPRB ,&
&  0.72237E+00_JPRB, 0.66733E+00_JPRB, 0.64386E+00_JPRB, 0.64326E+00_JPRB, 0.65758E+00_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.77122E+00_JPRB, 0.78450E+00_JPRB, 0.81862E+00_JPRB, 0.87420E+00_JPRB, 0.92681E+00_JPRB, 0.97844E+00_JPRB, 0.99965E+00_JPRB ,&
&  0.99024E+00_JPRB, 0.96096E+00_JPRB, 0.94557E+00_JPRB, 0.90634E+00_JPRB, 0.87992E+00_JPRB, 0.86883E+00_JPRB, 0.91153E+00_JPRB ,&
&  0.10466E+01_JPRB, 0.12433E+01_JPRB, 0.14268E+01_JPRB, 0.15087E+01_JPRB, 0.15035E+01_JPRB, 0.16252E+01_JPRB, 0.15336E+01_JPRB ,&
&  0.11336E+01_JPRB, 0.16856E+01_JPRB, 0.12690E+01_JPRB, 0.12972E+01_JPRB, 0.13222E+01_JPRB, 0.14259E+01_JPRB, 0.12560E+01_JPRB ,&
&  0.11302E+01_JPRB, 0.10932E+01_JPRB, 0.10562E+01_JPRB, 0.10398E+01_JPRB, 0.10469E+01_JPRB, 0.10861E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.12626E+01_JPRB, 0.13384E+01_JPRB, 0.13983E+01_JPRB, 0.14365E+01_JPRB, 0.14853E+01_JPRB, 0.15467E+01_JPRB, 0.17117E+01_JPRB ,&
&  0.19176E+01_JPRB, 0.22478E+01_JPRB, 0.26307E+01_JPRB, 0.30376E+01_JPRB, 0.30189E+01_JPRB, 0.30035E+01_JPRB, 0.31216E+01_JPRB ,&
&  0.30455E+01_JPRB, 0.28694E+01_JPRB, 0.29222E+01_JPRB, 0.27629E+01_JPRB, 0.25689E+01_JPRB, 0.26511E+01_JPRB, 0.25694E+01_JPRB ,&
&  0.33501E+01_JPRB, 0.38535E+01_JPRB, 0.50856E+01_JPRB, 0.46231E+01_JPRB, 0.40677E+01_JPRB, 0.42415E+01_JPRB, 0.36745E+01_JPRB ,&
&  0.28675E+01_JPRB, 0.26382E+01_JPRB, 0.24337E+01_JPRB, 0.22043E+01_JPRB, 0.20443E+01_JPRB, 0.18474E+01_JPRB, 0.18788E+01_JPRB ,&
&  0.18476E+01_JPRB, 0.17294E+01_JPRB, 0.13299E+01_JPRB, 0.71429E+00_JPRB, 0.54232E+00_JPRB, 0.50863E+00_JPRB, 0.59104E+00_JPRB ,&
&  0.77031E+00_JPRB, 0.90857E+00_JPRB, 0.98098E+00_JPRB, 0.90105E+00_JPRB, 0.78416E+00_JPRB, 0.68498E+00_JPRB, 0.61679E+00_JPRB ,&
&  0.59997E+00_JPRB, 0.60266E+00_JPRB, 0.61282E+00_JPRB, 0.66071E+00_JPRB, 0.71779E+00_JPRB, 0.75319E+00_JPRB, 0.71306E+00_JPRB ,&
&  0.68088E+00_JPRB, 0.68017E+00_JPRB, 0.70908E+00_JPRB, 0.73544E+00_JPRB, 0.75909E+00_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.92018E+00_JPRB, 0.97126E+00_JPRB, 0.10468E+01_JPRB, 0.11630E+01_JPRB, 0.13234E+01_JPRB, 0.14792E+01_JPRB, 0.13858E+01_JPRB ,&
&  0.12723E+01_JPRB, 0.11332E+01_JPRB, 0.11675E+01_JPRB, 0.12009E+01_JPRB, 0.12315E+01_JPRB, 0.12947E+01_JPRB, 0.13892E+01_JPRB ,&
&  0.14948E+01_JPRB, 0.15905E+01_JPRB, 0.16824E+01_JPRB, 0.17616E+01_JPRB, 0.18415E+01_JPRB, 0.20954E+01_JPRB, 0.21146E+01_JPRB ,&
&  0.16227E+01_JPRB, 0.12775E+01_JPRB, 0.15776E+01_JPRB, 0.13883E+01_JPRB, 0.14079E+01_JPRB, 0.15065E+01_JPRB, 0.14794E+01_JPRB ,&
&  0.13343E+01_JPRB, 0.12840E+01_JPRB, 0.12522E+01_JPRB, 0.12493E+01_JPRB, 0.12423E+01_JPRB, 0.13103E+01_JPRB, 0.13890E+01_JPRB ,&
&  0.14509E+01_JPRB, 0.14958E+01_JPRB, 0.15496E+01_JPRB, 0.15994E+01_JPRB, 0.16627E+01_JPRB, 0.17672E+01_JPRB, 0.19684E+01_JPRB ,&
&  0.21765E+01_JPRB, 0.24480E+01_JPRB, 0.27279E+01_JPRB, 0.28648E+01_JPRB, 0.28098E+01_JPRB, 0.27733E+01_JPRB, 0.27978E+01_JPRB ,&
&  0.27848E+01_JPRB, 0.27834E+01_JPRB, 0.27244E+01_JPRB, 0.25889E+01_JPRB, 0.24165E+01_JPRB, 0.27156E+01_JPRB, 0.29612E+01_JPRB ,&
&  0.36568E+01_JPRB, 0.41038E+01_JPRB, 0.51499E+01_JPRB, 0.44547E+01_JPRB, 0.38676E+01_JPRB, 0.34907E+01_JPRB, 0.29827E+01_JPRB ,&
&  0.24551E+01_JPRB, 0.24427E+01_JPRB, 0.23400E+01_JPRB, 0.20037E+01_JPRB, 0.16905E+01_JPRB, 0.13251E+01_JPRB, 0.14313E+01_JPRB ,&
&  0.14583E+01_JPRB, 0.15215E+01_JPRB, 0.10976E+01_JPRB, 0.81287E+00_JPRB, 0.61367E+00_JPRB, 0.55302E+00_JPRB, 0.63341E+00_JPRB ,&
&  0.80843E+00_JPRB, 0.87059E+00_JPRB, 0.79437E+00_JPRB, 0.76090E+00_JPRB, 0.70055E+00_JPRB, 0.61033E+00_JPRB, 0.53823E+00_JPRB ,&
&  0.55970E+00_JPRB, 0.58209E+00_JPRB, 0.61673E+00_JPRB, 0.67261E+00_JPRB, 0.70390E+00_JPRB, 0.74496E+00_JPRB, 0.75007E+00_JPRB ,&
&  0.78454E+00_JPRB, 0.81674E+00_JPRB, 0.83332E+00_JPRB, 0.85278E+00_JPRB, 0.87957E+00_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.99215E+00_JPRB, 0.10376E+01_JPRB, 0.10971E+01_JPRB, 0.11757E+01_JPRB, 0.12583E+01_JPRB, 0.12885E+01_JPRB, 0.12887E+01_JPRB ,&
&  0.13284E+01_JPRB, 0.13905E+01_JPRB, 0.14649E+01_JPRB, 0.15185E+01_JPRB, 0.15381E+01_JPRB, 0.15876E+01_JPRB, 0.16465E+01_JPRB ,&
&  0.17090E+01_JPRB, 0.17862E+01_JPRB, 0.18875E+01_JPRB, 0.20094E+01_JPRB, 0.21850E+01_JPRB, 0.24050E+01_JPRB, 0.25115E+01_JPRB ,&
&  0.19548E+01_JPRB, 0.13422E+01_JPRB, 0.16833E+01_JPRB, 0.16404E+01_JPRB, 0.15130E+01_JPRB, 0.14960E+01_JPRB, 0.14754E+01_JPRB ,&
&  0.14204E+01_JPRB, 0.14193E+01_JPRB, 0.14406E+01_JPRB, 0.14467E+01_JPRB, 0.14427E+01_JPRB, 0.14880E+01_JPRB, 0.15379E+01_JPRB ,&
&  0.15839E+01_JPRB, 0.16288E+01_JPRB, 0.16750E+01_JPRB, 0.17109E+01_JPRB, 0.17775E+01_JPRB, 0.18912E+01_JPRB, 0.20737E+01_JPRB ,&
&  0.22840E+01_JPRB, 0.24683E+01_JPRB, 0.26504E+01_JPRB, 0.27429E+01_JPRB, 0.26848E+01_JPRB, 0.26374E+01_JPRB, 0.26452E+01_JPRB ,&
&  0.26751E+01_JPRB, 0.26415E+01_JPRB, 0.26803E+01_JPRB, 0.28028E+01_JPRB, 0.29865E+01_JPRB, 0.29757E+01_JPRB, 0.35029E+01_JPRB ,&
&  0.39985E+01_JPRB, 0.42014E+01_JPRB, 0.44679E+01_JPRB, 0.51922E+01_JPRB, 0.33240E+01_JPRB, 0.29270E+01_JPRB, 0.24045E+01_JPRB ,&
&  0.25346E+01_JPRB, 0.24932E+01_JPRB, 0.22258E+01_JPRB, 0.18101E+01_JPRB, 0.14822E+01_JPRB, 0.10598E+01_JPRB, 0.10871E+01_JPRB ,&
&  0.12392E+01_JPRB, 0.12793E+01_JPRB, 0.10213E+01_JPRB, 0.91154E+00_JPRB, 0.75741E+00_JPRB, 0.73982E+00_JPRB, 0.79173E+00_JPRB ,&
&  0.84234E+00_JPRB, 0.80235E+00_JPRB, 0.78711E+00_JPRB, 0.83394E+00_JPRB, 0.76748E+00_JPRB, 0.55613E+00_JPRB, 0.48214E+00_JPRB ,&
&  0.49845E+00_JPRB, 0.54043E+00_JPRB, 0.59825E+00_JPRB, 0.66144E+00_JPRB, 0.69581E+00_JPRB, 0.73333E+00_JPRB, 0.77434E+00_JPRB ,&
&  0.84411E+00_JPRB, 0.89481E+00_JPRB, 0.93016E+00_JPRB, 0.93743E+00_JPRB, 0.96628E+00_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.99396E+00_JPRB, 0.10337E+01_JPRB, 0.10593E+01_JPRB, 0.11031E+01_JPRB, 0.11502E+01_JPRB, 0.12037E+01_JPRB, 0.12733E+01_JPRB ,&
&  0.13641E+01_JPRB, 0.14628E+01_JPRB, 0.15541E+01_JPRB, 0.16202E+01_JPRB, 0.16802E+01_JPRB, 0.17242E+01_JPRB, 0.17647E+01_JPRB ,&
&  0.17977E+01_JPRB, 0.18664E+01_JPRB, 0.19409E+01_JPRB, 0.20356E+01_JPRB, 0.21584E+01_JPRB, 0.23035E+01_JPRB, 0.24397E+01_JPRB ,&
&  0.25145E+01_JPRB, 0.23788E+01_JPRB, 0.20443E+01_JPRB, 0.14573E+01_JPRB, 0.12352E+01_JPRB, 0.14316E+01_JPRB, 0.14624E+01_JPRB ,&
&  0.14845E+01_JPRB, 0.14869E+01_JPRB, 0.14794E+01_JPRB, 0.15197E+01_JPRB, 0.15499E+01_JPRB, 0.15773E+01_JPRB, 0.15906E+01_JPRB ,&
&  0.16141E+01_JPRB, 0.16554E+01_JPRB, 0.16803E+01_JPRB, 0.17042E+01_JPRB, 0.17554E+01_JPRB, 0.18599E+01_JPRB, 0.20231E+01_JPRB ,&
&  0.22248E+01_JPRB, 0.23972E+01_JPRB, 0.25197E+01_JPRB, 0.25996E+01_JPRB, 0.25556E+01_JPRB, 0.25641E+01_JPRB, 0.26453E+01_JPRB ,&
&  0.27739E+01_JPRB, 0.28407E+01_JPRB, 0.29795E+01_JPRB, 0.32428E+01_JPRB, 0.34220E+01_JPRB, 0.32861E+01_JPRB, 0.35973E+01_JPRB ,&
&  0.39731E+01_JPRB, 0.40728E+01_JPRB, 0.39968E+01_JPRB, 0.50323E+01_JPRB, 0.31459E+01_JPRB, 0.30488E+01_JPRB, 0.28085E+01_JPRB ,&
&  0.26424E+01_JPRB, 0.26228E+01_JPRB, 0.20882E+01_JPRB, 0.15441E+01_JPRB, 0.13231E+01_JPRB, 0.99540E+00_JPRB, 0.10280E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.10766E+01_JPRB, 0.92071E+00_JPRB, 0.86675E+00_JPRB, 0.86282E+00_JPRB, 0.87850E+00_JPRB, 0.90112E+00_JPRB ,&
&  0.76718E+00_JPRB, 0.76676E+00_JPRB, 0.79134E+00_JPRB, 0.98238E+00_JPRB, 0.10488E+01_JPRB, 0.53014E+00_JPRB, 0.45311E+00_JPRB ,&
&  0.47291E+00_JPRB, 0.51766E+00_JPRB, 0.57057E+00_JPRB, 0.60917E+00_JPRB, 0.64624E+00_JPRB, 0.69354E+00_JPRB, 0.74481E+00_JPRB ,&
&  0.80192E+00_JPRB, 0.86758E+00_JPRB, 0.91307E+00_JPRB, 0.94229E+00_JPRB, 0.96359E+00_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.83307E+00_JPRB, 0.87533E+00_JPRB, 0.91541E+00_JPRB, 0.93881E+00_JPRB, 0.94225E+00_JPRB, 0.95061E+00_JPRB, 0.95206E+00_JPRB ,&
&  0.96206E+00_JPRB, 0.96234E+00_JPRB, 0.99214E+00_JPRB, 0.10605E+01_JPRB, 0.11957E+01_JPRB, 0.12712E+01_JPRB, 0.13661E+01_JPRB ,&
&  0.13754E+01_JPRB, 0.13958E+01_JPRB, 0.14225E+01_JPRB, 0.14612E+01_JPRB, 0.15353E+01_JPRB, 0.16472E+01_JPRB, 0.18848E+01_JPRB ,&
&  0.23210E+01_JPRB, 0.25942E+01_JPRB, 0.25491E+01_JPRB, 0.21991E+01_JPRB, 0.17144E+01_JPRB, 0.14714E+01_JPRB, 0.16417E+01_JPRB ,&
&  0.16791E+01_JPRB, 0.15867E+01_JPRB, 0.15429E+01_JPRB, 0.15362E+01_JPRB, 0.15333E+01_JPRB, 0.15410E+01_JPRB, 0.15444E+01_JPRB ,&
&  0.15613E+01_JPRB, 0.15892E+01_JPRB, 0.16082E+01_JPRB, 0.16275E+01_JPRB, 0.16731E+01_JPRB, 0.17489E+01_JPRB, 0.18550E+01_JPRB ,&
&  0.20030E+01_JPRB, 0.21232E+01_JPRB, 0.21927E+01_JPRB, 0.22225E+01_JPRB, 0.22821E+01_JPRB, 0.24327E+01_JPRB, 0.26635E+01_JPRB ,&
&  0.28755E+01_JPRB, 0.29646E+01_JPRB, 0.32171E+01_JPRB, 0.34757E+01_JPRB, 0.34767E+01_JPRB, 0.34181E+01_JPRB, 0.35524E+01_JPRB ,&
&  0.36389E+01_JPRB, 0.36398E+01_JPRB, 0.31492E+01_JPRB, 0.43158E+01_JPRB, 0.43950E+01_JPRB, 0.36989E+01_JPRB, 0.29529E+01_JPRB ,&
&  0.26466E+01_JPRB, 0.24591E+01_JPRB, 0.20980E+01_JPRB, 0.16339E+01_JPRB, 0.13851E+01_JPRB, 0.13084E+01_JPRB, 0.10924E+01_JPRB ,&
&  0.11103E+01_JPRB, 0.94895E+00_JPRB, 0.87625E+00_JPRB, 0.86951E+00_JPRB, 0.81157E+00_JPRB, 0.85326E+00_JPRB, 0.78062E+00_JPRB ,&
&  0.64912E+00_JPRB, 0.68710E+00_JPRB, 0.75856E+00_JPRB, 0.11426E+01_JPRB, 0.18922E+01_JPRB, 0.50590E+00_JPRB, 0.41797E+00_JPRB ,&
&  0.44071E+00_JPRB, 0.47325E+00_JPRB, 0.51154E+00_JPRB, 0.52909E+00_JPRB, 0.53538E+00_JPRB, 0.57052E+00_JPRB, 0.62303E+00_JPRB ,&
&  0.65669E+00_JPRB, 0.68187E+00_JPRB, 0.71336E+00_JPRB, 0.76902E+00_JPRB, 0.79855E+00_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.58940E+00_JPRB, 0.61808E+00_JPRB, 0.63531E+00_JPRB, 0.65585E+00_JPRB, 0.66420E+00_JPRB, 0.65640E+00_JPRB, 0.62764E+00_JPRB ,&
&  0.60375E+00_JPRB, 0.57863E+00_JPRB, 0.56794E+00_JPRB, 0.55289E+00_JPRB, 0.57043E+00_JPRB, 0.60618E+00_JPRB, 0.63731E+00_JPRB ,&
&  0.71069E+00_JPRB, 0.66912E+00_JPRB, 0.67218E+00_JPRB, 0.70437E+00_JPRB, 0.70996E+00_JPRB, 0.74729E+00_JPRB, 0.87529E+00_JPRB ,&
&  0.10771E+01_JPRB, 0.13041E+01_JPRB, 0.16405E+01_JPRB, 0.21641E+01_JPRB, 0.22674E+01_JPRB, 0.17298E+01_JPRB, 0.17064E+01_JPRB ,&
&  0.15074E+01_JPRB, 0.13172E+01_JPRB, 0.13594E+01_JPRB, 0.13775E+01_JPRB, 0.14198E+01_JPRB, 0.14130E+01_JPRB, 0.14353E+01_JPRB ,&
&  0.14655E+01_JPRB, 0.15041E+01_JPRB, 0.15401E+01_JPRB, 0.15655E+01_JPRB, 0.15991E+01_JPRB, 0.16450E+01_JPRB, 0.17176E+01_JPRB ,&
&  0.17617E+01_JPRB, 0.17923E+01_JPRB, 0.18594E+01_JPRB, 0.18795E+01_JPRB, 0.20735E+01_JPRB, 0.23420E+01_JPRB, 0.26018E+01_JPRB ,&
&  0.27695E+01_JPRB, 0.28818E+01_JPRB, 0.31163E+01_JPRB, 0.31777E+01_JPRB, 0.31930E+01_JPRB, 0.32072E+01_JPRB, 0.31439E+01_JPRB ,&
&  0.29738E+01_JPRB, 0.24333E+01_JPRB, 0.22150E+01_JPRB, 0.34014E+01_JPRB, 0.32944E+01_JPRB, 0.26346E+01_JPRB, 0.26579E+01_JPRB ,&
&  0.24582E+01_JPRB, 0.23501E+01_JPRB, 0.20961E+01_JPRB, 0.17315E+01_JPRB, 0.14504E+01_JPRB, 0.12319E+01_JPRB, 0.10688E+01_JPRB ,&
&  0.98908E+00_JPRB, 0.86200E+00_JPRB, 0.85137E+00_JPRB, 0.86807E+00_JPRB, 0.79899E+00_JPRB, 0.77899E+00_JPRB, 0.69749E+00_JPRB ,&
&  0.68071E+00_JPRB, 0.56933E+00_JPRB, 0.59901E+00_JPRB, 0.73583E+00_JPRB, 0.64504E+00_JPRB, 0.42090E+00_JPRB, 0.38956E+00_JPRB ,&
&  0.40426E+00_JPRB, 0.41789E+00_JPRB, 0.43079E+00_JPRB, 0.44219E+00_JPRB, 0.45116E+00_JPRB, 0.47132E+00_JPRB, 0.48964E+00_JPRB ,&
&  0.50959E+00_JPRB, 0.53997E+00_JPRB, 0.55344E+00_JPRB, 0.56793E+00_JPRB, 0.58451E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.53567E+00_JPRB, 0.53226E+00_JPRB, 0.52493E+00_JPRB, 0.52552E+00_JPRB, 0.53058E+00_JPRB, 0.54376E+00_JPRB, 0.55222E+00_JPRB ,&
&  0.55043E+00_JPRB, 0.55491E+00_JPRB, 0.53837E+00_JPRB, 0.53899E+00_JPRB, 0.55322E+00_JPRB, 0.56185E+00_JPRB, 0.56991E+00_JPRB ,&
&  0.60260E+00_JPRB, 0.61563E+00_JPRB, 0.61361E+00_JPRB, 0.61774E+00_JPRB, 0.61261E+00_JPRB, 0.65563E+00_JPRB, 0.65975E+00_JPRB ,&
&  0.64478E+00_JPRB, 0.67011E+00_JPRB, 0.78838E+00_JPRB, 0.10856E+01_JPRB, 0.13532E+01_JPRB, 0.13304E+01_JPRB, 0.14932E+01_JPRB ,&
&  0.13291E+01_JPRB, 0.10732E+01_JPRB, 0.11705E+01_JPRB, 0.10472E+01_JPRB, 0.10741E+01_JPRB, 0.11193E+01_JPRB, 0.11160E+01_JPRB ,&
&  0.11324E+01_JPRB, 0.11252E+01_JPRB, 0.11374E+01_JPRB, 0.11578E+01_JPRB, 0.11663E+01_JPRB, 0.11808E+01_JPRB, 0.12044E+01_JPRB ,&
&  0.11450E+01_JPRB, 0.11735E+01_JPRB, 0.13024E+01_JPRB, 0.15349E+01_JPRB, 0.17952E+01_JPRB, 0.21414E+01_JPRB, 0.22820E+01_JPRB ,&
&  0.23789E+01_JPRB, 0.24909E+01_JPRB, 0.23487E+01_JPRB, 0.24116E+01_JPRB, 0.25972E+01_JPRB, 0.24254E+01_JPRB, 0.23113E+01_JPRB ,&
&  0.21906E+01_JPRB, 0.17272E+01_JPRB, 0.16490E+01_JPRB, 0.24000E+01_JPRB, 0.25174E+01_JPRB, 0.23686E+01_JPRB, 0.23750E+01_JPRB ,&
&  0.22582E+01_JPRB, 0.20992E+01_JPRB, 0.18089E+01_JPRB, 0.15743E+01_JPRB, 0.13750E+01_JPRB, 0.11593E+01_JPRB, 0.10618E+01_JPRB ,&
&  0.87779E+00_JPRB, 0.83516E+00_JPRB, 0.90156E+00_JPRB, 0.90976E+00_JPRB, 0.85632E+00_JPRB, 0.85267E+00_JPRB, 0.69806E+00_JPRB ,&
&  0.60787E+00_JPRB, 0.51783E+00_JPRB, 0.64886E+00_JPRB, 0.81741E+00_JPRB, 0.49232E+00_JPRB, 0.41505E+00_JPRB, 0.37728E+00_JPRB ,&
&  0.39375E+00_JPRB, 0.41238E+00_JPRB, 0.41907E+00_JPRB, 0.42816E+00_JPRB, 0.44576E+00_JPRB, 0.46139E+00_JPRB, 0.48483E+00_JPRB ,&
&  0.49983E+00_JPRB, 0.51179E+00_JPRB, 0.52425E+00_JPRB, 0.53761E+00_JPRB, 0.53345E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.57968E+00_JPRB, 0.58660E+00_JPRB, 0.59180E+00_JPRB, 0.60769E+00_JPRB, 0.63177E+00_JPRB, 0.63603E+00_JPRB, 0.64918E+00_JPRB ,&
&  0.65638E+00_JPRB, 0.67186E+00_JPRB, 0.68863E+00_JPRB, 0.70857E+00_JPRB, 0.71918E+00_JPRB, 0.72269E+00_JPRB, 0.73971E+00_JPRB ,&
&  0.75295E+00_JPRB, 0.74662E+00_JPRB, 0.75459E+00_JPRB, 0.77154E+00_JPRB, 0.77597E+00_JPRB, 0.80294E+00_JPRB, 0.80101E+00_JPRB ,&
&  0.82886E+00_JPRB, 0.83451E+00_JPRB, 0.86221E+00_JPRB, 0.94170E+00_JPRB, 0.10344E+01_JPRB, 0.12237E+01_JPRB, 0.15630E+01_JPRB ,&
&  0.12804E+01_JPRB, 0.93496E+00_JPRB, 0.87085E+00_JPRB, 0.74952E+00_JPRB, 0.74873E+00_JPRB, 0.77266E+00_JPRB, 0.76851E+00_JPRB ,&
&  0.75156E+00_JPRB, 0.71717E+00_JPRB, 0.70940E+00_JPRB, 0.71259E+00_JPRB, 0.75176E+00_JPRB, 0.84561E+00_JPRB, 0.91072E+00_JPRB ,&
&  0.91577E+00_JPRB, 0.10452E+01_JPRB, 0.12077E+01_JPRB, 0.14386E+01_JPRB, 0.15845E+01_JPRB, 0.19818E+01_JPRB, 0.19988E+01_JPRB ,&
&  0.19355E+01_JPRB, 0.18728E+01_JPRB, 0.16026E+01_JPRB, 0.16774E+01_JPRB, 0.18512E+01_JPRB, 0.17869E+01_JPRB, 0.15949E+01_JPRB ,&
&  0.15806E+01_JPRB, 0.14947E+01_JPRB, 0.15429E+01_JPRB, 0.20389E+01_JPRB, 0.21850E+01_JPRB, 0.22095E+01_JPRB, 0.20867E+01_JPRB ,&
&  0.19306E+01_JPRB, 0.17243E+01_JPRB, 0.15243E+01_JPRB, 0.14160E+01_JPRB, 0.13567E+01_JPRB, 0.12085E+01_JPRB, 0.10213E+01_JPRB ,&
&  0.87434E+00_JPRB, 0.90130E+00_JPRB, 0.95386E+00_JPRB, 0.92794E+00_JPRB, 0.98103E+00_JPRB, 0.75972E+00_JPRB, 0.84684E+00_JPRB ,&
&  0.54703E+00_JPRB, 0.50326E+00_JPRB, 0.44669E+00_JPRB, 0.56285E+00_JPRB, 0.48283E+00_JPRB, 0.42975E+00_JPRB, 0.43984E+00_JPRB ,&
&  0.45284E+00_JPRB, 0.46237E+00_JPRB, 0.47701E+00_JPRB, 0.48859E+00_JPRB, 0.50298E+00_JPRB, 0.52709E+00_JPRB, 0.55323E+00_JPRB ,&
&  0.57414E+00_JPRB, 0.56202E+00_JPRB, 0.56243E+00_JPRB, 0.56656E+00_JPRB, 0.57893E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.61312E+00_JPRB, 0.62986E+00_JPRB, 0.64680E+00_JPRB, 0.66409E+00_JPRB, 0.67916E+00_JPRB, 0.69333E+00_JPRB, 0.70811E+00_JPRB ,&
&  0.72286E+00_JPRB, 0.74003E+00_JPRB, 0.75813E+00_JPRB, 0.77754E+00_JPRB, 0.79280E+00_JPRB, 0.79840E+00_JPRB, 0.80813E+00_JPRB ,&
&  0.81411E+00_JPRB, 0.81422E+00_JPRB, 0.80564E+00_JPRB, 0.82474E+00_JPRB, 0.84532E+00_JPRB, 0.85998E+00_JPRB, 0.88289E+00_JPRB ,&
&  0.90983E+00_JPRB, 0.89957E+00_JPRB, 0.89777E+00_JPRB, 0.92104E+00_JPRB, 0.93517E+00_JPRB, 0.88807E+00_JPRB, 0.96688E+00_JPRB ,&
&  0.72665E+00_JPRB, 0.83613E+00_JPRB, 0.77622E+00_JPRB, 0.68539E+00_JPRB, 0.66571E+00_JPRB, 0.59746E+00_JPRB, 0.56403E+00_JPRB ,&
&  0.61109E+00_JPRB, 0.62380E+00_JPRB, 0.63626E+00_JPRB, 0.67618E+00_JPRB, 0.76807E+00_JPRB, 0.91706E+00_JPRB, 0.10246E+01_JPRB ,&
&  0.11729E+01_JPRB, 0.13154E+01_JPRB, 0.14822E+01_JPRB, 0.16665E+01_JPRB, 0.18725E+01_JPRB, 0.19787E+01_JPRB, 0.19382E+01_JPRB ,&
&  0.18400E+01_JPRB, 0.17324E+01_JPRB, 0.13233E+01_JPRB, 0.14861E+01_JPRB, 0.17489E+01_JPRB, 0.17074E+01_JPRB, 0.14618E+01_JPRB ,&
&  0.13158E+01_JPRB, 0.13102E+01_JPRB, 0.14923E+01_JPRB, 0.16757E+01_JPRB, 0.17794E+01_JPRB, 0.18028E+01_JPRB, 0.16992E+01_JPRB ,&
&  0.15858E+01_JPRB, 0.13942E+01_JPRB, 0.12959E+01_JPRB, 0.13209E+01_JPRB, 0.12958E+01_JPRB, 0.11947E+01_JPRB, 0.98400E+00_JPRB ,&
&  0.90389E+00_JPRB, 0.92152E+00_JPRB, 0.94414E+00_JPRB, 0.90964E+00_JPRB, 0.83180E+00_JPRB, 0.98524E+00_JPRB, 0.10304E+01_JPRB ,&
&  0.54630E+00_JPRB, 0.47810E+00_JPRB, 0.39519E+00_JPRB, 0.49432E+00_JPRB, 0.53783E+00_JPRB, 0.55463E+00_JPRB, 0.60681E+00_JPRB ,&
&  0.63913E+00_JPRB, 0.60059E+00_JPRB, 0.59187E+00_JPRB, 0.56920E+00_JPRB, 0.56805E+00_JPRB, 0.56346E+00_JPRB, 0.57591E+00_JPRB ,&
&  0.58296E+00_JPRB, 0.58216E+00_JPRB, 0.58815E+00_JPRB, 0.59689E+00_JPRB, 0.60242E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.63612E+00_JPRB, 0.65377E+00_JPRB, 0.67161E+00_JPRB, 0.68807E+00_JPRB, 0.70395E+00_JPRB, 0.71820E+00_JPRB, 0.73373E+00_JPRB ,&
&  0.74996E+00_JPRB, 0.76718E+00_JPRB, 0.78348E+00_JPRB, 0.80396E+00_JPRB, 0.82384E+00_JPRB, 0.83362E+00_JPRB, 0.84730E+00_JPRB ,&
&  0.85230E+00_JPRB, 0.87972E+00_JPRB, 0.88909E+00_JPRB, 0.91461E+00_JPRB, 0.93676E+00_JPRB, 0.94893E+00_JPRB, 0.97731E+00_JPRB ,&
&  0.99138E+00_JPRB, 0.10020E+01_JPRB, 0.10138E+01_JPRB, 0.10382E+01_JPRB, 0.10455E+01_JPRB, 0.11013E+01_JPRB, 0.89284E+00_JPRB ,&
&  0.88420E+00_JPRB, 0.82445E+00_JPRB, 0.76894E+00_JPRB, 0.72324E+00_JPRB, 0.68548E+00_JPRB, 0.64779E+00_JPRB, 0.63189E+00_JPRB ,&
&  0.65394E+00_JPRB, 0.68400E+00_JPRB, 0.72735E+00_JPRB, 0.78874E+00_JPRB, 0.89404E+00_JPRB, 0.10475E+01_JPRB, 0.11710E+01_JPRB ,&
&  0.13152E+01_JPRB, 0.14798E+01_JPRB, 0.16360E+01_JPRB, 0.18002E+01_JPRB, 0.19859E+01_JPRB, 0.21716E+01_JPRB, 0.23211E+01_JPRB ,&
&  0.23360E+01_JPRB, 0.22906E+01_JPRB, 0.19545E+01_JPRB, 0.21537E+01_JPRB, 0.23035E+01_JPRB, 0.20822E+01_JPRB, 0.16021E+01_JPRB ,&
&  0.12669E+01_JPRB, 0.10799E+01_JPRB, 0.11302E+01_JPRB, 0.13361E+01_JPRB, 0.13958E+01_JPRB, 0.13875E+01_JPRB, 0.13111E+01_JPRB ,&
&  0.12249E+01_JPRB, 0.11085E+01_JPRB, 0.10612E+01_JPRB, 0.10870E+01_JPRB, 0.11390E+01_JPRB, 0.11505E+01_JPRB, 0.10581E+01_JPRB ,&
&  0.10016E+01_JPRB, 0.95924E+00_JPRB, 0.89425E+00_JPRB, 0.78755E+00_JPRB, 0.71695E+00_JPRB, 0.63492E+00_JPRB, 0.70023E+00_JPRB ,&
&  0.50959E+00_JPRB, 0.43836E+00_JPRB, 0.47090E+00_JPRB, 0.53498E+00_JPRB, 0.60478E+00_JPRB, 0.67634E+00_JPRB, 0.84561E+00_JPRB ,&
&  0.98560E+00_JPRB, 0.10355E+01_JPRB, 0.96845E+00_JPRB, 0.79135E+00_JPRB, 0.65248E+00_JPRB, 0.59806E+00_JPRB, 0.59649E+00_JPRB ,&
&  0.59630E+00_JPRB, 0.59168E+00_JPRB, 0.59762E+00_JPRB, 0.61027E+00_JPRB, 0.62103E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.60717E+00_JPRB, 0.62216E+00_JPRB, 0.63960E+00_JPRB, 0.65417E+00_JPRB, 0.67385E+00_JPRB, 0.70253E+00_JPRB, 0.73059E+00_JPRB ,&
&  0.75379E+00_JPRB, 0.76992E+00_JPRB, 0.78008E+00_JPRB, 0.79735E+00_JPRB, 0.81982E+00_JPRB, 0.84981E+00_JPRB, 0.88924E+00_JPRB ,&
&  0.90022E+00_JPRB, 0.93279E+00_JPRB, 0.94474E+00_JPRB, 0.97518E+00_JPRB, 0.98244E+00_JPRB, 0.10208E+01_JPRB, 0.10461E+01_JPRB ,&
&  0.10598E+01_JPRB, 0.10666E+01_JPRB, 0.10722E+01_JPRB, 0.10897E+01_JPRB, 0.10704E+01_JPRB, 0.10958E+01_JPRB, 0.86073E+00_JPRB ,&
&  0.94793E+00_JPRB, 0.91447E+00_JPRB, 0.84758E+00_JPRB, 0.82053E+00_JPRB, 0.79671E+00_JPRB, 0.77980E+00_JPRB, 0.77414E+00_JPRB ,&
&  0.77286E+00_JPRB, 0.79386E+00_JPRB, 0.79620E+00_JPRB, 0.84173E+00_JPRB, 0.93989E+00_JPRB, 0.10843E+01_JPRB, 0.12129E+01_JPRB ,&
&  0.13663E+01_JPRB, 0.15207E+01_JPRB, 0.16802E+01_JPRB, 0.18872E+01_JPRB, 0.21601E+01_JPRB, 0.24470E+01_JPRB, 0.27302E+01_JPRB ,&
&  0.28339E+01_JPRB, 0.28848E+01_JPRB, 0.26452E+01_JPRB, 0.26702E+01_JPRB, 0.27361E+01_JPRB, 0.22589E+01_JPRB, 0.16035E+01_JPRB ,&
&  0.10193E+01_JPRB, 0.95066E+00_JPRB, 0.10441E+01_JPRB, 0.11407E+01_JPRB, 0.10587E+01_JPRB, 0.99233E+00_JPRB, 0.90554E+00_JPRB ,&
&  0.83066E+00_JPRB, 0.76799E+00_JPRB, 0.77243E+00_JPRB, 0.83395E+00_JPRB, 0.91180E+00_JPRB, 0.99481E+00_JPRB, 0.10163E+01_JPRB ,&
&  0.98372E+00_JPRB, 0.89939E+00_JPRB, 0.80567E+00_JPRB, 0.71418E+00_JPRB, 0.62981E+00_JPRB, 0.56509E+00_JPRB, 0.62551E+00_JPRB ,&
&  0.55832E+00_JPRB, 0.53376E+00_JPRB, 0.56580E+00_JPRB, 0.49452E+00_JPRB, 0.67960E+00_JPRB, 0.83949E+00_JPRB, 0.10848E+01_JPRB ,&
&  0.12149E+01_JPRB, 0.16552E+01_JPRB, 0.20787E+01_JPRB, 0.14863E+01_JPRB, 0.10993E+01_JPRB, 0.67211E+00_JPRB, 0.59942E+00_JPRB ,&
&  0.59672E+00_JPRB, 0.58081E+00_JPRB, 0.57885E+00_JPRB, 0.58640E+00_JPRB, 0.59505E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.57526E+00_JPRB, 0.58451E+00_JPRB, 0.60151E+00_JPRB, 0.60195E+00_JPRB, 0.60931E+00_JPRB, 0.62659E+00_JPRB, 0.64395E+00_JPRB ,&
&  0.66265E+00_JPRB, 0.67081E+00_JPRB, 0.68212E+00_JPRB, 0.69561E+00_JPRB, 0.71173E+00_JPRB, 0.75109E+00_JPRB, 0.81135E+00_JPRB ,&
&  0.85350E+00_JPRB, 0.89260E+00_JPRB, 0.94399E+00_JPRB, 0.95519E+00_JPRB, 0.98381E+00_JPRB, 0.10284E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.10942E+01_JPRB, 0.11156E+01_JPRB, 0.11368E+01_JPRB, 0.11913E+01_JPRB, 0.11547E+01_JPRB, 0.11726E+01_JPRB, 0.91182E+00_JPRB ,&
&  0.97259E+00_JPRB, 0.95538E+00_JPRB, 0.91634E+00_JPRB, 0.89980E+00_JPRB, 0.87955E+00_JPRB, 0.84588E+00_JPRB, 0.82049E+00_JPRB ,&
&  0.81027E+00_JPRB, 0.78198E+00_JPRB, 0.75415E+00_JPRB, 0.80429E+00_JPRB, 0.87463E+00_JPRB, 0.99074E+00_JPRB, 0.11470E+01_JPRB ,&
&  0.13116E+01_JPRB, 0.14672E+01_JPRB, 0.16261E+01_JPRB, 0.18586E+01_JPRB, 0.21662E+01_JPRB, 0.24921E+01_JPRB, 0.28341E+01_JPRB ,&
&  0.30652E+01_JPRB, 0.32389E+01_JPRB, 0.32868E+01_JPRB, 0.27917E+01_JPRB, 0.26465E+01_JPRB, 0.20398E+01_JPRB, 0.13709E+01_JPRB ,&
&  0.10251E+01_JPRB, 0.92293E+00_JPRB, 0.94478E+00_JPRB, 0.96695E+00_JPRB, 0.84182E+00_JPRB, 0.75798E+00_JPRB, 0.67234E+00_JPRB ,&
&  0.61471E+00_JPRB, 0.60119E+00_JPRB, 0.62184E+00_JPRB, 0.68490E+00_JPRB, 0.75973E+00_JPRB, 0.84501E+00_JPRB, 0.87947E+00_JPRB ,&
&  0.84890E+00_JPRB, 0.77149E+00_JPRB, 0.70011E+00_JPRB, 0.63848E+00_JPRB, 0.61767E+00_JPRB, 0.63690E+00_JPRB, 0.59579E+00_JPRB ,&
&  0.59433E+00_JPRB, 0.55502E+00_JPRB, 0.55659E+00_JPRB, 0.61861E+00_JPRB, 0.71687E+00_JPRB, 0.93459E+00_JPRB, 0.13375E+01_JPRB ,&
&  0.19281E+01_JPRB, 0.20750E+01_JPRB, 0.25673E+01_JPRB, 0.34360E+01_JPRB, 0.25301E+01_JPRB, 0.17760E+01_JPRB, 0.54516E+00_JPRB ,&
&  0.55879E+00_JPRB, 0.55390E+00_JPRB, 0.55257E+00_JPRB, 0.55115E+00_JPRB, 0.55785E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.48581E+00_JPRB, 0.50491E+00_JPRB, 0.49494E+00_JPRB, 0.49727E+00_JPRB, 0.50349E+00_JPRB, 0.51350E+00_JPRB, 0.53499E+00_JPRB ,&
&  0.53940E+00_JPRB, 0.56669E+00_JPRB, 0.59432E+00_JPRB, 0.61558E+00_JPRB, 0.64664E+00_JPRB, 0.69045E+00_JPRB, 0.74454E+00_JPRB ,&
&  0.79839E+00_JPRB, 0.82848E+00_JPRB, 0.85171E+00_JPRB, 0.86642E+00_JPRB, 0.89741E+00_JPRB, 0.94273E+00_JPRB, 0.96298E+00_JPRB ,&
&  0.10200E+01_JPRB, 0.10591E+01_JPRB, 0.11089E+01_JPRB, 0.11818E+01_JPRB, 0.11288E+01_JPRB, 0.11241E+01_JPRB, 0.82553E+00_JPRB ,&
&  0.93655E+00_JPRB, 0.94545E+00_JPRB, 0.93652E+00_JPRB, 0.92211E+00_JPRB, 0.89154E+00_JPRB, 0.84003E+00_JPRB, 0.80842E+00_JPRB ,&
&  0.78488E+00_JPRB, 0.76042E+00_JPRB, 0.73152E+00_JPRB, 0.72198E+00_JPRB, 0.81149E+00_JPRB, 0.87404E+00_JPRB, 0.10140E+01_JPRB ,&
&  0.11845E+01_JPRB, 0.13760E+01_JPRB, 0.15589E+01_JPRB, 0.17823E+01_JPRB, 0.20738E+01_JPRB, 0.24077E+01_JPRB, 0.27486E+01_JPRB ,&
&  0.30019E+01_JPRB, 0.32146E+01_JPRB, 0.31798E+01_JPRB, 0.26343E+01_JPRB, 0.23297E+01_JPRB, 0.17655E+01_JPRB, 0.11872E+01_JPRB ,&
&  0.96185E+00_JPRB, 0.89990E+00_JPRB, 0.92133E+00_JPRB, 0.89120E+00_JPRB, 0.75236E+00_JPRB, 0.64969E+00_JPRB, 0.57084E+00_JPRB ,&
&  0.51656E+00_JPRB, 0.51302E+00_JPRB, 0.55955E+00_JPRB, 0.61857E+00_JPRB, 0.68088E+00_JPRB, 0.74003E+00_JPRB, 0.74015E+00_JPRB ,&
&  0.70368E+00_JPRB, 0.63978E+00_JPRB, 0.59622E+00_JPRB, 0.59506E+00_JPRB, 0.65966E+00_JPRB, 0.73709E+00_JPRB, 0.80226E+00_JPRB ,&
&  0.64028E+00_JPRB, 0.53469E+00_JPRB, 0.55328E+00_JPRB, 0.59092E+00_JPRB, 0.64459E+00_JPRB, 0.72154E+00_JPRB, 0.84138E+00_JPRB ,&
&  0.10159E+01_JPRB, 0.13707E+01_JPRB, 0.13464E+01_JPRB, 0.11161E+01_JPRB, 0.16312E+01_JPRB, 0.13528E+01_JPRB, 0.49584E+00_JPRB ,&
&  0.47578E+00_JPRB, 0.46774E+00_JPRB, 0.47134E+00_JPRB, 0.47355E+00_JPRB, 0.48367E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.45277E+00_JPRB, 0.44090E+00_JPRB, 0.42322E+00_JPRB, 0.43068E+00_JPRB, 0.43196E+00_JPRB, 0.44744E+00_JPRB, 0.46510E+00_JPRB ,&
&  0.48877E+00_JPRB, 0.53358E+00_JPRB, 0.57433E+00_JPRB, 0.60594E+00_JPRB, 0.62263E+00_JPRB, 0.64464E+00_JPRB, 0.66656E+00_JPRB ,&
&  0.67843E+00_JPRB, 0.69177E+00_JPRB, 0.71018E+00_JPRB, 0.73893E+00_JPRB, 0.77665E+00_JPRB, 0.81928E+00_JPRB, 0.87209E+00_JPRB ,&
&  0.93041E+00_JPRB, 0.99403E+00_JPRB, 0.10758E+01_JPRB, 0.10977E+01_JPRB, 0.11197E+01_JPRB, 0.11495E+01_JPRB, 0.11262E+01_JPRB ,&
&  0.82232E+00_JPRB, 0.96438E+00_JPRB, 0.94472E+00_JPRB, 0.91305E+00_JPRB, 0.88932E+00_JPRB, 0.85082E+00_JPRB, 0.82010E+00_JPRB ,&
&  0.79767E+00_JPRB, 0.75010E+00_JPRB, 0.73123E+00_JPRB, 0.70430E+00_JPRB, 0.74644E+00_JPRB, 0.77701E+00_JPRB, 0.86922E+00_JPRB ,&
&  0.10077E+01_JPRB, 0.12101E+01_JPRB, 0.14557E+01_JPRB, 0.16902E+01_JPRB, 0.19213E+01_JPRB, 0.21751E+01_JPRB, 0.25378E+01_JPRB ,&
&  0.29255E+01_JPRB, 0.31883E+01_JPRB, 0.33447E+01_JPRB, 0.28814E+01_JPRB, 0.22854E+01_JPRB, 0.19739E+01_JPRB, 0.14701E+01_JPRB ,&
&  0.98773E+00_JPRB, 0.82314E+00_JPRB, 0.85119E+00_JPRB, 0.85236E+00_JPRB, 0.73288E+00_JPRB, 0.64364E+00_JPRB, 0.56405E+00_JPRB ,&
&  0.50195E+00_JPRB, 0.46528E+00_JPRB, 0.48511E+00_JPRB, 0.55944E+00_JPRB, 0.60149E+00_JPRB, 0.62402E+00_JPRB, 0.61031E+00_JPRB ,&
&  0.59060E+00_JPRB, 0.54645E+00_JPRB, 0.50522E+00_JPRB, 0.50605E+00_JPRB, 0.53934E+00_JPRB, 0.57398E+00_JPRB, 0.57461E+00_JPRB ,&
&  0.56034E+00_JPRB, 0.55550E+00_JPRB, 0.56361E+00_JPRB, 0.58505E+00_JPRB, 0.60480E+00_JPRB, 0.63098E+00_JPRB, 0.65707E+00_JPRB ,&
&  0.69196E+00_JPRB, 0.73954E+00_JPRB, 0.73070E+00_JPRB, 0.72345E+00_JPRB, 0.81109E+00_JPRB, 0.86440E+00_JPRB, 0.55470E+00_JPRB ,&
&  0.47217E+00_JPRB, 0.43936E+00_JPRB, 0.43688E+00_JPRB, 0.43041E+00_JPRB, 0.46087E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.45374E+00_JPRB, 0.43636E+00_JPRB, 0.42564E+00_JPRB, 0.41722E+00_JPRB, 0.40615E+00_JPRB, 0.42704E+00_JPRB, 0.43091E+00_JPRB ,&
&  0.45153E+00_JPRB, 0.48678E+00_JPRB, 0.52840E+00_JPRB, 0.54898E+00_JPRB, 0.55188E+00_JPRB, 0.57155E+00_JPRB, 0.58805E+00_JPRB ,&
&  0.59667E+00_JPRB, 0.60190E+00_JPRB, 0.61125E+00_JPRB, 0.62419E+00_JPRB, 0.64854E+00_JPRB, 0.68636E+00_JPRB, 0.74318E+00_JPRB ,&
&  0.80755E+00_JPRB, 0.87511E+00_JPRB, 0.94632E+00_JPRB, 0.10110E+01_JPRB, 0.10502E+01_JPRB, 0.10821E+01_JPRB, 0.10497E+01_JPRB ,&
&  0.66409E+00_JPRB, 0.87219E+00_JPRB, 0.99386E+00_JPRB, 0.94365E+00_JPRB, 0.89800E+00_JPRB, 0.86161E+00_JPRB, 0.82837E+00_JPRB ,&
&  0.79002E+00_JPRB, 0.74525E+00_JPRB, 0.73224E+00_JPRB, 0.75200E+00_JPRB, 0.69574E+00_JPRB, 0.70329E+00_JPRB, 0.75280E+00_JPRB ,&
&  0.84053E+00_JPRB, 0.98923E+00_JPRB, 0.12606E+01_JPRB, 0.15534E+01_JPRB, 0.17167E+01_JPRB, 0.18762E+01_JPRB, 0.21764E+01_JPRB ,&
&  0.26301E+01_JPRB, 0.29741E+01_JPRB, 0.32185E+01_JPRB, 0.24242E+01_JPRB, 0.20586E+01_JPRB, 0.19915E+01_JPRB, 0.17980E+01_JPRB ,&
&  0.11294E+01_JPRB, 0.83440E+00_JPRB, 0.84778E+00_JPRB, 0.85819E+00_JPRB, 0.74671E+00_JPRB, 0.68322E+00_JPRB, 0.61360E+00_JPRB ,&
&  0.53533E+00_JPRB, 0.48026E+00_JPRB, 0.44622E+00_JPRB, 0.47206E+00_JPRB, 0.50184E+00_JPRB, 0.51674E+00_JPRB, 0.49797E+00_JPRB ,&
&  0.51725E+00_JPRB, 0.49720E+00_JPRB, 0.48656E+00_JPRB, 0.49314E+00_JPRB, 0.54050E+00_JPRB, 0.55978E+00_JPRB, 0.56702E+00_JPRB ,&
&  0.57846E+00_JPRB, 0.58693E+00_JPRB, 0.58980E+00_JPRB, 0.60088E+00_JPRB, 0.60958E+00_JPRB, 0.61206E+00_JPRB, 0.61208E+00_JPRB ,&
&  0.60845E+00_JPRB, 0.61481E+00_JPRB, 0.60781E+00_JPRB, 0.59747E+00_JPRB, 0.57643E+00_JPRB, 0.59431E+00_JPRB, 0.53053E+00_JPRB ,&
&  0.48664E+00_JPRB, 0.45456E+00_JPRB, 0.43128E+00_JPRB, 0.42574E+00_JPRB, 0.43305E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.45949E+00_JPRB, 0.45418E+00_JPRB, 0.44715E+00_JPRB, 0.43742E+00_JPRB, 0.42718E+00_JPRB, 0.42703E+00_JPRB, 0.41517E+00_JPRB ,&
&  0.41152E+00_JPRB, 0.43057E+00_JPRB, 0.45906E+00_JPRB, 0.48257E+00_JPRB, 0.50288E+00_JPRB, 0.51202E+00_JPRB, 0.52788E+00_JPRB ,&
&  0.52802E+00_JPRB, 0.52507E+00_JPRB, 0.52177E+00_JPRB, 0.51973E+00_JPRB, 0.52494E+00_JPRB, 0.54579E+00_JPRB, 0.58361E+00_JPRB ,&
&  0.63677E+00_JPRB, 0.70623E+00_JPRB, 0.78921E+00_JPRB, 0.87489E+00_JPRB, 0.95236E+00_JPRB, 0.10244E+01_JPRB, 0.10476E+01_JPRB ,&
&  0.74432E+00_JPRB, 0.46194E+00_JPRB, 0.88910E+00_JPRB, 0.10014E+01_JPRB, 0.94314E+00_JPRB, 0.91301E+00_JPRB, 0.84814E+00_JPRB ,&
&  0.78204E+00_JPRB, 0.76564E+00_JPRB, 0.73705E+00_JPRB, 0.72269E+00_JPRB, 0.66783E+00_JPRB, 0.66090E+00_JPRB, 0.67203E+00_JPRB ,&
&  0.71679E+00_JPRB, 0.81331E+00_JPRB, 0.10073E+01_JPRB, 0.13288E+01_JPRB, 0.15445E+01_JPRB, 0.16304E+01_JPRB, 0.18254E+01_JPRB ,&
&  0.22136E+01_JPRB, 0.25422E+01_JPRB, 0.26691E+01_JPRB, 0.18112E+01_JPRB, 0.12818E+01_JPRB, 0.11491E+01_JPRB, 0.10729E+01_JPRB ,&
&  0.10295E+01_JPRB, 0.87067E+00_JPRB, 0.85994E+00_JPRB, 0.87919E+00_JPRB, 0.80272E+00_JPRB, 0.70819E+00_JPRB, 0.69728E+00_JPRB ,&
&  0.62912E+00_JPRB, 0.56435E+00_JPRB, 0.51086E+00_JPRB, 0.48358E+00_JPRB, 0.47354E+00_JPRB, 0.48274E+00_JPRB, 0.47952E+00_JPRB ,&
&  0.47804E+00_JPRB, 0.48681E+00_JPRB, 0.48620E+00_JPRB, 0.49327E+00_JPRB, 0.52115E+00_JPRB, 0.54838E+00_JPRB, 0.56573E+00_JPRB ,&
&  0.57553E+00_JPRB, 0.59021E+00_JPRB, 0.60129E+00_JPRB, 0.60789E+00_JPRB, 0.61077E+00_JPRB, 0.60016E+00_JPRB, 0.60622E+00_JPRB ,&
&  0.61667E+00_JPRB, 0.62537E+00_JPRB, 0.61874E+00_JPRB, 0.61541E+00_JPRB, 0.58924E+00_JPRB, 0.57230E+00_JPRB, 0.51758E+00_JPRB ,&
&  0.49579E+00_JPRB, 0.45972E+00_JPRB, 0.43343E+00_JPRB, 0.42549E+00_JPRB, 0.43448E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.45822E+00_JPRB, 0.45733E+00_JPRB, 0.45618E+00_JPRB, 0.45919E+00_JPRB, 0.45537E+00_JPRB, 0.45786E+00_JPRB, 0.44405E+00_JPRB ,&
&  0.42056E+00_JPRB, 0.41292E+00_JPRB, 0.42451E+00_JPRB, 0.42916E+00_JPRB, 0.43135E+00_JPRB, 0.44651E+00_JPRB, 0.46284E+00_JPRB ,&
&  0.46849E+00_JPRB, 0.46913E+00_JPRB, 0.46652E+00_JPRB, 0.46501E+00_JPRB, 0.45900E+00_JPRB, 0.46091E+00_JPRB, 0.47637E+00_JPRB ,&
&  0.50037E+00_JPRB, 0.54401E+00_JPRB, 0.60876E+00_JPRB, 0.68713E+00_JPRB, 0.76863E+00_JPRB, 0.87338E+00_JPRB, 0.95848E+00_JPRB ,&
&  0.11191E+01_JPRB, 0.79097E+00_JPRB, 0.53713E+00_JPRB, 0.88630E+00_JPRB, 0.10079E+01_JPRB, 0.10116E+01_JPRB, 0.88794E+00_JPRB ,&
&  0.85168E+00_JPRB, 0.82750E+00_JPRB, 0.81276E+00_JPRB, 0.77159E+00_JPRB, 0.69050E+00_JPRB, 0.65519E+00_JPRB, 0.62479E+00_JPRB ,&
&  0.62071E+00_JPRB, 0.66540E+00_JPRB, 0.77707E+00_JPRB, 0.10039E+01_JPRB, 0.12891E+01_JPRB, 0.14379E+01_JPRB, 0.15265E+01_JPRB ,&
&  0.17936E+01_JPRB, 0.21315E+01_JPRB, 0.22041E+01_JPRB, 0.14850E+01_JPRB, 0.11093E+01_JPRB, 0.10108E+01_JPRB, 0.99135E+00_JPRB ,&
&  0.95273E+00_JPRB, 0.96490E+00_JPRB, 0.98782E+00_JPRB, 0.91169E+00_JPRB, 0.82353E+00_JPRB, 0.75718E+00_JPRB, 0.76182E+00_JPRB ,&
&  0.69772E+00_JPRB, 0.64195E+00_JPRB, 0.59235E+00_JPRB, 0.53885E+00_JPRB, 0.49405E+00_JPRB, 0.48938E+00_JPRB, 0.48148E+00_JPRB ,&
&  0.48279E+00_JPRB, 0.49481E+00_JPRB, 0.49286E+00_JPRB, 0.50509E+00_JPRB, 0.51827E+00_JPRB, 0.53342E+00_JPRB, 0.54757E+00_JPRB ,&
&  0.55838E+00_JPRB, 0.57242E+00_JPRB, 0.58709E+00_JPRB, 0.59070E+00_JPRB, 0.58657E+00_JPRB, 0.57915E+00_JPRB, 0.59329E+00_JPRB ,&
&  0.60407E+00_JPRB, 0.62245E+00_JPRB, 0.62602E+00_JPRB, 0.63802E+00_JPRB, 0.62096E+00_JPRB, 0.60564E+00_JPRB, 0.56982E+00_JPRB ,&
&  0.56762E+00_JPRB, 0.53554E+00_JPRB, 0.50307E+00_JPRB, 0.45673E+00_JPRB, 0.45434E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.47300E+00_JPRB, 0.46459E+00_JPRB, 0.46229E+00_JPRB, 0.46362E+00_JPRB, 0.46265E+00_JPRB, 0.46511E+00_JPRB, 0.46780E+00_JPRB ,&
&  0.44686E+00_JPRB, 0.42606E+00_JPRB, 0.41026E+00_JPRB, 0.40331E+00_JPRB, 0.40102E+00_JPRB, 0.41250E+00_JPRB, 0.41665E+00_JPRB ,&
&  0.41348E+00_JPRB, 0.41305E+00_JPRB, 0.42712E+00_JPRB, 0.43002E+00_JPRB, 0.42942E+00_JPRB, 0.42634E+00_JPRB, 0.42641E+00_JPRB ,&
&  0.43527E+00_JPRB, 0.45317E+00_JPRB, 0.48816E+00_JPRB, 0.53804E+00_JPRB, 0.58803E+00_JPRB, 0.66502E+00_JPRB, 0.75141E+00_JPRB ,&
&  0.89876E+00_JPRB, 0.90750E+00_JPRB, 0.49662E+00_JPRB, 0.78756E+00_JPRB, 0.10330E+01_JPRB, 0.10635E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.94458E+00_JPRB, 0.89815E+00_JPRB, 0.95492E+00_JPRB, 0.91392E+00_JPRB, 0.76780E+00_JPRB, 0.67541E+00_JPRB, 0.60748E+00_JPRB ,&
&  0.57151E+00_JPRB, 0.57143E+00_JPRB, 0.61395E+00_JPRB, 0.72887E+00_JPRB, 0.95445E+00_JPRB, 0.11663E+01_JPRB, 0.12728E+01_JPRB ,&
&  0.14010E+01_JPRB, 0.17248E+01_JPRB, 0.19419E+01_JPRB, 0.13664E+01_JPRB, 0.11102E+01_JPRB, 0.11265E+01_JPRB, 0.11124E+01_JPRB ,&
&  0.10653E+01_JPRB, 0.10522E+01_JPRB, 0.10350E+01_JPRB, 0.95326E+00_JPRB, 0.85745E+00_JPRB, 0.81502E+00_JPRB, 0.82127E+00_JPRB ,&
&  0.75929E+00_JPRB, 0.70419E+00_JPRB, 0.65526E+00_JPRB, 0.60521E+00_JPRB, 0.54875E+00_JPRB, 0.50709E+00_JPRB, 0.50672E+00_JPRB ,&
&  0.50746E+00_JPRB, 0.51819E+00_JPRB, 0.51595E+00_JPRB, 0.52266E+00_JPRB, 0.52919E+00_JPRB, 0.53591E+00_JPRB, 0.54023E+00_JPRB ,&
&  0.54748E+00_JPRB, 0.55670E+00_JPRB, 0.56769E+00_JPRB, 0.56649E+00_JPRB, 0.56549E+00_JPRB, 0.56517E+00_JPRB, 0.56977E+00_JPRB ,&
&  0.58066E+00_JPRB, 0.59343E+00_JPRB, 0.59897E+00_JPRB, 0.60877E+00_JPRB, 0.63708E+00_JPRB, 0.64415E+00_JPRB, 0.63553E+00_JPRB ,&
&  0.68450E+00_JPRB, 0.72650E+00_JPRB, 0.67082E+00_JPRB, 0.52217E+00_JPRB, 0.49139E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.49718E+00_JPRB, 0.48543E+00_JPRB, 0.47677E+00_JPRB, 0.47511E+00_JPRB, 0.47210E+00_JPRB, 0.47081E+00_JPRB, 0.47828E+00_JPRB ,&
&  0.47248E+00_JPRB, 0.46000E+00_JPRB, 0.44089E+00_JPRB, 0.42126E+00_JPRB, 0.40545E+00_JPRB, 0.40648E+00_JPRB, 0.41117E+00_JPRB ,&
&  0.41142E+00_JPRB, 0.40931E+00_JPRB, 0.40907E+00_JPRB, 0.41208E+00_JPRB, 0.41064E+00_JPRB, 0.40855E+00_JPRB, 0.40625E+00_JPRB ,&
&  0.40531E+00_JPRB, 0.40941E+00_JPRB, 0.42441E+00_JPRB, 0.45018E+00_JPRB, 0.47952E+00_JPRB, 0.52218E+00_JPRB, 0.57871E+00_JPRB ,&
&  0.70079E+00_JPRB, 0.76852E+00_JPRB, 0.84076E+00_JPRB, 0.12535E+01_JPRB, 0.13164E+01_JPRB, 0.12183E+01_JPRB, 0.11634E+01_JPRB ,&
&  0.11100E+01_JPRB, 0.10659E+01_JPRB, 0.11703E+01_JPRB, 0.99775E+00_JPRB, 0.84044E+00_JPRB, 0.70814E+00_JPRB, 0.61938E+00_JPRB ,&
&  0.56928E+00_JPRB, 0.53735E+00_JPRB, 0.53673E+00_JPRB, 0.56803E+00_JPRB, 0.68188E+00_JPRB, 0.88042E+00_JPRB, 0.10386E+01_JPRB ,&
&  0.11556E+01_JPRB, 0.13746E+01_JPRB, 0.16925E+01_JPRB, 0.14349E+01_JPRB, 0.11748E+01_JPRB, 0.13283E+01_JPRB, 0.14811E+01_JPRB ,&
&  0.16620E+01_JPRB, 0.12758E+01_JPRB, 0.11239E+01_JPRB, 0.10189E+01_JPRB, 0.89681E+00_JPRB, 0.96459E+00_JPRB, 0.87374E+00_JPRB ,&
&  0.81140E+00_JPRB, 0.76256E+00_JPRB, 0.71222E+00_JPRB, 0.66565E+00_JPRB, 0.61987E+00_JPRB, 0.57443E+00_JPRB, 0.54927E+00_JPRB ,&
&  0.54656E+00_JPRB, 0.55014E+00_JPRB, 0.54731E+00_JPRB, 0.54925E+00_JPRB, 0.54599E+00_JPRB, 0.54971E+00_JPRB, 0.55306E+00_JPRB ,&
&  0.55821E+00_JPRB, 0.55746E+00_JPRB, 0.54138E+00_JPRB, 0.54096E+00_JPRB, 0.54288E+00_JPRB, 0.54206E+00_JPRB, 0.53952E+00_JPRB ,&
&  0.54829E+00_JPRB, 0.56533E+00_JPRB, 0.57699E+00_JPRB, 0.58880E+00_JPRB, 0.60550E+00_JPRB, 0.61281E+00_JPRB, 0.60155E+00_JPRB ,&
&  0.59869E+00_JPRB, 0.61106E+00_JPRB, 0.61336E+00_JPRB, 0.54764E+00_JPRB, 0.51848E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.50404E+00_JPRB, 0.49401E+00_JPRB, 0.48751E+00_JPRB, 0.48092E+00_JPRB, 0.47409E+00_JPRB, 0.47327E+00_JPRB, 0.47860E+00_JPRB ,&
&  0.47944E+00_JPRB, 0.47172E+00_JPRB, 0.45899E+00_JPRB, 0.44624E+00_JPRB, 0.42788E+00_JPRB, 0.41736E+00_JPRB, 0.41176E+00_JPRB ,&
&  0.40767E+00_JPRB, 0.40750E+00_JPRB, 0.40379E+00_JPRB, 0.40176E+00_JPRB, 0.39960E+00_JPRB, 0.39268E+00_JPRB, 0.38972E+00_JPRB ,&
&  0.38704E+00_JPRB, 0.39002E+00_JPRB, 0.39755E+00_JPRB, 0.41182E+00_JPRB, 0.43269E+00_JPRB, 0.46217E+00_JPRB, 0.49671E+00_JPRB ,&
&  0.57936E+00_JPRB, 0.70159E+00_JPRB, 0.71742E+00_JPRB, 0.13448E+01_JPRB, 0.13216E+01_JPRB, 0.12723E+01_JPRB, 0.11757E+01_JPRB ,&
&  0.11603E+01_JPRB, 0.12709E+01_JPRB, 0.11854E+01_JPRB, 0.10247E+01_JPRB, 0.85346E+00_JPRB, 0.73023E+00_JPRB, 0.63692E+00_JPRB ,&
&  0.57871E+00_JPRB, 0.54239E+00_JPRB, 0.52519E+00_JPRB, 0.51960E+00_JPRB, 0.53282E+00_JPRB, 0.64304E+00_JPRB, 0.81952E+00_JPRB ,&
&  0.95490E+00_JPRB, 0.11170E+01_JPRB, 0.14135E+01_JPRB, 0.13509E+01_JPRB, 0.12620E+01_JPRB, 0.14743E+01_JPRB, 0.18822E+01_JPRB ,&
&  0.22596E+01_JPRB, 0.17544E+01_JPRB, 0.13241E+01_JPRB, 0.11398E+01_JPRB, 0.10380E+01_JPRB, 0.10153E+01_JPRB, 0.93576E+00_JPRB ,&
&  0.84943E+00_JPRB, 0.80187E+00_JPRB, 0.76038E+00_JPRB, 0.72062E+00_JPRB, 0.67726E+00_JPRB, 0.64021E+00_JPRB, 0.60315E+00_JPRB ,&
&  0.58679E+00_JPRB, 0.58003E+00_JPRB, 0.57569E+00_JPRB, 0.58000E+00_JPRB, 0.57879E+00_JPRB, 0.58225E+00_JPRB, 0.58115E+00_JPRB ,&
&  0.58318E+00_JPRB, 0.57434E+00_JPRB, 0.53976E+00_JPRB, 0.52782E+00_JPRB, 0.52793E+00_JPRB, 0.52571E+00_JPRB, 0.52200E+00_JPRB ,&
&  0.53439E+00_JPRB, 0.54772E+00_JPRB, 0.55318E+00_JPRB, 0.55725E+00_JPRB, 0.58105E+00_JPRB, 0.59761E+00_JPRB, 0.59273E+00_JPRB ,&
&  0.57977E+00_JPRB, 0.56817E+00_JPRB, 0.55425E+00_JPRB, 0.52801E+00_JPRB, 0.51672E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.50450E+00_JPRB, 0.49746E+00_JPRB, 0.49081E+00_JPRB, 0.48235E+00_JPRB, 0.47118E+00_JPRB, 0.46572E+00_JPRB, 0.46972E+00_JPRB ,&
&  0.47622E+00_JPRB, 0.47222E+00_JPRB, 0.46408E+00_JPRB, 0.45110E+00_JPRB, 0.43279E+00_JPRB, 0.42379E+00_JPRB, 0.41803E+00_JPRB ,&
&  0.40868E+00_JPRB, 0.40147E+00_JPRB, 0.39082E+00_JPRB, 0.38599E+00_JPRB, 0.38500E+00_JPRB, 0.38062E+00_JPRB, 0.37198E+00_JPRB ,&
&  0.37435E+00_JPRB, 0.38163E+00_JPRB, 0.39059E+00_JPRB, 0.39916E+00_JPRB, 0.41563E+00_JPRB, 0.43958E+00_JPRB, 0.47346E+00_JPRB ,&
&  0.53565E+00_JPRB, 0.75177E+00_JPRB, 0.66021E+00_JPRB, 0.10495E+01_JPRB, 0.10766E+01_JPRB, 0.11395E+01_JPRB, 0.11469E+01_JPRB ,&
&  0.11672E+01_JPRB, 0.12037E+01_JPRB, 0.11464E+01_JPRB, 0.97143E+00_JPRB, 0.80329E+00_JPRB, 0.70076E+00_JPRB, 0.63700E+00_JPRB ,&
&  0.59488E+00_JPRB, 0.56204E+00_JPRB, 0.53797E+00_JPRB, 0.52087E+00_JPRB, 0.50836E+00_JPRB, 0.52160E+00_JPRB, 0.61311E+00_JPRB ,&
&  0.76648E+00_JPRB, 0.90897E+00_JPRB, 0.11359E+01_JPRB, 0.13742E+01_JPRB, 0.12662E+01_JPRB, 0.14141E+01_JPRB, 0.20168E+01_JPRB ,&
&  0.24181E+01_JPRB, 0.24368E+01_JPRB, 0.16743E+01_JPRB, 0.13417E+01_JPRB, 0.12049E+01_JPRB, 0.11172E+01_JPRB, 0.98731E+00_JPRB ,&
&  0.87077E+00_JPRB, 0.81604E+00_JPRB, 0.78475E+00_JPRB, 0.75721E+00_JPRB, 0.72322E+00_JPRB, 0.67751E+00_JPRB, 0.63406E+00_JPRB ,&
&  0.59834E+00_JPRB, 0.58954E+00_JPRB, 0.58911E+00_JPRB, 0.59229E+00_JPRB, 0.59893E+00_JPRB, 0.60516E+00_JPRB, 0.60841E+00_JPRB ,&
&  0.60546E+00_JPRB, 0.59859E+00_JPRB, 0.55892E+00_JPRB, 0.53779E+00_JPRB, 0.53202E+00_JPRB, 0.52792E+00_JPRB, 0.52294E+00_JPRB ,&
&  0.53292E+00_JPRB, 0.53998E+00_JPRB, 0.54152E+00_JPRB, 0.55136E+00_JPRB, 0.56902E+00_JPRB, 0.59449E+00_JPRB, 0.56912E+00_JPRB ,&
&  0.57121E+00_JPRB, 0.55892E+00_JPRB, 0.54429E+00_JPRB, 0.52457E+00_JPRB, 0.51523E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.50425E+00_JPRB, 0.49828E+00_JPRB, 0.49185E+00_JPRB, 0.48489E+00_JPRB, 0.47619E+00_JPRB, 0.46910E+00_JPRB, 0.46624E+00_JPRB ,&
&  0.46684E+00_JPRB, 0.46958E+00_JPRB, 0.46518E+00_JPRB, 0.45140E+00_JPRB, 0.43556E+00_JPRB, 0.42127E+00_JPRB, 0.41384E+00_JPRB ,&
&  0.40786E+00_JPRB, 0.39979E+00_JPRB, 0.38956E+00_JPRB, 0.37919E+00_JPRB, 0.37631E+00_JPRB, 0.37823E+00_JPRB, 0.37394E+00_JPRB ,&
&  0.37955E+00_JPRB, 0.38678E+00_JPRB, 0.39374E+00_JPRB, 0.40034E+00_JPRB, 0.41515E+00_JPRB, 0.43791E+00_JPRB, 0.47000E+00_JPRB ,&
&  0.52800E+00_JPRB, 0.45488E+00_JPRB, 0.71479E+00_JPRB, 0.82258E+00_JPRB, 0.88950E+00_JPRB, 0.94955E+00_JPRB, 0.10679E+01_JPRB ,&
&  0.11642E+01_JPRB, 0.11329E+01_JPRB, 0.10447E+01_JPRB, 0.90578E+00_JPRB, 0.77952E+00_JPRB, 0.68895E+00_JPRB, 0.63141E+00_JPRB ,&
&  0.59578E+00_JPRB, 0.56871E+00_JPRB, 0.54423E+00_JPRB, 0.52293E+00_JPRB, 0.50801E+00_JPRB, 0.50055E+00_JPRB, 0.51248E+00_JPRB ,&
&  0.57328E+00_JPRB, 0.68716E+00_JPRB, 0.84180E+00_JPRB, 0.11157E+01_JPRB, 0.12523E+01_JPRB, 0.13479E+01_JPRB, 0.15771E+01_JPRB ,&
&  0.20910E+01_JPRB, 0.23977E+01_JPRB, 0.19330E+01_JPRB, 0.15566E+01_JPRB, 0.13633E+01_JPRB, 0.11778E+01_JPRB, 0.10152E+01_JPRB ,&
&  0.89341E+00_JPRB, 0.82275E+00_JPRB, 0.77976E+00_JPRB, 0.75310E+00_JPRB, 0.73605E+00_JPRB, 0.71065E+00_JPRB, 0.67533E+00_JPRB ,&
&  0.64452E+00_JPRB, 0.62446E+00_JPRB, 0.61690E+00_JPRB, 0.61739E+00_JPRB, 0.61881E+00_JPRB, 0.62482E+00_JPRB, 0.62454E+00_JPRB ,&
&  0.61591E+00_JPRB, 0.60310E+00_JPRB, 0.57763E+00_JPRB, 0.55541E+00_JPRB, 0.54842E+00_JPRB, 0.54093E+00_JPRB, 0.53278E+00_JPRB ,&
&  0.53173E+00_JPRB, 0.53515E+00_JPRB, 0.53908E+00_JPRB, 0.55206E+00_JPRB, 0.55538E+00_JPRB, 0.58009E+00_JPRB, 0.55611E+00_JPRB ,&
&  0.55320E+00_JPRB, 0.54134E+00_JPRB, 0.53038E+00_JPRB, 0.51765E+00_JPRB, 0.51416E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.50963E+00_JPRB, 0.50384E+00_JPRB, 0.49611E+00_JPRB, 0.48834E+00_JPRB, 0.47897E+00_JPRB, 0.47004E+00_JPRB, 0.46463E+00_JPRB ,&
&  0.46183E+00_JPRB, 0.46230E+00_JPRB, 0.46329E+00_JPRB, 0.45328E+00_JPRB, 0.43948E+00_JPRB, 0.42685E+00_JPRB, 0.41926E+00_JPRB ,&
&  0.41396E+00_JPRB, 0.40836E+00_JPRB, 0.40120E+00_JPRB, 0.39189E+00_JPRB, 0.38746E+00_JPRB, 0.38546E+00_JPRB, 0.38448E+00_JPRB ,&
&  0.38962E+00_JPRB, 0.39428E+00_JPRB, 0.40043E+00_JPRB, 0.40905E+00_JPRB, 0.42355E+00_JPRB, 0.44460E+00_JPRB, 0.47629E+00_JPRB ,&
&  0.53199E+00_JPRB, 0.61835E+00_JPRB, 0.75133E+00_JPRB, 0.71325E+00_JPRB, 0.73233E+00_JPRB, 0.76632E+00_JPRB, 0.87883E+00_JPRB ,&
&  0.96885E+00_JPRB, 0.99650E+00_JPRB, 0.92998E+00_JPRB, 0.83312E+00_JPRB, 0.74506E+00_JPRB, 0.67671E+00_JPRB, 0.62868E+00_JPRB ,&
&  0.58466E+00_JPRB, 0.54562E+00_JPRB, 0.51828E+00_JPRB, 0.49942E+00_JPRB, 0.48685E+00_JPRB, 0.47920E+00_JPRB, 0.47709E+00_JPRB ,&
&  0.48767E+00_JPRB, 0.52615E+00_JPRB, 0.61599E+00_JPRB, 0.80152E+00_JPRB, 0.11944E+01_JPRB, 0.12010E+01_JPRB, 0.14730E+01_JPRB ,&
&  0.19863E+01_JPRB, 0.19467E+01_JPRB, 0.17723E+01_JPRB, 0.15540E+01_JPRB, 0.13604E+01_JPRB, 0.11867E+01_JPRB, 0.10384E+01_JPRB ,&
&  0.93879E+00_JPRB, 0.86668E+00_JPRB, 0.80237E+00_JPRB, 0.75473E+00_JPRB, 0.72385E+00_JPRB, 0.70953E+00_JPRB, 0.69697E+00_JPRB ,&
&  0.68081E+00_JPRB, 0.66307E+00_JPRB, 0.65024E+00_JPRB, 0.64480E+00_JPRB, 0.63982E+00_JPRB, 0.63661E+00_JPRB, 0.62095E+00_JPRB ,&
&  0.60316E+00_JPRB, 0.58749E+00_JPRB, 0.57432E+00_JPRB, 0.55515E+00_JPRB, 0.55240E+00_JPRB, 0.54902E+00_JPRB, 0.53831E+00_JPRB ,&
&  0.52569E+00_JPRB, 0.52474E+00_JPRB, 0.53654E+00_JPRB, 0.54442E+00_JPRB, 0.55896E+00_JPRB, 0.58116E+00_JPRB, 0.55430E+00_JPRB ,&
&  0.54628E+00_JPRB, 0.52952E+00_JPRB, 0.51472E+00_JPRB, 0.50978E+00_JPRB, 0.51748E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.51212E+00_JPRB, 0.50396E+00_JPRB, 0.49571E+00_JPRB, 0.48801E+00_JPRB, 0.47737E+00_JPRB, 0.46828E+00_JPRB, 0.45969E+00_JPRB ,&
&  0.45368E+00_JPRB, 0.45076E+00_JPRB, 0.45156E+00_JPRB, 0.45236E+00_JPRB, 0.45009E+00_JPRB, 0.44276E+00_JPRB, 0.42983E+00_JPRB ,&
&  0.42152E+00_JPRB, 0.41485E+00_JPRB, 0.40676E+00_JPRB, 0.39686E+00_JPRB, 0.38722E+00_JPRB, 0.38310E+00_JPRB, 0.38348E+00_JPRB ,&
&  0.38803E+00_JPRB, 0.39386E+00_JPRB, 0.39781E+00_JPRB, 0.40936E+00_JPRB, 0.42564E+00_JPRB, 0.44676E+00_JPRB, 0.47603E+00_JPRB ,&
&  0.51874E+00_JPRB, 0.61040E+00_JPRB, 0.64154E+00_JPRB, 0.64134E+00_JPRB, 0.64885E+00_JPRB, 0.67104E+00_JPRB, 0.73572E+00_JPRB ,&
&  0.79866E+00_JPRB, 0.84054E+00_JPRB, 0.82159E+00_JPRB, 0.74865E+00_JPRB, 0.68809E+00_JPRB, 0.64510E+00_JPRB, 0.60317E+00_JPRB ,&
&  0.56280E+00_JPRB, 0.53140E+00_JPRB, 0.50651E+00_JPRB, 0.48961E+00_JPRB, 0.47691E+00_JPRB, 0.46778E+00_JPRB, 0.46326E+00_JPRB ,&
&  0.46385E+00_JPRB, 0.47023E+00_JPRB, 0.49618E+00_JPRB, 0.56011E+00_JPRB, 0.83012E+00_JPRB, 0.10680E+01_JPRB, 0.12792E+01_JPRB ,&
&  0.13559E+01_JPRB, 0.14022E+01_JPRB, 0.14245E+01_JPRB, 0.13516E+01_JPRB, 0.12385E+01_JPRB, 0.11018E+01_JPRB, 0.99898E+00_JPRB ,&
&  0.93368E+00_JPRB, 0.86589E+00_JPRB, 0.79988E+00_JPRB, 0.73803E+00_JPRB, 0.69438E+00_JPRB, 0.67088E+00_JPRB, 0.65720E+00_JPRB ,&
&  0.64273E+00_JPRB, 0.63338E+00_JPRB, 0.62622E+00_JPRB, 0.61922E+00_JPRB, 0.61106E+00_JPRB, 0.60004E+00_JPRB, 0.58516E+00_JPRB ,&
&  0.56728E+00_JPRB, 0.55016E+00_JPRB, 0.54159E+00_JPRB, 0.53642E+00_JPRB, 0.53310E+00_JPRB, 0.52679E+00_JPRB, 0.51865E+00_JPRB ,&
&  0.51857E+00_JPRB, 0.52590E+00_JPRB, 0.53163E+00_JPRB, 0.54039E+00_JPRB, 0.54656E+00_JPRB, 0.56318E+00_JPRB, 0.54539E+00_JPRB ,&
&  0.53147E+00_JPRB, 0.51493E+00_JPRB, 0.50143E+00_JPRB, 0.49564E+00_JPRB, 0.51511E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.52170E+00_JPRB, 0.50980E+00_JPRB, 0.49578E+00_JPRB, 0.47855E+00_JPRB, 0.46369E+00_JPRB, 0.45401E+00_JPRB, 0.44729E+00_JPRB ,&
&  0.44364E+00_JPRB, 0.44203E+00_JPRB, 0.44085E+00_JPRB, 0.43927E+00_JPRB, 0.43676E+00_JPRB, 0.43365E+00_JPRB, 0.42698E+00_JPRB ,&
&  0.41887E+00_JPRB, 0.40935E+00_JPRB, 0.40169E+00_JPRB, 0.39496E+00_JPRB, 0.38294E+00_JPRB, 0.37603E+00_JPRB, 0.37780E+00_JPRB ,&
&  0.38261E+00_JPRB, 0.38846E+00_JPRB, 0.39296E+00_JPRB, 0.40365E+00_JPRB, 0.42338E+00_JPRB, 0.44410E+00_JPRB, 0.46953E+00_JPRB ,&
&  0.51697E+00_JPRB, 0.50243E+00_JPRB, 0.54165E+00_JPRB, 0.55372E+00_JPRB, 0.55842E+00_JPRB, 0.58856E+00_JPRB, 0.62939E+00_JPRB ,&
&  0.65824E+00_JPRB, 0.70139E+00_JPRB, 0.69509E+00_JPRB, 0.64591E+00_JPRB, 0.59636E+00_JPRB, 0.56096E+00_JPRB, 0.53872E+00_JPRB ,&
&  0.52149E+00_JPRB, 0.50667E+00_JPRB, 0.49373E+00_JPRB, 0.48587E+00_JPRB, 0.47821E+00_JPRB, 0.47077E+00_JPRB, 0.46334E+00_JPRB ,&
&  0.45955E+00_JPRB, 0.45735E+00_JPRB, 0.45934E+00_JPRB, 0.47534E+00_JPRB, 0.53906E+00_JPRB, 0.65908E+00_JPRB, 0.82668E+00_JPRB ,&
&  0.96654E+00_JPRB, 0.10519E+01_JPRB, 0.11278E+01_JPRB, 0.11320E+01_JPRB, 0.10871E+01_JPRB, 0.10316E+01_JPRB, 0.96899E+00_JPRB ,&
&  0.88425E+00_JPRB, 0.78755E+00_JPRB, 0.71528E+00_JPRB, 0.66422E+00_JPRB, 0.62993E+00_JPRB, 0.60909E+00_JPRB, 0.59704E+00_JPRB ,&
&  0.58773E+00_JPRB, 0.58558E+00_JPRB, 0.59766E+00_JPRB, 0.61078E+00_JPRB, 0.61017E+00_JPRB, 0.59178E+00_JPRB, 0.56542E+00_JPRB ,&
&  0.54170E+00_JPRB, 0.52342E+00_JPRB, 0.51043E+00_JPRB, 0.50130E+00_JPRB, 0.49470E+00_JPRB, 0.49042E+00_JPRB, 0.48564E+00_JPRB ,&
&  0.48555E+00_JPRB, 0.49914E+00_JPRB, 0.50123E+00_JPRB, 0.51897E+00_JPRB, 0.54683E+00_JPRB, 0.53312E+00_JPRB, 0.51443E+00_JPRB ,&
&  0.50091E+00_JPRB, 0.49347E+00_JPRB, 0.48469E+00_JPRB, 0.48016E+00_JPRB, 0.53432E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.45807E+00_JPRB, 0.45899E+00_JPRB, 0.45145E+00_JPRB, 0.44189E+00_JPRB, 0.43353E+00_JPRB, 0.42911E+00_JPRB, 0.42565E+00_JPRB ,&
&  0.42386E+00_JPRB, 0.42287E+00_JPRB, 0.42210E+00_JPRB, 0.42015E+00_JPRB, 0.41641E+00_JPRB, 0.41053E+00_JPRB, 0.40309E+00_JPRB ,&
&  0.39641E+00_JPRB, 0.38968E+00_JPRB, 0.38402E+00_JPRB, 0.37764E+00_JPRB, 0.36970E+00_JPRB, 0.36575E+00_JPRB, 0.36832E+00_JPRB ,&
&  0.37397E+00_JPRB, 0.38085E+00_JPRB, 0.38710E+00_JPRB, 0.39720E+00_JPRB, 0.41903E+00_JPRB, 0.44039E+00_JPRB, 0.46040E+00_JPRB ,&
&  0.49961E+00_JPRB, 0.45802E+00_JPRB, 0.47627E+00_JPRB, 0.50036E+00_JPRB, 0.51999E+00_JPRB, 0.53382E+00_JPRB, 0.55138E+00_JPRB ,&
&  0.56812E+00_JPRB, 0.58087E+00_JPRB, 0.58145E+00_JPRB, 0.56106E+00_JPRB, 0.52208E+00_JPRB, 0.48718E+00_JPRB, 0.46679E+00_JPRB ,&
&  0.45618E+00_JPRB, 0.45163E+00_JPRB, 0.44742E+00_JPRB, 0.44379E+00_JPRB, 0.44166E+00_JPRB, 0.43983E+00_JPRB, 0.43777E+00_JPRB ,&
&  0.43587E+00_JPRB, 0.43438E+00_JPRB, 0.43378E+00_JPRB, 0.43577E+00_JPRB, 0.44939E+00_JPRB, 0.49087E+00_JPRB, 0.57913E+00_JPRB ,&
&  0.70567E+00_JPRB, 0.82493E+00_JPRB, 0.88915E+00_JPRB, 0.92915E+00_JPRB, 0.95971E+00_JPRB, 0.92777E+00_JPRB, 0.82912E+00_JPRB ,&
&  0.73592E+00_JPRB, 0.65809E+00_JPRB, 0.60282E+00_JPRB, 0.56411E+00_JPRB, 0.53830E+00_JPRB, 0.52325E+00_JPRB, 0.51598E+00_JPRB ,&
&  0.51357E+00_JPRB, 0.51390E+00_JPRB, 0.51806E+00_JPRB, 0.52204E+00_JPRB, 0.52446E+00_JPRB, 0.52140E+00_JPRB, 0.51215E+00_JPRB ,&
&  0.49836E+00_JPRB, 0.48847E+00_JPRB, 0.47923E+00_JPRB, 0.46935E+00_JPRB, 0.46105E+00_JPRB, 0.45288E+00_JPRB, 0.44704E+00_JPRB ,&
&  0.44548E+00_JPRB, 0.45332E+00_JPRB, 0.46666E+00_JPRB, 0.49232E+00_JPRB, 0.48561E+00_JPRB, 0.47409E+00_JPRB, 0.46343E+00_JPRB ,&
&  0.45927E+00_JPRB, 0.46005E+00_JPRB, 0.45378E+00_JPRB, 0.45003E+00_JPRB, 0.45440E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.42676E+00_JPRB, 0.42611E+00_JPRB, 0.41996E+00_JPRB, 0.41271E+00_JPRB, 0.40917E+00_JPRB, 0.40805E+00_JPRB, 0.40794E+00_JPRB ,&
&  0.40650E+00_JPRB, 0.40359E+00_JPRB, 0.39981E+00_JPRB, 0.39735E+00_JPRB, 0.39390E+00_JPRB, 0.38876E+00_JPRB, 0.38244E+00_JPRB ,&
&  0.37654E+00_JPRB, 0.37097E+00_JPRB, 0.36674E+00_JPRB, 0.36204E+00_JPRB, 0.35838E+00_JPRB, 0.35859E+00_JPRB, 0.36031E+00_JPRB ,&
&  0.36597E+00_JPRB, 0.37352E+00_JPRB, 0.38002E+00_JPRB, 0.39354E+00_JPRB, 0.40866E+00_JPRB, 0.42950E+00_JPRB, 0.44914E+00_JPRB ,&
&  0.48439E+00_JPRB, 0.41558E+00_JPRB, 0.42951E+00_JPRB, 0.46694E+00_JPRB, 0.48581E+00_JPRB, 0.50165E+00_JPRB, 0.50166E+00_JPRB ,&
&  0.50118E+00_JPRB, 0.50243E+00_JPRB, 0.50015E+00_JPRB, 0.49328E+00_JPRB, 0.47838E+00_JPRB, 0.45179E+00_JPRB, 0.42587E+00_JPRB ,&
&  0.40742E+00_JPRB, 0.40102E+00_JPRB, 0.39835E+00_JPRB, 0.39458E+00_JPRB, 0.39085E+00_JPRB, 0.38953E+00_JPRB, 0.39063E+00_JPRB ,&
&  0.39123E+00_JPRB, 0.39125E+00_JPRB, 0.39059E+00_JPRB, 0.39000E+00_JPRB, 0.39242E+00_JPRB, 0.40558E+00_JPRB, 0.44236E+00_JPRB ,&
&  0.51413E+00_JPRB, 0.59571E+00_JPRB, 0.67007E+00_JPRB, 0.72860E+00_JPRB, 0.74224E+00_JPRB, 0.72201E+00_JPRB, 0.68046E+00_JPRB ,&
&  0.63116E+00_JPRB, 0.58578E+00_JPRB, 0.55079E+00_JPRB, 0.52345E+00_JPRB, 0.50173E+00_JPRB, 0.48485E+00_JPRB, 0.47287E+00_JPRB ,&
&  0.46640E+00_JPRB, 0.46210E+00_JPRB, 0.45897E+00_JPRB, 0.45443E+00_JPRB, 0.45033E+00_JPRB, 0.44743E+00_JPRB, 0.44628E+00_JPRB ,&
&  0.44389E+00_JPRB, 0.44082E+00_JPRB, 0.43842E+00_JPRB, 0.43596E+00_JPRB, 0.43296E+00_JPRB, 0.42960E+00_JPRB, 0.42709E+00_JPRB ,&
&  0.42765E+00_JPRB, 0.43039E+00_JPRB, 0.43659E+00_JPRB, 0.44268E+00_JPRB, 0.45176E+00_JPRB, 0.44443E+00_JPRB, 0.43886E+00_JPRB ,&
&  0.43703E+00_JPRB, 0.43455E+00_JPRB, 0.43390E+00_JPRB, 0.39915E+00_JPRB, 0.42242E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.39785E+00_JPRB, 0.39944E+00_JPRB, 0.39783E+00_JPRB, 0.39229E+00_JPRB, 0.38842E+00_JPRB, 0.38587E+00_JPRB, 0.38460E+00_JPRB ,&
&  0.38412E+00_JPRB, 0.38326E+00_JPRB, 0.38119E+00_JPRB, 0.37684E+00_JPRB, 0.37307E+00_JPRB, 0.36810E+00_JPRB, 0.36289E+00_JPRB ,&
&  0.35853E+00_JPRB, 0.35468E+00_JPRB, 0.35264E+00_JPRB, 0.34994E+00_JPRB, 0.34883E+00_JPRB, 0.35023E+00_JPRB, 0.35232E+00_JPRB ,&
&  0.35522E+00_JPRB, 0.35789E+00_JPRB, 0.36595E+00_JPRB, 0.37540E+00_JPRB, 0.39237E+00_JPRB, 0.40392E+00_JPRB, 0.41960E+00_JPRB ,&
&  0.45391E+00_JPRB, 0.38250E+00_JPRB, 0.40363E+00_JPRB, 0.43088E+00_JPRB, 0.44702E+00_JPRB, 0.46374E+00_JPRB, 0.45570E+00_JPRB ,&
&  0.44961E+00_JPRB, 0.44464E+00_JPRB, 0.43696E+00_JPRB, 0.42962E+00_JPRB, 0.42505E+00_JPRB, 0.41635E+00_JPRB, 0.40537E+00_JPRB ,&
&  0.39287E+00_JPRB, 0.38395E+00_JPRB, 0.37704E+00_JPRB, 0.37070E+00_JPRB, 0.36529E+00_JPRB, 0.36201E+00_JPRB, 0.36059E+00_JPRB ,&
&  0.36068E+00_JPRB, 0.36098E+00_JPRB, 0.36090E+00_JPRB, 0.36152E+00_JPRB, 0.36368E+00_JPRB, 0.36814E+00_JPRB, 0.37669E+00_JPRB ,&
&  0.39917E+00_JPRB, 0.43762E+00_JPRB, 0.48001E+00_JPRB, 0.52081E+00_JPRB, 0.55168E+00_JPRB, 0.56417E+00_JPRB, 0.56606E+00_JPRB ,&
&  0.55484E+00_JPRB, 0.53526E+00_JPRB, 0.51323E+00_JPRB, 0.49237E+00_JPRB, 0.47412E+00_JPRB, 0.45872E+00_JPRB, 0.44440E+00_JPRB ,&
&  0.43188E+00_JPRB, 0.42038E+00_JPRB, 0.41156E+00_JPRB, 0.40529E+00_JPRB, 0.39945E+00_JPRB, 0.39430E+00_JPRB, 0.39167E+00_JPRB ,&
&  0.39034E+00_JPRB, 0.38996E+00_JPRB, 0.38993E+00_JPRB, 0.39126E+00_JPRB, 0.39304E+00_JPRB, 0.39453E+00_JPRB, 0.39533E+00_JPRB ,&
&  0.39527E+00_JPRB, 0.39719E+00_JPRB, 0.39957E+00_JPRB, 0.41009E+00_JPRB, 0.41205E+00_JPRB, 0.41221E+00_JPRB, 0.41141E+00_JPRB ,&
&  0.41091E+00_JPRB, 0.40832E+00_JPRB, 0.36228E+00_JPRB, 0.38378E+00_JPRB, 0.39238E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.37810E+00_JPRB, 0.37689E+00_JPRB, 0.37694E+00_JPRB, 0.37617E+00_JPRB, 0.37517E+00_JPRB, 0.37366E+00_JPRB, 0.37133E+00_JPRB ,&
&  0.36795E+00_JPRB, 0.36571E+00_JPRB, 0.36354E+00_JPRB, 0.36067E+00_JPRB, 0.35755E+00_JPRB, 0.35483E+00_JPRB, 0.35210E+00_JPRB ,&
&  0.34928E+00_JPRB, 0.34634E+00_JPRB, 0.34453E+00_JPRB, 0.34339E+00_JPRB, 0.34197E+00_JPRB, 0.34160E+00_JPRB, 0.34282E+00_JPRB ,&
&  0.34314E+00_JPRB, 0.34561E+00_JPRB, 0.35484E+00_JPRB, 0.36509E+00_JPRB, 0.37675E+00_JPRB, 0.38668E+00_JPRB, 0.40013E+00_JPRB ,&
&  0.38543E+00_JPRB, 0.34584E+00_JPRB, 0.36588E+00_JPRB, 0.38767E+00_JPRB, 0.40975E+00_JPRB, 0.42864E+00_JPRB, 0.42347E+00_JPRB ,&
&  0.41516E+00_JPRB, 0.40365E+00_JPRB, 0.39250E+00_JPRB, 0.38360E+00_JPRB, 0.38042E+00_JPRB, 0.37792E+00_JPRB, 0.37494E+00_JPRB ,&
&  0.37126E+00_JPRB, 0.36553E+00_JPRB, 0.35994E+00_JPRB, 0.35503E+00_JPRB, 0.35179E+00_JPRB, 0.34916E+00_JPRB, 0.34683E+00_JPRB ,&
&  0.34604E+00_JPRB, 0.34620E+00_JPRB, 0.34644E+00_JPRB, 0.34685E+00_JPRB, 0.34788E+00_JPRB, 0.35093E+00_JPRB, 0.35482E+00_JPRB ,&
&  0.35953E+00_JPRB, 0.36925E+00_JPRB, 0.38783E+00_JPRB, 0.41075E+00_JPRB, 0.43422E+00_JPRB, 0.45191E+00_JPRB, 0.46290E+00_JPRB ,&
&  0.46922E+00_JPRB, 0.47002E+00_JPRB, 0.46467E+00_JPRB, 0.45342E+00_JPRB, 0.43915E+00_JPRB, 0.42541E+00_JPRB, 0.41251E+00_JPRB ,&
&  0.40006E+00_JPRB, 0.38681E+00_JPRB, 0.37450E+00_JPRB, 0.36703E+00_JPRB, 0.36318E+00_JPRB, 0.35967E+00_JPRB, 0.35694E+00_JPRB ,&
&  0.35582E+00_JPRB, 0.35578E+00_JPRB, 0.35649E+00_JPRB, 0.35751E+00_JPRB, 0.35890E+00_JPRB, 0.36125E+00_JPRB, 0.36365E+00_JPRB ,&
&  0.36577E+00_JPRB, 0.36984E+00_JPRB, 0.37578E+00_JPRB, 0.38688E+00_JPRB, 0.39428E+00_JPRB, 0.39771E+00_JPRB, 0.39741E+00_JPRB ,&
&  0.39336E+00_JPRB, 0.38816E+00_JPRB, 0.38288E+00_JPRB, 0.37838E+00_JPRB, 0.37773E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.36128E+00_JPRB, 0.36032E+00_JPRB, 0.35969E+00_JPRB, 0.35839E+00_JPRB, 0.35737E+00_JPRB, 0.35647E+00_JPRB, 0.35371E+00_JPRB ,&
&  0.35294E+00_JPRB, 0.34997E+00_JPRB, 0.34700E+00_JPRB, 0.34586E+00_JPRB, 0.34428E+00_JPRB, 0.34200E+00_JPRB, 0.34018E+00_JPRB ,&
&  0.33811E+00_JPRB, 0.33584E+00_JPRB, 0.33442E+00_JPRB, 0.33241E+00_JPRB, 0.32931E+00_JPRB, 0.32707E+00_JPRB, 0.32773E+00_JPRB ,&
&  0.32877E+00_JPRB, 0.33138E+00_JPRB, 0.33891E+00_JPRB, 0.34636E+00_JPRB, 0.35395E+00_JPRB, 0.36081E+00_JPRB, 0.36746E+00_JPRB ,&
&  0.37676E+00_JPRB, 0.33780E+00_JPRB, 0.35589E+00_JPRB, 0.36787E+00_JPRB, 0.38589E+00_JPRB, 0.40100E+00_JPRB, 0.39746E+00_JPRB ,&
&  0.38611E+00_JPRB, 0.37177E+00_JPRB, 0.35977E+00_JPRB, 0.35168E+00_JPRB, 0.35023E+00_JPRB, 0.35034E+00_JPRB, 0.34897E+00_JPRB ,&
&  0.34711E+00_JPRB, 0.34267E+00_JPRB, 0.33767E+00_JPRB, 0.33431E+00_JPRB, 0.33255E+00_JPRB, 0.33130E+00_JPRB, 0.32985E+00_JPRB ,&
&  0.32916E+00_JPRB, 0.32927E+00_JPRB, 0.32941E+00_JPRB, 0.32945E+00_JPRB, 0.33014E+00_JPRB, 0.33189E+00_JPRB, 0.33423E+00_JPRB ,&
&  0.33619E+00_JPRB, 0.33723E+00_JPRB, 0.34019E+00_JPRB, 0.34708E+00_JPRB, 0.35715E+00_JPRB, 0.36750E+00_JPRB, 0.37541E+00_JPRB ,&
&  0.38083E+00_JPRB, 0.38495E+00_JPRB, 0.38710E+00_JPRB, 0.38576E+00_JPRB, 0.38067E+00_JPRB, 0.37308E+00_JPRB, 0.36523E+00_JPRB ,&
&  0.35742E+00_JPRB, 0.34967E+00_JPRB, 0.34071E+00_JPRB, 0.33576E+00_JPRB, 0.33388E+00_JPRB, 0.33294E+00_JPRB, 0.33249E+00_JPRB ,&
&  0.33241E+00_JPRB, 0.33224E+00_JPRB, 0.33254E+00_JPRB, 0.33313E+00_JPRB, 0.33352E+00_JPRB, 0.33415E+00_JPRB, 0.33581E+00_JPRB ,&
&  0.33899E+00_JPRB, 0.34442E+00_JPRB, 0.35152E+00_JPRB, 0.36153E+00_JPRB, 0.37022E+00_JPRB, 0.37545E+00_JPRB, 0.37806E+00_JPRB ,&
&  0.37630E+00_JPRB, 0.37211E+00_JPRB, 0.36707E+00_JPRB, 0.36262E+00_JPRB, 0.36198E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.34902E+00_JPRB, 0.34637E+00_JPRB, 0.34456E+00_JPRB, 0.34137E+00_JPRB, 0.33918E+00_JPRB, 0.33766E+00_JPRB, 0.33577E+00_JPRB ,&
&  0.33440E+00_JPRB, 0.33483E+00_JPRB, 0.33374E+00_JPRB, 0.33313E+00_JPRB, 0.33225E+00_JPRB, 0.32966E+00_JPRB, 0.32647E+00_JPRB ,&
&  0.32475E+00_JPRB, 0.32315E+00_JPRB, 0.32173E+00_JPRB, 0.32012E+00_JPRB, 0.31861E+00_JPRB, 0.31644E+00_JPRB, 0.31669E+00_JPRB ,&
&  0.31801E+00_JPRB, 0.32075E+00_JPRB, 0.32372E+00_JPRB, 0.32721E+00_JPRB, 0.33218E+00_JPRB, 0.33870E+00_JPRB, 0.34480E+00_JPRB ,&
&  0.35102E+00_JPRB, 0.33973E+00_JPRB, 0.35064E+00_JPRB, 0.35676E+00_JPRB, 0.36723E+00_JPRB, 0.37503E+00_JPRB, 0.37413E+00_JPRB ,&
&  0.36929E+00_JPRB, 0.35838E+00_JPRB, 0.34775E+00_JPRB, 0.34031E+00_JPRB, 0.33773E+00_JPRB, 0.33624E+00_JPRB, 0.33259E+00_JPRB ,&
&  0.32819E+00_JPRB, 0.32268E+00_JPRB, 0.31830E+00_JPRB, 0.31589E+00_JPRB, 0.31519E+00_JPRB, 0.31532E+00_JPRB, 0.31432E+00_JPRB ,&
&  0.31325E+00_JPRB, 0.31206E+00_JPRB, 0.31153E+00_JPRB, 0.31129E+00_JPRB, 0.31096E+00_JPRB, 0.31125E+00_JPRB, 0.31217E+00_JPRB ,&
&  0.31320E+00_JPRB, 0.31396E+00_JPRB, 0.31449E+00_JPRB, 0.31589E+00_JPRB, 0.31853E+00_JPRB, 0.32187E+00_JPRB, 0.32508E+00_JPRB ,&
&  0.32693E+00_JPRB, 0.32865E+00_JPRB, 0.32995E+00_JPRB, 0.33095E+00_JPRB, 0.32976E+00_JPRB, 0.32623E+00_JPRB, 0.32182E+00_JPRB ,&
&  0.31827E+00_JPRB, 0.31547E+00_JPRB, 0.31300E+00_JPRB, 0.31113E+00_JPRB, 0.31029E+00_JPRB, 0.31059E+00_JPRB, 0.31132E+00_JPRB ,&
&  0.31216E+00_JPRB, 0.31306E+00_JPRB, 0.31379E+00_JPRB, 0.31468E+00_JPRB, 0.31499E+00_JPRB, 0.31557E+00_JPRB, 0.31681E+00_JPRB ,&
&  0.32019E+00_JPRB, 0.32451E+00_JPRB, 0.33066E+00_JPRB, 0.33987E+00_JPRB, 0.34812E+00_JPRB, 0.35372E+00_JPRB, 0.35807E+00_JPRB ,&
&  0.35937E+00_JPRB, 0.35791E+00_JPRB, 0.35551E+00_JPRB, 0.35390E+00_JPRB, 0.35217E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.32804E+00_JPRB, 0.32478E+00_JPRB, 0.32236E+00_JPRB, 0.32030E+00_JPRB, 0.32040E+00_JPRB, 0.31932E+00_JPRB, 0.31656E+00_JPRB ,&
&  0.31445E+00_JPRB, 0.31556E+00_JPRB, 0.31792E+00_JPRB, 0.31873E+00_JPRB, 0.31789E+00_JPRB, 0.31645E+00_JPRB, 0.31372E+00_JPRB ,&
&  0.31002E+00_JPRB, 0.30823E+00_JPRB, 0.30806E+00_JPRB, 0.30882E+00_JPRB, 0.30882E+00_JPRB, 0.30734E+00_JPRB, 0.30624E+00_JPRB ,&
&  0.30730E+00_JPRB, 0.30876E+00_JPRB, 0.30899E+00_JPRB, 0.31106E+00_JPRB, 0.31461E+00_JPRB, 0.32082E+00_JPRB, 0.32664E+00_JPRB ,&
&  0.32819E+00_JPRB, 0.32877E+00_JPRB, 0.32280E+00_JPRB, 0.33289E+00_JPRB, 0.33836E+00_JPRB, 0.34555E+00_JPRB, 0.34909E+00_JPRB ,&
&  0.34950E+00_JPRB, 0.34543E+00_JPRB, 0.33547E+00_JPRB, 0.32784E+00_JPRB, 0.32343E+00_JPRB, 0.31952E+00_JPRB, 0.31347E+00_JPRB ,&
&  0.30712E+00_JPRB, 0.30143E+00_JPRB, 0.29809E+00_JPRB, 0.29665E+00_JPRB, 0.29711E+00_JPRB, 0.29785E+00_JPRB, 0.29638E+00_JPRB ,&
&  0.29468E+00_JPRB, 0.29299E+00_JPRB, 0.29236E+00_JPRB, 0.29163E+00_JPRB, 0.29049E+00_JPRB, 0.28958E+00_JPRB, 0.28936E+00_JPRB ,&
&  0.28957E+00_JPRB, 0.28978E+00_JPRB, 0.29031E+00_JPRB, 0.29125E+00_JPRB, 0.29191E+00_JPRB, 0.29204E+00_JPRB, 0.29222E+00_JPRB ,&
&  0.29238E+00_JPRB, 0.29252E+00_JPRB, 0.29267E+00_JPRB, 0.29264E+00_JPRB, 0.29234E+00_JPRB, 0.29174E+00_JPRB, 0.29036E+00_JPRB ,&
&  0.28866E+00_JPRB, 0.28830E+00_JPRB, 0.28870E+00_JPRB, 0.28839E+00_JPRB, 0.28781E+00_JPRB, 0.28814E+00_JPRB, 0.28949E+00_JPRB ,&
&  0.29091E+00_JPRB, 0.29270E+00_JPRB, 0.29412E+00_JPRB, 0.29510E+00_JPRB, 0.29511E+00_JPRB, 0.29566E+00_JPRB, 0.29661E+00_JPRB ,&
&  0.29842E+00_JPRB, 0.30128E+00_JPRB, 0.30544E+00_JPRB, 0.31090E+00_JPRB, 0.31657E+00_JPRB, 0.32081E+00_JPRB, 0.32510E+00_JPRB ,&
&  0.32880E+00_JPRB, 0.33040E+00_JPRB, 0.33057E+00_JPRB, 0.33127E+00_JPRB, 0.33073E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.30479E+00_JPRB, 0.30216E+00_JPRB, 0.30021E+00_JPRB, 0.29990E+00_JPRB, 0.30153E+00_JPRB, 0.30157E+00_JPRB, 0.29966E+00_JPRB ,&
&  0.29780E+00_JPRB, 0.29882E+00_JPRB, 0.30167E+00_JPRB, 0.30313E+00_JPRB, 0.30364E+00_JPRB, 0.30435E+00_JPRB, 0.30332E+00_JPRB ,&
&  0.30020E+00_JPRB, 0.29705E+00_JPRB, 0.29762E+00_JPRB, 0.29912E+00_JPRB, 0.30029E+00_JPRB, 0.29908E+00_JPRB, 0.29760E+00_JPRB ,&
&  0.29825E+00_JPRB, 0.29835E+00_JPRB, 0.29998E+00_JPRB, 0.30179E+00_JPRB, 0.30509E+00_JPRB, 0.31011E+00_JPRB, 0.31366E+00_JPRB ,&
&  0.31422E+00_JPRB, 0.31235E+00_JPRB, 0.31172E+00_JPRB, 0.31386E+00_JPRB, 0.31623E+00_JPRB, 0.32189E+00_JPRB, 0.32732E+00_JPRB ,&
&  0.32873E+00_JPRB, 0.32525E+00_JPRB, 0.31533E+00_JPRB, 0.30736E+00_JPRB, 0.30281E+00_JPRB, 0.29835E+00_JPRB, 0.29402E+00_JPRB ,&
&  0.28923E+00_JPRB, 0.28566E+00_JPRB, 0.28359E+00_JPRB, 0.28275E+00_JPRB, 0.28258E+00_JPRB, 0.28211E+00_JPRB, 0.28009E+00_JPRB ,&
&  0.27744E+00_JPRB, 0.27541E+00_JPRB, 0.27382E+00_JPRB, 0.27258E+00_JPRB, 0.27141E+00_JPRB, 0.27089E+00_JPRB, 0.27028E+00_JPRB ,&
&  0.27000E+00_JPRB, 0.27043E+00_JPRB, 0.27127E+00_JPRB, 0.27135E+00_JPRB, 0.27127E+00_JPRB, 0.27069E+00_JPRB, 0.27029E+00_JPRB ,&
&  0.26996E+00_JPRB, 0.26853E+00_JPRB, 0.26735E+00_JPRB, 0.26755E+00_JPRB, 0.26784E+00_JPRB, 0.26855E+00_JPRB, 0.26906E+00_JPRB ,&
&  0.26900E+00_JPRB, 0.26918E+00_JPRB, 0.26983E+00_JPRB, 0.27043E+00_JPRB, 0.27076E+00_JPRB, 0.27088E+00_JPRB, 0.27125E+00_JPRB ,&
&  0.27265E+00_JPRB, 0.27435E+00_JPRB, 0.27557E+00_JPRB, 0.27660E+00_JPRB, 0.27755E+00_JPRB, 0.27923E+00_JPRB, 0.27975E+00_JPRB ,&
&  0.28015E+00_JPRB, 0.28139E+00_JPRB, 0.28423E+00_JPRB, 0.28819E+00_JPRB, 0.29195E+00_JPRB, 0.29454E+00_JPRB, 0.29728E+00_JPRB ,&
&  0.29974E+00_JPRB, 0.30181E+00_JPRB, 0.30310E+00_JPRB, 0.30376E+00_JPRB, 0.30465E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.28235E+00_JPRB, 0.28139E+00_JPRB, 0.28104E+00_JPRB, 0.28143E+00_JPRB, 0.28314E+00_JPRB, 0.28323E+00_JPRB, 0.28274E+00_JPRB ,&
&  0.28213E+00_JPRB, 0.28374E+00_JPRB, 0.28713E+00_JPRB, 0.28924E+00_JPRB, 0.28955E+00_JPRB, 0.28948E+00_JPRB, 0.28922E+00_JPRB ,&
&  0.28920E+00_JPRB, 0.28921E+00_JPRB, 0.28842E+00_JPRB, 0.28882E+00_JPRB, 0.28904E+00_JPRB, 0.28831E+00_JPRB, 0.28932E+00_JPRB ,&
&  0.28955E+00_JPRB, 0.29007E+00_JPRB, 0.29225E+00_JPRB, 0.29393E+00_JPRB, 0.29746E+00_JPRB, 0.30281E+00_JPRB, 0.30504E+00_JPRB ,&
&  0.30396E+00_JPRB, 0.29992E+00_JPRB, 0.29661E+00_JPRB, 0.29716E+00_JPRB, 0.29876E+00_JPRB, 0.30198E+00_JPRB, 0.30528E+00_JPRB ,&
&  0.30559E+00_JPRB, 0.29932E+00_JPRB, 0.29021E+00_JPRB, 0.28418E+00_JPRB, 0.28066E+00_JPRB, 0.27840E+00_JPRB, 0.27649E+00_JPRB ,&
&  0.27404E+00_JPRB, 0.27229E+00_JPRB, 0.27164E+00_JPRB, 0.27175E+00_JPRB, 0.27160E+00_JPRB, 0.26901E+00_JPRB, 0.26604E+00_JPRB ,&
&  0.26213E+00_JPRB, 0.25940E+00_JPRB, 0.25712E+00_JPRB, 0.25635E+00_JPRB, 0.25649E+00_JPRB, 0.25629E+00_JPRB, 0.25533E+00_JPRB ,&
&  0.25408E+00_JPRB, 0.25368E+00_JPRB, 0.25332E+00_JPRB, 0.25219E+00_JPRB, 0.25079E+00_JPRB, 0.25043E+00_JPRB, 0.24953E+00_JPRB ,&
&  0.24773E+00_JPRB, 0.24693E+00_JPRB, 0.24734E+00_JPRB, 0.24865E+00_JPRB, 0.24891E+00_JPRB, 0.25005E+00_JPRB, 0.25192E+00_JPRB ,&
&  0.25144E+00_JPRB, 0.24920E+00_JPRB, 0.24789E+00_JPRB, 0.24820E+00_JPRB, 0.24981E+00_JPRB, 0.25203E+00_JPRB, 0.25305E+00_JPRB ,&
&  0.25396E+00_JPRB, 0.25387E+00_JPRB, 0.25469E+00_JPRB, 0.25597E+00_JPRB, 0.25769E+00_JPRB, 0.25903E+00_JPRB, 0.25976E+00_JPRB ,&
&  0.26035E+00_JPRB, 0.26090E+00_JPRB, 0.26292E+00_JPRB, 0.26603E+00_JPRB, 0.26871E+00_JPRB, 0.27175E+00_JPRB, 0.27409E+00_JPRB ,&
&  0.27569E+00_JPRB, 0.27741E+00_JPRB, 0.27858E+00_JPRB, 0.27952E+00_JPRB, 0.28090E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.26795E+00_JPRB, 0.26862E+00_JPRB, 0.26973E+00_JPRB, 0.27001E+00_JPRB, 0.27082E+00_JPRB, 0.27129E+00_JPRB, 0.27234E+00_JPRB ,&
&  0.27267E+00_JPRB, 0.27394E+00_JPRB, 0.27592E+00_JPRB, 0.27589E+00_JPRB, 0.27560E+00_JPRB, 0.27553E+00_JPRB, 0.27631E+00_JPRB ,&
&  0.27873E+00_JPRB, 0.28019E+00_JPRB, 0.27999E+00_JPRB, 0.28007E+00_JPRB, 0.27950E+00_JPRB, 0.27896E+00_JPRB, 0.28105E+00_JPRB ,&
&  0.28164E+00_JPRB, 0.28246E+00_JPRB, 0.28431E+00_JPRB, 0.28580E+00_JPRB, 0.28931E+00_JPRB, 0.29347E+00_JPRB, 0.29605E+00_JPRB ,&
&  0.29555E+00_JPRB, 0.29248E+00_JPRB, 0.28757E+00_JPRB, 0.28371E+00_JPRB, 0.28300E+00_JPRB, 0.28388E+00_JPRB, 0.28603E+00_JPRB ,&
&  0.28507E+00_JPRB, 0.27945E+00_JPRB, 0.27234E+00_JPRB, 0.26848E+00_JPRB, 0.26708E+00_JPRB, 0.26529E+00_JPRB, 0.26232E+00_JPRB ,&
&  0.25998E+00_JPRB, 0.25950E+00_JPRB, 0.25976E+00_JPRB, 0.25912E+00_JPRB, 0.25761E+00_JPRB, 0.25491E+00_JPRB, 0.25141E+00_JPRB ,&
&  0.24836E+00_JPRB, 0.24529E+00_JPRB, 0.24336E+00_JPRB, 0.24201E+00_JPRB, 0.24192E+00_JPRB, 0.24216E+00_JPRB, 0.24138E+00_JPRB ,&
&  0.23955E+00_JPRB, 0.23836E+00_JPRB, 0.23678E+00_JPRB, 0.23373E+00_JPRB, 0.23157E+00_JPRB, 0.23094E+00_JPRB, 0.22998E+00_JPRB ,&
&  0.22864E+00_JPRB, 0.22829E+00_JPRB, 0.22823E+00_JPRB, 0.22795E+00_JPRB, 0.22903E+00_JPRB, 0.23096E+00_JPRB, 0.22870E+00_JPRB ,&
&  0.22673E+00_JPRB, 0.22542E+00_JPRB, 0.22547E+00_JPRB, 0.22689E+00_JPRB, 0.22930E+00_JPRB, 0.23199E+00_JPRB, 0.23424E+00_JPRB ,&
&  0.23519E+00_JPRB, 0.23544E+00_JPRB, 0.23574E+00_JPRB, 0.23586E+00_JPRB, 0.23697E+00_JPRB, 0.23933E+00_JPRB, 0.24209E+00_JPRB ,&
&  0.24342E+00_JPRB, 0.24585E+00_JPRB, 0.24847E+00_JPRB, 0.25175E+00_JPRB, 0.25365E+00_JPRB, 0.25542E+00_JPRB, 0.25709E+00_JPRB ,&
&  0.25839E+00_JPRB, 0.26007E+00_JPRB, 0.26235E+00_JPRB, 0.26425E+00_JPRB, 0.26627E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.25720E+00_JPRB, 0.25869E+00_JPRB, 0.25948E+00_JPRB, 0.26066E+00_JPRB, 0.26281E+00_JPRB, 0.26413E+00_JPRB, 0.26415E+00_JPRB ,&
&  0.26416E+00_JPRB, 0.26525E+00_JPRB, 0.26606E+00_JPRB, 0.26643E+00_JPRB, 0.26417E+00_JPRB, 0.26396E+00_JPRB, 0.26573E+00_JPRB ,&
&  0.26805E+00_JPRB, 0.27044E+00_JPRB, 0.27083E+00_JPRB, 0.27098E+00_JPRB, 0.27060E+00_JPRB, 0.26965E+00_JPRB, 0.27127E+00_JPRB ,&
&  0.27267E+00_JPRB, 0.27333E+00_JPRB, 0.27470E+00_JPRB, 0.27600E+00_JPRB, 0.27879E+00_JPRB, 0.28203E+00_JPRB, 0.28377E+00_JPRB ,&
&  0.28358E+00_JPRB, 0.28183E+00_JPRB, 0.27542E+00_JPRB, 0.26879E+00_JPRB, 0.26917E+00_JPRB, 0.27025E+00_JPRB, 0.27016E+00_JPRB ,&
&  0.26806E+00_JPRB, 0.26528E+00_JPRB, 0.26338E+00_JPRB, 0.26188E+00_JPRB, 0.26012E+00_JPRB, 0.25707E+00_JPRB, 0.25375E+00_JPRB ,&
&  0.25090E+00_JPRB, 0.24997E+00_JPRB, 0.24869E+00_JPRB, 0.24651E+00_JPRB, 0.24327E+00_JPRB, 0.24066E+00_JPRB, 0.23798E+00_JPRB ,&
&  0.23318E+00_JPRB, 0.22847E+00_JPRB, 0.22595E+00_JPRB, 0.22541E+00_JPRB, 0.22588E+00_JPRB, 0.22552E+00_JPRB, 0.22389E+00_JPRB ,&
&  0.22196E+00_JPRB, 0.21951E+00_JPRB, 0.21803E+00_JPRB, 0.21764E+00_JPRB, 0.21880E+00_JPRB, 0.21780E+00_JPRB, 0.21371E+00_JPRB ,&
&  0.21143E+00_JPRB, 0.21000E+00_JPRB, 0.21021E+00_JPRB, 0.20941E+00_JPRB, 0.20870E+00_JPRB, 0.20758E+00_JPRB, 0.20796E+00_JPRB ,&
&  0.20897E+00_JPRB, 0.20929E+00_JPRB, 0.21178E+00_JPRB, 0.21613E+00_JPRB, 0.22009E+00_JPRB, 0.22240E+00_JPRB, 0.22382E+00_JPRB ,&
&  0.22438E+00_JPRB, 0.22382E+00_JPRB, 0.22322E+00_JPRB, 0.22367E+00_JPRB, 0.22562E+00_JPRB, 0.22834E+00_JPRB, 0.23086E+00_JPRB ,&
&  0.23143E+00_JPRB, 0.23245E+00_JPRB, 0.23493E+00_JPRB, 0.23870E+00_JPRB, 0.24093E+00_JPRB, 0.24280E+00_JPRB, 0.24438E+00_JPRB ,&
&  0.24514E+00_JPRB, 0.24653E+00_JPRB, 0.24997E+00_JPRB, 0.25211E+00_JPRB, 0.25441E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.24584E+00_JPRB, 0.24685E+00_JPRB, 0.24873E+00_JPRB, 0.25079E+00_JPRB, 0.25158E+00_JPRB, 0.25098E+00_JPRB, 0.25017E+00_JPRB ,&
&  0.25044E+00_JPRB, 0.25233E+00_JPRB, 0.25434E+00_JPRB, 0.25478E+00_JPRB, 0.25503E+00_JPRB, 0.25543E+00_JPRB, 0.25720E+00_JPRB ,&
&  0.25985E+00_JPRB, 0.26294E+00_JPRB, 0.26305E+00_JPRB, 0.26289E+00_JPRB, 0.26269E+00_JPRB, 0.26188E+00_JPRB, 0.26315E+00_JPRB ,&
&  0.26623E+00_JPRB, 0.26654E+00_JPRB, 0.26644E+00_JPRB, 0.26571E+00_JPRB, 0.26721E+00_JPRB, 0.27015E+00_JPRB, 0.27172E+00_JPRB ,&
&  0.27008E+00_JPRB, 0.26324E+00_JPRB, 0.25518E+00_JPRB, 0.25472E+00_JPRB, 0.25636E+00_JPRB, 0.25804E+00_JPRB, 0.25824E+00_JPRB ,&
&  0.25778E+00_JPRB, 0.25695E+00_JPRB, 0.25592E+00_JPRB, 0.25493E+00_JPRB, 0.25381E+00_JPRB, 0.25035E+00_JPRB, 0.24631E+00_JPRB ,&
&  0.24310E+00_JPRB, 0.24003E+00_JPRB, 0.23696E+00_JPRB, 0.23279E+00_JPRB, 0.22852E+00_JPRB, 0.22545E+00_JPRB, 0.22199E+00_JPRB ,&
&  0.21769E+00_JPRB, 0.21460E+00_JPRB, 0.21369E+00_JPRB, 0.21374E+00_JPRB, 0.21396E+00_JPRB, 0.21326E+00_JPRB, 0.21237E+00_JPRB ,&
&  0.21078E+00_JPRB, 0.20955E+00_JPRB, 0.20812E+00_JPRB, 0.20742E+00_JPRB, 0.20274E+00_JPRB, 0.19451E+00_JPRB, 0.18880E+00_JPRB ,&
&  0.18806E+00_JPRB, 0.19087E+00_JPRB, 0.19489E+00_JPRB, 0.19568E+00_JPRB, 0.19747E+00_JPRB, 0.19849E+00_JPRB, 0.19818E+00_JPRB ,&
&  0.19582E+00_JPRB, 0.19196E+00_JPRB, 0.19186E+00_JPRB, 0.19278E+00_JPRB, 0.19312E+00_JPRB, 0.19414E+00_JPRB, 0.19558E+00_JPRB ,&
&  0.19835E+00_JPRB, 0.20078E+00_JPRB, 0.20300E+00_JPRB, 0.20524E+00_JPRB, 0.20666E+00_JPRB, 0.20635E+00_JPRB, 0.20422E+00_JPRB ,&
&  0.20396E+00_JPRB, 0.20917E+00_JPRB, 0.21863E+00_JPRB, 0.22918E+00_JPRB, 0.23157E+00_JPRB, 0.23377E+00_JPRB, 0.23579E+00_JPRB ,&
&  0.23676E+00_JPRB, 0.23978E+00_JPRB, 0.24233E+00_JPRB, 0.24368E+00_JPRB, 0.24467E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.23882E+00_JPRB, 0.23845E+00_JPRB, 0.23704E+00_JPRB, 0.23579E+00_JPRB, 0.23419E+00_JPRB, 0.23267E+00_JPRB, 0.23323E+00_JPRB ,&
&  0.23556E+00_JPRB, 0.23864E+00_JPRB, 0.24162E+00_JPRB, 0.24391E+00_JPRB, 0.24573E+00_JPRB, 0.24666E+00_JPRB, 0.24811E+00_JPRB ,&
&  0.24983E+00_JPRB, 0.25276E+00_JPRB, 0.25377E+00_JPRB, 0.25468E+00_JPRB, 0.25414E+00_JPRB, 0.25360E+00_JPRB, 0.25478E+00_JPRB ,&
&  0.25803E+00_JPRB, 0.25961E+00_JPRB, 0.25884E+00_JPRB, 0.25683E+00_JPRB, 0.25731E+00_JPRB, 0.25908E+00_JPRB, 0.25829E+00_JPRB ,&
&  0.25188E+00_JPRB, 0.23873E+00_JPRB, 0.22951E+00_JPRB, 0.23196E+00_JPRB, 0.24050E+00_JPRB, 0.24439E+00_JPRB, 0.24701E+00_JPRB ,&
&  0.24867E+00_JPRB, 0.24893E+00_JPRB, 0.24819E+00_JPRB, 0.24683E+00_JPRB, 0.24441E+00_JPRB, 0.24017E+00_JPRB, 0.23507E+00_JPRB ,&
&  0.23043E+00_JPRB, 0.22803E+00_JPRB, 0.22523E+00_JPRB, 0.22027E+00_JPRB, 0.21533E+00_JPRB, 0.21220E+00_JPRB, 0.20862E+00_JPRB ,&
&  0.20499E+00_JPRB, 0.20218E+00_JPRB, 0.20154E+00_JPRB, 0.20123E+00_JPRB, 0.20122E+00_JPRB, 0.20077E+00_JPRB, 0.19874E+00_JPRB ,&
&  0.19531E+00_JPRB, 0.19116E+00_JPRB, 0.18558E+00_JPRB, 0.18057E+00_JPRB, 0.17699E+00_JPRB, 0.17353E+00_JPRB, 0.16946E+00_JPRB ,&
&  0.16742E+00_JPRB, 0.16971E+00_JPRB, 0.17792E+00_JPRB, 0.18726E+00_JPRB, 0.19318E+00_JPRB, 0.18163E+00_JPRB, 0.17731E+00_JPRB ,&
&  0.17621E+00_JPRB, 0.17586E+00_JPRB, 0.17612E+00_JPRB, 0.17740E+00_JPRB, 0.17803E+00_JPRB, 0.17782E+00_JPRB, 0.17838E+00_JPRB ,&
&  0.18027E+00_JPRB, 0.18223E+00_JPRB, 0.18328E+00_JPRB, 0.18440E+00_JPRB, 0.18507E+00_JPRB, 0.18484E+00_JPRB, 0.18446E+00_JPRB ,&
&  0.18504E+00_JPRB, 0.18786E+00_JPRB, 0.19257E+00_JPRB, 0.19925E+00_JPRB, 0.20419E+00_JPRB, 0.20870E+00_JPRB, 0.21452E+00_JPRB ,&
&  0.22190E+00_JPRB, 0.22929E+00_JPRB, 0.23314E+00_JPRB, 0.23651E+00_JPRB, 0.23806E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.23238E+00_JPRB, 0.23268E+00_JPRB, 0.22891E+00_JPRB, 0.22554E+00_JPRB, 0.22392E+00_JPRB, 0.22344E+00_JPRB, 0.22465E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22968E+00_JPRB, 0.23186E+00_JPRB, 0.23376E+00_JPRB, 0.23457E+00_JPRB, 0.23451E+00_JPRB, 0.23550E+00_JPRB ,&
&  0.23649E+00_JPRB, 0.23899E+00_JPRB, 0.24040E+00_JPRB, 0.24292E+00_JPRB, 0.24323E+00_JPRB, 0.24403E+00_JPRB, 0.24526E+00_JPRB ,&
&  0.24488E+00_JPRB, 0.24550E+00_JPRB, 0.24699E+00_JPRB, 0.24487E+00_JPRB, 0.24343E+00_JPRB, 0.24123E+00_JPRB, 0.23937E+00_JPRB ,&
&  0.23315E+00_JPRB, 0.22666E+00_JPRB, 0.21998E+00_JPRB, 0.22315E+00_JPRB, 0.23101E+00_JPRB, 0.23287E+00_JPRB, 0.23656E+00_JPRB ,&
&  0.23754E+00_JPRB, 0.23596E+00_JPRB, 0.23430E+00_JPRB, 0.23124E+00_JPRB, 0.22826E+00_JPRB, 0.22494E+00_JPRB, 0.22232E+00_JPRB ,&
&  0.21943E+00_JPRB, 0.22000E+00_JPRB, 0.21630E+00_JPRB, 0.20327E+00_JPRB, 0.19633E+00_JPRB, 0.19203E+00_JPRB, 0.18539E+00_JPRB ,&
&  0.17882E+00_JPRB, 0.17686E+00_JPRB, 0.17641E+00_JPRB, 0.17431E+00_JPRB, 0.17430E+00_JPRB, 0.17462E+00_JPRB, 0.17457E+00_JPRB ,&
&  0.17335E+00_JPRB, 0.17311E+00_JPRB, 0.17181E+00_JPRB, 0.16990E+00_JPRB, 0.16810E+00_JPRB, 0.16737E+00_JPRB, 0.16774E+00_JPRB ,&
&  0.16919E+00_JPRB, 0.17084E+00_JPRB, 0.17630E+00_JPRB, 0.17985E+00_JPRB, 0.17556E+00_JPRB, 0.17273E+00_JPRB, 0.17274E+00_JPRB ,&
&  0.16903E+00_JPRB, 0.16828E+00_JPRB, 0.16825E+00_JPRB, 0.16823E+00_JPRB, 0.16861E+00_JPRB, 0.16951E+00_JPRB, 0.16951E+00_JPRB ,&
&  0.17017E+00_JPRB, 0.17139E+00_JPRB, 0.17170E+00_JPRB, 0.17170E+00_JPRB, 0.17144E+00_JPRB, 0.17184E+00_JPRB, 0.17298E+00_JPRB ,&
&  0.17435E+00_JPRB, 0.17688E+00_JPRB, 0.17954E+00_JPRB, 0.18342E+00_JPRB, 0.18654E+00_JPRB, 0.18900E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19732E+00_JPRB, 0.20620E+00_JPRB, 0.22130E+00_JPRB, 0.23178E+00_JPRB, 0.22858E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.22337E+00_JPRB, 0.22205E+00_JPRB, 0.22077E+00_JPRB, 0.21944E+00_JPRB, 0.22023E+00_JPRB, 0.22134E+00_JPRB, 0.22282E+00_JPRB ,&
&  0.22449E+00_JPRB, 0.22493E+00_JPRB, 0.22470E+00_JPRB, 0.22262E+00_JPRB, 0.22139E+00_JPRB, 0.21523E+00_JPRB, 0.21298E+00_JPRB ,&
&  0.21134E+00_JPRB, 0.21314E+00_JPRB, 0.21566E+00_JPRB, 0.21947E+00_JPRB, 0.22117E+00_JPRB, 0.22293E+00_JPRB, 0.22208E+00_JPRB ,&
&  0.21998E+00_JPRB, 0.21740E+00_JPRB, 0.21496E+00_JPRB, 0.21302E+00_JPRB, 0.21246E+00_JPRB, 0.21090E+00_JPRB, 0.20922E+00_JPRB ,&
&  0.20912E+00_JPRB, 0.21045E+00_JPRB, 0.21121E+00_JPRB, 0.21198E+00_JPRB, 0.21789E+00_JPRB, 0.21853E+00_JPRB, 0.22052E+00_JPRB ,&
&  0.22104E+00_JPRB, 0.22160E+00_JPRB, 0.22093E+00_JPRB, 0.21960E+00_JPRB, 0.21840E+00_JPRB, 0.21335E+00_JPRB, 0.21112E+00_JPRB ,&
&  0.20392E+00_JPRB, 0.19967E+00_JPRB, 0.19301E+00_JPRB, 0.18706E+00_JPRB, 0.18067E+00_JPRB, 0.17590E+00_JPRB, 0.17189E+00_JPRB ,&
&  0.16871E+00_JPRB, 0.16636E+00_JPRB, 0.16441E+00_JPRB, 0.16415E+00_JPRB, 0.16424E+00_JPRB, 0.16391E+00_JPRB, 0.16406E+00_JPRB ,&
&  0.16484E+00_JPRB, 0.16569E+00_JPRB, 0.16563E+00_JPRB, 0.16619E+00_JPRB, 0.16671E+00_JPRB, 0.16747E+00_JPRB, 0.16890E+00_JPRB ,&
&  0.17111E+00_JPRB, 0.17306E+00_JPRB, 0.17387E+00_JPRB, 0.17561E+00_JPRB, 0.17495E+00_JPRB, 0.17238E+00_JPRB, 0.16982E+00_JPRB ,&
&  0.16701E+00_JPRB, 0.16578E+00_JPRB, 0.16446E+00_JPRB, 0.16373E+00_JPRB, 0.16365E+00_JPRB, 0.16387E+00_JPRB, 0.16465E+00_JPRB ,&
&  0.16547E+00_JPRB, 0.16631E+00_JPRB, 0.16664E+00_JPRB, 0.16708E+00_JPRB, 0.16747E+00_JPRB, 0.16828E+00_JPRB, 0.16980E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17426E+00_JPRB, 0.17659E+00_JPRB, 0.17944E+00_JPRB, 0.18415E+00_JPRB, 0.19050E+00_JPRB, 0.20016E+00_JPRB ,&
&  0.20794E+00_JPRB, 0.22073E+00_JPRB, 0.22112E+00_JPRB, 0.22258E+00_JPRB, 0.22374E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.21678E+00_JPRB, 0.21663E+00_JPRB, 0.21590E+00_JPRB, 0.21538E+00_JPRB, 0.21581E+00_JPRB, 0.21658E+00_JPRB, 0.21756E+00_JPRB ,&
&  0.21416E+00_JPRB, 0.21154E+00_JPRB, 0.20720E+00_JPRB, 0.20242E+00_JPRB, 0.19824E+00_JPRB, 0.19488E+00_JPRB, 0.19292E+00_JPRB ,&
&  0.19270E+00_JPRB, 0.19471E+00_JPRB, 0.19634E+00_JPRB, 0.19827E+00_JPRB, 0.19844E+00_JPRB, 0.19863E+00_JPRB, 0.19732E+00_JPRB ,&
&  0.19588E+00_JPRB, 0.19531E+00_JPRB, 0.19556E+00_JPRB, 0.19729E+00_JPRB, 0.19997E+00_JPRB, 0.20179E+00_JPRB, 0.20394E+00_JPRB ,&
&  0.20680E+00_JPRB, 0.20616E+00_JPRB, 0.20691E+00_JPRB, 0.20636E+00_JPRB, 0.20662E+00_JPRB, 0.20747E+00_JPRB, 0.20787E+00_JPRB ,&
&  0.20872E+00_JPRB, 0.20956E+00_JPRB, 0.20946E+00_JPRB, 0.20880E+00_JPRB, 0.20469E+00_JPRB, 0.20204E+00_JPRB, 0.19743E+00_JPRB ,&
&  0.19196E+00_JPRB, 0.18710E+00_JPRB, 0.18162E+00_JPRB, 0.17779E+00_JPRB, 0.17336E+00_JPRB, 0.17073E+00_JPRB, 0.16870E+00_JPRB ,&
&  0.16678E+00_JPRB, 0.16604E+00_JPRB, 0.16500E+00_JPRB, 0.16390E+00_JPRB, 0.16366E+00_JPRB, 0.16257E+00_JPRB, 0.16249E+00_JPRB ,&
&  0.16274E+00_JPRB, 0.16298E+00_JPRB, 0.16390E+00_JPRB, 0.16436E+00_JPRB, 0.16514E+00_JPRB, 0.16589E+00_JPRB, 0.16648E+00_JPRB ,&
&  0.16762E+00_JPRB, 0.16903E+00_JPRB, 0.16907E+00_JPRB, 0.16936E+00_JPRB, 0.16855E+00_JPRB, 0.16683E+00_JPRB, 0.16496E+00_JPRB ,&
&  0.16297E+00_JPRB, 0.16143E+00_JPRB, 0.16094E+00_JPRB, 0.16027E+00_JPRB, 0.16041E+00_JPRB, 0.16079E+00_JPRB, 0.16099E+00_JPRB ,&
&  0.16164E+00_JPRB, 0.16222E+00_JPRB, 0.16303E+00_JPRB, 0.16480E+00_JPRB, 0.16606E+00_JPRB, 0.16766E+00_JPRB, 0.16935E+00_JPRB ,&
&  0.17070E+00_JPRB, 0.17313E+00_JPRB, 0.17528E+00_JPRB, 0.17822E+00_JPRB, 0.18494E+00_JPRB, 0.19092E+00_JPRB, 0.20160E+00_JPRB ,&
&  0.20683E+00_JPRB, 0.21871E+00_JPRB, 0.21689E+00_JPRB, 0.21630E+00_JPRB, 0.21721E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.21173E+00_JPRB, 0.21255E+00_JPRB, 0.21253E+00_JPRB, 0.21347E+00_JPRB, 0.21428E+00_JPRB, 0.21531E+00_JPRB, 0.21549E+00_JPRB ,&
&  0.21490E+00_JPRB, 0.21066E+00_JPRB, 0.20829E+00_JPRB, 0.20499E+00_JPRB, 0.20184E+00_JPRB, 0.19855E+00_JPRB, 0.19585E+00_JPRB ,&
&  0.19410E+00_JPRB, 0.19276E+00_JPRB, 0.19155E+00_JPRB, 0.18986E+00_JPRB, 0.18780E+00_JPRB, 0.18573E+00_JPRB, 0.18384E+00_JPRB ,&
&  0.18214E+00_JPRB, 0.18219E+00_JPRB, 0.18338E+00_JPRB, 0.18553E+00_JPRB, 0.18946E+00_JPRB, 0.19254E+00_JPRB, 0.19649E+00_JPRB ,&
&  0.20271E+00_JPRB, 0.19974E+00_JPRB, 0.19990E+00_JPRB, 0.20014E+00_JPRB, 0.19987E+00_JPRB, 0.19968E+00_JPRB, 0.19673E+00_JPRB ,&
&  0.20010E+00_JPRB, 0.20033E+00_JPRB, 0.20377E+00_JPRB, 0.20300E+00_JPRB, 0.19856E+00_JPRB, 0.19493E+00_JPRB, 0.19000E+00_JPRB ,&
&  0.18638E+00_JPRB, 0.18194E+00_JPRB, 0.17895E+00_JPRB, 0.17627E+00_JPRB, 0.17318E+00_JPRB, 0.17190E+00_JPRB, 0.16977E+00_JPRB ,&
&  0.16846E+00_JPRB, 0.16817E+00_JPRB, 0.16723E+00_JPRB, 0.16718E+00_JPRB, 0.16639E+00_JPRB, 0.16527E+00_JPRB, 0.16452E+00_JPRB ,&
&  0.16387E+00_JPRB, 0.16318E+00_JPRB, 0.16294E+00_JPRB, 0.16270E+00_JPRB, 0.16284E+00_JPRB, 0.16287E+00_JPRB, 0.16296E+00_JPRB ,&
&  0.16319E+00_JPRB, 0.16336E+00_JPRB, 0.16298E+00_JPRB, 0.16284E+00_JPRB, 0.16198E+00_JPRB, 0.16123E+00_JPRB, 0.16005E+00_JPRB ,&
&  0.15934E+00_JPRB, 0.15892E+00_JPRB, 0.15828E+00_JPRB, 0.15786E+00_JPRB, 0.15775E+00_JPRB, 0.15823E+00_JPRB, 0.15883E+00_JPRB ,&
&  0.15953E+00_JPRB, 0.16078E+00_JPRB, 0.16182E+00_JPRB, 0.16320E+00_JPRB, 0.16474E+00_JPRB, 0.16597E+00_JPRB, 0.16772E+00_JPRB ,&
&  0.16946E+00_JPRB, 0.17124E+00_JPRB, 0.17424E+00_JPRB, 0.17759E+00_JPRB, 0.18203E+00_JPRB, 0.18705E+00_JPRB, 0.18989E+00_JPRB ,&
&  0.19311E+00_JPRB, 0.20012E+00_JPRB, 0.20272E+00_JPRB, 0.20983E+00_JPRB, 0.21093E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.20275E+00_JPRB, 0.20422E+00_JPRB, 0.20566E+00_JPRB, 0.20701E+00_JPRB, 0.20852E+00_JPRB, 0.20911E+00_JPRB, 0.20903E+00_JPRB ,&
&  0.20670E+00_JPRB, 0.20611E+00_JPRB, 0.20457E+00_JPRB, 0.20267E+00_JPRB, 0.20114E+00_JPRB, 0.19833E+00_JPRB, 0.19559E+00_JPRB ,&
&  0.19283E+00_JPRB, 0.19018E+00_JPRB, 0.18781E+00_JPRB, 0.18455E+00_JPRB, 0.18144E+00_JPRB, 0.17837E+00_JPRB, 0.17744E+00_JPRB ,&
&  0.17707E+00_JPRB, 0.17757E+00_JPRB, 0.17924E+00_JPRB, 0.18063E+00_JPRB, 0.18264E+00_JPRB, 0.18479E+00_JPRB, 0.18804E+00_JPRB ,&
&  0.19118E+00_JPRB, 0.19351E+00_JPRB, 0.19386E+00_JPRB, 0.19275E+00_JPRB, 0.19181E+00_JPRB, 0.19152E+00_JPRB, 0.19112E+00_JPRB ,&
&  0.19037E+00_JPRB, 0.18992E+00_JPRB, 0.18932E+00_JPRB, 0.18880E+00_JPRB, 0.18739E+00_JPRB, 0.18487E+00_JPRB, 0.18235E+00_JPRB ,&
&  0.17999E+00_JPRB, 0.17789E+00_JPRB, 0.17593E+00_JPRB, 0.17421E+00_JPRB, 0.17320E+00_JPRB, 0.17218E+00_JPRB, 0.17093E+00_JPRB ,&
&  0.17019E+00_JPRB, 0.16941E+00_JPRB, 0.16852E+00_JPRB, 0.16785E+00_JPRB, 0.16710E+00_JPRB, 0.16656E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16549E+00_JPRB, 0.16478E+00_JPRB, 0.16417E+00_JPRB, 0.16351E+00_JPRB, 0.16321E+00_JPRB, 0.16248E+00_JPRB, 0.16178E+00_JPRB ,&
&  0.16091E+00_JPRB, 0.16038E+00_JPRB, 0.15939E+00_JPRB, 0.15880E+00_JPRB, 0.15812E+00_JPRB, 0.15749E+00_JPRB, 0.15706E+00_JPRB ,&
&  0.15712E+00_JPRB, 0.15712E+00_JPRB, 0.15671E+00_JPRB, 0.15663E+00_JPRB, 0.15666E+00_JPRB, 0.15709E+00_JPRB, 0.15834E+00_JPRB ,&
&  0.15915E+00_JPRB, 0.16004E+00_JPRB, 0.16091E+00_JPRB, 0.16160E+00_JPRB, 0.16273E+00_JPRB, 0.16384E+00_JPRB, 0.16496E+00_JPRB ,&
&  0.16629E+00_JPRB, 0.16743E+00_JPRB, 0.16876E+00_JPRB, 0.17033E+00_JPRB, 0.17413E+00_JPRB, 0.17683E+00_JPRB, 0.17972E+00_JPRB ,&
&  0.18183E+00_JPRB, 0.18082E+00_JPRB, 0.19469E+00_JPRB, 0.19736E+00_JPRB, 0.19936E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.17796E+00_JPRB, 0.18082E+00_JPRB, 0.18267E+00_JPRB, 0.18429E+00_JPRB, 0.17891E+00_JPRB, 0.19021E+00_JPRB, 0.18700E+00_JPRB ,&
&  0.18644E+00_JPRB, 0.18496E+00_JPRB, 0.18647E+00_JPRB, 0.18642E+00_JPRB, 0.18574E+00_JPRB, 0.18477E+00_JPRB, 0.18356E+00_JPRB ,&
&  0.18240E+00_JPRB, 0.18097E+00_JPRB, 0.18003E+00_JPRB, 0.17903E+00_JPRB, 0.17823E+00_JPRB, 0.17745E+00_JPRB, 0.17691E+00_JPRB ,&
&  0.17691E+00_JPRB, 0.17768E+00_JPRB, 0.17906E+00_JPRB, 0.17850E+00_JPRB, 0.17854E+00_JPRB, 0.17896E+00_JPRB, 0.17957E+00_JPRB ,&
&  0.17986E+00_JPRB, 0.18000E+00_JPRB, 0.17976E+00_JPRB, 0.17876E+00_JPRB, 0.17874E+00_JPRB, 0.17875E+00_JPRB, 0.17885E+00_JPRB ,&
&  0.17877E+00_JPRB, 0.17848E+00_JPRB, 0.17762E+00_JPRB, 0.17683E+00_JPRB, 0.17601E+00_JPRB, 0.17504E+00_JPRB, 0.17388E+00_JPRB ,&
&  0.17292E+00_JPRB, 0.17216E+00_JPRB, 0.17127E+00_JPRB, 0.17039E+00_JPRB, 0.16962E+00_JPRB, 0.16888E+00_JPRB, 0.16819E+00_JPRB ,&
&  0.16774E+00_JPRB, 0.16741E+00_JPRB, 0.16704E+00_JPRB, 0.16651E+00_JPRB, 0.16592E+00_JPRB, 0.16534E+00_JPRB, 0.16483E+00_JPRB ,&
&  0.16449E+00_JPRB, 0.16393E+00_JPRB, 0.16339E+00_JPRB, 0.16279E+00_JPRB, 0.16234E+00_JPRB, 0.16160E+00_JPRB, 0.16095E+00_JPRB ,&
&  0.16033E+00_JPRB, 0.15977E+00_JPRB, 0.15934E+00_JPRB, 0.15902E+00_JPRB, 0.15873E+00_JPRB, 0.15853E+00_JPRB, 0.15839E+00_JPRB ,&
&  0.15829E+00_JPRB, 0.15823E+00_JPRB, 0.15824E+00_JPRB, 0.15820E+00_JPRB, 0.15812E+00_JPRB, 0.15811E+00_JPRB, 0.15839E+00_JPRB ,&
&  0.15863E+00_JPRB, 0.15881E+00_JPRB, 0.15892E+00_JPRB, 0.15866E+00_JPRB, 0.15905E+00_JPRB, 0.15938E+00_JPRB, 0.15979E+00_JPRB ,&
&  0.16016E+00_JPRB, 0.16092E+00_JPRB, 0.16184E+00_JPRB, 0.16294E+00_JPRB, 0.16502E+00_JPRB, 0.16688E+00_JPRB, 0.16885E+00_JPRB ,&
&  0.17106E+00_JPRB, 0.17425E+00_JPRB, 0.17561E+00_JPRB, 0.17651E+00_JPRB, 0.17725E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.16717E+00_JPRB, 0.16715E+00_JPRB, 0.16699E+00_JPRB, 0.16630E+00_JPRB, 0.16422E+00_JPRB, 0.16386E+00_JPRB, 0.16384E+00_JPRB ,&
&  0.16401E+00_JPRB, 0.16413E+00_JPRB, 0.16495E+00_JPRB, 0.16584E+00_JPRB, 0.16672E+00_JPRB, 0.16739E+00_JPRB, 0.16796E+00_JPRB ,&
&  0.16828E+00_JPRB, 0.16859E+00_JPRB, 0.16755E+00_JPRB, 0.16705E+00_JPRB, 0.16691E+00_JPRB, 0.16692E+00_JPRB, 0.16692E+00_JPRB ,&
&  0.16687E+00_JPRB, 0.16686E+00_JPRB, 0.16674E+00_JPRB, 0.16693E+00_JPRB, 0.16722E+00_JPRB, 0.16746E+00_JPRB, 0.16775E+00_JPRB ,&
&  0.16797E+00_JPRB, 0.16816E+00_JPRB, 0.16828E+00_JPRB, 0.16837E+00_JPRB, 0.16867E+00_JPRB, 0.16882E+00_JPRB, 0.16873E+00_JPRB ,&
&  0.16856E+00_JPRB, 0.16821E+00_JPRB, 0.16783E+00_JPRB, 0.16743E+00_JPRB, 0.16714E+00_JPRB, 0.16686E+00_JPRB, 0.16659E+00_JPRB ,&
&  0.16631E+00_JPRB, 0.16602E+00_JPRB, 0.16570E+00_JPRB, 0.16534E+00_JPRB, 0.16500E+00_JPRB, 0.16474E+00_JPRB, 0.16447E+00_JPRB ,&
&  0.16424E+00_JPRB, 0.16403E+00_JPRB, 0.16380E+00_JPRB, 0.16350E+00_JPRB, 0.16319E+00_JPRB, 0.16288E+00_JPRB, 0.16253E+00_JPRB ,&
&  0.16210E+00_JPRB, 0.16179E+00_JPRB, 0.16149E+00_JPRB, 0.16122E+00_JPRB, 0.16099E+00_JPRB, 0.16074E+00_JPRB, 0.16053E+00_JPRB ,&
&  0.16029E+00_JPRB, 0.16007E+00_JPRB, 0.15982E+00_JPRB, 0.15958E+00_JPRB, 0.15937E+00_JPRB, 0.15918E+00_JPRB, 0.15903E+00_JPRB ,&
&  0.15888E+00_JPRB, 0.15874E+00_JPRB, 0.15861E+00_JPRB, 0.15862E+00_JPRB, 0.15860E+00_JPRB, 0.15863E+00_JPRB, 0.15865E+00_JPRB ,&
&  0.15868E+00_JPRB, 0.15870E+00_JPRB, 0.15871E+00_JPRB, 0.15862E+00_JPRB, 0.15875E+00_JPRB, 0.15895E+00_JPRB, 0.15925E+00_JPRB ,&
&  0.15960E+00_JPRB, 0.16005E+00_JPRB, 0.16052E+00_JPRB, 0.16102E+00_JPRB, 0.16160E+00_JPRB, 0.16208E+00_JPRB, 0.16261E+00_JPRB ,&
&  0.16322E+00_JPRB, 0.16385E+00_JPRB, 0.16431E+00_JPRB, 0.16484E+00_JPRB, 0.16566E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB, 0.16162E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB, 0.44554E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.45245E+00_JPRB, 0.45202E+00_JPRB, 0.45168E+00_JPRB, 0.45161E+00_JPRB, 0.45170E+00_JPRB, 0.45171E+00_JPRB, 0.45113E+00_JPRB ,&
&  0.45043E+00_JPRB, 0.44942E+00_JPRB, 0.44829E+00_JPRB, 0.44695E+00_JPRB, 0.44533E+00_JPRB, 0.44340E+00_JPRB, 0.44161E+00_JPRB ,&
&  0.43994E+00_JPRB, 0.43859E+00_JPRB, 0.43715E+00_JPRB, 0.43613E+00_JPRB, 0.43493E+00_JPRB, 0.43364E+00_JPRB, 0.43230E+00_JPRB ,&
&  0.43122E+00_JPRB, 0.43023E+00_JPRB, 0.42927E+00_JPRB, 0.42839E+00_JPRB, 0.42792E+00_JPRB, 0.42738E+00_JPRB, 0.42682E+00_JPRB ,&
&  0.42635E+00_JPRB, 0.42605E+00_JPRB, 0.42579E+00_JPRB, 0.42553E+00_JPRB, 0.42533E+00_JPRB, 0.42541E+00_JPRB, 0.42551E+00_JPRB ,&
&  0.42573E+00_JPRB, 0.42635E+00_JPRB, 0.42736E+00_JPRB, 0.42830E+00_JPRB, 0.42930E+00_JPRB, 0.43069E+00_JPRB, 0.43231E+00_JPRB ,&
&  0.43370E+00_JPRB, 0.43495E+00_JPRB, 0.43639E+00_JPRB, 0.43794E+00_JPRB, 0.43933E+00_JPRB, 0.44080E+00_JPRB, 0.44219E+00_JPRB ,&
&  0.44360E+00_JPRB, 0.44476E+00_JPRB, 0.44598E+00_JPRB, 0.44647E+00_JPRB, 0.44725E+00_JPRB, 0.44788E+00_JPRB, 0.44866E+00_JPRB ,&
&  0.44876E+00_JPRB, 0.44878E+00_JPRB, 0.44888E+00_JPRB, 0.44916E+00_JPRB, 0.44929E+00_JPRB, 0.44951E+00_JPRB, 0.44980E+00_JPRB ,&
&  0.45027E+00_JPRB, 0.45051E+00_JPRB, 0.45095E+00_JPRB, 0.45125E+00_JPRB, 0.45167E+00_JPRB, 0.45172E+00_JPRB, 0.45195E+00_JPRB ,&
&  0.45217E+00_JPRB, 0.45258E+00_JPRB, 0.45266E+00_JPRB, 0.45279E+00_JPRB, 0.45283E+00_JPRB, 0.45296E+00_JPRB, 0.45306E+00_JPRB ,&
&  0.45326E+00_JPRB, 0.45337E+00_JPRB, 0.45350E+00_JPRB, 0.45345E+00_JPRB, 0.45318E+00_JPRB, 0.45291E+00_JPRB, 0.45273E+00_JPRB ,&
&  0.45272E+00_JPRB, 0.45273E+00_JPRB, 0.45300E+00_JPRB, 0.45335E+00_JPRB, 0.45400E+00_JPRB, 0.45418E+00_JPRB, 0.45432E+00_JPRB ,&
&  0.45440E+00_JPRB, 0.45443E+00_JPRB, 0.45407E+00_JPRB, 0.45354E+00_JPRB, 0.45292E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.45352E+00_JPRB, 0.45258E+00_JPRB, 0.45207E+00_JPRB, 0.45212E+00_JPRB, 0.45242E+00_JPRB, 0.45260E+00_JPRB, 0.45272E+00_JPRB ,&
&  0.45192E+00_JPRB, 0.44979E+00_JPRB, 0.44716E+00_JPRB, 0.44423E+00_JPRB, 0.44054E+00_JPRB, 0.43712E+00_JPRB, 0.43471E+00_JPRB ,&
&  0.43213E+00_JPRB, 0.42956E+00_JPRB, 0.42704E+00_JPRB, 0.42490E+00_JPRB, 0.42251E+00_JPRB, 0.42000E+00_JPRB, 0.41754E+00_JPRB ,&
&  0.41551E+00_JPRB, 0.41320E+00_JPRB, 0.41061E+00_JPRB, 0.40739E+00_JPRB, 0.40517E+00_JPRB, 0.40281E+00_JPRB, 0.40076E+00_JPRB ,&
&  0.39822E+00_JPRB, 0.39698E+00_JPRB, 0.39619E+00_JPRB, 0.39604E+00_JPRB, 0.39770E+00_JPRB, 0.39914E+00_JPRB, 0.39934E+00_JPRB ,&
&  0.39894E+00_JPRB, 0.39693E+00_JPRB, 0.39731E+00_JPRB, 0.39816E+00_JPRB, 0.39971E+00_JPRB, 0.40290E+00_JPRB, 0.40710E+00_JPRB ,&
&  0.41303E+00_JPRB, 0.42011E+00_JPRB, 0.42727E+00_JPRB, 0.43288E+00_JPRB, 0.43832E+00_JPRB, 0.44325E+00_JPRB, 0.44669E+00_JPRB ,&
&  0.45069E+00_JPRB, 0.45393E+00_JPRB, 0.45737E+00_JPRB, 0.46049E+00_JPRB, 0.46396E+00_JPRB, 0.46665E+00_JPRB, 0.46866E+00_JPRB ,&
&  0.46909E+00_JPRB, 0.46868E+00_JPRB, 0.46830E+00_JPRB, 0.46808E+00_JPRB, 0.46699E+00_JPRB, 0.46615E+00_JPRB, 0.46581E+00_JPRB ,&
&  0.46584E+00_JPRB, 0.46558E+00_JPRB, 0.46605E+00_JPRB, 0.46668E+00_JPRB, 0.46748E+00_JPRB, 0.46701E+00_JPRB, 0.46713E+00_JPRB ,&
&  0.46706E+00_JPRB, 0.46704E+00_JPRB, 0.46617E+00_JPRB, 0.46469E+00_JPRB, 0.46337E+00_JPRB, 0.46268E+00_JPRB, 0.46176E+00_JPRB ,&
&  0.46086E+00_JPRB, 0.46020E+00_JPRB, 0.45996E+00_JPRB, 0.45958E+00_JPRB, 0.45871E+00_JPRB, 0.45791E+00_JPRB, 0.45763E+00_JPRB ,&
&  0.45741E+00_JPRB, 0.45694E+00_JPRB, 0.45738E+00_JPRB, 0.45792E+00_JPRB, 0.45965E+00_JPRB, 0.46011E+00_JPRB, 0.46034E+00_JPRB ,&
&  0.45983E+00_JPRB, 0.45951E+00_JPRB, 0.45836E+00_JPRB, 0.45670E+00_JPRB, 0.45474E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.45989E+00_JPRB, 0.45963E+00_JPRB, 0.46066E+00_JPRB, 0.46030E+00_JPRB, 0.45978E+00_JPRB, 0.45824E+00_JPRB, 0.45766E+00_JPRB ,&
&  0.45564E+00_JPRB, 0.45193E+00_JPRB, 0.44580E+00_JPRB, 0.44022E+00_JPRB, 0.43455E+00_JPRB, 0.43089E+00_JPRB, 0.42728E+00_JPRB ,&
&  0.42366E+00_JPRB, 0.42064E+00_JPRB, 0.41737E+00_JPRB, 0.41349E+00_JPRB, 0.40874E+00_JPRB, 0.40305E+00_JPRB, 0.39579E+00_JPRB ,&
&  0.39013E+00_JPRB, 0.38472E+00_JPRB, 0.37991E+00_JPRB, 0.37656E+00_JPRB, 0.37537E+00_JPRB, 0.36223E+00_JPRB, 0.35953E+00_JPRB ,&
&  0.36462E+00_JPRB, 0.35782E+00_JPRB, 0.35169E+00_JPRB, 0.34183E+00_JPRB, 0.34061E+00_JPRB, 0.34660E+00_JPRB, 0.35080E+00_JPRB ,&
&  0.34552E+00_JPRB, 0.36177E+00_JPRB, 0.36298E+00_JPRB, 0.36375E+00_JPRB, 0.36281E+00_JPRB, 0.36084E+00_JPRB, 0.36759E+00_JPRB ,&
&  0.38270E+00_JPRB, 0.38929E+00_JPRB, 0.40347E+00_JPRB, 0.41842E+00_JPRB, 0.42814E+00_JPRB, 0.43903E+00_JPRB, 0.45037E+00_JPRB ,&
&  0.46110E+00_JPRB, 0.46844E+00_JPRB, 0.47380E+00_JPRB, 0.47912E+00_JPRB, 0.48704E+00_JPRB, 0.49479E+00_JPRB, 0.50153E+00_JPRB ,&
&  0.50628E+00_JPRB, 0.50894E+00_JPRB, 0.51228E+00_JPRB, 0.51529E+00_JPRB, 0.51973E+00_JPRB, 0.52354E+00_JPRB, 0.52738E+00_JPRB ,&
&  0.53027E+00_JPRB, 0.53241E+00_JPRB, 0.53343E+00_JPRB, 0.53313E+00_JPRB, 0.53230E+00_JPRB, 0.52921E+00_JPRB, 0.52777E+00_JPRB ,&
&  0.52364E+00_JPRB, 0.51959E+00_JPRB, 0.51445E+00_JPRB, 0.50819E+00_JPRB, 0.50335E+00_JPRB, 0.49965E+00_JPRB, 0.49572E+00_JPRB ,&
&  0.49175E+00_JPRB, 0.48789E+00_JPRB, 0.48459E+00_JPRB, 0.48029E+00_JPRB, 0.47629E+00_JPRB, 0.47210E+00_JPRB, 0.46944E+00_JPRB ,&
&  0.46782E+00_JPRB, 0.46671E+00_JPRB, 0.46705E+00_JPRB, 0.46709E+00_JPRB, 0.46852E+00_JPRB, 0.46901E+00_JPRB, 0.47022E+00_JPRB ,&
&  0.47038E+00_JPRB, 0.46842E+00_JPRB, 0.46511E+00_JPRB, 0.46237E+00_JPRB, 0.45998E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.46568E+00_JPRB, 0.46546E+00_JPRB, 0.46808E+00_JPRB, 0.46711E+00_JPRB, 0.46600E+00_JPRB, 0.46192E+00_JPRB, 0.46070E+00_JPRB ,&
&  0.45608E+00_JPRB, 0.45144E+00_JPRB, 0.44126E+00_JPRB, 0.43455E+00_JPRB, 0.42817E+00_JPRB, 0.42440E+00_JPRB, 0.42098E+00_JPRB ,&
&  0.41820E+00_JPRB, 0.41487E+00_JPRB, 0.41088E+00_JPRB, 0.40520E+00_JPRB, 0.39824E+00_JPRB, 0.39021E+00_JPRB, 0.37926E+00_JPRB ,&
&  0.36928E+00_JPRB, 0.36214E+00_JPRB, 0.35192E+00_JPRB, 0.35472E+00_JPRB, 0.35049E+00_JPRB, 0.34736E+00_JPRB, 0.34587E+00_JPRB ,&
&  0.34176E+00_JPRB, 0.33425E+00_JPRB, 0.32449E+00_JPRB, 0.31693E+00_JPRB, 0.31062E+00_JPRB, 0.30408E+00_JPRB, 0.30972E+00_JPRB ,&
&  0.31407E+00_JPRB, 0.32827E+00_JPRB, 0.33391E+00_JPRB, 0.33533E+00_JPRB, 0.33770E+00_JPRB, 0.33887E+00_JPRB, 0.34519E+00_JPRB ,&
&  0.35394E+00_JPRB, 0.36392E+00_JPRB, 0.39559E+00_JPRB, 0.41109E+00_JPRB, 0.41962E+00_JPRB, 0.43132E+00_JPRB, 0.44141E+00_JPRB ,&
&  0.45378E+00_JPRB, 0.46271E+00_JPRB, 0.46934E+00_JPRB, 0.47804E+00_JPRB, 0.49051E+00_JPRB, 0.50482E+00_JPRB, 0.51712E+00_JPRB ,&
&  0.52836E+00_JPRB, 0.53479E+00_JPRB, 0.54322E+00_JPRB, 0.55239E+00_JPRB, 0.56571E+00_JPRB, 0.57978E+00_JPRB, 0.59493E+00_JPRB ,&
&  0.61041E+00_JPRB, 0.62651E+00_JPRB, 0.63885E+00_JPRB, 0.64941E+00_JPRB, 0.65318E+00_JPRB, 0.65072E+00_JPRB, 0.64515E+00_JPRB ,&
&  0.63223E+00_JPRB, 0.61910E+00_JPRB, 0.60292E+00_JPRB, 0.58078E+00_JPRB, 0.56464E+00_JPRB, 0.55701E+00_JPRB, 0.54262E+00_JPRB ,&
&  0.53106E+00_JPRB, 0.52038E+00_JPRB, 0.51084E+00_JPRB, 0.49950E+00_JPRB, 0.48912E+00_JPRB, 0.47912E+00_JPRB, 0.47239E+00_JPRB ,&
&  0.46733E+00_JPRB, 0.46443E+00_JPRB, 0.46332E+00_JPRB, 0.46331E+00_JPRB, 0.46541E+00_JPRB, 0.46741E+00_JPRB, 0.47160E+00_JPRB ,&
&  0.47384E+00_JPRB, 0.47351E+00_JPRB, 0.47052E+00_JPRB, 0.46794E+00_JPRB, 0.46541E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.47631E+00_JPRB, 0.47512E+00_JPRB, 0.47358E+00_JPRB, 0.46748E+00_JPRB, 0.46352E+00_JPRB, 0.45834E+00_JPRB, 0.45652E+00_JPRB ,&
&  0.45113E+00_JPRB, 0.44641E+00_JPRB, 0.43574E+00_JPRB, 0.42930E+00_JPRB, 0.42187E+00_JPRB, 0.41777E+00_JPRB, 0.41389E+00_JPRB ,&
&  0.41153E+00_JPRB, 0.40843E+00_JPRB, 0.40660E+00_JPRB, 0.40160E+00_JPRB, 0.39677E+00_JPRB, 0.38877E+00_JPRB, 0.38096E+00_JPRB ,&
&  0.37205E+00_JPRB, 0.36335E+00_JPRB, 0.35532E+00_JPRB, 0.35683E+00_JPRB, 0.35296E+00_JPRB, 0.35026E+00_JPRB, 0.34952E+00_JPRB ,&
&  0.35655E+00_JPRB, 0.32581E+00_JPRB, 0.32496E+00_JPRB, 0.31504E+00_JPRB, 0.30377E+00_JPRB, 0.29068E+00_JPRB, 0.28125E+00_JPRB ,&
&  0.28287E+00_JPRB, 0.28412E+00_JPRB, 0.29027E+00_JPRB, 0.29969E+00_JPRB, 0.30381E+00_JPRB, 0.31286E+00_JPRB, 0.32068E+00_JPRB ,&
&  0.32197E+00_JPRB, 0.37103E+00_JPRB, 0.39420E+00_JPRB, 0.40702E+00_JPRB, 0.42149E+00_JPRB, 0.43630E+00_JPRB, 0.45055E+00_JPRB ,&
&  0.46832E+00_JPRB, 0.48854E+00_JPRB, 0.50522E+00_JPRB, 0.52076E+00_JPRB, 0.54486E+00_JPRB, 0.56126E+00_JPRB, 0.57504E+00_JPRB ,&
&  0.58746E+00_JPRB, 0.59092E+00_JPRB, 0.59922E+00_JPRB, 0.61029E+00_JPRB, 0.62695E+00_JPRB, 0.64497E+00_JPRB, 0.66606E+00_JPRB ,&
&  0.68856E+00_JPRB, 0.70928E+00_JPRB, 0.73028E+00_JPRB, 0.74702E+00_JPRB, 0.75777E+00_JPRB, 0.76167E+00_JPRB, 0.76585E+00_JPRB ,&
&  0.75141E+00_JPRB, 0.73437E+00_JPRB, 0.70946E+00_JPRB, 0.68063E+00_JPRB, 0.64842E+00_JPRB, 0.61403E+00_JPRB, 0.58023E+00_JPRB ,&
&  0.56376E+00_JPRB, 0.54563E+00_JPRB, 0.52836E+00_JPRB, 0.51096E+00_JPRB, 0.49539E+00_JPRB, 0.48214E+00_JPRB, 0.47452E+00_JPRB ,&
&  0.46728E+00_JPRB, 0.46293E+00_JPRB, 0.46044E+00_JPRB, 0.45923E+00_JPRB, 0.45921E+00_JPRB, 0.45869E+00_JPRB, 0.46051E+00_JPRB ,&
&  0.46394E+00_JPRB, 0.46875E+00_JPRB, 0.47126E+00_JPRB, 0.47349E+00_JPRB, 0.47485E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.48759E+00_JPRB, 0.49020E+00_JPRB, 0.48986E+00_JPRB, 0.48624E+00_JPRB, 0.48282E+00_JPRB, 0.47461E+00_JPRB, 0.46810E+00_JPRB ,&
&  0.46281E+00_JPRB, 0.45805E+00_JPRB, 0.45058E+00_JPRB, 0.44255E+00_JPRB, 0.43219E+00_JPRB, 0.42484E+00_JPRB, 0.41710E+00_JPRB ,&
&  0.41136E+00_JPRB, 0.40485E+00_JPRB, 0.39805E+00_JPRB, 0.39194E+00_JPRB, 0.38739E+00_JPRB, 0.38165E+00_JPRB, 0.37787E+00_JPRB ,&
&  0.37308E+00_JPRB, 0.36634E+00_JPRB, 0.36139E+00_JPRB, 0.36155E+00_JPRB, 0.35749E+00_JPRB, 0.35616E+00_JPRB, 0.35843E+00_JPRB ,&
&  0.36481E+00_JPRB, 0.36161E+00_JPRB, 0.35799E+00_JPRB, 0.34528E+00_JPRB, 0.32748E+00_JPRB, 0.28000E+00_JPRB, 0.28328E+00_JPRB ,&
&  0.27961E+00_JPRB, 0.27406E+00_JPRB, 0.27577E+00_JPRB, 0.28064E+00_JPRB, 0.28605E+00_JPRB, 0.30145E+00_JPRB, 0.30408E+00_JPRB ,&
&  0.30178E+00_JPRB, 0.36989E+00_JPRB, 0.38594E+00_JPRB, 0.40559E+00_JPRB, 0.42651E+00_JPRB, 0.45095E+00_JPRB, 0.47702E+00_JPRB ,&
&  0.48965E+00_JPRB, 0.51048E+00_JPRB, 0.52816E+00_JPRB, 0.55787E+00_JPRB, 0.58107E+00_JPRB, 0.60691E+00_JPRB, 0.62866E+00_JPRB ,&
&  0.63723E+00_JPRB, 0.64081E+00_JPRB, 0.64661E+00_JPRB, 0.64875E+00_JPRB, 0.65558E+00_JPRB, 0.67197E+00_JPRB, 0.69699E+00_JPRB ,&
&  0.73091E+00_JPRB, 0.74130E+00_JPRB, 0.78000E+00_JPRB, 0.80547E+00_JPRB, 0.81386E+00_JPRB, 0.83131E+00_JPRB, 0.85397E+00_JPRB ,&
&  0.84301E+00_JPRB, 0.83516E+00_JPRB, 0.81401E+00_JPRB, 0.75760E+00_JPRB, 0.69949E+00_JPRB, 0.66159E+00_JPRB, 0.62737E+00_JPRB ,&
&  0.60153E+00_JPRB, 0.57634E+00_JPRB, 0.54796E+00_JPRB, 0.52592E+00_JPRB, 0.50861E+00_JPRB, 0.49584E+00_JPRB, 0.48927E+00_JPRB ,&
&  0.48140E+00_JPRB, 0.47405E+00_JPRB, 0.46773E+00_JPRB, 0.46575E+00_JPRB, 0.45998E+00_JPRB, 0.45512E+00_JPRB, 0.44996E+00_JPRB ,&
&  0.45036E+00_JPRB, 0.45365E+00_JPRB, 0.46290E+00_JPRB, 0.47248E+00_JPRB, 0.47919E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.49117E+00_JPRB, 0.50573E+00_JPRB, 0.51550E+00_JPRB, 0.51986E+00_JPRB, 0.52553E+00_JPRB, 0.51280E+00_JPRB, 0.50102E+00_JPRB ,&
&  0.49146E+00_JPRB, 0.48030E+00_JPRB, 0.47048E+00_JPRB, 0.46127E+00_JPRB, 0.45202E+00_JPRB, 0.44547E+00_JPRB, 0.43973E+00_JPRB ,&
&  0.43513E+00_JPRB, 0.40859E+00_JPRB, 0.39553E+00_JPRB, 0.38836E+00_JPRB, 0.38195E+00_JPRB, 0.37952E+00_JPRB, 0.37913E+00_JPRB ,&
&  0.37460E+00_JPRB, 0.37122E+00_JPRB, 0.37120E+00_JPRB, 0.36887E+00_JPRB, 0.36380E+00_JPRB, 0.35707E+00_JPRB, 0.35966E+00_JPRB ,&
&  0.36963E+00_JPRB, 0.36136E+00_JPRB, 0.35179E+00_JPRB, 0.34391E+00_JPRB, 0.33213E+00_JPRB, 0.30831E+00_JPRB, 0.29097E+00_JPRB ,&
&  0.28580E+00_JPRB, 0.27211E+00_JPRB, 0.27287E+00_JPRB, 0.27491E+00_JPRB, 0.27598E+00_JPRB, 0.28326E+00_JPRB, 0.29960E+00_JPRB ,&
&  0.32081E+00_JPRB, 0.37952E+00_JPRB, 0.40256E+00_JPRB, 0.43395E+00_JPRB, 0.46486E+00_JPRB, 0.49124E+00_JPRB, 0.51555E+00_JPRB ,&
&  0.52975E+00_JPRB, 0.55019E+00_JPRB, 0.57403E+00_JPRB, 0.60287E+00_JPRB, 0.62912E+00_JPRB, 0.64713E+00_JPRB, 0.67052E+00_JPRB ,&
&  0.69964E+00_JPRB, 0.69366E+00_JPRB, 0.69738E+00_JPRB, 0.68495E+00_JPRB, 0.68443E+00_JPRB, 0.70306E+00_JPRB, 0.68858E+00_JPRB ,&
&  0.70119E+00_JPRB, 0.73926E+00_JPRB, 0.80217E+00_JPRB, 0.86944E+00_JPRB, 0.89921E+00_JPRB, 0.93356E+00_JPRB, 0.94822E+00_JPRB ,&
&  0.90415E+00_JPRB, 0.87647E+00_JPRB, 0.89189E+00_JPRB, 0.82771E+00_JPRB, 0.75741E+00_JPRB, 0.69997E+00_JPRB, 0.66238E+00_JPRB ,&
&  0.62404E+00_JPRB, 0.58565E+00_JPRB, 0.56121E+00_JPRB, 0.54178E+00_JPRB, 0.52152E+00_JPRB, 0.50529E+00_JPRB, 0.50438E+00_JPRB ,&
&  0.50389E+00_JPRB, 0.50223E+00_JPRB, 0.49777E+00_JPRB, 0.49098E+00_JPRB, 0.47661E+00_JPRB, 0.46022E+00_JPRB, 0.44552E+00_JPRB ,&
&  0.44185E+00_JPRB, 0.44490E+00_JPRB, 0.45609E+00_JPRB, 0.46712E+00_JPRB, 0.47831E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.48319E+00_JPRB, 0.49506E+00_JPRB, 0.50705E+00_JPRB, 0.51950E+00_JPRB, 0.54263E+00_JPRB, 0.51098E+00_JPRB, 0.48922E+00_JPRB ,&
&  0.47773E+00_JPRB, 0.47895E+00_JPRB, 0.47960E+00_JPRB, 0.47417E+00_JPRB, 0.46440E+00_JPRB, 0.45546E+00_JPRB, 0.44860E+00_JPRB ,&
&  0.44313E+00_JPRB, 0.43034E+00_JPRB, 0.41001E+00_JPRB, 0.37910E+00_JPRB, 0.37141E+00_JPRB, 0.37084E+00_JPRB, 0.37439E+00_JPRB ,&
&  0.37631E+00_JPRB, 0.37236E+00_JPRB, 0.37126E+00_JPRB, 0.36601E+00_JPRB, 0.36159E+00_JPRB, 0.35076E+00_JPRB, 0.34098E+00_JPRB ,&
&  0.34312E+00_JPRB, 0.34499E+00_JPRB, 0.36092E+00_JPRB, 0.35141E+00_JPRB, 0.34859E+00_JPRB, 0.35072E+00_JPRB, 0.29252E+00_JPRB ,&
&  0.28137E+00_JPRB, 0.26984E+00_JPRB, 0.26662E+00_JPRB, 0.26629E+00_JPRB, 0.27656E+00_JPRB, 0.28743E+00_JPRB, 0.30562E+00_JPRB ,&
&  0.33515E+00_JPRB, 0.39284E+00_JPRB, 0.41359E+00_JPRB, 0.44123E+00_JPRB, 0.46986E+00_JPRB, 0.50024E+00_JPRB, 0.53885E+00_JPRB ,&
&  0.57264E+00_JPRB, 0.60406E+00_JPRB, 0.62729E+00_JPRB, 0.64751E+00_JPRB, 0.66832E+00_JPRB, 0.63406E+00_JPRB, 0.65668E+00_JPRB ,&
&  0.81955E+00_JPRB, 0.80102E+00_JPRB, 0.76514E+00_JPRB, 0.74137E+00_JPRB, 0.73631E+00_JPRB, 0.72588E+00_JPRB, 0.71090E+00_JPRB ,&
&  0.74184E+00_JPRB, 0.81396E+00_JPRB, 0.93709E+00_JPRB, 0.10284E+01_JPRB, 0.10308E+01_JPRB, 0.10425E+01_JPRB, 0.10299E+01_JPRB ,&
&  0.96187E+00_JPRB, 0.91393E+00_JPRB, 0.95188E+00_JPRB, 0.79423E+00_JPRB, 0.68841E+00_JPRB, 0.63523E+00_JPRB, 0.61525E+00_JPRB ,&
&  0.59219E+00_JPRB, 0.56706E+00_JPRB, 0.55371E+00_JPRB, 0.53535E+00_JPRB, 0.50887E+00_JPRB, 0.49394E+00_JPRB, 0.49234E+00_JPRB ,&
&  0.49161E+00_JPRB, 0.48861E+00_JPRB, 0.48674E+00_JPRB, 0.48202E+00_JPRB, 0.47127E+00_JPRB, 0.45831E+00_JPRB, 0.45562E+00_JPRB ,&
&  0.45862E+00_JPRB, 0.46137E+00_JPRB, 0.46432E+00_JPRB, 0.46687E+00_JPRB, 0.47272E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.46761E+00_JPRB, 0.48189E+00_JPRB, 0.50168E+00_JPRB, 0.50233E+00_JPRB, 0.50542E+00_JPRB, 0.45796E+00_JPRB, 0.40735E+00_JPRB ,&
&  0.37322E+00_JPRB, 0.37651E+00_JPRB, 0.39135E+00_JPRB, 0.40371E+00_JPRB, 0.40404E+00_JPRB, 0.39718E+00_JPRB, 0.39749E+00_JPRB ,&
&  0.40364E+00_JPRB, 0.40196E+00_JPRB, 0.38569E+00_JPRB, 0.37775E+00_JPRB, 0.36367E+00_JPRB, 0.36324E+00_JPRB, 0.36656E+00_JPRB ,&
&  0.37001E+00_JPRB, 0.36874E+00_JPRB, 0.36587E+00_JPRB, 0.36233E+00_JPRB, 0.36096E+00_JPRB, 0.36161E+00_JPRB, 0.35972E+00_JPRB ,&
&  0.35224E+00_JPRB, 0.34771E+00_JPRB, 0.35446E+00_JPRB, 0.37342E+00_JPRB, 0.37196E+00_JPRB, 0.37407E+00_JPRB, 0.31078E+00_JPRB ,&
&  0.28520E+00_JPRB, 0.26900E+00_JPRB, 0.26859E+00_JPRB, 0.27937E+00_JPRB, 0.30554E+00_JPRB, 0.33177E+00_JPRB, 0.37999E+00_JPRB ,&
&  0.42596E+00_JPRB, 0.44979E+00_JPRB, 0.46274E+00_JPRB, 0.47314E+00_JPRB, 0.50031E+00_JPRB, 0.55564E+00_JPRB, 0.62237E+00_JPRB ,&
&  0.66414E+00_JPRB, 0.69325E+00_JPRB, 0.69922E+00_JPRB, 0.57136E+00_JPRB, 0.55308E+00_JPRB, 0.58800E+00_JPRB, 0.63184E+00_JPRB ,&
&  0.69961E+00_JPRB, 0.71674E+00_JPRB, 0.72880E+00_JPRB, 0.75992E+00_JPRB, 0.74094E+00_JPRB, 0.72042E+00_JPRB, 0.69253E+00_JPRB ,&
&  0.74473E+00_JPRB, 0.84364E+00_JPRB, 0.10122E+01_JPRB, 0.11021E+01_JPRB, 0.10768E+01_JPRB, 0.10872E+01_JPRB, 0.10505E+01_JPRB ,&
&  0.10136E+01_JPRB, 0.91192E+00_JPRB, 0.81853E+00_JPRB, 0.71143E+00_JPRB, 0.62994E+00_JPRB, 0.58453E+00_JPRB, 0.55990E+00_JPRB ,&
&  0.53828E+00_JPRB, 0.51940E+00_JPRB, 0.51994E+00_JPRB, 0.50975E+00_JPRB, 0.48352E+00_JPRB, 0.45892E+00_JPRB, 0.43952E+00_JPRB ,&
&  0.42650E+00_JPRB, 0.41982E+00_JPRB, 0.41863E+00_JPRB, 0.41251E+00_JPRB, 0.41498E+00_JPRB, 0.42368E+00_JPRB, 0.42962E+00_JPRB ,&
&  0.43086E+00_JPRB, 0.43076E+00_JPRB, 0.43590E+00_JPRB, 0.44151E+00_JPRB, 0.44971E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.52878E+00_JPRB, 0.51163E+00_JPRB, 0.48566E+00_JPRB, 0.48593E+00_JPRB, 0.46544E+00_JPRB, 0.45328E+00_JPRB, 0.41440E+00_JPRB ,&
&  0.36447E+00_JPRB, 0.34693E+00_JPRB, 0.36229E+00_JPRB, 0.36387E+00_JPRB, 0.34924E+00_JPRB, 0.34391E+00_JPRB, 0.34831E+00_JPRB ,&
&  0.35642E+00_JPRB, 0.37576E+00_JPRB, 0.37526E+00_JPRB, 0.36344E+00_JPRB, 0.35740E+00_JPRB, 0.35656E+00_JPRB, 0.36180E+00_JPRB ,&
&  0.36534E+00_JPRB, 0.36690E+00_JPRB, 0.36238E+00_JPRB, 0.35462E+00_JPRB, 0.35487E+00_JPRB, 0.35811E+00_JPRB, 0.36064E+00_JPRB ,&
&  0.36467E+00_JPRB, 0.36603E+00_JPRB, 0.37904E+00_JPRB, 0.39254E+00_JPRB, 0.38731E+00_JPRB, 0.37896E+00_JPRB, 0.32667E+00_JPRB ,&
&  0.29072E+00_JPRB, 0.28145E+00_JPRB, 0.30667E+00_JPRB, 0.38195E+00_JPRB, 0.41359E+00_JPRB, 0.45166E+00_JPRB, 0.48352E+00_JPRB ,&
&  0.45507E+00_JPRB, 0.45740E+00_JPRB, 0.49024E+00_JPRB, 0.60640E+00_JPRB, 0.64840E+00_JPRB, 0.70735E+00_JPRB, 0.74975E+00_JPRB ,&
&  0.78271E+00_JPRB, 0.80878E+00_JPRB, 0.81674E+00_JPRB, 0.66993E+00_JPRB, 0.66723E+00_JPRB, 0.80892E+00_JPRB, 0.72213E+00_JPRB ,&
&  0.74574E+00_JPRB, 0.78222E+00_JPRB, 0.81922E+00_JPRB, 0.78320E+00_JPRB, 0.72022E+00_JPRB, 0.70668E+00_JPRB, 0.73375E+00_JPRB ,&
&  0.79985E+00_JPRB, 0.91308E+00_JPRB, 0.11434E+01_JPRB, 0.12688E+01_JPRB, 0.12463E+01_JPRB, 0.12309E+01_JPRB, 0.12073E+01_JPRB ,&
&  0.11868E+01_JPRB, 0.10778E+01_JPRB, 0.93618E+00_JPRB, 0.80392E+00_JPRB, 0.69069E+00_JPRB, 0.60394E+00_JPRB, 0.55429E+00_JPRB ,&
&  0.53656E+00_JPRB, 0.52509E+00_JPRB, 0.52514E+00_JPRB, 0.51247E+00_JPRB, 0.48084E+00_JPRB, 0.44588E+00_JPRB, 0.41803E+00_JPRB ,&
&  0.40067E+00_JPRB, 0.37943E+00_JPRB, 0.35898E+00_JPRB, 0.35386E+00_JPRB, 0.37686E+00_JPRB, 0.41083E+00_JPRB, 0.42381E+00_JPRB ,&
&  0.41357E+00_JPRB, 0.41761E+00_JPRB, 0.44454E+00_JPRB, 0.47411E+00_JPRB, 0.49020E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.52786E+00_JPRB, 0.50413E+00_JPRB, 0.48404E+00_JPRB, 0.46297E+00_JPRB, 0.45549E+00_JPRB, 0.44552E+00_JPRB, 0.40425E+00_JPRB ,&
&  0.36042E+00_JPRB, 0.32991E+00_JPRB, 0.33707E+00_JPRB, 0.32775E+00_JPRB, 0.31756E+00_JPRB, 0.31110E+00_JPRB, 0.30995E+00_JPRB ,&
&  0.31562E+00_JPRB, 0.33463E+00_JPRB, 0.34936E+00_JPRB, 0.35185E+00_JPRB, 0.34893E+00_JPRB, 0.34693E+00_JPRB, 0.35367E+00_JPRB ,&
&  0.36330E+00_JPRB, 0.37456E+00_JPRB, 0.38058E+00_JPRB, 0.38092E+00_JPRB, 0.37832E+00_JPRB, 0.37992E+00_JPRB, 0.38635E+00_JPRB ,&
&  0.39821E+00_JPRB, 0.40663E+00_JPRB, 0.39848E+00_JPRB, 0.40480E+00_JPRB, 0.38614E+00_JPRB, 0.38865E+00_JPRB, 0.38039E+00_JPRB ,&
&  0.31716E+00_JPRB, 0.32965E+00_JPRB, 0.45064E+00_JPRB, 0.48063E+00_JPRB, 0.49669E+00_JPRB, 0.53166E+00_JPRB, 0.57741E+00_JPRB ,&
&  0.62018E+00_JPRB, 0.66036E+00_JPRB, 0.69626E+00_JPRB, 0.73566E+00_JPRB, 0.77315E+00_JPRB, 0.82398E+00_JPRB, 0.86732E+00_JPRB ,&
&  0.94249E+00_JPRB, 0.76809E+00_JPRB, 0.71232E+00_JPRB, 0.75334E+00_JPRB, 0.10086E+01_JPRB, 0.10297E+01_JPRB, 0.98147E+00_JPRB ,&
&  0.95250E+00_JPRB, 0.90301E+00_JPRB, 0.86377E+00_JPRB, 0.84106E+00_JPRB, 0.85410E+00_JPRB, 0.89733E+00_JPRB, 0.97948E+00_JPRB ,&
&  0.10553E+01_JPRB, 0.12018E+01_JPRB, 0.14387E+01_JPRB, 0.14931E+01_JPRB, 0.14713E+01_JPRB, 0.14587E+01_JPRB, 0.14143E+01_JPRB ,&
&  0.13564E+01_JPRB, 0.12390E+01_JPRB, 0.10461E+01_JPRB, 0.88497E+00_JPRB, 0.75122E+00_JPRB, 0.65013E+00_JPRB, 0.57149E+00_JPRB ,&
&  0.53289E+00_JPRB, 0.51991E+00_JPRB, 0.51200E+00_JPRB, 0.49981E+00_JPRB, 0.48035E+00_JPRB, 0.45864E+00_JPRB, 0.43381E+00_JPRB ,&
&  0.40493E+00_JPRB, 0.37644E+00_JPRB, 0.35344E+00_JPRB, 0.36283E+00_JPRB, 0.39613E+00_JPRB, 0.43093E+00_JPRB, 0.44821E+00_JPRB ,&
&  0.45690E+00_JPRB, 0.46616E+00_JPRB, 0.46826E+00_JPRB, 0.47758E+00_JPRB, 0.49827E+00_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.52764E+00_JPRB, 0.49349E+00_JPRB, 0.46500E+00_JPRB, 0.44454E+00_JPRB, 0.44052E+00_JPRB, 0.43363E+00_JPRB, 0.42891E+00_JPRB ,&
&  0.47656E+00_JPRB, 0.39575E+00_JPRB, 0.47862E+00_JPRB, 0.37787E+00_JPRB, 0.34535E+00_JPRB, 0.34007E+00_JPRB, 0.32529E+00_JPRB ,&
&  0.31658E+00_JPRB, 0.31821E+00_JPRB, 0.32214E+00_JPRB, 0.32613E+00_JPRB, 0.32455E+00_JPRB, 0.32240E+00_JPRB, 0.32871E+00_JPRB ,&
&  0.34252E+00_JPRB, 0.36257E+00_JPRB, 0.38709E+00_JPRB, 0.39839E+00_JPRB, 0.40378E+00_JPRB, 0.40524E+00_JPRB, 0.41703E+00_JPRB ,&
&  0.39346E+00_JPRB, 0.38974E+00_JPRB, 0.40033E+00_JPRB, 0.40282E+00_JPRB, 0.39931E+00_JPRB, 0.39041E+00_JPRB, 0.40939E+00_JPRB ,&
&  0.41132E+00_JPRB, 0.38682E+00_JPRB, 0.49383E+00_JPRB, 0.50652E+00_JPRB, 0.53460E+00_JPRB, 0.56575E+00_JPRB, 0.60005E+00_JPRB ,&
&  0.63610E+00_JPRB, 0.68009E+00_JPRB, 0.73039E+00_JPRB, 0.76794E+00_JPRB, 0.82886E+00_JPRB, 0.90079E+00_JPRB, 0.95915E+00_JPRB ,&
&  0.10401E+01_JPRB, 0.78835E+00_JPRB, 0.87148E+00_JPRB, 0.91495E+00_JPRB, 0.11195E+01_JPRB, 0.13517E+01_JPRB, 0.13326E+01_JPRB ,&
&  0.13480E+01_JPRB, 0.12565E+01_JPRB, 0.11661E+01_JPRB, 0.11018E+01_JPRB, 0.11335E+01_JPRB, 0.11989E+01_JPRB, 0.12862E+01_JPRB ,&
&  0.14010E+01_JPRB, 0.15493E+01_JPRB, 0.17955E+01_JPRB, 0.18003E+01_JPRB, 0.17684E+01_JPRB, 0.17164E+01_JPRB, 0.16247E+01_JPRB ,&
&  0.14975E+01_JPRB, 0.13440E+01_JPRB, 0.11487E+01_JPRB, 0.94386E+00_JPRB, 0.80436E+00_JPRB, 0.68557E+00_JPRB, 0.59098E+00_JPRB ,&
&  0.52137E+00_JPRB, 0.47971E+00_JPRB, 0.47334E+00_JPRB, 0.46096E+00_JPRB, 0.45321E+00_JPRB, 0.44066E+00_JPRB, 0.41518E+00_JPRB ,&
&  0.39871E+00_JPRB, 0.40149E+00_JPRB, 0.44284E+00_JPRB, 0.48651E+00_JPRB, 0.50281E+00_JPRB, 0.51578E+00_JPRB, 0.57063E+00_JPRB ,&
&  0.51580E+00_JPRB, 0.55745E+00_JPRB, 0.60063E+00_JPRB, 0.57820E+00_JPRB, 0.55912E+00_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.53164E+00_JPRB, 0.48886E+00_JPRB, 0.44597E+00_JPRB, 0.43106E+00_JPRB, 0.44206E+00_JPRB, 0.45704E+00_JPRB, 0.44682E+00_JPRB ,&
&  0.48533E+00_JPRB, 0.47238E+00_JPRB, 0.50008E+00_JPRB, 0.53924E+00_JPRB, 0.57932E+00_JPRB, 0.49909E+00_JPRB, 0.37143E+00_JPRB ,&
&  0.33179E+00_JPRB, 0.33234E+00_JPRB, 0.33927E+00_JPRB, 0.33674E+00_JPRB, 0.33616E+00_JPRB, 0.33221E+00_JPRB, 0.33232E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.34782E+00_JPRB, 0.37850E+00_JPRB, 0.39016E+00_JPRB, 0.39603E+00_JPRB, 0.39801E+00_JPRB, 0.41221E+00_JPRB ,&
&  0.39012E+00_JPRB, 0.38452E+00_JPRB, 0.38677E+00_JPRB, 0.36301E+00_JPRB, 0.39947E+00_JPRB, 0.41316E+00_JPRB, 0.42446E+00_JPRB ,&
&  0.43890E+00_JPRB, 0.46759E+00_JPRB, 0.50077E+00_JPRB, 0.52982E+00_JPRB, 0.55594E+00_JPRB, 0.58301E+00_JPRB, 0.61251E+00_JPRB ,&
&  0.64369E+00_JPRB, 0.68370E+00_JPRB, 0.73129E+00_JPRB, 0.78931E+00_JPRB, 0.92043E+00_JPRB, 0.90893E+00_JPRB, 0.10069E+01_JPRB ,&
&  0.10546E+01_JPRB, 0.11719E+01_JPRB, 0.12944E+01_JPRB, 0.11939E+01_JPRB, 0.12805E+01_JPRB, 0.14306E+01_JPRB, 0.13839E+01_JPRB ,&
&  0.13522E+01_JPRB, 0.12752E+01_JPRB, 0.12763E+01_JPRB, 0.13218E+01_JPRB, 0.14139E+01_JPRB, 0.14936E+01_JPRB, 0.15780E+01_JPRB ,&
&  0.17285E+01_JPRB, 0.20044E+01_JPRB, 0.21175E+01_JPRB, 0.20570E+01_JPRB, 0.19770E+01_JPRB, 0.18789E+01_JPRB, 0.17354E+01_JPRB ,&
&  0.15721E+01_JPRB, 0.13874E+01_JPRB, 0.11818E+01_JPRB, 0.97565E+00_JPRB, 0.80293E+00_JPRB, 0.70827E+00_JPRB, 0.60838E+00_JPRB ,&
&  0.50544E+00_JPRB, 0.43662E+00_JPRB, 0.43353E+00_JPRB, 0.43300E+00_JPRB, 0.42771E+00_JPRB, 0.41018E+00_JPRB, 0.39446E+00_JPRB ,&
&  0.42612E+00_JPRB, 0.49222E+00_JPRB, 0.67016E+00_JPRB, 0.65465E+00_JPRB, 0.65612E+00_JPRB, 0.64799E+00_JPRB, 0.53352E+00_JPRB ,&
&  0.49796E+00_JPRB, 0.63174E+00_JPRB, 0.59059E+00_JPRB, 0.58417E+00_JPRB, 0.56715E+00_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.54837E+00_JPRB, 0.49178E+00_JPRB, 0.43596E+00_JPRB, 0.42477E+00_JPRB, 0.44322E+00_JPRB, 0.45537E+00_JPRB, 0.46471E+00_JPRB ,&
&  0.48224E+00_JPRB, 0.49073E+00_JPRB, 0.50378E+00_JPRB, 0.51916E+00_JPRB, 0.52370E+00_JPRB, 0.55336E+00_JPRB, 0.49202E+00_JPRB ,&
&  0.40116E+00_JPRB, 0.37804E+00_JPRB, 0.35840E+00_JPRB, 0.35750E+00_JPRB, 0.37755E+00_JPRB, 0.37719E+00_JPRB, 0.37541E+00_JPRB ,&
&  0.38180E+00_JPRB, 0.37942E+00_JPRB, 0.38865E+00_JPRB, 0.38852E+00_JPRB, 0.38707E+00_JPRB, 0.39279E+00_JPRB, 0.40324E+00_JPRB ,&
&  0.37597E+00_JPRB, 0.35543E+00_JPRB, 0.34492E+00_JPRB, 0.33968E+00_JPRB, 0.36959E+00_JPRB, 0.41201E+00_JPRB, 0.43826E+00_JPRB ,&
&  0.45666E+00_JPRB, 0.47745E+00_JPRB, 0.50141E+00_JPRB, 0.53058E+00_JPRB, 0.56184E+00_JPRB, 0.59325E+00_JPRB, 0.61814E+00_JPRB ,&
&  0.64558E+00_JPRB, 0.67839E+00_JPRB, 0.72487E+00_JPRB, 0.80596E+00_JPRB, 0.85605E+00_JPRB, 0.92967E+00_JPRB, 0.10789E+01_JPRB ,&
&  0.11760E+01_JPRB, 0.12877E+01_JPRB, 0.13921E+01_JPRB, 0.15174E+01_JPRB, 0.16067E+01_JPRB, 0.15410E+01_JPRB, 0.14628E+01_JPRB ,&
&  0.14413E+01_JPRB, 0.14119E+01_JPRB, 0.15478E+01_JPRB, 0.16445E+01_JPRB, 0.16992E+01_JPRB, 0.17886E+01_JPRB, 0.18657E+01_JPRB ,&
&  0.20146E+01_JPRB, 0.21705E+01_JPRB, 0.22035E+01_JPRB, 0.21288E+01_JPRB, 0.20324E+01_JPRB, 0.19331E+01_JPRB, 0.17724E+01_JPRB ,&
&  0.15870E+01_JPRB, 0.13697E+01_JPRB, 0.11269E+01_JPRB, 0.92205E+00_JPRB, 0.74864E+00_JPRB, 0.68917E+00_JPRB, 0.62076E+00_JPRB ,&
&  0.49885E+00_JPRB, 0.44000E+00_JPRB, 0.43203E+00_JPRB, 0.44940E+00_JPRB, 0.44627E+00_JPRB, 0.42459E+00_JPRB, 0.42086E+00_JPRB ,&
&  0.47039E+00_JPRB, 0.61568E+00_JPRB, 0.64630E+00_JPRB, 0.66814E+00_JPRB, 0.68169E+00_JPRB, 0.68701E+00_JPRB, 0.48566E+00_JPRB ,&
&  0.52088E+00_JPRB, 0.63173E+00_JPRB, 0.61200E+00_JPRB, 0.59830E+00_JPRB, 0.58209E+00_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.56663E+00_JPRB, 0.50625E+00_JPRB, 0.45508E+00_JPRB, 0.43757E+00_JPRB, 0.45504E+00_JPRB, 0.47295E+00_JPRB, 0.48514E+00_JPRB ,&
&  0.50066E+00_JPRB, 0.51195E+00_JPRB, 0.52069E+00_JPRB, 0.52886E+00_JPRB, 0.53360E+00_JPRB, 0.52529E+00_JPRB, 0.57334E+00_JPRB ,&
&  0.43479E+00_JPRB, 0.39237E+00_JPRB, 0.36283E+00_JPRB, 0.36913E+00_JPRB, 0.44389E+00_JPRB, 0.45213E+00_JPRB, 0.46776E+00_JPRB ,&
&  0.46147E+00_JPRB, 0.46244E+00_JPRB, 0.45400E+00_JPRB, 0.44059E+00_JPRB, 0.42774E+00_JPRB, 0.42469E+00_JPRB, 0.41373E+00_JPRB ,&
&  0.37797E+00_JPRB, 0.35329E+00_JPRB, 0.34974E+00_JPRB, 0.35549E+00_JPRB, 0.38789E+00_JPRB, 0.41577E+00_JPRB, 0.46610E+00_JPRB ,&
&  0.49547E+00_JPRB, 0.52050E+00_JPRB, 0.54613E+00_JPRB, 0.57372E+00_JPRB, 0.59750E+00_JPRB, 0.62822E+00_JPRB, 0.66402E+00_JPRB ,&
&  0.70124E+00_JPRB, 0.73999E+00_JPRB, 0.81085E+00_JPRB, 0.89764E+00_JPRB, 0.90314E+00_JPRB, 0.98440E+00_JPRB, 0.11778E+01_JPRB ,&
&  0.13472E+01_JPRB, 0.14026E+01_JPRB, 0.16108E+01_JPRB, 0.20324E+01_JPRB, 0.21004E+01_JPRB, 0.20162E+01_JPRB, 0.18963E+01_JPRB ,&
&  0.18210E+01_JPRB, 0.17607E+01_JPRB, 0.17890E+01_JPRB, 0.18608E+01_JPRB, 0.19166E+01_JPRB, 0.20385E+01_JPRB, 0.21458E+01_JPRB ,&
&  0.21874E+01_JPRB, 0.21914E+01_JPRB, 0.21478E+01_JPRB, 0.20568E+01_JPRB, 0.19235E+01_JPRB, 0.18147E+01_JPRB, 0.16973E+01_JPRB ,&
&  0.15365E+01_JPRB, 0.13360E+01_JPRB, 0.10582E+01_JPRB, 0.87469E+00_JPRB, 0.67561E+00_JPRB, 0.61515E+00_JPRB, 0.68273E+00_JPRB ,&
&  0.52756E+00_JPRB, 0.46202E+00_JPRB, 0.46395E+00_JPRB, 0.49331E+00_JPRB, 0.50010E+00_JPRB, 0.49648E+00_JPRB, 0.45721E+00_JPRB ,&
&  0.42120E+00_JPRB, 0.51263E+00_JPRB, 0.70118E+00_JPRB, 0.73914E+00_JPRB, 0.74153E+00_JPRB, 0.72267E+00_JPRB, 0.57412E+00_JPRB ,&
&  0.66824E+00_JPRB, 0.62902E+00_JPRB, 0.62342E+00_JPRB, 0.61307E+00_JPRB, 0.60274E+00_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.57855E+00_JPRB, 0.52184E+00_JPRB, 0.48197E+00_JPRB, 0.46611E+00_JPRB, 0.47926E+00_JPRB, 0.49974E+00_JPRB, 0.51170E+00_JPRB ,&
&  0.51986E+00_JPRB, 0.52473E+00_JPRB, 0.52906E+00_JPRB, 0.53377E+00_JPRB, 0.53819E+00_JPRB, 0.52223E+00_JPRB, 0.50854E+00_JPRB ,&
&  0.50371E+00_JPRB, 0.41665E+00_JPRB, 0.38928E+00_JPRB, 0.38104E+00_JPRB, 0.44291E+00_JPRB, 0.48686E+00_JPRB, 0.53595E+00_JPRB ,&
&  0.57216E+00_JPRB, 0.60680E+00_JPRB, 0.59815E+00_JPRB, 0.56960E+00_JPRB, 0.54032E+00_JPRB, 0.51710E+00_JPRB, 0.47345E+00_JPRB ,&
&  0.42297E+00_JPRB, 0.40207E+00_JPRB, 0.40591E+00_JPRB, 0.44011E+00_JPRB, 0.50034E+00_JPRB, 0.46480E+00_JPRB, 0.52842E+00_JPRB ,&
&  0.56489E+00_JPRB, 0.60038E+00_JPRB, 0.61591E+00_JPRB, 0.63866E+00_JPRB, 0.69805E+00_JPRB, 0.77279E+00_JPRB, 0.83999E+00_JPRB ,&
&  0.90104E+00_JPRB, 0.91916E+00_JPRB, 0.90941E+00_JPRB, 0.98153E+00_JPRB, 0.10560E+01_JPRB, 0.11041E+01_JPRB, 0.12084E+01_JPRB ,&
&  0.14079E+01_JPRB, 0.15256E+01_JPRB, 0.18022E+01_JPRB, 0.23060E+01_JPRB, 0.23795E+01_JPRB, 0.22987E+01_JPRB, 0.22792E+01_JPRB ,&
&  0.22406E+01_JPRB, 0.21657E+01_JPRB, 0.21075E+01_JPRB, 0.20542E+01_JPRB, 0.21368E+01_JPRB, 0.22926E+01_JPRB, 0.23991E+01_JPRB ,&
&  0.22870E+01_JPRB, 0.21122E+01_JPRB, 0.20287E+01_JPRB, 0.18781E+01_JPRB, 0.17426E+01_JPRB, 0.15878E+01_JPRB, 0.15854E+01_JPRB ,&
&  0.15677E+01_JPRB, 0.11497E+01_JPRB, 0.94583E+00_JPRB, 0.87940E+00_JPRB, 0.66800E+00_JPRB, 0.57369E+00_JPRB, 0.58309E+00_JPRB ,&
&  0.48406E+00_JPRB, 0.44476E+00_JPRB, 0.47890E+00_JPRB, 0.51790E+00_JPRB, 0.56468E+00_JPRB, 0.59575E+00_JPRB, 0.57329E+00_JPRB ,&
&  0.53707E+00_JPRB, 0.58116E+00_JPRB, 0.81236E+00_JPRB, 0.85458E+00_JPRB, 0.83909E+00_JPRB, 0.77444E+00_JPRB, 0.71637E+00_JPRB ,&
&  0.65751E+00_JPRB, 0.66001E+00_JPRB, 0.65581E+00_JPRB, 0.64690E+00_JPRB, 0.63343E+00_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.63169E+00_JPRB, 0.57169E+00_JPRB, 0.52760E+00_JPRB, 0.51705E+00_JPRB, 0.53507E+00_JPRB, 0.54671E+00_JPRB, 0.55202E+00_JPRB ,&
&  0.53834E+00_JPRB, 0.53140E+00_JPRB, 0.54017E+00_JPRB, 0.54032E+00_JPRB, 0.54043E+00_JPRB, 0.52535E+00_JPRB, 0.49159E+00_JPRB ,&
&  0.47461E+00_JPRB, 0.45877E+00_JPRB, 0.41405E+00_JPRB, 0.41672E+00_JPRB, 0.44146E+00_JPRB, 0.52753E+00_JPRB, 0.60106E+00_JPRB ,&
&  0.67424E+00_JPRB, 0.73094E+00_JPRB, 0.75954E+00_JPRB, 0.77708E+00_JPRB, 0.75875E+00_JPRB, 0.70622E+00_JPRB, 0.65208E+00_JPRB ,&
&  0.60408E+00_JPRB, 0.58531E+00_JPRB, 0.56137E+00_JPRB, 0.63814E+00_JPRB, 0.66635E+00_JPRB, 0.62446E+00_JPRB, 0.71646E+00_JPRB ,&
&  0.74930E+00_JPRB, 0.78097E+00_JPRB, 0.79659E+00_JPRB, 0.85970E+00_JPRB, 0.92886E+00_JPRB, 0.95316E+00_JPRB, 0.94408E+00_JPRB ,&
&  0.92939E+00_JPRB, 0.94216E+00_JPRB, 0.98450E+00_JPRB, 0.97428E+00_JPRB, 0.10259E+01_JPRB, 0.11376E+01_JPRB, 0.13511E+01_JPRB ,&
&  0.16410E+01_JPRB, 0.16378E+01_JPRB, 0.16559E+01_JPRB, 0.22411E+01_JPRB, 0.27179E+01_JPRB, 0.26584E+01_JPRB, 0.25424E+01_JPRB ,&
&  0.25378E+01_JPRB, 0.23794E+01_JPRB, 0.22201E+01_JPRB, 0.21653E+01_JPRB, 0.24093E+01_JPRB, 0.25809E+01_JPRB, 0.26015E+01_JPRB ,&
&  0.24110E+01_JPRB, 0.21661E+01_JPRB, 0.20356E+01_JPRB, 0.18735E+01_JPRB, 0.16757E+01_JPRB, 0.15911E+01_JPRB, 0.16065E+01_JPRB ,&
&  0.15596E+01_JPRB, 0.13794E+01_JPRB, 0.10651E+01_JPRB, 0.95381E+00_JPRB, 0.76525E+00_JPRB, 0.66402E+00_JPRB, 0.63551E+00_JPRB ,&
&  0.54419E+00_JPRB, 0.51162E+00_JPRB, 0.51539E+00_JPRB, 0.54408E+00_JPRB, 0.72531E+00_JPRB, 0.77948E+00_JPRB, 0.83324E+00_JPRB ,&
&  0.83025E+00_JPRB, 0.81521E+00_JPRB, 0.10395E+01_JPRB, 0.10547E+01_JPRB, 0.98632E+00_JPRB, 0.86299E+00_JPRB, 0.75293E+00_JPRB ,&
&  0.70710E+00_JPRB, 0.72060E+00_JPRB, 0.71243E+00_JPRB, 0.70183E+00_JPRB, 0.68384E+00_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.63252E+00_JPRB, 0.57155E+00_JPRB, 0.52785E+00_JPRB, 0.52098E+00_JPRB, 0.53526E+00_JPRB, 0.54588E+00_JPRB, 0.54855E+00_JPRB ,&
&  0.55015E+00_JPRB, 0.55551E+00_JPRB, 0.55688E+00_JPRB, 0.54156E+00_JPRB, 0.52605E+00_JPRB, 0.50804E+00_JPRB, 0.47835E+00_JPRB ,&
&  0.45437E+00_JPRB, 0.45224E+00_JPRB, 0.42059E+00_JPRB, 0.42804E+00_JPRB, 0.47507E+00_JPRB, 0.59714E+00_JPRB, 0.72828E+00_JPRB ,&
&  0.83484E+00_JPRB, 0.90640E+00_JPRB, 0.96496E+00_JPRB, 0.10125E+01_JPRB, 0.10533E+01_JPRB, 0.10419E+01_JPRB, 0.99144E+00_JPRB ,&
&  0.10268E+01_JPRB, 0.88024E+00_JPRB, 0.95885E+00_JPRB, 0.94666E+00_JPRB, 0.99822E+00_JPRB, 0.99201E+00_JPRB, 0.95738E+00_JPRB ,&
&  0.92352E+00_JPRB, 0.92490E+00_JPRB, 0.92227E+00_JPRB, 0.94672E+00_JPRB, 0.98010E+00_JPRB, 0.99747E+00_JPRB, 0.10204E+01_JPRB ,&
&  0.10457E+01_JPRB, 0.10064E+01_JPRB, 0.93664E+00_JPRB, 0.93057E+00_JPRB, 0.12410E+01_JPRB, 0.16318E+01_JPRB, 0.19804E+01_JPRB ,&
&  0.19184E+01_JPRB, 0.17697E+01_JPRB, 0.25659E+01_JPRB, 0.28111E+01_JPRB, 0.27048E+01_JPRB, 0.28516E+01_JPRB, 0.27761E+01_JPRB ,&
&  0.25670E+01_JPRB, 0.22622E+01_JPRB, 0.20142E+01_JPRB, 0.20475E+01_JPRB, 0.27636E+01_JPRB, 0.29577E+01_JPRB, 0.29300E+01_JPRB ,&
&  0.26321E+01_JPRB, 0.23345E+01_JPRB, 0.20552E+01_JPRB, 0.18546E+01_JPRB, 0.17367E+01_JPRB, 0.17818E+01_JPRB, 0.16603E+01_JPRB ,&
&  0.15309E+01_JPRB, 0.15922E+01_JPRB, 0.13396E+01_JPRB, 0.10673E+01_JPRB, 0.89499E+00_JPRB, 0.80475E+00_JPRB, 0.74499E+00_JPRB ,&
&  0.65991E+00_JPRB, 0.61044E+00_JPRB, 0.60559E+00_JPRB, 0.71256E+00_JPRB, 0.97678E+00_JPRB, 0.99682E+00_JPRB, 0.10195E+01_JPRB ,&
&  0.10617E+01_JPRB, 0.13634E+01_JPRB, 0.12827E+01_JPRB, 0.11876E+01_JPRB, 0.10910E+01_JPRB, 0.96755E+00_JPRB, 0.86886E+00_JPRB ,&
&  0.79890E+00_JPRB, 0.79940E+00_JPRB, 0.79051E+00_JPRB, 0.75350E+00_JPRB, 0.70197E+00_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.67052E+00_JPRB, 0.60330E+00_JPRB, 0.57050E+00_JPRB, 0.55719E+00_JPRB, 0.55943E+00_JPRB, 0.55757E+00_JPRB, 0.53981E+00_JPRB ,&
&  0.53146E+00_JPRB, 0.54290E+00_JPRB, 0.53221E+00_JPRB, 0.52811E+00_JPRB, 0.52483E+00_JPRB, 0.49845E+00_JPRB, 0.46969E+00_JPRB ,&
&  0.45878E+00_JPRB, 0.45214E+00_JPRB, 0.44904E+00_JPRB, 0.50043E+00_JPRB, 0.57009E+00_JPRB, 0.67078E+00_JPRB, 0.81748E+00_JPRB ,&
&  0.94080E+00_JPRB, 0.10478E+01_JPRB, 0.11259E+01_JPRB, 0.12732E+01_JPRB, 0.14116E+01_JPRB, 0.15092E+01_JPRB, 0.14808E+01_JPRB ,&
&  0.13376E+01_JPRB, 0.13603E+01_JPRB, 0.14364E+01_JPRB, 0.13711E+01_JPRB, 0.13191E+01_JPRB, 0.12495E+01_JPRB, 0.11765E+01_JPRB ,&
&  0.11015E+01_JPRB, 0.10418E+01_JPRB, 0.10268E+01_JPRB, 0.10379E+01_JPRB, 0.10648E+01_JPRB, 0.11260E+01_JPRB, 0.11184E+01_JPRB ,&
&  0.10411E+01_JPRB, 0.99448E+00_JPRB, 0.94023E+00_JPRB, 0.11450E+01_JPRB, 0.16302E+01_JPRB, 0.23649E+01_JPRB, 0.25627E+01_JPRB ,&
&  0.27245E+01_JPRB, 0.25154E+01_JPRB, 0.26217E+01_JPRB, 0.29199E+01_JPRB, 0.23042E+01_JPRB, 0.24306E+01_JPRB, 0.25019E+01_JPRB ,&
&  0.22922E+01_JPRB, 0.21486E+01_JPRB, 0.23468E+01_JPRB, 0.26977E+01_JPRB, 0.31848E+01_JPRB, 0.38826E+01_JPRB, 0.37947E+01_JPRB ,&
&  0.31541E+01_JPRB, 0.26192E+01_JPRB, 0.21129E+01_JPRB, 0.19391E+01_JPRB, 0.17005E+01_JPRB, 0.16392E+01_JPRB, 0.13316E+01_JPRB ,&
&  0.13353E+01_JPRB, 0.11536E+01_JPRB, 0.12230E+01_JPRB, 0.10774E+01_JPRB, 0.97658E+00_JPRB, 0.87397E+00_JPRB, 0.79350E+00_JPRB ,&
&  0.74667E+00_JPRB, 0.71851E+00_JPRB, 0.76936E+00_JPRB, 0.10083E+01_JPRB, 0.13020E+01_JPRB, 0.11256E+01_JPRB, 0.13678E+01_JPRB ,&
&  0.17581E+01_JPRB, 0.17445E+01_JPRB, 0.13827E+01_JPRB, 0.13191E+01_JPRB, 0.11550E+01_JPRB, 0.10334E+01_JPRB, 0.98116E+00_JPRB ,&
&  0.90554E+00_JPRB, 0.86365E+00_JPRB, 0.83373E+00_JPRB, 0.79241E+00_JPRB, 0.74047E+00_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.71946E+00_JPRB, 0.64559E+00_JPRB, 0.61371E+00_JPRB, 0.61055E+00_JPRB, 0.60396E+00_JPRB, 0.56125E+00_JPRB, 0.54985E+00_JPRB ,&
&  0.56979E+00_JPRB, 0.54856E+00_JPRB, 0.51405E+00_JPRB, 0.49726E+00_JPRB, 0.50012E+00_JPRB, 0.50598E+00_JPRB, 0.48144E+00_JPRB ,&
&  0.48792E+00_JPRB, 0.52890E+00_JPRB, 0.55191E+00_JPRB, 0.58969E+00_JPRB, 0.66984E+00_JPRB, 0.75558E+00_JPRB, 0.81789E+00_JPRB ,&
&  0.10823E+01_JPRB, 0.12062E+01_JPRB, 0.12611E+01_JPRB, 0.14559E+01_JPRB, 0.16910E+01_JPRB, 0.18813E+01_JPRB, 0.18937E+01_JPRB ,&
&  0.19355E+01_JPRB, 0.19422E+01_JPRB, 0.16391E+01_JPRB, 0.14274E+01_JPRB, 0.12992E+01_JPRB, 0.12270E+01_JPRB, 0.11488E+01_JPRB ,&
&  0.10497E+01_JPRB, 0.97799E+00_JPRB, 0.95764E+00_JPRB, 0.94817E+00_JPRB, 0.98511E+00_JPRB, 0.92586E+00_JPRB, 0.90799E+00_JPRB ,&
&  0.91840E+00_JPRB, 0.96553E+00_JPRB, 0.10421E+01_JPRB, 0.14014E+01_JPRB, 0.21461E+01_JPRB, 0.30051E+01_JPRB, 0.32333E+01_JPRB ,&
&  0.29084E+01_JPRB, 0.25888E+01_JPRB, 0.28015E+01_JPRB, 0.30792E+01_JPRB, 0.25875E+01_JPRB, 0.23338E+01_JPRB, 0.21883E+01_JPRB ,&
&  0.18352E+01_JPRB, 0.18417E+01_JPRB, 0.22252E+01_JPRB, 0.28942E+01_JPRB, 0.38808E+01_JPRB, 0.49190E+01_JPRB, 0.47870E+01_JPRB ,&
&  0.38203E+01_JPRB, 0.30434E+01_JPRB, 0.24626E+01_JPRB, 0.19556E+01_JPRB, 0.18481E+01_JPRB, 0.14177E+01_JPRB, 0.15722E+01_JPRB ,&
&  0.15718E+01_JPRB, 0.14926E+01_JPRB, 0.12395E+01_JPRB, 0.10175E+01_JPRB, 0.82166E+00_JPRB, 0.86972E+00_JPRB, 0.82659E+00_JPRB ,&
&  0.83592E+00_JPRB, 0.86409E+00_JPRB, 0.13134E+01_JPRB, 0.15138E+01_JPRB, 0.15806E+01_JPRB, 0.15260E+01_JPRB, 0.21002E+01_JPRB ,&
&  0.20316E+01_JPRB, 0.18988E+01_JPRB, 0.16510E+01_JPRB, 0.14527E+01_JPRB, 0.12719E+01_JPRB, 0.11463E+01_JPRB, 0.10813E+01_JPRB ,&
&  0.99554E+00_JPRB, 0.94905E+00_JPRB, 0.91055E+00_JPRB, 0.86037E+00_JPRB, 0.80005E+00_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.74729E+00_JPRB, 0.70923E+00_JPRB, 0.69084E+00_JPRB, 0.65047E+00_JPRB, 0.59170E+00_JPRB, 0.56327E+00_JPRB, 0.56969E+00_JPRB ,&
&  0.55933E+00_JPRB, 0.53618E+00_JPRB, 0.51345E+00_JPRB, 0.49603E+00_JPRB, 0.49857E+00_JPRB, 0.50923E+00_JPRB, 0.51569E+00_JPRB ,&
&  0.57466E+00_JPRB, 0.70806E+00_JPRB, 0.69989E+00_JPRB, 0.73190E+00_JPRB, 0.77540E+00_JPRB, 0.84283E+00_JPRB, 0.91188E+00_JPRB ,&
&  0.12126E+01_JPRB, 0.12512E+01_JPRB, 0.12217E+01_JPRB, 0.14217E+01_JPRB, 0.18515E+01_JPRB, 0.19032E+01_JPRB, 0.19113E+01_JPRB ,&
&  0.19313E+01_JPRB, 0.18047E+01_JPRB, 0.15353E+01_JPRB, 0.13143E+01_JPRB, 0.11647E+01_JPRB, 0.10776E+01_JPRB, 0.98182E+00_JPRB ,&
&  0.92040E+00_JPRB, 0.91118E+00_JPRB, 0.92122E+00_JPRB, 0.95183E+00_JPRB, 0.92011E+00_JPRB, 0.88586E+00_JPRB, 0.89483E+00_JPRB ,&
&  0.91614E+00_JPRB, 0.97631E+00_JPRB, 0.11213E+01_JPRB, 0.16319E+01_JPRB, 0.24393E+01_JPRB, 0.33720E+01_JPRB, 0.38500E+01_JPRB ,&
&  0.31353E+01_JPRB, 0.30144E+01_JPRB, 0.32486E+01_JPRB, 0.42096E+01_JPRB, 0.25959E+01_JPRB, 0.22628E+01_JPRB, 0.21605E+01_JPRB ,&
&  0.19801E+01_JPRB, 0.22572E+01_JPRB, 0.29945E+01_JPRB, 0.38276E+01_JPRB, 0.40292E+01_JPRB, 0.46968E+01_JPRB, 0.54262E+01_JPRB ,&
&  0.39951E+01_JPRB, 0.31182E+01_JPRB, 0.26497E+01_JPRB, 0.24218E+01_JPRB, 0.15904E+01_JPRB, 0.10833E+01_JPRB, 0.15050E+01_JPRB ,&
&  0.14726E+01_JPRB, 0.85592E+00_JPRB, 0.63978E+00_JPRB, 0.71530E+00_JPRB, 0.53234E+00_JPRB, 0.59271E+00_JPRB, 0.78834E+00_JPRB ,&
&  0.91846E+00_JPRB, 0.10666E+01_JPRB, 0.19338E+01_JPRB, 0.19009E+01_JPRB, 0.19149E+01_JPRB, 0.22759E+01_JPRB, 0.27231E+01_JPRB ,&
&  0.24554E+01_JPRB, 0.17013E+01_JPRB, 0.17435E+01_JPRB, 0.16207E+01_JPRB, 0.13967E+01_JPRB, 0.12169E+01_JPRB, 0.10865E+01_JPRB ,&
&  0.97172E+00_JPRB, 0.88726E+00_JPRB, 0.83631E+00_JPRB, 0.80625E+00_JPRB, 0.79167E+00_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.67426E+00_JPRB, 0.67463E+00_JPRB, 0.65897E+00_JPRB, 0.60301E+00_JPRB, 0.56802E+00_JPRB, 0.56541E+00_JPRB, 0.55397E+00_JPRB ,&
&  0.53551E+00_JPRB, 0.51159E+00_JPRB, 0.49771E+00_JPRB, 0.50271E+00_JPRB, 0.51262E+00_JPRB, 0.52837E+00_JPRB, 0.55952E+00_JPRB ,&
&  0.61440E+00_JPRB, 0.71216E+00_JPRB, 0.82983E+00_JPRB, 0.91020E+00_JPRB, 0.95481E+00_JPRB, 0.87969E+00_JPRB, 0.10398E+01_JPRB ,&
&  0.12301E+01_JPRB, 0.11997E+01_JPRB, 0.12174E+01_JPRB, 0.14632E+01_JPRB, 0.16608E+01_JPRB, 0.16184E+01_JPRB, 0.16398E+01_JPRB ,&
&  0.16169E+01_JPRB, 0.15181E+01_JPRB, 0.13610E+01_JPRB, 0.12550E+01_JPRB, 0.11201E+01_JPRB, 0.97652E+00_JPRB, 0.93172E+00_JPRB ,&
&  0.92617E+00_JPRB, 0.96816E+00_JPRB, 0.10210E+01_JPRB, 0.97722E+00_JPRB, 0.95797E+00_JPRB, 0.93449E+00_JPRB, 0.94947E+00_JPRB ,&
&  0.97848E+00_JPRB, 0.10638E+01_JPRB, 0.13031E+01_JPRB, 0.20341E+01_JPRB, 0.32500E+01_JPRB, 0.44619E+01_JPRB, 0.36839E+01_JPRB ,&
&  0.27980E+01_JPRB, 0.25980E+01_JPRB, 0.35218E+01_JPRB, 0.39293E+01_JPRB, 0.25227E+01_JPRB, 0.23635E+01_JPRB, 0.22459E+01_JPRB ,&
&  0.24828E+01_JPRB, 0.28768E+01_JPRB, 0.30865E+01_JPRB, 0.37998E+01_JPRB, 0.46572E+01_JPRB, 0.47633E+01_JPRB, 0.48585E+01_JPRB ,&
&  0.39803E+01_JPRB, 0.30501E+01_JPRB, 0.22098E+01_JPRB, 0.17317E+01_JPRB, 0.18427E+01_JPRB, 0.13208E+01_JPRB, 0.84663E+00_JPRB ,&
&  0.66659E+00_JPRB, 0.52792E+00_JPRB, 0.42747E+00_JPRB, 0.41534E+00_JPRB, 0.36361E+00_JPRB, 0.46267E+00_JPRB, 0.70419E+00_JPRB ,&
&  0.12007E+01_JPRB, 0.16492E+01_JPRB, 0.21110E+01_JPRB, 0.22032E+01_JPRB, 0.20036E+01_JPRB, 0.24616E+01_JPRB, 0.33856E+01_JPRB ,&
&  0.21673E+01_JPRB, 0.18413E+01_JPRB, 0.18566E+01_JPRB, 0.15776E+01_JPRB, 0.12526E+01_JPRB, 0.10391E+01_JPRB, 0.88508E+00_JPRB ,&
&  0.78460E+00_JPRB, 0.72620E+00_JPRB, 0.69646E+00_JPRB, 0.68803E+00_JPRB, 0.68839E+00_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.63788E+00_JPRB, 0.62820E+00_JPRB, 0.61168E+00_JPRB, 0.57803E+00_JPRB, 0.55746E+00_JPRB, 0.55033E+00_JPRB, 0.54075E+00_JPRB ,&
&  0.52201E+00_JPRB, 0.50276E+00_JPRB, 0.50344E+00_JPRB, 0.52097E+00_JPRB, 0.54760E+00_JPRB, 0.58688E+00_JPRB, 0.62457E+00_JPRB ,&
&  0.66767E+00_JPRB, 0.77907E+00_JPRB, 0.94958E+00_JPRB, 0.10925E+01_JPRB, 0.12061E+01_JPRB, 0.10232E+01_JPRB, 0.11463E+01_JPRB ,&
&  0.12002E+01_JPRB, 0.12166E+01_JPRB, 0.13026E+01_JPRB, 0.14945E+01_JPRB, 0.15963E+01_JPRB, 0.14272E+01_JPRB, 0.14515E+01_JPRB ,&
&  0.14125E+01_JPRB, 0.13530E+01_JPRB, 0.12474E+01_JPRB, 0.11654E+01_JPRB, 0.10512E+01_JPRB, 0.10410E+01_JPRB, 0.10985E+01_JPRB ,&
&  0.11324E+01_JPRB, 0.11832E+01_JPRB, 0.11396E+01_JPRB, 0.10947E+01_JPRB, 0.10693E+01_JPRB, 0.10537E+01_JPRB, 0.10754E+01_JPRB ,&
&  0.11386E+01_JPRB, 0.13162E+01_JPRB, 0.18054E+01_JPRB, 0.29389E+01_JPRB, 0.37986E+01_JPRB, 0.38001E+01_JPRB, 0.34245E+01_JPRB ,&
&  0.31161E+01_JPRB, 0.32626E+01_JPRB, 0.34665E+01_JPRB, 0.40300E+01_JPRB, 0.26755E+01_JPRB, 0.23948E+01_JPRB, 0.23237E+01_JPRB ,&
&  0.26003E+01_JPRB, 0.33474E+01_JPRB, 0.35501E+01_JPRB, 0.40028E+01_JPRB, 0.56885E+01_JPRB, 0.58398E+01_JPRB, 0.45201E+01_JPRB ,&
&  0.38816E+01_JPRB, 0.29359E+01_JPRB, 0.23099E+01_JPRB, 0.19058E+01_JPRB, 0.24417E+01_JPRB, 0.22661E+01_JPRB, 0.88480E+00_JPRB ,&
&  0.53649E+00_JPRB, 0.42959E+00_JPRB, 0.32100E+00_JPRB, 0.30867E+00_JPRB, 0.31589E+00_JPRB, 0.43559E+00_JPRB, 0.95524E+00_JPRB ,&
&  0.11139E+01_JPRB, 0.17344E+01_JPRB, 0.19386E+01_JPRB, 0.20863E+01_JPRB, 0.19766E+01_JPRB, 0.19617E+01_JPRB, 0.28849E+01_JPRB ,&
&  0.26705E+01_JPRB, 0.21197E+01_JPRB, 0.15892E+01_JPRB, 0.12335E+01_JPRB, 0.97348E+00_JPRB, 0.84015E+00_JPRB, 0.75708E+00_JPRB ,&
&  0.71078E+00_JPRB, 0.68612E+00_JPRB, 0.67006E+00_JPRB, 0.65497E+00_JPRB, 0.63699E+00_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.59601E+00_JPRB, 0.59436E+00_JPRB, 0.60272E+00_JPRB, 0.59334E+00_JPRB, 0.58045E+00_JPRB, 0.57357E+00_JPRB, 0.56180E+00_JPRB ,&
&  0.54088E+00_JPRB, 0.52575E+00_JPRB, 0.53880E+00_JPRB, 0.56705E+00_JPRB, 0.61165E+00_JPRB, 0.66060E+00_JPRB, 0.69707E+00_JPRB ,&
&  0.76129E+00_JPRB, 0.90173E+00_JPRB, 0.10890E+01_JPRB, 0.13382E+01_JPRB, 0.14140E+01_JPRB, 0.12344E+01_JPRB, 0.12330E+01_JPRB ,&
&  0.12250E+01_JPRB, 0.13095E+01_JPRB, 0.13580E+01_JPRB, 0.15168E+01_JPRB, 0.15390E+01_JPRB, 0.14557E+01_JPRB, 0.14122E+01_JPRB ,&
&  0.13876E+01_JPRB, 0.13745E+01_JPRB, 0.13014E+01_JPRB, 0.12970E+01_JPRB, 0.13401E+01_JPRB, 0.13980E+01_JPRB, 0.14567E+01_JPRB ,&
&  0.14797E+01_JPRB, 0.14233E+01_JPRB, 0.13332E+01_JPRB, 0.12678E+01_JPRB, 0.12413E+01_JPRB, 0.12555E+01_JPRB, 0.13197E+01_JPRB ,&
&  0.14663E+01_JPRB, 0.18324E+01_JPRB, 0.26436E+01_JPRB, 0.36790E+01_JPRB, 0.35201E+01_JPRB, 0.38229E+01_JPRB, 0.34897E+01_JPRB ,&
&  0.32418E+01_JPRB, 0.33211E+01_JPRB, 0.32125E+01_JPRB, 0.35027E+01_JPRB, 0.28622E+01_JPRB, 0.26265E+01_JPRB, 0.25118E+01_JPRB ,&
&  0.32666E+01_JPRB, 0.37839E+01_JPRB, 0.45186E+01_JPRB, 0.48416E+01_JPRB, 0.58854E+01_JPRB, 0.74812E+01_JPRB, 0.46537E+01_JPRB ,&
&  0.33266E+01_JPRB, 0.30412E+01_JPRB, 0.23881E+01_JPRB, 0.19962E+01_JPRB, 0.22836E+01_JPRB, 0.26643E+01_JPRB, 0.18388E+01_JPRB ,&
&  0.64431E+00_JPRB, 0.42044E+00_JPRB, 0.35565E+00_JPRB, 0.34678E+00_JPRB, 0.29921E+00_JPRB, 0.43129E+00_JPRB, 0.12919E+01_JPRB ,&
&  0.11824E+01_JPRB, 0.15618E+01_JPRB, 0.17316E+01_JPRB, 0.18414E+01_JPRB, 0.18410E+01_JPRB, 0.14637E+01_JPRB, 0.15733E+01_JPRB ,&
&  0.11525E+01_JPRB, 0.91683E+00_JPRB, 0.83003E+00_JPRB, 0.78503E+00_JPRB, 0.71421E+00_JPRB, 0.67561E+00_JPRB, 0.63579E+00_JPRB ,&
&  0.61684E+00_JPRB, 0.61354E+00_JPRB, 0.60741E+00_JPRB, 0.59931E+00_JPRB, 0.60129E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.57788E+00_JPRB, 0.59171E+00_JPRB, 0.60925E+00_JPRB, 0.62672E+00_JPRB, 0.63249E+00_JPRB, 0.61919E+00_JPRB, 0.60040E+00_JPRB ,&
&  0.59151E+00_JPRB, 0.58807E+00_JPRB, 0.60833E+00_JPRB, 0.64071E+00_JPRB, 0.69946E+00_JPRB, 0.75843E+00_JPRB, 0.83898E+00_JPRB ,&
&  0.96112E+00_JPRB, 0.11235E+01_JPRB, 0.13156E+01_JPRB, 0.15112E+01_JPRB, 0.15167E+01_JPRB, 0.12760E+01_JPRB, 0.12112E+01_JPRB ,&
&  0.12659E+01_JPRB, 0.14380E+01_JPRB, 0.14262E+01_JPRB, 0.15933E+01_JPRB, 0.16992E+01_JPRB, 0.17006E+01_JPRB, 0.16481E+01_JPRB ,&
&  0.16233E+01_JPRB, 0.16158E+01_JPRB, 0.16100E+01_JPRB, 0.16578E+01_JPRB, 0.17103E+01_JPRB, 0.17487E+01_JPRB, 0.17540E+01_JPRB ,&
&  0.16788E+01_JPRB, 0.15769E+01_JPRB, 0.15135E+01_JPRB, 0.14950E+01_JPRB, 0.15196E+01_JPRB, 0.15714E+01_JPRB, 0.17092E+01_JPRB ,&
&  0.20129E+01_JPRB, 0.25620E+01_JPRB, 0.33160E+01_JPRB, 0.36359E+01_JPRB, 0.35690E+01_JPRB, 0.34673E+01_JPRB, 0.33104E+01_JPRB ,&
&  0.32002E+01_JPRB, 0.33039E+01_JPRB, 0.32743E+01_JPRB, 0.31922E+01_JPRB, 0.28799E+01_JPRB, 0.28802E+01_JPRB, 0.29669E+01_JPRB ,&
&  0.39163E+01_JPRB, 0.47006E+01_JPRB, 0.48569E+01_JPRB, 0.48572E+01_JPRB, 0.51057E+01_JPRB, 0.58721E+01_JPRB, 0.53882E+01_JPRB ,&
&  0.39432E+01_JPRB, 0.27950E+01_JPRB, 0.22489E+01_JPRB, 0.22008E+01_JPRB, 0.21660E+01_JPRB, 0.21629E+01_JPRB, 0.23224E+01_JPRB ,&
&  0.18273E+01_JPRB, 0.72844E+00_JPRB, 0.46646E+00_JPRB, 0.59183E+00_JPRB, 0.39589E+00_JPRB, 0.44154E+00_JPRB, 0.74046E+00_JPRB ,&
&  0.95491E+00_JPRB, 0.12159E+01_JPRB, 0.13873E+01_JPRB, 0.15374E+01_JPRB, 0.13879E+01_JPRB, 0.11163E+01_JPRB, 0.89177E+00_JPRB ,&
&  0.73918E+00_JPRB, 0.70306E+00_JPRB, 0.70987E+00_JPRB, 0.67387E+00_JPRB, 0.62273E+00_JPRB, 0.60145E+00_JPRB, 0.57752E+00_JPRB ,&
&  0.56807E+00_JPRB, 0.58333E+00_JPRB, 0.59439E+00_JPRB, 0.58747E+00_JPRB, 0.57386E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.61399E+00_JPRB, 0.63489E+00_JPRB, 0.67573E+00_JPRB, 0.68520E+00_JPRB, 0.67075E+00_JPRB, 0.66605E+00_JPRB, 0.67507E+00_JPRB ,&
&  0.68866E+00_JPRB, 0.70515E+00_JPRB, 0.74105E+00_JPRB, 0.80255E+00_JPRB, 0.90137E+00_JPRB, 0.99757E+00_JPRB, 0.11152E+01_JPRB ,&
&  0.12438E+01_JPRB, 0.13778E+01_JPRB, 0.15198E+01_JPRB, 0.16498E+01_JPRB, 0.17153E+01_JPRB, 0.17410E+01_JPRB, 0.11564E+01_JPRB ,&
&  0.10835E+01_JPRB, 0.15894E+01_JPRB, 0.15378E+01_JPRB, 0.17080E+01_JPRB, 0.18189E+01_JPRB, 0.18938E+01_JPRB, 0.18404E+01_JPRB ,&
&  0.18292E+01_JPRB, 0.18376E+01_JPRB, 0.18668E+01_JPRB, 0.18716E+01_JPRB, 0.18699E+01_JPRB, 0.18657E+01_JPRB, 0.18216E+01_JPRB ,&
&  0.17643E+01_JPRB, 0.17304E+01_JPRB, 0.17419E+01_JPRB, 0.17854E+01_JPRB, 0.18671E+01_JPRB, 0.20009E+01_JPRB, 0.22325E+01_JPRB ,&
&  0.25723E+01_JPRB, 0.29961E+01_JPRB, 0.34040E+01_JPRB, 0.33068E+01_JPRB, 0.32302E+01_JPRB, 0.31372E+01_JPRB, 0.31511E+01_JPRB ,&
&  0.30240E+01_JPRB, 0.28196E+01_JPRB, 0.30267E+01_JPRB, 0.31767E+01_JPRB, 0.30171E+01_JPRB, 0.30245E+01_JPRB, 0.33213E+01_JPRB ,&
&  0.43618E+01_JPRB, 0.49743E+01_JPRB, 0.51891E+01_JPRB, 0.44044E+01_JPRB, 0.41173E+01_JPRB, 0.45315E+01_JPRB, 0.46771E+01_JPRB ,&
&  0.39888E+01_JPRB, 0.31243E+01_JPRB, 0.25805E+01_JPRB, 0.21798E+01_JPRB, 0.19077E+01_JPRB, 0.17522E+01_JPRB, 0.17723E+01_JPRB ,&
&  0.17064E+01_JPRB, 0.17350E+01_JPRB, 0.13240E+01_JPRB, 0.77415E+00_JPRB, 0.57977E+00_JPRB, 0.59145E+00_JPRB, 0.72896E+00_JPRB ,&
&  0.87190E+00_JPRB, 0.10055E+01_JPRB, 0.10227E+01_JPRB, 0.13381E+01_JPRB, 0.92601E+00_JPRB, 0.75924E+00_JPRB, 0.64743E+00_JPRB ,&
&  0.63782E+00_JPRB, 0.65945E+00_JPRB, 0.66573E+00_JPRB, 0.63330E+00_JPRB, 0.59719E+00_JPRB, 0.59713E+00_JPRB, 0.59422E+00_JPRB ,&
&  0.60672E+00_JPRB, 0.60944E+00_JPRB, 0.60090E+00_JPRB, 0.60277E+00_JPRB, 0.61148E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.70089E+00_JPRB, 0.73312E+00_JPRB, 0.75884E+00_JPRB, 0.76104E+00_JPRB, 0.76682E+00_JPRB, 0.78528E+00_JPRB, 0.82606E+00_JPRB ,&
&  0.87321E+00_JPRB, 0.92435E+00_JPRB, 0.99331E+00_JPRB, 0.10789E+01_JPRB, 0.11893E+01_JPRB, 0.12828E+01_JPRB, 0.13831E+01_JPRB ,&
&  0.14702E+01_JPRB, 0.15430E+01_JPRB, 0.16134E+01_JPRB, 0.16761E+01_JPRB, 0.17091E+01_JPRB, 0.17572E+01_JPRB, 0.14690E+01_JPRB ,&
&  0.10214E+01_JPRB, 0.17788E+01_JPRB, 0.16004E+01_JPRB, 0.17573E+01_JPRB, 0.17875E+01_JPRB, 0.19165E+01_JPRB, 0.19146E+01_JPRB ,&
&  0.19327E+01_JPRB, 0.19167E+01_JPRB, 0.18962E+01_JPRB, 0.18966E+01_JPRB, 0.18910E+01_JPRB, 0.18812E+01_JPRB, 0.18713E+01_JPRB ,&
&  0.18772E+01_JPRB, 0.19171E+01_JPRB, 0.19919E+01_JPRB, 0.20795E+01_JPRB, 0.22232E+01_JPRB, 0.24158E+01_JPRB, 0.26069E+01_JPRB ,&
&  0.28225E+01_JPRB, 0.30589E+01_JPRB, 0.31304E+01_JPRB, 0.30137E+01_JPRB, 0.29328E+01_JPRB, 0.29778E+01_JPRB, 0.31167E+01_JPRB ,&
&  0.30192E+01_JPRB, 0.28626E+01_JPRB, 0.29944E+01_JPRB, 0.29808E+01_JPRB, 0.28998E+01_JPRB, 0.32419E+01_JPRB, 0.35240E+01_JPRB ,&
&  0.43098E+01_JPRB, 0.44256E+01_JPRB, 0.50689E+01_JPRB, 0.41632E+01_JPRB, 0.34436E+01_JPRB, 0.35790E+01_JPRB, 0.32604E+01_JPRB ,&
&  0.26717E+01_JPRB, 0.25063E+01_JPRB, 0.22478E+01_JPRB, 0.18521E+01_JPRB, 0.16967E+01_JPRB, 0.14958E+01_JPRB, 0.14913E+01_JPRB ,&
&  0.14832E+01_JPRB, 0.14359E+01_JPRB, 0.11006E+01_JPRB, 0.68959E+00_JPRB, 0.61368E+00_JPRB, 0.63988E+00_JPRB, 0.75650E+00_JPRB ,&
&  0.82864E+00_JPRB, 0.93234E+00_JPRB, 0.99461E+00_JPRB, 0.90803E+00_JPRB, 0.68420E+00_JPRB, 0.59068E+00_JPRB, 0.57337E+00_JPRB ,&
&  0.61247E+00_JPRB, 0.64778E+00_JPRB, 0.64642E+00_JPRB, 0.63486E+00_JPRB, 0.61285E+00_JPRB, 0.63835E+00_JPRB, 0.65131E+00_JPRB ,&
&  0.67863E+00_JPRB, 0.67174E+00_JPRB, 0.68460E+00_JPRB, 0.69406E+00_JPRB, 0.69593E+00_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.87427E+00_JPRB, 0.93515E+00_JPRB, 0.10064E+01_JPRB, 0.10752E+01_JPRB, 0.11103E+01_JPRB, 0.11429E+01_JPRB, 0.11561E+01_JPRB ,&
&  0.11290E+01_JPRB, 0.10993E+01_JPRB, 0.11760E+01_JPRB, 0.12516E+01_JPRB, 0.13467E+01_JPRB, 0.14288E+01_JPRB, 0.15074E+01_JPRB ,&
&  0.15674E+01_JPRB, 0.16249E+01_JPRB, 0.16883E+01_JPRB, 0.17758E+01_JPRB, 0.18660E+01_JPRB, 0.20192E+01_JPRB, 0.18675E+01_JPRB ,&
&  0.13501E+01_JPRB, 0.13108E+01_JPRB, 0.17031E+01_JPRB, 0.17128E+01_JPRB, 0.16499E+01_JPRB, 0.17850E+01_JPRB, 0.18983E+01_JPRB ,&
&  0.19015E+01_JPRB, 0.19251E+01_JPRB, 0.19081E+01_JPRB, 0.19097E+01_JPRB, 0.18864E+01_JPRB, 0.19067E+01_JPRB, 0.19445E+01_JPRB ,&
&  0.20144E+01_JPRB, 0.21048E+01_JPRB, 0.22070E+01_JPRB, 0.23352E+01_JPRB, 0.24758E+01_JPRB, 0.26347E+01_JPRB, 0.27765E+01_JPRB ,&
&  0.28968E+01_JPRB, 0.30173E+01_JPRB, 0.30194E+01_JPRB, 0.28717E+01_JPRB, 0.28556E+01_JPRB, 0.29313E+01_JPRB, 0.29594E+01_JPRB ,&
&  0.29302E+01_JPRB, 0.29915E+01_JPRB, 0.30932E+01_JPRB, 0.30833E+01_JPRB, 0.29866E+01_JPRB, 0.33821E+01_JPRB, 0.35955E+01_JPRB ,&
&  0.39586E+01_JPRB, 0.40053E+01_JPRB, 0.46535E+01_JPRB, 0.38738E+01_JPRB, 0.32602E+01_JPRB, 0.30245E+01_JPRB, 0.26434E+01_JPRB ,&
&  0.23229E+01_JPRB, 0.22691E+01_JPRB, 0.21707E+01_JPRB, 0.16758E+01_JPRB, 0.15475E+01_JPRB, 0.11826E+01_JPRB, 0.13540E+01_JPRB ,&
&  0.14206E+01_JPRB, 0.15401E+01_JPRB, 0.10125E+01_JPRB, 0.75983E+00_JPRB, 0.64213E+00_JPRB, 0.67412E+00_JPRB, 0.72541E+00_JPRB ,&
&  0.75143E+00_JPRB, 0.78801E+00_JPRB, 0.74783E+00_JPRB, 0.73100E+00_JPRB, 0.58758E+00_JPRB, 0.54676E+00_JPRB, 0.54655E+00_JPRB ,&
&  0.59428E+00_JPRB, 0.63420E+00_JPRB, 0.64088E+00_JPRB, 0.64861E+00_JPRB, 0.66785E+00_JPRB, 0.70767E+00_JPRB, 0.73460E+00_JPRB ,&
&  0.76783E+00_JPRB, 0.78289E+00_JPRB, 0.80473E+00_JPRB, 0.81462E+00_JPRB, 0.82999E+00_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.89988E+00_JPRB, 0.94643E+00_JPRB, 0.10217E+01_JPRB, 0.10814E+01_JPRB, 0.11166E+01_JPRB, 0.11219E+01_JPRB, 0.11029E+01_JPRB ,&
&  0.11040E+01_JPRB, 0.11332E+01_JPRB, 0.12020E+01_JPRB, 0.12580E+01_JPRB, 0.13207E+01_JPRB, 0.13801E+01_JPRB, 0.14396E+01_JPRB ,&
&  0.14929E+01_JPRB, 0.15404E+01_JPRB, 0.16092E+01_JPRB, 0.17016E+01_JPRB, 0.18119E+01_JPRB, 0.19965E+01_JPRB, 0.20635E+01_JPRB ,&
&  0.16642E+01_JPRB, 0.12106E+01_JPRB, 0.14924E+01_JPRB, 0.15706E+01_JPRB, 0.14796E+01_JPRB, 0.16071E+01_JPRB, 0.17051E+01_JPRB ,&
&  0.16880E+01_JPRB, 0.17313E+01_JPRB, 0.17928E+01_JPRB, 0.18338E+01_JPRB, 0.18803E+01_JPRB, 0.19640E+01_JPRB, 0.20409E+01_JPRB ,&
&  0.21341E+01_JPRB, 0.22247E+01_JPRB, 0.23107E+01_JPRB, 0.24052E+01_JPRB, 0.24942E+01_JPRB, 0.25987E+01_JPRB, 0.27281E+01_JPRB ,&
&  0.28728E+01_JPRB, 0.29691E+01_JPRB, 0.29577E+01_JPRB, 0.29070E+01_JPRB, 0.28789E+01_JPRB, 0.29297E+01_JPRB, 0.29822E+01_JPRB ,&
&  0.30101E+01_JPRB, 0.29919E+01_JPRB, 0.31822E+01_JPRB, 0.34337E+01_JPRB, 0.35439E+01_JPRB, 0.33918E+01_JPRB, 0.35790E+01_JPRB ,&
&  0.37198E+01_JPRB, 0.36489E+01_JPRB, 0.38587E+01_JPRB, 0.42705E+01_JPRB, 0.29329E+01_JPRB, 0.27793E+01_JPRB, 0.22346E+01_JPRB ,&
&  0.23971E+01_JPRB, 0.23080E+01_JPRB, 0.21334E+01_JPRB, 0.18664E+01_JPRB, 0.14072E+01_JPRB, 0.10219E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.12867E+01_JPRB, 0.12029E+01_JPRB, 0.90974E+00_JPRB, 0.76377E+00_JPRB, 0.68160E+00_JPRB, 0.75809E+00_JPRB, 0.77678E+00_JPRB ,&
&  0.74218E+00_JPRB, 0.76381E+00_JPRB, 0.75661E+00_JPRB, 0.81967E+00_JPRB, 0.71836E+00_JPRB, 0.54634E+00_JPRB, 0.52188E+00_JPRB ,&
&  0.56956E+00_JPRB, 0.61712E+00_JPRB, 0.65893E+00_JPRB, 0.69083E+00_JPRB, 0.72054E+00_JPRB, 0.75182E+00_JPRB, 0.77428E+00_JPRB ,&
&  0.82053E+00_JPRB, 0.81353E+00_JPRB, 0.84891E+00_JPRB, 0.86392E+00_JPRB, 0.88585E+00_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.89402E+00_JPRB, 0.91831E+00_JPRB, 0.94289E+00_JPRB, 0.97949E+00_JPRB, 0.10116E+01_JPRB, 0.10196E+01_JPRB, 0.10128E+01_JPRB ,&
&  0.10244E+01_JPRB, 0.10596E+01_JPRB, 0.10989E+01_JPRB, 0.11340E+01_JPRB, 0.11837E+01_JPRB, 0.12269E+01_JPRB, 0.12735E+01_JPRB ,&
&  0.13238E+01_JPRB, 0.13925E+01_JPRB, 0.14963E+01_JPRB, 0.16179E+01_JPRB, 0.17313E+01_JPRB, 0.18851E+01_JPRB, 0.20381E+01_JPRB ,&
&  0.21842E+01_JPRB, 0.19980E+01_JPRB, 0.19191E+01_JPRB, 0.15347E+01_JPRB, 0.11043E+01_JPRB, 0.13755E+01_JPRB, 0.14186E+01_JPRB ,&
&  0.14741E+01_JPRB, 0.15226E+01_JPRB, 0.15635E+01_JPRB, 0.16339E+01_JPRB, 0.17649E+01_JPRB, 0.18754E+01_JPRB, 0.19548E+01_JPRB ,&
&  0.20235E+01_JPRB, 0.20846E+01_JPRB, 0.21332E+01_JPRB, 0.21908E+01_JPRB, 0.22590E+01_JPRB, 0.23251E+01_JPRB, 0.23969E+01_JPRB ,&
&  0.25054E+01_JPRB, 0.26206E+01_JPRB, 0.26979E+01_JPRB, 0.27792E+01_JPRB, 0.27829E+01_JPRB, 0.28855E+01_JPRB, 0.30078E+01_JPRB ,&
&  0.30807E+01_JPRB, 0.31191E+01_JPRB, 0.32968E+01_JPRB, 0.35131E+01_JPRB, 0.35314E+01_JPRB, 0.31532E+01_JPRB, 0.32502E+01_JPRB ,&
&  0.34068E+01_JPRB, 0.32900E+01_JPRB, 0.32054E+01_JPRB, 0.41132E+01_JPRB, 0.25433E+01_JPRB, 0.26866E+01_JPRB, 0.26438E+01_JPRB ,&
&  0.23478E+01_JPRB, 0.22681E+01_JPRB, 0.20489E+01_JPRB, 0.16605E+01_JPRB, 0.13598E+01_JPRB, 0.96408E+00_JPRB, 0.10370E+01_JPRB ,&
&  0.11419E+01_JPRB, 0.97744E+00_JPRB, 0.78199E+00_JPRB, 0.66391E+00_JPRB, 0.67345E+00_JPRB, 0.77804E+00_JPRB, 0.79024E+00_JPRB ,&
&  0.68197E+00_JPRB, 0.76290E+00_JPRB, 0.71474E+00_JPRB, 0.97330E+00_JPRB, 0.10779E+01_JPRB, 0.53323E+00_JPRB, 0.46365E+00_JPRB ,&
&  0.49763E+00_JPRB, 0.56345E+00_JPRB, 0.63589E+00_JPRB, 0.68308E+00_JPRB, 0.69786E+00_JPRB, 0.71812E+00_JPRB, 0.73877E+00_JPRB ,&
&  0.76263E+00_JPRB, 0.79973E+00_JPRB, 0.84873E+00_JPRB, 0.86864E+00_JPRB, 0.88329E+00_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.77357E+00_JPRB, 0.79442E+00_JPRB, 0.80973E+00_JPRB, 0.81240E+00_JPRB, 0.80237E+00_JPRB, 0.79021E+00_JPRB, 0.76558E+00_JPRB ,&
&  0.73823E+00_JPRB, 0.74635E+00_JPRB, 0.78221E+00_JPRB, 0.86490E+00_JPRB, 0.92086E+00_JPRB, 0.95411E+00_JPRB, 0.10069E+01_JPRB ,&
&  0.10613E+01_JPRB, 0.10861E+01_JPRB, 0.10895E+01_JPRB, 0.11300E+01_JPRB, 0.11738E+01_JPRB, 0.12218E+01_JPRB, 0.14035E+01_JPRB ,&
&  0.16694E+01_JPRB, 0.19903E+01_JPRB, 0.23097E+01_JPRB, 0.25443E+01_JPRB, 0.20941E+01_JPRB, 0.14011E+01_JPRB, 0.14614E+01_JPRB ,&
&  0.16148E+01_JPRB, 0.15457E+01_JPRB, 0.13945E+01_JPRB, 0.13957E+01_JPRB, 0.14254E+01_JPRB, 0.15026E+01_JPRB, 0.15996E+01_JPRB ,&
&  0.16667E+01_JPRB, 0.17122E+01_JPRB, 0.17586E+01_JPRB, 0.17974E+01_JPRB, 0.18382E+01_JPRB, 0.18790E+01_JPRB, 0.19331E+01_JPRB ,&
&  0.19723E+01_JPRB, 0.20014E+01_JPRB, 0.21014E+01_JPRB, 0.22515E+01_JPRB, 0.23919E+01_JPRB, 0.25778E+01_JPRB, 0.27340E+01_JPRB ,&
&  0.28240E+01_JPRB, 0.28961E+01_JPRB, 0.32029E+01_JPRB, 0.34008E+01_JPRB, 0.32133E+01_JPRB, 0.29211E+01_JPRB, 0.30168E+01_JPRB ,&
&  0.30227E+01_JPRB, 0.28221E+01_JPRB, 0.22412E+01_JPRB, 0.32395E+01_JPRB, 0.32721E+01_JPRB, 0.28281E+01_JPRB, 0.23510E+01_JPRB ,&
&  0.21179E+01_JPRB, 0.20284E+01_JPRB, 0.18588E+01_JPRB, 0.16259E+01_JPRB, 0.13631E+01_JPRB, 0.13015E+01_JPRB, 0.10750E+01_JPRB ,&
&  0.10682E+01_JPRB, 0.88708E+00_JPRB, 0.70376E+00_JPRB, 0.61243E+00_JPRB, 0.58882E+00_JPRB, 0.70370E+00_JPRB, 0.68523E+00_JPRB ,&
&  0.58440E+00_JPRB, 0.64059E+00_JPRB, 0.62757E+00_JPRB, 0.10243E+01_JPRB, 0.20852E+01_JPRB, 0.55129E+00_JPRB, 0.42124E+00_JPRB ,&
&  0.44837E+00_JPRB, 0.48943E+00_JPRB, 0.52868E+00_JPRB, 0.54266E+00_JPRB, 0.54432E+00_JPRB, 0.56684E+00_JPRB, 0.60567E+00_JPRB ,&
&  0.64803E+00_JPRB, 0.67685E+00_JPRB, 0.69009E+00_JPRB, 0.72396E+00_JPRB, 0.75132E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.60672E+00_JPRB, 0.61929E+00_JPRB, 0.61695E+00_JPRB, 0.62041E+00_JPRB, 0.61315E+00_JPRB, 0.61238E+00_JPRB, 0.59629E+00_JPRB ,&
&  0.58267E+00_JPRB, 0.58820E+00_JPRB, 0.61432E+00_JPRB, 0.66571E+00_JPRB, 0.67671E+00_JPRB, 0.68990E+00_JPRB, 0.69082E+00_JPRB ,&
&  0.70933E+00_JPRB, 0.70297E+00_JPRB, 0.70263E+00_JPRB, 0.69001E+00_JPRB, 0.70362E+00_JPRB, 0.74471E+00_JPRB, 0.82950E+00_JPRB ,&
&  0.94571E+00_JPRB, 0.10911E+01_JPRB, 0.13018E+01_JPRB, 0.15405E+01_JPRB, 0.20526E+01_JPRB, 0.17171E+01_JPRB, 0.14168E+01_JPRB ,&
&  0.14794E+01_JPRB, 0.12716E+01_JPRB, 0.11951E+01_JPRB, 0.11420E+01_JPRB, 0.10960E+01_JPRB, 0.10594E+01_JPRB, 0.10841E+01_JPRB ,&
&  0.11010E+01_JPRB, 0.11401E+01_JPRB, 0.11863E+01_JPRB, 0.12047E+01_JPRB, 0.12255E+01_JPRB, 0.12124E+01_JPRB, 0.11931E+01_JPRB ,&
&  0.11499E+01_JPRB, 0.12065E+01_JPRB, 0.13307E+01_JPRB, 0.14209E+01_JPRB, 0.16629E+01_JPRB, 0.18498E+01_JPRB, 0.20023E+01_JPRB ,&
&  0.20792E+01_JPRB, 0.22056E+01_JPRB, 0.24862E+01_JPRB, 0.25614E+01_JPRB, 0.26131E+01_JPRB, 0.24923E+01_JPRB, 0.24108E+01_JPRB ,&
&  0.22667E+01_JPRB, 0.18363E+01_JPRB, 0.15189E+01_JPRB, 0.22958E+01_JPRB, 0.23934E+01_JPRB, 0.19382E+01_JPRB, 0.20377E+01_JPRB ,&
&  0.18949E+01_JPRB, 0.17955E+01_JPRB, 0.17520E+01_JPRB, 0.15527E+01_JPRB, 0.12832E+01_JPRB, 0.12185E+01_JPRB, 0.11216E+01_JPRB ,&
&  0.10057E+01_JPRB, 0.78323E+00_JPRB, 0.62032E+00_JPRB, 0.56168E+00_JPRB, 0.54828E+00_JPRB, 0.62295E+00_JPRB, 0.62587E+00_JPRB ,&
&  0.65001E+00_JPRB, 0.52822E+00_JPRB, 0.52177E+00_JPRB, 0.70279E+00_JPRB, 0.68729E+00_JPRB, 0.44717E+00_JPRB, 0.38275E+00_JPRB ,&
&  0.40275E+00_JPRB, 0.42440E+00_JPRB, 0.44518E+00_JPRB, 0.45640E+00_JPRB, 0.46059E+00_JPRB, 0.46767E+00_JPRB, 0.49708E+00_JPRB ,&
&  0.52116E+00_JPRB, 0.54293E+00_JPRB, 0.56117E+00_JPRB, 0.58657E+00_JPRB, 0.60048E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.61593E+00_JPRB, 0.62667E+00_JPRB, 0.64270E+00_JPRB, 0.65244E+00_JPRB, 0.66299E+00_JPRB, 0.66887E+00_JPRB, 0.69421E+00_JPRB ,&
&  0.71090E+00_JPRB, 0.72625E+00_JPRB, 0.74899E+00_JPRB, 0.77313E+00_JPRB, 0.78280E+00_JPRB, 0.79349E+00_JPRB, 0.79695E+00_JPRB ,&
&  0.81072E+00_JPRB, 0.82731E+00_JPRB, 0.82519E+00_JPRB, 0.83130E+00_JPRB, 0.83764E+00_JPRB, 0.89977E+00_JPRB, 0.90054E+00_JPRB ,&
&  0.89237E+00_JPRB, 0.90598E+00_JPRB, 0.92083E+00_JPRB, 0.97234E+00_JPRB, 0.11227E+01_JPRB, 0.12319E+01_JPRB, 0.12275E+01_JPRB ,&
&  0.12005E+01_JPRB, 0.10131E+01_JPRB, 0.10376E+01_JPRB, 0.84203E+00_JPRB, 0.80487E+00_JPRB, 0.81844E+00_JPRB, 0.78926E+00_JPRB ,&
&  0.77991E+00_JPRB, 0.79794E+00_JPRB, 0.84342E+00_JPRB, 0.84408E+00_JPRB, 0.88203E+00_JPRB, 0.91062E+00_JPRB, 0.92241E+00_JPRB ,&
&  0.94969E+00_JPRB, 0.99214E+00_JPRB, 0.10399E+01_JPRB, 0.11310E+01_JPRB, 0.11910E+01_JPRB, 0.13841E+01_JPRB, 0.14596E+01_JPRB ,&
&  0.14838E+01_JPRB, 0.15451E+01_JPRB, 0.14383E+01_JPRB, 0.14654E+01_JPRB, 0.16710E+01_JPRB, 0.16739E+01_JPRB, 0.16302E+01_JPRB ,&
&  0.16781E+01_JPRB, 0.15190E+01_JPRB, 0.11356E+01_JPRB, 0.17029E+01_JPRB, 0.19119E+01_JPRB, 0.17871E+01_JPRB, 0.17642E+01_JPRB ,&
&  0.16484E+01_JPRB, 0.14905E+01_JPRB, 0.13852E+01_JPRB, 0.13503E+01_JPRB, 0.12259E+01_JPRB, 0.11141E+01_JPRB, 0.10467E+01_JPRB ,&
&  0.86128E+00_JPRB, 0.69170E+00_JPRB, 0.58025E+00_JPRB, 0.58458E+00_JPRB, 0.64720E+00_JPRB, 0.78863E+00_JPRB, 0.73404E+00_JPRB ,&
&  0.61933E+00_JPRB, 0.49474E+00_JPRB, 0.59785E+00_JPRB, 0.79146E+00_JPRB, 0.52715E+00_JPRB, 0.42928E+00_JPRB, 0.38348E+00_JPRB ,&
&  0.40022E+00_JPRB, 0.42610E+00_JPRB, 0.45117E+00_JPRB, 0.47038E+00_JPRB, 0.49367E+00_JPRB, 0.51409E+00_JPRB, 0.52959E+00_JPRB ,&
&  0.54500E+00_JPRB, 0.56404E+00_JPRB, 0.58015E+00_JPRB, 0.59849E+00_JPRB, 0.61334E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.72657E+00_JPRB, 0.75371E+00_JPRB, 0.77164E+00_JPRB, 0.78596E+00_JPRB, 0.79272E+00_JPRB, 0.79972E+00_JPRB, 0.81561E+00_JPRB ,&
&  0.82809E+00_JPRB, 0.84151E+00_JPRB, 0.85632E+00_JPRB, 0.87797E+00_JPRB, 0.90593E+00_JPRB, 0.92743E+00_JPRB, 0.93599E+00_JPRB ,&
&  0.94187E+00_JPRB, 0.97404E+00_JPRB, 0.99484E+00_JPRB, 0.10347E+01_JPRB, 0.10504E+01_JPRB, 0.10772E+01_JPRB, 0.10841E+01_JPRB ,&
&  0.10818E+01_JPRB, 0.10866E+01_JPRB, 0.10958E+01_JPRB, 0.11175E+01_JPRB, 0.11365E+01_JPRB, 0.12477E+01_JPRB, 0.14130E+01_JPRB ,&
&  0.11551E+01_JPRB, 0.10115E+01_JPRB, 0.84310E+00_JPRB, 0.70257E+00_JPRB, 0.70306E+00_JPRB, 0.72599E+00_JPRB, 0.74110E+00_JPRB ,&
&  0.76327E+00_JPRB, 0.78908E+00_JPRB, 0.81693E+00_JPRB, 0.82632E+00_JPRB, 0.87993E+00_JPRB, 0.97713E+00_JPRB, 0.10034E+01_JPRB ,&
&  0.10568E+01_JPRB, 0.11063E+01_JPRB, 0.11318E+01_JPRB, 0.12453E+01_JPRB, 0.12350E+01_JPRB, 0.14659E+01_JPRB, 0.15764E+01_JPRB ,&
&  0.16030E+01_JPRB, 0.15135E+01_JPRB, 0.11697E+01_JPRB, 0.11737E+01_JPRB, 0.12976E+01_JPRB, 0.12724E+01_JPRB, 0.12329E+01_JPRB ,&
&  0.14326E+01_JPRB, 0.14338E+01_JPRB, 0.12642E+01_JPRB, 0.15994E+01_JPRB, 0.16967E+01_JPRB, 0.16627E+01_JPRB, 0.14817E+01_JPRB ,&
&  0.13093E+01_JPRB, 0.10507E+01_JPRB, 0.10497E+01_JPRB, 0.10705E+01_JPRB, 0.10247E+01_JPRB, 0.96385E+00_JPRB, 0.84152E+00_JPRB ,&
&  0.71647E+00_JPRB, 0.62608E+00_JPRB, 0.58215E+00_JPRB, 0.59152E+00_JPRB, 0.77492E+00_JPRB, 0.71974E+00_JPRB, 0.91032E+00_JPRB ,&
&  0.52183E+00_JPRB, 0.48869E+00_JPRB, 0.42404E+00_JPRB, 0.51855E+00_JPRB, 0.47013E+00_JPRB, 0.39499E+00_JPRB, 0.41709E+00_JPRB ,&
&  0.47359E+00_JPRB, 0.51913E+00_JPRB, 0.55128E+00_JPRB, 0.57409E+00_JPRB, 0.59356E+00_JPRB, 0.61632E+00_JPRB, 0.62646E+00_JPRB ,&
&  0.64173E+00_JPRB, 0.65836E+00_JPRB, 0.67447E+00_JPRB, 0.69796E+00_JPRB, 0.70883E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.76277E+00_JPRB, 0.78408E+00_JPRB, 0.80273E+00_JPRB, 0.82005E+00_JPRB, 0.83803E+00_JPRB, 0.85553E+00_JPRB, 0.87013E+00_JPRB ,&
&  0.88348E+00_JPRB, 0.89950E+00_JPRB, 0.91298E+00_JPRB, 0.92644E+00_JPRB, 0.94665E+00_JPRB, 0.96444E+00_JPRB, 0.97512E+00_JPRB ,&
&  0.99313E+00_JPRB, 0.10174E+01_JPRB, 0.10476E+01_JPRB, 0.10733E+01_JPRB, 0.10854E+01_JPRB, 0.10882E+01_JPRB, 0.10866E+01_JPRB ,&
&  0.10754E+01_JPRB, 0.10689E+01_JPRB, 0.10731E+01_JPRB, 0.10879E+01_JPRB, 0.11176E+01_JPRB, 0.11016E+01_JPRB, 0.11755E+01_JPRB ,&
&  0.92159E+00_JPRB, 0.10756E+01_JPRB, 0.90652E+00_JPRB, 0.80394E+00_JPRB, 0.81431E+00_JPRB, 0.78007E+00_JPRB, 0.79152E+00_JPRB ,&
&  0.83706E+00_JPRB, 0.85920E+00_JPRB, 0.87986E+00_JPRB, 0.89763E+00_JPRB, 0.94194E+00_JPRB, 0.10135E+01_JPRB, 0.10608E+01_JPRB ,&
&  0.11462E+01_JPRB, 0.12456E+01_JPRB, 0.13217E+01_JPRB, 0.14184E+01_JPRB, 0.16168E+01_JPRB, 0.17793E+01_JPRB, 0.18591E+01_JPRB ,&
&  0.18408E+01_JPRB, 0.17597E+01_JPRB, 0.13455E+01_JPRB, 0.13957E+01_JPRB, 0.14895E+01_JPRB, 0.13990E+01_JPRB, 0.12852E+01_JPRB ,&
&  0.13338E+01_JPRB, 0.11645E+01_JPRB, 0.13416E+01_JPRB, 0.13851E+01_JPRB, 0.13336E+01_JPRB, 0.11992E+01_JPRB, 0.10101E+01_JPRB ,&
&  0.84090E+00_JPRB, 0.71329E+00_JPRB, 0.73352E+00_JPRB, 0.83882E+00_JPRB, 0.81380E+00_JPRB, 0.68475E+00_JPRB, 0.62327E+00_JPRB ,&
&  0.55411E+00_JPRB, 0.49289E+00_JPRB, 0.47116E+00_JPRB, 0.49828E+00_JPRB, 0.56079E+00_JPRB, 0.83406E+00_JPRB, 0.89746E+00_JPRB ,&
&  0.50460E+00_JPRB, 0.46177E+00_JPRB, 0.37940E+00_JPRB, 0.44830E+00_JPRB, 0.46042E+00_JPRB, 0.43905E+00_JPRB, 0.49300E+00_JPRB ,&
&  0.60709E+00_JPRB, 0.70458E+00_JPRB, 0.71320E+00_JPRB, 0.64982E+00_JPRB, 0.62677E+00_JPRB, 0.63355E+00_JPRB, 0.65188E+00_JPRB ,&
&  0.67083E+00_JPRB, 0.68813E+00_JPRB, 0.70528E+00_JPRB, 0.72368E+00_JPRB, 0.74225E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.78164E+00_JPRB, 0.80368E+00_JPRB, 0.82192E+00_JPRB, 0.83649E+00_JPRB, 0.85499E+00_JPRB, 0.87420E+00_JPRB, 0.89047E+00_JPRB ,&
&  0.90525E+00_JPRB, 0.91858E+00_JPRB, 0.93386E+00_JPRB, 0.94766E+00_JPRB, 0.96413E+00_JPRB, 0.98422E+00_JPRB, 0.10041E+01_JPRB ,&
&  0.10233E+01_JPRB, 0.10348E+01_JPRB, 0.10634E+01_JPRB, 0.10985E+01_JPRB, 0.11213E+01_JPRB, 0.11261E+01_JPRB, 0.11252E+01_JPRB ,&
&  0.11249E+01_JPRB, 0.11443E+01_JPRB, 0.11578E+01_JPRB, 0.11749E+01_JPRB, 0.12012E+01_JPRB, 0.12607E+01_JPRB, 0.10615E+01_JPRB ,&
&  0.11504E+01_JPRB, 0.10713E+01_JPRB, 0.93550E+00_JPRB, 0.88946E+00_JPRB, 0.88190E+00_JPRB, 0.87949E+00_JPRB, 0.88764E+00_JPRB ,&
&  0.91707E+00_JPRB, 0.91469E+00_JPRB, 0.92761E+00_JPRB, 0.94294E+00_JPRB, 0.98751E+00_JPRB, 0.10474E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.12294E+01_JPRB, 0.13764E+01_JPRB, 0.15486E+01_JPRB, 0.17375E+01_JPRB, 0.19624E+01_JPRB, 0.21475E+01_JPRB, 0.22687E+01_JPRB ,&
&  0.22948E+01_JPRB, 0.22703E+01_JPRB, 0.19392E+01_JPRB, 0.20470E+01_JPRB, 0.20426E+01_JPRB, 0.17273E+01_JPRB, 0.14416E+01_JPRB ,&
&  0.12096E+01_JPRB, 0.97672E+00_JPRB, 0.10253E+01_JPRB, 0.10986E+01_JPRB, 0.10150E+01_JPRB, 0.87170E+00_JPRB, 0.74008E+00_JPRB ,&
&  0.58629E+00_JPRB, 0.53818E+00_JPRB, 0.53435E+00_JPRB, 0.61671E+00_JPRB, 0.60618E+00_JPRB, 0.53513E+00_JPRB, 0.51708E+00_JPRB ,&
&  0.48458E+00_JPRB, 0.44633E+00_JPRB, 0.43413E+00_JPRB, 0.47212E+00_JPRB, 0.55465E+00_JPRB, 0.54975E+00_JPRB, 0.61229E+00_JPRB ,&
&  0.50271E+00_JPRB, 0.44625E+00_JPRB, 0.46985E+00_JPRB, 0.51186E+00_JPRB, 0.55534E+00_JPRB, 0.59617E+00_JPRB, 0.69958E+00_JPRB ,&
&  0.86272E+00_JPRB, 0.10722E+01_JPRB, 0.10437E+01_JPRB, 0.83348E+00_JPRB, 0.67214E+00_JPRB, 0.64022E+00_JPRB, 0.65516E+00_JPRB ,&
&  0.67578E+00_JPRB, 0.69220E+00_JPRB, 0.71196E+00_JPRB, 0.73325E+00_JPRB, 0.75632E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.78842E+00_JPRB, 0.80834E+00_JPRB, 0.82325E+00_JPRB, 0.83384E+00_JPRB, 0.85022E+00_JPRB, 0.86713E+00_JPRB, 0.88296E+00_JPRB ,&
&  0.89652E+00_JPRB, 0.90674E+00_JPRB, 0.92079E+00_JPRB, 0.93266E+00_JPRB, 0.94719E+00_JPRB, 0.96467E+00_JPRB, 0.98534E+00_JPRB ,&
&  0.10009E+01_JPRB, 0.10206E+01_JPRB, 0.10300E+01_JPRB, 0.10637E+01_JPRB, 0.11075E+01_JPRB, 0.11295E+01_JPRB, 0.11392E+01_JPRB ,&
&  0.11342E+01_JPRB, 0.11497E+01_JPRB, 0.11662E+01_JPRB, 0.11878E+01_JPRB, 0.11931E+01_JPRB, 0.12383E+01_JPRB, 0.10677E+01_JPRB ,&
&  0.11184E+01_JPRB, 0.10593E+01_JPRB, 0.96577E+00_JPRB, 0.95120E+00_JPRB, 0.94380E+00_JPRB, 0.94571E+00_JPRB, 0.94735E+00_JPRB ,&
&  0.95019E+00_JPRB, 0.96789E+00_JPRB, 0.97001E+00_JPRB, 0.98223E+00_JPRB, 0.10163E+01_JPRB, 0.10750E+01_JPRB, 0.11560E+01_JPRB ,&
&  0.12730E+01_JPRB, 0.14349E+01_JPRB, 0.16300E+01_JPRB, 0.18456E+01_JPRB, 0.20909E+01_JPRB, 0.23100E+01_JPRB, 0.25181E+01_JPRB ,&
&  0.26751E+01_JPRB, 0.27677E+01_JPRB, 0.25515E+01_JPRB, 0.25303E+01_JPRB, 0.24146E+01_JPRB, 0.17267E+01_JPRB, 0.12911E+01_JPRB ,&
&  0.94660E+00_JPRB, 0.91461E+00_JPRB, 0.95896E+00_JPRB, 0.96365E+00_JPRB, 0.79531E+00_JPRB, 0.69654E+00_JPRB, 0.57968E+00_JPRB ,&
&  0.48809E+00_JPRB, 0.45091E+00_JPRB, 0.45168E+00_JPRB, 0.49596E+00_JPRB, 0.47521E+00_JPRB, 0.44610E+00_JPRB, 0.45558E+00_JPRB ,&
&  0.44557E+00_JPRB, 0.44880E+00_JPRB, 0.46386E+00_JPRB, 0.49650E+00_JPRB, 0.53015E+00_JPRB, 0.52658E+00_JPRB, 0.58242E+00_JPRB ,&
&  0.56583E+00_JPRB, 0.55869E+00_JPRB, 0.56409E+00_JPRB, 0.49318E+00_JPRB, 0.67238E+00_JPRB, 0.76735E+00_JPRB, 0.89823E+00_JPRB ,&
&  0.10354E+01_JPRB, 0.14323E+01_JPRB, 0.16859E+01_JPRB, 0.13127E+01_JPRB, 0.99940E+00_JPRB, 0.70475E+00_JPRB, 0.67018E+00_JPRB ,&
&  0.68669E+00_JPRB, 0.69124E+00_JPRB, 0.71063E+00_JPRB, 0.73551E+00_JPRB, 0.76298E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.78135E+00_JPRB, 0.80483E+00_JPRB, 0.81487E+00_JPRB, 0.81386E+00_JPRB, 0.82756E+00_JPRB, 0.85222E+00_JPRB, 0.86045E+00_JPRB ,&
&  0.86928E+00_JPRB, 0.86608E+00_JPRB, 0.87657E+00_JPRB, 0.88735E+00_JPRB, 0.89822E+00_JPRB, 0.91094E+00_JPRB, 0.92349E+00_JPRB ,&
&  0.93051E+00_JPRB, 0.96010E+00_JPRB, 0.97389E+00_JPRB, 0.10074E+01_JPRB, 0.10424E+01_JPRB, 0.10933E+01_JPRB, 0.11190E+01_JPRB ,&
&  0.11180E+01_JPRB, 0.11439E+01_JPRB, 0.11722E+01_JPRB, 0.12095E+01_JPRB, 0.11988E+01_JPRB, 0.12018E+01_JPRB, 0.95508E+00_JPRB ,&
&  0.10477E+01_JPRB, 0.10228E+01_JPRB, 0.98323E+00_JPRB, 0.97296E+00_JPRB, 0.95978E+00_JPRB, 0.94359E+00_JPRB, 0.92700E+00_JPRB ,&
&  0.93226E+00_JPRB, 0.93381E+00_JPRB, 0.91921E+00_JPRB, 0.97753E+00_JPRB, 0.10189E+01_JPRB, 0.10684E+01_JPRB, 0.11538E+01_JPRB ,&
&  0.12479E+01_JPRB, 0.13885E+01_JPRB, 0.15704E+01_JPRB, 0.17939E+01_JPRB, 0.20378E+01_JPRB, 0.22797E+01_JPRB, 0.25203E+01_JPRB ,&
&  0.27426E+01_JPRB, 0.29196E+01_JPRB, 0.29551E+01_JPRB, 0.24445E+01_JPRB, 0.20750E+01_JPRB, 0.15169E+01_JPRB, 0.11895E+01_JPRB ,&
&  0.99803E+00_JPRB, 0.91583E+00_JPRB, 0.90657E+00_JPRB, 0.88264E+00_JPRB, 0.70963E+00_JPRB, 0.60232E+00_JPRB, 0.50396E+00_JPRB ,&
&  0.46426E+00_JPRB, 0.42901E+00_JPRB, 0.42415E+00_JPRB, 0.41866E+00_JPRB, 0.41451E+00_JPRB, 0.41250E+00_JPRB, 0.42370E+00_JPRB ,&
&  0.42951E+00_JPRB, 0.45048E+00_JPRB, 0.49850E+00_JPRB, 0.55602E+00_JPRB, 0.60661E+00_JPRB, 0.63214E+00_JPRB, 0.59313E+00_JPRB ,&
&  0.61392E+00_JPRB, 0.59845E+00_JPRB, 0.59633E+00_JPRB, 0.64982E+00_JPRB, 0.74437E+00_JPRB, 0.91202E+00_JPRB, 0.11184E+01_JPRB ,&
&  0.13722E+01_JPRB, 0.14314E+01_JPRB, 0.17387E+01_JPRB, 0.22930E+01_JPRB, 0.20708E+01_JPRB, 0.17538E+01_JPRB, 0.64058E+00_JPRB ,&
&  0.67309E+00_JPRB, 0.68079E+00_JPRB, 0.70816E+00_JPRB, 0.73487E+00_JPRB, 0.76842E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.64780E+00_JPRB, 0.67264E+00_JPRB, 0.68970E+00_JPRB, 0.69017E+00_JPRB, 0.71561E+00_JPRB, 0.73280E+00_JPRB, 0.74121E+00_JPRB ,&
&  0.74938E+00_JPRB, 0.76492E+00_JPRB, 0.77187E+00_JPRB, 0.79106E+00_JPRB, 0.80360E+00_JPRB, 0.81622E+00_JPRB, 0.82411E+00_JPRB ,&
&  0.83366E+00_JPRB, 0.85417E+00_JPRB, 0.87932E+00_JPRB, 0.91752E+00_JPRB, 0.95423E+00_JPRB, 0.10016E+01_JPRB, 0.10418E+01_JPRB ,&
&  0.10693E+01_JPRB, 0.11057E+01_JPRB, 0.11492E+01_JPRB, 0.11735E+01_JPRB, 0.11563E+01_JPRB, 0.11342E+01_JPRB, 0.86306E+00_JPRB ,&
&  0.97285E+00_JPRB, 0.98564E+00_JPRB, 0.97898E+00_JPRB, 0.96205E+00_JPRB, 0.93900E+00_JPRB, 0.90851E+00_JPRB, 0.89582E+00_JPRB ,&
&  0.89363E+00_JPRB, 0.88541E+00_JPRB, 0.87751E+00_JPRB, 0.88113E+00_JPRB, 0.99065E+00_JPRB, 0.10431E+01_JPRB, 0.11146E+01_JPRB ,&
&  0.12138E+01_JPRB, 0.13354E+01_JPRB, 0.14841E+01_JPRB, 0.17004E+01_JPRB, 0.19411E+01_JPRB, 0.21781E+01_JPRB, 0.24223E+01_JPRB ,&
&  0.26083E+01_JPRB, 0.27063E+01_JPRB, 0.27412E+01_JPRB, 0.21982E+01_JPRB, 0.18384E+01_JPRB, 0.14760E+01_JPRB, 0.11119E+01_JPRB ,&
&  0.94703E+00_JPRB, 0.88970E+00_JPRB, 0.89238E+00_JPRB, 0.82986E+00_JPRB, 0.68321E+00_JPRB, 0.56106E+00_JPRB, 0.47754E+00_JPRB ,&
&  0.44275E+00_JPRB, 0.40508E+00_JPRB, 0.39942E+00_JPRB, 0.39904E+00_JPRB, 0.40774E+00_JPRB, 0.41450E+00_JPRB, 0.42564E+00_JPRB ,&
&  0.44117E+00_JPRB, 0.46813E+00_JPRB, 0.51768E+00_JPRB, 0.59301E+00_JPRB, 0.69090E+00_JPRB, 0.79762E+00_JPRB, 0.76792E+00_JPRB ,&
&  0.63618E+00_JPRB, 0.56517E+00_JPRB, 0.56787E+00_JPRB, 0.58165E+00_JPRB, 0.61574E+00_JPRB, 0.67718E+00_JPRB, 0.75268E+00_JPRB ,&
&  0.87920E+00_JPRB, 0.95404E+00_JPRB, 0.86787E+00_JPRB, 0.72703E+00_JPRB, 0.10030E+01_JPRB, 0.98504E+00_JPRB, 0.51939E+00_JPRB ,&
&  0.56539E+00_JPRB, 0.55596E+00_JPRB, 0.57776E+00_JPRB, 0.60357E+00_JPRB, 0.63563E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.49689E+00_JPRB, 0.53942E+00_JPRB, 0.55673E+00_JPRB, 0.57807E+00_JPRB, 0.59274E+00_JPRB, 0.61302E+00_JPRB, 0.62978E+00_JPRB ,&
&  0.64519E+00_JPRB, 0.66475E+00_JPRB, 0.67284E+00_JPRB, 0.69255E+00_JPRB, 0.70663E+00_JPRB, 0.71742E+00_JPRB, 0.72521E+00_JPRB ,&
&  0.73447E+00_JPRB, 0.75485E+00_JPRB, 0.77695E+00_JPRB, 0.80410E+00_JPRB, 0.83051E+00_JPRB, 0.86696E+00_JPRB, 0.91886E+00_JPRB ,&
&  0.98193E+00_JPRB, 0.10406E+01_JPRB, 0.10887E+01_JPRB, 0.11246E+01_JPRB, 0.11385E+01_JPRB, 0.11746E+01_JPRB, 0.11550E+01_JPRB ,&
&  0.85775E+00_JPRB, 0.10024E+01_JPRB, 0.97513E+00_JPRB, 0.93637E+00_JPRB, 0.90788E+00_JPRB, 0.88529E+00_JPRB, 0.88382E+00_JPRB ,&
&  0.88110E+00_JPRB, 0.84161E+00_JPRB, 0.84548E+00_JPRB, 0.86508E+00_JPRB, 0.92492E+00_JPRB, 0.97992E+00_JPRB, 0.10549E+01_JPRB ,&
&  0.11541E+01_JPRB, 0.12788E+01_JPRB, 0.14149E+01_JPRB, 0.15628E+01_JPRB, 0.17652E+01_JPRB, 0.20187E+01_JPRB, 0.22463E+01_JPRB ,&
&  0.24830E+01_JPRB, 0.27262E+01_JPRB, 0.28697E+01_JPRB, 0.24221E+01_JPRB, 0.20030E+01_JPRB, 0.17951E+01_JPRB, 0.13497E+01_JPRB ,&
&  0.95549E+00_JPRB, 0.80840E+00_JPRB, 0.81941E+00_JPRB, 0.79767E+00_JPRB, 0.66045E+00_JPRB, 0.55753E+00_JPRB, 0.48522E+00_JPRB ,&
&  0.45586E+00_JPRB, 0.42147E+00_JPRB, 0.40803E+00_JPRB, 0.40806E+00_JPRB, 0.41410E+00_JPRB, 0.42264E+00_JPRB, 0.44461E+00_JPRB ,&
&  0.45929E+00_JPRB, 0.47337E+00_JPRB, 0.49504E+00_JPRB, 0.52024E+00_JPRB, 0.56510E+00_JPRB, 0.63327E+00_JPRB, 0.63950E+00_JPRB ,&
&  0.61202E+00_JPRB, 0.58044E+00_JPRB, 0.55524E+00_JPRB, 0.55648E+00_JPRB, 0.55719E+00_JPRB, 0.56177E+00_JPRB, 0.58076E+00_JPRB ,&
&  0.60910E+00_JPRB, 0.63090E+00_JPRB, 0.59959E+00_JPRB, 0.57798E+00_JPRB, 0.58050E+00_JPRB, 0.55187E+00_JPRB, 0.49777E+00_JPRB ,&
&  0.47717E+00_JPRB, 0.45934E+00_JPRB, 0.44278E+00_JPRB, 0.44942E+00_JPRB, 0.47023E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.41517E+00_JPRB, 0.43315E+00_JPRB, 0.45292E+00_JPRB, 0.47096E+00_JPRB, 0.48712E+00_JPRB, 0.50076E+00_JPRB, 0.52293E+00_JPRB ,&
&  0.54712E+00_JPRB, 0.56259E+00_JPRB, 0.56703E+00_JPRB, 0.58077E+00_JPRB, 0.59616E+00_JPRB, 0.61621E+00_JPRB, 0.62975E+00_JPRB ,&
&  0.64656E+00_JPRB, 0.67208E+00_JPRB, 0.69423E+00_JPRB, 0.71854E+00_JPRB, 0.74247E+00_JPRB, 0.76703E+00_JPRB, 0.80260E+00_JPRB ,&
&  0.84915E+00_JPRB, 0.91010E+00_JPRB, 0.97839E+00_JPRB, 0.10392E+01_JPRB, 0.10896E+01_JPRB, 0.11436E+01_JPRB, 0.11431E+01_JPRB ,&
&  0.75372E+00_JPRB, 0.93651E+00_JPRB, 0.10437E+01_JPRB, 0.96420E+00_JPRB, 0.89551E+00_JPRB, 0.86285E+00_JPRB, 0.85700E+00_JPRB ,&
&  0.83551E+00_JPRB, 0.79928E+00_JPRB, 0.79958E+00_JPRB, 0.86828E+00_JPRB, 0.85601E+00_JPRB, 0.89238E+00_JPRB, 0.94605E+00_JPRB ,&
&  0.10252E+01_JPRB, 0.11236E+01_JPRB, 0.12161E+01_JPRB, 0.13267E+01_JPRB, 0.14866E+01_JPRB, 0.16883E+01_JPRB, 0.19404E+01_JPRB ,&
&  0.22564E+01_JPRB, 0.26134E+01_JPRB, 0.29440E+01_JPRB, 0.23711E+01_JPRB, 0.21595E+01_JPRB, 0.20355E+01_JPRB, 0.17544E+01_JPRB ,&
&  0.11027E+01_JPRB, 0.80923E+00_JPRB, 0.80243E+00_JPRB, 0.77959E+00_JPRB, 0.67230E+00_JPRB, 0.59461E+00_JPRB, 0.53498E+00_JPRB ,&
&  0.49930E+00_JPRB, 0.47072E+00_JPRB, 0.45624E+00_JPRB, 0.44897E+00_JPRB, 0.44909E+00_JPRB, 0.46366E+00_JPRB, 0.47987E+00_JPRB ,&
&  0.49365E+00_JPRB, 0.49497E+00_JPRB, 0.51355E+00_JPRB, 0.52258E+00_JPRB, 0.55426E+00_JPRB, 0.58126E+00_JPRB, 0.59672E+00_JPRB ,&
&  0.60283E+00_JPRB, 0.59582E+00_JPRB, 0.59192E+00_JPRB, 0.58972E+00_JPRB, 0.59049E+00_JPRB, 0.59063E+00_JPRB, 0.59032E+00_JPRB ,&
&  0.58975E+00_JPRB, 0.60552E+00_JPRB, 0.60073E+00_JPRB, 0.59510E+00_JPRB, 0.60237E+00_JPRB, 0.57880E+00_JPRB, 0.56477E+00_JPRB ,&
&  0.54364E+00_JPRB, 0.49707E+00_JPRB, 0.46229E+00_JPRB, 0.43671E+00_JPRB, 0.42008E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.43071E+00_JPRB, 0.40951E+00_JPRB, 0.41192E+00_JPRB, 0.41548E+00_JPRB, 0.42934E+00_JPRB, 0.44243E+00_JPRB, 0.43731E+00_JPRB ,&
&  0.44261E+00_JPRB, 0.44984E+00_JPRB, 0.46006E+00_JPRB, 0.47358E+00_JPRB, 0.49124E+00_JPRB, 0.50918E+00_JPRB, 0.52109E+00_JPRB ,&
&  0.54037E+00_JPRB, 0.56316E+00_JPRB, 0.59015E+00_JPRB, 0.61720E+00_JPRB, 0.64678E+00_JPRB, 0.66991E+00_JPRB, 0.69111E+00_JPRB ,&
&  0.71613E+00_JPRB, 0.75857E+00_JPRB, 0.81578E+00_JPRB, 0.88194E+00_JPRB, 0.96337E+00_JPRB, 0.10580E+01_JPRB, 0.11074E+01_JPRB ,&
&  0.84392E+00_JPRB, 0.52152E+00_JPRB, 0.94871E+00_JPRB, 0.10554E+01_JPRB, 0.93939E+00_JPRB, 0.88707E+00_JPRB, 0.83669E+00_JPRB ,&
&  0.80187E+00_JPRB, 0.80999E+00_JPRB, 0.79002E+00_JPRB, 0.81947E+00_JPRB, 0.80398E+00_JPRB, 0.81545E+00_JPRB, 0.83971E+00_JPRB ,&
&  0.86744E+00_JPRB, 0.92667E+00_JPRB, 0.10202E+01_JPRB, 0.10787E+01_JPRB, 0.11641E+01_JPRB, 0.13161E+01_JPRB, 0.15444E+01_JPRB ,&
&  0.18663E+01_JPRB, 0.22566E+01_JPRB, 0.26482E+01_JPRB, 0.20772E+01_JPRB, 0.16459E+01_JPRB, 0.13336E+01_JPRB, 0.11183E+01_JPRB ,&
&  0.99280E+00_JPRB, 0.81934E+00_JPRB, 0.78461E+00_JPRB, 0.76199E+00_JPRB, 0.69405E+00_JPRB, 0.60650E+00_JPRB, 0.58925E+00_JPRB ,&
&  0.55250E+00_JPRB, 0.52751E+00_JPRB, 0.51405E+00_JPRB, 0.50436E+00_JPRB, 0.50255E+00_JPRB, 0.51247E+00_JPRB, 0.52190E+00_JPRB ,&
&  0.53423E+00_JPRB, 0.54032E+00_JPRB, 0.55416E+00_JPRB, 0.57045E+00_JPRB, 0.57729E+00_JPRB, 0.58171E+00_JPRB, 0.59186E+00_JPRB ,&
&  0.59236E+00_JPRB, 0.60236E+00_JPRB, 0.61331E+00_JPRB, 0.61551E+00_JPRB, 0.61909E+00_JPRB, 0.61248E+00_JPRB, 0.61839E+00_JPRB ,&
&  0.62332E+00_JPRB, 0.62861E+00_JPRB, 0.62615E+00_JPRB, 0.63425E+00_JPRB, 0.64390E+00_JPRB, 0.64528E+00_JPRB, 0.66384E+00_JPRB ,&
&  0.65745E+00_JPRB, 0.63009E+00_JPRB, 0.56971E+00_JPRB, 0.50084E+00_JPRB, 0.46707E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.51395E+00_JPRB, 0.48054E+00_JPRB, 0.44986E+00_JPRB, 0.43380E+00_JPRB, 0.42277E+00_JPRB, 0.41818E+00_JPRB, 0.41678E+00_JPRB ,&
&  0.42004E+00_JPRB, 0.42386E+00_JPRB, 0.41100E+00_JPRB, 0.41468E+00_JPRB, 0.42610E+00_JPRB, 0.43892E+00_JPRB, 0.45601E+00_JPRB ,&
&  0.46531E+00_JPRB, 0.47208E+00_JPRB, 0.48076E+00_JPRB, 0.49796E+00_JPRB, 0.52167E+00_JPRB, 0.54796E+00_JPRB, 0.57283E+00_JPRB ,&
&  0.59754E+00_JPRB, 0.62371E+00_JPRB, 0.65893E+00_JPRB, 0.70569E+00_JPRB, 0.77110E+00_JPRB, 0.86010E+00_JPRB, 0.95200E+00_JPRB ,&
&  0.10545E+01_JPRB, 0.80812E+00_JPRB, 0.61154E+00_JPRB, 0.94861E+00_JPRB, 0.10133E+01_JPRB, 0.96432E+00_JPRB, 0.84962E+00_JPRB ,&
&  0.84373E+00_JPRB, 0.85129E+00_JPRB, 0.82165E+00_JPRB, 0.80503E+00_JPRB, 0.75386E+00_JPRB, 0.75300E+00_JPRB, 0.75774E+00_JPRB ,&
&  0.75339E+00_JPRB, 0.76016E+00_JPRB, 0.81785E+00_JPRB, 0.86093E+00_JPRB, 0.89620E+00_JPRB, 0.99370E+00_JPRB, 0.11965E+01_JPRB ,&
&  0.14759E+01_JPRB, 0.18413E+01_JPRB, 0.22552E+01_JPRB, 0.19125E+01_JPRB, 0.14607E+01_JPRB, 0.11274E+01_JPRB, 0.10035E+01_JPRB ,&
&  0.93243E+00_JPRB, 0.89523E+00_JPRB, 0.83395E+00_JPRB, 0.76513E+00_JPRB, 0.69586E+00_JPRB, 0.63389E+00_JPRB, 0.62046E+00_JPRB ,&
&  0.59937E+00_JPRB, 0.56791E+00_JPRB, 0.54703E+00_JPRB, 0.52855E+00_JPRB, 0.52679E+00_JPRB, 0.53588E+00_JPRB, 0.54465E+00_JPRB ,&
&  0.55228E+00_JPRB, 0.56069E+00_JPRB, 0.57595E+00_JPRB, 0.59481E+00_JPRB, 0.60017E+00_JPRB, 0.59709E+00_JPRB, 0.59725E+00_JPRB ,&
&  0.59655E+00_JPRB, 0.59910E+00_JPRB, 0.61471E+00_JPRB, 0.62132E+00_JPRB, 0.62032E+00_JPRB, 0.60911E+00_JPRB, 0.61958E+00_JPRB ,&
&  0.62176E+00_JPRB, 0.63205E+00_JPRB, 0.63138E+00_JPRB, 0.64364E+00_JPRB, 0.65138E+00_JPRB, 0.67611E+00_JPRB, 0.70266E+00_JPRB ,&
&  0.72599E+00_JPRB, 0.72446E+00_JPRB, 0.69348E+00_JPRB, 0.58547E+00_JPRB, 0.54547E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.54771E+00_JPRB, 0.52572E+00_JPRB, 0.50936E+00_JPRB, 0.49318E+00_JPRB, 0.47605E+00_JPRB, 0.45801E+00_JPRB, 0.44398E+00_JPRB ,&
&  0.42728E+00_JPRB, 0.42467E+00_JPRB, 0.42026E+00_JPRB, 0.41490E+00_JPRB, 0.40870E+00_JPRB, 0.40956E+00_JPRB, 0.41653E+00_JPRB ,&
&  0.42260E+00_JPRB, 0.42519E+00_JPRB, 0.42800E+00_JPRB, 0.42891E+00_JPRB, 0.43203E+00_JPRB, 0.44158E+00_JPRB, 0.45765E+00_JPRB ,&
&  0.48062E+00_JPRB, 0.50376E+00_JPRB, 0.53242E+00_JPRB, 0.56496E+00_JPRB, 0.59633E+00_JPRB, 0.64430E+00_JPRB, 0.70645E+00_JPRB ,&
&  0.83374E+00_JPRB, 0.83488E+00_JPRB, 0.54642E+00_JPRB, 0.86809E+00_JPRB, 0.10748E+01_JPRB, 0.10204E+01_JPRB, 0.96155E+00_JPRB ,&
&  0.94823E+00_JPRB, 0.89528E+00_JPRB, 0.86708E+00_JPRB, 0.79908E+00_JPRB, 0.72077E+00_JPRB, 0.70422E+00_JPRB, 0.70181E+00_JPRB ,&
&  0.68052E+00_JPRB, 0.66596E+00_JPRB, 0.67449E+00_JPRB, 0.68851E+00_JPRB, 0.70204E+00_JPRB, 0.74307E+00_JPRB, 0.88402E+00_JPRB ,&
&  0.11119E+01_JPRB, 0.14588E+01_JPRB, 0.18493E+01_JPRB, 0.17552E+01_JPRB, 0.13473E+01_JPRB, 0.11607E+01_JPRB, 0.10748E+01_JPRB ,&
&  0.10487E+01_JPRB, 0.99992E+00_JPRB, 0.86257E+00_JPRB, 0.76663E+00_JPRB, 0.69315E+00_JPRB, 0.64062E+00_JPRB, 0.63957E+00_JPRB ,&
&  0.62235E+00_JPRB, 0.59498E+00_JPRB, 0.57788E+00_JPRB, 0.55369E+00_JPRB, 0.54313E+00_JPRB, 0.53979E+00_JPRB, 0.54639E+00_JPRB ,&
&  0.55472E+00_JPRB, 0.56158E+00_JPRB, 0.57160E+00_JPRB, 0.58439E+00_JPRB, 0.59185E+00_JPRB, 0.59551E+00_JPRB, 0.59439E+00_JPRB ,&
&  0.59544E+00_JPRB, 0.60222E+00_JPRB, 0.61400E+00_JPRB, 0.61983E+00_JPRB, 0.61857E+00_JPRB, 0.61489E+00_JPRB, 0.60744E+00_JPRB ,&
&  0.60429E+00_JPRB, 0.60450E+00_JPRB, 0.59849E+00_JPRB, 0.59462E+00_JPRB, 0.62392E+00_JPRB, 0.63447E+00_JPRB, 0.64169E+00_JPRB ,&
&  0.68262E+00_JPRB, 0.71756E+00_JPRB, 0.72123E+00_JPRB, 0.61399E+00_JPRB, 0.57542E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.55004E+00_JPRB, 0.53316E+00_JPRB, 0.51979E+00_JPRB, 0.50953E+00_JPRB, 0.49654E+00_JPRB, 0.48313E+00_JPRB, 0.47257E+00_JPRB ,&
&  0.45685E+00_JPRB, 0.44704E+00_JPRB, 0.43984E+00_JPRB, 0.43218E+00_JPRB, 0.42487E+00_JPRB, 0.41809E+00_JPRB, 0.41246E+00_JPRB ,&
&  0.40820E+00_JPRB, 0.40493E+00_JPRB, 0.40276E+00_JPRB, 0.39790E+00_JPRB, 0.39305E+00_JPRB, 0.39117E+00_JPRB, 0.39411E+00_JPRB ,&
&  0.40858E+00_JPRB, 0.42863E+00_JPRB, 0.45088E+00_JPRB, 0.47240E+00_JPRB, 0.48990E+00_JPRB, 0.51344E+00_JPRB, 0.54268E+00_JPRB ,&
&  0.65546E+00_JPRB, 0.70422E+00_JPRB, 0.84938E+00_JPRB, 0.12985E+01_JPRB, 0.13190E+01_JPRB, 0.11775E+01_JPRB, 0.11436E+01_JPRB ,&
&  0.11126E+01_JPRB, 0.10291E+01_JPRB, 0.99926E+00_JPRB, 0.85801E+00_JPRB, 0.74117E+00_JPRB, 0.68356E+00_JPRB, 0.65284E+00_JPRB ,&
&  0.61915E+00_JPRB, 0.59210E+00_JPRB, 0.58968E+00_JPRB, 0.58544E+00_JPRB, 0.58921E+00_JPRB, 0.64068E+00_JPRB, 0.71346E+00_JPRB ,&
&  0.86177E+00_JPRB, 0.11126E+01_JPRB, 0.14695E+01_JPRB, 0.16985E+01_JPRB, 0.13453E+01_JPRB, 0.12589E+01_JPRB, 0.12828E+01_JPRB ,&
&  0.15271E+01_JPRB, 0.12420E+01_JPRB, 0.95183E+00_JPRB, 0.78694E+00_JPRB, 0.69129E+00_JPRB, 0.66191E+00_JPRB, 0.62872E+00_JPRB ,&
&  0.61393E+00_JPRB, 0.59836E+00_JPRB, 0.58564E+00_JPRB, 0.56913E+00_JPRB, 0.55844E+00_JPRB, 0.54956E+00_JPRB, 0.54692E+00_JPRB ,&
&  0.54964E+00_JPRB, 0.55362E+00_JPRB, 0.55747E+00_JPRB, 0.56467E+00_JPRB, 0.57621E+00_JPRB, 0.58479E+00_JPRB, 0.58796E+00_JPRB ,&
&  0.58955E+00_JPRB, 0.59761E+00_JPRB, 0.58799E+00_JPRB, 0.59200E+00_JPRB, 0.60033E+00_JPRB, 0.59871E+00_JPRB, 0.57755E+00_JPRB ,&
&  0.56826E+00_JPRB, 0.57907E+00_JPRB, 0.58175E+00_JPRB, 0.57824E+00_JPRB, 0.58803E+00_JPRB, 0.59860E+00_JPRB, 0.57812E+00_JPRB ,&
&  0.58167E+00_JPRB, 0.58847E+00_JPRB, 0.59799E+00_JPRB, 0.58386E+00_JPRB, 0.56841E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.55128E+00_JPRB, 0.53816E+00_JPRB, 0.52400E+00_JPRB, 0.50922E+00_JPRB, 0.49299E+00_JPRB, 0.48114E+00_JPRB, 0.47506E+00_JPRB ,&
&  0.46469E+00_JPRB, 0.45546E+00_JPRB, 0.44867E+00_JPRB, 0.44577E+00_JPRB, 0.44352E+00_JPRB, 0.43588E+00_JPRB, 0.42125E+00_JPRB ,&
&  0.40650E+00_JPRB, 0.39706E+00_JPRB, 0.39233E+00_JPRB, 0.38728E+00_JPRB, 0.38115E+00_JPRB, 0.37741E+00_JPRB, 0.37791E+00_JPRB ,&
&  0.38372E+00_JPRB, 0.39313E+00_JPRB, 0.41159E+00_JPRB, 0.43151E+00_JPRB, 0.44576E+00_JPRB, 0.45711E+00_JPRB, 0.46386E+00_JPRB ,&
&  0.53048E+00_JPRB, 0.62851E+00_JPRB, 0.70252E+00_JPRB, 0.11168E+01_JPRB, 0.11820E+01_JPRB, 0.11495E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.11178E+01_JPRB, 0.11761E+01_JPRB, 0.10499E+01_JPRB, 0.91669E+00_JPRB, 0.79479E+00_JPRB, 0.70943E+00_JPRB, 0.64857E+00_JPRB ,&
&  0.60419E+00_JPRB, 0.56729E+00_JPRB, 0.54768E+00_JPRB, 0.53811E+00_JPRB, 0.53572E+00_JPRB, 0.56193E+00_JPRB, 0.63124E+00_JPRB ,&
&  0.73678E+00_JPRB, 0.91581E+00_JPRB, 0.11752E+01_JPRB, 0.13507E+01_JPRB, 0.12985E+01_JPRB, 0.13225E+01_JPRB, 0.14766E+01_JPRB ,&
&  0.18752E+01_JPRB, 0.16486E+01_JPRB, 0.11232E+01_JPRB, 0.83925E+00_JPRB, 0.71037E+00_JPRB, 0.65852E+00_JPRB, 0.61897E+00_JPRB ,&
&  0.60109E+00_JPRB, 0.58710E+00_JPRB, 0.57623E+00_JPRB, 0.56628E+00_JPRB, 0.55773E+00_JPRB, 0.55520E+00_JPRB, 0.55307E+00_JPRB ,&
&  0.55112E+00_JPRB, 0.55078E+00_JPRB, 0.55031E+00_JPRB, 0.55050E+00_JPRB, 0.55463E+00_JPRB, 0.56348E+00_JPRB, 0.57333E+00_JPRB ,&
&  0.57959E+00_JPRB, 0.58742E+00_JPRB, 0.57762E+00_JPRB, 0.56893E+00_JPRB, 0.57321E+00_JPRB, 0.57122E+00_JPRB, 0.55630E+00_JPRB ,&
&  0.55895E+00_JPRB, 0.56874E+00_JPRB, 0.56864E+00_JPRB, 0.55623E+00_JPRB, 0.56499E+00_JPRB, 0.58600E+00_JPRB, 0.56189E+00_JPRB ,&
&  0.55731E+00_JPRB, 0.56035E+00_JPRB, 0.56703E+00_JPRB, 0.56274E+00_JPRB, 0.55737E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.54235E+00_JPRB, 0.53611E+00_JPRB, 0.52311E+00_JPRB, 0.50748E+00_JPRB, 0.49151E+00_JPRB, 0.47809E+00_JPRB, 0.47169E+00_JPRB ,&
&  0.46420E+00_JPRB, 0.45640E+00_JPRB, 0.44926E+00_JPRB, 0.44561E+00_JPRB, 0.44428E+00_JPRB, 0.43718E+00_JPRB, 0.42409E+00_JPRB ,&
&  0.41020E+00_JPRB, 0.39731E+00_JPRB, 0.38707E+00_JPRB, 0.38188E+00_JPRB, 0.37608E+00_JPRB, 0.37300E+00_JPRB, 0.37257E+00_JPRB ,&
&  0.37355E+00_JPRB, 0.37919E+00_JPRB, 0.39135E+00_JPRB, 0.41141E+00_JPRB, 0.42681E+00_JPRB, 0.43284E+00_JPRB, 0.43667E+00_JPRB ,&
&  0.47542E+00_JPRB, 0.66115E+00_JPRB, 0.63934E+00_JPRB, 0.87160E+00_JPRB, 0.93181E+00_JPRB, 0.97236E+00_JPRB, 0.10210E+01_JPRB ,&
&  0.10452E+01_JPRB, 0.10711E+01_JPRB, 0.10177E+01_JPRB, 0.93050E+00_JPRB, 0.82558E+00_JPRB, 0.74691E+00_JPRB, 0.68104E+00_JPRB ,&
&  0.63212E+00_JPRB, 0.59540E+00_JPRB, 0.56678E+00_JPRB, 0.54311E+00_JPRB, 0.52800E+00_JPRB, 0.53128E+00_JPRB, 0.58508E+00_JPRB ,&
&  0.70671E+00_JPRB, 0.85598E+00_JPRB, 0.10242E+01_JPRB, 0.12095E+01_JPRB, 0.11596E+01_JPRB, 0.13033E+01_JPRB, 0.17185E+01_JPRB ,&
&  0.20621E+01_JPRB, 0.20751E+01_JPRB, 0.14098E+01_JPRB, 0.95565E+00_JPRB, 0.76059E+00_JPRB, 0.66833E+00_JPRB, 0.61264E+00_JPRB ,&
&  0.58111E+00_JPRB, 0.56466E+00_JPRB, 0.55573E+00_JPRB, 0.55081E+00_JPRB, 0.54961E+00_JPRB, 0.55254E+00_JPRB, 0.55229E+00_JPRB ,&
&  0.55250E+00_JPRB, 0.55229E+00_JPRB, 0.55227E+00_JPRB, 0.55023E+00_JPRB, 0.54581E+00_JPRB, 0.54622E+00_JPRB, 0.55226E+00_JPRB ,&
&  0.56196E+00_JPRB, 0.57034E+00_JPRB, 0.56579E+00_JPRB, 0.55959E+00_JPRB, 0.55885E+00_JPRB, 0.55657E+00_JPRB, 0.54820E+00_JPRB ,&
&  0.55433E+00_JPRB, 0.55732E+00_JPRB, 0.55620E+00_JPRB, 0.55628E+00_JPRB, 0.55352E+00_JPRB, 0.57902E+00_JPRB, 0.55240E+00_JPRB ,&
&  0.54534E+00_JPRB, 0.54709E+00_JPRB, 0.54964E+00_JPRB, 0.54452E+00_JPRB, 0.54298E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.52803E+00_JPRB, 0.52243E+00_JPRB, 0.50957E+00_JPRB, 0.49465E+00_JPRB, 0.48369E+00_JPRB, 0.47217E+00_JPRB, 0.46344E+00_JPRB ,&
&  0.45694E+00_JPRB, 0.45161E+00_JPRB, 0.44773E+00_JPRB, 0.44294E+00_JPRB, 0.43636E+00_JPRB, 0.42718E+00_JPRB, 0.41815E+00_JPRB ,&
&  0.40853E+00_JPRB, 0.39909E+00_JPRB, 0.38988E+00_JPRB, 0.38032E+00_JPRB, 0.37309E+00_JPRB, 0.36800E+00_JPRB, 0.36447E+00_JPRB ,&
&  0.36406E+00_JPRB, 0.36684E+00_JPRB, 0.37442E+00_JPRB, 0.39061E+00_JPRB, 0.40892E+00_JPRB, 0.41898E+00_JPRB, 0.42901E+00_JPRB ,&
&  0.47552E+00_JPRB, 0.42029E+00_JPRB, 0.69007E+00_JPRB, 0.75486E+00_JPRB, 0.73722E+00_JPRB, 0.75154E+00_JPRB, 0.85853E+00_JPRB ,&
&  0.93989E+00_JPRB, 0.96559E+00_JPRB, 0.96510E+00_JPRB, 0.92437E+00_JPRB, 0.84349E+00_JPRB, 0.78159E+00_JPRB, 0.73309E+00_JPRB ,&
&  0.69588E+00_JPRB, 0.66453E+00_JPRB, 0.62705E+00_JPRB, 0.58885E+00_JPRB, 0.56025E+00_JPRB, 0.55484E+00_JPRB, 0.58420E+00_JPRB ,&
&  0.71438E+00_JPRB, 0.83429E+00_JPRB, 0.92724E+00_JPRB, 0.10456E+01_JPRB, 0.10710E+01_JPRB, 0.12463E+01_JPRB, 0.14750E+01_JPRB ,&
&  0.20250E+01_JPRB, 0.21075E+01_JPRB, 0.15963E+01_JPRB, 0.11585E+01_JPRB, 0.88332E+00_JPRB, 0.73767E+00_JPRB, 0.63915E+00_JPRB ,&
&  0.58640E+00_JPRB, 0.55626E+00_JPRB, 0.53823E+00_JPRB, 0.53325E+00_JPRB, 0.53974E+00_JPRB, 0.55192E+00_JPRB, 0.56141E+00_JPRB ,&
&  0.57342E+00_JPRB, 0.58214E+00_JPRB, 0.58281E+00_JPRB, 0.57828E+00_JPRB, 0.56394E+00_JPRB, 0.54537E+00_JPRB, 0.53542E+00_JPRB ,&
&  0.53671E+00_JPRB, 0.54534E+00_JPRB, 0.54718E+00_JPRB, 0.54386E+00_JPRB, 0.54544E+00_JPRB, 0.54626E+00_JPRB, 0.54474E+00_JPRB ,&
&  0.54568E+00_JPRB, 0.54774E+00_JPRB, 0.54696E+00_JPRB, 0.55244E+00_JPRB, 0.54030E+00_JPRB, 0.56649E+00_JPRB, 0.55051E+00_JPRB ,&
&  0.54155E+00_JPRB, 0.53774E+00_JPRB, 0.53604E+00_JPRB, 0.53090E+00_JPRB, 0.52708E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.51418E+00_JPRB, 0.50754E+00_JPRB, 0.49365E+00_JPRB, 0.47839E+00_JPRB, 0.46814E+00_JPRB, 0.46027E+00_JPRB, 0.45464E+00_JPRB ,&
&  0.45138E+00_JPRB, 0.44473E+00_JPRB, 0.43862E+00_JPRB, 0.43005E+00_JPRB, 0.41921E+00_JPRB, 0.41009E+00_JPRB, 0.40457E+00_JPRB ,&
&  0.39733E+00_JPRB, 0.39368E+00_JPRB, 0.38942E+00_JPRB, 0.38071E+00_JPRB, 0.37115E+00_JPRB, 0.36322E+00_JPRB, 0.35809E+00_JPRB ,&
&  0.35587E+00_JPRB, 0.35824E+00_JPRB, 0.36384E+00_JPRB, 0.37644E+00_JPRB, 0.38779E+00_JPRB, 0.39916E+00_JPRB, 0.41728E+00_JPRB ,&
&  0.47591E+00_JPRB, 0.61907E+00_JPRB, 0.73753E+00_JPRB, 0.67536E+00_JPRB, 0.63724E+00_JPRB, 0.61193E+00_JPRB, 0.68464E+00_JPRB ,&
&  0.75249E+00_JPRB, 0.82310E+00_JPRB, 0.86179E+00_JPRB, 0.87320E+00_JPRB, 0.83240E+00_JPRB, 0.80039E+00_JPRB, 0.76760E+00_JPRB ,&
&  0.73273E+00_JPRB, 0.69429E+00_JPRB, 0.64449E+00_JPRB, 0.59472E+00_JPRB, 0.56709E+00_JPRB, 0.57005E+00_JPRB, 0.61008E+00_JPRB ,&
&  0.70903E+00_JPRB, 0.80803E+00_JPRB, 0.85259E+00_JPRB, 0.95003E+00_JPRB, 0.11002E+01_JPRB, 0.11233E+01_JPRB, 0.14645E+01_JPRB ,&
&  0.19048E+01_JPRB, 0.17261E+01_JPRB, 0.14610E+01_JPRB, 0.11802E+01_JPRB, 0.95903E+00_JPRB, 0.78167E+00_JPRB, 0.67946E+00_JPRB ,&
&  0.61166E+00_JPRB, 0.56802E+00_JPRB, 0.53815E+00_JPRB, 0.52350E+00_JPRB, 0.53326E+00_JPRB, 0.55892E+00_JPRB, 0.58036E+00_JPRB ,&
&  0.60905E+00_JPRB, 0.63906E+00_JPRB, 0.65522E+00_JPRB, 0.65247E+00_JPRB, 0.62995E+00_JPRB, 0.59526E+00_JPRB, 0.56190E+00_JPRB ,&
&  0.54153E+00_JPRB, 0.53229E+00_JPRB, 0.53309E+00_JPRB, 0.52791E+00_JPRB, 0.53260E+00_JPRB, 0.54323E+00_JPRB, 0.54271E+00_JPRB ,&
&  0.53833E+00_JPRB, 0.53977E+00_JPRB, 0.55001E+00_JPRB, 0.54517E+00_JPRB, 0.53799E+00_JPRB, 0.56675E+00_JPRB, 0.55275E+00_JPRB ,&
&  0.54041E+00_JPRB, 0.53012E+00_JPRB, 0.52570E+00_JPRB, 0.51405E+00_JPRB, 0.51238E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.51102E+00_JPRB, 0.50029E+00_JPRB, 0.47943E+00_JPRB, 0.46032E+00_JPRB, 0.44845E+00_JPRB, 0.44017E+00_JPRB, 0.43604E+00_JPRB ,&
&  0.43408E+00_JPRB, 0.42732E+00_JPRB, 0.42010E+00_JPRB, 0.40939E+00_JPRB, 0.39995E+00_JPRB, 0.39189E+00_JPRB, 0.38455E+00_JPRB ,&
&  0.37848E+00_JPRB, 0.37510E+00_JPRB, 0.37212E+00_JPRB, 0.36886E+00_JPRB, 0.36360E+00_JPRB, 0.35586E+00_JPRB, 0.35108E+00_JPRB ,&
&  0.34906E+00_JPRB, 0.34981E+00_JPRB, 0.35386E+00_JPRB, 0.36220E+00_JPRB, 0.37104E+00_JPRB, 0.38167E+00_JPRB, 0.39888E+00_JPRB ,&
&  0.44004E+00_JPRB, 0.53888E+00_JPRB, 0.56587E+00_JPRB, 0.56806E+00_JPRB, 0.56533E+00_JPRB, 0.56206E+00_JPRB, 0.59203E+00_JPRB ,&
&  0.64551E+00_JPRB, 0.69513E+00_JPRB, 0.73394E+00_JPRB, 0.74603E+00_JPRB, 0.72486E+00_JPRB, 0.70149E+00_JPRB, 0.67189E+00_JPRB ,&
&  0.64191E+00_JPRB, 0.60989E+00_JPRB, 0.57885E+00_JPRB, 0.55100E+00_JPRB, 0.53066E+00_JPRB, 0.53966E+00_JPRB, 0.57084E+00_JPRB ,&
&  0.63140E+00_JPRB, 0.68587E+00_JPRB, 0.72373E+00_JPRB, 0.80407E+00_JPRB, 0.97674E+00_JPRB, 0.10859E+01_JPRB, 0.12360E+01_JPRB ,&
&  0.12725E+01_JPRB, 0.12361E+01_JPRB, 0.11732E+01_JPRB, 0.10839E+01_JPRB, 0.94863E+00_JPRB, 0.81283E+00_JPRB, 0.70324E+00_JPRB ,&
&  0.62521E+00_JPRB, 0.57404E+00_JPRB, 0.54286E+00_JPRB, 0.52987E+00_JPRB, 0.53737E+00_JPRB, 0.56289E+00_JPRB, 0.59543E+00_JPRB ,&
&  0.63493E+00_JPRB, 0.67512E+00_JPRB, 0.70070E+00_JPRB, 0.71079E+00_JPRB, 0.70108E+00_JPRB, 0.67619E+00_JPRB, 0.63726E+00_JPRB ,&
&  0.60127E+00_JPRB, 0.57212E+00_JPRB, 0.55472E+00_JPRB, 0.54874E+00_JPRB, 0.54930E+00_JPRB, 0.55098E+00_JPRB, 0.55216E+00_JPRB ,&
&  0.55784E+00_JPRB, 0.56119E+00_JPRB, 0.55880E+00_JPRB, 0.54834E+00_JPRB, 0.53605E+00_JPRB, 0.55295E+00_JPRB, 0.54187E+00_JPRB ,&
&  0.53108E+00_JPRB, 0.51892E+00_JPRB, 0.51070E+00_JPRB, 0.49708E+00_JPRB, 0.50882E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.54804E+00_JPRB, 0.52166E+00_JPRB, 0.48580E+00_JPRB, 0.45781E+00_JPRB, 0.43820E+00_JPRB, 0.42370E+00_JPRB, 0.41547E+00_JPRB ,&
&  0.41265E+00_JPRB, 0.40372E+00_JPRB, 0.39085E+00_JPRB, 0.38001E+00_JPRB, 0.37448E+00_JPRB, 0.37016E+00_JPRB, 0.36441E+00_JPRB ,&
&  0.35967E+00_JPRB, 0.35543E+00_JPRB, 0.35258E+00_JPRB, 0.35139E+00_JPRB, 0.35071E+00_JPRB, 0.34952E+00_JPRB, 0.34604E+00_JPRB ,&
&  0.34077E+00_JPRB, 0.33978E+00_JPRB, 0.34132E+00_JPRB, 0.34776E+00_JPRB, 0.35638E+00_JPRB, 0.36664E+00_JPRB, 0.38507E+00_JPRB ,&
&  0.43613E+00_JPRB, 0.42605E+00_JPRB, 0.45766E+00_JPRB, 0.46669E+00_JPRB, 0.47432E+00_JPRB, 0.50214E+00_JPRB, 0.52991E+00_JPRB ,&
&  0.55901E+00_JPRB, 0.59667E+00_JPRB, 0.63591E+00_JPRB, 0.65076E+00_JPRB, 0.63378E+00_JPRB, 0.60385E+00_JPRB, 0.57668E+00_JPRB ,&
&  0.55173E+00_JPRB, 0.53464E+00_JPRB, 0.51924E+00_JPRB, 0.50679E+00_JPRB, 0.50450E+00_JPRB, 0.51552E+00_JPRB, 0.53463E+00_JPRB ,&
&  0.54936E+00_JPRB, 0.55478E+00_JPRB, 0.60646E+00_JPRB, 0.66991E+00_JPRB, 0.76341E+00_JPRB, 0.83356E+00_JPRB, 0.90435E+00_JPRB ,&
&  0.96479E+00_JPRB, 0.99984E+00_JPRB, 0.10202E+01_JPRB, 0.10079E+01_JPRB, 0.94271E+00_JPRB, 0.85924E+00_JPRB, 0.76962E+00_JPRB ,&
&  0.68527E+00_JPRB, 0.62864E+00_JPRB, 0.59907E+00_JPRB, 0.58533E+00_JPRB, 0.58378E+00_JPRB, 0.58971E+00_JPRB, 0.60449E+00_JPRB ,&
&  0.62760E+00_JPRB, 0.64989E+00_JPRB, 0.66806E+00_JPRB, 0.67642E+00_JPRB, 0.67392E+00_JPRB, 0.66920E+00_JPRB, 0.66144E+00_JPRB ,&
&  0.65107E+00_JPRB, 0.63651E+00_JPRB, 0.61808E+00_JPRB, 0.59900E+00_JPRB, 0.58283E+00_JPRB, 0.57234E+00_JPRB, 0.56477E+00_JPRB ,&
&  0.55782E+00_JPRB, 0.55640E+00_JPRB, 0.54019E+00_JPRB, 0.53550E+00_JPRB, 0.55045E+00_JPRB, 0.54049E+00_JPRB, 0.52803E+00_JPRB ,&
&  0.51196E+00_JPRB, 0.49868E+00_JPRB, 0.48660E+00_JPRB, 0.47286E+00_JPRB, 0.53015E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.46212E+00_JPRB, 0.46196E+00_JPRB, 0.44908E+00_JPRB, 0.43066E+00_JPRB, 0.41667E+00_JPRB, 0.40379E+00_JPRB, 0.39277E+00_JPRB ,&
&  0.38551E+00_JPRB, 0.37696E+00_JPRB, 0.36685E+00_JPRB, 0.35924E+00_JPRB, 0.35794E+00_JPRB, 0.35548E+00_JPRB, 0.35186E+00_JPRB ,&
&  0.34526E+00_JPRB, 0.33894E+00_JPRB, 0.33465E+00_JPRB, 0.33313E+00_JPRB, 0.33368E+00_JPRB, 0.33549E+00_JPRB, 0.33645E+00_JPRB ,&
&  0.33475E+00_JPRB, 0.33115E+00_JPRB, 0.33124E+00_JPRB, 0.33365E+00_JPRB, 0.33876E+00_JPRB, 0.34677E+00_JPRB, 0.36552E+00_JPRB ,&
&  0.40838E+00_JPRB, 0.38539E+00_JPRB, 0.40297E+00_JPRB, 0.41340E+00_JPRB, 0.42559E+00_JPRB, 0.45424E+00_JPRB, 0.48272E+00_JPRB ,&
&  0.50179E+00_JPRB, 0.51455E+00_JPRB, 0.53893E+00_JPRB, 0.55996E+00_JPRB, 0.55687E+00_JPRB, 0.53931E+00_JPRB, 0.51621E+00_JPRB ,&
&  0.49287E+00_JPRB, 0.47556E+00_JPRB, 0.46153E+00_JPRB, 0.45179E+00_JPRB, 0.45065E+00_JPRB, 0.45688E+00_JPRB, 0.46296E+00_JPRB ,&
&  0.45714E+00_JPRB, 0.46191E+00_JPRB, 0.49426E+00_JPRB, 0.54346E+00_JPRB, 0.60758E+00_JPRB, 0.67239E+00_JPRB, 0.73921E+00_JPRB ,&
&  0.80047E+00_JPRB, 0.84462E+00_JPRB, 0.87760E+00_JPRB, 0.90034E+00_JPRB, 0.89339E+00_JPRB, 0.87370E+00_JPRB, 0.82036E+00_JPRB ,&
&  0.75683E+00_JPRB, 0.70345E+00_JPRB, 0.66790E+00_JPRB, 0.63977E+00_JPRB, 0.61892E+00_JPRB, 0.60541E+00_JPRB, 0.59804E+00_JPRB ,&
&  0.59698E+00_JPRB, 0.59963E+00_JPRB, 0.59975E+00_JPRB, 0.59692E+00_JPRB, 0.59343E+00_JPRB, 0.59133E+00_JPRB, 0.58888E+00_JPRB ,&
&  0.58702E+00_JPRB, 0.58517E+00_JPRB, 0.57996E+00_JPRB, 0.56915E+00_JPRB, 0.55409E+00_JPRB, 0.53952E+00_JPRB, 0.52637E+00_JPRB ,&
&  0.51656E+00_JPRB, 0.51564E+00_JPRB, 0.51147E+00_JPRB, 0.51491E+00_JPRB, 0.50620E+00_JPRB, 0.49773E+00_JPRB, 0.48930E+00_JPRB ,&
&  0.47911E+00_JPRB, 0.46840E+00_JPRB, 0.45360E+00_JPRB, 0.43893E+00_JPRB, 0.44005E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.41063E+00_JPRB, 0.41776E+00_JPRB, 0.41800E+00_JPRB, 0.40682E+00_JPRB, 0.39401E+00_JPRB, 0.38639E+00_JPRB, 0.37707E+00_JPRB ,&
&  0.36672E+00_JPRB, 0.35812E+00_JPRB, 0.34785E+00_JPRB, 0.34384E+00_JPRB, 0.34144E+00_JPRB, 0.34041E+00_JPRB, 0.33872E+00_JPRB ,&
&  0.33452E+00_JPRB, 0.32786E+00_JPRB, 0.32263E+00_JPRB, 0.32036E+00_JPRB, 0.32088E+00_JPRB, 0.32231E+00_JPRB, 0.32392E+00_JPRB ,&
&  0.32317E+00_JPRB, 0.32264E+00_JPRB, 0.32183E+00_JPRB, 0.32174E+00_JPRB, 0.32342E+00_JPRB, 0.33108E+00_JPRB, 0.34628E+00_JPRB ,&
&  0.38391E+00_JPRB, 0.34976E+00_JPRB, 0.35953E+00_JPRB, 0.37969E+00_JPRB, 0.40021E+00_JPRB, 0.42723E+00_JPRB, 0.45277E+00_JPRB ,&
&  0.46400E+00_JPRB, 0.46114E+00_JPRB, 0.46668E+00_JPRB, 0.48009E+00_JPRB, 0.47839E+00_JPRB, 0.47211E+00_JPRB, 0.46135E+00_JPRB ,&
&  0.44510E+00_JPRB, 0.42656E+00_JPRB, 0.40833E+00_JPRB, 0.39761E+00_JPRB, 0.39730E+00_JPRB, 0.40139E+00_JPRB, 0.40281E+00_JPRB ,&
&  0.39279E+00_JPRB, 0.38900E+00_JPRB, 0.40289E+00_JPRB, 0.42490E+00_JPRB, 0.45509E+00_JPRB, 0.50315E+00_JPRB, 0.56599E+00_JPRB ,&
&  0.61467E+00_JPRB, 0.66484E+00_JPRB, 0.71498E+00_JPRB, 0.75516E+00_JPRB, 0.79062E+00_JPRB, 0.80053E+00_JPRB, 0.77739E+00_JPRB ,&
&  0.74155E+00_JPRB, 0.69949E+00_JPRB, 0.65977E+00_JPRB, 0.62357E+00_JPRB, 0.59402E+00_JPRB, 0.57546E+00_JPRB, 0.56366E+00_JPRB ,&
&  0.55518E+00_JPRB, 0.54849E+00_JPRB, 0.53786E+00_JPRB, 0.52340E+00_JPRB, 0.50897E+00_JPRB, 0.49716E+00_JPRB, 0.48673E+00_JPRB ,&
&  0.47752E+00_JPRB, 0.46818E+00_JPRB, 0.46134E+00_JPRB, 0.45768E+00_JPRB, 0.45412E+00_JPRB, 0.45198E+00_JPRB, 0.45096E+00_JPRB ,&
&  0.44964E+00_JPRB, 0.45278E+00_JPRB, 0.45253E+00_JPRB, 0.45363E+00_JPRB, 0.45946E+00_JPRB, 0.45506E+00_JPRB, 0.45174E+00_JPRB ,&
&  0.44377E+00_JPRB, 0.43480E+00_JPRB, 0.42547E+00_JPRB, 0.38768E+00_JPRB, 0.40403E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.37630E+00_JPRB, 0.38222E+00_JPRB, 0.38306E+00_JPRB, 0.37826E+00_JPRB, 0.37005E+00_JPRB, 0.36435E+00_JPRB, 0.35535E+00_JPRB ,&
&  0.34540E+00_JPRB, 0.33965E+00_JPRB, 0.33294E+00_JPRB, 0.33016E+00_JPRB, 0.33019E+00_JPRB, 0.32672E+00_JPRB, 0.32384E+00_JPRB ,&
&  0.32135E+00_JPRB, 0.31681E+00_JPRB, 0.31392E+00_JPRB, 0.31123E+00_JPRB, 0.31098E+00_JPRB, 0.31117E+00_JPRB, 0.31113E+00_JPRB ,&
&  0.30992E+00_JPRB, 0.30765E+00_JPRB, 0.30577E+00_JPRB, 0.30478E+00_JPRB, 0.30719E+00_JPRB, 0.31416E+00_JPRB, 0.32737E+00_JPRB ,&
&  0.35561E+00_JPRB, 0.31964E+00_JPRB, 0.33196E+00_JPRB, 0.34727E+00_JPRB, 0.37235E+00_JPRB, 0.38900E+00_JPRB, 0.39563E+00_JPRB ,&
&  0.40338E+00_JPRB, 0.41153E+00_JPRB, 0.41573E+00_JPRB, 0.41917E+00_JPRB, 0.41406E+00_JPRB, 0.40646E+00_JPRB, 0.40092E+00_JPRB ,&
&  0.39381E+00_JPRB, 0.38502E+00_JPRB, 0.37621E+00_JPRB, 0.36747E+00_JPRB, 0.36423E+00_JPRB, 0.36511E+00_JPRB, 0.36268E+00_JPRB ,&
&  0.35494E+00_JPRB, 0.34777E+00_JPRB, 0.34401E+00_JPRB, 0.34648E+00_JPRB, 0.35630E+00_JPRB, 0.37405E+00_JPRB, 0.40915E+00_JPRB ,&
&  0.44841E+00_JPRB, 0.49214E+00_JPRB, 0.54058E+00_JPRB, 0.58214E+00_JPRB, 0.61532E+00_JPRB, 0.63201E+00_JPRB, 0.63587E+00_JPRB ,&
&  0.62611E+00_JPRB, 0.61095E+00_JPRB, 0.59571E+00_JPRB, 0.57598E+00_JPRB, 0.55075E+00_JPRB, 0.52727E+00_JPRB, 0.50835E+00_JPRB ,&
&  0.49361E+00_JPRB, 0.47845E+00_JPRB, 0.46170E+00_JPRB, 0.44380E+00_JPRB, 0.42530E+00_JPRB, 0.40875E+00_JPRB, 0.39496E+00_JPRB ,&
&  0.38342E+00_JPRB, 0.37340E+00_JPRB, 0.36739E+00_JPRB, 0.36600E+00_JPRB, 0.36921E+00_JPRB, 0.37399E+00_JPRB, 0.37785E+00_JPRB ,&
&  0.38267E+00_JPRB, 0.38983E+00_JPRB, 0.39546E+00_JPRB, 0.40588E+00_JPRB, 0.40839E+00_JPRB, 0.40824E+00_JPRB, 0.40825E+00_JPRB ,&
&  0.40501E+00_JPRB, 0.39836E+00_JPRB, 0.35013E+00_JPRB, 0.36851E+00_JPRB, 0.37173E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.35216E+00_JPRB, 0.34983E+00_JPRB, 0.34710E+00_JPRB, 0.34880E+00_JPRB, 0.34761E+00_JPRB, 0.34402E+00_JPRB, 0.33788E+00_JPRB ,&
&  0.32817E+00_JPRB, 0.32137E+00_JPRB, 0.31616E+00_JPRB, 0.31460E+00_JPRB, 0.31702E+00_JPRB, 0.31538E+00_JPRB, 0.31145E+00_JPRB ,&
&  0.30844E+00_JPRB, 0.30501E+00_JPRB, 0.30255E+00_JPRB, 0.29961E+00_JPRB, 0.29726E+00_JPRB, 0.29636E+00_JPRB, 0.29585E+00_JPRB ,&
&  0.29446E+00_JPRB, 0.29219E+00_JPRB, 0.29039E+00_JPRB, 0.28985E+00_JPRB, 0.29289E+00_JPRB, 0.29936E+00_JPRB, 0.30952E+00_JPRB ,&
&  0.30669E+00_JPRB, 0.28878E+00_JPRB, 0.30541E+00_JPRB, 0.32599E+00_JPRB, 0.33908E+00_JPRB, 0.35047E+00_JPRB, 0.35879E+00_JPRB ,&
&  0.36630E+00_JPRB, 0.36842E+00_JPRB, 0.36992E+00_JPRB, 0.37017E+00_JPRB, 0.36823E+00_JPRB, 0.36343E+00_JPRB, 0.35844E+00_JPRB ,&
&  0.35515E+00_JPRB, 0.35300E+00_JPRB, 0.35050E+00_JPRB, 0.34557E+00_JPRB, 0.34062E+00_JPRB, 0.33604E+00_JPRB, 0.33170E+00_JPRB ,&
&  0.32744E+00_JPRB, 0.32303E+00_JPRB, 0.31989E+00_JPRB, 0.31919E+00_JPRB, 0.32149E+00_JPRB, 0.32612E+00_JPRB, 0.33474E+00_JPRB ,&
&  0.34542E+00_JPRB, 0.36375E+00_JPRB, 0.39613E+00_JPRB, 0.42902E+00_JPRB, 0.45531E+00_JPRB, 0.47360E+00_JPRB, 0.48856E+00_JPRB ,&
&  0.49330E+00_JPRB, 0.49228E+00_JPRB, 0.48950E+00_JPRB, 0.48309E+00_JPRB, 0.47265E+00_JPRB, 0.45809E+00_JPRB, 0.44045E+00_JPRB ,&
&  0.42454E+00_JPRB, 0.41048E+00_JPRB, 0.39700E+00_JPRB, 0.38410E+00_JPRB, 0.37138E+00_JPRB, 0.35935E+00_JPRB, 0.34897E+00_JPRB ,&
&  0.33984E+00_JPRB, 0.33149E+00_JPRB, 0.32557E+00_JPRB, 0.32297E+00_JPRB, 0.32455E+00_JPRB, 0.32859E+00_JPRB, 0.33420E+00_JPRB ,&
&  0.33973E+00_JPRB, 0.34751E+00_JPRB, 0.35537E+00_JPRB, 0.36402E+00_JPRB, 0.37052E+00_JPRB, 0.37142E+00_JPRB, 0.37174E+00_JPRB ,&
&  0.37087E+00_JPRB, 0.36614E+00_JPRB, 0.36236E+00_JPRB, 0.35677E+00_JPRB, 0.35244E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.32500E+00_JPRB, 0.32109E+00_JPRB, 0.32101E+00_JPRB, 0.32452E+00_JPRB, 0.32803E+00_JPRB, 0.32721E+00_JPRB, 0.32311E+00_JPRB ,&
&  0.31418E+00_JPRB, 0.30630E+00_JPRB, 0.29988E+00_JPRB, 0.29683E+00_JPRB, 0.29823E+00_JPRB, 0.29838E+00_JPRB, 0.29601E+00_JPRB ,&
&  0.29347E+00_JPRB, 0.28891E+00_JPRB, 0.28510E+00_JPRB, 0.28130E+00_JPRB, 0.27854E+00_JPRB, 0.27734E+00_JPRB, 0.27694E+00_JPRB ,&
&  0.27570E+00_JPRB, 0.27434E+00_JPRB, 0.27443E+00_JPRB, 0.27492E+00_JPRB, 0.27710E+00_JPRB, 0.28168E+00_JPRB, 0.28707E+00_JPRB ,&
&  0.29586E+00_JPRB, 0.28080E+00_JPRB, 0.29720E+00_JPRB, 0.30210E+00_JPRB, 0.30611E+00_JPRB, 0.31280E+00_JPRB, 0.32046E+00_JPRB ,&
&  0.32742E+00_JPRB, 0.33183E+00_JPRB, 0.33414E+00_JPRB, 0.33401E+00_JPRB, 0.33184E+00_JPRB, 0.32973E+00_JPRB, 0.32568E+00_JPRB ,&
&  0.32078E+00_JPRB, 0.31886E+00_JPRB, 0.31801E+00_JPRB, 0.31601E+00_JPRB, 0.31388E+00_JPRB, 0.31090E+00_JPRB, 0.30799E+00_JPRB ,&
&  0.30554E+00_JPRB, 0.30294E+00_JPRB, 0.30084E+00_JPRB, 0.29874E+00_JPRB, 0.29744E+00_JPRB, 0.29798E+00_JPRB, 0.30053E+00_JPRB ,&
&  0.30337E+00_JPRB, 0.30792E+00_JPRB, 0.31810E+00_JPRB, 0.33258E+00_JPRB, 0.34853E+00_JPRB, 0.36286E+00_JPRB, 0.37482E+00_JPRB ,&
&  0.38237E+00_JPRB, 0.38652E+00_JPRB, 0.38728E+00_JPRB, 0.38588E+00_JPRB, 0.38183E+00_JPRB, 0.37647E+00_JPRB, 0.36851E+00_JPRB ,&
&  0.36070E+00_JPRB, 0.35340E+00_JPRB, 0.34569E+00_JPRB, 0.33830E+00_JPRB, 0.33021E+00_JPRB, 0.32147E+00_JPRB, 0.31347E+00_JPRB ,&
&  0.30711E+00_JPRB, 0.30255E+00_JPRB, 0.29930E+00_JPRB, 0.29751E+00_JPRB, 0.29677E+00_JPRB, 0.29824E+00_JPRB, 0.30072E+00_JPRB ,&
&  0.30437E+00_JPRB, 0.30950E+00_JPRB, 0.31605E+00_JPRB, 0.32267E+00_JPRB, 0.32937E+00_JPRB, 0.33287E+00_JPRB, 0.33564E+00_JPRB ,&
&  0.33602E+00_JPRB, 0.33371E+00_JPRB, 0.33200E+00_JPRB, 0.32980E+00_JPRB, 0.32787E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.29939E+00_JPRB, 0.29811E+00_JPRB, 0.30069E+00_JPRB, 0.30524E+00_JPRB, 0.31123E+00_JPRB, 0.31004E+00_JPRB, 0.30763E+00_JPRB ,&
&  0.30236E+00_JPRB, 0.29555E+00_JPRB, 0.28896E+00_JPRB, 0.28284E+00_JPRB, 0.28005E+00_JPRB, 0.28041E+00_JPRB, 0.28014E+00_JPRB ,&
&  0.27890E+00_JPRB, 0.27580E+00_JPRB, 0.27139E+00_JPRB, 0.26707E+00_JPRB, 0.26395E+00_JPRB, 0.26229E+00_JPRB, 0.26088E+00_JPRB ,&
&  0.26008E+00_JPRB, 0.25900E+00_JPRB, 0.25997E+00_JPRB, 0.26009E+00_JPRB, 0.26100E+00_JPRB, 0.26494E+00_JPRB, 0.26805E+00_JPRB ,&
&  0.26958E+00_JPRB, 0.27137E+00_JPRB, 0.27127E+00_JPRB, 0.27192E+00_JPRB, 0.27517E+00_JPRB, 0.28249E+00_JPRB, 0.29116E+00_JPRB ,&
&  0.29871E+00_JPRB, 0.30389E+00_JPRB, 0.30622E+00_JPRB, 0.30520E+00_JPRB, 0.30258E+00_JPRB, 0.29996E+00_JPRB, 0.29808E+00_JPRB ,&
&  0.29610E+00_JPRB, 0.29247E+00_JPRB, 0.28859E+00_JPRB, 0.28536E+00_JPRB, 0.28391E+00_JPRB, 0.28286E+00_JPRB, 0.28152E+00_JPRB ,&
&  0.27996E+00_JPRB, 0.27846E+00_JPRB, 0.27604E+00_JPRB, 0.27316E+00_JPRB, 0.27059E+00_JPRB, 0.26978E+00_JPRB, 0.27096E+00_JPRB ,&
&  0.27362E+00_JPRB, 0.27721E+00_JPRB, 0.28227E+00_JPRB, 0.28875E+00_JPRB, 0.29507E+00_JPRB, 0.30219E+00_JPRB, 0.30812E+00_JPRB ,&
&  0.31243E+00_JPRB, 0.31472E+00_JPRB, 0.31540E+00_JPRB, 0.31514E+00_JPRB, 0.31396E+00_JPRB, 0.31187E+00_JPRB, 0.30936E+00_JPRB ,&
&  0.30687E+00_JPRB, 0.30418E+00_JPRB, 0.30124E+00_JPRB, 0.29795E+00_JPRB, 0.29331E+00_JPRB, 0.28828E+00_JPRB, 0.28427E+00_JPRB ,&
&  0.28160E+00_JPRB, 0.27982E+00_JPRB, 0.27857E+00_JPRB, 0.27736E+00_JPRB, 0.27668E+00_JPRB, 0.27697E+00_JPRB, 0.27848E+00_JPRB ,&
&  0.28129E+00_JPRB, 0.28521E+00_JPRB, 0.29000E+00_JPRB, 0.29512E+00_JPRB, 0.29897E+00_JPRB, 0.30232E+00_JPRB, 0.30560E+00_JPRB ,&
&  0.30690E+00_JPRB, 0.30545E+00_JPRB, 0.30439E+00_JPRB, 0.30390E+00_JPRB, 0.30202E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.27745E+00_JPRB, 0.27833E+00_JPRB, 0.28289E+00_JPRB, 0.28781E+00_JPRB, 0.29279E+00_JPRB, 0.29394E+00_JPRB, 0.29153E+00_JPRB ,&
&  0.28880E+00_JPRB, 0.28509E+00_JPRB, 0.27964E+00_JPRB, 0.27292E+00_JPRB, 0.26683E+00_JPRB, 0.26515E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26543E+00_JPRB, 0.26398E+00_JPRB, 0.26087E+00_JPRB, 0.25737E+00_JPRB, 0.25328E+00_JPRB, 0.24971E+00_JPRB, 0.24704E+00_JPRB ,&
&  0.24627E+00_JPRB, 0.24592E+00_JPRB, 0.24648E+00_JPRB, 0.24683E+00_JPRB, 0.24776E+00_JPRB, 0.25072E+00_JPRB, 0.25602E+00_JPRB ,&
&  0.25367E+00_JPRB, 0.25409E+00_JPRB, 0.24783E+00_JPRB, 0.25036E+00_JPRB, 0.25277E+00_JPRB, 0.25741E+00_JPRB, 0.26244E+00_JPRB ,&
&  0.26735E+00_JPRB, 0.27186E+00_JPRB, 0.27473E+00_JPRB, 0.27503E+00_JPRB, 0.27385E+00_JPRB, 0.27376E+00_JPRB, 0.27355E+00_JPRB ,&
&  0.27276E+00_JPRB, 0.27170E+00_JPRB, 0.26982E+00_JPRB, 0.26617E+00_JPRB, 0.26114E+00_JPRB, 0.25678E+00_JPRB, 0.25372E+00_JPRB ,&
&  0.25319E+00_JPRB, 0.25307E+00_JPRB, 0.25199E+00_JPRB, 0.25076E+00_JPRB, 0.24944E+00_JPRB, 0.24892E+00_JPRB, 0.24998E+00_JPRB ,&
&  0.25184E+00_JPRB, 0.25357E+00_JPRB, 0.25499E+00_JPRB, 0.25745E+00_JPRB, 0.26113E+00_JPRB, 0.26470E+00_JPRB, 0.26784E+00_JPRB ,&
&  0.27008E+00_JPRB, 0.27054E+00_JPRB, 0.27021E+00_JPRB, 0.26944E+00_JPRB, 0.26897E+00_JPRB, 0.26850E+00_JPRB, 0.26853E+00_JPRB ,&
&  0.26849E+00_JPRB, 0.26799E+00_JPRB, 0.26665E+00_JPRB, 0.26453E+00_JPRB, 0.26243E+00_JPRB, 0.26045E+00_JPRB, 0.25918E+00_JPRB ,&
&  0.25865E+00_JPRB, 0.25838E+00_JPRB, 0.25776E+00_JPRB, 0.25699E+00_JPRB, 0.25707E+00_JPRB, 0.25745E+00_JPRB, 0.25854E+00_JPRB ,&
&  0.26033E+00_JPRB, 0.26289E+00_JPRB, 0.26635E+00_JPRB, 0.27004E+00_JPRB, 0.27091E+00_JPRB, 0.27167E+00_JPRB, 0.27399E+00_JPRB ,&
&  0.27623E+00_JPRB, 0.27697E+00_JPRB, 0.27697E+00_JPRB, 0.27699E+00_JPRB, 0.27772E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.25927E+00_JPRB, 0.26173E+00_JPRB, 0.26577E+00_JPRB, 0.27168E+00_JPRB, 0.27492E+00_JPRB, 0.27798E+00_JPRB, 0.27765E+00_JPRB ,&
&  0.27520E+00_JPRB, 0.27179E+00_JPRB, 0.26773E+00_JPRB, 0.26273E+00_JPRB, 0.25749E+00_JPRB, 0.25350E+00_JPRB, 0.25207E+00_JPRB ,&
&  0.25198E+00_JPRB, 0.25110E+00_JPRB, 0.24870E+00_JPRB, 0.24539E+00_JPRB, 0.24169E+00_JPRB, 0.23874E+00_JPRB, 0.23644E+00_JPRB ,&
&  0.23570E+00_JPRB, 0.23600E+00_JPRB, 0.23689E+00_JPRB, 0.23738E+00_JPRB, 0.23768E+00_JPRB, 0.24005E+00_JPRB, 0.24386E+00_JPRB ,&
&  0.24174E+00_JPRB, 0.23934E+00_JPRB, 0.23452E+00_JPRB, 0.23410E+00_JPRB, 0.23541E+00_JPRB, 0.23767E+00_JPRB, 0.23985E+00_JPRB ,&
&  0.24169E+00_JPRB, 0.24431E+00_JPRB, 0.24707E+00_JPRB, 0.24914E+00_JPRB, 0.25057E+00_JPRB, 0.25194E+00_JPRB, 0.25282E+00_JPRB ,&
&  0.25301E+00_JPRB, 0.25190E+00_JPRB, 0.25007E+00_JPRB, 0.24676E+00_JPRB, 0.24252E+00_JPRB, 0.23964E+00_JPRB, 0.23825E+00_JPRB ,&
&  0.23687E+00_JPRB, 0.23681E+00_JPRB, 0.23694E+00_JPRB, 0.23654E+00_JPRB, 0.23579E+00_JPRB, 0.23575E+00_JPRB, 0.23657E+00_JPRB ,&
&  0.23722E+00_JPRB, 0.23711E+00_JPRB, 0.23647E+00_JPRB, 0.23666E+00_JPRB, 0.23797E+00_JPRB, 0.24002E+00_JPRB, 0.24178E+00_JPRB ,&
&  0.24307E+00_JPRB, 0.24299E+00_JPRB, 0.24221E+00_JPRB, 0.24043E+00_JPRB, 0.23912E+00_JPRB, 0.23931E+00_JPRB, 0.24068E+00_JPRB ,&
&  0.24174E+00_JPRB, 0.24197E+00_JPRB, 0.24223E+00_JPRB, 0.24110E+00_JPRB, 0.23991E+00_JPRB, 0.23962E+00_JPRB, 0.23962E+00_JPRB ,&
&  0.23928E+00_JPRB, 0.23868E+00_JPRB, 0.23801E+00_JPRB, 0.23759E+00_JPRB, 0.23782E+00_JPRB, 0.23776E+00_JPRB, 0.23850E+00_JPRB ,&
&  0.24082E+00_JPRB, 0.24362E+00_JPRB, 0.24647E+00_JPRB, 0.24858E+00_JPRB, 0.24976E+00_JPRB, 0.24978E+00_JPRB, 0.25156E+00_JPRB ,&
&  0.25442E+00_JPRB, 0.25642E+00_JPRB, 0.25618E+00_JPRB, 0.25568E+00_JPRB, 0.25723E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.24358E+00_JPRB, 0.24606E+00_JPRB, 0.25028E+00_JPRB, 0.25516E+00_JPRB, 0.25847E+00_JPRB, 0.26238E+00_JPRB, 0.26294E+00_JPRB ,&
&  0.26211E+00_JPRB, 0.26026E+00_JPRB, 0.25630E+00_JPRB, 0.25151E+00_JPRB, 0.24707E+00_JPRB, 0.24340E+00_JPRB, 0.24088E+00_JPRB ,&
&  0.23952E+00_JPRB, 0.23836E+00_JPRB, 0.23697E+00_JPRB, 0.23469E+00_JPRB, 0.23186E+00_JPRB, 0.22932E+00_JPRB, 0.22729E+00_JPRB ,&
&  0.22649E+00_JPRB, 0.22630E+00_JPRB, 0.22654E+00_JPRB, 0.22637E+00_JPRB, 0.22624E+00_JPRB, 0.22749E+00_JPRB, 0.22909E+00_JPRB ,&
&  0.22818E+00_JPRB, 0.22416E+00_JPRB, 0.22168E+00_JPRB, 0.22152E+00_JPRB, 0.22168E+00_JPRB, 0.22225E+00_JPRB, 0.22300E+00_JPRB ,&
&  0.22326E+00_JPRB, 0.22402E+00_JPRB, 0.22532E+00_JPRB, 0.22676E+00_JPRB, 0.22774E+00_JPRB, 0.23006E+00_JPRB, 0.23125E+00_JPRB ,&
&  0.23136E+00_JPRB, 0.23066E+00_JPRB, 0.22937E+00_JPRB, 0.22684E+00_JPRB, 0.22505E+00_JPRB, 0.22448E+00_JPRB, 0.22386E+00_JPRB ,&
&  0.22458E+00_JPRB, 0.22534E+00_JPRB, 0.22607E+00_JPRB, 0.22544E+00_JPRB, 0.22396E+00_JPRB, 0.22385E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22447E+00_JPRB, 0.22348E+00_JPRB, 0.22191E+00_JPRB, 0.22108E+00_JPRB, 0.22048E+00_JPRB, 0.22068E+00_JPRB, 0.22086E+00_JPRB ,&
&  0.22071E+00_JPRB, 0.21969E+00_JPRB, 0.21902E+00_JPRB, 0.21877E+00_JPRB, 0.21865E+00_JPRB, 0.21913E+00_JPRB, 0.21906E+00_JPRB ,&
&  0.21963E+00_JPRB, 0.22029E+00_JPRB, 0.21951E+00_JPRB, 0.21910E+00_JPRB, 0.21885E+00_JPRB, 0.21895E+00_JPRB, 0.21881E+00_JPRB ,&
&  0.21827E+00_JPRB, 0.21760E+00_JPRB, 0.21711E+00_JPRB, 0.21704E+00_JPRB, 0.21642E+00_JPRB, 0.21565E+00_JPRB, 0.21635E+00_JPRB ,&
&  0.21936E+00_JPRB, 0.22260E+00_JPRB, 0.22541E+00_JPRB, 0.22731E+00_JPRB, 0.22798E+00_JPRB, 0.22936E+00_JPRB, 0.23191E+00_JPRB ,&
&  0.23411E+00_JPRB, 0.23500E+00_JPRB, 0.23685E+00_JPRB, 0.23856E+00_JPRB, 0.24057E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.22990E+00_JPRB, 0.23304E+00_JPRB, 0.23544E+00_JPRB, 0.23877E+00_JPRB, 0.24190E+00_JPRB, 0.24649E+00_JPRB, 0.24833E+00_JPRB ,&
&  0.24809E+00_JPRB, 0.24762E+00_JPRB, 0.24607E+00_JPRB, 0.24362E+00_JPRB, 0.23987E+00_JPRB, 0.23607E+00_JPRB, 0.23252E+00_JPRB ,&
&  0.22984E+00_JPRB, 0.22767E+00_JPRB, 0.22594E+00_JPRB, 0.22438E+00_JPRB, 0.22218E+00_JPRB, 0.22010E+00_JPRB, 0.21851E+00_JPRB ,&
&  0.21771E+00_JPRB, 0.21691E+00_JPRB, 0.21648E+00_JPRB, 0.21625E+00_JPRB, 0.21599E+00_JPRB, 0.21578E+00_JPRB, 0.21582E+00_JPRB ,&
&  0.21483E+00_JPRB, 0.21159E+00_JPRB, 0.20976E+00_JPRB, 0.20921E+00_JPRB, 0.20872E+00_JPRB, 0.20918E+00_JPRB, 0.20891E+00_JPRB ,&
&  0.20874E+00_JPRB, 0.20937E+00_JPRB, 0.21080E+00_JPRB, 0.21240E+00_JPRB, 0.21251E+00_JPRB, 0.21302E+00_JPRB, 0.21411E+00_JPRB ,&
&  0.21451E+00_JPRB, 0.21391E+00_JPRB, 0.21226E+00_JPRB, 0.21172E+00_JPRB, 0.21228E+00_JPRB, 0.21229E+00_JPRB, 0.21270E+00_JPRB ,&
&  0.21339E+00_JPRB, 0.21451E+00_JPRB, 0.21511E+00_JPRB, 0.21370E+00_JPRB, 0.21229E+00_JPRB, 0.21179E+00_JPRB, 0.21181E+00_JPRB ,&
&  0.21155E+00_JPRB, 0.21067E+00_JPRB, 0.20882E+00_JPRB, 0.20720E+00_JPRB, 0.20475E+00_JPRB, 0.20278E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20235E+00_JPRB, 0.20284E+00_JPRB, 0.20420E+00_JPRB, 0.20541E+00_JPRB, 0.20614E+00_JPRB, 0.20628E+00_JPRB, 0.20491E+00_JPRB ,&
&  0.20322E+00_JPRB, 0.20250E+00_JPRB, 0.20154E+00_JPRB, 0.20059E+00_JPRB, 0.20076E+00_JPRB, 0.20065E+00_JPRB, 0.20017E+00_JPRB ,&
&  0.19958E+00_JPRB, 0.19984E+00_JPRB, 0.20010E+00_JPRB, 0.19986E+00_JPRB, 0.19940E+00_JPRB, 0.19863E+00_JPRB, 0.19910E+00_JPRB ,&
&  0.20042E+00_JPRB, 0.20233E+00_JPRB, 0.20458E+00_JPRB, 0.20708E+00_JPRB, 0.20976E+00_JPRB, 0.21217E+00_JPRB, 0.21447E+00_JPRB ,&
&  0.21735E+00_JPRB, 0.21842E+00_JPRB, 0.22023E+00_JPRB, 0.22310E+00_JPRB, 0.22596E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.21331E+00_JPRB, 0.21779E+00_JPRB, 0.22029E+00_JPRB, 0.22253E+00_JPRB, 0.22514E+00_JPRB, 0.22965E+00_JPRB, 0.23284E+00_JPRB ,&
&  0.23352E+00_JPRB, 0.23353E+00_JPRB, 0.23311E+00_JPRB, 0.23204E+00_JPRB, 0.23036E+00_JPRB, 0.22883E+00_JPRB, 0.22621E+00_JPRB ,&
&  0.22295E+00_JPRB, 0.21999E+00_JPRB, 0.21732E+00_JPRB, 0.21480E+00_JPRB, 0.21244E+00_JPRB, 0.21028E+00_JPRB, 0.20855E+00_JPRB ,&
&  0.20796E+00_JPRB, 0.20761E+00_JPRB, 0.20664E+00_JPRB, 0.20635E+00_JPRB, 0.20640E+00_JPRB, 0.20581E+00_JPRB, 0.20457E+00_JPRB ,&
&  0.20302E+00_JPRB, 0.20030E+00_JPRB, 0.19752E+00_JPRB, 0.19648E+00_JPRB, 0.19661E+00_JPRB, 0.19591E+00_JPRB, 0.19490E+00_JPRB ,&
&  0.19612E+00_JPRB, 0.19815E+00_JPRB, 0.20034E+00_JPRB, 0.20138E+00_JPRB, 0.20206E+00_JPRB, 0.20311E+00_JPRB, 0.20443E+00_JPRB ,&
&  0.20459E+00_JPRB, 0.20224E+00_JPRB, 0.20018E+00_JPRB, 0.19974E+00_JPRB, 0.19963E+00_JPRB, 0.19932E+00_JPRB, 0.19968E+00_JPRB ,&
&  0.20029E+00_JPRB, 0.20162E+00_JPRB, 0.20254E+00_JPRB, 0.20165E+00_JPRB, 0.20002E+00_JPRB, 0.19881E+00_JPRB, 0.19805E+00_JPRB ,&
&  0.19768E+00_JPRB, 0.19724E+00_JPRB, 0.19642E+00_JPRB, 0.19508E+00_JPRB, 0.19360E+00_JPRB, 0.19227E+00_JPRB, 0.19092E+00_JPRB ,&
&  0.18978E+00_JPRB, 0.19005E+00_JPRB, 0.19219E+00_JPRB, 0.19367E+00_JPRB, 0.19350E+00_JPRB, 0.19227E+00_JPRB, 0.19055E+00_JPRB ,&
&  0.18941E+00_JPRB, 0.18733E+00_JPRB, 0.18561E+00_JPRB, 0.18427E+00_JPRB, 0.18324E+00_JPRB, 0.18183E+00_JPRB, 0.18128E+00_JPRB ,&
&  0.18152E+00_JPRB, 0.18294E+00_JPRB, 0.18342E+00_JPRB, 0.18353E+00_JPRB, 0.18321E+00_JPRB, 0.18293E+00_JPRB, 0.18377E+00_JPRB ,&
&  0.18494E+00_JPRB, 0.18678E+00_JPRB, 0.18875E+00_JPRB, 0.19136E+00_JPRB, 0.19394E+00_JPRB, 0.19723E+00_JPRB, 0.19995E+00_JPRB ,&
&  0.20166E+00_JPRB, 0.20217E+00_JPRB, 0.20335E+00_JPRB, 0.20515E+00_JPRB, 0.20826E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.19547E+00_JPRB, 0.20062E+00_JPRB, 0.20513E+00_JPRB, 0.20753E+00_JPRB, 0.20911E+00_JPRB, 0.21150E+00_JPRB, 0.21401E+00_JPRB ,&
&  0.21580E+00_JPRB, 0.21681E+00_JPRB, 0.21726E+00_JPRB, 0.21716E+00_JPRB, 0.21629E+00_JPRB, 0.21485E+00_JPRB, 0.21318E+00_JPRB ,&
&  0.21134E+00_JPRB, 0.20875E+00_JPRB, 0.20636E+00_JPRB, 0.20455E+00_JPRB, 0.20301E+00_JPRB, 0.20120E+00_JPRB, 0.19976E+00_JPRB ,&
&  0.19820E+00_JPRB, 0.19759E+00_JPRB, 0.19689E+00_JPRB, 0.19624E+00_JPRB, 0.19576E+00_JPRB, 0.19493E+00_JPRB, 0.19419E+00_JPRB ,&
&  0.19218E+00_JPRB, 0.18883E+00_JPRB, 0.18720E+00_JPRB, 0.18656E+00_JPRB, 0.18672E+00_JPRB, 0.18617E+00_JPRB, 0.18547E+00_JPRB ,&
&  0.18709E+00_JPRB, 0.18987E+00_JPRB, 0.19187E+00_JPRB, 0.19295E+00_JPRB, 0.19368E+00_JPRB, 0.19468E+00_JPRB, 0.19585E+00_JPRB ,&
&  0.19369E+00_JPRB, 0.19110E+00_JPRB, 0.18922E+00_JPRB, 0.18827E+00_JPRB, 0.18816E+00_JPRB, 0.18892E+00_JPRB, 0.19043E+00_JPRB ,&
&  0.19180E+00_JPRB, 0.19258E+00_JPRB, 0.19227E+00_JPRB, 0.19103E+00_JPRB, 0.18951E+00_JPRB, 0.18878E+00_JPRB, 0.18834E+00_JPRB ,&
&  0.18814E+00_JPRB, 0.18790E+00_JPRB, 0.18758E+00_JPRB, 0.18665E+00_JPRB, 0.18167E+00_JPRB, 0.17784E+00_JPRB, 0.17523E+00_JPRB ,&
&  0.17575E+00_JPRB, 0.17607E+00_JPRB, 0.17546E+00_JPRB, 0.18036E+00_JPRB, 0.18560E+00_JPRB, 0.18148E+00_JPRB, 0.17649E+00_JPRB ,&
&  0.16971E+00_JPRB, 0.16995E+00_JPRB, 0.16728E+00_JPRB, 0.16290E+00_JPRB, 0.16161E+00_JPRB, 0.16161E+00_JPRB, 0.16123E+00_JPRB ,&
&  0.16189E+00_JPRB, 0.16203E+00_JPRB, 0.16312E+00_JPRB, 0.16364E+00_JPRB, 0.16392E+00_JPRB, 0.16459E+00_JPRB, 0.16530E+00_JPRB ,&
&  0.16713E+00_JPRB, 0.17065E+00_JPRB, 0.17485E+00_JPRB, 0.17661E+00_JPRB, 0.17909E+00_JPRB, 0.18139E+00_JPRB, 0.18292E+00_JPRB ,&
&  0.18415E+00_JPRB, 0.18591E+00_JPRB, 0.18760E+00_JPRB, 0.18898E+00_JPRB, 0.19136E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.18315E+00_JPRB, 0.18542E+00_JPRB, 0.18824E+00_JPRB, 0.19106E+00_JPRB, 0.19310E+00_JPRB, 0.19509E+00_JPRB, 0.19684E+00_JPRB ,&
&  0.19841E+00_JPRB, 0.19949E+00_JPRB, 0.20020E+00_JPRB, 0.20058E+00_JPRB, 0.20083E+00_JPRB, 0.20056E+00_JPRB, 0.19983E+00_JPRB ,&
&  0.19907E+00_JPRB, 0.19775E+00_JPRB, 0.19605E+00_JPRB, 0.19467E+00_JPRB, 0.19376E+00_JPRB, 0.19266E+00_JPRB, 0.19105E+00_JPRB ,&
&  0.18932E+00_JPRB, 0.18778E+00_JPRB, 0.18662E+00_JPRB, 0.18558E+00_JPRB, 0.18558E+00_JPRB, 0.18616E+00_JPRB, 0.18665E+00_JPRB ,&
&  0.18297E+00_JPRB, 0.17792E+00_JPRB, 0.17852E+00_JPRB, 0.17954E+00_JPRB, 0.17823E+00_JPRB, 0.17951E+00_JPRB, 0.18026E+00_JPRB ,&
&  0.18133E+00_JPRB, 0.18269E+00_JPRB, 0.18400E+00_JPRB, 0.18467E+00_JPRB, 0.18511E+00_JPRB, 0.18576E+00_JPRB, 0.18513E+00_JPRB ,&
&  0.18316E+00_JPRB, 0.18148E+00_JPRB, 0.18001E+00_JPRB, 0.17854E+00_JPRB, 0.17822E+00_JPRB, 0.17967E+00_JPRB, 0.18053E+00_JPRB ,&
&  0.17968E+00_JPRB, 0.17838E+00_JPRB, 0.17717E+00_JPRB, 0.17745E+00_JPRB, 0.17818E+00_JPRB, 0.17856E+00_JPRB, 0.17669E+00_JPRB ,&
&  0.17532E+00_JPRB, 0.17268E+00_JPRB, 0.17017E+00_JPRB, 0.16773E+00_JPRB, 0.16641E+00_JPRB, 0.16311E+00_JPRB, 0.15963E+00_JPRB ,&
&  0.15885E+00_JPRB, 0.16250E+00_JPRB, 0.16828E+00_JPRB, 0.17075E+00_JPRB, 0.16212E+00_JPRB, 0.16186E+00_JPRB, 0.16157E+00_JPRB ,&
&  0.15721E+00_JPRB, 0.15242E+00_JPRB, 0.14898E+00_JPRB, 0.14752E+00_JPRB, 0.14696E+00_JPRB, 0.14645E+00_JPRB, 0.14685E+00_JPRB ,&
&  0.14768E+00_JPRB, 0.14816E+00_JPRB, 0.14874E+00_JPRB, 0.14983E+00_JPRB, 0.15090E+00_JPRB, 0.15198E+00_JPRB, 0.15292E+00_JPRB ,&
&  0.15498E+00_JPRB, 0.15753E+00_JPRB, 0.15960E+00_JPRB, 0.16236E+00_JPRB, 0.16533E+00_JPRB, 0.16828E+00_JPRB, 0.17126E+00_JPRB ,&
&  0.17457E+00_JPRB, 0.17669E+00_JPRB, 0.17815E+00_JPRB, 0.18068E+00_JPRB, 0.18189E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.17749E+00_JPRB, 0.17994E+00_JPRB, 0.18173E+00_JPRB, 0.18339E+00_JPRB, 0.18467E+00_JPRB, 0.18604E+00_JPRB, 0.18664E+00_JPRB ,&
&  0.18718E+00_JPRB, 0.18715E+00_JPRB, 0.18754E+00_JPRB, 0.18748E+00_JPRB, 0.18763E+00_JPRB, 0.18658E+00_JPRB, 0.18601E+00_JPRB ,&
&  0.18558E+00_JPRB, 0.18588E+00_JPRB, 0.18583E+00_JPRB, 0.18568E+00_JPRB, 0.18497E+00_JPRB, 0.18405E+00_JPRB, 0.18260E+00_JPRB ,&
&  0.17963E+00_JPRB, 0.17763E+00_JPRB, 0.17792E+00_JPRB, 0.17746E+00_JPRB, 0.17786E+00_JPRB, 0.17790E+00_JPRB, 0.17836E+00_JPRB ,&
&  0.17441E+00_JPRB, 0.17294E+00_JPRB, 0.17305E+00_JPRB, 0.17213E+00_JPRB, 0.17442E+00_JPRB, 0.17541E+00_JPRB, 0.17713E+00_JPRB ,&
&  0.17761E+00_JPRB, 0.17796E+00_JPRB, 0.17868E+00_JPRB, 0.17827E+00_JPRB, 0.17860E+00_JPRB, 0.17901E+00_JPRB, 0.17789E+00_JPRB ,&
&  0.17506E+00_JPRB, 0.17283E+00_JPRB, 0.16972E+00_JPRB, 0.16201E+00_JPRB, 0.16002E+00_JPRB, 0.16024E+00_JPRB, 0.15887E+00_JPRB ,&
&  0.15627E+00_JPRB, 0.15654E+00_JPRB, 0.15613E+00_JPRB, 0.15476E+00_JPRB, 0.15652E+00_JPRB, 0.15903E+00_JPRB, 0.15919E+00_JPRB ,&
&  0.15810E+00_JPRB, 0.15783E+00_JPRB, 0.15723E+00_JPRB, 0.15602E+00_JPRB, 0.15461E+00_JPRB, 0.15401E+00_JPRB, 0.15348E+00_JPRB ,&
&  0.15322E+00_JPRB, 0.15208E+00_JPRB, 0.15539E+00_JPRB, 0.15631E+00_JPRB, 0.15180E+00_JPRB, 0.15337E+00_JPRB, 0.15108E+00_JPRB ,&
&  0.14478E+00_JPRB, 0.14194E+00_JPRB, 0.14035E+00_JPRB, 0.13902E+00_JPRB, 0.13876E+00_JPRB, 0.13886E+00_JPRB, 0.13872E+00_JPRB ,&
&  0.13953E+00_JPRB, 0.14053E+00_JPRB, 0.14103E+00_JPRB, 0.14168E+00_JPRB, 0.14256E+00_JPRB, 0.14383E+00_JPRB, 0.14550E+00_JPRB ,&
&  0.14697E+00_JPRB, 0.14901E+00_JPRB, 0.15007E+00_JPRB, 0.15179E+00_JPRB, 0.15308E+00_JPRB, 0.15456E+00_JPRB, 0.15810E+00_JPRB ,&
&  0.16106E+00_JPRB, 0.16684E+00_JPRB, 0.17485E+00_JPRB, 0.17760E+00_JPRB, 0.17520E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.17232E+00_JPRB, 0.17418E+00_JPRB, 0.17620E+00_JPRB, 0.17776E+00_JPRB, 0.17931E+00_JPRB, 0.18060E+00_JPRB, 0.18129E+00_JPRB ,&
&  0.18141E+00_JPRB, 0.18080E+00_JPRB, 0.18082E+00_JPRB, 0.17962E+00_JPRB, 0.17849E+00_JPRB, 0.17433E+00_JPRB, 0.17372E+00_JPRB ,&
&  0.17271E+00_JPRB, 0.17326E+00_JPRB, 0.17284E+00_JPRB, 0.17309E+00_JPRB, 0.17244E+00_JPRB, 0.17212E+00_JPRB, 0.17066E+00_JPRB ,&
&  0.16848E+00_JPRB, 0.16680E+00_JPRB, 0.16515E+00_JPRB, 0.16465E+00_JPRB, 0.16538E+00_JPRB, 0.16593E+00_JPRB, 0.16565E+00_JPRB ,&
&  0.16568E+00_JPRB, 0.16599E+00_JPRB, 0.16645E+00_JPRB, 0.16654E+00_JPRB, 0.17164E+00_JPRB, 0.17374E+00_JPRB, 0.17482E+00_JPRB ,&
&  0.17469E+00_JPRB, 0.17506E+00_JPRB, 0.17519E+00_JPRB, 0.17439E+00_JPRB, 0.17358E+00_JPRB, 0.16974E+00_JPRB, 0.16723E+00_JPRB ,&
&  0.15955E+00_JPRB, 0.15406E+00_JPRB, 0.14988E+00_JPRB, 0.14731E+00_JPRB, 0.14627E+00_JPRB, 0.14625E+00_JPRB, 0.14578E+00_JPRB ,&
&  0.14432E+00_JPRB, 0.14298E+00_JPRB, 0.14216E+00_JPRB, 0.14281E+00_JPRB, 0.14386E+00_JPRB, 0.14504E+00_JPRB, 0.14638E+00_JPRB ,&
&  0.14755E+00_JPRB, 0.14787E+00_JPRB, 0.14787E+00_JPRB, 0.14828E+00_JPRB, 0.14860E+00_JPRB, 0.14910E+00_JPRB, 0.14948E+00_JPRB ,&
&  0.15045E+00_JPRB, 0.14973E+00_JPRB, 0.15032E+00_JPRB, 0.15134E+00_JPRB, 0.14915E+00_JPRB, 0.14563E+00_JPRB, 0.14213E+00_JPRB ,&
&  0.13885E+00_JPRB, 0.13699E+00_JPRB, 0.13548E+00_JPRB, 0.13450E+00_JPRB, 0.13426E+00_JPRB, 0.13433E+00_JPRB, 0.13482E+00_JPRB ,&
&  0.13547E+00_JPRB, 0.13638E+00_JPRB, 0.13698E+00_JPRB, 0.13777E+00_JPRB, 0.13872E+00_JPRB, 0.13975E+00_JPRB, 0.14097E+00_JPRB ,&
&  0.14248E+00_JPRB, 0.14392E+00_JPRB, 0.14493E+00_JPRB, 0.14638E+00_JPRB, 0.14910E+00_JPRB, 0.15275E+00_JPRB, 0.15829E+00_JPRB ,&
&  0.16453E+00_JPRB, 0.16989E+00_JPRB, 0.16764E+00_JPRB, 0.16975E+00_JPRB, 0.17147E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.17228E+00_JPRB, 0.17371E+00_JPRB, 0.17518E+00_JPRB, 0.17664E+00_JPRB, 0.17790E+00_JPRB, 0.17911E+00_JPRB, 0.17991E+00_JPRB ,&
&  0.17752E+00_JPRB, 0.17610E+00_JPRB, 0.17350E+00_JPRB, 0.17114E+00_JPRB, 0.16840E+00_JPRB, 0.16541E+00_JPRB, 0.16437E+00_JPRB ,&
&  0.16291E+00_JPRB, 0.16261E+00_JPRB, 0.16143E+00_JPRB, 0.16093E+00_JPRB, 0.15996E+00_JPRB, 0.15939E+00_JPRB, 0.15841E+00_JPRB ,&
&  0.15646E+00_JPRB, 0.15546E+00_JPRB, 0.15451E+00_JPRB, 0.15489E+00_JPRB, 0.15633E+00_JPRB, 0.15789E+00_JPRB, 0.15954E+00_JPRB ,&
&  0.16192E+00_JPRB, 0.16303E+00_JPRB, 0.16531E+00_JPRB, 0.16701E+00_JPRB, 0.16982E+00_JPRB, 0.17216E+00_JPRB, 0.17262E+00_JPRB ,&
&  0.17260E+00_JPRB, 0.17094E+00_JPRB, 0.17048E+00_JPRB, 0.16850E+00_JPRB, 0.16272E+00_JPRB, 0.15941E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.14962E+00_JPRB, 0.14576E+00_JPRB, 0.14311E+00_JPRB, 0.14226E+00_JPRB, 0.14143E+00_JPRB, 0.14121E+00_JPRB, 0.14102E+00_JPRB ,&
&  0.14029E+00_JPRB, 0.13944E+00_JPRB, 0.13844E+00_JPRB, 0.13791E+00_JPRB, 0.13827E+00_JPRB, 0.13899E+00_JPRB, 0.14051E+00_JPRB ,&
&  0.14203E+00_JPRB, 0.14288E+00_JPRB, 0.14381E+00_JPRB, 0.14413E+00_JPRB, 0.14426E+00_JPRB, 0.14450E+00_JPRB, 0.14459E+00_JPRB ,&
&  0.14523E+00_JPRB, 0.14538E+00_JPRB, 0.14491E+00_JPRB, 0.14409E+00_JPRB, 0.14203E+00_JPRB, 0.13934E+00_JPRB, 0.13720E+00_JPRB ,&
&  0.13537E+00_JPRB, 0.13418E+00_JPRB, 0.13374E+00_JPRB, 0.13330E+00_JPRB, 0.13351E+00_JPRB, 0.13385E+00_JPRB, 0.13417E+00_JPRB ,&
&  0.13472E+00_JPRB, 0.13521E+00_JPRB, 0.13568E+00_JPRB, 0.13677E+00_JPRB, 0.13767E+00_JPRB, 0.13901E+00_JPRB, 0.14041E+00_JPRB ,&
&  0.14139E+00_JPRB, 0.14288E+00_JPRB, 0.14414E+00_JPRB, 0.14552E+00_JPRB, 0.14962E+00_JPRB, 0.15292E+00_JPRB, 0.15959E+00_JPRB ,&
&  0.16396E+00_JPRB, 0.16586E+00_JPRB, 0.16681E+00_JPRB, 0.17183E+00_JPRB, 0.17187E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.17317E+00_JPRB, 0.17356E+00_JPRB, 0.17350E+00_JPRB, 0.17442E+00_JPRB, 0.17520E+00_JPRB, 0.17605E+00_JPRB, 0.17557E+00_JPRB ,&
&  0.17514E+00_JPRB, 0.17363E+00_JPRB, 0.17245E+00_JPRB, 0.16951E+00_JPRB, 0.16716E+00_JPRB, 0.16436E+00_JPRB, 0.16194E+00_JPRB ,&
&  0.15980E+00_JPRB, 0.15775E+00_JPRB, 0.15586E+00_JPRB, 0.15422E+00_JPRB, 0.15272E+00_JPRB, 0.15140E+00_JPRB, 0.15014E+00_JPRB ,&
&  0.14827E+00_JPRB, 0.14777E+00_JPRB, 0.14782E+00_JPRB, 0.14905E+00_JPRB, 0.15152E+00_JPRB, 0.15317E+00_JPRB, 0.15516E+00_JPRB ,&
&  0.15747E+00_JPRB, 0.15694E+00_JPRB, 0.15978E+00_JPRB, 0.16254E+00_JPRB, 0.16479E+00_JPRB, 0.16660E+00_JPRB, 0.16744E+00_JPRB ,&
&  0.16802E+00_JPRB, 0.16352E+00_JPRB, 0.16472E+00_JPRB, 0.16085E+00_JPRB, 0.15637E+00_JPRB, 0.15427E+00_JPRB, 0.15040E+00_JPRB ,&
&  0.14856E+00_JPRB, 0.14556E+00_JPRB, 0.14362E+00_JPRB, 0.14200E+00_JPRB, 0.14013E+00_JPRB, 0.13970E+00_JPRB, 0.13894E+00_JPRB ,&
&  0.13851E+00_JPRB, 0.13858E+00_JPRB, 0.13787E+00_JPRB, 0.13763E+00_JPRB, 0.13721E+00_JPRB, 0.13682E+00_JPRB, 0.13716E+00_JPRB ,&
&  0.13771E+00_JPRB, 0.13837E+00_JPRB, 0.13900E+00_JPRB, 0.13909E+00_JPRB, 0.13903E+00_JPRB, 0.13901E+00_JPRB, 0.13894E+00_JPRB ,&
&  0.13919E+00_JPRB, 0.13920E+00_JPRB, 0.13883E+00_JPRB, 0.13839E+00_JPRB, 0.13740E+00_JPRB, 0.13642E+00_JPRB, 0.13544E+00_JPRB ,&
&  0.13458E+00_JPRB, 0.13412E+00_JPRB, 0.13372E+00_JPRB, 0.13362E+00_JPRB, 0.13364E+00_JPRB, 0.13395E+00_JPRB, 0.13427E+00_JPRB ,&
&  0.13457E+00_JPRB, 0.13525E+00_JPRB, 0.13576E+00_JPRB, 0.13659E+00_JPRB, 0.13759E+00_JPRB, 0.13850E+00_JPRB, 0.13988E+00_JPRB ,&
&  0.14137E+00_JPRB, 0.14300E+00_JPRB, 0.14531E+00_JPRB, 0.14734E+00_JPRB, 0.14989E+00_JPRB, 0.15323E+00_JPRB, 0.15573E+00_JPRB ,&
&  0.15843E+00_JPRB, 0.16120E+00_JPRB, 0.16157E+00_JPRB, 0.16834E+00_JPRB, 0.17213E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.16636E+00_JPRB, 0.16708E+00_JPRB, 0.16744E+00_JPRB, 0.16783E+00_JPRB, 0.16882E+00_JPRB, 0.16882E+00_JPRB, 0.16863E+00_JPRB ,&
&  0.16662E+00_JPRB, 0.16581E+00_JPRB, 0.16453E+00_JPRB, 0.16276E+00_JPRB, 0.16097E+00_JPRB, 0.15806E+00_JPRB, 0.15619E+00_JPRB ,&
&  0.15437E+00_JPRB, 0.15244E+00_JPRB, 0.15115E+00_JPRB, 0.14950E+00_JPRB, 0.14808E+00_JPRB, 0.14651E+00_JPRB, 0.14595E+00_JPRB ,&
&  0.14518E+00_JPRB, 0.14513E+00_JPRB, 0.14590E+00_JPRB, 0.14653E+00_JPRB, 0.14768E+00_JPRB, 0.14891E+00_JPRB, 0.15048E+00_JPRB ,&
&  0.15093E+00_JPRB, 0.15172E+00_JPRB, 0.15246E+00_JPRB, 0.15294E+00_JPRB, 0.15457E+00_JPRB, 0.15574E+00_JPRB, 0.15614E+00_JPRB ,&
&  0.15568E+00_JPRB, 0.15485E+00_JPRB, 0.15320E+00_JPRB, 0.15133E+00_JPRB, 0.14919E+00_JPRB, 0.14682E+00_JPRB, 0.14457E+00_JPRB ,&
&  0.14282E+00_JPRB, 0.14120E+00_JPRB, 0.13997E+00_JPRB, 0.13898E+00_JPRB, 0.13838E+00_JPRB, 0.13791E+00_JPRB, 0.13745E+00_JPRB ,&
&  0.13719E+00_JPRB, 0.13698E+00_JPRB, 0.13664E+00_JPRB, 0.13662E+00_JPRB, 0.13652E+00_JPRB, 0.13656E+00_JPRB, 0.13659E+00_JPRB ,&
&  0.13666E+00_JPRB, 0.13662E+00_JPRB, 0.13651E+00_JPRB, 0.13620E+00_JPRB, 0.13590E+00_JPRB, 0.13557E+00_JPRB, 0.13525E+00_JPRB ,&
&  0.13490E+00_JPRB, 0.13486E+00_JPRB, 0.13459E+00_JPRB, 0.13440E+00_JPRB, 0.13416E+00_JPRB, 0.13375E+00_JPRB, 0.13364E+00_JPRB ,&
&  0.13373E+00_JPRB, 0.13386E+00_JPRB, 0.13374E+00_JPRB, 0.13386E+00_JPRB, 0.13405E+00_JPRB, 0.13434E+00_JPRB, 0.13509E+00_JPRB ,&
&  0.13556E+00_JPRB, 0.13605E+00_JPRB, 0.13651E+00_JPRB, 0.13716E+00_JPRB, 0.13826E+00_JPRB, 0.13936E+00_JPRB, 0.14059E+00_JPRB ,&
&  0.14207E+00_JPRB, 0.14345E+00_JPRB, 0.14484E+00_JPRB, 0.14622E+00_JPRB, 0.14848E+00_JPRB, 0.15013E+00_JPRB, 0.15231E+00_JPRB ,&
&  0.15424E+00_JPRB, 0.15340E+00_JPRB, 0.16132E+00_JPRB, 0.16215E+00_JPRB, 0.16343E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.15112E+00_JPRB, 0.15137E+00_JPRB, 0.15078E+00_JPRB, 0.15017E+00_JPRB, 0.14441E+00_JPRB, 0.15048E+00_JPRB, 0.14979E+00_JPRB ,&
&  0.15018E+00_JPRB, 0.15000E+00_JPRB, 0.15117E+00_JPRB, 0.15126E+00_JPRB, 0.15091E+00_JPRB, 0.15068E+00_JPRB, 0.15038E+00_JPRB ,&
&  0.15003E+00_JPRB, 0.14929E+00_JPRB, 0.14889E+00_JPRB, 0.14834E+00_JPRB, 0.14779E+00_JPRB, 0.14691E+00_JPRB, 0.14598E+00_JPRB ,&
&  0.14557E+00_JPRB, 0.14564E+00_JPRB, 0.14599E+00_JPRB, 0.14454E+00_JPRB, 0.14392E+00_JPRB, 0.14353E+00_JPRB, 0.14335E+00_JPRB ,&
&  0.14302E+00_JPRB, 0.14294E+00_JPRB, 0.14295E+00_JPRB, 0.14289E+00_JPRB, 0.14403E+00_JPRB, 0.14436E+00_JPRB, 0.14462E+00_JPRB ,&
&  0.14465E+00_JPRB, 0.14449E+00_JPRB, 0.14384E+00_JPRB, 0.14325E+00_JPRB, 0.14265E+00_JPRB, 0.14164E+00_JPRB, 0.14057E+00_JPRB ,&
&  0.13966E+00_JPRB, 0.13895E+00_JPRB, 0.13817E+00_JPRB, 0.13756E+00_JPRB, 0.13706E+00_JPRB, 0.13659E+00_JPRB, 0.13629E+00_JPRB ,&
&  0.13620E+00_JPRB, 0.13620E+00_JPRB, 0.13617E+00_JPRB, 0.13615E+00_JPRB, 0.13608E+00_JPRB, 0.13604E+00_JPRB, 0.13600E+00_JPRB ,&
&  0.13624E+00_JPRB, 0.13621E+00_JPRB, 0.13604E+00_JPRB, 0.13577E+00_JPRB, 0.13564E+00_JPRB, 0.13533E+00_JPRB, 0.13512E+00_JPRB ,&
&  0.13494E+00_JPRB, 0.13473E+00_JPRB, 0.13463E+00_JPRB, 0.13460E+00_JPRB, 0.13460E+00_JPRB, 0.13459E+00_JPRB, 0.13467E+00_JPRB ,&
&  0.13480E+00_JPRB, 0.13498E+00_JPRB, 0.13532E+00_JPRB, 0.13559E+00_JPRB, 0.13585E+00_JPRB, 0.13607E+00_JPRB, 0.13653E+00_JPRB ,&
&  0.13702E+00_JPRB, 0.13750E+00_JPRB, 0.13798E+00_JPRB, 0.13831E+00_JPRB, 0.13900E+00_JPRB, 0.13970E+00_JPRB, 0.14045E+00_JPRB ,&
&  0.14126E+00_JPRB, 0.14227E+00_JPRB, 0.14332E+00_JPRB, 0.14437E+00_JPRB, 0.14575E+00_JPRB, 0.14687E+00_JPRB, 0.14797E+00_JPRB ,&
&  0.14919E+00_JPRB, 0.15143E+00_JPRB, 0.15208E+00_JPRB, 0.15226E+00_JPRB, 0.15188E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.14629E+00_JPRB, 0.14627E+00_JPRB, 0.14617E+00_JPRB, 0.14585E+00_JPRB, 0.14523E+00_JPRB, 0.14509E+00_JPRB, 0.14489E+00_JPRB ,&
&  0.14461E+00_JPRB, 0.14401E+00_JPRB, 0.14400E+00_JPRB, 0.14393E+00_JPRB, 0.14384E+00_JPRB, 0.14373E+00_JPRB, 0.14352E+00_JPRB ,&
&  0.14308E+00_JPRB, 0.14241E+00_JPRB, 0.14040E+00_JPRB, 0.13943E+00_JPRB, 0.13882E+00_JPRB, 0.13849E+00_JPRB, 0.13821E+00_JPRB ,&
&  0.13805E+00_JPRB, 0.13796E+00_JPRB, 0.13787E+00_JPRB, 0.13825E+00_JPRB, 0.13841E+00_JPRB, 0.13847E+00_JPRB, 0.13851E+00_JPRB ,&
&  0.13850E+00_JPRB, 0.13858E+00_JPRB, 0.13859E+00_JPRB, 0.13858E+00_JPRB, 0.13869E+00_JPRB, 0.13868E+00_JPRB, 0.13850E+00_JPRB ,&
&  0.13830E+00_JPRB, 0.13806E+00_JPRB, 0.13782E+00_JPRB, 0.13752E+00_JPRB, 0.13731E+00_JPRB, 0.13713E+00_JPRB, 0.13706E+00_JPRB ,&
&  0.13691E+00_JPRB, 0.13679E+00_JPRB, 0.13667E+00_JPRB, 0.13660E+00_JPRB, 0.13650E+00_JPRB, 0.13645E+00_JPRB, 0.13633E+00_JPRB ,&
&  0.13640E+00_JPRB, 0.13642E+00_JPRB, 0.13646E+00_JPRB, 0.13648E+00_JPRB, 0.13649E+00_JPRB, 0.13645E+00_JPRB, 0.13638E+00_JPRB ,&
&  0.13622E+00_JPRB, 0.13613E+00_JPRB, 0.13602E+00_JPRB, 0.13594E+00_JPRB, 0.13591E+00_JPRB, 0.13589E+00_JPRB, 0.13589E+00_JPRB ,&
&  0.13590E+00_JPRB, 0.13598E+00_JPRB, 0.13601E+00_JPRB, 0.13606E+00_JPRB, 0.13612E+00_JPRB, 0.13618E+00_JPRB, 0.13623E+00_JPRB ,&
&  0.13631E+00_JPRB, 0.13638E+00_JPRB, 0.13648E+00_JPRB, 0.13661E+00_JPRB, 0.13677E+00_JPRB, 0.13698E+00_JPRB, 0.13724E+00_JPRB ,&
&  0.13752E+00_JPRB, 0.13780E+00_JPRB, 0.13808E+00_JPRB, 0.13836E+00_JPRB, 0.13879E+00_JPRB, 0.13922E+00_JPRB, 0.13973E+00_JPRB ,&
&  0.14024E+00_JPRB, 0.14074E+00_JPRB, 0.14120E+00_JPRB, 0.14170E+00_JPRB, 0.14232E+00_JPRB, 0.14265E+00_JPRB, 0.14293E+00_JPRB ,&
&  0.14334E+00_JPRB, 0.14377E+00_JPRB, 0.14420E+00_JPRB, 0.14459E+00_JPRB, 0.14521E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB, 0.13784E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB, 0.42885E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.42902E+00_JPRB, 0.42845E+00_JPRB, 0.42802E+00_JPRB, 0.42740E+00_JPRB, 0.42643E+00_JPRB, 0.42558E+00_JPRB, 0.42476E+00_JPRB ,&
&  0.42408E+00_JPRB, 0.42368E+00_JPRB, 0.42351E+00_JPRB, 0.42350E+00_JPRB, 0.42370E+00_JPRB, 0.42388E+00_JPRB, 0.42406E+00_JPRB ,&
&  0.42418E+00_JPRB, 0.42434E+00_JPRB, 0.42440E+00_JPRB, 0.42431E+00_JPRB, 0.42411E+00_JPRB, 0.42384E+00_JPRB, 0.42342E+00_JPRB ,&
&  0.42272E+00_JPRB, 0.42209E+00_JPRB, 0.42159E+00_JPRB, 0.42108E+00_JPRB, 0.42069E+00_JPRB, 0.42025E+00_JPRB, 0.41977E+00_JPRB ,&
&  0.41940E+00_JPRB, 0.41938E+00_JPRB, 0.41947E+00_JPRB, 0.41957E+00_JPRB, 0.41988E+00_JPRB, 0.42085E+00_JPRB, 0.42177E+00_JPRB ,&
&  0.42280E+00_JPRB, 0.42401E+00_JPRB, 0.42585E+00_JPRB, 0.42765E+00_JPRB, 0.42932E+00_JPRB, 0.43073E+00_JPRB, 0.43160E+00_JPRB ,&
&  0.43160E+00_JPRB, 0.43115E+00_JPRB, 0.43036E+00_JPRB, 0.42945E+00_JPRB, 0.42807E+00_JPRB, 0.42679E+00_JPRB, 0.42574E+00_JPRB ,&
&  0.42451E+00_JPRB, 0.42348E+00_JPRB, 0.42252E+00_JPRB, 0.42128E+00_JPRB, 0.41911E+00_JPRB, 0.41750E+00_JPRB, 0.41651E+00_JPRB ,&
&  0.41603E+00_JPRB, 0.41558E+00_JPRB, 0.41540E+00_JPRB, 0.41545E+00_JPRB, 0.41593E+00_JPRB, 0.41661E+00_JPRB, 0.41753E+00_JPRB ,&
&  0.41855E+00_JPRB, 0.41987E+00_JPRB, 0.42170E+00_JPRB, 0.42361E+00_JPRB, 0.42563E+00_JPRB, 0.42785E+00_JPRB, 0.42991E+00_JPRB ,&
&  0.43186E+00_JPRB, 0.43391E+00_JPRB, 0.43566E+00_JPRB, 0.43758E+00_JPRB, 0.43960E+00_JPRB, 0.44183E+00_JPRB, 0.44361E+00_JPRB ,&
&  0.44470E+00_JPRB, 0.44552E+00_JPRB, 0.44619E+00_JPRB, 0.44613E+00_JPRB, 0.44562E+00_JPRB, 0.44517E+00_JPRB, 0.44469E+00_JPRB ,&
&  0.44401E+00_JPRB, 0.44311E+00_JPRB, 0.44223E+00_JPRB, 0.44120E+00_JPRB, 0.43976E+00_JPRB, 0.43827E+00_JPRB, 0.43683E+00_JPRB ,&
&  0.43542E+00_JPRB, 0.43387E+00_JPRB, 0.43251E+00_JPRB, 0.43122E+00_JPRB, 0.43009E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.43546E+00_JPRB, 0.43225E+00_JPRB, 0.42949E+00_JPRB, 0.42700E+00_JPRB, 0.42505E+00_JPRB, 0.42376E+00_JPRB, 0.42295E+00_JPRB ,&
&  0.42243E+00_JPRB, 0.42258E+00_JPRB, 0.42255E+00_JPRB, 0.42241E+00_JPRB, 0.42220E+00_JPRB, 0.42122E+00_JPRB, 0.41983E+00_JPRB ,&
&  0.41854E+00_JPRB, 0.41765E+00_JPRB, 0.41656E+00_JPRB, 0.41541E+00_JPRB, 0.41389E+00_JPRB, 0.41175E+00_JPRB, 0.40930E+00_JPRB ,&
&  0.40659E+00_JPRB, 0.40314E+00_JPRB, 0.39910E+00_JPRB, 0.39515E+00_JPRB, 0.39331E+00_JPRB, 0.39111E+00_JPRB, 0.38908E+00_JPRB ,&
&  0.38752E+00_JPRB, 0.38764E+00_JPRB, 0.38780E+00_JPRB, 0.38780E+00_JPRB, 0.38804E+00_JPRB, 0.38997E+00_JPRB, 0.39151E+00_JPRB ,&
&  0.39330E+00_JPRB, 0.39617E+00_JPRB, 0.40140E+00_JPRB, 0.40707E+00_JPRB, 0.41351E+00_JPRB, 0.42108E+00_JPRB, 0.42975E+00_JPRB ,&
&  0.43594E+00_JPRB, 0.43856E+00_JPRB, 0.43978E+00_JPRB, 0.44059E+00_JPRB, 0.44001E+00_JPRB, 0.43867E+00_JPRB, 0.43647E+00_JPRB ,&
&  0.43518E+00_JPRB, 0.43365E+00_JPRB, 0.43255E+00_JPRB, 0.43168E+00_JPRB, 0.42939E+00_JPRB, 0.42749E+00_JPRB, 0.42576E+00_JPRB ,&
&  0.42363E+00_JPRB, 0.42197E+00_JPRB, 0.42105E+00_JPRB, 0.42036E+00_JPRB, 0.42062E+00_JPRB, 0.42184E+00_JPRB, 0.42388E+00_JPRB ,&
&  0.42714E+00_JPRB, 0.43173E+00_JPRB, 0.43764E+00_JPRB, 0.44429E+00_JPRB, 0.45192E+00_JPRB, 0.46156E+00_JPRB, 0.46927E+00_JPRB ,&
&  0.47644E+00_JPRB, 0.48291E+00_JPRB, 0.48882E+00_JPRB, 0.49363E+00_JPRB, 0.49740E+00_JPRB, 0.50124E+00_JPRB, 0.50186E+00_JPRB ,&
&  0.50184E+00_JPRB, 0.50127E+00_JPRB, 0.50080E+00_JPRB, 0.49860E+00_JPRB, 0.49585E+00_JPRB, 0.49291E+00_JPRB, 0.49013E+00_JPRB ,&
&  0.48608E+00_JPRB, 0.48190E+00_JPRB, 0.47771E+00_JPRB, 0.47354E+00_JPRB, 0.46903E+00_JPRB, 0.46473E+00_JPRB, 0.46045E+00_JPRB ,&
&  0.45608E+00_JPRB, 0.45167E+00_JPRB, 0.44751E+00_JPRB, 0.44348E+00_JPRB, 0.43942E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.44172E+00_JPRB, 0.43611E+00_JPRB, 0.43115E+00_JPRB, 0.42691E+00_JPRB, 0.42349E+00_JPRB, 0.42119E+00_JPRB, 0.41986E+00_JPRB ,&
&  0.41914E+00_JPRB, 0.41952E+00_JPRB, 0.41982E+00_JPRB, 0.41960E+00_JPRB, 0.41893E+00_JPRB, 0.41906E+00_JPRB, 0.41800E+00_JPRB ,&
&  0.41583E+00_JPRB, 0.41269E+00_JPRB, 0.40938E+00_JPRB, 0.40561E+00_JPRB, 0.40091E+00_JPRB, 0.39579E+00_JPRB, 0.38964E+00_JPRB ,&
&  0.38512E+00_JPRB, 0.37863E+00_JPRB, 0.37119E+00_JPRB, 0.36280E+00_JPRB, 0.35704E+00_JPRB, 0.34347E+00_JPRB, 0.33811E+00_JPRB ,&
&  0.33432E+00_JPRB, 0.32917E+00_JPRB, 0.32773E+00_JPRB, 0.32929E+00_JPRB, 0.34417E+00_JPRB, 0.35399E+00_JPRB, 0.35819E+00_JPRB ,&
&  0.34664E+00_JPRB, 0.34606E+00_JPRB, 0.34377E+00_JPRB, 0.34281E+00_JPRB, 0.34557E+00_JPRB, 0.35237E+00_JPRB, 0.37423E+00_JPRB ,&
&  0.41231E+00_JPRB, 0.42646E+00_JPRB, 0.43317E+00_JPRB, 0.43780E+00_JPRB, 0.43843E+00_JPRB, 0.43915E+00_JPRB, 0.43713E+00_JPRB ,&
&  0.43580E+00_JPRB, 0.43353E+00_JPRB, 0.43356E+00_JPRB, 0.43379E+00_JPRB, 0.43494E+00_JPRB, 0.43579E+00_JPRB, 0.43659E+00_JPRB ,&
&  0.43362E+00_JPRB, 0.43130E+00_JPRB, 0.42931E+00_JPRB, 0.42920E+00_JPRB, 0.43046E+00_JPRB, 0.43396E+00_JPRB, 0.43839E+00_JPRB ,&
&  0.44373E+00_JPRB, 0.44774E+00_JPRB, 0.45572E+00_JPRB, 0.46720E+00_JPRB, 0.48106E+00_JPRB, 0.49886E+00_JPRB, 0.51729E+00_JPRB ,&
&  0.53602E+00_JPRB, 0.55203E+00_JPRB, 0.56790E+00_JPRB, 0.57715E+00_JPRB, 0.58332E+00_JPRB, 0.58802E+00_JPRB, 0.58911E+00_JPRB ,&
&  0.58890E+00_JPRB, 0.58685E+00_JPRB, 0.58398E+00_JPRB, 0.57965E+00_JPRB, 0.57390E+00_JPRB, 0.56567E+00_JPRB, 0.55649E+00_JPRB ,&
&  0.54576E+00_JPRB, 0.53518E+00_JPRB, 0.52449E+00_JPRB, 0.51379E+00_JPRB, 0.50331E+00_JPRB, 0.49372E+00_JPRB, 0.48518E+00_JPRB ,&
&  0.47704E+00_JPRB, 0.46916E+00_JPRB, 0.46202E+00_JPRB, 0.45525E+00_JPRB, 0.44844E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.44974E+00_JPRB, 0.44298E+00_JPRB, 0.43588E+00_JPRB, 0.42981E+00_JPRB, 0.42498E+00_JPRB, 0.42179E+00_JPRB, 0.42007E+00_JPRB ,&
&  0.41926E+00_JPRB, 0.41805E+00_JPRB, 0.41669E+00_JPRB, 0.41521E+00_JPRB, 0.41333E+00_JPRB, 0.41373E+00_JPRB, 0.41383E+00_JPRB ,&
&  0.41362E+00_JPRB, 0.41058E+00_JPRB, 0.40561E+00_JPRB, 0.39876E+00_JPRB, 0.39179E+00_JPRB, 0.38628E+00_JPRB, 0.38156E+00_JPRB ,&
&  0.37426E+00_JPRB, 0.36594E+00_JPRB, 0.35097E+00_JPRB, 0.34006E+00_JPRB, 0.32626E+00_JPRB, 0.31823E+00_JPRB, 0.31446E+00_JPRB ,&
&  0.31300E+00_JPRB, 0.31508E+00_JPRB, 0.31983E+00_JPRB, 0.32328E+00_JPRB, 0.31971E+00_JPRB, 0.31441E+00_JPRB, 0.31108E+00_JPRB ,&
&  0.30559E+00_JPRB, 0.30223E+00_JPRB, 0.29994E+00_JPRB, 0.29963E+00_JPRB, 0.30171E+00_JPRB, 0.30840E+00_JPRB, 0.32194E+00_JPRB ,&
&  0.35368E+00_JPRB, 0.39448E+00_JPRB, 0.43239E+00_JPRB, 0.44786E+00_JPRB, 0.43914E+00_JPRB, 0.43957E+00_JPRB, 0.43485E+00_JPRB ,&
&  0.43188E+00_JPRB, 0.42721E+00_JPRB, 0.42454E+00_JPRB, 0.42703E+00_JPRB, 0.43105E+00_JPRB, 0.43516E+00_JPRB, 0.43928E+00_JPRB ,&
&  0.43985E+00_JPRB, 0.44118E+00_JPRB, 0.43640E+00_JPRB, 0.43609E+00_JPRB, 0.43968E+00_JPRB, 0.44286E+00_JPRB, 0.44299E+00_JPRB ,&
&  0.44636E+00_JPRB, 0.45228E+00_JPRB, 0.46645E+00_JPRB, 0.48891E+00_JPRB, 0.51303E+00_JPRB, 0.53592E+00_JPRB, 0.55635E+00_JPRB ,&
&  0.57688E+00_JPRB, 0.60266E+00_JPRB, 0.62761E+00_JPRB, 0.63831E+00_JPRB, 0.64013E+00_JPRB, 0.64533E+00_JPRB, 0.64292E+00_JPRB ,&
&  0.64122E+00_JPRB, 0.63720E+00_JPRB, 0.63625E+00_JPRB, 0.63505E+00_JPRB, 0.62864E+00_JPRB, 0.61646E+00_JPRB, 0.60368E+00_JPRB ,&
&  0.58964E+00_JPRB, 0.57336E+00_JPRB, 0.55770E+00_JPRB, 0.54144E+00_JPRB, 0.52627E+00_JPRB, 0.51134E+00_JPRB, 0.49888E+00_JPRB ,&
&  0.48691E+00_JPRB, 0.47776E+00_JPRB, 0.46939E+00_JPRB, 0.46261E+00_JPRB, 0.45621E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.46060E+00_JPRB, 0.45607E+00_JPRB, 0.45150E+00_JPRB, 0.44574E+00_JPRB, 0.43988E+00_JPRB, 0.43476E+00_JPRB, 0.43104E+00_JPRB ,&
&  0.42841E+00_JPRB, 0.42566E+00_JPRB, 0.42311E+00_JPRB, 0.41970E+00_JPRB, 0.41657E+00_JPRB, 0.41419E+00_JPRB, 0.41238E+00_JPRB ,&
&  0.41206E+00_JPRB, 0.40809E+00_JPRB, 0.40347E+00_JPRB, 0.39738E+00_JPRB, 0.39156E+00_JPRB, 0.38504E+00_JPRB, 0.37869E+00_JPRB ,&
&  0.37308E+00_JPRB, 0.36230E+00_JPRB, 0.34797E+00_JPRB, 0.33859E+00_JPRB, 0.32555E+00_JPRB, 0.32102E+00_JPRB, 0.32168E+00_JPRB ,&
&  0.33881E+00_JPRB, 0.33177E+00_JPRB, 0.33403E+00_JPRB, 0.32233E+00_JPRB, 0.31149E+00_JPRB, 0.30126E+00_JPRB, 0.29478E+00_JPRB ,&
&  0.28943E+00_JPRB, 0.28549E+00_JPRB, 0.28488E+00_JPRB, 0.28641E+00_JPRB, 0.28798E+00_JPRB, 0.29541E+00_JPRB, 0.30830E+00_JPRB ,&
&  0.34373E+00_JPRB, 0.43053E+00_JPRB, 0.44116E+00_JPRB, 0.45436E+00_JPRB, 0.45051E+00_JPRB, 0.44956E+00_JPRB, 0.44924E+00_JPRB ,&
&  0.44718E+00_JPRB, 0.44418E+00_JPRB, 0.43751E+00_JPRB, 0.43121E+00_JPRB, 0.43631E+00_JPRB, 0.44048E+00_JPRB, 0.44728E+00_JPRB ,&
&  0.44984E+00_JPRB, 0.45260E+00_JPRB, 0.45076E+00_JPRB, 0.45051E+00_JPRB, 0.45470E+00_JPRB, 0.45339E+00_JPRB, 0.44895E+00_JPRB ,&
&  0.45469E+00_JPRB, 0.46574E+00_JPRB, 0.48423E+00_JPRB, 0.50915E+00_JPRB, 0.52748E+00_JPRB, 0.54381E+00_JPRB, 0.56786E+00_JPRB ,&
&  0.58526E+00_JPRB, 0.61354E+00_JPRB, 0.64351E+00_JPRB, 0.65797E+00_JPRB, 0.66187E+00_JPRB, 0.66020E+00_JPRB, 0.65668E+00_JPRB ,&
&  0.65766E+00_JPRB, 0.64995E+00_JPRB, 0.64727E+00_JPRB, 0.64636E+00_JPRB, 0.63886E+00_JPRB, 0.62673E+00_JPRB, 0.61419E+00_JPRB ,&
&  0.59905E+00_JPRB, 0.58477E+00_JPRB, 0.57206E+00_JPRB, 0.55731E+00_JPRB, 0.54198E+00_JPRB, 0.52710E+00_JPRB, 0.51268E+00_JPRB ,&
&  0.49860E+00_JPRB, 0.48715E+00_JPRB, 0.47667E+00_JPRB, 0.46995E+00_JPRB, 0.46418E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.47309E+00_JPRB, 0.46874E+00_JPRB, 0.46493E+00_JPRB, 0.46122E+00_JPRB, 0.45685E+00_JPRB, 0.45225E+00_JPRB, 0.44743E+00_JPRB ,&
&  0.44351E+00_JPRB, 0.43980E+00_JPRB, 0.43637E+00_JPRB, 0.43129E+00_JPRB, 0.42636E+00_JPRB, 0.42076E+00_JPRB, 0.41702E+00_JPRB ,&
&  0.41354E+00_JPRB, 0.40854E+00_JPRB, 0.40009E+00_JPRB, 0.39244E+00_JPRB, 0.38704E+00_JPRB, 0.38291E+00_JPRB, 0.37713E+00_JPRB ,&
&  0.37223E+00_JPRB, 0.36460E+00_JPRB, 0.34850E+00_JPRB, 0.33266E+00_JPRB, 0.32097E+00_JPRB, 0.32156E+00_JPRB, 0.34639E+00_JPRB ,&
&  0.35884E+00_JPRB, 0.36254E+00_JPRB, 0.36635E+00_JPRB, 0.36321E+00_JPRB, 0.35834E+00_JPRB, 0.31182E+00_JPRB, 0.30662E+00_JPRB ,&
&  0.29368E+00_JPRB, 0.28454E+00_JPRB, 0.28147E+00_JPRB, 0.28102E+00_JPRB, 0.28482E+00_JPRB, 0.29457E+00_JPRB, 0.31066E+00_JPRB ,&
&  0.34460E+00_JPRB, 0.44046E+00_JPRB, 0.45383E+00_JPRB, 0.46692E+00_JPRB, 0.46718E+00_JPRB, 0.46675E+00_JPRB, 0.46913E+00_JPRB ,&
&  0.46748E+00_JPRB, 0.46790E+00_JPRB, 0.46804E+00_JPRB, 0.46990E+00_JPRB, 0.47174E+00_JPRB, 0.46836E+00_JPRB, 0.47101E+00_JPRB ,&
&  0.47284E+00_JPRB, 0.47592E+00_JPRB, 0.47363E+00_JPRB, 0.46745E+00_JPRB, 0.46835E+00_JPRB, 0.45989E+00_JPRB, 0.45446E+00_JPRB ,&
&  0.46737E+00_JPRB, 0.49247E+00_JPRB, 0.51417E+00_JPRB, 0.53593E+00_JPRB, 0.55413E+00_JPRB, 0.57586E+00_JPRB, 0.60293E+00_JPRB ,&
&  0.62110E+00_JPRB, 0.64706E+00_JPRB, 0.67734E+00_JPRB, 0.68831E+00_JPRB, 0.68768E+00_JPRB, 0.68737E+00_JPRB, 0.69125E+00_JPRB ,&
&  0.69665E+00_JPRB, 0.70316E+00_JPRB, 0.68645E+00_JPRB, 0.67710E+00_JPRB, 0.65753E+00_JPRB, 0.62886E+00_JPRB, 0.60540E+00_JPRB ,&
&  0.57740E+00_JPRB, 0.55304E+00_JPRB, 0.53778E+00_JPRB, 0.52680E+00_JPRB, 0.51848E+00_JPRB, 0.51270E+00_JPRB, 0.50763E+00_JPRB ,&
&  0.50279E+00_JPRB, 0.49659E+00_JPRB, 0.48982E+00_JPRB, 0.48317E+00_JPRB, 0.47761E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.48361E+00_JPRB, 0.48185E+00_JPRB, 0.47853E+00_JPRB, 0.47503E+00_JPRB, 0.46962E+00_JPRB, 0.46533E+00_JPRB, 0.46114E+00_JPRB ,&
&  0.45815E+00_JPRB, 0.45598E+00_JPRB, 0.45412E+00_JPRB, 0.45169E+00_JPRB, 0.44859E+00_JPRB, 0.44596E+00_JPRB, 0.44187E+00_JPRB ,&
&  0.43464E+00_JPRB, 0.41092E+00_JPRB, 0.39342E+00_JPRB, 0.38736E+00_JPRB, 0.38246E+00_JPRB, 0.37904E+00_JPRB, 0.37708E+00_JPRB ,&
&  0.37088E+00_JPRB, 0.36347E+00_JPRB, 0.35231E+00_JPRB, 0.33466E+00_JPRB, 0.32697E+00_JPRB, 0.33080E+00_JPRB, 0.33953E+00_JPRB ,&
&  0.35489E+00_JPRB, 0.36300E+00_JPRB, 0.35708E+00_JPRB, 0.35916E+00_JPRB, 0.37768E+00_JPRB, 0.37137E+00_JPRB, 0.32214E+00_JPRB ,&
&  0.29948E+00_JPRB, 0.28587E+00_JPRB, 0.28055E+00_JPRB, 0.27913E+00_JPRB, 0.28326E+00_JPRB, 0.29745E+00_JPRB, 0.33540E+00_JPRB ,&
&  0.40942E+00_JPRB, 0.48953E+00_JPRB, 0.47939E+00_JPRB, 0.47638E+00_JPRB, 0.48154E+00_JPRB, 0.48456E+00_JPRB, 0.48770E+00_JPRB ,&
&  0.49018E+00_JPRB, 0.49664E+00_JPRB, 0.50532E+00_JPRB, 0.51286E+00_JPRB, 0.52206E+00_JPRB, 0.53055E+00_JPRB, 0.53148E+00_JPRB ,&
&  0.53239E+00_JPRB, 0.51646E+00_JPRB, 0.50758E+00_JPRB, 0.49433E+00_JPRB, 0.48780E+00_JPRB, 0.47426E+00_JPRB, 0.47331E+00_JPRB ,&
&  0.49057E+00_JPRB, 0.51523E+00_JPRB, 0.54085E+00_JPRB, 0.56089E+00_JPRB, 0.58226E+00_JPRB, 0.61230E+00_JPRB, 0.64486E+00_JPRB ,&
&  0.67336E+00_JPRB, 0.71173E+00_JPRB, 0.76828E+00_JPRB, 0.75928E+00_JPRB, 0.74828E+00_JPRB, 0.74369E+00_JPRB, 0.74767E+00_JPRB ,&
&  0.74966E+00_JPRB, 0.75969E+00_JPRB, 0.73083E+00_JPRB, 0.68532E+00_JPRB, 0.64674E+00_JPRB, 0.59782E+00_JPRB, 0.56724E+00_JPRB ,&
&  0.54839E+00_JPRB, 0.52037E+00_JPRB, 0.50652E+00_JPRB, 0.49913E+00_JPRB, 0.49495E+00_JPRB, 0.48938E+00_JPRB, 0.48468E+00_JPRB ,&
&  0.48325E+00_JPRB, 0.48326E+00_JPRB, 0.48451E+00_JPRB, 0.48442E+00_JPRB, 0.48431E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.47728E+00_JPRB, 0.48218E+00_JPRB, 0.48630E+00_JPRB, 0.48625E+00_JPRB, 0.47633E+00_JPRB, 0.45226E+00_JPRB, 0.44032E+00_JPRB ,&
&  0.43365E+00_JPRB, 0.43438E+00_JPRB, 0.43302E+00_JPRB, 0.42892E+00_JPRB, 0.42774E+00_JPRB, 0.43182E+00_JPRB, 0.43688E+00_JPRB ,&
&  0.43876E+00_JPRB, 0.43439E+00_JPRB, 0.42576E+00_JPRB, 0.40843E+00_JPRB, 0.39356E+00_JPRB, 0.38142E+00_JPRB, 0.37586E+00_JPRB ,&
&  0.37565E+00_JPRB, 0.37193E+00_JPRB, 0.36315E+00_JPRB, 0.35123E+00_JPRB, 0.34156E+00_JPRB, 0.33252E+00_JPRB, 0.33390E+00_JPRB ,&
&  0.34262E+00_JPRB, 0.34788E+00_JPRB, 0.35783E+00_JPRB, 0.36267E+00_JPRB, 0.37187E+00_JPRB, 0.39709E+00_JPRB, 0.34787E+00_JPRB ,&
&  0.31452E+00_JPRB, 0.30001E+00_JPRB, 0.28880E+00_JPRB, 0.28569E+00_JPRB, 0.29137E+00_JPRB, 0.31160E+00_JPRB, 0.35623E+00_JPRB ,&
&  0.47090E+00_JPRB, 0.48380E+00_JPRB, 0.48831E+00_JPRB, 0.49959E+00_JPRB, 0.50845E+00_JPRB, 0.51111E+00_JPRB, 0.51776E+00_JPRB ,&
&  0.53095E+00_JPRB, 0.55275E+00_JPRB, 0.57394E+00_JPRB, 0.57112E+00_JPRB, 0.56193E+00_JPRB, 0.53951E+00_JPRB, 0.57796E+00_JPRB ,&
&  0.70723E+00_JPRB, 0.66541E+00_JPRB, 0.59633E+00_JPRB, 0.54252E+00_JPRB, 0.51264E+00_JPRB, 0.49770E+00_JPRB, 0.50668E+00_JPRB ,&
&  0.52304E+00_JPRB, 0.54497E+00_JPRB, 0.57767E+00_JPRB, 0.60188E+00_JPRB, 0.62493E+00_JPRB, 0.66058E+00_JPRB, 0.71257E+00_JPRB ,&
&  0.76338E+00_JPRB, 0.83179E+00_JPRB, 0.10003E+01_JPRB, 0.87480E+00_JPRB, 0.81057E+00_JPRB, 0.79195E+00_JPRB, 0.78775E+00_JPRB ,&
&  0.76695E+00_JPRB, 0.77063E+00_JPRB, 0.72278E+00_JPRB, 0.65603E+00_JPRB, 0.62051E+00_JPRB, 0.58137E+00_JPRB, 0.55392E+00_JPRB ,&
&  0.52765E+00_JPRB, 0.50592E+00_JPRB, 0.49737E+00_JPRB, 0.48945E+00_JPRB, 0.48823E+00_JPRB, 0.48586E+00_JPRB, 0.48195E+00_JPRB ,&
&  0.47671E+00_JPRB, 0.47256E+00_JPRB, 0.46947E+00_JPRB, 0.46856E+00_JPRB, 0.47062E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.45106E+00_JPRB, 0.47879E+00_JPRB, 0.50706E+00_JPRB, 0.49650E+00_JPRB, 0.48052E+00_JPRB, 0.43335E+00_JPRB, 0.39983E+00_JPRB ,&
&  0.37956E+00_JPRB, 0.37723E+00_JPRB, 0.37750E+00_JPRB, 0.37981E+00_JPRB, 0.38454E+00_JPRB, 0.39266E+00_JPRB, 0.39760E+00_JPRB ,&
&  0.40225E+00_JPRB, 0.39890E+00_JPRB, 0.39749E+00_JPRB, 0.39274E+00_JPRB, 0.38416E+00_JPRB, 0.37272E+00_JPRB, 0.36461E+00_JPRB ,&
&  0.36134E+00_JPRB, 0.36141E+00_JPRB, 0.35898E+00_JPRB, 0.35429E+00_JPRB, 0.35053E+00_JPRB, 0.34462E+00_JPRB, 0.34953E+00_JPRB ,&
&  0.34431E+00_JPRB, 0.34209E+00_JPRB, 0.35348E+00_JPRB, 0.36865E+00_JPRB, 0.37904E+00_JPRB, 0.40302E+00_JPRB, 0.36892E+00_JPRB ,&
&  0.32994E+00_JPRB, 0.31170E+00_JPRB, 0.30409E+00_JPRB, 0.30969E+00_JPRB, 0.34798E+00_JPRB, 0.45413E+00_JPRB, 0.48512E+00_JPRB ,&
&  0.50054E+00_JPRB, 0.51330E+00_JPRB, 0.52811E+00_JPRB, 0.54779E+00_JPRB, 0.55170E+00_JPRB, 0.55629E+00_JPRB, 0.57178E+00_JPRB ,&
&  0.59515E+00_JPRB, 0.63182E+00_JPRB, 0.66924E+00_JPRB, 0.57634E+00_JPRB, 0.53574E+00_JPRB, 0.54857E+00_JPRB, 0.56798E+00_JPRB ,&
&  0.59501E+00_JPRB, 0.58596E+00_JPRB, 0.57812E+00_JPRB, 0.59187E+00_JPRB, 0.58997E+00_JPRB, 0.58895E+00_JPRB, 0.57762E+00_JPRB ,&
&  0.57547E+00_JPRB, 0.58974E+00_JPRB, 0.63627E+00_JPRB, 0.66126E+00_JPRB, 0.70227E+00_JPRB, 0.76637E+00_JPRB, 0.82358E+00_JPRB ,&
&  0.88789E+00_JPRB, 0.93459E+00_JPRB, 0.95497E+00_JPRB, 0.91959E+00_JPRB, 0.89345E+00_JPRB, 0.89211E+00_JPRB, 0.87528E+00_JPRB ,&
&  0.81318E+00_JPRB, 0.77022E+00_JPRB, 0.73445E+00_JPRB, 0.69693E+00_JPRB, 0.64273E+00_JPRB, 0.59821E+00_JPRB, 0.55201E+00_JPRB ,&
&  0.50667E+00_JPRB, 0.47379E+00_JPRB, 0.45926E+00_JPRB, 0.45277E+00_JPRB, 0.45305E+00_JPRB, 0.45308E+00_JPRB, 0.44716E+00_JPRB ,&
&  0.43600E+00_JPRB, 0.42321E+00_JPRB, 0.41944E+00_JPRB, 0.42525E+00_JPRB, 0.44118E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.50478E+00_JPRB, 0.50508E+00_JPRB, 0.47731E+00_JPRB, 0.49227E+00_JPRB, 0.46961E+00_JPRB, 0.45464E+00_JPRB, 0.42887E+00_JPRB ,&
&  0.41836E+00_JPRB, 0.39408E+00_JPRB, 0.37881E+00_JPRB, 0.35499E+00_JPRB, 0.34036E+00_JPRB, 0.33008E+00_JPRB, 0.33951E+00_JPRB ,&
&  0.35417E+00_JPRB, 0.36382E+00_JPRB, 0.36835E+00_JPRB, 0.36689E+00_JPRB, 0.36436E+00_JPRB, 0.35902E+00_JPRB, 0.35378E+00_JPRB ,&
&  0.34991E+00_JPRB, 0.35261E+00_JPRB, 0.35216E+00_JPRB, 0.35168E+00_JPRB, 0.35027E+00_JPRB, 0.35130E+00_JPRB, 0.35469E+00_JPRB ,&
&  0.34842E+00_JPRB, 0.34170E+00_JPRB, 0.34530E+00_JPRB, 0.36552E+00_JPRB, 0.38078E+00_JPRB, 0.40147E+00_JPRB, 0.37553E+00_JPRB ,&
&  0.33874E+00_JPRB, 0.33104E+00_JPRB, 0.34439E+00_JPRB, 0.44523E+00_JPRB, 0.46342E+00_JPRB, 0.46876E+00_JPRB, 0.49407E+00_JPRB ,&
&  0.47313E+00_JPRB, 0.46863E+00_JPRB, 0.48430E+00_JPRB, 0.55456E+00_JPRB, 0.57750E+00_JPRB, 0.59045E+00_JPRB, 0.61314E+00_JPRB ,&
&  0.64476E+00_JPRB, 0.69013E+00_JPRB, 0.74281E+00_JPRB, 0.65277E+00_JPRB, 0.68286E+00_JPRB, 0.80877E+00_JPRB, 0.71260E+00_JPRB ,&
&  0.66102E+00_JPRB, 0.63218E+00_JPRB, 0.64616E+00_JPRB, 0.64578E+00_JPRB, 0.63625E+00_JPRB, 0.64952E+00_JPRB, 0.65632E+00_JPRB ,&
&  0.63617E+00_JPRB, 0.65522E+00_JPRB, 0.71737E+00_JPRB, 0.78256E+00_JPRB, 0.84055E+00_JPRB, 0.91752E+00_JPRB, 0.99829E+00_JPRB ,&
&  0.11001E+01_JPRB, 0.11938E+01_JPRB, 0.12609E+01_JPRB, 0.12507E+01_JPRB, 0.12191E+01_JPRB, 0.11597E+01_JPRB, 0.10748E+01_JPRB ,&
&  0.95805E+00_JPRB, 0.87434E+00_JPRB, 0.81744E+00_JPRB, 0.77232E+00_JPRB, 0.71671E+00_JPRB, 0.64442E+00_JPRB, 0.55092E+00_JPRB ,&
&  0.47964E+00_JPRB, 0.43323E+00_JPRB, 0.41515E+00_JPRB, 0.41168E+00_JPRB, 0.41630E+00_JPRB, 0.42011E+00_JPRB, 0.41674E+00_JPRB ,&
&  0.40927E+00_JPRB, 0.40538E+00_JPRB, 0.40682E+00_JPRB, 0.42462E+00_JPRB, 0.47017E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.51384E+00_JPRB, 0.50516E+00_JPRB, 0.49321E+00_JPRB, 0.49080E+00_JPRB, 0.50055E+00_JPRB, 0.47612E+00_JPRB, 0.42299E+00_JPRB ,&
&  0.44049E+00_JPRB, 0.39294E+00_JPRB, 0.37976E+00_JPRB, 0.36769E+00_JPRB, 0.35819E+00_JPRB, 0.33693E+00_JPRB, 0.31737E+00_JPRB ,&
&  0.31702E+00_JPRB, 0.32861E+00_JPRB, 0.34292E+00_JPRB, 0.35377E+00_JPRB, 0.35606E+00_JPRB, 0.35208E+00_JPRB, 0.34738E+00_JPRB ,&
&  0.34453E+00_JPRB, 0.34648E+00_JPRB, 0.35111E+00_JPRB, 0.35774E+00_JPRB, 0.35998E+00_JPRB, 0.36204E+00_JPRB, 0.36709E+00_JPRB ,&
&  0.37087E+00_JPRB, 0.37246E+00_JPRB, 0.36458E+00_JPRB, 0.37387E+00_JPRB, 0.38002E+00_JPRB, 0.39422E+00_JPRB, 0.42396E+00_JPRB ,&
&  0.35946E+00_JPRB, 0.36369E+00_JPRB, 0.45710E+00_JPRB, 0.45924E+00_JPRB, 0.47100E+00_JPRB, 0.48438E+00_JPRB, 0.49424E+00_JPRB ,&
&  0.50958E+00_JPRB, 0.53461E+00_JPRB, 0.56523E+00_JPRB, 0.59249E+00_JPRB, 0.61556E+00_JPRB, 0.64585E+00_JPRB, 0.67031E+00_JPRB ,&
&  0.73881E+00_JPRB, 0.66040E+00_JPRB, 0.61702E+00_JPRB, 0.65464E+00_JPRB, 0.91698E+00_JPRB, 0.91931E+00_JPRB, 0.87774E+00_JPRB ,&
&  0.83085E+00_JPRB, 0.76867E+00_JPRB, 0.73889E+00_JPRB, 0.74133E+00_JPRB, 0.74484E+00_JPRB, 0.77626E+00_JPRB, 0.81888E+00_JPRB ,&
&  0.81008E+00_JPRB, 0.85242E+00_JPRB, 0.91432E+00_JPRB, 0.97240E+00_JPRB, 0.10352E+01_JPRB, 0.11303E+01_JPRB, 0.12398E+01_JPRB ,&
&  0.13625E+01_JPRB, 0.14697E+01_JPRB, 0.15186E+01_JPRB, 0.14971E+01_JPRB, 0.14305E+01_JPRB, 0.13421E+01_JPRB, 0.12310E+01_JPRB ,&
&  0.10864E+01_JPRB, 0.98295E+00_JPRB, 0.91305E+00_JPRB, 0.84530E+00_JPRB, 0.78452E+00_JPRB, 0.70472E+00_JPRB, 0.58877E+00_JPRB ,&
&  0.46958E+00_JPRB, 0.41694E+00_JPRB, 0.40938E+00_JPRB, 0.40891E+00_JPRB, 0.41216E+00_JPRB, 0.41823E+00_JPRB, 0.42566E+00_JPRB ,&
&  0.42890E+00_JPRB, 0.42387E+00_JPRB, 0.42655E+00_JPRB, 0.44807E+00_JPRB, 0.47564E+00_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.48893E+00_JPRB, 0.48328E+00_JPRB, 0.48154E+00_JPRB, 0.50072E+00_JPRB, 0.50634E+00_JPRB, 0.48017E+00_JPRB, 0.44396E+00_JPRB ,&
&  0.54977E+00_JPRB, 0.49880E+00_JPRB, 0.56153E+00_JPRB, 0.42885E+00_JPRB, 0.37403E+00_JPRB, 0.35985E+00_JPRB, 0.34412E+00_JPRB ,&
&  0.33205E+00_JPRB, 0.32756E+00_JPRB, 0.32752E+00_JPRB, 0.33573E+00_JPRB, 0.34069E+00_JPRB, 0.33887E+00_JPRB, 0.33658E+00_JPRB ,&
&  0.33555E+00_JPRB, 0.33924E+00_JPRB, 0.34900E+00_JPRB, 0.35563E+00_JPRB, 0.36109E+00_JPRB, 0.36688E+00_JPRB, 0.37728E+00_JPRB ,&
&  0.36652E+00_JPRB, 0.36469E+00_JPRB, 0.37194E+00_JPRB, 0.37631E+00_JPRB, 0.38556E+00_JPRB, 0.39081E+00_JPRB, 0.40659E+00_JPRB ,&
&  0.45338E+00_JPRB, 0.42172E+00_JPRB, 0.49576E+00_JPRB, 0.49956E+00_JPRB, 0.50786E+00_JPRB, 0.51717E+00_JPRB, 0.52582E+00_JPRB ,&
&  0.53474E+00_JPRB, 0.55293E+00_JPRB, 0.58066E+00_JPRB, 0.60535E+00_JPRB, 0.64193E+00_JPRB, 0.68845E+00_JPRB, 0.72161E+00_JPRB ,&
&  0.81608E+00_JPRB, 0.65575E+00_JPRB, 0.71314E+00_JPRB, 0.73935E+00_JPRB, 0.93719E+00_JPRB, 0.11545E+01_JPRB, 0.10863E+01_JPRB ,&
&  0.11414E+01_JPRB, 0.10310E+01_JPRB, 0.94017E+00_JPRB, 0.87334E+00_JPRB, 0.87699E+00_JPRB, 0.91000E+00_JPRB, 0.95533E+00_JPRB ,&
&  0.99235E+00_JPRB, 0.10349E+01_JPRB, 0.11211E+01_JPRB, 0.11976E+01_JPRB, 0.12782E+01_JPRB, 0.13742E+01_JPRB, 0.14687E+01_JPRB ,&
&  0.15565E+01_JPRB, 0.16122E+01_JPRB, 0.16407E+01_JPRB, 0.15504E+01_JPRB, 0.14711E+01_JPRB, 0.13850E+01_JPRB, 0.12191E+01_JPRB ,&
&  0.10893E+01_JPRB, 0.98969E+00_JPRB, 0.90902E+00_JPRB, 0.84803E+00_JPRB, 0.74481E+00_JPRB, 0.65906E+00_JPRB, 0.57747E+00_JPRB ,&
&  0.47124E+00_JPRB, 0.43365E+00_JPRB, 0.44654E+00_JPRB, 0.45138E+00_JPRB, 0.44648E+00_JPRB, 0.44963E+00_JPRB, 0.45811E+00_JPRB ,&
&  0.45122E+00_JPRB, 0.45357E+00_JPRB, 0.47423E+00_JPRB, 0.48301E+00_JPRB, 0.48720E+00_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.50410E+00_JPRB, 0.51009E+00_JPRB, 0.51675E+00_JPRB, 0.52548E+00_JPRB, 0.51276E+00_JPRB, 0.53317E+00_JPRB, 0.47630E+00_JPRB ,&
&  0.52838E+00_JPRB, 0.52413E+00_JPRB, 0.50442E+00_JPRB, 0.52059E+00_JPRB, 0.56683E+00_JPRB, 0.53547E+00_JPRB, 0.40785E+00_JPRB ,&
&  0.35426E+00_JPRB, 0.34821E+00_JPRB, 0.35219E+00_JPRB, 0.35466E+00_JPRB, 0.35646E+00_JPRB, 0.35577E+00_JPRB, 0.35645E+00_JPRB ,&
&  0.35736E+00_JPRB, 0.35708E+00_JPRB, 0.36313E+00_JPRB, 0.36952E+00_JPRB, 0.37278E+00_JPRB, 0.37663E+00_JPRB, 0.38812E+00_JPRB ,&
&  0.37374E+00_JPRB, 0.36397E+00_JPRB, 0.36016E+00_JPRB, 0.34980E+00_JPRB, 0.36880E+00_JPRB, 0.38462E+00_JPRB, 0.41121E+00_JPRB ,&
&  0.45067E+00_JPRB, 0.49116E+00_JPRB, 0.51089E+00_JPRB, 0.52715E+00_JPRB, 0.54151E+00_JPRB, 0.55748E+00_JPRB, 0.57479E+00_JPRB ,&
&  0.59131E+00_JPRB, 0.61123E+00_JPRB, 0.63936E+00_JPRB, 0.66068E+00_JPRB, 0.72187E+00_JPRB, 0.72092E+00_JPRB, 0.82255E+00_JPRB ,&
&  0.87858E+00_JPRB, 0.99608E+00_JPRB, 0.10701E+01_JPRB, 0.92100E+00_JPRB, 0.10613E+01_JPRB, 0.12096E+01_JPRB, 0.11611E+01_JPRB ,&
&  0.11355E+01_JPRB, 0.10776E+01_JPRB, 0.10718E+01_JPRB, 0.10912E+01_JPRB, 0.11785E+01_JPRB, 0.12424E+01_JPRB, 0.12681E+01_JPRB ,&
&  0.12899E+01_JPRB, 0.14124E+01_JPRB, 0.15019E+01_JPRB, 0.15650E+01_JPRB, 0.15981E+01_JPRB, 0.16262E+01_JPRB, 0.16702E+01_JPRB ,&
&  0.17025E+01_JPRB, 0.17276E+01_JPRB, 0.17165E+01_JPRB, 0.16558E+01_JPRB, 0.15684E+01_JPRB, 0.14490E+01_JPRB, 0.12888E+01_JPRB ,&
&  0.11450E+01_JPRB, 0.96949E+00_JPRB, 0.82895E+00_JPRB, 0.74177E+00_JPRB, 0.66069E+00_JPRB, 0.58257E+00_JPRB, 0.52729E+00_JPRB ,&
&  0.47095E+00_JPRB, 0.46750E+00_JPRB, 0.51576E+00_JPRB, 0.51074E+00_JPRB, 0.51135E+00_JPRB, 0.51429E+00_JPRB, 0.47740E+00_JPRB ,&
&  0.46383E+00_JPRB, 0.49256E+00_JPRB, 0.49927E+00_JPRB, 0.50041E+00_JPRB, 0.50133E+00_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.54816E+00_JPRB, 0.54789E+00_JPRB, 0.55295E+00_JPRB, 0.54647E+00_JPRB, 0.53570E+00_JPRB, 0.53355E+00_JPRB, 0.53354E+00_JPRB ,&
&  0.52620E+00_JPRB, 0.52547E+00_JPRB, 0.52389E+00_JPRB, 0.51732E+00_JPRB, 0.51834E+00_JPRB, 0.54524E+00_JPRB, 0.49321E+00_JPRB ,&
&  0.40632E+00_JPRB, 0.38827E+00_JPRB, 0.38063E+00_JPRB, 0.39083E+00_JPRB, 0.40651E+00_JPRB, 0.41133E+00_JPRB, 0.41368E+00_JPRB ,&
&  0.41812E+00_JPRB, 0.41139E+00_JPRB, 0.41413E+00_JPRB, 0.41579E+00_JPRB, 0.41376E+00_JPRB, 0.41609E+00_JPRB, 0.41863E+00_JPRB ,&
&  0.39138E+00_JPRB, 0.37214E+00_JPRB, 0.36211E+00_JPRB, 0.35639E+00_JPRB, 0.37092E+00_JPRB, 0.39803E+00_JPRB, 0.43857E+00_JPRB ,&
&  0.48636E+00_JPRB, 0.52105E+00_JPRB, 0.55143E+00_JPRB, 0.58236E+00_JPRB, 0.60793E+00_JPRB, 0.63585E+00_JPRB, 0.66164E+00_JPRB ,&
&  0.68249E+00_JPRB, 0.70058E+00_JPRB, 0.71148E+00_JPRB, 0.73044E+00_JPRB, 0.73438E+00_JPRB, 0.83727E+00_JPRB, 0.10289E+01_JPRB ,&
&  0.10727E+01_JPRB, 0.11590E+01_JPRB, 0.11907E+01_JPRB, 0.12666E+01_JPRB, 0.13334E+01_JPRB, 0.12870E+01_JPRB, 0.12750E+01_JPRB ,&
&  0.13278E+01_JPRB, 0.13892E+01_JPRB, 0.15568E+01_JPRB, 0.15578E+01_JPRB, 0.15789E+01_JPRB, 0.16190E+01_JPRB, 0.16015E+01_JPRB ,&
&  0.16323E+01_JPRB, 0.16799E+01_JPRB, 0.17704E+01_JPRB, 0.17790E+01_JPRB, 0.17578E+01_JPRB, 0.17735E+01_JPRB, 0.17730E+01_JPRB ,&
&  0.17882E+01_JPRB, 0.17994E+01_JPRB, 0.17153E+01_JPRB, 0.16373E+01_JPRB, 0.14964E+01_JPRB, 0.14876E+01_JPRB, 0.13775E+01_JPRB ,&
&  0.11563E+01_JPRB, 0.87186E+00_JPRB, 0.79015E+00_JPRB, 0.70074E+00_JPRB, 0.63949E+00_JPRB, 0.56736E+00_JPRB, 0.51043E+00_JPRB ,&
&  0.48613E+00_JPRB, 0.52041E+00_JPRB, 0.52368E+00_JPRB, 0.53369E+00_JPRB, 0.54409E+00_JPRB, 0.55701E+00_JPRB, 0.49017E+00_JPRB ,&
&  0.49901E+00_JPRB, 0.54484E+00_JPRB, 0.54317E+00_JPRB, 0.54712E+00_JPRB, 0.54750E+00_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.60649E+00_JPRB, 0.60011E+00_JPRB, 0.59467E+00_JPRB, 0.58628E+00_JPRB, 0.57480E+00_JPRB, 0.56632E+00_JPRB, 0.55787E+00_JPRB ,&
&  0.55038E+00_JPRB, 0.55267E+00_JPRB, 0.55639E+00_JPRB, 0.54470E+00_JPRB, 0.53406E+00_JPRB, 0.53340E+00_JPRB, 0.57334E+00_JPRB ,&
&  0.44923E+00_JPRB, 0.43404E+00_JPRB, 0.43640E+00_JPRB, 0.46856E+00_JPRB, 0.54502E+00_JPRB, 0.54996E+00_JPRB, 0.56631E+00_JPRB ,&
&  0.55779E+00_JPRB, 0.54515E+00_JPRB, 0.53784E+00_JPRB, 0.53065E+00_JPRB, 0.51488E+00_JPRB, 0.50656E+00_JPRB, 0.49179E+00_JPRB ,&
&  0.44553E+00_JPRB, 0.40941E+00_JPRB, 0.39698E+00_JPRB, 0.39333E+00_JPRB, 0.41849E+00_JPRB, 0.44777E+00_JPRB, 0.50222E+00_JPRB ,&
&  0.55172E+00_JPRB, 0.59739E+00_JPRB, 0.64193E+00_JPRB, 0.69616E+00_JPRB, 0.74055E+00_JPRB, 0.78279E+00_JPRB, 0.80950E+00_JPRB ,&
&  0.82169E+00_JPRB, 0.82232E+00_JPRB, 0.83447E+00_JPRB, 0.81928E+00_JPRB, 0.82597E+00_JPRB, 0.10535E+01_JPRB, 0.13330E+01_JPRB ,&
&  0.13886E+01_JPRB, 0.14332E+01_JPRB, 0.17228E+01_JPRB, 0.20383E+01_JPRB, 0.19139E+01_JPRB, 0.17574E+01_JPRB, 0.16885E+01_JPRB ,&
&  0.17573E+01_JPRB, 0.18736E+01_JPRB, 0.19937E+01_JPRB, 0.20180E+01_JPRB, 0.19902E+01_JPRB, 0.20206E+01_JPRB, 0.20198E+01_JPRB ,&
&  0.19981E+01_JPRB, 0.20116E+01_JPRB, 0.19797E+01_JPRB, 0.19163E+01_JPRB, 0.18667E+01_JPRB, 0.18988E+01_JPRB, 0.18933E+01_JPRB ,&
&  0.18577E+01_JPRB, 0.17725E+01_JPRB, 0.16340E+01_JPRB, 0.15587E+01_JPRB, 0.13183E+01_JPRB, 0.12775E+01_JPRB, 0.14092E+01_JPRB ,&
&  0.11663E+01_JPRB, 0.92594E+00_JPRB, 0.79575E+00_JPRB, 0.71143E+00_JPRB, 0.67199E+00_JPRB, 0.63694E+00_JPRB, 0.57771E+00_JPRB ,&
&  0.52031E+00_JPRB, 0.54089E+00_JPRB, 0.59132E+00_JPRB, 0.62251E+00_JPRB, 0.63168E+00_JPRB, 0.64531E+00_JPRB, 0.58982E+00_JPRB ,&
&  0.62955E+00_JPRB, 0.61338E+00_JPRB, 0.60668E+00_JPRB, 0.61004E+00_JPRB, 0.60885E+00_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.66517E+00_JPRB, 0.64629E+00_JPRB, 0.63372E+00_JPRB, 0.61215E+00_JPRB, 0.59560E+00_JPRB, 0.58848E+00_JPRB, 0.58349E+00_JPRB ,&
&  0.58208E+00_JPRB, 0.57900E+00_JPRB, 0.57651E+00_JPRB, 0.56246E+00_JPRB, 0.55324E+00_JPRB, 0.54613E+00_JPRB, 0.56352E+00_JPRB ,&
&  0.56459E+00_JPRB, 0.50511E+00_JPRB, 0.50107E+00_JPRB, 0.51317E+00_JPRB, 0.62266E+00_JPRB, 0.70843E+00_JPRB, 0.77960E+00_JPRB ,&
&  0.80069E+00_JPRB, 0.81225E+00_JPRB, 0.80272E+00_JPRB, 0.76769E+00_JPRB, 0.72780E+00_JPRB, 0.69145E+00_JPRB, 0.64590E+00_JPRB ,&
&  0.58854E+00_JPRB, 0.53038E+00_JPRB, 0.50257E+00_JPRB, 0.50857E+00_JPRB, 0.56025E+00_JPRB, 0.53809E+00_JPRB, 0.59691E+00_JPRB ,&
&  0.65503E+00_JPRB, 0.72094E+00_JPRB, 0.77866E+00_JPRB, 0.85161E+00_JPRB, 0.90180E+00_JPRB, 0.93868E+00_JPRB, 0.94473E+00_JPRB ,&
&  0.95932E+00_JPRB, 0.93026E+00_JPRB, 0.82453E+00_JPRB, 0.86005E+00_JPRB, 0.95404E+00_JPRB, 0.13351E+01_JPRB, 0.17144E+01_JPRB ,&
&  0.19191E+01_JPRB, 0.22372E+01_JPRB, 0.26296E+01_JPRB, 0.27587E+01_JPRB, 0.24440E+01_JPRB, 0.22525E+01_JPRB, 0.22501E+01_JPRB ,&
&  0.23547E+01_JPRB, 0.24852E+01_JPRB, 0.25189E+01_JPRB, 0.23958E+01_JPRB, 0.24021E+01_JPRB, 0.24277E+01_JPRB, 0.24475E+01_JPRB ,&
&  0.24063E+01_JPRB, 0.23041E+01_JPRB, 0.21855E+01_JPRB, 0.20742E+01_JPRB, 0.19998E+01_JPRB, 0.18516E+01_JPRB, 0.18016E+01_JPRB ,&
&  0.18117E+01_JPRB, 0.15457E+01_JPRB, 0.13937E+01_JPRB, 0.14400E+01_JPRB, 0.11954E+01_JPRB, 0.11694E+01_JPRB, 0.12468E+01_JPRB ,&
&  0.10668E+01_JPRB, 0.89733E+00_JPRB, 0.80408E+00_JPRB, 0.71533E+00_JPRB, 0.68566E+00_JPRB, 0.71513E+00_JPRB, 0.71279E+00_JPRB ,&
&  0.67468E+00_JPRB, 0.66339E+00_JPRB, 0.77534E+00_JPRB, 0.82665E+00_JPRB, 0.85202E+00_JPRB, 0.82819E+00_JPRB, 0.79508E+00_JPRB ,&
&  0.76135E+00_JPRB, 0.73432E+00_JPRB, 0.71906E+00_JPRB, 0.70429E+00_JPRB, 0.68819E+00_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.72670E+00_JPRB, 0.72078E+00_JPRB, 0.69603E+00_JPRB, 0.66001E+00_JPRB, 0.63758E+00_JPRB, 0.63063E+00_JPRB, 0.61468E+00_JPRB ,&
&  0.59269E+00_JPRB, 0.57778E+00_JPRB, 0.57412E+00_JPRB, 0.56358E+00_JPRB, 0.56005E+00_JPRB, 0.56357E+00_JPRB, 0.56364E+00_JPRB ,&
&  0.55632E+00_JPRB, 0.56580E+00_JPRB, 0.53720E+00_JPRB, 0.55366E+00_JPRB, 0.64676E+00_JPRB, 0.90484E+00_JPRB, 0.10874E+01_JPRB ,&
&  0.11585E+01_JPRB, 0.11663E+01_JPRB, 0.11508E+01_JPRB, 0.11486E+01_JPRB, 0.11076E+01_JPRB, 0.10160E+01_JPRB, 0.96149E+00_JPRB ,&
&  0.90075E+00_JPRB, 0.81866E+00_JPRB, 0.71687E+00_JPRB, 0.72939E+00_JPRB, 0.74854E+00_JPRB, 0.70446E+00_JPRB, 0.79573E+00_JPRB ,&
&  0.86339E+00_JPRB, 0.91187E+00_JPRB, 0.96044E+00_JPRB, 0.10049E+01_JPRB, 0.10246E+01_JPRB, 0.95876E+00_JPRB, 0.87122E+00_JPRB ,&
&  0.83139E+00_JPRB, 0.86270E+00_JPRB, 0.89018E+00_JPRB, 0.10014E+01_JPRB, 0.13414E+01_JPRB, 0.19155E+01_JPRB, 0.23642E+01_JPRB ,&
&  0.26583E+01_JPRB, 0.27351E+01_JPRB, 0.25174E+01_JPRB, 0.26677E+01_JPRB, 0.29395E+01_JPRB, 0.28225E+01_JPRB, 0.26651E+01_JPRB ,&
&  0.26709E+01_JPRB, 0.26535E+01_JPRB, 0.25889E+01_JPRB, 0.26013E+01_JPRB, 0.27814E+01_JPRB, 0.28735E+01_JPRB, 0.28518E+01_JPRB ,&
&  0.24893E+01_JPRB, 0.23528E+01_JPRB, 0.22829E+01_JPRB, 0.21799E+01_JPRB, 0.19463E+01_JPRB, 0.18445E+01_JPRB, 0.18703E+01_JPRB ,&
&  0.18088E+01_JPRB, 0.17621E+01_JPRB, 0.14028E+01_JPRB, 0.13131E+01_JPRB, 0.10269E+01_JPRB, 0.99327E+00_JPRB, 0.11011E+01_JPRB ,&
&  0.10111E+01_JPRB, 0.92318E+00_JPRB, 0.80077E+00_JPRB, 0.74254E+00_JPRB, 0.81912E+00_JPRB, 0.89505E+00_JPRB, 0.10106E+01_JPRB ,&
&  0.10009E+01_JPRB, 0.90471E+00_JPRB, 0.10413E+01_JPRB, 0.10716E+01_JPRB, 0.11218E+01_JPRB, 0.10709E+01_JPRB, 0.98017E+00_JPRB ,&
&  0.91083E+00_JPRB, 0.86257E+00_JPRB, 0.81336E+00_JPRB, 0.76712E+00_JPRB, 0.74025E+00_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.73134E+00_JPRB, 0.71562E+00_JPRB, 0.68615E+00_JPRB, 0.65369E+00_JPRB, 0.63494E+00_JPRB, 0.62066E+00_JPRB, 0.60950E+00_JPRB ,&
&  0.60094E+00_JPRB, 0.58995E+00_JPRB, 0.58199E+00_JPRB, 0.57122E+00_JPRB, 0.56034E+00_JPRB, 0.55144E+00_JPRB, 0.54399E+00_JPRB ,&
&  0.54508E+00_JPRB, 0.59024E+00_JPRB, 0.57808E+00_JPRB, 0.59785E+00_JPRB, 0.73868E+00_JPRB, 0.10402E+01_JPRB, 0.13202E+01_JPRB ,&
&  0.14396E+01_JPRB, 0.14298E+01_JPRB, 0.14375E+01_JPRB, 0.14516E+01_JPRB, 0.14815E+01_JPRB, 0.14627E+01_JPRB, 0.14207E+01_JPRB ,&
&  0.13443E+01_JPRB, 0.10789E+01_JPRB, 0.10228E+01_JPRB, 0.96163E+00_JPRB, 0.99949E+00_JPRB, 0.10243E+01_JPRB, 0.10079E+01_JPRB ,&
&  0.98336E+00_JPRB, 0.98210E+00_JPRB, 0.98643E+00_JPRB, 0.94908E+00_JPRB, 0.92072E+00_JPRB, 0.86028E+00_JPRB, 0.84905E+00_JPRB ,&
&  0.91497E+00_JPRB, 0.95543E+00_JPRB, 0.11755E+01_JPRB, 0.15696E+01_JPRB, 0.22811E+01_JPRB, 0.30893E+01_JPRB, 0.31410E+01_JPRB ,&
&  0.25598E+01_JPRB, 0.24191E+01_JPRB, 0.32860E+01_JPRB, 0.30452E+01_JPRB, 0.30510E+01_JPRB, 0.33124E+01_JPRB, 0.31067E+01_JPRB ,&
&  0.28532E+01_JPRB, 0.26421E+01_JPRB, 0.25541E+01_JPRB, 0.26702E+01_JPRB, 0.34312E+01_JPRB, 0.35745E+01_JPRB, 0.33239E+01_JPRB ,&
&  0.27920E+01_JPRB, 0.26521E+01_JPRB, 0.24398E+01_JPRB, 0.21937E+01_JPRB, 0.20351E+01_JPRB, 0.21078E+01_JPRB, 0.19835E+01_JPRB ,&
&  0.18239E+01_JPRB, 0.19276E+01_JPRB, 0.15406E+01_JPRB, 0.11894E+01_JPRB, 0.99674E+00_JPRB, 0.10107E+01_JPRB, 0.10647E+01_JPRB ,&
&  0.10408E+01_JPRB, 0.97095E+00_JPRB, 0.89484E+00_JPRB, 0.97618E+00_JPRB, 0.12082E+01_JPRB, 0.13480E+01_JPRB, 0.13237E+01_JPRB ,&
&  0.13226E+01_JPRB, 0.15027E+01_JPRB, 0.14528E+01_JPRB, 0.14516E+01_JPRB, 0.15156E+01_JPRB, 0.13802E+01_JPRB, 0.11829E+01_JPRB ,&
&  0.10166E+01_JPRB, 0.91363E+00_JPRB, 0.84198E+00_JPRB, 0.79054E+00_JPRB, 0.75204E+00_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.75763E+00_JPRB, 0.72709E+00_JPRB, 0.70134E+00_JPRB, 0.67778E+00_JPRB, 0.65131E+00_JPRB, 0.63372E+00_JPRB, 0.61429E+00_JPRB ,&
&  0.59196E+00_JPRB, 0.57663E+00_JPRB, 0.58153E+00_JPRB, 0.59137E+00_JPRB, 0.58601E+00_JPRB, 0.55552E+00_JPRB, 0.54839E+00_JPRB ,&
&  0.58413E+00_JPRB, 0.62274E+00_JPRB, 0.61820E+00_JPRB, 0.68328E+00_JPRB, 0.82201E+00_JPRB, 0.10223E+01_JPRB, 0.12986E+01_JPRB ,&
&  0.14718E+01_JPRB, 0.14449E+01_JPRB, 0.14457E+01_JPRB, 0.16253E+01_JPRB, 0.18577E+01_JPRB, 0.19708E+01_JPRB, 0.19549E+01_JPRB ,&
&  0.17091E+01_JPRB, 0.15430E+01_JPRB, 0.14760E+01_JPRB, 0.13189E+01_JPRB, 0.12430E+01_JPRB, 0.11542E+01_JPRB, 0.10419E+01_JPRB ,&
&  0.95479E+00_JPRB, 0.92513E+00_JPRB, 0.90993E+00_JPRB, 0.89675E+00_JPRB, 0.93010E+00_JPRB, 0.97336E+00_JPRB, 0.99986E+00_JPRB ,&
&  0.10712E+01_JPRB, 0.12730E+01_JPRB, 0.16596E+01_JPRB, 0.22471E+01_JPRB, 0.26310E+01_JPRB, 0.36551E+01_JPRB, 0.34007E+01_JPRB ,&
&  0.31648E+01_JPRB, 0.30989E+01_JPRB, 0.30547E+01_JPRB, 0.32827E+01_JPRB, 0.27847E+01_JPRB, 0.29496E+01_JPRB, 0.30625E+01_JPRB ,&
&  0.29579E+01_JPRB, 0.28557E+01_JPRB, 0.29766E+01_JPRB, 0.30028E+01_JPRB, 0.35378E+01_JPRB, 0.44611E+01_JPRB, 0.40768E+01_JPRB ,&
&  0.33633E+01_JPRB, 0.29892E+01_JPRB, 0.25851E+01_JPRB, 0.23277E+01_JPRB, 0.19482E+01_JPRB, 0.19091E+01_JPRB, 0.16662E+01_JPRB ,&
&  0.15126E+01_JPRB, 0.12297E+01_JPRB, 0.13120E+01_JPRB, 0.11217E+01_JPRB, 0.10468E+01_JPRB, 0.10208E+01_JPRB, 0.10095E+01_JPRB ,&
&  0.10323E+01_JPRB, 0.10411E+01_JPRB, 0.11466E+01_JPRB, 0.14152E+01_JPRB, 0.17632E+01_JPRB, 0.15459E+01_JPRB, 0.16900E+01_JPRB ,&
&  0.18767E+01_JPRB, 0.19206E+01_JPRB, 0.17135E+01_JPRB, 0.17273E+01_JPRB, 0.16326E+01_JPRB, 0.15129E+01_JPRB, 0.13474E+01_JPRB ,&
&  0.11597E+01_JPRB, 0.98979E+00_JPRB, 0.89285E+00_JPRB, 0.83322E+00_JPRB, 0.78981E+00_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.73315E+00_JPRB, 0.70645E+00_JPRB, 0.69163E+00_JPRB, 0.67979E+00_JPRB, 0.66724E+00_JPRB, 0.63878E+00_JPRB, 0.60214E+00_JPRB ,&
&  0.57350E+00_JPRB, 0.56214E+00_JPRB, 0.56071E+00_JPRB, 0.56933E+00_JPRB, 0.56499E+00_JPRB, 0.57288E+00_JPRB, 0.58293E+00_JPRB ,&
&  0.63652E+00_JPRB, 0.68907E+00_JPRB, 0.68492E+00_JPRB, 0.71694E+00_JPRB, 0.84218E+00_JPRB, 0.10046E+01_JPRB, 0.11345E+01_JPRB ,&
&  0.14153E+01_JPRB, 0.13095E+01_JPRB, 0.13146E+01_JPRB, 0.15938E+01_JPRB, 0.21850E+01_JPRB, 0.25516E+01_JPRB, 0.24856E+01_JPRB ,&
&  0.24543E+01_JPRB, 0.21635E+01_JPRB, 0.17235E+01_JPRB, 0.13964E+01_JPRB, 0.11612E+01_JPRB, 0.99705E+00_JPRB, 0.86191E+00_JPRB ,&
&  0.80016E+00_JPRB, 0.78939E+00_JPRB, 0.79873E+00_JPRB, 0.81273E+00_JPRB, 0.87178E+00_JPRB, 0.87462E+00_JPRB, 0.10289E+01_JPRB ,&
&  0.12103E+01_JPRB, 0.14689E+01_JPRB, 0.18336E+01_JPRB, 0.21732E+01_JPRB, 0.25752E+01_JPRB, 0.38002E+01_JPRB, 0.42618E+01_JPRB ,&
&  0.33755E+01_JPRB, 0.31975E+01_JPRB, 0.34415E+01_JPRB, 0.35859E+01_JPRB, 0.34043E+01_JPRB, 0.30762E+01_JPRB, 0.30207E+01_JPRB ,&
&  0.24108E+01_JPRB, 0.21230E+01_JPRB, 0.21537E+01_JPRB, 0.26301E+01_JPRB, 0.41078E+01_JPRB, 0.53997E+01_JPRB, 0.50129E+01_JPRB ,&
&  0.41709E+01_JPRB, 0.35454E+01_JPRB, 0.30017E+01_JPRB, 0.22719E+01_JPRB, 0.19453E+01_JPRB, 0.16786E+01_JPRB, 0.19212E+01_JPRB ,&
&  0.17905E+01_JPRB, 0.15969E+01_JPRB, 0.13100E+01_JPRB, 0.10140E+01_JPRB, 0.80786E+00_JPRB, 0.91697E+00_JPRB, 0.97516E+00_JPRB ,&
&  0.10650E+01_JPRB, 0.11639E+01_JPRB, 0.17016E+01_JPRB, 0.18970E+01_JPRB, 0.19471E+01_JPRB, 0.17184E+01_JPRB, 0.22060E+01_JPRB ,&
&  0.21738E+01_JPRB, 0.22601E+01_JPRB, 0.21338E+01_JPRB, 0.17846E+01_JPRB, 0.14933E+01_JPRB, 0.12639E+01_JPRB, 0.11276E+01_JPRB ,&
&  0.10593E+01_JPRB, 0.10023E+01_JPRB, 0.92520E+00_JPRB, 0.85091E+00_JPRB, 0.79101E+00_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.65788E+00_JPRB, 0.63874E+00_JPRB, 0.63107E+00_JPRB, 0.62349E+00_JPRB, 0.61147E+00_JPRB, 0.58961E+00_JPRB, 0.56195E+00_JPRB ,&
&  0.54961E+00_JPRB, 0.54369E+00_JPRB, 0.54350E+00_JPRB, 0.56078E+00_JPRB, 0.57394E+00_JPRB, 0.60151E+00_JPRB, 0.62504E+00_JPRB ,&
&  0.68705E+00_JPRB, 0.79173E+00_JPRB, 0.77214E+00_JPRB, 0.79479E+00_JPRB, 0.88980E+00_JPRB, 0.98563E+00_JPRB, 0.10900E+01_JPRB ,&
&  0.13364E+01_JPRB, 0.12441E+01_JPRB, 0.12970E+01_JPRB, 0.15965E+01_JPRB, 0.24486E+01_JPRB, 0.26395E+01_JPRB, 0.24383E+01_JPRB ,&
&  0.24745E+01_JPRB, 0.20644E+01_JPRB, 0.14906E+01_JPRB, 0.11757E+01_JPRB, 0.98343E+00_JPRB, 0.83729E+00_JPRB, 0.76562E+00_JPRB ,&
&  0.76212E+00_JPRB, 0.79325E+00_JPRB, 0.83222E+00_JPRB, 0.86255E+00_JPRB, 0.87180E+00_JPRB, 0.95082E+00_JPRB, 0.11443E+01_JPRB ,&
&  0.13462E+01_JPRB, 0.15709E+01_JPRB, 0.17625E+01_JPRB, 0.19852E+01_JPRB, 0.24514E+01_JPRB, 0.37330E+01_JPRB, 0.51682E+01_JPRB ,&
&  0.42924E+01_JPRB, 0.41863E+01_JPRB, 0.42812E+01_JPRB, 0.45683E+01_JPRB, 0.35936E+01_JPRB, 0.32096E+01_JPRB, 0.30064E+01_JPRB ,&
&  0.23563E+01_JPRB, 0.22126E+01_JPRB, 0.27035E+01_JPRB, 0.36278E+01_JPRB, 0.43600E+01_JPRB, 0.52014E+01_JPRB, 0.60487E+01_JPRB ,&
&  0.45222E+01_JPRB, 0.37364E+01_JPRB, 0.32401E+01_JPRB, 0.27567E+01_JPRB, 0.16111E+01_JPRB, 0.11391E+01_JPRB, 0.15243E+01_JPRB ,&
&  0.13922E+01_JPRB, 0.80122E+00_JPRB, 0.62325E+00_JPRB, 0.76606E+00_JPRB, 0.56206E+00_JPRB, 0.63179E+00_JPRB, 0.92720E+00_JPRB ,&
&  0.11347E+01_JPRB, 0.13608E+01_JPRB, 0.22141E+01_JPRB, 0.20631E+01_JPRB, 0.20689E+01_JPRB, 0.24961E+01_JPRB, 0.29122E+01_JPRB ,&
&  0.28515E+01_JPRB, 0.21251E+01_JPRB, 0.17625E+01_JPRB, 0.13504E+01_JPRB, 0.10791E+01_JPRB, 0.97300E+00_JPRB, 0.92091E+00_JPRB ,&
&  0.88183E+00_JPRB, 0.83336E+00_JPRB, 0.78875E+00_JPRB, 0.72891E+00_JPRB, 0.69198E+00_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.58078E+00_JPRB, 0.56567E+00_JPRB, 0.56541E+00_JPRB, 0.56021E+00_JPRB, 0.53887E+00_JPRB, 0.52616E+00_JPRB, 0.52125E+00_JPRB ,&
&  0.52466E+00_JPRB, 0.52895E+00_JPRB, 0.54060E+00_JPRB, 0.56967E+00_JPRB, 0.60847E+00_JPRB, 0.65139E+00_JPRB, 0.67750E+00_JPRB ,&
&  0.72418E+00_JPRB, 0.82207E+00_JPRB, 0.89384E+00_JPRB, 0.95595E+00_JPRB, 0.10583E+01_JPRB, 0.10139E+01_JPRB, 0.12390E+01_JPRB ,&
&  0.13091E+01_JPRB, 0.12306E+01_JPRB, 0.13572E+01_JPRB, 0.17002E+01_JPRB, 0.20988E+01_JPRB, 0.20860E+01_JPRB, 0.19417E+01_JPRB ,&
&  0.17683E+01_JPRB, 0.14495E+01_JPRB, 0.11527E+01_JPRB, 0.96469E+00_JPRB, 0.84327E+00_JPRB, 0.78332E+00_JPRB, 0.80232E+00_JPRB ,&
&  0.83779E+00_JPRB, 0.91807E+00_JPRB, 0.97425E+00_JPRB, 0.90581E+00_JPRB, 0.96343E+00_JPRB, 0.10694E+01_JPRB, 0.12479E+01_JPRB ,&
&  0.14114E+01_JPRB, 0.15675E+01_JPRB, 0.17116E+01_JPRB, 0.20104E+01_JPRB, 0.30317E+01_JPRB, 0.43619E+01_JPRB, 0.46402E+01_JPRB ,&
&  0.42600E+01_JPRB, 0.40032E+01_JPRB, 0.46604E+01_JPRB, 0.46175E+01_JPRB, 0.38591E+01_JPRB, 0.37313E+01_JPRB, 0.33580E+01_JPRB ,&
&  0.32319E+01_JPRB, 0.33612E+01_JPRB, 0.35542E+01_JPRB, 0.42762E+01_JPRB, 0.53031E+01_JPRB, 0.54333E+01_JPRB, 0.55131E+01_JPRB ,&
&  0.48727E+01_JPRB, 0.37738E+01_JPRB, 0.26109E+01_JPRB, 0.18279E+01_JPRB, 0.16993E+01_JPRB, 0.13275E+01_JPRB, 0.87063E+00_JPRB ,&
&  0.70323E+00_JPRB, 0.58306E+00_JPRB, 0.42351E+00_JPRB, 0.46582E+00_JPRB, 0.42309E+00_JPRB, 0.49430E+00_JPRB, 0.81586E+00_JPRB ,&
&  0.13649E+01_JPRB, 0.17526E+01_JPRB, 0.21667E+01_JPRB, 0.20159E+01_JPRB, 0.20097E+01_JPRB, 0.25340E+01_JPRB, 0.39251E+01_JPRB ,&
&  0.21774E+01_JPRB, 0.16520E+01_JPRB, 0.12481E+01_JPRB, 0.97490E+00_JPRB, 0.87625E+00_JPRB, 0.82272E+00_JPRB, 0.78518E+00_JPRB ,&
&  0.74037E+00_JPRB, 0.70169E+00_JPRB, 0.68232E+00_JPRB, 0.64896E+00_JPRB, 0.60812E+00_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.54833E+00_JPRB, 0.55229E+00_JPRB, 0.55318E+00_JPRB, 0.54287E+00_JPRB, 0.51028E+00_JPRB, 0.50667E+00_JPRB, 0.51970E+00_JPRB ,&
&  0.53537E+00_JPRB, 0.55417E+00_JPRB, 0.58515E+00_JPRB, 0.63009E+00_JPRB, 0.69204E+00_JPRB, 0.73271E+00_JPRB, 0.75415E+00_JPRB ,&
&  0.78981E+00_JPRB, 0.89841E+00_JPRB, 0.10266E+01_JPRB, 0.11511E+01_JPRB, 0.12788E+01_JPRB, 0.11393E+01_JPRB, 0.13860E+01_JPRB ,&
&  0.12626E+01_JPRB, 0.12430E+01_JPRB, 0.13709E+01_JPRB, 0.15894E+01_JPRB, 0.17247E+01_JPRB, 0.16149E+01_JPRB, 0.14433E+01_JPRB ,&
&  0.12383E+01_JPRB, 0.11398E+01_JPRB, 0.95837E+00_JPRB, 0.85953E+00_JPRB, 0.80341E+00_JPRB, 0.82512E+00_JPRB, 0.87632E+00_JPRB ,&
&  0.93832E+00_JPRB, 0.10315E+01_JPRB, 0.98740E+00_JPRB, 0.10269E+01_JPRB, 0.11628E+01_JPRB, 0.12835E+01_JPRB, 0.14215E+01_JPRB ,&
&  0.15058E+01_JPRB, 0.16467E+01_JPRB, 0.19095E+01_JPRB, 0.26686E+01_JPRB, 0.33236E+01_JPRB, 0.35809E+01_JPRB, 0.42052E+01_JPRB ,&
&  0.46935E+01_JPRB, 0.48387E+01_JPRB, 0.43474E+01_JPRB, 0.49909E+01_JPRB, 0.44091E+01_JPRB, 0.38198E+01_JPRB, 0.36325E+01_JPRB ,&
&  0.37086E+01_JPRB, 0.43696E+01_JPRB, 0.44914E+01_JPRB, 0.47734E+01_JPRB, 0.65876E+01_JPRB, 0.70462E+01_JPRB, 0.56163E+01_JPRB ,&
&  0.49908E+01_JPRB, 0.36254E+01_JPRB, 0.25708E+01_JPRB, 0.18661E+01_JPRB, 0.22324E+01_JPRB, 0.22245E+01_JPRB, 0.11094E+01_JPRB ,&
&  0.72726E+00_JPRB, 0.54262E+00_JPRB, 0.38573E+00_JPRB, 0.37952E+00_JPRB, 0.38299E+00_JPRB, 0.47407E+00_JPRB, 0.98490E+00_JPRB ,&
&  0.12232E+01_JPRB, 0.19059E+01_JPRB, 0.21468E+01_JPRB, 0.20926E+01_JPRB, 0.21255E+01_JPRB, 0.20823E+01_JPRB, 0.26811E+01_JPRB ,&
&  0.18283E+01_JPRB, 0.11442E+01_JPRB, 0.84890E+00_JPRB, 0.81930E+00_JPRB, 0.81367E+00_JPRB, 0.76999E+00_JPRB, 0.71329E+00_JPRB ,&
&  0.67896E+00_JPRB, 0.67728E+00_JPRB, 0.65529E+00_JPRB, 0.62643E+00_JPRB, 0.58668E+00_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.56136E+00_JPRB, 0.57945E+00_JPRB, 0.58882E+00_JPRB, 0.55888E+00_JPRB, 0.54587E+00_JPRB, 0.55210E+00_JPRB, 0.54928E+00_JPRB ,&
&  0.57839E+00_JPRB, 0.61476E+00_JPRB, 0.65599E+00_JPRB, 0.72770E+00_JPRB, 0.79362E+00_JPRB, 0.81778E+00_JPRB, 0.84369E+00_JPRB ,&
&  0.88035E+00_JPRB, 0.99253E+00_JPRB, 0.11675E+01_JPRB, 0.13674E+01_JPRB, 0.14128E+01_JPRB, 0.12988E+01_JPRB, 0.14069E+01_JPRB ,&
&  0.12465E+01_JPRB, 0.12871E+01_JPRB, 0.13278E+01_JPRB, 0.14835E+01_JPRB, 0.14986E+01_JPRB, 0.14413E+01_JPRB, 0.12806E+01_JPRB ,&
&  0.11083E+01_JPRB, 0.11147E+01_JPRB, 0.91090E+00_JPRB, 0.93526E+00_JPRB, 0.91061E+00_JPRB, 0.96063E+00_JPRB, 0.99768E+00_JPRB ,&
&  0.10781E+01_JPRB, 0.10752E+01_JPRB, 0.10939E+01_JPRB, 0.12043E+01_JPRB, 0.13619E+01_JPRB, 0.15022E+01_JPRB, 0.16414E+01_JPRB ,&
&  0.17248E+01_JPRB, 0.19699E+01_JPRB, 0.24502E+01_JPRB, 0.31692E+01_JPRB, 0.29429E+01_JPRB, 0.38884E+01_JPRB, 0.41560E+01_JPRB ,&
&  0.45440E+01_JPRB, 0.45979E+01_JPRB, 0.41143E+01_JPRB, 0.44185E+01_JPRB, 0.44134E+01_JPRB, 0.40425E+01_JPRB, 0.39083E+01_JPRB ,&
&  0.46301E+01_JPRB, 0.49088E+01_JPRB, 0.53728E+01_JPRB, 0.57220E+01_JPRB, 0.73532E+01_JPRB, 0.98699E+01_JPRB, 0.58803E+01_JPRB ,&
&  0.40718E+01_JPRB, 0.36374E+01_JPRB, 0.25874E+01_JPRB, 0.19807E+01_JPRB, 0.23894E+01_JPRB, 0.27045E+01_JPRB, 0.22041E+01_JPRB ,&
&  0.91393E+00_JPRB, 0.51366E+00_JPRB, 0.43303E+00_JPRB, 0.43998E+00_JPRB, 0.38669E+00_JPRB, 0.49773E+00_JPRB, 0.15150E+01_JPRB ,&
&  0.12442E+01_JPRB, 0.18796E+01_JPRB, 0.20194E+01_JPRB, 0.19734E+01_JPRB, 0.21609E+01_JPRB, 0.17856E+01_JPRB, 0.15988E+01_JPRB ,&
&  0.77601E+00_JPRB, 0.70489E+00_JPRB, 0.73807E+00_JPRB, 0.72256E+00_JPRB, 0.72038E+00_JPRB, 0.68174E+00_JPRB, 0.62817E+00_JPRB ,&
&  0.62627E+00_JPRB, 0.63197E+00_JPRB, 0.59865E+00_JPRB, 0.57230E+00_JPRB, 0.55831E+00_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.59320E+00_JPRB, 0.61076E+00_JPRB, 0.61111E+00_JPRB, 0.60851E+00_JPRB, 0.61181E+00_JPRB, 0.61953E+00_JPRB, 0.61510E+00_JPRB ,&
&  0.64726E+00_JPRB, 0.69854E+00_JPRB, 0.77152E+00_JPRB, 0.86031E+00_JPRB, 0.91711E+00_JPRB, 0.93066E+00_JPRB, 0.96077E+00_JPRB ,&
&  0.10262E+01_JPRB, 0.11654E+01_JPRB, 0.13457E+01_JPRB, 0.14739E+01_JPRB, 0.14435E+01_JPRB, 0.13008E+01_JPRB, 0.13036E+01_JPRB ,&
&  0.12574E+01_JPRB, 0.13358E+01_JPRB, 0.12123E+01_JPRB, 0.13332E+01_JPRB, 0.14639E+01_JPRB, 0.15284E+01_JPRB, 0.13872E+01_JPRB ,&
&  0.12855E+01_JPRB, 0.12690E+01_JPRB, 0.11542E+01_JPRB, 0.11558E+01_JPRB, 0.11737E+01_JPRB, 0.12215E+01_JPRB, 0.12583E+01_JPRB ,&
&  0.12098E+01_JPRB, 0.12352E+01_JPRB, 0.13241E+01_JPRB, 0.14692E+01_JPRB, 0.16406E+01_JPRB, 0.17910E+01_JPRB, 0.19570E+01_JPRB ,&
&  0.21743E+01_JPRB, 0.25408E+01_JPRB, 0.29955E+01_JPRB, 0.30972E+01_JPRB, 0.32570E+01_JPRB, 0.35874E+01_JPRB, 0.39252E+01_JPRB ,&
&  0.42354E+01_JPRB, 0.44116E+01_JPRB, 0.42956E+01_JPRB, 0.41960E+01_JPRB, 0.42382E+01_JPRB, 0.42976E+01_JPRB, 0.45726E+01_JPRB ,&
&  0.53968E+01_JPRB, 0.58858E+01_JPRB, 0.58979E+01_JPRB, 0.61763E+01_JPRB, 0.68040E+01_JPRB, 0.72724E+01_JPRB, 0.60525E+01_JPRB ,&
&  0.44183E+01_JPRB, 0.30979E+01_JPRB, 0.23877E+01_JPRB, 0.23871E+01_JPRB, 0.26399E+01_JPRB, 0.26366E+01_JPRB, 0.30341E+01_JPRB ,&
&  0.25676E+01_JPRB, 0.98417E+00_JPRB, 0.67032E+00_JPRB, 0.82947E+00_JPRB, 0.48357E+00_JPRB, 0.46552E+00_JPRB, 0.82022E+00_JPRB ,&
&  0.12776E+01_JPRB, 0.17390E+01_JPRB, 0.18006E+01_JPRB, 0.17843E+01_JPRB, 0.19558E+01_JPRB, 0.13508E+01_JPRB, 0.76507E+00_JPRB ,&
&  0.62851E+00_JPRB, 0.66900E+00_JPRB, 0.66711E+00_JPRB, 0.62830E+00_JPRB, 0.61641E+00_JPRB, 0.63555E+00_JPRB, 0.58711E+00_JPRB ,&
&  0.59863E+00_JPRB, 0.60418E+00_JPRB, 0.60463E+00_JPRB, 0.59899E+00_JPRB, 0.58852E+00_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.66893E+00_JPRB, 0.68639E+00_JPRB, 0.71676E+00_JPRB, 0.73651E+00_JPRB, 0.73479E+00_JPRB, 0.74280E+00_JPRB, 0.74665E+00_JPRB ,&
&  0.78377E+00_JPRB, 0.85723E+00_JPRB, 0.93996E+00_JPRB, 0.10176E+01_JPRB, 0.10485E+01_JPRB, 0.10627E+01_JPRB, 0.11287E+01_JPRB ,&
&  0.12188E+01_JPRB, 0.13533E+01_JPRB, 0.14805E+01_JPRB, 0.15701E+01_JPRB, 0.15782E+01_JPRB, 0.15835E+01_JPRB, 0.12273E+01_JPRB ,&
&  0.11047E+01_JPRB, 0.13993E+01_JPRB, 0.12494E+01_JPRB, 0.13461E+01_JPRB, 0.14827E+01_JPRB, 0.15963E+01_JPRB, 0.15815E+01_JPRB ,&
&  0.15436E+01_JPRB, 0.15115E+01_JPRB, 0.14462E+01_JPRB, 0.14372E+01_JPRB, 0.14817E+01_JPRB, 0.15150E+01_JPRB, 0.15151E+01_JPRB ,&
&  0.15466E+01_JPRB, 0.15844E+01_JPRB, 0.16821E+01_JPRB, 0.18204E+01_JPRB, 0.19726E+01_JPRB, 0.21447E+01_JPRB, 0.23908E+01_JPRB ,&
&  0.26570E+01_JPRB, 0.28906E+01_JPRB, 0.31095E+01_JPRB, 0.30288E+01_JPRB, 0.31869E+01_JPRB, 0.34489E+01_JPRB, 0.37114E+01_JPRB ,&
&  0.37098E+01_JPRB, 0.36535E+01_JPRB, 0.39992E+01_JPRB, 0.44283E+01_JPRB, 0.46321E+01_JPRB, 0.46363E+01_JPRB, 0.49093E+01_JPRB ,&
&  0.57504E+01_JPRB, 0.62574E+01_JPRB, 0.63396E+01_JPRB, 0.55228E+01_JPRB, 0.50679E+01_JPRB, 0.47705E+01_JPRB, 0.47105E+01_JPRB ,&
&  0.40118E+01_JPRB, 0.31938E+01_JPRB, 0.27075E+01_JPRB, 0.25454E+01_JPRB, 0.24848E+01_JPRB, 0.24773E+01_JPRB, 0.26421E+01_JPRB ,&
&  0.25803E+01_JPRB, 0.21229E+01_JPRB, 0.16236E+01_JPRB, 0.10322E+01_JPRB, 0.69365E+00_JPRB, 0.63259E+00_JPRB, 0.80575E+00_JPRB ,&
&  0.12358E+01_JPRB, 0.14165E+01_JPRB, 0.14619E+01_JPRB, 0.18348E+01_JPRB, 0.15080E+01_JPRB, 0.10002E+01_JPRB, 0.62917E+00_JPRB ,&
&  0.56541E+00_JPRB, 0.60120E+00_JPRB, 0.61488E+00_JPRB, 0.61036E+00_JPRB, 0.63588E+00_JPRB, 0.66172E+00_JPRB, 0.65261E+00_JPRB ,&
&  0.66456E+00_JPRB, 0.67460E+00_JPRB, 0.66938E+00_JPRB, 0.65639E+00_JPRB, 0.66116E+00_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.83152E+00_JPRB, 0.86907E+00_JPRB, 0.89755E+00_JPRB, 0.91541E+00_JPRB, 0.92027E+00_JPRB, 0.88910E+00_JPRB, 0.88793E+00_JPRB ,&
&  0.94369E+00_JPRB, 0.10172E+01_JPRB, 0.10817E+01_JPRB, 0.11239E+01_JPRB, 0.11451E+01_JPRB, 0.11902E+01_JPRB, 0.12749E+01_JPRB ,&
&  0.13449E+01_JPRB, 0.14294E+01_JPRB, 0.14938E+01_JPRB, 0.15639E+01_JPRB, 0.16036E+01_JPRB, 0.17120E+01_JPRB, 0.14316E+01_JPRB ,&
&  0.10384E+01_JPRB, 0.16074E+01_JPRB, 0.13070E+01_JPRB, 0.14204E+01_JPRB, 0.15151E+01_JPRB, 0.16366E+01_JPRB, 0.16570E+01_JPRB ,&
&  0.16756E+01_JPRB, 0.17002E+01_JPRB, 0.16967E+01_JPRB, 0.16640E+01_JPRB, 0.16516E+01_JPRB, 0.16987E+01_JPRB, 0.17703E+01_JPRB ,&
&  0.18413E+01_JPRB, 0.18953E+01_JPRB, 0.19818E+01_JPRB, 0.20994E+01_JPRB, 0.22398E+01_JPRB, 0.24183E+01_JPRB, 0.26358E+01_JPRB ,&
&  0.28065E+01_JPRB, 0.29686E+01_JPRB, 0.29793E+01_JPRB, 0.29724E+01_JPRB, 0.30282E+01_JPRB, 0.31917E+01_JPRB, 0.33918E+01_JPRB ,&
&  0.34830E+01_JPRB, 0.35381E+01_JPRB, 0.39080E+01_JPRB, 0.41391E+01_JPRB, 0.41976E+01_JPRB, 0.44529E+01_JPRB, 0.45336E+01_JPRB ,&
&  0.52536E+01_JPRB, 0.51859E+01_JPRB, 0.52104E+01_JPRB, 0.43237E+01_JPRB, 0.34146E+01_JPRB, 0.30019E+01_JPRB, 0.28801E+01_JPRB ,&
&  0.25080E+01_JPRB, 0.23730E+01_JPRB, 0.21343E+01_JPRB, 0.20372E+01_JPRB, 0.21534E+01_JPRB, 0.22951E+01_JPRB, 0.21379E+01_JPRB ,&
&  0.18976E+01_JPRB, 0.13573E+01_JPRB, 0.10680E+01_JPRB, 0.89397E+00_JPRB, 0.68431E+00_JPRB, 0.63370E+00_JPRB, 0.86764E+00_JPRB ,&
&  0.12310E+01_JPRB, 0.13966E+01_JPRB, 0.15281E+01_JPRB, 0.14433E+01_JPRB, 0.10062E+01_JPRB, 0.68360E+00_JPRB, 0.54860E+00_JPRB ,&
&  0.53385E+00_JPRB, 0.57797E+00_JPRB, 0.60540E+00_JPRB, 0.62984E+00_JPRB, 0.66976E+00_JPRB, 0.67890E+00_JPRB, 0.72411E+00_JPRB ,&
&  0.73287E+00_JPRB, 0.75127E+00_JPRB, 0.77229E+00_JPRB, 0.77662E+00_JPRB, 0.79709E+00_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.10675E+01_JPRB, 0.11677E+01_JPRB, 0.12614E+01_JPRB, 0.13190E+01_JPRB, 0.12668E+01_JPRB, 0.12024E+01_JPRB, 0.11454E+01_JPRB ,&
&  0.10943E+01_JPRB, 0.10808E+01_JPRB, 0.11310E+01_JPRB, 0.11784E+01_JPRB, 0.12206E+01_JPRB, 0.12886E+01_JPRB, 0.13842E+01_JPRB ,&
&  0.14723E+01_JPRB, 0.15543E+01_JPRB, 0.16330E+01_JPRB, 0.17276E+01_JPRB, 0.18407E+01_JPRB, 0.20253E+01_JPRB, 0.19051E+01_JPRB ,&
&  0.13426E+01_JPRB, 0.12381E+01_JPRB, 0.15033E+01_JPRB, 0.14674E+01_JPRB, 0.14761E+01_JPRB, 0.15804E+01_JPRB, 0.16558E+01_JPRB ,&
&  0.17090E+01_JPRB, 0.17414E+01_JPRB, 0.17594E+01_JPRB, 0.17391E+01_JPRB, 0.17336E+01_JPRB, 0.18179E+01_JPRB, 0.19146E+01_JPRB ,&
&  0.20026E+01_JPRB, 0.20582E+01_JPRB, 0.21250E+01_JPRB, 0.22201E+01_JPRB, 0.23324E+01_JPRB, 0.24630E+01_JPRB, 0.26184E+01_JPRB ,&
&  0.27087E+01_JPRB, 0.28182E+01_JPRB, 0.28432E+01_JPRB, 0.28237E+01_JPRB, 0.28621E+01_JPRB, 0.29017E+01_JPRB, 0.29605E+01_JPRB ,&
&  0.31040E+01_JPRB, 0.33394E+01_JPRB, 0.35759E+01_JPRB, 0.36760E+01_JPRB, 0.36138E+01_JPRB, 0.38938E+01_JPRB, 0.39955E+01_JPRB ,&
&  0.41302E+01_JPRB, 0.38075E+01_JPRB, 0.37859E+01_JPRB, 0.31927E+01_JPRB, 0.26516E+01_JPRB, 0.21608E+01_JPRB, 0.20070E+01_JPRB ,&
&  0.18805E+01_JPRB, 0.17330E+01_JPRB, 0.15587E+01_JPRB, 0.14837E+01_JPRB, 0.16725E+01_JPRB, 0.17137E+01_JPRB, 0.16882E+01_JPRB ,&
&  0.13700E+01_JPRB, 0.11181E+01_JPRB, 0.84515E+00_JPRB, 0.82091E+00_JPRB, 0.66407E+00_JPRB, 0.72834E+00_JPRB, 0.90798E+00_JPRB ,&
&  0.11430E+01_JPRB, 0.11007E+01_JPRB, 0.10768E+01_JPRB, 0.10340E+01_JPRB, 0.73180E+00_JPRB, 0.55869E+00_JPRB, 0.49268E+00_JPRB ,&
&  0.48135E+00_JPRB, 0.52266E+00_JPRB, 0.55648E+00_JPRB, 0.58097E+00_JPRB, 0.63414E+00_JPRB, 0.68680E+00_JPRB, 0.74765E+00_JPRB ,&
&  0.77680E+00_JPRB, 0.81633E+00_JPRB, 0.86143E+00_JPRB, 0.90864E+00_JPRB, 0.97880E+00_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.10800E+01_JPRB, 0.11333E+01_JPRB, 0.11805E+01_JPRB, 0.11519E+01_JPRB, 0.11361E+01_JPRB, 0.10874E+01_JPRB, 0.10372E+01_JPRB ,&
&  0.10528E+01_JPRB, 0.10798E+01_JPRB, 0.11354E+01_JPRB, 0.11846E+01_JPRB, 0.12285E+01_JPRB, 0.13042E+01_JPRB, 0.13924E+01_JPRB ,&
&  0.14401E+01_JPRB, 0.14841E+01_JPRB, 0.15365E+01_JPRB, 0.16134E+01_JPRB, 0.16944E+01_JPRB, 0.18100E+01_JPRB, 0.18849E+01_JPRB ,&
&  0.14719E+01_JPRB, 0.10316E+01_JPRB, 0.12382E+01_JPRB, 0.14081E+01_JPRB, 0.13936E+01_JPRB, 0.15003E+01_JPRB, 0.15851E+01_JPRB ,&
&  0.15780E+01_JPRB, 0.16180E+01_JPRB, 0.16683E+01_JPRB, 0.16685E+01_JPRB, 0.16722E+01_JPRB, 0.17759E+01_JPRB, 0.18820E+01_JPRB ,&
&  0.19805E+01_JPRB, 0.20659E+01_JPRB, 0.21394E+01_JPRB, 0.22225E+01_JPRB, 0.22966E+01_JPRB, 0.23720E+01_JPRB, 0.24405E+01_JPRB ,&
&  0.25132E+01_JPRB, 0.25387E+01_JPRB, 0.25820E+01_JPRB, 0.26613E+01_JPRB, 0.26600E+01_JPRB, 0.26287E+01_JPRB, 0.25849E+01_JPRB ,&
&  0.26503E+01_JPRB, 0.27784E+01_JPRB, 0.30039E+01_JPRB, 0.31934E+01_JPRB, 0.33141E+01_JPRB, 0.32338E+01_JPRB, 0.32334E+01_JPRB ,&
&  0.30093E+01_JPRB, 0.26433E+01_JPRB, 0.26639E+01_JPRB, 0.27897E+01_JPRB, 0.19535E+01_JPRB, 0.16464E+01_JPRB, 0.14408E+01_JPRB ,&
&  0.15407E+01_JPRB, 0.13680E+01_JPRB, 0.11961E+01_JPRB, 0.11619E+01_JPRB, 0.12942E+01_JPRB, 0.11953E+01_JPRB, 0.12058E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.82794E+00_JPRB, 0.74958E+00_JPRB, 0.82902E+00_JPRB, 0.79066E+00_JPRB, 0.10196E+01_JPRB, 0.95500E+00_JPRB ,&
&  0.90677E+00_JPRB, 0.90530E+00_JPRB, 0.93796E+00_JPRB, 0.90239E+00_JPRB, 0.72125E+00_JPRB, 0.55171E+00_JPRB, 0.49663E+00_JPRB ,&
&  0.47429E+00_JPRB, 0.48683E+00_JPRB, 0.51726E+00_JPRB, 0.53800E+00_JPRB, 0.58118E+00_JPRB, 0.62857E+00_JPRB, 0.71589E+00_JPRB ,&
&  0.78448E+00_JPRB, 0.84297E+00_JPRB, 0.89014E+00_JPRB, 0.94564E+00_JPRB, 0.10094E+01_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.81801E+00_JPRB, 0.84438E+00_JPRB, 0.84237E+00_JPRB, 0.89555E+00_JPRB, 0.94448E+00_JPRB, 0.95558E+00_JPRB, 0.96183E+00_JPRB ,&
&  0.99682E+00_JPRB, 0.10613E+01_JPRB, 0.11159E+01_JPRB, 0.11587E+01_JPRB, 0.12057E+01_JPRB, 0.12736E+01_JPRB, 0.12961E+01_JPRB ,&
&  0.12834E+01_JPRB, 0.12781E+01_JPRB, 0.12944E+01_JPRB, 0.13242E+01_JPRB, 0.13774E+01_JPRB, 0.14414E+01_JPRB, 0.16104E+01_JPRB ,&
&  0.17445E+01_JPRB, 0.17354E+01_JPRB, 0.17150E+01_JPRB, 0.14214E+01_JPRB, 0.10020E+01_JPRB, 0.12722E+01_JPRB, 0.12572E+01_JPRB ,&
&  0.13587E+01_JPRB, 0.14346E+01_JPRB, 0.14589E+01_JPRB, 0.15250E+01_JPRB, 0.16001E+01_JPRB, 0.16820E+01_JPRB, 0.17761E+01_JPRB ,&
&  0.18453E+01_JPRB, 0.19246E+01_JPRB, 0.20018E+01_JPRB, 0.20734E+01_JPRB, 0.21316E+01_JPRB, 0.21745E+01_JPRB, 0.21882E+01_JPRB ,&
&  0.22099E+01_JPRB, 0.22471E+01_JPRB, 0.22824E+01_JPRB, 0.24098E+01_JPRB, 0.24200E+01_JPRB, 0.24213E+01_JPRB, 0.23257E+01_JPRB ,&
&  0.23129E+01_JPRB, 0.23483E+01_JPRB, 0.24946E+01_JPRB, 0.26072E+01_JPRB, 0.25981E+01_JPRB, 0.24136E+01_JPRB, 0.22881E+01_JPRB ,&
&  0.21239E+01_JPRB, 0.19330E+01_JPRB, 0.19379E+01_JPRB, 0.21223E+01_JPRB, 0.14097E+01_JPRB, 0.14253E+01_JPRB, 0.15368E+01_JPRB ,&
&  0.13720E+01_JPRB, 0.12459E+01_JPRB, 0.11481E+01_JPRB, 0.10256E+01_JPRB, 0.10317E+01_JPRB, 0.92991E+00_JPRB, 0.10213E+01_JPRB ,&
&  0.94749E+00_JPRB, 0.73594E+00_JPRB, 0.69885E+00_JPRB, 0.79419E+00_JPRB, 0.89283E+00_JPRB, 0.10602E+01_JPRB, 0.86311E+00_JPRB ,&
&  0.75945E+00_JPRB, 0.95343E+00_JPRB, 0.98407E+00_JPRB, 0.11344E+01_JPRB, 0.10529E+01_JPRB, 0.57470E+00_JPRB, 0.48633E+00_JPRB ,&
&  0.46979E+00_JPRB, 0.48205E+00_JPRB, 0.50206E+00_JPRB, 0.51721E+00_JPRB, 0.53924E+00_JPRB, 0.57146E+00_JPRB, 0.60180E+00_JPRB ,&
&  0.62584E+00_JPRB, 0.64153E+00_JPRB, 0.67409E+00_JPRB, 0.71894E+00_JPRB, 0.77732E+00_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.59007E+00_JPRB, 0.58948E+00_JPRB, 0.57620E+00_JPRB, 0.58778E+00_JPRB, 0.60874E+00_JPRB, 0.65813E+00_JPRB, 0.70424E+00_JPRB ,&
&  0.75908E+00_JPRB, 0.83124E+00_JPRB, 0.93486E+00_JPRB, 0.10303E+01_JPRB, 0.10568E+01_JPRB, 0.10476E+01_JPRB, 0.96565E+00_JPRB ,&
&  0.86020E+00_JPRB, 0.75148E+00_JPRB, 0.69758E+00_JPRB, 0.69123E+00_JPRB, 0.74493E+00_JPRB, 0.77720E+00_JPRB, 0.84667E+00_JPRB ,&
&  0.95562E+00_JPRB, 0.11402E+01_JPRB, 0.13495E+01_JPRB, 0.18109E+01_JPRB, 0.21210E+01_JPRB, 0.14190E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.13557E+01_JPRB, 0.13959E+01_JPRB, 0.12215E+01_JPRB, 0.12158E+01_JPRB, 0.13175E+01_JPRB, 0.14010E+01_JPRB, 0.15066E+01_JPRB ,&
&  0.15586E+01_JPRB, 0.16163E+01_JPRB, 0.16687E+01_JPRB, 0.16995E+01_JPRB, 0.17143E+01_JPRB, 0.16897E+01_JPRB, 0.16664E+01_JPRB ,&
&  0.16508E+01_JPRB, 0.16313E+01_JPRB, 0.17005E+01_JPRB, 0.17958E+01_JPRB, 0.19037E+01_JPRB, 0.19690E+01_JPRB, 0.19523E+01_JPRB ,&
&  0.19362E+01_JPRB, 0.19433E+01_JPRB, 0.21104E+01_JPRB, 0.21366E+01_JPRB, 0.19909E+01_JPRB, 0.18499E+01_JPRB, 0.16571E+01_JPRB ,&
&  0.14766E+01_JPRB, 0.12533E+01_JPRB, 0.97677E+00_JPRB, 0.14375E+01_JPRB, 0.16624E+01_JPRB, 0.14828E+01_JPRB, 0.12776E+01_JPRB ,&
&  0.12414E+01_JPRB, 0.11750E+01_JPRB, 0.10948E+01_JPRB, 0.10302E+01_JPRB, 0.10002E+01_JPRB, 0.11405E+01_JPRB, 0.97307E+00_JPRB ,&
&  0.88694E+00_JPRB, 0.67864E+00_JPRB, 0.60439E+00_JPRB, 0.63687E+00_JPRB, 0.70034E+00_JPRB, 0.78966E+00_JPRB, 0.67547E+00_JPRB ,&
&  0.67532E+00_JPRB, 0.96119E+00_JPRB, 0.10713E+01_JPRB, 0.14874E+01_JPRB, 0.18387E+01_JPRB, 0.50982E+00_JPRB, 0.48471E+00_JPRB ,&
&  0.48039E+00_JPRB, 0.49018E+00_JPRB, 0.51287E+00_JPRB, 0.51404E+00_JPRB, 0.51550E+00_JPRB, 0.52061E+00_JPRB, 0.51876E+00_JPRB ,&
&  0.52321E+00_JPRB, 0.54027E+00_JPRB, 0.55803E+00_JPRB, 0.56525E+00_JPRB, 0.58105E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.57620E+00_JPRB, 0.57557E+00_JPRB, 0.57926E+00_JPRB, 0.58573E+00_JPRB, 0.58517E+00_JPRB, 0.58177E+00_JPRB, 0.59079E+00_JPRB ,&
&  0.60798E+00_JPRB, 0.64446E+00_JPRB, 0.70214E+00_JPRB, 0.73474E+00_JPRB, 0.71949E+00_JPRB, 0.67903E+00_JPRB, 0.66802E+00_JPRB ,&
&  0.67233E+00_JPRB, 0.67416E+00_JPRB, 0.70440E+00_JPRB, 0.75376E+00_JPRB, 0.81838E+00_JPRB, 0.84078E+00_JPRB, 0.90128E+00_JPRB ,&
&  0.90168E+00_JPRB, 0.92219E+00_JPRB, 0.10066E+01_JPRB, 0.10885E+01_JPRB, 0.16687E+01_JPRB, 0.15668E+01_JPRB, 0.12233E+01_JPRB ,&
&  0.10637E+01_JPRB, 0.10321E+01_JPRB, 0.10650E+01_JPRB, 0.99989E+00_JPRB, 0.10046E+01_JPRB, 0.97432E+00_JPRB, 0.99014E+00_JPRB ,&
&  0.98477E+00_JPRB, 0.10480E+01_JPRB, 0.11035E+01_JPRB, 0.11690E+01_JPRB, 0.12145E+01_JPRB, 0.12293E+01_JPRB, 0.12757E+01_JPRB ,&
&  0.13055E+01_JPRB, 0.13361E+01_JPRB, 0.13735E+01_JPRB, 0.12126E+01_JPRB, 0.12969E+01_JPRB, 0.14001E+01_JPRB, 0.14641E+01_JPRB ,&
&  0.14782E+01_JPRB, 0.14785E+01_JPRB, 0.15891E+01_JPRB, 0.15366E+01_JPRB, 0.14584E+01_JPRB, 0.13240E+01_JPRB, 0.12003E+01_JPRB ,&
&  0.11139E+01_JPRB, 0.84079E+00_JPRB, 0.56441E+00_JPRB, 0.81746E+00_JPRB, 0.10228E+01_JPRB, 0.95137E+00_JPRB, 0.10640E+01_JPRB ,&
&  0.10603E+01_JPRB, 0.10314E+01_JPRB, 0.99440E+00_JPRB, 0.97262E+00_JPRB, 0.91913E+00_JPRB, 0.98590E+00_JPRB, 0.92211E+00_JPRB ,&
&  0.81677E+00_JPRB, 0.64269E+00_JPRB, 0.54877E+00_JPRB, 0.53578E+00_JPRB, 0.55304E+00_JPRB, 0.61097E+00_JPRB, 0.68557E+00_JPRB ,&
&  0.78921E+00_JPRB, 0.66613E+00_JPRB, 0.69410E+00_JPRB, 0.80490E+00_JPRB, 0.57327E+00_JPRB, 0.44875E+00_JPRB, 0.44913E+00_JPRB ,&
&  0.45514E+00_JPRB, 0.47060E+00_JPRB, 0.49457E+00_JPRB, 0.50892E+00_JPRB, 0.51994E+00_JPRB, 0.53036E+00_JPRB, 0.54381E+00_JPRB ,&
&  0.55170E+00_JPRB, 0.56200E+00_JPRB, 0.56606E+00_JPRB, 0.57047E+00_JPRB, 0.57473E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.74581E+00_JPRB, 0.75927E+00_JPRB, 0.77757E+00_JPRB, 0.79649E+00_JPRB, 0.80634E+00_JPRB, 0.82230E+00_JPRB, 0.83621E+00_JPRB ,&
&  0.85617E+00_JPRB, 0.88556E+00_JPRB, 0.90345E+00_JPRB, 0.90978E+00_JPRB, 0.92557E+00_JPRB, 0.94714E+00_JPRB, 0.95977E+00_JPRB ,&
&  0.98477E+00_JPRB, 0.99623E+00_JPRB, 0.10505E+01_JPRB, 0.10777E+01_JPRB, 0.11167E+01_JPRB, 0.11680E+01_JPRB, 0.11771E+01_JPRB ,&
&  0.11716E+01_JPRB, 0.11612E+01_JPRB, 0.11764E+01_JPRB, 0.11788E+01_JPRB, 0.12664E+01_JPRB, 0.12803E+01_JPRB, 0.12138E+01_JPRB ,&
&  0.97293E+00_JPRB, 0.86713E+00_JPRB, 0.10439E+01_JPRB, 0.91394E+00_JPRB, 0.91468E+00_JPRB, 0.91712E+00_JPRB, 0.85141E+00_JPRB ,&
&  0.85720E+00_JPRB, 0.90159E+00_JPRB, 0.95653E+00_JPRB, 0.10071E+01_JPRB, 0.10757E+01_JPRB, 0.11087E+01_JPRB, 0.11268E+01_JPRB ,&
&  0.11507E+01_JPRB, 0.11846E+01_JPRB, 0.12024E+01_JPRB, 0.11568E+01_JPRB, 0.11196E+01_JPRB, 0.12783E+01_JPRB, 0.13398E+01_JPRB ,&
&  0.13581E+01_JPRB, 0.13754E+01_JPRB, 0.11478E+01_JPRB, 0.10625E+01_JPRB, 0.11113E+01_JPRB, 0.98993E+00_JPRB, 0.94565E+00_JPRB ,&
&  0.10302E+01_JPRB, 0.90403E+00_JPRB, 0.52093E+00_JPRB, 0.70538E+00_JPRB, 0.85585E+00_JPRB, 0.93155E+00_JPRB, 0.96524E+00_JPRB ,&
&  0.92552E+00_JPRB, 0.82584E+00_JPRB, 0.80672E+00_JPRB, 0.86265E+00_JPRB, 0.90594E+00_JPRB, 0.85516E+00_JPRB, 0.82602E+00_JPRB ,&
&  0.72366E+00_JPRB, 0.62666E+00_JPRB, 0.56700E+00_JPRB, 0.62370E+00_JPRB, 0.70606E+00_JPRB, 0.84998E+00_JPRB, 0.87237E+00_JPRB ,&
&  0.76627E+00_JPRB, 0.60633E+00_JPRB, 0.78239E+00_JPRB, 0.10101E+01_JPRB, 0.52881E+00_JPRB, 0.48341E+00_JPRB, 0.48349E+00_JPRB ,&
&  0.51297E+00_JPRB, 0.52956E+00_JPRB, 0.54172E+00_JPRB, 0.56436E+00_JPRB, 0.59757E+00_JPRB, 0.61012E+00_JPRB, 0.61395E+00_JPRB ,&
&  0.63069E+00_JPRB, 0.65835E+00_JPRB, 0.67884E+00_JPRB, 0.70392E+00_JPRB, 0.72602E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.87049E+00_JPRB, 0.88899E+00_JPRB, 0.90070E+00_JPRB, 0.91218E+00_JPRB, 0.92807E+00_JPRB, 0.94393E+00_JPRB, 0.95096E+00_JPRB ,&
&  0.96764E+00_JPRB, 0.98315E+00_JPRB, 0.99046E+00_JPRB, 0.10097E+01_JPRB, 0.10177E+01_JPRB, 0.10692E+01_JPRB, 0.10894E+01_JPRB ,&
&  0.11299E+01_JPRB, 0.11522E+01_JPRB, 0.11593E+01_JPRB, 0.11628E+01_JPRB, 0.11769E+01_JPRB, 0.11763E+01_JPRB, 0.11726E+01_JPRB ,&
&  0.11940E+01_JPRB, 0.12356E+01_JPRB, 0.12886E+01_JPRB, 0.13424E+01_JPRB, 0.13766E+01_JPRB, 0.14762E+01_JPRB, 0.16060E+01_JPRB ,&
&  0.11292E+01_JPRB, 0.10147E+01_JPRB, 0.10848E+01_JPRB, 0.96385E+00_JPRB, 0.97621E+00_JPRB, 0.99600E+00_JPRB, 0.96321E+00_JPRB ,&
&  0.95067E+00_JPRB, 0.98865E+00_JPRB, 0.10229E+01_JPRB, 0.10479E+01_JPRB, 0.10754E+01_JPRB, 0.11117E+01_JPRB, 0.11303E+01_JPRB ,&
&  0.11698E+01_JPRB, 0.12027E+01_JPRB, 0.12383E+01_JPRB, 0.13007E+01_JPRB, 0.12495E+01_JPRB, 0.14321E+01_JPRB, 0.14836E+01_JPRB ,&
&  0.14912E+01_JPRB, 0.14215E+01_JPRB, 0.10885E+01_JPRB, 0.10307E+01_JPRB, 0.10666E+01_JPRB, 0.10189E+01_JPRB, 0.96345E+00_JPRB ,&
&  0.10452E+01_JPRB, 0.10199E+01_JPRB, 0.77240E+00_JPRB, 0.87261E+00_JPRB, 0.91688E+00_JPRB, 0.93054E+00_JPRB, 0.87984E+00_JPRB ,&
&  0.81055E+00_JPRB, 0.70506E+00_JPRB, 0.67741E+00_JPRB, 0.70221E+00_JPRB, 0.71110E+00_JPRB, 0.69435E+00_JPRB, 0.70818E+00_JPRB ,&
&  0.68933E+00_JPRB, 0.65194E+00_JPRB, 0.65987E+00_JPRB, 0.61446E+00_JPRB, 0.77997E+00_JPRB, 0.74378E+00_JPRB, 0.11082E+01_JPRB ,&
&  0.63495E+00_JPRB, 0.56790E+00_JPRB, 0.52868E+00_JPRB, 0.74132E+00_JPRB, 0.59744E+00_JPRB, 0.49164E+00_JPRB, 0.53793E+00_JPRB ,&
&  0.59891E+00_JPRB, 0.62642E+00_JPRB, 0.63577E+00_JPRB, 0.65032E+00_JPRB, 0.69214E+00_JPRB, 0.71370E+00_JPRB, 0.73218E+00_JPRB ,&
&  0.73264E+00_JPRB, 0.76210E+00_JPRB, 0.80309E+00_JPRB, 0.83340E+00_JPRB, 0.84472E+00_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.92107E+00_JPRB, 0.94344E+00_JPRB, 0.96120E+00_JPRB, 0.97492E+00_JPRB, 0.98751E+00_JPRB, 0.10032E+01_JPRB, 0.10182E+01_JPRB ,&
&  0.10333E+01_JPRB, 0.10455E+01_JPRB, 0.10538E+01_JPRB, 0.10694E+01_JPRB, 0.10858E+01_JPRB, 0.11211E+01_JPRB, 0.11363E+01_JPRB ,&
&  0.11689E+01_JPRB, 0.11815E+01_JPRB, 0.11810E+01_JPRB, 0.11978E+01_JPRB, 0.11901E+01_JPRB, 0.11700E+01_JPRB, 0.11648E+01_JPRB ,&
&  0.11677E+01_JPRB, 0.11636E+01_JPRB, 0.11627E+01_JPRB, 0.11669E+01_JPRB, 0.11730E+01_JPRB, 0.11479E+01_JPRB, 0.12116E+01_JPRB ,&
&  0.87456E+00_JPRB, 0.11721E+01_JPRB, 0.12565E+01_JPRB, 0.11414E+01_JPRB, 0.10889E+01_JPRB, 0.10053E+01_JPRB, 0.99928E+00_JPRB ,&
&  0.10145E+01_JPRB, 0.10176E+01_JPRB, 0.10327E+01_JPRB, 0.10632E+01_JPRB, 0.10982E+01_JPRB, 0.11409E+01_JPRB, 0.11724E+01_JPRB ,&
&  0.12253E+01_JPRB, 0.12690E+01_JPRB, 0.13101E+01_JPRB, 0.13517E+01_JPRB, 0.14307E+01_JPRB, 0.14693E+01_JPRB, 0.14802E+01_JPRB ,&
&  0.14820E+01_JPRB, 0.14990E+01_JPRB, 0.11850E+01_JPRB, 0.11528E+01_JPRB, 0.12354E+01_JPRB, 0.11854E+01_JPRB, 0.10610E+01_JPRB ,&
&  0.10683E+01_JPRB, 0.10317E+01_JPRB, 0.10088E+01_JPRB, 0.95639E+00_JPRB, 0.92419E+00_JPRB, 0.87587E+00_JPRB, 0.81532E+00_JPRB ,&
&  0.69185E+00_JPRB, 0.58720E+00_JPRB, 0.56030E+00_JPRB, 0.59755E+00_JPRB, 0.61062E+00_JPRB, 0.60343E+00_JPRB, 0.56587E+00_JPRB ,&
&  0.54565E+00_JPRB, 0.54067E+00_JPRB, 0.55523E+00_JPRB, 0.58443E+00_JPRB, 0.57244E+00_JPRB, 0.91585E+00_JPRB, 0.11340E+01_JPRB ,&
&  0.64966E+00_JPRB, 0.55977E+00_JPRB, 0.49399E+00_JPRB, 0.66023E+00_JPRB, 0.67124E+00_JPRB, 0.62315E+00_JPRB, 0.66127E+00_JPRB ,&
&  0.76403E+00_JPRB, 0.81442E+00_JPRB, 0.83392E+00_JPRB, 0.76548E+00_JPRB, 0.77616E+00_JPRB, 0.75618E+00_JPRB, 0.75712E+00_JPRB ,&
&  0.77301E+00_JPRB, 0.80030E+00_JPRB, 0.83453E+00_JPRB, 0.86720E+00_JPRB, 0.89509E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.93913E+00_JPRB, 0.95990E+00_JPRB, 0.98005E+00_JPRB, 0.10018E+01_JPRB, 0.10233E+01_JPRB, 0.10385E+01_JPRB, 0.10566E+01_JPRB ,&
&  0.10765E+01_JPRB, 0.10923E+01_JPRB, 0.10997E+01_JPRB, 0.11094E+01_JPRB, 0.11250E+01_JPRB, 0.11581E+01_JPRB, 0.11862E+01_JPRB ,&
&  0.11704E+01_JPRB, 0.11844E+01_JPRB, 0.11929E+01_JPRB, 0.11974E+01_JPRB, 0.11858E+01_JPRB, 0.11564E+01_JPRB, 0.11642E+01_JPRB ,&
&  0.11568E+01_JPRB, 0.11546E+01_JPRB, 0.11552E+01_JPRB, 0.11783E+01_JPRB, 0.12160E+01_JPRB, 0.12924E+01_JPRB, 0.10673E+01_JPRB ,&
&  0.11644E+01_JPRB, 0.12706E+01_JPRB, 0.12975E+01_JPRB, 0.12100E+01_JPRB, 0.10997E+01_JPRB, 0.10694E+01_JPRB, 0.10662E+01_JPRB ,&
&  0.10781E+01_JPRB, 0.10649E+01_JPRB, 0.10742E+01_JPRB, 0.10853E+01_JPRB, 0.11054E+01_JPRB, 0.11465E+01_JPRB, 0.11745E+01_JPRB ,&
&  0.12238E+01_JPRB, 0.13029E+01_JPRB, 0.13829E+01_JPRB, 0.14621E+01_JPRB, 0.15623E+01_JPRB, 0.16546E+01_JPRB, 0.17071E+01_JPRB ,&
&  0.17383E+01_JPRB, 0.17719E+01_JPRB, 0.16218E+01_JPRB, 0.16294E+01_JPRB, 0.17023E+01_JPRB, 0.15469E+01_JPRB, 0.13047E+01_JPRB ,&
&  0.11994E+01_JPRB, 0.10131E+01_JPRB, 0.90805E+00_JPRB, 0.10260E+01_JPRB, 0.94750E+00_JPRB, 0.85082E+00_JPRB, 0.74215E+00_JPRB ,&
&  0.61485E+00_JPRB, 0.54351E+00_JPRB, 0.54130E+00_JPRB, 0.58091E+00_JPRB, 0.57327E+00_JPRB, 0.57053E+00_JPRB, 0.57538E+00_JPRB ,&
&  0.58885E+00_JPRB, 0.60053E+00_JPRB, 0.62458E+00_JPRB, 0.67904E+00_JPRB, 0.74015E+00_JPRB, 0.70934E+00_JPRB, 0.85604E+00_JPRB ,&
&  0.71624E+00_JPRB, 0.58771E+00_JPRB, 0.66332E+00_JPRB, 0.77991E+00_JPRB, 0.81432E+00_JPRB, 0.82079E+00_JPRB, 0.92387E+00_JPRB ,&
&  0.10688E+01_JPRB, 0.12028E+01_JPRB, 0.12107E+01_JPRB, 0.98997E+00_JPRB, 0.87113E+00_JPRB, 0.81890E+00_JPRB, 0.80929E+00_JPRB ,&
&  0.81550E+00_JPRB, 0.83102E+00_JPRB, 0.85806E+00_JPRB, 0.88940E+00_JPRB, 0.91671E+00_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.94190E+00_JPRB, 0.96379E+00_JPRB, 0.98527E+00_JPRB, 0.10049E+01_JPRB, 0.10186E+01_JPRB, 0.10284E+01_JPRB, 0.10348E+01_JPRB ,&
&  0.10498E+01_JPRB, 0.10651E+01_JPRB, 0.10764E+01_JPRB, 0.10822E+01_JPRB, 0.10872E+01_JPRB, 0.11032E+01_JPRB, 0.11263E+01_JPRB ,&
&  0.11552E+01_JPRB, 0.11682E+01_JPRB, 0.12011E+01_JPRB, 0.11816E+01_JPRB, 0.11614E+01_JPRB, 0.11640E+01_JPRB, 0.11866E+01_JPRB ,&
&  0.11904E+01_JPRB, 0.11807E+01_JPRB, 0.11744E+01_JPRB, 0.12023E+01_JPRB, 0.12335E+01_JPRB, 0.12746E+01_JPRB, 0.10898E+01_JPRB ,&
&  0.12459E+01_JPRB, 0.13479E+01_JPRB, 0.13280E+01_JPRB, 0.12742E+01_JPRB, 0.11718E+01_JPRB, 0.11430E+01_JPRB, 0.11376E+01_JPRB ,&
&  0.11333E+01_JPRB, 0.11334E+01_JPRB, 0.11285E+01_JPRB, 0.11297E+01_JPRB, 0.11407E+01_JPRB, 0.11570E+01_JPRB, 0.11912E+01_JPRB ,&
&  0.12320E+01_JPRB, 0.12933E+01_JPRB, 0.13913E+01_JPRB, 0.14960E+01_JPRB, 0.16383E+01_JPRB, 0.17763E+01_JPRB, 0.18711E+01_JPRB ,&
&  0.19729E+01_JPRB, 0.20761E+01_JPRB, 0.19664E+01_JPRB, 0.20096E+01_JPRB, 0.21243E+01_JPRB, 0.18996E+01_JPRB, 0.14920E+01_JPRB ,&
&  0.10888E+01_JPRB, 0.97778E+00_JPRB, 0.99561E+00_JPRB, 0.10624E+01_JPRB, 0.94334E+00_JPRB, 0.82720E+00_JPRB, 0.75396E+00_JPRB ,&
&  0.65341E+00_JPRB, 0.57148E+00_JPRB, 0.57920E+00_JPRB, 0.57269E+00_JPRB, 0.56124E+00_JPRB, 0.58169E+00_JPRB, 0.60488E+00_JPRB ,&
&  0.63054E+00_JPRB, 0.67971E+00_JPRB, 0.72992E+00_JPRB, 0.78270E+00_JPRB, 0.81290E+00_JPRB, 0.77804E+00_JPRB, 0.86697E+00_JPRB ,&
&  0.85112E+00_JPRB, 0.83797E+00_JPRB, 0.87364E+00_JPRB, 0.74016E+00_JPRB, 0.10255E+01_JPRB, 0.10938E+01_JPRB, 0.12102E+01_JPRB ,&
&  0.11820E+01_JPRB, 0.15945E+01_JPRB, 0.19743E+01_JPRB, 0.15378E+01_JPRB, 0.12036E+01_JPRB, 0.87484E+00_JPRB, 0.81936E+00_JPRB ,&
&  0.82290E+00_JPRB, 0.83502E+00_JPRB, 0.85894E+00_JPRB, 0.88946E+00_JPRB, 0.91672E+00_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.94359E+00_JPRB, 0.97046E+00_JPRB, 0.99990E+00_JPRB, 0.10161E+01_JPRB, 0.10233E+01_JPRB, 0.10064E+01_JPRB, 0.99184E+00_JPRB ,&
&  0.98912E+00_JPRB, 0.99470E+00_JPRB, 0.10068E+01_JPRB, 0.10203E+01_JPRB, 0.10269E+01_JPRB, 0.10326E+01_JPRB, 0.10477E+01_JPRB ,&
&  0.10671E+01_JPRB, 0.10926E+01_JPRB, 0.11257E+01_JPRB, 0.11487E+01_JPRB, 0.11718E+01_JPRB, 0.11879E+01_JPRB, 0.12146E+01_JPRB ,&
&  0.12355E+01_JPRB, 0.12758E+01_JPRB, 0.12809E+01_JPRB, 0.13066E+01_JPRB, 0.13200E+01_JPRB, 0.13556E+01_JPRB, 0.10951E+01_JPRB ,&
&  0.12708E+01_JPRB, 0.13365E+01_JPRB, 0.13285E+01_JPRB, 0.12729E+01_JPRB, 0.11805E+01_JPRB, 0.11379E+01_JPRB, 0.11125E+01_JPRB ,&
&  0.11088E+01_JPRB, 0.10931E+01_JPRB, 0.10563E+01_JPRB, 0.11189E+01_JPRB, 0.11654E+01_JPRB, 0.11463E+01_JPRB, 0.11879E+01_JPRB ,&
&  0.12330E+01_JPRB, 0.12952E+01_JPRB, 0.13779E+01_JPRB, 0.15046E+01_JPRB, 0.16677E+01_JPRB, 0.18177E+01_JPRB, 0.19651E+01_JPRB ,&
&  0.21183E+01_JPRB, 0.22834E+01_JPRB, 0.24212E+01_JPRB, 0.22041E+01_JPRB, 0.22010E+01_JPRB, 0.19745E+01_JPRB, 0.15097E+01_JPRB ,&
&  0.10694E+01_JPRB, 0.95169E+00_JPRB, 0.99396E+00_JPRB, 0.10658E+01_JPRB, 0.95039E+00_JPRB, 0.80765E+00_JPRB, 0.73723E+00_JPRB ,&
&  0.65860E+00_JPRB, 0.61144E+00_JPRB, 0.59775E+00_JPRB, 0.57128E+00_JPRB, 0.55839E+00_JPRB, 0.58625E+00_JPRB, 0.61730E+00_JPRB ,&
&  0.65725E+00_JPRB, 0.71106E+00_JPRB, 0.77285E+00_JPRB, 0.83581E+00_JPRB, 0.90454E+00_JPRB, 0.92703E+00_JPRB, 0.87215E+00_JPRB ,&
&  0.90948E+00_JPRB, 0.93901E+00_JPRB, 0.99858E+00_JPRB, 0.11075E+01_JPRB, 0.11798E+01_JPRB, 0.14016E+01_JPRB, 0.16389E+01_JPRB ,&
&  0.18603E+01_JPRB, 0.17427E+01_JPRB, 0.20722E+01_JPRB, 0.28803E+01_JPRB, 0.26531E+01_JPRB, 0.21408E+01_JPRB, 0.81040E+00_JPRB ,&
&  0.80129E+00_JPRB, 0.79969E+00_JPRB, 0.83119E+00_JPRB, 0.87103E+00_JPRB, 0.91782E+00_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.80210E+00_JPRB, 0.85044E+00_JPRB, 0.89853E+00_JPRB, 0.95795E+00_JPRB, 0.10050E+01_JPRB, 0.99126E+00_JPRB, 0.96176E+00_JPRB ,&
&  0.94088E+00_JPRB, 0.93068E+00_JPRB, 0.93487E+00_JPRB, 0.94620E+00_JPRB, 0.95433E+00_JPRB, 0.95668E+00_JPRB, 0.96483E+00_JPRB ,&
&  0.97858E+00_JPRB, 0.99836E+00_JPRB, 0.10199E+01_JPRB, 0.10453E+01_JPRB, 0.10649E+01_JPRB, 0.10941E+01_JPRB, 0.11288E+01_JPRB ,&
&  0.11640E+01_JPRB, 0.12100E+01_JPRB, 0.12510E+01_JPRB, 0.12943E+01_JPRB, 0.13359E+01_JPRB, 0.13329E+01_JPRB, 0.99277E+00_JPRB ,&
&  0.12265E+01_JPRB, 0.12620E+01_JPRB, 0.12832E+01_JPRB, 0.12326E+01_JPRB, 0.11402E+01_JPRB, 0.10814E+01_JPRB, 0.10671E+01_JPRB ,&
&  0.10570E+01_JPRB, 0.10387E+01_JPRB, 0.10140E+01_JPRB, 0.10023E+01_JPRB, 0.11006E+01_JPRB, 0.10944E+01_JPRB, 0.11513E+01_JPRB ,&
&  0.12137E+01_JPRB, 0.12787E+01_JPRB, 0.13495E+01_JPRB, 0.14658E+01_JPRB, 0.16319E+01_JPRB, 0.17990E+01_JPRB, 0.19841E+01_JPRB ,&
&  0.21642E+01_JPRB, 0.23239E+01_JPRB, 0.24198E+01_JPRB, 0.21256E+01_JPRB, 0.20229E+01_JPRB, 0.18778E+01_JPRB, 0.13202E+01_JPRB ,&
&  0.10636E+01_JPRB, 0.10189E+01_JPRB, 0.10896E+01_JPRB, 0.10830E+01_JPRB, 0.97071E+00_JPRB, 0.84951E+00_JPRB, 0.76101E+00_JPRB ,&
&  0.65620E+00_JPRB, 0.60366E+00_JPRB, 0.59052E+00_JPRB, 0.57227E+00_JPRB, 0.56501E+00_JPRB, 0.59068E+00_JPRB, 0.62716E+00_JPRB ,&
&  0.66995E+00_JPRB, 0.72243E+00_JPRB, 0.76495E+00_JPRB, 0.80061E+00_JPRB, 0.84574E+00_JPRB, 0.92600E+00_JPRB, 0.97346E+00_JPRB ,&
&  0.89458E+00_JPRB, 0.85814E+00_JPRB, 0.91011E+00_JPRB, 0.95554E+00_JPRB, 0.10044E+01_JPRB, 0.10830E+01_JPRB, 0.11648E+01_JPRB ,&
&  0.12916E+01_JPRB, 0.13611E+01_JPRB, 0.11559E+01_JPRB, 0.97071E+00_JPRB, 0.12903E+01_JPRB, 0.13732E+01_JPRB, 0.70462E+00_JPRB ,&
&  0.67136E+00_JPRB, 0.68869E+00_JPRB, 0.69832E+00_JPRB, 0.73044E+00_JPRB, 0.76455E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.64355E+00_JPRB, 0.65868E+00_JPRB, 0.72639E+00_JPRB, 0.74572E+00_JPRB, 0.81089E+00_JPRB, 0.83528E+00_JPRB, 0.85919E+00_JPRB ,&
&  0.85959E+00_JPRB, 0.85866E+00_JPRB, 0.85619E+00_JPRB, 0.85700E+00_JPRB, 0.85782E+00_JPRB, 0.86317E+00_JPRB, 0.86744E+00_JPRB ,&
&  0.87674E+00_JPRB, 0.89007E+00_JPRB, 0.90661E+00_JPRB, 0.92451E+00_JPRB, 0.94886E+00_JPRB, 0.97809E+00_JPRB, 0.10149E+01_JPRB ,&
&  0.10611E+01_JPRB, 0.11160E+01_JPRB, 0.11790E+01_JPRB, 0.12534E+01_JPRB, 0.13330E+01_JPRB, 0.13902E+01_JPRB, 0.13905E+01_JPRB ,&
&  0.10192E+01_JPRB, 0.12252E+01_JPRB, 0.12099E+01_JPRB, 0.11640E+01_JPRB, 0.11008E+01_JPRB, 0.10589E+01_JPRB, 0.10540E+01_JPRB ,&
&  0.10486E+01_JPRB, 0.10010E+01_JPRB, 0.99915E+00_JPRB, 0.10143E+01_JPRB, 0.10265E+01_JPRB, 0.10169E+01_JPRB, 0.10699E+01_JPRB ,&
&  0.11296E+01_JPRB, 0.12070E+01_JPRB, 0.12978E+01_JPRB, 0.13907E+01_JPRB, 0.15448E+01_JPRB, 0.17459E+01_JPRB, 0.19588E+01_JPRB ,&
&  0.21814E+01_JPRB, 0.24017E+01_JPRB, 0.25432E+01_JPRB, 0.22591E+01_JPRB, 0.19816E+01_JPRB, 0.18700E+01_JPRB, 0.14045E+01_JPRB ,&
&  0.11645E+01_JPRB, 0.10396E+01_JPRB, 0.11256E+01_JPRB, 0.11480E+01_JPRB, 0.10212E+01_JPRB, 0.91857E+00_JPRB, 0.83245E+00_JPRB ,&
&  0.72832E+00_JPRB, 0.63898E+00_JPRB, 0.60709E+00_JPRB, 0.59528E+00_JPRB, 0.59326E+00_JPRB, 0.61148E+00_JPRB, 0.62984E+00_JPRB ,&
&  0.65480E+00_JPRB, 0.68409E+00_JPRB, 0.70903E+00_JPRB, 0.73645E+00_JPRB, 0.75231E+00_JPRB, 0.77020E+00_JPRB, 0.77865E+00_JPRB ,&
&  0.79477E+00_JPRB, 0.83046E+00_JPRB, 0.83807E+00_JPRB, 0.86462E+00_JPRB, 0.87157E+00_JPRB, 0.89694E+00_JPRB, 0.92698E+00_JPRB ,&
&  0.95481E+00_JPRB, 0.97894E+00_JPRB, 0.97884E+00_JPRB, 0.95502E+00_JPRB, 0.89612E+00_JPRB, 0.81834E+00_JPRB, 0.65878E+00_JPRB ,&
&  0.63137E+00_JPRB, 0.61255E+00_JPRB, 0.61690E+00_JPRB, 0.62032E+00_JPRB, 0.63674E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.55337E+00_JPRB, 0.56763E+00_JPRB, 0.58350E+00_JPRB, 0.60364E+00_JPRB, 0.63385E+00_JPRB, 0.66815E+00_JPRB, 0.71222E+00_JPRB ,&
&  0.75609E+00_JPRB, 0.76655E+00_JPRB, 0.76784E+00_JPRB, 0.76765E+00_JPRB, 0.76750E+00_JPRB, 0.77327E+00_JPRB, 0.78239E+00_JPRB ,&
&  0.79137E+00_JPRB, 0.80028E+00_JPRB, 0.80889E+00_JPRB, 0.81834E+00_JPRB, 0.83166E+00_JPRB, 0.85215E+00_JPRB, 0.88152E+00_JPRB ,&
&  0.91288E+00_JPRB, 0.97264E+00_JPRB, 0.10559E+01_JPRB, 0.11493E+01_JPRB, 0.12478E+01_JPRB, 0.13242E+01_JPRB, 0.13294E+01_JPRB ,&
&  0.82306E+00_JPRB, 0.10343E+01_JPRB, 0.12168E+01_JPRB, 0.11687E+01_JPRB, 0.10996E+01_JPRB, 0.10660E+01_JPRB, 0.10697E+01_JPRB ,&
&  0.10403E+01_JPRB, 0.98806E+00_JPRB, 0.97981E+00_JPRB, 0.10382E+01_JPRB, 0.97121E+00_JPRB, 0.95025E+00_JPRB, 0.97326E+00_JPRB ,&
&  0.10132E+01_JPRB, 0.10620E+01_JPRB, 0.11386E+01_JPRB, 0.12389E+01_JPRB, 0.13825E+01_JPRB, 0.15958E+01_JPRB, 0.18369E+01_JPRB ,&
&  0.21031E+01_JPRB, 0.23963E+01_JPRB, 0.26707E+01_JPRB, 0.24036E+01_JPRB, 0.23627E+01_JPRB, 0.24122E+01_JPRB, 0.21875E+01_JPRB ,&
&  0.15122E+01_JPRB, 0.12002E+01_JPRB, 0.12456E+01_JPRB, 0.12585E+01_JPRB, 0.10978E+01_JPRB, 0.10073E+01_JPRB, 0.92708E+00_JPRB ,&
&  0.84619E+00_JPRB, 0.75675E+00_JPRB, 0.66249E+00_JPRB, 0.63314E+00_JPRB, 0.63004E+00_JPRB, 0.64209E+00_JPRB, 0.65929E+00_JPRB ,&
&  0.66789E+00_JPRB, 0.66951E+00_JPRB, 0.67860E+00_JPRB, 0.68998E+00_JPRB, 0.71717E+00_JPRB, 0.73075E+00_JPRB, 0.73712E+00_JPRB ,&
&  0.76272E+00_JPRB, 0.77846E+00_JPRB, 0.80222E+00_JPRB, 0.83313E+00_JPRB, 0.86151E+00_JPRB, 0.88106E+00_JPRB, 0.88389E+00_JPRB ,&
&  0.88274E+00_JPRB, 0.90587E+00_JPRB, 0.93536E+00_JPRB, 0.93804E+00_JPRB, 0.90712E+00_JPRB, 0.82183E+00_JPRB, 0.74634E+00_JPRB ,&
&  0.66371E+00_JPRB, 0.64212E+00_JPRB, 0.60746E+00_JPRB, 0.58101E+00_JPRB, 0.56153E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.53654E+00_JPRB, 0.53127E+00_JPRB, 0.53422E+00_JPRB, 0.53902E+00_JPRB, 0.55045E+00_JPRB, 0.57572E+00_JPRB, 0.60373E+00_JPRB ,&
&  0.62611E+00_JPRB, 0.66384E+00_JPRB, 0.68930E+00_JPRB, 0.68988E+00_JPRB, 0.68384E+00_JPRB, 0.68601E+00_JPRB, 0.69295E+00_JPRB ,&
&  0.70538E+00_JPRB, 0.71443E+00_JPRB, 0.71854E+00_JPRB, 0.72286E+00_JPRB, 0.73127E+00_JPRB, 0.74123E+00_JPRB, 0.75510E+00_JPRB ,&
&  0.77821E+00_JPRB, 0.81916E+00_JPRB, 0.88022E+00_JPRB, 0.97096E+00_JPRB, 0.10911E+01_JPRB, 0.12205E+01_JPRB, 0.12832E+01_JPRB ,&
&  0.95977E+00_JPRB, 0.55752E+00_JPRB, 0.10374E+01_JPRB, 0.12354E+01_JPRB, 0.11436E+01_JPRB, 0.11089E+01_JPRB, 0.10783E+01_JPRB ,&
&  0.10369E+01_JPRB, 0.10430E+01_JPRB, 0.10188E+01_JPRB, 0.10280E+01_JPRB, 0.93033E+00_JPRB, 0.89442E+00_JPRB, 0.88490E+00_JPRB ,&
&  0.89625E+00_JPRB, 0.92100E+00_JPRB, 0.97395E+00_JPRB, 0.10516E+01_JPRB, 0.11666E+01_JPRB, 0.13453E+01_JPRB, 0.16139E+01_JPRB ,&
&  0.19039E+01_JPRB, 0.22426E+01_JPRB, 0.26941E+01_JPRB, 0.25953E+01_JPRB, 0.26984E+01_JPRB, 0.24706E+01_JPRB, 0.20921E+01_JPRB ,&
&  0.16818E+01_JPRB, 0.13927E+01_JPRB, 0.13999E+01_JPRB, 0.14222E+01_JPRB, 0.12248E+01_JPRB, 0.10462E+01_JPRB, 0.10460E+01_JPRB ,&
&  0.92868E+00_JPRB, 0.87489E+00_JPRB, 0.77707E+00_JPRB, 0.70219E+00_JPRB, 0.67442E+00_JPRB, 0.67978E+00_JPRB, 0.69086E+00_JPRB ,&
&  0.69219E+00_JPRB, 0.69023E+00_JPRB, 0.69468E+00_JPRB, 0.70178E+00_JPRB, 0.71599E+00_JPRB, 0.72568E+00_JPRB, 0.73009E+00_JPRB ,&
&  0.75344E+00_JPRB, 0.77938E+00_JPRB, 0.80494E+00_JPRB, 0.83082E+00_JPRB, 0.85198E+00_JPRB, 0.84276E+00_JPRB, 0.84845E+00_JPRB ,&
&  0.86350E+00_JPRB, 0.87436E+00_JPRB, 0.88422E+00_JPRB, 0.90226E+00_JPRB, 0.90044E+00_JPRB, 0.85881E+00_JPRB, 0.84462E+00_JPRB ,&
&  0.78372E+00_JPRB, 0.71277E+00_JPRB, 0.62874E+00_JPRB, 0.57805E+00_JPRB, 0.55921E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.60722E+00_JPRB, 0.58332E+00_JPRB, 0.54728E+00_JPRB, 0.53800E+00_JPRB, 0.53647E+00_JPRB, 0.53554E+00_JPRB, 0.55131E+00_JPRB ,&
&  0.56604E+00_JPRB, 0.58721E+00_JPRB, 0.58955E+00_JPRB, 0.60887E+00_JPRB, 0.61933E+00_JPRB, 0.61946E+00_JPRB, 0.62563E+00_JPRB ,&
&  0.62790E+00_JPRB, 0.63014E+00_JPRB, 0.63209E+00_JPRB, 0.63787E+00_JPRB, 0.64551E+00_JPRB, 0.64784E+00_JPRB, 0.65320E+00_JPRB ,&
&  0.66699E+00_JPRB, 0.68374E+00_JPRB, 0.70865E+00_JPRB, 0.76937E+00_JPRB, 0.87052E+00_JPRB, 0.99290E+00_JPRB, 0.11190E+01_JPRB ,&
&  0.11933E+01_JPRB, 0.88476E+00_JPRB, 0.66583E+00_JPRB, 0.10967E+01_JPRB, 0.12307E+01_JPRB, 0.12099E+01_JPRB, 0.10840E+01_JPRB ,&
&  0.10970E+01_JPRB, 0.10863E+01_JPRB, 0.10732E+01_JPRB, 0.10494E+01_JPRB, 0.93470E+00_JPRB, 0.87344E+00_JPRB, 0.82807E+00_JPRB ,&
&  0.80485E+00_JPRB, 0.79968E+00_JPRB, 0.83015E+00_JPRB, 0.90242E+00_JPRB, 0.98729E+00_JPRB, 0.11071E+01_JPRB, 0.13339E+01_JPRB ,&
&  0.16210E+01_JPRB, 0.19702E+01_JPRB, 0.25049E+01_JPRB, 0.27442E+01_JPRB, 0.31266E+01_JPRB, 0.26927E+01_JPRB, 0.22285E+01_JPRB ,&
&  0.17811E+01_JPRB, 0.17208E+01_JPRB, 0.17806E+01_JPRB, 0.16080E+01_JPRB, 0.13244E+01_JPRB, 0.11890E+01_JPRB, 0.11857E+01_JPRB ,&
&  0.10442E+01_JPRB, 0.97000E+00_JPRB, 0.90695E+00_JPRB, 0.81806E+00_JPRB, 0.75970E+00_JPRB, 0.73827E+00_JPRB, 0.73242E+00_JPRB ,&
&  0.72748E+00_JPRB, 0.71504E+00_JPRB, 0.70890E+00_JPRB, 0.71354E+00_JPRB, 0.72174E+00_JPRB, 0.72518E+00_JPRB, 0.73585E+00_JPRB ,&
&  0.74949E+00_JPRB, 0.76941E+00_JPRB, 0.79356E+00_JPRB, 0.80963E+00_JPRB, 0.80678E+00_JPRB, 0.79394E+00_JPRB, 0.81452E+00_JPRB ,&
&  0.82557E+00_JPRB, 0.85049E+00_JPRB, 0.85373E+00_JPRB, 0.86645E+00_JPRB, 0.87211E+00_JPRB, 0.88389E+00_JPRB, 0.88627E+00_JPRB ,&
&  0.88457E+00_JPRB, 0.85763E+00_JPRB, 0.77301E+00_JPRB, 0.66388E+00_JPRB, 0.63684E+00_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.66723E+00_JPRB, 0.64631E+00_JPRB, 0.61799E+00_JPRB, 0.58908E+00_JPRB, 0.57335E+00_JPRB, 0.55841E+00_JPRB, 0.54069E+00_JPRB ,&
&  0.53240E+00_JPRB, 0.52859E+00_JPRB, 0.51935E+00_JPRB, 0.52951E+00_JPRB, 0.55143E+00_JPRB, 0.56240E+00_JPRB, 0.56817E+00_JPRB ,&
&  0.57431E+00_JPRB, 0.57900E+00_JPRB, 0.56886E+00_JPRB, 0.56572E+00_JPRB, 0.57076E+00_JPRB, 0.57492E+00_JPRB, 0.57527E+00_JPRB ,&
&  0.57840E+00_JPRB, 0.58693E+00_JPRB, 0.59903E+00_JPRB, 0.62333E+00_JPRB, 0.67848E+00_JPRB, 0.75893E+00_JPRB, 0.86366E+00_JPRB ,&
&  0.99788E+00_JPRB, 0.97435E+00_JPRB, 0.58593E+00_JPRB, 0.93287E+00_JPRB, 0.12534E+01_JPRB, 0.12640E+01_JPRB, 0.12038E+01_JPRB ,&
&  0.11951E+01_JPRB, 0.11639E+01_JPRB, 0.11659E+01_JPRB, 0.10870E+01_JPRB, 0.98429E+00_JPRB, 0.90041E+00_JPRB, 0.82819E+00_JPRB ,&
&  0.76571E+00_JPRB, 0.72718E+00_JPRB, 0.71957E+00_JPRB, 0.75918E+00_JPRB, 0.82933E+00_JPRB, 0.89733E+00_JPRB, 0.10268E+01_JPRB ,&
&  0.12607E+01_JPRB, 0.15580E+01_JPRB, 0.20316E+01_JPRB, 0.24876E+01_JPRB, 0.32250E+01_JPRB, 0.32251E+01_JPRB, 0.26448E+01_JPRB ,&
&  0.21155E+01_JPRB, 0.21071E+01_JPRB, 0.20955E+01_JPRB, 0.18472E+01_JPRB, 0.14545E+01_JPRB, 0.13324E+01_JPRB, 0.13144E+01_JPRB ,&
&  0.11709E+01_JPRB, 0.10593E+01_JPRB, 0.10085E+01_JPRB, 0.95306E+00_JPRB, 0.87328E+00_JPRB, 0.80279E+00_JPRB, 0.76718E+00_JPRB ,&
&  0.75594E+00_JPRB, 0.74514E+00_JPRB, 0.73655E+00_JPRB, 0.72944E+00_JPRB, 0.72805E+00_JPRB, 0.72935E+00_JPRB, 0.73476E+00_JPRB ,&
&  0.74565E+00_JPRB, 0.75933E+00_JPRB, 0.77912E+00_JPRB, 0.78379E+00_JPRB, 0.78564E+00_JPRB, 0.78794E+00_JPRB, 0.78737E+00_JPRB ,&
&  0.78646E+00_JPRB, 0.79190E+00_JPRB, 0.79360E+00_JPRB, 0.77528E+00_JPRB, 0.82008E+00_JPRB, 0.83124E+00_JPRB, 0.86057E+00_JPRB ,&
&  0.89374E+00_JPRB, 0.89945E+00_JPRB, 0.84394E+00_JPRB, 0.71396E+00_JPRB, 0.69267E+00_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.67283E+00_JPRB, 0.65583E+00_JPRB, 0.63742E+00_JPRB, 0.62295E+00_JPRB, 0.61237E+00_JPRB, 0.60127E+00_JPRB, 0.58315E+00_JPRB ,&
&  0.56015E+00_JPRB, 0.54670E+00_JPRB, 0.52237E+00_JPRB, 0.51661E+00_JPRB, 0.52166E+00_JPRB, 0.52608E+00_JPRB, 0.53498E+00_JPRB ,&
&  0.53913E+00_JPRB, 0.54190E+00_JPRB, 0.54146E+00_JPRB, 0.53914E+00_JPRB, 0.53867E+00_JPRB, 0.53612E+00_JPRB, 0.53528E+00_JPRB ,&
&  0.53170E+00_JPRB, 0.53257E+00_JPRB, 0.53891E+00_JPRB, 0.54812E+00_JPRB, 0.57250E+00_JPRB, 0.61481E+00_JPRB, 0.67068E+00_JPRB ,&
&  0.79569E+00_JPRB, 0.83684E+00_JPRB, 0.90973E+00_JPRB, 0.12715E+01_JPRB, 0.14848E+01_JPRB, 0.14184E+01_JPRB, 0.13883E+01_JPRB ,&
&  0.13953E+01_JPRB, 0.13668E+01_JPRB, 0.13429E+01_JPRB, 0.11372E+01_JPRB, 0.97331E+00_JPRB, 0.88440E+00_JPRB, 0.81833E+00_JPRB ,&
&  0.75507E+00_JPRB, 0.70764E+00_JPRB, 0.67228E+00_JPRB, 0.66265E+00_JPRB, 0.68987E+00_JPRB, 0.74769E+00_JPRB, 0.81444E+00_JPRB ,&
&  0.95168E+00_JPRB, 0.11820E+01_JPRB, 0.15110E+01_JPRB, 0.19048E+01_JPRB, 0.26360E+01_JPRB, 0.31775E+01_JPRB, 0.30428E+01_JPRB ,&
&  0.28718E+01_JPRB, 0.26588E+01_JPRB, 0.23632E+01_JPRB, 0.21083E+01_JPRB, 0.16120E+01_JPRB, 0.15707E+01_JPRB, 0.13974E+01_JPRB ,&
&  0.12360E+01_JPRB, 0.11187E+01_JPRB, 0.10536E+01_JPRB, 0.10152E+01_JPRB, 0.96088E+00_JPRB, 0.88389E+00_JPRB, 0.81941E+00_JPRB ,&
&  0.78197E+00_JPRB, 0.77017E+00_JPRB, 0.76495E+00_JPRB, 0.75471E+00_JPRB, 0.74780E+00_JPRB, 0.74642E+00_JPRB, 0.74750E+00_JPRB ,&
&  0.74556E+00_JPRB, 0.74614E+00_JPRB, 0.73641E+00_JPRB, 0.75032E+00_JPRB, 0.75356E+00_JPRB, 0.75127E+00_JPRB, 0.74178E+00_JPRB ,&
&  0.74477E+00_JPRB, 0.76225E+00_JPRB, 0.77093E+00_JPRB, 0.77090E+00_JPRB, 0.76841E+00_JPRB, 0.76915E+00_JPRB, 0.76244E+00_JPRB ,&
&  0.76120E+00_JPRB, 0.73815E+00_JPRB, 0.70792E+00_JPRB, 0.69407E+00_JPRB, 0.68413E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.66576E+00_JPRB, 0.65075E+00_JPRB, 0.63944E+00_JPRB, 0.62622E+00_JPRB, 0.61448E+00_JPRB, 0.60917E+00_JPRB, 0.60409E+00_JPRB ,&
&  0.59405E+00_JPRB, 0.56944E+00_JPRB, 0.54342E+00_JPRB, 0.53174E+00_JPRB, 0.52812E+00_JPRB, 0.51862E+00_JPRB, 0.52053E+00_JPRB ,&
&  0.52231E+00_JPRB, 0.52667E+00_JPRB, 0.52199E+00_JPRB, 0.51886E+00_JPRB, 0.51717E+00_JPRB, 0.51563E+00_JPRB, 0.51251E+00_JPRB ,&
&  0.50685E+00_JPRB, 0.50400E+00_JPRB, 0.50802E+00_JPRB, 0.51313E+00_JPRB, 0.52178E+00_JPRB, 0.54091E+00_JPRB, 0.56025E+00_JPRB ,&
&  0.64175E+00_JPRB, 0.71061E+00_JPRB, 0.78154E+00_JPRB, 0.12650E+01_JPRB, 0.14665E+01_JPRB, 0.15340E+01_JPRB, 0.14416E+01_JPRB ,&
&  0.14511E+01_JPRB, 0.15996E+01_JPRB, 0.13984E+01_JPRB, 0.11163E+01_JPRB, 0.91251E+00_JPRB, 0.81957E+00_JPRB, 0.75934E+00_JPRB ,&
&  0.70944E+00_JPRB, 0.66350E+00_JPRB, 0.63466E+00_JPRB, 0.61798E+00_JPRB, 0.62459E+00_JPRB, 0.64115E+00_JPRB, 0.67692E+00_JPRB ,&
&  0.74908E+00_JPRB, 0.90650E+00_JPRB, 0.11423E+01_JPRB, 0.13359E+01_JPRB, 0.18735E+01_JPRB, 0.25000E+01_JPRB, 0.28652E+01_JPRB ,&
&  0.30567E+01_JPRB, 0.30640E+01_JPRB, 0.25460E+01_JPRB, 0.22105E+01_JPRB, 0.18245E+01_JPRB, 0.16215E+01_JPRB, 0.14152E+01_JPRB ,&
&  0.12705E+01_JPRB, 0.11565E+01_JPRB, 0.10740E+01_JPRB, 0.10271E+01_JPRB, 0.98219E+00_JPRB, 0.92905E+00_JPRB, 0.86338E+00_JPRB ,&
&  0.80473E+00_JPRB, 0.78042E+00_JPRB, 0.77125E+00_JPRB, 0.76202E+00_JPRB, 0.75805E+00_JPRB, 0.75332E+00_JPRB, 0.74850E+00_JPRB ,&
&  0.74354E+00_JPRB, 0.73650E+00_JPRB, 0.71757E+00_JPRB, 0.72341E+00_JPRB, 0.71757E+00_JPRB, 0.69759E+00_JPRB, 0.69709E+00_JPRB ,&
&  0.72522E+00_JPRB, 0.74755E+00_JPRB, 0.75163E+00_JPRB, 0.73623E+00_JPRB, 0.75157E+00_JPRB, 0.76781E+00_JPRB, 0.73402E+00_JPRB ,&
&  0.72500E+00_JPRB, 0.71429E+00_JPRB, 0.69374E+00_JPRB, 0.68089E+00_JPRB, 0.67495E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.65853E+00_JPRB, 0.63654E+00_JPRB, 0.62129E+00_JPRB, 0.60881E+00_JPRB, 0.59794E+00_JPRB, 0.58857E+00_JPRB, 0.58643E+00_JPRB ,&
&  0.58392E+00_JPRB, 0.57752E+00_JPRB, 0.56072E+00_JPRB, 0.54101E+00_JPRB, 0.53231E+00_JPRB, 0.52998E+00_JPRB, 0.51993E+00_JPRB ,&
&  0.51610E+00_JPRB, 0.51645E+00_JPRB, 0.51380E+00_JPRB, 0.50637E+00_JPRB, 0.50017E+00_JPRB, 0.49646E+00_JPRB, 0.49412E+00_JPRB ,&
&  0.48387E+00_JPRB, 0.47871E+00_JPRB, 0.47968E+00_JPRB, 0.48070E+00_JPRB, 0.48356E+00_JPRB, 0.50117E+00_JPRB, 0.52008E+00_JPRB ,&
&  0.56379E+00_JPRB, 0.71504E+00_JPRB, 0.75741E+00_JPRB, 0.10542E+01_JPRB, 0.11893E+01_JPRB, 0.13300E+01_JPRB, 0.14008E+01_JPRB ,&
&  0.14906E+01_JPRB, 0.15927E+01_JPRB, 0.13284E+01_JPRB, 0.10672E+01_JPRB, 0.84406E+00_JPRB, 0.72389E+00_JPRB, 0.66520E+00_JPRB ,&
&  0.63492E+00_JPRB, 0.60670E+00_JPRB, 0.58675E+00_JPRB, 0.58156E+00_JPRB, 0.58612E+00_JPRB, 0.59391E+00_JPRB, 0.60377E+00_JPRB ,&
&  0.64295E+00_JPRB, 0.73609E+00_JPRB, 0.95279E+00_JPRB, 0.12151E+01_JPRB, 0.13505E+01_JPRB, 0.17783E+01_JPRB, 0.24121E+01_JPRB ,&
&  0.29648E+01_JPRB, 0.32124E+01_JPRB, 0.26754E+01_JPRB, 0.22355E+01_JPRB, 0.19099E+01_JPRB, 0.16645E+01_JPRB, 0.14551E+01_JPRB ,&
&  0.12931E+01_JPRB, 0.11719E+01_JPRB, 0.10849E+01_JPRB, 0.10238E+01_JPRB, 0.98175E+00_JPRB, 0.95084E+00_JPRB, 0.90609E+00_JPRB ,&
&  0.85346E+00_JPRB, 0.81498E+00_JPRB, 0.79231E+00_JPRB, 0.78035E+00_JPRB, 0.77427E+00_JPRB, 0.76211E+00_JPRB, 0.74228E+00_JPRB ,&
&  0.72711E+00_JPRB, 0.72155E+00_JPRB, 0.70501E+00_JPRB, 0.70825E+00_JPRB, 0.70431E+00_JPRB, 0.68142E+00_JPRB, 0.67434E+00_JPRB ,&
&  0.70531E+00_JPRB, 0.72212E+00_JPRB, 0.72868E+00_JPRB, 0.73415E+00_JPRB, 0.74941E+00_JPRB, 0.79218E+00_JPRB, 0.74268E+00_JPRB ,&
&  0.71256E+00_JPRB, 0.70027E+00_JPRB, 0.68659E+00_JPRB, 0.67923E+00_JPRB, 0.67145E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.65067E+00_JPRB, 0.62730E+00_JPRB, 0.60860E+00_JPRB, 0.59031E+00_JPRB, 0.57695E+00_JPRB, 0.56334E+00_JPRB, 0.55439E+00_JPRB ,&
&  0.55086E+00_JPRB, 0.55111E+00_JPRB, 0.54921E+00_JPRB, 0.54123E+00_JPRB, 0.52759E+00_JPRB, 0.51724E+00_JPRB, 0.51053E+00_JPRB ,&
&  0.50363E+00_JPRB, 0.50254E+00_JPRB, 0.50195E+00_JPRB, 0.49439E+00_JPRB, 0.48604E+00_JPRB, 0.47953E+00_JPRB, 0.47256E+00_JPRB ,&
&  0.46629E+00_JPRB, 0.46263E+00_JPRB, 0.45969E+00_JPRB, 0.45723E+00_JPRB, 0.45609E+00_JPRB, 0.46477E+00_JPRB, 0.50132E+00_JPRB ,&
&  0.57577E+00_JPRB, 0.49885E+00_JPRB, 0.86990E+00_JPRB, 0.97214E+00_JPRB, 0.10144E+01_JPRB, 0.10794E+01_JPRB, 0.12375E+01_JPRB ,&
&  0.14075E+01_JPRB, 0.13820E+01_JPRB, 0.12138E+01_JPRB, 0.10100E+01_JPRB, 0.82750E+00_JPRB, 0.70306E+00_JPRB, 0.63719E+00_JPRB ,&
&  0.59022E+00_JPRB, 0.55844E+00_JPRB, 0.54481E+00_JPRB, 0.53710E+00_JPRB, 0.54040E+00_JPRB, 0.55065E+00_JPRB, 0.55924E+00_JPRB ,&
&  0.58234E+00_JPRB, 0.63574E+00_JPRB, 0.78339E+00_JPRB, 0.10638E+01_JPRB, 0.13246E+01_JPRB, 0.15314E+01_JPRB, 0.19508E+01_JPRB ,&
&  0.26866E+01_JPRB, 0.28429E+01_JPRB, 0.23623E+01_JPRB, 0.20104E+01_JPRB, 0.17845E+01_JPRB, 0.16201E+01_JPRB, 0.14595E+01_JPRB ,&
&  0.13066E+01_JPRB, 0.11609E+01_JPRB, 0.10561E+01_JPRB, 0.98426E+00_JPRB, 0.94256E+00_JPRB, 0.92563E+00_JPRB, 0.91161E+00_JPRB ,&
&  0.89365E+00_JPRB, 0.86785E+00_JPRB, 0.84290E+00_JPRB, 0.81899E+00_JPRB, 0.79924E+00_JPRB, 0.77589E+00_JPRB, 0.74241E+00_JPRB ,&
&  0.71677E+00_JPRB, 0.70426E+00_JPRB, 0.68721E+00_JPRB, 0.67859E+00_JPRB, 0.68004E+00_JPRB, 0.67729E+00_JPRB, 0.67894E+00_JPRB ,&
&  0.69099E+00_JPRB, 0.69654E+00_JPRB, 0.69994E+00_JPRB, 0.71118E+00_JPRB, 0.70859E+00_JPRB, 0.75098E+00_JPRB, 0.73401E+00_JPRB ,&
&  0.72600E+00_JPRB, 0.70832E+00_JPRB, 0.68546E+00_JPRB, 0.67324E+00_JPRB, 0.66567E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.64281E+00_JPRB, 0.62554E+00_JPRB, 0.59987E+00_JPRB, 0.57732E+00_JPRB, 0.56003E+00_JPRB, 0.54224E+00_JPRB, 0.52992E+00_JPRB ,&
&  0.52085E+00_JPRB, 0.52136E+00_JPRB, 0.52467E+00_JPRB, 0.52625E+00_JPRB, 0.52125E+00_JPRB, 0.50843E+00_JPRB, 0.49564E+00_JPRB ,&
&  0.48717E+00_JPRB, 0.48549E+00_JPRB, 0.48578E+00_JPRB, 0.48270E+00_JPRB, 0.47731E+00_JPRB, 0.47007E+00_JPRB, 0.46320E+00_JPRB ,&
&  0.45681E+00_JPRB, 0.45144E+00_JPRB, 0.44619E+00_JPRB, 0.44275E+00_JPRB, 0.43863E+00_JPRB, 0.44261E+00_JPRB, 0.48251E+00_JPRB ,&
&  0.58827E+00_JPRB, 0.67342E+00_JPRB, 0.81224E+00_JPRB, 0.79940E+00_JPRB, 0.81158E+00_JPRB, 0.85575E+00_JPRB, 0.98101E+00_JPRB ,&
&  0.10537E+01_JPRB, 0.10732E+01_JPRB, 0.10215E+01_JPRB, 0.91794E+00_JPRB, 0.79240E+00_JPRB, 0.69721E+00_JPRB, 0.62594E+00_JPRB ,&
&  0.56723E+00_JPRB, 0.52946E+00_JPRB, 0.50798E+00_JPRB, 0.50240E+00_JPRB, 0.50677E+00_JPRB, 0.51686E+00_JPRB, 0.52218E+00_JPRB ,&
&  0.53578E+00_JPRB, 0.56634E+00_JPRB, 0.67832E+00_JPRB, 0.93640E+00_JPRB, 0.13628E+01_JPRB, 0.14227E+01_JPRB, 0.19122E+01_JPRB ,&
&  0.24841E+01_JPRB, 0.22129E+01_JPRB, 0.19892E+01_JPRB, 0.18041E+01_JPRB, 0.16608E+01_JPRB, 0.15341E+01_JPRB, 0.13440E+01_JPRB ,&
&  0.11949E+01_JPRB, 0.10609E+01_JPRB, 0.96258E+00_JPRB, 0.90861E+00_JPRB, 0.87525E+00_JPRB, 0.86140E+00_JPRB, 0.85827E+00_JPRB ,&
&  0.85499E+00_JPRB, 0.84806E+00_JPRB, 0.84322E+00_JPRB, 0.82766E+00_JPRB, 0.80277E+00_JPRB, 0.77366E+00_JPRB, 0.74165E+00_JPRB ,&
&  0.71433E+00_JPRB, 0.69091E+00_JPRB, 0.66926E+00_JPRB, 0.64945E+00_JPRB, 0.65135E+00_JPRB, 0.66022E+00_JPRB, 0.66198E+00_JPRB ,&
&  0.65564E+00_JPRB, 0.65441E+00_JPRB, 0.66768E+00_JPRB, 0.66929E+00_JPRB, 0.67644E+00_JPRB, 0.71980E+00_JPRB, 0.71977E+00_JPRB ,&
&  0.71713E+00_JPRB, 0.69686E+00_JPRB, 0.67568E+00_JPRB, 0.65825E+00_JPRB, 0.65263E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.63948E+00_JPRB, 0.61707E+00_JPRB, 0.59072E+00_JPRB, 0.56620E+00_JPRB, 0.54424E+00_JPRB, 0.52742E+00_JPRB, 0.51728E+00_JPRB ,&
&  0.50882E+00_JPRB, 0.50331E+00_JPRB, 0.50447E+00_JPRB, 0.50854E+00_JPRB, 0.50785E+00_JPRB, 0.49972E+00_JPRB, 0.48797E+00_JPRB ,&
&  0.47629E+00_JPRB, 0.46787E+00_JPRB, 0.46216E+00_JPRB, 0.46044E+00_JPRB, 0.45676E+00_JPRB, 0.45293E+00_JPRB, 0.44661E+00_JPRB ,&
&  0.44210E+00_JPRB, 0.43780E+00_JPRB, 0.43102E+00_JPRB, 0.42400E+00_JPRB, 0.41942E+00_JPRB, 0.42062E+00_JPRB, 0.45459E+00_JPRB ,&
&  0.54341E+00_JPRB, 0.61019E+00_JPRB, 0.62673E+00_JPRB, 0.65856E+00_JPRB, 0.69762E+00_JPRB, 0.72190E+00_JPRB, 0.77864E+00_JPRB ,&
&  0.81187E+00_JPRB, 0.82752E+00_JPRB, 0.83324E+00_JPRB, 0.79660E+00_JPRB, 0.72743E+00_JPRB, 0.67269E+00_JPRB, 0.61548E+00_JPRB ,&
&  0.56210E+00_JPRB, 0.52021E+00_JPRB, 0.49356E+00_JPRB, 0.48322E+00_JPRB, 0.48641E+00_JPRB, 0.49356E+00_JPRB, 0.49630E+00_JPRB ,&
&  0.50126E+00_JPRB, 0.50876E+00_JPRB, 0.55277E+00_JPRB, 0.65879E+00_JPRB, 0.93124E+00_JPRB, 0.11071E+01_JPRB, 0.13984E+01_JPRB ,&
&  0.14805E+01_JPRB, 0.14512E+01_JPRB, 0.13992E+01_JPRB, 0.13382E+01_JPRB, 0.12725E+01_JPRB, 0.11819E+01_JPRB, 0.11057E+01_JPRB ,&
&  0.10180E+01_JPRB, 0.93056E+00_JPRB, 0.86234E+00_JPRB, 0.81694E+00_JPRB, 0.78956E+00_JPRB, 0.77403E+00_JPRB, 0.76946E+00_JPRB ,&
&  0.75997E+00_JPRB, 0.75732E+00_JPRB, 0.74912E+00_JPRB, 0.74420E+00_JPRB, 0.74176E+00_JPRB, 0.72517E+00_JPRB, 0.69749E+00_JPRB ,&
&  0.66807E+00_JPRB, 0.64322E+00_JPRB, 0.62765E+00_JPRB, 0.62107E+00_JPRB, 0.61910E+00_JPRB, 0.61674E+00_JPRB, 0.61401E+00_JPRB ,&
&  0.62095E+00_JPRB, 0.62671E+00_JPRB, 0.63113E+00_JPRB, 0.62965E+00_JPRB, 0.62766E+00_JPRB, 0.66014E+00_JPRB, 0.66230E+00_JPRB ,&
&  0.66449E+00_JPRB, 0.66277E+00_JPRB, 0.65223E+00_JPRB, 0.63654E+00_JPRB, 0.64882E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.65264E+00_JPRB, 0.63183E+00_JPRB, 0.60255E+00_JPRB, 0.57335E+00_JPRB, 0.54582E+00_JPRB, 0.52420E+00_JPRB, 0.51121E+00_JPRB ,&
&  0.50290E+00_JPRB, 0.49749E+00_JPRB, 0.49440E+00_JPRB, 0.49234E+00_JPRB, 0.48922E+00_JPRB, 0.48599E+00_JPRB, 0.48063E+00_JPRB ,&
&  0.47392E+00_JPRB, 0.46580E+00_JPRB, 0.45847E+00_JPRB, 0.45360E+00_JPRB, 0.44939E+00_JPRB, 0.44710E+00_JPRB, 0.44433E+00_JPRB ,&
&  0.43802E+00_JPRB, 0.43002E+00_JPRB, 0.41969E+00_JPRB, 0.40912E+00_JPRB, 0.40301E+00_JPRB, 0.40264E+00_JPRB, 0.42531E+00_JPRB ,&
&  0.49588E+00_JPRB, 0.49283E+00_JPRB, 0.52815E+00_JPRB, 0.56017E+00_JPRB, 0.59305E+00_JPRB, 0.62184E+00_JPRB, 0.64721E+00_JPRB ,&
&  0.66374E+00_JPRB, 0.67992E+00_JPRB, 0.70289E+00_JPRB, 0.69728E+00_JPRB, 0.65579E+00_JPRB, 0.61855E+00_JPRB, 0.58226E+00_JPRB ,&
&  0.54740E+00_JPRB, 0.51561E+00_JPRB, 0.48869E+00_JPRB, 0.47814E+00_JPRB, 0.47693E+00_JPRB, 0.47981E+00_JPRB, 0.47973E+00_JPRB ,&
&  0.48145E+00_JPRB, 0.47990E+00_JPRB, 0.48549E+00_JPRB, 0.50682E+00_JPRB, 0.55125E+00_JPRB, 0.63429E+00_JPRB, 0.77746E+00_JPRB ,&
&  0.84720E+00_JPRB, 0.85381E+00_JPRB, 0.87073E+00_JPRB, 0.88340E+00_JPRB, 0.86270E+00_JPRB, 0.83635E+00_JPRB, 0.81068E+00_JPRB ,&
&  0.78559E+00_JPRB, 0.76042E+00_JPRB, 0.73932E+00_JPRB, 0.71336E+00_JPRB, 0.69206E+00_JPRB, 0.67779E+00_JPRB, 0.66861E+00_JPRB ,&
&  0.66143E+00_JPRB, 0.65873E+00_JPRB, 0.65277E+00_JPRB, 0.63862E+00_JPRB, 0.62894E+00_JPRB, 0.61905E+00_JPRB, 0.60238E+00_JPRB ,&
&  0.58000E+00_JPRB, 0.56678E+00_JPRB, 0.56137E+00_JPRB, 0.56245E+00_JPRB, 0.56736E+00_JPRB, 0.57312E+00_JPRB, 0.57548E+00_JPRB ,&
&  0.58024E+00_JPRB, 0.59338E+00_JPRB, 0.58685E+00_JPRB, 0.58351E+00_JPRB, 0.61239E+00_JPRB, 0.61419E+00_JPRB, 0.61939E+00_JPRB ,&
&  0.62487E+00_JPRB, 0.62602E+00_JPRB, 0.62018E+00_JPRB, 0.60445E+00_JPRB, 0.65050E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.56158E+00_JPRB, 0.56649E+00_JPRB, 0.55376E+00_JPRB, 0.53680E+00_JPRB, 0.52033E+00_JPRB, 0.50735E+00_JPRB, 0.49638E+00_JPRB ,&
&  0.48887E+00_JPRB, 0.48361E+00_JPRB, 0.48024E+00_JPRB, 0.47816E+00_JPRB, 0.47500E+00_JPRB, 0.46985E+00_JPRB, 0.46417E+00_JPRB ,&
&  0.45954E+00_JPRB, 0.45403E+00_JPRB, 0.44539E+00_JPRB, 0.43951E+00_JPRB, 0.43616E+00_JPRB, 0.43188E+00_JPRB, 0.42759E+00_JPRB ,&
&  0.42227E+00_JPRB, 0.41136E+00_JPRB, 0.40044E+00_JPRB, 0.39353E+00_JPRB, 0.38830E+00_JPRB, 0.38605E+00_JPRB, 0.39971E+00_JPRB ,&
&  0.44731E+00_JPRB, 0.42042E+00_JPRB, 0.45191E+00_JPRB, 0.48655E+00_JPRB, 0.52368E+00_JPRB, 0.54775E+00_JPRB, 0.56695E+00_JPRB ,&
&  0.57989E+00_JPRB, 0.59549E+00_JPRB, 0.60706E+00_JPRB, 0.62181E+00_JPRB, 0.60088E+00_JPRB, 0.56944E+00_JPRB, 0.54352E+00_JPRB ,&
&  0.51970E+00_JPRB, 0.49726E+00_JPRB, 0.47894E+00_JPRB, 0.46672E+00_JPRB, 0.46367E+00_JPRB, 0.46259E+00_JPRB, 0.45918E+00_JPRB ,&
&  0.45902E+00_JPRB, 0.45610E+00_JPRB, 0.45398E+00_JPRB, 0.45932E+00_JPRB, 0.45948E+00_JPRB, 0.48305E+00_JPRB, 0.51982E+00_JPRB ,&
&  0.55210E+00_JPRB, 0.59519E+00_JPRB, 0.61746E+00_JPRB, 0.63220E+00_JPRB, 0.62810E+00_JPRB, 0.62197E+00_JPRB, 0.61493E+00_JPRB ,&
&  0.60805E+00_JPRB, 0.60127E+00_JPRB, 0.59472E+00_JPRB, 0.58587E+00_JPRB, 0.57585E+00_JPRB, 0.56800E+00_JPRB, 0.56129E+00_JPRB ,&
&  0.55590E+00_JPRB, 0.55111E+00_JPRB, 0.54797E+00_JPRB, 0.53752E+00_JPRB, 0.52135E+00_JPRB, 0.50807E+00_JPRB, 0.49789E+00_JPRB ,&
&  0.48985E+00_JPRB, 0.48786E+00_JPRB, 0.48958E+00_JPRB, 0.49684E+00_JPRB, 0.50422E+00_JPRB, 0.51273E+00_JPRB, 0.51828E+00_JPRB ,&
&  0.52340E+00_JPRB, 0.53500E+00_JPRB, 0.55126E+00_JPRB, 0.57118E+00_JPRB, 0.57945E+00_JPRB, 0.58436E+00_JPRB, 0.58830E+00_JPRB ,&
&  0.59000E+00_JPRB, 0.59009E+00_JPRB, 0.58208E+00_JPRB, 0.57244E+00_JPRB, 0.55009E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.51105E+00_JPRB, 0.51622E+00_JPRB, 0.51159E+00_JPRB, 0.50087E+00_JPRB, 0.49090E+00_JPRB, 0.48150E+00_JPRB, 0.47396E+00_JPRB ,&
&  0.46338E+00_JPRB, 0.45455E+00_JPRB, 0.45147E+00_JPRB, 0.45129E+00_JPRB, 0.45054E+00_JPRB, 0.44667E+00_JPRB, 0.44130E+00_JPRB ,&
&  0.43555E+00_JPRB, 0.42897E+00_JPRB, 0.42126E+00_JPRB, 0.41358E+00_JPRB, 0.40727E+00_JPRB, 0.40007E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.38967E+00_JPRB, 0.38551E+00_JPRB, 0.37860E+00_JPRB, 0.36981E+00_JPRB, 0.36614E+00_JPRB, 0.36132E+00_JPRB, 0.37393E+00_JPRB ,&
&  0.41699E+00_JPRB, 0.36179E+00_JPRB, 0.38389E+00_JPRB, 0.42376E+00_JPRB, 0.45973E+00_JPRB, 0.50103E+00_JPRB, 0.52257E+00_JPRB ,&
&  0.53717E+00_JPRB, 0.54620E+00_JPRB, 0.55063E+00_JPRB, 0.55849E+00_JPRB, 0.55627E+00_JPRB, 0.53617E+00_JPRB, 0.50997E+00_JPRB ,&
&  0.49104E+00_JPRB, 0.47204E+00_JPRB, 0.45974E+00_JPRB, 0.44797E+00_JPRB, 0.44308E+00_JPRB, 0.44184E+00_JPRB, 0.43630E+00_JPRB ,&
&  0.43057E+00_JPRB, 0.43151E+00_JPRB, 0.43349E+00_JPRB, 0.43879E+00_JPRB, 0.44286E+00_JPRB, 0.44742E+00_JPRB, 0.44233E+00_JPRB ,&
&  0.43824E+00_JPRB, 0.45375E+00_JPRB, 0.47896E+00_JPRB, 0.49273E+00_JPRB, 0.49798E+00_JPRB, 0.49478E+00_JPRB, 0.48755E+00_JPRB ,&
&  0.48239E+00_JPRB, 0.47899E+00_JPRB, 0.47913E+00_JPRB, 0.47695E+00_JPRB, 0.47340E+00_JPRB, 0.46751E+00_JPRB, 0.46082E+00_JPRB ,&
&  0.45481E+00_JPRB, 0.44984E+00_JPRB, 0.44587E+00_JPRB, 0.44193E+00_JPRB, 0.43677E+00_JPRB, 0.42899E+00_JPRB, 0.42496E+00_JPRB ,&
&  0.42358E+00_JPRB, 0.42513E+00_JPRB, 0.42953E+00_JPRB, 0.43639E+00_JPRB, 0.44576E+00_JPRB, 0.45430E+00_JPRB, 0.46458E+00_JPRB ,&
&  0.47475E+00_JPRB, 0.48626E+00_JPRB, 0.50106E+00_JPRB, 0.51161E+00_JPRB, 0.53628E+00_JPRB, 0.54353E+00_JPRB, 0.55076E+00_JPRB ,&
&  0.55347E+00_JPRB, 0.55365E+00_JPRB, 0.55393E+00_JPRB, 0.48573E+00_JPRB, 0.50526E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.46984E+00_JPRB, 0.47101E+00_JPRB, 0.46701E+00_JPRB, 0.46124E+00_JPRB, 0.45607E+00_JPRB, 0.45030E+00_JPRB, 0.44535E+00_JPRB ,&
&  0.43793E+00_JPRB, 0.42880E+00_JPRB, 0.42233E+00_JPRB, 0.41969E+00_JPRB, 0.41839E+00_JPRB, 0.41558E+00_JPRB, 0.41041E+00_JPRB ,&
&  0.40382E+00_JPRB, 0.39692E+00_JPRB, 0.39051E+00_JPRB, 0.38282E+00_JPRB, 0.37566E+00_JPRB, 0.36916E+00_JPRB, 0.36402E+00_JPRB ,&
&  0.35887E+00_JPRB, 0.35326E+00_JPRB, 0.34769E+00_JPRB, 0.34397E+00_JPRB, 0.34092E+00_JPRB, 0.33958E+00_JPRB, 0.35008E+00_JPRB ,&
&  0.38227E+00_JPRB, 0.32943E+00_JPRB, 0.34662E+00_JPRB, 0.37137E+00_JPRB, 0.40707E+00_JPRB, 0.45234E+00_JPRB, 0.47907E+00_JPRB ,&
&  0.49350E+00_JPRB, 0.50233E+00_JPRB, 0.50455E+00_JPRB, 0.50529E+00_JPRB, 0.50754E+00_JPRB, 0.50051E+00_JPRB, 0.48472E+00_JPRB ,&
&  0.46837E+00_JPRB, 0.45232E+00_JPRB, 0.43964E+00_JPRB, 0.43030E+00_JPRB, 0.42294E+00_JPRB, 0.41819E+00_JPRB, 0.41094E+00_JPRB ,&
&  0.40091E+00_JPRB, 0.39809E+00_JPRB, 0.39831E+00_JPRB, 0.39786E+00_JPRB, 0.39790E+00_JPRB, 0.39561E+00_JPRB, 0.38563E+00_JPRB ,&
&  0.37739E+00_JPRB, 0.37374E+00_JPRB, 0.37487E+00_JPRB, 0.38003E+00_JPRB, 0.38510E+00_JPRB, 0.38728E+00_JPRB, 0.38497E+00_JPRB ,&
&  0.38119E+00_JPRB, 0.37583E+00_JPRB, 0.37336E+00_JPRB, 0.37267E+00_JPRB, 0.37226E+00_JPRB, 0.37355E+00_JPRB, 0.37402E+00_JPRB ,&
&  0.37296E+00_JPRB, 0.37318E+00_JPRB, 0.37553E+00_JPRB, 0.37781E+00_JPRB, 0.37847E+00_JPRB, 0.37759E+00_JPRB, 0.37607E+00_JPRB ,&
&  0.37386E+00_JPRB, 0.37261E+00_JPRB, 0.37415E+00_JPRB, 0.37961E+00_JPRB, 0.38717E+00_JPRB, 0.39575E+00_JPRB, 0.40523E+00_JPRB ,&
&  0.41593E+00_JPRB, 0.42737E+00_JPRB, 0.44038E+00_JPRB, 0.46334E+00_JPRB, 0.47563E+00_JPRB, 0.48777E+00_JPRB, 0.49883E+00_JPRB ,&
&  0.50037E+00_JPRB, 0.49809E+00_JPRB, 0.42187E+00_JPRB, 0.45331E+00_JPRB, 0.46231E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.44538E+00_JPRB, 0.43720E+00_JPRB, 0.42730E+00_JPRB, 0.42116E+00_JPRB, 0.41916E+00_JPRB, 0.41657E+00_JPRB, 0.41372E+00_JPRB ,&
&  0.40877E+00_JPRB, 0.40301E+00_JPRB, 0.39739E+00_JPRB, 0.39233E+00_JPRB, 0.38817E+00_JPRB, 0.38513E+00_JPRB, 0.38125E+00_JPRB ,&
&  0.37561E+00_JPRB, 0.36925E+00_JPRB, 0.36292E+00_JPRB, 0.35584E+00_JPRB, 0.34648E+00_JPRB, 0.33907E+00_JPRB, 0.33321E+00_JPRB ,&
&  0.32858E+00_JPRB, 0.32492E+00_JPRB, 0.32373E+00_JPRB, 0.32217E+00_JPRB, 0.32045E+00_JPRB, 0.32073E+00_JPRB, 0.33072E+00_JPRB ,&
&  0.32506E+00_JPRB, 0.30030E+00_JPRB, 0.31606E+00_JPRB, 0.33728E+00_JPRB, 0.36900E+00_JPRB, 0.41656E+00_JPRB, 0.44949E+00_JPRB ,&
&  0.46790E+00_JPRB, 0.47701E+00_JPRB, 0.48032E+00_JPRB, 0.47777E+00_JPRB, 0.47334E+00_JPRB, 0.47235E+00_JPRB, 0.46658E+00_JPRB ,&
&  0.45453E+00_JPRB, 0.43995E+00_JPRB, 0.42546E+00_JPRB, 0.41475E+00_JPRB, 0.40369E+00_JPRB, 0.39533E+00_JPRB, 0.38639E+00_JPRB ,&
&  0.37793E+00_JPRB, 0.37227E+00_JPRB, 0.37013E+00_JPRB, 0.36825E+00_JPRB, 0.36558E+00_JPRB, 0.35937E+00_JPRB, 0.35194E+00_JPRB ,&
&  0.34675E+00_JPRB, 0.34250E+00_JPRB, 0.33800E+00_JPRB, 0.33388E+00_JPRB, 0.33146E+00_JPRB, 0.32979E+00_JPRB, 0.32789E+00_JPRB ,&
&  0.32743E+00_JPRB, 0.32789E+00_JPRB, 0.32913E+00_JPRB, 0.33047E+00_JPRB, 0.33105E+00_JPRB, 0.33036E+00_JPRB, 0.33065E+00_JPRB ,&
&  0.33171E+00_JPRB, 0.33024E+00_JPRB, 0.33007E+00_JPRB, 0.33339E+00_JPRB, 0.33754E+00_JPRB, 0.34033E+00_JPRB, 0.34134E+00_JPRB ,&
&  0.34036E+00_JPRB, 0.33992E+00_JPRB, 0.34102E+00_JPRB, 0.34409E+00_JPRB, 0.34803E+00_JPRB, 0.35385E+00_JPRB, 0.36097E+00_JPRB ,&
&  0.36845E+00_JPRB, 0.37778E+00_JPRB, 0.39051E+00_JPRB, 0.40909E+00_JPRB, 0.42362E+00_JPRB, 0.43482E+00_JPRB, 0.44355E+00_JPRB ,&
&  0.44500E+00_JPRB, 0.44478E+00_JPRB, 0.44904E+00_JPRB, 0.44889E+00_JPRB, 0.44854E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.39638E+00_JPRB, 0.39125E+00_JPRB, 0.38503E+00_JPRB, 0.38004E+00_JPRB, 0.37800E+00_JPRB, 0.37649E+00_JPRB, 0.37310E+00_JPRB ,&
&  0.36997E+00_JPRB, 0.36709E+00_JPRB, 0.36400E+00_JPRB, 0.36131E+00_JPRB, 0.35897E+00_JPRB, 0.35716E+00_JPRB, 0.35384E+00_JPRB ,&
&  0.34854E+00_JPRB, 0.34187E+00_JPRB, 0.33450E+00_JPRB, 0.32849E+00_JPRB, 0.32107E+00_JPRB, 0.31344E+00_JPRB, 0.30784E+00_JPRB ,&
&  0.30320E+00_JPRB, 0.30011E+00_JPRB, 0.29914E+00_JPRB, 0.29766E+00_JPRB, 0.29871E+00_JPRB, 0.30203E+00_JPRB, 0.31043E+00_JPRB ,&
&  0.32046E+00_JPRB, 0.30102E+00_JPRB, 0.31573E+00_JPRB, 0.32831E+00_JPRB, 0.35479E+00_JPRB, 0.39105E+00_JPRB, 0.41632E+00_JPRB ,&
&  0.43344E+00_JPRB, 0.44668E+00_JPRB, 0.45621E+00_JPRB, 0.45889E+00_JPRB, 0.45306E+00_JPRB, 0.44885E+00_JPRB, 0.44575E+00_JPRB ,&
&  0.43809E+00_JPRB, 0.42716E+00_JPRB, 0.41251E+00_JPRB, 0.39705E+00_JPRB, 0.38288E+00_JPRB, 0.37238E+00_JPRB, 0.36395E+00_JPRB ,&
&  0.35718E+00_JPRB, 0.35128E+00_JPRB, 0.34650E+00_JPRB, 0.33957E+00_JPRB, 0.33169E+00_JPRB, 0.32370E+00_JPRB, 0.31645E+00_JPRB ,&
&  0.30940E+00_JPRB, 0.30242E+00_JPRB, 0.29622E+00_JPRB, 0.29096E+00_JPRB, 0.28811E+00_JPRB, 0.28685E+00_JPRB, 0.28649E+00_JPRB ,&
&  0.28698E+00_JPRB, 0.28884E+00_JPRB, 0.29102E+00_JPRB, 0.29309E+00_JPRB, 0.29472E+00_JPRB, 0.29476E+00_JPRB, 0.29275E+00_JPRB ,&
&  0.29241E+00_JPRB, 0.29226E+00_JPRB, 0.29256E+00_JPRB, 0.29406E+00_JPRB, 0.29494E+00_JPRB, 0.29616E+00_JPRB, 0.29892E+00_JPRB ,&
&  0.30142E+00_JPRB, 0.30282E+00_JPRB, 0.30417E+00_JPRB, 0.30576E+00_JPRB, 0.30781E+00_JPRB, 0.31116E+00_JPRB, 0.31609E+00_JPRB ,&
&  0.32182E+00_JPRB, 0.32829E+00_JPRB, 0.33769E+00_JPRB, 0.35017E+00_JPRB, 0.36230E+00_JPRB, 0.37280E+00_JPRB, 0.38148E+00_JPRB ,&
&  0.38500E+00_JPRB, 0.38791E+00_JPRB, 0.39296E+00_JPRB, 0.39584E+00_JPRB, 0.39830E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.35638E+00_JPRB, 0.35379E+00_JPRB, 0.35084E+00_JPRB, 0.34858E+00_JPRB, 0.34718E+00_JPRB, 0.34414E+00_JPRB, 0.34030E+00_JPRB ,&
&  0.33784E+00_JPRB, 0.33556E+00_JPRB, 0.33399E+00_JPRB, 0.33127E+00_JPRB, 0.32997E+00_JPRB, 0.32930E+00_JPRB, 0.32574E+00_JPRB ,&
&  0.32137E+00_JPRB, 0.31494E+00_JPRB, 0.30856E+00_JPRB, 0.30280E+00_JPRB, 0.29729E+00_JPRB, 0.29220E+00_JPRB, 0.28657E+00_JPRB ,&
&  0.28134E+00_JPRB, 0.27751E+00_JPRB, 0.27555E+00_JPRB, 0.27388E+00_JPRB, 0.27537E+00_JPRB, 0.28136E+00_JPRB, 0.29093E+00_JPRB ,&
&  0.29681E+00_JPRB, 0.29797E+00_JPRB, 0.30871E+00_JPRB, 0.31936E+00_JPRB, 0.33926E+00_JPRB, 0.36548E+00_JPRB, 0.38453E+00_JPRB ,&
&  0.39731E+00_JPRB, 0.41133E+00_JPRB, 0.42614E+00_JPRB, 0.43481E+00_JPRB, 0.43554E+00_JPRB, 0.43018E+00_JPRB, 0.42620E+00_JPRB ,&
&  0.42111E+00_JPRB, 0.41150E+00_JPRB, 0.40114E+00_JPRB, 0.38804E+00_JPRB, 0.37401E+00_JPRB, 0.36241E+00_JPRB, 0.35315E+00_JPRB ,&
&  0.34451E+00_JPRB, 0.33609E+00_JPRB, 0.32833E+00_JPRB, 0.32035E+00_JPRB, 0.31164E+00_JPRB, 0.30246E+00_JPRB, 0.29276E+00_JPRB ,&
&  0.28328E+00_JPRB, 0.27520E+00_JPRB, 0.26889E+00_JPRB, 0.26573E+00_JPRB, 0.26414E+00_JPRB, 0.26282E+00_JPRB, 0.26194E+00_JPRB ,&
&  0.26231E+00_JPRB, 0.26357E+00_JPRB, 0.26543E+00_JPRB, 0.26743E+00_JPRB, 0.26925E+00_JPRB, 0.26981E+00_JPRB, 0.26811E+00_JPRB ,&
&  0.26620E+00_JPRB, 0.26580E+00_JPRB, 0.26607E+00_JPRB, 0.26404E+00_JPRB, 0.26174E+00_JPRB, 0.26236E+00_JPRB, 0.26462E+00_JPRB ,&
&  0.26875E+00_JPRB, 0.27248E+00_JPRB, 0.27557E+00_JPRB, 0.27806E+00_JPRB, 0.27989E+00_JPRB, 0.28306E+00_JPRB, 0.28696E+00_JPRB ,&
&  0.29191E+00_JPRB, 0.29820E+00_JPRB, 0.30706E+00_JPRB, 0.31823E+00_JPRB, 0.32848E+00_JPRB, 0.33691E+00_JPRB, 0.34197E+00_JPRB ,&
&  0.34500E+00_JPRB, 0.34719E+00_JPRB, 0.34942E+00_JPRB, 0.35267E+00_JPRB, 0.35574E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.31647E+00_JPRB, 0.31650E+00_JPRB, 0.31720E+00_JPRB, 0.31757E+00_JPRB, 0.31506E+00_JPRB, 0.31242E+00_JPRB, 0.30930E+00_JPRB ,&
&  0.30803E+00_JPRB, 0.30714E+00_JPRB, 0.30692E+00_JPRB, 0.30431E+00_JPRB, 0.30239E+00_JPRB, 0.30141E+00_JPRB, 0.29828E+00_JPRB ,&
&  0.29522E+00_JPRB, 0.28802E+00_JPRB, 0.28021E+00_JPRB, 0.27435E+00_JPRB, 0.27064E+00_JPRB, 0.26814E+00_JPRB, 0.26316E+00_JPRB ,&
&  0.25969E+00_JPRB, 0.25697E+00_JPRB, 0.25577E+00_JPRB, 0.25604E+00_JPRB, 0.25825E+00_JPRB, 0.26416E+00_JPRB, 0.27395E+00_JPRB ,&
&  0.28106E+00_JPRB, 0.28937E+00_JPRB, 0.28963E+00_JPRB, 0.30316E+00_JPRB, 0.31635E+00_JPRB, 0.33394E+00_JPRB, 0.34856E+00_JPRB ,&
&  0.35913E+00_JPRB, 0.36768E+00_JPRB, 0.37872E+00_JPRB, 0.38937E+00_JPRB, 0.39415E+00_JPRB, 0.39593E+00_JPRB, 0.39505E+00_JPRB ,&
&  0.39201E+00_JPRB, 0.38698E+00_JPRB, 0.38114E+00_JPRB, 0.37393E+00_JPRB, 0.36407E+00_JPRB, 0.35257E+00_JPRB, 0.34110E+00_JPRB ,&
&  0.33057E+00_JPRB, 0.32022E+00_JPRB, 0.30999E+00_JPRB, 0.29895E+00_JPRB, 0.28797E+00_JPRB, 0.27789E+00_JPRB, 0.26773E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25261E+00_JPRB, 0.24776E+00_JPRB, 0.24439E+00_JPRB, 0.24192E+00_JPRB, 0.23950E+00_JPRB, 0.23847E+00_JPRB ,&
&  0.23873E+00_JPRB, 0.24035E+00_JPRB, 0.24269E+00_JPRB, 0.24458E+00_JPRB, 0.24687E+00_JPRB, 0.24734E+00_JPRB, 0.24480E+00_JPRB ,&
&  0.24205E+00_JPRB, 0.24095E+00_JPRB, 0.23711E+00_JPRB, 0.23487E+00_JPRB, 0.23449E+00_JPRB, 0.23498E+00_JPRB, 0.23682E+00_JPRB ,&
&  0.24050E+00_JPRB, 0.24562E+00_JPRB, 0.25036E+00_JPRB, 0.25449E+00_JPRB, 0.25803E+00_JPRB, 0.26162E+00_JPRB, 0.26583E+00_JPRB ,&
&  0.27032E+00_JPRB, 0.27469E+00_JPRB, 0.28155E+00_JPRB, 0.28930E+00_JPRB, 0.29594E+00_JPRB, 0.30301E+00_JPRB, 0.30729E+00_JPRB ,&
&  0.30891E+00_JPRB, 0.30955E+00_JPRB, 0.31154E+00_JPRB, 0.31450E+00_JPRB, 0.31634E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.28367E+00_JPRB, 0.28517E+00_JPRB, 0.28708E+00_JPRB, 0.28741E+00_JPRB, 0.28688E+00_JPRB, 0.28687E+00_JPRB, 0.28588E+00_JPRB ,&
&  0.28372E+00_JPRB, 0.28262E+00_JPRB, 0.28295E+00_JPRB, 0.28173E+00_JPRB, 0.27917E+00_JPRB, 0.27603E+00_JPRB, 0.27258E+00_JPRB ,&
&  0.26977E+00_JPRB, 0.26681E+00_JPRB, 0.26175E+00_JPRB, 0.25503E+00_JPRB, 0.24922E+00_JPRB, 0.24549E+00_JPRB, 0.24358E+00_JPRB ,&
&  0.24281E+00_JPRB, 0.24290E+00_JPRB, 0.24264E+00_JPRB, 0.24335E+00_JPRB, 0.24551E+00_JPRB, 0.25036E+00_JPRB, 0.25751E+00_JPRB ,&
&  0.26475E+00_JPRB, 0.27134E+00_JPRB, 0.27805E+00_JPRB, 0.28535E+00_JPRB, 0.29403E+00_JPRB, 0.30415E+00_JPRB, 0.31317E+00_JPRB ,&
&  0.32095E+00_JPRB, 0.32372E+00_JPRB, 0.32512E+00_JPRB, 0.32952E+00_JPRB, 0.33757E+00_JPRB, 0.34317E+00_JPRB, 0.34669E+00_JPRB ,&
&  0.34846E+00_JPRB, 0.34720E+00_JPRB, 0.34483E+00_JPRB, 0.33988E+00_JPRB, 0.33269E+00_JPRB, 0.32339E+00_JPRB, 0.31341E+00_JPRB ,&
&  0.30355E+00_JPRB, 0.29339E+00_JPRB, 0.28241E+00_JPRB, 0.27194E+00_JPRB, 0.26300E+00_JPRB, 0.25506E+00_JPRB, 0.24708E+00_JPRB ,&
&  0.23972E+00_JPRB, 0.23385E+00_JPRB, 0.22920E+00_JPRB, 0.22609E+00_JPRB, 0.22361E+00_JPRB, 0.22169E+00_JPRB, 0.22127E+00_JPRB ,&
&  0.22167E+00_JPRB, 0.22308E+00_JPRB, 0.22384E+00_JPRB, 0.22533E+00_JPRB, 0.22775E+00_JPRB, 0.22878E+00_JPRB, 0.22769E+00_JPRB ,&
&  0.22384E+00_JPRB, 0.21995E+00_JPRB, 0.21633E+00_JPRB, 0.21574E+00_JPRB, 0.21575E+00_JPRB, 0.21698E+00_JPRB, 0.21875E+00_JPRB ,&
&  0.22229E+00_JPRB, 0.22740E+00_JPRB, 0.23197E+00_JPRB, 0.23527E+00_JPRB, 0.23904E+00_JPRB, 0.24235E+00_JPRB, 0.24567E+00_JPRB ,&
&  0.24940E+00_JPRB, 0.25344E+00_JPRB, 0.25883E+00_JPRB, 0.26426E+00_JPRB, 0.26918E+00_JPRB, 0.27305E+00_JPRB, 0.27586E+00_JPRB ,&
&  0.27765E+00_JPRB, 0.27807E+00_JPRB, 0.27831E+00_JPRB, 0.27979E+00_JPRB, 0.28202E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.25428E+00_JPRB, 0.25635E+00_JPRB, 0.25910E+00_JPRB, 0.26139E+00_JPRB, 0.26224E+00_JPRB, 0.26177E+00_JPRB, 0.26099E+00_JPRB ,&
&  0.26087E+00_JPRB, 0.26087E+00_JPRB, 0.25952E+00_JPRB, 0.25717E+00_JPRB, 0.25479E+00_JPRB, 0.25227E+00_JPRB, 0.25045E+00_JPRB ,&
&  0.24860E+00_JPRB, 0.24654E+00_JPRB, 0.24290E+00_JPRB, 0.23755E+00_JPRB, 0.23202E+00_JPRB, 0.22881E+00_JPRB, 0.22730E+00_JPRB ,&
&  0.22722E+00_JPRB, 0.22776E+00_JPRB, 0.23030E+00_JPRB, 0.23214E+00_JPRB, 0.23465E+00_JPRB, 0.23879E+00_JPRB, 0.24444E+00_JPRB ,&
&  0.24980E+00_JPRB, 0.25490E+00_JPRB, 0.26010E+00_JPRB, 0.26449E+00_JPRB, 0.26887E+00_JPRB, 0.27417E+00_JPRB, 0.27972E+00_JPRB ,&
&  0.28559E+00_JPRB, 0.28939E+00_JPRB, 0.28992E+00_JPRB, 0.29082E+00_JPRB, 0.29320E+00_JPRB, 0.29599E+00_JPRB, 0.29763E+00_JPRB ,&
&  0.29877E+00_JPRB, 0.29707E+00_JPRB, 0.29438E+00_JPRB, 0.29050E+00_JPRB, 0.28631E+00_JPRB, 0.28165E+00_JPRB, 0.27542E+00_JPRB ,&
&  0.26797E+00_JPRB, 0.25990E+00_JPRB, 0.25220E+00_JPRB, 0.24491E+00_JPRB, 0.23796E+00_JPRB, 0.23151E+00_JPRB, 0.22564E+00_JPRB ,&
&  0.21998E+00_JPRB, 0.21563E+00_JPRB, 0.21141E+00_JPRB, 0.20893E+00_JPRB, 0.20753E+00_JPRB, 0.20584E+00_JPRB, 0.20560E+00_JPRB ,&
&  0.20484E+00_JPRB, 0.20449E+00_JPRB, 0.20449E+00_JPRB, 0.20614E+00_JPRB, 0.20921E+00_JPRB, 0.21105E+00_JPRB, 0.20990E+00_JPRB ,&
&  0.20665E+00_JPRB, 0.20465E+00_JPRB, 0.20355E+00_JPRB, 0.20270E+00_JPRB, 0.20042E+00_JPRB, 0.20010E+00_JPRB, 0.20194E+00_JPRB ,&
&  0.20604E+00_JPRB, 0.21052E+00_JPRB, 0.21432E+00_JPRB, 0.21663E+00_JPRB, 0.21905E+00_JPRB, 0.22101E+00_JPRB, 0.22276E+00_JPRB ,&
&  0.22470E+00_JPRB, 0.22613E+00_JPRB, 0.22721E+00_JPRB, 0.22867E+00_JPRB, 0.23107E+00_JPRB, 0.23342E+00_JPRB, 0.23573E+00_JPRB ,&
&  0.23822E+00_JPRB, 0.24206E+00_JPRB, 0.24598E+00_JPRB, 0.24885E+00_JPRB, 0.25223E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.23364E+00_JPRB, 0.23623E+00_JPRB, 0.23881E+00_JPRB, 0.23903E+00_JPRB, 0.23739E+00_JPRB, 0.23659E+00_JPRB, 0.23673E+00_JPRB ,&
&  0.23707E+00_JPRB, 0.23716E+00_JPRB, 0.23600E+00_JPRB, 0.23595E+00_JPRB, 0.23603E+00_JPRB, 0.23517E+00_JPRB, 0.23456E+00_JPRB ,&
&  0.23339E+00_JPRB, 0.23051E+00_JPRB, 0.22642E+00_JPRB, 0.22138E+00_JPRB, 0.21762E+00_JPRB, 0.21565E+00_JPRB, 0.21537E+00_JPRB ,&
&  0.21609E+00_JPRB, 0.21664E+00_JPRB, 0.21916E+00_JPRB, 0.22140E+00_JPRB, 0.22422E+00_JPRB, 0.22795E+00_JPRB, 0.23184E+00_JPRB ,&
&  0.23476E+00_JPRB, 0.23780E+00_JPRB, 0.24079E+00_JPRB, 0.24356E+00_JPRB, 0.24715E+00_JPRB, 0.25244E+00_JPRB, 0.25681E+00_JPRB ,&
&  0.26123E+00_JPRB, 0.26315E+00_JPRB, 0.26047E+00_JPRB, 0.25757E+00_JPRB, 0.25740E+00_JPRB, 0.25886E+00_JPRB, 0.26030E+00_JPRB ,&
&  0.26143E+00_JPRB, 0.26265E+00_JPRB, 0.26365E+00_JPRB, 0.26311E+00_JPRB, 0.26033E+00_JPRB, 0.25559E+00_JPRB, 0.24939E+00_JPRB ,&
&  0.24267E+00_JPRB, 0.23599E+00_JPRB, 0.22965E+00_JPRB, 0.22338E+00_JPRB, 0.21805E+00_JPRB, 0.21282E+00_JPRB, 0.20799E+00_JPRB ,&
&  0.20343E+00_JPRB, 0.19908E+00_JPRB, 0.19579E+00_JPRB, 0.19378E+00_JPRB, 0.19332E+00_JPRB, 0.19229E+00_JPRB, 0.19230E+00_JPRB ,&
&  0.19193E+00_JPRB, 0.19188E+00_JPRB, 0.19223E+00_JPRB, 0.19323E+00_JPRB, 0.19465E+00_JPRB, 0.19501E+00_JPRB, 0.19408E+00_JPRB ,&
&  0.19354E+00_JPRB, 0.19467E+00_JPRB, 0.19445E+00_JPRB, 0.19217E+00_JPRB, 0.18932E+00_JPRB, 0.18859E+00_JPRB, 0.18986E+00_JPRB ,&
&  0.19240E+00_JPRB, 0.19502E+00_JPRB, 0.19731E+00_JPRB, 0.19937E+00_JPRB, 0.20109E+00_JPRB, 0.20322E+00_JPRB, 0.20516E+00_JPRB ,&
&  0.20679E+00_JPRB, 0.20758E+00_JPRB, 0.20905E+00_JPRB, 0.21061E+00_JPRB, 0.21226E+00_JPRB, 0.21343E+00_JPRB, 0.21331E+00_JPRB ,&
&  0.21289E+00_JPRB, 0.21439E+00_JPRB, 0.21912E+00_JPRB, 0.22628E+00_JPRB, 0.23079E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.21652E+00_JPRB, 0.21880E+00_JPRB, 0.22007E+00_JPRB, 0.22000E+00_JPRB, 0.21954E+00_JPRB, 0.22026E+00_JPRB, 0.22076E+00_JPRB ,&
&  0.22130E+00_JPRB, 0.22143E+00_JPRB, 0.22027E+00_JPRB, 0.21927E+00_JPRB, 0.22090E+00_JPRB, 0.22044E+00_JPRB, 0.21996E+00_JPRB ,&
&  0.21845E+00_JPRB, 0.21663E+00_JPRB, 0.21385E+00_JPRB, 0.21065E+00_JPRB, 0.20672E+00_JPRB, 0.20430E+00_JPRB, 0.20460E+00_JPRB ,&
&  0.20612E+00_JPRB, 0.20672E+00_JPRB, 0.20758E+00_JPRB, 0.20874E+00_JPRB, 0.21109E+00_JPRB, 0.21410E+00_JPRB, 0.21638E+00_JPRB ,&
&  0.21822E+00_JPRB, 0.22138E+00_JPRB, 0.22339E+00_JPRB, 0.22223E+00_JPRB, 0.22650E+00_JPRB, 0.23039E+00_JPRB, 0.23374E+00_JPRB ,&
&  0.23655E+00_JPRB, 0.23818E+00_JPRB, 0.23816E+00_JPRB, 0.23717E+00_JPRB, 0.23638E+00_JPRB, 0.23728E+00_JPRB, 0.23767E+00_JPRB ,&
&  0.23808E+00_JPRB, 0.24015E+00_JPRB, 0.24259E+00_JPRB, 0.24328E+00_JPRB, 0.24150E+00_JPRB, 0.23724E+00_JPRB, 0.23072E+00_JPRB ,&
&  0.22244E+00_JPRB, 0.21379E+00_JPRB, 0.20680E+00_JPRB, 0.20125E+00_JPRB, 0.19698E+00_JPRB, 0.19346E+00_JPRB, 0.19060E+00_JPRB ,&
&  0.18745E+00_JPRB, 0.18524E+00_JPRB, 0.18520E+00_JPRB, 0.18583E+00_JPRB, 0.18605E+00_JPRB, 0.18472E+00_JPRB, 0.18378E+00_JPRB ,&
&  0.18291E+00_JPRB, 0.18336E+00_JPRB, 0.18423E+00_JPRB, 0.18458E+00_JPRB, 0.18470E+00_JPRB, 0.18440E+00_JPRB, 0.18414E+00_JPRB ,&
&  0.18492E+00_JPRB, 0.18513E+00_JPRB, 0.18421E+00_JPRB, 0.18248E+00_JPRB, 0.18101E+00_JPRB, 0.18074E+00_JPRB, 0.18096E+00_JPRB ,&
&  0.18152E+00_JPRB, 0.18271E+00_JPRB, 0.18445E+00_JPRB, 0.18626E+00_JPRB, 0.18769E+00_JPRB, 0.18833E+00_JPRB, 0.18853E+00_JPRB ,&
&  0.18870E+00_JPRB, 0.18854E+00_JPRB, 0.18943E+00_JPRB, 0.19229E+00_JPRB, 0.19536E+00_JPRB, 0.19786E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19897E+00_JPRB, 0.19971E+00_JPRB, 0.20284E+00_JPRB, 0.20866E+00_JPRB, 0.21305E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.20390E+00_JPRB, 0.20601E+00_JPRB, 0.20675E+00_JPRB, 0.20640E+00_JPRB, 0.20595E+00_JPRB, 0.20686E+00_JPRB, 0.20814E+00_JPRB ,&
&  0.20876E+00_JPRB, 0.20880E+00_JPRB, 0.20789E+00_JPRB, 0.20613E+00_JPRB, 0.20483E+00_JPRB, 0.20507E+00_JPRB, 0.20354E+00_JPRB ,&
&  0.20214E+00_JPRB, 0.20240E+00_JPRB, 0.20120E+00_JPRB, 0.19979E+00_JPRB, 0.19751E+00_JPRB, 0.19616E+00_JPRB, 0.19674E+00_JPRB ,&
&  0.19884E+00_JPRB, 0.19951E+00_JPRB, 0.20025E+00_JPRB, 0.20124E+00_JPRB, 0.20308E+00_JPRB, 0.20525E+00_JPRB, 0.20674E+00_JPRB ,&
&  0.20786E+00_JPRB, 0.20722E+00_JPRB, 0.20557E+00_JPRB, 0.20835E+00_JPRB, 0.21137E+00_JPRB, 0.21522E+00_JPRB, 0.21776E+00_JPRB ,&
&  0.21996E+00_JPRB, 0.22165E+00_JPRB, 0.22246E+00_JPRB, 0.22242E+00_JPRB, 0.22277E+00_JPRB, 0.22445E+00_JPRB, 0.22562E+00_JPRB ,&
&  0.22599E+00_JPRB, 0.22551E+00_JPRB, 0.22400E+00_JPRB, 0.22250E+00_JPRB, 0.21906E+00_JPRB, 0.21399E+00_JPRB, 0.20657E+00_JPRB ,&
&  0.19891E+00_JPRB, 0.19281E+00_JPRB, 0.18821E+00_JPRB, 0.18447E+00_JPRB, 0.18185E+00_JPRB, 0.18151E+00_JPRB, 0.18147E+00_JPRB ,&
&  0.18107E+00_JPRB, 0.18069E+00_JPRB, 0.18206E+00_JPRB, 0.18288E+00_JPRB, 0.17926E+00_JPRB, 0.17399E+00_JPRB, 0.16961E+00_JPRB ,&
&  0.16862E+00_JPRB, 0.16988E+00_JPRB, 0.17187E+00_JPRB, 0.17372E+00_JPRB, 0.17682E+00_JPRB, 0.17756E+00_JPRB, 0.17664E+00_JPRB ,&
&  0.17158E+00_JPRB, 0.16611E+00_JPRB, 0.16296E+00_JPRB, 0.16061E+00_JPRB, 0.15915E+00_JPRB, 0.15903E+00_JPRB, 0.15970E+00_JPRB ,&
&  0.16175E+00_JPRB, 0.16365E+00_JPRB, 0.16600E+00_JPRB, 0.16721E+00_JPRB, 0.16698E+00_JPRB, 0.16658E+00_JPRB, 0.16585E+00_JPRB ,&
&  0.16700E+00_JPRB, 0.17110E+00_JPRB, 0.17741E+00_JPRB, 0.18084E+00_JPRB, 0.18466E+00_JPRB, 0.18726E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18951E+00_JPRB, 0.19008E+00_JPRB, 0.19071E+00_JPRB, 0.19490E+00_JPRB, 0.20047E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.19324E+00_JPRB, 0.19503E+00_JPRB, 0.19564E+00_JPRB, 0.19565E+00_JPRB, 0.19547E+00_JPRB, 0.19592E+00_JPRB, 0.19641E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19564E+00_JPRB, 0.19445E+00_JPRB, 0.19245E+00_JPRB, 0.19004E+00_JPRB, 0.18862E+00_JPRB, 0.18962E+00_JPRB ,&
&  0.19074E+00_JPRB, 0.19097E+00_JPRB, 0.19130E+00_JPRB, 0.19128E+00_JPRB, 0.19086E+00_JPRB, 0.19107E+00_JPRB, 0.19223E+00_JPRB ,&
&  0.19359E+00_JPRB, 0.19448E+00_JPRB, 0.19582E+00_JPRB, 0.19696E+00_JPRB, 0.19896E+00_JPRB, 0.20077E+00_JPRB, 0.20150E+00_JPRB ,&
&  0.19916E+00_JPRB, 0.19143E+00_JPRB, 0.18600E+00_JPRB, 0.18843E+00_JPRB, 0.19670E+00_JPRB, 0.20158E+00_JPRB, 0.20453E+00_JPRB ,&
&  0.20655E+00_JPRB, 0.20762E+00_JPRB, 0.20788E+00_JPRB, 0.20915E+00_JPRB, 0.21105E+00_JPRB, 0.21286E+00_JPRB, 0.21249E+00_JPRB ,&
&  0.21052E+00_JPRB, 0.20879E+00_JPRB, 0.20628E+00_JPRB, 0.20198E+00_JPRB, 0.19620E+00_JPRB, 0.19054E+00_JPRB, 0.18574E+00_JPRB ,&
&  0.18150E+00_JPRB, 0.17740E+00_JPRB, 0.17391E+00_JPRB, 0.17165E+00_JPRB, 0.17096E+00_JPRB, 0.17216E+00_JPRB, 0.17196E+00_JPRB ,&
&  0.17179E+00_JPRB, 0.17005E+00_JPRB, 0.16789E+00_JPRB, 0.16538E+00_JPRB, 0.16321E+00_JPRB, 0.15936E+00_JPRB, 0.15466E+00_JPRB ,&
&  0.15223E+00_JPRB, 0.15444E+00_JPRB, 0.16077E+00_JPRB, 0.16606E+00_JPRB, 0.16690E+00_JPRB, 0.15966E+00_JPRB, 0.15558E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.14765E+00_JPRB, 0.14518E+00_JPRB, 0.14461E+00_JPRB, 0.14478E+00_JPRB, 0.14503E+00_JPRB, 0.14598E+00_JPRB ,&
&  0.14748E+00_JPRB, 0.14880E+00_JPRB, 0.14956E+00_JPRB, 0.15037E+00_JPRB, 0.15072E+00_JPRB, 0.15079E+00_JPRB, 0.15091E+00_JPRB ,&
&  0.15238E+00_JPRB, 0.15547E+00_JPRB, 0.15967E+00_JPRB, 0.16442E+00_JPRB, 0.16811E+00_JPRB, 0.17149E+00_JPRB, 0.17467E+00_JPRB ,&
&  0.17800E+00_JPRB, 0.18025E+00_JPRB, 0.18316E+00_JPRB, 0.18702E+00_JPRB, 0.19034E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.18108E+00_JPRB, 0.18472E+00_JPRB, 0.18629E+00_JPRB, 0.18633E+00_JPRB, 0.18530E+00_JPRB, 0.18442E+00_JPRB, 0.18423E+00_JPRB ,&
&  0.18410E+00_JPRB, 0.18302E+00_JPRB, 0.18220E+00_JPRB, 0.18154E+00_JPRB, 0.18083E+00_JPRB, 0.18007E+00_JPRB, 0.18031E+00_JPRB ,&
&  0.18151E+00_JPRB, 0.18309E+00_JPRB, 0.18426E+00_JPRB, 0.18476E+00_JPRB, 0.18434E+00_JPRB, 0.18468E+00_JPRB, 0.18554E+00_JPRB ,&
&  0.18551E+00_JPRB, 0.18578E+00_JPRB, 0.18898E+00_JPRB, 0.18941E+00_JPRB, 0.19033E+00_JPRB, 0.19043E+00_JPRB, 0.19047E+00_JPRB ,&
&  0.18590E+00_JPRB, 0.18075E+00_JPRB, 0.17768E+00_JPRB, 0.17971E+00_JPRB, 0.18784E+00_JPRB, 0.18938E+00_JPRB, 0.19260E+00_JPRB ,&
&  0.19468E+00_JPRB, 0.19577E+00_JPRB, 0.19707E+00_JPRB, 0.19826E+00_JPRB, 0.19969E+00_JPRB, 0.19883E+00_JPRB, 0.19668E+00_JPRB ,&
&  0.19402E+00_JPRB, 0.19302E+00_JPRB, 0.18981E+00_JPRB, 0.17609E+00_JPRB, 0.16760E+00_JPRB, 0.16279E+00_JPRB, 0.15779E+00_JPRB ,&
&  0.15325E+00_JPRB, 0.15067E+00_JPRB, 0.14996E+00_JPRB, 0.14874E+00_JPRB, 0.14958E+00_JPRB, 0.15146E+00_JPRB, 0.15230E+00_JPRB ,&
&  0.15203E+00_JPRB, 0.15244E+00_JPRB, 0.15216E+00_JPRB, 0.15126E+00_JPRB, 0.14967E+00_JPRB, 0.14868E+00_JPRB, 0.14813E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14841E+00_JPRB, 0.15223E+00_JPRB, 0.15360E+00_JPRB, 0.14914E+00_JPRB, 0.14753E+00_JPRB, 0.14555E+00_JPRB ,&
&  0.14075E+00_JPRB, 0.13803E+00_JPRB, 0.13654E+00_JPRB, 0.13596E+00_JPRB, 0.13688E+00_JPRB, 0.13821E+00_JPRB, 0.13893E+00_JPRB ,&
&  0.13922E+00_JPRB, 0.14019E+00_JPRB, 0.14066E+00_JPRB, 0.14118E+00_JPRB, 0.14154E+00_JPRB, 0.14232E+00_JPRB, 0.14347E+00_JPRB ,&
&  0.14492E+00_JPRB, 0.14752E+00_JPRB, 0.15010E+00_JPRB, 0.15301E+00_JPRB, 0.15475E+00_JPRB, 0.15613E+00_JPRB, 0.15925E+00_JPRB ,&
&  0.16127E+00_JPRB, 0.16802E+00_JPRB, 0.17791E+00_JPRB, 0.18144E+00_JPRB, 0.17684E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.17428E+00_JPRB, 0.17618E+00_JPRB, 0.17789E+00_JPRB, 0.17700E+00_JPRB, 0.17717E+00_JPRB, 0.17717E+00_JPRB, 0.17725E+00_JPRB ,&
&  0.17720E+00_JPRB, 0.17664E+00_JPRB, 0.17688E+00_JPRB, 0.17525E+00_JPRB, 0.17431E+00_JPRB, 0.17080E+00_JPRB, 0.16954E+00_JPRB ,&
&  0.16765E+00_JPRB, 0.16720E+00_JPRB, 0.16744E+00_JPRB, 0.16857E+00_JPRB, 0.16900E+00_JPRB, 0.17025E+00_JPRB, 0.16958E+00_JPRB ,&
&  0.16977E+00_JPRB, 0.16970E+00_JPRB, 0.16846E+00_JPRB, 0.16810E+00_JPRB, 0.16853E+00_JPRB, 0.16902E+00_JPRB, 0.16933E+00_JPRB ,&
&  0.17111E+00_JPRB, 0.17329E+00_JPRB, 0.17539E+00_JPRB, 0.17675E+00_JPRB, 0.18178E+00_JPRB, 0.18323E+00_JPRB, 0.18512E+00_JPRB ,&
&  0.18624E+00_JPRB, 0.18768E+00_JPRB, 0.18868E+00_JPRB, 0.18809E+00_JPRB, 0.18692E+00_JPRB, 0.18347E+00_JPRB, 0.18216E+00_JPRB ,&
&  0.17215E+00_JPRB, 0.16537E+00_JPRB, 0.15866E+00_JPRB, 0.15273E+00_JPRB, 0.14843E+00_JPRB, 0.14546E+00_JPRB, 0.14304E+00_JPRB ,&
&  0.14104E+00_JPRB, 0.13913E+00_JPRB, 0.13705E+00_JPRB, 0.13707E+00_JPRB, 0.13780E+00_JPRB, 0.13892E+00_JPRB, 0.14005E+00_JPRB ,&
&  0.14134E+00_JPRB, 0.14209E+00_JPRB, 0.14245E+00_JPRB, 0.14319E+00_JPRB, 0.14341E+00_JPRB, 0.14379E+00_JPRB, 0.14404E+00_JPRB ,&
&  0.14529E+00_JPRB, 0.14573E+00_JPRB, 0.14579E+00_JPRB, 0.14708E+00_JPRB, 0.14546E+00_JPRB, 0.14290E+00_JPRB, 0.14007E+00_JPRB ,&
&  0.13730E+00_JPRB, 0.13556E+00_JPRB, 0.13412E+00_JPRB, 0.13356E+00_JPRB, 0.13392E+00_JPRB, 0.13461E+00_JPRB, 0.13553E+00_JPRB ,&
&  0.13578E+00_JPRB, 0.13636E+00_JPRB, 0.13681E+00_JPRB, 0.13763E+00_JPRB, 0.13851E+00_JPRB, 0.13950E+00_JPRB, 0.14058E+00_JPRB ,&
&  0.14222E+00_JPRB, 0.14410E+00_JPRB, 0.14601E+00_JPRB, 0.14795E+00_JPRB, 0.15106E+00_JPRB, 0.15485E+00_JPRB, 0.16086E+00_JPRB ,&
&  0.16641E+00_JPRB, 0.17409E+00_JPRB, 0.17306E+00_JPRB, 0.17524E+00_JPRB, 0.17542E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.16997E+00_JPRB, 0.17077E+00_JPRB, 0.16904E+00_JPRB, 0.16948E+00_JPRB, 0.17026E+00_JPRB, 0.17069E+00_JPRB, 0.17116E+00_JPRB ,&
&  0.16878E+00_JPRB, 0.16788E+00_JPRB, 0.16559E+00_JPRB, 0.16349E+00_JPRB, 0.16100E+00_JPRB, 0.15838E+00_JPRB, 0.15635E+00_JPRB ,&
&  0.15452E+00_JPRB, 0.15378E+00_JPRB, 0.15293E+00_JPRB, 0.15361E+00_JPRB, 0.15363E+00_JPRB, 0.15519E+00_JPRB, 0.15647E+00_JPRB ,&
&  0.15721E+00_JPRB, 0.15817E+00_JPRB, 0.15697E+00_JPRB, 0.15685E+00_JPRB, 0.15857E+00_JPRB, 0.16021E+00_JPRB, 0.16282E+00_JPRB ,&
&  0.16560E+00_JPRB, 0.16860E+00_JPRB, 0.17346E+00_JPRB, 0.17524E+00_JPRB, 0.17737E+00_JPRB, 0.17874E+00_JPRB, 0.17877E+00_JPRB ,&
&  0.17851E+00_JPRB, 0.17859E+00_JPRB, 0.17947E+00_JPRB, 0.17923E+00_JPRB, 0.17256E+00_JPRB, 0.16739E+00_JPRB, 0.16196E+00_JPRB ,&
&  0.15687E+00_JPRB, 0.15206E+00_JPRB, 0.14779E+00_JPRB, 0.14492E+00_JPRB, 0.14200E+00_JPRB, 0.13988E+00_JPRB, 0.13823E+00_JPRB ,&
&  0.13654E+00_JPRB, 0.13553E+00_JPRB, 0.13426E+00_JPRB, 0.13388E+00_JPRB, 0.13414E+00_JPRB, 0.13486E+00_JPRB, 0.13585E+00_JPRB ,&
&  0.13702E+00_JPRB, 0.13784E+00_JPRB, 0.13888E+00_JPRB, 0.13988E+00_JPRB, 0.14062E+00_JPRB, 0.14110E+00_JPRB, 0.14091E+00_JPRB ,&
&  0.14113E+00_JPRB, 0.14138E+00_JPRB, 0.14097E+00_JPRB, 0.14089E+00_JPRB, 0.13983E+00_JPRB, 0.13859E+00_JPRB, 0.13730E+00_JPRB ,&
&  0.13562E+00_JPRB, 0.13431E+00_JPRB, 0.13383E+00_JPRB, 0.13355E+00_JPRB, 0.13420E+00_JPRB, 0.13518E+00_JPRB, 0.13584E+00_JPRB ,&
&  0.13620E+00_JPRB, 0.13617E+00_JPRB, 0.13624E+00_JPRB, 0.13731E+00_JPRB, 0.13843E+00_JPRB, 0.13983E+00_JPRB, 0.14129E+00_JPRB ,&
&  0.14179E+00_JPRB, 0.14300E+00_JPRB, 0.14408E+00_JPRB, 0.14539E+00_JPRB, 0.14976E+00_JPRB, 0.15330E+00_JPRB, 0.16005E+00_JPRB ,&
&  0.16364E+00_JPRB, 0.16570E+00_JPRB, 0.16573E+00_JPRB, 0.16951E+00_JPRB, 0.16960E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.16811E+00_JPRB, 0.17053E+00_JPRB, 0.16933E+00_JPRB, 0.16924E+00_JPRB, 0.16876E+00_JPRB, 0.16869E+00_JPRB, 0.16776E+00_JPRB ,&
&  0.16756E+00_JPRB, 0.16552E+00_JPRB, 0.16504E+00_JPRB, 0.16335E+00_JPRB, 0.16157E+00_JPRB, 0.15918E+00_JPRB, 0.15681E+00_JPRB ,&
&  0.15442E+00_JPRB, 0.15230E+00_JPRB, 0.15084E+00_JPRB, 0.15011E+00_JPRB, 0.14891E+00_JPRB, 0.14877E+00_JPRB, 0.15040E+00_JPRB ,&
&  0.15076E+00_JPRB, 0.15177E+00_JPRB, 0.15194E+00_JPRB, 0.15319E+00_JPRB, 0.15481E+00_JPRB, 0.15567E+00_JPRB, 0.15801E+00_JPRB ,&
&  0.15956E+00_JPRB, 0.16103E+00_JPRB, 0.16639E+00_JPRB, 0.16944E+00_JPRB, 0.17121E+00_JPRB, 0.17377E+00_JPRB, 0.17369E+00_JPRB ,&
&  0.17243E+00_JPRB, 0.16890E+00_JPRB, 0.17211E+00_JPRB, 0.17008E+00_JPRB, 0.16381E+00_JPRB, 0.16087E+00_JPRB, 0.15570E+00_JPRB ,&
&  0.15251E+00_JPRB, 0.14887E+00_JPRB, 0.14612E+00_JPRB, 0.14401E+00_JPRB, 0.14126E+00_JPRB, 0.13946E+00_JPRB, 0.13746E+00_JPRB ,&
&  0.13557E+00_JPRB, 0.13472E+00_JPRB, 0.13340E+00_JPRB, 0.13319E+00_JPRB, 0.13326E+00_JPRB, 0.13361E+00_JPRB, 0.13421E+00_JPRB ,&
&  0.13502E+00_JPRB, 0.13579E+00_JPRB, 0.13651E+00_JPRB, 0.13714E+00_JPRB, 0.13768E+00_JPRB, 0.13800E+00_JPRB, 0.13790E+00_JPRB ,&
&  0.13800E+00_JPRB, 0.13791E+00_JPRB, 0.13748E+00_JPRB, 0.13723E+00_JPRB, 0.13637E+00_JPRB, 0.13596E+00_JPRB, 0.13555E+00_JPRB ,&
&  0.13536E+00_JPRB, 0.13538E+00_JPRB, 0.13531E+00_JPRB, 0.13551E+00_JPRB, 0.13579E+00_JPRB, 0.13645E+00_JPRB, 0.13701E+00_JPRB ,&
&  0.13746E+00_JPRB, 0.13829E+00_JPRB, 0.13879E+00_JPRB, 0.13949E+00_JPRB, 0.14045E+00_JPRB, 0.14091E+00_JPRB, 0.14202E+00_JPRB ,&
&  0.14299E+00_JPRB, 0.14412E+00_JPRB, 0.14618E+00_JPRB, 0.14827E+00_JPRB, 0.15125E+00_JPRB, 0.15507E+00_JPRB, 0.15679E+00_JPRB ,&
&  0.15841E+00_JPRB, 0.16002E+00_JPRB, 0.15907E+00_JPRB, 0.16369E+00_JPRB, 0.16582E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.16213E+00_JPRB, 0.16393E+00_JPRB, 0.16506E+00_JPRB, 0.16586E+00_JPRB, 0.16699E+00_JPRB, 0.16748E+00_JPRB, 0.16793E+00_JPRB ,&
&  0.16613E+00_JPRB, 0.16549E+00_JPRB, 0.16443E+00_JPRB, 0.16312E+00_JPRB, 0.16217E+00_JPRB, 0.15999E+00_JPRB, 0.15778E+00_JPRB ,&
&  0.15532E+00_JPRB, 0.15310E+00_JPRB, 0.15169E+00_JPRB, 0.15031E+00_JPRB, 0.14894E+00_JPRB, 0.14780E+00_JPRB, 0.14935E+00_JPRB ,&
&  0.15023E+00_JPRB, 0.15082E+00_JPRB, 0.15200E+00_JPRB, 0.15203E+00_JPRB, 0.15333E+00_JPRB, 0.15535E+00_JPRB, 0.15804E+00_JPRB ,&
&  0.15992E+00_JPRB, 0.16271E+00_JPRB, 0.16437E+00_JPRB, 0.16521E+00_JPRB, 0.16633E+00_JPRB, 0.16708E+00_JPRB, 0.16677E+00_JPRB ,&
&  0.16553E+00_JPRB, 0.16388E+00_JPRB, 0.16197E+00_JPRB, 0.15953E+00_JPRB, 0.15679E+00_JPRB, 0.15411E+00_JPRB, 0.15145E+00_JPRB ,&
&  0.14946E+00_JPRB, 0.14767E+00_JPRB, 0.14580E+00_JPRB, 0.14411E+00_JPRB, 0.14250E+00_JPRB, 0.14105E+00_JPRB, 0.13960E+00_JPRB ,&
&  0.13844E+00_JPRB, 0.13753E+00_JPRB, 0.13640E+00_JPRB, 0.13577E+00_JPRB, 0.13515E+00_JPRB, 0.13486E+00_JPRB, 0.13471E+00_JPRB ,&
&  0.13503E+00_JPRB, 0.13547E+00_JPRB, 0.13592E+00_JPRB, 0.13617E+00_JPRB, 0.13641E+00_JPRB, 0.13651E+00_JPRB, 0.13672E+00_JPRB ,&
&  0.13692E+00_JPRB, 0.13715E+00_JPRB, 0.13680E+00_JPRB, 0.13648E+00_JPRB, 0.13604E+00_JPRB, 0.13555E+00_JPRB, 0.13540E+00_JPRB ,&
&  0.13567E+00_JPRB, 0.13590E+00_JPRB, 0.13579E+00_JPRB, 0.13606E+00_JPRB, 0.13652E+00_JPRB, 0.13701E+00_JPRB, 0.13819E+00_JPRB ,&
&  0.13885E+00_JPRB, 0.13949E+00_JPRB, 0.14009E+00_JPRB, 0.14056E+00_JPRB, 0.14127E+00_JPRB, 0.14185E+00_JPRB, 0.14257E+00_JPRB ,&
&  0.14343E+00_JPRB, 0.14429E+00_JPRB, 0.14547E+00_JPRB, 0.14695E+00_JPRB, 0.15043E+00_JPRB, 0.15284E+00_JPRB, 0.15499E+00_JPRB ,&
&  0.15615E+00_JPRB, 0.15344E+00_JPRB, 0.16062E+00_JPRB, 0.16001E+00_JPRB, 0.15990E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.15288E+00_JPRB, 0.15469E+00_JPRB, 0.15615E+00_JPRB, 0.15809E+00_JPRB, 0.15457E+00_JPRB, 0.16156E+00_JPRB, 0.15906E+00_JPRB ,&
&  0.15645E+00_JPRB, 0.15171E+00_JPRB, 0.15170E+00_JPRB, 0.15082E+00_JPRB, 0.14963E+00_JPRB, 0.14894E+00_JPRB, 0.14868E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14833E+00_JPRB, 0.14864E+00_JPRB, 0.14887E+00_JPRB, 0.14904E+00_JPRB, 0.14916E+00_JPRB, 0.14939E+00_JPRB ,&
&  0.14983E+00_JPRB, 0.15065E+00_JPRB, 0.15152E+00_JPRB, 0.14996E+00_JPRB, 0.14983E+00_JPRB, 0.15015E+00_JPRB, 0.15078E+00_JPRB ,&
&  0.15132E+00_JPRB, 0.15238E+00_JPRB, 0.15316E+00_JPRB, 0.15369E+00_JPRB, 0.15511E+00_JPRB, 0.15513E+00_JPRB, 0.15495E+00_JPRB ,&
&  0.15455E+00_JPRB, 0.15379E+00_JPRB, 0.15268E+00_JPRB, 0.15166E+00_JPRB, 0.15060E+00_JPRB, 0.14923E+00_JPRB, 0.14791E+00_JPRB ,&
&  0.14682E+00_JPRB, 0.14593E+00_JPRB, 0.14480E+00_JPRB, 0.14370E+00_JPRB, 0.14269E+00_JPRB, 0.14172E+00_JPRB, 0.14089E+00_JPRB ,&
&  0.14027E+00_JPRB, 0.13976E+00_JPRB, 0.13924E+00_JPRB, 0.13882E+00_JPRB, 0.13839E+00_JPRB, 0.13799E+00_JPRB, 0.13763E+00_JPRB ,&
&  0.13776E+00_JPRB, 0.13788E+00_JPRB, 0.13800E+00_JPRB, 0.13810E+00_JPRB, 0.13845E+00_JPRB, 0.13852E+00_JPRB, 0.13860E+00_JPRB ,&
&  0.13864E+00_JPRB, 0.13855E+00_JPRB, 0.13846E+00_JPRB, 0.13828E+00_JPRB, 0.13806E+00_JPRB, 0.13779E+00_JPRB, 0.13767E+00_JPRB ,&
&  0.13767E+00_JPRB, 0.13777E+00_JPRB, 0.13807E+00_JPRB, 0.13827E+00_JPRB, 0.13848E+00_JPRB, 0.13865E+00_JPRB, 0.13905E+00_JPRB ,&
&  0.13948E+00_JPRB, 0.13991E+00_JPRB, 0.14032E+00_JPRB, 0.14025E+00_JPRB, 0.14048E+00_JPRB, 0.14070E+00_JPRB, 0.14091E+00_JPRB ,&
&  0.14104E+00_JPRB, 0.14148E+00_JPRB, 0.14216E+00_JPRB, 0.14308E+00_JPRB, 0.14473E+00_JPRB, 0.14617E+00_JPRB, 0.14750E+00_JPRB ,&
&  0.14893E+00_JPRB, 0.15177E+00_JPRB, 0.15296E+00_JPRB, 0.15341E+00_JPRB, 0.15351E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.14393E+00_JPRB, 0.14465E+00_JPRB, 0.14516E+00_JPRB, 0.14558E+00_JPRB, 0.14543E+00_JPRB, 0.14550E+00_JPRB, 0.14557E+00_JPRB ,&
&  0.14557E+00_JPRB, 0.14556E+00_JPRB, 0.14580E+00_JPRB, 0.14603E+00_JPRB, 0.14627E+00_JPRB, 0.14653E+00_JPRB, 0.14683E+00_JPRB ,&
&  0.14695E+00_JPRB, 0.14721E+00_JPRB, 0.14618E+00_JPRB, 0.14534E+00_JPRB, 0.14482E+00_JPRB, 0.14450E+00_JPRB, 0.14434E+00_JPRB ,&
&  0.14420E+00_JPRB, 0.14411E+00_JPRB, 0.14398E+00_JPRB, 0.14472E+00_JPRB, 0.14520E+00_JPRB, 0.14564E+00_JPRB, 0.14598E+00_JPRB ,&
&  0.14612E+00_JPRB, 0.14632E+00_JPRB, 0.14649E+00_JPRB, 0.14666E+00_JPRB, 0.14687E+00_JPRB, 0.14683E+00_JPRB, 0.14670E+00_JPRB ,&
&  0.14651E+00_JPRB, 0.14621E+00_JPRB, 0.14586E+00_JPRB, 0.14553E+00_JPRB, 0.14526E+00_JPRB, 0.14495E+00_JPRB, 0.14462E+00_JPRB ,&
&  0.14429E+00_JPRB, 0.14391E+00_JPRB, 0.14355E+00_JPRB, 0.14321E+00_JPRB, 0.14288E+00_JPRB, 0.14260E+00_JPRB, 0.14229E+00_JPRB ,&
&  0.14208E+00_JPRB, 0.14188E+00_JPRB, 0.14169E+00_JPRB, 0.14155E+00_JPRB, 0.14135E+00_JPRB, 0.14126E+00_JPRB, 0.14108E+00_JPRB ,&
&  0.14102E+00_JPRB, 0.14095E+00_JPRB, 0.14090E+00_JPRB, 0.14083E+00_JPRB, 0.14083E+00_JPRB, 0.14081E+00_JPRB, 0.14080E+00_JPRB ,&
&  0.14077E+00_JPRB, 0.14088E+00_JPRB, 0.14077E+00_JPRB, 0.14062E+00_JPRB, 0.14046E+00_JPRB, 0.14030E+00_JPRB, 0.14015E+00_JPRB ,&
&  0.14002E+00_JPRB, 0.13988E+00_JPRB, 0.13979E+00_JPRB, 0.13980E+00_JPRB, 0.13982E+00_JPRB, 0.13987E+00_JPRB, 0.13993E+00_JPRB ,&
&  0.14000E+00_JPRB, 0.14010E+00_JPRB, 0.14021E+00_JPRB, 0.14018E+00_JPRB, 0.14021E+00_JPRB, 0.14023E+00_JPRB, 0.14026E+00_JPRB ,&
&  0.14028E+00_JPRB, 0.14033E+00_JPRB, 0.14041E+00_JPRB, 0.14052E+00_JPRB, 0.14074E+00_JPRB, 0.14085E+00_JPRB, 0.14100E+00_JPRB ,&
&  0.14120E+00_JPRB, 0.14155E+00_JPRB, 0.14187E+00_JPRB, 0.14221E+00_JPRB, 0.14262E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB, 0.14173E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB ,&
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB ,&
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB ,&
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB ,&
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB ,&
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB ,&
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB ,&
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB ,&
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB ,&
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB ,&
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB ,&
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB ,&
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB ,&
&  0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB, 0.39606E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.39048E+00_JPRB, 0.38981E+00_JPRB, 0.38917E+00_JPRB, 0.38855E+00_JPRB, 0.38778E+00_JPRB, 0.38724E+00_JPRB, 0.38679E+00_JPRB ,&
&  0.38640E+00_JPRB, 0.38602E+00_JPRB, 0.38562E+00_JPRB, 0.38534E+00_JPRB, 0.38519E+00_JPRB, 0.38495E+00_JPRB, 0.38469E+00_JPRB ,&
&  0.38444E+00_JPRB, 0.38430E+00_JPRB, 0.38429E+00_JPRB, 0.38407E+00_JPRB, 0.38390E+00_JPRB, 0.38372E+00_JPRB, 0.38337E+00_JPRB ,&
&  0.38284E+00_JPRB, 0.38238E+00_JPRB, 0.38189E+00_JPRB, 0.38156E+00_JPRB, 0.38136E+00_JPRB, 0.38092E+00_JPRB, 0.38033E+00_JPRB ,&
&  0.37988E+00_JPRB, 0.37967E+00_JPRB, 0.37969E+00_JPRB, 0.37994E+00_JPRB, 0.38035E+00_JPRB, 0.38067E+00_JPRB, 0.38096E+00_JPRB ,&
&  0.38126E+00_JPRB, 0.38168E+00_JPRB, 0.38221E+00_JPRB, 0.38299E+00_JPRB, 0.38396E+00_JPRB, 0.38510E+00_JPRB, 0.38661E+00_JPRB ,&
&  0.38837E+00_JPRB, 0.39021E+00_JPRB, 0.39201E+00_JPRB, 0.39359E+00_JPRB, 0.39499E+00_JPRB, 0.39613E+00_JPRB, 0.39683E+00_JPRB ,&
&  0.39763E+00_JPRB, 0.39855E+00_JPRB, 0.39954E+00_JPRB, 0.40017E+00_JPRB, 0.40070E+00_JPRB, 0.40121E+00_JPRB, 0.40184E+00_JPRB ,&
&  0.40207E+00_JPRB, 0.40228E+00_JPRB, 0.40254E+00_JPRB, 0.40295E+00_JPRB, 0.40311E+00_JPRB, 0.40314E+00_JPRB, 0.40322E+00_JPRB ,&
&  0.40346E+00_JPRB, 0.40360E+00_JPRB, 0.40352E+00_JPRB, 0.40350E+00_JPRB, 0.40364E+00_JPRB, 0.40392E+00_JPRB, 0.40416E+00_JPRB ,&
&  0.40436E+00_JPRB, 0.40451E+00_JPRB, 0.40452E+00_JPRB, 0.40437E+00_JPRB, 0.40413E+00_JPRB, 0.40375E+00_JPRB, 0.40308E+00_JPRB ,&
&  0.40231E+00_JPRB, 0.40150E+00_JPRB, 0.40077E+00_JPRB, 0.40023E+00_JPRB, 0.39964E+00_JPRB, 0.39908E+00_JPRB, 0.39861E+00_JPRB ,&
&  0.39832E+00_JPRB, 0.39809E+00_JPRB, 0.39777E+00_JPRB, 0.39745E+00_JPRB, 0.39708E+00_JPRB, 0.39666E+00_JPRB, 0.39610E+00_JPRB ,&
&  0.39540E+00_JPRB, 0.39450E+00_JPRB, 0.39360E+00_JPRB, 0.39258E+00_JPRB, 0.39151E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.38006E+00_JPRB, 0.37881E+00_JPRB, 0.37790E+00_JPRB, 0.37734E+00_JPRB, 0.37675E+00_JPRB, 0.37624E+00_JPRB, 0.37580E+00_JPRB ,&
&  0.37536E+00_JPRB, 0.37533E+00_JPRB, 0.37504E+00_JPRB, 0.37483E+00_JPRB, 0.37456E+00_JPRB, 0.37463E+00_JPRB, 0.37444E+00_JPRB ,&
&  0.37420E+00_JPRB, 0.37358E+00_JPRB, 0.37341E+00_JPRB, 0.37278E+00_JPRB, 0.37300E+00_JPRB, 0.37340E+00_JPRB, 0.37487E+00_JPRB ,&
&  0.37531E+00_JPRB, 0.37556E+00_JPRB, 0.37520E+00_JPRB, 0.37306E+00_JPRB, 0.37125E+00_JPRB, 0.36954E+00_JPRB, 0.36821E+00_JPRB ,&
&  0.36708E+00_JPRB, 0.36645E+00_JPRB, 0.36668E+00_JPRB, 0.36731E+00_JPRB, 0.36944E+00_JPRB, 0.37005E+00_JPRB, 0.37021E+00_JPRB ,&
&  0.36973E+00_JPRB, 0.36788E+00_JPRB, 0.36826E+00_JPRB, 0.36890E+00_JPRB, 0.37003E+00_JPRB, 0.37271E+00_JPRB, 0.37514E+00_JPRB ,&
&  0.37828E+00_JPRB, 0.38200E+00_JPRB, 0.38752E+00_JPRB, 0.39157E+00_JPRB, 0.39581E+00_JPRB, 0.39947E+00_JPRB, 0.40129E+00_JPRB ,&
&  0.40291E+00_JPRB, 0.40438E+00_JPRB, 0.40585E+00_JPRB, 0.40698E+00_JPRB, 0.40798E+00_JPRB, 0.40900E+00_JPRB, 0.41029E+00_JPRB ,&
&  0.41141E+00_JPRB, 0.41270E+00_JPRB, 0.41390E+00_JPRB, 0.41502E+00_JPRB, 0.41571E+00_JPRB, 0.41611E+00_JPRB, 0.41676E+00_JPRB ,&
&  0.41748E+00_JPRB, 0.41739E+00_JPRB, 0.41710E+00_JPRB, 0.41753E+00_JPRB, 0.41802E+00_JPRB, 0.41851E+00_JPRB, 0.41800E+00_JPRB ,&
&  0.41765E+00_JPRB, 0.41702E+00_JPRB, 0.41651E+00_JPRB, 0.41548E+00_JPRB, 0.41437E+00_JPRB, 0.41323E+00_JPRB, 0.41191E+00_JPRB ,&
&  0.41011E+00_JPRB, 0.40831E+00_JPRB, 0.40690E+00_JPRB, 0.40588E+00_JPRB, 0.40455E+00_JPRB, 0.40371E+00_JPRB, 0.40303E+00_JPRB ,&
&  0.40233E+00_JPRB, 0.40053E+00_JPRB, 0.39866E+00_JPRB, 0.39664E+00_JPRB, 0.39467E+00_JPRB, 0.39274E+00_JPRB, 0.39099E+00_JPRB ,&
&  0.38922E+00_JPRB, 0.38733E+00_JPRB, 0.38547E+00_JPRB, 0.38359E+00_JPRB, 0.38179E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.38159E+00_JPRB, 0.37886E+00_JPRB, 0.37694E+00_JPRB, 0.37556E+00_JPRB, 0.37417E+00_JPRB, 0.37288E+00_JPRB, 0.37248E+00_JPRB ,&
&  0.37226E+00_JPRB, 0.37150E+00_JPRB, 0.37099E+00_JPRB, 0.37062E+00_JPRB, 0.36995E+00_JPRB, 0.37020E+00_JPRB, 0.36993E+00_JPRB ,&
&  0.37005E+00_JPRB, 0.36944E+00_JPRB, 0.37009E+00_JPRB, 0.36932E+00_JPRB, 0.36899E+00_JPRB, 0.36885E+00_JPRB, 0.36840E+00_JPRB ,&
&  0.36961E+00_JPRB, 0.36997E+00_JPRB, 0.36787E+00_JPRB, 0.36483E+00_JPRB, 0.36288E+00_JPRB, 0.35203E+00_JPRB, 0.35285E+00_JPRB ,&
&  0.35699E+00_JPRB, 0.35240E+00_JPRB, 0.35120E+00_JPRB, 0.34970E+00_JPRB, 0.34938E+00_JPRB, 0.34972E+00_JPRB, 0.35252E+00_JPRB ,&
&  0.34779E+00_JPRB, 0.35472E+00_JPRB, 0.35523E+00_JPRB, 0.35547E+00_JPRB, 0.35629E+00_JPRB, 0.35558E+00_JPRB, 0.35932E+00_JPRB ,&
&  0.37421E+00_JPRB, 0.37741E+00_JPRB, 0.38374E+00_JPRB, 0.38946E+00_JPRB, 0.39645E+00_JPRB, 0.40271E+00_JPRB, 0.40884E+00_JPRB ,&
&  0.41335E+00_JPRB, 0.41627E+00_JPRB, 0.41808E+00_JPRB, 0.41838E+00_JPRB, 0.41883E+00_JPRB, 0.41877E+00_JPRB, 0.41932E+00_JPRB ,&
&  0.41914E+00_JPRB, 0.42001E+00_JPRB, 0.42164E+00_JPRB, 0.42388E+00_JPRB, 0.42687E+00_JPRB, 0.42901E+00_JPRB, 0.43090E+00_JPRB ,&
&  0.43212E+00_JPRB, 0.43279E+00_JPRB, 0.43235E+00_JPRB, 0.43242E+00_JPRB, 0.43167E+00_JPRB, 0.43217E+00_JPRB, 0.43157E+00_JPRB ,&
&  0.43165E+00_JPRB, 0.43076E+00_JPRB, 0.43049E+00_JPRB, 0.42926E+00_JPRB, 0.42818E+00_JPRB, 0.42735E+00_JPRB, 0.42635E+00_JPRB ,&
&  0.42541E+00_JPRB, 0.42427E+00_JPRB, 0.42311E+00_JPRB, 0.42212E+00_JPRB, 0.42063E+00_JPRB, 0.41965E+00_JPRB, 0.41857E+00_JPRB ,&
&  0.41780E+00_JPRB, 0.41496E+00_JPRB, 0.41194E+00_JPRB, 0.40875E+00_JPRB, 0.40583E+00_JPRB, 0.40322E+00_JPRB, 0.40058E+00_JPRB ,&
&  0.39819E+00_JPRB, 0.39507E+00_JPRB, 0.39180E+00_JPRB, 0.38820E+00_JPRB, 0.38469E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.39123E+00_JPRB, 0.38749E+00_JPRB, 0.38421E+00_JPRB, 0.38184E+00_JPRB, 0.37982E+00_JPRB, 0.37802E+00_JPRB, 0.37675E+00_JPRB ,&
&  0.37563E+00_JPRB, 0.37498E+00_JPRB, 0.37437E+00_JPRB, 0.37402E+00_JPRB, 0.37360E+00_JPRB, 0.37304E+00_JPRB, 0.37214E+00_JPRB ,&
&  0.37261E+00_JPRB, 0.37174E+00_JPRB, 0.37217E+00_JPRB, 0.37062E+00_JPRB, 0.36962E+00_JPRB, 0.36784E+00_JPRB, 0.36454E+00_JPRB ,&
&  0.36397E+00_JPRB, 0.36243E+00_JPRB, 0.35176E+00_JPRB, 0.35581E+00_JPRB, 0.35292E+00_JPRB, 0.35203E+00_JPRB, 0.35104E+00_JPRB ,&
&  0.34686E+00_JPRB, 0.34464E+00_JPRB, 0.34126E+00_JPRB, 0.34061E+00_JPRB, 0.34144E+00_JPRB, 0.33895E+00_JPRB, 0.33940E+00_JPRB ,&
&  0.33624E+00_JPRB, 0.33512E+00_JPRB, 0.33537E+00_JPRB, 0.33553E+00_JPRB, 0.33787E+00_JPRB, 0.33857E+00_JPRB, 0.34146E+00_JPRB ,&
&  0.34873E+00_JPRB, 0.35914E+00_JPRB, 0.37688E+00_JPRB, 0.38660E+00_JPRB, 0.39638E+00_JPRB, 0.40385E+00_JPRB, 0.41116E+00_JPRB ,&
&  0.41569E+00_JPRB, 0.41888E+00_JPRB, 0.42111E+00_JPRB, 0.42246E+00_JPRB, 0.42229E+00_JPRB, 0.41989E+00_JPRB, 0.41907E+00_JPRB ,&
&  0.41854E+00_JPRB, 0.41880E+00_JPRB, 0.41955E+00_JPRB, 0.42048E+00_JPRB, 0.42308E+00_JPRB, 0.42572E+00_JPRB, 0.42932E+00_JPRB ,&
&  0.43301E+00_JPRB, 0.43716E+00_JPRB, 0.43798E+00_JPRB, 0.43983E+00_JPRB, 0.43970E+00_JPRB, 0.43977E+00_JPRB, 0.43937E+00_JPRB ,&
&  0.43903E+00_JPRB, 0.43860E+00_JPRB, 0.43886E+00_JPRB, 0.43406E+00_JPRB, 0.43228E+00_JPRB, 0.43638E+00_JPRB, 0.43560E+00_JPRB ,&
&  0.43510E+00_JPRB, 0.43317E+00_JPRB, 0.43086E+00_JPRB, 0.42879E+00_JPRB, 0.42716E+00_JPRB, 0.42603E+00_JPRB, 0.42522E+00_JPRB ,&
&  0.42417E+00_JPRB, 0.42088E+00_JPRB, 0.41753E+00_JPRB, 0.41448E+00_JPRB, 0.41123E+00_JPRB, 0.40892E+00_JPRB, 0.40677E+00_JPRB ,&
&  0.40533E+00_JPRB, 0.40425E+00_JPRB, 0.40285E+00_JPRB, 0.40003E+00_JPRB, 0.39598E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.40269E+00_JPRB, 0.39968E+00_JPRB, 0.39730E+00_JPRB, 0.39498E+00_JPRB, 0.39365E+00_JPRB, 0.39215E+00_JPRB, 0.39087E+00_JPRB ,&
&  0.38919E+00_JPRB, 0.38778E+00_JPRB, 0.38581E+00_JPRB, 0.38454E+00_JPRB, 0.38291E+00_JPRB, 0.38147E+00_JPRB, 0.37958E+00_JPRB ,&
&  0.37853E+00_JPRB, 0.37684E+00_JPRB, 0.37619E+00_JPRB, 0.37378E+00_JPRB, 0.37360E+00_JPRB, 0.37138E+00_JPRB, 0.36678E+00_JPRB ,&
&  0.36303E+00_JPRB, 0.35878E+00_JPRB, 0.35009E+00_JPRB, 0.35301E+00_JPRB, 0.35246E+00_JPRB, 0.35378E+00_JPRB, 0.34907E+00_JPRB ,&
&  0.34712E+00_JPRB, 0.33664E+00_JPRB, 0.34402E+00_JPRB, 0.33935E+00_JPRB, 0.33577E+00_JPRB, 0.33007E+00_JPRB, 0.32284E+00_JPRB ,&
&  0.32067E+00_JPRB, 0.31667E+00_JPRB, 0.31652E+00_JPRB, 0.31885E+00_JPRB, 0.32190E+00_JPRB, 0.32622E+00_JPRB, 0.33115E+00_JPRB ,&
&  0.33350E+00_JPRB, 0.36038E+00_JPRB, 0.37649E+00_JPRB, 0.38834E+00_JPRB, 0.39785E+00_JPRB, 0.40526E+00_JPRB, 0.41241E+00_JPRB ,&
&  0.41869E+00_JPRB, 0.42430E+00_JPRB, 0.42948E+00_JPRB, 0.43011E+00_JPRB, 0.43551E+00_JPRB, 0.43712E+00_JPRB, 0.43873E+00_JPRB ,&
&  0.44017E+00_JPRB, 0.43939E+00_JPRB, 0.43882E+00_JPRB, 0.43680E+00_JPRB, 0.43618E+00_JPRB, 0.43431E+00_JPRB, 0.43567E+00_JPRB ,&
&  0.43508E+00_JPRB, 0.43539E+00_JPRB, 0.43662E+00_JPRB, 0.43767E+00_JPRB, 0.43815E+00_JPRB, 0.43867E+00_JPRB, 0.43956E+00_JPRB ,&
&  0.44035E+00_JPRB, 0.44277E+00_JPRB, 0.44455E+00_JPRB, 0.44748E+00_JPRB, 0.44943E+00_JPRB, 0.44842E+00_JPRB, 0.44757E+00_JPRB ,&
&  0.44791E+00_JPRB, 0.44371E+00_JPRB, 0.44002E+00_JPRB, 0.43692E+00_JPRB, 0.43492E+00_JPRB, 0.43364E+00_JPRB, 0.43339E+00_JPRB ,&
&  0.43272E+00_JPRB, 0.42973E+00_JPRB, 0.42642E+00_JPRB, 0.42266E+00_JPRB, 0.41886E+00_JPRB, 0.41657E+00_JPRB, 0.41460E+00_JPRB ,&
&  0.41364E+00_JPRB, 0.41292E+00_JPRB, 0.41135E+00_JPRB, 0.40857E+00_JPRB, 0.40560E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.41778E+00_JPRB, 0.41606E+00_JPRB, 0.41489E+00_JPRB, 0.41303E+00_JPRB, 0.41205E+00_JPRB, 0.41051E+00_JPRB, 0.40917E+00_JPRB ,&
&  0.40798E+00_JPRB, 0.40568E+00_JPRB, 0.40274E+00_JPRB, 0.39921E+00_JPRB, 0.39646E+00_JPRB, 0.39381E+00_JPRB, 0.39082E+00_JPRB ,&
&  0.38618E+00_JPRB, 0.38081E+00_JPRB, 0.37536E+00_JPRB, 0.37077E+00_JPRB, 0.37005E+00_JPRB, 0.36826E+00_JPRB, 0.36628E+00_JPRB ,&
&  0.36202E+00_JPRB, 0.35744E+00_JPRB, 0.35134E+00_JPRB, 0.35031E+00_JPRB, 0.35103E+00_JPRB, 0.34721E+00_JPRB, 0.34895E+00_JPRB ,&
&  0.35578E+00_JPRB, 0.35739E+00_JPRB, 0.35831E+00_JPRB, 0.35857E+00_JPRB, 0.34432E+00_JPRB, 0.31655E+00_JPRB, 0.32443E+00_JPRB ,&
&  0.32231E+00_JPRB, 0.31455E+00_JPRB, 0.31119E+00_JPRB, 0.30924E+00_JPRB, 0.31228E+00_JPRB, 0.32021E+00_JPRB, 0.32908E+00_JPRB ,&
&  0.33465E+00_JPRB, 0.37955E+00_JPRB, 0.39772E+00_JPRB, 0.40998E+00_JPRB, 0.41717E+00_JPRB, 0.42374E+00_JPRB, 0.43123E+00_JPRB ,&
&  0.43755E+00_JPRB, 0.44669E+00_JPRB, 0.45827E+00_JPRB, 0.46856E+00_JPRB, 0.47718E+00_JPRB, 0.48370E+00_JPRB, 0.48927E+00_JPRB ,&
&  0.49566E+00_JPRB, 0.49418E+00_JPRB, 0.48780E+00_JPRB, 0.47977E+00_JPRB, 0.47227E+00_JPRB, 0.46409E+00_JPRB, 0.45678E+00_JPRB ,&
&  0.45209E+00_JPRB, 0.44672E+00_JPRB, 0.44736E+00_JPRB, 0.44573E+00_JPRB, 0.44452E+00_JPRB, 0.44481E+00_JPRB, 0.44877E+00_JPRB ,&
&  0.45148E+00_JPRB, 0.45506E+00_JPRB, 0.45723E+00_JPRB, 0.46047E+00_JPRB, 0.46235E+00_JPRB, 0.46100E+00_JPRB, 0.45994E+00_JPRB ,&
&  0.45775E+00_JPRB, 0.45949E+00_JPRB, 0.45853E+00_JPRB, 0.45896E+00_JPRB, 0.46143E+00_JPRB, 0.46215E+00_JPRB, 0.46376E+00_JPRB ,&
&  0.46462E+00_JPRB, 0.46484E+00_JPRB, 0.46264E+00_JPRB, 0.45853E+00_JPRB, 0.45297E+00_JPRB, 0.44676E+00_JPRB, 0.43983E+00_JPRB ,&
&  0.43377E+00_JPRB, 0.42766E+00_JPRB, 0.42274E+00_JPRB, 0.42022E+00_JPRB, 0.41858E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.43969E+00_JPRB, 0.43640E+00_JPRB, 0.43375E+00_JPRB, 0.43238E+00_JPRB, 0.43116E+00_JPRB, 0.42865E+00_JPRB, 0.42641E+00_JPRB ,&
&  0.42440E+00_JPRB, 0.42238E+00_JPRB, 0.41931E+00_JPRB, 0.41589E+00_JPRB, 0.41276E+00_JPRB, 0.41030E+00_JPRB, 0.40829E+00_JPRB ,&
&  0.40294E+00_JPRB, 0.38838E+00_JPRB, 0.38231E+00_JPRB, 0.37566E+00_JPRB, 0.36715E+00_JPRB, 0.36255E+00_JPRB, 0.35955E+00_JPRB ,&
&  0.35555E+00_JPRB, 0.35346E+00_JPRB, 0.35365E+00_JPRB, 0.34970E+00_JPRB, 0.34764E+00_JPRB, 0.35131E+00_JPRB, 0.35415E+00_JPRB ,&
&  0.34584E+00_JPRB, 0.34705E+00_JPRB, 0.36144E+00_JPRB, 0.36257E+00_JPRB, 0.35020E+00_JPRB, 0.34695E+00_JPRB, 0.33622E+00_JPRB ,&
&  0.33098E+00_JPRB, 0.31858E+00_JPRB, 0.31412E+00_JPRB, 0.31121E+00_JPRB, 0.31647E+00_JPRB, 0.32491E+00_JPRB, 0.34605E+00_JPRB ,&
&  0.36980E+00_JPRB, 0.41234E+00_JPRB, 0.42755E+00_JPRB, 0.43818E+00_JPRB, 0.44923E+00_JPRB, 0.46064E+00_JPRB, 0.47331E+00_JPRB ,&
&  0.49008E+00_JPRB, 0.50827E+00_JPRB, 0.52981E+00_JPRB, 0.54794E+00_JPRB, 0.56505E+00_JPRB, 0.57364E+00_JPRB, 0.57371E+00_JPRB ,&
&  0.58067E+00_JPRB, 0.57710E+00_JPRB, 0.56053E+00_JPRB, 0.54491E+00_JPRB, 0.52684E+00_JPRB, 0.50864E+00_JPRB, 0.48668E+00_JPRB ,&
&  0.47880E+00_JPRB, 0.48308E+00_JPRB, 0.47573E+00_JPRB, 0.46935E+00_JPRB, 0.46429E+00_JPRB, 0.46204E+00_JPRB, 0.46168E+00_JPRB ,&
&  0.45916E+00_JPRB, 0.46822E+00_JPRB, 0.51049E+00_JPRB, 0.51459E+00_JPRB, 0.51166E+00_JPRB, 0.50660E+00_JPRB, 0.50181E+00_JPRB ,&
&  0.50093E+00_JPRB, 0.50441E+00_JPRB, 0.51021E+00_JPRB, 0.51310E+00_JPRB, 0.51717E+00_JPRB, 0.51476E+00_JPRB, 0.51905E+00_JPRB ,&
&  0.51667E+00_JPRB, 0.51402E+00_JPRB, 0.50862E+00_JPRB, 0.50525E+00_JPRB, 0.50107E+00_JPRB, 0.49665E+00_JPRB, 0.49051E+00_JPRB ,&
&  0.48322E+00_JPRB, 0.47376E+00_JPRB, 0.46384E+00_JPRB, 0.45453E+00_JPRB, 0.44595E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.46643E+00_JPRB, 0.45503E+00_JPRB, 0.44812E+00_JPRB, 0.44634E+00_JPRB, 0.44649E+00_JPRB, 0.43361E+00_JPRB, 0.42429E+00_JPRB ,&
&  0.41918E+00_JPRB, 0.41936E+00_JPRB, 0.41914E+00_JPRB, 0.41646E+00_JPRB, 0.41553E+00_JPRB, 0.41461E+00_JPRB, 0.41440E+00_JPRB ,&
&  0.41268E+00_JPRB, 0.40778E+00_JPRB, 0.39748E+00_JPRB, 0.38033E+00_JPRB, 0.36727E+00_JPRB, 0.35850E+00_JPRB, 0.35364E+00_JPRB ,&
&  0.35063E+00_JPRB, 0.35008E+00_JPRB, 0.35429E+00_JPRB, 0.35564E+00_JPRB, 0.35494E+00_JPRB, 0.35195E+00_JPRB, 0.34794E+00_JPRB ,&
&  0.34774E+00_JPRB, 0.35447E+00_JPRB, 0.35079E+00_JPRB, 0.35279E+00_JPRB, 0.36135E+00_JPRB, 0.36056E+00_JPRB, 0.34752E+00_JPRB ,&
&  0.33732E+00_JPRB, 0.32708E+00_JPRB, 0.32170E+00_JPRB, 0.32051E+00_JPRB, 0.32939E+00_JPRB, 0.34494E+00_JPRB, 0.37372E+00_JPRB ,&
&  0.42254E+00_JPRB, 0.44337E+00_JPRB, 0.45428E+00_JPRB, 0.46687E+00_JPRB, 0.48038E+00_JPRB, 0.49515E+00_JPRB, 0.51805E+00_JPRB ,&
&  0.55183E+00_JPRB, 0.58599E+00_JPRB, 0.61854E+00_JPRB, 0.64028E+00_JPRB, 0.66489E+00_JPRB, 0.64021E+00_JPRB, 0.63353E+00_JPRB ,&
&  0.68760E+00_JPRB, 0.71169E+00_JPRB, 0.68084E+00_JPRB, 0.64855E+00_JPRB, 0.61348E+00_JPRB, 0.58064E+00_JPRB, 0.55197E+00_JPRB ,&
&  0.53181E+00_JPRB, 0.51779E+00_JPRB, 0.50573E+00_JPRB, 0.49136E+00_JPRB, 0.48081E+00_JPRB, 0.47891E+00_JPRB, 0.48743E+00_JPRB ,&
&  0.50833E+00_JPRB, 0.57119E+00_JPRB, 0.72034E+00_JPRB, 0.60776E+00_JPRB, 0.56658E+00_JPRB, 0.55736E+00_JPRB, 0.55866E+00_JPRB ,&
&  0.57111E+00_JPRB, 0.58225E+00_JPRB, 0.58913E+00_JPRB, 0.58152E+00_JPRB, 0.56476E+00_JPRB, 0.54438E+00_JPRB, 0.53103E+00_JPRB ,&
&  0.51986E+00_JPRB, 0.50879E+00_JPRB, 0.50087E+00_JPRB, 0.49901E+00_JPRB, 0.49471E+00_JPRB, 0.48968E+00_JPRB, 0.48600E+00_JPRB ,&
&  0.48367E+00_JPRB, 0.48349E+00_JPRB, 0.48524E+00_JPRB, 0.48336E+00_JPRB, 0.47922E+00_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.45825E+00_JPRB, 0.46652E+00_JPRB, 0.47215E+00_JPRB, 0.46982E+00_JPRB, 0.46630E+00_JPRB, 0.43770E+00_JPRB, 0.41543E+00_JPRB ,&
&  0.40544E+00_JPRB, 0.40257E+00_JPRB, 0.40495E+00_JPRB, 0.40704E+00_JPRB, 0.41002E+00_JPRB, 0.41252E+00_JPRB, 0.41213E+00_JPRB ,&
&  0.40701E+00_JPRB, 0.40135E+00_JPRB, 0.39542E+00_JPRB, 0.38528E+00_JPRB, 0.37251E+00_JPRB, 0.35927E+00_JPRB, 0.34952E+00_JPRB ,&
&  0.34358E+00_JPRB, 0.34207E+00_JPRB, 0.34692E+00_JPRB, 0.35271E+00_JPRB, 0.35843E+00_JPRB, 0.36085E+00_JPRB, 0.36121E+00_JPRB ,&
&  0.35531E+00_JPRB, 0.34995E+00_JPRB, 0.35512E+00_JPRB, 0.35889E+00_JPRB, 0.36232E+00_JPRB, 0.36944E+00_JPRB, 0.35837E+00_JPRB ,&
&  0.34640E+00_JPRB, 0.33615E+00_JPRB, 0.33516E+00_JPRB, 0.34479E+00_JPRB, 0.37558E+00_JPRB, 0.43003E+00_JPRB, 0.44490E+00_JPRB ,&
&  0.45713E+00_JPRB, 0.46736E+00_JPRB, 0.47932E+00_JPRB, 0.49880E+00_JPRB, 0.52228E+00_JPRB, 0.54892E+00_JPRB, 0.59342E+00_JPRB ,&
&  0.65073E+00_JPRB, 0.69577E+00_JPRB, 0.74333E+00_JPRB, 0.68257E+00_JPRB, 0.65771E+00_JPRB, 0.68127E+00_JPRB, 0.71057E+00_JPRB ,&
&  0.73765E+00_JPRB, 0.74602E+00_JPRB, 0.74061E+00_JPRB, 0.74863E+00_JPRB, 0.71542E+00_JPRB, 0.67055E+00_JPRB, 0.60970E+00_JPRB ,&
&  0.56399E+00_JPRB, 0.52968E+00_JPRB, 0.51019E+00_JPRB, 0.49890E+00_JPRB, 0.50326E+00_JPRB, 0.51612E+00_JPRB, 0.53772E+00_JPRB ,&
&  0.57240E+00_JPRB, 0.61350E+00_JPRB, 0.63272E+00_JPRB, 0.60841E+00_JPRB, 0.61693E+00_JPRB, 0.64095E+00_JPRB, 0.65984E+00_JPRB ,&
&  0.66084E+00_JPRB, 0.66001E+00_JPRB, 0.66419E+00_JPRB, 0.65777E+00_JPRB, 0.61686E+00_JPRB, 0.57957E+00_JPRB, 0.53747E+00_JPRB ,&
&  0.49745E+00_JPRB, 0.46196E+00_JPRB, 0.44300E+00_JPRB, 0.43814E+00_JPRB, 0.43436E+00_JPRB, 0.43018E+00_JPRB, 0.42283E+00_JPRB ,&
&  0.41706E+00_JPRB, 0.41745E+00_JPRB, 0.42561E+00_JPRB, 0.43505E+00_JPRB, 0.44857E+00_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.48736E+00_JPRB, 0.49819E+00_JPRB, 0.50001E+00_JPRB, 0.50677E+00_JPRB, 0.48647E+00_JPRB, 0.46829E+00_JPRB, 0.45008E+00_JPRB ,&
&  0.44561E+00_JPRB, 0.43282E+00_JPRB, 0.42574E+00_JPRB, 0.41293E+00_JPRB, 0.40121E+00_JPRB, 0.39466E+00_JPRB, 0.39671E+00_JPRB ,&
&  0.40092E+00_JPRB, 0.39992E+00_JPRB, 0.39511E+00_JPRB, 0.38289E+00_JPRB, 0.37133E+00_JPRB, 0.36150E+00_JPRB, 0.35354E+00_JPRB ,&
&  0.34779E+00_JPRB, 0.34795E+00_JPRB, 0.35145E+00_JPRB, 0.35827E+00_JPRB, 0.36335E+00_JPRB, 0.36765E+00_JPRB, 0.36920E+00_JPRB ,&
&  0.36581E+00_JPRB, 0.35924E+00_JPRB, 0.36076E+00_JPRB, 0.36932E+00_JPRB, 0.37250E+00_JPRB, 0.38242E+00_JPRB, 0.36833E+00_JPRB ,&
&  0.35391E+00_JPRB, 0.35303E+00_JPRB, 0.36815E+00_JPRB, 0.42861E+00_JPRB, 0.44709E+00_JPRB, 0.45463E+00_JPRB, 0.46709E+00_JPRB ,&
&  0.45977E+00_JPRB, 0.46031E+00_JPRB, 0.47910E+00_JPRB, 0.52459E+00_JPRB, 0.56916E+00_JPRB, 0.63747E+00_JPRB, 0.71155E+00_JPRB ,&
&  0.77173E+00_JPRB, 0.79765E+00_JPRB, 0.81886E+00_JPRB, 0.75629E+00_JPRB, 0.75440E+00_JPRB, 0.87659E+00_JPRB, 0.89924E+00_JPRB ,&
&  0.90915E+00_JPRB, 0.89722E+00_JPRB, 0.90751E+00_JPRB, 0.86508E+00_JPRB, 0.77285E+00_JPRB, 0.70395E+00_JPRB, 0.65008E+00_JPRB ,&
&  0.59499E+00_JPRB, 0.56070E+00_JPRB, 0.55013E+00_JPRB, 0.56115E+00_JPRB, 0.58932E+00_JPRB, 0.64564E+00_JPRB, 0.71730E+00_JPRB ,&
&  0.78621E+00_JPRB, 0.82755E+00_JPRB, 0.85057E+00_JPRB, 0.84122E+00_JPRB, 0.82732E+00_JPRB, 0.80960E+00_JPRB, 0.79715E+00_JPRB ,&
&  0.78340E+00_JPRB, 0.77975E+00_JPRB, 0.77349E+00_JPRB, 0.76394E+00_JPRB, 0.71169E+00_JPRB, 0.62656E+00_JPRB, 0.54186E+00_JPRB ,&
&  0.48179E+00_JPRB, 0.44528E+00_JPRB, 0.41996E+00_JPRB, 0.41445E+00_JPRB, 0.41738E+00_JPRB, 0.42003E+00_JPRB, 0.41163E+00_JPRB ,&
&  0.40916E+00_JPRB, 0.41814E+00_JPRB, 0.43120E+00_JPRB, 0.44567E+00_JPRB, 0.46478E+00_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.51848E+00_JPRB, 0.52612E+00_JPRB, 0.53105E+00_JPRB, 0.52930E+00_JPRB, 0.52256E+00_JPRB, 0.49859E+00_JPRB, 0.46325E+00_JPRB ,&
&  0.45922E+00_JPRB, 0.43371E+00_JPRB, 0.42259E+00_JPRB, 0.41211E+00_JPRB, 0.40516E+00_JPRB, 0.39407E+00_JPRB, 0.38059E+00_JPRB ,&
&  0.37971E+00_JPRB, 0.38576E+00_JPRB, 0.39178E+00_JPRB, 0.39005E+00_JPRB, 0.38299E+00_JPRB, 0.37377E+00_JPRB, 0.36538E+00_JPRB ,&
&  0.36000E+00_JPRB, 0.36091E+00_JPRB, 0.36781E+00_JPRB, 0.37718E+00_JPRB, 0.38098E+00_JPRB, 0.38327E+00_JPRB, 0.38760E+00_JPRB ,&
&  0.38797E+00_JPRB, 0.38431E+00_JPRB, 0.38084E+00_JPRB, 0.38456E+00_JPRB, 0.38606E+00_JPRB, 0.39823E+00_JPRB, 0.40681E+00_JPRB ,&
&  0.37536E+00_JPRB, 0.38257E+00_JPRB, 0.43422E+00_JPRB, 0.44965E+00_JPRB, 0.46250E+00_JPRB, 0.47433E+00_JPRB, 0.48672E+00_JPRB ,&
&  0.50061E+00_JPRB, 0.52006E+00_JPRB, 0.54124E+00_JPRB, 0.56008E+00_JPRB, 0.62847E+00_JPRB, 0.77678E+00_JPRB, 0.83167E+00_JPRB ,&
&  0.89198E+00_JPRB, 0.79429E+00_JPRB, 0.74048E+00_JPRB, 0.74619E+00_JPRB, 0.88162E+00_JPRB, 0.95666E+00_JPRB, 0.99476E+00_JPRB ,&
&  0.10310E+01_JPRB, 0.10089E+01_JPRB, 0.97697E+00_JPRB, 0.91553E+00_JPRB, 0.83293E+00_JPRB, 0.77091E+00_JPRB, 0.73885E+00_JPRB ,&
&  0.70169E+00_JPRB, 0.68998E+00_JPRB, 0.71628E+00_JPRB, 0.78513E+00_JPRB, 0.86548E+00_JPRB, 0.93464E+00_JPRB, 0.98397E+00_JPRB ,&
&  0.10327E+01_JPRB, 0.10559E+01_JPRB, 0.10420E+01_JPRB, 0.10163E+01_JPRB, 0.97362E+00_JPRB, 0.94848E+00_JPRB, 0.93923E+00_JPRB ,&
&  0.92759E+00_JPRB, 0.91692E+00_JPRB, 0.89368E+00_JPRB, 0.86030E+00_JPRB, 0.80615E+00_JPRB, 0.75328E+00_JPRB, 0.67909E+00_JPRB ,&
&  0.56106E+00_JPRB, 0.49117E+00_JPRB, 0.44295E+00_JPRB, 0.43020E+00_JPRB, 0.43371E+00_JPRB, 0.44001E+00_JPRB, 0.44357E+00_JPRB ,&
&  0.45669E+00_JPRB, 0.46511E+00_JPRB, 0.46707E+00_JPRB, 0.47843E+00_JPRB, 0.49534E+00_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.55181E+00_JPRB, 0.55346E+00_JPRB, 0.55194E+00_JPRB, 0.55200E+00_JPRB, 0.55068E+00_JPRB, 0.52677E+00_JPRB, 0.50765E+00_JPRB ,&
&  0.56695E+00_JPRB, 0.52859E+00_JPRB, 0.54867E+00_JPRB, 0.46345E+00_JPRB, 0.42705E+00_JPRB, 0.41872E+00_JPRB, 0.40276E+00_JPRB ,&
&  0.39616E+00_JPRB, 0.39863E+00_JPRB, 0.39777E+00_JPRB, 0.39815E+00_JPRB, 0.39713E+00_JPRB, 0.38900E+00_JPRB, 0.38237E+00_JPRB ,&
&  0.38038E+00_JPRB, 0.38454E+00_JPRB, 0.39454E+00_JPRB, 0.40128E+00_JPRB, 0.40704E+00_JPRB, 0.41156E+00_JPRB, 0.41611E+00_JPRB ,&
&  0.40507E+00_JPRB, 0.40009E+00_JPRB, 0.40104E+00_JPRB, 0.39889E+00_JPRB, 0.40228E+00_JPRB, 0.41158E+00_JPRB, 0.42818E+00_JPRB ,&
&  0.44645E+00_JPRB, 0.43170E+00_JPRB, 0.46615E+00_JPRB, 0.48222E+00_JPRB, 0.49580E+00_JPRB, 0.50803E+00_JPRB, 0.51915E+00_JPRB ,&
&  0.53136E+00_JPRB, 0.54940E+00_JPRB, 0.57027E+00_JPRB, 0.59031E+00_JPRB, 0.71194E+00_JPRB, 0.94187E+00_JPRB, 0.94854E+00_JPRB ,&
&  0.94944E+00_JPRB, 0.79007E+00_JPRB, 0.87189E+00_JPRB, 0.90979E+00_JPRB, 0.10184E+01_JPRB, 0.11290E+01_JPRB, 0.11197E+01_JPRB ,&
&  0.12141E+01_JPRB, 0.12454E+01_JPRB, 0.11807E+01_JPRB, 0.10604E+01_JPRB, 0.95962E+00_JPRB, 0.90535E+00_JPRB, 0.87541E+00_JPRB ,&
&  0.87221E+00_JPRB, 0.85275E+00_JPRB, 0.94195E+00_JPRB, 0.10121E+01_JPRB, 0.10472E+01_JPRB, 0.10835E+01_JPRB, 0.11311E+01_JPRB ,&
&  0.11634E+01_JPRB, 0.11662E+01_JPRB, 0.11585E+01_JPRB, 0.10960E+01_JPRB, 0.10641E+01_JPRB, 0.10404E+01_JPRB, 0.99965E+00_JPRB ,&
&  0.97265E+00_JPRB, 0.94885E+00_JPRB, 0.91457E+00_JPRB, 0.91563E+00_JPRB, 0.83694E+00_JPRB, 0.78168E+00_JPRB, 0.74614E+00_JPRB ,&
&  0.65429E+00_JPRB, 0.55006E+00_JPRB, 0.51878E+00_JPRB, 0.50291E+00_JPRB, 0.49136E+00_JPRB, 0.49265E+00_JPRB, 0.50842E+00_JPRB ,&
&  0.49692E+00_JPRB, 0.50603E+00_JPRB, 0.53717E+00_JPRB, 0.54482E+00_JPRB, 0.54627E+00_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.58354E+00_JPRB, 0.58799E+00_JPRB, 0.58735E+00_JPRB, 0.58778E+00_JPRB, 0.58630E+00_JPRB, 0.58154E+00_JPRB, 0.55143E+00_JPRB ,&
&  0.58003E+00_JPRB, 0.56661E+00_JPRB, 0.54087E+00_JPRB, 0.52888E+00_JPRB, 0.53578E+00_JPRB, 0.50022E+00_JPRB, 0.42414E+00_JPRB ,&
&  0.39997E+00_JPRB, 0.40399E+00_JPRB, 0.41506E+00_JPRB, 0.41943E+00_JPRB, 0.42067E+00_JPRB, 0.41853E+00_JPRB, 0.41713E+00_JPRB ,&
&  0.41859E+00_JPRB, 0.42217E+00_JPRB, 0.43224E+00_JPRB, 0.44510E+00_JPRB, 0.44790E+00_JPRB, 0.45077E+00_JPRB, 0.45688E+00_JPRB ,&
&  0.43980E+00_JPRB, 0.42339E+00_JPRB, 0.41476E+00_JPRB, 0.39974E+00_JPRB, 0.42116E+00_JPRB, 0.43761E+00_JPRB, 0.45960E+00_JPRB ,&
&  0.48178E+00_JPRB, 0.50327E+00_JPRB, 0.52347E+00_JPRB, 0.53948E+00_JPRB, 0.54964E+00_JPRB, 0.56040E+00_JPRB, 0.57087E+00_JPRB ,&
&  0.58317E+00_JPRB, 0.60063E+00_JPRB, 0.62320E+00_JPRB, 0.64368E+00_JPRB, 0.87929E+00_JPRB, 0.11042E+01_JPRB, 0.11548E+01_JPRB ,&
&  0.11145E+01_JPRB, 0.12150E+01_JPRB, 0.12661E+01_JPRB, 0.11894E+01_JPRB, 0.12818E+01_JPRB, 0.13331E+01_JPRB, 0.12930E+01_JPRB ,&
&  0.13405E+01_JPRB, 0.13145E+01_JPRB, 0.12645E+01_JPRB, 0.12081E+01_JPRB, 0.11678E+01_JPRB, 0.11183E+01_JPRB, 0.10797E+01_JPRB ,&
&  0.10830E+01_JPRB, 0.11648E+01_JPRB, 0.11677E+01_JPRB, 0.11850E+01_JPRB, 0.11891E+01_JPRB, 0.12044E+01_JPRB, 0.12220E+01_JPRB ,&
&  0.12324E+01_JPRB, 0.12442E+01_JPRB, 0.12395E+01_JPRB, 0.12034E+01_JPRB, 0.11595E+01_JPRB, 0.11140E+01_JPRB, 0.10590E+01_JPRB ,&
&  0.10137E+01_JPRB, 0.91574E+00_JPRB, 0.83830E+00_JPRB, 0.81091E+00_JPRB, 0.78387E+00_JPRB, 0.72467E+00_JPRB, 0.70456E+00_JPRB ,&
&  0.68184E+00_JPRB, 0.63241E+00_JPRB, 0.64506E+00_JPRB, 0.61000E+00_JPRB, 0.59183E+00_JPRB, 0.57144E+00_JPRB, 0.52198E+00_JPRB ,&
&  0.50051E+00_JPRB, 0.54491E+00_JPRB, 0.56393E+00_JPRB, 0.57417E+00_JPRB, 0.57825E+00_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.62948E+00_JPRB, 0.63850E+00_JPRB, 0.64798E+00_JPRB, 0.64564E+00_JPRB, 0.62704E+00_JPRB, 0.60311E+00_JPRB, 0.58450E+00_JPRB ,&
&  0.57534E+00_JPRB, 0.56221E+00_JPRB, 0.54893E+00_JPRB, 0.53737E+00_JPRB, 0.53218E+00_JPRB, 0.53679E+00_JPRB, 0.50080E+00_JPRB ,&
&  0.44576E+00_JPRB, 0.43815E+00_JPRB, 0.42961E+00_JPRB, 0.43325E+00_JPRB, 0.44588E+00_JPRB, 0.45164E+00_JPRB, 0.45716E+00_JPRB ,&
&  0.47365E+00_JPRB, 0.48424E+00_JPRB, 0.48962E+00_JPRB, 0.49623E+00_JPRB, 0.50109E+00_JPRB, 0.51051E+00_JPRB, 0.51984E+00_JPRB ,&
&  0.49471E+00_JPRB, 0.46063E+00_JPRB, 0.44231E+00_JPRB, 0.43513E+00_JPRB, 0.45790E+00_JPRB, 0.49002E+00_JPRB, 0.52562E+00_JPRB ,&
&  0.55555E+00_JPRB, 0.57972E+00_JPRB, 0.60181E+00_JPRB, 0.61881E+00_JPRB, 0.62636E+00_JPRB, 0.63454E+00_JPRB, 0.64740E+00_JPRB ,&
&  0.66883E+00_JPRB, 0.68110E+00_JPRB, 0.69366E+00_JPRB, 0.72487E+00_JPRB, 0.10726E+01_JPRB, 0.14677E+01_JPRB, 0.16239E+01_JPRB ,&
&  0.15888E+01_JPRB, 0.16118E+01_JPRB, 0.16301E+01_JPRB, 0.16869E+01_JPRB, 0.16779E+01_JPRB, 0.15803E+01_JPRB, 0.14841E+01_JPRB ,&
&  0.14830E+01_JPRB, 0.13953E+01_JPRB, 0.14366E+01_JPRB, 0.14059E+01_JPRB, 0.13892E+01_JPRB, 0.13839E+01_JPRB, 0.13494E+01_JPRB ,&
&  0.13593E+01_JPRB, 0.13670E+01_JPRB, 0.13463E+01_JPRB, 0.13025E+01_JPRB, 0.12816E+01_JPRB, 0.12918E+01_JPRB, 0.12993E+01_JPRB ,&
&  0.13303E+01_JPRB, 0.13512E+01_JPRB, 0.12903E+01_JPRB, 0.12436E+01_JPRB, 0.11877E+01_JPRB, 0.12412E+01_JPRB, 0.11494E+01_JPRB ,&
&  0.99514E+00_JPRB, 0.80880E+00_JPRB, 0.79524E+00_JPRB, 0.78211E+00_JPRB, 0.78357E+00_JPRB, 0.75926E+00_JPRB, 0.71222E+00_JPRB ,&
&  0.71378E+00_JPRB, 0.75855E+00_JPRB, 0.73822E+00_JPRB, 0.68534E+00_JPRB, 0.66676E+00_JPRB, 0.63044E+00_JPRB, 0.54581E+00_JPRB ,&
&  0.54540E+00_JPRB, 0.59664E+00_JPRB, 0.60363E+00_JPRB, 0.60884E+00_JPRB, 0.61601E+00_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.68673E+00_JPRB, 0.68549E+00_JPRB, 0.69437E+00_JPRB, 0.68751E+00_JPRB, 0.66528E+00_JPRB, 0.63769E+00_JPRB, 0.61134E+00_JPRB ,&
&  0.59383E+00_JPRB, 0.58345E+00_JPRB, 0.57531E+00_JPRB, 0.56868E+00_JPRB, 0.56503E+00_JPRB, 0.55698E+00_JPRB, 0.56662E+00_JPRB ,&
&  0.48960E+00_JPRB, 0.46958E+00_JPRB, 0.45649E+00_JPRB, 0.46076E+00_JPRB, 0.50330E+00_JPRB, 0.50163E+00_JPRB, 0.50752E+00_JPRB ,&
&  0.51637E+00_JPRB, 0.52491E+00_JPRB, 0.52079E+00_JPRB, 0.52268E+00_JPRB, 0.53754E+00_JPRB, 0.57307E+00_JPRB, 0.61540E+00_JPRB ,&
&  0.59628E+00_JPRB, 0.54549E+00_JPRB, 0.52639E+00_JPRB, 0.52221E+00_JPRB, 0.55899E+00_JPRB, 0.58792E+00_JPRB, 0.64737E+00_JPRB ,&
&  0.67552E+00_JPRB, 0.69126E+00_JPRB, 0.70035E+00_JPRB, 0.70725E+00_JPRB, 0.71898E+00_JPRB, 0.74104E+00_JPRB, 0.77034E+00_JPRB ,&
&  0.79630E+00_JPRB, 0.78918E+00_JPRB, 0.79755E+00_JPRB, 0.85920E+00_JPRB, 0.13513E+01_JPRB, 0.18966E+01_JPRB, 0.22091E+01_JPRB ,&
&  0.23104E+01_JPRB, 0.21966E+01_JPRB, 0.22065E+01_JPRB, 0.23941E+01_JPRB, 0.22818E+01_JPRB, 0.20537E+01_JPRB, 0.18379E+01_JPRB ,&
&  0.17234E+01_JPRB, 0.16133E+01_JPRB, 0.16250E+01_JPRB, 0.16385E+01_JPRB, 0.15632E+01_JPRB, 0.15357E+01_JPRB, 0.15129E+01_JPRB ,&
&  0.14998E+01_JPRB, 0.14971E+01_JPRB, 0.14389E+01_JPRB, 0.13772E+01_JPRB, 0.13320E+01_JPRB, 0.13662E+01_JPRB, 0.13886E+01_JPRB ,&
&  0.14511E+01_JPRB, 0.14222E+01_JPRB, 0.12745E+01_JPRB, 0.12662E+01_JPRB, 0.10432E+01_JPRB, 0.99439E+00_JPRB, 0.11138E+01_JPRB ,&
&  0.96037E+00_JPRB, 0.82841E+00_JPRB, 0.78173E+00_JPRB, 0.78925E+00_JPRB, 0.80426E+00_JPRB, 0.83721E+00_JPRB, 0.80213E+00_JPRB ,&
&  0.72628E+00_JPRB, 0.76119E+00_JPRB, 0.83012E+00_JPRB, 0.77848E+00_JPRB, 0.75789E+00_JPRB, 0.72018E+00_JPRB, 0.65065E+00_JPRB ,&
&  0.67701E+00_JPRB, 0.66689E+00_JPRB, 0.66374E+00_JPRB, 0.66781E+00_JPRB, 0.67856E+00_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.71370E+00_JPRB, 0.70102E+00_JPRB, 0.71987E+00_JPRB, 0.68055E+00_JPRB, 0.66091E+00_JPRB, 0.64381E+00_JPRB, 0.63106E+00_JPRB ,&
&  0.62325E+00_JPRB, 0.61451E+00_JPRB, 0.60660E+00_JPRB, 0.59878E+00_JPRB, 0.59348E+00_JPRB, 0.58456E+00_JPRB, 0.57934E+00_JPRB ,&
&  0.58012E+00_JPRB, 0.53856E+00_JPRB, 0.52633E+00_JPRB, 0.50298E+00_JPRB, 0.53690E+00_JPRB, 0.53822E+00_JPRB, 0.54894E+00_JPRB ,&
&  0.56412E+00_JPRB, 0.58603E+00_JPRB, 0.58727E+00_JPRB, 0.59293E+00_JPRB, 0.61410E+00_JPRB, 0.67333E+00_JPRB, 0.75021E+00_JPRB ,&
&  0.77895E+00_JPRB, 0.73971E+00_JPRB, 0.71746E+00_JPRB, 0.73967E+00_JPRB, 0.82017E+00_JPRB, 0.76122E+00_JPRB, 0.79009E+00_JPRB ,&
&  0.79473E+00_JPRB, 0.78884E+00_JPRB, 0.78640E+00_JPRB, 0.79848E+00_JPRB, 0.83938E+00_JPRB, 0.89336E+00_JPRB, 0.91847E+00_JPRB ,&
&  0.92809E+00_JPRB, 0.87360E+00_JPRB, 0.83754E+00_JPRB, 0.10747E+01_JPRB, 0.16747E+01_JPRB, 0.23864E+01_JPRB, 0.26601E+01_JPRB ,&
&  0.26988E+01_JPRB, 0.26365E+01_JPRB, 0.26220E+01_JPRB, 0.27907E+01_JPRB, 0.25636E+01_JPRB, 0.22920E+01_JPRB, 0.21291E+01_JPRB ,&
&  0.20003E+01_JPRB, 0.19137E+01_JPRB, 0.18913E+01_JPRB, 0.17632E+01_JPRB, 0.16239E+01_JPRB, 0.15415E+01_JPRB, 0.14870E+01_JPRB ,&
&  0.14157E+01_JPRB, 0.13966E+01_JPRB, 0.13813E+01_JPRB, 0.13022E+01_JPRB, 0.12654E+01_JPRB, 0.12418E+01_JPRB, 0.13507E+01_JPRB ,&
&  0.14897E+01_JPRB, 0.11729E+01_JPRB, 0.99785E+00_JPRB, 0.10807E+01_JPRB, 0.85219E+00_JPRB, 0.82294E+00_JPRB, 0.90288E+00_JPRB ,&
&  0.84601E+00_JPRB, 0.81330E+00_JPRB, 0.82394E+00_JPRB, 0.79282E+00_JPRB, 0.81456E+00_JPRB, 0.87013E+00_JPRB, 0.86248E+00_JPRB ,&
&  0.82435E+00_JPRB, 0.80230E+00_JPRB, 0.92997E+00_JPRB, 0.88587E+00_JPRB, 0.86139E+00_JPRB, 0.81827E+00_JPRB, 0.78248E+00_JPRB ,&
&  0.76620E+00_JPRB, 0.75010E+00_JPRB, 0.73294E+00_JPRB, 0.72393E+00_JPRB, 0.72417E+00_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.73214E+00_JPRB, 0.71718E+00_JPRB, 0.74681E+00_JPRB, 0.72306E+00_JPRB, 0.71549E+00_JPRB, 0.70767E+00_JPRB, 0.69769E+00_JPRB ,&
&  0.67300E+00_JPRB, 0.66313E+00_JPRB, 0.65374E+00_JPRB, 0.63467E+00_JPRB, 0.61996E+00_JPRB, 0.60804E+00_JPRB, 0.60306E+00_JPRB ,&
&  0.59413E+00_JPRB, 0.60736E+00_JPRB, 0.56933E+00_JPRB, 0.56025E+00_JPRB, 0.54289E+00_JPRB, 0.56789E+00_JPRB, 0.59192E+00_JPRB ,&
&  0.63199E+00_JPRB, 0.67023E+00_JPRB, 0.71346E+00_JPRB, 0.77534E+00_JPRB, 0.84278E+00_JPRB, 0.91167E+00_JPRB, 0.10283E+01_JPRB ,&
&  0.10848E+01_JPRB, 0.10361E+01_JPRB, 0.97291E+00_JPRB, 0.99757E+00_JPRB, 0.10195E+01_JPRB, 0.94299E+00_JPRB, 0.95177E+00_JPRB ,&
&  0.93178E+00_JPRB, 0.92016E+00_JPRB, 0.92141E+00_JPRB, 0.95956E+00_JPRB, 0.96658E+00_JPRB, 0.90511E+00_JPRB, 0.84476E+00_JPRB ,&
&  0.82179E+00_JPRB, 0.84881E+00_JPRB, 0.10209E+01_JPRB, 0.14231E+01_JPRB, 0.21263E+01_JPRB, 0.29094E+01_JPRB, 0.30657E+01_JPRB ,&
&  0.30798E+01_JPRB, 0.28007E+01_JPRB, 0.21941E+01_JPRB, 0.23658E+01_JPRB, 0.27656E+01_JPRB, 0.25481E+01_JPRB, 0.22659E+01_JPRB ,&
&  0.22116E+01_JPRB, 0.20282E+01_JPRB, 0.19141E+01_JPRB, 0.17255E+01_JPRB, 0.15956E+01_JPRB, 0.14806E+01_JPRB, 0.13745E+01_JPRB ,&
&  0.12836E+01_JPRB, 0.12495E+01_JPRB, 0.12403E+01_JPRB, 0.12434E+01_JPRB, 0.12390E+01_JPRB, 0.13244E+01_JPRB, 0.14813E+01_JPRB ,&
&  0.14787E+01_JPRB, 0.13651E+01_JPRB, 0.10767E+01_JPRB, 0.10334E+01_JPRB, 0.80320E+00_JPRB, 0.77372E+00_JPRB, 0.83910E+00_JPRB ,&
&  0.81741E+00_JPRB, 0.87202E+00_JPRB, 0.93638E+00_JPRB, 0.91697E+00_JPRB, 0.10024E+01_JPRB, 0.10217E+01_JPRB, 0.10229E+01_JPRB ,&
&  0.99235E+00_JPRB, 0.91975E+00_JPRB, 0.10707E+01_JPRB, 0.10099E+01_JPRB, 0.96504E+00_JPRB, 0.90060E+00_JPRB, 0.84852E+00_JPRB ,&
&  0.83127E+00_JPRB, 0.80689E+00_JPRB, 0.78049E+00_JPRB, 0.76927E+00_JPRB, 0.75537E+00_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.72704E+00_JPRB, 0.71917E+00_JPRB, 0.75498E+00_JPRB, 0.73194E+00_JPRB, 0.73779E+00_JPRB, 0.70709E+00_JPRB, 0.69277E+00_JPRB ,&
&  0.68718E+00_JPRB, 0.67748E+00_JPRB, 0.65954E+00_JPRB, 0.63906E+00_JPRB, 0.62380E+00_JPRB, 0.61206E+00_JPRB, 0.61147E+00_JPRB ,&
&  0.60469E+00_JPRB, 0.59796E+00_JPRB, 0.54083E+00_JPRB, 0.51267E+00_JPRB, 0.50450E+00_JPRB, 0.54337E+00_JPRB, 0.61232E+00_JPRB ,&
&  0.69319E+00_JPRB, 0.76852E+00_JPRB, 0.88695E+00_JPRB, 0.99286E+00_JPRB, 0.10988E+01_JPRB, 0.11987E+01_JPRB, 0.13355E+01_JPRB ,&
&  0.14622E+01_JPRB, 0.12592E+01_JPRB, 0.12089E+01_JPRB, 0.11046E+01_JPRB, 0.10758E+01_JPRB, 0.10512E+01_JPRB, 0.10079E+01_JPRB ,&
&  0.97403E+00_JPRB, 0.96989E+00_JPRB, 0.96315E+00_JPRB, 0.91848E+00_JPRB, 0.86217E+00_JPRB, 0.82785E+00_JPRB, 0.82953E+00_JPRB ,&
&  0.87619E+00_JPRB, 0.98798E+00_JPRB, 0.13846E+01_JPRB, 0.19813E+01_JPRB, 0.29647E+01_JPRB, 0.36782E+01_JPRB, 0.35615E+01_JPRB ,&
&  0.27780E+01_JPRB, 0.25044E+01_JPRB, 0.32395E+01_JPRB, 0.27757E+01_JPRB, 0.27168E+01_JPRB, 0.27438E+01_JPRB, 0.24914E+01_JPRB ,&
&  0.21881E+01_JPRB, 0.18322E+01_JPRB, 0.17460E+01_JPRB, 0.16332E+01_JPRB, 0.20048E+01_JPRB, 0.17369E+01_JPRB, 0.15324E+01_JPRB ,&
&  0.14017E+01_JPRB, 0.13737E+01_JPRB, 0.13390E+01_JPRB, 0.13674E+01_JPRB, 0.14928E+01_JPRB, 0.17359E+01_JPRB, 0.17915E+01_JPRB ,&
&  0.16082E+01_JPRB, 0.15981E+01_JPRB, 0.13405E+01_JPRB, 0.10996E+01_JPRB, 0.93371E+00_JPRB, 0.91088E+00_JPRB, 0.91130E+00_JPRB ,&
&  0.92543E+00_JPRB, 0.94950E+00_JPRB, 0.10666E+01_JPRB, 0.12075E+01_JPRB, 0.13428E+01_JPRB, 0.13295E+01_JPRB, 0.11898E+01_JPRB ,&
&  0.11659E+01_JPRB, 0.13097E+01_JPRB, 0.13267E+01_JPRB, 0.11398E+01_JPRB, 0.10532E+01_JPRB, 0.97183E+00_JPRB, 0.91514E+00_JPRB ,&
&  0.86817E+00_JPRB, 0.83055E+00_JPRB, 0.80551E+00_JPRB, 0.78031E+00_JPRB, 0.75825E+00_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.75363E+00_JPRB, 0.75095E+00_JPRB, 0.74067E+00_JPRB, 0.73317E+00_JPRB, 0.72365E+00_JPRB, 0.67875E+00_JPRB, 0.65117E+00_JPRB ,&
&  0.63309E+00_JPRB, 0.61870E+00_JPRB, 0.62965E+00_JPRB, 0.63844E+00_JPRB, 0.63434E+00_JPRB, 0.62036E+00_JPRB, 0.61779E+00_JPRB ,&
&  0.61105E+00_JPRB, 0.56572E+00_JPRB, 0.51777E+00_JPRB, 0.52477E+00_JPRB, 0.52948E+00_JPRB, 0.53998E+00_JPRB, 0.62745E+00_JPRB ,&
&  0.74915E+00_JPRB, 0.89743E+00_JPRB, 0.10633E+01_JPRB, 0.12040E+01_JPRB, 0.12930E+01_JPRB, 0.13866E+01_JPRB, 0.15092E+01_JPRB ,&
&  0.14330E+01_JPRB, 0.14207E+01_JPRB, 0.13790E+01_JPRB, 0.12453E+01_JPRB, 0.11475E+01_JPRB, 0.10580E+01_JPRB, 0.10133E+01_JPRB ,&
&  0.99050E+00_JPRB, 0.10003E+01_JPRB, 0.96499E+00_JPRB, 0.90857E+00_JPRB, 0.92200E+00_JPRB, 0.93712E+00_JPRB, 0.94062E+00_JPRB ,&
&  0.94564E+00_JPRB, 0.12047E+01_JPRB, 0.17215E+01_JPRB, 0.24183E+01_JPRB, 0.25759E+01_JPRB, 0.32228E+01_JPRB, 0.32110E+01_JPRB ,&
&  0.36622E+01_JPRB, 0.35258E+01_JPRB, 0.32934E+01_JPRB, 0.35364E+01_JPRB, 0.28121E+01_JPRB, 0.26404E+01_JPRB, 0.25000E+01_JPRB ,&
&  0.22380E+01_JPRB, 0.20743E+01_JPRB, 0.22128E+01_JPRB, 0.20160E+01_JPRB, 0.21164E+01_JPRB, 0.23563E+01_JPRB, 0.21109E+01_JPRB ,&
&  0.19865E+01_JPRB, 0.18794E+01_JPRB, 0.17566E+01_JPRB, 0.18208E+01_JPRB, 0.18086E+01_JPRB, 0.18395E+01_JPRB, 0.15713E+01_JPRB ,&
&  0.14853E+01_JPRB, 0.12633E+01_JPRB, 0.13342E+01_JPRB, 0.11924E+01_JPRB, 0.11332E+01_JPRB, 0.10741E+01_JPRB, 0.10255E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.10754E+01_JPRB, 0.12943E+01_JPRB, 0.15306E+01_JPRB, 0.17431E+01_JPRB, 0.15770E+01_JPRB, 0.14601E+01_JPRB ,&
&  0.15696E+01_JPRB, 0.16527E+01_JPRB, 0.14378E+01_JPRB, 0.12339E+01_JPRB, 0.10877E+01_JPRB, 0.10160E+01_JPRB, 0.96322E+00_JPRB ,&
&  0.90879E+00_JPRB, 0.85877E+00_JPRB, 0.82949E+00_JPRB, 0.80488E+00_JPRB, 0.78234E+00_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.70760E+00_JPRB, 0.70770E+00_JPRB, 0.69826E+00_JPRB, 0.69720E+00_JPRB, 0.69736E+00_JPRB, 0.66072E+00_JPRB, 0.64556E+00_JPRB ,&
&  0.62272E+00_JPRB, 0.61287E+00_JPRB, 0.61805E+00_JPRB, 0.63350E+00_JPRB, 0.62499E+00_JPRB, 0.62429E+00_JPRB, 0.61791E+00_JPRB ,&
&  0.61204E+00_JPRB, 0.58173E+00_JPRB, 0.54119E+00_JPRB, 0.53732E+00_JPRB, 0.56430E+00_JPRB, 0.58101E+00_JPRB, 0.64092E+00_JPRB ,&
&  0.84661E+00_JPRB, 0.10550E+01_JPRB, 0.12120E+01_JPRB, 0.13331E+01_JPRB, 0.13922E+01_JPRB, 0.14392E+01_JPRB, 0.14488E+01_JPRB ,&
&  0.14241E+01_JPRB, 0.13101E+01_JPRB, 0.12214E+01_JPRB, 0.11542E+01_JPRB, 0.10941E+01_JPRB, 0.10445E+01_JPRB, 0.10007E+01_JPRB ,&
&  0.96816E+00_JPRB, 0.89878E+00_JPRB, 0.82820E+00_JPRB, 0.78371E+00_JPRB, 0.81317E+00_JPRB, 0.79874E+00_JPRB, 0.86247E+00_JPRB ,&
&  0.10419E+01_JPRB, 0.13386E+01_JPRB, 0.17538E+01_JPRB, 0.22793E+01_JPRB, 0.25485E+01_JPRB, 0.31767E+01_JPRB, 0.36707E+01_JPRB ,&
&  0.35622E+01_JPRB, 0.32656E+01_JPRB, 0.37069E+01_JPRB, 0.41820E+01_JPRB, 0.37084E+01_JPRB, 0.30813E+01_JPRB, 0.28093E+01_JPRB ,&
&  0.21568E+01_JPRB, 0.17396E+01_JPRB, 0.16160E+01_JPRB, 0.18173E+01_JPRB, 0.26182E+01_JPRB, 0.33024E+01_JPRB, 0.31504E+01_JPRB ,&
&  0.30529E+01_JPRB, 0.28541E+01_JPRB, 0.26195E+01_JPRB, 0.22660E+01_JPRB, 0.20356E+01_JPRB, 0.18468E+01_JPRB, 0.22511E+01_JPRB ,&
&  0.21776E+01_JPRB, 0.21214E+01_JPRB, 0.17754E+01_JPRB, 0.14198E+01_JPRB, 0.11555E+01_JPRB, 0.12065E+01_JPRB, 0.11745E+01_JPRB ,&
&  0.12030E+01_JPRB, 0.12682E+01_JPRB, 0.19426E+01_JPRB, 0.21945E+01_JPRB, 0.21256E+01_JPRB, 0.20470E+01_JPRB, 0.20757E+01_JPRB ,&
&  0.19031E+01_JPRB, 0.19611E+01_JPRB, 0.16452E+01_JPRB, 0.13041E+01_JPRB, 0.11005E+01_JPRB, 0.97112E+00_JPRB, 0.89038E+00_JPRB ,&
&  0.86513E+00_JPRB, 0.84315E+00_JPRB, 0.81098E+00_JPRB, 0.76765E+00_JPRB, 0.73941E+00_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.65900E+00_JPRB, 0.65811E+00_JPRB, 0.64363E+00_JPRB, 0.64584E+00_JPRB, 0.65218E+00_JPRB, 0.65349E+00_JPRB, 0.63910E+00_JPRB ,&
&  0.62062E+00_JPRB, 0.63412E+00_JPRB, 0.63699E+00_JPRB, 0.63305E+00_JPRB, 0.61969E+00_JPRB, 0.62777E+00_JPRB, 0.62902E+00_JPRB ,&
&  0.64284E+00_JPRB, 0.67048E+00_JPRB, 0.60233E+00_JPRB, 0.60231E+00_JPRB, 0.63505E+00_JPRB, 0.67646E+00_JPRB, 0.73983E+00_JPRB ,&
&  0.10032E+01_JPRB, 0.11650E+01_JPRB, 0.12182E+01_JPRB, 0.13249E+01_JPRB, 0.14509E+01_JPRB, 0.13212E+01_JPRB, 0.12854E+01_JPRB ,&
&  0.12787E+01_JPRB, 0.11786E+01_JPRB, 0.11148E+01_JPRB, 0.10667E+01_JPRB, 0.10481E+01_JPRB, 0.10374E+01_JPRB, 0.99040E+00_JPRB ,&
&  0.90973E+00_JPRB, 0.82646E+00_JPRB, 0.78996E+00_JPRB, 0.78356E+00_JPRB, 0.72877E+00_JPRB, 0.73743E+00_JPRB, 0.81340E+00_JPRB ,&
&  0.10607E+01_JPRB, 0.13760E+01_JPRB, 0.16079E+01_JPRB, 0.19713E+01_JPRB, 0.22985E+01_JPRB, 0.29994E+01_JPRB, 0.42780E+01_JPRB ,&
&  0.40089E+01_JPRB, 0.40143E+01_JPRB, 0.40413E+01_JPRB, 0.47958E+01_JPRB, 0.39241E+01_JPRB, 0.32823E+01_JPRB, 0.28723E+01_JPRB ,&
&  0.21265E+01_JPRB, 0.17854E+01_JPRB, 0.19813E+01_JPRB, 0.24660E+01_JPRB, 0.26788E+01_JPRB, 0.30478E+01_JPRB, 0.37358E+01_JPRB ,&
&  0.33472E+01_JPRB, 0.32249E+01_JPRB, 0.30877E+01_JPRB, 0.28872E+01_JPRB, 0.17607E+01_JPRB, 0.14286E+01_JPRB, 0.20093E+01_JPRB ,&
&  0.20700E+01_JPRB, 0.13517E+01_JPRB, 0.90030E+00_JPRB, 0.10938E+01_JPRB, 0.79941E+00_JPRB, 0.85100E+00_JPRB, 0.11069E+01_JPRB ,&
&  0.13181E+01_JPRB, 0.16521E+01_JPRB, 0.26606E+01_JPRB, 0.25452E+01_JPRB, 0.24188E+01_JPRB, 0.28255E+01_JPRB, 0.26407E+01_JPRB ,&
&  0.24548E+01_JPRB, 0.19832E+01_JPRB, 0.16495E+01_JPRB, 0.12283E+01_JPRB, 0.95894E+00_JPRB, 0.80808E+00_JPRB, 0.74654E+00_JPRB ,&
&  0.72268E+00_JPRB, 0.72316E+00_JPRB, 0.71427E+00_JPRB, 0.69149E+00_JPRB, 0.67549E+00_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.67034E+00_JPRB, 0.63405E+00_JPRB, 0.59142E+00_JPRB, 0.60000E+00_JPRB, 0.61360E+00_JPRB, 0.63272E+00_JPRB, 0.62164E+00_JPRB ,&
&  0.62777E+00_JPRB, 0.64658E+00_JPRB, 0.66875E+00_JPRB, 0.66824E+00_JPRB, 0.64409E+00_JPRB, 0.65468E+00_JPRB, 0.64549E+00_JPRB ,&
&  0.64977E+00_JPRB, 0.69304E+00_JPRB, 0.72096E+00_JPRB, 0.75263E+00_JPRB, 0.79905E+00_JPRB, 0.82437E+00_JPRB, 0.96847E+00_JPRB ,&
&  0.11506E+01_JPRB, 0.12003E+01_JPRB, 0.12432E+01_JPRB, 0.13136E+01_JPRB, 0.12934E+01_JPRB, 0.11349E+01_JPRB, 0.11939E+01_JPRB ,&
&  0.11801E+01_JPRB, 0.11147E+01_JPRB, 0.10436E+01_JPRB, 0.10040E+01_JPRB, 0.10030E+01_JPRB, 0.10006E+01_JPRB, 0.96279E+00_JPRB ,&
&  0.88264E+00_JPRB, 0.85511E+00_JPRB, 0.82373E+00_JPRB, 0.73039E+00_JPRB, 0.69648E+00_JPRB, 0.72306E+00_JPRB, 0.81576E+00_JPRB ,&
&  0.10486E+01_JPRB, 0.13077E+01_JPRB, 0.14495E+01_JPRB, 0.16812E+01_JPRB, 0.25064E+01_JPRB, 0.30608E+01_JPRB, 0.32531E+01_JPRB ,&
&  0.32810E+01_JPRB, 0.33455E+01_JPRB, 0.43519E+01_JPRB, 0.45668E+01_JPRB, 0.41812E+01_JPRB, 0.39243E+01_JPRB, 0.32322E+01_JPRB ,&
&  0.27721E+01_JPRB, 0.24767E+01_JPRB, 0.25182E+01_JPRB, 0.28615E+01_JPRB, 0.31921E+01_JPRB, 0.31209E+01_JPRB, 0.37499E+01_JPRB ,&
&  0.37989E+01_JPRB, 0.35780E+01_JPRB, 0.28296E+01_JPRB, 0.21529E+01_JPRB, 0.19602E+01_JPRB, 0.14414E+01_JPRB, 0.10050E+01_JPRB ,&
&  0.88289E+00_JPRB, 0.83682E+00_JPRB, 0.73713E+00_JPRB, 0.73836E+00_JPRB, 0.65619E+00_JPRB, 0.70948E+00_JPRB, 0.96045E+00_JPRB ,&
&  0.15832E+01_JPRB, 0.22695E+01_JPRB, 0.26362E+01_JPRB, 0.23741E+01_JPRB, 0.22982E+01_JPRB, 0.28345E+01_JPRB, 0.33998E+01_JPRB ,&
&  0.19839E+01_JPRB, 0.17571E+01_JPRB, 0.12491E+01_JPRB, 0.90477E+00_JPRB, 0.76301E+00_JPRB, 0.67820E+00_JPRB, 0.64589E+00_JPRB ,&
&  0.63837E+00_JPRB, 0.63187E+00_JPRB, 0.63597E+00_JPRB, 0.64645E+00_JPRB, 0.65352E+00_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.65524E+00_JPRB, 0.59427E+00_JPRB, 0.55341E+00_JPRB, 0.55132E+00_JPRB, 0.57855E+00_JPRB, 0.59423E+00_JPRB, 0.60344E+00_JPRB ,&
&  0.63090E+00_JPRB, 0.67115E+00_JPRB, 0.69292E+00_JPRB, 0.67933E+00_JPRB, 0.66957E+00_JPRB, 0.67261E+00_JPRB, 0.66275E+00_JPRB ,&
&  0.67645E+00_JPRB, 0.74236E+00_JPRB, 0.85592E+00_JPRB, 0.95247E+00_JPRB, 0.10579E+01_JPRB, 0.10287E+01_JPRB, 0.11787E+01_JPRB ,&
&  0.11871E+01_JPRB, 0.12100E+01_JPRB, 0.13292E+01_JPRB, 0.13261E+01_JPRB, 0.12830E+01_JPRB, 0.11755E+01_JPRB, 0.11828E+01_JPRB ,&
&  0.11066E+01_JPRB, 0.10687E+01_JPRB, 0.99183E+00_JPRB, 0.99589E+00_JPRB, 0.99889E+00_JPRB, 0.98093E+00_JPRB, 0.93640E+00_JPRB ,&
&  0.88649E+00_JPRB, 0.89578E+00_JPRB, 0.77286E+00_JPRB, 0.74766E+00_JPRB, 0.75140E+00_JPRB, 0.80467E+00_JPRB, 0.90244E+00_JPRB ,&
&  0.10925E+01_JPRB, 0.12915E+01_JPRB, 0.14533E+01_JPRB, 0.18860E+01_JPRB, 0.22399E+01_JPRB, 0.22259E+01_JPRB, 0.29399E+01_JPRB ,&
&  0.35549E+01_JPRB, 0.39688E+01_JPRB, 0.40057E+01_JPRB, 0.46460E+01_JPRB, 0.45270E+01_JPRB, 0.39731E+01_JPRB, 0.35389E+01_JPRB ,&
&  0.30101E+01_JPRB, 0.30306E+01_JPRB, 0.34560E+01_JPRB, 0.36570E+01_JPRB, 0.43552E+01_JPRB, 0.40806E+01_JPRB, 0.42464E+01_JPRB ,&
&  0.43857E+01_JPRB, 0.40073E+01_JPRB, 0.32517E+01_JPRB, 0.25798E+01_JPRB, 0.27905E+01_JPRB, 0.24176E+01_JPRB, 0.12990E+01_JPRB ,&
&  0.10695E+01_JPRB, 0.93112E+00_JPRB, 0.74545E+00_JPRB, 0.64815E+00_JPRB, 0.60476E+00_JPRB, 0.69251E+00_JPRB, 0.11067E+01_JPRB ,&
&  0.13947E+01_JPRB, 0.23796E+01_JPRB, 0.27568E+01_JPRB, 0.25236E+01_JPRB, 0.23159E+01_JPRB, 0.29184E+01_JPRB, 0.28920E+01_JPRB ,&
&  0.20935E+01_JPRB, 0.16043E+01_JPRB, 0.92503E+00_JPRB, 0.72618E+00_JPRB, 0.68077E+00_JPRB, 0.64701E+00_JPRB, 0.61169E+00_JPRB ,&
&  0.57757E+00_JPRB, 0.57318E+00_JPRB, 0.58840E+00_JPRB, 0.60756E+00_JPRB, 0.62250E+00_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.57176E+00_JPRB, 0.53049E+00_JPRB, 0.52276E+00_JPRB, 0.51940E+00_JPRB, 0.56066E+00_JPRB, 0.60335E+00_JPRB, 0.60950E+00_JPRB ,&
&  0.65170E+00_JPRB, 0.69145E+00_JPRB, 0.71167E+00_JPRB, 0.70178E+00_JPRB, 0.68568E+00_JPRB, 0.68970E+00_JPRB, 0.71068E+00_JPRB ,&
&  0.76264E+00_JPRB, 0.86751E+00_JPRB, 0.10231E+01_JPRB, 0.11848E+01_JPRB, 0.12460E+01_JPRB, 0.12306E+01_JPRB, 0.12365E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.12120E+01_JPRB, 0.13754E+01_JPRB, 0.14281E+01_JPRB, 0.12946E+01_JPRB, 0.12549E+01_JPRB, 0.11647E+01_JPRB ,&
&  0.10825E+01_JPRB, 0.10624E+01_JPRB, 0.10297E+01_JPRB, 0.10334E+01_JPRB, 0.10131E+01_JPRB, 0.98473E+00_JPRB, 0.96571E+00_JPRB ,&
&  0.94008E+00_JPRB, 0.83353E+00_JPRB, 0.77579E+00_JPRB, 0.77158E+00_JPRB, 0.85237E+00_JPRB, 0.97436E+00_JPRB, 0.11057E+01_JPRB ,&
&  0.12716E+01_JPRB, 0.14500E+01_JPRB, 0.16897E+01_JPRB, 0.21403E+01_JPRB, 0.19039E+01_JPRB, 0.24985E+01_JPRB, 0.29873E+01_JPRB ,&
&  0.32549E+01_JPRB, 0.34930E+01_JPRB, 0.34675E+01_JPRB, 0.40023E+01_JPRB, 0.42911E+01_JPRB, 0.39012E+01_JPRB, 0.33674E+01_JPRB ,&
&  0.32775E+01_JPRB, 0.32562E+01_JPRB, 0.42842E+01_JPRB, 0.48013E+01_JPRB, 0.52759E+01_JPRB, 0.67191E+01_JPRB, 0.49442E+01_JPRB ,&
&  0.41668E+01_JPRB, 0.43212E+01_JPRB, 0.36715E+01_JPRB, 0.29150E+01_JPRB, 0.29296E+01_JPRB, 0.30607E+01_JPRB, 0.27486E+01_JPRB ,&
&  0.13292E+01_JPRB, 0.86595E+00_JPRB, 0.79771E+00_JPRB, 0.71581E+00_JPRB, 0.60673E+00_JPRB, 0.68161E+00_JPRB, 0.15677E+01_JPRB ,&
&  0.13560E+01_JPRB, 0.21991E+01_JPRB, 0.25208E+01_JPRB, 0.22781E+01_JPRB, 0.22721E+01_JPRB, 0.25037E+01_JPRB, 0.27875E+01_JPRB ,&
&  0.15510E+01_JPRB, 0.11069E+01_JPRB, 0.79437E+00_JPRB, 0.66676E+00_JPRB, 0.64712E+00_JPRB, 0.63172E+00_JPRB, 0.59757E+00_JPRB ,&
&  0.56227E+00_JPRB, 0.54348E+00_JPRB, 0.55354E+00_JPRB, 0.55029E+00_JPRB, 0.57565E+00_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.52816E+00_JPRB, 0.52648E+00_JPRB, 0.56641E+00_JPRB, 0.59499E+00_JPRB, 0.61669E+00_JPRB, 0.62823E+00_JPRB, 0.65050E+00_JPRB ,&
&  0.69770E+00_JPRB, 0.73508E+00_JPRB, 0.76163E+00_JPRB, 0.75628E+00_JPRB, 0.74769E+00_JPRB, 0.78238E+00_JPRB, 0.84579E+00_JPRB ,&
&  0.93104E+00_JPRB, 0.10520E+01_JPRB, 0.11971E+01_JPRB, 0.12946E+01_JPRB, 0.13315E+01_JPRB, 0.12133E+01_JPRB, 0.11155E+01_JPRB ,&
&  0.10700E+01_JPRB, 0.11635E+01_JPRB, 0.12181E+01_JPRB, 0.12873E+01_JPRB, 0.13031E+01_JPRB, 0.12289E+01_JPRB, 0.11363E+01_JPRB ,&
&  0.10921E+01_JPRB, 0.10673E+01_JPRB, 0.10291E+01_JPRB, 0.10313E+01_JPRB, 0.10411E+01_JPRB, 0.10508E+01_JPRB, 0.10155E+01_JPRB ,&
&  0.90997E+00_JPRB, 0.86464E+00_JPRB, 0.84500E+00_JPRB, 0.90407E+00_JPRB, 0.10649E+01_JPRB, 0.12380E+01_JPRB, 0.14072E+01_JPRB ,&
&  0.15814E+01_JPRB, 0.17437E+01_JPRB, 0.19517E+01_JPRB, 0.20869E+01_JPRB, 0.23350E+01_JPRB, 0.26227E+01_JPRB, 0.28609E+01_JPRB ,&
&  0.29791E+01_JPRB, 0.32793E+01_JPRB, 0.35747E+01_JPRB, 0.37934E+01_JPRB, 0.39236E+01_JPRB, 0.36980E+01_JPRB, 0.33414E+01_JPRB ,&
&  0.35070E+01_JPRB, 0.40645E+01_JPRB, 0.48830E+01_JPRB, 0.52731E+01_JPRB, 0.55632E+01_JPRB, 0.67227E+01_JPRB, 0.66048E+01_JPRB ,&
&  0.51719E+01_JPRB, 0.38190E+01_JPRB, 0.32639E+01_JPRB, 0.31177E+01_JPRB, 0.31580E+01_JPRB, 0.32227E+01_JPRB, 0.39258E+01_JPRB ,&
&  0.35500E+01_JPRB, 0.17296E+01_JPRB, 0.11171E+01_JPRB, 0.11453E+01_JPRB, 0.68537E+00_JPRB, 0.61706E+00_JPRB, 0.98373E+00_JPRB ,&
&  0.13426E+01_JPRB, 0.19738E+01_JPRB, 0.23068E+01_JPRB, 0.19442E+01_JPRB, 0.20342E+01_JPRB, 0.18993E+01_JPRB, 0.17414E+01_JPRB ,&
&  0.11054E+01_JPRB, 0.84448E+00_JPRB, 0.67604E+00_JPRB, 0.62535E+00_JPRB, 0.60360E+00_JPRB, 0.61243E+00_JPRB, 0.61142E+00_JPRB ,&
&  0.56705E+00_JPRB, 0.51811E+00_JPRB, 0.51914E+00_JPRB, 0.52582E+00_JPRB, 0.52316E+00_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.57087E+00_JPRB, 0.60034E+00_JPRB, 0.66404E+00_JPRB, 0.71817E+00_JPRB, 0.74077E+00_JPRB, 0.74368E+00_JPRB, 0.73686E+00_JPRB ,&
&  0.76305E+00_JPRB, 0.81109E+00_JPRB, 0.83881E+00_JPRB, 0.85779E+00_JPRB, 0.89382E+00_JPRB, 0.95534E+00_JPRB, 0.10240E+01_JPRB ,&
&  0.11002E+01_JPRB, 0.11874E+01_JPRB, 0.12821E+01_JPRB, 0.13769E+01_JPRB, 0.14938E+01_JPRB, 0.15325E+01_JPRB, 0.97658E+00_JPRB ,&
&  0.91320E+00_JPRB, 0.11756E+01_JPRB, 0.11516E+01_JPRB, 0.12134E+01_JPRB, 0.12418E+01_JPRB, 0.12603E+01_JPRB, 0.11394E+01_JPRB ,&
&  0.10718E+01_JPRB, 0.10361E+01_JPRB, 0.10128E+01_JPRB, 0.10364E+01_JPRB, 0.10885E+01_JPRB, 0.11394E+01_JPRB, 0.11141E+01_JPRB ,&
&  0.10995E+01_JPRB, 0.11231E+01_JPRB, 0.11668E+01_JPRB, 0.13154E+01_JPRB, 0.15226E+01_JPRB, 0.16888E+01_JPRB, 0.18041E+01_JPRB ,&
&  0.18959E+01_JPRB, 0.20052E+01_JPRB, 0.21609E+01_JPRB, 0.22753E+01_JPRB, 0.25391E+01_JPRB, 0.26934E+01_JPRB, 0.28555E+01_JPRB ,&
&  0.28814E+01_JPRB, 0.28463E+01_JPRB, 0.32439E+01_JPRB, 0.35331E+01_JPRB, 0.35552E+01_JPRB, 0.33336E+01_JPRB, 0.32636E+01_JPRB ,&
&  0.40172E+01_JPRB, 0.49912E+01_JPRB, 0.60970E+01_JPRB, 0.54452E+01_JPRB, 0.52265E+01_JPRB, 0.57994E+01_JPRB, 0.62709E+01_JPRB ,&
&  0.51730E+01_JPRB, 0.40172E+01_JPRB, 0.36351E+01_JPRB, 0.32756E+01_JPRB, 0.32379E+01_JPRB, 0.31611E+01_JPRB, 0.34397E+01_JPRB ,&
&  0.34267E+01_JPRB, 0.25502E+01_JPRB, 0.15257E+01_JPRB, 0.89257E+00_JPRB, 0.66519E+00_JPRB, 0.69998E+00_JPRB, 0.89114E+00_JPRB ,&
&  0.14147E+01_JPRB, 0.18026E+01_JPRB, 0.20931E+01_JPRB, 0.22337E+01_JPRB, 0.15506E+01_JPRB, 0.12923E+01_JPRB, 0.10598E+01_JPRB ,&
&  0.71248E+00_JPRB, 0.60527E+00_JPRB, 0.59851E+00_JPRB, 0.59270E+00_JPRB, 0.57569E+00_JPRB, 0.59797E+00_JPRB, 0.62499E+00_JPRB ,&
&  0.58727E+00_JPRB, 0.52623E+00_JPRB, 0.52842E+00_JPRB, 0.53284E+00_JPRB, 0.54503E+00_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.63921E+00_JPRB, 0.68801E+00_JPRB, 0.75085E+00_JPRB, 0.83988E+00_JPRB, 0.92618E+00_JPRB, 0.98137E+00_JPRB, 0.91387E+00_JPRB ,&
&  0.84679E+00_JPRB, 0.86824E+00_JPRB, 0.90570E+00_JPRB, 0.95181E+00_JPRB, 0.10052E+01_JPRB, 0.10635E+01_JPRB, 0.11178E+01_JPRB ,&
&  0.11733E+01_JPRB, 0.12359E+01_JPRB, 0.12997E+01_JPRB, 0.13828E+01_JPRB, 0.14657E+01_JPRB, 0.14962E+01_JPRB, 0.11006E+01_JPRB ,&
&  0.86070E+00_JPRB, 0.13993E+01_JPRB, 0.11536E+01_JPRB, 0.12115E+01_JPRB, 0.12316E+01_JPRB, 0.13119E+01_JPRB, 0.11682E+01_JPRB ,&
&  0.10361E+01_JPRB, 0.10324E+01_JPRB, 0.10505E+01_JPRB, 0.10373E+01_JPRB, 0.10846E+01_JPRB, 0.11679E+01_JPRB, 0.12808E+01_JPRB ,&
&  0.13411E+01_JPRB, 0.14302E+01_JPRB, 0.15593E+01_JPRB, 0.17364E+01_JPRB, 0.18700E+01_JPRB, 0.19818E+01_JPRB, 0.20682E+01_JPRB ,&
&  0.21509E+01_JPRB, 0.22599E+01_JPRB, 0.23409E+01_JPRB, 0.24999E+01_JPRB, 0.25950E+01_JPRB, 0.27622E+01_JPRB, 0.29102E+01_JPRB ,&
&  0.28937E+01_JPRB, 0.28347E+01_JPRB, 0.30207E+01_JPRB, 0.30089E+01_JPRB, 0.30421E+01_JPRB, 0.34612E+01_JPRB, 0.36568E+01_JPRB ,&
&  0.46055E+01_JPRB, 0.50533E+01_JPRB, 0.60336E+01_JPRB, 0.48273E+01_JPRB, 0.43578E+01_JPRB, 0.45356E+01_JPRB, 0.41742E+01_JPRB ,&
&  0.34745E+01_JPRB, 0.33299E+01_JPRB, 0.31780E+01_JPRB, 0.31401E+01_JPRB, 0.31265E+01_JPRB, 0.30599E+01_JPRB, 0.29362E+01_JPRB ,&
&  0.22716E+01_JPRB, 0.12972E+01_JPRB, 0.95702E+00_JPRB, 0.72880E+00_JPRB, 0.60556E+00_JPRB, 0.66652E+00_JPRB, 0.10213E+01_JPRB ,&
&  0.16356E+01_JPRB, 0.19203E+01_JPRB, 0.20755E+01_JPRB, 0.15660E+01_JPRB, 0.10486E+01_JPRB, 0.80552E+00_JPRB, 0.64081E+00_JPRB ,&
&  0.50754E+00_JPRB, 0.49638E+00_JPRB, 0.53510E+00_JPRB, 0.54637E+00_JPRB, 0.53714E+00_JPRB, 0.55353E+00_JPRB, 0.56645E+00_JPRB ,&
&  0.55544E+00_JPRB, 0.55135E+00_JPRB, 0.57976E+00_JPRB, 0.58058E+00_JPRB, 0.60324E+00_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.71129E+00_JPRB, 0.77375E+00_JPRB, 0.87517E+00_JPRB, 0.10112E+01_JPRB, 0.11929E+01_JPRB, 0.13600E+01_JPRB, 0.13888E+01_JPRB ,&
&  0.11311E+01_JPRB, 0.88531E+00_JPRB, 0.95481E+00_JPRB, 0.10272E+01_JPRB, 0.10936E+01_JPRB, 0.11643E+01_JPRB, 0.12250E+01_JPRB ,&
&  0.13037E+01_JPRB, 0.13853E+01_JPRB, 0.14721E+01_JPRB, 0.15761E+01_JPRB, 0.16839E+01_JPRB, 0.17759E+01_JPRB, 0.16358E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.10519E+01_JPRB, 0.13523E+01_JPRB, 0.12441E+01_JPRB, 0.11777E+01_JPRB, 0.12519E+01_JPRB, 0.12501E+01_JPRB ,&
&  0.11437E+01_JPRB, 0.10619E+01_JPRB, 0.10998E+01_JPRB, 0.10530E+01_JPRB, 0.95628E+00_JPRB, 0.10760E+01_JPRB, 0.12769E+01_JPRB ,&
&  0.14043E+01_JPRB, 0.15286E+01_JPRB, 0.16816E+01_JPRB, 0.18488E+01_JPRB, 0.19703E+01_JPRB, 0.21198E+01_JPRB, 0.22489E+01_JPRB ,&
&  0.23526E+01_JPRB, 0.24702E+01_JPRB, 0.25131E+01_JPRB, 0.25289E+01_JPRB, 0.26404E+01_JPRB, 0.27783E+01_JPRB, 0.28521E+01_JPRB ,&
&  0.28723E+01_JPRB, 0.30126E+01_JPRB, 0.31284E+01_JPRB, 0.32005E+01_JPRB, 0.32082E+01_JPRB, 0.37468E+01_JPRB, 0.40641E+01_JPRB ,&
&  0.45000E+01_JPRB, 0.45229E+01_JPRB, 0.49157E+01_JPRB, 0.38257E+01_JPRB, 0.34133E+01_JPRB, 0.31673E+01_JPRB, 0.29486E+01_JPRB ,&
&  0.27649E+01_JPRB, 0.27929E+01_JPRB, 0.27470E+01_JPRB, 0.27818E+01_JPRB, 0.28923E+01_JPRB, 0.26477E+01_JPRB, 0.24036E+01_JPRB ,&
&  0.14506E+01_JPRB, 0.10729E+01_JPRB, 0.79821E+00_JPRB, 0.70743E+00_JPRB, 0.63439E+00_JPRB, 0.73208E+00_JPRB, 0.98927E+00_JPRB ,&
&  0.13860E+01_JPRB, 0.13071E+01_JPRB, 0.11824E+01_JPRB, 0.10874E+01_JPRB, 0.74006E+00_JPRB, 0.56907E+00_JPRB, 0.48913E+00_JPRB ,&
&  0.45971E+00_JPRB, 0.47353E+00_JPRB, 0.49864E+00_JPRB, 0.50346E+00_JPRB, 0.49287E+00_JPRB, 0.50830E+00_JPRB, 0.53725E+00_JPRB ,&
&  0.55729E+00_JPRB, 0.57775E+00_JPRB, 0.61436E+00_JPRB, 0.63046E+00_JPRB, 0.67209E+00_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.69573E+00_JPRB, 0.78003E+00_JPRB, 0.84305E+00_JPRB, 0.90074E+00_JPRB, 0.92384E+00_JPRB, 0.96864E+00_JPRB, 0.94918E+00_JPRB ,&
&  0.86885E+00_JPRB, 0.81042E+00_JPRB, 0.87109E+00_JPRB, 0.95202E+00_JPRB, 0.10271E+01_JPRB, 0.10730E+01_JPRB, 0.11190E+01_JPRB ,&
&  0.11758E+01_JPRB, 0.12337E+01_JPRB, 0.12991E+01_JPRB, 0.13622E+01_JPRB, 0.14485E+01_JPRB, 0.15426E+01_JPRB, 0.15733E+01_JPRB ,&
&  0.13115E+01_JPRB, 0.97774E+00_JPRB, 0.12132E+01_JPRB, 0.13446E+01_JPRB, 0.11312E+01_JPRB, 0.11527E+01_JPRB, 0.12170E+01_JPRB ,&
&  0.11009E+01_JPRB, 0.10423E+01_JPRB, 0.10570E+01_JPRB, 0.10417E+01_JPRB, 0.89677E+00_JPRB, 0.98082E+00_JPRB, 0.11558E+01_JPRB ,&
&  0.13196E+01_JPRB, 0.14804E+01_JPRB, 0.16431E+01_JPRB, 0.17815E+01_JPRB, 0.19211E+01_JPRB, 0.21032E+01_JPRB, 0.22710E+01_JPRB ,&
&  0.24160E+01_JPRB, 0.25185E+01_JPRB, 0.25810E+01_JPRB, 0.26374E+01_JPRB, 0.27065E+01_JPRB, 0.28259E+01_JPRB, 0.28642E+01_JPRB ,&
&  0.29391E+01_JPRB, 0.29914E+01_JPRB, 0.32451E+01_JPRB, 0.35621E+01_JPRB, 0.37927E+01_JPRB, 0.36656E+01_JPRB, 0.38297E+01_JPRB ,&
&  0.38340E+01_JPRB, 0.36352E+01_JPRB, 0.36398E+01_JPRB, 0.35945E+01_JPRB, 0.23940E+01_JPRB, 0.21946E+01_JPRB, 0.22032E+01_JPRB ,&
&  0.24839E+01_JPRB, 0.23988E+01_JPRB, 0.23365E+01_JPRB, 0.23546E+01_JPRB, 0.22721E+01_JPRB, 0.20664E+01_JPRB, 0.17023E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.95987E+00_JPRB, 0.81691E+00_JPRB, 0.78233E+00_JPRB, 0.73388E+00_JPRB, 0.91731E+00_JPRB, 0.87313E+00_JPRB ,&
&  0.10835E+01_JPRB, 0.95735E+00_JPRB, 0.88014E+00_JPRB, 0.92460E+00_JPRB, 0.75247E+00_JPRB, 0.56567E+00_JPRB, 0.49320E+00_JPRB ,&
&  0.46752E+00_JPRB, 0.47893E+00_JPRB, 0.48303E+00_JPRB, 0.47647E+00_JPRB, 0.47183E+00_JPRB, 0.49214E+00_JPRB, 0.51550E+00_JPRB ,&
&  0.53342E+00_JPRB, 0.56065E+00_JPRB, 0.58816E+00_JPRB, 0.61947E+00_JPRB, 0.66416E+00_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.57987E+00_JPRB, 0.60479E+00_JPRB, 0.63838E+00_JPRB, 0.67941E+00_JPRB, 0.70372E+00_JPRB, 0.71979E+00_JPRB, 0.71113E+00_JPRB ,&
&  0.72338E+00_JPRB, 0.74033E+00_JPRB, 0.77349E+00_JPRB, 0.80498E+00_JPRB, 0.82229E+00_JPRB, 0.86509E+00_JPRB, 0.89824E+00_JPRB ,&
&  0.90842E+00_JPRB, 0.90815E+00_JPRB, 0.96245E+00_JPRB, 0.10468E+01_JPRB, 0.11991E+01_JPRB, 0.14388E+01_JPRB, 0.16820E+01_JPRB ,&
&  0.18942E+01_JPRB, 0.18944E+01_JPRB, 0.18530E+01_JPRB, 0.12293E+01_JPRB, 0.84911E+00_JPRB, 0.95991E+00_JPRB, 0.10269E+01_JPRB ,&
&  0.10734E+01_JPRB, 0.10817E+01_JPRB, 0.99707E+00_JPRB, 0.93556E+00_JPRB, 0.91176E+00_JPRB, 0.93660E+00_JPRB, 0.10253E+01_JPRB ,&
&  0.11481E+01_JPRB, 0.12933E+01_JPRB, 0.14578E+01_JPRB, 0.15880E+01_JPRB, 0.17304E+01_JPRB, 0.19142E+01_JPRB, 0.21121E+01_JPRB ,&
&  0.22804E+01_JPRB, 0.24128E+01_JPRB, 0.25061E+01_JPRB, 0.26566E+01_JPRB, 0.27496E+01_JPRB, 0.28952E+01_JPRB, 0.30013E+01_JPRB ,&
&  0.31206E+01_JPRB, 0.31658E+01_JPRB, 0.33708E+01_JPRB, 0.36070E+01_JPRB, 0.36189E+01_JPRB, 0.32669E+01_JPRB, 0.31240E+01_JPRB ,&
&  0.30780E+01_JPRB, 0.28343E+01_JPRB, 0.27470E+01_JPRB, 0.28374E+01_JPRB, 0.18729E+01_JPRB, 0.19064E+01_JPRB, 0.22043E+01_JPRB ,&
&  0.21532E+01_JPRB, 0.20977E+01_JPRB, 0.19167E+01_JPRB, 0.16742E+01_JPRB, 0.15864E+01_JPRB, 0.14185E+01_JPRB, 0.12624E+01_JPRB ,&
&  0.11642E+01_JPRB, 0.97760E+00_JPRB, 0.83492E+00_JPRB, 0.70972E+00_JPRB, 0.74835E+00_JPRB, 0.94065E+00_JPRB, 0.76796E+00_JPRB ,&
&  0.74152E+00_JPRB, 0.88729E+00_JPRB, 0.79920E+00_JPRB, 0.99657E+00_JPRB, 0.10619E+01_JPRB, 0.71357E+00_JPRB, 0.55336E+00_JPRB ,&
&  0.47261E+00_JPRB, 0.46424E+00_JPRB, 0.47445E+00_JPRB, 0.46727E+00_JPRB, 0.47146E+00_JPRB, 0.49261E+00_JPRB, 0.49599E+00_JPRB ,&
&  0.52615E+00_JPRB, 0.54020E+00_JPRB, 0.55619E+00_JPRB, 0.57201E+00_JPRB, 0.56610E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.47382E+00_JPRB, 0.47615E+00_JPRB, 0.48496E+00_JPRB, 0.52197E+00_JPRB, 0.53973E+00_JPRB, 0.54742E+00_JPRB, 0.57072E+00_JPRB ,&
&  0.59841E+00_JPRB, 0.64988E+00_JPRB, 0.68499E+00_JPRB, 0.69903E+00_JPRB, 0.67590E+00_JPRB, 0.65152E+00_JPRB, 0.64185E+00_JPRB ,&
&  0.64260E+00_JPRB, 0.60415E+00_JPRB, 0.58568E+00_JPRB, 0.61994E+00_JPRB, 0.68695E+00_JPRB, 0.80292E+00_JPRB, 0.93553E+00_JPRB ,&
&  0.11125E+01_JPRB, 0.12998E+01_JPRB, 0.15423E+01_JPRB, 0.21198E+01_JPRB, 0.22918E+01_JPRB, 0.10902E+01_JPRB, 0.11013E+01_JPRB ,&
&  0.13620E+01_JPRB, 0.13773E+01_JPRB, 0.11120E+01_JPRB, 0.98179E+00_JPRB, 0.92459E+00_JPRB, 0.89960E+00_JPRB, 0.91878E+00_JPRB ,&
&  0.99945E+00_JPRB, 0.10849E+01_JPRB, 0.12045E+01_JPRB, 0.13574E+01_JPRB, 0.15191E+01_JPRB, 0.16683E+01_JPRB, 0.18530E+01_JPRB ,&
&  0.20121E+01_JPRB, 0.20870E+01_JPRB, 0.21792E+01_JPRB, 0.23012E+01_JPRB, 0.24896E+01_JPRB, 0.27387E+01_JPRB, 0.29675E+01_JPRB ,&
&  0.31450E+01_JPRB, 0.32218E+01_JPRB, 0.34252E+01_JPRB, 0.35389E+01_JPRB, 0.32324E+01_JPRB, 0.29184E+01_JPRB, 0.26370E+01_JPRB ,&
&  0.23766E+01_JPRB, 0.20344E+01_JPRB, 0.17816E+01_JPRB, 0.21308E+01_JPRB, 0.22239E+01_JPRB, 0.19659E+01_JPRB, 0.18726E+01_JPRB ,&
&  0.19206E+01_JPRB, 0.18577E+01_JPRB, 0.16881E+01_JPRB, 0.13752E+01_JPRB, 0.11949E+01_JPRB, 0.12511E+01_JPRB, 0.11018E+01_JPRB ,&
&  0.10551E+01_JPRB, 0.89254E+00_JPRB, 0.77857E+00_JPRB, 0.69976E+00_JPRB, 0.64779E+00_JPRB, 0.75859E+00_JPRB, 0.62943E+00_JPRB ,&
&  0.55728E+00_JPRB, 0.74284E+00_JPRB, 0.69810E+00_JPRB, 0.10458E+01_JPRB, 0.19630E+01_JPRB, 0.99826E+00_JPRB, 0.61172E+00_JPRB ,&
&  0.47339E+00_JPRB, 0.46937E+00_JPRB, 0.48560E+00_JPRB, 0.49024E+00_JPRB, 0.48743E+00_JPRB, 0.49874E+00_JPRB, 0.49993E+00_JPRB ,&
&  0.50678E+00_JPRB, 0.50148E+00_JPRB, 0.50398E+00_JPRB, 0.49140E+00_JPRB, 0.46762E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.54854E+00_JPRB, 0.54950E+00_JPRB, 0.53790E+00_JPRB, 0.52967E+00_JPRB, 0.52894E+00_JPRB, 0.53159E+00_JPRB, 0.53762E+00_JPRB ,&
&  0.54942E+00_JPRB, 0.56923E+00_JPRB, 0.59014E+00_JPRB, 0.62095E+00_JPRB, 0.63043E+00_JPRB, 0.60807E+00_JPRB, 0.61915E+00_JPRB ,&
&  0.61873E+00_JPRB, 0.63485E+00_JPRB, 0.62946E+00_JPRB, 0.70707E+00_JPRB, 0.73434E+00_JPRB, 0.74586E+00_JPRB, 0.83515E+00_JPRB ,&
&  0.82465E+00_JPRB, 0.91068E+00_JPRB, 0.97991E+00_JPRB, 0.11046E+01_JPRB, 0.18392E+01_JPRB, 0.16751E+01_JPRB, 0.11483E+01_JPRB ,&
&  0.11502E+01_JPRB, 0.11685E+01_JPRB, 0.10507E+01_JPRB, 0.93150E+00_JPRB, 0.89031E+00_JPRB, 0.82209E+00_JPRB, 0.80302E+00_JPRB ,&
&  0.84543E+00_JPRB, 0.90500E+00_JPRB, 0.10276E+01_JPRB, 0.11499E+01_JPRB, 0.12522E+01_JPRB, 0.13514E+01_JPRB, 0.14701E+01_JPRB ,&
&  0.15515E+01_JPRB, 0.16286E+01_JPRB, 0.16526E+01_JPRB, 0.16573E+01_JPRB, 0.18967E+01_JPRB, 0.21414E+01_JPRB, 0.23307E+01_JPRB ,&
&  0.24776E+01_JPRB, 0.26158E+01_JPRB, 0.28516E+01_JPRB, 0.27416E+01_JPRB, 0.24820E+01_JPRB, 0.23228E+01_JPRB, 0.19514E+01_JPRB ,&
&  0.15931E+01_JPRB, 0.12547E+01_JPRB, 0.11126E+01_JPRB, 0.13752E+01_JPRB, 0.14705E+01_JPRB, 0.13247E+01_JPRB, 0.15248E+01_JPRB ,&
&  0.15803E+01_JPRB, 0.16018E+01_JPRB, 0.14505E+01_JPRB, 0.13055E+01_JPRB, 0.11411E+01_JPRB, 0.11194E+01_JPRB, 0.10247E+01_JPRB ,&
&  0.91882E+00_JPRB, 0.77875E+00_JPRB, 0.69110E+00_JPRB, 0.63035E+00_JPRB, 0.58104E+00_JPRB, 0.63651E+00_JPRB, 0.67021E+00_JPRB ,&
&  0.71331E+00_JPRB, 0.64299E+00_JPRB, 0.65035E+00_JPRB, 0.94124E+00_JPRB, 0.95485E+00_JPRB, 0.60030E+00_JPRB, 0.48069E+00_JPRB ,&
&  0.46988E+00_JPRB, 0.47993E+00_JPRB, 0.48487E+00_JPRB, 0.48696E+00_JPRB, 0.49932E+00_JPRB, 0.51162E+00_JPRB, 0.50228E+00_JPRB ,&
&  0.50403E+00_JPRB, 0.49956E+00_JPRB, 0.49684E+00_JPRB, 0.50335E+00_JPRB, 0.52004E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.68744E+00_JPRB, 0.69081E+00_JPRB, 0.69511E+00_JPRB, 0.70266E+00_JPRB, 0.71774E+00_JPRB, 0.73386E+00_JPRB, 0.74399E+00_JPRB ,&
&  0.76786E+00_JPRB, 0.78843E+00_JPRB, 0.81104E+00_JPRB, 0.81903E+00_JPRB, 0.83629E+00_JPRB, 0.85273E+00_JPRB, 0.85925E+00_JPRB ,&
&  0.87965E+00_JPRB, 0.90397E+00_JPRB, 0.90620E+00_JPRB, 0.91756E+00_JPRB, 0.94638E+00_JPRB, 0.96554E+00_JPRB, 0.94832E+00_JPRB ,&
&  0.92664E+00_JPRB, 0.89988E+00_JPRB, 0.91944E+00_JPRB, 0.95715E+00_JPRB, 0.11472E+01_JPRB, 0.12613E+01_JPRB, 0.12432E+01_JPRB ,&
&  0.10680E+01_JPRB, 0.10404E+01_JPRB, 0.11731E+01_JPRB, 0.97552E+00_JPRB, 0.89555E+00_JPRB, 0.85143E+00_JPRB, 0.80409E+00_JPRB ,&
&  0.81832E+00_JPRB, 0.84983E+00_JPRB, 0.88059E+00_JPRB, 0.91882E+00_JPRB, 0.97431E+00_JPRB, 0.10017E+01_JPRB, 0.10528E+01_JPRB ,&
&  0.10584E+01_JPRB, 0.10856E+01_JPRB, 0.11238E+01_JPRB, 0.11913E+01_JPRB, 0.13417E+01_JPRB, 0.15818E+01_JPRB, 0.16110E+01_JPRB ,&
&  0.16304E+01_JPRB, 0.17553E+01_JPRB, 0.16162E+01_JPRB, 0.15465E+01_JPRB, 0.14935E+01_JPRB, 0.12869E+01_JPRB, 0.10903E+01_JPRB ,&
&  0.10663E+01_JPRB, 0.99577E+00_JPRB, 0.77432E+00_JPRB, 0.98727E+00_JPRB, 0.11404E+01_JPRB, 0.11965E+01_JPRB, 0.13177E+01_JPRB ,&
&  0.13596E+01_JPRB, 0.12375E+01_JPRB, 0.10914E+01_JPRB, 0.10912E+01_JPRB, 0.10969E+01_JPRB, 0.96756E+00_JPRB, 0.92069E+00_JPRB ,&
&  0.79681E+00_JPRB, 0.71294E+00_JPRB, 0.65949E+00_JPRB, 0.65216E+00_JPRB, 0.66145E+00_JPRB, 0.79704E+00_JPRB, 0.85381E+00_JPRB ,&
&  0.76826E+00_JPRB, 0.64325E+00_JPRB, 0.70433E+00_JPRB, 0.10001E+01_JPRB, 0.63752E+00_JPRB, 0.54624E+00_JPRB, 0.53291E+00_JPRB ,&
&  0.55283E+00_JPRB, 0.55729E+00_JPRB, 0.55971E+00_JPRB, 0.56127E+00_JPRB, 0.56483E+00_JPRB, 0.58138E+00_JPRB, 0.57648E+00_JPRB ,&
&  0.57396E+00_JPRB, 0.57434E+00_JPRB, 0.59295E+00_JPRB, 0.61691E+00_JPRB, 0.65763E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.77510E+00_JPRB, 0.76586E+00_JPRB, 0.77541E+00_JPRB, 0.79025E+00_JPRB, 0.80338E+00_JPRB, 0.81793E+00_JPRB, 0.84375E+00_JPRB ,&
&  0.87599E+00_JPRB, 0.90287E+00_JPRB, 0.93003E+00_JPRB, 0.94982E+00_JPRB, 0.95674E+00_JPRB, 0.97049E+00_JPRB, 0.98267E+00_JPRB ,&
&  0.99619E+00_JPRB, 0.99219E+00_JPRB, 0.10104E+01_JPRB, 0.10410E+01_JPRB, 0.10745E+01_JPRB, 0.10879E+01_JPRB, 0.10985E+01_JPRB ,&
&  0.11125E+01_JPRB, 0.11513E+01_JPRB, 0.11971E+01_JPRB, 0.12525E+01_JPRB, 0.13462E+01_JPRB, 0.15118E+01_JPRB, 0.16820E+01_JPRB ,&
&  0.11881E+01_JPRB, 0.11748E+01_JPRB, 0.11752E+01_JPRB, 0.99015E+00_JPRB, 0.92323E+00_JPRB, 0.95338E+00_JPRB, 0.93112E+00_JPRB ,&
&  0.91004E+00_JPRB, 0.93322E+00_JPRB, 0.94935E+00_JPRB, 0.96957E+00_JPRB, 0.99721E+00_JPRB, 0.10408E+01_JPRB, 0.10430E+01_JPRB ,&
&  0.10664E+01_JPRB, 0.10835E+01_JPRB, 0.10983E+01_JPRB, 0.11473E+01_JPRB, 0.11114E+01_JPRB, 0.13458E+01_JPRB, 0.13117E+01_JPRB ,&
&  0.12454E+01_JPRB, 0.12266E+01_JPRB, 0.93567E+00_JPRB, 0.93679E+00_JPRB, 0.94424E+00_JPRB, 0.82611E+00_JPRB, 0.75121E+00_JPRB ,&
&  0.95328E+00_JPRB, 0.10627E+01_JPRB, 0.86400E+00_JPRB, 0.10301E+01_JPRB, 0.11042E+01_JPRB, 0.11350E+01_JPRB, 0.11397E+01_JPRB ,&
&  0.11112E+01_JPRB, 0.10010E+01_JPRB, 0.91105E+00_JPRB, 0.85911E+00_JPRB, 0.83911E+00_JPRB, 0.81651E+00_JPRB, 0.77318E+00_JPRB ,&
&  0.71757E+00_JPRB, 0.68212E+00_JPRB, 0.63389E+00_JPRB, 0.60123E+00_JPRB, 0.76269E+00_JPRB, 0.71606E+00_JPRB, 0.10862E+01_JPRB ,&
&  0.72469E+00_JPRB, 0.62733E+00_JPRB, 0.53735E+00_JPRB, 0.75952E+00_JPRB, 0.66982E+00_JPRB, 0.58454E+00_JPRB, 0.65073E+00_JPRB ,&
&  0.70454E+00_JPRB, 0.73856E+00_JPRB, 0.72200E+00_JPRB, 0.69043E+00_JPRB, 0.67688E+00_JPRB, 0.67707E+00_JPRB, 0.65704E+00_JPRB ,&
&  0.65075E+00_JPRB, 0.66995E+00_JPRB, 0.70358E+00_JPRB, 0.71524E+00_JPRB, 0.74639E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.81243E+00_JPRB, 0.82723E+00_JPRB, 0.84382E+00_JPRB, 0.86414E+00_JPRB, 0.88357E+00_JPRB, 0.90120E+00_JPRB, 0.92209E+00_JPRB ,&
&  0.94938E+00_JPRB, 0.97049E+00_JPRB, 0.99192E+00_JPRB, 0.10056E+01_JPRB, 0.10123E+01_JPRB, 0.10256E+01_JPRB, 0.10301E+01_JPRB ,&
&  0.10325E+01_JPRB, 0.10348E+01_JPRB, 0.10516E+01_JPRB, 0.10622E+01_JPRB, 0.10624E+01_JPRB, 0.10600E+01_JPRB, 0.10742E+01_JPRB ,&
&  0.10755E+01_JPRB, 0.11051E+01_JPRB, 0.11305E+01_JPRB, 0.11621E+01_JPRB, 0.11889E+01_JPRB, 0.11968E+01_JPRB, 0.12654E+01_JPRB ,&
&  0.94864E+00_JPRB, 0.13838E+01_JPRB, 0.13924E+01_JPRB, 0.11325E+01_JPRB, 0.10760E+01_JPRB, 0.10215E+01_JPRB, 0.10117E+01_JPRB ,&
&  0.10172E+01_JPRB, 0.10299E+01_JPRB, 0.10550E+01_JPRB, 0.10988E+01_JPRB, 0.11411E+01_JPRB, 0.11816E+01_JPRB, 0.11999E+01_JPRB ,&
&  0.12282E+01_JPRB, 0.12549E+01_JPRB, 0.12820E+01_JPRB, 0.13034E+01_JPRB, 0.13376E+01_JPRB, 0.13509E+01_JPRB, 0.13223E+01_JPRB ,&
&  0.12394E+01_JPRB, 0.11694E+01_JPRB, 0.86129E+00_JPRB, 0.82143E+00_JPRB, 0.86098E+00_JPRB, 0.81726E+00_JPRB, 0.73377E+00_JPRB ,&
&  0.92496E+00_JPRB, 0.10446E+01_JPRB, 0.10446E+01_JPRB, 0.10579E+01_JPRB, 0.10454E+01_JPRB, 0.97189E+00_JPRB, 0.93683E+00_JPRB ,&
&  0.86387E+00_JPRB, 0.76532E+00_JPRB, 0.71256E+00_JPRB, 0.68100E+00_JPRB, 0.68705E+00_JPRB, 0.67834E+00_JPRB, 0.65119E+00_JPRB ,&
&  0.60709E+00_JPRB, 0.59009E+00_JPRB, 0.58894E+00_JPRB, 0.57698E+00_JPRB, 0.54824E+00_JPRB, 0.87439E+00_JPRB, 0.11169E+01_JPRB ,&
&  0.66182E+00_JPRB, 0.58221E+00_JPRB, 0.53418E+00_JPRB, 0.71313E+00_JPRB, 0.75074E+00_JPRB, 0.77901E+00_JPRB, 0.89620E+00_JPRB ,&
&  0.99010E+00_JPRB, 0.10011E+01_JPRB, 0.90605E+00_JPRB, 0.79526E+00_JPRB, 0.75251E+00_JPRB, 0.73735E+00_JPRB, 0.71598E+00_JPRB ,&
&  0.71700E+00_JPRB, 0.74086E+00_JPRB, 0.76279E+00_JPRB, 0.77894E+00_JPRB, 0.79588E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.85360E+00_JPRB, 0.86646E+00_JPRB, 0.87930E+00_JPRB, 0.90275E+00_JPRB, 0.92858E+00_JPRB, 0.95317E+00_JPRB, 0.97279E+00_JPRB ,&
&  0.99299E+00_JPRB, 0.10077E+01_JPRB, 0.10262E+01_JPRB, 0.10431E+01_JPRB, 0.10551E+01_JPRB, 0.10619E+01_JPRB, 0.10623E+01_JPRB ,&
&  0.10652E+01_JPRB, 0.10692E+01_JPRB, 0.10814E+01_JPRB, 0.10904E+01_JPRB, 0.10871E+01_JPRB, 0.10938E+01_JPRB, 0.11102E+01_JPRB ,&
&  0.11217E+01_JPRB, 0.11325E+01_JPRB, 0.11625E+01_JPRB, 0.12051E+01_JPRB, 0.12624E+01_JPRB, 0.13737E+01_JPRB, 0.12302E+01_JPRB ,&
&  0.13205E+01_JPRB, 0.14775E+01_JPRB, 0.14955E+01_JPRB, 0.12238E+01_JPRB, 0.11164E+01_JPRB, 0.10895E+01_JPRB, 0.10874E+01_JPRB ,&
&  0.10960E+01_JPRB, 0.11031E+01_JPRB, 0.11227E+01_JPRB, 0.11619E+01_JPRB, 0.12228E+01_JPRB, 0.12866E+01_JPRB, 0.13192E+01_JPRB ,&
&  0.13637E+01_JPRB, 0.14142E+01_JPRB, 0.14697E+01_JPRB, 0.15224E+01_JPRB, 0.16112E+01_JPRB, 0.16515E+01_JPRB, 0.16279E+01_JPRB ,&
&  0.15577E+01_JPRB, 0.15125E+01_JPRB, 0.13035E+01_JPRB, 0.12328E+01_JPRB, 0.12623E+01_JPRB, 0.11181E+01_JPRB, 0.99682E+00_JPRB ,&
&  0.10757E+01_JPRB, 0.98329E+00_JPRB, 0.90628E+00_JPRB, 0.10317E+01_JPRB, 0.98843E+00_JPRB, 0.86501E+00_JPRB, 0.82425E+00_JPRB ,&
&  0.72242E+00_JPRB, 0.63158E+00_JPRB, 0.58232E+00_JPRB, 0.57688E+00_JPRB, 0.58648E+00_JPRB, 0.61566E+00_JPRB, 0.61144E+00_JPRB ,&
&  0.61623E+00_JPRB, 0.63535E+00_JPRB, 0.64567E+00_JPRB, 0.65411E+00_JPRB, 0.73822E+00_JPRB, 0.68669E+00_JPRB, 0.86517E+00_JPRB ,&
&  0.73552E+00_JPRB, 0.61804E+00_JPRB, 0.71786E+00_JPRB, 0.85857E+00_JPRB, 0.92338E+00_JPRB, 0.99611E+00_JPRB, 0.11498E+01_JPRB ,&
&  0.13280E+01_JPRB, 0.14669E+01_JPRB, 0.13203E+01_JPRB, 0.11164E+01_JPRB, 0.90451E+00_JPRB, 0.81455E+00_JPRB, 0.77853E+00_JPRB ,&
&  0.77367E+00_JPRB, 0.79840E+00_JPRB, 0.81706E+00_JPRB, 0.82626E+00_JPRB, 0.83963E+00_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.86129E+00_JPRB, 0.87500E+00_JPRB, 0.88445E+00_JPRB, 0.89991E+00_JPRB, 0.91945E+00_JPRB, 0.94206E+00_JPRB, 0.95888E+00_JPRB ,&
&  0.97752E+00_JPRB, 0.99186E+00_JPRB, 0.10048E+01_JPRB, 0.10208E+01_JPRB, 0.10320E+01_JPRB, 0.10484E+01_JPRB, 0.10518E+01_JPRB ,&
&  0.10692E+01_JPRB, 0.10637E+01_JPRB, 0.10706E+01_JPRB, 0.10763E+01_JPRB, 0.10804E+01_JPRB, 0.10831E+01_JPRB, 0.10931E+01_JPRB ,&
&  0.10990E+01_JPRB, 0.10933E+01_JPRB, 0.11006E+01_JPRB, 0.11119E+01_JPRB, 0.11149E+01_JPRB, 0.11794E+01_JPRB, 0.10378E+01_JPRB ,&
&  0.14341E+01_JPRB, 0.16165E+01_JPRB, 0.15614E+01_JPRB, 0.12972E+01_JPRB, 0.11907E+01_JPRB, 0.11661E+01_JPRB, 0.11637E+01_JPRB ,&
&  0.11613E+01_JPRB, 0.11661E+01_JPRB, 0.11664E+01_JPRB, 0.11940E+01_JPRB, 0.12579E+01_JPRB, 0.13506E+01_JPRB, 0.14311E+01_JPRB ,&
&  0.14921E+01_JPRB, 0.15660E+01_JPRB, 0.16599E+01_JPRB, 0.17650E+01_JPRB, 0.19063E+01_JPRB, 0.19899E+01_JPRB, 0.20126E+01_JPRB ,&
&  0.20026E+01_JPRB, 0.19881E+01_JPRB, 0.18243E+01_JPRB, 0.18787E+01_JPRB, 0.19257E+01_JPRB, 0.15711E+01_JPRB, 0.12849E+01_JPRB ,&
&  0.10788E+01_JPRB, 0.96756E+00_JPRB, 0.97890E+00_JPRB, 0.10063E+01_JPRB, 0.91159E+00_JPRB, 0.83306E+00_JPRB, 0.77565E+00_JPRB ,&
&  0.66595E+00_JPRB, 0.60953E+00_JPRB, 0.58245E+00_JPRB, 0.55048E+00_JPRB, 0.55024E+00_JPRB, 0.57033E+00_JPRB, 0.61816E+00_JPRB ,&
&  0.64432E+00_JPRB, 0.67348E+00_JPRB, 0.70974E+00_JPRB, 0.73206E+00_JPRB, 0.78356E+00_JPRB, 0.76359E+00_JPRB, 0.90018E+00_JPRB ,&
&  0.87496E+00_JPRB, 0.86377E+00_JPRB, 0.90941E+00_JPRB, 0.77137E+00_JPRB, 0.10254E+01_JPRB, 0.11067E+01_JPRB, 0.12524E+01_JPRB ,&
&  0.12995E+01_JPRB, 0.17611E+01_JPRB, 0.21340E+01_JPRB, 0.17913E+01_JPRB, 0.12992E+01_JPRB, 0.89114E+00_JPRB, 0.79718E+00_JPRB ,&
&  0.79303E+00_JPRB, 0.81180E+00_JPRB, 0.83098E+00_JPRB, 0.83974E+00_JPRB, 0.85015E+00_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.83452E+00_JPRB, 0.84680E+00_JPRB, 0.85796E+00_JPRB, 0.86546E+00_JPRB, 0.86514E+00_JPRB, 0.87425E+00_JPRB, 0.89276E+00_JPRB ,&
&  0.90976E+00_JPRB, 0.92621E+00_JPRB, 0.94177E+00_JPRB, 0.95384E+00_JPRB, 0.96776E+00_JPRB, 0.97434E+00_JPRB, 0.99403E+00_JPRB ,&
&  0.10231E+01_JPRB, 0.10507E+01_JPRB, 0.10630E+01_JPRB, 0.10912E+01_JPRB, 0.11050E+01_JPRB, 0.11064E+01_JPRB, 0.11385E+01_JPRB ,&
&  0.11727E+01_JPRB, 0.12203E+01_JPRB, 0.12535E+01_JPRB, 0.12716E+01_JPRB, 0.12645E+01_JPRB, 0.13313E+01_JPRB, 0.11833E+01_JPRB ,&
&  0.15110E+01_JPRB, 0.16995E+01_JPRB, 0.16025E+01_JPRB, 0.13469E+01_JPRB, 0.12505E+01_JPRB, 0.12077E+01_JPRB, 0.11734E+01_JPRB ,&
&  0.11661E+01_JPRB, 0.11472E+01_JPRB, 0.11074E+01_JPRB, 0.11728E+01_JPRB, 0.12423E+01_JPRB, 0.13441E+01_JPRB, 0.14660E+01_JPRB ,&
&  0.15658E+01_JPRB, 0.16720E+01_JPRB, 0.17853E+01_JPRB, 0.19020E+01_JPRB, 0.20593E+01_JPRB, 0.21940E+01_JPRB, 0.22925E+01_JPRB ,&
&  0.23688E+01_JPRB, 0.24276E+01_JPRB, 0.24838E+01_JPRB, 0.23304E+01_JPRB, 0.23184E+01_JPRB, 0.20628E+01_JPRB, 0.15768E+01_JPRB ,&
&  0.10701E+01_JPRB, 0.92573E+00_JPRB, 0.95485E+00_JPRB, 0.97137E+00_JPRB, 0.91112E+00_JPRB, 0.78789E+00_JPRB, 0.74645E+00_JPRB ,&
&  0.64596E+00_JPRB, 0.58533E+00_JPRB, 0.57219E+00_JPRB, 0.55617E+00_JPRB, 0.54650E+00_JPRB, 0.57929E+00_JPRB, 0.60611E+00_JPRB ,&
&  0.64904E+00_JPRB, 0.71174E+00_JPRB, 0.76108E+00_JPRB, 0.79910E+00_JPRB, 0.88665E+00_JPRB, 0.96987E+00_JPRB, 0.91591E+00_JPRB ,&
&  0.95443E+00_JPRB, 0.96639E+00_JPRB, 0.99527E+00_JPRB, 0.11097E+01_JPRB, 0.10907E+01_JPRB, 0.12905E+01_JPRB, 0.15062E+01_JPRB ,&
&  0.18037E+01_JPRB, 0.17463E+01_JPRB, 0.20555E+01_JPRB, 0.30882E+01_JPRB, 0.26469E+01_JPRB, 0.22286E+01_JPRB, 0.82346E+00_JPRB ,&
&  0.81044E+00_JPRB, 0.82252E+00_JPRB, 0.84249E+00_JPRB, 0.84160E+00_JPRB, 0.84089E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.76031E+00_JPRB, 0.79095E+00_JPRB, 0.79359E+00_JPRB, 0.81580E+00_JPRB, 0.81115E+00_JPRB, 0.79796E+00_JPRB, 0.80420E+00_JPRB ,&
&  0.81674E+00_JPRB, 0.82543E+00_JPRB, 0.84565E+00_JPRB, 0.86588E+00_JPRB, 0.88075E+00_JPRB, 0.88391E+00_JPRB, 0.89529E+00_JPRB ,&
&  0.91631E+00_JPRB, 0.94535E+00_JPRB, 0.98029E+00_JPRB, 0.10276E+01_JPRB, 0.10595E+01_JPRB, 0.10794E+01_JPRB, 0.11169E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.12100E+01_JPRB, 0.12837E+01_JPRB, 0.13568E+01_JPRB, 0.14010E+01_JPRB, 0.14200E+01_JPRB, 0.11493E+01_JPRB ,&
&  0.15580E+01_JPRB, 0.16167E+01_JPRB, 0.15537E+01_JPRB, 0.13562E+01_JPRB, 0.12524E+01_JPRB, 0.11889E+01_JPRB, 0.11493E+01_JPRB ,&
&  0.11282E+01_JPRB, 0.11031E+01_JPRB, 0.10732E+01_JPRB, 0.10618E+01_JPRB, 0.12041E+01_JPRB, 0.12714E+01_JPRB, 0.13837E+01_JPRB ,&
&  0.15040E+01_JPRB, 0.16151E+01_JPRB, 0.17276E+01_JPRB, 0.18473E+01_JPRB, 0.20127E+01_JPRB, 0.22023E+01_JPRB, 0.23807E+01_JPRB ,&
&  0.25346E+01_JPRB, 0.26228E+01_JPRB, 0.27631E+01_JPRB, 0.24735E+01_JPRB, 0.24054E+01_JPRB, 0.22758E+01_JPRB, 0.15075E+01_JPRB ,&
&  0.98668E+00_JPRB, 0.92681E+00_JPRB, 0.99049E+00_JPRB, 0.98558E+00_JPRB, 0.90262E+00_JPRB, 0.78623E+00_JPRB, 0.70643E+00_JPRB ,&
&  0.62452E+00_JPRB, 0.60321E+00_JPRB, 0.58562E+00_JPRB, 0.56781E+00_JPRB, 0.55802E+00_JPRB, 0.58443E+00_JPRB, 0.60847E+00_JPRB ,&
&  0.66171E+00_JPRB, 0.70193E+00_JPRB, 0.75402E+00_JPRB, 0.80415E+00_JPRB, 0.87571E+00_JPRB, 0.96624E+00_JPRB, 0.10558E+01_JPRB ,&
&  0.92243E+00_JPRB, 0.85237E+00_JPRB, 0.88734E+00_JPRB, 0.94581E+00_JPRB, 0.98464E+00_JPRB, 0.10725E+01_JPRB, 0.11776E+01_JPRB ,&
&  0.13037E+01_JPRB, 0.13989E+01_JPRB, 0.12802E+01_JPRB, 0.10761E+01_JPRB, 0.14429E+01_JPRB, 0.14915E+01_JPRB, 0.69310E+00_JPRB ,&
&  0.68205E+00_JPRB, 0.71346E+00_JPRB, 0.73059E+00_JPRB, 0.75086E+00_JPRB, 0.77010E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.62404E+00_JPRB, 0.63064E+00_JPRB, 0.63059E+00_JPRB, 0.66427E+00_JPRB, 0.67155E+00_JPRB, 0.68495E+00_JPRB, 0.69671E+00_JPRB ,&
&  0.70597E+00_JPRB, 0.72484E+00_JPRB, 0.74717E+00_JPRB, 0.77162E+00_JPRB, 0.79051E+00_JPRB, 0.80417E+00_JPRB, 0.81435E+00_JPRB ,&
&  0.82567E+00_JPRB, 0.84015E+00_JPRB, 0.85931E+00_JPRB, 0.88374E+00_JPRB, 0.92390E+00_JPRB, 0.96867E+00_JPRB, 0.10152E+01_JPRB ,&
&  0.10586E+01_JPRB, 0.11180E+01_JPRB, 0.11942E+01_JPRB, 0.12887E+01_JPRB, 0.13837E+01_JPRB, 0.14500E+01_JPRB, 0.14826E+01_JPRB ,&
&  0.12114E+01_JPRB, 0.14644E+01_JPRB, 0.14407E+01_JPRB, 0.13280E+01_JPRB, 0.12251E+01_JPRB, 0.11724E+01_JPRB, 0.11355E+01_JPRB ,&
&  0.11125E+01_JPRB, 0.10600E+01_JPRB, 0.10571E+01_JPRB, 0.10814E+01_JPRB, 0.11437E+01_JPRB, 0.11857E+01_JPRB, 0.12617E+01_JPRB ,&
&  0.13713E+01_JPRB, 0.14595E+01_JPRB, 0.15663E+01_JPRB, 0.16734E+01_JPRB, 0.18232E+01_JPRB, 0.20542E+01_JPRB, 0.23219E+01_JPRB ,&
&  0.26030E+01_JPRB, 0.28225E+01_JPRB, 0.29889E+01_JPRB, 0.26478E+01_JPRB, 0.24379E+01_JPRB, 0.22368E+01_JPRB, 0.14461E+01_JPRB ,&
&  0.10208E+01_JPRB, 0.89321E+00_JPRB, 0.95937E+00_JPRB, 0.10081E+01_JPRB, 0.90389E+00_JPRB, 0.80413E+00_JPRB, 0.71382E+00_JPRB ,&
&  0.64239E+00_JPRB, 0.59747E+00_JPRB, 0.58797E+00_JPRB, 0.56899E+00_JPRB, 0.56540E+00_JPRB, 0.58162E+00_JPRB, 0.59866E+00_JPRB ,&
&  0.63508E+00_JPRB, 0.64970E+00_JPRB, 0.69016E+00_JPRB, 0.72617E+00_JPRB, 0.76587E+00_JPRB, 0.80078E+00_JPRB, 0.83376E+00_JPRB ,&
&  0.85604E+00_JPRB, 0.86552E+00_JPRB, 0.87051E+00_JPRB, 0.89709E+00_JPRB, 0.90900E+00_JPRB, 0.92732E+00_JPRB, 0.95504E+00_JPRB ,&
&  0.98628E+00_JPRB, 0.10063E+01_JPRB, 0.98784E+00_JPRB, 0.96624E+00_JPRB, 0.92401E+00_JPRB, 0.88197E+00_JPRB, 0.67651E+00_JPRB ,&
&  0.64177E+00_JPRB, 0.60442E+00_JPRB, 0.61917E+00_JPRB, 0.63789E+00_JPRB, 0.62920E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.55127E+00_JPRB, 0.53837E+00_JPRB, 0.52439E+00_JPRB, 0.52697E+00_JPRB, 0.53074E+00_JPRB, 0.55030E+00_JPRB, 0.55114E+00_JPRB ,&
&  0.56483E+00_JPRB, 0.60725E+00_JPRB, 0.65762E+00_JPRB, 0.68342E+00_JPRB, 0.70432E+00_JPRB, 0.72476E+00_JPRB, 0.74080E+00_JPRB ,&
&  0.75001E+00_JPRB, 0.75713E+00_JPRB, 0.76471E+00_JPRB, 0.77494E+00_JPRB, 0.79344E+00_JPRB, 0.82146E+00_JPRB, 0.86589E+00_JPRB ,&
&  0.90969E+00_JPRB, 0.95633E+00_JPRB, 0.10157E+01_JPRB, 0.10924E+01_JPRB, 0.11740E+01_JPRB, 0.12125E+01_JPRB, 0.12285E+01_JPRB ,&
&  0.78590E+00_JPRB, 0.11967E+01_JPRB, 0.14143E+01_JPRB, 0.13361E+01_JPRB, 0.12246E+01_JPRB, 0.11649E+01_JPRB, 0.11240E+01_JPRB ,&
&  0.10605E+01_JPRB, 0.99672E+00_JPRB, 0.99209E+00_JPRB, 0.10931E+01_JPRB, 0.10717E+01_JPRB, 0.10919E+01_JPRB, 0.11183E+01_JPRB ,&
&  0.11991E+01_JPRB, 0.12579E+01_JPRB, 0.12897E+01_JPRB, 0.13391E+01_JPRB, 0.14254E+01_JPRB, 0.16205E+01_JPRB, 0.19498E+01_JPRB ,&
&  0.23910E+01_JPRB, 0.28495E+01_JPRB, 0.31072E+01_JPRB, 0.26672E+01_JPRB, 0.27401E+01_JPRB, 0.25019E+01_JPRB, 0.20396E+01_JPRB ,&
&  0.12466E+01_JPRB, 0.94377E+00_JPRB, 0.97354E+00_JPRB, 0.10035E+01_JPRB, 0.90787E+00_JPRB, 0.83349E+00_JPRB, 0.75720E+00_JPRB ,&
&  0.67715E+00_JPRB, 0.62968E+00_JPRB, 0.60725E+00_JPRB, 0.59075E+00_JPRB, 0.58550E+00_JPRB, 0.58708E+00_JPRB, 0.59903E+00_JPRB ,&
&  0.60451E+00_JPRB, 0.61964E+00_JPRB, 0.64314E+00_JPRB, 0.66998E+00_JPRB, 0.70597E+00_JPRB, 0.74506E+00_JPRB, 0.78733E+00_JPRB ,&
&  0.81836E+00_JPRB, 0.84732E+00_JPRB, 0.87104E+00_JPRB, 0.89014E+00_JPRB, 0.89936E+00_JPRB, 0.90894E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89009E+00_JPRB, 0.90525E+00_JPRB, 0.91486E+00_JPRB, 0.90690E+00_JPRB, 0.87424E+00_JPRB, 0.84000E+00_JPRB, 0.81081E+00_JPRB ,&
&  0.75368E+00_JPRB, 0.66715E+00_JPRB, 0.58981E+00_JPRB, 0.56780E+00_JPRB, 0.55519E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.51340E+00_JPRB, 0.49015E+00_JPRB, 0.48108E+00_JPRB, 0.48010E+00_JPRB, 0.48855E+00_JPRB, 0.49373E+00_JPRB, 0.49315E+00_JPRB ,&
&  0.49554E+00_JPRB, 0.50619E+00_JPRB, 0.51958E+00_JPRB, 0.56328E+00_JPRB, 0.60734E+00_JPRB, 0.63919E+00_JPRB, 0.65858E+00_JPRB ,&
&  0.67448E+00_JPRB, 0.67892E+00_JPRB, 0.67656E+00_JPRB, 0.67805E+00_JPRB, 0.68028E+00_JPRB, 0.69108E+00_JPRB, 0.71566E+00_JPRB ,&
&  0.75639E+00_JPRB, 0.79883E+00_JPRB, 0.85117E+00_JPRB, 0.91789E+00_JPRB, 0.99873E+00_JPRB, 0.10724E+01_JPRB, 0.11248E+01_JPRB ,&
&  0.87185E+00_JPRB, 0.52280E+00_JPRB, 0.12104E+01_JPRB, 0.14306E+01_JPRB, 0.12736E+01_JPRB, 0.12082E+01_JPRB, 0.11194E+01_JPRB ,&
&  0.10032E+01_JPRB, 0.96923E+00_JPRB, 0.94153E+00_JPRB, 0.99210E+00_JPRB, 0.98768E+00_JPRB, 0.10092E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.10751E+01_JPRB, 0.11389E+01_JPRB, 0.11509E+01_JPRB, 0.11357E+01_JPRB, 0.11397E+01_JPRB, 0.12109E+01_JPRB, 0.14284E+01_JPRB ,&
&  0.18645E+01_JPRB, 0.25042E+01_JPRB, 0.31017E+01_JPRB, 0.27796E+01_JPRB, 0.28705E+01_JPRB, 0.22696E+01_JPRB, 0.16600E+01_JPRB ,&
&  0.12615E+01_JPRB, 0.10317E+01_JPRB, 0.10007E+01_JPRB, 0.10178E+01_JPRB, 0.93824E+00_JPRB, 0.83716E+00_JPRB, 0.82051E+00_JPRB ,&
&  0.72909E+00_JPRB, 0.66883E+00_JPRB, 0.62995E+00_JPRB, 0.61046E+00_JPRB, 0.60294E+00_JPRB, 0.60377E+00_JPRB, 0.60807E+00_JPRB ,&
&  0.60844E+00_JPRB, 0.61616E+00_JPRB, 0.62612E+00_JPRB, 0.64979E+00_JPRB, 0.68159E+00_JPRB, 0.72825E+00_JPRB, 0.76766E+00_JPRB ,&
&  0.79131E+00_JPRB, 0.81101E+00_JPRB, 0.82983E+00_JPRB, 0.84172E+00_JPRB, 0.85122E+00_JPRB, 0.83174E+00_JPRB, 0.82409E+00_JPRB ,&
&  0.83296E+00_JPRB, 0.84719E+00_JPRB, 0.86918E+00_JPRB, 0.89642E+00_JPRB, 0.86799E+00_JPRB, 0.84349E+00_JPRB, 0.86412E+00_JPRB ,&
&  0.85779E+00_JPRB, 0.80221E+00_JPRB, 0.68925E+00_JPRB, 0.57661E+00_JPRB, 0.53639E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.54417E+00_JPRB, 0.52304E+00_JPRB, 0.49606E+00_JPRB, 0.48813E+00_JPRB, 0.49577E+00_JPRB, 0.49062E+00_JPRB, 0.48710E+00_JPRB ,&
&  0.48294E+00_JPRB, 0.47967E+00_JPRB, 0.46583E+00_JPRB, 0.47409E+00_JPRB, 0.50130E+00_JPRB, 0.54072E+00_JPRB, 0.57412E+00_JPRB ,&
&  0.59802E+00_JPRB, 0.60735E+00_JPRB, 0.60771E+00_JPRB, 0.60709E+00_JPRB, 0.60723E+00_JPRB, 0.60751E+00_JPRB, 0.61040E+00_JPRB ,&
&  0.62696E+00_JPRB, 0.65460E+00_JPRB, 0.69092E+00_JPRB, 0.72900E+00_JPRB, 0.77753E+00_JPRB, 0.85031E+00_JPRB, 0.96793E+00_JPRB ,&
&  0.11501E+01_JPRB, 0.87367E+00_JPRB, 0.62263E+00_JPRB, 0.12195E+01_JPRB, 0.13544E+01_JPRB, 0.13230E+01_JPRB, 0.11445E+01_JPRB ,&
&  0.10682E+01_JPRB, 0.97256E+00_JPRB, 0.91835E+00_JPRB, 0.95046E+00_JPRB, 0.90114E+00_JPRB, 0.91596E+00_JPRB, 0.95491E+00_JPRB ,&
&  0.98882E+00_JPRB, 0.10251E+01_JPRB, 0.10544E+01_JPRB, 0.10444E+01_JPRB, 0.10087E+01_JPRB, 0.99610E+00_JPRB, 0.10711E+01_JPRB ,&
&  0.13402E+01_JPRB, 0.18900E+01_JPRB, 0.27711E+01_JPRB, 0.28303E+01_JPRB, 0.29169E+01_JPRB, 0.20778E+01_JPRB, 0.15433E+01_JPRB ,&
&  0.12812E+01_JPRB, 0.12344E+01_JPRB, 0.11778E+01_JPRB, 0.10695E+01_JPRB, 0.94777E+00_JPRB, 0.87086E+00_JPRB, 0.88022E+00_JPRB ,&
&  0.79753E+00_JPRB, 0.73680E+00_JPRB, 0.68646E+00_JPRB, 0.65518E+00_JPRB, 0.63773E+00_JPRB, 0.62711E+00_JPRB, 0.61843E+00_JPRB ,&
&  0.62277E+00_JPRB, 0.62554E+00_JPRB, 0.63737E+00_JPRB, 0.65172E+00_JPRB, 0.67007E+00_JPRB, 0.69678E+00_JPRB, 0.73414E+00_JPRB ,&
&  0.76325E+00_JPRB, 0.77837E+00_JPRB, 0.78549E+00_JPRB, 0.78353E+00_JPRB, 0.76170E+00_JPRB, 0.74746E+00_JPRB, 0.76649E+00_JPRB ,&
&  0.76524E+00_JPRB, 0.77754E+00_JPRB, 0.78024E+00_JPRB, 0.80402E+00_JPRB, 0.82427E+00_JPRB, 0.83678E+00_JPRB, 0.86574E+00_JPRB ,&
&  0.90521E+00_JPRB, 0.89822E+00_JPRB, 0.80859E+00_JPRB, 0.61659E+00_JPRB, 0.56777E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.60087E+00_JPRB, 0.58520E+00_JPRB, 0.57141E+00_JPRB, 0.56939E+00_JPRB, 0.56649E+00_JPRB, 0.55152E+00_JPRB, 0.53223E+00_JPRB ,&
&  0.50802E+00_JPRB, 0.49013E+00_JPRB, 0.46505E+00_JPRB, 0.44230E+00_JPRB, 0.43897E+00_JPRB, 0.45116E+00_JPRB, 0.47652E+00_JPRB ,&
&  0.50132E+00_JPRB, 0.51496E+00_JPRB, 0.52492E+00_JPRB, 0.53460E+00_JPRB, 0.54313E+00_JPRB, 0.54661E+00_JPRB, 0.54483E+00_JPRB ,&
&  0.54782E+00_JPRB, 0.55813E+00_JPRB, 0.57465E+00_JPRB, 0.59251E+00_JPRB, 0.61556E+00_JPRB, 0.65280E+00_JPRB, 0.72017E+00_JPRB ,&
&  0.86233E+00_JPRB, 0.93397E+00_JPRB, 0.54194E+00_JPRB, 0.98296E+00_JPRB, 0.13524E+01_JPRB, 0.13330E+01_JPRB, 0.12582E+01_JPRB ,&
&  0.11592E+01_JPRB, 0.99903E+00_JPRB, 0.97095E+00_JPRB, 0.93130E+00_JPRB, 0.85765E+00_JPRB, 0.84011E+00_JPRB, 0.85236E+00_JPRB ,&
&  0.86307E+00_JPRB, 0.85237E+00_JPRB, 0.86118E+00_JPRB, 0.89003E+00_JPRB, 0.88339E+00_JPRB, 0.85676E+00_JPRB, 0.87017E+00_JPRB ,&
&  0.99974E+00_JPRB, 0.13791E+01_JPRB, 0.22064E+01_JPRB, 0.27002E+01_JPRB, 0.28689E+01_JPRB, 0.22579E+01_JPRB, 0.17090E+01_JPRB ,&
&  0.14930E+01_JPRB, 0.15243E+01_JPRB, 0.14136E+01_JPRB, 0.11918E+01_JPRB, 0.10056E+01_JPRB, 0.94299E+00_JPRB, 0.93175E+00_JPRB ,&
&  0.86065E+00_JPRB, 0.79656E+00_JPRB, 0.75651E+00_JPRB, 0.72476E+00_JPRB, 0.70001E+00_JPRB, 0.67599E+00_JPRB, 0.65612E+00_JPRB ,&
&  0.65127E+00_JPRB, 0.64881E+00_JPRB, 0.65027E+00_JPRB, 0.65422E+00_JPRB, 0.65862E+00_JPRB, 0.66822E+00_JPRB, 0.69071E+00_JPRB ,&
&  0.72069E+00_JPRB, 0.73715E+00_JPRB, 0.74577E+00_JPRB, 0.72892E+00_JPRB, 0.71292E+00_JPRB, 0.71244E+00_JPRB, 0.72944E+00_JPRB ,&
&  0.73372E+00_JPRB, 0.74445E+00_JPRB, 0.73968E+00_JPRB, 0.73772E+00_JPRB, 0.77821E+00_JPRB, 0.79233E+00_JPRB, 0.82878E+00_JPRB ,&
&  0.87871E+00_JPRB, 0.94758E+00_JPRB, 0.95395E+00_JPRB, 0.67950E+00_JPRB, 0.63368E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.64149E+00_JPRB, 0.62227E+00_JPRB, 0.61841E+00_JPRB, 0.61857E+00_JPRB, 0.61571E+00_JPRB, 0.60675E+00_JPRB, 0.58914E+00_JPRB ,&
&  0.56718E+00_JPRB, 0.55103E+00_JPRB, 0.53217E+00_JPRB, 0.50795E+00_JPRB, 0.47308E+00_JPRB, 0.44112E+00_JPRB, 0.43787E+00_JPRB ,&
&  0.45377E+00_JPRB, 0.46196E+00_JPRB, 0.46929E+00_JPRB, 0.47824E+00_JPRB, 0.48743E+00_JPRB, 0.49856E+00_JPRB, 0.50578E+00_JPRB ,&
&  0.50803E+00_JPRB, 0.51228E+00_JPRB, 0.52089E+00_JPRB, 0.52874E+00_JPRB, 0.53959E+00_JPRB, 0.55969E+00_JPRB, 0.59268E+00_JPRB ,&
&  0.69038E+00_JPRB, 0.81977E+00_JPRB, 0.84940E+00_JPRB, 0.13792E+01_JPRB, 0.15967E+01_JPRB, 0.14892E+01_JPRB, 0.14409E+01_JPRB ,&
&  0.13688E+01_JPRB, 0.12022E+01_JPRB, 0.11227E+01_JPRB, 0.96153E+00_JPRB, 0.86531E+00_JPRB, 0.81942E+00_JPRB, 0.79797E+00_JPRB ,&
&  0.78007E+00_JPRB, 0.75577E+00_JPRB, 0.72781E+00_JPRB, 0.72745E+00_JPRB, 0.73239E+00_JPRB, 0.72806E+00_JPRB, 0.73837E+00_JPRB ,&
&  0.80067E+00_JPRB, 0.10302E+01_JPRB, 0.16229E+01_JPRB, 0.25470E+01_JPRB, 0.28397E+01_JPRB, 0.25608E+01_JPRB, 0.21214E+01_JPRB ,&
&  0.21127E+01_JPRB, 0.19987E+01_JPRB, 0.17950E+01_JPRB, 0.14371E+01_JPRB, 0.11570E+01_JPRB, 0.11354E+01_JPRB, 0.98937E+00_JPRB ,&
&  0.90082E+00_JPRB, 0.84080E+00_JPRB, 0.80599E+00_JPRB, 0.78030E+00_JPRB, 0.76011E+00_JPRB, 0.73460E+00_JPRB, 0.70388E+00_JPRB ,&
&  0.67929E+00_JPRB, 0.66625E+00_JPRB, 0.65462E+00_JPRB, 0.65033E+00_JPRB, 0.64722E+00_JPRB, 0.64654E+00_JPRB, 0.65352E+00_JPRB ,&
&  0.67715E+00_JPRB, 0.69625E+00_JPRB, 0.68020E+00_JPRB, 0.67543E+00_JPRB, 0.67604E+00_JPRB, 0.67709E+00_JPRB, 0.68014E+00_JPRB ,&
&  0.68132E+00_JPRB, 0.70261E+00_JPRB, 0.72380E+00_JPRB, 0.72688E+00_JPRB, 0.74077E+00_JPRB, 0.75890E+00_JPRB, 0.76950E+00_JPRB ,&
&  0.78075E+00_JPRB, 0.77144E+00_JPRB, 0.75988E+00_JPRB, 0.69667E+00_JPRB, 0.67016E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.65800E+00_JPRB, 0.64030E+00_JPRB, 0.62829E+00_JPRB, 0.62257E+00_JPRB, 0.61750E+00_JPRB, 0.61230E+00_JPRB, 0.59798E+00_JPRB ,&
&  0.58220E+00_JPRB, 0.57253E+00_JPRB, 0.55758E+00_JPRB, 0.54255E+00_JPRB, 0.51271E+00_JPRB, 0.47520E+00_JPRB, 0.45876E+00_JPRB ,&
&  0.44914E+00_JPRB, 0.44848E+00_JPRB, 0.45712E+00_JPRB, 0.46133E+00_JPRB, 0.46452E+00_JPRB, 0.47236E+00_JPRB, 0.47918E+00_JPRB ,&
&  0.48552E+00_JPRB, 0.48702E+00_JPRB, 0.49263E+00_JPRB, 0.49782E+00_JPRB, 0.50083E+00_JPRB, 0.51332E+00_JPRB, 0.53237E+00_JPRB ,&
&  0.60012E+00_JPRB, 0.78353E+00_JPRB, 0.70500E+00_JPRB, 0.13313E+01_JPRB, 0.15295E+01_JPRB, 0.15159E+01_JPRB, 0.13888E+01_JPRB ,&
&  0.13557E+01_JPRB, 0.13449E+01_JPRB, 0.11220E+01_JPRB, 0.97301E+00_JPRB, 0.86774E+00_JPRB, 0.81150E+00_JPRB, 0.77737E+00_JPRB ,&
&  0.75141E+00_JPRB, 0.72290E+00_JPRB, 0.68493E+00_JPRB, 0.65493E+00_JPRB, 0.64348E+00_JPRB, 0.63823E+00_JPRB, 0.64671E+00_JPRB ,&
&  0.67467E+00_JPRB, 0.79964E+00_JPRB, 0.11923E+01_JPRB, 0.20065E+01_JPRB, 0.27249E+01_JPRB, 0.28175E+01_JPRB, 0.25342E+01_JPRB ,&
&  0.25502E+01_JPRB, 0.25940E+01_JPRB, 0.23426E+01_JPRB, 0.18257E+01_JPRB, 0.14635E+01_JPRB, 0.12822E+01_JPRB, 0.10995E+01_JPRB ,&
&  0.98071E+00_JPRB, 0.90492E+00_JPRB, 0.86019E+00_JPRB, 0.82293E+00_JPRB, 0.79309E+00_JPRB, 0.76412E+00_JPRB, 0.72893E+00_JPRB ,&
&  0.69600E+00_JPRB, 0.66783E+00_JPRB, 0.65283E+00_JPRB, 0.64163E+00_JPRB, 0.63932E+00_JPRB, 0.63780E+00_JPRB, 0.63782E+00_JPRB ,&
&  0.64775E+00_JPRB, 0.66791E+00_JPRB, 0.64468E+00_JPRB, 0.63793E+00_JPRB, 0.64020E+00_JPRB, 0.64270E+00_JPRB, 0.65255E+00_JPRB ,&
&  0.67266E+00_JPRB, 0.68037E+00_JPRB, 0.69077E+00_JPRB, 0.68747E+00_JPRB, 0.70589E+00_JPRB, 0.74336E+00_JPRB, 0.74673E+00_JPRB ,&
&  0.75103E+00_JPRB, 0.74017E+00_JPRB, 0.72291E+00_JPRB, 0.69551E+00_JPRB, 0.67611E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.65740E+00_JPRB, 0.64356E+00_JPRB, 0.62546E+00_JPRB, 0.60831E+00_JPRB, 0.60270E+00_JPRB, 0.59953E+00_JPRB, 0.58719E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.56397E+00_JPRB, 0.54862E+00_JPRB, 0.53013E+00_JPRB, 0.50913E+00_JPRB, 0.48307E+00_JPRB, 0.45910E+00_JPRB ,&
&  0.44774E+00_JPRB, 0.43862E+00_JPRB, 0.44091E+00_JPRB, 0.44631E+00_JPRB, 0.45450E+00_JPRB, 0.46039E+00_JPRB, 0.46723E+00_JPRB ,&
&  0.47317E+00_JPRB, 0.47628E+00_JPRB, 0.48016E+00_JPRB, 0.48326E+00_JPRB, 0.48378E+00_JPRB, 0.48767E+00_JPRB, 0.50889E+00_JPRB ,&
&  0.55794E+00_JPRB, 0.81250E+00_JPRB, 0.69768E+00_JPRB, 0.11004E+01_JPRB, 0.12754E+01_JPRB, 0.13490E+01_JPRB, 0.12578E+01_JPRB ,&
&  0.12427E+01_JPRB, 0.11877E+01_JPRB, 0.10454E+01_JPRB, 0.93606E+00_JPRB, 0.85285E+00_JPRB, 0.79752E+00_JPRB, 0.75850E+00_JPRB ,&
&  0.73093E+00_JPRB, 0.69679E+00_JPRB, 0.65652E+00_JPRB, 0.61991E+00_JPRB, 0.60083E+00_JPRB, 0.59122E+00_JPRB, 0.59230E+00_JPRB ,&
&  0.60194E+00_JPRB, 0.66253E+00_JPRB, 0.88627E+00_JPRB, 0.15620E+01_JPRB, 0.23206E+01_JPRB, 0.27846E+01_JPRB, 0.28857E+01_JPRB ,&
&  0.29684E+01_JPRB, 0.31645E+01_JPRB, 0.28744E+01_JPRB, 0.22898E+01_JPRB, 0.18370E+01_JPRB, 0.15064E+01_JPRB, 0.12573E+01_JPRB ,&
&  0.11047E+01_JPRB, 0.10129E+01_JPRB, 0.95042E+00_JPRB, 0.89799E+00_JPRB, 0.84651E+00_JPRB, 0.79021E+00_JPRB, 0.73392E+00_JPRB ,&
&  0.69123E+00_JPRB, 0.66159E+00_JPRB, 0.64280E+00_JPRB, 0.63794E+00_JPRB, 0.63591E+00_JPRB, 0.63559E+00_JPRB, 0.63758E+00_JPRB ,&
&  0.64278E+00_JPRB, 0.65422E+00_JPRB, 0.63690E+00_JPRB, 0.62985E+00_JPRB, 0.62811E+00_JPRB, 0.63416E+00_JPRB, 0.63977E+00_JPRB ,&
&  0.66757E+00_JPRB, 0.67239E+00_JPRB, 0.66606E+00_JPRB, 0.66841E+00_JPRB, 0.68843E+00_JPRB, 0.73921E+00_JPRB, 0.74302E+00_JPRB ,&
&  0.73925E+00_JPRB, 0.72307E+00_JPRB, 0.70454E+00_JPRB, 0.68919E+00_JPRB, 0.67520E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.65514E+00_JPRB, 0.63660E+00_JPRB, 0.62378E+00_JPRB, 0.61061E+00_JPRB, 0.60330E+00_JPRB, 0.59654E+00_JPRB, 0.58302E+00_JPRB ,&
&  0.56597E+00_JPRB, 0.54664E+00_JPRB, 0.52744E+00_JPRB, 0.51072E+00_JPRB, 0.49775E+00_JPRB, 0.48297E+00_JPRB, 0.47104E+00_JPRB ,&
&  0.46031E+00_JPRB, 0.44724E+00_JPRB, 0.43861E+00_JPRB, 0.43562E+00_JPRB, 0.43899E+00_JPRB, 0.44389E+00_JPRB, 0.45066E+00_JPRB ,&
&  0.45878E+00_JPRB, 0.46424E+00_JPRB, 0.46530E+00_JPRB, 0.46748E+00_JPRB, 0.46560E+00_JPRB, 0.46836E+00_JPRB, 0.49161E+00_JPRB ,&
&  0.54730E+00_JPRB, 0.51141E+00_JPRB, 0.76235E+00_JPRB, 0.89688E+00_JPRB, 0.10158E+01_JPRB, 0.11222E+01_JPRB, 0.11712E+01_JPRB ,&
&  0.11927E+01_JPRB, 0.10799E+01_JPRB, 0.95489E+00_JPRB, 0.87365E+00_JPRB, 0.80969E+00_JPRB, 0.75995E+00_JPRB, 0.71964E+00_JPRB ,&
&  0.67983E+00_JPRB, 0.64917E+00_JPRB, 0.62454E+00_JPRB, 0.59204E+00_JPRB, 0.57113E+00_JPRB, 0.56260E+00_JPRB, 0.56049E+00_JPRB ,&
&  0.55740E+00_JPRB, 0.57908E+00_JPRB, 0.68993E+00_JPRB, 0.10577E+01_JPRB, 0.19266E+01_JPRB, 0.26030E+01_JPRB, 0.30936E+01_JPRB ,&
&  0.37040E+01_JPRB, 0.38332E+01_JPRB, 0.33984E+01_JPRB, 0.27440E+01_JPRB, 0.21243E+01_JPRB, 0.17065E+01_JPRB, 0.13708E+01_JPRB ,&
&  0.11775E+01_JPRB, 0.10698E+01_JPRB, 0.10067E+01_JPRB, 0.95598E+00_JPRB, 0.90198E+00_JPRB, 0.82962E+00_JPRB, 0.74875E+00_JPRB ,&
&  0.68554E+00_JPRB, 0.64983E+00_JPRB, 0.62787E+00_JPRB, 0.61593E+00_JPRB, 0.60991E+00_JPRB, 0.61045E+00_JPRB, 0.61702E+00_JPRB ,&
&  0.62783E+00_JPRB, 0.64303E+00_JPRB, 0.63150E+00_JPRB, 0.62445E+00_JPRB, 0.62784E+00_JPRB, 0.64028E+00_JPRB, 0.65256E+00_JPRB ,&
&  0.65525E+00_JPRB, 0.64676E+00_JPRB, 0.63440E+00_JPRB, 0.63902E+00_JPRB, 0.66017E+00_JPRB, 0.73036E+00_JPRB, 0.73878E+00_JPRB ,&
&  0.73483E+00_JPRB, 0.71881E+00_JPRB, 0.69278E+00_JPRB, 0.68211E+00_JPRB, 0.67663E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.64781E+00_JPRB, 0.63059E+00_JPRB, 0.62607E+00_JPRB, 0.62744E+00_JPRB, 0.62806E+00_JPRB, 0.61555E+00_JPRB, 0.58665E+00_JPRB ,&
&  0.56108E+00_JPRB, 0.53916E+00_JPRB, 0.51784E+00_JPRB, 0.50050E+00_JPRB, 0.48929E+00_JPRB, 0.48133E+00_JPRB, 0.47262E+00_JPRB ,&
&  0.46666E+00_JPRB, 0.46164E+00_JPRB, 0.44931E+00_JPRB, 0.43776E+00_JPRB, 0.43282E+00_JPRB, 0.43216E+00_JPRB, 0.43589E+00_JPRB ,&
&  0.44012E+00_JPRB, 0.44541E+00_JPRB, 0.44927E+00_JPRB, 0.45207E+00_JPRB, 0.45177E+00_JPRB, 0.45623E+00_JPRB, 0.47722E+00_JPRB ,&
&  0.53982E+00_JPRB, 0.64307E+00_JPRB, 0.73930E+00_JPRB, 0.76194E+00_JPRB, 0.83395E+00_JPRB, 0.95397E+00_JPRB, 0.10759E+01_JPRB ,&
&  0.10702E+01_JPRB, 0.98318E+00_JPRB, 0.87302E+00_JPRB, 0.79754E+00_JPRB, 0.74082E+00_JPRB, 0.70060E+00_JPRB, 0.65827E+00_JPRB ,&
&  0.62236E+00_JPRB, 0.59816E+00_JPRB, 0.57386E+00_JPRB, 0.55204E+00_JPRB, 0.54457E+00_JPRB, 0.54537E+00_JPRB, 0.54539E+00_JPRB ,&
&  0.54280E+00_JPRB, 0.54348E+00_JPRB, 0.59234E+00_JPRB, 0.84409E+00_JPRB, 0.15953E+01_JPRB, 0.21006E+01_JPRB, 0.30255E+01_JPRB ,&
&  0.38586E+01_JPRB, 0.37641E+01_JPRB, 0.33594E+01_JPRB, 0.28047E+01_JPRB, 0.22099E+01_JPRB, 0.17141E+01_JPRB, 0.14053E+01_JPRB ,&
&  0.12113E+01_JPRB, 0.10584E+01_JPRB, 0.96594E+00_JPRB, 0.90735E+00_JPRB, 0.86004E+00_JPRB, 0.81576E+00_JPRB, 0.76190E+00_JPRB ,&
&  0.70552E+00_JPRB, 0.65943E+00_JPRB, 0.62793E+00_JPRB, 0.60568E+00_JPRB, 0.59361E+00_JPRB, 0.59008E+00_JPRB, 0.59133E+00_JPRB ,&
&  0.59239E+00_JPRB, 0.60264E+00_JPRB, 0.61226E+00_JPRB, 0.61099E+00_JPRB, 0.62108E+00_JPRB, 0.63755E+00_JPRB, 0.64721E+00_JPRB ,&
&  0.63054E+00_JPRB, 0.61770E+00_JPRB, 0.61354E+00_JPRB, 0.61323E+00_JPRB, 0.65626E+00_JPRB, 0.73357E+00_JPRB, 0.72910E+00_JPRB ,&
&  0.72231E+00_JPRB, 0.70835E+00_JPRB, 0.69223E+00_JPRB, 0.67225E+00_JPRB, 0.67179E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.63960E+00_JPRB, 0.61981E+00_JPRB, 0.60934E+00_JPRB, 0.61569E+00_JPRB, 0.62598E+00_JPRB, 0.61789E+00_JPRB, 0.59021E+00_JPRB ,&
&  0.55567E+00_JPRB, 0.52749E+00_JPRB, 0.50754E+00_JPRB, 0.49352E+00_JPRB, 0.48339E+00_JPRB, 0.47468E+00_JPRB, 0.46503E+00_JPRB ,&
&  0.45659E+00_JPRB, 0.45224E+00_JPRB, 0.44912E+00_JPRB, 0.44458E+00_JPRB, 0.43541E+00_JPRB, 0.42810E+00_JPRB, 0.42569E+00_JPRB ,&
&  0.42433E+00_JPRB, 0.42499E+00_JPRB, 0.43020E+00_JPRB, 0.43600E+00_JPRB, 0.43823E+00_JPRB, 0.44077E+00_JPRB, 0.45539E+00_JPRB ,&
&  0.50312E+00_JPRB, 0.58315E+00_JPRB, 0.59492E+00_JPRB, 0.64926E+00_JPRB, 0.73225E+00_JPRB, 0.83735E+00_JPRB, 0.92994E+00_JPRB ,&
&  0.94000E+00_JPRB, 0.89559E+00_JPRB, 0.81679E+00_JPRB, 0.73634E+00_JPRB, 0.67236E+00_JPRB, 0.63590E+00_JPRB, 0.60969E+00_JPRB ,&
&  0.58420E+00_JPRB, 0.55773E+00_JPRB, 0.53472E+00_JPRB, 0.51761E+00_JPRB, 0.50619E+00_JPRB, 0.50249E+00_JPRB, 0.50370E+00_JPRB ,&
&  0.50536E+00_JPRB, 0.50547E+00_JPRB, 0.51610E+00_JPRB, 0.60654E+00_JPRB, 0.10169E+01_JPRB, 0.15575E+01_JPRB, 0.21655E+01_JPRB ,&
&  0.23435E+01_JPRB, 0.24257E+01_JPRB, 0.23892E+01_JPRB, 0.22355E+01_JPRB, 0.19715E+01_JPRB, 0.17117E+01_JPRB, 0.14321E+01_JPRB ,&
&  0.12272E+01_JPRB, 0.10871E+01_JPRB, 0.98619E+00_JPRB, 0.90168E+00_JPRB, 0.82617E+00_JPRB, 0.76593E+00_JPRB, 0.72403E+00_JPRB ,&
&  0.69369E+00_JPRB, 0.66763E+00_JPRB, 0.64318E+00_JPRB, 0.62072E+00_JPRB, 0.60339E+00_JPRB, 0.59305E+00_JPRB, 0.59223E+00_JPRB ,&
&  0.58989E+00_JPRB, 0.58222E+00_JPRB, 0.58390E+00_JPRB, 0.58997E+00_JPRB, 0.59834E+00_JPRB, 0.60177E+00_JPRB, 0.60257E+00_JPRB ,&
&  0.60773E+00_JPRB, 0.61375E+00_JPRB, 0.61156E+00_JPRB, 0.61319E+00_JPRB, 0.64586E+00_JPRB, 0.70385E+00_JPRB, 0.71830E+00_JPRB ,&
&  0.71744E+00_JPRB, 0.70173E+00_JPRB, 0.68889E+00_JPRB, 0.67803E+00_JPRB, 0.67133E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.73761E+00_JPRB, 0.65838E+00_JPRB, 0.61145E+00_JPRB, 0.59281E+00_JPRB, 0.59050E+00_JPRB, 0.57757E+00_JPRB, 0.55828E+00_JPRB ,&
&  0.53833E+00_JPRB, 0.52241E+00_JPRB, 0.50993E+00_JPRB, 0.49583E+00_JPRB, 0.48177E+00_JPRB, 0.46998E+00_JPRB, 0.45849E+00_JPRB ,&
&  0.44728E+00_JPRB, 0.44073E+00_JPRB, 0.43578E+00_JPRB, 0.43250E+00_JPRB, 0.43004E+00_JPRB, 0.42481E+00_JPRB, 0.41887E+00_JPRB ,&
&  0.41432E+00_JPRB, 0.40820E+00_JPRB, 0.40693E+00_JPRB, 0.41320E+00_JPRB, 0.41878E+00_JPRB, 0.42210E+00_JPRB, 0.43636E+00_JPRB ,&
&  0.48007E+00_JPRB, 0.49186E+00_JPRB, 0.53997E+00_JPRB, 0.58282E+00_JPRB, 0.62301E+00_JPRB, 0.68053E+00_JPRB, 0.70855E+00_JPRB ,&
&  0.70320E+00_JPRB, 0.71135E+00_JPRB, 0.70032E+00_JPRB, 0.65707E+00_JPRB, 0.60867E+00_JPRB, 0.57667E+00_JPRB, 0.55429E+00_JPRB ,&
&  0.53578E+00_JPRB, 0.51439E+00_JPRB, 0.49200E+00_JPRB, 0.47433E+00_JPRB, 0.46426E+00_JPRB, 0.46114E+00_JPRB, 0.46147E+00_JPRB ,&
&  0.46320E+00_JPRB, 0.46720E+00_JPRB, 0.47183E+00_JPRB, 0.48722E+00_JPRB, 0.58251E+00_JPRB, 0.82638E+00_JPRB, 0.12098E+01_JPRB ,&
&  0.13917E+01_JPRB, 0.14469E+01_JPRB, 0.14895E+01_JPRB, 0.14941E+01_JPRB, 0.13997E+01_JPRB, 0.12956E+01_JPRB, 0.11840E+01_JPRB ,&
&  0.10725E+01_JPRB, 0.95545E+00_JPRB, 0.85686E+00_JPRB, 0.78006E+00_JPRB, 0.72049E+00_JPRB, 0.67981E+00_JPRB, 0.65811E+00_JPRB ,&
&  0.64871E+00_JPRB, 0.64357E+00_JPRB, 0.63876E+00_JPRB, 0.62837E+00_JPRB, 0.61444E+00_JPRB, 0.60480E+00_JPRB, 0.59862E+00_JPRB ,&
&  0.59099E+00_JPRB, 0.58421E+00_JPRB, 0.57980E+00_JPRB, 0.57744E+00_JPRB, 0.57820E+00_JPRB, 0.57757E+00_JPRB, 0.57983E+00_JPRB ,&
&  0.58535E+00_JPRB, 0.59681E+00_JPRB, 0.59104E+00_JPRB, 0.59826E+00_JPRB, 0.64758E+00_JPRB, 0.66663E+00_JPRB, 0.68767E+00_JPRB ,&
&  0.70792E+00_JPRB, 0.70272E+00_JPRB, 0.68369E+00_JPRB, 0.67930E+00_JPRB, 0.79409E+00_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.66042E+00_JPRB, 0.62975E+00_JPRB, 0.58817E+00_JPRB, 0.56163E+00_JPRB, 0.54328E+00_JPRB, 0.52198E+00_JPRB, 0.50675E+00_JPRB ,&
&  0.49523E+00_JPRB, 0.48676E+00_JPRB, 0.47738E+00_JPRB, 0.46580E+00_JPRB, 0.45478E+00_JPRB, 0.44740E+00_JPRB, 0.44060E+00_JPRB ,&
&  0.43375E+00_JPRB, 0.42714E+00_JPRB, 0.42067E+00_JPRB, 0.41578E+00_JPRB, 0.40981E+00_JPRB, 0.40504E+00_JPRB, 0.40065E+00_JPRB ,&
&  0.39528E+00_JPRB, 0.39003E+00_JPRB, 0.38370E+00_JPRB, 0.38754E+00_JPRB, 0.39600E+00_JPRB, 0.40310E+00_JPRB, 0.41257E+00_JPRB ,&
&  0.44380E+00_JPRB, 0.43595E+00_JPRB, 0.47506E+00_JPRB, 0.51821E+00_JPRB, 0.54749E+00_JPRB, 0.57432E+00_JPRB, 0.58484E+00_JPRB ,&
&  0.57764E+00_JPRB, 0.58767E+00_JPRB, 0.58185E+00_JPRB, 0.56271E+00_JPRB, 0.54761E+00_JPRB, 0.51707E+00_JPRB, 0.48983E+00_JPRB ,&
&  0.47101E+00_JPRB, 0.45898E+00_JPRB, 0.44801E+00_JPRB, 0.43569E+00_JPRB, 0.42817E+00_JPRB, 0.42682E+00_JPRB, 0.42954E+00_JPRB ,&
&  0.43232E+00_JPRB, 0.43704E+00_JPRB, 0.44217E+00_JPRB, 0.44890E+00_JPRB, 0.46487E+00_JPRB, 0.53920E+00_JPRB, 0.72860E+00_JPRB ,&
&  0.86716E+00_JPRB, 0.94835E+00_JPRB, 0.10004E+01_JPRB, 0.10220E+01_JPRB, 0.99218E+00_JPRB, 0.93684E+00_JPRB, 0.87090E+00_JPRB ,&
&  0.80232E+00_JPRB, 0.73064E+00_JPRB, 0.66552E+00_JPRB, 0.61532E+00_JPRB, 0.57660E+00_JPRB, 0.55132E+00_JPRB, 0.53898E+00_JPRB ,&
&  0.53883E+00_JPRB, 0.54730E+00_JPRB, 0.55952E+00_JPRB, 0.57111E+00_JPRB, 0.57749E+00_JPRB, 0.57737E+00_JPRB, 0.57391E+00_JPRB ,&
&  0.56788E+00_JPRB, 0.56021E+00_JPRB, 0.55286E+00_JPRB, 0.54754E+00_JPRB, 0.54430E+00_JPRB, 0.54275E+00_JPRB, 0.54306E+00_JPRB ,&
&  0.54514E+00_JPRB, 0.54991E+00_JPRB, 0.55897E+00_JPRB, 0.58164E+00_JPRB, 0.59133E+00_JPRB, 0.60764E+00_JPRB, 0.62024E+00_JPRB ,&
&  0.63538E+00_JPRB, 0.64090E+00_JPRB, 0.63374E+00_JPRB, 0.61309E+00_JPRB, 0.62866E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.57409E+00_JPRB, 0.56650E+00_JPRB, 0.54640E+00_JPRB, 0.52393E+00_JPRB, 0.50441E+00_JPRB, 0.48662E+00_JPRB, 0.47216E+00_JPRB ,&
&  0.46167E+00_JPRB, 0.45305E+00_JPRB, 0.44395E+00_JPRB, 0.43552E+00_JPRB, 0.42681E+00_JPRB, 0.41977E+00_JPRB, 0.41235E+00_JPRB ,&
&  0.40732E+00_JPRB, 0.40399E+00_JPRB, 0.40080E+00_JPRB, 0.39658E+00_JPRB, 0.39190E+00_JPRB, 0.38700E+00_JPRB, 0.38115E+00_JPRB ,&
&  0.37632E+00_JPRB, 0.37373E+00_JPRB, 0.36854E+00_JPRB, 0.36705E+00_JPRB, 0.37324E+00_JPRB, 0.38107E+00_JPRB, 0.39001E+00_JPRB ,&
&  0.41560E+00_JPRB, 0.37938E+00_JPRB, 0.39134E+00_JPRB, 0.43726E+00_JPRB, 0.46856E+00_JPRB, 0.50763E+00_JPRB, 0.53172E+00_JPRB ,&
&  0.54115E+00_JPRB, 0.54986E+00_JPRB, 0.54154E+00_JPRB, 0.52653E+00_JPRB, 0.50081E+00_JPRB, 0.47770E+00_JPRB, 0.45950E+00_JPRB ,&
&  0.44387E+00_JPRB, 0.43077E+00_JPRB, 0.42014E+00_JPRB, 0.41119E+00_JPRB, 0.40305E+00_JPRB, 0.39959E+00_JPRB, 0.40047E+00_JPRB ,&
&  0.40223E+00_JPRB, 0.40473E+00_JPRB, 0.40854E+00_JPRB, 0.41538E+00_JPRB, 0.42401E+00_JPRB, 0.44151E+00_JPRB, 0.50512E+00_JPRB ,&
&  0.60608E+00_JPRB, 0.67776E+00_JPRB, 0.73820E+00_JPRB, 0.77111E+00_JPRB, 0.76695E+00_JPRB, 0.73261E+00_JPRB, 0.67189E+00_JPRB ,&
&  0.60834E+00_JPRB, 0.55967E+00_JPRB, 0.53006E+00_JPRB, 0.51248E+00_JPRB, 0.50324E+00_JPRB, 0.49645E+00_JPRB, 0.49114E+00_JPRB ,&
&  0.48631E+00_JPRB, 0.48365E+00_JPRB, 0.48549E+00_JPRB, 0.49160E+00_JPRB, 0.50031E+00_JPRB, 0.50747E+00_JPRB, 0.51033E+00_JPRB ,&
&  0.50967E+00_JPRB, 0.50721E+00_JPRB, 0.50585E+00_JPRB, 0.50461E+00_JPRB, 0.50293E+00_JPRB, 0.49978E+00_JPRB, 0.49891E+00_JPRB ,&
&  0.50122E+00_JPRB, 0.50664E+00_JPRB, 0.51607E+00_JPRB, 0.52231E+00_JPRB, 0.54375E+00_JPRB, 0.55370E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56788E+00_JPRB, 0.56798E+00_JPRB, 0.56822E+00_JPRB, 0.50068E+00_JPRB, 0.55804E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.49660E+00_JPRB, 0.49530E+00_JPRB, 0.48261E+00_JPRB, 0.46867E+00_JPRB, 0.45452E+00_JPRB, 0.44111E+00_JPRB, 0.43153E+00_JPRB ,&
&  0.42496E+00_JPRB, 0.41608E+00_JPRB, 0.40836E+00_JPRB, 0.40337E+00_JPRB, 0.39785E+00_JPRB, 0.39287E+00_JPRB, 0.38697E+00_JPRB ,&
&  0.38240E+00_JPRB, 0.37971E+00_JPRB, 0.37785E+00_JPRB, 0.37544E+00_JPRB, 0.37244E+00_JPRB, 0.36851E+00_JPRB, 0.36446E+00_JPRB ,&
&  0.36045E+00_JPRB, 0.35764E+00_JPRB, 0.35418E+00_JPRB, 0.35235E+00_JPRB, 0.35489E+00_JPRB, 0.36064E+00_JPRB, 0.36620E+00_JPRB ,&
&  0.38584E+00_JPRB, 0.33849E+00_JPRB, 0.35173E+00_JPRB, 0.37463E+00_JPRB, 0.39997E+00_JPRB, 0.43421E+00_JPRB, 0.46303E+00_JPRB ,&
&  0.47838E+00_JPRB, 0.49389E+00_JPRB, 0.49051E+00_JPRB, 0.48090E+00_JPRB, 0.46269E+00_JPRB, 0.44342E+00_JPRB, 0.42721E+00_JPRB ,&
&  0.41654E+00_JPRB, 0.40902E+00_JPRB, 0.39999E+00_JPRB, 0.38977E+00_JPRB, 0.38006E+00_JPRB, 0.37323E+00_JPRB, 0.37044E+00_JPRB ,&
&  0.37093E+00_JPRB, 0.37184E+00_JPRB, 0.37193E+00_JPRB, 0.37452E+00_JPRB, 0.38045E+00_JPRB, 0.38900E+00_JPRB, 0.40698E+00_JPRB ,&
&  0.44481E+00_JPRB, 0.49575E+00_JPRB, 0.53910E+00_JPRB, 0.57867E+00_JPRB, 0.60280E+00_JPRB, 0.60801E+00_JPRB, 0.58086E+00_JPRB ,&
&  0.54406E+00_JPRB, 0.51057E+00_JPRB, 0.48878E+00_JPRB, 0.47733E+00_JPRB, 0.47013E+00_JPRB, 0.46507E+00_JPRB, 0.45800E+00_JPRB ,&
&  0.44834E+00_JPRB, 0.43709E+00_JPRB, 0.42719E+00_JPRB, 0.41850E+00_JPRB, 0.41259E+00_JPRB, 0.41140E+00_JPRB, 0.41364E+00_JPRB ,&
&  0.41688E+00_JPRB, 0.41959E+00_JPRB, 0.42228E+00_JPRB, 0.42610E+00_JPRB, 0.43001E+00_JPRB, 0.43418E+00_JPRB, 0.43747E+00_JPRB ,&
&  0.44234E+00_JPRB, 0.44845E+00_JPRB, 0.45528E+00_JPRB, 0.46935E+00_JPRB, 0.48097E+00_JPRB, 0.48881E+00_JPRB, 0.49495E+00_JPRB ,&
&  0.49759E+00_JPRB, 0.50430E+00_JPRB, 0.43354E+00_JPRB, 0.46847E+00_JPRB, 0.48723E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.43993E+00_JPRB, 0.43901E+00_JPRB, 0.43453E+00_JPRB, 0.42425E+00_JPRB, 0.41260E+00_JPRB, 0.40269E+00_JPRB, 0.39670E+00_JPRB ,&
&  0.39313E+00_JPRB, 0.38776E+00_JPRB, 0.38316E+00_JPRB, 0.38010E+00_JPRB, 0.37613E+00_JPRB, 0.37172E+00_JPRB, 0.36545E+00_JPRB ,&
&  0.36038E+00_JPRB, 0.35646E+00_JPRB, 0.35352E+00_JPRB, 0.35176E+00_JPRB, 0.35076E+00_JPRB, 0.34958E+00_JPRB, 0.34763E+00_JPRB ,&
&  0.34551E+00_JPRB, 0.34407E+00_JPRB, 0.34288E+00_JPRB, 0.34283E+00_JPRB, 0.34340E+00_JPRB, 0.34678E+00_JPRB, 0.35250E+00_JPRB ,&
&  0.33889E+00_JPRB, 0.31366E+00_JPRB, 0.32567E+00_JPRB, 0.33979E+00_JPRB, 0.35608E+00_JPRB, 0.38234E+00_JPRB, 0.41041E+00_JPRB ,&
&  0.42656E+00_JPRB, 0.43680E+00_JPRB, 0.43161E+00_JPRB, 0.42220E+00_JPRB, 0.41171E+00_JPRB, 0.40123E+00_JPRB, 0.39293E+00_JPRB ,&
&  0.38463E+00_JPRB, 0.37890E+00_JPRB, 0.37388E+00_JPRB, 0.36644E+00_JPRB, 0.35733E+00_JPRB, 0.34982E+00_JPRB, 0.34531E+00_JPRB ,&
&  0.34369E+00_JPRB, 0.34423E+00_JPRB, 0.34520E+00_JPRB, 0.34649E+00_JPRB, 0.34918E+00_JPRB, 0.35450E+00_JPRB, 0.36231E+00_JPRB ,&
&  0.37186E+00_JPRB, 0.38935E+00_JPRB, 0.41396E+00_JPRB, 0.43734E+00_JPRB, 0.45940E+00_JPRB, 0.47414E+00_JPRB, 0.48212E+00_JPRB ,&
&  0.48076E+00_JPRB, 0.47038E+00_JPRB, 0.45679E+00_JPRB, 0.44275E+00_JPRB, 0.42663E+00_JPRB, 0.40769E+00_JPRB, 0.38794E+00_JPRB ,&
&  0.37161E+00_JPRB, 0.35909E+00_JPRB, 0.35077E+00_JPRB, 0.34723E+00_JPRB, 0.34639E+00_JPRB, 0.34844E+00_JPRB, 0.35193E+00_JPRB ,&
&  0.35608E+00_JPRB, 0.36053E+00_JPRB, 0.36388E+00_JPRB, 0.36836E+00_JPRB, 0.37287E+00_JPRB, 0.37814E+00_JPRB, 0.38383E+00_JPRB ,&
&  0.39030E+00_JPRB, 0.39649E+00_JPRB, 0.40332E+00_JPRB, 0.41488E+00_JPRB, 0.42498E+00_JPRB, 0.43418E+00_JPRB, 0.43998E+00_JPRB ,&
&  0.43982E+00_JPRB, 0.43998E+00_JPRB, 0.43983E+00_JPRB, 0.43812E+00_JPRB, 0.44089E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.39817E+00_JPRB, 0.39595E+00_JPRB, 0.39390E+00_JPRB, 0.39116E+00_JPRB, 0.38685E+00_JPRB, 0.38052E+00_JPRB, 0.37564E+00_JPRB ,&
&  0.37253E+00_JPRB, 0.36901E+00_JPRB, 0.36608E+00_JPRB, 0.36236E+00_JPRB, 0.35786E+00_JPRB, 0.35239E+00_JPRB, 0.34508E+00_JPRB ,&
&  0.33844E+00_JPRB, 0.33229E+00_JPRB, 0.32838E+00_JPRB, 0.32663E+00_JPRB, 0.32682E+00_JPRB, 0.32709E+00_JPRB, 0.32575E+00_JPRB ,&
&  0.32497E+00_JPRB, 0.32496E+00_JPRB, 0.32479E+00_JPRB, 0.32493E+00_JPRB, 0.32542E+00_JPRB, 0.32842E+00_JPRB, 0.33195E+00_JPRB ,&
&  0.33593E+00_JPRB, 0.31229E+00_JPRB, 0.32212E+00_JPRB, 0.32561E+00_JPRB, 0.33541E+00_JPRB, 0.35126E+00_JPRB, 0.36928E+00_JPRB ,&
&  0.38251E+00_JPRB, 0.38329E+00_JPRB, 0.37864E+00_JPRB, 0.37094E+00_JPRB, 0.36503E+00_JPRB, 0.35971E+00_JPRB, 0.35396E+00_JPRB ,&
&  0.34835E+00_JPRB, 0.34464E+00_JPRB, 0.34138E+00_JPRB, 0.33811E+00_JPRB, 0.33333E+00_JPRB, 0.32717E+00_JPRB, 0.32106E+00_JPRB ,&
&  0.31664E+00_JPRB, 0.31494E+00_JPRB, 0.31553E+00_JPRB, 0.31720E+00_JPRB, 0.31945E+00_JPRB, 0.32335E+00_JPRB, 0.32821E+00_JPRB ,&
&  0.33350E+00_JPRB, 0.33676E+00_JPRB, 0.34082E+00_JPRB, 0.34862E+00_JPRB, 0.35714E+00_JPRB, 0.36354E+00_JPRB, 0.36714E+00_JPRB ,&
&  0.36832E+00_JPRB, 0.36571E+00_JPRB, 0.36072E+00_JPRB, 0.35382E+00_JPRB, 0.34614E+00_JPRB, 0.33832E+00_JPRB, 0.33146E+00_JPRB ,&
&  0.32586E+00_JPRB, 0.32083E+00_JPRB, 0.31627E+00_JPRB, 0.31301E+00_JPRB, 0.31175E+00_JPRB, 0.31187E+00_JPRB, 0.31319E+00_JPRB ,&
&  0.31640E+00_JPRB, 0.32025E+00_JPRB, 0.32340E+00_JPRB, 0.32664E+00_JPRB, 0.32975E+00_JPRB, 0.33269E+00_JPRB, 0.33684E+00_JPRB ,&
&  0.34263E+00_JPRB, 0.34724E+00_JPRB, 0.35157E+00_JPRB, 0.35777E+00_JPRB, 0.36548E+00_JPRB, 0.37407E+00_JPRB, 0.38211E+00_JPRB ,&
&  0.38759E+00_JPRB, 0.39034E+00_JPRB, 0.39366E+00_JPRB, 0.39550E+00_JPRB, 0.39880E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.37159E+00_JPRB, 0.37004E+00_JPRB, 0.36734E+00_JPRB, 0.36570E+00_JPRB, 0.36345E+00_JPRB, 0.36023E+00_JPRB, 0.35663E+00_JPRB ,&
&  0.35370E+00_JPRB, 0.35093E+00_JPRB, 0.34808E+00_JPRB, 0.34410E+00_JPRB, 0.33858E+00_JPRB, 0.33324E+00_JPRB, 0.32687E+00_JPRB ,&
&  0.32082E+00_JPRB, 0.31557E+00_JPRB, 0.31205E+00_JPRB, 0.31012E+00_JPRB, 0.30848E+00_JPRB, 0.30720E+00_JPRB, 0.30563E+00_JPRB ,&
&  0.30483E+00_JPRB, 0.30466E+00_JPRB, 0.30472E+00_JPRB, 0.30520E+00_JPRB, 0.30594E+00_JPRB, 0.30857E+00_JPRB, 0.31031E+00_JPRB ,&
&  0.31229E+00_JPRB, 0.30869E+00_JPRB, 0.31527E+00_JPRB, 0.31987E+00_JPRB, 0.32708E+00_JPRB, 0.33265E+00_JPRB, 0.33691E+00_JPRB ,&
&  0.34033E+00_JPRB, 0.34219E+00_JPRB, 0.33896E+00_JPRB, 0.33427E+00_JPRB, 0.33055E+00_JPRB, 0.32835E+00_JPRB, 0.32387E+00_JPRB ,&
&  0.31924E+00_JPRB, 0.31592E+00_JPRB, 0.31328E+00_JPRB, 0.31144E+00_JPRB, 0.30948E+00_JPRB, 0.30655E+00_JPRB, 0.30329E+00_JPRB ,&
&  0.29910E+00_JPRB, 0.29600E+00_JPRB, 0.29352E+00_JPRB, 0.29255E+00_JPRB, 0.29277E+00_JPRB, 0.29391E+00_JPRB, 0.29565E+00_JPRB ,&
&  0.29843E+00_JPRB, 0.30120E+00_JPRB, 0.30271E+00_JPRB, 0.30364E+00_JPRB, 0.30470E+00_JPRB, 0.30644E+00_JPRB, 0.30763E+00_JPRB ,&
&  0.30748E+00_JPRB, 0.30640E+00_JPRB, 0.30458E+00_JPRB, 0.30203E+00_JPRB, 0.29902E+00_JPRB, 0.29666E+00_JPRB, 0.29465E+00_JPRB ,&
&  0.29256E+00_JPRB, 0.29022E+00_JPRB, 0.28776E+00_JPRB, 0.28642E+00_JPRB, 0.28559E+00_JPRB, 0.28525E+00_JPRB, 0.28617E+00_JPRB ,&
&  0.28839E+00_JPRB, 0.29108E+00_JPRB, 0.29407E+00_JPRB, 0.29681E+00_JPRB, 0.29940E+00_JPRB, 0.30169E+00_JPRB, 0.30471E+00_JPRB ,&
&  0.30974E+00_JPRB, 0.31528E+00_JPRB, 0.32082E+00_JPRB, 0.32619E+00_JPRB, 0.33239E+00_JPRB, 0.33999E+00_JPRB, 0.34803E+00_JPRB ,&
&  0.35364E+00_JPRB, 0.35738E+00_JPRB, 0.36162E+00_JPRB, 0.36623E+00_JPRB, 0.37032E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.33661E+00_JPRB, 0.33826E+00_JPRB, 0.33856E+00_JPRB, 0.33919E+00_JPRB, 0.33895E+00_JPRB, 0.33749E+00_JPRB, 0.33467E+00_JPRB ,&
&  0.33158E+00_JPRB, 0.32800E+00_JPRB, 0.32440E+00_JPRB, 0.31968E+00_JPRB, 0.31472E+00_JPRB, 0.31067E+00_JPRB, 0.30616E+00_JPRB ,&
&  0.30152E+00_JPRB, 0.29721E+00_JPRB, 0.29430E+00_JPRB, 0.29198E+00_JPRB, 0.28972E+00_JPRB, 0.28762E+00_JPRB, 0.28476E+00_JPRB ,&
&  0.28230E+00_JPRB, 0.27994E+00_JPRB, 0.28035E+00_JPRB, 0.28276E+00_JPRB, 0.28473E+00_JPRB, 0.28617E+00_JPRB, 0.28737E+00_JPRB ,&
&  0.28772E+00_JPRB, 0.29302E+00_JPRB, 0.28722E+00_JPRB, 0.29617E+00_JPRB, 0.29837E+00_JPRB, 0.29899E+00_JPRB, 0.29918E+00_JPRB ,&
&  0.30100E+00_JPRB, 0.30310E+00_JPRB, 0.30437E+00_JPRB, 0.30164E+00_JPRB, 0.29785E+00_JPRB, 0.29492E+00_JPRB, 0.29150E+00_JPRB ,&
&  0.28681E+00_JPRB, 0.28499E+00_JPRB, 0.28514E+00_JPRB, 0.28557E+00_JPRB, 0.28573E+00_JPRB, 0.28532E+00_JPRB, 0.28427E+00_JPRB ,&
&  0.28284E+00_JPRB, 0.28074E+00_JPRB, 0.27824E+00_JPRB, 0.27571E+00_JPRB, 0.27410E+00_JPRB, 0.27217E+00_JPRB, 0.27035E+00_JPRB ,&
&  0.26956E+00_JPRB, 0.26994E+00_JPRB, 0.27035E+00_JPRB, 0.27045E+00_JPRB, 0.26992E+00_JPRB, 0.26963E+00_JPRB, 0.26973E+00_JPRB ,&
&  0.26938E+00_JPRB, 0.26876E+00_JPRB, 0.26788E+00_JPRB, 0.26655E+00_JPRB, 0.26504E+00_JPRB, 0.26425E+00_JPRB, 0.26422E+00_JPRB ,&
&  0.26388E+00_JPRB, 0.26239E+00_JPRB, 0.26017E+00_JPRB, 0.25967E+00_JPRB, 0.25959E+00_JPRB, 0.25987E+00_JPRB, 0.26040E+00_JPRB ,&
&  0.26159E+00_JPRB, 0.26375E+00_JPRB, 0.26616E+00_JPRB, 0.26849E+00_JPRB, 0.27081E+00_JPRB, 0.27414E+00_JPRB, 0.27814E+00_JPRB ,&
&  0.28270E+00_JPRB, 0.28766E+00_JPRB, 0.29256E+00_JPRB, 0.29731E+00_JPRB, 0.30143E+00_JPRB, 0.30618E+00_JPRB, 0.31220E+00_JPRB ,&
&  0.31722E+00_JPRB, 0.32114E+00_JPRB, 0.32446E+00_JPRB, 0.32855E+00_JPRB, 0.33309E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.30704E+00_JPRB, 0.31164E+00_JPRB, 0.31421E+00_JPRB, 0.31539E+00_JPRB, 0.31565E+00_JPRB, 0.31418E+00_JPRB, 0.31181E+00_JPRB ,&
&  0.30821E+00_JPRB, 0.30468E+00_JPRB, 0.30078E+00_JPRB, 0.29628E+00_JPRB, 0.29223E+00_JPRB, 0.28885E+00_JPRB, 0.28594E+00_JPRB ,&
&  0.28301E+00_JPRB, 0.28030E+00_JPRB, 0.27811E+00_JPRB, 0.27583E+00_JPRB, 0.27317E+00_JPRB, 0.26976E+00_JPRB, 0.26526E+00_JPRB ,&
&  0.26264E+00_JPRB, 0.26061E+00_JPRB, 0.26160E+00_JPRB, 0.26439E+00_JPRB, 0.26654E+00_JPRB, 0.26789E+00_JPRB, 0.26889E+00_JPRB ,&
&  0.27029E+00_JPRB, 0.27138E+00_JPRB, 0.27238E+00_JPRB, 0.27469E+00_JPRB, 0.27691E+00_JPRB, 0.27663E+00_JPRB, 0.27506E+00_JPRB ,&
&  0.27368E+00_JPRB, 0.27389E+00_JPRB, 0.27453E+00_JPRB, 0.27481E+00_JPRB, 0.27417E+00_JPRB, 0.27209E+00_JPRB, 0.26871E+00_JPRB ,&
&  0.26266E+00_JPRB, 0.25815E+00_JPRB, 0.25713E+00_JPRB, 0.25820E+00_JPRB, 0.26001E+00_JPRB, 0.26141E+00_JPRB, 0.26229E+00_JPRB ,&
&  0.26259E+00_JPRB, 0.26207E+00_JPRB, 0.26104E+00_JPRB, 0.25980E+00_JPRB, 0.25812E+00_JPRB, 0.25613E+00_JPRB, 0.25497E+00_JPRB ,&
&  0.25364E+00_JPRB, 0.25247E+00_JPRB, 0.25168E+00_JPRB, 0.25123E+00_JPRB, 0.25014E+00_JPRB, 0.24861E+00_JPRB, 0.24778E+00_JPRB ,&
&  0.24663E+00_JPRB, 0.24603E+00_JPRB, 0.24510E+00_JPRB, 0.24419E+00_JPRB, 0.24298E+00_JPRB, 0.24180E+00_JPRB, 0.24151E+00_JPRB ,&
&  0.24166E+00_JPRB, 0.24058E+00_JPRB, 0.23925E+00_JPRB, 0.23915E+00_JPRB, 0.23969E+00_JPRB, 0.24056E+00_JPRB, 0.24129E+00_JPRB ,&
&  0.24202E+00_JPRB, 0.24294E+00_JPRB, 0.24492E+00_JPRB, 0.24714E+00_JPRB, 0.24965E+00_JPRB, 0.25271E+00_JPRB, 0.25622E+00_JPRB ,&
&  0.26045E+00_JPRB, 0.26542E+00_JPRB, 0.27106E+00_JPRB, 0.27627E+00_JPRB, 0.28072E+00_JPRB, 0.28366E+00_JPRB, 0.28737E+00_JPRB ,&
&  0.29176E+00_JPRB, 0.29470E+00_JPRB, 0.29704E+00_JPRB, 0.29973E+00_JPRB, 0.30265E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.27774E+00_JPRB, 0.28241E+00_JPRB, 0.28606E+00_JPRB, 0.28811E+00_JPRB, 0.28819E+00_JPRB, 0.28699E+00_JPRB, 0.28452E+00_JPRB ,&
&  0.28095E+00_JPRB, 0.27740E+00_JPRB, 0.27382E+00_JPRB, 0.27091E+00_JPRB, 0.26917E+00_JPRB, 0.26780E+00_JPRB, 0.26677E+00_JPRB ,&
&  0.26523E+00_JPRB, 0.26327E+00_JPRB, 0.26073E+00_JPRB, 0.25850E+00_JPRB, 0.25582E+00_JPRB, 0.25197E+00_JPRB, 0.24918E+00_JPRB ,&
&  0.24727E+00_JPRB, 0.24540E+00_JPRB, 0.24485E+00_JPRB, 0.24608E+00_JPRB, 0.24840E+00_JPRB, 0.25069E+00_JPRB, 0.25321E+00_JPRB ,&
&  0.25497E+00_JPRB, 0.25573E+00_JPRB, 0.25625E+00_JPRB, 0.25607E+00_JPRB, 0.25607E+00_JPRB, 0.25577E+00_JPRB, 0.25505E+00_JPRB ,&
&  0.25421E+00_JPRB, 0.25341E+00_JPRB, 0.25204E+00_JPRB, 0.25175E+00_JPRB, 0.25153E+00_JPRB, 0.25031E+00_JPRB, 0.24823E+00_JPRB ,&
&  0.24491E+00_JPRB, 0.24035E+00_JPRB, 0.23670E+00_JPRB, 0.23485E+00_JPRB, 0.23426E+00_JPRB, 0.23456E+00_JPRB, 0.23544E+00_JPRB ,&
&  0.23660E+00_JPRB, 0.23824E+00_JPRB, 0.23974E+00_JPRB, 0.24047E+00_JPRB, 0.24033E+00_JPRB, 0.24017E+00_JPRB, 0.23899E+00_JPRB ,&
&  0.23783E+00_JPRB, 0.23709E+00_JPRB, 0.23654E+00_JPRB, 0.23593E+00_JPRB, 0.23391E+00_JPRB, 0.23143E+00_JPRB, 0.23094E+00_JPRB ,&
&  0.22963E+00_JPRB, 0.22877E+00_JPRB, 0.22749E+00_JPRB, 0.22691E+00_JPRB, 0.22618E+00_JPRB, 0.22504E+00_JPRB, 0.22378E+00_JPRB ,&
&  0.22275E+00_JPRB, 0.22222E+00_JPRB, 0.22189E+00_JPRB, 0.22109E+00_JPRB, 0.22152E+00_JPRB, 0.22227E+00_JPRB, 0.22369E+00_JPRB ,&
&  0.22489E+00_JPRB, 0.22607E+00_JPRB, 0.22765E+00_JPRB, 0.22916E+00_JPRB, 0.23149E+00_JPRB, 0.23427E+00_JPRB, 0.23746E+00_JPRB ,&
&  0.24013E+00_JPRB, 0.24382E+00_JPRB, 0.24804E+00_JPRB, 0.25200E+00_JPRB, 0.25511E+00_JPRB, 0.25729E+00_JPRB, 0.26019E+00_JPRB ,&
&  0.26489E+00_JPRB, 0.26863E+00_JPRB, 0.27029E+00_JPRB, 0.27230E+00_JPRB, 0.27410E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.25671E+00_JPRB, 0.25814E+00_JPRB, 0.25969E+00_JPRB, 0.26151E+00_JPRB, 0.26275E+00_JPRB, 0.26319E+00_JPRB, 0.26186E+00_JPRB ,&
&  0.25936E+00_JPRB, 0.25658E+00_JPRB, 0.25381E+00_JPRB, 0.25148E+00_JPRB, 0.25018E+00_JPRB, 0.24951E+00_JPRB, 0.24919E+00_JPRB ,&
&  0.24897E+00_JPRB, 0.24869E+00_JPRB, 0.24698E+00_JPRB, 0.24434E+00_JPRB, 0.24084E+00_JPRB, 0.23786E+00_JPRB, 0.23711E+00_JPRB ,&
&  0.23565E+00_JPRB, 0.23461E+00_JPRB, 0.23392E+00_JPRB, 0.23318E+00_JPRB, 0.23292E+00_JPRB, 0.23390E+00_JPRB, 0.23565E+00_JPRB ,&
&  0.23755E+00_JPRB, 0.23904E+00_JPRB, 0.24020E+00_JPRB, 0.24108E+00_JPRB, 0.24058E+00_JPRB, 0.23946E+00_JPRB, 0.23815E+00_JPRB ,&
&  0.23655E+00_JPRB, 0.23502E+00_JPRB, 0.23366E+00_JPRB, 0.23188E+00_JPRB, 0.22929E+00_JPRB, 0.22620E+00_JPRB, 0.22399E+00_JPRB ,&
&  0.22229E+00_JPRB, 0.22072E+00_JPRB, 0.21909E+00_JPRB, 0.21825E+00_JPRB, 0.21767E+00_JPRB, 0.21746E+00_JPRB, 0.21756E+00_JPRB ,&
&  0.21800E+00_JPRB, 0.21924E+00_JPRB, 0.22095E+00_JPRB, 0.22278E+00_JPRB, 0.22329E+00_JPRB, 0.22346E+00_JPRB, 0.22373E+00_JPRB ,&
&  0.22313E+00_JPRB, 0.22190E+00_JPRB, 0.22022E+00_JPRB, 0.21837E+00_JPRB, 0.21721E+00_JPRB, 0.21635E+00_JPRB, 0.21537E+00_JPRB ,&
&  0.21356E+00_JPRB, 0.21242E+00_JPRB, 0.21177E+00_JPRB, 0.21162E+00_JPRB, 0.21180E+00_JPRB, 0.21184E+00_JPRB, 0.21118E+00_JPRB ,&
&  0.21036E+00_JPRB, 0.20965E+00_JPRB, 0.20872E+00_JPRB, 0.20781E+00_JPRB, 0.20627E+00_JPRB, 0.20672E+00_JPRB, 0.20769E+00_JPRB ,&
&  0.20812E+00_JPRB, 0.20831E+00_JPRB, 0.20855E+00_JPRB, 0.20970E+00_JPRB, 0.21096E+00_JPRB, 0.21269E+00_JPRB, 0.21517E+00_JPRB ,&
&  0.21722E+00_JPRB, 0.21990E+00_JPRB, 0.22384E+00_JPRB, 0.22747E+00_JPRB, 0.23011E+00_JPRB, 0.23137E+00_JPRB, 0.23385E+00_JPRB ,&
&  0.23935E+00_JPRB, 0.24523E+00_JPRB, 0.24892E+00_JPRB, 0.25250E+00_JPRB, 0.25558E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.23586E+00_JPRB, 0.23647E+00_JPRB, 0.23717E+00_JPRB, 0.23823E+00_JPRB, 0.23978E+00_JPRB, 0.24065E+00_JPRB, 0.24135E+00_JPRB ,&
&  0.24076E+00_JPRB, 0.23914E+00_JPRB, 0.23725E+00_JPRB, 0.23525E+00_JPRB, 0.23446E+00_JPRB, 0.23455E+00_JPRB, 0.23488E+00_JPRB ,&
&  0.23460E+00_JPRB, 0.23464E+00_JPRB, 0.23309E+00_JPRB, 0.23031E+00_JPRB, 0.22703E+00_JPRB, 0.22483E+00_JPRB, 0.22426E+00_JPRB ,&
&  0.22375E+00_JPRB, 0.22291E+00_JPRB, 0.22257E+00_JPRB, 0.22219E+00_JPRB, 0.22024E+00_JPRB, 0.21921E+00_JPRB, 0.21957E+00_JPRB ,&
&  0.22036E+00_JPRB, 0.22173E+00_JPRB, 0.22197E+00_JPRB, 0.21942E+00_JPRB, 0.22164E+00_JPRB, 0.22134E+00_JPRB, 0.21947E+00_JPRB ,&
&  0.21709E+00_JPRB, 0.21463E+00_JPRB, 0.21239E+00_JPRB, 0.21021E+00_JPRB, 0.20802E+00_JPRB, 0.20635E+00_JPRB, 0.20524E+00_JPRB ,&
&  0.20491E+00_JPRB, 0.20502E+00_JPRB, 0.20540E+00_JPRB, 0.20628E+00_JPRB, 0.20694E+00_JPRB, 0.20730E+00_JPRB, 0.20735E+00_JPRB ,&
&  0.20747E+00_JPRB, 0.20768E+00_JPRB, 0.20806E+00_JPRB, 0.20829E+00_JPRB, 0.20856E+00_JPRB, 0.20884E+00_JPRB, 0.20880E+00_JPRB ,&
&  0.20784E+00_JPRB, 0.20619E+00_JPRB, 0.20392E+00_JPRB, 0.20313E+00_JPRB, 0.20312E+00_JPRB, 0.20176E+00_JPRB, 0.19949E+00_JPRB ,&
&  0.19726E+00_JPRB, 0.19670E+00_JPRB, 0.19771E+00_JPRB, 0.19888E+00_JPRB, 0.19982E+00_JPRB, 0.20011E+00_JPRB, 0.19983E+00_JPRB ,&
&  0.19889E+00_JPRB, 0.19686E+00_JPRB, 0.19438E+00_JPRB, 0.19249E+00_JPRB, 0.19151E+00_JPRB, 0.19157E+00_JPRB, 0.19246E+00_JPRB ,&
&  0.19260E+00_JPRB, 0.19263E+00_JPRB, 0.19367E+00_JPRB, 0.19458E+00_JPRB, 0.19521E+00_JPRB, 0.19602E+00_JPRB, 0.19748E+00_JPRB ,&
&  0.19948E+00_JPRB, 0.20264E+00_JPRB, 0.20648E+00_JPRB, 0.21058E+00_JPRB, 0.21343E+00_JPRB, 0.21264E+00_JPRB, 0.21323E+00_JPRB ,&
&  0.21760E+00_JPRB, 0.22286E+00_JPRB, 0.22718E+00_JPRB, 0.23056E+00_JPRB, 0.23362E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.21784E+00_JPRB, 0.21879E+00_JPRB, 0.21968E+00_JPRB, 0.22054E+00_JPRB, 0.22218E+00_JPRB, 0.22344E+00_JPRB, 0.22500E+00_JPRB ,&
&  0.22614E+00_JPRB, 0.22490E+00_JPRB, 0.22327E+00_JPRB, 0.22201E+00_JPRB, 0.22158E+00_JPRB, 0.22109E+00_JPRB, 0.22077E+00_JPRB ,&
&  0.22166E+00_JPRB, 0.22115E+00_JPRB, 0.21871E+00_JPRB, 0.21697E+00_JPRB, 0.21396E+00_JPRB, 0.21197E+00_JPRB, 0.21158E+00_JPRB ,&
&  0.21313E+00_JPRB, 0.21292E+00_JPRB, 0.21239E+00_JPRB, 0.21148E+00_JPRB, 0.21020E+00_JPRB, 0.20955E+00_JPRB, 0.20845E+00_JPRB ,&
&  0.20783E+00_JPRB, 0.20693E+00_JPRB, 0.20222E+00_JPRB, 0.20283E+00_JPRB, 0.20239E+00_JPRB, 0.20225E+00_JPRB, 0.20166E+00_JPRB ,&
&  0.20100E+00_JPRB, 0.19999E+00_JPRB, 0.19823E+00_JPRB, 0.19631E+00_JPRB, 0.19469E+00_JPRB, 0.19379E+00_JPRB, 0.19365E+00_JPRB ,&
&  0.19395E+00_JPRB, 0.19481E+00_JPRB, 0.19633E+00_JPRB, 0.19836E+00_JPRB, 0.19923E+00_JPRB, 0.19986E+00_JPRB, 0.20080E+00_JPRB ,&
&  0.20119E+00_JPRB, 0.20126E+00_JPRB, 0.20126E+00_JPRB, 0.20124E+00_JPRB, 0.20102E+00_JPRB, 0.20029E+00_JPRB, 0.19973E+00_JPRB ,&
&  0.19826E+00_JPRB, 0.19601E+00_JPRB, 0.19451E+00_JPRB, 0.19262E+00_JPRB, 0.18741E+00_JPRB, 0.17724E+00_JPRB, 0.17157E+00_JPRB ,&
&  0.17161E+00_JPRB, 0.17620E+00_JPRB, 0.18201E+00_JPRB, 0.18509E+00_JPRB, 0.18995E+00_JPRB, 0.19015E+00_JPRB, 0.18675E+00_JPRB ,&
&  0.17900E+00_JPRB, 0.17348E+00_JPRB, 0.16964E+00_JPRB, 0.16568E+00_JPRB, 0.16327E+00_JPRB, 0.16278E+00_JPRB, 0.16329E+00_JPRB ,&
&  0.16526E+00_JPRB, 0.16813E+00_JPRB, 0.17091E+00_JPRB, 0.17245E+00_JPRB, 0.17259E+00_JPRB, 0.17125E+00_JPRB, 0.16971E+00_JPRB ,&
&  0.16927E+00_JPRB, 0.17305E+00_JPRB, 0.18103E+00_JPRB, 0.19012E+00_JPRB, 0.19422E+00_JPRB, 0.19583E+00_JPRB, 0.19745E+00_JPRB ,&
&  0.20082E+00_JPRB, 0.20540E+00_JPRB, 0.21002E+00_JPRB, 0.21322E+00_JPRB, 0.21583E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.19974E+00_JPRB, 0.20071E+00_JPRB, 0.20180E+00_JPRB, 0.20298E+00_JPRB, 0.20433E+00_JPRB, 0.20587E+00_JPRB, 0.20717E+00_JPRB ,&
&  0.20863E+00_JPRB, 0.20959E+00_JPRB, 0.20983E+00_JPRB, 0.20883E+00_JPRB, 0.20812E+00_JPRB, 0.20838E+00_JPRB, 0.20944E+00_JPRB ,&
&  0.20775E+00_JPRB, 0.20501E+00_JPRB, 0.20392E+00_JPRB, 0.20384E+00_JPRB, 0.20335E+00_JPRB, 0.20309E+00_JPRB, 0.20382E+00_JPRB ,&
&  0.20474E+00_JPRB, 0.20440E+00_JPRB, 0.20406E+00_JPRB, 0.20301E+00_JPRB, 0.20213E+00_JPRB, 0.20142E+00_JPRB, 0.20043E+00_JPRB ,&
&  0.19734E+00_JPRB, 0.18751E+00_JPRB, 0.18097E+00_JPRB, 0.18407E+00_JPRB, 0.19102E+00_JPRB, 0.19040E+00_JPRB, 0.19039E+00_JPRB ,&
&  0.18990E+00_JPRB, 0.18859E+00_JPRB, 0.18712E+00_JPRB, 0.18575E+00_JPRB, 0.18514E+00_JPRB, 0.18539E+00_JPRB, 0.18615E+00_JPRB ,&
&  0.18701E+00_JPRB, 0.18902E+00_JPRB, 0.19135E+00_JPRB, 0.19243E+00_JPRB, 0.19348E+00_JPRB, 0.19399E+00_JPRB, 0.19416E+00_JPRB ,&
&  0.19382E+00_JPRB, 0.19285E+00_JPRB, 0.19220E+00_JPRB, 0.19155E+00_JPRB, 0.19093E+00_JPRB, 0.18991E+00_JPRB, 0.18650E+00_JPRB ,&
&  0.18122E+00_JPRB, 0.17614E+00_JPRB, 0.16994E+00_JPRB, 0.16375E+00_JPRB, 0.15812E+00_JPRB, 0.15404E+00_JPRB, 0.15074E+00_JPRB ,&
&  0.15022E+00_JPRB, 0.15376E+00_JPRB, 0.16122E+00_JPRB, 0.17052E+00_JPRB, 0.17675E+00_JPRB, 0.16670E+00_JPRB, 0.16055E+00_JPRB ,&
&  0.15489E+00_JPRB, 0.15047E+00_JPRB, 0.14783E+00_JPRB, 0.14647E+00_JPRB, 0.14559E+00_JPRB, 0.14569E+00_JPRB, 0.14633E+00_JPRB ,&
&  0.14759E+00_JPRB, 0.14896E+00_JPRB, 0.15024E+00_JPRB, 0.15103E+00_JPRB, 0.15131E+00_JPRB, 0.15114E+00_JPRB, 0.15077E+00_JPRB ,&
&  0.15118E+00_JPRB, 0.15334E+00_JPRB, 0.15726E+00_JPRB, 0.16191E+00_JPRB, 0.16592E+00_JPRB, 0.16972E+00_JPRB, 0.17499E+00_JPRB ,&
&  0.18163E+00_JPRB, 0.18710E+00_JPRB, 0.19140E+00_JPRB, 0.19582E+00_JPRB, 0.19824E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.18645E+00_JPRB, 0.18816E+00_JPRB, 0.18875E+00_JPRB, 0.18991E+00_JPRB, 0.19029E+00_JPRB, 0.19155E+00_JPRB, 0.19256E+00_JPRB ,&
&  0.19494E+00_JPRB, 0.19711E+00_JPRB, 0.19761E+00_JPRB, 0.19585E+00_JPRB, 0.19485E+00_JPRB, 0.19225E+00_JPRB, 0.19130E+00_JPRB ,&
&  0.19116E+00_JPRB, 0.19224E+00_JPRB, 0.19245E+00_JPRB, 0.19256E+00_JPRB, 0.19257E+00_JPRB, 0.19256E+00_JPRB, 0.19302E+00_JPRB ,&
&  0.19108E+00_JPRB, 0.19038E+00_JPRB, 0.19241E+00_JPRB, 0.19162E+00_JPRB, 0.19096E+00_JPRB, 0.18966E+00_JPRB, 0.18894E+00_JPRB ,&
&  0.18504E+00_JPRB, 0.17942E+00_JPRB, 0.17277E+00_JPRB, 0.17555E+00_JPRB, 0.18265E+00_JPRB, 0.18051E+00_JPRB, 0.17976E+00_JPRB ,&
&  0.17916E+00_JPRB, 0.17799E+00_JPRB, 0.17752E+00_JPRB, 0.17705E+00_JPRB, 0.17760E+00_JPRB, 0.17817E+00_JPRB, 0.17975E+00_JPRB ,&
&  0.18196E+00_JPRB, 0.18513E+00_JPRB, 0.18500E+00_JPRB, 0.17893E+00_JPRB, 0.17580E+00_JPRB, 0.17464E+00_JPRB, 0.17142E+00_JPRB ,&
&  0.16766E+00_JPRB, 0.16615E+00_JPRB, 0.16517E+00_JPRB, 0.16288E+00_JPRB, 0.16223E+00_JPRB, 0.16178E+00_JPRB, 0.15952E+00_JPRB ,&
&  0.15652E+00_JPRB, 0.15429E+00_JPRB, 0.15108E+00_JPRB, 0.14777E+00_JPRB, 0.14548E+00_JPRB, 0.14441E+00_JPRB, 0.14447E+00_JPRB ,&
&  0.14618E+00_JPRB, 0.14801E+00_JPRB, 0.15363E+00_JPRB, 0.15732E+00_JPRB, 0.15446E+00_JPRB, 0.15112E+00_JPRB, 0.14762E+00_JPRB ,&
&  0.14261E+00_JPRB, 0.14024E+00_JPRB, 0.13841E+00_JPRB, 0.13710E+00_JPRB, 0.13703E+00_JPRB, 0.13780E+00_JPRB, 0.13840E+00_JPRB ,&
&  0.13915E+00_JPRB, 0.14010E+00_JPRB, 0.14056E+00_JPRB, 0.14078E+00_JPRB, 0.14082E+00_JPRB, 0.14107E+00_JPRB, 0.14166E+00_JPRB ,&
&  0.14243E+00_JPRB, 0.14420E+00_JPRB, 0.14663E+00_JPRB, 0.14919E+00_JPRB, 0.15107E+00_JPRB, 0.15258E+00_JPRB, 0.15570E+00_JPRB ,&
&  0.15932E+00_JPRB, 0.16770E+00_JPRB, 0.17960E+00_JPRB, 0.18861E+00_JPRB, 0.18530E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.18387E+00_JPRB, 0.18498E+00_JPRB, 0.18472E+00_JPRB, 0.18487E+00_JPRB, 0.18442E+00_JPRB, 0.18551E+00_JPRB, 0.18475E+00_JPRB ,&
&  0.18469E+00_JPRB, 0.18366E+00_JPRB, 0.18262E+00_JPRB, 0.18012E+00_JPRB, 0.17878E+00_JPRB, 0.17284E+00_JPRB, 0.17129E+00_JPRB ,&
&  0.17002E+00_JPRB, 0.17086E+00_JPRB, 0.17111E+00_JPRB, 0.17160E+00_JPRB, 0.17249E+00_JPRB, 0.17410E+00_JPRB, 0.17410E+00_JPRB ,&
&  0.17303E+00_JPRB, 0.17099E+00_JPRB, 0.16862E+00_JPRB, 0.16645E+00_JPRB, 0.16678E+00_JPRB, 0.16644E+00_JPRB, 0.16551E+00_JPRB ,&
&  0.16452E+00_JPRB, 0.16507E+00_JPRB, 0.16554E+00_JPRB, 0.16684E+00_JPRB, 0.17171E+00_JPRB, 0.17017E+00_JPRB, 0.16894E+00_JPRB ,&
&  0.16873E+00_JPRB, 0.16861E+00_JPRB, 0.16907E+00_JPRB, 0.16942E+00_JPRB, 0.17132E+00_JPRB, 0.17084E+00_JPRB, 0.17340E+00_JPRB ,&
&  0.17090E+00_JPRB, 0.16926E+00_JPRB, 0.16706E+00_JPRB, 0.16422E+00_JPRB, 0.16059E+00_JPRB, 0.15827E+00_JPRB, 0.15607E+00_JPRB ,&
&  0.15365E+00_JPRB, 0.15165E+00_JPRB, 0.14907E+00_JPRB, 0.14813E+00_JPRB, 0.14682E+00_JPRB, 0.14565E+00_JPRB, 0.14460E+00_JPRB ,&
&  0.14380E+00_JPRB, 0.14269E+00_JPRB, 0.14173E+00_JPRB, 0.14110E+00_JPRB, 0.14093E+00_JPRB, 0.14108E+00_JPRB, 0.14208E+00_JPRB ,&
&  0.14399E+00_JPRB, 0.14557E+00_JPRB, 0.14681E+00_JPRB, 0.14782E+00_JPRB, 0.14646E+00_JPRB, 0.14394E+00_JPRB, 0.14142E+00_JPRB ,&
&  0.13813E+00_JPRB, 0.13632E+00_JPRB, 0.13465E+00_JPRB, 0.13366E+00_JPRB, 0.13342E+00_JPRB, 0.13341E+00_JPRB, 0.13370E+00_JPRB ,&
&  0.13400E+00_JPRB, 0.13447E+00_JPRB, 0.13480E+00_JPRB, 0.13524E+00_JPRB, 0.13581E+00_JPRB, 0.13638E+00_JPRB, 0.13721E+00_JPRB ,&
&  0.13849E+00_JPRB, 0.14012E+00_JPRB, 0.14227E+00_JPRB, 0.14475E+00_JPRB, 0.14778E+00_JPRB, 0.15179E+00_JPRB, 0.15767E+00_JPRB ,&
&  0.16379E+00_JPRB, 0.17407E+00_JPRB, 0.17518E+00_JPRB, 0.18015E+00_JPRB, 0.18304E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.18158E+00_JPRB, 0.18329E+00_JPRB, 0.18400E+00_JPRB, 0.18495E+00_JPRB, 0.18476E+00_JPRB, 0.18250E+00_JPRB, 0.18021E+00_JPRB ,&
&  0.17500E+00_JPRB, 0.17077E+00_JPRB, 0.16564E+00_JPRB, 0.16084E+00_JPRB, 0.15632E+00_JPRB, 0.15341E+00_JPRB, 0.15173E+00_JPRB ,&
&  0.15177E+00_JPRB, 0.15321E+00_JPRB, 0.15428E+00_JPRB, 0.15489E+00_JPRB, 0.15532E+00_JPRB, 0.15632E+00_JPRB, 0.15647E+00_JPRB ,&
&  0.15607E+00_JPRB, 0.15498E+00_JPRB, 0.15419E+00_JPRB, 0.15504E+00_JPRB, 0.15697E+00_JPRB, 0.15899E+00_JPRB, 0.16066E+00_JPRB ,&
&  0.16215E+00_JPRB, 0.16153E+00_JPRB, 0.16152E+00_JPRB, 0.16073E+00_JPRB, 0.16015E+00_JPRB, 0.15944E+00_JPRB, 0.15917E+00_JPRB ,&
&  0.15983E+00_JPRB, 0.16037E+00_JPRB, 0.16202E+00_JPRB, 0.16293E+00_JPRB, 0.16176E+00_JPRB, 0.16347E+00_JPRB, 0.16353E+00_JPRB ,&
&  0.16269E+00_JPRB, 0.15974E+00_JPRB, 0.15659E+00_JPRB, 0.15414E+00_JPRB, 0.15159E+00_JPRB, 0.14952E+00_JPRB, 0.14781E+00_JPRB ,&
&  0.14623E+00_JPRB, 0.14537E+00_JPRB, 0.14363E+00_JPRB, 0.14216E+00_JPRB, 0.14124E+00_JPRB, 0.14027E+00_JPRB, 0.13967E+00_JPRB ,&
&  0.13927E+00_JPRB, 0.13859E+00_JPRB, 0.13845E+00_JPRB, 0.13811E+00_JPRB, 0.13824E+00_JPRB, 0.13840E+00_JPRB, 0.13875E+00_JPRB ,&
&  0.13951E+00_JPRB, 0.14039E+00_JPRB, 0.14031E+00_JPRB, 0.14016E+00_JPRB, 0.13877E+00_JPRB, 0.13724E+00_JPRB, 0.13590E+00_JPRB ,&
&  0.13421E+00_JPRB, 0.13295E+00_JPRB, 0.13218E+00_JPRB, 0.13144E+00_JPRB, 0.13134E+00_JPRB, 0.13140E+00_JPRB, 0.13140E+00_JPRB ,&
&  0.13163E+00_JPRB, 0.13194E+00_JPRB, 0.13242E+00_JPRB, 0.13335E+00_JPRB, 0.13417E+00_JPRB, 0.13534E+00_JPRB, 0.13663E+00_JPRB ,&
&  0.13767E+00_JPRB, 0.13952E+00_JPRB, 0.14146E+00_JPRB, 0.14371E+00_JPRB, 0.14815E+00_JPRB, 0.15209E+00_JPRB, 0.15922E+00_JPRB ,&
&  0.16493E+00_JPRB, 0.17428E+00_JPRB, 0.17249E+00_JPRB, 0.17618E+00_JPRB, 0.17967E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.18101E+00_JPRB, 0.18215E+00_JPRB, 0.18198E+00_JPRB, 0.18230E+00_JPRB, 0.18135E+00_JPRB, 0.17980E+00_JPRB, 0.17760E+00_JPRB ,&
&  0.17508E+00_JPRB, 0.16950E+00_JPRB, 0.16559E+00_JPRB, 0.16128E+00_JPRB, 0.15765E+00_JPRB, 0.15448E+00_JPRB, 0.15226E+00_JPRB ,&
&  0.15130E+00_JPRB, 0.15100E+00_JPRB, 0.15085E+00_JPRB, 0.15002E+00_JPRB, 0.14852E+00_JPRB, 0.14751E+00_JPRB, 0.14622E+00_JPRB ,&
&  0.14527E+00_JPRB, 0.14516E+00_JPRB, 0.14594E+00_JPRB, 0.14769E+00_JPRB, 0.15079E+00_JPRB, 0.15309E+00_JPRB, 0.15516E+00_JPRB ,&
&  0.15731E+00_JPRB, 0.15400E+00_JPRB, 0.15375E+00_JPRB, 0.15401E+00_JPRB, 0.15385E+00_JPRB, 0.15309E+00_JPRB, 0.15164E+00_JPRB ,&
&  0.15431E+00_JPRB, 0.15498E+00_JPRB, 0.15965E+00_JPRB, 0.15950E+00_JPRB, 0.15847E+00_JPRB, 0.15939E+00_JPRB, 0.15794E+00_JPRB ,&
&  0.15743E+00_JPRB, 0.15446E+00_JPRB, 0.15226E+00_JPRB, 0.14926E+00_JPRB, 0.14633E+00_JPRB, 0.14485E+00_JPRB, 0.14315E+00_JPRB ,&
&  0.14258E+00_JPRB, 0.14244E+00_JPRB, 0.14117E+00_JPRB, 0.14037E+00_JPRB, 0.13894E+00_JPRB, 0.13732E+00_JPRB, 0.13630E+00_JPRB ,&
&  0.13525E+00_JPRB, 0.13442E+00_JPRB, 0.13397E+00_JPRB, 0.13355E+00_JPRB, 0.13338E+00_JPRB, 0.13331E+00_JPRB, 0.13324E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13320E+00_JPRB, 0.13270E+00_JPRB, 0.13241E+00_JPRB, 0.13150E+00_JPRB, 0.13086E+00_JPRB, 0.13035E+00_JPRB ,&
&  0.12990E+00_JPRB, 0.12976E+00_JPRB, 0.12969E+00_JPRB, 0.12975E+00_JPRB, 0.12989E+00_JPRB, 0.13033E+00_JPRB, 0.13083E+00_JPRB ,&
&  0.13134E+00_JPRB, 0.13222E+00_JPRB, 0.13290E+00_JPRB, 0.13386E+00_JPRB, 0.13500E+00_JPRB, 0.13597E+00_JPRB, 0.13751E+00_JPRB ,&
&  0.13920E+00_JPRB, 0.14116E+00_JPRB, 0.14402E+00_JPRB, 0.14664E+00_JPRB, 0.14976E+00_JPRB, 0.15411E+00_JPRB, 0.15888E+00_JPRB ,&
&  0.16412E+00_JPRB, 0.17034E+00_JPRB, 0.17106E+00_JPRB, 0.17603E+00_JPRB, 0.17925E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.17456E+00_JPRB, 0.17521E+00_JPRB, 0.17581E+00_JPRB, 0.17605E+00_JPRB, 0.17611E+00_JPRB, 0.17542E+00_JPRB, 0.17456E+00_JPRB ,&
&  0.17090E+00_JPRB, 0.16822E+00_JPRB, 0.16525E+00_JPRB, 0.16232E+00_JPRB, 0.15974E+00_JPRB, 0.15717E+00_JPRB, 0.15484E+00_JPRB ,&
&  0.15273E+00_JPRB, 0.15116E+00_JPRB, 0.14969E+00_JPRB, 0.14766E+00_JPRB, 0.14508E+00_JPRB, 0.14272E+00_JPRB, 0.14117E+00_JPRB ,&
&  0.14070E+00_JPRB, 0.14095E+00_JPRB, 0.14227E+00_JPRB, 0.14365E+00_JPRB, 0.14513E+00_JPRB, 0.14625E+00_JPRB, 0.14737E+00_JPRB ,&
&  0.14680E+00_JPRB, 0.14641E+00_JPRB, 0.14607E+00_JPRB, 0.14548E+00_JPRB, 0.14601E+00_JPRB, 0.14673E+00_JPRB, 0.14714E+00_JPRB ,&
&  0.14763E+00_JPRB, 0.14894E+00_JPRB, 0.14985E+00_JPRB, 0.15071E+00_JPRB, 0.15058E+00_JPRB, 0.14981E+00_JPRB, 0.14841E+00_JPRB ,&
&  0.14668E+00_JPRB, 0.14486E+00_JPRB, 0.14325E+00_JPRB, 0.14188E+00_JPRB, 0.14082E+00_JPRB, 0.13966E+00_JPRB, 0.13868E+00_JPRB ,&
&  0.13820E+00_JPRB, 0.13776E+00_JPRB, 0.13712E+00_JPRB, 0.13648E+00_JPRB, 0.13550E+00_JPRB, 0.13465E+00_JPRB, 0.13376E+00_JPRB ,&
&  0.13295E+00_JPRB, 0.13218E+00_JPRB, 0.13148E+00_JPRB, 0.13076E+00_JPRB, 0.13027E+00_JPRB, 0.12982E+00_JPRB, 0.12939E+00_JPRB ,&
&  0.12894E+00_JPRB, 0.12878E+00_JPRB, 0.12845E+00_JPRB, 0.12834E+00_JPRB, 0.12815E+00_JPRB, 0.12803E+00_JPRB, 0.12820E+00_JPRB ,&
&  0.12869E+00_JPRB, 0.12919E+00_JPRB, 0.12945E+00_JPRB, 0.12978E+00_JPRB, 0.13010E+00_JPRB, 0.13053E+00_JPRB, 0.13120E+00_JPRB ,&
&  0.13172E+00_JPRB, 0.13237E+00_JPRB, 0.13292E+00_JPRB, 0.13356E+00_JPRB, 0.13474E+00_JPRB, 0.13596E+00_JPRB, 0.13735E+00_JPRB ,&
&  0.13908E+00_JPRB, 0.14090E+00_JPRB, 0.14300E+00_JPRB, 0.14505E+00_JPRB, 0.14779E+00_JPRB, 0.14951E+00_JPRB, 0.15271E+00_JPRB ,&
&  0.15625E+00_JPRB, 0.15767E+00_JPRB, 0.16943E+00_JPRB, 0.17057E+00_JPRB, 0.17216E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.14954E+00_JPRB, 0.15154E+00_JPRB, 0.15297E+00_JPRB, 0.15471E+00_JPRB, 0.15408E+00_JPRB, 0.16570E+00_JPRB, 0.16375E+00_JPRB ,&
&  0.16262E+00_JPRB, 0.15784E+00_JPRB, 0.15744E+00_JPRB, 0.15581E+00_JPRB, 0.15384E+00_JPRB, 0.15165E+00_JPRB, 0.14964E+00_JPRB ,&
&  0.14779E+00_JPRB, 0.14573E+00_JPRB, 0.14424E+00_JPRB, 0.14302E+00_JPRB, 0.14191E+00_JPRB, 0.14090E+00_JPRB, 0.14011E+00_JPRB ,&
&  0.14005E+00_JPRB, 0.14056E+00_JPRB, 0.14154E+00_JPRB, 0.14142E+00_JPRB, 0.14114E+00_JPRB, 0.14076E+00_JPRB, 0.14055E+00_JPRB ,&
&  0.14001E+00_JPRB, 0.13961E+00_JPRB, 0.13926E+00_JPRB, 0.13873E+00_JPRB, 0.13928E+00_JPRB, 0.13982E+00_JPRB, 0.14037E+00_JPRB ,&
&  0.14089E+00_JPRB, 0.14139E+00_JPRB, 0.14135E+00_JPRB, 0.14119E+00_JPRB, 0.14080E+00_JPRB, 0.14027E+00_JPRB, 0.13947E+00_JPRB ,&
&  0.13871E+00_JPRB, 0.13789E+00_JPRB, 0.13702E+00_JPRB, 0.13642E+00_JPRB, 0.13589E+00_JPRB, 0.13539E+00_JPRB, 0.13496E+00_JPRB ,&
&  0.13474E+00_JPRB, 0.13460E+00_JPRB, 0.13442E+00_JPRB, 0.13415E+00_JPRB, 0.13380E+00_JPRB, 0.13345E+00_JPRB, 0.13313E+00_JPRB ,&
&  0.13289E+00_JPRB, 0.13245E+00_JPRB, 0.13205E+00_JPRB, 0.13162E+00_JPRB, 0.13131E+00_JPRB, 0.13089E+00_JPRB, 0.13054E+00_JPRB ,&
&  0.13021E+00_JPRB, 0.12997E+00_JPRB, 0.12990E+00_JPRB, 0.12986E+00_JPRB, 0.12983E+00_JPRB, 0.12987E+00_JPRB, 0.12999E+00_JPRB ,&
&  0.13016E+00_JPRB, 0.13035E+00_JPRB, 0.13061E+00_JPRB, 0.13080E+00_JPRB, 0.13099E+00_JPRB, 0.13115E+00_JPRB, 0.13153E+00_JPRB ,&
&  0.13199E+00_JPRB, 0.13248E+00_JPRB, 0.13294E+00_JPRB, 0.13317E+00_JPRB, 0.13392E+00_JPRB, 0.13471E+00_JPRB, 0.13556E+00_JPRB ,&
&  0.13646E+00_JPRB, 0.13761E+00_JPRB, 0.13887E+00_JPRB, 0.14021E+00_JPRB, 0.14178E+00_JPRB, 0.14282E+00_JPRB, 0.14395E+00_JPRB ,&
&  0.14548E+00_JPRB, 0.14799E+00_JPRB, 0.14891E+00_JPRB, 0.14929E+00_JPRB, 0.14953E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.13923E+00_JPRB, 0.13906E+00_JPRB, 0.13887E+00_JPRB, 0.13865E+00_JPRB, 0.13796E+00_JPRB, 0.13765E+00_JPRB, 0.13737E+00_JPRB ,&
&  0.13721E+00_JPRB, 0.13711E+00_JPRB, 0.13718E+00_JPRB, 0.13731E+00_JPRB, 0.13744E+00_JPRB, 0.13755E+00_JPRB, 0.13764E+00_JPRB ,&
&  0.13760E+00_JPRB, 0.13759E+00_JPRB, 0.13697E+00_JPRB, 0.13649E+00_JPRB, 0.13620E+00_JPRB, 0.13598E+00_JPRB, 0.13568E+00_JPRB ,&
&  0.13545E+00_JPRB, 0.13515E+00_JPRB, 0.13480E+00_JPRB, 0.13454E+00_JPRB, 0.13455E+00_JPRB, 0.13441E+00_JPRB, 0.13441E+00_JPRB ,&
&  0.13438E+00_JPRB, 0.13452E+00_JPRB, 0.13456E+00_JPRB, 0.13462E+00_JPRB, 0.13493E+00_JPRB, 0.13519E+00_JPRB, 0.13523E+00_JPRB ,&
&  0.13524E+00_JPRB, 0.13506E+00_JPRB, 0.13488E+00_JPRB, 0.13475E+00_JPRB, 0.13468E+00_JPRB, 0.13466E+00_JPRB, 0.13462E+00_JPRB ,&
&  0.13450E+00_JPRB, 0.13431E+00_JPRB, 0.13408E+00_JPRB, 0.13381E+00_JPRB, 0.13357E+00_JPRB, 0.13345E+00_JPRB, 0.13342E+00_JPRB ,&
&  0.13340E+00_JPRB, 0.13327E+00_JPRB, 0.13308E+00_JPRB, 0.13279E+00_JPRB, 0.13251E+00_JPRB, 0.13225E+00_JPRB, 0.13200E+00_JPRB ,&
&  0.13177E+00_JPRB, 0.13165E+00_JPRB, 0.13153E+00_JPRB, 0.13143E+00_JPRB, 0.13136E+00_JPRB, 0.13125E+00_JPRB, 0.13116E+00_JPRB ,&
&  0.13107E+00_JPRB, 0.13099E+00_JPRB, 0.13091E+00_JPRB, 0.13084E+00_JPRB, 0.13080E+00_JPRB, 0.13077E+00_JPRB, 0.13077E+00_JPRB ,&
&  0.13079E+00_JPRB, 0.13081E+00_JPRB, 0.13085E+00_JPRB, 0.13098E+00_JPRB, 0.13114E+00_JPRB, 0.13134E+00_JPRB, 0.13156E+00_JPRB ,&
&  0.13181E+00_JPRB, 0.13209E+00_JPRB, 0.13239E+00_JPRB, 0.13261E+00_JPRB, 0.13295E+00_JPRB, 0.13329E+00_JPRB, 0.13364E+00_JPRB ,&
&  0.13400E+00_JPRB, 0.13442E+00_JPRB, 0.13490E+00_JPRB, 0.13543E+00_JPRB, 0.13602E+00_JPRB, 0.13634E+00_JPRB, 0.13674E+00_JPRB ,&
&  0.13721E+00_JPRB, 0.13782E+00_JPRB, 0.13816E+00_JPRB, 0.13841E+00_JPRB, 0.13870E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB, 0.13314E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB ,&
&  0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB, 0.37762E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.37836E+00_JPRB, 0.37834E+00_JPRB, 0.37832E+00_JPRB, 0.37834E+00_JPRB, 0.37836E+00_JPRB, 0.37845E+00_JPRB, 0.37852E+00_JPRB ,&
&  0.37873E+00_JPRB, 0.37891E+00_JPRB, 0.37904E+00_JPRB, 0.37910E+00_JPRB, 0.37924E+00_JPRB, 0.37933E+00_JPRB, 0.37937E+00_JPRB ,&
&  0.37933E+00_JPRB, 0.37936E+00_JPRB, 0.37922E+00_JPRB, 0.37902E+00_JPRB, 0.37876E+00_JPRB, 0.37860E+00_JPRB, 0.37833E+00_JPRB ,&
&  0.37810E+00_JPRB, 0.37771E+00_JPRB, 0.37736E+00_JPRB, 0.37677E+00_JPRB, 0.37634E+00_JPRB, 0.37585E+00_JPRB, 0.37543E+00_JPRB ,&
&  0.37504E+00_JPRB, 0.37480E+00_JPRB, 0.37455E+00_JPRB, 0.37423E+00_JPRB, 0.37397E+00_JPRB, 0.37406E+00_JPRB, 0.37423E+00_JPRB ,&
&  0.37437E+00_JPRB, 0.37456E+00_JPRB, 0.37484E+00_JPRB, 0.37496E+00_JPRB, 0.37490E+00_JPRB, 0.37489E+00_JPRB, 0.37485E+00_JPRB ,&
&  0.37476E+00_JPRB, 0.37457E+00_JPRB, 0.37438E+00_JPRB, 0.37434E+00_JPRB, 0.37432E+00_JPRB, 0.37420E+00_JPRB, 0.37411E+00_JPRB ,&
&  0.37404E+00_JPRB, 0.37394E+00_JPRB, 0.37377E+00_JPRB, 0.37349E+00_JPRB, 0.37327E+00_JPRB, 0.37305E+00_JPRB, 0.37282E+00_JPRB ,&
&  0.37257E+00_JPRB, 0.37228E+00_JPRB, 0.37198E+00_JPRB, 0.37164E+00_JPRB, 0.37126E+00_JPRB, 0.37093E+00_JPRB, 0.37066E+00_JPRB ,&
&  0.37045E+00_JPRB, 0.37035E+00_JPRB, 0.37028E+00_JPRB, 0.37022E+00_JPRB, 0.37023E+00_JPRB, 0.37034E+00_JPRB, 0.37051E+00_JPRB ,&
&  0.37075E+00_JPRB, 0.37108E+00_JPRB, 0.37154E+00_JPRB, 0.37211E+00_JPRB, 0.37265E+00_JPRB, 0.37320E+00_JPRB, 0.37376E+00_JPRB ,&
&  0.37428E+00_JPRB, 0.37468E+00_JPRB, 0.37496E+00_JPRB, 0.37523E+00_JPRB, 0.37574E+00_JPRB, 0.37610E+00_JPRB, 0.37635E+00_JPRB ,&
&  0.37654E+00_JPRB, 0.37682E+00_JPRB, 0.37707E+00_JPRB, 0.37733E+00_JPRB, 0.37743E+00_JPRB, 0.37759E+00_JPRB, 0.37776E+00_JPRB ,&
&  0.37793E+00_JPRB, 0.37806E+00_JPRB, 0.37816E+00_JPRB, 0.37825E+00_JPRB, 0.37830E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.37403E+00_JPRB, 0.37413E+00_JPRB, 0.37443E+00_JPRB, 0.37488E+00_JPRB, 0.37540E+00_JPRB, 0.37589E+00_JPRB, 0.37643E+00_JPRB ,&
&  0.37708E+00_JPRB, 0.37756E+00_JPRB, 0.37762E+00_JPRB, 0.37797E+00_JPRB, 0.37861E+00_JPRB, 0.37935E+00_JPRB, 0.37934E+00_JPRB ,&
&  0.37941E+00_JPRB, 0.37946E+00_JPRB, 0.37970E+00_JPRB, 0.37942E+00_JPRB, 0.37901E+00_JPRB, 0.37838E+00_JPRB, 0.37806E+00_JPRB ,&
&  0.37772E+00_JPRB, 0.37713E+00_JPRB, 0.37632E+00_JPRB, 0.37489E+00_JPRB, 0.37362E+00_JPRB, 0.37227E+00_JPRB, 0.37089E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36758E+00_JPRB, 0.36635E+00_JPRB, 0.36529E+00_JPRB, 0.36668E+00_JPRB, 0.36761E+00_JPRB, 0.36816E+00_JPRB ,&
&  0.36826E+00_JPRB, 0.36675E+00_JPRB, 0.36625E+00_JPRB, 0.36591E+00_JPRB, 0.36565E+00_JPRB, 0.36683E+00_JPRB, 0.36762E+00_JPRB ,&
&  0.36827E+00_JPRB, 0.36881E+00_JPRB, 0.37032E+00_JPRB, 0.37110E+00_JPRB, 0.37170E+00_JPRB, 0.37208E+00_JPRB, 0.37202E+00_JPRB ,&
&  0.37240E+00_JPRB, 0.37280E+00_JPRB, 0.37316E+00_JPRB, 0.37313E+00_JPRB, 0.37303E+00_JPRB, 0.37288E+00_JPRB, 0.37247E+00_JPRB ,&
&  0.37179E+00_JPRB, 0.37132E+00_JPRB, 0.37070E+00_JPRB, 0.36997E+00_JPRB, 0.36911E+00_JPRB, 0.36856E+00_JPRB, 0.36808E+00_JPRB ,&
&  0.36764E+00_JPRB, 0.36737E+00_JPRB, 0.36719E+00_JPRB, 0.36701E+00_JPRB, 0.36688E+00_JPRB, 0.36694E+00_JPRB, 0.36725E+00_JPRB ,&
&  0.36772E+00_JPRB, 0.36825E+00_JPRB, 0.36882E+00_JPRB, 0.36957E+00_JPRB, 0.37027E+00_JPRB, 0.37095E+00_JPRB, 0.37201E+00_JPRB ,&
&  0.37330E+00_JPRB, 0.37425E+00_JPRB, 0.37496E+00_JPRB, 0.37553E+00_JPRB, 0.37638E+00_JPRB, 0.37676E+00_JPRB, 0.37682E+00_JPRB ,&
&  0.37674E+00_JPRB, 0.37705E+00_JPRB, 0.37733E+00_JPRB, 0.37762E+00_JPRB, 0.37755E+00_JPRB, 0.37720E+00_JPRB, 0.37669E+00_JPRB ,&
&  0.37612E+00_JPRB, 0.37565E+00_JPRB, 0.37511E+00_JPRB, 0.37458E+00_JPRB, 0.37419E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.37170E+00_JPRB, 0.37323E+00_JPRB, 0.37519E+00_JPRB, 0.37723E+00_JPRB, 0.37953E+00_JPRB, 0.38110E+00_JPRB, 0.38284E+00_JPRB ,&
&  0.38401E+00_JPRB, 0.38499E+00_JPRB, 0.38513E+00_JPRB, 0.38589E+00_JPRB, 0.38621E+00_JPRB, 0.38674E+00_JPRB, 0.38647E+00_JPRB ,&
&  0.38657E+00_JPRB, 0.38605E+00_JPRB, 0.38675E+00_JPRB, 0.38697E+00_JPRB, 0.38729E+00_JPRB, 0.38738E+00_JPRB, 0.38512E+00_JPRB ,&
&  0.38286E+00_JPRB, 0.38148E+00_JPRB, 0.37829E+00_JPRB, 0.37529E+00_JPRB, 0.37328E+00_JPRB, 0.35762E+00_JPRB, 0.35283E+00_JPRB ,&
&  0.35749E+00_JPRB, 0.36080E+00_JPRB, 0.36407E+00_JPRB, 0.36658E+00_JPRB, 0.36182E+00_JPRB, 0.35453E+00_JPRB, 0.35034E+00_JPRB ,&
&  0.33732E+00_JPRB, 0.33979E+00_JPRB, 0.34152E+00_JPRB, 0.34297E+00_JPRB, 0.34607E+00_JPRB, 0.34720E+00_JPRB, 0.35055E+00_JPRB ,&
&  0.35925E+00_JPRB, 0.36143E+00_JPRB, 0.36431E+00_JPRB, 0.36671E+00_JPRB, 0.36809E+00_JPRB, 0.36974E+00_JPRB, 0.37346E+00_JPRB ,&
&  0.37521E+00_JPRB, 0.37671E+00_JPRB, 0.37780E+00_JPRB, 0.37778E+00_JPRB, 0.37781E+00_JPRB, 0.37721E+00_JPRB, 0.37641E+00_JPRB ,&
&  0.37482E+00_JPRB, 0.37402E+00_JPRB, 0.37351E+00_JPRB, 0.37294E+00_JPRB, 0.37272E+00_JPRB, 0.37151E+00_JPRB, 0.37060E+00_JPRB ,&
&  0.36999E+00_JPRB, 0.36970E+00_JPRB, 0.36981E+00_JPRB, 0.37042E+00_JPRB, 0.37144E+00_JPRB, 0.37258E+00_JPRB, 0.37353E+00_JPRB ,&
&  0.37464E+00_JPRB, 0.37546E+00_JPRB, 0.37654E+00_JPRB, 0.37754E+00_JPRB, 0.37812E+00_JPRB, 0.37853E+00_JPRB, 0.37893E+00_JPRB ,&
&  0.37986E+00_JPRB, 0.38091E+00_JPRB, 0.38137E+00_JPRB, 0.38112E+00_JPRB, 0.38183E+00_JPRB, 0.38221E+00_JPRB, 0.38218E+00_JPRB ,&
&  0.38181E+00_JPRB, 0.38163E+00_JPRB, 0.38062E+00_JPRB, 0.37926E+00_JPRB, 0.37763E+00_JPRB, 0.37624E+00_JPRB, 0.37482E+00_JPRB ,&
&  0.37350E+00_JPRB, 0.37234E+00_JPRB, 0.37154E+00_JPRB, 0.37096E+00_JPRB, 0.37089E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.37671E+00_JPRB, 0.38006E+00_JPRB, 0.38422E+00_JPRB, 0.38671E+00_JPRB, 0.38986E+00_JPRB, 0.39108E+00_JPRB, 0.39296E+00_JPRB ,&
&  0.39355E+00_JPRB, 0.39450E+00_JPRB, 0.39511E+00_JPRB, 0.39547E+00_JPRB, 0.39544E+00_JPRB, 0.39514E+00_JPRB, 0.39490E+00_JPRB ,&
&  0.39508E+00_JPRB, 0.39485E+00_JPRB, 0.39564E+00_JPRB, 0.39600E+00_JPRB, 0.39646E+00_JPRB, 0.39644E+00_JPRB, 0.39391E+00_JPRB ,&
&  0.39058E+00_JPRB, 0.38767E+00_JPRB, 0.36775E+00_JPRB, 0.36499E+00_JPRB, 0.36096E+00_JPRB, 0.35883E+00_JPRB, 0.35957E+00_JPRB ,&
&  0.35790E+00_JPRB, 0.35550E+00_JPRB, 0.34728E+00_JPRB, 0.33850E+00_JPRB, 0.33103E+00_JPRB, 0.32264E+00_JPRB, 0.32113E+00_JPRB ,&
&  0.31963E+00_JPRB, 0.32164E+00_JPRB, 0.32388E+00_JPRB, 0.32569E+00_JPRB, 0.33082E+00_JPRB, 0.33438E+00_JPRB, 0.34053E+00_JPRB ,&
&  0.34818E+00_JPRB, 0.35368E+00_JPRB, 0.35463E+00_JPRB, 0.35364E+00_JPRB, 0.36311E+00_JPRB, 0.36924E+00_JPRB, 0.37476E+00_JPRB ,&
&  0.37810E+00_JPRB, 0.37887E+00_JPRB, 0.38086E+00_JPRB, 0.38120E+00_JPRB, 0.38169E+00_JPRB, 0.38319E+00_JPRB, 0.38343E+00_JPRB ,&
&  0.38446E+00_JPRB, 0.38430E+00_JPRB, 0.38354E+00_JPRB, 0.38228E+00_JPRB, 0.37991E+00_JPRB, 0.37838E+00_JPRB, 0.37721E+00_JPRB ,&
&  0.37706E+00_JPRB, 0.37788E+00_JPRB, 0.37906E+00_JPRB, 0.38106E+00_JPRB, 0.38326E+00_JPRB, 0.38464E+00_JPRB, 0.38522E+00_JPRB ,&
&  0.38529E+00_JPRB, 0.38494E+00_JPRB, 0.38632E+00_JPRB, 0.38528E+00_JPRB, 0.38529E+00_JPRB, 0.38791E+00_JPRB, 0.38662E+00_JPRB ,&
&  0.38670E+00_JPRB, 0.38813E+00_JPRB, 0.38899E+00_JPRB, 0.38976E+00_JPRB, 0.39037E+00_JPRB, 0.39056E+00_JPRB, 0.39100E+00_JPRB ,&
&  0.39078E+00_JPRB, 0.38996E+00_JPRB, 0.38820E+00_JPRB, 0.38599E+00_JPRB, 0.38348E+00_JPRB, 0.38099E+00_JPRB, 0.37786E+00_JPRB ,&
&  0.37519E+00_JPRB, 0.37288E+00_JPRB, 0.37188E+00_JPRB, 0.37188E+00_JPRB, 0.37377E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.38770E+00_JPRB, 0.39122E+00_JPRB, 0.39590E+00_JPRB, 0.39770E+00_JPRB, 0.40039E+00_JPRB, 0.40146E+00_JPRB, 0.40255E+00_JPRB ,&
&  0.40305E+00_JPRB, 0.40367E+00_JPRB, 0.40359E+00_JPRB, 0.40367E+00_JPRB, 0.40326E+00_JPRB, 0.40325E+00_JPRB, 0.40284E+00_JPRB ,&
&  0.40274E+00_JPRB, 0.40240E+00_JPRB, 0.40256E+00_JPRB, 0.40254E+00_JPRB, 0.40324E+00_JPRB, 0.40353E+00_JPRB, 0.40239E+00_JPRB ,&
&  0.39963E+00_JPRB, 0.39323E+00_JPRB, 0.37814E+00_JPRB, 0.37372E+00_JPRB, 0.36975E+00_JPRB, 0.36936E+00_JPRB, 0.36822E+00_JPRB ,&
&  0.36691E+00_JPRB, 0.33968E+00_JPRB, 0.34037E+00_JPRB, 0.33346E+00_JPRB, 0.32963E+00_JPRB, 0.32602E+00_JPRB, 0.31937E+00_JPRB ,&
&  0.31691E+00_JPRB, 0.31450E+00_JPRB, 0.31419E+00_JPRB, 0.31776E+00_JPRB, 0.32153E+00_JPRB, 0.32743E+00_JPRB, 0.33466E+00_JPRB ,&
&  0.33588E+00_JPRB, 0.34746E+00_JPRB, 0.35330E+00_JPRB, 0.35688E+00_JPRB, 0.36638E+00_JPRB, 0.37389E+00_JPRB, 0.37613E+00_JPRB ,&
&  0.37864E+00_JPRB, 0.37870E+00_JPRB, 0.38140E+00_JPRB, 0.38554E+00_JPRB, 0.39023E+00_JPRB, 0.39068E+00_JPRB, 0.39147E+00_JPRB ,&
&  0.39179E+00_JPRB, 0.39218E+00_JPRB, 0.39240E+00_JPRB, 0.39216E+00_JPRB, 0.39166E+00_JPRB, 0.39188E+00_JPRB, 0.39247E+00_JPRB ,&
&  0.39455E+00_JPRB, 0.39673E+00_JPRB, 0.39926E+00_JPRB, 0.40113E+00_JPRB, 0.40240E+00_JPRB, 0.40323E+00_JPRB, 0.40416E+00_JPRB ,&
&  0.40465E+00_JPRB, 0.40456E+00_JPRB, 0.40289E+00_JPRB, 0.40147E+00_JPRB, 0.39946E+00_JPRB, 0.39682E+00_JPRB, 0.39601E+00_JPRB ,&
&  0.39713E+00_JPRB, 0.39755E+00_JPRB, 0.39767E+00_JPRB, 0.39807E+00_JPRB, 0.39894E+00_JPRB, 0.39930E+00_JPRB, 0.39967E+00_JPRB ,&
&  0.39967E+00_JPRB, 0.39903E+00_JPRB, 0.39762E+00_JPRB, 0.39427E+00_JPRB, 0.39071E+00_JPRB, 0.38862E+00_JPRB, 0.38503E+00_JPRB ,&
&  0.38181E+00_JPRB, 0.37851E+00_JPRB, 0.37813E+00_JPRB, 0.37877E+00_JPRB, 0.38274E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.40118E+00_JPRB, 0.40648E+00_JPRB, 0.41171E+00_JPRB, 0.41430E+00_JPRB, 0.41688E+00_JPRB, 0.41724E+00_JPRB, 0.41799E+00_JPRB ,&
&  0.41827E+00_JPRB, 0.41893E+00_JPRB, 0.41893E+00_JPRB, 0.41888E+00_JPRB, 0.41830E+00_JPRB, 0.41761E+00_JPRB, 0.41633E+00_JPRB ,&
&  0.41449E+00_JPRB, 0.41271E+00_JPRB, 0.41042E+00_JPRB, 0.40882E+00_JPRB, 0.40789E+00_JPRB, 0.40889E+00_JPRB, 0.40849E+00_JPRB ,&
&  0.40580E+00_JPRB, 0.39979E+00_JPRB, 0.38821E+00_JPRB, 0.38155E+00_JPRB, 0.37965E+00_JPRB, 0.37346E+00_JPRB, 0.37319E+00_JPRB ,&
&  0.37142E+00_JPRB, 0.37128E+00_JPRB, 0.36806E+00_JPRB, 0.36630E+00_JPRB, 0.35879E+00_JPRB, 0.32023E+00_JPRB, 0.31585E+00_JPRB ,&
&  0.31264E+00_JPRB, 0.30809E+00_JPRB, 0.30745E+00_JPRB, 0.30858E+00_JPRB, 0.31235E+00_JPRB, 0.32409E+00_JPRB, 0.33494E+00_JPRB ,&
&  0.33565E+00_JPRB, 0.34946E+00_JPRB, 0.35420E+00_JPRB, 0.35899E+00_JPRB, 0.36769E+00_JPRB, 0.37569E+00_JPRB, 0.37860E+00_JPRB ,&
&  0.38239E+00_JPRB, 0.38656E+00_JPRB, 0.39309E+00_JPRB, 0.39975E+00_JPRB, 0.40436E+00_JPRB, 0.40650E+00_JPRB, 0.40913E+00_JPRB ,&
&  0.41124E+00_JPRB, 0.41413E+00_JPRB, 0.41761E+00_JPRB, 0.42030E+00_JPRB, 0.42064E+00_JPRB, 0.41893E+00_JPRB, 0.41767E+00_JPRB ,&
&  0.41871E+00_JPRB, 0.41868E+00_JPRB, 0.42253E+00_JPRB, 0.42078E+00_JPRB, 0.42057E+00_JPRB, 0.41982E+00_JPRB, 0.42043E+00_JPRB ,&
&  0.41935E+00_JPRB, 0.42217E+00_JPRB, 0.42390E+00_JPRB, 0.42468E+00_JPRB, 0.41950E+00_JPRB, 0.41451E+00_JPRB, 0.41131E+00_JPRB ,&
&  0.41005E+00_JPRB, 0.40987E+00_JPRB, 0.40662E+00_JPRB, 0.40591E+00_JPRB, 0.40596E+00_JPRB, 0.40643E+00_JPRB, 0.40729E+00_JPRB ,&
&  0.40764E+00_JPRB, 0.40646E+00_JPRB, 0.40399E+00_JPRB, 0.39974E+00_JPRB, 0.39574E+00_JPRB, 0.39227E+00_JPRB, 0.38630E+00_JPRB ,&
&  0.38255E+00_JPRB, 0.38021E+00_JPRB, 0.38188E+00_JPRB, 0.38556E+00_JPRB, 0.39280E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.40994E+00_JPRB, 0.41716E+00_JPRB, 0.42455E+00_JPRB, 0.43071E+00_JPRB, 0.43360E+00_JPRB, 0.43352E+00_JPRB, 0.43174E+00_JPRB ,&
&  0.42985E+00_JPRB, 0.42898E+00_JPRB, 0.42813E+00_JPRB, 0.42761E+00_JPRB, 0.42711E+00_JPRB, 0.42622E+00_JPRB, 0.42462E+00_JPRB ,&
&  0.42367E+00_JPRB, 0.41416E+00_JPRB, 0.40750E+00_JPRB, 0.40588E+00_JPRB, 0.40537E+00_JPRB, 0.40633E+00_JPRB, 0.40826E+00_JPRB ,&
&  0.40603E+00_JPRB, 0.39970E+00_JPRB, 0.39431E+00_JPRB, 0.38682E+00_JPRB, 0.37933E+00_JPRB, 0.37901E+00_JPRB, 0.37801E+00_JPRB ,&
&  0.37062E+00_JPRB, 0.36450E+00_JPRB, 0.36798E+00_JPRB, 0.36917E+00_JPRB, 0.35706E+00_JPRB, 0.34107E+00_JPRB, 0.32146E+00_JPRB ,&
&  0.31882E+00_JPRB, 0.31121E+00_JPRB, 0.30960E+00_JPRB, 0.31062E+00_JPRB, 0.31604E+00_JPRB, 0.32264E+00_JPRB, 0.33429E+00_JPRB ,&
&  0.33917E+00_JPRB, 0.34395E+00_JPRB, 0.35324E+00_JPRB, 0.36362E+00_JPRB, 0.37211E+00_JPRB, 0.38115E+00_JPRB, 0.38821E+00_JPRB ,&
&  0.39775E+00_JPRB, 0.40784E+00_JPRB, 0.41670E+00_JPRB, 0.42381E+00_JPRB, 0.43053E+00_JPRB, 0.43222E+00_JPRB, 0.43431E+00_JPRB ,&
&  0.45037E+00_JPRB, 0.46495E+00_JPRB, 0.47394E+00_JPRB, 0.47967E+00_JPRB, 0.48037E+00_JPRB, 0.47761E+00_JPRB, 0.46731E+00_JPRB ,&
&  0.46147E+00_JPRB, 0.46137E+00_JPRB, 0.45885E+00_JPRB, 0.45734E+00_JPRB, 0.45446E+00_JPRB, 0.45046E+00_JPRB, 0.44341E+00_JPRB ,&
&  0.43443E+00_JPRB, 0.43436E+00_JPRB, 0.46965E+00_JPRB, 0.47391E+00_JPRB, 0.46424E+00_JPRB, 0.44996E+00_JPRB, 0.43618E+00_JPRB ,&
&  0.42667E+00_JPRB, 0.42110E+00_JPRB, 0.41786E+00_JPRB, 0.41527E+00_JPRB, 0.41354E+00_JPRB, 0.41048E+00_JPRB, 0.41109E+00_JPRB ,&
&  0.40995E+00_JPRB, 0.40717E+00_JPRB, 0.40415E+00_JPRB, 0.40017E+00_JPRB, 0.39596E+00_JPRB, 0.39255E+00_JPRB, 0.38788E+00_JPRB ,&
&  0.38464E+00_JPRB, 0.38328E+00_JPRB, 0.38667E+00_JPRB, 0.39311E+00_JPRB, 0.40211E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.41532E+00_JPRB, 0.42342E+00_JPRB, 0.43376E+00_JPRB, 0.44126E+00_JPRB, 0.44319E+00_JPRB, 0.43370E+00_JPRB, 0.42521E+00_JPRB ,&
&  0.41983E+00_JPRB, 0.41796E+00_JPRB, 0.41777E+00_JPRB, 0.41711E+00_JPRB, 0.41709E+00_JPRB, 0.41706E+00_JPRB, 0.41687E+00_JPRB ,&
&  0.41686E+00_JPRB, 0.41557E+00_JPRB, 0.41217E+00_JPRB, 0.40549E+00_JPRB, 0.40102E+00_JPRB, 0.40011E+00_JPRB, 0.40128E+00_JPRB ,&
&  0.40177E+00_JPRB, 0.40000E+00_JPRB, 0.39804E+00_JPRB, 0.39107E+00_JPRB, 0.38302E+00_JPRB, 0.37421E+00_JPRB, 0.36809E+00_JPRB ,&
&  0.36781E+00_JPRB, 0.36916E+00_JPRB, 0.36647E+00_JPRB, 0.36735E+00_JPRB, 0.36830E+00_JPRB, 0.35310E+00_JPRB, 0.33381E+00_JPRB ,&
&  0.32310E+00_JPRB, 0.31599E+00_JPRB, 0.31274E+00_JPRB, 0.31356E+00_JPRB, 0.32526E+00_JPRB, 0.34282E+00_JPRB, 0.34967E+00_JPRB ,&
&  0.33844E+00_JPRB, 0.34901E+00_JPRB, 0.36291E+00_JPRB, 0.37255E+00_JPRB, 0.38261E+00_JPRB, 0.39585E+00_JPRB, 0.40932E+00_JPRB ,&
&  0.42105E+00_JPRB, 0.43101E+00_JPRB, 0.43689E+00_JPRB, 0.44802E+00_JPRB, 0.45345E+00_JPRB, 0.44709E+00_JPRB, 0.45310E+00_JPRB ,&
&  0.49934E+00_JPRB, 0.54073E+00_JPRB, 0.55521E+00_JPRB, 0.55385E+00_JPRB, 0.54752E+00_JPRB, 0.53640E+00_JPRB, 0.52796E+00_JPRB ,&
&  0.51999E+00_JPRB, 0.51103E+00_JPRB, 0.50226E+00_JPRB, 0.49359E+00_JPRB, 0.48561E+00_JPRB, 0.47636E+00_JPRB, 0.46963E+00_JPRB ,&
&  0.46352E+00_JPRB, 0.46987E+00_JPRB, 0.58256E+00_JPRB, 0.55432E+00_JPRB, 0.51810E+00_JPRB, 0.49504E+00_JPRB, 0.47451E+00_JPRB ,&
&  0.46003E+00_JPRB, 0.45064E+00_JPRB, 0.44485E+00_JPRB, 0.43771E+00_JPRB, 0.42339E+00_JPRB, 0.41267E+00_JPRB, 0.40814E+00_JPRB ,&
&  0.40584E+00_JPRB, 0.40110E+00_JPRB, 0.39695E+00_JPRB, 0.39302E+00_JPRB, 0.39017E+00_JPRB, 0.38922E+00_JPRB, 0.38720E+00_JPRB ,&
&  0.38629E+00_JPRB, 0.38976E+00_JPRB, 0.39530E+00_JPRB, 0.40355E+00_JPRB, 0.40994E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.41902E+00_JPRB, 0.43539E+00_JPRB, 0.44859E+00_JPRB, 0.45381E+00_JPRB, 0.45229E+00_JPRB, 0.42842E+00_JPRB, 0.40666E+00_JPRB ,&
&  0.39458E+00_JPRB, 0.39098E+00_JPRB, 0.39218E+00_JPRB, 0.39377E+00_JPRB, 0.39635E+00_JPRB, 0.39977E+00_JPRB, 0.40263E+00_JPRB ,&
&  0.40323E+00_JPRB, 0.40287E+00_JPRB, 0.40255E+00_JPRB, 0.40140E+00_JPRB, 0.39905E+00_JPRB, 0.39691E+00_JPRB, 0.39638E+00_JPRB ,&
&  0.39589E+00_JPRB, 0.39379E+00_JPRB, 0.39135E+00_JPRB, 0.38659E+00_JPRB, 0.38338E+00_JPRB, 0.38137E+00_JPRB, 0.37806E+00_JPRB ,&
&  0.37201E+00_JPRB, 0.37098E+00_JPRB, 0.37599E+00_JPRB, 0.37882E+00_JPRB, 0.37882E+00_JPRB, 0.36841E+00_JPRB, 0.34637E+00_JPRB ,&
&  0.33178E+00_JPRB, 0.32534E+00_JPRB, 0.32977E+00_JPRB, 0.34863E+00_JPRB, 0.36263E+00_JPRB, 0.35260E+00_JPRB, 0.35994E+00_JPRB ,&
&  0.36387E+00_JPRB, 0.36707E+00_JPRB, 0.37294E+00_JPRB, 0.38571E+00_JPRB, 0.40168E+00_JPRB, 0.41734E+00_JPRB, 0.42980E+00_JPRB ,&
&  0.44144E+00_JPRB, 0.45153E+00_JPRB, 0.46291E+00_JPRB, 0.45151E+00_JPRB, 0.44962E+00_JPRB, 0.47998E+00_JPRB, 0.51667E+00_JPRB ,&
&  0.54696E+00_JPRB, 0.56269E+00_JPRB, 0.56754E+00_JPRB, 0.58546E+00_JPRB, 0.59279E+00_JPRB, 0.59030E+00_JPRB, 0.58049E+00_JPRB ,&
&  0.56768E+00_JPRB, 0.54597E+00_JPRB, 0.52677E+00_JPRB, 0.51479E+00_JPRB, 0.51136E+00_JPRB, 0.51312E+00_JPRB, 0.51160E+00_JPRB ,&
&  0.50823E+00_JPRB, 0.49445E+00_JPRB, 0.48111E+00_JPRB, 0.45635E+00_JPRB, 0.45370E+00_JPRB, 0.46190E+00_JPRB, 0.46925E+00_JPRB ,&
&  0.46635E+00_JPRB, 0.46090E+00_JPRB, 0.45666E+00_JPRB, 0.44865E+00_JPRB, 0.42430E+00_JPRB, 0.40770E+00_JPRB, 0.39789E+00_JPRB ,&
&  0.39201E+00_JPRB, 0.38339E+00_JPRB, 0.38216E+00_JPRB, 0.37971E+00_JPRB, 0.38164E+00_JPRB, 0.38480E+00_JPRB, 0.38371E+00_JPRB ,&
&  0.38421E+00_JPRB, 0.38665E+00_JPRB, 0.39280E+00_JPRB, 0.39909E+00_JPRB, 0.40779E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.46063E+00_JPRB, 0.46731E+00_JPRB, 0.46488E+00_JPRB, 0.46586E+00_JPRB, 0.45517E+00_JPRB, 0.44255E+00_JPRB, 0.42945E+00_JPRB ,&
&  0.42649E+00_JPRB, 0.41457E+00_JPRB, 0.40122E+00_JPRB, 0.38737E+00_JPRB, 0.37926E+00_JPRB, 0.37552E+00_JPRB, 0.37807E+00_JPRB ,&
&  0.38442E+00_JPRB, 0.39235E+00_JPRB, 0.39771E+00_JPRB, 0.39692E+00_JPRB, 0.39390E+00_JPRB, 0.39235E+00_JPRB, 0.39284E+00_JPRB ,&
&  0.39226E+00_JPRB, 0.39069E+00_JPRB, 0.38818E+00_JPRB, 0.38529E+00_JPRB, 0.38341E+00_JPRB, 0.38280E+00_JPRB, 0.38342E+00_JPRB ,&
&  0.38421E+00_JPRB, 0.38361E+00_JPRB, 0.38593E+00_JPRB, 0.38780E+00_JPRB, 0.38844E+00_JPRB, 0.37959E+00_JPRB, 0.36002E+00_JPRB ,&
&  0.34366E+00_JPRB, 0.34171E+00_JPRB, 0.36420E+00_JPRB, 0.36157E+00_JPRB, 0.37178E+00_JPRB, 0.38071E+00_JPRB, 0.38147E+00_JPRB ,&
&  0.38108E+00_JPRB, 0.38348E+00_JPRB, 0.39572E+00_JPRB, 0.41431E+00_JPRB, 0.42857E+00_JPRB, 0.44368E+00_JPRB, 0.45702E+00_JPRB ,&
&  0.47387E+00_JPRB, 0.49160E+00_JPRB, 0.50345E+00_JPRB, 0.49172E+00_JPRB, 0.53843E+00_JPRB, 0.69098E+00_JPRB, 0.70151E+00_JPRB ,&
&  0.70208E+00_JPRB, 0.70156E+00_JPRB, 0.71538E+00_JPRB, 0.72005E+00_JPRB, 0.69233E+00_JPRB, 0.66849E+00_JPRB, 0.64821E+00_JPRB ,&
&  0.61746E+00_JPRB, 0.59498E+00_JPRB, 0.58068E+00_JPRB, 0.58469E+00_JPRB, 0.58506E+00_JPRB, 0.58708E+00_JPRB, 0.59189E+00_JPRB ,&
&  0.59541E+00_JPRB, 0.58444E+00_JPRB, 0.56958E+00_JPRB, 0.54453E+00_JPRB, 0.53050E+00_JPRB, 0.52072E+00_JPRB, 0.51477E+00_JPRB ,&
&  0.50548E+00_JPRB, 0.49919E+00_JPRB, 0.49203E+00_JPRB, 0.47911E+00_JPRB, 0.44870E+00_JPRB, 0.41968E+00_JPRB, 0.40207E+00_JPRB ,&
&  0.38861E+00_JPRB, 0.37596E+00_JPRB, 0.37388E+00_JPRB, 0.37578E+00_JPRB, 0.38119E+00_JPRB, 0.38749E+00_JPRB, 0.38747E+00_JPRB ,&
&  0.39113E+00_JPRB, 0.39850E+00_JPRB, 0.41229E+00_JPRB, 0.42925E+00_JPRB, 0.44439E+00_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.49286E+00_JPRB, 0.49389E+00_JPRB, 0.48795E+00_JPRB, 0.48136E+00_JPRB, 0.47498E+00_JPRB, 0.46049E+00_JPRB, 0.43650E+00_JPRB ,&
&  0.43477E+00_JPRB, 0.40685E+00_JPRB, 0.38887E+00_JPRB, 0.38346E+00_JPRB, 0.38133E+00_JPRB, 0.37609E+00_JPRB, 0.36822E+00_JPRB ,&
&  0.36930E+00_JPRB, 0.37808E+00_JPRB, 0.38901E+00_JPRB, 0.39403E+00_JPRB, 0.39641E+00_JPRB, 0.39532E+00_JPRB, 0.39519E+00_JPRB ,&
&  0.39527E+00_JPRB, 0.39534E+00_JPRB, 0.39540E+00_JPRB, 0.39225E+00_JPRB, 0.38953E+00_JPRB, 0.38936E+00_JPRB, 0.38880E+00_JPRB ,&
&  0.39352E+00_JPRB, 0.39603E+00_JPRB, 0.39673E+00_JPRB, 0.39585E+00_JPRB, 0.39259E+00_JPRB, 0.39141E+00_JPRB, 0.37397E+00_JPRB ,&
&  0.35705E+00_JPRB, 0.36596E+00_JPRB, 0.36798E+00_JPRB, 0.37576E+00_JPRB, 0.38552E+00_JPRB, 0.39338E+00_JPRB, 0.40149E+00_JPRB ,&
&  0.40830E+00_JPRB, 0.41705E+00_JPRB, 0.42804E+00_JPRB, 0.44162E+00_JPRB, 0.45920E+00_JPRB, 0.47733E+00_JPRB, 0.49313E+00_JPRB ,&
&  0.52565E+00_JPRB, 0.50675E+00_JPRB, 0.50899E+00_JPRB, 0.56012E+00_JPRB, 0.74434E+00_JPRB, 0.79195E+00_JPRB, 0.82552E+00_JPRB ,&
&  0.85620E+00_JPRB, 0.86265E+00_JPRB, 0.85377E+00_JPRB, 0.84816E+00_JPRB, 0.82073E+00_JPRB, 0.79222E+00_JPRB, 0.77823E+00_JPRB ,&
&  0.75156E+00_JPRB, 0.73988E+00_JPRB, 0.73030E+00_JPRB, 0.73566E+00_JPRB, 0.73376E+00_JPRB, 0.74268E+00_JPRB, 0.75024E+00_JPRB ,&
&  0.74573E+00_JPRB, 0.72750E+00_JPRB, 0.69037E+00_JPRB, 0.65073E+00_JPRB, 0.61463E+00_JPRB, 0.59361E+00_JPRB, 0.57623E+00_JPRB ,&
&  0.55761E+00_JPRB, 0.54383E+00_JPRB, 0.52729E+00_JPRB, 0.50504E+00_JPRB, 0.47679E+00_JPRB, 0.45335E+00_JPRB, 0.43337E+00_JPRB ,&
&  0.40616E+00_JPRB, 0.39220E+00_JPRB, 0.39123E+00_JPRB, 0.39257E+00_JPRB, 0.39806E+00_JPRB, 0.41000E+00_JPRB, 0.41114E+00_JPRB ,&
&  0.42432E+00_JPRB, 0.43756E+00_JPRB, 0.44500E+00_JPRB, 0.45578E+00_JPRB, 0.47167E+00_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.52536E+00_JPRB, 0.52369E+00_JPRB, 0.51608E+00_JPRB, 0.50820E+00_JPRB, 0.50123E+00_JPRB, 0.47936E+00_JPRB, 0.46447E+00_JPRB ,&
&  0.49668E+00_JPRB, 0.45854E+00_JPRB, 0.45341E+00_JPRB, 0.41095E+00_JPRB, 0.38987E+00_JPRB, 0.38068E+00_JPRB, 0.37307E+00_JPRB ,&
&  0.37356E+00_JPRB, 0.38124E+00_JPRB, 0.38881E+00_JPRB, 0.39575E+00_JPRB, 0.40070E+00_JPRB, 0.40051E+00_JPRB, 0.39881E+00_JPRB ,&
&  0.39847E+00_JPRB, 0.40097E+00_JPRB, 0.40465E+00_JPRB, 0.40502E+00_JPRB, 0.40534E+00_JPRB, 0.40528E+00_JPRB, 0.40324E+00_JPRB ,&
&  0.39926E+00_JPRB, 0.40001E+00_JPRB, 0.40739E+00_JPRB, 0.40683E+00_JPRB, 0.40597E+00_JPRB, 0.40465E+00_JPRB, 0.40352E+00_JPRB ,&
&  0.39346E+00_JPRB, 0.38423E+00_JPRB, 0.38998E+00_JPRB, 0.39899E+00_JPRB, 0.40999E+00_JPRB, 0.41986E+00_JPRB, 0.42942E+00_JPRB ,&
&  0.44117E+00_JPRB, 0.45540E+00_JPRB, 0.47189E+00_JPRB, 0.48897E+00_JPRB, 0.50884E+00_JPRB, 0.53446E+00_JPRB, 0.55301E+00_JPRB ,&
&  0.60949E+00_JPRB, 0.56873E+00_JPRB, 0.67871E+00_JPRB, 0.76765E+00_JPRB, 0.92075E+00_JPRB, 0.10275E+01_JPRB, 0.10308E+01_JPRB ,&
&  0.10887E+01_JPRB, 0.10859E+01_JPRB, 0.10728E+01_JPRB, 0.10352E+01_JPRB, 0.99749E+00_JPRB, 0.97031E+00_JPRB, 0.95846E+00_JPRB ,&
&  0.95323E+00_JPRB, 0.92951E+00_JPRB, 0.98237E+00_JPRB, 0.97426E+00_JPRB, 0.96111E+00_JPRB, 0.96181E+00_JPRB, 0.94785E+00_JPRB ,&
&  0.91955E+00_JPRB, 0.87466E+00_JPRB, 0.81500E+00_JPRB, 0.72462E+00_JPRB, 0.68390E+00_JPRB, 0.65023E+00_JPRB, 0.60285E+00_JPRB ,&
&  0.57629E+00_JPRB, 0.54742E+00_JPRB, 0.52177E+00_JPRB, 0.51585E+00_JPRB, 0.48109E+00_JPRB, 0.45516E+00_JPRB, 0.43813E+00_JPRB ,&
&  0.41654E+00_JPRB, 0.40976E+00_JPRB, 0.41614E+00_JPRB, 0.42575E+00_JPRB, 0.43260E+00_JPRB, 0.43830E+00_JPRB, 0.45659E+00_JPRB ,&
&  0.45141E+00_JPRB, 0.47151E+00_JPRB, 0.50407E+00_JPRB, 0.51429E+00_JPRB, 0.52080E+00_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.57052E+00_JPRB, 0.56565E+00_JPRB, 0.55626E+00_JPRB, 0.54663E+00_JPRB, 0.52965E+00_JPRB, 0.50824E+00_JPRB, 0.48218E+00_JPRB ,&
&  0.49527E+00_JPRB, 0.48370E+00_JPRB, 0.47074E+00_JPRB, 0.45364E+00_JPRB, 0.43665E+00_JPRB, 0.41340E+00_JPRB, 0.38255E+00_JPRB ,&
&  0.37725E+00_JPRB, 0.38896E+00_JPRB, 0.40309E+00_JPRB, 0.41216E+00_JPRB, 0.41285E+00_JPRB, 0.41203E+00_JPRB, 0.41299E+00_JPRB ,&
&  0.41482E+00_JPRB, 0.41685E+00_JPRB, 0.42038E+00_JPRB, 0.42304E+00_JPRB, 0.42243E+00_JPRB, 0.42234E+00_JPRB, 0.42513E+00_JPRB ,&
&  0.41355E+00_JPRB, 0.40653E+00_JPRB, 0.40598E+00_JPRB, 0.40227E+00_JPRB, 0.42626E+00_JPRB, 0.43847E+00_JPRB, 0.44938E+00_JPRB ,&
&  0.45191E+00_JPRB, 0.44915E+00_JPRB, 0.44980E+00_JPRB, 0.46145E+00_JPRB, 0.47331E+00_JPRB, 0.47921E+00_JPRB, 0.48856E+00_JPRB ,&
&  0.50304E+00_JPRB, 0.52086E+00_JPRB, 0.54166E+00_JPRB, 0.56969E+00_JPRB, 0.60585E+00_JPRB, 0.61363E+00_JPRB, 0.65955E+00_JPRB ,&
&  0.72887E+00_JPRB, 0.85201E+00_JPRB, 0.99171E+00_JPRB, 0.10052E+01_JPRB, 0.11374E+01_JPRB, 0.12209E+01_JPRB, 0.12113E+01_JPRB ,&
&  0.12414E+01_JPRB, 0.12280E+01_JPRB, 0.12632E+01_JPRB, 0.12541E+01_JPRB, 0.12536E+01_JPRB, 0.12504E+01_JPRB, 0.12366E+01_JPRB ,&
&  0.12367E+01_JPRB, 0.12676E+01_JPRB, 0.12941E+01_JPRB, 0.12918E+01_JPRB, 0.12519E+01_JPRB, 0.11958E+01_JPRB, 0.11219E+01_JPRB ,&
&  0.10525E+01_JPRB, 0.97702E+00_JPRB, 0.90195E+00_JPRB, 0.84109E+00_JPRB, 0.77566E+00_JPRB, 0.71002E+00_JPRB, 0.64788E+00_JPRB ,&
&  0.58880E+00_JPRB, 0.52174E+00_JPRB, 0.49241E+00_JPRB, 0.49030E+00_JPRB, 0.48038E+00_JPRB, 0.44908E+00_JPRB, 0.43222E+00_JPRB ,&
&  0.42708E+00_JPRB, 0.43777E+00_JPRB, 0.45264E+00_JPRB, 0.46172E+00_JPRB, 0.47778E+00_JPRB, 0.49841E+00_JPRB, 0.47961E+00_JPRB ,&
&  0.46920E+00_JPRB, 0.51871E+00_JPRB, 0.54707E+00_JPRB, 0.56182E+00_JPRB, 0.56923E+00_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.61096E+00_JPRB, 0.59978E+00_JPRB, 0.58874E+00_JPRB, 0.57809E+00_JPRB, 0.56059E+00_JPRB, 0.53630E+00_JPRB, 0.51841E+00_JPRB ,&
&  0.51327E+00_JPRB, 0.49875E+00_JPRB, 0.48304E+00_JPRB, 0.46720E+00_JPRB, 0.45569E+00_JPRB, 0.44574E+00_JPRB, 0.43167E+00_JPRB ,&
&  0.41663E+00_JPRB, 0.42693E+00_JPRB, 0.42394E+00_JPRB, 0.42850E+00_JPRB, 0.43570E+00_JPRB, 0.43135E+00_JPRB, 0.43097E+00_JPRB ,&
&  0.43997E+00_JPRB, 0.44236E+00_JPRB, 0.44074E+00_JPRB, 0.43695E+00_JPRB, 0.43472E+00_JPRB, 0.43873E+00_JPRB, 0.44414E+00_JPRB ,&
&  0.42906E+00_JPRB, 0.41581E+00_JPRB, 0.40603E+00_JPRB, 0.40774E+00_JPRB, 0.43357E+00_JPRB, 0.47125E+00_JPRB, 0.50107E+00_JPRB ,&
&  0.52538E+00_JPRB, 0.54356E+00_JPRB, 0.55088E+00_JPRB, 0.55713E+00_JPRB, 0.55836E+00_JPRB, 0.55663E+00_JPRB, 0.55713E+00_JPRB ,&
&  0.56249E+00_JPRB, 0.57598E+00_JPRB, 0.59720E+00_JPRB, 0.63671E+00_JPRB, 0.66694E+00_JPRB, 0.70474E+00_JPRB, 0.79294E+00_JPRB ,&
&  0.92275E+00_JPRB, 0.10933E+01_JPRB, 0.11860E+01_JPRB, 0.13095E+01_JPRB, 0.14068E+01_JPRB, 0.14250E+01_JPRB, 0.14075E+01_JPRB ,&
&  0.14260E+01_JPRB, 0.13860E+01_JPRB, 0.14600E+01_JPRB, 0.14347E+01_JPRB, 0.14320E+01_JPRB, 0.14801E+01_JPRB, 0.14669E+01_JPRB ,&
&  0.14398E+01_JPRB, 0.14159E+01_JPRB, 0.14658E+01_JPRB, 0.14577E+01_JPRB, 0.14213E+01_JPRB, 0.13656E+01_JPRB, 0.12750E+01_JPRB ,&
&  0.11862E+01_JPRB, 0.10882E+01_JPRB, 0.91860E+00_JPRB, 0.79504E+00_JPRB, 0.68262E+00_JPRB, 0.67174E+00_JPRB, 0.61988E+00_JPRB ,&
&  0.56450E+00_JPRB, 0.49642E+00_JPRB, 0.49107E+00_JPRB, 0.49075E+00_JPRB, 0.49793E+00_JPRB, 0.48303E+00_JPRB, 0.45633E+00_JPRB ,&
&  0.45331E+00_JPRB, 0.47068E+00_JPRB, 0.47694E+00_JPRB, 0.48887E+00_JPRB, 0.51247E+00_JPRB, 0.55538E+00_JPRB, 0.50294E+00_JPRB ,&
&  0.52070E+00_JPRB, 0.57916E+00_JPRB, 0.60428E+00_JPRB, 0.61468E+00_JPRB, 0.61638E+00_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.62862E+00_JPRB, 0.61301E+00_JPRB, 0.59857E+00_JPRB, 0.58760E+00_JPRB, 0.57168E+00_JPRB, 0.55978E+00_JPRB, 0.54475E+00_JPRB ,&
&  0.53130E+00_JPRB, 0.51348E+00_JPRB, 0.49949E+00_JPRB, 0.48930E+00_JPRB, 0.48267E+00_JPRB, 0.47247E+00_JPRB, 0.47976E+00_JPRB ,&
&  0.45218E+00_JPRB, 0.45803E+00_JPRB, 0.43161E+00_JPRB, 0.44125E+00_JPRB, 0.48092E+00_JPRB, 0.45803E+00_JPRB, 0.45643E+00_JPRB ,&
&  0.45761E+00_JPRB, 0.46428E+00_JPRB, 0.45817E+00_JPRB, 0.45059E+00_JPRB, 0.44721E+00_JPRB, 0.45172E+00_JPRB, 0.46462E+00_JPRB ,&
&  0.45648E+00_JPRB, 0.42757E+00_JPRB, 0.41301E+00_JPRB, 0.41793E+00_JPRB, 0.45207E+00_JPRB, 0.49647E+00_JPRB, 0.54721E+00_JPRB ,&
&  0.57636E+00_JPRB, 0.60159E+00_JPRB, 0.61985E+00_JPRB, 0.63119E+00_JPRB, 0.63277E+00_JPRB, 0.63320E+00_JPRB, 0.63627E+00_JPRB ,&
&  0.63996E+00_JPRB, 0.65524E+00_JPRB, 0.68606E+00_JPRB, 0.72651E+00_JPRB, 0.73727E+00_JPRB, 0.80691E+00_JPRB, 0.98502E+00_JPRB ,&
&  0.12683E+01_JPRB, 0.13580E+01_JPRB, 0.14674E+01_JPRB, 0.17077E+01_JPRB, 0.18119E+01_JPRB, 0.18193E+01_JPRB, 0.17908E+01_JPRB ,&
&  0.17636E+01_JPRB, 0.17170E+01_JPRB, 0.16906E+01_JPRB, 0.17242E+01_JPRB, 0.16818E+01_JPRB, 0.16499E+01_JPRB, 0.15853E+01_JPRB ,&
&  0.15164E+01_JPRB, 0.15191E+01_JPRB, 0.15340E+01_JPRB, 0.14846E+01_JPRB, 0.14115E+01_JPRB, 0.13753E+01_JPRB, 0.13243E+01_JPRB ,&
&  0.12388E+01_JPRB, 0.10740E+01_JPRB, 0.79825E+00_JPRB, 0.70632E+00_JPRB, 0.56400E+00_JPRB, 0.56436E+00_JPRB, 0.63813E+00_JPRB ,&
&  0.61521E+00_JPRB, 0.57205E+00_JPRB, 0.54638E+00_JPRB, 0.53647E+00_JPRB, 0.53616E+00_JPRB, 0.54770E+00_JPRB, 0.52076E+00_JPRB ,&
&  0.49015E+00_JPRB, 0.49528E+00_JPRB, 0.50814E+00_JPRB, 0.53568E+00_JPRB, 0.58404E+00_JPRB, 0.62850E+00_JPRB, 0.59567E+00_JPRB ,&
&  0.63925E+00_JPRB, 0.65724E+00_JPRB, 0.66296E+00_JPRB, 0.66101E+00_JPRB, 0.64791E+00_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.62229E+00_JPRB, 0.60682E+00_JPRB, 0.59147E+00_JPRB, 0.58121E+00_JPRB, 0.57424E+00_JPRB, 0.57370E+00_JPRB, 0.56343E+00_JPRB ,&
&  0.54385E+00_JPRB, 0.52765E+00_JPRB, 0.51746E+00_JPRB, 0.51558E+00_JPRB, 0.51069E+00_JPRB, 0.49959E+00_JPRB, 0.49889E+00_JPRB ,&
&  0.52701E+00_JPRB, 0.52326E+00_JPRB, 0.47321E+00_JPRB, 0.43949E+00_JPRB, 0.46707E+00_JPRB, 0.46080E+00_JPRB, 0.46831E+00_JPRB ,&
&  0.46158E+00_JPRB, 0.48419E+00_JPRB, 0.48087E+00_JPRB, 0.48085E+00_JPRB, 0.48506E+00_JPRB, 0.50272E+00_JPRB, 0.54172E+00_JPRB ,&
&  0.54760E+00_JPRB, 0.50030E+00_JPRB, 0.47467E+00_JPRB, 0.48893E+00_JPRB, 0.55058E+00_JPRB, 0.54388E+00_JPRB, 0.58683E+00_JPRB ,&
&  0.61889E+00_JPRB, 0.65101E+00_JPRB, 0.67871E+00_JPRB, 0.70385E+00_JPRB, 0.72128E+00_JPRB, 0.73409E+00_JPRB, 0.74782E+00_JPRB ,&
&  0.76621E+00_JPRB, 0.78193E+00_JPRB, 0.77897E+00_JPRB, 0.80794E+00_JPRB, 0.89000E+00_JPRB, 0.10766E+01_JPRB, 0.12839E+01_JPRB ,&
&  0.14456E+01_JPRB, 0.14809E+01_JPRB, 0.15421E+01_JPRB, 0.17803E+01_JPRB, 0.19437E+01_JPRB, 0.20124E+01_JPRB, 0.20821E+01_JPRB ,&
&  0.20695E+01_JPRB, 0.20328E+01_JPRB, 0.19657E+01_JPRB, 0.17788E+01_JPRB, 0.15431E+01_JPRB, 0.14038E+01_JPRB, 0.13571E+01_JPRB ,&
&  0.13188E+01_JPRB, 0.13346E+01_JPRB, 0.13902E+01_JPRB, 0.13596E+01_JPRB, 0.13402E+01_JPRB, 0.12743E+01_JPRB, 0.13236E+01_JPRB ,&
&  0.12626E+01_JPRB, 0.78678E+00_JPRB, 0.61938E+00_JPRB, 0.64539E+00_JPRB, 0.54968E+00_JPRB, 0.55274E+00_JPRB, 0.60446E+00_JPRB ,&
&  0.58681E+00_JPRB, 0.58266E+00_JPRB, 0.60153E+00_JPRB, 0.59955E+00_JPRB, 0.59743E+00_JPRB, 0.62367E+00_JPRB, 0.62094E+00_JPRB ,&
&  0.59331E+00_JPRB, 0.56450E+00_JPRB, 0.58200E+00_JPRB, 0.62437E+00_JPRB, 0.68560E+00_JPRB, 0.71602E+00_JPRB, 0.72111E+00_JPRB ,&
&  0.72253E+00_JPRB, 0.71630E+00_JPRB, 0.70230E+00_JPRB, 0.67516E+00_JPRB, 0.64250E+00_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.64340E+00_JPRB, 0.62889E+00_JPRB, 0.61258E+00_JPRB, 0.60860E+00_JPRB, 0.60703E+00_JPRB, 0.60308E+00_JPRB, 0.58871E+00_JPRB ,&
&  0.55483E+00_JPRB, 0.54028E+00_JPRB, 0.53657E+00_JPRB, 0.53717E+00_JPRB, 0.53693E+00_JPRB, 0.53275E+00_JPRB, 0.52958E+00_JPRB ,&
&  0.51797E+00_JPRB, 0.54529E+00_JPRB, 0.49204E+00_JPRB, 0.46389E+00_JPRB, 0.42934E+00_JPRB, 0.44730E+00_JPRB, 0.47081E+00_JPRB ,&
&  0.49050E+00_JPRB, 0.50239E+00_JPRB, 0.53363E+00_JPRB, 0.57929E+00_JPRB, 0.62521E+00_JPRB, 0.67033E+00_JPRB, 0.76319E+00_JPRB ,&
&  0.75328E+00_JPRB, 0.67763E+00_JPRB, 0.60250E+00_JPRB, 0.62095E+00_JPRB, 0.65521E+00_JPRB, 0.61667E+00_JPRB, 0.67909E+00_JPRB ,&
&  0.71921E+00_JPRB, 0.75530E+00_JPRB, 0.79074E+00_JPRB, 0.81167E+00_JPRB, 0.81698E+00_JPRB, 0.79054E+00_JPRB, 0.77727E+00_JPRB ,&
&  0.77546E+00_JPRB, 0.79444E+00_JPRB, 0.82878E+00_JPRB, 0.85825E+00_JPRB, 0.10465E+01_JPRB, 0.12666E+01_JPRB, 0.14216E+01_JPRB ,&
&  0.15770E+01_JPRB, 0.13721E+01_JPRB, 0.12703E+01_JPRB, 0.16947E+01_JPRB, 0.23321E+01_JPRB, 0.24862E+01_JPRB, 0.22726E+01_JPRB ,&
&  0.23335E+01_JPRB, 0.20579E+01_JPRB, 0.17514E+01_JPRB, 0.15536E+01_JPRB, 0.14272E+01_JPRB, 0.13408E+01_JPRB, 0.13008E+01_JPRB ,&
&  0.12236E+01_JPRB, 0.12158E+01_JPRB, 0.12775E+01_JPRB, 0.13106E+01_JPRB, 0.12987E+01_JPRB, 0.12739E+01_JPRB, 0.12874E+01_JPRB ,&
&  0.11944E+01_JPRB, 0.97292E+00_JPRB, 0.70196E+00_JPRB, 0.64867E+00_JPRB, 0.54171E+00_JPRB, 0.53496E+00_JPRB, 0.58544E+00_JPRB ,&
&  0.57979E+00_JPRB, 0.60887E+00_JPRB, 0.62187E+00_JPRB, 0.61569E+00_JPRB, 0.66166E+00_JPRB, 0.68441E+00_JPRB, 0.73356E+00_JPRB ,&
&  0.75928E+00_JPRB, 0.67327E+00_JPRB, 0.71330E+00_JPRB, 0.76635E+00_JPRB, 0.81391E+00_JPRB, 0.81562E+00_JPRB, 0.79462E+00_JPRB ,&
&  0.77551E+00_JPRB, 0.75414E+00_JPRB, 0.72384E+00_JPRB, 0.68562E+00_JPRB, 0.65963E+00_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.64724E+00_JPRB, 0.63272E+00_JPRB, 0.61246E+00_JPRB, 0.60975E+00_JPRB, 0.60677E+00_JPRB, 0.60332E+00_JPRB, 0.59954E+00_JPRB ,&
&  0.59361E+00_JPRB, 0.59394E+00_JPRB, 0.58743E+00_JPRB, 0.57999E+00_JPRB, 0.57458E+00_JPRB, 0.56173E+00_JPRB, 0.55094E+00_JPRB ,&
&  0.53350E+00_JPRB, 0.53011E+00_JPRB, 0.47002E+00_JPRB, 0.43309E+00_JPRB, 0.41354E+00_JPRB, 0.42718E+00_JPRB, 0.45908E+00_JPRB ,&
&  0.50760E+00_JPRB, 0.57026E+00_JPRB, 0.68036E+00_JPRB, 0.78614E+00_JPRB, 0.93736E+00_JPRB, 0.10739E+01_JPRB, 0.11798E+01_JPRB ,&
&  0.11980E+01_JPRB, 0.90713E+00_JPRB, 0.86368E+00_JPRB, 0.79880E+00_JPRB, 0.80456E+00_JPRB, 0.78183E+00_JPRB, 0.78599E+00_JPRB ,&
&  0.78889E+00_JPRB, 0.81499E+00_JPRB, 0.83739E+00_JPRB, 0.82611E+00_JPRB, 0.80645E+00_JPRB, 0.79230E+00_JPRB, 0.81936E+00_JPRB ,&
&  0.85403E+00_JPRB, 0.85256E+00_JPRB, 0.87803E+00_JPRB, 0.10222E+01_JPRB, 0.13352E+01_JPRB, 0.15617E+01_JPRB, 0.16998E+01_JPRB ,&
&  0.16141E+01_JPRB, 0.15692E+01_JPRB, 0.22887E+01_JPRB, 0.25701E+01_JPRB, 0.23730E+01_JPRB, 0.25777E+01_JPRB, 0.24686E+01_JPRB ,&
&  0.21897E+01_JPRB, 0.17303E+01_JPRB, 0.14978E+01_JPRB, 0.13499E+01_JPRB, 0.15470E+01_JPRB, 0.14686E+01_JPRB, 0.13201E+01_JPRB ,&
&  0.11854E+01_JPRB, 0.12439E+01_JPRB, 0.12701E+01_JPRB, 0.13193E+01_JPRB, 0.13720E+01_JPRB, 0.13753E+01_JPRB, 0.12370E+01_JPRB ,&
&  0.10828E+01_JPRB, 0.11300E+01_JPRB, 0.91469E+00_JPRB, 0.70847E+00_JPRB, 0.61285E+00_JPRB, 0.61108E+00_JPRB, 0.62412E+00_JPRB ,&
&  0.63666E+00_JPRB, 0.64385E+00_JPRB, 0.64361E+00_JPRB, 0.67951E+00_JPRB, 0.75332E+00_JPRB, 0.79734E+00_JPRB, 0.83213E+00_JPRB ,&
&  0.82129E+00_JPRB, 0.87869E+00_JPRB, 0.89503E+00_JPRB, 0.90995E+00_JPRB, 0.90108E+00_JPRB, 0.84935E+00_JPRB, 0.80755E+00_JPRB ,&
&  0.78350E+00_JPRB, 0.77192E+00_JPRB, 0.73508E+00_JPRB, 0.69351E+00_JPRB, 0.65871E+00_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.66044E+00_JPRB, 0.65408E+00_JPRB, 0.64388E+00_JPRB, 0.63124E+00_JPRB, 0.62362E+00_JPRB, 0.62552E+00_JPRB, 0.63588E+00_JPRB ,&
&  0.63792E+00_JPRB, 0.62686E+00_JPRB, 0.63062E+00_JPRB, 0.64168E+00_JPRB, 0.63733E+00_JPRB, 0.61060E+00_JPRB, 0.58406E+00_JPRB ,&
&  0.56632E+00_JPRB, 0.51375E+00_JPRB, 0.44698E+00_JPRB, 0.44737E+00_JPRB, 0.44914E+00_JPRB, 0.44280E+00_JPRB, 0.47882E+00_JPRB ,&
&  0.54127E+00_JPRB, 0.67354E+00_JPRB, 0.84249E+00_JPRB, 0.10556E+01_JPRB, 0.12204E+01_JPRB, 0.13046E+01_JPRB, 0.13511E+01_JPRB ,&
&  0.12295E+01_JPRB, 0.11420E+01_JPRB, 0.10765E+01_JPRB, 0.10026E+01_JPRB, 0.96957E+00_JPRB, 0.94764E+00_JPRB, 0.90751E+00_JPRB ,&
&  0.86840E+00_JPRB, 0.86063E+00_JPRB, 0.86316E+00_JPRB, 0.83736E+00_JPRB, 0.83737E+00_JPRB, 0.85855E+00_JPRB, 0.89135E+00_JPRB ,&
&  0.88070E+00_JPRB, 0.87809E+00_JPRB, 0.95518E+00_JPRB, 0.11939E+01_JPRB, 0.12588E+01_JPRB, 0.14339E+01_JPRB, 0.16273E+01_JPRB ,&
&  0.22525E+01_JPRB, 0.25154E+01_JPRB, 0.24831E+01_JPRB, 0.31414E+01_JPRB, 0.19245E+01_JPRB, 0.20665E+01_JPRB, 0.22167E+01_JPRB ,&
&  0.20152E+01_JPRB, 0.18380E+01_JPRB, 0.18897E+01_JPRB, 0.16795E+01_JPRB, 0.14507E+01_JPRB, 0.19131E+01_JPRB, 0.15942E+01_JPRB ,&
&  0.13627E+01_JPRB, 0.13611E+01_JPRB, 0.14229E+01_JPRB, 0.15531E+01_JPRB, 0.13259E+01_JPRB, 0.11671E+01_JPRB, 0.95451E+00_JPRB ,&
&  0.93799E+00_JPRB, 0.85070E+00_JPRB, 0.90454E+00_JPRB, 0.79874E+00_JPRB, 0.78001E+00_JPRB, 0.72401E+00_JPRB, 0.67129E+00_JPRB ,&
&  0.67016E+00_JPRB, 0.66500E+00_JPRB, 0.68859E+00_JPRB, 0.78326E+00_JPRB, 0.93160E+00_JPRB, 0.88525E+00_JPRB, 0.94009E+00_JPRB ,&
&  0.98213E+00_JPRB, 0.10195E+01_JPRB, 0.99210E+00_JPRB, 0.97047E+00_JPRB, 0.88704E+00_JPRB, 0.83424E+00_JPRB, 0.80610E+00_JPRB ,&
&  0.78519E+00_JPRB, 0.74607E+00_JPRB, 0.70319E+00_JPRB, 0.68847E+00_JPRB, 0.66143E+00_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.67172E+00_JPRB, 0.65968E+00_JPRB, 0.66023E+00_JPRB, 0.65554E+00_JPRB, 0.66238E+00_JPRB, 0.66773E+00_JPRB, 0.67469E+00_JPRB ,&
&  0.66911E+00_JPRB, 0.66055E+00_JPRB, 0.66806E+00_JPRB, 0.67152E+00_JPRB, 0.65993E+00_JPRB, 0.63984E+00_JPRB, 0.60177E+00_JPRB ,&
&  0.57776E+00_JPRB, 0.53832E+00_JPRB, 0.46411E+00_JPRB, 0.44722E+00_JPRB, 0.46560E+00_JPRB, 0.46719E+00_JPRB, 0.49514E+00_JPRB ,&
&  0.63884E+00_JPRB, 0.83721E+00_JPRB, 0.10007E+01_JPRB, 0.11809E+01_JPRB, 0.13211E+01_JPRB, 0.13175E+01_JPRB, 0.12952E+01_JPRB ,&
&  0.12936E+01_JPRB, 0.11205E+01_JPRB, 0.10144E+01_JPRB, 0.99723E+00_JPRB, 0.96124E+00_JPRB, 0.92063E+00_JPRB, 0.88490E+00_JPRB ,&
&  0.84995E+00_JPRB, 0.81691E+00_JPRB, 0.79394E+00_JPRB, 0.77956E+00_JPRB, 0.79743E+00_JPRB, 0.81870E+00_JPRB, 0.84569E+00_JPRB ,&
&  0.86481E+00_JPRB, 0.91806E+00_JPRB, 0.10946E+01_JPRB, 0.13255E+01_JPRB, 0.14514E+01_JPRB, 0.16176E+01_JPRB, 0.20659E+01_JPRB ,&
&  0.23376E+01_JPRB, 0.23723E+01_JPRB, 0.26929E+01_JPRB, 0.30642E+01_JPRB, 0.26320E+01_JPRB, 0.21036E+01_JPRB, 0.21472E+01_JPRB ,&
&  0.16411E+01_JPRB, 0.14353E+01_JPRB, 0.13900E+01_JPRB, 0.14146E+01_JPRB, 0.18064E+01_JPRB, 0.25795E+01_JPRB, 0.22811E+01_JPRB ,&
&  0.20323E+01_JPRB, 0.17765E+01_JPRB, 0.17890E+01_JPRB, 0.16230E+01_JPRB, 0.13906E+01_JPRB, 0.10272E+01_JPRB, 0.12157E+01_JPRB ,&
&  0.12406E+01_JPRB, 0.12500E+01_JPRB, 0.11253E+01_JPRB, 0.93391E+00_JPRB, 0.78188E+00_JPRB, 0.82207E+00_JPRB, 0.78681E+00_JPRB ,&
&  0.75615E+00_JPRB, 0.74590E+00_JPRB, 0.92639E+00_JPRB, 0.10115E+01_JPRB, 0.10662E+01_JPRB, 0.10975E+01_JPRB, 0.11840E+01_JPRB ,&
&  0.11234E+01_JPRB, 0.11532E+01_JPRB, 0.10957E+01_JPRB, 0.96185E+00_JPRB, 0.87356E+00_JPRB, 0.82658E+00_JPRB, 0.79211E+00_JPRB ,&
&  0.76048E+00_JPRB, 0.71132E+00_JPRB, 0.69319E+00_JPRB, 0.67488E+00_JPRB, 0.66147E+00_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.67997E+00_JPRB, 0.68642E+00_JPRB, 0.68929E+00_JPRB, 0.68843E+00_JPRB, 0.67766E+00_JPRB, 0.66788E+00_JPRB, 0.67731E+00_JPRB ,&
&  0.67601E+00_JPRB, 0.66807E+00_JPRB, 0.65400E+00_JPRB, 0.64491E+00_JPRB, 0.62965E+00_JPRB, 0.61885E+00_JPRB, 0.59763E+00_JPRB ,&
&  0.58712E+00_JPRB, 0.59007E+00_JPRB, 0.50458E+00_JPRB, 0.48655E+00_JPRB, 0.49578E+00_JPRB, 0.50914E+00_JPRB, 0.53865E+00_JPRB ,&
&  0.73457E+00_JPRB, 0.94824E+00_JPRB, 0.10535E+01_JPRB, 0.11752E+01_JPRB, 0.12847E+01_JPRB, 0.12762E+01_JPRB, 0.12698E+01_JPRB ,&
&  0.12684E+01_JPRB, 0.11695E+01_JPRB, 0.10616E+01_JPRB, 0.99088E+00_JPRB, 0.92344E+00_JPRB, 0.88229E+00_JPRB, 0.84851E+00_JPRB ,&
&  0.81427E+00_JPRB, 0.78472E+00_JPRB, 0.77523E+00_JPRB, 0.78269E+00_JPRB, 0.76806E+00_JPRB, 0.81522E+00_JPRB, 0.87181E+00_JPRB ,&
&  0.92980E+00_JPRB, 0.10612E+01_JPRB, 0.12308E+01_JPRB, 0.14810E+01_JPRB, 0.16801E+01_JPRB, 0.18733E+01_JPRB, 0.25764E+01_JPRB ,&
&  0.25194E+01_JPRB, 0.25620E+01_JPRB, 0.27847E+01_JPRB, 0.37648E+01_JPRB, 0.25716E+01_JPRB, 0.20453E+01_JPRB, 0.20458E+01_JPRB ,&
&  0.15449E+01_JPRB, 0.14031E+01_JPRB, 0.16225E+01_JPRB, 0.20531E+01_JPRB, 0.21205E+01_JPRB, 0.23355E+01_JPRB, 0.29506E+01_JPRB ,&
&  0.25988E+01_JPRB, 0.22695E+01_JPRB, 0.21247E+01_JPRB, 0.20328E+01_JPRB, 0.10866E+01_JPRB, 0.84309E+00_JPRB, 0.12056E+01_JPRB ,&
&  0.12419E+01_JPRB, 0.86284E+00_JPRB, 0.68071E+00_JPRB, 0.80235E+00_JPRB, 0.64404E+00_JPRB, 0.70284E+00_JPRB, 0.86186E+00_JPRB ,&
&  0.86325E+00_JPRB, 0.92405E+00_JPRB, 0.12545E+01_JPRB, 0.12212E+01_JPRB, 0.11833E+01_JPRB, 0.14626E+01_JPRB, 0.14245E+01_JPRB ,&
&  0.13221E+01_JPRB, 0.11580E+01_JPRB, 0.10746E+01_JPRB, 0.92593E+00_JPRB, 0.85022E+00_JPRB, 0.79487E+00_JPRB, 0.75227E+00_JPRB ,&
&  0.70352E+00_JPRB, 0.68257E+00_JPRB, 0.68922E+00_JPRB, 0.69038E+00_JPRB, 0.66613E+00_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.68767E+00_JPRB, 0.68619E+00_JPRB, 0.68172E+00_JPRB, 0.68978E+00_JPRB, 0.67550E+00_JPRB, 0.65762E+00_JPRB, 0.68366E+00_JPRB ,&
&  0.67955E+00_JPRB, 0.65405E+00_JPRB, 0.62564E+00_JPRB, 0.61598E+00_JPRB, 0.60810E+00_JPRB, 0.61801E+00_JPRB, 0.61737E+00_JPRB ,&
&  0.61900E+00_JPRB, 0.62023E+00_JPRB, 0.60776E+00_JPRB, 0.59331E+00_JPRB, 0.59153E+00_JPRB, 0.59355E+00_JPRB, 0.66544E+00_JPRB ,&
&  0.87841E+00_JPRB, 0.10199E+01_JPRB, 0.10679E+01_JPRB, 0.11383E+01_JPRB, 0.12075E+01_JPRB, 0.12084E+01_JPRB, 0.12857E+01_JPRB ,&
&  0.12429E+01_JPRB, 0.11310E+01_JPRB, 0.10372E+01_JPRB, 0.95403E+00_JPRB, 0.90639E+00_JPRB, 0.87354E+00_JPRB, 0.84574E+00_JPRB ,&
&  0.80519E+00_JPRB, 0.78050E+00_JPRB, 0.79400E+00_JPRB, 0.76988E+00_JPRB, 0.78985E+00_JPRB, 0.85828E+00_JPRB, 0.93486E+00_JPRB ,&
&  0.10412E+01_JPRB, 0.12032E+01_JPRB, 0.13634E+01_JPRB, 0.16535E+01_JPRB, 0.20345E+01_JPRB, 0.21950E+01_JPRB, 0.20637E+01_JPRB ,&
&  0.19365E+01_JPRB, 0.19329E+01_JPRB, 0.26063E+01_JPRB, 0.30771E+01_JPRB, 0.26736E+01_JPRB, 0.23358E+01_JPRB, 0.20961E+01_JPRB ,&
&  0.18224E+01_JPRB, 0.17247E+01_JPRB, 0.18645E+01_JPRB, 0.23584E+01_JPRB, 0.25962E+01_JPRB, 0.24163E+01_JPRB, 0.30125E+01_JPRB ,&
&  0.30090E+01_JPRB, 0.26336E+01_JPRB, 0.18562E+01_JPRB, 0.13891E+01_JPRB, 0.12205E+01_JPRB, 0.98751E+00_JPRB, 0.73652E+00_JPRB ,&
&  0.67898E+00_JPRB, 0.67087E+00_JPRB, 0.62914E+00_JPRB, 0.63140E+00_JPRB, 0.60921E+00_JPRB, 0.70046E+00_JPRB, 0.84371E+00_JPRB ,&
&  0.10485E+01_JPRB, 0.12886E+01_JPRB, 0.14027E+01_JPRB, 0.13671E+01_JPRB, 0.12996E+01_JPRB, 0.16315E+01_JPRB, 0.16271E+01_JPRB ,&
&  0.11991E+01_JPRB, 0.10778E+01_JPRB, 0.97147E+00_JPRB, 0.86630E+00_JPRB, 0.80146E+00_JPRB, 0.74271E+00_JPRB, 0.70152E+00_JPRB ,&
&  0.68238E+00_JPRB, 0.68343E+00_JPRB, 0.68520E+00_JPRB, 0.68037E+00_JPRB, 0.66569E+00_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.69234E+00_JPRB, 0.68540E+00_JPRB, 0.67464E+00_JPRB, 0.66770E+00_JPRB, 0.64134E+00_JPRB, 0.64945E+00_JPRB, 0.68528E+00_JPRB ,&
&  0.67804E+00_JPRB, 0.63077E+00_JPRB, 0.60438E+00_JPRB, 0.60481E+00_JPRB, 0.62441E+00_JPRB, 0.64150E+00_JPRB, 0.64204E+00_JPRB ,&
&  0.65330E+00_JPRB, 0.66062E+00_JPRB, 0.70884E+00_JPRB, 0.72129E+00_JPRB, 0.76219E+00_JPRB, 0.74957E+00_JPRB, 0.87596E+00_JPRB ,&
&  0.10223E+01_JPRB, 0.10565E+01_JPRB, 0.11050E+01_JPRB, 0.11653E+01_JPRB, 0.12293E+01_JPRB, 0.12688E+01_JPRB, 0.12887E+01_JPRB ,&
&  0.11771E+01_JPRB, 0.10742E+01_JPRB, 0.97514E+00_JPRB, 0.91750E+00_JPRB, 0.88476E+00_JPRB, 0.83891E+00_JPRB, 0.81433E+00_JPRB ,&
&  0.79372E+00_JPRB, 0.79955E+00_JPRB, 0.77714E+00_JPRB, 0.79437E+00_JPRB, 0.87417E+00_JPRB, 0.96686E+00_JPRB, 0.10657E+01_JPRB ,&
&  0.11851E+01_JPRB, 0.13659E+01_JPRB, 0.15657E+01_JPRB, 0.18884E+01_JPRB, 0.18239E+01_JPRB, 0.15884E+01_JPRB, 0.18186E+01_JPRB ,&
&  0.20195E+01_JPRB, 0.22316E+01_JPRB, 0.22302E+01_JPRB, 0.29434E+01_JPRB, 0.28338E+01_JPRB, 0.23342E+01_JPRB, 0.20682E+01_JPRB ,&
&  0.18269E+01_JPRB, 0.20493E+01_JPRB, 0.22511E+01_JPRB, 0.25361E+01_JPRB, 0.32715E+01_JPRB, 0.30718E+01_JPRB, 0.32733E+01_JPRB ,&
&  0.33894E+01_JPRB, 0.28261E+01_JPRB, 0.20696E+01_JPRB, 0.15654E+01_JPRB, 0.17197E+01_JPRB, 0.16593E+01_JPRB, 0.84688E+00_JPRB ,&
&  0.71520E+00_JPRB, 0.67665E+00_JPRB, 0.65514E+00_JPRB, 0.61615E+00_JPRB, 0.60553E+00_JPRB, 0.71069E+00_JPRB, 0.10657E+01_JPRB ,&
&  0.10251E+01_JPRB, 0.15932E+01_JPRB, 0.15099E+01_JPRB, 0.14710E+01_JPRB, 0.13894E+01_JPRB, 0.17716E+01_JPRB, 0.16570E+01_JPRB ,&
&  0.13037E+01_JPRB, 0.11388E+01_JPRB, 0.95349E+00_JPRB, 0.86111E+00_JPRB, 0.77753E+00_JPRB, 0.71683E+00_JPRB, 0.70298E+00_JPRB ,&
&  0.68337E+00_JPRB, 0.65713E+00_JPRB, 0.64989E+00_JPRB, 0.65842E+00_JPRB, 0.67282E+00_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.67589E+00_JPRB, 0.66892E+00_JPRB, 0.67189E+00_JPRB, 0.67773E+00_JPRB, 0.66186E+00_JPRB, 0.68682E+00_JPRB, 0.69126E+00_JPRB ,&
&  0.66484E+00_JPRB, 0.63373E+00_JPRB, 0.62446E+00_JPRB, 0.63505E+00_JPRB, 0.65346E+00_JPRB, 0.67282E+00_JPRB, 0.69439E+00_JPRB ,&
&  0.72026E+00_JPRB, 0.75440E+00_JPRB, 0.81051E+00_JPRB, 0.91213E+00_JPRB, 0.96438E+00_JPRB, 0.96567E+00_JPRB, 0.10852E+01_JPRB ,&
&  0.10852E+01_JPRB, 0.10443E+01_JPRB, 0.11368E+01_JPRB, 0.12241E+01_JPRB, 0.12167E+01_JPRB, 0.12112E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.10815E+01_JPRB, 0.10214E+01_JPRB, 0.94414E+00_JPRB, 0.91972E+00_JPRB, 0.91128E+00_JPRB, 0.87214E+00_JPRB, 0.85661E+00_JPRB ,&
&  0.84884E+00_JPRB, 0.82833E+00_JPRB, 0.83115E+00_JPRB, 0.87171E+00_JPRB, 0.98158E+00_JPRB, 0.10739E+01_JPRB, 0.11949E+01_JPRB ,&
&  0.13507E+01_JPRB, 0.15187E+01_JPRB, 0.17088E+01_JPRB, 0.19504E+01_JPRB, 0.16384E+01_JPRB, 0.19435E+01_JPRB, 0.20485E+01_JPRB ,&
&  0.21700E+01_JPRB, 0.21489E+01_JPRB, 0.19957E+01_JPRB, 0.24881E+01_JPRB, 0.26021E+01_JPRB, 0.21977E+01_JPRB, 0.19113E+01_JPRB ,&
&  0.20252E+01_JPRB, 0.21972E+01_JPRB, 0.26412E+01_JPRB, 0.30292E+01_JPRB, 0.34407E+01_JPRB, 0.46705E+01_JPRB, 0.33082E+01_JPRB ,&
&  0.27769E+01_JPRB, 0.27257E+01_JPRB, 0.22517E+01_JPRB, 0.18491E+01_JPRB, 0.19521E+01_JPRB, 0.22788E+01_JPRB, 0.16756E+01_JPRB ,&
&  0.76607E+00_JPRB, 0.62535E+00_JPRB, 0.67209E+00_JPRB, 0.67405E+00_JPRB, 0.61406E+00_JPRB, 0.70256E+00_JPRB, 0.15683E+01_JPRB ,&
&  0.12279E+01_JPRB, 0.16969E+01_JPRB, 0.15161E+01_JPRB, 0.14571E+01_JPRB, 0.14982E+01_JPRB, 0.17465E+01_JPRB, 0.18642E+01_JPRB ,&
&  0.12555E+01_JPRB, 0.10689E+01_JPRB, 0.91337E+00_JPRB, 0.82264E+00_JPRB, 0.74850E+00_JPRB, 0.70613E+00_JPRB, 0.68069E+00_JPRB ,&
&  0.65996E+00_JPRB, 0.64812E+00_JPRB, 0.64404E+00_JPRB, 0.65105E+00_JPRB, 0.66481E+00_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.68270E+00_JPRB, 0.66512E+00_JPRB, 0.66550E+00_JPRB, 0.66179E+00_JPRB, 0.70220E+00_JPRB, 0.73618E+00_JPRB, 0.69027E+00_JPRB ,&
&  0.68525E+00_JPRB, 0.68152E+00_JPRB, 0.69361E+00_JPRB, 0.68743E+00_JPRB, 0.69753E+00_JPRB, 0.74928E+00_JPRB, 0.79545E+00_JPRB ,&
&  0.84698E+00_JPRB, 0.90203E+00_JPRB, 0.97455E+00_JPRB, 0.10855E+01_JPRB, 0.11680E+01_JPRB, 0.11274E+01_JPRB, 0.10999E+01_JPRB ,&
&  0.10726E+01_JPRB, 0.10683E+01_JPRB, 0.10937E+01_JPRB, 0.12250E+01_JPRB, 0.12401E+01_JPRB, 0.12141E+01_JPRB, 0.11809E+01_JPRB ,&
&  0.10984E+01_JPRB, 0.10555E+01_JPRB, 0.98484E+00_JPRB, 0.97024E+00_JPRB, 0.94426E+00_JPRB, 0.95745E+00_JPRB, 0.10126E+01_JPRB ,&
&  0.97868E+00_JPRB, 0.96230E+00_JPRB, 0.98700E+00_JPRB, 0.10344E+01_JPRB, 0.11424E+01_JPRB, 0.12394E+01_JPRB, 0.13650E+01_JPRB ,&
&  0.15125E+01_JPRB, 0.16504E+01_JPRB, 0.18183E+01_JPRB, 0.18684E+01_JPRB, 0.18925E+01_JPRB, 0.19768E+01_JPRB, 0.21682E+01_JPRB ,&
&  0.22337E+01_JPRB, 0.22001E+01_JPRB, 0.21480E+01_JPRB, 0.22392E+01_JPRB, 0.22148E+01_JPRB, 0.20418E+01_JPRB, 0.19306E+01_JPRB ,&
&  0.22654E+01_JPRB, 0.27804E+01_JPRB, 0.31467E+01_JPRB, 0.34910E+01_JPRB, 0.37202E+01_JPRB, 0.46557E+01_JPRB, 0.42384E+01_JPRB ,&
&  0.32260E+01_JPRB, 0.24855E+01_JPRB, 0.22058E+01_JPRB, 0.22345E+01_JPRB, 0.22785E+01_JPRB, 0.21934E+01_JPRB, 0.23128E+01_JPRB ,&
&  0.22205E+01_JPRB, 0.13059E+01_JPRB, 0.10252E+01_JPRB, 0.13113E+01_JPRB, 0.93571E+00_JPRB, 0.71271E+00_JPRB, 0.95050E+00_JPRB ,&
&  0.13165E+01_JPRB, 0.15887E+01_JPRB, 0.15636E+01_JPRB, 0.14572E+01_JPRB, 0.16013E+01_JPRB, 0.16488E+01_JPRB, 0.15470E+01_JPRB ,&
&  0.11084E+01_JPRB, 0.92947E+00_JPRB, 0.82250E+00_JPRB, 0.75179E+00_JPRB, 0.69868E+00_JPRB, 0.68007E+00_JPRB, 0.67531E+00_JPRB ,&
&  0.66053E+00_JPRB, 0.66198E+00_JPRB, 0.66113E+00_JPRB, 0.65993E+00_JPRB, 0.67573E+00_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.67485E+00_JPRB, 0.66159E+00_JPRB, 0.65125E+00_JPRB, 0.65929E+00_JPRB, 0.72333E+00_JPRB, 0.75142E+00_JPRB, 0.73396E+00_JPRB ,&
&  0.72160E+00_JPRB, 0.72227E+00_JPRB, 0.75380E+00_JPRB, 0.74177E+00_JPRB, 0.78735E+00_JPRB, 0.89322E+00_JPRB, 0.96115E+00_JPRB ,&
&  0.10133E+01_JPRB, 0.10695E+01_JPRB, 0.11455E+01_JPRB, 0.12628E+01_JPRB, 0.14109E+01_JPRB, 0.15912E+01_JPRB, 0.10159E+01_JPRB ,&
&  0.94559E+00_JPRB, 0.11087E+01_JPRB, 0.11499E+01_JPRB, 0.12874E+01_JPRB, 0.12653E+01_JPRB, 0.12458E+01_JPRB, 0.11865E+01_JPRB ,&
&  0.11548E+01_JPRB, 0.11227E+01_JPRB, 0.11134E+01_JPRB, 0.11284E+01_JPRB, 0.11526E+01_JPRB, 0.12483E+01_JPRB, 0.12710E+01_JPRB ,&
&  0.12399E+01_JPRB, 0.12469E+01_JPRB, 0.13116E+01_JPRB, 0.13724E+01_JPRB, 0.14418E+01_JPRB, 0.15052E+01_JPRB, 0.15858E+01_JPRB ,&
&  0.16815E+01_JPRB, 0.18002E+01_JPRB, 0.19523E+01_JPRB, 0.18779E+01_JPRB, 0.18822E+01_JPRB, 0.19423E+01_JPRB, 0.20847E+01_JPRB ,&
&  0.20064E+01_JPRB, 0.18372E+01_JPRB, 0.19394E+01_JPRB, 0.20624E+01_JPRB, 0.20104E+01_JPRB, 0.19106E+01_JPRB, 0.21238E+01_JPRB ,&
&  0.27889E+01_JPRB, 0.34601E+01_JPRB, 0.42030E+01_JPRB, 0.37695E+01_JPRB, 0.37932E+01_JPRB, 0.41013E+01_JPRB, 0.38793E+01_JPRB ,&
&  0.34824E+01_JPRB, 0.28527E+01_JPRB, 0.25551E+01_JPRB, 0.24201E+01_JPRB, 0.22181E+01_JPRB, 0.19005E+01_JPRB, 0.18502E+01_JPRB ,&
&  0.21000E+01_JPRB, 0.24679E+01_JPRB, 0.20936E+01_JPRB, 0.14102E+01_JPRB, 0.10562E+01_JPRB, 0.80393E+00_JPRB, 0.92037E+00_JPRB ,&
&  0.14143E+01_JPRB, 0.14552E+01_JPRB, 0.14442E+01_JPRB, 0.16612E+01_JPRB, 0.15027E+01_JPRB, 0.14020E+01_JPRB, 0.11567E+01_JPRB ,&
&  0.89013E+00_JPRB, 0.74944E+00_JPRB, 0.69321E+00_JPRB, 0.65083E+00_JPRB, 0.64975E+00_JPRB, 0.65523E+00_JPRB, 0.67313E+00_JPRB ,&
&  0.68798E+00_JPRB, 0.68851E+00_JPRB, 0.67646E+00_JPRB, 0.66848E+00_JPRB, 0.67758E+00_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.68916E+00_JPRB, 0.67227E+00_JPRB, 0.65384E+00_JPRB, 0.67888E+00_JPRB, 0.74125E+00_JPRB, 0.76243E+00_JPRB, 0.76953E+00_JPRB ,&
&  0.77459E+00_JPRB, 0.76641E+00_JPRB, 0.80685E+00_JPRB, 0.82489E+00_JPRB, 0.91667E+00_JPRB, 0.10153E+01_JPRB, 0.10871E+01_JPRB ,&
&  0.11505E+01_JPRB, 0.12047E+01_JPRB, 0.12862E+01_JPRB, 0.14006E+01_JPRB, 0.15035E+01_JPRB, 0.15787E+01_JPRB, 0.12182E+01_JPRB ,&
&  0.87508E+00_JPRB, 0.13363E+01_JPRB, 0.12537E+01_JPRB, 0.13709E+01_JPRB, 0.13668E+01_JPRB, 0.14159E+01_JPRB, 0.13489E+01_JPRB ,&
&  0.13255E+01_JPRB, 0.12959E+01_JPRB, 0.13662E+01_JPRB, 0.13943E+01_JPRB, 0.14285E+01_JPRB, 0.14825E+01_JPRB, 0.15427E+01_JPRB ,&
&  0.15923E+01_JPRB, 0.16554E+01_JPRB, 0.16820E+01_JPRB, 0.16855E+01_JPRB, 0.17030E+01_JPRB, 0.17267E+01_JPRB, 0.17727E+01_JPRB ,&
&  0.18667E+01_JPRB, 0.19829E+01_JPRB, 0.20046E+01_JPRB, 0.19875E+01_JPRB, 0.18969E+01_JPRB, 0.19474E+01_JPRB, 0.20158E+01_JPRB ,&
&  0.18843E+01_JPRB, 0.17196E+01_JPRB, 0.18071E+01_JPRB, 0.18376E+01_JPRB, 0.18971E+01_JPRB, 0.22923E+01_JPRB, 0.26805E+01_JPRB ,&
&  0.34044E+01_JPRB, 0.36990E+01_JPRB, 0.46390E+01_JPRB, 0.34197E+01_JPRB, 0.32566E+01_JPRB, 0.34397E+01_JPRB, 0.30091E+01_JPRB ,&
&  0.25965E+01_JPRB, 0.25530E+01_JPRB, 0.23647E+01_JPRB, 0.22885E+01_JPRB, 0.20967E+01_JPRB, 0.18060E+01_JPRB, 0.16030E+01_JPRB ,&
&  0.15977E+01_JPRB, 0.16450E+01_JPRB, 0.14601E+01_JPRB, 0.10496E+01_JPRB, 0.91766E+00_JPRB, 0.89123E+00_JPRB, 0.12015E+01_JPRB ,&
&  0.14697E+01_JPRB, 0.15224E+01_JPRB, 0.15389E+01_JPRB, 0.14932E+01_JPRB, 0.11875E+01_JPRB, 0.99173E+00_JPRB, 0.77382E+00_JPRB ,&
&  0.62785E+00_JPRB, 0.57671E+00_JPRB, 0.61565E+00_JPRB, 0.61226E+00_JPRB, 0.63088E+00_JPRB, 0.65316E+00_JPRB, 0.66808E+00_JPRB ,&
&  0.67867E+00_JPRB, 0.68195E+00_JPRB, 0.68483E+00_JPRB, 0.68498E+00_JPRB, 0.69202E+00_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.64690E+00_JPRB, 0.64281E+00_JPRB, 0.65315E+00_JPRB, 0.73443E+00_JPRB, 0.85570E+00_JPRB, 0.95059E+00_JPRB, 0.97077E+00_JPRB ,&
&  0.97618E+00_JPRB, 0.80668E+00_JPRB, 0.84598E+00_JPRB, 0.87294E+00_JPRB, 0.96013E+00_JPRB, 0.10577E+01_JPRB, 0.11659E+01_JPRB ,&
&  0.12686E+01_JPRB, 0.13714E+01_JPRB, 0.14631E+01_JPRB, 0.15609E+01_JPRB, 0.16807E+01_JPRB, 0.17982E+01_JPRB, 0.15859E+01_JPRB ,&
&  0.11752E+01_JPRB, 0.10782E+01_JPRB, 0.13864E+01_JPRB, 0.13868E+01_JPRB, 0.13721E+01_JPRB, 0.14305E+01_JPRB, 0.14508E+01_JPRB ,&
&  0.14725E+01_JPRB, 0.14393E+01_JPRB, 0.14878E+01_JPRB, 0.15231E+01_JPRB, 0.15226E+01_JPRB, 0.16004E+01_JPRB, 0.17048E+01_JPRB ,&
&  0.17767E+01_JPRB, 0.18387E+01_JPRB, 0.18593E+01_JPRB, 0.18531E+01_JPRB, 0.18642E+01_JPRB, 0.19089E+01_JPRB, 0.19798E+01_JPRB ,&
&  0.20872E+01_JPRB, 0.22095E+01_JPRB, 0.22627E+01_JPRB, 0.21476E+01_JPRB, 0.21072E+01_JPRB, 0.21496E+01_JPRB, 0.21270E+01_JPRB ,&
&  0.20287E+01_JPRB, 0.20287E+01_JPRB, 0.21336E+01_JPRB, 0.23111E+01_JPRB, 0.23948E+01_JPRB, 0.29297E+01_JPRB, 0.32070E+01_JPRB ,&
&  0.35635E+01_JPRB, 0.37337E+01_JPRB, 0.41184E+01_JPRB, 0.29009E+01_JPRB, 0.28565E+01_JPRB, 0.28015E+01_JPRB, 0.25209E+01_JPRB ,&
&  0.23254E+01_JPRB, 0.22988E+01_JPRB, 0.21755E+01_JPRB, 0.20386E+01_JPRB, 0.19154E+01_JPRB, 0.15883E+01_JPRB, 0.15043E+01_JPRB ,&
&  0.13720E+01_JPRB, 0.14657E+01_JPRB, 0.12010E+01_JPRB, 0.10667E+01_JPRB, 0.94726E+00_JPRB, 0.10120E+01_JPRB, 0.11684E+01_JPRB ,&
&  0.13937E+01_JPRB, 0.14272E+01_JPRB, 0.13027E+01_JPRB, 0.10779E+01_JPRB, 0.77164E+00_JPRB, 0.62449E+00_JPRB, 0.55225E+00_JPRB ,&
&  0.55125E+00_JPRB, 0.56506E+00_JPRB, 0.58613E+00_JPRB, 0.60230E+00_JPRB, 0.61914E+00_JPRB, 0.63543E+00_JPRB, 0.63819E+00_JPRB ,&
&  0.63429E+00_JPRB, 0.62494E+00_JPRB, 0.62007E+00_JPRB, 0.64744E+00_JPRB, 0.66079E+00_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.57122E+00_JPRB, 0.56411E+00_JPRB, 0.58035E+00_JPRB, 0.66764E+00_JPRB, 0.75747E+00_JPRB, 0.81334E+00_JPRB, 0.79005E+00_JPRB ,&
&  0.74450E+00_JPRB, 0.70383E+00_JPRB, 0.74509E+00_JPRB, 0.79151E+00_JPRB, 0.86463E+00_JPRB, 0.96736E+00_JPRB, 0.10773E+01_JPRB ,&
&  0.11992E+01_JPRB, 0.12975E+01_JPRB, 0.13712E+01_JPRB, 0.14661E+01_JPRB, 0.15846E+01_JPRB, 0.17036E+01_JPRB, 0.17560E+01_JPRB ,&
&  0.14008E+01_JPRB, 0.10161E+01_JPRB, 0.12060E+01_JPRB, 0.13309E+01_JPRB, 0.14054E+01_JPRB, 0.14585E+01_JPRB, 0.14629E+01_JPRB ,&
&  0.13838E+01_JPRB, 0.13904E+01_JPRB, 0.14469E+01_JPRB, 0.14932E+01_JPRB, 0.15066E+01_JPRB, 0.15979E+01_JPRB, 0.16968E+01_JPRB ,&
&  0.18109E+01_JPRB, 0.18951E+01_JPRB, 0.19436E+01_JPRB, 0.19923E+01_JPRB, 0.20386E+01_JPRB, 0.21137E+01_JPRB, 0.22358E+01_JPRB ,&
&  0.23808E+01_JPRB, 0.25155E+01_JPRB, 0.26033E+01_JPRB, 0.25845E+01_JPRB, 0.25635E+01_JPRB, 0.26179E+01_JPRB, 0.25873E+01_JPRB ,&
&  0.25734E+01_JPRB, 0.25555E+01_JPRB, 0.28016E+01_JPRB, 0.31476E+01_JPRB, 0.33476E+01_JPRB, 0.30548E+01_JPRB, 0.33211E+01_JPRB ,&
&  0.33798E+01_JPRB, 0.32833E+01_JPRB, 0.30374E+01_JPRB, 0.31058E+01_JPRB, 0.21405E+01_JPRB, 0.21791E+01_JPRB, 0.20735E+01_JPRB ,&
&  0.21831E+01_JPRB, 0.20829E+01_JPRB, 0.19595E+01_JPRB, 0.17976E+01_JPRB, 0.17532E+01_JPRB, 0.14340E+01_JPRB, 0.13070E+01_JPRB ,&
&  0.12624E+01_JPRB, 0.12894E+01_JPRB, 0.11695E+01_JPRB, 0.11338E+01_JPRB, 0.10483E+01_JPRB, 0.11134E+01_JPRB, 0.11292E+01_JPRB ,&
&  0.12536E+01_JPRB, 0.11308E+01_JPRB, 0.96309E+00_JPRB, 0.79150E+00_JPRB, 0.56990E+00_JPRB, 0.49230E+00_JPRB, 0.50284E+00_JPRB ,&
&  0.52283E+00_JPRB, 0.54317E+00_JPRB, 0.54747E+00_JPRB, 0.56754E+00_JPRB, 0.58001E+00_JPRB, 0.57293E+00_JPRB, 0.56033E+00_JPRB ,&
&  0.55757E+00_JPRB, 0.57084E+00_JPRB, 0.56854E+00_JPRB, 0.57255E+00_JPRB, 0.57688E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.50477E+00_JPRB, 0.51217E+00_JPRB, 0.53625E+00_JPRB, 0.58090E+00_JPRB, 0.64546E+00_JPRB, 0.67932E+00_JPRB, 0.67671E+00_JPRB ,&
&  0.65079E+00_JPRB, 0.65362E+00_JPRB, 0.67609E+00_JPRB, 0.67696E+00_JPRB, 0.71879E+00_JPRB, 0.80862E+00_JPRB, 0.89677E+00_JPRB ,&
&  0.10153E+01_JPRB, 0.11251E+01_JPRB, 0.12299E+01_JPRB, 0.13933E+01_JPRB, 0.15579E+01_JPRB, 0.17432E+01_JPRB, 0.18462E+01_JPRB ,&
&  0.18071E+01_JPRB, 0.15604E+01_JPRB, 0.15174E+01_JPRB, 0.12636E+01_JPRB, 0.11002E+01_JPRB, 0.12934E+01_JPRB, 0.12762E+01_JPRB ,&
&  0.12804E+01_JPRB, 0.13175E+01_JPRB, 0.13279E+01_JPRB, 0.13823E+01_JPRB, 0.14719E+01_JPRB, 0.15609E+01_JPRB, 0.16686E+01_JPRB ,&
&  0.17968E+01_JPRB, 0.19232E+01_JPRB, 0.20374E+01_JPRB, 0.21415E+01_JPRB, 0.22120E+01_JPRB, 0.23100E+01_JPRB, 0.24615E+01_JPRB ,&
&  0.26273E+01_JPRB, 0.27556E+01_JPRB, 0.28922E+01_JPRB, 0.30643E+01_JPRB, 0.30342E+01_JPRB, 0.31192E+01_JPRB, 0.31500E+01_JPRB ,&
&  0.31652E+01_JPRB, 0.31172E+01_JPRB, 0.31317E+01_JPRB, 0.32439E+01_JPRB, 0.32106E+01_JPRB, 0.28421E+01_JPRB, 0.28086E+01_JPRB ,&
&  0.28727E+01_JPRB, 0.26318E+01_JPRB, 0.22552E+01_JPRB, 0.25018E+01_JPRB, 0.16648E+01_JPRB, 0.18498E+01_JPRB, 0.19919E+01_JPRB ,&
&  0.18903E+01_JPRB, 0.17899E+01_JPRB, 0.16233E+01_JPRB, 0.15662E+01_JPRB, 0.14998E+01_JPRB, 0.12779E+01_JPRB, 0.12222E+01_JPRB ,&
&  0.12199E+01_JPRB, 0.10458E+01_JPRB, 0.98100E+00_JPRB, 0.10139E+01_JPRB, 0.10726E+01_JPRB, 0.10034E+01_JPRB, 0.89595E+00_JPRB ,&
&  0.83128E+00_JPRB, 0.97802E+00_JPRB, 0.86818E+00_JPRB, 0.95832E+00_JPRB, 0.84986E+00_JPRB, 0.47510E+00_JPRB, 0.47450E+00_JPRB ,&
&  0.48732E+00_JPRB, 0.50115E+00_JPRB, 0.50482E+00_JPRB, 0.52449E+00_JPRB, 0.52667E+00_JPRB, 0.51975E+00_JPRB, 0.52233E+00_JPRB ,&
&  0.52647E+00_JPRB, 0.52380E+00_JPRB, 0.52536E+00_JPRB, 0.52351E+00_JPRB, 0.51884E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.53040E+00_JPRB, 0.52853E+00_JPRB, 0.53576E+00_JPRB, 0.53370E+00_JPRB, 0.54423E+00_JPRB, 0.54348E+00_JPRB, 0.54564E+00_JPRB ,&
&  0.55556E+00_JPRB, 0.57276E+00_JPRB, 0.56403E+00_JPRB, 0.54031E+00_JPRB, 0.53462E+00_JPRB, 0.54371E+00_JPRB, 0.57793E+00_JPRB ,&
&  0.65779E+00_JPRB, 0.73900E+00_JPRB, 0.81381E+00_JPRB, 0.92613E+00_JPRB, 0.11315E+01_JPRB, 0.14769E+01_JPRB, 0.18113E+01_JPRB ,&
&  0.20532E+01_JPRB, 0.22214E+01_JPRB, 0.22269E+01_JPRB, 0.23341E+01_JPRB, 0.18517E+01_JPRB, 0.12760E+01_JPRB, 0.14184E+01_JPRB ,&
&  0.15216E+01_JPRB, 0.15042E+01_JPRB, 0.13624E+01_JPRB, 0.13306E+01_JPRB, 0.13389E+01_JPRB, 0.14164E+01_JPRB, 0.15708E+01_JPRB ,&
&  0.17315E+01_JPRB, 0.19013E+01_JPRB, 0.20442E+01_JPRB, 0.21707E+01_JPRB, 0.22763E+01_JPRB, 0.24011E+01_JPRB, 0.25166E+01_JPRB ,&
&  0.26450E+01_JPRB, 0.27373E+01_JPRB, 0.28326E+01_JPRB, 0.29939E+01_JPRB, 0.31522E+01_JPRB, 0.33108E+01_JPRB, 0.33901E+01_JPRB ,&
&  0.33915E+01_JPRB, 0.31858E+01_JPRB, 0.31802E+01_JPRB, 0.31874E+01_JPRB, 0.29737E+01_JPRB, 0.26833E+01_JPRB, 0.25524E+01_JPRB ,&
&  0.24042E+01_JPRB, 0.21762E+01_JPRB, 0.16496E+01_JPRB, 0.19447E+01_JPRB, 0.21549E+01_JPRB, 0.18757E+01_JPRB, 0.17504E+01_JPRB ,&
&  0.16074E+01_JPRB, 0.15090E+01_JPRB, 0.13837E+01_JPRB, 0.13647E+01_JPRB, 0.14244E+01_JPRB, 0.15957E+01_JPRB, 0.12015E+01_JPRB ,&
&  0.11214E+01_JPRB, 0.85110E+00_JPRB, 0.77321E+00_JPRB, 0.81400E+00_JPRB, 0.85415E+00_JPRB, 0.80647E+00_JPRB, 0.69284E+00_JPRB ,&
&  0.70308E+00_JPRB, 0.88599E+00_JPRB, 0.93066E+00_JPRB, 0.13425E+01_JPRB, 0.17046E+01_JPRB, 0.49279E+00_JPRB, 0.45536E+00_JPRB ,&
&  0.46381E+00_JPRB, 0.46848E+00_JPRB, 0.47345E+00_JPRB, 0.48176E+00_JPRB, 0.47602E+00_JPRB, 0.48299E+00_JPRB, 0.49746E+00_JPRB ,&
&  0.50717E+00_JPRB, 0.51390E+00_JPRB, 0.52137E+00_JPRB, 0.51813E+00_JPRB, 0.52673E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.61340E+00_JPRB, 0.61403E+00_JPRB, 0.61684E+00_JPRB, 0.61239E+00_JPRB, 0.60782E+00_JPRB, 0.60532E+00_JPRB, 0.60605E+00_JPRB ,&
&  0.60522E+00_JPRB, 0.59564E+00_JPRB, 0.58626E+00_JPRB, 0.57822E+00_JPRB, 0.56554E+00_JPRB, 0.54060E+00_JPRB, 0.50441E+00_JPRB ,&
&  0.51017E+00_JPRB, 0.52066E+00_JPRB, 0.53722E+00_JPRB, 0.57739E+00_JPRB, 0.65901E+00_JPRB, 0.78328E+00_JPRB, 0.94643E+00_JPRB ,&
&  0.12651E+01_JPRB, 0.16194E+01_JPRB, 0.18695E+01_JPRB, 0.19594E+01_JPRB, 0.21668E+01_JPRB, 0.15558E+01_JPRB, 0.13927E+01_JPRB ,&
&  0.14745E+01_JPRB, 0.12989E+01_JPRB, 0.12529E+01_JPRB, 0.12226E+01_JPRB, 0.12336E+01_JPRB, 0.12820E+01_JPRB, 0.13895E+01_JPRB ,&
&  0.15576E+01_JPRB, 0.17287E+01_JPRB, 0.18826E+01_JPRB, 0.20389E+01_JPRB, 0.21846E+01_JPRB, 0.23394E+01_JPRB, 0.24593E+01_JPRB ,&
&  0.25284E+01_JPRB, 0.26105E+01_JPRB, 0.27520E+01_JPRB, 0.27837E+01_JPRB, 0.30054E+01_JPRB, 0.31556E+01_JPRB, 0.32178E+01_JPRB ,&
&  0.31275E+01_JPRB, 0.29550E+01_JPRB, 0.29320E+01_JPRB, 0.27549E+01_JPRB, 0.25678E+01_JPRB, 0.23964E+01_JPRB, 0.22073E+01_JPRB ,&
&  0.20223E+01_JPRB, 0.17609E+01_JPRB, 0.13918E+01_JPRB, 0.15167E+01_JPRB, 0.15564E+01_JPRB, 0.13793E+01_JPRB, 0.15720E+01_JPRB ,&
&  0.14573E+01_JPRB, 0.14245E+01_JPRB, 0.13377E+01_JPRB, 0.12572E+01_JPRB, 0.12030E+01_JPRB, 0.13219E+01_JPRB, 0.11533E+01_JPRB ,&
&  0.94542E+00_JPRB, 0.77486E+00_JPRB, 0.65529E+00_JPRB, 0.63524E+00_JPRB, 0.66453E+00_JPRB, 0.70871E+00_JPRB, 0.70214E+00_JPRB ,&
&  0.76281E+00_JPRB, 0.71185E+00_JPRB, 0.72748E+00_JPRB, 0.86061E+00_JPRB, 0.64091E+00_JPRB, 0.46533E+00_JPRB, 0.44983E+00_JPRB ,&
&  0.44844E+00_JPRB, 0.45281E+00_JPRB, 0.46028E+00_JPRB, 0.48109E+00_JPRB, 0.50124E+00_JPRB, 0.51882E+00_JPRB, 0.53241E+00_JPRB ,&
&  0.54087E+00_JPRB, 0.56121E+00_JPRB, 0.58135E+00_JPRB, 0.59831E+00_JPRB, 0.61175E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.70761E+00_JPRB, 0.72008E+00_JPRB, 0.71391E+00_JPRB, 0.72125E+00_JPRB, 0.72340E+00_JPRB, 0.73510E+00_JPRB, 0.73214E+00_JPRB ,&
&  0.72678E+00_JPRB, 0.73262E+00_JPRB, 0.71305E+00_JPRB, 0.71749E+00_JPRB, 0.71362E+00_JPRB, 0.71762E+00_JPRB, 0.70829E+00_JPRB ,&
&  0.71113E+00_JPRB, 0.72706E+00_JPRB, 0.73605E+00_JPRB, 0.74476E+00_JPRB, 0.74143E+00_JPRB, 0.72425E+00_JPRB, 0.70453E+00_JPRB ,&
&  0.74291E+00_JPRB, 0.84404E+00_JPRB, 0.96029E+00_JPRB, 0.10455E+01_JPRB, 0.12412E+01_JPRB, 0.12323E+01_JPRB, 0.13259E+01_JPRB ,&
&  0.12968E+01_JPRB, 0.11618E+01_JPRB, 0.12632E+01_JPRB, 0.11290E+01_JPRB, 0.11181E+01_JPRB, 0.10917E+01_JPRB, 0.10870E+01_JPRB ,&
&  0.11246E+01_JPRB, 0.11991E+01_JPRB, 0.12875E+01_JPRB, 0.13886E+01_JPRB, 0.14759E+01_JPRB, 0.15134E+01_JPRB, 0.15405E+01_JPRB ,&
&  0.15469E+01_JPRB, 0.16746E+01_JPRB, 0.20254E+01_JPRB, 0.22738E+01_JPRB, 0.25565E+01_JPRB, 0.27035E+01_JPRB, 0.27138E+01_JPRB ,&
&  0.26678E+01_JPRB, 0.26403E+01_JPRB, 0.22475E+01_JPRB, 0.19529E+01_JPRB, 0.18747E+01_JPRB, 0.16678E+01_JPRB, 0.15938E+01_JPRB ,&
&  0.16308E+01_JPRB, 0.15064E+01_JPRB, 0.11670E+01_JPRB, 0.12366E+01_JPRB, 0.13012E+01_JPRB, 0.12774E+01_JPRB, 0.13876E+01_JPRB ,&
&  0.12345E+01_JPRB, 0.11433E+01_JPRB, 0.11062E+01_JPRB, 0.10953E+01_JPRB, 0.10735E+01_JPRB, 0.10071E+01_JPRB, 0.96852E+00_JPRB ,&
&  0.83652E+00_JPRB, 0.75566E+00_JPRB, 0.71187E+00_JPRB, 0.69123E+00_JPRB, 0.71946E+00_JPRB, 0.80368E+00_JPRB, 0.70687E+00_JPRB ,&
&  0.72889E+00_JPRB, 0.65050E+00_JPRB, 0.74785E+00_JPRB, 0.95045E+00_JPRB, 0.55437E+00_JPRB, 0.49481E+00_JPRB, 0.47025E+00_JPRB ,&
&  0.49199E+00_JPRB, 0.52391E+00_JPRB, 0.54833E+00_JPRB, 0.57964E+00_JPRB, 0.59514E+00_JPRB, 0.59597E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.62962E+00_JPRB, 0.65496E+00_JPRB, 0.69022E+00_JPRB, 0.69406E+00_JPRB, 0.69275E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.76441E+00_JPRB, 0.77840E+00_JPRB, 0.77855E+00_JPRB, 0.79325E+00_JPRB, 0.81125E+00_JPRB, 0.81460E+00_JPRB, 0.81514E+00_JPRB ,&
&  0.80468E+00_JPRB, 0.80702E+00_JPRB, 0.80329E+00_JPRB, 0.82069E+00_JPRB, 0.82265E+00_JPRB, 0.82558E+00_JPRB, 0.82838E+00_JPRB ,&
&  0.84187E+00_JPRB, 0.85344E+00_JPRB, 0.88340E+00_JPRB, 0.88889E+00_JPRB, 0.88564E+00_JPRB, 0.89277E+00_JPRB, 0.91229E+00_JPRB ,&
&  0.93615E+00_JPRB, 0.96476E+00_JPRB, 0.10017E+01_JPRB, 0.10338E+01_JPRB, 0.11404E+01_JPRB, 0.12953E+01_JPRB, 0.15664E+01_JPRB ,&
&  0.13217E+01_JPRB, 0.12167E+01_JPRB, 0.11734E+01_JPRB, 0.10151E+01_JPRB, 0.10010E+01_JPRB, 0.10179E+01_JPRB, 0.97576E+00_JPRB ,&
&  0.93116E+00_JPRB, 0.91258E+00_JPRB, 0.89572E+00_JPRB, 0.89335E+00_JPRB, 0.90359E+00_JPRB, 0.89511E+00_JPRB, 0.88836E+00_JPRB ,&
&  0.85639E+00_JPRB, 0.87269E+00_JPRB, 0.10750E+01_JPRB, 0.13422E+01_JPRB, 0.15655E+01_JPRB, 0.17902E+01_JPRB, 0.16283E+01_JPRB ,&
&  0.15664E+01_JPRB, 0.16282E+01_JPRB, 0.14438E+01_JPRB, 0.13183E+01_JPRB, 0.11950E+01_JPRB, 0.10726E+01_JPRB, 0.10092E+01_JPRB ,&
&  0.10949E+01_JPRB, 0.11640E+01_JPRB, 0.10646E+01_JPRB, 0.12338E+01_JPRB, 0.12526E+01_JPRB, 0.12846E+01_JPRB, 0.11399E+01_JPRB ,&
&  0.10035E+01_JPRB, 0.92409E+00_JPRB, 0.91287E+00_JPRB, 0.89541E+00_JPRB, 0.90140E+00_JPRB, 0.86605E+00_JPRB, 0.80501E+00_JPRB ,&
&  0.78786E+00_JPRB, 0.73597E+00_JPRB, 0.71924E+00_JPRB, 0.67076E+00_JPRB, 0.76965E+00_JPRB, 0.68653E+00_JPRB, 0.88727E+00_JPRB ,&
&  0.69226E+00_JPRB, 0.62040E+00_JPRB, 0.54510E+00_JPRB, 0.74696E+00_JPRB, 0.57016E+00_JPRB, 0.49361E+00_JPRB, 0.52561E+00_JPRB ,&
&  0.59803E+00_JPRB, 0.63715E+00_JPRB, 0.65265E+00_JPRB, 0.67895E+00_JPRB, 0.68533E+00_JPRB, 0.67708E+00_JPRB, 0.68300E+00_JPRB ,&
&  0.71801E+00_JPRB, 0.72404E+00_JPRB, 0.72977E+00_JPRB, 0.73078E+00_JPRB, 0.74705E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.78789E+00_JPRB, 0.81002E+00_JPRB, 0.82301E+00_JPRB, 0.83602E+00_JPRB, 0.85300E+00_JPRB, 0.86034E+00_JPRB, 0.86596E+00_JPRB ,&
&  0.86730E+00_JPRB, 0.86397E+00_JPRB, 0.87080E+00_JPRB, 0.88468E+00_JPRB, 0.89244E+00_JPRB, 0.89109E+00_JPRB, 0.89747E+00_JPRB ,&
&  0.91152E+00_JPRB, 0.93437E+00_JPRB, 0.95768E+00_JPRB, 0.95122E+00_JPRB, 0.94630E+00_JPRB, 0.95870E+00_JPRB, 0.97611E+00_JPRB ,&
&  0.97700E+00_JPRB, 0.97375E+00_JPRB, 0.98075E+00_JPRB, 0.98970E+00_JPRB, 0.10072E+01_JPRB, 0.10088E+01_JPRB, 0.11009E+01_JPRB ,&
&  0.97229E+00_JPRB, 0.13288E+01_JPRB, 0.12845E+01_JPRB, 0.11128E+01_JPRB, 0.10950E+01_JPRB, 0.10226E+01_JPRB, 0.10127E+01_JPRB ,&
&  0.10116E+01_JPRB, 0.98805E+00_JPRB, 0.96455E+00_JPRB, 0.94496E+00_JPRB, 0.95061E+00_JPRB, 0.96587E+00_JPRB, 0.94559E+00_JPRB ,&
&  0.95724E+00_JPRB, 0.98265E+00_JPRB, 0.10476E+01_JPRB, 0.11357E+01_JPRB, 0.12121E+01_JPRB, 0.12259E+01_JPRB, 0.11419E+01_JPRB ,&
&  0.10832E+01_JPRB, 0.99339E+00_JPRB, 0.80175E+00_JPRB, 0.78179E+00_JPRB, 0.75793E+00_JPRB, 0.63260E+00_JPRB, 0.57831E+00_JPRB ,&
&  0.75353E+00_JPRB, 0.86507E+00_JPRB, 0.11075E+01_JPRB, 0.11662E+01_JPRB, 0.11296E+01_JPRB, 0.11295E+01_JPRB, 0.10086E+01_JPRB ,&
&  0.90395E+00_JPRB, 0.82277E+00_JPRB, 0.80766E+00_JPRB, 0.77271E+00_JPRB, 0.77456E+00_JPRB, 0.76384E+00_JPRB, 0.71827E+00_JPRB ,&
&  0.67992E+00_JPRB, 0.65452E+00_JPRB, 0.64377E+00_JPRB, 0.65346E+00_JPRB, 0.59554E+00_JPRB, 0.82753E+00_JPRB, 0.12620E+01_JPRB ,&
&  0.73410E+00_JPRB, 0.56120E+00_JPRB, 0.44763E+00_JPRB, 0.56862E+00_JPRB, 0.55075E+00_JPRB, 0.54207E+00_JPRB, 0.60242E+00_JPRB ,&
&  0.71526E+00_JPRB, 0.76073E+00_JPRB, 0.76764E+00_JPRB, 0.75783E+00_JPRB, 0.73351E+00_JPRB, 0.71073E+00_JPRB, 0.70073E+00_JPRB ,&
&  0.71483E+00_JPRB, 0.72868E+00_JPRB, 0.74447E+00_JPRB, 0.75256E+00_JPRB, 0.76566E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.80319E+00_JPRB, 0.82388E+00_JPRB, 0.83966E+00_JPRB, 0.85780E+00_JPRB, 0.87660E+00_JPRB, 0.89373E+00_JPRB, 0.90964E+00_JPRB ,&
&  0.92283E+00_JPRB, 0.93381E+00_JPRB, 0.94380E+00_JPRB, 0.95813E+00_JPRB, 0.97723E+00_JPRB, 0.99240E+00_JPRB, 0.10118E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.10654E+01_JPRB, 0.10904E+01_JPRB, 0.10866E+01_JPRB, 0.10858E+01_JPRB, 0.11043E+01_JPRB, 0.11225E+01_JPRB ,&
&  0.11287E+01_JPRB, 0.11423E+01_JPRB, 0.11648E+01_JPRB, 0.11911E+01_JPRB, 0.12228E+01_JPRB, 0.12799E+01_JPRB, 0.12107E+01_JPRB ,&
&  0.12743E+01_JPRB, 0.14010E+01_JPRB, 0.13555E+01_JPRB, 0.12181E+01_JPRB, 0.11563E+01_JPRB, 0.11403E+01_JPRB, 0.11551E+01_JPRB ,&
&  0.11864E+01_JPRB, 0.11760E+01_JPRB, 0.11721E+01_JPRB, 0.11614E+01_JPRB, 0.11646E+01_JPRB, 0.11872E+01_JPRB, 0.12092E+01_JPRB ,&
&  0.12420E+01_JPRB, 0.12820E+01_JPRB, 0.13405E+01_JPRB, 0.14166E+01_JPRB, 0.14993E+01_JPRB, 0.14858E+01_JPRB, 0.14363E+01_JPRB ,&
&  0.13376E+01_JPRB, 0.12396E+01_JPRB, 0.97650E+00_JPRB, 0.89982E+00_JPRB, 0.83639E+00_JPRB, 0.72488E+00_JPRB, 0.63076E+00_JPRB ,&
&  0.76969E+00_JPRB, 0.83083E+00_JPRB, 0.96893E+00_JPRB, 0.11387E+01_JPRB, 0.10673E+01_JPRB, 0.99083E+00_JPRB, 0.89222E+00_JPRB ,&
&  0.81248E+00_JPRB, 0.75341E+00_JPRB, 0.72958E+00_JPRB, 0.69925E+00_JPRB, 0.69461E+00_JPRB, 0.70254E+00_JPRB, 0.65495E+00_JPRB ,&
&  0.63511E+00_JPRB, 0.62972E+00_JPRB, 0.63110E+00_JPRB, 0.63172E+00_JPRB, 0.66866E+00_JPRB, 0.64547E+00_JPRB, 0.87958E+00_JPRB ,&
&  0.70050E+00_JPRB, 0.49725E+00_JPRB, 0.53825E+00_JPRB, 0.60344E+00_JPRB, 0.59587E+00_JPRB, 0.60463E+00_JPRB, 0.68195E+00_JPRB ,&
&  0.83242E+00_JPRB, 0.94901E+00_JPRB, 0.97524E+00_JPRB, 0.91969E+00_JPRB, 0.82605E+00_JPRB, 0.77042E+00_JPRB, 0.74108E+00_JPRB ,&
&  0.74914E+00_JPRB, 0.75705E+00_JPRB, 0.76776E+00_JPRB, 0.77025E+00_JPRB, 0.78086E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.79855E+00_JPRB, 0.82205E+00_JPRB, 0.84110E+00_JPRB, 0.85951E+00_JPRB, 0.87587E+00_JPRB, 0.89094E+00_JPRB, 0.90848E+00_JPRB ,&
&  0.91958E+00_JPRB, 0.93661E+00_JPRB, 0.95780E+00_JPRB, 0.97941E+00_JPRB, 0.99719E+00_JPRB, 0.10230E+01_JPRB, 0.10446E+01_JPRB ,&
&  0.10763E+01_JPRB, 0.10970E+01_JPRB, 0.11158E+01_JPRB, 0.11162E+01_JPRB, 0.11125E+01_JPRB, 0.11199E+01_JPRB, 0.11227E+01_JPRB ,&
&  0.11141E+01_JPRB, 0.11135E+01_JPRB, 0.11216E+01_JPRB, 0.11406E+01_JPRB, 0.11472E+01_JPRB, 0.11882E+01_JPRB, 0.10341E+01_JPRB ,&
&  0.13546E+01_JPRB, 0.15219E+01_JPRB, 0.14569E+01_JPRB, 0.13274E+01_JPRB, 0.12552E+01_JPRB, 0.12757E+01_JPRB, 0.13087E+01_JPRB ,&
&  0.13564E+01_JPRB, 0.14287E+01_JPRB, 0.13873E+01_JPRB, 0.13710E+01_JPRB, 0.13836E+01_JPRB, 0.14197E+01_JPRB, 0.14821E+01_JPRB ,&
&  0.15492E+01_JPRB, 0.16238E+01_JPRB, 0.17048E+01_JPRB, 0.17834E+01_JPRB, 0.18508E+01_JPRB, 0.18905E+01_JPRB, 0.18812E+01_JPRB ,&
&  0.18217E+01_JPRB, 0.17452E+01_JPRB, 0.15725E+01_JPRB, 0.15350E+01_JPRB, 0.14498E+01_JPRB, 0.12009E+01_JPRB, 0.95861E+00_JPRB ,&
&  0.90548E+00_JPRB, 0.94263E+00_JPRB, 0.96361E+00_JPRB, 0.10727E+01_JPRB, 0.10197E+01_JPRB, 0.93625E+00_JPRB, 0.85067E+00_JPRB ,&
&  0.77795E+00_JPRB, 0.71819E+00_JPRB, 0.70218E+00_JPRB, 0.66465E+00_JPRB, 0.63975E+00_JPRB, 0.62277E+00_JPRB, 0.60601E+00_JPRB ,&
&  0.61345E+00_JPRB, 0.62346E+00_JPRB, 0.63918E+00_JPRB, 0.64819E+00_JPRB, 0.64704E+00_JPRB, 0.59344E+00_JPRB, 0.77920E+00_JPRB ,&
&  0.68812E+00_JPRB, 0.60262E+00_JPRB, 0.71090E+00_JPRB, 0.52992E+00_JPRB, 0.63758E+00_JPRB, 0.70394E+00_JPRB, 0.82942E+00_JPRB ,&
&  0.91650E+00_JPRB, 0.12897E+01_JPRB, 0.17795E+01_JPRB, 0.14193E+01_JPRB, 0.11372E+01_JPRB, 0.85614E+00_JPRB, 0.76306E+00_JPRB ,&
&  0.75923E+00_JPRB, 0.76052E+00_JPRB, 0.76729E+00_JPRB, 0.76825E+00_JPRB, 0.77741E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.78993E+00_JPRB, 0.80767E+00_JPRB, 0.82798E+00_JPRB, 0.84610E+00_JPRB, 0.85662E+00_JPRB, 0.86566E+00_JPRB, 0.87777E+00_JPRB ,&
&  0.88285E+00_JPRB, 0.88194E+00_JPRB, 0.89085E+00_JPRB, 0.90672E+00_JPRB, 0.92722E+00_JPRB, 0.94898E+00_JPRB, 0.97664E+00_JPRB ,&
&  0.10139E+01_JPRB, 0.10472E+01_JPRB, 0.10898E+01_JPRB, 0.11169E+01_JPRB, 0.11309E+01_JPRB, 0.11441E+01_JPRB, 0.11536E+01_JPRB ,&
&  0.11769E+01_JPRB, 0.11903E+01_JPRB, 0.12063E+01_JPRB, 0.12477E+01_JPRB, 0.12745E+01_JPRB, 0.13021E+01_JPRB, 0.10901E+01_JPRB ,&
&  0.14430E+01_JPRB, 0.17005E+01_JPRB, 0.15901E+01_JPRB, 0.14224E+01_JPRB, 0.13334E+01_JPRB, 0.13382E+01_JPRB, 0.13504E+01_JPRB ,&
&  0.13821E+01_JPRB, 0.14018E+01_JPRB, 0.13606E+01_JPRB, 0.14640E+01_JPRB, 0.15437E+01_JPRB, 0.15778E+01_JPRB, 0.16777E+01_JPRB ,&
&  0.17801E+01_JPRB, 0.18747E+01_JPRB, 0.19693E+01_JPRB, 0.20596E+01_JPRB, 0.21314E+01_JPRB, 0.22122E+01_JPRB, 0.22248E+01_JPRB ,&
&  0.22211E+01_JPRB, 0.22191E+01_JPRB, 0.22063E+01_JPRB, 0.20018E+01_JPRB, 0.19138E+01_JPRB, 0.17981E+01_JPRB, 0.14578E+01_JPRB ,&
&  0.11822E+01_JPRB, 0.98314E+00_JPRB, 0.92327E+00_JPRB, 0.96952E+00_JPRB, 0.92129E+00_JPRB, 0.83154E+00_JPRB, 0.76446E+00_JPRB ,&
&  0.70685E+00_JPRB, 0.66726E+00_JPRB, 0.67061E+00_JPRB, 0.64162E+00_JPRB, 0.57580E+00_JPRB, 0.58825E+00_JPRB, 0.60834E+00_JPRB ,&
&  0.59460E+00_JPRB, 0.60655E+00_JPRB, 0.62717E+00_JPRB, 0.62363E+00_JPRB, 0.63784E+00_JPRB, 0.67732E+00_JPRB, 0.67769E+00_JPRB ,&
&  0.73713E+00_JPRB, 0.65962E+00_JPRB, 0.67049E+00_JPRB, 0.72781E+00_JPRB, 0.72398E+00_JPRB, 0.94124E+00_JPRB, 0.12866E+01_JPRB ,&
&  0.18674E+01_JPRB, 0.20339E+01_JPRB, 0.26567E+01_JPRB, 0.41483E+01_JPRB, 0.31513E+01_JPRB, 0.20900E+01_JPRB, 0.79699E+00_JPRB ,&
&  0.79626E+00_JPRB, 0.78757E+00_JPRB, 0.80001E+00_JPRB, 0.79417E+00_JPRB, 0.79382E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.69400E+00_JPRB, 0.69074E+00_JPRB, 0.71437E+00_JPRB, 0.73847E+00_JPRB, 0.75988E+00_JPRB, 0.77983E+00_JPRB, 0.79424E+00_JPRB ,&
&  0.81565E+00_JPRB, 0.83166E+00_JPRB, 0.83301E+00_JPRB, 0.82670E+00_JPRB, 0.83705E+00_JPRB, 0.83552E+00_JPRB, 0.85461E+00_JPRB ,&
&  0.88490E+00_JPRB, 0.91986E+00_JPRB, 0.96587E+00_JPRB, 0.99241E+00_JPRB, 0.10158E+01_JPRB, 0.10574E+01_JPRB, 0.10944E+01_JPRB ,&
&  0.11356E+01_JPRB, 0.11874E+01_JPRB, 0.12367E+01_JPRB, 0.12936E+01_JPRB, 0.13436E+01_JPRB, 0.13293E+01_JPRB, 0.10402E+01_JPRB ,&
&  0.14896E+01_JPRB, 0.17385E+01_JPRB, 0.16785E+01_JPRB, 0.14933E+01_JPRB, 0.13754E+01_JPRB, 0.13334E+01_JPRB, 0.13342E+01_JPRB ,&
&  0.13379E+01_JPRB, 0.13129E+01_JPRB, 0.12944E+01_JPRB, 0.13230E+01_JPRB, 0.15532E+01_JPRB, 0.15774E+01_JPRB, 0.16866E+01_JPRB ,&
&  0.17980E+01_JPRB, 0.18970E+01_JPRB, 0.19830E+01_JPRB, 0.20566E+01_JPRB, 0.21107E+01_JPRB, 0.21795E+01_JPRB, 0.22522E+01_JPRB ,&
&  0.23170E+01_JPRB, 0.23555E+01_JPRB, 0.24311E+01_JPRB, 0.22147E+01_JPRB, 0.21176E+01_JPRB, 0.22688E+01_JPRB, 0.20352E+01_JPRB ,&
&  0.13695E+01_JPRB, 0.99546E+00_JPRB, 0.99602E+00_JPRB, 0.96221E+00_JPRB, 0.87196E+00_JPRB, 0.77186E+00_JPRB, 0.71547E+00_JPRB ,&
&  0.67695E+00_JPRB, 0.64561E+00_JPRB, 0.62971E+00_JPRB, 0.60101E+00_JPRB, 0.59486E+00_JPRB, 0.60737E+00_JPRB, 0.60910E+00_JPRB ,&
&  0.62704E+00_JPRB, 0.60235E+00_JPRB, 0.59311E+00_JPRB, 0.59597E+00_JPRB, 0.63004E+00_JPRB, 0.69987E+00_JPRB, 0.85331E+00_JPRB ,&
&  0.98410E+00_JPRB, 0.62439E+00_JPRB, 0.67419E+00_JPRB, 0.72390E+00_JPRB, 0.79529E+00_JPRB, 0.92180E+00_JPRB, 0.11208E+01_JPRB ,&
&  0.14573E+01_JPRB, 0.19235E+01_JPRB, 0.18698E+01_JPRB, 0.14896E+01_JPRB, 0.18923E+01_JPRB, 0.15864E+01_JPRB, 0.70438E+00_JPRB ,&
&  0.70796E+00_JPRB, 0.68852E+00_JPRB, 0.69350E+00_JPRB, 0.69918E+00_JPRB, 0.70044E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.57059E+00_JPRB, 0.55799E+00_JPRB, 0.56521E+00_JPRB, 0.58229E+00_JPRB, 0.59599E+00_JPRB, 0.62385E+00_JPRB, 0.65866E+00_JPRB ,&
&  0.70310E+00_JPRB, 0.73068E+00_JPRB, 0.73950E+00_JPRB, 0.73957E+00_JPRB, 0.74591E+00_JPRB, 0.74601E+00_JPRB, 0.75664E+00_JPRB ,&
&  0.77877E+00_JPRB, 0.80166E+00_JPRB, 0.83141E+00_JPRB, 0.85671E+00_JPRB, 0.87409E+00_JPRB, 0.91880E+00_JPRB, 0.96816E+00_JPRB ,&
&  0.10297E+01_JPRB, 0.10942E+01_JPRB, 0.11741E+01_JPRB, 0.12618E+01_JPRB, 0.13506E+01_JPRB, 0.13806E+01_JPRB, 0.13751E+01_JPRB ,&
&  0.10916E+01_JPRB, 0.15865E+01_JPRB, 0.16364E+01_JPRB, 0.15099E+01_JPRB, 0.13851E+01_JPRB, 0.13452E+01_JPRB, 0.13071E+01_JPRB ,&
&  0.12718E+01_JPRB, 0.11895E+01_JPRB, 0.12076E+01_JPRB, 0.12776E+01_JPRB, 0.14153E+01_JPRB, 0.14807E+01_JPRB, 0.15801E+01_JPRB ,&
&  0.16757E+01_JPRB, 0.17516E+01_JPRB, 0.18170E+01_JPRB, 0.18812E+01_JPRB, 0.19669E+01_JPRB, 0.20856E+01_JPRB, 0.22415E+01_JPRB ,&
&  0.23714E+01_JPRB, 0.24739E+01_JPRB, 0.25495E+01_JPRB, 0.23918E+01_JPRB, 0.22316E+01_JPRB, 0.24635E+01_JPRB, 0.23571E+01_JPRB ,&
&  0.15239E+01_JPRB, 0.10424E+01_JPRB, 0.10709E+01_JPRB, 0.10799E+01_JPRB, 0.93426E+00_JPRB, 0.83163E+00_JPRB, 0.75555E+00_JPRB ,&
&  0.67445E+00_JPRB, 0.61724E+00_JPRB, 0.62035E+00_JPRB, 0.61887E+00_JPRB, 0.62443E+00_JPRB, 0.62568E+00_JPRB, 0.63676E+00_JPRB ,&
&  0.65152E+00_JPRB, 0.62787E+00_JPRB, 0.60952E+00_JPRB, 0.59774E+00_JPRB, 0.60675E+00_JPRB, 0.62454E+00_JPRB, 0.64706E+00_JPRB ,&
&  0.70248E+00_JPRB, 0.66396E+00_JPRB, 0.67631E+00_JPRB, 0.72226E+00_JPRB, 0.76839E+00_JPRB, 0.82199E+00_JPRB, 0.87527E+00_JPRB ,&
&  0.91953E+00_JPRB, 0.96528E+00_JPRB, 0.10084E+01_JPRB, 0.10144E+01_JPRB, 0.94203E+00_JPRB, 0.86089E+00_JPRB, 0.62321E+00_JPRB ,&
&  0.60947E+00_JPRB, 0.59605E+00_JPRB, 0.59761E+00_JPRB, 0.59482E+00_JPRB, 0.59180E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.50163E+00_JPRB, 0.49350E+00_JPRB, 0.49784E+00_JPRB, 0.50617E+00_JPRB, 0.52140E+00_JPRB, 0.54679E+00_JPRB, 0.58552E+00_JPRB ,&
&  0.62974E+00_JPRB, 0.65341E+00_JPRB, 0.67277E+00_JPRB, 0.66170E+00_JPRB, 0.66237E+00_JPRB, 0.67676E+00_JPRB, 0.68437E+00_JPRB ,&
&  0.70901E+00_JPRB, 0.73516E+00_JPRB, 0.74902E+00_JPRB, 0.75085E+00_JPRB, 0.75923E+00_JPRB, 0.78462E+00_JPRB, 0.82084E+00_JPRB ,&
&  0.86419E+00_JPRB, 0.92192E+00_JPRB, 0.10010E+01_JPRB, 0.11089E+01_JPRB, 0.12275E+01_JPRB, 0.12600E+01_JPRB, 0.12360E+01_JPRB ,&
&  0.64665E+00_JPRB, 0.11438E+01_JPRB, 0.15302E+01_JPRB, 0.15117E+01_JPRB, 0.14390E+01_JPRB, 0.13730E+01_JPRB, 0.13288E+01_JPRB ,&
&  0.12007E+01_JPRB, 0.10712E+01_JPRB, 0.10559E+01_JPRB, 0.12310E+01_JPRB, 0.12474E+01_JPRB, 0.13379E+01_JPRB, 0.14219E+01_JPRB ,&
&  0.15133E+01_JPRB, 0.15900E+01_JPRB, 0.16377E+01_JPRB, 0.16786E+01_JPRB, 0.17523E+01_JPRB, 0.18661E+01_JPRB, 0.20107E+01_JPRB ,&
&  0.21939E+01_JPRB, 0.24187E+01_JPRB, 0.25833E+01_JPRB, 0.24614E+01_JPRB, 0.24062E+01_JPRB, 0.25678E+01_JPRB, 0.31406E+01_JPRB ,&
&  0.19956E+01_JPRB, 0.12169E+01_JPRB, 0.12125E+01_JPRB, 0.12360E+01_JPRB, 0.10405E+01_JPRB, 0.92719E+00_JPRB, 0.84265E+00_JPRB ,&
&  0.73834E+00_JPRB, 0.68517E+00_JPRB, 0.66287E+00_JPRB, 0.64718E+00_JPRB, 0.66002E+00_JPRB, 0.68041E+00_JPRB, 0.71483E+00_JPRB ,&
&  0.72599E+00_JPRB, 0.71330E+00_JPRB, 0.69154E+00_JPRB, 0.67931E+00_JPRB, 0.67501E+00_JPRB, 0.67844E+00_JPRB, 0.67307E+00_JPRB ,&
&  0.68843E+00_JPRB, 0.68862E+00_JPRB, 0.70440E+00_JPRB, 0.74132E+00_JPRB, 0.78920E+00_JPRB, 0.80507E+00_JPRB, 0.80626E+00_JPRB ,&
&  0.77369E+00_JPRB, 0.78438E+00_JPRB, 0.79471E+00_JPRB, 0.77946E+00_JPRB, 0.76008E+00_JPRB, 0.71079E+00_JPRB, 0.62002E+00_JPRB ,&
&  0.57714E+00_JPRB, 0.53327E+00_JPRB, 0.51262E+00_JPRB, 0.50366E+00_JPRB, 0.51355E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.48068E+00_JPRB, 0.47320E+00_JPRB, 0.47138E+00_JPRB, 0.48735E+00_JPRB, 0.48396E+00_JPRB, 0.49647E+00_JPRB, 0.51300E+00_JPRB ,&
&  0.55693E+00_JPRB, 0.59687E+00_JPRB, 0.63090E+00_JPRB, 0.62508E+00_JPRB, 0.62364E+00_JPRB, 0.61020E+00_JPRB, 0.61359E+00_JPRB ,&
&  0.62375E+00_JPRB, 0.64250E+00_JPRB, 0.66308E+00_JPRB, 0.67063E+00_JPRB, 0.67539E+00_JPRB, 0.68177E+00_JPRB, 0.69455E+00_JPRB ,&
&  0.71591E+00_JPRB, 0.74681E+00_JPRB, 0.80054E+00_JPRB, 0.89732E+00_JPRB, 0.10468E+01_JPRB, 0.11903E+01_JPRB, 0.12863E+01_JPRB ,&
&  0.88736E+00_JPRB, 0.42722E+00_JPRB, 0.11118E+01_JPRB, 0.14736E+01_JPRB, 0.14475E+01_JPRB, 0.13686E+01_JPRB, 0.13045E+01_JPRB ,&
&  0.11383E+01_JPRB, 0.10588E+01_JPRB, 0.95977E+00_JPRB, 0.10595E+01_JPRB, 0.10226E+01_JPRB, 0.10862E+01_JPRB, 0.11862E+01_JPRB ,&
&  0.12935E+01_JPRB, 0.13916E+01_JPRB, 0.14813E+01_JPRB, 0.15186E+01_JPRB, 0.15341E+01_JPRB, 0.15797E+01_JPRB, 0.16775E+01_JPRB ,&
&  0.18554E+01_JPRB, 0.21447E+01_JPRB, 0.25109E+01_JPRB, 0.26525E+01_JPRB, 0.30603E+01_JPRB, 0.29033E+01_JPRB, 0.31256E+01_JPRB ,&
&  0.20900E+01_JPRB, 0.14551E+01_JPRB, 0.13723E+01_JPRB, 0.13995E+01_JPRB, 0.11813E+01_JPRB, 0.97556E+00_JPRB, 0.96466E+00_JPRB ,&
&  0.81620E+00_JPRB, 0.74975E+00_JPRB, 0.73376E+00_JPRB, 0.71124E+00_JPRB, 0.69273E+00_JPRB, 0.71524E+00_JPRB, 0.75222E+00_JPRB ,&
&  0.78302E+00_JPRB, 0.78850E+00_JPRB, 0.77234E+00_JPRB, 0.75739E+00_JPRB, 0.74816E+00_JPRB, 0.73290E+00_JPRB, 0.72083E+00_JPRB ,&
&  0.73285E+00_JPRB, 0.74827E+00_JPRB, 0.76921E+00_JPRB, 0.79098E+00_JPRB, 0.81628E+00_JPRB, 0.80657E+00_JPRB, 0.79726E+00_JPRB ,&
&  0.78940E+00_JPRB, 0.79628E+00_JPRB, 0.77758E+00_JPRB, 0.77302E+00_JPRB, 0.76212E+00_JPRB, 0.72811E+00_JPRB, 0.70137E+00_JPRB ,&
&  0.64966E+00_JPRB, 0.56744E+00_JPRB, 0.51435E+00_JPRB, 0.50006E+00_JPRB, 0.48640E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.50044E+00_JPRB, 0.48976E+00_JPRB, 0.48634E+00_JPRB, 0.49466E+00_JPRB, 0.50151E+00_JPRB, 0.50874E+00_JPRB, 0.49531E+00_JPRB ,&
&  0.50864E+00_JPRB, 0.54236E+00_JPRB, 0.56140E+00_JPRB, 0.57934E+00_JPRB, 0.57428E+00_JPRB, 0.56308E+00_JPRB, 0.55704E+00_JPRB ,&
&  0.55443E+00_JPRB, 0.55770E+00_JPRB, 0.57073E+00_JPRB, 0.57712E+00_JPRB, 0.58074E+00_JPRB, 0.58669E+00_JPRB, 0.59037E+00_JPRB ,&
&  0.59591E+00_JPRB, 0.60887E+00_JPRB, 0.63568E+00_JPRB, 0.68682E+00_JPRB, 0.77336E+00_JPRB, 0.91436E+00_JPRB, 0.11047E+01_JPRB ,&
&  0.14140E+01_JPRB, 0.10184E+01_JPRB, 0.52901E+00_JPRB, 0.11616E+01_JPRB, 0.14843E+01_JPRB, 0.14694E+01_JPRB, 0.12595E+01_JPRB ,&
&  0.11722E+01_JPRB, 0.10340E+01_JPRB, 0.95725E+00_JPRB, 0.10034E+01_JPRB, 0.87874E+00_JPRB, 0.84852E+00_JPRB, 0.88159E+00_JPRB ,&
&  0.96166E+00_JPRB, 0.10845E+01_JPRB, 0.12160E+01_JPRB, 0.13084E+01_JPRB, 0.13586E+01_JPRB, 0.13806E+01_JPRB, 0.14185E+01_JPRB ,&
&  0.15358E+01_JPRB, 0.17174E+01_JPRB, 0.20407E+01_JPRB, 0.25866E+01_JPRB, 0.37141E+01_JPRB, 0.34877E+01_JPRB, 0.34684E+01_JPRB ,&
&  0.22153E+01_JPRB, 0.18872E+01_JPRB, 0.17593E+01_JPRB, 0.16126E+01_JPRB, 0.13087E+01_JPRB, 0.11349E+01_JPRB, 0.11400E+01_JPRB ,&
&  0.94386E+00_JPRB, 0.82414E+00_JPRB, 0.78449E+00_JPRB, 0.74872E+00_JPRB, 0.72749E+00_JPRB, 0.73511E+00_JPRB, 0.76275E+00_JPRB ,&
&  0.79647E+00_JPRB, 0.79905E+00_JPRB, 0.78749E+00_JPRB, 0.77446E+00_JPRB, 0.76699E+00_JPRB, 0.75992E+00_JPRB, 0.75172E+00_JPRB ,&
&  0.75395E+00_JPRB, 0.77104E+00_JPRB, 0.79615E+00_JPRB, 0.79630E+00_JPRB, 0.77880E+00_JPRB, 0.77302E+00_JPRB, 0.80235E+00_JPRB ,&
&  0.79078E+00_JPRB, 0.80702E+00_JPRB, 0.79036E+00_JPRB, 0.79191E+00_JPRB, 0.76888E+00_JPRB, 0.75702E+00_JPRB, 0.74751E+00_JPRB ,&
&  0.72430E+00_JPRB, 0.65956E+00_JPRB, 0.58674E+00_JPRB, 0.51048E+00_JPRB, 0.50008E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.54984E+00_JPRB, 0.53348E+00_JPRB, 0.53489E+00_JPRB, 0.52877E+00_JPRB, 0.51843E+00_JPRB, 0.51541E+00_JPRB, 0.50909E+00_JPRB ,&
&  0.50132E+00_JPRB, 0.49837E+00_JPRB, 0.49482E+00_JPRB, 0.52202E+00_JPRB, 0.53455E+00_JPRB, 0.52502E+00_JPRB, 0.51548E+00_JPRB ,&
&  0.50884E+00_JPRB, 0.50919E+00_JPRB, 0.51271E+00_JPRB, 0.51575E+00_JPRB, 0.51692E+00_JPRB, 0.52167E+00_JPRB, 0.52606E+00_JPRB ,&
&  0.52275E+00_JPRB, 0.52377E+00_JPRB, 0.53275E+00_JPRB, 0.55379E+00_JPRB, 0.59474E+00_JPRB, 0.66293E+00_JPRB, 0.77934E+00_JPRB ,&
&  0.10347E+01_JPRB, 0.10507E+01_JPRB, 0.52969E+00_JPRB, 0.94546E+00_JPRB, 0.14979E+01_JPRB, 0.15412E+01_JPRB, 0.14427E+01_JPRB ,&
&  0.12934E+01_JPRB, 0.10325E+01_JPRB, 0.99312E+00_JPRB, 0.10159E+01_JPRB, 0.89045E+00_JPRB, 0.79708E+00_JPRB, 0.74292E+00_JPRB ,&
&  0.72494E+00_JPRB, 0.76337E+00_JPRB, 0.86359E+00_JPRB, 0.97939E+00_JPRB, 0.10848E+01_JPRB, 0.11501E+01_JPRB, 0.11842E+01_JPRB ,&
&  0.12330E+01_JPRB, 0.13329E+01_JPRB, 0.15412E+01_JPRB, 0.18141E+01_JPRB, 0.30279E+01_JPRB, 0.36984E+01_JPRB, 0.38986E+01_JPRB ,&
&  0.29043E+01_JPRB, 0.25379E+01_JPRB, 0.22199E+01_JPRB, 0.18935E+01_JPRB, 0.15032E+01_JPRB, 0.13724E+01_JPRB, 0.13590E+01_JPRB ,&
&  0.11346E+01_JPRB, 0.95104E+00_JPRB, 0.87343E+00_JPRB, 0.81624E+00_JPRB, 0.77271E+00_JPRB, 0.75400E+00_JPRB, 0.74769E+00_JPRB ,&
&  0.76195E+00_JPRB, 0.77139E+00_JPRB, 0.76331E+00_JPRB, 0.75366E+00_JPRB, 0.75414E+00_JPRB, 0.75495E+00_JPRB, 0.75754E+00_JPRB ,&
&  0.75451E+00_JPRB, 0.76992E+00_JPRB, 0.79372E+00_JPRB, 0.79043E+00_JPRB, 0.79902E+00_JPRB, 0.81772E+00_JPRB, 0.79444E+00_JPRB ,&
&  0.76986E+00_JPRB, 0.78109E+00_JPRB, 0.77007E+00_JPRB, 0.74640E+00_JPRB, 0.78311E+00_JPRB, 0.78062E+00_JPRB, 0.78216E+00_JPRB ,&
&  0.83358E+00_JPRB, 0.95514E+00_JPRB, 0.92145E+00_JPRB, 0.61647E+00_JPRB, 0.56519E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.57959E+00_JPRB, 0.55504E+00_JPRB, 0.55045E+00_JPRB, 0.54613E+00_JPRB, 0.53473E+00_JPRB, 0.52634E+00_JPRB, 0.52079E+00_JPRB ,&
&  0.51066E+00_JPRB, 0.49200E+00_JPRB, 0.47611E+00_JPRB, 0.47869E+00_JPRB, 0.49122E+00_JPRB, 0.50024E+00_JPRB, 0.49157E+00_JPRB ,&
&  0.48109E+00_JPRB, 0.47685E+00_JPRB, 0.48379E+00_JPRB, 0.48739E+00_JPRB, 0.48541E+00_JPRB, 0.48737E+00_JPRB, 0.48436E+00_JPRB ,&
&  0.48227E+00_JPRB, 0.47830E+00_JPRB, 0.47981E+00_JPRB, 0.49077E+00_JPRB, 0.51015E+00_JPRB, 0.54413E+00_JPRB, 0.59438E+00_JPRB ,&
&  0.75275E+00_JPRB, 0.87673E+00_JPRB, 0.10689E+01_JPRB, 0.16555E+01_JPRB, 0.19957E+01_JPRB, 0.18508E+01_JPRB, 0.16717E+01_JPRB ,&
&  0.15088E+01_JPRB, 0.12726E+01_JPRB, 0.11923E+01_JPRB, 0.10787E+01_JPRB, 0.92867E+00_JPRB, 0.81505E+00_JPRB, 0.73987E+00_JPRB ,&
&  0.69755E+00_JPRB, 0.67347E+00_JPRB, 0.68074E+00_JPRB, 0.72173E+00_JPRB, 0.78672E+00_JPRB, 0.86057E+00_JPRB, 0.91323E+00_JPRB ,&
&  0.95783E+00_JPRB, 0.10297E+01_JPRB, 0.11705E+01_JPRB, 0.13412E+01_JPRB, 0.20222E+01_JPRB, 0.31707E+01_JPRB, 0.37714E+01_JPRB ,&
&  0.37738E+01_JPRB, 0.33781E+01_JPRB, 0.28035E+01_JPRB, 0.22716E+01_JPRB, 0.17013E+01_JPRB, 0.17268E+01_JPRB, 0.15426E+01_JPRB ,&
&  0.13198E+01_JPRB, 0.11417E+01_JPRB, 0.10139E+01_JPRB, 0.92014E+00_JPRB, 0.84318E+00_JPRB, 0.78170E+00_JPRB, 0.74869E+00_JPRB ,&
&  0.73910E+00_JPRB, 0.74177E+00_JPRB, 0.73386E+00_JPRB, 0.72006E+00_JPRB, 0.72699E+00_JPRB, 0.73409E+00_JPRB, 0.74268E+00_JPRB ,&
&  0.74485E+00_JPRB, 0.75408E+00_JPRB, 0.74288E+00_JPRB, 0.76246E+00_JPRB, 0.78886E+00_JPRB, 0.78813E+00_JPRB, 0.76445E+00_JPRB ,&
&  0.73112E+00_JPRB, 0.74238E+00_JPRB, 0.74895E+00_JPRB, 0.74478E+00_JPRB, 0.76360E+00_JPRB, 0.77981E+00_JPRB, 0.77759E+00_JPRB ,&
&  0.79891E+00_JPRB, 0.78156E+00_JPRB, 0.73731E+00_JPRB, 0.64672E+00_JPRB, 0.60945E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.61484E+00_JPRB, 0.58851E+00_JPRB, 0.57064E+00_JPRB, 0.55894E+00_JPRB, 0.55348E+00_JPRB, 0.55344E+00_JPRB, 0.54774E+00_JPRB ,&
&  0.53940E+00_JPRB, 0.52202E+00_JPRB, 0.49916E+00_JPRB, 0.47951E+00_JPRB, 0.48049E+00_JPRB, 0.48020E+00_JPRB, 0.47945E+00_JPRB ,&
&  0.47064E+00_JPRB, 0.46670E+00_JPRB, 0.46803E+00_JPRB, 0.46686E+00_JPRB, 0.46365E+00_JPRB, 0.45966E+00_JPRB, 0.45772E+00_JPRB ,&
&  0.46115E+00_JPRB, 0.46136E+00_JPRB, 0.46047E+00_JPRB, 0.46360E+00_JPRB, 0.47279E+00_JPRB, 0.49464E+00_JPRB, 0.52564E+00_JPRB ,&
&  0.61545E+00_JPRB, 0.80322E+00_JPRB, 0.98244E+00_JPRB, 0.18535E+01_JPRB, 0.21431E+01_JPRB, 0.20480E+01_JPRB, 0.17712E+01_JPRB ,&
&  0.15377E+01_JPRB, 0.15127E+01_JPRB, 0.13057E+01_JPRB, 0.11014E+01_JPRB, 0.91561E+00_JPRB, 0.80893E+00_JPRB, 0.73247E+00_JPRB ,&
&  0.67877E+00_JPRB, 0.64913E+00_JPRB, 0.64006E+00_JPRB, 0.64037E+00_JPRB, 0.64459E+00_JPRB, 0.67105E+00_JPRB, 0.70189E+00_JPRB ,&
&  0.73730E+00_JPRB, 0.81715E+00_JPRB, 0.96283E+00_JPRB, 0.10896E+01_JPRB, 0.14962E+01_JPRB, 0.23939E+01_JPRB, 0.31805E+01_JPRB ,&
&  0.35723E+01_JPRB, 0.33944E+01_JPRB, 0.29996E+01_JPRB, 0.24647E+01_JPRB, 0.20091E+01_JPRB, 0.18322E+01_JPRB, 0.16390E+01_JPRB ,&
&  0.14437E+01_JPRB, 0.13125E+01_JPRB, 0.11404E+01_JPRB, 0.10189E+01_JPRB, 0.95508E+00_JPRB, 0.86674E+00_JPRB, 0.79223E+00_JPRB ,&
&  0.74796E+00_JPRB, 0.72717E+00_JPRB, 0.71474E+00_JPRB, 0.70173E+00_JPRB, 0.69919E+00_JPRB, 0.70747E+00_JPRB, 0.71734E+00_JPRB ,&
&  0.72081E+00_JPRB, 0.73144E+00_JPRB, 0.70661E+00_JPRB, 0.70674E+00_JPRB, 0.73301E+00_JPRB, 0.73987E+00_JPRB, 0.72322E+00_JPRB ,&
&  0.70917E+00_JPRB, 0.70998E+00_JPRB, 0.71725E+00_JPRB, 0.71105E+00_JPRB, 0.71338E+00_JPRB, 0.78357E+00_JPRB, 0.78232E+00_JPRB ,&
&  0.78754E+00_JPRB, 0.73201E+00_JPRB, 0.70087E+00_JPRB, 0.66274E+00_JPRB, 0.64040E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.62687E+00_JPRB, 0.61013E+00_JPRB, 0.59608E+00_JPRB, 0.58202E+00_JPRB, 0.57317E+00_JPRB, 0.56871E+00_JPRB, 0.56166E+00_JPRB ,&
&  0.54948E+00_JPRB, 0.53204E+00_JPRB, 0.51358E+00_JPRB, 0.48833E+00_JPRB, 0.47434E+00_JPRB, 0.47259E+00_JPRB, 0.47908E+00_JPRB ,&
&  0.47252E+00_JPRB, 0.45882E+00_JPRB, 0.45860E+00_JPRB, 0.46060E+00_JPRB, 0.45868E+00_JPRB, 0.45162E+00_JPRB, 0.44363E+00_JPRB ,&
&  0.44594E+00_JPRB, 0.44924E+00_JPRB, 0.45185E+00_JPRB, 0.45448E+00_JPRB, 0.45829E+00_JPRB, 0.46848E+00_JPRB, 0.49237E+00_JPRB ,&
&  0.56162E+00_JPRB, 0.91508E+00_JPRB, 0.99982E+00_JPRB, 0.17007E+01_JPRB, 0.18410E+01_JPRB, 0.17978E+01_JPRB, 0.16207E+01_JPRB ,&
&  0.14543E+01_JPRB, 0.13557E+01_JPRB, 0.11657E+01_JPRB, 0.10202E+01_JPRB, 0.87717E+00_JPRB, 0.77043E+00_JPRB, 0.70299E+00_JPRB ,&
&  0.65705E+00_JPRB, 0.62175E+00_JPRB, 0.60090E+00_JPRB, 0.59473E+00_JPRB, 0.59114E+00_JPRB, 0.59392E+00_JPRB, 0.60358E+00_JPRB ,&
&  0.62216E+00_JPRB, 0.67716E+00_JPRB, 0.81818E+00_JPRB, 0.11271E+01_JPRB, 0.13622E+01_JPRB, 0.19915E+01_JPRB, 0.29450E+01_JPRB ,&
&  0.34947E+01_JPRB, 0.33800E+01_JPRB, 0.28360E+01_JPRB, 0.23581E+01_JPRB, 0.20043E+01_JPRB, 0.17649E+01_JPRB, 0.15633E+01_JPRB ,&
&  0.14261E+01_JPRB, 0.13255E+01_JPRB, 0.12214E+01_JPRB, 0.10943E+01_JPRB, 0.10059E+01_JPRB, 0.93928E+00_JPRB, 0.86714E+00_JPRB ,&
&  0.79411E+00_JPRB, 0.73822E+00_JPRB, 0.70722E+00_JPRB, 0.68428E+00_JPRB, 0.67106E+00_JPRB, 0.67499E+00_JPRB, 0.68295E+00_JPRB ,&
&  0.68396E+00_JPRB, 0.69296E+00_JPRB, 0.67310E+00_JPRB, 0.67042E+00_JPRB, 0.69136E+00_JPRB, 0.70133E+00_JPRB, 0.68622E+00_JPRB ,&
&  0.68670E+00_JPRB, 0.69281E+00_JPRB, 0.70272E+00_JPRB, 0.71944E+00_JPRB, 0.70984E+00_JPRB, 0.78679E+00_JPRB, 0.76877E+00_JPRB ,&
&  0.76750E+00_JPRB, 0.72277E+00_JPRB, 0.68571E+00_JPRB, 0.67185E+00_JPRB, 0.65250E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.62738E+00_JPRB, 0.61252E+00_JPRB, 0.60122E+00_JPRB, 0.59492E+00_JPRB, 0.59035E+00_JPRB, 0.57950E+00_JPRB, 0.56808E+00_JPRB ,&
&  0.55690E+00_JPRB, 0.54389E+00_JPRB, 0.52365E+00_JPRB, 0.49461E+00_JPRB, 0.47293E+00_JPRB, 0.46465E+00_JPRB, 0.46863E+00_JPRB ,&
&  0.46967E+00_JPRB, 0.46186E+00_JPRB, 0.45663E+00_JPRB, 0.45595E+00_JPRB, 0.45514E+00_JPRB, 0.44844E+00_JPRB, 0.43968E+00_JPRB ,&
&  0.43278E+00_JPRB, 0.43350E+00_JPRB, 0.43669E+00_JPRB, 0.44168E+00_JPRB, 0.44610E+00_JPRB, 0.45271E+00_JPRB, 0.46920E+00_JPRB ,&
&  0.53010E+00_JPRB, 0.53067E+00_JPRB, 0.11166E+01_JPRB, 0.14507E+01_JPRB, 0.14830E+01_JPRB, 0.14379E+01_JPRB, 0.14132E+01_JPRB ,&
&  0.13159E+01_JPRB, 0.11711E+01_JPRB, 0.10090E+01_JPRB, 0.87314E+00_JPRB, 0.77241E+00_JPRB, 0.70611E+00_JPRB, 0.65692E+00_JPRB ,&
&  0.61927E+00_JPRB, 0.59003E+00_JPRB, 0.56560E+00_JPRB, 0.55353E+00_JPRB, 0.55611E+00_JPRB, 0.56160E+00_JPRB, 0.56692E+00_JPRB ,&
&  0.57685E+00_JPRB, 0.60033E+00_JPRB, 0.67837E+00_JPRB, 0.92138E+00_JPRB, 0.14511E+01_JPRB, 0.19293E+01_JPRB, 0.27270E+01_JPRB ,&
&  0.34525E+01_JPRB, 0.30750E+01_JPRB, 0.24734E+01_JPRB, 0.20613E+01_JPRB, 0.17701E+01_JPRB, 0.15691E+01_JPRB, 0.14178E+01_JPRB ,&
&  0.13029E+01_JPRB, 0.12209E+01_JPRB, 0.11546E+01_JPRB, 0.10786E+01_JPRB, 0.10157E+01_JPRB, 0.96727E+00_JPRB, 0.91459E+00_JPRB ,&
&  0.85429E+00_JPRB, 0.79537E+00_JPRB, 0.74181E+00_JPRB, 0.69572E+00_JPRB, 0.65767E+00_JPRB, 0.64732E+00_JPRB, 0.64693E+00_JPRB ,&
&  0.64458E+00_JPRB, 0.65093E+00_JPRB, 0.64813E+00_JPRB, 0.65462E+00_JPRB, 0.67887E+00_JPRB, 0.68582E+00_JPRB, 0.68237E+00_JPRB ,&
&  0.66538E+00_JPRB, 0.67639E+00_JPRB, 0.68335E+00_JPRB, 0.69137E+00_JPRB, 0.66849E+00_JPRB, 0.75255E+00_JPRB, 0.73920E+00_JPRB ,&
&  0.73446E+00_JPRB, 0.72096E+00_JPRB, 0.68372E+00_JPRB, 0.66816E+00_JPRB, 0.65019E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.62759E+00_JPRB, 0.60814E+00_JPRB, 0.59811E+00_JPRB, 0.59190E+00_JPRB, 0.58765E+00_JPRB, 0.58382E+00_JPRB, 0.57306E+00_JPRB ,&
&  0.55795E+00_JPRB, 0.54907E+00_JPRB, 0.53698E+00_JPRB, 0.52018E+00_JPRB, 0.49592E+00_JPRB, 0.47917E+00_JPRB, 0.47592E+00_JPRB ,&
&  0.47322E+00_JPRB, 0.46790E+00_JPRB, 0.45845E+00_JPRB, 0.45369E+00_JPRB, 0.44773E+00_JPRB, 0.43993E+00_JPRB, 0.43552E+00_JPRB ,&
&  0.42623E+00_JPRB, 0.42212E+00_JPRB, 0.42429E+00_JPRB, 0.43024E+00_JPRB, 0.43584E+00_JPRB, 0.44346E+00_JPRB, 0.45248E+00_JPRB ,&
&  0.49625E+00_JPRB, 0.68585E+00_JPRB, 0.98544E+00_JPRB, 0.11581E+01_JPRB, 0.11882E+01_JPRB, 0.11301E+01_JPRB, 0.11305E+01_JPRB ,&
&  0.10288E+01_JPRB, 0.93667E+00_JPRB, 0.83682E+00_JPRB, 0.74214E+00_JPRB, 0.67090E+00_JPRB, 0.63467E+00_JPRB, 0.61371E+00_JPRB ,&
&  0.58792E+00_JPRB, 0.56510E+00_JPRB, 0.54370E+00_JPRB, 0.52563E+00_JPRB, 0.51940E+00_JPRB, 0.52790E+00_JPRB, 0.54035E+00_JPRB ,&
&  0.54567E+00_JPRB, 0.55520E+00_JPRB, 0.59437E+00_JPRB, 0.74571E+00_JPRB, 0.11971E+01_JPRB, 0.14485E+01_JPRB, 0.21948E+01_JPRB ,&
&  0.25940E+01_JPRB, 0.23022E+01_JPRB, 0.19118E+01_JPRB, 0.16332E+01_JPRB, 0.14146E+01_JPRB, 0.12514E+01_JPRB, 0.11339E+01_JPRB ,&
&  0.10667E+01_JPRB, 0.10198E+01_JPRB, 0.98053E+00_JPRB, 0.94250E+00_JPRB, 0.90754E+00_JPRB, 0.88062E+00_JPRB, 0.85813E+00_JPRB ,&
&  0.83111E+00_JPRB, 0.79764E+00_JPRB, 0.76021E+00_JPRB, 0.71643E+00_JPRB, 0.67172E+00_JPRB, 0.64346E+00_JPRB, 0.62784E+00_JPRB ,&
&  0.61613E+00_JPRB, 0.61877E+00_JPRB, 0.62639E+00_JPRB, 0.63666E+00_JPRB, 0.66127E+00_JPRB, 0.67500E+00_JPRB, 0.67718E+00_JPRB ,&
&  0.65025E+00_JPRB, 0.64366E+00_JPRB, 0.64518E+00_JPRB, 0.63394E+00_JPRB, 0.64569E+00_JPRB, 0.70714E+00_JPRB, 0.68629E+00_JPRB ,&
&  0.67349E+00_JPRB, 0.66949E+00_JPRB, 0.66013E+00_JPRB, 0.64791E+00_JPRB, 0.65250E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.64403E+00_JPRB, 0.61463E+00_JPRB, 0.59808E+00_JPRB, 0.58843E+00_JPRB, 0.58430E+00_JPRB, 0.58061E+00_JPRB, 0.57048E+00_JPRB ,&
&  0.55675E+00_JPRB, 0.54340E+00_JPRB, 0.53261E+00_JPRB, 0.52495E+00_JPRB, 0.50848E+00_JPRB, 0.48193E+00_JPRB, 0.46158E+00_JPRB ,&
&  0.45631E+00_JPRB, 0.45165E+00_JPRB, 0.44797E+00_JPRB, 0.44048E+00_JPRB, 0.43374E+00_JPRB, 0.43104E+00_JPRB, 0.42797E+00_JPRB ,&
&  0.42406E+00_JPRB, 0.42000E+00_JPRB, 0.42113E+00_JPRB, 0.42540E+00_JPRB, 0.42975E+00_JPRB, 0.43384E+00_JPRB, 0.43930E+00_JPRB ,&
&  0.46033E+00_JPRB, 0.56316E+00_JPRB, 0.69658E+00_JPRB, 0.87420E+00_JPRB, 0.92324E+00_JPRB, 0.89013E+00_JPRB, 0.86631E+00_JPRB ,&
&  0.82154E+00_JPRB, 0.76971E+00_JPRB, 0.71820E+00_JPRB, 0.66766E+00_JPRB, 0.61173E+00_JPRB, 0.58134E+00_JPRB, 0.56770E+00_JPRB ,&
&  0.55711E+00_JPRB, 0.54150E+00_JPRB, 0.52386E+00_JPRB, 0.50528E+00_JPRB, 0.49262E+00_JPRB, 0.49567E+00_JPRB, 0.50521E+00_JPRB ,&
&  0.50353E+00_JPRB, 0.50507E+00_JPRB, 0.51479E+00_JPRB, 0.57450E+00_JPRB, 0.80884E+00_JPRB, 0.10382E+01_JPRB, 0.13609E+01_JPRB ,&
&  0.13871E+01_JPRB, 0.14498E+01_JPRB, 0.13234E+01_JPRB, 0.11878E+01_JPRB, 0.10747E+01_JPRB, 0.98781E+00_JPRB, 0.92191E+00_JPRB ,&
&  0.87685E+00_JPRB, 0.84098E+00_JPRB, 0.81454E+00_JPRB, 0.79474E+00_JPRB, 0.77824E+00_JPRB, 0.75868E+00_JPRB, 0.74912E+00_JPRB ,&
&  0.74384E+00_JPRB, 0.72873E+00_JPRB, 0.71074E+00_JPRB, 0.69025E+00_JPRB, 0.66415E+00_JPRB, 0.64152E+00_JPRB, 0.62456E+00_JPRB ,&
&  0.60894E+00_JPRB, 0.59477E+00_JPRB, 0.59644E+00_JPRB, 0.60868E+00_JPRB, 0.61963E+00_JPRB, 0.62802E+00_JPRB, 0.62757E+00_JPRB ,&
&  0.62300E+00_JPRB, 0.62038E+00_JPRB, 0.61270E+00_JPRB, 0.60206E+00_JPRB, 0.60887E+00_JPRB, 0.65110E+00_JPRB, 0.63900E+00_JPRB ,&
&  0.62384E+00_JPRB, 0.61447E+00_JPRB, 0.61494E+00_JPRB, 0.61865E+00_JPRB, 0.67051E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.67847E+00_JPRB, 0.64491E+00_JPRB, 0.61159E+00_JPRB, 0.59142E+00_JPRB, 0.57838E+00_JPRB, 0.56814E+00_JPRB, 0.55475E+00_JPRB ,&
&  0.54221E+00_JPRB, 0.52896E+00_JPRB, 0.51788E+00_JPRB, 0.50986E+00_JPRB, 0.50102E+00_JPRB, 0.48243E+00_JPRB, 0.45713E+00_JPRB ,&
&  0.43964E+00_JPRB, 0.43497E+00_JPRB, 0.43178E+00_JPRB, 0.42552E+00_JPRB, 0.41995E+00_JPRB, 0.41854E+00_JPRB, 0.41654E+00_JPRB ,&
&  0.41696E+00_JPRB, 0.41771E+00_JPRB, 0.42117E+00_JPRB, 0.42318E+00_JPRB, 0.42180E+00_JPRB, 0.42838E+00_JPRB, 0.42831E+00_JPRB ,&
&  0.44715E+00_JPRB, 0.46241E+00_JPRB, 0.54693E+00_JPRB, 0.64204E+00_JPRB, 0.70370E+00_JPRB, 0.73022E+00_JPRB, 0.72596E+00_JPRB ,&
&  0.71171E+00_JPRB, 0.69438E+00_JPRB, 0.66624E+00_JPRB, 0.63481E+00_JPRB, 0.59836E+00_JPRB, 0.55822E+00_JPRB, 0.53397E+00_JPRB ,&
&  0.52746E+00_JPRB, 0.51930E+00_JPRB, 0.50954E+00_JPRB, 0.49653E+00_JPRB, 0.48338E+00_JPRB, 0.47971E+00_JPRB, 0.48148E+00_JPRB ,&
&  0.47802E+00_JPRB, 0.47461E+00_JPRB, 0.47155E+00_JPRB, 0.47714E+00_JPRB, 0.51726E+00_JPRB, 0.62947E+00_JPRB, 0.77773E+00_JPRB ,&
&  0.83519E+00_JPRB, 0.87594E+00_JPRB, 0.89513E+00_JPRB, 0.88608E+00_JPRB, 0.85096E+00_JPRB, 0.80227E+00_JPRB, 0.75092E+00_JPRB ,&
&  0.71314E+00_JPRB, 0.68711E+00_JPRB, 0.67384E+00_JPRB, 0.66870E+00_JPRB, 0.66648E+00_JPRB, 0.66137E+00_JPRB, 0.65157E+00_JPRB ,&
&  0.64402E+00_JPRB, 0.64037E+00_JPRB, 0.64013E+00_JPRB, 0.63565E+00_JPRB, 0.62930E+00_JPRB, 0.62144E+00_JPRB, 0.61574E+00_JPRB ,&
&  0.61234E+00_JPRB, 0.60548E+00_JPRB, 0.59398E+00_JPRB, 0.58558E+00_JPRB, 0.58894E+00_JPRB, 0.59139E+00_JPRB, 0.59051E+00_JPRB ,&
&  0.58821E+00_JPRB, 0.59276E+00_JPRB, 0.58268E+00_JPRB, 0.59199E+00_JPRB, 0.62440E+00_JPRB, 0.61106E+00_JPRB, 0.60363E+00_JPRB ,&
&  0.58629E+00_JPRB, 0.57656E+00_JPRB, 0.57886E+00_JPRB, 0.59497E+00_JPRB, 0.66710E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.57055E+00_JPRB, 0.56646E+00_JPRB, 0.55910E+00_JPRB, 0.55244E+00_JPRB, 0.54224E+00_JPRB, 0.53308E+00_JPRB, 0.52093E+00_JPRB ,&
&  0.50981E+00_JPRB, 0.49875E+00_JPRB, 0.48890E+00_JPRB, 0.48564E+00_JPRB, 0.47903E+00_JPRB, 0.46919E+00_JPRB, 0.45132E+00_JPRB ,&
&  0.43620E+00_JPRB, 0.42434E+00_JPRB, 0.41715E+00_JPRB, 0.41423E+00_JPRB, 0.41252E+00_JPRB, 0.40982E+00_JPRB, 0.40892E+00_JPRB ,&
&  0.41018E+00_JPRB, 0.41030E+00_JPRB, 0.41311E+00_JPRB, 0.41323E+00_JPRB, 0.41681E+00_JPRB, 0.41308E+00_JPRB, 0.41069E+00_JPRB ,&
&  0.42707E+00_JPRB, 0.41867E+00_JPRB, 0.46600E+00_JPRB, 0.51993E+00_JPRB, 0.57006E+00_JPRB, 0.60441E+00_JPRB, 0.63763E+00_JPRB ,&
&  0.66192E+00_JPRB, 0.65535E+00_JPRB, 0.63549E+00_JPRB, 0.60142E+00_JPRB, 0.57472E+00_JPRB, 0.54473E+00_JPRB, 0.51447E+00_JPRB ,&
&  0.49946E+00_JPRB, 0.49254E+00_JPRB, 0.48660E+00_JPRB, 0.48103E+00_JPRB, 0.47470E+00_JPRB, 0.47129E+00_JPRB, 0.47024E+00_JPRB ,&
&  0.46517E+00_JPRB, 0.45869E+00_JPRB, 0.45071E+00_JPRB, 0.44828E+00_JPRB, 0.44981E+00_JPRB, 0.47732E+00_JPRB, 0.54699E+00_JPRB ,&
&  0.58183E+00_JPRB, 0.60128E+00_JPRB, 0.61819E+00_JPRB, 0.64345E+00_JPRB, 0.65936E+00_JPRB, 0.65868E+00_JPRB, 0.63047E+00_JPRB ,&
&  0.60377E+00_JPRB, 0.58404E+00_JPRB, 0.57331E+00_JPRB, 0.56884E+00_JPRB, 0.56948E+00_JPRB, 0.56991E+00_JPRB, 0.57132E+00_JPRB ,&
&  0.56661E+00_JPRB, 0.56130E+00_JPRB, 0.56011E+00_JPRB, 0.56126E+00_JPRB, 0.56437E+00_JPRB, 0.56702E+00_JPRB, 0.57149E+00_JPRB ,&
&  0.58000E+00_JPRB, 0.59067E+00_JPRB, 0.59938E+00_JPRB, 0.59207E+00_JPRB, 0.57747E+00_JPRB, 0.56714E+00_JPRB, 0.56121E+00_JPRB ,&
&  0.55715E+00_JPRB, 0.55986E+00_JPRB, 0.56575E+00_JPRB, 0.58383E+00_JPRB, 0.58901E+00_JPRB, 0.58121E+00_JPRB, 0.56701E+00_JPRB ,&
&  0.55407E+00_JPRB, 0.54588E+00_JPRB, 0.54636E+00_JPRB, 0.55621E+00_JPRB, 0.54651E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.51362E+00_JPRB, 0.51323E+00_JPRB, 0.51014E+00_JPRB, 0.50781E+00_JPRB, 0.50196E+00_JPRB, 0.49529E+00_JPRB, 0.48857E+00_JPRB ,&
&  0.47979E+00_JPRB, 0.47108E+00_JPRB, 0.46344E+00_JPRB, 0.46099E+00_JPRB, 0.45876E+00_JPRB, 0.45620E+00_JPRB, 0.44947E+00_JPRB ,&
&  0.43971E+00_JPRB, 0.43016E+00_JPRB, 0.42158E+00_JPRB, 0.41621E+00_JPRB, 0.41465E+00_JPRB, 0.41254E+00_JPRB, 0.40952E+00_JPRB ,&
&  0.40788E+00_JPRB, 0.40898E+00_JPRB, 0.40870E+00_JPRB, 0.40814E+00_JPRB, 0.40705E+00_JPRB, 0.40211E+00_JPRB, 0.40232E+00_JPRB ,&
&  0.41318E+00_JPRB, 0.37503E+00_JPRB, 0.40139E+00_JPRB, 0.43106E+00_JPRB, 0.45337E+00_JPRB, 0.49035E+00_JPRB, 0.54148E+00_JPRB ,&
&  0.58905E+00_JPRB, 0.60368E+00_JPRB, 0.59791E+00_JPRB, 0.57334E+00_JPRB, 0.55395E+00_JPRB, 0.53911E+00_JPRB, 0.51662E+00_JPRB ,&
&  0.49422E+00_JPRB, 0.47956E+00_JPRB, 0.46974E+00_JPRB, 0.46180E+00_JPRB, 0.45639E+00_JPRB, 0.45173E+00_JPRB, 0.44762E+00_JPRB ,&
&  0.44165E+00_JPRB, 0.43551E+00_JPRB, 0.42991E+00_JPRB, 0.42710E+00_JPRB, 0.42864E+00_JPRB, 0.43490E+00_JPRB, 0.45776E+00_JPRB ,&
&  0.47919E+00_JPRB, 0.49232E+00_JPRB, 0.49596E+00_JPRB, 0.50723E+00_JPRB, 0.52386E+00_JPRB, 0.53813E+00_JPRB, 0.54031E+00_JPRB ,&
&  0.52770E+00_JPRB, 0.50977E+00_JPRB, 0.49507E+00_JPRB, 0.48486E+00_JPRB, 0.48015E+00_JPRB, 0.48136E+00_JPRB, 0.48745E+00_JPRB ,&
&  0.49004E+00_JPRB, 0.48965E+00_JPRB, 0.48984E+00_JPRB, 0.49255E+00_JPRB, 0.49648E+00_JPRB, 0.50377E+00_JPRB, 0.51394E+00_JPRB ,&
&  0.52635E+00_JPRB, 0.54023E+00_JPRB, 0.55609E+00_JPRB, 0.56845E+00_JPRB, 0.57131E+00_JPRB, 0.56974E+00_JPRB, 0.56427E+00_JPRB ,&
&  0.55776E+00_JPRB, 0.55339E+00_JPRB, 0.55011E+00_JPRB, 0.54065E+00_JPRB, 0.56135E+00_JPRB, 0.56271E+00_JPRB, 0.54933E+00_JPRB ,&
&  0.53750E+00_JPRB, 0.52311E+00_JPRB, 0.51868E+00_JPRB, 0.47244E+00_JPRB, 0.50479E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.46797E+00_JPRB, 0.46812E+00_JPRB, 0.46516E+00_JPRB, 0.46177E+00_JPRB, 0.45931E+00_JPRB, 0.45540E+00_JPRB, 0.45228E+00_JPRB ,&
&  0.44783E+00_JPRB, 0.44496E+00_JPRB, 0.43955E+00_JPRB, 0.43620E+00_JPRB, 0.43437E+00_JPRB, 0.43381E+00_JPRB, 0.43272E+00_JPRB ,&
&  0.42893E+00_JPRB, 0.42230E+00_JPRB, 0.41536E+00_JPRB, 0.40978E+00_JPRB, 0.40573E+00_JPRB, 0.40147E+00_JPRB, 0.39762E+00_JPRB ,&
&  0.39427E+00_JPRB, 0.39129E+00_JPRB, 0.38862E+00_JPRB, 0.38896E+00_JPRB, 0.38958E+00_JPRB, 0.38924E+00_JPRB, 0.38765E+00_JPRB ,&
&  0.39458E+00_JPRB, 0.34540E+00_JPRB, 0.35746E+00_JPRB, 0.37096E+00_JPRB, 0.38460E+00_JPRB, 0.41756E+00_JPRB, 0.46414E+00_JPRB ,&
&  0.50409E+00_JPRB, 0.53463E+00_JPRB, 0.54434E+00_JPRB, 0.53421E+00_JPRB, 0.51996E+00_JPRB, 0.51027E+00_JPRB, 0.50191E+00_JPRB ,&
&  0.48601E+00_JPRB, 0.47184E+00_JPRB, 0.45899E+00_JPRB, 0.44668E+00_JPRB, 0.43490E+00_JPRB, 0.42504E+00_JPRB, 0.41615E+00_JPRB ,&
&  0.41060E+00_JPRB, 0.40849E+00_JPRB, 0.40698E+00_JPRB, 0.40576E+00_JPRB, 0.40651E+00_JPRB, 0.40931E+00_JPRB, 0.41227E+00_JPRB ,&
&  0.41751E+00_JPRB, 0.42689E+00_JPRB, 0.43097E+00_JPRB, 0.43437E+00_JPRB, 0.44151E+00_JPRB, 0.45315E+00_JPRB, 0.45829E+00_JPRB ,&
&  0.45704E+00_JPRB, 0.45092E+00_JPRB, 0.44309E+00_JPRB, 0.43549E+00_JPRB, 0.42883E+00_JPRB, 0.42290E+00_JPRB, 0.42174E+00_JPRB ,&
&  0.42123E+00_JPRB, 0.42269E+00_JPRB, 0.42493E+00_JPRB, 0.42974E+00_JPRB, 0.43535E+00_JPRB, 0.44141E+00_JPRB, 0.44803E+00_JPRB ,&
&  0.45523E+00_JPRB, 0.46310E+00_JPRB, 0.47377E+00_JPRB, 0.48558E+00_JPRB, 0.49489E+00_JPRB, 0.49955E+00_JPRB, 0.50082E+00_JPRB ,&
&  0.50033E+00_JPRB, 0.50019E+00_JPRB, 0.49701E+00_JPRB, 0.50236E+00_JPRB, 0.50537E+00_JPRB, 0.50917E+00_JPRB, 0.50506E+00_JPRB ,&
&  0.49867E+00_JPRB, 0.49255E+00_JPRB, 0.42531E+00_JPRB, 0.45447E+00_JPRB, 0.46286E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.44269E+00_JPRB, 0.43905E+00_JPRB, 0.43497E+00_JPRB, 0.42995E+00_JPRB, 0.42552E+00_JPRB, 0.42241E+00_JPRB, 0.42101E+00_JPRB ,&
&  0.42174E+00_JPRB, 0.42293E+00_JPRB, 0.42188E+00_JPRB, 0.41735E+00_JPRB, 0.41316E+00_JPRB, 0.41049E+00_JPRB, 0.40935E+00_JPRB ,&
&  0.40750E+00_JPRB, 0.40402E+00_JPRB, 0.39907E+00_JPRB, 0.39430E+00_JPRB, 0.38872E+00_JPRB, 0.38400E+00_JPRB, 0.38008E+00_JPRB ,&
&  0.37690E+00_JPRB, 0.37558E+00_JPRB, 0.37469E+00_JPRB, 0.37423E+00_JPRB, 0.37503E+00_JPRB, 0.37505E+00_JPRB, 0.37993E+00_JPRB ,&
&  0.35475E+00_JPRB, 0.32343E+00_JPRB, 0.33580E+00_JPRB, 0.34781E+00_JPRB, 0.35934E+00_JPRB, 0.38432E+00_JPRB, 0.41689E+00_JPRB ,&
&  0.45155E+00_JPRB, 0.48063E+00_JPRB, 0.49676E+00_JPRB, 0.49415E+00_JPRB, 0.48294E+00_JPRB, 0.47472E+00_JPRB, 0.47047E+00_JPRB ,&
&  0.46214E+00_JPRB, 0.45228E+00_JPRB, 0.44413E+00_JPRB, 0.43447E+00_JPRB, 0.42413E+00_JPRB, 0.41447E+00_JPRB, 0.40660E+00_JPRB ,&
&  0.40035E+00_JPRB, 0.39662E+00_JPRB, 0.39583E+00_JPRB, 0.39461E+00_JPRB, 0.39366E+00_JPRB, 0.39439E+00_JPRB, 0.39424E+00_JPRB ,&
&  0.39480E+00_JPRB, 0.39523E+00_JPRB, 0.39743E+00_JPRB, 0.39922E+00_JPRB, 0.40185E+00_JPRB, 0.40338E+00_JPRB, 0.40335E+00_JPRB ,&
&  0.40157E+00_JPRB, 0.40024E+00_JPRB, 0.40011E+00_JPRB, 0.39781E+00_JPRB, 0.39099E+00_JPRB, 0.38391E+00_JPRB, 0.38039E+00_JPRB ,&
&  0.37995E+00_JPRB, 0.38049E+00_JPRB, 0.38131E+00_JPRB, 0.38365E+00_JPRB, 0.38709E+00_JPRB, 0.39116E+00_JPRB, 0.39592E+00_JPRB ,&
&  0.40144E+00_JPRB, 0.40681E+00_JPRB, 0.41155E+00_JPRB, 0.41670E+00_JPRB, 0.42441E+00_JPRB, 0.43130E+00_JPRB, 0.43749E+00_JPRB ,&
&  0.44284E+00_JPRB, 0.44776E+00_JPRB, 0.45227E+00_JPRB, 0.45916E+00_JPRB, 0.46589E+00_JPRB, 0.47047E+00_JPRB, 0.46970E+00_JPRB ,&
&  0.46485E+00_JPRB, 0.45672E+00_JPRB, 0.45451E+00_JPRB, 0.44873E+00_JPRB, 0.44459E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.40133E+00_JPRB, 0.39917E+00_JPRB, 0.39576E+00_JPRB, 0.39179E+00_JPRB, 0.38746E+00_JPRB, 0.38528E+00_JPRB, 0.38459E+00_JPRB ,&
&  0.38628E+00_JPRB, 0.38650E+00_JPRB, 0.38797E+00_JPRB, 0.38620E+00_JPRB, 0.38387E+00_JPRB, 0.38214E+00_JPRB, 0.38019E+00_JPRB ,&
&  0.37783E+00_JPRB, 0.37475E+00_JPRB, 0.37171E+00_JPRB, 0.36924E+00_JPRB, 0.36625E+00_JPRB, 0.36205E+00_JPRB, 0.35813E+00_JPRB ,&
&  0.35522E+00_JPRB, 0.35400E+00_JPRB, 0.35332E+00_JPRB, 0.35288E+00_JPRB, 0.35248E+00_JPRB, 0.35364E+00_JPRB, 0.35455E+00_JPRB ,&
&  0.35712E+00_JPRB, 0.32775E+00_JPRB, 0.33812E+00_JPRB, 0.34235E+00_JPRB, 0.35033E+00_JPRB, 0.36735E+00_JPRB, 0.38749E+00_JPRB ,&
&  0.40762E+00_JPRB, 0.42492E+00_JPRB, 0.43622E+00_JPRB, 0.43777E+00_JPRB, 0.43222E+00_JPRB, 0.42828E+00_JPRB, 0.42836E+00_JPRB ,&
&  0.42674E+00_JPRB, 0.42290E+00_JPRB, 0.41950E+00_JPRB, 0.41549E+00_JPRB, 0.40974E+00_JPRB, 0.40142E+00_JPRB, 0.39304E+00_JPRB ,&
&  0.38838E+00_JPRB, 0.38589E+00_JPRB, 0.38371E+00_JPRB, 0.38132E+00_JPRB, 0.37683E+00_JPRB, 0.37271E+00_JPRB, 0.36913E+00_JPRB ,&
&  0.36583E+00_JPRB, 0.36113E+00_JPRB, 0.35866E+00_JPRB, 0.35778E+00_JPRB, 0.35614E+00_JPRB, 0.35582E+00_JPRB, 0.35558E+00_JPRB ,&
&  0.35421E+00_JPRB, 0.35185E+00_JPRB, 0.35061E+00_JPRB, 0.35056E+00_JPRB, 0.34908E+00_JPRB, 0.34732E+00_JPRB, 0.34510E+00_JPRB ,&
&  0.34453E+00_JPRB, 0.34438E+00_JPRB, 0.34377E+00_JPRB, 0.34398E+00_JPRB, 0.34533E+00_JPRB, 0.34832E+00_JPRB, 0.35198E+00_JPRB ,&
&  0.35607E+00_JPRB, 0.36067E+00_JPRB, 0.36484E+00_JPRB, 0.36844E+00_JPRB, 0.37354E+00_JPRB, 0.37816E+00_JPRB, 0.38421E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39789E+00_JPRB, 0.40401E+00_JPRB, 0.41018E+00_JPRB, 0.41498E+00_JPRB, 0.41739E+00_JPRB, 0.41854E+00_JPRB ,&
&  0.41859E+00_JPRB, 0.41382E+00_JPRB, 0.41012E+00_JPRB, 0.40580E+00_JPRB, 0.40223E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.36965E+00_JPRB, 0.36642E+00_JPRB, 0.36248E+00_JPRB, 0.35942E+00_JPRB, 0.35646E+00_JPRB, 0.35323E+00_JPRB, 0.35179E+00_JPRB ,&
&  0.35180E+00_JPRB, 0.35281E+00_JPRB, 0.35471E+00_JPRB, 0.35553E+00_JPRB, 0.35580E+00_JPRB, 0.35654E+00_JPRB, 0.35626E+00_JPRB ,&
&  0.35543E+00_JPRB, 0.35406E+00_JPRB, 0.35223E+00_JPRB, 0.34960E+00_JPRB, 0.34639E+00_JPRB, 0.34340E+00_JPRB, 0.33944E+00_JPRB ,&
&  0.33674E+00_JPRB, 0.33525E+00_JPRB, 0.33446E+00_JPRB, 0.33432E+00_JPRB, 0.33550E+00_JPRB, 0.33715E+00_JPRB, 0.33873E+00_JPRB ,&
&  0.33870E+00_JPRB, 0.32830E+00_JPRB, 0.33368E+00_JPRB, 0.33675E+00_JPRB, 0.34515E+00_JPRB, 0.35937E+00_JPRB, 0.37225E+00_JPRB ,&
&  0.38054E+00_JPRB, 0.38930E+00_JPRB, 0.39679E+00_JPRB, 0.40075E+00_JPRB, 0.39890E+00_JPRB, 0.39663E+00_JPRB, 0.39752E+00_JPRB ,&
&  0.39813E+00_JPRB, 0.39714E+00_JPRB, 0.39583E+00_JPRB, 0.39383E+00_JPRB, 0.39074E+00_JPRB, 0.38747E+00_JPRB, 0.38242E+00_JPRB ,&
&  0.37748E+00_JPRB, 0.37402E+00_JPRB, 0.37053E+00_JPRB, 0.36691E+00_JPRB, 0.36142E+00_JPRB, 0.35485E+00_JPRB, 0.34689E+00_JPRB ,&
&  0.33827E+00_JPRB, 0.33217E+00_JPRB, 0.32838E+00_JPRB, 0.32496E+00_JPRB, 0.32180E+00_JPRB, 0.32014E+00_JPRB, 0.31814E+00_JPRB ,&
&  0.31746E+00_JPRB, 0.31815E+00_JPRB, 0.31744E+00_JPRB, 0.31710E+00_JPRB, 0.31796E+00_JPRB, 0.31813E+00_JPRB, 0.31905E+00_JPRB ,&
&  0.31856E+00_JPRB, 0.31778E+00_JPRB, 0.31737E+00_JPRB, 0.31694E+00_JPRB, 0.31636E+00_JPRB, 0.31769E+00_JPRB, 0.32021E+00_JPRB ,&
&  0.32293E+00_JPRB, 0.32693E+00_JPRB, 0.33126E+00_JPRB, 0.33468E+00_JPRB, 0.33779E+00_JPRB, 0.34279E+00_JPRB, 0.34926E+00_JPRB ,&
&  0.35668E+00_JPRB, 0.36480E+00_JPRB, 0.37237E+00_JPRB, 0.37862E+00_JPRB, 0.38120E+00_JPRB, 0.38229E+00_JPRB, 0.38314E+00_JPRB ,&
&  0.38236E+00_JPRB, 0.37791E+00_JPRB, 0.37589E+00_JPRB, 0.37451E+00_JPRB, 0.37226E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.32738E+00_JPRB, 0.32629E+00_JPRB, 0.32544E+00_JPRB, 0.32526E+00_JPRB, 0.32508E+00_JPRB, 0.32493E+00_JPRB, 0.32530E+00_JPRB ,&
&  0.32643E+00_JPRB, 0.32720E+00_JPRB, 0.33039E+00_JPRB, 0.33179E+00_JPRB, 0.33214E+00_JPRB, 0.33318E+00_JPRB, 0.33393E+00_JPRB ,&
&  0.33277E+00_JPRB, 0.33180E+00_JPRB, 0.33072E+00_JPRB, 0.32855E+00_JPRB, 0.32500E+00_JPRB, 0.32315E+00_JPRB, 0.32157E+00_JPRB ,&
&  0.31895E+00_JPRB, 0.31577E+00_JPRB, 0.31302E+00_JPRB, 0.31171E+00_JPRB, 0.31203E+00_JPRB, 0.31189E+00_JPRB, 0.31268E+00_JPRB ,&
&  0.31303E+00_JPRB, 0.31601E+00_JPRB, 0.30438E+00_JPRB, 0.31393E+00_JPRB, 0.32300E+00_JPRB, 0.33193E+00_JPRB, 0.34137E+00_JPRB ,&
&  0.34800E+00_JPRB, 0.35424E+00_JPRB, 0.36103E+00_JPRB, 0.36271E+00_JPRB, 0.36139E+00_JPRB, 0.35801E+00_JPRB, 0.35673E+00_JPRB ,&
&  0.35599E+00_JPRB, 0.35346E+00_JPRB, 0.35175E+00_JPRB, 0.35176E+00_JPRB, 0.35198E+00_JPRB, 0.35071E+00_JPRB, 0.34866E+00_JPRB ,&
&  0.34688E+00_JPRB, 0.34455E+00_JPRB, 0.34062E+00_JPRB, 0.33618E+00_JPRB, 0.33139E+00_JPRB, 0.32533E+00_JPRB, 0.31704E+00_JPRB ,&
&  0.30750E+00_JPRB, 0.29998E+00_JPRB, 0.29372E+00_JPRB, 0.28987E+00_JPRB, 0.28826E+00_JPRB, 0.28682E+00_JPRB, 0.28550E+00_JPRB ,&
&  0.28542E+00_JPRB, 0.28601E+00_JPRB, 0.28624E+00_JPRB, 0.28606E+00_JPRB, 0.28687E+00_JPRB, 0.28761E+00_JPRB, 0.29026E+00_JPRB ,&
&  0.29327E+00_JPRB, 0.29466E+00_JPRB, 0.29515E+00_JPRB, 0.29560E+00_JPRB, 0.29437E+00_JPRB, 0.29494E+00_JPRB, 0.29669E+00_JPRB ,&
&  0.29864E+00_JPRB, 0.30123E+00_JPRB, 0.30307E+00_JPRB, 0.30485E+00_JPRB, 0.30728E+00_JPRB, 0.31093E+00_JPRB, 0.31646E+00_JPRB ,&
&  0.32183E+00_JPRB, 0.32771E+00_JPRB, 0.33405E+00_JPRB, 0.33913E+00_JPRB, 0.34221E+00_JPRB, 0.34327E+00_JPRB, 0.34377E+00_JPRB ,&
&  0.34235E+00_JPRB, 0.33840E+00_JPRB, 0.33438E+00_JPRB, 0.33150E+00_JPRB, 0.32919E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.29713E+00_JPRB, 0.29754E+00_JPRB, 0.29884E+00_JPRB, 0.29951E+00_JPRB, 0.29961E+00_JPRB, 0.30043E+00_JPRB, 0.30331E+00_JPRB ,&
&  0.30590E+00_JPRB, 0.30763E+00_JPRB, 0.30915E+00_JPRB, 0.30992E+00_JPRB, 0.31021E+00_JPRB, 0.31184E+00_JPRB, 0.31312E+00_JPRB ,&
&  0.31336E+00_JPRB, 0.31235E+00_JPRB, 0.31116E+00_JPRB, 0.30967E+00_JPRB, 0.30760E+00_JPRB, 0.30538E+00_JPRB, 0.30358E+00_JPRB ,&
&  0.30117E+00_JPRB, 0.29846E+00_JPRB, 0.29583E+00_JPRB, 0.29480E+00_JPRB, 0.29478E+00_JPRB, 0.29495E+00_JPRB, 0.29583E+00_JPRB ,&
&  0.29671E+00_JPRB, 0.29918E+00_JPRB, 0.30211E+00_JPRB, 0.30555E+00_JPRB, 0.30899E+00_JPRB, 0.31357E+00_JPRB, 0.31862E+00_JPRB ,&
&  0.32338E+00_JPRB, 0.32583E+00_JPRB, 0.32764E+00_JPRB, 0.32580E+00_JPRB, 0.32525E+00_JPRB, 0.32460E+00_JPRB, 0.32402E+00_JPRB ,&
&  0.32281E+00_JPRB, 0.31790E+00_JPRB, 0.31392E+00_JPRB, 0.31356E+00_JPRB, 0.31435E+00_JPRB, 0.31487E+00_JPRB, 0.31495E+00_JPRB ,&
&  0.31400E+00_JPRB, 0.31252E+00_JPRB, 0.30969E+00_JPRB, 0.30576E+00_JPRB, 0.30160E+00_JPRB, 0.29652E+00_JPRB, 0.28920E+00_JPRB ,&
&  0.28235E+00_JPRB, 0.27680E+00_JPRB, 0.27253E+00_JPRB, 0.26883E+00_JPRB, 0.26647E+00_JPRB, 0.26395E+00_JPRB, 0.26347E+00_JPRB ,&
&  0.26412E+00_JPRB, 0.26617E+00_JPRB, 0.26729E+00_JPRB, 0.26654E+00_JPRB, 0.26540E+00_JPRB, 0.26553E+00_JPRB, 0.26765E+00_JPRB ,&
&  0.27163E+00_JPRB, 0.27477E+00_JPRB, 0.27586E+00_JPRB, 0.27563E+00_JPRB, 0.27494E+00_JPRB, 0.27587E+00_JPRB, 0.27778E+00_JPRB ,&
&  0.27875E+00_JPRB, 0.27926E+00_JPRB, 0.28033E+00_JPRB, 0.28017E+00_JPRB, 0.28206E+00_JPRB, 0.28531E+00_JPRB, 0.28891E+00_JPRB ,&
&  0.29240E+00_JPRB, 0.29675E+00_JPRB, 0.30105E+00_JPRB, 0.30426E+00_JPRB, 0.30694E+00_JPRB, 0.30674E+00_JPRB, 0.30669E+00_JPRB ,&
&  0.30644E+00_JPRB, 0.30494E+00_JPRB, 0.30349E+00_JPRB, 0.30138E+00_JPRB, 0.29872E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.27116E+00_JPRB, 0.27398E+00_JPRB, 0.27757E+00_JPRB, 0.27849E+00_JPRB, 0.27904E+00_JPRB, 0.28068E+00_JPRB, 0.28390E+00_JPRB ,&
&  0.28625E+00_JPRB, 0.28711E+00_JPRB, 0.28732E+00_JPRB, 0.28736E+00_JPRB, 0.28791E+00_JPRB, 0.28962E+00_JPRB, 0.29041E+00_JPRB ,&
&  0.28984E+00_JPRB, 0.28927E+00_JPRB, 0.28847E+00_JPRB, 0.28785E+00_JPRB, 0.28641E+00_JPRB, 0.28425E+00_JPRB, 0.28242E+00_JPRB ,&
&  0.27988E+00_JPRB, 0.27754E+00_JPRB, 0.27583E+00_JPRB, 0.27451E+00_JPRB, 0.27381E+00_JPRB, 0.27385E+00_JPRB, 0.27539E+00_JPRB ,&
&  0.27732E+00_JPRB, 0.27979E+00_JPRB, 0.28280E+00_JPRB, 0.28626E+00_JPRB, 0.28974E+00_JPRB, 0.29370E+00_JPRB, 0.29720E+00_JPRB ,&
&  0.29997E+00_JPRB, 0.30229E+00_JPRB, 0.30377E+00_JPRB, 0.30087E+00_JPRB, 0.29723E+00_JPRB, 0.29620E+00_JPRB, 0.29565E+00_JPRB ,&
&  0.29312E+00_JPRB, 0.28956E+00_JPRB, 0.28481E+00_JPRB, 0.28212E+00_JPRB, 0.28134E+00_JPRB, 0.28030E+00_JPRB, 0.27901E+00_JPRB ,&
&  0.27811E+00_JPRB, 0.27748E+00_JPRB, 0.27636E+00_JPRB, 0.27439E+00_JPRB, 0.27113E+00_JPRB, 0.26663E+00_JPRB, 0.26213E+00_JPRB ,&
&  0.25927E+00_JPRB, 0.25728E+00_JPRB, 0.25736E+00_JPRB, 0.25547E+00_JPRB, 0.25159E+00_JPRB, 0.24801E+00_JPRB, 0.24774E+00_JPRB ,&
&  0.24815E+00_JPRB, 0.24899E+00_JPRB, 0.24898E+00_JPRB, 0.24780E+00_JPRB, 0.24795E+00_JPRB, 0.24873E+00_JPRB, 0.25011E+00_JPRB ,&
&  0.25404E+00_JPRB, 0.25687E+00_JPRB, 0.25678E+00_JPRB, 0.25574E+00_JPRB, 0.25382E+00_JPRB, 0.25338E+00_JPRB, 0.25404E+00_JPRB ,&
&  0.25474E+00_JPRB, 0.25501E+00_JPRB, 0.25523E+00_JPRB, 0.25464E+00_JPRB, 0.25472E+00_JPRB, 0.25617E+00_JPRB, 0.25801E+00_JPRB ,&
&  0.26024E+00_JPRB, 0.26298E+00_JPRB, 0.26538E+00_JPRB, 0.26778E+00_JPRB, 0.26993E+00_JPRB, 0.27084E+00_JPRB, 0.27166E+00_JPRB ,&
&  0.27258E+00_JPRB, 0.27265E+00_JPRB, 0.27230E+00_JPRB, 0.27209E+00_JPRB, 0.27182E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.25139E+00_JPRB, 0.25541E+00_JPRB, 0.26073E+00_JPRB, 0.26207E+00_JPRB, 0.26463E+00_JPRB, 0.26727E+00_JPRB, 0.27082E+00_JPRB ,&
&  0.27318E+00_JPRB, 0.27352E+00_JPRB, 0.27221E+00_JPRB, 0.27086E+00_JPRB, 0.27018E+00_JPRB, 0.27001E+00_JPRB, 0.26952E+00_JPRB ,&
&  0.26934E+00_JPRB, 0.26951E+00_JPRB, 0.26945E+00_JPRB, 0.26926E+00_JPRB, 0.26808E+00_JPRB, 0.26630E+00_JPRB, 0.26431E+00_JPRB ,&
&  0.26100E+00_JPRB, 0.25882E+00_JPRB, 0.25816E+00_JPRB, 0.25798E+00_JPRB, 0.25731E+00_JPRB, 0.25786E+00_JPRB, 0.25960E+00_JPRB ,&
&  0.26334E+00_JPRB, 0.26632E+00_JPRB, 0.27013E+00_JPRB, 0.27402E+00_JPRB, 0.27631E+00_JPRB, 0.27796E+00_JPRB, 0.27992E+00_JPRB ,&
&  0.28112E+00_JPRB, 0.28138E+00_JPRB, 0.28135E+00_JPRB, 0.28125E+00_JPRB, 0.27837E+00_JPRB, 0.27731E+00_JPRB, 0.27673E+00_JPRB ,&
&  0.27552E+00_JPRB, 0.27292E+00_JPRB, 0.26925E+00_JPRB, 0.26610E+00_JPRB, 0.26294E+00_JPRB, 0.26068E+00_JPRB, 0.25878E+00_JPRB ,&
&  0.25740E+00_JPRB, 0.25679E+00_JPRB, 0.25684E+00_JPRB, 0.25548E+00_JPRB, 0.25046E+00_JPRB, 0.24550E+00_JPRB, 0.24275E+00_JPRB ,&
&  0.24150E+00_JPRB, 0.24126E+00_JPRB, 0.24269E+00_JPRB, 0.24156E+00_JPRB, 0.23801E+00_JPRB, 0.23520E+00_JPRB, 0.23419E+00_JPRB ,&
&  0.23284E+00_JPRB, 0.23228E+00_JPRB, 0.23302E+00_JPRB, 0.23356E+00_JPRB, 0.23404E+00_JPRB, 0.23410E+00_JPRB, 0.23514E+00_JPRB ,&
&  0.23619E+00_JPRB, 0.23772E+00_JPRB, 0.23760E+00_JPRB, 0.23492E+00_JPRB, 0.23121E+00_JPRB, 0.22855E+00_JPRB, 0.22814E+00_JPRB ,&
&  0.22974E+00_JPRB, 0.23128E+00_JPRB, 0.23174E+00_JPRB, 0.23191E+00_JPRB, 0.23296E+00_JPRB, 0.23539E+00_JPRB, 0.23853E+00_JPRB ,&
&  0.23987E+00_JPRB, 0.24116E+00_JPRB, 0.24298E+00_JPRB, 0.24482E+00_JPRB, 0.24597E+00_JPRB, 0.24653E+00_JPRB, 0.24657E+00_JPRB ,&
&  0.24588E+00_JPRB, 0.24656E+00_JPRB, 0.24809E+00_JPRB, 0.24991E+00_JPRB, 0.25113E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.23834E+00_JPRB, 0.24141E+00_JPRB, 0.24449E+00_JPRB, 0.24626E+00_JPRB, 0.24810E+00_JPRB, 0.25001E+00_JPRB, 0.25338E+00_JPRB ,&
&  0.25735E+00_JPRB, 0.25871E+00_JPRB, 0.25716E+00_JPRB, 0.25452E+00_JPRB, 0.25284E+00_JPRB, 0.25213E+00_JPRB, 0.25197E+00_JPRB ,&
&  0.25184E+00_JPRB, 0.25171E+00_JPRB, 0.25090E+00_JPRB, 0.24906E+00_JPRB, 0.24647E+00_JPRB, 0.24388E+00_JPRB, 0.24194E+00_JPRB ,&
&  0.24165E+00_JPRB, 0.24307E+00_JPRB, 0.24536E+00_JPRB, 0.24564E+00_JPRB, 0.24584E+00_JPRB, 0.24709E+00_JPRB, 0.24910E+00_JPRB ,&
&  0.25296E+00_JPRB, 0.25487E+00_JPRB, 0.25687E+00_JPRB, 0.25276E+00_JPRB, 0.25719E+00_JPRB, 0.25874E+00_JPRB, 0.25917E+00_JPRB ,&
&  0.26037E+00_JPRB, 0.26133E+00_JPRB, 0.26228E+00_JPRB, 0.26159E+00_JPRB, 0.25730E+00_JPRB, 0.25541E+00_JPRB, 0.25598E+00_JPRB ,&
&  0.25655E+00_JPRB, 0.25495E+00_JPRB, 0.25211E+00_JPRB, 0.24945E+00_JPRB, 0.24721E+00_JPRB, 0.24563E+00_JPRB, 0.24432E+00_JPRB ,&
&  0.24368E+00_JPRB, 0.24296E+00_JPRB, 0.24110E+00_JPRB, 0.23600E+00_JPRB, 0.23017E+00_JPRB, 0.22634E+00_JPRB, 0.22371E+00_JPRB ,&
&  0.22380E+00_JPRB, 0.22576E+00_JPRB, 0.22747E+00_JPRB, 0.22768E+00_JPRB, 0.22654E+00_JPRB, 0.22438E+00_JPRB, 0.22118E+00_JPRB ,&
&  0.21641E+00_JPRB, 0.21441E+00_JPRB, 0.21384E+00_JPRB, 0.21291E+00_JPRB, 0.21330E+00_JPRB, 0.21501E+00_JPRB, 0.21673E+00_JPRB ,&
&  0.21881E+00_JPRB, 0.21882E+00_JPRB, 0.21744E+00_JPRB, 0.21506E+00_JPRB, 0.21257E+00_JPRB, 0.21240E+00_JPRB, 0.21304E+00_JPRB ,&
&  0.21364E+00_JPRB, 0.21415E+00_JPRB, 0.21503E+00_JPRB, 0.21620E+00_JPRB, 0.21775E+00_JPRB, 0.22002E+00_JPRB, 0.22191E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22072E+00_JPRB, 0.22174E+00_JPRB, 0.22341E+00_JPRB, 0.22425E+00_JPRB, 0.22476E+00_JPRB, 0.22511E+00_JPRB ,&
&  0.22479E+00_JPRB, 0.22489E+00_JPRB, 0.22682E+00_JPRB, 0.23142E+00_JPRB, 0.23559E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.22666E+00_JPRB, 0.22796E+00_JPRB, 0.22814E+00_JPRB, 0.22858E+00_JPRB, 0.22919E+00_JPRB, 0.23107E+00_JPRB, 0.23524E+00_JPRB ,&
&  0.23934E+00_JPRB, 0.24128E+00_JPRB, 0.23985E+00_JPRB, 0.23767E+00_JPRB, 0.23656E+00_JPRB, 0.23598E+00_JPRB, 0.23580E+00_JPRB ,&
&  0.23561E+00_JPRB, 0.23457E+00_JPRB, 0.23165E+00_JPRB, 0.22764E+00_JPRB, 0.22469E+00_JPRB, 0.22393E+00_JPRB, 0.22434E+00_JPRB ,&
&  0.22809E+00_JPRB, 0.23069E+00_JPRB, 0.23314E+00_JPRB, 0.23529E+00_JPRB, 0.23581E+00_JPRB, 0.23545E+00_JPRB, 0.23683E+00_JPRB ,&
&  0.23781E+00_JPRB, 0.23546E+00_JPRB, 0.23008E+00_JPRB, 0.23415E+00_JPRB, 0.23618E+00_JPRB, 0.23742E+00_JPRB, 0.23667E+00_JPRB ,&
&  0.23732E+00_JPRB, 0.23941E+00_JPRB, 0.24143E+00_JPRB, 0.24304E+00_JPRB, 0.24229E+00_JPRB, 0.23914E+00_JPRB, 0.23806E+00_JPRB ,&
&  0.23822E+00_JPRB, 0.23713E+00_JPRB, 0.23448E+00_JPRB, 0.23207E+00_JPRB, 0.22936E+00_JPRB, 0.22816E+00_JPRB, 0.22797E+00_JPRB ,&
&  0.22713E+00_JPRB, 0.22501E+00_JPRB, 0.22071E+00_JPRB, 0.21650E+00_JPRB, 0.21342E+00_JPRB, 0.21046E+00_JPRB, 0.20927E+00_JPRB ,&
&  0.20975E+00_JPRB, 0.21099E+00_JPRB, 0.21110E+00_JPRB, 0.20975E+00_JPRB, 0.20516E+00_JPRB, 0.19507E+00_JPRB, 0.18798E+00_JPRB ,&
&  0.18673E+00_JPRB, 0.19136E+00_JPRB, 0.19954E+00_JPRB, 0.20089E+00_JPRB, 0.20303E+00_JPRB, 0.20449E+00_JPRB, 0.20580E+00_JPRB ,&
&  0.20441E+00_JPRB, 0.19125E+00_JPRB, 0.18545E+00_JPRB, 0.18220E+00_JPRB, 0.17908E+00_JPRB, 0.17862E+00_JPRB, 0.18014E+00_JPRB ,&
&  0.18327E+00_JPRB, 0.18736E+00_JPRB, 0.19018E+00_JPRB, 0.19178E+00_JPRB, 0.19223E+00_JPRB, 0.19186E+00_JPRB, 0.19115E+00_JPRB ,&
&  0.19129E+00_JPRB, 0.19509E+00_JPRB, 0.20284E+00_JPRB, 0.21116E+00_JPRB, 0.21217E+00_JPRB, 0.21377E+00_JPRB, 0.21427E+00_JPRB ,&
&  0.21474E+00_JPRB, 0.21664E+00_JPRB, 0.21782E+00_JPRB, 0.22057E+00_JPRB, 0.22395E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.21632E+00_JPRB, 0.21753E+00_JPRB, 0.21774E+00_JPRB, 0.21692E+00_JPRB, 0.21570E+00_JPRB, 0.21592E+00_JPRB, 0.21735E+00_JPRB ,&
&  0.22027E+00_JPRB, 0.22240E+00_JPRB, 0.22294E+00_JPRB, 0.22298E+00_JPRB, 0.22205E+00_JPRB, 0.21999E+00_JPRB, 0.21824E+00_JPRB ,&
&  0.21688E+00_JPRB, 0.21606E+00_JPRB, 0.21541E+00_JPRB, 0.21519E+00_JPRB, 0.21512E+00_JPRB, 0.21609E+00_JPRB, 0.21705E+00_JPRB ,&
&  0.22071E+00_JPRB, 0.22077E+00_JPRB, 0.22010E+00_JPRB, 0.21986E+00_JPRB, 0.21962E+00_JPRB, 0.21955E+00_JPRB, 0.21941E+00_JPRB ,&
&  0.21682E+00_JPRB, 0.20905E+00_JPRB, 0.20212E+00_JPRB, 0.20745E+00_JPRB, 0.21953E+00_JPRB, 0.21988E+00_JPRB, 0.21993E+00_JPRB ,&
&  0.22125E+00_JPRB, 0.22412E+00_JPRB, 0.22612E+00_JPRB, 0.22712E+00_JPRB, 0.22590E+00_JPRB, 0.22406E+00_JPRB, 0.22346E+00_JPRB ,&
&  0.22363E+00_JPRB, 0.22188E+00_JPRB, 0.21680E+00_JPRB, 0.21101E+00_JPRB, 0.20521E+00_JPRB, 0.20293E+00_JPRB, 0.20129E+00_JPRB ,&
&  0.19888E+00_JPRB, 0.19749E+00_JPRB, 0.19695E+00_JPRB, 0.19647E+00_JPRB, 0.19688E+00_JPRB, 0.19492E+00_JPRB, 0.19217E+00_JPRB ,&
&  0.18860E+00_JPRB, 0.18459E+00_JPRB, 0.17970E+00_JPRB, 0.17455E+00_JPRB, 0.17010E+00_JPRB, 0.16537E+00_JPRB, 0.16213E+00_JPRB ,&
&  0.16153E+00_JPRB, 0.16534E+00_JPRB, 0.17480E+00_JPRB, 0.18495E+00_JPRB, 0.19260E+00_JPRB, 0.18158E+00_JPRB, 0.17468E+00_JPRB ,&
&  0.16742E+00_JPRB, 0.16159E+00_JPRB, 0.15829E+00_JPRB, 0.15753E+00_JPRB, 0.15769E+00_JPRB, 0.15852E+00_JPRB, 0.16008E+00_JPRB ,&
&  0.16209E+00_JPRB, 0.16466E+00_JPRB, 0.16663E+00_JPRB, 0.16802E+00_JPRB, 0.16899E+00_JPRB, 0.16926E+00_JPRB, 0.16905E+00_JPRB ,&
&  0.16880E+00_JPRB, 0.17001E+00_JPRB, 0.17242E+00_JPRB, 0.17661E+00_JPRB, 0.17961E+00_JPRB, 0.18346E+00_JPRB, 0.18879E+00_JPRB ,&
&  0.19652E+00_JPRB, 0.20566E+00_JPRB, 0.20887E+00_JPRB, 0.21102E+00_JPRB, 0.21392E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.20529E+00_JPRB, 0.20968E+00_JPRB, 0.21212E+00_JPRB, 0.21165E+00_JPRB, 0.21127E+00_JPRB, 0.21186E+00_JPRB, 0.21260E+00_JPRB ,&
&  0.21361E+00_JPRB, 0.21312E+00_JPRB, 0.21167E+00_JPRB, 0.21022E+00_JPRB, 0.20868E+00_JPRB, 0.20669E+00_JPRB, 0.20534E+00_JPRB ,&
&  0.20430E+00_JPRB, 0.20432E+00_JPRB, 0.20403E+00_JPRB, 0.20494E+00_JPRB, 0.20552E+00_JPRB, 0.20788E+00_JPRB, 0.20643E+00_JPRB ,&
&  0.20292E+00_JPRB, 0.20100E+00_JPRB, 0.20322E+00_JPRB, 0.20356E+00_JPRB, 0.20406E+00_JPRB, 0.20480E+00_JPRB, 0.20611E+00_JPRB ,&
&  0.20365E+00_JPRB, 0.20059E+00_JPRB, 0.19640E+00_JPRB, 0.20159E+00_JPRB, 0.21029E+00_JPRB, 0.21005E+00_JPRB, 0.20938E+00_JPRB ,&
&  0.21081E+00_JPRB, 0.21347E+00_JPRB, 0.21473E+00_JPRB, 0.21502E+00_JPRB, 0.21353E+00_JPRB, 0.21279E+00_JPRB, 0.21165E+00_JPRB ,&
&  0.20889E+00_JPRB, 0.20622E+00_JPRB, 0.19777E+00_JPRB, 0.18200E+00_JPRB, 0.17562E+00_JPRB, 0.17260E+00_JPRB, 0.16872E+00_JPRB ,&
&  0.16372E+00_JPRB, 0.16139E+00_JPRB, 0.16114E+00_JPRB, 0.15960E+00_JPRB, 0.15980E+00_JPRB, 0.15937E+00_JPRB, 0.15840E+00_JPRB ,&
&  0.15715E+00_JPRB, 0.15626E+00_JPRB, 0.15478E+00_JPRB, 0.15242E+00_JPRB, 0.15067E+00_JPRB, 0.15030E+00_JPRB, 0.15108E+00_JPRB ,&
&  0.15300E+00_JPRB, 0.15596E+00_JPRB, 0.16424E+00_JPRB, 0.16908E+00_JPRB, 0.16714E+00_JPRB, 0.16313E+00_JPRB, 0.15703E+00_JPRB ,&
&  0.15167E+00_JPRB, 0.14998E+00_JPRB, 0.14879E+00_JPRB, 0.14780E+00_JPRB, 0.14813E+00_JPRB, 0.14916E+00_JPRB, 0.15021E+00_JPRB ,&
&  0.15143E+00_JPRB, 0.15282E+00_JPRB, 0.15367E+00_JPRB, 0.15414E+00_JPRB, 0.15459E+00_JPRB, 0.15484E+00_JPRB, 0.15533E+00_JPRB ,&
&  0.15549E+00_JPRB, 0.15682E+00_JPRB, 0.15799E+00_JPRB, 0.16029E+00_JPRB, 0.16187E+00_JPRB, 0.16387E+00_JPRB, 0.16804E+00_JPRB ,&
&  0.17163E+00_JPRB, 0.18215E+00_JPRB, 0.19681E+00_JPRB, 0.20456E+00_JPRB, 0.20175E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.19752E+00_JPRB, 0.19988E+00_JPRB, 0.20213E+00_JPRB, 0.20087E+00_JPRB, 0.20089E+00_JPRB, 0.20115E+00_JPRB, 0.20228E+00_JPRB ,&
&  0.20280E+00_JPRB, 0.20236E+00_JPRB, 0.20199E+00_JPRB, 0.20055E+00_JPRB, 0.19905E+00_JPRB, 0.19218E+00_JPRB, 0.18922E+00_JPRB ,&
&  0.18692E+00_JPRB, 0.18741E+00_JPRB, 0.18789E+00_JPRB, 0.18984E+00_JPRB, 0.19004E+00_JPRB, 0.18919E+00_JPRB, 0.18621E+00_JPRB ,&
&  0.18380E+00_JPRB, 0.18228E+00_JPRB, 0.18068E+00_JPRB, 0.18086E+00_JPRB, 0.18282E+00_JPRB, 0.18497E+00_JPRB, 0.18626E+00_JPRB ,&
&  0.18676E+00_JPRB, 0.18868E+00_JPRB, 0.19033E+00_JPRB, 0.19303E+00_JPRB, 0.19988E+00_JPRB, 0.20102E+00_JPRB, 0.20179E+00_JPRB ,&
&  0.20269E+00_JPRB, 0.20392E+00_JPRB, 0.20468E+00_JPRB, 0.20447E+00_JPRB, 0.20281E+00_JPRB, 0.19809E+00_JPRB, 0.19681E+00_JPRB ,&
&  0.18438E+00_JPRB, 0.17706E+00_JPRB, 0.17010E+00_JPRB, 0.16494E+00_JPRB, 0.15980E+00_JPRB, 0.15703E+00_JPRB, 0.15392E+00_JPRB ,&
&  0.15129E+00_JPRB, 0.14844E+00_JPRB, 0.14630E+00_JPRB, 0.14534E+00_JPRB, 0.14504E+00_JPRB, 0.14479E+00_JPRB, 0.14419E+00_JPRB ,&
&  0.14429E+00_JPRB, 0.14386E+00_JPRB, 0.14356E+00_JPRB, 0.14324E+00_JPRB, 0.14304E+00_JPRB, 0.14316E+00_JPRB, 0.14391E+00_JPRB ,&
&  0.14581E+00_JPRB, 0.14761E+00_JPRB, 0.14944E+00_JPRB, 0.15108E+00_JPRB, 0.15062E+00_JPRB, 0.14896E+00_JPRB, 0.14645E+00_JPRB ,&
&  0.14429E+00_JPRB, 0.14291E+00_JPRB, 0.14141E+00_JPRB, 0.14103E+00_JPRB, 0.14163E+00_JPRB, 0.14238E+00_JPRB, 0.14365E+00_JPRB ,&
&  0.14477E+00_JPRB, 0.14569E+00_JPRB, 0.14630E+00_JPRB, 0.14673E+00_JPRB, 0.14740E+00_JPRB, 0.14805E+00_JPRB, 0.14912E+00_JPRB ,&
&  0.15032E+00_JPRB, 0.15181E+00_JPRB, 0.15318E+00_JPRB, 0.15503E+00_JPRB, 0.15776E+00_JPRB, 0.16193E+00_JPRB, 0.16904E+00_JPRB ,&
&  0.17616E+00_JPRB, 0.19483E+00_JPRB, 0.19705E+00_JPRB, 0.19591E+00_JPRB, 0.19728E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.18889E+00_JPRB, 0.19081E+00_JPRB, 0.19228E+00_JPRB, 0.19364E+00_JPRB, 0.19382E+00_JPRB, 0.19406E+00_JPRB, 0.19475E+00_JPRB ,&
&  0.19097E+00_JPRB, 0.18823E+00_JPRB, 0.18367E+00_JPRB, 0.17937E+00_JPRB, 0.17528E+00_JPRB, 0.17181E+00_JPRB, 0.16971E+00_JPRB ,&
&  0.16862E+00_JPRB, 0.16867E+00_JPRB, 0.16892E+00_JPRB, 0.16956E+00_JPRB, 0.16890E+00_JPRB, 0.16801E+00_JPRB, 0.16714E+00_JPRB ,&
&  0.16603E+00_JPRB, 0.16630E+00_JPRB, 0.16679E+00_JPRB, 0.17005E+00_JPRB, 0.17465E+00_JPRB, 0.17984E+00_JPRB, 0.18358E+00_JPRB ,&
&  0.18623E+00_JPRB, 0.18822E+00_JPRB, 0.19064E+00_JPRB, 0.19166E+00_JPRB, 0.19263E+00_JPRB, 0.19386E+00_JPRB, 0.19440E+00_JPRB ,&
&  0.19570E+00_JPRB, 0.19582E+00_JPRB, 0.19574E+00_JPRB, 0.19606E+00_JPRB, 0.18786E+00_JPRB, 0.18174E+00_JPRB, 0.17553E+00_JPRB ,&
&  0.16954E+00_JPRB, 0.16349E+00_JPRB, 0.15802E+00_JPRB, 0.15495E+00_JPRB, 0.15181E+00_JPRB, 0.14997E+00_JPRB, 0.14837E+00_JPRB ,&
&  0.14718E+00_JPRB, 0.14594E+00_JPRB, 0.14420E+00_JPRB, 0.14249E+00_JPRB, 0.14149E+00_JPRB, 0.14065E+00_JPRB, 0.13960E+00_JPRB ,&
&  0.13950E+00_JPRB, 0.13888E+00_JPRB, 0.13882E+00_JPRB, 0.13845E+00_JPRB, 0.13818E+00_JPRB, 0.13796E+00_JPRB, 0.13799E+00_JPRB ,&
&  0.13838E+00_JPRB, 0.13912E+00_JPRB, 0.13940E+00_JPRB, 0.13974E+00_JPRB, 0.13961E+00_JPRB, 0.13900E+00_JPRB, 0.13848E+00_JPRB ,&
&  0.13794E+00_JPRB, 0.13751E+00_JPRB, 0.13753E+00_JPRB, 0.13746E+00_JPRB, 0.13796E+00_JPRB, 0.13864E+00_JPRB, 0.13931E+00_JPRB ,&
&  0.14024E+00_JPRB, 0.14094E+00_JPRB, 0.14165E+00_JPRB, 0.14253E+00_JPRB, 0.14329E+00_JPRB, 0.14427E+00_JPRB, 0.14539E+00_JPRB ,&
&  0.14627E+00_JPRB, 0.14797E+00_JPRB, 0.14937E+00_JPRB, 0.15110E+00_JPRB, 0.15433E+00_JPRB, 0.15788E+00_JPRB, 0.16610E+00_JPRB ,&
&  0.17408E+00_JPRB, 0.19084E+00_JPRB, 0.19227E+00_JPRB, 0.18718E+00_JPRB, 0.18773E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.18359E+00_JPRB, 0.18500E+00_JPRB, 0.18574E+00_JPRB, 0.18678E+00_JPRB, 0.18726E+00_JPRB, 0.18875E+00_JPRB, 0.18896E+00_JPRB ,&
&  0.18907E+00_JPRB, 0.18520E+00_JPRB, 0.18285E+00_JPRB, 0.18020E+00_JPRB, 0.17790E+00_JPRB, 0.17525E+00_JPRB, 0.17267E+00_JPRB ,&
&  0.17065E+00_JPRB, 0.16887E+00_JPRB, 0.16698E+00_JPRB, 0.16436E+00_JPRB, 0.16173E+00_JPRB, 0.15981E+00_JPRB, 0.15841E+00_JPRB ,&
&  0.15704E+00_JPRB, 0.15770E+00_JPRB, 0.15918E+00_JPRB, 0.16224E+00_JPRB, 0.16644E+00_JPRB, 0.17000E+00_JPRB, 0.17470E+00_JPRB ,&
&  0.18075E+00_JPRB, 0.18257E+00_JPRB, 0.18470E+00_JPRB, 0.18569E+00_JPRB, 0.18537E+00_JPRB, 0.18584E+00_JPRB, 0.18350E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18142E+00_JPRB, 0.18588E+00_JPRB, 0.18315E+00_JPRB, 0.17671E+00_JPRB, 0.17366E+00_JPRB, 0.16767E+00_JPRB ,&
&  0.16320E+00_JPRB, 0.15844E+00_JPRB, 0.15522E+00_JPRB, 0.15336E+00_JPRB, 0.15048E+00_JPRB, 0.14890E+00_JPRB, 0.14651E+00_JPRB ,&
&  0.14432E+00_JPRB, 0.14355E+00_JPRB, 0.14215E+00_JPRB, 0.14155E+00_JPRB, 0.14078E+00_JPRB, 0.13973E+00_JPRB, 0.13859E+00_JPRB ,&
&  0.13782E+00_JPRB, 0.13714E+00_JPRB, 0.13678E+00_JPRB, 0.13628E+00_JPRB, 0.13559E+00_JPRB, 0.13489E+00_JPRB, 0.13411E+00_JPRB ,&
&  0.13359E+00_JPRB, 0.13330E+00_JPRB, 0.13307E+00_JPRB, 0.13299E+00_JPRB, 0.13290E+00_JPRB, 0.13268E+00_JPRB, 0.13278E+00_JPRB ,&
&  0.13304E+00_JPRB, 0.13363E+00_JPRB, 0.13414E+00_JPRB, 0.13483E+00_JPRB, 0.13546E+00_JPRB, 0.13629E+00_JPRB, 0.13693E+00_JPRB ,&
&  0.13762E+00_JPRB, 0.13854E+00_JPRB, 0.13935E+00_JPRB, 0.14030E+00_JPRB, 0.14134E+00_JPRB, 0.14223E+00_JPRB, 0.14360E+00_JPRB ,&
&  0.14501E+00_JPRB, 0.14640E+00_JPRB, 0.14833E+00_JPRB, 0.15033E+00_JPRB, 0.15288E+00_JPRB, 0.15723E+00_JPRB, 0.16223E+00_JPRB ,&
&  0.16849E+00_JPRB, 0.17784E+00_JPRB, 0.17820E+00_JPRB, 0.18082E+00_JPRB, 0.18235E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.18291E+00_JPRB, 0.18413E+00_JPRB, 0.18515E+00_JPRB, 0.18566E+00_JPRB, 0.18620E+00_JPRB, 0.18623E+00_JPRB, 0.18614E+00_JPRB ,&
&  0.18340E+00_JPRB, 0.18221E+00_JPRB, 0.18048E+00_JPRB, 0.17852E+00_JPRB, 0.17680E+00_JPRB, 0.17466E+00_JPRB, 0.17254E+00_JPRB ,&
&  0.17055E+00_JPRB, 0.16820E+00_JPRB, 0.16575E+00_JPRB, 0.16287E+00_JPRB, 0.16011E+00_JPRB, 0.15750E+00_JPRB, 0.15603E+00_JPRB ,&
&  0.15532E+00_JPRB, 0.15562E+00_JPRB, 0.15713E+00_JPRB, 0.15836E+00_JPRB, 0.16012E+00_JPRB, 0.16237E+00_JPRB, 0.16593E+00_JPRB ,&
&  0.16864E+00_JPRB, 0.17199E+00_JPRB, 0.17421E+00_JPRB, 0.17491E+00_JPRB, 0.17422E+00_JPRB, 0.17386E+00_JPRB, 0.17299E+00_JPRB ,&
&  0.17122E+00_JPRB, 0.16937E+00_JPRB, 0.16778E+00_JPRB, 0.16607E+00_JPRB, 0.16417E+00_JPRB, 0.16184E+00_JPRB, 0.15896E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15406E+00_JPRB, 0.15191E+00_JPRB, 0.15025E+00_JPRB, 0.14864E+00_JPRB, 0.14717E+00_JPRB, 0.14562E+00_JPRB ,&
&  0.14442E+00_JPRB, 0.14338E+00_JPRB, 0.14206E+00_JPRB, 0.14097E+00_JPRB, 0.13973E+00_JPRB, 0.13870E+00_JPRB, 0.13758E+00_JPRB ,&
&  0.13671E+00_JPRB, 0.13595E+00_JPRB, 0.13532E+00_JPRB, 0.13473E+00_JPRB, 0.13436E+00_JPRB, 0.13400E+00_JPRB, 0.13358E+00_JPRB ,&
&  0.13322E+00_JPRB, 0.13318E+00_JPRB, 0.13318E+00_JPRB, 0.13320E+00_JPRB, 0.13329E+00_JPRB, 0.13300E+00_JPRB, 0.13307E+00_JPRB ,&
&  0.13339E+00_JPRB, 0.13372E+00_JPRB, 0.13384E+00_JPRB, 0.13428E+00_JPRB, 0.13496E+00_JPRB, 0.13578E+00_JPRB, 0.13697E+00_JPRB ,&
&  0.13791E+00_JPRB, 0.13885E+00_JPRB, 0.13973E+00_JPRB, 0.14065E+00_JPRB, 0.14181E+00_JPRB, 0.14306E+00_JPRB, 0.14438E+00_JPRB ,&
&  0.14592E+00_JPRB, 0.14738E+00_JPRB, 0.14889E+00_JPRB, 0.15046E+00_JPRB, 0.15295E+00_JPRB, 0.15541E+00_JPRB, 0.15918E+00_JPRB ,&
&  0.16300E+00_JPRB, 0.16500E+00_JPRB, 0.17791E+00_JPRB, 0.17895E+00_JPRB, 0.18047E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.16176E+00_JPRB, 0.16406E+00_JPRB, 0.16569E+00_JPRB, 0.16748E+00_JPRB, 0.16608E+00_JPRB, 0.17758E+00_JPRB, 0.17562E+00_JPRB ,&
&  0.17532E+00_JPRB, 0.17017E+00_JPRB, 0.17042E+00_JPRB, 0.16958E+00_JPRB, 0.16813E+00_JPRB, 0.16650E+00_JPRB, 0.16472E+00_JPRB ,&
&  0.16299E+00_JPRB, 0.16082E+00_JPRB, 0.15901E+00_JPRB, 0.15751E+00_JPRB, 0.15613E+00_JPRB, 0.15493E+00_JPRB, 0.15372E+00_JPRB ,&
&  0.15355E+00_JPRB, 0.15417E+00_JPRB, 0.15517E+00_JPRB, 0.15457E+00_JPRB, 0.15497E+00_JPRB, 0.15582E+00_JPRB, 0.15695E+00_JPRB ,&
&  0.15791E+00_JPRB, 0.15922E+00_JPRB, 0.16006E+00_JPRB, 0.16027E+00_JPRB, 0.16053E+00_JPRB, 0.15983E+00_JPRB, 0.15935E+00_JPRB ,&
&  0.15869E+00_JPRB, 0.15777E+00_JPRB, 0.15661E+00_JPRB, 0.15561E+00_JPRB, 0.15450E+00_JPRB, 0.15306E+00_JPRB, 0.15157E+00_JPRB ,&
&  0.15023E+00_JPRB, 0.14904E+00_JPRB, 0.14779E+00_JPRB, 0.14675E+00_JPRB, 0.14575E+00_JPRB, 0.14476E+00_JPRB, 0.14385E+00_JPRB ,&
&  0.14313E+00_JPRB, 0.14249E+00_JPRB, 0.14179E+00_JPRB, 0.14105E+00_JPRB, 0.14024E+00_JPRB, 0.13945E+00_JPRB, 0.13870E+00_JPRB ,&
&  0.13833E+00_JPRB, 0.13800E+00_JPRB, 0.13766E+00_JPRB, 0.13731E+00_JPRB, 0.13718E+00_JPRB, 0.13705E+00_JPRB, 0.13689E+00_JPRB ,&
&  0.13672E+00_JPRB, 0.13647E+00_JPRB, 0.13638E+00_JPRB, 0.13630E+00_JPRB, 0.13622E+00_JPRB, 0.13617E+00_JPRB, 0.13627E+00_JPRB ,&
&  0.13644E+00_JPRB, 0.13668E+00_JPRB, 0.13715E+00_JPRB, 0.13765E+00_JPRB, 0.13818E+00_JPRB, 0.13870E+00_JPRB, 0.13945E+00_JPRB ,&
&  0.14026E+00_JPRB, 0.14104E+00_JPRB, 0.14171E+00_JPRB, 0.14228E+00_JPRB, 0.14316E+00_JPRB, 0.14398E+00_JPRB, 0.14479E+00_JPRB ,&
&  0.14568E+00_JPRB, 0.14681E+00_JPRB, 0.14799E+00_JPRB, 0.14919E+00_JPRB, 0.15059E+00_JPRB, 0.15179E+00_JPRB, 0.15298E+00_JPRB ,&
&  0.15440E+00_JPRB, 0.15698E+00_JPRB, 0.15875E+00_JPRB, 0.15993E+00_JPRB, 0.16124E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.14786E+00_JPRB, 0.14803E+00_JPRB, 0.14811E+00_JPRB, 0.14813E+00_JPRB, 0.14798E+00_JPRB, 0.14797E+00_JPRB, 0.14795E+00_JPRB ,&
&  0.14794E+00_JPRB, 0.14780E+00_JPRB, 0.14799E+00_JPRB, 0.14809E+00_JPRB, 0.14813E+00_JPRB, 0.14813E+00_JPRB, 0.14802E+00_JPRB ,&
&  0.14775E+00_JPRB, 0.14743E+00_JPRB, 0.14657E+00_JPRB, 0.14642E+00_JPRB, 0.14645E+00_JPRB, 0.14666E+00_JPRB, 0.14680E+00_JPRB ,&
&  0.14703E+00_JPRB, 0.14725E+00_JPRB, 0.14743E+00_JPRB, 0.14796E+00_JPRB, 0.14820E+00_JPRB, 0.14847E+00_JPRB, 0.14874E+00_JPRB ,&
&  0.14899E+00_JPRB, 0.14928E+00_JPRB, 0.14949E+00_JPRB, 0.14972E+00_JPRB, 0.14992E+00_JPRB, 0.14985E+00_JPRB, 0.14965E+00_JPRB ,&
&  0.14935E+00_JPRB, 0.14887E+00_JPRB, 0.14837E+00_JPRB, 0.14792E+00_JPRB, 0.14752E+00_JPRB, 0.14709E+00_JPRB, 0.14669E+00_JPRB ,&
&  0.14625E+00_JPRB, 0.14575E+00_JPRB, 0.14523E+00_JPRB, 0.14475E+00_JPRB, 0.14427E+00_JPRB, 0.14384E+00_JPRB, 0.14337E+00_JPRB ,&
&  0.14299E+00_JPRB, 0.14261E+00_JPRB, 0.14225E+00_JPRB, 0.14188E+00_JPRB, 0.14156E+00_JPRB, 0.14123E+00_JPRB, 0.14091E+00_JPRB ,&
&  0.14058E+00_JPRB, 0.14040E+00_JPRB, 0.14016E+00_JPRB, 0.13995E+00_JPRB, 0.13975E+00_JPRB, 0.13961E+00_JPRB, 0.13942E+00_JPRB ,&
&  0.13924E+00_JPRB, 0.13907E+00_JPRB, 0.13900E+00_JPRB, 0.13887E+00_JPRB, 0.13882E+00_JPRB, 0.13880E+00_JPRB, 0.13886E+00_JPRB ,&
&  0.13892E+00_JPRB, 0.13901E+00_JPRB, 0.13917E+00_JPRB, 0.13942E+00_JPRB, 0.13965E+00_JPRB, 0.13991E+00_JPRB, 0.14016E+00_JPRB ,&
&  0.14045E+00_JPRB, 0.14074E+00_JPRB, 0.14105E+00_JPRB, 0.14135E+00_JPRB, 0.14171E+00_JPRB, 0.14208E+00_JPRB, 0.14249E+00_JPRB ,&
&  0.14286E+00_JPRB, 0.14327E+00_JPRB, 0.14361E+00_JPRB, 0.14402E+00_JPRB, 0.14434E+00_JPRB, 0.14455E+00_JPRB, 0.14474E+00_JPRB ,&
&  0.14508E+00_JPRB, 0.14542E+00_JPRB, 0.14583E+00_JPRB, 0.14619E+00_JPRB, 0.14685E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB, 0.14199E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB, 0.36911E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.36475E+00_JPRB, 0.36480E+00_JPRB, 0.36483E+00_JPRB, 0.36485E+00_JPRB, 0.36487E+00_JPRB, 0.36490E+00_JPRB, 0.36493E+00_JPRB ,&
&  0.36493E+00_JPRB, 0.36504E+00_JPRB, 0.36521E+00_JPRB, 0.36541E+00_JPRB, 0.36560E+00_JPRB, 0.36580E+00_JPRB, 0.36599E+00_JPRB ,&
&  0.36615E+00_JPRB, 0.36626E+00_JPRB, 0.36643E+00_JPRB, 0.36674E+00_JPRB, 0.36713E+00_JPRB, 0.36762E+00_JPRB, 0.36826E+00_JPRB ,&
&  0.36854E+00_JPRB, 0.36855E+00_JPRB, 0.36859E+00_JPRB, 0.36896E+00_JPRB, 0.36895E+00_JPRB, 0.36898E+00_JPRB, 0.36905E+00_JPRB ,&
&  0.36912E+00_JPRB, 0.36911E+00_JPRB, 0.36901E+00_JPRB, 0.36890E+00_JPRB, 0.36887E+00_JPRB, 0.36895E+00_JPRB, 0.36904E+00_JPRB ,&
&  0.36917E+00_JPRB, 0.36924E+00_JPRB, 0.36955E+00_JPRB, 0.36991E+00_JPRB, 0.37032E+00_JPRB, 0.37055E+00_JPRB, 0.37081E+00_JPRB ,&
&  0.37094E+00_JPRB, 0.37110E+00_JPRB, 0.37136E+00_JPRB, 0.37178E+00_JPRB, 0.37227E+00_JPRB, 0.37273E+00_JPRB, 0.37306E+00_JPRB ,&
&  0.37365E+00_JPRB, 0.37443E+00_JPRB, 0.37526E+00_JPRB, 0.37618E+00_JPRB, 0.37689E+00_JPRB, 0.37763E+00_JPRB, 0.37842E+00_JPRB ,&
&  0.37911E+00_JPRB, 0.37955E+00_JPRB, 0.38008E+00_JPRB, 0.38065E+00_JPRB, 0.38110E+00_JPRB, 0.38090E+00_JPRB, 0.38076E+00_JPRB ,&
&  0.38064E+00_JPRB, 0.38035E+00_JPRB, 0.37943E+00_JPRB, 0.37865E+00_JPRB, 0.37792E+00_JPRB, 0.37713E+00_JPRB, 0.37628E+00_JPRB ,&
&  0.37552E+00_JPRB, 0.37485E+00_JPRB, 0.37413E+00_JPRB, 0.37338E+00_JPRB, 0.37269E+00_JPRB, 0.37203E+00_JPRB, 0.37137E+00_JPRB ,&
&  0.37077E+00_JPRB, 0.37025E+00_JPRB, 0.36974E+00_JPRB, 0.36928E+00_JPRB, 0.36878E+00_JPRB, 0.36830E+00_JPRB, 0.36779E+00_JPRB ,&
&  0.36735E+00_JPRB, 0.36689E+00_JPRB, 0.36647E+00_JPRB, 0.36605E+00_JPRB, 0.36574E+00_JPRB, 0.36552E+00_JPRB, 0.36532E+00_JPRB ,&
&  0.36514E+00_JPRB, 0.36501E+00_JPRB, 0.36495E+00_JPRB, 0.36486E+00_JPRB, 0.36476E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.36212E+00_JPRB, 0.36238E+00_JPRB, 0.36261E+00_JPRB, 0.36279E+00_JPRB, 0.36306E+00_JPRB, 0.36340E+00_JPRB, 0.36369E+00_JPRB ,&
&  0.36391E+00_JPRB, 0.36429E+00_JPRB, 0.36474E+00_JPRB, 0.36520E+00_JPRB, 0.36562E+00_JPRB, 0.36612E+00_JPRB, 0.36658E+00_JPRB ,&
&  0.36709E+00_JPRB, 0.36762E+00_JPRB, 0.36818E+00_JPRB, 0.36869E+00_JPRB, 0.36925E+00_JPRB, 0.36972E+00_JPRB, 0.37112E+00_JPRB ,&
&  0.37238E+00_JPRB, 0.37315E+00_JPRB, 0.37311E+00_JPRB, 0.37050E+00_JPRB, 0.36861E+00_JPRB, 0.36693E+00_JPRB, 0.36558E+00_JPRB ,&
&  0.36421E+00_JPRB, 0.36437E+00_JPRB, 0.36476E+00_JPRB, 0.36516E+00_JPRB, 0.36673E+00_JPRB, 0.36648E+00_JPRB, 0.36612E+00_JPRB ,&
&  0.36516E+00_JPRB, 0.36260E+00_JPRB, 0.36280E+00_JPRB, 0.36311E+00_JPRB, 0.36345E+00_JPRB, 0.36458E+00_JPRB, 0.36526E+00_JPRB ,&
&  0.36609E+00_JPRB, 0.36700E+00_JPRB, 0.36897E+00_JPRB, 0.37116E+00_JPRB, 0.37318E+00_JPRB, 0.37488E+00_JPRB, 0.37647E+00_JPRB ,&
&  0.37788E+00_JPRB, 0.37927E+00_JPRB, 0.38079E+00_JPRB, 0.38249E+00_JPRB, 0.38374E+00_JPRB, 0.38514E+00_JPRB, 0.38640E+00_JPRB ,&
&  0.38728E+00_JPRB, 0.38773E+00_JPRB, 0.38893E+00_JPRB, 0.39068E+00_JPRB, 0.39243E+00_JPRB, 0.39281E+00_JPRB, 0.39258E+00_JPRB ,&
&  0.39172E+00_JPRB, 0.39068E+00_JPRB, 0.38897E+00_JPRB, 0.38745E+00_JPRB, 0.38587E+00_JPRB, 0.38430E+00_JPRB, 0.38246E+00_JPRB ,&
&  0.38052E+00_JPRB, 0.37866E+00_JPRB, 0.37721E+00_JPRB, 0.37599E+00_JPRB, 0.37467E+00_JPRB, 0.37339E+00_JPRB, 0.37247E+00_JPRB ,&
&  0.37159E+00_JPRB, 0.37081E+00_JPRB, 0.36998E+00_JPRB, 0.36919E+00_JPRB, 0.36797E+00_JPRB, 0.36675E+00_JPRB, 0.36545E+00_JPRB ,&
&  0.36439E+00_JPRB, 0.36341E+00_JPRB, 0.36259E+00_JPRB, 0.36191E+00_JPRB, 0.36161E+00_JPRB, 0.36148E+00_JPRB, 0.36141E+00_JPRB ,&
&  0.36138E+00_JPRB, 0.36151E+00_JPRB, 0.36169E+00_JPRB, 0.36181E+00_JPRB, 0.36194E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.36455E+00_JPRB, 0.36511E+00_JPRB, 0.36571E+00_JPRB, 0.36609E+00_JPRB, 0.36679E+00_JPRB, 0.36742E+00_JPRB, 0.36800E+00_JPRB ,&
&  0.36855E+00_JPRB, 0.36886E+00_JPRB, 0.36908E+00_JPRB, 0.36930E+00_JPRB, 0.36946E+00_JPRB, 0.37000E+00_JPRB, 0.37056E+00_JPRB ,&
&  0.37111E+00_JPRB, 0.37150E+00_JPRB, 0.37257E+00_JPRB, 0.37367E+00_JPRB, 0.37453E+00_JPRB, 0.37543E+00_JPRB, 0.37359E+00_JPRB ,&
&  0.37245E+00_JPRB, 0.37196E+00_JPRB, 0.37123E+00_JPRB, 0.36895E+00_JPRB, 0.36793E+00_JPRB, 0.35893E+00_JPRB, 0.36171E+00_JPRB ,&
&  0.37255E+00_JPRB, 0.37178E+00_JPRB, 0.36954E+00_JPRB, 0.36567E+00_JPRB, 0.35212E+00_JPRB, 0.34620E+00_JPRB, 0.34534E+00_JPRB ,&
&  0.33622E+00_JPRB, 0.34389E+00_JPRB, 0.34589E+00_JPRB, 0.34665E+00_JPRB, 0.34854E+00_JPRB, 0.34711E+00_JPRB, 0.34733E+00_JPRB ,&
&  0.35319E+00_JPRB, 0.35307E+00_JPRB, 0.35561E+00_JPRB, 0.35873E+00_JPRB, 0.36218E+00_JPRB, 0.36546E+00_JPRB, 0.36941E+00_JPRB ,&
&  0.37276E+00_JPRB, 0.37526E+00_JPRB, 0.37700E+00_JPRB, 0.37768E+00_JPRB, 0.37851E+00_JPRB, 0.37942E+00_JPRB, 0.38118E+00_JPRB ,&
&  0.38410E+00_JPRB, 0.38707E+00_JPRB, 0.38876E+00_JPRB, 0.38835E+00_JPRB, 0.38894E+00_JPRB, 0.38933E+00_JPRB, 0.38963E+00_JPRB ,&
&  0.38930E+00_JPRB, 0.38853E+00_JPRB, 0.38648E+00_JPRB, 0.38584E+00_JPRB, 0.38570E+00_JPRB, 0.38507E+00_JPRB, 0.38404E+00_JPRB ,&
&  0.38237E+00_JPRB, 0.37995E+00_JPRB, 0.37784E+00_JPRB, 0.37667E+00_JPRB, 0.37527E+00_JPRB, 0.37399E+00_JPRB, 0.37321E+00_JPRB ,&
&  0.37209E+00_JPRB, 0.37115E+00_JPRB, 0.36968E+00_JPRB, 0.36768E+00_JPRB, 0.36577E+00_JPRB, 0.36419E+00_JPRB, 0.36263E+00_JPRB ,&
&  0.36184E+00_JPRB, 0.36148E+00_JPRB, 0.36135E+00_JPRB, 0.36117E+00_JPRB, 0.36130E+00_JPRB, 0.36172E+00_JPRB, 0.36216E+00_JPRB ,&
&  0.36262E+00_JPRB, 0.36305E+00_JPRB, 0.36334E+00_JPRB, 0.36374E+00_JPRB, 0.36394E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.36610E+00_JPRB, 0.36635E+00_JPRB, 0.36678E+00_JPRB, 0.36719E+00_JPRB, 0.36788E+00_JPRB, 0.36851E+00_JPRB, 0.36978E+00_JPRB ,&
&  0.37102E+00_JPRB, 0.37219E+00_JPRB, 0.37332E+00_JPRB, 0.37428E+00_JPRB, 0.37525E+00_JPRB, 0.37565E+00_JPRB, 0.37585E+00_JPRB ,&
&  0.37595E+00_JPRB, 0.37591E+00_JPRB, 0.37644E+00_JPRB, 0.37721E+00_JPRB, 0.37765E+00_JPRB, 0.37838E+00_JPRB, 0.37607E+00_JPRB ,&
&  0.37447E+00_JPRB, 0.37375E+00_JPRB, 0.36482E+00_JPRB, 0.36949E+00_JPRB, 0.36880E+00_JPRB, 0.36949E+00_JPRB, 0.36821E+00_JPRB ,&
&  0.35886E+00_JPRB, 0.35492E+00_JPRB, 0.34449E+00_JPRB, 0.33540E+00_JPRB, 0.33545E+00_JPRB, 0.32839E+00_JPRB, 0.32848E+00_JPRB ,&
&  0.32668E+00_JPRB, 0.32397E+00_JPRB, 0.32282E+00_JPRB, 0.32219E+00_JPRB, 0.32401E+00_JPRB, 0.32556E+00_JPRB, 0.32733E+00_JPRB ,&
&  0.33135E+00_JPRB, 0.33476E+00_JPRB, 0.33970E+00_JPRB, 0.33984E+00_JPRB, 0.34517E+00_JPRB, 0.35101E+00_JPRB, 0.35710E+00_JPRB ,&
&  0.36108E+00_JPRB, 0.36063E+00_JPRB, 0.36310E+00_JPRB, 0.36693E+00_JPRB, 0.37015E+00_JPRB, 0.37344E+00_JPRB, 0.37758E+00_JPRB ,&
&  0.38243E+00_JPRB, 0.38686E+00_JPRB, 0.38825E+00_JPRB, 0.38717E+00_JPRB, 0.38526E+00_JPRB, 0.38356E+00_JPRB, 0.38290E+00_JPRB ,&
&  0.38226E+00_JPRB, 0.38286E+00_JPRB, 0.38251E+00_JPRB, 0.38299E+00_JPRB, 0.38356E+00_JPRB, 0.38553E+00_JPRB, 0.38709E+00_JPRB ,&
&  0.38759E+00_JPRB, 0.38719E+00_JPRB, 0.38746E+00_JPRB, 0.38463E+00_JPRB, 0.38340E+00_JPRB, 0.38255E+00_JPRB, 0.37925E+00_JPRB ,&
&  0.37650E+00_JPRB, 0.37412E+00_JPRB, 0.37169E+00_JPRB, 0.36958E+00_JPRB, 0.36807E+00_JPRB, 0.36696E+00_JPRB, 0.36650E+00_JPRB ,&
&  0.36652E+00_JPRB, 0.36645E+00_JPRB, 0.36605E+00_JPRB, 0.36528E+00_JPRB, 0.36523E+00_JPRB, 0.36445E+00_JPRB, 0.36411E+00_JPRB ,&
&  0.36401E+00_JPRB, 0.36458E+00_JPRB, 0.36467E+00_JPRB, 0.36522E+00_JPRB, 0.36587E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.36560E+00_JPRB, 0.36752E+00_JPRB, 0.37049E+00_JPRB, 0.37309E+00_JPRB, 0.37542E+00_JPRB, 0.37739E+00_JPRB, 0.37845E+00_JPRB ,&
&  0.37921E+00_JPRB, 0.37951E+00_JPRB, 0.37982E+00_JPRB, 0.38026E+00_JPRB, 0.38102E+00_JPRB, 0.38165E+00_JPRB, 0.38249E+00_JPRB ,&
&  0.38284E+00_JPRB, 0.38299E+00_JPRB, 0.38287E+00_JPRB, 0.38269E+00_JPRB, 0.38264E+00_JPRB, 0.38310E+00_JPRB, 0.38237E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38059E+00_JPRB, 0.37401E+00_JPRB, 0.37630E+00_JPRB, 0.37730E+00_JPRB, 0.37835E+00_JPRB, 0.37510E+00_JPRB ,&
&  0.36658E+00_JPRB, 0.34609E+00_JPRB, 0.35529E+00_JPRB, 0.34735E+00_JPRB, 0.33908E+00_JPRB, 0.33079E+00_JPRB, 0.31965E+00_JPRB ,&
&  0.31295E+00_JPRB, 0.30455E+00_JPRB, 0.29839E+00_JPRB, 0.29638E+00_JPRB, 0.29642E+00_JPRB, 0.30009E+00_JPRB, 0.30506E+00_JPRB ,&
&  0.30775E+00_JPRB, 0.31522E+00_JPRB, 0.31891E+00_JPRB, 0.32325E+00_JPRB, 0.32876E+00_JPRB, 0.33588E+00_JPRB, 0.33896E+00_JPRB ,&
&  0.34426E+00_JPRB, 0.34738E+00_JPRB, 0.35306E+00_JPRB, 0.35993E+00_JPRB, 0.36969E+00_JPRB, 0.37556E+00_JPRB, 0.37907E+00_JPRB ,&
&  0.38202E+00_JPRB, 0.38292E+00_JPRB, 0.38288E+00_JPRB, 0.38281E+00_JPRB, 0.38332E+00_JPRB, 0.38415E+00_JPRB, 0.38531E+00_JPRB ,&
&  0.38681E+00_JPRB, 0.38884E+00_JPRB, 0.39108E+00_JPRB, 0.39373E+00_JPRB, 0.39545E+00_JPRB, 0.39921E+00_JPRB, 0.40336E+00_JPRB ,&
&  0.40652E+00_JPRB, 0.40771E+00_JPRB, 0.41006E+00_JPRB, 0.40622E+00_JPRB, 0.40440E+00_JPRB, 0.39791E+00_JPRB, 0.39320E+00_JPRB ,&
&  0.38995E+00_JPRB, 0.38542E+00_JPRB, 0.38169E+00_JPRB, 0.37873E+00_JPRB, 0.37677E+00_JPRB, 0.37476E+00_JPRB, 0.37369E+00_JPRB ,&
&  0.37250E+00_JPRB, 0.37095E+00_JPRB, 0.36955E+00_JPRB, 0.36772E+00_JPRB, 0.36658E+00_JPRB, 0.36534E+00_JPRB, 0.36496E+00_JPRB ,&
&  0.36489E+00_JPRB, 0.36501E+00_JPRB, 0.36483E+00_JPRB, 0.36530E+00_JPRB, 0.36557E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.37001E+00_JPRB, 0.37329E+00_JPRB, 0.37721E+00_JPRB, 0.38068E+00_JPRB, 0.38251E+00_JPRB, 0.38398E+00_JPRB, 0.38353E+00_JPRB ,&
&  0.38320E+00_JPRB, 0.38237E+00_JPRB, 0.38172E+00_JPRB, 0.38155E+00_JPRB, 0.38209E+00_JPRB, 0.38289E+00_JPRB, 0.38407E+00_JPRB ,&
&  0.38513E+00_JPRB, 0.38648E+00_JPRB, 0.38694E+00_JPRB, 0.38756E+00_JPRB, 0.38826E+00_JPRB, 0.39022E+00_JPRB, 0.39078E+00_JPRB ,&
&  0.39174E+00_JPRB, 0.38873E+00_JPRB, 0.38172E+00_JPRB, 0.37946E+00_JPRB, 0.38298E+00_JPRB, 0.37725E+00_JPRB, 0.37921E+00_JPRB ,&
&  0.38375E+00_JPRB, 0.38476E+00_JPRB, 0.37873E+00_JPRB, 0.37400E+00_JPRB, 0.36218E+00_JPRB, 0.31990E+00_JPRB, 0.31760E+00_JPRB ,&
&  0.30968E+00_JPRB, 0.29691E+00_JPRB, 0.29035E+00_JPRB, 0.28477E+00_JPRB, 0.28252E+00_JPRB, 0.28759E+00_JPRB, 0.29564E+00_JPRB ,&
&  0.29844E+00_JPRB, 0.30953E+00_JPRB, 0.31332E+00_JPRB, 0.31878E+00_JPRB, 0.32495E+00_JPRB, 0.33337E+00_JPRB, 0.33854E+00_JPRB ,&
&  0.34425E+00_JPRB, 0.35027E+00_JPRB, 0.35789E+00_JPRB, 0.36755E+00_JPRB, 0.37551E+00_JPRB, 0.37914E+00_JPRB, 0.37791E+00_JPRB ,&
&  0.37990E+00_JPRB, 0.37618E+00_JPRB, 0.37761E+00_JPRB, 0.37992E+00_JPRB, 0.38101E+00_JPRB, 0.38339E+00_JPRB, 0.38411E+00_JPRB ,&
&  0.38812E+00_JPRB, 0.39494E+00_JPRB, 0.40292E+00_JPRB, 0.40704E+00_JPRB, 0.41036E+00_JPRB, 0.41450E+00_JPRB, 0.42101E+00_JPRB ,&
&  0.42667E+00_JPRB, 0.43352E+00_JPRB, 0.43907E+00_JPRB, 0.43112E+00_JPRB, 0.42780E+00_JPRB, 0.41935E+00_JPRB, 0.41316E+00_JPRB ,&
&  0.40601E+00_JPRB, 0.40075E+00_JPRB, 0.39211E+00_JPRB, 0.38788E+00_JPRB, 0.38384E+00_JPRB, 0.38011E+00_JPRB, 0.37751E+00_JPRB ,&
&  0.37560E+00_JPRB, 0.37378E+00_JPRB, 0.37221E+00_JPRB, 0.37097E+00_JPRB, 0.36947E+00_JPRB, 0.36793E+00_JPRB, 0.36735E+00_JPRB ,&
&  0.36599E+00_JPRB, 0.36535E+00_JPRB, 0.36482E+00_JPRB, 0.36540E+00_JPRB, 0.36678E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.36915E+00_JPRB, 0.37374E+00_JPRB, 0.37793E+00_JPRB, 0.38357E+00_JPRB, 0.38498E+00_JPRB, 0.38731E+00_JPRB, 0.38477E+00_JPRB ,&
&  0.38287E+00_JPRB, 0.38196E+00_JPRB, 0.38156E+00_JPRB, 0.38260E+00_JPRB, 0.38395E+00_JPRB, 0.38546E+00_JPRB, 0.38764E+00_JPRB ,&
&  0.38955E+00_JPRB, 0.38759E+00_JPRB, 0.38822E+00_JPRB, 0.39117E+00_JPRB, 0.39215E+00_JPRB, 0.39433E+00_JPRB, 0.39550E+00_JPRB ,&
&  0.39529E+00_JPRB, 0.39280E+00_JPRB, 0.39039E+00_JPRB, 0.38517E+00_JPRB, 0.38344E+00_JPRB, 0.38544E+00_JPRB, 0.38801E+00_JPRB ,&
&  0.38122E+00_JPRB, 0.37788E+00_JPRB, 0.38684E+00_JPRB, 0.38879E+00_JPRB, 0.36801E+00_JPRB, 0.34810E+00_JPRB, 0.33219E+00_JPRB ,&
&  0.31682E+00_JPRB, 0.29704E+00_JPRB, 0.28976E+00_JPRB, 0.28498E+00_JPRB, 0.28674E+00_JPRB, 0.28990E+00_JPRB, 0.29723E+00_JPRB ,&
&  0.30329E+00_JPRB, 0.31089E+00_JPRB, 0.31880E+00_JPRB, 0.32860E+00_JPRB, 0.33561E+00_JPRB, 0.34303E+00_JPRB, 0.34957E+00_JPRB ,&
&  0.35656E+00_JPRB, 0.36442E+00_JPRB, 0.37351E+00_JPRB, 0.38438E+00_JPRB, 0.39106E+00_JPRB, 0.39561E+00_JPRB, 0.39637E+00_JPRB ,&
&  0.40684E+00_JPRB, 0.39876E+00_JPRB, 0.40048E+00_JPRB, 0.40114E+00_JPRB, 0.40101E+00_JPRB, 0.40227E+00_JPRB, 0.40314E+00_JPRB ,&
&  0.40921E+00_JPRB, 0.41854E+00_JPRB, 0.42130E+00_JPRB, 0.42315E+00_JPRB, 0.42464E+00_JPRB, 0.42666E+00_JPRB, 0.43459E+00_JPRB ,&
&  0.44261E+00_JPRB, 0.45911E+00_JPRB, 0.49935E+00_JPRB, 0.47715E+00_JPRB, 0.46315E+00_JPRB, 0.44916E+00_JPRB, 0.43555E+00_JPRB ,&
&  0.42330E+00_JPRB, 0.41232E+00_JPRB, 0.40145E+00_JPRB, 0.39164E+00_JPRB, 0.38507E+00_JPRB, 0.37839E+00_JPRB, 0.37787E+00_JPRB ,&
&  0.37714E+00_JPRB, 0.37555E+00_JPRB, 0.37388E+00_JPRB, 0.37218E+00_JPRB, 0.36898E+00_JPRB, 0.36648E+00_JPRB, 0.36540E+00_JPRB ,&
&  0.36345E+00_JPRB, 0.36393E+00_JPRB, 0.36390E+00_JPRB, 0.36502E+00_JPRB, 0.36617E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.36834E+00_JPRB, 0.37784E+00_JPRB, 0.38620E+00_JPRB, 0.39225E+00_JPRB, 0.39272E+00_JPRB, 0.39365E+00_JPRB, 0.38879E+00_JPRB ,&
&  0.38631E+00_JPRB, 0.38695E+00_JPRB, 0.38797E+00_JPRB, 0.38887E+00_JPRB, 0.38950E+00_JPRB, 0.38981E+00_JPRB, 0.39048E+00_JPRB ,&
&  0.39173E+00_JPRB, 0.39323E+00_JPRB, 0.39386E+00_JPRB, 0.39474E+00_JPRB, 0.39625E+00_JPRB, 0.39844E+00_JPRB, 0.40073E+00_JPRB ,&
&  0.40117E+00_JPRB, 0.39885E+00_JPRB, 0.39869E+00_JPRB, 0.39532E+00_JPRB, 0.39188E+00_JPRB, 0.38845E+00_JPRB, 0.38655E+00_JPRB ,&
&  0.39057E+00_JPRB, 0.39637E+00_JPRB, 0.39281E+00_JPRB, 0.38945E+00_JPRB, 0.38765E+00_JPRB, 0.36994E+00_JPRB, 0.34760E+00_JPRB ,&
&  0.32552E+00_JPRB, 0.30652E+00_JPRB, 0.29742E+00_JPRB, 0.29336E+00_JPRB, 0.29705E+00_JPRB, 0.30577E+00_JPRB, 0.31538E+00_JPRB ,&
&  0.32045E+00_JPRB, 0.33308E+00_JPRB, 0.34582E+00_JPRB, 0.35389E+00_JPRB, 0.36050E+00_JPRB, 0.36672E+00_JPRB, 0.37284E+00_JPRB ,&
&  0.38253E+00_JPRB, 0.39482E+00_JPRB, 0.41373E+00_JPRB, 0.43159E+00_JPRB, 0.44406E+00_JPRB, 0.44616E+00_JPRB, 0.46652E+00_JPRB ,&
&  0.52086E+00_JPRB, 0.48405E+00_JPRB, 0.46422E+00_JPRB, 0.45478E+00_JPRB, 0.44600E+00_JPRB, 0.44245E+00_JPRB, 0.44429E+00_JPRB ,&
&  0.44940E+00_JPRB, 0.45430E+00_JPRB, 0.45397E+00_JPRB, 0.45177E+00_JPRB, 0.45185E+00_JPRB, 0.45500E+00_JPRB, 0.46360E+00_JPRB ,&
&  0.47392E+00_JPRB, 0.50458E+00_JPRB, 0.58647E+00_JPRB, 0.51421E+00_JPRB, 0.47704E+00_JPRB, 0.45905E+00_JPRB, 0.44350E+00_JPRB ,&
&  0.43084E+00_JPRB, 0.42003E+00_JPRB, 0.40862E+00_JPRB, 0.39407E+00_JPRB, 0.38384E+00_JPRB, 0.37513E+00_JPRB, 0.37249E+00_JPRB ,&
&  0.36939E+00_JPRB, 0.36617E+00_JPRB, 0.36574E+00_JPRB, 0.36412E+00_JPRB, 0.36204E+00_JPRB, 0.36325E+00_JPRB, 0.36392E+00_JPRB ,&
&  0.36150E+00_JPRB, 0.36059E+00_JPRB, 0.36068E+00_JPRB, 0.36116E+00_JPRB, 0.36271E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.37785E+00_JPRB, 0.38642E+00_JPRB, 0.39898E+00_JPRB, 0.41701E+00_JPRB, 0.42653E+00_JPRB, 0.42048E+00_JPRB, 0.40576E+00_JPRB ,&
&  0.39732E+00_JPRB, 0.39338E+00_JPRB, 0.39639E+00_JPRB, 0.40036E+00_JPRB, 0.40206E+00_JPRB, 0.40179E+00_JPRB, 0.40030E+00_JPRB ,&
&  0.39705E+00_JPRB, 0.39535E+00_JPRB, 0.39529E+00_JPRB, 0.39645E+00_JPRB, 0.39834E+00_JPRB, 0.40019E+00_JPRB, 0.40309E+00_JPRB ,&
&  0.40532E+00_JPRB, 0.40467E+00_JPRB, 0.40311E+00_JPRB, 0.40005E+00_JPRB, 0.39914E+00_JPRB, 0.40085E+00_JPRB, 0.40032E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39827E+00_JPRB, 0.40204E+00_JPRB, 0.39689E+00_JPRB, 0.39493E+00_JPRB, 0.38271E+00_JPRB, 0.36093E+00_JPRB ,&
&  0.34129E+00_JPRB, 0.32306E+00_JPRB, 0.31742E+00_JPRB, 0.32000E+00_JPRB, 0.32818E+00_JPRB, 0.33838E+00_JPRB, 0.35041E+00_JPRB ,&
&  0.36113E+00_JPRB, 0.37061E+00_JPRB, 0.37769E+00_JPRB, 0.38093E+00_JPRB, 0.38518E+00_JPRB, 0.39280E+00_JPRB, 0.40810E+00_JPRB ,&
&  0.43600E+00_JPRB, 0.47049E+00_JPRB, 0.50505E+00_JPRB, 0.48332E+00_JPRB, 0.48157E+00_JPRB, 0.49295E+00_JPRB, 0.50876E+00_JPRB ,&
&  0.52179E+00_JPRB, 0.51523E+00_JPRB, 0.50904E+00_JPRB, 0.52181E+00_JPRB, 0.52872E+00_JPRB, 0.53668E+00_JPRB, 0.53164E+00_JPRB ,&
&  0.52023E+00_JPRB, 0.50806E+00_JPRB, 0.50048E+00_JPRB, 0.49960E+00_JPRB, 0.50702E+00_JPRB, 0.51429E+00_JPRB, 0.51474E+00_JPRB ,&
&  0.50406E+00_JPRB, 0.48490E+00_JPRB, 0.46289E+00_JPRB, 0.44010E+00_JPRB, 0.43479E+00_JPRB, 0.43615E+00_JPRB, 0.43587E+00_JPRB ,&
&  0.42856E+00_JPRB, 0.41913E+00_JPRB, 0.40993E+00_JPRB, 0.39533E+00_JPRB, 0.38344E+00_JPRB, 0.37088E+00_JPRB, 0.36352E+00_JPRB ,&
&  0.35836E+00_JPRB, 0.35554E+00_JPRB, 0.35768E+00_JPRB, 0.35325E+00_JPRB, 0.35513E+00_JPRB, 0.35934E+00_JPRB, 0.35836E+00_JPRB ,&
&  0.35749E+00_JPRB, 0.35796E+00_JPRB, 0.35631E+00_JPRB, 0.35900E+00_JPRB, 0.36480E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.40143E+00_JPRB, 0.42523E+00_JPRB, 0.44648E+00_JPRB, 0.46290E+00_JPRB, 0.46613E+00_JPRB, 0.46541E+00_JPRB, 0.45284E+00_JPRB ,&
&  0.45366E+00_JPRB, 0.43147E+00_JPRB, 0.41767E+00_JPRB, 0.40176E+00_JPRB, 0.38937E+00_JPRB, 0.38111E+00_JPRB, 0.38154E+00_JPRB ,&
&  0.38541E+00_JPRB, 0.39016E+00_JPRB, 0.39313E+00_JPRB, 0.39371E+00_JPRB, 0.39564E+00_JPRB, 0.39877E+00_JPRB, 0.40300E+00_JPRB ,&
&  0.40759E+00_JPRB, 0.40979E+00_JPRB, 0.41120E+00_JPRB, 0.41156E+00_JPRB, 0.41152E+00_JPRB, 0.41090E+00_JPRB, 0.40714E+00_JPRB ,&
&  0.40697E+00_JPRB, 0.40456E+00_JPRB, 0.40497E+00_JPRB, 0.40261E+00_JPRB, 0.40152E+00_JPRB, 0.39284E+00_JPRB, 0.36896E+00_JPRB ,&
&  0.35383E+00_JPRB, 0.34670E+00_JPRB, 0.35039E+00_JPRB, 0.36245E+00_JPRB, 0.37111E+00_JPRB, 0.38085E+00_JPRB, 0.38730E+00_JPRB ,&
&  0.38625E+00_JPRB, 0.38694E+00_JPRB, 0.39607E+00_JPRB, 0.40564E+00_JPRB, 0.41283E+00_JPRB, 0.42443E+00_JPRB, 0.43792E+00_JPRB ,&
&  0.46242E+00_JPRB, 0.50037E+00_JPRB, 0.56106E+00_JPRB, 0.57018E+00_JPRB, 0.62253E+00_JPRB, 0.74196E+00_JPRB, 0.64281E+00_JPRB ,&
&  0.63492E+00_JPRB, 0.63248E+00_JPRB, 0.64123E+00_JPRB, 0.66667E+00_JPRB, 0.64613E+00_JPRB, 0.63344E+00_JPRB, 0.62295E+00_JPRB ,&
&  0.59498E+00_JPRB, 0.58223E+00_JPRB, 0.57324E+00_JPRB, 0.58342E+00_JPRB, 0.58077E+00_JPRB, 0.57803E+00_JPRB, 0.57254E+00_JPRB ,&
&  0.54997E+00_JPRB, 0.52072E+00_JPRB, 0.49032E+00_JPRB, 0.46197E+00_JPRB, 0.44762E+00_JPRB, 0.43928E+00_JPRB, 0.43371E+00_JPRB ,&
&  0.42656E+00_JPRB, 0.42084E+00_JPRB, 0.41589E+00_JPRB, 0.40631E+00_JPRB, 0.39198E+00_JPRB, 0.37616E+00_JPRB, 0.36912E+00_JPRB ,&
&  0.36246E+00_JPRB, 0.35731E+00_JPRB, 0.35518E+00_JPRB, 0.35350E+00_JPRB, 0.35685E+00_JPRB, 0.35957E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36355E+00_JPRB, 0.36111E+00_JPRB, 0.35907E+00_JPRB, 0.36358E+00_JPRB, 0.38014E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.43916E+00_JPRB, 0.47056E+00_JPRB, 0.48505E+00_JPRB, 0.49399E+00_JPRB, 0.50178E+00_JPRB, 0.50018E+00_JPRB, 0.47477E+00_JPRB ,&
&  0.47402E+00_JPRB, 0.42622E+00_JPRB, 0.40978E+00_JPRB, 0.39528E+00_JPRB, 0.38871E+00_JPRB, 0.38193E+00_JPRB, 0.37329E+00_JPRB ,&
&  0.37458E+00_JPRB, 0.38249E+00_JPRB, 0.39186E+00_JPRB, 0.39616E+00_JPRB, 0.39957E+00_JPRB, 0.40199E+00_JPRB, 0.40428E+00_JPRB ,&
&  0.40822E+00_JPRB, 0.41321E+00_JPRB, 0.41813E+00_JPRB, 0.41959E+00_JPRB, 0.41897E+00_JPRB, 0.41804E+00_JPRB, 0.41640E+00_JPRB ,&
&  0.41500E+00_JPRB, 0.40877E+00_JPRB, 0.40858E+00_JPRB, 0.40840E+00_JPRB, 0.40518E+00_JPRB, 0.40460E+00_JPRB, 0.39344E+00_JPRB ,&
&  0.37071E+00_JPRB, 0.37790E+00_JPRB, 0.38903E+00_JPRB, 0.39908E+00_JPRB, 0.40787E+00_JPRB, 0.41539E+00_JPRB, 0.41983E+00_JPRB ,&
&  0.42159E+00_JPRB, 0.42231E+00_JPRB, 0.42543E+00_JPRB, 0.43057E+00_JPRB, 0.43659E+00_JPRB, 0.44429E+00_JPRB, 0.45122E+00_JPRB ,&
&  0.46196E+00_JPRB, 0.45095E+00_JPRB, 0.49393E+00_JPRB, 0.60774E+00_JPRB, 0.86372E+00_JPRB, 0.82926E+00_JPRB, 0.82538E+00_JPRB ,&
&  0.84736E+00_JPRB, 0.80812E+00_JPRB, 0.79071E+00_JPRB, 0.77789E+00_JPRB, 0.75804E+00_JPRB, 0.73956E+00_JPRB, 0.73646E+00_JPRB ,&
&  0.71008E+00_JPRB, 0.71525E+00_JPRB, 0.70014E+00_JPRB, 0.69232E+00_JPRB, 0.67357E+00_JPRB, 0.66513E+00_JPRB, 0.65048E+00_JPRB ,&
&  0.62554E+00_JPRB, 0.60222E+00_JPRB, 0.56299E+00_JPRB, 0.51870E+00_JPRB, 0.48440E+00_JPRB, 0.46630E+00_JPRB, 0.45030E+00_JPRB ,&
&  0.43870E+00_JPRB, 0.43206E+00_JPRB, 0.42445E+00_JPRB, 0.41992E+00_JPRB, 0.41058E+00_JPRB, 0.40085E+00_JPRB, 0.39582E+00_JPRB ,&
&  0.38308E+00_JPRB, 0.37523E+00_JPRB, 0.36915E+00_JPRB, 0.36681E+00_JPRB, 0.37216E+00_JPRB, 0.37952E+00_JPRB, 0.38606E+00_JPRB ,&
&  0.39311E+00_JPRB, 0.39112E+00_JPRB, 0.39140E+00_JPRB, 0.39718E+00_JPRB, 0.41160E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.47144E+00_JPRB, 0.49473E+00_JPRB, 0.50588E+00_JPRB, 0.51777E+00_JPRB, 0.53147E+00_JPRB, 0.52957E+00_JPRB, 0.52877E+00_JPRB ,&
&  0.55545E+00_JPRB, 0.52106E+00_JPRB, 0.55839E+00_JPRB, 0.45082E+00_JPRB, 0.40945E+00_JPRB, 0.40023E+00_JPRB, 0.38870E+00_JPRB ,&
&  0.38713E+00_JPRB, 0.39233E+00_JPRB, 0.39728E+00_JPRB, 0.40115E+00_JPRB, 0.40669E+00_JPRB, 0.40668E+00_JPRB, 0.40672E+00_JPRB ,&
&  0.40903E+00_JPRB, 0.41494E+00_JPRB, 0.42594E+00_JPRB, 0.42891E+00_JPRB, 0.43197E+00_JPRB, 0.43583E+00_JPRB, 0.43278E+00_JPRB ,&
&  0.42382E+00_JPRB, 0.42227E+00_JPRB, 0.42276E+00_JPRB, 0.41687E+00_JPRB, 0.41102E+00_JPRB, 0.41177E+00_JPRB, 0.42264E+00_JPRB ,&
&  0.42467E+00_JPRB, 0.41313E+00_JPRB, 0.42655E+00_JPRB, 0.43389E+00_JPRB, 0.43862E+00_JPRB, 0.44088E+00_JPRB, 0.44169E+00_JPRB ,&
&  0.44356E+00_JPRB, 0.44439E+00_JPRB, 0.44851E+00_JPRB, 0.45145E+00_JPRB, 0.45550E+00_JPRB, 0.46288E+00_JPRB, 0.46490E+00_JPRB ,&
&  0.48314E+00_JPRB, 0.47436E+00_JPRB, 0.58731E+00_JPRB, 0.70136E+00_JPRB, 0.94411E+00_JPRB, 0.10569E+01_JPRB, 0.10170E+01_JPRB ,&
&  0.10809E+01_JPRB, 0.99426E+00_JPRB, 0.10043E+01_JPRB, 0.96000E+00_JPRB, 0.92023E+00_JPRB, 0.87617E+00_JPRB, 0.83256E+00_JPRB ,&
&  0.82161E+00_JPRB, 0.82509E+00_JPRB, 0.84868E+00_JPRB, 0.82331E+00_JPRB, 0.77775E+00_JPRB, 0.76298E+00_JPRB, 0.75456E+00_JPRB ,&
&  0.74936E+00_JPRB, 0.73751E+00_JPRB, 0.70006E+00_JPRB, 0.60641E+00_JPRB, 0.56057E+00_JPRB, 0.52427E+00_JPRB, 0.48103E+00_JPRB ,&
&  0.45986E+00_JPRB, 0.43839E+00_JPRB, 0.42297E+00_JPRB, 0.41884E+00_JPRB, 0.41211E+00_JPRB, 0.40655E+00_JPRB, 0.40497E+00_JPRB ,&
&  0.39334E+00_JPRB, 0.39143E+00_JPRB, 0.39079E+00_JPRB, 0.39481E+00_JPRB, 0.39892E+00_JPRB, 0.40562E+00_JPRB, 0.41469E+00_JPRB ,&
&  0.41509E+00_JPRB, 0.41477E+00_JPRB, 0.42439E+00_JPRB, 0.43237E+00_JPRB, 0.44825E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.48785E+00_JPRB, 0.50770E+00_JPRB, 0.52463E+00_JPRB, 0.54051E+00_JPRB, 0.56149E+00_JPRB, 0.58430E+00_JPRB, 0.58431E+00_JPRB ,&
&  0.59873E+00_JPRB, 0.60343E+00_JPRB, 0.60005E+00_JPRB, 0.58820E+00_JPRB, 0.56438E+00_JPRB, 0.49506E+00_JPRB, 0.40909E+00_JPRB ,&
&  0.38474E+00_JPRB, 0.38984E+00_JPRB, 0.40018E+00_JPRB, 0.40443E+00_JPRB, 0.40617E+00_JPRB, 0.40882E+00_JPRB, 0.40836E+00_JPRB ,&
&  0.41160E+00_JPRB, 0.41696E+00_JPRB, 0.42742E+00_JPRB, 0.44136E+00_JPRB, 0.45376E+00_JPRB, 0.45725E+00_JPRB, 0.45455E+00_JPRB ,&
&  0.44266E+00_JPRB, 0.43686E+00_JPRB, 0.42940E+00_JPRB, 0.41781E+00_JPRB, 0.42358E+00_JPRB, 0.43163E+00_JPRB, 0.44901E+00_JPRB ,&
&  0.46466E+00_JPRB, 0.47622E+00_JPRB, 0.48105E+00_JPRB, 0.48647E+00_JPRB, 0.48943E+00_JPRB, 0.48667E+00_JPRB, 0.48044E+00_JPRB ,&
&  0.47512E+00_JPRB, 0.47143E+00_JPRB, 0.46914E+00_JPRB, 0.47141E+00_JPRB, 0.47230E+00_JPRB, 0.48777E+00_JPRB, 0.50852E+00_JPRB ,&
&  0.53569E+00_JPRB, 0.63130E+00_JPRB, 0.77405E+00_JPRB, 0.79835E+00_JPRB, 0.10356E+01_JPRB, 0.11203E+01_JPRB, 0.11344E+01_JPRB ,&
&  0.11192E+01_JPRB, 0.10300E+01_JPRB, 0.11193E+01_JPRB, 0.11228E+01_JPRB, 0.11122E+01_JPRB, 0.10618E+01_JPRB, 0.97693E+00_JPRB ,&
&  0.93391E+00_JPRB, 0.99585E+00_JPRB, 0.10114E+01_JPRB, 0.99462E+00_JPRB, 0.94947E+00_JPRB, 0.94297E+00_JPRB, 0.93882E+00_JPRB ,&
&  0.95496E+00_JPRB, 0.96173E+00_JPRB, 0.90662E+00_JPRB, 0.80641E+00_JPRB, 0.69811E+00_JPRB, 0.61013E+00_JPRB, 0.54220E+00_JPRB ,&
&  0.48943E+00_JPRB, 0.43966E+00_JPRB, 0.42183E+00_JPRB, 0.41668E+00_JPRB, 0.41581E+00_JPRB, 0.40814E+00_JPRB, 0.40581E+00_JPRB ,&
&  0.40562E+00_JPRB, 0.40837E+00_JPRB, 0.41380E+00_JPRB, 0.41814E+00_JPRB, 0.42747E+00_JPRB, 0.43429E+00_JPRB, 0.42621E+00_JPRB ,&
&  0.42010E+00_JPRB, 0.42812E+00_JPRB, 0.43686E+00_JPRB, 0.44884E+00_JPRB, 0.46717E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.50939E+00_JPRB, 0.53133E+00_JPRB, 0.54844E+00_JPRB, 0.56356E+00_JPRB, 0.58825E+00_JPRB, 0.62462E+00_JPRB, 0.64159E+00_JPRB ,&
&  0.64252E+00_JPRB, 0.64062E+00_JPRB, 0.63628E+00_JPRB, 0.62530E+00_JPRB, 0.60441E+00_JPRB, 0.57208E+00_JPRB, 0.49849E+00_JPRB ,&
&  0.41809E+00_JPRB, 0.41635E+00_JPRB, 0.40466E+00_JPRB, 0.41084E+00_JPRB, 0.41572E+00_JPRB, 0.41273E+00_JPRB, 0.40824E+00_JPRB ,&
&  0.41329E+00_JPRB, 0.41873E+00_JPRB, 0.43074E+00_JPRB, 0.44547E+00_JPRB, 0.44743E+00_JPRB, 0.45064E+00_JPRB, 0.45912E+00_JPRB ,&
&  0.44942E+00_JPRB, 0.43550E+00_JPRB, 0.42553E+00_JPRB, 0.42299E+00_JPRB, 0.43781E+00_JPRB, 0.46228E+00_JPRB, 0.48516E+00_JPRB ,&
&  0.50214E+00_JPRB, 0.51293E+00_JPRB, 0.52148E+00_JPRB, 0.52823E+00_JPRB, 0.53186E+00_JPRB, 0.52862E+00_JPRB, 0.51851E+00_JPRB ,&
&  0.50585E+00_JPRB, 0.49773E+00_JPRB, 0.49327E+00_JPRB, 0.49779E+00_JPRB, 0.50565E+00_JPRB, 0.54593E+00_JPRB, 0.59590E+00_JPRB ,&
&  0.66554E+00_JPRB, 0.77790E+00_JPRB, 0.87247E+00_JPRB, 0.98897E+00_JPRB, 0.11570E+01_JPRB, 0.12480E+01_JPRB, 0.12575E+01_JPRB ,&
&  0.12614E+01_JPRB, 0.11939E+01_JPRB, 0.13488E+01_JPRB, 0.13348E+01_JPRB, 0.12469E+01_JPRB, 0.12147E+01_JPRB, 0.11170E+01_JPRB ,&
&  0.10424E+01_JPRB, 0.99518E+00_JPRB, 0.10531E+01_JPRB, 0.10940E+01_JPRB, 0.11184E+01_JPRB, 0.11537E+01_JPRB, 0.11552E+01_JPRB ,&
&  0.11603E+01_JPRB, 0.11443E+01_JPRB, 0.98051E+00_JPRB, 0.81810E+00_JPRB, 0.66632E+00_JPRB, 0.63896E+00_JPRB, 0.57214E+00_JPRB ,&
&  0.50262E+00_JPRB, 0.43940E+00_JPRB, 0.43220E+00_JPRB, 0.42721E+00_JPRB, 0.43291E+00_JPRB, 0.43157E+00_JPRB, 0.42449E+00_JPRB ,&
&  0.42832E+00_JPRB, 0.43758E+00_JPRB, 0.43740E+00_JPRB, 0.44262E+00_JPRB, 0.45360E+00_JPRB, 0.45671E+00_JPRB, 0.44373E+00_JPRB ,&
&  0.44254E+00_JPRB, 0.44817E+00_JPRB, 0.44739E+00_JPRB, 0.45928E+00_JPRB, 0.48156E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.53293E+00_JPRB, 0.55248E+00_JPRB, 0.56818E+00_JPRB, 0.59083E+00_JPRB, 0.62539E+00_JPRB, 0.66402E+00_JPRB, 0.67774E+00_JPRB ,&
&  0.67615E+00_JPRB, 0.67051E+00_JPRB, 0.66068E+00_JPRB, 0.64443E+00_JPRB, 0.62711E+00_JPRB, 0.59419E+00_JPRB, 0.56148E+00_JPRB ,&
&  0.45910E+00_JPRB, 0.43654E+00_JPRB, 0.39789E+00_JPRB, 0.40154E+00_JPRB, 0.44480E+00_JPRB, 0.43297E+00_JPRB, 0.43020E+00_JPRB ,&
&  0.42410E+00_JPRB, 0.43033E+00_JPRB, 0.43726E+00_JPRB, 0.44594E+00_JPRB, 0.43892E+00_JPRB, 0.44686E+00_JPRB, 0.46173E+00_JPRB ,&
&  0.45559E+00_JPRB, 0.44490E+00_JPRB, 0.44508E+00_JPRB, 0.45183E+00_JPRB, 0.47143E+00_JPRB, 0.50181E+00_JPRB, 0.53859E+00_JPRB ,&
&  0.55707E+00_JPRB, 0.57307E+00_JPRB, 0.58185E+00_JPRB, 0.58787E+00_JPRB, 0.58914E+00_JPRB, 0.58383E+00_JPRB, 0.57274E+00_JPRB ,&
&  0.55741E+00_JPRB, 0.54446E+00_JPRB, 0.54522E+00_JPRB, 0.55782E+00_JPRB, 0.55990E+00_JPRB, 0.61996E+00_JPRB, 0.74756E+00_JPRB ,&
&  0.90704E+00_JPRB, 0.97527E+00_JPRB, 0.10805E+01_JPRB, 0.12894E+01_JPRB, 0.14257E+01_JPRB, 0.14866E+01_JPRB, 0.14556E+01_JPRB ,&
&  0.14159E+01_JPRB, 0.13686E+01_JPRB, 0.13470E+01_JPRB, 0.13531E+01_JPRB, 0.12921E+01_JPRB, 0.12973E+01_JPRB, 0.12239E+01_JPRB ,&
&  0.11150E+01_JPRB, 0.10950E+01_JPRB, 0.11569E+01_JPRB, 0.12063E+01_JPRB, 0.12275E+01_JPRB, 0.12678E+01_JPRB, 0.12939E+01_JPRB ,&
&  0.12636E+01_JPRB, 0.11080E+01_JPRB, 0.83287E+00_JPRB, 0.74176E+00_JPRB, 0.57447E+00_JPRB, 0.55820E+00_JPRB, 0.60388E+00_JPRB ,&
&  0.58547E+00_JPRB, 0.52194E+00_JPRB, 0.47893E+00_JPRB, 0.45959E+00_JPRB, 0.45921E+00_JPRB, 0.49043E+00_JPRB, 0.47790E+00_JPRB ,&
&  0.45827E+00_JPRB, 0.46668E+00_JPRB, 0.47584E+00_JPRB, 0.48485E+00_JPRB, 0.49745E+00_JPRB, 0.50230E+00_JPRB, 0.48942E+00_JPRB ,&
&  0.49734E+00_JPRB, 0.49027E+00_JPRB, 0.48493E+00_JPRB, 0.49136E+00_JPRB, 0.50914E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.57073E+00_JPRB, 0.58269E+00_JPRB, 0.60105E+00_JPRB, 0.62801E+00_JPRB, 0.66844E+00_JPRB, 0.70024E+00_JPRB, 0.70411E+00_JPRB ,&
&  0.69797E+00_JPRB, 0.68973E+00_JPRB, 0.67543E+00_JPRB, 0.65957E+00_JPRB, 0.63300E+00_JPRB, 0.59213E+00_JPRB, 0.55527E+00_JPRB ,&
&  0.55336E+00_JPRB, 0.49592E+00_JPRB, 0.44280E+00_JPRB, 0.40567E+00_JPRB, 0.43191E+00_JPRB, 0.43117E+00_JPRB, 0.44424E+00_JPRB ,&
&  0.44043E+00_JPRB, 0.45700E+00_JPRB, 0.46042E+00_JPRB, 0.45520E+00_JPRB, 0.44740E+00_JPRB, 0.46831E+00_JPRB, 0.49585E+00_JPRB ,&
&  0.51411E+00_JPRB, 0.51957E+00_JPRB, 0.52223E+00_JPRB, 0.54121E+00_JPRB, 0.58119E+00_JPRB, 0.56274E+00_JPRB, 0.60115E+00_JPRB ,&
&  0.62006E+00_JPRB, 0.63338E+00_JPRB, 0.64303E+00_JPRB, 0.65669E+00_JPRB, 0.66100E+00_JPRB, 0.66624E+00_JPRB, 0.67221E+00_JPRB ,&
&  0.67969E+00_JPRB, 0.67844E+00_JPRB, 0.65075E+00_JPRB, 0.64418E+00_JPRB, 0.65658E+00_JPRB, 0.76314E+00_JPRB, 0.83984E+00_JPRB ,&
&  0.93633E+00_JPRB, 0.95443E+00_JPRB, 0.10088E+01_JPRB, 0.12221E+01_JPRB, 0.13691E+01_JPRB, 0.14165E+01_JPRB, 0.14561E+01_JPRB ,&
&  0.14642E+01_JPRB, 0.14594E+01_JPRB, 0.15005E+01_JPRB, 0.15122E+01_JPRB, 0.15030E+01_JPRB, 0.14789E+01_JPRB, 0.14385E+01_JPRB ,&
&  0.13466E+01_JPRB, 0.13152E+01_JPRB, 0.13566E+01_JPRB, 0.13294E+01_JPRB, 0.13074E+01_JPRB, 0.12473E+01_JPRB, 0.13215E+01_JPRB ,&
&  0.13019E+01_JPRB, 0.79618E+00_JPRB, 0.64323E+00_JPRB, 0.71534E+00_JPRB, 0.60284E+00_JPRB, 0.57966E+00_JPRB, 0.60454E+00_JPRB ,&
&  0.57231E+00_JPRB, 0.54095E+00_JPRB, 0.52806E+00_JPRB, 0.50409E+00_JPRB, 0.50620E+00_JPRB, 0.57227E+00_JPRB, 0.56368E+00_JPRB ,&
&  0.54360E+00_JPRB, 0.53820E+00_JPRB, 0.55771E+00_JPRB, 0.56002E+00_JPRB, 0.56241E+00_JPRB, 0.57229E+00_JPRB, 0.56821E+00_JPRB ,&
&  0.55228E+00_JPRB, 0.54166E+00_JPRB, 0.53643E+00_JPRB, 0.54213E+00_JPRB, 0.55724E+00_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.63841E+00_JPRB, 0.64554E+00_JPRB, 0.65685E+00_JPRB, 0.67976E+00_JPRB, 0.71482E+00_JPRB, 0.73785E+00_JPRB, 0.73157E+00_JPRB ,&
&  0.72233E+00_JPRB, 0.70629E+00_JPRB, 0.69070E+00_JPRB, 0.66841E+00_JPRB, 0.63256E+00_JPRB, 0.59102E+00_JPRB, 0.55353E+00_JPRB ,&
&  0.53198E+00_JPRB, 0.51368E+00_JPRB, 0.44660E+00_JPRB, 0.42103E+00_JPRB, 0.40001E+00_JPRB, 0.41550E+00_JPRB, 0.43578E+00_JPRB ,&
&  0.45779E+00_JPRB, 0.46686E+00_JPRB, 0.47334E+00_JPRB, 0.47804E+00_JPRB, 0.48592E+00_JPRB, 0.52578E+00_JPRB, 0.58512E+00_JPRB ,&
&  0.62058E+00_JPRB, 0.63966E+00_JPRB, 0.61474E+00_JPRB, 0.66314E+00_JPRB, 0.66877E+00_JPRB, 0.64252E+00_JPRB, 0.69484E+00_JPRB ,&
&  0.71962E+00_JPRB, 0.73738E+00_JPRB, 0.73927E+00_JPRB, 0.75448E+00_JPRB, 0.76320E+00_JPRB, 0.74430E+00_JPRB, 0.72091E+00_JPRB ,&
&  0.69093E+00_JPRB, 0.67430E+00_JPRB, 0.67181E+00_JPRB, 0.66644E+00_JPRB, 0.67857E+00_JPRB, 0.74514E+00_JPRB, 0.76140E+00_JPRB ,&
&  0.77160E+00_JPRB, 0.65534E+00_JPRB, 0.60671E+00_JPRB, 0.83387E+00_JPRB, 0.11805E+01_JPRB, 0.13497E+01_JPRB, 0.13164E+01_JPRB ,&
&  0.14437E+01_JPRB, 0.15368E+01_JPRB, 0.16751E+01_JPRB, 0.17492E+01_JPRB, 0.17428E+01_JPRB, 0.16797E+01_JPRB, 0.16169E+01_JPRB ,&
&  0.15564E+01_JPRB, 0.15241E+01_JPRB, 0.15045E+01_JPRB, 0.14739E+01_JPRB, 0.14352E+01_JPRB, 0.14471E+01_JPRB, 0.15097E+01_JPRB ,&
&  0.13885E+01_JPRB, 0.11779E+01_JPRB, 0.80453E+00_JPRB, 0.78970E+00_JPRB, 0.62437E+00_JPRB, 0.59673E+00_JPRB, 0.62742E+00_JPRB ,&
&  0.59405E+00_JPRB, 0.59442E+00_JPRB, 0.57979E+00_JPRB, 0.55637E+00_JPRB, 0.59267E+00_JPRB, 0.66784E+00_JPRB, 0.69519E+00_JPRB ,&
&  0.70940E+00_JPRB, 0.66292E+00_JPRB, 0.69970E+00_JPRB, 0.70228E+00_JPRB, 0.70198E+00_JPRB, 0.69779E+00_JPRB, 0.68530E+00_JPRB ,&
&  0.65954E+00_JPRB, 0.63460E+00_JPRB, 0.62155E+00_JPRB, 0.62119E+00_JPRB, 0.62741E+00_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.68839E+00_JPRB, 0.68196E+00_JPRB, 0.68264E+00_JPRB, 0.69876E+00_JPRB, 0.72584E+00_JPRB, 0.73932E+00_JPRB, 0.72783E+00_JPRB ,&
&  0.71857E+00_JPRB, 0.70539E+00_JPRB, 0.69639E+00_JPRB, 0.66267E+00_JPRB, 0.62902E+00_JPRB, 0.58609E+00_JPRB, 0.54711E+00_JPRB ,&
&  0.52404E+00_JPRB, 0.48443E+00_JPRB, 0.42535E+00_JPRB, 0.40114E+00_JPRB, 0.38270E+00_JPRB, 0.38991E+00_JPRB, 0.41566E+00_JPRB ,&
&  0.44773E+00_JPRB, 0.47066E+00_JPRB, 0.51339E+00_JPRB, 0.53314E+00_JPRB, 0.56677E+00_JPRB, 0.63602E+00_JPRB, 0.71858E+00_JPRB ,&
&  0.83317E+00_JPRB, 0.72057E+00_JPRB, 0.74241E+00_JPRB, 0.73596E+00_JPRB, 0.75666E+00_JPRB, 0.75906E+00_JPRB, 0.75487E+00_JPRB ,&
&  0.76233E+00_JPRB, 0.76835E+00_JPRB, 0.78461E+00_JPRB, 0.78065E+00_JPRB, 0.76367E+00_JPRB, 0.74464E+00_JPRB, 0.73127E+00_JPRB ,&
&  0.72848E+00_JPRB, 0.71827E+00_JPRB, 0.69349E+00_JPRB, 0.66518E+00_JPRB, 0.73260E+00_JPRB, 0.78818E+00_JPRB, 0.74868E+00_JPRB ,&
&  0.61942E+00_JPRB, 0.63195E+00_JPRB, 0.82340E+00_JPRB, 0.10509E+01_JPRB, 0.12793E+01_JPRB, 0.14868E+01_JPRB, 0.16162E+01_JPRB ,&
&  0.17543E+01_JPRB, 0.18482E+01_JPRB, 0.19412E+01_JPRB, 0.17406E+01_JPRB, 0.18553E+01_JPRB, 0.17941E+01_JPRB, 0.17036E+01_JPRB ,&
&  0.16072E+01_JPRB, 0.15811E+01_JPRB, 0.15151E+01_JPRB, 0.15354E+01_JPRB, 0.16157E+01_JPRB, 0.17365E+01_JPRB, 0.15348E+01_JPRB ,&
&  0.12805E+01_JPRB, 0.13393E+01_JPRB, 0.11460E+01_JPRB, 0.91305E+00_JPRB, 0.73096E+00_JPRB, 0.70663E+00_JPRB, 0.69103E+00_JPRB ,&
&  0.66721E+00_JPRB, 0.64743E+00_JPRB, 0.62066E+00_JPRB, 0.64835E+00_JPRB, 0.73420E+00_JPRB, 0.80215E+00_JPRB, 0.81436E+00_JPRB ,&
&  0.79696E+00_JPRB, 0.85002E+00_JPRB, 0.86891E+00_JPRB, 0.86830E+00_JPRB, 0.88600E+00_JPRB, 0.86987E+00_JPRB, 0.83009E+00_JPRB ,&
&  0.78357E+00_JPRB, 0.74966E+00_JPRB, 0.72964E+00_JPRB, 0.71271E+00_JPRB, 0.69858E+00_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.72805E+00_JPRB, 0.71714E+00_JPRB, 0.71640E+00_JPRB, 0.72713E+00_JPRB, 0.74535E+00_JPRB, 0.74522E+00_JPRB, 0.73280E+00_JPRB ,&
&  0.71449E+00_JPRB, 0.69375E+00_JPRB, 0.66781E+00_JPRB, 0.64109E+00_JPRB, 0.61844E+00_JPRB, 0.58313E+00_JPRB, 0.55011E+00_JPRB ,&
&  0.52011E+00_JPRB, 0.46653E+00_JPRB, 0.42341E+00_JPRB, 0.42387E+00_JPRB, 0.41676E+00_JPRB, 0.39662E+00_JPRB, 0.41197E+00_JPRB ,&
&  0.44226E+00_JPRB, 0.48922E+00_JPRB, 0.55436E+00_JPRB, 0.63931E+00_JPRB, 0.71599E+00_JPRB, 0.81919E+00_JPRB, 0.89709E+00_JPRB ,&
&  0.86266E+00_JPRB, 0.84256E+00_JPRB, 0.84493E+00_JPRB, 0.83535E+00_JPRB, 0.79646E+00_JPRB, 0.78339E+00_JPRB, 0.78860E+00_JPRB ,&
&  0.79874E+00_JPRB, 0.80887E+00_JPRB, 0.81257E+00_JPRB, 0.80323E+00_JPRB, 0.80134E+00_JPRB, 0.79991E+00_JPRB, 0.80435E+00_JPRB ,&
&  0.76389E+00_JPRB, 0.72601E+00_JPRB, 0.68260E+00_JPRB, 0.70286E+00_JPRB, 0.65256E+00_JPRB, 0.63230E+00_JPRB, 0.62600E+00_JPRB ,&
&  0.70730E+00_JPRB, 0.82863E+00_JPRB, 0.89661E+00_JPRB, 0.13052E+01_JPRB, 0.13448E+01_JPRB, 0.14292E+01_JPRB, 0.17111E+01_JPRB ,&
&  0.18941E+01_JPRB, 0.19627E+01_JPRB, 0.19678E+01_JPRB, 0.16052E+01_JPRB, 0.13798E+01_JPRB, 0.17760E+01_JPRB, 0.17557E+01_JPRB ,&
&  0.16450E+01_JPRB, 0.15923E+01_JPRB, 0.16018E+01_JPRB, 0.18065E+01_JPRB, 0.15086E+01_JPRB, 0.13209E+01_JPRB, 0.10468E+01_JPRB ,&
&  0.10869E+01_JPRB, 0.95944E+00_JPRB, 0.10698E+01_JPRB, 0.94325E+00_JPRB, 0.89014E+00_JPRB, 0.82300E+00_JPRB, 0.73853E+00_JPRB ,&
&  0.69892E+00_JPRB, 0.67558E+00_JPRB, 0.68082E+00_JPRB, 0.79141E+00_JPRB, 0.92165E+00_JPRB, 0.94929E+00_JPRB, 0.98757E+00_JPRB ,&
&  0.10121E+01_JPRB, 0.10312E+01_JPRB, 0.10110E+01_JPRB, 0.10194E+01_JPRB, 0.99680E+00_JPRB, 0.96749E+00_JPRB, 0.93192E+00_JPRB ,&
&  0.89348E+00_JPRB, 0.85491E+00_JPRB, 0.81385E+00_JPRB, 0.77646E+00_JPRB, 0.74720E+00_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.75909E+00_JPRB, 0.74867E+00_JPRB, 0.75045E+00_JPRB, 0.75678E+00_JPRB, 0.75690E+00_JPRB, 0.74426E+00_JPRB, 0.72646E+00_JPRB ,&
&  0.70917E+00_JPRB, 0.67497E+00_JPRB, 0.63893E+00_JPRB, 0.61044E+00_JPRB, 0.59405E+00_JPRB, 0.58103E+00_JPRB, 0.55754E+00_JPRB ,&
&  0.52694E+00_JPRB, 0.49714E+00_JPRB, 0.45462E+00_JPRB, 0.43625E+00_JPRB, 0.46080E+00_JPRB, 0.43194E+00_JPRB, 0.41482E+00_JPRB ,&
&  0.45855E+00_JPRB, 0.51858E+00_JPRB, 0.61703E+00_JPRB, 0.75291E+00_JPRB, 0.87694E+00_JPRB, 0.99906E+00_JPRB, 0.10630E+01_JPRB ,&
&  0.10136E+01_JPRB, 0.92474E+00_JPRB, 0.85377E+00_JPRB, 0.79426E+00_JPRB, 0.74815E+00_JPRB, 0.73970E+00_JPRB, 0.74439E+00_JPRB ,&
&  0.76980E+00_JPRB, 0.79550E+00_JPRB, 0.78371E+00_JPRB, 0.75041E+00_JPRB, 0.74537E+00_JPRB, 0.70604E+00_JPRB, 0.68088E+00_JPRB ,&
&  0.68850E+00_JPRB, 0.68529E+00_JPRB, 0.67855E+00_JPRB, 0.73711E+00_JPRB, 0.66661E+00_JPRB, 0.62154E+00_JPRB, 0.64009E+00_JPRB ,&
&  0.71819E+00_JPRB, 0.84096E+00_JPRB, 0.95300E+00_JPRB, 0.13125E+01_JPRB, 0.18001E+01_JPRB, 0.18557E+01_JPRB, 0.20152E+01_JPRB ,&
&  0.17284E+01_JPRB, 0.14527E+01_JPRB, 0.12424E+01_JPRB, 0.10594E+01_JPRB, 0.12899E+01_JPRB, 0.18325E+01_JPRB, 0.19664E+01_JPRB ,&
&  0.19034E+01_JPRB, 0.18767E+01_JPRB, 0.20083E+01_JPRB, 0.18547E+01_JPRB, 0.13967E+01_JPRB, 0.99036E+00_JPRB, 0.12472E+01_JPRB ,&
&  0.13315E+01_JPRB, 0.13081E+01_JPRB, 0.11817E+01_JPRB, 0.98140E+00_JPRB, 0.80357E+00_JPRB, 0.89029E+00_JPRB, 0.83143E+00_JPRB ,&
&  0.75343E+00_JPRB, 0.71835E+00_JPRB, 0.83088E+00_JPRB, 0.10285E+01_JPRB, 0.11061E+01_JPRB, 0.12430E+01_JPRB, 0.12724E+01_JPRB ,&
&  0.12210E+01_JPRB, 0.11895E+01_JPRB, 0.11410E+01_JPRB, 0.10821E+01_JPRB, 0.10339E+01_JPRB, 0.97704E+00_JPRB, 0.93453E+00_JPRB ,&
&  0.89777E+00_JPRB, 0.86798E+00_JPRB, 0.83397E+00_JPRB, 0.80236E+00_JPRB, 0.77665E+00_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.77657E+00_JPRB, 0.77150E+00_JPRB, 0.76764E+00_JPRB, 0.76130E+00_JPRB, 0.75112E+00_JPRB, 0.73550E+00_JPRB, 0.71691E+00_JPRB ,&
&  0.68522E+00_JPRB, 0.65267E+00_JPRB, 0.61710E+00_JPRB, 0.59698E+00_JPRB, 0.57534E+00_JPRB, 0.55770E+00_JPRB, 0.54692E+00_JPRB ,&
&  0.54344E+00_JPRB, 0.55450E+00_JPRB, 0.49861E+00_JPRB, 0.47852E+00_JPRB, 0.48660E+00_JPRB, 0.48162E+00_JPRB, 0.42995E+00_JPRB ,&
&  0.49352E+00_JPRB, 0.56040E+00_JPRB, 0.64882E+00_JPRB, 0.80561E+00_JPRB, 0.10577E+01_JPRB, 0.10510E+01_JPRB, 0.10297E+01_JPRB ,&
&  0.99822E+00_JPRB, 0.93134E+00_JPRB, 0.86967E+00_JPRB, 0.79686E+00_JPRB, 0.74757E+00_JPRB, 0.74153E+00_JPRB, 0.74735E+00_JPRB ,&
&  0.77744E+00_JPRB, 0.79148E+00_JPRB, 0.76525E+00_JPRB, 0.72763E+00_JPRB, 0.69197E+00_JPRB, 0.67167E+00_JPRB, 0.65786E+00_JPRB ,&
&  0.66320E+00_JPRB, 0.66608E+00_JPRB, 0.67328E+00_JPRB, 0.74892E+00_JPRB, 0.68239E+00_JPRB, 0.64261E+00_JPRB, 0.76267E+00_JPRB ,&
&  0.85730E+00_JPRB, 0.10650E+01_JPRB, 0.11405E+01_JPRB, 0.14531E+01_JPRB, 0.17930E+01_JPRB, 0.19323E+01_JPRB, 0.19987E+01_JPRB ,&
&  0.15596E+01_JPRB, 0.13313E+01_JPRB, 0.14221E+01_JPRB, 0.15649E+01_JPRB, 0.15029E+01_JPRB, 0.16369E+01_JPRB, 0.25859E+01_JPRB ,&
&  0.19147E+01_JPRB, 0.19988E+01_JPRB, 0.21965E+01_JPRB, 0.21526E+01_JPRB, 0.10421E+01_JPRB, 0.72677E+00_JPRB, 0.10642E+01_JPRB ,&
&  0.10841E+01_JPRB, 0.69607E+00_JPRB, 0.56071E+00_JPRB, 0.68808E+00_JPRB, 0.56346E+00_JPRB, 0.63841E+00_JPRB, 0.77859E+00_JPRB ,&
&  0.82025E+00_JPRB, 0.84126E+00_JPRB, 0.99400E+00_JPRB, 0.10733E+01_JPRB, 0.11652E+01_JPRB, 0.15455E+01_JPRB, 0.15155E+01_JPRB ,&
&  0.14316E+01_JPRB, 0.12806E+01_JPRB, 0.12377E+01_JPRB, 0.11166E+01_JPRB, 0.10330E+01_JPRB, 0.95749E+00_JPRB, 0.89714E+00_JPRB ,&
&  0.85385E+00_JPRB, 0.82398E+00_JPRB, 0.80336E+00_JPRB, 0.78743E+00_JPRB, 0.78338E+00_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.78447E+00_JPRB, 0.77882E+00_JPRB, 0.77236E+00_JPRB, 0.76298E+00_JPRB, 0.74862E+00_JPRB, 0.72794E+00_JPRB, 0.69981E+00_JPRB ,&
&  0.66323E+00_JPRB, 0.62803E+00_JPRB, 0.59818E+00_JPRB, 0.57623E+00_JPRB, 0.55534E+00_JPRB, 0.53970E+00_JPRB, 0.53849E+00_JPRB ,&
&  0.54545E+00_JPRB, 0.56491E+00_JPRB, 0.56325E+00_JPRB, 0.54346E+00_JPRB, 0.53983E+00_JPRB, 0.53133E+00_JPRB, 0.50689E+00_JPRB ,&
&  0.57455E+00_JPRB, 0.62318E+00_JPRB, 0.69189E+00_JPRB, 0.81388E+00_JPRB, 0.94550E+00_JPRB, 0.94603E+00_JPRB, 0.98213E+00_JPRB ,&
&  0.95688E+00_JPRB, 0.92937E+00_JPRB, 0.86044E+00_JPRB, 0.80043E+00_JPRB, 0.76925E+00_JPRB, 0.75842E+00_JPRB, 0.78131E+00_JPRB ,&
&  0.79286E+00_JPRB, 0.79908E+00_JPRB, 0.76189E+00_JPRB, 0.70188E+00_JPRB, 0.67728E+00_JPRB, 0.66025E+00_JPRB, 0.65170E+00_JPRB ,&
&  0.65301E+00_JPRB, 0.67050E+00_JPRB, 0.68813E+00_JPRB, 0.75584E+00_JPRB, 0.86552E+00_JPRB, 0.92100E+00_JPRB, 0.85825E+00_JPRB ,&
&  0.83106E+00_JPRB, 0.97828E+00_JPRB, 0.12574E+01_JPRB, 0.14034E+01_JPRB, 0.16228E+01_JPRB, 0.18173E+01_JPRB, 0.17859E+01_JPRB ,&
&  0.17111E+01_JPRB, 0.18102E+01_JPRB, 0.17640E+01_JPRB, 0.19452E+01_JPRB, 0.20195E+01_JPRB, 0.16459E+01_JPRB, 0.20539E+01_JPRB ,&
&  0.20954E+01_JPRB, 0.20163E+01_JPRB, 0.15980E+01_JPRB, 0.11379E+01_JPRB, 0.11904E+01_JPRB, 0.86107E+00_JPRB, 0.57846E+00_JPRB ,&
&  0.52547E+00_JPRB, 0.49734E+00_JPRB, 0.47446E+00_JPRB, 0.49349E+00_JPRB, 0.49779E+00_JPRB, 0.59795E+00_JPRB, 0.74666E+00_JPRB ,&
&  0.95344E+00_JPRB, 0.10486E+01_JPRB, 0.10560E+01_JPRB, 0.10763E+01_JPRB, 0.11052E+01_JPRB, 0.13830E+01_JPRB, 0.15182E+01_JPRB ,&
&  0.13591E+01_JPRB, 0.13076E+01_JPRB, 0.11717E+01_JPRB, 0.10596E+01_JPRB, 0.98700E+00_JPRB, 0.92911E+00_JPRB, 0.88457E+00_JPRB ,&
&  0.85647E+00_JPRB, 0.83565E+00_JPRB, 0.81879E+00_JPRB, 0.80588E+00_JPRB, 0.79209E+00_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.77279E+00_JPRB, 0.76493E+00_JPRB, 0.75856E+00_JPRB, 0.74416E+00_JPRB, 0.72513E+00_JPRB, 0.70346E+00_JPRB, 0.67280E+00_JPRB ,&
&  0.62867E+00_JPRB, 0.59083E+00_JPRB, 0.56410E+00_JPRB, 0.54318E+00_JPRB, 0.53410E+00_JPRB, 0.52872E+00_JPRB, 0.53735E+00_JPRB ,&
&  0.55075E+00_JPRB, 0.57167E+00_JPRB, 0.61098E+00_JPRB, 0.61222E+00_JPRB, 0.65735E+00_JPRB, 0.64572E+00_JPRB, 0.65207E+00_JPRB ,&
&  0.69974E+00_JPRB, 0.72918E+00_JPRB, 0.78923E+00_JPRB, 0.86240E+00_JPRB, 0.95360E+00_JPRB, 0.95637E+00_JPRB, 0.95241E+00_JPRB ,&
&  0.93230E+00_JPRB, 0.92925E+00_JPRB, 0.87134E+00_JPRB, 0.82045E+00_JPRB, 0.78608E+00_JPRB, 0.78994E+00_JPRB, 0.79187E+00_JPRB ,&
&  0.80419E+00_JPRB, 0.79988E+00_JPRB, 0.76599E+00_JPRB, 0.71378E+00_JPRB, 0.68645E+00_JPRB, 0.67398E+00_JPRB, 0.67900E+00_JPRB ,&
&  0.68463E+00_JPRB, 0.71730E+00_JPRB, 0.74975E+00_JPRB, 0.83365E+00_JPRB, 0.79181E+00_JPRB, 0.69436E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.89539E+00_JPRB, 0.11204E+01_JPRB, 0.12043E+01_JPRB, 0.14092E+01_JPRB, 0.15227E+01_JPRB, 0.15361E+01_JPRB, 0.16578E+01_JPRB ,&
&  0.17833E+01_JPRB, 0.23780E+01_JPRB, 0.18514E+01_JPRB, 0.18474E+01_JPRB, 0.26988E+01_JPRB, 0.21857E+01_JPRB, 0.20655E+01_JPRB ,&
&  0.23477E+01_JPRB, 0.19799E+01_JPRB, 0.17473E+01_JPRB, 0.14200E+01_JPRB, 0.20259E+01_JPRB, 0.19646E+01_JPRB, 0.73643E+00_JPRB ,&
&  0.56518E+00_JPRB, 0.51917E+00_JPRB, 0.48366E+00_JPRB, 0.46067E+00_JPRB, 0.48879E+00_JPRB, 0.61552E+00_JPRB, 0.87203E+00_JPRB ,&
&  0.99839E+00_JPRB, 0.11246E+01_JPRB, 0.11079E+01_JPRB, 0.11286E+01_JPRB, 0.10960E+01_JPRB, 0.12747E+01_JPRB, 0.13952E+01_JPRB ,&
&  0.12741E+01_JPRB, 0.12147E+01_JPRB, 0.10701E+01_JPRB, 0.97973E+00_JPRB, 0.93094E+00_JPRB, 0.89555E+00_JPRB, 0.86256E+00_JPRB ,&
&  0.83727E+00_JPRB, 0.82279E+00_JPRB, 0.80967E+00_JPRB, 0.79481E+00_JPRB, 0.78259E+00_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.74906E+00_JPRB, 0.74113E+00_JPRB, 0.72754E+00_JPRB, 0.70557E+00_JPRB, 0.69717E+00_JPRB, 0.68578E+00_JPRB, 0.65142E+00_JPRB ,&
&  0.60619E+00_JPRB, 0.57772E+00_JPRB, 0.55015E+00_JPRB, 0.52926E+00_JPRB, 0.52813E+00_JPRB, 0.53230E+00_JPRB, 0.55021E+00_JPRB ,&
&  0.57311E+00_JPRB, 0.60074E+00_JPRB, 0.62821E+00_JPRB, 0.69568E+00_JPRB, 0.78099E+00_JPRB, 0.77993E+00_JPRB, 0.87620E+00_JPRB ,&
&  0.88752E+00_JPRB, 0.88183E+00_JPRB, 0.94231E+00_JPRB, 0.98396E+00_JPRB, 0.10159E+01_JPRB, 0.98582E+00_JPRB, 0.96512E+00_JPRB ,&
&  0.92275E+00_JPRB, 0.93457E+00_JPRB, 0.88245E+00_JPRB, 0.84744E+00_JPRB, 0.83631E+00_JPRB, 0.83027E+00_JPRB, 0.82532E+00_JPRB ,&
&  0.82429E+00_JPRB, 0.81163E+00_JPRB, 0.78592E+00_JPRB, 0.74755E+00_JPRB, 0.72130E+00_JPRB, 0.70642E+00_JPRB, 0.71409E+00_JPRB ,&
&  0.74257E+00_JPRB, 0.79204E+00_JPRB, 0.83754E+00_JPRB, 0.89809E+00_JPRB, 0.74948E+00_JPRB, 0.87103E+00_JPRB, 0.92967E+00_JPRB ,&
&  0.10461E+01_JPRB, 0.11475E+01_JPRB, 0.11508E+01_JPRB, 0.12573E+01_JPRB, 0.14224E+01_JPRB, 0.14722E+01_JPRB, 0.16052E+01_JPRB ,&
&  0.19791E+01_JPRB, 0.19475E+01_JPRB, 0.17217E+01_JPRB, 0.18706E+01_JPRB, 0.24241E+01_JPRB, 0.34116E+01_JPRB, 0.19056E+01_JPRB ,&
&  0.15513E+01_JPRB, 0.18663E+01_JPRB, 0.17471E+01_JPRB, 0.15934E+01_JPRB, 0.21000E+01_JPRB, 0.25002E+01_JPRB, 0.16983E+01_JPRB ,&
&  0.64909E+00_JPRB, 0.52547E+00_JPRB, 0.55337E+00_JPRB, 0.52821E+00_JPRB, 0.52383E+00_JPRB, 0.64770E+00_JPRB, 0.12659E+01_JPRB ,&
&  0.10888E+01_JPRB, 0.11740E+01_JPRB, 0.11565E+01_JPRB, 0.11370E+01_JPRB, 0.11351E+01_JPRB, 0.12525E+01_JPRB, 0.13457E+01_JPRB ,&
&  0.10774E+01_JPRB, 0.10225E+01_JPRB, 0.93040E+00_JPRB, 0.88514E+00_JPRB, 0.86536E+00_JPRB, 0.85215E+00_JPRB, 0.82436E+00_JPRB ,&
&  0.80357E+00_JPRB, 0.79041E+00_JPRB, 0.77867E+00_JPRB, 0.76831E+00_JPRB, 0.75965E+00_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.72039E+00_JPRB, 0.70710E+00_JPRB, 0.68816E+00_JPRB, 0.68561E+00_JPRB, 0.70308E+00_JPRB, 0.68745E+00_JPRB, 0.65848E+00_JPRB ,&
&  0.61112E+00_JPRB, 0.58743E+00_JPRB, 0.56252E+00_JPRB, 0.55188E+00_JPRB, 0.54979E+00_JPRB, 0.56259E+00_JPRB, 0.58578E+00_JPRB ,&
&  0.61551E+00_JPRB, 0.64519E+00_JPRB, 0.68148E+00_JPRB, 0.79708E+00_JPRB, 0.95994E+00_JPRB, 0.93875E+00_JPRB, 0.10092E+01_JPRB ,&
&  0.11406E+01_JPRB, 0.11348E+01_JPRB, 0.10961E+01_JPRB, 0.10964E+01_JPRB, 0.11104E+01_JPRB, 0.10862E+01_JPRB, 0.10171E+01_JPRB ,&
&  0.94284E+00_JPRB, 0.97268E+00_JPRB, 0.92439E+00_JPRB, 0.88950E+00_JPRB, 0.86018E+00_JPRB, 0.85914E+00_JPRB, 0.87584E+00_JPRB ,&
&  0.85686E+00_JPRB, 0.86047E+00_JPRB, 0.82626E+00_JPRB, 0.80028E+00_JPRB, 0.77987E+00_JPRB, 0.76843E+00_JPRB, 0.79872E+00_JPRB ,&
&  0.85735E+00_JPRB, 0.91621E+00_JPRB, 0.95217E+00_JPRB, 0.94220E+00_JPRB, 0.95918E+00_JPRB, 0.98805E+00_JPRB, 0.10926E+01_JPRB ,&
&  0.11700E+01_JPRB, 0.12222E+01_JPRB, 0.12588E+01_JPRB, 0.12843E+01_JPRB, 0.14091E+01_JPRB, 0.15615E+01_JPRB, 0.17429E+01_JPRB ,&
&  0.20156E+01_JPRB, 0.20641E+01_JPRB, 0.18117E+01_JPRB, 0.18955E+01_JPRB, 0.22524E+01_JPRB, 0.26707E+01_JPRB, 0.23566E+01_JPRB ,&
&  0.18452E+01_JPRB, 0.14987E+01_JPRB, 0.15430E+01_JPRB, 0.17982E+01_JPRB, 0.19680E+01_JPRB, 0.18908E+01_JPRB, 0.21487E+01_JPRB ,&
&  0.18405E+01_JPRB, 0.10083E+01_JPRB, 0.83969E+00_JPRB, 0.97392E+00_JPRB, 0.78824E+00_JPRB, 0.80056E+00_JPRB, 0.10295E+01_JPRB ,&
&  0.10942E+01_JPRB, 0.11786E+01_JPRB, 0.11844E+01_JPRB, 0.10774E+01_JPRB, 0.11373E+01_JPRB, 0.12138E+01_JPRB, 0.11672E+01_JPRB ,&
&  0.99434E+00_JPRB, 0.92389E+00_JPRB, 0.86785E+00_JPRB, 0.83050E+00_JPRB, 0.80768E+00_JPRB, 0.78217E+00_JPRB, 0.76637E+00_JPRB ,&
&  0.74778E+00_JPRB, 0.73448E+00_JPRB, 0.72806E+00_JPRB, 0.72348E+00_JPRB, 0.72544E+00_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.69667E+00_JPRB, 0.68426E+00_JPRB, 0.69719E+00_JPRB, 0.72745E+00_JPRB, 0.73538E+00_JPRB, 0.73624E+00_JPRB, 0.70880E+00_JPRB ,&
&  0.64329E+00_JPRB, 0.62708E+00_JPRB, 0.62417E+00_JPRB, 0.61810E+00_JPRB, 0.60972E+00_JPRB, 0.62261E+00_JPRB, 0.64379E+00_JPRB ,&
&  0.67524E+00_JPRB, 0.73315E+00_JPRB, 0.83611E+00_JPRB, 0.10594E+01_JPRB, 0.12434E+01_JPRB, 0.14269E+01_JPRB, 0.10342E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.13022E+01_JPRB, 0.12028E+01_JPRB, 0.11941E+01_JPRB, 0.11929E+01_JPRB, 0.11746E+01_JPRB, 0.10678E+01_JPRB ,&
&  0.10273E+01_JPRB, 0.10336E+01_JPRB, 0.98686E+00_JPRB, 0.91938E+00_JPRB, 0.88899E+00_JPRB, 0.93597E+00_JPRB, 0.92567E+00_JPRB ,&
&  0.92818E+00_JPRB, 0.94420E+00_JPRB, 0.89355E+00_JPRB, 0.88140E+00_JPRB, 0.86616E+00_JPRB, 0.86260E+00_JPRB, 0.91389E+00_JPRB ,&
&  0.98004E+00_JPRB, 0.10323E+01_JPRB, 0.10685E+01_JPRB, 0.10314E+01_JPRB, 0.10861E+01_JPRB, 0.11585E+01_JPRB, 0.12811E+01_JPRB ,&
&  0.12939E+01_JPRB, 0.11941E+01_JPRB, 0.12527E+01_JPRB, 0.13883E+01_JPRB, 0.15392E+01_JPRB, 0.16094E+01_JPRB, 0.17690E+01_JPRB ,&
&  0.21508E+01_JPRB, 0.24137E+01_JPRB, 0.24491E+01_JPRB, 0.20655E+01_JPRB, 0.20863E+01_JPRB, 0.23822E+01_JPRB, 0.24705E+01_JPRB ,&
&  0.21888E+01_JPRB, 0.18247E+01_JPRB, 0.18116E+01_JPRB, 0.17415E+01_JPRB, 0.16676E+01_JPRB, 0.15382E+01_JPRB, 0.15836E+01_JPRB ,&
&  0.17260E+01_JPRB, 0.18296E+01_JPRB, 0.14664E+01_JPRB, 0.12161E+01_JPRB, 0.11723E+01_JPRB, 0.10558E+01_JPRB, 0.10321E+01_JPRB ,&
&  0.11438E+01_JPRB, 0.11441E+01_JPRB, 0.11448E+01_JPRB, 0.12924E+01_JPRB, 0.12243E+01_JPRB, 0.11675E+01_JPRB, 0.99156E+00_JPRB ,&
&  0.83478E+00_JPRB, 0.78845E+00_JPRB, 0.75517E+00_JPRB, 0.73415E+00_JPRB, 0.73728E+00_JPRB, 0.73326E+00_JPRB, 0.73118E+00_JPRB ,&
&  0.72028E+00_JPRB, 0.71657E+00_JPRB, 0.71746E+00_JPRB, 0.71831E+00_JPRB, 0.71229E+00_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.69779E+00_JPRB, 0.71214E+00_JPRB, 0.74124E+00_JPRB, 0.77313E+00_JPRB, 0.79743E+00_JPRB, 0.82911E+00_JPRB, 0.81316E+00_JPRB ,&
&  0.72623E+00_JPRB, 0.66789E+00_JPRB, 0.67455E+00_JPRB, 0.66192E+00_JPRB, 0.65377E+00_JPRB, 0.66706E+00_JPRB, 0.69828E+00_JPRB ,&
&  0.75694E+00_JPRB, 0.88593E+00_JPRB, 0.10826E+01_JPRB, 0.13354E+01_JPRB, 0.14293E+01_JPRB, 0.14491E+01_JPRB, 0.13691E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.14345E+01_JPRB, 0.12204E+01_JPRB, 0.12576E+01_JPRB, 0.12396E+01_JPRB, 0.13035E+01_JPRB, 0.12270E+01_JPRB ,&
&  0.11211E+01_JPRB, 0.10600E+01_JPRB, 0.10667E+01_JPRB, 0.10092E+01_JPRB, 0.99696E+00_JPRB, 0.98568E+00_JPRB, 0.99358E+00_JPRB ,&
&  0.97891E+00_JPRB, 0.96952E+00_JPRB, 0.96747E+00_JPRB, 0.97400E+00_JPRB, 0.96688E+00_JPRB, 0.97358E+00_JPRB, 0.10019E+01_JPRB ,&
&  0.10759E+01_JPRB, 0.11397E+01_JPRB, 0.11595E+01_JPRB, 0.11937E+01_JPRB, 0.12291E+01_JPRB, 0.13703E+01_JPRB, 0.14988E+01_JPRB ,&
&  0.14681E+01_JPRB, 0.13361E+01_JPRB, 0.13691E+01_JPRB, 0.14513E+01_JPRB, 0.15243E+01_JPRB, 0.17436E+01_JPRB, 0.18599E+01_JPRB ,&
&  0.22683E+01_JPRB, 0.22143E+01_JPRB, 0.25547E+01_JPRB, 0.20182E+01_JPRB, 0.19427E+01_JPRB, 0.21844E+01_JPRB, 0.19545E+01_JPRB ,&
&  0.16344E+01_JPRB, 0.17142E+01_JPRB, 0.16694E+01_JPRB, 0.16631E+01_JPRB, 0.15951E+01_JPRB, 0.14498E+01_JPRB, 0.13643E+01_JPRB ,&
&  0.13762E+01_JPRB, 0.14403E+01_JPRB, 0.14106E+01_JPRB, 0.12176E+01_JPRB, 0.11457E+01_JPRB, 0.10592E+01_JPRB, 0.11116E+01_JPRB ,&
&  0.12402E+01_JPRB, 0.13371E+01_JPRB, 0.14140E+01_JPRB, 0.13746E+01_JPRB, 0.10491E+01_JPRB, 0.90969E+00_JPRB, 0.80797E+00_JPRB ,&
&  0.75273E+00_JPRB, 0.71843E+00_JPRB, 0.71232E+00_JPRB, 0.71872E+00_JPRB, 0.72672E+00_JPRB, 0.73053E+00_JPRB, 0.72225E+00_JPRB ,&
&  0.71733E+00_JPRB, 0.71516E+00_JPRB, 0.71170E+00_JPRB, 0.70634E+00_JPRB, 0.70149E+00_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.74405E+00_JPRB, 0.77288E+00_JPRB, 0.81555E+00_JPRB, 0.88095E+00_JPRB, 0.94403E+00_JPRB, 0.10380E+01_JPRB, 0.11174E+01_JPRB ,&
&  0.99160E+00_JPRB, 0.65727E+00_JPRB, 0.66949E+00_JPRB, 0.68346E+00_JPRB, 0.70823E+00_JPRB, 0.75019E+00_JPRB, 0.81372E+00_JPRB ,&
&  0.92220E+00_JPRB, 0.10695E+01_JPRB, 0.12361E+01_JPRB, 0.13776E+01_JPRB, 0.14969E+01_JPRB, 0.15259E+01_JPRB, 0.14489E+01_JPRB ,&
&  0.11496E+01_JPRB, 0.11067E+01_JPRB, 0.13913E+01_JPRB, 0.13703E+01_JPRB, 0.13684E+01_JPRB, 0.13434E+01_JPRB, 0.13220E+01_JPRB ,&
&  0.12029E+01_JPRB, 0.11474E+01_JPRB, 0.11520E+01_JPRB, 0.11393E+01_JPRB, 0.10769E+01_JPRB, 0.10938E+01_JPRB, 0.11111E+01_JPRB ,&
&  0.11068E+01_JPRB, 0.10975E+01_JPRB, 0.11054E+01_JPRB, 0.11285E+01_JPRB, 0.11125E+01_JPRB, 0.11155E+01_JPRB, 0.11538E+01_JPRB ,&
&  0.12504E+01_JPRB, 0.13268E+01_JPRB, 0.13732E+01_JPRB, 0.13818E+01_JPRB, 0.14671E+01_JPRB, 0.16064E+01_JPRB, 0.16478E+01_JPRB ,&
&  0.15787E+01_JPRB, 0.15603E+01_JPRB, 0.15889E+01_JPRB, 0.16442E+01_JPRB, 0.16075E+01_JPRB, 0.18895E+01_JPRB, 0.19762E+01_JPRB ,&
&  0.21104E+01_JPRB, 0.20942E+01_JPRB, 0.23999E+01_JPRB, 0.17933E+01_JPRB, 0.18749E+01_JPRB, 0.19273E+01_JPRB, 0.16799E+01_JPRB ,&
&  0.15058E+01_JPRB, 0.15576E+01_JPRB, 0.14982E+01_JPRB, 0.15001E+01_JPRB, 0.14701E+01_JPRB, 0.12998E+01_JPRB, 0.13376E+01_JPRB ,&
&  0.12795E+01_JPRB, 0.13868E+01_JPRB, 0.13316E+01_JPRB, 0.12804E+01_JPRB, 0.11253E+01_JPRB, 0.10800E+01_JPRB, 0.11260E+01_JPRB ,&
&  0.13031E+01_JPRB, 0.14387E+01_JPRB, 0.13350E+01_JPRB, 0.11134E+01_JPRB, 0.83136E+00_JPRB, 0.72816E+00_JPRB, 0.72211E+00_JPRB ,&
&  0.71420E+00_JPRB, 0.71336E+00_JPRB, 0.71917E+00_JPRB, 0.73112E+00_JPRB, 0.73723E+00_JPRB, 0.73304E+00_JPRB, 0.72713E+00_JPRB ,&
&  0.72714E+00_JPRB, 0.72675E+00_JPRB, 0.72278E+00_JPRB, 0.72408E+00_JPRB, 0.73112E+00_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.76511E+00_JPRB, 0.77933E+00_JPRB, 0.79800E+00_JPRB, 0.80341E+00_JPRB, 0.79717E+00_JPRB, 0.77300E+00_JPRB, 0.71641E+00_JPRB ,&
&  0.66094E+00_JPRB, 0.63441E+00_JPRB, 0.66282E+00_JPRB, 0.70063E+00_JPRB, 0.74541E+00_JPRB, 0.80282E+00_JPRB, 0.88993E+00_JPRB ,&
&  0.10004E+01_JPRB, 0.10975E+01_JPRB, 0.11937E+01_JPRB, 0.13508E+01_JPRB, 0.15065E+01_JPRB, 0.14883E+01_JPRB, 0.14631E+01_JPRB ,&
&  0.12685E+01_JPRB, 0.10368E+01_JPRB, 0.13229E+01_JPRB, 0.15381E+01_JPRB, 0.14985E+01_JPRB, 0.14398E+01_JPRB, 0.13881E+01_JPRB ,&
&  0.12494E+01_JPRB, 0.12770E+01_JPRB, 0.13189E+01_JPRB, 0.13271E+01_JPRB, 0.12385E+01_JPRB, 0.12547E+01_JPRB, 0.12696E+01_JPRB ,&
&  0.12889E+01_JPRB, 0.12922E+01_JPRB, 0.13344E+01_JPRB, 0.13603E+01_JPRB, 0.13595E+01_JPRB, 0.13966E+01_JPRB, 0.14568E+01_JPRB ,&
&  0.15182E+01_JPRB, 0.15675E+01_JPRB, 0.16419E+01_JPRB, 0.16922E+01_JPRB, 0.17470E+01_JPRB, 0.18180E+01_JPRB, 0.17932E+01_JPRB ,&
&  0.17242E+01_JPRB, 0.16574E+01_JPRB, 0.17422E+01_JPRB, 0.19329E+01_JPRB, 0.20709E+01_JPRB, 0.18744E+01_JPRB, 0.20197E+01_JPRB ,&
&  0.19881E+01_JPRB, 0.19629E+01_JPRB, 0.19037E+01_JPRB, 0.21065E+01_JPRB, 0.14850E+01_JPRB, 0.15940E+01_JPRB, 0.14059E+01_JPRB ,&
&  0.14823E+01_JPRB, 0.14204E+01_JPRB, 0.13503E+01_JPRB, 0.13186E+01_JPRB, 0.13051E+01_JPRB, 0.11771E+01_JPRB, 0.12272E+01_JPRB ,&
&  0.12865E+01_JPRB, 0.13211E+01_JPRB, 0.13104E+01_JPRB, 0.13225E+01_JPRB, 0.12510E+01_JPRB, 0.12552E+01_JPRB, 0.12902E+01_JPRB ,&
&  0.14384E+01_JPRB, 0.13038E+01_JPRB, 0.10938E+01_JPRB, 0.90162E+00_JPRB, 0.77645E+00_JPRB, 0.68243E+00_JPRB, 0.71506E+00_JPRB ,&
&  0.72705E+00_JPRB, 0.73087E+00_JPRB, 0.72718E+00_JPRB, 0.73277E+00_JPRB, 0.73609E+00_JPRB, 0.73300E+00_JPRB, 0.74114E+00_JPRB ,&
&  0.74455E+00_JPRB, 0.74734E+00_JPRB, 0.74774E+00_JPRB, 0.75003E+00_JPRB, 0.75538E+00_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.76459E+00_JPRB, 0.76026E+00_JPRB, 0.74163E+00_JPRB, 0.70516E+00_JPRB, 0.66098E+00_JPRB, 0.61537E+00_JPRB, 0.58653E+00_JPRB ,&
&  0.55873E+00_JPRB, 0.57480E+00_JPRB, 0.61546E+00_JPRB, 0.67212E+00_JPRB, 0.74195E+00_JPRB, 0.81915E+00_JPRB, 0.89904E+00_JPRB ,&
&  0.95206E+00_JPRB, 0.10051E+01_JPRB, 0.10892E+01_JPRB, 0.12776E+01_JPRB, 0.14725E+01_JPRB, 0.15900E+01_JPRB, 0.16266E+01_JPRB ,&
&  0.16321E+01_JPRB, 0.16326E+01_JPRB, 0.15704E+01_JPRB, 0.13205E+01_JPRB, 0.11621E+01_JPRB, 0.14376E+01_JPRB, 0.13681E+01_JPRB ,&
&  0.13666E+01_JPRB, 0.13594E+01_JPRB, 0.13835E+01_JPRB, 0.14090E+01_JPRB, 0.13933E+01_JPRB, 0.13910E+01_JPRB, 0.14291E+01_JPRB ,&
&  0.14657E+01_JPRB, 0.15133E+01_JPRB, 0.15594E+01_JPRB, 0.15833E+01_JPRB, 0.16270E+01_JPRB, 0.16901E+01_JPRB, 0.17458E+01_JPRB ,&
&  0.17858E+01_JPRB, 0.18570E+01_JPRB, 0.19405E+01_JPRB, 0.20133E+01_JPRB, 0.19899E+01_JPRB, 0.20209E+01_JPRB, 0.19774E+01_JPRB ,&
&  0.19143E+01_JPRB, 0.18664E+01_JPRB, 0.18732E+01_JPRB, 0.19795E+01_JPRB, 0.20034E+01_JPRB, 0.17566E+01_JPRB, 0.17524E+01_JPRB ,&
&  0.18279E+01_JPRB, 0.18230E+01_JPRB, 0.15921E+01_JPRB, 0.19639E+01_JPRB, 0.12005E+01_JPRB, 0.13951E+01_JPRB, 0.15031E+01_JPRB ,&
&  0.14519E+01_JPRB, 0.13239E+01_JPRB, 0.12652E+01_JPRB, 0.12500E+01_JPRB, 0.12068E+01_JPRB, 0.11516E+01_JPRB, 0.12543E+01_JPRB ,&
&  0.13097E+01_JPRB, 0.12865E+01_JPRB, 0.13031E+01_JPRB, 0.12930E+01_JPRB, 0.12924E+01_JPRB, 0.12303E+01_JPRB, 0.11397E+01_JPRB ,&
&  0.10424E+01_JPRB, 0.11730E+01_JPRB, 0.10648E+01_JPRB, 0.10186E+01_JPRB, 0.91597E+00_JPRB, 0.69834E+00_JPRB, 0.73035E+00_JPRB ,&
&  0.74031E+00_JPRB, 0.73599E+00_JPRB, 0.74370E+00_JPRB, 0.74559E+00_JPRB, 0.73371E+00_JPRB, 0.73367E+00_JPRB, 0.74479E+00_JPRB ,&
&  0.74367E+00_JPRB, 0.74409E+00_JPRB, 0.74780E+00_JPRB, 0.75012E+00_JPRB, 0.75821E+00_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.65970E+00_JPRB, 0.64713E+00_JPRB, 0.62737E+00_JPRB, 0.60493E+00_JPRB, 0.58341E+00_JPRB, 0.55459E+00_JPRB, 0.55410E+00_JPRB ,&
&  0.54096E+00_JPRB, 0.54654E+00_JPRB, 0.56608E+00_JPRB, 0.58366E+00_JPRB, 0.61852E+00_JPRB, 0.67617E+00_JPRB, 0.73425E+00_JPRB ,&
&  0.77702E+00_JPRB, 0.79745E+00_JPRB, 0.86064E+00_JPRB, 0.95887E+00_JPRB, 0.11229E+01_JPRB, 0.13827E+01_JPRB, 0.17945E+01_JPRB ,&
&  0.19732E+01_JPRB, 0.20882E+01_JPRB, 0.20643E+01_JPRB, 0.19274E+01_JPRB, 0.16615E+01_JPRB, 0.15948E+01_JPRB, 0.17479E+01_JPRB ,&
&  0.18317E+01_JPRB, 0.17190E+01_JPRB, 0.15360E+01_JPRB, 0.15091E+01_JPRB, 0.15050E+01_JPRB, 0.15179E+01_JPRB, 0.15550E+01_JPRB ,&
&  0.16085E+01_JPRB, 0.16477E+01_JPRB, 0.16830E+01_JPRB, 0.17414E+01_JPRB, 0.17905E+01_JPRB, 0.18419E+01_JPRB, 0.19108E+01_JPRB ,&
&  0.19785E+01_JPRB, 0.20337E+01_JPRB, 0.20841E+01_JPRB, 0.21142E+01_JPRB, 0.21949E+01_JPRB, 0.22741E+01_JPRB, 0.22678E+01_JPRB ,&
&  0.22153E+01_JPRB, 0.20663E+01_JPRB, 0.20907E+01_JPRB, 0.21641E+01_JPRB, 0.20602E+01_JPRB, 0.18581E+01_JPRB, 0.17902E+01_JPRB ,&
&  0.18351E+01_JPRB, 0.18602E+01_JPRB, 0.13040E+01_JPRB, 0.15862E+01_JPRB, 0.17474E+01_JPRB, 0.15421E+01_JPRB, 0.13978E+01_JPRB ,&
&  0.13468E+01_JPRB, 0.12553E+01_JPRB, 0.12358E+01_JPRB, 0.11884E+01_JPRB, 0.12409E+01_JPRB, 0.12156E+01_JPRB, 0.10756E+01_JPRB ,&
&  0.11602E+01_JPRB, 0.10985E+01_JPRB, 0.11390E+01_JPRB, 0.11619E+01_JPRB, 0.11238E+01_JPRB, 0.10425E+01_JPRB, 0.92828E+00_JPRB ,&
&  0.86643E+00_JPRB, 0.10954E+01_JPRB, 0.10486E+01_JPRB, 0.12035E+01_JPRB, 0.11524E+01_JPRB, 0.70153E+00_JPRB, 0.73139E+00_JPRB ,&
&  0.73220E+00_JPRB, 0.73380E+00_JPRB, 0.73762E+00_JPRB, 0.73244E+00_JPRB, 0.72640E+00_JPRB, 0.72478E+00_JPRB, 0.72814E+00_JPRB ,&
&  0.71976E+00_JPRB, 0.70874E+00_JPRB, 0.69768E+00_JPRB, 0.68999E+00_JPRB, 0.67757E+00_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.61636E+00_JPRB, 0.61606E+00_JPRB, 0.61519E+00_JPRB, 0.61995E+00_JPRB, 0.63043E+00_JPRB, 0.63252E+00_JPRB, 0.62429E+00_JPRB ,&
&  0.60647E+00_JPRB, 0.59735E+00_JPRB, 0.57566E+00_JPRB, 0.55054E+00_JPRB, 0.55055E+00_JPRB, 0.59513E+00_JPRB, 0.64230E+00_JPRB ,&
&  0.67254E+00_JPRB, 0.68865E+00_JPRB, 0.69274E+00_JPRB, 0.69916E+00_JPRB, 0.74020E+00_JPRB, 0.77083E+00_JPRB, 0.96328E+00_JPRB ,&
&  0.13183E+01_JPRB, 0.18348E+01_JPRB, 0.22216E+01_JPRB, 0.24180E+01_JPRB, 0.22519E+01_JPRB, 0.18395E+01_JPRB, 0.20063E+01_JPRB ,&
&  0.18069E+01_JPRB, 0.14768E+01_JPRB, 0.13954E+01_JPRB, 0.13996E+01_JPRB, 0.14308E+01_JPRB, 0.14488E+01_JPRB, 0.14952E+01_JPRB ,&
&  0.15494E+01_JPRB, 0.16100E+01_JPRB, 0.16749E+01_JPRB, 0.17527E+01_JPRB, 0.18349E+01_JPRB, 0.19292E+01_JPRB, 0.20251E+01_JPRB ,&
&  0.21292E+01_JPRB, 0.21861E+01_JPRB, 0.22964E+01_JPRB, 0.21886E+01_JPRB, 0.23467E+01_JPRB, 0.24876E+01_JPRB, 0.25534E+01_JPRB ,&
&  0.25069E+01_JPRB, 0.23900E+01_JPRB, 0.24284E+01_JPRB, 0.24068E+01_JPRB, 0.23755E+01_JPRB, 0.22418E+01_JPRB, 0.20703E+01_JPRB ,&
&  0.21096E+01_JPRB, 0.19674E+01_JPRB, 0.13365E+01_JPRB, 0.13245E+01_JPRB, 0.14316E+01_JPRB, 0.11996E+01_JPRB, 0.13531E+01_JPRB ,&
&  0.12615E+01_JPRB, 0.11898E+01_JPRB, 0.11109E+01_JPRB, 0.10389E+01_JPRB, 0.10866E+01_JPRB, 0.11400E+01_JPRB, 0.99752E+00_JPRB ,&
&  0.97743E+00_JPRB, 0.92424E+00_JPRB, 0.92938E+00_JPRB, 0.95461E+00_JPRB, 0.93685E+00_JPRB, 0.90149E+00_JPRB, 0.86272E+00_JPRB ,&
&  0.89957E+00_JPRB, 0.86598E+00_JPRB, 0.75360E+00_JPRB, 0.77423E+00_JPRB, 0.66360E+00_JPRB, 0.67645E+00_JPRB, 0.69041E+00_JPRB ,&
&  0.69126E+00_JPRB, 0.67458E+00_JPRB, 0.67216E+00_JPRB, 0.66811E+00_JPRB, 0.66938E+00_JPRB, 0.65667E+00_JPRB, 0.66237E+00_JPRB ,&
&  0.66052E+00_JPRB, 0.65442E+00_JPRB, 0.64546E+00_JPRB, 0.64401E+00_JPRB, 0.62961E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.71342E+00_JPRB, 0.72270E+00_JPRB, 0.73871E+00_JPRB, 0.75100E+00_JPRB, 0.77238E+00_JPRB, 0.76445E+00_JPRB, 0.75657E+00_JPRB ,&
&  0.72833E+00_JPRB, 0.72319E+00_JPRB, 0.71703E+00_JPRB, 0.71439E+00_JPRB, 0.71913E+00_JPRB, 0.74185E+00_JPRB, 0.78483E+00_JPRB ,&
&  0.79842E+00_JPRB, 0.80421E+00_JPRB, 0.79896E+00_JPRB, 0.79018E+00_JPRB, 0.76059E+00_JPRB, 0.73375E+00_JPRB, 0.71136E+00_JPRB ,&
&  0.73116E+00_JPRB, 0.91956E+00_JPRB, 0.11694E+01_JPRB, 0.14026E+01_JPRB, 0.16624E+01_JPRB, 0.15378E+01_JPRB, 0.17373E+01_JPRB ,&
&  0.17613E+01_JPRB, 0.13638E+01_JPRB, 0.14129E+01_JPRB, 0.12704E+01_JPRB, 0.12864E+01_JPRB, 0.12923E+01_JPRB, 0.13358E+01_JPRB ,&
&  0.14099E+01_JPRB, 0.14628E+01_JPRB, 0.14983E+01_JPRB, 0.15421E+01_JPRB, 0.16094E+01_JPRB, 0.16792E+01_JPRB, 0.17267E+01_JPRB ,&
&  0.17607E+01_JPRB, 0.17199E+01_JPRB, 0.17639E+01_JPRB, 0.20398E+01_JPRB, 0.22776E+01_JPRB, 0.24099E+01_JPRB, 0.25143E+01_JPRB ,&
&  0.26513E+01_JPRB, 0.27154E+01_JPRB, 0.24480E+01_JPRB, 0.23990E+01_JPRB, 0.25212E+01_JPRB, 0.22686E+01_JPRB, 0.22558E+01_JPRB ,&
&  0.23309E+01_JPRB, 0.20157E+01_JPRB, 0.12352E+01_JPRB, 0.13031E+01_JPRB, 0.13441E+01_JPRB, 0.13036E+01_JPRB, 0.13537E+01_JPRB ,&
&  0.11921E+01_JPRB, 0.11022E+01_JPRB, 0.98110E+00_JPRB, 0.94102E+00_JPRB, 0.94648E+00_JPRB, 0.10008E+01_JPRB, 0.95531E+00_JPRB ,&
&  0.88574E+00_JPRB, 0.84822E+00_JPRB, 0.89842E+00_JPRB, 0.93636E+00_JPRB, 0.90555E+00_JPRB, 0.85856E+00_JPRB, 0.81373E+00_JPRB ,&
&  0.84649E+00_JPRB, 0.76848E+00_JPRB, 0.81044E+00_JPRB, 0.83949E+00_JPRB, 0.63539E+00_JPRB, 0.68647E+00_JPRB, 0.68899E+00_JPRB ,&
&  0.69598E+00_JPRB, 0.68788E+00_JPRB, 0.67464E+00_JPRB, 0.67494E+00_JPRB, 0.66694E+00_JPRB, 0.65910E+00_JPRB, 0.67401E+00_JPRB ,&
&  0.69576E+00_JPRB, 0.69410E+00_JPRB, 0.69139E+00_JPRB, 0.69315E+00_JPRB, 0.70131E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.81827E+00_JPRB, 0.83741E+00_JPRB, 0.86011E+00_JPRB, 0.86333E+00_JPRB, 0.86956E+00_JPRB, 0.86416E+00_JPRB, 0.86205E+00_JPRB ,&
&  0.85087E+00_JPRB, 0.84843E+00_JPRB, 0.85100E+00_JPRB, 0.85610E+00_JPRB, 0.86518E+00_JPRB, 0.87671E+00_JPRB, 0.90599E+00_JPRB ,&
&  0.90605E+00_JPRB, 0.91132E+00_JPRB, 0.91872E+00_JPRB, 0.93321E+00_JPRB, 0.94852E+00_JPRB, 0.94871E+00_JPRB, 0.92578E+00_JPRB ,&
&  0.91012E+00_JPRB, 0.91002E+00_JPRB, 0.92445E+00_JPRB, 0.95198E+00_JPRB, 0.11093E+01_JPRB, 0.14091E+01_JPRB, 0.18815E+01_JPRB ,&
&  0.16855E+01_JPRB, 0.13746E+01_JPRB, 0.12546E+01_JPRB, 0.10872E+01_JPRB, 0.10784E+01_JPRB, 0.11090E+01_JPRB, 0.10687E+01_JPRB ,&
&  0.10600E+01_JPRB, 0.10360E+01_JPRB, 0.10461E+01_JPRB, 0.10833E+01_JPRB, 0.11170E+01_JPRB, 0.11094E+01_JPRB, 0.10928E+01_JPRB ,&
&  0.10103E+01_JPRB, 0.98671E+00_JPRB, 0.10505E+01_JPRB, 0.12233E+01_JPRB, 0.15751E+01_JPRB, 0.18281E+01_JPRB, 0.17534E+01_JPRB ,&
&  0.19254E+01_JPRB, 0.22040E+01_JPRB, 0.22423E+01_JPRB, 0.22257E+01_JPRB, 0.21370E+01_JPRB, 0.20022E+01_JPRB, 0.19129E+01_JPRB ,&
&  0.18713E+01_JPRB, 0.16617E+01_JPRB, 0.12551E+01_JPRB, 0.13382E+01_JPRB, 0.13127E+01_JPRB, 0.13829E+01_JPRB, 0.12687E+01_JPRB ,&
&  0.11291E+01_JPRB, 0.10405E+01_JPRB, 0.95929E+00_JPRB, 0.87975E+00_JPRB, 0.84635E+00_JPRB, 0.88257E+00_JPRB, 0.88948E+00_JPRB ,&
&  0.84368E+00_JPRB, 0.85360E+00_JPRB, 0.84379E+00_JPRB, 0.82494E+00_JPRB, 0.91614E+00_JPRB, 0.77813E+00_JPRB, 0.89315E+00_JPRB ,&
&  0.78515E+00_JPRB, 0.72764E+00_JPRB, 0.59969E+00_JPRB, 0.82889E+00_JPRB, 0.67933E+00_JPRB, 0.65472E+00_JPRB, 0.68572E+00_JPRB ,&
&  0.72321E+00_JPRB, 0.74053E+00_JPRB, 0.76362E+00_JPRB, 0.78152E+00_JPRB, 0.77493E+00_JPRB, 0.76579E+00_JPRB, 0.77534E+00_JPRB ,&
&  0.78403E+00_JPRB, 0.78436E+00_JPRB, 0.78688E+00_JPRB, 0.78967E+00_JPRB, 0.80346E+00_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.88891E+00_JPRB, 0.90520E+00_JPRB, 0.91814E+00_JPRB, 0.92662E+00_JPRB, 0.93023E+00_JPRB, 0.93348E+00_JPRB, 0.93802E+00_JPRB ,&
&  0.93949E+00_JPRB, 0.93737E+00_JPRB, 0.93554E+00_JPRB, 0.95036E+00_JPRB, 0.97000E+00_JPRB, 0.97838E+00_JPRB, 0.10000E+01_JPRB ,&
&  0.99489E+00_JPRB, 0.10067E+01_JPRB, 0.10253E+01_JPRB, 0.10379E+01_JPRB, 0.10603E+01_JPRB, 0.10601E+01_JPRB, 0.10452E+01_JPRB ,&
&  0.10411E+01_JPRB, 0.10326E+01_JPRB, 0.10329E+01_JPRB, 0.10346E+01_JPRB, 0.10465E+01_JPRB, 0.10611E+01_JPRB, 0.12209E+01_JPRB ,&
&  0.12008E+01_JPRB, 0.14672E+01_JPRB, 0.13689E+01_JPRB, 0.12239E+01_JPRB, 0.12215E+01_JPRB, 0.11219E+01_JPRB, 0.10733E+01_JPRB ,&
&  0.10032E+01_JPRB, 0.93367E+00_JPRB, 0.90197E+00_JPRB, 0.89243E+00_JPRB, 0.93297E+00_JPRB, 0.98652E+00_JPRB, 0.10127E+01_JPRB ,&
&  0.10819E+01_JPRB, 0.11294E+01_JPRB, 0.11849E+01_JPRB, 0.12279E+01_JPRB, 0.12629E+01_JPRB, 0.13767E+01_JPRB, 0.13116E+01_JPRB ,&
&  0.13402E+01_JPRB, 0.14829E+01_JPRB, 0.15169E+01_JPRB, 0.15516E+01_JPRB, 0.14557E+01_JPRB, 0.13460E+01_JPRB, 0.12064E+01_JPRB ,&
&  0.10662E+01_JPRB, 0.10291E+01_JPRB, 0.11359E+01_JPRB, 0.12005E+01_JPRB, 0.12286E+01_JPRB, 0.12704E+01_JPRB, 0.11673E+01_JPRB ,&
&  0.10674E+01_JPRB, 0.99649E+00_JPRB, 0.91797E+00_JPRB, 0.84237E+00_JPRB, 0.81084E+00_JPRB, 0.80670E+00_JPRB, 0.79136E+00_JPRB ,&
&  0.76171E+00_JPRB, 0.74595E+00_JPRB, 0.74206E+00_JPRB, 0.74672E+00_JPRB, 0.71478E+00_JPRB, 0.95692E+00_JPRB, 0.11755E+01_JPRB ,&
&  0.80156E+00_JPRB, 0.62965E+00_JPRB, 0.47432E+00_JPRB, 0.65947E+00_JPRB, 0.60736E+00_JPRB, 0.63954E+00_JPRB, 0.71340E+00_JPRB ,&
&  0.80923E+00_JPRB, 0.88089E+00_JPRB, 0.90538E+00_JPRB, 0.93105E+00_JPRB, 0.82805E+00_JPRB, 0.80830E+00_JPRB, 0.80268E+00_JPRB ,&
&  0.80889E+00_JPRB, 0.82202E+00_JPRB, 0.83629E+00_JPRB, 0.85169E+00_JPRB, 0.87053E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.92964E+00_JPRB, 0.94058E+00_JPRB, 0.95043E+00_JPRB, 0.96414E+00_JPRB, 0.97674E+00_JPRB, 0.99314E+00_JPRB, 0.10040E+01_JPRB ,&
&  0.10148E+01_JPRB, 0.10219E+01_JPRB, 0.10218E+01_JPRB, 0.10349E+01_JPRB, 0.10469E+01_JPRB, 0.10701E+01_JPRB, 0.10967E+01_JPRB ,&
&  0.11265E+01_JPRB, 0.11572E+01_JPRB, 0.11918E+01_JPRB, 0.12102E+01_JPRB, 0.12238E+01_JPRB, 0.12404E+01_JPRB, 0.12557E+01_JPRB ,&
&  0.12650E+01_JPRB, 0.12695E+01_JPRB, 0.12863E+01_JPRB, 0.13161E+01_JPRB, 0.13521E+01_JPRB, 0.13946E+01_JPRB, 0.11991E+01_JPRB ,&
&  0.13820E+01_JPRB, 0.15373E+01_JPRB, 0.15259E+01_JPRB, 0.14168E+01_JPRB, 0.13895E+01_JPRB, 0.13759E+01_JPRB, 0.13926E+01_JPRB ,&
&  0.14396E+01_JPRB, 0.14002E+01_JPRB, 0.13464E+01_JPRB, 0.12536E+01_JPRB, 0.12076E+01_JPRB, 0.12353E+01_JPRB, 0.12904E+01_JPRB ,&
&  0.13266E+01_JPRB, 0.13642E+01_JPRB, 0.13695E+01_JPRB, 0.13608E+01_JPRB, 0.13601E+01_JPRB, 0.13421E+01_JPRB, 0.12671E+01_JPRB ,&
&  0.12001E+01_JPRB, 0.11815E+01_JPRB, 0.10273E+01_JPRB, 0.10196E+01_JPRB, 0.10097E+01_JPRB, 0.92028E+00_JPRB, 0.79024E+00_JPRB ,&
&  0.74528E+00_JPRB, 0.71701E+00_JPRB, 0.78899E+00_JPRB, 0.10992E+01_JPRB, 0.11761E+01_JPRB, 0.11449E+01_JPRB, 0.10801E+01_JPRB ,&
&  0.99112E+00_JPRB, 0.91149E+00_JPRB, 0.81644E+00_JPRB, 0.75759E+00_JPRB, 0.73479E+00_JPRB, 0.75305E+00_JPRB, 0.75951E+00_JPRB ,&
&  0.74863E+00_JPRB, 0.72921E+00_JPRB, 0.71581E+00_JPRB, 0.69971E+00_JPRB, 0.73581E+00_JPRB, 0.77212E+00_JPRB, 0.10617E+01_JPRB ,&
&  0.81893E+00_JPRB, 0.52105E+00_JPRB, 0.51046E+00_JPRB, 0.59425E+00_JPRB, 0.55147E+00_JPRB, 0.59872E+00_JPRB, 0.70909E+00_JPRB ,&
&  0.91305E+00_JPRB, 0.10334E+01_JPRB, 0.10179E+01_JPRB, 0.98850E+00_JPRB, 0.93321E+00_JPRB, 0.93158E+00_JPRB, 0.91057E+00_JPRB ,&
&  0.89529E+00_JPRB, 0.89490E+00_JPRB, 0.89922E+00_JPRB, 0.90505E+00_JPRB, 0.91629E+00_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.95285E+00_JPRB, 0.96581E+00_JPRB, 0.97728E+00_JPRB, 0.98832E+00_JPRB, 0.99670E+00_JPRB, 0.10143E+01_JPRB, 0.10206E+01_JPRB ,&
&  0.10314E+01_JPRB, 0.10367E+01_JPRB, 0.10515E+01_JPRB, 0.10710E+01_JPRB, 0.10876E+01_JPRB, 0.11114E+01_JPRB, 0.11410E+01_JPRB ,&
&  0.11763E+01_JPRB, 0.12007E+01_JPRB, 0.12293E+01_JPRB, 0.12410E+01_JPRB, 0.12297E+01_JPRB, 0.12317E+01_JPRB, 0.12308E+01_JPRB ,&
&  0.12160E+01_JPRB, 0.12074E+01_JPRB, 0.12194E+01_JPRB, 0.12371E+01_JPRB, 0.12341E+01_JPRB, 0.12392E+01_JPRB, 0.96596E+00_JPRB ,&
&  0.14415E+01_JPRB, 0.16534E+01_JPRB, 0.17162E+01_JPRB, 0.16115E+01_JPRB, 0.15595E+01_JPRB, 0.15978E+01_JPRB, 0.16558E+01_JPRB ,&
&  0.17405E+01_JPRB, 0.18221E+01_JPRB, 0.17844E+01_JPRB, 0.16871E+01_JPRB, 0.16060E+01_JPRB, 0.16128E+01_JPRB, 0.16319E+01_JPRB ,&
&  0.16552E+01_JPRB, 0.16741E+01_JPRB, 0.16665E+01_JPRB, 0.16461E+01_JPRB, 0.16251E+01_JPRB, 0.15966E+01_JPRB, 0.15385E+01_JPRB ,&
&  0.14810E+01_JPRB, 0.14266E+01_JPRB, 0.12653E+01_JPRB, 0.13506E+01_JPRB, 0.12587E+01_JPRB, 0.10564E+01_JPRB, 0.85865E+00_JPRB ,&
&  0.75300E+00_JPRB, 0.75502E+00_JPRB, 0.93015E+00_JPRB, 0.10879E+01_JPRB, 0.10638E+01_JPRB, 0.10206E+01_JPRB, 0.96864E+00_JPRB ,&
&  0.87490E+00_JPRB, 0.78928E+00_JPRB, 0.73002E+00_JPRB, 0.70569E+00_JPRB, 0.70636E+00_JPRB, 0.72664E+00_JPRB, 0.73720E+00_JPRB ,&
&  0.73466E+00_JPRB, 0.73237E+00_JPRB, 0.71701E+00_JPRB, 0.70177E+00_JPRB, 0.69567E+00_JPRB, 0.66299E+00_JPRB, 0.90078E+00_JPRB ,&
&  0.78645E+00_JPRB, 0.58941E+00_JPRB, 0.69764E+00_JPRB, 0.41689E+00_JPRB, 0.51221E+00_JPRB, 0.53386E+00_JPRB, 0.63246E+00_JPRB ,&
&  0.76483E+00_JPRB, 0.10880E+01_JPRB, 0.15086E+01_JPRB, 0.13221E+01_JPRB, 0.11924E+01_JPRB, 0.10811E+01_JPRB, 0.98860E+00_JPRB ,&
&  0.95094E+00_JPRB, 0.93798E+00_JPRB, 0.93433E+00_JPRB, 0.93069E+00_JPRB, 0.93903E+00_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.98747E+00_JPRB, 0.10031E+01_JPRB, 0.10020E+01_JPRB, 0.10158E+01_JPRB, 0.99418E+00_JPRB, 0.10103E+01_JPRB, 0.10102E+01_JPRB ,&
&  0.10272E+01_JPRB, 0.10363E+01_JPRB, 0.10537E+01_JPRB, 0.10693E+01_JPRB, 0.10949E+01_JPRB, 0.11179E+01_JPRB, 0.11359E+01_JPRB ,&
&  0.11614E+01_JPRB, 0.11889E+01_JPRB, 0.12111E+01_JPRB, 0.12289E+01_JPRB, 0.12114E+01_JPRB, 0.12148E+01_JPRB, 0.12102E+01_JPRB ,&
&  0.12294E+01_JPRB, 0.12194E+01_JPRB, 0.12299E+01_JPRB, 0.12347E+01_JPRB, 0.12209E+01_JPRB, 0.12374E+01_JPRB, 0.91921E+00_JPRB ,&
&  0.15086E+01_JPRB, 0.17321E+01_JPRB, 0.18446E+01_JPRB, 0.17710E+01_JPRB, 0.16940E+01_JPRB, 0.16887E+01_JPRB, 0.17137E+01_JPRB ,&
&  0.17878E+01_JPRB, 0.18475E+01_JPRB, 0.18305E+01_JPRB, 0.19430E+01_JPRB, 0.19947E+01_JPRB, 0.19537E+01_JPRB, 0.20028E+01_JPRB ,&
&  0.20432E+01_JPRB, 0.20805E+01_JPRB, 0.21122E+01_JPRB, 0.21381E+01_JPRB, 0.21508E+01_JPRB, 0.22046E+01_JPRB, 0.22295E+01_JPRB ,&
&  0.22076E+01_JPRB, 0.22058E+01_JPRB, 0.22012E+01_JPRB, 0.20809E+01_JPRB, 0.19486E+01_JPRB, 0.16350E+01_JPRB, 0.13456E+01_JPRB ,&
&  0.97254E+00_JPRB, 0.87491E+00_JPRB, 0.95344E+00_JPRB, 0.10616E+01_JPRB, 0.10210E+01_JPRB, 0.95830E+00_JPRB, 0.87277E+00_JPRB ,&
&  0.77364E+00_JPRB, 0.74248E+00_JPRB, 0.71464E+00_JPRB, 0.70934E+00_JPRB, 0.71683E+00_JPRB, 0.73324E+00_JPRB, 0.74222E+00_JPRB ,&
&  0.74876E+00_JPRB, 0.75258E+00_JPRB, 0.72950E+00_JPRB, 0.70301E+00_JPRB, 0.69559E+00_JPRB, 0.73541E+00_JPRB, 0.69906E+00_JPRB ,&
&  0.81225E+00_JPRB, 0.67338E+00_JPRB, 0.63746E+00_JPRB, 0.59917E+00_JPRB, 0.54296E+00_JPRB, 0.57032E+00_JPRB, 0.74195E+00_JPRB ,&
&  0.12282E+01_JPRB, 0.16058E+01_JPRB, 0.23195E+01_JPRB, 0.39449E+01_JPRB, 0.27850E+01_JPRB, 0.20567E+01_JPRB, 0.10556E+01_JPRB ,&
&  0.98836E+00_JPRB, 0.96506E+00_JPRB, 0.95272E+00_JPRB, 0.94406E+00_JPRB, 0.96388E+00_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.92286E+00_JPRB, 0.94591E+00_JPRB, 0.97135E+00_JPRB, 0.99338E+00_JPRB, 0.99943E+00_JPRB, 0.10002E+01_JPRB, 0.99811E+00_JPRB ,&
&  0.10216E+01_JPRB, 0.99924E+00_JPRB, 0.10217E+01_JPRB, 0.99773E+00_JPRB, 0.10175E+01_JPRB, 0.10016E+01_JPRB, 0.10067E+01_JPRB ,&
&  0.10193E+01_JPRB, 0.10466E+01_JPRB, 0.10809E+01_JPRB, 0.10954E+01_JPRB, 0.10970E+01_JPRB, 0.11287E+01_JPRB, 0.11744E+01_JPRB ,&
&  0.12304E+01_JPRB, 0.12579E+01_JPRB, 0.12976E+01_JPRB, 0.13177E+01_JPRB, 0.12987E+01_JPRB, 0.12780E+01_JPRB, 0.93351E+00_JPRB ,&
&  0.14337E+01_JPRB, 0.17357E+01_JPRB, 0.19393E+01_JPRB, 0.19260E+01_JPRB, 0.18176E+01_JPRB, 0.17214E+01_JPRB, 0.17363E+01_JPRB ,&
&  0.17702E+01_JPRB, 0.17737E+01_JPRB, 0.17697E+01_JPRB, 0.18274E+01_JPRB, 0.20746E+01_JPRB, 0.20371E+01_JPRB, 0.20910E+01_JPRB ,&
&  0.21503E+01_JPRB, 0.22209E+01_JPRB, 0.23123E+01_JPRB, 0.24240E+01_JPRB, 0.25285E+01_JPRB, 0.26334E+01_JPRB, 0.27502E+01_JPRB ,&
&  0.28336E+01_JPRB, 0.28604E+01_JPRB, 0.28892E+01_JPRB, 0.25738E+01_JPRB, 0.23642E+01_JPRB, 0.23560E+01_JPRB, 0.17891E+01_JPRB ,&
&  0.11199E+01_JPRB, 0.97060E+00_JPRB, 0.10347E+01_JPRB, 0.10833E+01_JPRB, 0.10426E+01_JPRB, 0.92681E+00_JPRB, 0.84658E+00_JPRB ,&
&  0.79118E+00_JPRB, 0.76525E+00_JPRB, 0.74348E+00_JPRB, 0.75726E+00_JPRB, 0.76171E+00_JPRB, 0.77424E+00_JPRB, 0.78436E+00_JPRB ,&
&  0.79568E+00_JPRB, 0.77400E+00_JPRB, 0.74341E+00_JPRB, 0.72599E+00_JPRB, 0.73397E+00_JPRB, 0.80717E+00_JPRB, 0.93613E+00_JPRB ,&
&  0.10091E+01_JPRB, 0.63748E+00_JPRB, 0.67101E+00_JPRB, 0.64568E+00_JPRB, 0.65101E+00_JPRB, 0.71245E+00_JPRB, 0.88904E+00_JPRB ,&
&  0.13418E+01_JPRB, 0.20672E+01_JPRB, 0.23399E+01_JPRB, 0.20116E+01_JPRB, 0.24810E+01_JPRB, 0.17769E+01_JPRB, 0.98795E+00_JPRB ,&
&  0.96756E+00_JPRB, 0.93785E+00_JPRB, 0.89991E+00_JPRB, 0.88869E+00_JPRB, 0.89731E+00_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.77233E+00_JPRB, 0.78683E+00_JPRB, 0.80839E+00_JPRB, 0.84287E+00_JPRB, 0.88073E+00_JPRB, 0.90643E+00_JPRB, 0.93190E+00_JPRB ,&
&  0.94082E+00_JPRB, 0.95384E+00_JPRB, 0.96459E+00_JPRB, 0.92147E+00_JPRB, 0.93091E+00_JPRB, 0.91351E+00_JPRB, 0.90614E+00_JPRB ,&
&  0.90184E+00_JPRB, 0.91743E+00_JPRB, 0.93772E+00_JPRB, 0.95725E+00_JPRB, 0.98797E+00_JPRB, 0.10275E+01_JPRB, 0.10816E+01_JPRB ,&
&  0.11487E+01_JPRB, 0.12082E+01_JPRB, 0.12794E+01_JPRB, 0.13208E+01_JPRB, 0.13611E+01_JPRB, 0.13804E+01_JPRB, 0.13522E+01_JPRB ,&
&  0.87032E+00_JPRB, 0.15384E+01_JPRB, 0.17290E+01_JPRB, 0.18316E+01_JPRB, 0.18985E+01_JPRB, 0.17995E+01_JPRB, 0.17972E+01_JPRB ,&
&  0.17374E+01_JPRB, 0.16341E+01_JPRB, 0.16722E+01_JPRB, 0.17776E+01_JPRB, 0.19044E+01_JPRB, 0.18864E+01_JPRB, 0.19034E+01_JPRB ,&
&  0.19381E+01_JPRB, 0.20153E+01_JPRB, 0.21187E+01_JPRB, 0.22386E+01_JPRB, 0.23870E+01_JPRB, 0.25614E+01_JPRB, 0.27829E+01_JPRB ,&
&  0.30063E+01_JPRB, 0.32101E+01_JPRB, 0.33834E+01_JPRB, 0.30172E+01_JPRB, 0.26638E+01_JPRB, 0.27059E+01_JPRB, 0.22815E+01_JPRB ,&
&  0.13826E+01_JPRB, 0.10163E+01_JPRB, 0.10739E+01_JPRB, 0.11896E+01_JPRB, 0.11112E+01_JPRB, 0.10138E+01_JPRB, 0.92083E+00_JPRB ,&
&  0.86956E+00_JPRB, 0.82880E+00_JPRB, 0.84559E+00_JPRB, 0.81638E+00_JPRB, 0.84232E+00_JPRB, 0.84009E+00_JPRB, 0.84982E+00_JPRB ,&
&  0.85684E+00_JPRB, 0.82892E+00_JPRB, 0.81103E+00_JPRB, 0.79925E+00_JPRB, 0.78130E+00_JPRB, 0.76540E+00_JPRB, 0.75612E+00_JPRB ,&
&  0.76629E+00_JPRB, 0.73025E+00_JPRB, 0.71312E+00_JPRB, 0.73197E+00_JPRB, 0.76199E+00_JPRB, 0.80349E+00_JPRB, 0.89923E+00_JPRB ,&
&  0.10572E+01_JPRB, 0.12234E+01_JPRB, 0.15072E+01_JPRB, 0.16016E+01_JPRB, 0.15518E+01_JPRB, 0.11711E+01_JPRB, 0.86123E+00_JPRB ,&
&  0.85551E+00_JPRB, 0.81208E+00_JPRB, 0.77714E+00_JPRB, 0.75739E+00_JPRB, 0.76005E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.70000E+00_JPRB, 0.70989E+00_JPRB, 0.70284E+00_JPRB, 0.72541E+00_JPRB, 0.74481E+00_JPRB, 0.77384E+00_JPRB, 0.80169E+00_JPRB ,&
&  0.82523E+00_JPRB, 0.84738E+00_JPRB, 0.87620E+00_JPRB, 0.85405E+00_JPRB, 0.86211E+00_JPRB, 0.84927E+00_JPRB, 0.82380E+00_JPRB ,&
&  0.81772E+00_JPRB, 0.82877E+00_JPRB, 0.84214E+00_JPRB, 0.85593E+00_JPRB, 0.87090E+00_JPRB, 0.90078E+00_JPRB, 0.93945E+00_JPRB ,&
&  0.98826E+00_JPRB, 0.10505E+01_JPRB, 0.11329E+01_JPRB, 0.12012E+01_JPRB, 0.12666E+01_JPRB, 0.12770E+01_JPRB, 0.11938E+01_JPRB ,&
&  0.59528E+00_JPRB, 0.84378E+00_JPRB, 0.14144E+01_JPRB, 0.16180E+01_JPRB, 0.17140E+01_JPRB, 0.17542E+01_JPRB, 0.17433E+01_JPRB ,&
&  0.16364E+01_JPRB, 0.14873E+01_JPRB, 0.15084E+01_JPRB, 0.17269E+01_JPRB, 0.16973E+01_JPRB, 0.16959E+01_JPRB, 0.16914E+01_JPRB ,&
&  0.17043E+01_JPRB, 0.17510E+01_JPRB, 0.18254E+01_JPRB, 0.19183E+01_JPRB, 0.20716E+01_JPRB, 0.22709E+01_JPRB, 0.25338E+01_JPRB ,&
&  0.28558E+01_JPRB, 0.31877E+01_JPRB, 0.34859E+01_JPRB, 0.30827E+01_JPRB, 0.30041E+01_JPRB, 0.29605E+01_JPRB, 0.30509E+01_JPRB ,&
&  0.17270E+01_JPRB, 0.11268E+01_JPRB, 0.11807E+01_JPRB, 0.12894E+01_JPRB, 0.11764E+01_JPRB, 0.11072E+01_JPRB, 0.10591E+01_JPRB ,&
&  0.10096E+01_JPRB, 0.95190E+00_JPRB, 0.93427E+00_JPRB, 0.90751E+00_JPRB, 0.89399E+00_JPRB, 0.90367E+00_JPRB, 0.92429E+00_JPRB ,&
&  0.91866E+00_JPRB, 0.90704E+00_JPRB, 0.90144E+00_JPRB, 0.90139E+00_JPRB, 0.87341E+00_JPRB, 0.84677E+00_JPRB, 0.83885E+00_JPRB ,&
&  0.84152E+00_JPRB, 0.82261E+00_JPRB, 0.83127E+00_JPRB, 0.86500E+00_JPRB, 0.90297E+00_JPRB, 0.92607E+00_JPRB, 0.94300E+00_JPRB ,&
&  0.93654E+00_JPRB, 0.95570E+00_JPRB, 0.10860E+01_JPRB, 0.11385E+01_JPRB, 0.11358E+01_JPRB, 0.95921E+00_JPRB, 0.79336E+00_JPRB ,&
&  0.77398E+00_JPRB, 0.74620E+00_JPRB, 0.71989E+00_JPRB, 0.71043E+00_JPRB, 0.70054E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.64164E+00_JPRB, 0.62323E+00_JPRB, 0.60918E+00_JPRB, 0.61387E+00_JPRB, 0.62288E+00_JPRB, 0.64225E+00_JPRB, 0.69140E+00_JPRB ,&
&  0.73025E+00_JPRB, 0.74617E+00_JPRB, 0.76679E+00_JPRB, 0.76705E+00_JPRB, 0.77554E+00_JPRB, 0.75968E+00_JPRB, 0.75490E+00_JPRB ,&
&  0.74101E+00_JPRB, 0.74843E+00_JPRB, 0.76433E+00_JPRB, 0.78028E+00_JPRB, 0.79267E+00_JPRB, 0.80350E+00_JPRB, 0.81903E+00_JPRB ,&
&  0.84105E+00_JPRB, 0.87420E+00_JPRB, 0.92827E+00_JPRB, 0.10072E+01_JPRB, 0.11283E+01_JPRB, 0.12698E+01_JPRB, 0.13497E+01_JPRB ,&
&  0.10031E+01_JPRB, 0.48712E+00_JPRB, 0.91191E+00_JPRB, 0.14082E+01_JPRB, 0.14817E+01_JPRB, 0.15407E+01_JPRB, 0.15067E+01_JPRB ,&
&  0.14034E+01_JPRB, 0.13379E+01_JPRB, 0.13168E+01_JPRB, 0.15152E+01_JPRB, 0.15347E+01_JPRB, 0.15402E+01_JPRB, 0.15362E+01_JPRB ,&
&  0.15239E+01_JPRB, 0.15350E+01_JPRB, 0.15610E+01_JPRB, 0.16231E+01_JPRB, 0.17428E+01_JPRB, 0.19328E+01_JPRB, 0.21816E+01_JPRB ,&
&  0.25179E+01_JPRB, 0.29766E+01_JPRB, 0.34194E+01_JPRB, 0.31663E+01_JPRB, 0.32409E+01_JPRB, 0.28784E+01_JPRB, 0.24974E+01_JPRB ,&
&  0.16089E+01_JPRB, 0.12116E+01_JPRB, 0.12332E+01_JPRB, 0.13667E+01_JPRB, 0.12492E+01_JPRB, 0.10785E+01_JPRB, 0.11321E+01_JPRB ,&
&  0.10704E+01_JPRB, 0.10205E+01_JPRB, 0.99222E+00_JPRB, 0.95898E+00_JPRB, 0.94458E+00_JPRB, 0.93969E+00_JPRB, 0.94112E+00_JPRB ,&
&  0.93561E+00_JPRB, 0.94001E+00_JPRB, 0.94724E+00_JPRB, 0.94163E+00_JPRB, 0.91941E+00_JPRB, 0.91279E+00_JPRB, 0.92517E+00_JPRB ,&
&  0.93761E+00_JPRB, 0.93179E+00_JPRB, 0.94735E+00_JPRB, 0.96617E+00_JPRB, 0.98569E+00_JPRB, 0.96565E+00_JPRB, 0.96500E+00_JPRB ,&
&  0.95403E+00_JPRB, 0.95405E+00_JPRB, 0.95494E+00_JPRB, 0.94534E+00_JPRB, 0.93921E+00_JPRB, 0.87535E+00_JPRB, 0.79379E+00_JPRB ,&
&  0.76698E+00_JPRB, 0.73634E+00_JPRB, 0.71250E+00_JPRB, 0.69135E+00_JPRB, 0.66823E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.68034E+00_JPRB, 0.66343E+00_JPRB, 0.63332E+00_JPRB, 0.59477E+00_JPRB, 0.58950E+00_JPRB, 0.58667E+00_JPRB, 0.59923E+00_JPRB ,&
&  0.63915E+00_JPRB, 0.69048E+00_JPRB, 0.69776E+00_JPRB, 0.69289E+00_JPRB, 0.67412E+00_JPRB, 0.67067E+00_JPRB, 0.67037E+00_JPRB ,&
&  0.67571E+00_JPRB, 0.67753E+00_JPRB, 0.69196E+00_JPRB, 0.71252E+00_JPRB, 0.73601E+00_JPRB, 0.75329E+00_JPRB, 0.75485E+00_JPRB ,&
&  0.75861E+00_JPRB, 0.76116E+00_JPRB, 0.77824E+00_JPRB, 0.80447E+00_JPRB, 0.85707E+00_JPRB, 0.95621E+00_JPRB, 0.11113E+01_JPRB ,&
&  0.14969E+01_JPRB, 0.12040E+01_JPRB, 0.63533E+00_JPRB, 0.11660E+01_JPRB, 0.13575E+01_JPRB, 0.14623E+01_JPRB, 0.12827E+01_JPRB ,&
&  0.12361E+01_JPRB, 0.11329E+01_JPRB, 0.11038E+01_JPRB, 0.12313E+01_JPRB, 0.12615E+01_JPRB, 0.13007E+01_JPRB, 0.13106E+01_JPRB ,&
&  0.12725E+01_JPRB, 0.12838E+01_JPRB, 0.13014E+01_JPRB, 0.13377E+01_JPRB, 0.14035E+01_JPRB, 0.15465E+01_JPRB, 0.18018E+01_JPRB ,&
&  0.21202E+01_JPRB, 0.25415E+01_JPRB, 0.31603E+01_JPRB, 0.34189E+01_JPRB, 0.37048E+01_JPRB, 0.28016E+01_JPRB, 0.22019E+01_JPRB ,&
&  0.15122E+01_JPRB, 0.14013E+01_JPRB, 0.14716E+01_JPRB, 0.14690E+01_JPRB, 0.13357E+01_JPRB, 0.11942E+01_JPRB, 0.12433E+01_JPRB ,&
&  0.11577E+01_JPRB, 0.10937E+01_JPRB, 0.10623E+01_JPRB, 0.10262E+01_JPRB, 0.99853E+00_JPRB, 0.98214E+00_JPRB, 0.97204E+00_JPRB ,&
&  0.96112E+00_JPRB, 0.96663E+00_JPRB, 0.97201E+00_JPRB, 0.97692E+00_JPRB, 0.97870E+00_JPRB, 0.97268E+00_JPRB, 0.98523E+00_JPRB ,&
&  0.99522E+00_JPRB, 0.10015E+01_JPRB, 0.10116E+01_JPRB, 0.10280E+01_JPRB, 0.10042E+01_JPRB, 0.96188E+00_JPRB, 0.96909E+00_JPRB ,&
&  0.96322E+00_JPRB, 0.97459E+00_JPRB, 0.96211E+00_JPRB, 0.98030E+00_JPRB, 0.94285E+00_JPRB, 0.87907E+00_JPRB, 0.81528E+00_JPRB ,&
&  0.81302E+00_JPRB, 0.80705E+00_JPRB, 0.76642E+00_JPRB, 0.70843E+00_JPRB, 0.68525E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.73146E+00_JPRB, 0.73555E+00_JPRB, 0.72672E+00_JPRB, 0.68679E+00_JPRB, 0.64882E+00_JPRB, 0.61185E+00_JPRB, 0.58540E+00_JPRB ,&
&  0.58127E+00_JPRB, 0.58998E+00_JPRB, 0.61885E+00_JPRB, 0.63947E+00_JPRB, 0.63059E+00_JPRB, 0.61100E+00_JPRB, 0.60360E+00_JPRB ,&
&  0.60789E+00_JPRB, 0.62324E+00_JPRB, 0.63290E+00_JPRB, 0.65063E+00_JPRB, 0.67040E+00_JPRB, 0.68806E+00_JPRB, 0.70731E+00_JPRB ,&
&  0.69513E+00_JPRB, 0.69715E+00_JPRB, 0.70827E+00_JPRB, 0.71702E+00_JPRB, 0.73454E+00_JPRB, 0.77311E+00_JPRB, 0.80460E+00_JPRB ,&
&  0.93380E+00_JPRB, 0.14292E+01_JPRB, 0.89089E+00_JPRB, 0.12167E+01_JPRB, 0.13965E+01_JPRB, 0.14006E+01_JPRB, 0.12662E+01_JPRB ,&
&  0.11462E+01_JPRB, 0.10088E+01_JPRB, 0.10169E+01_JPRB, 0.10456E+01_JPRB, 0.97990E+00_JPRB, 0.99703E+00_JPRB, 0.10192E+01_JPRB ,&
&  0.10222E+01_JPRB, 0.10509E+01_JPRB, 0.10736E+01_JPRB, 0.10992E+01_JPRB, 0.11181E+01_JPRB, 0.11715E+01_JPRB, 0.13232E+01_JPRB ,&
&  0.16093E+01_JPRB, 0.20000E+01_JPRB, 0.26601E+01_JPRB, 0.32300E+01_JPRB, 0.39100E+01_JPRB, 0.30733E+01_JPRB, 0.23109E+01_JPRB ,&
&  0.17349E+01_JPRB, 0.16753E+01_JPRB, 0.16918E+01_JPRB, 0.16656E+01_JPRB, 0.14932E+01_JPRB, 0.14472E+01_JPRB, 0.14785E+01_JPRB ,&
&  0.13335E+01_JPRB, 0.12155E+01_JPRB, 0.11587E+01_JPRB, 0.11163E+01_JPRB, 0.10700E+01_JPRB, 0.10361E+01_JPRB, 0.10148E+01_JPRB ,&
&  0.99736E+00_JPRB, 0.99384E+00_JPRB, 0.10059E+01_JPRB, 0.10127E+01_JPRB, 0.10229E+01_JPRB, 0.10231E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.10111E+01_JPRB, 0.10079E+01_JPRB, 0.10110E+01_JPRB, 0.10072E+01_JPRB, 0.98638E+00_JPRB, 0.95752E+00_JPRB, 0.95277E+00_JPRB ,&
&  0.95657E+00_JPRB, 0.98061E+00_JPRB, 0.97674E+00_JPRB, 0.95882E+00_JPRB, 0.99663E+00_JPRB, 0.95080E+00_JPRB, 0.89271E+00_JPRB ,&
&  0.92935E+00_JPRB, 0.10766E+01_JPRB, 0.11278E+01_JPRB, 0.77428E+00_JPRB, 0.74176E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.74490E+00_JPRB, 0.74235E+00_JPRB, 0.72695E+00_JPRB, 0.72151E+00_JPRB, 0.68796E+00_JPRB, 0.65361E+00_JPRB, 0.62343E+00_JPRB ,&
&  0.60457E+00_JPRB, 0.59576E+00_JPRB, 0.60228E+00_JPRB, 0.60377E+00_JPRB, 0.60455E+00_JPRB, 0.59677E+00_JPRB, 0.59126E+00_JPRB ,&
&  0.57798E+00_JPRB, 0.58137E+00_JPRB, 0.59483E+00_JPRB, 0.61684E+00_JPRB, 0.64214E+00_JPRB, 0.65127E+00_JPRB, 0.66134E+00_JPRB ,&
&  0.66178E+00_JPRB, 0.65739E+00_JPRB, 0.65753E+00_JPRB, 0.66521E+00_JPRB, 0.68158E+00_JPRB, 0.69933E+00_JPRB, 0.72105E+00_JPRB ,&
&  0.75585E+00_JPRB, 0.12614E+01_JPRB, 0.18930E+01_JPRB, 0.21459E+01_JPRB, 0.18356E+01_JPRB, 0.15075E+01_JPRB, 0.13314E+01_JPRB ,&
&  0.12217E+01_JPRB, 0.11103E+01_JPRB, 0.11581E+01_JPRB, 0.10302E+01_JPRB, 0.89651E+00_JPRB, 0.84753E+00_JPRB, 0.86025E+00_JPRB ,&
&  0.90890E+00_JPRB, 0.92564E+00_JPRB, 0.93619E+00_JPRB, 0.94024E+00_JPRB, 0.92560E+00_JPRB, 0.92900E+00_JPRB, 0.97073E+00_JPRB ,&
&  0.10994E+01_JPRB, 0.13721E+01_JPRB, 0.18601E+01_JPRB, 0.26991E+01_JPRB, 0.36090E+01_JPRB, 0.32512E+01_JPRB, 0.26586E+01_JPRB ,&
&  0.24709E+01_JPRB, 0.22765E+01_JPRB, 0.20830E+01_JPRB, 0.19678E+01_JPRB, 0.16061E+01_JPRB, 0.18896E+01_JPRB, 0.17797E+01_JPRB ,&
&  0.16495E+01_JPRB, 0.15559E+01_JPRB, 0.14254E+01_JPRB, 0.12941E+01_JPRB, 0.11956E+01_JPRB, 0.11058E+01_JPRB, 0.10411E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10251E+01_JPRB, 0.10377E+01_JPRB, 0.10593E+01_JPRB, 0.10837E+01_JPRB, 0.10939E+01_JPRB, 0.10791E+01_JPRB ,&
&  0.10354E+01_JPRB, 0.99944E+00_JPRB, 0.94104E+00_JPRB, 0.93165E+00_JPRB, 0.92366E+00_JPRB, 0.89669E+00_JPRB, 0.88019E+00_JPRB ,&
&  0.89906E+00_JPRB, 0.94036E+00_JPRB, 0.95723E+00_JPRB, 0.95405E+00_JPRB, 0.95080E+00_JPRB, 0.95848E+00_JPRB, 0.89392E+00_JPRB ,&
&  0.87007E+00_JPRB, 0.92318E+00_JPRB, 0.88370E+00_JPRB, 0.80573E+00_JPRB, 0.77068E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.75591E+00_JPRB, 0.73128E+00_JPRB, 0.71523E+00_JPRB, 0.71745E+00_JPRB, 0.69540E+00_JPRB, 0.68036E+00_JPRB, 0.67482E+00_JPRB ,&
&  0.64037E+00_JPRB, 0.61669E+00_JPRB, 0.59941E+00_JPRB, 0.59642E+00_JPRB, 0.59884E+00_JPRB, 0.59245E+00_JPRB, 0.59338E+00_JPRB ,&
&  0.58407E+00_JPRB, 0.57383E+00_JPRB, 0.57466E+00_JPRB, 0.59399E+00_JPRB, 0.62107E+00_JPRB, 0.62883E+00_JPRB, 0.64298E+00_JPRB ,&
&  0.65719E+00_JPRB, 0.65211E+00_JPRB, 0.64168E+00_JPRB, 0.63962E+00_JPRB, 0.64312E+00_JPRB, 0.63865E+00_JPRB, 0.65208E+00_JPRB ,&
&  0.69302E+00_JPRB, 0.13074E+01_JPRB, 0.14355E+01_JPRB, 0.23582E+01_JPRB, 0.20351E+01_JPRB, 0.17050E+01_JPRB, 0.13108E+01_JPRB ,&
&  0.12562E+01_JPRB, 0.14094E+01_JPRB, 0.12526E+01_JPRB, 0.11100E+01_JPRB, 0.95560E+00_JPRB, 0.83838E+00_JPRB, 0.81108E+00_JPRB ,&
&  0.83779E+00_JPRB, 0.86391E+00_JPRB, 0.87873E+00_JPRB, 0.85235E+00_JPRB, 0.82163E+00_JPRB, 0.81276E+00_JPRB, 0.82474E+00_JPRB ,&
&  0.85917E+00_JPRB, 0.94880E+00_JPRB, 0.12008E+01_JPRB, 0.17079E+01_JPRB, 0.28355E+01_JPRB, 0.33702E+01_JPRB, 0.30857E+01_JPRB ,&
&  0.29727E+01_JPRB, 0.30927E+01_JPRB, 0.28046E+01_JPRB, 0.24851E+01_JPRB, 0.20900E+01_JPRB, 0.20746E+01_JPRB, 0.19637E+01_JPRB ,&
&  0.18499E+01_JPRB, 0.18132E+01_JPRB, 0.17453E+01_JPRB, 0.16159E+01_JPRB, 0.14665E+01_JPRB, 0.13122E+01_JPRB, 0.11701E+01_JPRB ,&
&  0.10743E+01_JPRB, 0.10318E+01_JPRB, 0.10143E+01_JPRB, 0.10394E+01_JPRB, 0.10886E+01_JPRB, 0.11233E+01_JPRB, 0.11322E+01_JPRB ,&
&  0.10961E+01_JPRB, 0.10416E+01_JPRB, 0.93325E+00_JPRB, 0.87716E+00_JPRB, 0.87350E+00_JPRB, 0.84867E+00_JPRB, 0.83704E+00_JPRB ,&
&  0.87156E+00_JPRB, 0.90410E+00_JPRB, 0.91540E+00_JPRB, 0.90744E+00_JPRB, 0.91312E+00_JPRB, 0.95682E+00_JPRB, 0.90526E+00_JPRB ,&
&  0.87032E+00_JPRB, 0.89116E+00_JPRB, 0.89893E+00_JPRB, 0.83557E+00_JPRB, 0.79712E+00_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.79180E+00_JPRB, 0.75187E+00_JPRB, 0.72332E+00_JPRB, 0.71156E+00_JPRB, 0.70378E+00_JPRB, 0.69201E+00_JPRB, 0.68185E+00_JPRB ,&
&  0.66178E+00_JPRB, 0.63768E+00_JPRB, 0.62050E+00_JPRB, 0.60559E+00_JPRB, 0.60629E+00_JPRB, 0.60060E+00_JPRB, 0.59154E+00_JPRB ,&
&  0.57915E+00_JPRB, 0.57344E+00_JPRB, 0.57236E+00_JPRB, 0.58021E+00_JPRB, 0.59829E+00_JPRB, 0.60939E+00_JPRB, 0.62286E+00_JPRB ,&
&  0.63016E+00_JPRB, 0.63887E+00_JPRB, 0.63936E+00_JPRB, 0.63422E+00_JPRB, 0.62584E+00_JPRB, 0.61334E+00_JPRB, 0.59673E+00_JPRB ,&
&  0.67016E+00_JPRB, 0.14301E+01_JPRB, 0.12369E+01_JPRB, 0.21157E+01_JPRB, 0.18471E+01_JPRB, 0.16545E+01_JPRB, 0.13044E+01_JPRB ,&
&  0.12228E+01_JPRB, 0.13257E+01_JPRB, 0.12430E+01_JPRB, 0.11433E+01_JPRB, 0.10443E+01_JPRB, 0.94222E+00_JPRB, 0.87443E+00_JPRB ,&
&  0.84972E+00_JPRB, 0.84437E+00_JPRB, 0.83279E+00_JPRB, 0.80818E+00_JPRB, 0.79499E+00_JPRB, 0.78155E+00_JPRB, 0.78304E+00_JPRB ,&
&  0.79361E+00_JPRB, 0.82067E+00_JPRB, 0.91547E+00_JPRB, 0.12439E+01_JPRB, 0.19246E+01_JPRB, 0.28603E+01_JPRB, 0.34977E+01_JPRB ,&
&  0.36359E+01_JPRB, 0.39758E+01_JPRB, 0.35488E+01_JPRB, 0.30400E+01_JPRB, 0.26532E+01_JPRB, 0.24139E+01_JPRB, 0.22060E+01_JPRB ,&
&  0.19977E+01_JPRB, 0.18739E+01_JPRB, 0.18347E+01_JPRB, 0.18048E+01_JPRB, 0.17421E+01_JPRB, 0.16243E+01_JPRB, 0.14701E+01_JPRB ,&
&  0.13106E+01_JPRB, 0.12008E+01_JPRB, 0.11238E+01_JPRB, 0.10907E+01_JPRB, 0.10916E+01_JPRB, 0.11122E+01_JPRB, 0.11237E+01_JPRB ,&
&  0.10907E+01_JPRB, 0.10510E+01_JPRB, 0.96176E+00_JPRB, 0.88849E+00_JPRB, 0.87720E+00_JPRB, 0.85477E+00_JPRB, 0.83505E+00_JPRB ,&
&  0.86812E+00_JPRB, 0.88953E+00_JPRB, 0.88518E+00_JPRB, 0.88495E+00_JPRB, 0.87616E+00_JPRB, 0.94952E+00_JPRB, 0.89582E+00_JPRB ,&
&  0.86617E+00_JPRB, 0.86385E+00_JPRB, 0.86650E+00_JPRB, 0.84620E+00_JPRB, 0.82848E+00_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.82330E+00_JPRB, 0.79281E+00_JPRB, 0.76349E+00_JPRB, 0.74280E+00_JPRB, 0.72507E+00_JPRB, 0.71305E+00_JPRB, 0.70057E+00_JPRB ,&
&  0.67701E+00_JPRB, 0.65415E+00_JPRB, 0.64380E+00_JPRB, 0.63577E+00_JPRB, 0.62823E+00_JPRB, 0.62225E+00_JPRB, 0.60553E+00_JPRB ,&
&  0.58827E+00_JPRB, 0.56983E+00_JPRB, 0.57840E+00_JPRB, 0.58996E+00_JPRB, 0.60124E+00_JPRB, 0.61064E+00_JPRB, 0.62759E+00_JPRB ,&
&  0.62247E+00_JPRB, 0.62196E+00_JPRB, 0.62399E+00_JPRB, 0.62483E+00_JPRB, 0.61492E+00_JPRB, 0.61471E+00_JPRB, 0.58672E+00_JPRB ,&
&  0.64485E+00_JPRB, 0.69312E+00_JPRB, 0.13116E+01_JPRB, 0.17964E+01_JPRB, 0.16620E+01_JPRB, 0.14957E+01_JPRB, 0.13459E+01_JPRB ,&
&  0.13311E+01_JPRB, 0.12564E+01_JPRB, 0.11473E+01_JPRB, 0.10804E+01_JPRB, 0.10185E+01_JPRB, 0.96882E+00_JPRB, 0.92946E+00_JPRB ,&
&  0.89967E+00_JPRB, 0.86000E+00_JPRB, 0.82864E+00_JPRB, 0.79639E+00_JPRB, 0.77874E+00_JPRB, 0.76710E+00_JPRB, 0.76617E+00_JPRB ,&
&  0.76792E+00_JPRB, 0.77051E+00_JPRB, 0.79832E+00_JPRB, 0.96454E+00_JPRB, 0.15379E+01_JPRB, 0.23329E+01_JPRB, 0.32575E+01_JPRB ,&
&  0.40230E+01_JPRB, 0.42807E+01_JPRB, 0.38886E+01_JPRB, 0.34221E+01_JPRB, 0.29839E+01_JPRB, 0.26534E+01_JPRB, 0.24002E+01_JPRB ,&
&  0.21694E+01_JPRB, 0.19740E+01_JPRB, 0.18252E+01_JPRB, 0.17470E+01_JPRB, 0.16992E+01_JPRB, 0.16594E+01_JPRB, 0.16043E+01_JPRB ,&
&  0.15232E+01_JPRB, 0.14137E+01_JPRB, 0.13039E+01_JPRB, 0.12137E+01_JPRB, 0.11533E+01_JPRB, 0.11314E+01_JPRB, 0.11146E+01_JPRB ,&
&  0.10679E+01_JPRB, 0.10268E+01_JPRB, 0.96694E+00_JPRB, 0.92226E+00_JPRB, 0.92553E+00_JPRB, 0.91505E+00_JPRB, 0.90088E+00_JPRB ,&
&  0.89652E+00_JPRB, 0.90209E+00_JPRB, 0.88971E+00_JPRB, 0.89270E+00_JPRB, 0.86572E+00_JPRB, 0.93588E+00_JPRB, 0.90434E+00_JPRB ,&
&  0.87127E+00_JPRB, 0.86451E+00_JPRB, 0.85008E+00_JPRB, 0.84185E+00_JPRB, 0.83949E+00_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.85213E+00_JPRB, 0.83877E+00_JPRB, 0.80417E+00_JPRB, 0.77040E+00_JPRB, 0.74715E+00_JPRB, 0.73093E+00_JPRB, 0.71655E+00_JPRB ,&
&  0.69764E+00_JPRB, 0.67270E+00_JPRB, 0.65570E+00_JPRB, 0.64994E+00_JPRB, 0.65006E+00_JPRB, 0.64730E+00_JPRB, 0.63091E+00_JPRB ,&
&  0.60987E+00_JPRB, 0.59425E+00_JPRB, 0.59636E+00_JPRB, 0.60344E+00_JPRB, 0.61494E+00_JPRB, 0.61072E+00_JPRB, 0.62068E+00_JPRB ,&
&  0.62553E+00_JPRB, 0.62193E+00_JPRB, 0.61889E+00_JPRB, 0.61662E+00_JPRB, 0.61195E+00_JPRB, 0.60825E+00_JPRB, 0.58814E+00_JPRB ,&
&  0.61447E+00_JPRB, 0.75806E+00_JPRB, 0.11790E+01_JPRB, 0.15107E+01_JPRB, 0.15203E+01_JPRB, 0.13674E+01_JPRB, 0.12759E+01_JPRB ,&
&  0.12010E+01_JPRB, 0.11422E+01_JPRB, 0.10492E+01_JPRB, 0.98888E+00_JPRB, 0.93714E+00_JPRB, 0.90199E+00_JPRB, 0.87808E+00_JPRB ,&
&  0.86422E+00_JPRB, 0.84717E+00_JPRB, 0.82030E+00_JPRB, 0.79409E+00_JPRB, 0.77533E+00_JPRB, 0.75994E+00_JPRB, 0.75378E+00_JPRB ,&
&  0.75405E+00_JPRB, 0.75091E+00_JPRB, 0.75662E+00_JPRB, 0.82438E+00_JPRB, 0.12442E+01_JPRB, 0.15628E+01_JPRB, 0.26385E+01_JPRB ,&
&  0.35308E+01_JPRB, 0.33221E+01_JPRB, 0.31595E+01_JPRB, 0.29033E+01_JPRB, 0.25923E+01_JPRB, 0.23402E+01_JPRB, 0.20839E+01_JPRB ,&
&  0.18632E+01_JPRB, 0.16880E+01_JPRB, 0.15455E+01_JPRB, 0.14493E+01_JPRB, 0.13885E+01_JPRB, 0.13558E+01_JPRB, 0.13445E+01_JPRB ,&
&  0.13326E+01_JPRB, 0.13186E+01_JPRB, 0.12936E+01_JPRB, 0.12540E+01_JPRB, 0.11948E+01_JPRB, 0.11345E+01_JPRB, 0.10777E+01_JPRB ,&
&  0.10177E+01_JPRB, 0.98320E+00_JPRB, 0.94797E+00_JPRB, 0.92702E+00_JPRB, 0.95286E+00_JPRB, 0.98230E+00_JPRB, 0.96750E+00_JPRB ,&
&  0.92758E+00_JPRB, 0.91606E+00_JPRB, 0.91592E+00_JPRB, 0.88966E+00_JPRB, 0.86482E+00_JPRB, 0.94157E+00_JPRB, 0.92308E+00_JPRB ,&
&  0.89357E+00_JPRB, 0.87552E+00_JPRB, 0.86050E+00_JPRB, 0.84233E+00_JPRB, 0.85484E+00_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.87974E+00_JPRB, 0.84930E+00_JPRB, 0.82052E+00_JPRB, 0.78776E+00_JPRB, 0.76514E+00_JPRB, 0.74069E+00_JPRB, 0.71696E+00_JPRB ,&
&  0.69649E+00_JPRB, 0.68482E+00_JPRB, 0.67446E+00_JPRB, 0.66283E+00_JPRB, 0.66344E+00_JPRB, 0.65812E+00_JPRB, 0.64469E+00_JPRB ,&
&  0.62438E+00_JPRB, 0.60915E+00_JPRB, 0.60370E+00_JPRB, 0.60511E+00_JPRB, 0.61550E+00_JPRB, 0.62231E+00_JPRB, 0.63012E+00_JPRB ,&
&  0.63403E+00_JPRB, 0.61428E+00_JPRB, 0.60426E+00_JPRB, 0.60603E+00_JPRB, 0.60707E+00_JPRB, 0.61030E+00_JPRB, 0.59475E+00_JPRB ,&
&  0.60337E+00_JPRB, 0.67268E+00_JPRB, 0.91862E+00_JPRB, 0.12425E+01_JPRB, 0.12869E+01_JPRB, 0.12243E+01_JPRB, 0.11574E+01_JPRB ,&
&  0.10953E+01_JPRB, 0.10376E+01_JPRB, 0.96205E+00_JPRB, 0.88090E+00_JPRB, 0.81992E+00_JPRB, 0.78513E+00_JPRB, 0.76878E+00_JPRB ,&
&  0.76753E+00_JPRB, 0.76744E+00_JPRB, 0.75948E+00_JPRB, 0.75312E+00_JPRB, 0.74462E+00_JPRB, 0.73730E+00_JPRB, 0.73460E+00_JPRB ,&
&  0.73025E+00_JPRB, 0.72392E+00_JPRB, 0.72039E+00_JPRB, 0.73527E+00_JPRB, 0.93372E+00_JPRB, 0.11405E+01_JPRB, 0.16146E+01_JPRB ,&
&  0.18180E+01_JPRB, 0.18964E+01_JPRB, 0.19714E+01_JPRB, 0.19484E+01_JPRB, 0.18433E+01_JPRB, 0.17388E+01_JPRB, 0.16326E+01_JPRB ,&
&  0.15147E+01_JPRB, 0.13899E+01_JPRB, 0.13036E+01_JPRB, 0.12401E+01_JPRB, 0.11805E+01_JPRB, 0.11359E+01_JPRB, 0.11072E+01_JPRB ,&
&  0.11019E+01_JPRB, 0.11192E+01_JPRB, 0.11359E+01_JPRB, 0.11446E+01_JPRB, 0.11276E+01_JPRB, 0.10961E+01_JPRB, 0.10524E+01_JPRB ,&
&  0.99228E+00_JPRB, 0.94614E+00_JPRB, 0.92874E+00_JPRB, 0.93391E+00_JPRB, 0.94779E+00_JPRB, 0.95213E+00_JPRB, 0.95801E+00_JPRB ,&
&  0.96237E+00_JPRB, 0.95346E+00_JPRB, 0.93930E+00_JPRB, 0.92006E+00_JPRB, 0.89719E+00_JPRB, 0.95801E+00_JPRB, 0.94617E+00_JPRB ,&
&  0.92094E+00_JPRB, 0.89544E+00_JPRB, 0.87885E+00_JPRB, 0.85843E+00_JPRB, 0.89841E+00_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.95507E+00_JPRB, 0.89814E+00_JPRB, 0.85210E+00_JPRB, 0.81911E+00_JPRB, 0.79790E+00_JPRB, 0.76947E+00_JPRB, 0.73447E+00_JPRB ,&
&  0.70867E+00_JPRB, 0.69976E+00_JPRB, 0.69001E+00_JPRB, 0.67377E+00_JPRB, 0.66242E+00_JPRB, 0.64991E+00_JPRB, 0.63867E+00_JPRB ,&
&  0.62223E+00_JPRB, 0.61565E+00_JPRB, 0.60935E+00_JPRB, 0.61131E+00_JPRB, 0.61068E+00_JPRB, 0.61439E+00_JPRB, 0.62505E+00_JPRB ,&
&  0.62742E+00_JPRB, 0.60791E+00_JPRB, 0.60508E+00_JPRB, 0.60643E+00_JPRB, 0.60676E+00_JPRB, 0.60739E+00_JPRB, 0.59642E+00_JPRB ,&
&  0.61116E+00_JPRB, 0.61724E+00_JPRB, 0.77120E+00_JPRB, 0.91622E+00_JPRB, 0.98641E+00_JPRB, 0.10355E+01_JPRB, 0.10249E+01_JPRB ,&
&  0.99628E+00_JPRB, 0.94439E+00_JPRB, 0.88547E+00_JPRB, 0.82305E+00_JPRB, 0.76667E+00_JPRB, 0.73105E+00_JPRB, 0.71298E+00_JPRB ,&
&  0.70389E+00_JPRB, 0.70242E+00_JPRB, 0.69902E+00_JPRB, 0.69761E+00_JPRB, 0.69597E+00_JPRB, 0.69187E+00_JPRB, 0.69408E+00_JPRB ,&
&  0.70077E+00_JPRB, 0.69104E+00_JPRB, 0.68702E+00_JPRB, 0.70037E+00_JPRB, 0.73880E+00_JPRB, 0.81921E+00_JPRB, 0.10339E+01_JPRB ,&
&  0.11689E+01_JPRB, 0.12671E+01_JPRB, 0.13727E+01_JPRB, 0.14115E+01_JPRB, 0.13821E+01_JPRB, 0.13304E+01_JPRB, 0.12545E+01_JPRB ,&
&  0.11968E+01_JPRB, 0.11534E+01_JPRB, 0.11123E+01_JPRB, 0.10704E+01_JPRB, 0.10329E+01_JPRB, 0.10058E+01_JPRB, 0.98744E+00_JPRB ,&
&  0.97425E+00_JPRB, 0.98069E+00_JPRB, 0.99323E+00_JPRB, 0.10039E+01_JPRB, 0.10041E+01_JPRB, 0.99910E+00_JPRB, 0.98832E+00_JPRB ,&
&  0.95825E+00_JPRB, 0.91631E+00_JPRB, 0.89286E+00_JPRB, 0.89824E+00_JPRB, 0.92398E+00_JPRB, 0.93568E+00_JPRB, 0.93967E+00_JPRB ,&
&  0.94990E+00_JPRB, 0.95494E+00_JPRB, 0.92861E+00_JPRB, 0.92152E+00_JPRB, 0.98028E+00_JPRB, 0.96603E+00_JPRB, 0.94571E+00_JPRB ,&
&  0.92037E+00_JPRB, 0.90552E+00_JPRB, 0.88497E+00_JPRB, 0.85964E+00_JPRB, 0.93742E+00_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.81384E+00_JPRB, 0.81663E+00_JPRB, 0.80007E+00_JPRB, 0.78396E+00_JPRB, 0.77578E+00_JPRB, 0.76701E+00_JPRB, 0.74843E+00_JPRB ,&
&  0.72694E+00_JPRB, 0.70713E+00_JPRB, 0.69531E+00_JPRB, 0.67883E+00_JPRB, 0.66266E+00_JPRB, 0.65288E+00_JPRB, 0.64213E+00_JPRB ,&
&  0.62585E+00_JPRB, 0.62068E+00_JPRB, 0.61975E+00_JPRB, 0.61703E+00_JPRB, 0.61186E+00_JPRB, 0.60865E+00_JPRB, 0.61520E+00_JPRB ,&
&  0.61600E+00_JPRB, 0.61464E+00_JPRB, 0.61534E+00_JPRB, 0.61155E+00_JPRB, 0.60605E+00_JPRB, 0.60483E+00_JPRB, 0.59395E+00_JPRB ,&
&  0.59679E+00_JPRB, 0.55629E+00_JPRB, 0.63363E+00_JPRB, 0.72563E+00_JPRB, 0.80060E+00_JPRB, 0.85259E+00_JPRB, 0.90095E+00_JPRB ,&
&  0.91004E+00_JPRB, 0.87276E+00_JPRB, 0.82848E+00_JPRB, 0.79600E+00_JPRB, 0.75898E+00_JPRB, 0.72986E+00_JPRB, 0.70663E+00_JPRB ,&
&  0.69169E+00_JPRB, 0.68495E+00_JPRB, 0.68026E+00_JPRB, 0.67590E+00_JPRB, 0.67128E+00_JPRB, 0.66768E+00_JPRB, 0.66763E+00_JPRB ,&
&  0.66612E+00_JPRB, 0.66788E+00_JPRB, 0.67083E+00_JPRB, 0.67019E+00_JPRB, 0.68062E+00_JPRB, 0.73000E+00_JPRB, 0.82797E+00_JPRB ,&
&  0.89173E+00_JPRB, 0.96220E+00_JPRB, 0.10548E+01_JPRB, 0.11039E+01_JPRB, 0.10911E+01_JPRB, 0.10646E+01_JPRB, 0.10272E+01_JPRB ,&
&  0.99617E+00_JPRB, 0.95911E+00_JPRB, 0.93840E+00_JPRB, 0.91561E+00_JPRB, 0.89047E+00_JPRB, 0.87076E+00_JPRB, 0.86040E+00_JPRB ,&
&  0.85736E+00_JPRB, 0.85729E+00_JPRB, 0.86593E+00_JPRB, 0.87781E+00_JPRB, 0.88124E+00_JPRB, 0.88125E+00_JPRB, 0.88537E+00_JPRB ,&
&  0.88963E+00_JPRB, 0.87880E+00_JPRB, 0.85868E+00_JPRB, 0.86327E+00_JPRB, 0.88629E+00_JPRB, 0.90307E+00_JPRB, 0.91267E+00_JPRB ,&
&  0.92126E+00_JPRB, 0.92987E+00_JPRB, 0.93680E+00_JPRB, 0.95591E+00_JPRB, 0.94895E+00_JPRB, 0.92313E+00_JPRB, 0.90041E+00_JPRB ,&
&  0.88137E+00_JPRB, 0.86242E+00_JPRB, 0.84798E+00_JPRB, 0.83062E+00_JPRB, 0.76402E+00_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.76621E+00_JPRB, 0.76429E+00_JPRB, 0.75257E+00_JPRB, 0.73900E+00_JPRB, 0.73316E+00_JPRB, 0.73150E+00_JPRB, 0.73167E+00_JPRB ,&
&  0.72655E+00_JPRB, 0.71792E+00_JPRB, 0.70754E+00_JPRB, 0.69460E+00_JPRB, 0.68068E+00_JPRB, 0.66919E+00_JPRB, 0.65813E+00_JPRB ,&
&  0.63991E+00_JPRB, 0.62960E+00_JPRB, 0.62941E+00_JPRB, 0.62689E+00_JPRB, 0.62416E+00_JPRB, 0.61973E+00_JPRB, 0.61999E+00_JPRB ,&
&  0.62338E+00_JPRB, 0.62562E+00_JPRB, 0.62699E+00_JPRB, 0.62119E+00_JPRB, 0.61620E+00_JPRB, 0.61587E+00_JPRB, 0.61176E+00_JPRB ,&
&  0.61074E+00_JPRB, 0.50708E+00_JPRB, 0.56276E+00_JPRB, 0.64334E+00_JPRB, 0.68130E+00_JPRB, 0.73807E+00_JPRB, 0.81472E+00_JPRB ,&
&  0.85506E+00_JPRB, 0.84190E+00_JPRB, 0.81243E+00_JPRB, 0.78510E+00_JPRB, 0.75349E+00_JPRB, 0.72404E+00_JPRB, 0.69953E+00_JPRB ,&
&  0.68451E+00_JPRB, 0.67639E+00_JPRB, 0.67075E+00_JPRB, 0.66350E+00_JPRB, 0.65586E+00_JPRB, 0.65364E+00_JPRB, 0.65298E+00_JPRB ,&
&  0.65482E+00_JPRB, 0.66037E+00_JPRB, 0.66388E+00_JPRB, 0.67210E+00_JPRB, 0.67972E+00_JPRB, 0.69974E+00_JPRB, 0.74700E+00_JPRB ,&
&  0.77892E+00_JPRB, 0.81818E+00_JPRB, 0.87480E+00_JPRB, 0.89804E+00_JPRB, 0.89110E+00_JPRB, 0.88061E+00_JPRB, 0.86421E+00_JPRB ,&
&  0.84271E+00_JPRB, 0.81079E+00_JPRB, 0.79494E+00_JPRB, 0.77962E+00_JPRB, 0.76711E+00_JPRB, 0.76096E+00_JPRB, 0.76692E+00_JPRB ,&
&  0.77394E+00_JPRB, 0.77533E+00_JPRB, 0.77922E+00_JPRB, 0.78701E+00_JPRB, 0.79137E+00_JPRB, 0.79027E+00_JPRB, 0.79594E+00_JPRB ,&
&  0.81143E+00_JPRB, 0.83029E+00_JPRB, 0.83590E+00_JPRB, 0.83182E+00_JPRB, 0.83606E+00_JPRB, 0.85127E+00_JPRB, 0.86407E+00_JPRB ,&
&  0.87390E+00_JPRB, 0.87887E+00_JPRB, 0.88826E+00_JPRB, 0.85645E+00_JPRB, 0.89799E+00_JPRB, 0.88523E+00_JPRB, 0.86579E+00_JPRB ,&
&  0.84580E+00_JPRB, 0.82599E+00_JPRB, 0.81671E+00_JPRB, 0.69692E+00_JPRB, 0.73844E+00_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.71552E+00_JPRB, 0.71105E+00_JPRB, 0.70002E+00_JPRB, 0.68802E+00_JPRB, 0.67955E+00_JPRB, 0.67669E+00_JPRB, 0.67594E+00_JPRB ,&
&  0.68014E+00_JPRB, 0.68403E+00_JPRB, 0.68488E+00_JPRB, 0.68188E+00_JPRB, 0.67682E+00_JPRB, 0.66869E+00_JPRB, 0.66198E+00_JPRB ,&
&  0.64957E+00_JPRB, 0.63616E+00_JPRB, 0.63208E+00_JPRB, 0.63272E+00_JPRB, 0.63107E+00_JPRB, 0.62492E+00_JPRB, 0.62169E+00_JPRB ,&
&  0.62116E+00_JPRB, 0.62455E+00_JPRB, 0.61955E+00_JPRB, 0.61640E+00_JPRB, 0.61187E+00_JPRB, 0.61389E+00_JPRB, 0.61508E+00_JPRB ,&
&  0.61429E+00_JPRB, 0.48902E+00_JPRB, 0.54776E+00_JPRB, 0.60010E+00_JPRB, 0.62522E+00_JPRB, 0.67498E+00_JPRB, 0.72889E+00_JPRB ,&
&  0.76727E+00_JPRB, 0.78520E+00_JPRB, 0.77907E+00_JPRB, 0.76380E+00_JPRB, 0.74001E+00_JPRB, 0.71585E+00_JPRB, 0.69359E+00_JPRB ,&
&  0.67259E+00_JPRB, 0.65831E+00_JPRB, 0.64783E+00_JPRB, 0.63902E+00_JPRB, 0.63482E+00_JPRB, 0.63077E+00_JPRB, 0.62689E+00_JPRB ,&
&  0.62915E+00_JPRB, 0.63731E+00_JPRB, 0.63911E+00_JPRB, 0.64271E+00_JPRB, 0.64696E+00_JPRB, 0.65071E+00_JPRB, 0.66925E+00_JPRB ,&
&  0.68652E+00_JPRB, 0.70020E+00_JPRB, 0.71511E+00_JPRB, 0.72403E+00_JPRB, 0.72295E+00_JPRB, 0.72562E+00_JPRB, 0.72300E+00_JPRB ,&
&  0.71870E+00_JPRB, 0.70805E+00_JPRB, 0.69668E+00_JPRB, 0.68752E+00_JPRB, 0.68099E+00_JPRB, 0.67648E+00_JPRB, 0.67924E+00_JPRB ,&
&  0.68669E+00_JPRB, 0.68514E+00_JPRB, 0.68299E+00_JPRB, 0.68462E+00_JPRB, 0.68736E+00_JPRB, 0.69154E+00_JPRB, 0.70055E+00_JPRB ,&
&  0.71721E+00_JPRB, 0.74009E+00_JPRB, 0.75968E+00_JPRB, 0.76734E+00_JPRB, 0.76878E+00_JPRB, 0.77584E+00_JPRB, 0.78602E+00_JPRB ,&
&  0.79725E+00_JPRB, 0.80301E+00_JPRB, 0.80784E+00_JPRB, 0.83042E+00_JPRB, 0.83456E+00_JPRB, 0.82535E+00_JPRB, 0.80986E+00_JPRB ,&
&  0.78534E+00_JPRB, 0.77399E+00_JPRB, 0.62647E+00_JPRB, 0.68561E+00_JPRB, 0.70098E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.68005E+00_JPRB, 0.66994E+00_JPRB, 0.65514E+00_JPRB, 0.64344E+00_JPRB, 0.63441E+00_JPRB, 0.63161E+00_JPRB, 0.63007E+00_JPRB ,&
&  0.63451E+00_JPRB, 0.64208E+00_JPRB, 0.64956E+00_JPRB, 0.65387E+00_JPRB, 0.65704E+00_JPRB, 0.65732E+00_JPRB, 0.65400E+00_JPRB ,&
&  0.65141E+00_JPRB, 0.64081E+00_JPRB, 0.63286E+00_JPRB, 0.63360E+00_JPRB, 0.63137E+00_JPRB, 0.62862E+00_JPRB, 0.62499E+00_JPRB ,&
&  0.62123E+00_JPRB, 0.61830E+00_JPRB, 0.60955E+00_JPRB, 0.60557E+00_JPRB, 0.60480E+00_JPRB, 0.60740E+00_JPRB, 0.61760E+00_JPRB ,&
&  0.54348E+00_JPRB, 0.46395E+00_JPRB, 0.51820E+00_JPRB, 0.57347E+00_JPRB, 0.60267E+00_JPRB, 0.64143E+00_JPRB, 0.67697E+00_JPRB ,&
&  0.72313E+00_JPRB, 0.75171E+00_JPRB, 0.76604E+00_JPRB, 0.76005E+00_JPRB, 0.74283E+00_JPRB, 0.72430E+00_JPRB, 0.70544E+00_JPRB ,&
&  0.68224E+00_JPRB, 0.65544E+00_JPRB, 0.63821E+00_JPRB, 0.63034E+00_JPRB, 0.62365E+00_JPRB, 0.61628E+00_JPRB, 0.61221E+00_JPRB ,&
&  0.61127E+00_JPRB, 0.61078E+00_JPRB, 0.60945E+00_JPRB, 0.60885E+00_JPRB, 0.61103E+00_JPRB, 0.61374E+00_JPRB, 0.61848E+00_JPRB ,&
&  0.62428E+00_JPRB, 0.62981E+00_JPRB, 0.63354E+00_JPRB, 0.63339E+00_JPRB, 0.63405E+00_JPRB, 0.64039E+00_JPRB, 0.64371E+00_JPRB ,&
&  0.64689E+00_JPRB, 0.64634E+00_JPRB, 0.64443E+00_JPRB, 0.64161E+00_JPRB, 0.63651E+00_JPRB, 0.63167E+00_JPRB, 0.62945E+00_JPRB ,&
&  0.63029E+00_JPRB, 0.62697E+00_JPRB, 0.62707E+00_JPRB, 0.63073E+00_JPRB, 0.63571E+00_JPRB, 0.64101E+00_JPRB, 0.64563E+00_JPRB ,&
&  0.65459E+00_JPRB, 0.67126E+00_JPRB, 0.68424E+00_JPRB, 0.69273E+00_JPRB, 0.69836E+00_JPRB, 0.70376E+00_JPRB, 0.71222E+00_JPRB ,&
&  0.72001E+00_JPRB, 0.72770E+00_JPRB, 0.73753E+00_JPRB, 0.74937E+00_JPRB, 0.75308E+00_JPRB, 0.74910E+00_JPRB, 0.73978E+00_JPRB ,&
&  0.72492E+00_JPRB, 0.70533E+00_JPRB, 0.70030E+00_JPRB, 0.69199E+00_JPRB, 0.68512E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.60710E+00_JPRB, 0.59947E+00_JPRB, 0.58831E+00_JPRB, 0.58206E+00_JPRB, 0.57985E+00_JPRB, 0.58329E+00_JPRB, 0.58792E+00_JPRB ,&
&  0.59269E+00_JPRB, 0.59452E+00_JPRB, 0.60187E+00_JPRB, 0.60956E+00_JPRB, 0.61434E+00_JPRB, 0.61777E+00_JPRB, 0.61728E+00_JPRB ,&
&  0.61749E+00_JPRB, 0.61798E+00_JPRB, 0.61415E+00_JPRB, 0.61114E+00_JPRB, 0.60652E+00_JPRB, 0.60770E+00_JPRB, 0.60805E+00_JPRB ,&
&  0.60689E+00_JPRB, 0.60366E+00_JPRB, 0.59719E+00_JPRB, 0.58852E+00_JPRB, 0.58476E+00_JPRB, 0.58435E+00_JPRB, 0.58229E+00_JPRB ,&
&  0.58850E+00_JPRB, 0.50791E+00_JPRB, 0.54438E+00_JPRB, 0.55855E+00_JPRB, 0.57815E+00_JPRB, 0.60461E+00_JPRB, 0.63525E+00_JPRB ,&
&  0.66639E+00_JPRB, 0.69167E+00_JPRB, 0.70031E+00_JPRB, 0.70025E+00_JPRB, 0.68807E+00_JPRB, 0.66899E+00_JPRB, 0.64738E+00_JPRB ,&
&  0.62875E+00_JPRB, 0.61441E+00_JPRB, 0.59696E+00_JPRB, 0.58698E+00_JPRB, 0.57886E+00_JPRB, 0.56849E+00_JPRB, 0.55980E+00_JPRB ,&
&  0.55633E+00_JPRB, 0.55435E+00_JPRB, 0.55443E+00_JPRB, 0.55649E+00_JPRB, 0.55965E+00_JPRB, 0.56202E+00_JPRB, 0.56212E+00_JPRB ,&
&  0.56227E+00_JPRB, 0.56275E+00_JPRB, 0.56227E+00_JPRB, 0.56388E+00_JPRB, 0.56684E+00_JPRB, 0.57229E+00_JPRB, 0.57188E+00_JPRB ,&
&  0.56694E+00_JPRB, 0.56871E+00_JPRB, 0.57146E+00_JPRB, 0.57197E+00_JPRB, 0.57276E+00_JPRB, 0.57333E+00_JPRB, 0.57154E+00_JPRB ,&
&  0.56993E+00_JPRB, 0.57226E+00_JPRB, 0.57516E+00_JPRB, 0.58115E+00_JPRB, 0.58537E+00_JPRB, 0.59430E+00_JPRB, 0.60220E+00_JPRB ,&
&  0.61027E+00_JPRB, 0.62318E+00_JPRB, 0.63609E+00_JPRB, 0.64562E+00_JPRB, 0.65132E+00_JPRB, 0.65593E+00_JPRB, 0.66077E+00_JPRB ,&
&  0.66476E+00_JPRB, 0.66860E+00_JPRB, 0.67267E+00_JPRB, 0.67321E+00_JPRB, 0.66903E+00_JPRB, 0.66341E+00_JPRB, 0.65803E+00_JPRB ,&
&  0.64630E+00_JPRB, 0.63123E+00_JPRB, 0.62348E+00_JPRB, 0.61691E+00_JPRB, 0.61093E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.55940E+00_JPRB, 0.55366E+00_JPRB, 0.54819E+00_JPRB, 0.54734E+00_JPRB, 0.54649E+00_JPRB, 0.54880E+00_JPRB, 0.55187E+00_JPRB ,&
&  0.55878E+00_JPRB, 0.56324E+00_JPRB, 0.57246E+00_JPRB, 0.57997E+00_JPRB, 0.58364E+00_JPRB, 0.58804E+00_JPRB, 0.59098E+00_JPRB ,&
&  0.59226E+00_JPRB, 0.59405E+00_JPRB, 0.59519E+00_JPRB, 0.59979E+00_JPRB, 0.60078E+00_JPRB, 0.60032E+00_JPRB, 0.59953E+00_JPRB ,&
&  0.59780E+00_JPRB, 0.59572E+00_JPRB, 0.59065E+00_JPRB, 0.58364E+00_JPRB, 0.57484E+00_JPRB, 0.57111E+00_JPRB, 0.56522E+00_JPRB ,&
&  0.56650E+00_JPRB, 0.53518E+00_JPRB, 0.54678E+00_JPRB, 0.54516E+00_JPRB, 0.55928E+00_JPRB, 0.58172E+00_JPRB, 0.61299E+00_JPRB ,&
&  0.63489E+00_JPRB, 0.64683E+00_JPRB, 0.64888E+00_JPRB, 0.64715E+00_JPRB, 0.63331E+00_JPRB, 0.61873E+00_JPRB, 0.60227E+00_JPRB ,&
&  0.58358E+00_JPRB, 0.57318E+00_JPRB, 0.55981E+00_JPRB, 0.55005E+00_JPRB, 0.54608E+00_JPRB, 0.54329E+00_JPRB, 0.53869E+00_JPRB ,&
&  0.53167E+00_JPRB, 0.52566E+00_JPRB, 0.52647E+00_JPRB, 0.53011E+00_JPRB, 0.53391E+00_JPRB, 0.53636E+00_JPRB, 0.53540E+00_JPRB ,&
&  0.52924E+00_JPRB, 0.52429E+00_JPRB, 0.52303E+00_JPRB, 0.52390E+00_JPRB, 0.52111E+00_JPRB, 0.51504E+00_JPRB, 0.50864E+00_JPRB ,&
&  0.50754E+00_JPRB, 0.51044E+00_JPRB, 0.51401E+00_JPRB, 0.51854E+00_JPRB, 0.52164E+00_JPRB, 0.52535E+00_JPRB, 0.52695E+00_JPRB ,&
&  0.53139E+00_JPRB, 0.53565E+00_JPRB, 0.54447E+00_JPRB, 0.55293E+00_JPRB, 0.55475E+00_JPRB, 0.56316E+00_JPRB, 0.57200E+00_JPRB ,&
&  0.58099E+00_JPRB, 0.59165E+00_JPRB, 0.60023E+00_JPRB, 0.60650E+00_JPRB, 0.61302E+00_JPRB, 0.61667E+00_JPRB, 0.62225E+00_JPRB ,&
&  0.62973E+00_JPRB, 0.63137E+00_JPRB, 0.63681E+00_JPRB, 0.63668E+00_JPRB, 0.63108E+00_JPRB, 0.62229E+00_JPRB, 0.61043E+00_JPRB ,&
&  0.59711E+00_JPRB, 0.58039E+00_JPRB, 0.57122E+00_JPRB, 0.56545E+00_JPRB, 0.56200E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.49759E+00_JPRB, 0.49505E+00_JPRB, 0.49511E+00_JPRB, 0.49820E+00_JPRB, 0.50171E+00_JPRB, 0.50452E+00_JPRB, 0.51023E+00_JPRB ,&
&  0.52020E+00_JPRB, 0.52807E+00_JPRB, 0.53763E+00_JPRB, 0.54731E+00_JPRB, 0.55470E+00_JPRB, 0.55969E+00_JPRB, 0.56241E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.55835E+00_JPRB, 0.56136E+00_JPRB, 0.56608E+00_JPRB, 0.56758E+00_JPRB, 0.56725E+00_JPRB, 0.56732E+00_JPRB ,&
&  0.56692E+00_JPRB, 0.56475E+00_JPRB, 0.56196E+00_JPRB, 0.55773E+00_JPRB, 0.55271E+00_JPRB, 0.55038E+00_JPRB, 0.54828E+00_JPRB ,&
&  0.54190E+00_JPRB, 0.54152E+00_JPRB, 0.49130E+00_JPRB, 0.50654E+00_JPRB, 0.51954E+00_JPRB, 0.53984E+00_JPRB, 0.55957E+00_JPRB ,&
&  0.57366E+00_JPRB, 0.58272E+00_JPRB, 0.58732E+00_JPRB, 0.58579E+00_JPRB, 0.57360E+00_JPRB, 0.56571E+00_JPRB, 0.55667E+00_JPRB ,&
&  0.54542E+00_JPRB, 0.53739E+00_JPRB, 0.52437E+00_JPRB, 0.51051E+00_JPRB, 0.50585E+00_JPRB, 0.50635E+00_JPRB, 0.50638E+00_JPRB ,&
&  0.50288E+00_JPRB, 0.49966E+00_JPRB, 0.50124E+00_JPRB, 0.50235E+00_JPRB, 0.50224E+00_JPRB, 0.49705E+00_JPRB, 0.49042E+00_JPRB ,&
&  0.48002E+00_JPRB, 0.46989E+00_JPRB, 0.46293E+00_JPRB, 0.45802E+00_JPRB, 0.45393E+00_JPRB, 0.44940E+00_JPRB, 0.44491E+00_JPRB ,&
&  0.44894E+00_JPRB, 0.46055E+00_JPRB, 0.47130E+00_JPRB, 0.48008E+00_JPRB, 0.48583E+00_JPRB, 0.48966E+00_JPRB, 0.49474E+00_JPRB ,&
&  0.50062E+00_JPRB, 0.51075E+00_JPRB, 0.51778E+00_JPRB, 0.52393E+00_JPRB, 0.52837E+00_JPRB, 0.53261E+00_JPRB, 0.53756E+00_JPRB ,&
&  0.54172E+00_JPRB, 0.54871E+00_JPRB, 0.55402E+00_JPRB, 0.56045E+00_JPRB, 0.56434E+00_JPRB, 0.56958E+00_JPRB, 0.57557E+00_JPRB ,&
&  0.58220E+00_JPRB, 0.58799E+00_JPRB, 0.59025E+00_JPRB, 0.58838E+00_JPRB, 0.58082E+00_JPRB, 0.57084E+00_JPRB, 0.56068E+00_JPRB ,&
&  0.54776E+00_JPRB, 0.53285E+00_JPRB, 0.51808E+00_JPRB, 0.50868E+00_JPRB, 0.50072E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.45106E+00_JPRB, 0.45142E+00_JPRB, 0.45585E+00_JPRB, 0.46040E+00_JPRB, 0.46373E+00_JPRB, 0.46859E+00_JPRB, 0.47813E+00_JPRB ,&
&  0.48776E+00_JPRB, 0.49707E+00_JPRB, 0.50349E+00_JPRB, 0.51126E+00_JPRB, 0.52215E+00_JPRB, 0.52951E+00_JPRB, 0.53541E+00_JPRB ,&
&  0.53786E+00_JPRB, 0.53822E+00_JPRB, 0.53923E+00_JPRB, 0.54234E+00_JPRB, 0.54194E+00_JPRB, 0.53837E+00_JPRB, 0.53714E+00_JPRB ,&
&  0.53711E+00_JPRB, 0.53550E+00_JPRB, 0.53537E+00_JPRB, 0.53655E+00_JPRB, 0.53438E+00_JPRB, 0.53074E+00_JPRB, 0.53041E+00_JPRB ,&
&  0.53086E+00_JPRB, 0.51736E+00_JPRB, 0.50991E+00_JPRB, 0.50310E+00_JPRB, 0.50695E+00_JPRB, 0.51861E+00_JPRB, 0.53137E+00_JPRB ,&
&  0.53911E+00_JPRB, 0.54434E+00_JPRB, 0.54409E+00_JPRB, 0.54322E+00_JPRB, 0.53865E+00_JPRB, 0.52725E+00_JPRB, 0.51740E+00_JPRB ,&
&  0.51068E+00_JPRB, 0.50120E+00_JPRB, 0.47877E+00_JPRB, 0.46592E+00_JPRB, 0.46266E+00_JPRB, 0.46528E+00_JPRB, 0.46917E+00_JPRB ,&
&  0.47086E+00_JPRB, 0.46844E+00_JPRB, 0.46254E+00_JPRB, 0.45539E+00_JPRB, 0.44554E+00_JPRB, 0.43416E+00_JPRB, 0.42419E+00_JPRB ,&
&  0.41854E+00_JPRB, 0.41443E+00_JPRB, 0.41103E+00_JPRB, 0.41366E+00_JPRB, 0.41309E+00_JPRB, 0.40952E+00_JPRB, 0.41177E+00_JPRB ,&
&  0.42347E+00_JPRB, 0.43815E+00_JPRB, 0.45165E+00_JPRB, 0.45907E+00_JPRB, 0.46209E+00_JPRB, 0.46558E+00_JPRB, 0.46963E+00_JPRB ,&
&  0.47543E+00_JPRB, 0.48112E+00_JPRB, 0.48618E+00_JPRB, 0.49260E+00_JPRB, 0.50057E+00_JPRB, 0.50773E+00_JPRB, 0.51066E+00_JPRB ,&
&  0.51316E+00_JPRB, 0.51471E+00_JPRB, 0.51600E+00_JPRB, 0.52123E+00_JPRB, 0.52915E+00_JPRB, 0.53467E+00_JPRB, 0.53912E+00_JPRB ,&
&  0.54375E+00_JPRB, 0.54585E+00_JPRB, 0.54440E+00_JPRB, 0.54021E+00_JPRB, 0.53461E+00_JPRB, 0.52356E+00_JPRB, 0.50981E+00_JPRB ,&
&  0.49396E+00_JPRB, 0.47941E+00_JPRB, 0.46868E+00_JPRB, 0.45942E+00_JPRB, 0.45299E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.41602E+00_JPRB, 0.41569E+00_JPRB, 0.41839E+00_JPRB, 0.42456E+00_JPRB, 0.42879E+00_JPRB, 0.43273E+00_JPRB, 0.44047E+00_JPRB ,&
&  0.44748E+00_JPRB, 0.45367E+00_JPRB, 0.46111E+00_JPRB, 0.46930E+00_JPRB, 0.47564E+00_JPRB, 0.48472E+00_JPRB, 0.49330E+00_JPRB ,&
&  0.49878E+00_JPRB, 0.50097E+00_JPRB, 0.50220E+00_JPRB, 0.50326E+00_JPRB, 0.50306E+00_JPRB, 0.50049E+00_JPRB, 0.50004E+00_JPRB ,&
&  0.49825E+00_JPRB, 0.49628E+00_JPRB, 0.49852E+00_JPRB, 0.49897E+00_JPRB, 0.50078E+00_JPRB, 0.50371E+00_JPRB, 0.50402E+00_JPRB ,&
&  0.50216E+00_JPRB, 0.48935E+00_JPRB, 0.48475E+00_JPRB, 0.48200E+00_JPRB, 0.48639E+00_JPRB, 0.48765E+00_JPRB, 0.48643E+00_JPRB ,&
&  0.48031E+00_JPRB, 0.47661E+00_JPRB, 0.47860E+00_JPRB, 0.48105E+00_JPRB, 0.47539E+00_JPRB, 0.46244E+00_JPRB, 0.45204E+00_JPRB ,&
&  0.44375E+00_JPRB, 0.43137E+00_JPRB, 0.41356E+00_JPRB, 0.40311E+00_JPRB, 0.39869E+00_JPRB, 0.39377E+00_JPRB, 0.38921E+00_JPRB ,&
&  0.38677E+00_JPRB, 0.38600E+00_JPRB, 0.38747E+00_JPRB, 0.38663E+00_JPRB, 0.38297E+00_JPRB, 0.37848E+00_JPRB, 0.37672E+00_JPRB ,&
&  0.37939E+00_JPRB, 0.38192E+00_JPRB, 0.38557E+00_JPRB, 0.38821E+00_JPRB, 0.38770E+00_JPRB, 0.38828E+00_JPRB, 0.39226E+00_JPRB ,&
&  0.39842E+00_JPRB, 0.40434E+00_JPRB, 0.41010E+00_JPRB, 0.41254E+00_JPRB, 0.41568E+00_JPRB, 0.42242E+00_JPRB, 0.42290E+00_JPRB ,&
&  0.42137E+00_JPRB, 0.42320E+00_JPRB, 0.42724E+00_JPRB, 0.43503E+00_JPRB, 0.44708E+00_JPRB, 0.45487E+00_JPRB, 0.45590E+00_JPRB ,&
&  0.45585E+00_JPRB, 0.45803E+00_JPRB, 0.46191E+00_JPRB, 0.47132E+00_JPRB, 0.48145E+00_JPRB, 0.48866E+00_JPRB, 0.49164E+00_JPRB ,&
&  0.48847E+00_JPRB, 0.48167E+00_JPRB, 0.46944E+00_JPRB, 0.45515E+00_JPRB, 0.44252E+00_JPRB, 0.43267E+00_JPRB, 0.42464E+00_JPRB ,&
&  0.41674E+00_JPRB, 0.41334E+00_JPRB, 0.41304E+00_JPRB, 0.41492E+00_JPRB, 0.41587E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.38923E+00_JPRB, 0.39098E+00_JPRB, 0.39314E+00_JPRB, 0.39602E+00_JPRB, 0.39919E+00_JPRB, 0.40120E+00_JPRB, 0.40513E+00_JPRB ,&
&  0.40732E+00_JPRB, 0.41372E+00_JPRB, 0.42430E+00_JPRB, 0.43498E+00_JPRB, 0.44279E+00_JPRB, 0.44947E+00_JPRB, 0.45596E+00_JPRB ,&
&  0.46121E+00_JPRB, 0.46819E+00_JPRB, 0.46902E+00_JPRB, 0.46652E+00_JPRB, 0.46380E+00_JPRB, 0.46171E+00_JPRB, 0.45986E+00_JPRB ,&
&  0.45738E+00_JPRB, 0.45730E+00_JPRB, 0.46112E+00_JPRB, 0.46543E+00_JPRB, 0.46990E+00_JPRB, 0.47125E+00_JPRB, 0.47008E+00_JPRB ,&
&  0.47390E+00_JPRB, 0.46463E+00_JPRB, 0.45866E+00_JPRB, 0.45160E+00_JPRB, 0.44538E+00_JPRB, 0.43405E+00_JPRB, 0.42576E+00_JPRB ,&
&  0.41591E+00_JPRB, 0.40731E+00_JPRB, 0.40369E+00_JPRB, 0.39980E+00_JPRB, 0.39019E+00_JPRB, 0.37887E+00_JPRB, 0.37039E+00_JPRB ,&
&  0.36473E+00_JPRB, 0.35899E+00_JPRB, 0.35220E+00_JPRB, 0.34670E+00_JPRB, 0.34222E+00_JPRB, 0.34071E+00_JPRB, 0.34253E+00_JPRB ,&
&  0.34663E+00_JPRB, 0.35038E+00_JPRB, 0.35250E+00_JPRB, 0.35293E+00_JPRB, 0.35133E+00_JPRB, 0.34875E+00_JPRB, 0.34723E+00_JPRB ,&
&  0.34789E+00_JPRB, 0.34943E+00_JPRB, 0.35167E+00_JPRB, 0.35689E+00_JPRB, 0.35775E+00_JPRB, 0.35644E+00_JPRB, 0.35115E+00_JPRB ,&
&  0.34896E+00_JPRB, 0.34797E+00_JPRB, 0.34980E+00_JPRB, 0.35498E+00_JPRB, 0.36335E+00_JPRB, 0.37137E+00_JPRB, 0.37704E+00_JPRB ,&
&  0.37985E+00_JPRB, 0.38154E+00_JPRB, 0.38457E+00_JPRB, 0.38814E+00_JPRB, 0.38950E+00_JPRB, 0.38869E+00_JPRB, 0.38706E+00_JPRB ,&
&  0.38974E+00_JPRB, 0.39709E+00_JPRB, 0.40495E+00_JPRB, 0.41007E+00_JPRB, 0.41255E+00_JPRB, 0.41146E+00_JPRB, 0.40450E+00_JPRB ,&
&  0.39193E+00_JPRB, 0.37755E+00_JPRB, 0.36354E+00_JPRB, 0.35168E+00_JPRB, 0.34351E+00_JPRB, 0.33963E+00_JPRB, 0.34257E+00_JPRB ,&
&  0.35040E+00_JPRB, 0.36043E+00_JPRB, 0.36971E+00_JPRB, 0.37666E+00_JPRB, 0.38414E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.36264E+00_JPRB, 0.36243E+00_JPRB, 0.36387E+00_JPRB, 0.36271E+00_JPRB, 0.36003E+00_JPRB, 0.35807E+00_JPRB, 0.36166E+00_JPRB ,&
&  0.36855E+00_JPRB, 0.38007E+00_JPRB, 0.39537E+00_JPRB, 0.40883E+00_JPRB, 0.41699E+00_JPRB, 0.42202E+00_JPRB, 0.42969E+00_JPRB ,&
&  0.43680E+00_JPRB, 0.44472E+00_JPRB, 0.44487E+00_JPRB, 0.43601E+00_JPRB, 0.42849E+00_JPRB, 0.42451E+00_JPRB, 0.42398E+00_JPRB ,&
&  0.42333E+00_JPRB, 0.42517E+00_JPRB, 0.42841E+00_JPRB, 0.43188E+00_JPRB, 0.43129E+00_JPRB, 0.42953E+00_JPRB, 0.43322E+00_JPRB ,&
&  0.43135E+00_JPRB, 0.42185E+00_JPRB, 0.41225E+00_JPRB, 0.38742E+00_JPRB, 0.38688E+00_JPRB, 0.37668E+00_JPRB, 0.36878E+00_JPRB ,&
&  0.35816E+00_JPRB, 0.34510E+00_JPRB, 0.33564E+00_JPRB, 0.32959E+00_JPRB, 0.32693E+00_JPRB, 0.32436E+00_JPRB, 0.32225E+00_JPRB ,&
&  0.32079E+00_JPRB, 0.32039E+00_JPRB, 0.32154E+00_JPRB, 0.31922E+00_JPRB, 0.31803E+00_JPRB, 0.31883E+00_JPRB, 0.32053E+00_JPRB ,&
&  0.32022E+00_JPRB, 0.31985E+00_JPRB, 0.31953E+00_JPRB, 0.31846E+00_JPRB, 0.31679E+00_JPRB, 0.31436E+00_JPRB, 0.31205E+00_JPRB ,&
&  0.31114E+00_JPRB, 0.31230E+00_JPRB, 0.31277E+00_JPRB, 0.30912E+00_JPRB, 0.30164E+00_JPRB, 0.29264E+00_JPRB, 0.28693E+00_JPRB ,&
&  0.28471E+00_JPRB, 0.28701E+00_JPRB, 0.29496E+00_JPRB, 0.30618E+00_JPRB, 0.31659E+00_JPRB, 0.32268E+00_JPRB, 0.32628E+00_JPRB ,&
&  0.33052E+00_JPRB, 0.32872E+00_JPRB, 0.32493E+00_JPRB, 0.32150E+00_JPRB, 0.31837E+00_JPRB, 0.31431E+00_JPRB, 0.31419E+00_JPRB ,&
&  0.31670E+00_JPRB, 0.31915E+00_JPRB, 0.31986E+00_JPRB, 0.31873E+00_JPRB, 0.31670E+00_JPRB, 0.31245E+00_JPRB, 0.30644E+00_JPRB ,&
&  0.29904E+00_JPRB, 0.29043E+00_JPRB, 0.28836E+00_JPRB, 0.28879E+00_JPRB, 0.29250E+00_JPRB, 0.30027E+00_JPRB, 0.31092E+00_JPRB ,&
&  0.32211E+00_JPRB, 0.33140E+00_JPRB, 0.34143E+00_JPRB, 0.35076E+00_JPRB, 0.35905E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.34595E+00_JPRB, 0.34742E+00_JPRB, 0.34484E+00_JPRB, 0.34193E+00_JPRB, 0.34152E+00_JPRB, 0.34316E+00_JPRB, 0.34891E+00_JPRB ,&
&  0.35864E+00_JPRB, 0.37115E+00_JPRB, 0.38311E+00_JPRB, 0.39304E+00_JPRB, 0.40206E+00_JPRB, 0.40604E+00_JPRB, 0.41203E+00_JPRB ,&
&  0.41847E+00_JPRB, 0.42436E+00_JPRB, 0.42512E+00_JPRB, 0.41485E+00_JPRB, 0.40485E+00_JPRB, 0.39952E+00_JPRB, 0.39951E+00_JPRB ,&
&  0.40131E+00_JPRB, 0.40398E+00_JPRB, 0.41019E+00_JPRB, 0.41252E+00_JPRB, 0.40668E+00_JPRB, 0.40530E+00_JPRB, 0.40316E+00_JPRB ,&
&  0.39763E+00_JPRB, 0.38641E+00_JPRB, 0.35413E+00_JPRB, 0.34864E+00_JPRB, 0.34049E+00_JPRB, 0.33147E+00_JPRB, 0.32241E+00_JPRB ,&
&  0.31215E+00_JPRB, 0.30191E+00_JPRB, 0.29496E+00_JPRB, 0.29091E+00_JPRB, 0.29167E+00_JPRB, 0.29290E+00_JPRB, 0.29237E+00_JPRB ,&
&  0.29433E+00_JPRB, 0.29694E+00_JPRB, 0.29908E+00_JPRB, 0.29922E+00_JPRB, 0.29867E+00_JPRB, 0.29739E+00_JPRB, 0.29271E+00_JPRB ,&
&  0.28849E+00_JPRB, 0.28618E+00_JPRB, 0.28436E+00_JPRB, 0.28280E+00_JPRB, 0.28254E+00_JPRB, 0.28294E+00_JPRB, 0.28309E+00_JPRB ,&
&  0.28105E+00_JPRB, 0.27759E+00_JPRB, 0.27370E+00_JPRB, 0.26697E+00_JPRB, 0.25547E+00_JPRB, 0.24007E+00_JPRB, 0.23076E+00_JPRB ,&
&  0.22928E+00_JPRB, 0.24025E+00_JPRB, 0.26191E+00_JPRB, 0.27075E+00_JPRB, 0.27805E+00_JPRB, 0.28199E+00_JPRB, 0.28644E+00_JPRB ,&
&  0.28209E+00_JPRB, 0.25772E+00_JPRB, 0.24427E+00_JPRB, 0.23724E+00_JPRB, 0.23164E+00_JPRB, 0.22730E+00_JPRB, 0.22705E+00_JPRB ,&
&  0.23014E+00_JPRB, 0.23657E+00_JPRB, 0.24291E+00_JPRB, 0.24602E+00_JPRB, 0.24535E+00_JPRB, 0.24206E+00_JPRB, 0.23872E+00_JPRB ,&
&  0.23590E+00_JPRB, 0.24037E+00_JPRB, 0.25501E+00_JPRB, 0.27749E+00_JPRB, 0.28556E+00_JPRB, 0.29178E+00_JPRB, 0.29762E+00_JPRB ,&
&  0.30387E+00_JPRB, 0.31331E+00_JPRB, 0.32481E+00_JPRB, 0.33398E+00_JPRB, 0.34210E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.32001E+00_JPRB, 0.32425E+00_JPRB, 0.32695E+00_JPRB, 0.33038E+00_JPRB, 0.33793E+00_JPRB, 0.34855E+00_JPRB, 0.36065E+00_JPRB ,&
&  0.37180E+00_JPRB, 0.38082E+00_JPRB, 0.38524E+00_JPRB, 0.39038E+00_JPRB, 0.39531E+00_JPRB, 0.39767E+00_JPRB, 0.40286E+00_JPRB ,&
&  0.40360E+00_JPRB, 0.40021E+00_JPRB, 0.39412E+00_JPRB, 0.38605E+00_JPRB, 0.38177E+00_JPRB, 0.38303E+00_JPRB, 0.38593E+00_JPRB ,&
&  0.38784E+00_JPRB, 0.38934E+00_JPRB, 0.38942E+00_JPRB, 0.38602E+00_JPRB, 0.38276E+00_JPRB, 0.38084E+00_JPRB, 0.37686E+00_JPRB ,&
&  0.36796E+00_JPRB, 0.31857E+00_JPRB, 0.28090E+00_JPRB, 0.27645E+00_JPRB, 0.29319E+00_JPRB, 0.29548E+00_JPRB, 0.29022E+00_JPRB ,&
&  0.28443E+00_JPRB, 0.27976E+00_JPRB, 0.27710E+00_JPRB, 0.27499E+00_JPRB, 0.27230E+00_JPRB, 0.26991E+00_JPRB, 0.27066E+00_JPRB ,&
&  0.27191E+00_JPRB, 0.26965E+00_JPRB, 0.26655E+00_JPRB, 0.26028E+00_JPRB, 0.25574E+00_JPRB, 0.25369E+00_JPRB, 0.25198E+00_JPRB ,&
&  0.24775E+00_JPRB, 0.24363E+00_JPRB, 0.24344E+00_JPRB, 0.24451E+00_JPRB, 0.24623E+00_JPRB, 0.24746E+00_JPRB, 0.24496E+00_JPRB ,&
&  0.23887E+00_JPRB, 0.23252E+00_JPRB, 0.22436E+00_JPRB, 0.21465E+00_JPRB, 0.20533E+00_JPRB, 0.19873E+00_JPRB, 0.19362E+00_JPRB ,&
&  0.19268E+00_JPRB, 0.19804E+00_JPRB, 0.21075E+00_JPRB, 0.22633E+00_JPRB, 0.24605E+00_JPRB, 0.23516E+00_JPRB, 0.22397E+00_JPRB ,&
&  0.21343E+00_JPRB, 0.20503E+00_JPRB, 0.19880E+00_JPRB, 0.19592E+00_JPRB, 0.19479E+00_JPRB, 0.19481E+00_JPRB, 0.19592E+00_JPRB ,&
&  0.19792E+00_JPRB, 0.20055E+00_JPRB, 0.20291E+00_JPRB, 0.20402E+00_JPRB, 0.20465E+00_JPRB, 0.20524E+00_JPRB, 0.20519E+00_JPRB ,&
&  0.20457E+00_JPRB, 0.20686E+00_JPRB, 0.21353E+00_JPRB, 0.22197E+00_JPRB, 0.23022E+00_JPRB, 0.23794E+00_JPRB, 0.25089E+00_JPRB ,&
&  0.26677E+00_JPRB, 0.28686E+00_JPRB, 0.29905E+00_JPRB, 0.30828E+00_JPRB, 0.31635E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.31487E+00_JPRB, 0.32373E+00_JPRB, 0.33577E+00_JPRB, 0.34612E+00_JPRB, 0.35506E+00_JPRB, 0.36359E+00_JPRB, 0.37100E+00_JPRB ,&
&  0.37493E+00_JPRB, 0.37720E+00_JPRB, 0.37804E+00_JPRB, 0.38151E+00_JPRB, 0.37851E+00_JPRB, 0.37180E+00_JPRB, 0.36630E+00_JPRB ,&
&  0.36176E+00_JPRB, 0.35819E+00_JPRB, 0.35293E+00_JPRB, 0.35015E+00_JPRB, 0.34988E+00_JPRB, 0.34995E+00_JPRB, 0.34431E+00_JPRB ,&
&  0.33391E+00_JPRB, 0.33308E+00_JPRB, 0.33782E+00_JPRB, 0.33255E+00_JPRB, 0.32694E+00_JPRB, 0.32159E+00_JPRB, 0.31576E+00_JPRB ,&
&  0.30008E+00_JPRB, 0.28023E+00_JPRB, 0.25319E+00_JPRB, 0.24874E+00_JPRB, 0.26458E+00_JPRB, 0.25846E+00_JPRB, 0.26140E+00_JPRB ,&
&  0.26183E+00_JPRB, 0.26216E+00_JPRB, 0.26068E+00_JPRB, 0.25916E+00_JPRB, 0.25559E+00_JPRB, 0.25426E+00_JPRB, 0.25278E+00_JPRB ,&
&  0.24737E+00_JPRB, 0.24248E+00_JPRB, 0.23474E+00_JPRB, 0.21729E+00_JPRB, 0.21201E+00_JPRB, 0.21005E+00_JPRB, 0.20507E+00_JPRB ,&
&  0.19740E+00_JPRB, 0.19154E+00_JPRB, 0.18905E+00_JPRB, 0.18561E+00_JPRB, 0.18773E+00_JPRB, 0.18950E+00_JPRB, 0.18952E+00_JPRB ,&
&  0.18950E+00_JPRB, 0.18915E+00_JPRB, 0.18687E+00_JPRB, 0.18307E+00_JPRB, 0.18013E+00_JPRB, 0.17910E+00_JPRB, 0.17982E+00_JPRB ,&
&  0.18213E+00_JPRB, 0.18669E+00_JPRB, 0.19740E+00_JPRB, 0.20582E+00_JPRB, 0.20497E+00_JPRB, 0.20042E+00_JPRB, 0.19158E+00_JPRB ,&
&  0.18410E+00_JPRB, 0.18125E+00_JPRB, 0.17881E+00_JPRB, 0.17753E+00_JPRB, 0.17786E+00_JPRB, 0.17928E+00_JPRB, 0.18010E+00_JPRB ,&
&  0.18114E+00_JPRB, 0.18213E+00_JPRB, 0.18295E+00_JPRB, 0.18346E+00_JPRB, 0.18431E+00_JPRB, 0.18530E+00_JPRB, 0.18711E+00_JPRB ,&
&  0.18977E+00_JPRB, 0.19414E+00_JPRB, 0.19986E+00_JPRB, 0.20600E+00_JPRB, 0.21045E+00_JPRB, 0.21262E+00_JPRB, 0.21634E+00_JPRB ,&
&  0.22213E+00_JPRB, 0.23682E+00_JPRB, 0.26523E+00_JPRB, 0.31055E+00_JPRB, 0.31460E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.31589E+00_JPRB, 0.32792E+00_JPRB, 0.34324E+00_JPRB, 0.35272E+00_JPRB, 0.35799E+00_JPRB, 0.36064E+00_JPRB, 0.35954E+00_JPRB ,&
&  0.35616E+00_JPRB, 0.35567E+00_JPRB, 0.36679E+00_JPRB, 0.34898E+00_JPRB, 0.33128E+00_JPRB, 0.29796E+00_JPRB, 0.28148E+00_JPRB ,&
&  0.26917E+00_JPRB, 0.26705E+00_JPRB, 0.26681E+00_JPRB, 0.27151E+00_JPRB, 0.27541E+00_JPRB, 0.27850E+00_JPRB, 0.27485E+00_JPRB ,&
&  0.26960E+00_JPRB, 0.26041E+00_JPRB, 0.25271E+00_JPRB, 0.24547E+00_JPRB, 0.24553E+00_JPRB, 0.24443E+00_JPRB, 0.24184E+00_JPRB ,&
&  0.23884E+00_JPRB, 0.23691E+00_JPRB, 0.23676E+00_JPRB, 0.23690E+00_JPRB, 0.24849E+00_JPRB, 0.24782E+00_JPRB, 0.24772E+00_JPRB ,&
&  0.24731E+00_JPRB, 0.24725E+00_JPRB, 0.24626E+00_JPRB, 0.24452E+00_JPRB, 0.23979E+00_JPRB, 0.22723E+00_JPRB, 0.22669E+00_JPRB ,&
&  0.21727E+00_JPRB, 0.20667E+00_JPRB, 0.19647E+00_JPRB, 0.18937E+00_JPRB, 0.18505E+00_JPRB, 0.18311E+00_JPRB, 0.17850E+00_JPRB ,&
&  0.17354E+00_JPRB, 0.16790E+00_JPRB, 0.16524E+00_JPRB, 0.16499E+00_JPRB, 0.16554E+00_JPRB, 0.16612E+00_JPRB, 0.16660E+00_JPRB ,&
&  0.16782E+00_JPRB, 0.16841E+00_JPRB, 0.16877E+00_JPRB, 0.16945E+00_JPRB, 0.17013E+00_JPRB, 0.17117E+00_JPRB, 0.17294E+00_JPRB ,&
&  0.17606E+00_JPRB, 0.17944E+00_JPRB, 0.18255E+00_JPRB, 0.18507E+00_JPRB, 0.18408E+00_JPRB, 0.18086E+00_JPRB, 0.17655E+00_JPRB ,&
&  0.17343E+00_JPRB, 0.17240E+00_JPRB, 0.17144E+00_JPRB, 0.17138E+00_JPRB, 0.17157E+00_JPRB, 0.17201E+00_JPRB, 0.17263E+00_JPRB ,&
&  0.17345E+00_JPRB, 0.17414E+00_JPRB, 0.17498E+00_JPRB, 0.17608E+00_JPRB, 0.17763E+00_JPRB, 0.17904E+00_JPRB, 0.18102E+00_JPRB ,&
&  0.18397E+00_JPRB, 0.18721E+00_JPRB, 0.19182E+00_JPRB, 0.19598E+00_JPRB, 0.20152E+00_JPRB, 0.20965E+00_JPRB, 0.22296E+00_JPRB ,&
&  0.23735E+00_JPRB, 0.27181E+00_JPRB, 0.28772E+00_JPRB, 0.29826E+00_JPRB, 0.30561E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.31648E+00_JPRB, 0.32599E+00_JPRB, 0.34008E+00_JPRB, 0.34916E+00_JPRB, 0.35641E+00_JPRB, 0.35229E+00_JPRB, 0.34838E+00_JPRB ,&
&  0.34432E+00_JPRB, 0.33023E+00_JPRB, 0.31309E+00_JPRB, 0.28392E+00_JPRB, 0.25935E+00_JPRB, 0.24054E+00_JPRB, 0.23016E+00_JPRB ,&
&  0.22399E+00_JPRB, 0.22195E+00_JPRB, 0.22004E+00_JPRB, 0.21956E+00_JPRB, 0.21840E+00_JPRB, 0.22000E+00_JPRB, 0.22135E+00_JPRB ,&
&  0.22093E+00_JPRB, 0.21982E+00_JPRB, 0.21919E+00_JPRB, 0.22228E+00_JPRB, 0.22827E+00_JPRB, 0.23297E+00_JPRB, 0.23748E+00_JPRB ,&
&  0.24078E+00_JPRB, 0.23821E+00_JPRB, 0.23933E+00_JPRB, 0.23731E+00_JPRB, 0.23512E+00_JPRB, 0.23523E+00_JPRB, 0.23611E+00_JPRB ,&
&  0.23654E+00_JPRB, 0.23708E+00_JPRB, 0.23637E+00_JPRB, 0.23340E+00_JPRB, 0.22178E+00_JPRB, 0.21573E+00_JPRB, 0.20803E+00_JPRB ,&
&  0.20005E+00_JPRB, 0.19072E+00_JPRB, 0.18322E+00_JPRB, 0.17958E+00_JPRB, 0.17531E+00_JPRB, 0.17362E+00_JPRB, 0.17048E+00_JPRB ,&
&  0.16690E+00_JPRB, 0.16350E+00_JPRB, 0.16122E+00_JPRB, 0.16008E+00_JPRB, 0.15968E+00_JPRB, 0.15868E+00_JPRB, 0.15833E+00_JPRB ,&
&  0.15860E+00_JPRB, 0.15892E+00_JPRB, 0.15973E+00_JPRB, 0.16054E+00_JPRB, 0.16145E+00_JPRB, 0.16229E+00_JPRB, 0.16332E+00_JPRB ,&
&  0.16493E+00_JPRB, 0.16688E+00_JPRB, 0.16776E+00_JPRB, 0.16849E+00_JPRB, 0.16822E+00_JPRB, 0.16774E+00_JPRB, 0.16700E+00_JPRB ,&
&  0.16642E+00_JPRB, 0.16624E+00_JPRB, 0.16624E+00_JPRB, 0.16618E+00_JPRB, 0.16666E+00_JPRB, 0.16754E+00_JPRB, 0.16849E+00_JPRB ,&
&  0.17002E+00_JPRB, 0.17142E+00_JPRB, 0.17301E+00_JPRB, 0.17486E+00_JPRB, 0.17699E+00_JPRB, 0.17950E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18490E+00_JPRB, 0.18816E+00_JPRB, 0.19093E+00_JPRB, 0.19316E+00_JPRB, 0.19825E+00_JPRB, 0.20533E+00_JPRB, 0.21940E+00_JPRB ,&
&  0.23574E+00_JPRB, 0.27589E+00_JPRB, 0.31824E+00_JPRB, 0.30345E+00_JPRB, 0.30842E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.28966E+00_JPRB, 0.30052E+00_JPRB, 0.30930E+00_JPRB, 0.31479E+00_JPRB, 0.32043E+00_JPRB, 0.32263E+00_JPRB, 0.33096E+00_JPRB ,&
&  0.32550E+00_JPRB, 0.30740E+00_JPRB, 0.29624E+00_JPRB, 0.27986E+00_JPRB, 0.25959E+00_JPRB, 0.24306E+00_JPRB, 0.23478E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22600E+00_JPRB, 0.22131E+00_JPRB, 0.21615E+00_JPRB, 0.21094E+00_JPRB, 0.20709E+00_JPRB, 0.20456E+00_JPRB ,&
&  0.20227E+00_JPRB, 0.20279E+00_JPRB, 0.20527E+00_JPRB, 0.20951E+00_JPRB, 0.21554E+00_JPRB, 0.21980E+00_JPRB, 0.22311E+00_JPRB ,&
&  0.22541E+00_JPRB, 0.21668E+00_JPRB, 0.21814E+00_JPRB, 0.21993E+00_JPRB, 0.21859E+00_JPRB, 0.22142E+00_JPRB, 0.22585E+00_JPRB ,&
&  0.22360E+00_JPRB, 0.21717E+00_JPRB, 0.21989E+00_JPRB, 0.21527E+00_JPRB, 0.20694E+00_JPRB, 0.20494E+00_JPRB, 0.19916E+00_JPRB ,&
&  0.19604E+00_JPRB, 0.18885E+00_JPRB, 0.18375E+00_JPRB, 0.18039E+00_JPRB, 0.17508E+00_JPRB, 0.17287E+00_JPRB, 0.16954E+00_JPRB ,&
&  0.16609E+00_JPRB, 0.16365E+00_JPRB, 0.16077E+00_JPRB, 0.15933E+00_JPRB, 0.15811E+00_JPRB, 0.15691E+00_JPRB, 0.15598E+00_JPRB ,&
&  0.15546E+00_JPRB, 0.15498E+00_JPRB, 0.15501E+00_JPRB, 0.15505E+00_JPRB, 0.15506E+00_JPRB, 0.15539E+00_JPRB, 0.15559E+00_JPRB ,&
&  0.15621E+00_JPRB, 0.15692E+00_JPRB, 0.15748E+00_JPRB, 0.15838E+00_JPRB, 0.15879E+00_JPRB, 0.15932E+00_JPRB, 0.15972E+00_JPRB ,&
&  0.16017E+00_JPRB, 0.16084E+00_JPRB, 0.16122E+00_JPRB, 0.16191E+00_JPRB, 0.16270E+00_JPRB, 0.16419E+00_JPRB, 0.16567E+00_JPRB ,&
&  0.16746E+00_JPRB, 0.16953E+00_JPRB, 0.17174E+00_JPRB, 0.17408E+00_JPRB, 0.17698E+00_JPRB, 0.17966E+00_JPRB, 0.18317E+00_JPRB ,&
&  0.18696E+00_JPRB, 0.19057E+00_JPRB, 0.19476E+00_JPRB, 0.20008E+00_JPRB, 0.20708E+00_JPRB, 0.21666E+00_JPRB, 0.22562E+00_JPRB ,&
&  0.23594E+00_JPRB, 0.26227E+00_JPRB, 0.26869E+00_JPRB, 0.27260E+00_JPRB, 0.28111E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.28208E+00_JPRB, 0.28692E+00_JPRB, 0.29064E+00_JPRB, 0.29302E+00_JPRB, 0.29526E+00_JPRB, 0.29688E+00_JPRB, 0.29746E+00_JPRB ,&
&  0.29103E+00_JPRB, 0.28604E+00_JPRB, 0.27911E+00_JPRB, 0.27079E+00_JPRB, 0.26321E+00_JPRB, 0.25633E+00_JPRB, 0.24839E+00_JPRB ,&
&  0.24189E+00_JPRB, 0.23575E+00_JPRB, 0.22928E+00_JPRB, 0.22251E+00_JPRB, 0.21542E+00_JPRB, 0.20827E+00_JPRB, 0.20471E+00_JPRB ,&
&  0.20360E+00_JPRB, 0.20459E+00_JPRB, 0.20647E+00_JPRB, 0.20801E+00_JPRB, 0.20893E+00_JPRB, 0.20954E+00_JPRB, 0.20840E+00_JPRB ,&
&  0.20473E+00_JPRB, 0.20535E+00_JPRB, 0.20620E+00_JPRB, 0.20674E+00_JPRB, 0.20823E+00_JPRB, 0.20950E+00_JPRB, 0.20929E+00_JPRB ,&
&  0.20749E+00_JPRB, 0.20507E+00_JPRB, 0.20178E+00_JPRB, 0.19872E+00_JPRB, 0.19751E+00_JPRB, 0.19531E+00_JPRB, 0.19282E+00_JPRB ,&
&  0.19076E+00_JPRB, 0.18687E+00_JPRB, 0.18296E+00_JPRB, 0.18020E+00_JPRB, 0.17663E+00_JPRB, 0.17420E+00_JPRB, 0.17104E+00_JPRB ,&
&  0.16848E+00_JPRB, 0.16615E+00_JPRB, 0.16359E+00_JPRB, 0.16160E+00_JPRB, 0.15973E+00_JPRB, 0.15835E+00_JPRB, 0.15697E+00_JPRB ,&
&  0.15617E+00_JPRB, 0.15541E+00_JPRB, 0.15493E+00_JPRB, 0.15440E+00_JPRB, 0.15408E+00_JPRB, 0.15376E+00_JPRB, 0.15352E+00_JPRB ,&
&  0.15338E+00_JPRB, 0.15368E+00_JPRB, 0.15390E+00_JPRB, 0.15430E+00_JPRB, 0.15477E+00_JPRB, 0.15506E+00_JPRB, 0.15560E+00_JPRB ,&
&  0.15661E+00_JPRB, 0.15771E+00_JPRB, 0.15845E+00_JPRB, 0.15941E+00_JPRB, 0.16048E+00_JPRB, 0.16179E+00_JPRB, 0.16355E+00_JPRB ,&
&  0.16541E+00_JPRB, 0.16749E+00_JPRB, 0.16982E+00_JPRB, 0.17214E+00_JPRB, 0.17510E+00_JPRB, 0.17814E+00_JPRB, 0.18133E+00_JPRB ,&
&  0.18510E+00_JPRB, 0.18914E+00_JPRB, 0.19327E+00_JPRB, 0.19782E+00_JPRB, 0.20435E+00_JPRB, 0.21037E+00_JPRB, 0.21786E+00_JPRB ,&
&  0.22520E+00_JPRB, 0.23118E+00_JPRB, 0.26314E+00_JPRB, 0.26798E+00_JPRB, 0.27470E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.20858E+00_JPRB, 0.21541E+00_JPRB, 0.22229E+00_JPRB, 0.23206E+00_JPRB, 0.23916E+00_JPRB, 0.27462E+00_JPRB, 0.26955E+00_JPRB ,&
&  0.26614E+00_JPRB, 0.25236E+00_JPRB, 0.25078E+00_JPRB, 0.24713E+00_JPRB, 0.24259E+00_JPRB, 0.23766E+00_JPRB, 0.23269E+00_JPRB ,&
&  0.22804E+00_JPRB, 0.22258E+00_JPRB, 0.21788E+00_JPRB, 0.21418E+00_JPRB, 0.21087E+00_JPRB, 0.20762E+00_JPRB, 0.20479E+00_JPRB ,&
&  0.20356E+00_JPRB, 0.20339E+00_JPRB, 0.20260E+00_JPRB, 0.19861E+00_JPRB, 0.19522E+00_JPRB, 0.19312E+00_JPRB, 0.19246E+00_JPRB ,&
&  0.19238E+00_JPRB, 0.19397E+00_JPRB, 0.19493E+00_JPRB, 0.19568E+00_JPRB, 0.19681E+00_JPRB, 0.19595E+00_JPRB, 0.19511E+00_JPRB ,&
&  0.19391E+00_JPRB, 0.19258E+00_JPRB, 0.19118E+00_JPRB, 0.18997E+00_JPRB, 0.18876E+00_JPRB, 0.18698E+00_JPRB, 0.18543E+00_JPRB ,&
&  0.18388E+00_JPRB, 0.18192E+00_JPRB, 0.17935E+00_JPRB, 0.17764E+00_JPRB, 0.17565E+00_JPRB, 0.17396E+00_JPRB, 0.17227E+00_JPRB ,&
&  0.17102E+00_JPRB, 0.16987E+00_JPRB, 0.16852E+00_JPRB, 0.16688E+00_JPRB, 0.16524E+00_JPRB, 0.16380E+00_JPRB, 0.16242E+00_JPRB ,&
&  0.16166E+00_JPRB, 0.16087E+00_JPRB, 0.16010E+00_JPRB, 0.15927E+00_JPRB, 0.15885E+00_JPRB, 0.15842E+00_JPRB, 0.15804E+00_JPRB ,&
&  0.15771E+00_JPRB, 0.15754E+00_JPRB, 0.15757E+00_JPRB, 0.15766E+00_JPRB, 0.15776E+00_JPRB, 0.15812E+00_JPRB, 0.15856E+00_JPRB ,&
&  0.15915E+00_JPRB, 0.15988E+00_JPRB, 0.16087E+00_JPRB, 0.16191E+00_JPRB, 0.16284E+00_JPRB, 0.16359E+00_JPRB, 0.16459E+00_JPRB ,&
&  0.16617E+00_JPRB, 0.16800E+00_JPRB, 0.17000E+00_JPRB, 0.17156E+00_JPRB, 0.17372E+00_JPRB, 0.17568E+00_JPRB, 0.17762E+00_JPRB ,&
&  0.17955E+00_JPRB, 0.18182E+00_JPRB, 0.18432E+00_JPRB, 0.18698E+00_JPRB, 0.19002E+00_JPRB, 0.19274E+00_JPRB, 0.19526E+00_JPRB ,&
&  0.19783E+00_JPRB, 0.20141E+00_JPRB, 0.20345E+00_JPRB, 0.20468E+00_JPRB, 0.20628E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.18717E+00_JPRB, 0.18736E+00_JPRB, 0.18783E+00_JPRB, 0.18838E+00_JPRB, 0.18895E+00_JPRB, 0.18971E+00_JPRB, 0.18997E+00_JPRB ,&
&  0.19004E+00_JPRB, 0.18998E+00_JPRB, 0.19032E+00_JPRB, 0.19072E+00_JPRB, 0.19096E+00_JPRB, 0.19097E+00_JPRB, 0.19058E+00_JPRB ,&
&  0.18992E+00_JPRB, 0.18913E+00_JPRB, 0.18797E+00_JPRB, 0.18745E+00_JPRB, 0.18706E+00_JPRB, 0.18667E+00_JPRB, 0.18590E+00_JPRB ,&
&  0.18523E+00_JPRB, 0.18404E+00_JPRB, 0.18238E+00_JPRB, 0.18105E+00_JPRB, 0.18078E+00_JPRB, 0.18098E+00_JPRB, 0.18152E+00_JPRB ,&
&  0.18194E+00_JPRB, 0.18220E+00_JPRB, 0.18227E+00_JPRB, 0.18241E+00_JPRB, 0.18262E+00_JPRB, 0.18246E+00_JPRB, 0.18219E+00_JPRB ,&
&  0.18185E+00_JPRB, 0.18131E+00_JPRB, 0.18080E+00_JPRB, 0.18033E+00_JPRB, 0.17991E+00_JPRB, 0.17942E+00_JPRB, 0.17883E+00_JPRB ,&
&  0.17816E+00_JPRB, 0.17734E+00_JPRB, 0.17637E+00_JPRB, 0.17558E+00_JPRB, 0.17477E+00_JPRB, 0.17412E+00_JPRB, 0.17333E+00_JPRB ,&
&  0.17268E+00_JPRB, 0.17207E+00_JPRB, 0.17146E+00_JPRB, 0.17089E+00_JPRB, 0.17035E+00_JPRB, 0.16983E+00_JPRB, 0.16931E+00_JPRB ,&
&  0.16882E+00_JPRB, 0.16841E+00_JPRB, 0.16796E+00_JPRB, 0.16754E+00_JPRB, 0.16725E+00_JPRB, 0.16701E+00_JPRB, 0.16680E+00_JPRB ,&
&  0.16662E+00_JPRB, 0.16656E+00_JPRB, 0.16654E+00_JPRB, 0.16652E+00_JPRB, 0.16651E+00_JPRB, 0.16662E+00_JPRB, 0.16673E+00_JPRB ,&
&  0.16689E+00_JPRB, 0.16701E+00_JPRB, 0.16714E+00_JPRB, 0.16733E+00_JPRB, 0.16754E+00_JPRB, 0.16778E+00_JPRB, 0.16811E+00_JPRB ,&
&  0.16851E+00_JPRB, 0.16899E+00_JPRB, 0.16949E+00_JPRB, 0.16982E+00_JPRB, 0.17049E+00_JPRB, 0.17124E+00_JPRB, 0.17210E+00_JPRB ,&
&  0.17305E+00_JPRB, 0.17417E+00_JPRB, 0.17538E+00_JPRB, 0.17677E+00_JPRB, 0.17818E+00_JPRB, 0.17910E+00_JPRB, 0.18002E+00_JPRB ,&
&  0.18099E+00_JPRB, 0.18211E+00_JPRB, 0.18318E+00_JPRB, 0.18428E+00_JPRB, 0.18562E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB, 0.17302E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB, 0.32723E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.33250E+00_JPRB, 0.33227E+00_JPRB, 0.33204E+00_JPRB, 0.33178E+00_JPRB, 0.33142E+00_JPRB, 0.33107E+00_JPRB, 0.33078E+00_JPRB ,&
&  0.33048E+00_JPRB, 0.33028E+00_JPRB, 0.32991E+00_JPRB, 0.32954E+00_JPRB, 0.32916E+00_JPRB, 0.32883E+00_JPRB, 0.32848E+00_JPRB ,&
&  0.32813E+00_JPRB, 0.32778E+00_JPRB, 0.32756E+00_JPRB, 0.32728E+00_JPRB, 0.32696E+00_JPRB, 0.32667E+00_JPRB, 0.32643E+00_JPRB ,&
&  0.32621E+00_JPRB, 0.32594E+00_JPRB, 0.32564E+00_JPRB, 0.32525E+00_JPRB, 0.32502E+00_JPRB, 0.32466E+00_JPRB, 0.32437E+00_JPRB ,&
&  0.32403E+00_JPRB, 0.32380E+00_JPRB, 0.32355E+00_JPRB, 0.32330E+00_JPRB, 0.32299E+00_JPRB, 0.32295E+00_JPRB, 0.32276E+00_JPRB ,&
&  0.32267E+00_JPRB, 0.32257E+00_JPRB, 0.32257E+00_JPRB, 0.32253E+00_JPRB, 0.32254E+00_JPRB, 0.32255E+00_JPRB, 0.32276E+00_JPRB ,&
&  0.32292E+00_JPRB, 0.32313E+00_JPRB, 0.32346E+00_JPRB, 0.32375E+00_JPRB, 0.32400E+00_JPRB, 0.32426E+00_JPRB, 0.32454E+00_JPRB ,&
&  0.32496E+00_JPRB, 0.32538E+00_JPRB, 0.32578E+00_JPRB, 0.32618E+00_JPRB, 0.32658E+00_JPRB, 0.32688E+00_JPRB, 0.32712E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32741E+00_JPRB, 0.32759E+00_JPRB, 0.32778E+00_JPRB, 0.32779E+00_JPRB, 0.32789E+00_JPRB, 0.32790E+00_JPRB ,&
&  0.32791E+00_JPRB, 0.32773E+00_JPRB, 0.32772E+00_JPRB, 0.32760E+00_JPRB, 0.32760E+00_JPRB, 0.32754E+00_JPRB, 0.32767E+00_JPRB ,&
&  0.32774E+00_JPRB, 0.32797E+00_JPRB, 0.32822E+00_JPRB, 0.32860E+00_JPRB, 0.32888E+00_JPRB, 0.32922E+00_JPRB, 0.32951E+00_JPRB ,&
&  0.32983E+00_JPRB, 0.33016E+00_JPRB, 0.33051E+00_JPRB, 0.33072E+00_JPRB, 0.33091E+00_JPRB, 0.33113E+00_JPRB, 0.33141E+00_JPRB ,&
&  0.33178E+00_JPRB, 0.33204E+00_JPRB, 0.33231E+00_JPRB, 0.33254E+00_JPRB, 0.33261E+00_JPRB, 0.33271E+00_JPRB, 0.33273E+00_JPRB ,&
&  0.33273E+00_JPRB, 0.33273E+00_JPRB, 0.33269E+00_JPRB, 0.33268E+00_JPRB, 0.33263E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.33731E+00_JPRB, 0.33729E+00_JPRB, 0.33714E+00_JPRB, 0.33678E+00_JPRB, 0.33630E+00_JPRB, 0.33573E+00_JPRB, 0.33515E+00_JPRB ,&
&  0.33446E+00_JPRB, 0.33379E+00_JPRB, 0.33297E+00_JPRB, 0.33218E+00_JPRB, 0.33135E+00_JPRB, 0.33062E+00_JPRB, 0.32986E+00_JPRB ,&
&  0.32916E+00_JPRB, 0.32846E+00_JPRB, 0.32802E+00_JPRB, 0.32755E+00_JPRB, 0.32703E+00_JPRB, 0.32633E+00_JPRB, 0.32580E+00_JPRB ,&
&  0.32522E+00_JPRB, 0.32449E+00_JPRB, 0.32366E+00_JPRB, 0.32209E+00_JPRB, 0.32080E+00_JPRB, 0.31961E+00_JPRB, 0.31859E+00_JPRB ,&
&  0.31766E+00_JPRB, 0.31677E+00_JPRB, 0.31609E+00_JPRB, 0.31568E+00_JPRB, 0.31634E+00_JPRB, 0.31648E+00_JPRB, 0.31634E+00_JPRB ,&
&  0.31609E+00_JPRB, 0.31497E+00_JPRB, 0.31459E+00_JPRB, 0.31444E+00_JPRB, 0.31440E+00_JPRB, 0.31503E+00_JPRB, 0.31554E+00_JPRB ,&
&  0.31609E+00_JPRB, 0.31653E+00_JPRB, 0.31751E+00_JPRB, 0.31842E+00_JPRB, 0.31928E+00_JPRB, 0.32021E+00_JPRB, 0.32069E+00_JPRB ,&
&  0.32124E+00_JPRB, 0.32210E+00_JPRB, 0.32294E+00_JPRB, 0.32381E+00_JPRB, 0.32430E+00_JPRB, 0.32475E+00_JPRB, 0.32509E+00_JPRB ,&
&  0.32536E+00_JPRB, 0.32577E+00_JPRB, 0.32626E+00_JPRB, 0.32673E+00_JPRB, 0.32704E+00_JPRB, 0.32729E+00_JPRB, 0.32741E+00_JPRB ,&
&  0.32741E+00_JPRB, 0.32713E+00_JPRB, 0.32693E+00_JPRB, 0.32673E+00_JPRB, 0.32642E+00_JPRB, 0.32607E+00_JPRB, 0.32592E+00_JPRB ,&
&  0.32579E+00_JPRB, 0.32557E+00_JPRB, 0.32523E+00_JPRB, 0.32534E+00_JPRB, 0.32551E+00_JPRB, 0.32556E+00_JPRB, 0.32549E+00_JPRB ,&
&  0.32631E+00_JPRB, 0.32748E+00_JPRB, 0.32885E+00_JPRB, 0.33063E+00_JPRB, 0.33199E+00_JPRB, 0.33288E+00_JPRB, 0.33351E+00_JPRB ,&
&  0.33368E+00_JPRB, 0.33390E+00_JPRB, 0.33417E+00_JPRB, 0.33466E+00_JPRB, 0.33504E+00_JPRB, 0.33568E+00_JPRB, 0.33655E+00_JPRB ,&
&  0.33716E+00_JPRB, 0.33741E+00_JPRB, 0.33748E+00_JPRB, 0.33744E+00_JPRB, 0.33736E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.33948E+00_JPRB, 0.33898E+00_JPRB, 0.33852E+00_JPRB, 0.33766E+00_JPRB, 0.33713E+00_JPRB, 0.33635E+00_JPRB, 0.33563E+00_JPRB ,&
&  0.33477E+00_JPRB, 0.33378E+00_JPRB, 0.33309E+00_JPRB, 0.33256E+00_JPRB, 0.33200E+00_JPRB, 0.33150E+00_JPRB, 0.33092E+00_JPRB ,&
&  0.33030E+00_JPRB, 0.32956E+00_JPRB, 0.32935E+00_JPRB, 0.32889E+00_JPRB, 0.32835E+00_JPRB, 0.32770E+00_JPRB, 0.32580E+00_JPRB ,&
&  0.32409E+00_JPRB, 0.32210E+00_JPRB, 0.32018E+00_JPRB, 0.31864E+00_JPRB, 0.31760E+00_JPRB, 0.30817E+00_JPRB, 0.30670E+00_JPRB ,&
&  0.31050E+00_JPRB, 0.31256E+00_JPRB, 0.31495E+00_JPRB, 0.31742E+00_JPRB, 0.31261E+00_JPRB, 0.30657E+00_JPRB, 0.30309E+00_JPRB ,&
&  0.29439E+00_JPRB, 0.29779E+00_JPRB, 0.29945E+00_JPRB, 0.30101E+00_JPRB, 0.30345E+00_JPRB, 0.30341E+00_JPRB, 0.30495E+00_JPRB ,&
&  0.31078E+00_JPRB, 0.31070E+00_JPRB, 0.31134E+00_JPRB, 0.31209E+00_JPRB, 0.31280E+00_JPRB, 0.31351E+00_JPRB, 0.31496E+00_JPRB ,&
&  0.31606E+00_JPRB, 0.31734E+00_JPRB, 0.31875E+00_JPRB, 0.31941E+00_JPRB, 0.32016E+00_JPRB, 0.32121E+00_JPRB, 0.32218E+00_JPRB ,&
&  0.32263E+00_JPRB, 0.32324E+00_JPRB, 0.32398E+00_JPRB, 0.32464E+00_JPRB, 0.32557E+00_JPRB, 0.32609E+00_JPRB, 0.32647E+00_JPRB ,&
&  0.32669E+00_JPRB, 0.32651E+00_JPRB, 0.32637E+00_JPRB, 0.32635E+00_JPRB, 0.32615E+00_JPRB, 0.32585E+00_JPRB, 0.32585E+00_JPRB ,&
&  0.32605E+00_JPRB, 0.32616E+00_JPRB, 0.32647E+00_JPRB, 0.32705E+00_JPRB, 0.32860E+00_JPRB, 0.33076E+00_JPRB, 0.33498E+00_JPRB ,&
&  0.33841E+00_JPRB, 0.34138E+00_JPRB, 0.34247E+00_JPRB, 0.34338E+00_JPRB, 0.34405E+00_JPRB, 0.34411E+00_JPRB, 0.34404E+00_JPRB ,&
&  0.34345E+00_JPRB, 0.34245E+00_JPRB, 0.34148E+00_JPRB, 0.34049E+00_JPRB, 0.33992E+00_JPRB, 0.33952E+00_JPRB, 0.33961E+00_JPRB ,&
&  0.34006E+00_JPRB, 0.34078E+00_JPRB, 0.34102E+00_JPRB, 0.34055E+00_JPRB, 0.33991E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.34374E+00_JPRB, 0.34236E+00_JPRB, 0.34186E+00_JPRB, 0.34066E+00_JPRB, 0.33937E+00_JPRB, 0.33775E+00_JPRB, 0.33646E+00_JPRB ,&
&  0.33503E+00_JPRB, 0.33389E+00_JPRB, 0.33302E+00_JPRB, 0.33228E+00_JPRB, 0.33208E+00_JPRB, 0.33180E+00_JPRB, 0.33169E+00_JPRB ,&
&  0.33143E+00_JPRB, 0.33096E+00_JPRB, 0.33080E+00_JPRB, 0.33044E+00_JPRB, 0.33001E+00_JPRB, 0.32944E+00_JPRB, 0.32771E+00_JPRB ,&
&  0.32517E+00_JPRB, 0.32206E+00_JPRB, 0.31405E+00_JPRB, 0.31592E+00_JPRB, 0.31476E+00_JPRB, 0.31366E+00_JPRB, 0.31417E+00_JPRB ,&
&  0.31005E+00_JPRB, 0.30901E+00_JPRB, 0.30442E+00_JPRB, 0.30003E+00_JPRB, 0.29822E+00_JPRB, 0.29356E+00_JPRB, 0.29332E+00_JPRB ,&
&  0.29161E+00_JPRB, 0.29081E+00_JPRB, 0.29130E+00_JPRB, 0.29115E+00_JPRB, 0.29392E+00_JPRB, 0.29544E+00_JPRB, 0.29763E+00_JPRB ,&
&  0.30130E+00_JPRB, 0.30420E+00_JPRB, 0.30613E+00_JPRB, 0.30300E+00_JPRB, 0.30452E+00_JPRB, 0.30597E+00_JPRB, 0.30781E+00_JPRB ,&
&  0.30946E+00_JPRB, 0.31049E+00_JPRB, 0.31145E+00_JPRB, 0.31213E+00_JPRB, 0.31325E+00_JPRB, 0.31495E+00_JPRB, 0.31675E+00_JPRB ,&
&  0.31883E+00_JPRB, 0.32029E+00_JPRB, 0.32134E+00_JPRB, 0.32236E+00_JPRB, 0.32346E+00_JPRB, 0.32437E+00_JPRB, 0.32543E+00_JPRB ,&
&  0.32656E+00_JPRB, 0.32797E+00_JPRB, 0.32945E+00_JPRB, 0.33102E+00_JPRB, 0.33248E+00_JPRB, 0.33442E+00_JPRB, 0.33685E+00_JPRB ,&
&  0.34045E+00_JPRB, 0.34384E+00_JPRB, 0.34924E+00_JPRB, 0.35240E+00_JPRB, 0.35901E+00_JPRB, 0.36575E+00_JPRB, 0.36980E+00_JPRB ,&
&  0.36908E+00_JPRB, 0.36718E+00_JPRB, 0.36497E+00_JPRB, 0.36246E+00_JPRB, 0.36036E+00_JPRB, 0.35798E+00_JPRB, 0.35637E+00_JPRB ,&
&  0.35497E+00_JPRB, 0.35358E+00_JPRB, 0.35226E+00_JPRB, 0.35087E+00_JPRB, 0.34972E+00_JPRB, 0.34831E+00_JPRB, 0.34728E+00_JPRB ,&
&  0.34679E+00_JPRB, 0.34696E+00_JPRB, 0.34690E+00_JPRB, 0.34633E+00_JPRB, 0.34510E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.34835E+00_JPRB, 0.34795E+00_JPRB, 0.34865E+00_JPRB, 0.34805E+00_JPRB, 0.34663E+00_JPRB, 0.34521E+00_JPRB, 0.34238E+00_JPRB ,&
&  0.33966E+00_JPRB, 0.33710E+00_JPRB, 0.33492E+00_JPRB, 0.33345E+00_JPRB, 0.33227E+00_JPRB, 0.33143E+00_JPRB, 0.33127E+00_JPRB ,&
&  0.33086E+00_JPRB, 0.33107E+00_JPRB, 0.33132E+00_JPRB, 0.33159E+00_JPRB, 0.33224E+00_JPRB, 0.33259E+00_JPRB, 0.33253E+00_JPRB ,&
&  0.33089E+00_JPRB, 0.32687E+00_JPRB, 0.32141E+00_JPRB, 0.32307E+00_JPRB, 0.32393E+00_JPRB, 0.32545E+00_JPRB, 0.32702E+00_JPRB ,&
&  0.32657E+00_JPRB, 0.31328E+00_JPRB, 0.31705E+00_JPRB, 0.31059E+00_JPRB, 0.30695E+00_JPRB, 0.30274E+00_JPRB, 0.29332E+00_JPRB ,&
&  0.29001E+00_JPRB, 0.28451E+00_JPRB, 0.28143E+00_JPRB, 0.28150E+00_JPRB, 0.28287E+00_JPRB, 0.28668E+00_JPRB, 0.29145E+00_JPRB ,&
&  0.29247E+00_JPRB, 0.29966E+00_JPRB, 0.30064E+00_JPRB, 0.30053E+00_JPRB, 0.30228E+00_JPRB, 0.30422E+00_JPRB, 0.30369E+00_JPRB ,&
&  0.30449E+00_JPRB, 0.30426E+00_JPRB, 0.30352E+00_JPRB, 0.30482E+00_JPRB, 0.30860E+00_JPRB, 0.31053E+00_JPRB, 0.31247E+00_JPRB ,&
&  0.31486E+00_JPRB, 0.31630E+00_JPRB, 0.31728E+00_JPRB, 0.31864E+00_JPRB, 0.32039E+00_JPRB, 0.32214E+00_JPRB, 0.32467E+00_JPRB ,&
&  0.32744E+00_JPRB, 0.33197E+00_JPRB, 0.33616E+00_JPRB, 0.34229E+00_JPRB, 0.34794E+00_JPRB, 0.35529E+00_JPRB, 0.36181E+00_JPRB ,&
&  0.36857E+00_JPRB, 0.37415E+00_JPRB, 0.38056E+00_JPRB, 0.38665E+00_JPRB, 0.39169E+00_JPRB, 0.39022E+00_JPRB, 0.38470E+00_JPRB ,&
&  0.37948E+00_JPRB, 0.37205E+00_JPRB, 0.36770E+00_JPRB, 0.36436E+00_JPRB, 0.36218E+00_JPRB, 0.36129E+00_JPRB, 0.36036E+00_JPRB ,&
&  0.36020E+00_JPRB, 0.35942E+00_JPRB, 0.35849E+00_JPRB, 0.35692E+00_JPRB, 0.35562E+00_JPRB, 0.35413E+00_JPRB, 0.35313E+00_JPRB ,&
&  0.35148E+00_JPRB, 0.35082E+00_JPRB, 0.34977E+00_JPRB, 0.34912E+00_JPRB, 0.34857E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.35270E+00_JPRB, 0.35447E+00_JPRB, 0.35451E+00_JPRB, 0.35364E+00_JPRB, 0.35059E+00_JPRB, 0.34808E+00_JPRB, 0.34411E+00_JPRB ,&
&  0.34041E+00_JPRB, 0.33680E+00_JPRB, 0.33362E+00_JPRB, 0.33170E+00_JPRB, 0.33041E+00_JPRB, 0.32958E+00_JPRB, 0.32932E+00_JPRB ,&
&  0.32946E+00_JPRB, 0.32998E+00_JPRB, 0.33044E+00_JPRB, 0.33098E+00_JPRB, 0.33213E+00_JPRB, 0.33358E+00_JPRB, 0.33501E+00_JPRB ,&
&  0.33547E+00_JPRB, 0.33379E+00_JPRB, 0.32869E+00_JPRB, 0.32881E+00_JPRB, 0.33255E+00_JPRB, 0.33097E+00_JPRB, 0.33796E+00_JPRB ,&
&  0.34243E+00_JPRB, 0.34477E+00_JPRB, 0.34032E+00_JPRB, 0.33755E+00_JPRB, 0.33175E+00_JPRB, 0.30284E+00_JPRB, 0.30003E+00_JPRB ,&
&  0.29443E+00_JPRB, 0.28680E+00_JPRB, 0.28334E+00_JPRB, 0.28061E+00_JPRB, 0.28016E+00_JPRB, 0.28466E+00_JPRB, 0.29120E+00_JPRB ,&
&  0.29327E+00_JPRB, 0.30465E+00_JPRB, 0.30497E+00_JPRB, 0.30489E+00_JPRB, 0.30625E+00_JPRB, 0.30921E+00_JPRB, 0.30927E+00_JPRB ,&
&  0.31041E+00_JPRB, 0.31085E+00_JPRB, 0.31044E+00_JPRB, 0.31247E+00_JPRB, 0.31453E+00_JPRB, 0.31610E+00_JPRB, 0.31882E+00_JPRB ,&
&  0.32412E+00_JPRB, 0.32784E+00_JPRB, 0.33034E+00_JPRB, 0.33296E+00_JPRB, 0.33706E+00_JPRB, 0.34144E+00_JPRB, 0.34655E+00_JPRB ,&
&  0.35078E+00_JPRB, 0.35489E+00_JPRB, 0.36000E+00_JPRB, 0.36396E+00_JPRB, 0.36807E+00_JPRB, 0.37217E+00_JPRB, 0.37717E+00_JPRB ,&
&  0.37919E+00_JPRB, 0.38293E+00_JPRB, 0.38761E+00_JPRB, 0.39725E+00_JPRB, 0.39831E+00_JPRB, 0.39075E+00_JPRB, 0.38453E+00_JPRB ,&
&  0.38037E+00_JPRB, 0.38021E+00_JPRB, 0.37631E+00_JPRB, 0.37442E+00_JPRB, 0.37209E+00_JPRB, 0.37168E+00_JPRB, 0.37024E+00_JPRB ,&
&  0.36953E+00_JPRB, 0.36774E+00_JPRB, 0.36512E+00_JPRB, 0.36206E+00_JPRB, 0.35959E+00_JPRB, 0.35693E+00_JPRB, 0.35480E+00_JPRB ,&
&  0.35240E+00_JPRB, 0.35142E+00_JPRB, 0.35068E+00_JPRB, 0.35080E+00_JPRB, 0.35064E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.35617E+00_JPRB, 0.35808E+00_JPRB, 0.35577E+00_JPRB, 0.35474E+00_JPRB, 0.35011E+00_JPRB, 0.34613E+00_JPRB, 0.34083E+00_JPRB ,&
&  0.33715E+00_JPRB, 0.33461E+00_JPRB, 0.33196E+00_JPRB, 0.33133E+00_JPRB, 0.33136E+00_JPRB, 0.33169E+00_JPRB, 0.33294E+00_JPRB ,&
&  0.33426E+00_JPRB, 0.33283E+00_JPRB, 0.33378E+00_JPRB, 0.33574E+00_JPRB, 0.33724E+00_JPRB, 0.33864E+00_JPRB, 0.33994E+00_JPRB ,&
&  0.33916E+00_JPRB, 0.33855E+00_JPRB, 0.33814E+00_JPRB, 0.33537E+00_JPRB, 0.33463E+00_JPRB, 0.33990E+00_JPRB, 0.34710E+00_JPRB ,&
&  0.34448E+00_JPRB, 0.34541E+00_JPRB, 0.34954E+00_JPRB, 0.35070E+00_JPRB, 0.34246E+00_JPRB, 0.32898E+00_JPRB, 0.31141E+00_JPRB ,&
&  0.29952E+00_JPRB, 0.28612E+00_JPRB, 0.28024E+00_JPRB, 0.27736E+00_JPRB, 0.27995E+00_JPRB, 0.28547E+00_JPRB, 0.29554E+00_JPRB ,&
&  0.30243E+00_JPRB, 0.30612E+00_JPRB, 0.30963E+00_JPRB, 0.31340E+00_JPRB, 0.31561E+00_JPRB, 0.31898E+00_JPRB, 0.32086E+00_JPRB ,&
&  0.32457E+00_JPRB, 0.32634E+00_JPRB, 0.32711E+00_JPRB, 0.32988E+00_JPRB, 0.33481E+00_JPRB, 0.33971E+00_JPRB, 0.34830E+00_JPRB ,&
&  0.36945E+00_JPRB, 0.37789E+00_JPRB, 0.38445E+00_JPRB, 0.38798E+00_JPRB, 0.39196E+00_JPRB, 0.39347E+00_JPRB, 0.38874E+00_JPRB ,&
&  0.38742E+00_JPRB, 0.39123E+00_JPRB, 0.39037E+00_JPRB, 0.38986E+00_JPRB, 0.39030E+00_JPRB, 0.39058E+00_JPRB, 0.39163E+00_JPRB ,&
&  0.38758E+00_JPRB, 0.39227E+00_JPRB, 0.42314E+00_JPRB, 0.43280E+00_JPRB, 0.42570E+00_JPRB, 0.41367E+00_JPRB, 0.40398E+00_JPRB ,&
&  0.39710E+00_JPRB, 0.39145E+00_JPRB, 0.38680E+00_JPRB, 0.38315E+00_JPRB, 0.38062E+00_JPRB, 0.37910E+00_JPRB, 0.37902E+00_JPRB ,&
&  0.37580E+00_JPRB, 0.37207E+00_JPRB, 0.36802E+00_JPRB, 0.36498E+00_JPRB, 0.36205E+00_JPRB, 0.35893E+00_JPRB, 0.35685E+00_JPRB ,&
&  0.35475E+00_JPRB, 0.35438E+00_JPRB, 0.35431E+00_JPRB, 0.35505E+00_JPRB, 0.35563E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.35617E+00_JPRB, 0.36001E+00_JPRB, 0.36220E+00_JPRB, 0.35824E+00_JPRB, 0.34946E+00_JPRB, 0.34188E+00_JPRB, 0.33647E+00_JPRB ,&
&  0.33526E+00_JPRB, 0.33500E+00_JPRB, 0.33454E+00_JPRB, 0.33451E+00_JPRB, 0.33494E+00_JPRB, 0.33595E+00_JPRB, 0.33752E+00_JPRB ,&
&  0.33966E+00_JPRB, 0.34148E+00_JPRB, 0.34203E+00_JPRB, 0.34162E+00_JPRB, 0.34236E+00_JPRB, 0.34459E+00_JPRB, 0.34716E+00_JPRB ,&
&  0.34834E+00_JPRB, 0.34771E+00_JPRB, 0.34847E+00_JPRB, 0.34802E+00_JPRB, 0.34793E+00_JPRB, 0.34751E+00_JPRB, 0.34790E+00_JPRB ,&
&  0.35214E+00_JPRB, 0.35621E+00_JPRB, 0.35336E+00_JPRB, 0.35523E+00_JPRB, 0.35918E+00_JPRB, 0.34691E+00_JPRB, 0.32597E+00_JPRB ,&
&  0.30410E+00_JPRB, 0.28755E+00_JPRB, 0.28050E+00_JPRB, 0.27901E+00_JPRB, 0.28374E+00_JPRB, 0.29451E+00_JPRB, 0.30786E+00_JPRB ,&
&  0.31223E+00_JPRB, 0.31611E+00_JPRB, 0.32229E+00_JPRB, 0.32703E+00_JPRB, 0.33003E+00_JPRB, 0.33167E+00_JPRB, 0.33328E+00_JPRB ,&
&  0.33610E+00_JPRB, 0.34101E+00_JPRB, 0.34970E+00_JPRB, 0.36091E+00_JPRB, 0.37824E+00_JPRB, 0.39082E+00_JPRB, 0.41282E+00_JPRB ,&
&  0.44721E+00_JPRB, 0.45035E+00_JPRB, 0.45253E+00_JPRB, 0.44972E+00_JPRB, 0.44620E+00_JPRB, 0.43980E+00_JPRB, 0.43465E+00_JPRB ,&
&  0.42995E+00_JPRB, 0.42498E+00_JPRB, 0.41838E+00_JPRB, 0.41203E+00_JPRB, 0.41053E+00_JPRB, 0.41239E+00_JPRB, 0.41914E+00_JPRB ,&
&  0.43118E+00_JPRB, 0.45505E+00_JPRB, 0.52505E+00_JPRB, 0.47841E+00_JPRB, 0.45321E+00_JPRB, 0.44136E+00_JPRB, 0.42826E+00_JPRB ,&
&  0.41853E+00_JPRB, 0.41262E+00_JPRB, 0.40886E+00_JPRB, 0.40352E+00_JPRB, 0.39432E+00_JPRB, 0.38747E+00_JPRB, 0.38257E+00_JPRB ,&
&  0.37776E+00_JPRB, 0.37315E+00_JPRB, 0.37010E+00_JPRB, 0.36778E+00_JPRB, 0.36498E+00_JPRB, 0.36248E+00_JPRB, 0.36082E+00_JPRB ,&
&  0.35804E+00_JPRB, 0.35624E+00_JPRB, 0.35577E+00_JPRB, 0.35509E+00_JPRB, 0.35525E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.35887E+00_JPRB, 0.36315E+00_JPRB, 0.36187E+00_JPRB, 0.35860E+00_JPRB, 0.35005E+00_JPRB, 0.34272E+00_JPRB, 0.33722E+00_JPRB ,&
&  0.33722E+00_JPRB, 0.33522E+00_JPRB, 0.33460E+00_JPRB, 0.33499E+00_JPRB, 0.33611E+00_JPRB, 0.33830E+00_JPRB, 0.33991E+00_JPRB ,&
&  0.34096E+00_JPRB, 0.34275E+00_JPRB, 0.34487E+00_JPRB, 0.34643E+00_JPRB, 0.34795E+00_JPRB, 0.34988E+00_JPRB, 0.35203E+00_JPRB ,&
&  0.35455E+00_JPRB, 0.35578E+00_JPRB, 0.35669E+00_JPRB, 0.35813E+00_JPRB, 0.36206E+00_JPRB, 0.36746E+00_JPRB, 0.36991E+00_JPRB ,&
&  0.36580E+00_JPRB, 0.36273E+00_JPRB, 0.36578E+00_JPRB, 0.36772E+00_JPRB, 0.37090E+00_JPRB, 0.36262E+00_JPRB, 0.33772E+00_JPRB ,&
&  0.31230E+00_JPRB, 0.29535E+00_JPRB, 0.29265E+00_JPRB, 0.29933E+00_JPRB, 0.31181E+00_JPRB, 0.32126E+00_JPRB, 0.32791E+00_JPRB ,&
&  0.33345E+00_JPRB, 0.33577E+00_JPRB, 0.33824E+00_JPRB, 0.34346E+00_JPRB, 0.34732E+00_JPRB, 0.34894E+00_JPRB, 0.35137E+00_JPRB ,&
&  0.35737E+00_JPRB, 0.36624E+00_JPRB, 0.37103E+00_JPRB, 0.36661E+00_JPRB, 0.38210E+00_JPRB, 0.43799E+00_JPRB, 0.47947E+00_JPRB ,&
&  0.49134E+00_JPRB, 0.49208E+00_JPRB, 0.49264E+00_JPRB, 0.50245E+00_JPRB, 0.50148E+00_JPRB, 0.49710E+00_JPRB, 0.48803E+00_JPRB ,&
&  0.47472E+00_JPRB, 0.46043E+00_JPRB, 0.44956E+00_JPRB, 0.44628E+00_JPRB, 0.45422E+00_JPRB, 0.47172E+00_JPRB, 0.48968E+00_JPRB ,&
&  0.49980E+00_JPRB, 0.48886E+00_JPRB, 0.45519E+00_JPRB, 0.42547E+00_JPRB, 0.41685E+00_JPRB, 0.41949E+00_JPRB, 0.42337E+00_JPRB ,&
&  0.42081E+00_JPRB, 0.41749E+00_JPRB, 0.41446E+00_JPRB, 0.41005E+00_JPRB, 0.39489E+00_JPRB, 0.38507E+00_JPRB, 0.37709E+00_JPRB ,&
&  0.37090E+00_JPRB, 0.36661E+00_JPRB, 0.36783E+00_JPRB, 0.36511E+00_JPRB, 0.36291E+00_JPRB, 0.36024E+00_JPRB, 0.35578E+00_JPRB ,&
&  0.35267E+00_JPRB, 0.35118E+00_JPRB, 0.35036E+00_JPRB, 0.35103E+00_JPRB, 0.35324E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.36471E+00_JPRB, 0.36567E+00_JPRB, 0.36672E+00_JPRB, 0.36451E+00_JPRB, 0.36263E+00_JPRB, 0.36215E+00_JPRB, 0.36202E+00_JPRB ,&
&  0.37129E+00_JPRB, 0.35828E+00_JPRB, 0.34684E+00_JPRB, 0.33804E+00_JPRB, 0.33305E+00_JPRB, 0.33122E+00_JPRB, 0.33405E+00_JPRB ,&
&  0.33893E+00_JPRB, 0.34380E+00_JPRB, 0.34766E+00_JPRB, 0.35129E+00_JPRB, 0.35477E+00_JPRB, 0.35694E+00_JPRB, 0.35904E+00_JPRB ,&
&  0.36205E+00_JPRB, 0.36517E+00_JPRB, 0.36860E+00_JPRB, 0.37486E+00_JPRB, 0.38072E+00_JPRB, 0.38659E+00_JPRB, 0.38720E+00_JPRB ,&
&  0.38344E+00_JPRB, 0.37860E+00_JPRB, 0.37779E+00_JPRB, 0.37985E+00_JPRB, 0.38073E+00_JPRB, 0.37377E+00_JPRB, 0.34773E+00_JPRB ,&
&  0.32294E+00_JPRB, 0.31385E+00_JPRB, 0.32361E+00_JPRB, 0.34216E+00_JPRB, 0.34718E+00_JPRB, 0.35159E+00_JPRB, 0.35304E+00_JPRB ,&
&  0.35041E+00_JPRB, 0.34958E+00_JPRB, 0.35539E+00_JPRB, 0.36426E+00_JPRB, 0.36699E+00_JPRB, 0.36928E+00_JPRB, 0.37336E+00_JPRB ,&
&  0.38419E+00_JPRB, 0.39555E+00_JPRB, 0.40273E+00_JPRB, 0.39732E+00_JPRB, 0.44939E+00_JPRB, 0.64097E+00_JPRB, 0.63032E+00_JPRB ,&
&  0.62484E+00_JPRB, 0.61574E+00_JPRB, 0.61399E+00_JPRB, 0.60917E+00_JPRB, 0.58633E+00_JPRB, 0.56899E+00_JPRB, 0.55571E+00_JPRB ,&
&  0.53302E+00_JPRB, 0.51952E+00_JPRB, 0.51482E+00_JPRB, 0.52984E+00_JPRB, 0.54210E+00_JPRB, 0.54940E+00_JPRB, 0.55580E+00_JPRB ,&
&  0.54982E+00_JPRB, 0.52398E+00_JPRB, 0.49263E+00_JPRB, 0.46425E+00_JPRB, 0.44989E+00_JPRB, 0.44105E+00_JPRB, 0.43569E+00_JPRB ,&
&  0.42940E+00_JPRB, 0.42682E+00_JPRB, 0.42657E+00_JPRB, 0.42405E+00_JPRB, 0.40981E+00_JPRB, 0.39254E+00_JPRB, 0.38251E+00_JPRB ,&
&  0.37591E+00_JPRB, 0.36947E+00_JPRB, 0.36414E+00_JPRB, 0.35939E+00_JPRB, 0.35961E+00_JPRB, 0.35940E+00_JPRB, 0.35606E+00_JPRB ,&
&  0.35633E+00_JPRB, 0.35388E+00_JPRB, 0.35421E+00_JPRB, 0.35634E+00_JPRB, 0.36176E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.36865E+00_JPRB, 0.37293E+00_JPRB, 0.37586E+00_JPRB, 0.37689E+00_JPRB, 0.37977E+00_JPRB, 0.38088E+00_JPRB, 0.37566E+00_JPRB ,&
&  0.39109E+00_JPRB, 0.36217E+00_JPRB, 0.35001E+00_JPRB, 0.34402E+00_JPRB, 0.34327E+00_JPRB, 0.34017E+00_JPRB, 0.33539E+00_JPRB ,&
&  0.33694E+00_JPRB, 0.34375E+00_JPRB, 0.35202E+00_JPRB, 0.35774E+00_JPRB, 0.36245E+00_JPRB, 0.36518E+00_JPRB, 0.36732E+00_JPRB ,&
&  0.37067E+00_JPRB, 0.37575E+00_JPRB, 0.38315E+00_JPRB, 0.39022E+00_JPRB, 0.39550E+00_JPRB, 0.40129E+00_JPRB, 0.40199E+00_JPRB ,&
&  0.40091E+00_JPRB, 0.39613E+00_JPRB, 0.39054E+00_JPRB, 0.38799E+00_JPRB, 0.38578E+00_JPRB, 0.38561E+00_JPRB, 0.37793E+00_JPRB ,&
&  0.34943E+00_JPRB, 0.34823E+00_JPRB, 0.36752E+00_JPRB, 0.37302E+00_JPRB, 0.37305E+00_JPRB, 0.37082E+00_JPRB, 0.37135E+00_JPRB ,&
&  0.37346E+00_JPRB, 0.37556E+00_JPRB, 0.38004E+00_JPRB, 0.38231E+00_JPRB, 0.38590E+00_JPRB, 0.38977E+00_JPRB, 0.40184E+00_JPRB ,&
&  0.43016E+00_JPRB, 0.41608E+00_JPRB, 0.42220E+00_JPRB, 0.45778E+00_JPRB, 0.62293E+00_JPRB, 0.68859E+00_JPRB, 0.73129E+00_JPRB ,&
&  0.77414E+00_JPRB, 0.75523E+00_JPRB, 0.72417E+00_JPRB, 0.70235E+00_JPRB, 0.68459E+00_JPRB, 0.66626E+00_JPRB, 0.65608E+00_JPRB ,&
&  0.63220E+00_JPRB, 0.62927E+00_JPRB, 0.63491E+00_JPRB, 0.64975E+00_JPRB, 0.65009E+00_JPRB, 0.64749E+00_JPRB, 0.64024E+00_JPRB ,&
&  0.62302E+00_JPRB, 0.59701E+00_JPRB, 0.55956E+00_JPRB, 0.52329E+00_JPRB, 0.49339E+00_JPRB, 0.47819E+00_JPRB, 0.46763E+00_JPRB ,&
&  0.45883E+00_JPRB, 0.45358E+00_JPRB, 0.44805E+00_JPRB, 0.44150E+00_JPRB, 0.43333E+00_JPRB, 0.42495E+00_JPRB, 0.42152E+00_JPRB ,&
&  0.40569E+00_JPRB, 0.39443E+00_JPRB, 0.38268E+00_JPRB, 0.37365E+00_JPRB, 0.37161E+00_JPRB, 0.37141E+00_JPRB, 0.37130E+00_JPRB ,&
&  0.37343E+00_JPRB, 0.36738E+00_JPRB, 0.36643E+00_JPRB, 0.36694E+00_JPRB, 0.36871E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.37452E+00_JPRB, 0.38242E+00_JPRB, 0.38624E+00_JPRB, 0.38931E+00_JPRB, 0.39560E+00_JPRB, 0.39864E+00_JPRB, 0.40162E+00_JPRB ,&
&  0.44388E+00_JPRB, 0.42025E+00_JPRB, 0.43151E+00_JPRB, 0.38293E+00_JPRB, 0.36310E+00_JPRB, 0.35522E+00_JPRB, 0.34439E+00_JPRB ,&
&  0.34316E+00_JPRB, 0.34763E+00_JPRB, 0.35282E+00_JPRB, 0.35877E+00_JPRB, 0.36615E+00_JPRB, 0.36986E+00_JPRB, 0.37278E+00_JPRB ,&
&  0.37683E+00_JPRB, 0.38262E+00_JPRB, 0.38964E+00_JPRB, 0.39497E+00_JPRB, 0.40005E+00_JPRB, 0.40643E+00_JPRB, 0.40648E+00_JPRB ,&
&  0.39726E+00_JPRB, 0.39280E+00_JPRB, 0.39197E+00_JPRB, 0.38897E+00_JPRB, 0.39054E+00_JPRB, 0.39176E+00_JPRB, 0.39913E+00_JPRB ,&
&  0.40098E+00_JPRB, 0.39107E+00_JPRB, 0.39970E+00_JPRB, 0.39698E+00_JPRB, 0.39535E+00_JPRB, 0.39297E+00_JPRB, 0.39057E+00_JPRB ,&
&  0.39018E+00_JPRB, 0.39305E+00_JPRB, 0.39912E+00_JPRB, 0.40424E+00_JPRB, 0.40908E+00_JPRB, 0.42048E+00_JPRB, 0.44499E+00_JPRB ,&
&  0.48894E+00_JPRB, 0.45847E+00_JPRB, 0.53238E+00_JPRB, 0.56729E+00_JPRB, 0.75214E+00_JPRB, 0.93371E+00_JPRB, 0.93085E+00_JPRB ,&
&  0.99589E+00_JPRB, 0.94511E+00_JPRB, 0.88729E+00_JPRB, 0.84030E+00_JPRB, 0.81550E+00_JPRB, 0.77306E+00_JPRB, 0.72895E+00_JPRB ,&
&  0.71680E+00_JPRB, 0.71074E+00_JPRB, 0.74750E+00_JPRB, 0.74577E+00_JPRB, 0.72470E+00_JPRB, 0.71332E+00_JPRB, 0.68980E+00_JPRB ,&
&  0.66951E+00_JPRB, 0.64482E+00_JPRB, 0.61780E+00_JPRB, 0.56852E+00_JPRB, 0.54806E+00_JPRB, 0.52834E+00_JPRB, 0.49605E+00_JPRB ,&
&  0.48163E+00_JPRB, 0.46403E+00_JPRB, 0.44827E+00_JPRB, 0.45113E+00_JPRB, 0.43923E+00_JPRB, 0.43321E+00_JPRB, 0.42976E+00_JPRB ,&
&  0.41820E+00_JPRB, 0.40830E+00_JPRB, 0.40862E+00_JPRB, 0.40366E+00_JPRB, 0.39470E+00_JPRB, 0.38725E+00_JPRB, 0.38436E+00_JPRB ,&
&  0.38058E+00_JPRB, 0.37845E+00_JPRB, 0.38132E+00_JPRB, 0.37626E+00_JPRB, 0.37358E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.38765E+00_JPRB, 0.39320E+00_JPRB, 0.39480E+00_JPRB, 0.40032E+00_JPRB, 0.41034E+00_JPRB, 0.42048E+00_JPRB, 0.42828E+00_JPRB ,&
&  0.44782E+00_JPRB, 0.45566E+00_JPRB, 0.45658E+00_JPRB, 0.45184E+00_JPRB, 0.44125E+00_JPRB, 0.40499E+00_JPRB, 0.35476E+00_JPRB ,&
&  0.34343E+00_JPRB, 0.35116E+00_JPRB, 0.36095E+00_JPRB, 0.36571E+00_JPRB, 0.37153E+00_JPRB, 0.37718E+00_JPRB, 0.38169E+00_JPRB ,&
&  0.38665E+00_JPRB, 0.39016E+00_JPRB, 0.39548E+00_JPRB, 0.40106E+00_JPRB, 0.40508E+00_JPRB, 0.41083E+00_JPRB, 0.41484E+00_JPRB ,&
&  0.40027E+00_JPRB, 0.39214E+00_JPRB, 0.38838E+00_JPRB, 0.38788E+00_JPRB, 0.40206E+00_JPRB, 0.40950E+00_JPRB, 0.41853E+00_JPRB ,&
&  0.42673E+00_JPRB, 0.43081E+00_JPRB, 0.42970E+00_JPRB, 0.42793E+00_JPRB, 0.42513E+00_JPRB, 0.42404E+00_JPRB, 0.42149E+00_JPRB ,&
&  0.42073E+00_JPRB, 0.42340E+00_JPRB, 0.43132E+00_JPRB, 0.43798E+00_JPRB, 0.44770E+00_JPRB, 0.46902E+00_JPRB, 0.52495E+00_JPRB ,&
&  0.58840E+00_JPRB, 0.66747E+00_JPRB, 0.71444E+00_JPRB, 0.71407E+00_JPRB, 0.86304E+00_JPRB, 0.96680E+00_JPRB, 0.97670E+00_JPRB ,&
&  0.99175E+00_JPRB, 0.92104E+00_JPRB, 0.94557E+00_JPRB, 0.96879E+00_JPRB, 0.97258E+00_JPRB, 0.92441E+00_JPRB, 0.85236E+00_JPRB ,&
&  0.83592E+00_JPRB, 0.89386E+00_JPRB, 0.89288E+00_JPRB, 0.88659E+00_JPRB, 0.84887E+00_JPRB, 0.82089E+00_JPRB, 0.78467E+00_JPRB ,&
&  0.77401E+00_JPRB, 0.75938E+00_JPRB, 0.72127E+00_JPRB, 0.68029E+00_JPRB, 0.63643E+00_JPRB, 0.59325E+00_JPRB, 0.54994E+00_JPRB ,&
&  0.50982E+00_JPRB, 0.46361E+00_JPRB, 0.44650E+00_JPRB, 0.44532E+00_JPRB, 0.44741E+00_JPRB, 0.43592E+00_JPRB, 0.43206E+00_JPRB ,&
&  0.42941E+00_JPRB, 0.43144E+00_JPRB, 0.43747E+00_JPRB, 0.42757E+00_JPRB, 0.41247E+00_JPRB, 0.39573E+00_JPRB, 0.38539E+00_JPRB ,&
&  0.37818E+00_JPRB, 0.38194E+00_JPRB, 0.38579E+00_JPRB, 0.38444E+00_JPRB, 0.38378E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.39898E+00_JPRB, 0.39889E+00_JPRB, 0.40116E+00_JPRB, 0.41084E+00_JPRB, 0.42480E+00_JPRB, 0.43951E+00_JPRB, 0.45698E+00_JPRB ,&
&  0.47353E+00_JPRB, 0.48255E+00_JPRB, 0.47786E+00_JPRB, 0.46815E+00_JPRB, 0.46007E+00_JPRB, 0.45623E+00_JPRB, 0.42871E+00_JPRB ,&
&  0.38471E+00_JPRB, 0.38339E+00_JPRB, 0.37552E+00_JPRB, 0.37811E+00_JPRB, 0.38832E+00_JPRB, 0.39383E+00_JPRB, 0.39903E+00_JPRB ,&
&  0.40754E+00_JPRB, 0.41005E+00_JPRB, 0.41181E+00_JPRB, 0.41358E+00_JPRB, 0.41780E+00_JPRB, 0.43141E+00_JPRB, 0.44219E+00_JPRB ,&
&  0.42145E+00_JPRB, 0.40795E+00_JPRB, 0.40579E+00_JPRB, 0.41281E+00_JPRB, 0.42721E+00_JPRB, 0.44272E+00_JPRB, 0.45316E+00_JPRB ,&
&  0.46237E+00_JPRB, 0.46749E+00_JPRB, 0.46854E+00_JPRB, 0.46702E+00_JPRB, 0.46433E+00_JPRB, 0.46054E+00_JPRB, 0.45943E+00_JPRB ,&
&  0.45987E+00_JPRB, 0.46396E+00_JPRB, 0.47320E+00_JPRB, 0.48266E+00_JPRB, 0.49311E+00_JPRB, 0.54421E+00_JPRB, 0.65457E+00_JPRB ,&
&  0.78032E+00_JPRB, 0.90220E+00_JPRB, 0.93248E+00_JPRB, 0.10027E+01_JPRB, 0.10601E+01_JPRB, 0.11030E+01_JPRB, 0.11012E+01_JPRB ,&
&  0.10907E+01_JPRB, 0.10054E+01_JPRB, 0.11015E+01_JPRB, 0.11306E+01_JPRB, 0.10859E+01_JPRB, 0.10818E+01_JPRB, 0.10143E+01_JPRB ,&
&  0.97350E+00_JPRB, 0.94885E+00_JPRB, 0.98369E+00_JPRB, 0.98612E+00_JPRB, 0.96085E+00_JPRB, 0.96366E+00_JPRB, 0.92134E+00_JPRB ,&
&  0.89438E+00_JPRB, 0.85175E+00_JPRB, 0.74166E+00_JPRB, 0.66082E+00_JPRB, 0.58582E+00_JPRB, 0.60097E+00_JPRB, 0.56757E+00_JPRB ,&
&  0.52337E+00_JPRB, 0.46923E+00_JPRB, 0.46853E+00_JPRB, 0.46678E+00_JPRB, 0.47467E+00_JPRB, 0.47643E+00_JPRB, 0.45948E+00_JPRB ,&
&  0.45883E+00_JPRB, 0.46832E+00_JPRB, 0.45723E+00_JPRB, 0.44624E+00_JPRB, 0.43242E+00_JPRB, 0.41358E+00_JPRB, 0.40279E+00_JPRB ,&
&  0.40217E+00_JPRB, 0.40515E+00_JPRB, 0.40382E+00_JPRB, 0.39966E+00_JPRB, 0.39848E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.42415E+00_JPRB, 0.42425E+00_JPRB, 0.42792E+00_JPRB, 0.43745E+00_JPRB, 0.45046E+00_JPRB, 0.46750E+00_JPRB, 0.48516E+00_JPRB ,&
&  0.49909E+00_JPRB, 0.50279E+00_JPRB, 0.49332E+00_JPRB, 0.48630E+00_JPRB, 0.48184E+00_JPRB, 0.47609E+00_JPRB, 0.47962E+00_JPRB ,&
&  0.42658E+00_JPRB, 0.40847E+00_JPRB, 0.38298E+00_JPRB, 0.38429E+00_JPRB, 0.40384E+00_JPRB, 0.41034E+00_JPRB, 0.41642E+00_JPRB ,&
&  0.42068E+00_JPRB, 0.42856E+00_JPRB, 0.42929E+00_JPRB, 0.43214E+00_JPRB, 0.43944E+00_JPRB, 0.46025E+00_JPRB, 0.48423E+00_JPRB ,&
&  0.46848E+00_JPRB, 0.45193E+00_JPRB, 0.45524E+00_JPRB, 0.46795E+00_JPRB, 0.47673E+00_JPRB, 0.49565E+00_JPRB, 0.51818E+00_JPRB ,&
&  0.52405E+00_JPRB, 0.52092E+00_JPRB, 0.51290E+00_JPRB, 0.50827E+00_JPRB, 0.50628E+00_JPRB, 0.50482E+00_JPRB, 0.50943E+00_JPRB ,&
&  0.51673E+00_JPRB, 0.52390E+00_JPRB, 0.53390E+00_JPRB, 0.54239E+00_JPRB, 0.54729E+00_JPRB, 0.62780E+00_JPRB, 0.77473E+00_JPRB ,&
&  0.96285E+00_JPRB, 0.10007E+01_JPRB, 0.10749E+01_JPRB, 0.12842E+01_JPRB, 0.13597E+01_JPRB, 0.13732E+01_JPRB, 0.13013E+01_JPRB ,&
&  0.12344E+01_JPRB, 0.11435E+01_JPRB, 0.11202E+01_JPRB, 0.11185E+01_JPRB, 0.10896E+01_JPRB, 0.11344E+01_JPRB, 0.11017E+01_JPRB ,&
&  0.10330E+01_JPRB, 0.10142E+01_JPRB, 0.10112E+01_JPRB, 0.10004E+01_JPRB, 0.96697E+00_JPRB, 0.96405E+00_JPRB, 0.92903E+00_JPRB ,&
&  0.87685E+00_JPRB, 0.76319E+00_JPRB, 0.61226E+00_JPRB, 0.58190E+00_JPRB, 0.50668E+00_JPRB, 0.52677E+00_JPRB, 0.59823E+00_JPRB ,&
&  0.60386E+00_JPRB, 0.57059E+00_JPRB, 0.53959E+00_JPRB, 0.52375E+00_JPRB, 0.51954E+00_JPRB, 0.54174E+00_JPRB, 0.51750E+00_JPRB ,&
&  0.49219E+00_JPRB, 0.49914E+00_JPRB, 0.49733E+00_JPRB, 0.48557E+00_JPRB, 0.46851E+00_JPRB, 0.45311E+00_JPRB, 0.44340E+00_JPRB ,&
&  0.44242E+00_JPRB, 0.44201E+00_JPRB, 0.43952E+00_JPRB, 0.42963E+00_JPRB, 0.42388E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.46403E+00_JPRB, 0.46674E+00_JPRB, 0.46834E+00_JPRB, 0.47672E+00_JPRB, 0.48795E+00_JPRB, 0.50214E+00_JPRB, 0.51659E+00_JPRB ,&
&  0.52401E+00_JPRB, 0.51965E+00_JPRB, 0.50872E+00_JPRB, 0.50684E+00_JPRB, 0.50229E+00_JPRB, 0.49370E+00_JPRB, 0.49265E+00_JPRB ,&
&  0.50419E+00_JPRB, 0.46232E+00_JPRB, 0.42358E+00_JPRB, 0.39446E+00_JPRB, 0.40318E+00_JPRB, 0.40827E+00_JPRB, 0.42090E+00_JPRB ,&
&  0.42828E+00_JPRB, 0.44588E+00_JPRB, 0.45314E+00_JPRB, 0.45807E+00_JPRB, 0.47174E+00_JPRB, 0.50495E+00_JPRB, 0.53727E+00_JPRB ,&
&  0.54307E+00_JPRB, 0.54638E+00_JPRB, 0.55880E+00_JPRB, 0.58270E+00_JPRB, 0.60152E+00_JPRB, 0.57326E+00_JPRB, 0.59369E+00_JPRB ,&
&  0.59043E+00_JPRB, 0.58004E+00_JPRB, 0.56431E+00_JPRB, 0.56430E+00_JPRB, 0.57081E+00_JPRB, 0.58307E+00_JPRB, 0.59835E+00_JPRB ,&
&  0.61651E+00_JPRB, 0.62033E+00_JPRB, 0.60117E+00_JPRB, 0.59933E+00_JPRB, 0.61809E+00_JPRB, 0.74352E+00_JPRB, 0.82874E+00_JPRB ,&
&  0.96958E+00_JPRB, 0.96589E+00_JPRB, 0.97769E+00_JPRB, 0.11624E+01_JPRB, 0.13181E+01_JPRB, 0.13773E+01_JPRB, 0.13712E+01_JPRB ,&
&  0.13297E+01_JPRB, 0.12729E+01_JPRB, 0.12409E+01_JPRB, 0.11769E+01_JPRB, 0.11671E+01_JPRB, 0.11849E+01_JPRB, 0.11484E+01_JPRB ,&
&  0.10688E+01_JPRB, 0.10069E+01_JPRB, 0.98535E+00_JPRB, 0.94601E+00_JPRB, 0.91242E+00_JPRB, 0.85138E+00_JPRB, 0.85946E+00_JPRB ,&
&  0.85679E+00_JPRB, 0.57387E+00_JPRB, 0.50133E+00_JPRB, 0.55181E+00_JPRB, 0.49972E+00_JPRB, 0.51505E+00_JPRB, 0.56912E+00_JPRB ,&
&  0.57210E+00_JPRB, 0.57054E+00_JPRB, 0.58748E+00_JPRB, 0.57186E+00_JPRB, 0.57926E+00_JPRB, 0.59803E+00_JPRB, 0.58376E+00_JPRB ,&
&  0.56584E+00_JPRB, 0.54618E+00_JPRB, 0.54883E+00_JPRB, 0.53480E+00_JPRB, 0.51549E+00_JPRB, 0.50088E+00_JPRB, 0.48980E+00_JPRB ,&
&  0.48060E+00_JPRB, 0.47631E+00_JPRB, 0.47524E+00_JPRB, 0.46710E+00_JPRB, 0.46155E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.52213E+00_JPRB, 0.52577E+00_JPRB, 0.52882E+00_JPRB, 0.53400E+00_JPRB, 0.53846E+00_JPRB, 0.54768E+00_JPRB, 0.55356E+00_JPRB ,&
&  0.55074E+00_JPRB, 0.53675E+00_JPRB, 0.52684E+00_JPRB, 0.53007E+00_JPRB, 0.52031E+00_JPRB, 0.50896E+00_JPRB, 0.50952E+00_JPRB ,&
&  0.52288E+00_JPRB, 0.51554E+00_JPRB, 0.44590E+00_JPRB, 0.41256E+00_JPRB, 0.38846E+00_JPRB, 0.39832E+00_JPRB, 0.40986E+00_JPRB ,&
&  0.43324E+00_JPRB, 0.44729E+00_JPRB, 0.45649E+00_JPRB, 0.47244E+00_JPRB, 0.50841E+00_JPRB, 0.55573E+00_JPRB, 0.60916E+00_JPRB ,&
&  0.63619E+00_JPRB, 0.66567E+00_JPRB, 0.65250E+00_JPRB, 0.69853E+00_JPRB, 0.70600E+00_JPRB, 0.66617E+00_JPRB, 0.68970E+00_JPRB ,&
&  0.68018E+00_JPRB, 0.66990E+00_JPRB, 0.65515E+00_JPRB, 0.65929E+00_JPRB, 0.66472E+00_JPRB, 0.65516E+00_JPRB, 0.64003E+00_JPRB ,&
&  0.62936E+00_JPRB, 0.62641E+00_JPRB, 0.62151E+00_JPRB, 0.61483E+00_JPRB, 0.64872E+00_JPRB, 0.73466E+00_JPRB, 0.77267E+00_JPRB ,&
&  0.85003E+00_JPRB, 0.72509E+00_JPRB, 0.64336E+00_JPRB, 0.85711E+00_JPRB, 0.11182E+01_JPRB, 0.11932E+01_JPRB, 0.11498E+01_JPRB ,&
&  0.12450E+01_JPRB, 0.12689E+01_JPRB, 0.12951E+01_JPRB, 0.13118E+01_JPRB, 0.13001E+01_JPRB, 0.12745E+01_JPRB, 0.12149E+01_JPRB ,&
&  0.11081E+01_JPRB, 0.10316E+01_JPRB, 0.99295E+00_JPRB, 0.10028E+01_JPRB, 0.96169E+00_JPRB, 0.92336E+00_JPRB, 0.88885E+00_JPRB ,&
&  0.78494E+00_JPRB, 0.70046E+00_JPRB, 0.56046E+00_JPRB, 0.56746E+00_JPRB, 0.48910E+00_JPRB, 0.49425E+00_JPRB, 0.54412E+00_JPRB ,&
&  0.54762E+00_JPRB, 0.57797E+00_JPRB, 0.59089E+00_JPRB, 0.58166E+00_JPRB, 0.64587E+00_JPRB, 0.66428E+00_JPRB, 0.64990E+00_JPRB ,&
&  0.63852E+00_JPRB, 0.59182E+00_JPRB, 0.60515E+00_JPRB, 0.60754E+00_JPRB, 0.59524E+00_JPRB, 0.58151E+00_JPRB, 0.56699E+00_JPRB ,&
&  0.55202E+00_JPRB, 0.54030E+00_JPRB, 0.53353E+00_JPRB, 0.52351E+00_JPRB, 0.51942E+00_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.57588E+00_JPRB, 0.56705E+00_JPRB, 0.56273E+00_JPRB, 0.56388E+00_JPRB, 0.56714E+00_JPRB, 0.57115E+00_JPRB, 0.57292E+00_JPRB ,&
&  0.56907E+00_JPRB, 0.55473E+00_JPRB, 0.55272E+00_JPRB, 0.55109E+00_JPRB, 0.53299E+00_JPRB, 0.51947E+00_JPRB, 0.52323E+00_JPRB ,&
&  0.54065E+00_JPRB, 0.50755E+00_JPRB, 0.42904E+00_JPRB, 0.38927E+00_JPRB, 0.37011E+00_JPRB, 0.37711E+00_JPRB, 0.39458E+00_JPRB ,&
&  0.41790E+00_JPRB, 0.43578E+00_JPRB, 0.45814E+00_JPRB, 0.49046E+00_JPRB, 0.54027E+00_JPRB, 0.59487E+00_JPRB, 0.70080E+00_JPRB ,&
&  0.83860E+00_JPRB, 0.75957E+00_JPRB, 0.78199E+00_JPRB, 0.77288E+00_JPRB, 0.79919E+00_JPRB, 0.78146E+00_JPRB, 0.75460E+00_JPRB ,&
&  0.71785E+00_JPRB, 0.69613E+00_JPRB, 0.67694E+00_JPRB, 0.65522E+00_JPRB, 0.63557E+00_JPRB, 0.62581E+00_JPRB, 0.63194E+00_JPRB ,&
&  0.65925E+00_JPRB, 0.66885E+00_JPRB, 0.65747E+00_JPRB, 0.66918E+00_JPRB, 0.74997E+00_JPRB, 0.77294E+00_JPRB, 0.76623E+00_JPRB ,&
&  0.70522E+00_JPRB, 0.72375E+00_JPRB, 0.92424E+00_JPRB, 0.10466E+01_JPRB, 0.11283E+01_JPRB, 0.11921E+01_JPRB, 0.12133E+01_JPRB ,&
&  0.12928E+01_JPRB, 0.12682E+01_JPRB, 0.12872E+01_JPRB, 0.11409E+01_JPRB, 0.12514E+01_JPRB, 0.12415E+01_JPRB, 0.11962E+01_JPRB ,&
&  0.11389E+01_JPRB, 0.11154E+01_JPRB, 0.10776E+01_JPRB, 0.11003E+01_JPRB, 0.10854E+01_JPRB, 0.10564E+01_JPRB, 0.84044E+00_JPRB ,&
&  0.69744E+00_JPRB, 0.72035E+00_JPRB, 0.66173E+00_JPRB, 0.59866E+00_JPRB, 0.54070E+00_JPRB, 0.54742E+00_JPRB, 0.55794E+00_JPRB ,&
&  0.56930E+00_JPRB, 0.57731E+00_JPRB, 0.57343E+00_JPRB, 0.59959E+00_JPRB, 0.67384E+00_JPRB, 0.75151E+00_JPRB, 0.74397E+00_JPRB ,&
&  0.69967E+00_JPRB, 0.71675E+00_JPRB, 0.72296E+00_JPRB, 0.70613E+00_JPRB, 0.73370E+00_JPRB, 0.71576E+00_JPRB, 0.68476E+00_JPRB ,&
&  0.65528E+00_JPRB, 0.63834E+00_JPRB, 0.62570E+00_JPRB, 0.60954E+00_JPRB, 0.58990E+00_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.63129E+00_JPRB, 0.61894E+00_JPRB, 0.60828E+00_JPRB, 0.60176E+00_JPRB, 0.59605E+00_JPRB, 0.59335E+00_JPRB, 0.59017E+00_JPRB ,&
&  0.57949E+00_JPRB, 0.56075E+00_JPRB, 0.56616E+00_JPRB, 0.55934E+00_JPRB, 0.54537E+00_JPRB, 0.53347E+00_JPRB, 0.53951E+00_JPRB ,&
&  0.55565E+00_JPRB, 0.49751E+00_JPRB, 0.42129E+00_JPRB, 0.41189E+00_JPRB, 0.40173E+00_JPRB, 0.38260E+00_JPRB, 0.39001E+00_JPRB ,&
&  0.40894E+00_JPRB, 0.43124E+00_JPRB, 0.45352E+00_JPRB, 0.51062E+00_JPRB, 0.58688E+00_JPRB, 0.68688E+00_JPRB, 0.82635E+00_JPRB ,&
&  0.85831E+00_JPRB, 0.89134E+00_JPRB, 0.94679E+00_JPRB, 0.89921E+00_JPRB, 0.84695E+00_JPRB, 0.79474E+00_JPRB, 0.74975E+00_JPRB ,&
&  0.71858E+00_JPRB, 0.69550E+00_JPRB, 0.67713E+00_JPRB, 0.66212E+00_JPRB, 0.66270E+00_JPRB, 0.66576E+00_JPRB, 0.67229E+00_JPRB ,&
&  0.68266E+00_JPRB, 0.67477E+00_JPRB, 0.65358E+00_JPRB, 0.72368E+00_JPRB, 0.65852E+00_JPRB, 0.63868E+00_JPRB, 0.64347E+00_JPRB ,&
&  0.80584E+00_JPRB, 0.90365E+00_JPRB, 0.96774E+00_JPRB, 0.11669E+01_JPRB, 0.10413E+01_JPRB, 0.11124E+01_JPRB, 0.13246E+01_JPRB ,&
&  0.13818E+01_JPRB, 0.13773E+01_JPRB, 0.14536E+01_JPRB, 0.11831E+01_JPRB, 0.10373E+01_JPRB, 0.13365E+01_JPRB, 0.13331E+01_JPRB ,&
&  0.12855E+01_JPRB, 0.12617E+01_JPRB, 0.12464E+01_JPRB, 0.13551E+01_JPRB, 0.95887E+00_JPRB, 0.77687E+00_JPRB, 0.62784E+00_JPRB ,&
&  0.61954E+00_JPRB, 0.59618E+00_JPRB, 0.65130E+00_JPRB, 0.62598E+00_JPRB, 0.61808E+00_JPRB, 0.60207E+00_JPRB, 0.57545E+00_JPRB ,&
&  0.57337E+00_JPRB, 0.57199E+00_JPRB, 0.57748E+00_JPRB, 0.64718E+00_JPRB, 0.76256E+00_JPRB, 0.79159E+00_JPRB, 0.85281E+00_JPRB ,&
&  0.86922E+00_JPRB, 0.90825E+00_JPRB, 0.87939E+00_JPRB, 0.87607E+00_JPRB, 0.85952E+00_JPRB, 0.82878E+00_JPRB, 0.78560E+00_JPRB ,&
&  0.75497E+00_JPRB, 0.72857E+00_JPRB, 0.70056E+00_JPRB, 0.67425E+00_JPRB, 0.65045E+00_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.67695E+00_JPRB, 0.65701E+00_JPRB, 0.63862E+00_JPRB, 0.62338E+00_JPRB, 0.61598E+00_JPRB, 0.60981E+00_JPRB, 0.60147E+00_JPRB ,&
&  0.58325E+00_JPRB, 0.56849E+00_JPRB, 0.57288E+00_JPRB, 0.55121E+00_JPRB, 0.53615E+00_JPRB, 0.53774E+00_JPRB, 0.55731E+00_JPRB ,&
&  0.56671E+00_JPRB, 0.54127E+00_JPRB, 0.46062E+00_JPRB, 0.42464E+00_JPRB, 0.43136E+00_JPRB, 0.40620E+00_JPRB, 0.39824E+00_JPRB ,&
&  0.42092E+00_JPRB, 0.44574E+00_JPRB, 0.49221E+00_JPRB, 0.59594E+00_JPRB, 0.71390E+00_JPRB, 0.85233E+00_JPRB, 0.10062E+01_JPRB ,&
&  0.10276E+01_JPRB, 0.10081E+01_JPRB, 0.94854E+00_JPRB, 0.87480E+00_JPRB, 0.79591E+00_JPRB, 0.74355E+00_JPRB, 0.70546E+00_JPRB ,&
&  0.67928E+00_JPRB, 0.66693E+00_JPRB, 0.66734E+00_JPRB, 0.65796E+00_JPRB, 0.66480E+00_JPRB, 0.63776E+00_JPRB, 0.63079E+00_JPRB ,&
&  0.64206E+00_JPRB, 0.65345E+00_JPRB, 0.66580E+00_JPRB, 0.77097E+00_JPRB, 0.72127E+00_JPRB, 0.66275E+00_JPRB, 0.68952E+00_JPRB ,&
&  0.78737E+00_JPRB, 0.84475E+00_JPRB, 0.95627E+00_JPRB, 0.11741E+01_JPRB, 0.14485E+01_JPRB, 0.15559E+01_JPRB, 0.16519E+01_JPRB ,&
&  0.13481E+01_JPRB, 0.11870E+01_JPRB, 0.11856E+01_JPRB, 0.11095E+01_JPRB, 0.13901E+01_JPRB, 0.18363E+01_JPRB, 0.18384E+01_JPRB ,&
&  0.17525E+01_JPRB, 0.16521E+01_JPRB, 0.16390E+01_JPRB, 0.13334E+01_JPRB, 0.93372E+00_JPRB, 0.68854E+00_JPRB, 0.76472E+00_JPRB ,&
&  0.78988E+00_JPRB, 0.79943E+00_JPRB, 0.73070E+00_JPRB, 0.64572E+00_JPRB, 0.58241E+00_JPRB, 0.65073E+00_JPRB, 0.63997E+00_JPRB ,&
&  0.61311E+00_JPRB, 0.60333E+00_JPRB, 0.69837E+00_JPRB, 0.83610E+00_JPRB, 0.91414E+00_JPRB, 0.97193E+00_JPRB, 0.10991E+01_JPRB ,&
&  0.11443E+01_JPRB, 0.11987E+01_JPRB, 0.11539E+01_JPRB, 0.10884E+01_JPRB, 0.10212E+01_JPRB, 0.94268E+00_JPRB, 0.87657E+00_JPRB ,&
&  0.82651E+00_JPRB, 0.78549E+00_JPRB, 0.75012E+00_JPRB, 0.71958E+00_JPRB, 0.69670E+00_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.71528E+00_JPRB, 0.68666E+00_JPRB, 0.66131E+00_JPRB, 0.64573E+00_JPRB, 0.63364E+00_JPRB, 0.62272E+00_JPRB, 0.60607E+00_JPRB ,&
&  0.58210E+00_JPRB, 0.58389E+00_JPRB, 0.57944E+00_JPRB, 0.55359E+00_JPRB, 0.54379E+00_JPRB, 0.54716E+00_JPRB, 0.55927E+00_JPRB ,&
&  0.56990E+00_JPRB, 0.59514E+00_JPRB, 0.50865E+00_JPRB, 0.46851E+00_JPRB, 0.46182E+00_JPRB, 0.44741E+00_JPRB, 0.41873E+00_JPRB ,&
&  0.44442E+00_JPRB, 0.47723E+00_JPRB, 0.53620E+00_JPRB, 0.66532E+00_JPRB, 0.87179E+00_JPRB, 0.96813E+00_JPRB, 0.99779E+00_JPRB ,&
&  0.10240E+01_JPRB, 0.97063E+00_JPRB, 0.90091E+00_JPRB, 0.83625E+00_JPRB, 0.76527E+00_JPRB, 0.72755E+00_JPRB, 0.70395E+00_JPRB ,&
&  0.69283E+00_JPRB, 0.70422E+00_JPRB, 0.70793E+00_JPRB, 0.72512E+00_JPRB, 0.69706E+00_JPRB, 0.66117E+00_JPRB, 0.64719E+00_JPRB ,&
&  0.64200E+00_JPRB, 0.65221E+00_JPRB, 0.67311E+00_JPRB, 0.78284E+00_JPRB, 0.81531E+00_JPRB, 0.76197E+00_JPRB, 0.85266E+00_JPRB ,&
&  0.89996E+00_JPRB, 0.99993E+00_JPRB, 0.10495E+01_JPRB, 0.13047E+01_JPRB, 0.15100E+01_JPRB, 0.16152E+01_JPRB, 0.17124E+01_JPRB ,&
&  0.16023E+01_JPRB, 0.16729E+01_JPRB, 0.19848E+01_JPRB, 0.23476E+01_JPRB, 0.20689E+01_JPRB, 0.20354E+01_JPRB, 0.26578E+01_JPRB ,&
&  0.19616E+01_JPRB, 0.19072E+01_JPRB, 0.19587E+01_JPRB, 0.18696E+01_JPRB, 0.79000E+00_JPRB, 0.61194E+00_JPRB, 0.78735E+00_JPRB ,&
&  0.80083E+00_JPRB, 0.58149E+00_JPRB, 0.48862E+00_JPRB, 0.54556E+00_JPRB, 0.47762E+00_JPRB, 0.53796E+00_JPRB, 0.68417E+00_JPRB ,&
&  0.71353E+00_JPRB, 0.73437E+00_JPRB, 0.92167E+00_JPRB, 0.10210E+01_JPRB, 0.10257E+01_JPRB, 0.12240E+01_JPRB, 0.14146E+01_JPRB ,&
&  0.14630E+01_JPRB, 0.14505E+01_JPRB, 0.14670E+01_JPRB, 0.13366E+01_JPRB, 0.11854E+01_JPRB, 0.10568E+01_JPRB, 0.95797E+00_JPRB ,&
&  0.87710E+00_JPRB, 0.82399E+00_JPRB, 0.78998E+00_JPRB, 0.75898E+00_JPRB, 0.73556E+00_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.71466E+00_JPRB, 0.68793E+00_JPRB, 0.66142E+00_JPRB, 0.64430E+00_JPRB, 0.63462E+00_JPRB, 0.62345E+00_JPRB, 0.60698E+00_JPRB ,&
&  0.59339E+00_JPRB, 0.60629E+00_JPRB, 0.58265E+00_JPRB, 0.56236E+00_JPRB, 0.54794E+00_JPRB, 0.54607E+00_JPRB, 0.55833E+00_JPRB ,&
&  0.56800E+00_JPRB, 0.58206E+00_JPRB, 0.56461E+00_JPRB, 0.52292E+00_JPRB, 0.50722E+00_JPRB, 0.48397E+00_JPRB, 0.46965E+00_JPRB ,&
&  0.49658E+00_JPRB, 0.54005E+00_JPRB, 0.61405E+00_JPRB, 0.73801E+00_JPRB, 0.85869E+00_JPRB, 0.92269E+00_JPRB, 0.99465E+00_JPRB ,&
&  0.97834E+00_JPRB, 0.92097E+00_JPRB, 0.86284E+00_JPRB, 0.80641E+00_JPRB, 0.75923E+00_JPRB, 0.73425E+00_JPRB, 0.71360E+00_JPRB ,&
&  0.70726E+00_JPRB, 0.71418E+00_JPRB, 0.72772E+00_JPRB, 0.69338E+00_JPRB, 0.66352E+00_JPRB, 0.64556E+00_JPRB, 0.63012E+00_JPRB ,&
&  0.62901E+00_JPRB, 0.66225E+00_JPRB, 0.70733E+00_JPRB, 0.77177E+00_JPRB, 0.87593E+00_JPRB, 0.90784E+00_JPRB, 0.84490E+00_JPRB ,&
&  0.80032E+00_JPRB, 0.88267E+00_JPRB, 0.11425E+01_JPRB, 0.12195E+01_JPRB, 0.12209E+01_JPRB, 0.13058E+01_JPRB, 0.13166E+01_JPRB ,&
&  0.14836E+01_JPRB, 0.18421E+01_JPRB, 0.18807E+01_JPRB, 0.22653E+01_JPRB, 0.24151E+01_JPRB, 0.18789E+01_JPRB, 0.20859E+01_JPRB ,&
&  0.20540E+01_JPRB, 0.17895E+01_JPRB, 0.13913E+01_JPRB, 0.10240E+01_JPRB, 0.10240E+01_JPRB, 0.83721E+00_JPRB, 0.56348E+00_JPRB ,&
&  0.51350E+00_JPRB, 0.49192E+00_JPRB, 0.47040E+00_JPRB, 0.47310E+00_JPRB, 0.47034E+00_JPRB, 0.54726E+00_JPRB, 0.68568E+00_JPRB ,&
&  0.87862E+00_JPRB, 0.99314E+00_JPRB, 0.10992E+01_JPRB, 0.11317E+01_JPRB, 0.10846E+01_JPRB, 0.12501E+01_JPRB, 0.16498E+01_JPRB ,&
&  0.14352E+01_JPRB, 0.13612E+01_JPRB, 0.13483E+01_JPRB, 0.12350E+01_JPRB, 0.11176E+01_JPRB, 0.10167E+01_JPRB, 0.92994E+00_JPRB ,&
&  0.86630E+00_JPRB, 0.82611E+00_JPRB, 0.79725E+00_JPRB, 0.76925E+00_JPRB, 0.74271E+00_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.70361E+00_JPRB, 0.68276E+00_JPRB, 0.66782E+00_JPRB, 0.65467E+00_JPRB, 0.64149E+00_JPRB, 0.63555E+00_JPRB, 0.61458E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.61337E+00_JPRB, 0.57560E+00_JPRB, 0.54774E+00_JPRB, 0.53221E+00_JPRB, 0.53286E+00_JPRB, 0.54697E+00_JPRB ,&
&  0.55518E+00_JPRB, 0.56914E+00_JPRB, 0.60322E+00_JPRB, 0.58646E+00_JPRB, 0.60692E+00_JPRB, 0.56210E+00_JPRB, 0.56864E+00_JPRB ,&
&  0.58572E+00_JPRB, 0.64494E+00_JPRB, 0.74179E+00_JPRB, 0.81973E+00_JPRB, 0.89383E+00_JPRB, 0.96376E+00_JPRB, 0.96584E+00_JPRB ,&
&  0.93822E+00_JPRB, 0.91759E+00_JPRB, 0.84098E+00_JPRB, 0.78468E+00_JPRB, 0.75810E+00_JPRB, 0.73053E+00_JPRB, 0.71279E+00_JPRB ,&
&  0.71490E+00_JPRB, 0.73075E+00_JPRB, 0.69837E+00_JPRB, 0.67977E+00_JPRB, 0.67246E+00_JPRB, 0.66283E+00_JPRB, 0.65094E+00_JPRB ,&
&  0.66607E+00_JPRB, 0.70388E+00_JPRB, 0.75466E+00_JPRB, 0.79962E+00_JPRB, 0.77436E+00_JPRB, 0.69043E+00_JPRB, 0.78472E+00_JPRB ,&
&  0.87429E+00_JPRB, 0.10363E+01_JPRB, 0.10865E+01_JPRB, 0.12835E+01_JPRB, 0.12085E+01_JPRB, 0.11598E+01_JPRB, 0.12743E+01_JPRB ,&
&  0.15451E+01_JPRB, 0.20803E+01_JPRB, 0.18053E+01_JPRB, 0.20171E+01_JPRB, 0.26316E+01_JPRB, 0.20659E+01_JPRB, 0.17714E+01_JPRB ,&
&  0.19046E+01_JPRB, 0.16061E+01_JPRB, 0.13343E+01_JPRB, 0.11400E+01_JPRB, 0.14540E+01_JPRB, 0.14726E+01_JPRB, 0.73622E+00_JPRB ,&
&  0.58567E+00_JPRB, 0.55685E+00_JPRB, 0.53108E+00_JPRB, 0.50391E+00_JPRB, 0.50836E+00_JPRB, 0.61715E+00_JPRB, 0.86074E+00_JPRB ,&
&  0.97436E+00_JPRB, 0.11949E+01_JPRB, 0.12595E+01_JPRB, 0.12420E+01_JPRB, 0.11093E+01_JPRB, 0.12925E+01_JPRB, 0.13909E+01_JPRB ,&
&  0.15255E+01_JPRB, 0.14612E+01_JPRB, 0.13226E+01_JPRB, 0.11934E+01_JPRB, 0.10706E+01_JPRB, 0.97339E+00_JPRB, 0.89825E+00_JPRB ,&
&  0.83794E+00_JPRB, 0.80029E+00_JPRB, 0.77392E+00_JPRB, 0.75263E+00_JPRB, 0.72693E+00_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.71048E+00_JPRB, 0.69063E+00_JPRB, 0.67289E+00_JPRB, 0.65591E+00_JPRB, 0.65092E+00_JPRB, 0.64304E+00_JPRB, 0.62635E+00_JPRB ,&
&  0.65679E+00_JPRB, 0.60994E+00_JPRB, 0.56520E+00_JPRB, 0.53540E+00_JPRB, 0.52530E+00_JPRB, 0.52668E+00_JPRB, 0.53602E+00_JPRB ,&
&  0.55074E+00_JPRB, 0.56171E+00_JPRB, 0.60070E+00_JPRB, 0.68975E+00_JPRB, 0.72113E+00_JPRB, 0.67233E+00_JPRB, 0.71337E+00_JPRB ,&
&  0.71667E+00_JPRB, 0.78047E+00_JPRB, 0.88891E+00_JPRB, 0.92695E+00_JPRB, 0.96917E+00_JPRB, 0.10127E+01_JPRB, 0.96466E+00_JPRB ,&
&  0.91246E+00_JPRB, 0.89849E+00_JPRB, 0.82950E+00_JPRB, 0.79322E+00_JPRB, 0.76742E+00_JPRB, 0.75977E+00_JPRB, 0.74629E+00_JPRB ,&
&  0.75282E+00_JPRB, 0.72465E+00_JPRB, 0.70819E+00_JPRB, 0.70446E+00_JPRB, 0.70391E+00_JPRB, 0.70033E+00_JPRB, 0.70643E+00_JPRB ,&
&  0.73891E+00_JPRB, 0.77277E+00_JPRB, 0.82242E+00_JPRB, 0.84877E+00_JPRB, 0.71197E+00_JPRB, 0.81231E+00_JPRB, 0.85088E+00_JPRB ,&
&  0.89532E+00_JPRB, 0.95588E+00_JPRB, 0.95899E+00_JPRB, 0.10891E+01_JPRB, 0.11724E+01_JPRB, 0.11516E+01_JPRB, 0.12478E+01_JPRB ,&
&  0.16613E+01_JPRB, 0.16807E+01_JPRB, 0.16746E+01_JPRB, 0.19504E+01_JPRB, 0.21880E+01_JPRB, 0.28921E+01_JPRB, 0.15763E+01_JPRB ,&
&  0.12514E+01_JPRB, 0.14620E+01_JPRB, 0.13645E+01_JPRB, 0.12738E+01_JPRB, 0.15140E+01_JPRB, 0.18102E+01_JPRB, 0.15201E+01_JPRB ,&
&  0.70521E+00_JPRB, 0.58048E+00_JPRB, 0.63093E+00_JPRB, 0.63485E+00_JPRB, 0.59523E+00_JPRB, 0.72993E+00_JPRB, 0.12944E+01_JPRB ,&
&  0.12395E+01_JPRB, 0.13836E+01_JPRB, 0.13581E+01_JPRB, 0.12126E+01_JPRB, 0.11293E+01_JPRB, 0.12642E+01_JPRB, 0.16220E+01_JPRB ,&
&  0.14911E+01_JPRB, 0.13103E+01_JPRB, 0.11924E+01_JPRB, 0.11032E+01_JPRB, 0.10000E+01_JPRB, 0.91025E+00_JPRB, 0.84037E+00_JPRB ,&
&  0.79393E+00_JPRB, 0.76952E+00_JPRB, 0.75707E+00_JPRB, 0.74476E+00_JPRB, 0.72913E+00_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.68850E+00_JPRB, 0.67521E+00_JPRB, 0.66750E+00_JPRB, 0.66427E+00_JPRB, 0.67343E+00_JPRB, 0.66555E+00_JPRB, 0.65733E+00_JPRB ,&
&  0.69147E+00_JPRB, 0.60626E+00_JPRB, 0.56221E+00_JPRB, 0.53007E+00_JPRB, 0.52533E+00_JPRB, 0.53421E+00_JPRB, 0.53984E+00_JPRB ,&
&  0.54516E+00_JPRB, 0.56698E+00_JPRB, 0.62923E+00_JPRB, 0.75869E+00_JPRB, 0.83443E+00_JPRB, 0.79428E+00_JPRB, 0.80931E+00_JPRB ,&
&  0.87825E+00_JPRB, 0.92173E+00_JPRB, 0.93296E+00_JPRB, 0.96492E+00_JPRB, 0.10577E+01_JPRB, 0.10607E+01_JPRB, 0.98653E+00_JPRB ,&
&  0.91696E+00_JPRB, 0.90636E+00_JPRB, 0.86164E+00_JPRB, 0.83303E+00_JPRB, 0.82610E+00_JPRB, 0.81764E+00_JPRB, 0.81374E+00_JPRB ,&
&  0.77595E+00_JPRB, 0.76067E+00_JPRB, 0.75641E+00_JPRB, 0.75288E+00_JPRB, 0.76363E+00_JPRB, 0.75583E+00_JPRB, 0.78073E+00_JPRB ,&
&  0.82476E+00_JPRB, 0.85524E+00_JPRB, 0.88666E+00_JPRB, 0.84307E+00_JPRB, 0.85039E+00_JPRB, 0.86335E+00_JPRB, 0.90712E+00_JPRB ,&
&  0.92107E+00_JPRB, 0.96229E+00_JPRB, 0.10234E+01_JPRB, 0.10830E+01_JPRB, 0.11866E+01_JPRB, 0.12635E+01_JPRB, 0.13934E+01_JPRB ,&
&  0.16490E+01_JPRB, 0.17360E+01_JPRB, 0.16661E+01_JPRB, 0.17745E+01_JPRB, 0.20115E+01_JPRB, 0.24708E+01_JPRB, 0.19614E+01_JPRB ,&
&  0.14938E+01_JPRB, 0.12564E+01_JPRB, 0.12908E+01_JPRB, 0.15000E+01_JPRB, 0.16840E+01_JPRB, 0.18550E+01_JPRB, 0.22304E+01_JPRB ,&
&  0.23724E+01_JPRB, 0.14294E+01_JPRB, 0.11878E+01_JPRB, 0.13603E+01_JPRB, 0.10997E+01_JPRB, 0.97929E+00_JPRB, 0.11984E+01_JPRB ,&
&  0.13442E+01_JPRB, 0.14081E+01_JPRB, 0.13630E+01_JPRB, 0.11685E+01_JPRB, 0.11848E+01_JPRB, 0.13961E+01_JPRB, 0.15430E+01_JPRB ,&
&  0.12951E+01_JPRB, 0.11776E+01_JPRB, 0.10674E+01_JPRB, 0.95877E+00_JPRB, 0.87329E+00_JPRB, 0.81568E+00_JPRB, 0.78061E+00_JPRB ,&
&  0.76647E+00_JPRB, 0.75646E+00_JPRB, 0.74452E+00_JPRB, 0.72622E+00_JPRB, 0.70631E+00_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.67924E+00_JPRB, 0.68447E+00_JPRB, 0.69714E+00_JPRB, 0.70319E+00_JPRB, 0.69976E+00_JPRB, 0.68222E+00_JPRB, 0.70117E+00_JPRB ,&
&  0.72449E+00_JPRB, 0.61752E+00_JPRB, 0.56296E+00_JPRB, 0.54059E+00_JPRB, 0.54282E+00_JPRB, 0.55643E+00_JPRB, 0.56262E+00_JPRB ,&
&  0.57473E+00_JPRB, 0.61709E+00_JPRB, 0.74738E+00_JPRB, 0.85843E+00_JPRB, 0.97647E+00_JPRB, 0.11429E+01_JPRB, 0.86197E+00_JPRB ,&
&  0.89659E+00_JPRB, 0.97232E+00_JPRB, 0.97195E+00_JPRB, 0.10515E+01_JPRB, 0.11063E+01_JPRB, 0.10851E+01_JPRB, 0.10155E+01_JPRB ,&
&  0.95021E+00_JPRB, 0.95762E+00_JPRB, 0.93318E+00_JPRB, 0.90914E+00_JPRB, 0.90700E+00_JPRB, 0.91056E+00_JPRB, 0.86891E+00_JPRB ,&
&  0.85927E+00_JPRB, 0.84696E+00_JPRB, 0.82378E+00_JPRB, 0.83244E+00_JPRB, 0.84140E+00_JPRB, 0.84079E+00_JPRB, 0.87684E+00_JPRB ,&
&  0.91097E+00_JPRB, 0.92922E+00_JPRB, 0.93368E+00_JPRB, 0.89260E+00_JPRB, 0.92038E+00_JPRB, 0.94538E+00_JPRB, 0.98491E+00_JPRB ,&
&  0.95209E+00_JPRB, 0.90132E+00_JPRB, 0.10267E+01_JPRB, 0.11826E+01_JPRB, 0.12954E+01_JPRB, 0.13182E+01_JPRB, 0.14410E+01_JPRB ,&
&  0.17380E+01_JPRB, 0.19515E+01_JPRB, 0.20169E+01_JPRB, 0.16907E+01_JPRB, 0.18429E+01_JPRB, 0.21710E+01_JPRB, 0.22119E+01_JPRB ,&
&  0.19970E+01_JPRB, 0.16229E+01_JPRB, 0.16071E+01_JPRB, 0.16402E+01_JPRB, 0.16831E+01_JPRB, 0.18194E+01_JPRB, 0.18459E+01_JPRB ,&
&  0.20828E+01_JPRB, 0.24582E+01_JPRB, 0.21410E+01_JPRB, 0.17478E+01_JPRB, 0.15137E+01_JPRB, 0.13271E+01_JPRB, 0.13203E+01_JPRB ,&
&  0.14626E+01_JPRB, 0.13956E+01_JPRB, 0.12884E+01_JPRB, 0.13301E+01_JPRB, 0.12642E+01_JPRB, 0.13617E+01_JPRB, 0.13188E+01_JPRB ,&
&  0.11014E+01_JPRB, 0.97687E+00_JPRB, 0.85441E+00_JPRB, 0.78975E+00_JPRB, 0.75301E+00_JPRB, 0.74523E+00_JPRB, 0.73798E+00_JPRB ,&
&  0.73438E+00_JPRB, 0.72486E+00_JPRB, 0.71126E+00_JPRB, 0.70127E+00_JPRB, 0.68753E+00_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.70455E+00_JPRB, 0.71499E+00_JPRB, 0.73643E+00_JPRB, 0.74403E+00_JPRB, 0.73168E+00_JPRB, 0.73802E+00_JPRB, 0.76496E+00_JPRB ,&
&  0.76377E+00_JPRB, 0.61695E+00_JPRB, 0.59215E+00_JPRB, 0.59296E+00_JPRB, 0.60641E+00_JPRB, 0.61039E+00_JPRB, 0.61173E+00_JPRB ,&
&  0.64458E+00_JPRB, 0.73805E+00_JPRB, 0.88067E+00_JPRB, 0.96312E+00_JPRB, 0.10409E+01_JPRB, 0.11433E+01_JPRB, 0.10839E+01_JPRB ,&
&  0.87738E+00_JPRB, 0.11054E+01_JPRB, 0.10521E+01_JPRB, 0.11734E+01_JPRB, 0.11663E+01_JPRB, 0.11831E+01_JPRB, 0.11152E+01_JPRB ,&
&  0.10544E+01_JPRB, 0.10561E+01_JPRB, 0.10860E+01_JPRB, 0.10428E+01_JPRB, 0.10200E+01_JPRB, 0.95947E+00_JPRB, 0.95972E+00_JPRB ,&
&  0.95460E+00_JPRB, 0.93759E+00_JPRB, 0.92931E+00_JPRB, 0.94613E+00_JPRB, 0.95484E+00_JPRB, 0.95346E+00_JPRB, 0.95996E+00_JPRB ,&
&  0.97062E+00_JPRB, 0.97590E+00_JPRB, 0.95859E+00_JPRB, 0.97287E+00_JPRB, 0.99080E+00_JPRB, 0.10400E+01_JPRB, 0.10843E+01_JPRB ,&
&  0.10675E+01_JPRB, 0.10307E+01_JPRB, 0.11158E+01_JPRB, 0.12038E+01_JPRB, 0.12783E+01_JPRB, 0.14311E+01_JPRB, 0.15188E+01_JPRB ,&
&  0.18125E+01_JPRB, 0.18330E+01_JPRB, 0.21115E+01_JPRB, 0.15799E+01_JPRB, 0.16608E+01_JPRB, 0.19182E+01_JPRB, 0.18406E+01_JPRB ,&
&  0.15032E+01_JPRB, 0.14977E+01_JPRB, 0.15165E+01_JPRB, 0.15908E+01_JPRB, 0.16451E+01_JPRB, 0.17071E+01_JPRB, 0.16254E+01_JPRB ,&
&  0.16452E+01_JPRB, 0.17492E+01_JPRB, 0.17210E+01_JPRB, 0.14781E+01_JPRB, 0.13783E+01_JPRB, 0.12595E+01_JPRB, 0.13677E+01_JPRB ,&
&  0.14868E+01_JPRB, 0.14627E+01_JPRB, 0.14697E+01_JPRB, 0.14315E+01_JPRB, 0.12087E+01_JPRB, 0.11565E+01_JPRB, 0.10269E+01_JPRB ,&
&  0.88327E+00_JPRB, 0.80570E+00_JPRB, 0.72798E+00_JPRB, 0.71947E+00_JPRB, 0.72117E+00_JPRB, 0.71575E+00_JPRB, 0.72379E+00_JPRB ,&
&  0.71844E+00_JPRB, 0.71033E+00_JPRB, 0.71094E+00_JPRB, 0.70741E+00_JPRB, 0.69974E+00_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.74150E+00_JPRB, 0.75281E+00_JPRB, 0.77637E+00_JPRB, 0.80516E+00_JPRB, 0.83631E+00_JPRB, 0.88599E+00_JPRB, 0.85005E+00_JPRB ,&
&  0.81428E+00_JPRB, 0.64562E+00_JPRB, 0.65571E+00_JPRB, 0.66337E+00_JPRB, 0.67502E+00_JPRB, 0.67058E+00_JPRB, 0.68836E+00_JPRB ,&
&  0.76137E+00_JPRB, 0.87292E+00_JPRB, 0.97467E+00_JPRB, 0.10664E+01_JPRB, 0.11534E+01_JPRB, 0.12662E+01_JPRB, 0.12458E+01_JPRB ,&
&  0.10089E+01_JPRB, 0.10257E+01_JPRB, 0.12071E+01_JPRB, 0.12640E+01_JPRB, 0.12290E+01_JPRB, 0.12615E+01_JPRB, 0.12685E+01_JPRB ,&
&  0.12382E+01_JPRB, 0.12235E+01_JPRB, 0.12707E+01_JPRB, 0.11556E+01_JPRB, 0.10260E+01_JPRB, 0.10244E+01_JPRB, 0.10450E+01_JPRB ,&
&  0.10505E+01_JPRB, 0.10529E+01_JPRB, 0.10598E+01_JPRB, 0.10806E+01_JPRB, 0.10898E+01_JPRB, 0.10733E+01_JPRB, 0.10534E+01_JPRB ,&
&  0.10498E+01_JPRB, 0.10578E+01_JPRB, 0.10565E+01_JPRB, 0.10475E+01_JPRB, 0.10863E+01_JPRB, 0.11362E+01_JPRB, 0.11747E+01_JPRB ,&
&  0.12018E+01_JPRB, 0.12492E+01_JPRB, 0.12976E+01_JPRB, 0.13572E+01_JPRB, 0.13531E+01_JPRB, 0.15499E+01_JPRB, 0.16437E+01_JPRB ,&
&  0.17792E+01_JPRB, 0.17942E+01_JPRB, 0.19424E+01_JPRB, 0.14362E+01_JPRB, 0.16037E+01_JPRB, 0.16944E+01_JPRB, 0.16211E+01_JPRB ,&
&  0.14394E+01_JPRB, 0.14673E+01_JPRB, 0.14614E+01_JPRB, 0.14810E+01_JPRB, 0.15412E+01_JPRB, 0.14740E+01_JPRB, 0.14930E+01_JPRB ,&
&  0.14242E+01_JPRB, 0.15516E+01_JPRB, 0.14960E+01_JPRB, 0.14155E+01_JPRB, 0.12737E+01_JPRB, 0.12279E+01_JPRB, 0.12937E+01_JPRB ,&
&  0.14131E+01_JPRB, 0.14100E+01_JPRB, 0.13594E+01_JPRB, 0.12300E+01_JPRB, 0.10262E+01_JPRB, 0.93746E+00_JPRB, 0.82967E+00_JPRB ,&
&  0.79629E+00_JPRB, 0.74522E+00_JPRB, 0.72657E+00_JPRB, 0.73071E+00_JPRB, 0.72501E+00_JPRB, 0.74627E+00_JPRB, 0.74242E+00_JPRB ,&
&  0.72889E+00_JPRB, 0.73301E+00_JPRB, 0.73739E+00_JPRB, 0.73673E+00_JPRB, 0.73673E+00_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.73263E+00_JPRB, 0.72749E+00_JPRB, 0.72707E+00_JPRB, 0.73704E+00_JPRB, 0.75769E+00_JPRB, 0.77211E+00_JPRB, 0.72631E+00_JPRB ,&
&  0.70895E+00_JPRB, 0.70094E+00_JPRB, 0.72678E+00_JPRB, 0.73709E+00_JPRB, 0.74240E+00_JPRB, 0.74787E+00_JPRB, 0.78750E+00_JPRB ,&
&  0.85798E+00_JPRB, 0.96166E+00_JPRB, 0.10605E+01_JPRB, 0.11191E+01_JPRB, 0.11745E+01_JPRB, 0.12580E+01_JPRB, 0.13525E+01_JPRB ,&
&  0.12195E+01_JPRB, 0.10461E+01_JPRB, 0.12536E+01_JPRB, 0.13484E+01_JPRB, 0.12789E+01_JPRB, 0.13379E+01_JPRB, 0.14898E+01_JPRB ,&
&  0.13078E+01_JPRB, 0.12624E+01_JPRB, 0.12651E+01_JPRB, 0.11973E+01_JPRB, 0.10957E+01_JPRB, 0.11105E+01_JPRB, 0.11379E+01_JPRB ,&
&  0.11560E+01_JPRB, 0.11634E+01_JPRB, 0.11945E+01_JPRB, 0.12236E+01_JPRB, 0.12358E+01_JPRB, 0.12162E+01_JPRB, 0.11873E+01_JPRB ,&
&  0.11941E+01_JPRB, 0.12106E+01_JPRB, 0.12357E+01_JPRB, 0.12354E+01_JPRB, 0.12586E+01_JPRB, 0.13099E+01_JPRB, 0.13571E+01_JPRB ,&
&  0.13985E+01_JPRB, 0.14039E+01_JPRB, 0.14693E+01_JPRB, 0.15978E+01_JPRB, 0.17103E+01_JPRB, 0.15637E+01_JPRB, 0.16837E+01_JPRB ,&
&  0.16529E+01_JPRB, 0.15872E+01_JPRB, 0.15097E+01_JPRB, 0.17248E+01_JPRB, 0.12519E+01_JPRB, 0.14099E+01_JPRB, 0.13133E+01_JPRB ,&
&  0.14600E+01_JPRB, 0.14218E+01_JPRB, 0.13989E+01_JPRB, 0.14297E+01_JPRB, 0.14701E+01_JPRB, 0.13692E+01_JPRB, 0.13607E+01_JPRB ,&
&  0.13874E+01_JPRB, 0.14095E+01_JPRB, 0.13953E+01_JPRB, 0.13904E+01_JPRB, 0.13076E+01_JPRB, 0.12991E+01_JPRB, 0.12960E+01_JPRB ,&
&  0.13480E+01_JPRB, 0.12597E+01_JPRB, 0.11369E+01_JPRB, 0.10021E+01_JPRB, 0.85979E+00_JPRB, 0.81010E+00_JPRB, 0.80434E+00_JPRB ,&
&  0.81376E+00_JPRB, 0.82531E+00_JPRB, 0.77840E+00_JPRB, 0.75294E+00_JPRB, 0.75202E+00_JPRB, 0.77377E+00_JPRB, 0.76225E+00_JPRB ,&
&  0.76284E+00_JPRB, 0.75662E+00_JPRB, 0.75926E+00_JPRB, 0.75434E+00_JPRB, 0.74718E+00_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.73252E+00_JPRB, 0.72493E+00_JPRB, 0.71803E+00_JPRB, 0.72894E+00_JPRB, 0.73526E+00_JPRB, 0.73139E+00_JPRB, 0.71886E+00_JPRB ,&
&  0.72138E+00_JPRB, 0.74491E+00_JPRB, 0.77387E+00_JPRB, 0.78767E+00_JPRB, 0.79063E+00_JPRB, 0.79848E+00_JPRB, 0.83890E+00_JPRB ,&
&  0.90586E+00_JPRB, 0.10081E+01_JPRB, 0.11323E+01_JPRB, 0.12200E+01_JPRB, 0.12961E+01_JPRB, 0.13448E+01_JPRB, 0.14460E+01_JPRB ,&
&  0.14434E+01_JPRB, 0.14728E+01_JPRB, 0.14365E+01_JPRB, 0.11953E+01_JPRB, 0.11049E+01_JPRB, 0.13399E+01_JPRB, 0.13460E+01_JPRB ,&
&  0.12898E+01_JPRB, 0.12574E+01_JPRB, 0.12291E+01_JPRB, 0.11989E+01_JPRB, 0.11806E+01_JPRB, 0.12136E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12588E+01_JPRB, 0.12924E+01_JPRB, 0.13278E+01_JPRB, 0.13543E+01_JPRB, 0.13719E+01_JPRB, 0.13947E+01_JPRB, 0.13962E+01_JPRB ,&
&  0.14405E+01_JPRB, 0.14795E+01_JPRB, 0.15265E+01_JPRB, 0.15429E+01_JPRB, 0.15226E+01_JPRB, 0.15639E+01_JPRB, 0.15935E+01_JPRB ,&
&  0.16265E+01_JPRB, 0.16163E+01_JPRB, 0.16013E+01_JPRB, 0.16551E+01_JPRB, 0.16717E+01_JPRB, 0.14607E+01_JPRB, 0.14444E+01_JPRB ,&
&  0.15062E+01_JPRB, 0.14562E+01_JPRB, 0.12952E+01_JPRB, 0.15759E+01_JPRB, 0.10447E+01_JPRB, 0.12678E+01_JPRB, 0.14223E+01_JPRB ,&
&  0.13916E+01_JPRB, 0.13679E+01_JPRB, 0.13696E+01_JPRB, 0.13835E+01_JPRB, 0.13778E+01_JPRB, 0.12899E+01_JPRB, 0.13409E+01_JPRB ,&
&  0.13889E+01_JPRB, 0.13591E+01_JPRB, 0.13322E+01_JPRB, 0.13186E+01_JPRB, 0.12870E+01_JPRB, 0.12056E+01_JPRB, 0.10911E+01_JPRB ,&
&  0.96296E+00_JPRB, 0.11216E+01_JPRB, 0.10336E+01_JPRB, 0.98160E+00_JPRB, 0.89582E+00_JPRB, 0.78145E+00_JPRB, 0.81217E+00_JPRB ,&
&  0.83383E+00_JPRB, 0.81462E+00_JPRB, 0.78654E+00_JPRB, 0.78548E+00_JPRB, 0.78370E+00_JPRB, 0.78638E+00_JPRB, 0.77411E+00_JPRB ,&
&  0.77422E+00_JPRB, 0.77318E+00_JPRB, 0.76970E+00_JPRB, 0.75806E+00_JPRB, 0.74519E+00_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.71195E+00_JPRB, 0.70611E+00_JPRB, 0.70231E+00_JPRB, 0.73110E+00_JPRB, 0.72636E+00_JPRB, 0.71362E+00_JPRB, 0.72011E+00_JPRB ,&
&  0.73043E+00_JPRB, 0.74528E+00_JPRB, 0.76526E+00_JPRB, 0.78564E+00_JPRB, 0.81506E+00_JPRB, 0.86221E+00_JPRB, 0.91414E+00_JPRB ,&
&  0.97443E+00_JPRB, 0.10448E+01_JPRB, 0.11608E+01_JPRB, 0.13084E+01_JPRB, 0.14918E+01_JPRB, 0.15833E+01_JPRB, 0.15754E+01_JPRB ,&
&  0.14625E+01_JPRB, 0.15065E+01_JPRB, 0.15800E+01_JPRB, 0.16222E+01_JPRB, 0.13998E+01_JPRB, 0.14607E+01_JPRB, 0.15712E+01_JPRB ,&
&  0.15569E+01_JPRB, 0.14056E+01_JPRB, 0.13018E+01_JPRB, 0.12767E+01_JPRB, 0.12623E+01_JPRB, 0.13019E+01_JPRB, 0.13436E+01_JPRB ,&
&  0.13716E+01_JPRB, 0.14133E+01_JPRB, 0.14429E+01_JPRB, 0.14760E+01_JPRB, 0.15286E+01_JPRB, 0.16105E+01_JPRB, 0.16483E+01_JPRB ,&
&  0.16781E+01_JPRB, 0.17459E+01_JPRB, 0.17694E+01_JPRB, 0.17270E+01_JPRB, 0.17670E+01_JPRB, 0.17888E+01_JPRB, 0.18032E+01_JPRB ,&
&  0.18431E+01_JPRB, 0.17553E+01_JPRB, 0.17716E+01_JPRB, 0.18005E+01_JPRB, 0.17114E+01_JPRB, 0.15238E+01_JPRB, 0.14662E+01_JPRB ,&
&  0.14833E+01_JPRB, 0.14661E+01_JPRB, 0.10936E+01_JPRB, 0.14222E+01_JPRB, 0.16650E+01_JPRB, 0.14558E+01_JPRB, 0.14209E+01_JPRB ,&
&  0.13514E+01_JPRB, 0.13525E+01_JPRB, 0.13799E+01_JPRB, 0.14016E+01_JPRB, 0.14095E+01_JPRB, 0.14099E+01_JPRB, 0.13402E+01_JPRB ,&
&  0.14650E+01_JPRB, 0.13808E+01_JPRB, 0.12906E+01_JPRB, 0.12330E+01_JPRB, 0.11566E+01_JPRB, 0.10305E+01_JPRB, 0.91298E+00_JPRB ,&
&  0.85147E+00_JPRB, 0.10783E+01_JPRB, 0.10585E+01_JPRB, 0.11484E+01_JPRB, 0.11662E+01_JPRB, 0.79889E+00_JPRB, 0.83705E+00_JPRB ,&
&  0.83486E+00_JPRB, 0.82631E+00_JPRB, 0.80375E+00_JPRB, 0.77613E+00_JPRB, 0.77288E+00_JPRB, 0.77552E+00_JPRB, 0.77526E+00_JPRB ,&
&  0.77042E+00_JPRB, 0.75764E+00_JPRB, 0.75009E+00_JPRB, 0.73160E+00_JPRB, 0.72122E+00_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.72900E+00_JPRB, 0.73108E+00_JPRB, 0.72808E+00_JPRB, 0.74136E+00_JPRB, 0.73596E+00_JPRB, 0.74897E+00_JPRB, 0.73432E+00_JPRB ,&
&  0.71931E+00_JPRB, 0.71554E+00_JPRB, 0.71917E+00_JPRB, 0.72331E+00_JPRB, 0.74427E+00_JPRB, 0.79042E+00_JPRB, 0.84907E+00_JPRB ,&
&  0.90715E+00_JPRB, 0.92368E+00_JPRB, 0.96684E+00_JPRB, 0.10740E+01_JPRB, 0.12918E+01_JPRB, 0.15005E+01_JPRB, 0.15388E+01_JPRB ,&
&  0.16224E+01_JPRB, 0.17563E+01_JPRB, 0.18708E+01_JPRB, 0.19781E+01_JPRB, 0.19525E+01_JPRB, 0.19010E+01_JPRB, 0.18883E+01_JPRB ,&
&  0.16277E+01_JPRB, 0.13389E+01_JPRB, 0.12999E+01_JPRB, 0.13169E+01_JPRB, 0.13511E+01_JPRB, 0.13753E+01_JPRB, 0.13939E+01_JPRB ,&
&  0.14212E+01_JPRB, 0.14662E+01_JPRB, 0.15070E+01_JPRB, 0.15717E+01_JPRB, 0.16671E+01_JPRB, 0.17791E+01_JPRB, 0.18434E+01_JPRB ,&
&  0.18703E+01_JPRB, 0.19175E+01_JPRB, 0.19915E+01_JPRB, 0.19011E+01_JPRB, 0.20925E+01_JPRB, 0.21611E+01_JPRB, 0.21844E+01_JPRB ,&
&  0.21463E+01_JPRB, 0.20206E+01_JPRB, 0.20167E+01_JPRB, 0.19971E+01_JPRB, 0.19646E+01_JPRB, 0.17747E+01_JPRB, 0.16436E+01_JPRB ,&
&  0.16154E+01_JPRB, 0.13947E+01_JPRB, 0.10368E+01_JPRB, 0.11173E+01_JPRB, 0.12393E+01_JPRB, 0.11600E+01_JPRB, 0.14436E+01_JPRB ,&
&  0.13865E+01_JPRB, 0.13455E+01_JPRB, 0.13364E+01_JPRB, 0.13389E+01_JPRB, 0.13953E+01_JPRB, 0.13843E+01_JPRB, 0.13671E+01_JPRB ,&
&  0.14096E+01_JPRB, 0.12826E+01_JPRB, 0.11681E+01_JPRB, 0.10668E+01_JPRB, 0.96328E+00_JPRB, 0.88265E+00_JPRB, 0.84592E+00_JPRB ,&
&  0.85870E+00_JPRB, 0.87387E+00_JPRB, 0.81827E+00_JPRB, 0.79346E+00_JPRB, 0.73278E+00_JPRB, 0.78221E+00_JPRB, 0.78713E+00_JPRB ,&
&  0.81321E+00_JPRB, 0.80759E+00_JPRB, 0.79122E+00_JPRB, 0.76840E+00_JPRB, 0.75524E+00_JPRB, 0.74650E+00_JPRB, 0.74411E+00_JPRB ,&
&  0.75169E+00_JPRB, 0.76027E+00_JPRB, 0.75393E+00_JPRB, 0.74571E+00_JPRB, 0.73572E+00_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.74312E+00_JPRB, 0.73938E+00_JPRB, 0.73854E+00_JPRB, 0.75382E+00_JPRB, 0.76684E+00_JPRB, 0.75752E+00_JPRB, 0.76515E+00_JPRB ,&
&  0.74404E+00_JPRB, 0.72733E+00_JPRB, 0.73758E+00_JPRB, 0.74305E+00_JPRB, 0.75850E+00_JPRB, 0.77133E+00_JPRB, 0.77876E+00_JPRB ,&
&  0.77595E+00_JPRB, 0.76911E+00_JPRB, 0.73953E+00_JPRB, 0.75151E+00_JPRB, 0.83350E+00_JPRB, 0.96827E+00_JPRB, 0.12223E+01_JPRB ,&
&  0.15024E+01_JPRB, 0.16411E+01_JPRB, 0.17452E+01_JPRB, 0.17610E+01_JPRB, 0.16613E+01_JPRB, 0.15161E+01_JPRB, 0.19464E+01_JPRB ,&
&  0.18170E+01_JPRB, 0.13233E+01_JPRB, 0.13722E+01_JPRB, 0.12921E+01_JPRB, 0.13251E+01_JPRB, 0.13280E+01_JPRB, 0.13510E+01_JPRB ,&
&  0.13614E+01_JPRB, 0.14084E+01_JPRB, 0.14833E+01_JPRB, 0.15896E+01_JPRB, 0.16989E+01_JPRB, 0.18155E+01_JPRB, 0.19037E+01_JPRB ,&
&  0.19511E+01_JPRB, 0.20016E+01_JPRB, 0.21162E+01_JPRB, 0.21364E+01_JPRB, 0.21432E+01_JPRB, 0.22500E+01_JPRB, 0.23587E+01_JPRB ,&
&  0.24341E+01_JPRB, 0.25502E+01_JPRB, 0.22239E+01_JPRB, 0.21838E+01_JPRB, 0.22492E+01_JPRB, 0.19185E+01_JPRB, 0.18103E+01_JPRB ,&
&  0.17552E+01_JPRB, 0.13881E+01_JPRB, 0.95781E+00_JPRB, 0.10219E+01_JPRB, 0.11175E+01_JPRB, 0.12118E+01_JPRB, 0.13202E+01_JPRB ,&
&  0.12770E+01_JPRB, 0.12169E+01_JPRB, 0.12189E+01_JPRB, 0.12824E+01_JPRB, 0.13002E+01_JPRB, 0.12534E+01_JPRB, 0.12057E+01_JPRB ,&
&  0.11783E+01_JPRB, 0.10566E+01_JPRB, 0.99377E+00_JPRB, 0.90484E+00_JPRB, 0.84323E+00_JPRB, 0.81067E+00_JPRB, 0.77137E+00_JPRB ,&
&  0.80450E+00_JPRB, 0.82055E+00_JPRB, 0.88158E+00_JPRB, 0.84174E+00_JPRB, 0.69735E+00_JPRB, 0.77682E+00_JPRB, 0.78206E+00_JPRB ,&
&  0.78737E+00_JPRB, 0.79085E+00_JPRB, 0.80379E+00_JPRB, 0.79375E+00_JPRB, 0.77390E+00_JPRB, 0.75472E+00_JPRB, 0.73711E+00_JPRB ,&
&  0.74899E+00_JPRB, 0.76292E+00_JPRB, 0.76887E+00_JPRB, 0.75554E+00_JPRB, 0.74476E+00_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.81252E+00_JPRB, 0.81434E+00_JPRB, 0.82945E+00_JPRB, 0.83122E+00_JPRB, 0.83105E+00_JPRB, 0.83872E+00_JPRB, 0.84106E+00_JPRB ,&
&  0.83521E+00_JPRB, 0.84267E+00_JPRB, 0.85294E+00_JPRB, 0.86436E+00_JPRB, 0.87846E+00_JPRB, 0.89076E+00_JPRB, 0.89737E+00_JPRB ,&
&  0.88790E+00_JPRB, 0.87837E+00_JPRB, 0.90078E+00_JPRB, 0.91930E+00_JPRB, 0.91775E+00_JPRB, 0.91299E+00_JPRB, 0.93193E+00_JPRB ,&
&  0.98064E+00_JPRB, 0.10104E+01_JPRB, 0.10196E+01_JPRB, 0.10069E+01_JPRB, 0.10387E+01_JPRB, 0.12443E+01_JPRB, 0.19386E+01_JPRB ,&
&  0.18637E+01_JPRB, 0.13395E+01_JPRB, 0.12408E+01_JPRB, 0.11291E+01_JPRB, 0.11508E+01_JPRB, 0.12147E+01_JPRB, 0.12170E+01_JPRB ,&
&  0.11935E+01_JPRB, 0.12189E+01_JPRB, 0.12478E+01_JPRB, 0.12748E+01_JPRB, 0.13237E+01_JPRB, 0.13641E+01_JPRB, 0.13848E+01_JPRB ,&
&  0.13564E+01_JPRB, 0.13548E+01_JPRB, 0.13966E+01_JPRB, 0.14744E+01_JPRB, 0.16615E+01_JPRB, 0.19769E+01_JPRB, 0.19230E+01_JPRB ,&
&  0.19239E+01_JPRB, 0.20401E+01_JPRB, 0.21227E+01_JPRB, 0.21572E+01_JPRB, 0.20616E+01_JPRB, 0.18514E+01_JPRB, 0.16710E+01_JPRB ,&
&  0.14914E+01_JPRB, 0.12304E+01_JPRB, 0.10178E+01_JPRB, 0.11458E+01_JPRB, 0.12020E+01_JPRB, 0.12655E+01_JPRB, 0.11893E+01_JPRB ,&
&  0.11354E+01_JPRB, 0.10340E+01_JPRB, 0.10236E+01_JPRB, 0.10769E+01_JPRB, 0.11360E+01_JPRB, 0.11200E+01_JPRB, 0.10672E+01_JPRB ,&
&  0.98107E+00_JPRB, 0.93546E+00_JPRB, 0.90787E+00_JPRB, 0.83988E+00_JPRB, 0.80850E+00_JPRB, 0.69987E+00_JPRB, 0.74766E+00_JPRB ,&
&  0.75280E+00_JPRB, 0.76844E+00_JPRB, 0.63787E+00_JPRB, 0.81970E+00_JPRB, 0.73043E+00_JPRB, 0.72312E+00_JPRB, 0.74071E+00_JPRB ,&
&  0.76086E+00_JPRB, 0.78853E+00_JPRB, 0.79406E+00_JPRB, 0.87180E+00_JPRB, 0.85650E+00_JPRB, 0.84361E+00_JPRB, 0.84509E+00_JPRB ,&
&  0.81286E+00_JPRB, 0.80384E+00_JPRB, 0.80179E+00_JPRB, 0.80986E+00_JPRB, 0.81134E+00_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.87534E+00_JPRB, 0.87750E+00_JPRB, 0.88269E+00_JPRB, 0.89215E+00_JPRB, 0.90190E+00_JPRB, 0.91213E+00_JPRB, 0.91674E+00_JPRB ,&
&  0.91233E+00_JPRB, 0.91507E+00_JPRB, 0.92465E+00_JPRB, 0.93988E+00_JPRB, 0.95369E+00_JPRB, 0.96564E+00_JPRB, 0.97903E+00_JPRB ,&
&  0.97735E+00_JPRB, 0.97938E+00_JPRB, 0.99625E+00_JPRB, 0.10162E+01_JPRB, 0.10269E+01_JPRB, 0.10258E+01_JPRB, 0.10280E+01_JPRB ,&
&  0.10231E+01_JPRB, 0.10329E+01_JPRB, 0.10609E+01_JPRB, 0.10810E+01_JPRB, 0.11075E+01_JPRB, 0.11733E+01_JPRB, 0.13766E+01_JPRB ,&
&  0.13278E+01_JPRB, 0.14684E+01_JPRB, 0.13158E+01_JPRB, 0.11752E+01_JPRB, 0.11684E+01_JPRB, 0.11194E+01_JPRB, 0.10858E+01_JPRB ,&
&  0.10501E+01_JPRB, 0.10283E+01_JPRB, 0.98413E+00_JPRB, 0.99071E+00_JPRB, 0.98530E+00_JPRB, 0.10209E+01_JPRB, 0.10127E+01_JPRB ,&
&  0.10584E+01_JPRB, 0.11033E+01_JPRB, 0.11880E+01_JPRB, 0.12667E+01_JPRB, 0.13786E+01_JPRB, 0.14386E+01_JPRB, 0.13195E+01_JPRB ,&
&  0.12919E+01_JPRB, 0.14068E+01_JPRB, 0.15562E+01_JPRB, 0.16650E+01_JPRB, 0.15706E+01_JPRB, 0.14066E+01_JPRB, 0.12702E+01_JPRB ,&
&  0.11348E+01_JPRB, 0.10539E+01_JPRB, 0.11825E+01_JPRB, 0.11758E+01_JPRB, 0.12107E+01_JPRB, 0.11807E+01_JPRB, 0.10655E+01_JPRB ,&
&  0.95721E+00_JPRB, 0.83672E+00_JPRB, 0.82617E+00_JPRB, 0.82911E+00_JPRB, 0.85738E+00_JPRB, 0.84621E+00_JPRB, 0.83054E+00_JPRB ,&
&  0.77085E+00_JPRB, 0.71238E+00_JPRB, 0.72741E+00_JPRB, 0.73485E+00_JPRB, 0.74047E+00_JPRB, 0.88233E+00_JPRB, 0.90482E+00_JPRB ,&
&  0.68266E+00_JPRB, 0.58256E+00_JPRB, 0.46615E+00_JPRB, 0.64580E+00_JPRB, 0.63130E+00_JPRB, 0.68377E+00_JPRB, 0.76060E+00_JPRB ,&
&  0.86821E+00_JPRB, 0.96087E+00_JPRB, 0.10226E+01_JPRB, 0.98108E+00_JPRB, 0.90435E+00_JPRB, 0.88651E+00_JPRB, 0.87435E+00_JPRB ,&
&  0.87639E+00_JPRB, 0.87717E+00_JPRB, 0.87584E+00_JPRB, 0.87318E+00_JPRB, 0.86718E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.91232E+00_JPRB, 0.92046E+00_JPRB, 0.93283E+00_JPRB, 0.94950E+00_JPRB, 0.96614E+00_JPRB, 0.98144E+00_JPRB, 0.99404E+00_JPRB ,&
&  0.10032E+01_JPRB, 0.10175E+01_JPRB, 0.10398E+01_JPRB, 0.10665E+01_JPRB, 0.10776E+01_JPRB, 0.10942E+01_JPRB, 0.10985E+01_JPRB ,&
&  0.11217E+01_JPRB, 0.11482E+01_JPRB, 0.11597E+01_JPRB, 0.11728E+01_JPRB, 0.11824E+01_JPRB, 0.11973E+01_JPRB, 0.12159E+01_JPRB ,&
&  0.12336E+01_JPRB, 0.12738E+01_JPRB, 0.13328E+01_JPRB, 0.13816E+01_JPRB, 0.14476E+01_JPRB, 0.15606E+01_JPRB, 0.13407E+01_JPRB ,&
&  0.14615E+01_JPRB, 0.14939E+01_JPRB, 0.14089E+01_JPRB, 0.12434E+01_JPRB, 0.11890E+01_JPRB, 0.11437E+01_JPRB, 0.11133E+01_JPRB ,&
&  0.10993E+01_JPRB, 0.10651E+01_JPRB, 0.10847E+01_JPRB, 0.10846E+01_JPRB, 0.10881E+01_JPRB, 0.11521E+01_JPRB, 0.11789E+01_JPRB ,&
&  0.12022E+01_JPRB, 0.12284E+01_JPRB, 0.12659E+01_JPRB, 0.12892E+01_JPRB, 0.13339E+01_JPRB, 0.13467E+01_JPRB, 0.12840E+01_JPRB ,&
&  0.11962E+01_JPRB, 0.11469E+01_JPRB, 0.10888E+01_JPRB, 0.10931E+01_JPRB, 0.10639E+01_JPRB, 0.93539E+00_JPRB, 0.80066E+00_JPRB ,&
&  0.87876E+00_JPRB, 0.87349E+00_JPRB, 0.91812E+00_JPRB, 0.12433E+01_JPRB, 0.12022E+01_JPRB, 0.11384E+01_JPRB, 0.95762E+00_JPRB ,&
&  0.85621E+00_JPRB, 0.72464E+00_JPRB, 0.67291E+00_JPRB, 0.68099E+00_JPRB, 0.67434E+00_JPRB, 0.66821E+00_JPRB, 0.64033E+00_JPRB ,&
&  0.59181E+00_JPRB, 0.55830E+00_JPRB, 0.57847E+00_JPRB, 0.62552E+00_JPRB, 0.71365E+00_JPRB, 0.74456E+00_JPRB, 0.96908E+00_JPRB ,&
&  0.69133E+00_JPRB, 0.44221E+00_JPRB, 0.51117E+00_JPRB, 0.58165E+00_JPRB, 0.58055E+00_JPRB, 0.62567E+00_JPRB, 0.75475E+00_JPRB ,&
&  0.96072E+00_JPRB, 0.10204E+01_JPRB, 0.10377E+01_JPRB, 0.98358E+00_JPRB, 0.97379E+00_JPRB, 0.96910E+00_JPRB, 0.94920E+00_JPRB ,&
&  0.93998E+00_JPRB, 0.93486E+00_JPRB, 0.93010E+00_JPRB, 0.91863E+00_JPRB, 0.91090E+00_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.96041E+00_JPRB, 0.97197E+00_JPRB, 0.99091E+00_JPRB, 0.10180E+01_JPRB, 0.10320E+01_JPRB, 0.10549E+01_JPRB, 0.10645E+01_JPRB ,&
&  0.10763E+01_JPRB, 0.11055E+01_JPRB, 0.11324E+01_JPRB, 0.11620E+01_JPRB, 0.11890E+01_JPRB, 0.12151E+01_JPRB, 0.12402E+01_JPRB ,&
&  0.12611E+01_JPRB, 0.12809E+01_JPRB, 0.12887E+01_JPRB, 0.12837E+01_JPRB, 0.12648E+01_JPRB, 0.12549E+01_JPRB, 0.12345E+01_JPRB ,&
&  0.12170E+01_JPRB, 0.12183E+01_JPRB, 0.12279E+01_JPRB, 0.12343E+01_JPRB, 0.12439E+01_JPRB, 0.13053E+01_JPRB, 0.95716E+00_JPRB ,&
&  0.13902E+01_JPRB, 0.14682E+01_JPRB, 0.14533E+01_JPRB, 0.13291E+01_JPRB, 0.12694E+01_JPRB, 0.12805E+01_JPRB, 0.13233E+01_JPRB ,&
&  0.13951E+01_JPRB, 0.14942E+01_JPRB, 0.15870E+01_JPRB, 0.15104E+01_JPRB, 0.14130E+01_JPRB, 0.14430E+01_JPRB, 0.14748E+01_JPRB ,&
&  0.14998E+01_JPRB, 0.15059E+01_JPRB, 0.15111E+01_JPRB, 0.14943E+01_JPRB, 0.14817E+01_JPRB, 0.14607E+01_JPRB, 0.14108E+01_JPRB ,&
&  0.13391E+01_JPRB, 0.12475E+01_JPRB, 0.11252E+01_JPRB, 0.11842E+01_JPRB, 0.11795E+01_JPRB, 0.99642E+00_JPRB, 0.72538E+00_JPRB ,&
&  0.65659E+00_JPRB, 0.78195E+00_JPRB, 0.95763E+00_JPRB, 0.12812E+01_JPRB, 0.11603E+01_JPRB, 0.10531E+01_JPRB, 0.95622E+00_JPRB ,&
&  0.80735E+00_JPRB, 0.71352E+00_JPRB, 0.65092E+00_JPRB, 0.64286E+00_JPRB, 0.64263E+00_JPRB, 0.65357E+00_JPRB, 0.63855E+00_JPRB ,&
&  0.61402E+00_JPRB, 0.58787E+00_JPRB, 0.59515E+00_JPRB, 0.62092E+00_JPRB, 0.68194E+00_JPRB, 0.65274E+00_JPRB, 0.93025E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.61863E+00_JPRB, 0.80221E+00_JPRB, 0.52309E+00_JPRB, 0.67042E+00_JPRB, 0.75798E+00_JPRB, 0.87426E+00_JPRB ,&
&  0.90809E+00_JPRB, 0.12009E+01_JPRB, 0.15011E+01_JPRB, 0.13288E+01_JPRB, 0.11913E+01_JPRB, 0.10753E+01_JPRB, 0.10277E+01_JPRB ,&
&  0.10134E+01_JPRB, 0.10000E+01_JPRB, 0.99281E+00_JPRB, 0.98076E+00_JPRB, 0.96497E+00_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.10415E+01_JPRB, 0.10441E+01_JPRB, 0.10372E+01_JPRB, 0.10680E+01_JPRB, 0.10655E+01_JPRB, 0.10808E+01_JPRB, 0.10821E+01_JPRB ,&
&  0.11049E+01_JPRB, 0.11237E+01_JPRB, 0.11457E+01_JPRB, 0.11822E+01_JPRB, 0.11922E+01_JPRB, 0.12077E+01_JPRB, 0.12363E+01_JPRB ,&
&  0.12640E+01_JPRB, 0.12994E+01_JPRB, 0.12974E+01_JPRB, 0.12822E+01_JPRB, 0.12467E+01_JPRB, 0.12219E+01_JPRB, 0.11926E+01_JPRB ,&
&  0.11672E+01_JPRB, 0.11621E+01_JPRB, 0.11674E+01_JPRB, 0.11806E+01_JPRB, 0.11890E+01_JPRB, 0.12171E+01_JPRB, 0.83530E+00_JPRB ,&
&  0.13402E+01_JPRB, 0.15065E+01_JPRB, 0.14647E+01_JPRB, 0.14338E+01_JPRB, 0.13662E+01_JPRB, 0.14033E+01_JPRB, 0.14743E+01_JPRB ,&
&  0.15817E+01_JPRB, 0.16870E+01_JPRB, 0.17390E+01_JPRB, 0.17524E+01_JPRB, 0.17401E+01_JPRB, 0.17930E+01_JPRB, 0.18394E+01_JPRB ,&
&  0.18717E+01_JPRB, 0.18910E+01_JPRB, 0.19109E+01_JPRB, 0.19161E+01_JPRB, 0.19306E+01_JPRB, 0.19575E+01_JPRB, 0.19438E+01_JPRB ,&
&  0.18960E+01_JPRB, 0.18504E+01_JPRB, 0.18608E+01_JPRB, 0.19518E+01_JPRB, 0.19242E+01_JPRB, 0.16473E+01_JPRB, 0.12570E+01_JPRB ,&
&  0.80490E+00_JPRB, 0.67520E+00_JPRB, 0.92141E+00_JPRB, 0.12412E+01_JPRB, 0.11074E+01_JPRB, 0.99379E+00_JPRB, 0.89533E+00_JPRB ,&
&  0.84036E+00_JPRB, 0.75693E+00_JPRB, 0.68174E+00_JPRB, 0.67538E+00_JPRB, 0.69820E+00_JPRB, 0.75026E+00_JPRB, 0.74256E+00_JPRB ,&
&  0.75480E+00_JPRB, 0.75732E+00_JPRB, 0.74291E+00_JPRB, 0.74550E+00_JPRB, 0.81346E+00_JPRB, 0.84791E+00_JPRB, 0.77522E+00_JPRB ,&
&  0.88730E+00_JPRB, 0.80364E+00_JPRB, 0.81671E+00_JPRB, 0.90463E+00_JPRB, 0.95756E+00_JPRB, 0.11437E+01_JPRB, 0.14349E+01_JPRB ,&
&  0.18777E+01_JPRB, 0.18379E+01_JPRB, 0.24481E+01_JPRB, 0.37412E+01_JPRB, 0.29382E+01_JPRB, 0.20921E+01_JPRB, 0.11242E+01_JPRB ,&
&  0.11084E+01_JPRB, 0.10793E+01_JPRB, 0.10862E+01_JPRB, 0.10664E+01_JPRB, 0.10505E+01_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.10095E+01_JPRB, 0.99435E+00_JPRB, 0.99322E+00_JPRB, 0.99142E+00_JPRB, 0.98902E+00_JPRB, 0.99770E+00_JPRB, 0.10203E+01_JPRB ,&
&  0.10311E+01_JPRB, 0.10662E+01_JPRB, 0.11197E+01_JPRB, 0.11530E+01_JPRB, 0.11970E+01_JPRB, 0.11979E+01_JPRB, 0.11624E+01_JPRB ,&
&  0.11525E+01_JPRB, 0.11827E+01_JPRB, 0.12101E+01_JPRB, 0.12192E+01_JPRB, 0.11805E+01_JPRB, 0.11562E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.11939E+01_JPRB, 0.12211E+01_JPRB, 0.12497E+01_JPRB, 0.12946E+01_JPRB, 0.12961E+01_JPRB, 0.12219E+01_JPRB, 0.84241E+00_JPRB ,&
&  0.12334E+01_JPRB, 0.14674E+01_JPRB, 0.15257E+01_JPRB, 0.14955E+01_JPRB, 0.14950E+01_JPRB, 0.14454E+01_JPRB, 0.15050E+01_JPRB ,&
&  0.16183E+01_JPRB, 0.17389E+01_JPRB, 0.17658E+01_JPRB, 0.17113E+01_JPRB, 0.19497E+01_JPRB, 0.20035E+01_JPRB, 0.20851E+01_JPRB ,&
&  0.21500E+01_JPRB, 0.22108E+01_JPRB, 0.22724E+01_JPRB, 0.23441E+01_JPRB, 0.24020E+01_JPRB, 0.24629E+01_JPRB, 0.25070E+01_JPRB ,&
&  0.25050E+01_JPRB, 0.24897E+01_JPRB, 0.25375E+01_JPRB, 0.24286E+01_JPRB, 0.23642E+01_JPRB, 0.24223E+01_JPRB, 0.20438E+01_JPRB ,&
&  0.11827E+01_JPRB, 0.79364E+00_JPRB, 0.10134E+01_JPRB, 0.11975E+01_JPRB, 0.10497E+01_JPRB, 0.96798E+00_JPRB, 0.95325E+00_JPRB ,&
&  0.87085E+00_JPRB, 0.78281E+00_JPRB, 0.75747E+00_JPRB, 0.80471E+00_JPRB, 0.88141E+00_JPRB, 0.94746E+00_JPRB, 0.96302E+00_JPRB ,&
&  0.92850E+00_JPRB, 0.90448E+00_JPRB, 0.90241E+00_JPRB, 0.91897E+00_JPRB, 0.96510E+00_JPRB, 0.10726E+01_JPRB, 0.12266E+01_JPRB ,&
&  0.11679E+01_JPRB, 0.90389E+00_JPRB, 0.10109E+01_JPRB, 0.10736E+01_JPRB, 0.11858E+01_JPRB, 0.13460E+01_JPRB, 0.16440E+01_JPRB ,&
&  0.21059E+01_JPRB, 0.25946E+01_JPRB, 0.25464E+01_JPRB, 0.21346E+01_JPRB, 0.26985E+01_JPRB, 0.18071E+01_JPRB, 0.96331E+00_JPRB ,&
&  0.99310E+00_JPRB, 0.10119E+01_JPRB, 0.10297E+01_JPRB, 0.10249E+01_JPRB, 0.10199E+01_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.87266E+00_JPRB, 0.85618E+00_JPRB, 0.84466E+00_JPRB, 0.83409E+00_JPRB, 0.84579E+00_JPRB, 0.87998E+00_JPRB, 0.91928E+00_JPRB ,&
&  0.95850E+00_JPRB, 0.10031E+01_JPRB, 0.10628E+01_JPRB, 0.10816E+01_JPRB, 0.11175E+01_JPRB, 0.11129E+01_JPRB, 0.10982E+01_JPRB ,&
&  0.10720E+01_JPRB, 0.10804E+01_JPRB, 0.11007E+01_JPRB, 0.11221E+01_JPRB, 0.11275E+01_JPRB, 0.11500E+01_JPRB, 0.11799E+01_JPRB ,&
&  0.12214E+01_JPRB, 0.12619E+01_JPRB, 0.13184E+01_JPRB, 0.14145E+01_JPRB, 0.14416E+01_JPRB, 0.14390E+01_JPRB, 0.14482E+01_JPRB ,&
&  0.79707E+00_JPRB, 0.13032E+01_JPRB, 0.14264E+01_JPRB, 0.14189E+01_JPRB, 0.15258E+01_JPRB, 0.15545E+01_JPRB, 0.15926E+01_JPRB ,&
&  0.16930E+01_JPRB, 0.16985E+01_JPRB, 0.17272E+01_JPRB, 0.18208E+01_JPRB, 0.19812E+01_JPRB, 0.20501E+01_JPRB, 0.21181E+01_JPRB ,&
&  0.21447E+01_JPRB, 0.21644E+01_JPRB, 0.22334E+01_JPRB, 0.23345E+01_JPRB, 0.24324E+01_JPRB, 0.25316E+01_JPRB, 0.26608E+01_JPRB ,&
&  0.27394E+01_JPRB, 0.28058E+01_JPRB, 0.29503E+01_JPRB, 0.26991E+01_JPRB, 0.25182E+01_JPRB, 0.26624E+01_JPRB, 0.24376E+01_JPRB ,&
&  0.16924E+01_JPRB, 0.99837E+00_JPRB, 0.10468E+01_JPRB, 0.12526E+01_JPRB, 0.10999E+01_JPRB, 0.10681E+01_JPRB, 0.99648E+00_JPRB ,&
&  0.87765E+00_JPRB, 0.82192E+00_JPRB, 0.88796E+00_JPRB, 0.96015E+00_JPRB, 0.10328E+01_JPRB, 0.10921E+01_JPRB, 0.10890E+01_JPRB ,&
&  0.10264E+01_JPRB, 0.98435E+00_JPRB, 0.93862E+00_JPRB, 0.92404E+00_JPRB, 0.93929E+00_JPRB, 0.95400E+00_JPRB, 0.94325E+00_JPRB ,&
&  0.94061E+00_JPRB, 0.93063E+00_JPRB, 0.97366E+00_JPRB, 0.10059E+01_JPRB, 0.10489E+01_JPRB, 0.10975E+01_JPRB, 0.12397E+01_JPRB ,&
&  0.14746E+01_JPRB, 0.16859E+01_JPRB, 0.19395E+01_JPRB, 0.18983E+01_JPRB, 0.17247E+01_JPRB, 0.12392E+01_JPRB, 0.83590E+00_JPRB ,&
&  0.85599E+00_JPRB, 0.87712E+00_JPRB, 0.88052E+00_JPRB, 0.89044E+00_JPRB, 0.88157E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.78406E+00_JPRB, 0.77662E+00_JPRB, 0.76917E+00_JPRB, 0.77298E+00_JPRB, 0.78169E+00_JPRB, 0.82982E+00_JPRB, 0.90739E+00_JPRB ,&
&  0.98190E+00_JPRB, 0.10265E+01_JPRB, 0.10663E+01_JPRB, 0.10842E+01_JPRB, 0.10844E+01_JPRB, 0.10404E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.10247E+01_JPRB, 0.10203E+01_JPRB, 0.10393E+01_JPRB, 0.10706E+01_JPRB, 0.10972E+01_JPRB, 0.11268E+01_JPRB, 0.11590E+01_JPRB ,&
&  0.12002E+01_JPRB, 0.12623E+01_JPRB, 0.13261E+01_JPRB, 0.14253E+01_JPRB, 0.14963E+01_JPRB, 0.15333E+01_JPRB, 0.14801E+01_JPRB ,&
&  0.68728E+00_JPRB, 0.81890E+00_JPRB, 0.12775E+01_JPRB, 0.12739E+01_JPRB, 0.13622E+01_JPRB, 0.14984E+01_JPRB, 0.16935E+01_JPRB ,&
&  0.16767E+01_JPRB, 0.15407E+01_JPRB, 0.16018E+01_JPRB, 0.18805E+01_JPRB, 0.18797E+01_JPRB, 0.19477E+01_JPRB, 0.20005E+01_JPRB ,&
&  0.20079E+01_JPRB, 0.19976E+01_JPRB, 0.20107E+01_JPRB, 0.20841E+01_JPRB, 0.21923E+01_JPRB, 0.23103E+01_JPRB, 0.24538E+01_JPRB ,&
&  0.25972E+01_JPRB, 0.27603E+01_JPRB, 0.29294E+01_JPRB, 0.24836E+01_JPRB, 0.26272E+01_JPRB, 0.28699E+01_JPRB, 0.31948E+01_JPRB ,&
&  0.24846E+01_JPRB, 0.14531E+01_JPRB, 0.12508E+01_JPRB, 0.14305E+01_JPRB, 0.12879E+01_JPRB, 0.12834E+01_JPRB, 0.10883E+01_JPRB ,&
&  0.10558E+01_JPRB, 0.98893E+00_JPRB, 0.10141E+01_JPRB, 0.10766E+01_JPRB, 0.11369E+01_JPRB, 0.11765E+01_JPRB, 0.11636E+01_JPRB ,&
&  0.10966E+01_JPRB, 0.10534E+01_JPRB, 0.10296E+01_JPRB, 0.10262E+01_JPRB, 0.10365E+01_JPRB, 0.10440E+01_JPRB, 0.10338E+01_JPRB ,&
&  0.10221E+01_JPRB, 0.10230E+01_JPRB, 0.10650E+01_JPRB, 0.10930E+01_JPRB, 0.11226E+01_JPRB, 0.11539E+01_JPRB, 0.11822E+01_JPRB ,&
&  0.11413E+01_JPRB, 0.11208E+01_JPRB, 0.12118E+01_JPRB, 0.11206E+01_JPRB, 0.10005E+01_JPRB, 0.85858E+00_JPRB, 0.72725E+00_JPRB ,&
&  0.74683E+00_JPRB, 0.75653E+00_JPRB, 0.75390E+00_JPRB, 0.76194E+00_JPRB, 0.77997E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.69969E+00_JPRB, 0.70520E+00_JPRB, 0.69871E+00_JPRB, 0.71509E+00_JPRB, 0.71678E+00_JPRB, 0.78277E+00_JPRB, 0.85220E+00_JPRB ,&
&  0.95926E+00_JPRB, 0.10376E+01_JPRB, 0.10863E+01_JPRB, 0.10966E+01_JPRB, 0.10764E+01_JPRB, 0.10197E+01_JPRB, 0.96643E+00_JPRB ,&
&  0.96447E+00_JPRB, 0.96684E+00_JPRB, 0.97785E+00_JPRB, 0.99788E+00_JPRB, 0.10179E+01_JPRB, 0.10499E+01_JPRB, 0.10823E+01_JPRB ,&
&  0.11231E+01_JPRB, 0.11838E+01_JPRB, 0.12582E+01_JPRB, 0.13676E+01_JPRB, 0.15065E+01_JPRB, 0.16973E+01_JPRB, 0.18884E+01_JPRB ,&
&  0.14566E+01_JPRB, 0.74777E+00_JPRB, 0.10755E+01_JPRB, 0.11211E+01_JPRB, 0.10597E+01_JPRB, 0.12413E+01_JPRB, 0.15039E+01_JPRB ,&
&  0.14886E+01_JPRB, 0.14764E+01_JPRB, 0.14994E+01_JPRB, 0.17597E+01_JPRB, 0.17787E+01_JPRB, 0.18205E+01_JPRB, 0.18550E+01_JPRB ,&
&  0.18734E+01_JPRB, 0.18650E+01_JPRB, 0.18576E+01_JPRB, 0.19212E+01_JPRB, 0.20246E+01_JPRB, 0.21619E+01_JPRB, 0.23044E+01_JPRB ,&
&  0.24751E+01_JPRB, 0.26579E+01_JPRB, 0.28949E+01_JPRB, 0.24804E+01_JPRB, 0.27363E+01_JPRB, 0.27820E+01_JPRB, 0.24978E+01_JPRB ,&
&  0.24076E+01_JPRB, 0.16517E+01_JPRB, 0.13613E+01_JPRB, 0.15456E+01_JPRB, 0.14305E+01_JPRB, 0.11289E+01_JPRB, 0.11429E+01_JPRB ,&
&  0.10888E+01_JPRB, 0.11004E+01_JPRB, 0.10814E+01_JPRB, 0.11501E+01_JPRB, 0.11869E+01_JPRB, 0.12108E+01_JPRB, 0.11858E+01_JPRB ,&
&  0.11589E+01_JPRB, 0.11158E+01_JPRB, 0.10971E+01_JPRB, 0.10980E+01_JPRB, 0.11005E+01_JPRB, 0.11270E+01_JPRB, 0.11279E+01_JPRB ,&
&  0.11238E+01_JPRB, 0.11414E+01_JPRB, 0.11863E+01_JPRB, 0.12360E+01_JPRB, 0.12588E+01_JPRB, 0.11954E+01_JPRB, 0.11399E+01_JPRB ,&
&  0.11010E+01_JPRB, 0.10611E+01_JPRB, 0.96575E+00_JPRB, 0.85613E+00_JPRB, 0.82843E+00_JPRB, 0.76092E+00_JPRB, 0.70404E+00_JPRB ,&
&  0.71428E+00_JPRB, 0.71217E+00_JPRB, 0.69780E+00_JPRB, 0.69678E+00_JPRB, 0.70165E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.73575E+00_JPRB, 0.73299E+00_JPRB, 0.70478E+00_JPRB, 0.68676E+00_JPRB, 0.68204E+00_JPRB, 0.70824E+00_JPRB, 0.75286E+00_JPRB ,&
&  0.82303E+00_JPRB, 0.91386E+00_JPRB, 0.98546E+00_JPRB, 0.10170E+01_JPRB, 0.10076E+01_JPRB, 0.10121E+01_JPRB, 0.96529E+00_JPRB ,&
&  0.91415E+00_JPRB, 0.90476E+00_JPRB, 0.91603E+00_JPRB, 0.92446E+00_JPRB, 0.93858E+00_JPRB, 0.95894E+00_JPRB, 0.98291E+00_JPRB ,&
&  0.99604E+00_JPRB, 0.10052E+01_JPRB, 0.10152E+01_JPRB, 0.10793E+01_JPRB, 0.11855E+01_JPRB, 0.13499E+01_JPRB, 0.16501E+01_JPRB ,&
&  0.23276E+01_JPRB, 0.22820E+01_JPRB, 0.12141E+01_JPRB, 0.12814E+01_JPRB, 0.11264E+01_JPRB, 0.11866E+01_JPRB, 0.11882E+01_JPRB ,&
&  0.13214E+01_JPRB, 0.13135E+01_JPRB, 0.13812E+01_JPRB, 0.15873E+01_JPRB, 0.16158E+01_JPRB, 0.16589E+01_JPRB, 0.16856E+01_JPRB ,&
&  0.17356E+01_JPRB, 0.17492E+01_JPRB, 0.17235E+01_JPRB, 0.17294E+01_JPRB, 0.18200E+01_JPRB, 0.19513E+01_JPRB, 0.21019E+01_JPRB ,&
&  0.22943E+01_JPRB, 0.24596E+01_JPRB, 0.27006E+01_JPRB, 0.24862E+01_JPRB, 0.27156E+01_JPRB, 0.25259E+01_JPRB, 0.20615E+01_JPRB ,&
&  0.17745E+01_JPRB, 0.16190E+01_JPRB, 0.16103E+01_JPRB, 0.16464E+01_JPRB, 0.15199E+01_JPRB, 0.11956E+01_JPRB, 0.12032E+01_JPRB ,&
&  0.11408E+01_JPRB, 0.11227E+01_JPRB, 0.11156E+01_JPRB, 0.11361E+01_JPRB, 0.11801E+01_JPRB, 0.11920E+01_JPRB, 0.11993E+01_JPRB ,&
&  0.11761E+01_JPRB, 0.11444E+01_JPRB, 0.11291E+01_JPRB, 0.11237E+01_JPRB, 0.11253E+01_JPRB, 0.11527E+01_JPRB, 0.11661E+01_JPRB ,&
&  0.11815E+01_JPRB, 0.12102E+01_JPRB, 0.12534E+01_JPRB, 0.12992E+01_JPRB, 0.12775E+01_JPRB, 0.12011E+01_JPRB, 0.11176E+01_JPRB ,&
&  0.10931E+01_JPRB, 0.10974E+01_JPRB, 0.10207E+01_JPRB, 0.96491E+00_JPRB, 0.91393E+00_JPRB, 0.90646E+00_JPRB, 0.88516E+00_JPRB ,&
&  0.88241E+00_JPRB, 0.88218E+00_JPRB, 0.84387E+00_JPRB, 0.73656E+00_JPRB, 0.73717E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.82541E+00_JPRB, 0.78515E+00_JPRB, 0.77767E+00_JPRB, 0.77107E+00_JPRB, 0.75610E+00_JPRB, 0.75239E+00_JPRB, 0.72135E+00_JPRB ,&
&  0.72769E+00_JPRB, 0.78134E+00_JPRB, 0.84605E+00_JPRB, 0.87862E+00_JPRB, 0.89738E+00_JPRB, 0.89687E+00_JPRB, 0.87127E+00_JPRB ,&
&  0.84015E+00_JPRB, 0.84698E+00_JPRB, 0.86247E+00_JPRB, 0.87187E+00_JPRB, 0.88877E+00_JPRB, 0.90378E+00_JPRB, 0.92819E+00_JPRB ,&
&  0.94718E+00_JPRB, 0.92609E+00_JPRB, 0.89976E+00_JPRB, 0.89125E+00_JPRB, 0.93665E+00_JPRB, 0.10042E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.15622E+01_JPRB, 0.31022E+01_JPRB, 0.16594E+01_JPRB, 0.14767E+01_JPRB, 0.13605E+01_JPRB, 0.12452E+01_JPRB, 0.11676E+01_JPRB ,&
&  0.12169E+01_JPRB, 0.11191E+01_JPRB, 0.11825E+01_JPRB, 0.13703E+01_JPRB, 0.13943E+01_JPRB, 0.14146E+01_JPRB, 0.14615E+01_JPRB ,&
&  0.14992E+01_JPRB, 0.15240E+01_JPRB, 0.15324E+01_JPRB, 0.14998E+01_JPRB, 0.15625E+01_JPRB, 0.16630E+01_JPRB, 0.18052E+01_JPRB ,&
&  0.19666E+01_JPRB, 0.21712E+01_JPRB, 0.24466E+01_JPRB, 0.23974E+01_JPRB, 0.25669E+01_JPRB, 0.24211E+01_JPRB, 0.20063E+01_JPRB ,&
&  0.17677E+01_JPRB, 0.17685E+01_JPRB, 0.17006E+01_JPRB, 0.16685E+01_JPRB, 0.15492E+01_JPRB, 0.13130E+01_JPRB, 0.13306E+01_JPRB ,&
&  0.12043E+01_JPRB, 0.11613E+01_JPRB, 0.11607E+01_JPRB, 0.11587E+01_JPRB, 0.11806E+01_JPRB, 0.11833E+01_JPRB, 0.11876E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.11470E+01_JPRB, 0.11312E+01_JPRB, 0.11280E+01_JPRB, 0.11260E+01_JPRB, 0.11306E+01_JPRB, 0.11516E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.12093E+01_JPRB, 0.12627E+01_JPRB, 0.12686E+01_JPRB, 0.12310E+01_JPRB, 0.11531E+01_JPRB, 0.10728E+01_JPRB ,&
&  0.10480E+01_JPRB, 0.10577E+01_JPRB, 0.10243E+01_JPRB, 0.98200E+00_JPRB, 0.98446E+00_JPRB, 0.96786E+00_JPRB, 0.10055E+01_JPRB ,&
&  0.10885E+01_JPRB, 0.12114E+01_JPRB, 0.13054E+01_JPRB, 0.86092E+00_JPRB, 0.85732E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.86676E+00_JPRB, 0.82363E+00_JPRB, 0.81729E+00_JPRB, 0.83559E+00_JPRB, 0.83081E+00_JPRB, 0.81329E+00_JPRB, 0.77845E+00_JPRB ,&
&  0.73906E+00_JPRB, 0.74818E+00_JPRB, 0.77204E+00_JPRB, 0.79027E+00_JPRB, 0.80092E+00_JPRB, 0.81072E+00_JPRB, 0.79608E+00_JPRB ,&
&  0.77119E+00_JPRB, 0.77517E+00_JPRB, 0.79058E+00_JPRB, 0.81835E+00_JPRB, 0.83850E+00_JPRB, 0.85907E+00_JPRB, 0.88931E+00_JPRB ,&
&  0.89759E+00_JPRB, 0.89199E+00_JPRB, 0.87874E+00_JPRB, 0.86638E+00_JPRB, 0.86287E+00_JPRB, 0.88001E+00_JPRB, 0.91629E+00_JPRB ,&
&  0.11300E+01_JPRB, 0.27335E+01_JPRB, 0.26981E+01_JPRB, 0.18712E+01_JPRB, 0.14374E+01_JPRB, 0.12135E+01_JPRB, 0.11178E+01_JPRB ,&
&  0.11898E+01_JPRB, 0.12142E+01_JPRB, 0.13800E+01_JPRB, 0.13159E+01_JPRB, 0.12610E+01_JPRB, 0.12619E+01_JPRB, 0.12983E+01_JPRB ,&
&  0.12912E+01_JPRB, 0.12973E+01_JPRB, 0.12867E+01_JPRB, 0.12679E+01_JPRB, 0.12755E+01_JPRB, 0.13599E+01_JPRB, 0.14704E+01_JPRB ,&
&  0.15798E+01_JPRB, 0.17731E+01_JPRB, 0.21101E+01_JPRB, 0.24256E+01_JPRB, 0.24195E+01_JPRB, 0.24780E+01_JPRB, 0.23615E+01_JPRB ,&
&  0.24681E+01_JPRB, 0.22550E+01_JPRB, 0.18978E+01_JPRB, 0.16315E+01_JPRB, 0.14364E+01_JPRB, 0.14791E+01_JPRB, 0.14042E+01_JPRB ,&
&  0.12640E+01_JPRB, 0.11807E+01_JPRB, 0.11562E+01_JPRB, 0.11599E+01_JPRB, 0.11689E+01_JPRB, 0.11705E+01_JPRB, 0.11452E+01_JPRB ,&
&  0.11241E+01_JPRB, 0.11161E+01_JPRB, 0.11127E+01_JPRB, 0.11155E+01_JPRB, 0.11123E+01_JPRB, 0.11114E+01_JPRB, 0.11286E+01_JPRB ,&
&  0.11343E+01_JPRB, 0.11576E+01_JPRB, 0.11662E+01_JPRB, 0.12027E+01_JPRB, 0.11681E+01_JPRB, 0.10860E+01_JPRB, 0.10131E+01_JPRB ,&
&  0.99343E+00_JPRB, 0.10170E+01_JPRB, 0.10228E+01_JPRB, 0.10121E+01_JPRB, 0.99454E+00_JPRB, 0.97014E+00_JPRB, 0.95111E+00_JPRB ,&
&  0.96371E+00_JPRB, 0.98169E+00_JPRB, 0.95797E+00_JPRB, 0.90074E+00_JPRB, 0.91051E+00_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.90711E+00_JPRB, 0.87726E+00_JPRB, 0.85336E+00_JPRB, 0.85732E+00_JPRB, 0.86223E+00_JPRB, 0.85138E+00_JPRB, 0.83082E+00_JPRB ,&
&  0.81366E+00_JPRB, 0.79410E+00_JPRB, 0.80162E+00_JPRB, 0.78930E+00_JPRB, 0.78057E+00_JPRB, 0.77086E+00_JPRB, 0.76424E+00_JPRB ,&
&  0.76585E+00_JPRB, 0.77320E+00_JPRB, 0.76116E+00_JPRB, 0.76469E+00_JPRB, 0.79047E+00_JPRB, 0.81598E+00_JPRB, 0.82626E+00_JPRB ,&
&  0.83649E+00_JPRB, 0.84847E+00_JPRB, 0.84963E+00_JPRB, 0.84867E+00_JPRB, 0.83543E+00_JPRB, 0.83657E+00_JPRB, 0.84725E+00_JPRB ,&
&  0.96113E+00_JPRB, 0.21502E+01_JPRB, 0.15998E+01_JPRB, 0.18586E+01_JPRB, 0.14143E+01_JPRB, 0.12074E+01_JPRB, 0.10083E+01_JPRB ,&
&  0.10934E+01_JPRB, 0.13614E+01_JPRB, 0.13120E+01_JPRB, 0.12698E+01_JPRB, 0.12070E+01_JPRB, 0.11794E+01_JPRB, 0.11621E+01_JPRB ,&
&  0.11519E+01_JPRB, 0.11614E+01_JPRB, 0.11464E+01_JPRB, 0.11426E+01_JPRB, 0.11217E+01_JPRB, 0.11598E+01_JPRB, 0.12249E+01_JPRB ,&
&  0.12852E+01_JPRB, 0.13868E+01_JPRB, 0.16130E+01_JPRB, 0.18836E+01_JPRB, 0.24376E+01_JPRB, 0.27424E+01_JPRB, 0.28827E+01_JPRB ,&
&  0.30040E+01_JPRB, 0.26917E+01_JPRB, 0.22135E+01_JPRB, 0.17884E+01_JPRB, 0.15562E+01_JPRB, 0.14964E+01_JPRB, 0.14767E+01_JPRB ,&
&  0.13506E+01_JPRB, 0.12573E+01_JPRB, 0.11985E+01_JPRB, 0.11719E+01_JPRB, 0.11807E+01_JPRB, 0.11797E+01_JPRB, 0.11335E+01_JPRB ,&
&  0.10978E+01_JPRB, 0.10988E+01_JPRB, 0.11072E+01_JPRB, 0.11140E+01_JPRB, 0.11021E+01_JPRB, 0.10801E+01_JPRB, 0.10818E+01_JPRB ,&
&  0.10945E+01_JPRB, 0.11071E+01_JPRB, 0.11118E+01_JPRB, 0.11707E+01_JPRB, 0.11341E+01_JPRB, 0.10693E+01_JPRB, 0.10055E+01_JPRB ,&
&  0.99855E+00_JPRB, 0.10018E+01_JPRB, 0.99829E+00_JPRB, 0.98599E+00_JPRB, 0.10043E+01_JPRB, 0.10121E+01_JPRB, 0.95052E+00_JPRB ,&
&  0.94568E+00_JPRB, 0.94326E+00_JPRB, 0.93906E+00_JPRB, 0.91428E+00_JPRB, 0.92805E+00_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.90207E+00_JPRB, 0.87988E+00_JPRB, 0.88172E+00_JPRB, 0.88315E+00_JPRB, 0.87800E+00_JPRB, 0.85804E+00_JPRB, 0.84571E+00_JPRB ,&
&  0.85890E+00_JPRB, 0.85697E+00_JPRB, 0.84825E+00_JPRB, 0.81912E+00_JPRB, 0.80203E+00_JPRB, 0.78069E+00_JPRB, 0.76618E+00_JPRB ,&
&  0.76780E+00_JPRB, 0.77076E+00_JPRB, 0.76341E+00_JPRB, 0.76319E+00_JPRB, 0.76311E+00_JPRB, 0.77885E+00_JPRB, 0.79268E+00_JPRB ,&
&  0.80542E+00_JPRB, 0.81592E+00_JPRB, 0.83282E+00_JPRB, 0.84116E+00_JPRB, 0.83444E+00_JPRB, 0.81987E+00_JPRB, 0.81701E+00_JPRB ,&
&  0.90404E+00_JPRB, 0.20625E+01_JPRB, 0.13199E+01_JPRB, 0.18345E+01_JPRB, 0.14018E+01_JPRB, 0.12247E+01_JPRB, 0.10311E+01_JPRB ,&
&  0.11139E+01_JPRB, 0.13400E+01_JPRB, 0.13053E+01_JPRB, 0.12034E+01_JPRB, 0.11706E+01_JPRB, 0.11425E+01_JPRB, 0.11015E+01_JPRB ,&
&  0.10840E+01_JPRB, 0.10979E+01_JPRB, 0.10880E+01_JPRB, 0.10900E+01_JPRB, 0.10843E+01_JPRB, 0.10664E+01_JPRB, 0.10825E+01_JPRB ,&
&  0.11034E+01_JPRB, 0.11440E+01_JPRB, 0.12803E+01_JPRB, 0.17627E+01_JPRB, 0.21740E+01_JPRB, 0.28897E+01_JPRB, 0.36597E+01_JPRB ,&
&  0.36043E+01_JPRB, 0.31667E+01_JPRB, 0.23542E+01_JPRB, 0.19676E+01_JPRB, 0.17175E+01_JPRB, 0.15953E+01_JPRB, 0.15252E+01_JPRB ,&
&  0.14487E+01_JPRB, 0.13957E+01_JPRB, 0.12935E+01_JPRB, 0.12417E+01_JPRB, 0.12308E+01_JPRB, 0.12167E+01_JPRB, 0.11825E+01_JPRB ,&
&  0.11441E+01_JPRB, 0.11399E+01_JPRB, 0.11475E+01_JPRB, 0.11416E+01_JPRB, 0.11246E+01_JPRB, 0.10902E+01_JPRB, 0.10767E+01_JPRB ,&
&  0.10903E+01_JPRB, 0.10934E+01_JPRB, 0.11135E+01_JPRB, 0.11995E+01_JPRB, 0.11657E+01_JPRB, 0.10840E+01_JPRB, 0.10263E+01_JPRB ,&
&  0.10273E+01_JPRB, 0.10054E+01_JPRB, 0.99753E+00_JPRB, 0.10074E+01_JPRB, 0.10244E+01_JPRB, 0.10778E+01_JPRB, 0.99914E+00_JPRB ,&
&  0.95813E+00_JPRB, 0.92467E+00_JPRB, 0.90741E+00_JPRB, 0.90063E+00_JPRB, 0.90753E+00_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.92305E+00_JPRB, 0.91077E+00_JPRB, 0.90956E+00_JPRB, 0.92132E+00_JPRB, 0.90736E+00_JPRB, 0.88547E+00_JPRB, 0.86318E+00_JPRB ,&
&  0.87993E+00_JPRB, 0.88332E+00_JPRB, 0.87960E+00_JPRB, 0.84481E+00_JPRB, 0.82355E+00_JPRB, 0.80340E+00_JPRB, 0.79377E+00_JPRB ,&
&  0.79549E+00_JPRB, 0.78769E+00_JPRB, 0.77399E+00_JPRB, 0.76754E+00_JPRB, 0.77026E+00_JPRB, 0.77506E+00_JPRB, 0.79302E+00_JPRB ,&
&  0.78695E+00_JPRB, 0.79383E+00_JPRB, 0.81683E+00_JPRB, 0.82804E+00_JPRB, 0.83620E+00_JPRB, 0.81710E+00_JPRB, 0.80710E+00_JPRB ,&
&  0.83962E+00_JPRB, 0.91565E+00_JPRB, 0.14173E+01_JPRB, 0.16796E+01_JPRB, 0.14148E+01_JPRB, 0.12681E+01_JPRB, 0.11855E+01_JPRB ,&
&  0.12356E+01_JPRB, 0.12770E+01_JPRB, 0.12486E+01_JPRB, 0.11851E+01_JPRB, 0.11490E+01_JPRB, 0.11040E+01_JPRB, 0.10711E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.10185E+01_JPRB, 0.10204E+01_JPRB, 0.10382E+01_JPRB, 0.10458E+01_JPRB, 0.10347E+01_JPRB, 0.10062E+01_JPRB ,&
&  0.99354E+00_JPRB, 0.99295E+00_JPRB, 0.10492E+01_JPRB, 0.13688E+01_JPRB, 0.20955E+01_JPRB, 0.29760E+01_JPRB, 0.36343E+01_JPRB ,&
&  0.37625E+01_JPRB, 0.31746E+01_JPRB, 0.23825E+01_JPRB, 0.20635E+01_JPRB, 0.18395E+01_JPRB, 0.16746E+01_JPRB, 0.15297E+01_JPRB ,&
&  0.14483E+01_JPRB, 0.14378E+01_JPRB, 0.13989E+01_JPRB, 0.13293E+01_JPRB, 0.12613E+01_JPRB, 0.12240E+01_JPRB, 0.11934E+01_JPRB ,&
&  0.11758E+01_JPRB, 0.11853E+01_JPRB, 0.11919E+01_JPRB, 0.11641E+01_JPRB, 0.11336E+01_JPRB, 0.11015E+01_JPRB, 0.10879E+01_JPRB ,&
&  0.10987E+01_JPRB, 0.11144E+01_JPRB, 0.11485E+01_JPRB, 0.12053E+01_JPRB, 0.11840E+01_JPRB, 0.11384E+01_JPRB, 0.10848E+01_JPRB ,&
&  0.10379E+01_JPRB, 0.10015E+01_JPRB, 0.98600E+00_JPRB, 0.10266E+01_JPRB, 0.10326E+01_JPRB, 0.11090E+01_JPRB, 0.10668E+01_JPRB ,&
&  0.10007E+01_JPRB, 0.95587E+00_JPRB, 0.93322E+00_JPRB, 0.92593E+00_JPRB, 0.93161E+00_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.95659E+00_JPRB, 0.95334E+00_JPRB, 0.96359E+00_JPRB, 0.97223E+00_JPRB, 0.96060E+00_JPRB, 0.93276E+00_JPRB, 0.90589E+00_JPRB ,&
&  0.89256E+00_JPRB, 0.88178E+00_JPRB, 0.86798E+00_JPRB, 0.84096E+00_JPRB, 0.82434E+00_JPRB, 0.82045E+00_JPRB, 0.82141E+00_JPRB ,&
&  0.82500E+00_JPRB, 0.81526E+00_JPRB, 0.80042E+00_JPRB, 0.78490E+00_JPRB, 0.78569E+00_JPRB, 0.79283E+00_JPRB, 0.80779E+00_JPRB ,&
&  0.80533E+00_JPRB, 0.80499E+00_JPRB, 0.81101E+00_JPRB, 0.81829E+00_JPRB, 0.82671E+00_JPRB, 0.81707E+00_JPRB, 0.80347E+00_JPRB ,&
&  0.80311E+00_JPRB, 0.10031E+01_JPRB, 0.12696E+01_JPRB, 0.13963E+01_JPRB, 0.13677E+01_JPRB, 0.12737E+01_JPRB, 0.12330E+01_JPRB ,&
&  0.11825E+01_JPRB, 0.11960E+01_JPRB, 0.12090E+01_JPRB, 0.11756E+01_JPRB, 0.11303E+01_JPRB, 0.10946E+01_JPRB, 0.10701E+01_JPRB ,&
&  0.10395E+01_JPRB, 0.10104E+01_JPRB, 0.98611E+00_JPRB, 0.98628E+00_JPRB, 0.99497E+00_JPRB, 0.10081E+01_JPRB, 0.98994E+00_JPRB ,&
&  0.96020E+00_JPRB, 0.93450E+00_JPRB, 0.93475E+00_JPRB, 0.10946E+01_JPRB, 0.16921E+01_JPRB, 0.20550E+01_JPRB, 0.29594E+01_JPRB ,&
&  0.33152E+01_JPRB, 0.25344E+01_JPRB, 0.20717E+01_JPRB, 0.18596E+01_JPRB, 0.17105E+01_JPRB, 0.15592E+01_JPRB, 0.14457E+01_JPRB ,&
&  0.13941E+01_JPRB, 0.13724E+01_JPRB, 0.13827E+01_JPRB, 0.13553E+01_JPRB, 0.12766E+01_JPRB, 0.11930E+01_JPRB, 0.11518E+01_JPRB ,&
&  0.11319E+01_JPRB, 0.11166E+01_JPRB, 0.11128E+01_JPRB, 0.11108E+01_JPRB, 0.10985E+01_JPRB, 0.10650E+01_JPRB, 0.10501E+01_JPRB ,&
&  0.10536E+01_JPRB, 0.10811E+01_JPRB, 0.11363E+01_JPRB, 0.11593E+01_JPRB, 0.11784E+01_JPRB, 0.11859E+01_JPRB, 0.11255E+01_JPRB ,&
&  0.10439E+01_JPRB, 0.99785E+00_JPRB, 0.99018E+00_JPRB, 0.10100E+01_JPRB, 0.10553E+01_JPRB, 0.11474E+01_JPRB, 0.11055E+01_JPRB ,&
&  0.10451E+01_JPRB, 0.99225E+00_JPRB, 0.96578E+00_JPRB, 0.94976E+00_JPRB, 0.96747E+00_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.98943E+00_JPRB, 0.97999E+00_JPRB, 0.98914E+00_JPRB, 0.99400E+00_JPRB, 0.98293E+00_JPRB, 0.95311E+00_JPRB, 0.92908E+00_JPRB ,&
&  0.91840E+00_JPRB, 0.88640E+00_JPRB, 0.85473E+00_JPRB, 0.83232E+00_JPRB, 0.83331E+00_JPRB, 0.83802E+00_JPRB, 0.83892E+00_JPRB ,&
&  0.83032E+00_JPRB, 0.82946E+00_JPRB, 0.81771E+00_JPRB, 0.80923E+00_JPRB, 0.80669E+00_JPRB, 0.80618E+00_JPRB, 0.79855E+00_JPRB ,&
&  0.80336E+00_JPRB, 0.80474E+00_JPRB, 0.80964E+00_JPRB, 0.81751E+00_JPRB, 0.81498E+00_JPRB, 0.81822E+00_JPRB, 0.81002E+00_JPRB ,&
&  0.78700E+00_JPRB, 0.83328E+00_JPRB, 0.96825E+00_JPRB, 0.11633E+01_JPRB, 0.12273E+01_JPRB, 0.12174E+01_JPRB, 0.11804E+01_JPRB ,&
&  0.11400E+01_JPRB, 0.11256E+01_JPRB, 0.11347E+01_JPRB, 0.11152E+01_JPRB, 0.10841E+01_JPRB, 0.10632E+01_JPRB, 0.10440E+01_JPRB ,&
&  0.10267E+01_JPRB, 0.10091E+01_JPRB, 0.10040E+01_JPRB, 0.99204E+00_JPRB, 0.97828E+00_JPRB, 0.98096E+00_JPRB, 0.99021E+00_JPRB ,&
&  0.95159E+00_JPRB, 0.92006E+00_JPRB, 0.89539E+00_JPRB, 0.91876E+00_JPRB, 0.12102E+01_JPRB, 0.15589E+01_JPRB, 0.20377E+01_JPRB ,&
&  0.19398E+01_JPRB, 0.17402E+01_JPRB, 0.16108E+01_JPRB, 0.15134E+01_JPRB, 0.14532E+01_JPRB, 0.13582E+01_JPRB, 0.12803E+01_JPRB ,&
&  0.12056E+01_JPRB, 0.11783E+01_JPRB, 0.11573E+01_JPRB, 0.11502E+01_JPRB, 0.11319E+01_JPRB, 0.11174E+01_JPRB, 0.10726E+01_JPRB ,&
&  0.10459E+01_JPRB, 0.10275E+01_JPRB, 0.10228E+01_JPRB, 0.10289E+01_JPRB, 0.10426E+01_JPRB, 0.10381E+01_JPRB, 0.10332E+01_JPRB ,&
&  0.10442E+01_JPRB, 0.10570E+01_JPRB, 0.10995E+01_JPRB, 0.11541E+01_JPRB, 0.11460E+01_JPRB, 0.11141E+01_JPRB, 0.10898E+01_JPRB ,&
&  0.10706E+01_JPRB, 0.10364E+01_JPRB, 0.10031E+01_JPRB, 0.10274E+01_JPRB, 0.10478E+01_JPRB, 0.11311E+01_JPRB, 0.11139E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.10151E+01_JPRB, 0.98767E+00_JPRB, 0.97026E+00_JPRB, 0.98958E+00_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.10316E+01_JPRB, 0.10342E+01_JPRB, 0.10278E+01_JPRB, 0.10223E+01_JPRB, 0.10076E+01_JPRB, 0.97804E+00_JPRB, 0.94750E+00_JPRB ,&
&  0.92668E+00_JPRB, 0.90235E+00_JPRB, 0.86651E+00_JPRB, 0.84740E+00_JPRB, 0.84241E+00_JPRB, 0.84311E+00_JPRB, 0.84006E+00_JPRB ,&
&  0.83669E+00_JPRB, 0.84215E+00_JPRB, 0.83784E+00_JPRB, 0.84011E+00_JPRB, 0.83477E+00_JPRB, 0.83297E+00_JPRB, 0.81582E+00_JPRB ,&
&  0.80510E+00_JPRB, 0.79773E+00_JPRB, 0.79631E+00_JPRB, 0.80358E+00_JPRB, 0.81006E+00_JPRB, 0.81472E+00_JPRB, 0.81958E+00_JPRB ,&
&  0.80737E+00_JPRB, 0.72803E+00_JPRB, 0.80747E+00_JPRB, 0.93147E+00_JPRB, 0.10160E+01_JPRB, 0.10890E+01_JPRB, 0.10792E+01_JPRB ,&
&  0.10694E+01_JPRB, 0.10573E+01_JPRB, 0.10664E+01_JPRB, 0.10523E+01_JPRB, 0.10209E+01_JPRB, 0.99393E+00_JPRB, 0.97803E+00_JPRB ,&
&  0.97910E+00_JPRB, 0.97575E+00_JPRB, 0.97479E+00_JPRB, 0.97087E+00_JPRB, 0.96593E+00_JPRB, 0.96309E+00_JPRB, 0.97072E+00_JPRB ,&
&  0.96244E+00_JPRB, 0.92975E+00_JPRB, 0.89464E+00_JPRB, 0.87109E+00_JPRB, 0.91140E+00_JPRB, 0.10369E+01_JPRB, 0.12589E+01_JPRB ,&
&  0.13810E+01_JPRB, 0.13242E+01_JPRB, 0.12733E+01_JPRB, 0.12289E+01_JPRB, 0.11945E+01_JPRB, 0.11774E+01_JPRB, 0.11555E+01_JPRB ,&
&  0.11403E+01_JPRB, 0.11252E+01_JPRB, 0.11160E+01_JPRB, 0.11036E+01_JPRB, 0.10905E+01_JPRB, 0.10787E+01_JPRB, 0.10611E+01_JPRB ,&
&  0.10449E+01_JPRB, 0.10383E+01_JPRB, 0.10271E+01_JPRB, 0.10310E+01_JPRB, 0.10314E+01_JPRB, 0.10279E+01_JPRB, 0.10259E+01_JPRB ,&
&  0.10375E+01_JPRB, 0.10675E+01_JPRB, 0.10886E+01_JPRB, 0.11121E+01_JPRB, 0.11085E+01_JPRB, 0.10878E+01_JPRB, 0.10782E+01_JPRB ,&
&  0.10611E+01_JPRB, 0.10458E+01_JPRB, 0.99307E+00_JPRB, 0.10041E+01_JPRB, 0.11017E+01_JPRB, 0.10924E+01_JPRB, 0.10800E+01_JPRB ,&
&  0.10722E+01_JPRB, 0.10477E+01_JPRB, 0.10052E+01_JPRB, 0.96093E+00_JPRB, 0.99915E+00_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.93692E+00_JPRB, 0.97262E+00_JPRB, 0.97489E+00_JPRB, 0.97508E+00_JPRB, 0.97243E+00_JPRB, 0.95702E+00_JPRB, 0.93666E+00_JPRB ,&
&  0.91578E+00_JPRB, 0.90431E+00_JPRB, 0.88804E+00_JPRB, 0.86623E+00_JPRB, 0.85255E+00_JPRB, 0.84510E+00_JPRB, 0.83826E+00_JPRB ,&
&  0.83633E+00_JPRB, 0.83421E+00_JPRB, 0.83275E+00_JPRB, 0.83046E+00_JPRB, 0.82279E+00_JPRB, 0.81633E+00_JPRB, 0.80445E+00_JPRB ,&
&  0.79675E+00_JPRB, 0.79680E+00_JPRB, 0.79813E+00_JPRB, 0.80300E+00_JPRB, 0.81120E+00_JPRB, 0.82036E+00_JPRB, 0.82664E+00_JPRB ,&
&  0.82394E+00_JPRB, 0.71343E+00_JPRB, 0.74043E+00_JPRB, 0.81240E+00_JPRB, 0.89477E+00_JPRB, 0.97178E+00_JPRB, 0.10008E+01_JPRB ,&
&  0.10129E+01_JPRB, 0.10165E+01_JPRB, 0.10094E+01_JPRB, 0.99607E+00_JPRB, 0.97926E+00_JPRB, 0.96855E+00_JPRB, 0.94249E+00_JPRB ,&
&  0.92719E+00_JPRB, 0.92364E+00_JPRB, 0.93007E+00_JPRB, 0.93272E+00_JPRB, 0.92983E+00_JPRB, 0.92476E+00_JPRB, 0.92600E+00_JPRB ,&
&  0.92297E+00_JPRB, 0.90344E+00_JPRB, 0.87915E+00_JPRB, 0.86090E+00_JPRB, 0.86366E+00_JPRB, 0.91973E+00_JPRB, 0.10147E+01_JPRB ,&
&  0.11045E+01_JPRB, 0.11022E+01_JPRB, 0.10756E+01_JPRB, 0.10483E+01_JPRB, 0.10378E+01_JPRB, 0.10403E+01_JPRB, 0.10502E+01_JPRB ,&
&  0.10481E+01_JPRB, 0.10443E+01_JPRB, 0.10556E+01_JPRB, 0.10582E+01_JPRB, 0.10609E+01_JPRB, 0.10591E+01_JPRB, 0.10559E+01_JPRB ,&
&  0.10470E+01_JPRB, 0.10354E+01_JPRB, 0.10294E+01_JPRB, 0.10265E+01_JPRB, 0.10303E+01_JPRB, 0.10284E+01_JPRB, 0.10197E+01_JPRB ,&
&  0.10085E+01_JPRB, 0.10192E+01_JPRB, 0.10546E+01_JPRB, 0.10852E+01_JPRB, 0.10797E+01_JPRB, 0.10647E+01_JPRB, 0.10523E+01_JPRB ,&
&  0.10511E+01_JPRB, 0.10395E+01_JPRB, 0.10301E+01_JPRB, 0.10643E+01_JPRB, 0.10608E+01_JPRB, 0.10486E+01_JPRB, 0.10377E+01_JPRB ,&
&  0.10307E+01_JPRB, 0.10115E+01_JPRB, 0.99165E+00_JPRB, 0.96024E+00_JPRB, 0.87942E+00_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.86423E+00_JPRB, 0.90090E+00_JPRB, 0.92172E+00_JPRB, 0.92766E+00_JPRB, 0.93517E+00_JPRB, 0.93797E+00_JPRB, 0.93462E+00_JPRB ,&
&  0.93235E+00_JPRB, 0.92727E+00_JPRB, 0.91475E+00_JPRB, 0.89550E+00_JPRB, 0.87609E+00_JPRB, 0.85492E+00_JPRB, 0.83889E+00_JPRB ,&
&  0.83529E+00_JPRB, 0.83385E+00_JPRB, 0.82273E+00_JPRB, 0.81218E+00_JPRB, 0.80184E+00_JPRB, 0.79356E+00_JPRB, 0.79051E+00_JPRB ,&
&  0.78632E+00_JPRB, 0.78494E+00_JPRB, 0.79094E+00_JPRB, 0.79886E+00_JPRB, 0.80914E+00_JPRB, 0.82046E+00_JPRB, 0.83538E+00_JPRB ,&
&  0.83774E+00_JPRB, 0.65156E+00_JPRB, 0.67680E+00_JPRB, 0.75324E+00_JPRB, 0.80377E+00_JPRB, 0.86882E+00_JPRB, 0.93101E+00_JPRB ,&
&  0.95649E+00_JPRB, 0.98327E+00_JPRB, 0.97996E+00_JPRB, 0.96864E+00_JPRB, 0.96836E+00_JPRB, 0.95300E+00_JPRB, 0.91717E+00_JPRB ,&
&  0.89015E+00_JPRB, 0.88874E+00_JPRB, 0.90130E+00_JPRB, 0.90992E+00_JPRB, 0.90934E+00_JPRB, 0.90333E+00_JPRB, 0.90053E+00_JPRB ,&
&  0.89755E+00_JPRB, 0.89423E+00_JPRB, 0.88935E+00_JPRB, 0.88117E+00_JPRB, 0.88969E+00_JPRB, 0.91767E+00_JPRB, 0.95560E+00_JPRB ,&
&  0.97475E+00_JPRB, 0.97763E+00_JPRB, 0.97284E+00_JPRB, 0.96161E+00_JPRB, 0.95876E+00_JPRB, 0.97217E+00_JPRB, 0.98437E+00_JPRB ,&
&  0.98814E+00_JPRB, 0.98939E+00_JPRB, 0.99734E+00_JPRB, 0.10013E+01_JPRB, 0.10104E+01_JPRB, 0.10198E+01_JPRB, 0.10359E+01_JPRB ,&
&  0.10466E+01_JPRB, 0.10435E+01_JPRB, 0.10383E+01_JPRB, 0.10264E+01_JPRB, 0.10071E+01_JPRB, 0.99431E+00_JPRB, 0.98989E+00_JPRB ,&
&  0.98624E+00_JPRB, 0.99298E+00_JPRB, 0.10116E+01_JPRB, 0.10331E+01_JPRB, 0.10386E+01_JPRB, 0.10384E+01_JPRB, 0.10392E+01_JPRB ,&
&  0.10443E+01_JPRB, 0.10458E+01_JPRB, 0.10511E+01_JPRB, 0.99500E+00_JPRB, 0.10335E+01_JPRB, 0.10297E+01_JPRB, 0.10071E+01_JPRB ,&
&  0.98025E+00_JPRB, 0.94762E+00_JPRB, 0.92941E+00_JPRB, 0.77328E+00_JPRB, 0.83566E+00_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.82506E+00_JPRB, 0.84660E+00_JPRB, 0.85282E+00_JPRB, 0.86695E+00_JPRB, 0.87660E+00_JPRB, 0.88324E+00_JPRB, 0.89285E+00_JPRB ,&
&  0.89933E+00_JPRB, 0.89605E+00_JPRB, 0.88993E+00_JPRB, 0.87698E+00_JPRB, 0.85526E+00_JPRB, 0.83196E+00_JPRB, 0.81318E+00_JPRB ,&
&  0.80016E+00_JPRB, 0.79936E+00_JPRB, 0.79280E+00_JPRB, 0.78371E+00_JPRB, 0.77826E+00_JPRB, 0.77543E+00_JPRB, 0.77793E+00_JPRB ,&
&  0.78603E+00_JPRB, 0.79041E+00_JPRB, 0.78584E+00_JPRB, 0.78515E+00_JPRB, 0.79457E+00_JPRB, 0.81122E+00_JPRB, 0.81937E+00_JPRB ,&
&  0.82171E+00_JPRB, 0.62669E+00_JPRB, 0.66693E+00_JPRB, 0.71398E+00_JPRB, 0.75190E+00_JPRB, 0.80793E+00_JPRB, 0.87071E+00_JPRB ,&
&  0.91166E+00_JPRB, 0.93753E+00_JPRB, 0.94047E+00_JPRB, 0.93268E+00_JPRB, 0.91535E+00_JPRB, 0.89923E+00_JPRB, 0.88094E+00_JPRB ,&
&  0.86579E+00_JPRB, 0.86235E+00_JPRB, 0.86795E+00_JPRB, 0.87358E+00_JPRB, 0.87639E+00_JPRB, 0.88100E+00_JPRB, 0.87605E+00_JPRB ,&
&  0.87476E+00_JPRB, 0.87229E+00_JPRB, 0.87128E+00_JPRB, 0.87230E+00_JPRB, 0.87328E+00_JPRB, 0.87754E+00_JPRB, 0.88388E+00_JPRB ,&
&  0.88101E+00_JPRB, 0.88173E+00_JPRB, 0.88172E+00_JPRB, 0.87913E+00_JPRB, 0.88434E+00_JPRB, 0.89617E+00_JPRB, 0.91334E+00_JPRB ,&
&  0.93345E+00_JPRB, 0.93745E+00_JPRB, 0.94408E+00_JPRB, 0.95251E+00_JPRB, 0.96534E+00_JPRB, 0.97943E+00_JPRB, 0.99663E+00_JPRB ,&
&  0.10054E+01_JPRB, 0.10024E+01_JPRB, 0.99953E+00_JPRB, 0.98972E+00_JPRB, 0.98780E+00_JPRB, 0.97106E+00_JPRB, 0.95148E+00_JPRB ,&
&  0.94848E+00_JPRB, 0.94940E+00_JPRB, 0.95218E+00_JPRB, 0.96342E+00_JPRB, 0.96966E+00_JPRB, 0.96792E+00_JPRB, 0.97352E+00_JPRB ,&
&  0.98297E+00_JPRB, 0.98883E+00_JPRB, 0.98387E+00_JPRB, 0.98165E+00_JPRB, 0.96841E+00_JPRB, 0.95648E+00_JPRB, 0.93324E+00_JPRB ,&
&  0.90478E+00_JPRB, 0.88319E+00_JPRB, 0.69133E+00_JPRB, 0.77485E+00_JPRB, 0.79715E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.82451E+00_JPRB, 0.83032E+00_JPRB, 0.83373E+00_JPRB, 0.83842E+00_JPRB, 0.83991E+00_JPRB, 0.83935E+00_JPRB, 0.84452E+00_JPRB ,&
&  0.85023E+00_JPRB, 0.84644E+00_JPRB, 0.83914E+00_JPRB, 0.83698E+00_JPRB, 0.82849E+00_JPRB, 0.81586E+00_JPRB, 0.80595E+00_JPRB ,&
&  0.79331E+00_JPRB, 0.78897E+00_JPRB, 0.78651E+00_JPRB, 0.78369E+00_JPRB, 0.77764E+00_JPRB, 0.76988E+00_JPRB, 0.77384E+00_JPRB ,&
&  0.78089E+00_JPRB, 0.78832E+00_JPRB, 0.79068E+00_JPRB, 0.79523E+00_JPRB, 0.80182E+00_JPRB, 0.80917E+00_JPRB, 0.82218E+00_JPRB ,&
&  0.71195E+00_JPRB, 0.58693E+00_JPRB, 0.63825E+00_JPRB, 0.69032E+00_JPRB, 0.73444E+00_JPRB, 0.80089E+00_JPRB, 0.84873E+00_JPRB ,&
&  0.88542E+00_JPRB, 0.91699E+00_JPRB, 0.92416E+00_JPRB, 0.90299E+00_JPRB, 0.88107E+00_JPRB, 0.87357E+00_JPRB, 0.87253E+00_JPRB ,&
&  0.86960E+00_JPRB, 0.86960E+00_JPRB, 0.87490E+00_JPRB, 0.88030E+00_JPRB, 0.88553E+00_JPRB, 0.89209E+00_JPRB, 0.89445E+00_JPRB ,&
&  0.89249E+00_JPRB, 0.88039E+00_JPRB, 0.86823E+00_JPRB, 0.84952E+00_JPRB, 0.83387E+00_JPRB, 0.83104E+00_JPRB, 0.82725E+00_JPRB ,&
&  0.81737E+00_JPRB, 0.81534E+00_JPRB, 0.82031E+00_JPRB, 0.82759E+00_JPRB, 0.83970E+00_JPRB, 0.85328E+00_JPRB, 0.87289E+00_JPRB ,&
&  0.89523E+00_JPRB, 0.91279E+00_JPRB, 0.92582E+00_JPRB, 0.94025E+00_JPRB, 0.94728E+00_JPRB, 0.95595E+00_JPRB, 0.97221E+00_JPRB ,&
&  0.98523E+00_JPRB, 0.97449E+00_JPRB, 0.96419E+00_JPRB, 0.95395E+00_JPRB, 0.95060E+00_JPRB, 0.94845E+00_JPRB, 0.93913E+00_JPRB ,&
&  0.93012E+00_JPRB, 0.92450E+00_JPRB, 0.92217E+00_JPRB, 0.92669E+00_JPRB, 0.92452E+00_JPRB, 0.91794E+00_JPRB, 0.91424E+00_JPRB ,&
&  0.91120E+00_JPRB, 0.90129E+00_JPRB, 0.88997E+00_JPRB, 0.88387E+00_JPRB, 0.87331E+00_JPRB, 0.86557E+00_JPRB, 0.85710E+00_JPRB ,&
&  0.84954E+00_JPRB, 0.83251E+00_JPRB, 0.83445E+00_JPRB, 0.82605E+00_JPRB, 0.82131E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.80560E+00_JPRB, 0.80846E+00_JPRB, 0.81242E+00_JPRB, 0.81289E+00_JPRB, 0.80940E+00_JPRB, 0.80771E+00_JPRB, 0.81024E+00_JPRB ,&
&  0.81251E+00_JPRB, 0.80558E+00_JPRB, 0.79848E+00_JPRB, 0.79326E+00_JPRB, 0.79166E+00_JPRB, 0.78397E+00_JPRB, 0.77189E+00_JPRB ,&
&  0.76350E+00_JPRB, 0.76323E+00_JPRB, 0.76917E+00_JPRB, 0.77270E+00_JPRB, 0.76467E+00_JPRB, 0.75609E+00_JPRB, 0.75350E+00_JPRB ,&
&  0.75498E+00_JPRB, 0.76147E+00_JPRB, 0.77212E+00_JPRB, 0.77867E+00_JPRB, 0.78792E+00_JPRB, 0.80255E+00_JPRB, 0.80434E+00_JPRB ,&
&  0.80281E+00_JPRB, 0.66608E+00_JPRB, 0.70389E+00_JPRB, 0.71911E+00_JPRB, 0.74887E+00_JPRB, 0.79138E+00_JPRB, 0.82582E+00_JPRB ,&
&  0.85855E+00_JPRB, 0.87712E+00_JPRB, 0.88539E+00_JPRB, 0.87908E+00_JPRB, 0.86429E+00_JPRB, 0.85087E+00_JPRB, 0.84711E+00_JPRB ,&
&  0.84577E+00_JPRB, 0.85268E+00_JPRB, 0.86680E+00_JPRB, 0.87680E+00_JPRB, 0.88051E+00_JPRB, 0.88172E+00_JPRB, 0.88431E+00_JPRB ,&
&  0.88135E+00_JPRB, 0.86564E+00_JPRB, 0.84337E+00_JPRB, 0.81633E+00_JPRB, 0.79528E+00_JPRB, 0.78611E+00_JPRB, 0.77578E+00_JPRB ,&
&  0.76094E+00_JPRB, 0.75399E+00_JPRB, 0.75612E+00_JPRB, 0.76227E+00_JPRB, 0.77403E+00_JPRB, 0.78707E+00_JPRB, 0.79802E+00_JPRB ,&
&  0.80985E+00_JPRB, 0.82762E+00_JPRB, 0.84665E+00_JPRB, 0.86463E+00_JPRB, 0.88161E+00_JPRB, 0.89619E+00_JPRB, 0.90645E+00_JPRB ,&
&  0.91225E+00_JPRB, 0.90854E+00_JPRB, 0.89568E+00_JPRB, 0.88596E+00_JPRB, 0.88216E+00_JPRB, 0.88654E+00_JPRB, 0.88747E+00_JPRB ,&
&  0.87938E+00_JPRB, 0.87264E+00_JPRB, 0.86928E+00_JPRB, 0.85841E+00_JPRB, 0.84613E+00_JPRB, 0.83461E+00_JPRB, 0.82276E+00_JPRB ,&
&  0.81372E+00_JPRB, 0.81017E+00_JPRB, 0.80699E+00_JPRB, 0.80087E+00_JPRB, 0.79080E+00_JPRB, 0.78568E+00_JPRB, 0.78162E+00_JPRB ,&
&  0.77752E+00_JPRB, 0.77248E+00_JPRB, 0.77319E+00_JPRB, 0.78246E+00_JPRB, 0.79620E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.77134E+00_JPRB, 0.77642E+00_JPRB, 0.78491E+00_JPRB, 0.79487E+00_JPRB, 0.79814E+00_JPRB, 0.79782E+00_JPRB, 0.79596E+00_JPRB ,&
&  0.79721E+00_JPRB, 0.78961E+00_JPRB, 0.78419E+00_JPRB, 0.78106E+00_JPRB, 0.78071E+00_JPRB, 0.77782E+00_JPRB, 0.76910E+00_JPRB ,&
&  0.76183E+00_JPRB, 0.75924E+00_JPRB, 0.76403E+00_JPRB, 0.76577E+00_JPRB, 0.75085E+00_JPRB, 0.74618E+00_JPRB, 0.73864E+00_JPRB ,&
&  0.73770E+00_JPRB, 0.73814E+00_JPRB, 0.74140E+00_JPRB, 0.74789E+00_JPRB, 0.76466E+00_JPRB, 0.77664E+00_JPRB, 0.78325E+00_JPRB ,&
&  0.78190E+00_JPRB, 0.70220E+00_JPRB, 0.72982E+00_JPRB, 0.74253E+00_JPRB, 0.76538E+00_JPRB, 0.78803E+00_JPRB, 0.80741E+00_JPRB ,&
&  0.83264E+00_JPRB, 0.84625E+00_JPRB, 0.85149E+00_JPRB, 0.85221E+00_JPRB, 0.85370E+00_JPRB, 0.85390E+00_JPRB, 0.84739E+00_JPRB ,&
&  0.84309E+00_JPRB, 0.85025E+00_JPRB, 0.86664E+00_JPRB, 0.87499E+00_JPRB, 0.87908E+00_JPRB, 0.88125E+00_JPRB, 0.87789E+00_JPRB ,&
&  0.87086E+00_JPRB, 0.85657E+00_JPRB, 0.83272E+00_JPRB, 0.80411E+00_JPRB, 0.77179E+00_JPRB, 0.74950E+00_JPRB, 0.73740E+00_JPRB ,&
&  0.73027E+00_JPRB, 0.73157E+00_JPRB, 0.73432E+00_JPRB, 0.73768E+00_JPRB, 0.74279E+00_JPRB, 0.75319E+00_JPRB, 0.75876E+00_JPRB ,&
&  0.76782E+00_JPRB, 0.77634E+00_JPRB, 0.78880E+00_JPRB, 0.80846E+00_JPRB, 0.82792E+00_JPRB, 0.83851E+00_JPRB, 0.83860E+00_JPRB ,&
&  0.84369E+00_JPRB, 0.84606E+00_JPRB, 0.84191E+00_JPRB, 0.84116E+00_JPRB, 0.83683E+00_JPRB, 0.83781E+00_JPRB, 0.83496E+00_JPRB ,&
&  0.82612E+00_JPRB, 0.81952E+00_JPRB, 0.81149E+00_JPRB, 0.79612E+00_JPRB, 0.78263E+00_JPRB, 0.77281E+00_JPRB, 0.76465E+00_JPRB ,&
&  0.76357E+00_JPRB, 0.76730E+00_JPRB, 0.77256E+00_JPRB, 0.76753E+00_JPRB, 0.75316E+00_JPRB, 0.74403E+00_JPRB, 0.73343E+00_JPRB ,&
&  0.72621E+00_JPRB, 0.72830E+00_JPRB, 0.72842E+00_JPRB, 0.73588E+00_JPRB, 0.75472E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.73021E+00_JPRB, 0.74234E+00_JPRB, 0.75752E+00_JPRB, 0.77316E+00_JPRB, 0.78298E+00_JPRB, 0.78910E+00_JPRB, 0.78714E+00_JPRB ,&
&  0.77673E+00_JPRB, 0.76639E+00_JPRB, 0.76837E+00_JPRB, 0.76857E+00_JPRB, 0.76783E+00_JPRB, 0.76630E+00_JPRB, 0.75989E+00_JPRB ,&
&  0.75059E+00_JPRB, 0.74265E+00_JPRB, 0.73939E+00_JPRB, 0.73655E+00_JPRB, 0.72688E+00_JPRB, 0.71945E+00_JPRB, 0.71147E+00_JPRB ,&
&  0.71153E+00_JPRB, 0.71439E+00_JPRB, 0.71756E+00_JPRB, 0.71812E+00_JPRB, 0.72646E+00_JPRB, 0.73708E+00_JPRB, 0.74166E+00_JPRB ,&
&  0.73629E+00_JPRB, 0.74242E+00_JPRB, 0.66046E+00_JPRB, 0.73038E+00_JPRB, 0.75578E+00_JPRB, 0.77102E+00_JPRB, 0.78620E+00_JPRB ,&
&  0.79607E+00_JPRB, 0.80720E+00_JPRB, 0.81506E+00_JPRB, 0.81928E+00_JPRB, 0.83604E+00_JPRB, 0.85026E+00_JPRB, 0.85416E+00_JPRB ,&
&  0.85565E+00_JPRB, 0.85486E+00_JPRB, 0.85623E+00_JPRB, 0.85630E+00_JPRB, 0.86069E+00_JPRB, 0.86050E+00_JPRB, 0.85445E+00_JPRB ,&
&  0.84631E+00_JPRB, 0.83458E+00_JPRB, 0.81725E+00_JPRB, 0.79716E+00_JPRB, 0.76551E+00_JPRB, 0.73782E+00_JPRB, 0.72019E+00_JPRB ,&
&  0.70590E+00_JPRB, 0.70207E+00_JPRB, 0.69740E+00_JPRB, 0.69278E+00_JPRB, 0.69181E+00_JPRB, 0.69527E+00_JPRB, 0.69555E+00_JPRB ,&
&  0.70301E+00_JPRB, 0.71215E+00_JPRB, 0.71648E+00_JPRB, 0.72391E+00_JPRB, 0.73359E+00_JPRB, 0.74766E+00_JPRB, 0.76092E+00_JPRB ,&
&  0.77322E+00_JPRB, 0.78161E+00_JPRB, 0.78271E+00_JPRB, 0.78248E+00_JPRB, 0.77983E+00_JPRB, 0.77840E+00_JPRB, 0.77112E+00_JPRB ,&
&  0.76529E+00_JPRB, 0.76211E+00_JPRB, 0.75558E+00_JPRB, 0.74352E+00_JPRB, 0.73608E+00_JPRB, 0.72980E+00_JPRB, 0.72820E+00_JPRB ,&
&  0.72905E+00_JPRB, 0.73036E+00_JPRB, 0.72794E+00_JPRB, 0.72875E+00_JPRB, 0.72253E+00_JPRB, 0.70221E+00_JPRB, 0.68617E+00_JPRB ,&
&  0.68510E+00_JPRB, 0.69023E+00_JPRB, 0.69498E+00_JPRB, 0.70388E+00_JPRB, 0.71839E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.74975E+00_JPRB, 0.75571E+00_JPRB, 0.76453E+00_JPRB, 0.77749E+00_JPRB, 0.79109E+00_JPRB, 0.79991E+00_JPRB, 0.79946E+00_JPRB ,&
&  0.78862E+00_JPRB, 0.77564E+00_JPRB, 0.77157E+00_JPRB, 0.77127E+00_JPRB, 0.76860E+00_JPRB, 0.76740E+00_JPRB, 0.76308E+00_JPRB ,&
&  0.75223E+00_JPRB, 0.73553E+00_JPRB, 0.72772E+00_JPRB, 0.71405E+00_JPRB, 0.70115E+00_JPRB, 0.69066E+00_JPRB, 0.69093E+00_JPRB ,&
&  0.69980E+00_JPRB, 0.70999E+00_JPRB, 0.72255E+00_JPRB, 0.73691E+00_JPRB, 0.74480E+00_JPRB, 0.74366E+00_JPRB, 0.74067E+00_JPRB ,&
&  0.73828E+00_JPRB, 0.73559E+00_JPRB, 0.74665E+00_JPRB, 0.75581E+00_JPRB, 0.76203E+00_JPRB, 0.77343E+00_JPRB, 0.78266E+00_JPRB ,&
&  0.79031E+00_JPRB, 0.80143E+00_JPRB, 0.81989E+00_JPRB, 0.83347E+00_JPRB, 0.84320E+00_JPRB, 0.85367E+00_JPRB, 0.85752E+00_JPRB ,&
&  0.86391E+00_JPRB, 0.87515E+00_JPRB, 0.87741E+00_JPRB, 0.86842E+00_JPRB, 0.85399E+00_JPRB, 0.84463E+00_JPRB, 0.83927E+00_JPRB ,&
&  0.82981E+00_JPRB, 0.81039E+00_JPRB, 0.79572E+00_JPRB, 0.78405E+00_JPRB, 0.76515E+00_JPRB, 0.74493E+00_JPRB, 0.72122E+00_JPRB ,&
&  0.69430E+00_JPRB, 0.67429E+00_JPRB, 0.65882E+00_JPRB, 0.64489E+00_JPRB, 0.64181E+00_JPRB, 0.63368E+00_JPRB, 0.62770E+00_JPRB ,&
&  0.63281E+00_JPRB, 0.65152E+00_JPRB, 0.66549E+00_JPRB, 0.67043E+00_JPRB, 0.67301E+00_JPRB, 0.68885E+00_JPRB, 0.70806E+00_JPRB ,&
&  0.72300E+00_JPRB, 0.72917E+00_JPRB, 0.72506E+00_JPRB, 0.72719E+00_JPRB, 0.73205E+00_JPRB, 0.73613E+00_JPRB, 0.72990E+00_JPRB ,&
&  0.72106E+00_JPRB, 0.71521E+00_JPRB, 0.71377E+00_JPRB, 0.70995E+00_JPRB, 0.71132E+00_JPRB, 0.71112E+00_JPRB, 0.70833E+00_JPRB ,&
&  0.70268E+00_JPRB, 0.69348E+00_JPRB, 0.68160E+00_JPRB, 0.67433E+00_JPRB, 0.66369E+00_JPRB, 0.64789E+00_JPRB, 0.64871E+00_JPRB ,&
&  0.65996E+00_JPRB, 0.67814E+00_JPRB, 0.69539E+00_JPRB, 0.71560E+00_JPRB, 0.73903E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.77652E+00_JPRB, 0.78434E+00_JPRB, 0.78978E+00_JPRB, 0.79367E+00_JPRB, 0.80205E+00_JPRB, 0.81268E+00_JPRB, 0.81242E+00_JPRB ,&
&  0.79778E+00_JPRB, 0.78422E+00_JPRB, 0.77489E+00_JPRB, 0.77242E+00_JPRB, 0.77420E+00_JPRB, 0.77653E+00_JPRB, 0.77081E+00_JPRB ,&
&  0.74995E+00_JPRB, 0.72764E+00_JPRB, 0.71397E+00_JPRB, 0.70039E+00_JPRB, 0.69202E+00_JPRB, 0.68675E+00_JPRB, 0.69378E+00_JPRB ,&
&  0.70413E+00_JPRB, 0.72033E+00_JPRB, 0.73356E+00_JPRB, 0.74318E+00_JPRB, 0.74716E+00_JPRB, 0.73595E+00_JPRB, 0.73744E+00_JPRB ,&
&  0.74475E+00_JPRB, 0.74714E+00_JPRB, 0.76237E+00_JPRB, 0.76200E+00_JPRB, 0.74901E+00_JPRB, 0.73860E+00_JPRB, 0.72825E+00_JPRB ,&
&  0.72562E+00_JPRB, 0.73694E+00_JPRB, 0.76223E+00_JPRB, 0.79873E+00_JPRB, 0.81969E+00_JPRB, 0.82758E+00_JPRB, 0.82468E+00_JPRB ,&
&  0.82759E+00_JPRB, 0.83958E+00_JPRB, 0.84628E+00_JPRB, 0.83799E+00_JPRB, 0.81962E+00_JPRB, 0.80454E+00_JPRB, 0.79120E+00_JPRB ,&
&  0.78120E+00_JPRB, 0.76921E+00_JPRB, 0.76108E+00_JPRB, 0.75102E+00_JPRB, 0.72920E+00_JPRB, 0.70337E+00_JPRB, 0.68084E+00_JPRB ,&
&  0.66417E+00_JPRB, 0.64775E+00_JPRB, 0.62776E+00_JPRB, 0.62159E+00_JPRB, 0.62578E+00_JPRB, 0.60300E+00_JPRB, 0.57849E+00_JPRB ,&
&  0.58244E+00_JPRB, 0.61404E+00_JPRB, 0.63876E+00_JPRB, 0.64885E+00_JPRB, 0.64981E+00_JPRB, 0.65414E+00_JPRB, 0.65751E+00_JPRB ,&
&  0.66391E+00_JPRB, 0.67242E+00_JPRB, 0.67886E+00_JPRB, 0.68985E+00_JPRB, 0.70123E+00_JPRB, 0.69952E+00_JPRB, 0.68514E+00_JPRB ,&
&  0.66932E+00_JPRB, 0.66009E+00_JPRB, 0.65601E+00_JPRB, 0.65792E+00_JPRB, 0.66113E+00_JPRB, 0.65905E+00_JPRB, 0.65321E+00_JPRB ,&
&  0.64149E+00_JPRB, 0.62305E+00_JPRB, 0.60741E+00_JPRB, 0.59865E+00_JPRB, 0.59816E+00_JPRB, 0.60477E+00_JPRB, 0.62101E+00_JPRB ,&
&  0.65227E+00_JPRB, 0.69985E+00_JPRB, 0.73502E+00_JPRB, 0.75463E+00_JPRB, 0.75958E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.80551E+00_JPRB, 0.81352E+00_JPRB, 0.82314E+00_JPRB, 0.83611E+00_JPRB, 0.84789E+00_JPRB, 0.84455E+00_JPRB, 0.83197E+00_JPRB ,&
&  0.80827E+00_JPRB, 0.79943E+00_JPRB, 0.80421E+00_JPRB, 0.80668E+00_JPRB, 0.80843E+00_JPRB, 0.80622E+00_JPRB, 0.79013E+00_JPRB ,&
&  0.75286E+00_JPRB, 0.72307E+00_JPRB, 0.70883E+00_JPRB, 0.70148E+00_JPRB, 0.69920E+00_JPRB, 0.70632E+00_JPRB, 0.71491E+00_JPRB ,&
&  0.73198E+00_JPRB, 0.74222E+00_JPRB, 0.75442E+00_JPRB, 0.76398E+00_JPRB, 0.76678E+00_JPRB, 0.77080E+00_JPRB, 0.77200E+00_JPRB ,&
&  0.75806E+00_JPRB, 0.75840E+00_JPRB, 0.76007E+00_JPRB, 0.75061E+00_JPRB, 0.72944E+00_JPRB, 0.70817E+00_JPRB, 0.68196E+00_JPRB ,&
&  0.66463E+00_JPRB, 0.65594E+00_JPRB, 0.68148E+00_JPRB, 0.74315E+00_JPRB, 0.78650E+00_JPRB, 0.79460E+00_JPRB, 0.78680E+00_JPRB ,&
&  0.78888E+00_JPRB, 0.79636E+00_JPRB, 0.79986E+00_JPRB, 0.78832E+00_JPRB, 0.76857E+00_JPRB, 0.75118E+00_JPRB, 0.73450E+00_JPRB ,&
&  0.72872E+00_JPRB, 0.72770E+00_JPRB, 0.71011E+00_JPRB, 0.67778E+00_JPRB, 0.65017E+00_JPRB, 0.63068E+00_JPRB, 0.62047E+00_JPRB ,&
&  0.61432E+00_JPRB, 0.60481E+00_JPRB, 0.59897E+00_JPRB, 0.60392E+00_JPRB, 0.58413E+00_JPRB, 0.56640E+00_JPRB, 0.57050E+00_JPRB ,&
&  0.58725E+00_JPRB, 0.59297E+00_JPRB, 0.60232E+00_JPRB, 0.60549E+00_JPRB, 0.60249E+00_JPRB, 0.59878E+00_JPRB, 0.59990E+00_JPRB ,&
&  0.60849E+00_JPRB, 0.61829E+00_JPRB, 0.62546E+00_JPRB, 0.63027E+00_JPRB, 0.63514E+00_JPRB, 0.62316E+00_JPRB, 0.60177E+00_JPRB ,&
&  0.58387E+00_JPRB, 0.58021E+00_JPRB, 0.58433E+00_JPRB, 0.58338E+00_JPRB, 0.58167E+00_JPRB, 0.58176E+00_JPRB, 0.58039E+00_JPRB ,&
&  0.57754E+00_JPRB, 0.57012E+00_JPRB, 0.56133E+00_JPRB, 0.56150E+00_JPRB, 0.57272E+00_JPRB, 0.59522E+00_JPRB, 0.63026E+00_JPRB ,&
&  0.68228E+00_JPRB, 0.72157E+00_JPRB, 0.75948E+00_JPRB, 0.77487E+00_JPRB, 0.77823E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.84191E+00_JPRB, 0.86342E+00_JPRB, 0.88200E+00_JPRB, 0.89456E+00_JPRB, 0.89621E+00_JPRB, 0.88924E+00_JPRB, 0.87324E+00_JPRB ,&
&  0.84314E+00_JPRB, 0.82601E+00_JPRB, 0.83496E+00_JPRB, 0.84968E+00_JPRB, 0.85865E+00_JPRB, 0.85863E+00_JPRB, 0.82706E+00_JPRB ,&
&  0.77273E+00_JPRB, 0.75224E+00_JPRB, 0.74935E+00_JPRB, 0.75905E+00_JPRB, 0.77740E+00_JPRB, 0.78447E+00_JPRB, 0.77243E+00_JPRB ,&
&  0.76017E+00_JPRB, 0.76184E+00_JPRB, 0.76474E+00_JPRB, 0.77174E+00_JPRB, 0.76113E+00_JPRB, 0.74019E+00_JPRB, 0.73125E+00_JPRB ,&
&  0.72863E+00_JPRB, 0.72119E+00_JPRB, 0.71349E+00_JPRB, 0.66466E+00_JPRB, 0.68889E+00_JPRB, 0.67739E+00_JPRB, 0.65923E+00_JPRB ,&
&  0.64255E+00_JPRB, 0.64517E+00_JPRB, 0.68692E+00_JPRB, 0.73340E+00_JPRB, 0.74487E+00_JPRB, 0.74761E+00_JPRB, 0.74279E+00_JPRB ,&
&  0.73943E+00_JPRB, 0.73981E+00_JPRB, 0.74118E+00_JPRB, 0.73299E+00_JPRB, 0.71256E+00_JPRB, 0.68622E+00_JPRB, 0.65734E+00_JPRB ,&
&  0.63982E+00_JPRB, 0.61478E+00_JPRB, 0.57123E+00_JPRB, 0.55093E+00_JPRB, 0.54897E+00_JPRB, 0.55690E+00_JPRB, 0.56073E+00_JPRB ,&
&  0.55772E+00_JPRB, 0.55040E+00_JPRB, 0.55446E+00_JPRB, 0.55168E+00_JPRB, 0.53006E+00_JPRB, 0.51706E+00_JPRB, 0.51124E+00_JPRB ,&
&  0.49999E+00_JPRB, 0.49903E+00_JPRB, 0.50384E+00_JPRB, 0.51554E+00_JPRB, 0.52588E+00_JPRB, 0.53990E+00_JPRB, 0.54567E+00_JPRB ,&
&  0.54788E+00_JPRB, 0.54438E+00_JPRB, 0.53845E+00_JPRB, 0.52634E+00_JPRB, 0.50777E+00_JPRB, 0.50064E+00_JPRB, 0.50199E+00_JPRB ,&
&  0.51069E+00_JPRB, 0.51637E+00_JPRB, 0.51556E+00_JPRB, 0.51427E+00_JPRB, 0.50259E+00_JPRB, 0.48988E+00_JPRB, 0.48017E+00_JPRB ,&
&  0.47558E+00_JPRB, 0.47839E+00_JPRB, 0.49361E+00_JPRB, 0.51834E+00_JPRB, 0.55016E+00_JPRB, 0.58876E+00_JPRB, 0.63347E+00_JPRB ,&
&  0.66506E+00_JPRB, 0.68364E+00_JPRB, 0.70797E+00_JPRB, 0.75548E+00_JPRB, 0.80134E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.84550E+00_JPRB, 0.87201E+00_JPRB, 0.90120E+00_JPRB, 0.91796E+00_JPRB, 0.91868E+00_JPRB, 0.91532E+00_JPRB, 0.90010E+00_JPRB ,&
&  0.86904E+00_JPRB, 0.84356E+00_JPRB, 0.83923E+00_JPRB, 0.84306E+00_JPRB, 0.85484E+00_JPRB, 0.87358E+00_JPRB, 0.87524E+00_JPRB ,&
&  0.84879E+00_JPRB, 0.82959E+00_JPRB, 0.82007E+00_JPRB, 0.81319E+00_JPRB, 0.79636E+00_JPRB, 0.77186E+00_JPRB, 0.75591E+00_JPRB ,&
&  0.75147E+00_JPRB, 0.75154E+00_JPRB, 0.74273E+00_JPRB, 0.72884E+00_JPRB, 0.69733E+00_JPRB, 0.65847E+00_JPRB, 0.62808E+00_JPRB ,&
&  0.61767E+00_JPRB, 0.60884E+00_JPRB, 0.56819E+00_JPRB, 0.58488E+00_JPRB, 0.58874E+00_JPRB, 0.58911E+00_JPRB, 0.58677E+00_JPRB ,&
&  0.59921E+00_JPRB, 0.63266E+00_JPRB, 0.66574E+00_JPRB, 0.67556E+00_JPRB, 0.68507E+00_JPRB, 0.68687E+00_JPRB, 0.68846E+00_JPRB ,&
&  0.69309E+00_JPRB, 0.68509E+00_JPRB, 0.65545E+00_JPRB, 0.61152E+00_JPRB, 0.56817E+00_JPRB, 0.53443E+00_JPRB, 0.50569E+00_JPRB ,&
&  0.48588E+00_JPRB, 0.47344E+00_JPRB, 0.46373E+00_JPRB, 0.46038E+00_JPRB, 0.46804E+00_JPRB, 0.48310E+00_JPRB, 0.47809E+00_JPRB ,&
&  0.46004E+00_JPRB, 0.44348E+00_JPRB, 0.42150E+00_JPRB, 0.39163E+00_JPRB, 0.36120E+00_JPRB, 0.32260E+00_JPRB, 0.30279E+00_JPRB ,&
&  0.30737E+00_JPRB, 0.34674E+00_JPRB, 0.39861E+00_JPRB, 0.41805E+00_JPRB, 0.44524E+00_JPRB, 0.47183E+00_JPRB, 0.46140E+00_JPRB ,&
&  0.41419E+00_JPRB, 0.34541E+00_JPRB, 0.32238E+00_JPRB, 0.30979E+00_JPRB, 0.30286E+00_JPRB, 0.29787E+00_JPRB, 0.29684E+00_JPRB ,&
&  0.30096E+00_JPRB, 0.31216E+00_JPRB, 0.32154E+00_JPRB, 0.31833E+00_JPRB, 0.30794E+00_JPRB, 0.30162E+00_JPRB, 0.30228E+00_JPRB ,&
&  0.31163E+00_JPRB, 0.33982E+00_JPRB, 0.40320E+00_JPRB, 0.51885E+00_JPRB, 0.55495E+00_JPRB, 0.58806E+00_JPRB, 0.62632E+00_JPRB ,&
&  0.66293E+00_JPRB, 0.69613E+00_JPRB, 0.72445E+00_JPRB, 0.75939E+00_JPRB, 0.80631E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.78466E+00_JPRB, 0.80285E+00_JPRB, 0.82403E+00_JPRB, 0.83525E+00_JPRB, 0.83631E+00_JPRB, 0.83334E+00_JPRB, 0.82464E+00_JPRB ,&
&  0.80971E+00_JPRB, 0.79175E+00_JPRB, 0.77853E+00_JPRB, 0.76955E+00_JPRB, 0.76560E+00_JPRB, 0.76854E+00_JPRB, 0.77396E+00_JPRB ,&
&  0.77326E+00_JPRB, 0.76926E+00_JPRB, 0.75717E+00_JPRB, 0.74142E+00_JPRB, 0.72477E+00_JPRB, 0.71190E+00_JPRB, 0.70207E+00_JPRB ,&
&  0.69243E+00_JPRB, 0.67781E+00_JPRB, 0.65491E+00_JPRB, 0.61961E+00_JPRB, 0.57785E+00_JPRB, 0.53967E+00_JPRB, 0.51029E+00_JPRB ,&
&  0.48514E+00_JPRB, 0.41816E+00_JPRB, 0.38112E+00_JPRB, 0.39301E+00_JPRB, 0.48531E+00_JPRB, 0.51639E+00_JPRB, 0.54068E+00_JPRB ,&
&  0.57775E+00_JPRB, 0.61715E+00_JPRB, 0.63605E+00_JPRB, 0.63660E+00_JPRB, 0.63113E+00_JPRB, 0.61856E+00_JPRB, 0.59894E+00_JPRB ,&
&  0.57401E+00_JPRB, 0.54348E+00_JPRB, 0.51750E+00_JPRB, 0.48850E+00_JPRB, 0.45868E+00_JPRB, 0.43668E+00_JPRB, 0.42137E+00_JPRB ,&
&  0.41076E+00_JPRB, 0.40113E+00_JPRB, 0.39350E+00_JPRB, 0.38378E+00_JPRB, 0.37467E+00_JPRB, 0.35368E+00_JPRB, 0.33179E+00_JPRB ,&
&  0.30862E+00_JPRB, 0.29112E+00_JPRB, 0.27253E+00_JPRB, 0.25433E+00_JPRB, 0.23941E+00_JPRB, 0.22970E+00_JPRB, 0.22257E+00_JPRB ,&
&  0.22039E+00_JPRB, 0.22636E+00_JPRB, 0.24514E+00_JPRB, 0.27817E+00_JPRB, 0.31635E+00_JPRB, 0.28737E+00_JPRB, 0.26677E+00_JPRB ,&
&  0.24592E+00_JPRB, 0.23385E+00_JPRB, 0.22920E+00_JPRB, 0.22855E+00_JPRB, 0.22974E+00_JPRB, 0.23196E+00_JPRB, 0.23426E+00_JPRB ,&
&  0.23743E+00_JPRB, 0.24038E+00_JPRB, 0.24348E+00_JPRB, 0.24570E+00_JPRB, 0.24706E+00_JPRB, 0.24967E+00_JPRB, 0.25426E+00_JPRB ,&
&  0.26149E+00_JPRB, 0.27350E+00_JPRB, 0.29327E+00_JPRB, 0.31902E+00_JPRB, 0.34587E+00_JPRB, 0.37022E+00_JPRB, 0.42327E+00_JPRB ,&
&  0.51071E+00_JPRB, 0.62281E+00_JPRB, 0.67967E+00_JPRB, 0.73543E+00_JPRB, 0.76315E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.72262E+00_JPRB, 0.73443E+00_JPRB, 0.74443E+00_JPRB, 0.74639E+00_JPRB, 0.74567E+00_JPRB, 0.74064E+00_JPRB, 0.72858E+00_JPRB ,&
&  0.70885E+00_JPRB, 0.68253E+00_JPRB, 0.65696E+00_JPRB, 0.63660E+00_JPRB, 0.62387E+00_JPRB, 0.62705E+00_JPRB, 0.63575E+00_JPRB ,&
&  0.65329E+00_JPRB, 0.67888E+00_JPRB, 0.68415E+00_JPRB, 0.67756E+00_JPRB, 0.64189E+00_JPRB, 0.60476E+00_JPRB, 0.57038E+00_JPRB ,&
&  0.53985E+00_JPRB, 0.51103E+00_JPRB, 0.50017E+00_JPRB, 0.48134E+00_JPRB, 0.46803E+00_JPRB, 0.45130E+00_JPRB, 0.42774E+00_JPRB ,&
&  0.38933E+00_JPRB, 0.35883E+00_JPRB, 0.34161E+00_JPRB, 0.35463E+00_JPRB, 0.44334E+00_JPRB, 0.50423E+00_JPRB, 0.51399E+00_JPRB ,&
&  0.54855E+00_JPRB, 0.57332E+00_JPRB, 0.56857E+00_JPRB, 0.54881E+00_JPRB, 0.52736E+00_JPRB, 0.50401E+00_JPRB, 0.48475E+00_JPRB ,&
&  0.46559E+00_JPRB, 0.43921E+00_JPRB, 0.38976E+00_JPRB, 0.31972E+00_JPRB, 0.28423E+00_JPRB, 0.27082E+00_JPRB, 0.25641E+00_JPRB ,&
&  0.24555E+00_JPRB, 0.23675E+00_JPRB, 0.23329E+00_JPRB, 0.22994E+00_JPRB, 0.22925E+00_JPRB, 0.22533E+00_JPRB, 0.22194E+00_JPRB ,&
&  0.21757E+00_JPRB, 0.21439E+00_JPRB, 0.20970E+00_JPRB, 0.20509E+00_JPRB, 0.20169E+00_JPRB, 0.20115E+00_JPRB, 0.20092E+00_JPRB ,&
&  0.20196E+00_JPRB, 0.20605E+00_JPRB, 0.21757E+00_JPRB, 0.22393E+00_JPRB, 0.22439E+00_JPRB, 0.21973E+00_JPRB, 0.21252E+00_JPRB ,&
&  0.20594E+00_JPRB, 0.20409E+00_JPRB, 0.20262E+00_JPRB, 0.20311E+00_JPRB, 0.20581E+00_JPRB, 0.20940E+00_JPRB, 0.21235E+00_JPRB ,&
&  0.21466E+00_JPRB, 0.21697E+00_JPRB, 0.21911E+00_JPRB, 0.22147E+00_JPRB, 0.22434E+00_JPRB, 0.22776E+00_JPRB, 0.23247E+00_JPRB ,&
&  0.23752E+00_JPRB, 0.24463E+00_JPRB, 0.25482E+00_JPRB, 0.26955E+00_JPRB, 0.28271E+00_JPRB, 0.29113E+00_JPRB, 0.30243E+00_JPRB ,&
&  0.31839E+00_JPRB, 0.36548E+00_JPRB, 0.49978E+00_JPRB, 0.72562E+00_JPRB, 0.75790E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.64625E+00_JPRB, 0.65757E+00_JPRB, 0.67174E+00_JPRB, 0.67479E+00_JPRB, 0.68490E+00_JPRB, 0.68373E+00_JPRB, 0.66653E+00_JPRB ,&
&  0.63829E+00_JPRB, 0.61177E+00_JPRB, 0.59724E+00_JPRB, 0.57970E+00_JPRB, 0.55205E+00_JPRB, 0.49175E+00_JPRB, 0.46863E+00_JPRB ,&
&  0.45017E+00_JPRB, 0.44981E+00_JPRB, 0.44062E+00_JPRB, 0.44076E+00_JPRB, 0.42977E+00_JPRB, 0.41846E+00_JPRB, 0.40003E+00_JPRB ,&
&  0.38294E+00_JPRB, 0.36584E+00_JPRB, 0.35402E+00_JPRB, 0.34823E+00_JPRB, 0.34601E+00_JPRB, 0.34145E+00_JPRB, 0.32933E+00_JPRB ,&
&  0.32074E+00_JPRB, 0.31904E+00_JPRB, 0.32169E+00_JPRB, 0.33133E+00_JPRB, 0.37496E+00_JPRB, 0.43116E+00_JPRB, 0.43169E+00_JPRB ,&
&  0.45442E+00_JPRB, 0.46803E+00_JPRB, 0.48126E+00_JPRB, 0.47048E+00_JPRB, 0.43798E+00_JPRB, 0.39942E+00_JPRB, 0.34554E+00_JPRB ,&
&  0.29419E+00_JPRB, 0.26539E+00_JPRB, 0.24204E+00_JPRB, 0.22577E+00_JPRB, 0.21475E+00_JPRB, 0.21021E+00_JPRB, 0.20441E+00_JPRB ,&
&  0.20059E+00_JPRB, 0.19769E+00_JPRB, 0.19649E+00_JPRB, 0.19586E+00_JPRB, 0.19515E+00_JPRB, 0.19313E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19312E+00_JPRB, 0.19333E+00_JPRB, 0.19318E+00_JPRB, 0.19403E+00_JPRB, 0.19503E+00_JPRB, 0.19646E+00_JPRB, 0.19776E+00_JPRB ,&
&  0.19970E+00_JPRB, 0.20077E+00_JPRB, 0.20244E+00_JPRB, 0.20424E+00_JPRB, 0.20327E+00_JPRB, 0.20210E+00_JPRB, 0.19931E+00_JPRB ,&
&  0.19620E+00_JPRB, 0.19586E+00_JPRB, 0.19529E+00_JPRB, 0.19583E+00_JPRB, 0.19738E+00_JPRB, 0.19948E+00_JPRB, 0.20227E+00_JPRB ,&
&  0.20496E+00_JPRB, 0.20773E+00_JPRB, 0.20986E+00_JPRB, 0.21199E+00_JPRB, 0.21556E+00_JPRB, 0.21938E+00_JPRB, 0.22425E+00_JPRB ,&
&  0.22930E+00_JPRB, 0.23503E+00_JPRB, 0.24299E+00_JPRB, 0.25328E+00_JPRB, 0.26803E+00_JPRB, 0.28471E+00_JPRB, 0.31051E+00_JPRB ,&
&  0.33464E+00_JPRB, 0.43486E+00_JPRB, 0.67109E+00_JPRB, 0.59930E+00_JPRB, 0.61844E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.60979E+00_JPRB, 0.62623E+00_JPRB, 0.65130E+00_JPRB, 0.64403E+00_JPRB, 0.66422E+00_JPRB, 0.65076E+00_JPRB, 0.62800E+00_JPRB ,&
&  0.58067E+00_JPRB, 0.52378E+00_JPRB, 0.47723E+00_JPRB, 0.42511E+00_JPRB, 0.38459E+00_JPRB, 0.35504E+00_JPRB, 0.34059E+00_JPRB ,&
&  0.33355E+00_JPRB, 0.33137E+00_JPRB, 0.33011E+00_JPRB, 0.32927E+00_JPRB, 0.32462E+00_JPRB, 0.32133E+00_JPRB, 0.31565E+00_JPRB ,&
&  0.30976E+00_JPRB, 0.30539E+00_JPRB, 0.30214E+00_JPRB, 0.30712E+00_JPRB, 0.31328E+00_JPRB, 0.31977E+00_JPRB, 0.32328E+00_JPRB ,&
&  0.32644E+00_JPRB, 0.32784E+00_JPRB, 0.33056E+00_JPRB, 0.32864E+00_JPRB, 0.33973E+00_JPRB, 0.36101E+00_JPRB, 0.37929E+00_JPRB ,&
&  0.39701E+00_JPRB, 0.41182E+00_JPRB, 0.40788E+00_JPRB, 0.38427E+00_JPRB, 0.31865E+00_JPRB, 0.28446E+00_JPRB, 0.25458E+00_JPRB ,&
&  0.23581E+00_JPRB, 0.22191E+00_JPRB, 0.21262E+00_JPRB, 0.20729E+00_JPRB, 0.20311E+00_JPRB, 0.20074E+00_JPRB, 0.19821E+00_JPRB ,&
&  0.19578E+00_JPRB, 0.19428E+00_JPRB, 0.19260E+00_JPRB, 0.19180E+00_JPRB, 0.19089E+00_JPRB, 0.18968E+00_JPRB, 0.18877E+00_JPRB ,&
&  0.18893E+00_JPRB, 0.18875E+00_JPRB, 0.18988E+00_JPRB, 0.19087E+00_JPRB, 0.19280E+00_JPRB, 0.19441E+00_JPRB, 0.19663E+00_JPRB ,&
&  0.19846E+00_JPRB, 0.19996E+00_JPRB, 0.19980E+00_JPRB, 0.19930E+00_JPRB, 0.19840E+00_JPRB, 0.19786E+00_JPRB, 0.19697E+00_JPRB ,&
&  0.19584E+00_JPRB, 0.19599E+00_JPRB, 0.19621E+00_JPRB, 0.19653E+00_JPRB, 0.19759E+00_JPRB, 0.19911E+00_JPRB, 0.20148E+00_JPRB ,&
&  0.20459E+00_JPRB, 0.20787E+00_JPRB, 0.21061E+00_JPRB, 0.21352E+00_JPRB, 0.21728E+00_JPRB, 0.22160E+00_JPRB, 0.22700E+00_JPRB ,&
&  0.23184E+00_JPRB, 0.24007E+00_JPRB, 0.25143E+00_JPRB, 0.26324E+00_JPRB, 0.28089E+00_JPRB, 0.29826E+00_JPRB, 0.32727E+00_JPRB ,&
&  0.37626E+00_JPRB, 0.54874E+00_JPRB, 0.83244E+00_JPRB, 0.60704E+00_JPRB, 0.59559E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.53128E+00_JPRB, 0.54028E+00_JPRB, 0.55452E+00_JPRB, 0.56606E+00_JPRB, 0.59026E+00_JPRB, 0.60386E+00_JPRB, 0.64988E+00_JPRB ,&
&  0.60412E+00_JPRB, 0.52516E+00_JPRB, 0.49317E+00_JPRB, 0.45847E+00_JPRB, 0.41884E+00_JPRB, 0.39034E+00_JPRB, 0.37026E+00_JPRB ,&
&  0.35824E+00_JPRB, 0.34798E+00_JPRB, 0.33808E+00_JPRB, 0.32574E+00_JPRB, 0.31307E+00_JPRB, 0.29894E+00_JPRB, 0.28831E+00_JPRB ,&
&  0.28197E+00_JPRB, 0.28022E+00_JPRB, 0.27982E+00_JPRB, 0.28406E+00_JPRB, 0.28988E+00_JPRB, 0.29769E+00_JPRB, 0.30666E+00_JPRB ,&
&  0.32708E+00_JPRB, 0.33335E+00_JPRB, 0.33367E+00_JPRB, 0.33364E+00_JPRB, 0.33482E+00_JPRB, 0.33711E+00_JPRB, 0.32752E+00_JPRB ,&
&  0.33650E+00_JPRB, 0.35816E+00_JPRB, 0.36413E+00_JPRB, 0.34540E+00_JPRB, 0.29092E+00_JPRB, 0.25819E+00_JPRB, 0.23564E+00_JPRB ,&
&  0.22709E+00_JPRB, 0.21741E+00_JPRB, 0.21138E+00_JPRB, 0.20567E+00_JPRB, 0.20202E+00_JPRB, 0.20017E+00_JPRB, 0.19833E+00_JPRB ,&
&  0.19699E+00_JPRB, 0.19599E+00_JPRB, 0.19402E+00_JPRB, 0.19314E+00_JPRB, 0.19209E+00_JPRB, 0.19153E+00_JPRB, 0.19122E+00_JPRB ,&
&  0.19107E+00_JPRB, 0.19076E+00_JPRB, 0.19096E+00_JPRB, 0.19148E+00_JPRB, 0.19239E+00_JPRB, 0.19311E+00_JPRB, 0.19377E+00_JPRB ,&
&  0.19461E+00_JPRB, 0.19538E+00_JPRB, 0.19555E+00_JPRB, 0.19600E+00_JPRB, 0.19639E+00_JPRB, 0.19750E+00_JPRB, 0.19799E+00_JPRB ,&
&  0.19852E+00_JPRB, 0.19895E+00_JPRB, 0.19893E+00_JPRB, 0.19949E+00_JPRB, 0.20038E+00_JPRB, 0.20217E+00_JPRB, 0.20422E+00_JPRB ,&
&  0.20686E+00_JPRB, 0.20992E+00_JPRB, 0.21334E+00_JPRB, 0.21677E+00_JPRB, 0.22128E+00_JPRB, 0.22632E+00_JPRB, 0.23280E+00_JPRB ,&
&  0.23987E+00_JPRB, 0.24875E+00_JPRB, 0.26069E+00_JPRB, 0.27530E+00_JPRB, 0.29308E+00_JPRB, 0.31126E+00_JPRB, 0.33210E+00_JPRB ,&
&  0.36724E+00_JPRB, 0.46712E+00_JPRB, 0.52734E+00_JPRB, 0.51495E+00_JPRB, 0.52535E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.51601E+00_JPRB, 0.51829E+00_JPRB, 0.52078E+00_JPRB, 0.52479E+00_JPRB, 0.52905E+00_JPRB, 0.53332E+00_JPRB, 0.53706E+00_JPRB ,&
&  0.51882E+00_JPRB, 0.49572E+00_JPRB, 0.47675E+00_JPRB, 0.45419E+00_JPRB, 0.43363E+00_JPRB, 0.41674E+00_JPRB, 0.39662E+00_JPRB ,&
&  0.37657E+00_JPRB, 0.35652E+00_JPRB, 0.33389E+00_JPRB, 0.31533E+00_JPRB, 0.29706E+00_JPRB, 0.28228E+00_JPRB, 0.27323E+00_JPRB ,&
&  0.26836E+00_JPRB, 0.26707E+00_JPRB, 0.26860E+00_JPRB, 0.27173E+00_JPRB, 0.27467E+00_JPRB, 0.27942E+00_JPRB, 0.28596E+00_JPRB ,&
&  0.29500E+00_JPRB, 0.30943E+00_JPRB, 0.32246E+00_JPRB, 0.32721E+00_JPRB, 0.32379E+00_JPRB, 0.31544E+00_JPRB, 0.30827E+00_JPRB ,&
&  0.29958E+00_JPRB, 0.28737E+00_JPRB, 0.27262E+00_JPRB, 0.25803E+00_JPRB, 0.24446E+00_JPRB, 0.23457E+00_JPRB, 0.22640E+00_JPRB ,&
&  0.21950E+00_JPRB, 0.21344E+00_JPRB, 0.20880E+00_JPRB, 0.20567E+00_JPRB, 0.20300E+00_JPRB, 0.20084E+00_JPRB, 0.19880E+00_JPRB ,&
&  0.19777E+00_JPRB, 0.19670E+00_JPRB, 0.19604E+00_JPRB, 0.19568E+00_JPRB, 0.19522E+00_JPRB, 0.19509E+00_JPRB, 0.19500E+00_JPRB ,&
&  0.19468E+00_JPRB, 0.19407E+00_JPRB, 0.19373E+00_JPRB, 0.19350E+00_JPRB, 0.19390E+00_JPRB, 0.19421E+00_JPRB, 0.19473E+00_JPRB ,&
&  0.19557E+00_JPRB, 0.19733E+00_JPRB, 0.19845E+00_JPRB, 0.19950E+00_JPRB, 0.20004E+00_JPRB, 0.20050E+00_JPRB, 0.20111E+00_JPRB ,&
&  0.20213E+00_JPRB, 0.20338E+00_JPRB, 0.20450E+00_JPRB, 0.20600E+00_JPRB, 0.20754E+00_JPRB, 0.20965E+00_JPRB, 0.21190E+00_JPRB ,&
&  0.21450E+00_JPRB, 0.21772E+00_JPRB, 0.22175E+00_JPRB, 0.22544E+00_JPRB, 0.23112E+00_JPRB, 0.23682E+00_JPRB, 0.24319E+00_JPRB ,&
&  0.24971E+00_JPRB, 0.25675E+00_JPRB, 0.26591E+00_JPRB, 0.27677E+00_JPRB, 0.29022E+00_JPRB, 0.29861E+00_JPRB, 0.31006E+00_JPRB ,&
&  0.32757E+00_JPRB, 0.36447E+00_JPRB, 0.49992E+00_JPRB, 0.51343E+00_JPRB, 0.51800E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.29768E+00_JPRB, 0.30346E+00_JPRB, 0.30751E+00_JPRB, 0.31663E+00_JPRB, 0.33005E+00_JPRB, 0.42656E+00_JPRB, 0.42154E+00_JPRB ,&
&  0.42055E+00_JPRB, 0.37765E+00_JPRB, 0.36952E+00_JPRB, 0.35733E+00_JPRB, 0.34198E+00_JPRB, 0.32300E+00_JPRB, 0.31123E+00_JPRB ,&
&  0.30143E+00_JPRB, 0.29006E+00_JPRB, 0.27937E+00_JPRB, 0.27314E+00_JPRB, 0.26819E+00_JPRB, 0.26384E+00_JPRB, 0.26137E+00_JPRB ,&
&  0.26051E+00_JPRB, 0.26104E+00_JPRB, 0.26191E+00_JPRB, 0.26325E+00_JPRB, 0.26667E+00_JPRB, 0.27038E+00_JPRB, 0.27408E+00_JPRB ,&
&  0.27599E+00_JPRB, 0.28011E+00_JPRB, 0.28086E+00_JPRB, 0.27767E+00_JPRB, 0.27241E+00_JPRB, 0.26527E+00_JPRB, 0.25804E+00_JPRB ,&
&  0.25061E+00_JPRB, 0.24366E+00_JPRB, 0.23793E+00_JPRB, 0.23274E+00_JPRB, 0.22687E+00_JPRB, 0.22136E+00_JPRB, 0.21786E+00_JPRB ,&
&  0.21470E+00_JPRB, 0.21150E+00_JPRB, 0.20900E+00_JPRB, 0.20773E+00_JPRB, 0.20631E+00_JPRB, 0.20537E+00_JPRB, 0.20468E+00_JPRB ,&
&  0.20450E+00_JPRB, 0.20440E+00_JPRB, 0.20448E+00_JPRB, 0.20478E+00_JPRB, 0.20476E+00_JPRB, 0.20469E+00_JPRB, 0.20466E+00_JPRB ,&
&  0.20521E+00_JPRB, 0.20524E+00_JPRB, 0.20515E+00_JPRB, 0.20491E+00_JPRB, 0.20553E+00_JPRB, 0.20571E+00_JPRB, 0.20606E+00_JPRB ,&
&  0.20666E+00_JPRB, 0.20750E+00_JPRB, 0.20814E+00_JPRB, 0.20888E+00_JPRB, 0.20959E+00_JPRB, 0.21045E+00_JPRB, 0.21144E+00_JPRB ,&
&  0.21244E+00_JPRB, 0.21392E+00_JPRB, 0.21542E+00_JPRB, 0.21676E+00_JPRB, 0.21796E+00_JPRB, 0.21957E+00_JPRB, 0.22140E+00_JPRB ,&
&  0.22413E+00_JPRB, 0.22679E+00_JPRB, 0.22952E+00_JPRB, 0.23121E+00_JPRB, 0.23389E+00_JPRB, 0.23614E+00_JPRB, 0.23850E+00_JPRB ,&
&  0.24122E+00_JPRB, 0.24556E+00_JPRB, 0.25181E+00_JPRB, 0.25857E+00_JPRB, 0.26602E+00_JPRB, 0.27255E+00_JPRB, 0.27689E+00_JPRB ,&
&  0.28071E+00_JPRB, 0.28481E+00_JPRB, 0.29062E+00_JPRB, 0.29371E+00_JPRB, 0.29619E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.26259E+00_JPRB, 0.26263E+00_JPRB, 0.26269E+00_JPRB, 0.26233E+00_JPRB, 0.26080E+00_JPRB, 0.25963E+00_JPRB, 0.25860E+00_JPRB ,&
&  0.25809E+00_JPRB, 0.25724E+00_JPRB, 0.25682E+00_JPRB, 0.25665E+00_JPRB, 0.25627E+00_JPRB, 0.25571E+00_JPRB, 0.25499E+00_JPRB ,&
&  0.25404E+00_JPRB, 0.25281E+00_JPRB, 0.25155E+00_JPRB, 0.25133E+00_JPRB, 0.25146E+00_JPRB, 0.25161E+00_JPRB, 0.25170E+00_JPRB ,&
&  0.25168E+00_JPRB, 0.25164E+00_JPRB, 0.25114E+00_JPRB, 0.25119E+00_JPRB, 0.25119E+00_JPRB, 0.25119E+00_JPRB, 0.25132E+00_JPRB ,&
&  0.25154E+00_JPRB, 0.25173E+00_JPRB, 0.25149E+00_JPRB, 0.25102E+00_JPRB, 0.25011E+00_JPRB, 0.24844E+00_JPRB, 0.24582E+00_JPRB ,&
&  0.24284E+00_JPRB, 0.23986E+00_JPRB, 0.23747E+00_JPRB, 0.23528E+00_JPRB, 0.23328E+00_JPRB, 0.23144E+00_JPRB, 0.22992E+00_JPRB ,&
&  0.22811E+00_JPRB, 0.22632E+00_JPRB, 0.22468E+00_JPRB, 0.22353E+00_JPRB, 0.22227E+00_JPRB, 0.22117E+00_JPRB, 0.22019E+00_JPRB ,&
&  0.21951E+00_JPRB, 0.21891E+00_JPRB, 0.21840E+00_JPRB, 0.21814E+00_JPRB, 0.21793E+00_JPRB, 0.21774E+00_JPRB, 0.21751E+00_JPRB ,&
&  0.21732E+00_JPRB, 0.21717E+00_JPRB, 0.21705E+00_JPRB, 0.21702E+00_JPRB, 0.21710E+00_JPRB, 0.21699E+00_JPRB, 0.21685E+00_JPRB ,&
&  0.21680E+00_JPRB, 0.21706E+00_JPRB, 0.21725E+00_JPRB, 0.21756E+00_JPRB, 0.21803E+00_JPRB, 0.21870E+00_JPRB, 0.21943E+00_JPRB ,&
&  0.22016E+00_JPRB, 0.22101E+00_JPRB, 0.22194E+00_JPRB, 0.22307E+00_JPRB, 0.22427E+00_JPRB, 0.22547E+00_JPRB, 0.22663E+00_JPRB ,&
&  0.22786E+00_JPRB, 0.22900E+00_JPRB, 0.23007E+00_JPRB, 0.23085E+00_JPRB, 0.23190E+00_JPRB, 0.23297E+00_JPRB, 0.23422E+00_JPRB ,&
&  0.23564E+00_JPRB, 0.23752E+00_JPRB, 0.24009E+00_JPRB, 0.24316E+00_JPRB, 0.24698E+00_JPRB, 0.24981E+00_JPRB, 0.25271E+00_JPRB ,&
&  0.25480E+00_JPRB, 0.25643E+00_JPRB, 0.25762E+00_JPRB, 0.25915E+00_JPRB, 0.26085E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB, 0.23406E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2100:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_A1B2100
