!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_OBS1920 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS1920* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB, 0.13048E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.16211E+00_JPRB, 0.16443E+00_JPRB, 0.16597E+00_JPRB, 0.16693E+00_JPRB, 0.16674E+00_JPRB, 0.16765E+00_JPRB, 0.16863E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.17142E+00_JPRB, 0.17243E+00_JPRB, 0.17340E+00_JPRB, 0.17389E+00_JPRB, 0.17387E+00_JPRB, 0.17385E+00_JPRB ,&
&  0.17367E+00_JPRB, 0.17343E+00_JPRB, 0.17084E+00_JPRB, 0.16850E+00_JPRB, 0.16688E+00_JPRB, 0.16607E+00_JPRB, 0.16527E+00_JPRB ,&
&  0.16458E+00_JPRB, 0.16351E+00_JPRB, 0.16213E+00_JPRB, 0.15998E+00_JPRB, 0.15685E+00_JPRB, 0.15381E+00_JPRB, 0.15073E+00_JPRB ,&
&  0.14796E+00_JPRB, 0.14563E+00_JPRB, 0.14349E+00_JPRB, 0.14129E+00_JPRB, 0.13987E+00_JPRB, 0.13828E+00_JPRB, 0.13670E+00_JPRB ,&
&  0.13478E+00_JPRB, 0.13275E+00_JPRB, 0.13079E+00_JPRB, 0.12879E+00_JPRB, 0.12687E+00_JPRB, 0.12495E+00_JPRB, 0.12355E+00_JPRB ,&
&  0.12207E+00_JPRB, 0.12092E+00_JPRB, 0.11967E+00_JPRB, 0.11854E+00_JPRB, 0.11735E+00_JPRB, 0.11623E+00_JPRB, 0.11502E+00_JPRB ,&
&  0.11419E+00_JPRB, 0.11333E+00_JPRB, 0.11246E+00_JPRB, 0.11165E+00_JPRB, 0.11085E+00_JPRB, 0.11007E+00_JPRB, 0.10919E+00_JPRB ,&
&  0.10840E+00_JPRB, 0.10788E+00_JPRB, 0.10733E+00_JPRB, 0.10672E+00_JPRB, 0.10616E+00_JPRB, 0.10576E+00_JPRB, 0.10541E+00_JPRB ,&
&  0.10509E+00_JPRB, 0.10505E+00_JPRB, 0.10519E+00_JPRB, 0.10533E+00_JPRB, 0.10548E+00_JPRB, 0.10583E+00_JPRB, 0.10636E+00_JPRB ,&
&  0.10698E+00_JPRB, 0.10765E+00_JPRB, 0.10854E+00_JPRB, 0.10978E+00_JPRB, 0.11116E+00_JPRB, 0.11274E+00_JPRB, 0.11448E+00_JPRB ,&
&  0.11634E+00_JPRB, 0.11836E+00_JPRB, 0.12053E+00_JPRB, 0.12274E+00_JPRB, 0.12506E+00_JPRB, 0.12740E+00_JPRB, 0.12966E+00_JPRB ,&
&  0.13199E+00_JPRB, 0.13426E+00_JPRB, 0.13652E+00_JPRB, 0.13857E+00_JPRB, 0.14155E+00_JPRB, 0.14349E+00_JPRB, 0.14561E+00_JPRB ,&
&  0.14763E+00_JPRB, 0.14996E+00_JPRB, 0.15232E+00_JPRB, 0.15525E+00_JPRB, 0.15816E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.18839E+00_JPRB, 0.18884E+00_JPRB, 0.19011E+00_JPRB, 0.19722E+00_JPRB, 0.21259E+00_JPRB, 0.28936E+00_JPRB, 0.28131E+00_JPRB ,&
&  0.27464E+00_JPRB, 0.24602E+00_JPRB, 0.24192E+00_JPRB, 0.23447E+00_JPRB, 0.22470E+00_JPRB, 0.21631E+00_JPRB, 0.21113E+00_JPRB ,&
&  0.20563E+00_JPRB, 0.19975E+00_JPRB, 0.19600E+00_JPRB, 0.19255E+00_JPRB, 0.18925E+00_JPRB, 0.18570E+00_JPRB, 0.18300E+00_JPRB ,&
&  0.18080E+00_JPRB, 0.17931E+00_JPRB, 0.17765E+00_JPRB, 0.17457E+00_JPRB, 0.17313E+00_JPRB, 0.17019E+00_JPRB, 0.16575E+00_JPRB ,&
&  0.16005E+00_JPRB, 0.15676E+00_JPRB, 0.15313E+00_JPRB, 0.14982E+00_JPRB, 0.15187E+00_JPRB, 0.15056E+00_JPRB, 0.14825E+00_JPRB ,&
&  0.14484E+00_JPRB, 0.14133E+00_JPRB, 0.13686E+00_JPRB, 0.13263E+00_JPRB, 0.12898E+00_JPRB, 0.12599E+00_JPRB, 0.12335E+00_JPRB ,&
&  0.12123E+00_JPRB, 0.11942E+00_JPRB, 0.11716E+00_JPRB, 0.11511E+00_JPRB, 0.11294E+00_JPRB, 0.11101E+00_JPRB, 0.10914E+00_JPRB ,&
&  0.10764E+00_JPRB, 0.10623E+00_JPRB, 0.10476E+00_JPRB, 0.10322E+00_JPRB, 0.10201E+00_JPRB, 0.10043E+00_JPRB, 0.99241E-01_JPRB ,&
&  0.98168E-01_JPRB, 0.96885E-01_JPRB, 0.95946E-01_JPRB, 0.94780E-01_JPRB, 0.93918E-01_JPRB, 0.93217E-01_JPRB, 0.92533E-01_JPRB ,&
&  0.91601E-01_JPRB, 0.90984E-01_JPRB, 0.90717E-01_JPRB, 0.90497E-01_JPRB, 0.90161E-01_JPRB, 0.90148E-01_JPRB, 0.90385E-01_JPRB ,&
&  0.90741E-01_JPRB, 0.91161E-01_JPRB, 0.91908E-01_JPRB, 0.92815E-01_JPRB, 0.94152E-01_JPRB, 0.95923E-01_JPRB, 0.98757E-01_JPRB ,&
&  0.10185E+00_JPRB, 0.10528E+00_JPRB, 0.10894E+00_JPRB, 0.11331E+00_JPRB, 0.11765E+00_JPRB, 0.12214E+00_JPRB, 0.12728E+00_JPRB ,&
&  0.13285E+00_JPRB, 0.13705E+00_JPRB, 0.14081E+00_JPRB, 0.14478E+00_JPRB, 0.15253E+00_JPRB, 0.15853E+00_JPRB, 0.16332E+00_JPRB ,&
&  0.16863E+00_JPRB, 0.17570E+00_JPRB, 0.18069E+00_JPRB, 0.18428E+00_JPRB, 0.18668E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.44554E+00_JPRB, 0.45688E+00_JPRB, 0.47189E+00_JPRB, 0.48096E+00_JPRB, 0.49044E+00_JPRB, 0.45767E+00_JPRB, 0.43188E+00_JPRB ,&
&  0.37948E+00_JPRB, 0.34968E+00_JPRB, 0.32496E+00_JPRB, 0.30225E+00_JPRB, 0.28213E+00_JPRB, 0.26493E+00_JPRB, 0.24780E+00_JPRB ,&
&  0.23679E+00_JPRB, 0.22996E+00_JPRB, 0.22067E+00_JPRB, 0.21056E+00_JPRB, 0.19988E+00_JPRB, 0.19056E+00_JPRB, 0.18529E+00_JPRB ,&
&  0.18092E+00_JPRB, 0.17775E+00_JPRB, 0.17647E+00_JPRB, 0.17701E+00_JPRB, 0.17902E+00_JPRB, 0.17958E+00_JPRB, 0.18054E+00_JPRB ,&
&  0.17829E+00_JPRB, 0.18522E+00_JPRB, 0.19423E+00_JPRB, 0.19836E+00_JPRB, 0.20571E+00_JPRB, 0.20413E+00_JPRB, 0.19936E+00_JPRB ,&
&  0.19284E+00_JPRB, 0.18473E+00_JPRB, 0.17585E+00_JPRB, 0.16486E+00_JPRB, 0.15439E+00_JPRB, 0.14642E+00_JPRB, 0.13872E+00_JPRB ,&
&  0.13393E+00_JPRB, 0.12862E+00_JPRB, 0.12281E+00_JPRB, 0.11790E+00_JPRB, 0.11366E+00_JPRB, 0.11003E+00_JPRB, 0.10681E+00_JPRB ,&
&  0.10426E+00_JPRB, 0.10263E+00_JPRB, 0.10104E+00_JPRB, 0.10030E+00_JPRB, 0.99085E-01_JPRB, 0.97971E-01_JPRB, 0.96900E-01_JPRB ,&
&  0.96248E-01_JPRB, 0.95126E-01_JPRB, 0.94283E-01_JPRB, 0.93523E-01_JPRB, 0.92426E-01_JPRB, 0.91473E-01_JPRB, 0.90378E-01_JPRB ,&
&  0.89400E-01_JPRB, 0.88839E-01_JPRB, 0.88515E-01_JPRB, 0.88386E-01_JPRB, 0.88547E-01_JPRB, 0.88454E-01_JPRB, 0.88596E-01_JPRB ,&
&  0.88804E-01_JPRB, 0.89136E-01_JPRB, 0.89137E-01_JPRB, 0.89360E-01_JPRB, 0.90074E-01_JPRB, 0.91070E-01_JPRB, 0.93478E-01_JPRB ,&
&  0.96293E-01_JPRB, 0.99947E-01_JPRB, 0.10400E+00_JPRB, 0.10886E+00_JPRB, 0.11377E+00_JPRB, 0.11884E+00_JPRB, 0.12441E+00_JPRB ,&
&  0.13160E+00_JPRB, 0.13806E+00_JPRB, 0.14535E+00_JPRB, 0.15631E+00_JPRB, 0.17848E+00_JPRB, 0.19182E+00_JPRB, 0.20773E+00_JPRB ,&
&  0.23575E+00_JPRB, 0.27556E+00_JPRB, 0.37847E+00_JPRB, 0.41163E+00_JPRB, 0.42866E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.62210E+00_JPRB, 0.65315E+00_JPRB, 0.67797E+00_JPRB, 0.68042E+00_JPRB, 0.67960E+00_JPRB, 0.64252E+00_JPRB, 0.58507E+00_JPRB ,&
&  0.47783E+00_JPRB, 0.40809E+00_JPRB, 0.36453E+00_JPRB, 0.32354E+00_JPRB, 0.28622E+00_JPRB, 0.26145E+00_JPRB, 0.24435E+00_JPRB ,&
&  0.23137E+00_JPRB, 0.22427E+00_JPRB, 0.21883E+00_JPRB, 0.21423E+00_JPRB, 0.20935E+00_JPRB, 0.20463E+00_JPRB, 0.19590E+00_JPRB ,&
&  0.19074E+00_JPRB, 0.18708E+00_JPRB, 0.18582E+00_JPRB, 0.18828E+00_JPRB, 0.19513E+00_JPRB, 0.20254E+00_JPRB, 0.21335E+00_JPRB ,&
&  0.22753E+00_JPRB, 0.24102E+00_JPRB, 0.24700E+00_JPRB, 0.25002E+00_JPRB, 0.25288E+00_JPRB, 0.25081E+00_JPRB, 0.22785E+00_JPRB ,&
&  0.23308E+00_JPRB, 0.25702E+00_JPRB, 0.27550E+00_JPRB, 0.28701E+00_JPRB, 0.24391E+00_JPRB, 0.20277E+00_JPRB, 0.16240E+00_JPRB ,&
&  0.14258E+00_JPRB, 0.12777E+00_JPRB, 0.12024E+00_JPRB, 0.11470E+00_JPRB, 0.11093E+00_JPRB, 0.10845E+00_JPRB, 0.10623E+00_JPRB ,&
&  0.10411E+00_JPRB, 0.10304E+00_JPRB, 0.10155E+00_JPRB, 0.10073E+00_JPRB, 0.99667E-01_JPRB, 0.99358E-01_JPRB, 0.98155E-01_JPRB ,&
&  0.96922E-01_JPRB, 0.95980E-01_JPRB, 0.95396E-01_JPRB, 0.95109E-01_JPRB, 0.94442E-01_JPRB, 0.93762E-01_JPRB, 0.92256E-01_JPRB ,&
&  0.91140E-01_JPRB, 0.90011E-01_JPRB, 0.89134E-01_JPRB, 0.89044E-01_JPRB, 0.89339E-01_JPRB, 0.90665E-01_JPRB, 0.91406E-01_JPRB ,&
&  0.91559E-01_JPRB, 0.91589E-01_JPRB, 0.91637E-01_JPRB, 0.92237E-01_JPRB, 0.93164E-01_JPRB, 0.94810E-01_JPRB, 0.97103E-01_JPRB ,&
&  0.99860E-01_JPRB, 0.10403E+00_JPRB, 0.10837E+00_JPRB, 0.11330E+00_JPRB, 0.11804E+00_JPRB, 0.12296E+00_JPRB, 0.12918E+00_JPRB ,&
&  0.13564E+00_JPRB, 0.14297E+00_JPRB, 0.15352E+00_JPRB, 0.16510E+00_JPRB, 0.18568E+00_JPRB, 0.20516E+00_JPRB, 0.23781E+00_JPRB ,&
&  0.30950E+00_JPRB, 0.45770E+00_JPRB, 0.54260E+00_JPRB, 0.55222E+00_JPRB, 0.58472E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.74232E+00_JPRB, 0.76075E+00_JPRB, 0.77728E+00_JPRB, 0.77470E+00_JPRB, 0.75752E+00_JPRB, 0.69982E+00_JPRB, 0.63645E+00_JPRB ,&
&  0.55778E+00_JPRB, 0.49131E+00_JPRB, 0.42215E+00_JPRB, 0.36193E+00_JPRB, 0.31173E+00_JPRB, 0.27324E+00_JPRB, 0.25097E+00_JPRB ,&
&  0.23545E+00_JPRB, 0.22875E+00_JPRB, 0.22665E+00_JPRB, 0.23129E+00_JPRB, 0.23401E+00_JPRB, 0.23945E+00_JPRB, 0.23395E+00_JPRB ,&
&  0.22708E+00_JPRB, 0.21884E+00_JPRB, 0.21691E+00_JPRB, 0.22569E+00_JPRB, 0.23861E+00_JPRB, 0.25124E+00_JPRB, 0.26753E+00_JPRB ,&
&  0.28570E+00_JPRB, 0.29148E+00_JPRB, 0.30507E+00_JPRB, 0.30889E+00_JPRB, 0.29401E+00_JPRB, 0.29554E+00_JPRB, 0.30137E+00_JPRB ,&
&  0.31445E+00_JPRB, 0.33305E+00_JPRB, 0.34380E+00_JPRB, 0.36427E+00_JPRB, 0.31277E+00_JPRB, 0.25265E+00_JPRB, 0.20152E+00_JPRB ,&
&  0.16368E+00_JPRB, 0.14232E+00_JPRB, 0.12795E+00_JPRB, 0.12058E+00_JPRB, 0.11388E+00_JPRB, 0.11019E+00_JPRB, 0.10618E+00_JPRB ,&
&  0.10439E+00_JPRB, 0.10416E+00_JPRB, 0.10353E+00_JPRB, 0.10302E+00_JPRB, 0.10315E+00_JPRB, 0.10350E+00_JPRB, 0.10283E+00_JPRB ,&
&  0.10199E+00_JPRB, 0.10164E+00_JPRB, 0.10117E+00_JPRB, 0.10087E+00_JPRB, 0.10013E+00_JPRB, 0.10001E+00_JPRB, 0.98782E-01_JPRB ,&
&  0.98141E-01_JPRB, 0.96975E-01_JPRB, 0.96280E-01_JPRB, 0.96098E-01_JPRB, 0.96338E-01_JPRB, 0.96607E-01_JPRB, 0.97064E-01_JPRB ,&
&  0.98013E-01_JPRB, 0.98887E-01_JPRB, 0.10038E+00_JPRB, 0.10196E+00_JPRB, 0.10443E+00_JPRB, 0.10680E+00_JPRB, 0.11013E+00_JPRB ,&
&  0.11364E+00_JPRB, 0.11790E+00_JPRB, 0.12267E+00_JPRB, 0.12799E+00_JPRB, 0.13277E+00_JPRB, 0.13752E+00_JPRB, 0.14276E+00_JPRB ,&
&  0.14698E+00_JPRB, 0.15350E+00_JPRB, 0.16088E+00_JPRB, 0.17309E+00_JPRB, 0.19854E+00_JPRB, 0.22252E+00_JPRB, 0.27185E+00_JPRB ,&
&  0.35757E+00_JPRB, 0.61322E+00_JPRB, 0.84336E+00_JPRB, 0.69311E+00_JPRB, 0.72394E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.75040E+00_JPRB, 0.76305E+00_JPRB, 0.78028E+00_JPRB, 0.75675E+00_JPRB, 0.74136E+00_JPRB, 0.74475E+00_JPRB, 0.74557E+00_JPRB ,&
&  0.72266E+00_JPRB, 0.68805E+00_JPRB, 0.63938E+00_JPRB, 0.59983E+00_JPRB, 0.53361E+00_JPRB, 0.44881E+00_JPRB, 0.41126E+00_JPRB ,&
&  0.37739E+00_JPRB, 0.37401E+00_JPRB, 0.37644E+00_JPRB, 0.39241E+00_JPRB, 0.40134E+00_JPRB, 0.39792E+00_JPRB, 0.37653E+00_JPRB ,&
&  0.34953E+00_JPRB, 0.32219E+00_JPRB, 0.30478E+00_JPRB, 0.29068E+00_JPRB, 0.29038E+00_JPRB, 0.28860E+00_JPRB, 0.28206E+00_JPRB ,&
&  0.27317E+00_JPRB, 0.27700E+00_JPRB, 0.29407E+00_JPRB, 0.32312E+00_JPRB, 0.37366E+00_JPRB, 0.35367E+00_JPRB, 0.35914E+00_JPRB ,&
&  0.38895E+00_JPRB, 0.41041E+00_JPRB, 0.41896E+00_JPRB, 0.42206E+00_JPRB, 0.41573E+00_JPRB, 0.44162E+00_JPRB, 0.38514E+00_JPRB ,&
&  0.27782E+00_JPRB, 0.21910E+00_JPRB, 0.17746E+00_JPRB, 0.15118E+00_JPRB, 0.13424E+00_JPRB, 0.12456E+00_JPRB, 0.11417E+00_JPRB ,&
&  0.10950E+00_JPRB, 0.10618E+00_JPRB, 0.10530E+00_JPRB, 0.10639E+00_JPRB, 0.10852E+00_JPRB, 0.11121E+00_JPRB, 0.11212E+00_JPRB ,&
&  0.11360E+00_JPRB, 0.11363E+00_JPRB, 0.11280E+00_JPRB, 0.11253E+00_JPRB, 0.11161E+00_JPRB, 0.11222E+00_JPRB, 0.11230E+00_JPRB ,&
&  0.11257E+00_JPRB, 0.11284E+00_JPRB, 0.11435E+00_JPRB, 0.11472E+00_JPRB, 0.11557E+00_JPRB, 0.11272E+00_JPRB, 0.11084E+00_JPRB ,&
&  0.11076E+00_JPRB, 0.11251E+00_JPRB, 0.11552E+00_JPRB, 0.11907E+00_JPRB, 0.12392E+00_JPRB, 0.12791E+00_JPRB, 0.13246E+00_JPRB ,&
&  0.13647E+00_JPRB, 0.14035E+00_JPRB, 0.14416E+00_JPRB, 0.14806E+00_JPRB, 0.15216E+00_JPRB, 0.15628E+00_JPRB, 0.16065E+00_JPRB ,&
&  0.16507E+00_JPRB, 0.17287E+00_JPRB, 0.18256E+00_JPRB, 0.19957E+00_JPRB, 0.22267E+00_JPRB, 0.25016E+00_JPRB, 0.29806E+00_JPRB ,&
&  0.36030E+00_JPRB, 0.59039E+00_JPRB, 0.88787E+00_JPRB, 0.71310E+00_JPRB, 0.73123E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.79964E+00_JPRB, 0.79771E+00_JPRB, 0.80599E+00_JPRB, 0.79567E+00_JPRB, 0.77655E+00_JPRB, 0.76741E+00_JPRB, 0.78437E+00_JPRB ,&
&  0.80417E+00_JPRB, 0.77490E+00_JPRB, 0.75348E+00_JPRB, 0.73436E+00_JPRB, 0.72100E+00_JPRB, 0.70741E+00_JPRB, 0.68503E+00_JPRB ,&
&  0.66492E+00_JPRB, 0.65765E+00_JPRB, 0.65589E+00_JPRB, 0.65091E+00_JPRB, 0.64064E+00_JPRB, 0.63057E+00_JPRB, 0.61280E+00_JPRB ,&
&  0.58053E+00_JPRB, 0.54674E+00_JPRB, 0.52869E+00_JPRB, 0.50514E+00_JPRB, 0.48827E+00_JPRB, 0.46888E+00_JPRB, 0.44118E+00_JPRB ,&
&  0.39676E+00_JPRB, 0.33874E+00_JPRB, 0.30129E+00_JPRB, 0.33712E+00_JPRB, 0.43378E+00_JPRB, 0.42835E+00_JPRB, 0.44688E+00_JPRB ,&
&  0.48136E+00_JPRB, 0.50264E+00_JPRB, 0.51297E+00_JPRB, 0.51623E+00_JPRB, 0.52526E+00_JPRB, 0.53036E+00_JPRB, 0.52682E+00_JPRB ,&
&  0.51267E+00_JPRB, 0.46057E+00_JPRB, 0.41884E+00_JPRB, 0.31700E+00_JPRB, 0.24506E+00_JPRB, 0.20734E+00_JPRB, 0.17899E+00_JPRB ,&
&  0.16091E+00_JPRB, 0.15000E+00_JPRB, 0.14754E+00_JPRB, 0.14809E+00_JPRB, 0.15321E+00_JPRB, 0.15792E+00_JPRB, 0.15786E+00_JPRB ,&
&  0.15450E+00_JPRB, 0.14724E+00_JPRB, 0.13643E+00_JPRB, 0.12967E+00_JPRB, 0.12634E+00_JPRB, 0.12651E+00_JPRB, 0.12844E+00_JPRB ,&
&  0.13039E+00_JPRB, 0.13508E+00_JPRB, 0.14744E+00_JPRB, 0.15857E+00_JPRB, 0.17201E+00_JPRB, 0.15325E+00_JPRB, 0.13839E+00_JPRB ,&
&  0.13341E+00_JPRB, 0.13715E+00_JPRB, 0.14315E+00_JPRB, 0.14773E+00_JPRB, 0.15264E+00_JPRB, 0.15698E+00_JPRB, 0.16142E+00_JPRB ,&
&  0.16552E+00_JPRB, 0.16840E+00_JPRB, 0.17004E+00_JPRB, 0.17176E+00_JPRB, 0.17337E+00_JPRB, 0.17570E+00_JPRB, 0.17868E+00_JPRB ,&
&  0.18370E+00_JPRB, 0.19269E+00_JPRB, 0.20449E+00_JPRB, 0.22392E+00_JPRB, 0.24217E+00_JPRB, 0.26051E+00_JPRB, 0.28131E+00_JPRB ,&
&  0.30270E+00_JPRB, 0.38223E+00_JPRB, 0.57634E+00_JPRB, 0.89879E+00_JPRB, 0.89202E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.79289E+00_JPRB, 0.81292E+00_JPRB, 0.82398E+00_JPRB, 0.83807E+00_JPRB, 0.83830E+00_JPRB, 0.80798E+00_JPRB, 0.79316E+00_JPRB ,&
&  0.79784E+00_JPRB, 0.79311E+00_JPRB, 0.78261E+00_JPRB, 0.77633E+00_JPRB, 0.76752E+00_JPRB, 0.75766E+00_JPRB, 0.74907E+00_JPRB ,&
&  0.74075E+00_JPRB, 0.73586E+00_JPRB, 0.74303E+00_JPRB, 0.75721E+00_JPRB, 0.76893E+00_JPRB, 0.77545E+00_JPRB, 0.78050E+00_JPRB ,&
&  0.77993E+00_JPRB, 0.74672E+00_JPRB, 0.70168E+00_JPRB, 0.66873E+00_JPRB, 0.65187E+00_JPRB, 0.63791E+00_JPRB, 0.60517E+00_JPRB ,&
&  0.53903E+00_JPRB, 0.41730E+00_JPRB, 0.35972E+00_JPRB, 0.40037E+00_JPRB, 0.52092E+00_JPRB, 0.48460E+00_JPRB, 0.50467E+00_JPRB ,&
&  0.53484E+00_JPRB, 0.55044E+00_JPRB, 0.54624E+00_JPRB, 0.53581E+00_JPRB, 0.53239E+00_JPRB, 0.53592E+00_JPRB, 0.53233E+00_JPRB ,&
&  0.52861E+00_JPRB, 0.51246E+00_JPRB, 0.50295E+00_JPRB, 0.49271E+00_JPRB, 0.47567E+00_JPRB, 0.45014E+00_JPRB, 0.43502E+00_JPRB ,&
&  0.39768E+00_JPRB, 0.36082E+00_JPRB, 0.35051E+00_JPRB, 0.35201E+00_JPRB, 0.32155E+00_JPRB, 0.30937E+00_JPRB, 0.29525E+00_JPRB ,&
&  0.25463E+00_JPRB, 0.23316E+00_JPRB, 0.20051E+00_JPRB, 0.17985E+00_JPRB, 0.16457E+00_JPRB, 0.15125E+00_JPRB, 0.14944E+00_JPRB ,&
&  0.15370E+00_JPRB, 0.16545E+00_JPRB, 0.18731E+00_JPRB, 0.22292E+00_JPRB, 0.25585E+00_JPRB, 0.20954E+00_JPRB, 0.19049E+00_JPRB ,&
&  0.17687E+00_JPRB, 0.17388E+00_JPRB, 0.17868E+00_JPRB, 0.18447E+00_JPRB, 0.19061E+00_JPRB, 0.19790E+00_JPRB, 0.20471E+00_JPRB ,&
&  0.20965E+00_JPRB, 0.21274E+00_JPRB, 0.21391E+00_JPRB, 0.21337E+00_JPRB, 0.21280E+00_JPRB, 0.21394E+00_JPRB, 0.21751E+00_JPRB ,&
&  0.22327E+00_JPRB, 0.23516E+00_JPRB, 0.25881E+00_JPRB, 0.29381E+00_JPRB, 0.33996E+00_JPRB, 0.40009E+00_JPRB, 0.49273E+00_JPRB ,&
&  0.61789E+00_JPRB, 0.78273E+00_JPRB, 0.82584E+00_JPRB, 0.83470E+00_JPRB, 0.80438E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.83109E+00_JPRB, 0.84179E+00_JPRB, 0.83970E+00_JPRB, 0.83870E+00_JPRB, 0.84057E+00_JPRB, 0.83357E+00_JPRB, 0.81919E+00_JPRB ,&
&  0.80490E+00_JPRB, 0.77842E+00_JPRB, 0.74823E+00_JPRB, 0.73483E+00_JPRB, 0.74268E+00_JPRB, 0.74280E+00_JPRB, 0.72408E+00_JPRB ,&
&  0.72230E+00_JPRB, 0.73624E+00_JPRB, 0.75932E+00_JPRB, 0.78510E+00_JPRB, 0.81996E+00_JPRB, 0.85611E+00_JPRB, 0.88325E+00_JPRB ,&
&  0.89388E+00_JPRB, 0.89173E+00_JPRB, 0.88391E+00_JPRB, 0.85706E+00_JPRB, 0.82130E+00_JPRB, 0.79386E+00_JPRB, 0.77302E+00_JPRB ,&
&  0.74742E+00_JPRB, 0.70050E+00_JPRB, 0.63037E+00_JPRB, 0.62489E+00_JPRB, 0.55997E+00_JPRB, 0.53869E+00_JPRB, 0.56058E+00_JPRB ,&
&  0.57593E+00_JPRB, 0.55799E+00_JPRB, 0.53445E+00_JPRB, 0.51491E+00_JPRB, 0.51134E+00_JPRB, 0.51005E+00_JPRB, 0.50498E+00_JPRB ,&
&  0.50105E+00_JPRB, 0.49431E+00_JPRB, 0.48028E+00_JPRB, 0.46726E+00_JPRB, 0.45503E+00_JPRB, 0.44283E+00_JPRB, 0.43468E+00_JPRB ,&
&  0.40723E+00_JPRB, 0.36508E+00_JPRB, 0.33386E+00_JPRB, 0.32063E+00_JPRB, 0.31345E+00_JPRB, 0.31720E+00_JPRB, 0.32957E+00_JPRB ,&
&  0.33923E+00_JPRB, 0.33928E+00_JPRB, 0.33344E+00_JPRB, 0.32144E+00_JPRB, 0.30391E+00_JPRB, 0.25762E+00_JPRB, 0.23161E+00_JPRB ,&
&  0.24677E+00_JPRB, 0.28137E+00_JPRB, 0.31707E+00_JPRB, 0.32433E+00_JPRB, 0.33406E+00_JPRB, 0.33689E+00_JPRB, 0.32281E+00_JPRB ,&
&  0.29524E+00_JPRB, 0.26282E+00_JPRB, 0.25590E+00_JPRB, 0.25785E+00_JPRB, 0.26157E+00_JPRB, 0.26899E+00_JPRB, 0.28457E+00_JPRB ,&
&  0.30831E+00_JPRB, 0.34753E+00_JPRB, 0.36941E+00_JPRB, 0.37196E+00_JPRB, 0.36465E+00_JPRB, 0.35293E+00_JPRB, 0.35117E+00_JPRB ,&
&  0.36480E+00_JPRB, 0.42609E+00_JPRB, 0.54853E+00_JPRB, 0.69709E+00_JPRB, 0.69478E+00_JPRB, 0.73841E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.80285E+00_JPRB, 0.81604E+00_JPRB, 0.80252E+00_JPRB, 0.80403E+00_JPRB, 0.81094E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.84347E+00_JPRB, 0.84820E+00_JPRB, 0.83892E+00_JPRB, 0.83487E+00_JPRB, 0.82436E+00_JPRB, 0.81094E+00_JPRB, 0.78854E+00_JPRB ,&
&  0.76203E+00_JPRB, 0.73375E+00_JPRB, 0.70726E+00_JPRB, 0.68561E+00_JPRB, 0.68980E+00_JPRB, 0.72262E+00_JPRB, 0.75021E+00_JPRB ,&
&  0.73918E+00_JPRB, 0.73523E+00_JPRB, 0.75242E+00_JPRB, 0.78316E+00_JPRB, 0.81671E+00_JPRB, 0.84773E+00_JPRB, 0.87032E+00_JPRB ,&
&  0.88878E+00_JPRB, 0.87055E+00_JPRB, 0.87383E+00_JPRB, 0.87543E+00_JPRB, 0.86905E+00_JPRB, 0.85102E+00_JPRB, 0.82217E+00_JPRB ,&
&  0.79343E+00_JPRB, 0.76620E+00_JPRB, 0.73242E+00_JPRB, 0.67269E+00_JPRB, 0.74592E+00_JPRB, 0.66779E+00_JPRB, 0.63875E+00_JPRB ,&
&  0.61174E+00_JPRB, 0.59385E+00_JPRB, 0.57379E+00_JPRB, 0.55196E+00_JPRB, 0.54442E+00_JPRB, 0.53969E+00_JPRB, 0.53170E+00_JPRB ,&
&  0.52517E+00_JPRB, 0.51841E+00_JPRB, 0.50818E+00_JPRB, 0.49482E+00_JPRB, 0.48218E+00_JPRB, 0.46857E+00_JPRB, 0.45611E+00_JPRB ,&
&  0.43822E+00_JPRB, 0.41217E+00_JPRB, 0.38480E+00_JPRB, 0.36489E+00_JPRB, 0.35472E+00_JPRB, 0.36388E+00_JPRB, 0.39576E+00_JPRB ,&
&  0.42866E+00_JPRB, 0.43411E+00_JPRB, 0.44351E+00_JPRB, 0.44581E+00_JPRB, 0.43723E+00_JPRB, 0.42755E+00_JPRB, 0.41012E+00_JPRB ,&
&  0.39634E+00_JPRB, 0.39373E+00_JPRB, 0.39440E+00_JPRB, 0.39762E+00_JPRB, 0.40294E+00_JPRB, 0.41049E+00_JPRB, 0.42106E+00_JPRB ,&
&  0.43085E+00_JPRB, 0.43409E+00_JPRB, 0.44621E+00_JPRB, 0.47287E+00_JPRB, 0.50343E+00_JPRB, 0.53434E+00_JPRB, 0.56043E+00_JPRB ,&
&  0.57785E+00_JPRB, 0.58976E+00_JPRB, 0.59777E+00_JPRB, 0.61284E+00_JPRB, 0.62840E+00_JPRB, 0.64429E+00_JPRB, 0.65264E+00_JPRB ,&
&  0.65036E+00_JPRB, 0.65372E+00_JPRB, 0.67027E+00_JPRB, 0.69818E+00_JPRB, 0.71783E+00_JPRB, 0.75160E+00_JPRB, 0.77273E+00_JPRB ,&
&  0.79523E+00_JPRB, 0.80115E+00_JPRB, 0.79585E+00_JPRB, 0.80850E+00_JPRB, 0.82551E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.80221E+00_JPRB, 0.82022E+00_JPRB, 0.83266E+00_JPRB, 0.83399E+00_JPRB, 0.81883E+00_JPRB, 0.79224E+00_JPRB, 0.76932E+00_JPRB ,&
&  0.74313E+00_JPRB, 0.71768E+00_JPRB, 0.69350E+00_JPRB, 0.66079E+00_JPRB, 0.64317E+00_JPRB, 0.64646E+00_JPRB, 0.69537E+00_JPRB ,&
&  0.76052E+00_JPRB, 0.76534E+00_JPRB, 0.75936E+00_JPRB, 0.78484E+00_JPRB, 0.83773E+00_JPRB, 0.86983E+00_JPRB, 0.89109E+00_JPRB ,&
&  0.89490E+00_JPRB, 0.86241E+00_JPRB, 0.82908E+00_JPRB, 0.81505E+00_JPRB, 0.83789E+00_JPRB, 0.85536E+00_JPRB, 0.86034E+00_JPRB ,&
&  0.85561E+00_JPRB, 0.83369E+00_JPRB, 0.81357E+00_JPRB, 0.79634E+00_JPRB, 0.79964E+00_JPRB, 0.77433E+00_JPRB, 0.72182E+00_JPRB ,&
&  0.65556E+00_JPRB, 0.62936E+00_JPRB, 0.61858E+00_JPRB, 0.60438E+00_JPRB, 0.59347E+00_JPRB, 0.59335E+00_JPRB, 0.58383E+00_JPRB ,&
&  0.57029E+00_JPRB, 0.55493E+00_JPRB, 0.54043E+00_JPRB, 0.52310E+00_JPRB, 0.50960E+00_JPRB, 0.49054E+00_JPRB, 0.47272E+00_JPRB ,&
&  0.45620E+00_JPRB, 0.43620E+00_JPRB, 0.41796E+00_JPRB, 0.40551E+00_JPRB, 0.40035E+00_JPRB, 0.41042E+00_JPRB, 0.44971E+00_JPRB ,&
&  0.48077E+00_JPRB, 0.47769E+00_JPRB, 0.47505E+00_JPRB, 0.47847E+00_JPRB, 0.47081E+00_JPRB, 0.45986E+00_JPRB, 0.45392E+00_JPRB ,&
&  0.44878E+00_JPRB, 0.44461E+00_JPRB, 0.44257E+00_JPRB, 0.44430E+00_JPRB, 0.44643E+00_JPRB, 0.44466E+00_JPRB, 0.44746E+00_JPRB ,&
&  0.45262E+00_JPRB, 0.45801E+00_JPRB, 0.47505E+00_JPRB, 0.49951E+00_JPRB, 0.51981E+00_JPRB, 0.53386E+00_JPRB, 0.54679E+00_JPRB ,&
&  0.56720E+00_JPRB, 0.57975E+00_JPRB, 0.56691E+00_JPRB, 0.55454E+00_JPRB, 0.54788E+00_JPRB, 0.55277E+00_JPRB, 0.56314E+00_JPRB ,&
&  0.58388E+00_JPRB, 0.59701E+00_JPRB, 0.60322E+00_JPRB, 0.60481E+00_JPRB, 0.60822E+00_JPRB, 0.60917E+00_JPRB, 0.62170E+00_JPRB ,&
&  0.65528E+00_JPRB, 0.68432E+00_JPRB, 0.69194E+00_JPRB, 0.72489E+00_JPRB, 0.76701E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.73282E+00_JPRB, 0.76476E+00_JPRB, 0.78963E+00_JPRB, 0.79935E+00_JPRB, 0.79790E+00_JPRB, 0.78931E+00_JPRB, 0.76666E+00_JPRB ,&
&  0.74200E+00_JPRB, 0.72437E+00_JPRB, 0.70563E+00_JPRB, 0.67893E+00_JPRB, 0.65674E+00_JPRB, 0.65199E+00_JPRB, 0.66441E+00_JPRB ,&
&  0.71058E+00_JPRB, 0.75163E+00_JPRB, 0.76840E+00_JPRB, 0.78241E+00_JPRB, 0.83274E+00_JPRB, 0.87503E+00_JPRB, 0.88470E+00_JPRB ,&
&  0.88064E+00_JPRB, 0.88293E+00_JPRB, 0.84660E+00_JPRB, 0.81328E+00_JPRB, 0.81300E+00_JPRB, 0.81794E+00_JPRB, 0.79934E+00_JPRB ,&
&  0.82486E+00_JPRB, 0.84134E+00_JPRB, 0.83401E+00_JPRB, 0.80683E+00_JPRB, 0.79778E+00_JPRB, 0.78699E+00_JPRB, 0.77526E+00_JPRB ,&
&  0.75344E+00_JPRB, 0.70537E+00_JPRB, 0.67660E+00_JPRB, 0.67583E+00_JPRB, 0.66882E+00_JPRB, 0.66118E+00_JPRB, 0.64720E+00_JPRB ,&
&  0.60712E+00_JPRB, 0.57142E+00_JPRB, 0.54485E+00_JPRB, 0.52665E+00_JPRB, 0.51543E+00_JPRB, 0.50688E+00_JPRB, 0.49815E+00_JPRB ,&
&  0.49072E+00_JPRB, 0.46481E+00_JPRB, 0.43832E+00_JPRB, 0.43374E+00_JPRB, 0.44055E+00_JPRB, 0.44734E+00_JPRB, 0.47360E+00_JPRB ,&
&  0.49852E+00_JPRB, 0.49690E+00_JPRB, 0.49306E+00_JPRB, 0.49488E+00_JPRB, 0.50301E+00_JPRB, 0.50241E+00_JPRB, 0.49175E+00_JPRB ,&
&  0.48627E+00_JPRB, 0.48347E+00_JPRB, 0.47393E+00_JPRB, 0.46709E+00_JPRB, 0.46644E+00_JPRB, 0.47072E+00_JPRB, 0.48562E+00_JPRB ,&
&  0.49488E+00_JPRB, 0.49641E+00_JPRB, 0.51020E+00_JPRB, 0.52751E+00_JPRB, 0.53774E+00_JPRB, 0.54562E+00_JPRB, 0.55103E+00_JPRB ,&
&  0.56415E+00_JPRB, 0.58348E+00_JPRB, 0.59846E+00_JPRB, 0.60039E+00_JPRB, 0.57724E+00_JPRB, 0.55285E+00_JPRB, 0.53361E+00_JPRB ,&
&  0.52548E+00_JPRB, 0.53013E+00_JPRB, 0.53758E+00_JPRB, 0.54454E+00_JPRB, 0.55093E+00_JPRB, 0.55503E+00_JPRB, 0.55404E+00_JPRB ,&
&  0.57346E+00_JPRB, 0.60464E+00_JPRB, 0.63009E+00_JPRB, 0.66058E+00_JPRB, 0.69227E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.69086E+00_JPRB, 0.70512E+00_JPRB, 0.71677E+00_JPRB, 0.73826E+00_JPRB, 0.75806E+00_JPRB, 0.75567E+00_JPRB, 0.74841E+00_JPRB ,&
&  0.74281E+00_JPRB, 0.73762E+00_JPRB, 0.72995E+00_JPRB, 0.71997E+00_JPRB, 0.71326E+00_JPRB, 0.71263E+00_JPRB, 0.71361E+00_JPRB ,&
&  0.71556E+00_JPRB, 0.73599E+00_JPRB, 0.76957E+00_JPRB, 0.79609E+00_JPRB, 0.82499E+00_JPRB, 0.86899E+00_JPRB, 0.88999E+00_JPRB ,&
&  0.87424E+00_JPRB, 0.87395E+00_JPRB, 0.86325E+00_JPRB, 0.84874E+00_JPRB, 0.85045E+00_JPRB, 0.84095E+00_JPRB, 0.81249E+00_JPRB ,&
&  0.80617E+00_JPRB, 0.79812E+00_JPRB, 0.79419E+00_JPRB, 0.82069E+00_JPRB, 0.84047E+00_JPRB, 0.83832E+00_JPRB, 0.83226E+00_JPRB ,&
&  0.81078E+00_JPRB, 0.77390E+00_JPRB, 0.74088E+00_JPRB, 0.72582E+00_JPRB, 0.72447E+00_JPRB, 0.70439E+00_JPRB, 0.67640E+00_JPRB ,&
&  0.61906E+00_JPRB, 0.57209E+00_JPRB, 0.54629E+00_JPRB, 0.53117E+00_JPRB, 0.52678E+00_JPRB, 0.52479E+00_JPRB, 0.52111E+00_JPRB ,&
&  0.52704E+00_JPRB, 0.52262E+00_JPRB, 0.49626E+00_JPRB, 0.46901E+00_JPRB, 0.46548E+00_JPRB, 0.47447E+00_JPRB, 0.49327E+00_JPRB ,&
&  0.50836E+00_JPRB, 0.50463E+00_JPRB, 0.51154E+00_JPRB, 0.51868E+00_JPRB, 0.53500E+00_JPRB, 0.55018E+00_JPRB, 0.55043E+00_JPRB ,&
&  0.54953E+00_JPRB, 0.54378E+00_JPRB, 0.53027E+00_JPRB, 0.52192E+00_JPRB, 0.51116E+00_JPRB, 0.50742E+00_JPRB, 0.51945E+00_JPRB ,&
&  0.53703E+00_JPRB, 0.54676E+00_JPRB, 0.55798E+00_JPRB, 0.57144E+00_JPRB, 0.58159E+00_JPRB, 0.58965E+00_JPRB, 0.59644E+00_JPRB ,&
&  0.60674E+00_JPRB, 0.62129E+00_JPRB, 0.63587E+00_JPRB, 0.64852E+00_JPRB, 0.64833E+00_JPRB, 0.63421E+00_JPRB, 0.60895E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.57850E+00_JPRB, 0.57488E+00_JPRB, 0.57939E+00_JPRB, 0.57399E+00_JPRB, 0.55436E+00_JPRB, 0.54479E+00_JPRB ,&
&  0.55590E+00_JPRB, 0.57980E+00_JPRB, 0.60282E+00_JPRB, 0.63412E+00_JPRB, 0.66701E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.64411E+00_JPRB, 0.65591E+00_JPRB, 0.66917E+00_JPRB, 0.68204E+00_JPRB, 0.69524E+00_JPRB, 0.69490E+00_JPRB, 0.69587E+00_JPRB ,&
&  0.71439E+00_JPRB, 0.71670E+00_JPRB, 0.71543E+00_JPRB, 0.70666E+00_JPRB, 0.70039E+00_JPRB, 0.70058E+00_JPRB, 0.69217E+00_JPRB ,&
&  0.68843E+00_JPRB, 0.69568E+00_JPRB, 0.71531E+00_JPRB, 0.74477E+00_JPRB, 0.77341E+00_JPRB, 0.81046E+00_JPRB, 0.83775E+00_JPRB ,&
&  0.84249E+00_JPRB, 0.83224E+00_JPRB, 0.83250E+00_JPRB, 0.83572E+00_JPRB, 0.83604E+00_JPRB, 0.81916E+00_JPRB, 0.81448E+00_JPRB ,&
&  0.79504E+00_JPRB, 0.79973E+00_JPRB, 0.67030E+00_JPRB, 0.73832E+00_JPRB, 0.78068E+00_JPRB, 0.81247E+00_JPRB, 0.81747E+00_JPRB ,&
&  0.80879E+00_JPRB, 0.79643E+00_JPRB, 0.77550E+00_JPRB, 0.75163E+00_JPRB, 0.74283E+00_JPRB, 0.72267E+00_JPRB, 0.70015E+00_JPRB ,&
&  0.65192E+00_JPRB, 0.59048E+00_JPRB, 0.55931E+00_JPRB, 0.54811E+00_JPRB, 0.54301E+00_JPRB, 0.54530E+00_JPRB, 0.55186E+00_JPRB ,&
&  0.55660E+00_JPRB, 0.55807E+00_JPRB, 0.54791E+00_JPRB, 0.53256E+00_JPRB, 0.51496E+00_JPRB, 0.50583E+00_JPRB, 0.50304E+00_JPRB ,&
&  0.49935E+00_JPRB, 0.48733E+00_JPRB, 0.49071E+00_JPRB, 0.52016E+00_JPRB, 0.54458E+00_JPRB, 0.56052E+00_JPRB, 0.56606E+00_JPRB ,&
&  0.56964E+00_JPRB, 0.56235E+00_JPRB, 0.55780E+00_JPRB, 0.54903E+00_JPRB, 0.52678E+00_JPRB, 0.52122E+00_JPRB, 0.54229E+00_JPRB ,&
&  0.56577E+00_JPRB, 0.57913E+00_JPRB, 0.58698E+00_JPRB, 0.59874E+00_JPRB, 0.61071E+00_JPRB, 0.61740E+00_JPRB, 0.62384E+00_JPRB ,&
&  0.62908E+00_JPRB, 0.63399E+00_JPRB, 0.64756E+00_JPRB, 0.66209E+00_JPRB, 0.66867E+00_JPRB, 0.66951E+00_JPRB, 0.66883E+00_JPRB ,&
&  0.66525E+00_JPRB, 0.65723E+00_JPRB, 0.64812E+00_JPRB, 0.63896E+00_JPRB, 0.61828E+00_JPRB, 0.59327E+00_JPRB, 0.57565E+00_JPRB ,&
&  0.56791E+00_JPRB, 0.57893E+00_JPRB, 0.58718E+00_JPRB, 0.60004E+00_JPRB, 0.62694E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.64427E+00_JPRB, 0.64130E+00_JPRB, 0.64054E+00_JPRB, 0.65113E+00_JPRB, 0.66778E+00_JPRB, 0.67128E+00_JPRB, 0.67181E+00_JPRB ,&
&  0.68311E+00_JPRB, 0.69158E+00_JPRB, 0.69089E+00_JPRB, 0.68966E+00_JPRB, 0.68430E+00_JPRB, 0.67163E+00_JPRB, 0.66240E+00_JPRB ,&
&  0.66915E+00_JPRB, 0.67641E+00_JPRB, 0.68069E+00_JPRB, 0.70660E+00_JPRB, 0.73999E+00_JPRB, 0.77897E+00_JPRB, 0.80556E+00_JPRB ,&
&  0.81539E+00_JPRB, 0.80853E+00_JPRB, 0.80505E+00_JPRB, 0.79976E+00_JPRB, 0.79441E+00_JPRB, 0.79608E+00_JPRB, 0.79044E+00_JPRB ,&
&  0.78144E+00_JPRB, 0.68539E+00_JPRB, 0.71381E+00_JPRB, 0.72201E+00_JPRB, 0.73567E+00_JPRB, 0.75675E+00_JPRB, 0.77822E+00_JPRB ,&
&  0.77511E+00_JPRB, 0.77145E+00_JPRB, 0.76430E+00_JPRB, 0.74737E+00_JPRB, 0.73122E+00_JPRB, 0.72175E+00_JPRB, 0.70600E+00_JPRB ,&
&  0.67948E+00_JPRB, 0.64319E+00_JPRB, 0.61087E+00_JPRB, 0.58916E+00_JPRB, 0.57972E+00_JPRB, 0.57838E+00_JPRB, 0.57572E+00_JPRB ,&
&  0.57576E+00_JPRB, 0.57601E+00_JPRB, 0.56697E+00_JPRB, 0.55879E+00_JPRB, 0.55056E+00_JPRB, 0.54584E+00_JPRB, 0.53411E+00_JPRB ,&
&  0.52637E+00_JPRB, 0.51605E+00_JPRB, 0.51572E+00_JPRB, 0.53515E+00_JPRB, 0.55473E+00_JPRB, 0.55749E+00_JPRB, 0.56850E+00_JPRB ,&
&  0.58299E+00_JPRB, 0.58812E+00_JPRB, 0.58851E+00_JPRB, 0.58518E+00_JPRB, 0.57767E+00_JPRB, 0.57713E+00_JPRB, 0.58441E+00_JPRB ,&
&  0.59037E+00_JPRB, 0.59942E+00_JPRB, 0.60195E+00_JPRB, 0.60598E+00_JPRB, 0.61219E+00_JPRB, 0.61690E+00_JPRB, 0.62815E+00_JPRB ,&
&  0.63402E+00_JPRB, 0.63834E+00_JPRB, 0.65016E+00_JPRB, 0.66169E+00_JPRB, 0.66609E+00_JPRB, 0.66790E+00_JPRB, 0.67608E+00_JPRB ,&
&  0.67852E+00_JPRB, 0.67570E+00_JPRB, 0.66727E+00_JPRB, 0.65476E+00_JPRB, 0.64479E+00_JPRB, 0.63054E+00_JPRB, 0.61086E+00_JPRB ,&
&  0.59580E+00_JPRB, 0.59505E+00_JPRB, 0.60543E+00_JPRB, 0.61647E+00_JPRB, 0.63432E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.63773E+00_JPRB, 0.63044E+00_JPRB, 0.62436E+00_JPRB, 0.63127E+00_JPRB, 0.64449E+00_JPRB, 0.65477E+00_JPRB, 0.65449E+00_JPRB ,&
&  0.65450E+00_JPRB, 0.65957E+00_JPRB, 0.66833E+00_JPRB, 0.67170E+00_JPRB, 0.66664E+00_JPRB, 0.65257E+00_JPRB, 0.64695E+00_JPRB ,&
&  0.64817E+00_JPRB, 0.65246E+00_JPRB, 0.65501E+00_JPRB, 0.66164E+00_JPRB, 0.68340E+00_JPRB, 0.71587E+00_JPRB, 0.74640E+00_JPRB ,&
&  0.76176E+00_JPRB, 0.75694E+00_JPRB, 0.75110E+00_JPRB, 0.74598E+00_JPRB, 0.74454E+00_JPRB, 0.74913E+00_JPRB, 0.74956E+00_JPRB ,&
&  0.74231E+00_JPRB, 0.59675E+00_JPRB, 0.64909E+00_JPRB, 0.66653E+00_JPRB, 0.68594E+00_JPRB, 0.70859E+00_JPRB, 0.73442E+00_JPRB ,&
&  0.73419E+00_JPRB, 0.72631E+00_JPRB, 0.71143E+00_JPRB, 0.70203E+00_JPRB, 0.68961E+00_JPRB, 0.68793E+00_JPRB, 0.68771E+00_JPRB ,&
&  0.68711E+00_JPRB, 0.68487E+00_JPRB, 0.66335E+00_JPRB, 0.62864E+00_JPRB, 0.61043E+00_JPRB, 0.60073E+00_JPRB, 0.59332E+00_JPRB ,&
&  0.58668E+00_JPRB, 0.57688E+00_JPRB, 0.56699E+00_JPRB, 0.55864E+00_JPRB, 0.56158E+00_JPRB, 0.56124E+00_JPRB, 0.55001E+00_JPRB ,&
&  0.54596E+00_JPRB, 0.54471E+00_JPRB, 0.55268E+00_JPRB, 0.56239E+00_JPRB, 0.56557E+00_JPRB, 0.55921E+00_JPRB, 0.56628E+00_JPRB ,&
&  0.58416E+00_JPRB, 0.59484E+00_JPRB, 0.60421E+00_JPRB, 0.61721E+00_JPRB, 0.62294E+00_JPRB, 0.62049E+00_JPRB, 0.61944E+00_JPRB ,&
&  0.62338E+00_JPRB, 0.62042E+00_JPRB, 0.61250E+00_JPRB, 0.60629E+00_JPRB, 0.60729E+00_JPRB, 0.60815E+00_JPRB, 0.61861E+00_JPRB ,&
&  0.62821E+00_JPRB, 0.63132E+00_JPRB, 0.62751E+00_JPRB, 0.63032E+00_JPRB, 0.63600E+00_JPRB, 0.64484E+00_JPRB, 0.65060E+00_JPRB ,&
&  0.65665E+00_JPRB, 0.65490E+00_JPRB, 0.64292E+00_JPRB, 0.64215E+00_JPRB, 0.64669E+00_JPRB, 0.64283E+00_JPRB, 0.63171E+00_JPRB ,&
&  0.62322E+00_JPRB, 0.61616E+00_JPRB, 0.62034E+00_JPRB, 0.62462E+00_JPRB, 0.63188E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.65506E+00_JPRB, 0.65249E+00_JPRB, 0.64587E+00_JPRB, 0.64571E+00_JPRB, 0.64687E+00_JPRB, 0.64511E+00_JPRB, 0.64481E+00_JPRB ,&
&  0.64370E+00_JPRB, 0.64188E+00_JPRB, 0.65161E+00_JPRB, 0.66096E+00_JPRB, 0.66430E+00_JPRB, 0.66156E+00_JPRB, 0.65986E+00_JPRB ,&
&  0.65976E+00_JPRB, 0.65602E+00_JPRB, 0.65138E+00_JPRB, 0.65121E+00_JPRB, 0.65624E+00_JPRB, 0.67213E+00_JPRB, 0.69506E+00_JPRB ,&
&  0.71223E+00_JPRB, 0.71250E+00_JPRB, 0.71657E+00_JPRB, 0.72751E+00_JPRB, 0.72951E+00_JPRB, 0.73038E+00_JPRB, 0.73654E+00_JPRB ,&
&  0.62023E+00_JPRB, 0.49514E+00_JPRB, 0.55442E+00_JPRB, 0.60561E+00_JPRB, 0.64023E+00_JPRB, 0.67923E+00_JPRB, 0.72184E+00_JPRB ,&
&  0.73460E+00_JPRB, 0.72465E+00_JPRB, 0.71699E+00_JPRB, 0.70643E+00_JPRB, 0.69054E+00_JPRB, 0.68088E+00_JPRB, 0.67299E+00_JPRB ,&
&  0.67278E+00_JPRB, 0.66354E+00_JPRB, 0.64843E+00_JPRB, 0.63650E+00_JPRB, 0.62744E+00_JPRB, 0.61567E+00_JPRB, 0.60422E+00_JPRB ,&
&  0.59226E+00_JPRB, 0.58227E+00_JPRB, 0.58095E+00_JPRB, 0.58164E+00_JPRB, 0.58611E+00_JPRB, 0.58786E+00_JPRB, 0.58454E+00_JPRB ,&
&  0.58173E+00_JPRB, 0.58391E+00_JPRB, 0.59319E+00_JPRB, 0.60085E+00_JPRB, 0.60257E+00_JPRB, 0.59940E+00_JPRB, 0.60579E+00_JPRB ,&
&  0.60830E+00_JPRB, 0.61536E+00_JPRB, 0.62473E+00_JPRB, 0.64393E+00_JPRB, 0.65764E+00_JPRB, 0.65737E+00_JPRB, 0.65540E+00_JPRB ,&
&  0.65694E+00_JPRB, 0.65005E+00_JPRB, 0.63489E+00_JPRB, 0.62489E+00_JPRB, 0.62030E+00_JPRB, 0.61382E+00_JPRB, 0.61235E+00_JPRB ,&
&  0.61742E+00_JPRB, 0.61888E+00_JPRB, 0.61313E+00_JPRB, 0.61494E+00_JPRB, 0.62375E+00_JPRB, 0.63733E+00_JPRB, 0.65000E+00_JPRB ,&
&  0.64350E+00_JPRB, 0.65120E+00_JPRB, 0.65145E+00_JPRB, 0.66013E+00_JPRB, 0.67053E+00_JPRB, 0.66163E+00_JPRB, 0.65570E+00_JPRB ,&
&  0.65023E+00_JPRB, 0.63157E+00_JPRB, 0.63567E+00_JPRB, 0.63953E+00_JPRB, 0.64405E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.63268E+00_JPRB, 0.66404E+00_JPRB, 0.67278E+00_JPRB, 0.66156E+00_JPRB, 0.64142E+00_JPRB, 0.63121E+00_JPRB, 0.62625E+00_JPRB ,&
&  0.62406E+00_JPRB, 0.62192E+00_JPRB, 0.63213E+00_JPRB, 0.64052E+00_JPRB, 0.64517E+00_JPRB, 0.64115E+00_JPRB, 0.63688E+00_JPRB ,&
&  0.62783E+00_JPRB, 0.61548E+00_JPRB, 0.60747E+00_JPRB, 0.60055E+00_JPRB, 0.60188E+00_JPRB, 0.61189E+00_JPRB, 0.62999E+00_JPRB ,&
&  0.65254E+00_JPRB, 0.66839E+00_JPRB, 0.68090E+00_JPRB, 0.69335E+00_JPRB, 0.69524E+00_JPRB, 0.69462E+00_JPRB, 0.69659E+00_JPRB ,&
&  0.70290E+00_JPRB, 0.51998E+00_JPRB, 0.55532E+00_JPRB, 0.60012E+00_JPRB, 0.63358E+00_JPRB, 0.68491E+00_JPRB, 0.73077E+00_JPRB ,&
&  0.75009E+00_JPRB, 0.73563E+00_JPRB, 0.72574E+00_JPRB, 0.71116E+00_JPRB, 0.69016E+00_JPRB, 0.67014E+00_JPRB, 0.64539E+00_JPRB ,&
&  0.62677E+00_JPRB, 0.61241E+00_JPRB, 0.60672E+00_JPRB, 0.60223E+00_JPRB, 0.59745E+00_JPRB, 0.59037E+00_JPRB, 0.58150E+00_JPRB ,&
&  0.57265E+00_JPRB, 0.56961E+00_JPRB, 0.56893E+00_JPRB, 0.56787E+00_JPRB, 0.57143E+00_JPRB, 0.57793E+00_JPRB, 0.58391E+00_JPRB ,&
&  0.59289E+00_JPRB, 0.59975E+00_JPRB, 0.60076E+00_JPRB, 0.59887E+00_JPRB, 0.59631E+00_JPRB, 0.59698E+00_JPRB, 0.60885E+00_JPRB ,&
&  0.61260E+00_JPRB, 0.62035E+00_JPRB, 0.62856E+00_JPRB, 0.64937E+00_JPRB, 0.66851E+00_JPRB, 0.67140E+00_JPRB, 0.66856E+00_JPRB ,&
&  0.66616E+00_JPRB, 0.65753E+00_JPRB, 0.64432E+00_JPRB, 0.63004E+00_JPRB, 0.62074E+00_JPRB, 0.61865E+00_JPRB, 0.61088E+00_JPRB ,&
&  0.60828E+00_JPRB, 0.60973E+00_JPRB, 0.60954E+00_JPRB, 0.61828E+00_JPRB, 0.63388E+00_JPRB, 0.64649E+00_JPRB, 0.64468E+00_JPRB ,&
&  0.64347E+00_JPRB, 0.64758E+00_JPRB, 0.64872E+00_JPRB, 0.66944E+00_JPRB, 0.67676E+00_JPRB, 0.67657E+00_JPRB, 0.66679E+00_JPRB ,&
&  0.65875E+00_JPRB, 0.65319E+00_JPRB, 0.49038E+00_JPRB, 0.57369E+00_JPRB, 0.59880E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.64913E+00_JPRB, 0.68390E+00_JPRB, 0.68695E+00_JPRB, 0.66090E+00_JPRB, 0.63304E+00_JPRB, 0.61645E+00_JPRB, 0.60681E+00_JPRB ,&
&  0.60588E+00_JPRB, 0.60146E+00_JPRB, 0.60371E+00_JPRB, 0.61387E+00_JPRB, 0.62540E+00_JPRB, 0.62895E+00_JPRB, 0.61519E+00_JPRB ,&
&  0.59775E+00_JPRB, 0.58578E+00_JPRB, 0.58258E+00_JPRB, 0.58292E+00_JPRB, 0.58141E+00_JPRB, 0.58145E+00_JPRB, 0.59189E+00_JPRB ,&
&  0.60427E+00_JPRB, 0.61852E+00_JPRB, 0.63293E+00_JPRB, 0.64412E+00_JPRB, 0.65170E+00_JPRB, 0.65808E+00_JPRB, 0.67501E+00_JPRB ,&
&  0.68254E+00_JPRB, 0.51113E+00_JPRB, 0.52842E+00_JPRB, 0.60770E+00_JPRB, 0.65904E+00_JPRB, 0.72660E+00_JPRB, 0.77009E+00_JPRB ,&
&  0.78160E+00_JPRB, 0.77444E+00_JPRB, 0.75589E+00_JPRB, 0.73103E+00_JPRB, 0.69978E+00_JPRB, 0.66851E+00_JPRB, 0.63613E+00_JPRB ,&
&  0.61472E+00_JPRB, 0.59831E+00_JPRB, 0.58985E+00_JPRB, 0.57816E+00_JPRB, 0.57215E+00_JPRB, 0.56984E+00_JPRB, 0.56760E+00_JPRB ,&
&  0.56642E+00_JPRB, 0.56329E+00_JPRB, 0.56720E+00_JPRB, 0.57457E+00_JPRB, 0.57651E+00_JPRB, 0.58473E+00_JPRB, 0.58711E+00_JPRB ,&
&  0.59618E+00_JPRB, 0.60954E+00_JPRB, 0.61288E+00_JPRB, 0.60972E+00_JPRB, 0.61043E+00_JPRB, 0.61845E+00_JPRB, 0.63001E+00_JPRB ,&
&  0.63642E+00_JPRB, 0.65268E+00_JPRB, 0.66394E+00_JPRB, 0.67828E+00_JPRB, 0.68717E+00_JPRB, 0.68318E+00_JPRB, 0.67028E+00_JPRB ,&
&  0.67115E+00_JPRB, 0.66468E+00_JPRB, 0.65683E+00_JPRB, 0.64727E+00_JPRB, 0.63682E+00_JPRB, 0.62665E+00_JPRB, 0.61752E+00_JPRB ,&
&  0.61389E+00_JPRB, 0.62005E+00_JPRB, 0.62665E+00_JPRB, 0.64626E+00_JPRB, 0.65843E+00_JPRB, 0.66114E+00_JPRB, 0.66396E+00_JPRB ,&
&  0.66871E+00_JPRB, 0.67066E+00_JPRB, 0.67375E+00_JPRB, 0.63990E+00_JPRB, 0.68721E+00_JPRB, 0.69574E+00_JPRB, 0.68899E+00_JPRB ,&
&  0.68059E+00_JPRB, 0.65546E+00_JPRB, 0.65010E+00_JPRB, 0.52757E+00_JPRB, 0.60518E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.73025E+00_JPRB, 0.72635E+00_JPRB, 0.69701E+00_JPRB, 0.65919E+00_JPRB, 0.62620E+00_JPRB, 0.60015E+00_JPRB, 0.58767E+00_JPRB ,&
&  0.57692E+00_JPRB, 0.56832E+00_JPRB, 0.57142E+00_JPRB, 0.57985E+00_JPRB, 0.58972E+00_JPRB, 0.59082E+00_JPRB, 0.58457E+00_JPRB ,&
&  0.57508E+00_JPRB, 0.56227E+00_JPRB, 0.55658E+00_JPRB, 0.55448E+00_JPRB, 0.54907E+00_JPRB, 0.54865E+00_JPRB, 0.54991E+00_JPRB ,&
&  0.55557E+00_JPRB, 0.56275E+00_JPRB, 0.56726E+00_JPRB, 0.57701E+00_JPRB, 0.59174E+00_JPRB, 0.61311E+00_JPRB, 0.63402E+00_JPRB ,&
&  0.63153E+00_JPRB, 0.52414E+00_JPRB, 0.56103E+00_JPRB, 0.64109E+00_JPRB, 0.71357E+00_JPRB, 0.75525E+00_JPRB, 0.77604E+00_JPRB ,&
&  0.77652E+00_JPRB, 0.76172E+00_JPRB, 0.73811E+00_JPRB, 0.71005E+00_JPRB, 0.67844E+00_JPRB, 0.64842E+00_JPRB, 0.61770E+00_JPRB ,&
&  0.59539E+00_JPRB, 0.57904E+00_JPRB, 0.57086E+00_JPRB, 0.56565E+00_JPRB, 0.56246E+00_JPRB, 0.55185E+00_JPRB, 0.54558E+00_JPRB ,&
&  0.54722E+00_JPRB, 0.55011E+00_JPRB, 0.55512E+00_JPRB, 0.56042E+00_JPRB, 0.57452E+00_JPRB, 0.59401E+00_JPRB, 0.60172E+00_JPRB ,&
&  0.60355E+00_JPRB, 0.61195E+00_JPRB, 0.61723E+00_JPRB, 0.61193E+00_JPRB, 0.61010E+00_JPRB, 0.63278E+00_JPRB, 0.65099E+00_JPRB ,&
&  0.66896E+00_JPRB, 0.69018E+00_JPRB, 0.71559E+00_JPRB, 0.72661E+00_JPRB, 0.72221E+00_JPRB, 0.70908E+00_JPRB, 0.68479E+00_JPRB ,&
&  0.67273E+00_JPRB, 0.65729E+00_JPRB, 0.64324E+00_JPRB, 0.63588E+00_JPRB, 0.62911E+00_JPRB, 0.61936E+00_JPRB, 0.60883E+00_JPRB ,&
&  0.60253E+00_JPRB, 0.59751E+00_JPRB, 0.60323E+00_JPRB, 0.62442E+00_JPRB, 0.64525E+00_JPRB, 0.65745E+00_JPRB, 0.66074E+00_JPRB ,&
&  0.65853E+00_JPRB, 0.65722E+00_JPRB, 0.66313E+00_JPRB, 0.70721E+00_JPRB, 0.69719E+00_JPRB, 0.69684E+00_JPRB, 0.68771E+00_JPRB ,&
&  0.67845E+00_JPRB, 0.66575E+00_JPRB, 0.64511E+00_JPRB, 0.63526E+00_JPRB, 0.65804E+00_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.78764E+00_JPRB, 0.73927E+00_JPRB, 0.69549E+00_JPRB, 0.64916E+00_JPRB, 0.61639E+00_JPRB, 0.60376E+00_JPRB, 0.59606E+00_JPRB ,&
&  0.58111E+00_JPRB, 0.56670E+00_JPRB, 0.56601E+00_JPRB, 0.56834E+00_JPRB, 0.56588E+00_JPRB, 0.55929E+00_JPRB, 0.54719E+00_JPRB ,&
&  0.54259E+00_JPRB, 0.54814E+00_JPRB, 0.54539E+00_JPRB, 0.53922E+00_JPRB, 0.53257E+00_JPRB, 0.52737E+00_JPRB, 0.52388E+00_JPRB ,&
&  0.52285E+00_JPRB, 0.52781E+00_JPRB, 0.53131E+00_JPRB, 0.54319E+00_JPRB, 0.56260E+00_JPRB, 0.58455E+00_JPRB, 0.61063E+00_JPRB ,&
&  0.60356E+00_JPRB, 0.50927E+00_JPRB, 0.60511E+00_JPRB, 0.71930E+00_JPRB, 0.79911E+00_JPRB, 0.81957E+00_JPRB, 0.79110E+00_JPRB ,&
&  0.77462E+00_JPRB, 0.73449E+00_JPRB, 0.70242E+00_JPRB, 0.67951E+00_JPRB, 0.65647E+00_JPRB, 0.63662E+00_JPRB, 0.62106E+00_JPRB ,&
&  0.60098E+00_JPRB, 0.58530E+00_JPRB, 0.56979E+00_JPRB, 0.55474E+00_JPRB, 0.55403E+00_JPRB, 0.55096E+00_JPRB, 0.54498E+00_JPRB ,&
&  0.54177E+00_JPRB, 0.54407E+00_JPRB, 0.54784E+00_JPRB, 0.55481E+00_JPRB, 0.56653E+00_JPRB, 0.60722E+00_JPRB, 0.63526E+00_JPRB ,&
&  0.64487E+00_JPRB, 0.65830E+00_JPRB, 0.65933E+00_JPRB, 0.65692E+00_JPRB, 0.66579E+00_JPRB, 0.68013E+00_JPRB, 0.70751E+00_JPRB ,&
&  0.74091E+00_JPRB, 0.75675E+00_JPRB, 0.76158E+00_JPRB, 0.76666E+00_JPRB, 0.75503E+00_JPRB, 0.74086E+00_JPRB, 0.72269E+00_JPRB ,&
&  0.70542E+00_JPRB, 0.68147E+00_JPRB, 0.66296E+00_JPRB, 0.65090E+00_JPRB, 0.63548E+00_JPRB, 0.62408E+00_JPRB, 0.61282E+00_JPRB ,&
&  0.60271E+00_JPRB, 0.59327E+00_JPRB, 0.60632E+00_JPRB, 0.62359E+00_JPRB, 0.63463E+00_JPRB, 0.63734E+00_JPRB, 0.64646E+00_JPRB ,&
&  0.65220E+00_JPRB, 0.66729E+00_JPRB, 0.65359E+00_JPRB, 0.69162E+00_JPRB, 0.74243E+00_JPRB, 0.72995E+00_JPRB, 0.71878E+00_JPRB ,&
&  0.70100E+00_JPRB, 0.68239E+00_JPRB, 0.65932E+00_JPRB, 0.65327E+00_JPRB, 0.78193E+00_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.67891E+00_JPRB, 0.64472E+00_JPRB, 0.60975E+00_JPRB, 0.58834E+00_JPRB, 0.57100E+00_JPRB, 0.55617E+00_JPRB, 0.55533E+00_JPRB ,&
&  0.55236E+00_JPRB, 0.54511E+00_JPRB, 0.55156E+00_JPRB, 0.54981E+00_JPRB, 0.54280E+00_JPRB, 0.52635E+00_JPRB, 0.52892E+00_JPRB ,&
&  0.53381E+00_JPRB, 0.53026E+00_JPRB, 0.52380E+00_JPRB, 0.51779E+00_JPRB, 0.51320E+00_JPRB, 0.50535E+00_JPRB, 0.50088E+00_JPRB ,&
&  0.49717E+00_JPRB, 0.49933E+00_JPRB, 0.50830E+00_JPRB, 0.52286E+00_JPRB, 0.54617E+00_JPRB, 0.57418E+00_JPRB, 0.60162E+00_JPRB ,&
&  0.58642E+00_JPRB, 0.55663E+00_JPRB, 0.70107E+00_JPRB, 0.91408E+00_JPRB, 0.95137E+00_JPRB, 0.86149E+00_JPRB, 0.79186E+00_JPRB ,&
&  0.74675E+00_JPRB, 0.70036E+00_JPRB, 0.67147E+00_JPRB, 0.65418E+00_JPRB, 0.63605E+00_JPRB, 0.62752E+00_JPRB, 0.61685E+00_JPRB ,&
&  0.60341E+00_JPRB, 0.58492E+00_JPRB, 0.56170E+00_JPRB, 0.54887E+00_JPRB, 0.54427E+00_JPRB, 0.54569E+00_JPRB, 0.54264E+00_JPRB ,&
&  0.54039E+00_JPRB, 0.54956E+00_JPRB, 0.55450E+00_JPRB, 0.55755E+00_JPRB, 0.59420E+00_JPRB, 0.66853E+00_JPRB, 0.74248E+00_JPRB ,&
&  0.73025E+00_JPRB, 0.72344E+00_JPRB, 0.72392E+00_JPRB, 0.72641E+00_JPRB, 0.74229E+00_JPRB, 0.75216E+00_JPRB, 0.77005E+00_JPRB ,&
&  0.78909E+00_JPRB, 0.78246E+00_JPRB, 0.76163E+00_JPRB, 0.74577E+00_JPRB, 0.72832E+00_JPRB, 0.72279E+00_JPRB, 0.70077E+00_JPRB ,&
&  0.68054E+00_JPRB, 0.66570E+00_JPRB, 0.65788E+00_JPRB, 0.63640E+00_JPRB, 0.62311E+00_JPRB, 0.61051E+00_JPRB, 0.60323E+00_JPRB ,&
&  0.59050E+00_JPRB, 0.58556E+00_JPRB, 0.61417E+00_JPRB, 0.62964E+00_JPRB, 0.63281E+00_JPRB, 0.63460E+00_JPRB, 0.64640E+00_JPRB ,&
&  0.66255E+00_JPRB, 0.67730E+00_JPRB, 0.68595E+00_JPRB, 0.70610E+00_JPRB, 0.69435E+00_JPRB, 0.74792E+00_JPRB, 0.73045E+00_JPRB ,&
&  0.71238E+00_JPRB, 0.67886E+00_JPRB, 0.64108E+00_JPRB, 0.62743E+00_JPRB, 0.68957E+00_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.63404E+00_JPRB, 0.59281E+00_JPRB, 0.56108E+00_JPRB, 0.54339E+00_JPRB, 0.52488E+00_JPRB, 0.51828E+00_JPRB, 0.51422E+00_JPRB ,&
&  0.52258E+00_JPRB, 0.52904E+00_JPRB, 0.54278E+00_JPRB, 0.53973E+00_JPRB, 0.51911E+00_JPRB, 0.50997E+00_JPRB, 0.51741E+00_JPRB ,&
&  0.51119E+00_JPRB, 0.51168E+00_JPRB, 0.51669E+00_JPRB, 0.51190E+00_JPRB, 0.50651E+00_JPRB, 0.49868E+00_JPRB, 0.48862E+00_JPRB ,&
&  0.49308E+00_JPRB, 0.50032E+00_JPRB, 0.51454E+00_JPRB, 0.53556E+00_JPRB, 0.56265E+00_JPRB, 0.59029E+00_JPRB, 0.60960E+00_JPRB ,&
&  0.60416E+00_JPRB, 0.66337E+00_JPRB, 0.90386E+00_JPRB, 0.10893E+01_JPRB, 0.10285E+01_JPRB, 0.85962E+00_JPRB, 0.78002E+00_JPRB ,&
&  0.72435E+00_JPRB, 0.67854E+00_JPRB, 0.65875E+00_JPRB, 0.64675E+00_JPRB, 0.62896E+00_JPRB, 0.62044E+00_JPRB, 0.62243E+00_JPRB ,&
&  0.61003E+00_JPRB, 0.58885E+00_JPRB, 0.56747E+00_JPRB, 0.55116E+00_JPRB, 0.54194E+00_JPRB, 0.54396E+00_JPRB, 0.54855E+00_JPRB ,&
&  0.55473E+00_JPRB, 0.56281E+00_JPRB, 0.57492E+00_JPRB, 0.57981E+00_JPRB, 0.59143E+00_JPRB, 0.58371E+00_JPRB, 0.72154E+00_JPRB ,&
&  0.90520E+00_JPRB, 0.88098E+00_JPRB, 0.85780E+00_JPRB, 0.84219E+00_JPRB, 0.82826E+00_JPRB, 0.80645E+00_JPRB, 0.80393E+00_JPRB ,&
&  0.78931E+00_JPRB, 0.75879E+00_JPRB, 0.73080E+00_JPRB, 0.71458E+00_JPRB, 0.69307E+00_JPRB, 0.67773E+00_JPRB, 0.65071E+00_JPRB ,&
&  0.63573E+00_JPRB, 0.62382E+00_JPRB, 0.61478E+00_JPRB, 0.60882E+00_JPRB, 0.60832E+00_JPRB, 0.59968E+00_JPRB, 0.59935E+00_JPRB ,&
&  0.59508E+00_JPRB, 0.61162E+00_JPRB, 0.63591E+00_JPRB, 0.62442E+00_JPRB, 0.63076E+00_JPRB, 0.65286E+00_JPRB, 0.66433E+00_JPRB ,&
&  0.64705E+00_JPRB, 0.65354E+00_JPRB, 0.68556E+00_JPRB, 0.69507E+00_JPRB, 0.68111E+00_JPRB, 0.74119E+00_JPRB, 0.71618E+00_JPRB ,&
&  0.68740E+00_JPRB, 0.65490E+00_JPRB, 0.63405E+00_JPRB, 0.61604E+00_JPRB, 0.65017E+00_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.59156E+00_JPRB, 0.55483E+00_JPRB, 0.53215E+00_JPRB, 0.51394E+00_JPRB, 0.51967E+00_JPRB, 0.51489E+00_JPRB, 0.50916E+00_JPRB ,&
&  0.52282E+00_JPRB, 0.53095E+00_JPRB, 0.54227E+00_JPRB, 0.53162E+00_JPRB, 0.51083E+00_JPRB, 0.50575E+00_JPRB, 0.50045E+00_JPRB ,&
&  0.49213E+00_JPRB, 0.51168E+00_JPRB, 0.53039E+00_JPRB, 0.54088E+00_JPRB, 0.53398E+00_JPRB, 0.51165E+00_JPRB, 0.49974E+00_JPRB ,&
&  0.50696E+00_JPRB, 0.52417E+00_JPRB, 0.54992E+00_JPRB, 0.57831E+00_JPRB, 0.60372E+00_JPRB, 0.62750E+00_JPRB, 0.65462E+00_JPRB ,&
&  0.70911E+00_JPRB, 0.79991E+00_JPRB, 0.10462E+01_JPRB, 0.12849E+01_JPRB, 0.10671E+01_JPRB, 0.85055E+00_JPRB, 0.71715E+00_JPRB ,&
&  0.71443E+00_JPRB, 0.67500E+00_JPRB, 0.66866E+00_JPRB, 0.66829E+00_JPRB, 0.65541E+00_JPRB, 0.64189E+00_JPRB, 0.63458E+00_JPRB ,&
&  0.61281E+00_JPRB, 0.59564E+00_JPRB, 0.58412E+00_JPRB, 0.57546E+00_JPRB, 0.57130E+00_JPRB, 0.57190E+00_JPRB, 0.57494E+00_JPRB ,&
&  0.58807E+00_JPRB, 0.59657E+00_JPRB, 0.60473E+00_JPRB, 0.61149E+00_JPRB, 0.59385E+00_JPRB, 0.62687E+00_JPRB, 0.70721E+00_JPRB ,&
&  0.80895E+00_JPRB, 0.95067E+00_JPRB, 0.90156E+00_JPRB, 0.86819E+00_JPRB, 0.84120E+00_JPRB, 0.82272E+00_JPRB, 0.79512E+00_JPRB ,&
&  0.77580E+00_JPRB, 0.74408E+00_JPRB, 0.71698E+00_JPRB, 0.69062E+00_JPRB, 0.65599E+00_JPRB, 0.63456E+00_JPRB, 0.62699E+00_JPRB ,&
&  0.61343E+00_JPRB, 0.61176E+00_JPRB, 0.60052E+00_JPRB, 0.60608E+00_JPRB, 0.60553E+00_JPRB, 0.60779E+00_JPRB, 0.61252E+00_JPRB ,&
&  0.62900E+00_JPRB, 0.67159E+00_JPRB, 0.68213E+00_JPRB, 0.63520E+00_JPRB, 0.61388E+00_JPRB, 0.62175E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.64786E+00_JPRB, 0.67257E+00_JPRB, 0.68523E+00_JPRB, 0.70152E+00_JPRB, 0.66793E+00_JPRB, 0.72269E+00_JPRB, 0.69481E+00_JPRB ,&
&  0.66250E+00_JPRB, 0.64331E+00_JPRB, 0.63552E+00_JPRB, 0.62829E+00_JPRB, 0.63189E+00_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.59765E+00_JPRB, 0.56752E+00_JPRB, 0.52905E+00_JPRB, 0.51322E+00_JPRB, 0.52195E+00_JPRB, 0.50803E+00_JPRB, 0.51170E+00_JPRB ,&
&  0.52441E+00_JPRB, 0.52161E+00_JPRB, 0.51992E+00_JPRB, 0.51212E+00_JPRB, 0.50756E+00_JPRB, 0.50098E+00_JPRB, 0.50067E+00_JPRB ,&
&  0.51247E+00_JPRB, 0.54063E+00_JPRB, 0.54494E+00_JPRB, 0.53362E+00_JPRB, 0.52761E+00_JPRB, 0.51394E+00_JPRB, 0.50905E+00_JPRB ,&
&  0.53263E+00_JPRB, 0.57567E+00_JPRB, 0.61940E+00_JPRB, 0.64388E+00_JPRB, 0.66339E+00_JPRB, 0.68374E+00_JPRB, 0.75596E+00_JPRB ,&
&  0.92155E+00_JPRB, 0.18894E+01_JPRB, 0.88990E+00_JPRB, 0.14021E+01_JPRB, 0.10373E+01_JPRB, 0.80329E+00_JPRB, 0.64503E+00_JPRB ,&
&  0.64009E+00_JPRB, 0.68957E+00_JPRB, 0.69242E+00_JPRB, 0.71618E+00_JPRB, 0.69401E+00_JPRB, 0.65104E+00_JPRB, 0.63021E+00_JPRB ,&
&  0.61419E+00_JPRB, 0.61071E+00_JPRB, 0.61638E+00_JPRB, 0.62101E+00_JPRB, 0.62165E+00_JPRB, 0.61948E+00_JPRB, 0.61990E+00_JPRB ,&
&  0.62831E+00_JPRB, 0.62749E+00_JPRB, 0.63774E+00_JPRB, 0.64298E+00_JPRB, 0.61148E+00_JPRB, 0.65652E+00_JPRB, 0.73763E+00_JPRB ,&
&  0.76903E+00_JPRB, 0.93237E+00_JPRB, 0.86317E+00_JPRB, 0.84026E+00_JPRB, 0.82703E+00_JPRB, 0.79521E+00_JPRB, 0.75481E+00_JPRB ,&
&  0.71786E+00_JPRB, 0.69820E+00_JPRB, 0.67575E+00_JPRB, 0.65428E+00_JPRB, 0.62769E+00_JPRB, 0.61760E+00_JPRB, 0.60357E+00_JPRB ,&
&  0.60606E+00_JPRB, 0.62340E+00_JPRB, 0.61720E+00_JPRB, 0.62173E+00_JPRB, 0.62390E+00_JPRB, 0.63897E+00_JPRB, 0.65492E+00_JPRB ,&
&  0.68437E+00_JPRB, 0.72903E+00_JPRB, 0.70744E+00_JPRB, 0.62201E+00_JPRB, 0.58989E+00_JPRB, 0.60147E+00_JPRB, 0.61421E+00_JPRB ,&
&  0.65685E+00_JPRB, 0.68426E+00_JPRB, 0.68299E+00_JPRB, 0.68429E+00_JPRB, 0.68015E+00_JPRB, 0.71703E+00_JPRB, 0.68317E+00_JPRB ,&
&  0.64931E+00_JPRB, 0.64352E+00_JPRB, 0.64466E+00_JPRB, 0.63326E+00_JPRB, 0.63590E+00_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.60220E+00_JPRB, 0.58551E+00_JPRB, 0.54929E+00_JPRB, 0.51804E+00_JPRB, 0.51784E+00_JPRB, 0.52229E+00_JPRB, 0.52791E+00_JPRB ,&
&  0.52530E+00_JPRB, 0.52060E+00_JPRB, 0.51542E+00_JPRB, 0.51120E+00_JPRB, 0.52023E+00_JPRB, 0.51987E+00_JPRB, 0.54719E+00_JPRB ,&
&  0.56512E+00_JPRB, 0.55699E+00_JPRB, 0.55902E+00_JPRB, 0.53754E+00_JPRB, 0.53427E+00_JPRB, 0.52986E+00_JPRB, 0.55067E+00_JPRB ,&
&  0.60258E+00_JPRB, 0.65779E+00_JPRB, 0.68987E+00_JPRB, 0.70195E+00_JPRB, 0.72088E+00_JPRB, 0.75251E+00_JPRB, 0.85912E+00_JPRB ,&
&  0.11260E+01_JPRB, 0.21627E+01_JPRB, 0.11383E+01_JPRB, 0.14323E+01_JPRB, 0.92934E+00_JPRB, 0.69209E+00_JPRB, 0.59079E+00_JPRB ,&
&  0.60287E+00_JPRB, 0.68577E+00_JPRB, 0.70053E+00_JPRB, 0.72783E+00_JPRB, 0.65918E+00_JPRB, 0.61910E+00_JPRB, 0.60686E+00_JPRB ,&
&  0.61258E+00_JPRB, 0.64625E+00_JPRB, 0.67245E+00_JPRB, 0.68479E+00_JPRB, 0.67654E+00_JPRB, 0.66189E+00_JPRB, 0.66450E+00_JPRB ,&
&  0.66595E+00_JPRB, 0.65433E+00_JPRB, 0.66416E+00_JPRB, 0.63695E+00_JPRB, 0.66532E+00_JPRB, 0.69775E+00_JPRB, 0.71977E+00_JPRB ,&
&  0.74580E+00_JPRB, 0.83832E+00_JPRB, 0.82814E+00_JPRB, 0.82918E+00_JPRB, 0.79934E+00_JPRB, 0.75956E+00_JPRB, 0.71477E+00_JPRB ,&
&  0.69107E+00_JPRB, 0.67303E+00_JPRB, 0.65411E+00_JPRB, 0.62686E+00_JPRB, 0.61455E+00_JPRB, 0.61946E+00_JPRB, 0.61149E+00_JPRB ,&
&  0.63299E+00_JPRB, 0.65809E+00_JPRB, 0.65838E+00_JPRB, 0.66474E+00_JPRB, 0.67516E+00_JPRB, 0.69207E+00_JPRB, 0.70992E+00_JPRB ,&
&  0.73836E+00_JPRB, 0.77175E+00_JPRB, 0.71618E+00_JPRB, 0.61981E+00_JPRB, 0.60743E+00_JPRB, 0.61578E+00_JPRB, 0.61400E+00_JPRB ,&
&  0.65019E+00_JPRB, 0.67773E+00_JPRB, 0.68092E+00_JPRB, 0.66117E+00_JPRB, 0.66987E+00_JPRB, 0.70092E+00_JPRB, 0.67299E+00_JPRB ,&
&  0.66047E+00_JPRB, 0.66280E+00_JPRB, 0.67560E+00_JPRB, 0.67547E+00_JPRB, 0.64891E+00_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.56631E+00_JPRB, 0.55498E+00_JPRB, 0.54040E+00_JPRB, 0.52621E+00_JPRB, 0.51910E+00_JPRB, 0.52651E+00_JPRB, 0.53092E+00_JPRB ,&
&  0.52270E+00_JPRB, 0.51821E+00_JPRB, 0.51178E+00_JPRB, 0.51780E+00_JPRB, 0.54499E+00_JPRB, 0.59328E+00_JPRB, 0.63735E+00_JPRB ,&
&  0.63190E+00_JPRB, 0.59507E+00_JPRB, 0.58858E+00_JPRB, 0.58172E+00_JPRB, 0.59531E+00_JPRB, 0.61337E+00_JPRB, 0.64879E+00_JPRB ,&
&  0.69539E+00_JPRB, 0.72694E+00_JPRB, 0.74426E+00_JPRB, 0.75768E+00_JPRB, 0.78313E+00_JPRB, 0.80695E+00_JPRB, 0.96410E+00_JPRB ,&
&  0.12980E+01_JPRB, 0.25023E+01_JPRB, 0.21038E+01_JPRB, 0.12217E+01_JPRB, 0.78347E+00_JPRB, 0.63515E+00_JPRB, 0.60807E+00_JPRB ,&
&  0.55320E+00_JPRB, 0.48617E+00_JPRB, 0.68676E+00_JPRB, 0.71896E+00_JPRB, 0.65998E+00_JPRB, 0.63247E+00_JPRB, 0.64444E+00_JPRB ,&
&  0.68270E+00_JPRB, 0.71381E+00_JPRB, 0.72996E+00_JPRB, 0.72584E+00_JPRB, 0.70583E+00_JPRB, 0.69911E+00_JPRB, 0.70470E+00_JPRB ,&
&  0.70415E+00_JPRB, 0.70411E+00_JPRB, 0.74962E+00_JPRB, 0.71634E+00_JPRB, 0.67887E+00_JPRB, 0.66592E+00_JPRB, 0.65843E+00_JPRB ,&
&  0.70225E+00_JPRB, 0.75660E+00_JPRB, 0.81316E+00_JPRB, 0.82079E+00_JPRB, 0.72795E+00_JPRB, 0.75739E+00_JPRB, 0.70431E+00_JPRB ,&
&  0.69979E+00_JPRB, 0.67829E+00_JPRB, 0.65414E+00_JPRB, 0.62099E+00_JPRB, 0.63671E+00_JPRB, 0.64940E+00_JPRB, 0.63982E+00_JPRB ,&
&  0.67042E+00_JPRB, 0.70406E+00_JPRB, 0.69374E+00_JPRB, 0.70362E+00_JPRB, 0.71138E+00_JPRB, 0.72912E+00_JPRB, 0.74389E+00_JPRB ,&
&  0.75426E+00_JPRB, 0.78199E+00_JPRB, 0.73071E+00_JPRB, 0.67846E+00_JPRB, 0.65766E+00_JPRB, 0.62958E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.64260E+00_JPRB, 0.68593E+00_JPRB, 0.68865E+00_JPRB, 0.69418E+00_JPRB, 0.68098E+00_JPRB, 0.68353E+00_JPRB, 0.67546E+00_JPRB ,&
&  0.68534E+00_JPRB, 0.69446E+00_JPRB, 0.75045E+00_JPRB, 0.69472E+00_JPRB, 0.61700E+00_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.51822E+00_JPRB, 0.51590E+00_JPRB, 0.50945E+00_JPRB, 0.51294E+00_JPRB, 0.51693E+00_JPRB, 0.51326E+00_JPRB, 0.51638E+00_JPRB ,&
&  0.52442E+00_JPRB, 0.53340E+00_JPRB, 0.53160E+00_JPRB, 0.56227E+00_JPRB, 0.63006E+00_JPRB, 0.70131E+00_JPRB, 0.69845E+00_JPRB ,&
&  0.68081E+00_JPRB, 0.66281E+00_JPRB, 0.65373E+00_JPRB, 0.66248E+00_JPRB, 0.66839E+00_JPRB, 0.69974E+00_JPRB, 0.73107E+00_JPRB ,&
&  0.75878E+00_JPRB, 0.78570E+00_JPRB, 0.81094E+00_JPRB, 0.84405E+00_JPRB, 0.89415E+00_JPRB, 0.95516E+00_JPRB, 0.11082E+01_JPRB ,&
&  0.13798E+01_JPRB, 0.23191E+01_JPRB, 0.79261E+00_JPRB, 0.80060E+00_JPRB, 0.74378E+00_JPRB, 0.62379E+00_JPRB, 0.54037E+00_JPRB ,&
&  0.46731E+00_JPRB, 0.41526E+00_JPRB, 0.59945E+00_JPRB, 0.71645E+00_JPRB, 0.68066E+00_JPRB, 0.68932E+00_JPRB, 0.71685E+00_JPRB ,&
&  0.75801E+00_JPRB, 0.77484E+00_JPRB, 0.77616E+00_JPRB, 0.75620E+00_JPRB, 0.73933E+00_JPRB, 0.73783E+00_JPRB, 0.75937E+00_JPRB ,&
&  0.78270E+00_JPRB, 0.81336E+00_JPRB, 0.82282E+00_JPRB, 0.70859E+00_JPRB, 0.67426E+00_JPRB, 0.63348E+00_JPRB, 0.60773E+00_JPRB ,&
&  0.59265E+00_JPRB, 0.68326E+00_JPRB, 0.79845E+00_JPRB, 0.82517E+00_JPRB, 0.72647E+00_JPRB, 0.67757E+00_JPRB, 0.71422E+00_JPRB ,&
&  0.69745E+00_JPRB, 0.66041E+00_JPRB, 0.63416E+00_JPRB, 0.61212E+00_JPRB, 0.62506E+00_JPRB, 0.65649E+00_JPRB, 0.65033E+00_JPRB ,&
&  0.67900E+00_JPRB, 0.71322E+00_JPRB, 0.69320E+00_JPRB, 0.71763E+00_JPRB, 0.73608E+00_JPRB, 0.74626E+00_JPRB, 0.74928E+00_JPRB ,&
&  0.75724E+00_JPRB, 0.78412E+00_JPRB, 0.79680E+00_JPRB, 0.77323E+00_JPRB, 0.71569E+00_JPRB, 0.67690E+00_JPRB, 0.68137E+00_JPRB ,&
&  0.68379E+00_JPRB, 0.70385E+00_JPRB, 0.69591E+00_JPRB, 0.66845E+00_JPRB, 0.69121E+00_JPRB, 0.70055E+00_JPRB, 0.73343E+00_JPRB ,&
&  0.77754E+00_JPRB, 0.86945E+00_JPRB, 0.92508E+00_JPRB, 0.64919E+00_JPRB, 0.55993E+00_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.49393E+00_JPRB, 0.49296E+00_JPRB, 0.48220E+00_JPRB, 0.49256E+00_JPRB, 0.49275E+00_JPRB, 0.50100E+00_JPRB, 0.51683E+00_JPRB ,&
&  0.54347E+00_JPRB, 0.57729E+00_JPRB, 0.60752E+00_JPRB, 0.64635E+00_JPRB, 0.70328E+00_JPRB, 0.75279E+00_JPRB, 0.74757E+00_JPRB ,&
&  0.72798E+00_JPRB, 0.71600E+00_JPRB, 0.71203E+00_JPRB, 0.71061E+00_JPRB, 0.72758E+00_JPRB, 0.75250E+00_JPRB, 0.77496E+00_JPRB ,&
&  0.80578E+00_JPRB, 0.84764E+00_JPRB, 0.88662E+00_JPRB, 0.93861E+00_JPRB, 0.99677E+00_JPRB, 0.10873E+01_JPRB, 0.11838E+01_JPRB ,&
&  0.14706E+01_JPRB, 0.12527E+01_JPRB, 0.58710E+00_JPRB, 0.79621E+00_JPRB, 0.69861E+00_JPRB, 0.56344E+00_JPRB, 0.45190E+00_JPRB ,&
&  0.38900E+00_JPRB, 0.45171E+00_JPRB, 0.62689E+00_JPRB, 0.73203E+00_JPRB, 0.72066E+00_JPRB, 0.75033E+00_JPRB, 0.78109E+00_JPRB ,&
&  0.80556E+00_JPRB, 0.81201E+00_JPRB, 0.80381E+00_JPRB, 0.79094E+00_JPRB, 0.79168E+00_JPRB, 0.80820E+00_JPRB, 0.84372E+00_JPRB ,&
&  0.87790E+00_JPRB, 0.89156E+00_JPRB, 0.87591E+00_JPRB, 0.73675E+00_JPRB, 0.70094E+00_JPRB, 0.61884E+00_JPRB, 0.56964E+00_JPRB ,&
&  0.52936E+00_JPRB, 0.59724E+00_JPRB, 0.71783E+00_JPRB, 0.77387E+00_JPRB, 0.70535E+00_JPRB, 0.64783E+00_JPRB, 0.71788E+00_JPRB ,&
&  0.68373E+00_JPRB, 0.63573E+00_JPRB, 0.62292E+00_JPRB, 0.61698E+00_JPRB, 0.62368E+00_JPRB, 0.64466E+00_JPRB, 0.66577E+00_JPRB ,&
&  0.68579E+00_JPRB, 0.68123E+00_JPRB, 0.65607E+00_JPRB, 0.67346E+00_JPRB, 0.69429E+00_JPRB, 0.71206E+00_JPRB, 0.72495E+00_JPRB ,&
&  0.74821E+00_JPRB, 0.77504E+00_JPRB, 0.79531E+00_JPRB, 0.81158E+00_JPRB, 0.77733E+00_JPRB, 0.73187E+00_JPRB, 0.73626E+00_JPRB ,&
&  0.69369E+00_JPRB, 0.68466E+00_JPRB, 0.68490E+00_JPRB, 0.68386E+00_JPRB, 0.67854E+00_JPRB, 0.72494E+00_JPRB, 0.75800E+00_JPRB ,&
&  0.80315E+00_JPRB, 0.84364E+00_JPRB, 0.77275E+00_JPRB, 0.58498E+00_JPRB, 0.52072E+00_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.50612E+00_JPRB, 0.50903E+00_JPRB, 0.49939E+00_JPRB, 0.49666E+00_JPRB, 0.48980E+00_JPRB, 0.49360E+00_JPRB, 0.50708E+00_JPRB ,&
&  0.53671E+00_JPRB, 0.58612E+00_JPRB, 0.64539E+00_JPRB, 0.69725E+00_JPRB, 0.76059E+00_JPRB, 0.80577E+00_JPRB, 0.79901E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.75998E+00_JPRB, 0.73689E+00_JPRB, 0.75036E+00_JPRB, 0.76187E+00_JPRB, 0.78435E+00_JPRB, 0.80901E+00_JPRB ,&
&  0.83732E+00_JPRB, 0.87253E+00_JPRB, 0.91362E+00_JPRB, 0.96082E+00_JPRB, 0.10307E+01_JPRB, 0.10957E+01_JPRB, 0.11269E+01_JPRB ,&
&  0.70044E+00_JPRB, 0.35740E+00_JPRB, 0.63871E+00_JPRB, 0.78201E+00_JPRB, 0.60091E+00_JPRB, 0.50560E+00_JPRB, 0.41139E+00_JPRB ,&
&  0.38087E+00_JPRB, 0.52092E+00_JPRB, 0.64816E+00_JPRB, 0.75517E+00_JPRB, 0.76548E+00_JPRB, 0.79299E+00_JPRB, 0.81935E+00_JPRB ,&
&  0.83141E+00_JPRB, 0.83775E+00_JPRB, 0.84041E+00_JPRB, 0.84132E+00_JPRB, 0.85439E+00_JPRB, 0.87426E+00_JPRB, 0.90437E+00_JPRB ,&
&  0.93783E+00_JPRB, 0.93142E+00_JPRB, 0.91411E+00_JPRB, 0.75228E+00_JPRB, 0.72374E+00_JPRB, 0.64418E+00_JPRB, 0.57705E+00_JPRB ,&
&  0.54401E+00_JPRB, 0.52149E+00_JPRB, 0.56425E+00_JPRB, 0.75397E+00_JPRB, 0.76119E+00_JPRB, 0.62898E+00_JPRB, 0.71263E+00_JPRB ,&
&  0.66211E+00_JPRB, 0.61499E+00_JPRB, 0.60438E+00_JPRB, 0.60641E+00_JPRB, 0.59828E+00_JPRB, 0.63178E+00_JPRB, 0.65431E+00_JPRB ,&
&  0.67362E+00_JPRB, 0.64934E+00_JPRB, 0.62242E+00_JPRB, 0.63122E+00_JPRB, 0.64923E+00_JPRB, 0.67048E+00_JPRB, 0.68660E+00_JPRB ,&
&  0.71209E+00_JPRB, 0.73075E+00_JPRB, 0.75361E+00_JPRB, 0.78268E+00_JPRB, 0.80944E+00_JPRB, 0.76840E+00_JPRB, 0.74839E+00_JPRB ,&
&  0.68060E+00_JPRB, 0.65428E+00_JPRB, 0.64247E+00_JPRB, 0.63926E+00_JPRB, 0.65008E+00_JPRB, 0.68347E+00_JPRB, 0.71815E+00_JPRB ,&
&  0.74937E+00_JPRB, 0.75551E+00_JPRB, 0.68829E+00_JPRB, 0.57754E+00_JPRB, 0.52205E+00_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.49818E+00_JPRB, 0.50125E+00_JPRB, 0.49813E+00_JPRB, 0.50574E+00_JPRB, 0.50278E+00_JPRB, 0.50117E+00_JPRB, 0.49703E+00_JPRB ,&
&  0.51920E+00_JPRB, 0.56257E+00_JPRB, 0.62874E+00_JPRB, 0.70242E+00_JPRB, 0.75479E+00_JPRB, 0.82078E+00_JPRB, 0.84521E+00_JPRB ,&
&  0.82677E+00_JPRB, 0.79426E+00_JPRB, 0.77872E+00_JPRB, 0.78995E+00_JPRB, 0.79444E+00_JPRB, 0.81248E+00_JPRB, 0.83165E+00_JPRB ,&
&  0.85131E+00_JPRB, 0.88224E+00_JPRB, 0.91444E+00_JPRB, 0.95073E+00_JPRB, 0.99795E+00_JPRB, 0.10039E+01_JPRB, 0.94347E+00_JPRB ,&
&  0.34986E+00_JPRB, 0.50345E+00_JPRB, 0.73094E+00_JPRB, 0.75614E+00_JPRB, 0.57834E+00_JPRB, 0.49250E+00_JPRB, 0.41684E+00_JPRB ,&
&  0.44882E+00_JPRB, 0.55006E+00_JPRB, 0.65366E+00_JPRB, 0.77486E+00_JPRB, 0.77167E+00_JPRB, 0.79744E+00_JPRB, 0.82959E+00_JPRB ,&
&  0.84067E+00_JPRB, 0.84888E+00_JPRB, 0.86992E+00_JPRB, 0.89447E+00_JPRB, 0.90296E+00_JPRB, 0.92892E+00_JPRB, 0.94706E+00_JPRB ,&
&  0.97086E+00_JPRB, 0.95199E+00_JPRB, 0.94206E+00_JPRB, 0.77888E+00_JPRB, 0.72812E+00_JPRB, 0.66176E+00_JPRB, 0.59303E+00_JPRB ,&
&  0.52575E+00_JPRB, 0.48645E+00_JPRB, 0.54236E+00_JPRB, 0.72341E+00_JPRB, 0.77484E+00_JPRB, 0.78437E+00_JPRB, 0.72236E+00_JPRB ,&
&  0.65533E+00_JPRB, 0.59212E+00_JPRB, 0.55305E+00_JPRB, 0.53983E+00_JPRB, 0.55642E+00_JPRB, 0.61012E+00_JPRB, 0.65055E+00_JPRB ,&
&  0.63209E+00_JPRB, 0.60956E+00_JPRB, 0.59813E+00_JPRB, 0.59359E+00_JPRB, 0.61727E+00_JPRB, 0.64334E+00_JPRB, 0.65299E+00_JPRB ,&
&  0.65359E+00_JPRB, 0.65761E+00_JPRB, 0.67510E+00_JPRB, 0.70029E+00_JPRB, 0.72561E+00_JPRB, 0.74480E+00_JPRB, 0.75630E+00_JPRB ,&
&  0.67323E+00_JPRB, 0.63322E+00_JPRB, 0.64388E+00_JPRB, 0.65309E+00_JPRB, 0.69878E+00_JPRB, 0.73853E+00_JPRB, 0.69397E+00_JPRB ,&
&  0.66603E+00_JPRB, 0.65309E+00_JPRB, 0.62406E+00_JPRB, 0.56323E+00_JPRB, 0.50737E+00_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.51164E+00_JPRB, 0.52595E+00_JPRB, 0.54435E+00_JPRB, 0.56602E+00_JPRB, 0.56106E+00_JPRB, 0.55964E+00_JPRB, 0.54587E+00_JPRB ,&
&  0.54220E+00_JPRB, 0.55839E+00_JPRB, 0.60009E+00_JPRB, 0.66981E+00_JPRB, 0.72068E+00_JPRB, 0.79191E+00_JPRB, 0.88525E+00_JPRB ,&
&  0.90139E+00_JPRB, 0.86940E+00_JPRB, 0.84505E+00_JPRB, 0.83455E+00_JPRB, 0.84255E+00_JPRB, 0.86034E+00_JPRB, 0.87424E+00_JPRB ,&
&  0.88639E+00_JPRB, 0.91979E+00_JPRB, 0.94323E+00_JPRB, 0.98262E+00_JPRB, 0.10200E+01_JPRB, 0.10190E+01_JPRB, 0.10069E+01_JPRB ,&
&  0.59950E+00_JPRB, 0.82396E+00_JPRB, 0.77441E+00_JPRB, 0.74307E+00_JPRB, 0.63053E+00_JPRB, 0.54551E+00_JPRB, 0.50917E+00_JPRB ,&
&  0.52753E+00_JPRB, 0.55769E+00_JPRB, 0.64288E+00_JPRB, 0.68897E+00_JPRB, 0.73572E+00_JPRB, 0.76579E+00_JPRB, 0.81241E+00_JPRB ,&
&  0.83192E+00_JPRB, 0.84987E+00_JPRB, 0.87196E+00_JPRB, 0.89354E+00_JPRB, 0.90483E+00_JPRB, 0.93599E+00_JPRB, 0.96078E+00_JPRB ,&
&  0.98238E+00_JPRB, 0.97454E+00_JPRB, 0.95011E+00_JPRB, 0.82257E+00_JPRB, 0.69110E+00_JPRB, 0.61890E+00_JPRB, 0.53396E+00_JPRB ,&
&  0.46611E+00_JPRB, 0.42518E+00_JPRB, 0.53617E+00_JPRB, 0.73009E+00_JPRB, 0.76339E+00_JPRB, 0.76218E+00_JPRB, 0.70928E+00_JPRB ,&
&  0.63725E+00_JPRB, 0.56437E+00_JPRB, 0.52764E+00_JPRB, 0.49626E+00_JPRB, 0.49701E+00_JPRB, 0.53946E+00_JPRB, 0.58401E+00_JPRB ,&
&  0.57514E+00_JPRB, 0.55716E+00_JPRB, 0.54912E+00_JPRB, 0.56242E+00_JPRB, 0.58967E+00_JPRB, 0.61184E+00_JPRB, 0.62763E+00_JPRB ,&
&  0.60278E+00_JPRB, 0.57000E+00_JPRB, 0.56283E+00_JPRB, 0.57714E+00_JPRB, 0.60444E+00_JPRB, 0.63583E+00_JPRB, 0.67231E+00_JPRB ,&
&  0.69717E+00_JPRB, 0.70981E+00_JPRB, 0.80841E+00_JPRB, 0.10038E+01_JPRB, 0.11457E+01_JPRB, 0.12352E+01_JPRB, 0.10168E+01_JPRB ,&
&  0.80749E+00_JPRB, 0.67436E+00_JPRB, 0.59465E+00_JPRB, 0.54156E+00_JPRB, 0.50097E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.50864E+00_JPRB, 0.55511E+00_JPRB, 0.59131E+00_JPRB, 0.64274E+00_JPRB, 0.65810E+00_JPRB, 0.65884E+00_JPRB, 0.65445E+00_JPRB ,&
&  0.64169E+00_JPRB, 0.64751E+00_JPRB, 0.66823E+00_JPRB, 0.71764E+00_JPRB, 0.76944E+00_JPRB, 0.81902E+00_JPRB, 0.87022E+00_JPRB ,&
&  0.93418E+00_JPRB, 0.96390E+00_JPRB, 0.93908E+00_JPRB, 0.93256E+00_JPRB, 0.94719E+00_JPRB, 0.96785E+00_JPRB, 0.96748E+00_JPRB ,&
&  0.95336E+00_JPRB, 0.95200E+00_JPRB, 0.95240E+00_JPRB, 0.97493E+00_JPRB, 0.10048E+01_JPRB, 0.10126E+01_JPRB, 0.69451E+00_JPRB ,&
&  0.86703E+00_JPRB, 0.79913E+00_JPRB, 0.76154E+00_JPRB, 0.75929E+00_JPRB, 0.71957E+00_JPRB, 0.65537E+00_JPRB, 0.61337E+00_JPRB ,&
&  0.59937E+00_JPRB, 0.59873E+00_JPRB, 0.61446E+00_JPRB, 0.63034E+00_JPRB, 0.68098E+00_JPRB, 0.70226E+00_JPRB, 0.74965E+00_JPRB ,&
&  0.79358E+00_JPRB, 0.80329E+00_JPRB, 0.81071E+00_JPRB, 0.81926E+00_JPRB, 0.82675E+00_JPRB, 0.84416E+00_JPRB, 0.86142E+00_JPRB ,&
&  0.87273E+00_JPRB, 0.87281E+00_JPRB, 0.84579E+00_JPRB, 0.78271E+00_JPRB, 0.68967E+00_JPRB, 0.61731E+00_JPRB, 0.52177E+00_JPRB ,&
&  0.41966E+00_JPRB, 0.40788E+00_JPRB, 0.58218E+00_JPRB, 0.76627E+00_JPRB, 0.75198E+00_JPRB, 0.73044E+00_JPRB, 0.67091E+00_JPRB ,&
&  0.58768E+00_JPRB, 0.54499E+00_JPRB, 0.52135E+00_JPRB, 0.50006E+00_JPRB, 0.48445E+00_JPRB, 0.49906E+00_JPRB, 0.52277E+00_JPRB ,&
&  0.52934E+00_JPRB, 0.52517E+00_JPRB, 0.51988E+00_JPRB, 0.53054E+00_JPRB, 0.54737E+00_JPRB, 0.62467E+00_JPRB, 0.78745E+00_JPRB ,&
&  0.95074E+00_JPRB, 0.47515E+00_JPRB, 0.45148E+00_JPRB, 0.47858E+00_JPRB, 0.52982E+00_JPRB, 0.57976E+00_JPRB, 0.64804E+00_JPRB ,&
&  0.80793E+00_JPRB, 0.10583E+01_JPRB, 0.10042E+01_JPRB, 0.12028E+01_JPRB, 0.24915E+01_JPRB, 0.27280E+01_JPRB, 0.15662E+01_JPRB ,&
&  0.97774E+00_JPRB, 0.68685E+00_JPRB, 0.55358E+00_JPRB, 0.51025E+00_JPRB, 0.49891E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.53628E+00_JPRB, 0.59656E+00_JPRB, 0.66261E+00_JPRB, 0.70227E+00_JPRB, 0.67306E+00_JPRB, 0.66378E+00_JPRB, 0.67386E+00_JPRB ,&
&  0.69286E+00_JPRB, 0.72110E+00_JPRB, 0.75186E+00_JPRB, 0.79340E+00_JPRB, 0.82201E+00_JPRB, 0.85857E+00_JPRB, 0.90742E+00_JPRB ,&
&  0.96099E+00_JPRB, 0.96295E+00_JPRB, 0.97576E+00_JPRB, 0.98971E+00_JPRB, 0.99908E+00_JPRB, 0.10126E+01_JPRB, 0.10210E+01_JPRB ,&
&  0.10192E+01_JPRB, 0.10228E+01_JPRB, 0.10144E+01_JPRB, 0.10273E+01_JPRB, 0.10213E+01_JPRB, 0.10873E+01_JPRB, 0.77658E+00_JPRB ,&
&  0.91656E+00_JPRB, 0.85066E+00_JPRB, 0.77254E+00_JPRB, 0.76824E+00_JPRB, 0.75109E+00_JPRB, 0.73007E+00_JPRB, 0.70435E+00_JPRB ,&
&  0.70142E+00_JPRB, 0.68262E+00_JPRB, 0.63133E+00_JPRB, 0.63627E+00_JPRB, 0.63080E+00_JPRB, 0.64461E+00_JPRB, 0.67352E+00_JPRB ,&
&  0.70470E+00_JPRB, 0.72896E+00_JPRB, 0.73311E+00_JPRB, 0.74708E+00_JPRB, 0.75079E+00_JPRB, 0.74302E+00_JPRB, 0.72945E+00_JPRB ,&
&  0.71302E+00_JPRB, 0.70373E+00_JPRB, 0.69633E+00_JPRB, 0.64940E+00_JPRB, 0.65378E+00_JPRB, 0.59997E+00_JPRB, 0.52077E+00_JPRB ,&
&  0.42352E+00_JPRB, 0.43912E+00_JPRB, 0.65051E+00_JPRB, 0.82130E+00_JPRB, 0.75714E+00_JPRB, 0.73343E+00_JPRB, 0.65147E+00_JPRB ,&
&  0.59594E+00_JPRB, 0.53659E+00_JPRB, 0.51755E+00_JPRB, 0.48501E+00_JPRB, 0.47145E+00_JPRB, 0.48195E+00_JPRB, 0.51285E+00_JPRB ,&
&  0.52673E+00_JPRB, 0.51832E+00_JPRB, 0.50906E+00_JPRB, 0.48552E+00_JPRB, 0.47907E+00_JPRB, 0.48774E+00_JPRB, 0.45237E+00_JPRB ,&
&  0.52443E+00_JPRB, 0.43267E+00_JPRB, 0.44880E+00_JPRB, 0.51485E+00_JPRB, 0.49563E+00_JPRB, 0.54304E+00_JPRB, 0.61261E+00_JPRB ,&
&  0.79871E+00_JPRB, 0.87789E+00_JPRB, 0.12140E+01_JPRB, 0.34107E+01_JPRB, 0.39351E+01_JPRB, 0.49362E+01_JPRB, 0.19601E+01_JPRB ,&
&  0.90361E+00_JPRB, 0.59495E+00_JPRB, 0.52537E+00_JPRB, 0.52317E+00_JPRB, 0.51627E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.55027E+00_JPRB, 0.63008E+00_JPRB, 0.69528E+00_JPRB, 0.62071E+00_JPRB, 0.62460E+00_JPRB, 0.63882E+00_JPRB, 0.66319E+00_JPRB ,&
&  0.69166E+00_JPRB, 0.72680E+00_JPRB, 0.76427E+00_JPRB, 0.80904E+00_JPRB, 0.84409E+00_JPRB, 0.87648E+00_JPRB, 0.91193E+00_JPRB ,&
&  0.94958E+00_JPRB, 0.95922E+00_JPRB, 0.98826E+00_JPRB, 0.98877E+00_JPRB, 0.10098E+01_JPRB, 0.10047E+01_JPRB, 0.10211E+01_JPRB ,&
&  0.10469E+01_JPRB, 0.10604E+01_JPRB, 0.10734E+01_JPRB, 0.10792E+01_JPRB, 0.10592E+01_JPRB, 0.10660E+01_JPRB, 0.78953E+00_JPRB ,&
&  0.97983E+00_JPRB, 0.89712E+00_JPRB, 0.81164E+00_JPRB, 0.78997E+00_JPRB, 0.75765E+00_JPRB, 0.75194E+00_JPRB, 0.74296E+00_JPRB ,&
&  0.73605E+00_JPRB, 0.73238E+00_JPRB, 0.71331E+00_JPRB, 0.66648E+00_JPRB, 0.63568E+00_JPRB, 0.64679E+00_JPRB, 0.65698E+00_JPRB ,&
&  0.67534E+00_JPRB, 0.70167E+00_JPRB, 0.72355E+00_JPRB, 0.74074E+00_JPRB, 0.75377E+00_JPRB, 0.75358E+00_JPRB, 0.74997E+00_JPRB ,&
&  0.73927E+00_JPRB, 0.73759E+00_JPRB, 0.69837E+00_JPRB, 0.68917E+00_JPRB, 0.72158E+00_JPRB, 0.64683E+00_JPRB, 0.52984E+00_JPRB ,&
&  0.43546E+00_JPRB, 0.56790E+00_JPRB, 0.75066E+00_JPRB, 0.89719E+00_JPRB, 0.81041E+00_JPRB, 0.75613E+00_JPRB, 0.69850E+00_JPRB ,&
&  0.62462E+00_JPRB, 0.56695E+00_JPRB, 0.54484E+00_JPRB, 0.50201E+00_JPRB, 0.48338E+00_JPRB, 0.48018E+00_JPRB, 0.47758E+00_JPRB ,&
&  0.47060E+00_JPRB, 0.46195E+00_JPRB, 0.44578E+00_JPRB, 0.43270E+00_JPRB, 0.38638E+00_JPRB, 0.33464E+00_JPRB, 0.46751E+00_JPRB ,&
&  0.41367E+00_JPRB, 0.34416E+00_JPRB, 0.50938E+00_JPRB, 0.39979E+00_JPRB, 0.53398E+00_JPRB, 0.59328E+00_JPRB, 0.64120E+00_JPRB ,&
&  0.60110E+00_JPRB, 0.76065E+00_JPRB, 0.11094E+01_JPRB, 0.11359E+01_JPRB, 0.11940E+01_JPRB, 0.12670E+01_JPRB, 0.74322E+00_JPRB ,&
&  0.59127E+00_JPRB, 0.57705E+00_JPRB, 0.57633E+00_JPRB, 0.53085E+00_JPRB, 0.52751E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.55268E+00_JPRB, 0.60994E+00_JPRB, 0.69704E+00_JPRB, 0.62408E+00_JPRB, 0.61986E+00_JPRB, 0.63172E+00_JPRB, 0.65085E+00_JPRB ,&
&  0.67068E+00_JPRB, 0.69752E+00_JPRB, 0.72638E+00_JPRB, 0.76709E+00_JPRB, 0.80247E+00_JPRB, 0.83413E+00_JPRB, 0.86406E+00_JPRB ,&
&  0.89141E+00_JPRB, 0.91450E+00_JPRB, 0.93682E+00_JPRB, 0.94605E+00_JPRB, 0.95781E+00_JPRB, 0.97608E+00_JPRB, 0.99995E+00_JPRB ,&
&  0.10404E+01_JPRB, 0.10572E+01_JPRB, 0.10801E+01_JPRB, 0.11219E+01_JPRB, 0.11500E+01_JPRB, 0.11675E+01_JPRB, 0.95414E+00_JPRB ,&
&  0.10295E+01_JPRB, 0.92898E+00_JPRB, 0.85508E+00_JPRB, 0.80193E+00_JPRB, 0.76396E+00_JPRB, 0.74560E+00_JPRB, 0.74579E+00_JPRB ,&
&  0.76742E+00_JPRB, 0.72527E+00_JPRB, 0.72169E+00_JPRB, 0.69417E+00_JPRB, 0.67887E+00_JPRB, 0.70000E+00_JPRB, 0.71450E+00_JPRB ,&
&  0.75315E+00_JPRB, 0.80552E+00_JPRB, 0.84854E+00_JPRB, 0.88076E+00_JPRB, 0.91645E+00_JPRB, 0.91860E+00_JPRB, 0.94391E+00_JPRB ,&
&  0.98132E+00_JPRB, 0.99609E+00_JPRB, 0.95369E+00_JPRB, 0.95021E+00_JPRB, 0.93565E+00_JPRB, 0.87434E+00_JPRB, 0.72161E+00_JPRB ,&
&  0.62299E+00_JPRB, 0.70383E+00_JPRB, 0.74760E+00_JPRB, 0.87854E+00_JPRB, 0.83915E+00_JPRB, 0.77591E+00_JPRB, 0.73837E+00_JPRB ,&
&  0.68143E+00_JPRB, 0.63749E+00_JPRB, 0.60836E+00_JPRB, 0.56125E+00_JPRB, 0.56688E+00_JPRB, 0.56938E+00_JPRB, 0.56404E+00_JPRB ,&
&  0.52734E+00_JPRB, 0.48721E+00_JPRB, 0.44867E+00_JPRB, 0.41715E+00_JPRB, 0.41587E+00_JPRB, 0.40712E+00_JPRB, 0.47016E+00_JPRB ,&
&  0.46844E+00_JPRB, 0.34342E+00_JPRB, 0.46073E+00_JPRB, 0.61891E+00_JPRB, 0.66152E+00_JPRB, 0.71111E+00_JPRB, 0.77021E+00_JPRB ,&
&  0.83897E+00_JPRB, 0.83117E+00_JPRB, 0.79331E+00_JPRB, 0.78277E+00_JPRB, 0.76933E+00_JPRB, 0.76250E+00_JPRB, 0.67977E+00_JPRB ,&
&  0.64863E+00_JPRB, 0.59275E+00_JPRB, 0.56174E+00_JPRB, 0.54903E+00_JPRB, 0.52455E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.59482E+00_JPRB, 0.61447E+00_JPRB, 0.67901E+00_JPRB, 0.63642E+00_JPRB, 0.63404E+00_JPRB, 0.64707E+00_JPRB, 0.66268E+00_JPRB ,&
&  0.66150E+00_JPRB, 0.66936E+00_JPRB, 0.68667E+00_JPRB, 0.71800E+00_JPRB, 0.74877E+00_JPRB, 0.77290E+00_JPRB, 0.79933E+00_JPRB ,&
&  0.81982E+00_JPRB, 0.83266E+00_JPRB, 0.84025E+00_JPRB, 0.84875E+00_JPRB, 0.87415E+00_JPRB, 0.90318E+00_JPRB, 0.94221E+00_JPRB ,&
&  0.98833E+00_JPRB, 0.99517E+00_JPRB, 0.98550E+00_JPRB, 0.10001E+01_JPRB, 0.10143E+01_JPRB, 0.10913E+01_JPRB, 0.11948E+01_JPRB ,&
&  0.91294E+00_JPRB, 0.95046E+00_JPRB, 0.88491E+00_JPRB, 0.80264E+00_JPRB, 0.79760E+00_JPRB, 0.76875E+00_JPRB, 0.77889E+00_JPRB ,&
&  0.79199E+00_JPRB, 0.77784E+00_JPRB, 0.78883E+00_JPRB, 0.79562E+00_JPRB, 0.79598E+00_JPRB, 0.81249E+00_JPRB, 0.83535E+00_JPRB ,&
&  0.86982E+00_JPRB, 0.92273E+00_JPRB, 0.96832E+00_JPRB, 0.10366E+01_JPRB, 0.11031E+01_JPRB, 0.11644E+01_JPRB, 0.12128E+01_JPRB ,&
&  0.12424E+01_JPRB, 0.12522E+01_JPRB, 0.11567E+01_JPRB, 0.11478E+01_JPRB, 0.11505E+01_JPRB, 0.11067E+01_JPRB, 0.99087E+00_JPRB ,&
&  0.88128E+00_JPRB, 0.83863E+00_JPRB, 0.83686E+00_JPRB, 0.80468E+00_JPRB, 0.81432E+00_JPRB, 0.82301E+00_JPRB, 0.78465E+00_JPRB ,&
&  0.76581E+00_JPRB, 0.71696E+00_JPRB, 0.68098E+00_JPRB, 0.65422E+00_JPRB, 0.65338E+00_JPRB, 0.67107E+00_JPRB, 0.66196E+00_JPRB ,&
&  0.62741E+00_JPRB, 0.58295E+00_JPRB, 0.54022E+00_JPRB, 0.49465E+00_JPRB, 0.45110E+00_JPRB, 0.53181E+00_JPRB, 0.54972E+00_JPRB ,&
&  0.58212E+00_JPRB, 0.52749E+00_JPRB, 0.53656E+00_JPRB, 0.73731E+00_JPRB, 0.80409E+00_JPRB, 0.79963E+00_JPRB, 0.82610E+00_JPRB ,&
&  0.83158E+00_JPRB, 0.82118E+00_JPRB, 0.83853E+00_JPRB, 0.79051E+00_JPRB, 0.75366E+00_JPRB, 0.73256E+00_JPRB, 0.71946E+00_JPRB ,&
&  0.66554E+00_JPRB, 0.61479E+00_JPRB, 0.56828E+00_JPRB, 0.56949E+00_JPRB, 0.54370E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.58451E+00_JPRB, 0.58999E+00_JPRB, 0.61422E+00_JPRB, 0.65169E+00_JPRB, 0.63076E+00_JPRB, 0.64787E+00_JPRB, 0.64325E+00_JPRB ,&
&  0.64166E+00_JPRB, 0.63476E+00_JPRB, 0.64501E+00_JPRB, 0.65600E+00_JPRB, 0.67550E+00_JPRB, 0.69484E+00_JPRB, 0.70074E+00_JPRB ,&
&  0.71247E+00_JPRB, 0.72532E+00_JPRB, 0.74560E+00_JPRB, 0.77863E+00_JPRB, 0.83594E+00_JPRB, 0.92702E+00_JPRB, 0.11025E+01_JPRB ,&
&  0.12707E+01_JPRB, 0.13328E+01_JPRB, 0.13206E+01_JPRB, 0.12714E+01_JPRB, 0.13741E+01_JPRB, 0.16954E+01_JPRB, 0.19214E+01_JPRB ,&
&  0.11625E+01_JPRB, 0.95725E+00_JPRB, 0.88159E+00_JPRB, 0.79651E+00_JPRB, 0.79836E+00_JPRB, 0.83896E+00_JPRB, 0.84194E+00_JPRB ,&
&  0.83227E+00_JPRB, 0.84687E+00_JPRB, 0.87762E+00_JPRB, 0.91681E+00_JPRB, 0.94427E+00_JPRB, 0.96159E+00_JPRB, 0.98752E+00_JPRB ,&
&  0.10249E+01_JPRB, 0.10788E+01_JPRB, 0.11412E+01_JPRB, 0.12146E+01_JPRB, 0.11892E+01_JPRB, 0.12855E+01_JPRB, 0.13047E+01_JPRB ,&
&  0.13141E+01_JPRB, 0.12299E+01_JPRB, 0.10754E+01_JPRB, 0.10857E+01_JPRB, 0.11660E+01_JPRB, 0.11660E+01_JPRB, 0.10657E+01_JPRB ,&
&  0.10911E+01_JPRB, 0.95344E+00_JPRB, 0.74322E+00_JPRB, 0.74073E+00_JPRB, 0.77209E+00_JPRB, 0.83608E+00_JPRB, 0.83035E+00_JPRB ,&
&  0.83929E+00_JPRB, 0.79501E+00_JPRB, 0.76201E+00_JPRB, 0.74778E+00_JPRB, 0.74783E+00_JPRB, 0.76559E+00_JPRB, 0.76730E+00_JPRB ,&
&  0.70643E+00_JPRB, 0.67454E+00_JPRB, 0.62812E+00_JPRB, 0.60434E+00_JPRB, 0.60285E+00_JPRB, 0.55278E+00_JPRB, 0.64331E+00_JPRB ,&
&  0.67396E+00_JPRB, 0.69268E+00_JPRB, 0.63271E+00_JPRB, 0.80592E+00_JPRB, 0.83413E+00_JPRB, 0.81187E+00_JPRB, 0.81477E+00_JPRB ,&
&  0.83429E+00_JPRB, 0.78997E+00_JPRB, 0.76294E+00_JPRB, 0.74134E+00_JPRB, 0.72820E+00_JPRB, 0.70430E+00_JPRB, 0.65615E+00_JPRB ,&
&  0.62174E+00_JPRB, 0.59829E+00_JPRB, 0.58466E+00_JPRB, 0.57445E+00_JPRB, 0.57731E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.56614E+00_JPRB, 0.58727E+00_JPRB, 0.60212E+00_JPRB, 0.59793E+00_JPRB, 0.60326E+00_JPRB, 0.60933E+00_JPRB, 0.59759E+00_JPRB ,&
&  0.58862E+00_JPRB, 0.58298E+00_JPRB, 0.59609E+00_JPRB, 0.59692E+00_JPRB, 0.60050E+00_JPRB, 0.61538E+00_JPRB, 0.63398E+00_JPRB ,&
&  0.65509E+00_JPRB, 0.67212E+00_JPRB, 0.73750E+00_JPRB, 0.86770E+00_JPRB, 0.11036E+01_JPRB, 0.13982E+01_JPRB, 0.15919E+01_JPRB ,&
&  0.16801E+01_JPRB, 0.17475E+01_JPRB, 0.17816E+01_JPRB, 0.17828E+01_JPRB, 0.15885E+01_JPRB, 0.15284E+01_JPRB, 0.15609E+01_JPRB ,&
&  0.11323E+01_JPRB, 0.86474E+00_JPRB, 0.92874E+00_JPRB, 0.87606E+00_JPRB, 0.86977E+00_JPRB, 0.87403E+00_JPRB, 0.86802E+00_JPRB ,&
&  0.86873E+00_JPRB, 0.88540E+00_JPRB, 0.91621E+00_JPRB, 0.96756E+00_JPRB, 0.10144E+01_JPRB, 0.10434E+01_JPRB, 0.10614E+01_JPRB ,&
&  0.10813E+01_JPRB, 0.11048E+01_JPRB, 0.11507E+01_JPRB, 0.11329E+01_JPRB, 0.11086E+01_JPRB, 0.11286E+01_JPRB, 0.11261E+01_JPRB ,&
&  0.11168E+01_JPRB, 0.10696E+01_JPRB, 0.94736E+00_JPRB, 0.94456E+00_JPRB, 0.10180E+01_JPRB, 0.96976E+00_JPRB, 0.94267E+00_JPRB ,&
&  0.10044E+01_JPRB, 0.96483E+00_JPRB, 0.68003E+00_JPRB, 0.66024E+00_JPRB, 0.71373E+00_JPRB, 0.79809E+00_JPRB, 0.88077E+00_JPRB ,&
&  0.89079E+00_JPRB, 0.86848E+00_JPRB, 0.83412E+00_JPRB, 0.82788E+00_JPRB, 0.81622E+00_JPRB, 0.82136E+00_JPRB, 0.82050E+00_JPRB ,&
&  0.76246E+00_JPRB, 0.73866E+00_JPRB, 0.70938E+00_JPRB, 0.68164E+00_JPRB, 0.67769E+00_JPRB, 0.67399E+00_JPRB, 0.69259E+00_JPRB ,&
&  0.73301E+00_JPRB, 0.73956E+00_JPRB, 0.75306E+00_JPRB, 0.76460E+00_JPRB, 0.75960E+00_JPRB, 0.80840E+00_JPRB, 0.78735E+00_JPRB ,&
&  0.79698E+00_JPRB, 0.76980E+00_JPRB, 0.74193E+00_JPRB, 0.71541E+00_JPRB, 0.69388E+00_JPRB, 0.66900E+00_JPRB, 0.63846E+00_JPRB ,&
&  0.60973E+00_JPRB, 0.58849E+00_JPRB, 0.57464E+00_JPRB, 0.56270E+00_JPRB, 0.55359E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.55252E+00_JPRB, 0.56895E+00_JPRB, 0.58967E+00_JPRB, 0.58495E+00_JPRB, 0.56585E+00_JPRB, 0.55606E+00_JPRB, 0.54984E+00_JPRB ,&
&  0.55334E+00_JPRB, 0.56961E+00_JPRB, 0.58001E+00_JPRB, 0.58129E+00_JPRB, 0.58301E+00_JPRB, 0.59993E+00_JPRB, 0.64263E+00_JPRB ,&
&  0.69871E+00_JPRB, 0.77024E+00_JPRB, 0.89865E+00_JPRB, 0.11530E+01_JPRB, 0.14570E+01_JPRB, 0.16874E+01_JPRB, 0.16142E+01_JPRB ,&
&  0.14799E+01_JPRB, 0.14589E+01_JPRB, 0.14204E+01_JPRB, 0.13735E+01_JPRB, 0.13434E+01_JPRB, 0.13038E+01_JPRB, 0.12163E+01_JPRB ,&
&  0.10325E+01_JPRB, 0.87811E+00_JPRB, 0.87382E+00_JPRB, 0.87802E+00_JPRB, 0.87779E+00_JPRB, 0.86251E+00_JPRB, 0.86617E+00_JPRB ,&
&  0.86885E+00_JPRB, 0.88212E+00_JPRB, 0.90791E+00_JPRB, 0.95573E+00_JPRB, 0.10109E+01_JPRB, 0.10424E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.10534E+01_JPRB, 0.10484E+01_JPRB, 0.10651E+01_JPRB, 0.98185E+00_JPRB, 0.99751E+00_JPRB, 0.99831E+00_JPRB, 0.98863E+00_JPRB ,&
&  0.96253E+00_JPRB, 0.91381E+00_JPRB, 0.94585E+00_JPRB, 0.94071E+00_JPRB, 0.93027E+00_JPRB, 0.90918E+00_JPRB, 0.90349E+00_JPRB ,&
&  0.92218E+00_JPRB, 0.89559E+00_JPRB, 0.69581E+00_JPRB, 0.69148E+00_JPRB, 0.72639E+00_JPRB, 0.72038E+00_JPRB, 0.85816E+00_JPRB ,&
&  0.86980E+00_JPRB, 0.87931E+00_JPRB, 0.88024E+00_JPRB, 0.88987E+00_JPRB, 0.89911E+00_JPRB, 0.91126E+00_JPRB, 0.84842E+00_JPRB ,&
&  0.82521E+00_JPRB, 0.78734E+00_JPRB, 0.77707E+00_JPRB, 0.76164E+00_JPRB, 0.76657E+00_JPRB, 0.76847E+00_JPRB, 0.77025E+00_JPRB ,&
&  0.78931E+00_JPRB, 0.81224E+00_JPRB, 0.83144E+00_JPRB, 0.85594E+00_JPRB, 0.81361E+00_JPRB, 0.77441E+00_JPRB, 0.75724E+00_JPRB ,&
&  0.74245E+00_JPRB, 0.70531E+00_JPRB, 0.69251E+00_JPRB, 0.67604E+00_JPRB, 0.66246E+00_JPRB, 0.64949E+00_JPRB, 0.63659E+00_JPRB ,&
&  0.61754E+00_JPRB, 0.60060E+00_JPRB, 0.58323E+00_JPRB, 0.56439E+00_JPRB, 0.55174E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.55622E+00_JPRB, 0.56467E+00_JPRB, 0.58444E+00_JPRB, 0.59523E+00_JPRB, 0.57575E+00_JPRB, 0.56345E+00_JPRB, 0.56058E+00_JPRB ,&
&  0.57041E+00_JPRB, 0.58677E+00_JPRB, 0.60091E+00_JPRB, 0.61434E+00_JPRB, 0.64414E+00_JPRB, 0.66306E+00_JPRB, 0.69491E+00_JPRB ,&
&  0.75179E+00_JPRB, 0.84093E+00_JPRB, 0.10048E+01_JPRB, 0.12598E+01_JPRB, 0.14905E+01_JPRB, 0.17068E+01_JPRB, 0.15836E+01_JPRB ,&
&  0.14640E+01_JPRB, 0.14060E+01_JPRB, 0.13667E+01_JPRB, 0.12855E+01_JPRB, 0.11053E+01_JPRB, 0.10964E+01_JPRB, 0.10405E+01_JPRB ,&
&  0.10111E+01_JPRB, 0.94860E+00_JPRB, 0.92110E+00_JPRB, 0.88149E+00_JPRB, 0.84247E+00_JPRB, 0.83300E+00_JPRB, 0.84247E+00_JPRB ,&
&  0.85496E+00_JPRB, 0.86687E+00_JPRB, 0.89037E+00_JPRB, 0.92397E+00_JPRB, 0.96657E+00_JPRB, 0.10078E+01_JPRB, 0.10238E+01_JPRB ,&
&  0.10103E+01_JPRB, 0.99454E+00_JPRB, 0.97911E+00_JPRB, 0.93928E+00_JPRB, 0.92348E+00_JPRB, 0.92146E+00_JPRB, 0.92065E+00_JPRB ,&
&  0.92835E+00_JPRB, 0.89535E+00_JPRB, 0.91303E+00_JPRB, 0.93047E+00_JPRB, 0.89307E+00_JPRB, 0.83893E+00_JPRB, 0.83095E+00_JPRB ,&
&  0.84530E+00_JPRB, 0.89960E+00_JPRB, 0.73406E+00_JPRB, 0.80368E+00_JPRB, 0.86824E+00_JPRB, 0.80770E+00_JPRB, 0.81045E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.87942E+00_JPRB, 0.91030E+00_JPRB, 0.93645E+00_JPRB, 0.95075E+00_JPRB, 0.96783E+00_JPRB, 0.84691E+00_JPRB ,&
&  0.85688E+00_JPRB, 0.81533E+00_JPRB, 0.80836E+00_JPRB, 0.80637E+00_JPRB, 0.81457E+00_JPRB, 0.81097E+00_JPRB, 0.78175E+00_JPRB ,&
&  0.76382E+00_JPRB, 0.88271E+00_JPRB, 0.10421E+01_JPRB, 0.12313E+01_JPRB, 0.12957E+01_JPRB, 0.75701E+00_JPRB, 0.76543E+00_JPRB ,&
&  0.74067E+00_JPRB, 0.70734E+00_JPRB, 0.69076E+00_JPRB, 0.67538E+00_JPRB, 0.66425E+00_JPRB, 0.65198E+00_JPRB, 0.64423E+00_JPRB ,&
&  0.63110E+00_JPRB, 0.61353E+00_JPRB, 0.59558E+00_JPRB, 0.57887E+00_JPRB, 0.56487E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.57234E+00_JPRB, 0.57130E+00_JPRB, 0.58563E+00_JPRB, 0.59856E+00_JPRB, 0.60102E+00_JPRB, 0.58837E+00_JPRB, 0.58513E+00_JPRB ,&
&  0.58186E+00_JPRB, 0.59366E+00_JPRB, 0.60487E+00_JPRB, 0.63427E+00_JPRB, 0.66956E+00_JPRB, 0.72833E+00_JPRB, 0.78713E+00_JPRB ,&
&  0.85336E+00_JPRB, 0.93963E+00_JPRB, 0.11014E+01_JPRB, 0.13065E+01_JPRB, 0.14698E+01_JPRB, 0.15642E+01_JPRB, 0.14943E+01_JPRB ,&
&  0.14038E+01_JPRB, 0.14212E+01_JPRB, 0.13934E+01_JPRB, 0.10791E+01_JPRB, 0.96659E+00_JPRB, 0.11210E+01_JPRB, 0.95686E+00_JPRB ,&
&  0.89546E+00_JPRB, 0.88014E+00_JPRB, 0.85433E+00_JPRB, 0.82222E+00_JPRB, 0.78651E+00_JPRB, 0.78461E+00_JPRB, 0.79585E+00_JPRB ,&
&  0.81851E+00_JPRB, 0.83861E+00_JPRB, 0.86150E+00_JPRB, 0.88592E+00_JPRB, 0.91432E+00_JPRB, 0.96501E+00_JPRB, 0.99606E+00_JPRB ,&
&  0.99421E+00_JPRB, 0.97922E+00_JPRB, 0.95970E+00_JPRB, 0.93901E+00_JPRB, 0.90043E+00_JPRB, 0.88825E+00_JPRB, 0.88963E+00_JPRB ,&
&  0.89023E+00_JPRB, 0.88429E+00_JPRB, 0.89075E+00_JPRB, 0.90644E+00_JPRB, 0.90648E+00_JPRB, 0.82715E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.82104E+00_JPRB, 0.85625E+00_JPRB, 0.80172E+00_JPRB, 0.87610E+00_JPRB, 0.65089E+00_JPRB, 0.69888E+00_JPRB, 0.78247E+00_JPRB ,&
&  0.81669E+00_JPRB, 0.84981E+00_JPRB, 0.88009E+00_JPRB, 0.91588E+00_JPRB, 0.92975E+00_JPRB, 0.86419E+00_JPRB, 0.89058E+00_JPRB ,&
&  0.89198E+00_JPRB, 0.83968E+00_JPRB, 0.82955E+00_JPRB, 0.81903E+00_JPRB, 0.82158E+00_JPRB, 0.81420E+00_JPRB, 0.77207E+00_JPRB ,&
&  0.73138E+00_JPRB, 0.84183E+00_JPRB, 0.96579E+00_JPRB, 0.10724E+01_JPRB, 0.10252E+01_JPRB, 0.78201E+00_JPRB, 0.77149E+00_JPRB ,&
&  0.76094E+00_JPRB, 0.72145E+00_JPRB, 0.69717E+00_JPRB, 0.67912E+00_JPRB, 0.66633E+00_JPRB, 0.65039E+00_JPRB, 0.63838E+00_JPRB ,&
&  0.62757E+00_JPRB, 0.61191E+00_JPRB, 0.60019E+00_JPRB, 0.58935E+00_JPRB, 0.57903E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.57765E+00_JPRB, 0.58198E+00_JPRB, 0.59919E+00_JPRB, 0.60380E+00_JPRB, 0.61444E+00_JPRB, 0.62255E+00_JPRB, 0.60677E+00_JPRB ,&
&  0.57590E+00_JPRB, 0.54952E+00_JPRB, 0.56056E+00_JPRB, 0.59426E+00_JPRB, 0.63713E+00_JPRB, 0.70284E+00_JPRB, 0.78623E+00_JPRB ,&
&  0.89079E+00_JPRB, 0.97989E+00_JPRB, 0.11194E+01_JPRB, 0.12426E+01_JPRB, 0.13307E+01_JPRB, 0.13850E+01_JPRB, 0.13655E+01_JPRB ,&
&  0.11005E+01_JPRB, 0.93091E+00_JPRB, 0.13475E+01_JPRB, 0.14326E+01_JPRB, 0.12691E+01_JPRB, 0.11653E+01_JPRB, 0.97844E+00_JPRB ,&
&  0.87815E+00_JPRB, 0.87160E+00_JPRB, 0.86096E+00_JPRB, 0.81452E+00_JPRB, 0.74457E+00_JPRB, 0.73785E+00_JPRB, 0.75084E+00_JPRB ,&
&  0.77647E+00_JPRB, 0.79940E+00_JPRB, 0.82792E+00_JPRB, 0.86193E+00_JPRB, 0.88486E+00_JPRB, 0.92541E+00_JPRB, 0.95803E+00_JPRB ,&
&  0.97546E+00_JPRB, 0.97470E+00_JPRB, 0.96893E+00_JPRB, 0.94261E+00_JPRB, 0.91763E+00_JPRB, 0.89521E+00_JPRB, 0.87419E+00_JPRB ,&
&  0.87416E+00_JPRB, 0.86726E+00_JPRB, 0.88495E+00_JPRB, 0.89411E+00_JPRB, 0.91544E+00_JPRB, 0.85861E+00_JPRB, 0.86484E+00_JPRB ,&
&  0.84355E+00_JPRB, 0.83806E+00_JPRB, 0.82724E+00_JPRB, 0.84769E+00_JPRB, 0.66655E+00_JPRB, 0.70219E+00_JPRB, 0.70536E+00_JPRB ,&
&  0.78488E+00_JPRB, 0.82067E+00_JPRB, 0.84984E+00_JPRB, 0.87577E+00_JPRB, 0.89568E+00_JPRB, 0.84404E+00_JPRB, 0.87516E+00_JPRB ,&
&  0.89228E+00_JPRB, 0.88675E+00_JPRB, 0.84744E+00_JPRB, 0.83438E+00_JPRB, 0.78495E+00_JPRB, 0.81316E+00_JPRB, 0.79925E+00_JPRB ,&
&  0.81989E+00_JPRB, 0.84046E+00_JPRB, 0.90160E+00_JPRB, 0.97479E+00_JPRB, 0.94351E+00_JPRB, 0.83980E+00_JPRB, 0.80871E+00_JPRB ,&
&  0.78808E+00_JPRB, 0.76023E+00_JPRB, 0.72144E+00_JPRB, 0.69160E+00_JPRB, 0.67051E+00_JPRB, 0.65006E+00_JPRB, 0.63035E+00_JPRB ,&
&  0.61092E+00_JPRB, 0.59513E+00_JPRB, 0.58454E+00_JPRB, 0.57607E+00_JPRB, 0.57413E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.55902E+00_JPRB, 0.57931E+00_JPRB, 0.60866E+00_JPRB, 0.64274E+00_JPRB, 0.75728E+00_JPRB, 0.88363E+00_JPRB, 0.95470E+00_JPRB ,&
&  0.90170E+00_JPRB, 0.53629E+00_JPRB, 0.52450E+00_JPRB, 0.54447E+00_JPRB, 0.57619E+00_JPRB, 0.63209E+00_JPRB, 0.69358E+00_JPRB ,&
&  0.78249E+00_JPRB, 0.87987E+00_JPRB, 0.10088E+01_JPRB, 0.11678E+01_JPRB, 0.12548E+01_JPRB, 0.13207E+01_JPRB, 0.12015E+01_JPRB ,&
&  0.83195E+00_JPRB, 0.95342E+00_JPRB, 0.15471E+01_JPRB, 0.14720E+01_JPRB, 0.13526E+01_JPRB, 0.12332E+01_JPRB, 0.10740E+01_JPRB ,&
&  0.94904E+00_JPRB, 0.86108E+00_JPRB, 0.83013E+00_JPRB, 0.79255E+00_JPRB, 0.71518E+00_JPRB, 0.68702E+00_JPRB, 0.68749E+00_JPRB ,&
&  0.70716E+00_JPRB, 0.73759E+00_JPRB, 0.77834E+00_JPRB, 0.82064E+00_JPRB, 0.86403E+00_JPRB, 0.89101E+00_JPRB, 0.92262E+00_JPRB ,&
&  0.95153E+00_JPRB, 0.96452E+00_JPRB, 0.94563E+00_JPRB, 0.91896E+00_JPRB, 0.92394E+00_JPRB, 0.91982E+00_JPRB, 0.90400E+00_JPRB ,&
&  0.89829E+00_JPRB, 0.93062E+00_JPRB, 0.94456E+00_JPRB, 0.88853E+00_JPRB, 0.82131E+00_JPRB, 0.87457E+00_JPRB, 0.84449E+00_JPRB ,&
&  0.84476E+00_JPRB, 0.80893E+00_JPRB, 0.86244E+00_JPRB, 0.73970E+00_JPRB, 0.69716E+00_JPRB, 0.73381E+00_JPRB, 0.73530E+00_JPRB ,&
&  0.73809E+00_JPRB, 0.77833E+00_JPRB, 0.81122E+00_JPRB, 0.84057E+00_JPRB, 0.85211E+00_JPRB, 0.80585E+00_JPRB, 0.85411E+00_JPRB ,&
&  0.83658E+00_JPRB, 0.91633E+00_JPRB, 0.88254E+00_JPRB, 0.85218E+00_JPRB, 0.76811E+00_JPRB, 0.76764E+00_JPRB, 0.77456E+00_JPRB ,&
&  0.81262E+00_JPRB, 0.83226E+00_JPRB, 0.84835E+00_JPRB, 0.86614E+00_JPRB, 0.85808E+00_JPRB, 0.85334E+00_JPRB, 0.85256E+00_JPRB ,&
&  0.82463E+00_JPRB, 0.78266E+00_JPRB, 0.75030E+00_JPRB, 0.71218E+00_JPRB, 0.68286E+00_JPRB, 0.65230E+00_JPRB, 0.61961E+00_JPRB ,&
&  0.59468E+00_JPRB, 0.57770E+00_JPRB, 0.56452E+00_JPRB, 0.55195E+00_JPRB, 0.54854E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.52816E+00_JPRB, 0.54505E+00_JPRB, 0.57031E+00_JPRB, 0.60964E+00_JPRB, 0.72301E+00_JPRB, 0.76864E+00_JPRB, 0.77126E+00_JPRB ,&
&  0.74892E+00_JPRB, 0.50940E+00_JPRB, 0.48456E+00_JPRB, 0.50287E+00_JPRB, 0.53859E+00_JPRB, 0.57982E+00_JPRB, 0.64314E+00_JPRB ,&
&  0.69822E+00_JPRB, 0.78130E+00_JPRB, 0.91131E+00_JPRB, 0.10624E+01_JPRB, 0.11678E+01_JPRB, 0.11718E+01_JPRB, 0.10179E+01_JPRB ,&
&  0.75594E+00_JPRB, 0.15107E+01_JPRB, 0.15925E+01_JPRB, 0.15996E+01_JPRB, 0.15345E+01_JPRB, 0.13954E+01_JPRB, 0.11515E+01_JPRB ,&
&  0.10141E+01_JPRB, 0.92109E+00_JPRB, 0.84016E+00_JPRB, 0.78074E+00_JPRB, 0.72012E+00_JPRB, 0.66076E+00_JPRB, 0.64025E+00_JPRB ,&
&  0.64742E+00_JPRB, 0.68107E+00_JPRB, 0.72518E+00_JPRB, 0.77073E+00_JPRB, 0.82515E+00_JPRB, 0.87238E+00_JPRB, 0.90436E+00_JPRB ,&
&  0.94616E+00_JPRB, 0.95657E+00_JPRB, 0.91955E+00_JPRB, 0.91711E+00_JPRB, 0.92315E+00_JPRB, 0.94366E+00_JPRB, 0.95050E+00_JPRB ,&
&  0.91288E+00_JPRB, 0.89157E+00_JPRB, 0.98347E+00_JPRB, 0.97662E+00_JPRB, 0.90911E+00_JPRB, 0.91146E+00_JPRB, 0.82334E+00_JPRB ,&
&  0.84762E+00_JPRB, 0.78322E+00_JPRB, 0.80841E+00_JPRB, 0.70745E+00_JPRB, 0.69420E+00_JPRB, 0.74097E+00_JPRB, 0.73850E+00_JPRB ,&
&  0.70667E+00_JPRB, 0.74111E+00_JPRB, 0.75367E+00_JPRB, 0.78496E+00_JPRB, 0.80157E+00_JPRB, 0.80008E+00_JPRB, 0.79159E+00_JPRB ,&
&  0.81064E+00_JPRB, 0.85471E+00_JPRB, 0.90979E+00_JPRB, 0.82291E+00_JPRB, 0.78136E+00_JPRB, 0.74371E+00_JPRB, 0.76983E+00_JPRB ,&
&  0.78741E+00_JPRB, 0.80089E+00_JPRB, 0.85582E+00_JPRB, 0.86036E+00_JPRB, 0.81140E+00_JPRB, 0.81854E+00_JPRB, 0.83362E+00_JPRB ,&
&  0.81889E+00_JPRB, 0.78543E+00_JPRB, 0.76482E+00_JPRB, 0.71908E+00_JPRB, 0.66416E+00_JPRB, 0.63057E+00_JPRB, 0.60350E+00_JPRB ,&
&  0.58018E+00_JPRB, 0.56366E+00_JPRB, 0.54656E+00_JPRB, 0.53036E+00_JPRB, 0.52304E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.50911E+00_JPRB, 0.52795E+00_JPRB, 0.55141E+00_JPRB, 0.58512E+00_JPRB, 0.65644E+00_JPRB, 0.67974E+00_JPRB, 0.62852E+00_JPRB ,&
&  0.56267E+00_JPRB, 0.45801E+00_JPRB, 0.44643E+00_JPRB, 0.46414E+00_JPRB, 0.49529E+00_JPRB, 0.53171E+00_JPRB, 0.59460E+00_JPRB ,&
&  0.66882E+00_JPRB, 0.72908E+00_JPRB, 0.85807E+00_JPRB, 0.96844E+00_JPRB, 0.10820E+01_JPRB, 0.11585E+01_JPRB, 0.78576E+00_JPRB ,&
&  0.82288E+00_JPRB, 0.14793E+01_JPRB, 0.15992E+01_JPRB, 0.16410E+01_JPRB, 0.16657E+01_JPRB, 0.16681E+01_JPRB, 0.13534E+01_JPRB ,&
&  0.11694E+01_JPRB, 0.10535E+01_JPRB, 0.92306E+00_JPRB, 0.80613E+00_JPRB, 0.74585E+00_JPRB, 0.69473E+00_JPRB, 0.63409E+00_JPRB ,&
&  0.61527E+00_JPRB, 0.63523E+00_JPRB, 0.67823E+00_JPRB, 0.72678E+00_JPRB, 0.78321E+00_JPRB, 0.84774E+00_JPRB, 0.89030E+00_JPRB ,&
&  0.93505E+00_JPRB, 0.95139E+00_JPRB, 0.94015E+00_JPRB, 0.88642E+00_JPRB, 0.91591E+00_JPRB, 0.94373E+00_JPRB, 0.97374E+00_JPRB ,&
&  0.93485E+00_JPRB, 0.92690E+00_JPRB, 0.10877E+01_JPRB, 0.11925E+01_JPRB, 0.11425E+01_JPRB, 0.10305E+01_JPRB, 0.89668E+00_JPRB ,&
&  0.85808E+00_JPRB, 0.79541E+00_JPRB, 0.72562E+00_JPRB, 0.66550E+00_JPRB, 0.68305E+00_JPRB, 0.72753E+00_JPRB, 0.73421E+00_JPRB ,&
&  0.70848E+00_JPRB, 0.70987E+00_JPRB, 0.70302E+00_JPRB, 0.70374E+00_JPRB, 0.71734E+00_JPRB, 0.72787E+00_JPRB, 0.75352E+00_JPRB ,&
&  0.80861E+00_JPRB, 0.85886E+00_JPRB, 0.87694E+00_JPRB, 0.79361E+00_JPRB, 0.76600E+00_JPRB, 0.67801E+00_JPRB, 0.70157E+00_JPRB ,&
&  0.73728E+00_JPRB, 0.73446E+00_JPRB, 0.75890E+00_JPRB, 0.83999E+00_JPRB, 0.81821E+00_JPRB, 0.82973E+00_JPRB, 0.85473E+00_JPRB ,&
&  0.80478E+00_JPRB, 0.75483E+00_JPRB, 0.73399E+00_JPRB, 0.71313E+00_JPRB, 0.67090E+00_JPRB, 0.62716E+00_JPRB, 0.59913E+00_JPRB ,&
&  0.57091E+00_JPRB, 0.55328E+00_JPRB, 0.53210E+00_JPRB, 0.51430E+00_JPRB, 0.50509E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.49039E+00_JPRB, 0.50269E+00_JPRB, 0.52588E+00_JPRB, 0.54709E+00_JPRB, 0.57104E+00_JPRB, 0.58140E+00_JPRB, 0.53409E+00_JPRB ,&
&  0.46893E+00_JPRB, 0.42766E+00_JPRB, 0.41239E+00_JPRB, 0.41912E+00_JPRB, 0.45063E+00_JPRB, 0.49507E+00_JPRB, 0.53908E+00_JPRB ,&
&  0.62587E+00_JPRB, 0.70435E+00_JPRB, 0.80428E+00_JPRB, 0.89038E+00_JPRB, 0.87781E+00_JPRB, 0.74829E+00_JPRB, 0.74698E+00_JPRB ,&
&  0.11364E+01_JPRB, 0.16102E+01_JPRB, 0.16056E+01_JPRB, 0.16295E+01_JPRB, 0.17339E+01_JPRB, 0.18669E+01_JPRB, 0.15519E+01_JPRB ,&
&  0.13372E+01_JPRB, 0.11897E+01_JPRB, 0.10258E+01_JPRB, 0.90304E+00_JPRB, 0.79804E+00_JPRB, 0.74815E+00_JPRB, 0.68338E+00_JPRB ,&
&  0.62154E+00_JPRB, 0.62185E+00_JPRB, 0.66866E+00_JPRB, 0.71711E+00_JPRB, 0.77183E+00_JPRB, 0.82781E+00_JPRB, 0.89194E+00_JPRB ,&
&  0.92875E+00_JPRB, 0.93582E+00_JPRB, 0.92099E+00_JPRB, 0.88291E+00_JPRB, 0.89956E+00_JPRB, 0.93999E+00_JPRB, 0.10341E+01_JPRB ,&
&  0.10886E+01_JPRB, 0.11942E+01_JPRB, 0.12716E+01_JPRB, 0.13456E+01_JPRB, 0.12788E+01_JPRB, 0.12089E+01_JPRB, 0.10671E+01_JPRB ,&
&  0.95154E+00_JPRB, 0.83270E+00_JPRB, 0.68499E+00_JPRB, 0.65709E+00_JPRB, 0.70857E+00_JPRB, 0.73110E+00_JPRB, 0.67172E+00_JPRB ,&
&  0.62436E+00_JPRB, 0.59666E+00_JPRB, 0.59366E+00_JPRB, 0.61127E+00_JPRB, 0.64703E+00_JPRB, 0.67368E+00_JPRB, 0.72158E+00_JPRB ,&
&  0.73000E+00_JPRB, 0.56795E+00_JPRB, 0.52733E+00_JPRB, 0.60327E+00_JPRB, 0.55996E+00_JPRB, 0.54323E+00_JPRB, 0.61969E+00_JPRB ,&
&  0.67096E+00_JPRB, 0.70269E+00_JPRB, 0.72043E+00_JPRB, 0.71098E+00_JPRB, 0.79417E+00_JPRB, 0.86149E+00_JPRB, 0.95150E+00_JPRB ,&
&  0.82928E+00_JPRB, 0.75377E+00_JPRB, 0.71035E+00_JPRB, 0.68596E+00_JPRB, 0.65657E+00_JPRB, 0.62343E+00_JPRB, 0.59921E+00_JPRB ,&
&  0.57390E+00_JPRB, 0.54733E+00_JPRB, 0.52396E+00_JPRB, 0.50350E+00_JPRB, 0.48943E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.47346E+00_JPRB, 0.47743E+00_JPRB, 0.49978E+00_JPRB, 0.51767E+00_JPRB, 0.51649E+00_JPRB, 0.50128E+00_JPRB, 0.47337E+00_JPRB ,&
&  0.44143E+00_JPRB, 0.41582E+00_JPRB, 0.39784E+00_JPRB, 0.39807E+00_JPRB, 0.41386E+00_JPRB, 0.45896E+00_JPRB, 0.51375E+00_JPRB ,&
&  0.59303E+00_JPRB, 0.70791E+00_JPRB, 0.80625E+00_JPRB, 0.86312E+00_JPRB, 0.77018E+00_JPRB, 0.61636E+00_JPRB, 0.72920E+00_JPRB ,&
&  0.10428E+01_JPRB, 0.15104E+01_JPRB, 0.17641E+01_JPRB, 0.17963E+01_JPRB, 0.18010E+01_JPRB, 0.18202E+01_JPRB, 0.17127E+01_JPRB ,&
&  0.15083E+01_JPRB, 0.12900E+01_JPRB, 0.11107E+01_JPRB, 0.98234E+00_JPRB, 0.86147E+00_JPRB, 0.76934E+00_JPRB, 0.72502E+00_JPRB ,&
&  0.66365E+00_JPRB, 0.61747E+00_JPRB, 0.64478E+00_JPRB, 0.70588E+00_JPRB, 0.75486E+00_JPRB, 0.80547E+00_JPRB, 0.86330E+00_JPRB ,&
&  0.89424E+00_JPRB, 0.91323E+00_JPRB, 0.91141E+00_JPRB, 0.91003E+00_JPRB, 0.76657E+00_JPRB, 0.89989E+00_JPRB, 0.95821E+00_JPRB ,&
&  0.10779E+01_JPRB, 0.12225E+01_JPRB, 0.13067E+01_JPRB, 0.15640E+01_JPRB, 0.14294E+01_JPRB, 0.13020E+01_JPRB, 0.11752E+01_JPRB ,&
&  0.10761E+01_JPRB, 0.90898E+00_JPRB, 0.74905E+00_JPRB, 0.71543E+00_JPRB, 0.73822E+00_JPRB, 0.72142E+00_JPRB, 0.55832E+00_JPRB ,&
&  0.52515E+00_JPRB, 0.58026E+00_JPRB, 0.55376E+00_JPRB, 0.52546E+00_JPRB, 0.58148E+00_JPRB, 0.62761E+00_JPRB, 0.55143E+00_JPRB ,&
&  0.38587E+00_JPRB, 0.34853E+00_JPRB, 0.35199E+00_JPRB, 0.35909E+00_JPRB, 0.35547E+00_JPRB, 0.39000E+00_JPRB, 0.57036E+00_JPRB ,&
&  0.60041E+00_JPRB, 0.65428E+00_JPRB, 0.65876E+00_JPRB, 0.65834E+00_JPRB, 0.72103E+00_JPRB, 0.83637E+00_JPRB, 0.98106E+00_JPRB ,&
&  0.83027E+00_JPRB, 0.77102E+00_JPRB, 0.71422E+00_JPRB, 0.68021E+00_JPRB, 0.65142E+00_JPRB, 0.62320E+00_JPRB, 0.59326E+00_JPRB ,&
&  0.56994E+00_JPRB, 0.54268E+00_JPRB, 0.51698E+00_JPRB, 0.49396E+00_JPRB, 0.47717E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.45131E+00_JPRB, 0.44969E+00_JPRB, 0.46052E+00_JPRB, 0.46555E+00_JPRB, 0.47407E+00_JPRB, 0.45231E+00_JPRB, 0.45334E+00_JPRB ,&
&  0.42786E+00_JPRB, 0.41584E+00_JPRB, 0.39032E+00_JPRB, 0.38178E+00_JPRB, 0.38597E+00_JPRB, 0.42595E+00_JPRB, 0.49526E+00_JPRB ,&
&  0.57491E+00_JPRB, 0.69876E+00_JPRB, 0.90175E+00_JPRB, 0.85489E+00_JPRB, 0.82667E+00_JPRB, 0.73656E+00_JPRB, 0.69590E+00_JPRB ,&
&  0.91472E+00_JPRB, 0.13006E+01_JPRB, 0.15358E+01_JPRB, 0.16683E+01_JPRB, 0.19358E+01_JPRB, 0.19955E+01_JPRB, 0.18984E+01_JPRB ,&
&  0.17491E+01_JPRB, 0.14888E+01_JPRB, 0.12767E+01_JPRB, 0.11131E+01_JPRB, 0.96215E+00_JPRB, 0.83279E+00_JPRB, 0.74220E+00_JPRB ,&
&  0.70284E+00_JPRB, 0.64614E+00_JPRB, 0.63442E+00_JPRB, 0.68265E+00_JPRB, 0.74066E+00_JPRB, 0.77286E+00_JPRB, 0.81502E+00_JPRB ,&
&  0.85392E+00_JPRB, 0.86828E+00_JPRB, 0.87500E+00_JPRB, 0.86243E+00_JPRB, 0.73579E+00_JPRB, 0.65752E+00_JPRB, 0.81065E+00_JPRB ,&
&  0.97620E+00_JPRB, 0.12164E+01_JPRB, 0.14337E+01_JPRB, 0.17840E+01_JPRB, 0.14838E+01_JPRB, 0.13481E+01_JPRB, 0.12945E+01_JPRB ,&
&  0.11544E+01_JPRB, 0.10898E+01_JPRB, 0.80646E+00_JPRB, 0.75876E+00_JPRB, 0.76566E+00_JPRB, 0.62511E+00_JPRB, 0.54983E+00_JPRB ,&
&  0.58101E+00_JPRB, 0.55167E+00_JPRB, 0.50639E+00_JPRB, 0.45519E+00_JPRB, 0.52185E+00_JPRB, 0.52858E+00_JPRB, 0.36484E+00_JPRB ,&
&  0.32720E+00_JPRB, 0.31709E+00_JPRB, 0.30462E+00_JPRB, 0.29547E+00_JPRB, 0.28818E+00_JPRB, 0.30184E+00_JPRB, 0.40375E+00_JPRB ,&
&  0.48199E+00_JPRB, 0.58831E+00_JPRB, 0.61930E+00_JPRB, 0.62906E+00_JPRB, 0.68084E+00_JPRB, 0.83394E+00_JPRB, 0.11078E+01_JPRB ,&
&  0.85988E+00_JPRB, 0.79442E+00_JPRB, 0.71036E+00_JPRB, 0.66716E+00_JPRB, 0.63768E+00_JPRB, 0.61365E+00_JPRB, 0.58869E+00_JPRB ,&
&  0.56023E+00_JPRB, 0.53395E+00_JPRB, 0.50792E+00_JPRB, 0.48062E+00_JPRB, 0.46053E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.42895E+00_JPRB, 0.42242E+00_JPRB, 0.41238E+00_JPRB, 0.41983E+00_JPRB, 0.44176E+00_JPRB, 0.42649E+00_JPRB, 0.42919E+00_JPRB ,&
&  0.42099E+00_JPRB, 0.41158E+00_JPRB, 0.39492E+00_JPRB, 0.38066E+00_JPRB, 0.37523E+00_JPRB, 0.40478E+00_JPRB, 0.45971E+00_JPRB ,&
&  0.54313E+00_JPRB, 0.66242E+00_JPRB, 0.81468E+00_JPRB, 0.70447E+00_JPRB, 0.62415E+00_JPRB, 0.56217E+00_JPRB, 0.52604E+00_JPRB ,&
&  0.78146E+00_JPRB, 0.10487E+01_JPRB, 0.12826E+01_JPRB, 0.15599E+01_JPRB, 0.19174E+01_JPRB, 0.19791E+01_JPRB, 0.21301E+01_JPRB ,&
&  0.20742E+01_JPRB, 0.17523E+01_JPRB, 0.14771E+01_JPRB, 0.12457E+01_JPRB, 0.10731E+01_JPRB, 0.91063E+00_JPRB, 0.78798E+00_JPRB ,&
&  0.70902E+00_JPRB, 0.66257E+00_JPRB, 0.62474E+00_JPRB, 0.64865E+00_JPRB, 0.70108E+00_JPRB, 0.73737E+00_JPRB, 0.77025E+00_JPRB ,&
&  0.79742E+00_JPRB, 0.81169E+00_JPRB, 0.81825E+00_JPRB, 0.82493E+00_JPRB, 0.84505E+00_JPRB, 0.80412E+00_JPRB, 0.75870E+00_JPRB ,&
&  0.76980E+00_JPRB, 0.88696E+00_JPRB, 0.13707E+01_JPRB, 0.16254E+01_JPRB, 0.14341E+01_JPRB, 0.14008E+01_JPRB, 0.13637E+01_JPRB ,&
&  0.12487E+01_JPRB, 0.11570E+01_JPRB, 0.87618E+00_JPRB, 0.86067E+00_JPRB, 0.74300E+00_JPRB, 0.54955E+00_JPRB, 0.55874E+00_JPRB ,&
&  0.58348E+00_JPRB, 0.55461E+00_JPRB, 0.48122E+00_JPRB, 0.40473E+00_JPRB, 0.41471E+00_JPRB, 0.36358E+00_JPRB, 0.30621E+00_JPRB ,&
&  0.29764E+00_JPRB, 0.28674E+00_JPRB, 0.26889E+00_JPRB, 0.26757E+00_JPRB, 0.26055E+00_JPRB, 0.28165E+00_JPRB, 0.35044E+00_JPRB ,&
&  0.43305E+00_JPRB, 0.52176E+00_JPRB, 0.57744E+00_JPRB, 0.59329E+00_JPRB, 0.61056E+00_JPRB, 0.64531E+00_JPRB, 0.85959E+00_JPRB ,&
&  0.74498E+00_JPRB, 0.78352E+00_JPRB, 0.67087E+00_JPRB, 0.63630E+00_JPRB, 0.60520E+00_JPRB, 0.57938E+00_JPRB, 0.55062E+00_JPRB ,&
&  0.52316E+00_JPRB, 0.50254E+00_JPRB, 0.48334E+00_JPRB, 0.45956E+00_JPRB, 0.44130E+00_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.40358E+00_JPRB, 0.39373E+00_JPRB, 0.37790E+00_JPRB, 0.38390E+00_JPRB, 0.40959E+00_JPRB, 0.40745E+00_JPRB, 0.40409E+00_JPRB ,&
&  0.41139E+00_JPRB, 0.39962E+00_JPRB, 0.38900E+00_JPRB, 0.37528E+00_JPRB, 0.36692E+00_JPRB, 0.38645E+00_JPRB, 0.43350E+00_JPRB ,&
&  0.52486E+00_JPRB, 0.67305E+00_JPRB, 0.58357E+00_JPRB, 0.51669E+00_JPRB, 0.51911E+00_JPRB, 0.51988E+00_JPRB, 0.45869E+00_JPRB ,&
&  0.65992E+00_JPRB, 0.82281E+00_JPRB, 0.10079E+01_JPRB, 0.14250E+01_JPRB, 0.21603E+01_JPRB, 0.23082E+01_JPRB, 0.23715E+01_JPRB ,&
&  0.23278E+01_JPRB, 0.21385E+01_JPRB, 0.17633E+01_JPRB, 0.14138E+01_JPRB, 0.11620E+01_JPRB, 0.99058E+00_JPRB, 0.82875E+00_JPRB ,&
&  0.71207E+00_JPRB, 0.66434E+00_JPRB, 0.64398E+00_JPRB, 0.63054E+00_JPRB, 0.65646E+00_JPRB, 0.68080E+00_JPRB, 0.69476E+00_JPRB ,&
&  0.72532E+00_JPRB, 0.75164E+00_JPRB, 0.75419E+00_JPRB, 0.74991E+00_JPRB, 0.73733E+00_JPRB, 0.68862E+00_JPRB, 0.92342E+00_JPRB ,&
&  0.98953E+00_JPRB, 0.10556E+01_JPRB, 0.11435E+01_JPRB, 0.15464E+01_JPRB, 0.15265E+01_JPRB, 0.13665E+01_JPRB, 0.15271E+01_JPRB ,&
&  0.11177E+01_JPRB, 0.92705E+00_JPRB, 0.83836E+00_JPRB, 0.76867E+00_JPRB, 0.58494E+00_JPRB, 0.57372E+00_JPRB, 0.68265E+00_JPRB ,&
&  0.56338E+00_JPRB, 0.58140E+00_JPRB, 0.58779E+00_JPRB, 0.53281E+00_JPRB, 0.34042E+00_JPRB, 0.31143E+00_JPRB, 0.38706E+00_JPRB ,&
&  0.40131E+00_JPRB, 0.32220E+00_JPRB, 0.27336E+00_JPRB, 0.30313E+00_JPRB, 0.26077E+00_JPRB, 0.28031E+00_JPRB, 0.34767E+00_JPRB ,&
&  0.39151E+00_JPRB, 0.43994E+00_JPRB, 0.53513E+00_JPRB, 0.55067E+00_JPRB, 0.54443E+00_JPRB, 0.55669E+00_JPRB, 0.66030E+00_JPRB ,&
&  0.68584E+00_JPRB, 0.75984E+00_JPRB, 0.69953E+00_JPRB, 0.60804E+00_JPRB, 0.55017E+00_JPRB, 0.52179E+00_JPRB, 0.50509E+00_JPRB ,&
&  0.48631E+00_JPRB, 0.46551E+00_JPRB, 0.44587E+00_JPRB, 0.42805E+00_JPRB, 0.41415E+00_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.36616E+00_JPRB, 0.36160E+00_JPRB, 0.34918E+00_JPRB, 0.35444E+00_JPRB, 0.37545E+00_JPRB, 0.38443E+00_JPRB, 0.38607E+00_JPRB ,&
&  0.39017E+00_JPRB, 0.38266E+00_JPRB, 0.37695E+00_JPRB, 0.36715E+00_JPRB, 0.35990E+00_JPRB, 0.37340E+00_JPRB, 0.40711E+00_JPRB ,&
&  0.47381E+00_JPRB, 0.50861E+00_JPRB, 0.45126E+00_JPRB, 0.42081E+00_JPRB, 0.51435E+00_JPRB, 0.48102E+00_JPRB, 0.49071E+00_JPRB ,&
&  0.58364E+00_JPRB, 0.66179E+00_JPRB, 0.80734E+00_JPRB, 0.11436E+01_JPRB, 0.15552E+01_JPRB, 0.21606E+01_JPRB, 0.24478E+01_JPRB ,&
&  0.24592E+01_JPRB, 0.23940E+01_JPRB, 0.20740E+01_JPRB, 0.16826E+01_JPRB, 0.13209E+01_JPRB, 0.10931E+01_JPRB, 0.92561E+00_JPRB ,&
&  0.78326E+00_JPRB, 0.67987E+00_JPRB, 0.64084E+00_JPRB, 0.63580E+00_JPRB, 0.63922E+00_JPRB, 0.64507E+00_JPRB, 0.63113E+00_JPRB ,&
&  0.63544E+00_JPRB, 0.66480E+00_JPRB, 0.67764E+00_JPRB, 0.67441E+00_JPRB, 0.63781E+00_JPRB, 0.58050E+00_JPRB, 0.69912E+00_JPRB ,&
&  0.97622E+00_JPRB, 0.10158E+01_JPRB, 0.11553E+01_JPRB, 0.14289E+01_JPRB, 0.15533E+01_JPRB, 0.14027E+01_JPRB, 0.17512E+01_JPRB ,&
&  0.13805E+01_JPRB, 0.10500E+01_JPRB, 0.82185E+00_JPRB, 0.58663E+00_JPRB, 0.54298E+00_JPRB, 0.67920E+00_JPRB, 0.68963E+00_JPRB ,&
&  0.65088E+00_JPRB, 0.62406E+00_JPRB, 0.59322E+00_JPRB, 0.49554E+00_JPRB, 0.37349E+00_JPRB, 0.31502E+00_JPRB, 0.34256E+00_JPRB ,&
&  0.36240E+00_JPRB, 0.38096E+00_JPRB, 0.37091E+00_JPRB, 0.34387E+00_JPRB, 0.29970E+00_JPRB, 0.33187E+00_JPRB, 0.35634E+00_JPRB ,&
&  0.38031E+00_JPRB, 0.37605E+00_JPRB, 0.41289E+00_JPRB, 0.47664E+00_JPRB, 0.49278E+00_JPRB, 0.48838E+00_JPRB, 0.54848E+00_JPRB ,&
&  0.54982E+00_JPRB, 0.58996E+00_JPRB, 0.54381E+00_JPRB, 0.49562E+00_JPRB, 0.47160E+00_JPRB, 0.45741E+00_JPRB, 0.44932E+00_JPRB ,&
&  0.43257E+00_JPRB, 0.41757E+00_JPRB, 0.40245E+00_JPRB, 0.38412E+00_JPRB, 0.37106E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.33764E+00_JPRB, 0.33448E+00_JPRB, 0.32272E+00_JPRB, 0.32547E+00_JPRB, 0.34169E+00_JPRB, 0.35856E+00_JPRB, 0.36941E+00_JPRB ,&
&  0.37161E+00_JPRB, 0.37184E+00_JPRB, 0.36577E+00_JPRB, 0.36009E+00_JPRB, 0.35287E+00_JPRB, 0.35471E+00_JPRB, 0.38280E+00_JPRB ,&
&  0.43953E+00_JPRB, 0.44078E+00_JPRB, 0.37740E+00_JPRB, 0.41375E+00_JPRB, 0.46407E+00_JPRB, 0.43640E+00_JPRB, 0.48322E+00_JPRB ,&
&  0.52315E+00_JPRB, 0.57686E+00_JPRB, 0.65118E+00_JPRB, 0.84354E+00_JPRB, 0.11246E+01_JPRB, 0.15254E+01_JPRB, 0.19124E+01_JPRB ,&
&  0.20518E+01_JPRB, 0.21879E+01_JPRB, 0.22057E+01_JPRB, 0.19838E+01_JPRB, 0.15999E+01_JPRB, 0.12845E+01_JPRB, 0.10975E+01_JPRB ,&
&  0.92447E+00_JPRB, 0.76754E+00_JPRB, 0.67421E+00_JPRB, 0.63859E+00_JPRB, 0.63499E+00_JPRB, 0.64037E+00_JPRB, 0.64212E+00_JPRB ,&
&  0.61000E+00_JPRB, 0.59515E+00_JPRB, 0.59771E+00_JPRB, 0.65211E+00_JPRB, 0.55538E+00_JPRB, 0.55928E+00_JPRB, 0.64302E+00_JPRB ,&
&  0.10772E+01_JPRB, 0.12260E+01_JPRB, 0.13015E+01_JPRB, 0.17296E+01_JPRB, 0.15232E+01_JPRB, 0.15611E+01_JPRB, 0.18576E+01_JPRB ,&
&  0.19984E+01_JPRB, 0.18959E+01_JPRB, 0.17281E+01_JPRB, 0.91786E+00_JPRB, 0.60257E+00_JPRB, 0.76918E+00_JPRB, 0.73393E+00_JPRB ,&
&  0.67302E+00_JPRB, 0.64361E+00_JPRB, 0.59672E+00_JPRB, 0.56722E+00_JPRB, 0.41504E+00_JPRB, 0.34077E+00_JPRB, 0.29629E+00_JPRB ,&
&  0.30097E+00_JPRB, 0.30418E+00_JPRB, 0.33458E+00_JPRB, 0.32786E+00_JPRB, 0.32365E+00_JPRB, 0.32877E+00_JPRB, 0.32854E+00_JPRB ,&
&  0.34088E+00_JPRB, 0.33848E+00_JPRB, 0.34394E+00_JPRB, 0.39393E+00_JPRB, 0.43280E+00_JPRB, 0.41948E+00_JPRB, 0.45354E+00_JPRB ,&
&  0.46483E+00_JPRB, 0.45890E+00_JPRB, 0.44844E+00_JPRB, 0.42564E+00_JPRB, 0.41872E+00_JPRB, 0.41047E+00_JPRB, 0.39878E+00_JPRB ,&
&  0.38349E+00_JPRB, 0.37146E+00_JPRB, 0.35907E+00_JPRB, 0.34838E+00_JPRB, 0.33983E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.32171E+00_JPRB, 0.31990E+00_JPRB, 0.30445E+00_JPRB, 0.30447E+00_JPRB, 0.31501E+00_JPRB, 0.33110E+00_JPRB, 0.34518E+00_JPRB ,&
&  0.35346E+00_JPRB, 0.35476E+00_JPRB, 0.35166E+00_JPRB, 0.35033E+00_JPRB, 0.34321E+00_JPRB, 0.34299E+00_JPRB, 0.36926E+00_JPRB ,&
&  0.41704E+00_JPRB, 0.48662E+00_JPRB, 0.41566E+00_JPRB, 0.36598E+00_JPRB, 0.33541E+00_JPRB, 0.38825E+00_JPRB, 0.42700E+00_JPRB ,&
&  0.50304E+00_JPRB, 0.53886E+00_JPRB, 0.61931E+00_JPRB, 0.71228E+00_JPRB, 0.80128E+00_JPRB, 0.10289E+01_JPRB, 0.13871E+01_JPRB ,&
&  0.18829E+01_JPRB, 0.18904E+01_JPRB, 0.20427E+01_JPRB, 0.19157E+01_JPRB, 0.17157E+01_JPRB, 0.14786E+01_JPRB, 0.12615E+01_JPRB ,&
&  0.10707E+01_JPRB, 0.88763E+00_JPRB, 0.74887E+00_JPRB, 0.65814E+00_JPRB, 0.62591E+00_JPRB, 0.61525E+00_JPRB, 0.63191E+00_JPRB ,&
&  0.62736E+00_JPRB, 0.60263E+00_JPRB, 0.57370E+00_JPRB, 0.57435E+00_JPRB, 0.62335E+00_JPRB, 0.67446E+00_JPRB, 0.83768E+00_JPRB ,&
&  0.10242E+01_JPRB, 0.10363E+01_JPRB, 0.14176E+01_JPRB, 0.17644E+01_JPRB, 0.21428E+01_JPRB, 0.22346E+01_JPRB, 0.21217E+01_JPRB ,&
&  0.20590E+01_JPRB, 0.18123E+01_JPRB, 0.15829E+01_JPRB, 0.10748E+01_JPRB, 0.10714E+01_JPRB, 0.99505E+00_JPRB, 0.85992E+00_JPRB ,&
&  0.75262E+00_JPRB, 0.66849E+00_JPRB, 0.61103E+00_JPRB, 0.56846E+00_JPRB, 0.52281E+00_JPRB, 0.47326E+00_JPRB, 0.36665E+00_JPRB ,&
&  0.31429E+00_JPRB, 0.33156E+00_JPRB, 0.31891E+00_JPRB, 0.30200E+00_JPRB, 0.28533E+00_JPRB, 0.29781E+00_JPRB, 0.30178E+00_JPRB ,&
&  0.30544E+00_JPRB, 0.29924E+00_JPRB, 0.30294E+00_JPRB, 0.33856E+00_JPRB, 0.38317E+00_JPRB, 0.39870E+00_JPRB, 0.40275E+00_JPRB ,&
&  0.38775E+00_JPRB, 0.39433E+00_JPRB, 0.39211E+00_JPRB, 0.37064E+00_JPRB, 0.38765E+00_JPRB, 0.37601E+00_JPRB, 0.35863E+00_JPRB ,&
&  0.34440E+00_JPRB, 0.33766E+00_JPRB, 0.33109E+00_JPRB, 0.32588E+00_JPRB, 0.32023E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.30797E+00_JPRB, 0.30989E+00_JPRB, 0.29940E+00_JPRB, 0.29792E+00_JPRB, 0.30475E+00_JPRB, 0.31546E+00_JPRB, 0.32606E+00_JPRB ,&
&  0.33471E+00_JPRB, 0.33541E+00_JPRB, 0.33276E+00_JPRB, 0.33388E+00_JPRB, 0.33369E+00_JPRB, 0.33815E+00_JPRB, 0.37002E+00_JPRB ,&
&  0.43511E+00_JPRB, 0.59599E+00_JPRB, 0.46676E+00_JPRB, 0.42391E+00_JPRB, 0.35810E+00_JPRB, 0.36999E+00_JPRB, 0.39803E+00_JPRB ,&
&  0.52842E+00_JPRB, 0.51882E+00_JPRB, 0.51928E+00_JPRB, 0.55467E+00_JPRB, 0.62555E+00_JPRB, 0.75417E+00_JPRB, 0.99598E+00_JPRB ,&
&  0.12302E+01_JPRB, 0.15444E+01_JPRB, 0.16740E+01_JPRB, 0.18155E+01_JPRB, 0.16671E+01_JPRB, 0.13930E+01_JPRB, 0.13130E+01_JPRB ,&
&  0.11560E+01_JPRB, 0.98698E+00_JPRB, 0.83491E+00_JPRB, 0.73350E+00_JPRB, 0.66892E+00_JPRB, 0.62521E+00_JPRB, 0.61545E+00_JPRB ,&
&  0.61509E+00_JPRB, 0.60261E+00_JPRB, 0.58383E+00_JPRB, 0.57390E+00_JPRB, 0.62617E+00_JPRB, 0.78631E+00_JPRB, 0.95856E+00_JPRB ,&
&  0.14312E+01_JPRB, 0.15452E+01_JPRB, 0.13715E+01_JPRB, 0.19104E+01_JPRB, 0.25869E+01_JPRB, 0.24707E+01_JPRB, 0.20265E+01_JPRB ,&
&  0.19222E+01_JPRB, 0.16783E+01_JPRB, 0.14534E+01_JPRB, 0.12741E+01_JPRB, 0.11394E+01_JPRB, 0.10204E+01_JPRB, 0.88298E+00_JPRB ,&
&  0.74635E+00_JPRB, 0.65140E+00_JPRB, 0.59366E+00_JPRB, 0.53323E+00_JPRB, 0.48474E+00_JPRB, 0.44392E+00_JPRB, 0.40848E+00_JPRB ,&
&  0.35972E+00_JPRB, 0.32156E+00_JPRB, 0.27249E+00_JPRB, 0.28180E+00_JPRB, 0.25370E+00_JPRB, 0.26000E+00_JPRB, 0.27555E+00_JPRB ,&
&  0.27431E+00_JPRB, 0.28152E+00_JPRB, 0.28554E+00_JPRB, 0.29436E+00_JPRB, 0.33172E+00_JPRB, 0.35173E+00_JPRB, 0.35784E+00_JPRB ,&
&  0.34857E+00_JPRB, 0.33150E+00_JPRB, 0.34744E+00_JPRB, 0.34665E+00_JPRB, 0.35177E+00_JPRB, 0.33371E+00_JPRB, 0.31498E+00_JPRB ,&
&  0.30407E+00_JPRB, 0.30184E+00_JPRB, 0.30130E+00_JPRB, 0.30166E+00_JPRB, 0.30273E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.28665E+00_JPRB, 0.28766E+00_JPRB, 0.28504E+00_JPRB, 0.28949E+00_JPRB, 0.29406E+00_JPRB, 0.29987E+00_JPRB, 0.30999E+00_JPRB ,&
&  0.32124E+00_JPRB, 0.32237E+00_JPRB, 0.31699E+00_JPRB, 0.31605E+00_JPRB, 0.31886E+00_JPRB, 0.32782E+00_JPRB, 0.36979E+00_JPRB ,&
&  0.46217E+00_JPRB, 0.48156E+00_JPRB, 0.39646E+00_JPRB, 0.32948E+00_JPRB, 0.36229E+00_JPRB, 0.35334E+00_JPRB, 0.37738E+00_JPRB ,&
&  0.36886E+00_JPRB, 0.39354E+00_JPRB, 0.38748E+00_JPRB, 0.39371E+00_JPRB, 0.42240E+00_JPRB, 0.52257E+00_JPRB, 0.67661E+00_JPRB ,&
&  0.81956E+00_JPRB, 0.10290E+01_JPRB, 0.12592E+01_JPRB, 0.12999E+01_JPRB, 0.13587E+01_JPRB, 0.12160E+01_JPRB, 0.11828E+01_JPRB ,&
&  0.10783E+01_JPRB, 0.94431E+00_JPRB, 0.82238E+00_JPRB, 0.74230E+00_JPRB, 0.68623E+00_JPRB, 0.64580E+00_JPRB, 0.62552E+00_JPRB ,&
&  0.62165E+00_JPRB, 0.61232E+00_JPRB, 0.59019E+00_JPRB, 0.59507E+00_JPRB, 0.70710E+00_JPRB, 0.99751E+00_JPRB, 0.13921E+01_JPRB ,&
&  0.17913E+01_JPRB, 0.20528E+01_JPRB, 0.21850E+01_JPRB, 0.24052E+01_JPRB, 0.24534E+01_JPRB, 0.23975E+01_JPRB, 0.21584E+01_JPRB ,&
&  0.17920E+01_JPRB, 0.14881E+01_JPRB, 0.12669E+01_JPRB, 0.10893E+01_JPRB, 0.96489E+00_JPRB, 0.90234E+00_JPRB, 0.82018E+00_JPRB ,&
&  0.68697E+00_JPRB, 0.60370E+00_JPRB, 0.55630E+00_JPRB, 0.48203E+00_JPRB, 0.44308E+00_JPRB, 0.40952E+00_JPRB, 0.40224E+00_JPRB ,&
&  0.36139E+00_JPRB, 0.27141E+00_JPRB, 0.25208E+00_JPRB, 0.27173E+00_JPRB, 0.25104E+00_JPRB, 0.25783E+00_JPRB, 0.27361E+00_JPRB ,&
&  0.26900E+00_JPRB, 0.26910E+00_JPRB, 0.27655E+00_JPRB, 0.28317E+00_JPRB, 0.30092E+00_JPRB, 0.32611E+00_JPRB, 0.32382E+00_JPRB ,&
&  0.31290E+00_JPRB, 0.30851E+00_JPRB, 0.32877E+00_JPRB, 0.33269E+00_JPRB, 0.32399E+00_JPRB, 0.29710E+00_JPRB, 0.27884E+00_JPRB ,&
&  0.27348E+00_JPRB, 0.27488E+00_JPRB, 0.27656E+00_JPRB, 0.27818E+00_JPRB, 0.28056E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.27056E+00_JPRB, 0.27119E+00_JPRB, 0.27150E+00_JPRB, 0.27987E+00_JPRB, 0.28911E+00_JPRB, 0.29332E+00_JPRB, 0.30261E+00_JPRB ,&
&  0.31207E+00_JPRB, 0.31429E+00_JPRB, 0.31165E+00_JPRB, 0.30803E+00_JPRB, 0.30896E+00_JPRB, 0.32016E+00_JPRB, 0.36692E+00_JPRB ,&
&  0.33949E+00_JPRB, 0.34181E+00_JPRB, 0.30218E+00_JPRB, 0.30649E+00_JPRB, 0.35357E+00_JPRB, 0.32383E+00_JPRB, 0.31695E+00_JPRB ,&
&  0.30651E+00_JPRB, 0.30108E+00_JPRB, 0.29097E+00_JPRB, 0.29115E+00_JPRB, 0.30620E+00_JPRB, 0.35799E+00_JPRB, 0.46837E+00_JPRB ,&
&  0.53975E+00_JPRB, 0.65734E+00_JPRB, 0.79284E+00_JPRB, 0.82017E+00_JPRB, 0.90094E+00_JPRB, 0.95473E+00_JPRB, 0.99152E+00_JPRB ,&
&  0.94919E+00_JPRB, 0.84034E+00_JPRB, 0.74236E+00_JPRB, 0.68424E+00_JPRB, 0.64431E+00_JPRB, 0.62007E+00_JPRB, 0.59939E+00_JPRB ,&
&  0.58117E+00_JPRB, 0.57891E+00_JPRB, 0.60120E+00_JPRB, 0.65870E+00_JPRB, 0.75370E+00_JPRB, 0.11603E+01_JPRB, 0.18486E+01_JPRB ,&
&  0.20370E+01_JPRB, 0.21652E+01_JPRB, 0.22905E+01_JPRB, 0.24982E+01_JPRB, 0.24442E+01_JPRB, 0.22057E+01_JPRB, 0.17964E+01_JPRB ,&
&  0.14483E+01_JPRB, 0.12224E+01_JPRB, 0.10589E+01_JPRB, 0.93902E+00_JPRB, 0.80613E+00_JPRB, 0.77282E+00_JPRB, 0.70574E+00_JPRB ,&
&  0.60041E+00_JPRB, 0.54315E+00_JPRB, 0.51113E+00_JPRB, 0.47554E+00_JPRB, 0.44486E+00_JPRB, 0.43790E+00_JPRB, 0.42112E+00_JPRB ,&
&  0.38781E+00_JPRB, 0.34061E+00_JPRB, 0.29183E+00_JPRB, 0.28412E+00_JPRB, 0.26018E+00_JPRB, 0.26885E+00_JPRB, 0.28958E+00_JPRB ,&
&  0.27896E+00_JPRB, 0.26935E+00_JPRB, 0.27437E+00_JPRB, 0.27962E+00_JPRB, 0.28528E+00_JPRB, 0.30499E+00_JPRB, 0.31248E+00_JPRB ,&
&  0.30128E+00_JPRB, 0.31090E+00_JPRB, 0.31824E+00_JPRB, 0.30845E+00_JPRB, 0.29125E+00_JPRB, 0.26597E+00_JPRB, 0.25054E+00_JPRB ,&
&  0.25228E+00_JPRB, 0.25372E+00_JPRB, 0.25922E+00_JPRB, 0.26278E+00_JPRB, 0.26604E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.25949E+00_JPRB, 0.26128E+00_JPRB, 0.26283E+00_JPRB, 0.26969E+00_JPRB, 0.28244E+00_JPRB, 0.28876E+00_JPRB, 0.29822E+00_JPRB ,&
&  0.30494E+00_JPRB, 0.30509E+00_JPRB, 0.30817E+00_JPRB, 0.30348E+00_JPRB, 0.30392E+00_JPRB, 0.31247E+00_JPRB, 0.31004E+00_JPRB ,&
&  0.27905E+00_JPRB, 0.28730E+00_JPRB, 0.27793E+00_JPRB, 0.27796E+00_JPRB, 0.27728E+00_JPRB, 0.26544E+00_JPRB, 0.25620E+00_JPRB ,&
&  0.26039E+00_JPRB, 0.25573E+00_JPRB, 0.25549E+00_JPRB, 0.26400E+00_JPRB, 0.28002E+00_JPRB, 0.31431E+00_JPRB, 0.38078E+00_JPRB ,&
&  0.42639E+00_JPRB, 0.49268E+00_JPRB, 0.55517E+00_JPRB, 0.56963E+00_JPRB, 0.66622E+00_JPRB, 0.74723E+00_JPRB, 0.77268E+00_JPRB ,&
&  0.77702E+00_JPRB, 0.72936E+00_JPRB, 0.66957E+00_JPRB, 0.62150E+00_JPRB, 0.57700E+00_JPRB, 0.55212E+00_JPRB, 0.53980E+00_JPRB ,&
&  0.53003E+00_JPRB, 0.52817E+00_JPRB, 0.54107E+00_JPRB, 0.57600E+00_JPRB, 0.64879E+00_JPRB, 0.95237E+00_JPRB, 0.13051E+01_JPRB ,&
&  0.14351E+01_JPRB, 0.15254E+01_JPRB, 0.15831E+01_JPRB, 0.15888E+01_JPRB, 0.15632E+01_JPRB, 0.14037E+01_JPRB, 0.11722E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.92597E+00_JPRB, 0.86466E+00_JPRB, 0.77878E+00_JPRB, 0.70291E+00_JPRB, 0.66074E+00_JPRB, 0.60043E+00_JPRB ,&
&  0.54483E+00_JPRB, 0.49056E+00_JPRB, 0.48468E+00_JPRB, 0.46327E+00_JPRB, 0.44371E+00_JPRB, 0.43319E+00_JPRB, 0.40700E+00_JPRB ,&
&  0.38595E+00_JPRB, 0.36493E+00_JPRB, 0.33102E+00_JPRB, 0.31166E+00_JPRB, 0.29745E+00_JPRB, 0.30344E+00_JPRB, 0.29156E+00_JPRB ,&
&  0.27712E+00_JPRB, 0.25914E+00_JPRB, 0.26674E+00_JPRB, 0.27453E+00_JPRB, 0.27842E+00_JPRB, 0.28399E+00_JPRB, 0.29236E+00_JPRB ,&
&  0.29890E+00_JPRB, 0.30817E+00_JPRB, 0.28988E+00_JPRB, 0.27336E+00_JPRB, 0.25850E+00_JPRB, 0.24323E+00_JPRB, 0.22955E+00_JPRB ,&
&  0.22833E+00_JPRB, 0.23756E+00_JPRB, 0.24536E+00_JPRB, 0.25185E+00_JPRB, 0.25701E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.25331E+00_JPRB, 0.25780E+00_JPRB, 0.26142E+00_JPRB, 0.26490E+00_JPRB, 0.27608E+00_JPRB, 0.28332E+00_JPRB, 0.28459E+00_JPRB ,&
&  0.29931E+00_JPRB, 0.29854E+00_JPRB, 0.29946E+00_JPRB, 0.29787E+00_JPRB, 0.29697E+00_JPRB, 0.27367E+00_JPRB, 0.23198E+00_JPRB ,&
&  0.22124E+00_JPRB, 0.22845E+00_JPRB, 0.24255E+00_JPRB, 0.24783E+00_JPRB, 0.24415E+00_JPRB, 0.23657E+00_JPRB, 0.23088E+00_JPRB ,&
&  0.23147E+00_JPRB, 0.23369E+00_JPRB, 0.24100E+00_JPRB, 0.25110E+00_JPRB, 0.25839E+00_JPRB, 0.26970E+00_JPRB, 0.29804E+00_JPRB ,&
&  0.32837E+00_JPRB, 0.37151E+00_JPRB, 0.42817E+00_JPRB, 0.41376E+00_JPRB, 0.49719E+00_JPRB, 0.56068E+00_JPRB, 0.56286E+00_JPRB ,&
&  0.58154E+00_JPRB, 0.56845E+00_JPRB, 0.54140E+00_JPRB, 0.52794E+00_JPRB, 0.50137E+00_JPRB, 0.47553E+00_JPRB, 0.46127E+00_JPRB ,&
&  0.45453E+00_JPRB, 0.45455E+00_JPRB, 0.46548E+00_JPRB, 0.48511E+00_JPRB, 0.53663E+00_JPRB, 0.70886E+00_JPRB, 0.84537E+00_JPRB ,&
&  0.89948E+00_JPRB, 0.98157E+00_JPRB, 0.10884E+01_JPRB, 0.10468E+01_JPRB, 0.10513E+01_JPRB, 0.10093E+01_JPRB, 0.92229E+00_JPRB ,&
&  0.87599E+00_JPRB, 0.77883E+00_JPRB, 0.73771E+00_JPRB, 0.66708E+00_JPRB, 0.61626E+00_JPRB, 0.57433E+00_JPRB, 0.52092E+00_JPRB ,&
&  0.48855E+00_JPRB, 0.45123E+00_JPRB, 0.41537E+00_JPRB, 0.39433E+00_JPRB, 0.37689E+00_JPRB, 0.36995E+00_JPRB, 0.35788E+00_JPRB ,&
&  0.34881E+00_JPRB, 0.34114E+00_JPRB, 0.33019E+00_JPRB, 0.31749E+00_JPRB, 0.30209E+00_JPRB, 0.28835E+00_JPRB, 0.27756E+00_JPRB ,&
&  0.27009E+00_JPRB, 0.25710E+00_JPRB, 0.25628E+00_JPRB, 0.26562E+00_JPRB, 0.26811E+00_JPRB, 0.26137E+00_JPRB, 0.26691E+00_JPRB ,&
&  0.27521E+00_JPRB, 0.27599E+00_JPRB, 0.25944E+00_JPRB, 0.25151E+00_JPRB, 0.24586E+00_JPRB, 0.23566E+00_JPRB, 0.22420E+00_JPRB ,&
&  0.21666E+00_JPRB, 0.22350E+00_JPRB, 0.23122E+00_JPRB, 0.23884E+00_JPRB, 0.24725E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.24796E+00_JPRB, 0.25724E+00_JPRB, 0.26066E+00_JPRB, 0.26148E+00_JPRB, 0.26755E+00_JPRB, 0.26646E+00_JPRB, 0.26890E+00_JPRB ,&
&  0.28912E+00_JPRB, 0.27994E+00_JPRB, 0.29461E+00_JPRB, 0.25068E+00_JPRB, 0.22947E+00_JPRB, 0.22438E+00_JPRB, 0.21697E+00_JPRB ,&
&  0.21665E+00_JPRB, 0.22167E+00_JPRB, 0.22531E+00_JPRB, 0.22734E+00_JPRB, 0.22898E+00_JPRB, 0.22329E+00_JPRB, 0.21959E+00_JPRB ,&
&  0.22045E+00_JPRB, 0.22566E+00_JPRB, 0.23323E+00_JPRB, 0.23434E+00_JPRB, 0.23369E+00_JPRB, 0.23468E+00_JPRB, 0.24159E+00_JPRB ,&
&  0.24823E+00_JPRB, 0.27321E+00_JPRB, 0.32020E+00_JPRB, 0.34381E+00_JPRB, 0.39160E+00_JPRB, 0.44859E+00_JPRB, 0.44910E+00_JPRB ,&
&  0.44231E+00_JPRB, 0.38692E+00_JPRB, 0.40930E+00_JPRB, 0.41624E+00_JPRB, 0.41693E+00_JPRB, 0.40628E+00_JPRB, 0.39400E+00_JPRB ,&
&  0.38656E+00_JPRB, 0.38643E+00_JPRB, 0.39366E+00_JPRB, 0.40724E+00_JPRB, 0.43316E+00_JPRB, 0.48826E+00_JPRB, 0.53431E+00_JPRB ,&
&  0.60146E+00_JPRB, 0.56759E+00_JPRB, 0.68706E+00_JPRB, 0.69698E+00_JPRB, 0.73810E+00_JPRB, 0.75253E+00_JPRB, 0.77853E+00_JPRB ,&
&  0.78850E+00_JPRB, 0.70809E+00_JPRB, 0.66454E+00_JPRB, 0.58623E+00_JPRB, 0.54328E+00_JPRB, 0.50003E+00_JPRB, 0.46075E+00_JPRB ,&
&  0.42969E+00_JPRB, 0.38582E+00_JPRB, 0.36359E+00_JPRB, 0.33614E+00_JPRB, 0.31766E+00_JPRB, 0.31216E+00_JPRB, 0.31099E+00_JPRB ,&
&  0.31056E+00_JPRB, 0.30774E+00_JPRB, 0.29596E+00_JPRB, 0.27639E+00_JPRB, 0.27035E+00_JPRB, 0.26461E+00_JPRB, 0.26080E+00_JPRB ,&
&  0.26454E+00_JPRB, 0.26390E+00_JPRB, 0.26652E+00_JPRB, 0.27351E+00_JPRB, 0.26615E+00_JPRB, 0.26125E+00_JPRB, 0.25701E+00_JPRB ,&
&  0.25967E+00_JPRB, 0.24976E+00_JPRB, 0.23860E+00_JPRB, 0.23586E+00_JPRB, 0.23008E+00_JPRB, 0.22320E+00_JPRB, 0.21150E+00_JPRB ,&
&  0.20784E+00_JPRB, 0.21235E+00_JPRB, 0.21909E+00_JPRB, 0.22720E+00_JPRB, 0.23699E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.23768E+00_JPRB, 0.25112E+00_JPRB, 0.25655E+00_JPRB, 0.25804E+00_JPRB, 0.26130E+00_JPRB, 0.25891E+00_JPRB, 0.24985E+00_JPRB ,&
&  0.25309E+00_JPRB, 0.24506E+00_JPRB, 0.23186E+00_JPRB, 0.22387E+00_JPRB, 0.22038E+00_JPRB, 0.21645E+00_JPRB, 0.20887E+00_JPRB ,&
&  0.20699E+00_JPRB, 0.20987E+00_JPRB, 0.21524E+00_JPRB, 0.21623E+00_JPRB, 0.21463E+00_JPRB, 0.21198E+00_JPRB, 0.21136E+00_JPRB ,&
&  0.21462E+00_JPRB, 0.22025E+00_JPRB, 0.22468E+00_JPRB, 0.22469E+00_JPRB, 0.22355E+00_JPRB, 0.22495E+00_JPRB, 0.22885E+00_JPRB ,&
&  0.22993E+00_JPRB, 0.23209E+00_JPRB, 0.24914E+00_JPRB, 0.27971E+00_JPRB, 0.33578E+00_JPRB, 0.38761E+00_JPRB, 0.38550E+00_JPRB ,&
&  0.27812E+00_JPRB, 0.26566E+00_JPRB, 0.32073E+00_JPRB, 0.33276E+00_JPRB, 0.34168E+00_JPRB, 0.33951E+00_JPRB, 0.33759E+00_JPRB ,&
&  0.33527E+00_JPRB, 0.33474E+00_JPRB, 0.33300E+00_JPRB, 0.33872E+00_JPRB, 0.35541E+00_JPRB, 0.37920E+00_JPRB, 0.39521E+00_JPRB ,&
&  0.43170E+00_JPRB, 0.41966E+00_JPRB, 0.42691E+00_JPRB, 0.44247E+00_JPRB, 0.54099E+00_JPRB, 0.59454E+00_JPRB, 0.63614E+00_JPRB ,&
&  0.63944E+00_JPRB, 0.62310E+00_JPRB, 0.63034E+00_JPRB, 0.58767E+00_JPRB, 0.55713E+00_JPRB, 0.50016E+00_JPRB, 0.44674E+00_JPRB ,&
&  0.41313E+00_JPRB, 0.37417E+00_JPRB, 0.34780E+00_JPRB, 0.32152E+00_JPRB, 0.29194E+00_JPRB, 0.27536E+00_JPRB, 0.27061E+00_JPRB ,&
&  0.27362E+00_JPRB, 0.27564E+00_JPRB, 0.26637E+00_JPRB, 0.25570E+00_JPRB, 0.25018E+00_JPRB, 0.25031E+00_JPRB, 0.25344E+00_JPRB ,&
&  0.25758E+00_JPRB, 0.26397E+00_JPRB, 0.26747E+00_JPRB, 0.26932E+00_JPRB, 0.26222E+00_JPRB, 0.25057E+00_JPRB, 0.24339E+00_JPRB ,&
&  0.24640E+00_JPRB, 0.22861E+00_JPRB, 0.22428E+00_JPRB, 0.22275E+00_JPRB, 0.22379E+00_JPRB, 0.22111E+00_JPRB, 0.21242E+00_JPRB ,&
&  0.20519E+00_JPRB, 0.20587E+00_JPRB, 0.20729E+00_JPRB, 0.21195E+00_JPRB, 0.22289E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.22470E+00_JPRB, 0.23401E+00_JPRB, 0.24190E+00_JPRB, 0.24742E+00_JPRB, 0.24713E+00_JPRB, 0.24348E+00_JPRB, 0.23852E+00_JPRB ,&
&  0.24120E+00_JPRB, 0.23887E+00_JPRB, 0.23250E+00_JPRB, 0.22268E+00_JPRB, 0.21519E+00_JPRB, 0.20879E+00_JPRB, 0.20793E+00_JPRB ,&
&  0.21004E+00_JPRB, 0.21230E+00_JPRB, 0.21151E+00_JPRB, 0.20795E+00_JPRB, 0.20479E+00_JPRB, 0.20383E+00_JPRB, 0.20649E+00_JPRB ,&
&  0.21245E+00_JPRB, 0.21747E+00_JPRB, 0.21933E+00_JPRB, 0.22009E+00_JPRB, 0.22258E+00_JPRB, 0.22631E+00_JPRB, 0.22794E+00_JPRB ,&
&  0.22504E+00_JPRB, 0.22451E+00_JPRB, 0.22820E+00_JPRB, 0.24206E+00_JPRB, 0.28596E+00_JPRB, 0.33120E+00_JPRB, 0.29413E+00_JPRB ,&
&  0.22947E+00_JPRB, 0.20933E+00_JPRB, 0.21654E+00_JPRB, 0.26226E+00_JPRB, 0.27543E+00_JPRB, 0.28340E+00_JPRB, 0.29135E+00_JPRB ,&
&  0.28314E+00_JPRB, 0.27487E+00_JPRB, 0.27783E+00_JPRB, 0.29676E+00_JPRB, 0.30632E+00_JPRB, 0.31931E+00_JPRB, 0.33251E+00_JPRB ,&
&  0.34921E+00_JPRB, 0.36666E+00_JPRB, 0.37915E+00_JPRB, 0.35174E+00_JPRB, 0.38268E+00_JPRB, 0.48330E+00_JPRB, 0.49474E+00_JPRB ,&
&  0.49604E+00_JPRB, 0.49551E+00_JPRB, 0.54809E+00_JPRB, 0.54042E+00_JPRB, 0.51792E+00_JPRB, 0.48240E+00_JPRB, 0.44612E+00_JPRB ,&
&  0.41258E+00_JPRB, 0.37836E+00_JPRB, 0.35082E+00_JPRB, 0.32403E+00_JPRB, 0.29654E+00_JPRB, 0.27427E+00_JPRB, 0.25841E+00_JPRB ,&
&  0.25374E+00_JPRB, 0.25350E+00_JPRB, 0.24640E+00_JPRB, 0.23842E+00_JPRB, 0.23495E+00_JPRB, 0.23598E+00_JPRB, 0.24283E+00_JPRB ,&
&  0.25133E+00_JPRB, 0.26096E+00_JPRB, 0.26937E+00_JPRB, 0.27245E+00_JPRB, 0.25884E+00_JPRB, 0.24205E+00_JPRB, 0.23674E+00_JPRB ,&
&  0.23779E+00_JPRB, 0.23037E+00_JPRB, 0.22389E+00_JPRB, 0.22577E+00_JPRB, 0.23147E+00_JPRB, 0.23450E+00_JPRB, 0.22883E+00_JPRB ,&
&  0.21839E+00_JPRB, 0.21223E+00_JPRB, 0.20976E+00_JPRB, 0.21290E+00_JPRB, 0.21799E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.20728E+00_JPRB, 0.21326E+00_JPRB, 0.22382E+00_JPRB, 0.23350E+00_JPRB, 0.23834E+00_JPRB, 0.22870E+00_JPRB, 0.21747E+00_JPRB ,&
&  0.21173E+00_JPRB, 0.21042E+00_JPRB, 0.21213E+00_JPRB, 0.21308E+00_JPRB, 0.21342E+00_JPRB, 0.21381E+00_JPRB, 0.21388E+00_JPRB ,&
&  0.21175E+00_JPRB, 0.20846E+00_JPRB, 0.20532E+00_JPRB, 0.20270E+00_JPRB, 0.20162E+00_JPRB, 0.20304E+00_JPRB, 0.20780E+00_JPRB ,&
&  0.21427E+00_JPRB, 0.21746E+00_JPRB, 0.21914E+00_JPRB, 0.21954E+00_JPRB, 0.22270E+00_JPRB, 0.22676E+00_JPRB, 0.22657E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22218E+00_JPRB, 0.22427E+00_JPRB, 0.23244E+00_JPRB, 0.25512E+00_JPRB, 0.29084E+00_JPRB, 0.26048E+00_JPRB ,&
&  0.21251E+00_JPRB, 0.18578E+00_JPRB, 0.17720E+00_JPRB, 0.18456E+00_JPRB, 0.20610E+00_JPRB, 0.24293E+00_JPRB, 0.25208E+00_JPRB ,&
&  0.25945E+00_JPRB, 0.26809E+00_JPRB, 0.27434E+00_JPRB, 0.28114E+00_JPRB, 0.28644E+00_JPRB, 0.28953E+00_JPRB, 0.29016E+00_JPRB ,&
&  0.29360E+00_JPRB, 0.30136E+00_JPRB, 0.30995E+00_JPRB, 0.29425E+00_JPRB, 0.28767E+00_JPRB, 0.32363E+00_JPRB, 0.35523E+00_JPRB ,&
&  0.37627E+00_JPRB, 0.36152E+00_JPRB, 0.38036E+00_JPRB, 0.40436E+00_JPRB, 0.44765E+00_JPRB, 0.43647E+00_JPRB, 0.41580E+00_JPRB ,&
&  0.39231E+00_JPRB, 0.36267E+00_JPRB, 0.34611E+00_JPRB, 0.32769E+00_JPRB, 0.30369E+00_JPRB, 0.28208E+00_JPRB, 0.26507E+00_JPRB ,&
&  0.25245E+00_JPRB, 0.24673E+00_JPRB, 0.23942E+00_JPRB, 0.23488E+00_JPRB, 0.23481E+00_JPRB, 0.23870E+00_JPRB, 0.24416E+00_JPRB ,&
&  0.24839E+00_JPRB, 0.25373E+00_JPRB, 0.26298E+00_JPRB, 0.26858E+00_JPRB, 0.25958E+00_JPRB, 0.24953E+00_JPRB, 0.24319E+00_JPRB ,&
&  0.24231E+00_JPRB, 0.24374E+00_JPRB, 0.24702E+00_JPRB, 0.24970E+00_JPRB, 0.24684E+00_JPRB, 0.24904E+00_JPRB, 0.24728E+00_JPRB ,&
&  0.23475E+00_JPRB, 0.22251E+00_JPRB, 0.21480E+00_JPRB, 0.21054E+00_JPRB, 0.20813E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.21093E+00_JPRB, 0.21095E+00_JPRB, 0.21342E+00_JPRB, 0.21837E+00_JPRB, 0.22403E+00_JPRB, 0.22272E+00_JPRB, 0.21978E+00_JPRB ,&
&  0.21757E+00_JPRB, 0.21799E+00_JPRB, 0.21720E+00_JPRB, 0.21553E+00_JPRB, 0.21409E+00_JPRB, 0.21257E+00_JPRB, 0.21087E+00_JPRB ,&
&  0.20894E+00_JPRB, 0.20684E+00_JPRB, 0.20462E+00_JPRB, 0.20229E+00_JPRB, 0.20237E+00_JPRB, 0.20590E+00_JPRB, 0.21287E+00_JPRB ,&
&  0.21893E+00_JPRB, 0.22117E+00_JPRB, 0.22341E+00_JPRB, 0.22501E+00_JPRB, 0.22491E+00_JPRB, 0.22331E+00_JPRB, 0.22220E+00_JPRB ,&
&  0.21826E+00_JPRB, 0.21769E+00_JPRB, 0.22154E+00_JPRB, 0.22982E+00_JPRB, 0.24467E+00_JPRB, 0.26133E+00_JPRB, 0.23170E+00_JPRB ,&
&  0.19463E+00_JPRB, 0.17125E+00_JPRB, 0.16013E+00_JPRB, 0.15800E+00_JPRB, 0.16503E+00_JPRB, 0.18017E+00_JPRB, 0.20337E+00_JPRB ,&
&  0.23958E+00_JPRB, 0.26385E+00_JPRB, 0.27388E+00_JPRB, 0.28408E+00_JPRB, 0.28655E+00_JPRB, 0.28476E+00_JPRB, 0.28222E+00_JPRB ,&
&  0.28425E+00_JPRB, 0.28831E+00_JPRB, 0.29135E+00_JPRB, 0.28561E+00_JPRB, 0.28050E+00_JPRB, 0.27105E+00_JPRB, 0.27373E+00_JPRB ,&
&  0.30066E+00_JPRB, 0.29457E+00_JPRB, 0.28622E+00_JPRB, 0.31258E+00_JPRB, 0.35911E+00_JPRB, 0.37654E+00_JPRB, 0.38014E+00_JPRB ,&
&  0.36644E+00_JPRB, 0.34090E+00_JPRB, 0.32017E+00_JPRB, 0.30853E+00_JPRB, 0.29430E+00_JPRB, 0.27968E+00_JPRB, 0.26821E+00_JPRB ,&
&  0.25656E+00_JPRB, 0.24709E+00_JPRB, 0.25451E+00_JPRB, 0.24286E+00_JPRB, 0.24031E+00_JPRB, 0.23830E+00_JPRB, 0.23889E+00_JPRB ,&
&  0.24250E+00_JPRB, 0.24853E+00_JPRB, 0.25739E+00_JPRB, 0.26655E+00_JPRB, 0.26591E+00_JPRB, 0.26071E+00_JPRB, 0.25689E+00_JPRB ,&
&  0.25607E+00_JPRB, 0.25559E+00_JPRB, 0.25867E+00_JPRB, 0.26463E+00_JPRB, 0.26656E+00_JPRB, 0.26793E+00_JPRB, 0.27072E+00_JPRB ,&
&  0.26017E+00_JPRB, 0.24311E+00_JPRB, 0.23383E+00_JPRB, 0.22524E+00_JPRB, 0.21645E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.21116E+00_JPRB, 0.20829E+00_JPRB, 0.20726E+00_JPRB, 0.20808E+00_JPRB, 0.20861E+00_JPRB, 0.20969E+00_JPRB, 0.20920E+00_JPRB ,&
&  0.20845E+00_JPRB, 0.20776E+00_JPRB, 0.20668E+00_JPRB, 0.20566E+00_JPRB, 0.20492E+00_JPRB, 0.20406E+00_JPRB, 0.20347E+00_JPRB ,&
&  0.20301E+00_JPRB, 0.20067E+00_JPRB, 0.20082E+00_JPRB, 0.20391E+00_JPRB, 0.20835E+00_JPRB, 0.21328E+00_JPRB, 0.21927E+00_JPRB ,&
&  0.22221E+00_JPRB, 0.22298E+00_JPRB, 0.22611E+00_JPRB, 0.22758E+00_JPRB, 0.22559E+00_JPRB, 0.22198E+00_JPRB, 0.21903E+00_JPRB ,&
&  0.21802E+00_JPRB, 0.21667E+00_JPRB, 0.22303E+00_JPRB, 0.22781E+00_JPRB, 0.22789E+00_JPRB, 0.24013E+00_JPRB, 0.20305E+00_JPRB ,&
&  0.17426E+00_JPRB, 0.15471E+00_JPRB, 0.14812E+00_JPRB, 0.14524E+00_JPRB, 0.14899E+00_JPRB, 0.15642E+00_JPRB, 0.17359E+00_JPRB ,&
&  0.20086E+00_JPRB, 0.24956E+00_JPRB, 0.26263E+00_JPRB, 0.26734E+00_JPRB, 0.27200E+00_JPRB, 0.26956E+00_JPRB, 0.26796E+00_JPRB ,&
&  0.26905E+00_JPRB, 0.26555E+00_JPRB, 0.25998E+00_JPRB, 0.25153E+00_JPRB, 0.24402E+00_JPRB, 0.23880E+00_JPRB, 0.23140E+00_JPRB ,&
&  0.23305E+00_JPRB, 0.23584E+00_JPRB, 0.24294E+00_JPRB, 0.26228E+00_JPRB, 0.29804E+00_JPRB, 0.32555E+00_JPRB, 0.33172E+00_JPRB ,&
&  0.31643E+00_JPRB, 0.29675E+00_JPRB, 0.27908E+00_JPRB, 0.26995E+00_JPRB, 0.26135E+00_JPRB, 0.25540E+00_JPRB, 0.25273E+00_JPRB ,&
&  0.25246E+00_JPRB, 0.25071E+00_JPRB, 0.25148E+00_JPRB, 0.25093E+00_JPRB, 0.24970E+00_JPRB, 0.24560E+00_JPRB, 0.24428E+00_JPRB ,&
&  0.24322E+00_JPRB, 0.24686E+00_JPRB, 0.25445E+00_JPRB, 0.26016E+00_JPRB, 0.26402E+00_JPRB, 0.26385E+00_JPRB, 0.26517E+00_JPRB ,&
&  0.26506E+00_JPRB, 0.26423E+00_JPRB, 0.26578E+00_JPRB, 0.27250E+00_JPRB, 0.27856E+00_JPRB, 0.28445E+00_JPRB, 0.28883E+00_JPRB ,&
&  0.27958E+00_JPRB, 0.25807E+00_JPRB, 0.24023E+00_JPRB, 0.22728E+00_JPRB, 0.21754E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.21805E+00_JPRB, 0.21161E+00_JPRB, 0.20706E+00_JPRB, 0.20476E+00_JPRB, 0.20331E+00_JPRB, 0.20323E+00_JPRB, 0.20286E+00_JPRB ,&
&  0.20286E+00_JPRB, 0.20275E+00_JPRB, 0.20251E+00_JPRB, 0.20228E+00_JPRB, 0.20221E+00_JPRB, 0.20239E+00_JPRB, 0.20316E+00_JPRB ,&
&  0.20433E+00_JPRB, 0.20663E+00_JPRB, 0.20957E+00_JPRB, 0.21230E+00_JPRB, 0.21643E+00_JPRB, 0.22041E+00_JPRB, 0.22422E+00_JPRB ,&
&  0.22631E+00_JPRB, 0.22864E+00_JPRB, 0.23050E+00_JPRB, 0.23070E+00_JPRB, 0.22706E+00_JPRB, 0.21950E+00_JPRB, 0.22008E+00_JPRB ,&
&  0.21997E+00_JPRB, 0.21890E+00_JPRB, 0.21536E+00_JPRB, 0.21743E+00_JPRB, 0.21876E+00_JPRB, 0.19000E+00_JPRB, 0.18210E+00_JPRB ,&
&  0.16709E+00_JPRB, 0.15198E+00_JPRB, 0.14624E+00_JPRB, 0.14239E+00_JPRB, 0.14289E+00_JPRB, 0.14780E+00_JPRB, 0.15616E+00_JPRB ,&
&  0.17417E+00_JPRB, 0.22538E+00_JPRB, 0.23626E+00_JPRB, 0.23824E+00_JPRB, 0.23392E+00_JPRB, 0.23257E+00_JPRB, 0.23217E+00_JPRB ,&
&  0.23288E+00_JPRB, 0.23058E+00_JPRB, 0.22686E+00_JPRB, 0.22292E+00_JPRB, 0.22113E+00_JPRB, 0.21792E+00_JPRB, 0.21741E+00_JPRB ,&
&  0.22087E+00_JPRB, 0.22735E+00_JPRB, 0.23069E+00_JPRB, 0.23750E+00_JPRB, 0.25041E+00_JPRB, 0.25767E+00_JPRB, 0.26472E+00_JPRB ,&
&  0.25944E+00_JPRB, 0.25350E+00_JPRB, 0.24865E+00_JPRB, 0.24386E+00_JPRB, 0.24041E+00_JPRB, 0.23755E+00_JPRB, 0.23508E+00_JPRB ,&
&  0.23463E+00_JPRB, 0.23798E+00_JPRB, 0.24022E+00_JPRB, 0.24131E+00_JPRB, 0.24322E+00_JPRB, 0.24228E+00_JPRB, 0.24466E+00_JPRB ,&
&  0.24810E+00_JPRB, 0.25249E+00_JPRB, 0.25668E+00_JPRB, 0.26065E+00_JPRB, 0.26352E+00_JPRB, 0.26340E+00_JPRB, 0.26442E+00_JPRB ,&
&  0.26542E+00_JPRB, 0.26748E+00_JPRB, 0.27354E+00_JPRB, 0.28123E+00_JPRB, 0.28750E+00_JPRB, 0.29177E+00_JPRB, 0.29432E+00_JPRB ,&
&  0.28676E+00_JPRB, 0.27051E+00_JPRB, 0.25448E+00_JPRB, 0.23805E+00_JPRB, 0.22618E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.23292E+00_JPRB, 0.22338E+00_JPRB, 0.21604E+00_JPRB, 0.21104E+00_JPRB, 0.20829E+00_JPRB, 0.20713E+00_JPRB, 0.20646E+00_JPRB ,&
&  0.20645E+00_JPRB, 0.20603E+00_JPRB, 0.20596E+00_JPRB, 0.20565E+00_JPRB, 0.20569E+00_JPRB, 0.20612E+00_JPRB, 0.20744E+00_JPRB ,&
&  0.20961E+00_JPRB, 0.21223E+00_JPRB, 0.21601E+00_JPRB, 0.21897E+00_JPRB, 0.22262E+00_JPRB, 0.22534E+00_JPRB, 0.22619E+00_JPRB ,&
&  0.22829E+00_JPRB, 0.23053E+00_JPRB, 0.23191E+00_JPRB, 0.23337E+00_JPRB, 0.22518E+00_JPRB, 0.21832E+00_JPRB, 0.21383E+00_JPRB ,&
&  0.21612E+00_JPRB, 0.20253E+00_JPRB, 0.20386E+00_JPRB, 0.19417E+00_JPRB, 0.18429E+00_JPRB, 0.17476E+00_JPRB, 0.16587E+00_JPRB ,&
&  0.16022E+00_JPRB, 0.15215E+00_JPRB, 0.14858E+00_JPRB, 0.14763E+00_JPRB, 0.14817E+00_JPRB, 0.15307E+00_JPRB, 0.15997E+00_JPRB ,&
&  0.17108E+00_JPRB, 0.20438E+00_JPRB, 0.21414E+00_JPRB, 0.21670E+00_JPRB, 0.21463E+00_JPRB, 0.21389E+00_JPRB, 0.21284E+00_JPRB ,&
&  0.21593E+00_JPRB, 0.21938E+00_JPRB, 0.21870E+00_JPRB, 0.21735E+00_JPRB, 0.21929E+00_JPRB, 0.21939E+00_JPRB, 0.21837E+00_JPRB ,&
&  0.21785E+00_JPRB, 0.21836E+00_JPRB, 0.21890E+00_JPRB, 0.21811E+00_JPRB, 0.21807E+00_JPRB, 0.21649E+00_JPRB, 0.21772E+00_JPRB ,&
&  0.21761E+00_JPRB, 0.22198E+00_JPRB, 0.22392E+00_JPRB, 0.22738E+00_JPRB, 0.22903E+00_JPRB, 0.22916E+00_JPRB, 0.23059E+00_JPRB ,&
&  0.23243E+00_JPRB, 0.23634E+00_JPRB, 0.23924E+00_JPRB, 0.24410E+00_JPRB, 0.24746E+00_JPRB, 0.24858E+00_JPRB, 0.25053E+00_JPRB ,&
&  0.25423E+00_JPRB, 0.25719E+00_JPRB, 0.25996E+00_JPRB, 0.26207E+00_JPRB, 0.26266E+00_JPRB, 0.26248E+00_JPRB, 0.26497E+00_JPRB ,&
&  0.26836E+00_JPRB, 0.27445E+00_JPRB, 0.28221E+00_JPRB, 0.28667E+00_JPRB, 0.28974E+00_JPRB, 0.29069E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28140E+00_JPRB, 0.27357E+00_JPRB, 0.26415E+00_JPRB, 0.25436E+00_JPRB, 0.24379E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.24627E+00_JPRB, 0.23902E+00_JPRB, 0.23096E+00_JPRB, 0.22472E+00_JPRB, 0.22057E+00_JPRB, 0.21821E+00_JPRB, 0.21649E+00_JPRB ,&
&  0.21536E+00_JPRB, 0.21452E+00_JPRB, 0.21417E+00_JPRB, 0.21387E+00_JPRB, 0.21382E+00_JPRB, 0.21389E+00_JPRB, 0.21486E+00_JPRB ,&
&  0.21678E+00_JPRB, 0.21943E+00_JPRB, 0.22318E+00_JPRB, 0.22686E+00_JPRB, 0.22994E+00_JPRB, 0.23099E+00_JPRB, 0.22866E+00_JPRB ,&
&  0.23070E+00_JPRB, 0.23267E+00_JPRB, 0.22839E+00_JPRB, 0.23224E+00_JPRB, 0.22393E+00_JPRB, 0.21723E+00_JPRB, 0.21429E+00_JPRB ,&
&  0.20867E+00_JPRB, 0.20716E+00_JPRB, 0.20209E+00_JPRB, 0.19691E+00_JPRB, 0.19336E+00_JPRB, 0.18503E+00_JPRB, 0.17947E+00_JPRB ,&
&  0.17332E+00_JPRB, 0.16570E+00_JPRB, 0.16173E+00_JPRB, 0.15916E+00_JPRB, 0.16020E+00_JPRB, 0.16402E+00_JPRB, 0.16983E+00_JPRB ,&
&  0.17671E+00_JPRB, 0.18628E+00_JPRB, 0.20139E+00_JPRB, 0.20653E+00_JPRB, 0.20831E+00_JPRB, 0.21067E+00_JPRB, 0.21166E+00_JPRB ,&
&  0.21111E+00_JPRB, 0.21440E+00_JPRB, 0.21466E+00_JPRB, 0.21405E+00_JPRB, 0.21282E+00_JPRB, 0.21111E+00_JPRB, 0.20802E+00_JPRB ,&
&  0.20585E+00_JPRB, 0.20483E+00_JPRB, 0.20305E+00_JPRB, 0.20057E+00_JPRB, 0.19857E+00_JPRB, 0.19688E+00_JPRB, 0.19797E+00_JPRB ,&
&  0.19980E+00_JPRB, 0.20468E+00_JPRB, 0.20903E+00_JPRB, 0.21606E+00_JPRB, 0.22033E+00_JPRB, 0.22486E+00_JPRB, 0.22750E+00_JPRB ,&
&  0.22993E+00_JPRB, 0.23292E+00_JPRB, 0.23527E+00_JPRB, 0.23544E+00_JPRB, 0.23614E+00_JPRB, 0.23920E+00_JPRB, 0.24056E+00_JPRB ,&
&  0.24296E+00_JPRB, 0.24554E+00_JPRB, 0.24776E+00_JPRB, 0.25014E+00_JPRB, 0.25273E+00_JPRB, 0.25690E+00_JPRB, 0.26163E+00_JPRB ,&
&  0.26958E+00_JPRB, 0.27585E+00_JPRB, 0.28146E+00_JPRB, 0.28234E+00_JPRB, 0.28232E+00_JPRB, 0.27958E+00_JPRB, 0.27525E+00_JPRB ,&
&  0.27086E+00_JPRB, 0.26705E+00_JPRB, 0.26194E+00_JPRB, 0.25765E+00_JPRB, 0.25244E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.23823E+00_JPRB, 0.23508E+00_JPRB, 0.23157E+00_JPRB, 0.22855E+00_JPRB, 0.22664E+00_JPRB, 0.22574E+00_JPRB, 0.22479E+00_JPRB ,&
&  0.22429E+00_JPRB, 0.22383E+00_JPRB, 0.22416E+00_JPRB, 0.22497E+00_JPRB, 0.22577E+00_JPRB, 0.22741E+00_JPRB, 0.22918E+00_JPRB ,&
&  0.23080E+00_JPRB, 0.23225E+00_JPRB, 0.23430E+00_JPRB, 0.23604E+00_JPRB, 0.23717E+00_JPRB, 0.23720E+00_JPRB, 0.23546E+00_JPRB ,&
&  0.23520E+00_JPRB, 0.23440E+00_JPRB, 0.23384E+00_JPRB, 0.23397E+00_JPRB, 0.23356E+00_JPRB, 0.21999E+00_JPRB, 0.21794E+00_JPRB ,&
&  0.22012E+00_JPRB, 0.21849E+00_JPRB, 0.21757E+00_JPRB, 0.21716E+00_JPRB, 0.21842E+00_JPRB, 0.21578E+00_JPRB, 0.21301E+00_JPRB ,&
&  0.19892E+00_JPRB, 0.19571E+00_JPRB, 0.19115E+00_JPRB, 0.18639E+00_JPRB, 0.18389E+00_JPRB, 0.18280E+00_JPRB, 0.18380E+00_JPRB ,&
&  0.19083E+00_JPRB, 0.19225E+00_JPRB, 0.19564E+00_JPRB, 0.20012E+00_JPRB, 0.20378E+00_JPRB, 0.20565E+00_JPRB, 0.20750E+00_JPRB ,&
&  0.20794E+00_JPRB, 0.20811E+00_JPRB, 0.20774E+00_JPRB, 0.20642E+00_JPRB, 0.20458E+00_JPRB, 0.20279E+00_JPRB, 0.20094E+00_JPRB ,&
&  0.19928E+00_JPRB, 0.19742E+00_JPRB, 0.19589E+00_JPRB, 0.19427E+00_JPRB, 0.19268E+00_JPRB, 0.19187E+00_JPRB, 0.19270E+00_JPRB ,&
&  0.19444E+00_JPRB, 0.19747E+00_JPRB, 0.20074E+00_JPRB, 0.20560E+00_JPRB, 0.21029E+00_JPRB, 0.21511E+00_JPRB, 0.21762E+00_JPRB ,&
&  0.22030E+00_JPRB, 0.22265E+00_JPRB, 0.22519E+00_JPRB, 0.22645E+00_JPRB, 0.22764E+00_JPRB, 0.22918E+00_JPRB, 0.23153E+00_JPRB ,&
&  0.23358E+00_JPRB, 0.23591E+00_JPRB, 0.23835E+00_JPRB, 0.24212E+00_JPRB, 0.24500E+00_JPRB, 0.24938E+00_JPRB, 0.25332E+00_JPRB ,&
&  0.25928E+00_JPRB, 0.26106E+00_JPRB, 0.26316E+00_JPRB, 0.26193E+00_JPRB, 0.26084E+00_JPRB, 0.25715E+00_JPRB, 0.25368E+00_JPRB ,&
&  0.25062E+00_JPRB, 0.24791E+00_JPRB, 0.24561E+00_JPRB, 0.24322E+00_JPRB, 0.24088E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.22700E+00_JPRB, 0.22550E+00_JPRB, 0.22452E+00_JPRB, 0.22426E+00_JPRB, 0.22407E+00_JPRB, 0.22420E+00_JPRB, 0.22450E+00_JPRB ,&
&  0.22488E+00_JPRB, 0.22519E+00_JPRB, 0.22550E+00_JPRB, 0.22615E+00_JPRB, 0.22706E+00_JPRB, 0.22813E+00_JPRB, 0.22900E+00_JPRB ,&
&  0.22991E+00_JPRB, 0.23061E+00_JPRB, 0.23121E+00_JPRB, 0.23115E+00_JPRB, 0.23109E+00_JPRB, 0.23063E+00_JPRB, 0.23013E+00_JPRB ,&
&  0.22898E+00_JPRB, 0.22766E+00_JPRB, 0.22651E+00_JPRB, 0.22557E+00_JPRB, 0.22440E+00_JPRB, 0.22294E+00_JPRB, 0.22203E+00_JPRB ,&
&  0.22152E+00_JPRB, 0.22041E+00_JPRB, 0.21947E+00_JPRB, 0.21856E+00_JPRB, 0.21696E+00_JPRB, 0.21448E+00_JPRB, 0.21219E+00_JPRB ,&
&  0.20936E+00_JPRB, 0.20600E+00_JPRB, 0.20337E+00_JPRB, 0.20091E+00_JPRB, 0.19872E+00_JPRB, 0.19685E+00_JPRB, 0.19549E+00_JPRB ,&
&  0.19595E+00_JPRB, 0.19747E+00_JPRB, 0.19949E+00_JPRB, 0.20062E+00_JPRB, 0.20144E+00_JPRB, 0.20186E+00_JPRB, 0.20228E+00_JPRB ,&
&  0.20206E+00_JPRB, 0.20174E+00_JPRB, 0.20133E+00_JPRB, 0.20083E+00_JPRB, 0.20007E+00_JPRB, 0.19970E+00_JPRB, 0.19952E+00_JPRB ,&
&  0.19935E+00_JPRB, 0.19804E+00_JPRB, 0.19685E+00_JPRB, 0.19574E+00_JPRB, 0.19490E+00_JPRB, 0.19468E+00_JPRB, 0.19516E+00_JPRB ,&
&  0.19622E+00_JPRB, 0.19805E+00_JPRB, 0.20031E+00_JPRB, 0.20309E+00_JPRB, 0.20633E+00_JPRB, 0.20968E+00_JPRB, 0.21261E+00_JPRB ,&
&  0.21556E+00_JPRB, 0.21862E+00_JPRB, 0.22158E+00_JPRB, 0.22384E+00_JPRB, 0.22616E+00_JPRB, 0.22846E+00_JPRB, 0.23093E+00_JPRB ,&
&  0.23270E+00_JPRB, 0.23485E+00_JPRB, 0.23715E+00_JPRB, 0.23987E+00_JPRB, 0.24134E+00_JPRB, 0.24314E+00_JPRB, 0.24470E+00_JPRB ,&
&  0.24667E+00_JPRB, 0.24581E+00_JPRB, 0.24533E+00_JPRB, 0.24403E+00_JPRB, 0.24268E+00_JPRB, 0.23983E+00_JPRB, 0.23737E+00_JPRB ,&
&  0.23531E+00_JPRB, 0.23360E+00_JPRB, 0.23174E+00_JPRB, 0.22997E+00_JPRB, 0.22861E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.21802E+00_JPRB, 0.21748E+00_JPRB, 0.21697E+00_JPRB, 0.21654E+00_JPRB, 0.21625E+00_JPRB, 0.21615E+00_JPRB, 0.21616E+00_JPRB ,&
&  0.21616E+00_JPRB, 0.21607E+00_JPRB, 0.21603E+00_JPRB, 0.21610E+00_JPRB, 0.21623E+00_JPRB, 0.21630E+00_JPRB, 0.21629E+00_JPRB ,&
&  0.21622E+00_JPRB, 0.21625E+00_JPRB, 0.21617E+00_JPRB, 0.21586E+00_JPRB, 0.21554E+00_JPRB, 0.21521E+00_JPRB, 0.21474E+00_JPRB ,&
&  0.21415E+00_JPRB, 0.21351E+00_JPRB, 0.21289E+00_JPRB, 0.21199E+00_JPRB, 0.21095E+00_JPRB, 0.20971E+00_JPRB, 0.20871E+00_JPRB ,&
&  0.20777E+00_JPRB, 0.20676E+00_JPRB, 0.20571E+00_JPRB, 0.20478E+00_JPRB, 0.20382E+00_JPRB, 0.20285E+00_JPRB, 0.20180E+00_JPRB ,&
&  0.20090E+00_JPRB, 0.20002E+00_JPRB, 0.19935E+00_JPRB, 0.19879E+00_JPRB, 0.19838E+00_JPRB, 0.19819E+00_JPRB, 0.19817E+00_JPRB ,&
&  0.19807E+00_JPRB, 0.19802E+00_JPRB, 0.19814E+00_JPRB, 0.19834E+00_JPRB, 0.19861E+00_JPRB, 0.19895E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19957E+00_JPRB, 0.19994E+00_JPRB, 0.20047E+00_JPRB, 0.20100E+00_JPRB, 0.20140E+00_JPRB, 0.20184E+00_JPRB, 0.20225E+00_JPRB ,&
&  0.20254E+00_JPRB, 0.20263E+00_JPRB, 0.20273E+00_JPRB, 0.20283E+00_JPRB, 0.20298E+00_JPRB, 0.20312E+00_JPRB, 0.20346E+00_JPRB ,&
&  0.20385E+00_JPRB, 0.20434E+00_JPRB, 0.20511E+00_JPRB, 0.20611E+00_JPRB, 0.20728E+00_JPRB, 0.20869E+00_JPRB, 0.21006E+00_JPRB ,&
&  0.21150E+00_JPRB, 0.21300E+00_JPRB, 0.21455E+00_JPRB, 0.21607E+00_JPRB, 0.21765E+00_JPRB, 0.21913E+00_JPRB, 0.22047E+00_JPRB ,&
&  0.22144E+00_JPRB, 0.22254E+00_JPRB, 0.22370E+00_JPRB, 0.22466E+00_JPRB, 0.22481E+00_JPRB, 0.22507E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22562E+00_JPRB, 0.22512E+00_JPRB, 0.22489E+00_JPRB, 0.22449E+00_JPRB, 0.22397E+00_JPRB, 0.22298E+00_JPRB, 0.22218E+00_JPRB ,&
&  0.22138E+00_JPRB, 0.22068E+00_JPRB, 0.21990E+00_JPRB, 0.21915E+00_JPRB, 0.21865E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB, 0.20700E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB, 0.22477E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.24588E+00_JPRB, 0.24842E+00_JPRB, 0.25067E+00_JPRB, 0.25241E+00_JPRB, 0.25305E+00_JPRB, 0.25502E+00_JPRB, 0.25568E+00_JPRB ,&
&  0.25615E+00_JPRB, 0.25623E+00_JPRB, 0.25702E+00_JPRB, 0.25784E+00_JPRB, 0.25840E+00_JPRB, 0.25867E+00_JPRB, 0.25871E+00_JPRB ,&
&  0.25829E+00_JPRB, 0.25741E+00_JPRB, 0.25418E+00_JPRB, 0.25239E+00_JPRB, 0.25140E+00_JPRB, 0.25103E+00_JPRB, 0.25088E+00_JPRB ,&
&  0.25033E+00_JPRB, 0.24961E+00_JPRB, 0.24835E+00_JPRB, 0.24811E+00_JPRB, 0.24730E+00_JPRB, 0.24624E+00_JPRB, 0.24479E+00_JPRB ,&
&  0.24336E+00_JPRB, 0.24221E+00_JPRB, 0.24075E+00_JPRB, 0.23926E+00_JPRB, 0.23740E+00_JPRB, 0.23520E+00_JPRB, 0.23253E+00_JPRB ,&
&  0.22975E+00_JPRB, 0.22689E+00_JPRB, 0.22454E+00_JPRB, 0.22235E+00_JPRB, 0.22047E+00_JPRB, 0.21898E+00_JPRB, 0.21797E+00_JPRB ,&
&  0.21686E+00_JPRB, 0.21561E+00_JPRB, 0.21435E+00_JPRB, 0.21330E+00_JPRB, 0.21226E+00_JPRB, 0.21137E+00_JPRB, 0.21096E+00_JPRB ,&
&  0.21071E+00_JPRB, 0.21042E+00_JPRB, 0.21005E+00_JPRB, 0.20975E+00_JPRB, 0.20934E+00_JPRB, 0.20896E+00_JPRB, 0.20858E+00_JPRB ,&
&  0.20827E+00_JPRB, 0.20806E+00_JPRB, 0.20773E+00_JPRB, 0.20738E+00_JPRB, 0.20717E+00_JPRB, 0.20680E+00_JPRB, 0.20639E+00_JPRB ,&
&  0.20589E+00_JPRB, 0.20557E+00_JPRB, 0.20522E+00_JPRB, 0.20480E+00_JPRB, 0.20440E+00_JPRB, 0.20413E+00_JPRB, 0.20412E+00_JPRB ,&
&  0.20407E+00_JPRB, 0.20396E+00_JPRB, 0.20400E+00_JPRB, 0.20467E+00_JPRB, 0.20537E+00_JPRB, 0.20622E+00_JPRB, 0.20718E+00_JPRB ,&
&  0.20845E+00_JPRB, 0.20976E+00_JPRB, 0.21122E+00_JPRB, 0.21262E+00_JPRB, 0.21445E+00_JPRB, 0.21627E+00_JPRB, 0.21808E+00_JPRB ,&
&  0.21980E+00_JPRB, 0.22156E+00_JPRB, 0.22330E+00_JPRB, 0.22502E+00_JPRB, 0.22723E+00_JPRB, 0.22924E+00_JPRB, 0.23153E+00_JPRB ,&
&  0.23371E+00_JPRB, 0.23605E+00_JPRB, 0.23852E+00_JPRB, 0.24067E+00_JPRB, 0.24294E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.28869E+00_JPRB, 0.29925E+00_JPRB, 0.30836E+00_JPRB, 0.31730E+00_JPRB, 0.32233E+00_JPRB, 0.38697E+00_JPRB, 0.36790E+00_JPRB ,&
&  0.35864E+00_JPRB, 0.33006E+00_JPRB, 0.33055E+00_JPRB, 0.32586E+00_JPRB, 0.31912E+00_JPRB, 0.31123E+00_JPRB, 0.30457E+00_JPRB ,&
&  0.29805E+00_JPRB, 0.29017E+00_JPRB, 0.28496E+00_JPRB, 0.27878E+00_JPRB, 0.27294E+00_JPRB, 0.26721E+00_JPRB, 0.26382E+00_JPRB ,&
&  0.26130E+00_JPRB, 0.25982E+00_JPRB, 0.25870E+00_JPRB, 0.25832E+00_JPRB, 0.26112E+00_JPRB, 0.26268E+00_JPRB, 0.26111E+00_JPRB ,&
&  0.25785E+00_JPRB, 0.25469E+00_JPRB, 0.25059E+00_JPRB, 0.24637E+00_JPRB, 0.24302E+00_JPRB, 0.24012E+00_JPRB, 0.23714E+00_JPRB ,&
&  0.23330E+00_JPRB, 0.22903E+00_JPRB, 0.22483E+00_JPRB, 0.22158E+00_JPRB, 0.21860E+00_JPRB, 0.21561E+00_JPRB, 0.21287E+00_JPRB ,&
&  0.21009E+00_JPRB, 0.20734E+00_JPRB, 0.20460E+00_JPRB, 0.20276E+00_JPRB, 0.20124E+00_JPRB, 0.19979E+00_JPRB, 0.19850E+00_JPRB ,&
&  0.19790E+00_JPRB, 0.19720E+00_JPRB, 0.19634E+00_JPRB, 0.19528E+00_JPRB, 0.19424E+00_JPRB, 0.19311E+00_JPRB, 0.19218E+00_JPRB ,&
&  0.19200E+00_JPRB, 0.19143E+00_JPRB, 0.19084E+00_JPRB, 0.19008E+00_JPRB, 0.18970E+00_JPRB, 0.18897E+00_JPRB, 0.18805E+00_JPRB ,&
&  0.18720E+00_JPRB, 0.18642E+00_JPRB, 0.18620E+00_JPRB, 0.18591E+00_JPRB, 0.18560E+00_JPRB, 0.18539E+00_JPRB, 0.18574E+00_JPRB ,&
&  0.18618E+00_JPRB, 0.18670E+00_JPRB, 0.18759E+00_JPRB, 0.18870E+00_JPRB, 0.18967E+00_JPRB, 0.19062E+00_JPRB, 0.19195E+00_JPRB ,&
&  0.19380E+00_JPRB, 0.19566E+00_JPRB, 0.19771E+00_JPRB, 0.19969E+00_JPRB, 0.20292E+00_JPRB, 0.20637E+00_JPRB, 0.21015E+00_JPRB ,&
&  0.21481E+00_JPRB, 0.22114E+00_JPRB, 0.22829E+00_JPRB, 0.23462E+00_JPRB, 0.24023E+00_JPRB, 0.24705E+00_JPRB, 0.25266E+00_JPRB ,&
&  0.25765E+00_JPRB, 0.26251E+00_JPRB, 0.26860E+00_JPRB, 0.27506E+00_JPRB, 0.28211E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.41558E+00_JPRB, 0.41612E+00_JPRB, 0.41622E+00_JPRB, 0.41244E+00_JPRB, 0.41072E+00_JPRB, 0.41057E+00_JPRB, 0.40776E+00_JPRB ,&
&  0.39994E+00_JPRB, 0.39734E+00_JPRB, 0.39257E+00_JPRB, 0.38583E+00_JPRB, 0.37933E+00_JPRB, 0.36893E+00_JPRB, 0.35281E+00_JPRB ,&
&  0.33109E+00_JPRB, 0.31211E+00_JPRB, 0.29761E+00_JPRB, 0.28451E+00_JPRB, 0.27164E+00_JPRB, 0.25908E+00_JPRB, 0.25359E+00_JPRB ,&
&  0.24901E+00_JPRB, 0.24698E+00_JPRB, 0.24622E+00_JPRB, 0.24988E+00_JPRB, 0.25749E+00_JPRB, 0.26703E+00_JPRB, 0.27739E+00_JPRB ,&
&  0.28948E+00_JPRB, 0.30067E+00_JPRB, 0.30220E+00_JPRB, 0.29532E+00_JPRB, 0.28840E+00_JPRB, 0.28401E+00_JPRB, 0.28066E+00_JPRB ,&
&  0.27677E+00_JPRB, 0.26837E+00_JPRB, 0.25903E+00_JPRB, 0.24933E+00_JPRB, 0.23957E+00_JPRB, 0.23243E+00_JPRB, 0.22404E+00_JPRB ,&
&  0.21794E+00_JPRB, 0.21249E+00_JPRB, 0.20686E+00_JPRB, 0.20205E+00_JPRB, 0.19879E+00_JPRB, 0.19603E+00_JPRB, 0.19331E+00_JPRB ,&
&  0.19181E+00_JPRB, 0.19033E+00_JPRB, 0.18865E+00_JPRB, 0.18695E+00_JPRB, 0.18492E+00_JPRB, 0.18331E+00_JPRB, 0.18183E+00_JPRB ,&
&  0.18068E+00_JPRB, 0.17971E+00_JPRB, 0.17911E+00_JPRB, 0.17850E+00_JPRB, 0.17804E+00_JPRB, 0.17718E+00_JPRB, 0.17614E+00_JPRB ,&
&  0.17541E+00_JPRB, 0.17497E+00_JPRB, 0.17420E+00_JPRB, 0.17373E+00_JPRB, 0.17336E+00_JPRB, 0.17267E+00_JPRB, 0.17303E+00_JPRB ,&
&  0.17401E+00_JPRB, 0.17514E+00_JPRB, 0.17552E+00_JPRB, 0.17659E+00_JPRB, 0.17788E+00_JPRB, 0.17949E+00_JPRB, 0.18189E+00_JPRB ,&
&  0.18378E+00_JPRB, 0.18574E+00_JPRB, 0.18815E+00_JPRB, 0.19033E+00_JPRB, 0.19359E+00_JPRB, 0.19761E+00_JPRB, 0.20254E+00_JPRB ,&
&  0.20925E+00_JPRB, 0.21733E+00_JPRB, 0.22840E+00_JPRB, 0.24183E+00_JPRB, 0.26069E+00_JPRB, 0.26926E+00_JPRB, 0.27790E+00_JPRB ,&
&  0.28574E+00_JPRB, 0.30138E+00_JPRB, 0.38335E+00_JPRB, 0.39852E+00_JPRB, 0.41326E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.41331E+00_JPRB, 0.42541E+00_JPRB, 0.44310E+00_JPRB, 0.45396E+00_JPRB, 0.46853E+00_JPRB, 0.47145E+00_JPRB, 0.46681E+00_JPRB ,&
&  0.44751E+00_JPRB, 0.41700E+00_JPRB, 0.40027E+00_JPRB, 0.38014E+00_JPRB, 0.35894E+00_JPRB, 0.33780E+00_JPRB, 0.32850E+00_JPRB ,&
&  0.31737E+00_JPRB, 0.30510E+00_JPRB, 0.29238E+00_JPRB, 0.28160E+00_JPRB, 0.27126E+00_JPRB, 0.26155E+00_JPRB, 0.25574E+00_JPRB ,&
&  0.24978E+00_JPRB, 0.24894E+00_JPRB, 0.24824E+00_JPRB, 0.25247E+00_JPRB, 0.26348E+00_JPRB, 0.28258E+00_JPRB, 0.30841E+00_JPRB ,&
&  0.34755E+00_JPRB, 0.35443E+00_JPRB, 0.33943E+00_JPRB, 0.33301E+00_JPRB, 0.32576E+00_JPRB, 0.31925E+00_JPRB, 0.30441E+00_JPRB ,&
&  0.31164E+00_JPRB, 0.32546E+00_JPRB, 0.34142E+00_JPRB, 0.33200E+00_JPRB, 0.28720E+00_JPRB, 0.26653E+00_JPRB, 0.24312E+00_JPRB ,&
&  0.23011E+00_JPRB, 0.21977E+00_JPRB, 0.21184E+00_JPRB, 0.20464E+00_JPRB, 0.19787E+00_JPRB, 0.19349E+00_JPRB, 0.18868E+00_JPRB ,&
&  0.18500E+00_JPRB, 0.18325E+00_JPRB, 0.18083E+00_JPRB, 0.18050E+00_JPRB, 0.18014E+00_JPRB, 0.17926E+00_JPRB, 0.17829E+00_JPRB ,&
&  0.17684E+00_JPRB, 0.17531E+00_JPRB, 0.17449E+00_JPRB, 0.17487E+00_JPRB, 0.17576E+00_JPRB, 0.17665E+00_JPRB, 0.17507E+00_JPRB ,&
&  0.17379E+00_JPRB, 0.17250E+00_JPRB, 0.17229E+00_JPRB, 0.17313E+00_JPRB, 0.17335E+00_JPRB, 0.17392E+00_JPRB, 0.17393E+00_JPRB ,&
&  0.17394E+00_JPRB, 0.17437E+00_JPRB, 0.17408E+00_JPRB, 0.17470E+00_JPRB, 0.17501E+00_JPRB, 0.17608E+00_JPRB, 0.17695E+00_JPRB ,&
&  0.17787E+00_JPRB, 0.17974E+00_JPRB, 0.18173E+00_JPRB, 0.18445E+00_JPRB, 0.18817E+00_JPRB, 0.19095E+00_JPRB, 0.19524E+00_JPRB ,&
&  0.20061E+00_JPRB, 0.20900E+00_JPRB, 0.22413E+00_JPRB, 0.24115E+00_JPRB, 0.26189E+00_JPRB, 0.27373E+00_JPRB, 0.28364E+00_JPRB ,&
&  0.29877E+00_JPRB, 0.33436E+00_JPRB, 0.38471E+00_JPRB, 0.40318E+00_JPRB, 0.40690E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.46999E+00_JPRB, 0.47747E+00_JPRB, 0.48897E+00_JPRB, 0.49375E+00_JPRB, 0.50210E+00_JPRB, 0.49049E+00_JPRB, 0.48373E+00_JPRB ,&
&  0.45363E+00_JPRB, 0.43068E+00_JPRB, 0.39938E+00_JPRB, 0.35956E+00_JPRB, 0.33306E+00_JPRB, 0.31246E+00_JPRB, 0.30577E+00_JPRB ,&
&  0.29954E+00_JPRB, 0.29540E+00_JPRB, 0.29181E+00_JPRB, 0.29052E+00_JPRB, 0.29020E+00_JPRB, 0.29169E+00_JPRB, 0.29194E+00_JPRB ,&
&  0.28865E+00_JPRB, 0.28304E+00_JPRB, 0.27658E+00_JPRB, 0.27989E+00_JPRB, 0.29281E+00_JPRB, 0.30794E+00_JPRB, 0.32553E+00_JPRB ,&
&  0.34566E+00_JPRB, 0.35673E+00_JPRB, 0.36690E+00_JPRB, 0.37115E+00_JPRB, 0.35589E+00_JPRB, 0.35841E+00_JPRB, 0.36067E+00_JPRB ,&
&  0.36877E+00_JPRB, 0.37646E+00_JPRB, 0.37405E+00_JPRB, 0.37067E+00_JPRB, 0.32066E+00_JPRB, 0.28714E+00_JPRB, 0.25867E+00_JPRB ,&
&  0.23722E+00_JPRB, 0.22333E+00_JPRB, 0.21240E+00_JPRB, 0.20321E+00_JPRB, 0.19446E+00_JPRB, 0.18868E+00_JPRB, 0.18431E+00_JPRB ,&
&  0.18069E+00_JPRB, 0.17894E+00_JPRB, 0.17722E+00_JPRB, 0.17656E+00_JPRB, 0.17660E+00_JPRB, 0.17589E+00_JPRB, 0.17554E+00_JPRB ,&
&  0.17502E+00_JPRB, 0.17402E+00_JPRB, 0.17363E+00_JPRB, 0.17436E+00_JPRB, 0.17601E+00_JPRB, 0.17889E+00_JPRB, 0.18007E+00_JPRB ,&
&  0.18081E+00_JPRB, 0.18115E+00_JPRB, 0.18113E+00_JPRB, 0.18276E+00_JPRB, 0.18358E+00_JPRB, 0.18376E+00_JPRB, 0.18217E+00_JPRB ,&
&  0.18056E+00_JPRB, 0.17937E+00_JPRB, 0.17904E+00_JPRB, 0.17935E+00_JPRB, 0.18046E+00_JPRB, 0.18235E+00_JPRB, 0.18287E+00_JPRB ,&
&  0.18368E+00_JPRB, 0.18345E+00_JPRB, 0.18330E+00_JPRB, 0.18476E+00_JPRB, 0.18652E+00_JPRB, 0.18851E+00_JPRB, 0.19127E+00_JPRB ,&
&  0.19317E+00_JPRB, 0.19973E+00_JPRB, 0.21159E+00_JPRB, 0.22953E+00_JPRB, 0.25571E+00_JPRB, 0.27188E+00_JPRB, 0.31100E+00_JPRB ,&
&  0.37720E+00_JPRB, 0.52404E+00_JPRB, 0.58804E+00_JPRB, 0.44121E+00_JPRB, 0.45495E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.49798E+00_JPRB, 0.50953E+00_JPRB, 0.52238E+00_JPRB, 0.51452E+00_JPRB, 0.50074E+00_JPRB, 0.49301E+00_JPRB, 0.49260E+00_JPRB ,&
&  0.49197E+00_JPRB, 0.48914E+00_JPRB, 0.47684E+00_JPRB, 0.46511E+00_JPRB, 0.44737E+00_JPRB, 0.39878E+00_JPRB, 0.37984E+00_JPRB ,&
&  0.36228E+00_JPRB, 0.36469E+00_JPRB, 0.36971E+00_JPRB, 0.37675E+00_JPRB, 0.39146E+00_JPRB, 0.40274E+00_JPRB, 0.40545E+00_JPRB ,&
&  0.39612E+00_JPRB, 0.37811E+00_JPRB, 0.36106E+00_JPRB, 0.34752E+00_JPRB, 0.34719E+00_JPRB, 0.34764E+00_JPRB, 0.34108E+00_JPRB ,&
&  0.32749E+00_JPRB, 0.32109E+00_JPRB, 0.32056E+00_JPRB, 0.34071E+00_JPRB, 0.39456E+00_JPRB, 0.41516E+00_JPRB, 0.38915E+00_JPRB ,&
&  0.39340E+00_JPRB, 0.38888E+00_JPRB, 0.38513E+00_JPRB, 0.38693E+00_JPRB, 0.37743E+00_JPRB, 0.36023E+00_JPRB, 0.32654E+00_JPRB ,&
&  0.28174E+00_JPRB, 0.25847E+00_JPRB, 0.23752E+00_JPRB, 0.21852E+00_JPRB, 0.20462E+00_JPRB, 0.19637E+00_JPRB, 0.18930E+00_JPRB ,&
&  0.18466E+00_JPRB, 0.18085E+00_JPRB, 0.17825E+00_JPRB, 0.17782E+00_JPRB, 0.17875E+00_JPRB, 0.17908E+00_JPRB, 0.17959E+00_JPRB ,&
&  0.18052E+00_JPRB, 0.18086E+00_JPRB, 0.18093E+00_JPRB, 0.18173E+00_JPRB, 0.18284E+00_JPRB, 0.18503E+00_JPRB, 0.18832E+00_JPRB ,&
&  0.19237E+00_JPRB, 0.19604E+00_JPRB, 0.19966E+00_JPRB, 0.20521E+00_JPRB, 0.20568E+00_JPRB, 0.20466E+00_JPRB, 0.19719E+00_JPRB ,&
&  0.19185E+00_JPRB, 0.18935E+00_JPRB, 0.18686E+00_JPRB, 0.18611E+00_JPRB, 0.18700E+00_JPRB, 0.18862E+00_JPRB, 0.19023E+00_JPRB ,&
&  0.19131E+00_JPRB, 0.19136E+00_JPRB, 0.19071E+00_JPRB, 0.19077E+00_JPRB, 0.19145E+00_JPRB, 0.19167E+00_JPRB, 0.19491E+00_JPRB ,&
&  0.20190E+00_JPRB, 0.21232E+00_JPRB, 0.22391E+00_JPRB, 0.24013E+00_JPRB, 0.25512E+00_JPRB, 0.27133E+00_JPRB, 0.29856E+00_JPRB ,&
&  0.33878E+00_JPRB, 0.44834E+00_JPRB, 0.51860E+00_JPRB, 0.45399E+00_JPRB, 0.47741E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.52819E+00_JPRB, 0.53203E+00_JPRB, 0.53660E+00_JPRB, 0.53926E+00_JPRB, 0.53403E+00_JPRB, 0.52696E+00_JPRB, 0.52237E+00_JPRB ,&
&  0.52748E+00_JPRB, 0.52724E+00_JPRB, 0.50363E+00_JPRB, 0.48497E+00_JPRB, 0.47417E+00_JPRB, 0.47061E+00_JPRB, 0.46739E+00_JPRB ,&
&  0.46136E+00_JPRB, 0.45943E+00_JPRB, 0.46094E+00_JPRB, 0.47257E+00_JPRB, 0.48220E+00_JPRB, 0.49059E+00_JPRB, 0.49813E+00_JPRB ,&
&  0.49422E+00_JPRB, 0.50772E+00_JPRB, 0.52967E+00_JPRB, 0.52929E+00_JPRB, 0.52692E+00_JPRB, 0.52007E+00_JPRB, 0.50597E+00_JPRB ,&
&  0.45882E+00_JPRB, 0.39494E+00_JPRB, 0.33181E+00_JPRB, 0.34256E+00_JPRB, 0.42185E+00_JPRB, 0.43276E+00_JPRB, 0.41615E+00_JPRB ,&
&  0.42630E+00_JPRB, 0.42272E+00_JPRB, 0.40507E+00_JPRB, 0.39231E+00_JPRB, 0.37704E+00_JPRB, 0.36707E+00_JPRB, 0.36030E+00_JPRB ,&
&  0.35760E+00_JPRB, 0.35594E+00_JPRB, 0.34207E+00_JPRB, 0.29486E+00_JPRB, 0.26634E+00_JPRB, 0.24931E+00_JPRB, 0.23196E+00_JPRB ,&
&  0.21855E+00_JPRB, 0.20967E+00_JPRB, 0.20782E+00_JPRB, 0.20797E+00_JPRB, 0.21197E+00_JPRB, 0.21327E+00_JPRB, 0.21449E+00_JPRB ,&
&  0.21381E+00_JPRB, 0.21247E+00_JPRB, 0.20837E+00_JPRB, 0.20379E+00_JPRB, 0.20066E+00_JPRB, 0.19999E+00_JPRB, 0.19977E+00_JPRB ,&
&  0.20224E+00_JPRB, 0.20626E+00_JPRB, 0.22217E+00_JPRB, 0.23954E+00_JPRB, 0.24016E+00_JPRB, 0.22496E+00_JPRB, 0.20970E+00_JPRB ,&
&  0.20045E+00_JPRB, 0.19565E+00_JPRB, 0.19269E+00_JPRB, 0.19214E+00_JPRB, 0.19461E+00_JPRB, 0.19804E+00_JPRB, 0.19899E+00_JPRB ,&
&  0.19895E+00_JPRB, 0.19973E+00_JPRB, 0.19994E+00_JPRB, 0.20055E+00_JPRB, 0.20133E+00_JPRB, 0.20373E+00_JPRB, 0.20818E+00_JPRB ,&
&  0.21348E+00_JPRB, 0.22298E+00_JPRB, 0.23385E+00_JPRB, 0.24850E+00_JPRB, 0.25912E+00_JPRB, 0.26668E+00_JPRB, 0.27565E+00_JPRB ,&
&  0.29250E+00_JPRB, 0.32984E+00_JPRB, 0.39863E+00_JPRB, 0.52563E+00_JPRB, 0.52140E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.58357E+00_JPRB, 0.57215E+00_JPRB, 0.56459E+00_JPRB, 0.55931E+00_JPRB, 0.55283E+00_JPRB, 0.55191E+00_JPRB, 0.55145E+00_JPRB ,&
&  0.54584E+00_JPRB, 0.53755E+00_JPRB, 0.53831E+00_JPRB, 0.52082E+00_JPRB, 0.49604E+00_JPRB, 0.47946E+00_JPRB, 0.47722E+00_JPRB ,&
&  0.48127E+00_JPRB, 0.48862E+00_JPRB, 0.49797E+00_JPRB, 0.50729E+00_JPRB, 0.51247E+00_JPRB, 0.52433E+00_JPRB, 0.54540E+00_JPRB ,&
&  0.56999E+00_JPRB, 0.58393E+00_JPRB, 0.58841E+00_JPRB, 0.58077E+00_JPRB, 0.56850E+00_JPRB, 0.55568E+00_JPRB, 0.54504E+00_JPRB ,&
&  0.53802E+00_JPRB, 0.43590E+00_JPRB, 0.36653E+00_JPRB, 0.37224E+00_JPRB, 0.44412E+00_JPRB, 0.42835E+00_JPRB, 0.42576E+00_JPRB ,&
&  0.43821E+00_JPRB, 0.43912E+00_JPRB, 0.42640E+00_JPRB, 0.40971E+00_JPRB, 0.39393E+00_JPRB, 0.38236E+00_JPRB, 0.37360E+00_JPRB ,&
&  0.36928E+00_JPRB, 0.37561E+00_JPRB, 0.38403E+00_JPRB, 0.39712E+00_JPRB, 0.40085E+00_JPRB, 0.38868E+00_JPRB, 0.37403E+00_JPRB ,&
&  0.35977E+00_JPRB, 0.35477E+00_JPRB, 0.35691E+00_JPRB, 0.36267E+00_JPRB, 0.37025E+00_JPRB, 0.37618E+00_JPRB, 0.37437E+00_JPRB ,&
&  0.34502E+00_JPRB, 0.32235E+00_JPRB, 0.29434E+00_JPRB, 0.26801E+00_JPRB, 0.24505E+00_JPRB, 0.22952E+00_JPRB, 0.22170E+00_JPRB ,&
&  0.21952E+00_JPRB, 0.22595E+00_JPRB, 0.24676E+00_JPRB, 0.29553E+00_JPRB, 0.36560E+00_JPRB, 0.28710E+00_JPRB, 0.24771E+00_JPRB ,&
&  0.22569E+00_JPRB, 0.21276E+00_JPRB, 0.20794E+00_JPRB, 0.20844E+00_JPRB, 0.21202E+00_JPRB, 0.21745E+00_JPRB, 0.22169E+00_JPRB ,&
&  0.22508E+00_JPRB, 0.22853E+00_JPRB, 0.23096E+00_JPRB, 0.23224E+00_JPRB, 0.23397E+00_JPRB, 0.23507E+00_JPRB, 0.23403E+00_JPRB ,&
&  0.23223E+00_JPRB, 0.23746E+00_JPRB, 0.25380E+00_JPRB, 0.27987E+00_JPRB, 0.31131E+00_JPRB, 0.34510E+00_JPRB, 0.39649E+00_JPRB ,&
&  0.46215E+00_JPRB, 0.53894E+00_JPRB, 0.55567E+00_JPRB, 0.56691E+00_JPRB, 0.57695E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.62248E+00_JPRB, 0.59747E+00_JPRB, 0.58075E+00_JPRB, 0.56750E+00_JPRB, 0.54925E+00_JPRB, 0.53543E+00_JPRB, 0.52989E+00_JPRB ,&
&  0.53821E+00_JPRB, 0.53708E+00_JPRB, 0.52744E+00_JPRB, 0.51928E+00_JPRB, 0.50330E+00_JPRB, 0.48898E+00_JPRB, 0.48314E+00_JPRB ,&
&  0.49192E+00_JPRB, 0.50851E+00_JPRB, 0.52873E+00_JPRB, 0.53751E+00_JPRB, 0.53725E+00_JPRB, 0.54578E+00_JPRB, 0.56500E+00_JPRB ,&
&  0.59337E+00_JPRB, 0.61317E+00_JPRB, 0.62054E+00_JPRB, 0.60796E+00_JPRB, 0.59084E+00_JPRB, 0.58288E+00_JPRB, 0.56844E+00_JPRB ,&
&  0.55170E+00_JPRB, 0.55559E+00_JPRB, 0.48748E+00_JPRB, 0.48002E+00_JPRB, 0.45650E+00_JPRB, 0.44239E+00_JPRB, 0.43894E+00_JPRB ,&
&  0.45243E+00_JPRB, 0.45704E+00_JPRB, 0.44938E+00_JPRB, 0.43788E+00_JPRB, 0.42965E+00_JPRB, 0.41833E+00_JPRB, 0.40760E+00_JPRB ,&
&  0.39979E+00_JPRB, 0.40205E+00_JPRB, 0.41107E+00_JPRB, 0.42036E+00_JPRB, 0.43155E+00_JPRB, 0.44219E+00_JPRB, 0.43859E+00_JPRB ,&
&  0.42694E+00_JPRB, 0.42482E+00_JPRB, 0.43134E+00_JPRB, 0.44529E+00_JPRB, 0.46054E+00_JPRB, 0.46787E+00_JPRB, 0.47220E+00_JPRB ,&
&  0.46523E+00_JPRB, 0.44882E+00_JPRB, 0.43465E+00_JPRB, 0.42292E+00_JPRB, 0.39836E+00_JPRB, 0.33463E+00_JPRB, 0.31159E+00_JPRB ,&
&  0.32186E+00_JPRB, 0.38102E+00_JPRB, 0.45226E+00_JPRB, 0.44302E+00_JPRB, 0.43747E+00_JPRB, 0.42349E+00_JPRB, 0.38818E+00_JPRB ,&
&  0.34881E+00_JPRB, 0.28698E+00_JPRB, 0.26441E+00_JPRB, 0.26037E+00_JPRB, 0.25892E+00_JPRB, 0.26350E+00_JPRB, 0.27520E+00_JPRB ,&
&  0.29085E+00_JPRB, 0.31860E+00_JPRB, 0.33644E+00_JPRB, 0.34344E+00_JPRB, 0.34230E+00_JPRB, 0.32623E+00_JPRB, 0.31256E+00_JPRB ,&
&  0.30298E+00_JPRB, 0.32702E+00_JPRB, 0.38453E+00_JPRB, 0.46738E+00_JPRB, 0.46486E+00_JPRB, 0.48797E+00_JPRB, 0.51195E+00_JPRB ,&
&  0.53485E+00_JPRB, 0.57379E+00_JPRB, 0.59973E+00_JPRB, 0.60556E+00_JPRB, 0.62392E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.64575E+00_JPRB, 0.61867E+00_JPRB, 0.59012E+00_JPRB, 0.56533E+00_JPRB, 0.55301E+00_JPRB, 0.53510E+00_JPRB, 0.52023E+00_JPRB ,&
&  0.51931E+00_JPRB, 0.52730E+00_JPRB, 0.52294E+00_JPRB, 0.51728E+00_JPRB, 0.50803E+00_JPRB, 0.50034E+00_JPRB, 0.49111E+00_JPRB ,&
&  0.48735E+00_JPRB, 0.49940E+00_JPRB, 0.52551E+00_JPRB, 0.53884E+00_JPRB, 0.54547E+00_JPRB, 0.57623E+00_JPRB, 0.60750E+00_JPRB ,&
&  0.61629E+00_JPRB, 0.62994E+00_JPRB, 0.63436E+00_JPRB, 0.60992E+00_JPRB, 0.59396E+00_JPRB, 0.58780E+00_JPRB, 0.59169E+00_JPRB ,&
&  0.58686E+00_JPRB, 0.59111E+00_JPRB, 0.58333E+00_JPRB, 0.51247E+00_JPRB, 0.51180E+00_JPRB, 0.48102E+00_JPRB, 0.47080E+00_JPRB ,&
&  0.46614E+00_JPRB, 0.47218E+00_JPRB, 0.46932E+00_JPRB, 0.46110E+00_JPRB, 0.45789E+00_JPRB, 0.46104E+00_JPRB, 0.46187E+00_JPRB ,&
&  0.45664E+00_JPRB, 0.44558E+00_JPRB, 0.44989E+00_JPRB, 0.45632E+00_JPRB, 0.46013E+00_JPRB, 0.46722E+00_JPRB, 0.47772E+00_JPRB ,&
&  0.46752E+00_JPRB, 0.45874E+00_JPRB, 0.45022E+00_JPRB, 0.45031E+00_JPRB, 0.45663E+00_JPRB, 0.47111E+00_JPRB, 0.47643E+00_JPRB ,&
&  0.47450E+00_JPRB, 0.47324E+00_JPRB, 0.47408E+00_JPRB, 0.48029E+00_JPRB, 0.49542E+00_JPRB, 0.51059E+00_JPRB, 0.51187E+00_JPRB ,&
&  0.51279E+00_JPRB, 0.51709E+00_JPRB, 0.52149E+00_JPRB, 0.52366E+00_JPRB, 0.51998E+00_JPRB, 0.50503E+00_JPRB, 0.48756E+00_JPRB ,&
&  0.47100E+00_JPRB, 0.45479E+00_JPRB, 0.44079E+00_JPRB, 0.43596E+00_JPRB, 0.44851E+00_JPRB, 0.47716E+00_JPRB, 0.49302E+00_JPRB ,&
&  0.49658E+00_JPRB, 0.49854E+00_JPRB, 0.50227E+00_JPRB, 0.50384E+00_JPRB, 0.50629E+00_JPRB, 0.50471E+00_JPRB, 0.49856E+00_JPRB ,&
&  0.48750E+00_JPRB, 0.47593E+00_JPRB, 0.47228E+00_JPRB, 0.47544E+00_JPRB, 0.47948E+00_JPRB, 0.49024E+00_JPRB, 0.50565E+00_JPRB ,&
&  0.52678E+00_JPRB, 0.56443E+00_JPRB, 0.59655E+00_JPRB, 0.62160E+00_JPRB, 0.63991E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.66318E+00_JPRB, 0.66129E+00_JPRB, 0.63299E+00_JPRB, 0.60156E+00_JPRB, 0.58251E+00_JPRB, 0.56122E+00_JPRB, 0.54403E+00_JPRB ,&
&  0.54247E+00_JPRB, 0.53993E+00_JPRB, 0.53814E+00_JPRB, 0.53598E+00_JPRB, 0.53440E+00_JPRB, 0.53126E+00_JPRB, 0.51916E+00_JPRB ,&
&  0.51489E+00_JPRB, 0.52167E+00_JPRB, 0.54428E+00_JPRB, 0.56995E+00_JPRB, 0.59066E+00_JPRB, 0.61727E+00_JPRB, 0.62826E+00_JPRB ,&
&  0.63957E+00_JPRB, 0.65173E+00_JPRB, 0.65417E+00_JPRB, 0.62212E+00_JPRB, 0.59571E+00_JPRB, 0.59699E+00_JPRB, 0.60796E+00_JPRB ,&
&  0.60822E+00_JPRB, 0.61144E+00_JPRB, 0.61547E+00_JPRB, 0.61149E+00_JPRB, 0.59051E+00_JPRB, 0.55242E+00_JPRB, 0.52269E+00_JPRB ,&
&  0.49862E+00_JPRB, 0.48015E+00_JPRB, 0.47506E+00_JPRB, 0.46464E+00_JPRB, 0.46382E+00_JPRB, 0.47406E+00_JPRB, 0.48654E+00_JPRB ,&
&  0.49926E+00_JPRB, 0.50473E+00_JPRB, 0.51159E+00_JPRB, 0.50835E+00_JPRB, 0.50390E+00_JPRB, 0.50633E+00_JPRB, 0.50592E+00_JPRB ,&
&  0.50491E+00_JPRB, 0.49577E+00_JPRB, 0.49323E+00_JPRB, 0.48636E+00_JPRB, 0.48259E+00_JPRB, 0.48328E+00_JPRB, 0.48654E+00_JPRB ,&
&  0.48785E+00_JPRB, 0.49327E+00_JPRB, 0.49487E+00_JPRB, 0.49690E+00_JPRB, 0.50623E+00_JPRB, 0.52274E+00_JPRB, 0.53937E+00_JPRB ,&
&  0.55183E+00_JPRB, 0.56163E+00_JPRB, 0.56948E+00_JPRB, 0.57528E+00_JPRB, 0.57502E+00_JPRB, 0.57050E+00_JPRB, 0.55669E+00_JPRB ,&
&  0.52605E+00_JPRB, 0.49314E+00_JPRB, 0.46828E+00_JPRB, 0.44832E+00_JPRB, 0.44424E+00_JPRB, 0.45469E+00_JPRB, 0.47214E+00_JPRB ,&
&  0.48536E+00_JPRB, 0.48959E+00_JPRB, 0.48398E+00_JPRB, 0.48255E+00_JPRB, 0.48804E+00_JPRB, 0.49784E+00_JPRB, 0.50542E+00_JPRB ,&
&  0.50473E+00_JPRB, 0.50134E+00_JPRB, 0.50455E+00_JPRB, 0.51065E+00_JPRB, 0.52365E+00_JPRB, 0.54029E+00_JPRB, 0.55847E+00_JPRB ,&
&  0.57682E+00_JPRB, 0.59131E+00_JPRB, 0.61034E+00_JPRB, 0.62975E+00_JPRB, 0.65519E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.66778E+00_JPRB, 0.65958E+00_JPRB, 0.64530E+00_JPRB, 0.61944E+00_JPRB, 0.60258E+00_JPRB, 0.57560E+00_JPRB, 0.56012E+00_JPRB ,&
&  0.56218E+00_JPRB, 0.56322E+00_JPRB, 0.56820E+00_JPRB, 0.57773E+00_JPRB, 0.57597E+00_JPRB, 0.57163E+00_JPRB, 0.56750E+00_JPRB ,&
&  0.57095E+00_JPRB, 0.57152E+00_JPRB, 0.58007E+00_JPRB, 0.59584E+00_JPRB, 0.61247E+00_JPRB, 0.62169E+00_JPRB, 0.63101E+00_JPRB ,&
&  0.63812E+00_JPRB, 0.63178E+00_JPRB, 0.62756E+00_JPRB, 0.59764E+00_JPRB, 0.58030E+00_JPRB, 0.58564E+00_JPRB, 0.59298E+00_JPRB ,&
&  0.60305E+00_JPRB, 0.61357E+00_JPRB, 0.62437E+00_JPRB, 0.62964E+00_JPRB, 0.61582E+00_JPRB, 0.61243E+00_JPRB, 0.60201E+00_JPRB ,&
&  0.56029E+00_JPRB, 0.51936E+00_JPRB, 0.49732E+00_JPRB, 0.49053E+00_JPRB, 0.48648E+00_JPRB, 0.49279E+00_JPRB, 0.49975E+00_JPRB ,&
&  0.51208E+00_JPRB, 0.52348E+00_JPRB, 0.52652E+00_JPRB, 0.52998E+00_JPRB, 0.53156E+00_JPRB, 0.53055E+00_JPRB, 0.53048E+00_JPRB ,&
&  0.52706E+00_JPRB, 0.52472E+00_JPRB, 0.53171E+00_JPRB, 0.53235E+00_JPRB, 0.52390E+00_JPRB, 0.52190E+00_JPRB, 0.52039E+00_JPRB ,&
&  0.51816E+00_JPRB, 0.51579E+00_JPRB, 0.51277E+00_JPRB, 0.52268E+00_JPRB, 0.54781E+00_JPRB, 0.57684E+00_JPRB, 0.60210E+00_JPRB ,&
&  0.61633E+00_JPRB, 0.62777E+00_JPRB, 0.63077E+00_JPRB, 0.63286E+00_JPRB, 0.63241E+00_JPRB, 0.62384E+00_JPRB, 0.62081E+00_JPRB ,&
&  0.61300E+00_JPRB, 0.59403E+00_JPRB, 0.56868E+00_JPRB, 0.52927E+00_JPRB, 0.49098E+00_JPRB, 0.46918E+00_JPRB, 0.46876E+00_JPRB ,&
&  0.48087E+00_JPRB, 0.49320E+00_JPRB, 0.49959E+00_JPRB, 0.50781E+00_JPRB, 0.52283E+00_JPRB, 0.54077E+00_JPRB, 0.55943E+00_JPRB ,&
&  0.57095E+00_JPRB, 0.57052E+00_JPRB, 0.56740E+00_JPRB, 0.57463E+00_JPRB, 0.58463E+00_JPRB, 0.60158E+00_JPRB, 0.62034E+00_JPRB ,&
&  0.63696E+00_JPRB, 0.63643E+00_JPRB, 0.63679E+00_JPRB, 0.63793E+00_JPRB, 0.65745E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.67057E+00_JPRB, 0.67155E+00_JPRB, 0.65637E+00_JPRB, 0.64570E+00_JPRB, 0.62986E+00_JPRB, 0.60917E+00_JPRB, 0.59066E+00_JPRB ,&
&  0.59173E+00_JPRB, 0.60631E+00_JPRB, 0.62048E+00_JPRB, 0.63096E+00_JPRB, 0.64651E+00_JPRB, 0.64988E+00_JPRB, 0.65686E+00_JPRB ,&
&  0.65340E+00_JPRB, 0.61680E+00_JPRB, 0.61116E+00_JPRB, 0.61902E+00_JPRB, 0.62092E+00_JPRB, 0.62657E+00_JPRB, 0.63679E+00_JPRB ,&
&  0.64627E+00_JPRB, 0.64136E+00_JPRB, 0.63322E+00_JPRB, 0.60391E+00_JPRB, 0.59728E+00_JPRB, 0.60220E+00_JPRB, 0.60704E+00_JPRB ,&
&  0.61733E+00_JPRB, 0.63349E+00_JPRB, 0.63855E+00_JPRB, 0.64640E+00_JPRB, 0.65618E+00_JPRB, 0.66100E+00_JPRB, 0.65785E+00_JPRB ,&
&  0.63251E+00_JPRB, 0.60704E+00_JPRB, 0.59680E+00_JPRB, 0.57988E+00_JPRB, 0.55883E+00_JPRB, 0.54396E+00_JPRB, 0.53593E+00_JPRB ,&
&  0.53135E+00_JPRB, 0.53616E+00_JPRB, 0.54237E+00_JPRB, 0.55627E+00_JPRB, 0.56140E+00_JPRB, 0.56548E+00_JPRB, 0.57051E+00_JPRB ,&
&  0.56963E+00_JPRB, 0.56498E+00_JPRB, 0.56467E+00_JPRB, 0.57439E+00_JPRB, 0.58062E+00_JPRB, 0.57890E+00_JPRB, 0.57508E+00_JPRB ,&
&  0.56286E+00_JPRB, 0.54904E+00_JPRB, 0.55637E+00_JPRB, 0.57102E+00_JPRB, 0.57742E+00_JPRB, 0.60200E+00_JPRB, 0.63400E+00_JPRB ,&
&  0.66110E+00_JPRB, 0.67899E+00_JPRB, 0.67868E+00_JPRB, 0.67562E+00_JPRB, 0.67433E+00_JPRB, 0.68171E+00_JPRB, 0.69422E+00_JPRB ,&
&  0.67869E+00_JPRB, 0.66966E+00_JPRB, 0.66775E+00_JPRB, 0.64886E+00_JPRB, 0.61311E+00_JPRB, 0.58278E+00_JPRB, 0.56022E+00_JPRB ,&
&  0.54209E+00_JPRB, 0.53161E+00_JPRB, 0.53510E+00_JPRB, 0.53893E+00_JPRB, 0.55210E+00_JPRB, 0.58762E+00_JPRB, 0.62434E+00_JPRB ,&
&  0.66405E+00_JPRB, 0.68423E+00_JPRB, 0.69010E+00_JPRB, 0.69747E+00_JPRB, 0.70253E+00_JPRB, 0.69100E+00_JPRB, 0.68143E+00_JPRB ,&
&  0.67850E+00_JPRB, 0.67017E+00_JPRB, 0.66087E+00_JPRB, 0.65101E+00_JPRB, 0.65792E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.66419E+00_JPRB, 0.68541E+00_JPRB, 0.65260E+00_JPRB, 0.63490E+00_JPRB, 0.61509E+00_JPRB, 0.61121E+00_JPRB, 0.60225E+00_JPRB ,&
&  0.61227E+00_JPRB, 0.62877E+00_JPRB, 0.65229E+00_JPRB, 0.66025E+00_JPRB, 0.66724E+00_JPRB, 0.67028E+00_JPRB, 0.68775E+00_JPRB ,&
&  0.68372E+00_JPRB, 0.67591E+00_JPRB, 0.67938E+00_JPRB, 0.66929E+00_JPRB, 0.66066E+00_JPRB, 0.66515E+00_JPRB, 0.67329E+00_JPRB ,&
&  0.68284E+00_JPRB, 0.67411E+00_JPRB, 0.65686E+00_JPRB, 0.63036E+00_JPRB, 0.61781E+00_JPRB, 0.61522E+00_JPRB, 0.62252E+00_JPRB ,&
&  0.62790E+00_JPRB, 0.64994E+00_JPRB, 0.58961E+00_JPRB, 0.64511E+00_JPRB, 0.65522E+00_JPRB, 0.65545E+00_JPRB, 0.65423E+00_JPRB ,&
&  0.65899E+00_JPRB, 0.64877E+00_JPRB, 0.63988E+00_JPRB, 0.63136E+00_JPRB, 0.60930E+00_JPRB, 0.59128E+00_JPRB, 0.57194E+00_JPRB ,&
&  0.55482E+00_JPRB, 0.54446E+00_JPRB, 0.54940E+00_JPRB, 0.56639E+00_JPRB, 0.58382E+00_JPRB, 0.59373E+00_JPRB, 0.59989E+00_JPRB ,&
&  0.60323E+00_JPRB, 0.59594E+00_JPRB, 0.58610E+00_JPRB, 0.58636E+00_JPRB, 0.59323E+00_JPRB, 0.59867E+00_JPRB, 0.60263E+00_JPRB ,&
&  0.60404E+00_JPRB, 0.58102E+00_JPRB, 0.56802E+00_JPRB, 0.58402E+00_JPRB, 0.60089E+00_JPRB, 0.61894E+00_JPRB, 0.64256E+00_JPRB ,&
&  0.66035E+00_JPRB, 0.66707E+00_JPRB, 0.66598E+00_JPRB, 0.67280E+00_JPRB, 0.67988E+00_JPRB, 0.68830E+00_JPRB, 0.69503E+00_JPRB ,&
&  0.68571E+00_JPRB, 0.68416E+00_JPRB, 0.68536E+00_JPRB, 0.68574E+00_JPRB, 0.67751E+00_JPRB, 0.65554E+00_JPRB, 0.62418E+00_JPRB ,&
&  0.60989E+00_JPRB, 0.59500E+00_JPRB, 0.58383E+00_JPRB, 0.58132E+00_JPRB, 0.58407E+00_JPRB, 0.60976E+00_JPRB, 0.64611E+00_JPRB ,&
&  0.68210E+00_JPRB, 0.71119E+00_JPRB, 0.73613E+00_JPRB, 0.75895E+00_JPRB, 0.75631E+00_JPRB, 0.73377E+00_JPRB, 0.72343E+00_JPRB ,&
&  0.70805E+00_JPRB, 0.69192E+00_JPRB, 0.68134E+00_JPRB, 0.67265E+00_JPRB, 0.66275E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.70002E+00_JPRB, 0.70074E+00_JPRB, 0.69045E+00_JPRB, 0.65250E+00_JPRB, 0.62391E+00_JPRB, 0.63409E+00_JPRB, 0.64376E+00_JPRB ,&
&  0.64323E+00_JPRB, 0.65778E+00_JPRB, 0.68291E+00_JPRB, 0.70858E+00_JPRB, 0.72586E+00_JPRB, 0.72614E+00_JPRB, 0.73075E+00_JPRB ,&
&  0.72892E+00_JPRB, 0.72727E+00_JPRB, 0.73754E+00_JPRB, 0.74255E+00_JPRB, 0.73198E+00_JPRB, 0.71086E+00_JPRB, 0.70795E+00_JPRB ,&
&  0.71364E+00_JPRB, 0.70938E+00_JPRB, 0.69100E+00_JPRB, 0.67846E+00_JPRB, 0.66834E+00_JPRB, 0.67339E+00_JPRB, 0.67940E+00_JPRB ,&
&  0.67453E+00_JPRB, 0.61982E+00_JPRB, 0.65783E+00_JPRB, 0.66991E+00_JPRB, 0.67929E+00_JPRB, 0.67069E+00_JPRB, 0.67598E+00_JPRB ,&
&  0.68769E+00_JPRB, 0.69508E+00_JPRB, 0.67507E+00_JPRB, 0.65120E+00_JPRB, 0.62983E+00_JPRB, 0.61244E+00_JPRB, 0.58481E+00_JPRB ,&
&  0.57440E+00_JPRB, 0.57169E+00_JPRB, 0.58495E+00_JPRB, 0.59886E+00_JPRB, 0.61488E+00_JPRB, 0.62140E+00_JPRB, 0.63175E+00_JPRB ,&
&  0.63432E+00_JPRB, 0.62937E+00_JPRB, 0.62500E+00_JPRB, 0.62200E+00_JPRB, 0.62211E+00_JPRB, 0.61687E+00_JPRB, 0.60925E+00_JPRB ,&
&  0.60379E+00_JPRB, 0.60239E+00_JPRB, 0.60681E+00_JPRB, 0.61612E+00_JPRB, 0.62727E+00_JPRB, 0.64372E+00_JPRB, 0.65604E+00_JPRB ,&
&  0.67158E+00_JPRB, 0.67603E+00_JPRB, 0.68011E+00_JPRB, 0.69293E+00_JPRB, 0.70151E+00_JPRB, 0.70000E+00_JPRB, 0.69703E+00_JPRB ,&
&  0.69473E+00_JPRB, 0.68991E+00_JPRB, 0.69526E+00_JPRB, 0.69508E+00_JPRB, 0.69636E+00_JPRB, 0.68885E+00_JPRB, 0.67462E+00_JPRB ,&
&  0.66937E+00_JPRB, 0.66528E+00_JPRB, 0.65668E+00_JPRB, 0.65398E+00_JPRB, 0.65187E+00_JPRB, 0.66916E+00_JPRB, 0.69958E+00_JPRB ,&
&  0.71559E+00_JPRB, 0.73718E+00_JPRB, 0.76857E+00_JPRB, 0.78906E+00_JPRB, 0.78737E+00_JPRB, 0.78529E+00_JPRB, 0.77281E+00_JPRB ,&
&  0.75302E+00_JPRB, 0.73925E+00_JPRB, 0.72699E+00_JPRB, 0.72310E+00_JPRB, 0.71970E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.73110E+00_JPRB, 0.72450E+00_JPRB, 0.71005E+00_JPRB, 0.68381E+00_JPRB, 0.65077E+00_JPRB, 0.66016E+00_JPRB, 0.67355E+00_JPRB ,&
&  0.66839E+00_JPRB, 0.65959E+00_JPRB, 0.67011E+00_JPRB, 0.69520E+00_JPRB, 0.72025E+00_JPRB, 0.73503E+00_JPRB, 0.73841E+00_JPRB ,&
&  0.73460E+00_JPRB, 0.73771E+00_JPRB, 0.74675E+00_JPRB, 0.76400E+00_JPRB, 0.76306E+00_JPRB, 0.74494E+00_JPRB, 0.73361E+00_JPRB ,&
&  0.73006E+00_JPRB, 0.72791E+00_JPRB, 0.72479E+00_JPRB, 0.70776E+00_JPRB, 0.69640E+00_JPRB, 0.69370E+00_JPRB, 0.69434E+00_JPRB ,&
&  0.69234E+00_JPRB, 0.55457E+00_JPRB, 0.65003E+00_JPRB, 0.66034E+00_JPRB, 0.67231E+00_JPRB, 0.67123E+00_JPRB, 0.67363E+00_JPRB ,&
&  0.67962E+00_JPRB, 0.69579E+00_JPRB, 0.68801E+00_JPRB, 0.66094E+00_JPRB, 0.63242E+00_JPRB, 0.60424E+00_JPRB, 0.58559E+00_JPRB ,&
&  0.57674E+00_JPRB, 0.58010E+00_JPRB, 0.59024E+00_JPRB, 0.60071E+00_JPRB, 0.61198E+00_JPRB, 0.61405E+00_JPRB, 0.61831E+00_JPRB ,&
&  0.62166E+00_JPRB, 0.62782E+00_JPRB, 0.63160E+00_JPRB, 0.63871E+00_JPRB, 0.64226E+00_JPRB, 0.64012E+00_JPRB, 0.63380E+00_JPRB ,&
&  0.62476E+00_JPRB, 0.61923E+00_JPRB, 0.61897E+00_JPRB, 0.62146E+00_JPRB, 0.62823E+00_JPRB, 0.63920E+00_JPRB, 0.64952E+00_JPRB ,&
&  0.66163E+00_JPRB, 0.66942E+00_JPRB, 0.67355E+00_JPRB, 0.67584E+00_JPRB, 0.68524E+00_JPRB, 0.69515E+00_JPRB, 0.69551E+00_JPRB ,&
&  0.69000E+00_JPRB, 0.68395E+00_JPRB, 0.68327E+00_JPRB, 0.68143E+00_JPRB, 0.68305E+00_JPRB, 0.68485E+00_JPRB, 0.68148E+00_JPRB ,&
&  0.67383E+00_JPRB, 0.67218E+00_JPRB, 0.67335E+00_JPRB, 0.66253E+00_JPRB, 0.66378E+00_JPRB, 0.67329E+00_JPRB, 0.69108E+00_JPRB ,&
&  0.71315E+00_JPRB, 0.74247E+00_JPRB, 0.77172E+00_JPRB, 0.79690E+00_JPRB, 0.81193E+00_JPRB, 0.81672E+00_JPRB, 0.81126E+00_JPRB ,&
&  0.79419E+00_JPRB, 0.76968E+00_JPRB, 0.75164E+00_JPRB, 0.74273E+00_JPRB, 0.74474E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.75642E+00_JPRB, 0.74716E+00_JPRB, 0.73468E+00_JPRB, 0.71787E+00_JPRB, 0.70323E+00_JPRB, 0.69611E+00_JPRB, 0.70290E+00_JPRB ,&
&  0.70255E+00_JPRB, 0.69132E+00_JPRB, 0.70102E+00_JPRB, 0.71453E+00_JPRB, 0.72604E+00_JPRB, 0.73720E+00_JPRB, 0.74258E+00_JPRB ,&
&  0.74851E+00_JPRB, 0.74753E+00_JPRB, 0.75204E+00_JPRB, 0.76471E+00_JPRB, 0.78325E+00_JPRB, 0.78469E+00_JPRB, 0.78160E+00_JPRB ,&
&  0.75867E+00_JPRB, 0.74679E+00_JPRB, 0.73600E+00_JPRB, 0.72306E+00_JPRB, 0.71185E+00_JPRB, 0.70883E+00_JPRB, 0.71261E+00_JPRB ,&
&  0.60243E+00_JPRB, 0.50610E+00_JPRB, 0.60271E+00_JPRB, 0.68372E+00_JPRB, 0.69077E+00_JPRB, 0.69793E+00_JPRB, 0.71567E+00_JPRB ,&
&  0.72244E+00_JPRB, 0.72574E+00_JPRB, 0.72313E+00_JPRB, 0.70883E+00_JPRB, 0.67646E+00_JPRB, 0.64420E+00_JPRB, 0.62562E+00_JPRB ,&
&  0.61419E+00_JPRB, 0.61909E+00_JPRB, 0.62145E+00_JPRB, 0.62933E+00_JPRB, 0.63017E+00_JPRB, 0.62375E+00_JPRB, 0.62326E+00_JPRB ,&
&  0.62548E+00_JPRB, 0.63703E+00_JPRB, 0.64843E+00_JPRB, 0.65732E+00_JPRB, 0.66166E+00_JPRB, 0.66394E+00_JPRB, 0.66273E+00_JPRB ,&
&  0.66229E+00_JPRB, 0.66142E+00_JPRB, 0.65827E+00_JPRB, 0.65928E+00_JPRB, 0.66096E+00_JPRB, 0.66593E+00_JPRB, 0.67500E+00_JPRB ,&
&  0.67839E+00_JPRB, 0.67722E+00_JPRB, 0.67754E+00_JPRB, 0.67871E+00_JPRB, 0.68887E+00_JPRB, 0.69758E+00_JPRB, 0.69664E+00_JPRB ,&
&  0.69337E+00_JPRB, 0.69207E+00_JPRB, 0.69953E+00_JPRB, 0.70570E+00_JPRB, 0.71305E+00_JPRB, 0.71665E+00_JPRB, 0.71290E+00_JPRB ,&
&  0.70446E+00_JPRB, 0.69046E+00_JPRB, 0.68000E+00_JPRB, 0.67874E+00_JPRB, 0.68705E+00_JPRB, 0.69283E+00_JPRB, 0.70757E+00_JPRB ,&
&  0.73591E+00_JPRB, 0.77125E+00_JPRB, 0.80157E+00_JPRB, 0.83791E+00_JPRB, 0.85971E+00_JPRB, 0.87078E+00_JPRB, 0.86578E+00_JPRB ,&
&  0.84867E+00_JPRB, 0.81059E+00_JPRB, 0.79898E+00_JPRB, 0.77722E+00_JPRB, 0.76615E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.73337E+00_JPRB, 0.73886E+00_JPRB, 0.74234E+00_JPRB, 0.73280E+00_JPRB, 0.72036E+00_JPRB, 0.71604E+00_JPRB, 0.72295E+00_JPRB ,&
&  0.71902E+00_JPRB, 0.70595E+00_JPRB, 0.71121E+00_JPRB, 0.71625E+00_JPRB, 0.72517E+00_JPRB, 0.73087E+00_JPRB, 0.73754E+00_JPRB ,&
&  0.74435E+00_JPRB, 0.75033E+00_JPRB, 0.74996E+00_JPRB, 0.75936E+00_JPRB, 0.75278E+00_JPRB, 0.74769E+00_JPRB, 0.74493E+00_JPRB ,&
&  0.73602E+00_JPRB, 0.72648E+00_JPRB, 0.71881E+00_JPRB, 0.70873E+00_JPRB, 0.70198E+00_JPRB, 0.69459E+00_JPRB, 0.68008E+00_JPRB ,&
&  0.69383E+00_JPRB, 0.53821E+00_JPRB, 0.59687E+00_JPRB, 0.69603E+00_JPRB, 0.70196E+00_JPRB, 0.70887E+00_JPRB, 0.72472E+00_JPRB ,&
&  0.74462E+00_JPRB, 0.75645E+00_JPRB, 0.74720E+00_JPRB, 0.72459E+00_JPRB, 0.68588E+00_JPRB, 0.66497E+00_JPRB, 0.65167E+00_JPRB ,&
&  0.63754E+00_JPRB, 0.62809E+00_JPRB, 0.62318E+00_JPRB, 0.62573E+00_JPRB, 0.62604E+00_JPRB, 0.62352E+00_JPRB, 0.62222E+00_JPRB ,&
&  0.62795E+00_JPRB, 0.63983E+00_JPRB, 0.64853E+00_JPRB, 0.65719E+00_JPRB, 0.66548E+00_JPRB, 0.66487E+00_JPRB, 0.66755E+00_JPRB ,&
&  0.66884E+00_JPRB, 0.67078E+00_JPRB, 0.66966E+00_JPRB, 0.66821E+00_JPRB, 0.66649E+00_JPRB, 0.67743E+00_JPRB, 0.68560E+00_JPRB ,&
&  0.68330E+00_JPRB, 0.68048E+00_JPRB, 0.68366E+00_JPRB, 0.68952E+00_JPRB, 0.69461E+00_JPRB, 0.69858E+00_JPRB, 0.69633E+00_JPRB ,&
&  0.69445E+00_JPRB, 0.69026E+00_JPRB, 0.69164E+00_JPRB, 0.69317E+00_JPRB, 0.69033E+00_JPRB, 0.69054E+00_JPRB, 0.69452E+00_JPRB ,&
&  0.68950E+00_JPRB, 0.67829E+00_JPRB, 0.68018E+00_JPRB, 0.69276E+00_JPRB, 0.70221E+00_JPRB, 0.70623E+00_JPRB, 0.71799E+00_JPRB ,&
&  0.74537E+00_JPRB, 0.77660E+00_JPRB, 0.80238E+00_JPRB, 0.83396E+00_JPRB, 0.85212E+00_JPRB, 0.86159E+00_JPRB, 0.86094E+00_JPRB ,&
&  0.84692E+00_JPRB, 0.82802E+00_JPRB, 0.59554E+00_JPRB, 0.69755E+00_JPRB, 0.72244E+00_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.73056E+00_JPRB, 0.74756E+00_JPRB, 0.74843E+00_JPRB, 0.74199E+00_JPRB, 0.73661E+00_JPRB, 0.73316E+00_JPRB, 0.73678E+00_JPRB ,&
&  0.73444E+00_JPRB, 0.72812E+00_JPRB, 0.73118E+00_JPRB, 0.73538E+00_JPRB, 0.74057E+00_JPRB, 0.74173E+00_JPRB, 0.75027E+00_JPRB ,&
&  0.75421E+00_JPRB, 0.75830E+00_JPRB, 0.76353E+00_JPRB, 0.75907E+00_JPRB, 0.74111E+00_JPRB, 0.72852E+00_JPRB, 0.71510E+00_JPRB ,&
&  0.71362E+00_JPRB, 0.71311E+00_JPRB, 0.71351E+00_JPRB, 0.70735E+00_JPRB, 0.68890E+00_JPRB, 0.67367E+00_JPRB, 0.66409E+00_JPRB ,&
&  0.66501E+00_JPRB, 0.51233E+00_JPRB, 0.54668E+00_JPRB, 0.68214E+00_JPRB, 0.74444E+00_JPRB, 0.75005E+00_JPRB, 0.75951E+00_JPRB ,&
&  0.77602E+00_JPRB, 0.77632E+00_JPRB, 0.75561E+00_JPRB, 0.72807E+00_JPRB, 0.69503E+00_JPRB, 0.68779E+00_JPRB, 0.67395E+00_JPRB ,&
&  0.65818E+00_JPRB, 0.65271E+00_JPRB, 0.64861E+00_JPRB, 0.65073E+00_JPRB, 0.64413E+00_JPRB, 0.63592E+00_JPRB, 0.64019E+00_JPRB ,&
&  0.65110E+00_JPRB, 0.65341E+00_JPRB, 0.66017E+00_JPRB, 0.66794E+00_JPRB, 0.66727E+00_JPRB, 0.68684E+00_JPRB, 0.69683E+00_JPRB ,&
&  0.70127E+00_JPRB, 0.70149E+00_JPRB, 0.70042E+00_JPRB, 0.69600E+00_JPRB, 0.69600E+00_JPRB, 0.71048E+00_JPRB, 0.72033E+00_JPRB ,&
&  0.71285E+00_JPRB, 0.70278E+00_JPRB, 0.71026E+00_JPRB, 0.71386E+00_JPRB, 0.70855E+00_JPRB, 0.69777E+00_JPRB, 0.68519E+00_JPRB ,&
&  0.68255E+00_JPRB, 0.68195E+00_JPRB, 0.68327E+00_JPRB, 0.68103E+00_JPRB, 0.67576E+00_JPRB, 0.67185E+00_JPRB, 0.67094E+00_JPRB ,&
&  0.67121E+00_JPRB, 0.67176E+00_JPRB, 0.67800E+00_JPRB, 0.69096E+00_JPRB, 0.70675E+00_JPRB, 0.72428E+00_JPRB, 0.74246E+00_JPRB ,&
&  0.76639E+00_JPRB, 0.79445E+00_JPRB, 0.82433E+00_JPRB, 0.78790E+00_JPRB, 0.84414E+00_JPRB, 0.85879E+00_JPRB, 0.86907E+00_JPRB ,&
&  0.86397E+00_JPRB, 0.83070E+00_JPRB, 0.78748E+00_JPRB, 0.62010E+00_JPRB, 0.69461E+00_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.74626E+00_JPRB, 0.77908E+00_JPRB, 0.77890E+00_JPRB, 0.75956E+00_JPRB, 0.74892E+00_JPRB, 0.74478E+00_JPRB, 0.74365E+00_JPRB ,&
&  0.73931E+00_JPRB, 0.72903E+00_JPRB, 0.73428E+00_JPRB, 0.74235E+00_JPRB, 0.74306E+00_JPRB, 0.73532E+00_JPRB, 0.73067E+00_JPRB ,&
&  0.73119E+00_JPRB, 0.73680E+00_JPRB, 0.74335E+00_JPRB, 0.73486E+00_JPRB, 0.71615E+00_JPRB, 0.70410E+00_JPRB, 0.69316E+00_JPRB ,&
&  0.69174E+00_JPRB, 0.69180E+00_JPRB, 0.68982E+00_JPRB, 0.69187E+00_JPRB, 0.67267E+00_JPRB, 0.65155E+00_JPRB, 0.63325E+00_JPRB ,&
&  0.61573E+00_JPRB, 0.51242E+00_JPRB, 0.57993E+00_JPRB, 0.71227E+00_JPRB, 0.78919E+00_JPRB, 0.81393E+00_JPRB, 0.80753E+00_JPRB ,&
&  0.80373E+00_JPRB, 0.77823E+00_JPRB, 0.74521E+00_JPRB, 0.71614E+00_JPRB, 0.70215E+00_JPRB, 0.69465E+00_JPRB, 0.69129E+00_JPRB ,&
&  0.68794E+00_JPRB, 0.68340E+00_JPRB, 0.68183E+00_JPRB, 0.66809E+00_JPRB, 0.65722E+00_JPRB, 0.65782E+00_JPRB, 0.66244E+00_JPRB ,&
&  0.66912E+00_JPRB, 0.66811E+00_JPRB, 0.67263E+00_JPRB, 0.67917E+00_JPRB, 0.66729E+00_JPRB, 0.67082E+00_JPRB, 0.67446E+00_JPRB ,&
&  0.67873E+00_JPRB, 0.69060E+00_JPRB, 0.69032E+00_JPRB, 0.71399E+00_JPRB, 0.71931E+00_JPRB, 0.72486E+00_JPRB, 0.72648E+00_JPRB ,&
&  0.72157E+00_JPRB, 0.72261E+00_JPRB, 0.71942E+00_JPRB, 0.71846E+00_JPRB, 0.71112E+00_JPRB, 0.69185E+00_JPRB, 0.67446E+00_JPRB ,&
&  0.67384E+00_JPRB, 0.67803E+00_JPRB, 0.67175E+00_JPRB, 0.67375E+00_JPRB, 0.67769E+00_JPRB, 0.67382E+00_JPRB, 0.67176E+00_JPRB ,&
&  0.67069E+00_JPRB, 0.67376E+00_JPRB, 0.67603E+00_JPRB, 0.68590E+00_JPRB, 0.70242E+00_JPRB, 0.72130E+00_JPRB, 0.74063E+00_JPRB ,&
&  0.75932E+00_JPRB, 0.77196E+00_JPRB, 0.79430E+00_JPRB, 0.84349E+00_JPRB, 0.84476E+00_JPRB, 0.84097E+00_JPRB, 0.85119E+00_JPRB ,&
&  0.84362E+00_JPRB, 0.81880E+00_JPRB, 0.79311E+00_JPRB, 0.75203E+00_JPRB, 0.69148E+00_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.83252E+00_JPRB, 0.82709E+00_JPRB, 0.81645E+00_JPRB, 0.79157E+00_JPRB, 0.77999E+00_JPRB, 0.76603E+00_JPRB, 0.75047E+00_JPRB ,&
&  0.74296E+00_JPRB, 0.73594E+00_JPRB, 0.73085E+00_JPRB, 0.71881E+00_JPRB, 0.70724E+00_JPRB, 0.70090E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69421E+00_JPRB, 0.69583E+00_JPRB, 0.68934E+00_JPRB, 0.68282E+00_JPRB, 0.68149E+00_JPRB, 0.68044E+00_JPRB, 0.68618E+00_JPRB ,&
&  0.67910E+00_JPRB, 0.67042E+00_JPRB, 0.67030E+00_JPRB, 0.66377E+00_JPRB, 0.65273E+00_JPRB, 0.64257E+00_JPRB, 0.61118E+00_JPRB ,&
&  0.59018E+00_JPRB, 0.53897E+00_JPRB, 0.67890E+00_JPRB, 0.84282E+00_JPRB, 0.85486E+00_JPRB, 0.87546E+00_JPRB, 0.85660E+00_JPRB ,&
&  0.83957E+00_JPRB, 0.79843E+00_JPRB, 0.75202E+00_JPRB, 0.72395E+00_JPRB, 0.72017E+00_JPRB, 0.71433E+00_JPRB, 0.70361E+00_JPRB ,&
&  0.68734E+00_JPRB, 0.67071E+00_JPRB, 0.65573E+00_JPRB, 0.65898E+00_JPRB, 0.65569E+00_JPRB, 0.65990E+00_JPRB, 0.66779E+00_JPRB ,&
&  0.67029E+00_JPRB, 0.67276E+00_JPRB, 0.66705E+00_JPRB, 0.66896E+00_JPRB, 0.67295E+00_JPRB, 0.68675E+00_JPRB, 0.70317E+00_JPRB ,&
&  0.72213E+00_JPRB, 0.71956E+00_JPRB, 0.72487E+00_JPRB, 0.72895E+00_JPRB, 0.73795E+00_JPRB, 0.73844E+00_JPRB, 0.73202E+00_JPRB ,&
&  0.72592E+00_JPRB, 0.72564E+00_JPRB, 0.70568E+00_JPRB, 0.71499E+00_JPRB, 0.70639E+00_JPRB, 0.69378E+00_JPRB, 0.68694E+00_JPRB ,&
&  0.67794E+00_JPRB, 0.68236E+00_JPRB, 0.67664E+00_JPRB, 0.68010E+00_JPRB, 0.68630E+00_JPRB, 0.68780E+00_JPRB, 0.68580E+00_JPRB ,&
&  0.69222E+00_JPRB, 0.69074E+00_JPRB, 0.69591E+00_JPRB, 0.71692E+00_JPRB, 0.74149E+00_JPRB, 0.75246E+00_JPRB, 0.76392E+00_JPRB ,&
&  0.77513E+00_JPRB, 0.79535E+00_JPRB, 0.76871E+00_JPRB, 0.77920E+00_JPRB, 0.87477E+00_JPRB, 0.87991E+00_JPRB, 0.84672E+00_JPRB ,&
&  0.82651E+00_JPRB, 0.80806E+00_JPRB, 0.80078E+00_JPRB, 0.77976E+00_JPRB, 0.80095E+00_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.80869E+00_JPRB, 0.78659E+00_JPRB, 0.77324E+00_JPRB, 0.75418E+00_JPRB, 0.74844E+00_JPRB, 0.73625E+00_JPRB, 0.71763E+00_JPRB ,&
&  0.70670E+00_JPRB, 0.70598E+00_JPRB, 0.70004E+00_JPRB, 0.68554E+00_JPRB, 0.67278E+00_JPRB, 0.66801E+00_JPRB, 0.66763E+00_JPRB ,&
&  0.66634E+00_JPRB, 0.66506E+00_JPRB, 0.66320E+00_JPRB, 0.65744E+00_JPRB, 0.66274E+00_JPRB, 0.67359E+00_JPRB, 0.68535E+00_JPRB ,&
&  0.67160E+00_JPRB, 0.66048E+00_JPRB, 0.64989E+00_JPRB, 0.63919E+00_JPRB, 0.63626E+00_JPRB, 0.61684E+00_JPRB, 0.58817E+00_JPRB ,&
&  0.60593E+00_JPRB, 0.62700E+00_JPRB, 0.81655E+00_JPRB, 0.99627E+00_JPRB, 0.91509E+00_JPRB, 0.84776E+00_JPRB, 0.82346E+00_JPRB ,&
&  0.80837E+00_JPRB, 0.76619E+00_JPRB, 0.74163E+00_JPRB, 0.71797E+00_JPRB, 0.71198E+00_JPRB, 0.70536E+00_JPRB, 0.67805E+00_JPRB ,&
&  0.65685E+00_JPRB, 0.64310E+00_JPRB, 0.63237E+00_JPRB, 0.62973E+00_JPRB, 0.62145E+00_JPRB, 0.63007E+00_JPRB, 0.63515E+00_JPRB ,&
&  0.64013E+00_JPRB, 0.64207E+00_JPRB, 0.64419E+00_JPRB, 0.64481E+00_JPRB, 0.66076E+00_JPRB, 0.71411E+00_JPRB, 0.75772E+00_JPRB ,&
&  0.74767E+00_JPRB, 0.74635E+00_JPRB, 0.74274E+00_JPRB, 0.74562E+00_JPRB, 0.74966E+00_JPRB, 0.73233E+00_JPRB, 0.72594E+00_JPRB ,&
&  0.71488E+00_JPRB, 0.71622E+00_JPRB, 0.69720E+00_JPRB, 0.69700E+00_JPRB, 0.70025E+00_JPRB, 0.69622E+00_JPRB, 0.69771E+00_JPRB ,&
&  0.69522E+00_JPRB, 0.69379E+00_JPRB, 0.69262E+00_JPRB, 0.70262E+00_JPRB, 0.70463E+00_JPRB, 0.69713E+00_JPRB, 0.69336E+00_JPRB ,&
&  0.69583E+00_JPRB, 0.68626E+00_JPRB, 0.69564E+00_JPRB, 0.72744E+00_JPRB, 0.75229E+00_JPRB, 0.75977E+00_JPRB, 0.77015E+00_JPRB ,&
&  0.78212E+00_JPRB, 0.78000E+00_JPRB, 0.76307E+00_JPRB, 0.77506E+00_JPRB, 0.79178E+00_JPRB, 0.87164E+00_JPRB, 0.82703E+00_JPRB ,&
&  0.80258E+00_JPRB, 0.79065E+00_JPRB, 0.78319E+00_JPRB, 0.76398E+00_JPRB, 0.81811E+00_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.76899E+00_JPRB, 0.75434E+00_JPRB, 0.74782E+00_JPRB, 0.74184E+00_JPRB, 0.72605E+00_JPRB, 0.70744E+00_JPRB, 0.70377E+00_JPRB ,&
&  0.70209E+00_JPRB, 0.68492E+00_JPRB, 0.66445E+00_JPRB, 0.63764E+00_JPRB, 0.63082E+00_JPRB, 0.63364E+00_JPRB, 0.63640E+00_JPRB ,&
&  0.63366E+00_JPRB, 0.63379E+00_JPRB, 0.63323E+00_JPRB, 0.63382E+00_JPRB, 0.63545E+00_JPRB, 0.66038E+00_JPRB, 0.67190E+00_JPRB ,&
&  0.66615E+00_JPRB, 0.66942E+00_JPRB, 0.64979E+00_JPRB, 0.62851E+00_JPRB, 0.61815E+00_JPRB, 0.60067E+00_JPRB, 0.58142E+00_JPRB ,&
&  0.65455E+00_JPRB, 0.74269E+00_JPRB, 0.10631E+01_JPRB, 0.10570E+01_JPRB, 0.91422E+00_JPRB, 0.79763E+00_JPRB, 0.77201E+00_JPRB ,&
&  0.77367E+00_JPRB, 0.75267E+00_JPRB, 0.72772E+00_JPRB, 0.70348E+00_JPRB, 0.68095E+00_JPRB, 0.67440E+00_JPRB, 0.66281E+00_JPRB ,&
&  0.65680E+00_JPRB, 0.65124E+00_JPRB, 0.62686E+00_JPRB, 0.61409E+00_JPRB, 0.60725E+00_JPRB, 0.60736E+00_JPRB, 0.60512E+00_JPRB ,&
&  0.61130E+00_JPRB, 0.61889E+00_JPRB, 0.62558E+00_JPRB, 0.61819E+00_JPRB, 0.59723E+00_JPRB, 0.56579E+00_JPRB, 0.67330E+00_JPRB ,&
&  0.87608E+00_JPRB, 0.84365E+00_JPRB, 0.78440E+00_JPRB, 0.76438E+00_JPRB, 0.75541E+00_JPRB, 0.72865E+00_JPRB, 0.72778E+00_JPRB ,&
&  0.72470E+00_JPRB, 0.73230E+00_JPRB, 0.72258E+00_JPRB, 0.73673E+00_JPRB, 0.74133E+00_JPRB, 0.73812E+00_JPRB, 0.73147E+00_JPRB ,&
&  0.71489E+00_JPRB, 0.71327E+00_JPRB, 0.71444E+00_JPRB, 0.71587E+00_JPRB, 0.70608E+00_JPRB, 0.70728E+00_JPRB, 0.70716E+00_JPRB ,&
&  0.70559E+00_JPRB, 0.68837E+00_JPRB, 0.69093E+00_JPRB, 0.71221E+00_JPRB, 0.77175E+00_JPRB, 0.80808E+00_JPRB, 0.78994E+00_JPRB ,&
&  0.74711E+00_JPRB, 0.71908E+00_JPRB, 0.72109E+00_JPRB, 0.72843E+00_JPRB, 0.74631E+00_JPRB, 0.83540E+00_JPRB, 0.78398E+00_JPRB ,&
&  0.75693E+00_JPRB, 0.75689E+00_JPRB, 0.75065E+00_JPRB, 0.74207E+00_JPRB, 0.76680E+00_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.75838E+00_JPRB, 0.74973E+00_JPRB, 0.73943E+00_JPRB, 0.72200E+00_JPRB, 0.69721E+00_JPRB, 0.68244E+00_JPRB, 0.67448E+00_JPRB ,&
&  0.67195E+00_JPRB, 0.64898E+00_JPRB, 0.63065E+00_JPRB, 0.63004E+00_JPRB, 0.63382E+00_JPRB, 0.62132E+00_JPRB, 0.61411E+00_JPRB ,&
&  0.61686E+00_JPRB, 0.61865E+00_JPRB, 0.61090E+00_JPRB, 0.60458E+00_JPRB, 0.61040E+00_JPRB, 0.64641E+00_JPRB, 0.66086E+00_JPRB ,&
&  0.66774E+00_JPRB, 0.66191E+00_JPRB, 0.64194E+00_JPRB, 0.61408E+00_JPRB, 0.60485E+00_JPRB, 0.60014E+00_JPRB, 0.60245E+00_JPRB ,&
&  0.74270E+00_JPRB, 0.80508E+00_JPRB, 0.12332E+01_JPRB, 0.12228E+01_JPRB, 0.92609E+00_JPRB, 0.78293E+00_JPRB, 0.70234E+00_JPRB ,&
&  0.74099E+00_JPRB, 0.72764E+00_JPRB, 0.71319E+00_JPRB, 0.68986E+00_JPRB, 0.66994E+00_JPRB, 0.65187E+00_JPRB, 0.64073E+00_JPRB ,&
&  0.64101E+00_JPRB, 0.62825E+00_JPRB, 0.61482E+00_JPRB, 0.61273E+00_JPRB, 0.60414E+00_JPRB, 0.60212E+00_JPRB, 0.60111E+00_JPRB ,&
&  0.60656E+00_JPRB, 0.60660E+00_JPRB, 0.60409E+00_JPRB, 0.59038E+00_JPRB, 0.56241E+00_JPRB, 0.60362E+00_JPRB, 0.67409E+00_JPRB ,&
&  0.73814E+00_JPRB, 0.90091E+00_JPRB, 0.81822E+00_JPRB, 0.78071E+00_JPRB, 0.75093E+00_JPRB, 0.72790E+00_JPRB, 0.73881E+00_JPRB ,&
&  0.74937E+00_JPRB, 0.76707E+00_JPRB, 0.76596E+00_JPRB, 0.77332E+00_JPRB, 0.77868E+00_JPRB, 0.75658E+00_JPRB, 0.74331E+00_JPRB ,&
&  0.73968E+00_JPRB, 0.73654E+00_JPRB, 0.72808E+00_JPRB, 0.71806E+00_JPRB, 0.70279E+00_JPRB, 0.69568E+00_JPRB, 0.69482E+00_JPRB ,&
&  0.69331E+00_JPRB, 0.68083E+00_JPRB, 0.68972E+00_JPRB, 0.71084E+00_JPRB, 0.74318E+00_JPRB, 0.74817E+00_JPRB, 0.73697E+00_JPRB ,&
&  0.71100E+00_JPRB, 0.69699E+00_JPRB, 0.68961E+00_JPRB, 0.71416E+00_JPRB, 0.70940E+00_JPRB, 0.77241E+00_JPRB, 0.73415E+00_JPRB ,&
&  0.72794E+00_JPRB, 0.72821E+00_JPRB, 0.71632E+00_JPRB, 0.71809E+00_JPRB, 0.74475E+00_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.72442E+00_JPRB, 0.69805E+00_JPRB, 0.68146E+00_JPRB, 0.66740E+00_JPRB, 0.65456E+00_JPRB, 0.64481E+00_JPRB, 0.65674E+00_JPRB ,&
&  0.65788E+00_JPRB, 0.63524E+00_JPRB, 0.61797E+00_JPRB, 0.62009E+00_JPRB, 0.61584E+00_JPRB, 0.61243E+00_JPRB, 0.60888E+00_JPRB ,&
&  0.59973E+00_JPRB, 0.58690E+00_JPRB, 0.58396E+00_JPRB, 0.59225E+00_JPRB, 0.62299E+00_JPRB, 0.66003E+00_JPRB, 0.66664E+00_JPRB ,&
&  0.66020E+00_JPRB, 0.65516E+00_JPRB, 0.63587E+00_JPRB, 0.61662E+00_JPRB, 0.60843E+00_JPRB, 0.60847E+00_JPRB, 0.62340E+00_JPRB ,&
&  0.84121E+00_JPRB, 0.17819E+01_JPRB, 0.97118E+00_JPRB, 0.12650E+01_JPRB, 0.92378E+00_JPRB, 0.74802E+00_JPRB, 0.61563E+00_JPRB ,&
&  0.63088E+00_JPRB, 0.69417E+00_JPRB, 0.70040E+00_JPRB, 0.68311E+00_JPRB, 0.64389E+00_JPRB, 0.62729E+00_JPRB, 0.62506E+00_JPRB ,&
&  0.64641E+00_JPRB, 0.62527E+00_JPRB, 0.61405E+00_JPRB, 0.60968E+00_JPRB, 0.59839E+00_JPRB, 0.59650E+00_JPRB, 0.60448E+00_JPRB ,&
&  0.60180E+00_JPRB, 0.59765E+00_JPRB, 0.59132E+00_JPRB, 0.61033E+00_JPRB, 0.57027E+00_JPRB, 0.65189E+00_JPRB, 0.74023E+00_JPRB ,&
&  0.71648E+00_JPRB, 0.93114E+00_JPRB, 0.83128E+00_JPRB, 0.78170E+00_JPRB, 0.75833E+00_JPRB, 0.75360E+00_JPRB, 0.76901E+00_JPRB ,&
&  0.76327E+00_JPRB, 0.75667E+00_JPRB, 0.74983E+00_JPRB, 0.75455E+00_JPRB, 0.76868E+00_JPRB, 0.74895E+00_JPRB, 0.74785E+00_JPRB ,&
&  0.75891E+00_JPRB, 0.73042E+00_JPRB, 0.72211E+00_JPRB, 0.71375E+00_JPRB, 0.72083E+00_JPRB, 0.72374E+00_JPRB, 0.71610E+00_JPRB ,&
&  0.70529E+00_JPRB, 0.69538E+00_JPRB, 0.69377E+00_JPRB, 0.69891E+00_JPRB, 0.70259E+00_JPRB, 0.69909E+00_JPRB, 0.67761E+00_JPRB ,&
&  0.68748E+00_JPRB, 0.68926E+00_JPRB, 0.68373E+00_JPRB, 0.70102E+00_JPRB, 0.70288E+00_JPRB, 0.73608E+00_JPRB, 0.70594E+00_JPRB ,&
&  0.71747E+00_JPRB, 0.71738E+00_JPRB, 0.70084E+00_JPRB, 0.71436E+00_JPRB, 0.74159E+00_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.68590E+00_JPRB, 0.66479E+00_JPRB, 0.64491E+00_JPRB, 0.62234E+00_JPRB, 0.60769E+00_JPRB, 0.61636E+00_JPRB, 0.64031E+00_JPRB ,&
&  0.62632E+00_JPRB, 0.60333E+00_JPRB, 0.59786E+00_JPRB, 0.61282E+00_JPRB, 0.62973E+00_JPRB, 0.62221E+00_JPRB, 0.61284E+00_JPRB ,&
&  0.59439E+00_JPRB, 0.58785E+00_JPRB, 0.57747E+00_JPRB, 0.58764E+00_JPRB, 0.63799E+00_JPRB, 0.63382E+00_JPRB, 0.62344E+00_JPRB ,&
&  0.64072E+00_JPRB, 0.63728E+00_JPRB, 0.61540E+00_JPRB, 0.61067E+00_JPRB, 0.61037E+00_JPRB, 0.62730E+00_JPRB, 0.65877E+00_JPRB ,&
&  0.92914E+00_JPRB, 0.19464E+01_JPRB, 0.11707E+01_JPRB, 0.13101E+01_JPRB, 0.89958E+00_JPRB, 0.70004E+00_JPRB, 0.55005E+00_JPRB ,&
&  0.55067E+00_JPRB, 0.67853E+00_JPRB, 0.68341E+00_JPRB, 0.66658E+00_JPRB, 0.61467E+00_JPRB, 0.59583E+00_JPRB, 0.61965E+00_JPRB ,&
&  0.63276E+00_JPRB, 0.61546E+00_JPRB, 0.61065E+00_JPRB, 0.62074E+00_JPRB, 0.62165E+00_JPRB, 0.61541E+00_JPRB, 0.61858E+00_JPRB ,&
&  0.61653E+00_JPRB, 0.61935E+00_JPRB, 0.61708E+00_JPRB, 0.59585E+00_JPRB, 0.61326E+00_JPRB, 0.69725E+00_JPRB, 0.78163E+00_JPRB ,&
&  0.76917E+00_JPRB, 0.89919E+00_JPRB, 0.81893E+00_JPRB, 0.78537E+00_JPRB, 0.75507E+00_JPRB, 0.75879E+00_JPRB, 0.75643E+00_JPRB ,&
&  0.73631E+00_JPRB, 0.72649E+00_JPRB, 0.70624E+00_JPRB, 0.70704E+00_JPRB, 0.74555E+00_JPRB, 0.75853E+00_JPRB, 0.74315E+00_JPRB ,&
&  0.76111E+00_JPRB, 0.76321E+00_JPRB, 0.76421E+00_JPRB, 0.74326E+00_JPRB, 0.74696E+00_JPRB, 0.74165E+00_JPRB, 0.72985E+00_JPRB ,&
&  0.72056E+00_JPRB, 0.71963E+00_JPRB, 0.70085E+00_JPRB, 0.69674E+00_JPRB, 0.69427E+00_JPRB, 0.68792E+00_JPRB, 0.66204E+00_JPRB ,&
&  0.67447E+00_JPRB, 0.68507E+00_JPRB, 0.69461E+00_JPRB, 0.68651E+00_JPRB, 0.68228E+00_JPRB, 0.70978E+00_JPRB, 0.71741E+00_JPRB ,&
&  0.73583E+00_JPRB, 0.73720E+00_JPRB, 0.70463E+00_JPRB, 0.70668E+00_JPRB, 0.71697E+00_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.66033E+00_JPRB, 0.63497E+00_JPRB, 0.61118E+00_JPRB, 0.56953E+00_JPRB, 0.57902E+00_JPRB, 0.59223E+00_JPRB, 0.57546E+00_JPRB ,&
&  0.53456E+00_JPRB, 0.53008E+00_JPRB, 0.54474E+00_JPRB, 0.56787E+00_JPRB, 0.58662E+00_JPRB, 0.57941E+00_JPRB, 0.58418E+00_JPRB ,&
&  0.57303E+00_JPRB, 0.56944E+00_JPRB, 0.58873E+00_JPRB, 0.62046E+00_JPRB, 0.62257E+00_JPRB, 0.61904E+00_JPRB, 0.62862E+00_JPRB ,&
&  0.63289E+00_JPRB, 0.61920E+00_JPRB, 0.60827E+00_JPRB, 0.60774E+00_JPRB, 0.62936E+00_JPRB, 0.66573E+00_JPRB, 0.71960E+00_JPRB ,&
&  0.99716E+00_JPRB, 0.21052E+01_JPRB, 0.16974E+01_JPRB, 0.12089E+01_JPRB, 0.83565E+00_JPRB, 0.62554E+00_JPRB, 0.51508E+00_JPRB ,&
&  0.48387E+00_JPRB, 0.50827E+00_JPRB, 0.67628E+00_JPRB, 0.64607E+00_JPRB, 0.59677E+00_JPRB, 0.57095E+00_JPRB, 0.61659E+00_JPRB ,&
&  0.64690E+00_JPRB, 0.64045E+00_JPRB, 0.64544E+00_JPRB, 0.65128E+00_JPRB, 0.65418E+00_JPRB, 0.64865E+00_JPRB, 0.63755E+00_JPRB ,&
&  0.63315E+00_JPRB, 0.65041E+00_JPRB, 0.66079E+00_JPRB, 0.63799E+00_JPRB, 0.63571E+00_JPRB, 0.70169E+00_JPRB, 0.78585E+00_JPRB ,&
&  0.79018E+00_JPRB, 0.82373E+00_JPRB, 0.79715E+00_JPRB, 0.80511E+00_JPRB, 0.72355E+00_JPRB, 0.76342E+00_JPRB, 0.73765E+00_JPRB ,&
&  0.71488E+00_JPRB, 0.68638E+00_JPRB, 0.68286E+00_JPRB, 0.69607E+00_JPRB, 0.71263E+00_JPRB, 0.72760E+00_JPRB, 0.73101E+00_JPRB ,&
&  0.74307E+00_JPRB, 0.75382E+00_JPRB, 0.75573E+00_JPRB, 0.74090E+00_JPRB, 0.73845E+00_JPRB, 0.74127E+00_JPRB, 0.75312E+00_JPRB ,&
&  0.75184E+00_JPRB, 0.75887E+00_JPRB, 0.72823E+00_JPRB, 0.71976E+00_JPRB, 0.71453E+00_JPRB, 0.70160E+00_JPRB, 0.66493E+00_JPRB ,&
&  0.66919E+00_JPRB, 0.69882E+00_JPRB, 0.70547E+00_JPRB, 0.69507E+00_JPRB, 0.69488E+00_JPRB, 0.71050E+00_JPRB, 0.71669E+00_JPRB ,&
&  0.74069E+00_JPRB, 0.74709E+00_JPRB, 0.71016E+00_JPRB, 0.68976E+00_JPRB, 0.69857E+00_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.65859E+00_JPRB, 0.63609E+00_JPRB, 0.58605E+00_JPRB, 0.56186E+00_JPRB, 0.56022E+00_JPRB, 0.55450E+00_JPRB, 0.54217E+00_JPRB ,&
&  0.52008E+00_JPRB, 0.51385E+00_JPRB, 0.54362E+00_JPRB, 0.57201E+00_JPRB, 0.59779E+00_JPRB, 0.59410E+00_JPRB, 0.59180E+00_JPRB ,&
&  0.59205E+00_JPRB, 0.61177E+00_JPRB, 0.65024E+00_JPRB, 0.65616E+00_JPRB, 0.63968E+00_JPRB, 0.62954E+00_JPRB, 0.62511E+00_JPRB ,&
&  0.62520E+00_JPRB, 0.62464E+00_JPRB, 0.63658E+00_JPRB, 0.65045E+00_JPRB, 0.67961E+00_JPRB, 0.73169E+00_JPRB, 0.84499E+00_JPRB ,&
&  0.11258E+01_JPRB, 0.18023E+01_JPRB, 0.57162E+00_JPRB, 0.69538E+00_JPRB, 0.64080E+00_JPRB, 0.55235E+00_JPRB, 0.49126E+00_JPRB ,&
&  0.44806E+00_JPRB, 0.45998E+00_JPRB, 0.57582E+00_JPRB, 0.62885E+00_JPRB, 0.57426E+00_JPRB, 0.60474E+00_JPRB, 0.66570E+00_JPRB ,&
&  0.71073E+00_JPRB, 0.70672E+00_JPRB, 0.69677E+00_JPRB, 0.68558E+00_JPRB, 0.68277E+00_JPRB, 0.67449E+00_JPRB, 0.67141E+00_JPRB ,&
&  0.67981E+00_JPRB, 0.70710E+00_JPRB, 0.72209E+00_JPRB, 0.64752E+00_JPRB, 0.69728E+00_JPRB, 0.73385E+00_JPRB, 0.73008E+00_JPRB ,&
&  0.68286E+00_JPRB, 0.75329E+00_JPRB, 0.79725E+00_JPRB, 0.83723E+00_JPRB, 0.71158E+00_JPRB, 0.63839E+00_JPRB, 0.72368E+00_JPRB ,&
&  0.71836E+00_JPRB, 0.69890E+00_JPRB, 0.67861E+00_JPRB, 0.66560E+00_JPRB, 0.66013E+00_JPRB, 0.69731E+00_JPRB, 0.71984E+00_JPRB ,&
&  0.71455E+00_JPRB, 0.73733E+00_JPRB, 0.74192E+00_JPRB, 0.72981E+00_JPRB, 0.73497E+00_JPRB, 0.74628E+00_JPRB, 0.77757E+00_JPRB ,&
&  0.78776E+00_JPRB, 0.79851E+00_JPRB, 0.80460E+00_JPRB, 0.78519E+00_JPRB, 0.78461E+00_JPRB, 0.74806E+00_JPRB, 0.70162E+00_JPRB ,&
&  0.69788E+00_JPRB, 0.71082E+00_JPRB, 0.69335E+00_JPRB, 0.66695E+00_JPRB, 0.69607E+00_JPRB, 0.70160E+00_JPRB, 0.70797E+00_JPRB ,&
&  0.76156E+00_JPRB, 0.79366E+00_JPRB, 0.78936E+00_JPRB, 0.67765E+00_JPRB, 0.69254E+00_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.65433E+00_JPRB, 0.60594E+00_JPRB, 0.57209E+00_JPRB, 0.56680E+00_JPRB, 0.55542E+00_JPRB, 0.56082E+00_JPRB, 0.56825E+00_JPRB ,&
&  0.56433E+00_JPRB, 0.57535E+00_JPRB, 0.57859E+00_JPRB, 0.58886E+00_JPRB, 0.60757E+00_JPRB, 0.60893E+00_JPRB, 0.62559E+00_JPRB ,&
&  0.64481E+00_JPRB, 0.67011E+00_JPRB, 0.68884E+00_JPRB, 0.64844E+00_JPRB, 0.63428E+00_JPRB, 0.62756E+00_JPRB, 0.62782E+00_JPRB ,&
&  0.64047E+00_JPRB, 0.66162E+00_JPRB, 0.68582E+00_JPRB, 0.71856E+00_JPRB, 0.76905E+00_JPRB, 0.86283E+00_JPRB, 0.10379E+01_JPRB ,&
&  0.14835E+01_JPRB, 0.11079E+01_JPRB, 0.43987E+00_JPRB, 0.53964E+00_JPRB, 0.52919E+00_JPRB, 0.50636E+00_JPRB, 0.43972E+00_JPRB ,&
&  0.42586E+00_JPRB, 0.47335E+00_JPRB, 0.54267E+00_JPRB, 0.60955E+00_JPRB, 0.63853E+00_JPRB, 0.66786E+00_JPRB, 0.70946E+00_JPRB ,&
&  0.73024E+00_JPRB, 0.73417E+00_JPRB, 0.72498E+00_JPRB, 0.72194E+00_JPRB, 0.72083E+00_JPRB, 0.72159E+00_JPRB, 0.72471E+00_JPRB ,&
&  0.73916E+00_JPRB, 0.76442E+00_JPRB, 0.78744E+00_JPRB, 0.69371E+00_JPRB, 0.74396E+00_JPRB, 0.72373E+00_JPRB, 0.68970E+00_JPRB ,&
&  0.65591E+00_JPRB, 0.72015E+00_JPRB, 0.77295E+00_JPRB, 0.79322E+00_JPRB, 0.64458E+00_JPRB, 0.58092E+00_JPRB, 0.69252E+00_JPRB ,&
&  0.74702E+00_JPRB, 0.70306E+00_JPRB, 0.68281E+00_JPRB, 0.67146E+00_JPRB, 0.63142E+00_JPRB, 0.66763E+00_JPRB, 0.71862E+00_JPRB ,&
&  0.71457E+00_JPRB, 0.71716E+00_JPRB, 0.74138E+00_JPRB, 0.73083E+00_JPRB, 0.72739E+00_JPRB, 0.74006E+00_JPRB, 0.77219E+00_JPRB ,&
&  0.79226E+00_JPRB, 0.81101E+00_JPRB, 0.81412E+00_JPRB, 0.84308E+00_JPRB, 0.86864E+00_JPRB, 0.80412E+00_JPRB, 0.72062E+00_JPRB ,&
&  0.69973E+00_JPRB, 0.70746E+00_JPRB, 0.69409E+00_JPRB, 0.68865E+00_JPRB, 0.68641E+00_JPRB, 0.68786E+00_JPRB, 0.70939E+00_JPRB ,&
&  0.80581E+00_JPRB, 0.75803E+00_JPRB, 0.71142E+00_JPRB, 0.64828E+00_JPRB, 0.67151E+00_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.59438E+00_JPRB, 0.56859E+00_JPRB, 0.55991E+00_JPRB, 0.54538E+00_JPRB, 0.55868E+00_JPRB, 0.58256E+00_JPRB, 0.60082E+00_JPRB ,&
&  0.60250E+00_JPRB, 0.60015E+00_JPRB, 0.60028E+00_JPRB, 0.61571E+00_JPRB, 0.63780E+00_JPRB, 0.66012E+00_JPRB, 0.68858E+00_JPRB ,&
&  0.70544E+00_JPRB, 0.71419E+00_JPRB, 0.68256E+00_JPRB, 0.63806E+00_JPRB, 0.63550E+00_JPRB, 0.64037E+00_JPRB, 0.65307E+00_JPRB ,&
&  0.67585E+00_JPRB, 0.69943E+00_JPRB, 0.73100E+00_JPRB, 0.79341E+00_JPRB, 0.89177E+00_JPRB, 0.10049E+01_JPRB, 0.11133E+01_JPRB ,&
&  0.83383E+00_JPRB, 0.40665E+00_JPRB, 0.41326E+00_JPRB, 0.47789E+00_JPRB, 0.43994E+00_JPRB, 0.44277E+00_JPRB, 0.44406E+00_JPRB ,&
&  0.44124E+00_JPRB, 0.48898E+00_JPRB, 0.53260E+00_JPRB, 0.64505E+00_JPRB, 0.70685E+00_JPRB, 0.72054E+00_JPRB, 0.73842E+00_JPRB ,&
&  0.75272E+00_JPRB, 0.76128E+00_JPRB, 0.75681E+00_JPRB, 0.75115E+00_JPRB, 0.75360E+00_JPRB, 0.75879E+00_JPRB, 0.76634E+00_JPRB ,&
&  0.78646E+00_JPRB, 0.81120E+00_JPRB, 0.85568E+00_JPRB, 0.71736E+00_JPRB, 0.73781E+00_JPRB, 0.71900E+00_JPRB, 0.66458E+00_JPRB ,&
&  0.63415E+00_JPRB, 0.66465E+00_JPRB, 0.67650E+00_JPRB, 0.76167E+00_JPRB, 0.70842E+00_JPRB, 0.59386E+00_JPRB, 0.71181E+00_JPRB ,&
&  0.75801E+00_JPRB, 0.76991E+00_JPRB, 0.75935E+00_JPRB, 0.68694E+00_JPRB, 0.61900E+00_JPRB, 0.60781E+00_JPRB, 0.65631E+00_JPRB ,&
&  0.67406E+00_JPRB, 0.68618E+00_JPRB, 0.69135E+00_JPRB, 0.69396E+00_JPRB, 0.67801E+00_JPRB, 0.68741E+00_JPRB, 0.71846E+00_JPRB ,&
&  0.75151E+00_JPRB, 0.78344E+00_JPRB, 0.79699E+00_JPRB, 0.82273E+00_JPRB, 0.85893E+00_JPRB, 0.80740E+00_JPRB, 0.72898E+00_JPRB ,&
&  0.70882E+00_JPRB, 0.70183E+00_JPRB, 0.67846E+00_JPRB, 0.67485E+00_JPRB, 0.68850E+00_JPRB, 0.74784E+00_JPRB, 0.86648E+00_JPRB ,&
&  0.86189E+00_JPRB, 0.76306E+00_JPRB, 0.65646E+00_JPRB, 0.62662E+00_JPRB, 0.63513E+00_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.50018E+00_JPRB, 0.49233E+00_JPRB, 0.49284E+00_JPRB, 0.49086E+00_JPRB, 0.52230E+00_JPRB, 0.55626E+00_JPRB, 0.56574E+00_JPRB ,&
&  0.55855E+00_JPRB, 0.57360E+00_JPRB, 0.58817E+00_JPRB, 0.61564E+00_JPRB, 0.67369E+00_JPRB, 0.69500E+00_JPRB, 0.72061E+00_JPRB ,&
&  0.74578E+00_JPRB, 0.73564E+00_JPRB, 0.69614E+00_JPRB, 0.66761E+00_JPRB, 0.67320E+00_JPRB, 0.67090E+00_JPRB, 0.68265E+00_JPRB ,&
&  0.70674E+00_JPRB, 0.73197E+00_JPRB, 0.76299E+00_JPRB, 0.80646E+00_JPRB, 0.85700E+00_JPRB, 0.88591E+00_JPRB, 0.83253E+00_JPRB ,&
&  0.36524E+00_JPRB, 0.35856E+00_JPRB, 0.46915E+00_JPRB, 0.47295E+00_JPRB, 0.43592E+00_JPRB, 0.43901E+00_JPRB, 0.45406E+00_JPRB ,&
&  0.47288E+00_JPRB, 0.47505E+00_JPRB, 0.53111E+00_JPRB, 0.71282E+00_JPRB, 0.71552E+00_JPRB, 0.75149E+00_JPRB, 0.74373E+00_JPRB ,&
&  0.75533E+00_JPRB, 0.76736E+00_JPRB, 0.77052E+00_JPRB, 0.76882E+00_JPRB, 0.77329E+00_JPRB, 0.77963E+00_JPRB, 0.78933E+00_JPRB ,&
&  0.80339E+00_JPRB, 0.85369E+00_JPRB, 0.90837E+00_JPRB, 0.74253E+00_JPRB, 0.70906E+00_JPRB, 0.67822E+00_JPRB, 0.61241E+00_JPRB ,&
&  0.55899E+00_JPRB, 0.61313E+00_JPRB, 0.63595E+00_JPRB, 0.71141E+00_JPRB, 0.72859E+00_JPRB, 0.77209E+00_JPRB, 0.79030E+00_JPRB ,&
&  0.77039E+00_JPRB, 0.75121E+00_JPRB, 0.71694E+00_JPRB, 0.65292E+00_JPRB, 0.58582E+00_JPRB, 0.57911E+00_JPRB, 0.57865E+00_JPRB ,&
&  0.60762E+00_JPRB, 0.61628E+00_JPRB, 0.62561E+00_JPRB, 0.63703E+00_JPRB, 0.59593E+00_JPRB, 0.59552E+00_JPRB, 0.62110E+00_JPRB ,&
&  0.65047E+00_JPRB, 0.67544E+00_JPRB, 0.68378E+00_JPRB, 0.69243E+00_JPRB, 0.71099E+00_JPRB, 0.72235E+00_JPRB, 0.69355E+00_JPRB ,&
&  0.66028E+00_JPRB, 0.64065E+00_JPRB, 0.65890E+00_JPRB, 0.66614E+00_JPRB, 0.84490E+00_JPRB, 0.11565E+01_JPRB, 0.12030E+01_JPRB ,&
&  0.10232E+01_JPRB, 0.77834E+00_JPRB, 0.61793E+00_JPRB, 0.56837E+00_JPRB, 0.54803E+00_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.45899E+00_JPRB, 0.44850E+00_JPRB, 0.44998E+00_JPRB, 0.44998E+00_JPRB, 0.48383E+00_JPRB, 0.50638E+00_JPRB, 0.52638E+00_JPRB ,&
&  0.55557E+00_JPRB, 0.57362E+00_JPRB, 0.60416E+00_JPRB, 0.65833E+00_JPRB, 0.71420E+00_JPRB, 0.75462E+00_JPRB, 0.76687E+00_JPRB ,&
&  0.76827E+00_JPRB, 0.75681E+00_JPRB, 0.73144E+00_JPRB, 0.69955E+00_JPRB, 0.69584E+00_JPRB, 0.68260E+00_JPRB, 0.68969E+00_JPRB ,&
&  0.70906E+00_JPRB, 0.72944E+00_JPRB, 0.75756E+00_JPRB, 0.78308E+00_JPRB, 0.80309E+00_JPRB, 0.80251E+00_JPRB, 0.73736E+00_JPRB ,&
&  0.37563E+00_JPRB, 0.54902E+00_JPRB, 0.58177E+00_JPRB, 0.56474E+00_JPRB, 0.51720E+00_JPRB, 0.47980E+00_JPRB, 0.47834E+00_JPRB ,&
&  0.49666E+00_JPRB, 0.49408E+00_JPRB, 0.53598E+00_JPRB, 0.60785E+00_JPRB, 0.68997E+00_JPRB, 0.72148E+00_JPRB, 0.72967E+00_JPRB ,&
&  0.74013E+00_JPRB, 0.74345E+00_JPRB, 0.75536E+00_JPRB, 0.76749E+00_JPRB, 0.78105E+00_JPRB, 0.78926E+00_JPRB, 0.80369E+00_JPRB ,&
&  0.82201E+00_JPRB, 0.86196E+00_JPRB, 0.92930E+00_JPRB, 0.81868E+00_JPRB, 0.70440E+00_JPRB, 0.65620E+00_JPRB, 0.57245E+00_JPRB ,&
&  0.49843E+00_JPRB, 0.52195E+00_JPRB, 0.60021E+00_JPRB, 0.73839E+00_JPRB, 0.77832E+00_JPRB, 0.79797E+00_JPRB, 0.79317E+00_JPRB ,&
&  0.73303E+00_JPRB, 0.69756E+00_JPRB, 0.64545E+00_JPRB, 0.60467E+00_JPRB, 0.56700E+00_JPRB, 0.54495E+00_JPRB, 0.53050E+00_JPRB ,&
&  0.52695E+00_JPRB, 0.54245E+00_JPRB, 0.55598E+00_JPRB, 0.54935E+00_JPRB, 0.51720E+00_JPRB, 0.51259E+00_JPRB, 0.54113E+00_JPRB ,&
&  0.59702E+00_JPRB, 0.53343E+00_JPRB, 0.49637E+00_JPRB, 0.49700E+00_JPRB, 0.50128E+00_JPRB, 0.50558E+00_JPRB, 0.50391E+00_JPRB ,&
&  0.51906E+00_JPRB, 0.52376E+00_JPRB, 0.63279E+00_JPRB, 0.10083E+01_JPRB, 0.14156E+01_JPRB, 0.16719E+01_JPRB, 0.13739E+01_JPRB ,&
&  0.96091E+00_JPRB, 0.70101E+00_JPRB, 0.55483E+00_JPRB, 0.51616E+00_JPRB, 0.49523E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.48056E+00_JPRB, 0.46396E+00_JPRB, 0.44889E+00_JPRB, 0.46145E+00_JPRB, 0.46979E+00_JPRB, 0.47467E+00_JPRB, 0.49929E+00_JPRB ,&
&  0.55117E+00_JPRB, 0.56402E+00_JPRB, 0.59019E+00_JPRB, 0.63109E+00_JPRB, 0.68988E+00_JPRB, 0.73101E+00_JPRB, 0.76800E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.82060E+00_JPRB, 0.79971E+00_JPRB, 0.77245E+00_JPRB, 0.75694E+00_JPRB, 0.74886E+00_JPRB, 0.74827E+00_JPRB ,&
&  0.73905E+00_JPRB, 0.73917E+00_JPRB, 0.75044E+00_JPRB, 0.76498E+00_JPRB, 0.77069E+00_JPRB, 0.73614E+00_JPRB, 0.45419E+00_JPRB ,&
&  0.62609E+00_JPRB, 0.67505E+00_JPRB, 0.69501E+00_JPRB, 0.66342E+00_JPRB, 0.60837E+00_JPRB, 0.54712E+00_JPRB, 0.51928E+00_JPRB ,&
&  0.52023E+00_JPRB, 0.53054E+00_JPRB, 0.53688E+00_JPRB, 0.55964E+00_JPRB, 0.64479E+00_JPRB, 0.67606E+00_JPRB, 0.69365E+00_JPRB ,&
&  0.70258E+00_JPRB, 0.72206E+00_JPRB, 0.73270E+00_JPRB, 0.74372E+00_JPRB, 0.75914E+00_JPRB, 0.76878E+00_JPRB, 0.77368E+00_JPRB ,&
&  0.77826E+00_JPRB, 0.77266E+00_JPRB, 0.78233E+00_JPRB, 0.72848E+00_JPRB, 0.65351E+00_JPRB, 0.56823E+00_JPRB, 0.51403E+00_JPRB ,&
&  0.43286E+00_JPRB, 0.42825E+00_JPRB, 0.57648E+00_JPRB, 0.78615E+00_JPRB, 0.78630E+00_JPRB, 0.79649E+00_JPRB, 0.75053E+00_JPRB ,&
&  0.67911E+00_JPRB, 0.61482E+00_JPRB, 0.58359E+00_JPRB, 0.56345E+00_JPRB, 0.54217E+00_JPRB, 0.51332E+00_JPRB, 0.49085E+00_JPRB ,&
&  0.46644E+00_JPRB, 0.45037E+00_JPRB, 0.46296E+00_JPRB, 0.45265E+00_JPRB, 0.45794E+00_JPRB, 0.49207E+00_JPRB, 0.59706E+00_JPRB ,&
&  0.89740E+00_JPRB, 0.48132E+00_JPRB, 0.42105E+00_JPRB, 0.41171E+00_JPRB, 0.39940E+00_JPRB, 0.39096E+00_JPRB, 0.38788E+00_JPRB ,&
&  0.40333E+00_JPRB, 0.46408E+00_JPRB, 0.61692E+00_JPRB, 0.13399E+01_JPRB, 0.24991E+01_JPRB, 0.25642E+01_JPRB, 0.17086E+01_JPRB ,&
&  0.10153E+01_JPRB, 0.68903E+00_JPRB, 0.54964E+00_JPRB, 0.51315E+00_JPRB, 0.48681E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.52311E+00_JPRB, 0.49351E+00_JPRB, 0.48591E+00_JPRB, 0.48407E+00_JPRB, 0.47499E+00_JPRB, 0.49313E+00_JPRB, 0.50682E+00_JPRB ,&
&  0.52590E+00_JPRB, 0.55152E+00_JPRB, 0.58319E+00_JPRB, 0.61085E+00_JPRB, 0.65161E+00_JPRB, 0.68990E+00_JPRB, 0.71827E+00_JPRB ,&
&  0.73912E+00_JPRB, 0.75239E+00_JPRB, 0.75761E+00_JPRB, 0.75617E+00_JPRB, 0.74791E+00_JPRB, 0.73917E+00_JPRB, 0.72765E+00_JPRB ,&
&  0.71713E+00_JPRB, 0.71934E+00_JPRB, 0.73576E+00_JPRB, 0.75415E+00_JPRB, 0.75396E+00_JPRB, 0.73307E+00_JPRB, 0.46849E+00_JPRB ,&
&  0.77798E+00_JPRB, 0.80571E+00_JPRB, 0.77511E+00_JPRB, 0.70264E+00_JPRB, 0.64247E+00_JPRB, 0.60115E+00_JPRB, 0.56414E+00_JPRB ,&
&  0.54079E+00_JPRB, 0.53750E+00_JPRB, 0.49702E+00_JPRB, 0.52255E+00_JPRB, 0.58517E+00_JPRB, 0.60341E+00_JPRB, 0.63358E+00_JPRB ,&
&  0.65587E+00_JPRB, 0.67646E+00_JPRB, 0.69346E+00_JPRB, 0.71360E+00_JPRB, 0.72595E+00_JPRB, 0.74087E+00_JPRB, 0.73592E+00_JPRB ,&
&  0.72883E+00_JPRB, 0.70971E+00_JPRB, 0.67617E+00_JPRB, 0.60998E+00_JPRB, 0.60419E+00_JPRB, 0.53302E+00_JPRB, 0.50562E+00_JPRB ,&
&  0.40317E+00_JPRB, 0.43487E+00_JPRB, 0.65782E+00_JPRB, 0.85411E+00_JPRB, 0.79148E+00_JPRB, 0.77651E+00_JPRB, 0.70530E+00_JPRB ,&
&  0.63935E+00_JPRB, 0.59634E+00_JPRB, 0.57590E+00_JPRB, 0.55977E+00_JPRB, 0.52830E+00_JPRB, 0.50427E+00_JPRB, 0.49279E+00_JPRB ,&
&  0.48684E+00_JPRB, 0.44225E+00_JPRB, 0.42041E+00_JPRB, 0.41157E+00_JPRB, 0.39250E+00_JPRB, 0.40926E+00_JPRB, 0.38167E+00_JPRB ,&
&  0.49298E+00_JPRB, 0.46778E+00_JPRB, 0.42247E+00_JPRB, 0.44079E+00_JPRB, 0.42435E+00_JPRB, 0.42450E+00_JPRB, 0.43461E+00_JPRB ,&
&  0.46559E+00_JPRB, 0.42184E+00_JPRB, 0.91389E+00_JPRB, 0.36857E+01_JPRB, 0.36491E+01_JPRB, 0.46397E+01_JPRB, 0.23956E+01_JPRB ,&
&  0.11101E+01_JPRB, 0.66560E+00_JPRB, 0.56637E+00_JPRB, 0.55534E+00_JPRB, 0.53622E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.53685E+00_JPRB, 0.52930E+00_JPRB, 0.51381E+00_JPRB, 0.48752E+00_JPRB, 0.47755E+00_JPRB, 0.49590E+00_JPRB, 0.52039E+00_JPRB ,&
&  0.53757E+00_JPRB, 0.55940E+00_JPRB, 0.58268E+00_JPRB, 0.60786E+00_JPRB, 0.63587E+00_JPRB, 0.66153E+00_JPRB, 0.68124E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.71196E+00_JPRB, 0.72624E+00_JPRB, 0.73634E+00_JPRB, 0.73595E+00_JPRB, 0.73511E+00_JPRB, 0.73052E+00_JPRB ,&
&  0.75716E+00_JPRB, 0.78040E+00_JPRB, 0.80945E+00_JPRB, 0.83565E+00_JPRB, 0.84256E+00_JPRB, 0.79463E+00_JPRB, 0.59325E+00_JPRB ,&
&  0.88924E+00_JPRB, 0.90113E+00_JPRB, 0.81182E+00_JPRB, 0.72128E+00_JPRB, 0.65816E+00_JPRB, 0.61774E+00_JPRB, 0.57954E+00_JPRB ,&
&  0.55200E+00_JPRB, 0.56363E+00_JPRB, 0.53396E+00_JPRB, 0.50703E+00_JPRB, 0.50871E+00_JPRB, 0.52968E+00_JPRB, 0.56215E+00_JPRB ,&
&  0.60640E+00_JPRB, 0.64862E+00_JPRB, 0.68844E+00_JPRB, 0.71424E+00_JPRB, 0.73410E+00_JPRB, 0.75265E+00_JPRB, 0.72794E+00_JPRB ,&
&  0.72012E+00_JPRB, 0.71932E+00_JPRB, 0.66546E+00_JPRB, 0.63958E+00_JPRB, 0.64998E+00_JPRB, 0.60401E+00_JPRB, 0.55937E+00_JPRB ,&
&  0.46366E+00_JPRB, 0.56206E+00_JPRB, 0.76876E+00_JPRB, 0.87404E+00_JPRB, 0.76845E+00_JPRB, 0.75004E+00_JPRB, 0.69699E+00_JPRB ,&
&  0.63447E+00_JPRB, 0.61411E+00_JPRB, 0.60708E+00_JPRB, 0.59285E+00_JPRB, 0.55350E+00_JPRB, 0.52700E+00_JPRB, 0.52257E+00_JPRB ,&
&  0.52532E+00_JPRB, 0.47687E+00_JPRB, 0.43486E+00_JPRB, 0.42645E+00_JPRB, 0.40294E+00_JPRB, 0.35055E+00_JPRB, 0.43291E+00_JPRB ,&
&  0.42045E+00_JPRB, 0.35281E+00_JPRB, 0.43506E+00_JPRB, 0.35730E+00_JPRB, 0.46602E+00_JPRB, 0.47598E+00_JPRB, 0.50480E+00_JPRB ,&
&  0.43884E+00_JPRB, 0.50615E+00_JPRB, 0.85967E+00_JPRB, 0.11869E+01_JPRB, 0.13674E+01_JPRB, 0.13494E+01_JPRB, 0.74252E+00_JPRB ,&
&  0.64282E+00_JPRB, 0.61205E+00_JPRB, 0.58931E+00_JPRB, 0.56623E+00_JPRB, 0.54108E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.55375E+00_JPRB, 0.55418E+00_JPRB, 0.53674E+00_JPRB, 0.51338E+00_JPRB, 0.50235E+00_JPRB, 0.51302E+00_JPRB, 0.52869E+00_JPRB ,&
&  0.54271E+00_JPRB, 0.56181E+00_JPRB, 0.58396E+00_JPRB, 0.60520E+00_JPRB, 0.62395E+00_JPRB, 0.64381E+00_JPRB, 0.66713E+00_JPRB ,&
&  0.68907E+00_JPRB, 0.71638E+00_JPRB, 0.74325E+00_JPRB, 0.76917E+00_JPRB, 0.79050E+00_JPRB, 0.82918E+00_JPRB, 0.85297E+00_JPRB ,&
&  0.89462E+00_JPRB, 0.93629E+00_JPRB, 0.98531E+00_JPRB, 0.10288E+01_JPRB, 0.10790E+01_JPRB, 0.11069E+01_JPRB, 0.88844E+00_JPRB ,&
&  0.97605E+00_JPRB, 0.93901E+00_JPRB, 0.82955E+00_JPRB, 0.73886E+00_JPRB, 0.68740E+00_JPRB, 0.65320E+00_JPRB, 0.62296E+00_JPRB ,&
&  0.61178E+00_JPRB, 0.58335E+00_JPRB, 0.55137E+00_JPRB, 0.53288E+00_JPRB, 0.52348E+00_JPRB, 0.54128E+00_JPRB, 0.53704E+00_JPRB ,&
&  0.57381E+00_JPRB, 0.63996E+00_JPRB, 0.68553E+00_JPRB, 0.75382E+00_JPRB, 0.81700E+00_JPRB, 0.86611E+00_JPRB, 0.87039E+00_JPRB ,&
&  0.87273E+00_JPRB, 0.89970E+00_JPRB, 0.87148E+00_JPRB, 0.87783E+00_JPRB, 0.86972E+00_JPRB, 0.85797E+00_JPRB, 0.83521E+00_JPRB ,&
&  0.70343E+00_JPRB, 0.71735E+00_JPRB, 0.78323E+00_JPRB, 0.84719E+00_JPRB, 0.76713E+00_JPRB, 0.72975E+00_JPRB, 0.70951E+00_JPRB ,&
&  0.64754E+00_JPRB, 0.63522E+00_JPRB, 0.63794E+00_JPRB, 0.61639E+00_JPRB, 0.60085E+00_JPRB, 0.58208E+00_JPRB, 0.57695E+00_JPRB ,&
&  0.58012E+00_JPRB, 0.51834E+00_JPRB, 0.48015E+00_JPRB, 0.45398E+00_JPRB, 0.44418E+00_JPRB, 0.41783E+00_JPRB, 0.44605E+00_JPRB ,&
&  0.44578E+00_JPRB, 0.34016E+00_JPRB, 0.40797E+00_JPRB, 0.53183E+00_JPRB, 0.54156E+00_JPRB, 0.57179E+00_JPRB, 0.61547E+00_JPRB ,&
&  0.66232E+00_JPRB, 0.68352E+00_JPRB, 0.66933E+00_JPRB, 0.68549E+00_JPRB, 0.71156E+00_JPRB, 0.69434E+00_JPRB, 0.63483E+00_JPRB ,&
&  0.63907E+00_JPRB, 0.60294E+00_JPRB, 0.58216E+00_JPRB, 0.57265E+00_JPRB, 0.56444E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.56509E+00_JPRB, 0.57330E+00_JPRB, 0.56433E+00_JPRB, 0.54765E+00_JPRB, 0.53456E+00_JPRB, 0.53089E+00_JPRB, 0.53919E+00_JPRB ,&
&  0.54866E+00_JPRB, 0.56790E+00_JPRB, 0.58276E+00_JPRB, 0.59913E+00_JPRB, 0.60736E+00_JPRB, 0.61377E+00_JPRB, 0.62686E+00_JPRB ,&
&  0.64413E+00_JPRB, 0.66307E+00_JPRB, 0.68583E+00_JPRB, 0.71736E+00_JPRB, 0.74402E+00_JPRB, 0.77647E+00_JPRB, 0.78548E+00_JPRB ,&
&  0.77893E+00_JPRB, 0.79501E+00_JPRB, 0.80159E+00_JPRB, 0.82280E+00_JPRB, 0.96621E+00_JPRB, 0.11056E+01_JPRB, 0.12236E+01_JPRB ,&
&  0.92799E+00_JPRB, 0.93218E+00_JPRB, 0.81960E+00_JPRB, 0.73713E+00_JPRB, 0.73639E+00_JPRB, 0.71765E+00_JPRB, 0.71799E+00_JPRB ,&
&  0.72128E+00_JPRB, 0.71146E+00_JPRB, 0.70848E+00_JPRB, 0.71138E+00_JPRB, 0.71997E+00_JPRB, 0.74592E+00_JPRB, 0.72002E+00_JPRB ,&
&  0.70858E+00_JPRB, 0.73862E+00_JPRB, 0.77416E+00_JPRB, 0.86722E+00_JPRB, 0.99255E+00_JPRB, 0.10911E+01_JPRB, 0.11403E+01_JPRB ,&
&  0.11578E+01_JPRB, 0.12012E+01_JPRB, 0.11373E+01_JPRB, 0.11312E+01_JPRB, 0.11212E+01_JPRB, 0.11061E+01_JPRB, 0.10961E+01_JPRB ,&
&  0.10130E+01_JPRB, 0.95728E+00_JPRB, 0.90121E+00_JPRB, 0.78585E+00_JPRB, 0.74883E+00_JPRB, 0.74392E+00_JPRB, 0.72522E+00_JPRB ,&
&  0.69776E+00_JPRB, 0.68773E+00_JPRB, 0.68463E+00_JPRB, 0.66279E+00_JPRB, 0.65792E+00_JPRB, 0.65017E+00_JPRB, 0.65066E+00_JPRB ,&
&  0.64062E+00_JPRB, 0.58326E+00_JPRB, 0.54334E+00_JPRB, 0.50459E+00_JPRB, 0.45844E+00_JPRB, 0.53230E+00_JPRB, 0.51762E+00_JPRB ,&
&  0.51098E+00_JPRB, 0.44047E+00_JPRB, 0.44046E+00_JPRB, 0.62018E+00_JPRB, 0.63205E+00_JPRB, 0.63991E+00_JPRB, 0.65467E+00_JPRB ,&
&  0.67593E+00_JPRB, 0.68850E+00_JPRB, 0.69541E+00_JPRB, 0.67861E+00_JPRB, 0.67305E+00_JPRB, 0.66804E+00_JPRB, 0.64949E+00_JPRB ,&
&  0.63533E+00_JPRB, 0.61826E+00_JPRB, 0.61410E+00_JPRB, 0.58356E+00_JPRB, 0.56726E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.56511E+00_JPRB, 0.56199E+00_JPRB, 0.57456E+00_JPRB, 0.56280E+00_JPRB, 0.58534E+00_JPRB, 0.57722E+00_JPRB, 0.60429E+00_JPRB ,&
&  0.58080E+00_JPRB, 0.57468E+00_JPRB, 0.57424E+00_JPRB, 0.58354E+00_JPRB, 0.59412E+00_JPRB, 0.61038E+00_JPRB, 0.62238E+00_JPRB ,&
&  0.64372E+00_JPRB, 0.68441E+00_JPRB, 0.73895E+00_JPRB, 0.79968E+00_JPRB, 0.87836E+00_JPRB, 0.95284E+00_JPRB, 0.10121E+01_JPRB ,&
&  0.10935E+01_JPRB, 0.11569E+01_JPRB, 0.11778E+01_JPRB, 0.12660E+01_JPRB, 0.15730E+01_JPRB, 0.19595E+01_JPRB, 0.19562E+01_JPRB ,&
&  0.12432E+01_JPRB, 0.85464E+00_JPRB, 0.78868E+00_JPRB, 0.72888E+00_JPRB, 0.74262E+00_JPRB, 0.79927E+00_JPRB, 0.81136E+00_JPRB ,&
&  0.81617E+00_JPRB, 0.82698E+00_JPRB, 0.85210E+00_JPRB, 0.87743E+00_JPRB, 0.91680E+00_JPRB, 0.97135E+00_JPRB, 0.98715E+00_JPRB ,&
&  0.98164E+00_JPRB, 0.10019E+01_JPRB, 0.10160E+01_JPRB, 0.10954E+01_JPRB, 0.11682E+01_JPRB, 0.12592E+01_JPRB, 0.13835E+01_JPRB ,&
&  0.14478E+01_JPRB, 0.13782E+01_JPRB, 0.11276E+01_JPRB, 0.11153E+01_JPRB, 0.11971E+01_JPRB, 0.11923E+01_JPRB, 0.11217E+01_JPRB ,&
&  0.12130E+01_JPRB, 0.11880E+01_JPRB, 0.83999E+00_JPRB, 0.74648E+00_JPRB, 0.73225E+00_JPRB, 0.76066E+00_JPRB, 0.74426E+00_JPRB ,&
&  0.74999E+00_JPRB, 0.74626E+00_JPRB, 0.74702E+00_JPRB, 0.73261E+00_JPRB, 0.71730E+00_JPRB, 0.70775E+00_JPRB, 0.71338E+00_JPRB ,&
&  0.66965E+00_JPRB, 0.64212E+00_JPRB, 0.62191E+00_JPRB, 0.58684E+00_JPRB, 0.60563E+00_JPRB, 0.55296E+00_JPRB, 0.57926E+00_JPRB ,&
&  0.55476E+00_JPRB, 0.56800E+00_JPRB, 0.52135E+00_JPRB, 0.64783E+00_JPRB, 0.70707E+00_JPRB, 0.68739E+00_JPRB, 0.70344E+00_JPRB ,&
&  0.71197E+00_JPRB, 0.72690E+00_JPRB, 0.71429E+00_JPRB, 0.71033E+00_JPRB, 0.71543E+00_JPRB, 0.70622E+00_JPRB, 0.69225E+00_JPRB ,&
&  0.67826E+00_JPRB, 0.66317E+00_JPRB, 0.63513E+00_JPRB, 0.60786E+00_JPRB, 0.58317E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.60454E+00_JPRB, 0.60500E+00_JPRB, 0.61320E+00_JPRB, 0.62553E+00_JPRB, 0.62491E+00_JPRB, 0.62299E+00_JPRB, 0.61789E+00_JPRB ,&
&  0.59191E+00_JPRB, 0.58902E+00_JPRB, 0.57875E+00_JPRB, 0.58489E+00_JPRB, 0.60728E+00_JPRB, 0.62881E+00_JPRB, 0.66681E+00_JPRB ,&
&  0.70647E+00_JPRB, 0.75881E+00_JPRB, 0.83840E+00_JPRB, 0.10050E+01_JPRB, 0.11855E+01_JPRB, 0.13082E+01_JPRB, 0.14518E+01_JPRB ,&
&  0.14868E+01_JPRB, 0.15114E+01_JPRB, 0.15851E+01_JPRB, 0.16801E+01_JPRB, 0.16641E+01_JPRB, 0.16207E+01_JPRB, 0.15019E+01_JPRB ,&
&  0.10863E+01_JPRB, 0.80320E+00_JPRB, 0.83898E+00_JPRB, 0.79433E+00_JPRB, 0.79798E+00_JPRB, 0.80214E+00_JPRB, 0.81645E+00_JPRB ,&
&  0.84929E+00_JPRB, 0.87675E+00_JPRB, 0.90472E+00_JPRB, 0.92984E+00_JPRB, 0.97374E+00_JPRB, 0.10235E+01_JPRB, 0.10370E+01_JPRB ,&
&  0.10492E+01_JPRB, 0.10704E+01_JPRB, 0.10985E+01_JPRB, 0.10746E+01_JPRB, 0.10974E+01_JPRB, 0.11232E+01_JPRB, 0.11679E+01_JPRB ,&
&  0.11956E+01_JPRB, 0.11329E+01_JPRB, 0.92819E+00_JPRB, 0.94429E+00_JPRB, 0.10718E+01_JPRB, 0.99904E+00_JPRB, 0.99336E+00_JPRB ,&
&  0.11372E+01_JPRB, 0.12418E+01_JPRB, 0.79666E+00_JPRB, 0.66277E+00_JPRB, 0.67662E+00_JPRB, 0.71130E+00_JPRB, 0.76194E+00_JPRB ,&
&  0.75294E+00_JPRB, 0.75979E+00_JPRB, 0.78039E+00_JPRB, 0.78421E+00_JPRB, 0.78668E+00_JPRB, 0.76902E+00_JPRB, 0.75270E+00_JPRB ,&
&  0.69768E+00_JPRB, 0.66547E+00_JPRB, 0.65095E+00_JPRB, 0.63814E+00_JPRB, 0.63903E+00_JPRB, 0.64744E+00_JPRB, 0.63860E+00_JPRB ,&
&  0.62829E+00_JPRB, 0.62820E+00_JPRB, 0.67783E+00_JPRB, 0.66547E+00_JPRB, 0.66671E+00_JPRB, 0.73315E+00_JPRB, 0.71910E+00_JPRB ,&
&  0.73993E+00_JPRB, 0.74231E+00_JPRB, 0.73837E+00_JPRB, 0.73105E+00_JPRB, 0.72579E+00_JPRB, 0.71300E+00_JPRB, 0.69912E+00_JPRB ,&
&  0.68308E+00_JPRB, 0.66401E+00_JPRB, 0.64411E+00_JPRB, 0.62702E+00_JPRB, 0.61298E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.60360E+00_JPRB, 0.61030E+00_JPRB, 0.62410E+00_JPRB, 0.63047E+00_JPRB, 0.63027E+00_JPRB, 0.63842E+00_JPRB, 0.63630E+00_JPRB ,&
&  0.62513E+00_JPRB, 0.61537E+00_JPRB, 0.60278E+00_JPRB, 0.61331E+00_JPRB, 0.63167E+00_JPRB, 0.66879E+00_JPRB, 0.71694E+00_JPRB ,&
&  0.78834E+00_JPRB, 0.89575E+00_JPRB, 0.10354E+01_JPRB, 0.11413E+01_JPRB, 0.12792E+01_JPRB, 0.13331E+01_JPRB, 0.13532E+01_JPRB ,&
&  0.13948E+01_JPRB, 0.14410E+01_JPRB, 0.14595E+01_JPRB, 0.14451E+01_JPRB, 0.13632E+01_JPRB, 0.12872E+01_JPRB, 0.12336E+01_JPRB ,&
&  0.10054E+01_JPRB, 0.81792E+00_JPRB, 0.79683E+00_JPRB, 0.78991E+00_JPRB, 0.77704E+00_JPRB, 0.76252E+00_JPRB, 0.78059E+00_JPRB ,&
&  0.81332E+00_JPRB, 0.84815E+00_JPRB, 0.89215E+00_JPRB, 0.92742E+00_JPRB, 0.96110E+00_JPRB, 0.98303E+00_JPRB, 0.99052E+00_JPRB ,&
&  0.10140E+01_JPRB, 0.10340E+01_JPRB, 0.10773E+01_JPRB, 0.10180E+01_JPRB, 0.10600E+01_JPRB, 0.10255E+01_JPRB, 0.99123E+00_JPRB ,&
&  0.94380E+00_JPRB, 0.85425E+00_JPRB, 0.86290E+00_JPRB, 0.86772E+00_JPRB, 0.87993E+00_JPRB, 0.86676E+00_JPRB, 0.86737E+00_JPRB ,&
&  0.94731E+00_JPRB, 0.99794E+00_JPRB, 0.80194E+00_JPRB, 0.68672E+00_JPRB, 0.69317E+00_JPRB, 0.65033E+00_JPRB, 0.74611E+00_JPRB ,&
&  0.76485E+00_JPRB, 0.76982E+00_JPRB, 0.77992E+00_JPRB, 0.81402E+00_JPRB, 0.84101E+00_JPRB, 0.86050E+00_JPRB, 0.80819E+00_JPRB ,&
&  0.77605E+00_JPRB, 0.75086E+00_JPRB, 0.73351E+00_JPRB, 0.72053E+00_JPRB, 0.72778E+00_JPRB, 0.73805E+00_JPRB, 0.73970E+00_JPRB ,&
&  0.72923E+00_JPRB, 0.71864E+00_JPRB, 0.75294E+00_JPRB, 0.74852E+00_JPRB, 0.72819E+00_JPRB, 0.78160E+00_JPRB, 0.73986E+00_JPRB ,&
&  0.72990E+00_JPRB, 0.72890E+00_JPRB, 0.71622E+00_JPRB, 0.69640E+00_JPRB, 0.68596E+00_JPRB, 0.66784E+00_JPRB, 0.65920E+00_JPRB ,&
&  0.65430E+00_JPRB, 0.64040E+00_JPRB, 0.62877E+00_JPRB, 0.62439E+00_JPRB, 0.60894E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.60433E+00_JPRB, 0.60387E+00_JPRB, 0.61774E+00_JPRB, 0.62176E+00_JPRB, 0.62056E+00_JPRB, 0.62663E+00_JPRB, 0.63180E+00_JPRB ,&
&  0.62962E+00_JPRB, 0.63030E+00_JPRB, 0.62721E+00_JPRB, 0.62706E+00_JPRB, 0.62967E+00_JPRB, 0.65174E+00_JPRB, 0.69660E+00_JPRB ,&
&  0.75652E+00_JPRB, 0.85560E+00_JPRB, 0.98897E+00_JPRB, 0.11129E+01_JPRB, 0.12493E+01_JPRB, 0.13209E+01_JPRB, 0.12918E+01_JPRB ,&
&  0.13362E+01_JPRB, 0.13841E+01_JPRB, 0.13614E+01_JPRB, 0.13204E+01_JPRB, 0.10500E+01_JPRB, 0.10498E+01_JPRB, 0.10622E+01_JPRB ,&
&  0.99058E+00_JPRB, 0.87535E+00_JPRB, 0.81853E+00_JPRB, 0.78751E+00_JPRB, 0.74853E+00_JPRB, 0.73277E+00_JPRB, 0.73364E+00_JPRB ,&
&  0.75073E+00_JPRB, 0.78089E+00_JPRB, 0.82418E+00_JPRB, 0.88233E+00_JPRB, 0.92031E+00_JPRB, 0.93068E+00_JPRB, 0.93826E+00_JPRB ,&
&  0.94673E+00_JPRB, 0.94215E+00_JPRB, 0.94153E+00_JPRB, 0.92540E+00_JPRB, 0.91120E+00_JPRB, 0.87995E+00_JPRB, 0.86341E+00_JPRB ,&
&  0.85385E+00_JPRB, 0.79931E+00_JPRB, 0.79400E+00_JPRB, 0.79506E+00_JPRB, 0.76006E+00_JPRB, 0.72556E+00_JPRB, 0.74658E+00_JPRB ,&
&  0.81027E+00_JPRB, 0.92240E+00_JPRB, 0.79356E+00_JPRB, 0.77968E+00_JPRB, 0.82594E+00_JPRB, 0.74538E+00_JPRB, 0.71379E+00_JPRB ,&
&  0.75206E+00_JPRB, 0.77446E+00_JPRB, 0.78881E+00_JPRB, 0.81787E+00_JPRB, 0.85986E+00_JPRB, 0.91464E+00_JPRB, 0.83209E+00_JPRB ,&
&  0.83845E+00_JPRB, 0.80590E+00_JPRB, 0.80859E+00_JPRB, 0.80158E+00_JPRB, 0.78372E+00_JPRB, 0.75728E+00_JPRB, 0.74983E+00_JPRB ,&
&  0.71833E+00_JPRB, 0.78358E+00_JPRB, 0.88778E+00_JPRB, 0.10166E+01_JPRB, 0.11954E+01_JPRB, 0.80914E+00_JPRB, 0.79751E+00_JPRB ,&
&  0.75439E+00_JPRB, 0.73769E+00_JPRB, 0.71960E+00_JPRB, 0.70007E+00_JPRB, 0.68331E+00_JPRB, 0.66830E+00_JPRB, 0.66067E+00_JPRB ,&
&  0.65259E+00_JPRB, 0.63871E+00_JPRB, 0.62966E+00_JPRB, 0.62608E+00_JPRB, 0.61077E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.61797E+00_JPRB, 0.61603E+00_JPRB, 0.62385E+00_JPRB, 0.63576E+00_JPRB, 0.64652E+00_JPRB, 0.64553E+00_JPRB, 0.63640E+00_JPRB ,&
&  0.61531E+00_JPRB, 0.60905E+00_JPRB, 0.61173E+00_JPRB, 0.61603E+00_JPRB, 0.60632E+00_JPRB, 0.60126E+00_JPRB, 0.61691E+00_JPRB ,&
&  0.65944E+00_JPRB, 0.73283E+00_JPRB, 0.83739E+00_JPRB, 0.96451E+00_JPRB, 0.11316E+01_JPRB, 0.12564E+01_JPRB, 0.12931E+01_JPRB ,&
&  0.13881E+01_JPRB, 0.14336E+01_JPRB, 0.12914E+01_JPRB, 0.92442E+00_JPRB, 0.79958E+00_JPRB, 0.90918E+00_JPRB, 0.87982E+00_JPRB ,&
&  0.84079E+00_JPRB, 0.82864E+00_JPRB, 0.81503E+00_JPRB, 0.79529E+00_JPRB, 0.76351E+00_JPRB, 0.73906E+00_JPRB, 0.71718E+00_JPRB ,&
&  0.70960E+00_JPRB, 0.71700E+00_JPRB, 0.74527E+00_JPRB, 0.80198E+00_JPRB, 0.85851E+00_JPRB, 0.88872E+00_JPRB, 0.90062E+00_JPRB ,&
&  0.90035E+00_JPRB, 0.88701E+00_JPRB, 0.87631E+00_JPRB, 0.86438E+00_JPRB, 0.83969E+00_JPRB, 0.83077E+00_JPRB, 0.81660E+00_JPRB ,&
&  0.80226E+00_JPRB, 0.78198E+00_JPRB, 0.76530E+00_JPRB, 0.75286E+00_JPRB, 0.73231E+00_JPRB, 0.66489E+00_JPRB, 0.66917E+00_JPRB ,&
&  0.72110E+00_JPRB, 0.80743E+00_JPRB, 0.80656E+00_JPRB, 0.87234E+00_JPRB, 0.61865E+00_JPRB, 0.64361E+00_JPRB, 0.69113E+00_JPRB ,&
&  0.69641E+00_JPRB, 0.73442E+00_JPRB, 0.76617E+00_JPRB, 0.79860E+00_JPRB, 0.83434E+00_JPRB, 0.82502E+00_JPRB, 0.89375E+00_JPRB ,&
&  0.88914E+00_JPRB, 0.84607E+00_JPRB, 0.84143E+00_JPRB, 0.85663E+00_JPRB, 0.83838E+00_JPRB, 0.78156E+00_JPRB, 0.75067E+00_JPRB ,&
&  0.70674E+00_JPRB, 0.76810E+00_JPRB, 0.78899E+00_JPRB, 0.84533E+00_JPRB, 0.91516E+00_JPRB, 0.74170E+00_JPRB, 0.75237E+00_JPRB ,&
&  0.75441E+00_JPRB, 0.74160E+00_JPRB, 0.73405E+00_JPRB, 0.71130E+00_JPRB, 0.69668E+00_JPRB, 0.68358E+00_JPRB, 0.67378E+00_JPRB ,&
&  0.66995E+00_JPRB, 0.66196E+00_JPRB, 0.64986E+00_JPRB, 0.63824E+00_JPRB, 0.62670E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.63666E+00_JPRB, 0.63882E+00_JPRB, 0.64934E+00_JPRB, 0.65606E+00_JPRB, 0.66852E+00_JPRB, 0.67987E+00_JPRB, 0.66993E+00_JPRB ,&
&  0.61899E+00_JPRB, 0.57972E+00_JPRB, 0.57350E+00_JPRB, 0.58036E+00_JPRB, 0.58386E+00_JPRB, 0.57993E+00_JPRB, 0.58358E+00_JPRB ,&
&  0.59870E+00_JPRB, 0.63106E+00_JPRB, 0.68640E+00_JPRB, 0.80889E+00_JPRB, 0.98435E+00_JPRB, 0.11594E+01_JPRB, 0.12874E+01_JPRB ,&
&  0.12559E+01_JPRB, 0.96049E+00_JPRB, 0.11835E+01_JPRB, 0.11596E+01_JPRB, 0.98018E+00_JPRB, 0.89987E+00_JPRB, 0.87971E+00_JPRB ,&
&  0.86441E+00_JPRB, 0.86713E+00_JPRB, 0.85776E+00_JPRB, 0.83726E+00_JPRB, 0.78422E+00_JPRB, 0.75574E+00_JPRB, 0.72399E+00_JPRB ,&
&  0.70618E+00_JPRB, 0.69675E+00_JPRB, 0.70284E+00_JPRB, 0.73707E+00_JPRB, 0.78166E+00_JPRB, 0.83079E+00_JPRB, 0.87134E+00_JPRB ,&
&  0.87361E+00_JPRB, 0.84739E+00_JPRB, 0.83364E+00_JPRB, 0.80678E+00_JPRB, 0.81048E+00_JPRB, 0.81155E+00_JPRB, 0.79890E+00_JPRB ,&
&  0.78813E+00_JPRB, 0.77223E+00_JPRB, 0.77930E+00_JPRB, 0.77835E+00_JPRB, 0.76736E+00_JPRB, 0.69946E+00_JPRB, 0.69764E+00_JPRB ,&
&  0.69293E+00_JPRB, 0.72103E+00_JPRB, 0.76255E+00_JPRB, 0.81154E+00_JPRB, 0.64418E+00_JPRB, 0.65725E+00_JPRB, 0.62326E+00_JPRB ,&
&  0.65099E+00_JPRB, 0.66699E+00_JPRB, 0.70278E+00_JPRB, 0.73076E+00_JPRB, 0.77680E+00_JPRB, 0.79137E+00_JPRB, 0.90495E+00_JPRB ,&
&  0.90883E+00_JPRB, 0.89713E+00_JPRB, 0.87253E+00_JPRB, 0.88983E+00_JPRB, 0.83663E+00_JPRB, 0.80229E+00_JPRB, 0.78348E+00_JPRB ,&
&  0.81231E+00_JPRB, 0.80180E+00_JPRB, 0.79667E+00_JPRB, 0.80509E+00_JPRB, 0.85445E+00_JPRB, 0.75528E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.71959E+00_JPRB, 0.73029E+00_JPRB, 0.72815E+00_JPRB, 0.71985E+00_JPRB, 0.71144E+00_JPRB, 0.69662E+00_JPRB, 0.68174E+00_JPRB ,&
&  0.67523E+00_JPRB, 0.66785E+00_JPRB, 0.66056E+00_JPRB, 0.65459E+00_JPRB, 0.64580E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.64799E+00_JPRB, 0.64753E+00_JPRB, 0.66005E+00_JPRB, 0.68253E+00_JPRB, 0.70638E+00_JPRB, 0.76250E+00_JPRB, 0.81668E+00_JPRB ,&
&  0.94817E+00_JPRB, 0.65587E+00_JPRB, 0.57197E+00_JPRB, 0.54791E+00_JPRB, 0.55063E+00_JPRB, 0.55780E+00_JPRB, 0.56635E+00_JPRB ,&
&  0.57605E+00_JPRB, 0.59544E+00_JPRB, 0.61762E+00_JPRB, 0.65399E+00_JPRB, 0.80122E+00_JPRB, 0.99396E+00_JPRB, 0.10623E+01_JPRB ,&
&  0.97271E+00_JPRB, 0.10885E+01_JPRB, 0.14087E+01_JPRB, 0.12124E+01_JPRB, 0.10964E+01_JPRB, 0.10625E+01_JPRB, 0.10054E+01_JPRB ,&
&  0.95174E+00_JPRB, 0.91288E+00_JPRB, 0.87862E+00_JPRB, 0.83287E+00_JPRB, 0.80328E+00_JPRB, 0.77729E+00_JPRB, 0.75286E+00_JPRB ,&
&  0.72482E+00_JPRB, 0.70413E+00_JPRB, 0.69556E+00_JPRB, 0.71028E+00_JPRB, 0.73425E+00_JPRB, 0.77019E+00_JPRB, 0.82636E+00_JPRB ,&
&  0.85524E+00_JPRB, 0.82926E+00_JPRB, 0.80354E+00_JPRB, 0.77200E+00_JPRB, 0.78716E+00_JPRB, 0.79662E+00_JPRB, 0.79287E+00_JPRB ,&
&  0.78552E+00_JPRB, 0.80617E+00_JPRB, 0.81977E+00_JPRB, 0.79229E+00_JPRB, 0.73634E+00_JPRB, 0.77327E+00_JPRB, 0.71659E+00_JPRB ,&
&  0.70412E+00_JPRB, 0.67202E+00_JPRB, 0.73750E+00_JPRB, 0.65780E+00_JPRB, 0.67695E+00_JPRB, 0.68332E+00_JPRB, 0.65534E+00_JPRB ,&
&  0.62704E+00_JPRB, 0.62670E+00_JPRB, 0.63110E+00_JPRB, 0.64990E+00_JPRB, 0.70367E+00_JPRB, 0.72970E+00_JPRB, 0.84490E+00_JPRB ,&
&  0.86002E+00_JPRB, 0.94917E+00_JPRB, 0.91779E+00_JPRB, 0.87979E+00_JPRB, 0.80102E+00_JPRB, 0.75575E+00_JPRB, 0.75196E+00_JPRB ,&
&  0.81421E+00_JPRB, 0.83349E+00_JPRB, 0.82699E+00_JPRB, 0.80225E+00_JPRB, 0.80576E+00_JPRB, 0.77654E+00_JPRB, 0.76450E+00_JPRB ,&
&  0.75269E+00_JPRB, 0.74162E+00_JPRB, 0.72977E+00_JPRB, 0.72611E+00_JPRB, 0.71285E+00_JPRB, 0.69798E+00_JPRB, 0.67527E+00_JPRB ,&
&  0.66334E+00_JPRB, 0.65609E+00_JPRB, 0.65138E+00_JPRB, 0.65148E+00_JPRB, 0.65218E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.63915E+00_JPRB, 0.63312E+00_JPRB, 0.62318E+00_JPRB, 0.62944E+00_JPRB, 0.63731E+00_JPRB, 0.65869E+00_JPRB, 0.69900E+00_JPRB ,&
&  0.78430E+00_JPRB, 0.64856E+00_JPRB, 0.58131E+00_JPRB, 0.53591E+00_JPRB, 0.52269E+00_JPRB, 0.53031E+00_JPRB, 0.53675E+00_JPRB ,&
&  0.55128E+00_JPRB, 0.56437E+00_JPRB, 0.57910E+00_JPRB, 0.56618E+00_JPRB, 0.63848E+00_JPRB, 0.78880E+00_JPRB, 0.76609E+00_JPRB ,&
&  0.78769E+00_JPRB, 0.14037E+01_JPRB, 0.13775E+01_JPRB, 0.13082E+01_JPRB, 0.12221E+01_JPRB, 0.12222E+01_JPRB, 0.11062E+01_JPRB ,&
&  0.10124E+01_JPRB, 0.96408E+00_JPRB, 0.94007E+00_JPRB, 0.88670E+00_JPRB, 0.84964E+00_JPRB, 0.80133E+00_JPRB, 0.77752E+00_JPRB ,&
&  0.75098E+00_JPRB, 0.73072E+00_JPRB, 0.70700E+00_JPRB, 0.70537E+00_JPRB, 0.71766E+00_JPRB, 0.74053E+00_JPRB, 0.78037E+00_JPRB ,&
&  0.82265E+00_JPRB, 0.82243E+00_JPRB, 0.79047E+00_JPRB, 0.78554E+00_JPRB, 0.79296E+00_JPRB, 0.80859E+00_JPRB, 0.81347E+00_JPRB ,&
&  0.78938E+00_JPRB, 0.77457E+00_JPRB, 0.85415E+00_JPRB, 0.87368E+00_JPRB, 0.85561E+00_JPRB, 0.86547E+00_JPRB, 0.76100E+00_JPRB ,&
&  0.76369E+00_JPRB, 0.67811E+00_JPRB, 0.69769E+00_JPRB, 0.62155E+00_JPRB, 0.65874E+00_JPRB, 0.68784E+00_JPRB, 0.67161E+00_JPRB ,&
&  0.62879E+00_JPRB, 0.63201E+00_JPRB, 0.61925E+00_JPRB, 0.62056E+00_JPRB, 0.65213E+00_JPRB, 0.68989E+00_JPRB, 0.74569E+00_JPRB ,&
&  0.80630E+00_JPRB, 0.89101E+00_JPRB, 0.95491E+00_JPRB, 0.81806E+00_JPRB, 0.76354E+00_JPRB, 0.71861E+00_JPRB, 0.70682E+00_JPRB ,&
&  0.78004E+00_JPRB, 0.80251E+00_JPRB, 0.83526E+00_JPRB, 0.82756E+00_JPRB, 0.79419E+00_JPRB, 0.78960E+00_JPRB, 0.79386E+00_JPRB ,&
&  0.78565E+00_JPRB, 0.77852E+00_JPRB, 0.74722E+00_JPRB, 0.71601E+00_JPRB, 0.69473E+00_JPRB, 0.68400E+00_JPRB, 0.65789E+00_JPRB ,&
&  0.64961E+00_JPRB, 0.63977E+00_JPRB, 0.63221E+00_JPRB, 0.63490E+00_JPRB, 0.63624E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.61441E+00_JPRB, 0.60897E+00_JPRB, 0.60764E+00_JPRB, 0.60128E+00_JPRB, 0.58776E+00_JPRB, 0.59881E+00_JPRB, 0.66290E+00_JPRB ,&
&  0.67223E+00_JPRB, 0.59675E+00_JPRB, 0.55781E+00_JPRB, 0.52927E+00_JPRB, 0.51213E+00_JPRB, 0.50851E+00_JPRB, 0.51701E+00_JPRB ,&
&  0.52519E+00_JPRB, 0.54073E+00_JPRB, 0.54529E+00_JPRB, 0.54177E+00_JPRB, 0.57124E+00_JPRB, 0.66391E+00_JPRB, 0.52409E+00_JPRB ,&
&  0.69109E+00_JPRB, 0.12481E+01_JPRB, 0.14182E+01_JPRB, 0.14446E+01_JPRB, 0.14373E+01_JPRB, 0.14141E+01_JPRB, 0.12946E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.10797E+01_JPRB, 0.10012E+01_JPRB, 0.96053E+00_JPRB, 0.93198E+00_JPRB, 0.87515E+00_JPRB, 0.81976E+00_JPRB ,&
&  0.78089E+00_JPRB, 0.74558E+00_JPRB, 0.71470E+00_JPRB, 0.70150E+00_JPRB, 0.70309E+00_JPRB, 0.71838E+00_JPRB, 0.74789E+00_JPRB ,&
&  0.78520E+00_JPRB, 0.81352E+00_JPRB, 0.81839E+00_JPRB, 0.77378E+00_JPRB, 0.78816E+00_JPRB, 0.79815E+00_JPRB, 0.81836E+00_JPRB ,&
&  0.78781E+00_JPRB, 0.77402E+00_JPRB, 0.91353E+00_JPRB, 0.10483E+01_JPRB, 0.10864E+01_JPRB, 0.10089E+01_JPRB, 0.88149E+00_JPRB ,&
&  0.83002E+00_JPRB, 0.74883E+00_JPRB, 0.68564E+00_JPRB, 0.62097E+00_JPRB, 0.64470E+00_JPRB, 0.68138E+00_JPRB, 0.69385E+00_JPRB ,&
&  0.66190E+00_JPRB, 0.63769E+00_JPRB, 0.62261E+00_JPRB, 0.60627E+00_JPRB, 0.61938E+00_JPRB, 0.64374E+00_JPRB, 0.71932E+00_JPRB ,&
&  0.80022E+00_JPRB, 0.89687E+00_JPRB, 0.90549E+00_JPRB, 0.77229E+00_JPRB, 0.71976E+00_JPRB, 0.66765E+00_JPRB, 0.62156E+00_JPRB ,&
&  0.69462E+00_JPRB, 0.70891E+00_JPRB, 0.69300E+00_JPRB, 0.77905E+00_JPRB, 0.74629E+00_JPRB, 0.76768E+00_JPRB, 0.82028E+00_JPRB ,&
&  0.80652E+00_JPRB, 0.78510E+00_JPRB, 0.75855E+00_JPRB, 0.74022E+00_JPRB, 0.70276E+00_JPRB, 0.66393E+00_JPRB, 0.64018E+00_JPRB ,&
&  0.62917E+00_JPRB, 0.62644E+00_JPRB, 0.62061E+00_JPRB, 0.61845E+00_JPRB, 0.61624E+00_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.57944E+00_JPRB, 0.57672E+00_JPRB, 0.57301E+00_JPRB, 0.56976E+00_JPRB, 0.55662E+00_JPRB, 0.55035E+00_JPRB, 0.58483E+00_JPRB ,&
&  0.58664E+00_JPRB, 0.55974E+00_JPRB, 0.51655E+00_JPRB, 0.50404E+00_JPRB, 0.49687E+00_JPRB, 0.49159E+00_JPRB, 0.49742E+00_JPRB ,&
&  0.50526E+00_JPRB, 0.52551E+00_JPRB, 0.55660E+00_JPRB, 0.55245E+00_JPRB, 0.50404E+00_JPRB, 0.45653E+00_JPRB, 0.49140E+00_JPRB ,&
&  0.90219E+00_JPRB, 0.13725E+01_JPRB, 0.14922E+01_JPRB, 0.16157E+01_JPRB, 0.17147E+01_JPRB, 0.18256E+01_JPRB, 0.16352E+01_JPRB ,&
&  0.14363E+01_JPRB, 0.13112E+01_JPRB, 0.11925E+01_JPRB, 0.11196E+01_JPRB, 0.10454E+01_JPRB, 0.96765E+00_JPRB, 0.90635E+00_JPRB ,&
&  0.84683E+00_JPRB, 0.80351E+00_JPRB, 0.75708E+00_JPRB, 0.70990E+00_JPRB, 0.70535E+00_JPRB, 0.71893E+00_JPRB, 0.73122E+00_JPRB ,&
&  0.74962E+00_JPRB, 0.79070E+00_JPRB, 0.80405E+00_JPRB, 0.76576E+00_JPRB, 0.76291E+00_JPRB, 0.77668E+00_JPRB, 0.83142E+00_JPRB ,&
&  0.85467E+00_JPRB, 0.92546E+00_JPRB, 0.10195E+01_JPRB, 0.11233E+01_JPRB, 0.12084E+01_JPRB, 0.11963E+01_JPRB, 0.10696E+01_JPRB ,&
&  0.94379E+00_JPRB, 0.80316E+00_JPRB, 0.67439E+00_JPRB, 0.64898E+00_JPRB, 0.68118E+00_JPRB, 0.71525E+00_JPRB, 0.65769E+00_JPRB ,&
&  0.59889E+00_JPRB, 0.56996E+00_JPRB, 0.56844E+00_JPRB, 0.57622E+00_JPRB, 0.61248E+00_JPRB, 0.62920E+00_JPRB, 0.70308E+00_JPRB ,&
&  0.70131E+00_JPRB, 0.55301E+00_JPRB, 0.50624E+00_JPRB, 0.58704E+00_JPRB, 0.54665E+00_JPRB, 0.51466E+00_JPRB, 0.57144E+00_JPRB ,&
&  0.61184E+00_JPRB, 0.64494E+00_JPRB, 0.62954E+00_JPRB, 0.61804E+00_JPRB, 0.67060E+00_JPRB, 0.71457E+00_JPRB, 0.85533E+00_JPRB ,&
&  0.80181E+00_JPRB, 0.75555E+00_JPRB, 0.74137E+00_JPRB, 0.73093E+00_JPRB, 0.69568E+00_JPRB, 0.65013E+00_JPRB, 0.61352E+00_JPRB ,&
&  0.59822E+00_JPRB, 0.59127E+00_JPRB, 0.58405E+00_JPRB, 0.58147E+00_JPRB, 0.57926E+00_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.54196E+00_JPRB, 0.54536E+00_JPRB, 0.54193E+00_JPRB, 0.53836E+00_JPRB, 0.52935E+00_JPRB, 0.51962E+00_JPRB, 0.53158E+00_JPRB ,&
&  0.52717E+00_JPRB, 0.51145E+00_JPRB, 0.49038E+00_JPRB, 0.48331E+00_JPRB, 0.47905E+00_JPRB, 0.47877E+00_JPRB, 0.48557E+00_JPRB ,&
&  0.49410E+00_JPRB, 0.51605E+00_JPRB, 0.57349E+00_JPRB, 0.58083E+00_JPRB, 0.50064E+00_JPRB, 0.41221E+00_JPRB, 0.51708E+00_JPRB ,&
&  0.10014E+01_JPRB, 0.15670E+01_JPRB, 0.18493E+01_JPRB, 0.20691E+01_JPRB, 0.20883E+01_JPRB, 0.21189E+01_JPRB, 0.19949E+01_JPRB ,&
&  0.18158E+01_JPRB, 0.16552E+01_JPRB, 0.14907E+01_JPRB, 0.13016E+01_JPRB, 0.11422E+01_JPRB, 0.10354E+01_JPRB, 0.94959E+00_JPRB ,&
&  0.89559E+00_JPRB, 0.85070E+00_JPRB, 0.80328E+00_JPRB, 0.74873E+00_JPRB, 0.72045E+00_JPRB, 0.72276E+00_JPRB, 0.72460E+00_JPRB ,&
&  0.72503E+00_JPRB, 0.76333E+00_JPRB, 0.78759E+00_JPRB, 0.78001E+00_JPRB, 0.65892E+00_JPRB, 0.75067E+00_JPRB, 0.79739E+00_JPRB ,&
&  0.87176E+00_JPRB, 0.95533E+00_JPRB, 0.97689E+00_JPRB, 0.12309E+01_JPRB, 0.13645E+01_JPRB, 0.12855E+01_JPRB, 0.11627E+01_JPRB ,&
&  0.10572E+01_JPRB, 0.87991E+00_JPRB, 0.74029E+00_JPRB, 0.73514E+00_JPRB, 0.75782E+00_JPRB, 0.74954E+00_JPRB, 0.57301E+00_JPRB ,&
&  0.54124E+00_JPRB, 0.59121E+00_JPRB, 0.57997E+00_JPRB, 0.53604E+00_JPRB, 0.59901E+00_JPRB, 0.61706E+00_JPRB, 0.54995E+00_JPRB ,&
&  0.37736E+00_JPRB, 0.34080E+00_JPRB, 0.34962E+00_JPRB, 0.36411E+00_JPRB, 0.35251E+00_JPRB, 0.39906E+00_JPRB, 0.56364E+00_JPRB ,&
&  0.53813E+00_JPRB, 0.59340E+00_JPRB, 0.58531E+00_JPRB, 0.57735E+00_JPRB, 0.61443E+00_JPRB, 0.68733E+00_JPRB, 0.94455E+00_JPRB ,&
&  0.82569E+00_JPRB, 0.77768E+00_JPRB, 0.73030E+00_JPRB, 0.70112E+00_JPRB, 0.66878E+00_JPRB, 0.62878E+00_JPRB, 0.59876E+00_JPRB ,&
&  0.57946E+00_JPRB, 0.57123E+00_JPRB, 0.56010E+00_JPRB, 0.55152E+00_JPRB, 0.54195E+00_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.51440E+00_JPRB, 0.51004E+00_JPRB, 0.50592E+00_JPRB, 0.50231E+00_JPRB, 0.49378E+00_JPRB, 0.48693E+00_JPRB, 0.49054E+00_JPRB ,&
&  0.48404E+00_JPRB, 0.46880E+00_JPRB, 0.45556E+00_JPRB, 0.45004E+00_JPRB, 0.45504E+00_JPRB, 0.46816E+00_JPRB, 0.47757E+00_JPRB ,&
&  0.49317E+00_JPRB, 0.53788E+00_JPRB, 0.66596E+00_JPRB, 0.64134E+00_JPRB, 0.61704E+00_JPRB, 0.58903E+00_JPRB, 0.62906E+00_JPRB ,&
&  0.95743E+00_JPRB, 0.14775E+01_JPRB, 0.18961E+01_JPRB, 0.22012E+01_JPRB, 0.25407E+01_JPRB, 0.26410E+01_JPRB, 0.25319E+01_JPRB ,&
&  0.22498E+01_JPRB, 0.19921E+01_JPRB, 0.17301E+01_JPRB, 0.14485E+01_JPRB, 0.12357E+01_JPRB, 0.10964E+01_JPRB, 0.99307E+00_JPRB ,&
&  0.90801E+00_JPRB, 0.86915E+00_JPRB, 0.82339E+00_JPRB, 0.77921E+00_JPRB, 0.74682E+00_JPRB, 0.74410E+00_JPRB, 0.74515E+00_JPRB ,&
&  0.73518E+00_JPRB, 0.73866E+00_JPRB, 0.75578E+00_JPRB, 0.74998E+00_JPRB, 0.66085E+00_JPRB, 0.60174E+00_JPRB, 0.70218E+00_JPRB ,&
&  0.82072E+00_JPRB, 0.10157E+01_JPRB, 0.11120E+01_JPRB, 0.14667E+01_JPRB, 0.14777E+01_JPRB, 0.13230E+01_JPRB, 0.12485E+01_JPRB ,&
&  0.11283E+01_JPRB, 0.10512E+01_JPRB, 0.80640E+00_JPRB, 0.80689E+00_JPRB, 0.85178E+00_JPRB, 0.69904E+00_JPRB, 0.61387E+00_JPRB ,&
&  0.65481E+00_JPRB, 0.62192E+00_JPRB, 0.58235E+00_JPRB, 0.51279E+00_JPRB, 0.59142E+00_JPRB, 0.55677E+00_JPRB, 0.37130E+00_JPRB ,&
&  0.33159E+00_JPRB, 0.32128E+00_JPRB, 0.31207E+00_JPRB, 0.30765E+00_JPRB, 0.30047E+00_JPRB, 0.31884E+00_JPRB, 0.39713E+00_JPRB ,&
&  0.42491E+00_JPRB, 0.49838E+00_JPRB, 0.54080E+00_JPRB, 0.56519E+00_JPRB, 0.61077E+00_JPRB, 0.75400E+00_JPRB, 0.10536E+01_JPRB ,&
&  0.88065E+00_JPRB, 0.86115E+00_JPRB, 0.74674E+00_JPRB, 0.68423E+00_JPRB, 0.64161E+00_JPRB, 0.60881E+00_JPRB, 0.58118E+00_JPRB ,&
&  0.55842E+00_JPRB, 0.54362E+00_JPRB, 0.53200E+00_JPRB, 0.52862E+00_JPRB, 0.52065E+00_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.47382E+00_JPRB, 0.46743E+00_JPRB, 0.46264E+00_JPRB, 0.45881E+00_JPRB, 0.45152E+00_JPRB, 0.44380E+00_JPRB, 0.45226E+00_JPRB ,&
&  0.45198E+00_JPRB, 0.43565E+00_JPRB, 0.42563E+00_JPRB, 0.42302E+00_JPRB, 0.43620E+00_JPRB, 0.46060E+00_JPRB, 0.47447E+00_JPRB ,&
&  0.49438E+00_JPRB, 0.58379E+00_JPRB, 0.71527E+00_JPRB, 0.62243E+00_JPRB, 0.54997E+00_JPRB, 0.51607E+00_JPRB, 0.53395E+00_JPRB ,&
&  0.87539E+00_JPRB, 0.12227E+01_JPRB, 0.15703E+01_JPRB, 0.20078E+01_JPRB, 0.24305E+01_JPRB, 0.24586E+01_JPRB, 0.26779E+01_JPRB ,&
&  0.25161E+01_JPRB, 0.21554E+01_JPRB, 0.18234E+01_JPRB, 0.15169E+01_JPRB, 0.12990E+01_JPRB, 0.11322E+01_JPRB, 0.10093E+01_JPRB ,&
&  0.93856E+00_JPRB, 0.89557E+00_JPRB, 0.85622E+00_JPRB, 0.81800E+00_JPRB, 0.79251E+00_JPRB, 0.77808E+00_JPRB, 0.77985E+00_JPRB ,&
&  0.76361E+00_JPRB, 0.74001E+00_JPRB, 0.73377E+00_JPRB, 0.73330E+00_JPRB, 0.74148E+00_JPRB, 0.73933E+00_JPRB, 0.72976E+00_JPRB ,&
&  0.73838E+00_JPRB, 0.84401E+00_JPRB, 0.12179E+01_JPRB, 0.15264E+01_JPRB, 0.14841E+01_JPRB, 0.13849E+01_JPRB, 0.12590E+01_JPRB ,&
&  0.11678E+01_JPRB, 0.11276E+01_JPRB, 0.93067E+00_JPRB, 0.94585E+00_JPRB, 0.84430E+00_JPRB, 0.61827E+00_JPRB, 0.66535E+00_JPRB ,&
&  0.70618E+00_JPRB, 0.67716E+00_JPRB, 0.58732E+00_JPRB, 0.46941E+00_JPRB, 0.47597E+00_JPRB, 0.39179E+00_JPRB, 0.31840E+00_JPRB ,&
&  0.30884E+00_JPRB, 0.29981E+00_JPRB, 0.28354E+00_JPRB, 0.28244E+00_JPRB, 0.27466E+00_JPRB, 0.29102E+00_JPRB, 0.33216E+00_JPRB ,&
&  0.37319E+00_JPRB, 0.42363E+00_JPRB, 0.48268E+00_JPRB, 0.52369E+00_JPRB, 0.55146E+00_JPRB, 0.58712E+00_JPRB, 0.74468E+00_JPRB ,&
&  0.72982E+00_JPRB, 0.78727E+00_JPRB, 0.69867E+00_JPRB, 0.63270E+00_JPRB, 0.58516E+00_JPRB, 0.55656E+00_JPRB, 0.53562E+00_JPRB ,&
&  0.51900E+00_JPRB, 0.50127E+00_JPRB, 0.48910E+00_JPRB, 0.48439E+00_JPRB, 0.48025E+00_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.43337E+00_JPRB, 0.42969E+00_JPRB, 0.42725E+00_JPRB, 0.42295E+00_JPRB, 0.41804E+00_JPRB, 0.41127E+00_JPRB, 0.41886E+00_JPRB ,&
&  0.42711E+00_JPRB, 0.40734E+00_JPRB, 0.39116E+00_JPRB, 0.39372E+00_JPRB, 0.41324E+00_JPRB, 0.44319E+00_JPRB, 0.47862E+00_JPRB ,&
&  0.52143E+00_JPRB, 0.75927E+00_JPRB, 0.58378E+00_JPRB, 0.46062E+00_JPRB, 0.50742E+00_JPRB, 0.53629E+00_JPRB, 0.50940E+00_JPRB ,&
&  0.76261E+00_JPRB, 0.10153E+01_JPRB, 0.12974E+01_JPRB, 0.18478E+01_JPRB, 0.26679E+01_JPRB, 0.27265E+01_JPRB, 0.28632E+01_JPRB ,&
&  0.28483E+01_JPRB, 0.23974E+01_JPRB, 0.19324E+01_JPRB, 0.15666E+01_JPRB, 0.13171E+01_JPRB, 0.11474E+01_JPRB, 0.10447E+01_JPRB ,&
&  0.97923E+00_JPRB, 0.93271E+00_JPRB, 0.89210E+00_JPRB, 0.86581E+00_JPRB, 0.83580E+00_JPRB, 0.80978E+00_JPRB, 0.80268E+00_JPRB ,&
&  0.78118E+00_JPRB, 0.75170E+00_JPRB, 0.74367E+00_JPRB, 0.74605E+00_JPRB, 0.77228E+00_JPRB, 0.75469E+00_JPRB, 0.10486E+01_JPRB ,&
&  0.10053E+01_JPRB, 0.11100E+01_JPRB, 0.12909E+01_JPRB, 0.17230E+01_JPRB, 0.17319E+01_JPRB, 0.15092E+01_JPRB, 0.15048E+01_JPRB ,&
&  0.11647E+01_JPRB, 0.98467E+00_JPRB, 0.92217E+00_JPRB, 0.87308E+00_JPRB, 0.66685E+00_JPRB, 0.66675E+00_JPRB, 0.85997E+00_JPRB ,&
&  0.73140E+00_JPRB, 0.76297E+00_JPRB, 0.76883E+00_JPRB, 0.68003E+00_JPRB, 0.40787E+00_JPRB, 0.34713E+00_JPRB, 0.42186E+00_JPRB ,&
&  0.43532E+00_JPRB, 0.35066E+00_JPRB, 0.29718E+00_JPRB, 0.32707E+00_JPRB, 0.28002E+00_JPRB, 0.29950E+00_JPRB, 0.33908E+00_JPRB ,&
&  0.34204E+00_JPRB, 0.35940E+00_JPRB, 0.44038E+00_JPRB, 0.48010E+00_JPRB, 0.50263E+00_JPRB, 0.50364E+00_JPRB, 0.55152E+00_JPRB ,&
&  0.58658E+00_JPRB, 0.68247E+00_JPRB, 0.67773E+00_JPRB, 0.60042E+00_JPRB, 0.54285E+00_JPRB, 0.50721E+00_JPRB, 0.48428E+00_JPRB ,&
&  0.46835E+00_JPRB, 0.44935E+00_JPRB, 0.43911E+00_JPRB, 0.43781E+00_JPRB, 0.43647E+00_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.39962E+00_JPRB, 0.39983E+00_JPRB, 0.40015E+00_JPRB, 0.39627E+00_JPRB, 0.39052E+00_JPRB, 0.38679E+00_JPRB, 0.39212E+00_JPRB ,&
&  0.40312E+00_JPRB, 0.38152E+00_JPRB, 0.36777E+00_JPRB, 0.37059E+00_JPRB, 0.38479E+00_JPRB, 0.40778E+00_JPRB, 0.44676E+00_JPRB ,&
&  0.49496E+00_JPRB, 0.55025E+00_JPRB, 0.42738E+00_JPRB, 0.38576E+00_JPRB, 0.52583E+00_JPRB, 0.52073E+00_JPRB, 0.57400E+00_JPRB ,&
&  0.71915E+00_JPRB, 0.86146E+00_JPRB, 0.11057E+01_JPRB, 0.16093E+01_JPRB, 0.20850E+01_JPRB, 0.25911E+01_JPRB, 0.28774E+01_JPRB ,&
&  0.29922E+01_JPRB, 0.27052E+01_JPRB, 0.22623E+01_JPRB, 0.18149E+01_JPRB, 0.14667E+01_JPRB, 0.12370E+01_JPRB, 0.10948E+01_JPRB ,&
&  0.10005E+01_JPRB, 0.92641E+00_JPRB, 0.87808E+00_JPRB, 0.83423E+00_JPRB, 0.82157E+00_JPRB, 0.79854E+00_JPRB, 0.79545E+00_JPRB ,&
&  0.79093E+00_JPRB, 0.77113E+00_JPRB, 0.77450E+00_JPRB, 0.76985E+00_JPRB, 0.76147E+00_JPRB, 0.78826E+00_JPRB, 0.92598E+00_JPRB ,&
&  0.11075E+01_JPRB, 0.12573E+01_JPRB, 0.14473E+01_JPRB, 0.16133E+01_JPRB, 0.16122E+01_JPRB, 0.14265E+01_JPRB, 0.15658E+01_JPRB ,&
&  0.12519E+01_JPRB, 0.97906E+00_JPRB, 0.77664E+00_JPRB, 0.60962E+00_JPRB, 0.58855E+00_JPRB, 0.75744E+00_JPRB, 0.79470E+00_JPRB ,&
&  0.79624E+00_JPRB, 0.81617E+00_JPRB, 0.80355E+00_JPRB, 0.65514E+00_JPRB, 0.47729E+00_JPRB, 0.38349E+00_JPRB, 0.41792E+00_JPRB ,&
&  0.42816E+00_JPRB, 0.43080E+00_JPRB, 0.40905E+00_JPRB, 0.37654E+00_JPRB, 0.32389E+00_JPRB, 0.35577E+00_JPRB, 0.35807E+00_JPRB ,&
&  0.33833E+00_JPRB, 0.33165E+00_JPRB, 0.38757E+00_JPRB, 0.43885E+00_JPRB, 0.44163E+00_JPRB, 0.41485E+00_JPRB, 0.44973E+00_JPRB ,&
&  0.46629E+00_JPRB, 0.50027E+00_JPRB, 0.49607E+00_JPRB, 0.47359E+00_JPRB, 0.45420E+00_JPRB, 0.43610E+00_JPRB, 0.41815E+00_JPRB ,&
&  0.40527E+00_JPRB, 0.39696E+00_JPRB, 0.38987E+00_JPRB, 0.39249E+00_JPRB, 0.39644E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.38203E+00_JPRB, 0.38532E+00_JPRB, 0.38707E+00_JPRB, 0.38649E+00_JPRB, 0.38259E+00_JPRB, 0.37687E+00_JPRB, 0.38035E+00_JPRB ,&
&  0.38474E+00_JPRB, 0.36384E+00_JPRB, 0.35618E+00_JPRB, 0.35846E+00_JPRB, 0.37010E+00_JPRB, 0.38776E+00_JPRB, 0.41561E+00_JPRB ,&
&  0.46204E+00_JPRB, 0.44313E+00_JPRB, 0.35021E+00_JPRB, 0.38293E+00_JPRB, 0.45517E+00_JPRB, 0.46434E+00_JPRB, 0.55252E+00_JPRB ,&
&  0.63334E+00_JPRB, 0.72739E+00_JPRB, 0.87475E+00_JPRB, 0.11998E+01_JPRB, 0.15702E+01_JPRB, 0.19421E+01_JPRB, 0.22163E+01_JPRB ,&
&  0.22836E+01_JPRB, 0.23908E+01_JPRB, 0.23282E+01_JPRB, 0.20808E+01_JPRB, 0.17466E+01_JPRB, 0.14559E+01_JPRB, 0.12590E+01_JPRB ,&
&  0.11135E+01_JPRB, 0.97639E+00_JPRB, 0.88071E+00_JPRB, 0.82180E+00_JPRB, 0.78495E+00_JPRB, 0.77183E+00_JPRB, 0.77895E+00_JPRB ,&
&  0.79596E+00_JPRB, 0.78767E+00_JPRB, 0.77616E+00_JPRB, 0.79584E+00_JPRB, 0.77082E+00_JPRB, 0.82150E+00_JPRB, 0.92724E+00_JPRB ,&
&  0.13352E+01_JPRB, 0.15605E+01_JPRB, 0.16368E+01_JPRB, 0.18256E+01_JPRB, 0.14818E+01_JPRB, 0.13423E+01_JPRB, 0.15150E+01_JPRB ,&
&  0.16100E+01_JPRB, 0.15805E+01_JPRB, 0.15314E+01_JPRB, 0.91972E+00_JPRB, 0.61813E+00_JPRB, 0.81596E+00_JPRB, 0.79763E+00_JPRB ,&
&  0.75061E+00_JPRB, 0.76326E+00_JPRB, 0.74971E+00_JPRB, 0.73869E+00_JPRB, 0.52347E+00_JPRB, 0.43069E+00_JPRB, 0.36421E+00_JPRB ,&
&  0.35680E+00_JPRB, 0.34222E+00_JPRB, 0.36763E+00_JPRB, 0.35548E+00_JPRB, 0.34332E+00_JPRB, 0.34111E+00_JPRB, 0.33633E+00_JPRB ,&
&  0.33172E+00_JPRB, 0.32477E+00_JPRB, 0.33265E+00_JPRB, 0.37002E+00_JPRB, 0.39021E+00_JPRB, 0.35394E+00_JPRB, 0.35930E+00_JPRB ,&
&  0.37901E+00_JPRB, 0.39664E+00_JPRB, 0.41193E+00_JPRB, 0.41163E+00_JPRB, 0.41779E+00_JPRB, 0.40885E+00_JPRB, 0.38963E+00_JPRB ,&
&  0.37301E+00_JPRB, 0.36856E+00_JPRB, 0.36883E+00_JPRB, 0.37270E+00_JPRB, 0.37809E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.36305E+00_JPRB, 0.36932E+00_JPRB, 0.37359E+00_JPRB, 0.37663E+00_JPRB, 0.37854E+00_JPRB, 0.37634E+00_JPRB, 0.37798E+00_JPRB ,&
&  0.37596E+00_JPRB, 0.36079E+00_JPRB, 0.35058E+00_JPRB, 0.35181E+00_JPRB, 0.36012E+00_JPRB, 0.37025E+00_JPRB, 0.38705E+00_JPRB ,&
&  0.43293E+00_JPRB, 0.47739E+00_JPRB, 0.37667E+00_JPRB, 0.33033E+00_JPRB, 0.31969E+00_JPRB, 0.39460E+00_JPRB, 0.46059E+00_JPRB ,&
&  0.56045E+00_JPRB, 0.60550E+00_JPRB, 0.74019E+00_JPRB, 0.94216E+00_JPRB, 0.11555E+01_JPRB, 0.13557E+01_JPRB, 0.15298E+01_JPRB ,&
&  0.18570E+01_JPRB, 0.17408E+01_JPRB, 0.18277E+01_JPRB, 0.17551E+01_JPRB, 0.17148E+01_JPRB, 0.15455E+01_JPRB, 0.13396E+01_JPRB ,&
&  0.11727E+01_JPRB, 0.10340E+01_JPRB, 0.90847E+00_JPRB, 0.81032E+00_JPRB, 0.75174E+00_JPRB, 0.72294E+00_JPRB, 0.72992E+00_JPRB ,&
&  0.78525E+00_JPRB, 0.81031E+00_JPRB, 0.80642E+00_JPRB, 0.81909E+00_JPRB, 0.88114E+00_JPRB, 0.98123E+00_JPRB, 0.11504E+01_JPRB ,&
&  0.12506E+01_JPRB, 0.12949E+01_JPRB, 0.18591E+01_JPRB, 0.19960E+01_JPRB, 0.20636E+01_JPRB, 0.18527E+01_JPRB, 0.16591E+01_JPRB ,&
&  0.16841E+01_JPRB, 0.16233E+01_JPRB, 0.15579E+01_JPRB, 0.10940E+01_JPRB, 0.10731E+01_JPRB, 0.10037E+01_JPRB, 0.87903E+00_JPRB ,&
&  0.79587E+00_JPRB, 0.77063E+00_JPRB, 0.71503E+00_JPRB, 0.67623E+00_JPRB, 0.63356E+00_JPRB, 0.58549E+00_JPRB, 0.45684E+00_JPRB ,&
&  0.38150E+00_JPRB, 0.38947E+00_JPRB, 0.37247E+00_JPRB, 0.33799E+00_JPRB, 0.30527E+00_JPRB, 0.30797E+00_JPRB, 0.30893E+00_JPRB ,&
&  0.31020E+00_JPRB, 0.31197E+00_JPRB, 0.30722E+00_JPRB, 0.31878E+00_JPRB, 0.32962E+00_JPRB, 0.31721E+00_JPRB, 0.30453E+00_JPRB ,&
&  0.29591E+00_JPRB, 0.31768E+00_JPRB, 0.32588E+00_JPRB, 0.32847E+00_JPRB, 0.36567E+00_JPRB, 0.36928E+00_JPRB, 0.35899E+00_JPRB ,&
&  0.34528E+00_JPRB, 0.34091E+00_JPRB, 0.34400E+00_JPRB, 0.34905E+00_JPRB, 0.35508E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.33532E+00_JPRB, 0.34909E+00_JPRB, 0.36078E+00_JPRB, 0.36915E+00_JPRB, 0.37562E+00_JPRB, 0.37538E+00_JPRB, 0.37458E+00_JPRB ,&
&  0.37116E+00_JPRB, 0.35833E+00_JPRB, 0.34491E+00_JPRB, 0.34651E+00_JPRB, 0.34985E+00_JPRB, 0.35764E+00_JPRB, 0.37599E+00_JPRB ,&
&  0.42058E+00_JPRB, 0.56549E+00_JPRB, 0.42852E+00_JPRB, 0.39042E+00_JPRB, 0.34793E+00_JPRB, 0.38002E+00_JPRB, 0.41651E+00_JPRB ,&
&  0.52316E+00_JPRB, 0.52111E+00_JPRB, 0.57149E+00_JPRB, 0.67699E+00_JPRB, 0.85220E+00_JPRB, 0.97297E+00_JPRB, 0.11351E+01_JPRB ,&
&  0.12053E+01_JPRB, 0.12893E+01_JPRB, 0.12159E+01_JPRB, 0.12721E+01_JPRB, 0.13111E+01_JPRB, 0.12231E+01_JPRB, 0.12181E+01_JPRB ,&
&  0.10759E+01_JPRB, 0.97356E+00_JPRB, 0.89284E+00_JPRB, 0.81970E+00_JPRB, 0.77056E+00_JPRB, 0.73236E+00_JPRB, 0.71886E+00_JPRB ,&
&  0.76514E+00_JPRB, 0.82699E+00_JPRB, 0.83445E+00_JPRB, 0.83991E+00_JPRB, 0.87225E+00_JPRB, 0.10122E+01_JPRB, 0.11809E+01_JPRB ,&
&  0.15632E+01_JPRB, 0.16187E+01_JPRB, 0.13757E+01_JPRB, 0.19289E+01_JPRB, 0.23984E+01_JPRB, 0.20826E+01_JPRB, 0.16719E+01_JPRB ,&
&  0.16433E+01_JPRB, 0.15623E+01_JPRB, 0.14455E+01_JPRB, 0.12782E+01_JPRB, 0.11494E+01_JPRB, 0.10394E+01_JPRB, 0.90271E+00_JPRB ,&
&  0.77022E+00_JPRB, 0.69766E+00_JPRB, 0.66055E+00_JPRB, 0.59783E+00_JPRB, 0.54148E+00_JPRB, 0.49607E+00_JPRB, 0.46111E+00_JPRB ,&
&  0.40925E+00_JPRB, 0.36456E+00_JPRB, 0.29515E+00_JPRB, 0.30567E+00_JPRB, 0.27149E+00_JPRB, 0.27394E+00_JPRB, 0.28645E+00_JPRB ,&
&  0.28445E+00_JPRB, 0.28602E+00_JPRB, 0.27962E+00_JPRB, 0.27243E+00_JPRB, 0.28430E+00_JPRB, 0.27970E+00_JPRB, 0.27373E+00_JPRB ,&
&  0.26448E+00_JPRB, 0.26180E+00_JPRB, 0.27742E+00_JPRB, 0.28327E+00_JPRB, 0.29927E+00_JPRB, 0.30568E+00_JPRB, 0.30900E+00_JPRB ,&
&  0.30814E+00_JPRB, 0.31032E+00_JPRB, 0.31705E+00_JPRB, 0.32347E+00_JPRB, 0.32690E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.30882E+00_JPRB, 0.32831E+00_JPRB, 0.34358E+00_JPRB, 0.35199E+00_JPRB, 0.36127E+00_JPRB, 0.35827E+00_JPRB, 0.35720E+00_JPRB ,&
&  0.35984E+00_JPRB, 0.34954E+00_JPRB, 0.33554E+00_JPRB, 0.33724E+00_JPRB, 0.33907E+00_JPRB, 0.34539E+00_JPRB, 0.36634E+00_JPRB ,&
&  0.42340E+00_JPRB, 0.44456E+00_JPRB, 0.37342E+00_JPRB, 0.32342E+00_JPRB, 0.37568E+00_JPRB, 0.35852E+00_JPRB, 0.37687E+00_JPRB ,&
&  0.36566E+00_JPRB, 0.38515E+00_JPRB, 0.40495E+00_JPRB, 0.44962E+00_JPRB, 0.55344E+00_JPRB, 0.70542E+00_JPRB, 0.80408E+00_JPRB ,&
&  0.84436E+00_JPRB, 0.86690E+00_JPRB, 0.85071E+00_JPRB, 0.84676E+00_JPRB, 0.87707E+00_JPRB, 0.82091E+00_JPRB, 0.84285E+00_JPRB ,&
&  0.82186E+00_JPRB, 0.77322E+00_JPRB, 0.73054E+00_JPRB, 0.72401E+00_JPRB, 0.71257E+00_JPRB, 0.69772E+00_JPRB, 0.70303E+00_JPRB ,&
&  0.75626E+00_JPRB, 0.83440E+00_JPRB, 0.84180E+00_JPRB, 0.85517E+00_JPRB, 0.91937E+00_JPRB, 0.11786E+01_JPRB, 0.16360E+01_JPRB ,&
&  0.18993E+01_JPRB, 0.20497E+01_JPRB, 0.21843E+01_JPRB, 0.23998E+01_JPRB, 0.23748E+01_JPRB, 0.21977E+01_JPRB, 0.19786E+01_JPRB ,&
&  0.17417E+01_JPRB, 0.15228E+01_JPRB, 0.13015E+01_JPRB, 0.11511E+01_JPRB, 0.10537E+01_JPRB, 0.98955E+00_JPRB, 0.87629E+00_JPRB ,&
&  0.71347E+00_JPRB, 0.62654E+00_JPRB, 0.59197E+00_JPRB, 0.51228E+00_JPRB, 0.46476E+00_JPRB, 0.42395E+00_JPRB, 0.42836E+00_JPRB ,&
&  0.40013E+00_JPRB, 0.29009E+00_JPRB, 0.26308E+00_JPRB, 0.28797E+00_JPRB, 0.26792E+00_JPRB, 0.26987E+00_JPRB, 0.28093E+00_JPRB ,&
&  0.27934E+00_JPRB, 0.26966E+00_JPRB, 0.25819E+00_JPRB, 0.25110E+00_JPRB, 0.25518E+00_JPRB, 0.26420E+00_JPRB, 0.25050E+00_JPRB ,&
&  0.24022E+00_JPRB, 0.24075E+00_JPRB, 0.25083E+00_JPRB, 0.25207E+00_JPRB, 0.26065E+00_JPRB, 0.26707E+00_JPRB, 0.27541E+00_JPRB ,&
&  0.27801E+00_JPRB, 0.28202E+00_JPRB, 0.28354E+00_JPRB, 0.29073E+00_JPRB, 0.30035E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.29326E+00_JPRB, 0.31460E+00_JPRB, 0.32863E+00_JPRB, 0.34001E+00_JPRB, 0.34944E+00_JPRB, 0.34441E+00_JPRB, 0.34353E+00_JPRB ,&
&  0.34689E+00_JPRB, 0.34048E+00_JPRB, 0.32918E+00_JPRB, 0.32883E+00_JPRB, 0.33030E+00_JPRB, 0.32960E+00_JPRB, 0.33216E+00_JPRB ,&
&  0.28287E+00_JPRB, 0.30488E+00_JPRB, 0.27787E+00_JPRB, 0.29663E+00_JPRB, 0.35079E+00_JPRB, 0.32846E+00_JPRB, 0.32390E+00_JPRB ,&
&  0.31366E+00_JPRB, 0.30676E+00_JPRB, 0.30505E+00_JPRB, 0.32457E+00_JPRB, 0.36492E+00_JPRB, 0.49943E+00_JPRB, 0.61536E+00_JPRB ,&
&  0.59283E+00_JPRB, 0.56173E+00_JPRB, 0.54916E+00_JPRB, 0.55260E+00_JPRB, 0.54493E+00_JPRB, 0.54793E+00_JPRB, 0.57688E+00_JPRB ,&
&  0.58615E+00_JPRB, 0.57975E+00_JPRB, 0.59028E+00_JPRB, 0.61728E+00_JPRB, 0.63267E+00_JPRB, 0.62808E+00_JPRB, 0.63515E+00_JPRB ,&
&  0.69031E+00_JPRB, 0.77961E+00_JPRB, 0.80748E+00_JPRB, 0.86616E+00_JPRB, 0.95837E+00_JPRB, 0.14768E+01_JPRB, 0.21953E+01_JPRB ,&
&  0.23036E+01_JPRB, 0.22520E+01_JPRB, 0.23579E+01_JPRB, 0.25389E+01_JPRB, 0.24629E+01_JPRB, 0.22126E+01_JPRB, 0.18762E+01_JPRB ,&
&  0.15578E+01_JPRB, 0.12969E+01_JPRB, 0.10809E+01_JPRB, 0.96540E+00_JPRB, 0.84232E+00_JPRB, 0.80240E+00_JPRB, 0.73170E+00_JPRB ,&
&  0.61831E+00_JPRB, 0.56095E+00_JPRB, 0.54977E+00_JPRB, 0.50365E+00_JPRB, 0.46603E+00_JPRB, 0.45324E+00_JPRB, 0.42893E+00_JPRB ,&
&  0.39363E+00_JPRB, 0.35150E+00_JPRB, 0.29987E+00_JPRB, 0.29121E+00_JPRB, 0.26650E+00_JPRB, 0.26889E+00_JPRB, 0.28815E+00_JPRB ,&
&  0.28373E+00_JPRB, 0.26266E+00_JPRB, 0.24470E+00_JPRB, 0.23603E+00_JPRB, 0.24180E+00_JPRB, 0.25244E+00_JPRB, 0.23668E+00_JPRB ,&
&  0.21928E+00_JPRB, 0.22410E+00_JPRB, 0.22900E+00_JPRB, 0.23148E+00_JPRB, 0.23515E+00_JPRB, 0.24348E+00_JPRB, 0.24792E+00_JPRB ,&
&  0.25325E+00_JPRB, 0.25762E+00_JPRB, 0.26371E+00_JPRB, 0.26740E+00_JPRB, 0.28039E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.27958E+00_JPRB, 0.29711E+00_JPRB, 0.30832E+00_JPRB, 0.32423E+00_JPRB, 0.33597E+00_JPRB, 0.33342E+00_JPRB, 0.33153E+00_JPRB ,&
&  0.33044E+00_JPRB, 0.32779E+00_JPRB, 0.32322E+00_JPRB, 0.32208E+00_JPRB, 0.32216E+00_JPRB, 0.31842E+00_JPRB, 0.28453E+00_JPRB ,&
&  0.24573E+00_JPRB, 0.25719E+00_JPRB, 0.25698E+00_JPRB, 0.26741E+00_JPRB, 0.28535E+00_JPRB, 0.27655E+00_JPRB, 0.26443E+00_JPRB ,&
&  0.26472E+00_JPRB, 0.26273E+00_JPRB, 0.27236E+00_JPRB, 0.28938E+00_JPRB, 0.30868E+00_JPRB, 0.37738E+00_JPRB, 0.50414E+00_JPRB ,&
&  0.47371E+00_JPRB, 0.45742E+00_JPRB, 0.44439E+00_JPRB, 0.44783E+00_JPRB, 0.45800E+00_JPRB, 0.45439E+00_JPRB, 0.45395E+00_JPRB ,&
&  0.45537E+00_JPRB, 0.46780E+00_JPRB, 0.48732E+00_JPRB, 0.51298E+00_JPRB, 0.53654E+00_JPRB, 0.54497E+00_JPRB, 0.54699E+00_JPRB ,&
&  0.58100E+00_JPRB, 0.64832E+00_JPRB, 0.69530E+00_JPRB, 0.75583E+00_JPRB, 0.86437E+00_JPRB, 0.13500E+01_JPRB, 0.17934E+01_JPRB ,&
&  0.18502E+01_JPRB, 0.18600E+01_JPRB, 0.18148E+01_JPRB, 0.17842E+01_JPRB, 0.17715E+01_JPRB, 0.15508E+01_JPRB, 0.13020E+01_JPRB ,&
&  0.10903E+01_JPRB, 0.92874E+00_JPRB, 0.83965E+00_JPRB, 0.78537E+00_JPRB, 0.71025E+00_JPRB, 0.65248E+00_JPRB, 0.58812E+00_JPRB ,&
&  0.53542E+00_JPRB, 0.48000E+00_JPRB, 0.47477E+00_JPRB, 0.46045E+00_JPRB, 0.44324E+00_JPRB, 0.42497E+00_JPRB, 0.39898E+00_JPRB ,&
&  0.37092E+00_JPRB, 0.34600E+00_JPRB, 0.31457E+00_JPRB, 0.29795E+00_JPRB, 0.28433E+00_JPRB, 0.29366E+00_JPRB, 0.28258E+00_JPRB ,&
&  0.26181E+00_JPRB, 0.23904E+00_JPRB, 0.22988E+00_JPRB, 0.22600E+00_JPRB, 0.23065E+00_JPRB, 0.23268E+00_JPRB, 0.21725E+00_JPRB ,&
&  0.20903E+00_JPRB, 0.21696E+00_JPRB, 0.21854E+00_JPRB, 0.21806E+00_JPRB, 0.22013E+00_JPRB, 0.22213E+00_JPRB, 0.21512E+00_JPRB ,&
&  0.21911E+00_JPRB, 0.23929E+00_JPRB, 0.25221E+00_JPRB, 0.25581E+00_JPRB, 0.26635E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.26498E+00_JPRB, 0.28142E+00_JPRB, 0.29454E+00_JPRB, 0.31053E+00_JPRB, 0.32063E+00_JPRB, 0.32100E+00_JPRB, 0.30847E+00_JPRB ,&
&  0.31812E+00_JPRB, 0.31617E+00_JPRB, 0.31406E+00_JPRB, 0.31692E+00_JPRB, 0.31937E+00_JPRB, 0.28613E+00_JPRB, 0.23055E+00_JPRB ,&
&  0.21609E+00_JPRB, 0.22367E+00_JPRB, 0.24074E+00_JPRB, 0.25250E+00_JPRB, 0.25217E+00_JPRB, 0.24475E+00_JPRB, 0.23810E+00_JPRB ,&
&  0.23764E+00_JPRB, 0.24551E+00_JPRB, 0.26898E+00_JPRB, 0.28876E+00_JPRB, 0.29699E+00_JPRB, 0.31462E+00_JPRB, 0.41366E+00_JPRB ,&
&  0.41878E+00_JPRB, 0.40843E+00_JPRB, 0.40838E+00_JPRB, 0.38863E+00_JPRB, 0.41632E+00_JPRB, 0.39554E+00_JPRB, 0.38086E+00_JPRB ,&
&  0.38558E+00_JPRB, 0.40095E+00_JPRB, 0.42532E+00_JPRB, 0.45590E+00_JPRB, 0.47708E+00_JPRB, 0.48511E+00_JPRB, 0.49015E+00_JPRB ,&
&  0.50531E+00_JPRB, 0.54388E+00_JPRB, 0.58870E+00_JPRB, 0.65329E+00_JPRB, 0.74378E+00_JPRB, 0.10601E+01_JPRB, 0.13706E+01_JPRB ,&
&  0.14144E+01_JPRB, 0.14764E+01_JPRB, 0.14799E+01_JPRB, 0.12948E+01_JPRB, 0.13081E+01_JPRB, 0.11757E+01_JPRB, 0.10187E+01_JPRB ,&
&  0.88270E+00_JPRB, 0.75882E+00_JPRB, 0.73367E+00_JPRB, 0.70491E+00_JPRB, 0.65953E+00_JPRB, 0.58845E+00_JPRB, 0.52303E+00_JPRB ,&
&  0.47618E+00_JPRB, 0.42511E+00_JPRB, 0.39370E+00_JPRB, 0.37862E+00_JPRB, 0.36854E+00_JPRB, 0.36443E+00_JPRB, 0.35275E+00_JPRB ,&
&  0.33482E+00_JPRB, 0.31932E+00_JPRB, 0.30709E+00_JPRB, 0.29630E+00_JPRB, 0.28812E+00_JPRB, 0.28131E+00_JPRB, 0.27110E+00_JPRB ,&
&  0.25339E+00_JPRB, 0.23308E+00_JPRB, 0.22232E+00_JPRB, 0.22383E+00_JPRB, 0.22907E+00_JPRB, 0.22237E+00_JPRB, 0.20676E+00_JPRB ,&
&  0.19974E+00_JPRB, 0.20496E+00_JPRB, 0.21147E+00_JPRB, 0.20887E+00_JPRB, 0.20629E+00_JPRB, 0.20718E+00_JPRB, 0.20652E+00_JPRB ,&
&  0.20994E+00_JPRB, 0.22965E+00_JPRB, 0.24389E+00_JPRB, 0.24998E+00_JPRB, 0.25441E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.25646E+00_JPRB, 0.27235E+00_JPRB, 0.28675E+00_JPRB, 0.30025E+00_JPRB, 0.30898E+00_JPRB, 0.30038E+00_JPRB, 0.30240E+00_JPRB ,&
&  0.34452E+00_JPRB, 0.29602E+00_JPRB, 0.31409E+00_JPRB, 0.25689E+00_JPRB, 0.23530E+00_JPRB, 0.23031E+00_JPRB, 0.22145E+00_JPRB ,&
&  0.22065E+00_JPRB, 0.22705E+00_JPRB, 0.23530E+00_JPRB, 0.24109E+00_JPRB, 0.24363E+00_JPRB, 0.23367E+00_JPRB, 0.22735E+00_JPRB ,&
&  0.22558E+00_JPRB, 0.23813E+00_JPRB, 0.26837E+00_JPRB, 0.29121E+00_JPRB, 0.30494E+00_JPRB, 0.31797E+00_JPRB, 0.36334E+00_JPRB ,&
&  0.36949E+00_JPRB, 0.36019E+00_JPRB, 0.36926E+00_JPRB, 0.37282E+00_JPRB, 0.37738E+00_JPRB, 0.35861E+00_JPRB, 0.34128E+00_JPRB ,&
&  0.33871E+00_JPRB, 0.33376E+00_JPRB, 0.38150E+00_JPRB, 0.41356E+00_JPRB, 0.43313E+00_JPRB, 0.44103E+00_JPRB, 0.45270E+00_JPRB ,&
&  0.46535E+00_JPRB, 0.48739E+00_JPRB, 0.52518E+00_JPRB, 0.58154E+00_JPRB, 0.64519E+00_JPRB, 0.77285E+00_JPRB, 0.90273E+00_JPRB ,&
&  0.10803E+01_JPRB, 0.88544E+00_JPRB, 0.10572E+01_JPRB, 0.10257E+01_JPRB, 0.10797E+01_JPRB, 0.10132E+01_JPRB, 0.93024E+00_JPRB ,&
&  0.79981E+00_JPRB, 0.70483E+00_JPRB, 0.66523E+00_JPRB, 0.61028E+00_JPRB, 0.56607E+00_JPRB, 0.52004E+00_JPRB, 0.47876E+00_JPRB ,&
&  0.44001E+00_JPRB, 0.38385E+00_JPRB, 0.36686E+00_JPRB, 0.34218E+00_JPRB, 0.32749E+00_JPRB, 0.32144E+00_JPRB, 0.31738E+00_JPRB ,&
&  0.31036E+00_JPRB, 0.30334E+00_JPRB, 0.29278E+00_JPRB, 0.27748E+00_JPRB, 0.27526E+00_JPRB, 0.26985E+00_JPRB, 0.25734E+00_JPRB ,&
&  0.24855E+00_JPRB, 0.23814E+00_JPRB, 0.23278E+00_JPRB, 0.23601E+00_JPRB, 0.23841E+00_JPRB, 0.22882E+00_JPRB, 0.21064E+00_JPRB ,&
&  0.19733E+00_JPRB, 0.19369E+00_JPRB, 0.19207E+00_JPRB, 0.19037E+00_JPRB, 0.19004E+00_JPRB, 0.19515E+00_JPRB, 0.20168E+00_JPRB ,&
&  0.20831E+00_JPRB, 0.22124E+00_JPRB, 0.23636E+00_JPRB, 0.24417E+00_JPRB, 0.24805E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.25138E+00_JPRB, 0.26478E+00_JPRB, 0.27486E+00_JPRB, 0.28281E+00_JPRB, 0.28920E+00_JPRB, 0.28555E+00_JPRB, 0.27183E+00_JPRB ,&
&  0.28191E+00_JPRB, 0.24458E+00_JPRB, 0.22877E+00_JPRB, 0.22025E+00_JPRB, 0.21968E+00_JPRB, 0.21649E+00_JPRB, 0.20955E+00_JPRB ,&
&  0.20913E+00_JPRB, 0.21545E+00_JPRB, 0.22775E+00_JPRB, 0.23241E+00_JPRB, 0.23082E+00_JPRB, 0.22688E+00_JPRB, 0.22421E+00_JPRB ,&
&  0.22598E+00_JPRB, 0.24068E+00_JPRB, 0.26680E+00_JPRB, 0.29977E+00_JPRB, 0.31719E+00_JPRB, 0.32569E+00_JPRB, 0.33779E+00_JPRB ,&
&  0.34506E+00_JPRB, 0.34631E+00_JPRB, 0.35016E+00_JPRB, 0.36025E+00_JPRB, 0.35287E+00_JPRB, 0.32652E+00_JPRB, 0.29835E+00_JPRB ,&
&  0.25693E+00_JPRB, 0.26251E+00_JPRB, 0.34520E+00_JPRB, 0.37749E+00_JPRB, 0.39822E+00_JPRB, 0.40813E+00_JPRB, 0.41922E+00_JPRB ,&
&  0.43588E+00_JPRB, 0.45664E+00_JPRB, 0.48355E+00_JPRB, 0.51989E+00_JPRB, 0.55549E+00_JPRB, 0.61174E+00_JPRB, 0.66117E+00_JPRB ,&
&  0.77190E+00_JPRB, 0.64966E+00_JPRB, 0.62396E+00_JPRB, 0.66371E+00_JPRB, 0.90342E+00_JPRB, 0.90965E+00_JPRB, 0.84238E+00_JPRB ,&
&  0.76417E+00_JPRB, 0.66782E+00_JPRB, 0.62016E+00_JPRB, 0.57017E+00_JPRB, 0.52172E+00_JPRB, 0.47044E+00_JPRB, 0.42050E+00_JPRB ,&
&  0.38346E+00_JPRB, 0.35511E+00_JPRB, 0.33886E+00_JPRB, 0.32685E+00_JPRB, 0.31283E+00_JPRB, 0.30603E+00_JPRB, 0.30287E+00_JPRB ,&
&  0.29823E+00_JPRB, 0.29404E+00_JPRB, 0.28326E+00_JPRB, 0.27366E+00_JPRB, 0.26926E+00_JPRB, 0.26457E+00_JPRB, 0.25733E+00_JPRB ,&
&  0.25040E+00_JPRB, 0.24777E+00_JPRB, 0.24928E+00_JPRB, 0.25316E+00_JPRB, 0.24678E+00_JPRB, 0.22727E+00_JPRB, 0.20810E+00_JPRB ,&
&  0.19554E+00_JPRB, 0.18348E+00_JPRB, 0.17619E+00_JPRB, 0.17365E+00_JPRB, 0.17668E+00_JPRB, 0.18255E+00_JPRB, 0.18963E+00_JPRB ,&
&  0.19633E+00_JPRB, 0.20520E+00_JPRB, 0.21601E+00_JPRB, 0.22646E+00_JPRB, 0.23664E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.24933E+00_JPRB, 0.25976E+00_JPRB, 0.26230E+00_JPRB, 0.26399E+00_JPRB, 0.26169E+00_JPRB, 0.25752E+00_JPRB, 0.24955E+00_JPRB ,&
&  0.25370E+00_JPRB, 0.23891E+00_JPRB, 0.22701E+00_JPRB, 0.21684E+00_JPRB, 0.21088E+00_JPRB, 0.20683E+00_JPRB, 0.20819E+00_JPRB ,&
&  0.21403E+00_JPRB, 0.22228E+00_JPRB, 0.22746E+00_JPRB, 0.22626E+00_JPRB, 0.22452E+00_JPRB, 0.22333E+00_JPRB, 0.22625E+00_JPRB ,&
&  0.23156E+00_JPRB, 0.24287E+00_JPRB, 0.26122E+00_JPRB, 0.28666E+00_JPRB, 0.31146E+00_JPRB, 0.32619E+00_JPRB, 0.33003E+00_JPRB ,&
&  0.32985E+00_JPRB, 0.33139E+00_JPRB, 0.33730E+00_JPRB, 0.34321E+00_JPRB, 0.32515E+00_JPRB, 0.28652E+00_JPRB, 0.24995E+00_JPRB ,&
&  0.22196E+00_JPRB, 0.21784E+00_JPRB, 0.24487E+00_JPRB, 0.32882E+00_JPRB, 0.35153E+00_JPRB, 0.36643E+00_JPRB, 0.37987E+00_JPRB ,&
&  0.36609E+00_JPRB, 0.36865E+00_JPRB, 0.39639E+00_JPRB, 0.45187E+00_JPRB, 0.46634E+00_JPRB, 0.49790E+00_JPRB, 0.52137E+00_JPRB ,&
&  0.55172E+00_JPRB, 0.56006E+00_JPRB, 0.59088E+00_JPRB, 0.55248E+00_JPRB, 0.61611E+00_JPRB, 0.79975E+00_JPRB, 0.73520E+00_JPRB ,&
&  0.67103E+00_JPRB, 0.59220E+00_JPRB, 0.56450E+00_JPRB, 0.50539E+00_JPRB, 0.46615E+00_JPRB, 0.42685E+00_JPRB, 0.38376E+00_JPRB ,&
&  0.34557E+00_JPRB, 0.32369E+00_JPRB, 0.31848E+00_JPRB, 0.31651E+00_JPRB, 0.31407E+00_JPRB, 0.31068E+00_JPRB, 0.30834E+00_JPRB ,&
&  0.30363E+00_JPRB, 0.29543E+00_JPRB, 0.28086E+00_JPRB, 0.26797E+00_JPRB, 0.26458E+00_JPRB, 0.25979E+00_JPRB, 0.25403E+00_JPRB ,&
&  0.25019E+00_JPRB, 0.25225E+00_JPRB, 0.25764E+00_JPRB, 0.25649E+00_JPRB, 0.23970E+00_JPRB, 0.21671E+00_JPRB, 0.19965E+00_JPRB ,&
&  0.18659E+00_JPRB, 0.17715E+00_JPRB, 0.17129E+00_JPRB, 0.16965E+00_JPRB, 0.17407E+00_JPRB, 0.17891E+00_JPRB, 0.18299E+00_JPRB ,&
&  0.18801E+00_JPRB, 0.19373E+00_JPRB, 0.20653E+00_JPRB, 0.22014E+00_JPRB, 0.23419E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.23332E+00_JPRB, 0.24835E+00_JPRB, 0.25724E+00_JPRB, 0.25789E+00_JPRB, 0.25700E+00_JPRB, 0.24479E+00_JPRB, 0.23123E+00_JPRB ,&
&  0.22728E+00_JPRB, 0.22443E+00_JPRB, 0.22399E+00_JPRB, 0.22389E+00_JPRB, 0.22447E+00_JPRB, 0.22606E+00_JPRB, 0.22780E+00_JPRB ,&
&  0.22734E+00_JPRB, 0.22720E+00_JPRB, 0.22846E+00_JPRB, 0.23132E+00_JPRB, 0.23368E+00_JPRB, 0.23475E+00_JPRB, 0.23747E+00_JPRB ,&
&  0.24070E+00_JPRB, 0.24627E+00_JPRB, 0.25872E+00_JPRB, 0.27507E+00_JPRB, 0.29195E+00_JPRB, 0.30813E+00_JPRB, 0.31929E+00_JPRB ,&
&  0.32210E+00_JPRB, 0.31555E+00_JPRB, 0.31436E+00_JPRB, 0.31298E+00_JPRB, 0.29038E+00_JPRB, 0.26498E+00_JPRB, 0.23343E+00_JPRB ,&
&  0.21023E+00_JPRB, 0.19696E+00_JPRB, 0.19282E+00_JPRB, 0.20537E+00_JPRB, 0.24216E+00_JPRB, 0.31798E+00_JPRB, 0.33890E+00_JPRB ,&
&  0.35364E+00_JPRB, 0.36234E+00_JPRB, 0.36989E+00_JPRB, 0.38173E+00_JPRB, 0.39517E+00_JPRB, 0.41688E+00_JPRB, 0.43340E+00_JPRB ,&
&  0.44943E+00_JPRB, 0.45972E+00_JPRB, 0.47410E+00_JPRB, 0.44566E+00_JPRB, 0.44329E+00_JPRB, 0.48677E+00_JPRB, 0.52388E+00_JPRB ,&
&  0.51140E+00_JPRB, 0.44851E+00_JPRB, 0.43521E+00_JPRB, 0.42720E+00_JPRB, 0.41792E+00_JPRB, 0.41277E+00_JPRB, 0.39154E+00_JPRB ,&
&  0.36331E+00_JPRB, 0.33805E+00_JPRB, 0.32483E+00_JPRB, 0.31752E+00_JPRB, 0.31807E+00_JPRB, 0.32007E+00_JPRB, 0.31518E+00_JPRB ,&
&  0.30615E+00_JPRB, 0.29505E+00_JPRB, 0.27723E+00_JPRB, 0.26022E+00_JPRB, 0.25719E+00_JPRB, 0.25894E+00_JPRB, 0.25743E+00_JPRB ,&
&  0.25580E+00_JPRB, 0.25800E+00_JPRB, 0.26240E+00_JPRB, 0.25827E+00_JPRB, 0.24099E+00_JPRB, 0.21936E+00_JPRB, 0.20209E+00_JPRB ,&
&  0.19048E+00_JPRB, 0.18321E+00_JPRB, 0.17938E+00_JPRB, 0.17807E+00_JPRB, 0.17870E+00_JPRB, 0.18096E+00_JPRB, 0.18122E+00_JPRB ,&
&  0.18404E+00_JPRB, 0.18903E+00_JPRB, 0.19787E+00_JPRB, 0.20981E+00_JPRB, 0.22177E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.23284E+00_JPRB, 0.24313E+00_JPRB, 0.25006E+00_JPRB, 0.25260E+00_JPRB, 0.25199E+00_JPRB, 0.24725E+00_JPRB, 0.24287E+00_JPRB ,&
&  0.24215E+00_JPRB, 0.24415E+00_JPRB, 0.24404E+00_JPRB, 0.24335E+00_JPRB, 0.24237E+00_JPRB, 0.24143E+00_JPRB, 0.24141E+00_JPRB ,&
&  0.24215E+00_JPRB, 0.24387E+00_JPRB, 0.24646E+00_JPRB, 0.24676E+00_JPRB, 0.24623E+00_JPRB, 0.24658E+00_JPRB, 0.24654E+00_JPRB ,&
&  0.24841E+00_JPRB, 0.25364E+00_JPRB, 0.26216E+00_JPRB, 0.27200E+00_JPRB, 0.27918E+00_JPRB, 0.28569E+00_JPRB, 0.28739E+00_JPRB ,&
&  0.28548E+00_JPRB, 0.28769E+00_JPRB, 0.29384E+00_JPRB, 0.28052E+00_JPRB, 0.25960E+00_JPRB, 0.24204E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.19434E+00_JPRB, 0.17873E+00_JPRB, 0.16895E+00_JPRB, 0.16666E+00_JPRB, 0.17303E+00_JPRB, 0.19040E+00_JPRB, 0.22882E+00_JPRB ,&
&  0.30296E+00_JPRB, 0.31816E+00_JPRB, 0.32357E+00_JPRB, 0.33536E+00_JPRB, 0.35077E+00_JPRB, 0.36790E+00_JPRB, 0.38341E+00_JPRB ,&
&  0.39214E+00_JPRB, 0.39403E+00_JPRB, 0.39510E+00_JPRB, 0.39721E+00_JPRB, 0.39997E+00_JPRB, 0.38493E+00_JPRB, 0.39612E+00_JPRB ,&
&  0.41958E+00_JPRB, 0.39921E+00_JPRB, 0.36279E+00_JPRB, 0.35297E+00_JPRB, 0.35580E+00_JPRB, 0.36130E+00_JPRB, 0.37022E+00_JPRB ,&
&  0.36832E+00_JPRB, 0.35678E+00_JPRB, 0.34302E+00_JPRB, 0.32972E+00_JPRB, 0.32421E+00_JPRB, 0.32033E+00_JPRB, 0.31361E+00_JPRB ,&
&  0.30193E+00_JPRB, 0.28873E+00_JPRB, 0.28069E+00_JPRB, 0.26580E+00_JPRB, 0.26263E+00_JPRB, 0.26497E+00_JPRB, 0.26025E+00_JPRB ,&
&  0.25914E+00_JPRB, 0.26116E+00_JPRB, 0.26340E+00_JPRB, 0.26098E+00_JPRB, 0.25141E+00_JPRB, 0.23457E+00_JPRB, 0.21726E+00_JPRB ,&
&  0.20466E+00_JPRB, 0.19426E+00_JPRB, 0.18608E+00_JPRB, 0.18357E+00_JPRB, 0.18429E+00_JPRB, 0.18403E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18460E+00_JPRB, 0.18983E+00_JPRB, 0.19951E+00_JPRB, 0.21414E+00_JPRB, 0.22445E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.22531E+00_JPRB, 0.23491E+00_JPRB, 0.24277E+00_JPRB, 0.24690E+00_JPRB, 0.24776E+00_JPRB, 0.24782E+00_JPRB, 0.24678E+00_JPRB ,&
&  0.24639E+00_JPRB, 0.24647E+00_JPRB, 0.24667E+00_JPRB, 0.24693E+00_JPRB, 0.24718E+00_JPRB, 0.24678E+00_JPRB, 0.24753E+00_JPRB ,&
&  0.24893E+00_JPRB, 0.24986E+00_JPRB, 0.25183E+00_JPRB, 0.25520E+00_JPRB, 0.25508E+00_JPRB, 0.25281E+00_JPRB, 0.24886E+00_JPRB ,&
&  0.24872E+00_JPRB, 0.25377E+00_JPRB, 0.25803E+00_JPRB, 0.25906E+00_JPRB, 0.25863E+00_JPRB, 0.25800E+00_JPRB, 0.26105E+00_JPRB ,&
&  0.26929E+00_JPRB, 0.26644E+00_JPRB, 0.26275E+00_JPRB, 0.24714E+00_JPRB, 0.23134E+00_JPRB, 0.22279E+00_JPRB, 0.19389E+00_JPRB ,&
&  0.17350E+00_JPRB, 0.16149E+00_JPRB, 0.15709E+00_JPRB, 0.15571E+00_JPRB, 0.15928E+00_JPRB, 0.16872E+00_JPRB, 0.18995E+00_JPRB ,&
&  0.22565E+00_JPRB, 0.28220E+00_JPRB, 0.28923E+00_JPRB, 0.30220E+00_JPRB, 0.32023E+00_JPRB, 0.33946E+00_JPRB, 0.35305E+00_JPRB ,&
&  0.35997E+00_JPRB, 0.36103E+00_JPRB, 0.36116E+00_JPRB, 0.35927E+00_JPRB, 0.36228E+00_JPRB, 0.36584E+00_JPRB, 0.36133E+00_JPRB ,&
&  0.35190E+00_JPRB, 0.32930E+00_JPRB, 0.32322E+00_JPRB, 0.31802E+00_JPRB, 0.32305E+00_JPRB, 0.33100E+00_JPRB, 0.32906E+00_JPRB ,&
&  0.32767E+00_JPRB, 0.33319E+00_JPRB, 0.32557E+00_JPRB, 0.31846E+00_JPRB, 0.31752E+00_JPRB, 0.31831E+00_JPRB, 0.31565E+00_JPRB ,&
&  0.30343E+00_JPRB, 0.29273E+00_JPRB, 0.28462E+00_JPRB, 0.27615E+00_JPRB, 0.27572E+00_JPRB, 0.27727E+00_JPRB, 0.27111E+00_JPRB ,&
&  0.26521E+00_JPRB, 0.26143E+00_JPRB, 0.25987E+00_JPRB, 0.25860E+00_JPRB, 0.25569E+00_JPRB, 0.24734E+00_JPRB, 0.23567E+00_JPRB ,&
&  0.22095E+00_JPRB, 0.20748E+00_JPRB, 0.19783E+00_JPRB, 0.19210E+00_JPRB, 0.18891E+00_JPRB, 0.18670E+00_JPRB, 0.18629E+00_JPRB ,&
&  0.18824E+00_JPRB, 0.19402E+00_JPRB, 0.20134E+00_JPRB, 0.20978E+00_JPRB, 0.21771E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.22239E+00_JPRB, 0.22883E+00_JPRB, 0.23401E+00_JPRB, 0.23712E+00_JPRB, 0.23869E+00_JPRB, 0.23965E+00_JPRB, 0.23915E+00_JPRB ,&
&  0.23875E+00_JPRB, 0.23863E+00_JPRB, 0.24076E+00_JPRB, 0.24319E+00_JPRB, 0.24655E+00_JPRB, 0.24933E+00_JPRB, 0.25367E+00_JPRB ,&
&  0.25903E+00_JPRB, 0.26368E+00_JPRB, 0.26322E+00_JPRB, 0.26016E+00_JPRB, 0.25696E+00_JPRB, 0.25458E+00_JPRB, 0.24761E+00_JPRB ,&
&  0.24282E+00_JPRB, 0.24606E+00_JPRB, 0.24775E+00_JPRB, 0.24645E+00_JPRB, 0.24564E+00_JPRB, 0.24366E+00_JPRB, 0.24641E+00_JPRB ,&
&  0.24680E+00_JPRB, 0.24192E+00_JPRB, 0.23495E+00_JPRB, 0.22477E+00_JPRB, 0.21527E+00_JPRB, 0.18352E+00_JPRB, 0.17463E+00_JPRB ,&
&  0.16447E+00_JPRB, 0.15670E+00_JPRB, 0.15427E+00_JPRB, 0.15341E+00_JPRB, 0.15501E+00_JPRB, 0.16042E+00_JPRB, 0.16884E+00_JPRB ,&
&  0.18116E+00_JPRB, 0.25094E+00_JPRB, 0.26112E+00_JPRB, 0.26831E+00_JPRB, 0.28249E+00_JPRB, 0.30042E+00_JPRB, 0.31571E+00_JPRB ,&
&  0.32519E+00_JPRB, 0.32722E+00_JPRB, 0.32839E+00_JPRB, 0.32772E+00_JPRB, 0.32995E+00_JPRB, 0.33413E+00_JPRB, 0.33398E+00_JPRB ,&
&  0.32824E+00_JPRB, 0.31819E+00_JPRB, 0.31909E+00_JPRB, 0.31669E+00_JPRB, 0.30914E+00_JPRB, 0.30956E+00_JPRB, 0.30868E+00_JPRB ,&
&  0.30495E+00_JPRB, 0.29788E+00_JPRB, 0.29594E+00_JPRB, 0.29837E+00_JPRB, 0.30522E+00_JPRB, 0.30998E+00_JPRB, 0.30872E+00_JPRB ,&
&  0.30236E+00_JPRB, 0.29246E+00_JPRB, 0.28227E+00_JPRB, 0.27832E+00_JPRB, 0.27674E+00_JPRB, 0.27382E+00_JPRB, 0.27039E+00_JPRB ,&
&  0.26458E+00_JPRB, 0.26081E+00_JPRB, 0.25775E+00_JPRB, 0.25726E+00_JPRB, 0.25497E+00_JPRB, 0.25080E+00_JPRB, 0.24436E+00_JPRB ,&
&  0.23454E+00_JPRB, 0.22314E+00_JPRB, 0.21469E+00_JPRB, 0.20703E+00_JPRB, 0.20237E+00_JPRB, 0.19739E+00_JPRB, 0.19641E+00_JPRB ,&
&  0.19735E+00_JPRB, 0.20146E+00_JPRB, 0.20466E+00_JPRB, 0.20995E+00_JPRB, 0.21509E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.22747E+00_JPRB, 0.23081E+00_JPRB, 0.23320E+00_JPRB, 0.23473E+00_JPRB, 0.23546E+00_JPRB, 0.23692E+00_JPRB, 0.23822E+00_JPRB ,&
&  0.23993E+00_JPRB, 0.24135E+00_JPRB, 0.24384E+00_JPRB, 0.24622E+00_JPRB, 0.24947E+00_JPRB, 0.25274E+00_JPRB, 0.25671E+00_JPRB ,&
&  0.26060E+00_JPRB, 0.26164E+00_JPRB, 0.25910E+00_JPRB, 0.25647E+00_JPRB, 0.25374E+00_JPRB, 0.24860E+00_JPRB, 0.24079E+00_JPRB ,&
&  0.23432E+00_JPRB, 0.23096E+00_JPRB, 0.22731E+00_JPRB, 0.22644E+00_JPRB, 0.22240E+00_JPRB, 0.22188E+00_JPRB, 0.21821E+00_JPRB ,&
&  0.22303E+00_JPRB, 0.20339E+00_JPRB, 0.19880E+00_JPRB, 0.18487E+00_JPRB, 0.17445E+00_JPRB, 0.16559E+00_JPRB, 0.15947E+00_JPRB ,&
&  0.15658E+00_JPRB, 0.15343E+00_JPRB, 0.15285E+00_JPRB, 0.15358E+00_JPRB, 0.15570E+00_JPRB, 0.15991E+00_JPRB, 0.16514E+00_JPRB ,&
&  0.18035E+00_JPRB, 0.23797E+00_JPRB, 0.25458E+00_JPRB, 0.25773E+00_JPRB, 0.26221E+00_JPRB, 0.27454E+00_JPRB, 0.28667E+00_JPRB ,&
&  0.29959E+00_JPRB, 0.30281E+00_JPRB, 0.30233E+00_JPRB, 0.29961E+00_JPRB, 0.30626E+00_JPRB, 0.31279E+00_JPRB, 0.31600E+00_JPRB ,&
&  0.31685E+00_JPRB, 0.31777E+00_JPRB, 0.31804E+00_JPRB, 0.31590E+00_JPRB, 0.30743E+00_JPRB, 0.30010E+00_JPRB, 0.29050E+00_JPRB ,&
&  0.28426E+00_JPRB, 0.27824E+00_JPRB, 0.27707E+00_JPRB, 0.27963E+00_JPRB, 0.28580E+00_JPRB, 0.29167E+00_JPRB, 0.29474E+00_JPRB ,&
&  0.29435E+00_JPRB, 0.28953E+00_JPRB, 0.28480E+00_JPRB, 0.28140E+00_JPRB, 0.27903E+00_JPRB, 0.27556E+00_JPRB, 0.27336E+00_JPRB ,&
&  0.27062E+00_JPRB, 0.26408E+00_JPRB, 0.25911E+00_JPRB, 0.25526E+00_JPRB, 0.25209E+00_JPRB, 0.24867E+00_JPRB, 0.24554E+00_JPRB ,&
&  0.24170E+00_JPRB, 0.23693E+00_JPRB, 0.23197E+00_JPRB, 0.22563E+00_JPRB, 0.22058E+00_JPRB, 0.21562E+00_JPRB, 0.21385E+00_JPRB ,&
&  0.21284E+00_JPRB, 0.21480E+00_JPRB, 0.21616E+00_JPRB, 0.21956E+00_JPRB, 0.22289E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.23570E+00_JPRB, 0.23642E+00_JPRB, 0.23718E+00_JPRB, 0.23765E+00_JPRB, 0.23839E+00_JPRB, 0.23899E+00_JPRB, 0.24096E+00_JPRB ,&
&  0.24294E+00_JPRB, 0.24504E+00_JPRB, 0.24614E+00_JPRB, 0.24697E+00_JPRB, 0.24835E+00_JPRB, 0.25011E+00_JPRB, 0.25197E+00_JPRB ,&
&  0.25338E+00_JPRB, 0.25324E+00_JPRB, 0.25159E+00_JPRB, 0.24932E+00_JPRB, 0.24606E+00_JPRB, 0.24148E+00_JPRB, 0.23518E+00_JPRB ,&
&  0.22753E+00_JPRB, 0.22096E+00_JPRB, 0.20972E+00_JPRB, 0.20679E+00_JPRB, 0.20033E+00_JPRB, 0.19619E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19031E+00_JPRB, 0.18690E+00_JPRB, 0.18343E+00_JPRB, 0.17837E+00_JPRB, 0.17463E+00_JPRB, 0.16792E+00_JPRB, 0.16389E+00_JPRB ,&
&  0.16064E+00_JPRB, 0.15860E+00_JPRB, 0.15824E+00_JPRB, 0.15839E+00_JPRB, 0.16086E+00_JPRB, 0.16291E+00_JPRB, 0.16989E+00_JPRB ,&
&  0.18682E+00_JPRB, 0.21317E+00_JPRB, 0.25091E+00_JPRB, 0.25704E+00_JPRB, 0.25670E+00_JPRB, 0.26242E+00_JPRB, 0.26793E+00_JPRB ,&
&  0.27433E+00_JPRB, 0.27512E+00_JPRB, 0.27540E+00_JPRB, 0.27581E+00_JPRB, 0.27778E+00_JPRB, 0.27947E+00_JPRB, 0.28013E+00_JPRB ,&
&  0.28008E+00_JPRB, 0.28120E+00_JPRB, 0.28367E+00_JPRB, 0.28598E+00_JPRB, 0.28623E+00_JPRB, 0.28405E+00_JPRB, 0.28085E+00_JPRB ,&
&  0.27810E+00_JPRB, 0.27567E+00_JPRB, 0.27598E+00_JPRB, 0.27838E+00_JPRB, 0.28352E+00_JPRB, 0.28901E+00_JPRB, 0.29385E+00_JPRB ,&
&  0.29665E+00_JPRB, 0.29163E+00_JPRB, 0.28457E+00_JPRB, 0.27473E+00_JPRB, 0.26819E+00_JPRB, 0.26684E+00_JPRB, 0.26369E+00_JPRB ,&
&  0.26133E+00_JPRB, 0.25879E+00_JPRB, 0.25598E+00_JPRB, 0.25247E+00_JPRB, 0.25014E+00_JPRB, 0.24799E+00_JPRB, 0.24667E+00_JPRB ,&
&  0.24518E+00_JPRB, 0.24376E+00_JPRB, 0.24218E+00_JPRB, 0.23977E+00_JPRB, 0.23763E+00_JPRB, 0.23543E+00_JPRB, 0.23434E+00_JPRB ,&
&  0.23273E+00_JPRB, 0.23260E+00_JPRB, 0.23257E+00_JPRB, 0.23376E+00_JPRB, 0.23452E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.25143E+00_JPRB, 0.25147E+00_JPRB, 0.25161E+00_JPRB, 0.25160E+00_JPRB, 0.25159E+00_JPRB, 0.25109E+00_JPRB, 0.25042E+00_JPRB ,&
&  0.24978E+00_JPRB, 0.24903E+00_JPRB, 0.24794E+00_JPRB, 0.24666E+00_JPRB, 0.24509E+00_JPRB, 0.24388E+00_JPRB, 0.24307E+00_JPRB ,&
&  0.24274E+00_JPRB, 0.24187E+00_JPRB, 0.24051E+00_JPRB, 0.23784E+00_JPRB, 0.23563E+00_JPRB, 0.23404E+00_JPRB, 0.23219E+00_JPRB ,&
&  0.22805E+00_JPRB, 0.22391E+00_JPRB, 0.22014E+00_JPRB, 0.21531E+00_JPRB, 0.21133E+00_JPRB, 0.19744E+00_JPRB, 0.19390E+00_JPRB ,&
&  0.19236E+00_JPRB, 0.19055E+00_JPRB, 0.19043E+00_JPRB, 0.19051E+00_JPRB, 0.19224E+00_JPRB, 0.18945E+00_JPRB, 0.18767E+00_JPRB ,&
&  0.17965E+00_JPRB, 0.18022E+00_JPRB, 0.18152E+00_JPRB, 0.18299E+00_JPRB, 0.18723E+00_JPRB, 0.19127E+00_JPRB, 0.19975E+00_JPRB ,&
&  0.21610E+00_JPRB, 0.22311E+00_JPRB, 0.23441E+00_JPRB, 0.24315E+00_JPRB, 0.24760E+00_JPRB, 0.24967E+00_JPRB, 0.25136E+00_JPRB ,&
&  0.25357E+00_JPRB, 0.25683E+00_JPRB, 0.25844E+00_JPRB, 0.25919E+00_JPRB, 0.25820E+00_JPRB, 0.25606E+00_JPRB, 0.25408E+00_JPRB ,&
&  0.25127E+00_JPRB, 0.24938E+00_JPRB, 0.24873E+00_JPRB, 0.24979E+00_JPRB, 0.25199E+00_JPRB, 0.25406E+00_JPRB, 0.25707E+00_JPRB ,&
&  0.25987E+00_JPRB, 0.26381E+00_JPRB, 0.26673E+00_JPRB, 0.27026E+00_JPRB, 0.27512E+00_JPRB, 0.28026E+00_JPRB, 0.28374E+00_JPRB ,&
&  0.28614E+00_JPRB, 0.28365E+00_JPRB, 0.27846E+00_JPRB, 0.27203E+00_JPRB, 0.26539E+00_JPRB, 0.26010E+00_JPRB, 0.25713E+00_JPRB ,&
&  0.25497E+00_JPRB, 0.25374E+00_JPRB, 0.25303E+00_JPRB, 0.25180E+00_JPRB, 0.25124E+00_JPRB, 0.25041E+00_JPRB, 0.24979E+00_JPRB ,&
&  0.24894E+00_JPRB, 0.24856E+00_JPRB, 0.24872E+00_JPRB, 0.24921E+00_JPRB, 0.25023E+00_JPRB, 0.25048E+00_JPRB, 0.25080E+00_JPRB ,&
&  0.25059E+00_JPRB, 0.25128E+00_JPRB, 0.25106E+00_JPRB, 0.25113E+00_JPRB, 0.25106E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.24472E+00_JPRB, 0.24377E+00_JPRB, 0.24284E+00_JPRB, 0.24217E+00_JPRB, 0.24155E+00_JPRB, 0.24060E+00_JPRB, 0.23971E+00_JPRB ,&
&  0.23884E+00_JPRB, 0.23794E+00_JPRB, 0.23660E+00_JPRB, 0.23524E+00_JPRB, 0.23376E+00_JPRB, 0.23273E+00_JPRB, 0.23195E+00_JPRB ,&
&  0.23149E+00_JPRB, 0.23089E+00_JPRB, 0.23051E+00_JPRB, 0.22952E+00_JPRB, 0.22862E+00_JPRB, 0.22806E+00_JPRB, 0.22828E+00_JPRB ,&
&  0.22773E+00_JPRB, 0.22696E+00_JPRB, 0.22607E+00_JPRB, 0.22469E+00_JPRB, 0.22338E+00_JPRB, 0.22198E+00_JPRB, 0.22029E+00_JPRB ,&
&  0.21818E+00_JPRB, 0.21705E+00_JPRB, 0.21615E+00_JPRB, 0.21505E+00_JPRB, 0.21410E+00_JPRB, 0.21435E+00_JPRB, 0.21439E+00_JPRB ,&
&  0.21446E+00_JPRB, 0.21458E+00_JPRB, 0.21639E+00_JPRB, 0.21816E+00_JPRB, 0.22003E+00_JPRB, 0.22176E+00_JPRB, 0.22463E+00_JPRB ,&
&  0.22734E+00_JPRB, 0.23070E+00_JPRB, 0.23481E+00_JPRB, 0.23844E+00_JPRB, 0.24108E+00_JPRB, 0.24256E+00_JPRB, 0.24457E+00_JPRB ,&
&  0.24574E+00_JPRB, 0.24745E+00_JPRB, 0.24837E+00_JPRB, 0.24826E+00_JPRB, 0.24676E+00_JPRB, 0.24521E+00_JPRB, 0.24408E+00_JPRB ,&
&  0.24256E+00_JPRB, 0.24188E+00_JPRB, 0.24124E+00_JPRB, 0.24068E+00_JPRB, 0.23942E+00_JPRB, 0.23868E+00_JPRB, 0.23783E+00_JPRB ,&
&  0.23798E+00_JPRB, 0.23958E+00_JPRB, 0.24092E+00_JPRB, 0.24266E+00_JPRB, 0.24508E+00_JPRB, 0.24856E+00_JPRB, 0.25084E+00_JPRB ,&
&  0.25299E+00_JPRB, 0.25346E+00_JPRB, 0.25288E+00_JPRB, 0.25075E+00_JPRB, 0.24858E+00_JPRB, 0.24650E+00_JPRB, 0.24564E+00_JPRB ,&
&  0.24459E+00_JPRB, 0.24400E+00_JPRB, 0.24367E+00_JPRB, 0.24345E+00_JPRB, 0.24295E+00_JPRB, 0.24258E+00_JPRB, 0.24255E+00_JPRB ,&
&  0.24221E+00_JPRB, 0.24223E+00_JPRB, 0.24249E+00_JPRB, 0.24315E+00_JPRB, 0.24365E+00_JPRB, 0.24419E+00_JPRB, 0.24485E+00_JPRB ,&
&  0.24553E+00_JPRB, 0.24569E+00_JPRB, 0.24565E+00_JPRB, 0.24557E+00_JPRB, 0.24545E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.22848E+00_JPRB, 0.22852E+00_JPRB, 0.22824E+00_JPRB, 0.22825E+00_JPRB, 0.22816E+00_JPRB, 0.22845E+00_JPRB, 0.22842E+00_JPRB ,&
&  0.22847E+00_JPRB, 0.22856E+00_JPRB, 0.22891E+00_JPRB, 0.22880E+00_JPRB, 0.22888E+00_JPRB, 0.22890E+00_JPRB, 0.22903E+00_JPRB ,&
&  0.22897E+00_JPRB, 0.22908E+00_JPRB, 0.22914E+00_JPRB, 0.22920E+00_JPRB, 0.22930E+00_JPRB, 0.22935E+00_JPRB, 0.22908E+00_JPRB ,&
&  0.22825E+00_JPRB, 0.22725E+00_JPRB, 0.22653E+00_JPRB, 0.22609E+00_JPRB, 0.22602E+00_JPRB, 0.22607E+00_JPRB, 0.22610E+00_JPRB ,&
&  0.22628E+00_JPRB, 0.22672E+00_JPRB, 0.22716E+00_JPRB, 0.22753E+00_JPRB, 0.22778E+00_JPRB, 0.22840E+00_JPRB, 0.22883E+00_JPRB ,&
&  0.22920E+00_JPRB, 0.22949E+00_JPRB, 0.23013E+00_JPRB, 0.23071E+00_JPRB, 0.23138E+00_JPRB, 0.23201E+00_JPRB, 0.23285E+00_JPRB ,&
&  0.23382E+00_JPRB, 0.23467E+00_JPRB, 0.23558E+00_JPRB, 0.23652E+00_JPRB, 0.23739E+00_JPRB, 0.23830E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.24016E+00_JPRB, 0.24095E+00_JPRB, 0.24168E+00_JPRB, 0.24204E+00_JPRB, 0.24214E+00_JPRB, 0.24188E+00_JPRB, 0.24150E+00_JPRB ,&
&  0.24083E+00_JPRB, 0.24037E+00_JPRB, 0.23962E+00_JPRB, 0.23886E+00_JPRB, 0.23795E+00_JPRB, 0.23739E+00_JPRB, 0.23675E+00_JPRB ,&
&  0.23641E+00_JPRB, 0.23599E+00_JPRB, 0.23574E+00_JPRB, 0.23539E+00_JPRB, 0.23524E+00_JPRB, 0.23514E+00_JPRB, 0.23498E+00_JPRB ,&
&  0.23488E+00_JPRB, 0.23469E+00_JPRB, 0.23463E+00_JPRB, 0.23396E+00_JPRB, 0.23328E+00_JPRB, 0.23262E+00_JPRB, 0.23227E+00_JPRB ,&
&  0.23183E+00_JPRB, 0.23142E+00_JPRB, 0.23108E+00_JPRB, 0.23084E+00_JPRB, 0.23065E+00_JPRB, 0.23044E+00_JPRB, 0.23037E+00_JPRB ,&
&  0.23017E+00_JPRB, 0.23017E+00_JPRB, 0.23014E+00_JPRB, 0.23023E+00_JPRB, 0.22998E+00_JPRB, 0.22982E+00_JPRB, 0.22947E+00_JPRB ,&
&  0.22939E+00_JPRB, 0.22909E+00_JPRB, 0.22904E+00_JPRB, 0.22883E+00_JPRB, 0.22877E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB, 0.22875E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB, 0.22099E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.23413E+00_JPRB, 0.23446E+00_JPRB, 0.23475E+00_JPRB, 0.23504E+00_JPRB, 0.23446E+00_JPRB, 0.23518E+00_JPRB, 0.23571E+00_JPRB ,&
&  0.23642E+00_JPRB, 0.23741E+00_JPRB, 0.23861E+00_JPRB, 0.23954E+00_JPRB, 0.24029E+00_JPRB, 0.24074E+00_JPRB, 0.24103E+00_JPRB ,&
&  0.24104E+00_JPRB, 0.24116E+00_JPRB, 0.23852E+00_JPRB, 0.23761E+00_JPRB, 0.23738E+00_JPRB, 0.23750E+00_JPRB, 0.23773E+00_JPRB ,&
&  0.23792E+00_JPRB, 0.23800E+00_JPRB, 0.23767E+00_JPRB, 0.23794E+00_JPRB, 0.23823E+00_JPRB, 0.23844E+00_JPRB, 0.23870E+00_JPRB ,&
&  0.23892E+00_JPRB, 0.23909E+00_JPRB, 0.23903E+00_JPRB, 0.23885E+00_JPRB, 0.23866E+00_JPRB, 0.23810E+00_JPRB, 0.23723E+00_JPRB ,&
&  0.23618E+00_JPRB, 0.23490E+00_JPRB, 0.23361E+00_JPRB, 0.23244E+00_JPRB, 0.23141E+00_JPRB, 0.23053E+00_JPRB, 0.22970E+00_JPRB ,&
&  0.22873E+00_JPRB, 0.22750E+00_JPRB, 0.22620E+00_JPRB, 0.22494E+00_JPRB, 0.22375E+00_JPRB, 0.22266E+00_JPRB, 0.22153E+00_JPRB ,&
&  0.22066E+00_JPRB, 0.21991E+00_JPRB, 0.21916E+00_JPRB, 0.21851E+00_JPRB, 0.21789E+00_JPRB, 0.21731E+00_JPRB, 0.21675E+00_JPRB ,&
&  0.21601E+00_JPRB, 0.21557E+00_JPRB, 0.21507E+00_JPRB, 0.21480E+00_JPRB, 0.21478E+00_JPRB, 0.21474E+00_JPRB, 0.21477E+00_JPRB ,&
&  0.21486E+00_JPRB, 0.21502E+00_JPRB, 0.21511E+00_JPRB, 0.21528E+00_JPRB, 0.21555E+00_JPRB, 0.21592E+00_JPRB, 0.21638E+00_JPRB ,&
&  0.21687E+00_JPRB, 0.21742E+00_JPRB, 0.21800E+00_JPRB, 0.21868E+00_JPRB, 0.21938E+00_JPRB, 0.22016E+00_JPRB, 0.22089E+00_JPRB ,&
&  0.22163E+00_JPRB, 0.22232E+00_JPRB, 0.22300E+00_JPRB, 0.22352E+00_JPRB, 0.22424E+00_JPRB, 0.22494E+00_JPRB, 0.22577E+00_JPRB ,&
&  0.22661E+00_JPRB, 0.22750E+00_JPRB, 0.22828E+00_JPRB, 0.22910E+00_JPRB, 0.22998E+00_JPRB, 0.22992E+00_JPRB, 0.22988E+00_JPRB ,&
&  0.22992E+00_JPRB, 0.23020E+00_JPRB, 0.23033E+00_JPRB, 0.23070E+00_JPRB, 0.23149E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.26311E+00_JPRB, 0.26964E+00_JPRB, 0.27475E+00_JPRB, 0.28020E+00_JPRB, 0.27909E+00_JPRB, 0.31601E+00_JPRB, 0.31461E+00_JPRB ,&
&  0.31486E+00_JPRB, 0.29980E+00_JPRB, 0.29916E+00_JPRB, 0.29705E+00_JPRB, 0.29366E+00_JPRB, 0.28889E+00_JPRB, 0.28448E+00_JPRB ,&
&  0.28062E+00_JPRB, 0.27573E+00_JPRB, 0.27018E+00_JPRB, 0.26551E+00_JPRB, 0.26171E+00_JPRB, 0.25846E+00_JPRB, 0.25572E+00_JPRB ,&
&  0.25512E+00_JPRB, 0.25633E+00_JPRB, 0.25870E+00_JPRB, 0.25826E+00_JPRB, 0.25973E+00_JPRB, 0.26202E+00_JPRB, 0.26457E+00_JPRB ,&
&  0.26703E+00_JPRB, 0.26901E+00_JPRB, 0.26915E+00_JPRB, 0.26761E+00_JPRB, 0.26605E+00_JPRB, 0.26350E+00_JPRB, 0.26120E+00_JPRB ,&
&  0.25853E+00_JPRB, 0.25557E+00_JPRB, 0.25215E+00_JPRB, 0.24941E+00_JPRB, 0.24647E+00_JPRB, 0.24288E+00_JPRB, 0.23941E+00_JPRB ,&
&  0.23643E+00_JPRB, 0.23378E+00_JPRB, 0.23102E+00_JPRB, 0.22845E+00_JPRB, 0.22603E+00_JPRB, 0.22355E+00_JPRB, 0.22144E+00_JPRB ,&
&  0.21985E+00_JPRB, 0.21850E+00_JPRB, 0.21714E+00_JPRB, 0.21572E+00_JPRB, 0.21430E+00_JPRB, 0.21292E+00_JPRB, 0.21176E+00_JPRB ,&
&  0.21146E+00_JPRB, 0.21075E+00_JPRB, 0.21004E+00_JPRB, 0.20959E+00_JPRB, 0.20949E+00_JPRB, 0.20900E+00_JPRB, 0.20876E+00_JPRB ,&
&  0.20867E+00_JPRB, 0.20867E+00_JPRB, 0.20889E+00_JPRB, 0.20918E+00_JPRB, 0.20958E+00_JPRB, 0.21009E+00_JPRB, 0.21093E+00_JPRB ,&
&  0.21192E+00_JPRB, 0.21312E+00_JPRB, 0.21439E+00_JPRB, 0.21572E+00_JPRB, 0.21707E+00_JPRB, 0.21850E+00_JPRB, 0.22010E+00_JPRB ,&
&  0.22173E+00_JPRB, 0.22332E+00_JPRB, 0.22490E+00_JPRB, 0.22644E+00_JPRB, 0.22882E+00_JPRB, 0.23114E+00_JPRB, 0.23362E+00_JPRB ,&
&  0.23629E+00_JPRB, 0.23941E+00_JPRB, 0.24269E+00_JPRB, 0.24617E+00_JPRB, 0.24997E+00_JPRB, 0.25198E+00_JPRB, 0.25359E+00_JPRB ,&
&  0.25483E+00_JPRB, 0.25763E+00_JPRB, 0.26032E+00_JPRB, 0.26073E+00_JPRB, 0.26089E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.32170E+00_JPRB, 0.31915E+00_JPRB, 0.31920E+00_JPRB, 0.31967E+00_JPRB, 0.32241E+00_JPRB, 0.32702E+00_JPRB, 0.33211E+00_JPRB ,&
&  0.33150E+00_JPRB, 0.33431E+00_JPRB, 0.33365E+00_JPRB, 0.33151E+00_JPRB, 0.32779E+00_JPRB, 0.32470E+00_JPRB, 0.31987E+00_JPRB ,&
&  0.31300E+00_JPRB, 0.30422E+00_JPRB, 0.29564E+00_JPRB, 0.28701E+00_JPRB, 0.27875E+00_JPRB, 0.27016E+00_JPRB, 0.26460E+00_JPRB ,&
&  0.26144E+00_JPRB, 0.26056E+00_JPRB, 0.26252E+00_JPRB, 0.26551E+00_JPRB, 0.27095E+00_JPRB, 0.27832E+00_JPRB, 0.28804E+00_JPRB ,&
&  0.29777E+00_JPRB, 0.30823E+00_JPRB, 0.31361E+00_JPRB, 0.31353E+00_JPRB, 0.31320E+00_JPRB, 0.31104E+00_JPRB, 0.30752E+00_JPRB ,&
&  0.30127E+00_JPRB, 0.29400E+00_JPRB, 0.28578E+00_JPRB, 0.27709E+00_JPRB, 0.26822E+00_JPRB, 0.25955E+00_JPRB, 0.25149E+00_JPRB ,&
&  0.24492E+00_JPRB, 0.23933E+00_JPRB, 0.23430E+00_JPRB, 0.23011E+00_JPRB, 0.22685E+00_JPRB, 0.22439E+00_JPRB, 0.22161E+00_JPRB ,&
&  0.21922E+00_JPRB, 0.21713E+00_JPRB, 0.21468E+00_JPRB, 0.21293E+00_JPRB, 0.21139E+00_JPRB, 0.21036E+00_JPRB, 0.20950E+00_JPRB ,&
&  0.20928E+00_JPRB, 0.20906E+00_JPRB, 0.20940E+00_JPRB, 0.20973E+00_JPRB, 0.21017E+00_JPRB, 0.20997E+00_JPRB, 0.20964E+00_JPRB ,&
&  0.20915E+00_JPRB, 0.20872E+00_JPRB, 0.20757E+00_JPRB, 0.20671E+00_JPRB, 0.20613E+00_JPRB, 0.20551E+00_JPRB, 0.20557E+00_JPRB ,&
&  0.20645E+00_JPRB, 0.20735E+00_JPRB, 0.20815E+00_JPRB, 0.20968E+00_JPRB, 0.21123E+00_JPRB, 0.21342E+00_JPRB, 0.21602E+00_JPRB ,&
&  0.21796E+00_JPRB, 0.22025E+00_JPRB, 0.22281E+00_JPRB, 0.22577E+00_JPRB, 0.22926E+00_JPRB, 0.23293E+00_JPRB, 0.23718E+00_JPRB ,&
&  0.24195E+00_JPRB, 0.24692E+00_JPRB, 0.25254E+00_JPRB, 0.25870E+00_JPRB, 0.26620E+00_JPRB, 0.27380E+00_JPRB, 0.28453E+00_JPRB ,&
&  0.29347E+00_JPRB, 0.29553E+00_JPRB, 0.33576E+00_JPRB, 0.32945E+00_JPRB, 0.32503E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.31689E+00_JPRB, 0.31913E+00_JPRB, 0.32234E+00_JPRB, 0.32617E+00_JPRB, 0.33100E+00_JPRB, 0.33517E+00_JPRB, 0.34065E+00_JPRB ,&
&  0.34362E+00_JPRB, 0.33954E+00_JPRB, 0.33966E+00_JPRB, 0.33583E+00_JPRB, 0.33183E+00_JPRB, 0.32681E+00_JPRB, 0.32096E+00_JPRB ,&
&  0.31455E+00_JPRB, 0.30569E+00_JPRB, 0.29621E+00_JPRB, 0.28774E+00_JPRB, 0.28067E+00_JPRB, 0.27495E+00_JPRB, 0.27023E+00_JPRB ,&
&  0.26700E+00_JPRB, 0.26702E+00_JPRB, 0.26918E+00_JPRB, 0.27515E+00_JPRB, 0.28518E+00_JPRB, 0.29796E+00_JPRB, 0.31706E+00_JPRB ,&
&  0.34430E+00_JPRB, 0.34651E+00_JPRB, 0.35069E+00_JPRB, 0.34963E+00_JPRB, 0.34542E+00_JPRB, 0.34309E+00_JPRB, 0.33850E+00_JPRB ,&
&  0.33892E+00_JPRB, 0.33277E+00_JPRB, 0.34275E+00_JPRB, 0.32774E+00_JPRB, 0.30239E+00_JPRB, 0.28709E+00_JPRB, 0.26908E+00_JPRB ,&
&  0.25532E+00_JPRB, 0.24369E+00_JPRB, 0.23489E+00_JPRB, 0.23082E+00_JPRB, 0.22734E+00_JPRB, 0.22564E+00_JPRB, 0.22235E+00_JPRB ,&
&  0.21882E+00_JPRB, 0.21683E+00_JPRB, 0.21384E+00_JPRB, 0.21339E+00_JPRB, 0.21337E+00_JPRB, 0.21284E+00_JPRB, 0.21292E+00_JPRB ,&
&  0.21328E+00_JPRB, 0.21410E+00_JPRB, 0.21553E+00_JPRB, 0.21687E+00_JPRB, 0.21765E+00_JPRB, 0.21811E+00_JPRB, 0.21799E+00_JPRB ,&
&  0.21834E+00_JPRB, 0.21803E+00_JPRB, 0.21725E+00_JPRB, 0.21600E+00_JPRB, 0.21421E+00_JPRB, 0.21237E+00_JPRB, 0.21064E+00_JPRB ,&
&  0.20931E+00_JPRB, 0.20889E+00_JPRB, 0.20859E+00_JPRB, 0.20893E+00_JPRB, 0.20962E+00_JPRB, 0.21132E+00_JPRB, 0.21294E+00_JPRB ,&
&  0.21477E+00_JPRB, 0.21721E+00_JPRB, 0.21940E+00_JPRB, 0.22205E+00_JPRB, 0.22455E+00_JPRB, 0.22622E+00_JPRB, 0.23009E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.24140E+00_JPRB, 0.24885E+00_JPRB, 0.25628E+00_JPRB, 0.26579E+00_JPRB, 0.28151E+00_JPRB, 0.30320E+00_JPRB ,&
&  0.32577E+00_JPRB, 0.35168E+00_JPRB, 0.34685E+00_JPRB, 0.31723E+00_JPRB, 0.31454E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.32651E+00_JPRB, 0.33287E+00_JPRB, 0.33989E+00_JPRB, 0.34734E+00_JPRB, 0.35665E+00_JPRB, 0.36055E+00_JPRB, 0.36254E+00_JPRB ,&
&  0.35861E+00_JPRB, 0.35363E+00_JPRB, 0.34259E+00_JPRB, 0.33188E+00_JPRB, 0.32219E+00_JPRB, 0.31437E+00_JPRB, 0.30885E+00_JPRB ,&
&  0.30599E+00_JPRB, 0.30377E+00_JPRB, 0.30070E+00_JPRB, 0.29759E+00_JPRB, 0.29510E+00_JPRB, 0.29500E+00_JPRB, 0.29499E+00_JPRB ,&
&  0.29401E+00_JPRB, 0.29391E+00_JPRB, 0.29281E+00_JPRB, 0.29802E+00_JPRB, 0.30704E+00_JPRB, 0.31829E+00_JPRB, 0.33293E+00_JPRB ,&
&  0.35347E+00_JPRB, 0.36489E+00_JPRB, 0.38041E+00_JPRB, 0.38118E+00_JPRB, 0.37927E+00_JPRB, 0.37539E+00_JPRB, 0.37232E+00_JPRB ,&
&  0.36778E+00_JPRB, 0.36128E+00_JPRB, 0.35666E+00_JPRB, 0.35108E+00_JPRB, 0.32269E+00_JPRB, 0.29994E+00_JPRB, 0.28260E+00_JPRB ,&
&  0.26718E+00_JPRB, 0.25267E+00_JPRB, 0.24015E+00_JPRB, 0.23374E+00_JPRB, 0.22892E+00_JPRB, 0.22607E+00_JPRB, 0.22312E+00_JPRB ,&
&  0.22001E+00_JPRB, 0.21865E+00_JPRB, 0.21671E+00_JPRB, 0.21605E+00_JPRB, 0.21695E+00_JPRB, 0.21703E+00_JPRB, 0.21784E+00_JPRB ,&
&  0.21849E+00_JPRB, 0.21936E+00_JPRB, 0.22136E+00_JPRB, 0.22289E+00_JPRB, 0.22453E+00_JPRB, 0.22613E+00_JPRB, 0.22701E+00_JPRB ,&
&  0.22865E+00_JPRB, 0.22999E+00_JPRB, 0.22995E+00_JPRB, 0.22926E+00_JPRB, 0.22762E+00_JPRB, 0.22415E+00_JPRB, 0.22062E+00_JPRB ,&
&  0.21692E+00_JPRB, 0.21422E+00_JPRB, 0.21300E+00_JPRB, 0.21210E+00_JPRB, 0.21283E+00_JPRB, 0.21366E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21652E+00_JPRB, 0.21821E+00_JPRB, 0.22008E+00_JPRB, 0.22263E+00_JPRB, 0.22468E+00_JPRB, 0.22651E+00_JPRB, 0.22889E+00_JPRB ,&
&  0.22956E+00_JPRB, 0.23362E+00_JPRB, 0.23871E+00_JPRB, 0.24489E+00_JPRB, 0.25490E+00_JPRB, 0.26454E+00_JPRB, 0.28655E+00_JPRB ,&
&  0.32388E+00_JPRB, 0.39506E+00_JPRB, 0.38994E+00_JPRB, 0.31937E+00_JPRB, 0.32107E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.34912E+00_JPRB, 0.35763E+00_JPRB, 0.36526E+00_JPRB, 0.37111E+00_JPRB, 0.37460E+00_JPRB, 0.37514E+00_JPRB, 0.37770E+00_JPRB ,&
&  0.38496E+00_JPRB, 0.38916E+00_JPRB, 0.39190E+00_JPRB, 0.39142E+00_JPRB, 0.39081E+00_JPRB, 0.37159E+00_JPRB, 0.36626E+00_JPRB ,&
&  0.36055E+00_JPRB, 0.35659E+00_JPRB, 0.35135E+00_JPRB, 0.35148E+00_JPRB, 0.35140E+00_JPRB, 0.35328E+00_JPRB, 0.35400E+00_JPRB ,&
&  0.35376E+00_JPRB, 0.35134E+00_JPRB, 0.34803E+00_JPRB, 0.34598E+00_JPRB, 0.34733E+00_JPRB, 0.34902E+00_JPRB, 0.34629E+00_JPRB ,&
&  0.34221E+00_JPRB, 0.34145E+00_JPRB, 0.34482E+00_JPRB, 0.35259E+00_JPRB, 0.39382E+00_JPRB, 0.38913E+00_JPRB, 0.38634E+00_JPRB ,&
&  0.38306E+00_JPRB, 0.37901E+00_JPRB, 0.37862E+00_JPRB, 0.37239E+00_JPRB, 0.36312E+00_JPRB, 0.34320E+00_JPRB, 0.33203E+00_JPRB ,&
&  0.29728E+00_JPRB, 0.27776E+00_JPRB, 0.26052E+00_JPRB, 0.24921E+00_JPRB, 0.23956E+00_JPRB, 0.23399E+00_JPRB, 0.22884E+00_JPRB ,&
&  0.22496E+00_JPRB, 0.22200E+00_JPRB, 0.21936E+00_JPRB, 0.21969E+00_JPRB, 0.22129E+00_JPRB, 0.22233E+00_JPRB, 0.22334E+00_JPRB ,&
&  0.22395E+00_JPRB, 0.22443E+00_JPRB, 0.22508E+00_JPRB, 0.22629E+00_JPRB, 0.22787E+00_JPRB, 0.22994E+00_JPRB, 0.23256E+00_JPRB ,&
&  0.23688E+00_JPRB, 0.24036E+00_JPRB, 0.24193E+00_JPRB, 0.24425E+00_JPRB, 0.24391E+00_JPRB, 0.23886E+00_JPRB, 0.23274E+00_JPRB ,&
&  0.22651E+00_JPRB, 0.22322E+00_JPRB, 0.22006E+00_JPRB, 0.21790E+00_JPRB, 0.21741E+00_JPRB, 0.21749E+00_JPRB, 0.21933E+00_JPRB ,&
&  0.22132E+00_JPRB, 0.22302E+00_JPRB, 0.22418E+00_JPRB, 0.22546E+00_JPRB, 0.22711E+00_JPRB, 0.22845E+00_JPRB, 0.23021E+00_JPRB ,&
&  0.23243E+00_JPRB, 0.23549E+00_JPRB, 0.24003E+00_JPRB, 0.24524E+00_JPRB, 0.25158E+00_JPRB, 0.26144E+00_JPRB, 0.27931E+00_JPRB ,&
&  0.30398E+00_JPRB, 0.36106E+00_JPRB, 0.36775E+00_JPRB, 0.33620E+00_JPRB, 0.34187E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.36151E+00_JPRB, 0.37107E+00_JPRB, 0.37624E+00_JPRB, 0.38132E+00_JPRB, 0.38333E+00_JPRB, 0.38465E+00_JPRB, 0.39246E+00_JPRB ,&
&  0.39721E+00_JPRB, 0.40082E+00_JPRB, 0.40290E+00_JPRB, 0.40330E+00_JPRB, 0.39858E+00_JPRB, 0.39179E+00_JPRB, 0.38928E+00_JPRB ,&
&  0.38804E+00_JPRB, 0.38852E+00_JPRB, 0.38743E+00_JPRB, 0.38749E+00_JPRB, 0.38802E+00_JPRB, 0.39281E+00_JPRB, 0.39945E+00_JPRB ,&
&  0.40098E+00_JPRB, 0.40572E+00_JPRB, 0.41802E+00_JPRB, 0.42578E+00_JPRB, 0.43071E+00_JPRB, 0.43066E+00_JPRB, 0.42525E+00_JPRB ,&
&  0.39903E+00_JPRB, 0.37150E+00_JPRB, 0.34354E+00_JPRB, 0.34520E+00_JPRB, 0.39962E+00_JPRB, 0.40638E+00_JPRB, 0.40527E+00_JPRB ,&
&  0.40366E+00_JPRB, 0.40818E+00_JPRB, 0.40836E+00_JPRB, 0.39397E+00_JPRB, 0.38341E+00_JPRB, 0.36855E+00_JPRB, 0.35603E+00_JPRB ,&
&  0.34576E+00_JPRB, 0.33794E+00_JPRB, 0.32516E+00_JPRB, 0.29848E+00_JPRB, 0.28045E+00_JPRB, 0.27124E+00_JPRB, 0.25763E+00_JPRB ,&
&  0.24754E+00_JPRB, 0.24282E+00_JPRB, 0.24165E+00_JPRB, 0.24166E+00_JPRB, 0.24582E+00_JPRB, 0.24769E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24429E+00_JPRB, 0.24206E+00_JPRB, 0.23871E+00_JPRB, 0.23381E+00_JPRB, 0.23113E+00_JPRB, 0.23015E+00_JPRB, 0.23035E+00_JPRB ,&
&  0.23364E+00_JPRB, 0.23916E+00_JPRB, 0.25164E+00_JPRB, 0.26068E+00_JPRB, 0.25841E+00_JPRB, 0.24772E+00_JPRB, 0.23715E+00_JPRB ,&
&  0.23066E+00_JPRB, 0.22792E+00_JPRB, 0.22591E+00_JPRB, 0.22391E+00_JPRB, 0.22444E+00_JPRB, 0.22627E+00_JPRB, 0.22780E+00_JPRB ,&
&  0.22824E+00_JPRB, 0.22969E+00_JPRB, 0.23045E+00_JPRB, 0.23145E+00_JPRB, 0.23229E+00_JPRB, 0.23372E+00_JPRB, 0.23573E+00_JPRB ,&
&  0.23781E+00_JPRB, 0.24277E+00_JPRB, 0.24843E+00_JPRB, 0.25470E+00_JPRB, 0.25859E+00_JPRB, 0.26149E+00_JPRB, 0.26689E+00_JPRB ,&
&  0.27323E+00_JPRB, 0.28888E+00_JPRB, 0.31888E+00_JPRB, 0.35704E+00_JPRB, 0.35315E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.37090E+00_JPRB, 0.37782E+00_JPRB, 0.38588E+00_JPRB, 0.39661E+00_JPRB, 0.39902E+00_JPRB, 0.40068E+00_JPRB, 0.40992E+00_JPRB ,&
&  0.41061E+00_JPRB, 0.40783E+00_JPRB, 0.40448E+00_JPRB, 0.40441E+00_JPRB, 0.40609E+00_JPRB, 0.40812E+00_JPRB, 0.40721E+00_JPRB ,&
&  0.39784E+00_JPRB, 0.39237E+00_JPRB, 0.39479E+00_JPRB, 0.39752E+00_JPRB, 0.40188E+00_JPRB, 0.40837E+00_JPRB, 0.41391E+00_JPRB ,&
&  0.41681E+00_JPRB, 0.42020E+00_JPRB, 0.42234E+00_JPRB, 0.42836E+00_JPRB, 0.43788E+00_JPRB, 0.44441E+00_JPRB, 0.45101E+00_JPRB ,&
&  0.43828E+00_JPRB, 0.39092E+00_JPRB, 0.36009E+00_JPRB, 0.36692E+00_JPRB, 0.41818E+00_JPRB, 0.42808E+00_JPRB, 0.42898E+00_JPRB ,&
&  0.43221E+00_JPRB, 0.43997E+00_JPRB, 0.43938E+00_JPRB, 0.43337E+00_JPRB, 0.42173E+00_JPRB, 0.40308E+00_JPRB, 0.38020E+00_JPRB ,&
&  0.36482E+00_JPRB, 0.35439E+00_JPRB, 0.34770E+00_JPRB, 0.34137E+00_JPRB, 0.33799E+00_JPRB, 0.33307E+00_JPRB, 0.32557E+00_JPRB ,&
&  0.32270E+00_JPRB, 0.32114E+00_JPRB, 0.32207E+00_JPRB, 0.32395E+00_JPRB, 0.32622E+00_JPRB, 0.32801E+00_JPRB, 0.32641E+00_JPRB ,&
&  0.31404E+00_JPRB, 0.29892E+00_JPRB, 0.28445E+00_JPRB, 0.26870E+00_JPRB, 0.25505E+00_JPRB, 0.24620E+00_JPRB, 0.23927E+00_JPRB ,&
&  0.23631E+00_JPRB, 0.24116E+00_JPRB, 0.25663E+00_JPRB, 0.27495E+00_JPRB, 0.30865E+00_JPRB, 0.28483E+00_JPRB, 0.26120E+00_JPRB ,&
&  0.24919E+00_JPRB, 0.24343E+00_JPRB, 0.24021E+00_JPRB, 0.23994E+00_JPRB, 0.24133E+00_JPRB, 0.24174E+00_JPRB, 0.24178E+00_JPRB ,&
&  0.24283E+00_JPRB, 0.24495E+00_JPRB, 0.24677E+00_JPRB, 0.24828E+00_JPRB, 0.25062E+00_JPRB, 0.25337E+00_JPRB, 0.25482E+00_JPRB ,&
&  0.25537E+00_JPRB, 0.25960E+00_JPRB, 0.26862E+00_JPRB, 0.27902E+00_JPRB, 0.28963E+00_JPRB, 0.30073E+00_JPRB, 0.31854E+00_JPRB ,&
&  0.34115E+00_JPRB, 0.36923E+00_JPRB, 0.36797E+00_JPRB, 0.36727E+00_JPRB, 0.36757E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.39209E+00_JPRB, 0.40311E+00_JPRB, 0.41619E+00_JPRB, 0.42556E+00_JPRB, 0.42485E+00_JPRB, 0.42504E+00_JPRB, 0.43142E+00_JPRB ,&
&  0.42595E+00_JPRB, 0.41907E+00_JPRB, 0.41308E+00_JPRB, 0.40677E+00_JPRB, 0.40428E+00_JPRB, 0.40266E+00_JPRB, 0.40539E+00_JPRB ,&
&  0.40733E+00_JPRB, 0.40607E+00_JPRB, 0.40425E+00_JPRB, 0.40478E+00_JPRB, 0.41109E+00_JPRB, 0.41911E+00_JPRB, 0.42581E+00_JPRB ,&
&  0.42688E+00_JPRB, 0.43018E+00_JPRB, 0.43561E+00_JPRB, 0.44027E+00_JPRB, 0.44271E+00_JPRB, 0.44433E+00_JPRB, 0.44331E+00_JPRB ,&
&  0.44103E+00_JPRB, 0.43977E+00_JPRB, 0.42295E+00_JPRB, 0.43744E+00_JPRB, 0.44189E+00_JPRB, 0.44290E+00_JPRB, 0.44901E+00_JPRB ,&
&  0.45712E+00_JPRB, 0.46581E+00_JPRB, 0.46968E+00_JPRB, 0.47079E+00_JPRB, 0.46619E+00_JPRB, 0.45704E+00_JPRB, 0.43319E+00_JPRB ,&
&  0.41318E+00_JPRB, 0.39228E+00_JPRB, 0.37927E+00_JPRB, 0.37148E+00_JPRB, 0.36550E+00_JPRB, 0.36018E+00_JPRB, 0.35483E+00_JPRB ,&
&  0.34745E+00_JPRB, 0.34087E+00_JPRB, 0.33691E+00_JPRB, 0.33713E+00_JPRB, 0.34150E+00_JPRB, 0.35251E+00_JPRB, 0.36615E+00_JPRB ,&
&  0.37240E+00_JPRB, 0.36549E+00_JPRB, 0.35778E+00_JPRB, 0.34951E+00_JPRB, 0.33657E+00_JPRB, 0.30851E+00_JPRB, 0.29165E+00_JPRB ,&
&  0.28723E+00_JPRB, 0.29971E+00_JPRB, 0.32124E+00_JPRB, 0.32676E+00_JPRB, 0.33145E+00_JPRB, 0.32923E+00_JPRB, 0.32405E+00_JPRB ,&
&  0.31614E+00_JPRB, 0.29226E+00_JPRB, 0.27880E+00_JPRB, 0.27464E+00_JPRB, 0.27212E+00_JPRB, 0.27183E+00_JPRB, 0.27420E+00_JPRB ,&
&  0.27810E+00_JPRB, 0.28575E+00_JPRB, 0.29110E+00_JPRB, 0.29331E+00_JPRB, 0.29648E+00_JPRB, 0.29651E+00_JPRB, 0.29541E+00_JPRB ,&
&  0.29153E+00_JPRB, 0.29759E+00_JPRB, 0.31530E+00_JPRB, 0.34434E+00_JPRB, 0.35057E+00_JPRB, 0.35784E+00_JPRB, 0.36682E+00_JPRB ,&
&  0.37209E+00_JPRB, 0.37680E+00_JPRB, 0.37688E+00_JPRB, 0.37786E+00_JPRB, 0.38320E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.40996E+00_JPRB, 0.42233E+00_JPRB, 0.43822E+00_JPRB, 0.44970E+00_JPRB, 0.45487E+00_JPRB, 0.46053E+00_JPRB, 0.46315E+00_JPRB ,&
&  0.45847E+00_JPRB, 0.45660E+00_JPRB, 0.45092E+00_JPRB, 0.44259E+00_JPRB, 0.43535E+00_JPRB, 0.43155E+00_JPRB, 0.43170E+00_JPRB ,&
&  0.43376E+00_JPRB, 0.43553E+00_JPRB, 0.43198E+00_JPRB, 0.43015E+00_JPRB, 0.43394E+00_JPRB, 0.43909E+00_JPRB, 0.44240E+00_JPRB ,&
&  0.44357E+00_JPRB, 0.44480E+00_JPRB, 0.44942E+00_JPRB, 0.45245E+00_JPRB, 0.45271E+00_JPRB, 0.44637E+00_JPRB, 0.44180E+00_JPRB ,&
&  0.43713E+00_JPRB, 0.43566E+00_JPRB, 0.44282E+00_JPRB, 0.43464E+00_JPRB, 0.44973E+00_JPRB, 0.45190E+00_JPRB, 0.45771E+00_JPRB ,&
&  0.46620E+00_JPRB, 0.47982E+00_JPRB, 0.49000E+00_JPRB, 0.49770E+00_JPRB, 0.50022E+00_JPRB, 0.50360E+00_JPRB, 0.49408E+00_JPRB ,&
&  0.48058E+00_JPRB, 0.46212E+00_JPRB, 0.44367E+00_JPRB, 0.42400E+00_JPRB, 0.40556E+00_JPRB, 0.39109E+00_JPRB, 0.38380E+00_JPRB ,&
&  0.37498E+00_JPRB, 0.36127E+00_JPRB, 0.34802E+00_JPRB, 0.34274E+00_JPRB, 0.34728E+00_JPRB, 0.35742E+00_JPRB, 0.37306E+00_JPRB ,&
&  0.38256E+00_JPRB, 0.38549E+00_JPRB, 0.38235E+00_JPRB, 0.37791E+00_JPRB, 0.37337E+00_JPRB, 0.36662E+00_JPRB, 0.36279E+00_JPRB ,&
&  0.35953E+00_JPRB, 0.35378E+00_JPRB, 0.35073E+00_JPRB, 0.35258E+00_JPRB, 0.35582E+00_JPRB, 0.35778E+00_JPRB, 0.35588E+00_JPRB ,&
&  0.35252E+00_JPRB, 0.34818E+00_JPRB, 0.34012E+00_JPRB, 0.33532E+00_JPRB, 0.33508E+00_JPRB, 0.33929E+00_JPRB, 0.34696E+00_JPRB ,&
&  0.34948E+00_JPRB, 0.35525E+00_JPRB, 0.36100E+00_JPRB, 0.36325E+00_JPRB, 0.36595E+00_JPRB, 0.36828E+00_JPRB, 0.36858E+00_JPRB ,&
&  0.36694E+00_JPRB, 0.36195E+00_JPRB, 0.35484E+00_JPRB, 0.35137E+00_JPRB, 0.35201E+00_JPRB, 0.35828E+00_JPRB, 0.36718E+00_JPRB ,&
&  0.37607E+00_JPRB, 0.38942E+00_JPRB, 0.39969E+00_JPRB, 0.40203E+00_JPRB, 0.40416E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.43797E+00_JPRB, 0.44297E+00_JPRB, 0.45576E+00_JPRB, 0.46643E+00_JPRB, 0.46999E+00_JPRB, 0.47956E+00_JPRB, 0.49090E+00_JPRB ,&
&  0.50043E+00_JPRB, 0.50247E+00_JPRB, 0.49869E+00_JPRB, 0.48814E+00_JPRB, 0.47563E+00_JPRB, 0.46533E+00_JPRB, 0.46109E+00_JPRB ,&
&  0.46314E+00_JPRB, 0.46621E+00_JPRB, 0.47174E+00_JPRB, 0.47478E+00_JPRB, 0.47835E+00_JPRB, 0.47710E+00_JPRB, 0.47735E+00_JPRB ,&
&  0.47897E+00_JPRB, 0.47989E+00_JPRB, 0.48041E+00_JPRB, 0.47693E+00_JPRB, 0.47426E+00_JPRB, 0.47204E+00_JPRB, 0.46394E+00_JPRB ,&
&  0.45475E+00_JPRB, 0.44850E+00_JPRB, 0.45257E+00_JPRB, 0.46431E+00_JPRB, 0.47179E+00_JPRB, 0.46907E+00_JPRB, 0.46791E+00_JPRB ,&
&  0.46814E+00_JPRB, 0.47279E+00_JPRB, 0.48023E+00_JPRB, 0.49169E+00_JPRB, 0.50010E+00_JPRB, 0.51131E+00_JPRB, 0.51203E+00_JPRB ,&
&  0.50570E+00_JPRB, 0.49658E+00_JPRB, 0.49380E+00_JPRB, 0.49295E+00_JPRB, 0.46522E+00_JPRB, 0.43467E+00_JPRB, 0.41964E+00_JPRB ,&
&  0.41161E+00_JPRB, 0.40155E+00_JPRB, 0.38586E+00_JPRB, 0.37467E+00_JPRB, 0.37327E+00_JPRB, 0.37777E+00_JPRB, 0.38907E+00_JPRB ,&
&  0.39736E+00_JPRB, 0.40947E+00_JPRB, 0.42390E+00_JPRB, 0.43305E+00_JPRB, 0.43311E+00_JPRB, 0.42140E+00_JPRB, 0.41080E+00_JPRB ,&
&  0.40471E+00_JPRB, 0.40324E+00_JPRB, 0.40189E+00_JPRB, 0.39978E+00_JPRB, 0.39772E+00_JPRB, 0.39715E+00_JPRB, 0.39598E+00_JPRB ,&
&  0.39169E+00_JPRB, 0.38428E+00_JPRB, 0.37519E+00_JPRB, 0.36997E+00_JPRB, 0.36853E+00_JPRB, 0.36804E+00_JPRB, 0.36941E+00_JPRB ,&
&  0.37840E+00_JPRB, 0.38957E+00_JPRB, 0.39878E+00_JPRB, 0.40331E+00_JPRB, 0.40071E+00_JPRB, 0.39310E+00_JPRB, 0.38837E+00_JPRB ,&
&  0.38454E+00_JPRB, 0.38179E+00_JPRB, 0.37828E+00_JPRB, 0.37650E+00_JPRB, 0.37595E+00_JPRB, 0.38259E+00_JPRB, 0.39753E+00_JPRB ,&
&  0.41645E+00_JPRB, 0.44159E+00_JPRB, 0.45310E+00_JPRB, 0.45167E+00_JPRB, 0.44431E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.49113E+00_JPRB, 0.47436E+00_JPRB, 0.47466E+00_JPRB, 0.48156E+00_JPRB, 0.48554E+00_JPRB, 0.49348E+00_JPRB, 0.51308E+00_JPRB ,&
&  0.52480E+00_JPRB, 0.53234E+00_JPRB, 0.53523E+00_JPRB, 0.52970E+00_JPRB, 0.52840E+00_JPRB, 0.52102E+00_JPRB, 0.50826E+00_JPRB ,&
&  0.50299E+00_JPRB, 0.50191E+00_JPRB, 0.50347E+00_JPRB, 0.50711E+00_JPRB, 0.50912E+00_JPRB, 0.51164E+00_JPRB, 0.51452E+00_JPRB ,&
&  0.51605E+00_JPRB, 0.51595E+00_JPRB, 0.51513E+00_JPRB, 0.51544E+00_JPRB, 0.51467E+00_JPRB, 0.50750E+00_JPRB, 0.49942E+00_JPRB ,&
&  0.49045E+00_JPRB, 0.48144E+00_JPRB, 0.47917E+00_JPRB, 0.48391E+00_JPRB, 0.48913E+00_JPRB, 0.49230E+00_JPRB, 0.49005E+00_JPRB ,&
&  0.48526E+00_JPRB, 0.48209E+00_JPRB, 0.48384E+00_JPRB, 0.49074E+00_JPRB, 0.49946E+00_JPRB, 0.50913E+00_JPRB, 0.51419E+00_JPRB ,&
&  0.50880E+00_JPRB, 0.49158E+00_JPRB, 0.48119E+00_JPRB, 0.48093E+00_JPRB, 0.47816E+00_JPRB, 0.46482E+00_JPRB, 0.45526E+00_JPRB ,&
&  0.45055E+00_JPRB, 0.44193E+00_JPRB, 0.43275E+00_JPRB, 0.42001E+00_JPRB, 0.40825E+00_JPRB, 0.40385E+00_JPRB, 0.40239E+00_JPRB ,&
&  0.40516E+00_JPRB, 0.42192E+00_JPRB, 0.45328E+00_JPRB, 0.48463E+00_JPRB, 0.49887E+00_JPRB, 0.49305E+00_JPRB, 0.47720E+00_JPRB ,&
&  0.47044E+00_JPRB, 0.46790E+00_JPRB, 0.46949E+00_JPRB, 0.46899E+00_JPRB, 0.46524E+00_JPRB, 0.45664E+00_JPRB, 0.44983E+00_JPRB ,&
&  0.43995E+00_JPRB, 0.42898E+00_JPRB, 0.42444E+00_JPRB, 0.42155E+00_JPRB, 0.41841E+00_JPRB, 0.41826E+00_JPRB, 0.41700E+00_JPRB ,&
&  0.41706E+00_JPRB, 0.42073E+00_JPRB, 0.43239E+00_JPRB, 0.43936E+00_JPRB, 0.44104E+00_JPRB, 0.43090E+00_JPRB, 0.43151E+00_JPRB ,&
&  0.43370E+00_JPRB, 0.43118E+00_JPRB, 0.42652E+00_JPRB, 0.42616E+00_JPRB, 0.42591E+00_JPRB, 0.42752E+00_JPRB, 0.43864E+00_JPRB ,&
&  0.46492E+00_JPRB, 0.48821E+00_JPRB, 0.50737E+00_JPRB, 0.51293E+00_JPRB, 0.50869E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.56243E+00_JPRB, 0.53418E+00_JPRB, 0.52307E+00_JPRB, 0.52480E+00_JPRB, 0.52831E+00_JPRB, 0.52446E+00_JPRB, 0.53203E+00_JPRB ,&
&  0.54340E+00_JPRB, 0.55190E+00_JPRB, 0.55649E+00_JPRB, 0.55274E+00_JPRB, 0.55019E+00_JPRB, 0.54731E+00_JPRB, 0.53279E+00_JPRB ,&
&  0.52805E+00_JPRB, 0.52627E+00_JPRB, 0.53014E+00_JPRB, 0.53677E+00_JPRB, 0.54391E+00_JPRB, 0.55557E+00_JPRB, 0.56420E+00_JPRB ,&
&  0.56994E+00_JPRB, 0.57355E+00_JPRB, 0.57765E+00_JPRB, 0.57787E+00_JPRB, 0.57455E+00_JPRB, 0.56598E+00_JPRB, 0.55950E+00_JPRB ,&
&  0.55583E+00_JPRB, 0.54446E+00_JPRB, 0.53486E+00_JPRB, 0.53372E+00_JPRB, 0.53688E+00_JPRB, 0.53976E+00_JPRB, 0.53869E+00_JPRB ,&
&  0.52522E+00_JPRB, 0.50828E+00_JPRB, 0.50624E+00_JPRB, 0.50625E+00_JPRB, 0.51166E+00_JPRB, 0.52364E+00_JPRB, 0.52708E+00_JPRB ,&
&  0.52430E+00_JPRB, 0.51083E+00_JPRB, 0.49231E+00_JPRB, 0.48912E+00_JPRB, 0.49201E+00_JPRB, 0.48666E+00_JPRB, 0.48097E+00_JPRB ,&
&  0.47990E+00_JPRB, 0.47669E+00_JPRB, 0.47062E+00_JPRB, 0.46246E+00_JPRB, 0.44994E+00_JPRB, 0.43350E+00_JPRB, 0.42477E+00_JPRB ,&
&  0.42351E+00_JPRB, 0.43703E+00_JPRB, 0.46206E+00_JPRB, 0.49105E+00_JPRB, 0.51912E+00_JPRB, 0.53318E+00_JPRB, 0.53316E+00_JPRB ,&
&  0.52238E+00_JPRB, 0.52049E+00_JPRB, 0.52604E+00_JPRB, 0.53032E+00_JPRB, 0.53096E+00_JPRB, 0.52036E+00_JPRB, 0.50404E+00_JPRB ,&
&  0.49246E+00_JPRB, 0.48355E+00_JPRB, 0.48234E+00_JPRB, 0.48712E+00_JPRB, 0.49200E+00_JPRB, 0.49309E+00_JPRB, 0.48668E+00_JPRB ,&
&  0.47971E+00_JPRB, 0.47273E+00_JPRB, 0.46901E+00_JPRB, 0.47223E+00_JPRB, 0.47818E+00_JPRB, 0.48389E+00_JPRB, 0.49612E+00_JPRB ,&
&  0.51363E+00_JPRB, 0.53150E+00_JPRB, 0.53572E+00_JPRB, 0.53165E+00_JPRB, 0.53131E+00_JPRB, 0.52882E+00_JPRB, 0.52638E+00_JPRB ,&
&  0.53631E+00_JPRB, 0.55819E+00_JPRB, 0.57384E+00_JPRB, 0.58276E+00_JPRB, 0.58310E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.60902E+00_JPRB, 0.58470E+00_JPRB, 0.56707E+00_JPRB, 0.56053E+00_JPRB, 0.55709E+00_JPRB, 0.54548E+00_JPRB, 0.54064E+00_JPRB ,&
&  0.54687E+00_JPRB, 0.54884E+00_JPRB, 0.55610E+00_JPRB, 0.55600E+00_JPRB, 0.55527E+00_JPRB, 0.55765E+00_JPRB, 0.54993E+00_JPRB ,&
&  0.54976E+00_JPRB, 0.55848E+00_JPRB, 0.56868E+00_JPRB, 0.58252E+00_JPRB, 0.59416E+00_JPRB, 0.60300E+00_JPRB, 0.61060E+00_JPRB ,&
&  0.61368E+00_JPRB, 0.62027E+00_JPRB, 0.62398E+00_JPRB, 0.62007E+00_JPRB, 0.60983E+00_JPRB, 0.60202E+00_JPRB, 0.60094E+00_JPRB ,&
&  0.60029E+00_JPRB, 0.59381E+00_JPRB, 0.55024E+00_JPRB, 0.57382E+00_JPRB, 0.58065E+00_JPRB, 0.58286E+00_JPRB, 0.58377E+00_JPRB ,&
&  0.57752E+00_JPRB, 0.56197E+00_JPRB, 0.55500E+00_JPRB, 0.54387E+00_JPRB, 0.54277E+00_JPRB, 0.54864E+00_JPRB, 0.54781E+00_JPRB ,&
&  0.54473E+00_JPRB, 0.53567E+00_JPRB, 0.52271E+00_JPRB, 0.51250E+00_JPRB, 0.51027E+00_JPRB, 0.50910E+00_JPRB, 0.50257E+00_JPRB ,&
&  0.49777E+00_JPRB, 0.49681E+00_JPRB, 0.49496E+00_JPRB, 0.48936E+00_JPRB, 0.47500E+00_JPRB, 0.46143E+00_JPRB, 0.45831E+00_JPRB ,&
&  0.46073E+00_JPRB, 0.46653E+00_JPRB, 0.47910E+00_JPRB, 0.49587E+00_JPRB, 0.51441E+00_JPRB, 0.53213E+00_JPRB, 0.54852E+00_JPRB ,&
&  0.55542E+00_JPRB, 0.55961E+00_JPRB, 0.56324E+00_JPRB, 0.56761E+00_JPRB, 0.56827E+00_JPRB, 0.56074E+00_JPRB, 0.54237E+00_JPRB ,&
&  0.52290E+00_JPRB, 0.51818E+00_JPRB, 0.52243E+00_JPRB, 0.53164E+00_JPRB, 0.53646E+00_JPRB, 0.53581E+00_JPRB, 0.52889E+00_JPRB ,&
&  0.52422E+00_JPRB, 0.51563E+00_JPRB, 0.50704E+00_JPRB, 0.50295E+00_JPRB, 0.50291E+00_JPRB, 0.50910E+00_JPRB, 0.52628E+00_JPRB ,&
&  0.54671E+00_JPRB, 0.56955E+00_JPRB, 0.59029E+00_JPRB, 0.59815E+00_JPRB, 0.60224E+00_JPRB, 0.60177E+00_JPRB, 0.59314E+00_JPRB ,&
&  0.59707E+00_JPRB, 0.61542E+00_JPRB, 0.62473E+00_JPRB, 0.62604E+00_JPRB, 0.62385E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.65774E+00_JPRB, 0.63912E+00_JPRB, 0.62809E+00_JPRB, 0.61710E+00_JPRB, 0.60908E+00_JPRB, 0.59147E+00_JPRB, 0.58154E+00_JPRB ,&
&  0.57542E+00_JPRB, 0.57186E+00_JPRB, 0.57200E+00_JPRB, 0.57281E+00_JPRB, 0.58276E+00_JPRB, 0.58878E+00_JPRB, 0.59072E+00_JPRB ,&
&  0.59776E+00_JPRB, 0.61459E+00_JPRB, 0.62996E+00_JPRB, 0.64304E+00_JPRB, 0.65366E+00_JPRB, 0.66166E+00_JPRB, 0.66205E+00_JPRB ,&
&  0.66633E+00_JPRB, 0.67481E+00_JPRB, 0.67919E+00_JPRB, 0.67015E+00_JPRB, 0.65857E+00_JPRB, 0.65253E+00_JPRB, 0.65240E+00_JPRB ,&
&  0.64314E+00_JPRB, 0.59598E+00_JPRB, 0.60942E+00_JPRB, 0.62063E+00_JPRB, 0.63295E+00_JPRB, 0.63738E+00_JPRB, 0.64197E+00_JPRB ,&
&  0.64247E+00_JPRB, 0.62656E+00_JPRB, 0.60751E+00_JPRB, 0.59365E+00_JPRB, 0.59680E+00_JPRB, 0.59445E+00_JPRB, 0.58649E+00_JPRB ,&
&  0.58416E+00_JPRB, 0.58240E+00_JPRB, 0.57620E+00_JPRB, 0.56412E+00_JPRB, 0.55316E+00_JPRB, 0.54624E+00_JPRB, 0.54207E+00_JPRB ,&
&  0.53197E+00_JPRB, 0.52341E+00_JPRB, 0.51871E+00_JPRB, 0.51340E+00_JPRB, 0.50865E+00_JPRB, 0.51218E+00_JPRB, 0.51483E+00_JPRB ,&
&  0.51274E+00_JPRB, 0.51209E+00_JPRB, 0.51859E+00_JPRB, 0.53145E+00_JPRB, 0.54233E+00_JPRB, 0.55525E+00_JPRB, 0.57082E+00_JPRB ,&
&  0.58602E+00_JPRB, 0.59904E+00_JPRB, 0.60313E+00_JPRB, 0.60724E+00_JPRB, 0.60279E+00_JPRB, 0.59537E+00_JPRB, 0.57975E+00_JPRB ,&
&  0.56282E+00_JPRB, 0.56130E+00_JPRB, 0.56377E+00_JPRB, 0.57090E+00_JPRB, 0.57051E+00_JPRB, 0.56818E+00_JPRB, 0.56629E+00_JPRB ,&
&  0.56215E+00_JPRB, 0.55869E+00_JPRB, 0.55392E+00_JPRB, 0.54929E+00_JPRB, 0.55239E+00_JPRB, 0.55807E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.58893E+00_JPRB, 0.60770E+00_JPRB, 0.62751E+00_JPRB, 0.64150E+00_JPRB, 0.64668E+00_JPRB, 0.65002E+00_JPRB, 0.65316E+00_JPRB ,&
&  0.65400E+00_JPRB, 0.65224E+00_JPRB, 0.65812E+00_JPRB, 0.66238E+00_JPRB, 0.66695E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.68653E+00_JPRB, 0.67013E+00_JPRB, 0.66090E+00_JPRB, 0.65725E+00_JPRB, 0.64895E+00_JPRB, 0.63010E+00_JPRB, 0.61454E+00_JPRB ,&
&  0.60064E+00_JPRB, 0.59380E+00_JPRB, 0.59153E+00_JPRB, 0.59112E+00_JPRB, 0.59676E+00_JPRB, 0.60174E+00_JPRB, 0.60776E+00_JPRB ,&
&  0.61833E+00_JPRB, 0.63574E+00_JPRB, 0.65086E+00_JPRB, 0.66528E+00_JPRB, 0.67854E+00_JPRB, 0.69188E+00_JPRB, 0.69605E+00_JPRB ,&
&  0.69950E+00_JPRB, 0.70746E+00_JPRB, 0.70968E+00_JPRB, 0.69852E+00_JPRB, 0.68635E+00_JPRB, 0.68831E+00_JPRB, 0.68470E+00_JPRB ,&
&  0.67781E+00_JPRB, 0.56515E+00_JPRB, 0.61398E+00_JPRB, 0.63443E+00_JPRB, 0.65757E+00_JPRB, 0.67783E+00_JPRB, 0.68789E+00_JPRB ,&
&  0.68459E+00_JPRB, 0.67051E+00_JPRB, 0.65389E+00_JPRB, 0.62906E+00_JPRB, 0.62142E+00_JPRB, 0.61508E+00_JPRB, 0.60784E+00_JPRB ,&
&  0.60325E+00_JPRB, 0.59857E+00_JPRB, 0.59207E+00_JPRB, 0.58242E+00_JPRB, 0.56951E+00_JPRB, 0.56012E+00_JPRB, 0.55197E+00_JPRB ,&
&  0.55062E+00_JPRB, 0.54411E+00_JPRB, 0.53778E+00_JPRB, 0.53846E+00_JPRB, 0.54968E+00_JPRB, 0.55403E+00_JPRB, 0.55263E+00_JPRB ,&
&  0.55031E+00_JPRB, 0.54851E+00_JPRB, 0.55314E+00_JPRB, 0.56414E+00_JPRB, 0.57407E+00_JPRB, 0.58086E+00_JPRB, 0.59022E+00_JPRB ,&
&  0.60546E+00_JPRB, 0.61735E+00_JPRB, 0.61864E+00_JPRB, 0.61066E+00_JPRB, 0.60668E+00_JPRB, 0.60853E+00_JPRB, 0.60117E+00_JPRB ,&
&  0.58880E+00_JPRB, 0.58291E+00_JPRB, 0.58395E+00_JPRB, 0.58556E+00_JPRB, 0.58372E+00_JPRB, 0.57871E+00_JPRB, 0.57824E+00_JPRB ,&
&  0.57979E+00_JPRB, 0.58067E+00_JPRB, 0.57996E+00_JPRB, 0.58252E+00_JPRB, 0.58825E+00_JPRB, 0.59599E+00_JPRB, 0.60683E+00_JPRB ,&
&  0.62268E+00_JPRB, 0.63734E+00_JPRB, 0.64622E+00_JPRB, 0.65861E+00_JPRB, 0.67323E+00_JPRB, 0.67974E+00_JPRB, 0.69346E+00_JPRB ,&
&  0.69358E+00_JPRB, 0.68904E+00_JPRB, 0.68740E+00_JPRB, 0.69326E+00_JPRB, 0.69643E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.71398E+00_JPRB, 0.69836E+00_JPRB, 0.69087E+00_JPRB, 0.68622E+00_JPRB, 0.68241E+00_JPRB, 0.67581E+00_JPRB, 0.66287E+00_JPRB ,&
&  0.65020E+00_JPRB, 0.64415E+00_JPRB, 0.63858E+00_JPRB, 0.63074E+00_JPRB, 0.63184E+00_JPRB, 0.63737E+00_JPRB, 0.64789E+00_JPRB ,&
&  0.65965E+00_JPRB, 0.67492E+00_JPRB, 0.68976E+00_JPRB, 0.70884E+00_JPRB, 0.72195E+00_JPRB, 0.73149E+00_JPRB, 0.73693E+00_JPRB ,&
&  0.74073E+00_JPRB, 0.74152E+00_JPRB, 0.73494E+00_JPRB, 0.72658E+00_JPRB, 0.72408E+00_JPRB, 0.72418E+00_JPRB, 0.71873E+00_JPRB ,&
&  0.62360E+00_JPRB, 0.53563E+00_JPRB, 0.58454E+00_JPRB, 0.63992E+00_JPRB, 0.68990E+00_JPRB, 0.73111E+00_JPRB, 0.74822E+00_JPRB ,&
&  0.73924E+00_JPRB, 0.71576E+00_JPRB, 0.69325E+00_JPRB, 0.66575E+00_JPRB, 0.65600E+00_JPRB, 0.65265E+00_JPRB, 0.64513E+00_JPRB ,&
&  0.63600E+00_JPRB, 0.62538E+00_JPRB, 0.61293E+00_JPRB, 0.59336E+00_JPRB, 0.58017E+00_JPRB, 0.57066E+00_JPRB, 0.57027E+00_JPRB ,&
&  0.57341E+00_JPRB, 0.57765E+00_JPRB, 0.58112E+00_JPRB, 0.58323E+00_JPRB, 0.58917E+00_JPRB, 0.59639E+00_JPRB, 0.60361E+00_JPRB ,&
&  0.61237E+00_JPRB, 0.61744E+00_JPRB, 0.62810E+00_JPRB, 0.63667E+00_JPRB, 0.64500E+00_JPRB, 0.65111E+00_JPRB, 0.65583E+00_JPRB ,&
&  0.65492E+00_JPRB, 0.65205E+00_JPRB, 0.63849E+00_JPRB, 0.63169E+00_JPRB, 0.63323E+00_JPRB, 0.63615E+00_JPRB, 0.63501E+00_JPRB ,&
&  0.62832E+00_JPRB, 0.62032E+00_JPRB, 0.61246E+00_JPRB, 0.60969E+00_JPRB, 0.61247E+00_JPRB, 0.61238E+00_JPRB, 0.60891E+00_JPRB ,&
&  0.60602E+00_JPRB, 0.60634E+00_JPRB, 0.61211E+00_JPRB, 0.62308E+00_JPRB, 0.63740E+00_JPRB, 0.65514E+00_JPRB, 0.66959E+00_JPRB ,&
&  0.68478E+00_JPRB, 0.70024E+00_JPRB, 0.71217E+00_JPRB, 0.73160E+00_JPRB, 0.75018E+00_JPRB, 0.75924E+00_JPRB, 0.75539E+00_JPRB ,&
&  0.75256E+00_JPRB, 0.74031E+00_JPRB, 0.73335E+00_JPRB, 0.73182E+00_JPRB, 0.72603E+00_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.69197E+00_JPRB, 0.70145E+00_JPRB, 0.70904E+00_JPRB, 0.70758E+00_JPRB, 0.70491E+00_JPRB, 0.69766E+00_JPRB, 0.68293E+00_JPRB ,&
&  0.66911E+00_JPRB, 0.66394E+00_JPRB, 0.66419E+00_JPRB, 0.66583E+00_JPRB, 0.66889E+00_JPRB, 0.67531E+00_JPRB, 0.68678E+00_JPRB ,&
&  0.70048E+00_JPRB, 0.71501E+00_JPRB, 0.73227E+00_JPRB, 0.74853E+00_JPRB, 0.75269E+00_JPRB, 0.75009E+00_JPRB, 0.75090E+00_JPRB ,&
&  0.75759E+00_JPRB, 0.75701E+00_JPRB, 0.75152E+00_JPRB, 0.75465E+00_JPRB, 0.76218E+00_JPRB, 0.75291E+00_JPRB, 0.73451E+00_JPRB ,&
&  0.72138E+00_JPRB, 0.57225E+00_JPRB, 0.60234E+00_JPRB, 0.65690E+00_JPRB, 0.70822E+00_JPRB, 0.76235E+00_JPRB, 0.78591E+00_JPRB ,&
&  0.78056E+00_JPRB, 0.75538E+00_JPRB, 0.73410E+00_JPRB, 0.70685E+00_JPRB, 0.69661E+00_JPRB, 0.68755E+00_JPRB, 0.67487E+00_JPRB ,&
&  0.65649E+00_JPRB, 0.63741E+00_JPRB, 0.61639E+00_JPRB, 0.59878E+00_JPRB, 0.58466E+00_JPRB, 0.57922E+00_JPRB, 0.57863E+00_JPRB ,&
&  0.58003E+00_JPRB, 0.59142E+00_JPRB, 0.59711E+00_JPRB, 0.60531E+00_JPRB, 0.61764E+00_JPRB, 0.63378E+00_JPRB, 0.64992E+00_JPRB ,&
&  0.66315E+00_JPRB, 0.67169E+00_JPRB, 0.68045E+00_JPRB, 0.68783E+00_JPRB, 0.69763E+00_JPRB, 0.70512E+00_JPRB, 0.70015E+00_JPRB ,&
&  0.68147E+00_JPRB, 0.65813E+00_JPRB, 0.65721E+00_JPRB, 0.66297E+00_JPRB, 0.66199E+00_JPRB, 0.66323E+00_JPRB, 0.66408E+00_JPRB ,&
&  0.65942E+00_JPRB, 0.65307E+00_JPRB, 0.65132E+00_JPRB, 0.64693E+00_JPRB, 0.64440E+00_JPRB, 0.64544E+00_JPRB, 0.63552E+00_JPRB ,&
&  0.62885E+00_JPRB, 0.62927E+00_JPRB, 0.63786E+00_JPRB, 0.65750E+00_JPRB, 0.68362E+00_JPRB, 0.70758E+00_JPRB, 0.72278E+00_JPRB ,&
&  0.73155E+00_JPRB, 0.74170E+00_JPRB, 0.74964E+00_JPRB, 0.77661E+00_JPRB, 0.78528E+00_JPRB, 0.78869E+00_JPRB, 0.78704E+00_JPRB ,&
&  0.78139E+00_JPRB, 0.77605E+00_JPRB, 0.59093E+00_JPRB, 0.65405E+00_JPRB, 0.67077E+00_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.70490E+00_JPRB, 0.73282E+00_JPRB, 0.74541E+00_JPRB, 0.74869E+00_JPRB, 0.74729E+00_JPRB, 0.73588E+00_JPRB, 0.72230E+00_JPRB ,&
&  0.70544E+00_JPRB, 0.69855E+00_JPRB, 0.69617E+00_JPRB, 0.70098E+00_JPRB, 0.70762E+00_JPRB, 0.71729E+00_JPRB, 0.73054E+00_JPRB ,&
&  0.74313E+00_JPRB, 0.75008E+00_JPRB, 0.75586E+00_JPRB, 0.75510E+00_JPRB, 0.75412E+00_JPRB, 0.75711E+00_JPRB, 0.76192E+00_JPRB ,&
&  0.77150E+00_JPRB, 0.77531E+00_JPRB, 0.77328E+00_JPRB, 0.78186E+00_JPRB, 0.79388E+00_JPRB, 0.78340E+00_JPRB, 0.76332E+00_JPRB ,&
&  0.74462E+00_JPRB, 0.60052E+00_JPRB, 0.61501E+00_JPRB, 0.70021E+00_JPRB, 0.75647E+00_JPRB, 0.82909E+00_JPRB, 0.85627E+00_JPRB ,&
&  0.84566E+00_JPRB, 0.81597E+00_JPRB, 0.78240E+00_JPRB, 0.75307E+00_JPRB, 0.73340E+00_JPRB, 0.70955E+00_JPRB, 0.69397E+00_JPRB ,&
&  0.68333E+00_JPRB, 0.67209E+00_JPRB, 0.65403E+00_JPRB, 0.64011E+00_JPRB, 0.62896E+00_JPRB, 0.61785E+00_JPRB, 0.61244E+00_JPRB ,&
&  0.61587E+00_JPRB, 0.62782E+00_JPRB, 0.63978E+00_JPRB, 0.65277E+00_JPRB, 0.67752E+00_JPRB, 0.69915E+00_JPRB, 0.71889E+00_JPRB ,&
&  0.72617E+00_JPRB, 0.73189E+00_JPRB, 0.74080E+00_JPRB, 0.74794E+00_JPRB, 0.75982E+00_JPRB, 0.76747E+00_JPRB, 0.75446E+00_JPRB ,&
&  0.72107E+00_JPRB, 0.70217E+00_JPRB, 0.69161E+00_JPRB, 0.68560E+00_JPRB, 0.68780E+00_JPRB, 0.69619E+00_JPRB, 0.70005E+00_JPRB ,&
&  0.69724E+00_JPRB, 0.69352E+00_JPRB, 0.68869E+00_JPRB, 0.68359E+00_JPRB, 0.68640E+00_JPRB, 0.69090E+00_JPRB, 0.68917E+00_JPRB ,&
&  0.68615E+00_JPRB, 0.69002E+00_JPRB, 0.70112E+00_JPRB, 0.72195E+00_JPRB, 0.74667E+00_JPRB, 0.77276E+00_JPRB, 0.78770E+00_JPRB ,&
&  0.79794E+00_JPRB, 0.80915E+00_JPRB, 0.82836E+00_JPRB, 0.78505E+00_JPRB, 0.81774E+00_JPRB, 0.82611E+00_JPRB, 0.82632E+00_JPRB ,&
&  0.81070E+00_JPRB, 0.78547E+00_JPRB, 0.75579E+00_JPRB, 0.62290E+00_JPRB, 0.67384E+00_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.74607E+00_JPRB, 0.77532E+00_JPRB, 0.78501E+00_JPRB, 0.78674E+00_JPRB, 0.77731E+00_JPRB, 0.76340E+00_JPRB, 0.75224E+00_JPRB ,&
&  0.74786E+00_JPRB, 0.73508E+00_JPRB, 0.73020E+00_JPRB, 0.72955E+00_JPRB, 0.73469E+00_JPRB, 0.74382E+00_JPRB, 0.74620E+00_JPRB ,&
&  0.73980E+00_JPRB, 0.72865E+00_JPRB, 0.72144E+00_JPRB, 0.72231E+00_JPRB, 0.73292E+00_JPRB, 0.74387E+00_JPRB, 0.75125E+00_JPRB ,&
&  0.75471E+00_JPRB, 0.74937E+00_JPRB, 0.74733E+00_JPRB, 0.75634E+00_JPRB, 0.76338E+00_JPRB, 0.75518E+00_JPRB, 0.73417E+00_JPRB ,&
&  0.71627E+00_JPRB, 0.62233E+00_JPRB, 0.68947E+00_JPRB, 0.77424E+00_JPRB, 0.83790E+00_JPRB, 0.86876E+00_JPRB, 0.88554E+00_JPRB ,&
&  0.86885E+00_JPRB, 0.83332E+00_JPRB, 0.78892E+00_JPRB, 0.76059E+00_JPRB, 0.73835E+00_JPRB, 0.71919E+00_JPRB, 0.70634E+00_JPRB ,&
&  0.68656E+00_JPRB, 0.66862E+00_JPRB, 0.65239E+00_JPRB, 0.64385E+00_JPRB, 0.64132E+00_JPRB, 0.63929E+00_JPRB, 0.63763E+00_JPRB ,&
&  0.63610E+00_JPRB, 0.63363E+00_JPRB, 0.63578E+00_JPRB, 0.64795E+00_JPRB, 0.66765E+00_JPRB, 0.70147E+00_JPRB, 0.73662E+00_JPRB ,&
&  0.74819E+00_JPRB, 0.76138E+00_JPRB, 0.77973E+00_JPRB, 0.80434E+00_JPRB, 0.81156E+00_JPRB, 0.81014E+00_JPRB, 0.79666E+00_JPRB ,&
&  0.77772E+00_JPRB, 0.74825E+00_JPRB, 0.72626E+00_JPRB, 0.70916E+00_JPRB, 0.70093E+00_JPRB, 0.69381E+00_JPRB, 0.69958E+00_JPRB ,&
&  0.70598E+00_JPRB, 0.71506E+00_JPRB, 0.71499E+00_JPRB, 0.71344E+00_JPRB, 0.71657E+00_JPRB, 0.70789E+00_JPRB, 0.70736E+00_JPRB ,&
&  0.71167E+00_JPRB, 0.72774E+00_JPRB, 0.75367E+00_JPRB, 0.77685E+00_JPRB, 0.80463E+00_JPRB, 0.82217E+00_JPRB, 0.82721E+00_JPRB ,&
&  0.82884E+00_JPRB, 0.82780E+00_JPRB, 0.82670E+00_JPRB, 0.85814E+00_JPRB, 0.85074E+00_JPRB, 0.84786E+00_JPRB, 0.83986E+00_JPRB ,&
&  0.82257E+00_JPRB, 0.78897E+00_JPRB, 0.74853E+00_JPRB, 0.73268E+00_JPRB, 0.68750E+00_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.86499E+00_JPRB, 0.85845E+00_JPRB, 0.83956E+00_JPRB, 0.82594E+00_JPRB, 0.81061E+00_JPRB, 0.79908E+00_JPRB, 0.78942E+00_JPRB ,&
&  0.78513E+00_JPRB, 0.77197E+00_JPRB, 0.76599E+00_JPRB, 0.76004E+00_JPRB, 0.75892E+00_JPRB, 0.75369E+00_JPRB, 0.74464E+00_JPRB ,&
&  0.73429E+00_JPRB, 0.72895E+00_JPRB, 0.72840E+00_JPRB, 0.73402E+00_JPRB, 0.74457E+00_JPRB, 0.75096E+00_JPRB, 0.74569E+00_JPRB ,&
&  0.73742E+00_JPRB, 0.72454E+00_JPRB, 0.70852E+00_JPRB, 0.72123E+00_JPRB, 0.73613E+00_JPRB, 0.73992E+00_JPRB, 0.72316E+00_JPRB ,&
&  0.70430E+00_JPRB, 0.65349E+00_JPRB, 0.82661E+00_JPRB, 0.91063E+00_JPRB, 0.92426E+00_JPRB, 0.96729E+00_JPRB, 0.93357E+00_JPRB ,&
&  0.89725E+00_JPRB, 0.83123E+00_JPRB, 0.78422E+00_JPRB, 0.76550E+00_JPRB, 0.75429E+00_JPRB, 0.72811E+00_JPRB, 0.70608E+00_JPRB ,&
&  0.67292E+00_JPRB, 0.65902E+00_JPRB, 0.64989E+00_JPRB, 0.64678E+00_JPRB, 0.64552E+00_JPRB, 0.64752E+00_JPRB, 0.65350E+00_JPRB ,&
&  0.65974E+00_JPRB, 0.66094E+00_JPRB, 0.66311E+00_JPRB, 0.68167E+00_JPRB, 0.70372E+00_JPRB, 0.75639E+00_JPRB, 0.79752E+00_JPRB ,&
&  0.81541E+00_JPRB, 0.83124E+00_JPRB, 0.85661E+00_JPRB, 0.87485E+00_JPRB, 0.86605E+00_JPRB, 0.84306E+00_JPRB, 0.82350E+00_JPRB ,&
&  0.80315E+00_JPRB, 0.78155E+00_JPRB, 0.75723E+00_JPRB, 0.74297E+00_JPRB, 0.72912E+00_JPRB, 0.72384E+00_JPRB, 0.73454E+00_JPRB ,&
&  0.74051E+00_JPRB, 0.75227E+00_JPRB, 0.75210E+00_JPRB, 0.74696E+00_JPRB, 0.73036E+00_JPRB, 0.71712E+00_JPRB, 0.71895E+00_JPRB ,&
&  0.73254E+00_JPRB, 0.75937E+00_JPRB, 0.79342E+00_JPRB, 0.82740E+00_JPRB, 0.85307E+00_JPRB, 0.86387E+00_JPRB, 0.86484E+00_JPRB ,&
&  0.87157E+00_JPRB, 0.87290E+00_JPRB, 0.82135E+00_JPRB, 0.81936E+00_JPRB, 0.90441E+00_JPRB, 0.89664E+00_JPRB, 0.88334E+00_JPRB ,&
&  0.85197E+00_JPRB, 0.80099E+00_JPRB, 0.76494E+00_JPRB, 0.74951E+00_JPRB, 0.82262E+00_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.82848E+00_JPRB, 0.81949E+00_JPRB, 0.80579E+00_JPRB, 0.79958E+00_JPRB, 0.80387E+00_JPRB, 0.79743E+00_JPRB, 0.79341E+00_JPRB ,&
&  0.79282E+00_JPRB, 0.78502E+00_JPRB, 0.77806E+00_JPRB, 0.76568E+00_JPRB, 0.75100E+00_JPRB, 0.74067E+00_JPRB, 0.73988E+00_JPRB ,&
&  0.74345E+00_JPRB, 0.73643E+00_JPRB, 0.74213E+00_JPRB, 0.74508E+00_JPRB, 0.74247E+00_JPRB, 0.72947E+00_JPRB, 0.71656E+00_JPRB ,&
&  0.71050E+00_JPRB, 0.69879E+00_JPRB, 0.69606E+00_JPRB, 0.71543E+00_JPRB, 0.73343E+00_JPRB, 0.73436E+00_JPRB, 0.71197E+00_JPRB ,&
&  0.70245E+00_JPRB, 0.71504E+00_JPRB, 0.10161E+01_JPRB, 0.11268E+01_JPRB, 0.10669E+01_JPRB, 0.10168E+01_JPRB, 0.94453E+00_JPRB ,&
&  0.88169E+00_JPRB, 0.81729E+00_JPRB, 0.79024E+00_JPRB, 0.76717E+00_JPRB, 0.73966E+00_JPRB, 0.71651E+00_JPRB, 0.69411E+00_JPRB ,&
&  0.66579E+00_JPRB, 0.65695E+00_JPRB, 0.65406E+00_JPRB, 0.65676E+00_JPRB, 0.65961E+00_JPRB, 0.66044E+00_JPRB, 0.66251E+00_JPRB ,&
&  0.66364E+00_JPRB, 0.66388E+00_JPRB, 0.67211E+00_JPRB, 0.68351E+00_JPRB, 0.71423E+00_JPRB, 0.80101E+00_JPRB, 0.89026E+00_JPRB ,&
&  0.89248E+00_JPRB, 0.90845E+00_JPRB, 0.89533E+00_JPRB, 0.88961E+00_JPRB, 0.88189E+00_JPRB, 0.85936E+00_JPRB, 0.83777E+00_JPRB ,&
&  0.81256E+00_JPRB, 0.79503E+00_JPRB, 0.78186E+00_JPRB, 0.77627E+00_JPRB, 0.76346E+00_JPRB, 0.76949E+00_JPRB, 0.78383E+00_JPRB ,&
&  0.80002E+00_JPRB, 0.80938E+00_JPRB, 0.79482E+00_JPRB, 0.77989E+00_JPRB, 0.75825E+00_JPRB, 0.75602E+00_JPRB, 0.75915E+00_JPRB ,&
&  0.76463E+00_JPRB, 0.78519E+00_JPRB, 0.82177E+00_JPRB, 0.85812E+00_JPRB, 0.88123E+00_JPRB, 0.88911E+00_JPRB, 0.88546E+00_JPRB ,&
&  0.89234E+00_JPRB, 0.88112E+00_JPRB, 0.86008E+00_JPRB, 0.86085E+00_JPRB, 0.85344E+00_JPRB, 0.91199E+00_JPRB, 0.87709E+00_JPRB ,&
&  0.82369E+00_JPRB, 0.78415E+00_JPRB, 0.76478E+00_JPRB, 0.75812E+00_JPRB, 0.81383E+00_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.81759E+00_JPRB, 0.80142E+00_JPRB, 0.79291E+00_JPRB, 0.78560E+00_JPRB, 0.77573E+00_JPRB, 0.77261E+00_JPRB, 0.76789E+00_JPRB ,&
&  0.77401E+00_JPRB, 0.77808E+00_JPRB, 0.77087E+00_JPRB, 0.74747E+00_JPRB, 0.72938E+00_JPRB, 0.72845E+00_JPRB, 0.73633E+00_JPRB ,&
&  0.73254E+00_JPRB, 0.72285E+00_JPRB, 0.72702E+00_JPRB, 0.71097E+00_JPRB, 0.68939E+00_JPRB, 0.67629E+00_JPRB, 0.67097E+00_JPRB ,&
&  0.67554E+00_JPRB, 0.68622E+00_JPRB, 0.71235E+00_JPRB, 0.73693E+00_JPRB, 0.73971E+00_JPRB, 0.72925E+00_JPRB, 0.70025E+00_JPRB ,&
&  0.71954E+00_JPRB, 0.76337E+00_JPRB, 0.12840E+01_JPRB, 0.13091E+01_JPRB, 0.11800E+01_JPRB, 0.10348E+01_JPRB, 0.97367E+00_JPRB ,&
&  0.89197E+00_JPRB, 0.83522E+00_JPRB, 0.79631E+00_JPRB, 0.74675E+00_JPRB, 0.70778E+00_JPRB, 0.68679E+00_JPRB, 0.67760E+00_JPRB ,&
&  0.66182E+00_JPRB, 0.65256E+00_JPRB, 0.65833E+00_JPRB, 0.66879E+00_JPRB, 0.67602E+00_JPRB, 0.67794E+00_JPRB, 0.66502E+00_JPRB ,&
&  0.65920E+00_JPRB, 0.65498E+00_JPRB, 0.66130E+00_JPRB, 0.66320E+00_JPRB, 0.67422E+00_JPRB, 0.67139E+00_JPRB, 0.80352E+00_JPRB ,&
&  0.10342E+01_JPRB, 0.99677E+00_JPRB, 0.94182E+00_JPRB, 0.92347E+00_JPRB, 0.91259E+00_JPRB, 0.87874E+00_JPRB, 0.85543E+00_JPRB ,&
&  0.83216E+00_JPRB, 0.81833E+00_JPRB, 0.82980E+00_JPRB, 0.83385E+00_JPRB, 0.84296E+00_JPRB, 0.83093E+00_JPRB, 0.82563E+00_JPRB ,&
&  0.83497E+00_JPRB, 0.81524E+00_JPRB, 0.79251E+00_JPRB, 0.77121E+00_JPRB, 0.78007E+00_JPRB, 0.79814E+00_JPRB, 0.80584E+00_JPRB ,&
&  0.81188E+00_JPRB, 0.81663E+00_JPRB, 0.83437E+00_JPRB, 0.84632E+00_JPRB, 0.88699E+00_JPRB, 0.92138E+00_JPRB, 0.91647E+00_JPRB ,&
&  0.86932E+00_JPRB, 0.85528E+00_JPRB, 0.85956E+00_JPRB, 0.82999E+00_JPRB, 0.82233E+00_JPRB, 0.88999E+00_JPRB, 0.83607E+00_JPRB ,&
&  0.78680E+00_JPRB, 0.75791E+00_JPRB, 0.74412E+00_JPRB, 0.76095E+00_JPRB, 0.82069E+00_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.83585E+00_JPRB, 0.81997E+00_JPRB, 0.80876E+00_JPRB, 0.78658E+00_JPRB, 0.78272E+00_JPRB, 0.77485E+00_JPRB, 0.77238E+00_JPRB ,&
&  0.77337E+00_JPRB, 0.76283E+00_JPRB, 0.75020E+00_JPRB, 0.73201E+00_JPRB, 0.72518E+00_JPRB, 0.73579E+00_JPRB, 0.73040E+00_JPRB ,&
&  0.71292E+00_JPRB, 0.68962E+00_JPRB, 0.68285E+00_JPRB, 0.67446E+00_JPRB, 0.66622E+00_JPRB, 0.66865E+00_JPRB, 0.67829E+00_JPRB ,&
&  0.68182E+00_JPRB, 0.69879E+00_JPRB, 0.72694E+00_JPRB, 0.75201E+00_JPRB, 0.74706E+00_JPRB, 0.73195E+00_JPRB, 0.72735E+00_JPRB ,&
&  0.76745E+00_JPRB, 0.72618E+00_JPRB, 0.13323E+01_JPRB, 0.14952E+01_JPRB, 0.12820E+01_JPRB, 0.11084E+01_JPRB, 0.94404E+00_JPRB ,&
&  0.90211E+00_JPRB, 0.82433E+00_JPRB, 0.78368E+00_JPRB, 0.73598E+00_JPRB, 0.70494E+00_JPRB, 0.66879E+00_JPRB, 0.66055E+00_JPRB ,&
&  0.66745E+00_JPRB, 0.66174E+00_JPRB, 0.66360E+00_JPRB, 0.68146E+00_JPRB, 0.68980E+00_JPRB, 0.69808E+00_JPRB, 0.68062E+00_JPRB ,&
&  0.66541E+00_JPRB, 0.65218E+00_JPRB, 0.65898E+00_JPRB, 0.66527E+00_JPRB, 0.66322E+00_JPRB, 0.71878E+00_JPRB, 0.78690E+00_JPRB ,&
&  0.84970E+00_JPRB, 0.10373E+01_JPRB, 0.96865E+00_JPRB, 0.93698E+00_JPRB, 0.90738E+00_JPRB, 0.87368E+00_JPRB, 0.86720E+00_JPRB ,&
&  0.84257E+00_JPRB, 0.84211E+00_JPRB, 0.85051E+00_JPRB, 0.88932E+00_JPRB, 0.87848E+00_JPRB, 0.88992E+00_JPRB, 0.88055E+00_JPRB ,&
&  0.85357E+00_JPRB, 0.82154E+00_JPRB, 0.79348E+00_JPRB, 0.78346E+00_JPRB, 0.80599E+00_JPRB, 0.81988E+00_JPRB, 0.81631E+00_JPRB ,&
&  0.81523E+00_JPRB, 0.83751E+00_JPRB, 0.85537E+00_JPRB, 0.86098E+00_JPRB, 0.88573E+00_JPRB, 0.89246E+00_JPRB, 0.89328E+00_JPRB ,&
&  0.88054E+00_JPRB, 0.87858E+00_JPRB, 0.83498E+00_JPRB, 0.82180E+00_JPRB, 0.78582E+00_JPRB, 0.82341E+00_JPRB, 0.78254E+00_JPRB ,&
&  0.75376E+00_JPRB, 0.73349E+00_JPRB, 0.75989E+00_JPRB, 0.81569E+00_JPRB, 0.84455E+00_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.83203E+00_JPRB, 0.83557E+00_JPRB, 0.80241E+00_JPRB, 0.80001E+00_JPRB, 0.78922E+00_JPRB, 0.79448E+00_JPRB, 0.81075E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.78075E+00_JPRB, 0.74991E+00_JPRB, 0.73773E+00_JPRB, 0.74784E+00_JPRB, 0.75843E+00_JPRB, 0.72539E+00_JPRB ,&
&  0.68724E+00_JPRB, 0.67592E+00_JPRB, 0.67117E+00_JPRB, 0.67529E+00_JPRB, 0.68612E+00_JPRB, 0.70387E+00_JPRB, 0.71247E+00_JPRB ,&
&  0.70725E+00_JPRB, 0.74318E+00_JPRB, 0.78159E+00_JPRB, 0.79179E+00_JPRB, 0.77156E+00_JPRB, 0.75250E+00_JPRB, 0.74336E+00_JPRB ,&
&  0.77731E+00_JPRB, 0.14532E+01_JPRB, 0.11039E+01_JPRB, 0.15936E+01_JPRB, 0.12885E+01_JPRB, 0.10781E+01_JPRB, 0.86817E+00_JPRB ,&
&  0.81140E+00_JPRB, 0.82216E+00_JPRB, 0.79831E+00_JPRB, 0.74042E+00_JPRB, 0.69608E+00_JPRB, 0.66372E+00_JPRB, 0.64037E+00_JPRB ,&
&  0.64044E+00_JPRB, 0.65566E+00_JPRB, 0.66174E+00_JPRB, 0.69404E+00_JPRB, 0.72107E+00_JPRB, 0.72325E+00_JPRB, 0.69926E+00_JPRB ,&
&  0.67561E+00_JPRB, 0.65840E+00_JPRB, 0.66382E+00_JPRB, 0.71352E+00_JPRB, 0.70841E+00_JPRB, 0.76442E+00_JPRB, 0.85884E+00_JPRB ,&
&  0.84106E+00_JPRB, 0.10536E+01_JPRB, 0.96790E+00_JPRB, 0.92544E+00_JPRB, 0.89520E+00_JPRB, 0.86312E+00_JPRB, 0.86871E+00_JPRB ,&
&  0.85261E+00_JPRB, 0.85474E+00_JPRB, 0.86345E+00_JPRB, 0.86746E+00_JPRB, 0.89195E+00_JPRB, 0.91617E+00_JPRB, 0.89071E+00_JPRB ,&
&  0.85135E+00_JPRB, 0.81781E+00_JPRB, 0.81869E+00_JPRB, 0.83068E+00_JPRB, 0.84719E+00_JPRB, 0.83477E+00_JPRB, 0.82520E+00_JPRB ,&
&  0.83405E+00_JPRB, 0.87371E+00_JPRB, 0.87820E+00_JPRB, 0.88425E+00_JPRB, 0.87814E+00_JPRB, 0.85916E+00_JPRB, 0.84147E+00_JPRB ,&
&  0.87253E+00_JPRB, 0.86187E+00_JPRB, 0.80586E+00_JPRB, 0.78777E+00_JPRB, 0.76642E+00_JPRB, 0.78368E+00_JPRB, 0.74602E+00_JPRB ,&
&  0.73701E+00_JPRB, 0.74890E+00_JPRB, 0.82460E+00_JPRB, 0.83184E+00_JPRB, 0.84047E+00_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.83177E+00_JPRB, 0.82401E+00_JPRB, 0.79666E+00_JPRB, 0.79996E+00_JPRB, 0.78427E+00_JPRB, 0.81541E+00_JPRB, 0.82082E+00_JPRB ,&
&  0.78660E+00_JPRB, 0.75496E+00_JPRB, 0.73837E+00_JPRB, 0.75772E+00_JPRB, 0.77721E+00_JPRB, 0.76210E+00_JPRB, 0.72092E+00_JPRB ,&
&  0.70799E+00_JPRB, 0.70704E+00_JPRB, 0.70841E+00_JPRB, 0.71219E+00_JPRB, 0.72763E+00_JPRB, 0.74951E+00_JPRB, 0.74618E+00_JPRB ,&
&  0.74262E+00_JPRB, 0.77137E+00_JPRB, 0.79539E+00_JPRB, 0.79268E+00_JPRB, 0.77843E+00_JPRB, 0.75811E+00_JPRB, 0.76310E+00_JPRB ,&
&  0.76744E+00_JPRB, 0.13406E+01_JPRB, 0.10696E+01_JPRB, 0.15873E+01_JPRB, 0.12665E+01_JPRB, 0.10675E+01_JPRB, 0.83632E+00_JPRB ,&
&  0.74518E+00_JPRB, 0.80943E+00_JPRB, 0.77758E+00_JPRB, 0.70590E+00_JPRB, 0.65367E+00_JPRB, 0.63378E+00_JPRB, 0.62109E+00_JPRB ,&
&  0.62444E+00_JPRB, 0.65697E+00_JPRB, 0.68082E+00_JPRB, 0.71632E+00_JPRB, 0.74002E+00_JPRB, 0.73175E+00_JPRB, 0.71581E+00_JPRB ,&
&  0.69855E+00_JPRB, 0.68583E+00_JPRB, 0.71404E+00_JPRB, 0.73977E+00_JPRB, 0.78141E+00_JPRB, 0.84731E+00_JPRB, 0.89667E+00_JPRB ,&
&  0.90042E+00_JPRB, 0.10257E+01_JPRB, 0.96938E+00_JPRB, 0.93503E+00_JPRB, 0.89659E+00_JPRB, 0.88021E+00_JPRB, 0.87724E+00_JPRB ,&
&  0.85017E+00_JPRB, 0.81676E+00_JPRB, 0.81807E+00_JPRB, 0.83861E+00_JPRB, 0.90063E+00_JPRB, 0.91528E+00_JPRB, 0.87533E+00_JPRB ,&
&  0.83436E+00_JPRB, 0.81731E+00_JPRB, 0.81683E+00_JPRB, 0.83173E+00_JPRB, 0.84551E+00_JPRB, 0.83986E+00_JPRB, 0.83653E+00_JPRB ,&
&  0.86403E+00_JPRB, 0.91028E+00_JPRB, 0.89904E+00_JPRB, 0.89283E+00_JPRB, 0.86140E+00_JPRB, 0.83415E+00_JPRB, 0.81398E+00_JPRB ,&
&  0.83262E+00_JPRB, 0.81889E+00_JPRB, 0.78831E+00_JPRB, 0.75436E+00_JPRB, 0.74479E+00_JPRB, 0.75308E+00_JPRB, 0.74284E+00_JPRB ,&
&  0.75749E+00_JPRB, 0.81334E+00_JPRB, 0.87070E+00_JPRB, 0.83401E+00_JPRB, 0.82804E+00_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.80544E+00_JPRB, 0.77808E+00_JPRB, 0.76851E+00_JPRB, 0.75795E+00_JPRB, 0.74669E+00_JPRB, 0.77811E+00_JPRB, 0.76822E+00_JPRB ,&
&  0.74425E+00_JPRB, 0.72112E+00_JPRB, 0.74084E+00_JPRB, 0.78379E+00_JPRB, 0.78327E+00_JPRB, 0.76075E+00_JPRB, 0.74503E+00_JPRB ,&
&  0.74337E+00_JPRB, 0.76073E+00_JPRB, 0.76452E+00_JPRB, 0.77514E+00_JPRB, 0.77445E+00_JPRB, 0.76359E+00_JPRB, 0.75600E+00_JPRB ,&
&  0.77175E+00_JPRB, 0.77954E+00_JPRB, 0.78456E+00_JPRB, 0.78319E+00_JPRB, 0.78874E+00_JPRB, 0.77164E+00_JPRB, 0.79893E+00_JPRB ,&
&  0.85499E+00_JPRB, 0.12505E+01_JPRB, 0.10495E+01_JPRB, 0.12465E+01_JPRB, 0.11444E+01_JPRB, 0.98245E+00_JPRB, 0.75697E+00_JPRB ,&
&  0.64568E+00_JPRB, 0.59997E+00_JPRB, 0.72358E+00_JPRB, 0.68381E+00_JPRB, 0.66431E+00_JPRB, 0.67786E+00_JPRB, 0.68505E+00_JPRB ,&
&  0.70818E+00_JPRB, 0.71587E+00_JPRB, 0.76249E+00_JPRB, 0.77562E+00_JPRB, 0.77528E+00_JPRB, 0.76453E+00_JPRB, 0.74752E+00_JPRB ,&
&  0.75313E+00_JPRB, 0.76109E+00_JPRB, 0.82099E+00_JPRB, 0.84564E+00_JPRB, 0.83148E+00_JPRB, 0.88239E+00_JPRB, 0.90213E+00_JPRB ,&
&  0.93385E+00_JPRB, 0.97587E+00_JPRB, 0.98087E+00_JPRB, 0.96544E+00_JPRB, 0.88491E+00_JPRB, 0.91806E+00_JPRB, 0.86908E+00_JPRB ,&
&  0.81417E+00_JPRB, 0.78190E+00_JPRB, 0.78909E+00_JPRB, 0.82328E+00_JPRB, 0.86725E+00_JPRB, 0.86703E+00_JPRB, 0.84366E+00_JPRB ,&
&  0.82939E+00_JPRB, 0.83405E+00_JPRB, 0.81926E+00_JPRB, 0.83021E+00_JPRB, 0.83833E+00_JPRB, 0.86329E+00_JPRB, 0.88688E+00_JPRB ,&
&  0.92251E+00_JPRB, 0.95018E+00_JPRB, 0.91744E+00_JPRB, 0.88322E+00_JPRB, 0.83739E+00_JPRB, 0.80603E+00_JPRB, 0.77376E+00_JPRB ,&
&  0.77927E+00_JPRB, 0.79965E+00_JPRB, 0.79875E+00_JPRB, 0.76971E+00_JPRB, 0.75315E+00_JPRB, 0.75361E+00_JPRB, 0.78279E+00_JPRB ,&
&  0.81797E+00_JPRB, 0.87957E+00_JPRB, 0.88915E+00_JPRB, 0.83701E+00_JPRB, 0.82567E+00_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.74607E+00_JPRB, 0.73987E+00_JPRB, 0.71884E+00_JPRB, 0.70510E+00_JPRB, 0.72759E+00_JPRB, 0.76088E+00_JPRB, 0.76030E+00_JPRB ,&
&  0.73834E+00_JPRB, 0.72734E+00_JPRB, 0.76236E+00_JPRB, 0.79567E+00_JPRB, 0.79170E+00_JPRB, 0.80558E+00_JPRB, 0.81268E+00_JPRB ,&
&  0.84139E+00_JPRB, 0.83284E+00_JPRB, 0.81223E+00_JPRB, 0.78381E+00_JPRB, 0.78066E+00_JPRB, 0.76711E+00_JPRB, 0.76529E+00_JPRB ,&
&  0.77092E+00_JPRB, 0.77394E+00_JPRB, 0.78000E+00_JPRB, 0.79191E+00_JPRB, 0.80214E+00_JPRB, 0.84120E+00_JPRB, 0.92889E+00_JPRB ,&
&  0.10821E+01_JPRB, 0.14009E+01_JPRB, 0.57788E+00_JPRB, 0.72475E+00_JPRB, 0.83846E+00_JPRB, 0.75559E+00_JPRB, 0.64552E+00_JPRB ,&
&  0.55537E+00_JPRB, 0.49536E+00_JPRB, 0.58670E+00_JPRB, 0.71537E+00_JPRB, 0.73305E+00_JPRB, 0.75322E+00_JPRB, 0.76757E+00_JPRB ,&
&  0.78484E+00_JPRB, 0.79659E+00_JPRB, 0.82664E+00_JPRB, 0.82942E+00_JPRB, 0.82887E+00_JPRB, 0.81854E+00_JPRB, 0.80297E+00_JPRB ,&
&  0.83197E+00_JPRB, 0.87402E+00_JPRB, 0.95487E+00_JPRB, 0.87469E+00_JPRB, 0.87719E+00_JPRB, 0.86282E+00_JPRB, 0.84213E+00_JPRB ,&
&  0.82972E+00_JPRB, 0.93194E+00_JPRB, 0.10048E+01_JPRB, 0.98024E+00_JPRB, 0.89048E+00_JPRB, 0.83879E+00_JPRB, 0.90913E+00_JPRB ,&
&  0.83512E+00_JPRB, 0.78863E+00_JPRB, 0.77062E+00_JPRB, 0.78050E+00_JPRB, 0.83560E+00_JPRB, 0.83117E+00_JPRB, 0.81299E+00_JPRB ,&
&  0.81929E+00_JPRB, 0.81032E+00_JPRB, 0.82258E+00_JPRB, 0.84747E+00_JPRB, 0.87394E+00_JPRB, 0.92404E+00_JPRB, 0.94733E+00_JPRB ,&
&  0.94457E+00_JPRB, 0.95387E+00_JPRB, 0.94760E+00_JPRB, 0.90535E+00_JPRB, 0.84342E+00_JPRB, 0.80234E+00_JPRB, 0.79033E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80944E+00_JPRB, 0.78012E+00_JPRB, 0.74393E+00_JPRB, 0.77844E+00_JPRB, 0.80560E+00_JPRB, 0.91675E+00_JPRB ,&
&  0.10309E+01_JPRB, 0.11102E+01_JPRB, 0.10735E+01_JPRB, 0.80515E+00_JPRB, 0.79597E+00_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.70718E+00_JPRB, 0.66671E+00_JPRB, 0.63587E+00_JPRB, 0.64741E+00_JPRB, 0.69759E+00_JPRB, 0.74894E+00_JPRB, 0.74833E+00_JPRB ,&
&  0.73411E+00_JPRB, 0.75243E+00_JPRB, 0.82445E+00_JPRB, 0.87835E+00_JPRB, 0.88490E+00_JPRB, 0.89427E+00_JPRB, 0.90823E+00_JPRB ,&
&  0.85115E+00_JPRB, 0.81584E+00_JPRB, 0.79705E+00_JPRB, 0.77704E+00_JPRB, 0.77508E+00_JPRB, 0.76716E+00_JPRB, 0.76356E+00_JPRB ,&
&  0.76622E+00_JPRB, 0.76089E+00_JPRB, 0.77421E+00_JPRB, 0.79572E+00_JPRB, 0.84670E+00_JPRB, 0.97790E+00_JPRB, 0.11757E+01_JPRB ,&
&  0.15469E+01_JPRB, 0.10198E+01_JPRB, 0.45881E+00_JPRB, 0.61957E+00_JPRB, 0.66777E+00_JPRB, 0.64790E+00_JPRB, 0.57779E+00_JPRB ,&
&  0.54186E+00_JPRB, 0.55846E+00_JPRB, 0.66168E+00_JPRB, 0.77895E+00_JPRB, 0.80547E+00_JPRB, 0.82939E+00_JPRB, 0.83531E+00_JPRB ,&
&  0.83718E+00_JPRB, 0.85433E+00_JPRB, 0.86503E+00_JPRB, 0.86818E+00_JPRB, 0.87268E+00_JPRB, 0.87480E+00_JPRB, 0.89152E+00_JPRB ,&
&  0.93677E+00_JPRB, 0.98294E+00_JPRB, 0.10418E+01_JPRB, 0.89632E+00_JPRB, 0.89476E+00_JPRB, 0.82613E+00_JPRB, 0.81621E+00_JPRB ,&
&  0.81918E+00_JPRB, 0.90872E+00_JPRB, 0.99053E+00_JPRB, 0.94660E+00_JPRB, 0.85697E+00_JPRB, 0.79331E+00_JPRB, 0.91183E+00_JPRB ,&
&  0.82600E+00_JPRB, 0.77371E+00_JPRB, 0.73552E+00_JPRB, 0.74421E+00_JPRB, 0.77790E+00_JPRB, 0.78664E+00_JPRB, 0.78418E+00_JPRB ,&
&  0.79098E+00_JPRB, 0.79128E+00_JPRB, 0.82509E+00_JPRB, 0.84556E+00_JPRB, 0.86252E+00_JPRB, 0.89900E+00_JPRB, 0.90044E+00_JPRB ,&
&  0.89209E+00_JPRB, 0.88564E+00_JPRB, 0.89319E+00_JPRB, 0.91062E+00_JPRB, 0.86366E+00_JPRB, 0.78757E+00_JPRB, 0.77589E+00_JPRB ,&
&  0.77191E+00_JPRB, 0.77125E+00_JPRB, 0.74753E+00_JPRB, 0.76418E+00_JPRB, 0.78071E+00_JPRB, 0.85654E+00_JPRB, 0.93829E+00_JPRB ,&
&  0.97455E+00_JPRB, 0.95151E+00_JPRB, 0.88839E+00_JPRB, 0.76578E+00_JPRB, 0.75234E+00_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.65867E+00_JPRB, 0.63231E+00_JPRB, 0.59272E+00_JPRB, 0.59215E+00_JPRB, 0.65200E+00_JPRB, 0.69897E+00_JPRB, 0.70072E+00_JPRB ,&
&  0.70643E+00_JPRB, 0.76880E+00_JPRB, 0.87947E+00_JPRB, 0.96628E+00_JPRB, 0.99159E+00_JPRB, 0.98029E+00_JPRB, 0.87858E+00_JPRB ,&
&  0.83644E+00_JPRB, 0.80876E+00_JPRB, 0.77916E+00_JPRB, 0.76271E+00_JPRB, 0.75159E+00_JPRB, 0.74593E+00_JPRB, 0.74773E+00_JPRB ,&
&  0.75164E+00_JPRB, 0.76675E+00_JPRB, 0.80758E+00_JPRB, 0.86016E+00_JPRB, 0.94545E+00_JPRB, 0.10366E+01_JPRB, 0.11094E+01_JPRB ,&
&  0.80351E+00_JPRB, 0.38371E+00_JPRB, 0.43407E+00_JPRB, 0.64183E+00_JPRB, 0.60185E+00_JPRB, 0.59016E+00_JPRB, 0.56147E+00_JPRB ,&
&  0.57318E+00_JPRB, 0.63959E+00_JPRB, 0.70321E+00_JPRB, 0.81252E+00_JPRB, 0.85033E+00_JPRB, 0.88412E+00_JPRB, 0.88129E+00_JPRB ,&
&  0.88807E+00_JPRB, 0.90156E+00_JPRB, 0.92043E+00_JPRB, 0.93454E+00_JPRB, 0.95094E+00_JPRB, 0.96946E+00_JPRB, 0.10092E+01_JPRB ,&
&  0.10494E+01_JPRB, 0.10779E+01_JPRB, 0.11000E+01_JPRB, 0.91272E+00_JPRB, 0.90400E+00_JPRB, 0.82379E+00_JPRB, 0.78122E+00_JPRB ,&
&  0.79962E+00_JPRB, 0.81164E+00_JPRB, 0.82441E+00_JPRB, 0.93225E+00_JPRB, 0.88176E+00_JPRB, 0.75427E+00_JPRB, 0.82089E+00_JPRB ,&
&  0.77240E+00_JPRB, 0.75325E+00_JPRB, 0.73466E+00_JPRB, 0.67607E+00_JPRB, 0.73400E+00_JPRB, 0.75767E+00_JPRB, 0.77300E+00_JPRB ,&
&  0.75598E+00_JPRB, 0.77161E+00_JPRB, 0.79226E+00_JPRB, 0.82734E+00_JPRB, 0.81625E+00_JPRB, 0.81326E+00_JPRB, 0.79742E+00_JPRB ,&
&  0.79709E+00_JPRB, 0.80128E+00_JPRB, 0.81501E+00_JPRB, 0.83901E+00_JPRB, 0.84716E+00_JPRB, 0.78000E+00_JPRB, 0.73066E+00_JPRB ,&
&  0.71011E+00_JPRB, 0.70514E+00_JPRB, 0.68673E+00_JPRB, 0.73753E+00_JPRB, 0.76327E+00_JPRB, 0.78924E+00_JPRB, 0.82287E+00_JPRB ,&
&  0.83859E+00_JPRB, 0.83975E+00_JPRB, 0.79287E+00_JPRB, 0.73858E+00_JPRB, 0.70209E+00_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.55595E+00_JPRB, 0.54328E+00_JPRB, 0.52843E+00_JPRB, 0.54632E+00_JPRB, 0.58027E+00_JPRB, 0.62111E+00_JPRB, 0.63820E+00_JPRB ,&
&  0.65979E+00_JPRB, 0.75561E+00_JPRB, 0.88116E+00_JPRB, 0.99038E+00_JPRB, 0.10035E+01_JPRB, 0.96584E+00_JPRB, 0.85833E+00_JPRB ,&
&  0.82227E+00_JPRB, 0.78811E+00_JPRB, 0.76420E+00_JPRB, 0.73597E+00_JPRB, 0.72349E+00_JPRB, 0.72957E+00_JPRB, 0.74818E+00_JPRB ,&
&  0.77382E+00_JPRB, 0.79960E+00_JPRB, 0.83074E+00_JPRB, 0.86058E+00_JPRB, 0.87501E+00_JPRB, 0.84395E+00_JPRB, 0.72857E+00_JPRB ,&
&  0.32069E+00_JPRB, 0.41062E+00_JPRB, 0.62253E+00_JPRB, 0.60651E+00_JPRB, 0.56811E+00_JPRB, 0.54675E+00_JPRB, 0.56915E+00_JPRB ,&
&  0.59658E+00_JPRB, 0.60619E+00_JPRB, 0.66417E+00_JPRB, 0.79807E+00_JPRB, 0.85502E+00_JPRB, 0.91555E+00_JPRB, 0.93074E+00_JPRB ,&
&  0.93845E+00_JPRB, 0.95225E+00_JPRB, 0.98334E+00_JPRB, 0.10061E+01_JPRB, 0.10190E+01_JPRB, 0.10273E+01_JPRB, 0.10473E+01_JPRB ,&
&  0.10936E+01_JPRB, 0.11263E+01_JPRB, 0.11352E+01_JPRB, 0.94911E+00_JPRB, 0.90462E+00_JPRB, 0.81645E+00_JPRB, 0.76105E+00_JPRB ,&
&  0.73668E+00_JPRB, 0.75564E+00_JPRB, 0.80797E+00_JPRB, 0.92258E+00_JPRB, 0.88447E+00_JPRB, 0.85028E+00_JPRB, 0.82004E+00_JPRB ,&
&  0.77090E+00_JPRB, 0.70127E+00_JPRB, 0.70396E+00_JPRB, 0.70711E+00_JPRB, 0.70615E+00_JPRB, 0.74578E+00_JPRB, 0.72093E+00_JPRB ,&
&  0.73826E+00_JPRB, 0.74901E+00_JPRB, 0.74291E+00_JPRB, 0.74716E+00_JPRB, 0.74290E+00_JPRB, 0.73944E+00_JPRB, 0.72139E+00_JPRB ,&
&  0.71633E+00_JPRB, 0.71072E+00_JPRB, 0.72060E+00_JPRB, 0.73964E+00_JPRB, 0.74685E+00_JPRB, 0.74182E+00_JPRB, 0.70346E+00_JPRB ,&
&  0.66804E+00_JPRB, 0.68421E+00_JPRB, 0.79218E+00_JPRB, 0.93304E+00_JPRB, 0.91867E+00_JPRB, 0.79237E+00_JPRB, 0.62232E+00_JPRB ,&
&  0.60741E+00_JPRB, 0.62840E+00_JPRB, 0.63119E+00_JPRB, 0.61104E+00_JPRB, 0.58508E+00_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.45299E+00_JPRB, 0.44173E+00_JPRB, 0.44127E+00_JPRB, 0.45186E+00_JPRB, 0.45846E+00_JPRB, 0.48209E+00_JPRB, 0.51193E+00_JPRB ,&
&  0.52617E+00_JPRB, 0.59275E+00_JPRB, 0.74057E+00_JPRB, 0.86506E+00_JPRB, 0.91085E+00_JPRB, 0.94305E+00_JPRB, 0.88345E+00_JPRB ,&
&  0.85764E+00_JPRB, 0.81334E+00_JPRB, 0.75046E+00_JPRB, 0.72717E+00_JPRB, 0.72574E+00_JPRB, 0.73559E+00_JPRB, 0.75091E+00_JPRB ,&
&  0.76958E+00_JPRB, 0.79025E+00_JPRB, 0.80432E+00_JPRB, 0.81680E+00_JPRB, 0.83280E+00_JPRB, 0.79121E+00_JPRB, 0.70885E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.67547E+00_JPRB, 0.65694E+00_JPRB, 0.60177E+00_JPRB, 0.54653E+00_JPRB, 0.51784E+00_JPRB, 0.55218E+00_JPRB ,&
&  0.58742E+00_JPRB, 0.57773E+00_JPRB, 0.61738E+00_JPRB, 0.71334E+00_JPRB, 0.84996E+00_JPRB, 0.91666E+00_JPRB, 0.96623E+00_JPRB ,&
&  0.10037E+01_JPRB, 0.10185E+01_JPRB, 0.10359E+01_JPRB, 0.10508E+01_JPRB, 0.10458E+01_JPRB, 0.10350E+01_JPRB, 0.10433E+01_JPRB ,&
&  0.10613E+01_JPRB, 0.11025E+01_JPRB, 0.11081E+01_JPRB, 0.97793E+00_JPRB, 0.82329E+00_JPRB, 0.78064E+00_JPRB, 0.72826E+00_JPRB ,&
&  0.70024E+00_JPRB, 0.69685E+00_JPRB, 0.80396E+00_JPRB, 0.88493E+00_JPRB, 0.88223E+00_JPRB, 0.87606E+00_JPRB, 0.84647E+00_JPRB ,&
&  0.79422E+00_JPRB, 0.72089E+00_JPRB, 0.63852E+00_JPRB, 0.64265E+00_JPRB, 0.67101E+00_JPRB, 0.68762E+00_JPRB, 0.68611E+00_JPRB ,&
&  0.69537E+00_JPRB, 0.68272E+00_JPRB, 0.66663E+00_JPRB, 0.64198E+00_JPRB, 0.63303E+00_JPRB, 0.64975E+00_JPRB, 0.65495E+00_JPRB ,&
&  0.67168E+00_JPRB, 0.62953E+00_JPRB, 0.60050E+00_JPRB, 0.61125E+00_JPRB, 0.62499E+00_JPRB, 0.64463E+00_JPRB, 0.64647E+00_JPRB ,&
&  0.69952E+00_JPRB, 0.85342E+00_JPRB, 0.12526E+01_JPRB, 0.16627E+01_JPRB, 0.13681E+01_JPRB, 0.10752E+01_JPRB, 0.60093E+00_JPRB ,&
&  0.47338E+00_JPRB, 0.45669E+00_JPRB, 0.47438E+00_JPRB, 0.47730E+00_JPRB, 0.47502E+00_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.40259E+00_JPRB, 0.39354E+00_JPRB, 0.40018E+00_JPRB, 0.39859E+00_JPRB, 0.40214E+00_JPRB, 0.40884E+00_JPRB, 0.41311E+00_JPRB ,&
&  0.41627E+00_JPRB, 0.43235E+00_JPRB, 0.51757E+00_JPRB, 0.63346E+00_JPRB, 0.74088E+00_JPRB, 0.82003E+00_JPRB, 0.84887E+00_JPRB ,&
&  0.87101E+00_JPRB, 0.86493E+00_JPRB, 0.82970E+00_JPRB, 0.79071E+00_JPRB, 0.77285E+00_JPRB, 0.79242E+00_JPRB, 0.80475E+00_JPRB ,&
&  0.81381E+00_JPRB, 0.81917E+00_JPRB, 0.82032E+00_JPRB, 0.82739E+00_JPRB, 0.83037E+00_JPRB, 0.74852E+00_JPRB, 0.46152E+00_JPRB ,&
&  0.69018E+00_JPRB, 0.73343E+00_JPRB, 0.71506E+00_JPRB, 0.64727E+00_JPRB, 0.59095E+00_JPRB, 0.54343E+00_JPRB, 0.54207E+00_JPRB ,&
&  0.55020E+00_JPRB, 0.54922E+00_JPRB, 0.56195E+00_JPRB, 0.60915E+00_JPRB, 0.81074E+00_JPRB, 0.88381E+00_JPRB, 0.92106E+00_JPRB ,&
&  0.95808E+00_JPRB, 0.99247E+00_JPRB, 0.10085E+01_JPRB, 0.10360E+01_JPRB, 0.10333E+01_JPRB, 0.10063E+01_JPRB, 0.98632E+00_JPRB ,&
&  0.96610E+00_JPRB, 0.97591E+00_JPRB, 0.95042E+00_JPRB, 0.86607E+00_JPRB, 0.74667E+00_JPRB, 0.75181E+00_JPRB, 0.72254E+00_JPRB ,&
&  0.65224E+00_JPRB, 0.65296E+00_JPRB, 0.75025E+00_JPRB, 0.84605E+00_JPRB, 0.85364E+00_JPRB, 0.82938E+00_JPRB, 0.77222E+00_JPRB ,&
&  0.71342E+00_JPRB, 0.66242E+00_JPRB, 0.62395E+00_JPRB, 0.60079E+00_JPRB, 0.60724E+00_JPRB, 0.62109E+00_JPRB, 0.61976E+00_JPRB ,&
&  0.61813E+00_JPRB, 0.59299E+00_JPRB, 0.56058E+00_JPRB, 0.55222E+00_JPRB, 0.55447E+00_JPRB, 0.59966E+00_JPRB, 0.72545E+00_JPRB ,&
&  0.81054E+00_JPRB, 0.50861E+00_JPRB, 0.49530E+00_JPRB, 0.48152E+00_JPRB, 0.49673E+00_JPRB, 0.53916E+00_JPRB, 0.62928E+00_JPRB ,&
&  0.84910E+00_JPRB, 0.15603E+01_JPRB, 0.24688E+01_JPRB, 0.18917E+01_JPRB, 0.21715E+01_JPRB, 0.14830E+01_JPRB, 0.58334E+00_JPRB ,&
&  0.44564E+00_JPRB, 0.43655E+00_JPRB, 0.44156E+00_JPRB, 0.43064E+00_JPRB, 0.41745E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.41043E+00_JPRB, 0.40055E+00_JPRB, 0.40691E+00_JPRB, 0.41594E+00_JPRB, 0.41166E+00_JPRB, 0.41074E+00_JPRB, 0.40568E+00_JPRB ,&
&  0.39615E+00_JPRB, 0.39008E+00_JPRB, 0.41068E+00_JPRB, 0.45000E+00_JPRB, 0.50357E+00_JPRB, 0.57240E+00_JPRB, 0.63745E+00_JPRB ,&
&  0.67369E+00_JPRB, 0.71242E+00_JPRB, 0.74193E+00_JPRB, 0.76276E+00_JPRB, 0.75172E+00_JPRB, 0.73887E+00_JPRB, 0.71338E+00_JPRB ,&
&  0.69187E+00_JPRB, 0.68678E+00_JPRB, 0.70230E+00_JPRB, 0.70813E+00_JPRB, 0.73143E+00_JPRB, 0.75182E+00_JPRB, 0.49740E+00_JPRB ,&
&  0.81850E+00_JPRB, 0.83110E+00_JPRB, 0.76963E+00_JPRB, 0.69262E+00_JPRB, 0.65186E+00_JPRB, 0.59303E+00_JPRB, 0.53223E+00_JPRB ,&
&  0.50483E+00_JPRB, 0.49448E+00_JPRB, 0.48889E+00_JPRB, 0.55809E+00_JPRB, 0.66184E+00_JPRB, 0.73697E+00_JPRB, 0.79905E+00_JPRB ,&
&  0.82326E+00_JPRB, 0.85528E+00_JPRB, 0.87245E+00_JPRB, 0.89462E+00_JPRB, 0.91004E+00_JPRB, 0.89183E+00_JPRB, 0.84439E+00_JPRB ,&
&  0.78377E+00_JPRB, 0.74415E+00_JPRB, 0.72075E+00_JPRB, 0.73791E+00_JPRB, 0.73403E+00_JPRB, 0.72253E+00_JPRB, 0.69558E+00_JPRB ,&
&  0.56801E+00_JPRB, 0.55904E+00_JPRB, 0.62810E+00_JPRB, 0.82560E+00_JPRB, 0.84402E+00_JPRB, 0.80933E+00_JPRB, 0.76048E+00_JPRB ,&
&  0.71350E+00_JPRB, 0.67079E+00_JPRB, 0.63561E+00_JPRB, 0.60514E+00_JPRB, 0.58748E+00_JPRB, 0.57804E+00_JPRB, 0.56763E+00_JPRB ,&
&  0.55203E+00_JPRB, 0.52224E+00_JPRB, 0.49055E+00_JPRB, 0.47597E+00_JPRB, 0.46204E+00_JPRB, 0.45556E+00_JPRB, 0.45394E+00_JPRB ,&
&  0.54816E+00_JPRB, 0.44822E+00_JPRB, 0.42195E+00_JPRB, 0.42707E+00_JPRB, 0.41002E+00_JPRB, 0.46345E+00_JPRB, 0.64199E+00_JPRB ,&
&  0.10633E+01_JPRB, 0.16066E+01_JPRB, 0.27525E+01_JPRB, 0.49822E+01_JPRB, 0.27442E+01_JPRB, 0.21078E+01_JPRB, 0.48240E+00_JPRB ,&
&  0.45947E+00_JPRB, 0.46117E+00_JPRB, 0.45733E+00_JPRB, 0.43938E+00_JPRB, 0.43421E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.44932E+00_JPRB, 0.44679E+00_JPRB, 0.45295E+00_JPRB, 0.45981E+00_JPRB, 0.46077E+00_JPRB, 0.46092E+00_JPRB, 0.45997E+00_JPRB ,&
&  0.46066E+00_JPRB, 0.45895E+00_JPRB, 0.45958E+00_JPRB, 0.46559E+00_JPRB, 0.47904E+00_JPRB, 0.50898E+00_JPRB, 0.54302E+00_JPRB ,&
&  0.57173E+00_JPRB, 0.59679E+00_JPRB, 0.61879E+00_JPRB, 0.63627E+00_JPRB, 0.65302E+00_JPRB, 0.66017E+00_JPRB, 0.69563E+00_JPRB ,&
&  0.74080E+00_JPRB, 0.79670E+00_JPRB, 0.85068E+00_JPRB, 0.88271E+00_JPRB, 0.89592E+00_JPRB, 0.81726E+00_JPRB, 0.60597E+00_JPRB ,&
&  0.98706E+00_JPRB, 0.98697E+00_JPRB, 0.87473E+00_JPRB, 0.77543E+00_JPRB, 0.71040E+00_JPRB, 0.65724E+00_JPRB, 0.60182E+00_JPRB ,&
&  0.54816E+00_JPRB, 0.54317E+00_JPRB, 0.50300E+00_JPRB, 0.49762E+00_JPRB, 0.51072E+00_JPRB, 0.54891E+00_JPRB, 0.63843E+00_JPRB ,&
&  0.69107E+00_JPRB, 0.72361E+00_JPRB, 0.76159E+00_JPRB, 0.81796E+00_JPRB, 0.84325E+00_JPRB, 0.82667E+00_JPRB, 0.74522E+00_JPRB ,&
&  0.69612E+00_JPRB, 0.67338E+00_JPRB, 0.63304E+00_JPRB, 0.66101E+00_JPRB, 0.68571E+00_JPRB, 0.68121E+00_JPRB, 0.66413E+00_JPRB ,&
&  0.55098E+00_JPRB, 0.56880E+00_JPRB, 0.65700E+00_JPRB, 0.87333E+00_JPRB, 0.82370E+00_JPRB, 0.76874E+00_JPRB, 0.75884E+00_JPRB ,&
&  0.71631E+00_JPRB, 0.67956E+00_JPRB, 0.65217E+00_JPRB, 0.61993E+00_JPRB, 0.59303E+00_JPRB, 0.57761E+00_JPRB, 0.55486E+00_JPRB ,&
&  0.52523E+00_JPRB, 0.48815E+00_JPRB, 0.45484E+00_JPRB, 0.42874E+00_JPRB, 0.40779E+00_JPRB, 0.34784E+00_JPRB, 0.41939E+00_JPRB ,&
&  0.44005E+00_JPRB, 0.39843E+00_JPRB, 0.42347E+00_JPRB, 0.33727E+00_JPRB, 0.42476E+00_JPRB, 0.48157E+00_JPRB, 0.60750E+00_JPRB ,&
&  0.77865E+00_JPRB, 0.12549E+01_JPRB, 0.19869E+01_JPRB, 0.17911E+01_JPRB, 0.13586E+01_JPRB, 0.85886E+00_JPRB, 0.49698E+00_JPRB ,&
&  0.47506E+00_JPRB, 0.46981E+00_JPRB, 0.47339E+00_JPRB, 0.46240E+00_JPRB, 0.45773E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.47736E+00_JPRB, 0.47607E+00_JPRB, 0.48796E+00_JPRB, 0.49992E+00_JPRB, 0.50613E+00_JPRB, 0.50932E+00_JPRB, 0.51531E+00_JPRB ,&
&  0.51650E+00_JPRB, 0.52021E+00_JPRB, 0.52501E+00_JPRB, 0.53662E+00_JPRB, 0.55572E+00_JPRB, 0.58650E+00_JPRB, 0.62130E+00_JPRB ,&
&  0.66301E+00_JPRB, 0.70810E+00_JPRB, 0.75713E+00_JPRB, 0.80964E+00_JPRB, 0.87543E+00_JPRB, 0.96569E+00_JPRB, 0.10901E+01_JPRB ,&
&  0.11548E+01_JPRB, 0.12359E+01_JPRB, 0.13291E+01_JPRB, 0.13986E+01_JPRB, 0.13859E+01_JPRB, 0.13202E+01_JPRB, 0.10177E+01_JPRB ,&
&  0.11150E+01_JPRB, 0.10855E+01_JPRB, 0.97019E+00_JPRB, 0.86061E+00_JPRB, 0.78977E+00_JPRB, 0.73453E+00_JPRB, 0.69516E+00_JPRB ,&
&  0.68319E+00_JPRB, 0.65495E+00_JPRB, 0.61227E+00_JPRB, 0.58296E+00_JPRB, 0.56598E+00_JPRB, 0.57049E+00_JPRB, 0.56484E+00_JPRB ,&
&  0.61194E+00_JPRB, 0.70223E+00_JPRB, 0.81868E+00_JPRB, 0.96722E+00_JPRB, 0.10471E+01_JPRB, 0.10328E+01_JPRB, 0.91177E+00_JPRB ,&
&  0.81616E+00_JPRB, 0.74514E+00_JPRB, 0.72596E+00_JPRB, 0.75922E+00_JPRB, 0.77979E+00_JPRB, 0.81765E+00_JPRB, 0.81432E+00_JPRB ,&
&  0.73330E+00_JPRB, 0.66433E+00_JPRB, 0.70770E+00_JPRB, 0.88225E+00_JPRB, 0.81256E+00_JPRB, 0.73384E+00_JPRB, 0.74015E+00_JPRB ,&
&  0.69564E+00_JPRB, 0.65948E+00_JPRB, 0.64376E+00_JPRB, 0.61379E+00_JPRB, 0.58342E+00_JPRB, 0.56329E+00_JPRB, 0.54652E+00_JPRB ,&
&  0.51138E+00_JPRB, 0.47122E+00_JPRB, 0.43560E+00_JPRB, 0.41927E+00_JPRB, 0.42314E+00_JPRB, 0.38782E+00_JPRB, 0.44867E+00_JPRB ,&
&  0.43694E+00_JPRB, 0.35435E+00_JPRB, 0.41590E+00_JPRB, 0.51116E+00_JPRB, 0.51434E+00_JPRB, 0.55633E+00_JPRB, 0.62644E+00_JPRB ,&
&  0.70016E+00_JPRB, 0.76467E+00_JPRB, 0.75678E+00_JPRB, 0.64478E+00_JPRB, 0.60028E+00_JPRB, 0.57554E+00_JPRB, 0.54912E+00_JPRB ,&
&  0.52792E+00_JPRB, 0.49104E+00_JPRB, 0.50159E+00_JPRB, 0.48743E+00_JPRB, 0.48323E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.52179E+00_JPRB, 0.51709E+00_JPRB, 0.51782E+00_JPRB, 0.51721E+00_JPRB, 0.52641E+00_JPRB, 0.52721E+00_JPRB, 0.52584E+00_JPRB ,&
&  0.52304E+00_JPRB, 0.52956E+00_JPRB, 0.53340E+00_JPRB, 0.54012E+00_JPRB, 0.54854E+00_JPRB, 0.57228E+00_JPRB, 0.59906E+00_JPRB ,&
&  0.63097E+00_JPRB, 0.66969E+00_JPRB, 0.71070E+00_JPRB, 0.76842E+00_JPRB, 0.84872E+00_JPRB, 0.89767E+00_JPRB, 0.95349E+00_JPRB ,&
&  0.10231E+01_JPRB, 0.11319E+01_JPRB, 0.12491E+01_JPRB, 0.14080E+01_JPRB, 0.15782E+01_JPRB, 0.15649E+01_JPRB, 0.14312E+01_JPRB ,&
&  0.10748E+01_JPRB, 0.10797E+01_JPRB, 0.99725E+00_JPRB, 0.89408E+00_JPRB, 0.89191E+00_JPRB, 0.86403E+00_JPRB, 0.87113E+00_JPRB ,&
&  0.89141E+00_JPRB, 0.88917E+00_JPRB, 0.88049E+00_JPRB, 0.88557E+00_JPRB, 0.88904E+00_JPRB, 0.89417E+00_JPRB, 0.87967E+00_JPRB ,&
&  0.87351E+00_JPRB, 0.90297E+00_JPRB, 0.10393E+01_JPRB, 0.12108E+01_JPRB, 0.13238E+01_JPRB, 0.13435E+01_JPRB, 0.13113E+01_JPRB ,&
&  0.12232E+01_JPRB, 0.11342E+01_JPRB, 0.10513E+01_JPRB, 0.10378E+01_JPRB, 0.10623E+01_JPRB, 0.10740E+01_JPRB, 0.11317E+01_JPRB ,&
&  0.10294E+01_JPRB, 0.84663E+00_JPRB, 0.82343E+00_JPRB, 0.81538E+00_JPRB, 0.76565E+00_JPRB, 0.73302E+00_JPRB, 0.70280E+00_JPRB ,&
&  0.70049E+00_JPRB, 0.66517E+00_JPRB, 0.64456E+00_JPRB, 0.61051E+00_JPRB, 0.57639E+00_JPRB, 0.55495E+00_JPRB, 0.53941E+00_JPRB ,&
&  0.51101E+00_JPRB, 0.48295E+00_JPRB, 0.45983E+00_JPRB, 0.45809E+00_JPRB, 0.43648E+00_JPRB, 0.48531E+00_JPRB, 0.49953E+00_JPRB ,&
&  0.49823E+00_JPRB, 0.45280E+00_JPRB, 0.44669E+00_JPRB, 0.58699E+00_JPRB, 0.60937E+00_JPRB, 0.62097E+00_JPRB, 0.62738E+00_JPRB ,&
&  0.63254E+00_JPRB, 0.62813E+00_JPRB, 0.60427E+00_JPRB, 0.59761E+00_JPRB, 0.60462E+00_JPRB, 0.60978E+00_JPRB, 0.59762E+00_JPRB ,&
&  0.56147E+00_JPRB, 0.54378E+00_JPRB, 0.56510E+00_JPRB, 0.53129E+00_JPRB, 0.52547E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.59361E+00_JPRB, 0.56818E+00_JPRB, 0.55267E+00_JPRB, 0.55963E+00_JPRB, 0.56757E+00_JPRB, 0.55669E+00_JPRB, 0.58361E+00_JPRB ,&
&  0.55976E+00_JPRB, 0.57848E+00_JPRB, 0.58966E+00_JPRB, 0.58709E+00_JPRB, 0.62214E+00_JPRB, 0.65748E+00_JPRB, 0.70211E+00_JPRB ,&
&  0.76274E+00_JPRB, 0.85946E+00_JPRB, 0.97194E+00_JPRB, 0.11079E+01_JPRB, 0.12711E+01_JPRB, 0.13393E+01_JPRB, 0.14053E+01_JPRB ,&
&  0.14964E+01_JPRB, 0.16152E+01_JPRB, 0.17942E+01_JPRB, 0.20061E+01_JPRB, 0.22695E+01_JPRB, 0.20696E+01_JPRB, 0.17491E+01_JPRB ,&
&  0.12969E+01_JPRB, 0.10174E+01_JPRB, 0.97356E+00_JPRB, 0.90036E+00_JPRB, 0.93267E+00_JPRB, 0.10227E+01_JPRB, 0.10589E+01_JPRB ,&
&  0.10820E+01_JPRB, 0.11086E+01_JPRB, 0.11364E+01_JPRB, 0.11698E+01_JPRB, 0.12092E+01_JPRB, 0.12514E+01_JPRB, 0.12723E+01_JPRB ,&
&  0.12884E+01_JPRB, 0.13051E+01_JPRB, 0.13348E+01_JPRB, 0.14171E+01_JPRB, 0.14907E+01_JPRB, 0.15395E+01_JPRB, 0.15810E+01_JPRB ,&
&  0.16135E+01_JPRB, 0.15073E+01_JPRB, 0.12827E+01_JPRB, 0.12781E+01_JPRB, 0.13103E+01_JPRB, 0.13278E+01_JPRB, 0.13300E+01_JPRB ,&
&  0.14254E+01_JPRB, 0.11480E+01_JPRB, 0.77164E+00_JPRB, 0.75965E+00_JPRB, 0.73161E+00_JPRB, 0.74115E+00_JPRB, 0.67757E+00_JPRB ,&
&  0.68034E+00_JPRB, 0.67475E+00_JPRB, 0.66866E+00_JPRB, 0.63446E+00_JPRB, 0.61343E+00_JPRB, 0.59177E+00_JPRB, 0.56414E+00_JPRB ,&
&  0.52848E+00_JPRB, 0.51381E+00_JPRB, 0.49270E+00_JPRB, 0.48453E+00_JPRB, 0.51354E+00_JPRB, 0.48454E+00_JPRB, 0.54908E+00_JPRB ,&
&  0.57027E+00_JPRB, 0.61718E+00_JPRB, 0.55824E+00_JPRB, 0.67279E+00_JPRB, 0.68040E+00_JPRB, 0.67345E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.67805E+00_JPRB, 0.68697E+00_JPRB, 0.69523E+00_JPRB, 0.68346E+00_JPRB, 0.69854E+00_JPRB, 0.72168E+00_JPRB, 0.71782E+00_JPRB ,&
&  0.67534E+00_JPRB, 0.65852E+00_JPRB, 0.63557E+00_JPRB, 0.60754E+00_JPRB, 0.60279E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.63480E+00_JPRB, 0.63894E+00_JPRB, 0.63912E+00_JPRB, 0.64518E+00_JPRB, 0.65049E+00_JPRB, 0.63881E+00_JPRB, 0.65995E+00_JPRB ,&
&  0.65533E+00_JPRB, 0.67552E+00_JPRB, 0.69873E+00_JPRB, 0.72256E+00_JPRB, 0.76762E+00_JPRB, 0.81308E+00_JPRB, 0.87462E+00_JPRB ,&
&  0.97589E+00_JPRB, 0.11376E+01_JPRB, 0.13333E+01_JPRB, 0.15684E+01_JPRB, 0.16131E+01_JPRB, 0.16273E+01_JPRB, 0.16907E+01_JPRB ,&
&  0.17608E+01_JPRB, 0.17961E+01_JPRB, 0.17558E+01_JPRB, 0.17439E+01_JPRB, 0.17388E+01_JPRB, 0.16157E+01_JPRB, 0.14927E+01_JPRB ,&
&  0.11989E+01_JPRB, 0.98600E+00_JPRB, 0.10711E+01_JPRB, 0.10182E+01_JPRB, 0.10488E+01_JPRB, 0.10765E+01_JPRB, 0.11016E+01_JPRB ,&
&  0.11259E+01_JPRB, 0.11356E+01_JPRB, 0.11490E+01_JPRB, 0.11693E+01_JPRB, 0.12032E+01_JPRB, 0.12321E+01_JPRB, 0.12673E+01_JPRB ,&
&  0.12972E+01_JPRB, 0.13250E+01_JPRB, 0.13617E+01_JPRB, 0.13277E+01_JPRB, 0.13610E+01_JPRB, 0.13959E+01_JPRB, 0.14444E+01_JPRB ,&
&  0.14989E+01_JPRB, 0.14332E+01_JPRB, 0.12372E+01_JPRB, 0.13108E+01_JPRB, 0.14294E+01_JPRB, 0.13567E+01_JPRB, 0.13883E+01_JPRB ,&
&  0.15624E+01_JPRB, 0.14322E+01_JPRB, 0.77993E+00_JPRB, 0.66322E+00_JPRB, 0.66561E+00_JPRB, 0.68393E+00_JPRB, 0.70324E+00_JPRB ,&
&  0.69685E+00_JPRB, 0.69118E+00_JPRB, 0.68570E+00_JPRB, 0.67975E+00_JPRB, 0.65520E+00_JPRB, 0.64458E+00_JPRB, 0.60253E+00_JPRB ,&
&  0.51491E+00_JPRB, 0.50313E+00_JPRB, 0.50340E+00_JPRB, 0.51028E+00_JPRB, 0.53914E+00_JPRB, 0.55764E+00_JPRB, 0.58664E+00_JPRB ,&
&  0.63710E+00_JPRB, 0.69310E+00_JPRB, 0.73894E+00_JPRB, 0.68755E+00_JPRB, 0.64232E+00_JPRB, 0.72278E+00_JPRB, 0.72418E+00_JPRB ,&
&  0.73693E+00_JPRB, 0.73769E+00_JPRB, 0.74548E+00_JPRB, 0.74717E+00_JPRB, 0.75221E+00_JPRB, 0.73677E+00_JPRB, 0.71885E+00_JPRB ,&
&  0.69957E+00_JPRB, 0.68327E+00_JPRB, 0.67244E+00_JPRB, 0.65412E+00_JPRB, 0.64057E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.68202E+00_JPRB, 0.66380E+00_JPRB, 0.65419E+00_JPRB, 0.64323E+00_JPRB, 0.64115E+00_JPRB, 0.63813E+00_JPRB, 0.63853E+00_JPRB ,&
&  0.64018E+00_JPRB, 0.64577E+00_JPRB, 0.66575E+00_JPRB, 0.70040E+00_JPRB, 0.74876E+00_JPRB, 0.81321E+00_JPRB, 0.92045E+00_JPRB ,&
&  0.10250E+01_JPRB, 0.11788E+01_JPRB, 0.13566E+01_JPRB, 0.15551E+01_JPRB, 0.16803E+01_JPRB, 0.16987E+01_JPRB, 0.17675E+01_JPRB ,&
&  0.17987E+01_JPRB, 0.17774E+01_JPRB, 0.16697E+01_JPRB, 0.15173E+01_JPRB, 0.13919E+01_JPRB, 0.13394E+01_JPRB, 0.13327E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.98859E+00_JPRB, 0.98721E+00_JPRB, 0.10115E+01_JPRB, 0.10285E+01_JPRB, 0.10430E+01_JPRB, 0.10730E+01_JPRB ,&
&  0.10970E+01_JPRB, 0.11109E+01_JPRB, 0.11175E+01_JPRB, 0.11258E+01_JPRB, 0.11654E+01_JPRB, 0.11733E+01_JPRB, 0.11935E+01_JPRB ,&
&  0.12075E+01_JPRB, 0.12233E+01_JPRB, 0.12741E+01_JPRB, 0.12130E+01_JPRB, 0.12988E+01_JPRB, 0.12890E+01_JPRB, 0.12494E+01_JPRB ,&
&  0.12053E+01_JPRB, 0.11128E+01_JPRB, 0.11331E+01_JPRB, 0.12127E+01_JPRB, 0.12989E+01_JPRB, 0.12725E+01_JPRB, 0.12080E+01_JPRB ,&
&  0.13230E+01_JPRB, 0.13322E+01_JPRB, 0.81232E+00_JPRB, 0.68440E+00_JPRB, 0.68885E+00_JPRB, 0.64404E+00_JPRB, 0.71813E+00_JPRB ,&
&  0.72003E+00_JPRB, 0.71431E+00_JPRB, 0.70809E+00_JPRB, 0.70957E+00_JPRB, 0.71342E+00_JPRB, 0.76535E+00_JPRB, 0.64185E+00_JPRB ,&
&  0.56345E+00_JPRB, 0.52902E+00_JPRB, 0.51633E+00_JPRB, 0.52509E+00_JPRB, 0.55778E+00_JPRB, 0.59094E+00_JPRB, 0.63926E+00_JPRB ,&
&  0.69582E+00_JPRB, 0.75697E+00_JPRB, 0.81544E+00_JPRB, 0.81357E+00_JPRB, 0.73831E+00_JPRB, 0.78273E+00_JPRB, 0.79922E+00_JPRB ,&
&  0.80833E+00_JPRB, 0.80744E+00_JPRB, 0.79505E+00_JPRB, 0.79187E+00_JPRB, 0.77650E+00_JPRB, 0.76888E+00_JPRB, 0.76003E+00_JPRB ,&
&  0.74814E+00_JPRB, 0.74508E+00_JPRB, 0.73615E+00_JPRB, 0.70820E+00_JPRB, 0.69254E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.73059E+00_JPRB, 0.70842E+00_JPRB, 0.69012E+00_JPRB, 0.67516E+00_JPRB, 0.66680E+00_JPRB, 0.64792E+00_JPRB, 0.62320E+00_JPRB ,&
&  0.61268E+00_JPRB, 0.61090E+00_JPRB, 0.62494E+00_JPRB, 0.65338E+00_JPRB, 0.69536E+00_JPRB, 0.73951E+00_JPRB, 0.81869E+00_JPRB ,&
&  0.91304E+00_JPRB, 0.10506E+01_JPRB, 0.12260E+01_JPRB, 0.14644E+01_JPRB, 0.16542E+01_JPRB, 0.18266E+01_JPRB, 0.18580E+01_JPRB ,&
&  0.19327E+01_JPRB, 0.19264E+01_JPRB, 0.17784E+01_JPRB, 0.15205E+01_JPRB, 0.11481E+01_JPRB, 0.10869E+01_JPRB, 0.11509E+01_JPRB ,&
&  0.11505E+01_JPRB, 0.10324E+01_JPRB, 0.97699E+00_JPRB, 0.96937E+00_JPRB, 0.96019E+00_JPRB, 0.98411E+00_JPRB, 0.10092E+01_JPRB ,&
&  0.10354E+01_JPRB, 0.10558E+01_JPRB, 0.10698E+01_JPRB, 0.10928E+01_JPRB, 0.11123E+01_JPRB, 0.11133E+01_JPRB, 0.11289E+01_JPRB ,&
&  0.11148E+01_JPRB, 0.11089E+01_JPRB, 0.11045E+01_JPRB, 0.10866E+01_JPRB, 0.10831E+01_JPRB, 0.10651E+01_JPRB, 0.10297E+01_JPRB ,&
&  0.10002E+01_JPRB, 0.93222E+00_JPRB, 0.96053E+00_JPRB, 0.99723E+00_JPRB, 0.10005E+01_JPRB, 0.95233E+00_JPRB, 0.95805E+00_JPRB ,&
&  0.10710E+01_JPRB, 0.12348E+01_JPRB, 0.80425E+00_JPRB, 0.77894E+00_JPRB, 0.81619E+00_JPRB, 0.74682E+00_JPRB, 0.72022E+00_JPRB ,&
&  0.72787E+00_JPRB, 0.71996E+00_JPRB, 0.71710E+00_JPRB, 0.71880E+00_JPRB, 0.74394E+00_JPRB, 0.81225E+00_JPRB, 0.66696E+00_JPRB ,&
&  0.62877E+00_JPRB, 0.59693E+00_JPRB, 0.59323E+00_JPRB, 0.60152E+00_JPRB, 0.60228E+00_JPRB, 0.57716E+00_JPRB, 0.58265E+00_JPRB ,&
&  0.61434E+00_JPRB, 0.88741E+00_JPRB, 0.11174E+01_JPRB, 0.13322E+01_JPRB, 0.12737E+01_JPRB, 0.80026E+00_JPRB, 0.86326E+00_JPRB ,&
&  0.87731E+00_JPRB, 0.87062E+00_JPRB, 0.83348E+00_JPRB, 0.82082E+00_JPRB, 0.81019E+00_JPRB, 0.79694E+00_JPRB, 0.79405E+00_JPRB ,&
&  0.78599E+00_JPRB, 0.78911E+00_JPRB, 0.79589E+00_JPRB, 0.77792E+00_JPRB, 0.75150E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.78809E+00_JPRB, 0.77061E+00_JPRB, 0.75020E+00_JPRB, 0.73019E+00_JPRB, 0.71188E+00_JPRB, 0.66747E+00_JPRB, 0.62232E+00_JPRB ,&
&  0.59296E+00_JPRB, 0.58056E+00_JPRB, 0.58702E+00_JPRB, 0.59920E+00_JPRB, 0.62706E+00_JPRB, 0.65785E+00_JPRB, 0.70954E+00_JPRB ,&
&  0.78854E+00_JPRB, 0.90119E+00_JPRB, 0.10598E+01_JPRB, 0.12575E+01_JPRB, 0.14939E+01_JPRB, 0.17672E+01_JPRB, 0.19445E+01_JPRB ,&
&  0.21064E+01_JPRB, 0.21341E+01_JPRB, 0.18804E+01_JPRB, 0.13161E+01_JPRB, 0.11289E+01_JPRB, 0.10657E+01_JPRB, 0.97109E+00_JPRB ,&
&  0.96111E+00_JPRB, 0.94839E+00_JPRB, 0.93055E+00_JPRB, 0.91625E+00_JPRB, 0.90927E+00_JPRB, 0.92474E+00_JPRB, 0.94161E+00_JPRB ,&
&  0.95868E+00_JPRB, 0.97297E+00_JPRB, 0.98850E+00_JPRB, 0.10181E+01_JPRB, 0.10237E+01_JPRB, 0.10365E+01_JPRB, 0.10557E+01_JPRB ,&
&  0.10589E+01_JPRB, 0.10437E+01_JPRB, 0.10000E+01_JPRB, 0.97462E+00_JPRB, 0.94380E+00_JPRB, 0.91282E+00_JPRB, 0.88055E+00_JPRB ,&
&  0.86820E+00_JPRB, 0.85048E+00_JPRB, 0.84467E+00_JPRB, 0.85680E+00_JPRB, 0.86023E+00_JPRB, 0.79593E+00_JPRB, 0.82345E+00_JPRB ,&
&  0.92429E+00_JPRB, 0.10299E+01_JPRB, 0.85617E+00_JPRB, 0.87837E+00_JPRB, 0.62813E+00_JPRB, 0.68510E+00_JPRB, 0.74566E+00_JPRB ,&
&  0.73576E+00_JPRB, 0.72707E+00_JPRB, 0.71274E+00_JPRB, 0.73185E+00_JPRB, 0.75479E+00_JPRB, 0.74663E+00_JPRB, 0.71552E+00_JPRB ,&
&  0.68617E+00_JPRB, 0.65510E+00_JPRB, 0.66450E+00_JPRB, 0.68326E+00_JPRB, 0.68031E+00_JPRB, 0.63436E+00_JPRB, 0.58787E+00_JPRB ,&
&  0.58443E+00_JPRB, 0.78234E+00_JPRB, 0.89471E+00_JPRB, 0.99806E+00_JPRB, 0.91715E+00_JPRB, 0.77155E+00_JPRB, 0.81928E+00_JPRB ,&
&  0.85285E+00_JPRB, 0.86456E+00_JPRB, 0.85328E+00_JPRB, 0.83293E+00_JPRB, 0.80887E+00_JPRB, 0.78818E+00_JPRB, 0.77779E+00_JPRB ,&
&  0.77503E+00_JPRB, 0.77240E+00_JPRB, 0.77967E+00_JPRB, 0.79043E+00_JPRB, 0.79707E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.77942E+00_JPRB, 0.77732E+00_JPRB, 0.77842E+00_JPRB, 0.77993E+00_JPRB, 0.77340E+00_JPRB, 0.74570E+00_JPRB, 0.68684E+00_JPRB ,&
&  0.59998E+00_JPRB, 0.53431E+00_JPRB, 0.52746E+00_JPRB, 0.53264E+00_JPRB, 0.54530E+00_JPRB, 0.57562E+00_JPRB, 0.62394E+00_JPRB ,&
&  0.68015E+00_JPRB, 0.77342E+00_JPRB, 0.90793E+00_JPRB, 0.10791E+01_JPRB, 0.13028E+01_JPRB, 0.16328E+01_JPRB, 0.19905E+01_JPRB ,&
&  0.19224E+01_JPRB, 0.14416E+01_JPRB, 0.18240E+01_JPRB, 0.18322E+01_JPRB, 0.16018E+01_JPRB, 0.12725E+01_JPRB, 0.10505E+01_JPRB ,&
&  0.98462E+00_JPRB, 0.95909E+00_JPRB, 0.94090E+00_JPRB, 0.91321E+00_JPRB, 0.86927E+00_JPRB, 0.87672E+00_JPRB, 0.88859E+00_JPRB ,&
&  0.90242E+00_JPRB, 0.91417E+00_JPRB, 0.92054E+00_JPRB, 0.95523E+00_JPRB, 0.96000E+00_JPRB, 0.96639E+00_JPRB, 0.98601E+00_JPRB ,&
&  0.98702E+00_JPRB, 0.97048E+00_JPRB, 0.93142E+00_JPRB, 0.87376E+00_JPRB, 0.85328E+00_JPRB, 0.82993E+00_JPRB, 0.81374E+00_JPRB ,&
&  0.81452E+00_JPRB, 0.80545E+00_JPRB, 0.81198E+00_JPRB, 0.81300E+00_JPRB, 0.82349E+00_JPRB, 0.76888E+00_JPRB, 0.78668E+00_JPRB ,&
&  0.83178E+00_JPRB, 0.92447E+00_JPRB, 0.90176E+00_JPRB, 0.90874E+00_JPRB, 0.68272E+00_JPRB, 0.71636E+00_JPRB, 0.70817E+00_JPRB ,&
&  0.75271E+00_JPRB, 0.72805E+00_JPRB, 0.70680E+00_JPRB, 0.71635E+00_JPRB, 0.74647E+00_JPRB, 0.75368E+00_JPRB, 0.76735E+00_JPRB ,&
&  0.71027E+00_JPRB, 0.69139E+00_JPRB, 0.69222E+00_JPRB, 0.72084E+00_JPRB, 0.70830E+00_JPRB, 0.68171E+00_JPRB, 0.66123E+00_JPRB ,&
&  0.73752E+00_JPRB, 0.78594E+00_JPRB, 0.85335E+00_JPRB, 0.88388E+00_JPRB, 0.82488E+00_JPRB, 0.75036E+00_JPRB, 0.76724E+00_JPRB ,&
&  0.78855E+00_JPRB, 0.80533E+00_JPRB, 0.80410E+00_JPRB, 0.78707E+00_JPRB, 0.75748E+00_JPRB, 0.73196E+00_JPRB, 0.71885E+00_JPRB ,&
&  0.70876E+00_JPRB, 0.71587E+00_JPRB, 0.72671E+00_JPRB, 0.74109E+00_JPRB, 0.76230E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.68817E+00_JPRB, 0.70811E+00_JPRB, 0.72482E+00_JPRB, 0.76904E+00_JPRB, 0.82306E+00_JPRB, 0.89448E+00_JPRB, 0.93002E+00_JPRB ,&
&  0.78891E+00_JPRB, 0.50632E+00_JPRB, 0.49625E+00_JPRB, 0.49312E+00_JPRB, 0.49528E+00_JPRB, 0.50629E+00_JPRB, 0.54833E+00_JPRB ,&
&  0.61160E+00_JPRB, 0.69868E+00_JPRB, 0.79895E+00_JPRB, 0.91706E+00_JPRB, 0.10757E+01_JPRB, 0.13876E+01_JPRB, 0.16006E+01_JPRB ,&
&  0.13688E+01_JPRB, 0.17067E+01_JPRB, 0.22710E+01_JPRB, 0.20606E+01_JPRB, 0.18248E+01_JPRB, 0.15347E+01_JPRB, 0.11134E+01_JPRB ,&
&  0.10444E+01_JPRB, 0.99331E+00_JPRB, 0.97615E+00_JPRB, 0.97925E+00_JPRB, 0.87914E+00_JPRB, 0.86892E+00_JPRB, 0.87116E+00_JPRB ,&
&  0.87816E+00_JPRB, 0.88069E+00_JPRB, 0.88284E+00_JPRB, 0.92212E+00_JPRB, 0.94877E+00_JPRB, 0.95569E+00_JPRB, 0.95106E+00_JPRB ,&
&  0.94097E+00_JPRB, 0.92903E+00_JPRB, 0.89813E+00_JPRB, 0.82993E+00_JPRB, 0.81815E+00_JPRB, 0.80993E+00_JPRB, 0.81588E+00_JPRB ,&
&  0.83027E+00_JPRB, 0.87277E+00_JPRB, 0.89227E+00_JPRB, 0.84889E+00_JPRB, 0.78567E+00_JPRB, 0.82153E+00_JPRB, 0.76255E+00_JPRB ,&
&  0.77052E+00_JPRB, 0.81342E+00_JPRB, 0.94884E+00_JPRB, 0.77773E+00_JPRB, 0.73497E+00_JPRB, 0.77175E+00_JPRB, 0.77401E+00_JPRB ,&
&  0.76359E+00_JPRB, 0.76999E+00_JPRB, 0.72709E+00_JPRB, 0.70559E+00_JPRB, 0.72465E+00_JPRB, 0.72593E+00_JPRB, 0.81154E+00_JPRB ,&
&  0.75571E+00_JPRB, 0.79387E+00_JPRB, 0.74213E+00_JPRB, 0.73384E+00_JPRB, 0.70072E+00_JPRB, 0.65794E+00_JPRB, 0.65540E+00_JPRB ,&
&  0.77179E+00_JPRB, 0.80957E+00_JPRB, 0.83370E+00_JPRB, 0.85676E+00_JPRB, 0.83840E+00_JPRB, 0.82277E+00_JPRB, 0.79005E+00_JPRB ,&
&  0.77455E+00_JPRB, 0.77455E+00_JPRB, 0.77491E+00_JPRB, 0.74668E+00_JPRB, 0.72134E+00_JPRB, 0.68644E+00_JPRB, 0.65761E+00_JPRB ,&
&  0.64144E+00_JPRB, 0.63461E+00_JPRB, 0.63587E+00_JPRB, 0.64848E+00_JPRB, 0.66626E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.61763E+00_JPRB, 0.62719E+00_JPRB, 0.62706E+00_JPRB, 0.60409E+00_JPRB, 0.59063E+00_JPRB, 0.57888E+00_JPRB, 0.54465E+00_JPRB ,&
&  0.51792E+00_JPRB, 0.50041E+00_JPRB, 0.48472E+00_JPRB, 0.47554E+00_JPRB, 0.47835E+00_JPRB, 0.47610E+00_JPRB, 0.49059E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.62098E+00_JPRB, 0.74288E+00_JPRB, 0.81839E+00_JPRB, 0.91028E+00_JPRB, 0.11064E+01_JPRB, 0.12057E+01_JPRB ,&
&  0.12069E+01_JPRB, 0.23795E+01_JPRB, 0.22784E+01_JPRB, 0.22520E+01_JPRB, 0.21145E+01_JPRB, 0.18408E+01_JPRB, 0.13206E+01_JPRB ,&
&  0.11449E+01_JPRB, 0.10567E+01_JPRB, 0.98425E+00_JPRB, 0.94901E+00_JPRB, 0.93205E+00_JPRB, 0.88968E+00_JPRB, 0.89184E+00_JPRB ,&
&  0.88925E+00_JPRB, 0.88491E+00_JPRB, 0.87253E+00_JPRB, 0.89859E+00_JPRB, 0.93648E+00_JPRB, 0.95305E+00_JPRB, 0.96371E+00_JPRB ,&
&  0.95052E+00_JPRB, 0.93443E+00_JPRB, 0.89724E+00_JPRB, 0.86841E+00_JPRB, 0.85823E+00_JPRB, 0.87560E+00_JPRB, 0.90282E+00_JPRB ,&
&  0.90635E+00_JPRB, 0.90883E+00_JPRB, 0.98478E+00_JPRB, 0.97703E+00_JPRB, 0.93041E+00_JPRB, 0.92326E+00_JPRB, 0.80624E+00_JPRB ,&
&  0.79271E+00_JPRB, 0.75112E+00_JPRB, 0.85513E+00_JPRB, 0.76083E+00_JPRB, 0.75865E+00_JPRB, 0.79160E+00_JPRB, 0.79682E+00_JPRB ,&
&  0.77134E+00_JPRB, 0.80204E+00_JPRB, 0.76896E+00_JPRB, 0.73560E+00_JPRB, 0.71816E+00_JPRB, 0.72556E+00_JPRB, 0.78695E+00_JPRB ,&
&  0.81899E+00_JPRB, 0.82663E+00_JPRB, 0.85062E+00_JPRB, 0.76440E+00_JPRB, 0.74009E+00_JPRB, 0.67417E+00_JPRB, 0.66679E+00_JPRB ,&
&  0.76502E+00_JPRB, 0.78198E+00_JPRB, 0.83603E+00_JPRB, 0.85177E+00_JPRB, 0.85263E+00_JPRB, 0.87575E+00_JPRB, 0.84799E+00_JPRB ,&
&  0.81159E+00_JPRB, 0.78753E+00_JPRB, 0.77094E+00_JPRB, 0.74763E+00_JPRB, 0.70935E+00_JPRB, 0.66911E+00_JPRB, 0.63800E+00_JPRB ,&
&  0.60938E+00_JPRB, 0.59117E+00_JPRB, 0.58831E+00_JPRB, 0.59204E+00_JPRB, 0.60291E+00_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.57695E+00_JPRB, 0.58445E+00_JPRB, 0.60028E+00_JPRB, 0.58787E+00_JPRB, 0.56841E+00_JPRB, 0.55014E+00_JPRB, 0.53873E+00_JPRB ,&
&  0.52056E+00_JPRB, 0.49666E+00_JPRB, 0.47734E+00_JPRB, 0.46769E+00_JPRB, 0.46303E+00_JPRB, 0.46615E+00_JPRB, 0.47722E+00_JPRB ,&
&  0.50750E+00_JPRB, 0.54871E+00_JPRB, 0.62910E+00_JPRB, 0.71495E+00_JPRB, 0.78501E+00_JPRB, 0.91343E+00_JPRB, 0.82278E+00_JPRB ,&
&  0.11405E+01_JPRB, 0.22163E+01_JPRB, 0.23381E+01_JPRB, 0.23762E+01_JPRB, 0.23174E+01_JPRB, 0.22373E+01_JPRB, 0.17007E+01_JPRB ,&
&  0.14021E+01_JPRB, 0.12640E+01_JPRB, 0.10841E+01_JPRB, 0.10027E+01_JPRB, 0.97314E+00_JPRB, 0.95994E+00_JPRB, 0.94274E+00_JPRB ,&
&  0.91978E+00_JPRB, 0.89757E+00_JPRB, 0.88904E+00_JPRB, 0.91365E+00_JPRB, 0.95219E+00_JPRB, 0.99371E+00_JPRB, 0.99968E+00_JPRB ,&
&  0.98022E+00_JPRB, 0.95616E+00_JPRB, 0.94972E+00_JPRB, 0.89881E+00_JPRB, 0.92585E+00_JPRB, 0.94975E+00_JPRB, 0.97962E+00_JPRB ,&
&  0.95369E+00_JPRB, 0.93612E+00_JPRB, 0.10684E+01_JPRB, 0.11633E+01_JPRB, 0.11559E+01_JPRB, 0.10420E+01_JPRB, 0.91003E+00_JPRB ,&
&  0.84666E+00_JPRB, 0.78377E+00_JPRB, 0.79435E+00_JPRB, 0.76016E+00_JPRB, 0.77215E+00_JPRB, 0.79132E+00_JPRB, 0.82017E+00_JPRB ,&
&  0.84446E+00_JPRB, 0.82980E+00_JPRB, 0.80042E+00_JPRB, 0.74615E+00_JPRB, 0.72953E+00_JPRB, 0.70829E+00_JPRB, 0.78458E+00_JPRB ,&
&  0.88327E+00_JPRB, 0.94541E+00_JPRB, 0.97119E+00_JPRB, 0.87023E+00_JPRB, 0.80135E+00_JPRB, 0.67503E+00_JPRB, 0.62372E+00_JPRB ,&
&  0.70854E+00_JPRB, 0.72190E+00_JPRB, 0.69323E+00_JPRB, 0.77692E+00_JPRB, 0.78556E+00_JPRB, 0.82356E+00_JPRB, 0.84121E+00_JPRB ,&
&  0.79176E+00_JPRB, 0.76411E+00_JPRB, 0.73455E+00_JPRB, 0.72006E+00_JPRB, 0.71932E+00_JPRB, 0.68936E+00_JPRB, 0.65944E+00_JPRB ,&
&  0.62984E+00_JPRB, 0.60845E+00_JPRB, 0.59047E+00_JPRB, 0.57964E+00_JPRB, 0.57344E+00_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.56600E+00_JPRB, 0.56185E+00_JPRB, 0.56600E+00_JPRB, 0.57582E+00_JPRB, 0.57844E+00_JPRB, 0.56107E+00_JPRB, 0.54183E+00_JPRB ,&
&  0.51977E+00_JPRB, 0.49460E+00_JPRB, 0.47673E+00_JPRB, 0.46275E+00_JPRB, 0.45866E+00_JPRB, 0.46132E+00_JPRB, 0.47185E+00_JPRB ,&
&  0.49221E+00_JPRB, 0.52305E+00_JPRB, 0.60533E+00_JPRB, 0.66170E+00_JPRB, 0.69620E+00_JPRB, 0.66491E+00_JPRB, 0.78801E+00_JPRB ,&
&  0.14704E+01_JPRB, 0.22471E+01_JPRB, 0.22416E+01_JPRB, 0.23480E+01_JPRB, 0.24751E+01_JPRB, 0.26391E+01_JPRB, 0.20647E+01_JPRB ,&
&  0.16874E+01_JPRB, 0.15303E+01_JPRB, 0.13045E+01_JPRB, 0.11630E+01_JPRB, 0.10694E+01_JPRB, 0.10262E+01_JPRB, 0.10085E+01_JPRB ,&
&  0.97715E+00_JPRB, 0.94117E+00_JPRB, 0.91660E+00_JPRB, 0.93609E+00_JPRB, 0.99997E+00_JPRB, 0.10451E+01_JPRB, 0.10582E+01_JPRB ,&
&  0.10134E+01_JPRB, 0.96302E+00_JPRB, 0.95410E+00_JPRB, 0.93687E+00_JPRB, 0.95909E+00_JPRB, 0.97642E+00_JPRB, 0.10094E+01_JPRB ,&
&  0.10201E+01_JPRB, 0.10937E+01_JPRB, 0.11799E+01_JPRB, 0.12157E+01_JPRB, 0.12169E+01_JPRB, 0.11728E+01_JPRB, 0.10576E+01_JPRB ,&
&  0.94076E+00_JPRB, 0.81529E+00_JPRB, 0.73576E+00_JPRB, 0.75624E+00_JPRB, 0.79054E+00_JPRB, 0.79853E+00_JPRB, 0.76499E+00_JPRB ,&
&  0.75032E+00_JPRB, 0.69909E+00_JPRB, 0.71481E+00_JPRB, 0.73501E+00_JPRB, 0.76755E+00_JPRB, 0.73685E+00_JPRB, 0.82590E+00_JPRB ,&
&  0.81861E+00_JPRB, 0.65034E+00_JPRB, 0.60004E+00_JPRB, 0.69918E+00_JPRB, 0.63575E+00_JPRB, 0.54751E+00_JPRB, 0.55777E+00_JPRB ,&
&  0.60851E+00_JPRB, 0.67559E+00_JPRB, 0.65302E+00_JPRB, 0.62852E+00_JPRB, 0.73532E+00_JPRB, 0.78086E+00_JPRB, 0.82581E+00_JPRB ,&
&  0.72735E+00_JPRB, 0.70433E+00_JPRB, 0.68706E+00_JPRB, 0.66776E+00_JPRB, 0.65615E+00_JPRB, 0.65523E+00_JPRB, 0.65253E+00_JPRB ,&
&  0.63560E+00_JPRB, 0.61939E+00_JPRB, 0.60952E+00_JPRB, 0.59844E+00_JPRB, 0.58114E+00_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.57255E+00_JPRB, 0.55583E+00_JPRB, 0.55007E+00_JPRB, 0.54852E+00_JPRB, 0.54760E+00_JPRB, 0.54128E+00_JPRB, 0.52855E+00_JPRB ,&
&  0.51071E+00_JPRB, 0.48751E+00_JPRB, 0.46798E+00_JPRB, 0.45530E+00_JPRB, 0.44922E+00_JPRB, 0.45283E+00_JPRB, 0.46591E+00_JPRB ,&
&  0.47947E+00_JPRB, 0.52292E+00_JPRB, 0.63338E+00_JPRB, 0.68207E+00_JPRB, 0.67040E+00_JPRB, 0.59683E+00_JPRB, 0.78533E+00_JPRB ,&
&  0.12560E+01_JPRB, 0.19206E+01_JPRB, 0.24691E+01_JPRB, 0.26567E+01_JPRB, 0.27642E+01_JPRB, 0.26611E+01_JPRB, 0.22641E+01_JPRB ,&
&  0.19314E+01_JPRB, 0.17704E+01_JPRB, 0.15669E+01_JPRB, 0.13701E+01_JPRB, 0.12100E+01_JPRB, 0.11189E+01_JPRB, 0.10643E+01_JPRB ,&
&  0.10369E+01_JPRB, 0.97809E+00_JPRB, 0.92949E+00_JPRB, 0.93774E+00_JPRB, 0.99939E+00_JPRB, 0.10703E+01_JPRB, 0.10815E+01_JPRB ,&
&  0.10397E+01_JPRB, 0.96942E+00_JPRB, 0.95319E+00_JPRB, 0.98316E+00_JPRB, 0.83270E+00_JPRB, 0.93831E+00_JPRB, 0.94193E+00_JPRB ,&
&  0.98152E+00_JPRB, 0.10503E+01_JPRB, 0.10676E+01_JPRB, 0.12682E+01_JPRB, 0.13090E+01_JPRB, 0.12427E+01_JPRB, 0.11750E+01_JPRB ,&
&  0.10854E+01_JPRB, 0.89693E+00_JPRB, 0.78100E+00_JPRB, 0.80953E+00_JPRB, 0.83494E+00_JPRB, 0.82204E+00_JPRB, 0.64888E+00_JPRB ,&
&  0.64890E+00_JPRB, 0.74507E+00_JPRB, 0.74464E+00_JPRB, 0.69643E+00_JPRB, 0.77537E+00_JPRB, 0.77397E+00_JPRB, 0.66557E+00_JPRB ,&
&  0.43937E+00_JPRB, 0.38749E+00_JPRB, 0.39938E+00_JPRB, 0.40469E+00_JPRB, 0.39364E+00_JPRB, 0.40833E+00_JPRB, 0.58718E+00_JPRB ,&
&  0.54929E+00_JPRB, 0.64135E+00_JPRB, 0.64114E+00_JPRB, 0.64330E+00_JPRB, 0.70817E+00_JPRB, 0.80560E+00_JPRB, 0.97207E+00_JPRB ,&
&  0.78438E+00_JPRB, 0.73438E+00_JPRB, 0.67866E+00_JPRB, 0.63896E+00_JPRB, 0.62894E+00_JPRB, 0.62515E+00_JPRB, 0.62600E+00_JPRB ,&
&  0.61828E+00_JPRB, 0.60511E+00_JPRB, 0.59909E+00_JPRB, 0.59697E+00_JPRB, 0.58944E+00_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.55670E+00_JPRB, 0.55114E+00_JPRB, 0.54538E+00_JPRB, 0.53927E+00_JPRB, 0.52936E+00_JPRB, 0.51779E+00_JPRB, 0.50799E+00_JPRB ,&
&  0.49651E+00_JPRB, 0.47592E+00_JPRB, 0.45782E+00_JPRB, 0.44520E+00_JPRB, 0.43935E+00_JPRB, 0.44173E+00_JPRB, 0.45396E+00_JPRB ,&
&  0.47318E+00_JPRB, 0.53473E+00_JPRB, 0.75645E+00_JPRB, 0.71701E+00_JPRB, 0.72815E+00_JPRB, 0.67380E+00_JPRB, 0.73871E+00_JPRB ,&
&  0.10931E+01_JPRB, 0.16234E+01_JPRB, 0.22295E+01_JPRB, 0.25946E+01_JPRB, 0.30060E+01_JPRB, 0.30647E+01_JPRB, 0.28059E+01_JPRB ,&
&  0.23187E+01_JPRB, 0.19521E+01_JPRB, 0.16561E+01_JPRB, 0.14202E+01_JPRB, 0.12314E+01_JPRB, 0.11215E+01_JPRB, 0.10622E+01_JPRB ,&
&  0.10194E+01_JPRB, 0.10020E+01_JPRB, 0.94706E+00_JPRB, 0.94292E+00_JPRB, 0.10119E+01_JPRB, 0.10902E+01_JPRB, 0.11115E+01_JPRB ,&
&  0.10632E+01_JPRB, 0.98404E+00_JPRB, 0.93855E+00_JPRB, 0.94472E+00_JPRB, 0.83289E+00_JPRB, 0.74983E+00_JPRB, 0.83886E+00_JPRB ,&
&  0.94231E+00_JPRB, 0.10816E+01_JPRB, 0.11091E+01_JPRB, 0.13239E+01_JPRB, 0.13574E+01_JPRB, 0.12987E+01_JPRB, 0.13680E+01_JPRB ,&
&  0.12820E+01_JPRB, 0.11526E+01_JPRB, 0.87377E+00_JPRB, 0.87599E+00_JPRB, 0.90237E+00_JPRB, 0.73723E+00_JPRB, 0.66461E+00_JPRB ,&
&  0.75917E+00_JPRB, 0.74004E+00_JPRB, 0.70871E+00_JPRB, 0.64417E+00_JPRB, 0.76190E+00_JPRB, 0.68898E+00_JPRB, 0.45363E+00_JPRB ,&
&  0.40290E+00_JPRB, 0.37868E+00_JPRB, 0.36023E+00_JPRB, 0.34163E+00_JPRB, 0.32148E+00_JPRB, 0.33119E+00_JPRB, 0.44760E+00_JPRB ,&
&  0.48164E+00_JPRB, 0.59286E+00_JPRB, 0.62464E+00_JPRB, 0.62398E+00_JPRB, 0.65940E+00_JPRB, 0.78573E+00_JPRB, 0.10120E+01_JPRB ,&
&  0.80640E+00_JPRB, 0.78942E+00_JPRB, 0.71571E+00_JPRB, 0.66449E+00_JPRB, 0.62870E+00_JPRB, 0.60851E+00_JPRB, 0.60516E+00_JPRB ,&
&  0.59722E+00_JPRB, 0.58792E+00_JPRB, 0.57947E+00_JPRB, 0.57190E+00_JPRB, 0.56460E+00_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.53862E+00_JPRB, 0.53322E+00_JPRB, 0.53207E+00_JPRB, 0.53117E+00_JPRB, 0.52541E+00_JPRB, 0.51268E+00_JPRB, 0.50091E+00_JPRB ,&
&  0.48948E+00_JPRB, 0.46809E+00_JPRB, 0.45341E+00_JPRB, 0.44309E+00_JPRB, 0.43561E+00_JPRB, 0.43538E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.45922E+00_JPRB, 0.56246E+00_JPRB, 0.72966E+00_JPRB, 0.63849E+00_JPRB, 0.59904E+00_JPRB, 0.56400E+00_JPRB, 0.57108E+00_JPRB ,&
&  0.94665E+00_JPRB, 0.13915E+01_JPRB, 0.18982E+01_JPRB, 0.25538E+01_JPRB, 0.30577E+01_JPRB, 0.29366E+01_JPRB, 0.30320E+01_JPRB ,&
&  0.26600E+01_JPRB, 0.21759E+01_JPRB, 0.18063E+01_JPRB, 0.15272E+01_JPRB, 0.12939E+01_JPRB, 0.11512E+01_JPRB, 0.10646E+01_JPRB ,&
&  0.10244E+01_JPRB, 0.10009E+01_JPRB, 0.96034E+00_JPRB, 0.94620E+00_JPRB, 0.10076E+01_JPRB, 0.11267E+01_JPRB, 0.11819E+01_JPRB ,&
&  0.10987E+01_JPRB, 0.10123E+01_JPRB, 0.94263E+00_JPRB, 0.91928E+00_JPRB, 0.91836E+00_JPRB, 0.88861E+00_JPRB, 0.82476E+00_JPRB ,&
&  0.81470E+00_JPRB, 0.89309E+00_JPRB, 0.11569E+01_JPRB, 0.12476E+01_JPRB, 0.13088E+01_JPRB, 0.13691E+01_JPRB, 0.14243E+01_JPRB ,&
&  0.13684E+01_JPRB, 0.12802E+01_JPRB, 0.10247E+01_JPRB, 0.99526E+00_JPRB, 0.84249E+00_JPRB, 0.70574E+00_JPRB, 0.78246E+00_JPRB ,&
&  0.83483E+00_JPRB, 0.81326E+00_JPRB, 0.72600E+00_JPRB, 0.58790E+00_JPRB, 0.58123E+00_JPRB, 0.47976E+00_JPRB, 0.37618E+00_JPRB ,&
&  0.36659E+00_JPRB, 0.35385E+00_JPRB, 0.32719E+00_JPRB, 0.31881E+00_JPRB, 0.30439E+00_JPRB, 0.32551E+00_JPRB, 0.39683E+00_JPRB ,&
&  0.47481E+00_JPRB, 0.52260E+00_JPRB, 0.54343E+00_JPRB, 0.53569E+00_JPRB, 0.53844E+00_JPRB, 0.56835E+00_JPRB, 0.69908E+00_JPRB ,&
&  0.66025E+00_JPRB, 0.74780E+00_JPRB, 0.69675E+00_JPRB, 0.64695E+00_JPRB, 0.62262E+00_JPRB, 0.60345E+00_JPRB, 0.59440E+00_JPRB ,&
&  0.58804E+00_JPRB, 0.58184E+00_JPRB, 0.57423E+00_JPRB, 0.56019E+00_JPRB, 0.54926E+00_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.53029E+00_JPRB, 0.52010E+00_JPRB, 0.51773E+00_JPRB, 0.51769E+00_JPRB, 0.51671E+00_JPRB, 0.50992E+00_JPRB, 0.50013E+00_JPRB ,&
&  0.48618E+00_JPRB, 0.46783E+00_JPRB, 0.45653E+00_JPRB, 0.44855E+00_JPRB, 0.43794E+00_JPRB, 0.43689E+00_JPRB, 0.44433E+00_JPRB ,&
&  0.47238E+00_JPRB, 0.68827E+00_JPRB, 0.53484E+00_JPRB, 0.45748E+00_JPRB, 0.51860E+00_JPRB, 0.53921E+00_JPRB, 0.50507E+00_JPRB ,&
&  0.83762E+00_JPRB, 0.12790E+01_JPRB, 0.16764E+01_JPRB, 0.25298E+01_JPRB, 0.34373E+01_JPRB, 0.32286E+01_JPRB, 0.31435E+01_JPRB ,&
&  0.30298E+01_JPRB, 0.24061E+01_JPRB, 0.19873E+01_JPRB, 0.16826E+01_JPRB, 0.13987E+01_JPRB, 0.12008E+01_JPRB, 0.10578E+01_JPRB ,&
&  0.99108E+00_JPRB, 0.98484E+00_JPRB, 0.96003E+00_JPRB, 0.96835E+00_JPRB, 0.10036E+01_JPRB, 0.10976E+01_JPRB, 0.11971E+01_JPRB ,&
&  0.11631E+01_JPRB, 0.10773E+01_JPRB, 0.98999E+00_JPRB, 0.96186E+00_JPRB, 0.90883E+00_JPRB, 0.87601E+00_JPRB, 0.10207E+01_JPRB ,&
&  0.10132E+01_JPRB, 0.11041E+01_JPRB, 0.11610E+01_JPRB, 0.14045E+01_JPRB, 0.15379E+01_JPRB, 0.14832E+01_JPRB, 0.15567E+01_JPRB ,&
&  0.12310E+01_JPRB, 0.10800E+01_JPRB, 0.10520E+01_JPRB, 0.96795E+00_JPRB, 0.77618E+00_JPRB, 0.83950E+00_JPRB, 0.11593E+01_JPRB ,&
&  0.93805E+00_JPRB, 0.99848E+00_JPRB, 0.10233E+01_JPRB, 0.89401E+00_JPRB, 0.51925E+00_JPRB, 0.43056E+00_JPRB, 0.55571E+00_JPRB ,&
&  0.57940E+00_JPRB, 0.42786E+00_JPRB, 0.34564E+00_JPRB, 0.37880E+00_JPRB, 0.32328E+00_JPRB, 0.35291E+00_JPRB, 0.41784E+00_JPRB ,&
&  0.44156E+00_JPRB, 0.46624E+00_JPRB, 0.51679E+00_JPRB, 0.48616E+00_JPRB, 0.49233E+00_JPRB, 0.50286E+00_JPRB, 0.57733E+00_JPRB ,&
&  0.58775E+00_JPRB, 0.69181E+00_JPRB, 0.70378E+00_JPRB, 0.63695E+00_JPRB, 0.60139E+00_JPRB, 0.58036E+00_JPRB, 0.57383E+00_JPRB ,&
&  0.57530E+00_JPRB, 0.57339E+00_JPRB, 0.57001E+00_JPRB, 0.55939E+00_JPRB, 0.54330E+00_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.51601E+00_JPRB, 0.50701E+00_JPRB, 0.50156E+00_JPRB, 0.50292E+00_JPRB, 0.50777E+00_JPRB, 0.50960E+00_JPRB, 0.50239E+00_JPRB ,&
&  0.48914E+00_JPRB, 0.47361E+00_JPRB, 0.45924E+00_JPRB, 0.45248E+00_JPRB, 0.44307E+00_JPRB, 0.44118E+00_JPRB, 0.44902E+00_JPRB ,&
&  0.48049E+00_JPRB, 0.53452E+00_JPRB, 0.40803E+00_JPRB, 0.38506E+00_JPRB, 0.52665E+00_JPRB, 0.52330E+00_JPRB, 0.54778E+00_JPRB ,&
&  0.80688E+00_JPRB, 0.10919E+01_JPRB, 0.14751E+01_JPRB, 0.22838E+01_JPRB, 0.27215E+01_JPRB, 0.31142E+01_JPRB, 0.31316E+01_JPRB ,&
&  0.31502E+01_JPRB, 0.25697E+01_JPRB, 0.20567E+01_JPRB, 0.16736E+01_JPRB, 0.13895E+01_JPRB, 0.12123E+01_JPRB, 0.10943E+01_JPRB ,&
&  0.98748E+00_JPRB, 0.93369E+00_JPRB, 0.94290E+00_JPRB, 0.94797E+00_JPRB, 0.99106E+00_JPRB, 0.10409E+01_JPRB, 0.11249E+01_JPRB ,&
&  0.11589E+01_JPRB, 0.10925E+01_JPRB, 0.99151E+00_JPRB, 0.95674E+00_JPRB, 0.85890E+00_JPRB, 0.80501E+00_JPRB, 0.87620E+00_JPRB ,&
&  0.10440E+01_JPRB, 0.11353E+01_JPRB, 0.12447E+01_JPRB, 0.14146E+01_JPRB, 0.15943E+01_JPRB, 0.15403E+01_JPRB, 0.17413E+01_JPRB ,&
&  0.14751E+01_JPRB, 0.12971E+01_JPRB, 0.11214E+01_JPRB, 0.91320E+00_JPRB, 0.86116E+00_JPRB, 0.12417E+01_JPRB, 0.13091E+01_JPRB ,&
&  0.12202E+01_JPRB, 0.11992E+01_JPRB, 0.11321E+01_JPRB, 0.87733E+00_JPRB, 0.60150E+00_JPRB, 0.46893E+00_JPRB, 0.51643E+00_JPRB ,&
&  0.52353E+00_JPRB, 0.55049E+00_JPRB, 0.49947E+00_JPRB, 0.42736E+00_JPRB, 0.36937E+00_JPRB, 0.41663E+00_JPRB, 0.41679E+00_JPRB ,&
&  0.41284E+00_JPRB, 0.40937E+00_JPRB, 0.44525E+00_JPRB, 0.43932E+00_JPRB, 0.44114E+00_JPRB, 0.42877E+00_JPRB, 0.50538E+00_JPRB ,&
&  0.51642E+00_JPRB, 0.55832E+00_JPRB, 0.58453E+00_JPRB, 0.57792E+00_JPRB, 0.57190E+00_JPRB, 0.56329E+00_JPRB, 0.55235E+00_JPRB ,&
&  0.54728E+00_JPRB, 0.54579E+00_JPRB, 0.53904E+00_JPRB, 0.53379E+00_JPRB, 0.52493E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.49573E+00_JPRB, 0.48824E+00_JPRB, 0.48362E+00_JPRB, 0.48347E+00_JPRB, 0.48999E+00_JPRB, 0.50055E+00_JPRB, 0.50183E+00_JPRB ,&
&  0.49374E+00_JPRB, 0.48009E+00_JPRB, 0.46840E+00_JPRB, 0.45895E+00_JPRB, 0.45209E+00_JPRB, 0.45000E+00_JPRB, 0.45365E+00_JPRB ,&
&  0.48307E+00_JPRB, 0.46007E+00_JPRB, 0.34758E+00_JPRB, 0.37523E+00_JPRB, 0.47122E+00_JPRB, 0.47250E+00_JPRB, 0.59356E+00_JPRB ,&
&  0.74176E+00_JPRB, 0.92190E+00_JPRB, 0.11556E+01_JPRB, 0.17545E+01_JPRB, 0.21804E+01_JPRB, 0.24792E+01_JPRB, 0.25534E+01_JPRB ,&
&  0.24133E+01_JPRB, 0.22237E+01_JPRB, 0.19609E+01_JPRB, 0.16844E+01_JPRB, 0.13935E+01_JPRB, 0.12670E+01_JPRB, 0.11765E+01_JPRB ,&
&  0.10981E+01_JPRB, 0.10010E+01_JPRB, 0.94532E+00_JPRB, 0.94888E+00_JPRB, 0.97000E+00_JPRB, 0.10093E+01_JPRB, 0.10878E+01_JPRB ,&
&  0.11312E+01_JPRB, 0.11116E+01_JPRB, 0.10006E+01_JPRB, 0.94243E+00_JPRB, 0.76503E+00_JPRB, 0.73986E+00_JPRB, 0.80687E+00_JPRB ,&
&  0.11994E+01_JPRB, 0.13033E+01_JPRB, 0.13176E+01_JPRB, 0.15990E+01_JPRB, 0.14568E+01_JPRB, 0.14849E+01_JPRB, 0.17658E+01_JPRB ,&
&  0.19585E+01_JPRB, 0.19760E+01_JPRB, 0.19848E+01_JPRB, 0.14566E+01_JPRB, 0.10542E+01_JPRB, 0.14484E+01_JPRB, 0.13964E+01_JPRB ,&
&  0.12867E+01_JPRB, 0.11746E+01_JPRB, 0.10128E+01_JPRB, 0.91575E+00_JPRB, 0.64840E+00_JPRB, 0.50081E+00_JPRB, 0.42051E+00_JPRB ,&
&  0.41137E+00_JPRB, 0.40028E+00_JPRB, 0.43035E+00_JPRB, 0.41311E+00_JPRB, 0.40984E+00_JPRB, 0.39701E+00_JPRB, 0.38229E+00_JPRB ,&
&  0.37961E+00_JPRB, 0.37027E+00_JPRB, 0.37352E+00_JPRB, 0.39411E+00_JPRB, 0.41080E+00_JPRB, 0.38761E+00_JPRB, 0.42817E+00_JPRB ,&
&  0.47032E+00_JPRB, 0.50195E+00_JPRB, 0.51734E+00_JPRB, 0.53465E+00_JPRB, 0.53611E+00_JPRB, 0.54246E+00_JPRB, 0.54209E+00_JPRB ,&
&  0.53417E+00_JPRB, 0.52702E+00_JPRB, 0.52016E+00_JPRB, 0.51146E+00_JPRB, 0.50376E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.44625E+00_JPRB, 0.44767E+00_JPRB, 0.45211E+00_JPRB, 0.45694E+00_JPRB, 0.46323E+00_JPRB, 0.47382E+00_JPRB, 0.47796E+00_JPRB ,&
&  0.47734E+00_JPRB, 0.46823E+00_JPRB, 0.45667E+00_JPRB, 0.44874E+00_JPRB, 0.44650E+00_JPRB, 0.44806E+00_JPRB, 0.45471E+00_JPRB ,&
&  0.47256E+00_JPRB, 0.50439E+00_JPRB, 0.39527E+00_JPRB, 0.35201E+00_JPRB, 0.35750E+00_JPRB, 0.44846E+00_JPRB, 0.59267E+00_JPRB ,&
&  0.71758E+00_JPRB, 0.76643E+00_JPRB, 0.97260E+00_JPRB, 0.13495E+01_JPRB, 0.16312E+01_JPRB, 0.17685E+01_JPRB, 0.18949E+01_JPRB ,&
&  0.21500E+01_JPRB, 0.17103E+01_JPRB, 0.16492E+01_JPRB, 0.14728E+01_JPRB, 0.13516E+01_JPRB, 0.12623E+01_JPRB, 0.11653E+01_JPRB ,&
&  0.10732E+01_JPRB, 0.10156E+01_JPRB, 0.93638E+00_JPRB, 0.89670E+00_JPRB, 0.87907E+00_JPRB, 0.88209E+00_JPRB, 0.92658E+00_JPRB ,&
&  0.98618E+00_JPRB, 0.10313E+01_JPRB, 0.10181E+01_JPRB, 0.98917E+00_JPRB, 0.95717E+00_JPRB, 0.94263E+00_JPRB, 0.10149E+01_JPRB ,&
&  0.10996E+01_JPRB, 0.10474E+01_JPRB, 0.13616E+01_JPRB, 0.15808E+01_JPRB, 0.18210E+01_JPRB, 0.19207E+01_JPRB, 0.19778E+01_JPRB ,&
&  0.20902E+01_JPRB, 0.20404E+01_JPRB, 0.19921E+01_JPRB, 0.17044E+01_JPRB, 0.17342E+01_JPRB, 0.16483E+01_JPRB, 0.14597E+01_JPRB ,&
&  0.12598E+01_JPRB, 0.11216E+01_JPRB, 0.93937E+00_JPRB, 0.80743E+00_JPRB, 0.72914E+00_JPRB, 0.65595E+00_JPRB, 0.50937E+00_JPRB ,&
&  0.43217E+00_JPRB, 0.45124E+00_JPRB, 0.42873E+00_JPRB, 0.40408E+00_JPRB, 0.37203E+00_JPRB, 0.37558E+00_JPRB, 0.37164E+00_JPRB ,&
&  0.36865E+00_JPRB, 0.36116E+00_JPRB, 0.35247E+00_JPRB, 0.36990E+00_JPRB, 0.39653E+00_JPRB, 0.39023E+00_JPRB, 0.39918E+00_JPRB ,&
&  0.38616E+00_JPRB, 0.42167E+00_JPRB, 0.43534E+00_JPRB, 0.43974E+00_JPRB, 0.49039E+00_JPRB, 0.51189E+00_JPRB, 0.50890E+00_JPRB ,&
&  0.49581E+00_JPRB, 0.48380E+00_JPRB, 0.47097E+00_JPRB, 0.45799E+00_JPRB, 0.44968E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.43534E+00_JPRB, 0.43702E+00_JPRB, 0.44055E+00_JPRB, 0.44578E+00_JPRB, 0.45121E+00_JPRB, 0.46340E+00_JPRB, 0.47168E+00_JPRB ,&
&  0.47330E+00_JPRB, 0.47153E+00_JPRB, 0.46359E+00_JPRB, 0.45499E+00_JPRB, 0.45020E+00_JPRB, 0.44634E+00_JPRB, 0.45207E+00_JPRB ,&
&  0.48037E+00_JPRB, 0.65561E+00_JPRB, 0.47007E+00_JPRB, 0.42869E+00_JPRB, 0.40138E+00_JPRB, 0.50289E+00_JPRB, 0.59125E+00_JPRB ,&
&  0.67073E+00_JPRB, 0.66562E+00_JPRB, 0.70440E+00_JPRB, 0.96290E+00_JPRB, 0.12322E+01_JPRB, 0.12512E+01_JPRB, 0.13964E+01_JPRB ,&
&  0.14504E+01_JPRB, 0.14279E+01_JPRB, 0.12927E+01_JPRB, 0.13328E+01_JPRB, 0.11865E+01_JPRB, 0.10272E+01_JPRB, 0.10437E+01_JPRB ,&
&  0.97805E+00_JPRB, 0.94783E+00_JPRB, 0.89989E+00_JPRB, 0.83936E+00_JPRB, 0.80928E+00_JPRB, 0.78859E+00_JPRB, 0.79223E+00_JPRB ,&
&  0.81614E+00_JPRB, 0.84942E+00_JPRB, 0.89752E+00_JPRB, 0.93976E+00_JPRB, 0.96582E+00_JPRB, 0.10477E+01_JPRB, 0.11110E+01_JPRB ,&
&  0.14206E+01_JPRB, 0.14952E+01_JPRB, 0.12865E+01_JPRB, 0.17034E+01_JPRB, 0.23858E+01_JPRB, 0.23561E+01_JPRB, 0.21618E+01_JPRB ,&
&  0.22241E+01_JPRB, 0.21308E+01_JPRB, 0.20144E+01_JPRB, 0.19620E+01_JPRB, 0.18011E+01_JPRB, 0.16161E+01_JPRB, 0.13860E+01_JPRB ,&
&  0.11467E+01_JPRB, 0.99400E+00_JPRB, 0.88413E+00_JPRB, 0.75844E+00_JPRB, 0.65297E+00_JPRB, 0.58659E+00_JPRB, 0.54157E+00_JPRB ,&
&  0.48100E+00_JPRB, 0.43131E+00_JPRB, 0.36592E+00_JPRB, 0.37322E+00_JPRB, 0.32343E+00_JPRB, 0.32909E+00_JPRB, 0.34397E+00_JPRB ,&
&  0.33982E+00_JPRB, 0.35086E+00_JPRB, 0.35262E+00_JPRB, 0.34575E+00_JPRB, 0.37444E+00_JPRB, 0.38278E+00_JPRB, 0.38846E+00_JPRB ,&
&  0.37699E+00_JPRB, 0.36486E+00_JPRB, 0.38647E+00_JPRB, 0.39232E+00_JPRB, 0.41125E+00_JPRB, 0.42176E+00_JPRB, 0.43061E+00_JPRB ,&
&  0.43536E+00_JPRB, 0.43506E+00_JPRB, 0.43311E+00_JPRB, 0.43249E+00_JPRB, 0.43397E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.40661E+00_JPRB, 0.40647E+00_JPRB, 0.40517E+00_JPRB, 0.41223E+00_JPRB, 0.42570E+00_JPRB, 0.44139E+00_JPRB, 0.45417E+00_JPRB ,&
&  0.45676E+00_JPRB, 0.45849E+00_JPRB, 0.45457E+00_JPRB, 0.44751E+00_JPRB, 0.44251E+00_JPRB, 0.43282E+00_JPRB, 0.44197E+00_JPRB ,&
&  0.49994E+00_JPRB, 0.57139E+00_JPRB, 0.44428E+00_JPRB, 0.37588E+00_JPRB, 0.44832E+00_JPRB, 0.49092E+00_JPRB, 0.55481E+00_JPRB ,&
&  0.56966E+00_JPRB, 0.59684E+00_JPRB, 0.59648E+00_JPRB, 0.65625E+00_JPRB, 0.86123E+00_JPRB, 0.97055E+00_JPRB, 0.10183E+01_JPRB ,&
&  0.10428E+01_JPRB, 0.10004E+01_JPRB, 0.10194E+01_JPRB, 0.10614E+01_JPRB, 0.10952E+01_JPRB, 0.97031E+00_JPRB, 0.93980E+00_JPRB ,&
&  0.88126E+00_JPRB, 0.83816E+00_JPRB, 0.78696E+00_JPRB, 0.76111E+00_JPRB, 0.74871E+00_JPRB, 0.74507E+00_JPRB, 0.75650E+00_JPRB ,&
&  0.77271E+00_JPRB, 0.78977E+00_JPRB, 0.81059E+00_JPRB, 0.86247E+00_JPRB, 0.92732E+00_JPRB, 0.11536E+01_JPRB, 0.15567E+01_JPRB ,&
&  0.18862E+01_JPRB, 0.21367E+01_JPRB, 0.23596E+01_JPRB, 0.26887E+01_JPRB, 0.27104E+01_JPRB, 0.26328E+01_JPRB, 0.25085E+01_JPRB ,&
&  0.23344E+01_JPRB, 0.21622E+01_JPRB, 0.20142E+01_JPRB, 0.18624E+01_JPRB, 0.17013E+01_JPRB, 0.15166E+01_JPRB, 0.12855E+01_JPRB ,&
&  0.10423E+01_JPRB, 0.90279E+00_JPRB, 0.84983E+00_JPRB, 0.74214E+00_JPRB, 0.65152E+00_JPRB, 0.57566E+00_JPRB, 0.54730E+00_JPRB ,&
&  0.48735E+00_JPRB, 0.35288E+00_JPRB, 0.32531E+00_JPRB, 0.35277E+00_JPRB, 0.32001E+00_JPRB, 0.33082E+00_JPRB, 0.34282E+00_JPRB ,&
&  0.33524E+00_JPRB, 0.33123E+00_JPRB, 0.33589E+00_JPRB, 0.32957E+00_JPRB, 0.34088E+00_JPRB, 0.36932E+00_JPRB, 0.35479E+00_JPRB ,&
&  0.34098E+00_JPRB, 0.34293E+00_JPRB, 0.35824E+00_JPRB, 0.36530E+00_JPRB, 0.37140E+00_JPRB, 0.37476E+00_JPRB, 0.37313E+00_JPRB ,&
&  0.37602E+00_JPRB, 0.38477E+00_JPRB, 0.39112E+00_JPRB, 0.39826E+00_JPRB, 0.40369E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.38893E+00_JPRB, 0.39111E+00_JPRB, 0.38796E+00_JPRB, 0.39496E+00_JPRB, 0.41373E+00_JPRB, 0.43271E+00_JPRB, 0.44602E+00_JPRB ,&
&  0.45055E+00_JPRB, 0.45407E+00_JPRB, 0.45023E+00_JPRB, 0.44499E+00_JPRB, 0.43875E+00_JPRB, 0.42391E+00_JPRB, 0.44163E+00_JPRB ,&
&  0.36905E+00_JPRB, 0.41817E+00_JPRB, 0.35434E+00_JPRB, 0.35977E+00_JPRB, 0.46251E+00_JPRB, 0.45503E+00_JPRB, 0.48369E+00_JPRB ,&
&  0.50018E+00_JPRB, 0.51869E+00_JPRB, 0.52109E+00_JPRB, 0.55427E+00_JPRB, 0.67380E+00_JPRB, 0.80076E+00_JPRB, 0.85177E+00_JPRB ,&
&  0.81893E+00_JPRB, 0.77824E+00_JPRB, 0.81465E+00_JPRB, 0.84139E+00_JPRB, 0.85090E+00_JPRB, 0.88709E+00_JPRB, 0.89528E+00_JPRB ,&
&  0.83651E+00_JPRB, 0.78929E+00_JPRB, 0.73333E+00_JPRB, 0.70830E+00_JPRB, 0.69634E+00_JPRB, 0.68955E+00_JPRB, 0.68013E+00_JPRB ,&
&  0.68258E+00_JPRB, 0.70038E+00_JPRB, 0.72600E+00_JPRB, 0.80586E+00_JPRB, 0.89298E+00_JPRB, 0.14830E+01_JPRB, 0.24251E+01_JPRB ,&
&  0.26366E+01_JPRB, 0.26169E+01_JPRB, 0.27834E+01_JPRB, 0.30871E+01_JPRB, 0.29757E+01_JPRB, 0.27767E+01_JPRB, 0.24834E+01_JPRB ,&
&  0.22770E+01_JPRB, 0.21105E+01_JPRB, 0.19278E+01_JPRB, 0.17500E+01_JPRB, 0.15063E+01_JPRB, 0.13338E+01_JPRB, 0.11367E+01_JPRB ,&
&  0.91315E+00_JPRB, 0.80182E+00_JPRB, 0.77958E+00_JPRB, 0.74335E+00_JPRB, 0.69951E+00_JPRB, 0.66903E+00_JPRB, 0.61746E+00_JPRB ,&
&  0.55154E+00_JPRB, 0.46404E+00_JPRB, 0.38166E+00_JPRB, 0.36536E+00_JPRB, 0.33514E+00_JPRB, 0.34073E+00_JPRB, 0.35835E+00_JPRB ,&
&  0.33568E+00_JPRB, 0.31713E+00_JPRB, 0.31268E+00_JPRB, 0.31192E+00_JPRB, 0.31759E+00_JPRB, 0.33888E+00_JPRB, 0.32981E+00_JPRB ,&
&  0.32008E+00_JPRB, 0.33452E+00_JPRB, 0.33621E+00_JPRB, 0.33875E+00_JPRB, 0.33987E+00_JPRB, 0.34285E+00_JPRB, 0.32571E+00_JPRB ,&
&  0.33608E+00_JPRB, 0.34416E+00_JPRB, 0.35722E+00_JPRB, 0.37288E+00_JPRB, 0.38275E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.38291E+00_JPRB, 0.38416E+00_JPRB, 0.38152E+00_JPRB, 0.39006E+00_JPRB, 0.40954E+00_JPRB, 0.42716E+00_JPRB, 0.43986E+00_JPRB ,&
&  0.44381E+00_JPRB, 0.44371E+00_JPRB, 0.44322E+00_JPRB, 0.44164E+00_JPRB, 0.43114E+00_JPRB, 0.41730E+00_JPRB, 0.37633E+00_JPRB ,&
&  0.31957E+00_JPRB, 0.35522E+00_JPRB, 0.33742E+00_JPRB, 0.35164E+00_JPRB, 0.39374E+00_JPRB, 0.38512E+00_JPRB, 0.38742E+00_JPRB ,&
&  0.41401E+00_JPRB, 0.42583E+00_JPRB, 0.45523E+00_JPRB, 0.48584E+00_JPRB, 0.53870E+00_JPRB, 0.64825E+00_JPRB, 0.69562E+00_JPRB ,&
&  0.66855E+00_JPRB, 0.64826E+00_JPRB, 0.68149E+00_JPRB, 0.74121E+00_JPRB, 0.80232E+00_JPRB, 0.85317E+00_JPRB, 0.84564E+00_JPRB ,&
&  0.81370E+00_JPRB, 0.77005E+00_JPRB, 0.70190E+00_JPRB, 0.65622E+00_JPRB, 0.63003E+00_JPRB, 0.61443E+00_JPRB, 0.60440E+00_JPRB ,&
&  0.59982E+00_JPRB, 0.61065E+00_JPRB, 0.64355E+00_JPRB, 0.70342E+00_JPRB, 0.85563E+00_JPRB, 0.13789E+01_JPRB, 0.18520E+01_JPRB ,&
&  0.18789E+01_JPRB, 0.19476E+01_JPRB, 0.20919E+01_JPRB, 0.22179E+01_JPRB, 0.23357E+01_JPRB, 0.22525E+01_JPRB, 0.21390E+01_JPRB ,&
&  0.20262E+01_JPRB, 0.19031E+01_JPRB, 0.17722E+01_JPRB, 0.15621E+01_JPRB, 0.13394E+01_JPRB, 0.11624E+01_JPRB, 0.98118E+00_JPRB ,&
&  0.83548E+00_JPRB, 0.71575E+00_JPRB, 0.69998E+00_JPRB, 0.68579E+00_JPRB, 0.66929E+00_JPRB, 0.64609E+00_JPRB, 0.60970E+00_JPRB ,&
&  0.56427E+00_JPRB, 0.50804E+00_JPRB, 0.43723E+00_JPRB, 0.39822E+00_JPRB, 0.37413E+00_JPRB, 0.37399E+00_JPRB, 0.36023E+00_JPRB ,&
&  0.34109E+00_JPRB, 0.31766E+00_JPRB, 0.31742E+00_JPRB, 0.31687E+00_JPRB, 0.31434E+00_JPRB, 0.31354E+00_JPRB, 0.30573E+00_JPRB ,&
&  0.30792E+00_JPRB, 0.31498E+00_JPRB, 0.30356E+00_JPRB, 0.30329E+00_JPRB, 0.30442E+00_JPRB, 0.30462E+00_JPRB, 0.28561E+00_JPRB ,&
&  0.29197E+00_JPRB, 0.31551E+00_JPRB, 0.33527E+00_JPRB, 0.35683E+00_JPRB, 0.37293E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.38922E+00_JPRB, 0.39323E+00_JPRB, 0.39796E+00_JPRB, 0.40020E+00_JPRB, 0.40710E+00_JPRB, 0.42548E+00_JPRB, 0.41640E+00_JPRB ,&
&  0.44697E+00_JPRB, 0.44957E+00_JPRB, 0.44251E+00_JPRB, 0.43980E+00_JPRB, 0.42318E+00_JPRB, 0.36123E+00_JPRB, 0.28754E+00_JPRB ,&
&  0.27196E+00_JPRB, 0.29110E+00_JPRB, 0.31551E+00_JPRB, 0.33002E+00_JPRB, 0.34897E+00_JPRB, 0.34983E+00_JPRB, 0.35140E+00_JPRB ,&
&  0.35808E+00_JPRB, 0.36496E+00_JPRB, 0.38906E+00_JPRB, 0.42751E+00_JPRB, 0.46624E+00_JPRB, 0.52363E+00_JPRB, 0.58579E+00_JPRB ,&
&  0.55437E+00_JPRB, 0.55738E+00_JPRB, 0.60355E+00_JPRB, 0.59411E+00_JPRB, 0.68783E+00_JPRB, 0.71598E+00_JPRB, 0.72270E+00_JPRB ,&
&  0.71088E+00_JPRB, 0.68670E+00_JPRB, 0.64788E+00_JPRB, 0.62221E+00_JPRB, 0.59466E+00_JPRB, 0.57139E+00_JPRB, 0.55680E+00_JPRB ,&
&  0.55871E+00_JPRB, 0.56856E+00_JPRB, 0.59627E+00_JPRB, 0.64504E+00_JPRB, 0.71331E+00_JPRB, 0.10185E+01_JPRB, 0.12695E+01_JPRB ,&
&  0.14074E+01_JPRB, 0.16353E+01_JPRB, 0.18603E+01_JPRB, 0.17757E+01_JPRB, 0.18962E+01_JPRB, 0.19479E+01_JPRB, 0.18685E+01_JPRB ,&
&  0.18548E+01_JPRB, 0.17188E+01_JPRB, 0.16118E+01_JPRB, 0.14364E+01_JPRB, 0.12500E+01_JPRB, 0.10752E+01_JPRB, 0.91762E+00_JPRB ,&
&  0.79673E+00_JPRB, 0.69332E+00_JPRB, 0.66265E+00_JPRB, 0.63421E+00_JPRB, 0.60121E+00_JPRB, 0.57460E+00_JPRB, 0.54557E+00_JPRB ,&
&  0.51992E+00_JPRB, 0.49209E+00_JPRB, 0.45809E+00_JPRB, 0.42676E+00_JPRB, 0.40278E+00_JPRB, 0.38212E+00_JPRB, 0.36898E+00_JPRB ,&
&  0.35497E+00_JPRB, 0.33122E+00_JPRB, 0.31853E+00_JPRB, 0.31740E+00_JPRB, 0.31040E+00_JPRB, 0.29830E+00_JPRB, 0.29418E+00_JPRB ,&
&  0.29079E+00_JPRB, 0.28383E+00_JPRB, 0.28028E+00_JPRB, 0.27926E+00_JPRB, 0.28357E+00_JPRB, 0.29046E+00_JPRB, 0.29043E+00_JPRB ,&
&  0.29397E+00_JPRB, 0.32273E+00_JPRB, 0.34352E+00_JPRB, 0.36247E+00_JPRB, 0.37663E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.38931E+00_JPRB, 0.40462E+00_JPRB, 0.41456E+00_JPRB, 0.41273E+00_JPRB, 0.41018E+00_JPRB, 0.40095E+00_JPRB, 0.41014E+00_JPRB ,&
&  0.45221E+00_JPRB, 0.41202E+00_JPRB, 0.44649E+00_JPRB, 0.35052E+00_JPRB, 0.30769E+00_JPRB, 0.28942E+00_JPRB, 0.27423E+00_JPRB ,&
&  0.27365E+00_JPRB, 0.28731E+00_JPRB, 0.30001E+00_JPRB, 0.30939E+00_JPRB, 0.32870E+00_JPRB, 0.33125E+00_JPRB, 0.33214E+00_JPRB ,&
&  0.33945E+00_JPRB, 0.34641E+00_JPRB, 0.37112E+00_JPRB, 0.37939E+00_JPRB, 0.40192E+00_JPRB, 0.43747E+00_JPRB, 0.49107E+00_JPRB ,&
&  0.46931E+00_JPRB, 0.47571E+00_JPRB, 0.52567E+00_JPRB, 0.53945E+00_JPRB, 0.55929E+00_JPRB, 0.56057E+00_JPRB, 0.55306E+00_JPRB ,&
&  0.54620E+00_JPRB, 0.46344E+00_JPRB, 0.48992E+00_JPRB, 0.48168E+00_JPRB, 0.47214E+00_JPRB, 0.45986E+00_JPRB, 0.44097E+00_JPRB ,&
&  0.43034E+00_JPRB, 0.43641E+00_JPRB, 0.45779E+00_JPRB, 0.48919E+00_JPRB, 0.54198E+00_JPRB, 0.68126E+00_JPRB, 0.84840E+00_JPRB ,&
&  0.10620E+01_JPRB, 0.10116E+01_JPRB, 0.12900E+01_JPRB, 0.13869E+01_JPRB, 0.15120E+01_JPRB, 0.16278E+01_JPRB, 0.16803E+01_JPRB ,&
&  0.16578E+01_JPRB, 0.14809E+01_JPRB, 0.13401E+01_JPRB, 0.11901E+01_JPRB, 0.10622E+01_JPRB, 0.94872E+00_JPRB, 0.86384E+00_JPRB ,&
&  0.78217E+00_JPRB, 0.66784E+00_JPRB, 0.63687E+00_JPRB, 0.58756E+00_JPRB, 0.54628E+00_JPRB, 0.51984E+00_JPRB, 0.49798E+00_JPRB ,&
&  0.48013E+00_JPRB, 0.46466E+00_JPRB, 0.44253E+00_JPRB, 0.41110E+00_JPRB, 0.40437E+00_JPRB, 0.39592E+00_JPRB, 0.37939E+00_JPRB ,&
&  0.36902E+00_JPRB, 0.35248E+00_JPRB, 0.33886E+00_JPRB, 0.33754E+00_JPRB, 0.32329E+00_JPRB, 0.31231E+00_JPRB, 0.30535E+00_JPRB ,&
&  0.29296E+00_JPRB, 0.27152E+00_JPRB, 0.26038E+00_JPRB, 0.26291E+00_JPRB, 0.26587E+00_JPRB, 0.27443E+00_JPRB, 0.29512E+00_JPRB ,&
&  0.30247E+00_JPRB, 0.31845E+00_JPRB, 0.33971E+00_JPRB, 0.35353E+00_JPRB, 0.37238E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.38147E+00_JPRB, 0.40292E+00_JPRB, 0.41975E+00_JPRB, 0.41696E+00_JPRB, 0.40702E+00_JPRB, 0.39473E+00_JPRB, 0.39128E+00_JPRB ,&
&  0.40301E+00_JPRB, 0.34636E+00_JPRB, 0.31853E+00_JPRB, 0.30327E+00_JPRB, 0.29559E+00_JPRB, 0.28206E+00_JPRB, 0.26886E+00_JPRB ,&
&  0.26736E+00_JPRB, 0.27513E+00_JPRB, 0.29072E+00_JPRB, 0.30243E+00_JPRB, 0.31211E+00_JPRB, 0.31854E+00_JPRB, 0.32331E+00_JPRB ,&
&  0.33044E+00_JPRB, 0.33828E+00_JPRB, 0.34295E+00_JPRB, 0.34801E+00_JPRB, 0.35412E+00_JPRB, 0.36717E+00_JPRB, 0.38525E+00_JPRB ,&
&  0.39974E+00_JPRB, 0.40735E+00_JPRB, 0.39413E+00_JPRB, 0.39554E+00_JPRB, 0.40196E+00_JPRB, 0.40681E+00_JPRB, 0.42081E+00_JPRB ,&
&  0.30472E+00_JPRB, 0.29748E+00_JPRB, 0.35384E+00_JPRB, 0.35581E+00_JPRB, 0.36313E+00_JPRB, 0.36395E+00_JPRB, 0.36428E+00_JPRB ,&
&  0.37082E+00_JPRB, 0.38786E+00_JPRB, 0.42091E+00_JPRB, 0.47703E+00_JPRB, 0.54035E+00_JPRB, 0.61713E+00_JPRB, 0.72271E+00_JPRB ,&
&  0.92387E+00_JPRB, 0.90087E+00_JPRB, 0.93937E+00_JPRB, 0.10474E+01_JPRB, 0.12522E+01_JPRB, 0.13519E+01_JPRB, 0.13641E+01_JPRB ,&
&  0.13180E+01_JPRB, 0.12104E+01_JPRB, 0.11211E+01_JPRB, 0.10546E+01_JPRB, 0.98054E+00_JPRB, 0.89586E+00_JPRB, 0.81498E+00_JPRB ,&
&  0.72963E+00_JPRB, 0.64632E+00_JPRB, 0.59255E+00_JPRB, 0.55417E+00_JPRB, 0.51309E+00_JPRB, 0.48558E+00_JPRB, 0.46778E+00_JPRB ,&
&  0.45197E+00_JPRB, 0.43567E+00_JPRB, 0.41609E+00_JPRB, 0.39864E+00_JPRB, 0.38599E+00_JPRB, 0.38184E+00_JPRB, 0.37459E+00_JPRB ,&
&  0.36476E+00_JPRB, 0.35776E+00_JPRB, 0.35241E+00_JPRB, 0.34986E+00_JPRB, 0.33509E+00_JPRB, 0.31690E+00_JPRB, 0.30342E+00_JPRB ,&
&  0.28742E+00_JPRB, 0.26597E+00_JPRB, 0.24688E+00_JPRB, 0.24284E+00_JPRB, 0.24901E+00_JPRB, 0.25832E+00_JPRB, 0.26919E+00_JPRB ,&
&  0.28214E+00_JPRB, 0.29675E+00_JPRB, 0.30767E+00_JPRB, 0.31813E+00_JPRB, 0.34334E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.36503E+00_JPRB, 0.38020E+00_JPRB, 0.39304E+00_JPRB, 0.40133E+00_JPRB, 0.39294E+00_JPRB, 0.38870E+00_JPRB, 0.38622E+00_JPRB ,&
&  0.39340E+00_JPRB, 0.35684E+00_JPRB, 0.33233E+00_JPRB, 0.31228E+00_JPRB, 0.29903E+00_JPRB, 0.28578E+00_JPRB, 0.28197E+00_JPRB ,&
&  0.28470E+00_JPRB, 0.29040E+00_JPRB, 0.29661E+00_JPRB, 0.30188E+00_JPRB, 0.30651E+00_JPRB, 0.30867E+00_JPRB, 0.31287E+00_JPRB ,&
&  0.31946E+00_JPRB, 0.32329E+00_JPRB, 0.32050E+00_JPRB, 0.31922E+00_JPRB, 0.32144E+00_JPRB, 0.32887E+00_JPRB, 0.33151E+00_JPRB ,&
&  0.33101E+00_JPRB, 0.32697E+00_JPRB, 0.32517E+00_JPRB, 0.33260E+00_JPRB, 0.33536E+00_JPRB, 0.33384E+00_JPRB, 0.30492E+00_JPRB ,&
&  0.25811E+00_JPRB, 0.24177E+00_JPRB, 0.25297E+00_JPRB, 0.30788E+00_JPRB, 0.32357E+00_JPRB, 0.33319E+00_JPRB, 0.34789E+00_JPRB ,&
&  0.34135E+00_JPRB, 0.34573E+00_JPRB, 0.38024E+00_JPRB, 0.46693E+00_JPRB, 0.52888E+00_JPRB, 0.59493E+00_JPRB, 0.67134E+00_JPRB ,&
&  0.79179E+00_JPRB, 0.88730E+00_JPRB, 0.97839E+00_JPRB, 0.92004E+00_JPRB, 0.10062E+01_JPRB, 0.12343E+01_JPRB, 0.11498E+01_JPRB ,&
&  0.10562E+01_JPRB, 0.99404E+00_JPRB, 0.99406E+00_JPRB, 0.92485E+00_JPRB, 0.87314E+00_JPRB, 0.79387E+00_JPRB, 0.71747E+00_JPRB ,&
&  0.64816E+00_JPRB, 0.58451E+00_JPRB, 0.54086E+00_JPRB, 0.51874E+00_JPRB, 0.49326E+00_JPRB, 0.46897E+00_JPRB, 0.45244E+00_JPRB ,&
&  0.43200E+00_JPRB, 0.40952E+00_JPRB, 0.38897E+00_JPRB, 0.37207E+00_JPRB, 0.36233E+00_JPRB, 0.35787E+00_JPRB, 0.35313E+00_JPRB ,&
&  0.34771E+00_JPRB, 0.34694E+00_JPRB, 0.35026E+00_JPRB, 0.35117E+00_JPRB, 0.32873E+00_JPRB, 0.30306E+00_JPRB, 0.28774E+00_JPRB ,&
&  0.27528E+00_JPRB, 0.26432E+00_JPRB, 0.25732E+00_JPRB, 0.25779E+00_JPRB, 0.26165E+00_JPRB, 0.26517E+00_JPRB, 0.26741E+00_JPRB ,&
&  0.27246E+00_JPRB, 0.27880E+00_JPRB, 0.29227E+00_JPRB, 0.31460E+00_JPRB, 0.34015E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.32275E+00_JPRB, 0.34196E+00_JPRB, 0.36630E+00_JPRB, 0.38130E+00_JPRB, 0.38933E+00_JPRB, 0.37515E+00_JPRB, 0.36065E+00_JPRB ,&
&  0.34585E+00_JPRB, 0.32639E+00_JPRB, 0.31700E+00_JPRB, 0.31145E+00_JPRB, 0.30906E+00_JPRB, 0.30836E+00_JPRB, 0.30893E+00_JPRB ,&
&  0.30968E+00_JPRB, 0.30918E+00_JPRB, 0.31006E+00_JPRB, 0.31419E+00_JPRB, 0.31852E+00_JPRB, 0.31876E+00_JPRB, 0.31725E+00_JPRB ,&
&  0.31660E+00_JPRB, 0.31591E+00_JPRB, 0.31155E+00_JPRB, 0.30450E+00_JPRB, 0.30363E+00_JPRB, 0.31258E+00_JPRB, 0.31666E+00_JPRB ,&
&  0.31604E+00_JPRB, 0.30864E+00_JPRB, 0.30843E+00_JPRB, 0.31512E+00_JPRB, 0.31338E+00_JPRB, 0.30639E+00_JPRB, 0.28015E+00_JPRB ,&
&  0.24880E+00_JPRB, 0.22395E+00_JPRB, 0.21528E+00_JPRB, 0.22692E+00_JPRB, 0.25337E+00_JPRB, 0.29879E+00_JPRB, 0.31318E+00_JPRB ,&
&  0.32451E+00_JPRB, 0.34475E+00_JPRB, 0.37643E+00_JPRB, 0.42390E+00_JPRB, 0.48697E+00_JPRB, 0.55075E+00_JPRB, 0.60678E+00_JPRB ,&
&  0.67407E+00_JPRB, 0.74239E+00_JPRB, 0.83712E+00_JPRB, 0.78981E+00_JPRB, 0.80842E+00_JPRB, 0.85854E+00_JPRB, 0.85223E+00_JPRB ,&
&  0.80929E+00_JPRB, 0.77975E+00_JPRB, 0.78866E+00_JPRB, 0.75978E+00_JPRB, 0.71882E+00_JPRB, 0.66655E+00_JPRB, 0.60492E+00_JPRB ,&
&  0.55315E+00_JPRB, 0.51170E+00_JPRB, 0.48207E+00_JPRB, 0.47004E+00_JPRB, 0.46240E+00_JPRB, 0.44384E+00_JPRB, 0.42448E+00_JPRB ,&
&  0.40430E+00_JPRB, 0.37815E+00_JPRB, 0.35153E+00_JPRB, 0.33577E+00_JPRB, 0.33119E+00_JPRB, 0.33313E+00_JPRB, 0.33182E+00_JPRB ,&
&  0.33187E+00_JPRB, 0.33283E+00_JPRB, 0.33423E+00_JPRB, 0.33217E+00_JPRB, 0.31667E+00_JPRB, 0.30064E+00_JPRB, 0.29026E+00_JPRB ,&
&  0.28686E+00_JPRB, 0.28390E+00_JPRB, 0.28480E+00_JPRB, 0.28790E+00_JPRB, 0.28788E+00_JPRB, 0.28967E+00_JPRB, 0.29250E+00_JPRB ,&
&  0.29398E+00_JPRB, 0.29443E+00_JPRB, 0.29888E+00_JPRB, 0.30313E+00_JPRB, 0.31037E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.33754E+00_JPRB, 0.34436E+00_JPRB, 0.35077E+00_JPRB, 0.36096E+00_JPRB, 0.37021E+00_JPRB, 0.36890E+00_JPRB, 0.36278E+00_JPRB ,&
&  0.35059E+00_JPRB, 0.33766E+00_JPRB, 0.32905E+00_JPRB, 0.32453E+00_JPRB, 0.32468E+00_JPRB, 0.32672E+00_JPRB, 0.32873E+00_JPRB ,&
&  0.33003E+00_JPRB, 0.33042E+00_JPRB, 0.32785E+00_JPRB, 0.32204E+00_JPRB, 0.31794E+00_JPRB, 0.31695E+00_JPRB, 0.31742E+00_JPRB ,&
&  0.31862E+00_JPRB, 0.31795E+00_JPRB, 0.31551E+00_JPRB, 0.31067E+00_JPRB, 0.30507E+00_JPRB, 0.31023E+00_JPRB, 0.30884E+00_JPRB ,&
&  0.30307E+00_JPRB, 0.30263E+00_JPRB, 0.30829E+00_JPRB, 0.30634E+00_JPRB, 0.30162E+00_JPRB, 0.29270E+00_JPRB, 0.26209E+00_JPRB ,&
&  0.23454E+00_JPRB, 0.21365E+00_JPRB, 0.20217E+00_JPRB, 0.19979E+00_JPRB, 0.20697E+00_JPRB, 0.22480E+00_JPRB, 0.25316E+00_JPRB ,&
&  0.29396E+00_JPRB, 0.31497E+00_JPRB, 0.34173E+00_JPRB, 0.37734E+00_JPRB, 0.42176E+00_JPRB, 0.47208E+00_JPRB, 0.51708E+00_JPRB ,&
&  0.55128E+00_JPRB, 0.59014E+00_JPRB, 0.64302E+00_JPRB, 0.69503E+00_JPRB, 0.75538E+00_JPRB, 0.71505E+00_JPRB, 0.69896E+00_JPRB ,&
&  0.73472E+00_JPRB, 0.69944E+00_JPRB, 0.68055E+00_JPRB, 0.62864E+00_JPRB, 0.57916E+00_JPRB, 0.52161E+00_JPRB, 0.47712E+00_JPRB ,&
&  0.45141E+00_JPRB, 0.43993E+00_JPRB, 0.43668E+00_JPRB, 0.43532E+00_JPRB, 0.42786E+00_JPRB, 0.41624E+00_JPRB, 0.39295E+00_JPRB ,&
&  0.36952E+00_JPRB, 0.35305E+00_JPRB, 0.35981E+00_JPRB, 0.35143E+00_JPRB, 0.34060E+00_JPRB, 0.33546E+00_JPRB, 0.32701E+00_JPRB ,&
&  0.32389E+00_JPRB, 0.32325E+00_JPRB, 0.32020E+00_JPRB, 0.31814E+00_JPRB, 0.31251E+00_JPRB, 0.30052E+00_JPRB, 0.29239E+00_JPRB ,&
&  0.29165E+00_JPRB, 0.29394E+00_JPRB, 0.29668E+00_JPRB, 0.29966E+00_JPRB, 0.30187E+00_JPRB, 0.30535E+00_JPRB, 0.31059E+00_JPRB ,&
&  0.31329E+00_JPRB, 0.31593E+00_JPRB, 0.32079E+00_JPRB, 0.32523E+00_JPRB, 0.33069E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.32897E+00_JPRB, 0.33160E+00_JPRB, 0.33472E+00_JPRB, 0.33688E+00_JPRB, 0.33972E+00_JPRB, 0.34157E+00_JPRB, 0.34074E+00_JPRB ,&
&  0.33703E+00_JPRB, 0.33501E+00_JPRB, 0.33424E+00_JPRB, 0.33651E+00_JPRB, 0.33924E+00_JPRB, 0.34095E+00_JPRB, 0.34157E+00_JPRB ,&
&  0.33892E+00_JPRB, 0.33048E+00_JPRB, 0.32421E+00_JPRB, 0.32136E+00_JPRB, 0.31958E+00_JPRB, 0.31956E+00_JPRB, 0.31969E+00_JPRB ,&
&  0.31791E+00_JPRB, 0.31448E+00_JPRB, 0.31216E+00_JPRB, 0.30596E+00_JPRB, 0.30180E+00_JPRB, 0.30216E+00_JPRB, 0.30668E+00_JPRB ,&
&  0.30874E+00_JPRB, 0.29904E+00_JPRB, 0.29804E+00_JPRB, 0.29486E+00_JPRB, 0.28464E+00_JPRB, 0.27837E+00_JPRB, 0.24068E+00_JPRB ,&
&  0.21745E+00_JPRB, 0.20276E+00_JPRB, 0.19735E+00_JPRB, 0.19508E+00_JPRB, 0.20043E+00_JPRB, 0.21371E+00_JPRB, 0.23968E+00_JPRB ,&
&  0.26853E+00_JPRB, 0.30731E+00_JPRB, 0.32461E+00_JPRB, 0.35090E+00_JPRB, 0.38103E+00_JPRB, 0.41751E+00_JPRB, 0.44991E+00_JPRB ,&
&  0.46288E+00_JPRB, 0.46956E+00_JPRB, 0.47944E+00_JPRB, 0.48413E+00_JPRB, 0.49377E+00_JPRB, 0.49686E+00_JPRB, 0.48776E+00_JPRB ,&
&  0.48925E+00_JPRB, 0.48873E+00_JPRB, 0.49552E+00_JPRB, 0.48361E+00_JPRB, 0.45570E+00_JPRB, 0.42603E+00_JPRB, 0.40569E+00_JPRB ,&
&  0.38479E+00_JPRB, 0.37726E+00_JPRB, 0.38195E+00_JPRB, 0.39198E+00_JPRB, 0.38840E+00_JPRB, 0.38019E+00_JPRB, 0.36735E+00_JPRB ,&
&  0.35115E+00_JPRB, 0.34381E+00_JPRB, 0.34563E+00_JPRB, 0.34080E+00_JPRB, 0.33664E+00_JPRB, 0.33437E+00_JPRB, 0.32870E+00_JPRB ,&
&  0.31967E+00_JPRB, 0.32197E+00_JPRB, 0.32364E+00_JPRB, 0.32193E+00_JPRB, 0.31921E+00_JPRB, 0.31112E+00_JPRB, 0.30747E+00_JPRB ,&
&  0.30739E+00_JPRB, 0.31084E+00_JPRB, 0.31301E+00_JPRB, 0.31495E+00_JPRB, 0.31641E+00_JPRB, 0.31774E+00_JPRB, 0.32170E+00_JPRB ,&
&  0.32294E+00_JPRB, 0.32519E+00_JPRB, 0.32705E+00_JPRB, 0.32790E+00_JPRB, 0.32859E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.31834E+00_JPRB, 0.31967E+00_JPRB, 0.32063E+00_JPRB, 0.32176E+00_JPRB, 0.32359E+00_JPRB, 0.32712E+00_JPRB, 0.33152E+00_JPRB ,&
&  0.33414E+00_JPRB, 0.33675E+00_JPRB, 0.33929E+00_JPRB, 0.33949E+00_JPRB, 0.33838E+00_JPRB, 0.33600E+00_JPRB, 0.33417E+00_JPRB ,&
&  0.33132E+00_JPRB, 0.32828E+00_JPRB, 0.32456E+00_JPRB, 0.32137E+00_JPRB, 0.31853E+00_JPRB, 0.31729E+00_JPRB, 0.31519E+00_JPRB ,&
&  0.31333E+00_JPRB, 0.31031E+00_JPRB, 0.30457E+00_JPRB, 0.29966E+00_JPRB, 0.29819E+00_JPRB, 0.29506E+00_JPRB, 0.29645E+00_JPRB ,&
&  0.29344E+00_JPRB, 0.28739E+00_JPRB, 0.27880E+00_JPRB, 0.27410E+00_JPRB, 0.26840E+00_JPRB, 0.23199E+00_JPRB, 0.22136E+00_JPRB ,&
&  0.20942E+00_JPRB, 0.19976E+00_JPRB, 0.19671E+00_JPRB, 0.19730E+00_JPRB, 0.20154E+00_JPRB, 0.21337E+00_JPRB, 0.22915E+00_JPRB ,&
&  0.24926E+00_JPRB, 0.30964E+00_JPRB, 0.32872E+00_JPRB, 0.35042E+00_JPRB, 0.36774E+00_JPRB, 0.38779E+00_JPRB, 0.39985E+00_JPRB ,&
&  0.40564E+00_JPRB, 0.40767E+00_JPRB, 0.41549E+00_JPRB, 0.42815E+00_JPRB, 0.43853E+00_JPRB, 0.44171E+00_JPRB, 0.44072E+00_JPRB ,&
&  0.42970E+00_JPRB, 0.41576E+00_JPRB, 0.39638E+00_JPRB, 0.37978E+00_JPRB, 0.36821E+00_JPRB, 0.35779E+00_JPRB, 0.35705E+00_JPRB ,&
&  0.35877E+00_JPRB, 0.35530E+00_JPRB, 0.35576E+00_JPRB, 0.35494E+00_JPRB, 0.35035E+00_JPRB, 0.34502E+00_JPRB, 0.33808E+00_JPRB ,&
&  0.32952E+00_JPRB, 0.33027E+00_JPRB, 0.32951E+00_JPRB, 0.33269E+00_JPRB, 0.33077E+00_JPRB, 0.32901E+00_JPRB, 0.32383E+00_JPRB ,&
&  0.32354E+00_JPRB, 0.32603E+00_JPRB, 0.32683E+00_JPRB, 0.32507E+00_JPRB, 0.32269E+00_JPRB, 0.31690E+00_JPRB, 0.31362E+00_JPRB ,&
&  0.31418E+00_JPRB, 0.31720E+00_JPRB, 0.31892E+00_JPRB, 0.32066E+00_JPRB, 0.32077E+00_JPRB, 0.32152E+00_JPRB, 0.32168E+00_JPRB ,&
&  0.32034E+00_JPRB, 0.32063E+00_JPRB, 0.31978E+00_JPRB, 0.31816E+00_JPRB, 0.31856E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.31095E+00_JPRB, 0.31237E+00_JPRB, 0.31309E+00_JPRB, 0.31425E+00_JPRB, 0.31608E+00_JPRB, 0.31958E+00_JPRB, 0.32294E+00_JPRB ,&
&  0.32548E+00_JPRB, 0.32528E+00_JPRB, 0.32552E+00_JPRB, 0.32416E+00_JPRB, 0.32303E+00_JPRB, 0.32125E+00_JPRB, 0.32087E+00_JPRB ,&
&  0.32106E+00_JPRB, 0.32078E+00_JPRB, 0.31965E+00_JPRB, 0.31725E+00_JPRB, 0.31556E+00_JPRB, 0.31308E+00_JPRB, 0.31060E+00_JPRB ,&
&  0.30741E+00_JPRB, 0.30314E+00_JPRB, 0.29331E+00_JPRB, 0.28767E+00_JPRB, 0.27963E+00_JPRB, 0.27607E+00_JPRB, 0.27148E+00_JPRB ,&
&  0.27559E+00_JPRB, 0.25181E+00_JPRB, 0.24541E+00_JPRB, 0.23339E+00_JPRB, 0.22424E+00_JPRB, 0.21734E+00_JPRB, 0.20989E+00_JPRB ,&
&  0.20652E+00_JPRB, 0.20355E+00_JPRB, 0.20382E+00_JPRB, 0.20810E+00_JPRB, 0.21509E+00_JPRB, 0.22862E+00_JPRB, 0.24397E+00_JPRB ,&
&  0.26334E+00_JPRB, 0.31128E+00_JPRB, 0.32862E+00_JPRB, 0.34640E+00_JPRB, 0.36248E+00_JPRB, 0.37923E+00_JPRB, 0.38246E+00_JPRB ,&
&  0.38830E+00_JPRB, 0.39008E+00_JPRB, 0.39637E+00_JPRB, 0.39989E+00_JPRB, 0.41277E+00_JPRB, 0.40920E+00_JPRB, 0.40311E+00_JPRB ,&
&  0.38094E+00_JPRB, 0.35814E+00_JPRB, 0.34008E+00_JPRB, 0.33405E+00_JPRB, 0.33400E+00_JPRB, 0.33474E+00_JPRB, 0.33551E+00_JPRB ,&
&  0.33574E+00_JPRB, 0.33519E+00_JPRB, 0.33220E+00_JPRB, 0.33233E+00_JPRB, 0.33201E+00_JPRB, 0.33165E+00_JPRB, 0.32831E+00_JPRB ,&
&  0.32631E+00_JPRB, 0.32847E+00_JPRB, 0.32851E+00_JPRB, 0.33149E+00_JPRB, 0.32885E+00_JPRB, 0.32734E+00_JPRB, 0.32526E+00_JPRB ,&
&  0.32726E+00_JPRB, 0.32691E+00_JPRB, 0.32446E+00_JPRB, 0.32147E+00_JPRB, 0.32034E+00_JPRB, 0.31667E+00_JPRB, 0.31559E+00_JPRB ,&
&  0.31699E+00_JPRB, 0.32163E+00_JPRB, 0.32640E+00_JPRB, 0.32867E+00_JPRB, 0.32559E+00_JPRB, 0.32215E+00_JPRB, 0.31915E+00_JPRB ,&
&  0.31583E+00_JPRB, 0.31324E+00_JPRB, 0.31017E+00_JPRB, 0.30872E+00_JPRB, 0.31001E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.30791E+00_JPRB, 0.30914E+00_JPRB, 0.31061E+00_JPRB, 0.31167E+00_JPRB, 0.31192E+00_JPRB, 0.31345E+00_JPRB, 0.31350E+00_JPRB ,&
&  0.31561E+00_JPRB, 0.31549E+00_JPRB, 0.31635E+00_JPRB, 0.31729E+00_JPRB, 0.31863E+00_JPRB, 0.31960E+00_JPRB, 0.32023E+00_JPRB ,&
&  0.32171E+00_JPRB, 0.32242E+00_JPRB, 0.32286E+00_JPRB, 0.32128E+00_JPRB, 0.31998E+00_JPRB, 0.31758E+00_JPRB, 0.31361E+00_JPRB ,&
&  0.30874E+00_JPRB, 0.30430E+00_JPRB, 0.28730E+00_JPRB, 0.28074E+00_JPRB, 0.27030E+00_JPRB, 0.26458E+00_JPRB, 0.26178E+00_JPRB ,&
&  0.25998E+00_JPRB, 0.25887E+00_JPRB, 0.25529E+00_JPRB, 0.25032E+00_JPRB, 0.24440E+00_JPRB, 0.23577E+00_JPRB, 0.23013E+00_JPRB ,&
&  0.22474E+00_JPRB, 0.22305E+00_JPRB, 0.22475E+00_JPRB, 0.23003E+00_JPRB, 0.23938E+00_JPRB, 0.25127E+00_JPRB, 0.26454E+00_JPRB ,&
&  0.28091E+00_JPRB, 0.29822E+00_JPRB, 0.32182E+00_JPRB, 0.33121E+00_JPRB, 0.34036E+00_JPRB, 0.34841E+00_JPRB, 0.35514E+00_JPRB ,&
&  0.35421E+00_JPRB, 0.35286E+00_JPRB, 0.35697E+00_JPRB, 0.35332E+00_JPRB, 0.34915E+00_JPRB, 0.33865E+00_JPRB, 0.32954E+00_JPRB ,&
&  0.31840E+00_JPRB, 0.31319E+00_JPRB, 0.30915E+00_JPRB, 0.30864E+00_JPRB, 0.30893E+00_JPRB, 0.30997E+00_JPRB, 0.31211E+00_JPRB ,&
&  0.31469E+00_JPRB, 0.31903E+00_JPRB, 0.32109E+00_JPRB, 0.32267E+00_JPRB, 0.32358E+00_JPRB, 0.32587E+00_JPRB, 0.32754E+00_JPRB ,&
&  0.32836E+00_JPRB, 0.32790E+00_JPRB, 0.32564E+00_JPRB, 0.32075E+00_JPRB, 0.31805E+00_JPRB, 0.32167E+00_JPRB, 0.32075E+00_JPRB ,&
&  0.32091E+00_JPRB, 0.31987E+00_JPRB, 0.31957E+00_JPRB, 0.31982E+00_JPRB, 0.32112E+00_JPRB, 0.32096E+00_JPRB, 0.32196E+00_JPRB ,&
&  0.32369E+00_JPRB, 0.32668E+00_JPRB, 0.32816E+00_JPRB, 0.32922E+00_JPRB, 0.32724E+00_JPRB, 0.32348E+00_JPRB, 0.31981E+00_JPRB ,&
&  0.31648E+00_JPRB, 0.31342E+00_JPRB, 0.31035E+00_JPRB, 0.30843E+00_JPRB, 0.30768E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.30244E+00_JPRB, 0.30218E+00_JPRB, 0.30141E+00_JPRB, 0.30068E+00_JPRB, 0.29982E+00_JPRB, 0.30031E+00_JPRB, 0.30177E+00_JPRB ,&
&  0.30387E+00_JPRB, 0.30473E+00_JPRB, 0.30567E+00_JPRB, 0.30764E+00_JPRB, 0.31076E+00_JPRB, 0.31273E+00_JPRB, 0.31479E+00_JPRB ,&
&  0.31686E+00_JPRB, 0.31831E+00_JPRB, 0.31951E+00_JPRB, 0.31968E+00_JPRB, 0.31885E+00_JPRB, 0.31706E+00_JPRB, 0.31604E+00_JPRB ,&
&  0.31435E+00_JPRB, 0.31276E+00_JPRB, 0.31039E+00_JPRB, 0.30769E+00_JPRB, 0.30450E+00_JPRB, 0.28249E+00_JPRB, 0.27593E+00_JPRB ,&
&  0.27433E+00_JPRB, 0.27436E+00_JPRB, 0.27749E+00_JPRB, 0.28209E+00_JPRB, 0.29070E+00_JPRB, 0.28966E+00_JPRB, 0.29055E+00_JPRB ,&
&  0.27845E+00_JPRB, 0.27891E+00_JPRB, 0.27860E+00_JPRB, 0.27931E+00_JPRB, 0.28250E+00_JPRB, 0.28581E+00_JPRB, 0.29218E+00_JPRB ,&
&  0.30749E+00_JPRB, 0.31049E+00_JPRB, 0.31246E+00_JPRB, 0.31439E+00_JPRB, 0.31602E+00_JPRB, 0.31658E+00_JPRB, 0.31615E+00_JPRB ,&
&  0.31428E+00_JPRB, 0.31201E+00_JPRB, 0.30923E+00_JPRB, 0.30562E+00_JPRB, 0.30289E+00_JPRB, 0.30051E+00_JPRB, 0.30033E+00_JPRB ,&
&  0.29922E+00_JPRB, 0.29964E+00_JPRB, 0.30027E+00_JPRB, 0.30102E+00_JPRB, 0.30110E+00_JPRB, 0.30208E+00_JPRB, 0.30478E+00_JPRB ,&
&  0.30743E+00_JPRB, 0.31013E+00_JPRB, 0.31194E+00_JPRB, 0.31389E+00_JPRB, 0.31486E+00_JPRB, 0.31643E+00_JPRB, 0.31677E+00_JPRB ,&
&  0.31713E+00_JPRB, 0.31628E+00_JPRB, 0.31610E+00_JPRB, 0.31587E+00_JPRB, 0.31530E+00_JPRB, 0.31535E+00_JPRB, 0.31512E+00_JPRB ,&
&  0.31567E+00_JPRB, 0.31580E+00_JPRB, 0.31649E+00_JPRB, 0.31754E+00_JPRB, 0.31865E+00_JPRB, 0.31878E+00_JPRB, 0.31921E+00_JPRB ,&
&  0.32029E+00_JPRB, 0.32187E+00_JPRB, 0.32271E+00_JPRB, 0.32362E+00_JPRB, 0.32304E+00_JPRB, 0.32059E+00_JPRB, 0.31701E+00_JPRB ,&
&  0.31352E+00_JPRB, 0.31042E+00_JPRB, 0.30735E+00_JPRB, 0.30529E+00_JPRB, 0.30354E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.29645E+00_JPRB, 0.29656E+00_JPRB, 0.29691E+00_JPRB, 0.29712E+00_JPRB, 0.29707E+00_JPRB, 0.29758E+00_JPRB, 0.29886E+00_JPRB ,&
&  0.30122E+00_JPRB, 0.30367E+00_JPRB, 0.30474E+00_JPRB, 0.30535E+00_JPRB, 0.30609E+00_JPRB, 0.30685E+00_JPRB, 0.30768E+00_JPRB ,&
&  0.30876E+00_JPRB, 0.30947E+00_JPRB, 0.30913E+00_JPRB, 0.30793E+00_JPRB, 0.30751E+00_JPRB, 0.30773E+00_JPRB, 0.30872E+00_JPRB ,&
&  0.30945E+00_JPRB, 0.30953E+00_JPRB, 0.30907E+00_JPRB, 0.30879E+00_JPRB, 0.30768E+00_JPRB, 0.30655E+00_JPRB, 0.30523E+00_JPRB ,&
&  0.30445E+00_JPRB, 0.30384E+00_JPRB, 0.30344E+00_JPRB, 0.30310E+00_JPRB, 0.30353E+00_JPRB, 0.30377E+00_JPRB, 0.30401E+00_JPRB ,&
&  0.30414E+00_JPRB, 0.30410E+00_JPRB, 0.30454E+00_JPRB, 0.30487E+00_JPRB, 0.30509E+00_JPRB, 0.30472E+00_JPRB, 0.30358E+00_JPRB ,&
&  0.30175E+00_JPRB, 0.29999E+00_JPRB, 0.29894E+00_JPRB, 0.29762E+00_JPRB, 0.29643E+00_JPRB, 0.29520E+00_JPRB, 0.29449E+00_JPRB ,&
&  0.29351E+00_JPRB, 0.29261E+00_JPRB, 0.29226E+00_JPRB, 0.29258E+00_JPRB, 0.29258E+00_JPRB, 0.29298E+00_JPRB, 0.29380E+00_JPRB ,&
&  0.29397E+00_JPRB, 0.29409E+00_JPRB, 0.29426E+00_JPRB, 0.29464E+00_JPRB, 0.29488E+00_JPRB, 0.29532E+00_JPRB, 0.29634E+00_JPRB ,&
&  0.29783E+00_JPRB, 0.29984E+00_JPRB, 0.30100E+00_JPRB, 0.30222E+00_JPRB, 0.30363E+00_JPRB, 0.30601E+00_JPRB, 0.30708E+00_JPRB ,&
&  0.30783E+00_JPRB, 0.30845E+00_JPRB, 0.30919E+00_JPRB, 0.30973E+00_JPRB, 0.31010E+00_JPRB, 0.31051E+00_JPRB, 0.31085E+00_JPRB ,&
&  0.31168E+00_JPRB, 0.31204E+00_JPRB, 0.31226E+00_JPRB, 0.31236E+00_JPRB, 0.31246E+00_JPRB, 0.31224E+00_JPRB, 0.31166E+00_JPRB ,&
&  0.31135E+00_JPRB, 0.31044E+00_JPRB, 0.30919E+00_JPRB, 0.30843E+00_JPRB, 0.30815E+00_JPRB, 0.30679E+00_JPRB, 0.30478E+00_JPRB ,&
&  0.30251E+00_JPRB, 0.30064E+00_JPRB, 0.29934E+00_JPRB, 0.29817E+00_JPRB, 0.29723E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.29576E+00_JPRB, 0.29576E+00_JPRB, 0.29590E+00_JPRB, 0.29619E+00_JPRB, 0.29639E+00_JPRB, 0.29646E+00_JPRB, 0.29642E+00_JPRB ,&
&  0.29655E+00_JPRB, 0.29728E+00_JPRB, 0.29832E+00_JPRB, 0.29922E+00_JPRB, 0.29982E+00_JPRB, 0.30006E+00_JPRB, 0.30006E+00_JPRB ,&
&  0.29998E+00_JPRB, 0.29979E+00_JPRB, 0.29912E+00_JPRB, 0.29857E+00_JPRB, 0.29897E+00_JPRB, 0.29938E+00_JPRB, 0.29963E+00_JPRB ,&
&  0.29978E+00_JPRB, 0.30011E+00_JPRB, 0.30054E+00_JPRB, 0.30082E+00_JPRB, 0.30100E+00_JPRB, 0.30107E+00_JPRB, 0.30120E+00_JPRB ,&
&  0.30123E+00_JPRB, 0.30110E+00_JPRB, 0.30072E+00_JPRB, 0.30044E+00_JPRB, 0.29999E+00_JPRB, 0.29978E+00_JPRB, 0.29951E+00_JPRB ,&
&  0.29941E+00_JPRB, 0.29906E+00_JPRB, 0.29860E+00_JPRB, 0.29801E+00_JPRB, 0.29731E+00_JPRB, 0.29603E+00_JPRB, 0.29486E+00_JPRB ,&
&  0.29382E+00_JPRB, 0.29299E+00_JPRB, 0.29246E+00_JPRB, 0.29233E+00_JPRB, 0.29224E+00_JPRB, 0.29223E+00_JPRB, 0.29198E+00_JPRB ,&
&  0.29161E+00_JPRB, 0.29134E+00_JPRB, 0.29116E+00_JPRB, 0.29092E+00_JPRB, 0.29057E+00_JPRB, 0.29016E+00_JPRB, 0.28983E+00_JPRB ,&
&  0.28932E+00_JPRB, 0.28916E+00_JPRB, 0.28935E+00_JPRB, 0.28965E+00_JPRB, 0.29010E+00_JPRB, 0.29055E+00_JPRB, 0.29113E+00_JPRB ,&
&  0.29183E+00_JPRB, 0.29260E+00_JPRB, 0.29331E+00_JPRB, 0.29405E+00_JPRB, 0.29484E+00_JPRB, 0.29568E+00_JPRB, 0.29634E+00_JPRB ,&
&  0.29700E+00_JPRB, 0.29772E+00_JPRB, 0.29860E+00_JPRB, 0.29908E+00_JPRB, 0.29948E+00_JPRB, 0.29981E+00_JPRB, 0.30007E+00_JPRB ,&
&  0.30022E+00_JPRB, 0.30022E+00_JPRB, 0.30020E+00_JPRB, 0.30013E+00_JPRB, 0.29981E+00_JPRB, 0.29940E+00_JPRB, 0.29889E+00_JPRB ,&
&  0.29844E+00_JPRB, 0.29774E+00_JPRB, 0.29707E+00_JPRB, 0.29670E+00_JPRB, 0.29645E+00_JPRB, 0.29618E+00_JPRB, 0.29588E+00_JPRB ,&
&  0.29554E+00_JPRB, 0.29543E+00_JPRB, 0.29561E+00_JPRB, 0.29578E+00_JPRB, 0.29591E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB, 0.28999E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB, 0.21469E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.22553E+00_JPRB, 0.22583E+00_JPRB, 0.22599E+00_JPRB, 0.22610E+00_JPRB, 0.22478E+00_JPRB, 0.22539E+00_JPRB, 0.22561E+00_JPRB ,&
&  0.22598E+00_JPRB, 0.22630E+00_JPRB, 0.22685E+00_JPRB, 0.22748E+00_JPRB, 0.22809E+00_JPRB, 0.22862E+00_JPRB, 0.22908E+00_JPRB ,&
&  0.22934E+00_JPRB, 0.22969E+00_JPRB, 0.22851E+00_JPRB, 0.22786E+00_JPRB, 0.22782E+00_JPRB, 0.22796E+00_JPRB, 0.22797E+00_JPRB ,&
&  0.22797E+00_JPRB, 0.22790E+00_JPRB, 0.22753E+00_JPRB, 0.22724E+00_JPRB, 0.22712E+00_JPRB, 0.22695E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22631E+00_JPRB, 0.22603E+00_JPRB, 0.22568E+00_JPRB, 0.22530E+00_JPRB, 0.22513E+00_JPRB, 0.22488E+00_JPRB, 0.22426E+00_JPRB ,&
&  0.22353E+00_JPRB, 0.22252E+00_JPRB, 0.22159E+00_JPRB, 0.22076E+00_JPRB, 0.22009E+00_JPRB, 0.21973E+00_JPRB, 0.21936E+00_JPRB ,&
&  0.21889E+00_JPRB, 0.21825E+00_JPRB, 0.21763E+00_JPRB, 0.21692E+00_JPRB, 0.21630E+00_JPRB, 0.21575E+00_JPRB, 0.21510E+00_JPRB ,&
&  0.21438E+00_JPRB, 0.21365E+00_JPRB, 0.21290E+00_JPRB, 0.21209E+00_JPRB, 0.21131E+00_JPRB, 0.21053E+00_JPRB, 0.20968E+00_JPRB ,&
&  0.20884E+00_JPRB, 0.20846E+00_JPRB, 0.20802E+00_JPRB, 0.20765E+00_JPRB, 0.20737E+00_JPRB, 0.20715E+00_JPRB, 0.20694E+00_JPRB ,&
&  0.20678E+00_JPRB, 0.20660E+00_JPRB, 0.20650E+00_JPRB, 0.20642E+00_JPRB, 0.20641E+00_JPRB, 0.20648E+00_JPRB, 0.20663E+00_JPRB ,&
&  0.20682E+00_JPRB, 0.20705E+00_JPRB, 0.20737E+00_JPRB, 0.20787E+00_JPRB, 0.20837E+00_JPRB, 0.20895E+00_JPRB, 0.20953E+00_JPRB ,&
&  0.21016E+00_JPRB, 0.21078E+00_JPRB, 0.21142E+00_JPRB, 0.21201E+00_JPRB, 0.21280E+00_JPRB, 0.21357E+00_JPRB, 0.21441E+00_JPRB ,&
&  0.21525E+00_JPRB, 0.21615E+00_JPRB, 0.21704E+00_JPRB, 0.21800E+00_JPRB, 0.21908E+00_JPRB, 0.21942E+00_JPRB, 0.21977E+00_JPRB ,&
&  0.22020E+00_JPRB, 0.22084E+00_JPRB, 0.22142E+00_JPRB, 0.22216E+00_JPRB, 0.22341E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.25207E+00_JPRB, 0.25849E+00_JPRB, 0.26300E+00_JPRB, 0.26800E+00_JPRB, 0.26838E+00_JPRB, 0.30609E+00_JPRB, 0.30509E+00_JPRB ,&
&  0.30369E+00_JPRB, 0.28879E+00_JPRB, 0.28594E+00_JPRB, 0.28098E+00_JPRB, 0.27532E+00_JPRB, 0.26920E+00_JPRB, 0.26392E+00_JPRB ,&
&  0.25914E+00_JPRB, 0.25388E+00_JPRB, 0.24954E+00_JPRB, 0.24554E+00_JPRB, 0.24226E+00_JPRB, 0.23934E+00_JPRB, 0.23728E+00_JPRB ,&
&  0.23671E+00_JPRB, 0.23807E+00_JPRB, 0.24099E+00_JPRB, 0.24281E+00_JPRB, 0.24373E+00_JPRB, 0.24450E+00_JPRB, 0.24523E+00_JPRB ,&
&  0.24539E+00_JPRB, 0.24554E+00_JPRB, 0.24460E+00_JPRB, 0.24279E+00_JPRB, 0.24094E+00_JPRB, 0.23934E+00_JPRB, 0.23804E+00_JPRB ,&
&  0.23679E+00_JPRB, 0.23548E+00_JPRB, 0.23408E+00_JPRB, 0.23315E+00_JPRB, 0.23197E+00_JPRB, 0.23044E+00_JPRB, 0.22843E+00_JPRB ,&
&  0.22644E+00_JPRB, 0.22447E+00_JPRB, 0.22218E+00_JPRB, 0.22035E+00_JPRB, 0.21857E+00_JPRB, 0.21678E+00_JPRB, 0.21500E+00_JPRB ,&
&  0.21400E+00_JPRB, 0.21301E+00_JPRB, 0.21180E+00_JPRB, 0.21024E+00_JPRB, 0.20904E+00_JPRB, 0.20777E+00_JPRB, 0.20664E+00_JPRB ,&
&  0.20588E+00_JPRB, 0.20511E+00_JPRB, 0.20432E+00_JPRB, 0.20351E+00_JPRB, 0.20290E+00_JPRB, 0.20221E+00_JPRB, 0.20160E+00_JPRB ,&
&  0.20106E+00_JPRB, 0.20047E+00_JPRB, 0.20033E+00_JPRB, 0.20027E+00_JPRB, 0.20035E+00_JPRB, 0.20042E+00_JPRB, 0.20072E+00_JPRB ,&
&  0.20116E+00_JPRB, 0.20180E+00_JPRB, 0.20258E+00_JPRB, 0.20332E+00_JPRB, 0.20404E+00_JPRB, 0.20479E+00_JPRB, 0.20588E+00_JPRB ,&
&  0.20708E+00_JPRB, 0.20837E+00_JPRB, 0.20971E+00_JPRB, 0.21078E+00_JPRB, 0.21286E+00_JPRB, 0.21484E+00_JPRB, 0.21695E+00_JPRB ,&
&  0.21938E+00_JPRB, 0.22223E+00_JPRB, 0.22527E+00_JPRB, 0.22848E+00_JPRB, 0.23264E+00_JPRB, 0.23547E+00_JPRB, 0.23756E+00_JPRB ,&
&  0.23987E+00_JPRB, 0.24362E+00_JPRB, 0.24539E+00_JPRB, 0.24689E+00_JPRB, 0.24910E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.32272E+00_JPRB, 0.32713E+00_JPRB, 0.33093E+00_JPRB, 0.33427E+00_JPRB, 0.33716E+00_JPRB, 0.33778E+00_JPRB, 0.33678E+00_JPRB ,&
&  0.32783E+00_JPRB, 0.32111E+00_JPRB, 0.31299E+00_JPRB, 0.30591E+00_JPRB, 0.29935E+00_JPRB, 0.29200E+00_JPRB, 0.28541E+00_JPRB ,&
&  0.27869E+00_JPRB, 0.27200E+00_JPRB, 0.26557E+00_JPRB, 0.25808E+00_JPRB, 0.25023E+00_JPRB, 0.24274E+00_JPRB, 0.23803E+00_JPRB ,&
&  0.23530E+00_JPRB, 0.23509E+00_JPRB, 0.23781E+00_JPRB, 0.24158E+00_JPRB, 0.24649E+00_JPRB, 0.25283E+00_JPRB, 0.25960E+00_JPRB ,&
&  0.26393E+00_JPRB, 0.26635E+00_JPRB, 0.26602E+00_JPRB, 0.26479E+00_JPRB, 0.26291E+00_JPRB, 0.26122E+00_JPRB, 0.25861E+00_JPRB ,&
&  0.25530E+00_JPRB, 0.25276E+00_JPRB, 0.25158E+00_JPRB, 0.25205E+00_JPRB, 0.25142E+00_JPRB, 0.24932E+00_JPRB, 0.24545E+00_JPRB ,&
&  0.24127E+00_JPRB, 0.23710E+00_JPRB, 0.23292E+00_JPRB, 0.22906E+00_JPRB, 0.22590E+00_JPRB, 0.22263E+00_JPRB, 0.21917E+00_JPRB ,&
&  0.21694E+00_JPRB, 0.21457E+00_JPRB, 0.21197E+00_JPRB, 0.20947E+00_JPRB, 0.20740E+00_JPRB, 0.20580E+00_JPRB, 0.20446E+00_JPRB ,&
&  0.20339E+00_JPRB, 0.20256E+00_JPRB, 0.20188E+00_JPRB, 0.20136E+00_JPRB, 0.20087E+00_JPRB, 0.20015E+00_JPRB, 0.19945E+00_JPRB ,&
&  0.19873E+00_JPRB, 0.19845E+00_JPRB, 0.19802E+00_JPRB, 0.19803E+00_JPRB, 0.19827E+00_JPRB, 0.19843E+00_JPRB, 0.19891E+00_JPRB ,&
&  0.19999E+00_JPRB, 0.20111E+00_JPRB, 0.20177E+00_JPRB, 0.20283E+00_JPRB, 0.20385E+00_JPRB, 0.20518E+00_JPRB, 0.20688E+00_JPRB ,&
&  0.20799E+00_JPRB, 0.20923E+00_JPRB, 0.21068E+00_JPRB, 0.21223E+00_JPRB, 0.21470E+00_JPRB, 0.21739E+00_JPRB, 0.22039E+00_JPRB ,&
&  0.22423E+00_JPRB, 0.22804E+00_JPRB, 0.23236E+00_JPRB, 0.23746E+00_JPRB, 0.24621E+00_JPRB, 0.25282E+00_JPRB, 0.26152E+00_JPRB ,&
&  0.27047E+00_JPRB, 0.27676E+00_JPRB, 0.30988E+00_JPRB, 0.31222E+00_JPRB, 0.31720E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.33100E+00_JPRB, 0.33747E+00_JPRB, 0.34437E+00_JPRB, 0.34937E+00_JPRB, 0.35308E+00_JPRB, 0.35606E+00_JPRB, 0.35512E+00_JPRB ,&
&  0.34898E+00_JPRB, 0.33352E+00_JPRB, 0.32201E+00_JPRB, 0.31103E+00_JPRB, 0.30166E+00_JPRB, 0.29285E+00_JPRB, 0.28459E+00_JPRB ,&
&  0.27679E+00_JPRB, 0.27035E+00_JPRB, 0.26516E+00_JPRB, 0.26066E+00_JPRB, 0.25448E+00_JPRB, 0.24909E+00_JPRB, 0.24476E+00_JPRB ,&
&  0.24117E+00_JPRB, 0.24100E+00_JPRB, 0.24313E+00_JPRB, 0.24738E+00_JPRB, 0.25525E+00_JPRB, 0.26351E+00_JPRB, 0.27279E+00_JPRB ,&
&  0.28716E+00_JPRB, 0.29177E+00_JPRB, 0.29080E+00_JPRB, 0.29011E+00_JPRB, 0.28522E+00_JPRB, 0.27922E+00_JPRB, 0.26981E+00_JPRB ,&
&  0.26978E+00_JPRB, 0.26790E+00_JPRB, 0.27450E+00_JPRB, 0.27488E+00_JPRB, 0.27376E+00_JPRB, 0.27319E+00_JPRB, 0.26564E+00_JPRB ,&
&  0.25967E+00_JPRB, 0.25083E+00_JPRB, 0.24404E+00_JPRB, 0.23735E+00_JPRB, 0.23109E+00_JPRB, 0.22732E+00_JPRB, 0.22247E+00_JPRB ,&
&  0.21881E+00_JPRB, 0.21595E+00_JPRB, 0.21233E+00_JPRB, 0.20993E+00_JPRB, 0.20795E+00_JPRB, 0.20594E+00_JPRB, 0.20492E+00_JPRB ,&
&  0.20415E+00_JPRB, 0.20387E+00_JPRB, 0.20395E+00_JPRB, 0.20414E+00_JPRB, 0.20458E+00_JPRB, 0.20543E+00_JPRB, 0.20607E+00_JPRB ,&
&  0.20678E+00_JPRB, 0.20699E+00_JPRB, 0.20660E+00_JPRB, 0.20615E+00_JPRB, 0.20509E+00_JPRB, 0.20437E+00_JPRB, 0.20347E+00_JPRB ,&
&  0.20296E+00_JPRB, 0.20327E+00_JPRB, 0.20359E+00_JPRB, 0.20426E+00_JPRB, 0.20496E+00_JPRB, 0.20643E+00_JPRB, 0.20781E+00_JPRB ,&
&  0.20930E+00_JPRB, 0.21107E+00_JPRB, 0.21255E+00_JPRB, 0.21439E+00_JPRB, 0.21662E+00_JPRB, 0.21821E+00_JPRB, 0.22085E+00_JPRB ,&
&  0.22349E+00_JPRB, 0.22674E+00_JPRB, 0.23210E+00_JPRB, 0.23817E+00_JPRB, 0.24634E+00_JPRB, 0.25697E+00_JPRB, 0.27062E+00_JPRB ,&
&  0.28823E+00_JPRB, 0.31347E+00_JPRB, 0.31560E+00_JPRB, 0.32147E+00_JPRB, 0.32607E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.33593E+00_JPRB, 0.33843E+00_JPRB, 0.34256E+00_JPRB, 0.34785E+00_JPRB, 0.35443E+00_JPRB, 0.36024E+00_JPRB, 0.36148E+00_JPRB ,&
&  0.35241E+00_JPRB, 0.33947E+00_JPRB, 0.32148E+00_JPRB, 0.30698E+00_JPRB, 0.29322E+00_JPRB, 0.28280E+00_JPRB, 0.27426E+00_JPRB ,&
&  0.26907E+00_JPRB, 0.26847E+00_JPRB, 0.26909E+00_JPRB, 0.27033E+00_JPRB, 0.26956E+00_JPRB, 0.27076E+00_JPRB, 0.27133E+00_JPRB ,&
&  0.27035E+00_JPRB, 0.27028E+00_JPRB, 0.27017E+00_JPRB, 0.27373E+00_JPRB, 0.28050E+00_JPRB, 0.28687E+00_JPRB, 0.29323E+00_JPRB ,&
&  0.30061E+00_JPRB, 0.30313E+00_JPRB, 0.30750E+00_JPRB, 0.30653E+00_JPRB, 0.30306E+00_JPRB, 0.29856E+00_JPRB, 0.29229E+00_JPRB ,&
&  0.28747E+00_JPRB, 0.28334E+00_JPRB, 0.28176E+00_JPRB, 0.28224E+00_JPRB, 0.28247E+00_JPRB, 0.28337E+00_JPRB, 0.27679E+00_JPRB ,&
&  0.26941E+00_JPRB, 0.25931E+00_JPRB, 0.24821E+00_JPRB, 0.24012E+00_JPRB, 0.23274E+00_JPRB, 0.22741E+00_JPRB, 0.22266E+00_JPRB ,&
&  0.21823E+00_JPRB, 0.21524E+00_JPRB, 0.21217E+00_JPRB, 0.20996E+00_JPRB, 0.20929E+00_JPRB, 0.20778E+00_JPRB, 0.20699E+00_JPRB ,&
&  0.20668E+00_JPRB, 0.20713E+00_JPRB, 0.20855E+00_JPRB, 0.20988E+00_JPRB, 0.21166E+00_JPRB, 0.21374E+00_JPRB, 0.21555E+00_JPRB ,&
&  0.21793E+00_JPRB, 0.21980E+00_JPRB, 0.22000E+00_JPRB, 0.21937E+00_JPRB, 0.21738E+00_JPRB, 0.21481E+00_JPRB, 0.21245E+00_JPRB ,&
&  0.21017E+00_JPRB, 0.20854E+00_JPRB, 0.20788E+00_JPRB, 0.20712E+00_JPRB, 0.20753E+00_JPRB, 0.20862E+00_JPRB, 0.20951E+00_JPRB ,&
&  0.21113E+00_JPRB, 0.21244E+00_JPRB, 0.21392E+00_JPRB, 0.21630E+00_JPRB, 0.21864E+00_JPRB, 0.22069E+00_JPRB, 0.22286E+00_JPRB ,&
&  0.22315E+00_JPRB, 0.22480E+00_JPRB, 0.22695E+00_JPRB, 0.23060E+00_JPRB, 0.24104E+00_JPRB, 0.24901E+00_JPRB, 0.26741E+00_JPRB ,&
&  0.28476E+00_JPRB, 0.32134E+00_JPRB, 0.32292E+00_JPRB, 0.32702E+00_JPRB, 0.33251E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.33873E+00_JPRB, 0.34312E+00_JPRB, 0.34416E+00_JPRB, 0.34743E+00_JPRB, 0.35169E+00_JPRB, 0.35675E+00_JPRB, 0.36086E+00_JPRB ,&
&  0.36574E+00_JPRB, 0.36547E+00_JPRB, 0.36031E+00_JPRB, 0.35366E+00_JPRB, 0.34796E+00_JPRB, 0.33111E+00_JPRB, 0.32198E+00_JPRB ,&
&  0.31464E+00_JPRB, 0.31390E+00_JPRB, 0.31452E+00_JPRB, 0.31607E+00_JPRB, 0.31723E+00_JPRB, 0.32219E+00_JPRB, 0.32381E+00_JPRB ,&
&  0.32155E+00_JPRB, 0.31662E+00_JPRB, 0.31169E+00_JPRB, 0.31091E+00_JPRB, 0.31349E+00_JPRB, 0.31556E+00_JPRB, 0.31295E+00_JPRB ,&
&  0.30938E+00_JPRB, 0.30910E+00_JPRB, 0.30980E+00_JPRB, 0.31129E+00_JPRB, 0.32108E+00_JPRB, 0.31720E+00_JPRB, 0.31294E+00_JPRB ,&
&  0.30586E+00_JPRB, 0.29965E+00_JPRB, 0.29526E+00_JPRB, 0.29552E+00_JPRB, 0.29808E+00_JPRB, 0.29855E+00_JPRB, 0.30105E+00_JPRB ,&
&  0.28881E+00_JPRB, 0.27870E+00_JPRB, 0.26417E+00_JPRB, 0.25290E+00_JPRB, 0.24258E+00_JPRB, 0.23549E+00_JPRB, 0.22876E+00_JPRB ,&
&  0.22300E+00_JPRB, 0.21833E+00_JPRB, 0.21385E+00_JPRB, 0.21232E+00_JPRB, 0.21314E+00_JPRB, 0.21338E+00_JPRB, 0.21321E+00_JPRB ,&
&  0.21388E+00_JPRB, 0.21485E+00_JPRB, 0.21562E+00_JPRB, 0.21655E+00_JPRB, 0.21796E+00_JPRB, 0.22035E+00_JPRB, 0.22351E+00_JPRB ,&
&  0.22854E+00_JPRB, 0.23260E+00_JPRB, 0.23482E+00_JPRB, 0.23567E+00_JPRB, 0.23260E+00_JPRB, 0.22756E+00_JPRB, 0.22223E+00_JPRB ,&
&  0.21716E+00_JPRB, 0.21475E+00_JPRB, 0.21270E+00_JPRB, 0.21173E+00_JPRB, 0.21161E+00_JPRB, 0.21248E+00_JPRB, 0.21417E+00_JPRB ,&
&  0.21584E+00_JPRB, 0.21784E+00_JPRB, 0.21918E+00_JPRB, 0.22081E+00_JPRB, 0.22282E+00_JPRB, 0.22458E+00_JPRB, 0.22642E+00_JPRB ,&
&  0.22847E+00_JPRB, 0.23007E+00_JPRB, 0.23278E+00_JPRB, 0.23608E+00_JPRB, 0.24226E+00_JPRB, 0.24993E+00_JPRB, 0.26377E+00_JPRB ,&
&  0.28052E+00_JPRB, 0.31365E+00_JPRB, 0.32554E+00_JPRB, 0.33036E+00_JPRB, 0.33656E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.33594E+00_JPRB, 0.34212E+00_JPRB, 0.34718E+00_JPRB, 0.35719E+00_JPRB, 0.35979E+00_JPRB, 0.36219E+00_JPRB, 0.36406E+00_JPRB ,&
&  0.36973E+00_JPRB, 0.37315E+00_JPRB, 0.37082E+00_JPRB, 0.37007E+00_JPRB, 0.36912E+00_JPRB, 0.36817E+00_JPRB, 0.36887E+00_JPRB ,&
&  0.36868E+00_JPRB, 0.37059E+00_JPRB, 0.36838E+00_JPRB, 0.36552E+00_JPRB, 0.36735E+00_JPRB, 0.37201E+00_JPRB, 0.37518E+00_JPRB ,&
&  0.37032E+00_JPRB, 0.36932E+00_JPRB, 0.37800E+00_JPRB, 0.38048E+00_JPRB, 0.38206E+00_JPRB, 0.38430E+00_JPRB, 0.38428E+00_JPRB ,&
&  0.37105E+00_JPRB, 0.34662E+00_JPRB, 0.31917E+00_JPRB, 0.31949E+00_JPRB, 0.33398E+00_JPRB, 0.32987E+00_JPRB, 0.32847E+00_JPRB ,&
&  0.32563E+00_JPRB, 0.32138E+00_JPRB, 0.31723E+00_JPRB, 0.31553E+00_JPRB, 0.31279E+00_JPRB, 0.30851E+00_JPRB, 0.30691E+00_JPRB ,&
&  0.30615E+00_JPRB, 0.31398E+00_JPRB, 0.31048E+00_JPRB, 0.29042E+00_JPRB, 0.27432E+00_JPRB, 0.26569E+00_JPRB, 0.25476E+00_JPRB ,&
&  0.24229E+00_JPRB, 0.23448E+00_JPRB, 0.23297E+00_JPRB, 0.23165E+00_JPRB, 0.23352E+00_JPRB, 0.23460E+00_JPRB, 0.23574E+00_JPRB ,&
&  0.23681E+00_JPRB, 0.23680E+00_JPRB, 0.23244E+00_JPRB, 0.22673E+00_JPRB, 0.22359E+00_JPRB, 0.22348E+00_JPRB, 0.22575E+00_JPRB ,&
&  0.23095E+00_JPRB, 0.23682E+00_JPRB, 0.25036E+00_JPRB, 0.25801E+00_JPRB, 0.25017E+00_JPRB, 0.24390E+00_JPRB, 0.23681E+00_JPRB ,&
&  0.22733E+00_JPRB, 0.22352E+00_JPRB, 0.22039E+00_JPRB, 0.21863E+00_JPRB, 0.21935E+00_JPRB, 0.22229E+00_JPRB, 0.22401E+00_JPRB ,&
&  0.22628E+00_JPRB, 0.22856E+00_JPRB, 0.22947E+00_JPRB, 0.23026E+00_JPRB, 0.23093E+00_JPRB, 0.23259E+00_JPRB, 0.23430E+00_JPRB ,&
&  0.23582E+00_JPRB, 0.23846E+00_JPRB, 0.24170E+00_JPRB, 0.24606E+00_JPRB, 0.24989E+00_JPRB, 0.25221E+00_JPRB, 0.25850E+00_JPRB ,&
&  0.26623E+00_JPRB, 0.28229E+00_JPRB, 0.30899E+00_JPRB, 0.33421E+00_JPRB, 0.33179E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.34272E+00_JPRB, 0.34751E+00_JPRB, 0.35406E+00_JPRB, 0.36138E+00_JPRB, 0.36652E+00_JPRB, 0.36866E+00_JPRB, 0.37092E+00_JPRB ,&
&  0.37397E+00_JPRB, 0.37525E+00_JPRB, 0.37626E+00_JPRB, 0.37902E+00_JPRB, 0.38451E+00_JPRB, 0.38720E+00_JPRB, 0.39165E+00_JPRB ,&
&  0.39330E+00_JPRB, 0.39123E+00_JPRB, 0.38465E+00_JPRB, 0.38258E+00_JPRB, 0.38592E+00_JPRB, 0.39054E+00_JPRB, 0.39582E+00_JPRB ,&
&  0.40142E+00_JPRB, 0.40661E+00_JPRB, 0.40917E+00_JPRB, 0.41008E+00_JPRB, 0.41046E+00_JPRB, 0.41050E+00_JPRB, 0.40802E+00_JPRB ,&
&  0.40385E+00_JPRB, 0.36540E+00_JPRB, 0.33877E+00_JPRB, 0.34045E+00_JPRB, 0.35255E+00_JPRB, 0.34787E+00_JPRB, 0.34643E+00_JPRB ,&
&  0.34492E+00_JPRB, 0.34176E+00_JPRB, 0.33697E+00_JPRB, 0.33322E+00_JPRB, 0.32838E+00_JPRB, 0.32220E+00_JPRB, 0.31711E+00_JPRB ,&
&  0.31620E+00_JPRB, 0.32072E+00_JPRB, 0.32251E+00_JPRB, 0.32550E+00_JPRB, 0.32524E+00_JPRB, 0.32801E+00_JPRB, 0.32754E+00_JPRB ,&
&  0.32295E+00_JPRB, 0.31763E+00_JPRB, 0.31616E+00_JPRB, 0.31606E+00_JPRB, 0.31710E+00_JPRB, 0.31966E+00_JPRB, 0.32004E+00_JPRB ,&
&  0.30668E+00_JPRB, 0.29532E+00_JPRB, 0.27709E+00_JPRB, 0.25945E+00_JPRB, 0.24644E+00_JPRB, 0.24042E+00_JPRB, 0.23600E+00_JPRB ,&
&  0.23479E+00_JPRB, 0.23976E+00_JPRB, 0.25484E+00_JPRB, 0.27372E+00_JPRB, 0.28945E+00_JPRB, 0.27144E+00_JPRB, 0.25531E+00_JPRB ,&
&  0.24532E+00_JPRB, 0.24012E+00_JPRB, 0.23584E+00_JPRB, 0.23413E+00_JPRB, 0.23423E+00_JPRB, 0.23610E+00_JPRB, 0.23894E+00_JPRB ,&
&  0.24311E+00_JPRB, 0.24714E+00_JPRB, 0.24990E+00_JPRB, 0.25124E+00_JPRB, 0.25192E+00_JPRB, 0.25260E+00_JPRB, 0.25199E+00_JPRB ,&
&  0.25156E+00_JPRB, 0.25425E+00_JPRB, 0.26034E+00_JPRB, 0.26857E+00_JPRB, 0.27649E+00_JPRB, 0.28511E+00_JPRB, 0.29736E+00_JPRB ,&
&  0.31121E+00_JPRB, 0.32349E+00_JPRB, 0.32771E+00_JPRB, 0.33328E+00_JPRB, 0.33823E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.36230E+00_JPRB, 0.36645E+00_JPRB, 0.37023E+00_JPRB, 0.37442E+00_JPRB, 0.38045E+00_JPRB, 0.38335E+00_JPRB, 0.38418E+00_JPRB ,&
&  0.38547E+00_JPRB, 0.38616E+00_JPRB, 0.38784E+00_JPRB, 0.39216E+00_JPRB, 0.39784E+00_JPRB, 0.40174E+00_JPRB, 0.40715E+00_JPRB ,&
&  0.40727E+00_JPRB, 0.39910E+00_JPRB, 0.39147E+00_JPRB, 0.39112E+00_JPRB, 0.39642E+00_JPRB, 0.40200E+00_JPRB, 0.40918E+00_JPRB ,&
&  0.41636E+00_JPRB, 0.42172E+00_JPRB, 0.42483E+00_JPRB, 0.42642E+00_JPRB, 0.42850E+00_JPRB, 0.42836E+00_JPRB, 0.42549E+00_JPRB ,&
&  0.42171E+00_JPRB, 0.41346E+00_JPRB, 0.39411E+00_JPRB, 0.39125E+00_JPRB, 0.38261E+00_JPRB, 0.37504E+00_JPRB, 0.36978E+00_JPRB ,&
&  0.36551E+00_JPRB, 0.36217E+00_JPRB, 0.35855E+00_JPRB, 0.35571E+00_JPRB, 0.35246E+00_JPRB, 0.34625E+00_JPRB, 0.34340E+00_JPRB ,&
&  0.34085E+00_JPRB, 0.33867E+00_JPRB, 0.33671E+00_JPRB, 0.33753E+00_JPRB, 0.33855E+00_JPRB, 0.34194E+00_JPRB, 0.34377E+00_JPRB ,&
&  0.34109E+00_JPRB, 0.33782E+00_JPRB, 0.33524E+00_JPRB, 0.33692E+00_JPRB, 0.33908E+00_JPRB, 0.34057E+00_JPRB, 0.34116E+00_JPRB ,&
&  0.33904E+00_JPRB, 0.33501E+00_JPRB, 0.33088E+00_JPRB, 0.32597E+00_JPRB, 0.32185E+00_JPRB, 0.29611E+00_JPRB, 0.28454E+00_JPRB ,&
&  0.28048E+00_JPRB, 0.28906E+00_JPRB, 0.30190E+00_JPRB, 0.30011E+00_JPRB, 0.30123E+00_JPRB, 0.30106E+00_JPRB, 0.29857E+00_JPRB ,&
&  0.29028E+00_JPRB, 0.27646E+00_JPRB, 0.26667E+00_JPRB, 0.26219E+00_JPRB, 0.25888E+00_JPRB, 0.25933E+00_JPRB, 0.26461E+00_JPRB ,&
&  0.27385E+00_JPRB, 0.28530E+00_JPRB, 0.29409E+00_JPRB, 0.29820E+00_JPRB, 0.29736E+00_JPRB, 0.29431E+00_JPRB, 0.29019E+00_JPRB ,&
&  0.28534E+00_JPRB, 0.29025E+00_JPRB, 0.30356E+00_JPRB, 0.32361E+00_JPRB, 0.32608E+00_JPRB, 0.33050E+00_JPRB, 0.33346E+00_JPRB ,&
&  0.33830E+00_JPRB, 0.34437E+00_JPRB, 0.34975E+00_JPRB, 0.35570E+00_JPRB, 0.35943E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.38722E+00_JPRB, 0.38955E+00_JPRB, 0.39178E+00_JPRB, 0.39324E+00_JPRB, 0.39581E+00_JPRB, 0.40046E+00_JPRB, 0.40231E+00_JPRB ,&
&  0.40055E+00_JPRB, 0.39862E+00_JPRB, 0.39848E+00_JPRB, 0.40165E+00_JPRB, 0.40780E+00_JPRB, 0.41442E+00_JPRB, 0.42206E+00_JPRB ,&
&  0.41847E+00_JPRB, 0.40669E+00_JPRB, 0.40239E+00_JPRB, 0.40431E+00_JPRB, 0.40904E+00_JPRB, 0.41438E+00_JPRB, 0.42105E+00_JPRB ,&
&  0.42740E+00_JPRB, 0.43167E+00_JPRB, 0.43610E+00_JPRB, 0.44001E+00_JPRB, 0.44258E+00_JPRB, 0.44105E+00_JPRB, 0.43683E+00_JPRB ,&
&  0.43179E+00_JPRB, 0.42687E+00_JPRB, 0.42275E+00_JPRB, 0.40695E+00_JPRB, 0.40444E+00_JPRB, 0.39549E+00_JPRB, 0.38796E+00_JPRB ,&
&  0.38266E+00_JPRB, 0.37834E+00_JPRB, 0.37573E+00_JPRB, 0.37304E+00_JPRB, 0.36933E+00_JPRB, 0.36288E+00_JPRB, 0.36158E+00_JPRB ,&
&  0.36139E+00_JPRB, 0.36315E+00_JPRB, 0.35931E+00_JPRB, 0.35759E+00_JPRB, 0.35572E+00_JPRB, 0.35437E+00_JPRB, 0.35919E+00_JPRB ,&
&  0.36467E+00_JPRB, 0.36246E+00_JPRB, 0.35872E+00_JPRB, 0.35834E+00_JPRB, 0.36248E+00_JPRB, 0.36408E+00_JPRB, 0.36513E+00_JPRB ,&
&  0.36711E+00_JPRB, 0.36800E+00_JPRB, 0.36392E+00_JPRB, 0.35918E+00_JPRB, 0.35056E+00_JPRB, 0.34031E+00_JPRB, 0.33787E+00_JPRB ,&
&  0.33504E+00_JPRB, 0.32898E+00_JPRB, 0.32497E+00_JPRB, 0.32368E+00_JPRB, 0.32315E+00_JPRB, 0.32369E+00_JPRB, 0.32344E+00_JPRB ,&
&  0.32299E+00_JPRB, 0.31933E+00_JPRB, 0.31392E+00_JPRB, 0.30903E+00_JPRB, 0.30251E+00_JPRB, 0.30435E+00_JPRB, 0.31314E+00_JPRB ,&
&  0.32439E+00_JPRB, 0.33112E+00_JPRB, 0.33557E+00_JPRB, 0.33906E+00_JPRB, 0.34221E+00_JPRB, 0.34380E+00_JPRB, 0.34620E+00_JPRB ,&
&  0.34938E+00_JPRB, 0.35083E+00_JPRB, 0.34924E+00_JPRB, 0.34842E+00_JPRB, 0.35060E+00_JPRB, 0.35498E+00_JPRB, 0.36055E+00_JPRB ,&
&  0.36627E+00_JPRB, 0.37114E+00_JPRB, 0.37528E+00_JPRB, 0.38055E+00_JPRB, 0.38393E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.41872E+00_JPRB, 0.41876E+00_JPRB, 0.41927E+00_JPRB, 0.42061E+00_JPRB, 0.42148E+00_JPRB, 0.42061E+00_JPRB, 0.42040E+00_JPRB ,&
&  0.41736E+00_JPRB, 0.41288E+00_JPRB, 0.40999E+00_JPRB, 0.41117E+00_JPRB, 0.41984E+00_JPRB, 0.43127E+00_JPRB, 0.43840E+00_JPRB ,&
&  0.43459E+00_JPRB, 0.42284E+00_JPRB, 0.42001E+00_JPRB, 0.42235E+00_JPRB, 0.42456E+00_JPRB, 0.42875E+00_JPRB, 0.43492E+00_JPRB ,&
&  0.44054E+00_JPRB, 0.44440E+00_JPRB, 0.44857E+00_JPRB, 0.45027E+00_JPRB, 0.45069E+00_JPRB, 0.44387E+00_JPRB, 0.43296E+00_JPRB ,&
&  0.42256E+00_JPRB, 0.41515E+00_JPRB, 0.41079E+00_JPRB, 0.40737E+00_JPRB, 0.40245E+00_JPRB, 0.39732E+00_JPRB, 0.39066E+00_JPRB ,&
&  0.38727E+00_JPRB, 0.38684E+00_JPRB, 0.38761E+00_JPRB, 0.38581E+00_JPRB, 0.38353E+00_JPRB, 0.37898E+00_JPRB, 0.37541E+00_JPRB ,&
&  0.37443E+00_JPRB, 0.37628E+00_JPRB, 0.37828E+00_JPRB, 0.37818E+00_JPRB, 0.37559E+00_JPRB, 0.37394E+00_JPRB, 0.37578E+00_JPRB ,&
&  0.38187E+00_JPRB, 0.38181E+00_JPRB, 0.38084E+00_JPRB, 0.38135E+00_JPRB, 0.38326E+00_JPRB, 0.38509E+00_JPRB, 0.38462E+00_JPRB ,&
&  0.38419E+00_JPRB, 0.38451E+00_JPRB, 0.38521E+00_JPRB, 0.38754E+00_JPRB, 0.38797E+00_JPRB, 0.38618E+00_JPRB, 0.38097E+00_JPRB ,&
&  0.37280E+00_JPRB, 0.36234E+00_JPRB, 0.35795E+00_JPRB, 0.35483E+00_JPRB, 0.35180E+00_JPRB, 0.35094E+00_JPRB, 0.35202E+00_JPRB ,&
&  0.35047E+00_JPRB, 0.34425E+00_JPRB, 0.34130E+00_JPRB, 0.34134E+00_JPRB, 0.33956E+00_JPRB, 0.34095E+00_JPRB, 0.34384E+00_JPRB ,&
&  0.35303E+00_JPRB, 0.35797E+00_JPRB, 0.35945E+00_JPRB, 0.35743E+00_JPRB, 0.35787E+00_JPRB, 0.35960E+00_JPRB, 0.36368E+00_JPRB ,&
&  0.36990E+00_JPRB, 0.37653E+00_JPRB, 0.38319E+00_JPRB, 0.39052E+00_JPRB, 0.40002E+00_JPRB, 0.40938E+00_JPRB, 0.41281E+00_JPRB ,&
&  0.41271E+00_JPRB, 0.41265E+00_JPRB, 0.41348E+00_JPRB, 0.41471E+00_JPRB, 0.41749E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.44360E+00_JPRB, 0.44405E+00_JPRB, 0.44302E+00_JPRB, 0.44214E+00_JPRB, 0.44112E+00_JPRB, 0.44021E+00_JPRB, 0.43845E+00_JPRB ,&
&  0.43373E+00_JPRB, 0.42765E+00_JPRB, 0.42415E+00_JPRB, 0.42250E+00_JPRB, 0.42804E+00_JPRB, 0.43635E+00_JPRB, 0.44378E+00_JPRB ,&
&  0.44325E+00_JPRB, 0.43468E+00_JPRB, 0.43225E+00_JPRB, 0.43441E+00_JPRB, 0.43710E+00_JPRB, 0.44004E+00_JPRB, 0.44395E+00_JPRB ,&
&  0.44822E+00_JPRB, 0.45259E+00_JPRB, 0.45445E+00_JPRB, 0.45221E+00_JPRB, 0.44615E+00_JPRB, 0.43290E+00_JPRB, 0.42117E+00_JPRB ,&
&  0.41352E+00_JPRB, 0.40762E+00_JPRB, 0.40435E+00_JPRB, 0.40337E+00_JPRB, 0.40125E+00_JPRB, 0.39745E+00_JPRB, 0.39203E+00_JPRB ,&
&  0.38726E+00_JPRB, 0.38493E+00_JPRB, 0.38688E+00_JPRB, 0.38840E+00_JPRB, 0.38930E+00_JPRB, 0.38931E+00_JPRB, 0.38688E+00_JPRB ,&
&  0.38583E+00_JPRB, 0.38592E+00_JPRB, 0.38625E+00_JPRB, 0.38850E+00_JPRB, 0.38691E+00_JPRB, 0.38556E+00_JPRB, 0.38800E+00_JPRB ,&
&  0.39265E+00_JPRB, 0.39653E+00_JPRB, 0.39990E+00_JPRB, 0.40227E+00_JPRB, 0.40307E+00_JPRB, 0.40500E+00_JPRB, 0.40641E+00_JPRB ,&
&  0.40578E+00_JPRB, 0.40266E+00_JPRB, 0.39977E+00_JPRB, 0.40075E+00_JPRB, 0.40348E+00_JPRB, 0.40648E+00_JPRB, 0.41025E+00_JPRB ,&
&  0.41109E+00_JPRB, 0.40610E+00_JPRB, 0.40100E+00_JPRB, 0.39640E+00_JPRB, 0.39213E+00_JPRB, 0.38868E+00_JPRB, 0.38687E+00_JPRB ,&
&  0.38617E+00_JPRB, 0.38308E+00_JPRB, 0.37721E+00_JPRB, 0.37227E+00_JPRB, 0.37112E+00_JPRB, 0.36947E+00_JPRB, 0.36512E+00_JPRB ,&
&  0.36500E+00_JPRB, 0.36702E+00_JPRB, 0.36577E+00_JPRB, 0.36390E+00_JPRB, 0.36591E+00_JPRB, 0.37098E+00_JPRB, 0.38123E+00_JPRB ,&
&  0.39500E+00_JPRB, 0.41097E+00_JPRB, 0.42577E+00_JPRB, 0.43887E+00_JPRB, 0.44674E+00_JPRB, 0.44865E+00_JPRB, 0.44686E+00_JPRB ,&
&  0.44596E+00_JPRB, 0.44634E+00_JPRB, 0.44499E+00_JPRB, 0.44299E+00_JPRB, 0.44275E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.46863E+00_JPRB, 0.46702E+00_JPRB, 0.46556E+00_JPRB, 0.46392E+00_JPRB, 0.46157E+00_JPRB, 0.45958E+00_JPRB, 0.45568E+00_JPRB ,&
&  0.45128E+00_JPRB, 0.44763E+00_JPRB, 0.44619E+00_JPRB, 0.44601E+00_JPRB, 0.44769E+00_JPRB, 0.45177E+00_JPRB, 0.45841E+00_JPRB ,&
&  0.45867E+00_JPRB, 0.45250E+00_JPRB, 0.44988E+00_JPRB, 0.45022E+00_JPRB, 0.45281E+00_JPRB, 0.45467E+00_JPRB, 0.45765E+00_JPRB ,&
&  0.45970E+00_JPRB, 0.46044E+00_JPRB, 0.46003E+00_JPRB, 0.45371E+00_JPRB, 0.43763E+00_JPRB, 0.42783E+00_JPRB, 0.41896E+00_JPRB ,&
&  0.41381E+00_JPRB, 0.40901E+00_JPRB, 0.40671E+00_JPRB, 0.40537E+00_JPRB, 0.40443E+00_JPRB, 0.40258E+00_JPRB, 0.40236E+00_JPRB ,&
&  0.40034E+00_JPRB, 0.39762E+00_JPRB, 0.39602E+00_JPRB, 0.39647E+00_JPRB, 0.39811E+00_JPRB, 0.39982E+00_JPRB, 0.40000E+00_JPRB ,&
&  0.40131E+00_JPRB, 0.40538E+00_JPRB, 0.40688E+00_JPRB, 0.40750E+00_JPRB, 0.40772E+00_JPRB, 0.40754E+00_JPRB, 0.40788E+00_JPRB ,&
&  0.41024E+00_JPRB, 0.41520E+00_JPRB, 0.41925E+00_JPRB, 0.42319E+00_JPRB, 0.42473E+00_JPRB, 0.42547E+00_JPRB, 0.42554E+00_JPRB ,&
&  0.42277E+00_JPRB, 0.41910E+00_JPRB, 0.41813E+00_JPRB, 0.42054E+00_JPRB, 0.42519E+00_JPRB, 0.43141E+00_JPRB, 0.43577E+00_JPRB ,&
&  0.43915E+00_JPRB, 0.44029E+00_JPRB, 0.43801E+00_JPRB, 0.43657E+00_JPRB, 0.43368E+00_JPRB, 0.42896E+00_JPRB, 0.42620E+00_JPRB ,&
&  0.42354E+00_JPRB, 0.42133E+00_JPRB, 0.41966E+00_JPRB, 0.41104E+00_JPRB, 0.40331E+00_JPRB, 0.39821E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39320E+00_JPRB, 0.39219E+00_JPRB, 0.39179E+00_JPRB, 0.39304E+00_JPRB, 0.39482E+00_JPRB, 0.40040E+00_JPRB, 0.41212E+00_JPRB ,&
&  0.43011E+00_JPRB, 0.44715E+00_JPRB, 0.45660E+00_JPRB, 0.46361E+00_JPRB, 0.46822E+00_JPRB, 0.47073E+00_JPRB, 0.47320E+00_JPRB ,&
&  0.47690E+00_JPRB, 0.47809E+00_JPRB, 0.47589E+00_JPRB, 0.47292E+00_JPRB, 0.47105E+00_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.49121E+00_JPRB, 0.48359E+00_JPRB, 0.47804E+00_JPRB, 0.47375E+00_JPRB, 0.47050E+00_JPRB, 0.46984E+00_JPRB, 0.46871E+00_JPRB ,&
&  0.46730E+00_JPRB, 0.46535E+00_JPRB, 0.46467E+00_JPRB, 0.46230E+00_JPRB, 0.45991E+00_JPRB, 0.46286E+00_JPRB, 0.46831E+00_JPRB ,&
&  0.46722E+00_JPRB, 0.46120E+00_JPRB, 0.45644E+00_JPRB, 0.45669E+00_JPRB, 0.46035E+00_JPRB, 0.46312E+00_JPRB, 0.46640E+00_JPRB ,&
&  0.46640E+00_JPRB, 0.46594E+00_JPRB, 0.46490E+00_JPRB, 0.45013E+00_JPRB, 0.43548E+00_JPRB, 0.42832E+00_JPRB, 0.42411E+00_JPRB ,&
&  0.42314E+00_JPRB, 0.42572E+00_JPRB, 0.41127E+00_JPRB, 0.41985E+00_JPRB, 0.41565E+00_JPRB, 0.40952E+00_JPRB, 0.40633E+00_JPRB ,&
&  0.40434E+00_JPRB, 0.40179E+00_JPRB, 0.40010E+00_JPRB, 0.40210E+00_JPRB, 0.40491E+00_JPRB, 0.40607E+00_JPRB, 0.40817E+00_JPRB ,&
&  0.40824E+00_JPRB, 0.41102E+00_JPRB, 0.41331E+00_JPRB, 0.41846E+00_JPRB, 0.42251E+00_JPRB, 0.42512E+00_JPRB, 0.42836E+00_JPRB ,&
&  0.43274E+00_JPRB, 0.43684E+00_JPRB, 0.44073E+00_JPRB, 0.44400E+00_JPRB, 0.44531E+00_JPRB, 0.44512E+00_JPRB, 0.44120E+00_JPRB ,&
&  0.43808E+00_JPRB, 0.43797E+00_JPRB, 0.44002E+00_JPRB, 0.44393E+00_JPRB, 0.44818E+00_JPRB, 0.45085E+00_JPRB, 0.45323E+00_JPRB ,&
&  0.45560E+00_JPRB, 0.45918E+00_JPRB, 0.45957E+00_JPRB, 0.45795E+00_JPRB, 0.45572E+00_JPRB, 0.45299E+00_JPRB, 0.44844E+00_JPRB ,&
&  0.44273E+00_JPRB, 0.43846E+00_JPRB, 0.43416E+00_JPRB, 0.42380E+00_JPRB, 0.41573E+00_JPRB, 0.41490E+00_JPRB, 0.41621E+00_JPRB ,&
&  0.41559E+00_JPRB, 0.41403E+00_JPRB, 0.41514E+00_JPRB, 0.42051E+00_JPRB, 0.41965E+00_JPRB, 0.42492E+00_JPRB, 0.43737E+00_JPRB ,&
&  0.45054E+00_JPRB, 0.46244E+00_JPRB, 0.47194E+00_JPRB, 0.48199E+00_JPRB, 0.49212E+00_JPRB, 0.49997E+00_JPRB, 0.50709E+00_JPRB ,&
&  0.51093E+00_JPRB, 0.50994E+00_JPRB, 0.50505E+00_JPRB, 0.50180E+00_JPRB, 0.49880E+00_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.53132E+00_JPRB, 0.51912E+00_JPRB, 0.50989E+00_JPRB, 0.50335E+00_JPRB, 0.49966E+00_JPRB, 0.49977E+00_JPRB, 0.49834E+00_JPRB ,&
&  0.49696E+00_JPRB, 0.49615E+00_JPRB, 0.49106E+00_JPRB, 0.48537E+00_JPRB, 0.48525E+00_JPRB, 0.49092E+00_JPRB, 0.49574E+00_JPRB ,&
&  0.49364E+00_JPRB, 0.48427E+00_JPRB, 0.47536E+00_JPRB, 0.47390E+00_JPRB, 0.47727E+00_JPRB, 0.48004E+00_JPRB, 0.47995E+00_JPRB ,&
&  0.48168E+00_JPRB, 0.48264E+00_JPRB, 0.47453E+00_JPRB, 0.45837E+00_JPRB, 0.45065E+00_JPRB, 0.44622E+00_JPRB, 0.45016E+00_JPRB ,&
&  0.45573E+00_JPRB, 0.44540E+00_JPRB, 0.45889E+00_JPRB, 0.45575E+00_JPRB, 0.45428E+00_JPRB, 0.44995E+00_JPRB, 0.44301E+00_JPRB ,&
&  0.43771E+00_JPRB, 0.43252E+00_JPRB, 0.42947E+00_JPRB, 0.42829E+00_JPRB, 0.42901E+00_JPRB, 0.43059E+00_JPRB, 0.43553E+00_JPRB ,&
&  0.43720E+00_JPRB, 0.43602E+00_JPRB, 0.43884E+00_JPRB, 0.44605E+00_JPRB, 0.45494E+00_JPRB, 0.46001E+00_JPRB, 0.46457E+00_JPRB ,&
&  0.46830E+00_JPRB, 0.47045E+00_JPRB, 0.47346E+00_JPRB, 0.47457E+00_JPRB, 0.47133E+00_JPRB, 0.47082E+00_JPRB, 0.47098E+00_JPRB ,&
&  0.47312E+00_JPRB, 0.47571E+00_JPRB, 0.47762E+00_JPRB, 0.47900E+00_JPRB, 0.48000E+00_JPRB, 0.47850E+00_JPRB, 0.47733E+00_JPRB ,&
&  0.47863E+00_JPRB, 0.48168E+00_JPRB, 0.48520E+00_JPRB, 0.48710E+00_JPRB, 0.48606E+00_JPRB, 0.47962E+00_JPRB, 0.47210E+00_JPRB ,&
&  0.46428E+00_JPRB, 0.45615E+00_JPRB, 0.44884E+00_JPRB, 0.44456E+00_JPRB, 0.44141E+00_JPRB, 0.43967E+00_JPRB, 0.43877E+00_JPRB ,&
&  0.43859E+00_JPRB, 0.43872E+00_JPRB, 0.44171E+00_JPRB, 0.44735E+00_JPRB, 0.44967E+00_JPRB, 0.45695E+00_JPRB, 0.46668E+00_JPRB ,&
&  0.47684E+00_JPRB, 0.48643E+00_JPRB, 0.49801E+00_JPRB, 0.51484E+00_JPRB, 0.53373E+00_JPRB, 0.54792E+00_JPRB, 0.55911E+00_JPRB ,&
&  0.56650E+00_JPRB, 0.56602E+00_JPRB, 0.55915E+00_JPRB, 0.55152E+00_JPRB, 0.54281E+00_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.57043E+00_JPRB, 0.55232E+00_JPRB, 0.53798E+00_JPRB, 0.52836E+00_JPRB, 0.52340E+00_JPRB, 0.52347E+00_JPRB, 0.52179E+00_JPRB ,&
&  0.52104E+00_JPRB, 0.51877E+00_JPRB, 0.51732E+00_JPRB, 0.52002E+00_JPRB, 0.52164E+00_JPRB, 0.52274E+00_JPRB, 0.52492E+00_JPRB ,&
&  0.52058E+00_JPRB, 0.50910E+00_JPRB, 0.49979E+00_JPRB, 0.49764E+00_JPRB, 0.49920E+00_JPRB, 0.49576E+00_JPRB, 0.49460E+00_JPRB ,&
&  0.49760E+00_JPRB, 0.48987E+00_JPRB, 0.47706E+00_JPRB, 0.46887E+00_JPRB, 0.46934E+00_JPRB, 0.46709E+00_JPRB, 0.46385E+00_JPRB ,&
&  0.48149E+00_JPRB, 0.43158E+00_JPRB, 0.47381E+00_JPRB, 0.47739E+00_JPRB, 0.48022E+00_JPRB, 0.48148E+00_JPRB, 0.47967E+00_JPRB ,&
&  0.47562E+00_JPRB, 0.47082E+00_JPRB, 0.46547E+00_JPRB, 0.46090E+00_JPRB, 0.45876E+00_JPRB, 0.45560E+00_JPRB, 0.45291E+00_JPRB ,&
&  0.45520E+00_JPRB, 0.45843E+00_JPRB, 0.46489E+00_JPRB, 0.47170E+00_JPRB, 0.47993E+00_JPRB, 0.48314E+00_JPRB, 0.48593E+00_JPRB ,&
&  0.48890E+00_JPRB, 0.49073E+00_JPRB, 0.49097E+00_JPRB, 0.49021E+00_JPRB, 0.48898E+00_JPRB, 0.49384E+00_JPRB, 0.49998E+00_JPRB ,&
&  0.50427E+00_JPRB, 0.50699E+00_JPRB, 0.51025E+00_JPRB, 0.51011E+00_JPRB, 0.50996E+00_JPRB, 0.50746E+00_JPRB, 0.50318E+00_JPRB ,&
&  0.49944E+00_JPRB, 0.49824E+00_JPRB, 0.49769E+00_JPRB, 0.49317E+00_JPRB, 0.48665E+00_JPRB, 0.48079E+00_JPRB, 0.47393E+00_JPRB ,&
&  0.46510E+00_JPRB, 0.45805E+00_JPRB, 0.45458E+00_JPRB, 0.45397E+00_JPRB, 0.45546E+00_JPRB, 0.45655E+00_JPRB, 0.45764E+00_JPRB ,&
&  0.46075E+00_JPRB, 0.46178E+00_JPRB, 0.46360E+00_JPRB, 0.46796E+00_JPRB, 0.47150E+00_JPRB, 0.47737E+00_JPRB, 0.48516E+00_JPRB ,&
&  0.49559E+00_JPRB, 0.50721E+00_JPRB, 0.51826E+00_JPRB, 0.53643E+00_JPRB, 0.56202E+00_JPRB, 0.58106E+00_JPRB, 0.59282E+00_JPRB ,&
&  0.60331E+00_JPRB, 0.60554E+00_JPRB, 0.60017E+00_JPRB, 0.59372E+00_JPRB, 0.58339E+00_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.61951E+00_JPRB, 0.60578E+00_JPRB, 0.58766E+00_JPRB, 0.57519E+00_JPRB, 0.56933E+00_JPRB, 0.56765E+00_JPRB, 0.56552E+00_JPRB ,&
&  0.56168E+00_JPRB, 0.56247E+00_JPRB, 0.56353E+00_JPRB, 0.56457E+00_JPRB, 0.56566E+00_JPRB, 0.56390E+00_JPRB, 0.55783E+00_JPRB ,&
&  0.55054E+00_JPRB, 0.54029E+00_JPRB, 0.53905E+00_JPRB, 0.53878E+00_JPRB, 0.53285E+00_JPRB, 0.52882E+00_JPRB, 0.52861E+00_JPRB ,&
&  0.52244E+00_JPRB, 0.51310E+00_JPRB, 0.49608E+00_JPRB, 0.49057E+00_JPRB, 0.49306E+00_JPRB, 0.49321E+00_JPRB, 0.49877E+00_JPRB ,&
&  0.47532E+00_JPRB, 0.44396E+00_JPRB, 0.48844E+00_JPRB, 0.51278E+00_JPRB, 0.51922E+00_JPRB, 0.52617E+00_JPRB, 0.53474E+00_JPRB ,&
&  0.53023E+00_JPRB, 0.53089E+00_JPRB, 0.52006E+00_JPRB, 0.50227E+00_JPRB, 0.49097E+00_JPRB, 0.48927E+00_JPRB, 0.49198E+00_JPRB ,&
&  0.50054E+00_JPRB, 0.50715E+00_JPRB, 0.51042E+00_JPRB, 0.51211E+00_JPRB, 0.51464E+00_JPRB, 0.51269E+00_JPRB, 0.51137E+00_JPRB ,&
&  0.51326E+00_JPRB, 0.51687E+00_JPRB, 0.51880E+00_JPRB, 0.52283E+00_JPRB, 0.53079E+00_JPRB, 0.54287E+00_JPRB, 0.55381E+00_JPRB ,&
&  0.56154E+00_JPRB, 0.56502E+00_JPRB, 0.56938E+00_JPRB, 0.57051E+00_JPRB, 0.56817E+00_JPRB, 0.56009E+00_JPRB, 0.55020E+00_JPRB ,&
&  0.53914E+00_JPRB, 0.53033E+00_JPRB, 0.52396E+00_JPRB, 0.51979E+00_JPRB, 0.51321E+00_JPRB, 0.50685E+00_JPRB, 0.49888E+00_JPRB ,&
&  0.49138E+00_JPRB, 0.48351E+00_JPRB, 0.48013E+00_JPRB, 0.47944E+00_JPRB, 0.48243E+00_JPRB, 0.48662E+00_JPRB, 0.49208E+00_JPRB ,&
&  0.49607E+00_JPRB, 0.49516E+00_JPRB, 0.49542E+00_JPRB, 0.49991E+00_JPRB, 0.50519E+00_JPRB, 0.51399E+00_JPRB, 0.52535E+00_JPRB ,&
&  0.53895E+00_JPRB, 0.55168E+00_JPRB, 0.56567E+00_JPRB, 0.59331E+00_JPRB, 0.62100E+00_JPRB, 0.64391E+00_JPRB, 0.65589E+00_JPRB ,&
&  0.66164E+00_JPRB, 0.65298E+00_JPRB, 0.65176E+00_JPRB, 0.64209E+00_JPRB, 0.62955E+00_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.61957E+00_JPRB, 0.61791E+00_JPRB, 0.60698E+00_JPRB, 0.60056E+00_JPRB, 0.59605E+00_JPRB, 0.59151E+00_JPRB, 0.58380E+00_JPRB ,&
&  0.57931E+00_JPRB, 0.57916E+00_JPRB, 0.57893E+00_JPRB, 0.58020E+00_JPRB, 0.58216E+00_JPRB, 0.57897E+00_JPRB, 0.57298E+00_JPRB ,&
&  0.56629E+00_JPRB, 0.55955E+00_JPRB, 0.56126E+00_JPRB, 0.56113E+00_JPRB, 0.55669E+00_JPRB, 0.55299E+00_JPRB, 0.54785E+00_JPRB ,&
&  0.53414E+00_JPRB, 0.52101E+00_JPRB, 0.51052E+00_JPRB, 0.51336E+00_JPRB, 0.51976E+00_JPRB, 0.51504E+00_JPRB, 0.52696E+00_JPRB ,&
&  0.56406E+00_JPRB, 0.47892E+00_JPRB, 0.50245E+00_JPRB, 0.54384E+00_JPRB, 0.55796E+00_JPRB, 0.57405E+00_JPRB, 0.58450E+00_JPRB ,&
&  0.58992E+00_JPRB, 0.59132E+00_JPRB, 0.57342E+00_JPRB, 0.55356E+00_JPRB, 0.53970E+00_JPRB, 0.53011E+00_JPRB, 0.52392E+00_JPRB ,&
&  0.52428E+00_JPRB, 0.53201E+00_JPRB, 0.53811E+00_JPRB, 0.53817E+00_JPRB, 0.53770E+00_JPRB, 0.53467E+00_JPRB, 0.53278E+00_JPRB ,&
&  0.53224E+00_JPRB, 0.53437E+00_JPRB, 0.53823E+00_JPRB, 0.54703E+00_JPRB, 0.55567E+00_JPRB, 0.56617E+00_JPRB, 0.58028E+00_JPRB ,&
&  0.59510E+00_JPRB, 0.60620E+00_JPRB, 0.61323E+00_JPRB, 0.61380E+00_JPRB, 0.60962E+00_JPRB, 0.60142E+00_JPRB, 0.58950E+00_JPRB ,&
&  0.57465E+00_JPRB, 0.55913E+00_JPRB, 0.54960E+00_JPRB, 0.54504E+00_JPRB, 0.53890E+00_JPRB, 0.53250E+00_JPRB, 0.52708E+00_JPRB ,&
&  0.52269E+00_JPRB, 0.51772E+00_JPRB, 0.51888E+00_JPRB, 0.52070E+00_JPRB, 0.52244E+00_JPRB, 0.52018E+00_JPRB, 0.51740E+00_JPRB ,&
&  0.51509E+00_JPRB, 0.51375E+00_JPRB, 0.51612E+00_JPRB, 0.52181E+00_JPRB, 0.53051E+00_JPRB, 0.54450E+00_JPRB, 0.56207E+00_JPRB ,&
&  0.57500E+00_JPRB, 0.58601E+00_JPRB, 0.59949E+00_JPRB, 0.62815E+00_JPRB, 0.64754E+00_JPRB, 0.66163E+00_JPRB, 0.66730E+00_JPRB ,&
&  0.66582E+00_JPRB, 0.66719E+00_JPRB, 0.52735E+00_JPRB, 0.59064E+00_JPRB, 0.60941E+00_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.63287E+00_JPRB, 0.63082E+00_JPRB, 0.62492E+00_JPRB, 0.62281E+00_JPRB, 0.62266E+00_JPRB, 0.61154E+00_JPRB, 0.60262E+00_JPRB ,&
&  0.59856E+00_JPRB, 0.60068E+00_JPRB, 0.60540E+00_JPRB, 0.60815E+00_JPRB, 0.60501E+00_JPRB, 0.59732E+00_JPRB, 0.59356E+00_JPRB ,&
&  0.58649E+00_JPRB, 0.57982E+00_JPRB, 0.57880E+00_JPRB, 0.57895E+00_JPRB, 0.57419E+00_JPRB, 0.56689E+00_JPRB, 0.56087E+00_JPRB ,&
&  0.54867E+00_JPRB, 0.54144E+00_JPRB, 0.54020E+00_JPRB, 0.54323E+00_JPRB, 0.54629E+00_JPRB, 0.54483E+00_JPRB, 0.55910E+00_JPRB ,&
&  0.58797E+00_JPRB, 0.49689E+00_JPRB, 0.50240E+00_JPRB, 0.57740E+00_JPRB, 0.61673E+00_JPRB, 0.63030E+00_JPRB, 0.64594E+00_JPRB ,&
&  0.66072E+00_JPRB, 0.65513E+00_JPRB, 0.64364E+00_JPRB, 0.61807E+00_JPRB, 0.60010E+00_JPRB, 0.59170E+00_JPRB, 0.58595E+00_JPRB ,&
&  0.58471E+00_JPRB, 0.58422E+00_JPRB, 0.58753E+00_JPRB, 0.58466E+00_JPRB, 0.58418E+00_JPRB, 0.57722E+00_JPRB, 0.57548E+00_JPRB ,&
&  0.57633E+00_JPRB, 0.58084E+00_JPRB, 0.58547E+00_JPRB, 0.58815E+00_JPRB, 0.59685E+00_JPRB, 0.61122E+00_JPRB, 0.63124E+00_JPRB ,&
&  0.65151E+00_JPRB, 0.66954E+00_JPRB, 0.67640E+00_JPRB, 0.67290E+00_JPRB, 0.66341E+00_JPRB, 0.65256E+00_JPRB, 0.63552E+00_JPRB ,&
&  0.61462E+00_JPRB, 0.59392E+00_JPRB, 0.57995E+00_JPRB, 0.57254E+00_JPRB, 0.56859E+00_JPRB, 0.56641E+00_JPRB, 0.56401E+00_JPRB ,&
&  0.56173E+00_JPRB, 0.56266E+00_JPRB, 0.56514E+00_JPRB, 0.56631E+00_JPRB, 0.56716E+00_JPRB, 0.56199E+00_JPRB, 0.55216E+00_JPRB ,&
&  0.54546E+00_JPRB, 0.54504E+00_JPRB, 0.55335E+00_JPRB, 0.56002E+00_JPRB, 0.56837E+00_JPRB, 0.58487E+00_JPRB, 0.60325E+00_JPRB ,&
&  0.61388E+00_JPRB, 0.62678E+00_JPRB, 0.65108E+00_JPRB, 0.64089E+00_JPRB, 0.67519E+00_JPRB, 0.67600E+00_JPRB, 0.67770E+00_JPRB ,&
&  0.66782E+00_JPRB, 0.65391E+00_JPRB, 0.64869E+00_JPRB, 0.55488E+00_JPRB, 0.61427E+00_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.64385E+00_JPRB, 0.64252E+00_JPRB, 0.63776E+00_JPRB, 0.63539E+00_JPRB, 0.63171E+00_JPRB, 0.62164E+00_JPRB, 0.61451E+00_JPRB ,&
&  0.61368E+00_JPRB, 0.61162E+00_JPRB, 0.61018E+00_JPRB, 0.60572E+00_JPRB, 0.59813E+00_JPRB, 0.59334E+00_JPRB, 0.59485E+00_JPRB ,&
&  0.59360E+00_JPRB, 0.59377E+00_JPRB, 0.59421E+00_JPRB, 0.59072E+00_JPRB, 0.58553E+00_JPRB, 0.57727E+00_JPRB, 0.56418E+00_JPRB ,&
&  0.54873E+00_JPRB, 0.54478E+00_JPRB, 0.55351E+00_JPRB, 0.56011E+00_JPRB, 0.56384E+00_JPRB, 0.56591E+00_JPRB, 0.57511E+00_JPRB ,&
&  0.59389E+00_JPRB, 0.52734E+00_JPRB, 0.57373E+00_JPRB, 0.65633E+00_JPRB, 0.70708E+00_JPRB, 0.71149E+00_JPRB, 0.71839E+00_JPRB ,&
&  0.70847E+00_JPRB, 0.69359E+00_JPRB, 0.67186E+00_JPRB, 0.64487E+00_JPRB, 0.62125E+00_JPRB, 0.60918E+00_JPRB, 0.60855E+00_JPRB ,&
&  0.60999E+00_JPRB, 0.61090E+00_JPRB, 0.61473E+00_JPRB, 0.61173E+00_JPRB, 0.60809E+00_JPRB, 0.60453E+00_JPRB, 0.60592E+00_JPRB ,&
&  0.60779E+00_JPRB, 0.60973E+00_JPRB, 0.61000E+00_JPRB, 0.61194E+00_JPRB, 0.61678E+00_JPRB, 0.62786E+00_JPRB, 0.64560E+00_JPRB ,&
&  0.66515E+00_JPRB, 0.68710E+00_JPRB, 0.69498E+00_JPRB, 0.68862E+00_JPRB, 0.68110E+00_JPRB, 0.67795E+00_JPRB, 0.65975E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.62207E+00_JPRB, 0.60875E+00_JPRB, 0.59789E+00_JPRB, 0.59154E+00_JPRB, 0.59063E+00_JPRB, 0.59074E+00_JPRB ,&
&  0.59096E+00_JPRB, 0.58961E+00_JPRB, 0.58696E+00_JPRB, 0.58622E+00_JPRB, 0.58605E+00_JPRB, 0.58090E+00_JPRB, 0.57505E+00_JPRB ,&
&  0.56851E+00_JPRB, 0.57167E+00_JPRB, 0.58354E+00_JPRB, 0.59044E+00_JPRB, 0.59674E+00_JPRB, 0.60936E+00_JPRB, 0.62476E+00_JPRB ,&
&  0.63327E+00_JPRB, 0.64760E+00_JPRB, 0.66149E+00_JPRB, 0.69934E+00_JPRB, 0.70104E+00_JPRB, 0.68564E+00_JPRB, 0.67618E+00_JPRB ,&
&  0.65752E+00_JPRB, 0.64024E+00_JPRB, 0.63040E+00_JPRB, 0.64200E+00_JPRB, 0.60789E+00_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.72337E+00_JPRB, 0.69205E+00_JPRB, 0.66098E+00_JPRB, 0.64092E+00_JPRB, 0.63283E+00_JPRB, 0.63042E+00_JPRB, 0.62941E+00_JPRB ,&
&  0.62633E+00_JPRB, 0.62239E+00_JPRB, 0.60843E+00_JPRB, 0.59894E+00_JPRB, 0.59294E+00_JPRB, 0.59235E+00_JPRB, 0.59177E+00_JPRB ,&
&  0.59385E+00_JPRB, 0.59468E+00_JPRB, 0.59159E+00_JPRB, 0.59136E+00_JPRB, 0.59343E+00_JPRB, 0.58950E+00_JPRB, 0.57494E+00_JPRB ,&
&  0.56070E+00_JPRB, 0.56343E+00_JPRB, 0.56793E+00_JPRB, 0.57015E+00_JPRB, 0.57517E+00_JPRB, 0.58430E+00_JPRB, 0.59871E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.54119E+00_JPRB, 0.65722E+00_JPRB, 0.74620E+00_JPRB, 0.79425E+00_JPRB, 0.82969E+00_JPRB, 0.81661E+00_JPRB ,&
&  0.79423E+00_JPRB, 0.74925E+00_JPRB, 0.70927E+00_JPRB, 0.66915E+00_JPRB, 0.64594E+00_JPRB, 0.63185E+00_JPRB, 0.62371E+00_JPRB ,&
&  0.62107E+00_JPRB, 0.62023E+00_JPRB, 0.61900E+00_JPRB, 0.61644E+00_JPRB, 0.61517E+00_JPRB, 0.61500E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61800E+00_JPRB, 0.62254E+00_JPRB, 0.62433E+00_JPRB, 0.62577E+00_JPRB, 0.62633E+00_JPRB, 0.65072E+00_JPRB, 0.68145E+00_JPRB ,&
&  0.70794E+00_JPRB, 0.72719E+00_JPRB, 0.72309E+00_JPRB, 0.71779E+00_JPRB, 0.71246E+00_JPRB, 0.69982E+00_JPRB, 0.68579E+00_JPRB ,&
&  0.67126E+00_JPRB, 0.65360E+00_JPRB, 0.63417E+00_JPRB, 0.62031E+00_JPRB, 0.61386E+00_JPRB, 0.61242E+00_JPRB, 0.61192E+00_JPRB ,&
&  0.61263E+00_JPRB, 0.61539E+00_JPRB, 0.61345E+00_JPRB, 0.61362E+00_JPRB, 0.61101E+00_JPRB, 0.60855E+00_JPRB, 0.60947E+00_JPRB ,&
&  0.60902E+00_JPRB, 0.60829E+00_JPRB, 0.61975E+00_JPRB, 0.62820E+00_JPRB, 0.63611E+00_JPRB, 0.64554E+00_JPRB, 0.65291E+00_JPRB ,&
&  0.66172E+00_JPRB, 0.68077E+00_JPRB, 0.65943E+00_JPRB, 0.66472E+00_JPRB, 0.73075E+00_JPRB, 0.70160E+00_JPRB, 0.66664E+00_JPRB ,&
&  0.63868E+00_JPRB, 0.62947E+00_JPRB, 0.63262E+00_JPRB, 0.66041E+00_JPRB, 0.71592E+00_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.68842E+00_JPRB, 0.65095E+00_JPRB, 0.63118E+00_JPRB, 0.62973E+00_JPRB, 0.63636E+00_JPRB, 0.63429E+00_JPRB, 0.63797E+00_JPRB ,&
&  0.63277E+00_JPRB, 0.62211E+00_JPRB, 0.60982E+00_JPRB, 0.59780E+00_JPRB, 0.59261E+00_JPRB, 0.58877E+00_JPRB, 0.58633E+00_JPRB ,&
&  0.58532E+00_JPRB, 0.58472E+00_JPRB, 0.58129E+00_JPRB, 0.58566E+00_JPRB, 0.58834E+00_JPRB, 0.58780E+00_JPRB, 0.56842E+00_JPRB ,&
&  0.56045E+00_JPRB, 0.56250E+00_JPRB, 0.56656E+00_JPRB, 0.57052E+00_JPRB, 0.57705E+00_JPRB, 0.58882E+00_JPRB, 0.59787E+00_JPRB ,&
&  0.59211E+00_JPRB, 0.56697E+00_JPRB, 0.71820E+00_JPRB, 0.83810E+00_JPRB, 0.87297E+00_JPRB, 0.87737E+00_JPRB, 0.85130E+00_JPRB ,&
&  0.82593E+00_JPRB, 0.80185E+00_JPRB, 0.77702E+00_JPRB, 0.72140E+00_JPRB, 0.67063E+00_JPRB, 0.64562E+00_JPRB, 0.63566E+00_JPRB ,&
&  0.63215E+00_JPRB, 0.63173E+00_JPRB, 0.63014E+00_JPRB, 0.62765E+00_JPRB, 0.62446E+00_JPRB, 0.62291E+00_JPRB, 0.62522E+00_JPRB ,&
&  0.62868E+00_JPRB, 0.62603E+00_JPRB, 0.62745E+00_JPRB, 0.61959E+00_JPRB, 0.64599E+00_JPRB, 0.68917E+00_JPRB, 0.73894E+00_JPRB ,&
&  0.75408E+00_JPRB, 0.76848E+00_JPRB, 0.75837E+00_JPRB, 0.73728E+00_JPRB, 0.71509E+00_JPRB, 0.69524E+00_JPRB, 0.68080E+00_JPRB ,&
&  0.66925E+00_JPRB, 0.65820E+00_JPRB, 0.64800E+00_JPRB, 0.64989E+00_JPRB, 0.65961E+00_JPRB, 0.65701E+00_JPRB, 0.65419E+00_JPRB ,&
&  0.65237E+00_JPRB, 0.65298E+00_JPRB, 0.65099E+00_JPRB, 0.64175E+00_JPRB, 0.62982E+00_JPRB, 0.62392E+00_JPRB, 0.62284E+00_JPRB ,&
&  0.62969E+00_JPRB, 0.63304E+00_JPRB, 0.65537E+00_JPRB, 0.66295E+00_JPRB, 0.66903E+00_JPRB, 0.67091E+00_JPRB, 0.67702E+00_JPRB ,&
&  0.69230E+00_JPRB, 0.68405E+00_JPRB, 0.68235E+00_JPRB, 0.68825E+00_JPRB, 0.68130E+00_JPRB, 0.70375E+00_JPRB, 0.65178E+00_JPRB ,&
&  0.62320E+00_JPRB, 0.61764E+00_JPRB, 0.62423E+00_JPRB, 0.64765E+00_JPRB, 0.69519E+00_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.65488E+00_JPRB, 0.63687E+00_JPRB, 0.63095E+00_JPRB, 0.63423E+00_JPRB, 0.63146E+00_JPRB, 0.63473E+00_JPRB, 0.64009E+00_JPRB ,&
&  0.63954E+00_JPRB, 0.62783E+00_JPRB, 0.61610E+00_JPRB, 0.60233E+00_JPRB, 0.59780E+00_JPRB, 0.59059E+00_JPRB, 0.58925E+00_JPRB ,&
&  0.58354E+00_JPRB, 0.58155E+00_JPRB, 0.57709E+00_JPRB, 0.57662E+00_JPRB, 0.57761E+00_JPRB, 0.57423E+00_JPRB, 0.55721E+00_JPRB ,&
&  0.55471E+00_JPRB, 0.55411E+00_JPRB, 0.56067E+00_JPRB, 0.57082E+00_JPRB, 0.58264E+00_JPRB, 0.58860E+00_JPRB, 0.58485E+00_JPRB ,&
&  0.58761E+00_JPRB, 0.56173E+00_JPRB, 0.83665E+00_JPRB, 0.93054E+00_JPRB, 0.96828E+00_JPRB, 0.91935E+00_JPRB, 0.87479E+00_JPRB ,&
&  0.84123E+00_JPRB, 0.82220E+00_JPRB, 0.81667E+00_JPRB, 0.77263E+00_JPRB, 0.71444E+00_JPRB, 0.67126E+00_JPRB, 0.65979E+00_JPRB ,&
&  0.65337E+00_JPRB, 0.64610E+00_JPRB, 0.64370E+00_JPRB, 0.64327E+00_JPRB, 0.63820E+00_JPRB, 0.63467E+00_JPRB, 0.63838E+00_JPRB ,&
&  0.63990E+00_JPRB, 0.64213E+00_JPRB, 0.63569E+00_JPRB, 0.63425E+00_JPRB, 0.63485E+00_JPRB, 0.60577E+00_JPRB, 0.67608E+00_JPRB ,&
&  0.84703E+00_JPRB, 0.84453E+00_JPRB, 0.79624E+00_JPRB, 0.73957E+00_JPRB, 0.70445E+00_JPRB, 0.69010E+00_JPRB, 0.68099E+00_JPRB ,&
&  0.67487E+00_JPRB, 0.66859E+00_JPRB, 0.67088E+00_JPRB, 0.68882E+00_JPRB, 0.69724E+00_JPRB, 0.69283E+00_JPRB, 0.68423E+00_JPRB ,&
&  0.68201E+00_JPRB, 0.68262E+00_JPRB, 0.67359E+00_JPRB, 0.65416E+00_JPRB, 0.64177E+00_JPRB, 0.64116E+00_JPRB, 0.63968E+00_JPRB ,&
&  0.63877E+00_JPRB, 0.64322E+00_JPRB, 0.65216E+00_JPRB, 0.64377E+00_JPRB, 0.66240E+00_JPRB, 0.69162E+00_JPRB, 0.69875E+00_JPRB ,&
&  0.67995E+00_JPRB, 0.66657E+00_JPRB, 0.68742E+00_JPRB, 0.67890E+00_JPRB, 0.67646E+00_JPRB, 0.69460E+00_JPRB, 0.64017E+00_JPRB ,&
&  0.61263E+00_JPRB, 0.61076E+00_JPRB, 0.63084E+00_JPRB, 0.65606E+00_JPRB, 0.69293E+00_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.63911E+00_JPRB, 0.63012E+00_JPRB, 0.62854E+00_JPRB, 0.62587E+00_JPRB, 0.61802E+00_JPRB, 0.60877E+00_JPRB, 0.61354E+00_JPRB ,&
&  0.61993E+00_JPRB, 0.62427E+00_JPRB, 0.61664E+00_JPRB, 0.60162E+00_JPRB, 0.59874E+00_JPRB, 0.59408E+00_JPRB, 0.58861E+00_JPRB ,&
&  0.58424E+00_JPRB, 0.58413E+00_JPRB, 0.57770E+00_JPRB, 0.57405E+00_JPRB, 0.57704E+00_JPRB, 0.57596E+00_JPRB, 0.56960E+00_JPRB ,&
&  0.56225E+00_JPRB, 0.55670E+00_JPRB, 0.56111E+00_JPRB, 0.56558E+00_JPRB, 0.57582E+00_JPRB, 0.57467E+00_JPRB, 0.56377E+00_JPRB ,&
&  0.57582E+00_JPRB, 0.50596E+00_JPRB, 0.92336E+00_JPRB, 0.11558E+01_JPRB, 0.11482E+01_JPRB, 0.10037E+01_JPRB, 0.89424E+00_JPRB ,&
&  0.89500E+00_JPRB, 0.85057E+00_JPRB, 0.82712E+00_JPRB, 0.78671E+00_JPRB, 0.74830E+00_JPRB, 0.70743E+00_JPRB, 0.69157E+00_JPRB ,&
&  0.68076E+00_JPRB, 0.66953E+00_JPRB, 0.66853E+00_JPRB, 0.66612E+00_JPRB, 0.66624E+00_JPRB, 0.66073E+00_JPRB, 0.66344E+00_JPRB ,&
&  0.66236E+00_JPRB, 0.65838E+00_JPRB, 0.65288E+00_JPRB, 0.66357E+00_JPRB, 0.64753E+00_JPRB, 0.64017E+00_JPRB, 0.66318E+00_JPRB ,&
&  0.73801E+00_JPRB, 0.85382E+00_JPRB, 0.79287E+00_JPRB, 0.72581E+00_JPRB, 0.69476E+00_JPRB, 0.68754E+00_JPRB, 0.68124E+00_JPRB ,&
&  0.67349E+00_JPRB, 0.67783E+00_JPRB, 0.70098E+00_JPRB, 0.70864E+00_JPRB, 0.69813E+00_JPRB, 0.68006E+00_JPRB, 0.68073E+00_JPRB ,&
&  0.68360E+00_JPRB, 0.67802E+00_JPRB, 0.66401E+00_JPRB, 0.65930E+00_JPRB, 0.65608E+00_JPRB, 0.64406E+00_JPRB, 0.63889E+00_JPRB ,&
&  0.63319E+00_JPRB, 0.64533E+00_JPRB, 0.64971E+00_JPRB, 0.65053E+00_JPRB, 0.65983E+00_JPRB, 0.67635E+00_JPRB, 0.68406E+00_JPRB ,&
&  0.68798E+00_JPRB, 0.69090E+00_JPRB, 0.67864E+00_JPRB, 0.68037E+00_JPRB, 0.64753E+00_JPRB, 0.67005E+00_JPRB, 0.63024E+00_JPRB ,&
&  0.61644E+00_JPRB, 0.61739E+00_JPRB, 0.62663E+00_JPRB, 0.63586E+00_JPRB, 0.65608E+00_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.62331E+00_JPRB, 0.61961E+00_JPRB, 0.62147E+00_JPRB, 0.61640E+00_JPRB, 0.60363E+00_JPRB, 0.59301E+00_JPRB, 0.59479E+00_JPRB ,&
&  0.59483E+00_JPRB, 0.59948E+00_JPRB, 0.60640E+00_JPRB, 0.59758E+00_JPRB, 0.59623E+00_JPRB, 0.59735E+00_JPRB, 0.59477E+00_JPRB ,&
&  0.58554E+00_JPRB, 0.58123E+00_JPRB, 0.57766E+00_JPRB, 0.57479E+00_JPRB, 0.57433E+00_JPRB, 0.56688E+00_JPRB, 0.56367E+00_JPRB ,&
&  0.56274E+00_JPRB, 0.56626E+00_JPRB, 0.56999E+00_JPRB, 0.57041E+00_JPRB, 0.57546E+00_JPRB, 0.56982E+00_JPRB, 0.55869E+00_JPRB ,&
&  0.56757E+00_JPRB, 0.83001E+00_JPRB, 0.82452E+00_JPRB, 0.13576E+01_JPRB, 0.12313E+01_JPRB, 0.10255E+01_JPRB, 0.86152E+00_JPRB ,&
&  0.85673E+00_JPRB, 0.88355E+00_JPRB, 0.84188E+00_JPRB, 0.79636E+00_JPRB, 0.75980E+00_JPRB, 0.72349E+00_JPRB, 0.69325E+00_JPRB ,&
&  0.67256E+00_JPRB, 0.66472E+00_JPRB, 0.66926E+00_JPRB, 0.67496E+00_JPRB, 0.68328E+00_JPRB, 0.68953E+00_JPRB, 0.68908E+00_JPRB ,&
&  0.68813E+00_JPRB, 0.68889E+00_JPRB, 0.69638E+00_JPRB, 0.72580E+00_JPRB, 0.67567E+00_JPRB, 0.66859E+00_JPRB, 0.70045E+00_JPRB ,&
&  0.72184E+00_JPRB, 0.83554E+00_JPRB, 0.76891E+00_JPRB, 0.71406E+00_JPRB, 0.69968E+00_JPRB, 0.69165E+00_JPRB, 0.68251E+00_JPRB ,&
&  0.68048E+00_JPRB, 0.69307E+00_JPRB, 0.71819E+00_JPRB, 0.70452E+00_JPRB, 0.67857E+00_JPRB, 0.67205E+00_JPRB, 0.67047E+00_JPRB ,&
&  0.67119E+00_JPRB, 0.66028E+00_JPRB, 0.67646E+00_JPRB, 0.68201E+00_JPRB, 0.67913E+00_JPRB, 0.66800E+00_JPRB, 0.65661E+00_JPRB ,&
&  0.65258E+00_JPRB, 0.66433E+00_JPRB, 0.65478E+00_JPRB, 0.64099E+00_JPRB, 0.64457E+00_JPRB, 0.64700E+00_JPRB, 0.64451E+00_JPRB ,&
&  0.68199E+00_JPRB, 0.69045E+00_JPRB, 0.67474E+00_JPRB, 0.66440E+00_JPRB, 0.65060E+00_JPRB, 0.67088E+00_JPRB, 0.62700E+00_JPRB ,&
&  0.61769E+00_JPRB, 0.61596E+00_JPRB, 0.62069E+00_JPRB, 0.63075E+00_JPRB, 0.63422E+00_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.62116E+00_JPRB, 0.61816E+00_JPRB, 0.62009E+00_JPRB, 0.60696E+00_JPRB, 0.59049E+00_JPRB, 0.58231E+00_JPRB, 0.58254E+00_JPRB ,&
&  0.58618E+00_JPRB, 0.59738E+00_JPRB, 0.60485E+00_JPRB, 0.59057E+00_JPRB, 0.58171E+00_JPRB, 0.58605E+00_JPRB, 0.58439E+00_JPRB ,&
&  0.58033E+00_JPRB, 0.57522E+00_JPRB, 0.57363E+00_JPRB, 0.57394E+00_JPRB, 0.57182E+00_JPRB, 0.56297E+00_JPRB, 0.56334E+00_JPRB ,&
&  0.57143E+00_JPRB, 0.57879E+00_JPRB, 0.58184E+00_JPRB, 0.58199E+00_JPRB, 0.58691E+00_JPRB, 0.57915E+00_JPRB, 0.57328E+00_JPRB ,&
&  0.58382E+00_JPRB, 0.87790E+00_JPRB, 0.92146E+00_JPRB, 0.15688E+01_JPRB, 0.13386E+01_JPRB, 0.11041E+01_JPRB, 0.92155E+00_JPRB ,&
&  0.86068E+00_JPRB, 0.90643E+00_JPRB, 0.83592E+00_JPRB, 0.75581E+00_JPRB, 0.71437E+00_JPRB, 0.70186E+00_JPRB, 0.68676E+00_JPRB ,&
&  0.67022E+00_JPRB, 0.68115E+00_JPRB, 0.69139E+00_JPRB, 0.70218E+00_JPRB, 0.71220E+00_JPRB, 0.71332E+00_JPRB, 0.72117E+00_JPRB ,&
&  0.72311E+00_JPRB, 0.73522E+00_JPRB, 0.75953E+00_JPRB, 0.73610E+00_JPRB, 0.69906E+00_JPRB, 0.71179E+00_JPRB, 0.71822E+00_JPRB ,&
&  0.72956E+00_JPRB, 0.78941E+00_JPRB, 0.74148E+00_JPRB, 0.70970E+00_JPRB, 0.68477E+00_JPRB, 0.68180E+00_JPRB, 0.67632E+00_JPRB ,&
&  0.67107E+00_JPRB, 0.68209E+00_JPRB, 0.68775E+00_JPRB, 0.67936E+00_JPRB, 0.66896E+00_JPRB, 0.66276E+00_JPRB, 0.65408E+00_JPRB ,&
&  0.65517E+00_JPRB, 0.67649E+00_JPRB, 0.68207E+00_JPRB, 0.68687E+00_JPRB, 0.69352E+00_JPRB, 0.69596E+00_JPRB, 0.68568E+00_JPRB ,&
&  0.68198E+00_JPRB, 0.68422E+00_JPRB, 0.65422E+00_JPRB, 0.64308E+00_JPRB, 0.64939E+00_JPRB, 0.64531E+00_JPRB, 0.64464E+00_JPRB ,&
&  0.66940E+00_JPRB, 0.67762E+00_JPRB, 0.66091E+00_JPRB, 0.63314E+00_JPRB, 0.64674E+00_JPRB, 0.66732E+00_JPRB, 0.62970E+00_JPRB ,&
&  0.62552E+00_JPRB, 0.62455E+00_JPRB, 0.62787E+00_JPRB, 0.62796E+00_JPRB, 0.62466E+00_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.59733E+00_JPRB, 0.59401E+00_JPRB, 0.59048E+00_JPRB, 0.55810E+00_JPRB, 0.53309E+00_JPRB, 0.52611E+00_JPRB, 0.54308E+00_JPRB ,&
&  0.57511E+00_JPRB, 0.59696E+00_JPRB, 0.59232E+00_JPRB, 0.57661E+00_JPRB, 0.56834E+00_JPRB, 0.56842E+00_JPRB, 0.57227E+00_JPRB ,&
&  0.56871E+00_JPRB, 0.55696E+00_JPRB, 0.55071E+00_JPRB, 0.55789E+00_JPRB, 0.55996E+00_JPRB, 0.56670E+00_JPRB, 0.57749E+00_JPRB ,&
&  0.59208E+00_JPRB, 0.59915E+00_JPRB, 0.59895E+00_JPRB, 0.59915E+00_JPRB, 0.61074E+00_JPRB, 0.60231E+00_JPRB, 0.60000E+00_JPRB ,&
&  0.64258E+00_JPRB, 0.95323E+00_JPRB, 0.11264E+01_JPRB, 0.14200E+01_JPRB, 0.12648E+01_JPRB, 0.10077E+01_JPRB, 0.88242E+00_JPRB ,&
&  0.78123E+00_JPRB, 0.70003E+00_JPRB, 0.76463E+00_JPRB, 0.70687E+00_JPRB, 0.70196E+00_JPRB, 0.68614E+00_JPRB, 0.68204E+00_JPRB ,&
&  0.69640E+00_JPRB, 0.71767E+00_JPRB, 0.73049E+00_JPRB, 0.73696E+00_JPRB, 0.74302E+00_JPRB, 0.74795E+00_JPRB, 0.75661E+00_JPRB ,&
&  0.76399E+00_JPRB, 0.78186E+00_JPRB, 0.79946E+00_JPRB, 0.75582E+00_JPRB, 0.68769E+00_JPRB, 0.71862E+00_JPRB, 0.72440E+00_JPRB ,&
&  0.73624E+00_JPRB, 0.74351E+00_JPRB, 0.72432E+00_JPRB, 0.69607E+00_JPRB, 0.64538E+00_JPRB, 0.67724E+00_JPRB, 0.66645E+00_JPRB ,&
&  0.64774E+00_JPRB, 0.66027E+00_JPRB, 0.65978E+00_JPRB, 0.66719E+00_JPRB, 0.66658E+00_JPRB, 0.65979E+00_JPRB, 0.65982E+00_JPRB ,&
&  0.66947E+00_JPRB, 0.67682E+00_JPRB, 0.68557E+00_JPRB, 0.69303E+00_JPRB, 0.70501E+00_JPRB, 0.71247E+00_JPRB, 0.70761E+00_JPRB ,&
&  0.69685E+00_JPRB, 0.69928E+00_JPRB, 0.67375E+00_JPRB, 0.67800E+00_JPRB, 0.67540E+00_JPRB, 0.67017E+00_JPRB, 0.64870E+00_JPRB ,&
&  0.64944E+00_JPRB, 0.66089E+00_JPRB, 0.64940E+00_JPRB, 0.64087E+00_JPRB, 0.65250E+00_JPRB, 0.65112E+00_JPRB, 0.62918E+00_JPRB ,&
&  0.63089E+00_JPRB, 0.62723E+00_JPRB, 0.62997E+00_JPRB, 0.61445E+00_JPRB, 0.60462E+00_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.58579E+00_JPRB, 0.57872E+00_JPRB, 0.55625E+00_JPRB, 0.50585E+00_JPRB, 0.49773E+00_JPRB, 0.51579E+00_JPRB, 0.57396E+00_JPRB ,&
&  0.59721E+00_JPRB, 0.60049E+00_JPRB, 0.58600E+00_JPRB, 0.56650E+00_JPRB, 0.55334E+00_JPRB, 0.54871E+00_JPRB, 0.55813E+00_JPRB ,&
&  0.56536E+00_JPRB, 0.55354E+00_JPRB, 0.55237E+00_JPRB, 0.55873E+00_JPRB, 0.56745E+00_JPRB, 0.58335E+00_JPRB, 0.59885E+00_JPRB ,&
&  0.61187E+00_JPRB, 0.61759E+00_JPRB, 0.61710E+00_JPRB, 0.63119E+00_JPRB, 0.64961E+00_JPRB, 0.65120E+00_JPRB, 0.67098E+00_JPRB ,&
&  0.81878E+00_JPRB, 0.10751E+01_JPRB, 0.60164E+00_JPRB, 0.92036E+00_JPRB, 0.10333E+01_JPRB, 0.90016E+00_JPRB, 0.80002E+00_JPRB ,&
&  0.72073E+00_JPRB, 0.64129E+00_JPRB, 0.66030E+00_JPRB, 0.71184E+00_JPRB, 0.70255E+00_JPRB, 0.71675E+00_JPRB, 0.72509E+00_JPRB ,&
&  0.74252E+00_JPRB, 0.75279E+00_JPRB, 0.76035E+00_JPRB, 0.76499E+00_JPRB, 0.76765E+00_JPRB, 0.77930E+00_JPRB, 0.78691E+00_JPRB ,&
&  0.78798E+00_JPRB, 0.80102E+00_JPRB, 0.79022E+00_JPRB, 0.69573E+00_JPRB, 0.68135E+00_JPRB, 0.70772E+00_JPRB, 0.69305E+00_JPRB ,&
&  0.66596E+00_JPRB, 0.70032E+00_JPRB, 0.71011E+00_JPRB, 0.67232E+00_JPRB, 0.62602E+00_JPRB, 0.61015E+00_JPRB, 0.64624E+00_JPRB ,&
&  0.63837E+00_JPRB, 0.63423E+00_JPRB, 0.63105E+00_JPRB, 0.63817E+00_JPRB, 0.63882E+00_JPRB, 0.64031E+00_JPRB, 0.64494E+00_JPRB ,&
&  0.66005E+00_JPRB, 0.67336E+00_JPRB, 0.68366E+00_JPRB, 0.69063E+00_JPRB, 0.70234E+00_JPRB, 0.71388E+00_JPRB, 0.71230E+00_JPRB ,&
&  0.71798E+00_JPRB, 0.71940E+00_JPRB, 0.72211E+00_JPRB, 0.71255E+00_JPRB, 0.69085E+00_JPRB, 0.67817E+00_JPRB, 0.67049E+00_JPRB ,&
&  0.66593E+00_JPRB, 0.66003E+00_JPRB, 0.64087E+00_JPRB, 0.63350E+00_JPRB, 0.67030E+00_JPRB, 0.66578E+00_JPRB, 0.71538E+00_JPRB ,&
&  0.78107E+00_JPRB, 0.83996E+00_JPRB, 0.80172E+00_JPRB, 0.61128E+00_JPRB, 0.59143E+00_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.56927E+00_JPRB, 0.54427E+00_JPRB, 0.49694E+00_JPRB, 0.46859E+00_JPRB, 0.49367E+00_JPRB, 0.55630E+00_JPRB, 0.60347E+00_JPRB ,&
&  0.61881E+00_JPRB, 0.61303E+00_JPRB, 0.60684E+00_JPRB, 0.60087E+00_JPRB, 0.60651E+00_JPRB, 0.59466E+00_JPRB, 0.59506E+00_JPRB ,&
&  0.59777E+00_JPRB, 0.59326E+00_JPRB, 0.59564E+00_JPRB, 0.59796E+00_JPRB, 0.60260E+00_JPRB, 0.61611E+00_JPRB, 0.63171E+00_JPRB ,&
&  0.64414E+00_JPRB, 0.65322E+00_JPRB, 0.66885E+00_JPRB, 0.69054E+00_JPRB, 0.72334E+00_JPRB, 0.78490E+00_JPRB, 0.89514E+00_JPRB ,&
&  0.11132E+01_JPRB, 0.79301E+00_JPRB, 0.53123E+00_JPRB, 0.87262E+00_JPRB, 0.90586E+00_JPRB, 0.83983E+00_JPRB, 0.73845E+00_JPRB ,&
&  0.70730E+00_JPRB, 0.68419E+00_JPRB, 0.69668E+00_JPRB, 0.74325E+00_JPRB, 0.75548E+00_JPRB, 0.78029E+00_JPRB, 0.80014E+00_JPRB ,&
&  0.79972E+00_JPRB, 0.79465E+00_JPRB, 0.79174E+00_JPRB, 0.79066E+00_JPRB, 0.79605E+00_JPRB, 0.80977E+00_JPRB, 0.81058E+00_JPRB ,&
&  0.80989E+00_JPRB, 0.81204E+00_JPRB, 0.78835E+00_JPRB, 0.66603E+00_JPRB, 0.66207E+00_JPRB, 0.66812E+00_JPRB, 0.66115E+00_JPRB ,&
&  0.63529E+00_JPRB, 0.65948E+00_JPRB, 0.67959E+00_JPRB, 0.65567E+00_JPRB, 0.62074E+00_JPRB, 0.59396E+00_JPRB, 0.63633E+00_JPRB ,&
&  0.64309E+00_JPRB, 0.61459E+00_JPRB, 0.58907E+00_JPRB, 0.57643E+00_JPRB, 0.59911E+00_JPRB, 0.61736E+00_JPRB, 0.61365E+00_JPRB ,&
&  0.62472E+00_JPRB, 0.64238E+00_JPRB, 0.65460E+00_JPRB, 0.66651E+00_JPRB, 0.68467E+00_JPRB, 0.70388E+00_JPRB, 0.71350E+00_JPRB ,&
&  0.72179E+00_JPRB, 0.72047E+00_JPRB, 0.72339E+00_JPRB, 0.71687E+00_JPRB, 0.68574E+00_JPRB, 0.65298E+00_JPRB, 0.65674E+00_JPRB ,&
&  0.65720E+00_JPRB, 0.66549E+00_JPRB, 0.66084E+00_JPRB, 0.67384E+00_JPRB, 0.68765E+00_JPRB, 0.71872E+00_JPRB, 0.75486E+00_JPRB ,&
&  0.78439E+00_JPRB, 0.77409E+00_JPRB, 0.69587E+00_JPRB, 0.60320E+00_JPRB, 0.58350E+00_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.53833E+00_JPRB, 0.48289E+00_JPRB, 0.43455E+00_JPRB, 0.45227E+00_JPRB, 0.51393E+00_JPRB, 0.56536E+00_JPRB, 0.59079E+00_JPRB ,&
&  0.61887E+00_JPRB, 0.62345E+00_JPRB, 0.62611E+00_JPRB, 0.63080E+00_JPRB, 0.64351E+00_JPRB, 0.63807E+00_JPRB, 0.64186E+00_JPRB ,&
&  0.63370E+00_JPRB, 0.61905E+00_JPRB, 0.61247E+00_JPRB, 0.62234E+00_JPRB, 0.63662E+00_JPRB, 0.65351E+00_JPRB, 0.67297E+00_JPRB ,&
&  0.69375E+00_JPRB, 0.71235E+00_JPRB, 0.74244E+00_JPRB, 0.77884E+00_JPRB, 0.82487E+00_JPRB, 0.87376E+00_JPRB, 0.90687E+00_JPRB ,&
&  0.67632E+00_JPRB, 0.43906E+00_JPRB, 0.74214E+00_JPRB, 0.88780E+00_JPRB, 0.82254E+00_JPRB, 0.78065E+00_JPRB, 0.74719E+00_JPRB ,&
&  0.71297E+00_JPRB, 0.70276E+00_JPRB, 0.70218E+00_JPRB, 0.81082E+00_JPRB, 0.82307E+00_JPRB, 0.82524E+00_JPRB, 0.82624E+00_JPRB ,&
&  0.82680E+00_JPRB, 0.82557E+00_JPRB, 0.82865E+00_JPRB, 0.83960E+00_JPRB, 0.85009E+00_JPRB, 0.86078E+00_JPRB, 0.85983E+00_JPRB ,&
&  0.85526E+00_JPRB, 0.84261E+00_JPRB, 0.81248E+00_JPRB, 0.64447E+00_JPRB, 0.63605E+00_JPRB, 0.64015E+00_JPRB, 0.62760E+00_JPRB ,&
&  0.62553E+00_JPRB, 0.61136E+00_JPRB, 0.61598E+00_JPRB, 0.65579E+00_JPRB, 0.63752E+00_JPRB, 0.57605E+00_JPRB, 0.58913E+00_JPRB ,&
&  0.60572E+00_JPRB, 0.59383E+00_JPRB, 0.56621E+00_JPRB, 0.55301E+00_JPRB, 0.58005E+00_JPRB, 0.58215E+00_JPRB, 0.59003E+00_JPRB ,&
&  0.58565E+00_JPRB, 0.60917E+00_JPRB, 0.63147E+00_JPRB, 0.64799E+00_JPRB, 0.66666E+00_JPRB, 0.68860E+00_JPRB, 0.68822E+00_JPRB ,&
&  0.69311E+00_JPRB, 0.68875E+00_JPRB, 0.69340E+00_JPRB, 0.68225E+00_JPRB, 0.67165E+00_JPRB, 0.64492E+00_JPRB, 0.64027E+00_JPRB ,&
&  0.64732E+00_JPRB, 0.65611E+00_JPRB, 0.66026E+00_JPRB, 0.65735E+00_JPRB, 0.68856E+00_JPRB, 0.69538E+00_JPRB, 0.70720E+00_JPRB ,&
&  0.70618E+00_JPRB, 0.68674E+00_JPRB, 0.62961E+00_JPRB, 0.58644E+00_JPRB, 0.56954E+00_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.45599E+00_JPRB, 0.40491E+00_JPRB, 0.39965E+00_JPRB, 0.44058E+00_JPRB, 0.48173E+00_JPRB, 0.52332E+00_JPRB, 0.55967E+00_JPRB ,&
&  0.59425E+00_JPRB, 0.61142E+00_JPRB, 0.64735E+00_JPRB, 0.65789E+00_JPRB, 0.66251E+00_JPRB, 0.65122E+00_JPRB, 0.63786E+00_JPRB ,&
&  0.64085E+00_JPRB, 0.64230E+00_JPRB, 0.64095E+00_JPRB, 0.65275E+00_JPRB, 0.66504E+00_JPRB, 0.68106E+00_JPRB, 0.70405E+00_JPRB ,&
&  0.72627E+00_JPRB, 0.75326E+00_JPRB, 0.77915E+00_JPRB, 0.81009E+00_JPRB, 0.82405E+00_JPRB, 0.82035E+00_JPRB, 0.75811E+00_JPRB ,&
&  0.44363E+00_JPRB, 0.69931E+00_JPRB, 0.85371E+00_JPRB, 0.83338E+00_JPRB, 0.80022E+00_JPRB, 0.76439E+00_JPRB, 0.76493E+00_JPRB ,&
&  0.73347E+00_JPRB, 0.69402E+00_JPRB, 0.70774E+00_JPRB, 0.85713E+00_JPRB, 0.83396E+00_JPRB, 0.83298E+00_JPRB, 0.83615E+00_JPRB ,&
&  0.84891E+00_JPRB, 0.85882E+00_JPRB, 0.86664E+00_JPRB, 0.87524E+00_JPRB, 0.89885E+00_JPRB, 0.91670E+00_JPRB, 0.92286E+00_JPRB ,&
&  0.91390E+00_JPRB, 0.89982E+00_JPRB, 0.89092E+00_JPRB, 0.70360E+00_JPRB, 0.67863E+00_JPRB, 0.65083E+00_JPRB, 0.61924E+00_JPRB ,&
&  0.59654E+00_JPRB, 0.59915E+00_JPRB, 0.62145E+00_JPRB, 0.68282E+00_JPRB, 0.65201E+00_JPRB, 0.62426E+00_JPRB, 0.57777E+00_JPRB ,&
&  0.58519E+00_JPRB, 0.58568E+00_JPRB, 0.58306E+00_JPRB, 0.56713E+00_JPRB, 0.56700E+00_JPRB, 0.54538E+00_JPRB, 0.58167E+00_JPRB ,&
&  0.59578E+00_JPRB, 0.58325E+00_JPRB, 0.58272E+00_JPRB, 0.60204E+00_JPRB, 0.62909E+00_JPRB, 0.65638E+00_JPRB, 0.66260E+00_JPRB ,&
&  0.65313E+00_JPRB, 0.63737E+00_JPRB, 0.63904E+00_JPRB, 0.64075E+00_JPRB, 0.64174E+00_JPRB, 0.64253E+00_JPRB, 0.63394E+00_JPRB ,&
&  0.62928E+00_JPRB, 0.64779E+00_JPRB, 0.69022E+00_JPRB, 0.64941E+00_JPRB, 0.69911E+00_JPRB, 0.63569E+00_JPRB, 0.61947E+00_JPRB ,&
&  0.60346E+00_JPRB, 0.58388E+00_JPRB, 0.53784E+00_JPRB, 0.51566E+00_JPRB, 0.48868E+00_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.37088E+00_JPRB, 0.35391E+00_JPRB, 0.36289E+00_JPRB, 0.38618E+00_JPRB, 0.41370E+00_JPRB, 0.45490E+00_JPRB, 0.50312E+00_JPRB ,&
&  0.54300E+00_JPRB, 0.57939E+00_JPRB, 0.64189E+00_JPRB, 0.64761E+00_JPRB, 0.64833E+00_JPRB, 0.67221E+00_JPRB, 0.67941E+00_JPRB ,&
&  0.68508E+00_JPRB, 0.67896E+00_JPRB, 0.67898E+00_JPRB, 0.68045E+00_JPRB, 0.69596E+00_JPRB, 0.71458E+00_JPRB, 0.72505E+00_JPRB ,&
&  0.74518E+00_JPRB, 0.76945E+00_JPRB, 0.81340E+00_JPRB, 0.85770E+00_JPRB, 0.89743E+00_JPRB, 0.88927E+00_JPRB, 0.86333E+00_JPRB ,&
&  0.64505E+00_JPRB, 0.82941E+00_JPRB, 0.82286E+00_JPRB, 0.80509E+00_JPRB, 0.77177E+00_JPRB, 0.73914E+00_JPRB, 0.73190E+00_JPRB ,&
&  0.72548E+00_JPRB, 0.69405E+00_JPRB, 0.70681E+00_JPRB, 0.76144E+00_JPRB, 0.83164E+00_JPRB, 0.83844E+00_JPRB, 0.84035E+00_JPRB ,&
&  0.85953E+00_JPRB, 0.87790E+00_JPRB, 0.89646E+00_JPRB, 0.92069E+00_JPRB, 0.94764E+00_JPRB, 0.97071E+00_JPRB, 0.98798E+00_JPRB ,&
&  0.99429E+00_JPRB, 0.98942E+00_JPRB, 0.10180E+01_JPRB, 0.87173E+00_JPRB, 0.78415E+00_JPRB, 0.73106E+00_JPRB, 0.65048E+00_JPRB ,&
&  0.60392E+00_JPRB, 0.58410E+00_JPRB, 0.63522E+00_JPRB, 0.69739E+00_JPRB, 0.66423E+00_JPRB, 0.63099E+00_JPRB, 0.58288E+00_JPRB ,&
&  0.57502E+00_JPRB, 0.57350E+00_JPRB, 0.57577E+00_JPRB, 0.56700E+00_JPRB, 0.57308E+00_JPRB, 0.56151E+00_JPRB, 0.56713E+00_JPRB ,&
&  0.58090E+00_JPRB, 0.57054E+00_JPRB, 0.56299E+00_JPRB, 0.57002E+00_JPRB, 0.59297E+00_JPRB, 0.62258E+00_JPRB, 0.62385E+00_JPRB ,&
&  0.62856E+00_JPRB, 0.56981E+00_JPRB, 0.55119E+00_JPRB, 0.56887E+00_JPRB, 0.57773E+00_JPRB, 0.58648E+00_JPRB, 0.61222E+00_JPRB ,&
&  0.69917E+00_JPRB, 0.80547E+00_JPRB, 0.90179E+00_JPRB, 0.90316E+00_JPRB, 0.96312E+00_JPRB, 0.75077E+00_JPRB, 0.55102E+00_JPRB ,&
&  0.51604E+00_JPRB, 0.50347E+00_JPRB, 0.46100E+00_JPRB, 0.43223E+00_JPRB, 0.39967E+00_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.36494E+00_JPRB, 0.35885E+00_JPRB, 0.36756E+00_JPRB, 0.38355E+00_JPRB, 0.39502E+00_JPRB, 0.42183E+00_JPRB, 0.45383E+00_JPRB ,&
&  0.48869E+00_JPRB, 0.53062E+00_JPRB, 0.56142E+00_JPRB, 0.57716E+00_JPRB, 0.60928E+00_JPRB, 0.65850E+00_JPRB, 0.67014E+00_JPRB ,&
&  0.70428E+00_JPRB, 0.71716E+00_JPRB, 0.71216E+00_JPRB, 0.70488E+00_JPRB, 0.71335E+00_JPRB, 0.73690E+00_JPRB, 0.75833E+00_JPRB ,&
&  0.77057E+00_JPRB, 0.79407E+00_JPRB, 0.83584E+00_JPRB, 0.85871E+00_JPRB, 0.86209E+00_JPRB, 0.83765E+00_JPRB, 0.65719E+00_JPRB ,&
&  0.83980E+00_JPRB, 0.84292E+00_JPRB, 0.83741E+00_JPRB, 0.79791E+00_JPRB, 0.76453E+00_JPRB, 0.73311E+00_JPRB, 0.72026E+00_JPRB ,&
&  0.71145E+00_JPRB, 0.69516E+00_JPRB, 0.68608E+00_JPRB, 0.69704E+00_JPRB, 0.80132E+00_JPRB, 0.82239E+00_JPRB, 0.84369E+00_JPRB ,&
&  0.84459E+00_JPRB, 0.85239E+00_JPRB, 0.87810E+00_JPRB, 0.90676E+00_JPRB, 0.93687E+00_JPRB, 0.95876E+00_JPRB, 0.97677E+00_JPRB ,&
&  0.99754E+00_JPRB, 0.10120E+01_JPRB, 0.10332E+01_JPRB, 0.92153E+00_JPRB, 0.85166E+00_JPRB, 0.81725E+00_JPRB, 0.72815E+00_JPRB ,&
&  0.61157E+00_JPRB, 0.58419E+00_JPRB, 0.63420E+00_JPRB, 0.68480E+00_JPRB, 0.64148E+00_JPRB, 0.60441E+00_JPRB, 0.56961E+00_JPRB ,&
&  0.56642E+00_JPRB, 0.57355E+00_JPRB, 0.58598E+00_JPRB, 0.58814E+00_JPRB, 0.57029E+00_JPRB, 0.55604E+00_JPRB, 0.54749E+00_JPRB ,&
&  0.56465E+00_JPRB, 0.56083E+00_JPRB, 0.55086E+00_JPRB, 0.56734E+00_JPRB, 0.56874E+00_JPRB, 0.61974E+00_JPRB, 0.70479E+00_JPRB ,&
&  0.71016E+00_JPRB, 0.46830E+00_JPRB, 0.48863E+00_JPRB, 0.51004E+00_JPRB, 0.55678E+00_JPRB, 0.63320E+00_JPRB, 0.78857E+00_JPRB ,&
&  0.11368E+01_JPRB, 0.19030E+01_JPRB, 0.21889E+01_JPRB, 0.14468E+01_JPRB, 0.19789E+01_JPRB, 0.12746E+01_JPRB, 0.49732E+00_JPRB ,&
&  0.46928E+00_JPRB, 0.45472E+00_JPRB, 0.42731E+00_JPRB, 0.39843E+00_JPRB, 0.37923E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.44107E+00_JPRB, 0.44190E+00_JPRB, 0.44835E+00_JPRB, 0.45239E+00_JPRB, 0.45661E+00_JPRB, 0.45896E+00_JPRB, 0.46173E+00_JPRB ,&
&  0.47396E+00_JPRB, 0.48056E+00_JPRB, 0.49793E+00_JPRB, 0.50867E+00_JPRB, 0.51802E+00_JPRB, 0.54239E+00_JPRB, 0.56004E+00_JPRB ,&
&  0.59077E+00_JPRB, 0.62997E+00_JPRB, 0.63958E+00_JPRB, 0.65790E+00_JPRB, 0.66536E+00_JPRB, 0.67033E+00_JPRB, 0.66932E+00_JPRB ,&
&  0.70559E+00_JPRB, 0.77733E+00_JPRB, 0.81869E+00_JPRB, 0.86334E+00_JPRB, 0.84884E+00_JPRB, 0.84723E+00_JPRB, 0.67424E+00_JPRB ,&
&  0.90242E+00_JPRB, 0.89164E+00_JPRB, 0.82551E+00_JPRB, 0.78540E+00_JPRB, 0.75806E+00_JPRB, 0.72731E+00_JPRB, 0.69094E+00_JPRB ,&
&  0.67222E+00_JPRB, 0.68753E+00_JPRB, 0.67871E+00_JPRB, 0.71873E+00_JPRB, 0.77112E+00_JPRB, 0.77664E+00_JPRB, 0.80984E+00_JPRB ,&
&  0.79856E+00_JPRB, 0.78621E+00_JPRB, 0.78780E+00_JPRB, 0.80715E+00_JPRB, 0.82206E+00_JPRB, 0.82990E+00_JPRB, 0.83574E+00_JPRB ,&
&  0.83526E+00_JPRB, 0.85044E+00_JPRB, 0.89598E+00_JPRB, 0.88129E+00_JPRB, 0.86457E+00_JPRB, 0.84597E+00_JPRB, 0.77645E+00_JPRB ,&
&  0.62257E+00_JPRB, 0.56926E+00_JPRB, 0.59256E+00_JPRB, 0.66846E+00_JPRB, 0.63661E+00_JPRB, 0.60003E+00_JPRB, 0.57377E+00_JPRB ,&
&  0.55201E+00_JPRB, 0.55454E+00_JPRB, 0.56196E+00_JPRB, 0.55833E+00_JPRB, 0.54313E+00_JPRB, 0.52004E+00_JPRB, 0.50002E+00_JPRB ,&
&  0.49887E+00_JPRB, 0.49055E+00_JPRB, 0.48552E+00_JPRB, 0.48815E+00_JPRB, 0.48676E+00_JPRB, 0.50015E+00_JPRB, 0.46382E+00_JPRB ,&
&  0.50875E+00_JPRB, 0.42751E+00_JPRB, 0.45680E+00_JPRB, 0.50001E+00_JPRB, 0.57550E+00_JPRB, 0.80381E+00_JPRB, 0.12051E+01_JPRB ,&
&  0.17993E+01_JPRB, 0.20701E+01_JPRB, 0.27592E+01_JPRB, 0.40612E+01_JPRB, 0.25507E+01_JPRB, 0.16170E+01_JPRB, 0.51057E+00_JPRB ,&
&  0.49906E+00_JPRB, 0.47338E+00_JPRB, 0.46571E+00_JPRB, 0.45105E+00_JPRB, 0.44681E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.50185E+00_JPRB, 0.50978E+00_JPRB, 0.51861E+00_JPRB, 0.52626E+00_JPRB, 0.53071E+00_JPRB, 0.53950E+00_JPRB, 0.54549E+00_JPRB ,&
&  0.55264E+00_JPRB, 0.55405E+00_JPRB, 0.55863E+00_JPRB, 0.56192E+00_JPRB, 0.56877E+00_JPRB, 0.57969E+00_JPRB, 0.59028E+00_JPRB ,&
&  0.59727E+00_JPRB, 0.60860E+00_JPRB, 0.62291E+00_JPRB, 0.62982E+00_JPRB, 0.65222E+00_JPRB, 0.69784E+00_JPRB, 0.74756E+00_JPRB ,&
&  0.78121E+00_JPRB, 0.82356E+00_JPRB, 0.86086E+00_JPRB, 0.86570E+00_JPRB, 0.84277E+00_JPRB, 0.81895E+00_JPRB, 0.68010E+00_JPRB ,&
&  0.10286E+01_JPRB, 0.10146E+01_JPRB, 0.90876E+00_JPRB, 0.84611E+00_JPRB, 0.79130E+00_JPRB, 0.74044E+00_JPRB, 0.68621E+00_JPRB ,&
&  0.64644E+00_JPRB, 0.68986E+00_JPRB, 0.68571E+00_JPRB, 0.69237E+00_JPRB, 0.67317E+00_JPRB, 0.67197E+00_JPRB, 0.72480E+00_JPRB ,&
&  0.77279E+00_JPRB, 0.74642E+00_JPRB, 0.68243E+00_JPRB, 0.67870E+00_JPRB, 0.68388E+00_JPRB, 0.67881E+00_JPRB, 0.69898E+00_JPRB ,&
&  0.68025E+00_JPRB, 0.66717E+00_JPRB, 0.63486E+00_JPRB, 0.70380E+00_JPRB, 0.73115E+00_JPRB, 0.72436E+00_JPRB, 0.67625E+00_JPRB ,&
&  0.53692E+00_JPRB, 0.54629E+00_JPRB, 0.59158E+00_JPRB, 0.68775E+00_JPRB, 0.64868E+00_JPRB, 0.61741E+00_JPRB, 0.58099E+00_JPRB ,&
&  0.55220E+00_JPRB, 0.53897E+00_JPRB, 0.51838E+00_JPRB, 0.50849E+00_JPRB, 0.50358E+00_JPRB, 0.48964E+00_JPRB, 0.46500E+00_JPRB ,&
&  0.44440E+00_JPRB, 0.43926E+00_JPRB, 0.43239E+00_JPRB, 0.42461E+00_JPRB, 0.41496E+00_JPRB, 0.36392E+00_JPRB, 0.41951E+00_JPRB ,&
&  0.41830E+00_JPRB, 0.36871E+00_JPRB, 0.45161E+00_JPRB, 0.37880E+00_JPRB, 0.49565E+00_JPRB, 0.61493E+00_JPRB, 0.80690E+00_JPRB ,&
&  0.85855E+00_JPRB, 0.11109E+01_JPRB, 0.14623E+01_JPRB, 0.11532E+01_JPRB, 0.86651E+00_JPRB, 0.56907E+00_JPRB, 0.53241E+00_JPRB ,&
&  0.52062E+00_JPRB, 0.51016E+00_JPRB, 0.51278E+00_JPRB, 0.50566E+00_JPRB, 0.50390E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.56987E+00_JPRB, 0.58102E+00_JPRB, 0.59444E+00_JPRB, 0.61199E+00_JPRB, 0.62198E+00_JPRB, 0.63743E+00_JPRB, 0.64865E+00_JPRB ,&
&  0.66516E+00_JPRB, 0.67666E+00_JPRB, 0.68547E+00_JPRB, 0.69444E+00_JPRB, 0.70768E+00_JPRB, 0.72622E+00_JPRB, 0.75047E+00_JPRB ,&
&  0.77683E+00_JPRB, 0.80299E+00_JPRB, 0.83174E+00_JPRB, 0.86492E+00_JPRB, 0.90343E+00_JPRB, 0.95107E+00_JPRB, 0.99504E+00_JPRB ,&
&  0.10344E+01_JPRB, 0.10724E+01_JPRB, 0.11072E+01_JPRB, 0.11336E+01_JPRB, 0.11371E+01_JPRB, 0.11247E+01_JPRB, 0.95583E+00_JPRB ,&
&  0.12050E+01_JPRB, 0.11898E+01_JPRB, 0.10956E+01_JPRB, 0.99324E+00_JPRB, 0.91002E+00_JPRB, 0.83935E+00_JPRB, 0.76758E+00_JPRB ,&
&  0.75754E+00_JPRB, 0.70716E+00_JPRB, 0.68030E+00_JPRB, 0.67345E+00_JPRB, 0.63637E+00_JPRB, 0.61164E+00_JPRB, 0.61431E+00_JPRB ,&
&  0.63704E+00_JPRB, 0.68379E+00_JPRB, 0.68974E+00_JPRB, 0.71043E+00_JPRB, 0.76488E+00_JPRB, 0.72310E+00_JPRB, 0.73755E+00_JPRB ,&
&  0.68687E+00_JPRB, 0.62571E+00_JPRB, 0.54536E+00_JPRB, 0.61168E+00_JPRB, 0.66259E+00_JPRB, 0.69530E+00_JPRB, 0.69228E+00_JPRB ,&
&  0.58301E+00_JPRB, 0.51789E+00_JPRB, 0.55179E+00_JPRB, 0.72982E+00_JPRB, 0.70913E+00_JPRB, 0.64442E+00_JPRB, 0.59541E+00_JPRB ,&
&  0.54946E+00_JPRB, 0.51668E+00_JPRB, 0.48093E+00_JPRB, 0.46621E+00_JPRB, 0.47878E+00_JPRB, 0.47553E+00_JPRB, 0.45474E+00_JPRB ,&
&  0.42381E+00_JPRB, 0.40780E+00_JPRB, 0.40258E+00_JPRB, 0.39520E+00_JPRB, 0.39610E+00_JPRB, 0.34371E+00_JPRB, 0.38519E+00_JPRB ,&
&  0.39178E+00_JPRB, 0.31171E+00_JPRB, 0.38773E+00_JPRB, 0.52041E+00_JPRB, 0.51544E+00_JPRB, 0.57619E+00_JPRB, 0.61079E+00_JPRB ,&
&  0.64336E+00_JPRB, 0.65253E+00_JPRB, 0.60777E+00_JPRB, 0.60359E+00_JPRB, 0.59115E+00_JPRB, 0.57814E+00_JPRB, 0.56314E+00_JPRB ,&
&  0.54966E+00_JPRB, 0.55021E+00_JPRB, 0.56392E+00_JPRB, 0.56381E+00_JPRB, 0.56443E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.57929E+00_JPRB, 0.59107E+00_JPRB, 0.59085E+00_JPRB, 0.59604E+00_JPRB, 0.60756E+00_JPRB, 0.61475E+00_JPRB, 0.62610E+00_JPRB ,&
&  0.63698E+00_JPRB, 0.63970E+00_JPRB, 0.64180E+00_JPRB, 0.64530E+00_JPRB, 0.64586E+00_JPRB, 0.65984E+00_JPRB, 0.67380E+00_JPRB ,&
&  0.69294E+00_JPRB, 0.70709E+00_JPRB, 0.72299E+00_JPRB, 0.74896E+00_JPRB, 0.78193E+00_JPRB, 0.83490E+00_JPRB, 0.89040E+00_JPRB ,&
&  0.96137E+00_JPRB, 0.10452E+01_JPRB, 0.11442E+01_JPRB, 0.12225E+01_JPRB, 0.12609E+01_JPRB, 0.12696E+01_JPRB, 0.13557E+01_JPRB ,&
&  0.11913E+01_JPRB, 0.12744E+01_JPRB, 0.11911E+01_JPRB, 0.10486E+01_JPRB, 0.10255E+01_JPRB, 0.95589E+00_JPRB, 0.92485E+00_JPRB ,&
&  0.91634E+00_JPRB, 0.86576E+00_JPRB, 0.84079E+00_JPRB, 0.81676E+00_JPRB, 0.79354E+00_JPRB, 0.74544E+00_JPRB, 0.69843E+00_JPRB ,&
&  0.67942E+00_JPRB, 0.69713E+00_JPRB, 0.76048E+00_JPRB, 0.90632E+00_JPRB, 0.10548E+01_JPRB, 0.11035E+01_JPRB, 0.10734E+01_JPRB ,&
&  0.10231E+01_JPRB, 0.98616E+00_JPRB, 0.92885E+00_JPRB, 0.90359E+00_JPRB, 0.94448E+00_JPRB, 0.98007E+00_JPRB, 0.96912E+00_JPRB ,&
&  0.82655E+00_JPRB, 0.70280E+00_JPRB, 0.66634E+00_JPRB, 0.72735E+00_JPRB, 0.73610E+00_JPRB, 0.71165E+00_JPRB, 0.62421E+00_JPRB ,&
&  0.54127E+00_JPRB, 0.49388E+00_JPRB, 0.46450E+00_JPRB, 0.46559E+00_JPRB, 0.50150E+00_JPRB, 0.50785E+00_JPRB, 0.49412E+00_JPRB ,&
&  0.45417E+00_JPRB, 0.42809E+00_JPRB, 0.40936E+00_JPRB, 0.39782E+00_JPRB, 0.34999E+00_JPRB, 0.40681E+00_JPRB, 0.40962E+00_JPRB ,&
&  0.42328E+00_JPRB, 0.39933E+00_JPRB, 0.41126E+00_JPRB, 0.55814E+00_JPRB, 0.59611E+00_JPRB, 0.61424E+00_JPRB, 0.61331E+00_JPRB ,&
&  0.60380E+00_JPRB, 0.60214E+00_JPRB, 0.59854E+00_JPRB, 0.58129E+00_JPRB, 0.56804E+00_JPRB, 0.56208E+00_JPRB, 0.58198E+00_JPRB ,&
&  0.57699E+00_JPRB, 0.57618E+00_JPRB, 0.57357E+00_JPRB, 0.58608E+00_JPRB, 0.58341E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.59340E+00_JPRB, 0.59877E+00_JPRB, 0.60856E+00_JPRB, 0.62583E+00_JPRB, 0.62252E+00_JPRB, 0.61923E+00_JPRB, 0.64374E+00_JPRB ,&
&  0.62639E+00_JPRB, 0.61297E+00_JPRB, 0.60848E+00_JPRB, 0.58147E+00_JPRB, 0.59479E+00_JPRB, 0.60659E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.66790E+00_JPRB, 0.68553E+00_JPRB, 0.70103E+00_JPRB, 0.72572E+00_JPRB, 0.77300E+00_JPRB, 0.80744E+00_JPRB, 0.83236E+00_JPRB ,&
&  0.88233E+00_JPRB, 0.96515E+00_JPRB, 0.10593E+01_JPRB, 0.12892E+01_JPRB, 0.18280E+01_JPRB, 0.22470E+01_JPRB, 0.22311E+01_JPRB ,&
&  0.16682E+01_JPRB, 0.11840E+01_JPRB, 0.11235E+01_JPRB, 0.10163E+01_JPRB, 0.10419E+01_JPRB, 0.11319E+01_JPRB, 0.11463E+01_JPRB ,&
&  0.11230E+01_JPRB, 0.11177E+01_JPRB, 0.11091E+01_JPRB, 0.11047E+01_JPRB, 0.11119E+01_JPRB, 0.11136E+01_JPRB, 0.10994E+01_JPRB ,&
&  0.10624E+01_JPRB, 0.10179E+01_JPRB, 0.10473E+01_JPRB, 0.12362E+01_JPRB, 0.13597E+01_JPRB, 0.15888E+01_JPRB, 0.15933E+01_JPRB ,&
&  0.15483E+01_JPRB, 0.13893E+01_JPRB, 0.13416E+01_JPRB, 0.13312E+01_JPRB, 0.13031E+01_JPRB, 0.12846E+01_JPRB, 0.12994E+01_JPRB ,&
&  0.12749E+01_JPRB, 0.10892E+01_JPRB, 0.71795E+00_JPRB, 0.75417E+00_JPRB, 0.76381E+00_JPRB, 0.78254E+00_JPRB, 0.63664E+00_JPRB ,&
&  0.53112E+00_JPRB, 0.50290E+00_JPRB, 0.49618E+00_JPRB, 0.51218E+00_JPRB, 0.55138E+00_JPRB, 0.58357E+00_JPRB, 0.55821E+00_JPRB ,&
&  0.51215E+00_JPRB, 0.46118E+00_JPRB, 0.42963E+00_JPRB, 0.42018E+00_JPRB, 0.41593E+00_JPRB, 0.38458E+00_JPRB, 0.44204E+00_JPRB ,&
&  0.47436E+00_JPRB, 0.53695E+00_JPRB, 0.50718E+00_JPRB, 0.66915E+00_JPRB, 0.71343E+00_JPRB, 0.70834E+00_JPRB, 0.69579E+00_JPRB ,&
&  0.68657E+00_JPRB, 0.66806E+00_JPRB, 0.65082E+00_JPRB, 0.63019E+00_JPRB, 0.60876E+00_JPRB, 0.62851E+00_JPRB, 0.65713E+00_JPRB ,&
&  0.62958E+00_JPRB, 0.64404E+00_JPRB, 0.62981E+00_JPRB, 0.60771E+00_JPRB, 0.60904E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.62163E+00_JPRB, 0.61539E+00_JPRB, 0.62978E+00_JPRB, 0.65191E+00_JPRB, 0.66311E+00_JPRB, 0.67365E+00_JPRB, 0.68435E+00_JPRB ,&
&  0.68695E+00_JPRB, 0.68795E+00_JPRB, 0.69085E+00_JPRB, 0.69044E+00_JPRB, 0.71952E+00_JPRB, 0.75111E+00_JPRB, 0.79070E+00_JPRB ,&
&  0.85042E+00_JPRB, 0.90313E+00_JPRB, 0.94916E+00_JPRB, 0.10390E+01_JPRB, 0.10976E+01_JPRB, 0.11480E+01_JPRB, 0.12379E+01_JPRB ,&
&  0.13571E+01_JPRB, 0.14760E+01_JPRB, 0.15924E+01_JPRB, 0.16974E+01_JPRB, 0.18145E+01_JPRB, 0.19132E+01_JPRB, 0.18900E+01_JPRB ,&
&  0.14029E+01_JPRB, 0.11337E+01_JPRB, 0.12538E+01_JPRB, 0.12020E+01_JPRB, 0.12311E+01_JPRB, 0.12463E+01_JPRB, 0.12422E+01_JPRB ,&
&  0.12570E+01_JPRB, 0.12610E+01_JPRB, 0.12619E+01_JPRB, 0.12666E+01_JPRB, 0.13042E+01_JPRB, 0.13593E+01_JPRB, 0.14359E+01_JPRB ,&
&  0.14690E+01_JPRB, 0.14976E+01_JPRB, 0.15194E+01_JPRB, 0.15048E+01_JPRB, 0.15560E+01_JPRB, 0.16754E+01_JPRB, 0.17827E+01_JPRB ,&
&  0.18369E+01_JPRB, 0.18186E+01_JPRB, 0.16109E+01_JPRB, 0.15878E+01_JPRB, 0.15879E+01_JPRB, 0.14789E+01_JPRB, 0.15171E+01_JPRB ,&
&  0.15877E+01_JPRB, 0.13811E+01_JPRB, 0.84247E+00_JPRB, 0.77239E+00_JPRB, 0.74834E+00_JPRB, 0.71726E+00_JPRB, 0.64221E+00_JPRB ,&
&  0.53927E+00_JPRB, 0.52108E+00_JPRB, 0.53538E+00_JPRB, 0.59173E+00_JPRB, 0.62936E+00_JPRB, 0.65653E+00_JPRB, 0.61433E+00_JPRB ,&
&  0.54768E+00_JPRB, 0.49611E+00_JPRB, 0.45017E+00_JPRB, 0.44710E+00_JPRB, 0.43252E+00_JPRB, 0.45682E+00_JPRB, 0.49189E+00_JPRB ,&
&  0.54700E+00_JPRB, 0.60828E+00_JPRB, 0.68943E+00_JPRB, 0.71405E+00_JPRB, 0.70245E+00_JPRB, 0.80880E+00_JPRB, 0.78235E+00_JPRB ,&
&  0.76025E+00_JPRB, 0.75216E+00_JPRB, 0.73730E+00_JPRB, 0.72506E+00_JPRB, 0.71326E+00_JPRB, 0.71004E+00_JPRB, 0.69194E+00_JPRB ,&
&  0.68468E+00_JPRB, 0.67100E+00_JPRB, 0.66478E+00_JPRB, 0.64198E+00_JPRB, 0.63206E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.63613E+00_JPRB, 0.63966E+00_JPRB, 0.64623E+00_JPRB, 0.66524E+00_JPRB, 0.68896E+00_JPRB, 0.70359E+00_JPRB, 0.70815E+00_JPRB ,&
&  0.72285E+00_JPRB, 0.74299E+00_JPRB, 0.77342E+00_JPRB, 0.80027E+00_JPRB, 0.83532E+00_JPRB, 0.86764E+00_JPRB, 0.91989E+00_JPRB ,&
&  0.96762E+00_JPRB, 0.10257E+01_JPRB, 0.11295E+01_JPRB, 0.12846E+01_JPRB, 0.14134E+01_JPRB, 0.15198E+01_JPRB, 0.16855E+01_JPRB ,&
&  0.18426E+01_JPRB, 0.19052E+01_JPRB, 0.19832E+01_JPRB, 0.19712E+01_JPRB, 0.18185E+01_JPRB, 0.16499E+01_JPRB, 0.16237E+01_JPRB ,&
&  0.13261E+01_JPRB, 0.11365E+01_JPRB, 0.11773E+01_JPRB, 0.12342E+01_JPRB, 0.12681E+01_JPRB, 0.12650E+01_JPRB, 0.12678E+01_JPRB ,&
&  0.12845E+01_JPRB, 0.12924E+01_JPRB, 0.12986E+01_JPRB, 0.13087E+01_JPRB, 0.13366E+01_JPRB, 0.13955E+01_JPRB, 0.14835E+01_JPRB ,&
&  0.14931E+01_JPRB, 0.15234E+01_JPRB, 0.16021E+01_JPRB, 0.14799E+01_JPRB, 0.15673E+01_JPRB, 0.16818E+01_JPRB, 0.17226E+01_JPRB ,&
&  0.16870E+01_JPRB, 0.15284E+01_JPRB, 0.15058E+01_JPRB, 0.15217E+01_JPRB, 0.15302E+01_JPRB, 0.14522E+01_JPRB, 0.13900E+01_JPRB ,&
&  0.14883E+01_JPRB, 0.14511E+01_JPRB, 0.95075E+00_JPRB, 0.79936E+00_JPRB, 0.77864E+00_JPRB, 0.65581E+00_JPRB, 0.64640E+00_JPRB ,&
&  0.57461E+00_JPRB, 0.55478E+00_JPRB, 0.60195E+00_JPRB, 0.63809E+00_JPRB, 0.66762E+00_JPRB, 0.73541E+00_JPRB, 0.59963E+00_JPRB ,&
&  0.56372E+00_JPRB, 0.53010E+00_JPRB, 0.49803E+00_JPRB, 0.49830E+00_JPRB, 0.50987E+00_JPRB, 0.55184E+00_JPRB, 0.59108E+00_JPRB ,&
&  0.63790E+00_JPRB, 0.70312E+00_JPRB, 0.76516E+00_JPRB, 0.82199E+00_JPRB, 0.79798E+00_JPRB, 0.83731E+00_JPRB, 0.80664E+00_JPRB ,&
&  0.77109E+00_JPRB, 0.76309E+00_JPRB, 0.74760E+00_JPRB, 0.72295E+00_JPRB, 0.70918E+00_JPRB, 0.69279E+00_JPRB, 0.68412E+00_JPRB ,&
&  0.67971E+00_JPRB, 0.67540E+00_JPRB, 0.66763E+00_JPRB, 0.66015E+00_JPRB, 0.64574E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.65704E+00_JPRB, 0.65282E+00_JPRB, 0.65519E+00_JPRB, 0.67056E+00_JPRB, 0.68992E+00_JPRB, 0.69701E+00_JPRB, 0.70426E+00_JPRB ,&
&  0.72270E+00_JPRB, 0.74536E+00_JPRB, 0.76912E+00_JPRB, 0.80292E+00_JPRB, 0.83717E+00_JPRB, 0.87344E+00_JPRB, 0.93075E+00_JPRB ,&
&  0.99938E+00_JPRB, 0.10951E+01_JPRB, 0.12287E+01_JPRB, 0.13703E+01_JPRB, 0.14997E+01_JPRB, 0.16458E+01_JPRB, 0.17883E+01_JPRB ,&
&  0.19508E+01_JPRB, 0.19927E+01_JPRB, 0.18891E+01_JPRB, 0.17034E+01_JPRB, 0.13124E+01_JPRB, 0.12946E+01_JPRB, 0.13859E+01_JPRB ,&
&  0.13927E+01_JPRB, 0.12263E+01_JPRB, 0.12064E+01_JPRB, 0.11950E+01_JPRB, 0.12022E+01_JPRB, 0.12219E+01_JPRB, 0.12279E+01_JPRB ,&
&  0.12472E+01_JPRB, 0.12674E+01_JPRB, 0.12836E+01_JPRB, 0.12951E+01_JPRB, 0.13079E+01_JPRB, 0.13419E+01_JPRB, 0.14241E+01_JPRB ,&
&  0.14308E+01_JPRB, 0.14445E+01_JPRB, 0.14580E+01_JPRB, 0.14173E+01_JPRB, 0.14787E+01_JPRB, 0.15519E+01_JPRB, 0.14823E+01_JPRB ,&
&  0.14095E+01_JPRB, 0.12556E+01_JPRB, 0.12169E+01_JPRB, 0.12477E+01_JPRB, 0.12225E+01_JPRB, 0.11067E+01_JPRB, 0.10925E+01_JPRB ,&
&  0.12102E+01_JPRB, 0.13754E+01_JPRB, 0.86685E+00_JPRB, 0.85955E+00_JPRB, 0.94049E+00_JPRB, 0.81884E+00_JPRB, 0.71449E+00_JPRB ,&
&  0.65197E+00_JPRB, 0.65235E+00_JPRB, 0.70209E+00_JPRB, 0.70694E+00_JPRB, 0.69032E+00_JPRB, 0.75307E+00_JPRB, 0.59681E+00_JPRB ,&
&  0.61655E+00_JPRB, 0.58395E+00_JPRB, 0.56664E+00_JPRB, 0.57953E+00_JPRB, 0.60530E+00_JPRB, 0.62178E+00_JPRB, 0.62352E+00_JPRB ,&
&  0.61121E+00_JPRB, 0.88305E+00_JPRB, 0.11299E+01_JPRB, 0.14052E+01_JPRB, 0.14339E+01_JPRB, 0.83893E+00_JPRB, 0.83062E+00_JPRB ,&
&  0.81720E+00_JPRB, 0.79631E+00_JPRB, 0.77172E+00_JPRB, 0.74994E+00_JPRB, 0.73301E+00_JPRB, 0.71532E+00_JPRB, 0.70078E+00_JPRB ,&
&  0.69046E+00_JPRB, 0.68341E+00_JPRB, 0.67912E+00_JPRB, 0.66980E+00_JPRB, 0.66151E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.67436E+00_JPRB, 0.67373E+00_JPRB, 0.66634E+00_JPRB, 0.67138E+00_JPRB, 0.70092E+00_JPRB, 0.72076E+00_JPRB, 0.70622E+00_JPRB ,&
&  0.70573E+00_JPRB, 0.71529E+00_JPRB, 0.72960E+00_JPRB, 0.75013E+00_JPRB, 0.77091E+00_JPRB, 0.78664E+00_JPRB, 0.83167E+00_JPRB ,&
&  0.88704E+00_JPRB, 0.99595E+00_JPRB, 0.11676E+01_JPRB, 0.13402E+01_JPRB, 0.14993E+01_JPRB, 0.16831E+01_JPRB, 0.18634E+01_JPRB ,&
&  0.20944E+01_JPRB, 0.21560E+01_JPRB, 0.19598E+01_JPRB, 0.12883E+01_JPRB, 0.10561E+01_JPRB, 0.11107E+01_JPRB, 0.11291E+01_JPRB ,&
&  0.11308E+01_JPRB, 0.11212E+01_JPRB, 0.11162E+01_JPRB, 0.11138E+01_JPRB, 0.11267E+01_JPRB, 0.11506E+01_JPRB, 0.11544E+01_JPRB ,&
&  0.11658E+01_JPRB, 0.11883E+01_JPRB, 0.12130E+01_JPRB, 0.12401E+01_JPRB, 0.12655E+01_JPRB, 0.12922E+01_JPRB, 0.13193E+01_JPRB ,&
&  0.13208E+01_JPRB, 0.13417E+01_JPRB, 0.13199E+01_JPRB, 0.12732E+01_JPRB, 0.12596E+01_JPRB, 0.12629E+01_JPRB, 0.12277E+01_JPRB ,&
&  0.11937E+01_JPRB, 0.10940E+01_JPRB, 0.10483E+01_JPRB, 0.10630E+01_JPRB, 0.10696E+01_JPRB, 0.96950E+00_JPRB, 0.97446E+00_JPRB ,&
&  0.10508E+01_JPRB, 0.11181E+01_JPRB, 0.89797E+00_JPRB, 0.95375E+00_JPRB, 0.66634E+00_JPRB, 0.76167E+00_JPRB, 0.81709E+00_JPRB ,&
&  0.77626E+00_JPRB, 0.78122E+00_JPRB, 0.80098E+00_JPRB, 0.81835E+00_JPRB, 0.81940E+00_JPRB, 0.73971E+00_JPRB, 0.70149E+00_JPRB ,&
&  0.69312E+00_JPRB, 0.63912E+00_JPRB, 0.59343E+00_JPRB, 0.62400E+00_JPRB, 0.66283E+00_JPRB, 0.65321E+00_JPRB, 0.67538E+00_JPRB ,&
&  0.60010E+00_JPRB, 0.75087E+00_JPRB, 0.91073E+00_JPRB, 0.11003E+01_JPRB, 0.11050E+01_JPRB, 0.83678E+00_JPRB, 0.84021E+00_JPRB ,&
&  0.84239E+00_JPRB, 0.81775E+00_JPRB, 0.79618E+00_JPRB, 0.76420E+00_JPRB, 0.74896E+00_JPRB, 0.73185E+00_JPRB, 0.71494E+00_JPRB ,&
&  0.70686E+00_JPRB, 0.70025E+00_JPRB, 0.69423E+00_JPRB, 0.68105E+00_JPRB, 0.67237E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.69245E+00_JPRB, 0.69460E+00_JPRB, 0.70218E+00_JPRB, 0.70791E+00_JPRB, 0.71585E+00_JPRB, 0.75744E+00_JPRB, 0.77404E+00_JPRB ,&
&  0.73573E+00_JPRB, 0.68071E+00_JPRB, 0.67885E+00_JPRB, 0.67939E+00_JPRB, 0.68317E+00_JPRB, 0.69539E+00_JPRB, 0.72110E+00_JPRB ,&
&  0.74819E+00_JPRB, 0.79765E+00_JPRB, 0.91588E+00_JPRB, 0.11262E+01_JPRB, 0.13641E+01_JPRB, 0.15950E+01_JPRB, 0.18275E+01_JPRB ,&
&  0.17086E+01_JPRB, 0.12557E+01_JPRB, 0.15861E+01_JPRB, 0.15139E+01_JPRB, 0.12722E+01_JPRB, 0.10830E+01_JPRB, 0.10526E+01_JPRB ,&
&  0.10920E+01_JPRB, 0.10896E+01_JPRB, 0.10785E+01_JPRB, 0.10723E+01_JPRB, 0.10444E+01_JPRB, 0.10597E+01_JPRB, 0.10818E+01_JPRB ,&
&  0.10928E+01_JPRB, 0.11031E+01_JPRB, 0.11146E+01_JPRB, 0.11340E+01_JPRB, 0.11793E+01_JPRB, 0.12331E+01_JPRB, 0.12576E+01_JPRB ,&
&  0.12501E+01_JPRB, 0.12337E+01_JPRB, 0.11587E+01_JPRB, 0.11028E+01_JPRB, 0.10877E+01_JPRB, 0.10876E+01_JPRB, 0.10800E+01_JPRB ,&
&  0.10974E+01_JPRB, 0.10115E+01_JPRB, 0.10051E+01_JPRB, 0.10240E+01_JPRB, 0.10307E+01_JPRB, 0.96397E+00_JPRB, 0.98926E+00_JPRB ,&
&  0.99435E+00_JPRB, 0.99783E+00_JPRB, 0.94557E+00_JPRB, 0.96976E+00_JPRB, 0.73977E+00_JPRB, 0.80098E+00_JPRB, 0.76087E+00_JPRB ,&
&  0.84076E+00_JPRB, 0.84829E+00_JPRB, 0.85742E+00_JPRB, 0.87709E+00_JPRB, 0.92407E+00_JPRB, 0.84291E+00_JPRB, 0.80781E+00_JPRB ,&
&  0.72548E+00_JPRB, 0.69836E+00_JPRB, 0.64316E+00_JPRB, 0.68707E+00_JPRB, 0.69329E+00_JPRB, 0.68465E+00_JPRB, 0.74245E+00_JPRB ,&
&  0.77582E+00_JPRB, 0.73972E+00_JPRB, 0.76006E+00_JPRB, 0.85701E+00_JPRB, 0.89632E+00_JPRB, 0.81361E+00_JPRB, 0.81578E+00_JPRB ,&
&  0.81860E+00_JPRB, 0.82402E+00_JPRB, 0.79636E+00_JPRB, 0.76653E+00_JPRB, 0.74129E+00_JPRB, 0.72260E+00_JPRB, 0.72010E+00_JPRB ,&
&  0.71121E+00_JPRB, 0.70413E+00_JPRB, 0.69911E+00_JPRB, 0.69316E+00_JPRB, 0.69027E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.68296E+00_JPRB, 0.68366E+00_JPRB, 0.68584E+00_JPRB, 0.70574E+00_JPRB, 0.75745E+00_JPRB, 0.81569E+00_JPRB, 0.95869E+00_JPRB ,&
&  0.11791E+01_JPRB, 0.75614E+00_JPRB, 0.65535E+00_JPRB, 0.63525E+00_JPRB, 0.62340E+00_JPRB, 0.61848E+00_JPRB, 0.63584E+00_JPRB ,&
&  0.65848E+00_JPRB, 0.68400E+00_JPRB, 0.73580E+00_JPRB, 0.84876E+00_JPRB, 0.10619E+01_JPRB, 0.13740E+01_JPRB, 0.14216E+01_JPRB ,&
&  0.11738E+01_JPRB, 0.13931E+01_JPRB, 0.17578E+01_JPRB, 0.14668E+01_JPRB, 0.13141E+01_JPRB, 0.11665E+01_JPRB, 0.10909E+01_JPRB ,&
&  0.10607E+01_JPRB, 0.10440E+01_JPRB, 0.10502E+01_JPRB, 0.10427E+01_JPRB, 0.10216E+01_JPRB, 0.10258E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10616E+01_JPRB, 0.10636E+01_JPRB, 0.10619E+01_JPRB, 0.10635E+01_JPRB, 0.10805E+01_JPRB, 0.11273E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.11866E+01_JPRB, 0.11786E+01_JPRB, 0.11047E+01_JPRB, 0.10417E+01_JPRB, 0.10278E+01_JPRB, 0.10369E+01_JPRB, 0.10382E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10104E+01_JPRB, 0.10006E+01_JPRB, 0.98694E+00_JPRB, 0.94065E+00_JPRB, 0.99622E+00_JPRB, 0.97100E+00_JPRB ,&
&  0.98420E+00_JPRB, 0.95963E+00_JPRB, 0.10226E+01_JPRB, 0.85241E+00_JPRB, 0.83201E+00_JPRB, 0.86900E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.81130E+00_JPRB, 0.85721E+00_JPRB, 0.90827E+00_JPRB, 0.92094E+00_JPRB, 0.93972E+00_JPRB, 0.88164E+00_JPRB, 0.89793E+00_JPRB ,&
&  0.76938E+00_JPRB, 0.81728E+00_JPRB, 0.72228E+00_JPRB, 0.73850E+00_JPRB, 0.72699E+00_JPRB, 0.69810E+00_JPRB, 0.74080E+00_JPRB ,&
&  0.80100E+00_JPRB, 0.77943E+00_JPRB, 0.75877E+00_JPRB, 0.76397E+00_JPRB, 0.76143E+00_JPRB, 0.75673E+00_JPRB, 0.76727E+00_JPRB ,&
&  0.77923E+00_JPRB, 0.81247E+00_JPRB, 0.79339E+00_JPRB, 0.77782E+00_JPRB, 0.75753E+00_JPRB, 0.73772E+00_JPRB, 0.72683E+00_JPRB ,&
&  0.71824E+00_JPRB, 0.70858E+00_JPRB, 0.70065E+00_JPRB, 0.69332E+00_JPRB, 0.68995E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.66445E+00_JPRB, 0.65169E+00_JPRB, 0.63570E+00_JPRB, 0.62467E+00_JPRB, 0.62367E+00_JPRB, 0.65534E+00_JPRB, 0.72778E+00_JPRB ,&
&  0.10138E+01_JPRB, 0.75525E+00_JPRB, 0.66161E+00_JPRB, 0.63042E+00_JPRB, 0.61526E+00_JPRB, 0.58952E+00_JPRB, 0.57840E+00_JPRB ,&
&  0.58447E+00_JPRB, 0.59881E+00_JPRB, 0.63483E+00_JPRB, 0.68925E+00_JPRB, 0.78761E+00_JPRB, 0.99858E+00_JPRB, 0.10723E+01_JPRB ,&
&  0.10146E+01_JPRB, 0.19511E+01_JPRB, 0.17082E+01_JPRB, 0.15920E+01_JPRB, 0.14044E+01_JPRB, 0.12813E+01_JPRB, 0.11200E+01_JPRB ,&
&  0.10634E+01_JPRB, 0.10559E+01_JPRB, 0.10725E+01_JPRB, 0.10822E+01_JPRB, 0.10824E+01_JPRB, 0.10803E+01_JPRB, 0.10863E+01_JPRB ,&
&  0.10793E+01_JPRB, 0.10710E+01_JPRB, 0.10679E+01_JPRB, 0.10642E+01_JPRB, 0.10614E+01_JPRB, 0.10816E+01_JPRB, 0.10966E+01_JPRB ,&
&  0.11193E+01_JPRB, 0.11595E+01_JPRB, 0.11004E+01_JPRB, 0.10452E+01_JPRB, 0.10255E+01_JPRB, 0.10376E+01_JPRB, 0.10446E+01_JPRB ,&
&  0.10020E+01_JPRB, 0.94734E+00_JPRB, 0.98223E+00_JPRB, 0.99972E+00_JPRB, 0.99806E+00_JPRB, 0.10439E+01_JPRB, 0.10070E+01_JPRB ,&
&  0.10304E+01_JPRB, 0.97877E+00_JPRB, 0.10322E+01_JPRB, 0.88532E+00_JPRB, 0.85909E+00_JPRB, 0.90932E+00_JPRB, 0.89487E+00_JPRB ,&
&  0.82083E+00_JPRB, 0.84851E+00_JPRB, 0.89182E+00_JPRB, 0.93085E+00_JPRB, 0.92103E+00_JPRB, 0.89170E+00_JPRB, 0.86691E+00_JPRB ,&
&  0.82658E+00_JPRB, 0.86282E+00_JPRB, 0.82329E+00_JPRB, 0.72841E+00_JPRB, 0.72689E+00_JPRB, 0.65506E+00_JPRB, 0.71491E+00_JPRB ,&
&  0.77230E+00_JPRB, 0.72499E+00_JPRB, 0.74174E+00_JPRB, 0.77302E+00_JPRB, 0.74891E+00_JPRB, 0.73513E+00_JPRB, 0.73622E+00_JPRB ,&
&  0.76388E+00_JPRB, 0.78601E+00_JPRB, 0.77578E+00_JPRB, 0.75350E+00_JPRB, 0.73954E+00_JPRB, 0.72540E+00_JPRB, 0.70450E+00_JPRB ,&
&  0.69115E+00_JPRB, 0.68189E+00_JPRB, 0.67585E+00_JPRB, 0.67193E+00_JPRB, 0.67489E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.67492E+00_JPRB, 0.67358E+00_JPRB, 0.64745E+00_JPRB, 0.61923E+00_JPRB, 0.60522E+00_JPRB, 0.61320E+00_JPRB, 0.64430E+00_JPRB ,&
&  0.85279E+00_JPRB, 0.74618E+00_JPRB, 0.66542E+00_JPRB, 0.62625E+00_JPRB, 0.60316E+00_JPRB, 0.58513E+00_JPRB, 0.56696E+00_JPRB ,&
&  0.55642E+00_JPRB, 0.55848E+00_JPRB, 0.59574E+00_JPRB, 0.62211E+00_JPRB, 0.66646E+00_JPRB, 0.76542E+00_JPRB, 0.68680E+00_JPRB ,&
&  0.96835E+00_JPRB, 0.18964E+01_JPRB, 0.18107E+01_JPRB, 0.17620E+01_JPRB, 0.16181E+01_JPRB, 0.15306E+01_JPRB, 0.12296E+01_JPRB ,&
&  0.11291E+01_JPRB, 0.11294E+01_JPRB, 0.11360E+01_JPRB, 0.11509E+01_JPRB, 0.11700E+01_JPRB, 0.11809E+01_JPRB, 0.11597E+01_JPRB ,&
&  0.11294E+01_JPRB, 0.10945E+01_JPRB, 0.10898E+01_JPRB, 0.10820E+01_JPRB, 0.10748E+01_JPRB, 0.10819E+01_JPRB, 0.10819E+01_JPRB ,&
&  0.10758E+01_JPRB, 0.11075E+01_JPRB, 0.11264E+01_JPRB, 0.10366E+01_JPRB, 0.10193E+01_JPRB, 0.10214E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.96863E+00_JPRB, 0.91426E+00_JPRB, 0.98004E+00_JPRB, 0.10552E+01_JPRB, 0.11046E+01_JPRB, 0.11158E+01_JPRB, 0.10926E+01_JPRB ,&
&  0.11017E+01_JPRB, 0.10379E+01_JPRB, 0.98935E+00_JPRB, 0.90463E+00_JPRB, 0.90861E+00_JPRB, 0.93911E+00_JPRB, 0.95259E+00_JPRB ,&
&  0.93668E+00_JPRB, 0.88317E+00_JPRB, 0.87980E+00_JPRB, 0.88140E+00_JPRB, 0.89725E+00_JPRB, 0.87114E+00_JPRB, 0.91817E+00_JPRB ,&
&  0.99331E+00_JPRB, 0.10853E+01_JPRB, 0.10160E+01_JPRB, 0.86902E+00_JPRB, 0.72540E+00_JPRB, 0.55279E+00_JPRB, 0.59005E+00_JPRB ,&
&  0.68764E+00_JPRB, 0.66351E+00_JPRB, 0.64329E+00_JPRB, 0.74996E+00_JPRB, 0.74466E+00_JPRB, 0.75494E+00_JPRB, 0.74813E+00_JPRB ,&
&  0.73600E+00_JPRB, 0.75692E+00_JPRB, 0.76898E+00_JPRB, 0.75861E+00_JPRB, 0.73885E+00_JPRB, 0.72053E+00_JPRB, 0.69260E+00_JPRB ,&
&  0.66721E+00_JPRB, 0.65575E+00_JPRB, 0.65625E+00_JPRB, 0.66371E+00_JPRB, 0.67366E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.66439E+00_JPRB, 0.67121E+00_JPRB, 0.64481E+00_JPRB, 0.60621E+00_JPRB, 0.58505E+00_JPRB, 0.58147E+00_JPRB, 0.58981E+00_JPRB ,&
&  0.67331E+00_JPRB, 0.67921E+00_JPRB, 0.64867E+00_JPRB, 0.61605E+00_JPRB, 0.60162E+00_JPRB, 0.57989E+00_JPRB, 0.56463E+00_JPRB ,&
&  0.54217E+00_JPRB, 0.53638E+00_JPRB, 0.59325E+00_JPRB, 0.61739E+00_JPRB, 0.64246E+00_JPRB, 0.54738E+00_JPRB, 0.64288E+00_JPRB ,&
&  0.11621E+01_JPRB, 0.19851E+01_JPRB, 0.19266E+01_JPRB, 0.19747E+01_JPRB, 0.19501E+01_JPRB, 0.19664E+01_JPRB, 0.14065E+01_JPRB ,&
&  0.12517E+01_JPRB, 0.12328E+01_JPRB, 0.12147E+01_JPRB, 0.12539E+01_JPRB, 0.12699E+01_JPRB, 0.12717E+01_JPRB, 0.12634E+01_JPRB ,&
&  0.12148E+01_JPRB, 0.11471E+01_JPRB, 0.11158E+01_JPRB, 0.11033E+01_JPRB, 0.10733E+01_JPRB, 0.10522E+01_JPRB, 0.10643E+01_JPRB ,&
&  0.10725E+01_JPRB, 0.10708E+01_JPRB, 0.10924E+01_JPRB, 0.10376E+01_JPRB, 0.97788E+00_JPRB, 0.96369E+00_JPRB, 0.98979E+00_JPRB ,&
&  0.98293E+00_JPRB, 0.99638E+00_JPRB, 0.10192E+01_JPRB, 0.10411E+01_JPRB, 0.11168E+01_JPRB, 0.11850E+01_JPRB, 0.12125E+01_JPRB ,&
&  0.12115E+01_JPRB, 0.11245E+01_JPRB, 0.98632E+00_JPRB, 0.96839E+00_JPRB, 0.10185E+01_JPRB, 0.10302E+01_JPRB, 0.94249E+00_JPRB ,&
&  0.86115E+00_JPRB, 0.77404E+00_JPRB, 0.76039E+00_JPRB, 0.80294E+00_JPRB, 0.87043E+00_JPRB, 0.90352E+00_JPRB, 0.10385E+01_JPRB ,&
&  0.10116E+01_JPRB, 0.67821E+00_JPRB, 0.60277E+00_JPRB, 0.72310E+00_JPRB, 0.58375E+00_JPRB, 0.46760E+00_JPRB, 0.56068E+00_JPRB ,&
&  0.63628E+00_JPRB, 0.66725E+00_JPRB, 0.65637E+00_JPRB, 0.64539E+00_JPRB, 0.73571E+00_JPRB, 0.80248E+00_JPRB, 0.83648E+00_JPRB ,&
&  0.80955E+00_JPRB, 0.79247E+00_JPRB, 0.76622E+00_JPRB, 0.74756E+00_JPRB, 0.73810E+00_JPRB, 0.72509E+00_JPRB, 0.69351E+00_JPRB ,&
&  0.67191E+00_JPRB, 0.65429E+00_JPRB, 0.64574E+00_JPRB, 0.65436E+00_JPRB, 0.66144E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.65845E+00_JPRB, 0.65115E+00_JPRB, 0.62437E+00_JPRB, 0.58811E+00_JPRB, 0.56973E+00_JPRB, 0.56162E+00_JPRB, 0.55870E+00_JPRB ,&
&  0.58426E+00_JPRB, 0.60697E+00_JPRB, 0.61294E+00_JPRB, 0.60391E+00_JPRB, 0.59425E+00_JPRB, 0.57675E+00_JPRB, 0.56030E+00_JPRB ,&
&  0.53951E+00_JPRB, 0.52488E+00_JPRB, 0.60588E+00_JPRB, 0.66558E+00_JPRB, 0.66170E+00_JPRB, 0.58239E+00_JPRB, 0.74760E+00_JPRB ,&
&  0.11944E+01_JPRB, 0.19772E+01_JPRB, 0.23484E+01_JPRB, 0.23999E+01_JPRB, 0.22224E+01_JPRB, 0.21468E+01_JPRB, 0.16764E+01_JPRB ,&
&  0.14580E+01_JPRB, 0.14183E+01_JPRB, 0.13532E+01_JPRB, 0.13540E+01_JPRB, 0.13812E+01_JPRB, 0.13710E+01_JPRB, 0.13265E+01_JPRB ,&
&  0.12827E+01_JPRB, 0.12104E+01_JPRB, 0.11638E+01_JPRB, 0.11420E+01_JPRB, 0.11048E+01_JPRB, 0.10512E+01_JPRB, 0.10271E+01_JPRB ,&
&  0.10467E+01_JPRB, 0.10453E+01_JPRB, 0.10584E+01_JPRB, 0.10709E+01_JPRB, 0.87565E+00_JPRB, 0.91910E+00_JPRB, 0.93837E+00_JPRB ,&
&  0.97001E+00_JPRB, 0.10082E+01_JPRB, 0.99751E+00_JPRB, 0.10666E+01_JPRB, 0.11540E+01_JPRB, 0.11825E+01_JPRB, 0.12452E+01_JPRB ,&
&  0.13142E+01_JPRB, 0.12351E+01_JPRB, 0.10894E+01_JPRB, 0.10813E+01_JPRB, 0.11433E+01_JPRB, 0.11231E+01_JPRB, 0.82315E+00_JPRB ,&
&  0.76963E+00_JPRB, 0.86122E+00_JPRB, 0.81322E+00_JPRB, 0.73293E+00_JPRB, 0.81960E+00_JPRB, 0.89038E+00_JPRB, 0.79862E+00_JPRB ,&
&  0.47308E+00_JPRB, 0.39891E+00_JPRB, 0.40184E+00_JPRB, 0.41096E+00_JPRB, 0.40394E+00_JPRB, 0.41996E+00_JPRB, 0.63625E+00_JPRB ,&
&  0.62037E+00_JPRB, 0.65949E+00_JPRB, 0.65592E+00_JPRB, 0.64598E+00_JPRB, 0.70974E+00_JPRB, 0.80464E+00_JPRB, 0.10433E+01_JPRB ,&
&  0.94168E+00_JPRB, 0.89331E+00_JPRB, 0.85970E+00_JPRB, 0.80413E+00_JPRB, 0.75834E+00_JPRB, 0.72261E+00_JPRB, 0.69457E+00_JPRB ,&
&  0.67133E+00_JPRB, 0.65444E+00_JPRB, 0.64778E+00_JPRB, 0.65110E+00_JPRB, 0.65793E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.65718E+00_JPRB, 0.63190E+00_JPRB, 0.59647E+00_JPRB, 0.55848E+00_JPRB, 0.54376E+00_JPRB, 0.53388E+00_JPRB, 0.52748E+00_JPRB ,&
&  0.53874E+00_JPRB, 0.55426E+00_JPRB, 0.57033E+00_JPRB, 0.57702E+00_JPRB, 0.57339E+00_JPRB, 0.56198E+00_JPRB, 0.54718E+00_JPRB ,&
&  0.52864E+00_JPRB, 0.51953E+00_JPRB, 0.65668E+00_JPRB, 0.68629E+00_JPRB, 0.72374E+00_JPRB, 0.67923E+00_JPRB, 0.80489E+00_JPRB ,&
&  0.11708E+01_JPRB, 0.18549E+01_JPRB, 0.24058E+01_JPRB, 0.25063E+01_JPRB, 0.26115E+01_JPRB, 0.25765E+01_JPRB, 0.23158E+01_JPRB ,&
&  0.20135E+01_JPRB, 0.18303E+01_JPRB, 0.16610E+01_JPRB, 0.15530E+01_JPRB, 0.14717E+01_JPRB, 0.14268E+01_JPRB, 0.13626E+01_JPRB ,&
&  0.13064E+01_JPRB, 0.12777E+01_JPRB, 0.12455E+01_JPRB, 0.12280E+01_JPRB, 0.11766E+01_JPRB, 0.11101E+01_JPRB, 0.10574E+01_JPRB ,&
&  0.10473E+01_JPRB, 0.10454E+01_JPRB, 0.10407E+01_JPRB, 0.10642E+01_JPRB, 0.94258E+00_JPRB, 0.84977E+00_JPRB, 0.90871E+00_JPRB ,&
&  0.97829E+00_JPRB, 0.10639E+01_JPRB, 0.10677E+01_JPRB, 0.11591E+01_JPRB, 0.11936E+01_JPRB, 0.12015E+01_JPRB, 0.13102E+01_JPRB ,&
&  0.14052E+01_JPRB, 0.14791E+01_JPRB, 0.12308E+01_JPRB, 0.12126E+01_JPRB, 0.12909E+01_JPRB, 0.10297E+01_JPRB, 0.89571E+00_JPRB ,&
&  0.95509E+00_JPRB, 0.87340E+00_JPRB, 0.80725E+00_JPRB, 0.69090E+00_JPRB, 0.77607E+00_JPRB, 0.74264E+00_JPRB, 0.47781E+00_JPRB ,&
&  0.40225E+00_JPRB, 0.37921E+00_JPRB, 0.36561E+00_JPRB, 0.36617E+00_JPRB, 0.36091E+00_JPRB, 0.39090E+00_JPRB, 0.53945E+00_JPRB ,&
&  0.58108E+00_JPRB, 0.62924E+00_JPRB, 0.64965E+00_JPRB, 0.65497E+00_JPRB, 0.68320E+00_JPRB, 0.72612E+00_JPRB, 0.10391E+01_JPRB ,&
&  0.11599E+01_JPRB, 0.11091E+01_JPRB, 0.99189E+00_JPRB, 0.91746E+00_JPRB, 0.88213E+00_JPRB, 0.82486E+00_JPRB, 0.76074E+00_JPRB ,&
&  0.70300E+00_JPRB, 0.67179E+00_JPRB, 0.65812E+00_JPRB, 0.66035E+00_JPRB, 0.67299E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.66450E+00_JPRB, 0.64487E+00_JPRB, 0.60257E+00_JPRB, 0.55648E+00_JPRB, 0.53297E+00_JPRB, 0.51389E+00_JPRB, 0.50317E+00_JPRB ,&
&  0.50796E+00_JPRB, 0.51798E+00_JPRB, 0.52801E+00_JPRB, 0.53667E+00_JPRB, 0.53511E+00_JPRB, 0.52890E+00_JPRB, 0.52183E+00_JPRB ,&
&  0.50689E+00_JPRB, 0.53371E+00_JPRB, 0.62569E+00_JPRB, 0.58809E+00_JPRB, 0.59962E+00_JPRB, 0.60987E+00_JPRB, 0.70581E+00_JPRB ,&
&  0.11274E+01_JPRB, 0.17656E+01_JPRB, 0.23336E+01_JPRB, 0.26644E+01_JPRB, 0.29341E+01_JPRB, 0.28487E+01_JPRB, 0.30813E+01_JPRB ,&
&  0.27544E+01_JPRB, 0.23934E+01_JPRB, 0.20841E+01_JPRB, 0.18694E+01_JPRB, 0.16653E+01_JPRB, 0.15275E+01_JPRB, 0.14578E+01_JPRB ,&
&  0.13742E+01_JPRB, 0.13147E+01_JPRB, 0.12931E+01_JPRB, 0.12685E+01_JPRB, 0.12284E+01_JPRB, 0.11787E+01_JPRB, 0.11330E+01_JPRB ,&
&  0.11149E+01_JPRB, 0.11019E+01_JPRB, 0.10759E+01_JPRB, 0.10458E+01_JPRB, 0.10712E+01_JPRB, 0.10147E+01_JPRB, 0.97265E+00_JPRB ,&
&  0.96417E+00_JPRB, 0.99341E+00_JPRB, 0.11339E+01_JPRB, 0.11554E+01_JPRB, 0.11916E+01_JPRB, 0.12617E+01_JPRB, 0.13525E+01_JPRB ,&
&  0.14641E+01_JPRB, 0.15437E+01_JPRB, 0.13835E+01_JPRB, 0.13905E+01_JPRB, 0.12853E+01_JPRB, 0.10400E+01_JPRB, 0.10411E+01_JPRB ,&
&  0.10832E+01_JPRB, 0.96583E+00_JPRB, 0.82204E+00_JPRB, 0.66242E+00_JPRB, 0.64910E+00_JPRB, 0.52589E+00_JPRB, 0.39468E+00_JPRB ,&
&  0.38282E+00_JPRB, 0.37387E+00_JPRB, 0.35671E+00_JPRB, 0.38360E+00_JPRB, 0.37810E+00_JPRB, 0.41749E+00_JPRB, 0.52260E+00_JPRB ,&
&  0.59158E+00_JPRB, 0.59100E+00_JPRB, 0.62105E+00_JPRB, 0.64645E+00_JPRB, 0.66432E+00_JPRB, 0.67933E+00_JPRB, 0.94376E+00_JPRB ,&
&  0.98575E+00_JPRB, 0.10381E+01_JPRB, 0.10122E+01_JPRB, 0.92395E+00_JPRB, 0.89470E+00_JPRB, 0.87849E+00_JPRB, 0.83938E+00_JPRB ,&
&  0.77740E+00_JPRB, 0.71513E+00_JPRB, 0.68800E+00_JPRB, 0.68198E+00_JPRB, 0.67868E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.68221E+00_JPRB, 0.66911E+00_JPRB, 0.62928E+00_JPRB, 0.58216E+00_JPRB, 0.54718E+00_JPRB, 0.52120E+00_JPRB, 0.50797E+00_JPRB ,&
&  0.51157E+00_JPRB, 0.51389E+00_JPRB, 0.51785E+00_JPRB, 0.51951E+00_JPRB, 0.51558E+00_JPRB, 0.50766E+00_JPRB, 0.50685E+00_JPRB ,&
&  0.49532E+00_JPRB, 0.63788E+00_JPRB, 0.48306E+00_JPRB, 0.42307E+00_JPRB, 0.49361E+00_JPRB, 0.57954E+00_JPRB, 0.60475E+00_JPRB ,&
&  0.10338E+01_JPRB, 0.16827E+01_JPRB, 0.22124E+01_JPRB, 0.28907E+01_JPRB, 0.38350E+01_JPRB, 0.36439E+01_JPRB, 0.36383E+01_JPRB ,&
&  0.34524E+01_JPRB, 0.30339E+01_JPRB, 0.26049E+01_JPRB, 0.22050E+01_JPRB, 0.18641E+01_JPRB, 0.16591E+01_JPRB, 0.15381E+01_JPRB ,&
&  0.14327E+01_JPRB, 0.13527E+01_JPRB, 0.13064E+01_JPRB, 0.12896E+01_JPRB, 0.12491E+01_JPRB, 0.12116E+01_JPRB, 0.11875E+01_JPRB ,&
&  0.11759E+01_JPRB, 0.11583E+01_JPRB, 0.11282E+01_JPRB, 0.11061E+01_JPRB, 0.10279E+01_JPRB, 0.97287E+00_JPRB, 0.11361E+01_JPRB ,&
&  0.11395E+01_JPRB, 0.11220E+01_JPRB, 0.11197E+01_JPRB, 0.12315E+01_JPRB, 0.12970E+01_JPRB, 0.13361E+01_JPRB, 0.14652E+01_JPRB ,&
&  0.14001E+01_JPRB, 0.13774E+01_JPRB, 0.13832E+01_JPRB, 0.13609E+01_JPRB, 0.12328E+01_JPRB, 0.12631E+01_JPRB, 0.14779E+01_JPRB ,&
&  0.12192E+01_JPRB, 0.11829E+01_JPRB, 0.11353E+01_JPRB, 0.98167E+00_JPRB, 0.63333E+00_JPRB, 0.51468E+00_JPRB, 0.65816E+00_JPRB ,&
&  0.68677E+00_JPRB, 0.50684E+00_JPRB, 0.43118E+00_JPRB, 0.53273E+00_JPRB, 0.45381E+00_JPRB, 0.48326E+00_JPRB, 0.57902E+00_JPRB ,&
&  0.57303E+00_JPRB, 0.53053E+00_JPRB, 0.59122E+00_JPRB, 0.65606E+00_JPRB, 0.70351E+00_JPRB, 0.67199E+00_JPRB, 0.83921E+00_JPRB ,&
&  0.91579E+00_JPRB, 0.94360E+00_JPRB, 0.99766E+00_JPRB, 0.93880E+00_JPRB, 0.88996E+00_JPRB, 0.86207E+00_JPRB, 0.83700E+00_JPRB ,&
&  0.79875E+00_JPRB, 0.75189E+00_JPRB, 0.72352E+00_JPRB, 0.71480E+00_JPRB, 0.69733E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.69263E+00_JPRB, 0.68033E+00_JPRB, 0.64020E+00_JPRB, 0.59942E+00_JPRB, 0.54980E+00_JPRB, 0.50930E+00_JPRB, 0.49744E+00_JPRB ,&
&  0.50492E+00_JPRB, 0.51361E+00_JPRB, 0.51664E+00_JPRB, 0.51325E+00_JPRB, 0.50657E+00_JPRB, 0.49959E+00_JPRB, 0.49500E+00_JPRB ,&
&  0.48742E+00_JPRB, 0.48306E+00_JPRB, 0.38001E+00_JPRB, 0.34570E+00_JPRB, 0.48816E+00_JPRB, 0.52754E+00_JPRB, 0.59922E+00_JPRB ,&
&  0.90505E+00_JPRB, 0.14233E+01_JPRB, 0.20574E+01_JPRB, 0.28092E+01_JPRB, 0.35426E+01_JPRB, 0.38924E+01_JPRB, 0.38608E+01_JPRB ,&
&  0.38972E+01_JPRB, 0.34790E+01_JPRB, 0.29299E+01_JPRB, 0.24743E+01_JPRB, 0.21569E+01_JPRB, 0.19371E+01_JPRB, 0.17182E+01_JPRB ,&
&  0.15290E+01_JPRB, 0.14035E+01_JPRB, 0.13333E+01_JPRB, 0.12857E+01_JPRB, 0.12585E+01_JPRB, 0.12161E+01_JPRB, 0.11741E+01_JPRB ,&
&  0.11748E+01_JPRB, 0.11921E+01_JPRB, 0.11965E+01_JPRB, 0.11624E+01_JPRB, 0.98728E+00_JPRB, 0.92608E+00_JPRB, 0.10605E+01_JPRB ,&
&  0.11858E+01_JPRB, 0.11447E+01_JPRB, 0.11661E+01_JPRB, 0.12508E+01_JPRB, 0.13356E+01_JPRB, 0.13264E+01_JPRB, 0.15270E+01_JPRB ,&
&  0.14750E+01_JPRB, 0.13861E+01_JPRB, 0.13111E+01_JPRB, 0.11502E+01_JPRB, 0.11312E+01_JPRB, 0.14700E+01_JPRB, 0.15526E+01_JPRB ,&
&  0.14636E+01_JPRB, 0.13690E+01_JPRB, 0.12491E+01_JPRB, 0.10365E+01_JPRB, 0.78366E+00_JPRB, 0.65513E+00_JPRB, 0.74196E+00_JPRB ,&
&  0.79673E+00_JPRB, 0.81160E+00_JPRB, 0.77240E+00_JPRB, 0.69106E+00_JPRB, 0.60967E+00_JPRB, 0.66731E+00_JPRB, 0.61459E+00_JPRB ,&
&  0.52299E+00_JPRB, 0.47764E+00_JPRB, 0.52403E+00_JPRB, 0.61660E+00_JPRB, 0.66255E+00_JPRB, 0.63187E+00_JPRB, 0.77566E+00_JPRB ,&
&  0.82905E+00_JPRB, 0.88216E+00_JPRB, 0.90100E+00_JPRB, 0.87864E+00_JPRB, 0.84887E+00_JPRB, 0.82318E+00_JPRB, 0.78959E+00_JPRB ,&
&  0.76690E+00_JPRB, 0.75049E+00_JPRB, 0.73125E+00_JPRB, 0.72155E+00_JPRB, 0.70323E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.70758E+00_JPRB, 0.68840E+00_JPRB, 0.64608E+00_JPRB, 0.60368E+00_JPRB, 0.54921E+00_JPRB, 0.49541E+00_JPRB, 0.48006E+00_JPRB ,&
&  0.48650E+00_JPRB, 0.49529E+00_JPRB, 0.49655E+00_JPRB, 0.49013E+00_JPRB, 0.48629E+00_JPRB, 0.48052E+00_JPRB, 0.47797E+00_JPRB ,&
&  0.47736E+00_JPRB, 0.43711E+00_JPRB, 0.34226E+00_JPRB, 0.34398E+00_JPRB, 0.42259E+00_JPRB, 0.47266E+00_JPRB, 0.59391E+00_JPRB ,&
&  0.77808E+00_JPRB, 0.12505E+01_JPRB, 0.17207E+01_JPRB, 0.22963E+01_JPRB, 0.28934E+01_JPRB, 0.33336E+01_JPRB, 0.33329E+01_JPRB ,&
&  0.31175E+01_JPRB, 0.31086E+01_JPRB, 0.28740E+01_JPRB, 0.25627E+01_JPRB, 0.22869E+01_JPRB, 0.20694E+01_JPRB, 0.18681E+01_JPRB ,&
&  0.16736E+01_JPRB, 0.14911E+01_JPRB, 0.13787E+01_JPRB, 0.13093E+01_JPRB, 0.12625E+01_JPRB, 0.12226E+01_JPRB, 0.11910E+01_JPRB ,&
&  0.11813E+01_JPRB, 0.11972E+01_JPRB, 0.11913E+01_JPRB, 0.11784E+01_JPRB, 0.97502E+00_JPRB, 0.98787E+00_JPRB, 0.11263E+01_JPRB ,&
&  0.13987E+01_JPRB, 0.13082E+01_JPRB, 0.13066E+01_JPRB, 0.15019E+01_JPRB, 0.13757E+01_JPRB, 0.14004E+01_JPRB, 0.16683E+01_JPRB ,&
&  0.19060E+01_JPRB, 0.19313E+01_JPRB, 0.19081E+01_JPRB, 0.14892E+01_JPRB, 0.11951E+01_JPRB, 0.16428E+01_JPRB, 0.16050E+01_JPRB ,&
&  0.15015E+01_JPRB, 0.14080E+01_JPRB, 0.13009E+01_JPRB, 0.12137E+01_JPRB, 0.97346E+00_JPRB, 0.82893E+00_JPRB, 0.71026E+00_JPRB ,&
&  0.74787E+00_JPRB, 0.72422E+00_JPRB, 0.78363E+00_JPRB, 0.74587E+00_JPRB, 0.69410E+00_JPRB, 0.63179E+00_JPRB, 0.53253E+00_JPRB ,&
&  0.47133E+00_JPRB, 0.43035E+00_JPRB, 0.42537E+00_JPRB, 0.49386E+00_JPRB, 0.55167E+00_JPRB, 0.53190E+00_JPRB, 0.62831E+00_JPRB ,&
&  0.74067E+00_JPRB, 0.82819E+00_JPRB, 0.81294E+00_JPRB, 0.84217E+00_JPRB, 0.84032E+00_JPRB, 0.82779E+00_JPRB, 0.79484E+00_JPRB ,&
&  0.76497E+00_JPRB, 0.74600E+00_JPRB, 0.73505E+00_JPRB, 0.72566E+00_JPRB, 0.71349E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.70521E+00_JPRB, 0.68624E+00_JPRB, 0.64577E+00_JPRB, 0.60416E+00_JPRB, 0.54123E+00_JPRB, 0.47886E+00_JPRB, 0.46576E+00_JPRB ,&
&  0.47743E+00_JPRB, 0.48120E+00_JPRB, 0.47594E+00_JPRB, 0.45929E+00_JPRB, 0.45632E+00_JPRB, 0.46338E+00_JPRB, 0.46675E+00_JPRB ,&
&  0.47021E+00_JPRB, 0.56015E+00_JPRB, 0.44391E+00_JPRB, 0.36579E+00_JPRB, 0.36043E+00_JPRB, 0.43376E+00_JPRB, 0.57461E+00_JPRB ,&
&  0.73379E+00_JPRB, 0.10237E+01_JPRB, 0.13619E+01_JPRB, 0.16545E+01_JPRB, 0.20353E+01_JPRB, 0.23123E+01_JPRB, 0.24380E+01_JPRB ,&
&  0.25076E+01_JPRB, 0.21515E+01_JPRB, 0.21760E+01_JPRB, 0.20131E+01_JPRB, 0.18664E+01_JPRB, 0.17385E+01_JPRB, 0.16268E+01_JPRB ,&
&  0.14878E+01_JPRB, 0.13803E+01_JPRB, 0.12585E+01_JPRB, 0.11753E+01_JPRB, 0.11296E+01_JPRB, 0.10878E+01_JPRB, 0.10534E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.11321E+01_JPRB, 0.11815E+01_JPRB, 0.11736E+01_JPRB, 0.12011E+01_JPRB, 0.12903E+01_JPRB, 0.14224E+01_JPRB ,&
&  0.14070E+01_JPRB, 0.12785E+01_JPRB, 0.15415E+01_JPRB, 0.17522E+01_JPRB, 0.19611E+01_JPRB, 0.20281E+01_JPRB, 0.20874E+01_JPRB ,&
&  0.22734E+01_JPRB, 0.21866E+01_JPRB, 0.20577E+01_JPRB, 0.17587E+01_JPRB, 0.17693E+01_JPRB, 0.17523E+01_JPRB, 0.16527E+01_JPRB ,&
&  0.15038E+01_JPRB, 0.14542E+01_JPRB, 0.13509E+01_JPRB, 0.12525E+01_JPRB, 0.11797E+01_JPRB, 0.11350E+01_JPRB, 0.10129E+01_JPRB ,&
&  0.88048E+00_JPRB, 0.90377E+00_JPRB, 0.82989E+00_JPRB, 0.71901E+00_JPRB, 0.58247E+00_JPRB, 0.51970E+00_JPRB, 0.45922E+00_JPRB ,&
&  0.42568E+00_JPRB, 0.40409E+00_JPRB, 0.39589E+00_JPRB, 0.42639E+00_JPRB, 0.46588E+00_JPRB, 0.47679E+00_JPRB, 0.49435E+00_JPRB ,&
&  0.52840E+00_JPRB, 0.66124E+00_JPRB, 0.70377E+00_JPRB, 0.71810E+00_JPRB, 0.76636E+00_JPRB, 0.79570E+00_JPRB, 0.79564E+00_JPRB ,&
&  0.76917E+00_JPRB, 0.74850E+00_JPRB, 0.73483E+00_JPRB, 0.72558E+00_JPRB, 0.71337E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.69520E+00_JPRB, 0.67868E+00_JPRB, 0.64675E+00_JPRB, 0.60358E+00_JPRB, 0.53754E+00_JPRB, 0.47739E+00_JPRB, 0.46216E+00_JPRB ,&
&  0.46065E+00_JPRB, 0.45842E+00_JPRB, 0.45150E+00_JPRB, 0.44744E+00_JPRB, 0.44998E+00_JPRB, 0.45419E+00_JPRB, 0.47251E+00_JPRB ,&
&  0.51780E+00_JPRB, 0.83872E+00_JPRB, 0.61773E+00_JPRB, 0.49920E+00_JPRB, 0.42473E+00_JPRB, 0.49413E+00_JPRB, 0.58888E+00_JPRB ,&
&  0.75502E+00_JPRB, 0.88259E+00_JPRB, 0.99337E+00_JPRB, 0.12101E+01_JPRB, 0.14380E+01_JPRB, 0.14921E+01_JPRB, 0.16255E+01_JPRB ,&
&  0.16315E+01_JPRB, 0.16247E+01_JPRB, 0.14946E+01_JPRB, 0.15009E+01_JPRB, 0.14410E+01_JPRB, 0.12987E+01_JPRB, 0.13153E+01_JPRB ,&
&  0.12246E+01_JPRB, 0.11480E+01_JPRB, 0.10669E+01_JPRB, 0.10216E+01_JPRB, 0.10177E+01_JPRB, 0.10111E+01_JPRB, 0.97708E+00_JPRB ,&
&  0.99374E+00_JPRB, 0.10430E+01_JPRB, 0.11210E+01_JPRB, 0.11828E+01_JPRB, 0.12502E+01_JPRB, 0.13974E+01_JPRB, 0.15312E+01_JPRB ,&
&  0.18580E+01_JPRB, 0.17733E+01_JPRB, 0.15150E+01_JPRB, 0.18895E+01_JPRB, 0.24736E+01_JPRB, 0.25032E+01_JPRB, 0.22830E+01_JPRB ,&
&  0.24091E+01_JPRB, 0.22871E+01_JPRB, 0.20619E+01_JPRB, 0.18741E+01_JPRB, 0.17302E+01_JPRB, 0.16798E+01_JPRB, 0.16420E+01_JPRB ,&
&  0.14773E+01_JPRB, 0.13853E+01_JPRB, 0.13286E+01_JPRB, 0.12085E+01_JPRB, 0.11240E+01_JPRB, 0.10757E+01_JPRB, 0.10417E+01_JPRB ,&
&  0.94034E+00_JPRB, 0.79655E+00_JPRB, 0.60059E+00_JPRB, 0.57385E+00_JPRB, 0.44953E+00_JPRB, 0.39943E+00_JPRB, 0.39852E+00_JPRB ,&
&  0.38585E+00_JPRB, 0.40112E+00_JPRB, 0.41910E+00_JPRB, 0.40742E+00_JPRB, 0.41265E+00_JPRB, 0.39883E+00_JPRB, 0.41339E+00_JPRB ,&
&  0.43008E+00_JPRB, 0.46081E+00_JPRB, 0.57537E+00_JPRB, 0.61262E+00_JPRB, 0.65727E+00_JPRB, 0.69160E+00_JPRB, 0.71097E+00_JPRB ,&
&  0.71724E+00_JPRB, 0.71327E+00_JPRB, 0.71020E+00_JPRB, 0.70453E+00_JPRB, 0.70330E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.63317E+00_JPRB, 0.61902E+00_JPRB, 0.59670E+00_JPRB, 0.56231E+00_JPRB, 0.51398E+00_JPRB, 0.46416E+00_JPRB, 0.44112E+00_JPRB ,&
&  0.43792E+00_JPRB, 0.42937E+00_JPRB, 0.42556E+00_JPRB, 0.42931E+00_JPRB, 0.43611E+00_JPRB, 0.44546E+00_JPRB, 0.47798E+00_JPRB ,&
&  0.56441E+00_JPRB, 0.68093E+00_JPRB, 0.55429E+00_JPRB, 0.46705E+00_JPRB, 0.53619E+00_JPRB, 0.53755E+00_JPRB, 0.59716E+00_JPRB ,&
&  0.66567E+00_JPRB, 0.80621E+00_JPRB, 0.84629E+00_JPRB, 0.89360E+00_JPRB, 0.10227E+01_JPRB, 0.10988E+01_JPRB, 0.11451E+01_JPRB ,&
&  0.11261E+01_JPRB, 0.10618E+01_JPRB, 0.10222E+01_JPRB, 0.10437E+01_JPRB, 0.11177E+01_JPRB, 0.10686E+01_JPRB, 0.10746E+01_JPRB ,&
&  0.10176E+01_JPRB, 0.96512E+00_JPRB, 0.89489E+00_JPRB, 0.87083E+00_JPRB, 0.88788E+00_JPRB, 0.92801E+00_JPRB, 0.95141E+00_JPRB ,&
&  0.10025E+01_JPRB, 0.10610E+01_JPRB, 0.11112E+01_JPRB, 0.12041E+01_JPRB, 0.13290E+01_JPRB, 0.16336E+01_JPRB, 0.20678E+01_JPRB ,&
&  0.23988E+01_JPRB, 0.25267E+01_JPRB, 0.26505E+01_JPRB, 0.29448E+01_JPRB, 0.29754E+01_JPRB, 0.28121E+01_JPRB, 0.26024E+01_JPRB ,&
&  0.23646E+01_JPRB, 0.21997E+01_JPRB, 0.20421E+01_JPRB, 0.18752E+01_JPRB, 0.17162E+01_JPRB, 0.16360E+01_JPRB, 0.15466E+01_JPRB ,&
&  0.13225E+01_JPRB, 0.12038E+01_JPRB, 0.11886E+01_JPRB, 0.10996E+01_JPRB, 0.10332E+01_JPRB, 0.95700E+00_JPRB, 0.93898E+00_JPRB ,&
&  0.82020E+00_JPRB, 0.55977E+00_JPRB, 0.47479E+00_JPRB, 0.48691E+00_JPRB, 0.40928E+00_JPRB, 0.38260E+00_JPRB, 0.39314E+00_JPRB ,&
&  0.38638E+00_JPRB, 0.39355E+00_JPRB, 0.40475E+00_JPRB, 0.38428E+00_JPRB, 0.37038E+00_JPRB, 0.36530E+00_JPRB, 0.35221E+00_JPRB ,&
&  0.35423E+00_JPRB, 0.37678E+00_JPRB, 0.47194E+00_JPRB, 0.53607E+00_JPRB, 0.57979E+00_JPRB, 0.60417E+00_JPRB, 0.62562E+00_JPRB ,&
&  0.63593E+00_JPRB, 0.64263E+00_JPRB, 0.64689E+00_JPRB, 0.65299E+00_JPRB, 0.65449E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.57128E+00_JPRB, 0.56724E+00_JPRB, 0.55271E+00_JPRB, 0.53190E+00_JPRB, 0.50346E+00_JPRB, 0.46978E+00_JPRB, 0.43544E+00_JPRB ,&
&  0.43251E+00_JPRB, 0.42589E+00_JPRB, 0.42401E+00_JPRB, 0.42683E+00_JPRB, 0.43325E+00_JPRB, 0.44147E+00_JPRB, 0.45782E+00_JPRB ,&
&  0.39586E+00_JPRB, 0.44604E+00_JPRB, 0.41024E+00_JPRB, 0.46084E+00_JPRB, 0.61182E+00_JPRB, 0.53650E+00_JPRB, 0.58564E+00_JPRB ,&
&  0.64712E+00_JPRB, 0.73562E+00_JPRB, 0.78173E+00_JPRB, 0.80000E+00_JPRB, 0.84581E+00_JPRB, 0.87442E+00_JPRB, 0.86456E+00_JPRB ,&
&  0.80448E+00_JPRB, 0.71838E+00_JPRB, 0.70106E+00_JPRB, 0.70890E+00_JPRB, 0.79089E+00_JPRB, 0.85811E+00_JPRB, 0.87860E+00_JPRB ,&
&  0.85175E+00_JPRB, 0.82349E+00_JPRB, 0.79539E+00_JPRB, 0.80829E+00_JPRB, 0.83989E+00_JPRB, 0.86628E+00_JPRB, 0.91733E+00_JPRB ,&
&  0.99086E+00_JPRB, 0.10719E+01_JPRB, 0.11685E+01_JPRB, 0.12956E+01_JPRB, 0.13091E+01_JPRB, 0.19893E+01_JPRB, 0.31993E+01_JPRB ,&
&  0.34306E+01_JPRB, 0.32719E+01_JPRB, 0.33707E+01_JPRB, 0.37770E+01_JPRB, 0.36526E+01_JPRB, 0.31943E+01_JPRB, 0.25821E+01_JPRB ,&
&  0.21493E+01_JPRB, 0.19583E+01_JPRB, 0.18658E+01_JPRB, 0.17309E+01_JPRB, 0.15064E+01_JPRB, 0.14307E+01_JPRB, 0.13616E+01_JPRB ,&
&  0.11965E+01_JPRB, 0.10734E+01_JPRB, 0.10279E+01_JPRB, 0.96548E+00_JPRB, 0.93004E+00_JPRB, 0.92205E+00_JPRB, 0.89424E+00_JPRB ,&
&  0.81907E+00_JPRB, 0.67457E+00_JPRB, 0.51771E+00_JPRB, 0.44849E+00_JPRB, 0.38800E+00_JPRB, 0.38536E+00_JPRB, 0.41274E+00_JPRB ,&
&  0.39959E+00_JPRB, 0.38365E+00_JPRB, 0.36355E+00_JPRB, 0.34155E+00_JPRB, 0.32871E+00_JPRB, 0.33195E+00_JPRB, 0.31911E+00_JPRB ,&
&  0.31609E+00_JPRB, 0.34634E+00_JPRB, 0.39640E+00_JPRB, 0.45302E+00_JPRB, 0.50060E+00_JPRB, 0.54315E+00_JPRB, 0.51676E+00_JPRB ,&
&  0.57974E+00_JPRB, 0.58882E+00_JPRB, 0.59814E+00_JPRB, 0.61001E+00_JPRB, 0.59801E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.50029E+00_JPRB, 0.51077E+00_JPRB, 0.49988E+00_JPRB, 0.49595E+00_JPRB, 0.47631E+00_JPRB, 0.46089E+00_JPRB, 0.43248E+00_JPRB ,&
&  0.43865E+00_JPRB, 0.43485E+00_JPRB, 0.42640E+00_JPRB, 0.42369E+00_JPRB, 0.43214E+00_JPRB, 0.43059E+00_JPRB, 0.38103E+00_JPRB ,&
&  0.34321E+00_JPRB, 0.36141E+00_JPRB, 0.35551E+00_JPRB, 0.39490E+00_JPRB, 0.43713E+00_JPRB, 0.43848E+00_JPRB, 0.47655E+00_JPRB ,&
&  0.57095E+00_JPRB, 0.62397E+00_JPRB, 0.66440E+00_JPRB, 0.68617E+00_JPRB, 0.70009E+00_JPRB, 0.71477E+00_JPRB, 0.72259E+00_JPRB ,&
&  0.67390E+00_JPRB, 0.63097E+00_JPRB, 0.59765E+00_JPRB, 0.59456E+00_JPRB, 0.67020E+00_JPRB, 0.71424E+00_JPRB, 0.70189E+00_JPRB ,&
&  0.68401E+00_JPRB, 0.68300E+00_JPRB, 0.69682E+00_JPRB, 0.75081E+00_JPRB, 0.79042E+00_JPRB, 0.81246E+00_JPRB, 0.86466E+00_JPRB ,&
&  0.94447E+00_JPRB, 0.10551E+01_JPRB, 0.11688E+01_JPRB, 0.12952E+01_JPRB, 0.13597E+01_JPRB, 0.20477E+01_JPRB, 0.29555E+01_JPRB ,&
&  0.29832E+01_JPRB, 0.30778E+01_JPRB, 0.31481E+01_JPRB, 0.31803E+01_JPRB, 0.31009E+01_JPRB, 0.25980E+01_JPRB, 0.20173E+01_JPRB ,&
&  0.16741E+01_JPRB, 0.15262E+01_JPRB, 0.15612E+01_JPRB, 0.14721E+01_JPRB, 0.13318E+01_JPRB, 0.12064E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.10480E+01_JPRB, 0.92073E+00_JPRB, 0.86313E+00_JPRB, 0.82852E+00_JPRB, 0.81419E+00_JPRB, 0.79502E+00_JPRB, 0.76598E+00_JPRB ,&
&  0.71247E+00_JPRB, 0.62639E+00_JPRB, 0.52260E+00_JPRB, 0.45483E+00_JPRB, 0.40967E+00_JPRB, 0.41386E+00_JPRB, 0.39112E+00_JPRB ,&
&  0.35732E+00_JPRB, 0.32735E+00_JPRB, 0.31900E+00_JPRB, 0.30727E+00_JPRB, 0.30044E+00_JPRB, 0.30496E+00_JPRB, 0.30374E+00_JPRB ,&
&  0.31923E+00_JPRB, 0.34982E+00_JPRB, 0.35902E+00_JPRB, 0.38795E+00_JPRB, 0.42695E+00_JPRB, 0.48236E+00_JPRB, 0.42382E+00_JPRB ,&
&  0.45161E+00_JPRB, 0.50959E+00_JPRB, 0.52335E+00_JPRB, 0.52506E+00_JPRB, 0.50816E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.46953E+00_JPRB, 0.48155E+00_JPRB, 0.47078E+00_JPRB, 0.47786E+00_JPRB, 0.46803E+00_JPRB, 0.46506E+00_JPRB, 0.41610E+00_JPRB ,&
&  0.44394E+00_JPRB, 0.43311E+00_JPRB, 0.41862E+00_JPRB, 0.41622E+00_JPRB, 0.43156E+00_JPRB, 0.37310E+00_JPRB, 0.31798E+00_JPRB ,&
&  0.29883E+00_JPRB, 0.30865E+00_JPRB, 0.34414E+00_JPRB, 0.36414E+00_JPRB, 0.37409E+00_JPRB, 0.38796E+00_JPRB, 0.39595E+00_JPRB ,&
&  0.44767E+00_JPRB, 0.47936E+00_JPRB, 0.53060E+00_JPRB, 0.56537E+00_JPRB, 0.56947E+00_JPRB, 0.58540E+00_JPRB, 0.61511E+00_JPRB ,&
&  0.58598E+00_JPRB, 0.56875E+00_JPRB, 0.56473E+00_JPRB, 0.51374E+00_JPRB, 0.58065E+00_JPRB, 0.56429E+00_JPRB, 0.54239E+00_JPRB ,&
&  0.54911E+00_JPRB, 0.58299E+00_JPRB, 0.62132E+00_JPRB, 0.66230E+00_JPRB, 0.69295E+00_JPRB, 0.72758E+00_JPRB, 0.77487E+00_JPRB ,&
&  0.84427E+00_JPRB, 0.97702E+00_JPRB, 0.11251E+01_JPRB, 0.12754E+01_JPRB, 0.14660E+01_JPRB, 0.19930E+01_JPRB, 0.26265E+01_JPRB ,&
&  0.27225E+01_JPRB, 0.28132E+01_JPRB, 0.28733E+01_JPRB, 0.24731E+01_JPRB, 0.25114E+01_JPRB, 0.23272E+01_JPRB, 0.18757E+01_JPRB ,&
&  0.15592E+01_JPRB, 0.12890E+01_JPRB, 0.11582E+01_JPRB, 0.11119E+01_JPRB, 0.10690E+01_JPRB, 0.96347E+00_JPRB, 0.89047E+00_JPRB ,&
&  0.83149E+00_JPRB, 0.77226E+00_JPRB, 0.72688E+00_JPRB, 0.71212E+00_JPRB, 0.68417E+00_JPRB, 0.65432E+00_JPRB, 0.62144E+00_JPRB ,&
&  0.58353E+00_JPRB, 0.52743E+00_JPRB, 0.47794E+00_JPRB, 0.44042E+00_JPRB, 0.41091E+00_JPRB, 0.38644E+00_JPRB, 0.37174E+00_JPRB ,&
&  0.35483E+00_JPRB, 0.32969E+00_JPRB, 0.31804E+00_JPRB, 0.30964E+00_JPRB, 0.30534E+00_JPRB, 0.30540E+00_JPRB, 0.31186E+00_JPRB ,&
&  0.32838E+00_JPRB, 0.35611E+00_JPRB, 0.37800E+00_JPRB, 0.38332E+00_JPRB, 0.40104E+00_JPRB, 0.43133E+00_JPRB, 0.41715E+00_JPRB ,&
&  0.40342E+00_JPRB, 0.46170E+00_JPRB, 0.46864E+00_JPRB, 0.47180E+00_JPRB, 0.46715E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.46111E+00_JPRB, 0.46592E+00_JPRB, 0.46019E+00_JPRB, 0.46271E+00_JPRB, 0.46911E+00_JPRB, 0.43557E+00_JPRB, 0.40289E+00_JPRB ,&
&  0.46643E+00_JPRB, 0.40129E+00_JPRB, 0.43223E+00_JPRB, 0.35460E+00_JPRB, 0.33008E+00_JPRB, 0.32672E+00_JPRB, 0.31378E+00_JPRB ,&
&  0.30247E+00_JPRB, 0.32048E+00_JPRB, 0.34090E+00_JPRB, 0.34870E+00_JPRB, 0.36529E+00_JPRB, 0.37717E+00_JPRB, 0.38011E+00_JPRB ,&
&  0.39160E+00_JPRB, 0.40964E+00_JPRB, 0.44214E+00_JPRB, 0.44832E+00_JPRB, 0.44057E+00_JPRB, 0.45014E+00_JPRB, 0.47031E+00_JPRB ,&
&  0.45762E+00_JPRB, 0.44177E+00_JPRB, 0.45687E+00_JPRB, 0.46199E+00_JPRB, 0.46388E+00_JPRB, 0.44553E+00_JPRB, 0.44006E+00_JPRB ,&
&  0.45666E+00_JPRB, 0.44297E+00_JPRB, 0.51153E+00_JPRB, 0.54278E+00_JPRB, 0.57991E+00_JPRB, 0.62224E+00_JPRB, 0.66366E+00_JPRB ,&
&  0.73169E+00_JPRB, 0.85021E+00_JPRB, 0.99938E+00_JPRB, 0.11304E+01_JPRB, 0.13211E+01_JPRB, 0.18270E+01_JPRB, 0.21931E+01_JPRB ,&
&  0.24685E+01_JPRB, 0.17512E+01_JPRB, 0.20518E+01_JPRB, 0.20237E+01_JPRB, 0.21138E+01_JPRB, 0.19672E+01_JPRB, 0.17002E+01_JPRB ,&
&  0.13721E+01_JPRB, 0.10837E+01_JPRB, 0.91589E+00_JPRB, 0.82108E+00_JPRB, 0.78055E+00_JPRB, 0.77694E+00_JPRB, 0.76894E+00_JPRB ,&
&  0.74266E+00_JPRB, 0.67208E+00_JPRB, 0.61676E+00_JPRB, 0.57973E+00_JPRB, 0.55449E+00_JPRB, 0.52927E+00_JPRB, 0.50661E+00_JPRB ,&
&  0.48346E+00_JPRB, 0.45535E+00_JPRB, 0.42658E+00_JPRB, 0.39928E+00_JPRB, 0.38762E+00_JPRB, 0.37485E+00_JPRB, 0.36825E+00_JPRB ,&
&  0.36297E+00_JPRB, 0.34749E+00_JPRB, 0.33209E+00_JPRB, 0.32235E+00_JPRB, 0.31785E+00_JPRB, 0.32175E+00_JPRB, 0.33119E+00_JPRB ,&
&  0.33205E+00_JPRB, 0.32895E+00_JPRB, 0.33414E+00_JPRB, 0.34358E+00_JPRB, 0.35258E+00_JPRB, 0.36765E+00_JPRB, 0.41134E+00_JPRB ,&
&  0.39796E+00_JPRB, 0.41829E+00_JPRB, 0.44813E+00_JPRB, 0.45499E+00_JPRB, 0.45780E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.45804E+00_JPRB, 0.45960E+00_JPRB, 0.45397E+00_JPRB, 0.44592E+00_JPRB, 0.45163E+00_JPRB, 0.43516E+00_JPRB, 0.38153E+00_JPRB ,&
&  0.38810E+00_JPRB, 0.35052E+00_JPRB, 0.32596E+00_JPRB, 0.31733E+00_JPRB, 0.31976E+00_JPRB, 0.31695E+00_JPRB, 0.31286E+00_JPRB ,&
&  0.31436E+00_JPRB, 0.32679E+00_JPRB, 0.34420E+00_JPRB, 0.35029E+00_JPRB, 0.35934E+00_JPRB, 0.36687E+00_JPRB, 0.36782E+00_JPRB ,&
&  0.37449E+00_JPRB, 0.38363E+00_JPRB, 0.39249E+00_JPRB, 0.39861E+00_JPRB, 0.38968E+00_JPRB, 0.38294E+00_JPRB, 0.37920E+00_JPRB ,&
&  0.37664E+00_JPRB, 0.37841E+00_JPRB, 0.38433E+00_JPRB, 0.40640E+00_JPRB, 0.39762E+00_JPRB, 0.38930E+00_JPRB, 0.39995E+00_JPRB ,&
&  0.32782E+00_JPRB, 0.34289E+00_JPRB, 0.43062E+00_JPRB, 0.46145E+00_JPRB, 0.49017E+00_JPRB, 0.52405E+00_JPRB, 0.57563E+00_JPRB ,&
&  0.63724E+00_JPRB, 0.72825E+00_JPRB, 0.84013E+00_JPRB, 0.94586E+00_JPRB, 0.10905E+01_JPRB, 0.14019E+01_JPRB, 0.16884E+01_JPRB ,&
&  0.19151E+01_JPRB, 0.15082E+01_JPRB, 0.14056E+01_JPRB, 0.14664E+01_JPRB, 0.16328E+01_JPRB, 0.13677E+01_JPRB, 0.11629E+01_JPRB ,&
&  0.97249E+00_JPRB, 0.84227E+00_JPRB, 0.77548E+00_JPRB, 0.78174E+00_JPRB, 0.76297E+00_JPRB, 0.76549E+00_JPRB, 0.74603E+00_JPRB ,&
&  0.65629E+00_JPRB, 0.58220E+00_JPRB, 0.53244E+00_JPRB, 0.51340E+00_JPRB, 0.49860E+00_JPRB, 0.48353E+00_JPRB, 0.46953E+00_JPRB ,&
&  0.45503E+00_JPRB, 0.43516E+00_JPRB, 0.41299E+00_JPRB, 0.38956E+00_JPRB, 0.37576E+00_JPRB, 0.36655E+00_JPRB, 0.35873E+00_JPRB ,&
&  0.35052E+00_JPRB, 0.34210E+00_JPRB, 0.33815E+00_JPRB, 0.33959E+00_JPRB, 0.34269E+00_JPRB, 0.34192E+00_JPRB, 0.34221E+00_JPRB ,&
&  0.33336E+00_JPRB, 0.31285E+00_JPRB, 0.30163E+00_JPRB, 0.30934E+00_JPRB, 0.32544E+00_JPRB, 0.34152E+00_JPRB, 0.35760E+00_JPRB ,&
&  0.37541E+00_JPRB, 0.39330E+00_JPRB, 0.39418E+00_JPRB, 0.40491E+00_JPRB, 0.42412E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.45627E+00_JPRB, 0.45580E+00_JPRB, 0.44256E+00_JPRB, 0.44079E+00_JPRB, 0.43161E+00_JPRB, 0.42645E+00_JPRB, 0.40693E+00_JPRB ,&
&  0.40269E+00_JPRB, 0.38141E+00_JPRB, 0.35662E+00_JPRB, 0.33130E+00_JPRB, 0.32032E+00_JPRB, 0.31699E+00_JPRB, 0.31973E+00_JPRB ,&
&  0.32700E+00_JPRB, 0.33615E+00_JPRB, 0.34553E+00_JPRB, 0.34629E+00_JPRB, 0.35251E+00_JPRB, 0.35783E+00_JPRB, 0.35890E+00_JPRB ,&
&  0.35931E+00_JPRB, 0.36075E+00_JPRB, 0.35748E+00_JPRB, 0.35094E+00_JPRB, 0.34878E+00_JPRB, 0.35104E+00_JPRB, 0.35065E+00_JPRB ,&
&  0.34490E+00_JPRB, 0.34408E+00_JPRB, 0.35577E+00_JPRB, 0.38481E+00_JPRB, 0.37018E+00_JPRB, 0.37496E+00_JPRB, 0.33398E+00_JPRB ,&
&  0.29419E+00_JPRB, 0.28935E+00_JPRB, 0.31550E+00_JPRB, 0.40204E+00_JPRB, 0.42824E+00_JPRB, 0.46327E+00_JPRB, 0.49228E+00_JPRB ,&
&  0.49725E+00_JPRB, 0.53334E+00_JPRB, 0.62413E+00_JPRB, 0.80535E+00_JPRB, 0.90301E+00_JPRB, 0.11055E+01_JPRB, 0.12797E+01_JPRB ,&
&  0.13937E+01_JPRB, 0.14234E+01_JPRB, 0.14130E+01_JPRB, 0.11371E+01_JPRB, 0.10147E+01_JPRB, 0.10780E+01_JPRB, 0.92422E+00_JPRB ,&
&  0.82639E+00_JPRB, 0.73758E+00_JPRB, 0.70894E+00_JPRB, 0.70855E+00_JPRB, 0.66188E+00_JPRB, 0.65638E+00_JPRB, 0.61925E+00_JPRB ,&
&  0.56125E+00_JPRB, 0.52800E+00_JPRB, 0.50621E+00_JPRB, 0.49208E+00_JPRB, 0.48202E+00_JPRB, 0.47547E+00_JPRB, 0.47289E+00_JPRB ,&
&  0.47319E+00_JPRB, 0.46135E+00_JPRB, 0.42437E+00_JPRB, 0.38221E+00_JPRB, 0.36340E+00_JPRB, 0.35329E+00_JPRB, 0.34550E+00_JPRB ,&
&  0.33896E+00_JPRB, 0.33760E+00_JPRB, 0.34300E+00_JPRB, 0.34714E+00_JPRB, 0.34518E+00_JPRB, 0.33374E+00_JPRB, 0.32472E+00_JPRB ,&
&  0.32054E+00_JPRB, 0.31275E+00_JPRB, 0.30950E+00_JPRB, 0.32299E+00_JPRB, 0.33930E+00_JPRB, 0.34981E+00_JPRB, 0.35172E+00_JPRB ,&
&  0.35449E+00_JPRB, 0.36586E+00_JPRB, 0.37838E+00_JPRB, 0.40165E+00_JPRB, 0.42522E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.39279E+00_JPRB, 0.41462E+00_JPRB, 0.43852E+00_JPRB, 0.43921E+00_JPRB, 0.44239E+00_JPRB, 0.41974E+00_JPRB, 0.39362E+00_JPRB ,&
&  0.37526E+00_JPRB, 0.36483E+00_JPRB, 0.36102E+00_JPRB, 0.35458E+00_JPRB, 0.35033E+00_JPRB, 0.34913E+00_JPRB, 0.34939E+00_JPRB ,&
&  0.35076E+00_JPRB, 0.35196E+00_JPRB, 0.35313E+00_JPRB, 0.35393E+00_JPRB, 0.35721E+00_JPRB, 0.36219E+00_JPRB, 0.36166E+00_JPRB ,&
&  0.35678E+00_JPRB, 0.35442E+00_JPRB, 0.34651E+00_JPRB, 0.33597E+00_JPRB, 0.33246E+00_JPRB, 0.33779E+00_JPRB, 0.34780E+00_JPRB ,&
&  0.34753E+00_JPRB, 0.34200E+00_JPRB, 0.34836E+00_JPRB, 0.36191E+00_JPRB, 0.35624E+00_JPRB, 0.36440E+00_JPRB, 0.33048E+00_JPRB ,&
&  0.29204E+00_JPRB, 0.26594E+00_JPRB, 0.25624E+00_JPRB, 0.27867E+00_JPRB, 0.32872E+00_JPRB, 0.40758E+00_JPRB, 0.44411E+00_JPRB ,&
&  0.47859E+00_JPRB, 0.52201E+00_JPRB, 0.57427E+00_JPRB, 0.64518E+00_JPRB, 0.73980E+00_JPRB, 0.89827E+00_JPRB, 0.10301E+01_JPRB ,&
&  0.11188E+01_JPRB, 0.11364E+01_JPRB, 0.11037E+01_JPRB, 0.86684E+00_JPRB, 0.75029E+00_JPRB, 0.69207E+00_JPRB, 0.72276E+00_JPRB ,&
&  0.72678E+00_JPRB, 0.62616E+00_JPRB, 0.58003E+00_JPRB, 0.56218E+00_JPRB, 0.55146E+00_JPRB, 0.55392E+00_JPRB, 0.55375E+00_JPRB ,&
&  0.53976E+00_JPRB, 0.51561E+00_JPRB, 0.49159E+00_JPRB, 0.47794E+00_JPRB, 0.46919E+00_JPRB, 0.46503E+00_JPRB, 0.45940E+00_JPRB ,&
&  0.45883E+00_JPRB, 0.46485E+00_JPRB, 0.42479E+00_JPRB, 0.37199E+00_JPRB, 0.35658E+00_JPRB, 0.35149E+00_JPRB, 0.34835E+00_JPRB ,&
&  0.34269E+00_JPRB, 0.34042E+00_JPRB, 0.34417E+00_JPRB, 0.34772E+00_JPRB, 0.34745E+00_JPRB, 0.34385E+00_JPRB, 0.34306E+00_JPRB ,&
&  0.34547E+00_JPRB, 0.34782E+00_JPRB, 0.35360E+00_JPRB, 0.36895E+00_JPRB, 0.37601E+00_JPRB, 0.37584E+00_JPRB, 0.36910E+00_JPRB ,&
&  0.36479E+00_JPRB, 0.36725E+00_JPRB, 0.36999E+00_JPRB, 0.37091E+00_JPRB, 0.37771E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.39877E+00_JPRB, 0.40761E+00_JPRB, 0.42073E+00_JPRB, 0.43047E+00_JPRB, 0.44207E+00_JPRB, 0.43741E+00_JPRB, 0.43152E+00_JPRB ,&
&  0.42968E+00_JPRB, 0.42742E+00_JPRB, 0.41568E+00_JPRB, 0.40305E+00_JPRB, 0.38737E+00_JPRB, 0.37598E+00_JPRB, 0.37306E+00_JPRB ,&
&  0.37469E+00_JPRB, 0.37578E+00_JPRB, 0.37667E+00_JPRB, 0.37437E+00_JPRB, 0.37222E+00_JPRB, 0.37309E+00_JPRB, 0.36924E+00_JPRB ,&
&  0.36190E+00_JPRB, 0.35615E+00_JPRB, 0.35028E+00_JPRB, 0.34453E+00_JPRB, 0.33636E+00_JPRB, 0.33524E+00_JPRB, 0.33516E+00_JPRB ,&
&  0.33428E+00_JPRB, 0.33671E+00_JPRB, 0.34465E+00_JPRB, 0.34074E+00_JPRB, 0.34548E+00_JPRB, 0.35460E+00_JPRB, 0.31699E+00_JPRB ,&
&  0.28073E+00_JPRB, 0.25626E+00_JPRB, 0.24347E+00_JPRB, 0.24461E+00_JPRB, 0.26099E+00_JPRB, 0.29664E+00_JPRB, 0.34679E+00_JPRB ,&
&  0.40856E+00_JPRB, 0.45131E+00_JPRB, 0.49059E+00_JPRB, 0.52863E+00_JPRB, 0.57108E+00_JPRB, 0.66589E+00_JPRB, 0.74267E+00_JPRB ,&
&  0.81745E+00_JPRB, 0.89592E+00_JPRB, 0.93261E+00_JPRB, 0.91882E+00_JPRB, 0.86035E+00_JPRB, 0.72889E+00_JPRB, 0.70492E+00_JPRB ,&
&  0.78687E+00_JPRB, 0.63609E+00_JPRB, 0.53104E+00_JPRB, 0.48283E+00_JPRB, 0.47354E+00_JPRB, 0.47499E+00_JPRB, 0.47760E+00_JPRB ,&
&  0.48085E+00_JPRB, 0.47924E+00_JPRB, 0.47136E+00_JPRB, 0.46277E+00_JPRB, 0.45722E+00_JPRB, 0.45256E+00_JPRB, 0.44675E+00_JPRB ,&
&  0.43600E+00_JPRB, 0.44012E+00_JPRB, 0.43070E+00_JPRB, 0.38037E+00_JPRB, 0.36702E+00_JPRB, 0.36159E+00_JPRB, 0.35338E+00_JPRB ,&
&  0.35072E+00_JPRB, 0.35179E+00_JPRB, 0.35679E+00_JPRB, 0.35931E+00_JPRB, 0.36170E+00_JPRB, 0.36593E+00_JPRB, 0.37352E+00_JPRB ,&
&  0.38498E+00_JPRB, 0.39409E+00_JPRB, 0.39805E+00_JPRB, 0.39983E+00_JPRB, 0.39494E+00_JPRB, 0.38537E+00_JPRB, 0.37998E+00_JPRB ,&
&  0.38132E+00_JPRB, 0.38488E+00_JPRB, 0.38919E+00_JPRB, 0.39279E+00_JPRB, 0.39487E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.40456E+00_JPRB, 0.40853E+00_JPRB, 0.41625E+00_JPRB, 0.42282E+00_JPRB, 0.43184E+00_JPRB, 0.43870E+00_JPRB, 0.44188E+00_JPRB ,&
&  0.44964E+00_JPRB, 0.44605E+00_JPRB, 0.43325E+00_JPRB, 0.41721E+00_JPRB, 0.40153E+00_JPRB, 0.39202E+00_JPRB, 0.38827E+00_JPRB ,&
&  0.39054E+00_JPRB, 0.38928E+00_JPRB, 0.38845E+00_JPRB, 0.38942E+00_JPRB, 0.39032E+00_JPRB, 0.39135E+00_JPRB, 0.38634E+00_JPRB ,&
&  0.37223E+00_JPRB, 0.36251E+00_JPRB, 0.35224E+00_JPRB, 0.34021E+00_JPRB, 0.32884E+00_JPRB, 0.32235E+00_JPRB, 0.32702E+00_JPRB ,&
&  0.33442E+00_JPRB, 0.33088E+00_JPRB, 0.33044E+00_JPRB, 0.33255E+00_JPRB, 0.33699E+00_JPRB, 0.33597E+00_JPRB, 0.27886E+00_JPRB ,&
&  0.25779E+00_JPRB, 0.24435E+00_JPRB, 0.23964E+00_JPRB, 0.24043E+00_JPRB, 0.25294E+00_JPRB, 0.27606E+00_JPRB, 0.31775E+00_JPRB ,&
&  0.36007E+00_JPRB, 0.40180E+00_JPRB, 0.43049E+00_JPRB, 0.46325E+00_JPRB, 0.51272E+00_JPRB, 0.57379E+00_JPRB, 0.62567E+00_JPRB ,&
&  0.67245E+00_JPRB, 0.73428E+00_JPRB, 0.77022E+00_JPRB, 0.78251E+00_JPRB, 0.75538E+00_JPRB, 0.70261E+00_JPRB, 0.65374E+00_JPRB ,&
&  0.61680E+00_JPRB, 0.54769E+00_JPRB, 0.51717E+00_JPRB, 0.49499E+00_JPRB, 0.48128E+00_JPRB, 0.47128E+00_JPRB, 0.45962E+00_JPRB ,&
&  0.45359E+00_JPRB, 0.45248E+00_JPRB, 0.44666E+00_JPRB, 0.44470E+00_JPRB, 0.44041E+00_JPRB, 0.43323E+00_JPRB, 0.42224E+00_JPRB ,&
&  0.41207E+00_JPRB, 0.39939E+00_JPRB, 0.38464E+00_JPRB, 0.38150E+00_JPRB, 0.37526E+00_JPRB, 0.36844E+00_JPRB, 0.36614E+00_JPRB ,&
&  0.36726E+00_JPRB, 0.37212E+00_JPRB, 0.37886E+00_JPRB, 0.38090E+00_JPRB, 0.38424E+00_JPRB, 0.38813E+00_JPRB, 0.39815E+00_JPRB ,&
&  0.40259E+00_JPRB, 0.40672E+00_JPRB, 0.40827E+00_JPRB, 0.40689E+00_JPRB, 0.40104E+00_JPRB, 0.39285E+00_JPRB, 0.38738E+00_JPRB ,&
&  0.38679E+00_JPRB, 0.38817E+00_JPRB, 0.39003E+00_JPRB, 0.39419E+00_JPRB, 0.39987E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.40193E+00_JPRB, 0.40452E+00_JPRB, 0.41023E+00_JPRB, 0.41675E+00_JPRB, 0.42352E+00_JPRB, 0.43024E+00_JPRB, 0.43369E+00_JPRB ,&
&  0.43573E+00_JPRB, 0.43341E+00_JPRB, 0.42964E+00_JPRB, 0.41872E+00_JPRB, 0.41131E+00_JPRB, 0.40813E+00_JPRB, 0.40441E+00_JPRB ,&
&  0.40058E+00_JPRB, 0.39967E+00_JPRB, 0.39965E+00_JPRB, 0.40106E+00_JPRB, 0.39956E+00_JPRB, 0.39994E+00_JPRB, 0.39508E+00_JPRB ,&
&  0.38301E+00_JPRB, 0.37333E+00_JPRB, 0.35650E+00_JPRB, 0.34184E+00_JPRB, 0.33187E+00_JPRB, 0.32516E+00_JPRB, 0.32726E+00_JPRB ,&
&  0.32663E+00_JPRB, 0.32706E+00_JPRB, 0.32822E+00_JPRB, 0.33013E+00_JPRB, 0.33537E+00_JPRB, 0.27671E+00_JPRB, 0.26062E+00_JPRB ,&
&  0.24929E+00_JPRB, 0.24212E+00_JPRB, 0.24086E+00_JPRB, 0.24181E+00_JPRB, 0.24801E+00_JPRB, 0.26521E+00_JPRB, 0.29175E+00_JPRB ,&
&  0.31492E+00_JPRB, 0.38135E+00_JPRB, 0.39749E+00_JPRB, 0.41848E+00_JPRB, 0.45416E+00_JPRB, 0.49373E+00_JPRB, 0.53007E+00_JPRB ,&
&  0.55906E+00_JPRB, 0.59514E+00_JPRB, 0.62508E+00_JPRB, 0.64467E+00_JPRB, 0.63522E+00_JPRB, 0.62044E+00_JPRB, 0.59573E+00_JPRB ,&
&  0.57473E+00_JPRB, 0.54654E+00_JPRB, 0.52863E+00_JPRB, 0.51153E+00_JPRB, 0.49821E+00_JPRB, 0.48201E+00_JPRB, 0.47067E+00_JPRB ,&
&  0.46225E+00_JPRB, 0.45264E+00_JPRB, 0.44560E+00_JPRB, 0.43594E+00_JPRB, 0.42490E+00_JPRB, 0.41620E+00_JPRB, 0.40712E+00_JPRB ,&
&  0.39746E+00_JPRB, 0.38812E+00_JPRB, 0.38048E+00_JPRB, 0.37736E+00_JPRB, 0.37391E+00_JPRB, 0.37166E+00_JPRB, 0.37112E+00_JPRB ,&
&  0.37279E+00_JPRB, 0.37924E+00_JPRB, 0.38472E+00_JPRB, 0.39068E+00_JPRB, 0.39538E+00_JPRB, 0.39623E+00_JPRB, 0.39770E+00_JPRB ,&
&  0.39956E+00_JPRB, 0.40295E+00_JPRB, 0.40348E+00_JPRB, 0.40197E+00_JPRB, 0.39699E+00_JPRB, 0.39042E+00_JPRB, 0.38622E+00_JPRB ,&
&  0.38556E+00_JPRB, 0.38752E+00_JPRB, 0.38740E+00_JPRB, 0.39089E+00_JPRB, 0.39635E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.39707E+00_JPRB, 0.39714E+00_JPRB, 0.39798E+00_JPRB, 0.40239E+00_JPRB, 0.40994E+00_JPRB, 0.41840E+00_JPRB, 0.41815E+00_JPRB ,&
&  0.41822E+00_JPRB, 0.41584E+00_JPRB, 0.41685E+00_JPRB, 0.41586E+00_JPRB, 0.41402E+00_JPRB, 0.41069E+00_JPRB, 0.40774E+00_JPRB ,&
&  0.40515E+00_JPRB, 0.39990E+00_JPRB, 0.39343E+00_JPRB, 0.39249E+00_JPRB, 0.39444E+00_JPRB, 0.39609E+00_JPRB, 0.39632E+00_JPRB ,&
&  0.39420E+00_JPRB, 0.38716E+00_JPRB, 0.36636E+00_JPRB, 0.35070E+00_JPRB, 0.33623E+00_JPRB, 0.33301E+00_JPRB, 0.32633E+00_JPRB ,&
&  0.33635E+00_JPRB, 0.30702E+00_JPRB, 0.30501E+00_JPRB, 0.29081E+00_JPRB, 0.27813E+00_JPRB, 0.26473E+00_JPRB, 0.25503E+00_JPRB ,&
&  0.25178E+00_JPRB, 0.24874E+00_JPRB, 0.24922E+00_JPRB, 0.25453E+00_JPRB, 0.26227E+00_JPRB, 0.27648E+00_JPRB, 0.29442E+00_JPRB ,&
&  0.31174E+00_JPRB, 0.35881E+00_JPRB, 0.37439E+00_JPRB, 0.38599E+00_JPRB, 0.40587E+00_JPRB, 0.42572E+00_JPRB, 0.44493E+00_JPRB ,&
&  0.46142E+00_JPRB, 0.47816E+00_JPRB, 0.49372E+00_JPRB, 0.49797E+00_JPRB, 0.50340E+00_JPRB, 0.49432E+00_JPRB, 0.49054E+00_JPRB ,&
&  0.48871E+00_JPRB, 0.48451E+00_JPRB, 0.47793E+00_JPRB, 0.46832E+00_JPRB, 0.45891E+00_JPRB, 0.44792E+00_JPRB, 0.44028E+00_JPRB ,&
&  0.43062E+00_JPRB, 0.42179E+00_JPRB, 0.41167E+00_JPRB, 0.40354E+00_JPRB, 0.39498E+00_JPRB, 0.38902E+00_JPRB, 0.38306E+00_JPRB ,&
&  0.37832E+00_JPRB, 0.37528E+00_JPRB, 0.37173E+00_JPRB, 0.36928E+00_JPRB, 0.36743E+00_JPRB, 0.36575E+00_JPRB, 0.36566E+00_JPRB ,&
&  0.37020E+00_JPRB, 0.37472E+00_JPRB, 0.38138E+00_JPRB, 0.38746E+00_JPRB, 0.39208E+00_JPRB, 0.39329E+00_JPRB, 0.39244E+00_JPRB ,&
&  0.39331E+00_JPRB, 0.39494E+00_JPRB, 0.39513E+00_JPRB, 0.39301E+00_JPRB, 0.38904E+00_JPRB, 0.38452E+00_JPRB, 0.38283E+00_JPRB ,&
&  0.38243E+00_JPRB, 0.38406E+00_JPRB, 0.38402E+00_JPRB, 0.38818E+00_JPRB, 0.39280E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.38493E+00_JPRB, 0.38571E+00_JPRB, 0.38599E+00_JPRB, 0.38832E+00_JPRB, 0.39125E+00_JPRB, 0.39590E+00_JPRB, 0.39719E+00_JPRB ,&
&  0.40141E+00_JPRB, 0.40504E+00_JPRB, 0.40872E+00_JPRB, 0.40988E+00_JPRB, 0.41038E+00_JPRB, 0.40917E+00_JPRB, 0.40676E+00_JPRB ,&
&  0.40422E+00_JPRB, 0.40073E+00_JPRB, 0.39781E+00_JPRB, 0.39465E+00_JPRB, 0.39170E+00_JPRB, 0.39046E+00_JPRB, 0.39025E+00_JPRB ,&
&  0.39097E+00_JPRB, 0.39114E+00_JPRB, 0.36417E+00_JPRB, 0.35394E+00_JPRB, 0.33965E+00_JPRB, 0.33276E+00_JPRB, 0.32970E+00_JPRB ,&
&  0.32679E+00_JPRB, 0.32375E+00_JPRB, 0.32042E+00_JPRB, 0.31302E+00_JPRB, 0.30217E+00_JPRB, 0.28674E+00_JPRB, 0.27856E+00_JPRB ,&
&  0.27124E+00_JPRB, 0.26922E+00_JPRB, 0.27059E+00_JPRB, 0.27559E+00_JPRB, 0.28503E+00_JPRB, 0.29569E+00_JPRB, 0.30884E+00_JPRB ,&
&  0.32673E+00_JPRB, 0.34703E+00_JPRB, 0.35977E+00_JPRB, 0.36294E+00_JPRB, 0.37655E+00_JPRB, 0.38510E+00_JPRB, 0.39213E+00_JPRB ,&
&  0.39766E+00_JPRB, 0.40233E+00_JPRB, 0.41094E+00_JPRB, 0.41336E+00_JPRB, 0.41068E+00_JPRB, 0.40882E+00_JPRB, 0.41142E+00_JPRB ,&
&  0.41264E+00_JPRB, 0.41235E+00_JPRB, 0.40899E+00_JPRB, 0.40575E+00_JPRB, 0.40191E+00_JPRB, 0.39705E+00_JPRB, 0.39267E+00_JPRB ,&
&  0.38821E+00_JPRB, 0.38417E+00_JPRB, 0.37853E+00_JPRB, 0.37269E+00_JPRB, 0.36886E+00_JPRB, 0.36549E+00_JPRB, 0.36259E+00_JPRB ,&
&  0.36037E+00_JPRB, 0.35895E+00_JPRB, 0.35732E+00_JPRB, 0.35325E+00_JPRB, 0.35224E+00_JPRB, 0.35528E+00_JPRB, 0.35634E+00_JPRB ,&
&  0.36066E+00_JPRB, 0.36470E+00_JPRB, 0.36972E+00_JPRB, 0.37364E+00_JPRB, 0.37716E+00_JPRB, 0.37922E+00_JPRB, 0.37949E+00_JPRB ,&
&  0.38010E+00_JPRB, 0.38069E+00_JPRB, 0.38068E+00_JPRB, 0.37859E+00_JPRB, 0.37633E+00_JPRB, 0.37391E+00_JPRB, 0.37367E+00_JPRB ,&
&  0.37338E+00_JPRB, 0.37423E+00_JPRB, 0.37436E+00_JPRB, 0.37803E+00_JPRB, 0.38136E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.37638E+00_JPRB, 0.37822E+00_JPRB, 0.37989E+00_JPRB, 0.38183E+00_JPRB, 0.38444E+00_JPRB, 0.38772E+00_JPRB, 0.38999E+00_JPRB ,&
&  0.39290E+00_JPRB, 0.39591E+00_JPRB, 0.39944E+00_JPRB, 0.40137E+00_JPRB, 0.40263E+00_JPRB, 0.40284E+00_JPRB, 0.40240E+00_JPRB ,&
&  0.40109E+00_JPRB, 0.39978E+00_JPRB, 0.39839E+00_JPRB, 0.39741E+00_JPRB, 0.39646E+00_JPRB, 0.39543E+00_JPRB, 0.39361E+00_JPRB ,&
&  0.39186E+00_JPRB, 0.39162E+00_JPRB, 0.38866E+00_JPRB, 0.38439E+00_JPRB, 0.38075E+00_JPRB, 0.35139E+00_JPRB, 0.34134E+00_JPRB ,&
&  0.33910E+00_JPRB, 0.33618E+00_JPRB, 0.33689E+00_JPRB, 0.33803E+00_JPRB, 0.34497E+00_JPRB, 0.34350E+00_JPRB, 0.34308E+00_JPRB ,&
&  0.32729E+00_JPRB, 0.32628E+00_JPRB, 0.32289E+00_JPRB, 0.32149E+00_JPRB, 0.32387E+00_JPRB, 0.32639E+00_JPRB, 0.33390E+00_JPRB ,&
&  0.35135E+00_JPRB, 0.35717E+00_JPRB, 0.36011E+00_JPRB, 0.36340E+00_JPRB, 0.36735E+00_JPRB, 0.37051E+00_JPRB, 0.37314E+00_JPRB ,&
&  0.37521E+00_JPRB, 0.37681E+00_JPRB, 0.37736E+00_JPRB, 0.37463E+00_JPRB, 0.37141E+00_JPRB, 0.37032E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.36881E+00_JPRB, 0.36883E+00_JPRB, 0.36904E+00_JPRB, 0.36906E+00_JPRB, 0.36901E+00_JPRB, 0.36837E+00_JPRB, 0.36753E+00_JPRB ,&
&  0.36644E+00_JPRB, 0.36497E+00_JPRB, 0.36353E+00_JPRB, 0.36161E+00_JPRB, 0.36055E+00_JPRB, 0.35849E+00_JPRB, 0.35720E+00_JPRB ,&
&  0.35577E+00_JPRB, 0.35501E+00_JPRB, 0.35382E+00_JPRB, 0.35374E+00_JPRB, 0.35338E+00_JPRB, 0.35461E+00_JPRB, 0.35592E+00_JPRB ,&
&  0.35853E+00_JPRB, 0.36056E+00_JPRB, 0.36252E+00_JPRB, 0.36382E+00_JPRB, 0.36530E+00_JPRB, 0.36643E+00_JPRB, 0.36670E+00_JPRB ,&
&  0.36679E+00_JPRB, 0.36641E+00_JPRB, 0.36585E+00_JPRB, 0.36490E+00_JPRB, 0.36422E+00_JPRB, 0.36380E+00_JPRB, 0.36416E+00_JPRB ,&
&  0.36436E+00_JPRB, 0.36588E+00_JPRB, 0.36833E+00_JPRB, 0.37132E+00_JPRB, 0.37397E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.38179E+00_JPRB, 0.38346E+00_JPRB, 0.38443E+00_JPRB, 0.38512E+00_JPRB, 0.38572E+00_JPRB, 0.38715E+00_JPRB, 0.38840E+00_JPRB ,&
&  0.38971E+00_JPRB, 0.39119E+00_JPRB, 0.39323E+00_JPRB, 0.39450E+00_JPRB, 0.39547E+00_JPRB, 0.39599E+00_JPRB, 0.39661E+00_JPRB ,&
&  0.39684E+00_JPRB, 0.39681E+00_JPRB, 0.39630E+00_JPRB, 0.39609E+00_JPRB, 0.39597E+00_JPRB, 0.39605E+00_JPRB, 0.39593E+00_JPRB ,&
&  0.39516E+00_JPRB, 0.39413E+00_JPRB, 0.39293E+00_JPRB, 0.39104E+00_JPRB, 0.38857E+00_JPRB, 0.38590E+00_JPRB, 0.38282E+00_JPRB ,&
&  0.37980E+00_JPRB, 0.37746E+00_JPRB, 0.37515E+00_JPRB, 0.37301E+00_JPRB, 0.37223E+00_JPRB, 0.37122E+00_JPRB, 0.37018E+00_JPRB ,&
&  0.36915E+00_JPRB, 0.36787E+00_JPRB, 0.36775E+00_JPRB, 0.36805E+00_JPRB, 0.36873E+00_JPRB, 0.37009E+00_JPRB, 0.37133E+00_JPRB ,&
&  0.37227E+00_JPRB, 0.37241E+00_JPRB, 0.37207E+00_JPRB, 0.37209E+00_JPRB, 0.37234E+00_JPRB, 0.37235E+00_JPRB, 0.37185E+00_JPRB ,&
&  0.37154E+00_JPRB, 0.37066E+00_JPRB, 0.36922E+00_JPRB, 0.36754E+00_JPRB, 0.36631E+00_JPRB, 0.36512E+00_JPRB, 0.36390E+00_JPRB ,&
&  0.36314E+00_JPRB, 0.36275E+00_JPRB, 0.36249E+00_JPRB, 0.36226E+00_JPRB, 0.36243E+00_JPRB, 0.36259E+00_JPRB, 0.36270E+00_JPRB ,&
&  0.36279E+00_JPRB, 0.36341E+00_JPRB, 0.36406E+00_JPRB, 0.36434E+00_JPRB, 0.36464E+00_JPRB, 0.36464E+00_JPRB, 0.36475E+00_JPRB ,&
&  0.36463E+00_JPRB, 0.36431E+00_JPRB, 0.36335E+00_JPRB, 0.36300E+00_JPRB, 0.36280E+00_JPRB, 0.36316E+00_JPRB, 0.36330E+00_JPRB ,&
&  0.36416E+00_JPRB, 0.36471E+00_JPRB, 0.36512E+00_JPRB, 0.36581E+00_JPRB, 0.36653E+00_JPRB, 0.36712E+00_JPRB, 0.36741E+00_JPRB ,&
&  0.36764E+00_JPRB, 0.36810E+00_JPRB, 0.36857E+00_JPRB, 0.36908E+00_JPRB, 0.37004E+00_JPRB, 0.37105E+00_JPRB, 0.37215E+00_JPRB ,&
&  0.37341E+00_JPRB, 0.37529E+00_JPRB, 0.37727E+00_JPRB, 0.37885E+00_JPRB, 0.38021E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.38607E+00_JPRB, 0.38628E+00_JPRB, 0.38633E+00_JPRB, 0.38631E+00_JPRB, 0.38645E+00_JPRB, 0.38678E+00_JPRB, 0.38703E+00_JPRB ,&
&  0.38733E+00_JPRB, 0.38766E+00_JPRB, 0.38828E+00_JPRB, 0.38866E+00_JPRB, 0.38908E+00_JPRB, 0.38935E+00_JPRB, 0.38972E+00_JPRB ,&
&  0.39002E+00_JPRB, 0.39031E+00_JPRB, 0.39053E+00_JPRB, 0.39084E+00_JPRB, 0.39118E+00_JPRB, 0.39167E+00_JPRB, 0.39207E+00_JPRB ,&
&  0.39215E+00_JPRB, 0.39194E+00_JPRB, 0.39181E+00_JPRB, 0.39158E+00_JPRB, 0.39128E+00_JPRB, 0.39055E+00_JPRB, 0.38961E+00_JPRB ,&
&  0.38845E+00_JPRB, 0.38730E+00_JPRB, 0.38640E+00_JPRB, 0.38564E+00_JPRB, 0.38515E+00_JPRB, 0.38470E+00_JPRB, 0.38440E+00_JPRB ,&
&  0.38398E+00_JPRB, 0.38377E+00_JPRB, 0.38357E+00_JPRB, 0.38345E+00_JPRB, 0.38309E+00_JPRB, 0.38269E+00_JPRB, 0.38214E+00_JPRB ,&
&  0.38156E+00_JPRB, 0.38082E+00_JPRB, 0.38017E+00_JPRB, 0.37948E+00_JPRB, 0.37868E+00_JPRB, 0.37754E+00_JPRB, 0.37628E+00_JPRB ,&
&  0.37496E+00_JPRB, 0.37359E+00_JPRB, 0.37219E+00_JPRB, 0.37094E+00_JPRB, 0.37001E+00_JPRB, 0.36917E+00_JPRB, 0.36827E+00_JPRB ,&
&  0.36783E+00_JPRB, 0.36784E+00_JPRB, 0.36790E+00_JPRB, 0.36781E+00_JPRB, 0.36808E+00_JPRB, 0.36827E+00_JPRB, 0.36853E+00_JPRB ,&
&  0.36872E+00_JPRB, 0.36919E+00_JPRB, 0.36984E+00_JPRB, 0.37047E+00_JPRB, 0.37115E+00_JPRB, 0.37186E+00_JPRB, 0.37273E+00_JPRB ,&
&  0.37359E+00_JPRB, 0.37446E+00_JPRB, 0.37542E+00_JPRB, 0.37639E+00_JPRB, 0.37690E+00_JPRB, 0.37725E+00_JPRB, 0.37730E+00_JPRB ,&
&  0.37752E+00_JPRB, 0.37766E+00_JPRB, 0.37795E+00_JPRB, 0.37831E+00_JPRB, 0.37883E+00_JPRB, 0.37931E+00_JPRB, 0.37983E+00_JPRB ,&
&  0.38046E+00_JPRB, 0.38119E+00_JPRB, 0.38178E+00_JPRB, 0.38234E+00_JPRB, 0.38289E+00_JPRB, 0.38354E+00_JPRB, 0.38407E+00_JPRB ,&
&  0.38450E+00_JPRB, 0.38504E+00_JPRB, 0.38553E+00_JPRB, 0.38573E+00_JPRB, 0.38589E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB ,&
&  0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB, 0.38265E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB, 0.18369E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.18704E+00_JPRB, 0.18734E+00_JPRB, 0.18748E+00_JPRB, 0.18754E+00_JPRB, 0.18633E+00_JPRB, 0.18595E+00_JPRB, 0.18555E+00_JPRB ,&
&  0.18525E+00_JPRB, 0.18509E+00_JPRB, 0.18544E+00_JPRB, 0.18590E+00_JPRB, 0.18635E+00_JPRB, 0.18678E+00_JPRB, 0.18711E+00_JPRB ,&
&  0.18723E+00_JPRB, 0.18736E+00_JPRB, 0.18647E+00_JPRB, 0.18606E+00_JPRB, 0.18596E+00_JPRB, 0.18605E+00_JPRB, 0.18605E+00_JPRB ,&
&  0.18619E+00_JPRB, 0.18629E+00_JPRB, 0.18630E+00_JPRB, 0.18659E+00_JPRB, 0.18709E+00_JPRB, 0.18752E+00_JPRB, 0.18797E+00_JPRB ,&
&  0.18840E+00_JPRB, 0.18900E+00_JPRB, 0.18954E+00_JPRB, 0.19005E+00_JPRB, 0.19063E+00_JPRB, 0.19116E+00_JPRB, 0.19142E+00_JPRB ,&
&  0.19158E+00_JPRB, 0.19154E+00_JPRB, 0.19156E+00_JPRB, 0.19157E+00_JPRB, 0.19169E+00_JPRB, 0.19186E+00_JPRB, 0.19186E+00_JPRB ,&
&  0.19173E+00_JPRB, 0.19155E+00_JPRB, 0.19129E+00_JPRB, 0.19080E+00_JPRB, 0.19028E+00_JPRB, 0.18978E+00_JPRB, 0.18930E+00_JPRB ,&
&  0.18888E+00_JPRB, 0.18842E+00_JPRB, 0.18790E+00_JPRB, 0.18736E+00_JPRB, 0.18679E+00_JPRB, 0.18623E+00_JPRB, 0.18567E+00_JPRB ,&
&  0.18511E+00_JPRB, 0.18474E+00_JPRB, 0.18433E+00_JPRB, 0.18398E+00_JPRB, 0.18372E+00_JPRB, 0.18347E+00_JPRB, 0.18322E+00_JPRB ,&
&  0.18297E+00_JPRB, 0.18267E+00_JPRB, 0.18240E+00_JPRB, 0.18208E+00_JPRB, 0.18181E+00_JPRB, 0.18150E+00_JPRB, 0.18126E+00_JPRB ,&
&  0.18103E+00_JPRB, 0.18083E+00_JPRB, 0.18066E+00_JPRB, 0.18059E+00_JPRB, 0.18055E+00_JPRB, 0.18060E+00_JPRB, 0.18064E+00_JPRB ,&
&  0.18072E+00_JPRB, 0.18077E+00_JPRB, 0.18082E+00_JPRB, 0.18074E+00_JPRB, 0.18092E+00_JPRB, 0.18112E+00_JPRB, 0.18138E+00_JPRB ,&
&  0.18164E+00_JPRB, 0.18203E+00_JPRB, 0.18241E+00_JPRB, 0.18284E+00_JPRB, 0.18348E+00_JPRB, 0.18371E+00_JPRB, 0.18392E+00_JPRB ,&
&  0.18416E+00_JPRB, 0.18464E+00_JPRB, 0.18507E+00_JPRB, 0.18531E+00_JPRB, 0.18582E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.20000E+00_JPRB, 0.20500E+00_JPRB, 0.20902E+00_JPRB, 0.21307E+00_JPRB, 0.21297E+00_JPRB, 0.23212E+00_JPRB, 0.23002E+00_JPRB ,&
&  0.22945E+00_JPRB, 0.22202E+00_JPRB, 0.22252E+00_JPRB, 0.22097E+00_JPRB, 0.21851E+00_JPRB, 0.21544E+00_JPRB, 0.21217E+00_JPRB ,&
&  0.20883E+00_JPRB, 0.20493E+00_JPRB, 0.20128E+00_JPRB, 0.19780E+00_JPRB, 0.19494E+00_JPRB, 0.19246E+00_JPRB, 0.19106E+00_JPRB ,&
&  0.19102E+00_JPRB, 0.19235E+00_JPRB, 0.19467E+00_JPRB, 0.19615E+00_JPRB, 0.19771E+00_JPRB, 0.19932E+00_JPRB, 0.20101E+00_JPRB ,&
&  0.20254E+00_JPRB, 0.20445E+00_JPRB, 0.20549E+00_JPRB, 0.20522E+00_JPRB, 0.20398E+00_JPRB, 0.20321E+00_JPRB, 0.20277E+00_JPRB ,&
&  0.20290E+00_JPRB, 0.20282E+00_JPRB, 0.20265E+00_JPRB, 0.20266E+00_JPRB, 0.20264E+00_JPRB, 0.20198E+00_JPRB, 0.20078E+00_JPRB ,&
&  0.19978E+00_JPRB, 0.19897E+00_JPRB, 0.19754E+00_JPRB, 0.19616E+00_JPRB, 0.19487E+00_JPRB, 0.19354E+00_JPRB, 0.19217E+00_JPRB ,&
&  0.19117E+00_JPRB, 0.19016E+00_JPRB, 0.18891E+00_JPRB, 0.18745E+00_JPRB, 0.18605E+00_JPRB, 0.18480E+00_JPRB, 0.18380E+00_JPRB ,&
&  0.18352E+00_JPRB, 0.18296E+00_JPRB, 0.18248E+00_JPRB, 0.18201E+00_JPRB, 0.18186E+00_JPRB, 0.18150E+00_JPRB, 0.18126E+00_JPRB ,&
&  0.18109E+00_JPRB, 0.18088E+00_JPRB, 0.18066E+00_JPRB, 0.18045E+00_JPRB, 0.18031E+00_JPRB, 0.18010E+00_JPRB, 0.17991E+00_JPRB ,&
&  0.17980E+00_JPRB, 0.17973E+00_JPRB, 0.17962E+00_JPRB, 0.17934E+00_JPRB, 0.17905E+00_JPRB, 0.17869E+00_JPRB, 0.17872E+00_JPRB ,&
&  0.17889E+00_JPRB, 0.17918E+00_JPRB, 0.17966E+00_JPRB, 0.17991E+00_JPRB, 0.18069E+00_JPRB, 0.18143E+00_JPRB, 0.18230E+00_JPRB ,&
&  0.18323E+00_JPRB, 0.18462E+00_JPRB, 0.18612E+00_JPRB, 0.18773E+00_JPRB, 0.19019E+00_JPRB, 0.19125E+00_JPRB, 0.19219E+00_JPRB ,&
&  0.19329E+00_JPRB, 0.19640E+00_JPRB, 0.19770E+00_JPRB, 0.19803E+00_JPRB, 0.19910E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.24223E+00_JPRB, 0.24494E+00_JPRB, 0.24684E+00_JPRB, 0.24809E+00_JPRB, 0.24926E+00_JPRB, 0.24859E+00_JPRB, 0.24718E+00_JPRB ,&
&  0.24325E+00_JPRB, 0.24183E+00_JPRB, 0.23916E+00_JPRB, 0.23644E+00_JPRB, 0.23323E+00_JPRB, 0.23001E+00_JPRB, 0.22603E+00_JPRB ,&
&  0.22167E+00_JPRB, 0.21697E+00_JPRB, 0.21126E+00_JPRB, 0.20520E+00_JPRB, 0.19966E+00_JPRB, 0.19451E+00_JPRB, 0.19117E+00_JPRB ,&
&  0.18958E+00_JPRB, 0.19013E+00_JPRB, 0.19262E+00_JPRB, 0.19593E+00_JPRB, 0.19997E+00_JPRB, 0.20456E+00_JPRB, 0.21029E+00_JPRB ,&
&  0.21530E+00_JPRB, 0.22137E+00_JPRB, 0.22508E+00_JPRB, 0.22675E+00_JPRB, 0.22536E+00_JPRB, 0.22514E+00_JPRB, 0.22409E+00_JPRB ,&
&  0.22268E+00_JPRB, 0.22033E+00_JPRB, 0.21910E+00_JPRB, 0.21873E+00_JPRB, 0.21747E+00_JPRB, 0.21522E+00_JPRB, 0.21241E+00_JPRB ,&
&  0.20993E+00_JPRB, 0.20780E+00_JPRB, 0.20535E+00_JPRB, 0.20326E+00_JPRB, 0.20166E+00_JPRB, 0.19980E+00_JPRB, 0.19743E+00_JPRB ,&
&  0.19524E+00_JPRB, 0.19303E+00_JPRB, 0.19040E+00_JPRB, 0.18810E+00_JPRB, 0.18582E+00_JPRB, 0.18415E+00_JPRB, 0.18266E+00_JPRB ,&
&  0.18197E+00_JPRB, 0.18126E+00_JPRB, 0.18102E+00_JPRB, 0.18087E+00_JPRB, 0.18121E+00_JPRB, 0.18099E+00_JPRB, 0.18066E+00_JPRB ,&
&  0.18024E+00_JPRB, 0.18014E+00_JPRB, 0.17970E+00_JPRB, 0.17955E+00_JPRB, 0.17940E+00_JPRB, 0.17889E+00_JPRB, 0.17868E+00_JPRB ,&
&  0.17886E+00_JPRB, 0.17890E+00_JPRB, 0.17856E+00_JPRB, 0.17862E+00_JPRB, 0.17843E+00_JPRB, 0.17810E+00_JPRB, 0.17839E+00_JPRB ,&
&  0.17828E+00_JPRB, 0.17851E+00_JPRB, 0.17884E+00_JPRB, 0.17936E+00_JPRB, 0.18040E+00_JPRB, 0.18173E+00_JPRB, 0.18352E+00_JPRB ,&
&  0.18618E+00_JPRB, 0.18862E+00_JPRB, 0.19110E+00_JPRB, 0.19404E+00_JPRB, 0.19924E+00_JPRB, 0.20252E+00_JPRB, 0.20802E+00_JPRB ,&
&  0.21350E+00_JPRB, 0.21464E+00_JPRB, 0.23693E+00_JPRB, 0.23788E+00_JPRB, 0.23929E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.25493E+00_JPRB, 0.25684E+00_JPRB, 0.25809E+00_JPRB, 0.26012E+00_JPRB, 0.26244E+00_JPRB, 0.26333E+00_JPRB, 0.26439E+00_JPRB ,&
&  0.26166E+00_JPRB, 0.25465E+00_JPRB, 0.24962E+00_JPRB, 0.24330E+00_JPRB, 0.23684E+00_JPRB, 0.23037E+00_JPRB, 0.22434E+00_JPRB ,&
&  0.21834E+00_JPRB, 0.21297E+00_JPRB, 0.20762E+00_JPRB, 0.20314E+00_JPRB, 0.20015E+00_JPRB, 0.19770E+00_JPRB, 0.19489E+00_JPRB ,&
&  0.19244E+00_JPRB, 0.19317E+00_JPRB, 0.19541E+00_JPRB, 0.19994E+00_JPRB, 0.20717E+00_JPRB, 0.21415E+00_JPRB, 0.22173E+00_JPRB ,&
&  0.23554E+00_JPRB, 0.23996E+00_JPRB, 0.23970E+00_JPRB, 0.24137E+00_JPRB, 0.24061E+00_JPRB, 0.24104E+00_JPRB, 0.23075E+00_JPRB ,&
&  0.23183E+00_JPRB, 0.23381E+00_JPRB, 0.24063E+00_JPRB, 0.24407E+00_JPRB, 0.23946E+00_JPRB, 0.23480E+00_JPRB, 0.22683E+00_JPRB ,&
&  0.21978E+00_JPRB, 0.21442E+00_JPRB, 0.21087E+00_JPRB, 0.20867E+00_JPRB, 0.20482E+00_JPRB, 0.20195E+00_JPRB, 0.19719E+00_JPRB ,&
&  0.19308E+00_JPRB, 0.19019E+00_JPRB, 0.18662E+00_JPRB, 0.18433E+00_JPRB, 0.18250E+00_JPRB, 0.18079E+00_JPRB, 0.17950E+00_JPRB ,&
&  0.17878E+00_JPRB, 0.17821E+00_JPRB, 0.17878E+00_JPRB, 0.17948E+00_JPRB, 0.18081E+00_JPRB, 0.18188E+00_JPRB, 0.18219E+00_JPRB ,&
&  0.18247E+00_JPRB, 0.18281E+00_JPRB, 0.18286E+00_JPRB, 0.18329E+00_JPRB, 0.18279E+00_JPRB, 0.18205E+00_JPRB, 0.18113E+00_JPRB ,&
&  0.18021E+00_JPRB, 0.17992E+00_JPRB, 0.17939E+00_JPRB, 0.17909E+00_JPRB, 0.17780E+00_JPRB, 0.17689E+00_JPRB, 0.17640E+00_JPRB ,&
&  0.17594E+00_JPRB, 0.17630E+00_JPRB, 0.17645E+00_JPRB, 0.17720E+00_JPRB, 0.17839E+00_JPRB, 0.17955E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18397E+00_JPRB, 0.18683E+00_JPRB, 0.19148E+00_JPRB, 0.19662E+00_JPRB, 0.20228E+00_JPRB, 0.21007E+00_JPRB, 0.21806E+00_JPRB ,&
&  0.22900E+00_JPRB, 0.24556E+00_JPRB, 0.24777E+00_JPRB, 0.24915E+00_JPRB, 0.25281E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.26430E+00_JPRB, 0.26526E+00_JPRB, 0.26678E+00_JPRB, 0.26912E+00_JPRB, 0.27203E+00_JPRB, 0.27321E+00_JPRB, 0.27352E+00_JPRB ,&
&  0.26449E+00_JPRB, 0.25665E+00_JPRB, 0.24990E+00_JPRB, 0.23952E+00_JPRB, 0.23128E+00_JPRB, 0.22376E+00_JPRB, 0.21828E+00_JPRB ,&
&  0.21393E+00_JPRB, 0.21147E+00_JPRB, 0.20920E+00_JPRB, 0.20845E+00_JPRB, 0.20871E+00_JPRB, 0.20957E+00_JPRB, 0.20837E+00_JPRB ,&
&  0.20680E+00_JPRB, 0.20765E+00_JPRB, 0.20825E+00_JPRB, 0.21270E+00_JPRB, 0.21908E+00_JPRB, 0.22640E+00_JPRB, 0.23351E+00_JPRB ,&
&  0.24257E+00_JPRB, 0.24667E+00_JPRB, 0.24972E+00_JPRB, 0.25047E+00_JPRB, 0.24890E+00_JPRB, 0.24798E+00_JPRB, 0.24569E+00_JPRB ,&
&  0.24484E+00_JPRB, 0.24412E+00_JPRB, 0.24497E+00_JPRB, 0.24585E+00_JPRB, 0.24468E+00_JPRB, 0.24167E+00_JPRB, 0.23565E+00_JPRB ,&
&  0.22729E+00_JPRB, 0.22042E+00_JPRB, 0.21450E+00_JPRB, 0.20996E+00_JPRB, 0.20427E+00_JPRB, 0.19917E+00_JPRB, 0.19408E+00_JPRB ,&
&  0.18953E+00_JPRB, 0.18598E+00_JPRB, 0.18192E+00_JPRB, 0.17894E+00_JPRB, 0.17757E+00_JPRB, 0.17598E+00_JPRB, 0.17529E+00_JPRB ,&
&  0.17522E+00_JPRB, 0.17552E+00_JPRB, 0.17726E+00_JPRB, 0.17877E+00_JPRB, 0.18113E+00_JPRB, 0.18385E+00_JPRB, 0.18565E+00_JPRB ,&
&  0.18858E+00_JPRB, 0.19112E+00_JPRB, 0.19247E+00_JPRB, 0.19336E+00_JPRB, 0.19242E+00_JPRB, 0.18995E+00_JPRB, 0.18731E+00_JPRB ,&
&  0.18406E+00_JPRB, 0.18170E+00_JPRB, 0.18017E+00_JPRB, 0.17855E+00_JPRB, 0.17776E+00_JPRB, 0.17712E+00_JPRB, 0.17649E+00_JPRB ,&
&  0.17603E+00_JPRB, 0.17587E+00_JPRB, 0.17567E+00_JPRB, 0.17654E+00_JPRB, 0.17754E+00_JPRB, 0.17932E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18358E+00_JPRB, 0.18671E+00_JPRB, 0.19017E+00_JPRB, 0.19482E+00_JPRB, 0.20281E+00_JPRB, 0.21001E+00_JPRB, 0.22544E+00_JPRB ,&
&  0.23963E+00_JPRB, 0.26954E+00_JPRB, 0.28123E+00_JPRB, 0.26719E+00_JPRB, 0.26500E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.27823E+00_JPRB, 0.27536E+00_JPRB, 0.27285E+00_JPRB, 0.27259E+00_JPRB, 0.27313E+00_JPRB, 0.27622E+00_JPRB, 0.27412E+00_JPRB ,&
&  0.26985E+00_JPRB, 0.26837E+00_JPRB, 0.26981E+00_JPRB, 0.27042E+00_JPRB, 0.26949E+00_JPRB, 0.25860E+00_JPRB, 0.25241E+00_JPRB ,&
&  0.24573E+00_JPRB, 0.24315E+00_JPRB, 0.24118E+00_JPRB, 0.24177E+00_JPRB, 0.24421E+00_JPRB, 0.24574E+00_JPRB, 0.24183E+00_JPRB ,&
&  0.23750E+00_JPRB, 0.23322E+00_JPRB, 0.22898E+00_JPRB, 0.22721E+00_JPRB, 0.22853E+00_JPRB, 0.23013E+00_JPRB, 0.23054E+00_JPRB ,&
&  0.23125E+00_JPRB, 0.23387E+00_JPRB, 0.23616E+00_JPRB, 0.23942E+00_JPRB, 0.25208E+00_JPRB, 0.25099E+00_JPRB, 0.24971E+00_JPRB ,&
&  0.24987E+00_JPRB, 0.25009E+00_JPRB, 0.25087E+00_JPRB, 0.25119E+00_JPRB, 0.25449E+00_JPRB, 0.25776E+00_JPRB, 0.25989E+00_JPRB ,&
&  0.24592E+00_JPRB, 0.23712E+00_JPRB, 0.22901E+00_JPRB, 0.22019E+00_JPRB, 0.21102E+00_JPRB, 0.20335E+00_JPRB, 0.19625E+00_JPRB ,&
&  0.19017E+00_JPRB, 0.18445E+00_JPRB, 0.17947E+00_JPRB, 0.17748E+00_JPRB, 0.17681E+00_JPRB, 0.17585E+00_JPRB, 0.17586E+00_JPRB ,&
&  0.17639E+00_JPRB, 0.17709E+00_JPRB, 0.17758E+00_JPRB, 0.17897E+00_JPRB, 0.18105E+00_JPRB, 0.18427E+00_JPRB, 0.18772E+00_JPRB ,&
&  0.19298E+00_JPRB, 0.19715E+00_JPRB, 0.20192E+00_JPRB, 0.20551E+00_JPRB, 0.20405E+00_JPRB, 0.19940E+00_JPRB, 0.19428E+00_JPRB ,&
&  0.18856E+00_JPRB, 0.18522E+00_JPRB, 0.18192E+00_JPRB, 0.17955E+00_JPRB, 0.17824E+00_JPRB, 0.17756E+00_JPRB, 0.17798E+00_JPRB ,&
&  0.17830E+00_JPRB, 0.17884E+00_JPRB, 0.17857E+00_JPRB, 0.17883E+00_JPRB, 0.17961E+00_JPRB, 0.18122E+00_JPRB, 0.18330E+00_JPRB ,&
&  0.18649E+00_JPRB, 0.19020E+00_JPRB, 0.19471E+00_JPRB, 0.19972E+00_JPRB, 0.20681E+00_JPRB, 0.21655E+00_JPRB, 0.23089E+00_JPRB ,&
&  0.24805E+00_JPRB, 0.28241E+00_JPRB, 0.29634E+00_JPRB, 0.28481E+00_JPRB, 0.28129E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.29419E+00_JPRB, 0.29054E+00_JPRB, 0.28871E+00_JPRB, 0.28904E+00_JPRB, 0.28927E+00_JPRB, 0.29215E+00_JPRB, 0.29187E+00_JPRB ,&
&  0.28775E+00_JPRB, 0.28315E+00_JPRB, 0.28158E+00_JPRB, 0.28116E+00_JPRB, 0.27994E+00_JPRB, 0.27880E+00_JPRB, 0.27927E+00_JPRB ,&
&  0.27771E+00_JPRB, 0.27912E+00_JPRB, 0.28156E+00_JPRB, 0.28276E+00_JPRB, 0.28351E+00_JPRB, 0.28372E+00_JPRB, 0.28376E+00_JPRB ,&
&  0.27780E+00_JPRB, 0.27638E+00_JPRB, 0.28182E+00_JPRB, 0.27937E+00_JPRB, 0.27576E+00_JPRB, 0.27268E+00_JPRB, 0.27169E+00_JPRB ,&
&  0.26407E+00_JPRB, 0.25331E+00_JPRB, 0.24028E+00_JPRB, 0.24434E+00_JPRB, 0.26124E+00_JPRB, 0.26003E+00_JPRB, 0.25785E+00_JPRB ,&
&  0.25752E+00_JPRB, 0.25692E+00_JPRB, 0.25771E+00_JPRB, 0.25816E+00_JPRB, 0.25877E+00_JPRB, 0.25862E+00_JPRB, 0.25936E+00_JPRB ,&
&  0.26048E+00_JPRB, 0.26536E+00_JPRB, 0.26931E+00_JPRB, 0.25194E+00_JPRB, 0.23602E+00_JPRB, 0.22382E+00_JPRB, 0.21238E+00_JPRB ,&
&  0.20224E+00_JPRB, 0.19879E+00_JPRB, 0.19558E+00_JPRB, 0.19073E+00_JPRB, 0.19136E+00_JPRB, 0.19178E+00_JPRB, 0.19117E+00_JPRB ,&
&  0.18962E+00_JPRB, 0.18866E+00_JPRB, 0.18729E+00_JPRB, 0.18505E+00_JPRB, 0.18413E+00_JPRB, 0.18516E+00_JPRB, 0.18809E+00_JPRB ,&
&  0.19292E+00_JPRB, 0.19882E+00_JPRB, 0.21210E+00_JPRB, 0.22381E+00_JPRB, 0.22135E+00_JPRB, 0.21289E+00_JPRB, 0.20524E+00_JPRB ,&
&  0.19615E+00_JPRB, 0.19148E+00_JPRB, 0.18697E+00_JPRB, 0.18337E+00_JPRB, 0.18180E+00_JPRB, 0.18219E+00_JPRB, 0.18320E+00_JPRB ,&
&  0.18453E+00_JPRB, 0.18633E+00_JPRB, 0.18708E+00_JPRB, 0.18665E+00_JPRB, 0.18589E+00_JPRB, 0.18619E+00_JPRB, 0.18748E+00_JPRB ,&
&  0.19002E+00_JPRB, 0.19452E+00_JPRB, 0.20013E+00_JPRB, 0.20696E+00_JPRB, 0.21279E+00_JPRB, 0.21819E+00_JPRB, 0.22631E+00_JPRB ,&
&  0.23391E+00_JPRB, 0.25067E+00_JPRB, 0.27599E+00_JPRB, 0.30425E+00_JPRB, 0.30132E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.29519E+00_JPRB, 0.29231E+00_JPRB, 0.29266E+00_JPRB, 0.29313E+00_JPRB, 0.29219E+00_JPRB, 0.29235E+00_JPRB, 0.29195E+00_JPRB ,&
&  0.29201E+00_JPRB, 0.29025E+00_JPRB, 0.28821E+00_JPRB, 0.28832E+00_JPRB, 0.29008E+00_JPRB, 0.29212E+00_JPRB, 0.29246E+00_JPRB ,&
&  0.29199E+00_JPRB, 0.29299E+00_JPRB, 0.29596E+00_JPRB, 0.29937E+00_JPRB, 0.30019E+00_JPRB, 0.30050E+00_JPRB, 0.30223E+00_JPRB ,&
&  0.30452E+00_JPRB, 0.30531E+00_JPRB, 0.30791E+00_JPRB, 0.30583E+00_JPRB, 0.30159E+00_JPRB, 0.29972E+00_JPRB, 0.29811E+00_JPRB ,&
&  0.29266E+00_JPRB, 0.27034E+00_JPRB, 0.25518E+00_JPRB, 0.25848E+00_JPRB, 0.27497E+00_JPRB, 0.27518E+00_JPRB, 0.27330E+00_JPRB ,&
&  0.27141E+00_JPRB, 0.26897E+00_JPRB, 0.26682E+00_JPRB, 0.26600E+00_JPRB, 0.26589E+00_JPRB, 0.26553E+00_JPRB, 0.26378E+00_JPRB ,&
&  0.26375E+00_JPRB, 0.26752E+00_JPRB, 0.27339E+00_JPRB, 0.27228E+00_JPRB, 0.26950E+00_JPRB, 0.26362E+00_JPRB, 0.25987E+00_JPRB ,&
&  0.25630E+00_JPRB, 0.25222E+00_JPRB, 0.25005E+00_JPRB, 0.24924E+00_JPRB, 0.24891E+00_JPRB, 0.24714E+00_JPRB, 0.24418E+00_JPRB ,&
&  0.23596E+00_JPRB, 0.22742E+00_JPRB, 0.21818E+00_JPRB, 0.20912E+00_JPRB, 0.20199E+00_JPRB, 0.19716E+00_JPRB, 0.19369E+00_JPRB ,&
&  0.19476E+00_JPRB, 0.20411E+00_JPRB, 0.22096E+00_JPRB, 0.24001E+00_JPRB, 0.26269E+00_JPRB, 0.23957E+00_JPRB, 0.22815E+00_JPRB ,&
&  0.21748E+00_JPRB, 0.20844E+00_JPRB, 0.20225E+00_JPRB, 0.19815E+00_JPRB, 0.19532E+00_JPRB, 0.19387E+00_JPRB, 0.19401E+00_JPRB ,&
&  0.19666E+00_JPRB, 0.20035E+00_JPRB, 0.20364E+00_JPRB, 0.20412E+00_JPRB, 0.20312E+00_JPRB, 0.20133E+00_JPRB, 0.20031E+00_JPRB ,&
&  0.20108E+00_JPRB, 0.20558E+00_JPRB, 0.21354E+00_JPRB, 0.22493E+00_JPRB, 0.23567E+00_JPRB, 0.24693E+00_JPRB, 0.26060E+00_JPRB ,&
&  0.27568E+00_JPRB, 0.29110E+00_JPRB, 0.29483E+00_JPRB, 0.29774E+00_JPRB, 0.29773E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.30305E+00_JPRB, 0.30109E+00_JPRB, 0.30143E+00_JPRB, 0.30253E+00_JPRB, 0.30250E+00_JPRB, 0.30219E+00_JPRB, 0.29999E+00_JPRB ,&
&  0.29797E+00_JPRB, 0.29840E+00_JPRB, 0.29970E+00_JPRB, 0.30204E+00_JPRB, 0.30613E+00_JPRB, 0.31042E+00_JPRB, 0.30961E+00_JPRB ,&
&  0.30637E+00_JPRB, 0.30482E+00_JPRB, 0.30675E+00_JPRB, 0.30898E+00_JPRB, 0.30950E+00_JPRB, 0.31010E+00_JPRB, 0.31351E+00_JPRB ,&
&  0.31791E+00_JPRB, 0.31983E+00_JPRB, 0.32259E+00_JPRB, 0.32214E+00_JPRB, 0.31865E+00_JPRB, 0.31885E+00_JPRB, 0.31714E+00_JPRB ,&
&  0.31239E+00_JPRB, 0.30697E+00_JPRB, 0.29826E+00_JPRB, 0.29943E+00_JPRB, 0.29592E+00_JPRB, 0.29313E+00_JPRB, 0.28906E+00_JPRB ,&
&  0.28470E+00_JPRB, 0.28129E+00_JPRB, 0.27741E+00_JPRB, 0.27423E+00_JPRB, 0.27250E+00_JPRB, 0.27245E+00_JPRB, 0.27031E+00_JPRB ,&
&  0.26878E+00_JPRB, 0.26983E+00_JPRB, 0.27131E+00_JPRB, 0.26973E+00_JPRB, 0.27062E+00_JPRB, 0.27736E+00_JPRB, 0.28235E+00_JPRB ,&
&  0.28394E+00_JPRB, 0.28374E+00_JPRB, 0.28136E+00_JPRB, 0.28077E+00_JPRB, 0.27969E+00_JPRB, 0.27773E+00_JPRB, 0.27451E+00_JPRB ,&
&  0.26993E+00_JPRB, 0.26771E+00_JPRB, 0.26620E+00_JPRB, 0.26311E+00_JPRB, 0.25882E+00_JPRB, 0.24142E+00_JPRB, 0.23183E+00_JPRB ,&
&  0.23078E+00_JPRB, 0.23935E+00_JPRB, 0.25475E+00_JPRB, 0.26046E+00_JPRB, 0.26473E+00_JPRB, 0.26936E+00_JPRB, 0.27049E+00_JPRB ,&
&  0.27321E+00_JPRB, 0.25251E+00_JPRB, 0.24135E+00_JPRB, 0.23330E+00_JPRB, 0.22602E+00_JPRB, 0.22100E+00_JPRB, 0.21977E+00_JPRB ,&
&  0.22240E+00_JPRB, 0.22840E+00_JPRB, 0.23315E+00_JPRB, 0.23526E+00_JPRB, 0.23474E+00_JPRB, 0.23123E+00_JPRB, 0.22789E+00_JPRB ,&
&  0.22699E+00_JPRB, 0.23260E+00_JPRB, 0.24439E+00_JPRB, 0.25754E+00_JPRB, 0.26594E+00_JPRB, 0.27481E+00_JPRB, 0.28243E+00_JPRB ,&
&  0.28765E+00_JPRB, 0.29250E+00_JPRB, 0.29869E+00_JPRB, 0.30111E+00_JPRB, 0.30241E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.30321E+00_JPRB, 0.30324E+00_JPRB, 0.30642E+00_JPRB, 0.30991E+00_JPRB, 0.31087E+00_JPRB, 0.31093E+00_JPRB, 0.30871E+00_JPRB ,&
&  0.30722E+00_JPRB, 0.30841E+00_JPRB, 0.31017E+00_JPRB, 0.31138E+00_JPRB, 0.31321E+00_JPRB, 0.31556E+00_JPRB, 0.31735E+00_JPRB ,&
&  0.31731E+00_JPRB, 0.31564E+00_JPRB, 0.31271E+00_JPRB, 0.31361E+00_JPRB, 0.31365E+00_JPRB, 0.31546E+00_JPRB, 0.32252E+00_JPRB ,&
&  0.32754E+00_JPRB, 0.33103E+00_JPRB, 0.33399E+00_JPRB, 0.33440E+00_JPRB, 0.33398E+00_JPRB, 0.33550E+00_JPRB, 0.33237E+00_JPRB ,&
&  0.32644E+00_JPRB, 0.32087E+00_JPRB, 0.31634E+00_JPRB, 0.30941E+00_JPRB, 0.31184E+00_JPRB, 0.30928E+00_JPRB, 0.30618E+00_JPRB ,&
&  0.30225E+00_JPRB, 0.29754E+00_JPRB, 0.29219E+00_JPRB, 0.28721E+00_JPRB, 0.28359E+00_JPRB, 0.28032E+00_JPRB, 0.27570E+00_JPRB ,&
&  0.27418E+00_JPRB, 0.27577E+00_JPRB, 0.28139E+00_JPRB, 0.28460E+00_JPRB, 0.28287E+00_JPRB, 0.28417E+00_JPRB, 0.29085E+00_JPRB ,&
&  0.29734E+00_JPRB, 0.30021E+00_JPRB, 0.30000E+00_JPRB, 0.30055E+00_JPRB, 0.30118E+00_JPRB, 0.30248E+00_JPRB, 0.30309E+00_JPRB ,&
&  0.30213E+00_JPRB, 0.30064E+00_JPRB, 0.29825E+00_JPRB, 0.29235E+00_JPRB, 0.28524E+00_JPRB, 0.27950E+00_JPRB, 0.27713E+00_JPRB ,&
&  0.27958E+00_JPRB, 0.27905E+00_JPRB, 0.27853E+00_JPRB, 0.27973E+00_JPRB, 0.28009E+00_JPRB, 0.27925E+00_JPRB, 0.27816E+00_JPRB ,&
&  0.27923E+00_JPRB, 0.27880E+00_JPRB, 0.28067E+00_JPRB, 0.27798E+00_JPRB, 0.27352E+00_JPRB, 0.26927E+00_JPRB, 0.26530E+00_JPRB ,&
&  0.26287E+00_JPRB, 0.26186E+00_JPRB, 0.26377E+00_JPRB, 0.26435E+00_JPRB, 0.26605E+00_JPRB, 0.26805E+00_JPRB, 0.26973E+00_JPRB ,&
&  0.27065E+00_JPRB, 0.27172E+00_JPRB, 0.27380E+00_JPRB, 0.27831E+00_JPRB, 0.28313E+00_JPRB, 0.28737E+00_JPRB, 0.29187E+00_JPRB ,&
&  0.29617E+00_JPRB, 0.29929E+00_JPRB, 0.30510E+00_JPRB, 0.30650E+00_JPRB, 0.30601E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.30769E+00_JPRB, 0.30975E+00_JPRB, 0.31179E+00_JPRB, 0.31363E+00_JPRB, 0.31488E+00_JPRB, 0.31587E+00_JPRB, 0.31510E+00_JPRB ,&
&  0.31494E+00_JPRB, 0.31652E+00_JPRB, 0.31858E+00_JPRB, 0.32026E+00_JPRB, 0.32116E+00_JPRB, 0.32289E+00_JPRB, 0.32450E+00_JPRB ,&
&  0.32555E+00_JPRB, 0.32474E+00_JPRB, 0.32191E+00_JPRB, 0.32092E+00_JPRB, 0.31991E+00_JPRB, 0.32463E+00_JPRB, 0.33281E+00_JPRB ,&
&  0.33641E+00_JPRB, 0.33873E+00_JPRB, 0.34162E+00_JPRB, 0.34238E+00_JPRB, 0.34291E+00_JPRB, 0.34275E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33366E+00_JPRB, 0.32922E+00_JPRB, 0.32677E+00_JPRB, 0.32351E+00_JPRB, 0.32301E+00_JPRB, 0.32025E+00_JPRB, 0.31811E+00_JPRB ,&
&  0.31665E+00_JPRB, 0.31436E+00_JPRB, 0.31189E+00_JPRB, 0.30757E+00_JPRB, 0.30200E+00_JPRB, 0.29483E+00_JPRB, 0.29007E+00_JPRB ,&
&  0.28825E+00_JPRB, 0.29156E+00_JPRB, 0.29959E+00_JPRB, 0.30365E+00_JPRB, 0.30114E+00_JPRB, 0.30102E+00_JPRB, 0.30444E+00_JPRB ,&
&  0.30910E+00_JPRB, 0.31409E+00_JPRB, 0.31564E+00_JPRB, 0.31477E+00_JPRB, 0.31301E+00_JPRB, 0.31460E+00_JPRB, 0.31865E+00_JPRB ,&
&  0.32226E+00_JPRB, 0.32435E+00_JPRB, 0.32386E+00_JPRB, 0.32313E+00_JPRB, 0.32061E+00_JPRB, 0.31459E+00_JPRB, 0.30642E+00_JPRB ,&
&  0.29902E+00_JPRB, 0.29446E+00_JPRB, 0.29107E+00_JPRB, 0.28792E+00_JPRB, 0.28684E+00_JPRB, 0.28731E+00_JPRB, 0.28948E+00_JPRB ,&
&  0.29035E+00_JPRB, 0.29219E+00_JPRB, 0.29233E+00_JPRB, 0.28824E+00_JPRB, 0.28495E+00_JPRB, 0.28612E+00_JPRB, 0.28755E+00_JPRB ,&
&  0.28997E+00_JPRB, 0.29226E+00_JPRB, 0.29466E+00_JPRB, 0.29577E+00_JPRB, 0.29736E+00_JPRB, 0.29984E+00_JPRB, 0.30121E+00_JPRB ,&
&  0.30189E+00_JPRB, 0.30247E+00_JPRB, 0.30422E+00_JPRB, 0.30703E+00_JPRB, 0.30941E+00_JPRB, 0.31116E+00_JPRB, 0.31026E+00_JPRB ,&
&  0.30899E+00_JPRB, 0.30783E+00_JPRB, 0.30849E+00_JPRB, 0.30894E+00_JPRB, 0.30759E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.32699E+00_JPRB, 0.32717E+00_JPRB, 0.32676E+00_JPRB, 0.32593E+00_JPRB, 0.32515E+00_JPRB, 0.32453E+00_JPRB, 0.32480E+00_JPRB ,&
&  0.32515E+00_JPRB, 0.32590E+00_JPRB, 0.32679E+00_JPRB, 0.32753E+00_JPRB, 0.32917E+00_JPRB, 0.33099E+00_JPRB, 0.33237E+00_JPRB ,&
&  0.33392E+00_JPRB, 0.33333E+00_JPRB, 0.33132E+00_JPRB, 0.32931E+00_JPRB, 0.33011E+00_JPRB, 0.33531E+00_JPRB, 0.34118E+00_JPRB ,&
&  0.34286E+00_JPRB, 0.34417E+00_JPRB, 0.34492E+00_JPRB, 0.34412E+00_JPRB, 0.34311E+00_JPRB, 0.34287E+00_JPRB, 0.33929E+00_JPRB ,&
&  0.33509E+00_JPRB, 0.33367E+00_JPRB, 0.33346E+00_JPRB, 0.33045E+00_JPRB, 0.32934E+00_JPRB, 0.33063E+00_JPRB, 0.32876E+00_JPRB ,&
&  0.32748E+00_JPRB, 0.32589E+00_JPRB, 0.32329E+00_JPRB, 0.32127E+00_JPRB, 0.31888E+00_JPRB, 0.31401E+00_JPRB, 0.30823E+00_JPRB ,&
&  0.30346E+00_JPRB, 0.30370E+00_JPRB, 0.30877E+00_JPRB, 0.31427E+00_JPRB, 0.31383E+00_JPRB, 0.31334E+00_JPRB, 0.31502E+00_JPRB ,&
&  0.31857E+00_JPRB, 0.32290E+00_JPRB, 0.32564E+00_JPRB, 0.32487E+00_JPRB, 0.32385E+00_JPRB, 0.32575E+00_JPRB, 0.32839E+00_JPRB ,&
&  0.32977E+00_JPRB, 0.33135E+00_JPRB, 0.33247E+00_JPRB, 0.33392E+00_JPRB, 0.33413E+00_JPRB, 0.33181E+00_JPRB, 0.32966E+00_JPRB ,&
&  0.32593E+00_JPRB, 0.32144E+00_JPRB, 0.31642E+00_JPRB, 0.31119E+00_JPRB, 0.30782E+00_JPRB, 0.30611E+00_JPRB, 0.30674E+00_JPRB ,&
&  0.30659E+00_JPRB, 0.30836E+00_JPRB, 0.30884E+00_JPRB, 0.30658E+00_JPRB, 0.30355E+00_JPRB, 0.30350E+00_JPRB, 0.30687E+00_JPRB ,&
&  0.31149E+00_JPRB, 0.31450E+00_JPRB, 0.31979E+00_JPRB, 0.32258E+00_JPRB, 0.32461E+00_JPRB, 0.32701E+00_JPRB, 0.32763E+00_JPRB ,&
&  0.32507E+00_JPRB, 0.32304E+00_JPRB, 0.32267E+00_JPRB, 0.32545E+00_JPRB, 0.32616E+00_JPRB, 0.32530E+00_JPRB, 0.32318E+00_JPRB ,&
&  0.32285E+00_JPRB, 0.32479E+00_JPRB, 0.32550E+00_JPRB, 0.32524E+00_JPRB, 0.32576E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.35049E+00_JPRB, 0.34909E+00_JPRB, 0.34779E+00_JPRB, 0.34580E+00_JPRB, 0.34370E+00_JPRB, 0.34188E+00_JPRB, 0.34029E+00_JPRB ,&
&  0.33934E+00_JPRB, 0.33940E+00_JPRB, 0.34014E+00_JPRB, 0.34043E+00_JPRB, 0.34025E+00_JPRB, 0.34103E+00_JPRB, 0.34131E+00_JPRB ,&
&  0.34279E+00_JPRB, 0.34313E+00_JPRB, 0.34145E+00_JPRB, 0.34151E+00_JPRB, 0.34425E+00_JPRB, 0.34910E+00_JPRB, 0.35127E+00_JPRB ,&
&  0.34940E+00_JPRB, 0.34961E+00_JPRB, 0.34986E+00_JPRB, 0.34829E+00_JPRB, 0.34727E+00_JPRB, 0.34649E+00_JPRB, 0.34260E+00_JPRB ,&
&  0.33827E+00_JPRB, 0.33796E+00_JPRB, 0.33997E+00_JPRB, 0.34199E+00_JPRB, 0.34053E+00_JPRB, 0.33752E+00_JPRB, 0.33757E+00_JPRB ,&
&  0.33577E+00_JPRB, 0.33568E+00_JPRB, 0.33559E+00_JPRB, 0.33630E+00_JPRB, 0.33483E+00_JPRB, 0.33277E+00_JPRB, 0.32909E+00_JPRB ,&
&  0.32124E+00_JPRB, 0.31896E+00_JPRB, 0.32106E+00_JPRB, 0.32510E+00_JPRB, 0.32475E+00_JPRB, 0.32571E+00_JPRB, 0.32881E+00_JPRB ,&
&  0.33266E+00_JPRB, 0.33555E+00_JPRB, 0.33745E+00_JPRB, 0.33889E+00_JPRB, 0.34076E+00_JPRB, 0.34253E+00_JPRB, 0.34362E+00_JPRB ,&
&  0.34498E+00_JPRB, 0.34708E+00_JPRB, 0.34934E+00_JPRB, 0.35136E+00_JPRB, 0.35158E+00_JPRB, 0.35059E+00_JPRB, 0.34876E+00_JPRB ,&
&  0.34611E+00_JPRB, 0.34192E+00_JPRB, 0.33756E+00_JPRB, 0.33216E+00_JPRB, 0.32723E+00_JPRB, 0.32499E+00_JPRB, 0.32435E+00_JPRB ,&
&  0.32188E+00_JPRB, 0.31975E+00_JPRB, 0.31822E+00_JPRB, 0.31819E+00_JPRB, 0.31868E+00_JPRB, 0.32010E+00_JPRB, 0.32252E+00_JPRB ,&
&  0.32448E+00_JPRB, 0.32723E+00_JPRB, 0.33335E+00_JPRB, 0.33771E+00_JPRB, 0.33998E+00_JPRB, 0.34303E+00_JPRB, 0.34449E+00_JPRB ,&
&  0.34293E+00_JPRB, 0.34144E+00_JPRB, 0.34034E+00_JPRB, 0.34224E+00_JPRB, 0.34425E+00_JPRB, 0.34533E+00_JPRB, 0.34645E+00_JPRB ,&
&  0.34933E+00_JPRB, 0.35224E+00_JPRB, 0.35270E+00_JPRB, 0.35235E+00_JPRB, 0.35196E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.37183E+00_JPRB, 0.36836E+00_JPRB, 0.36473E+00_JPRB, 0.36155E+00_JPRB, 0.35882E+00_JPRB, 0.35590E+00_JPRB, 0.35325E+00_JPRB ,&
&  0.35061E+00_JPRB, 0.34918E+00_JPRB, 0.34823E+00_JPRB, 0.34739E+00_JPRB, 0.34450E+00_JPRB, 0.34326E+00_JPRB, 0.34416E+00_JPRB ,&
&  0.34663E+00_JPRB, 0.34908E+00_JPRB, 0.35129E+00_JPRB, 0.35144E+00_JPRB, 0.35407E+00_JPRB, 0.35658E+00_JPRB, 0.35712E+00_JPRB ,&
&  0.35680E+00_JPRB, 0.35656E+00_JPRB, 0.35509E+00_JPRB, 0.35288E+00_JPRB, 0.35315E+00_JPRB, 0.35356E+00_JPRB, 0.34983E+00_JPRB ,&
&  0.34369E+00_JPRB, 0.34320E+00_JPRB, 0.33608E+00_JPRB, 0.34658E+00_JPRB, 0.34846E+00_JPRB, 0.34398E+00_JPRB, 0.33874E+00_JPRB ,&
&  0.33776E+00_JPRB, 0.33926E+00_JPRB, 0.34243E+00_JPRB, 0.34420E+00_JPRB, 0.34441E+00_JPRB, 0.34418E+00_JPRB, 0.34410E+00_JPRB ,&
&  0.33731E+00_JPRB, 0.33351E+00_JPRB, 0.33348E+00_JPRB, 0.33457E+00_JPRB, 0.33484E+00_JPRB, 0.33574E+00_JPRB, 0.33786E+00_JPRB ,&
&  0.33948E+00_JPRB, 0.34072E+00_JPRB, 0.34217E+00_JPRB, 0.34516E+00_JPRB, 0.34746E+00_JPRB, 0.34878E+00_JPRB, 0.35098E+00_JPRB ,&
&  0.35408E+00_JPRB, 0.35754E+00_JPRB, 0.36006E+00_JPRB, 0.36116E+00_JPRB, 0.36278E+00_JPRB, 0.36399E+00_JPRB, 0.36421E+00_JPRB ,&
&  0.36287E+00_JPRB, 0.36044E+00_JPRB, 0.35775E+00_JPRB, 0.35333E+00_JPRB, 0.34751E+00_JPRB, 0.34315E+00_JPRB, 0.34009E+00_JPRB ,&
&  0.33508E+00_JPRB, 0.33321E+00_JPRB, 0.33268E+00_JPRB, 0.33136E+00_JPRB, 0.33029E+00_JPRB, 0.33021E+00_JPRB, 0.33091E+00_JPRB ,&
&  0.33301E+00_JPRB, 0.33670E+00_JPRB, 0.33931E+00_JPRB, 0.34258E+00_JPRB, 0.34676E+00_JPRB, 0.35092E+00_JPRB, 0.35151E+00_JPRB ,&
&  0.35050E+00_JPRB, 0.35009E+00_JPRB, 0.35189E+00_JPRB, 0.35682E+00_JPRB, 0.36252E+00_JPRB, 0.36831E+00_JPRB, 0.37405E+00_JPRB ,&
&  0.37847E+00_JPRB, 0.38038E+00_JPRB, 0.37941E+00_JPRB, 0.37808E+00_JPRB, 0.37530E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.40020E+00_JPRB, 0.39355E+00_JPRB, 0.38660E+00_JPRB, 0.38139E+00_JPRB, 0.37770E+00_JPRB, 0.37374E+00_JPRB, 0.36900E+00_JPRB ,&
&  0.36553E+00_JPRB, 0.36087E+00_JPRB, 0.35745E+00_JPRB, 0.35426E+00_JPRB, 0.35156E+00_JPRB, 0.35085E+00_JPRB, 0.35198E+00_JPRB ,&
&  0.35466E+00_JPRB, 0.36004E+00_JPRB, 0.36348E+00_JPRB, 0.36282E+00_JPRB, 0.36496E+00_JPRB, 0.36741E+00_JPRB, 0.36822E+00_JPRB ,&
&  0.36650E+00_JPRB, 0.36395E+00_JPRB, 0.36110E+00_JPRB, 0.35949E+00_JPRB, 0.35961E+00_JPRB, 0.36058E+00_JPRB, 0.35721E+00_JPRB ,&
&  0.35223E+00_JPRB, 0.34144E+00_JPRB, 0.35064E+00_JPRB, 0.35388E+00_JPRB, 0.35454E+00_JPRB, 0.35355E+00_JPRB, 0.35127E+00_JPRB ,&
&  0.34695E+00_JPRB, 0.34420E+00_JPRB, 0.34477E+00_JPRB, 0.34628E+00_JPRB, 0.34821E+00_JPRB, 0.34873E+00_JPRB, 0.34928E+00_JPRB ,&
&  0.34603E+00_JPRB, 0.34450E+00_JPRB, 0.34328E+00_JPRB, 0.34286E+00_JPRB, 0.34399E+00_JPRB, 0.34527E+00_JPRB, 0.34607E+00_JPRB ,&
&  0.34538E+00_JPRB, 0.34643E+00_JPRB, 0.34785E+00_JPRB, 0.34771E+00_JPRB, 0.34973E+00_JPRB, 0.35248E+00_JPRB, 0.35589E+00_JPRB ,&
&  0.35973E+00_JPRB, 0.36440E+00_JPRB, 0.36903E+00_JPRB, 0.37428E+00_JPRB, 0.37970E+00_JPRB, 0.38341E+00_JPRB, 0.38551E+00_JPRB ,&
&  0.38579E+00_JPRB, 0.38387E+00_JPRB, 0.38004E+00_JPRB, 0.37378E+00_JPRB, 0.36835E+00_JPRB, 0.36422E+00_JPRB, 0.35856E+00_JPRB ,&
&  0.35095E+00_JPRB, 0.34657E+00_JPRB, 0.34428E+00_JPRB, 0.34355E+00_JPRB, 0.34242E+00_JPRB, 0.34219E+00_JPRB, 0.34363E+00_JPRB ,&
&  0.34664E+00_JPRB, 0.34969E+00_JPRB, 0.35168E+00_JPRB, 0.35314E+00_JPRB, 0.35518E+00_JPRB, 0.35505E+00_JPRB, 0.35525E+00_JPRB ,&
&  0.35778E+00_JPRB, 0.36302E+00_JPRB, 0.37181E+00_JPRB, 0.38582E+00_JPRB, 0.39824E+00_JPRB, 0.40706E+00_JPRB, 0.41261E+00_JPRB ,&
&  0.41503E+00_JPRB, 0.41527E+00_JPRB, 0.41287E+00_JPRB, 0.40999E+00_JPRB, 0.40622E+00_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.41779E+00_JPRB, 0.40860E+00_JPRB, 0.39914E+00_JPRB, 0.39311E+00_JPRB, 0.38904E+00_JPRB, 0.38488E+00_JPRB, 0.37983E+00_JPRB ,&
&  0.37406E+00_JPRB, 0.36901E+00_JPRB, 0.36426E+00_JPRB, 0.35961E+00_JPRB, 0.35770E+00_JPRB, 0.35709E+00_JPRB, 0.35830E+00_JPRB ,&
&  0.36136E+00_JPRB, 0.36628E+00_JPRB, 0.36905E+00_JPRB, 0.36959E+00_JPRB, 0.37059E+00_JPRB, 0.37066E+00_JPRB, 0.36939E+00_JPRB ,&
&  0.36588E+00_JPRB, 0.36151E+00_JPRB, 0.35934E+00_JPRB, 0.36047E+00_JPRB, 0.36090E+00_JPRB, 0.36141E+00_JPRB, 0.36058E+00_JPRB ,&
&  0.36326E+00_JPRB, 0.32875E+00_JPRB, 0.34882E+00_JPRB, 0.35717E+00_JPRB, 0.35565E+00_JPRB, 0.35619E+00_JPRB, 0.35397E+00_JPRB ,&
&  0.34850E+00_JPRB, 0.34530E+00_JPRB, 0.34349E+00_JPRB, 0.34464E+00_JPRB, 0.34712E+00_JPRB, 0.34804E+00_JPRB, 0.34796E+00_JPRB ,&
&  0.34522E+00_JPRB, 0.34539E+00_JPRB, 0.34537E+00_JPRB, 0.34596E+00_JPRB, 0.34714E+00_JPRB, 0.34826E+00_JPRB, 0.34864E+00_JPRB ,&
&  0.34782E+00_JPRB, 0.34544E+00_JPRB, 0.34301E+00_JPRB, 0.34323E+00_JPRB, 0.34518E+00_JPRB, 0.34914E+00_JPRB, 0.35538E+00_JPRB ,&
&  0.36297E+00_JPRB, 0.37102E+00_JPRB, 0.37925E+00_JPRB, 0.38709E+00_JPRB, 0.39442E+00_JPRB, 0.40000E+00_JPRB, 0.40395E+00_JPRB ,&
&  0.40326E+00_JPRB, 0.40027E+00_JPRB, 0.39409E+00_JPRB, 0.38613E+00_JPRB, 0.38013E+00_JPRB, 0.37378E+00_JPRB, 0.36721E+00_JPRB ,&
&  0.36206E+00_JPRB, 0.35726E+00_JPRB, 0.35434E+00_JPRB, 0.35407E+00_JPRB, 0.35456E+00_JPRB, 0.35437E+00_JPRB, 0.35495E+00_JPRB ,&
&  0.35595E+00_JPRB, 0.35705E+00_JPRB, 0.35808E+00_JPRB, 0.35881E+00_JPRB, 0.35834E+00_JPRB, 0.35782E+00_JPRB, 0.35943E+00_JPRB ,&
&  0.36362E+00_JPRB, 0.37110E+00_JPRB, 0.38367E+00_JPRB, 0.40128E+00_JPRB, 0.41951E+00_JPRB, 0.43324E+00_JPRB, 0.43977E+00_JPRB ,&
&  0.43925E+00_JPRB, 0.43600E+00_JPRB, 0.43029E+00_JPRB, 0.42709E+00_JPRB, 0.42468E+00_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.43028E+00_JPRB, 0.42571E+00_JPRB, 0.42086E+00_JPRB, 0.41587E+00_JPRB, 0.41026E+00_JPRB, 0.40344E+00_JPRB, 0.39594E+00_JPRB ,&
&  0.38981E+00_JPRB, 0.38455E+00_JPRB, 0.37950E+00_JPRB, 0.37595E+00_JPRB, 0.37301E+00_JPRB, 0.37265E+00_JPRB, 0.37344E+00_JPRB ,&
&  0.37465E+00_JPRB, 0.37482E+00_JPRB, 0.37464E+00_JPRB, 0.37560E+00_JPRB, 0.37579E+00_JPRB, 0.37404E+00_JPRB, 0.37135E+00_JPRB ,&
&  0.36658E+00_JPRB, 0.36250E+00_JPRB, 0.36136E+00_JPRB, 0.36417E+00_JPRB, 0.36362E+00_JPRB, 0.36532E+00_JPRB, 0.37147E+00_JPRB ,&
&  0.35390E+00_JPRB, 0.32269E+00_JPRB, 0.34600E+00_JPRB, 0.36894E+00_JPRB, 0.36436E+00_JPRB, 0.35979E+00_JPRB, 0.35710E+00_JPRB ,&
&  0.35342E+00_JPRB, 0.35195E+00_JPRB, 0.35081E+00_JPRB, 0.35086E+00_JPRB, 0.35232E+00_JPRB, 0.35199E+00_JPRB, 0.35036E+00_JPRB ,&
&  0.34926E+00_JPRB, 0.35160E+00_JPRB, 0.35403E+00_JPRB, 0.35545E+00_JPRB, 0.35559E+00_JPRB, 0.35514E+00_JPRB, 0.35238E+00_JPRB ,&
&  0.34984E+00_JPRB, 0.34853E+00_JPRB, 0.34905E+00_JPRB, 0.35272E+00_JPRB, 0.35854E+00_JPRB, 0.36721E+00_JPRB, 0.37878E+00_JPRB ,&
&  0.39046E+00_JPRB, 0.40133E+00_JPRB, 0.41156E+00_JPRB, 0.41948E+00_JPRB, 0.42562E+00_JPRB, 0.42969E+00_JPRB, 0.43132E+00_JPRB ,&
&  0.42644E+00_JPRB, 0.41735E+00_JPRB, 0.40629E+00_JPRB, 0.39830E+00_JPRB, 0.39268E+00_JPRB, 0.38889E+00_JPRB, 0.38655E+00_JPRB ,&
&  0.38587E+00_JPRB, 0.38491E+00_JPRB, 0.38327E+00_JPRB, 0.38241E+00_JPRB, 0.38208E+00_JPRB, 0.38087E+00_JPRB, 0.38023E+00_JPRB ,&
&  0.37988E+00_JPRB, 0.37888E+00_JPRB, 0.37686E+00_JPRB, 0.37482E+00_JPRB, 0.37324E+00_JPRB, 0.37398E+00_JPRB, 0.37653E+00_JPRB ,&
&  0.38237E+00_JPRB, 0.39366E+00_JPRB, 0.40846E+00_JPRB, 0.43474E+00_JPRB, 0.45359E+00_JPRB, 0.46723E+00_JPRB, 0.47273E+00_JPRB ,&
&  0.46768E+00_JPRB, 0.45410E+00_JPRB, 0.44296E+00_JPRB, 0.43683E+00_JPRB, 0.43662E+00_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.43756E+00_JPRB, 0.43746E+00_JPRB, 0.43372E+00_JPRB, 0.42850E+00_JPRB, 0.42362E+00_JPRB, 0.41716E+00_JPRB, 0.40888E+00_JPRB ,&
&  0.40115E+00_JPRB, 0.39364E+00_JPRB, 0.38941E+00_JPRB, 0.38592E+00_JPRB, 0.38051E+00_JPRB, 0.37818E+00_JPRB, 0.37962E+00_JPRB ,&
&  0.37954E+00_JPRB, 0.37595E+00_JPRB, 0.37137E+00_JPRB, 0.37126E+00_JPRB, 0.37012E+00_JPRB, 0.36821E+00_JPRB, 0.36521E+00_JPRB ,&
&  0.36189E+00_JPRB, 0.36114E+00_JPRB, 0.36232E+00_JPRB, 0.36415E+00_JPRB, 0.36165E+00_JPRB, 0.36308E+00_JPRB, 0.37092E+00_JPRB ,&
&  0.40355E+00_JPRB, 0.33800E+00_JPRB, 0.36382E+00_JPRB, 0.39304E+00_JPRB, 0.38548E+00_JPRB, 0.37804E+00_JPRB, 0.37099E+00_JPRB ,&
&  0.36717E+00_JPRB, 0.36728E+00_JPRB, 0.36790E+00_JPRB, 0.36553E+00_JPRB, 0.36237E+00_JPRB, 0.35831E+00_JPRB, 0.35672E+00_JPRB ,&
&  0.35665E+00_JPRB, 0.36051E+00_JPRB, 0.36312E+00_JPRB, 0.36378E+00_JPRB, 0.36201E+00_JPRB, 0.36025E+00_JPRB, 0.35905E+00_JPRB ,&
&  0.35919E+00_JPRB, 0.36114E+00_JPRB, 0.36358E+00_JPRB, 0.36662E+00_JPRB, 0.37171E+00_JPRB, 0.38111E+00_JPRB, 0.39411E+00_JPRB ,&
&  0.40911E+00_JPRB, 0.42232E+00_JPRB, 0.43384E+00_JPRB, 0.44315E+00_JPRB, 0.44938E+00_JPRB, 0.45186E+00_JPRB, 0.44857E+00_JPRB ,&
&  0.44086E+00_JPRB, 0.42743E+00_JPRB, 0.41860E+00_JPRB, 0.41186E+00_JPRB, 0.40485E+00_JPRB, 0.40021E+00_JPRB, 0.39763E+00_JPRB ,&
&  0.39658E+00_JPRB, 0.39581E+00_JPRB, 0.39437E+00_JPRB, 0.39170E+00_JPRB, 0.39080E+00_JPRB, 0.39040E+00_JPRB, 0.39167E+00_JPRB ,&
&  0.39158E+00_JPRB, 0.39063E+00_JPRB, 0.38770E+00_JPRB, 0.38382E+00_JPRB, 0.38362E+00_JPRB, 0.38509E+00_JPRB, 0.38788E+00_JPRB ,&
&  0.39432E+00_JPRB, 0.40591E+00_JPRB, 0.42121E+00_JPRB, 0.44867E+00_JPRB, 0.46350E+00_JPRB, 0.47500E+00_JPRB, 0.47775E+00_JPRB ,&
&  0.47012E+00_JPRB, 0.45726E+00_JPRB, 0.37182E+00_JPRB, 0.42416E+00_JPRB, 0.44014E+00_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.47513E+00_JPRB, 0.47000E+00_JPRB, 0.46051E+00_JPRB, 0.44517E+00_JPRB, 0.43399E+00_JPRB, 0.42557E+00_JPRB, 0.41672E+00_JPRB ,&
&  0.40889E+00_JPRB, 0.40280E+00_JPRB, 0.39817E+00_JPRB, 0.39244E+00_JPRB, 0.38591E+00_JPRB, 0.38551E+00_JPRB, 0.38659E+00_JPRB ,&
&  0.38494E+00_JPRB, 0.37937E+00_JPRB, 0.37381E+00_JPRB, 0.37301E+00_JPRB, 0.37286E+00_JPRB, 0.37294E+00_JPRB, 0.37046E+00_JPRB ,&
&  0.36604E+00_JPRB, 0.36579E+00_JPRB, 0.36994E+00_JPRB, 0.37179E+00_JPRB, 0.36863E+00_JPRB, 0.37063E+00_JPRB, 0.37887E+00_JPRB ,&
&  0.41672E+00_JPRB, 0.35561E+00_JPRB, 0.37778E+00_JPRB, 0.42685E+00_JPRB, 0.42443E+00_JPRB, 0.42486E+00_JPRB, 0.41751E+00_JPRB ,&
&  0.41093E+00_JPRB, 0.40871E+00_JPRB, 0.40597E+00_JPRB, 0.39571E+00_JPRB, 0.38373E+00_JPRB, 0.37678E+00_JPRB, 0.37534E+00_JPRB ,&
&  0.37766E+00_JPRB, 0.38000E+00_JPRB, 0.38202E+00_JPRB, 0.38003E+00_JPRB, 0.37948E+00_JPRB, 0.37957E+00_JPRB, 0.38048E+00_JPRB ,&
&  0.38141E+00_JPRB, 0.38423E+00_JPRB, 0.38718E+00_JPRB, 0.38806E+00_JPRB, 0.39270E+00_JPRB, 0.40686E+00_JPRB, 0.42616E+00_JPRB ,&
&  0.44442E+00_JPRB, 0.46061E+00_JPRB, 0.47084E+00_JPRB, 0.47715E+00_JPRB, 0.47769E+00_JPRB, 0.47679E+00_JPRB, 0.47136E+00_JPRB ,&
&  0.45834E+00_JPRB, 0.44497E+00_JPRB, 0.43802E+00_JPRB, 0.43034E+00_JPRB, 0.42259E+00_JPRB, 0.41715E+00_JPRB, 0.41255E+00_JPRB ,&
&  0.40897E+00_JPRB, 0.40618E+00_JPRB, 0.40419E+00_JPRB, 0.40002E+00_JPRB, 0.39732E+00_JPRB, 0.39839E+00_JPRB, 0.39919E+00_JPRB ,&
&  0.39896E+00_JPRB, 0.40024E+00_JPRB, 0.40243E+00_JPRB, 0.40519E+00_JPRB, 0.40803E+00_JPRB, 0.41172E+00_JPRB, 0.41599E+00_JPRB ,&
&  0.42734E+00_JPRB, 0.44441E+00_JPRB, 0.46464E+00_JPRB, 0.47329E+00_JPRB, 0.49082E+00_JPRB, 0.49417E+00_JPRB, 0.48625E+00_JPRB ,&
&  0.47338E+00_JPRB, 0.45827E+00_JPRB, 0.45144E+00_JPRB, 0.40422E+00_JPRB, 0.47550E+00_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.53070E+00_JPRB, 0.51158E+00_JPRB, 0.48182E+00_JPRB, 0.45333E+00_JPRB, 0.43357E+00_JPRB, 0.42098E+00_JPRB, 0.41379E+00_JPRB ,&
&  0.40882E+00_JPRB, 0.40520E+00_JPRB, 0.40093E+00_JPRB, 0.39355E+00_JPRB, 0.38927E+00_JPRB, 0.38812E+00_JPRB, 0.38773E+00_JPRB ,&
&  0.38561E+00_JPRB, 0.37811E+00_JPRB, 0.37603E+00_JPRB, 0.37413E+00_JPRB, 0.37342E+00_JPRB, 0.37268E+00_JPRB, 0.36901E+00_JPRB ,&
&  0.36410E+00_JPRB, 0.36211E+00_JPRB, 0.36664E+00_JPRB, 0.36961E+00_JPRB, 0.37220E+00_JPRB, 0.37719E+00_JPRB, 0.38972E+00_JPRB ,&
&  0.42651E+00_JPRB, 0.38891E+00_JPRB, 0.43032E+00_JPRB, 0.46009E+00_JPRB, 0.45731E+00_JPRB, 0.45529E+00_JPRB, 0.45455E+00_JPRB ,&
&  0.45035E+00_JPRB, 0.44669E+00_JPRB, 0.43461E+00_JPRB, 0.41451E+00_JPRB, 0.39782E+00_JPRB, 0.39550E+00_JPRB, 0.39211E+00_JPRB ,&
&  0.38939E+00_JPRB, 0.38957E+00_JPRB, 0.39089E+00_JPRB, 0.39289E+00_JPRB, 0.39445E+00_JPRB, 0.39493E+00_JPRB, 0.39539E+00_JPRB ,&
&  0.39678E+00_JPRB, 0.39911E+00_JPRB, 0.40001E+00_JPRB, 0.40350E+00_JPRB, 0.41455E+00_JPRB, 0.43124E+00_JPRB, 0.44946E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.47589E+00_JPRB, 0.48878E+00_JPRB, 0.49655E+00_JPRB, 0.49768E+00_JPRB, 0.49268E+00_JPRB, 0.48321E+00_JPRB ,&
&  0.47198E+00_JPRB, 0.46841E+00_JPRB, 0.45760E+00_JPRB, 0.44557E+00_JPRB, 0.43800E+00_JPRB, 0.43353E+00_JPRB, 0.43041E+00_JPRB ,&
&  0.42844E+00_JPRB, 0.42529E+00_JPRB, 0.42045E+00_JPRB, 0.41616E+00_JPRB, 0.41489E+00_JPRB, 0.41216E+00_JPRB, 0.40813E+00_JPRB ,&
&  0.40738E+00_JPRB, 0.41095E+00_JPRB, 0.41776E+00_JPRB, 0.42293E+00_JPRB, 0.42782E+00_JPRB, 0.43504E+00_JPRB, 0.44081E+00_JPRB ,&
&  0.45061E+00_JPRB, 0.46684E+00_JPRB, 0.48158E+00_JPRB, 0.50604E+00_JPRB, 0.49941E+00_JPRB, 0.49554E+00_JPRB, 0.48102E+00_JPRB ,&
&  0.46675E+00_JPRB, 0.45490E+00_JPRB, 0.44364E+00_JPRB, 0.46494E+00_JPRB, 0.52343E+00_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.57237E+00_JPRB, 0.52392E+00_JPRB, 0.48411E+00_JPRB, 0.45217E+00_JPRB, 0.43306E+00_JPRB, 0.42207E+00_JPRB, 0.41420E+00_JPRB ,&
&  0.41107E+00_JPRB, 0.41048E+00_JPRB, 0.40573E+00_JPRB, 0.40183E+00_JPRB, 0.39949E+00_JPRB, 0.39922E+00_JPRB, 0.39955E+00_JPRB ,&
&  0.39288E+00_JPRB, 0.39212E+00_JPRB, 0.39265E+00_JPRB, 0.38911E+00_JPRB, 0.38419E+00_JPRB, 0.37927E+00_JPRB, 0.37277E+00_JPRB ,&
&  0.36879E+00_JPRB, 0.36802E+00_JPRB, 0.37100E+00_JPRB, 0.37546E+00_JPRB, 0.37944E+00_JPRB, 0.39030E+00_JPRB, 0.40991E+00_JPRB ,&
&  0.44727E+00_JPRB, 0.41421E+00_JPRB, 0.48025E+00_JPRB, 0.50437E+00_JPRB, 0.47049E+00_JPRB, 0.47687E+00_JPRB, 0.48728E+00_JPRB ,&
&  0.49476E+00_JPRB, 0.49707E+00_JPRB, 0.48169E+00_JPRB, 0.45839E+00_JPRB, 0.43823E+00_JPRB, 0.42764E+00_JPRB, 0.42166E+00_JPRB ,&
&  0.41551E+00_JPRB, 0.41172E+00_JPRB, 0.40859E+00_JPRB, 0.41219E+00_JPRB, 0.41548E+00_JPRB, 0.41539E+00_JPRB, 0.41498E+00_JPRB ,&
&  0.41439E+00_JPRB, 0.41418E+00_JPRB, 0.41509E+00_JPRB, 0.42518E+00_JPRB, 0.44421E+00_JPRB, 0.46657E+00_JPRB, 0.48699E+00_JPRB ,&
&  0.49925E+00_JPRB, 0.51095E+00_JPRB, 0.52185E+00_JPRB, 0.52796E+00_JPRB, 0.52593E+00_JPRB, 0.51974E+00_JPRB, 0.51297E+00_JPRB ,&
&  0.50685E+00_JPRB, 0.49381E+00_JPRB, 0.47518E+00_JPRB, 0.46146E+00_JPRB, 0.45516E+00_JPRB, 0.45179E+00_JPRB, 0.45199E+00_JPRB ,&
&  0.45276E+00_JPRB, 0.45332E+00_JPRB, 0.45112E+00_JPRB, 0.44611E+00_JPRB, 0.43777E+00_JPRB, 0.43203E+00_JPRB, 0.42657E+00_JPRB ,&
&  0.42396E+00_JPRB, 0.42551E+00_JPRB, 0.43078E+00_JPRB, 0.43920E+00_JPRB, 0.44626E+00_JPRB, 0.45330E+00_JPRB, 0.45972E+00_JPRB ,&
&  0.47133E+00_JPRB, 0.49419E+00_JPRB, 0.48319E+00_JPRB, 0.48514E+00_JPRB, 0.52103E+00_JPRB, 0.50300E+00_JPRB, 0.48298E+00_JPRB ,&
&  0.46655E+00_JPRB, 0.45481E+00_JPRB, 0.45495E+00_JPRB, 0.48053E+00_JPRB, 0.60031E+00_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.48562E+00_JPRB, 0.46967E+00_JPRB, 0.45419E+00_JPRB, 0.43836E+00_JPRB, 0.42411E+00_JPRB, 0.41723E+00_JPRB, 0.41576E+00_JPRB ,&
&  0.41840E+00_JPRB, 0.42093E+00_JPRB, 0.41946E+00_JPRB, 0.41384E+00_JPRB, 0.41066E+00_JPRB, 0.40798E+00_JPRB, 0.40622E+00_JPRB ,&
&  0.40106E+00_JPRB, 0.40205E+00_JPRB, 0.39902E+00_JPRB, 0.39404E+00_JPRB, 0.38862E+00_JPRB, 0.38195E+00_JPRB, 0.37550E+00_JPRB ,&
&  0.37192E+00_JPRB, 0.37154E+00_JPRB, 0.37369E+00_JPRB, 0.37858E+00_JPRB, 0.39050E+00_JPRB, 0.40189E+00_JPRB, 0.41324E+00_JPRB ,&
&  0.44617E+00_JPRB, 0.44522E+00_JPRB, 0.55033E+00_JPRB, 0.61057E+00_JPRB, 0.55731E+00_JPRB, 0.54291E+00_JPRB, 0.54503E+00_JPRB ,&
&  0.54430E+00_JPRB, 0.53506E+00_JPRB, 0.51618E+00_JPRB, 0.49936E+00_JPRB, 0.48213E+00_JPRB, 0.47627E+00_JPRB, 0.46811E+00_JPRB ,&
&  0.45834E+00_JPRB, 0.45032E+00_JPRB, 0.43983E+00_JPRB, 0.43794E+00_JPRB, 0.43733E+00_JPRB, 0.43509E+00_JPRB, 0.43194E+00_JPRB ,&
&  0.43077E+00_JPRB, 0.43129E+00_JPRB, 0.43417E+00_JPRB, 0.44888E+00_JPRB, 0.47680E+00_JPRB, 0.50714E+00_JPRB, 0.54080E+00_JPRB ,&
&  0.54928E+00_JPRB, 0.55532E+00_JPRB, 0.55969E+00_JPRB, 0.55885E+00_JPRB, 0.54721E+00_JPRB, 0.53787E+00_JPRB, 0.53659E+00_JPRB ,&
&  0.53077E+00_JPRB, 0.50928E+00_JPRB, 0.48594E+00_JPRB, 0.47122E+00_JPRB, 0.46328E+00_JPRB, 0.46232E+00_JPRB, 0.46340E+00_JPRB ,&
&  0.46745E+00_JPRB, 0.47028E+00_JPRB, 0.46512E+00_JPRB, 0.45459E+00_JPRB, 0.44382E+00_JPRB, 0.43742E+00_JPRB, 0.43485E+00_JPRB ,&
&  0.43236E+00_JPRB, 0.43398E+00_JPRB, 0.44176E+00_JPRB, 0.44683E+00_JPRB, 0.44916E+00_JPRB, 0.45420E+00_JPRB, 0.46392E+00_JPRB ,&
&  0.47931E+00_JPRB, 0.48710E+00_JPRB, 0.48333E+00_JPRB, 0.48654E+00_JPRB, 0.48234E+00_JPRB, 0.49712E+00_JPRB, 0.48187E+00_JPRB ,&
&  0.46413E+00_JPRB, 0.46035E+00_JPRB, 0.47352E+00_JPRB, 0.48783E+00_JPRB, 0.49915E+00_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.46782E+00_JPRB, 0.45547E+00_JPRB, 0.44427E+00_JPRB, 0.42628E+00_JPRB, 0.41534E+00_JPRB, 0.40951E+00_JPRB, 0.41190E+00_JPRB ,&
&  0.41800E+00_JPRB, 0.42566E+00_JPRB, 0.42343E+00_JPRB, 0.42077E+00_JPRB, 0.41606E+00_JPRB, 0.41368E+00_JPRB, 0.40759E+00_JPRB ,&
&  0.40458E+00_JPRB, 0.40146E+00_JPRB, 0.39633E+00_JPRB, 0.39144E+00_JPRB, 0.38340E+00_JPRB, 0.38029E+00_JPRB, 0.37565E+00_JPRB ,&
&  0.37112E+00_JPRB, 0.36963E+00_JPRB, 0.37246E+00_JPRB, 0.38170E+00_JPRB, 0.39294E+00_JPRB, 0.40537E+00_JPRB, 0.41654E+00_JPRB ,&
&  0.45546E+00_JPRB, 0.43951E+00_JPRB, 0.67846E+00_JPRB, 0.71629E+00_JPRB, 0.68490E+00_JPRB, 0.64880E+00_JPRB, 0.65974E+00_JPRB ,&
&  0.65834E+00_JPRB, 0.62173E+00_JPRB, 0.58924E+00_JPRB, 0.56780E+00_JPRB, 0.54955E+00_JPRB, 0.53682E+00_JPRB, 0.51873E+00_JPRB ,&
&  0.50155E+00_JPRB, 0.48274E+00_JPRB, 0.45979E+00_JPRB, 0.44685E+00_JPRB, 0.43971E+00_JPRB, 0.43801E+00_JPRB, 0.44213E+00_JPRB ,&
&  0.44659E+00_JPRB, 0.44820E+00_JPRB, 0.45692E+00_JPRB, 0.47558E+00_JPRB, 0.48391E+00_JPRB, 0.47063E+00_JPRB, 0.51118E+00_JPRB ,&
&  0.60708E+00_JPRB, 0.60199E+00_JPRB, 0.58376E+00_JPRB, 0.57241E+00_JPRB, 0.54843E+00_JPRB, 0.53952E+00_JPRB, 0.53730E+00_JPRB ,&
&  0.52911E+00_JPRB, 0.51400E+00_JPRB, 0.49746E+00_JPRB, 0.48428E+00_JPRB, 0.47534E+00_JPRB, 0.47097E+00_JPRB, 0.47553E+00_JPRB ,&
&  0.47750E+00_JPRB, 0.47088E+00_JPRB, 0.45578E+00_JPRB, 0.44327E+00_JPRB, 0.43692E+00_JPRB, 0.43668E+00_JPRB, 0.43939E+00_JPRB ,&
&  0.44204E+00_JPRB, 0.44543E+00_JPRB, 0.44596E+00_JPRB, 0.43532E+00_JPRB, 0.44274E+00_JPRB, 0.45953E+00_JPRB, 0.46493E+00_JPRB ,&
&  0.45468E+00_JPRB, 0.45495E+00_JPRB, 0.46653E+00_JPRB, 0.46884E+00_JPRB, 0.46910E+00_JPRB, 0.49859E+00_JPRB, 0.48372E+00_JPRB ,&
&  0.46591E+00_JPRB, 0.46771E+00_JPRB, 0.48422E+00_JPRB, 0.49070E+00_JPRB, 0.49023E+00_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.45320E+00_JPRB, 0.44023E+00_JPRB, 0.43160E+00_JPRB, 0.41937E+00_JPRB, 0.40825E+00_JPRB, 0.40192E+00_JPRB, 0.40464E+00_JPRB ,&
&  0.41731E+00_JPRB, 0.42917E+00_JPRB, 0.42442E+00_JPRB, 0.42283E+00_JPRB, 0.42288E+00_JPRB, 0.41823E+00_JPRB, 0.40995E+00_JPRB ,&
&  0.40158E+00_JPRB, 0.39362E+00_JPRB, 0.38475E+00_JPRB, 0.37603E+00_JPRB, 0.36981E+00_JPRB, 0.37068E+00_JPRB, 0.36948E+00_JPRB ,&
&  0.36779E+00_JPRB, 0.37108E+00_JPRB, 0.37611E+00_JPRB, 0.38513E+00_JPRB, 0.39349E+00_JPRB, 0.40479E+00_JPRB, 0.41594E+00_JPRB ,&
&  0.45710E+00_JPRB, 0.37971E+00_JPRB, 0.73420E+00_JPRB, 0.87753E+00_JPRB, 0.92212E+00_JPRB, 0.87115E+00_JPRB, 0.82516E+00_JPRB ,&
&  0.80772E+00_JPRB, 0.73135E+00_JPRB, 0.67701E+00_JPRB, 0.62862E+00_JPRB, 0.57219E+00_JPRB, 0.53840E+00_JPRB, 0.52004E+00_JPRB ,&
&  0.50583E+00_JPRB, 0.49066E+00_JPRB, 0.47649E+00_JPRB, 0.46526E+00_JPRB, 0.45674E+00_JPRB, 0.45500E+00_JPRB, 0.46195E+00_JPRB ,&
&  0.46921E+00_JPRB, 0.47294E+00_JPRB, 0.48934E+00_JPRB, 0.50898E+00_JPRB, 0.49238E+00_JPRB, 0.48912E+00_JPRB, 0.50169E+00_JPRB ,&
&  0.55349E+00_JPRB, 0.62263E+00_JPRB, 0.58731E+00_JPRB, 0.55125E+00_JPRB, 0.52988E+00_JPRB, 0.52605E+00_JPRB, 0.52776E+00_JPRB ,&
&  0.52238E+00_JPRB, 0.51455E+00_JPRB, 0.50505E+00_JPRB, 0.49350E+00_JPRB, 0.48384E+00_JPRB, 0.47986E+00_JPRB, 0.47691E+00_JPRB ,&
&  0.46868E+00_JPRB, 0.45445E+00_JPRB, 0.44549E+00_JPRB, 0.44105E+00_JPRB, 0.43940E+00_JPRB, 0.44004E+00_JPRB, 0.44515E+00_JPRB ,&
&  0.45424E+00_JPRB, 0.45568E+00_JPRB, 0.44500E+00_JPRB, 0.43045E+00_JPRB, 0.42741E+00_JPRB, 0.43216E+00_JPRB, 0.43514E+00_JPRB ,&
&  0.44400E+00_JPRB, 0.45124E+00_JPRB, 0.45558E+00_JPRB, 0.46383E+00_JPRB, 0.45144E+00_JPRB, 0.48815E+00_JPRB, 0.47670E+00_JPRB ,&
&  0.46989E+00_JPRB, 0.47160E+00_JPRB, 0.48007E+00_JPRB, 0.48236E+00_JPRB, 0.47243E+00_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.43507E+00_JPRB, 0.42620E+00_JPRB, 0.41797E+00_JPRB, 0.41394E+00_JPRB, 0.40637E+00_JPRB, 0.40246E+00_JPRB, 0.40611E+00_JPRB ,&
&  0.41625E+00_JPRB, 0.41972E+00_JPRB, 0.41829E+00_JPRB, 0.41620E+00_JPRB, 0.41249E+00_JPRB, 0.40755E+00_JPRB, 0.40004E+00_JPRB ,&
&  0.39353E+00_JPRB, 0.38313E+00_JPRB, 0.36991E+00_JPRB, 0.36081E+00_JPRB, 0.36577E+00_JPRB, 0.37059E+00_JPRB, 0.36977E+00_JPRB ,&
&  0.36851E+00_JPRB, 0.37413E+00_JPRB, 0.37654E+00_JPRB, 0.38645E+00_JPRB, 0.39334E+00_JPRB, 0.40558E+00_JPRB, 0.41886E+00_JPRB ,&
&  0.45960E+00_JPRB, 0.59121E+00_JPRB, 0.66757E+00_JPRB, 0.99511E+00_JPRB, 0.11507E+01_JPRB, 0.11041E+01_JPRB, 0.94496E+00_JPRB ,&
&  0.86425E+00_JPRB, 0.81656E+00_JPRB, 0.73069E+00_JPRB, 0.65929E+00_JPRB, 0.60867E+00_JPRB, 0.57069E+00_JPRB, 0.53605E+00_JPRB ,&
&  0.50795E+00_JPRB, 0.48968E+00_JPRB, 0.47868E+00_JPRB, 0.46864E+00_JPRB, 0.46636E+00_JPRB, 0.47022E+00_JPRB, 0.48077E+00_JPRB ,&
&  0.48988E+00_JPRB, 0.49518E+00_JPRB, 0.52007E+00_JPRB, 0.54372E+00_JPRB, 0.50423E+00_JPRB, 0.51080E+00_JPRB, 0.54466E+00_JPRB ,&
&  0.56575E+00_JPRB, 0.63894E+00_JPRB, 0.57423E+00_JPRB, 0.53330E+00_JPRB, 0.52069E+00_JPRB, 0.52652E+00_JPRB, 0.52791E+00_JPRB ,&
&  0.52028E+00_JPRB, 0.51393E+00_JPRB, 0.50445E+00_JPRB, 0.49521E+00_JPRB, 0.48838E+00_JPRB, 0.48248E+00_JPRB, 0.46960E+00_JPRB ,&
&  0.45939E+00_JPRB, 0.45710E+00_JPRB, 0.45778E+00_JPRB, 0.45855E+00_JPRB, 0.46062E+00_JPRB, 0.46480E+00_JPRB, 0.46847E+00_JPRB ,&
&  0.47144E+00_JPRB, 0.46304E+00_JPRB, 0.43924E+00_JPRB, 0.42561E+00_JPRB, 0.42568E+00_JPRB, 0.42857E+00_JPRB, 0.42731E+00_JPRB ,&
&  0.44842E+00_JPRB, 0.45768E+00_JPRB, 0.45599E+00_JPRB, 0.45955E+00_JPRB, 0.46517E+00_JPRB, 0.51190E+00_JPRB, 0.48013E+00_JPRB ,&
&  0.47312E+00_JPRB, 0.47037E+00_JPRB, 0.47351E+00_JPRB, 0.47704E+00_JPRB, 0.45969E+00_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.42085E+00_JPRB, 0.41037E+00_JPRB, 0.40460E+00_JPRB, 0.40423E+00_JPRB, 0.40277E+00_JPRB, 0.40139E+00_JPRB, 0.40582E+00_JPRB ,&
&  0.40781E+00_JPRB, 0.40803E+00_JPRB, 0.40649E+00_JPRB, 0.40428E+00_JPRB, 0.39891E+00_JPRB, 0.39339E+00_JPRB, 0.38719E+00_JPRB ,&
&  0.38427E+00_JPRB, 0.37029E+00_JPRB, 0.36326E+00_JPRB, 0.37171E+00_JPRB, 0.37600E+00_JPRB, 0.37491E+00_JPRB, 0.37499E+00_JPRB ,&
&  0.37462E+00_JPRB, 0.37790E+00_JPRB, 0.37806E+00_JPRB, 0.38312E+00_JPRB, 0.39201E+00_JPRB, 0.41026E+00_JPRB, 0.42782E+00_JPRB ,&
&  0.48842E+00_JPRB, 0.58876E+00_JPRB, 0.66959E+00_JPRB, 0.11269E+01_JPRB, 0.12473E+01_JPRB, 0.11905E+01_JPRB, 0.10132E+01_JPRB ,&
&  0.89834E+00_JPRB, 0.89705E+00_JPRB, 0.80265E+00_JPRB, 0.71961E+00_JPRB, 0.66546E+00_JPRB, 0.61491E+00_JPRB, 0.55685E+00_JPRB ,&
&  0.50933E+00_JPRB, 0.48366E+00_JPRB, 0.47535E+00_JPRB, 0.47801E+00_JPRB, 0.48559E+00_JPRB, 0.49209E+00_JPRB, 0.50077E+00_JPRB ,&
&  0.50736E+00_JPRB, 0.52721E+00_JPRB, 0.55692E+00_JPRB, 0.53810E+00_JPRB, 0.51008E+00_JPRB, 0.52788E+00_JPRB, 0.54622E+00_JPRB ,&
&  0.57262E+00_JPRB, 0.59887E+00_JPRB, 0.54843E+00_JPRB, 0.52116E+00_JPRB, 0.51210E+00_JPRB, 0.52667E+00_JPRB, 0.52561E+00_JPRB ,&
&  0.51591E+00_JPRB, 0.50852E+00_JPRB, 0.49785E+00_JPRB, 0.49183E+00_JPRB, 0.48710E+00_JPRB, 0.47708E+00_JPRB, 0.47140E+00_JPRB ,&
&  0.47254E+00_JPRB, 0.47478E+00_JPRB, 0.48105E+00_JPRB, 0.48399E+00_JPRB, 0.48606E+00_JPRB, 0.48531E+00_JPRB, 0.48288E+00_JPRB ,&
&  0.47490E+00_JPRB, 0.45686E+00_JPRB, 0.43684E+00_JPRB, 0.43723E+00_JPRB, 0.44824E+00_JPRB, 0.44865E+00_JPRB, 0.44604E+00_JPRB ,&
&  0.46125E+00_JPRB, 0.46969E+00_JPRB, 0.46983E+00_JPRB, 0.46086E+00_JPRB, 0.47225E+00_JPRB, 0.49835E+00_JPRB, 0.47547E+00_JPRB ,&
&  0.47598E+00_JPRB, 0.47256E+00_JPRB, 0.47205E+00_JPRB, 0.47085E+00_JPRB, 0.44751E+00_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.42698E+00_JPRB, 0.40976E+00_JPRB, 0.40124E+00_JPRB, 0.39548E+00_JPRB, 0.39403E+00_JPRB, 0.39689E+00_JPRB, 0.39912E+00_JPRB ,&
&  0.39695E+00_JPRB, 0.39610E+00_JPRB, 0.39743E+00_JPRB, 0.39468E+00_JPRB, 0.38615E+00_JPRB, 0.37872E+00_JPRB, 0.37263E+00_JPRB ,&
&  0.37410E+00_JPRB, 0.37656E+00_JPRB, 0.38699E+00_JPRB, 0.39378E+00_JPRB, 0.39524E+00_JPRB, 0.39321E+00_JPRB, 0.39325E+00_JPRB ,&
&  0.39523E+00_JPRB, 0.39345E+00_JPRB, 0.39052E+00_JPRB, 0.39827E+00_JPRB, 0.41143E+00_JPRB, 0.43555E+00_JPRB, 0.46738E+00_JPRB ,&
&  0.55343E+00_JPRB, 0.61624E+00_JPRB, 0.69472E+00_JPRB, 0.10071E+01_JPRB, 0.12039E+01_JPRB, 0.11340E+01_JPRB, 0.10307E+01_JPRB ,&
&  0.90437E+00_JPRB, 0.79508E+00_JPRB, 0.83268E+00_JPRB, 0.75282E+00_JPRB, 0.68534E+00_JPRB, 0.62710E+00_JPRB, 0.57749E+00_JPRB ,&
&  0.53759E+00_JPRB, 0.51409E+00_JPRB, 0.51663E+00_JPRB, 0.52614E+00_JPRB, 0.53105E+00_JPRB, 0.52902E+00_JPRB, 0.53296E+00_JPRB ,&
&  0.54592E+00_JPRB, 0.57679E+00_JPRB, 0.59693E+00_JPRB, 0.56385E+00_JPRB, 0.50932E+00_JPRB, 0.53300E+00_JPRB, 0.54753E+00_JPRB ,&
&  0.57462E+00_JPRB, 0.56373E+00_JPRB, 0.53845E+00_JPRB, 0.52087E+00_JPRB, 0.49987E+00_JPRB, 0.52486E+00_JPRB, 0.51746E+00_JPRB ,&
&  0.51132E+00_JPRB, 0.50149E+00_JPRB, 0.49142E+00_JPRB, 0.48656E+00_JPRB, 0.48242E+00_JPRB, 0.48096E+00_JPRB, 0.47873E+00_JPRB ,&
&  0.47890E+00_JPRB, 0.48026E+00_JPRB, 0.48777E+00_JPRB, 0.49034E+00_JPRB, 0.48852E+00_JPRB, 0.47878E+00_JPRB, 0.46401E+00_JPRB ,&
&  0.45372E+00_JPRB, 0.45602E+00_JPRB, 0.45089E+00_JPRB, 0.46066E+00_JPRB, 0.46881E+00_JPRB, 0.47116E+00_JPRB, 0.46236E+00_JPRB ,&
&  0.46434E+00_JPRB, 0.47754E+00_JPRB, 0.48274E+00_JPRB, 0.48565E+00_JPRB, 0.48905E+00_JPRB, 0.49196E+00_JPRB, 0.47404E+00_JPRB ,&
&  0.46870E+00_JPRB, 0.46418E+00_JPRB, 0.47607E+00_JPRB, 0.47158E+00_JPRB, 0.44517E+00_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.43568E+00_JPRB, 0.42878E+00_JPRB, 0.39180E+00_JPRB, 0.39218E+00_JPRB, 0.39104E+00_JPRB, 0.39404E+00_JPRB, 0.38792E+00_JPRB ,&
&  0.38383E+00_JPRB, 0.38553E+00_JPRB, 0.38854E+00_JPRB, 0.38305E+00_JPRB, 0.36492E+00_JPRB, 0.35693E+00_JPRB, 0.36101E+00_JPRB ,&
&  0.37119E+00_JPRB, 0.39002E+00_JPRB, 0.41269E+00_JPRB, 0.41531E+00_JPRB, 0.41894E+00_JPRB, 0.41741E+00_JPRB, 0.41386E+00_JPRB ,&
&  0.41497E+00_JPRB, 0.41810E+00_JPRB, 0.42353E+00_JPRB, 0.43773E+00_JPRB, 0.46149E+00_JPRB, 0.49503E+00_JPRB, 0.54312E+00_JPRB ,&
&  0.65237E+00_JPRB, 0.66866E+00_JPRB, 0.39234E+00_JPRB, 0.66531E+00_JPRB, 0.94819E+00_JPRB, 0.98843E+00_JPRB, 0.93254E+00_JPRB ,&
&  0.86235E+00_JPRB, 0.75824E+00_JPRB, 0.74845E+00_JPRB, 0.74578E+00_JPRB, 0.68005E+00_JPRB, 0.64057E+00_JPRB, 0.60292E+00_JPRB ,&
&  0.57010E+00_JPRB, 0.56134E+00_JPRB, 0.57011E+00_JPRB, 0.58266E+00_JPRB, 0.57879E+00_JPRB, 0.57771E+00_JPRB, 0.58937E+00_JPRB ,&
&  0.60929E+00_JPRB, 0.62841E+00_JPRB, 0.62750E+00_JPRB, 0.55399E+00_JPRB, 0.52589E+00_JPRB, 0.54321E+00_JPRB, 0.53984E+00_JPRB ,&
&  0.52808E+00_JPRB, 0.54386E+00_JPRB, 0.54163E+00_JPRB, 0.51859E+00_JPRB, 0.49688E+00_JPRB, 0.49206E+00_JPRB, 0.50792E+00_JPRB ,&
&  0.50133E+00_JPRB, 0.48755E+00_JPRB, 0.47842E+00_JPRB, 0.47466E+00_JPRB, 0.47125E+00_JPRB, 0.47133E+00_JPRB, 0.46879E+00_JPRB ,&
&  0.46908E+00_JPRB, 0.47468E+00_JPRB, 0.47883E+00_JPRB, 0.47612E+00_JPRB, 0.46757E+00_JPRB, 0.45525E+00_JPRB, 0.44534E+00_JPRB ,&
&  0.44927E+00_JPRB, 0.46275E+00_JPRB, 0.47999E+00_JPRB, 0.48246E+00_JPRB, 0.48152E+00_JPRB, 0.48599E+00_JPRB, 0.48501E+00_JPRB ,&
&  0.48745E+00_JPRB, 0.49607E+00_JPRB, 0.48971E+00_JPRB, 0.48488E+00_JPRB, 0.50159E+00_JPRB, 0.49552E+00_JPRB, 0.49716E+00_JPRB ,&
&  0.54048E+00_JPRB, 0.62686E+00_JPRB, 0.76480E+00_JPRB, 0.51352E+00_JPRB, 0.46392E+00_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.44625E+00_JPRB, 0.41592E+00_JPRB, 0.38210E+00_JPRB, 0.38013E+00_JPRB, 0.39417E+00_JPRB, 0.39560E+00_JPRB, 0.39059E+00_JPRB ,&
&  0.37995E+00_JPRB, 0.37070E+00_JPRB, 0.35970E+00_JPRB, 0.35275E+00_JPRB, 0.35575E+00_JPRB, 0.36310E+00_JPRB, 0.38892E+00_JPRB ,&
&  0.41480E+00_JPRB, 0.43689E+00_JPRB, 0.45489E+00_JPRB, 0.45904E+00_JPRB, 0.45349E+00_JPRB, 0.45526E+00_JPRB, 0.46137E+00_JPRB ,&
&  0.46282E+00_JPRB, 0.47455E+00_JPRB, 0.49042E+00_JPRB, 0.51351E+00_JPRB, 0.54569E+00_JPRB, 0.59236E+00_JPRB, 0.66293E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.55716E+00_JPRB, 0.43959E+00_JPRB, 0.78846E+00_JPRB, 0.93704E+00_JPRB, 0.95945E+00_JPRB, 0.81750E+00_JPRB ,&
&  0.77150E+00_JPRB, 0.72728E+00_JPRB, 0.71948E+00_JPRB, 0.74204E+00_JPRB, 0.67646E+00_JPRB, 0.65247E+00_JPRB, 0.62707E+00_JPRB ,&
&  0.61174E+00_JPRB, 0.61293E+00_JPRB, 0.62386E+00_JPRB, 0.63333E+00_JPRB, 0.63572E+00_JPRB, 0.64997E+00_JPRB, 0.66482E+00_JPRB ,&
&  0.67803E+00_JPRB, 0.67226E+00_JPRB, 0.65791E+00_JPRB, 0.56320E+00_JPRB, 0.54239E+00_JPRB, 0.54704E+00_JPRB, 0.54513E+00_JPRB ,&
&  0.52504E+00_JPRB, 0.52850E+00_JPRB, 0.52935E+00_JPRB, 0.51070E+00_JPRB, 0.49487E+00_JPRB, 0.48178E+00_JPRB, 0.49629E+00_JPRB ,&
&  0.48223E+00_JPRB, 0.46567E+00_JPRB, 0.45353E+00_JPRB, 0.44767E+00_JPRB, 0.44826E+00_JPRB, 0.45111E+00_JPRB, 0.44750E+00_JPRB ,&
&  0.45239E+00_JPRB, 0.45402E+00_JPRB, 0.45807E+00_JPRB, 0.45170E+00_JPRB, 0.44775E+00_JPRB, 0.44631E+00_JPRB, 0.43797E+00_JPRB ,&
&  0.45587E+00_JPRB, 0.48018E+00_JPRB, 0.48611E+00_JPRB, 0.49733E+00_JPRB, 0.49405E+00_JPRB, 0.48517E+00_JPRB, 0.49214E+00_JPRB ,&
&  0.49262E+00_JPRB, 0.50455E+00_JPRB, 0.50109E+00_JPRB, 0.50809E+00_JPRB, 0.50507E+00_JPRB, 0.51061E+00_JPRB, 0.53005E+00_JPRB ,&
&  0.58118E+00_JPRB, 0.64267E+00_JPRB, 0.64814E+00_JPRB, 0.50625E+00_JPRB, 0.46538E+00_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.42737E+00_JPRB, 0.40383E+00_JPRB, 0.36773E+00_JPRB, 0.35438E+00_JPRB, 0.37217E+00_JPRB, 0.38508E+00_JPRB, 0.37725E+00_JPRB ,&
&  0.37013E+00_JPRB, 0.36213E+00_JPRB, 0.35771E+00_JPRB, 0.36312E+00_JPRB, 0.38924E+00_JPRB, 0.42380E+00_JPRB, 0.44591E+00_JPRB ,&
&  0.46890E+00_JPRB, 0.48667E+00_JPRB, 0.50241E+00_JPRB, 0.50869E+00_JPRB, 0.50780E+00_JPRB, 0.51368E+00_JPRB, 0.51991E+00_JPRB ,&
&  0.53129E+00_JPRB, 0.55018E+00_JPRB, 0.57656E+00_JPRB, 0.61363E+00_JPRB, 0.65852E+00_JPRB, 0.71042E+00_JPRB, 0.75872E+00_JPRB ,&
&  0.57613E+00_JPRB, 0.35659E+00_JPRB, 0.74893E+00_JPRB, 0.87054E+00_JPRB, 0.82157E+00_JPRB, 0.80563E+00_JPRB, 0.74808E+00_JPRB ,&
&  0.70304E+00_JPRB, 0.70256E+00_JPRB, 0.67850E+00_JPRB, 0.70831E+00_JPRB, 0.65847E+00_JPRB, 0.65584E+00_JPRB, 0.65822E+00_JPRB ,&
&  0.66666E+00_JPRB, 0.67482E+00_JPRB, 0.68474E+00_JPRB, 0.69476E+00_JPRB, 0.70779E+00_JPRB, 0.72639E+00_JPRB, 0.73012E+00_JPRB ,&
&  0.73551E+00_JPRB, 0.72743E+00_JPRB, 0.70538E+00_JPRB, 0.58154E+00_JPRB, 0.55207E+00_JPRB, 0.54807E+00_JPRB, 0.53987E+00_JPRB ,&
&  0.52672E+00_JPRB, 0.50140E+00_JPRB, 0.48935E+00_JPRB, 0.50125E+00_JPRB, 0.49528E+00_JPRB, 0.47111E+00_JPRB, 0.48102E+00_JPRB ,&
&  0.45826E+00_JPRB, 0.43486E+00_JPRB, 0.42108E+00_JPRB, 0.41175E+00_JPRB, 0.41906E+00_JPRB, 0.42386E+00_JPRB, 0.42676E+00_JPRB ,&
&  0.42147E+00_JPRB, 0.42337E+00_JPRB, 0.43385E+00_JPRB, 0.43641E+00_JPRB, 0.44199E+00_JPRB, 0.45033E+00_JPRB, 0.45324E+00_JPRB ,&
&  0.47870E+00_JPRB, 0.48635E+00_JPRB, 0.49510E+00_JPRB, 0.50678E+00_JPRB, 0.51112E+00_JPRB, 0.49538E+00_JPRB, 0.48936E+00_JPRB ,&
&  0.49277E+00_JPRB, 0.49911E+00_JPRB, 0.50034E+00_JPRB, 0.51064E+00_JPRB, 0.51656E+00_JPRB, 0.52904E+00_JPRB, 0.55178E+00_JPRB ,&
&  0.58039E+00_JPRB, 0.59030E+00_JPRB, 0.56146E+00_JPRB, 0.48776E+00_JPRB, 0.45515E+00_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.41129E+00_JPRB, 0.38980E+00_JPRB, 0.35588E+00_JPRB, 0.32341E+00_JPRB, 0.34164E+00_JPRB, 0.33836E+00_JPRB, 0.34913E+00_JPRB ,&
&  0.35169E+00_JPRB, 0.37196E+00_JPRB, 0.39020E+00_JPRB, 0.41764E+00_JPRB, 0.44837E+00_JPRB, 0.47693E+00_JPRB, 0.49851E+00_JPRB ,&
&  0.51659E+00_JPRB, 0.53584E+00_JPRB, 0.55101E+00_JPRB, 0.55506E+00_JPRB, 0.56315E+00_JPRB, 0.57651E+00_JPRB, 0.58972E+00_JPRB ,&
&  0.60678E+00_JPRB, 0.63242E+00_JPRB, 0.66998E+00_JPRB, 0.72023E+00_JPRB, 0.77934E+00_JPRB, 0.80133E+00_JPRB, 0.80152E+00_JPRB ,&
&  0.50888E+00_JPRB, 0.73603E+00_JPRB, 0.83055E+00_JPRB, 0.78218E+00_JPRB, 0.75525E+00_JPRB, 0.73304E+00_JPRB, 0.71457E+00_JPRB ,&
&  0.68746E+00_JPRB, 0.64975E+00_JPRB, 0.63573E+00_JPRB, 0.68060E+00_JPRB, 0.67048E+00_JPRB, 0.69563E+00_JPRB, 0.71376E+00_JPRB ,&
&  0.72911E+00_JPRB, 0.74111E+00_JPRB, 0.75228E+00_JPRB, 0.76687E+00_JPRB, 0.77866E+00_JPRB, 0.78676E+00_JPRB, 0.79021E+00_JPRB ,&
&  0.79770E+00_JPRB, 0.79609E+00_JPRB, 0.77174E+00_JPRB, 0.63526E+00_JPRB, 0.59311E+00_JPRB, 0.56614E+00_JPRB, 0.54146E+00_JPRB ,&
&  0.50522E+00_JPRB, 0.48549E+00_JPRB, 0.48403E+00_JPRB, 0.50637E+00_JPRB, 0.49570E+00_JPRB, 0.47944E+00_JPRB, 0.46125E+00_JPRB ,&
&  0.44072E+00_JPRB, 0.41740E+00_JPRB, 0.40635E+00_JPRB, 0.40404E+00_JPRB, 0.41672E+00_JPRB, 0.41789E+00_JPRB, 0.40659E+00_JPRB ,&
&  0.39170E+00_JPRB, 0.39469E+00_JPRB, 0.41838E+00_JPRB, 0.43632E+00_JPRB, 0.46290E+00_JPRB, 0.47913E+00_JPRB, 0.48762E+00_JPRB ,&
&  0.49912E+00_JPRB, 0.49260E+00_JPRB, 0.50292E+00_JPRB, 0.51183E+00_JPRB, 0.50981E+00_JPRB, 0.50424E+00_JPRB, 0.49361E+00_JPRB ,&
&  0.48990E+00_JPRB, 0.50439E+00_JPRB, 0.51804E+00_JPRB, 0.53090E+00_JPRB, 0.53470E+00_JPRB, 0.53535E+00_JPRB, 0.54116E+00_JPRB ,&
&  0.53396E+00_JPRB, 0.51656E+00_JPRB, 0.49487E+00_JPRB, 0.46385E+00_JPRB, 0.43548E+00_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.40418E+00_JPRB, 0.39278E+00_JPRB, 0.37724E+00_JPRB, 0.36759E+00_JPRB, 0.35867E+00_JPRB, 0.34890E+00_JPRB, 0.35685E+00_JPRB ,&
&  0.37982E+00_JPRB, 0.39918E+00_JPRB, 0.43173E+00_JPRB, 0.47980E+00_JPRB, 0.51877E+00_JPRB, 0.53416E+00_JPRB, 0.55253E+00_JPRB ,&
&  0.56691E+00_JPRB, 0.58528E+00_JPRB, 0.59955E+00_JPRB, 0.60976E+00_JPRB, 0.62451E+00_JPRB, 0.64325E+00_JPRB, 0.66871E+00_JPRB ,&
&  0.70212E+00_JPRB, 0.73578E+00_JPRB, 0.78150E+00_JPRB, 0.84362E+00_JPRB, 0.90347E+00_JPRB, 0.91818E+00_JPRB, 0.92756E+00_JPRB ,&
&  0.72149E+00_JPRB, 0.83112E+00_JPRB, 0.81100E+00_JPRB, 0.77330E+00_JPRB, 0.73280E+00_JPRB, 0.69869E+00_JPRB, 0.68024E+00_JPRB ,&
&  0.66942E+00_JPRB, 0.63481E+00_JPRB, 0.63420E+00_JPRB, 0.66952E+00_JPRB, 0.72223E+00_JPRB, 0.74298E+00_JPRB, 0.76559E+00_JPRB ,&
&  0.78557E+00_JPRB, 0.80327E+00_JPRB, 0.81818E+00_JPRB, 0.83115E+00_JPRB, 0.84111E+00_JPRB, 0.86033E+00_JPRB, 0.86293E+00_JPRB ,&
&  0.87354E+00_JPRB, 0.87020E+00_JPRB, 0.86278E+00_JPRB, 0.73751E+00_JPRB, 0.65952E+00_JPRB, 0.61129E+00_JPRB, 0.55526E+00_JPRB ,&
&  0.51047E+00_JPRB, 0.48266E+00_JPRB, 0.49948E+00_JPRB, 0.52497E+00_JPRB, 0.51014E+00_JPRB, 0.48162E+00_JPRB, 0.45113E+00_JPRB ,&
&  0.44278E+00_JPRB, 0.41055E+00_JPRB, 0.40048E+00_JPRB, 0.40686E+00_JPRB, 0.42370E+00_JPRB, 0.40797E+00_JPRB, 0.38736E+00_JPRB ,&
&  0.37241E+00_JPRB, 0.36565E+00_JPRB, 0.39453E+00_JPRB, 0.42716E+00_JPRB, 0.47013E+00_JPRB, 0.48789E+00_JPRB, 0.47840E+00_JPRB ,&
&  0.51331E+00_JPRB, 0.48510E+00_JPRB, 0.48926E+00_JPRB, 0.49984E+00_JPRB, 0.50566E+00_JPRB, 0.51167E+00_JPRB, 0.51988E+00_JPRB ,&
&  0.55344E+00_JPRB, 0.61641E+00_JPRB, 0.64118E+00_JPRB, 0.59689E+00_JPRB, 0.56381E+00_JPRB, 0.52706E+00_JPRB, 0.48401E+00_JPRB ,&
&  0.48757E+00_JPRB, 0.47711E+00_JPRB, 0.46616E+00_JPRB, 0.45037E+00_JPRB, 0.42804E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.45725E+00_JPRB, 0.44548E+00_JPRB, 0.43815E+00_JPRB, 0.43165E+00_JPRB, 0.41642E+00_JPRB, 0.41614E+00_JPRB, 0.41245E+00_JPRB ,&
&  0.42734E+00_JPRB, 0.44537E+00_JPRB, 0.47719E+00_JPRB, 0.51385E+00_JPRB, 0.55957E+00_JPRB, 0.58664E+00_JPRB, 0.61619E+00_JPRB ,&
&  0.62857E+00_JPRB, 0.64718E+00_JPRB, 0.67334E+00_JPRB, 0.68980E+00_JPRB, 0.71172E+00_JPRB, 0.71996E+00_JPRB, 0.74780E+00_JPRB ,&
&  0.77075E+00_JPRB, 0.79767E+00_JPRB, 0.84092E+00_JPRB, 0.87886E+00_JPRB, 0.91299E+00_JPRB, 0.90119E+00_JPRB, 0.71129E+00_JPRB ,&
&  0.85283E+00_JPRB, 0.84165E+00_JPRB, 0.81521E+00_JPRB, 0.76522E+00_JPRB, 0.71167E+00_JPRB, 0.66741E+00_JPRB, 0.65279E+00_JPRB ,&
&  0.64652E+00_JPRB, 0.64046E+00_JPRB, 0.64779E+00_JPRB, 0.66562E+00_JPRB, 0.74709E+00_JPRB, 0.78438E+00_JPRB, 0.80500E+00_JPRB ,&
&  0.82803E+00_JPRB, 0.84935E+00_JPRB, 0.86843E+00_JPRB, 0.88960E+00_JPRB, 0.90807E+00_JPRB, 0.93318E+00_JPRB, 0.96639E+00_JPRB ,&
&  0.97666E+00_JPRB, 0.97864E+00_JPRB, 0.94617E+00_JPRB, 0.79487E+00_JPRB, 0.73886E+00_JPRB, 0.68804E+00_JPRB, 0.61739E+00_JPRB ,&
&  0.54313E+00_JPRB, 0.51941E+00_JPRB, 0.54002E+00_JPRB, 0.56543E+00_JPRB, 0.53708E+00_JPRB, 0.49812E+00_JPRB, 0.47522E+00_JPRB ,&
&  0.44324E+00_JPRB, 0.43992E+00_JPRB, 0.42948E+00_JPRB, 0.44297E+00_JPRB, 0.44797E+00_JPRB, 0.43019E+00_JPRB, 0.41868E+00_JPRB ,&
&  0.40038E+00_JPRB, 0.38560E+00_JPRB, 0.40090E+00_JPRB, 0.43004E+00_JPRB, 0.49738E+00_JPRB, 0.55664E+00_JPRB, 0.56111E+00_JPRB ,&
&  0.60789E+00_JPRB, 0.42125E+00_JPRB, 0.45341E+00_JPRB, 0.48230E+00_JPRB, 0.52775E+00_JPRB, 0.57114E+00_JPRB, 0.65937E+00_JPRB ,&
&  0.84981E+00_JPRB, 0.11903E+01_JPRB, 0.10628E+01_JPRB, 0.80763E+00_JPRB, 0.89939E+00_JPRB, 0.82450E+00_JPRB, 0.43814E+00_JPRB ,&
&  0.44014E+00_JPRB, 0.44186E+00_JPRB, 0.45696E+00_JPRB, 0.46121E+00_JPRB, 0.46412E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.52832E+00_JPRB, 0.52260E+00_JPRB, 0.52164E+00_JPRB, 0.51227E+00_JPRB, 0.50688E+00_JPRB, 0.50723E+00_JPRB, 0.49450E+00_JPRB ,&
&  0.50375E+00_JPRB, 0.51890E+00_JPRB, 0.52409E+00_JPRB, 0.53898E+00_JPRB, 0.54668E+00_JPRB, 0.57532E+00_JPRB, 0.61677E+00_JPRB ,&
&  0.64025E+00_JPRB, 0.69888E+00_JPRB, 0.72012E+00_JPRB, 0.74845E+00_JPRB, 0.76952E+00_JPRB, 0.79037E+00_JPRB, 0.80689E+00_JPRB ,&
&  0.83496E+00_JPRB, 0.86624E+00_JPRB, 0.90376E+00_JPRB, 0.92754E+00_JPRB, 0.91595E+00_JPRB, 0.96550E+00_JPRB, 0.75987E+00_JPRB ,&
&  0.87611E+00_JPRB, 0.85726E+00_JPRB, 0.77776E+00_JPRB, 0.73015E+00_JPRB, 0.68561E+00_JPRB, 0.65454E+00_JPRB, 0.63260E+00_JPRB ,&
&  0.62635E+00_JPRB, 0.62265E+00_JPRB, 0.63013E+00_JPRB, 0.67256E+00_JPRB, 0.72950E+00_JPRB, 0.77646E+00_JPRB, 0.80400E+00_JPRB ,&
&  0.83108E+00_JPRB, 0.84619E+00_JPRB, 0.87686E+00_JPRB, 0.91228E+00_JPRB, 0.95269E+00_JPRB, 0.97908E+00_JPRB, 0.10090E+01_JPRB ,&
&  0.10146E+01_JPRB, 0.10209E+01_JPRB, 0.96002E+00_JPRB, 0.88474E+00_JPRB, 0.86892E+00_JPRB, 0.83173E+00_JPRB, 0.72269E+00_JPRB ,&
&  0.58869E+00_JPRB, 0.55461E+00_JPRB, 0.56753E+00_JPRB, 0.61246E+00_JPRB, 0.57442E+00_JPRB, 0.54207E+00_JPRB, 0.52716E+00_JPRB ,&
&  0.50412E+00_JPRB, 0.46705E+00_JPRB, 0.47771E+00_JPRB, 0.50537E+00_JPRB, 0.51187E+00_JPRB, 0.50204E+00_JPRB, 0.48324E+00_JPRB ,&
&  0.46319E+00_JPRB, 0.43495E+00_JPRB, 0.42848E+00_JPRB, 0.42764E+00_JPRB, 0.43394E+00_JPRB, 0.45296E+00_JPRB, 0.40372E+00_JPRB ,&
&  0.45521E+00_JPRB, 0.42096E+00_JPRB, 0.44618E+00_JPRB, 0.49612E+00_JPRB, 0.58981E+00_JPRB, 0.76139E+00_JPRB, 0.10867E+01_JPRB ,&
&  0.16699E+01_JPRB, 0.19339E+01_JPRB, 0.21938E+01_JPRB, 0.27502E+01_JPRB, 0.23988E+01_JPRB, 0.17588E+01_JPRB, 0.49381E+00_JPRB ,&
&  0.49025E+00_JPRB, 0.49387E+00_JPRB, 0.51331E+00_JPRB, 0.52127E+00_JPRB, 0.52569E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.56433E+00_JPRB, 0.57205E+00_JPRB, 0.58283E+00_JPRB, 0.59417E+00_JPRB, 0.60808E+00_JPRB, 0.62197E+00_JPRB, 0.63269E+00_JPRB ,&
&  0.64376E+00_JPRB, 0.65770E+00_JPRB, 0.67103E+00_JPRB, 0.67978E+00_JPRB, 0.68830E+00_JPRB, 0.70563E+00_JPRB, 0.72599E+00_JPRB ,&
&  0.73143E+00_JPRB, 0.76408E+00_JPRB, 0.77320E+00_JPRB, 0.80006E+00_JPRB, 0.82223E+00_JPRB, 0.85112E+00_JPRB, 0.86931E+00_JPRB ,&
&  0.88301E+00_JPRB, 0.90732E+00_JPRB, 0.90655E+00_JPRB, 0.89059E+00_JPRB, 0.88472E+00_JPRB, 0.90507E+00_JPRB, 0.70163E+00_JPRB ,&
&  0.93080E+00_JPRB, 0.89878E+00_JPRB, 0.78121E+00_JPRB, 0.71937E+00_JPRB, 0.67135E+00_JPRB, 0.64275E+00_JPRB, 0.62467E+00_JPRB ,&
&  0.61308E+00_JPRB, 0.61998E+00_JPRB, 0.59655E+00_JPRB, 0.59987E+00_JPRB, 0.65845E+00_JPRB, 0.73293E+00_JPRB, 0.78476E+00_JPRB ,&
&  0.78479E+00_JPRB, 0.81358E+00_JPRB, 0.82014E+00_JPRB, 0.83117E+00_JPRB, 0.86128E+00_JPRB, 0.88765E+00_JPRB, 0.90803E+00_JPRB ,&
&  0.91091E+00_JPRB, 0.91441E+00_JPRB, 0.87835E+00_JPRB, 0.92518E+00_JPRB, 0.96704E+00_JPRB, 0.92397E+00_JPRB, 0.78401E+00_JPRB ,&
&  0.58158E+00_JPRB, 0.59590E+00_JPRB, 0.63513E+00_JPRB, 0.69683E+00_JPRB, 0.67127E+00_JPRB, 0.64366E+00_JPRB, 0.58477E+00_JPRB ,&
&  0.55944E+00_JPRB, 0.54625E+00_JPRB, 0.56259E+00_JPRB, 0.58515E+00_JPRB, 0.60534E+00_JPRB, 0.59701E+00_JPRB, 0.56221E+00_JPRB ,&
&  0.52891E+00_JPRB, 0.48614E+00_JPRB, 0.45912E+00_JPRB, 0.44378E+00_JPRB, 0.41761E+00_JPRB, 0.35785E+00_JPRB, 0.39097E+00_JPRB ,&
&  0.38878E+00_JPRB, 0.37398E+00_JPRB, 0.43210E+00_JPRB, 0.37209E+00_JPRB, 0.55884E+00_JPRB, 0.72449E+00_JPRB, 0.98810E+00_JPRB ,&
&  0.11969E+01_JPRB, 0.17057E+01_JPRB, 0.21630E+01_JPRB, 0.16367E+01_JPRB, 0.11151E+01_JPRB, 0.61766E+00_JPRB, 0.52753E+00_JPRB ,&
&  0.53381E+00_JPRB, 0.53844E+00_JPRB, 0.54622E+00_JPRB, 0.54550E+00_JPRB, 0.55561E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.57973E+00_JPRB, 0.59064E+00_JPRB, 0.60585E+00_JPRB, 0.62644E+00_JPRB, 0.64581E+00_JPRB, 0.66007E+00_JPRB, 0.66885E+00_JPRB ,&
&  0.68495E+00_JPRB, 0.70677E+00_JPRB, 0.71884E+00_JPRB, 0.72465E+00_JPRB, 0.73789E+00_JPRB, 0.75034E+00_JPRB, 0.77345E+00_JPRB ,&
&  0.78720E+00_JPRB, 0.81968E+00_JPRB, 0.84295E+00_JPRB, 0.86670E+00_JPRB, 0.88455E+00_JPRB, 0.90699E+00_JPRB, 0.92789E+00_JPRB ,&
&  0.94465E+00_JPRB, 0.98027E+00_JPRB, 0.10134E+01_JPRB, 0.10403E+01_JPRB, 0.10779E+01_JPRB, 0.10869E+01_JPRB, 0.86229E+00_JPRB ,&
&  0.10416E+01_JPRB, 0.98534E+00_JPRB, 0.85137E+00_JPRB, 0.74928E+00_JPRB, 0.68478E+00_JPRB, 0.63302E+00_JPRB, 0.60322E+00_JPRB ,&
&  0.59643E+00_JPRB, 0.59452E+00_JPRB, 0.59215E+00_JPRB, 0.57239E+00_JPRB, 0.55298E+00_JPRB, 0.57550E+00_JPRB, 0.65107E+00_JPRB ,&
&  0.69575E+00_JPRB, 0.71856E+00_JPRB, 0.73955E+00_JPRB, 0.73166E+00_JPRB, 0.74964E+00_JPRB, 0.78224E+00_JPRB, 0.81204E+00_JPRB ,&
&  0.81059E+00_JPRB, 0.78868E+00_JPRB, 0.74745E+00_JPRB, 0.88457E+00_JPRB, 0.93407E+00_JPRB, 0.95754E+00_JPRB, 0.87190E+00_JPRB ,&
&  0.72670E+00_JPRB, 0.65710E+00_JPRB, 0.68077E+00_JPRB, 0.76462E+00_JPRB, 0.75290E+00_JPRB, 0.73846E+00_JPRB, 0.69441E+00_JPRB ,&
&  0.67043E+00_JPRB, 0.64974E+00_JPRB, 0.65293E+00_JPRB, 0.66992E+00_JPRB, 0.68047E+00_JPRB, 0.67558E+00_JPRB, 0.62654E+00_JPRB ,&
&  0.58248E+00_JPRB, 0.53745E+00_JPRB, 0.50520E+00_JPRB, 0.47407E+00_JPRB, 0.43744E+00_JPRB, 0.36418E+00_JPRB, 0.39537E+00_JPRB ,&
&  0.37078E+00_JPRB, 0.31142E+00_JPRB, 0.35963E+00_JPRB, 0.44419E+00_JPRB, 0.48854E+00_JPRB, 0.57324E+00_JPRB, 0.73543E+00_JPRB ,&
&  0.94501E+00_JPRB, 0.10889E+01_JPRB, 0.10648E+01_JPRB, 0.80116E+00_JPRB, 0.61778E+00_JPRB, 0.55873E+00_JPRB, 0.55705E+00_JPRB ,&
&  0.56417E+00_JPRB, 0.56217E+00_JPRB, 0.56429E+00_JPRB, 0.56800E+00_JPRB, 0.57494E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.56375E+00_JPRB, 0.56186E+00_JPRB, 0.56748E+00_JPRB, 0.57979E+00_JPRB, 0.58452E+00_JPRB, 0.58702E+00_JPRB, 0.58758E+00_JPRB ,&
&  0.60183E+00_JPRB, 0.61840E+00_JPRB, 0.63370E+00_JPRB, 0.64919E+00_JPRB, 0.66147E+00_JPRB, 0.67272E+00_JPRB, 0.70334E+00_JPRB ,&
&  0.71739E+00_JPRB, 0.73818E+00_JPRB, 0.75211E+00_JPRB, 0.76603E+00_JPRB, 0.77748E+00_JPRB, 0.77022E+00_JPRB, 0.77196E+00_JPRB ,&
&  0.77887E+00_JPRB, 0.80138E+00_JPRB, 0.82646E+00_JPRB, 0.86810E+00_JPRB, 0.92286E+00_JPRB, 0.93161E+00_JPRB, 0.10318E+01_JPRB ,&
&  0.96604E+00_JPRB, 0.11003E+01_JPRB, 0.94790E+00_JPRB, 0.80100E+00_JPRB, 0.76323E+00_JPRB, 0.68676E+00_JPRB, 0.64654E+00_JPRB ,&
&  0.64429E+00_JPRB, 0.61794E+00_JPRB, 0.58995E+00_JPRB, 0.55693E+00_JPRB, 0.52099E+00_JPRB, 0.51991E+00_JPRB, 0.52790E+00_JPRB ,&
&  0.58085E+00_JPRB, 0.61073E+00_JPRB, 0.67058E+00_JPRB, 0.73232E+00_JPRB, 0.80720E+00_JPRB, 0.83393E+00_JPRB, 0.85527E+00_JPRB ,&
&  0.85823E+00_JPRB, 0.84716E+00_JPRB, 0.80885E+00_JPRB, 0.91696E+00_JPRB, 0.10014E+01_JPRB, 0.10530E+01_JPRB, 0.10497E+01_JPRB ,&
&  0.97603E+00_JPRB, 0.86492E+00_JPRB, 0.84797E+00_JPRB, 0.83965E+00_JPRB, 0.82203E+00_JPRB, 0.82841E+00_JPRB, 0.79415E+00_JPRB ,&
&  0.76672E+00_JPRB, 0.73442E+00_JPRB, 0.71689E+00_JPRB, 0.72000E+00_JPRB, 0.71601E+00_JPRB, 0.69993E+00_JPRB, 0.63311E+00_JPRB ,&
&  0.59121E+00_JPRB, 0.55544E+00_JPRB, 0.53190E+00_JPRB, 0.50332E+00_JPRB, 0.41929E+00_JPRB, 0.45526E+00_JPRB, 0.42953E+00_JPRB ,&
&  0.39341E+00_JPRB, 0.37338E+00_JPRB, 0.35112E+00_JPRB, 0.45531E+00_JPRB, 0.49954E+00_JPRB, 0.55137E+00_JPRB, 0.61821E+00_JPRB ,&
&  0.63281E+00_JPRB, 0.62079E+00_JPRB, 0.55899E+00_JPRB, 0.49883E+00_JPRB, 0.51146E+00_JPRB, 0.51225E+00_JPRB, 0.51508E+00_JPRB ,&
&  0.52884E+00_JPRB, 0.53739E+00_JPRB, 0.54779E+00_JPRB, 0.56636E+00_JPRB, 0.56064E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.55556E+00_JPRB, 0.54423E+00_JPRB, 0.53295E+00_JPRB, 0.52172E+00_JPRB, 0.52650E+00_JPRB, 0.52588E+00_JPRB, 0.53308E+00_JPRB ,&
&  0.54130E+00_JPRB, 0.53825E+00_JPRB, 0.54008E+00_JPRB, 0.54005E+00_JPRB, 0.55081E+00_JPRB, 0.56037E+00_JPRB, 0.58706E+00_JPRB ,&
&  0.61160E+00_JPRB, 0.62356E+00_JPRB, 0.64113E+00_JPRB, 0.65765E+00_JPRB, 0.67019E+00_JPRB, 0.68264E+00_JPRB, 0.66586E+00_JPRB ,&
&  0.64727E+00_JPRB, 0.64701E+00_JPRB, 0.67974E+00_JPRB, 0.78844E+00_JPRB, 0.99815E+00_JPRB, 0.12746E+01_JPRB, 0.18161E+01_JPRB ,&
&  0.16758E+01_JPRB, 0.10938E+01_JPRB, 0.97980E+00_JPRB, 0.85773E+00_JPRB, 0.88921E+00_JPRB, 0.95511E+00_JPRB, 0.94792E+00_JPRB ,&
&  0.93425E+00_JPRB, 0.92136E+00_JPRB, 0.90333E+00_JPRB, 0.87823E+00_JPRB, 0.82743E+00_JPRB, 0.76095E+00_JPRB, 0.70771E+00_JPRB ,&
&  0.67872E+00_JPRB, 0.64660E+00_JPRB, 0.71794E+00_JPRB, 0.91300E+00_JPRB, 0.10427E+01_JPRB, 0.12711E+01_JPRB, 0.12333E+01_JPRB ,&
&  0.12452E+01_JPRB, 0.11970E+01_JPRB, 0.11657E+01_JPRB, 0.12598E+01_JPRB, 0.12878E+01_JPRB, 0.13360E+01_JPRB, 0.13279E+01_JPRB ,&
&  0.13594E+01_JPRB, 0.11717E+01_JPRB, 0.83539E+00_JPRB, 0.91035E+00_JPRB, 0.89498E+00_JPRB, 0.92418E+00_JPRB, 0.88530E+00_JPRB ,&
&  0.84993E+00_JPRB, 0.80010E+00_JPRB, 0.77118E+00_JPRB, 0.75338E+00_JPRB, 0.72847E+00_JPRB, 0.68773E+00_JPRB, 0.61111E+00_JPRB ,&
&  0.57624E+00_JPRB, 0.54996E+00_JPRB, 0.53365E+00_JPRB, 0.50286E+00_JPRB, 0.47549E+00_JPRB, 0.40530E+00_JPRB, 0.42385E+00_JPRB ,&
&  0.42307E+00_JPRB, 0.44821E+00_JPRB, 0.40175E+00_JPRB, 0.48142E+00_JPRB, 0.52345E+00_JPRB, 0.51281E+00_JPRB, 0.51694E+00_JPRB ,&
&  0.49558E+00_JPRB, 0.47729E+00_JPRB, 0.46352E+00_JPRB, 0.46638E+00_JPRB, 0.48052E+00_JPRB, 0.50264E+00_JPRB, 0.50982E+00_JPRB ,&
&  0.52065E+00_JPRB, 0.53584E+00_JPRB, 0.55508E+00_JPRB, 0.57582E+00_JPRB, 0.57350E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.58012E+00_JPRB, 0.58049E+00_JPRB, 0.58106E+00_JPRB, 0.58732E+00_JPRB, 0.58711E+00_JPRB, 0.58184E+00_JPRB, 0.56888E+00_JPRB ,&
&  0.56053E+00_JPRB, 0.54500E+00_JPRB, 0.53434E+00_JPRB, 0.52651E+00_JPRB, 0.53584E+00_JPRB, 0.56235E+00_JPRB, 0.60529E+00_JPRB ,&
&  0.61776E+00_JPRB, 0.61602E+00_JPRB, 0.59481E+00_JPRB, 0.58432E+00_JPRB, 0.60738E+00_JPRB, 0.61277E+00_JPRB, 0.59053E+00_JPRB ,&
&  0.57012E+00_JPRB, 0.62140E+00_JPRB, 0.76055E+00_JPRB, 0.94785E+00_JPRB, 0.12751E+01_JPRB, 0.14039E+01_JPRB, 0.17337E+01_JPRB ,&
&  0.15044E+01_JPRB, 0.11522E+01_JPRB, 0.12463E+01_JPRB, 0.11999E+01_JPRB, 0.12733E+01_JPRB, 0.13127E+01_JPRB, 0.13246E+01_JPRB ,&
&  0.13561E+01_JPRB, 0.13765E+01_JPRB, 0.13946E+01_JPRB, 0.14213E+01_JPRB, 0.14311E+01_JPRB, 0.14108E+01_JPRB, 0.13891E+01_JPRB ,&
&  0.13689E+01_JPRB, 0.13273E+01_JPRB, 0.13306E+01_JPRB, 0.13719E+01_JPRB, 0.14851E+01_JPRB, 0.16053E+01_JPRB, 0.16062E+01_JPRB ,&
&  0.15960E+01_JPRB, 0.16351E+01_JPRB, 0.15636E+01_JPRB, 0.16581E+01_JPRB, 0.16812E+01_JPRB, 0.16210E+01_JPRB, 0.17257E+01_JPRB ,&
&  0.17726E+01_JPRB, 0.14798E+01_JPRB, 0.93188E+00_JPRB, 0.95332E+00_JPRB, 0.91569E+00_JPRB, 0.90514E+00_JPRB, 0.94247E+00_JPRB ,&
&  0.89152E+00_JPRB, 0.85426E+00_JPRB, 0.82754E+00_JPRB, 0.80835E+00_JPRB, 0.75707E+00_JPRB, 0.69049E+00_JPRB, 0.64319E+00_JPRB ,&
&  0.59127E+00_JPRB, 0.55422E+00_JPRB, 0.52883E+00_JPRB, 0.49538E+00_JPRB, 0.45255E+00_JPRB, 0.45647E+00_JPRB, 0.47020E+00_JPRB ,&
&  0.48959E+00_JPRB, 0.48780E+00_JPRB, 0.50439E+00_JPRB, 0.50066E+00_JPRB, 0.48921E+00_JPRB, 0.54721E+00_JPRB, 0.53343E+00_JPRB ,&
&  0.52968E+00_JPRB, 0.52391E+00_JPRB, 0.52172E+00_JPRB, 0.51682E+00_JPRB, 0.53708E+00_JPRB, 0.56713E+00_JPRB, 0.57431E+00_JPRB ,&
&  0.58827E+00_JPRB, 0.57829E+00_JPRB, 0.58497E+00_JPRB, 0.59396E+00_JPRB, 0.59888E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.67674E+00_JPRB, 0.67616E+00_JPRB, 0.66413E+00_JPRB, 0.67259E+00_JPRB, 0.68330E+00_JPRB, 0.67820E+00_JPRB, 0.69029E+00_JPRB ,&
&  0.70807E+00_JPRB, 0.72469E+00_JPRB, 0.73360E+00_JPRB, 0.73845E+00_JPRB, 0.77948E+00_JPRB, 0.83180E+00_JPRB, 0.89365E+00_JPRB ,&
&  0.92941E+00_JPRB, 0.96461E+00_JPRB, 0.97592E+00_JPRB, 0.96835E+00_JPRB, 0.96648E+00_JPRB, 0.93976E+00_JPRB, 0.90189E+00_JPRB ,&
&  0.95938E+00_JPRB, 0.12045E+01_JPRB, 0.15932E+01_JPRB, 0.19164E+01_JPRB, 0.21744E+01_JPRB, 0.17492E+01_JPRB, 0.17034E+01_JPRB ,&
&  0.14642E+01_JPRB, 0.12327E+01_JPRB, 0.12887E+01_JPRB, 0.14003E+01_JPRB, 0.14851E+01_JPRB, 0.14880E+01_JPRB, 0.15215E+01_JPRB ,&
&  0.15381E+01_JPRB, 0.15510E+01_JPRB, 0.15725E+01_JPRB, 0.16321E+01_JPRB, 0.16794E+01_JPRB, 0.17135E+01_JPRB, 0.17617E+01_JPRB ,&
&  0.17964E+01_JPRB, 0.17917E+01_JPRB, 0.17775E+01_JPRB, 0.16277E+01_JPRB, 0.16576E+01_JPRB, 0.17333E+01_JPRB, 0.17970E+01_JPRB ,&
&  0.18104E+01_JPRB, 0.17746E+01_JPRB, 0.18678E+01_JPRB, 0.18911E+01_JPRB, 0.18813E+01_JPRB, 0.18225E+01_JPRB, 0.18190E+01_JPRB ,&
&  0.18152E+01_JPRB, 0.16295E+01_JPRB, 0.10597E+01_JPRB, 0.10745E+01_JPRB, 0.10142E+01_JPRB, 0.88936E+00_JPRB, 0.94857E+00_JPRB ,&
&  0.91875E+00_JPRB, 0.85690E+00_JPRB, 0.82971E+00_JPRB, 0.81567E+00_JPRB, 0.78623E+00_JPRB, 0.75975E+00_JPRB, 0.70308E+00_JPRB ,&
&  0.65976E+00_JPRB, 0.60176E+00_JPRB, 0.52775E+00_JPRB, 0.49704E+00_JPRB, 0.47923E+00_JPRB, 0.53001E+00_JPRB, 0.60828E+00_JPRB ,&
&  0.62020E+00_JPRB, 0.58751E+00_JPRB, 0.60991E+00_JPRB, 0.60875E+00_JPRB, 0.57363E+00_JPRB, 0.60524E+00_JPRB, 0.60325E+00_JPRB ,&
&  0.60857E+00_JPRB, 0.61652E+00_JPRB, 0.62219E+00_JPRB, 0.62337E+00_JPRB, 0.63303E+00_JPRB, 0.64997E+00_JPRB, 0.66620E+00_JPRB ,&
&  0.68094E+00_JPRB, 0.67810E+00_JPRB, 0.67665E+00_JPRB, 0.67503E+00_JPRB, 0.67176E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.71721E+00_JPRB, 0.69413E+00_JPRB, 0.67470E+00_JPRB, 0.66458E+00_JPRB, 0.66235E+00_JPRB, 0.67137E+00_JPRB, 0.68843E+00_JPRB ,&
&  0.71787E+00_JPRB, 0.76075E+00_JPRB, 0.80460E+00_JPRB, 0.84130E+00_JPRB, 0.89828E+00_JPRB, 0.97473E+00_JPRB, 0.10735E+01_JPRB ,&
&  0.11812E+01_JPRB, 0.12976E+01_JPRB, 0.13620E+01_JPRB, 0.13956E+01_JPRB, 0.14618E+01_JPRB, 0.15690E+01_JPRB, 0.17984E+01_JPRB ,&
&  0.21068E+01_JPRB, 0.23490E+01_JPRB, 0.24548E+01_JPRB, 0.21748E+01_JPRB, 0.16075E+01_JPRB, 0.14741E+01_JPRB, 0.15638E+01_JPRB ,&
&  0.15897E+01_JPRB, 0.14963E+01_JPRB, 0.14391E+01_JPRB, 0.14348E+01_JPRB, 0.14436E+01_JPRB, 0.14866E+01_JPRB, 0.15114E+01_JPRB ,&
&  0.15231E+01_JPRB, 0.15412E+01_JPRB, 0.15902E+01_JPRB, 0.16395E+01_JPRB, 0.17231E+01_JPRB, 0.18132E+01_JPRB, 0.19185E+01_JPRB ,&
&  0.19483E+01_JPRB, 0.19918E+01_JPRB, 0.19595E+01_JPRB, 0.18778E+01_JPRB, 0.18427E+01_JPRB, 0.18630E+01_JPRB, 0.19067E+01_JPRB ,&
&  0.19420E+01_JPRB, 0.18561E+01_JPRB, 0.18380E+01_JPRB, 0.18321E+01_JPRB, 0.17414E+01_JPRB, 0.15335E+01_JPRB, 0.14528E+01_JPRB ,&
&  0.14675E+01_JPRB, 0.15954E+01_JPRB, 0.10340E+01_JPRB, 0.11528E+01_JPRB, 0.12462E+01_JPRB, 0.11395E+01_JPRB, 0.10101E+01_JPRB ,&
&  0.99185E+00_JPRB, 0.92045E+00_JPRB, 0.89896E+00_JPRB, 0.91644E+00_JPRB, 0.88429E+00_JPRB, 0.87104E+00_JPRB, 0.74447E+00_JPRB ,&
&  0.75567E+00_JPRB, 0.67369E+00_JPRB, 0.56356E+00_JPRB, 0.48237E+00_JPRB, 0.50461E+00_JPRB, 0.57295E+00_JPRB, 0.64490E+00_JPRB ,&
&  0.63981E+00_JPRB, 0.75204E+00_JPRB, 0.88309E+00_JPRB, 0.10919E+01_JPRB, 0.12560E+01_JPRB, 0.61656E+00_JPRB, 0.62611E+00_JPRB ,&
&  0.63782E+00_JPRB, 0.66604E+00_JPRB, 0.67452E+00_JPRB, 0.67978E+00_JPRB, 0.68663E+00_JPRB, 0.70682E+00_JPRB, 0.73884E+00_JPRB ,&
&  0.75603E+00_JPRB, 0.76361E+00_JPRB, 0.77144E+00_JPRB, 0.76086E+00_JPRB, 0.73624E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.74860E+00_JPRB, 0.74608E+00_JPRB, 0.74115E+00_JPRB, 0.72625E+00_JPRB, 0.70022E+00_JPRB, 0.68209E+00_JPRB, 0.68357E+00_JPRB ,&
&  0.68645E+00_JPRB, 0.71865E+00_JPRB, 0.77585E+00_JPRB, 0.84704E+00_JPRB, 0.94138E+00_JPRB, 0.10480E+01_JPRB, 0.11606E+01_JPRB ,&
&  0.13092E+01_JPRB, 0.14615E+01_JPRB, 0.15651E+01_JPRB, 0.16532E+01_JPRB, 0.17559E+01_JPRB, 0.19279E+01_JPRB, 0.21294E+01_JPRB ,&
&  0.22804E+01_JPRB, 0.22824E+01_JPRB, 0.21415E+01_JPRB, 0.15336E+01_JPRB, 0.12232E+01_JPRB, 0.13986E+01_JPRB, 0.14241E+01_JPRB ,&
&  0.14264E+01_JPRB, 0.14116E+01_JPRB, 0.14078E+01_JPRB, 0.13894E+01_JPRB, 0.13994E+01_JPRB, 0.14397E+01_JPRB, 0.14684E+01_JPRB ,&
&  0.14732E+01_JPRB, 0.14840E+01_JPRB, 0.15243E+01_JPRB, 0.16084E+01_JPRB, 0.17293E+01_JPRB, 0.17813E+01_JPRB, 0.18495E+01_JPRB ,&
&  0.19086E+01_JPRB, 0.20105E+01_JPRB, 0.20500E+01_JPRB, 0.20405E+01_JPRB, 0.19689E+01_JPRB, 0.19440E+01_JPRB, 0.19454E+01_JPRB ,&
&  0.19061E+01_JPRB, 0.18160E+01_JPRB, 0.17809E+01_JPRB, 0.17634E+01_JPRB, 0.17069E+01_JPRB, 0.14372E+01_JPRB, 0.13143E+01_JPRB ,&
&  0.12750E+01_JPRB, 0.12451E+01_JPRB, 0.10591E+01_JPRB, 0.12583E+01_JPRB, 0.88318E+00_JPRB, 0.10177E+01_JPRB, 0.10778E+01_JPRB ,&
&  0.10570E+01_JPRB, 0.10438E+01_JPRB, 0.10536E+01_JPRB, 0.10469E+01_JPRB, 0.10231E+01_JPRB, 0.84311E+00_JPRB, 0.79329E+00_JPRB ,&
&  0.79648E+00_JPRB, 0.68374E+00_JPRB, 0.59202E+00_JPRB, 0.46897E+00_JPRB, 0.51311E+00_JPRB, 0.57698E+00_JPRB, 0.61699E+00_JPRB ,&
&  0.58868E+00_JPRB, 0.73895E+00_JPRB, 0.89947E+00_JPRB, 0.98579E+00_JPRB, 0.10266E+01_JPRB, 0.61543E+00_JPRB, 0.61500E+00_JPRB ,&
&  0.63782E+00_JPRB, 0.65570E+00_JPRB, 0.67240E+00_JPRB, 0.67118E+00_JPRB, 0.68791E+00_JPRB, 0.71950E+00_JPRB, 0.74876E+00_JPRB ,&
&  0.76112E+00_JPRB, 0.75718E+00_JPRB, 0.76150E+00_JPRB, 0.75615E+00_JPRB, 0.75129E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.74746E+00_JPRB, 0.75663E+00_JPRB, 0.78410E+00_JPRB, 0.80035E+00_JPRB, 0.80873E+00_JPRB, 0.76483E+00_JPRB, 0.73045E+00_JPRB ,&
&  0.68471E+00_JPRB, 0.63603E+00_JPRB, 0.69104E+00_JPRB, 0.75804E+00_JPRB, 0.83711E+00_JPRB, 0.92178E+00_JPRB, 0.10100E+01_JPRB ,&
&  0.11279E+01_JPRB, 0.12753E+01_JPRB, 0.14555E+01_JPRB, 0.15889E+01_JPRB, 0.17032E+01_JPRB, 0.18491E+01_JPRB, 0.19840E+01_JPRB ,&
&  0.16378E+01_JPRB, 0.11901E+01_JPRB, 0.15403E+01_JPRB, 0.16702E+01_JPRB, 0.13758E+01_JPRB, 0.12910E+01_JPRB, 0.12905E+01_JPRB ,&
&  0.13205E+01_JPRB, 0.13536E+01_JPRB, 0.13769E+01_JPRB, 0.13906E+01_JPRB, 0.13553E+01_JPRB, 0.13865E+01_JPRB, 0.14298E+01_JPRB ,&
&  0.14559E+01_JPRB, 0.14597E+01_JPRB, 0.14678E+01_JPRB, 0.15339E+01_JPRB, 0.16473E+01_JPRB, 0.16594E+01_JPRB, 0.17494E+01_JPRB ,&
&  0.18491E+01_JPRB, 0.19869E+01_JPRB, 0.20049E+01_JPRB, 0.19162E+01_JPRB, 0.19090E+01_JPRB, 0.19341E+01_JPRB, 0.19284E+01_JPRB ,&
&  0.19045E+01_JPRB, 0.18582E+01_JPRB, 0.18479E+01_JPRB, 0.18485E+01_JPRB, 0.17866E+01_JPRB, 0.15337E+01_JPRB, 0.14360E+01_JPRB ,&
&  0.13228E+01_JPRB, 0.11899E+01_JPRB, 0.10918E+01_JPRB, 0.12017E+01_JPRB, 0.95289E+00_JPRB, 0.10233E+01_JPRB, 0.98812E+00_JPRB ,&
&  0.11015E+01_JPRB, 0.11205E+01_JPRB, 0.10900E+01_JPRB, 0.10390E+01_JPRB, 0.99769E+00_JPRB, 0.84945E+00_JPRB, 0.78857E+00_JPRB ,&
&  0.71336E+00_JPRB, 0.70166E+00_JPRB, 0.59213E+00_JPRB, 0.49336E+00_JPRB, 0.43882E+00_JPRB, 0.47421E+00_JPRB, 0.54248E+00_JPRB ,&
&  0.62673E+00_JPRB, 0.65095E+00_JPRB, 0.79964E+00_JPRB, 0.85248E+00_JPRB, 0.86186E+00_JPRB, 0.66845E+00_JPRB, 0.64417E+00_JPRB ,&
&  0.65162E+00_JPRB, 0.65645E+00_JPRB, 0.63246E+00_JPRB, 0.64037E+00_JPRB, 0.66476E+00_JPRB, 0.70314E+00_JPRB, 0.71995E+00_JPRB ,&
&  0.71792E+00_JPRB, 0.70162E+00_JPRB, 0.69936E+00_JPRB, 0.70876E+00_JPRB, 0.73527E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.67405E+00_JPRB, 0.69890E+00_JPRB, 0.72876E+00_JPRB, 0.77922E+00_JPRB, 0.85218E+00_JPRB, 0.94378E+00_JPRB, 0.96193E+00_JPRB ,&
&  0.82115E+00_JPRB, 0.58697E+00_JPRB, 0.60463E+00_JPRB, 0.63244E+00_JPRB, 0.68265E+00_JPRB, 0.73049E+00_JPRB, 0.78395E+00_JPRB ,&
&  0.86589E+00_JPRB, 0.97549E+00_JPRB, 0.11650E+01_JPRB, 0.13737E+01_JPRB, 0.15438E+01_JPRB, 0.17449E+01_JPRB, 0.16191E+01_JPRB ,&
&  0.11201E+01_JPRB, 0.12097E+01_JPRB, 0.15260E+01_JPRB, 0.14855E+01_JPRB, 0.13092E+01_JPRB, 0.12983E+01_JPRB, 0.12605E+01_JPRB ,&
&  0.12415E+01_JPRB, 0.12883E+01_JPRB, 0.13136E+01_JPRB, 0.12901E+01_JPRB, 0.12628E+01_JPRB, 0.12939E+01_JPRB, 0.13496E+01_JPRB ,&
&  0.14063E+01_JPRB, 0.14502E+01_JPRB, 0.14780E+01_JPRB, 0.15040E+01_JPRB, 0.15529E+01_JPRB, 0.15613E+01_JPRB, 0.16416E+01_JPRB ,&
&  0.17693E+01_JPRB, 0.18789E+01_JPRB, 0.18913E+01_JPRB, 0.18216E+01_JPRB, 0.18460E+01_JPRB, 0.19349E+01_JPRB, 0.19900E+01_JPRB ,&
&  0.20186E+01_JPRB, 0.20777E+01_JPRB, 0.20929E+01_JPRB, 0.20046E+01_JPRB, 0.17855E+01_JPRB, 0.18230E+01_JPRB, 0.15603E+01_JPRB ,&
&  0.14511E+01_JPRB, 0.12588E+01_JPRB, 0.12530E+01_JPRB, 0.99941E+00_JPRB, 0.10484E+01_JPRB, 0.10754E+01_JPRB, 0.10752E+01_JPRB ,&
&  0.10979E+01_JPRB, 0.11724E+01_JPRB, 0.11540E+01_JPRB, 0.10551E+01_JPRB, 0.10313E+01_JPRB, 0.87019E+00_JPRB, 0.81482E+00_JPRB ,&
&  0.66995E+00_JPRB, 0.72422E+00_JPRB, 0.60652E+00_JPRB, 0.46881E+00_JPRB, 0.37105E+00_JPRB, 0.36910E+00_JPRB, 0.42960E+00_JPRB ,&
&  0.57856E+00_JPRB, 0.65114E+00_JPRB, 0.70112E+00_JPRB, 0.73275E+00_JPRB, 0.75199E+00_JPRB, 0.67173E+00_JPRB, 0.67210E+00_JPRB ,&
&  0.68659E+00_JPRB, 0.66719E+00_JPRB, 0.62135E+00_JPRB, 0.62237E+00_JPRB, 0.64438E+00_JPRB, 0.67950E+00_JPRB, 0.67413E+00_JPRB ,&
&  0.67621E+00_JPRB, 0.66658E+00_JPRB, 0.65104E+00_JPRB, 0.64050E+00_JPRB, 0.65053E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.59756E+00_JPRB, 0.59848E+00_JPRB, 0.59056E+00_JPRB, 0.58788E+00_JPRB, 0.59139E+00_JPRB, 0.60318E+00_JPRB, 0.62798E+00_JPRB ,&
&  0.57336E+00_JPRB, 0.56365E+00_JPRB, 0.56839E+00_JPRB, 0.56532E+00_JPRB, 0.57604E+00_JPRB, 0.58825E+00_JPRB, 0.61248E+00_JPRB ,&
&  0.64252E+00_JPRB, 0.67004E+00_JPRB, 0.78353E+00_JPRB, 0.10077E+01_JPRB, 0.12253E+01_JPRB, 0.14668E+01_JPRB, 0.13381E+01_JPRB ,&
&  0.10499E+01_JPRB, 0.17252E+01_JPRB, 0.13723E+01_JPRB, 0.14282E+01_JPRB, 0.13796E+01_JPRB, 0.13591E+01_JPRB, 0.12480E+01_JPRB ,&
&  0.12223E+01_JPRB, 0.12271E+01_JPRB, 0.12614E+01_JPRB, 0.12471E+01_JPRB, 0.12535E+01_JPRB, 0.12692E+01_JPRB, 0.13006E+01_JPRB ,&
&  0.13547E+01_JPRB, 0.14062E+01_JPRB, 0.14410E+01_JPRB, 0.14732E+01_JPRB, 0.15126E+01_JPRB, 0.15233E+01_JPRB, 0.15462E+01_JPRB ,&
&  0.16353E+01_JPRB, 0.17603E+01_JPRB, 0.17879E+01_JPRB, 0.18209E+01_JPRB, 0.18706E+01_JPRB, 0.20214E+01_JPRB, 0.21261E+01_JPRB ,&
&  0.20840E+01_JPRB, 0.20652E+01_JPRB, 0.22608E+01_JPRB, 0.22705E+01_JPRB, 0.21747E+01_JPRB, 0.21388E+01_JPRB, 0.17379E+01_JPRB ,&
&  0.16303E+01_JPRB, 0.13732E+01_JPRB, 0.13516E+01_JPRB, 0.10822E+01_JPRB, 0.11019E+01_JPRB, 0.11460E+01_JPRB, 0.10807E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.11562E+01_JPRB, 0.11410E+01_JPRB, 0.11051E+01_JPRB, 0.10252E+01_JPRB, 0.93736E+00_JPRB, 0.83164E+00_JPRB ,&
&  0.73330E+00_JPRB, 0.70420E+00_JPRB, 0.69393E+00_JPRB, 0.48976E+00_JPRB, 0.40503E+00_JPRB, 0.37561E+00_JPRB, 0.39733E+00_JPRB ,&
&  0.51202E+00_JPRB, 0.59064E+00_JPRB, 0.73107E+00_JPRB, 0.71117E+00_JPRB, 0.70721E+00_JPRB, 0.68395E+00_JPRB, 0.69456E+00_JPRB ,&
&  0.68196E+00_JPRB, 0.68085E+00_JPRB, 0.64501E+00_JPRB, 0.63290E+00_JPRB, 0.65355E+00_JPRB, 0.67606E+00_JPRB, 0.66404E+00_JPRB ,&
&  0.66707E+00_JPRB, 0.66380E+00_JPRB, 0.64740E+00_JPRB, 0.62215E+00_JPRB, 0.60599E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.59869E+00_JPRB, 0.58359E+00_JPRB, 0.58208E+00_JPRB, 0.57545E+00_JPRB, 0.57077E+00_JPRB, 0.56431E+00_JPRB, 0.57348E+00_JPRB ,&
&  0.55665E+00_JPRB, 0.54469E+00_JPRB, 0.54507E+00_JPRB, 0.53475E+00_JPRB, 0.53777E+00_JPRB, 0.52470E+00_JPRB, 0.53322E+00_JPRB ,&
&  0.54128E+00_JPRB, 0.51711E+00_JPRB, 0.55377E+00_JPRB, 0.68972E+00_JPRB, 0.88227E+00_JPRB, 0.11897E+01_JPRB, 0.90356E+00_JPRB ,&
&  0.11175E+01_JPRB, 0.16953E+01_JPRB, 0.14955E+01_JPRB, 0.15258E+01_JPRB, 0.15917E+01_JPRB, 0.15984E+01_JPRB, 0.13267E+01_JPRB ,&
&  0.12033E+01_JPRB, 0.11654E+01_JPRB, 0.11864E+01_JPRB, 0.12377E+01_JPRB, 0.13086E+01_JPRB, 0.13790E+01_JPRB, 0.13819E+01_JPRB ,&
&  0.14016E+01_JPRB, 0.14364E+01_JPRB, 0.14562E+01_JPRB, 0.14603E+01_JPRB, 0.14785E+01_JPRB, 0.15009E+01_JPRB, 0.15058E+01_JPRB ,&
&  0.15314E+01_JPRB, 0.16065E+01_JPRB, 0.17555E+01_JPRB, 0.17469E+01_JPRB, 0.18634E+01_JPRB, 0.20830E+01_JPRB, 0.22109E+01_JPRB ,&
&  0.20837E+01_JPRB, 0.20382E+01_JPRB, 0.23844E+01_JPRB, 0.26998E+01_JPRB, 0.27294E+01_JPRB, 0.25221E+01_JPRB, 0.20602E+01_JPRB ,&
&  0.17995E+01_JPRB, 0.15471E+01_JPRB, 0.13958E+01_JPRB, 0.11788E+01_JPRB, 0.11609E+01_JPRB, 0.12000E+01_JPRB, 0.11821E+01_JPRB ,&
&  0.12331E+01_JPRB, 0.12122E+01_JPRB, 0.11966E+01_JPRB, 0.11147E+01_JPRB, 0.10191E+01_JPRB, 0.94319E+00_JPRB, 0.93176E+00_JPRB ,&
&  0.90841E+00_JPRB, 0.91037E+00_JPRB, 0.87452E+00_JPRB, 0.59731E+00_JPRB, 0.47136E+00_JPRB, 0.36578E+00_JPRB, 0.38474E+00_JPRB ,&
&  0.48636E+00_JPRB, 0.53026E+00_JPRB, 0.56345E+00_JPRB, 0.78321E+00_JPRB, 0.76109E+00_JPRB, 0.74518E+00_JPRB, 0.75329E+00_JPRB ,&
&  0.72675E+00_JPRB, 0.74628E+00_JPRB, 0.72947E+00_JPRB, 0.69633E+00_JPRB, 0.69737E+00_JPRB, 0.70006E+00_JPRB, 0.69906E+00_JPRB ,&
&  0.68276E+00_JPRB, 0.67542E+00_JPRB, 0.67198E+00_JPRB, 0.65559E+00_JPRB, 0.62588E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.60514E+00_JPRB, 0.59377E+00_JPRB, 0.57347E+00_JPRB, 0.56388E+00_JPRB, 0.56669E+00_JPRB, 0.55243E+00_JPRB, 0.54076E+00_JPRB ,&
&  0.54240E+00_JPRB, 0.52881E+00_JPRB, 0.51558E+00_JPRB, 0.51140E+00_JPRB, 0.51104E+00_JPRB, 0.50427E+00_JPRB, 0.50313E+00_JPRB ,&
&  0.49684E+00_JPRB, 0.47002E+00_JPRB, 0.51249E+00_JPRB, 0.57365E+00_JPRB, 0.69651E+00_JPRB, 0.76048E+00_JPRB, 0.84659E+00_JPRB ,&
&  0.14090E+01_JPRB, 0.18727E+01_JPRB, 0.16769E+01_JPRB, 0.18275E+01_JPRB, 0.20685E+01_JPRB, 0.23494E+01_JPRB, 0.17161E+01_JPRB ,&
&  0.14459E+01_JPRB, 0.12862E+01_JPRB, 0.11798E+01_JPRB, 0.12399E+01_JPRB, 0.13692E+01_JPRB, 0.14829E+01_JPRB, 0.15178E+01_JPRB ,&
&  0.14693E+01_JPRB, 0.14390E+01_JPRB, 0.14477E+01_JPRB, 0.14498E+01_JPRB, 0.14495E+01_JPRB, 0.14883E+01_JPRB, 0.15012E+01_JPRB ,&
&  0.15014E+01_JPRB, 0.15348E+01_JPRB, 0.16516E+01_JPRB, 0.16872E+01_JPRB, 0.18353E+01_JPRB, 0.20619E+01_JPRB, 0.23329E+01_JPRB ,&
&  0.23447E+01_JPRB, 0.24244E+01_JPRB, 0.26605E+01_JPRB, 0.29626E+01_JPRB, 0.31474E+01_JPRB, 0.30395E+01_JPRB, 0.25024E+01_JPRB ,&
&  0.20982E+01_JPRB, 0.16903E+01_JPRB, 0.13796E+01_JPRB, 0.12791E+01_JPRB, 0.12882E+01_JPRB, 0.13281E+01_JPRB, 0.11795E+01_JPRB ,&
&  0.11203E+01_JPRB, 0.10071E+01_JPRB, 0.98066E+00_JPRB, 0.10513E+01_JPRB, 0.10490E+01_JPRB, 0.99771E+00_JPRB, 0.10977E+01_JPRB ,&
&  0.97737E+00_JPRB, 0.63391E+00_JPRB, 0.55835E+00_JPRB, 0.62699E+00_JPRB, 0.45734E+00_JPRB, 0.34649E+00_JPRB, 0.42090E+00_JPRB ,&
&  0.48959E+00_JPRB, 0.55261E+00_JPRB, 0.56778E+00_JPRB, 0.61391E+00_JPRB, 0.77885E+00_JPRB, 0.77253E+00_JPRB, 0.83656E+00_JPRB ,&
&  0.76576E+00_JPRB, 0.77983E+00_JPRB, 0.78096E+00_JPRB, 0.73059E+00_JPRB, 0.70701E+00_JPRB, 0.72593E+00_JPRB, 0.73231E+00_JPRB ,&
&  0.72006E+00_JPRB, 0.70763E+00_JPRB, 0.68192E+00_JPRB, 0.65614E+00_JPRB, 0.62597E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.64084E+00_JPRB, 0.62456E+00_JPRB, 0.61448E+00_JPRB, 0.59108E+00_JPRB, 0.57742E+00_JPRB, 0.55953E+00_JPRB, 0.53668E+00_JPRB ,&
&  0.52298E+00_JPRB, 0.51422E+00_JPRB, 0.50716E+00_JPRB, 0.49901E+00_JPRB, 0.49552E+00_JPRB, 0.49326E+00_JPRB, 0.49977E+00_JPRB ,&
&  0.48350E+00_JPRB, 0.46700E+00_JPRB, 0.56756E+00_JPRB, 0.63184E+00_JPRB, 0.67147E+00_JPRB, 0.68051E+00_JPRB, 0.90992E+00_JPRB ,&
&  0.14871E+01_JPRB, 0.20936E+01_JPRB, 0.25308E+01_JPRB, 0.26918E+01_JPRB, 0.28838E+01_JPRB, 0.31375E+01_JPRB, 0.26647E+01_JPRB ,&
&  0.21194E+01_JPRB, 0.16897E+01_JPRB, 0.13901E+01_JPRB, 0.13703E+01_JPRB, 0.14920E+01_JPRB, 0.16018E+01_JPRB, 0.15991E+01_JPRB ,&
&  0.15595E+01_JPRB, 0.14755E+01_JPRB, 0.14266E+01_JPRB, 0.13956E+01_JPRB, 0.14116E+01_JPRB, 0.14618E+01_JPRB, 0.14908E+01_JPRB ,&
&  0.14931E+01_JPRB, 0.14734E+01_JPRB, 0.15397E+01_JPRB, 0.17061E+01_JPRB, 0.16032E+01_JPRB, 0.20097E+01_JPRB, 0.23501E+01_JPRB ,&
&  0.25724E+01_JPRB, 0.26670E+01_JPRB, 0.27077E+01_JPRB, 0.33272E+01_JPRB, 0.36809E+01_JPRB, 0.34635E+01_JPRB, 0.28930E+01_JPRB ,&
&  0.25278E+01_JPRB, 0.20091E+01_JPRB, 0.15901E+01_JPRB, 0.14357E+01_JPRB, 0.13982E+01_JPRB, 0.14034E+01_JPRB, 0.99769E+00_JPRB ,&
&  0.98444E+00_JPRB, 0.11586E+01_JPRB, 0.10653E+01_JPRB, 0.97043E+00_JPRB, 0.10724E+01_JPRB, 0.10642E+01_JPRB, 0.84191E+00_JPRB ,&
&  0.48735E+00_JPRB, 0.41694E+00_JPRB, 0.41549E+00_JPRB, 0.40436E+00_JPRB, 0.36356E+00_JPRB, 0.34740E+00_JPRB, 0.54704E+00_JPRB ,&
&  0.51687E+00_JPRB, 0.59919E+00_JPRB, 0.59836E+00_JPRB, 0.63630E+00_JPRB, 0.75422E+00_JPRB, 0.78296E+00_JPRB, 0.10287E+01_JPRB ,&
&  0.88126E+00_JPRB, 0.84138E+00_JPRB, 0.81986E+00_JPRB, 0.78191E+00_JPRB, 0.76442E+00_JPRB, 0.74749E+00_JPRB, 0.75178E+00_JPRB ,&
&  0.75940E+00_JPRB, 0.76345E+00_JPRB, 0.74315E+00_JPRB, 0.70601E+00_JPRB, 0.67147E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.69088E+00_JPRB, 0.65526E+00_JPRB, 0.63287E+00_JPRB, 0.62470E+00_JPRB, 0.61255E+00_JPRB, 0.58179E+00_JPRB, 0.54134E+00_JPRB ,&
&  0.51762E+00_JPRB, 0.50418E+00_JPRB, 0.49593E+00_JPRB, 0.49201E+00_JPRB, 0.49645E+00_JPRB, 0.49899E+00_JPRB, 0.49788E+00_JPRB ,&
&  0.47288E+00_JPRB, 0.48439E+00_JPRB, 0.63359E+00_JPRB, 0.69117E+00_JPRB, 0.72698E+00_JPRB, 0.73289E+00_JPRB, 0.94192E+00_JPRB ,&
&  0.14943E+01_JPRB, 0.21385E+01_JPRB, 0.26184E+01_JPRB, 0.27447E+01_JPRB, 0.36730E+01_JPRB, 0.42011E+01_JPRB, 0.41431E+01_JPRB ,&
&  0.33500E+01_JPRB, 0.26211E+01_JPRB, 0.20217E+01_JPRB, 0.17199E+01_JPRB, 0.17222E+01_JPRB, 0.17634E+01_JPRB, 0.17462E+01_JPRB ,&
&  0.16549E+01_JPRB, 0.15641E+01_JPRB, 0.14812E+01_JPRB, 0.14315E+01_JPRB, 0.14280E+01_JPRB, 0.14625E+01_JPRB, 0.15132E+01_JPRB ,&
&  0.15188E+01_JPRB, 0.14614E+01_JPRB, 0.14362E+01_JPRB, 0.16050E+01_JPRB, 0.17047E+01_JPRB, 0.18016E+01_JPRB, 0.21927E+01_JPRB ,&
&  0.26021E+01_JPRB, 0.29029E+01_JPRB, 0.29367E+01_JPRB, 0.38989E+01_JPRB, 0.41314E+01_JPRB, 0.37891E+01_JPRB, 0.33925E+01_JPRB ,&
&  0.29814E+01_JPRB, 0.26248E+01_JPRB, 0.18405E+01_JPRB, 0.15785E+01_JPRB, 0.15161E+01_JPRB, 0.12411E+01_JPRB, 0.10903E+01_JPRB ,&
&  0.12386E+01_JPRB, 0.11647E+01_JPRB, 0.10320E+01_JPRB, 0.88941E+00_JPRB, 0.10692E+01_JPRB, 0.95227E+00_JPRB, 0.53634E+00_JPRB ,&
&  0.45419E+00_JPRB, 0.43071E+00_JPRB, 0.39050E+00_JPRB, 0.37363E+00_JPRB, 0.35117E+00_JPRB, 0.35388E+00_JPRB, 0.48434E+00_JPRB ,&
&  0.52366E+00_JPRB, 0.59384E+00_JPRB, 0.61110E+00_JPRB, 0.65654E+00_JPRB, 0.75392E+00_JPRB, 0.86595E+00_JPRB, 0.14246E+01_JPRB ,&
&  0.11395E+01_JPRB, 0.10235E+01_JPRB, 0.91233E+00_JPRB, 0.85700E+00_JPRB, 0.81224E+00_JPRB, 0.77700E+00_JPRB, 0.76732E+00_JPRB ,&
&  0.76464E+00_JPRB, 0.76674E+00_JPRB, 0.75360E+00_JPRB, 0.73445E+00_JPRB, 0.71455E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.71754E+00_JPRB, 0.68816E+00_JPRB, 0.65435E+00_JPRB, 0.62858E+00_JPRB, 0.61466E+00_JPRB, 0.59817E+00_JPRB, 0.56996E+00_JPRB ,&
&  0.53761E+00_JPRB, 0.52072E+00_JPRB, 0.51114E+00_JPRB, 0.49837E+00_JPRB, 0.49796E+00_JPRB, 0.50410E+00_JPRB, 0.50184E+00_JPRB ,&
&  0.48135E+00_JPRB, 0.52055E+00_JPRB, 0.62065E+00_JPRB, 0.62707E+00_JPRB, 0.64220E+00_JPRB, 0.66246E+00_JPRB, 0.83166E+00_JPRB ,&
&  0.14074E+01_JPRB, 0.20666E+01_JPRB, 0.25171E+01_JPRB, 0.29951E+01_JPRB, 0.40349E+01_JPRB, 0.41975E+01_JPRB, 0.47792E+01_JPRB ,&
&  0.42407E+01_JPRB, 0.35394E+01_JPRB, 0.29185E+01_JPRB, 0.25086E+01_JPRB, 0.22555E+01_JPRB, 0.20621E+01_JPRB, 0.19555E+01_JPRB ,&
&  0.18404E+01_JPRB, 0.16904E+01_JPRB, 0.16246E+01_JPRB, 0.15612E+01_JPRB, 0.15334E+01_JPRB, 0.15248E+01_JPRB, 0.15885E+01_JPRB ,&
&  0.15879E+01_JPRB, 0.15219E+01_JPRB, 0.14022E+01_JPRB, 0.14636E+01_JPRB, 0.18018E+01_JPRB, 0.20477E+01_JPRB, 0.21495E+01_JPRB ,&
&  0.22836E+01_JPRB, 0.24175E+01_JPRB, 0.30169E+01_JPRB, 0.38286E+01_JPRB, 0.43262E+01_JPRB, 0.43001E+01_JPRB, 0.36181E+01_JPRB ,&
&  0.31728E+01_JPRB, 0.26871E+01_JPRB, 0.19494E+01_JPRB, 0.16724E+01_JPRB, 0.14347E+01_JPRB, 0.11840E+01_JPRB, 0.12721E+01_JPRB ,&
&  0.13633E+01_JPRB, 0.12749E+01_JPRB, 0.10689E+01_JPRB, 0.84364E+00_JPRB, 0.84412E+00_JPRB, 0.66458E+00_JPRB, 0.50327E+00_JPRB ,&
&  0.47627E+00_JPRB, 0.44377E+00_JPRB, 0.40530E+00_JPRB, 0.40855E+00_JPRB, 0.38672E+00_JPRB, 0.40761E+00_JPRB, 0.50856E+00_JPRB ,&
&  0.58575E+00_JPRB, 0.61681E+00_JPRB, 0.66166E+00_JPRB, 0.68464E+00_JPRB, 0.71318E+00_JPRB, 0.69065E+00_JPRB, 0.12246E+01_JPRB ,&
&  0.91317E+00_JPRB, 0.92611E+00_JPRB, 0.94718E+00_JPRB, 0.92445E+00_JPRB, 0.90775E+00_JPRB, 0.85793E+00_JPRB, 0.80631E+00_JPRB ,&
&  0.77716E+00_JPRB, 0.75739E+00_JPRB, 0.74382E+00_JPRB, 0.74113E+00_JPRB, 0.73428E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.71681E+00_JPRB, 0.68333E+00_JPRB, 0.65445E+00_JPRB, 0.62735E+00_JPRB, 0.60524E+00_JPRB, 0.58872E+00_JPRB, 0.57610E+00_JPRB ,&
&  0.55812E+00_JPRB, 0.54445E+00_JPRB, 0.52705E+00_JPRB, 0.50842E+00_JPRB, 0.49921E+00_JPRB, 0.49850E+00_JPRB, 0.49060E+00_JPRB ,&
&  0.47408E+00_JPRB, 0.56410E+00_JPRB, 0.44751E+00_JPRB, 0.43182E+00_JPRB, 0.53214E+00_JPRB, 0.61774E+00_JPRB, 0.70048E+00_JPRB ,&
&  0.11469E+01_JPRB, 0.18414E+01_JPRB, 0.23218E+01_JPRB, 0.32661E+01_JPRB, 0.51922E+01_JPRB, 0.49528E+01_JPRB, 0.50008E+01_JPRB ,&
&  0.49654E+01_JPRB, 0.41192E+01_JPRB, 0.35134E+01_JPRB, 0.29837E+01_JPRB, 0.25930E+01_JPRB, 0.22911E+01_JPRB, 0.20367E+01_JPRB ,&
&  0.18275E+01_JPRB, 0.16818E+01_JPRB, 0.16054E+01_JPRB, 0.15989E+01_JPRB, 0.16132E+01_JPRB, 0.16402E+01_JPRB, 0.16661E+01_JPRB ,&
&  0.16619E+01_JPRB, 0.16100E+01_JPRB, 0.14758E+01_JPRB, 0.14668E+01_JPRB, 0.16140E+01_JPRB, 0.18847E+01_JPRB, 0.25872E+01_JPRB ,&
&  0.25895E+01_JPRB, 0.27341E+01_JPRB, 0.27226E+01_JPRB, 0.39816E+01_JPRB, 0.45793E+01_JPRB, 0.41963E+01_JPRB, 0.37458E+01_JPRB ,&
&  0.27579E+01_JPRB, 0.22035E+01_JPRB, 0.18488E+01_JPRB, 0.15650E+01_JPRB, 0.13630E+01_JPRB, 0.14117E+01_JPRB, 0.18204E+01_JPRB ,&
&  0.15251E+01_JPRB, 0.15777E+01_JPRB, 0.15495E+01_JPRB, 0.12850E+01_JPRB, 0.79684E+00_JPRB, 0.65725E+00_JPRB, 0.84039E+00_JPRB ,&
&  0.87234E+00_JPRB, 0.59601E+00_JPRB, 0.48215E+00_JPRB, 0.55208E+00_JPRB, 0.45812E+00_JPRB, 0.47747E+00_JPRB, 0.55768E+00_JPRB ,&
&  0.58235E+00_JPRB, 0.60251E+00_JPRB, 0.71623E+00_JPRB, 0.72851E+00_JPRB, 0.73274E+00_JPRB, 0.65231E+00_JPRB, 0.91126E+00_JPRB ,&
&  0.83371E+00_JPRB, 0.81742E+00_JPRB, 0.86315E+00_JPRB, 0.84613E+00_JPRB, 0.84766E+00_JPRB, 0.83103E+00_JPRB, 0.79390E+00_JPRB ,&
&  0.76956E+00_JPRB, 0.74897E+00_JPRB, 0.73892E+00_JPRB, 0.73647E+00_JPRB, 0.73673E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.71012E+00_JPRB, 0.67677E+00_JPRB, 0.64158E+00_JPRB, 0.61690E+00_JPRB, 0.59648E+00_JPRB, 0.57487E+00_JPRB, 0.55629E+00_JPRB ,&
&  0.54526E+00_JPRB, 0.53446E+00_JPRB, 0.51475E+00_JPRB, 0.50066E+00_JPRB, 0.49082E+00_JPRB, 0.49303E+00_JPRB, 0.49080E+00_JPRB ,&
&  0.47698E+00_JPRB, 0.45409E+00_JPRB, 0.38128E+00_JPRB, 0.37157E+00_JPRB, 0.49812E+00_JPRB, 0.53581E+00_JPRB, 0.60899E+00_JPRB ,&
&  0.90539E+00_JPRB, 0.15574E+01_JPRB, 0.22986E+01_JPRB, 0.32859E+01_JPRB, 0.46774E+01_JPRB, 0.54605E+01_JPRB, 0.52866E+01_JPRB ,&
&  0.53274E+01_JPRB, 0.45163E+01_JPRB, 0.38690E+01_JPRB, 0.33249E+01_JPRB, 0.28080E+01_JPRB, 0.23280E+01_JPRB, 0.19584E+01_JPRB ,&
&  0.17483E+01_JPRB, 0.16027E+01_JPRB, 0.15202E+01_JPRB, 0.14464E+01_JPRB, 0.14697E+01_JPRB, 0.15414E+01_JPRB, 0.16206E+01_JPRB ,&
&  0.16461E+01_JPRB, 0.16306E+01_JPRB, 0.15647E+01_JPRB, 0.14911E+01_JPRB, 0.16411E+01_JPRB, 0.19318E+01_JPRB, 0.22400E+01_JPRB ,&
&  0.25163E+01_JPRB, 0.26169E+01_JPRB, 0.27966E+01_JPRB, 0.33056E+01_JPRB, 0.39223E+01_JPRB, 0.36475E+01_JPRB, 0.35386E+01_JPRB ,&
&  0.27550E+01_JPRB, 0.22642E+01_JPRB, 0.18580E+01_JPRB, 0.13808E+01_JPRB, 0.13330E+01_JPRB, 0.18317E+01_JPRB, 0.19902E+01_JPRB ,&
&  0.19197E+01_JPRB, 0.18993E+01_JPRB, 0.17090E+01_JPRB, 0.13013E+01_JPRB, 0.98191E+00_JPRB, 0.80239E+00_JPRB, 0.90188E+00_JPRB ,&
&  0.91399E+00_JPRB, 0.90118E+00_JPRB, 0.82785E+00_JPRB, 0.72099E+00_JPRB, 0.62471E+00_JPRB, 0.65283E+00_JPRB, 0.59915E+00_JPRB ,&
&  0.56041E+00_JPRB, 0.55688E+00_JPRB, 0.66652E+00_JPRB, 0.72016E+00_JPRB, 0.71182E+00_JPRB, 0.60313E+00_JPRB, 0.79219E+00_JPRB ,&
&  0.75927E+00_JPRB, 0.77803E+00_JPRB, 0.74798E+00_JPRB, 0.73831E+00_JPRB, 0.75292E+00_JPRB, 0.76040E+00_JPRB, 0.76019E+00_JPRB ,&
&  0.74140E+00_JPRB, 0.73224E+00_JPRB, 0.73088E+00_JPRB, 0.72995E+00_JPRB, 0.72487E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.69774E+00_JPRB, 0.66695E+00_JPRB, 0.63692E+00_JPRB, 0.61456E+00_JPRB, 0.59321E+00_JPRB, 0.57201E+00_JPRB, 0.54832E+00_JPRB ,&
&  0.53100E+00_JPRB, 0.52049E+00_JPRB, 0.50598E+00_JPRB, 0.49456E+00_JPRB, 0.49109E+00_JPRB, 0.49453E+00_JPRB, 0.49951E+00_JPRB ,&
&  0.49500E+00_JPRB, 0.47151E+00_JPRB, 0.36931E+00_JPRB, 0.37345E+00_JPRB, 0.42912E+00_JPRB, 0.44538E+00_JPRB, 0.54773E+00_JPRB ,&
&  0.75360E+00_JPRB, 0.12362E+01_JPRB, 0.18683E+01_JPRB, 0.27357E+01_JPRB, 0.37064E+01_JPRB, 0.44429E+01_JPRB, 0.45204E+01_JPRB ,&
&  0.39510E+01_JPRB, 0.38700E+01_JPRB, 0.33538E+01_JPRB, 0.28669E+01_JPRB, 0.23760E+01_JPRB, 0.20131E+01_JPRB, 0.17644E+01_JPRB ,&
&  0.15539E+01_JPRB, 0.14192E+01_JPRB, 0.13542E+01_JPRB, 0.13322E+01_JPRB, 0.13549E+01_JPRB, 0.14107E+01_JPRB, 0.14880E+01_JPRB ,&
&  0.15159E+01_JPRB, 0.15232E+01_JPRB, 0.14918E+01_JPRB, 0.15258E+01_JPRB, 0.15176E+01_JPRB, 0.19597E+01_JPRB, 0.21241E+01_JPRB ,&
&  0.26512E+01_JPRB, 0.28940E+01_JPRB, 0.29049E+01_JPRB, 0.36314E+01_JPRB, 0.30492E+01_JPRB, 0.32262E+01_JPRB, 0.32873E+01_JPRB ,&
&  0.30547E+01_JPRB, 0.27895E+01_JPRB, 0.25742E+01_JPRB, 0.20289E+01_JPRB, 0.15924E+01_JPRB, 0.20364E+01_JPRB, 0.20208E+01_JPRB ,&
&  0.18458E+01_JPRB, 0.17628E+01_JPRB, 0.15795E+01_JPRB, 0.13935E+01_JPRB, 0.11384E+01_JPRB, 0.95378E+00_JPRB, 0.78691E+00_JPRB ,&
&  0.79663E+00_JPRB, 0.75532E+00_JPRB, 0.79495E+00_JPRB, 0.74979E+00_JPRB, 0.71014E+00_JPRB, 0.67397E+00_JPRB, 0.60975E+00_JPRB ,&
&  0.55828E+00_JPRB, 0.53414E+00_JPRB, 0.56431E+00_JPRB, 0.59714E+00_JPRB, 0.62024E+00_JPRB, 0.53982E+00_JPRB, 0.61651E+00_JPRB ,&
&  0.69889E+00_JPRB, 0.73296E+00_JPRB, 0.69118E+00_JPRB, 0.72781E+00_JPRB, 0.74607E+00_JPRB, 0.73827E+00_JPRB, 0.73567E+00_JPRB ,&
&  0.72899E+00_JPRB, 0.72227E+00_JPRB, 0.72247E+00_JPRB, 0.72947E+00_JPRB, 0.71993E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.65946E+00_JPRB, 0.63731E+00_JPRB, 0.61389E+00_JPRB, 0.59588E+00_JPRB, 0.58462E+00_JPRB, 0.57161E+00_JPRB, 0.54892E+00_JPRB ,&
&  0.52276E+00_JPRB, 0.50638E+00_JPRB, 0.49715E+00_JPRB, 0.49024E+00_JPRB, 0.48790E+00_JPRB, 0.49347E+00_JPRB, 0.50094E+00_JPRB ,&
&  0.50403E+00_JPRB, 0.62075E+00_JPRB, 0.46886E+00_JPRB, 0.37592E+00_JPRB, 0.34161E+00_JPRB, 0.40853E+00_JPRB, 0.52277E+00_JPRB ,&
&  0.70572E+00_JPRB, 0.10402E+01_JPRB, 0.16032E+01_JPRB, 0.20769E+01_JPRB, 0.26020E+01_JPRB, 0.30960E+01_JPRB, 0.32245E+01_JPRB ,&
&  0.33410E+01_JPRB, 0.26517E+01_JPRB, 0.24048E+01_JPRB, 0.19564E+01_JPRB, 0.16961E+01_JPRB, 0.15147E+01_JPRB, 0.13584E+01_JPRB ,&
&  0.12284E+01_JPRB, 0.11608E+01_JPRB, 0.11178E+01_JPRB, 0.11233E+01_JPRB, 0.11330E+01_JPRB, 0.11649E+01_JPRB, 0.12233E+01_JPRB ,&
&  0.12869E+01_JPRB, 0.13207E+01_JPRB, 0.13206E+01_JPRB, 0.13905E+01_JPRB, 0.16440E+01_JPRB, 0.19876E+01_JPRB, 0.22510E+01_JPRB ,&
&  0.21449E+01_JPRB, 0.20957E+01_JPRB, 0.32455E+01_JPRB, 0.38308E+01_JPRB, 0.39671E+01_JPRB, 0.39064E+01_JPRB, 0.35111E+01_JPRB ,&
&  0.31708E+01_JPRB, 0.29029E+01_JPRB, 0.26223E+01_JPRB, 0.22134E+01_JPRB, 0.22088E+01_JPRB, 0.21568E+01_JPRB, 0.20845E+01_JPRB ,&
&  0.18620E+01_JPRB, 0.16937E+01_JPRB, 0.15343E+01_JPRB, 0.14081E+01_JPRB, 0.12847E+01_JPRB, 0.11916E+01_JPRB, 0.10681E+01_JPRB ,&
&  0.94217E+00_JPRB, 0.96336E+00_JPRB, 0.87598E+00_JPRB, 0.76871E+00_JPRB, 0.67166E+00_JPRB, 0.63497E+00_JPRB, 0.57922E+00_JPRB ,&
&  0.53224E+00_JPRB, 0.49414E+00_JPRB, 0.49924E+00_JPRB, 0.51605E+00_JPRB, 0.52967E+00_JPRB, 0.52110E+00_JPRB, 0.50806E+00_JPRB ,&
&  0.51231E+00_JPRB, 0.63931E+00_JPRB, 0.70698E+00_JPRB, 0.72976E+00_JPRB, 0.75751E+00_JPRB, 0.73428E+00_JPRB, 0.72331E+00_JPRB ,&
&  0.70979E+00_JPRB, 0.70204E+00_JPRB, 0.69830E+00_JPRB, 0.69204E+00_JPRB, 0.67899E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.60855E+00_JPRB, 0.59997E+00_JPRB, 0.59160E+00_JPRB, 0.57998E+00_JPRB, 0.57341E+00_JPRB, 0.56318E+00_JPRB, 0.54526E+00_JPRB ,&
&  0.52929E+00_JPRB, 0.51367E+00_JPRB, 0.49975E+00_JPRB, 0.49066E+00_JPRB, 0.48684E+00_JPRB, 0.48902E+00_JPRB, 0.48958E+00_JPRB ,&
&  0.50379E+00_JPRB, 0.84012E+00_JPRB, 0.60202E+00_JPRB, 0.48528E+00_JPRB, 0.40962E+00_JPRB, 0.45738E+00_JPRB, 0.53796E+00_JPRB ,&
&  0.72999E+00_JPRB, 0.85830E+00_JPRB, 0.11049E+01_JPRB, 0.14618E+01_JPRB, 0.17902E+01_JPRB, 0.20007E+01_JPRB, 0.21529E+01_JPRB ,&
&  0.19658E+01_JPRB, 0.17726E+01_JPRB, 0.13900E+01_JPRB, 0.13067E+01_JPRB, 0.11802E+01_JPRB, 0.10096E+01_JPRB, 0.10328E+01_JPRB ,&
&  0.10009E+01_JPRB, 0.10011E+01_JPRB, 0.99469E+00_JPRB, 0.10086E+01_JPRB, 0.10260E+01_JPRB, 0.10507E+01_JPRB, 0.10847E+01_JPRB ,&
&  0.11173E+01_JPRB, 0.11535E+01_JPRB, 0.11848E+01_JPRB, 0.12451E+01_JPRB, 0.14263E+01_JPRB, 0.17688E+01_JPRB, 0.20168E+01_JPRB ,&
&  0.24112E+01_JPRB, 0.23683E+01_JPRB, 0.22740E+01_JPRB, 0.31843E+01_JPRB, 0.42127E+01_JPRB, 0.39524E+01_JPRB, 0.32596E+01_JPRB ,&
&  0.29240E+01_JPRB, 0.27262E+01_JPRB, 0.25033E+01_JPRB, 0.22718E+01_JPRB, 0.19873E+01_JPRB, 0.18970E+01_JPRB, 0.17964E+01_JPRB ,&
&  0.16222E+01_JPRB, 0.14772E+01_JPRB, 0.13730E+01_JPRB, 0.12727E+01_JPRB, 0.11944E+01_JPRB, 0.11510E+01_JPRB, 0.11162E+01_JPRB ,&
&  0.10472E+01_JPRB, 0.92376E+00_JPRB, 0.69029E+00_JPRB, 0.67199E+00_JPRB, 0.55961E+00_JPRB, 0.50787E+00_JPRB, 0.48202E+00_JPRB ,&
&  0.43727E+00_JPRB, 0.44920E+00_JPRB, 0.46295E+00_JPRB, 0.45485E+00_JPRB, 0.50134E+00_JPRB, 0.47057E+00_JPRB, 0.45077E+00_JPRB ,&
&  0.43894E+00_JPRB, 0.44824E+00_JPRB, 0.61479E+00_JPRB, 0.68568E+00_JPRB, 0.71066E+00_JPRB, 0.70714E+00_JPRB, 0.69468E+00_JPRB ,&
&  0.66128E+00_JPRB, 0.63928E+00_JPRB, 0.63252E+00_JPRB, 0.63037E+00_JPRB, 0.61943E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.55064E+00_JPRB, 0.55402E+00_JPRB, 0.55181E+00_JPRB, 0.53870E+00_JPRB, 0.52820E+00_JPRB, 0.52478E+00_JPRB, 0.52775E+00_JPRB ,&
&  0.52581E+00_JPRB, 0.51184E+00_JPRB, 0.50071E+00_JPRB, 0.48715E+00_JPRB, 0.47973E+00_JPRB, 0.47612E+00_JPRB, 0.47714E+00_JPRB ,&
&  0.54292E+00_JPRB, 0.70151E+00_JPRB, 0.53715E+00_JPRB, 0.43240E+00_JPRB, 0.50055E+00_JPRB, 0.48727E+00_JPRB, 0.52898E+00_JPRB ,&
&  0.58274E+00_JPRB, 0.70975E+00_JPRB, 0.78925E+00_JPRB, 0.92352E+00_JPRB, 0.10900E+01_JPRB, 0.12969E+01_JPRB, 0.13696E+01_JPRB ,&
&  0.12134E+01_JPRB, 0.95598E+00_JPRB, 0.79466E+00_JPRB, 0.76949E+00_JPRB, 0.82169E+00_JPRB, 0.73959E+00_JPRB, 0.78602E+00_JPRB ,&
&  0.80919E+00_JPRB, 0.83745E+00_JPRB, 0.86172E+00_JPRB, 0.87185E+00_JPRB, 0.89605E+00_JPRB, 0.93353E+00_JPRB, 0.97884E+00_JPRB ,&
&  0.10362E+01_JPRB, 0.10901E+01_JPRB, 0.11226E+01_JPRB, 0.11744E+01_JPRB, 0.13694E+01_JPRB, 0.18863E+01_JPRB, 0.23789E+01_JPRB ,&
&  0.27152E+01_JPRB, 0.29977E+01_JPRB, 0.35080E+01_JPRB, 0.41743E+01_JPRB, 0.42239E+01_JPRB, 0.39944E+01_JPRB, 0.34549E+01_JPRB ,&
&  0.28811E+01_JPRB, 0.24019E+01_JPRB, 0.20660E+01_JPRB, 0.17753E+01_JPRB, 0.15660E+01_JPRB, 0.15054E+01_JPRB, 0.14294E+01_JPRB ,&
&  0.13016E+01_JPRB, 0.12183E+01_JPRB, 0.11811E+01_JPRB, 0.10887E+01_JPRB, 0.10271E+01_JPRB, 0.95754E+00_JPRB, 0.94259E+00_JPRB ,&
&  0.86317E+00_JPRB, 0.63889E+00_JPRB, 0.56148E+00_JPRB, 0.57799E+00_JPRB, 0.50468E+00_JPRB, 0.45903E+00_JPRB, 0.43807E+00_JPRB ,&
&  0.39694E+00_JPRB, 0.40441E+00_JPRB, 0.42584E+00_JPRB, 0.40319E+00_JPRB, 0.42668E+00_JPRB, 0.44055E+00_JPRB, 0.39659E+00_JPRB ,&
&  0.38024E+00_JPRB, 0.39221E+00_JPRB, 0.49548E+00_JPRB, 0.57441E+00_JPRB, 0.61559E+00_JPRB, 0.61351E+00_JPRB, 0.59461E+00_JPRB ,&
&  0.55511E+00_JPRB, 0.54065E+00_JPRB, 0.54408E+00_JPRB, 0.54571E+00_JPRB, 0.54842E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.49352E+00_JPRB, 0.50378E+00_JPRB, 0.50889E+00_JPRB, 0.51010E+00_JPRB, 0.51007E+00_JPRB, 0.50713E+00_JPRB, 0.51049E+00_JPRB ,&
&  0.51431E+00_JPRB, 0.50667E+00_JPRB, 0.49324E+00_JPRB, 0.48106E+00_JPRB, 0.46777E+00_JPRB, 0.46156E+00_JPRB, 0.48114E+00_JPRB ,&
&  0.40775E+00_JPRB, 0.47790E+00_JPRB, 0.40169E+00_JPRB, 0.41383E+00_JPRB, 0.57528E+00_JPRB, 0.51953E+00_JPRB, 0.51422E+00_JPRB ,&
&  0.50406E+00_JPRB, 0.52252E+00_JPRB, 0.54352E+00_JPRB, 0.63017E+00_JPRB, 0.70591E+00_JPRB, 0.82181E+00_JPRB, 0.89185E+00_JPRB ,&
&  0.76144E+00_JPRB, 0.58118E+00_JPRB, 0.51003E+00_JPRB, 0.50228E+00_JPRB, 0.55377E+00_JPRB, 0.59075E+00_JPRB, 0.66024E+00_JPRB ,&
&  0.70000E+00_JPRB, 0.73023E+00_JPRB, 0.75578E+00_JPRB, 0.79184E+00_JPRB, 0.81116E+00_JPRB, 0.83488E+00_JPRB, 0.87117E+00_JPRB ,&
&  0.91604E+00_JPRB, 0.97781E+00_JPRB, 0.10402E+01_JPRB, 0.11690E+01_JPRB, 0.13071E+01_JPRB, 0.20569E+01_JPRB, 0.33465E+01_JPRB ,&
&  0.35132E+01_JPRB, 0.36458E+01_JPRB, 0.41326E+01_JPRB, 0.49740E+01_JPRB, 0.49056E+01_JPRB, 0.43953E+01_JPRB, 0.35938E+01_JPRB ,&
&  0.28593E+01_JPRB, 0.22604E+01_JPRB, 0.18888E+01_JPRB, 0.16148E+01_JPRB, 0.13612E+01_JPRB, 0.12871E+01_JPRB, 0.11990E+01_JPRB ,&
&  0.10647E+01_JPRB, 0.10064E+01_JPRB, 0.98095E+00_JPRB, 0.94018E+00_JPRB, 0.89363E+00_JPRB, 0.85939E+00_JPRB, 0.82914E+00_JPRB ,&
&  0.77617E+00_JPRB, 0.68186E+00_JPRB, 0.55924E+00_JPRB, 0.50623E+00_JPRB, 0.46256E+00_JPRB, 0.43223E+00_JPRB, 0.43593E+00_JPRB ,&
&  0.39383E+00_JPRB, 0.37899E+00_JPRB, 0.37351E+00_JPRB, 0.36275E+00_JPRB, 0.36907E+00_JPRB, 0.39048E+00_JPRB, 0.36333E+00_JPRB ,&
&  0.35293E+00_JPRB, 0.39524E+00_JPRB, 0.43543E+00_JPRB, 0.48406E+00_JPRB, 0.51066E+00_JPRB, 0.51460E+00_JPRB, 0.44261E+00_JPRB ,&
&  0.46698E+00_JPRB, 0.46116E+00_JPRB, 0.46995E+00_JPRB, 0.47033E+00_JPRB, 0.47882E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.45233E+00_JPRB, 0.46337E+00_JPRB, 0.47373E+00_JPRB, 0.48681E+00_JPRB, 0.50072E+00_JPRB, 0.49822E+00_JPRB, 0.49801E+00_JPRB ,&
&  0.50368E+00_JPRB, 0.49933E+00_JPRB, 0.48301E+00_JPRB, 0.47121E+00_JPRB, 0.45587E+00_JPRB, 0.45447E+00_JPRB, 0.40284E+00_JPRB ,&
&  0.35419E+00_JPRB, 0.39197E+00_JPRB, 0.38205E+00_JPRB, 0.40964E+00_JPRB, 0.45964E+00_JPRB, 0.45638E+00_JPRB, 0.46884E+00_JPRB ,&
&  0.47561E+00_JPRB, 0.46159E+00_JPRB, 0.46136E+00_JPRB, 0.48589E+00_JPRB, 0.50903E+00_JPRB, 0.56454E+00_JPRB, 0.63762E+00_JPRB ,&
&  0.57726E+00_JPRB, 0.46945E+00_JPRB, 0.42281E+00_JPRB, 0.42774E+00_JPRB, 0.48783E+00_JPRB, 0.54587E+00_JPRB, 0.58415E+00_JPRB ,&
&  0.61716E+00_JPRB, 0.65055E+00_JPRB, 0.69007E+00_JPRB, 0.73710E+00_JPRB, 0.76108E+00_JPRB, 0.78200E+00_JPRB, 0.80611E+00_JPRB ,&
&  0.83466E+00_JPRB, 0.88390E+00_JPRB, 0.95334E+00_JPRB, 0.10828E+01_JPRB, 0.12390E+01_JPRB, 0.19724E+01_JPRB, 0.29223E+01_JPRB ,&
&  0.31962E+01_JPRB, 0.34158E+01_JPRB, 0.37139E+01_JPRB, 0.40005E+01_JPRB, 0.39698E+01_JPRB, 0.34871E+01_JPRB, 0.29184E+01_JPRB ,&
&  0.24534E+01_JPRB, 0.20033E+01_JPRB, 0.16984E+01_JPRB, 0.14025E+01_JPRB, 0.11897E+01_JPRB, 0.10897E+01_JPRB, 0.99654E+00_JPRB ,&
&  0.89770E+00_JPRB, 0.82631E+00_JPRB, 0.79786E+00_JPRB, 0.78123E+00_JPRB, 0.73809E+00_JPRB, 0.71517E+00_JPRB, 0.68953E+00_JPRB ,&
&  0.66852E+00_JPRB, 0.63525E+00_JPRB, 0.56099E+00_JPRB, 0.50304E+00_JPRB, 0.45350E+00_JPRB, 0.43266E+00_JPRB, 0.40373E+00_JPRB ,&
&  0.38811E+00_JPRB, 0.37398E+00_JPRB, 0.35452E+00_JPRB, 0.33539E+00_JPRB, 0.32668E+00_JPRB, 0.33615E+00_JPRB, 0.33663E+00_JPRB ,&
&  0.36439E+00_JPRB, 0.40808E+00_JPRB, 0.40714E+00_JPRB, 0.41256E+00_JPRB, 0.42160E+00_JPRB, 0.44183E+00_JPRB, 0.36068E+00_JPRB ,&
&  0.37238E+00_JPRB, 0.42636E+00_JPRB, 0.42825E+00_JPRB, 0.42962E+00_JPRB, 0.43564E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.42629E+00_JPRB, 0.43980E+00_JPRB, 0.45397E+00_JPRB, 0.47690E+00_JPRB, 0.49564E+00_JPRB, 0.50206E+00_JPRB, 0.46690E+00_JPRB ,&
&  0.50606E+00_JPRB, 0.48564E+00_JPRB, 0.46756E+00_JPRB, 0.46565E+00_JPRB, 0.45891E+00_JPRB, 0.39301E+00_JPRB, 0.32487E+00_JPRB ,&
&  0.31257E+00_JPRB, 0.32977E+00_JPRB, 0.36418E+00_JPRB, 0.38401E+00_JPRB, 0.40740E+00_JPRB, 0.41356E+00_JPRB, 0.41823E+00_JPRB ,&
&  0.42037E+00_JPRB, 0.41052E+00_JPRB, 0.41891E+00_JPRB, 0.42848E+00_JPRB, 0.43603E+00_JPRB, 0.44946E+00_JPRB, 0.48657E+00_JPRB ,&
&  0.45829E+00_JPRB, 0.43241E+00_JPRB, 0.43513E+00_JPRB, 0.42136E+00_JPRB, 0.48066E+00_JPRB, 0.50588E+00_JPRB, 0.54195E+00_JPRB ,&
&  0.58222E+00_JPRB, 0.62151E+00_JPRB, 0.68542E+00_JPRB, 0.73286E+00_JPRB, 0.76438E+00_JPRB, 0.78370E+00_JPRB, 0.81413E+00_JPRB ,&
&  0.83428E+00_JPRB, 0.87376E+00_JPRB, 0.94994E+00_JPRB, 0.10959E+01_JPRB, 0.13422E+01_JPRB, 0.18047E+01_JPRB, 0.26323E+01_JPRB ,&
&  0.29087E+01_JPRB, 0.32539E+01_JPRB, 0.35112E+01_JPRB, 0.30579E+01_JPRB, 0.31298E+01_JPRB, 0.31384E+01_JPRB, 0.28059E+01_JPRB ,&
&  0.25004E+01_JPRB, 0.20043E+01_JPRB, 0.16361E+01_JPRB, 0.13778E+01_JPRB, 0.11532E+01_JPRB, 0.94906E+00_JPRB, 0.81505E+00_JPRB ,&
&  0.75482E+00_JPRB, 0.69986E+00_JPRB, 0.61123E+00_JPRB, 0.57841E+00_JPRB, 0.55559E+00_JPRB, 0.54763E+00_JPRB, 0.53755E+00_JPRB ,&
&  0.52771E+00_JPRB, 0.51752E+00_JPRB, 0.49100E+00_JPRB, 0.45975E+00_JPRB, 0.42099E+00_JPRB, 0.40614E+00_JPRB, 0.39515E+00_JPRB ,&
&  0.39317E+00_JPRB, 0.37725E+00_JPRB, 0.34393E+00_JPRB, 0.33004E+00_JPRB, 0.32021E+00_JPRB, 0.31587E+00_JPRB, 0.31871E+00_JPRB ,&
&  0.33498E+00_JPRB, 0.34956E+00_JPRB, 0.39471E+00_JPRB, 0.39967E+00_JPRB, 0.40592E+00_JPRB, 0.43141E+00_JPRB, 0.38524E+00_JPRB ,&
&  0.36596E+00_JPRB, 0.42625E+00_JPRB, 0.41577E+00_JPRB, 0.40962E+00_JPRB, 0.41374E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.40826E+00_JPRB, 0.43213E+00_JPRB, 0.45064E+00_JPRB, 0.47214E+00_JPRB, 0.49550E+00_JPRB, 0.45644E+00_JPRB, 0.42888E+00_JPRB ,&
&  0.49248E+00_JPRB, 0.41833E+00_JPRB, 0.46463E+00_JPRB, 0.37709E+00_JPRB, 0.33765E+00_JPRB, 0.32601E+00_JPRB, 0.31810E+00_JPRB ,&
&  0.31252E+00_JPRB, 0.33171E+00_JPRB, 0.34937E+00_JPRB, 0.36213E+00_JPRB, 0.39053E+00_JPRB, 0.39311E+00_JPRB, 0.38786E+00_JPRB ,&
&  0.37855E+00_JPRB, 0.36893E+00_JPRB, 0.37525E+00_JPRB, 0.37893E+00_JPRB, 0.39347E+00_JPRB, 0.40986E+00_JPRB, 0.42216E+00_JPRB ,&
&  0.40356E+00_JPRB, 0.40700E+00_JPRB, 0.43458E+00_JPRB, 0.45471E+00_JPRB, 0.46868E+00_JPRB, 0.47880E+00_JPRB, 0.50525E+00_JPRB ,&
&  0.53834E+00_JPRB, 0.55234E+00_JPRB, 0.70371E+00_JPRB, 0.74206E+00_JPRB, 0.78425E+00_JPRB, 0.81075E+00_JPRB, 0.84255E+00_JPRB ,&
&  0.88247E+00_JPRB, 0.93226E+00_JPRB, 0.98499E+00_JPRB, 0.10656E+01_JPRB, 0.12341E+01_JPRB, 0.16187E+01_JPRB, 0.20755E+01_JPRB ,&
&  0.28302E+01_JPRB, 0.21120E+01_JPRB, 0.25544E+01_JPRB, 0.24324E+01_JPRB, 0.26092E+01_JPRB, 0.26973E+01_JPRB, 0.25136E+01_JPRB ,&
&  0.22540E+01_JPRB, 0.17984E+01_JPRB, 0.14097E+01_JPRB, 0.11242E+01_JPRB, 0.92492E+00_JPRB, 0.82351E+00_JPRB, 0.72991E+00_JPRB ,&
&  0.66024E+00_JPRB, 0.59639E+00_JPRB, 0.52712E+00_JPRB, 0.49459E+00_JPRB, 0.49458E+00_JPRB, 0.49471E+00_JPRB, 0.47297E+00_JPRB ,&
&  0.45591E+00_JPRB, 0.44258E+00_JPRB, 0.42250E+00_JPRB, 0.40740E+00_JPRB, 0.39617E+00_JPRB, 0.38705E+00_JPRB, 0.38243E+00_JPRB ,&
&  0.38263E+00_JPRB, 0.37713E+00_JPRB, 0.35588E+00_JPRB, 0.34609E+00_JPRB, 0.34023E+00_JPRB, 0.32718E+00_JPRB, 0.31698E+00_JPRB ,&
&  0.30961E+00_JPRB, 0.30334E+00_JPRB, 0.31526E+00_JPRB, 0.34038E+00_JPRB, 0.34699E+00_JPRB, 0.35473E+00_JPRB, 0.39340E+00_JPRB ,&
&  0.37433E+00_JPRB, 0.39284E+00_JPRB, 0.39522E+00_JPRB, 0.38370E+00_JPRB, 0.38756E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.38727E+00_JPRB, 0.41731E+00_JPRB, 0.44544E+00_JPRB, 0.46663E+00_JPRB, 0.48893E+00_JPRB, 0.44469E+00_JPRB, 0.40174E+00_JPRB ,&
&  0.42938E+00_JPRB, 0.37290E+00_JPRB, 0.34151E+00_JPRB, 0.32833E+00_JPRB, 0.32545E+00_JPRB, 0.31873E+00_JPRB, 0.30963E+00_JPRB ,&
&  0.30749E+00_JPRB, 0.31584E+00_JPRB, 0.33298E+00_JPRB, 0.34580E+00_JPRB, 0.35750E+00_JPRB, 0.36290E+00_JPRB, 0.35976E+00_JPRB ,&
&  0.35512E+00_JPRB, 0.35222E+00_JPRB, 0.35121E+00_JPRB, 0.35420E+00_JPRB, 0.36003E+00_JPRB, 0.37017E+00_JPRB, 0.37780E+00_JPRB ,&
&  0.38223E+00_JPRB, 0.39096E+00_JPRB, 0.40624E+00_JPRB, 0.43280E+00_JPRB, 0.43833E+00_JPRB, 0.44253E+00_JPRB, 0.44857E+00_JPRB ,&
&  0.37000E+00_JPRB, 0.43187E+00_JPRB, 0.66620E+00_JPRB, 0.73144E+00_JPRB, 0.77196E+00_JPRB, 0.82785E+00_JPRB, 0.87990E+00_JPRB ,&
&  0.93209E+00_JPRB, 0.10030E+01_JPRB, 0.10927E+01_JPRB, 0.11897E+01_JPRB, 0.12967E+01_JPRB, 0.14686E+01_JPRB, 0.17230E+01_JPRB ,&
&  0.23231E+01_JPRB, 0.18837E+01_JPRB, 0.17224E+01_JPRB, 0.17294E+01_JPRB, 0.23324E+01_JPRB, 0.22047E+01_JPRB, 0.18262E+01_JPRB ,&
&  0.15350E+01_JPRB, 0.13404E+01_JPRB, 0.11791E+01_JPRB, 0.10207E+01_JPRB, 0.90375E+00_JPRB, 0.84181E+00_JPRB, 0.75311E+00_JPRB ,&
&  0.65154E+00_JPRB, 0.58669E+00_JPRB, 0.52829E+00_JPRB, 0.50970E+00_JPRB, 0.51136E+00_JPRB, 0.48693E+00_JPRB, 0.45251E+00_JPRB ,&
&  0.42194E+00_JPRB, 0.39964E+00_JPRB, 0.38290E+00_JPRB, 0.36955E+00_JPRB, 0.36024E+00_JPRB, 0.35666E+00_JPRB, 0.35794E+00_JPRB ,&
&  0.36188E+00_JPRB, 0.36545E+00_JPRB, 0.35989E+00_JPRB, 0.35707E+00_JPRB, 0.34822E+00_JPRB, 0.32827E+00_JPRB, 0.30942E+00_JPRB ,&
&  0.30469E+00_JPRB, 0.29378E+00_JPRB, 0.29174E+00_JPRB, 0.29768E+00_JPRB, 0.30400E+00_JPRB, 0.31479E+00_JPRB, 0.33151E+00_JPRB ,&
&  0.34712E+00_JPRB, 0.35425E+00_JPRB, 0.33897E+00_JPRB, 0.33424E+00_JPRB, 0.34717E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.36630E+00_JPRB, 0.40754E+00_JPRB, 0.43999E+00_JPRB, 0.46571E+00_JPRB, 0.46498E+00_JPRB, 0.44652E+00_JPRB, 0.39827E+00_JPRB ,&
&  0.38927E+00_JPRB, 0.36549E+00_JPRB, 0.35269E+00_JPRB, 0.33805E+00_JPRB, 0.32781E+00_JPRB, 0.32039E+00_JPRB, 0.31841E+00_JPRB ,&
&  0.32304E+00_JPRB, 0.33150E+00_JPRB, 0.33885E+00_JPRB, 0.34274E+00_JPRB, 0.34521E+00_JPRB, 0.34564E+00_JPRB, 0.34527E+00_JPRB ,&
&  0.34412E+00_JPRB, 0.34283E+00_JPRB, 0.34296E+00_JPRB, 0.34537E+00_JPRB, 0.35211E+00_JPRB, 0.36029E+00_JPRB, 0.36247E+00_JPRB ,&
&  0.35946E+00_JPRB, 0.35791E+00_JPRB, 0.37431E+00_JPRB, 0.39968E+00_JPRB, 0.39995E+00_JPRB, 0.40303E+00_JPRB, 0.36745E+00_JPRB ,&
&  0.33199E+00_JPRB, 0.34879E+00_JPRB, 0.43815E+00_JPRB, 0.63867E+00_JPRB, 0.70176E+00_JPRB, 0.76562E+00_JPRB, 0.81136E+00_JPRB ,&
&  0.77358E+00_JPRB, 0.80771E+00_JPRB, 0.91650E+00_JPRB, 0.11736E+01_JPRB, 0.12449E+01_JPRB, 0.13571E+01_JPRB, 0.14706E+01_JPRB ,&
&  0.17168E+01_JPRB, 0.18631E+01_JPRB, 0.18893E+01_JPRB, 0.14647E+01_JPRB, 0.15245E+01_JPRB, 0.17658E+01_JPRB, 0.14468E+01_JPRB ,&
&  0.12633E+01_JPRB, 0.11512E+01_JPRB, 0.11127E+01_JPRB, 0.97609E+00_JPRB, 0.86490E+00_JPRB, 0.78134E+00_JPRB, 0.72370E+00_JPRB ,&
&  0.67394E+00_JPRB, 0.62463E+00_JPRB, 0.54839E+00_JPRB, 0.50208E+00_JPRB, 0.49287E+00_JPRB, 0.47897E+00_JPRB, 0.46332E+00_JPRB ,&
&  0.43771E+00_JPRB, 0.40235E+00_JPRB, 0.36900E+00_JPRB, 0.34089E+00_JPRB, 0.32330E+00_JPRB, 0.32711E+00_JPRB, 0.34117E+00_JPRB ,&
&  0.35392E+00_JPRB, 0.35708E+00_JPRB, 0.35704E+00_JPRB, 0.35208E+00_JPRB, 0.33921E+00_JPRB, 0.32052E+00_JPRB, 0.30809E+00_JPRB ,&
&  0.29974E+00_JPRB, 0.29139E+00_JPRB, 0.28949E+00_JPRB, 0.29391E+00_JPRB, 0.30002E+00_JPRB, 0.30952E+00_JPRB, 0.32079E+00_JPRB ,&
&  0.32551E+00_JPRB, 0.33225E+00_JPRB, 0.32959E+00_JPRB, 0.32970E+00_JPRB, 0.33492E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.33405E+00_JPRB, 0.37248E+00_JPRB, 0.42854E+00_JPRB, 0.46125E+00_JPRB, 0.46691E+00_JPRB, 0.43143E+00_JPRB, 0.39354E+00_JPRB ,&
&  0.37321E+00_JPRB, 0.36111E+00_JPRB, 0.35855E+00_JPRB, 0.35655E+00_JPRB, 0.35584E+00_JPRB, 0.35614E+00_JPRB, 0.35722E+00_JPRB ,&
&  0.35645E+00_JPRB, 0.35426E+00_JPRB, 0.35188E+00_JPRB, 0.35124E+00_JPRB, 0.35254E+00_JPRB, 0.35273E+00_JPRB, 0.35268E+00_JPRB ,&
&  0.35295E+00_JPRB, 0.35248E+00_JPRB, 0.35078E+00_JPRB, 0.35009E+00_JPRB, 0.35268E+00_JPRB, 0.35899E+00_JPRB, 0.36198E+00_JPRB ,&
&  0.35605E+00_JPRB, 0.34761E+00_JPRB, 0.35687E+00_JPRB, 0.37684E+00_JPRB, 0.37798E+00_JPRB, 0.37906E+00_JPRB, 0.34957E+00_JPRB ,&
&  0.32396E+00_JPRB, 0.30741E+00_JPRB, 0.31725E+00_JPRB, 0.36712E+00_JPRB, 0.45824E+00_JPRB, 0.62996E+00_JPRB, 0.71429E+00_JPRB ,&
&  0.79687E+00_JPRB, 0.86310E+00_JPRB, 0.92577E+00_JPRB, 0.98970E+00_JPRB, 0.10734E+01_JPRB, 0.11492E+01_JPRB, 0.11903E+01_JPRB ,&
&  0.12534E+01_JPRB, 0.13331E+01_JPRB, 0.14514E+01_JPRB, 0.10805E+01_JPRB, 0.98649E+00_JPRB, 0.10255E+01_JPRB, 0.10719E+01_JPRB ,&
&  0.10406E+01_JPRB, 0.92709E+00_JPRB, 0.90307E+00_JPRB, 0.87952E+00_JPRB, 0.79436E+00_JPRB, 0.72060E+00_JPRB, 0.67067E+00_JPRB ,&
&  0.63071E+00_JPRB, 0.60528E+00_JPRB, 0.54973E+00_JPRB, 0.48263E+00_JPRB, 0.45735E+00_JPRB, 0.45156E+00_JPRB, 0.44733E+00_JPRB ,&
&  0.44627E+00_JPRB, 0.42784E+00_JPRB, 0.37040E+00_JPRB, 0.32015E+00_JPRB, 0.31405E+00_JPRB, 0.32412E+00_JPRB, 0.34177E+00_JPRB ,&
&  0.35165E+00_JPRB, 0.35057E+00_JPRB, 0.35041E+00_JPRB, 0.34630E+00_JPRB, 0.33940E+00_JPRB, 0.32610E+00_JPRB, 0.31409E+00_JPRB ,&
&  0.30385E+00_JPRB, 0.30202E+00_JPRB, 0.30914E+00_JPRB, 0.31239E+00_JPRB, 0.32296E+00_JPRB, 0.33320E+00_JPRB, 0.34386E+00_JPRB ,&
&  0.34165E+00_JPRB, 0.33928E+00_JPRB, 0.33572E+00_JPRB, 0.32547E+00_JPRB, 0.31975E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.34515E+00_JPRB, 0.37711E+00_JPRB, 0.41976E+00_JPRB, 0.45008E+00_JPRB, 0.45535E+00_JPRB, 0.43320E+00_JPRB, 0.41441E+00_JPRB ,&
&  0.40413E+00_JPRB, 0.40170E+00_JPRB, 0.39919E+00_JPRB, 0.39546E+00_JPRB, 0.39155E+00_JPRB, 0.38739E+00_JPRB, 0.38327E+00_JPRB ,&
&  0.38010E+00_JPRB, 0.37784E+00_JPRB, 0.37489E+00_JPRB, 0.36929E+00_JPRB, 0.36546E+00_JPRB, 0.36514E+00_JPRB, 0.36788E+00_JPRB ,&
&  0.37165E+00_JPRB, 0.37123E+00_JPRB, 0.37055E+00_JPRB, 0.36667E+00_JPRB, 0.35798E+00_JPRB, 0.34800E+00_JPRB, 0.33986E+00_JPRB ,&
&  0.33520E+00_JPRB, 0.33931E+00_JPRB, 0.34802E+00_JPRB, 0.36424E+00_JPRB, 0.36495E+00_JPRB, 0.36382E+00_JPRB, 0.33603E+00_JPRB ,&
&  0.31658E+00_JPRB, 0.29489E+00_JPRB, 0.28660E+00_JPRB, 0.29527E+00_JPRB, 0.32546E+00_JPRB, 0.37340E+00_JPRB, 0.45268E+00_JPRB ,&
&  0.59795E+00_JPRB, 0.65662E+00_JPRB, 0.70380E+00_JPRB, 0.76028E+00_JPRB, 0.81600E+00_JPRB, 0.86874E+00_JPRB, 0.90899E+00_JPRB ,&
&  0.96061E+00_JPRB, 0.10173E+01_JPRB, 0.10730E+01_JPRB, 0.10408E+01_JPRB, 0.10231E+01_JPRB, 0.92058E+00_JPRB, 0.98052E+00_JPRB ,&
&  0.11587E+01_JPRB, 0.10079E+01_JPRB, 0.87922E+00_JPRB, 0.78217E+00_JPRB, 0.71106E+00_JPRB, 0.64870E+00_JPRB, 0.60121E+00_JPRB ,&
&  0.56409E+00_JPRB, 0.54294E+00_JPRB, 0.52151E+00_JPRB, 0.47463E+00_JPRB, 0.43651E+00_JPRB, 0.42001E+00_JPRB, 0.41328E+00_JPRB ,&
&  0.41653E+00_JPRB, 0.40309E+00_JPRB, 0.39674E+00_JPRB, 0.35705E+00_JPRB, 0.36153E+00_JPRB, 0.37003E+00_JPRB, 0.37745E+00_JPRB ,&
&  0.37374E+00_JPRB, 0.36532E+00_JPRB, 0.36230E+00_JPRB, 0.35913E+00_JPRB, 0.35348E+00_JPRB, 0.34529E+00_JPRB, 0.33872E+00_JPRB ,&
&  0.33449E+00_JPRB, 0.33820E+00_JPRB, 0.34698E+00_JPRB, 0.35395E+00_JPRB, 0.35799E+00_JPRB, 0.35972E+00_JPRB, 0.36193E+00_JPRB ,&
&  0.36122E+00_JPRB, 0.35811E+00_JPRB, 0.35408E+00_JPRB, 0.34438E+00_JPRB, 0.33785E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.36351E+00_JPRB, 0.38785E+00_JPRB, 0.41600E+00_JPRB, 0.43491E+00_JPRB, 0.44524E+00_JPRB, 0.43484E+00_JPRB, 0.42351E+00_JPRB ,&
&  0.41420E+00_JPRB, 0.40718E+00_JPRB, 0.40260E+00_JPRB, 0.39929E+00_JPRB, 0.39511E+00_JPRB, 0.39107E+00_JPRB, 0.38720E+00_JPRB ,&
&  0.38416E+00_JPRB, 0.37645E+00_JPRB, 0.37230E+00_JPRB, 0.37254E+00_JPRB, 0.37383E+00_JPRB, 0.37717E+00_JPRB, 0.38175E+00_JPRB ,&
&  0.38200E+00_JPRB, 0.37758E+00_JPRB, 0.37417E+00_JPRB, 0.36253E+00_JPRB, 0.34861E+00_JPRB, 0.34036E+00_JPRB, 0.34325E+00_JPRB ,&
&  0.34315E+00_JPRB, 0.33544E+00_JPRB, 0.35051E+00_JPRB, 0.36452E+00_JPRB, 0.35794E+00_JPRB, 0.35246E+00_JPRB, 0.30607E+00_JPRB ,&
&  0.29278E+00_JPRB, 0.28044E+00_JPRB, 0.28014E+00_JPRB, 0.28454E+00_JPRB, 0.30339E+00_JPRB, 0.34016E+00_JPRB, 0.39553E+00_JPRB ,&
&  0.45555E+00_JPRB, 0.55521E+00_JPRB, 0.58007E+00_JPRB, 0.62798E+00_JPRB, 0.67708E+00_JPRB, 0.72464E+00_JPRB, 0.76340E+00_JPRB ,&
&  0.81680E+00_JPRB, 0.88640E+00_JPRB, 0.92825E+00_JPRB, 0.93508E+00_JPRB, 0.94403E+00_JPRB, 0.94013E+00_JPRB, 0.95055E+00_JPRB ,&
&  0.96722E+00_JPRB, 0.82210E+00_JPRB, 0.72323E+00_JPRB, 0.65001E+00_JPRB, 0.60205E+00_JPRB, 0.57911E+00_JPRB, 0.53371E+00_JPRB ,&
&  0.51089E+00_JPRB, 0.49784E+00_JPRB, 0.48112E+00_JPRB, 0.47225E+00_JPRB, 0.46004E+00_JPRB, 0.44123E+00_JPRB, 0.41198E+00_JPRB ,&
&  0.38312E+00_JPRB, 0.36383E+00_JPRB, 0.37329E+00_JPRB, 0.41559E+00_JPRB, 0.42842E+00_JPRB, 0.41993E+00_JPRB, 0.40025E+00_JPRB ,&
&  0.37751E+00_JPRB, 0.36488E+00_JPRB, 0.36322E+00_JPRB, 0.36371E+00_JPRB, 0.36422E+00_JPRB, 0.36307E+00_JPRB, 0.36780E+00_JPRB ,&
&  0.36799E+00_JPRB, 0.37394E+00_JPRB, 0.38077E+00_JPRB, 0.38443E+00_JPRB, 0.38351E+00_JPRB, 0.37959E+00_JPRB, 0.37599E+00_JPRB ,&
&  0.37222E+00_JPRB, 0.36808E+00_JPRB, 0.36429E+00_JPRB, 0.35925E+00_JPRB, 0.35631E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.37925E+00_JPRB, 0.39237E+00_JPRB, 0.41147E+00_JPRB, 0.42581E+00_JPRB, 0.43694E+00_JPRB, 0.43478E+00_JPRB, 0.42638E+00_JPRB ,&
&  0.41796E+00_JPRB, 0.41131E+00_JPRB, 0.40531E+00_JPRB, 0.39921E+00_JPRB, 0.39208E+00_JPRB, 0.38644E+00_JPRB, 0.38219E+00_JPRB ,&
&  0.38010E+00_JPRB, 0.37944E+00_JPRB, 0.37962E+00_JPRB, 0.37982E+00_JPRB, 0.38068E+00_JPRB, 0.38235E+00_JPRB, 0.38414E+00_JPRB ,&
&  0.38618E+00_JPRB, 0.38281E+00_JPRB, 0.36952E+00_JPRB, 0.35406E+00_JPRB, 0.34715E+00_JPRB, 0.34038E+00_JPRB, 0.34645E+00_JPRB ,&
&  0.34805E+00_JPRB, 0.35130E+00_JPRB, 0.35281E+00_JPRB, 0.35572E+00_JPRB, 0.34713E+00_JPRB, 0.29099E+00_JPRB, 0.29159E+00_JPRB ,&
&  0.28427E+00_JPRB, 0.27734E+00_JPRB, 0.27993E+00_JPRB, 0.28467E+00_JPRB, 0.29496E+00_JPRB, 0.31767E+00_JPRB, 0.35607E+00_JPRB ,&
&  0.39314E+00_JPRB, 0.52442E+00_JPRB, 0.55857E+00_JPRB, 0.59391E+00_JPRB, 0.62513E+00_JPRB, 0.66575E+00_JPRB, 0.71519E+00_JPRB ,&
&  0.75041E+00_JPRB, 0.79056E+00_JPRB, 0.81819E+00_JPRB, 0.83953E+00_JPRB, 0.83985E+00_JPRB, 0.82679E+00_JPRB, 0.81170E+00_JPRB ,&
&  0.77929E+00_JPRB, 0.69640E+00_JPRB, 0.63371E+00_JPRB, 0.59183E+00_JPRB, 0.55353E+00_JPRB, 0.52672E+00_JPRB, 0.50605E+00_JPRB ,&
&  0.49963E+00_JPRB, 0.47548E+00_JPRB, 0.46320E+00_JPRB, 0.44354E+00_JPRB, 0.42839E+00_JPRB, 0.41189E+00_JPRB, 0.39215E+00_JPRB ,&
&  0.37543E+00_JPRB, 0.37080E+00_JPRB, 0.37836E+00_JPRB, 0.40135E+00_JPRB, 0.41073E+00_JPRB, 0.40912E+00_JPRB, 0.40412E+00_JPRB ,&
&  0.39176E+00_JPRB, 0.38860E+00_JPRB, 0.38766E+00_JPRB, 0.38989E+00_JPRB, 0.39092E+00_JPRB, 0.39395E+00_JPRB, 0.39771E+00_JPRB ,&
&  0.40143E+00_JPRB, 0.40280E+00_JPRB, 0.40350E+00_JPRB, 0.40447E+00_JPRB, 0.40428E+00_JPRB, 0.40237E+00_JPRB, 0.39840E+00_JPRB ,&
&  0.39395E+00_JPRB, 0.39086E+00_JPRB, 0.38477E+00_JPRB, 0.37922E+00_JPRB, 0.37644E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.39325E+00_JPRB, 0.39764E+00_JPRB, 0.40461E+00_JPRB, 0.40991E+00_JPRB, 0.41338E+00_JPRB, 0.41264E+00_JPRB, 0.40672E+00_JPRB ,&
&  0.40108E+00_JPRB, 0.39579E+00_JPRB, 0.39157E+00_JPRB, 0.38826E+00_JPRB, 0.38658E+00_JPRB, 0.38391E+00_JPRB, 0.38089E+00_JPRB ,&
&  0.37868E+00_JPRB, 0.37760E+00_JPRB, 0.37748E+00_JPRB, 0.37742E+00_JPRB, 0.37866E+00_JPRB, 0.37866E+00_JPRB, 0.37979E+00_JPRB ,&
&  0.38073E+00_JPRB, 0.37431E+00_JPRB, 0.35559E+00_JPRB, 0.34535E+00_JPRB, 0.33794E+00_JPRB, 0.33914E+00_JPRB, 0.34216E+00_JPRB ,&
&  0.34695E+00_JPRB, 0.30759E+00_JPRB, 0.31000E+00_JPRB, 0.30010E+00_JPRB, 0.29257E+00_JPRB, 0.29169E+00_JPRB, 0.28707E+00_JPRB ,&
&  0.28468E+00_JPRB, 0.28184E+00_JPRB, 0.28580E+00_JPRB, 0.29349E+00_JPRB, 0.30026E+00_JPRB, 0.31481E+00_JPRB, 0.33854E+00_JPRB ,&
&  0.37656E+00_JPRB, 0.49047E+00_JPRB, 0.53882E+00_JPRB, 0.57152E+00_JPRB, 0.59567E+00_JPRB, 0.62528E+00_JPRB, 0.65449E+00_JPRB ,&
&  0.67686E+00_JPRB, 0.70525E+00_JPRB, 0.71808E+00_JPRB, 0.71225E+00_JPRB, 0.71884E+00_JPRB, 0.70525E+00_JPRB, 0.67771E+00_JPRB ,&
&  0.63657E+00_JPRB, 0.58635E+00_JPRB, 0.54872E+00_JPRB, 0.52270E+00_JPRB, 0.50019E+00_JPRB, 0.48070E+00_JPRB, 0.46682E+00_JPRB ,&
&  0.46420E+00_JPRB, 0.44999E+00_JPRB, 0.43314E+00_JPRB, 0.41003E+00_JPRB, 0.39157E+00_JPRB, 0.37980E+00_JPRB, 0.37436E+00_JPRB ,&
&  0.37313E+00_JPRB, 0.37971E+00_JPRB, 0.38982E+00_JPRB, 0.40144E+00_JPRB, 0.40432E+00_JPRB, 0.40343E+00_JPRB, 0.40543E+00_JPRB ,&
&  0.40424E+00_JPRB, 0.40358E+00_JPRB, 0.40354E+00_JPRB, 0.40376E+00_JPRB, 0.40203E+00_JPRB, 0.39948E+00_JPRB, 0.39900E+00_JPRB ,&
&  0.39993E+00_JPRB, 0.40082E+00_JPRB, 0.40195E+00_JPRB, 0.40393E+00_JPRB, 0.40773E+00_JPRB, 0.41100E+00_JPRB, 0.41181E+00_JPRB ,&
&  0.40554E+00_JPRB, 0.39938E+00_JPRB, 0.39340E+00_JPRB, 0.39190E+00_JPRB, 0.39164E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.37856E+00_JPRB, 0.37954E+00_JPRB, 0.38090E+00_JPRB, 0.38184E+00_JPRB, 0.38199E+00_JPRB, 0.38201E+00_JPRB, 0.38155E+00_JPRB ,&
&  0.38120E+00_JPRB, 0.38141E+00_JPRB, 0.38140E+00_JPRB, 0.38078E+00_JPRB, 0.38097E+00_JPRB, 0.38153E+00_JPRB, 0.38153E+00_JPRB ,&
&  0.37948E+00_JPRB, 0.37675E+00_JPRB, 0.37434E+00_JPRB, 0.37128E+00_JPRB, 0.36858E+00_JPRB, 0.36670E+00_JPRB, 0.36388E+00_JPRB ,&
&  0.36217E+00_JPRB, 0.35727E+00_JPRB, 0.33456E+00_JPRB, 0.33355E+00_JPRB, 0.32868E+00_JPRB, 0.32803E+00_JPRB, 0.33140E+00_JPRB ,&
&  0.32828E+00_JPRB, 0.32921E+00_JPRB, 0.32386E+00_JPRB, 0.31355E+00_JPRB, 0.30631E+00_JPRB, 0.29591E+00_JPRB, 0.29504E+00_JPRB ,&
&  0.29354E+00_JPRB, 0.29685E+00_JPRB, 0.30559E+00_JPRB, 0.31122E+00_JPRB, 0.31589E+00_JPRB, 0.32204E+00_JPRB, 0.33727E+00_JPRB ,&
&  0.36912E+00_JPRB, 0.41413E+00_JPRB, 0.47089E+00_JPRB, 0.49851E+00_JPRB, 0.52136E+00_JPRB, 0.54482E+00_JPRB, 0.56653E+00_JPRB ,&
&  0.58270E+00_JPRB, 0.59575E+00_JPRB, 0.59708E+00_JPRB, 0.59481E+00_JPRB, 0.58138E+00_JPRB, 0.55786E+00_JPRB, 0.53109E+00_JPRB ,&
&  0.50424E+00_JPRB, 0.48467E+00_JPRB, 0.46903E+00_JPRB, 0.45768E+00_JPRB, 0.44751E+00_JPRB, 0.43525E+00_JPRB, 0.42457E+00_JPRB ,&
&  0.41229E+00_JPRB, 0.39470E+00_JPRB, 0.37806E+00_JPRB, 0.36332E+00_JPRB, 0.35526E+00_JPRB, 0.35085E+00_JPRB, 0.35180E+00_JPRB ,&
&  0.35445E+00_JPRB, 0.36072E+00_JPRB, 0.36747E+00_JPRB, 0.37079E+00_JPRB, 0.37475E+00_JPRB, 0.37964E+00_JPRB, 0.38013E+00_JPRB ,&
&  0.38104E+00_JPRB, 0.38260E+00_JPRB, 0.38267E+00_JPRB, 0.38220E+00_JPRB, 0.37995E+00_JPRB, 0.37701E+00_JPRB, 0.37433E+00_JPRB ,&
&  0.37292E+00_JPRB, 0.37307E+00_JPRB, 0.37595E+00_JPRB, 0.37917E+00_JPRB, 0.38315E+00_JPRB, 0.38477E+00_JPRB, 0.38298E+00_JPRB ,&
&  0.37799E+00_JPRB, 0.37545E+00_JPRB, 0.37361E+00_JPRB, 0.37517E+00_JPRB, 0.37698E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.36367E+00_JPRB, 0.36566E+00_JPRB, 0.36701E+00_JPRB, 0.36775E+00_JPRB, 0.36823E+00_JPRB, 0.36909E+00_JPRB, 0.36974E+00_JPRB ,&
&  0.37039E+00_JPRB, 0.37078E+00_JPRB, 0.37134E+00_JPRB, 0.37165E+00_JPRB, 0.37168E+00_JPRB, 0.37405E+00_JPRB, 0.37465E+00_JPRB ,&
&  0.37408E+00_JPRB, 0.37284E+00_JPRB, 0.37161E+00_JPRB, 0.37002E+00_JPRB, 0.36804E+00_JPRB, 0.36540E+00_JPRB, 0.36052E+00_JPRB ,&
&  0.35797E+00_JPRB, 0.35478E+00_JPRB, 0.35099E+00_JPRB, 0.35065E+00_JPRB, 0.34960E+00_JPRB, 0.33416E+00_JPRB, 0.33364E+00_JPRB ,&
&  0.34379E+00_JPRB, 0.34917E+00_JPRB, 0.35250E+00_JPRB, 0.35658E+00_JPRB, 0.34899E+00_JPRB, 0.33340E+00_JPRB, 0.32240E+00_JPRB ,&
&  0.30359E+00_JPRB, 0.31806E+00_JPRB, 0.32816E+00_JPRB, 0.33295E+00_JPRB, 0.33570E+00_JPRB, 0.34014E+00_JPRB, 0.36222E+00_JPRB ,&
&  0.40703E+00_JPRB, 0.41688E+00_JPRB, 0.42774E+00_JPRB, 0.43959E+00_JPRB, 0.45168E+00_JPRB, 0.46034E+00_JPRB, 0.46981E+00_JPRB ,&
&  0.47548E+00_JPRB, 0.47919E+00_JPRB, 0.47807E+00_JPRB, 0.47457E+00_JPRB, 0.46516E+00_JPRB, 0.45572E+00_JPRB, 0.44583E+00_JPRB ,&
&  0.43731E+00_JPRB, 0.42907E+00_JPRB, 0.42011E+00_JPRB, 0.40947E+00_JPRB, 0.39846E+00_JPRB, 0.38760E+00_JPRB, 0.37676E+00_JPRB ,&
&  0.36550E+00_JPRB, 0.35442E+00_JPRB, 0.34671E+00_JPRB, 0.34049E+00_JPRB, 0.33829E+00_JPRB, 0.33750E+00_JPRB, 0.33931E+00_JPRB ,&
&  0.34088E+00_JPRB, 0.34347E+00_JPRB, 0.34625E+00_JPRB, 0.35029E+00_JPRB, 0.35328E+00_JPRB, 0.35589E+00_JPRB, 0.35850E+00_JPRB ,&
&  0.36105E+00_JPRB, 0.36337E+00_JPRB, 0.36527E+00_JPRB, 0.36681E+00_JPRB, 0.36778E+00_JPRB, 0.36851E+00_JPRB, 0.36843E+00_JPRB ,&
&  0.36827E+00_JPRB, 0.36799E+00_JPRB, 0.36756E+00_JPRB, 0.36672E+00_JPRB, 0.36534E+00_JPRB, 0.36265E+00_JPRB, 0.35987E+00_JPRB ,&
&  0.35746E+00_JPRB, 0.35687E+00_JPRB, 0.35704E+00_JPRB, 0.35886E+00_JPRB, 0.36116E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.36594E+00_JPRB, 0.36721E+00_JPRB, 0.36811E+00_JPRB, 0.36856E+00_JPRB, 0.36885E+00_JPRB, 0.36906E+00_JPRB, 0.36879E+00_JPRB ,&
&  0.36799E+00_JPRB, 0.36736E+00_JPRB, 0.36718E+00_JPRB, 0.36680E+00_JPRB, 0.36599E+00_JPRB, 0.36675E+00_JPRB, 0.36862E+00_JPRB ,&
&  0.36864E+00_JPRB, 0.36838E+00_JPRB, 0.36815E+00_JPRB, 0.36823E+00_JPRB, 0.36823E+00_JPRB, 0.36797E+00_JPRB, 0.36800E+00_JPRB ,&
&  0.36848E+00_JPRB, 0.36875E+00_JPRB, 0.36880E+00_JPRB, 0.36828E+00_JPRB, 0.36916E+00_JPRB, 0.36997E+00_JPRB, 0.37034E+00_JPRB ,&
&  0.36998E+00_JPRB, 0.36908E+00_JPRB, 0.36832E+00_JPRB, 0.36828E+00_JPRB, 0.37076E+00_JPRB, 0.37400E+00_JPRB, 0.37843E+00_JPRB ,&
&  0.38387E+00_JPRB, 0.39022E+00_JPRB, 0.39647E+00_JPRB, 0.40224E+00_JPRB, 0.40686E+00_JPRB, 0.41204E+00_JPRB, 0.41569E+00_JPRB ,&
&  0.41694E+00_JPRB, 0.41608E+00_JPRB, 0.41475E+00_JPRB, 0.41455E+00_JPRB, 0.41447E+00_JPRB, 0.41406E+00_JPRB, 0.41364E+00_JPRB ,&
&  0.41322E+00_JPRB, 0.41271E+00_JPRB, 0.41221E+00_JPRB, 0.41055E+00_JPRB, 0.40783E+00_JPRB, 0.40490E+00_JPRB, 0.40151E+00_JPRB ,&
&  0.39773E+00_JPRB, 0.39306E+00_JPRB, 0.38798E+00_JPRB, 0.38218E+00_JPRB, 0.37615E+00_JPRB, 0.37001E+00_JPRB, 0.36433E+00_JPRB ,&
&  0.36004E+00_JPRB, 0.35747E+00_JPRB, 0.35571E+00_JPRB, 0.35337E+00_JPRB, 0.35150E+00_JPRB, 0.35035E+00_JPRB, 0.35064E+00_JPRB ,&
&  0.35115E+00_JPRB, 0.35252E+00_JPRB, 0.35424E+00_JPRB, 0.35660E+00_JPRB, 0.35844E+00_JPRB, 0.36009E+00_JPRB, 0.36153E+00_JPRB ,&
&  0.36274E+00_JPRB, 0.36385E+00_JPRB, 0.36495E+00_JPRB, 0.36610E+00_JPRB, 0.36651E+00_JPRB, 0.36661E+00_JPRB, 0.36622E+00_JPRB ,&
&  0.36565E+00_JPRB, 0.36491E+00_JPRB, 0.36389E+00_JPRB, 0.36274E+00_JPRB, 0.36169E+00_JPRB, 0.36049E+00_JPRB, 0.36007E+00_JPRB ,&
&  0.35998E+00_JPRB, 0.36044E+00_JPRB, 0.36126E+00_JPRB, 0.36287E+00_JPRB, 0.36458E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.36805E+00_JPRB, 0.36927E+00_JPRB, 0.37046E+00_JPRB, 0.37147E+00_JPRB, 0.37228E+00_JPRB, 0.37294E+00_JPRB, 0.37348E+00_JPRB ,&
&  0.37369E+00_JPRB, 0.37428E+00_JPRB, 0.37505E+00_JPRB, 0.37577E+00_JPRB, 0.37645E+00_JPRB, 0.37708E+00_JPRB, 0.37765E+00_JPRB ,&
&  0.37850E+00_JPRB, 0.38055E+00_JPRB, 0.38215E+00_JPRB, 0.38325E+00_JPRB, 0.38443E+00_JPRB, 0.38564E+00_JPRB, 0.38680E+00_JPRB ,&
&  0.38799E+00_JPRB, 0.38918E+00_JPRB, 0.39034E+00_JPRB, 0.39137E+00_JPRB, 0.39257E+00_JPRB, 0.39371E+00_JPRB, 0.39480E+00_JPRB ,&
&  0.39596E+00_JPRB, 0.39749E+00_JPRB, 0.39921E+00_JPRB, 0.40106E+00_JPRB, 0.40292E+00_JPRB, 0.40507E+00_JPRB, 0.40709E+00_JPRB ,&
&  0.40881E+00_JPRB, 0.41005E+00_JPRB, 0.40998E+00_JPRB, 0.40904E+00_JPRB, 0.40739E+00_JPRB, 0.40520E+00_JPRB, 0.40258E+00_JPRB ,&
&  0.39957E+00_JPRB, 0.39687E+00_JPRB, 0.39446E+00_JPRB, 0.39221E+00_JPRB, 0.39012E+00_JPRB, 0.38833E+00_JPRB, 0.38650E+00_JPRB ,&
&  0.38478E+00_JPRB, 0.38306E+00_JPRB, 0.38150E+00_JPRB, 0.37989E+00_JPRB, 0.37826E+00_JPRB, 0.37654E+00_JPRB, 0.37484E+00_JPRB ,&
&  0.37325E+00_JPRB, 0.37155E+00_JPRB, 0.37008E+00_JPRB, 0.36853E+00_JPRB, 0.36707E+00_JPRB, 0.36586E+00_JPRB, 0.36479E+00_JPRB ,&
&  0.36377E+00_JPRB, 0.36301E+00_JPRB, 0.36247E+00_JPRB, 0.36176E+00_JPRB, 0.36101E+00_JPRB, 0.36032E+00_JPRB, 0.35979E+00_JPRB ,&
&  0.35938E+00_JPRB, 0.35931E+00_JPRB, 0.35930E+00_JPRB, 0.35962E+00_JPRB, 0.35977E+00_JPRB, 0.35989E+00_JPRB, 0.36017E+00_JPRB ,&
&  0.36047E+00_JPRB, 0.36084E+00_JPRB, 0.36116E+00_JPRB, 0.36182E+00_JPRB, 0.36224E+00_JPRB, 0.36240E+00_JPRB, 0.36235E+00_JPRB ,&
&  0.36228E+00_JPRB, 0.36220E+00_JPRB, 0.36222E+00_JPRB, 0.36220E+00_JPRB, 0.36240E+00_JPRB, 0.36285E+00_JPRB, 0.36335E+00_JPRB ,&
&  0.36384E+00_JPRB, 0.36441E+00_JPRB, 0.36506E+00_JPRB, 0.36600E+00_JPRB, 0.36691E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB, 0.37716E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB, 0.14128E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.14624E+00_JPRB, 0.14628E+00_JPRB, 0.14620E+00_JPRB, 0.14570E+00_JPRB, 0.14399E+00_JPRB, 0.14371E+00_JPRB, 0.14372E+00_JPRB ,&
&  0.14388E+00_JPRB, 0.14398E+00_JPRB, 0.14469E+00_JPRB, 0.14545E+00_JPRB, 0.14619E+00_JPRB, 0.14675E+00_JPRB, 0.14722E+00_JPRB ,&
&  0.14748E+00_JPRB, 0.14773E+00_JPRB, 0.14686E+00_JPRB, 0.14644E+00_JPRB, 0.14633E+00_JPRB, 0.14634E+00_JPRB, 0.14634E+00_JPRB ,&
&  0.14629E+00_JPRB, 0.14628E+00_JPRB, 0.14616E+00_JPRB, 0.14630E+00_JPRB, 0.14654E+00_JPRB, 0.14674E+00_JPRB, 0.14698E+00_JPRB ,&
&  0.14715E+00_JPRB, 0.14731E+00_JPRB, 0.14741E+00_JPRB, 0.14747E+00_JPRB, 0.14770E+00_JPRB, 0.14779E+00_JPRB, 0.14768E+00_JPRB ,&
&  0.14749E+00_JPRB, 0.14715E+00_JPRB, 0.14677E+00_JPRB, 0.14638E+00_JPRB, 0.14609E+00_JPRB, 0.14582E+00_JPRB, 0.14556E+00_JPRB ,&
&  0.14530E+00_JPRB, 0.14501E+00_JPRB, 0.14471E+00_JPRB, 0.14437E+00_JPRB, 0.14405E+00_JPRB, 0.14381E+00_JPRB, 0.14356E+00_JPRB ,&
&  0.14335E+00_JPRB, 0.14316E+00_JPRB, 0.14294E+00_JPRB, 0.14268E+00_JPRB, 0.14241E+00_JPRB, 0.14214E+00_JPRB, 0.14184E+00_JPRB ,&
&  0.14147E+00_JPRB, 0.14121E+00_JPRB, 0.14095E+00_JPRB, 0.14072E+00_JPRB, 0.14053E+00_JPRB, 0.14033E+00_JPRB, 0.14016E+00_JPRB ,&
&  0.13996E+00_JPRB, 0.13978E+00_JPRB, 0.13956E+00_JPRB, 0.13937E+00_JPRB, 0.13919E+00_JPRB, 0.13904E+00_JPRB, 0.13892E+00_JPRB ,&
&  0.13882E+00_JPRB, 0.13870E+00_JPRB, 0.13861E+00_JPRB, 0.13864E+00_JPRB, 0.13864E+00_JPRB, 0.13869E+00_JPRB, 0.13872E+00_JPRB ,&
&  0.13876E+00_JPRB, 0.13879E+00_JPRB, 0.13882E+00_JPRB, 0.13875E+00_JPRB, 0.13888E+00_JPRB, 0.13906E+00_JPRB, 0.13932E+00_JPRB ,&
&  0.13963E+00_JPRB, 0.14003E+00_JPRB, 0.14044E+00_JPRB, 0.14087E+00_JPRB, 0.14137E+00_JPRB, 0.14178E+00_JPRB, 0.14224E+00_JPRB ,&
&  0.14277E+00_JPRB, 0.14333E+00_JPRB, 0.14374E+00_JPRB, 0.14420E+00_JPRB, 0.14493E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.15676E+00_JPRB, 0.15956E+00_JPRB, 0.16152E+00_JPRB, 0.16341E+00_JPRB, 0.15952E+00_JPRB, 0.17055E+00_JPRB, 0.16743E+00_JPRB ,&
&  0.16679E+00_JPRB, 0.16476E+00_JPRB, 0.16585E+00_JPRB, 0.16557E+00_JPRB, 0.16474E+00_JPRB, 0.16363E+00_JPRB, 0.16232E+00_JPRB ,&
&  0.16110E+00_JPRB, 0.15964E+00_JPRB, 0.15862E+00_JPRB, 0.15755E+00_JPRB, 0.15670E+00_JPRB, 0.15587E+00_JPRB, 0.15531E+00_JPRB ,&
&  0.15524E+00_JPRB, 0.15589E+00_JPRB, 0.15706E+00_JPRB, 0.15660E+00_JPRB, 0.15670E+00_JPRB, 0.15714E+00_JPRB, 0.15776E+00_JPRB ,&
&  0.15807E+00_JPRB, 0.15826E+00_JPRB, 0.15808E+00_JPRB, 0.15720E+00_JPRB, 0.15706E+00_JPRB, 0.15694E+00_JPRB, 0.15689E+00_JPRB ,&
&  0.15669E+00_JPRB, 0.15632E+00_JPRB, 0.15545E+00_JPRB, 0.15465E+00_JPRB, 0.15383E+00_JPRB, 0.15288E+00_JPRB, 0.15180E+00_JPRB ,&
&  0.15089E+00_JPRB, 0.15017E+00_JPRB, 0.14935E+00_JPRB, 0.14856E+00_JPRB, 0.14787E+00_JPRB, 0.14720E+00_JPRB, 0.14660E+00_JPRB ,&
&  0.14621E+00_JPRB, 0.14593E+00_JPRB, 0.14561E+00_JPRB, 0.14514E+00_JPRB, 0.14461E+00_JPRB, 0.14410E+00_JPRB, 0.14363E+00_JPRB ,&
&  0.14332E+00_JPRB, 0.14282E+00_JPRB, 0.14234E+00_JPRB, 0.14181E+00_JPRB, 0.14140E+00_JPRB, 0.14075E+00_JPRB, 0.14019E+00_JPRB ,&
&  0.13965E+00_JPRB, 0.13916E+00_JPRB, 0.13881E+00_JPRB, 0.13855E+00_JPRB, 0.13833E+00_JPRB, 0.13818E+00_JPRB, 0.13811E+00_JPRB ,&
&  0.13809E+00_JPRB, 0.13810E+00_JPRB, 0.13819E+00_JPRB, 0.13821E+00_JPRB, 0.13821E+00_JPRB, 0.13826E+00_JPRB, 0.13854E+00_JPRB ,&
&  0.13878E+00_JPRB, 0.13897E+00_JPRB, 0.13911E+00_JPRB, 0.13891E+00_JPRB, 0.13927E+00_JPRB, 0.13958E+00_JPRB, 0.13996E+00_JPRB ,&
&  0.14030E+00_JPRB, 0.14099E+00_JPRB, 0.14181E+00_JPRB, 0.14280E+00_JPRB, 0.14462E+00_JPRB, 0.14629E+00_JPRB, 0.14805E+00_JPRB ,&
&  0.15002E+00_JPRB, 0.15285E+00_JPRB, 0.15416E+00_JPRB, 0.15508E+00_JPRB, 0.15591E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.18120E+00_JPRB, 0.18251E+00_JPRB, 0.18379E+00_JPRB, 0.18498E+00_JPRB, 0.18623E+00_JPRB, 0.18668E+00_JPRB, 0.18653E+00_JPRB ,&
&  0.18426E+00_JPRB, 0.18357E+00_JPRB, 0.18201E+00_JPRB, 0.18017E+00_JPRB, 0.17870E+00_JPRB, 0.17629E+00_JPRB, 0.17372E+00_JPRB ,&
&  0.17109E+00_JPRB, 0.16856E+00_JPRB, 0.16625E+00_JPRB, 0.16310E+00_JPRB, 0.16006E+00_JPRB, 0.15709E+00_JPRB, 0.15604E+00_JPRB ,&
&  0.15555E+00_JPRB, 0.15589E+00_JPRB, 0.15733E+00_JPRB, 0.15863E+00_JPRB, 0.16051E+00_JPRB, 0.16254E+00_JPRB, 0.16563E+00_JPRB ,&
&  0.16875E+00_JPRB, 0.17109E+00_JPRB, 0.17150E+00_JPRB, 0.17051E+00_JPRB, 0.16952E+00_JPRB, 0.16911E+00_JPRB, 0.16859E+00_JPRB ,&
&  0.16771E+00_JPRB, 0.16705E+00_JPRB, 0.16631E+00_JPRB, 0.16563E+00_JPRB, 0.16415E+00_JPRB, 0.16172E+00_JPRB, 0.15933E+00_JPRB ,&
&  0.15713E+00_JPRB, 0.15518E+00_JPRB, 0.15340E+00_JPRB, 0.15187E+00_JPRB, 0.15097E+00_JPRB, 0.15008E+00_JPRB, 0.14904E+00_JPRB ,&
&  0.14841E+00_JPRB, 0.14775E+00_JPRB, 0.14697E+00_JPRB, 0.14638E+00_JPRB, 0.14569E+00_JPRB, 0.14519E+00_JPRB, 0.14463E+00_JPRB ,&
&  0.14418E+00_JPRB, 0.14352E+00_JPRB, 0.14295E+00_JPRB, 0.14233E+00_JPRB, 0.14204E+00_JPRB, 0.14138E+00_JPRB, 0.14075E+00_JPRB ,&
&  0.13999E+00_JPRB, 0.13952E+00_JPRB, 0.13866E+00_JPRB, 0.13816E+00_JPRB, 0.13758E+00_JPRB, 0.13707E+00_JPRB, 0.13674E+00_JPRB ,&
&  0.13687E+00_JPRB, 0.13695E+00_JPRB, 0.13671E+00_JPRB, 0.13674E+00_JPRB, 0.13686E+00_JPRB, 0.13728E+00_JPRB, 0.13842E+00_JPRB ,&
&  0.13915E+00_JPRB, 0.13997E+00_JPRB, 0.14079E+00_JPRB, 0.14144E+00_JPRB, 0.14248E+00_JPRB, 0.14349E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14573E+00_JPRB, 0.14678E+00_JPRB, 0.14802E+00_JPRB, 0.14949E+00_JPRB, 0.15291E+00_JPRB, 0.15554E+00_JPRB, 0.15845E+00_JPRB ,&
&  0.16079E+00_JPRB, 0.16052E+00_JPRB, 0.17376E+00_JPRB, 0.17638E+00_JPRB, 0.17828E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.18920E+00_JPRB, 0.19000E+00_JPRB, 0.19008E+00_JPRB, 0.19100E+00_JPRB, 0.19185E+00_JPRB, 0.19284E+00_JPRB, 0.19319E+00_JPRB ,&
&  0.19245E+00_JPRB, 0.18799E+00_JPRB, 0.18547E+00_JPRB, 0.18225E+00_JPRB, 0.17908E+00_JPRB, 0.17587E+00_JPRB, 0.17320E+00_JPRB ,&
&  0.17144E+00_JPRB, 0.17011E+00_JPRB, 0.16890E+00_JPRB, 0.16723E+00_JPRB, 0.16526E+00_JPRB, 0.16328E+00_JPRB, 0.16146E+00_JPRB ,&
&  0.15985E+00_JPRB, 0.15982E+00_JPRB, 0.16086E+00_JPRB, 0.16288E+00_JPRB, 0.16657E+00_JPRB, 0.16968E+00_JPRB, 0.17364E+00_JPRB ,&
&  0.18012E+00_JPRB, 0.17780E+00_JPRB, 0.17789E+00_JPRB, 0.17796E+00_JPRB, 0.17756E+00_JPRB, 0.17725E+00_JPRB, 0.17388E+00_JPRB ,&
&  0.17639E+00_JPRB, 0.17656E+00_JPRB, 0.17968E+00_JPRB, 0.17919E+00_JPRB, 0.17513E+00_JPRB, 0.17143E+00_JPRB, 0.16666E+00_JPRB ,&
&  0.16297E+00_JPRB, 0.15879E+00_JPRB, 0.15600E+00_JPRB, 0.15358E+00_JPRB, 0.15096E+00_JPRB, 0.14988E+00_JPRB, 0.14808E+00_JPRB ,&
&  0.14694E+00_JPRB, 0.14668E+00_JPRB, 0.14585E+00_JPRB, 0.14579E+00_JPRB, 0.14512E+00_JPRB, 0.14416E+00_JPRB, 0.14350E+00_JPRB ,&
&  0.14289E+00_JPRB, 0.14226E+00_JPRB, 0.14203E+00_JPRB, 0.14181E+00_JPRB, 0.14195E+00_JPRB, 0.14201E+00_JPRB, 0.14214E+00_JPRB ,&
&  0.14238E+00_JPRB, 0.14254E+00_JPRB, 0.14221E+00_JPRB, 0.14204E+00_JPRB, 0.14126E+00_JPRB, 0.14058E+00_JPRB, 0.13954E+00_JPRB ,&
&  0.13892E+00_JPRB, 0.13858E+00_JPRB, 0.13812E+00_JPRB, 0.13781E+00_JPRB, 0.13777E+00_JPRB, 0.13824E+00_JPRB, 0.13883E+00_JPRB ,&
&  0.13948E+00_JPRB, 0.14061E+00_JPRB, 0.14155E+00_JPRB, 0.14279E+00_JPRB, 0.14419E+00_JPRB, 0.14532E+00_JPRB, 0.14690E+00_JPRB ,&
&  0.14849E+00_JPRB, 0.15010E+00_JPRB, 0.15276E+00_JPRB, 0.15579E+00_JPRB, 0.15982E+00_JPRB, 0.16473E+00_JPRB, 0.16792E+00_JPRB ,&
&  0.17164E+00_JPRB, 0.17917E+00_JPRB, 0.18170E+00_JPRB, 0.18758E+00_JPRB, 0.18848E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.19406E+00_JPRB, 0.19392E+00_JPRB, 0.19335E+00_JPRB, 0.19286E+00_JPRB, 0.19328E+00_JPRB, 0.19400E+00_JPRB, 0.19481E+00_JPRB ,&
&  0.19140E+00_JPRB, 0.18859E+00_JPRB, 0.18428E+00_JPRB, 0.17949E+00_JPRB, 0.17535E+00_JPRB, 0.17184E+00_JPRB, 0.16976E+00_JPRB ,&
&  0.16948E+00_JPRB, 0.17121E+00_JPRB, 0.17277E+00_JPRB, 0.17447E+00_JPRB, 0.17467E+00_JPRB, 0.17486E+00_JPRB, 0.17371E+00_JPRB ,&
&  0.17243E+00_JPRB, 0.17188E+00_JPRB, 0.17215E+00_JPRB, 0.17400E+00_JPRB, 0.17670E+00_JPRB, 0.17866E+00_JPRB, 0.18091E+00_JPRB ,&
&  0.18386E+00_JPRB, 0.18347E+00_JPRB, 0.18423E+00_JPRB, 0.18375E+00_JPRB, 0.18383E+00_JPRB, 0.18442E+00_JPRB, 0.18465E+00_JPRB ,&
&  0.18518E+00_JPRB, 0.18578E+00_JPRB, 0.18550E+00_JPRB, 0.18487E+00_JPRB, 0.18097E+00_JPRB, 0.17808E+00_JPRB, 0.17346E+00_JPRB ,&
&  0.16810E+00_JPRB, 0.16345E+00_JPRB, 0.15845E+00_JPRB, 0.15500E+00_JPRB, 0.15116E+00_JPRB, 0.14886E+00_JPRB, 0.14709E+00_JPRB ,&
&  0.14542E+00_JPRB, 0.14475E+00_JPRB, 0.14385E+00_JPRB, 0.14292E+00_JPRB, 0.14276E+00_JPRB, 0.14188E+00_JPRB, 0.14180E+00_JPRB ,&
&  0.14200E+00_JPRB, 0.14218E+00_JPRB, 0.14298E+00_JPRB, 0.14343E+00_JPRB, 0.14419E+00_JPRB, 0.14494E+00_JPRB, 0.14560E+00_JPRB ,&
&  0.14670E+00_JPRB, 0.14797E+00_JPRB, 0.14803E+00_JPRB, 0.14822E+00_JPRB, 0.14743E+00_JPRB, 0.14589E+00_JPRB, 0.14415E+00_JPRB ,&
&  0.14236E+00_JPRB, 0.14096E+00_JPRB, 0.14052E+00_JPRB, 0.13992E+00_JPRB, 0.14008E+00_JPRB, 0.14042E+00_JPRB, 0.14065E+00_JPRB ,&
&  0.14129E+00_JPRB, 0.14186E+00_JPRB, 0.14260E+00_JPRB, 0.14418E+00_JPRB, 0.14534E+00_JPRB, 0.14681E+00_JPRB, 0.14833E+00_JPRB ,&
&  0.14955E+00_JPRB, 0.15168E+00_JPRB, 0.15357E+00_JPRB, 0.15616E+00_JPRB, 0.16203E+00_JPRB, 0.16756E+00_JPRB, 0.17737E+00_JPRB ,&
&  0.18318E+00_JPRB, 0.19654E+00_JPRB, 0.19548E+00_JPRB, 0.19375E+00_JPRB, 0.19448E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.19993E+00_JPRB, 0.19866E+00_JPRB, 0.19767E+00_JPRB, 0.19649E+00_JPRB, 0.19720E+00_JPRB, 0.19823E+00_JPRB, 0.19951E+00_JPRB ,&
&  0.20099E+00_JPRB, 0.20134E+00_JPRB, 0.20100E+00_JPRB, 0.19916E+00_JPRB, 0.19792E+00_JPRB, 0.19150E+00_JPRB, 0.18909E+00_JPRB ,&
&  0.18728E+00_JPRB, 0.18873E+00_JPRB, 0.19093E+00_JPRB, 0.19427E+00_JPRB, 0.19592E+00_JPRB, 0.19760E+00_JPRB, 0.19689E+00_JPRB ,&
&  0.19488E+00_JPRB, 0.19240E+00_JPRB, 0.19013E+00_JPRB, 0.18830E+00_JPRB, 0.18779E+00_JPRB, 0.18650E+00_JPRB, 0.18503E+00_JPRB ,&
&  0.18492E+00_JPRB, 0.18615E+00_JPRB, 0.18694E+00_JPRB, 0.18775E+00_JPRB, 0.19381E+00_JPRB, 0.19475E+00_JPRB, 0.19655E+00_JPRB ,&
&  0.19691E+00_JPRB, 0.19723E+00_JPRB, 0.19637E+00_JPRB, 0.19500E+00_JPRB, 0.19375E+00_JPRB, 0.18937E+00_JPRB, 0.18725E+00_JPRB ,&
&  0.17989E+00_JPRB, 0.17546E+00_JPRB, 0.16902E+00_JPRB, 0.16344E+00_JPRB, 0.15769E+00_JPRB, 0.15341E+00_JPRB, 0.14988E+00_JPRB ,&
&  0.14710E+00_JPRB, 0.14505E+00_JPRB, 0.14331E+00_JPRB, 0.14308E+00_JPRB, 0.14322E+00_JPRB, 0.14298E+00_JPRB, 0.14314E+00_JPRB ,&
&  0.14381E+00_JPRB, 0.14454E+00_JPRB, 0.14452E+00_JPRB, 0.14506E+00_JPRB, 0.14558E+00_JPRB, 0.14634E+00_JPRB, 0.14770E+00_JPRB ,&
&  0.14979E+00_JPRB, 0.15172E+00_JPRB, 0.15265E+00_JPRB, 0.15413E+00_JPRB, 0.15346E+00_JPRB, 0.15100E+00_JPRB, 0.14853E+00_JPRB ,&
&  0.14597E+00_JPRB, 0.14483E+00_JPRB, 0.14367E+00_JPRB, 0.14303E+00_JPRB, 0.14298E+00_JPRB, 0.14320E+00_JPRB, 0.14392E+00_JPRB ,&
&  0.14469E+00_JPRB, 0.14547E+00_JPRB, 0.14581E+00_JPRB, 0.14627E+00_JPRB, 0.14670E+00_JPRB, 0.14747E+00_JPRB, 0.14884E+00_JPRB ,&
&  0.15067E+00_JPRB, 0.15277E+00_JPRB, 0.15481E+00_JPRB, 0.15733E+00_JPRB, 0.16153E+00_JPRB, 0.16735E+00_JPRB, 0.17622E+00_JPRB ,&
&  0.18382E+00_JPRB, 0.19734E+00_JPRB, 0.19869E+00_JPRB, 0.19907E+00_JPRB, 0.20016E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.20813E+00_JPRB, 0.20824E+00_JPRB, 0.20478E+00_JPRB, 0.20160E+00_JPRB, 0.20013E+00_JPRB, 0.19963E+00_JPRB, 0.20064E+00_JPRB ,&
&  0.20311E+00_JPRB, 0.20497E+00_JPRB, 0.20681E+00_JPRB, 0.20844E+00_JPRB, 0.20913E+00_JPRB, 0.20914E+00_JPRB, 0.21004E+00_JPRB ,&
&  0.21091E+00_JPRB, 0.21309E+00_JPRB, 0.21405E+00_JPRB, 0.21613E+00_JPRB, 0.21638E+00_JPRB, 0.21713E+00_JPRB, 0.21830E+00_JPRB ,&
&  0.21796E+00_JPRB, 0.21879E+00_JPRB, 0.22039E+00_JPRB, 0.21854E+00_JPRB, 0.21724E+00_JPRB, 0.21520E+00_JPRB, 0.21341E+00_JPRB ,&
&  0.20776E+00_JPRB, 0.20168E+00_JPRB, 0.19477E+00_JPRB, 0.19736E+00_JPRB, 0.20549E+00_JPRB, 0.20785E+00_JPRB, 0.21128E+00_JPRB ,&
&  0.21208E+00_JPRB, 0.21054E+00_JPRB, 0.20879E+00_JPRB, 0.20585E+00_JPRB, 0.20292E+00_JPRB, 0.19976E+00_JPRB, 0.19731E+00_JPRB ,&
&  0.19468E+00_JPRB, 0.19511E+00_JPRB, 0.19193E+00_JPRB, 0.17968E+00_JPRB, 0.17275E+00_JPRB, 0.16851E+00_JPRB, 0.16241E+00_JPRB ,&
&  0.15653E+00_JPRB, 0.15450E+00_JPRB, 0.15398E+00_JPRB, 0.15226E+00_JPRB, 0.15238E+00_JPRB, 0.15271E+00_JPRB, 0.15273E+00_JPRB ,&
&  0.15171E+00_JPRB, 0.15152E+00_JPRB, 0.15030E+00_JPRB, 0.14855E+00_JPRB, 0.14696E+00_JPRB, 0.14635E+00_JPRB, 0.14672E+00_JPRB ,&
&  0.14808E+00_JPRB, 0.14977E+00_JPRB, 0.15494E+00_JPRB, 0.15849E+00_JPRB, 0.15494E+00_JPRB, 0.15174E+00_JPRB, 0.15114E+00_JPRB ,&
&  0.14783E+00_JPRB, 0.14709E+00_JPRB, 0.14706E+00_JPRB, 0.14702E+00_JPRB, 0.14734E+00_JPRB, 0.14815E+00_JPRB, 0.14823E+00_JPRB ,&
&  0.14881E+00_JPRB, 0.14982E+00_JPRB, 0.15005E+00_JPRB, 0.15005E+00_JPRB, 0.14987E+00_JPRB, 0.15027E+00_JPRB, 0.15132E+00_JPRB ,&
&  0.15259E+00_JPRB, 0.15487E+00_JPRB, 0.15729E+00_JPRB, 0.16073E+00_JPRB, 0.16350E+00_JPRB, 0.16565E+00_JPRB, 0.16939E+00_JPRB ,&
&  0.17298E+00_JPRB, 0.18113E+00_JPRB, 0.19529E+00_JPRB, 0.20672E+00_JPRB, 0.20480E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.21306E+00_JPRB, 0.21290E+00_JPRB, 0.21161E+00_JPRB, 0.21041E+00_JPRB, 0.20887E+00_JPRB, 0.20733E+00_JPRB, 0.20781E+00_JPRB ,&
&  0.20987E+00_JPRB, 0.21260E+00_JPRB, 0.21524E+00_JPRB, 0.21730E+00_JPRB, 0.21896E+00_JPRB, 0.21978E+00_JPRB, 0.22112E+00_JPRB ,&
&  0.22273E+00_JPRB, 0.22538E+00_JPRB, 0.22605E+00_JPRB, 0.22665E+00_JPRB, 0.22612E+00_JPRB, 0.22564E+00_JPRB, 0.22682E+00_JPRB ,&
&  0.23003E+00_JPRB, 0.23173E+00_JPRB, 0.23115E+00_JPRB, 0.22935E+00_JPRB, 0.22987E+00_JPRB, 0.23150E+00_JPRB, 0.23058E+00_JPRB ,&
&  0.22477E+00_JPRB, 0.21214E+00_JPRB, 0.20283E+00_JPRB, 0.20472E+00_JPRB, 0.21366E+00_JPRB, 0.21774E+00_JPRB, 0.22018E+00_JPRB ,&
&  0.22173E+00_JPRB, 0.22200E+00_JPRB, 0.22131E+00_JPRB, 0.22003E+00_JPRB, 0.21777E+00_JPRB, 0.21390E+00_JPRB, 0.20918E+00_JPRB ,&
&  0.20485E+00_JPRB, 0.20243E+00_JPRB, 0.19986E+00_JPRB, 0.19544E+00_JPRB, 0.19091E+00_JPRB, 0.18798E+00_JPRB, 0.18498E+00_JPRB ,&
&  0.18198E+00_JPRB, 0.17943E+00_JPRB, 0.17880E+00_JPRB, 0.17860E+00_JPRB, 0.17874E+00_JPRB, 0.17855E+00_JPRB, 0.17699E+00_JPRB ,&
&  0.17340E+00_JPRB, 0.16948E+00_JPRB, 0.16398E+00_JPRB, 0.15912E+00_JPRB, 0.15559E+00_JPRB, 0.15232E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14669E+00_JPRB, 0.14865E+00_JPRB, 0.15602E+00_JPRB, 0.16482E+00_JPRB, 0.17226E+00_JPRB, 0.16069E+00_JPRB, 0.15586E+00_JPRB ,&
&  0.15424E+00_JPRB, 0.15370E+00_JPRB, 0.15385E+00_JPRB, 0.15497E+00_JPRB, 0.15558E+00_JPRB, 0.15552E+00_JPRB, 0.15602E+00_JPRB ,&
&  0.15757E+00_JPRB, 0.15924E+00_JPRB, 0.16016E+00_JPRB, 0.16105E+00_JPRB, 0.16155E+00_JPRB, 0.16133E+00_JPRB, 0.16104E+00_JPRB ,&
&  0.16153E+00_JPRB, 0.16394E+00_JPRB, 0.16814E+00_JPRB, 0.17419E+00_JPRB, 0.17889E+00_JPRB, 0.18321E+00_JPRB, 0.18894E+00_JPRB ,&
&  0.19618E+00_JPRB, 0.20383E+00_JPRB, 0.20732E+00_JPRB, 0.21042E+00_JPRB, 0.21207E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.21867E+00_JPRB, 0.21983E+00_JPRB, 0.22172E+00_JPRB, 0.22363E+00_JPRB, 0.22424E+00_JPRB, 0.22361E+00_JPRB, 0.22295E+00_JPRB ,&
&  0.22317E+00_JPRB, 0.22478E+00_JPRB, 0.22647E+00_JPRB, 0.22666E+00_JPRB, 0.22673E+00_JPRB, 0.22701E+00_JPRB, 0.22866E+00_JPRB ,&
&  0.23122E+00_JPRB, 0.23419E+00_JPRB, 0.23422E+00_JPRB, 0.23389E+00_JPRB, 0.23380E+00_JPRB, 0.23311E+00_JPRB, 0.23440E+00_JPRB ,&
&  0.23744E+00_JPRB, 0.23784E+00_JPRB, 0.23791E+00_JPRB, 0.23728E+00_JPRB, 0.23871E+00_JPRB, 0.24148E+00_JPRB, 0.24281E+00_JPRB ,&
&  0.24099E+00_JPRB, 0.23443E+00_JPRB, 0.22655E+00_JPRB, 0.22626E+00_JPRB, 0.22792E+00_JPRB, 0.22967E+00_JPRB, 0.22998E+00_JPRB ,&
&  0.22956E+00_JPRB, 0.22876E+00_JPRB, 0.22773E+00_JPRB, 0.22678E+00_JPRB, 0.22580E+00_JPRB, 0.22277E+00_JPRB, 0.21925E+00_JPRB ,&
&  0.21647E+00_JPRB, 0.21355E+00_JPRB, 0.21057E+00_JPRB, 0.20679E+00_JPRB, 0.20285E+00_JPRB, 0.20005E+00_JPRB, 0.19704E+00_JPRB ,&
&  0.19327E+00_JPRB, 0.19044E+00_JPRB, 0.18964E+00_JPRB, 0.18982E+00_JPRB, 0.19019E+00_JPRB, 0.18965E+00_JPRB, 0.18898E+00_JPRB ,&
&  0.18770E+00_JPRB, 0.18668E+00_JPRB, 0.18534E+00_JPRB, 0.18482E+00_JPRB, 0.18096E+00_JPRB, 0.17254E+00_JPRB, 0.16684E+00_JPRB ,&
&  0.16587E+00_JPRB, 0.16865E+00_JPRB, 0.17291E+00_JPRB, 0.17332E+00_JPRB, 0.17460E+00_JPRB, 0.17551E+00_JPRB, 0.17529E+00_JPRB ,&
&  0.17341E+00_JPRB, 0.16891E+00_JPRB, 0.16814E+00_JPRB, 0.16882E+00_JPRB, 0.16909E+00_JPRB, 0.16991E+00_JPRB, 0.17136E+00_JPRB ,&
&  0.17397E+00_JPRB, 0.17641E+00_JPRB, 0.17844E+00_JPRB, 0.18036E+00_JPRB, 0.18142E+00_JPRB, 0.18064E+00_JPRB, 0.17864E+00_JPRB ,&
&  0.17822E+00_JPRB, 0.18296E+00_JPRB, 0.19194E+00_JPRB, 0.20288E+00_JPRB, 0.20495E+00_JPRB, 0.20691E+00_JPRB, 0.20882E+00_JPRB ,&
&  0.20969E+00_JPRB, 0.21241E+00_JPRB, 0.21478E+00_JPRB, 0.21616E+00_JPRB, 0.21730E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.22840E+00_JPRB, 0.22996E+00_JPRB, 0.23074E+00_JPRB, 0.23172E+00_JPRB, 0.23356E+00_JPRB, 0.23488E+00_JPRB, 0.23507E+00_JPRB ,&
&  0.23512E+00_JPRB, 0.23599E+00_JPRB, 0.23655E+00_JPRB, 0.23677E+00_JPRB, 0.23449E+00_JPRB, 0.23427E+00_JPRB, 0.23593E+00_JPRB ,&
&  0.23810E+00_JPRB, 0.24058E+00_JPRB, 0.24091E+00_JPRB, 0.24095E+00_JPRB, 0.24077E+00_JPRB, 0.24011E+00_JPRB, 0.24185E+00_JPRB ,&
&  0.24327E+00_JPRB, 0.24397E+00_JPRB, 0.24537E+00_JPRB, 0.24661E+00_JPRB, 0.24919E+00_JPRB, 0.25215E+00_JPRB, 0.25347E+00_JPRB ,&
&  0.25277E+00_JPRB, 0.25096E+00_JPRB, 0.24507E+00_JPRB, 0.23873E+00_JPRB, 0.23949E+00_JPRB, 0.24082E+00_JPRB, 0.24087E+00_JPRB ,&
&  0.23890E+00_JPRB, 0.23623E+00_JPRB, 0.23439E+00_JPRB, 0.23293E+00_JPRB, 0.23128E+00_JPRB, 0.22867E+00_JPRB, 0.22588E+00_JPRB ,&
&  0.22338E+00_JPRB, 0.22252E+00_JPRB, 0.22116E+00_JPRB, 0.21907E+00_JPRB, 0.21612E+00_JPRB, 0.21385E+00_JPRB, 0.21156E+00_JPRB ,&
&  0.20730E+00_JPRB, 0.20303E+00_JPRB, 0.20077E+00_JPRB, 0.20037E+00_JPRB, 0.20099E+00_JPRB, 0.20081E+00_JPRB, 0.19944E+00_JPRB ,&
&  0.19782E+00_JPRB, 0.19558E+00_JPRB, 0.19423E+00_JPRB, 0.19385E+00_JPRB, 0.19489E+00_JPRB, 0.19412E+00_JPRB, 0.19030E+00_JPRB ,&
&  0.18803E+00_JPRB, 0.18634E+00_JPRB, 0.18613E+00_JPRB, 0.18511E+00_JPRB, 0.18418E+00_JPRB, 0.18291E+00_JPRB, 0.18318E+00_JPRB ,&
&  0.18401E+00_JPRB, 0.18443E+00_JPRB, 0.18672E+00_JPRB, 0.19076E+00_JPRB, 0.19448E+00_JPRB, 0.19666E+00_JPRB, 0.19792E+00_JPRB ,&
&  0.19836E+00_JPRB, 0.19759E+00_JPRB, 0.19684E+00_JPRB, 0.19711E+00_JPRB, 0.19869E+00_JPRB, 0.20092E+00_JPRB, 0.20314E+00_JPRB ,&
&  0.20385E+00_JPRB, 0.20481E+00_JPRB, 0.20691E+00_JPRB, 0.21012E+00_JPRB, 0.21206E+00_JPRB, 0.21378E+00_JPRB, 0.21536E+00_JPRB ,&
&  0.21622E+00_JPRB, 0.21760E+00_JPRB, 0.22103E+00_JPRB, 0.22323E+00_JPRB, 0.22557E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.23710E+00_JPRB, 0.23787E+00_JPRB, 0.23889E+00_JPRB, 0.23909E+00_JPRB, 0.23992E+00_JPRB, 0.24064E+00_JPRB, 0.24188E+00_JPRB ,&
&  0.24225E+00_JPRB, 0.24339E+00_JPRB, 0.24518E+00_JPRB, 0.24508E+00_JPRB, 0.24475E+00_JPRB, 0.24464E+00_JPRB, 0.24534E+00_JPRB ,&
&  0.24765E+00_JPRB, 0.24921E+00_JPRB, 0.24899E+00_JPRB, 0.24890E+00_JPRB, 0.24861E+00_JPRB, 0.24844E+00_JPRB, 0.25070E+00_JPRB ,&
&  0.25134E+00_JPRB, 0.25218E+00_JPRB, 0.25397E+00_JPRB, 0.25534E+00_JPRB, 0.25853E+00_JPRB, 0.26220E+00_JPRB, 0.26411E+00_JPRB ,&
&  0.26305E+00_JPRB, 0.26011E+00_JPRB, 0.25583E+00_JPRB, 0.25246E+00_JPRB, 0.25206E+00_JPRB, 0.25305E+00_JPRB, 0.25520E+00_JPRB ,&
&  0.25430E+00_JPRB, 0.24894E+00_JPRB, 0.24217E+00_JPRB, 0.23854E+00_JPRB, 0.23736E+00_JPRB, 0.23601E+00_JPRB, 0.23352E+00_JPRB ,&
&  0.23146E+00_JPRB, 0.23102E+00_JPRB, 0.23109E+00_JPRB, 0.23036E+00_JPRB, 0.22900E+00_JPRB, 0.22669E+00_JPRB, 0.22366E+00_JPRB ,&
&  0.22110E+00_JPRB, 0.21846E+00_JPRB, 0.21682E+00_JPRB, 0.21558E+00_JPRB, 0.21552E+00_JPRB, 0.21588E+00_JPRB, 0.21530E+00_JPRB ,&
&  0.21374E+00_JPRB, 0.21279E+00_JPRB, 0.21144E+00_JPRB, 0.20858E+00_JPRB, 0.20651E+00_JPRB, 0.20590E+00_JPRB, 0.20486E+00_JPRB ,&
&  0.20336E+00_JPRB, 0.20270E+00_JPRB, 0.20235E+00_JPRB, 0.20173E+00_JPRB, 0.20239E+00_JPRB, 0.20391E+00_JPRB, 0.20147E+00_JPRB ,&
&  0.19947E+00_JPRB, 0.19815E+00_JPRB, 0.19813E+00_JPRB, 0.19937E+00_JPRB, 0.20154E+00_JPRB, 0.20395E+00_JPRB, 0.20602E+00_JPRB ,&
&  0.20690E+00_JPRB, 0.20713E+00_JPRB, 0.20747E+00_JPRB, 0.20747E+00_JPRB, 0.20823E+00_JPRB, 0.21001E+00_JPRB, 0.21231E+00_JPRB ,&
&  0.21353E+00_JPRB, 0.21588E+00_JPRB, 0.21827E+00_JPRB, 0.22125E+00_JPRB, 0.22288E+00_JPRB, 0.22441E+00_JPRB, 0.22597E+00_JPRB ,&
&  0.22724E+00_JPRB, 0.22883E+00_JPRB, 0.23113E+00_JPRB, 0.23311E+00_JPRB, 0.23527E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.24851E+00_JPRB, 0.24784E+00_JPRB, 0.24770E+00_JPRB, 0.24834E+00_JPRB, 0.25041E+00_JPRB, 0.25098E+00_JPRB, 0.25085E+00_JPRB ,&
&  0.25044E+00_JPRB, 0.25199E+00_JPRB, 0.25516E+00_JPRB, 0.25705E+00_JPRB, 0.25717E+00_JPRB, 0.25693E+00_JPRB, 0.25671E+00_JPRB ,&
&  0.25689E+00_JPRB, 0.25704E+00_JPRB, 0.25626E+00_JPRB, 0.25642E+00_JPRB, 0.25685E+00_JPRB, 0.25651E+00_JPRB, 0.25770E+00_JPRB ,&
&  0.25805E+00_JPRB, 0.25857E+00_JPRB, 0.26076E+00_JPRB, 0.26244E+00_JPRB, 0.26570E+00_JPRB, 0.27037E+00_JPRB, 0.27188E+00_JPRB ,&
&  0.27029E+00_JPRB, 0.26656E+00_JPRB, 0.26378E+00_JPRB, 0.26446E+00_JPRB, 0.26590E+00_JPRB, 0.26868E+00_JPRB, 0.27137E+00_JPRB ,&
&  0.27121E+00_JPRB, 0.26492E+00_JPRB, 0.25632E+00_JPRB, 0.25106E+00_JPRB, 0.24832E+00_JPRB, 0.24689E+00_JPRB, 0.24581E+00_JPRB ,&
&  0.24396E+00_JPRB, 0.24248E+00_JPRB, 0.24182E+00_JPRB, 0.24186E+00_JPRB, 0.24181E+00_JPRB, 0.23951E+00_JPRB, 0.23696E+00_JPRB ,&
&  0.23344E+00_JPRB, 0.23103E+00_JPRB, 0.22901E+00_JPRB, 0.22836E+00_JPRB, 0.22855E+00_JPRB, 0.22844E+00_JPRB, 0.22764E+00_JPRB ,&
&  0.22653E+00_JPRB, 0.22624E+00_JPRB, 0.22599E+00_JPRB, 0.22502E+00_JPRB, 0.22384E+00_JPRB, 0.22363E+00_JPRB, 0.22267E+00_JPRB ,&
&  0.22047E+00_JPRB, 0.21901E+00_JPRB, 0.21877E+00_JPRB, 0.21946E+00_JPRB, 0.21932E+00_JPRB, 0.21997E+00_JPRB, 0.22135E+00_JPRB ,&
&  0.22044E+00_JPRB, 0.21811E+00_JPRB, 0.21698E+00_JPRB, 0.21754E+00_JPRB, 0.21923E+00_JPRB, 0.22143E+00_JPRB, 0.22247E+00_JPRB ,&
&  0.22341E+00_JPRB, 0.22333E+00_JPRB, 0.22414E+00_JPRB, 0.22531E+00_JPRB, 0.22660E+00_JPRB, 0.22716E+00_JPRB, 0.22734E+00_JPRB ,&
&  0.22769E+00_JPRB, 0.22810E+00_JPRB, 0.22985E+00_JPRB, 0.23272E+00_JPRB, 0.23522E+00_JPRB, 0.23810E+00_JPRB, 0.24032E+00_JPRB ,&
&  0.24177E+00_JPRB, 0.24332E+00_JPRB, 0.24442E+00_JPRB, 0.24538E+00_JPRB, 0.24684E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.26668E+00_JPRB, 0.26488E+00_JPRB, 0.26375E+00_JPRB, 0.26419E+00_JPRB, 0.26638E+00_JPRB, 0.26697E+00_JPRB, 0.26572E+00_JPRB ,&
&  0.26440E+00_JPRB, 0.26556E+00_JPRB, 0.26822E+00_JPRB, 0.26937E+00_JPRB, 0.26964E+00_JPRB, 0.27025E+00_JPRB, 0.26940E+00_JPRB ,&
&  0.26666E+00_JPRB, 0.26372E+00_JPRB, 0.26419E+00_JPRB, 0.26539E+00_JPRB, 0.26664E+00_JPRB, 0.26577E+00_JPRB, 0.26459E+00_JPRB ,&
&  0.26547E+00_JPRB, 0.26559E+00_JPRB, 0.26731E+00_JPRB, 0.26933E+00_JPRB, 0.27240E+00_JPRB, 0.27666E+00_JPRB, 0.27955E+00_JPRB ,&
&  0.27950E+00_JPRB, 0.27755E+00_JPRB, 0.27727E+00_JPRB, 0.27900E+00_JPRB, 0.28075E+00_JPRB, 0.28514E+00_JPRB, 0.28893E+00_JPRB ,&
&  0.28924E+00_JPRB, 0.28534E+00_JPRB, 0.27590E+00_JPRB, 0.26893E+00_JPRB, 0.26556E+00_JPRB, 0.26257E+00_JPRB, 0.25976E+00_JPRB ,&
&  0.25634E+00_JPRB, 0.25366E+00_JPRB, 0.25208E+00_JPRB, 0.25146E+00_JPRB, 0.25135E+00_JPRB, 0.25099E+00_JPRB, 0.24924E+00_JPRB ,&
&  0.24692E+00_JPRB, 0.24517E+00_JPRB, 0.24378E+00_JPRB, 0.24267E+00_JPRB, 0.24161E+00_JPRB, 0.24118E+00_JPRB, 0.24067E+00_JPRB ,&
&  0.24044E+00_JPRB, 0.24092E+00_JPRB, 0.24182E+00_JPRB, 0.24203E+00_JPRB, 0.24209E+00_JPRB, 0.24157E+00_JPRB, 0.24082E+00_JPRB ,&
&  0.23938E+00_JPRB, 0.23684E+00_JPRB, 0.23463E+00_JPRB, 0.23381E+00_JPRB, 0.23333E+00_JPRB, 0.23356E+00_JPRB, 0.23398E+00_JPRB ,&
&  0.23379E+00_JPRB, 0.23372E+00_JPRB, 0.23454E+00_JPRB, 0.23572E+00_JPRB, 0.23665E+00_JPRB, 0.23704E+00_JPRB, 0.23758E+00_JPRB ,&
&  0.23902E+00_JPRB, 0.24069E+00_JPRB, 0.24182E+00_JPRB, 0.24255E+00_JPRB, 0.24288E+00_JPRB, 0.24378E+00_JPRB, 0.24367E+00_JPRB ,&
&  0.24371E+00_JPRB, 0.24462E+00_JPRB, 0.24717E+00_JPRB, 0.25093E+00_JPRB, 0.25458E+00_JPRB, 0.25701E+00_JPRB, 0.25945E+00_JPRB ,&
&  0.26141E+00_JPRB, 0.26300E+00_JPRB, 0.26395E+00_JPRB, 0.26464E+00_JPRB, 0.26590E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.28524E+00_JPRB, 0.28386E+00_JPRB, 0.28295E+00_JPRB, 0.28191E+00_JPRB, 0.28272E+00_JPRB, 0.28226E+00_JPRB, 0.28021E+00_JPRB ,&
&  0.27873E+00_JPRB, 0.28002E+00_JPRB, 0.28231E+00_JPRB, 0.28309E+00_JPRB, 0.28232E+00_JPRB, 0.28104E+00_JPRB, 0.27852E+00_JPRB ,&
&  0.27502E+00_JPRB, 0.27328E+00_JPRB, 0.27300E+00_JPRB, 0.27355E+00_JPRB, 0.27366E+00_JPRB, 0.27250E+00_JPRB, 0.27177E+00_JPRB ,&
&  0.27316E+00_JPRB, 0.27464E+00_JPRB, 0.27490E+00_JPRB, 0.27715E+00_JPRB, 0.28032E+00_JPRB, 0.28561E+00_JPRB, 0.29065E+00_JPRB ,&
&  0.29152E+00_JPRB, 0.29165E+00_JPRB, 0.28587E+00_JPRB, 0.29482E+00_JPRB, 0.29887E+00_JPRB, 0.30390E+00_JPRB, 0.30534E+00_JPRB ,&
&  0.30494E+00_JPRB, 0.30084E+00_JPRB, 0.29153E+00_JPRB, 0.28467E+00_JPRB, 0.28117E+00_JPRB, 0.27870E+00_JPRB, 0.27469E+00_JPRB ,&
&  0.27036E+00_JPRB, 0.26626E+00_JPRB, 0.26391E+00_JPRB, 0.26296E+00_JPRB, 0.26360E+00_JPRB, 0.26441E+00_JPRB, 0.26312E+00_JPRB ,&
&  0.26174E+00_JPRB, 0.26033E+00_JPRB, 0.25987E+00_JPRB, 0.25926E+00_JPRB, 0.25819E+00_JPRB, 0.25737E+00_JPRB, 0.25724E+00_JPRB ,&
&  0.25748E+00_JPRB, 0.25767E+00_JPRB, 0.25815E+00_JPRB, 0.25918E+00_JPRB, 0.25981E+00_JPRB, 0.25919E+00_JPRB, 0.25750E+00_JPRB ,&
&  0.25548E+00_JPRB, 0.25357E+00_JPRB, 0.25176E+00_JPRB, 0.24997E+00_JPRB, 0.24849E+00_JPRB, 0.24754E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24612E+00_JPRB, 0.24651E+00_JPRB, 0.24792E+00_JPRB, 0.24874E+00_JPRB, 0.24909E+00_JPRB, 0.24998E+00_JPRB, 0.25166E+00_JPRB ,&
&  0.25333E+00_JPRB, 0.25516E+00_JPRB, 0.25640E+00_JPRB, 0.25669E+00_JPRB, 0.25585E+00_JPRB, 0.25551E+00_JPRB, 0.25589E+00_JPRB ,&
&  0.25730E+00_JPRB, 0.25977E+00_JPRB, 0.26355E+00_JPRB, 0.26859E+00_JPRB, 0.27386E+00_JPRB, 0.27754E+00_JPRB, 0.28096E+00_JPRB ,&
&  0.28369E+00_JPRB, 0.28443E+00_JPRB, 0.28437E+00_JPRB, 0.28548E+00_JPRB, 0.28616E+00_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.30403E+00_JPRB, 0.30264E+00_JPRB, 0.30172E+00_JPRB, 0.29937E+00_JPRB, 0.29809E+00_JPRB, 0.29746E+00_JPRB, 0.29630E+00_JPRB ,&
&  0.29549E+00_JPRB, 0.29634E+00_JPRB, 0.29563E+00_JPRB, 0.29533E+00_JPRB, 0.29474E+00_JPRB, 0.29245E+00_JPRB, 0.28944E+00_JPRB ,&
&  0.28778E+00_JPRB, 0.28618E+00_JPRB, 0.28475E+00_JPRB, 0.28324E+00_JPRB, 0.28197E+00_JPRB, 0.28010E+00_JPRB, 0.28056E+00_JPRB ,&
&  0.28218E+00_JPRB, 0.28505E+00_JPRB, 0.28785E+00_JPRB, 0.29115E+00_JPRB, 0.29545E+00_JPRB, 0.30105E+00_JPRB, 0.30631E+00_JPRB ,&
&  0.31156E+00_JPRB, 0.30026E+00_JPRB, 0.30918E+00_JPRB, 0.31425E+00_JPRB, 0.32208E+00_JPRB, 0.32695E+00_JPRB, 0.32501E+00_JPRB ,&
&  0.32096E+00_JPRB, 0.31189E+00_JPRB, 0.30273E+00_JPRB, 0.29580E+00_JPRB, 0.29298E+00_JPRB, 0.29171E+00_JPRB, 0.28919E+00_JPRB ,&
&  0.28657E+00_JPRB, 0.28309E+00_JPRB, 0.28025E+00_JPRB, 0.27867E+00_JPRB, 0.27840E+00_JPRB, 0.27875E+00_JPRB, 0.27803E+00_JPRB ,&
&  0.27732E+00_JPRB, 0.27645E+00_JPRB, 0.27611E+00_JPRB, 0.27599E+00_JPRB, 0.27573E+00_JPRB, 0.27611E+00_JPRB, 0.27702E+00_JPRB ,&
&  0.27790E+00_JPRB, 0.27858E+00_JPRB, 0.27916E+00_JPRB, 0.28006E+00_JPRB, 0.28027E+00_JPRB, 0.27943E+00_JPRB, 0.27798E+00_JPRB ,&
&  0.27593E+00_JPRB, 0.27402E+00_JPRB, 0.27168E+00_JPRB, 0.27002E+00_JPRB, 0.26822E+00_JPRB, 0.26615E+00_JPRB, 0.26437E+00_JPRB ,&
&  0.26351E+00_JPRB, 0.26359E+00_JPRB, 0.26402E+00_JPRB, 0.26451E+00_JPRB, 0.26520E+00_JPRB, 0.26634E+00_JPRB, 0.26764E+00_JPRB ,&
&  0.26898E+00_JPRB, 0.27012E+00_JPRB, 0.27036E+00_JPRB, 0.27018E+00_JPRB, 0.26938E+00_JPRB, 0.26924E+00_JPRB, 0.27035E+00_JPRB ,&
&  0.27339E+00_JPRB, 0.27738E+00_JPRB, 0.28307E+00_JPRB, 0.29179E+00_JPRB, 0.29949E+00_JPRB, 0.30423E+00_JPRB, 0.30778E+00_JPRB ,&
&  0.30837E+00_JPRB, 0.30663E+00_JPRB, 0.30485E+00_JPRB, 0.30512E+00_JPRB, 0.30541E+00_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.31723E+00_JPRB, 0.31638E+00_JPRB, 0.31571E+00_JPRB, 0.31448E+00_JPRB, 0.31357E+00_JPRB, 0.31301E+00_JPRB, 0.31101E+00_JPRB ,&
&  0.31096E+00_JPRB, 0.30875E+00_JPRB, 0.30641E+00_JPRB, 0.30583E+00_JPRB, 0.30473E+00_JPRB, 0.30274E+00_JPRB, 0.30099E+00_JPRB ,&
&  0.29897E+00_JPRB, 0.29686E+00_JPRB, 0.29573E+00_JPRB, 0.29400E+00_JPRB, 0.29126E+00_JPRB, 0.28924E+00_JPRB, 0.28997E+00_JPRB ,&
&  0.29119E+00_JPRB, 0.29391E+00_JPRB, 0.30090E+00_JPRB, 0.30798E+00_JPRB, 0.31456E+00_JPRB, 0.32021E+00_JPRB, 0.32620E+00_JPRB ,&
&  0.33395E+00_JPRB, 0.29861E+00_JPRB, 0.31311E+00_JPRB, 0.32173E+00_JPRB, 0.33416E+00_JPRB, 0.34466E+00_JPRB, 0.34227E+00_JPRB ,&
&  0.33436E+00_JPRB, 0.32394E+00_JPRB, 0.31388E+00_JPRB, 0.30629E+00_JPRB, 0.30398E+00_JPRB, 0.30308E+00_JPRB, 0.30177E+00_JPRB ,&
&  0.30093E+00_JPRB, 0.29834E+00_JPRB, 0.29537E+00_JPRB, 0.29328E+00_JPRB, 0.29223E+00_JPRB, 0.29146E+00_JPRB, 0.29051E+00_JPRB ,&
&  0.29025E+00_JPRB, 0.29061E+00_JPRB, 0.29091E+00_JPRB, 0.29105E+00_JPRB, 0.29174E+00_JPRB, 0.29336E+00_JPRB, 0.29541E+00_JPRB ,&
&  0.29740E+00_JPRB, 0.29852E+00_JPRB, 0.29963E+00_JPRB, 0.30013E+00_JPRB, 0.30042E+00_JPRB, 0.30040E+00_JPRB, 0.29972E+00_JPRB ,&
&  0.29790E+00_JPRB, 0.29565E+00_JPRB, 0.29352E+00_JPRB, 0.29151E+00_JPRB, 0.28927E+00_JPRB, 0.28659E+00_JPRB, 0.28450E+00_JPRB ,&
&  0.28276E+00_JPRB, 0.28149E+00_JPRB, 0.27906E+00_JPRB, 0.27869E+00_JPRB, 0.27952E+00_JPRB, 0.28035E+00_JPRB, 0.28133E+00_JPRB ,&
&  0.28212E+00_JPRB, 0.28194E+00_JPRB, 0.28124E+00_JPRB, 0.28061E+00_JPRB, 0.28027E+00_JPRB, 0.28069E+00_JPRB, 0.28221E+00_JPRB ,&
&  0.28509E+00_JPRB, 0.29027E+00_JPRB, 0.29727E+00_JPRB, 0.30738E+00_JPRB, 0.31600E+00_JPRB, 0.32110E+00_JPRB, 0.32385E+00_JPRB ,&
&  0.32272E+00_JPRB, 0.31997E+00_JPRB, 0.31770E+00_JPRB, 0.31615E+00_JPRB, 0.31706E+00_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.33625E+00_JPRB, 0.33432E+00_JPRB, 0.33340E+00_JPRB, 0.33168E+00_JPRB, 0.33014E+00_JPRB, 0.32847E+00_JPRB, 0.32639E+00_JPRB ,&
&  0.32360E+00_JPRB, 0.32201E+00_JPRB, 0.32035E+00_JPRB, 0.31796E+00_JPRB, 0.31546E+00_JPRB, 0.31330E+00_JPRB, 0.31095E+00_JPRB ,&
&  0.30845E+00_JPRB, 0.30586E+00_JPRB, 0.30439E+00_JPRB, 0.30352E+00_JPRB, 0.30226E+00_JPRB, 0.30201E+00_JPRB, 0.30323E+00_JPRB ,&
&  0.30372E+00_JPRB, 0.30617E+00_JPRB, 0.31520E+00_JPRB, 0.32502E+00_JPRB, 0.33507E+00_JPRB, 0.34287E+00_JPRB, 0.35493E+00_JPRB ,&
&  0.33967E+00_JPRB, 0.30269E+00_JPRB, 0.31859E+00_JPRB, 0.33408E+00_JPRB, 0.34832E+00_JPRB, 0.36167E+00_JPRB, 0.36056E+00_JPRB ,&
&  0.35800E+00_JPRB, 0.35060E+00_JPRB, 0.34107E+00_JPRB, 0.33272E+00_JPRB, 0.32886E+00_JPRB, 0.32581E+00_JPRB, 0.32273E+00_JPRB ,&
&  0.31985E+00_JPRB, 0.31583E+00_JPRB, 0.31231E+00_JPRB, 0.30937E+00_JPRB, 0.30763E+00_JPRB, 0.30609E+00_JPRB, 0.30460E+00_JPRB ,&
&  0.30436E+00_JPRB, 0.30481E+00_JPRB, 0.30526E+00_JPRB, 0.30573E+00_JPRB, 0.30675E+00_JPRB, 0.30949E+00_JPRB, 0.31331E+00_JPRB ,&
&  0.31756E+00_JPRB, 0.32200E+00_JPRB, 0.32616E+00_JPRB, 0.32919E+00_JPRB, 0.33138E+00_JPRB, 0.33192E+00_JPRB, 0.33006E+00_JPRB ,&
&  0.32732E+00_JPRB, 0.32359E+00_JPRB, 0.31977E+00_JPRB, 0.31553E+00_JPRB, 0.31053E+00_JPRB, 0.30589E+00_JPRB, 0.30226E+00_JPRB ,&
&  0.29956E+00_JPRB, 0.29687E+00_JPRB, 0.29481E+00_JPRB, 0.29445E+00_JPRB, 0.29517E+00_JPRB, 0.29563E+00_JPRB, 0.29561E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29452E+00_JPRB, 0.29405E+00_JPRB, 0.29434E+00_JPRB, 0.29527E+00_JPRB, 0.29729E+00_JPRB, 0.29976E+00_JPRB ,&
&  0.30231E+00_JPRB, 0.30747E+00_JPRB, 0.31465E+00_JPRB, 0.32694E+00_JPRB, 0.33572E+00_JPRB, 0.34015E+00_JPRB, 0.34147E+00_JPRB ,&
&  0.34024E+00_JPRB, 0.33811E+00_JPRB, 0.33729E+00_JPRB, 0.33570E+00_JPRB, 0.33603E+00_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.35708E+00_JPRB, 0.35755E+00_JPRB, 0.35468E+00_JPRB, 0.34811E+00_JPRB, 0.34321E+00_JPRB, 0.33985E+00_JPRB, 0.33780E+00_JPRB ,&
&  0.33685E+00_JPRB, 0.33600E+00_JPRB, 0.33441E+00_JPRB, 0.33080E+00_JPRB, 0.32777E+00_JPRB, 0.32367E+00_JPRB, 0.31932E+00_JPRB ,&
&  0.31582E+00_JPRB, 0.31264E+00_JPRB, 0.31111E+00_JPRB, 0.30882E+00_JPRB, 0.30782E+00_JPRB, 0.30917E+00_JPRB, 0.31129E+00_JPRB ,&
&  0.31403E+00_JPRB, 0.31680E+00_JPRB, 0.32518E+00_JPRB, 0.33431E+00_JPRB, 0.34989E+00_JPRB, 0.36006E+00_JPRB, 0.37237E+00_JPRB ,&
&  0.39986E+00_JPRB, 0.33286E+00_JPRB, 0.34689E+00_JPRB, 0.36473E+00_JPRB, 0.37098E+00_JPRB, 0.38230E+00_JPRB, 0.38116E+00_JPRB ,&
&  0.38087E+00_JPRB, 0.37923E+00_JPRB, 0.37225E+00_JPRB, 0.36487E+00_JPRB, 0.36048E+00_JPRB, 0.35382E+00_JPRB, 0.34594E+00_JPRB ,&
&  0.33671E+00_JPRB, 0.33041E+00_JPRB, 0.32571E+00_JPRB, 0.32171E+00_JPRB, 0.31850E+00_JPRB, 0.31669E+00_JPRB, 0.31601E+00_JPRB ,&
&  0.31643E+00_JPRB, 0.31682E+00_JPRB, 0.31680E+00_JPRB, 0.31745E+00_JPRB, 0.31944E+00_JPRB, 0.32362E+00_JPRB, 0.32981E+00_JPRB ,&
&  0.33909E+00_JPRB, 0.34873E+00_JPRB, 0.35643E+00_JPRB, 0.36098E+00_JPRB, 0.36319E+00_JPRB, 0.36138E+00_JPRB, 0.35716E+00_JPRB ,&
&  0.35098E+00_JPRB, 0.34351E+00_JPRB, 0.33707E+00_JPRB, 0.33146E+00_JPRB, 0.32649E+00_JPRB, 0.32199E+00_JPRB, 0.31713E+00_JPRB ,&
&  0.31363E+00_JPRB, 0.31172E+00_JPRB, 0.31161E+00_JPRB, 0.31176E+00_JPRB, 0.31153E+00_JPRB, 0.31118E+00_JPRB, 0.31132E+00_JPRB ,&
&  0.31109E+00_JPRB, 0.31092E+00_JPRB, 0.31115E+00_JPRB, 0.31245E+00_JPRB, 0.31459E+00_JPRB, 0.31662E+00_JPRB, 0.31841E+00_JPRB ,&
&  0.32044E+00_JPRB, 0.32484E+00_JPRB, 0.33035E+00_JPRB, 0.34430E+00_JPRB, 0.34963E+00_JPRB, 0.35251E+00_JPRB, 0.35506E+00_JPRB ,&
&  0.35672E+00_JPRB, 0.35671E+00_JPRB, 0.31866E+00_JPRB, 0.34111E+00_JPRB, 0.35163E+00_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.38495E+00_JPRB, 0.38304E+00_JPRB, 0.37553E+00_JPRB, 0.36692E+00_JPRB, 0.36198E+00_JPRB, 0.35946E+00_JPRB, 0.35805E+00_JPRB ,&
&  0.35589E+00_JPRB, 0.35279E+00_JPRB, 0.34927E+00_JPRB, 0.34727E+00_JPRB, 0.34454E+00_JPRB, 0.34038E+00_JPRB, 0.33524E+00_JPRB ,&
&  0.33053E+00_JPRB, 0.32604E+00_JPRB, 0.32275E+00_JPRB, 0.31882E+00_JPRB, 0.31569E+00_JPRB, 0.31627E+00_JPRB, 0.31820E+00_JPRB ,&
&  0.32361E+00_JPRB, 0.33111E+00_JPRB, 0.33811E+00_JPRB, 0.35144E+00_JPRB, 0.36571E+00_JPRB, 0.38391E+00_JPRB, 0.39992E+00_JPRB ,&
&  0.42746E+00_JPRB, 0.35832E+00_JPRB, 0.36517E+00_JPRB, 0.38914E+00_JPRB, 0.39641E+00_JPRB, 0.40810E+00_JPRB, 0.41206E+00_JPRB ,&
&  0.41460E+00_JPRB, 0.41689E+00_JPRB, 0.41460E+00_JPRB, 0.40824E+00_JPRB, 0.39738E+00_JPRB, 0.37943E+00_JPRB, 0.36200E+00_JPRB ,&
&  0.34902E+00_JPRB, 0.34487E+00_JPRB, 0.34315E+00_JPRB, 0.34071E+00_JPRB, 0.33845E+00_JPRB, 0.33807E+00_JPRB, 0.33958E+00_JPRB ,&
&  0.34049E+00_JPRB, 0.34075E+00_JPRB, 0.34040E+00_JPRB, 0.34024E+00_JPRB, 0.34216E+00_JPRB, 0.35010E+00_JPRB, 0.36427E+00_JPRB ,&
&  0.38143E+00_JPRB, 0.39583E+00_JPRB, 0.40502E+00_JPRB, 0.40849E+00_JPRB, 0.40603E+00_JPRB, 0.39817E+00_JPRB, 0.38518E+00_JPRB ,&
&  0.37108E+00_JPRB, 0.35981E+00_JPRB, 0.35291E+00_JPRB, 0.34772E+00_JPRB, 0.34350E+00_JPRB, 0.33980E+00_JPRB, 0.33774E+00_JPRB ,&
&  0.33728E+00_JPRB, 0.33746E+00_JPRB, 0.33725E+00_JPRB, 0.33581E+00_JPRB, 0.33513E+00_JPRB, 0.33474E+00_JPRB, 0.33496E+00_JPRB ,&
&  0.33527E+00_JPRB, 0.33555E+00_JPRB, 0.33679E+00_JPRB, 0.33847E+00_JPRB, 0.33954E+00_JPRB, 0.33977E+00_JPRB, 0.34063E+00_JPRB ,&
&  0.34438E+00_JPRB, 0.35055E+00_JPRB, 0.36087E+00_JPRB, 0.37376E+00_JPRB, 0.38362E+00_JPRB, 0.37978E+00_JPRB, 0.37698E+00_JPRB ,&
&  0.37696E+00_JPRB, 0.37634E+00_JPRB, 0.37914E+00_JPRB, 0.35587E+00_JPRB, 0.38219E+00_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.40884E+00_JPRB, 0.41000E+00_JPRB, 0.40210E+00_JPRB, 0.39170E+00_JPRB, 0.38199E+00_JPRB, 0.37623E+00_JPRB, 0.37186E+00_JPRB ,&
&  0.36940E+00_JPRB, 0.36797E+00_JPRB, 0.36683E+00_JPRB, 0.36514E+00_JPRB, 0.36226E+00_JPRB, 0.35757E+00_JPRB, 0.35143E+00_JPRB ,&
&  0.34595E+00_JPRB, 0.34040E+00_JPRB, 0.33599E+00_JPRB, 0.33079E+00_JPRB, 0.32421E+00_JPRB, 0.32138E+00_JPRB, 0.32434E+00_JPRB ,&
&  0.33007E+00_JPRB, 0.33722E+00_JPRB, 0.34398E+00_JPRB, 0.35428E+00_JPRB, 0.37461E+00_JPRB, 0.39195E+00_JPRB, 0.40786E+00_JPRB ,&
&  0.43747E+00_JPRB, 0.38269E+00_JPRB, 0.39368E+00_JPRB, 0.40728E+00_JPRB, 0.41519E+00_JPRB, 0.42398E+00_JPRB, 0.43915E+00_JPRB ,&
&  0.45380E+00_JPRB, 0.46211E+00_JPRB, 0.46397E+00_JPRB, 0.45115E+00_JPRB, 0.42579E+00_JPRB, 0.40400E+00_JPRB, 0.39111E+00_JPRB ,&
&  0.38438E+00_JPRB, 0.38178E+00_JPRB, 0.37915E+00_JPRB, 0.37718E+00_JPRB, 0.37652E+00_JPRB, 0.37601E+00_JPRB, 0.37535E+00_JPRB ,&
&  0.37469E+00_JPRB, 0.37424E+00_JPRB, 0.37444E+00_JPRB, 0.37612E+00_JPRB, 0.38186E+00_JPRB, 0.39339E+00_JPRB, 0.40789E+00_JPRB ,&
&  0.42586E+00_JPRB, 0.43789E+00_JPRB, 0.44087E+00_JPRB, 0.44077E+00_JPRB, 0.43810E+00_JPRB, 0.42821E+00_JPRB, 0.40955E+00_JPRB ,&
&  0.39289E+00_JPRB, 0.37943E+00_JPRB, 0.37041E+00_JPRB, 0.36461E+00_JPRB, 0.36114E+00_JPRB, 0.35983E+00_JPRB, 0.36057E+00_JPRB ,&
&  0.36153E+00_JPRB, 0.36139E+00_JPRB, 0.36174E+00_JPRB, 0.36153E+00_JPRB, 0.36194E+00_JPRB, 0.36179E+00_JPRB, 0.35993E+00_JPRB ,&
&  0.35756E+00_JPRB, 0.35662E+00_JPRB, 0.35616E+00_JPRB, 0.35581E+00_JPRB, 0.35597E+00_JPRB, 0.35507E+00_JPRB, 0.35513E+00_JPRB ,&
&  0.35811E+00_JPRB, 0.36971E+00_JPRB, 0.38532E+00_JPRB, 0.41340E+00_JPRB, 0.41029E+00_JPRB, 0.40167E+00_JPRB, 0.39281E+00_JPRB ,&
&  0.38963E+00_JPRB, 0.39199E+00_JPRB, 0.38908E+00_JPRB, 0.39378E+00_JPRB, 0.40683E+00_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.46454E+00_JPRB, 0.45298E+00_JPRB, 0.43909E+00_JPRB, 0.42194E+00_JPRB, 0.40652E+00_JPRB, 0.39552E+00_JPRB, 0.38781E+00_JPRB ,&
&  0.38338E+00_JPRB, 0.38107E+00_JPRB, 0.37964E+00_JPRB, 0.37846E+00_JPRB, 0.37646E+00_JPRB, 0.37426E+00_JPRB, 0.36863E+00_JPRB ,&
&  0.36187E+00_JPRB, 0.35424E+00_JPRB, 0.34844E+00_JPRB, 0.34329E+00_JPRB, 0.33352E+00_JPRB, 0.32852E+00_JPRB, 0.33109E+00_JPRB ,&
&  0.33630E+00_JPRB, 0.34282E+00_JPRB, 0.34791E+00_JPRB, 0.35875E+00_JPRB, 0.37729E+00_JPRB, 0.39309E+00_JPRB, 0.41283E+00_JPRB ,&
&  0.44703E+00_JPRB, 0.39175E+00_JPRB, 0.41849E+00_JPRB, 0.42762E+00_JPRB, 0.42880E+00_JPRB, 0.45157E+00_JPRB, 0.48416E+00_JPRB ,&
&  0.51104E+00_JPRB, 0.54137E+00_JPRB, 0.53731E+00_JPRB, 0.50785E+00_JPRB, 0.47864E+00_JPRB, 0.45634E+00_JPRB, 0.44251E+00_JPRB ,&
&  0.43123E+00_JPRB, 0.42161E+00_JPRB, 0.41296E+00_JPRB, 0.40842E+00_JPRB, 0.40392E+00_JPRB, 0.39966E+00_JPRB, 0.39538E+00_JPRB ,&
&  0.39392E+00_JPRB, 0.39347E+00_JPRB, 0.39415E+00_JPRB, 0.40065E+00_JPRB, 0.41348E+00_JPRB, 0.43137E+00_JPRB, 0.45120E+00_JPRB ,&
&  0.46811E+00_JPRB, 0.47389E+00_JPRB, 0.47787E+00_JPRB, 0.47166E+00_JPRB, 0.45870E+00_JPRB, 0.44505E+00_JPRB, 0.43258E+00_JPRB ,&
&  0.41880E+00_JPRB, 0.40385E+00_JPRB, 0.39360E+00_JPRB, 0.38726E+00_JPRB, 0.38397E+00_JPRB, 0.38326E+00_JPRB, 0.38375E+00_JPRB ,&
&  0.38317E+00_JPRB, 0.38353E+00_JPRB, 0.38689E+00_JPRB, 0.39131E+00_JPRB, 0.39179E+00_JPRB, 0.38897E+00_JPRB, 0.38372E+00_JPRB ,&
&  0.37934E+00_JPRB, 0.37546E+00_JPRB, 0.37483E+00_JPRB, 0.37573E+00_JPRB, 0.37745E+00_JPRB, 0.38014E+00_JPRB, 0.38200E+00_JPRB ,&
&  0.38694E+00_JPRB, 0.40441E+00_JPRB, 0.41105E+00_JPRB, 0.43181E+00_JPRB, 0.45781E+00_JPRB, 0.44690E+00_JPRB, 0.43139E+00_JPRB ,&
&  0.42028E+00_JPRB, 0.41634E+00_JPRB, 0.41021E+00_JPRB, 0.41510E+00_JPRB, 0.47843E+00_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.44356E+00_JPRB, 0.43670E+00_JPRB, 0.42915E+00_JPRB, 0.42269E+00_JPRB, 0.41277E+00_JPRB, 0.40296E+00_JPRB, 0.39318E+00_JPRB ,&
&  0.38628E+00_JPRB, 0.38321E+00_JPRB, 0.38415E+00_JPRB, 0.38550E+00_JPRB, 0.38416E+00_JPRB, 0.37837E+00_JPRB, 0.36718E+00_JPRB ,&
&  0.36006E+00_JPRB, 0.35504E+00_JPRB, 0.34940E+00_JPRB, 0.34227E+00_JPRB, 0.33527E+00_JPRB, 0.33307E+00_JPRB, 0.33456E+00_JPRB ,&
&  0.33951E+00_JPRB, 0.34595E+00_JPRB, 0.35055E+00_JPRB, 0.36233E+00_JPRB, 0.37747E+00_JPRB, 0.39356E+00_JPRB, 0.41527E+00_JPRB ,&
&  0.44106E+00_JPRB, 0.42610E+00_JPRB, 0.45449E+00_JPRB, 0.46959E+00_JPRB, 0.47065E+00_JPRB, 0.49658E+00_JPRB, 0.54856E+00_JPRB ,&
&  0.59923E+00_JPRB, 0.63283E+00_JPRB, 0.62512E+00_JPRB, 0.57978E+00_JPRB, 0.54270E+00_JPRB, 0.51532E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.46106E+00_JPRB, 0.44034E+00_JPRB, 0.42403E+00_JPRB, 0.41302E+00_JPRB, 0.40478E+00_JPRB, 0.39919E+00_JPRB, 0.39691E+00_JPRB ,&
&  0.39820E+00_JPRB, 0.40075E+00_JPRB, 0.40691E+00_JPRB, 0.41842E+00_JPRB, 0.44376E+00_JPRB, 0.47087E+00_JPRB, 0.49447E+00_JPRB ,&
&  0.50595E+00_JPRB, 0.51347E+00_JPRB, 0.51676E+00_JPRB, 0.50631E+00_JPRB, 0.48767E+00_JPRB, 0.46500E+00_JPRB, 0.44690E+00_JPRB ,&
&  0.43389E+00_JPRB, 0.42205E+00_JPRB, 0.41226E+00_JPRB, 0.40336E+00_JPRB, 0.39828E+00_JPRB, 0.39691E+00_JPRB, 0.39756E+00_JPRB ,&
&  0.39698E+00_JPRB, 0.39832E+00_JPRB, 0.40073E+00_JPRB, 0.40235E+00_JPRB, 0.40221E+00_JPRB, 0.40125E+00_JPRB, 0.39882E+00_JPRB ,&
&  0.39557E+00_JPRB, 0.39327E+00_JPRB, 0.39797E+00_JPRB, 0.40164E+00_JPRB, 0.40536E+00_JPRB, 0.40638E+00_JPRB, 0.40541E+00_JPRB ,&
&  0.40879E+00_JPRB, 0.42041E+00_JPRB, 0.42647E+00_JPRB, 0.43899E+00_JPRB, 0.44901E+00_JPRB, 0.46308E+00_JPRB, 0.44910E+00_JPRB ,&
&  0.44121E+00_JPRB, 0.42944E+00_JPRB, 0.41952E+00_JPRB, 0.42011E+00_JPRB, 0.44525E+00_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.43387E+00_JPRB, 0.42971E+00_JPRB, 0.42326E+00_JPRB, 0.41693E+00_JPRB, 0.40881E+00_JPRB, 0.39929E+00_JPRB, 0.39241E+00_JPRB ,&
&  0.38903E+00_JPRB, 0.38997E+00_JPRB, 0.39207E+00_JPRB, 0.38494E+00_JPRB, 0.37407E+00_JPRB, 0.36383E+00_JPRB, 0.35776E+00_JPRB ,&
&  0.35345E+00_JPRB, 0.34931E+00_JPRB, 0.34399E+00_JPRB, 0.33688E+00_JPRB, 0.33405E+00_JPRB, 0.33378E+00_JPRB, 0.33420E+00_JPRB ,&
&  0.33957E+00_JPRB, 0.34481E+00_JPRB, 0.35167E+00_JPRB, 0.36064E+00_JPRB, 0.37355E+00_JPRB, 0.38949E+00_JPRB, 0.41078E+00_JPRB ,&
&  0.44030E+00_JPRB, 0.42564E+00_JPRB, 0.51401E+00_JPRB, 0.50804E+00_JPRB, 0.53390E+00_JPRB, 0.57099E+00_JPRB, 0.64874E+00_JPRB ,&
&  0.71712E+00_JPRB, 0.73774E+00_JPRB, 0.70503E+00_JPRB, 0.64452E+00_JPRB, 0.58473E+00_JPRB, 0.53904E+00_JPRB, 0.50818E+00_JPRB ,&
&  0.47910E+00_JPRB, 0.45274E+00_JPRB, 0.43476E+00_JPRB, 0.42284E+00_JPRB, 0.41469E+00_JPRB, 0.40990E+00_JPRB, 0.40835E+00_JPRB ,&
&  0.41072E+00_JPRB, 0.41706E+00_JPRB, 0.42878E+00_JPRB, 0.44696E+00_JPRB, 0.46141E+00_JPRB, 0.44834E+00_JPRB, 0.47028E+00_JPRB ,&
&  0.53556E+00_JPRB, 0.54096E+00_JPRB, 0.53718E+00_JPRB, 0.52173E+00_JPRB, 0.50115E+00_JPRB, 0.48005E+00_JPRB, 0.45960E+00_JPRB ,&
&  0.44419E+00_JPRB, 0.43263E+00_JPRB, 0.42193E+00_JPRB, 0.41555E+00_JPRB, 0.41339E+00_JPRB, 0.41528E+00_JPRB, 0.41738E+00_JPRB ,&
&  0.41732E+00_JPRB, 0.41708E+00_JPRB, 0.41793E+00_JPRB, 0.41976E+00_JPRB, 0.42199E+00_JPRB, 0.42472E+00_JPRB, 0.42022E+00_JPRB ,&
&  0.41690E+00_JPRB, 0.41790E+00_JPRB, 0.42194E+00_JPRB, 0.41508E+00_JPRB, 0.41827E+00_JPRB, 0.42153E+00_JPRB, 0.41927E+00_JPRB ,&
&  0.41254E+00_JPRB, 0.41451E+00_JPRB, 0.42576E+00_JPRB, 0.43505E+00_JPRB, 0.45487E+00_JPRB, 0.47492E+00_JPRB, 0.45206E+00_JPRB ,&
&  0.44947E+00_JPRB, 0.43931E+00_JPRB, 0.42899E+00_JPRB, 0.42896E+00_JPRB, 0.44007E+00_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.42654E+00_JPRB, 0.42171E+00_JPRB, 0.41615E+00_JPRB, 0.41060E+00_JPRB, 0.40335E+00_JPRB, 0.39619E+00_JPRB, 0.39194E+00_JPRB ,&
&  0.39213E+00_JPRB, 0.39604E+00_JPRB, 0.39435E+00_JPRB, 0.38458E+00_JPRB, 0.37207E+00_JPRB, 0.36071E+00_JPRB, 0.35538E+00_JPRB ,&
&  0.35116E+00_JPRB, 0.34496E+00_JPRB, 0.33666E+00_JPRB, 0.32769E+00_JPRB, 0.32535E+00_JPRB, 0.32818E+00_JPRB, 0.32583E+00_JPRB ,&
&  0.33177E+00_JPRB, 0.33867E+00_JPRB, 0.34581E+00_JPRB, 0.35233E+00_JPRB, 0.36476E+00_JPRB, 0.38117E+00_JPRB, 0.40040E+00_JPRB ,&
&  0.43224E+00_JPRB, 0.36993E+00_JPRB, 0.53523E+00_JPRB, 0.60061E+00_JPRB, 0.65462E+00_JPRB, 0.70485E+00_JPRB, 0.78295E+00_JPRB ,&
&  0.84895E+00_JPRB, 0.83146E+00_JPRB, 0.78390E+00_JPRB, 0.69323E+00_JPRB, 0.60770E+00_JPRB, 0.54808E+00_JPRB, 0.51136E+00_JPRB ,&
&  0.48894E+00_JPRB, 0.47202E+00_JPRB, 0.45625E+00_JPRB, 0.44237E+00_JPRB, 0.43246E+00_JPRB, 0.42644E+00_JPRB, 0.42481E+00_JPRB ,&
&  0.43169E+00_JPRB, 0.44510E+00_JPRB, 0.45941E+00_JPRB, 0.48550E+00_JPRB, 0.47565E+00_JPRB, 0.47161E+00_JPRB, 0.47083E+00_JPRB ,&
&  0.50781E+00_JPRB, 0.56432E+00_JPRB, 0.54694E+00_JPRB, 0.51974E+00_JPRB, 0.50437E+00_JPRB, 0.48851E+00_JPRB, 0.47020E+00_JPRB ,&
&  0.45368E+00_JPRB, 0.44323E+00_JPRB, 0.43559E+00_JPRB, 0.43096E+00_JPRB, 0.43160E+00_JPRB, 0.43202E+00_JPRB, 0.42877E+00_JPRB ,&
&  0.42803E+00_JPRB, 0.42897E+00_JPRB, 0.43127E+00_JPRB, 0.43499E+00_JPRB, 0.43585E+00_JPRB, 0.43680E+00_JPRB, 0.43575E+00_JPRB ,&
&  0.43467E+00_JPRB, 0.43607E+00_JPRB, 0.42954E+00_JPRB, 0.42001E+00_JPRB, 0.41918E+00_JPRB, 0.41705E+00_JPRB, 0.41543E+00_JPRB ,&
&  0.41840E+00_JPRB, 0.42277E+00_JPRB, 0.42794E+00_JPRB, 0.44240E+00_JPRB, 0.44833E+00_JPRB, 0.46988E+00_JPRB, 0.45289E+00_JPRB ,&
&  0.45557E+00_JPRB, 0.44964E+00_JPRB, 0.44427E+00_JPRB, 0.43683E+00_JPRB, 0.43569E+00_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.42696E+00_JPRB, 0.42103E+00_JPRB, 0.41535E+00_JPRB, 0.40844E+00_JPRB, 0.39958E+00_JPRB, 0.39409E+00_JPRB, 0.39641E+00_JPRB ,&
&  0.40234E+00_JPRB, 0.40021E+00_JPRB, 0.39570E+00_JPRB, 0.38640E+00_JPRB, 0.37173E+00_JPRB, 0.36542E+00_JPRB, 0.36173E+00_JPRB ,&
&  0.35477E+00_JPRB, 0.34958E+00_JPRB, 0.34037E+00_JPRB, 0.33560E+00_JPRB, 0.33407E+00_JPRB, 0.33115E+00_JPRB, 0.32537E+00_JPRB ,&
&  0.32847E+00_JPRB, 0.33519E+00_JPRB, 0.34331E+00_JPRB, 0.35021E+00_JPRB, 0.36299E+00_JPRB, 0.37925E+00_JPRB, 0.39940E+00_JPRB ,&
&  0.43348E+00_JPRB, 0.52797E+00_JPRB, 0.50850E+00_JPRB, 0.76075E+00_JPRB, 0.79786E+00_JPRB, 0.84655E+00_JPRB, 0.84798E+00_JPRB ,&
&  0.85218E+00_JPRB, 0.87499E+00_JPRB, 0.85124E+00_JPRB, 0.73362E+00_JPRB, 0.62214E+00_JPRB, 0.55716E+00_JPRB, 0.51727E+00_JPRB ,&
&  0.49030E+00_JPRB, 0.46932E+00_JPRB, 0.45299E+00_JPRB, 0.44106E+00_JPRB, 0.43139E+00_JPRB, 0.42841E+00_JPRB, 0.44104E+00_JPRB ,&
&  0.46069E+00_JPRB, 0.47807E+00_JPRB, 0.50128E+00_JPRB, 0.52270E+00_JPRB, 0.48749E+00_JPRB, 0.48839E+00_JPRB, 0.50191E+00_JPRB ,&
&  0.51590E+00_JPRB, 0.57148E+00_JPRB, 0.53034E+00_JPRB, 0.50948E+00_JPRB, 0.50103E+00_JPRB, 0.49343E+00_JPRB, 0.47924E+00_JPRB ,&
&  0.46465E+00_JPRB, 0.45783E+00_JPRB, 0.45307E+00_JPRB, 0.45130E+00_JPRB, 0.44881E+00_JPRB, 0.44520E+00_JPRB, 0.44054E+00_JPRB ,&
&  0.43644E+00_JPRB, 0.44121E+00_JPRB, 0.44563E+00_JPRB, 0.44761E+00_JPRB, 0.44890E+00_JPRB, 0.44692E+00_JPRB, 0.44491E+00_JPRB ,&
&  0.44566E+00_JPRB, 0.44883E+00_JPRB, 0.43038E+00_JPRB, 0.41741E+00_JPRB, 0.41341E+00_JPRB, 0.41127E+00_JPRB, 0.40996E+00_JPRB ,&
&  0.42024E+00_JPRB, 0.42782E+00_JPRB, 0.43264E+00_JPRB, 0.44459E+00_JPRB, 0.46295E+00_JPRB, 0.48500E+00_JPRB, 0.46687E+00_JPRB ,&
&  0.47549E+00_JPRB, 0.46867E+00_JPRB, 0.45925E+00_JPRB, 0.44377E+00_JPRB, 0.43624E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.43212E+00_JPRB, 0.42326E+00_JPRB, 0.41759E+00_JPRB, 0.41187E+00_JPRB, 0.40636E+00_JPRB, 0.40492E+00_JPRB, 0.40769E+00_JPRB ,&
&  0.40893E+00_JPRB, 0.40386E+00_JPRB, 0.39513E+00_JPRB, 0.38637E+00_JPRB, 0.37154E+00_JPRB, 0.36313E+00_JPRB, 0.35932E+00_JPRB ,&
&  0.35643E+00_JPRB, 0.35725E+00_JPRB, 0.35370E+00_JPRB, 0.35077E+00_JPRB, 0.34820E+00_JPRB, 0.34330E+00_JPRB, 0.34259E+00_JPRB ,&
&  0.34087E+00_JPRB, 0.34358E+00_JPRB, 0.34967E+00_JPRB, 0.36025E+00_JPRB, 0.37521E+00_JPRB, 0.39485E+00_JPRB, 0.41479E+00_JPRB ,&
&  0.46322E+00_JPRB, 0.52115E+00_JPRB, 0.56295E+00_JPRB, 0.99804E+00_JPRB, 0.99804E+00_JPRB, 0.96013E+00_JPRB, 0.86964E+00_JPRB ,&
&  0.83266E+00_JPRB, 0.88503E+00_JPRB, 0.86986E+00_JPRB, 0.77023E+00_JPRB, 0.66201E+00_JPRB, 0.58335E+00_JPRB, 0.52167E+00_JPRB ,&
&  0.48236E+00_JPRB, 0.45649E+00_JPRB, 0.44341E+00_JPRB, 0.43802E+00_JPRB, 0.43622E+00_JPRB, 0.45210E+00_JPRB, 0.47748E+00_JPRB ,&
&  0.49384E+00_JPRB, 0.51418E+00_JPRB, 0.54180E+00_JPRB, 0.52032E+00_JPRB, 0.49256E+00_JPRB, 0.50155E+00_JPRB, 0.51026E+00_JPRB ,&
&  0.52105E+00_JPRB, 0.53190E+00_JPRB, 0.51100E+00_JPRB, 0.49831E+00_JPRB, 0.48412E+00_JPRB, 0.49040E+00_JPRB, 0.48355E+00_JPRB ,&
&  0.47415E+00_JPRB, 0.46971E+00_JPRB, 0.46582E+00_JPRB, 0.46329E+00_JPRB, 0.45853E+00_JPRB, 0.45495E+00_JPRB, 0.44507E+00_JPRB ,&
&  0.44725E+00_JPRB, 0.45058E+00_JPRB, 0.45165E+00_JPRB, 0.45593E+00_JPRB, 0.45132E+00_JPRB, 0.44940E+00_JPRB, 0.44863E+00_JPRB ,&
&  0.45430E+00_JPRB, 0.45447E+00_JPRB, 0.43032E+00_JPRB, 0.41749E+00_JPRB, 0.41484E+00_JPRB, 0.41249E+00_JPRB, 0.41190E+00_JPRB ,&
&  0.42323E+00_JPRB, 0.43604E+00_JPRB, 0.44367E+00_JPRB, 0.44915E+00_JPRB, 0.47257E+00_JPRB, 0.48794E+00_JPRB, 0.49443E+00_JPRB ,&
&  0.49156E+00_JPRB, 0.48673E+00_JPRB, 0.47717E+00_JPRB, 0.45324E+00_JPRB, 0.44295E+00_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.43429E+00_JPRB, 0.42339E+00_JPRB, 0.41472E+00_JPRB, 0.41211E+00_JPRB, 0.40792E+00_JPRB, 0.40500E+00_JPRB, 0.41102E+00_JPRB ,&
&  0.40750E+00_JPRB, 0.39873E+00_JPRB, 0.38423E+00_JPRB, 0.36880E+00_JPRB, 0.35614E+00_JPRB, 0.35769E+00_JPRB, 0.36236E+00_JPRB ,&
&  0.36284E+00_JPRB, 0.36156E+00_JPRB, 0.36009E+00_JPRB, 0.36072E+00_JPRB, 0.35921E+00_JPRB, 0.35863E+00_JPRB, 0.35783E+00_JPRB ,&
&  0.35739E+00_JPRB, 0.36081E+00_JPRB, 0.37224E+00_JPRB, 0.39091E+00_JPRB, 0.41183E+00_JPRB, 0.43992E+00_JPRB, 0.47386E+00_JPRB ,&
&  0.54605E+00_JPRB, 0.56648E+00_JPRB, 0.62844E+00_JPRB, 0.96005E+00_JPRB, 0.10269E+01_JPRB, 0.93717E+00_JPRB, 0.86204E+00_JPRB ,&
&  0.78085E+00_JPRB, 0.73585E+00_JPRB, 0.84568E+00_JPRB, 0.76236E+00_JPRB, 0.66402E+00_JPRB, 0.57671E+00_JPRB, 0.51438E+00_JPRB ,&
&  0.47786E+00_JPRB, 0.45230E+00_JPRB, 0.44534E+00_JPRB, 0.45032E+00_JPRB, 0.47161E+00_JPRB, 0.50459E+00_JPRB, 0.52389E+00_JPRB ,&
&  0.53754E+00_JPRB, 0.55682E+00_JPRB, 0.58318E+00_JPRB, 0.53311E+00_JPRB, 0.48214E+00_JPRB, 0.49290E+00_JPRB, 0.49327E+00_JPRB ,&
&  0.50545E+00_JPRB, 0.50335E+00_JPRB, 0.50484E+00_JPRB, 0.49433E+00_JPRB, 0.47591E+00_JPRB, 0.50005E+00_JPRB, 0.48484E+00_JPRB ,&
&  0.48078E+00_JPRB, 0.47660E+00_JPRB, 0.47233E+00_JPRB, 0.47037E+00_JPRB, 0.46309E+00_JPRB, 0.44924E+00_JPRB, 0.44053E+00_JPRB ,&
&  0.44295E+00_JPRB, 0.44949E+00_JPRB, 0.44809E+00_JPRB, 0.44776E+00_JPRB, 0.44275E+00_JPRB, 0.44412E+00_JPRB, 0.44830E+00_JPRB ,&
&  0.45728E+00_JPRB, 0.45912E+00_JPRB, 0.44171E+00_JPRB, 0.43577E+00_JPRB, 0.43375E+00_JPRB, 0.43265E+00_JPRB, 0.43067E+00_JPRB ,&
&  0.43791E+00_JPRB, 0.45406E+00_JPRB, 0.46607E+00_JPRB, 0.47825E+00_JPRB, 0.49492E+00_JPRB, 0.50752E+00_JPRB, 0.51404E+00_JPRB ,&
&  0.52000E+00_JPRB, 0.53882E+00_JPRB, 0.54634E+00_JPRB, 0.48122E+00_JPRB, 0.45400E+00_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.41769E+00_JPRB, 0.40873E+00_JPRB, 0.40509E+00_JPRB, 0.40474E+00_JPRB, 0.40245E+00_JPRB, 0.40460E+00_JPRB, 0.40874E+00_JPRB ,&
&  0.39247E+00_JPRB, 0.37449E+00_JPRB, 0.36216E+00_JPRB, 0.35772E+00_JPRB, 0.35719E+00_JPRB, 0.36757E+00_JPRB, 0.37094E+00_JPRB ,&
&  0.36762E+00_JPRB, 0.36630E+00_JPRB, 0.37559E+00_JPRB, 0.37686E+00_JPRB, 0.37759E+00_JPRB, 0.37613E+00_JPRB, 0.37666E+00_JPRB ,&
&  0.38403E+00_JPRB, 0.39732E+00_JPRB, 0.42274E+00_JPRB, 0.45987E+00_JPRB, 0.49544E+00_JPRB, 0.54627E+00_JPRB, 0.59346E+00_JPRB ,&
&  0.67093E+00_JPRB, 0.66558E+00_JPRB, 0.40406E+00_JPRB, 0.63565E+00_JPRB, 0.82153E+00_JPRB, 0.83072E+00_JPRB, 0.77701E+00_JPRB ,&
&  0.71888E+00_JPRB, 0.67353E+00_JPRB, 0.73128E+00_JPRB, 0.72829E+00_JPRB, 0.62832E+00_JPRB, 0.56292E+00_JPRB, 0.51042E+00_JPRB ,&
&  0.47920E+00_JPRB, 0.46822E+00_JPRB, 0.47282E+00_JPRB, 0.49562E+00_JPRB, 0.53330E+00_JPRB, 0.56319E+00_JPRB, 0.57044E+00_JPRB ,&
&  0.57997E+00_JPRB, 0.60111E+00_JPRB, 0.60845E+00_JPRB, 0.51246E+00_JPRB, 0.47792E+00_JPRB, 0.47830E+00_JPRB, 0.47001E+00_JPRB ,&
&  0.46297E+00_JPRB, 0.49179E+00_JPRB, 0.51348E+00_JPRB, 0.49680E+00_JPRB, 0.48176E+00_JPRB, 0.47312E+00_JPRB, 0.48686E+00_JPRB ,&
&  0.48180E+00_JPRB, 0.47898E+00_JPRB, 0.47657E+00_JPRB, 0.46566E+00_JPRB, 0.44343E+00_JPRB, 0.42232E+00_JPRB, 0.42706E+00_JPRB ,&
&  0.42966E+00_JPRB, 0.43662E+00_JPRB, 0.43247E+00_JPRB, 0.43761E+00_JPRB, 0.44215E+00_JPRB, 0.44826E+00_JPRB, 0.45307E+00_JPRB ,&
&  0.46072E+00_JPRB, 0.46694E+00_JPRB, 0.47377E+00_JPRB, 0.46730E+00_JPRB, 0.46228E+00_JPRB, 0.46029E+00_JPRB, 0.46336E+00_JPRB ,&
&  0.47286E+00_JPRB, 0.48556E+00_JPRB, 0.49043E+00_JPRB, 0.50172E+00_JPRB, 0.53362E+00_JPRB, 0.55376E+00_JPRB, 0.56636E+00_JPRB ,&
&  0.62144E+00_JPRB, 0.66871E+00_JPRB, 0.61661E+00_JPRB, 0.46706E+00_JPRB, 0.43646E+00_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.40639E+00_JPRB, 0.40453E+00_JPRB, 0.40185E+00_JPRB, 0.40440E+00_JPRB, 0.40053E+00_JPRB, 0.40381E+00_JPRB, 0.39397E+00_JPRB ,&
&  0.37339E+00_JPRB, 0.36547E+00_JPRB, 0.37931E+00_JPRB, 0.38428E+00_JPRB, 0.38692E+00_JPRB, 0.39886E+00_JPRB, 0.41221E+00_JPRB ,&
&  0.41516E+00_JPRB, 0.41284E+00_JPRB, 0.40963E+00_JPRB, 0.40864E+00_JPRB, 0.40506E+00_JPRB, 0.40680E+00_JPRB, 0.41890E+00_JPRB ,&
&  0.43620E+00_JPRB, 0.46800E+00_JPRB, 0.51535E+00_JPRB, 0.57219E+00_JPRB, 0.62504E+00_JPRB, 0.68403E+00_JPRB, 0.71977E+00_JPRB ,&
&  0.79061E+00_JPRB, 0.58605E+00_JPRB, 0.43890E+00_JPRB, 0.72266E+00_JPRB, 0.81212E+00_JPRB, 0.80224E+00_JPRB, 0.70728E+00_JPRB ,&
&  0.67090E+00_JPRB, 0.65166E+00_JPRB, 0.65462E+00_JPRB, 0.63802E+00_JPRB, 0.58121E+00_JPRB, 0.55163E+00_JPRB, 0.52259E+00_JPRB ,&
&  0.50558E+00_JPRB, 0.50908E+00_JPRB, 0.52668E+00_JPRB, 0.56120E+00_JPRB, 0.59609E+00_JPRB, 0.60226E+00_JPRB, 0.60603E+00_JPRB ,&
&  0.62710E+00_JPRB, 0.64009E+00_JPRB, 0.62256E+00_JPRB, 0.50746E+00_JPRB, 0.48061E+00_JPRB, 0.47015E+00_JPRB, 0.46907E+00_JPRB ,&
&  0.46681E+00_JPRB, 0.49359E+00_JPRB, 0.52003E+00_JPRB, 0.50822E+00_JPRB, 0.49320E+00_JPRB, 0.47559E+00_JPRB, 0.48965E+00_JPRB ,&
&  0.48322E+00_JPRB, 0.47673E+00_JPRB, 0.46559E+00_JPRB, 0.44381E+00_JPRB, 0.41934E+00_JPRB, 0.42096E+00_JPRB, 0.41493E+00_JPRB ,&
&  0.41645E+00_JPRB, 0.42595E+00_JPRB, 0.42335E+00_JPRB, 0.43282E+00_JPRB, 0.44347E+00_JPRB, 0.45825E+00_JPRB, 0.47083E+00_JPRB ,&
&  0.47773E+00_JPRB, 0.48673E+00_JPRB, 0.49863E+00_JPRB, 0.49927E+00_JPRB, 0.49278E+00_JPRB, 0.48265E+00_JPRB, 0.49386E+00_JPRB ,&
&  0.50368E+00_JPRB, 0.52070E+00_JPRB, 0.52623E+00_JPRB, 0.54193E+00_JPRB, 0.53693E+00_JPRB, 0.54114E+00_JPRB, 0.51761E+00_JPRB ,&
&  0.51993E+00_JPRB, 0.49231E+00_JPRB, 0.45996E+00_JPRB, 0.41097E+00_JPRB, 0.40554E+00_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.41318E+00_JPRB, 0.40742E+00_JPRB, 0.40004E+00_JPRB, 0.39242E+00_JPRB, 0.38470E+00_JPRB, 0.38510E+00_JPRB, 0.37548E+00_JPRB ,&
&  0.37198E+00_JPRB, 0.38964E+00_JPRB, 0.41432E+00_JPRB, 0.43411E+00_JPRB, 0.45062E+00_JPRB, 0.45504E+00_JPRB, 0.46682E+00_JPRB ,&
&  0.46416E+00_JPRB, 0.46035E+00_JPRB, 0.45809E+00_JPRB, 0.45652E+00_JPRB, 0.46017E+00_JPRB, 0.47517E+00_JPRB, 0.50248E+00_JPRB ,&
&  0.54173E+00_JPRB, 0.59235E+00_JPRB, 0.65121E+00_JPRB, 0.70836E+00_JPRB, 0.75117E+00_JPRB, 0.79003E+00_JPRB, 0.79717E+00_JPRB ,&
&  0.59986E+00_JPRB, 0.40170E+00_JPRB, 0.74396E+00_JPRB, 0.82211E+00_JPRB, 0.76501E+00_JPRB, 0.73447E+00_JPRB, 0.68437E+00_JPRB ,&
&  0.63583E+00_JPRB, 0.62863E+00_JPRB, 0.61144E+00_JPRB, 0.60875E+00_JPRB, 0.56258E+00_JPRB, 0.55009E+00_JPRB, 0.54442E+00_JPRB ,&
&  0.54495E+00_JPRB, 0.55308E+00_JPRB, 0.57633E+00_JPRB, 0.61124E+00_JPRB, 0.62629E+00_JPRB, 0.62554E+00_JPRB, 0.64145E+00_JPRB ,&
&  0.66627E+00_JPRB, 0.66851E+00_JPRB, 0.64472E+00_JPRB, 0.51199E+00_JPRB, 0.47735E+00_JPRB, 0.47310E+00_JPRB, 0.46934E+00_JPRB ,&
&  0.48604E+00_JPRB, 0.49243E+00_JPRB, 0.50014E+00_JPRB, 0.51780E+00_JPRB, 0.50740E+00_JPRB, 0.47583E+00_JPRB, 0.48383E+00_JPRB ,&
&  0.47398E+00_JPRB, 0.45018E+00_JPRB, 0.42464E+00_JPRB, 0.41153E+00_JPRB, 0.40829E+00_JPRB, 0.41849E+00_JPRB, 0.41737E+00_JPRB ,&
&  0.41896E+00_JPRB, 0.42807E+00_JPRB, 0.42898E+00_JPRB, 0.43439E+00_JPRB, 0.45882E+00_JPRB, 0.48331E+00_JPRB, 0.49832E+00_JPRB ,&
&  0.50237E+00_JPRB, 0.51558E+00_JPRB, 0.52675E+00_JPRB, 0.53190E+00_JPRB, 0.53270E+00_JPRB, 0.51843E+00_JPRB, 0.51854E+00_JPRB ,&
&  0.52638E+00_JPRB, 0.53479E+00_JPRB, 0.53564E+00_JPRB, 0.54085E+00_JPRB, 0.52663E+00_JPRB, 0.52836E+00_JPRB, 0.47972E+00_JPRB ,&
&  0.45867E+00_JPRB, 0.42365E+00_JPRB, 0.39745E+00_JPRB, 0.38687E+00_JPRB, 0.39252E+00_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.41611E+00_JPRB, 0.39983E+00_JPRB, 0.38990E+00_JPRB, 0.38192E+00_JPRB, 0.37270E+00_JPRB, 0.39185E+00_JPRB, 0.39513E+00_JPRB ,&
&  0.41360E+00_JPRB, 0.44519E+00_JPRB, 0.48045E+00_JPRB, 0.49422E+00_JPRB, 0.49152E+00_JPRB, 0.50525E+00_JPRB, 0.51711E+00_JPRB ,&
&  0.52237E+00_JPRB, 0.52582E+00_JPRB, 0.53221E+00_JPRB, 0.54089E+00_JPRB, 0.55816E+00_JPRB, 0.58527E+00_JPRB, 0.62590E+00_JPRB ,&
&  0.67114E+00_JPRB, 0.71763E+00_JPRB, 0.76552E+00_JPRB, 0.80634E+00_JPRB, 0.82548E+00_JPRB, 0.84450E+00_JPRB, 0.82002E+00_JPRB ,&
&  0.55290E+00_JPRB, 0.72345E+00_JPRB, 0.81443E+00_JPRB, 0.76689E+00_JPRB, 0.72382E+00_JPRB, 0.69225E+00_JPRB, 0.67018E+00_JPRB ,&
&  0.64305E+00_JPRB, 0.60831E+00_JPRB, 0.59705E+00_JPRB, 0.61864E+00_JPRB, 0.57940E+00_JPRB, 0.56584E+00_JPRB, 0.56715E+00_JPRB ,&
&  0.57160E+00_JPRB, 0.58339E+00_JPRB, 0.61203E+00_JPRB, 0.64179E+00_JPRB, 0.65084E+00_JPRB, 0.65599E+00_JPRB, 0.67467E+00_JPRB ,&
&  0.69697E+00_JPRB, 0.70098E+00_JPRB, 0.68364E+00_JPRB, 0.54949E+00_JPRB, 0.50631E+00_JPRB, 0.49824E+00_JPRB, 0.48960E+00_JPRB ,&
&  0.48809E+00_JPRB, 0.50449E+00_JPRB, 0.51384E+00_JPRB, 0.53395E+00_JPRB, 0.50200E+00_JPRB, 0.47884E+00_JPRB, 0.45506E+00_JPRB ,&
&  0.42752E+00_JPRB, 0.39819E+00_JPRB, 0.38023E+00_JPRB, 0.40134E+00_JPRB, 0.42316E+00_JPRB, 0.43508E+00_JPRB, 0.42817E+00_JPRB ,&
&  0.44763E+00_JPRB, 0.43647E+00_JPRB, 0.43490E+00_JPRB, 0.44501E+00_JPRB, 0.49003E+00_JPRB, 0.50689E+00_JPRB, 0.51112E+00_JPRB ,&
&  0.51655E+00_JPRB, 0.52368E+00_JPRB, 0.52950E+00_JPRB, 0.53721E+00_JPRB, 0.54249E+00_JPRB, 0.54299E+00_JPRB, 0.53995E+00_JPRB ,&
&  0.53763E+00_JPRB, 0.54672E+00_JPRB, 0.54496E+00_JPRB, 0.54245E+00_JPRB, 0.53690E+00_JPRB, 0.56513E+00_JPRB, 0.50105E+00_JPRB ,&
&  0.45604E+00_JPRB, 0.42260E+00_JPRB, 0.39717E+00_JPRB, 0.39038E+00_JPRB, 0.39599E+00_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.42163E+00_JPRB, 0.40954E+00_JPRB, 0.39241E+00_JPRB, 0.39931E+00_JPRB, 0.40081E+00_JPRB, 0.41518E+00_JPRB, 0.43154E+00_JPRB ,&
&  0.45264E+00_JPRB, 0.49181E+00_JPRB, 0.52490E+00_JPRB, 0.54930E+00_JPRB, 0.55857E+00_JPRB, 0.57427E+00_JPRB, 0.58988E+00_JPRB ,&
&  0.59545E+00_JPRB, 0.60410E+00_JPRB, 0.61498E+00_JPRB, 0.63449E+00_JPRB, 0.66033E+00_JPRB, 0.68904E+00_JPRB, 0.72473E+00_JPRB ,&
&  0.76386E+00_JPRB, 0.80810E+00_JPRB, 0.86723E+00_JPRB, 0.87404E+00_JPRB, 0.88436E+00_JPRB, 0.90580E+00_JPRB, 0.87822E+00_JPRB ,&
&  0.66998E+00_JPRB, 0.77935E+00_JPRB, 0.75567E+00_JPRB, 0.72260E+00_JPRB, 0.69750E+00_JPRB, 0.66526E+00_JPRB, 0.63863E+00_JPRB ,&
&  0.61736E+00_JPRB, 0.57881E+00_JPRB, 0.56309E+00_JPRB, 0.55839E+00_JPRB, 0.59207E+00_JPRB, 0.57502E+00_JPRB, 0.57925E+00_JPRB ,&
&  0.58948E+00_JPRB, 0.61162E+00_JPRB, 0.63819E+00_JPRB, 0.66136E+00_JPRB, 0.67741E+00_JPRB, 0.68930E+00_JPRB, 0.71291E+00_JPRB ,&
&  0.74020E+00_JPRB, 0.74539E+00_JPRB, 0.73660E+00_JPRB, 0.64000E+00_JPRB, 0.56604E+00_JPRB, 0.54848E+00_JPRB, 0.52842E+00_JPRB ,&
&  0.51122E+00_JPRB, 0.50344E+00_JPRB, 0.53388E+00_JPRB, 0.55594E+00_JPRB, 0.51616E+00_JPRB, 0.47384E+00_JPRB, 0.43883E+00_JPRB ,&
&  0.41393E+00_JPRB, 0.39082E+00_JPRB, 0.40535E+00_JPRB, 0.45027E+00_JPRB, 0.47211E+00_JPRB, 0.48929E+00_JPRB, 0.48994E+00_JPRB ,&
&  0.48363E+00_JPRB, 0.45958E+00_JPRB, 0.44411E+00_JPRB, 0.45515E+00_JPRB, 0.48852E+00_JPRB, 0.52026E+00_JPRB, 0.51979E+00_JPRB ,&
&  0.50552E+00_JPRB, 0.50159E+00_JPRB, 0.51322E+00_JPRB, 0.53642E+00_JPRB, 0.55567E+00_JPRB, 0.58285E+00_JPRB, 0.60581E+00_JPRB ,&
&  0.63726E+00_JPRB, 0.68342E+00_JPRB, 0.67757E+00_JPRB, 0.68227E+00_JPRB, 0.78480E+00_JPRB, 0.84159E+00_JPRB, 0.53155E+00_JPRB ,&
&  0.44883E+00_JPRB, 0.41554E+00_JPRB, 0.41013E+00_JPRB, 0.40282E+00_JPRB, 0.42905E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.45534E+00_JPRB, 0.47181E+00_JPRB, 0.46160E+00_JPRB, 0.46348E+00_JPRB, 0.46967E+00_JPRB, 0.47904E+00_JPRB, 0.49912E+00_JPRB ,&
&  0.50284E+00_JPRB, 0.52638E+00_JPRB, 0.54857E+00_JPRB, 0.56551E+00_JPRB, 0.59148E+00_JPRB, 0.62834E+00_JPRB, 0.67190E+00_JPRB ,&
&  0.71311E+00_JPRB, 0.73402E+00_JPRB, 0.74568E+00_JPRB, 0.74988E+00_JPRB, 0.76576E+00_JPRB, 0.79332E+00_JPRB, 0.79888E+00_JPRB ,&
&  0.83930E+00_JPRB, 0.86504E+00_JPRB, 0.90100E+00_JPRB, 0.95584E+00_JPRB, 0.90545E+00_JPRB, 0.90097E+00_JPRB, 0.66810E+00_JPRB ,&
&  0.74911E+00_JPRB, 0.74933E+00_JPRB, 0.72921E+00_JPRB, 0.70866E+00_JPRB, 0.68096E+00_JPRB, 0.63876E+00_JPRB, 0.60892E+00_JPRB ,&
&  0.58693E+00_JPRB, 0.56747E+00_JPRB, 0.54943E+00_JPRB, 0.54305E+00_JPRB, 0.58863E+00_JPRB, 0.57549E+00_JPRB, 0.58606E+00_JPRB ,&
&  0.60966E+00_JPRB, 0.63527E+00_JPRB, 0.65006E+00_JPRB, 0.66515E+00_JPRB, 0.68676E+00_JPRB, 0.71636E+00_JPRB, 0.74708E+00_JPRB ,&
&  0.76998E+00_JPRB, 0.79210E+00_JPRB, 0.78408E+00_JPRB, 0.68971E+00_JPRB, 0.65748E+00_JPRB, 0.61110E+00_JPRB, 0.58181E+00_JPRB ,&
&  0.54098E+00_JPRB, 0.53225E+00_JPRB, 0.56981E+00_JPRB, 0.59645E+00_JPRB, 0.53890E+00_JPRB, 0.48368E+00_JPRB, 0.44546E+00_JPRB ,&
&  0.42555E+00_JPRB, 0.42275E+00_JPRB, 0.44578E+00_JPRB, 0.47006E+00_JPRB, 0.50335E+00_JPRB, 0.53361E+00_JPRB, 0.53905E+00_JPRB ,&
&  0.52810E+00_JPRB, 0.49836E+00_JPRB, 0.48482E+00_JPRB, 0.50049E+00_JPRB, 0.55387E+00_JPRB, 0.61095E+00_JPRB, 0.65761E+00_JPRB ,&
&  0.54864E+00_JPRB, 0.47233E+00_JPRB, 0.50547E+00_JPRB, 0.54549E+00_JPRB, 0.60054E+00_JPRB, 0.67619E+00_JPRB, 0.79086E+00_JPRB ,&
&  0.96066E+00_JPRB, 0.13115E+01_JPRB, 0.12974E+01_JPRB, 0.10862E+01_JPRB, 0.16097E+01_JPRB, 0.13299E+01_JPRB, 0.47332E+00_JPRB ,&
&  0.45331E+00_JPRB, 0.44516E+00_JPRB, 0.44669E+00_JPRB, 0.44720E+00_JPRB, 0.45492E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.53940E+00_JPRB, 0.54588E+00_JPRB, 0.56056E+00_JPRB, 0.55933E+00_JPRB, 0.56602E+00_JPRB, 0.58246E+00_JPRB, 0.59871E+00_JPRB ,&
&  0.61708E+00_JPRB, 0.62272E+00_JPRB, 0.63294E+00_JPRB, 0.64456E+00_JPRB, 0.65803E+00_JPRB, 0.69007E+00_JPRB, 0.74028E+00_JPRB ,&
&  0.76953E+00_JPRB, 0.79528E+00_JPRB, 0.83007E+00_JPRB, 0.82800E+00_JPRB, 0.83911E+00_JPRB, 0.86777E+00_JPRB, 0.87651E+00_JPRB ,&
&  0.90642E+00_JPRB, 0.92015E+00_JPRB, 0.93354E+00_JPRB, 0.97509E+00_JPRB, 0.93383E+00_JPRB, 0.93982E+00_JPRB, 0.72996E+00_JPRB ,&
&  0.76975E+00_JPRB, 0.74661E+00_JPRB, 0.69852E+00_JPRB, 0.67327E+00_JPRB, 0.65052E+00_JPRB, 0.61892E+00_JPRB, 0.59403E+00_JPRB ,&
&  0.58209E+00_JPRB, 0.56035E+00_JPRB, 0.53968E+00_JPRB, 0.55727E+00_JPRB, 0.56407E+00_JPRB, 0.57592E+00_JPRB, 0.59688E+00_JPRB ,&
&  0.61954E+00_JPRB, 0.63878E+00_JPRB, 0.64833E+00_JPRB, 0.66906E+00_JPRB, 0.70547E+00_JPRB, 0.73975E+00_JPRB, 0.77695E+00_JPRB ,&
&  0.79716E+00_JPRB, 0.82061E+00_JPRB, 0.83510E+00_JPRB, 0.76699E+00_JPRB, 0.83843E+00_JPRB, 0.78275E+00_JPRB, 0.66769E+00_JPRB ,&
&  0.56462E+00_JPRB, 0.53465E+00_JPRB, 0.56413E+00_JPRB, 0.62495E+00_JPRB, 0.57015E+00_JPRB, 0.52549E+00_JPRB, 0.48425E+00_JPRB ,&
&  0.46641E+00_JPRB, 0.46274E+00_JPRB, 0.46668E+00_JPRB, 0.48784E+00_JPRB, 0.52053E+00_JPRB, 0.55434E+00_JPRB, 0.57447E+00_JPRB ,&
&  0.57061E+00_JPRB, 0.53779E+00_JPRB, 0.50428E+00_JPRB, 0.48081E+00_JPRB, 0.48410E+00_JPRB, 0.50573E+00_JPRB, 0.48361E+00_JPRB ,&
&  0.50250E+00_JPRB, 0.49158E+00_JPRB, 0.50439E+00_JPRB, 0.56707E+00_JPRB, 0.67636E+00_JPRB, 0.88588E+00_JPRB, 0.12850E+01_JPRB ,&
&  0.18745E+01_JPRB, 0.20386E+01_JPRB, 0.25386E+01_JPRB, 0.34111E+01_JPRB, 0.25101E+01_JPRB, 0.17526E+01_JPRB, 0.52131E+00_JPRB ,&
&  0.53358E+00_JPRB, 0.52718E+00_JPRB, 0.52414E+00_JPRB, 0.52073E+00_JPRB, 0.52496E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.56633E+00_JPRB, 0.57838E+00_JPRB, 0.59294E+00_JPRB, 0.60543E+00_JPRB, 0.62256E+00_JPRB, 0.64784E+00_JPRB, 0.67263E+00_JPRB ,&
&  0.69275E+00_JPRB, 0.70586E+00_JPRB, 0.71325E+00_JPRB, 0.72753E+00_JPRB, 0.74598E+00_JPRB, 0.76936E+00_JPRB, 0.80070E+00_JPRB ,&
&  0.80419E+00_JPRB, 0.82858E+00_JPRB, 0.83166E+00_JPRB, 0.84998E+00_JPRB, 0.84654E+00_JPRB, 0.87113E+00_JPRB, 0.88516E+00_JPRB ,&
&  0.89233E+00_JPRB, 0.89493E+00_JPRB, 0.89374E+00_JPRB, 0.90416E+00_JPRB, 0.87842E+00_JPRB, 0.88221E+00_JPRB, 0.66374E+00_JPRB ,&
&  0.75589E+00_JPRB, 0.72770E+00_JPRB, 0.65375E+00_JPRB, 0.61874E+00_JPRB, 0.58952E+00_JPRB, 0.56697E+00_JPRB, 0.55112E+00_JPRB ,&
&  0.53375E+00_JPRB, 0.53462E+00_JPRB, 0.52888E+00_JPRB, 0.53126E+00_JPRB, 0.55597E+00_JPRB, 0.58029E+00_JPRB, 0.59576E+00_JPRB ,&
&  0.62217E+00_JPRB, 0.64023E+00_JPRB, 0.65232E+00_JPRB, 0.67142E+00_JPRB, 0.70375E+00_JPRB, 0.73645E+00_JPRB, 0.76831E+00_JPRB ,&
&  0.77551E+00_JPRB, 0.79520E+00_JPRB, 0.76472E+00_JPRB, 0.79455E+00_JPRB, 0.92216E+00_JPRB, 0.91317E+00_JPRB, 0.76086E+00_JPRB ,&
&  0.52534E+00_JPRB, 0.52692E+00_JPRB, 0.57858E+00_JPRB, 0.67218E+00_JPRB, 0.63131E+00_JPRB, 0.59081E+00_JPRB, 0.54704E+00_JPRB ,&
&  0.52592E+00_JPRB, 0.50576E+00_JPRB, 0.50706E+00_JPRB, 0.52857E+00_JPRB, 0.56295E+00_JPRB, 0.59503E+00_JPRB, 0.60878E+00_JPRB ,&
&  0.59654E+00_JPRB, 0.56351E+00_JPRB, 0.51705E+00_JPRB, 0.47251E+00_JPRB, 0.44179E+00_JPRB, 0.41385E+00_JPRB, 0.46930E+00_JPRB ,&
&  0.46802E+00_JPRB, 0.45149E+00_JPRB, 0.49196E+00_JPRB, 0.46607E+00_JPRB, 0.65031E+00_JPRB, 0.80443E+00_JPRB, 0.10510E+01_JPRB ,&
&  0.11870E+01_JPRB, 0.16279E+01_JPRB, 0.20530E+01_JPRB, 0.14630E+01_JPRB, 0.10766E+01_JPRB, 0.64648E+00_JPRB, 0.57210E+00_JPRB ,&
&  0.56756E+00_JPRB, 0.54938E+00_JPRB, 0.54535E+00_JPRB, 0.55067E+00_JPRB, 0.55698E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.59650E+00_JPRB, 0.61150E+00_JPRB, 0.62657E+00_JPRB, 0.64082E+00_JPRB, 0.65447E+00_JPRB, 0.66626E+00_JPRB, 0.67943E+00_JPRB ,&
&  0.69334E+00_JPRB, 0.70824E+00_JPRB, 0.72238E+00_JPRB, 0.73994E+00_JPRB, 0.75657E+00_JPRB, 0.76405E+00_JPRB, 0.77283E+00_JPRB ,&
&  0.77549E+00_JPRB, 0.79589E+00_JPRB, 0.79998E+00_JPRB, 0.81760E+00_JPRB, 0.83343E+00_JPRB, 0.83906E+00_JPRB, 0.86071E+00_JPRB ,&
&  0.87065E+00_JPRB, 0.87924E+00_JPRB, 0.88923E+00_JPRB, 0.90860E+00_JPRB, 0.91017E+00_JPRB, 0.94393E+00_JPRB, 0.73086E+00_JPRB ,&
&  0.72327E+00_JPRB, 0.67976E+00_JPRB, 0.61650E+00_JPRB, 0.56585E+00_JPRB, 0.52674E+00_JPRB, 0.48627E+00_JPRB, 0.46084E+00_JPRB ,&
&  0.45360E+00_JPRB, 0.45167E+00_JPRB, 0.45881E+00_JPRB, 0.47355E+00_JPRB, 0.49886E+00_JPRB, 0.53301E+00_JPRB, 0.56137E+00_JPRB ,&
&  0.58574E+00_JPRB, 0.61493E+00_JPRB, 0.63761E+00_JPRB, 0.65889E+00_JPRB, 0.68289E+00_JPRB, 0.70094E+00_JPRB, 0.71013E+00_JPRB ,&
&  0.70378E+00_JPRB, 0.70506E+00_JPRB, 0.65719E+00_JPRB, 0.72643E+00_JPRB, 0.83676E+00_JPRB, 0.87061E+00_JPRB, 0.76129E+00_JPRB ,&
&  0.57966E+00_JPRB, 0.53639E+00_JPRB, 0.56624E+00_JPRB, 0.69898E+00_JPRB, 0.71847E+00_JPRB, 0.69145E+00_JPRB, 0.65815E+00_JPRB ,&
&  0.63135E+00_JPRB, 0.60363E+00_JPRB, 0.58843E+00_JPRB, 0.60160E+00_JPRB, 0.63073E+00_JPRB, 0.64235E+00_JPRB, 0.61036E+00_JPRB ,&
&  0.58007E+00_JPRB, 0.55842E+00_JPRB, 0.52033E+00_JPRB, 0.46198E+00_JPRB, 0.42505E+00_JPRB, 0.37564E+00_JPRB, 0.42437E+00_JPRB ,&
&  0.41176E+00_JPRB, 0.36839E+00_JPRB, 0.40986E+00_JPRB, 0.50747E+00_JPRB, 0.57078E+00_JPRB, 0.65696E+00_JPRB, 0.82587E+00_JPRB ,&
&  0.96816E+00_JPRB, 0.10123E+01_JPRB, 0.94304E+00_JPRB, 0.76550E+00_JPRB, 0.62610E+00_JPRB, 0.57095E+00_JPRB, 0.56822E+00_JPRB ,&
&  0.56674E+00_JPRB, 0.56019E+00_JPRB, 0.56395E+00_JPRB, 0.57481E+00_JPRB, 0.58364E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.57681E+00_JPRB, 0.59138E+00_JPRB, 0.60619E+00_JPRB, 0.62122E+00_JPRB, 0.63408E+00_JPRB, 0.64580E+00_JPRB, 0.65841E+00_JPRB ,&
&  0.67114E+00_JPRB, 0.68626E+00_JPRB, 0.70246E+00_JPRB, 0.71954E+00_JPRB, 0.73191E+00_JPRB, 0.73472E+00_JPRB, 0.74119E+00_JPRB ,&
&  0.74398E+00_JPRB, 0.74072E+00_JPRB, 0.73031E+00_JPRB, 0.74564E+00_JPRB, 0.76175E+00_JPRB, 0.77289E+00_JPRB, 0.79202E+00_JPRB ,&
&  0.81499E+00_JPRB, 0.80707E+00_JPRB, 0.80506E+00_JPRB, 0.82352E+00_JPRB, 0.83252E+00_JPRB, 0.78610E+00_JPRB, 0.84325E+00_JPRB ,&
&  0.59832E+00_JPRB, 0.70857E+00_JPRB, 0.64981E+00_JPRB, 0.56373E+00_JPRB, 0.53943E+00_JPRB, 0.47854E+00_JPRB, 0.44184E+00_JPRB ,&
&  0.45472E+00_JPRB, 0.43241E+00_JPRB, 0.41820E+00_JPRB, 0.41048E+00_JPRB, 0.42347E+00_JPRB, 0.45774E+00_JPRB, 0.49004E+00_JPRB ,&
&  0.52807E+00_JPRB, 0.55890E+00_JPRB, 0.59971E+00_JPRB, 0.64472E+00_JPRB, 0.69279E+00_JPRB, 0.71490E+00_JPRB, 0.67804E+00_JPRB ,&
&  0.63549E+00_JPRB, 0.62259E+00_JPRB, 0.53068E+00_JPRB, 0.60503E+00_JPRB, 0.70201E+00_JPRB, 0.72943E+00_JPRB, 0.66809E+00_JPRB ,&
&  0.56907E+00_JPRB, 0.56401E+00_JPRB, 0.66615E+00_JPRB, 0.76087E+00_JPRB, 0.79961E+00_JPRB, 0.80558E+00_JPRB, 0.76692E+00_JPRB ,&
&  0.73294E+00_JPRB, 0.69019E+00_JPRB, 0.65817E+00_JPRB, 0.67416E+00_JPRB, 0.67621E+00_JPRB, 0.65223E+00_JPRB, 0.57938E+00_JPRB ,&
&  0.53657E+00_JPRB, 0.52466E+00_JPRB, 0.50777E+00_JPRB, 0.47306E+00_JPRB, 0.40930E+00_JPRB, 0.43253E+00_JPRB, 0.39568E+00_JPRB ,&
&  0.41425E+00_JPRB, 0.38482E+00_JPRB, 0.36175E+00_JPRB, 0.46148E+00_JPRB, 0.50297E+00_JPRB, 0.53604E+00_JPRB, 0.58701E+00_JPRB ,&
&  0.61757E+00_JPRB, 0.57548E+00_JPRB, 0.56629E+00_JPRB, 0.54299E+00_JPRB, 0.54156E+00_JPRB, 0.53638E+00_JPRB, 0.54798E+00_JPRB ,&
&  0.55432E+00_JPRB, 0.55237E+00_JPRB, 0.55691E+00_JPRB, 0.56399E+00_JPRB, 0.56801E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.54966E+00_JPRB, 0.55554E+00_JPRB, 0.55968E+00_JPRB, 0.57405E+00_JPRB, 0.59601E+00_JPRB, 0.59759E+00_JPRB, 0.60790E+00_JPRB ,&
&  0.61276E+00_JPRB, 0.62655E+00_JPRB, 0.64148E+00_JPRB, 0.65974E+00_JPRB, 0.66773E+00_JPRB, 0.66935E+00_JPRB, 0.68301E+00_JPRB ,&
&  0.69301E+00_JPRB, 0.68544E+00_JPRB, 0.69131E+00_JPRB, 0.70518E+00_JPRB, 0.70808E+00_JPRB, 0.73154E+00_JPRB, 0.73001E+00_JPRB ,&
&  0.75731E+00_JPRB, 0.76567E+00_JPRB, 0.78994E+00_JPRB, 0.85641E+00_JPRB, 0.93673E+00_JPRB, 0.11104E+01_JPRB, 0.14147E+01_JPRB ,&
&  0.10943E+01_JPRB, 0.78799E+00_JPRB, 0.74881E+00_JPRB, 0.63556E+00_JPRB, 0.63052E+00_JPRB, 0.64731E+00_JPRB, 0.64052E+00_JPRB ,&
&  0.61202E+00_JPRB, 0.56544E+00_JPRB, 0.52975E+00_JPRB, 0.49748E+00_JPRB, 0.47794E+00_JPRB, 0.48200E+00_JPRB, 0.47574E+00_JPRB ,&
&  0.45393E+00_JPRB, 0.49035E+00_JPRB, 0.54360E+00_JPRB, 0.62929E+00_JPRB, 0.69759E+00_JPRB, 0.82574E+00_JPRB, 0.81295E+00_JPRB ,&
&  0.77212E+00_JPRB, 0.74502E+00_JPRB, 0.68143E+00_JPRB, 0.71486E+00_JPRB, 0.76383E+00_JPRB, 0.74410E+00_JPRB, 0.66460E+00_JPRB ,&
&  0.63508E+00_JPRB, 0.58400E+00_JPRB, 0.61043E+00_JPRB, 0.82088E+00_JPRB, 0.89899E+00_JPRB, 0.93768E+00_JPRB, 0.90211E+00_JPRB ,&
&  0.85169E+00_JPRB, 0.81139E+00_JPRB, 0.75408E+00_JPRB, 0.71120E+00_JPRB, 0.69025E+00_JPRB, 0.65054E+00_JPRB, 0.57698E+00_JPRB ,&
&  0.51088E+00_JPRB, 0.49509E+00_JPRB, 0.49089E+00_JPRB, 0.45934E+00_JPRB, 0.45624E+00_JPRB, 0.38565E+00_JPRB, 0.38599E+00_JPRB ,&
&  0.35175E+00_JPRB, 0.36793E+00_JPRB, 0.33612E+00_JPRB, 0.40082E+00_JPRB, 0.41344E+00_JPRB, 0.40446E+00_JPRB, 0.41716E+00_JPRB ,&
&  0.42767E+00_JPRB, 0.43685E+00_JPRB, 0.45084E+00_JPRB, 0.46287E+00_JPRB, 0.47773E+00_JPRB, 0.50191E+00_JPRB, 0.52819E+00_JPRB ,&
&  0.54828E+00_JPRB, 0.53521E+00_JPRB, 0.53476E+00_JPRB, 0.53796E+00_JPRB, 0.54992E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.51179E+00_JPRB, 0.50766E+00_JPRB, 0.49991E+00_JPRB, 0.49991E+00_JPRB, 0.50393E+00_JPRB, 0.51583E+00_JPRB, 0.52305E+00_JPRB ,&
&  0.52056E+00_JPRB, 0.52425E+00_JPRB, 0.50763E+00_JPRB, 0.50726E+00_JPRB, 0.51958E+00_JPRB, 0.52585E+00_JPRB, 0.53167E+00_JPRB ,&
&  0.55894E+00_JPRB, 0.57051E+00_JPRB, 0.56916E+00_JPRB, 0.57192E+00_JPRB, 0.56760E+00_JPRB, 0.60740E+00_JPRB, 0.61036E+00_JPRB ,&
&  0.59711E+00_JPRB, 0.62066E+00_JPRB, 0.71712E+00_JPRB, 0.95742E+00_JPRB, 0.11592E+01_JPRB, 0.10889E+01_JPRB, 0.11990E+01_JPRB ,&
&  0.98817E+00_JPRB, 0.86704E+00_JPRB, 0.99263E+00_JPRB, 0.90128E+00_JPRB, 0.92990E+00_JPRB, 0.95822E+00_JPRB, 0.94237E+00_JPRB ,&
&  0.94766E+00_JPRB, 0.92315E+00_JPRB, 0.90986E+00_JPRB, 0.90039E+00_JPRB, 0.87814E+00_JPRB, 0.85327E+00_JPRB, 0.81361E+00_JPRB ,&
&  0.75116E+00_JPRB, 0.72843E+00_JPRB, 0.75573E+00_JPRB, 0.86185E+00_JPRB, 0.98734E+00_JPRB, 0.10825E+01_JPRB, 0.10915E+01_JPRB ,&
&  0.11009E+01_JPRB, 0.11104E+01_JPRB, 0.10470E+01_JPRB, 0.10640E+01_JPRB, 0.10999E+01_JPRB, 0.10113E+01_JPRB, 0.92475E+00_JPRB ,&
&  0.85062E+00_JPRB, 0.66525E+00_JPRB, 0.62384E+00_JPRB, 0.86713E+00_JPRB, 0.95037E+00_JPRB, 0.97576E+00_JPRB, 0.10067E+01_JPRB ,&
&  0.96698E+00_JPRB, 0.95542E+00_JPRB, 0.88011E+00_JPRB, 0.78874E+00_JPRB, 0.70619E+00_JPRB, 0.63723E+00_JPRB, 0.59820E+00_JPRB ,&
&  0.52212E+00_JPRB, 0.47864E+00_JPRB, 0.46869E+00_JPRB, 0.44642E+00_JPRB, 0.41987E+00_JPRB, 0.41479E+00_JPRB, 0.37405E+00_JPRB ,&
&  0.33998E+00_JPRB, 0.33008E+00_JPRB, 0.35637E+00_JPRB, 0.36120E+00_JPRB, 0.31901E+00_JPRB, 0.35208E+00_JPRB, 0.34710E+00_JPRB ,&
&  0.36108E+00_JPRB, 0.38082E+00_JPRB, 0.38825E+00_JPRB, 0.39841E+00_JPRB, 0.41732E+00_JPRB, 0.43414E+00_JPRB, 0.45812E+00_JPRB ,&
&  0.47394E+00_JPRB, 0.48669E+00_JPRB, 0.50031E+00_JPRB, 0.51389E+00_JPRB, 0.51000E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.55810E+00_JPRB, 0.58378E+00_JPRB, 0.59817E+00_JPRB, 0.61616E+00_JPRB, 0.62286E+00_JPRB, 0.61630E+00_JPRB, 0.58990E+00_JPRB ,&
&  0.56863E+00_JPRB, 0.54568E+00_JPRB, 0.53576E+00_JPRB, 0.51994E+00_JPRB, 0.53371E+00_JPRB, 0.56229E+00_JPRB, 0.58799E+00_JPRB ,&
&  0.65174E+00_JPRB, 0.61852E+00_JPRB, 0.62504E+00_JPRB, 0.65628E+00_JPRB, 0.66845E+00_JPRB, 0.71138E+00_JPRB, 0.82924E+00_JPRB ,&
&  0.10205E+01_JPRB, 0.12349E+01_JPRB, 0.15134E+01_JPRB, 0.19077E+01_JPRB, 0.19321E+01_JPRB, 0.13735E+01_JPRB, 0.12170E+01_JPRB ,&
&  0.10754E+01_JPRB, 0.10255E+01_JPRB, 0.11083E+01_JPRB, 0.11676E+01_JPRB, 0.12240E+01_JPRB, 0.12132E+01_JPRB, 0.12239E+01_JPRB ,&
&  0.12394E+01_JPRB, 0.12606E+01_JPRB, 0.12690E+01_JPRB, 0.12646E+01_JPRB, 0.12698E+01_JPRB, 0.12875E+01_JPRB, 0.13181E+01_JPRB ,&
&  0.13377E+01_JPRB, 0.13299E+01_JPRB, 0.13228E+01_JPRB, 0.12938E+01_JPRB, 0.13468E+01_JPRB, 0.13772E+01_JPRB, 0.13999E+01_JPRB ,&
&  0.14013E+01_JPRB, 0.14029E+01_JPRB, 0.14705E+01_JPRB, 0.14558E+01_JPRB, 0.14087E+01_JPRB, 0.13558E+01_JPRB, 0.12482E+01_JPRB ,&
&  0.11263E+01_JPRB, 0.90892E+00_JPRB, 0.80138E+00_JPRB, 0.11409E+01_JPRB, 0.11406E+01_JPRB, 0.10085E+01_JPRB, 0.11011E+01_JPRB ,&
&  0.10607E+01_JPRB, 0.10624E+01_JPRB, 0.10334E+01_JPRB, 0.90835E+00_JPRB, 0.79527E+00_JPRB, 0.69823E+00_JPRB, 0.58640E+00_JPRB ,&
&  0.57839E+00_JPRB, 0.51133E+00_JPRB, 0.46404E+00_JPRB, 0.44520E+00_JPRB, 0.42334E+00_JPRB, 0.41396E+00_JPRB, 0.40978E+00_JPRB ,&
&  0.39691E+00_JPRB, 0.37549E+00_JPRB, 0.39488E+00_JPRB, 0.44392E+00_JPRB, 0.46525E+00_JPRB, 0.36050E+00_JPRB, 0.34541E+00_JPRB ,&
&  0.36042E+00_JPRB, 0.37433E+00_JPRB, 0.38770E+00_JPRB, 0.40181E+00_JPRB, 0.41330E+00_JPRB, 0.43352E+00_JPRB, 0.45376E+00_JPRB ,&
&  0.47710E+00_JPRB, 0.50969E+00_JPRB, 0.52476E+00_JPRB, 0.54019E+00_JPRB, 0.55510E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.77890E+00_JPRB, 0.81452E+00_JPRB, 0.84757E+00_JPRB, 0.86588E+00_JPRB, 0.86733E+00_JPRB, 0.87544E+00_JPRB, 0.87689E+00_JPRB ,&
&  0.88682E+00_JPRB, 0.88871E+00_JPRB, 0.91782E+00_JPRB, 0.97932E+00_JPRB, 0.11006E+01_JPRB, 0.11654E+01_JPRB, 0.12495E+01_JPRB ,&
&  0.12572E+01_JPRB, 0.12853E+01_JPRB, 0.13176E+01_JPRB, 0.13597E+01_JPRB, 0.14387E+01_JPRB, 0.15551E+01_JPRB, 0.17863E+01_JPRB ,&
&  0.22093E+01_JPRB, 0.24604E+01_JPRB, 0.23634E+01_JPRB, 0.19385E+01_JPRB, 0.14649E+01_JPRB, 0.11526E+01_JPRB, 0.12278E+01_JPRB ,&
&  0.12731E+01_JPRB, 0.12975E+01_JPRB, 0.13327E+01_JPRB, 0.13448E+01_JPRB, 0.13383E+01_JPRB, 0.13406E+01_JPRB, 0.13356E+01_JPRB ,&
&  0.13403E+01_JPRB, 0.13500E+01_JPRB, 0.13475E+01_JPRB, 0.13410E+01_JPRB, 0.13626E+01_JPRB, 0.14128E+01_JPRB, 0.14875E+01_JPRB ,&
&  0.16023E+01_JPRB, 0.16860E+01_JPRB, 0.17029E+01_JPRB, 0.16915E+01_JPRB, 0.16631E+01_JPRB, 0.16349E+01_JPRB, 0.16319E+01_JPRB ,&
&  0.16409E+01_JPRB, 0.16142E+01_JPRB, 0.16830E+01_JPRB, 0.17221E+01_JPRB, 0.16397E+01_JPRB, 0.15387E+01_JPRB, 0.14535E+01_JPRB ,&
&  0.13908E+01_JPRB, 0.13240E+01_JPRB, 0.11214E+01_JPRB, 0.14117E+01_JPRB, 0.14501E+01_JPRB, 0.13112E+01_JPRB, 0.11919E+01_JPRB ,&
&  0.11614E+01_JPRB, 0.11537E+01_JPRB, 0.11031E+01_JPRB, 0.96696E+00_JPRB, 0.84985E+00_JPRB, 0.79455E+00_JPRB, 0.61855E+00_JPRB ,&
&  0.64159E+00_JPRB, 0.57175E+00_JPRB, 0.50106E+00_JPRB, 0.46816E+00_JPRB, 0.45497E+00_JPRB, 0.43850E+00_JPRB, 0.43549E+00_JPRB ,&
&  0.40240E+00_JPRB, 0.46098E+00_JPRB, 0.56939E+00_JPRB, 0.92504E+00_JPRB, 0.16973E+01_JPRB, 0.42981E+00_JPRB, 0.36010E+00_JPRB ,&
&  0.38091E+00_JPRB, 0.40952E+00_JPRB, 0.44155E+00_JPRB, 0.46675E+00_JPRB, 0.48711E+00_JPRB, 0.51959E+00_JPRB, 0.57103E+00_JPRB ,&
&  0.60745E+00_JPRB, 0.63631E+00_JPRB, 0.67013E+00_JPRB, 0.72374E+00_JPRB, 0.74956E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.91348E+00_JPRB, 0.94666E+00_JPRB, 0.96724E+00_JPRB, 0.10073E+01_JPRB, 0.10477E+01_JPRB, 0.10920E+01_JPRB, 0.11551E+01_JPRB ,&
&  0.12398E+01_JPRB, 0.13329E+01_JPRB, 0.14218E+01_JPRB, 0.14855E+01_JPRB, 0.15443E+01_JPRB, 0.15892E+01_JPRB, 0.16285E+01_JPRB ,&
&  0.16574E+01_JPRB, 0.17258E+01_JPRB, 0.18042E+01_JPRB, 0.19085E+01_JPRB, 0.20284E+01_JPRB, 0.21686E+01_JPRB, 0.23160E+01_JPRB ,&
&  0.24060E+01_JPRB, 0.22869E+01_JPRB, 0.19639E+01_JPRB, 0.13534E+01_JPRB, 0.10759E+01_JPRB, 0.11669E+01_JPRB, 0.12185E+01_JPRB ,&
&  0.12851E+01_JPRB, 0.13336E+01_JPRB, 0.13340E+01_JPRB, 0.13678E+01_JPRB, 0.13905E+01_JPRB, 0.14041E+01_JPRB, 0.14041E+01_JPRB ,&
&  0.14129E+01_JPRB, 0.14272E+01_JPRB, 0.14293E+01_JPRB, 0.14274E+01_JPRB, 0.14536E+01_JPRB, 0.15359E+01_JPRB, 0.16704E+01_JPRB ,&
&  0.18360E+01_JPRB, 0.19492E+01_JPRB, 0.20224E+01_JPRB, 0.20600E+01_JPRB, 0.19665E+01_JPRB, 0.18720E+01_JPRB, 0.18109E+01_JPRB ,&
&  0.18054E+01_JPRB, 0.18165E+01_JPRB, 0.18696E+01_JPRB, 0.19088E+01_JPRB, 0.18842E+01_JPRB, 0.17031E+01_JPRB, 0.15983E+01_JPRB ,&
&  0.15836E+01_JPRB, 0.15040E+01_JPRB, 0.14137E+01_JPRB, 0.16123E+01_JPRB, 0.10627E+01_JPRB, 0.10990E+01_JPRB, 0.11732E+01_JPRB ,&
&  0.12930E+01_JPRB, 0.13679E+01_JPRB, 0.12278E+01_JPRB, 0.10159E+01_JPRB, 0.90123E+00_JPRB, 0.66299E+00_JPRB, 0.62907E+00_JPRB ,&
&  0.69785E+00_JPRB, 0.66755E+00_JPRB, 0.56802E+00_JPRB, 0.50763E+00_JPRB, 0.52192E+00_JPRB, 0.47475E+00_JPRB, 0.47348E+00_JPRB ,&
&  0.44559E+00_JPRB, 0.47535E+00_JPRB, 0.55148E+00_JPRB, 0.70211E+00_JPRB, 0.88177E+00_JPRB, 0.44054E+00_JPRB, 0.38508E+00_JPRB ,&
&  0.40399E+00_JPRB, 0.44493E+00_JPRB, 0.49391E+00_JPRB, 0.53777E+00_JPRB, 0.57947E+00_JPRB, 0.62479E+00_JPRB, 0.67406E+00_JPRB ,&
&  0.72964E+00_JPRB, 0.79663E+00_JPRB, 0.84003E+00_JPRB, 0.86812E+00_JPRB, 0.88689E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.89708E+00_JPRB, 0.93371E+00_JPRB, 0.98162E+00_JPRB, 0.10494E+01_JPRB, 0.11201E+01_JPRB, 0.11405E+01_JPRB, 0.11314E+01_JPRB ,&
&  0.11657E+01_JPRB, 0.12272E+01_JPRB, 0.13046E+01_JPRB, 0.13618E+01_JPRB, 0.13869E+01_JPRB, 0.14381E+01_JPRB, 0.14992E+01_JPRB ,&
&  0.15582E+01_JPRB, 0.16237E+01_JPRB, 0.17038E+01_JPRB, 0.18015E+01_JPRB, 0.19319E+01_JPRB, 0.20945E+01_JPRB, 0.21743E+01_JPRB ,&
&  0.17063E+01_JPRB, 0.12283E+01_JPRB, 0.16336E+01_JPRB, 0.15779E+01_JPRB, 0.13267E+01_JPRB, 0.11600E+01_JPRB, 0.11651E+01_JPRB ,&
&  0.12815E+01_JPRB, 0.13548E+01_JPRB, 0.13646E+01_JPRB, 0.13662E+01_JPRB, 0.13516E+01_JPRB, 0.13848E+01_JPRB, 0.14168E+01_JPRB ,&
&  0.14428E+01_JPRB, 0.14609E+01_JPRB, 0.14723E+01_JPRB, 0.14808E+01_JPRB, 0.15234E+01_JPRB, 0.16221E+01_JPRB, 0.17862E+01_JPRB ,&
&  0.19591E+01_JPRB, 0.20695E+01_JPRB, 0.21703E+01_JPRB, 0.21893E+01_JPRB, 0.20881E+01_JPRB, 0.20012E+01_JPRB, 0.19706E+01_JPRB ,&
&  0.19925E+01_JPRB, 0.20179E+01_JPRB, 0.20433E+01_JPRB, 0.20450E+01_JPRB, 0.20627E+01_JPRB, 0.18386E+01_JPRB, 0.17609E+01_JPRB ,&
&  0.17422E+01_JPRB, 0.16225E+01_JPRB, 0.15620E+01_JPRB, 0.15963E+01_JPRB, 0.11092E+01_JPRB, 0.10996E+01_JPRB, 0.10658E+01_JPRB ,&
&  0.13079E+01_JPRB, 0.14504E+01_JPRB, 0.14041E+01_JPRB, 0.12220E+01_JPRB, 0.10110E+01_JPRB, 0.69065E+00_JPRB, 0.65415E+00_JPRB ,&
&  0.70400E+00_JPRB, 0.75577E+00_JPRB, 0.63948E+00_JPRB, 0.59000E+00_JPRB, 0.47949E+00_JPRB, 0.42919E+00_JPRB, 0.44318E+00_JPRB ,&
&  0.47436E+00_JPRB, 0.47146E+00_JPRB, 0.51294E+00_JPRB, 0.57104E+00_JPRB, 0.60495E+00_JPRB, 0.45241E+00_JPRB, 0.40191E+00_JPRB ,&
&  0.41991E+00_JPRB, 0.46026E+00_JPRB, 0.51654E+00_JPRB, 0.58002E+00_JPRB, 0.61384E+00_JPRB, 0.64856E+00_JPRB, 0.69218E+00_JPRB ,&
&  0.75460E+00_JPRB, 0.80549E+00_JPRB, 0.84005E+00_JPRB, 0.84812E+00_JPRB, 0.87400E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.81572E+00_JPRB, 0.86325E+00_JPRB, 0.92988E+00_JPRB, 0.10321E+01_JPRB, 0.11776E+01_JPRB, 0.13163E+01_JPRB, 0.12131E+01_JPRB ,&
&  0.10928E+01_JPRB, 0.95672E+00_JPRB, 0.10004E+01_JPRB, 0.10434E+01_JPRB, 0.10844E+01_JPRB, 0.11544E+01_JPRB, 0.12488E+01_JPRB ,&
&  0.13487E+01_JPRB, 0.14423E+01_JPRB, 0.15304E+01_JPRB, 0.16011E+01_JPRB, 0.16324E+01_JPRB, 0.17566E+01_JPRB, 0.16193E+01_JPRB ,&
&  0.10777E+01_JPRB, 0.10845E+01_JPRB, 0.16038E+01_JPRB, 0.13609E+01_JPRB, 0.13414E+01_JPRB, 0.12982E+01_JPRB, 0.12014E+01_JPRB ,&
&  0.12457E+01_JPRB, 0.13112E+01_JPRB, 0.12923E+01_JPRB, 0.12794E+01_JPRB, 0.12533E+01_JPRB, 0.13199E+01_JPRB, 0.13865E+01_JPRB ,&
&  0.14232E+01_JPRB, 0.14450E+01_JPRB, 0.14608E+01_JPRB, 0.14712E+01_JPRB, 0.15079E+01_JPRB, 0.16046E+01_JPRB, 0.17991E+01_JPRB ,&
&  0.19618E+01_JPRB, 0.21121E+01_JPRB, 0.22617E+01_JPRB, 0.22706E+01_JPRB, 0.21924E+01_JPRB, 0.21732E+01_JPRB, 0.22215E+01_JPRB ,&
&  0.22749E+01_JPRB, 0.23673E+01_JPRB, 0.23791E+01_JPRB, 0.22955E+01_JPRB, 0.20768E+01_JPRB, 0.21626E+01_JPRB, 0.18678E+01_JPRB ,&
&  0.18470E+01_JPRB, 0.17065E+01_JPRB, 0.17927E+01_JPRB, 0.13435E+01_JPRB, 0.12251E+01_JPRB, 0.12245E+01_JPRB, 0.12203E+01_JPRB ,&
&  0.12392E+01_JPRB, 0.14006E+01_JPRB, 0.14372E+01_JPRB, 0.12831E+01_JPRB, 0.10938E+01_JPRB, 0.80412E+00_JPRB, 0.79800E+00_JPRB ,&
&  0.77197E+00_JPRB, 0.86052E+00_JPRB, 0.70353E+00_JPRB, 0.55676E+00_JPRB, 0.40715E+00_JPRB, 0.34864E+00_JPRB, 0.37764E+00_JPRB ,&
&  0.45274E+00_JPRB, 0.49810E+00_JPRB, 0.49318E+00_JPRB, 0.48999E+00_JPRB, 0.50866E+00_JPRB, 0.46867E+00_JPRB, 0.42652E+00_JPRB ,&
&  0.44608E+00_JPRB, 0.47086E+00_JPRB, 0.50686E+00_JPRB, 0.56568E+00_JPRB, 0.60462E+00_JPRB, 0.64917E+00_JPRB, 0.65861E+00_JPRB ,&
&  0.69045E+00_JPRB, 0.72025E+00_JPRB, 0.73441E+00_JPRB, 0.74888E+00_JPRB, 0.77480E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.66777E+00_JPRB, 0.67937E+00_JPRB, 0.70865E+00_JPRB, 0.75735E+00_JPRB, 0.80110E+00_JPRB, 0.84071E+00_JPRB, 0.84642E+00_JPRB ,&
&  0.82066E+00_JPRB, 0.79487E+00_JPRB, 0.79358E+00_JPRB, 0.77374E+00_JPRB, 0.76846E+00_JPRB, 0.78087E+00_JPRB, 0.83886E+00_JPRB ,&
&  0.96750E+00_JPRB, 0.11470E+01_JPRB, 0.13234E+01_JPRB, 0.14370E+01_JPRB, 0.14725E+01_JPRB, 0.15431E+01_JPRB, 0.12986E+01_JPRB ,&
&  0.89171E+00_JPRB, 0.15945E+01_JPRB, 0.13348E+01_JPRB, 0.13480E+01_JPRB, 0.13786E+01_JPRB, 0.14248E+01_JPRB, 0.12317E+01_JPRB ,&
&  0.12083E+01_JPRB, 0.12398E+01_JPRB, 0.12327E+01_JPRB, 0.12369E+01_JPRB, 0.12244E+01_JPRB, 0.12733E+01_JPRB, 0.13395E+01_JPRB ,&
&  0.14011E+01_JPRB, 0.14400E+01_JPRB, 0.14575E+01_JPRB, 0.14542E+01_JPRB, 0.14672E+01_JPRB, 0.15098E+01_JPRB, 0.16789E+01_JPRB ,&
&  0.18543E+01_JPRB, 0.20657E+01_JPRB, 0.22614E+01_JPRB, 0.24411E+01_JPRB, 0.23647E+01_JPRB, 0.23950E+01_JPRB, 0.25445E+01_JPRB ,&
&  0.25513E+01_JPRB, 0.24506E+01_JPRB, 0.26056E+01_JPRB, 0.25883E+01_JPRB, 0.24813E+01_JPRB, 0.25227E+01_JPRB, 0.20638E+01_JPRB ,&
&  0.20454E+01_JPRB, 0.17812E+01_JPRB, 0.18420E+01_JPRB, 0.14465E+01_JPRB, 0.13404E+01_JPRB, 0.13436E+01_JPRB, 0.13090E+01_JPRB ,&
&  0.12129E+01_JPRB, 0.13344E+01_JPRB, 0.13195E+01_JPRB, 0.12463E+01_JPRB, 0.12031E+01_JPRB, 0.10418E+01_JPRB, 0.99054E+00_JPRB ,&
&  0.94918E+00_JPRB, 0.89927E+00_JPRB, 0.78869E+00_JPRB, 0.47710E+00_JPRB, 0.37178E+00_JPRB, 0.30935E+00_JPRB, 0.32849E+00_JPRB ,&
&  0.40417E+00_JPRB, 0.46479E+00_JPRB, 0.52131E+00_JPRB, 0.49059E+00_JPRB, 0.47807E+00_JPRB, 0.47186E+00_JPRB, 0.46122E+00_JPRB ,&
&  0.46604E+00_JPRB, 0.47694E+00_JPRB, 0.48912E+00_JPRB, 0.53438E+00_JPRB, 0.59143E+00_JPRB, 0.62907E+00_JPRB, 0.61237E+00_JPRB ,&
&  0.59321E+00_JPRB, 0.59443E+00_JPRB, 0.61734E+00_JPRB, 0.63477E+00_JPRB, 0.65272E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.58177E+00_JPRB, 0.57211E+00_JPRB, 0.57099E+00_JPRB, 0.58741E+00_JPRB, 0.59088E+00_JPRB, 0.59083E+00_JPRB, 0.62554E+00_JPRB ,&
&  0.63298E+00_JPRB, 0.62032E+00_JPRB, 0.62968E+00_JPRB, 0.63192E+00_JPRB, 0.62605E+00_JPRB, 0.60687E+00_JPRB, 0.60312E+00_JPRB ,&
&  0.62958E+00_JPRB, 0.73683E+00_JPRB, 0.94862E+00_JPRB, 0.12007E+01_JPRB, 0.13659E+01_JPRB, 0.16267E+01_JPRB, 0.10869E+01_JPRB ,&
&  0.97396E+00_JPRB, 0.13617E+01_JPRB, 0.12848E+01_JPRB, 0.13940E+01_JPRB, 0.14892E+01_JPRB, 0.15903E+01_JPRB, 0.13706E+01_JPRB ,&
&  0.13015E+01_JPRB, 0.12983E+01_JPRB, 0.12576E+01_JPRB, 0.12684E+01_JPRB, 0.12843E+01_JPRB, 0.12927E+01_JPRB, 0.12954E+01_JPRB ,&
&  0.13289E+01_JPRB, 0.13849E+01_JPRB, 0.14254E+01_JPRB, 0.14304E+01_JPRB, 0.14307E+01_JPRB, 0.14510E+01_JPRB, 0.15866E+01_JPRB ,&
&  0.17611E+01_JPRB, 0.19437E+01_JPRB, 0.22029E+01_JPRB, 0.23989E+01_JPRB, 0.25286E+01_JPRB, 0.25888E+01_JPRB, 0.28508E+01_JPRB ,&
&  0.28257E+01_JPRB, 0.26978E+01_JPRB, 0.28605E+01_JPRB, 0.29492E+01_JPRB, 0.29235E+01_JPRB, 0.28815E+01_JPRB, 0.24494E+01_JPRB ,&
&  0.22279E+01_JPRB, 0.20441E+01_JPRB, 0.19156E+01_JPRB, 0.16014E+01_JPRB, 0.14621E+01_JPRB, 0.14288E+01_JPRB, 0.14357E+01_JPRB ,&
&  0.13718E+01_JPRB, 0.13925E+01_JPRB, 0.14156E+01_JPRB, 0.13173E+01_JPRB, 0.12588E+01_JPRB, 0.11463E+01_JPRB, 0.12046E+01_JPRB ,&
&  0.11552E+01_JPRB, 0.98470E+00_JPRB, 0.65242E+00_JPRB, 0.37368E+00_JPRB, 0.30392E+00_JPRB, 0.25471E+00_JPRB, 0.28197E+00_JPRB ,&
&  0.35615E+00_JPRB, 0.41359E+00_JPRB, 0.45920E+00_JPRB, 0.55387E+00_JPRB, 0.54954E+00_JPRB, 0.53225E+00_JPRB, 0.51216E+00_JPRB ,&
&  0.50882E+00_JPRB, 0.49727E+00_JPRB, 0.49168E+00_JPRB, 0.50077E+00_JPRB, 0.54948E+00_JPRB, 0.59941E+00_JPRB, 0.61917E+00_JPRB ,&
&  0.59441E+00_JPRB, 0.56742E+00_JPRB, 0.55485E+00_JPRB, 0.55490E+00_JPRB, 0.56415E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.52975E+00_JPRB, 0.52502E+00_JPRB, 0.52501E+00_JPRB, 0.53142E+00_JPRB, 0.53551E+00_JPRB, 0.51428E+00_JPRB, 0.48197E+00_JPRB ,&
&  0.48706E+00_JPRB, 0.47097E+00_JPRB, 0.48689E+00_JPRB, 0.50206E+00_JPRB, 0.52275E+00_JPRB, 0.52540E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52207E+00_JPRB, 0.55404E+00_JPRB, 0.71092E+00_JPRB, 0.96509E+00_JPRB, 0.11798E+01_JPRB, 0.13112E+01_JPRB, 0.12861E+01_JPRB ,&
&  0.12353E+01_JPRB, 0.13740E+01_JPRB, 0.14134E+01_JPRB, 0.17356E+01_JPRB, 0.20024E+01_JPRB, 0.23994E+01_JPRB, 0.15786E+01_JPRB ,&
&  0.14200E+01_JPRB, 0.13843E+01_JPRB, 0.13584E+01_JPRB, 0.13602E+01_JPRB, 0.14007E+01_JPRB, 0.13770E+01_JPRB, 0.13531E+01_JPRB ,&
&  0.13158E+01_JPRB, 0.13179E+01_JPRB, 0.13411E+01_JPRB, 0.13582E+01_JPRB, 0.13671E+01_JPRB, 0.13921E+01_JPRB, 0.15172E+01_JPRB ,&
&  0.17258E+01_JPRB, 0.19561E+01_JPRB, 0.21978E+01_JPRB, 0.24827E+01_JPRB, 0.26380E+01_JPRB, 0.28007E+01_JPRB, 0.31671E+01_JPRB ,&
&  0.33231E+01_JPRB, 0.33977E+01_JPRB, 0.32953E+01_JPRB, 0.31916E+01_JPRB, 0.31118E+01_JPRB, 0.32744E+01_JPRB, 0.29024E+01_JPRB ,&
&  0.25020E+01_JPRB, 0.21683E+01_JPRB, 0.18736E+01_JPRB, 0.17312E+01_JPRB, 0.16291E+01_JPRB, 0.15836E+01_JPRB, 0.13769E+01_JPRB ,&
&  0.11867E+01_JPRB, 0.10948E+01_JPRB, 0.11084E+01_JPRB, 0.12024E+01_JPRB, 0.13098E+01_JPRB, 0.13021E+01_JPRB, 0.14612E+01_JPRB ,&
&  0.11127E+01_JPRB, 0.58337E+00_JPRB, 0.36984E+00_JPRB, 0.35521E+00_JPRB, 0.25629E+00_JPRB, 0.20878E+00_JPRB, 0.27608E+00_JPRB ,&
&  0.35086E+00_JPRB, 0.42535E+00_JPRB, 0.44544E+00_JPRB, 0.49059E+00_JPRB, 0.60474E+00_JPRB, 0.62128E+00_JPRB, 0.59934E+00_JPRB ,&
&  0.56174E+00_JPRB, 0.53848E+00_JPRB, 0.53081E+00_JPRB, 0.52684E+00_JPRB, 0.56161E+00_JPRB, 0.60606E+00_JPRB, 0.63049E+00_JPRB ,&
&  0.61907E+00_JPRB, 0.58854E+00_JPRB, 0.54644E+00_JPRB, 0.51530E+00_JPRB, 0.51317E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.52461E+00_JPRB, 0.53970E+00_JPRB, 0.55321E+00_JPRB, 0.53942E+00_JPRB, 0.51572E+00_JPRB, 0.50714E+00_JPRB, 0.46945E+00_JPRB ,&
&  0.44479E+00_JPRB, 0.42799E+00_JPRB, 0.41850E+00_JPRB, 0.42727E+00_JPRB, 0.44972E+00_JPRB, 0.46687E+00_JPRB, 0.46525E+00_JPRB ,&
&  0.45750E+00_JPRB, 0.48931E+00_JPRB, 0.59290E+00_JPRB, 0.83032E+00_JPRB, 0.11093E+01_JPRB, 0.14001E+01_JPRB, 0.14369E+01_JPRB ,&
&  0.13094E+01_JPRB, 0.18212E+01_JPRB, 0.25969E+01_JPRB, 0.29858E+01_JPRB, 0.28239E+01_JPRB, 0.30075E+01_JPRB, 0.21348E+01_JPRB ,&
&  0.17679E+01_JPRB, 0.17215E+01_JPRB, 0.16839E+01_JPRB, 0.16565E+01_JPRB, 0.15932E+01_JPRB, 0.15005E+01_JPRB, 0.14475E+01_JPRB ,&
&  0.14013E+01_JPRB, 0.13143E+01_JPRB, 0.12815E+01_JPRB, 0.12769E+01_JPRB, 0.12953E+01_JPRB, 0.13281E+01_JPRB, 0.14491E+01_JPRB ,&
&  0.16710E+01_JPRB, 0.19113E+01_JPRB, 0.21650E+01_JPRB, 0.28056E+01_JPRB, 0.23951E+01_JPRB, 0.30810E+01_JPRB, 0.34072E+01_JPRB ,&
&  0.35576E+01_JPRB, 0.36041E+01_JPRB, 0.33058E+01_JPRB, 0.36550E+01_JPRB, 0.35493E+01_JPRB, 0.35659E+01_JPRB, 0.32427E+01_JPRB ,&
&  0.28114E+01_JPRB, 0.22838E+01_JPRB, 0.20081E+01_JPRB, 0.18350E+01_JPRB, 0.17027E+01_JPRB, 0.16545E+01_JPRB, 0.11551E+01_JPRB ,&
&  0.10402E+01_JPRB, 0.11561E+01_JPRB, 0.11366E+01_JPRB, 0.10448E+01_JPRB, 0.12607E+01_JPRB, 0.13924E+01_JPRB, 0.10856E+01_JPRB ,&
&  0.54227E+00_JPRB, 0.43584E+00_JPRB, 0.38255E+00_JPRB, 0.33550E+00_JPRB, 0.29108E+00_JPRB, 0.26619E+00_JPRB, 0.44263E+00_JPRB ,&
&  0.38806E+00_JPRB, 0.47993E+00_JPRB, 0.49132E+00_JPRB, 0.55158E+00_JPRB, 0.73876E+00_JPRB, 0.81640E+00_JPRB, 0.12813E+01_JPRB ,&
&  0.11602E+01_JPRB, 0.97713E+00_JPRB, 0.83697E+00_JPRB, 0.73377E+00_JPRB, 0.69132E+00_JPRB, 0.67793E+00_JPRB, 0.67069E+00_JPRB ,&
&  0.66026E+00_JPRB, 0.62616E+00_JPRB, 0.59529E+00_JPRB, 0.55258E+00_JPRB, 0.51734E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.55032E+00_JPRB, 0.55432E+00_JPRB, 0.55752E+00_JPRB, 0.55437E+00_JPRB, 0.54172E+00_JPRB, 0.53737E+00_JPRB, 0.51133E+00_JPRB ,&
&  0.47372E+00_JPRB, 0.44856E+00_JPRB, 0.42552E+00_JPRB, 0.42681E+00_JPRB, 0.43688E+00_JPRB, 0.44625E+00_JPRB, 0.43828E+00_JPRB ,&
&  0.42819E+00_JPRB, 0.45818E+00_JPRB, 0.62467E+00_JPRB, 0.75098E+00_JPRB, 0.10805E+01_JPRB, 0.13932E+01_JPRB, 0.15968E+01_JPRB ,&
&  0.14953E+01_JPRB, 0.20094E+01_JPRB, 0.28638E+01_JPRB, 0.32949E+01_JPRB, 0.44343E+01_JPRB, 0.44176E+01_JPRB, 0.37626E+01_JPRB ,&
&  0.28214E+01_JPRB, 0.26743E+01_JPRB, 0.25598E+01_JPRB, 0.23398E+01_JPRB, 0.20560E+01_JPRB, 0.17764E+01_JPRB, 0.15661E+01_JPRB ,&
&  0.14439E+01_JPRB, 0.13816E+01_JPRB, 0.12824E+01_JPRB, 0.12269E+01_JPRB, 0.12341E+01_JPRB, 0.12639E+01_JPRB, 0.13919E+01_JPRB ,&
&  0.16076E+01_JPRB, 0.18418E+01_JPRB, 0.20703E+01_JPRB, 0.26796E+01_JPRB, 0.26933E+01_JPRB, 0.28911E+01_JPRB, 0.34631E+01_JPRB ,&
&  0.37223E+01_JPRB, 0.38807E+01_JPRB, 0.35286E+01_JPRB, 0.43215E+01_JPRB, 0.39075E+01_JPRB, 0.38571E+01_JPRB, 0.37111E+01_JPRB ,&
&  0.31583E+01_JPRB, 0.27210E+01_JPRB, 0.20560E+01_JPRB, 0.18382E+01_JPRB, 0.17562E+01_JPRB, 0.14163E+01_JPRB, 0.11576E+01_JPRB ,&
&  0.12289E+01_JPRB, 0.11743E+01_JPRB, 0.11175E+01_JPRB, 0.10055E+01_JPRB, 0.12148E+01_JPRB, 0.11779E+01_JPRB, 0.64345E+00_JPRB ,&
&  0.50679E+00_JPRB, 0.49252E+00_JPRB, 0.40400E+00_JPRB, 0.35509E+00_JPRB, 0.31181E+00_JPRB, 0.28956E+00_JPRB, 0.39029E+00_JPRB ,&
&  0.41027E+00_JPRB, 0.54085E+00_JPRB, 0.61624E+00_JPRB, 0.73854E+00_JPRB, 0.87826E+00_JPRB, 0.10042E+01_JPRB, 0.18233E+01_JPRB ,&
&  0.18011E+01_JPRB, 0.15112E+01_JPRB, 0.12115E+01_JPRB, 0.10500E+01_JPRB, 0.91604E+00_JPRB, 0.80453E+00_JPRB, 0.73016E+00_JPRB ,&
&  0.69431E+00_JPRB, 0.64901E+00_JPRB, 0.62462E+00_JPRB, 0.61018E+00_JPRB, 0.57179E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.55318E+00_JPRB, 0.54398E+00_JPRB, 0.54373E+00_JPRB, 0.53020E+00_JPRB, 0.51687E+00_JPRB, 0.51291E+00_JPRB, 0.52365E+00_JPRB ,&
&  0.50610E+00_JPRB, 0.48053E+00_JPRB, 0.45512E+00_JPRB, 0.44444E+00_JPRB, 0.45069E+00_JPRB, 0.45513E+00_JPRB, 0.44087E+00_JPRB ,&
&  0.43307E+00_JPRB, 0.46633E+00_JPRB, 0.61315E+00_JPRB, 0.72403E+00_JPRB, 0.92056E+00_JPRB, 0.12062E+01_JPRB, 0.14549E+01_JPRB ,&
&  0.16474E+01_JPRB, 0.21432E+01_JPRB, 0.28537E+01_JPRB, 0.39992E+01_JPRB, 0.54236E+01_JPRB, 0.60652E+01_JPRB, 0.64706E+01_JPRB ,&
&  0.53180E+01_JPRB, 0.42216E+01_JPRB, 0.35643E+01_JPRB, 0.30992E+01_JPRB, 0.25731E+01_JPRB, 0.21363E+01_JPRB, 0.17855E+01_JPRB ,&
&  0.15570E+01_JPRB, 0.13806E+01_JPRB, 0.13278E+01_JPRB, 0.12435E+01_JPRB, 0.12074E+01_JPRB, 0.12197E+01_JPRB, 0.13527E+01_JPRB ,&
&  0.15465E+01_JPRB, 0.17958E+01_JPRB, 0.21072E+01_JPRB, 0.25031E+01_JPRB, 0.30219E+01_JPRB, 0.35754E+01_JPRB, 0.36820E+01_JPRB ,&
&  0.34695E+01_JPRB, 0.33384E+01_JPRB, 0.38720E+01_JPRB, 0.44485E+01_JPRB, 0.41333E+01_JPRB, 0.42435E+01_JPRB, 0.39348E+01_JPRB ,&
&  0.33578E+01_JPRB, 0.27762E+01_JPRB, 0.21289E+01_JPRB, 0.19800E+01_JPRB, 0.16118E+01_JPRB, 0.12628E+01_JPRB, 0.13238E+01_JPRB ,&
&  0.13631E+01_JPRB, 0.12734E+01_JPRB, 0.11205E+01_JPRB, 0.95205E+00_JPRB, 0.96021E+00_JPRB, 0.79605E+00_JPRB, 0.61041E+00_JPRB ,&
&  0.55460E+00_JPRB, 0.49252E+00_JPRB, 0.43265E+00_JPRB, 0.43358E+00_JPRB, 0.37905E+00_JPRB, 0.36524E+00_JPRB, 0.44402E+00_JPRB ,&
&  0.51491E+00_JPRB, 0.57542E+00_JPRB, 0.75251E+00_JPRB, 0.85311E+00_JPRB, 0.88211E+00_JPRB, 0.82798E+00_JPRB, 0.12863E+01_JPRB ,&
&  0.10484E+01_JPRB, 0.11215E+01_JPRB, 0.11074E+01_JPRB, 0.10200E+01_JPRB, 0.93756E+00_JPRB, 0.85293E+00_JPRB, 0.76700E+00_JPRB ,&
&  0.70078E+00_JPRB, 0.64451E+00_JPRB, 0.60561E+00_JPRB, 0.59244E+00_JPRB, 0.57574E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.53521E+00_JPRB, 0.52130E+00_JPRB, 0.49971E+00_JPRB, 0.46673E+00_JPRB, 0.45530E+00_JPRB, 0.46589E+00_JPRB, 0.48990E+00_JPRB ,&
&  0.49799E+00_JPRB, 0.49970E+00_JPRB, 0.48782E+00_JPRB, 0.47277E+00_JPRB, 0.47205E+00_JPRB, 0.46526E+00_JPRB, 0.44774E+00_JPRB ,&
&  0.44242E+00_JPRB, 0.48622E+00_JPRB, 0.43104E+00_JPRB, 0.48704E+00_JPRB, 0.68493E+00_JPRB, 0.10785E+01_JPRB, 0.11198E+01_JPRB ,&
&  0.15722E+01_JPRB, 0.21415E+01_JPRB, 0.27137E+01_JPRB, 0.40592E+01_JPRB, 0.73207E+01_JPRB, 0.75209E+01_JPRB, 0.72282E+01_JPRB ,&
&  0.66639E+01_JPRB, 0.53999E+01_JPRB, 0.42576E+01_JPRB, 0.33626E+01_JPRB, 0.26946E+01_JPRB, 0.22465E+01_JPRB, 0.19058E+01_JPRB ,&
&  0.16769E+01_JPRB, 0.14739E+01_JPRB, 0.13327E+01_JPRB, 0.12930E+01_JPRB, 0.12516E+01_JPRB, 0.12315E+01_JPRB, 0.13349E+01_JPRB ,&
&  0.14881E+01_JPRB, 0.17299E+01_JPRB, 0.21847E+01_JPRB, 0.26535E+01_JPRB, 0.28626E+01_JPRB, 0.32959E+01_JPRB, 0.42881E+01_JPRB ,&
&  0.38947E+01_JPRB, 0.38191E+01_JPRB, 0.37214E+01_JPRB, 0.46679E+01_JPRB, 0.45071E+01_JPRB, 0.42510E+01_JPRB, 0.41048E+01_JPRB ,&
&  0.29837E+01_JPRB, 0.23636E+01_JPRB, 0.20870E+01_JPRB, 0.18613E+01_JPRB, 0.14396E+01_JPRB, 0.14395E+01_JPRB, 0.18182E+01_JPRB ,&
&  0.14611E+01_JPRB, 0.14240E+01_JPRB, 0.14083E+01_JPRB, 0.13321E+01_JPRB, 0.92802E+00_JPRB, 0.69402E+00_JPRB, 0.90679E+00_JPRB ,&
&  0.87128E+00_JPRB, 0.57886E+00_JPRB, 0.47963E+00_JPRB, 0.54925E+00_JPRB, 0.45975E+00_JPRB, 0.48408E+00_JPRB, 0.58436E+00_JPRB ,&
&  0.58228E+00_JPRB, 0.61242E+00_JPRB, 0.86171E+00_JPRB, 0.92027E+00_JPRB, 0.89313E+00_JPRB, 0.78822E+00_JPRB, 0.95152E+00_JPRB ,&
&  0.99826E+00_JPRB, 0.98817E+00_JPRB, 0.10279E+01_JPRB, 0.95009E+00_JPRB, 0.88108E+00_JPRB, 0.82567E+00_JPRB, 0.76464E+00_JPRB ,&
&  0.71532E+00_JPRB, 0.65578E+00_JPRB, 0.60141E+00_JPRB, 0.56037E+00_JPRB, 0.54536E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.51750E+00_JPRB, 0.50370E+00_JPRB, 0.48685E+00_JPRB, 0.46548E+00_JPRB, 0.44247E+00_JPRB, 0.43733E+00_JPRB, 0.44597E+00_JPRB ,&
&  0.45773E+00_JPRB, 0.47285E+00_JPRB, 0.48020E+00_JPRB, 0.48280E+00_JPRB, 0.48214E+00_JPRB, 0.47539E+00_JPRB, 0.45200E+00_JPRB ,&
&  0.44659E+00_JPRB, 0.44486E+00_JPRB, 0.38240E+00_JPRB, 0.39655E+00_JPRB, 0.61474E+00_JPRB, 0.87025E+00_JPRB, 0.99591E+00_JPRB ,&
&  0.13941E+01_JPRB, 0.19812E+01_JPRB, 0.27236E+01_JPRB, 0.41639E+01_JPRB, 0.56695E+01_JPRB, 0.65854E+01_JPRB, 0.61785E+01_JPRB ,&
&  0.61057E+01_JPRB, 0.54473E+01_JPRB, 0.45100E+01_JPRB, 0.37350E+01_JPRB, 0.30311E+01_JPRB, 0.24792E+01_JPRB, 0.20680E+01_JPRB ,&
&  0.17992E+01_JPRB, 0.16070E+01_JPRB, 0.14425E+01_JPRB, 0.13435E+01_JPRB, 0.13209E+01_JPRB, 0.12948E+01_JPRB, 0.13468E+01_JPRB ,&
&  0.14319E+01_JPRB, 0.16159E+01_JPRB, 0.21725E+01_JPRB, 0.29257E+01_JPRB, 0.32811E+01_JPRB, 0.34349E+01_JPRB, 0.36999E+01_JPRB ,&
&  0.37785E+01_JPRB, 0.36389E+01_JPRB, 0.38377E+01_JPRB, 0.39788E+01_JPRB, 0.42178E+01_JPRB, 0.40865E+01_JPRB, 0.43096E+01_JPRB ,&
&  0.34832E+01_JPRB, 0.27856E+01_JPRB, 0.21538E+01_JPRB, 0.16728E+01_JPRB, 0.15110E+01_JPRB, 0.19033E+01_JPRB, 0.19153E+01_JPRB ,&
&  0.16758E+01_JPRB, 0.15703E+01_JPRB, 0.14341E+01_JPRB, 0.12357E+01_JPRB, 0.11131E+01_JPRB, 0.74508E+00_JPRB, 0.80868E+00_JPRB ,&
&  0.85073E+00_JPRB, 0.85025E+00_JPRB, 0.78264E+00_JPRB, 0.68975E+00_JPRB, 0.62598E+00_JPRB, 0.63682E+00_JPRB, 0.64009E+00_JPRB ,&
&  0.63508E+00_JPRB, 0.57978E+00_JPRB, 0.67515E+00_JPRB, 0.81169E+00_JPRB, 0.87807E+00_JPRB, 0.76159E+00_JPRB, 0.87312E+00_JPRB ,&
&  0.86453E+00_JPRB, 0.93762E+00_JPRB, 0.93756E+00_JPRB, 0.89637E+00_JPRB, 0.83188E+00_JPRB, 0.77397E+00_JPRB, 0.73204E+00_JPRB ,&
&  0.68840E+00_JPRB, 0.63868E+00_JPRB, 0.58025E+00_JPRB, 0.53997E+00_JPRB, 0.52628E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.53992E+00_JPRB, 0.52105E+00_JPRB, 0.49711E+00_JPRB, 0.47676E+00_JPRB, 0.45696E+00_JPRB, 0.44892E+00_JPRB, 0.45075E+00_JPRB ,&
&  0.45797E+00_JPRB, 0.46336E+00_JPRB, 0.47422E+00_JPRB, 0.48460E+00_JPRB, 0.49032E+00_JPRB, 0.48705E+00_JPRB, 0.46090E+00_JPRB ,&
&  0.46376E+00_JPRB, 0.47566E+00_JPRB, 0.37575E+00_JPRB, 0.40326E+00_JPRB, 0.52803E+00_JPRB, 0.66447E+00_JPRB, 0.93242E+00_JPRB ,&
&  0.11665E+01_JPRB, 0.16356E+01_JPRB, 0.22371E+01_JPRB, 0.32062E+01_JPRB, 0.38718E+01_JPRB, 0.42125E+01_JPRB, 0.41776E+01_JPRB ,&
&  0.37832E+01_JPRB, 0.37636E+01_JPRB, 0.35135E+01_JPRB, 0.31155E+01_JPRB, 0.27349E+01_JPRB, 0.24395E+01_JPRB, 0.21950E+01_JPRB ,&
&  0.19790E+01_JPRB, 0.17953E+01_JPRB, 0.16553E+01_JPRB, 0.15393E+01_JPRB, 0.14850E+01_JPRB, 0.14547E+01_JPRB, 0.14598E+01_JPRB ,&
&  0.14317E+01_JPRB, 0.14624E+01_JPRB, 0.18358E+01_JPRB, 0.27133E+01_JPRB, 0.30600E+01_JPRB, 0.34988E+01_JPRB, 0.35466E+01_JPRB ,&
&  0.40916E+01_JPRB, 0.42330E+01_JPRB, 0.41295E+01_JPRB, 0.46878E+01_JPRB, 0.40522E+01_JPRB, 0.41647E+01_JPRB, 0.42168E+01_JPRB ,&
&  0.44020E+01_JPRB, 0.39164E+01_JPRB, 0.33287E+01_JPRB, 0.25894E+01_JPRB, 0.19025E+01_JPRB, 0.21746E+01_JPRB, 0.19367E+01_JPRB ,&
&  0.16596E+01_JPRB, 0.15986E+01_JPRB, 0.14813E+01_JPRB, 0.13726E+01_JPRB, 0.11372E+01_JPRB, 0.97827E+00_JPRB, 0.74247E+00_JPRB ,&
&  0.73307E+00_JPRB, 0.67197E+00_JPRB, 0.66962E+00_JPRB, 0.62317E+00_JPRB, 0.60464E+00_JPRB, 0.55736E+00_JPRB, 0.52048E+00_JPRB ,&
&  0.52883E+00_JPRB, 0.48227E+00_JPRB, 0.48127E+00_JPRB, 0.54137E+00_JPRB, 0.68946E+00_JPRB, 0.67514E+00_JPRB, 0.74732E+00_JPRB ,&
&  0.79063E+00_JPRB, 0.83717E+00_JPRB, 0.76207E+00_JPRB, 0.79140E+00_JPRB, 0.75205E+00_JPRB, 0.71299E+00_JPRB, 0.69451E+00_JPRB ,&
&  0.64649E+00_JPRB, 0.61248E+00_JPRB, 0.58286E+00_JPRB, 0.55185E+00_JPRB, 0.54600E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.51808E+00_JPRB, 0.49490E+00_JPRB, 0.48156E+00_JPRB, 0.47423E+00_JPRB, 0.45995E+00_JPRB, 0.45233E+00_JPRB, 0.46564E+00_JPRB ,&
&  0.47180E+00_JPRB, 0.46507E+00_JPRB, 0.46328E+00_JPRB, 0.46856E+00_JPRB, 0.47696E+00_JPRB, 0.48071E+00_JPRB, 0.46940E+00_JPRB ,&
&  0.47166E+00_JPRB, 0.63138E+00_JPRB, 0.46477E+00_JPRB, 0.37578E+00_JPRB, 0.36503E+00_JPRB, 0.49437E+00_JPRB, 0.68870E+00_JPRB ,&
&  0.94142E+00_JPRB, 0.12200E+01_JPRB, 0.16917E+01_JPRB, 0.20494E+01_JPRB, 0.23648E+01_JPRB, 0.25338E+01_JPRB, 0.26195E+01_JPRB ,&
&  0.26595E+01_JPRB, 0.20850E+01_JPRB, 0.19920E+01_JPRB, 0.19048E+01_JPRB, 0.18928E+01_JPRB, 0.18598E+01_JPRB, 0.18540E+01_JPRB ,&
&  0.18047E+01_JPRB, 0.17610E+01_JPRB, 0.17026E+01_JPRB, 0.16330E+01_JPRB, 0.15621E+01_JPRB, 0.15031E+01_JPRB, 0.14548E+01_JPRB ,&
&  0.14484E+01_JPRB, 0.14306E+01_JPRB, 0.15211E+01_JPRB, 0.20009E+01_JPRB, 0.28951E+01_JPRB, 0.35567E+01_JPRB, 0.40441E+01_JPRB ,&
&  0.37053E+01_JPRB, 0.34395E+01_JPRB, 0.46379E+01_JPRB, 0.50071E+01_JPRB, 0.51280E+01_JPRB, 0.52754E+01_JPRB, 0.48345E+01_JPRB ,&
&  0.46665E+01_JPRB, 0.41030E+01_JPRB, 0.34158E+01_JPRB, 0.26964E+01_JPRB, 0.25017E+01_JPRB, 0.23171E+01_JPRB, 0.20786E+01_JPRB ,&
&  0.18289E+01_JPRB, 0.18118E+01_JPRB, 0.16362E+01_JPRB, 0.14160E+01_JPRB, 0.12685E+01_JPRB, 0.12066E+01_JPRB, 0.94733E+00_JPRB ,&
&  0.75607E+00_JPRB, 0.68878E+00_JPRB, 0.56106E+00_JPRB, 0.48927E+00_JPRB, 0.44796E+00_JPRB, 0.40257E+00_JPRB, 0.38043E+00_JPRB ,&
&  0.37492E+00_JPRB, 0.37710E+00_JPRB, 0.39021E+00_JPRB, 0.42236E+00_JPRB, 0.52071E+00_JPRB, 0.57722E+00_JPRB, 0.60323E+00_JPRB ,&
&  0.57820E+00_JPRB, 0.66822E+00_JPRB, 0.70664E+00_JPRB, 0.71630E+00_JPRB, 0.75778E+00_JPRB, 0.68691E+00_JPRB, 0.65923E+00_JPRB ,&
&  0.61519E+00_JPRB, 0.59822E+00_JPRB, 0.59360E+00_JPRB, 0.56882E+00_JPRB, 0.54767E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.52619E+00_JPRB, 0.50898E+00_JPRB, 0.49993E+00_JPRB, 0.48940E+00_JPRB, 0.48365E+00_JPRB, 0.48066E+00_JPRB, 0.48978E+00_JPRB ,&
&  0.48817E+00_JPRB, 0.48460E+00_JPRB, 0.47921E+00_JPRB, 0.47748E+00_JPRB, 0.47903E+00_JPRB, 0.47849E+00_JPRB, 0.47666E+00_JPRB ,&
&  0.48944E+00_JPRB, 0.75282E+00_JPRB, 0.58240E+00_JPRB, 0.49556E+00_JPRB, 0.41766E+00_JPRB, 0.47994E+00_JPRB, 0.58958E+00_JPRB ,&
&  0.79027E+00_JPRB, 0.89434E+00_JPRB, 0.10951E+01_JPRB, 0.13256E+01_JPRB, 0.15440E+01_JPRB, 0.14740E+01_JPRB, 0.15463E+01_JPRB ,&
&  0.14181E+01_JPRB, 0.13059E+01_JPRB, 0.10946E+01_JPRB, 0.12026E+01_JPRB, 0.12213E+01_JPRB, 0.11119E+01_JPRB, 0.13331E+01_JPRB ,&
&  0.14349E+01_JPRB, 0.15130E+01_JPRB, 0.15689E+01_JPRB, 0.15987E+01_JPRB, 0.16077E+01_JPRB, 0.15706E+01_JPRB, 0.15141E+01_JPRB ,&
&  0.14349E+01_JPRB, 0.14169E+01_JPRB, 0.14914E+01_JPRB, 0.16246E+01_JPRB, 0.20226E+01_JPRB, 0.25729E+01_JPRB, 0.36521E+01_JPRB ,&
&  0.43917E+01_JPRB, 0.41397E+01_JPRB, 0.38256E+01_JPRB, 0.47457E+01_JPRB, 0.53298E+01_JPRB, 0.48508E+01_JPRB, 0.42355E+01_JPRB ,&
&  0.40961E+01_JPRB, 0.38548E+01_JPRB, 0.34515E+01_JPRB, 0.30988E+01_JPRB, 0.27607E+01_JPRB, 0.25269E+01_JPRB, 0.22805E+01_JPRB ,&
&  0.20132E+01_JPRB, 0.19242E+01_JPRB, 0.18303E+01_JPRB, 0.15244E+01_JPRB, 0.12628E+01_JPRB, 0.11127E+01_JPRB, 0.97570E+00_JPRB ,&
&  0.83678E+00_JPRB, 0.56667E+00_JPRB, 0.40495E+00_JPRB, 0.31976E+00_JPRB, 0.29925E+00_JPRB, 0.27691E+00_JPRB, 0.27698E+00_JPRB ,&
&  0.27615E+00_JPRB, 0.30449E+00_JPRB, 0.36277E+00_JPRB, 0.39024E+00_JPRB, 0.50038E+00_JPRB, 0.55061E+00_JPRB, 0.51882E+00_JPRB ,&
&  0.46877E+00_JPRB, 0.47932E+00_JPRB, 0.67590E+00_JPRB, 0.71133E+00_JPRB, 0.72775E+00_JPRB, 0.66833E+00_JPRB, 0.64917E+00_JPRB ,&
&  0.62543E+00_JPRB, 0.61737E+00_JPRB, 0.60717E+00_JPRB, 0.57919E+00_JPRB, 0.54744E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.52873E+00_JPRB, 0.52369E+00_JPRB, 0.51761E+00_JPRB, 0.50469E+00_JPRB, 0.49525E+00_JPRB, 0.48253E+00_JPRB, 0.48720E+00_JPRB ,&
&  0.48777E+00_JPRB, 0.48893E+00_JPRB, 0.48631E+00_JPRB, 0.48659E+00_JPRB, 0.48859E+00_JPRB, 0.48737E+00_JPRB, 0.48731E+00_JPRB ,&
&  0.54483E+00_JPRB, 0.67454E+00_JPRB, 0.58459E+00_JPRB, 0.45798E+00_JPRB, 0.51552E+00_JPRB, 0.51300E+00_JPRB, 0.56692E+00_JPRB ,&
&  0.62312E+00_JPRB, 0.71614E+00_JPRB, 0.77334E+00_JPRB, 0.81594E+00_JPRB, 0.90341E+00_JPRB, 0.97301E+00_JPRB, 0.96653E+00_JPRB ,&
&  0.88466E+00_JPRB, 0.78409E+00_JPRB, 0.74624E+00_JPRB, 0.74841E+00_JPRB, 0.85214E+00_JPRB, 0.79383E+00_JPRB, 0.93326E+00_JPRB ,&
&  0.10329E+01_JPRB, 0.11257E+01_JPRB, 0.12146E+01_JPRB, 0.13095E+01_JPRB, 0.13828E+01_JPRB, 0.14435E+01_JPRB, 0.14654E+01_JPRB ,&
&  0.14595E+01_JPRB, 0.14595E+01_JPRB, 0.14717E+01_JPRB, 0.15765E+01_JPRB, 0.18042E+01_JPRB, 0.24925E+01_JPRB, 0.39875E+01_JPRB ,&
&  0.42943E+01_JPRB, 0.42730E+01_JPRB, 0.48938E+01_JPRB, 0.53178E+01_JPRB, 0.49120E+01_JPRB, 0.43681E+01_JPRB, 0.38524E+01_JPRB ,&
&  0.35615E+01_JPRB, 0.34041E+01_JPRB, 0.32970E+01_JPRB, 0.32104E+01_JPRB, 0.30377E+01_JPRB, 0.27156E+01_JPRB, 0.24325E+01_JPRB ,&
&  0.21270E+01_JPRB, 0.20103E+01_JPRB, 0.19274E+01_JPRB, 0.15883E+01_JPRB, 0.12256E+01_JPRB, 0.98589E+00_JPRB, 0.84143E+00_JPRB ,&
&  0.70543E+00_JPRB, 0.46431E+00_JPRB, 0.34390E+00_JPRB, 0.27006E+00_JPRB, 0.25920E+00_JPRB, 0.25071E+00_JPRB, 0.28287E+00_JPRB ,&
&  0.27210E+00_JPRB, 0.31542E+00_JPRB, 0.38592E+00_JPRB, 0.41627E+00_JPRB, 0.44754E+00_JPRB, 0.47084E+00_JPRB, 0.38892E+00_JPRB ,&
&  0.36575E+00_JPRB, 0.43164E+00_JPRB, 0.63626E+00_JPRB, 0.67272E+00_JPRB, 0.69545E+00_JPRB, 0.67212E+00_JPRB, 0.66643E+00_JPRB ,&
&  0.63861E+00_JPRB, 0.61625E+00_JPRB, 0.57822E+00_JPRB, 0.54981E+00_JPRB, 0.53823E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.55297E+00_JPRB, 0.53832E+00_JPRB, 0.52772E+00_JPRB, 0.51561E+00_JPRB, 0.49451E+00_JPRB, 0.47722E+00_JPRB, 0.48055E+00_JPRB ,&
&  0.47931E+00_JPRB, 0.47740E+00_JPRB, 0.48085E+00_JPRB, 0.48272E+00_JPRB, 0.48038E+00_JPRB, 0.48575E+00_JPRB, 0.51574E+00_JPRB ,&
&  0.43486E+00_JPRB, 0.48307E+00_JPRB, 0.42682E+00_JPRB, 0.43591E+00_JPRB, 0.58190E+00_JPRB, 0.53353E+00_JPRB, 0.53997E+00_JPRB ,&
&  0.54935E+00_JPRB, 0.58357E+00_JPRB, 0.59180E+00_JPRB, 0.59708E+00_JPRB, 0.61049E+00_JPRB, 0.67182E+00_JPRB, 0.70166E+00_JPRB ,&
&  0.61803E+00_JPRB, 0.54309E+00_JPRB, 0.53608E+00_JPRB, 0.54389E+00_JPRB, 0.59744E+00_JPRB, 0.68084E+00_JPRB, 0.79510E+00_JPRB ,&
&  0.85239E+00_JPRB, 0.90630E+00_JPRB, 0.98243E+00_JPRB, 0.10613E+01_JPRB, 0.11329E+01_JPRB, 0.12012E+01_JPRB, 0.12328E+01_JPRB ,&
&  0.12392E+01_JPRB, 0.12811E+01_JPRB, 0.13478E+01_JPRB, 0.15027E+01_JPRB, 0.16186E+01_JPRB, 0.24871E+01_JPRB, 0.44218E+01_JPRB ,&
&  0.48172E+01_JPRB, 0.41760E+01_JPRB, 0.42098E+01_JPRB, 0.48857E+01_JPRB, 0.47115E+01_JPRB, 0.42480E+01_JPRB, 0.36253E+01_JPRB ,&
&  0.31662E+01_JPRB, 0.29092E+01_JPRB, 0.27279E+01_JPRB, 0.26636E+01_JPRB, 0.26440E+01_JPRB, 0.25733E+01_JPRB, 0.24650E+01_JPRB ,&
&  0.22091E+01_JPRB, 0.20460E+01_JPRB, 0.19242E+01_JPRB, 0.16058E+01_JPRB, 0.12827E+01_JPRB, 0.99958E+00_JPRB, 0.76918E+00_JPRB ,&
&  0.59856E+00_JPRB, 0.45036E+00_JPRB, 0.35677E+00_JPRB, 0.30959E+00_JPRB, 0.29765E+00_JPRB, 0.30144E+00_JPRB, 0.36624E+00_JPRB ,&
&  0.32336E+00_JPRB, 0.34871E+00_JPRB, 0.36542E+00_JPRB, 0.37234E+00_JPRB, 0.37677E+00_JPRB, 0.36981E+00_JPRB, 0.35079E+00_JPRB ,&
&  0.37310E+00_JPRB, 0.50125E+00_JPRB, 0.61192E+00_JPRB, 0.64192E+00_JPRB, 0.65580E+00_JPRB, 0.63292E+00_JPRB, 0.55554E+00_JPRB ,&
&  0.64805E+00_JPRB, 0.57367E+00_JPRB, 0.54628E+00_JPRB, 0.54792E+00_JPRB, 0.56309E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.54987E+00_JPRB, 0.54042E+00_JPRB, 0.52824E+00_JPRB, 0.50854E+00_JPRB, 0.48517E+00_JPRB, 0.46722E+00_JPRB, 0.46184E+00_JPRB ,&
&  0.46579E+00_JPRB, 0.46917E+00_JPRB, 0.47187E+00_JPRB, 0.46971E+00_JPRB, 0.46067E+00_JPRB, 0.48039E+00_JPRB, 0.43279E+00_JPRB ,&
&  0.37749E+00_JPRB, 0.39452E+00_JPRB, 0.38916E+00_JPRB, 0.41407E+00_JPRB, 0.44769E+00_JPRB, 0.43730E+00_JPRB, 0.43811E+00_JPRB ,&
&  0.45819E+00_JPRB, 0.47039E+00_JPRB, 0.49034E+00_JPRB, 0.49856E+00_JPRB, 0.50426E+00_JPRB, 0.54461E+00_JPRB, 0.60316E+00_JPRB ,&
&  0.56193E+00_JPRB, 0.51381E+00_JPRB, 0.49245E+00_JPRB, 0.51188E+00_JPRB, 0.59431E+00_JPRB, 0.69464E+00_JPRB, 0.75312E+00_JPRB ,&
&  0.78047E+00_JPRB, 0.81248E+00_JPRB, 0.87834E+00_JPRB, 0.92596E+00_JPRB, 0.96103E+00_JPRB, 0.99557E+00_JPRB, 0.10215E+01_JPRB ,&
&  0.10467E+01_JPRB, 0.11064E+01_JPRB, 0.11943E+01_JPRB, 0.13469E+01_JPRB, 0.14857E+01_JPRB, 0.20356E+01_JPRB, 0.28689E+01_JPRB ,&
&  0.30558E+01_JPRB, 0.30350E+01_JPRB, 0.29562E+01_JPRB, 0.30168E+01_JPRB, 0.31509E+01_JPRB, 0.29593E+01_JPRB, 0.26764E+01_JPRB ,&
&  0.25250E+01_JPRB, 0.22339E+01_JPRB, 0.22925E+01_JPRB, 0.22922E+01_JPRB, 0.22369E+01_JPRB, 0.22385E+01_JPRB, 0.21924E+01_JPRB ,&
&  0.20672E+01_JPRB, 0.18766E+01_JPRB, 0.18609E+01_JPRB, 0.16294E+01_JPRB, 0.13352E+01_JPRB, 0.11021E+01_JPRB, 0.86894E+00_JPRB ,&
&  0.68163E+00_JPRB, 0.53309E+00_JPRB, 0.42969E+00_JPRB, 0.38903E+00_JPRB, 0.35809E+00_JPRB, 0.35179E+00_JPRB, 0.35624E+00_JPRB ,&
&  0.34863E+00_JPRB, 0.34844E+00_JPRB, 0.34163E+00_JPRB, 0.35033E+00_JPRB, 0.34061E+00_JPRB, 0.33777E+00_JPRB, 0.37903E+00_JPRB ,&
&  0.45541E+00_JPRB, 0.64861E+00_JPRB, 0.60277E+00_JPRB, 0.61835E+00_JPRB, 0.61021E+00_JPRB, 0.58344E+00_JPRB, 0.43450E+00_JPRB ,&
&  0.48710E+00_JPRB, 0.55562E+00_JPRB, 0.53270E+00_JPRB, 0.53699E+00_JPRB, 0.54593E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.52039E+00_JPRB, 0.52440E+00_JPRB, 0.50602E+00_JPRB, 0.48520E+00_JPRB, 0.47281E+00_JPRB, 0.44913E+00_JPRB, 0.42718E+00_JPRB ,&
&  0.47652E+00_JPRB, 0.46677E+00_JPRB, 0.46732E+00_JPRB, 0.47454E+00_JPRB, 0.47061E+00_JPRB, 0.41581E+00_JPRB, 0.36805E+00_JPRB ,&
&  0.34014E+00_JPRB, 0.33513E+00_JPRB, 0.35653E+00_JPRB, 0.36797E+00_JPRB, 0.37360E+00_JPRB, 0.36874E+00_JPRB, 0.37308E+00_JPRB ,&
&  0.38339E+00_JPRB, 0.40201E+00_JPRB, 0.43192E+00_JPRB, 0.45413E+00_JPRB, 0.46527E+00_JPRB, 0.48525E+00_JPRB, 0.54320E+00_JPRB ,&
&  0.53022E+00_JPRB, 0.54261E+00_JPRB, 0.56350E+00_JPRB, 0.51949E+00_JPRB, 0.63744E+00_JPRB, 0.67663E+00_JPRB, 0.70108E+00_JPRB ,&
&  0.75064E+00_JPRB, 0.79827E+00_JPRB, 0.82716E+00_JPRB, 0.84659E+00_JPRB, 0.88136E+00_JPRB, 0.89820E+00_JPRB, 0.91902E+00_JPRB ,&
&  0.94750E+00_JPRB, 0.98759E+00_JPRB, 0.10399E+01_JPRB, 0.11199E+01_JPRB, 0.13183E+01_JPRB, 0.15307E+01_JPRB, 0.18230E+01_JPRB ,&
&  0.19957E+01_JPRB, 0.21066E+01_JPRB, 0.22248E+01_JPRB, 0.20020E+01_JPRB, 0.24943E+01_JPRB, 0.26868E+01_JPRB, 0.24109E+01_JPRB ,&
&  0.23138E+01_JPRB, 0.19945E+01_JPRB, 0.19390E+01_JPRB, 0.19968E+01_JPRB, 0.20145E+01_JPRB, 0.20154E+01_JPRB, 0.20143E+01_JPRB ,&
&  0.18801E+01_JPRB, 0.17362E+01_JPRB, 0.17213E+01_JPRB, 0.15497E+01_JPRB, 0.13316E+01_JPRB, 0.11568E+01_JPRB, 0.95969E+00_JPRB ,&
&  0.77689E+00_JPRB, 0.62086E+00_JPRB, 0.52786E+00_JPRB, 0.48524E+00_JPRB, 0.42647E+00_JPRB, 0.41191E+00_JPRB, 0.39725E+00_JPRB ,&
&  0.37851E+00_JPRB, 0.36032E+00_JPRB, 0.35381E+00_JPRB, 0.34466E+00_JPRB, 0.33810E+00_JPRB, 0.34849E+00_JPRB, 0.36106E+00_JPRB ,&
&  0.40379E+00_JPRB, 0.47640E+00_JPRB, 0.63830E+00_JPRB, 0.58773E+00_JPRB, 0.56137E+00_JPRB, 0.53653E+00_JPRB, 0.45864E+00_JPRB ,&
&  0.43441E+00_JPRB, 0.51759E+00_JPRB, 0.51239E+00_JPRB, 0.51132E+00_JPRB, 0.50663E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.48401E+00_JPRB, 0.48965E+00_JPRB, 0.47862E+00_JPRB, 0.46652E+00_JPRB, 0.45146E+00_JPRB, 0.40229E+00_JPRB, 0.40060E+00_JPRB ,&
&  0.46989E+00_JPRB, 0.41856E+00_JPRB, 0.46877E+00_JPRB, 0.36877E+00_JPRB, 0.34656E+00_JPRB, 0.33191E+00_JPRB, 0.31515E+00_JPRB ,&
&  0.29725E+00_JPRB, 0.30245E+00_JPRB, 0.31902E+00_JPRB, 0.33020E+00_JPRB, 0.33798E+00_JPRB, 0.33691E+00_JPRB, 0.34489E+00_JPRB ,&
&  0.35977E+00_JPRB, 0.38538E+00_JPRB, 0.41854E+00_JPRB, 0.43802E+00_JPRB, 0.45992E+00_JPRB, 0.49185E+00_JPRB, 0.52407E+00_JPRB ,&
&  0.49852E+00_JPRB, 0.50396E+00_JPRB, 0.52062E+00_JPRB, 0.54662E+00_JPRB, 0.59016E+00_JPRB, 0.65543E+00_JPRB, 0.69840E+00_JPRB ,&
&  0.71967E+00_JPRB, 0.59644E+00_JPRB, 0.71290E+00_JPRB, 0.74514E+00_JPRB, 0.81108E+00_JPRB, 0.84660E+00_JPRB, 0.86860E+00_JPRB ,&
&  0.90129E+00_JPRB, 0.93501E+00_JPRB, 0.97195E+00_JPRB, 0.10183E+01_JPRB, 0.10781E+01_JPRB, 0.12040E+01_JPRB, 0.12992E+01_JPRB ,&
&  0.15050E+01_JPRB, 0.11140E+01_JPRB, 0.12459E+01_JPRB, 0.13358E+01_JPRB, 0.18133E+01_JPRB, 0.19843E+01_JPRB, 0.19108E+01_JPRB ,&
&  0.18911E+01_JPRB, 0.17582E+01_JPRB, 0.16665E+01_JPRB, 0.16319E+01_JPRB, 0.16688E+01_JPRB, 0.17106E+01_JPRB, 0.17214E+01_JPRB ,&
&  0.16391E+01_JPRB, 0.15599E+01_JPRB, 0.16078E+01_JPRB, 0.14897E+01_JPRB, 0.13192E+01_JPRB, 0.11408E+01_JPRB, 0.94808E+00_JPRB ,&
&  0.81115E+00_JPRB, 0.68485E+00_JPRB, 0.57515E+00_JPRB, 0.50608E+00_JPRB, 0.45699E+00_JPRB, 0.42841E+00_JPRB, 0.41823E+00_JPRB ,&
&  0.41812E+00_JPRB, 0.41363E+00_JPRB, 0.40537E+00_JPRB, 0.38056E+00_JPRB, 0.37039E+00_JPRB, 0.36813E+00_JPRB, 0.38573E+00_JPRB ,&
&  0.38698E+00_JPRB, 0.36744E+00_JPRB, 0.39548E+00_JPRB, 0.42405E+00_JPRB, 0.42162E+00_JPRB, 0.43422E+00_JPRB, 0.48457E+00_JPRB ,&
&  0.41661E+00_JPRB, 0.44377E+00_JPRB, 0.49007E+00_JPRB, 0.49684E+00_JPRB, 0.48408E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.45925E+00_JPRB, 0.45248E+00_JPRB, 0.44916E+00_JPRB, 0.44065E+00_JPRB, 0.42769E+00_JPRB, 0.39103E+00_JPRB, 0.36734E+00_JPRB ,&
&  0.38185E+00_JPRB, 0.36941E+00_JPRB, 0.35742E+00_JPRB, 0.33196E+00_JPRB, 0.31751E+00_JPRB, 0.30332E+00_JPRB, 0.29053E+00_JPRB ,&
&  0.28343E+00_JPRB, 0.29791E+00_JPRB, 0.32493E+00_JPRB, 0.33270E+00_JPRB, 0.33476E+00_JPRB, 0.33813E+00_JPRB, 0.35082E+00_JPRB ,&
&  0.37006E+00_JPRB, 0.39284E+00_JPRB, 0.40815E+00_JPRB, 0.42662E+00_JPRB, 0.45101E+00_JPRB, 0.47811E+00_JPRB, 0.47734E+00_JPRB ,&
&  0.47335E+00_JPRB, 0.45855E+00_JPRB, 0.44198E+00_JPRB, 0.48126E+00_JPRB, 0.54892E+00_JPRB, 0.61375E+00_JPRB, 0.59789E+00_JPRB ,&
&  0.41599E+00_JPRB, 0.41719E+00_JPRB, 0.59765E+00_JPRB, 0.65425E+00_JPRB, 0.71161E+00_JPRB, 0.74705E+00_JPRB, 0.79407E+00_JPRB ,&
&  0.83672E+00_JPRB, 0.87657E+00_JPRB, 0.90710E+00_JPRB, 0.94171E+00_JPRB, 0.99210E+00_JPRB, 0.10633E+01_JPRB, 0.10888E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.94873E+00_JPRB, 0.90029E+00_JPRB, 0.96481E+00_JPRB, 0.13601E+01_JPRB, 0.13806E+01_JPRB, 0.13293E+01_JPRB ,&
&  0.13588E+01_JPRB, 0.13704E+01_JPRB, 0.13862E+01_JPRB, 0.13925E+01_JPRB, 0.13420E+01_JPRB, 0.13412E+01_JPRB, 0.13668E+01_JPRB ,&
&  0.14583E+01_JPRB, 0.15180E+01_JPRB, 0.14964E+01_JPRB, 0.14465E+01_JPRB, 0.13479E+01_JPRB, 0.12321E+01_JPRB, 0.10609E+01_JPRB ,&
&  0.85795E+00_JPRB, 0.71205E+00_JPRB, 0.57985E+00_JPRB, 0.49670E+00_JPRB, 0.45831E+00_JPRB, 0.44885E+00_JPRB, 0.44473E+00_JPRB ,&
&  0.45909E+00_JPRB, 0.46828E+00_JPRB, 0.46133E+00_JPRB, 0.42394E+00_JPRB, 0.39544E+00_JPRB, 0.38481E+00_JPRB, 0.37587E+00_JPRB ,&
&  0.36704E+00_JPRB, 0.33211E+00_JPRB, 0.30532E+00_JPRB, 0.31378E+00_JPRB, 0.34346E+00_JPRB, 0.36884E+00_JPRB, 0.38482E+00_JPRB ,&
&  0.38155E+00_JPRB, 0.38344E+00_JPRB, 0.39534E+00_JPRB, 0.41840E+00_JPRB, 0.43639E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.44587E+00_JPRB, 0.41823E+00_JPRB, 0.40223E+00_JPRB, 0.40719E+00_JPRB, 0.37523E+00_JPRB, 0.34849E+00_JPRB, 0.33758E+00_JPRB ,&
&  0.34481E+00_JPRB, 0.35027E+00_JPRB, 0.35272E+00_JPRB, 0.33448E+00_JPRB, 0.32014E+00_JPRB, 0.31215E+00_JPRB, 0.31025E+00_JPRB ,&
&  0.32240E+00_JPRB, 0.33757E+00_JPRB, 0.34858E+00_JPRB, 0.34570E+00_JPRB, 0.34559E+00_JPRB, 0.35263E+00_JPRB, 0.36778E+00_JPRB ,&
&  0.38913E+00_JPRB, 0.40268E+00_JPRB, 0.40400E+00_JPRB, 0.40787E+00_JPRB, 0.43308E+00_JPRB, 0.45988E+00_JPRB, 0.43212E+00_JPRB ,&
&  0.40165E+00_JPRB, 0.38243E+00_JPRB, 0.38131E+00_JPRB, 0.43113E+00_JPRB, 0.50312E+00_JPRB, 0.54095E+00_JPRB, 0.41941E+00_JPRB ,&
&  0.33132E+00_JPRB, 0.31580E+00_JPRB, 0.35300E+00_JPRB, 0.49146E+00_JPRB, 0.53563E+00_JPRB, 0.58427E+00_JPRB, 0.65845E+00_JPRB ,&
&  0.60501E+00_JPRB, 0.60452E+00_JPRB, 0.62601E+00_JPRB, 0.77542E+00_JPRB, 0.82774E+00_JPRB, 0.86806E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.91021E+00_JPRB, 0.96712E+00_JPRB, 0.11371E+01_JPRB, 0.96827E+00_JPRB, 0.10524E+01_JPRB, 0.12685E+01_JPRB, 0.12169E+01_JPRB ,&
&  0.12716E+01_JPRB, 0.13062E+01_JPRB, 0.14708E+01_JPRB, 0.13727E+01_JPRB, 0.12209E+01_JPRB, 0.11351E+01_JPRB, 0.11446E+01_JPRB ,&
&  0.13075E+01_JPRB, 0.14183E+01_JPRB, 0.13475E+01_JPRB, 0.12546E+01_JPRB, 0.12030E+01_JPRB, 0.11318E+01_JPRB, 0.10065E+01_JPRB ,&
&  0.83900E+00_JPRB, 0.71068E+00_JPRB, 0.59317E+00_JPRB, 0.51271E+00_JPRB, 0.48794E+00_JPRB, 0.49190E+00_JPRB, 0.49161E+00_JPRB ,&
&  0.50414E+00_JPRB, 0.51118E+00_JPRB, 0.50415E+00_JPRB, 0.45286E+00_JPRB, 0.40810E+00_JPRB, 0.37245E+00_JPRB, 0.34783E+00_JPRB ,&
&  0.33412E+00_JPRB, 0.31299E+00_JPRB, 0.30034E+00_JPRB, 0.31006E+00_JPRB, 0.34674E+00_JPRB, 0.38727E+00_JPRB, 0.40417E+00_JPRB ,&
&  0.39185E+00_JPRB, 0.38642E+00_JPRB, 0.40757E+00_JPRB, 0.43923E+00_JPRB, 0.45262E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.46577E+00_JPRB, 0.46634E+00_JPRB, 0.45397E+00_JPRB, 0.42623E+00_JPRB, 0.40253E+00_JPRB, 0.37505E+00_JPRB, 0.35744E+00_JPRB ,&
&  0.34972E+00_JPRB, 0.34710E+00_JPRB, 0.34785E+00_JPRB, 0.35139E+00_JPRB, 0.35696E+00_JPRB, 0.36598E+00_JPRB, 0.36974E+00_JPRB ,&
&  0.37451E+00_JPRB, 0.37713E+00_JPRB, 0.38227E+00_JPRB, 0.39067E+00_JPRB, 0.40120E+00_JPRB, 0.41314E+00_JPRB, 0.42476E+00_JPRB ,&
&  0.43422E+00_JPRB, 0.42925E+00_JPRB, 0.40978E+00_JPRB, 0.39832E+00_JPRB, 0.40564E+00_JPRB, 0.42363E+00_JPRB, 0.40942E+00_JPRB ,&
&  0.37201E+00_JPRB, 0.34941E+00_JPRB, 0.35202E+00_JPRB, 0.41629E+00_JPRB, 0.45525E+00_JPRB, 0.47839E+00_JPRB, 0.36367E+00_JPRB ,&
&  0.30111E+00_JPRB, 0.27442E+00_JPRB, 0.27218E+00_JPRB, 0.29559E+00_JPRB, 0.34843E+00_JPRB, 0.43162E+00_JPRB, 0.49284E+00_JPRB ,&
&  0.53871E+00_JPRB, 0.58038E+00_JPRB, 0.59278E+00_JPRB, 0.61687E+00_JPRB, 0.65820E+00_JPRB, 0.69932E+00_JPRB, 0.73985E+00_JPRB ,&
&  0.77117E+00_JPRB, 0.83151E+00_JPRB, 0.97672E+00_JPRB, 0.87660E+00_JPRB, 0.91093E+00_JPRB, 0.95428E+00_JPRB, 0.10514E+01_JPRB ,&
&  0.12072E+01_JPRB, 0.12442E+01_JPRB, 0.13224E+01_JPRB, 0.14394E+01_JPRB, 0.13288E+01_JPRB, 0.12236E+01_JPRB, 0.11763E+01_JPRB ,&
&  0.12819E+01_JPRB, 0.13726E+01_JPRB, 0.13060E+01_JPRB, 0.11574E+01_JPRB, 0.10925E+01_JPRB, 0.10505E+01_JPRB, 0.92834E+00_JPRB ,&
&  0.78268E+00_JPRB, 0.69466E+00_JPRB, 0.59143E+00_JPRB, 0.54171E+00_JPRB, 0.54019E+00_JPRB, 0.54283E+00_JPRB, 0.54614E+00_JPRB ,&
&  0.55068E+00_JPRB, 0.54772E+00_JPRB, 0.53742E+00_JPRB, 0.48978E+00_JPRB, 0.44389E+00_JPRB, 0.40096E+00_JPRB, 0.37277E+00_JPRB ,&
&  0.35651E+00_JPRB, 0.35164E+00_JPRB, 0.35471E+00_JPRB, 0.36876E+00_JPRB, 0.39718E+00_JPRB, 0.42241E+00_JPRB, 0.43999E+00_JPRB ,&
&  0.44152E+00_JPRB, 0.42985E+00_JPRB, 0.42639E+00_JPRB, 0.42816E+00_JPRB, 0.45033E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.52730E+00_JPRB, 0.53918E+00_JPRB, 0.53237E+00_JPRB, 0.50853E+00_JPRB, 0.47732E+00_JPRB, 0.44286E+00_JPRB, 0.42458E+00_JPRB ,&
&  0.41869E+00_JPRB, 0.42128E+00_JPRB, 0.41912E+00_JPRB, 0.41638E+00_JPRB, 0.41722E+00_JPRB, 0.41952E+00_JPRB, 0.42302E+00_JPRB ,&
&  0.42901E+00_JPRB, 0.43836E+00_JPRB, 0.44711E+00_JPRB, 0.44829E+00_JPRB, 0.45429E+00_JPRB, 0.45953E+00_JPRB, 0.46445E+00_JPRB ,&
&  0.46082E+00_JPRB, 0.43969E+00_JPRB, 0.41243E+00_JPRB, 0.39794E+00_JPRB, 0.38329E+00_JPRB, 0.38270E+00_JPRB, 0.37153E+00_JPRB ,&
&  0.34583E+00_JPRB, 0.34486E+00_JPRB, 0.37855E+00_JPRB, 0.40868E+00_JPRB, 0.40681E+00_JPRB, 0.43226E+00_JPRB, 0.31869E+00_JPRB ,&
&  0.27929E+00_JPRB, 0.25886E+00_JPRB, 0.25374E+00_JPRB, 0.26109E+00_JPRB, 0.28310E+00_JPRB, 0.31478E+00_JPRB, 0.35368E+00_JPRB ,&
&  0.42082E+00_JPRB, 0.47793E+00_JPRB, 0.51342E+00_JPRB, 0.54891E+00_JPRB, 0.58583E+00_JPRB, 0.61933E+00_JPRB, 0.64030E+00_JPRB ,&
&  0.65673E+00_JPRB, 0.69503E+00_JPRB, 0.76256E+00_JPRB, 0.85686E+00_JPRB, 0.10539E+01_JPRB, 0.10248E+01_JPRB, 0.10531E+01_JPRB ,&
&  0.13284E+01_JPRB, 0.14965E+01_JPRB, 0.13985E+01_JPRB, 0.13012E+01_JPRB, 0.12727E+01_JPRB, 0.12936E+01_JPRB, 0.13021E+01_JPRB ,&
&  0.13638E+01_JPRB, 0.13987E+01_JPRB, 0.13464E+01_JPRB, 0.11892E+01_JPRB, 0.11021E+01_JPRB, 0.10430E+01_JPRB, 0.93117E+00_JPRB ,&
&  0.80501E+00_JPRB, 0.75041E+00_JPRB, 0.65596E+00_JPRB, 0.61350E+00_JPRB, 0.59356E+00_JPRB, 0.58716E+00_JPRB, 0.58477E+00_JPRB ,&
&  0.57842E+00_JPRB, 0.56735E+00_JPRB, 0.54499E+00_JPRB, 0.51953E+00_JPRB, 0.49297E+00_JPRB, 0.46947E+00_JPRB, 0.45125E+00_JPRB ,&
&  0.44077E+00_JPRB, 0.43327E+00_JPRB, 0.43089E+00_JPRB, 0.42956E+00_JPRB, 0.42488E+00_JPRB, 0.42474E+00_JPRB, 0.43354E+00_JPRB ,&
&  0.44732E+00_JPRB, 0.45751E+00_JPRB, 0.47233E+00_JPRB, 0.48653E+00_JPRB, 0.50543E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.53020E+00_JPRB, 0.54505E+00_JPRB, 0.55472E+00_JPRB, 0.55397E+00_JPRB, 0.54223E+00_JPRB, 0.52652E+00_JPRB, 0.51453E+00_JPRB ,&
&  0.50566E+00_JPRB, 0.49787E+00_JPRB, 0.48868E+00_JPRB, 0.48184E+00_JPRB, 0.47652E+00_JPRB, 0.47482E+00_JPRB, 0.47412E+00_JPRB ,&
&  0.47567E+00_JPRB, 0.46898E+00_JPRB, 0.46765E+00_JPRB, 0.47146E+00_JPRB, 0.47744E+00_JPRB, 0.47737E+00_JPRB, 0.47341E+00_JPRB ,&
&  0.45358E+00_JPRB, 0.42652E+00_JPRB, 0.40165E+00_JPRB, 0.38236E+00_JPRB, 0.36873E+00_JPRB, 0.36282E+00_JPRB, 0.36855E+00_JPRB ,&
&  0.38012E+00_JPRB, 0.38178E+00_JPRB, 0.38522E+00_JPRB, 0.40557E+00_JPRB, 0.42047E+00_JPRB, 0.36335E+00_JPRB, 0.28218E+00_JPRB ,&
&  0.26651E+00_JPRB, 0.25269E+00_JPRB, 0.25161E+00_JPRB, 0.25615E+00_JPRB, 0.26767E+00_JPRB, 0.28746E+00_JPRB, 0.32124E+00_JPRB ,&
&  0.37234E+00_JPRB, 0.48010E+00_JPRB, 0.53557E+00_JPRB, 0.57615E+00_JPRB, 0.61088E+00_JPRB, 0.63909E+00_JPRB, 0.64837E+00_JPRB ,&
&  0.65393E+00_JPRB, 0.67173E+00_JPRB, 0.71006E+00_JPRB, 0.75604E+00_JPRB, 0.82881E+00_JPRB, 0.97402E+00_JPRB, 0.10908E+01_JPRB ,&
&  0.12535E+01_JPRB, 0.13146E+01_JPRB, 0.12726E+01_JPRB, 0.11745E+01_JPRB, 0.11852E+01_JPRB, 0.12397E+01_JPRB, 0.12460E+01_JPRB ,&
&  0.12740E+01_JPRB, 0.12999E+01_JPRB, 0.12437E+01_JPRB, 0.11531E+01_JPRB, 0.10958E+01_JPRB, 0.10479E+01_JPRB, 0.95040E+00_JPRB ,&
&  0.84851E+00_JPRB, 0.77552E+00_JPRB, 0.70101E+00_JPRB, 0.67027E+00_JPRB, 0.64822E+00_JPRB, 0.61683E+00_JPRB, 0.59403E+00_JPRB ,&
&  0.57743E+00_JPRB, 0.56377E+00_JPRB, 0.53858E+00_JPRB, 0.51779E+00_JPRB, 0.50750E+00_JPRB, 0.50060E+00_JPRB, 0.49370E+00_JPRB ,&
&  0.48832E+00_JPRB, 0.48106E+00_JPRB, 0.47261E+00_JPRB, 0.46449E+00_JPRB, 0.45291E+00_JPRB, 0.44814E+00_JPRB, 0.45181E+00_JPRB ,&
&  0.46100E+00_JPRB, 0.47546E+00_JPRB, 0.49058E+00_JPRB, 0.50366E+00_JPRB, 0.51790E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.55453E+00_JPRB, 0.56337E+00_JPRB, 0.56712E+00_JPRB, 0.56222E+00_JPRB, 0.55154E+00_JPRB, 0.53759E+00_JPRB, 0.52467E+00_JPRB ,&
&  0.51404E+00_JPRB, 0.50630E+00_JPRB, 0.49792E+00_JPRB, 0.49248E+00_JPRB, 0.48554E+00_JPRB, 0.48229E+00_JPRB, 0.47783E+00_JPRB ,&
&  0.47733E+00_JPRB, 0.47853E+00_JPRB, 0.47441E+00_JPRB, 0.46878E+00_JPRB, 0.45460E+00_JPRB, 0.44096E+00_JPRB, 0.43026E+00_JPRB ,&
&  0.42013E+00_JPRB, 0.41249E+00_JPRB, 0.39585E+00_JPRB, 0.38270E+00_JPRB, 0.37736E+00_JPRB, 0.37621E+00_JPRB, 0.38613E+00_JPRB ,&
&  0.38941E+00_JPRB, 0.39865E+00_JPRB, 0.40699E+00_JPRB, 0.40266E+00_JPRB, 0.39744E+00_JPRB, 0.30534E+00_JPRB, 0.27746E+00_JPRB ,&
&  0.26858E+00_JPRB, 0.26224E+00_JPRB, 0.26092E+00_JPRB, 0.26609E+00_JPRB, 0.27559E+00_JPRB, 0.29990E+00_JPRB, 0.32316E+00_JPRB ,&
&  0.34811E+00_JPRB, 0.48825E+00_JPRB, 0.54602E+00_JPRB, 0.59966E+00_JPRB, 0.63485E+00_JPRB, 0.66088E+00_JPRB, 0.67654E+00_JPRB ,&
&  0.69182E+00_JPRB, 0.71712E+00_JPRB, 0.73138E+00_JPRB, 0.76188E+00_JPRB, 0.81895E+00_JPRB, 0.92335E+00_JPRB, 0.10098E+01_JPRB ,&
&  0.11057E+01_JPRB, 0.11302E+01_JPRB, 0.11458E+01_JPRB, 0.11266E+01_JPRB, 0.11299E+01_JPRB, 0.11262E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.11108E+01_JPRB, 0.10808E+01_JPRB, 0.10757E+01_JPRB, 0.10504E+01_JPRB, 0.10262E+01_JPRB, 0.99823E+00_JPRB, 0.92471E+00_JPRB ,&
&  0.83522E+00_JPRB, 0.76238E+00_JPRB, 0.70052E+00_JPRB, 0.66333E+00_JPRB, 0.63363E+00_JPRB, 0.61121E+00_JPRB, 0.60326E+00_JPRB ,&
&  0.60445E+00_JPRB, 0.60727E+00_JPRB, 0.59302E+00_JPRB, 0.57474E+00_JPRB, 0.56383E+00_JPRB, 0.55889E+00_JPRB, 0.54203E+00_JPRB ,&
&  0.52751E+00_JPRB, 0.50958E+00_JPRB, 0.49982E+00_JPRB, 0.48870E+00_JPRB, 0.47835E+00_JPRB, 0.47421E+00_JPRB, 0.47486E+00_JPRB ,&
&  0.48284E+00_JPRB, 0.49330E+00_JPRB, 0.50958E+00_JPRB, 0.52643E+00_JPRB, 0.54492E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.58823E+00_JPRB, 0.57654E+00_JPRB, 0.55196E+00_JPRB, 0.51826E+00_JPRB, 0.49497E+00_JPRB, 0.48078E+00_JPRB, 0.47625E+00_JPRB ,&
&  0.47471E+00_JPRB, 0.47511E+00_JPRB, 0.47517E+00_JPRB, 0.47431E+00_JPRB, 0.47111E+00_JPRB, 0.46565E+00_JPRB, 0.46002E+00_JPRB ,&
&  0.45026E+00_JPRB, 0.44098E+00_JPRB, 0.43013E+00_JPRB, 0.42409E+00_JPRB, 0.41854E+00_JPRB, 0.41437E+00_JPRB, 0.41116E+00_JPRB ,&
&  0.40711E+00_JPRB, 0.40225E+00_JPRB, 0.38580E+00_JPRB, 0.37818E+00_JPRB, 0.37182E+00_JPRB, 0.37372E+00_JPRB, 0.37353E+00_JPRB ,&
&  0.39187E+00_JPRB, 0.34920E+00_JPRB, 0.34112E+00_JPRB, 0.32607E+00_JPRB, 0.31370E+00_JPRB, 0.29993E+00_JPRB, 0.28949E+00_JPRB ,&
&  0.28353E+00_JPRB, 0.27859E+00_JPRB, 0.27756E+00_JPRB, 0.28375E+00_JPRB, 0.29185E+00_JPRB, 0.30809E+00_JPRB, 0.32657E+00_JPRB ,&
&  0.36606E+00_JPRB, 0.49443E+00_JPRB, 0.55784E+00_JPRB, 0.60693E+00_JPRB, 0.65268E+00_JPRB, 0.69404E+00_JPRB, 0.71701E+00_JPRB ,&
&  0.73944E+00_JPRB, 0.77095E+00_JPRB, 0.79561E+00_JPRB, 0.80346E+00_JPRB, 0.86944E+00_JPRB, 0.91592E+00_JPRB, 0.96080E+00_JPRB ,&
&  0.99837E+00_JPRB, 0.10224E+01_JPRB, 0.10407E+01_JPRB, 0.10454E+01_JPRB, 0.10510E+01_JPRB, 0.10323E+01_JPRB, 0.10045E+01_JPRB ,&
&  0.98357E+00_JPRB, 0.97317E+00_JPRB, 0.96479E+00_JPRB, 0.93011E+00_JPRB, 0.90427E+00_JPRB, 0.88235E+00_JPRB, 0.84525E+00_JPRB ,&
&  0.80438E+00_JPRB, 0.76459E+00_JPRB, 0.72461E+00_JPRB, 0.68370E+00_JPRB, 0.64889E+00_JPRB, 0.63101E+00_JPRB, 0.63066E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63968E+00_JPRB, 0.63221E+00_JPRB, 0.62003E+00_JPRB, 0.60610E+00_JPRB, 0.59920E+00_JPRB, 0.58596E+00_JPRB ,&
&  0.57108E+00_JPRB, 0.54989E+00_JPRB, 0.53669E+00_JPRB, 0.52414E+00_JPRB, 0.51754E+00_JPRB, 0.51688E+00_JPRB, 0.52067E+00_JPRB ,&
&  0.52754E+00_JPRB, 0.53758E+00_JPRB, 0.55334E+00_JPRB, 0.57015E+00_JPRB, 0.58354E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.54170E+00_JPRB, 0.51057E+00_JPRB, 0.47848E+00_JPRB, 0.46197E+00_JPRB, 0.45987E+00_JPRB, 0.45839E+00_JPRB, 0.45774E+00_JPRB ,&
&  0.45733E+00_JPRB, 0.46033E+00_JPRB, 0.46349E+00_JPRB, 0.46500E+00_JPRB, 0.46379E+00_JPRB, 0.46062E+00_JPRB, 0.45604E+00_JPRB ,&
&  0.44922E+00_JPRB, 0.44179E+00_JPRB, 0.43433E+00_JPRB, 0.42820E+00_JPRB, 0.42149E+00_JPRB, 0.41505E+00_JPRB, 0.40703E+00_JPRB ,&
&  0.39823E+00_JPRB, 0.38981E+00_JPRB, 0.35970E+00_JPRB, 0.35606E+00_JPRB, 0.34905E+00_JPRB, 0.34790E+00_JPRB, 0.35121E+00_JPRB ,&
&  0.35383E+00_JPRB, 0.34931E+00_JPRB, 0.34407E+00_JPRB, 0.33205E+00_JPRB, 0.32110E+00_JPRB, 0.30929E+00_JPRB, 0.30227E+00_JPRB ,&
&  0.29955E+00_JPRB, 0.30107E+00_JPRB, 0.30654E+00_JPRB, 0.31395E+00_JPRB, 0.32430E+00_JPRB, 0.33717E+00_JPRB, 0.35615E+00_JPRB ,&
&  0.40364E+00_JPRB, 0.47239E+00_JPRB, 0.55125E+00_JPRB, 0.59502E+00_JPRB, 0.64426E+00_JPRB, 0.69039E+00_JPRB, 0.72126E+00_JPRB ,&
&  0.74738E+00_JPRB, 0.77547E+00_JPRB, 0.80665E+00_JPRB, 0.84475E+00_JPRB, 0.87841E+00_JPRB, 0.90295E+00_JPRB, 0.93653E+00_JPRB ,&
&  0.96305E+00_JPRB, 0.97814E+00_JPRB, 0.97820E+00_JPRB, 0.97257E+00_JPRB, 0.96290E+00_JPRB, 0.94420E+00_JPRB, 0.92651E+00_JPRB ,&
&  0.91565E+00_JPRB, 0.90736E+00_JPRB, 0.89506E+00_JPRB, 0.86822E+00_JPRB, 0.84999E+00_JPRB, 0.83550E+00_JPRB, 0.81542E+00_JPRB ,&
&  0.79271E+00_JPRB, 0.76794E+00_JPRB, 0.74549E+00_JPRB, 0.71657E+00_JPRB, 0.69771E+00_JPRB, 0.69218E+00_JPRB, 0.69230E+00_JPRB ,&
&  0.69133E+00_JPRB, 0.68941E+00_JPRB, 0.68523E+00_JPRB, 0.67901E+00_JPRB, 0.66522E+00_JPRB, 0.65200E+00_JPRB, 0.63863E+00_JPRB ,&
&  0.62676E+00_JPRB, 0.61264E+00_JPRB, 0.59985E+00_JPRB, 0.58425E+00_JPRB, 0.57483E+00_JPRB, 0.56658E+00_JPRB, 0.56502E+00_JPRB ,&
&  0.56678E+00_JPRB, 0.57141E+00_JPRB, 0.57410E+00_JPRB, 0.57343E+00_JPRB, 0.56249E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.51450E+00_JPRB, 0.48633E+00_JPRB, 0.47161E+00_JPRB, 0.46863E+00_JPRB, 0.46703E+00_JPRB, 0.46558E+00_JPRB, 0.46407E+00_JPRB ,&
&  0.46091E+00_JPRB, 0.45858E+00_JPRB, 0.45861E+00_JPRB, 0.45862E+00_JPRB, 0.45698E+00_JPRB, 0.45503E+00_JPRB, 0.45281E+00_JPRB ,&
&  0.45035E+00_JPRB, 0.44670E+00_JPRB, 0.44323E+00_JPRB, 0.43888E+00_JPRB, 0.43433E+00_JPRB, 0.42862E+00_JPRB, 0.42089E+00_JPRB ,&
&  0.41354E+00_JPRB, 0.40614E+00_JPRB, 0.39645E+00_JPRB, 0.39196E+00_JPRB, 0.38935E+00_JPRB, 0.36875E+00_JPRB, 0.36350E+00_JPRB ,&
&  0.36657E+00_JPRB, 0.36843E+00_JPRB, 0.37062E+00_JPRB, 0.37314E+00_JPRB, 0.37786E+00_JPRB, 0.37351E+00_JPRB, 0.37342E+00_JPRB ,&
&  0.35897E+00_JPRB, 0.36823E+00_JPRB, 0.37553E+00_JPRB, 0.38359E+00_JPRB, 0.39309E+00_JPRB, 0.40845E+00_JPRB, 0.43415E+00_JPRB ,&
&  0.48278E+00_JPRB, 0.50857E+00_JPRB, 0.53601E+00_JPRB, 0.56413E+00_JPRB, 0.59813E+00_JPRB, 0.63002E+00_JPRB, 0.65601E+00_JPRB ,&
&  0.67656E+00_JPRB, 0.69932E+00_JPRB, 0.72688E+00_JPRB, 0.76565E+00_JPRB, 0.79668E+00_JPRB, 0.82471E+00_JPRB, 0.84389E+00_JPRB ,&
&  0.86759E+00_JPRB, 0.88443E+00_JPRB, 0.89567E+00_JPRB, 0.90307E+00_JPRB, 0.91099E+00_JPRB, 0.91361E+00_JPRB, 0.91619E+00_JPRB ,&
&  0.91586E+00_JPRB, 0.91428E+00_JPRB, 0.90607E+00_JPRB, 0.89219E+00_JPRB, 0.88077E+00_JPRB, 0.87129E+00_JPRB, 0.85869E+00_JPRB ,&
&  0.84499E+00_JPRB, 0.83147E+00_JPRB, 0.81642E+00_JPRB, 0.79994E+00_JPRB, 0.78251E+00_JPRB, 0.76742E+00_JPRB, 0.75096E+00_JPRB ,&
&  0.73873E+00_JPRB, 0.73012E+00_JPRB, 0.72351E+00_JPRB, 0.71479E+00_JPRB, 0.70419E+00_JPRB, 0.69421E+00_JPRB, 0.68247E+00_JPRB ,&
&  0.67141E+00_JPRB, 0.66107E+00_JPRB, 0.65283E+00_JPRB, 0.64282E+00_JPRB, 0.63602E+00_JPRB, 0.62611E+00_JPRB, 0.61729E+00_JPRB ,&
&  0.60714E+00_JPRB, 0.59978E+00_JPRB, 0.58618E+00_JPRB, 0.56627E+00_JPRB, 0.53867E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.49585E+00_JPRB, 0.48391E+00_JPRB, 0.47855E+00_JPRB, 0.47586E+00_JPRB, 0.47410E+00_JPRB, 0.47154E+00_JPRB, 0.46851E+00_JPRB ,&
&  0.46592E+00_JPRB, 0.46303E+00_JPRB, 0.46013E+00_JPRB, 0.45914E+00_JPRB, 0.45774E+00_JPRB, 0.45724E+00_JPRB, 0.45671E+00_JPRB ,&
&  0.45637E+00_JPRB, 0.45559E+00_JPRB, 0.45510E+00_JPRB, 0.45403E+00_JPRB, 0.45225E+00_JPRB, 0.44956E+00_JPRB, 0.44736E+00_JPRB ,&
&  0.44588E+00_JPRB, 0.44485E+00_JPRB, 0.44293E+00_JPRB, 0.44134E+00_JPRB, 0.44152E+00_JPRB, 0.44097E+00_JPRB, 0.43991E+00_JPRB ,&
&  0.44109E+00_JPRB, 0.44226E+00_JPRB, 0.44220E+00_JPRB, 0.44120E+00_JPRB, 0.44007E+00_JPRB, 0.44053E+00_JPRB, 0.44348E+00_JPRB ,&
&  0.44733E+00_JPRB, 0.45773E+00_JPRB, 0.46552E+00_JPRB, 0.47308E+00_JPRB, 0.48081E+00_JPRB, 0.49016E+00_JPRB, 0.50124E+00_JPRB ,&
&  0.51248E+00_JPRB, 0.52132E+00_JPRB, 0.53113E+00_JPRB, 0.54464E+00_JPRB, 0.55796E+00_JPRB, 0.57059E+00_JPRB, 0.58469E+00_JPRB ,&
&  0.59612E+00_JPRB, 0.60873E+00_JPRB, 0.62338E+00_JPRB, 0.63932E+00_JPRB, 0.65242E+00_JPRB, 0.67119E+00_JPRB, 0.69493E+00_JPRB ,&
&  0.72593E+00_JPRB, 0.74105E+00_JPRB, 0.75769E+00_JPRB, 0.77410E+00_JPRB, 0.79505E+00_JPRB, 0.80691E+00_JPRB, 0.81738E+00_JPRB ,&
&  0.82460E+00_JPRB, 0.83157E+00_JPRB, 0.82991E+00_JPRB, 0.82666E+00_JPRB, 0.82368E+00_JPRB, 0.81833E+00_JPRB, 0.81233E+00_JPRB ,&
&  0.80661E+00_JPRB, 0.80182E+00_JPRB, 0.79296E+00_JPRB, 0.78510E+00_JPRB, 0.77701E+00_JPRB, 0.76991E+00_JPRB, 0.75519E+00_JPRB ,&
&  0.74218E+00_JPRB, 0.73284E+00_JPRB, 0.72704E+00_JPRB, 0.71797E+00_JPRB, 0.70910E+00_JPRB, 0.70087E+00_JPRB, 0.69285E+00_JPRB ,&
&  0.68463E+00_JPRB, 0.67483E+00_JPRB, 0.66426E+00_JPRB, 0.65273E+00_JPRB, 0.64260E+00_JPRB, 0.62870E+00_JPRB, 0.61275E+00_JPRB ,&
&  0.59401E+00_JPRB, 0.57774E+00_JPRB, 0.55794E+00_JPRB, 0.53427E+00_JPRB, 0.51061E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.51286E+00_JPRB, 0.50950E+00_JPRB, 0.50641E+00_JPRB, 0.50326E+00_JPRB, 0.49980E+00_JPRB, 0.49590E+00_JPRB, 0.49173E+00_JPRB ,&
&  0.48785E+00_JPRB, 0.48468E+00_JPRB, 0.48184E+00_JPRB, 0.47937E+00_JPRB, 0.47719E+00_JPRB, 0.47549E+00_JPRB, 0.47469E+00_JPRB ,&
&  0.47436E+00_JPRB, 0.47394E+00_JPRB, 0.47379E+00_JPRB, 0.47380E+00_JPRB, 0.47359E+00_JPRB, 0.47320E+00_JPRB, 0.47273E+00_JPRB ,&
&  0.47254E+00_JPRB, 0.47239E+00_JPRB, 0.47217E+00_JPRB, 0.47234E+00_JPRB, 0.47286E+00_JPRB, 0.47338E+00_JPRB, 0.47387E+00_JPRB ,&
&  0.47478E+00_JPRB, 0.47592E+00_JPRB, 0.47746E+00_JPRB, 0.47928E+00_JPRB, 0.48290E+00_JPRB, 0.48697E+00_JPRB, 0.49115E+00_JPRB ,&
&  0.49514E+00_JPRB, 0.49924E+00_JPRB, 0.50260E+00_JPRB, 0.50538E+00_JPRB, 0.50785E+00_JPRB, 0.51076E+00_JPRB, 0.51410E+00_JPRB ,&
&  0.51769E+00_JPRB, 0.52170E+00_JPRB, 0.52636E+00_JPRB, 0.53099E+00_JPRB, 0.53534E+00_JPRB, 0.53993E+00_JPRB, 0.54506E+00_JPRB ,&
&  0.55014E+00_JPRB, 0.55535E+00_JPRB, 0.56069E+00_JPRB, 0.56611E+00_JPRB, 0.57129E+00_JPRB, 0.57750E+00_JPRB, 0.58500E+00_JPRB ,&
&  0.59358E+00_JPRB, 0.60027E+00_JPRB, 0.60801E+00_JPRB, 0.61645E+00_JPRB, 0.62913E+00_JPRB, 0.63555E+00_JPRB, 0.64171E+00_JPRB ,&
&  0.64716E+00_JPRB, 0.65591E+00_JPRB, 0.66122E+00_JPRB, 0.66661E+00_JPRB, 0.67147E+00_JPRB, 0.67582E+00_JPRB, 0.67858E+00_JPRB ,&
&  0.68092E+00_JPRB, 0.68286E+00_JPRB, 0.68320E+00_JPRB, 0.68280E+00_JPRB, 0.68157E+00_JPRB, 0.67988E+00_JPRB, 0.67505E+00_JPRB ,&
&  0.67057E+00_JPRB, 0.66618E+00_JPRB, 0.66217E+00_JPRB, 0.65637E+00_JPRB, 0.65052E+00_JPRB, 0.64447E+00_JPRB, 0.63819E+00_JPRB ,&
&  0.63000E+00_JPRB, 0.62158E+00_JPRB, 0.61247E+00_JPRB, 0.60284E+00_JPRB, 0.59360E+00_JPRB, 0.58401E+00_JPRB, 0.57356E+00_JPRB ,&
&  0.56173E+00_JPRB, 0.55077E+00_JPRB, 0.53840E+00_JPRB, 0.52606E+00_JPRB, 0.51690E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB, 0.53243E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB, 0.11921E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.12620E+00_JPRB, 0.12622E+00_JPRB, 0.12617E+00_JPRB, 0.12594E+00_JPRB, 0.12545E+00_JPRB, 0.12534E+00_JPRB, 0.12518E+00_JPRB ,&
&  0.12496E+00_JPRB, 0.12446E+00_JPRB, 0.12445E+00_JPRB, 0.12438E+00_JPRB, 0.12430E+00_JPRB, 0.12421E+00_JPRB, 0.12403E+00_JPRB ,&
&  0.12367E+00_JPRB, 0.12311E+00_JPRB, 0.12144E+00_JPRB, 0.12063E+00_JPRB, 0.12013E+00_JPRB, 0.11988E+00_JPRB, 0.11967E+00_JPRB ,&
&  0.11955E+00_JPRB, 0.11949E+00_JPRB, 0.11943E+00_JPRB, 0.11978E+00_JPRB, 0.11993E+00_JPRB, 0.11999E+00_JPRB, 0.12003E+00_JPRB ,&
&  0.12004E+00_JPRB, 0.12011E+00_JPRB, 0.12012E+00_JPRB, 0.12012E+00_JPRB, 0.12022E+00_JPRB, 0.12021E+00_JPRB, 0.12006E+00_JPRB ,&
&  0.11989E+00_JPRB, 0.11966E+00_JPRB, 0.11945E+00_JPRB, 0.11918E+00_JPRB, 0.11899E+00_JPRB, 0.11883E+00_JPRB, 0.11876E+00_JPRB ,&
&  0.11862E+00_JPRB, 0.11851E+00_JPRB, 0.11840E+00_JPRB, 0.11833E+00_JPRB, 0.11824E+00_JPRB, 0.11818E+00_JPRB, 0.11808E+00_JPRB ,&
&  0.11813E+00_JPRB, 0.11813E+00_JPRB, 0.11816E+00_JPRB, 0.11815E+00_JPRB, 0.11814E+00_JPRB, 0.11809E+00_JPRB, 0.11800E+00_JPRB ,&
&  0.11784E+00_JPRB, 0.11774E+00_JPRB, 0.11763E+00_JPRB, 0.11755E+00_JPRB, 0.11751E+00_JPRB, 0.11748E+00_JPRB, 0.11747E+00_JPRB ,&
&  0.11746E+00_JPRB, 0.11752E+00_JPRB, 0.11754E+00_JPRB, 0.11757E+00_JPRB, 0.11761E+00_JPRB, 0.11766E+00_JPRB, 0.11770E+00_JPRB ,&
&  0.11776E+00_JPRB, 0.11782E+00_JPRB, 0.11790E+00_JPRB, 0.11802E+00_JPRB, 0.11815E+00_JPRB, 0.11832E+00_JPRB, 0.11855E+00_JPRB ,&
&  0.11880E+00_JPRB, 0.11905E+00_JPRB, 0.11929E+00_JPRB, 0.11954E+00_JPRB, 0.11992E+00_JPRB, 0.12029E+00_JPRB, 0.12072E+00_JPRB ,&
&  0.12116E+00_JPRB, 0.12159E+00_JPRB, 0.12198E+00_JPRB, 0.12239E+00_JPRB, 0.12290E+00_JPRB, 0.12317E+00_JPRB, 0.12340E+00_JPRB ,&
&  0.12375E+00_JPRB, 0.12409E+00_JPRB, 0.12445E+00_JPRB, 0.12477E+00_JPRB, 0.12529E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.13097E+00_JPRB, 0.13138E+00_JPRB, 0.13107E+00_JPRB, 0.13080E+00_JPRB, 0.12614E+00_JPRB, 0.13184E+00_JPRB, 0.13111E+00_JPRB ,&
&  0.13133E+00_JPRB, 0.13077E+00_JPRB, 0.13164E+00_JPRB, 0.13156E+00_JPRB, 0.13113E+00_JPRB, 0.13080E+00_JPRB, 0.13044E+00_JPRB ,&
&  0.13007E+00_JPRB, 0.12937E+00_JPRB, 0.12896E+00_JPRB, 0.12844E+00_JPRB, 0.12794E+00_JPRB, 0.12718E+00_JPRB, 0.12640E+00_JPRB ,&
&  0.12606E+00_JPRB, 0.12615E+00_JPRB, 0.12648E+00_JPRB, 0.12534E+00_JPRB, 0.12490E+00_JPRB, 0.12464E+00_JPRB, 0.12453E+00_JPRB ,&
&  0.12431E+00_JPRB, 0.12430E+00_JPRB, 0.12433E+00_JPRB, 0.12428E+00_JPRB, 0.12523E+00_JPRB, 0.12549E+00_JPRB, 0.12569E+00_JPRB ,&
&  0.12570E+00_JPRB, 0.12552E+00_JPRB, 0.12491E+00_JPRB, 0.12437E+00_JPRB, 0.12382E+00_JPRB, 0.12292E+00_JPRB, 0.12197E+00_JPRB ,&
&  0.12114E+00_JPRB, 0.12050E+00_JPRB, 0.11979E+00_JPRB, 0.11924E+00_JPRB, 0.11878E+00_JPRB, 0.11834E+00_JPRB, 0.11805E+00_JPRB ,&
&  0.11796E+00_JPRB, 0.11792E+00_JPRB, 0.11787E+00_JPRB, 0.11782E+00_JPRB, 0.11773E+00_JPRB, 0.11766E+00_JPRB, 0.11757E+00_JPRB ,&
&  0.11771E+00_JPRB, 0.11763E+00_JPRB, 0.11746E+00_JPRB, 0.11720E+00_JPRB, 0.11705E+00_JPRB, 0.11674E+00_JPRB, 0.11652E+00_JPRB ,&
&  0.11634E+00_JPRB, 0.11612E+00_JPRB, 0.11602E+00_JPRB, 0.11597E+00_JPRB, 0.11595E+00_JPRB, 0.11592E+00_JPRB, 0.11598E+00_JPRB ,&
&  0.11609E+00_JPRB, 0.11624E+00_JPRB, 0.11654E+00_JPRB, 0.11679E+00_JPRB, 0.11702E+00_JPRB, 0.11723E+00_JPRB, 0.11764E+00_JPRB ,&
&  0.11809E+00_JPRB, 0.11853E+00_JPRB, 0.11898E+00_JPRB, 0.11929E+00_JPRB, 0.11992E+00_JPRB, 0.12055E+00_JPRB, 0.12122E+00_JPRB ,&
&  0.12193E+00_JPRB, 0.12281E+00_JPRB, 0.12372E+00_JPRB, 0.12462E+00_JPRB, 0.12579E+00_JPRB, 0.12676E+00_JPRB, 0.12770E+00_JPRB ,&
&  0.12873E+00_JPRB, 0.13062E+00_JPRB, 0.13126E+00_JPRB, 0.13156E+00_JPRB, 0.13140E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.14513E+00_JPRB, 0.14575E+00_JPRB, 0.14604E+00_JPRB, 0.14644E+00_JPRB, 0.14733E+00_JPRB, 0.14736E+00_JPRB, 0.14721E+00_JPRB ,&
&  0.14541E+00_JPRB, 0.14464E+00_JPRB, 0.14345E+00_JPRB, 0.14184E+00_JPRB, 0.14023E+00_JPRB, 0.13772E+00_JPRB, 0.13602E+00_JPRB ,&
&  0.13431E+00_JPRB, 0.13252E+00_JPRB, 0.13129E+00_JPRB, 0.12978E+00_JPRB, 0.12850E+00_JPRB, 0.12710E+00_JPRB, 0.12660E+00_JPRB ,&
&  0.12595E+00_JPRB, 0.12592E+00_JPRB, 0.12661E+00_JPRB, 0.12723E+00_JPRB, 0.12831E+00_JPRB, 0.12947E+00_JPRB, 0.13094E+00_JPRB ,&
&  0.13149E+00_JPRB, 0.13231E+00_JPRB, 0.13304E+00_JPRB, 0.13349E+00_JPRB, 0.13489E+00_JPRB, 0.13591E+00_JPRB, 0.13624E+00_JPRB ,&
&  0.13581E+00_JPRB, 0.13509E+00_JPRB, 0.13363E+00_JPRB, 0.13199E+00_JPRB, 0.13005E+00_JPRB, 0.12786E+00_JPRB, 0.12578E+00_JPRB ,&
&  0.12413E+00_JPRB, 0.12263E+00_JPRB, 0.12148E+00_JPRB, 0.12055E+00_JPRB, 0.11995E+00_JPRB, 0.11948E+00_JPRB, 0.11903E+00_JPRB ,&
&  0.11875E+00_JPRB, 0.11852E+00_JPRB, 0.11818E+00_JPRB, 0.11811E+00_JPRB, 0.11798E+00_JPRB, 0.11798E+00_JPRB, 0.11796E+00_JPRB ,&
&  0.11794E+00_JPRB, 0.11784E+00_JPRB, 0.11768E+00_JPRB, 0.11737E+00_JPRB, 0.11706E+00_JPRB, 0.11674E+00_JPRB, 0.11641E+00_JPRB ,&
&  0.11606E+00_JPRB, 0.11598E+00_JPRB, 0.11571E+00_JPRB, 0.11551E+00_JPRB, 0.11527E+00_JPRB, 0.11491E+00_JPRB, 0.11480E+00_JPRB ,&
&  0.11489E+00_JPRB, 0.11502E+00_JPRB, 0.11495E+00_JPRB, 0.11510E+00_JPRB, 0.11530E+00_JPRB, 0.11558E+00_JPRB, 0.11625E+00_JPRB ,&
&  0.11667E+00_JPRB, 0.11712E+00_JPRB, 0.11755E+00_JPRB, 0.11814E+00_JPRB, 0.11912E+00_JPRB, 0.12010E+00_JPRB, 0.12119E+00_JPRB ,&
&  0.12249E+00_JPRB, 0.12373E+00_JPRB, 0.12496E+00_JPRB, 0.12619E+00_JPRB, 0.12818E+00_JPRB, 0.12968E+00_JPRB, 0.13165E+00_JPRB ,&
&  0.13342E+00_JPRB, 0.13301E+00_JPRB, 0.14036E+00_JPRB, 0.14129E+00_JPRB, 0.14259E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.15095E+00_JPRB, 0.15109E+00_JPRB, 0.15097E+00_JPRB, 0.15181E+00_JPRB, 0.15265E+00_JPRB, 0.15353E+00_JPRB, 0.15342E+00_JPRB ,&
&  0.15324E+00_JPRB, 0.15165E+00_JPRB, 0.15042E+00_JPRB, 0.14767E+00_JPRB, 0.14544E+00_JPRB, 0.14286E+00_JPRB, 0.14062E+00_JPRB ,&
&  0.13863E+00_JPRB, 0.13676E+00_JPRB, 0.13504E+00_JPRB, 0.13358E+00_JPRB, 0.13227E+00_JPRB, 0.13116E+00_JPRB, 0.13014E+00_JPRB ,&
&  0.12857E+00_JPRB, 0.12817E+00_JPRB, 0.12824E+00_JPRB, 0.12937E+00_JPRB, 0.13157E+00_JPRB, 0.13315E+00_JPRB, 0.13502E+00_JPRB ,&
&  0.13742E+00_JPRB, 0.13717E+00_JPRB, 0.13970E+00_JPRB, 0.14216E+00_JPRB, 0.14417E+00_JPRB, 0.14570E+00_JPRB, 0.14599E+00_JPRB ,&
&  0.14660E+00_JPRB, 0.14301E+00_JPRB, 0.14437E+00_JPRB, 0.14124E+00_JPRB, 0.13713E+00_JPRB, 0.13498E+00_JPRB, 0.13128E+00_JPRB ,&
&  0.12928E+00_JPRB, 0.12643E+00_JPRB, 0.12456E+00_JPRB, 0.12306E+00_JPRB, 0.12142E+00_JPRB, 0.12098E+00_JPRB, 0.12026E+00_JPRB ,&
&  0.11981E+00_JPRB, 0.11980E+00_JPRB, 0.11916E+00_JPRB, 0.11891E+00_JPRB, 0.11847E+00_JPRB, 0.11805E+00_JPRB, 0.11825E+00_JPRB ,&
&  0.11861E+00_JPRB, 0.11911E+00_JPRB, 0.11963E+00_JPRB, 0.11970E+00_JPRB, 0.11965E+00_JPRB, 0.11963E+00_JPRB, 0.11956E+00_JPRB ,&
&  0.11974E+00_JPRB, 0.11974E+00_JPRB, 0.11938E+00_JPRB, 0.11898E+00_JPRB, 0.11810E+00_JPRB, 0.11725E+00_JPRB, 0.11641E+00_JPRB ,&
&  0.11569E+00_JPRB, 0.11530E+00_JPRB, 0.11497E+00_JPRB, 0.11491E+00_JPRB, 0.11497E+00_JPRB, 0.11527E+00_JPRB, 0.11558E+00_JPRB ,&
&  0.11586E+00_JPRB, 0.11647E+00_JPRB, 0.11692E+00_JPRB, 0.11766E+00_JPRB, 0.11854E+00_JPRB, 0.11934E+00_JPRB, 0.12056E+00_JPRB ,&
&  0.12187E+00_JPRB, 0.12331E+00_JPRB, 0.12533E+00_JPRB, 0.12714E+00_JPRB, 0.12940E+00_JPRB, 0.13240E+00_JPRB, 0.13470E+00_JPRB ,&
&  0.13724E+00_JPRB, 0.14005E+00_JPRB, 0.14074E+00_JPRB, 0.14682E+00_JPRB, 0.15023E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.14971E+00_JPRB, 0.15090E+00_JPRB, 0.15230E+00_JPRB, 0.15365E+00_JPRB, 0.15488E+00_JPRB, 0.15610E+00_JPRB, 0.15691E+00_JPRB ,&
&  0.15481E+00_JPRB, 0.15346E+00_JPRB, 0.15101E+00_JPRB, 0.14866E+00_JPRB, 0.14604E+00_JPRB, 0.14325E+00_JPRB, 0.14218E+00_JPRB ,&
&  0.14088E+00_JPRB, 0.14058E+00_JPRB, 0.13959E+00_JPRB, 0.13918E+00_JPRB, 0.13841E+00_JPRB, 0.13801E+00_JPRB, 0.13733E+00_JPRB ,&
&  0.13580E+00_JPRB, 0.13497E+00_JPRB, 0.13417E+00_JPRB, 0.13457E+00_JPRB, 0.13590E+00_JPRB, 0.13738E+00_JPRB, 0.13896E+00_JPRB ,&
&  0.14121E+00_JPRB, 0.14239E+00_JPRB, 0.14455E+00_JPRB, 0.14613E+00_JPRB, 0.14854E+00_JPRB, 0.15053E+00_JPRB, 0.15087E+00_JPRB ,&
&  0.15092E+00_JPRB, 0.14976E+00_JPRB, 0.14959E+00_JPRB, 0.14806E+00_JPRB, 0.14275E+00_JPRB, 0.13955E+00_JPRB, 0.13518E+00_JPRB ,&
&  0.13030E+00_JPRB, 0.12662E+00_JPRB, 0.12412E+00_JPRB, 0.12323E+00_JPRB, 0.12245E+00_JPRB, 0.12217E+00_JPRB, 0.12191E+00_JPRB ,&
&  0.12122E+00_JPRB, 0.12043E+00_JPRB, 0.11952E+00_JPRB, 0.11897E+00_JPRB, 0.11922E+00_JPRB, 0.11980E+00_JPRB, 0.12104E+00_JPRB ,&
&  0.12231E+00_JPRB, 0.12301E+00_JPRB, 0.12379E+00_JPRB, 0.12406E+00_JPRB, 0.12418E+00_JPRB, 0.12437E+00_JPRB, 0.12444E+00_JPRB ,&
&  0.12496E+00_JPRB, 0.12510E+00_JPRB, 0.12466E+00_JPRB, 0.12392E+00_JPRB, 0.12214E+00_JPRB, 0.11983E+00_JPRB, 0.11795E+00_JPRB ,&
&  0.11638E+00_JPRB, 0.11536E+00_JPRB, 0.11499E+00_JPRB, 0.11465E+00_JPRB, 0.11487E+00_JPRB, 0.11520E+00_JPRB, 0.11551E+00_JPRB ,&
&  0.11604E+00_JPRB, 0.11650E+00_JPRB, 0.11695E+00_JPRB, 0.11790E+00_JPRB, 0.11869E+00_JPRB, 0.11983E+00_JPRB, 0.12102E+00_JPRB ,&
&  0.12185E+00_JPRB, 0.12313E+00_JPRB, 0.12419E+00_JPRB, 0.12538E+00_JPRB, 0.12885E+00_JPRB, 0.13172E+00_JPRB, 0.13751E+00_JPRB ,&
&  0.14158E+00_JPRB, 0.14400E+00_JPRB, 0.14590E+00_JPRB, 0.15008E+00_JPRB, 0.14962E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.14982E+00_JPRB, 0.15146E+00_JPRB, 0.15340E+00_JPRB, 0.15486E+00_JPRB, 0.15625E+00_JPRB, 0.15743E+00_JPRB, 0.15807E+00_JPRB ,&
&  0.15820E+00_JPRB, 0.15768E+00_JPRB, 0.15774E+00_JPRB, 0.15679E+00_JPRB, 0.15588E+00_JPRB, 0.15182E+00_JPRB, 0.15104E+00_JPRB ,&
&  0.15000E+00_JPRB, 0.15038E+00_JPRB, 0.14999E+00_JPRB, 0.15016E+00_JPRB, 0.14961E+00_JPRB, 0.14935E+00_JPRB, 0.14819E+00_JPRB ,&
&  0.14644E+00_JPRB, 0.14502E+00_JPRB, 0.14359E+00_JPRB, 0.14306E+00_JPRB, 0.14370E+00_JPRB, 0.14424E+00_JPRB, 0.14407E+00_JPRB ,&
&  0.14411E+00_JPRB, 0.14446E+00_JPRB, 0.14495E+00_JPRB, 0.14519E+00_JPRB, 0.15005E+00_JPRB, 0.15200E+00_JPRB, 0.15301E+00_JPRB ,&
&  0.15306E+00_JPRB, 0.15361E+00_JPRB, 0.15387E+00_JPRB, 0.15329E+00_JPRB, 0.15267E+00_JPRB, 0.14947E+00_JPRB, 0.14724E+00_JPRB ,&
&  0.13971E+00_JPRB, 0.13436E+00_JPRB, 0.13028E+00_JPRB, 0.12776E+00_JPRB, 0.12670E+00_JPRB, 0.12650E+00_JPRB, 0.12592E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12333E+00_JPRB, 0.12258E+00_JPRB, 0.12311E+00_JPRB, 0.12396E+00_JPRB, 0.12494E+00_JPRB, 0.12609E+00_JPRB ,&
&  0.12714E+00_JPRB, 0.12741E+00_JPRB, 0.12743E+00_JPRB, 0.12776E+00_JPRB, 0.12801E+00_JPRB, 0.12840E+00_JPRB, 0.12871E+00_JPRB ,&
&  0.12953E+00_JPRB, 0.12900E+00_JPRB, 0.12948E+00_JPRB, 0.13028E+00_JPRB, 0.12836E+00_JPRB, 0.12525E+00_JPRB, 0.12212E+00_JPRB ,&
&  0.11930E+00_JPRB, 0.11770E+00_JPRB, 0.11646E+00_JPRB, 0.11570E+00_JPRB, 0.11556E+00_JPRB, 0.11572E+00_JPRB, 0.11623E+00_JPRB ,&
&  0.11687E+00_JPRB, 0.11772E+00_JPRB, 0.11831E+00_JPRB, 0.11901E+00_JPRB, 0.11985E+00_JPRB, 0.12070E+00_JPRB, 0.12171E+00_JPRB ,&
&  0.12294E+00_JPRB, 0.12413E+00_JPRB, 0.12496E+00_JPRB, 0.12618E+00_JPRB, 0.12849E+00_JPRB, 0.13168E+00_JPRB, 0.13658E+00_JPRB ,&
&  0.14232E+00_JPRB, 0.14777E+00_JPRB, 0.14647E+00_JPRB, 0.14790E+00_JPRB, 0.14917E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.15506E+00_JPRB, 0.15721E+00_JPRB, 0.15885E+00_JPRB, 0.16033E+00_JPRB, 0.16146E+00_JPRB, 0.16272E+00_JPRB, 0.16328E+00_JPRB ,&
&  0.16379E+00_JPRB, 0.16378E+00_JPRB, 0.16413E+00_JPRB, 0.16404E+00_JPRB, 0.16414E+00_JPRB, 0.16315E+00_JPRB, 0.16253E+00_JPRB ,&
&  0.16210E+00_JPRB, 0.16228E+00_JPRB, 0.16215E+00_JPRB, 0.16189E+00_JPRB, 0.16114E+00_JPRB, 0.16020E+00_JPRB, 0.15882E+00_JPRB ,&
&  0.15612E+00_JPRB, 0.15450E+00_JPRB, 0.15491E+00_JPRB, 0.15464E+00_JPRB, 0.15508E+00_JPRB, 0.15522E+00_JPRB, 0.15569E+00_JPRB ,&
&  0.15225E+00_JPRB, 0.15078E+00_JPRB, 0.15048E+00_JPRB, 0.14977E+00_JPRB, 0.15251E+00_JPRB, 0.15368E+00_JPRB, 0.15532E+00_JPRB ,&
&  0.15594E+00_JPRB, 0.15645E+00_JPRB, 0.15714E+00_JPRB, 0.15687E+00_JPRB, 0.15724E+00_JPRB, 0.15762E+00_JPRB, 0.15658E+00_JPRB ,&
&  0.15405E+00_JPRB, 0.15189E+00_JPRB, 0.14899E+00_JPRB, 0.14168E+00_JPRB, 0.13940E+00_JPRB, 0.13924E+00_JPRB, 0.13769E+00_JPRB ,&
&  0.13516E+00_JPRB, 0.13510E+00_JPRB, 0.13472E+00_JPRB, 0.13355E+00_JPRB, 0.13496E+00_JPRB, 0.13705E+00_JPRB, 0.13725E+00_JPRB ,&
&  0.13637E+00_JPRB, 0.13618E+00_JPRB, 0.13566E+00_JPRB, 0.13462E+00_JPRB, 0.13338E+00_JPRB, 0.13283E+00_JPRB, 0.13235E+00_JPRB ,&
&  0.13215E+00_JPRB, 0.13129E+00_JPRB, 0.13438E+00_JPRB, 0.13545E+00_JPRB, 0.13143E+00_JPRB, 0.13204E+00_JPRB, 0.12975E+00_JPRB ,&
&  0.12436E+00_JPRB, 0.12184E+00_JPRB, 0.12055E+00_JPRB, 0.11951E+00_JPRB, 0.11939E+00_JPRB, 0.11963E+00_JPRB, 0.11968E+00_JPRB ,&
&  0.12050E+00_JPRB, 0.12148E+00_JPRB, 0.12199E+00_JPRB, 0.12257E+00_JPRB, 0.12333E+00_JPRB, 0.12440E+00_JPRB, 0.12578E+00_JPRB ,&
&  0.12695E+00_JPRB, 0.12862E+00_JPRB, 0.12954E+00_JPRB, 0.13103E+00_JPRB, 0.13216E+00_JPRB, 0.13349E+00_JPRB, 0.13656E+00_JPRB ,&
&  0.13930E+00_JPRB, 0.14460E+00_JPRB, 0.15204E+00_JPRB, 0.15502E+00_JPRB, 0.15313E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.16017E+00_JPRB, 0.16224E+00_JPRB, 0.16484E+00_JPRB, 0.16752E+00_JPRB, 0.16952E+00_JPRB, 0.17150E+00_JPRB, 0.17323E+00_JPRB ,&
&  0.17478E+00_JPRB, 0.17582E+00_JPRB, 0.17646E+00_JPRB, 0.17675E+00_JPRB, 0.17691E+00_JPRB, 0.17654E+00_JPRB, 0.17573E+00_JPRB ,&
&  0.17489E+00_JPRB, 0.17348E+00_JPRB, 0.17174E+00_JPRB, 0.17028E+00_JPRB, 0.16929E+00_JPRB, 0.16815E+00_JPRB, 0.16655E+00_JPRB ,&
&  0.16485E+00_JPRB, 0.16338E+00_JPRB, 0.16238E+00_JPRB, 0.16161E+00_JPRB, 0.16176E+00_JPRB, 0.16238E+00_JPRB, 0.16290E+00_JPRB ,&
&  0.16013E+00_JPRB, 0.15527E+00_JPRB, 0.15524E+00_JPRB, 0.15622E+00_JPRB, 0.15601E+00_JPRB, 0.15734E+00_JPRB, 0.15814E+00_JPRB ,&
&  0.15931E+00_JPRB, 0.16074E+00_JPRB, 0.16200E+00_JPRB, 0.16267E+00_JPRB, 0.16313E+00_JPRB, 0.16374E+00_JPRB, 0.16309E+00_JPRB ,&
&  0.16119E+00_JPRB, 0.15953E+00_JPRB, 0.15803E+00_JPRB, 0.15656E+00_JPRB, 0.15618E+00_JPRB, 0.15733E+00_JPRB, 0.15795E+00_JPRB ,&
&  0.15709E+00_JPRB, 0.15588E+00_JPRB, 0.15476E+00_JPRB, 0.15486E+00_JPRB, 0.15546E+00_JPRB, 0.15576E+00_JPRB, 0.15416E+00_JPRB ,&
&  0.15259E+00_JPRB, 0.15014E+00_JPRB, 0.14774E+00_JPRB, 0.14544E+00_JPRB, 0.14408E+00_JPRB, 0.14111E+00_JPRB, 0.13805E+00_JPRB ,&
&  0.13729E+00_JPRB, 0.14046E+00_JPRB, 0.14578E+00_JPRB, 0.14865E+00_JPRB, 0.14259E+00_JPRB, 0.14074E+00_JPRB, 0.13964E+00_JPRB ,&
&  0.13540E+00_JPRB, 0.13106E+00_JPRB, 0.12806E+00_JPRB, 0.12681E+00_JPRB, 0.12638E+00_JPRB, 0.12606E+00_JPRB, 0.12654E+00_JPRB ,&
&  0.12742E+00_JPRB, 0.12804E+00_JPRB, 0.12868E+00_JPRB, 0.12969E+00_JPRB, 0.13065E+00_JPRB, 0.13159E+00_JPRB, 0.13238E+00_JPRB ,&
&  0.13401E+00_JPRB, 0.13602E+00_JPRB, 0.13786E+00_JPRB, 0.14041E+00_JPRB, 0.14314E+00_JPRB, 0.14595E+00_JPRB, 0.14895E+00_JPRB ,&
&  0.15230E+00_JPRB, 0.15474E+00_JPRB, 0.15601E+00_JPRB, 0.15813E+00_JPRB, 0.15910E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.17147E+00_JPRB, 0.17625E+00_JPRB, 0.18054E+00_JPRB, 0.18296E+00_JPRB, 0.18470E+00_JPRB, 0.18718E+00_JPRB, 0.18969E+00_JPRB ,&
&  0.19139E+00_JPRB, 0.19226E+00_JPRB, 0.19256E+00_JPRB, 0.19232E+00_JPRB, 0.19133E+00_JPRB, 0.18979E+00_JPRB, 0.18808E+00_JPRB ,&
&  0.18622E+00_JPRB, 0.18368E+00_JPRB, 0.18136E+00_JPRB, 0.17957E+00_JPRB, 0.17802E+00_JPRB, 0.17620E+00_JPRB, 0.17473E+00_JPRB ,&
&  0.17309E+00_JPRB, 0.17232E+00_JPRB, 0.17158E+00_JPRB, 0.17107E+00_JPRB, 0.17080E+00_JPRB, 0.17019E+00_JPRB, 0.16963E+00_JPRB ,&
&  0.16808E+00_JPRB, 0.16537E+00_JPRB, 0.16375E+00_JPRB, 0.16338E+00_JPRB, 0.16355E+00_JPRB, 0.16317E+00_JPRB, 0.16269E+00_JPRB ,&
&  0.16438E+00_JPRB, 0.16711E+00_JPRB, 0.16896E+00_JPRB, 0.16997E+00_JPRB, 0.17062E+00_JPRB, 0.17144E+00_JPRB, 0.17252E+00_JPRB ,&
&  0.17044E+00_JPRB, 0.16800E+00_JPRB, 0.16623E+00_JPRB, 0.16526E+00_JPRB, 0.16503E+00_JPRB, 0.16560E+00_JPRB, 0.16688E+00_JPRB ,&
&  0.16804E+00_JPRB, 0.16864E+00_JPRB, 0.16825E+00_JPRB, 0.16710E+00_JPRB, 0.16576E+00_JPRB, 0.16513E+00_JPRB, 0.16471E+00_JPRB ,&
&  0.16452E+00_JPRB, 0.16435E+00_JPRB, 0.16409E+00_JPRB, 0.16336E+00_JPRB, 0.15921E+00_JPRB, 0.15513E+00_JPRB, 0.15253E+00_JPRB ,&
&  0.15293E+00_JPRB, 0.15383E+00_JPRB, 0.15431E+00_JPRB, 0.15821E+00_JPRB, 0.16256E+00_JPRB, 0.15905E+00_JPRB, 0.15440E+00_JPRB ,&
&  0.14827E+00_JPRB, 0.14727E+00_JPRB, 0.14463E+00_JPRB, 0.14077E+00_JPRB, 0.13942E+00_JPRB, 0.13929E+00_JPRB, 0.13910E+00_JPRB ,&
&  0.13983E+00_JPRB, 0.14034E+00_JPRB, 0.14153E+00_JPRB, 0.14217E+00_JPRB, 0.14248E+00_JPRB, 0.14300E+00_JPRB, 0.14357E+00_JPRB ,&
&  0.14496E+00_JPRB, 0.14795E+00_JPRB, 0.15192E+00_JPRB, 0.15424E+00_JPRB, 0.15638E+00_JPRB, 0.15853E+00_JPRB, 0.16003E+00_JPRB ,&
&  0.16121E+00_JPRB, 0.16281E+00_JPRB, 0.16435E+00_JPRB, 0.16555E+00_JPRB, 0.16770E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.18780E+00_JPRB, 0.19195E+00_JPRB, 0.19442E+00_JPRB, 0.19683E+00_JPRB, 0.19959E+00_JPRB, 0.20404E+00_JPRB, 0.20713E+00_JPRB ,&
&  0.20773E+00_JPRB, 0.20761E+00_JPRB, 0.20706E+00_JPRB, 0.20589E+00_JPRB, 0.20414E+00_JPRB, 0.20258E+00_JPRB, 0.20000E+00_JPRB ,&
&  0.19683E+00_JPRB, 0.19396E+00_JPRB, 0.19139E+00_JPRB, 0.18893E+00_JPRB, 0.18666E+00_JPRB, 0.18458E+00_JPRB, 0.18287E+00_JPRB ,&
&  0.18217E+00_JPRB, 0.18163E+00_JPRB, 0.18056E+00_JPRB, 0.18023E+00_JPRB, 0.18040E+00_JPRB, 0.18013E+00_JPRB, 0.17917E+00_JPRB ,&
&  0.17795E+00_JPRB, 0.17566E+00_JPRB, 0.17329E+00_JPRB, 0.17219E+00_JPRB, 0.17251E+00_JPRB, 0.17197E+00_JPRB, 0.17120E+00_JPRB ,&
&  0.17246E+00_JPRB, 0.17445E+00_JPRB, 0.17655E+00_JPRB, 0.17745E+00_JPRB, 0.17798E+00_JPRB, 0.17885E+00_JPRB, 0.17999E+00_JPRB ,&
&  0.18016E+00_JPRB, 0.17800E+00_JPRB, 0.17606E+00_JPRB, 0.17560E+00_JPRB, 0.17536E+00_JPRB, 0.17496E+00_JPRB, 0.17517E+00_JPRB ,&
&  0.17565E+00_JPRB, 0.17676E+00_JPRB, 0.17754E+00_JPRB, 0.17679E+00_JPRB, 0.17542E+00_JPRB, 0.17443E+00_JPRB, 0.17376E+00_JPRB ,&
&  0.17335E+00_JPRB, 0.17286E+00_JPRB, 0.17201E+00_JPRB, 0.17073E+00_JPRB, 0.16939E+00_JPRB, 0.16825E+00_JPRB, 0.16713E+00_JPRB ,&
&  0.16615E+00_JPRB, 0.16639E+00_JPRB, 0.16823E+00_JPRB, 0.16967E+00_JPRB, 0.16972E+00_JPRB, 0.16852E+00_JPRB, 0.16646E+00_JPRB ,&
&  0.16495E+00_JPRB, 0.16305E+00_JPRB, 0.16157E+00_JPRB, 0.16039E+00_JPRB, 0.15940E+00_JPRB, 0.15806E+00_JPRB, 0.15752E+00_JPRB ,&
&  0.15762E+00_JPRB, 0.15894E+00_JPRB, 0.15961E+00_JPRB, 0.16004E+00_JPRB, 0.15993E+00_JPRB, 0.15982E+00_JPRB, 0.16065E+00_JPRB ,&
&  0.16178E+00_JPRB, 0.16342E+00_JPRB, 0.16511E+00_JPRB, 0.16732E+00_JPRB, 0.16967E+00_JPRB, 0.17279E+00_JPRB, 0.17540E+00_JPRB ,&
&  0.17701E+00_JPRB, 0.17749E+00_JPRB, 0.17862E+00_JPRB, 0.18026E+00_JPRB, 0.18311E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.20294E+00_JPRB, 0.20591E+00_JPRB, 0.20837E+00_JPRB, 0.21176E+00_JPRB, 0.21498E+00_JPRB, 0.21942E+00_JPRB, 0.22120E+00_JPRB ,&
&  0.22088E+00_JPRB, 0.22020E+00_JPRB, 0.21852E+00_JPRB, 0.21607E+00_JPRB, 0.21239E+00_JPRB, 0.20875E+00_JPRB, 0.20540E+00_JPRB ,&
&  0.20290E+00_JPRB, 0.20086E+00_JPRB, 0.19919E+00_JPRB, 0.19766E+00_JPRB, 0.19552E+00_JPRB, 0.19351E+00_JPRB, 0.19192E+00_JPRB ,&
&  0.19107E+00_JPRB, 0.19017E+00_JPRB, 0.18967E+00_JPRB, 0.18939E+00_JPRB, 0.18928E+00_JPRB, 0.18935E+00_JPRB, 0.18952E+00_JPRB ,&
&  0.18869E+00_JPRB, 0.18576E+00_JPRB, 0.18415E+00_JPRB, 0.18371E+00_JPRB, 0.18340E+00_JPRB, 0.18393E+00_JPRB, 0.18385E+00_JPRB ,&
&  0.18379E+00_JPRB, 0.18453E+00_JPRB, 0.18583E+00_JPRB, 0.18715E+00_JPRB, 0.18720E+00_JPRB, 0.18766E+00_JPRB, 0.18865E+00_JPRB ,&
&  0.18906E+00_JPRB, 0.18861E+00_JPRB, 0.18709E+00_JPRB, 0.18648E+00_JPRB, 0.18681E+00_JPRB, 0.18661E+00_JPRB, 0.18689E+00_JPRB ,&
&  0.18747E+00_JPRB, 0.18842E+00_JPRB, 0.18884E+00_JPRB, 0.18752E+00_JPRB, 0.18631E+00_JPRB, 0.18590E+00_JPRB, 0.18590E+00_JPRB ,&
&  0.18554E+00_JPRB, 0.18460E+00_JPRB, 0.18277E+00_JPRB, 0.18126E+00_JPRB, 0.17895E+00_JPRB, 0.17708E+00_JPRB, 0.17617E+00_JPRB ,&
&  0.17672E+00_JPRB, 0.17719E+00_JPRB, 0.17845E+00_JPRB, 0.17964E+00_JPRB, 0.18041E+00_JPRB, 0.18060E+00_JPRB, 0.17907E+00_JPRB ,&
&  0.17698E+00_JPRB, 0.17606E+00_JPRB, 0.17515E+00_JPRB, 0.17437E+00_JPRB, 0.17448E+00_JPRB, 0.17437E+00_JPRB, 0.17395E+00_JPRB ,&
&  0.17342E+00_JPRB, 0.17363E+00_JPRB, 0.17400E+00_JPRB, 0.17427E+00_JPRB, 0.17426E+00_JPRB, 0.17378E+00_JPRB, 0.17434E+00_JPRB ,&
&  0.17559E+00_JPRB, 0.17733E+00_JPRB, 0.17935E+00_JPRB, 0.18159E+00_JPRB, 0.18409E+00_JPRB, 0.18639E+00_JPRB, 0.18852E+00_JPRB ,&
&  0.19124E+00_JPRB, 0.19236E+00_JPRB, 0.19400E+00_JPRB, 0.19662E+00_JPRB, 0.19930E+00_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.21523E+00_JPRB, 0.21780E+00_JPRB, 0.22206E+00_JPRB, 0.22689E+00_JPRB, 0.23031E+00_JPRB, 0.23405E+00_JPRB, 0.23451E+00_JPRB ,&
&  0.23352E+00_JPRB, 0.23154E+00_JPRB, 0.22764E+00_JPRB, 0.22301E+00_JPRB, 0.21880E+00_JPRB, 0.21539E+00_JPRB, 0.21310E+00_JPRB ,&
&  0.21180E+00_JPRB, 0.21072E+00_JPRB, 0.20944E+00_JPRB, 0.20727E+00_JPRB, 0.20452E+00_JPRB, 0.20203E+00_JPRB, 0.19998E+00_JPRB ,&
&  0.19906E+00_JPRB, 0.19873E+00_JPRB, 0.19886E+00_JPRB, 0.19867E+00_JPRB, 0.19873E+00_JPRB, 0.20018E+00_JPRB, 0.20186E+00_JPRB ,&
&  0.20098E+00_JPRB, 0.19708E+00_JPRB, 0.19482E+00_JPRB, 0.19471E+00_JPRB, 0.19500E+00_JPRB, 0.19567E+00_JPRB, 0.19655E+00_JPRB ,&
&  0.19696E+00_JPRB, 0.19774E+00_JPRB, 0.19864E+00_JPRB, 0.19967E+00_JPRB, 0.20059E+00_JPRB, 0.20297E+00_JPRB, 0.20418E+00_JPRB ,&
&  0.20432E+00_JPRB, 0.20370E+00_JPRB, 0.20248E+00_JPRB, 0.19999E+00_JPRB, 0.19809E+00_JPRB, 0.19734E+00_JPRB, 0.19659E+00_JPRB ,&
&  0.19715E+00_JPRB, 0.19772E+00_JPRB, 0.19823E+00_JPRB, 0.19759E+00_JPRB, 0.19617E+00_JPRB, 0.19603E+00_JPRB, 0.19650E+00_JPRB ,&
&  0.19639E+00_JPRB, 0.19525E+00_JPRB, 0.19368E+00_JPRB, 0.19283E+00_JPRB, 0.19200E+00_JPRB, 0.19185E+00_JPRB, 0.19188E+00_JPRB ,&
&  0.19181E+00_JPRB, 0.19106E+00_JPRB, 0.19064E+00_JPRB, 0.19054E+00_JPRB, 0.19051E+00_JPRB, 0.19093E+00_JPRB, 0.19063E+00_JPRB ,&
&  0.19054E+00_JPRB, 0.19069E+00_JPRB, 0.18977E+00_JPRB, 0.18956E+00_JPRB, 0.18959E+00_JPRB, 0.18991E+00_JPRB, 0.19001E+00_JPRB ,&
&  0.18975E+00_JPRB, 0.18935E+00_JPRB, 0.18901E+00_JPRB, 0.18906E+00_JPRB, 0.18876E+00_JPRB, 0.18837E+00_JPRB, 0.18917E+00_JPRB ,&
&  0.19206E+00_JPRB, 0.19511E+00_JPRB, 0.19775E+00_JPRB, 0.19955E+00_JPRB, 0.20023E+00_JPRB, 0.20161E+00_JPRB, 0.20410E+00_JPRB ,&
&  0.20633E+00_JPRB, 0.20724E+00_JPRB, 0.20894E+00_JPRB, 0.21046E+00_JPRB, 0.21232E+00_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.22939E+00_JPRB, 0.23223E+00_JPRB, 0.23628E+00_JPRB, 0.24216E+00_JPRB, 0.24547E+00_JPRB, 0.24817E+00_JPRB, 0.24763E+00_JPRB ,&
&  0.24515E+00_JPRB, 0.24169E+00_JPRB, 0.23762E+00_JPRB, 0.23281E+00_JPRB, 0.22786E+00_JPRB, 0.22411E+00_JPRB, 0.22267E+00_JPRB ,&
&  0.22254E+00_JPRB, 0.22183E+00_JPRB, 0.21969E+00_JPRB, 0.21667E+00_JPRB, 0.21322E+00_JPRB, 0.21050E+00_JPRB, 0.20831E+00_JPRB ,&
&  0.20747E+00_JPRB, 0.20763E+00_JPRB, 0.20830E+00_JPRB, 0.20867E+00_JPRB, 0.20907E+00_JPRB, 0.21163E+00_JPRB, 0.21546E+00_JPRB ,&
&  0.21340E+00_JPRB, 0.21095E+00_JPRB, 0.20659E+00_JPRB, 0.20618E+00_JPRB, 0.20738E+00_JPRB, 0.20961E+00_JPRB, 0.21169E+00_JPRB ,&
&  0.21323E+00_JPRB, 0.21552E+00_JPRB, 0.21773E+00_JPRB, 0.21936E+00_JPRB, 0.22094E+00_JPRB, 0.22251E+00_JPRB, 0.22342E+00_JPRB ,&
&  0.22363E+00_JPRB, 0.22256E+00_JPRB, 0.22084E+00_JPRB, 0.21772E+00_JPRB, 0.21365E+00_JPRB, 0.21091E+00_JPRB, 0.20937E+00_JPRB ,&
&  0.20770E+00_JPRB, 0.20733E+00_JPRB, 0.20728E+00_JPRB, 0.20683E+00_JPRB, 0.20612E+00_JPRB, 0.20597E+00_JPRB, 0.20649E+00_JPRB ,&
&  0.20677E+00_JPRB, 0.20634E+00_JPRB, 0.20559E+00_JPRB, 0.20552E+00_JPRB, 0.20603E+00_JPRB, 0.20715E+00_JPRB, 0.20822E+00_JPRB ,&
&  0.20896E+00_JPRB, 0.20885E+00_JPRB, 0.20837E+00_JPRB, 0.20692E+00_JPRB, 0.20584E+00_JPRB, 0.20599E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20758E+00_JPRB, 0.20757E+00_JPRB, 0.20782E+00_JPRB, 0.20700E+00_JPRB, 0.20633E+00_JPRB, 0.20651E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20720E+00_JPRB, 0.20708E+00_JPRB, 0.20674E+00_JPRB, 0.20652E+00_JPRB, 0.20689E+00_JPRB, 0.20701E+00_JPRB, 0.20792E+00_JPRB ,&
&  0.21026E+00_JPRB, 0.21293E+00_JPRB, 0.21570E+00_JPRB, 0.21786E+00_JPRB, 0.21920E+00_JPRB, 0.21955E+00_JPRB, 0.22150E+00_JPRB ,&
&  0.22436E+00_JPRB, 0.22622E+00_JPRB, 0.22589E+00_JPRB, 0.22541E+00_JPRB, 0.22706E+00_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.24590E+00_JPRB, 0.24733E+00_JPRB, 0.25186E+00_JPRB, 0.25682E+00_JPRB, 0.26166E+00_JPRB, 0.26238E+00_JPRB, 0.25974E+00_JPRB ,&
&  0.25689E+00_JPRB, 0.25309E+00_JPRB, 0.24772E+00_JPRB, 0.24130E+00_JPRB, 0.23542E+00_JPRB, 0.23366E+00_JPRB, 0.23354E+00_JPRB ,&
&  0.23371E+00_JPRB, 0.23261E+00_JPRB, 0.23001E+00_JPRB, 0.22691E+00_JPRB, 0.22318E+00_JPRB, 0.21991E+00_JPRB, 0.21748E+00_JPRB ,&
&  0.21674E+00_JPRB, 0.21644E+00_JPRB, 0.21690E+00_JPRB, 0.21709E+00_JPRB, 0.21798E+00_JPRB, 0.22116E+00_JPRB, 0.22653E+00_JPRB ,&
&  0.22429E+00_JPRB, 0.22385E+00_JPRB, 0.21811E+00_JPRB, 0.22049E+00_JPRB, 0.22262E+00_JPRB, 0.22690E+00_JPRB, 0.23125E+00_JPRB ,&
&  0.23525E+00_JPRB, 0.23925E+00_JPRB, 0.24181E+00_JPRB, 0.24218E+00_JPRB, 0.24145E+00_JPRB, 0.24152E+00_JPRB, 0.24126E+00_JPRB ,&
&  0.24048E+00_JPRB, 0.23942E+00_JPRB, 0.23766E+00_JPRB, 0.23430E+00_JPRB, 0.22958E+00_JPRB, 0.22536E+00_JPRB, 0.22226E+00_JPRB ,&
&  0.22142E+00_JPRB, 0.22096E+00_JPRB, 0.21971E+00_JPRB, 0.21857E+00_JPRB, 0.21744E+00_JPRB, 0.21684E+00_JPRB, 0.21744E+00_JPRB ,&
&  0.21863E+00_JPRB, 0.21966E+00_JPRB, 0.22048E+00_JPRB, 0.22164E+00_JPRB, 0.22355E+00_JPRB, 0.22531E+00_JPRB, 0.22645E+00_JPRB ,&
&  0.22670E+00_JPRB, 0.22626E+00_JPRB, 0.22592E+00_JPRB, 0.22549E+00_JPRB, 0.22518E+00_JPRB, 0.22471E+00_JPRB, 0.22471E+00_JPRB ,&
&  0.22490E+00_JPRB, 0.22495E+00_JPRB, 0.22442E+00_JPRB, 0.22344E+00_JPRB, 0.22261E+00_JPRB, 0.22191E+00_JPRB, 0.22176E+00_JPRB ,&
&  0.22214E+00_JPRB, 0.22256E+00_JPRB, 0.22253E+00_JPRB, 0.22219E+00_JPRB, 0.22246E+00_JPRB, 0.22296E+00_JPRB, 0.22420E+00_JPRB ,&
&  0.22610E+00_JPRB, 0.22868E+00_JPRB, 0.23207E+00_JPRB, 0.23577E+00_JPRB, 0.23699E+00_JPRB, 0.23814E+00_JPRB, 0.24076E+00_JPRB ,&
&  0.24319E+00_JPRB, 0.24398E+00_JPRB, 0.24417E+00_JPRB, 0.24443E+00_JPRB, 0.24554E+00_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.26627E+00_JPRB, 0.26549E+00_JPRB, 0.26807E+00_JPRB, 0.27264E+00_JPRB, 0.27823E+00_JPRB, 0.27654E+00_JPRB, 0.27377E+00_JPRB ,&
&  0.26835E+00_JPRB, 0.26165E+00_JPRB, 0.25520E+00_JPRB, 0.24917E+00_JPRB, 0.24603E+00_JPRB, 0.24596E+00_JPRB, 0.24550E+00_JPRB ,&
&  0.24434E+00_JPRB, 0.24187E+00_JPRB, 0.23823E+00_JPRB, 0.23454E+00_JPRB, 0.23183E+00_JPRB, 0.23041E+00_JPRB, 0.22920E+00_JPRB ,&
&  0.22846E+00_JPRB, 0.22766E+00_JPRB, 0.22873E+00_JPRB, 0.22891E+00_JPRB, 0.22998E+00_JPRB, 0.23424E+00_JPRB, 0.23749E+00_JPRB ,&
&  0.23938E+00_JPRB, 0.23847E+00_JPRB, 0.23905E+00_JPRB, 0.24013E+00_JPRB, 0.24318E+00_JPRB, 0.24982E+00_JPRB, 0.25683E+00_JPRB ,&
&  0.26277E+00_JPRB, 0.26739E+00_JPRB, 0.26979E+00_JPRB, 0.26910E+00_JPRB, 0.26689E+00_JPRB, 0.26419E+00_JPRB, 0.26217E+00_JPRB ,&
&  0.26006E+00_JPRB, 0.25652E+00_JPRB, 0.25286E+00_JPRB, 0.24983E+00_JPRB, 0.24829E+00_JPRB, 0.24709E+00_JPRB, 0.24554E+00_JPRB ,&
&  0.24360E+00_JPRB, 0.24182E+00_JPRB, 0.23939E+00_JPRB, 0.23690E+00_JPRB, 0.23477E+00_JPRB, 0.23374E+00_JPRB, 0.23420E+00_JPRB ,&
&  0.23594E+00_JPRB, 0.23851E+00_JPRB, 0.24150E+00_JPRB, 0.24490E+00_JPRB, 0.24723E+00_JPRB, 0.24890E+00_JPRB, 0.24913E+00_JPRB ,&
&  0.24889E+00_JPRB, 0.24853E+00_JPRB, 0.24828E+00_JPRB, 0.24804E+00_JPRB, 0.24751E+00_JPRB, 0.24656E+00_JPRB, 0.24545E+00_JPRB ,&
&  0.24446E+00_JPRB, 0.24379E+00_JPRB, 0.24333E+00_JPRB, 0.24294E+00_JPRB, 0.24171E+00_JPRB, 0.24006E+00_JPRB, 0.23899E+00_JPRB ,&
&  0.23850E+00_JPRB, 0.23836E+00_JPRB, 0.23828E+00_JPRB, 0.23786E+00_JPRB, 0.23766E+00_JPRB, 0.23823E+00_JPRB, 0.23973E+00_JPRB ,&
&  0.24231E+00_JPRB, 0.24592E+00_JPRB, 0.25032E+00_JPRB, 0.25542E+00_JPRB, 0.25959E+00_JPRB, 0.26353E+00_JPRB, 0.26731E+00_JPRB ,&
&  0.26931E+00_JPRB, 0.26855E+00_JPRB, 0.26833E+00_JPRB, 0.26882E+00_JPRB, 0.26804E+00_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.29057E+00_JPRB, 0.28732E+00_JPRB, 0.28709E+00_JPRB, 0.29037E+00_JPRB, 0.29333E+00_JPRB, 0.29180E+00_JPRB, 0.28733E+00_JPRB ,&
&  0.27839E+00_JPRB, 0.27052E+00_JPRB, 0.26386E+00_JPRB, 0.26032E+00_JPRB, 0.26095E+00_JPRB, 0.26066E+00_JPRB, 0.25837E+00_JPRB ,&
&  0.25610E+00_JPRB, 0.25229E+00_JPRB, 0.24923E+00_JPRB, 0.24617E+00_JPRB, 0.24388E+00_JPRB, 0.24293E+00_JPRB, 0.24270E+00_JPRB ,&
&  0.24168E+00_JPRB, 0.24063E+00_JPRB, 0.24114E+00_JPRB, 0.24210E+00_JPRB, 0.24473E+00_JPRB, 0.24977E+00_JPRB, 0.25541E+00_JPRB ,&
&  0.26410E+00_JPRB, 0.24882E+00_JPRB, 0.26402E+00_JPRB, 0.26885E+00_JPRB, 0.27192E+00_JPRB, 0.27717E+00_JPRB, 0.28272E+00_JPRB ,&
&  0.28796E+00_JPRB, 0.29178E+00_JPRB, 0.29393E+00_JPRB, 0.29346E+00_JPRB, 0.29095E+00_JPRB, 0.28840E+00_JPRB, 0.28423E+00_JPRB ,&
&  0.27942E+00_JPRB, 0.27762E+00_JPRB, 0.27681E+00_JPRB, 0.27490E+00_JPRB, 0.27264E+00_JPRB, 0.26943E+00_JPRB, 0.26623E+00_JPRB ,&
&  0.26333E+00_JPRB, 0.26059E+00_JPRB, 0.25848E+00_JPRB, 0.25657E+00_JPRB, 0.25522E+00_JPRB, 0.25497E+00_JPRB, 0.25602E+00_JPRB ,&
&  0.25758E+00_JPRB, 0.25981E+00_JPRB, 0.26435E+00_JPRB, 0.26892E+00_JPRB, 0.27176E+00_JPRB, 0.27308E+00_JPRB, 0.27362E+00_JPRB ,&
&  0.27361E+00_JPRB, 0.27338E+00_JPRB, 0.27281E+00_JPRB, 0.27201E+00_JPRB, 0.27066E+00_JPRB, 0.26920E+00_JPRB, 0.26658E+00_JPRB ,&
&  0.26451E+00_JPRB, 0.26299E+00_JPRB, 0.26135E+00_JPRB, 0.26033E+00_JPRB, 0.25893E+00_JPRB, 0.25673E+00_JPRB, 0.25448E+00_JPRB ,&
&  0.25269E+00_JPRB, 0.25142E+00_JPRB, 0.25051E+00_JPRB, 0.25016E+00_JPRB, 0.25035E+00_JPRB, 0.25203E+00_JPRB, 0.25401E+00_JPRB ,&
&  0.25677E+00_JPRB, 0.26101E+00_JPRB, 0.26713E+00_JPRB, 0.27416E+00_JPRB, 0.28157E+00_JPRB, 0.28616E+00_JPRB, 0.29005E+00_JPRB ,&
&  0.29200E+00_JPRB, 0.29144E+00_JPRB, 0.29181E+00_JPRB, 0.29206E+00_JPRB, 0.29192E+00_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.31668E+00_JPRB, 0.31460E+00_JPRB, 0.31155E+00_JPRB, 0.31281E+00_JPRB, 0.31095E+00_JPRB, 0.30673E+00_JPRB, 0.30017E+00_JPRB ,&
&  0.29029E+00_JPRB, 0.28303E+00_JPRB, 0.27727E+00_JPRB, 0.27515E+00_JPRB, 0.27700E+00_JPRB, 0.27521E+00_JPRB, 0.27164E+00_JPRB ,&
&  0.26899E+00_JPRB, 0.26595E+00_JPRB, 0.26382E+00_JPRB, 0.26132E+00_JPRB, 0.25945E+00_JPRB, 0.25897E+00_JPRB, 0.25869E+00_JPRB ,&
&  0.25753E+00_JPRB, 0.25585E+00_JPRB, 0.25480E+00_JPRB, 0.25504E+00_JPRB, 0.25869E+00_JPRB, 0.26556E+00_JPRB, 0.27632E+00_JPRB ,&
&  0.27359E+00_JPRB, 0.25550E+00_JPRB, 0.27083E+00_JPRB, 0.29006E+00_JPRB, 0.30142E+00_JPRB, 0.31022E+00_JPRB, 0.31608E+00_JPRB ,&
&  0.32142E+00_JPRB, 0.32260E+00_JPRB, 0.32292E+00_JPRB, 0.32157E+00_JPRB, 0.31845E+00_JPRB, 0.31332E+00_JPRB, 0.30825E+00_JPRB ,&
&  0.30551E+00_JPRB, 0.30421E+00_JPRB, 0.30259E+00_JPRB, 0.29855E+00_JPRB, 0.29380E+00_JPRB, 0.28840E+00_JPRB, 0.28340E+00_JPRB ,&
&  0.27978E+00_JPRB, 0.27616E+00_JPRB, 0.27341E+00_JPRB, 0.27221E+00_JPRB, 0.27288E+00_JPRB, 0.27493E+00_JPRB, 0.27856E+00_JPRB ,&
&  0.28280E+00_JPRB, 0.28896E+00_JPRB, 0.29672E+00_JPRB, 0.30205E+00_JPRB, 0.30509E+00_JPRB, 0.30565E+00_JPRB, 0.30549E+00_JPRB ,&
&  0.30386E+00_JPRB, 0.30210E+00_JPRB, 0.30128E+00_JPRB, 0.29986E+00_JPRB, 0.29766E+00_JPRB, 0.29447E+00_JPRB, 0.29018E+00_JPRB ,&
&  0.28672E+00_JPRB, 0.28399E+00_JPRB, 0.28136E+00_JPRB, 0.27899E+00_JPRB, 0.27654E+00_JPRB, 0.27380E+00_JPRB, 0.27138E+00_JPRB ,&
&  0.26873E+00_JPRB, 0.26599E+00_JPRB, 0.26417E+00_JPRB, 0.26416E+00_JPRB, 0.26648E+00_JPRB, 0.26948E+00_JPRB, 0.27339E+00_JPRB ,&
&  0.27791E+00_JPRB, 0.28477E+00_JPRB, 0.29271E+00_JPRB, 0.30251E+00_JPRB, 0.31039E+00_JPRB, 0.31288E+00_JPRB, 0.31514E+00_JPRB ,&
&  0.31684E+00_JPRB, 0.31540E+00_JPRB, 0.31670E+00_JPRB, 0.31618E+00_JPRB, 0.31521E+00_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.34041E+00_JPRB, 0.34579E+00_JPRB, 0.34598E+00_JPRB, 0.34027E+00_JPRB, 0.33116E+00_JPRB, 0.32480E+00_JPRB, 0.31519E+00_JPRB ,&
&  0.30487E+00_JPRB, 0.29848E+00_JPRB, 0.29115E+00_JPRB, 0.28809E+00_JPRB, 0.28779E+00_JPRB, 0.28438E+00_JPRB, 0.28186E+00_JPRB ,&
&  0.27978E+00_JPRB, 0.27599E+00_JPRB, 0.27366E+00_JPRB, 0.27128E+00_JPRB, 0.27108E+00_JPRB, 0.27121E+00_JPRB, 0.27121E+00_JPRB ,&
&  0.27055E+00_JPRB, 0.26898E+00_JPRB, 0.26781E+00_JPRB, 0.26754E+00_JPRB, 0.27059E+00_JPRB, 0.27788E+00_JPRB, 0.29131E+00_JPRB ,&
&  0.31829E+00_JPRB, 0.28423E+00_JPRB, 0.29425E+00_JPRB, 0.30733E+00_JPRB, 0.32931E+00_JPRB, 0.34125E+00_JPRB, 0.34487E+00_JPRB ,&
&  0.34901E+00_JPRB, 0.35380E+00_JPRB, 0.35443E+00_JPRB, 0.35451E+00_JPRB, 0.34844E+00_JPRB, 0.34158E+00_JPRB, 0.33788E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.32791E+00_JPRB, 0.32191E+00_JPRB, 0.31506E+00_JPRB, 0.31094E+00_JPRB, 0.30785E+00_JPRB, 0.30466E+00_JPRB ,&
&  0.30051E+00_JPRB, 0.29555E+00_JPRB, 0.29199E+00_JPRB, 0.29098E+00_JPRB, 0.29321E+00_JPRB, 0.29870E+00_JPRB, 0.30850E+00_JPRB ,&
&  0.31761E+00_JPRB, 0.32774E+00_JPRB, 0.33696E+00_JPRB, 0.34184E+00_JPRB, 0.34375E+00_JPRB, 0.34210E+00_JPRB, 0.33913E+00_JPRB ,&
&  0.33515E+00_JPRB, 0.33110E+00_JPRB, 0.32826E+00_JPRB, 0.32422E+00_JPRB, 0.31886E+00_JPRB, 0.31475E+00_JPRB, 0.31178E+00_JPRB ,&
&  0.30978E+00_JPRB, 0.30682E+00_JPRB, 0.30289E+00_JPRB, 0.29900E+00_JPRB, 0.29489E+00_JPRB, 0.29155E+00_JPRB, 0.28866E+00_JPRB ,&
&  0.28587E+00_JPRB, 0.28296E+00_JPRB, 0.28162E+00_JPRB, 0.28327E+00_JPRB, 0.28659E+00_JPRB, 0.29052E+00_JPRB, 0.29429E+00_JPRB ,&
&  0.29989E+00_JPRB, 0.30820E+00_JPRB, 0.31616E+00_JPRB, 0.32874E+00_JPRB, 0.33435E+00_JPRB, 0.33722E+00_JPRB, 0.34005E+00_JPRB ,&
&  0.34026E+00_JPRB, 0.33905E+00_JPRB, 0.30471E+00_JPRB, 0.32634E+00_JPRB, 0.33423E+00_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.37550E+00_JPRB, 0.38093E+00_JPRB, 0.37898E+00_JPRB, 0.36610E+00_JPRB, 0.35218E+00_JPRB, 0.34366E+00_JPRB, 0.33351E+00_JPRB ,&
&  0.32255E+00_JPRB, 0.31346E+00_JPRB, 0.30316E+00_JPRB, 0.29940E+00_JPRB, 0.29678E+00_JPRB, 0.29549E+00_JPRB, 0.29412E+00_JPRB ,&
&  0.29081E+00_JPRB, 0.28534E+00_JPRB, 0.28089E+00_JPRB, 0.27888E+00_JPRB, 0.27917E+00_JPRB, 0.28018E+00_JPRB, 0.28168E+00_JPRB ,&
&  0.28153E+00_JPRB, 0.28176E+00_JPRB, 0.28169E+00_JPRB, 0.28236E+00_JPRB, 0.28464E+00_JPRB, 0.29235E+00_JPRB, 0.30747E+00_JPRB ,&
&  0.34200E+00_JPRB, 0.30847E+00_JPRB, 0.31594E+00_JPRB, 0.33208E+00_JPRB, 0.34848E+00_JPRB, 0.36900E+00_JPRB, 0.38562E+00_JPRB ,&
&  0.39014E+00_JPRB, 0.38577E+00_JPRB, 0.38856E+00_JPRB, 0.39655E+00_JPRB, 0.39466E+00_JPRB, 0.39081E+00_JPRB, 0.38455E+00_JPRB ,&
&  0.37375E+00_JPRB, 0.36098E+00_JPRB, 0.34776E+00_JPRB, 0.33912E+00_JPRB, 0.33549E+00_JPRB, 0.33307E+00_JPRB, 0.33256E+00_JPRB ,&
&  0.32889E+00_JPRB, 0.32646E+00_JPRB, 0.32900E+00_JPRB, 0.33240E+00_JPRB, 0.33517E+00_JPRB, 0.34312E+00_JPRB, 0.35565E+00_JPRB ,&
&  0.36449E+00_JPRB, 0.37450E+00_JPRB, 0.38192E+00_JPRB, 0.38385E+00_JPRB, 0.38481E+00_JPRB, 0.38224E+00_JPRB, 0.37380E+00_JPRB ,&
&  0.36508E+00_JPRB, 0.35613E+00_JPRB, 0.34856E+00_JPRB, 0.34160E+00_JPRB, 0.33653E+00_JPRB, 0.33468E+00_JPRB, 0.33382E+00_JPRB ,&
&  0.33311E+00_JPRB, 0.33267E+00_JPRB, 0.33062E+00_JPRB, 0.32749E+00_JPRB, 0.32478E+00_JPRB, 0.32317E+00_JPRB, 0.32160E+00_JPRB ,&
&  0.31964E+00_JPRB, 0.31745E+00_JPRB, 0.31701E+00_JPRB, 0.31926E+00_JPRB, 0.32211E+00_JPRB, 0.32601E+00_JPRB, 0.33093E+00_JPRB ,&
&  0.33589E+00_JPRB, 0.34436E+00_JPRB, 0.35120E+00_JPRB, 0.36214E+00_JPRB, 0.36843E+00_JPRB, 0.36877E+00_JPRB, 0.37000E+00_JPRB ,&
&  0.36769E+00_JPRB, 0.36554E+00_JPRB, 0.36445E+00_JPRB, 0.34464E+00_JPRB, 0.36709E+00_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.42282E+00_JPRB, 0.42190E+00_JPRB, 0.40712E+00_JPRB, 0.38707E+00_JPRB, 0.37137E+00_JPRB, 0.35722E+00_JPRB, 0.34514E+00_JPRB ,&
&  0.33709E+00_JPRB, 0.32834E+00_JPRB, 0.31886E+00_JPRB, 0.31181E+00_JPRB, 0.31050E+00_JPRB, 0.30803E+00_JPRB, 0.30489E+00_JPRB ,&
&  0.29920E+00_JPRB, 0.29381E+00_JPRB, 0.29023E+00_JPRB, 0.28877E+00_JPRB, 0.28905E+00_JPRB, 0.29071E+00_JPRB, 0.29204E+00_JPRB ,&
&  0.29141E+00_JPRB, 0.28910E+00_JPRB, 0.29014E+00_JPRB, 0.29305E+00_JPRB, 0.29805E+00_JPRB, 0.30550E+00_JPRB, 0.32359E+00_JPRB ,&
&  0.36069E+00_JPRB, 0.33182E+00_JPRB, 0.34620E+00_JPRB, 0.35305E+00_JPRB, 0.35895E+00_JPRB, 0.37890E+00_JPRB, 0.39641E+00_JPRB ,&
&  0.40952E+00_JPRB, 0.41891E+00_JPRB, 0.43662E+00_JPRB, 0.45371E+00_JPRB, 0.45363E+00_JPRB, 0.44395E+00_JPRB, 0.42925E+00_JPRB ,&
&  0.41293E+00_JPRB, 0.40081E+00_JPRB, 0.39043E+00_JPRB, 0.38184E+00_JPRB, 0.37557E+00_JPRB, 0.37279E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37003E+00_JPRB, 0.36850E+00_JPRB, 0.37066E+00_JPRB, 0.37898E+00_JPRB, 0.38662E+00_JPRB, 0.39421E+00_JPRB, 0.40281E+00_JPRB ,&
&  0.40861E+00_JPRB, 0.41249E+00_JPRB, 0.41408E+00_JPRB, 0.41332E+00_JPRB, 0.40790E+00_JPRB, 0.40187E+00_JPRB, 0.38959E+00_JPRB ,&
&  0.37720E+00_JPRB, 0.36776E+00_JPRB, 0.36327E+00_JPRB, 0.36007E+00_JPRB, 0.35794E+00_JPRB, 0.35699E+00_JPRB, 0.35576E+00_JPRB ,&
&  0.35476E+00_JPRB, 0.35608E+00_JPRB, 0.35762E+00_JPRB, 0.35891E+00_JPRB, 0.35993E+00_JPRB, 0.36056E+00_JPRB, 0.35983E+00_JPRB ,&
&  0.35859E+00_JPRB, 0.35805E+00_JPRB, 0.35850E+00_JPRB, 0.35938E+00_JPRB, 0.36064E+00_JPRB, 0.36252E+00_JPRB, 0.36536E+00_JPRB ,&
&  0.36966E+00_JPRB, 0.37893E+00_JPRB, 0.38410E+00_JPRB, 0.39647E+00_JPRB, 0.39613E+00_JPRB, 0.39501E+00_JPRB, 0.39441E+00_JPRB ,&
&  0.39187E+00_JPRB, 0.38919E+00_JPRB, 0.38318E+00_JPRB, 0.38190E+00_JPRB, 0.39906E+00_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.50260E+00_JPRB, 0.47568E+00_JPRB, 0.43876E+00_JPRB, 0.40954E+00_JPRB, 0.38859E+00_JPRB, 0.37238E+00_JPRB, 0.36232E+00_JPRB ,&
&  0.35823E+00_JPRB, 0.34946E+00_JPRB, 0.33794E+00_JPRB, 0.32841E+00_JPRB, 0.32366E+00_JPRB, 0.31995E+00_JPRB, 0.31493E+00_JPRB ,&
&  0.31047E+00_JPRB, 0.30665E+00_JPRB, 0.30444E+00_JPRB, 0.30350E+00_JPRB, 0.30296E+00_JPRB, 0.30235E+00_JPRB, 0.30022E+00_JPRB ,&
&  0.29673E+00_JPRB, 0.29703E+00_JPRB, 0.29935E+00_JPRB, 0.30576E+00_JPRB, 0.31332E+00_JPRB, 0.32213E+00_JPRB, 0.33916E+00_JPRB ,&
&  0.37941E+00_JPRB, 0.34474E+00_JPRB, 0.37023E+00_JPRB, 0.37718E+00_JPRB, 0.37836E+00_JPRB, 0.39549E+00_JPRB, 0.41608E+00_JPRB ,&
&  0.44047E+00_JPRB, 0.47280E+00_JPRB, 0.50317E+00_JPRB, 0.51917E+00_JPRB, 0.51210E+00_JPRB, 0.49365E+00_JPRB, 0.47528E+00_JPRB ,&
&  0.45778E+00_JPRB, 0.44609E+00_JPRB, 0.43526E+00_JPRB, 0.42520E+00_JPRB, 0.41769E+00_JPRB, 0.41657E+00_JPRB, 0.41974E+00_JPRB ,&
&  0.42024E+00_JPRB, 0.41289E+00_JPRB, 0.41588E+00_JPRB, 0.42380E+00_JPRB, 0.43555E+00_JPRB, 0.44196E+00_JPRB, 0.44755E+00_JPRB ,&
&  0.45122E+00_JPRB, 0.44890E+00_JPRB, 0.44325E+00_JPRB, 0.43540E+00_JPRB, 0.42170E+00_JPRB, 0.40805E+00_JPRB, 0.39421E+00_JPRB ,&
&  0.37997E+00_JPRB, 0.37089E+00_JPRB, 0.36701E+00_JPRB, 0.36516E+00_JPRB, 0.36586E+00_JPRB, 0.36803E+00_JPRB, 0.37203E+00_JPRB ,&
&  0.37837E+00_JPRB, 0.38435E+00_JPRB, 0.38943E+00_JPRB, 0.39225E+00_JPRB, 0.39194E+00_JPRB, 0.39032E+00_JPRB, 0.38705E+00_JPRB ,&
&  0.38284E+00_JPRB, 0.37916E+00_JPRB, 0.37775E+00_JPRB, 0.37916E+00_JPRB, 0.38188E+00_JPRB, 0.38551E+00_JPRB, 0.38975E+00_JPRB ,&
&  0.39513E+00_JPRB, 0.40559E+00_JPRB, 0.40266E+00_JPRB, 0.41115E+00_JPRB, 0.42603E+00_JPRB, 0.42254E+00_JPRB, 0.41831E+00_JPRB ,&
&  0.41181E+00_JPRB, 0.40840E+00_JPRB, 0.40577E+00_JPRB, 0.40873E+00_JPRB, 0.48261E+00_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.45404E+00_JPRB, 0.44536E+00_JPRB, 0.42459E+00_JPRB, 0.40503E+00_JPRB, 0.39212E+00_JPRB, 0.38243E+00_JPRB, 0.37672E+00_JPRB ,&
&  0.37344E+00_JPRB, 0.36657E+00_JPRB, 0.36007E+00_JPRB, 0.35092E+00_JPRB, 0.34303E+00_JPRB, 0.33666E+00_JPRB, 0.33049E+00_JPRB ,&
&  0.32526E+00_JPRB, 0.32236E+00_JPRB, 0.32016E+00_JPRB, 0.31794E+00_JPRB, 0.31403E+00_JPRB, 0.30804E+00_JPRB, 0.30474E+00_JPRB ,&
&  0.30420E+00_JPRB, 0.30586E+00_JPRB, 0.31034E+00_JPRB, 0.31830E+00_JPRB, 0.32545E+00_JPRB, 0.33377E+00_JPRB, 0.34792E+00_JPRB ,&
&  0.37636E+00_JPRB, 0.38198E+00_JPRB, 0.40999E+00_JPRB, 0.42390E+00_JPRB, 0.41587E+00_JPRB, 0.41870E+00_JPRB, 0.44664E+00_JPRB ,&
&  0.49238E+00_JPRB, 0.53601E+00_JPRB, 0.57216E+00_JPRB, 0.58725E+00_JPRB, 0.57825E+00_JPRB, 0.56481E+00_JPRB, 0.54505E+00_JPRB ,&
&  0.52527E+00_JPRB, 0.50333E+00_JPRB, 0.48130E+00_JPRB, 0.46059E+00_JPRB, 0.44141E+00_JPRB, 0.43846E+00_JPRB, 0.44448E+00_JPRB ,&
&  0.45394E+00_JPRB, 0.45827E+00_JPRB, 0.45683E+00_JPRB, 0.45873E+00_JPRB, 0.46332E+00_JPRB, 0.47192E+00_JPRB, 0.48937E+00_JPRB ,&
&  0.49557E+00_JPRB, 0.48574E+00_JPRB, 0.47131E+00_JPRB, 0.45283E+00_JPRB, 0.43013E+00_JPRB, 0.41145E+00_JPRB, 0.39575E+00_JPRB ,&
&  0.38309E+00_JPRB, 0.37412E+00_JPRB, 0.36893E+00_JPRB, 0.36784E+00_JPRB, 0.37185E+00_JPRB, 0.38112E+00_JPRB, 0.39093E+00_JPRB ,&
&  0.40063E+00_JPRB, 0.40878E+00_JPRB, 0.41284E+00_JPRB, 0.41377E+00_JPRB, 0.41037E+00_JPRB, 0.40345E+00_JPRB, 0.39270E+00_JPRB ,&
&  0.38331E+00_JPRB, 0.37982E+00_JPRB, 0.38602E+00_JPRB, 0.39183E+00_JPRB, 0.39617E+00_JPRB, 0.40016E+00_JPRB, 0.40414E+00_JPRB ,&
&  0.41233E+00_JPRB, 0.41842E+00_JPRB, 0.41902E+00_JPRB, 0.41736E+00_JPRB, 0.41568E+00_JPRB, 0.43140E+00_JPRB, 0.42609E+00_JPRB ,&
&  0.42258E+00_JPRB, 0.41942E+00_JPRB, 0.42055E+00_JPRB, 0.42199E+00_JPRB, 0.44966E+00_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.44498E+00_JPRB, 0.44150E+00_JPRB, 0.42887E+00_JPRB, 0.41389E+00_JPRB, 0.40319E+00_JPRB, 0.39471E+00_JPRB, 0.38849E+00_JPRB ,&
&  0.38490E+00_JPRB, 0.37872E+00_JPRB, 0.37341E+00_JPRB, 0.36624E+00_JPRB, 0.35749E+00_JPRB, 0.35048E+00_JPRB, 0.34634E+00_JPRB ,&
&  0.34049E+00_JPRB, 0.33779E+00_JPRB, 0.33475E+00_JPRB, 0.32801E+00_JPRB, 0.32072E+00_JPRB, 0.31471E+00_JPRB, 0.31094E+00_JPRB ,&
&  0.30983E+00_JPRB, 0.31267E+00_JPRB, 0.31828E+00_JPRB, 0.32978E+00_JPRB, 0.33884E+00_JPRB, 0.34694E+00_JPRB, 0.35989E+00_JPRB ,&
&  0.39219E+00_JPRB, 0.41155E+00_JPRB, 0.48486E+00_JPRB, 0.47135E+00_JPRB, 0.45927E+00_JPRB, 0.45527E+00_JPRB, 0.51032E+00_JPRB ,&
&  0.56937E+00_JPRB, 0.63200E+00_JPRB, 0.67089E+00_JPRB, 0.68661E+00_JPRB, 0.66341E+00_JPRB, 0.64394E+00_JPRB, 0.62162E+00_JPRB ,&
&  0.59628E+00_JPRB, 0.56682E+00_JPRB, 0.52846E+00_JPRB, 0.49068E+00_JPRB, 0.46901E+00_JPRB, 0.46932E+00_JPRB, 0.47945E+00_JPRB ,&
&  0.49127E+00_JPRB, 0.49516E+00_JPRB, 0.48339E+00_JPRB, 0.47694E+00_JPRB, 0.45882E+00_JPRB, 0.44153E+00_JPRB, 0.47871E+00_JPRB ,&
&  0.56273E+00_JPRB, 0.54955E+00_JPRB, 0.51648E+00_JPRB, 0.47247E+00_JPRB, 0.44352E+00_JPRB, 0.42051E+00_JPRB, 0.40590E+00_JPRB ,&
&  0.39407E+00_JPRB, 0.38585E+00_JPRB, 0.38204E+00_JPRB, 0.38265E+00_JPRB, 0.38900E+00_JPRB, 0.39828E+00_JPRB, 0.40511E+00_JPRB ,&
&  0.41174E+00_JPRB, 0.41620E+00_JPRB, 0.41798E+00_JPRB, 0.41519E+00_JPRB, 0.40946E+00_JPRB, 0.40112E+00_JPRB, 0.39307E+00_JPRB ,&
&  0.38984E+00_JPRB, 0.39447E+00_JPRB, 0.40614E+00_JPRB, 0.40631E+00_JPRB, 0.41215E+00_JPRB, 0.42228E+00_JPRB, 0.42361E+00_JPRB ,&
&  0.41870E+00_JPRB, 0.41875E+00_JPRB, 0.42589E+00_JPRB, 0.42335E+00_JPRB, 0.42364E+00_JPRB, 0.44751E+00_JPRB, 0.43680E+00_JPRB ,&
&  0.42840E+00_JPRB, 0.42367E+00_JPRB, 0.42678E+00_JPRB, 0.42752E+00_JPRB, 0.43812E+00_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.44603E+00_JPRB, 0.44436E+00_JPRB, 0.43414E+00_JPRB, 0.42082E+00_JPRB, 0.41039E+00_JPRB, 0.39988E+00_JPRB, 0.39213E+00_JPRB ,&
&  0.38697E+00_JPRB, 0.38301E+00_JPRB, 0.38028E+00_JPRB, 0.37671E+00_JPRB, 0.37153E+00_JPRB, 0.36451E+00_JPRB, 0.35781E+00_JPRB ,&
&  0.35045E+00_JPRB, 0.34312E+00_JPRB, 0.33611E+00_JPRB, 0.32871E+00_JPRB, 0.32340E+00_JPRB, 0.31953E+00_JPRB, 0.31706E+00_JPRB ,&
&  0.31732E+00_JPRB, 0.32039E+00_JPRB, 0.32742E+00_JPRB, 0.34182E+00_JPRB, 0.35691E+00_JPRB, 0.36252E+00_JPRB, 0.36682E+00_JPRB ,&
&  0.39037E+00_JPRB, 0.34035E+00_JPRB, 0.49955E+00_JPRB, 0.54017E+00_JPRB, 0.53921E+00_JPRB, 0.56160E+00_JPRB, 0.64401E+00_JPRB ,&
&  0.71749E+00_JPRB, 0.74761E+00_JPRB, 0.75707E+00_JPRB, 0.73583E+00_JPRB, 0.68183E+00_JPRB, 0.63761E+00_JPRB, 0.60115E+00_JPRB ,&
&  0.57041E+00_JPRB, 0.54199E+00_JPRB, 0.51109E+00_JPRB, 0.48245E+00_JPRB, 0.46308E+00_JPRB, 0.46078E+00_JPRB, 0.46663E+00_JPRB ,&
&  0.48451E+00_JPRB, 0.48784E+00_JPRB, 0.48254E+00_JPRB, 0.48476E+00_JPRB, 0.45859E+00_JPRB, 0.46087E+00_JPRB, 0.47379E+00_JPRB ,&
&  0.54486E+00_JPRB, 0.60492E+00_JPRB, 0.55157E+00_JPRB, 0.49029E+00_JPRB, 0.45345E+00_JPRB, 0.43563E+00_JPRB, 0.41780E+00_JPRB ,&
&  0.40806E+00_JPRB, 0.40139E+00_JPRB, 0.39836E+00_JPRB, 0.39802E+00_JPRB, 0.40178E+00_JPRB, 0.40672E+00_JPRB, 0.40855E+00_JPRB ,&
&  0.41069E+00_JPRB, 0.41224E+00_JPRB, 0.41347E+00_JPRB, 0.41276E+00_JPRB, 0.40936E+00_JPRB, 0.40381E+00_JPRB, 0.40238E+00_JPRB ,&
&  0.40743E+00_JPRB, 0.41951E+00_JPRB, 0.42826E+00_JPRB, 0.42958E+00_JPRB, 0.43444E+00_JPRB, 0.43928E+00_JPRB, 0.44137E+00_JPRB ,&
&  0.44046E+00_JPRB, 0.43965E+00_JPRB, 0.43690E+00_JPRB, 0.44045E+00_JPRB, 0.43071E+00_JPRB, 0.45206E+00_JPRB, 0.43886E+00_JPRB ,&
&  0.43088E+00_JPRB, 0.42854E+00_JPRB, 0.43012E+00_JPRB, 0.43361E+00_JPRB, 0.43945E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.45080E+00_JPRB, 0.44912E+00_JPRB, 0.43978E+00_JPRB, 0.42729E+00_JPRB, 0.41400E+00_JPRB, 0.40297E+00_JPRB, 0.39857E+00_JPRB ,&
&  0.39361E+00_JPRB, 0.38806E+00_JPRB, 0.38276E+00_JPRB, 0.38034E+00_JPRB, 0.37972E+00_JPRB, 0.37435E+00_JPRB, 0.36437E+00_JPRB ,&
&  0.35356E+00_JPRB, 0.34311E+00_JPRB, 0.33495E+00_JPRB, 0.33145E+00_JPRB, 0.32748E+00_JPRB, 0.32518E+00_JPRB, 0.32544E+00_JPRB ,&
&  0.32664E+00_JPRB, 0.33183E+00_JPRB, 0.34253E+00_JPRB, 0.36050E+00_JPRB, 0.37311E+00_JPRB, 0.37554E+00_JPRB, 0.37452E+00_JPRB ,&
&  0.39589E+00_JPRB, 0.47006E+00_JPRB, 0.48307E+00_JPRB, 0.62977E+00_JPRB, 0.69634E+00_JPRB, 0.74549E+00_JPRB, 0.79462E+00_JPRB ,&
&  0.81797E+00_JPRB, 0.83776E+00_JPRB, 0.80450E+00_JPRB, 0.74798E+00_JPRB, 0.67182E+00_JPRB, 0.61289E+00_JPRB, 0.56252E+00_JPRB ,&
&  0.52076E+00_JPRB, 0.48805E+00_JPRB, 0.46529E+00_JPRB, 0.44819E+00_JPRB, 0.43872E+00_JPRB, 0.43849E+00_JPRB, 0.44859E+00_JPRB ,&
&  0.46529E+00_JPRB, 0.47672E+00_JPRB, 0.49283E+00_JPRB, 0.51609E+00_JPRB, 0.48596E+00_JPRB, 0.48757E+00_JPRB, 0.51186E+00_JPRB ,&
&  0.54730E+00_JPRB, 0.61085E+00_JPRB, 0.54602E+00_JPRB, 0.48559E+00_JPRB, 0.45678E+00_JPRB, 0.44623E+00_JPRB, 0.43514E+00_JPRB ,&
&  0.42686E+00_JPRB, 0.42144E+00_JPRB, 0.41880E+00_JPRB, 0.41549E+00_JPRB, 0.41353E+00_JPRB, 0.41314E+00_JPRB, 0.41330E+00_JPRB ,&
&  0.41445E+00_JPRB, 0.41635E+00_JPRB, 0.41897E+00_JPRB, 0.42029E+00_JPRB, 0.41854E+00_JPRB, 0.41948E+00_JPRB, 0.42567E+00_JPRB ,&
&  0.43667E+00_JPRB, 0.44711E+00_JPRB, 0.44964E+00_JPRB, 0.44830E+00_JPRB, 0.45205E+00_JPRB, 0.45729E+00_JPRB, 0.45511E+00_JPRB ,&
&  0.46012E+00_JPRB, 0.45985E+00_JPRB, 0.45664E+00_JPRB, 0.45382E+00_JPRB, 0.45066E+00_JPRB, 0.47300E+00_JPRB, 0.44666E+00_JPRB ,&
&  0.43844E+00_JPRB, 0.43815E+00_JPRB, 0.44083E+00_JPRB, 0.44093E+00_JPRB, 0.44603E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.45726E+00_JPRB, 0.44966E+00_JPRB, 0.44037E+00_JPRB, 0.42907E+00_JPRB, 0.41662E+00_JPRB, 0.40849E+00_JPRB, 0.40498E+00_JPRB ,&
&  0.39760E+00_JPRB, 0.39072E+00_JPRB, 0.38560E+00_JPRB, 0.38383E+00_JPRB, 0.38236E+00_JPRB, 0.37628E+00_JPRB, 0.36445E+00_JPRB ,&
&  0.35260E+00_JPRB, 0.34519E+00_JPRB, 0.34181E+00_JPRB, 0.33833E+00_JPRB, 0.33378E+00_JPRB, 0.33119E+00_JPRB, 0.33217E+00_JPRB ,&
&  0.33703E+00_JPRB, 0.34446E+00_JPRB, 0.35978E+00_JPRB, 0.37711E+00_JPRB, 0.39047E+00_JPRB, 0.40004E+00_JPRB, 0.40191E+00_JPRB ,&
&  0.44681E+00_JPRB, 0.49128E+00_JPRB, 0.55535E+00_JPRB, 0.83653E+00_JPRB, 0.91670E+00_JPRB, 0.91740E+00_JPRB, 0.90738E+00_JPRB ,&
&  0.88645E+00_JPRB, 0.91117E+00_JPRB, 0.83424E+00_JPRB, 0.74069E+00_JPRB, 0.65033E+00_JPRB, 0.58747E+00_JPRB, 0.53952E+00_JPRB ,&
&  0.49541E+00_JPRB, 0.46508E+00_JPRB, 0.45043E+00_JPRB, 0.44421E+00_JPRB, 0.44238E+00_JPRB, 0.44479E+00_JPRB, 0.45308E+00_JPRB ,&
&  0.46772E+00_JPRB, 0.48968E+00_JPRB, 0.52907E+00_JPRB, 0.54920E+00_JPRB, 0.52359E+00_JPRB, 0.51334E+00_JPRB, 0.51902E+00_JPRB ,&
&  0.55434E+00_JPRB, 0.57909E+00_JPRB, 0.52563E+00_JPRB, 0.48859E+00_JPRB, 0.46498E+00_JPRB, 0.46386E+00_JPRB, 0.45102E+00_JPRB ,&
&  0.44460E+00_JPRB, 0.43998E+00_JPRB, 0.43692E+00_JPRB, 0.43079E+00_JPRB, 0.42374E+00_JPRB, 0.42142E+00_JPRB, 0.42208E+00_JPRB ,&
&  0.42403E+00_JPRB, 0.42776E+00_JPRB, 0.42983E+00_JPRB, 0.43111E+00_JPRB, 0.43264E+00_JPRB, 0.43867E+00_JPRB, 0.44834E+00_JPRB ,&
&  0.45603E+00_JPRB, 0.47005E+00_JPRB, 0.46792E+00_JPRB, 0.46431E+00_JPRB, 0.47433E+00_JPRB, 0.48183E+00_JPRB, 0.47157E+00_JPRB ,&
&  0.46920E+00_JPRB, 0.47507E+00_JPRB, 0.47379E+00_JPRB, 0.46057E+00_JPRB, 0.46699E+00_JPRB, 0.48188E+00_JPRB, 0.46037E+00_JPRB ,&
&  0.45548E+00_JPRB, 0.45637E+00_JPRB, 0.46159E+00_JPRB, 0.45980E+00_JPRB, 0.45869E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.46376E+00_JPRB, 0.45257E+00_JPRB, 0.44357E+00_JPRB, 0.43656E+00_JPRB, 0.42713E+00_JPRB, 0.41708E+00_JPRB, 0.40949E+00_JPRB ,&
&  0.39695E+00_JPRB, 0.38857E+00_JPRB, 0.38263E+00_JPRB, 0.37596E+00_JPRB, 0.36980E+00_JPRB, 0.36438E+00_JPRB, 0.35986E+00_JPRB ,&
&  0.35720E+00_JPRB, 0.35507E+00_JPRB, 0.35346E+00_JPRB, 0.34982E+00_JPRB, 0.34656E+00_JPRB, 0.34577E+00_JPRB, 0.34776E+00_JPRB ,&
&  0.35868E+00_JPRB, 0.37410E+00_JPRB, 0.39185E+00_JPRB, 0.41002E+00_JPRB, 0.42622E+00_JPRB, 0.44592E+00_JPRB, 0.46555E+00_JPRB ,&
&  0.54476E+00_JPRB, 0.56180E+00_JPRB, 0.68420E+00_JPRB, 0.10299E+01_JPRB, 0.10821E+01_JPRB, 0.98624E+00_JPRB, 0.94934E+00_JPRB ,&
&  0.89657E+00_JPRB, 0.79216E+00_JPRB, 0.80148E+00_JPRB, 0.70083E+00_JPRB, 0.61222E+00_JPRB, 0.56740E+00_JPRB, 0.53539E+00_JPRB ,&
&  0.50223E+00_JPRB, 0.48211E+00_JPRB, 0.47643E+00_JPRB, 0.47356E+00_JPRB, 0.46720E+00_JPRB, 0.46997E+00_JPRB, 0.47939E+00_JPRB ,&
&  0.50081E+00_JPRB, 0.53469E+00_JPRB, 0.59290E+00_JPRB, 0.62096E+00_JPRB, 0.53960E+00_JPRB, 0.52159E+00_JPRB, 0.51889E+00_JPRB ,&
&  0.54756E+00_JPRB, 0.54432E+00_JPRB, 0.51874E+00_JPRB, 0.49523E+00_JPRB, 0.47496E+00_JPRB, 0.48392E+00_JPRB, 0.46541E+00_JPRB ,&
&  0.45917E+00_JPRB, 0.45330E+00_JPRB, 0.44664E+00_JPRB, 0.43593E+00_JPRB, 0.42942E+00_JPRB, 0.42455E+00_JPRB, 0.42529E+00_JPRB ,&
&  0.43112E+00_JPRB, 0.43644E+00_JPRB, 0.44095E+00_JPRB, 0.44618E+00_JPRB, 0.45271E+00_JPRB, 0.45903E+00_JPRB, 0.46623E+00_JPRB ,&
&  0.47351E+00_JPRB, 0.49035E+00_JPRB, 0.48785E+00_JPRB, 0.49765E+00_JPRB, 0.51179E+00_JPRB, 0.51580E+00_JPRB, 0.49327E+00_JPRB ,&
&  0.47903E+00_JPRB, 0.48581E+00_JPRB, 0.48632E+00_JPRB, 0.48276E+00_JPRB, 0.48950E+00_JPRB, 0.49468E+00_JPRB, 0.47885E+00_JPRB ,&
&  0.48173E+00_JPRB, 0.48708E+00_JPRB, 0.50007E+00_JPRB, 0.48815E+00_JPRB, 0.47659E+00_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.47381E+00_JPRB, 0.45698E+00_JPRB, 0.44517E+00_JPRB, 0.43265E+00_JPRB, 0.41904E+00_JPRB, 0.40388E+00_JPRB, 0.39186E+00_JPRB ,&
&  0.37697E+00_JPRB, 0.37349E+00_JPRB, 0.37022E+00_JPRB, 0.36545E+00_JPRB, 0.36008E+00_JPRB, 0.36074E+00_JPRB, 0.36681E+00_JPRB ,&
&  0.37216E+00_JPRB, 0.37359E+00_JPRB, 0.37491E+00_JPRB, 0.37588E+00_JPRB, 0.37957E+00_JPRB, 0.38748E+00_JPRB, 0.39953E+00_JPRB ,&
&  0.41542E+00_JPRB, 0.43355E+00_JPRB, 0.45748E+00_JPRB, 0.48592E+00_JPRB, 0.51241E+00_JPRB, 0.54909E+00_JPRB, 0.59489E+00_JPRB ,&
&  0.67963E+00_JPRB, 0.67003E+00_JPRB, 0.46489E+00_JPRB, 0.73453E+00_JPRB, 0.91945E+00_JPRB, 0.88025E+00_JPRB, 0.83110E+00_JPRB ,&
&  0.80283E+00_JPRB, 0.73654E+00_JPRB, 0.71201E+00_JPRB, 0.65912E+00_JPRB, 0.59462E+00_JPRB, 0.57694E+00_JPRB, 0.56083E+00_JPRB ,&
&  0.53948E+00_JPRB, 0.52531E+00_JPRB, 0.51305E+00_JPRB, 0.50699E+00_JPRB, 0.49707E+00_JPRB, 0.49708E+00_JPRB, 0.52022E+00_JPRB ,&
&  0.55262E+00_JPRB, 0.60788E+00_JPRB, 0.66947E+00_JPRB, 0.63022E+00_JPRB, 0.55269E+00_JPRB, 0.52300E+00_JPRB, 0.51047E+00_JPRB ,&
&  0.50524E+00_JPRB, 0.52491E+00_JPRB, 0.52520E+00_JPRB, 0.50537E+00_JPRB, 0.48666E+00_JPRB, 0.47402E+00_JPRB, 0.47471E+00_JPRB ,&
&  0.46256E+00_JPRB, 0.45361E+00_JPRB, 0.44438E+00_JPRB, 0.43083E+00_JPRB, 0.42702E+00_JPRB, 0.42619E+00_JPRB, 0.43387E+00_JPRB ,&
&  0.44276E+00_JPRB, 0.44923E+00_JPRB, 0.45641E+00_JPRB, 0.46454E+00_JPRB, 0.46860E+00_JPRB, 0.47601E+00_JPRB, 0.48390E+00_JPRB ,&
&  0.49389E+00_JPRB, 0.51164E+00_JPRB, 0.53060E+00_JPRB, 0.54096E+00_JPRB, 0.54132E+00_JPRB, 0.53583E+00_JPRB, 0.52055E+00_JPRB ,&
&  0.51410E+00_JPRB, 0.51270E+00_JPRB, 0.50420E+00_JPRB, 0.49994E+00_JPRB, 0.52262E+00_JPRB, 0.53237E+00_JPRB, 0.54714E+00_JPRB ,&
&  0.58751E+00_JPRB, 0.62369E+00_JPRB, 0.63289E+00_JPRB, 0.52992E+00_JPRB, 0.49663E+00_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.45279E+00_JPRB, 0.42718E+00_JPRB, 0.40109E+00_JPRB, 0.38746E+00_JPRB, 0.37790E+00_JPRB, 0.37354E+00_JPRB, 0.37108E+00_JPRB ,&
&  0.37146E+00_JPRB, 0.36994E+00_JPRB, 0.36587E+00_JPRB, 0.36899E+00_JPRB, 0.37875E+00_JPRB, 0.38895E+00_JPRB, 0.40264E+00_JPRB ,&
&  0.40829E+00_JPRB, 0.41104E+00_JPRB, 0.41686E+00_JPRB, 0.43184E+00_JPRB, 0.45266E+00_JPRB, 0.47330E+00_JPRB, 0.49114E+00_JPRB ,&
&  0.51087E+00_JPRB, 0.53294E+00_JPRB, 0.56192E+00_JPRB, 0.59987E+00_JPRB, 0.65007E+00_JPRB, 0.71253E+00_JPRB, 0.77281E+00_JPRB ,&
&  0.82487E+00_JPRB, 0.65520E+00_JPRB, 0.52890E+00_JPRB, 0.83374E+00_JPRB, 0.89596E+00_JPRB, 0.84957E+00_JPRB, 0.74435E+00_JPRB ,&
&  0.71601E+00_JPRB, 0.70627E+00_JPRB, 0.67972E+00_JPRB, 0.65709E+00_JPRB, 0.60710E+00_JPRB, 0.59128E+00_JPRB, 0.57649E+00_JPRB ,&
&  0.56136E+00_JPRB, 0.54473E+00_JPRB, 0.53966E+00_JPRB, 0.53904E+00_JPRB, 0.53728E+00_JPRB, 0.54879E+00_JPRB, 0.57892E+00_JPRB ,&
&  0.62175E+00_JPRB, 0.67801E+00_JPRB, 0.73143E+00_JPRB, 0.64473E+00_JPRB, 0.58396E+00_JPRB, 0.53530E+00_JPRB, 0.52077E+00_JPRB ,&
&  0.51177E+00_JPRB, 0.53032E+00_JPRB, 0.53593E+00_JPRB, 0.51985E+00_JPRB, 0.50194E+00_JPRB, 0.47848E+00_JPRB, 0.46936E+00_JPRB ,&
&  0.45677E+00_JPRB, 0.44306E+00_JPRB, 0.43365E+00_JPRB, 0.42525E+00_JPRB, 0.42742E+00_JPRB, 0.43659E+00_JPRB, 0.44559E+00_JPRB ,&
&  0.45242E+00_JPRB, 0.45799E+00_JPRB, 0.46826E+00_JPRB, 0.48170E+00_JPRB, 0.49032E+00_JPRB, 0.49788E+00_JPRB, 0.50862E+00_JPRB ,&
&  0.51812E+00_JPRB, 0.52909E+00_JPRB, 0.54876E+00_JPRB, 0.55840E+00_JPRB, 0.55603E+00_JPRB, 0.53741E+00_JPRB, 0.53784E+00_JPRB ,&
&  0.53611E+00_JPRB, 0.54338E+00_JPRB, 0.54118E+00_JPRB, 0.55065E+00_JPRB, 0.55918E+00_JPRB, 0.58896E+00_JPRB, 0.62048E+00_JPRB ,&
&  0.64437E+00_JPRB, 0.64641E+00_JPRB, 0.61973E+00_JPRB, 0.51551E+00_JPRB, 0.47939E+00_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.38768E+00_JPRB, 0.36945E+00_JPRB, 0.37137E+00_JPRB, 0.37642E+00_JPRB, 0.38910E+00_JPRB, 0.39977E+00_JPRB, 0.39401E+00_JPRB ,&
&  0.39712E+00_JPRB, 0.40199E+00_JPRB, 0.41135E+00_JPRB, 0.42115E+00_JPRB, 0.43480E+00_JPRB, 0.44825E+00_JPRB, 0.45403E+00_JPRB ,&
&  0.46767E+00_JPRB, 0.48543E+00_JPRB, 0.50959E+00_JPRB, 0.53362E+00_JPRB, 0.55844E+00_JPRB, 0.57695E+00_JPRB, 0.59311E+00_JPRB ,&
&  0.61311E+00_JPRB, 0.64764E+00_JPRB, 0.69198E+00_JPRB, 0.74132E+00_JPRB, 0.79898E+00_JPRB, 0.86836E+00_JPRB, 0.90586E+00_JPRB ,&
&  0.72494E+00_JPRB, 0.47124E+00_JPRB, 0.86429E+00_JPRB, 0.94999E+00_JPRB, 0.83667E+00_JPRB, 0.77908E+00_JPRB, 0.72279E+00_JPRB ,&
&  0.67497E+00_JPRB, 0.67057E+00_JPRB, 0.64271E+00_JPRB, 0.64623E+00_JPRB, 0.60997E+00_JPRB, 0.59093E+00_JPRB, 0.58226E+00_JPRB ,&
&  0.57177E+00_JPRB, 0.56555E+00_JPRB, 0.57493E+00_JPRB, 0.58188E+00_JPRB, 0.59137E+00_JPRB, 0.60974E+00_JPRB, 0.64208E+00_JPRB ,&
&  0.68633E+00_JPRB, 0.73307E+00_JPRB, 0.76129E+00_JPRB, 0.64588E+00_JPRB, 0.60228E+00_JPRB, 0.56275E+00_JPRB, 0.53955E+00_JPRB ,&
&  0.54055E+00_JPRB, 0.53397E+00_JPRB, 0.52943E+00_JPRB, 0.53317E+00_JPRB, 0.51744E+00_JPRB, 0.47265E+00_JPRB, 0.46125E+00_JPRB ,&
&  0.44105E+00_JPRB, 0.43169E+00_JPRB, 0.42823E+00_JPRB, 0.42557E+00_JPRB, 0.42648E+00_JPRB, 0.43306E+00_JPRB, 0.43985E+00_JPRB ,&
&  0.45022E+00_JPRB, 0.45454E+00_JPRB, 0.46573E+00_JPRB, 0.47991E+00_JPRB, 0.49252E+00_JPRB, 0.50704E+00_JPRB, 0.52336E+00_JPRB ,&
&  0.53025E+00_JPRB, 0.54548E+00_JPRB, 0.55961E+00_JPRB, 0.56184E+00_JPRB, 0.56355E+00_JPRB, 0.55062E+00_JPRB, 0.54664E+00_JPRB ,&
&  0.54664E+00_JPRB, 0.54972E+00_JPRB, 0.54934E+00_JPRB, 0.55868E+00_JPRB, 0.57151E+00_JPRB, 0.58201E+00_JPRB, 0.60272E+00_JPRB ,&
&  0.59799E+00_JPRB, 0.57308E+00_JPRB, 0.51621E+00_JPRB, 0.45040E+00_JPRB, 0.41969E+00_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.38126E+00_JPRB, 0.39828E+00_JPRB, 0.41664E+00_JPRB, 0.43280E+00_JPRB, 0.44631E+00_JPRB, 0.45652E+00_JPRB, 0.47383E+00_JPRB ,&
&  0.49241E+00_JPRB, 0.50256E+00_JPRB, 0.50318E+00_JPRB, 0.51157E+00_JPRB, 0.52139E+00_JPRB, 0.53613E+00_JPRB, 0.54513E+00_JPRB ,&
&  0.55951E+00_JPRB, 0.58161E+00_JPRB, 0.60133E+00_JPRB, 0.62214E+00_JPRB, 0.64129E+00_JPRB, 0.66026E+00_JPRB, 0.68807E+00_JPRB ,&
&  0.72427E+00_JPRB, 0.77131E+00_JPRB, 0.82222E+00_JPRB, 0.86576E+00_JPRB, 0.90013E+00_JPRB, 0.94339E+00_JPRB, 0.94512E+00_JPRB ,&
&  0.65945E+00_JPRB, 0.84078E+00_JPRB, 0.93368E+00_JPRB, 0.85559E+00_JPRB, 0.78780E+00_JPRB, 0.74400E+00_JPRB, 0.71831E+00_JPRB ,&
&  0.67860E+00_JPRB, 0.63550E+00_JPRB, 0.61961E+00_JPRB, 0.64244E+00_JPRB, 0.60385E+00_JPRB, 0.59456E+00_JPRB, 0.59358E+00_JPRB ,&
&  0.59649E+00_JPRB, 0.60337E+00_JPRB, 0.61465E+00_JPRB, 0.62923E+00_JPRB, 0.65064E+00_JPRB, 0.67409E+00_JPRB, 0.70831E+00_JPRB ,&
&  0.74871E+00_JPRB, 0.77861E+00_JPRB, 0.79188E+00_JPRB, 0.67160E+00_JPRB, 0.63158E+00_JPRB, 0.59952E+00_JPRB, 0.57132E+00_JPRB ,&
&  0.55227E+00_JPRB, 0.55986E+00_JPRB, 0.55934E+00_JPRB, 0.56681E+00_JPRB, 0.52758E+00_JPRB, 0.48578E+00_JPRB, 0.44388E+00_JPRB ,&
&  0.42446E+00_JPRB, 0.40993E+00_JPRB, 0.40356E+00_JPRB, 0.39728E+00_JPRB, 0.39583E+00_JPRB, 0.40576E+00_JPRB, 0.41740E+00_JPRB ,&
&  0.42847E+00_JPRB, 0.42978E+00_JPRB, 0.44568E+00_JPRB, 0.45671E+00_JPRB, 0.49091E+00_JPRB, 0.52295E+00_JPRB, 0.53813E+00_JPRB ,&
&  0.54688E+00_JPRB, 0.54357E+00_JPRB, 0.54079E+00_JPRB, 0.53896E+00_JPRB, 0.53740E+00_JPRB, 0.53400E+00_JPRB, 0.53144E+00_JPRB ,&
&  0.52911E+00_JPRB, 0.54572E+00_JPRB, 0.54186E+00_JPRB, 0.54029E+00_JPRB, 0.55272E+00_JPRB, 0.53577E+00_JPRB, 0.52343E+00_JPRB ,&
&  0.50271E+00_JPRB, 0.45784E+00_JPRB, 0.42372E+00_JPRB, 0.39966E+00_JPRB, 0.38511E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.46360E+00_JPRB, 0.50262E+00_JPRB, 0.51767E+00_JPRB, 0.53579E+00_JPRB, 0.54667E+00_JPRB, 0.56148E+00_JPRB, 0.57203E+00_JPRB ,&
&  0.58123E+00_JPRB, 0.59300E+00_JPRB, 0.59457E+00_JPRB, 0.60912E+00_JPRB, 0.61987E+00_JPRB, 0.62903E+00_JPRB, 0.63364E+00_JPRB ,&
&  0.64102E+00_JPRB, 0.65700E+00_JPRB, 0.67478E+00_JPRB, 0.69638E+00_JPRB, 0.71714E+00_JPRB, 0.74512E+00_JPRB, 0.78460E+00_JPRB ,&
&  0.83173E+00_JPRB, 0.87488E+00_JPRB, 0.90951E+00_JPRB, 0.93467E+00_JPRB, 0.94171E+00_JPRB, 0.97187E+00_JPRB, 0.95180E+00_JPRB ,&
&  0.74214E+00_JPRB, 0.87348E+00_JPRB, 0.84414E+00_JPRB, 0.80262E+00_JPRB, 0.76457E+00_JPRB, 0.72569E+00_JPRB, 0.69603E+00_JPRB ,&
&  0.66653E+00_JPRB, 0.62261E+00_JPRB, 0.60940E+00_JPRB, 0.61232E+00_JPRB, 0.61409E+00_JPRB, 0.61056E+00_JPRB, 0.61469E+00_JPRB ,&
&  0.62541E+00_JPRB, 0.63720E+00_JPRB, 0.65225E+00_JPRB, 0.67039E+00_JPRB, 0.70250E+00_JPRB, 0.74528E+00_JPRB, 0.77475E+00_JPRB ,&
&  0.80736E+00_JPRB, 0.84019E+00_JPRB, 0.85364E+00_JPRB, 0.74762E+00_JPRB, 0.67338E+00_JPRB, 0.64430E+00_JPRB, 0.61045E+00_JPRB ,&
&  0.57941E+00_JPRB, 0.56749E+00_JPRB, 0.59472E+00_JPRB, 0.60594E+00_JPRB, 0.54767E+00_JPRB, 0.48108E+00_JPRB, 0.42809E+00_JPRB ,&
&  0.41024E+00_JPRB, 0.38330E+00_JPRB, 0.37055E+00_JPRB, 0.36905E+00_JPRB, 0.37197E+00_JPRB, 0.37812E+00_JPRB, 0.39557E+00_JPRB ,&
&  0.40722E+00_JPRB, 0.41994E+00_JPRB, 0.44055E+00_JPRB, 0.46791E+00_JPRB, 0.51151E+00_JPRB, 0.57347E+00_JPRB, 0.57858E+00_JPRB ,&
&  0.55753E+00_JPRB, 0.52960E+00_JPRB, 0.50648E+00_JPRB, 0.50890E+00_JPRB, 0.51050E+00_JPRB, 0.51598E+00_JPRB, 0.53351E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.58227E+00_JPRB, 0.55477E+00_JPRB, 0.53896E+00_JPRB, 0.54875E+00_JPRB, 0.52087E+00_JPRB, 0.46649E+00_JPRB ,&
&  0.44597E+00_JPRB, 0.42843E+00_JPRB, 0.41286E+00_JPRB, 0.41956E+00_JPRB, 0.43897E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.60717E+00_JPRB, 0.62998E+00_JPRB, 0.64408E+00_JPRB, 0.64170E+00_JPRB, 0.66263E+00_JPRB, 0.67321E+00_JPRB, 0.67616E+00_JPRB ,&
&  0.67890E+00_JPRB, 0.68832E+00_JPRB, 0.68993E+00_JPRB, 0.70437E+00_JPRB, 0.71349E+00_JPRB, 0.72338E+00_JPRB, 0.72678E+00_JPRB ,&
&  0.73197E+00_JPRB, 0.74622E+00_JPRB, 0.76539E+00_JPRB, 0.79527E+00_JPRB, 0.82351E+00_JPRB, 0.85917E+00_JPRB, 0.88715E+00_JPRB ,&
&  0.90231E+00_JPRB, 0.92751E+00_JPRB, 0.96280E+00_JPRB, 0.98005E+00_JPRB, 0.96045E+00_JPRB, 0.94063E+00_JPRB, 0.72590E+00_JPRB ,&
&  0.82118E+00_JPRB, 0.83101E+00_JPRB, 0.81501E+00_JPRB, 0.78654E+00_JPRB, 0.75179E+00_JPRB, 0.70563E+00_JPRB, 0.67046E+00_JPRB ,&
&  0.64718E+00_JPRB, 0.62469E+00_JPRB, 0.60637E+00_JPRB, 0.59156E+00_JPRB, 0.61768E+00_JPRB, 0.61449E+00_JPRB, 0.62179E+00_JPRB ,&
&  0.63674E+00_JPRB, 0.65863E+00_JPRB, 0.68138E+00_JPRB, 0.71892E+00_JPRB, 0.76454E+00_JPRB, 0.80659E+00_JPRB, 0.85070E+00_JPRB ,&
&  0.87838E+00_JPRB, 0.89949E+00_JPRB, 0.92295E+00_JPRB, 0.79512E+00_JPRB, 0.74382E+00_JPRB, 0.69074E+00_JPRB, 0.66808E+00_JPRB ,&
&  0.62386E+00_JPRB, 0.61278E+00_JPRB, 0.64992E+00_JPRB, 0.65466E+00_JPRB, 0.57933E+00_JPRB, 0.49523E+00_JPRB, 0.43452E+00_JPRB ,&
&  0.40783E+00_JPRB, 0.37209E+00_JPRB, 0.36427E+00_JPRB, 0.36211E+00_JPRB, 0.36910E+00_JPRB, 0.37589E+00_JPRB, 0.38492E+00_JPRB ,&
&  0.39750E+00_JPRB, 0.41993E+00_JPRB, 0.46280E+00_JPRB, 0.52439E+00_JPRB, 0.60052E+00_JPRB, 0.68104E+00_JPRB, 0.65180E+00_JPRB ,&
&  0.55359E+00_JPRB, 0.50031E+00_JPRB, 0.51562E+00_JPRB, 0.53343E+00_JPRB, 0.57183E+00_JPRB, 0.63206E+00_JPRB, 0.70643E+00_JPRB ,&
&  0.83192E+00_JPRB, 0.91178E+00_JPRB, 0.83177E+00_JPRB, 0.69857E+00_JPRB, 0.97763E+00_JPRB, 0.95682E+00_JPRB, 0.49139E+00_JPRB ,&
&  0.53466E+00_JPRB, 0.52554E+00_JPRB, 0.54542E+00_JPRB, 0.56865E+00_JPRB, 0.59752E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.72897E+00_JPRB, 0.74899E+00_JPRB, 0.75578E+00_JPRB, 0.75312E+00_JPRB, 0.76362E+00_JPRB, 0.78371E+00_JPRB, 0.78719E+00_JPRB ,&
&  0.79058E+00_JPRB, 0.78131E+00_JPRB, 0.78665E+00_JPRB, 0.79360E+00_JPRB, 0.80096E+00_JPRB, 0.80981E+00_JPRB, 0.81735E+00_JPRB ,&
&  0.81899E+00_JPRB, 0.84184E+00_JPRB, 0.84980E+00_JPRB, 0.87498E+00_JPRB, 0.90078E+00_JPRB, 0.93936E+00_JPRB, 0.95434E+00_JPRB ,&
&  0.94720E+00_JPRB, 0.96524E+00_JPRB, 0.98649E+00_JPRB, 0.10148E+01_JPRB, 0.99755E+00_JPRB, 0.99254E+00_JPRB, 0.78835E+00_JPRB ,&
&  0.85940E+00_JPRB, 0.82987E+00_JPRB, 0.77987E+00_JPRB, 0.75148E+00_JPRB, 0.71708E+00_JPRB, 0.68245E+00_JPRB, 0.65164E+00_JPRB ,&
&  0.63727E+00_JPRB, 0.62083E+00_JPRB, 0.59791E+00_JPRB, 0.60859E+00_JPRB, 0.60982E+00_JPRB, 0.60605E+00_JPRB, 0.62324E+00_JPRB ,&
&  0.63947E+00_JPRB, 0.67134E+00_JPRB, 0.70885E+00_JPRB, 0.75654E+00_JPRB, 0.80509E+00_JPRB, 0.85267E+00_JPRB, 0.90917E+00_JPRB ,&
&  0.96174E+00_JPRB, 0.10141E+01_JPRB, 0.10380E+01_JPRB, 0.92665E+00_JPRB, 0.91810E+00_JPRB, 0.82165E+00_JPRB, 0.73602E+00_JPRB ,&
&  0.64947E+00_JPRB, 0.61526E+00_JPRB, 0.64100E+00_JPRB, 0.69429E+00_JPRB, 0.59937E+00_JPRB, 0.52622E+00_JPRB, 0.45750E+00_JPRB ,&
&  0.42870E+00_JPRB, 0.39545E+00_JPRB, 0.38680E+00_JPRB, 0.37765E+00_JPRB, 0.37114E+00_JPRB, 0.37490E+00_JPRB, 0.38175E+00_JPRB ,&
&  0.38389E+00_JPRB, 0.39866E+00_JPRB, 0.43299E+00_JPRB, 0.46885E+00_JPRB, 0.49683E+00_JPRB, 0.50879E+00_JPRB, 0.48826E+00_JPRB ,&
&  0.52615E+00_JPRB, 0.53131E+00_JPRB, 0.53996E+00_JPRB, 0.59955E+00_JPRB, 0.70784E+00_JPRB, 0.87188E+00_JPRB, 0.10799E+01_JPRB ,&
&  0.13345E+01_JPRB, 0.14050E+01_JPRB, 0.17148E+01_JPRB, 0.22688E+01_JPRB, 0.20477E+01_JPRB, 0.17243E+01_JPRB, 0.60926E+00_JPRB ,&
&  0.63736E+00_JPRB, 0.64239E+00_JPRB, 0.66666E+00_JPRB, 0.68989E+00_JPRB, 0.71921E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.72880E+00_JPRB, 0.74538E+00_JPRB, 0.75728E+00_JPRB, 0.76450E+00_JPRB, 0.77731E+00_JPRB, 0.78977E+00_JPRB, 0.80124E+00_JPRB ,&
&  0.81084E+00_JPRB, 0.81635E+00_JPRB, 0.82623E+00_JPRB, 0.83484E+00_JPRB, 0.84581E+00_JPRB, 0.85941E+00_JPRB, 0.87583E+00_JPRB ,&
&  0.88668E+00_JPRB, 0.90254E+00_JPRB, 0.90735E+00_JPRB, 0.93292E+00_JPRB, 0.96792E+00_JPRB, 0.98022E+00_JPRB, 0.98326E+00_JPRB ,&
&  0.97258E+00_JPRB, 0.98191E+00_JPRB, 0.98967E+00_JPRB, 0.10059E+01_JPRB, 0.10061E+01_JPRB, 0.10307E+01_JPRB, 0.85463E+00_JPRB ,&
&  0.89642E+00_JPRB, 0.83949E+00_JPRB, 0.74031E+00_JPRB, 0.70519E+00_JPRB, 0.67374E+00_JPRB, 0.65719E+00_JPRB, 0.64253E+00_JPRB ,&
&  0.62950E+00_JPRB, 0.62064E+00_JPRB, 0.60464E+00_JPRB, 0.58987E+00_JPRB, 0.58828E+00_JPRB, 0.58705E+00_JPRB, 0.59897E+00_JPRB ,&
&  0.62338E+00_JPRB, 0.66168E+00_JPRB, 0.70672E+00_JPRB, 0.75832E+00_JPRB, 0.81877E+00_JPRB, 0.87402E+00_JPRB, 0.93336E+00_JPRB ,&
&  0.98598E+00_JPRB, 0.10378E+01_JPRB, 0.99546E+00_JPRB, 0.10213E+01_JPRB, 0.11073E+01_JPRB, 0.96662E+00_JPRB, 0.80059E+00_JPRB ,&
&  0.60490E+00_JPRB, 0.60446E+00_JPRB, 0.64747E+00_JPRB, 0.72986E+00_JPRB, 0.63745E+00_JPRB, 0.57672E+00_JPRB, 0.50490E+00_JPRB ,&
&  0.44477E+00_JPRB, 0.41361E+00_JPRB, 0.40643E+00_JPRB, 0.43274E+00_JPRB, 0.40727E+00_JPRB, 0.39208E+00_JPRB, 0.39640E+00_JPRB ,&
&  0.38238E+00_JPRB, 0.38239E+00_JPRB, 0.38859E+00_JPRB, 0.40171E+00_JPRB, 0.41190E+00_JPRB, 0.40622E+00_JPRB, 0.46327E+00_JPRB ,&
&  0.47687E+00_JPRB, 0.47634E+00_JPRB, 0.49814E+00_JPRB, 0.46167E+00_JPRB, 0.64262E+00_JPRB, 0.73625E+00_JPRB, 0.86950E+00_JPRB ,&
&  0.10105E+01_JPRB, 0.14063E+01_JPRB, 0.16612E+01_JPRB, 0.12870E+01_JPRB, 0.96913E+00_JPRB, 0.66919E+00_JPRB, 0.63151E+00_JPRB ,&
&  0.64395E+00_JPRB, 0.64594E+00_JPRB, 0.66210E+00_JPRB, 0.68335E+00_JPRB, 0.70679E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.72309E+00_JPRB, 0.74155E+00_JPRB, 0.75632E+00_JPRB, 0.76777E+00_JPRB, 0.78265E+00_JPRB, 0.79771E+00_JPRB, 0.81003E+00_JPRB ,&
&  0.82121E+00_JPRB, 0.83132E+00_JPRB, 0.84438E+00_JPRB, 0.85604E+00_JPRB, 0.87038E+00_JPRB, 0.88816E+00_JPRB, 0.90492E+00_JPRB ,&
&  0.92070E+00_JPRB, 0.92790E+00_JPRB, 0.95156E+00_JPRB, 0.98004E+00_JPRB, 0.99670E+00_JPRB, 0.99458E+00_JPRB, 0.98801E+00_JPRB ,&
&  0.98317E+00_JPRB, 0.99785E+00_JPRB, 0.10107E+01_JPRB, 0.10234E+01_JPRB, 0.10439E+01_JPRB, 0.10822E+01_JPRB, 0.87143E+00_JPRB ,&
&  0.92950E+00_JPRB, 0.84302E+00_JPRB, 0.70634E+00_JPRB, 0.64611E+00_JPRB, 0.61892E+00_JPRB, 0.60347E+00_JPRB, 0.59501E+00_JPRB ,&
&  0.59917E+00_JPRB, 0.57638E+00_JPRB, 0.56454E+00_JPRB, 0.55481E+00_JPRB, 0.55587E+00_JPRB, 0.55661E+00_JPRB, 0.56537E+00_JPRB ,&
&  0.58350E+00_JPRB, 0.61660E+00_JPRB, 0.65824E+00_JPRB, 0.70563E+00_JPRB, 0.76788E+00_JPRB, 0.82074E+00_JPRB, 0.86190E+00_JPRB ,&
&  0.88593E+00_JPRB, 0.89604E+00_JPRB, 0.82524E+00_JPRB, 0.90552E+00_JPRB, 0.10086E+01_JPRB, 0.98526E+00_JPRB, 0.88034E+00_JPRB ,&
&  0.71221E+00_JPRB, 0.61437E+00_JPRB, 0.63872E+00_JPRB, 0.74460E+00_JPRB, 0.70724E+00_JPRB, 0.62548E+00_JPRB, 0.56952E+00_JPRB ,&
&  0.48250E+00_JPRB, 0.45757E+00_JPRB, 0.44596E+00_JPRB, 0.48789E+00_JPRB, 0.47498E+00_JPRB, 0.43629E+00_JPRB, 0.42413E+00_JPRB ,&
&  0.38883E+00_JPRB, 0.35416E+00_JPRB, 0.33673E+00_JPRB, 0.34960E+00_JPRB, 0.38191E+00_JPRB, 0.36482E+00_JPRB, 0.42540E+00_JPRB ,&
&  0.41520E+00_JPRB, 0.37237E+00_JPRB, 0.40874E+00_JPRB, 0.48334E+00_JPRB, 0.52444E+00_JPRB, 0.57560E+00_JPRB, 0.68152E+00_JPRB ,&
&  0.84566E+00_JPRB, 0.10486E+01_JPRB, 0.10168E+01_JPRB, 0.80291E+00_JPRB, 0.63771E+00_JPRB, 0.60308E+00_JPRB, 0.61504E+00_JPRB ,&
&  0.63306E+00_JPRB, 0.64674E+00_JPRB, 0.66332E+00_JPRB, 0.68164E+00_JPRB, 0.70128E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.71025E+00_JPRB, 0.72826E+00_JPRB, 0.74406E+00_JPRB, 0.75858E+00_JPRB, 0.77332E+00_JPRB, 0.78700E+00_JPRB, 0.79836E+00_JPRB ,&
&  0.80876E+00_JPRB, 0.82189E+00_JPRB, 0.83326E+00_JPRB, 0.84502E+00_JPRB, 0.86264E+00_JPRB, 0.87809E+00_JPRB, 0.88677E+00_JPRB ,&
&  0.90315E+00_JPRB, 0.92377E+00_JPRB, 0.94989E+00_JPRB, 0.97052E+00_JPRB, 0.97845E+00_JPRB, 0.97525E+00_JPRB, 0.97037E+00_JPRB ,&
&  0.95703E+00_JPRB, 0.95003E+00_JPRB, 0.95328E+00_JPRB, 0.96471E+00_JPRB, 0.99007E+00_JPRB, 0.97270E+00_JPRB, 0.10369E+01_JPRB ,&
&  0.74666E+00_JPRB, 0.84709E+00_JPRB, 0.69697E+00_JPRB, 0.59121E+00_JPRB, 0.57930E+00_JPRB, 0.54286E+00_JPRB, 0.53742E+00_JPRB ,&
&  0.55010E+00_JPRB, 0.54844E+00_JPRB, 0.54158E+00_JPRB, 0.53397E+00_JPRB, 0.53162E+00_JPRB, 0.53807E+00_JPRB, 0.53565E+00_JPRB ,&
&  0.54509E+00_JPRB, 0.56264E+00_JPRB, 0.57234E+00_JPRB, 0.60120E+00_JPRB, 0.65464E+00_JPRB, 0.70327E+00_JPRB, 0.72707E+00_JPRB ,&
&  0.73216E+00_JPRB, 0.72408E+00_JPRB, 0.60020E+00_JPRB, 0.67595E+00_JPRB, 0.78055E+00_JPRB, 0.81065E+00_JPRB, 0.77187E+00_JPRB ,&
&  0.75170E+00_JPRB, 0.67462E+00_JPRB, 0.76323E+00_JPRB, 0.79533E+00_JPRB, 0.76970E+00_JPRB, 0.71475E+00_JPRB, 0.63992E+00_JPRB ,&
&  0.57071E+00_JPRB, 0.51478E+00_JPRB, 0.53051E+00_JPRB, 0.58448E+00_JPRB, 0.57467E+00_JPRB, 0.50315E+00_JPRB, 0.46798E+00_JPRB ,&
&  0.41351E+00_JPRB, 0.35748E+00_JPRB, 0.31832E+00_JPRB, 0.31582E+00_JPRB, 0.30789E+00_JPRB, 0.38502E+00_JPRB, 0.39952E+00_JPRB ,&
&  0.40972E+00_JPRB, 0.37534E+00_JPRB, 0.34454E+00_JPRB, 0.41988E+00_JPRB, 0.42495E+00_JPRB, 0.42358E+00_JPRB, 0.47818E+00_JPRB ,&
&  0.58942E+00_JPRB, 0.68301E+00_JPRB, 0.68686E+00_JPRB, 0.61954E+00_JPRB, 0.59341E+00_JPRB, 0.59761E+00_JPRB, 0.61375E+00_JPRB ,&
&  0.63056E+00_JPRB, 0.64583E+00_JPRB, 0.66066E+00_JPRB, 0.67653E+00_JPRB, 0.69250E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.68465E+00_JPRB, 0.70803E+00_JPRB, 0.72308E+00_JPRB, 0.73481E+00_JPRB, 0.73814E+00_JPRB, 0.74237E+00_JPRB, 0.75597E+00_JPRB ,&
&  0.76607E+00_JPRB, 0.77713E+00_JPRB, 0.78964E+00_JPRB, 0.80899E+00_JPRB, 0.83446E+00_JPRB, 0.85321E+00_JPRB, 0.85964E+00_JPRB ,&
&  0.86410E+00_JPRB, 0.89321E+00_JPRB, 0.91192E+00_JPRB, 0.94758E+00_JPRB, 0.96027E+00_JPRB, 0.98252E+00_JPRB, 0.98819E+00_JPRB ,&
&  0.98498E+00_JPRB, 0.98973E+00_JPRB, 0.99864E+00_JPRB, 0.10199E+01_JPRB, 0.10400E+01_JPRB, 0.11437E+01_JPRB, 0.12976E+01_JPRB ,&
&  0.98108E+00_JPRB, 0.79824E+00_JPRB, 0.68227E+00_JPRB, 0.54682E+00_JPRB, 0.53144E+00_JPRB, 0.52803E+00_JPRB, 0.51888E+00_JPRB ,&
&  0.52580E+00_JPRB, 0.53508E+00_JPRB, 0.53671E+00_JPRB, 0.52283E+00_JPRB, 0.52703E+00_JPRB, 0.54760E+00_JPRB, 0.53824E+00_JPRB ,&
&  0.53292E+00_JPRB, 0.53199E+00_JPRB, 0.52438E+00_JPRB, 0.56586E+00_JPRB, 0.54324E+00_JPRB, 0.58871E+00_JPRB, 0.61979E+00_JPRB ,&
&  0.63418E+00_JPRB, 0.59695E+00_JPRB, 0.48343E+00_JPRB, 0.51730E+00_JPRB, 0.60903E+00_JPRB, 0.65517E+00_JPRB, 0.66216E+00_JPRB ,&
&  0.74313E+00_JPRB, 0.75828E+00_JPRB, 0.65550E+00_JPRB, 0.80412E+00_JPRB, 0.84946E+00_JPRB, 0.85452E+00_JPRB, 0.79342E+00_JPRB ,&
&  0.73326E+00_JPRB, 0.64619E+00_JPRB, 0.66246E+00_JPRB, 0.67566E+00_JPRB, 0.66449E+00_JPRB, 0.62521E+00_JPRB, 0.54456E+00_JPRB ,&
&  0.46902E+00_JPRB, 0.40163E+00_JPRB, 0.34822E+00_JPRB, 0.32989E+00_JPRB, 0.35709E+00_JPRB, 0.35105E+00_JPRB, 0.37596E+00_JPRB ,&
&  0.35138E+00_JPRB, 0.36570E+00_JPRB, 0.32037E+00_JPRB, 0.37042E+00_JPRB, 0.37870E+00_JPRB, 0.36699E+00_JPRB, 0.39758E+00_JPRB ,&
&  0.45318E+00_JPRB, 0.49684E+00_JPRB, 0.52656E+00_JPRB, 0.54724E+00_JPRB, 0.56485E+00_JPRB, 0.58592E+00_JPRB, 0.59472E+00_JPRB ,&
&  0.60885E+00_JPRB, 0.62421E+00_JPRB, 0.63913E+00_JPRB, 0.66063E+00_JPRB, 0.66959E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.58506E+00_JPRB, 0.59421E+00_JPRB, 0.60834E+00_JPRB, 0.61649E+00_JPRB, 0.62445E+00_JPRB, 0.62915E+00_JPRB, 0.65184E+00_JPRB ,&
&  0.66501E+00_JPRB, 0.67772E+00_JPRB, 0.69762E+00_JPRB, 0.71773E+00_JPRB, 0.72655E+00_JPRB, 0.73666E+00_JPRB, 0.73961E+00_JPRB ,&
&  0.75268E+00_JPRB, 0.76818E+00_JPRB, 0.76799E+00_JPRB, 0.77516E+00_JPRB, 0.78247E+00_JPRB, 0.84067E+00_JPRB, 0.84195E+00_JPRB ,&
&  0.83286E+00_JPRB, 0.84141E+00_JPRB, 0.85297E+00_JPRB, 0.89333E+00_JPRB, 0.10131E+01_JPRB, 0.10745E+01_JPRB, 0.10525E+01_JPRB ,&
&  0.91492E+00_JPRB, 0.79817E+00_JPRB, 0.86381E+00_JPRB, 0.71315E+00_JPRB, 0.67391E+00_JPRB, 0.66199E+00_JPRB, 0.61678E+00_JPRB ,&
&  0.60542E+00_JPRB, 0.60654E+00_JPRB, 0.62296E+00_JPRB, 0.60476E+00_JPRB, 0.60446E+00_JPRB, 0.59259E+00_JPRB, 0.56253E+00_JPRB ,&
&  0.55153E+00_JPRB, 0.54642E+00_JPRB, 0.54376E+00_JPRB, 0.58027E+00_JPRB, 0.58269E+00_JPRB, 0.60818E+00_JPRB, 0.60773E+00_JPRB ,&
&  0.60262E+00_JPRB, 0.60754E+00_JPRB, 0.57272E+00_JPRB, 0.59151E+00_JPRB, 0.67760E+00_JPRB, 0.70364E+00_JPRB, 0.70517E+00_JPRB ,&
&  0.75540E+00_JPRB, 0.73160E+00_JPRB, 0.52723E+00_JPRB, 0.74472E+00_JPRB, 0.86224E+00_JPRB, 0.88095E+00_JPRB, 0.89343E+00_JPRB ,&
&  0.85401E+00_JPRB, 0.81412E+00_JPRB, 0.78955E+00_JPRB, 0.79089E+00_JPRB, 0.75070E+00_JPRB, 0.69993E+00_JPRB, 0.64545E+00_JPRB ,&
&  0.54681E+00_JPRB, 0.44697E+00_JPRB, 0.36861E+00_JPRB, 0.33649E+00_JPRB, 0.32382E+00_JPRB, 0.36311E+00_JPRB, 0.34270E+00_JPRB ,&
&  0.32223E+00_JPRB, 0.30641E+00_JPRB, 0.33738E+00_JPRB, 0.32571E+00_JPRB, 0.32103E+00_JPRB, 0.35151E+00_JPRB, 0.35619E+00_JPRB ,&
&  0.37753E+00_JPRB, 0.40312E+00_JPRB, 0.42740E+00_JPRB, 0.44593E+00_JPRB, 0.46839E+00_JPRB, 0.48792E+00_JPRB, 0.50321E+00_JPRB ,&
&  0.51820E+00_JPRB, 0.53663E+00_JPRB, 0.55229E+00_JPRB, 0.56964E+00_JPRB, 0.58341E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.57596E+00_JPRB, 0.58741E+00_JPRB, 0.58477E+00_JPRB, 0.58699E+00_JPRB, 0.57960E+00_JPRB, 0.57793E+00_JPRB, 0.56357E+00_JPRB ,&
&  0.55066E+00_JPRB, 0.55594E+00_JPRB, 0.57916E+00_JPRB, 0.62242E+00_JPRB, 0.62910E+00_JPRB, 0.64241E+00_JPRB, 0.63997E+00_JPRB ,&
&  0.65608E+00_JPRB, 0.65159E+00_JPRB, 0.65346E+00_JPRB, 0.64414E+00_JPRB, 0.65829E+00_JPRB, 0.69750E+00_JPRB, 0.77622E+00_JPRB ,&
&  0.88253E+00_JPRB, 0.10084E+01_JPRB, 0.11963E+01_JPRB, 0.14013E+01_JPRB, 0.18339E+01_JPRB, 0.14624E+01_JPRB, 0.10607E+01_JPRB ,&
&  0.10461E+01_JPRB, 0.98819E+00_JPRB, 0.10030E+01_JPRB, 0.10052E+01_JPRB, 0.97625E+00_JPRB, 0.92402E+00_JPRB, 0.93896E+00_JPRB ,&
&  0.94632E+00_JPRB, 0.96922E+00_JPRB, 0.99278E+00_JPRB, 0.99251E+00_JPRB, 0.99144E+00_JPRB, 0.96464E+00_JPRB, 0.91546E+00_JPRB ,&
&  0.84826E+00_JPRB, 0.85526E+00_JPRB, 0.85320E+00_JPRB, 0.88026E+00_JPRB, 0.95269E+00_JPRB, 0.94925E+00_JPRB, 0.93651E+00_JPRB ,&
&  0.91465E+00_JPRB, 0.94188E+00_JPRB, 0.10280E+01_JPRB, 0.10386E+01_JPRB, 0.10480E+01_JPRB, 0.98652E+00_JPRB, 0.91782E+00_JPRB ,&
&  0.86192E+00_JPRB, 0.74131E+00_JPRB, 0.58385E+00_JPRB, 0.84445E+00_JPRB, 0.94391E+00_JPRB, 0.86718E+00_JPRB, 0.97073E+00_JPRB ,&
&  0.93526E+00_JPRB, 0.90686E+00_JPRB, 0.91967E+00_JPRB, 0.89844E+00_JPRB, 0.81865E+00_JPRB, 0.76151E+00_JPRB, 0.65518E+00_JPRB ,&
&  0.61433E+00_JPRB, 0.50648E+00_JPRB, 0.41124E+00_JPRB, 0.35684E+00_JPRB, 0.33226E+00_JPRB, 0.34132E+00_JPRB, 0.34454E+00_JPRB ,&
&  0.34149E+00_JPRB, 0.30184E+00_JPRB, 0.32312E+00_JPRB, 0.37509E+00_JPRB, 0.48332E+00_JPRB, 0.38010E+00_JPRB, 0.35085E+00_JPRB ,&
&  0.37461E+00_JPRB, 0.39544E+00_JPRB, 0.41626E+00_JPRB, 0.42915E+00_JPRB, 0.43496E+00_JPRB, 0.44199E+00_JPRB, 0.47069E+00_JPRB ,&
&  0.49394E+00_JPRB, 0.51489E+00_JPRB, 0.53263E+00_JPRB, 0.55689E+00_JPRB, 0.56993E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.73343E+00_JPRB, 0.75448E+00_JPRB, 0.76822E+00_JPRB, 0.77047E+00_JPRB, 0.76069E+00_JPRB, 0.75030E+00_JPRB, 0.72878E+00_JPRB ,&
&  0.70395E+00_JPRB, 0.71141E+00_JPRB, 0.74312E+00_JPRB, 0.81596E+00_JPRB, 0.86227E+00_JPRB, 0.88947E+00_JPRB, 0.93268E+00_JPRB ,&
&  0.97515E+00_JPRB, 0.99609E+00_JPRB, 0.99955E+00_JPRB, 0.10302E+01_JPRB, 0.10655E+01_JPRB, 0.11115E+01_JPRB, 0.12853E+01_JPRB ,&
&  0.15276E+01_JPRB, 0.18207E+01_JPRB, 0.21232E+01_JPRB, 0.23483E+01_JPRB, 0.19298E+01_JPRB, 0.11691E+01_JPRB, 0.11477E+01_JPRB ,&
&  0.12299E+01_JPRB, 0.12701E+01_JPRB, 0.12570E+01_JPRB, 0.12735E+01_JPRB, 0.12947E+01_JPRB, 0.13533E+01_JPRB, 0.14279E+01_JPRB ,&
&  0.14757E+01_JPRB, 0.15055E+01_JPRB, 0.15374E+01_JPRB, 0.15560E+01_JPRB, 0.15855E+01_JPRB, 0.16103E+01_JPRB, 0.16304E+01_JPRB ,&
&  0.16098E+01_JPRB, 0.15589E+01_JPRB, 0.15253E+01_JPRB, 0.15514E+01_JPRB, 0.15118E+01_JPRB, 0.14553E+01_JPRB, 0.13978E+01_JPRB ,&
&  0.13536E+01_JPRB, 0.13433E+01_JPRB, 0.14356E+01_JPRB, 0.14693E+01_JPRB, 0.13613E+01_JPRB, 0.12455E+01_JPRB, 0.11819E+01_JPRB ,&
&  0.11114E+01_JPRB, 0.10197E+01_JPRB, 0.79770E+00_JPRB, 0.11204E+01_JPRB, 0.11845E+01_JPRB, 0.11251E+01_JPRB, 0.10497E+01_JPRB ,&
&  0.10108E+01_JPRB, 0.10084E+01_JPRB, 0.99091E+00_JPRB, 0.98344E+00_JPRB, 0.88127E+00_JPRB, 0.82797E+00_JPRB, 0.63096E+00_JPRB ,&
&  0.64273E+00_JPRB, 0.55809E+00_JPRB, 0.45945E+00_JPRB, 0.39259E+00_JPRB, 0.37303E+00_JPRB, 0.36798E+00_JPRB, 0.37412E+00_JPRB ,&
&  0.34146E+00_JPRB, 0.35042E+00_JPRB, 0.43690E+00_JPRB, 0.79090E+00_JPRB, 0.18512E+01_JPRB, 0.47716E+00_JPRB, 0.38025E+00_JPRB ,&
&  0.40875E+00_JPRB, 0.44419E+00_JPRB, 0.47797E+00_JPRB, 0.49878E+00_JPRB, 0.51046E+00_JPRB, 0.53045E+00_JPRB, 0.56819E+00_JPRB ,&
&  0.61023E+00_JPRB, 0.63852E+00_JPRB, 0.65093E+00_JPRB, 0.68465E+00_JPRB, 0.71166E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.85797E+00_JPRB, 0.88354E+00_JPRB, 0.90561E+00_JPRB, 0.93928E+00_JPRB, 0.97034E+00_JPRB, 0.97740E+00_JPRB, 0.97055E+00_JPRB ,&
&  0.98381E+00_JPRB, 0.10156E+01_JPRB, 0.10490E+01_JPRB, 0.10774E+01_JPRB, 0.11164E+01_JPRB, 0.11493E+01_JPRB, 0.11845E+01_JPRB ,&
&  0.12224E+01_JPRB, 0.12790E+01_JPRB, 0.13727E+01_JPRB, 0.14730E+01_JPRB, 0.15542E+01_JPRB, 0.16842E+01_JPRB, 0.18236E+01_JPRB ,&
&  0.19601E+01_JPRB, 0.18374E+01_JPRB, 0.17915E+01_JPRB, 0.14336E+01_JPRB, 0.97689E+00_JPRB, 0.11859E+01_JPRB, 0.12415E+01_JPRB ,&
&  0.13037E+01_JPRB, 0.13800E+01_JPRB, 0.14559E+01_JPRB, 0.15258E+01_JPRB, 0.16356E+01_JPRB, 0.17159E+01_JPRB, 0.17549E+01_JPRB ,&
&  0.17942E+01_JPRB, 0.18319E+01_JPRB, 0.18709E+01_JPRB, 0.19212E+01_JPRB, 0.19873E+01_JPRB, 0.20502E+01_JPRB, 0.20797E+01_JPRB ,&
&  0.21168E+01_JPRB, 0.21071E+01_JPRB, 0.21074E+01_JPRB, 0.21024E+01_JPRB, 0.19554E+01_JPRB, 0.18349E+01_JPRB, 0.17384E+01_JPRB ,&
&  0.16807E+01_JPRB, 0.16625E+01_JPRB, 0.16921E+01_JPRB, 0.17194E+01_JPRB, 0.16837E+01_JPRB, 0.15218E+01_JPRB, 0.14005E+01_JPRB ,&
&  0.13172E+01_JPRB, 0.11839E+01_JPRB, 0.11294E+01_JPRB, 0.14092E+01_JPRB, 0.92338E+00_JPRB, 0.10447E+01_JPRB, 0.11349E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.11541E+01_JPRB, 0.11405E+01_JPRB, 0.10426E+01_JPRB, 0.90350E+00_JPRB, 0.62482E+00_JPRB, 0.61946E+00_JPRB ,&
&  0.67414E+00_JPRB, 0.59852E+00_JPRB, 0.50114E+00_JPRB, 0.41788E+00_JPRB, 0.41585E+00_JPRB, 0.40084E+00_JPRB, 0.40425E+00_JPRB ,&
&  0.37822E+00_JPRB, 0.39260E+00_JPRB, 0.44590E+00_JPRB, 0.62523E+00_JPRB, 0.90440E+00_JPRB, 0.46091E+00_JPRB, 0.41270E+00_JPRB ,&
&  0.45022E+00_JPRB, 0.51354E+00_JPRB, 0.58198E+00_JPRB, 0.63123E+00_JPRB, 0.65118E+00_JPRB, 0.66980E+00_JPRB, 0.69002E+00_JPRB ,&
&  0.71420E+00_JPRB, 0.75256E+00_JPRB, 0.80472E+00_JPRB, 0.82761E+00_JPRB, 0.84484E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.87806E+00_JPRB, 0.92668E+00_JPRB, 0.10011E+01_JPRB, 0.10585E+01_JPRB, 0.10910E+01_JPRB, 0.10947E+01_JPRB, 0.10734E+01_JPRB ,&
&  0.10766E+01_JPRB, 0.11067E+01_JPRB, 0.11689E+01_JPRB, 0.12210E+01_JPRB, 0.12735E+01_JPRB, 0.13222E+01_JPRB, 0.13716E+01_JPRB ,&
&  0.14169E+01_JPRB, 0.14586E+01_JPRB, 0.15252E+01_JPRB, 0.16062E+01_JPRB, 0.16749E+01_JPRB, 0.17937E+01_JPRB, 0.17700E+01_JPRB ,&
&  0.14365E+01_JPRB, 0.11440E+01_JPRB, 0.14165E+01_JPRB, 0.14543E+01_JPRB, 0.12746E+01_JPRB, 0.13399E+01_JPRB, 0.14292E+01_JPRB ,&
&  0.15410E+01_JPRB, 0.16194E+01_JPRB, 0.16887E+01_JPRB, 0.17340E+01_JPRB, 0.17668E+01_JPRB, 0.18249E+01_JPRB, 0.18692E+01_JPRB ,&
&  0.19257E+01_JPRB, 0.19965E+01_JPRB, 0.20700E+01_JPRB, 0.21675E+01_JPRB, 0.22694E+01_JPRB, 0.23833E+01_JPRB, 0.24585E+01_JPRB ,&
&  0.25150E+01_JPRB, 0.25304E+01_JPRB, 0.24935E+01_JPRB, 0.23877E+01_JPRB, 0.22261E+01_JPRB, 0.21038E+01_JPRB, 0.20023E+01_JPRB ,&
&  0.19596E+01_JPRB, 0.19455E+01_JPRB, 0.19827E+01_JPRB, 0.20050E+01_JPRB, 0.19758E+01_JPRB, 0.17315E+01_JPRB, 0.16032E+01_JPRB ,&
&  0.15086E+01_JPRB, 0.13620E+01_JPRB, 0.13660E+01_JPRB, 0.14372E+01_JPRB, 0.10606E+01_JPRB, 0.10789E+01_JPRB, 0.10085E+01_JPRB ,&
&  0.11739E+01_JPRB, 0.12147E+01_JPRB, 0.12244E+01_JPRB, 0.11687E+01_JPRB, 0.92879E+00_JPRB, 0.64765E+00_JPRB, 0.65665E+00_JPRB ,&
&  0.71076E+00_JPRB, 0.69353E+00_JPRB, 0.55005E+00_JPRB, 0.46610E+00_JPRB, 0.40103E+00_JPRB, 0.39705E+00_JPRB, 0.41091E+00_JPRB ,&
&  0.40155E+00_JPRB, 0.39803E+00_JPRB, 0.44552E+00_JPRB, 0.52865E+00_JPRB, 0.58933E+00_JPRB, 0.47149E+00_JPRB, 0.46153E+00_JPRB ,&
&  0.51267E+00_JPRB, 0.56270E+00_JPRB, 0.60639E+00_JPRB, 0.64072E+00_JPRB, 0.67557E+00_JPRB, 0.70971E+00_JPRB, 0.73446E+00_JPRB ,&
&  0.78089E+00_JPRB, 0.77276E+00_JPRB, 0.81005E+00_JPRB, 0.82943E+00_JPRB, 0.85760E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.86197E+00_JPRB, 0.92460E+00_JPRB, 0.99713E+00_JPRB, 0.10662E+01_JPRB, 0.10982E+01_JPRB, 0.11263E+01_JPRB, 0.11380E+01_JPRB ,&
&  0.11124E+01_JPRB, 0.10878E+01_JPRB, 0.11645E+01_JPRB, 0.12375E+01_JPRB, 0.13300E+01_JPRB, 0.14081E+01_JPRB, 0.14804E+01_JPRB ,&
&  0.15423E+01_JPRB, 0.16148E+01_JPRB, 0.17043E+01_JPRB, 0.18127E+01_JPRB, 0.18662E+01_JPRB, 0.19132E+01_JPRB, 0.16372E+01_JPRB ,&
&  0.11524E+01_JPRB, 0.12664E+01_JPRB, 0.17768E+01_JPRB, 0.16992E+01_JPRB, 0.15840E+01_JPRB, 0.16598E+01_JPRB, 0.17415E+01_JPRB ,&
&  0.18029E+01_JPRB, 0.18380E+01_JPRB, 0.18120E+01_JPRB, 0.18116E+01_JPRB, 0.17982E+01_JPRB, 0.18168E+01_JPRB, 0.18360E+01_JPRB ,&
&  0.18844E+01_JPRB, 0.19790E+01_JPRB, 0.20968E+01_JPRB, 0.22600E+01_JPRB, 0.24288E+01_JPRB, 0.25874E+01_JPRB, 0.26526E+01_JPRB ,&
&  0.26665E+01_JPRB, 0.26981E+01_JPRB, 0.26888E+01_JPRB, 0.24683E+01_JPRB, 0.23478E+01_JPRB, 0.23251E+01_JPRB, 0.23284E+01_JPRB ,&
&  0.23043E+01_JPRB, 0.23759E+01_JPRB, 0.23389E+01_JPRB, 0.21291E+01_JPRB, 0.18427E+01_JPRB, 0.18406E+01_JPRB, 0.15986E+01_JPRB ,&
&  0.15809E+01_JPRB, 0.14761E+01_JPRB, 0.15438E+01_JPRB, 0.12235E+01_JPRB, 0.11542E+01_JPRB, 0.11713E+01_JPRB, 0.11555E+01_JPRB ,&
&  0.11598E+01_JPRB, 0.12184E+01_JPRB, 0.12607E+01_JPRB, 0.10801E+01_JPRB, 0.10118E+01_JPRB, 0.73654E+00_JPRB, 0.78293E+00_JPRB ,&
&  0.76298E+00_JPRB, 0.84087E+00_JPRB, 0.59970E+00_JPRB, 0.45682E+00_JPRB, 0.37946E+00_JPRB, 0.37898E+00_JPRB, 0.40050E+00_JPRB ,&
&  0.40014E+00_JPRB, 0.41732E+00_JPRB, 0.44042E+00_JPRB, 0.46587E+00_JPRB, 0.47812E+00_JPRB, 0.46359E+00_JPRB, 0.47564E+00_JPRB ,&
&  0.52953E+00_JPRB, 0.57479E+00_JPRB, 0.58658E+00_JPRB, 0.60067E+00_JPRB, 0.62622E+00_JPRB, 0.67086E+00_JPRB, 0.70275E+00_JPRB ,&
&  0.73909E+00_JPRB, 0.75818E+00_JPRB, 0.78198E+00_JPRB, 0.79539E+00_JPRB, 0.81426E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.69738E+00_JPRB, 0.72996E+00_JPRB, 0.75349E+00_JPRB, 0.75294E+00_JPRB, 0.75617E+00_JPRB, 0.77297E+00_JPRB, 0.81604E+00_JPRB ,&
&  0.87068E+00_JPRB, 0.93525E+00_JPRB, 0.10098E+01_JPRB, 0.11012E+01_JPRB, 0.12177E+01_JPRB, 0.13191E+01_JPRB, 0.14358E+01_JPRB ,&
&  0.15523E+01_JPRB, 0.16687E+01_JPRB, 0.17932E+01_JPRB, 0.19233E+01_JPRB, 0.19825E+01_JPRB, 0.19634E+01_JPRB, 0.15517E+01_JPRB ,&
&  0.11027E+01_JPRB, 0.19248E+01_JPRB, 0.18667E+01_JPRB, 0.19524E+01_JPRB, 0.19635E+01_JPRB, 0.20237E+01_JPRB, 0.20200E+01_JPRB ,&
&  0.20234E+01_JPRB, 0.19394E+01_JPRB, 0.18674E+01_JPRB, 0.18296E+01_JPRB, 0.18242E+01_JPRB, 0.18366E+01_JPRB, 0.18201E+01_JPRB ,&
&  0.18134E+01_JPRB, 0.18634E+01_JPRB, 0.19748E+01_JPRB, 0.21464E+01_JPRB, 0.23580E+01_JPRB, 0.25870E+01_JPRB, 0.27248E+01_JPRB ,&
&  0.28172E+01_JPRB, 0.29089E+01_JPRB, 0.29189E+01_JPRB, 0.27214E+01_JPRB, 0.26022E+01_JPRB, 0.26652E+01_JPRB, 0.28531E+01_JPRB ,&
&  0.27995E+01_JPRB, 0.26945E+01_JPRB, 0.27707E+01_JPRB, 0.25620E+01_JPRB, 0.21941E+01_JPRB, 0.20333E+01_JPRB, 0.16504E+01_JPRB ,&
&  0.16616E+01_JPRB, 0.14874E+01_JPRB, 0.15266E+01_JPRB, 0.12284E+01_JPRB, 0.11848E+01_JPRB, 0.12663E+01_JPRB, 0.12621E+01_JPRB ,&
&  0.11813E+01_JPRB, 0.12544E+01_JPRB, 0.12780E+01_JPRB, 0.11330E+01_JPRB, 0.10464E+01_JPRB, 0.87984E+00_JPRB, 0.81716E+00_JPRB ,&
&  0.76991E+00_JPRB, 0.73771E+00_JPRB, 0.61206E+00_JPRB, 0.40309E+00_JPRB, 0.35855E+00_JPRB, 0.36206E+00_JPRB, 0.39857E+00_JPRB ,&
&  0.40775E+00_JPRB, 0.44131E+00_JPRB, 0.48395E+00_JPRB, 0.47450E+00_JPRB, 0.45896E+00_JPRB, 0.46334E+00_JPRB, 0.48570E+00_JPRB ,&
&  0.53797E+00_JPRB, 0.57976E+00_JPRB, 0.58533E+00_JPRB, 0.58014E+00_JPRB, 0.56604E+00_JPRB, 0.59856E+00_JPRB, 0.61967E+00_JPRB ,&
&  0.65083E+00_JPRB, 0.64927E+00_JPRB, 0.66865E+00_JPRB, 0.68469E+00_JPRB, 0.69064E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.61168E+00_JPRB, 0.63120E+00_JPRB, 0.66530E+00_JPRB, 0.67014E+00_JPRB, 0.65793E+00_JPRB, 0.65812E+00_JPRB, 0.67289E+00_JPRB ,&
&  0.69545E+00_JPRB, 0.72553E+00_JPRB, 0.77489E+00_JPRB, 0.84110E+00_JPRB, 0.94338E+00_JPRB, 0.10488E+01_JPRB, 0.11886E+01_JPRB ,&
&  0.13720E+01_JPRB, 0.15827E+01_JPRB, 0.18132E+01_JPRB, 0.20629E+01_JPRB, 0.22419E+01_JPRB, 0.22328E+01_JPRB, 0.14224E+01_JPRB ,&
&  0.13013E+01_JPRB, 0.19785E+01_JPRB, 0.20507E+01_JPRB, 0.22484E+01_JPRB, 0.24185E+01_JPRB, 0.24634E+01_JPRB, 0.22835E+01_JPRB ,&
&  0.21920E+01_JPRB, 0.20766E+01_JPRB, 0.20095E+01_JPRB, 0.19457E+01_JPRB, 0.19128E+01_JPRB, 0.19100E+01_JPRB, 0.18636E+01_JPRB ,&
&  0.18002E+01_JPRB, 0.17685E+01_JPRB, 0.18037E+01_JPRB, 0.19196E+01_JPRB, 0.21003E+01_JPRB, 0.22975E+01_JPRB, 0.25740E+01_JPRB ,&
&  0.28522E+01_JPRB, 0.30949E+01_JPRB, 0.33234E+01_JPRB, 0.30917E+01_JPRB, 0.30232E+01_JPRB, 0.30089E+01_JPRB, 0.31662E+01_JPRB ,&
&  0.31139E+01_JPRB, 0.29577E+01_JPRB, 0.32185E+01_JPRB, 0.33292E+01_JPRB, 0.29327E+01_JPRB, 0.24164E+01_JPRB, 0.18881E+01_JPRB ,&
&  0.17348E+01_JPRB, 0.16337E+01_JPRB, 0.15330E+01_JPRB, 0.13350E+01_JPRB, 0.12741E+01_JPRB, 0.13862E+01_JPRB, 0.15041E+01_JPRB ,&
&  0.14677E+01_JPRB, 0.14746E+01_JPRB, 0.14029E+01_JPRB, 0.12412E+01_JPRB, 0.11002E+01_JPRB, 0.96087E+00_JPRB, 0.91288E+00_JPRB ,&
&  0.83036E+00_JPRB, 0.78695E+00_JPRB, 0.62996E+00_JPRB, 0.39574E+00_JPRB, 0.33155E+00_JPRB, 0.29412E+00_JPRB, 0.33986E+00_JPRB ,&
&  0.39791E+00_JPRB, 0.43966E+00_JPRB, 0.45219E+00_JPRB, 0.54251E+00_JPRB, 0.48870E+00_JPRB, 0.47920E+00_JPRB, 0.49097E+00_JPRB ,&
&  0.53559E+00_JPRB, 0.57575E+00_JPRB, 0.59262E+00_JPRB, 0.57223E+00_JPRB, 0.54181E+00_JPRB, 0.54641E+00_JPRB, 0.55509E+00_JPRB ,&
&  0.57602E+00_JPRB, 0.58350E+00_JPRB, 0.57931E+00_JPRB, 0.59064E+00_JPRB, 0.60865E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.56479E+00_JPRB, 0.57841E+00_JPRB, 0.59195E+00_JPRB, 0.60375E+00_JPRB, 0.60835E+00_JPRB, 0.60409E+00_JPRB, 0.59687E+00_JPRB ,&
&  0.59791E+00_JPRB, 0.60868E+00_JPRB, 0.64260E+00_JPRB, 0.68314E+00_JPRB, 0.74434E+00_JPRB, 0.80756E+00_JPRB, 0.91057E+00_JPRB ,&
&  0.10953E+01_JPRB, 0.13472E+01_JPRB, 0.16946E+01_JPRB, 0.20887E+01_JPRB, 0.22444E+01_JPRB, 0.19875E+01_JPRB, 0.17496E+01_JPRB ,&
&  0.16832E+01_JPRB, 0.20742E+01_JPRB, 0.21615E+01_JPRB, 0.25480E+01_JPRB, 0.29193E+01_JPRB, 0.30642E+01_JPRB, 0.24770E+01_JPRB ,&
&  0.23123E+01_JPRB, 0.21648E+01_JPRB, 0.20186E+01_JPRB, 0.19666E+01_JPRB, 0.19453E+01_JPRB, 0.19603E+01_JPRB, 0.19469E+01_JPRB ,&
&  0.18741E+01_JPRB, 0.17820E+01_JPRB, 0.17303E+01_JPRB, 0.17448E+01_JPRB, 0.18549E+01_JPRB, 0.19917E+01_JPRB, 0.21871E+01_JPRB ,&
&  0.25355E+01_JPRB, 0.30082E+01_JPRB, 0.34868E+01_JPRB, 0.35331E+01_JPRB, 0.33631E+01_JPRB, 0.33324E+01_JPRB, 0.33653E+01_JPRB ,&
&  0.34381E+01_JPRB, 0.37223E+01_JPRB, 0.37968E+01_JPRB, 0.36951E+01_JPRB, 0.32631E+01_JPRB, 0.29027E+01_JPRB, 0.22857E+01_JPRB ,&
&  0.19308E+01_JPRB, 0.17237E+01_JPRB, 0.15590E+01_JPRB, 0.15340E+01_JPRB, 0.15053E+01_JPRB, 0.15696E+01_JPRB, 0.14682E+01_JPRB ,&
&  0.12751E+01_JPRB, 0.11270E+01_JPRB, 0.10701E+01_JPRB, 0.11289E+01_JPRB, 0.11721E+01_JPRB, 0.11162E+01_JPRB, 0.10975E+01_JPRB ,&
&  0.81742E+00_JPRB, 0.36013E+00_JPRB, 0.24328E+00_JPRB, 0.29579E+00_JPRB, 0.22880E+00_JPRB, 0.20631E+00_JPRB, 0.29082E+00_JPRB ,&
&  0.37319E+00_JPRB, 0.45143E+00_JPRB, 0.49388E+00_JPRB, 0.52229E+00_JPRB, 0.56460E+00_JPRB, 0.54048E+00_JPRB, 0.52675E+00_JPRB ,&
&  0.54163E+00_JPRB, 0.57231E+00_JPRB, 0.60816E+00_JPRB, 0.59964E+00_JPRB, 0.55738E+00_JPRB, 0.54089E+00_JPRB, 0.52646E+00_JPRB ,&
&  0.52728E+00_JPRB, 0.55046E+00_JPRB, 0.56411E+00_JPRB, 0.56465E+00_JPRB, 0.56026E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.56623E+00_JPRB, 0.56522E+00_JPRB, 0.57193E+00_JPRB, 0.56011E+00_JPRB, 0.54050E+00_JPRB, 0.53044E+00_JPRB, 0.53908E+00_JPRB ,&
&  0.53489E+00_JPRB, 0.53461E+00_JPRB, 0.55828E+00_JPRB, 0.59344E+00_JPRB, 0.64145E+00_JPRB, 0.69347E+00_JPRB, 0.74465E+00_JPRB ,&
&  0.86239E+00_JPRB, 0.10847E+01_JPRB, 0.14623E+01_JPRB, 0.20028E+01_JPRB, 0.23237E+01_JPRB, 0.21606E+01_JPRB, 0.20784E+01_JPRB ,&
&  0.18698E+01_JPRB, 0.22468E+01_JPRB, 0.27708E+01_JPRB, 0.35631E+01_JPRB, 0.37849E+01_JPRB, 0.35514E+01_JPRB, 0.28028E+01_JPRB ,&
&  0.26497E+01_JPRB, 0.24150E+01_JPRB, 0.21704E+01_JPRB, 0.19313E+01_JPRB, 0.17972E+01_JPRB, 0.17767E+01_JPRB, 0.18197E+01_JPRB ,&
&  0.18401E+01_JPRB, 0.17786E+01_JPRB, 0.17030E+01_JPRB, 0.16716E+01_JPRB, 0.17136E+01_JPRB, 0.18122E+01_JPRB, 0.19363E+01_JPRB ,&
&  0.21217E+01_JPRB, 0.24832E+01_JPRB, 0.32374E+01_JPRB, 0.39485E+01_JPRB, 0.32128E+01_JPRB, 0.36857E+01_JPRB, 0.35132E+01_JPRB ,&
&  0.34919E+01_JPRB, 0.38517E+01_JPRB, 0.39010E+01_JPRB, 0.44173E+01_JPRB, 0.36310E+01_JPRB, 0.30527E+01_JPRB, 0.24928E+01_JPRB ,&
&  0.21522E+01_JPRB, 0.17452E+01_JPRB, 0.16280E+01_JPRB, 0.16625E+01_JPRB, 0.16994E+01_JPRB, 0.16936E+01_JPRB, 0.11614E+01_JPRB ,&
&  0.10878E+01_JPRB, 0.12037E+01_JPRB, 0.11221E+01_JPRB, 0.98012E+00_JPRB, 0.11679E+01_JPRB, 0.12788E+01_JPRB, 0.86283E+00_JPRB ,&
&  0.35538E+00_JPRB, 0.25889E+00_JPRB, 0.22226E+00_JPRB, 0.21023E+00_JPRB, 0.18569E+00_JPRB, 0.19515E+00_JPRB, 0.38451E+00_JPRB ,&
&  0.40532E+00_JPRB, 0.50117E+00_JPRB, 0.54804E+00_JPRB, 0.58244E+00_JPRB, 0.65981E+00_JPRB, 0.62899E+00_JPRB, 0.99090E+00_JPRB ,&
&  0.78446E+00_JPRB, 0.69017E+00_JPRB, 0.67029E+00_JPRB, 0.66598E+00_JPRB, 0.62245E+00_JPRB, 0.59673E+00_JPRB, 0.56664E+00_JPRB ,&
&  0.55637E+00_JPRB, 0.56240E+00_JPRB, 0.56135E+00_JPRB, 0.55849E+00_JPRB, 0.56973E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.58334E+00_JPRB, 0.57859E+00_JPRB, 0.56494E+00_JPRB, 0.53077E+00_JPRB, 0.50389E+00_JPRB, 0.48901E+00_JPRB, 0.48293E+00_JPRB ,&
&  0.48048E+00_JPRB, 0.48224E+00_JPRB, 0.49247E+00_JPRB, 0.51726E+00_JPRB, 0.54520E+00_JPRB, 0.58679E+00_JPRB, 0.64504E+00_JPRB ,&
&  0.74967E+00_JPRB, 0.95109E+00_JPRB, 0.13368E+01_JPRB, 0.17573E+01_JPRB, 0.22086E+01_JPRB, 0.21715E+01_JPRB, 0.22792E+01_JPRB ,&
&  0.20438E+01_JPRB, 0.22679E+01_JPRB, 0.29429E+01_JPRB, 0.40583E+01_JPRB, 0.56426E+01_JPRB, 0.56196E+01_JPRB, 0.48371E+01_JPRB ,&
&  0.41430E+01_JPRB, 0.36214E+01_JPRB, 0.32052E+01_JPRB, 0.27474E+01_JPRB, 0.20792E+01_JPRB, 0.16911E+01_JPRB, 0.15992E+01_JPRB ,&
&  0.15800E+01_JPRB, 0.16176E+01_JPRB, 0.15798E+01_JPRB, 0.15641E+01_JPRB, 0.16306E+01_JPRB, 0.17268E+01_JPRB, 0.18535E+01_JPRB ,&
&  0.19399E+01_JPRB, 0.20764E+01_JPRB, 0.25792E+01_JPRB, 0.36624E+01_JPRB, 0.37379E+01_JPRB, 0.34854E+01_JPRB, 0.34526E+01_JPRB ,&
&  0.34171E+01_JPRB, 0.39344E+01_JPRB, 0.44537E+01_JPRB, 0.53436E+01_JPRB, 0.36989E+01_JPRB, 0.30213E+01_JPRB, 0.26674E+01_JPRB ,&
&  0.23480E+01_JPRB, 0.21148E+01_JPRB, 0.16409E+01_JPRB, 0.16095E+01_JPRB, 0.17395E+01_JPRB, 0.14202E+01_JPRB, 0.11954E+01_JPRB ,&
&  0.13234E+01_JPRB, 0.12099E+01_JPRB, 0.11185E+01_JPRB, 0.97405E+00_JPRB, 0.12310E+01_JPRB, 0.11068E+01_JPRB, 0.50321E+00_JPRB ,&
&  0.33419E+00_JPRB, 0.27523E+00_JPRB, 0.21296E+00_JPRB, 0.20318E+00_JPRB, 0.19060E+00_JPRB, 0.20634E+00_JPRB, 0.33998E+00_JPRB ,&
&  0.39379E+00_JPRB, 0.54263E+00_JPRB, 0.60969E+00_JPRB, 0.66528E+00_JPRB, 0.72664E+00_JPRB, 0.87913E+00_JPRB, 0.22646E+01_JPRB ,&
&  0.21201E+01_JPRB, 0.16512E+01_JPRB, 0.12616E+01_JPRB, 0.10147E+01_JPRB, 0.82156E+00_JPRB, 0.71900E+00_JPRB, 0.65331E+00_JPRB ,&
&  0.62138E+00_JPRB, 0.60393E+00_JPRB, 0.59243E+00_JPRB, 0.58277E+00_JPRB, 0.57547E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.59103E+00_JPRB, 0.59491E+00_JPRB, 0.58238E+00_JPRB, 0.52897E+00_JPRB, 0.49511E+00_JPRB, 0.49096E+00_JPRB, 0.47902E+00_JPRB ,&
&  0.46305E+00_JPRB, 0.45242E+00_JPRB, 0.45157E+00_JPRB, 0.46141E+00_JPRB, 0.46873E+00_JPRB, 0.48515E+00_JPRB, 0.54258E+00_JPRB ,&
&  0.67386E+00_JPRB, 0.90792E+00_JPRB, 0.12384E+01_JPRB, 0.15377E+01_JPRB, 0.18181E+01_JPRB, 0.18956E+01_JPRB, 0.22469E+01_JPRB ,&
&  0.22977E+01_JPRB, 0.23416E+01_JPRB, 0.28276E+01_JPRB, 0.41314E+01_JPRB, 0.62146E+01_JPRB, 0.74687E+01_JPRB, 0.77874E+01_JPRB ,&
&  0.66210E+01_JPRB, 0.53979E+01_JPRB, 0.45692E+01_JPRB, 0.39982E+01_JPRB, 0.32333E+01_JPRB, 0.23105E+01_JPRB, 0.17143E+01_JPRB ,&
&  0.14739E+01_JPRB, 0.14923E+01_JPRB, 0.15291E+01_JPRB, 0.15004E+01_JPRB, 0.15681E+01_JPRB, 0.16671E+01_JPRB, 0.18299E+01_JPRB ,&
&  0.19211E+01_JPRB, 0.19696E+01_JPRB, 0.21685E+01_JPRB, 0.29789E+01_JPRB, 0.40937E+01_JPRB, 0.43411E+01_JPRB, 0.36100E+01_JPRB ,&
&  0.31092E+01_JPRB, 0.32036E+01_JPRB, 0.49373E+01_JPRB, 0.54326E+01_JPRB, 0.37457E+01_JPRB, 0.31679E+01_JPRB, 0.27842E+01_JPRB ,&
&  0.25831E+01_JPRB, 0.23556E+01_JPRB, 0.18803E+01_JPRB, 0.17694E+01_JPRB, 0.15538E+01_JPRB, 0.12809E+01_JPRB, 0.14057E+01_JPRB ,&
&  0.14320E+01_JPRB, 0.13005E+01_JPRB, 0.11126E+01_JPRB, 0.94114E+00_JPRB, 0.10091E+01_JPRB, 0.76783E+00_JPRB, 0.53520E+00_JPRB ,&
&  0.43820E+00_JPRB, 0.34813E+00_JPRB, 0.28102E+00_JPRB, 0.27302E+00_JPRB, 0.23792E+00_JPRB, 0.24926E+00_JPRB, 0.31083E+00_JPRB ,&
&  0.42733E+00_JPRB, 0.52226E+00_JPRB, 0.65723E+00_JPRB, 0.76312E+00_JPRB, 0.77909E+00_JPRB, 0.73155E+00_JPRB, 0.19342E+01_JPRB ,&
&  0.15014E+01_JPRB, 0.14745E+01_JPRB, 0.15123E+01_JPRB, 0.12794E+01_JPRB, 0.10381E+01_JPRB, 0.87458E+00_JPRB, 0.75145E+00_JPRB ,&
&  0.67205E+00_JPRB, 0.62497E+00_JPRB, 0.59882E+00_JPRB, 0.59338E+00_JPRB, 0.59751E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.63960E+00_JPRB, 0.60488E+00_JPRB, 0.58577E+00_JPRB, 0.55024E+00_JPRB, 0.49923E+00_JPRB, 0.47863E+00_JPRB, 0.48570E+00_JPRB ,&
&  0.47529E+00_JPRB, 0.45687E+00_JPRB, 0.44052E+00_JPRB, 0.42755E+00_JPRB, 0.42948E+00_JPRB, 0.43927E+00_JPRB, 0.46819E+00_JPRB ,&
&  0.60798E+00_JPRB, 0.96634E+00_JPRB, 0.10206E+01_JPRB, 0.11677E+01_JPRB, 0.14229E+01_JPRB, 0.18318E+01_JPRB, 0.20409E+01_JPRB ,&
&  0.24639E+01_JPRB, 0.25456E+01_JPRB, 0.28094E+01_JPRB, 0.41759E+01_JPRB, 0.76142E+01_JPRB, 0.90924E+01_JPRB, 0.99548E+01_JPRB ,&
&  0.96965E+01_JPRB, 0.79674E+01_JPRB, 0.63075E+01_JPRB, 0.49285E+01_JPRB, 0.39996E+01_JPRB, 0.33600E+01_JPRB, 0.25215E+01_JPRB ,&
&  0.19039E+01_JPRB, 0.16544E+01_JPRB, 0.16250E+01_JPRB, 0.15983E+01_JPRB, 0.15910E+01_JPRB, 0.16410E+01_JPRB, 0.17681E+01_JPRB ,&
&  0.19073E+01_JPRB, 0.20128E+01_JPRB, 0.21237E+01_JPRB, 0.27104E+01_JPRB, 0.34114E+01_JPRB, 0.37402E+01_JPRB, 0.44217E+01_JPRB ,&
&  0.40326E+01_JPRB, 0.42035E+01_JPRB, 0.49285E+01_JPRB, 0.60096E+01_JPRB, 0.40384E+01_JPRB, 0.31503E+01_JPRB, 0.28590E+01_JPRB ,&
&  0.22532E+01_JPRB, 0.20181E+01_JPRB, 0.19454E+01_JPRB, 0.17133E+01_JPRB, 0.14167E+01_JPRB, 0.15287E+01_JPRB, 0.19542E+01_JPRB ,&
&  0.15198E+01_JPRB, 0.14580E+01_JPRB, 0.14076E+01_JPRB, 0.13290E+01_JPRB, 0.93994E+00_JPRB, 0.69637E+00_JPRB, 0.95980E+00_JPRB ,&
&  0.95875E+00_JPRB, 0.56393E+00_JPRB, 0.42456E+00_JPRB, 0.46513E+00_JPRB, 0.34615E+00_JPRB, 0.33823E+00_JPRB, 0.37266E+00_JPRB ,&
&  0.39898E+00_JPRB, 0.41901E+00_JPRB, 0.64546E+00_JPRB, 0.70553E+00_JPRB, 0.74446E+00_JPRB, 0.64653E+00_JPRB, 0.12441E+01_JPRB ,&
&  0.14029E+01_JPRB, 0.11974E+01_JPRB, 0.13194E+01_JPRB, 0.12512E+01_JPRB, 0.11095E+01_JPRB, 0.98514E+00_JPRB, 0.90047E+00_JPRB ,&
&  0.81513E+00_JPRB, 0.74512E+00_JPRB, 0.70419E+00_JPRB, 0.68355E+00_JPRB, 0.67677E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.60918E+00_JPRB, 0.53790E+00_JPRB, 0.50658E+00_JPRB, 0.50542E+00_JPRB, 0.50188E+00_JPRB, 0.46934E+00_JPRB, 0.46527E+00_JPRB ,&
&  0.48447E+00_JPRB, 0.46482E+00_JPRB, 0.43530E+00_JPRB, 0.42085E+00_JPRB, 0.42352E+00_JPRB, 0.42716E+00_JPRB, 0.42540E+00_JPRB ,&
&  0.50718E+00_JPRB, 0.72136E+00_JPRB, 0.83812E+00_JPRB, 0.97101E+00_JPRB, 0.12680E+01_JPRB, 0.16474E+01_JPRB, 0.18628E+01_JPRB ,&
&  0.23378E+01_JPRB, 0.26328E+01_JPRB, 0.31240E+01_JPRB, 0.46598E+01_JPRB, 0.67501E+01_JPRB, 0.87445E+01_JPRB, 0.94210E+01_JPRB ,&
&  0.10231E+02_JPRB, 0.96865E+01_JPRB, 0.76757E+01_JPRB, 0.62060E+01_JPRB, 0.52069E+01_JPRB, 0.45513E+01_JPRB, 0.38228E+01_JPRB ,&
&  0.29333E+01_JPRB, 0.22367E+01_JPRB, 0.19416E+01_JPRB, 0.18179E+01_JPRB, 0.17940E+01_JPRB, 0.17372E+01_JPRB, 0.17928E+01_JPRB ,&
&  0.19226E+01_JPRB, 0.20746E+01_JPRB, 0.21434E+01_JPRB, 0.26392E+01_JPRB, 0.34965E+01_JPRB, 0.39177E+01_JPRB, 0.43825E+01_JPRB ,&
&  0.44639E+01_JPRB, 0.41908E+01_JPRB, 0.48221E+01_JPRB, 0.50305E+01_JPRB, 0.43190E+01_JPRB, 0.34514E+01_JPRB, 0.32325E+01_JPRB ,&
&  0.25195E+01_JPRB, 0.21356E+01_JPRB, 0.19180E+01_JPRB, 0.15768E+01_JPRB, 0.15669E+01_JPRB, 0.19856E+01_JPRB, 0.20049E+01_JPRB ,&
&  0.17680E+01_JPRB, 0.16310E+01_JPRB, 0.14709E+01_JPRB, 0.12510E+01_JPRB, 0.11476E+01_JPRB, 0.92933E+00_JPRB, 0.10713E+01_JPRB ,&
&  0.11029E+01_JPRB, 0.10299E+01_JPRB, 0.84710E+00_JPRB, 0.69843E+00_JPRB, 0.53823E+00_JPRB, 0.49932E+00_JPRB, 0.43597E+00_JPRB ,&
&  0.41135E+00_JPRB, 0.39624E+00_JPRB, 0.50597E+00_JPRB, 0.60303E+00_JPRB, 0.63176E+00_JPRB, 0.59955E+00_JPRB, 0.90469E+00_JPRB ,&
&  0.10316E+01_JPRB, 0.11645E+01_JPRB, 0.11335E+01_JPRB, 0.10459E+01_JPRB, 0.95283E+00_JPRB, 0.89344E+00_JPRB, 0.87918E+00_JPRB ,&
&  0.82273E+00_JPRB, 0.78936E+00_JPRB, 0.76640E+00_JPRB, 0.73212E+00_JPRB, 0.68592E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.56003E+00_JPRB, 0.49362E+00_JPRB, 0.46154E+00_JPRB, 0.45320E+00_JPRB, 0.46117E+00_JPRB, 0.46351E+00_JPRB, 0.45353E+00_JPRB ,&
&  0.45115E+00_JPRB, 0.46328E+00_JPRB, 0.44885E+00_JPRB, 0.43478E+00_JPRB, 0.43046E+00_JPRB, 0.42886E+00_JPRB, 0.41492E+00_JPRB ,&
&  0.45162E+00_JPRB, 0.59183E+00_JPRB, 0.66772E+00_JPRB, 0.85294E+00_JPRB, 0.10968E+01_JPRB, 0.14760E+01_JPRB, 0.19083E+01_JPRB ,&
&  0.21319E+01_JPRB, 0.24259E+01_JPRB, 0.28796E+01_JPRB, 0.40205E+01_JPRB, 0.53701E+01_JPRB, 0.65478E+01_JPRB, 0.68116E+01_JPRB ,&
&  0.64296E+01_JPRB, 0.65642E+01_JPRB, 0.61576E+01_JPRB, 0.55133E+01_JPRB, 0.49781E+01_JPRB, 0.45657E+01_JPRB, 0.40420E+01_JPRB ,&
&  0.34119E+01_JPRB, 0.27705E+01_JPRB, 0.23197E+01_JPRB, 0.20855E+01_JPRB, 0.19935E+01_JPRB, 0.20633E+01_JPRB, 0.21149E+01_JPRB ,&
&  0.21102E+01_JPRB, 0.21657E+01_JPRB, 0.20966E+01_JPRB, 0.23547E+01_JPRB, 0.29632E+01_JPRB, 0.38902E+01_JPRB, 0.42691E+01_JPRB ,&
&  0.49541E+01_JPRB, 0.46595E+01_JPRB, 0.50571E+01_JPRB, 0.54580E+01_JPRB, 0.39786E+01_JPRB, 0.37557E+01_JPRB, 0.38647E+01_JPRB ,&
&  0.37017E+01_JPRB, 0.32290E+01_JPRB, 0.28586E+01_JPRB, 0.21376E+01_JPRB, 0.17434E+01_JPRB, 0.21380E+01_JPRB, 0.20329E+01_JPRB ,&
&  0.18342E+01_JPRB, 0.17336E+01_JPRB, 0.15654E+01_JPRB, 0.14925E+01_JPRB, 0.13063E+01_JPRB, 0.12296E+01_JPRB, 0.10294E+01_JPRB ,&
&  0.10172E+01_JPRB, 0.88667E+00_JPRB, 0.91001E+00_JPRB, 0.79646E+00_JPRB, 0.70149E+00_JPRB, 0.59058E+00_JPRB, 0.47454E+00_JPRB ,&
&  0.41685E+00_JPRB, 0.39582E+00_JPRB, 0.42581E+00_JPRB, 0.52538E+00_JPRB, 0.59249E+00_JPRB, 0.56165E+00_JPRB, 0.66232E+00_JPRB ,&
&  0.86153E+00_JPRB, 0.99669E+00_JPRB, 0.91338E+00_JPRB, 0.94312E+00_JPRB, 0.86160E+00_JPRB, 0.79418E+00_JPRB, 0.78717E+00_JPRB ,&
&  0.73793E+00_JPRB, 0.72177E+00_JPRB, 0.70649E+00_JPRB, 0.67763E+00_JPRB, 0.63280E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.51743E+00_JPRB, 0.46000E+00_JPRB, 0.42256E+00_JPRB, 0.41861E+00_JPRB, 0.43532E+00_JPRB, 0.44654E+00_JPRB, 0.45083E+00_JPRB ,&
&  0.44402E+00_JPRB, 0.45130E+00_JPRB, 0.45647E+00_JPRB, 0.45121E+00_JPRB, 0.44687E+00_JPRB, 0.44192E+00_JPRB, 0.41995E+00_JPRB ,&
&  0.41524E+00_JPRB, 0.54128E+00_JPRB, 0.57112E+00_JPRB, 0.65316E+00_JPRB, 0.83808E+00_JPRB, 0.12838E+01_JPRB, 0.17209E+01_JPRB ,&
&  0.19974E+01_JPRB, 0.22182E+01_JPRB, 0.26015E+01_JPRB, 0.31740E+01_JPRB, 0.37093E+01_JPRB, 0.39713E+01_JPRB, 0.40211E+01_JPRB ,&
&  0.43650E+01_JPRB, 0.36604E+01_JPRB, 0.37294E+01_JPRB, 0.33860E+01_JPRB, 0.32202E+01_JPRB, 0.30489E+01_JPRB, 0.28818E+01_JPRB ,&
&  0.26928E+01_JPRB, 0.24485E+01_JPRB, 0.21936E+01_JPRB, 0.20264E+01_JPRB, 0.19522E+01_JPRB, 0.19505E+01_JPRB, 0.20116E+01_JPRB ,&
&  0.20740E+01_JPRB, 0.20894E+01_JPRB, 0.20726E+01_JPRB, 0.19619E+01_JPRB, 0.23520E+01_JPRB, 0.32468E+01_JPRB, 0.41938E+01_JPRB ,&
&  0.41193E+01_JPRB, 0.36656E+01_JPRB, 0.54415E+01_JPRB, 0.56270E+01_JPRB, 0.48496E+01_JPRB, 0.45914E+01_JPRB, 0.44112E+01_JPRB ,&
&  0.42017E+01_JPRB, 0.36276E+01_JPRB, 0.29274E+01_JPRB, 0.22461E+01_JPRB, 0.23373E+01_JPRB, 0.23016E+01_JPRB, 0.21634E+01_JPRB ,&
&  0.19807E+01_JPRB, 0.18837E+01_JPRB, 0.17300E+01_JPRB, 0.15852E+01_JPRB, 0.14762E+01_JPRB, 0.14538E+01_JPRB, 0.13145E+01_JPRB ,&
&  0.11884E+01_JPRB, 0.11623E+01_JPRB, 0.99572E+00_JPRB, 0.81576E+00_JPRB, 0.70258E+00_JPRB, 0.60900E+00_JPRB, 0.51086E+00_JPRB ,&
&  0.42050E+00_JPRB, 0.38962E+00_JPRB, 0.39337E+00_JPRB, 0.44289E+00_JPRB, 0.52963E+00_JPRB, 0.51075E+00_JPRB, 0.55092E+00_JPRB ,&
&  0.59660E+00_JPRB, 0.82449E+00_JPRB, 0.83563E+00_JPRB, 0.82932E+00_JPRB, 0.81135E+00_JPRB, 0.74707E+00_JPRB, 0.69959E+00_JPRB ,&
&  0.64779E+00_JPRB, 0.64720E+00_JPRB, 0.64414E+00_JPRB, 0.61640E+00_JPRB, 0.58191E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.49334E+00_JPRB, 0.44144E+00_JPRB, 0.40584E+00_JPRB, 0.40329E+00_JPRB, 0.42818E+00_JPRB, 0.44113E+00_JPRB, 0.44719E+00_JPRB ,&
&  0.44628E+00_JPRB, 0.44624E+00_JPRB, 0.45840E+00_JPRB, 0.46425E+00_JPRB, 0.47004E+00_JPRB, 0.46284E+00_JPRB, 0.43502E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.50830E+00_JPRB, 0.49734E+00_JPRB, 0.57570E+00_JPRB, 0.69941E+00_JPRB, 0.99813E+00_JPRB, 0.12863E+01_JPRB ,&
&  0.15580E+01_JPRB, 0.16928E+01_JPRB, 0.18884E+01_JPRB, 0.21862E+01_JPRB, 0.23514E+01_JPRB, 0.22678E+01_JPRB, 0.21710E+01_JPRB ,&
&  0.20490E+01_JPRB, 0.20045E+01_JPRB, 0.17368E+01_JPRB, 0.18595E+01_JPRB, 0.18341E+01_JPRB, 0.16129E+01_JPRB, 0.18038E+01_JPRB ,&
&  0.18113E+01_JPRB, 0.18230E+01_JPRB, 0.17770E+01_JPRB, 0.17386E+01_JPRB, 0.17295E+01_JPRB, 0.17237E+01_JPRB, 0.17135E+01_JPRB ,&
&  0.17139E+01_JPRB, 0.17400E+01_JPRB, 0.18706E+01_JPRB, 0.18397E+01_JPRB, 0.18806E+01_JPRB, 0.23075E+01_JPRB, 0.34200E+01_JPRB ,&
&  0.45528E+01_JPRB, 0.45009E+01_JPRB, 0.43378E+01_JPRB, 0.53756E+01_JPRB, 0.56681E+01_JPRB, 0.51173E+01_JPRB, 0.45538E+01_JPRB ,&
&  0.43114E+01_JPRB, 0.38170E+01_JPRB, 0.33091E+01_JPRB, 0.28404E+01_JPRB, 0.27485E+01_JPRB, 0.27154E+01_JPRB, 0.24982E+01_JPRB ,&
&  0.21796E+01_JPRB, 0.19585E+01_JPRB, 0.18286E+01_JPRB, 0.16440E+01_JPRB, 0.14802E+01_JPRB, 0.13621E+01_JPRB, 0.13105E+01_JPRB ,&
&  0.12094E+01_JPRB, 0.10548E+01_JPRB, 0.84801E+00_JPRB, 0.74659E+00_JPRB, 0.62461E+00_JPRB, 0.54061E+00_JPRB, 0.49174E+00_JPRB ,&
&  0.41790E+00_JPRB, 0.36628E+00_JPRB, 0.36526E+00_JPRB, 0.38302E+00_JPRB, 0.45259E+00_JPRB, 0.44462E+00_JPRB, 0.48379E+00_JPRB ,&
&  0.49082E+00_JPRB, 0.50221E+00_JPRB, 0.67664E+00_JPRB, 0.74501E+00_JPRB, 0.74313E+00_JPRB, 0.67136E+00_JPRB, 0.60433E+00_JPRB ,&
&  0.57287E+00_JPRB, 0.59136E+00_JPRB, 0.58964E+00_JPRB, 0.57459E+00_JPRB, 0.55042E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.46848E+00_JPRB, 0.41840E+00_JPRB, 0.38775E+00_JPRB, 0.38053E+00_JPRB, 0.40033E+00_JPRB, 0.42204E+00_JPRB, 0.43462E+00_JPRB ,&
&  0.44626E+00_JPRB, 0.45455E+00_JPRB, 0.46182E+00_JPRB, 0.47073E+00_JPRB, 0.47924E+00_JPRB, 0.46755E+00_JPRB, 0.45815E+00_JPRB ,&
&  0.46244E+00_JPRB, 0.42444E+00_JPRB, 0.43139E+00_JPRB, 0.47206E+00_JPRB, 0.68230E+00_JPRB, 0.82821E+00_JPRB, 0.99199E+00_JPRB ,&
&  0.11267E+01_JPRB, 0.12388E+01_JPRB, 0.12716E+01_JPRB, 0.12901E+01_JPRB, 0.12948E+01_JPRB, 0.12914E+01_JPRB, 0.11858E+01_JPRB ,&
&  0.10284E+01_JPRB, 0.94016E+00_JPRB, 0.90819E+00_JPRB, 0.97172E+00_JPRB, 0.10791E+01_JPRB, 0.95716E+00_JPRB, 0.10805E+01_JPRB ,&
&  0.11629E+01_JPRB, 0.12447E+01_JPRB, 0.12851E+01_JPRB, 0.12955E+01_JPRB, 0.13195E+01_JPRB, 0.13534E+01_JPRB, 0.13895E+01_JPRB ,&
&  0.14332E+01_JPRB, 0.14394E+01_JPRB, 0.15011E+01_JPRB, 0.16924E+01_JPRB, 0.18199E+01_JPRB, 0.22557E+01_JPRB, 0.36059E+01_JPRB ,&
&  0.45790E+01_JPRB, 0.47945E+01_JPRB, 0.57190E+01_JPRB, 0.66847E+01_JPRB, 0.60336E+01_JPRB, 0.50879E+01_JPRB, 0.44003E+01_JPRB ,&
&  0.39217E+01_JPRB, 0.35400E+01_JPRB, 0.32150E+01_JPRB, 0.29646E+01_JPRB, 0.27770E+01_JPRB, 0.27794E+01_JPRB, 0.26310E+01_JPRB ,&
&  0.23143E+01_JPRB, 0.20516E+01_JPRB, 0.19094E+01_JPRB, 0.17078E+01_JPRB, 0.15621E+01_JPRB, 0.14004E+01_JPRB, 0.13535E+01_JPRB ,&
&  0.12052E+01_JPRB, 0.93133E+00_JPRB, 0.77922E+00_JPRB, 0.70496E+00_JPRB, 0.55355E+00_JPRB, 0.47560E+00_JPRB, 0.46984E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.34803E+00_JPRB, 0.36629E+00_JPRB, 0.39346E+00_JPRB, 0.40902E+00_JPRB, 0.40412E+00_JPRB, 0.39374E+00_JPRB ,&
&  0.36127E+00_JPRB, 0.39659E+00_JPRB, 0.57708E+00_JPRB, 0.62370E+00_JPRB, 0.63531E+00_JPRB, 0.60153E+00_JPRB, 0.57230E+00_JPRB ,&
&  0.52995E+00_JPRB, 0.54133E+00_JPRB, 0.54143E+00_JPRB, 0.53246E+00_JPRB, 0.51783E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.46798E+00_JPRB, 0.41650E+00_JPRB, 0.37707E+00_JPRB, 0.36906E+00_JPRB, 0.39103E+00_JPRB, 0.41266E+00_JPRB, 0.42775E+00_JPRB ,&
&  0.44590E+00_JPRB, 0.45972E+00_JPRB, 0.47154E+00_JPRB, 0.48252E+00_JPRB, 0.49015E+00_JPRB, 0.48340E+00_JPRB, 0.53456E+00_JPRB ,&
&  0.40132E+00_JPRB, 0.38572E+00_JPRB, 0.37440E+00_JPRB, 0.42825E+00_JPRB, 0.64062E+00_JPRB, 0.67603E+00_JPRB, 0.73819E+00_JPRB ,&
&  0.74654E+00_JPRB, 0.74874E+00_JPRB, 0.73385E+00_JPRB, 0.73031E+00_JPRB, 0.73334E+00_JPRB, 0.75788E+00_JPRB, 0.74516E+00_JPRB ,&
&  0.65359E+00_JPRB, 0.58631E+00_JPRB, 0.56910E+00_JPRB, 0.57466E+00_JPRB, 0.64261E+00_JPRB, 0.69843E+00_JPRB, 0.79791E+00_JPRB ,&
&  0.86067E+00_JPRB, 0.92999E+00_JPRB, 0.99724E+00_JPRB, 0.10492E+01_JPRB, 0.10704E+01_JPRB, 0.10771E+01_JPRB, 0.10852E+01_JPRB ,&
&  0.11035E+01_JPRB, 0.11327E+01_JPRB, 0.12073E+01_JPRB, 0.14679E+01_JPRB, 0.16935E+01_JPRB, 0.25679E+01_JPRB, 0.45922E+01_JPRB ,&
&  0.50033E+01_JPRB, 0.45218E+01_JPRB, 0.50530E+01_JPRB, 0.61213E+01_JPRB, 0.55209E+01_JPRB, 0.45461E+01_JPRB, 0.36444E+01_JPRB ,&
&  0.30651E+01_JPRB, 0.27226E+01_JPRB, 0.25158E+01_JPRB, 0.25498E+01_JPRB, 0.24956E+01_JPRB, 0.24964E+01_JPRB, 0.24673E+01_JPRB ,&
&  0.22983E+01_JPRB, 0.21706E+01_JPRB, 0.20602E+01_JPRB, 0.18784E+01_JPRB, 0.17362E+01_JPRB, 0.16092E+01_JPRB, 0.14900E+01_JPRB ,&
&  0.13102E+01_JPRB, 0.11006E+01_JPRB, 0.86943E+00_JPRB, 0.72217E+00_JPRB, 0.57874E+00_JPRB, 0.52224E+00_JPRB, 0.55433E+00_JPRB ,&
&  0.43384E+00_JPRB, 0.37944E+00_JPRB, 0.37983E+00_JPRB, 0.39970E+00_JPRB, 0.39356E+00_JPRB, 0.39097E+00_JPRB, 0.36683E+00_JPRB ,&
&  0.33471E+00_JPRB, 0.40136E+00_JPRB, 0.55381E+00_JPRB, 0.57730E+00_JPRB, 0.57288E+00_JPRB, 0.56585E+00_JPRB, 0.43985E+00_JPRB ,&
&  0.55024E+00_JPRB, 0.51855E+00_JPRB, 0.51677E+00_JPRB, 0.50777E+00_JPRB, 0.49873E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.47895E+00_JPRB, 0.43004E+00_JPRB, 0.38453E+00_JPRB, 0.37844E+00_JPRB, 0.40037E+00_JPRB, 0.41545E+00_JPRB, 0.42705E+00_JPRB ,&
&  0.44568E+00_JPRB, 0.45522E+00_JPRB, 0.46976E+00_JPRB, 0.48654E+00_JPRB, 0.49325E+00_JPRB, 0.52240E+00_JPRB, 0.46808E+00_JPRB ,&
&  0.38577E+00_JPRB, 0.37922E+00_JPRB, 0.36914E+00_JPRB, 0.39066E+00_JPRB, 0.45577E+00_JPRB, 0.47412E+00_JPRB, 0.48406E+00_JPRB ,&
&  0.49820E+00_JPRB, 0.48691E+00_JPRB, 0.50030E+00_JPRB, 0.50402E+00_JPRB, 0.50703E+00_JPRB, 0.52399E+00_JPRB, 0.54478E+00_JPRB ,&
&  0.49701E+00_JPRB, 0.46034E+00_JPRB, 0.44697E+00_JPRB, 0.44438E+00_JPRB, 0.51139E+00_JPRB, 0.59648E+00_JPRB, 0.64993E+00_JPRB ,&
&  0.69708E+00_JPRB, 0.74795E+00_JPRB, 0.80285E+00_JPRB, 0.85923E+00_JPRB, 0.91024E+00_JPRB, 0.94421E+00_JPRB, 0.95632E+00_JPRB ,&
&  0.96687E+00_JPRB, 0.98840E+00_JPRB, 0.10251E+01_JPRB, 0.11636E+01_JPRB, 0.14582E+01_JPRB, 0.22043E+01_JPRB, 0.32606E+01_JPRB ,&
&  0.34297E+01_JPRB, 0.33497E+01_JPRB, 0.33881E+01_JPRB, 0.35832E+01_JPRB, 0.35310E+01_JPRB, 0.29782E+01_JPRB, 0.24464E+01_JPRB ,&
&  0.22077E+01_JPRB, 0.21100E+01_JPRB, 0.20591E+01_JPRB, 0.21284E+01_JPRB, 0.21091E+01_JPRB, 0.21151E+01_JPRB, 0.20966E+01_JPRB ,&
&  0.20879E+01_JPRB, 0.20653E+01_JPRB, 0.20658E+01_JPRB, 0.19473E+01_JPRB, 0.18488E+01_JPRB, 0.17356E+01_JPRB, 0.15941E+01_JPRB ,&
&  0.14097E+01_JPRB, 0.12070E+01_JPRB, 0.98280E+00_JPRB, 0.79514E+00_JPRB, 0.64812E+00_JPRB, 0.60236E+00_JPRB, 0.54287E+00_JPRB ,&
&  0.43274E+00_JPRB, 0.38905E+00_JPRB, 0.37884E+00_JPRB, 0.39150E+00_JPRB, 0.37742E+00_JPRB, 0.36418E+00_JPRB, 0.37539E+00_JPRB ,&
&  0.42057E+00_JPRB, 0.54814E+00_JPRB, 0.55834E+00_JPRB, 0.55715E+00_JPRB, 0.55871E+00_JPRB, 0.56865E+00_JPRB, 0.39525E+00_JPRB ,&
&  0.44232E+00_JPRB, 0.54989E+00_JPRB, 0.53134E+00_JPRB, 0.51993E+00_JPRB, 0.50740E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.47866E+00_JPRB, 0.44344E+00_JPRB, 0.40827E+00_JPRB, 0.39785E+00_JPRB, 0.41209E+00_JPRB, 0.42945E+00_JPRB, 0.42186E+00_JPRB ,&
&  0.46076E+00_JPRB, 0.44837E+00_JPRB, 0.47484E+00_JPRB, 0.51386E+00_JPRB, 0.55501E+00_JPRB, 0.47905E+00_JPRB, 0.35731E+00_JPRB ,&
&  0.32452E+00_JPRB, 0.32971E+00_JPRB, 0.34755E+00_JPRB, 0.35672E+00_JPRB, 0.36749E+00_JPRB, 0.36875E+00_JPRB, 0.37206E+00_JPRB ,&
&  0.38095E+00_JPRB, 0.39766E+00_JPRB, 0.43602E+00_JPRB, 0.45975E+00_JPRB, 0.46867E+00_JPRB, 0.46871E+00_JPRB, 0.48482E+00_JPRB ,&
&  0.45115E+00_JPRB, 0.44049E+00_JPRB, 0.44564E+00_JPRB, 0.42359E+00_JPRB, 0.49097E+00_JPRB, 0.52511E+00_JPRB, 0.55354E+00_JPRB ,&
&  0.58586E+00_JPRB, 0.63291E+00_JPRB, 0.69079E+00_JPRB, 0.74627E+00_JPRB, 0.79747E+00_JPRB, 0.84127E+00_JPRB, 0.87622E+00_JPRB ,&
&  0.90509E+00_JPRB, 0.93784E+00_JPRB, 0.97686E+00_JPRB, 0.10534E+01_JPRB, 0.13209E+01_JPRB, 0.17816E+01_JPRB, 0.24116E+01_JPRB ,&
&  0.25529E+01_JPRB, 0.27208E+01_JPRB, 0.28147E+01_JPRB, 0.25454E+01_JPRB, 0.24904E+01_JPRB, 0.23564E+01_JPRB, 0.20676E+01_JPRB ,&
&  0.18951E+01_JPRB, 0.17810E+01_JPRB, 0.17699E+01_JPRB, 0.18192E+01_JPRB, 0.19089E+01_JPRB, 0.19620E+01_JPRB, 0.19721E+01_JPRB ,&
&  0.19856E+01_JPRB, 0.19940E+01_JPRB, 0.20545E+01_JPRB, 0.19669E+01_JPRB, 0.18743E+01_JPRB, 0.17569E+01_JPRB, 0.16022E+01_JPRB ,&
&  0.14338E+01_JPRB, 0.12729E+01_JPRB, 0.10901E+01_JPRB, 0.89386E+00_JPRB, 0.73572E+00_JPRB, 0.64725E+00_JPRB, 0.55386E+00_JPRB ,&
&  0.45698E+00_JPRB, 0.39198E+00_JPRB, 0.39707E+00_JPRB, 0.40223E+00_JPRB, 0.38774E+00_JPRB, 0.36961E+00_JPRB, 0.35796E+00_JPRB ,&
&  0.39043E+00_JPRB, 0.45120E+00_JPRB, 0.60998E+00_JPRB, 0.57520E+00_JPRB, 0.57053E+00_JPRB, 0.56442E+00_JPRB, 0.46176E+00_JPRB ,&
&  0.43574E+00_JPRB, 0.56831E+00_JPRB, 0.52824E+00_JPRB, 0.52295E+00_JPRB, 0.50850E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.50054E+00_JPRB, 0.47267E+00_JPRB, 0.44738E+00_JPRB, 0.42796E+00_JPRB, 0.42375E+00_JPRB, 0.41697E+00_JPRB, 0.41408E+00_JPRB ,&
&  0.46439E+00_JPRB, 0.38655E+00_JPRB, 0.46502E+00_JPRB, 0.36464E+00_JPRB, 0.33219E+00_JPRB, 0.32938E+00_JPRB, 0.31712E+00_JPRB ,&
&  0.31170E+00_JPRB, 0.31921E+00_JPRB, 0.33151E+00_JPRB, 0.34181E+00_JPRB, 0.34648E+00_JPRB, 0.35028E+00_JPRB, 0.36256E+00_JPRB ,&
&  0.38034E+00_JPRB, 0.40489E+00_JPRB, 0.43871E+00_JPRB, 0.45602E+00_JPRB, 0.45931E+00_JPRB, 0.45823E+00_JPRB, 0.46796E+00_JPRB ,&
&  0.43779E+00_JPRB, 0.43355E+00_JPRB, 0.44740E+00_JPRB, 0.45773E+00_JPRB, 0.45939E+00_JPRB, 0.45678E+00_JPRB, 0.47914E+00_JPRB ,&
&  0.48331E+00_JPRB, 0.45534E+00_JPRB, 0.59450E+00_JPRB, 0.64201E+00_JPRB, 0.69848E+00_JPRB, 0.75080E+00_JPRB, 0.79999E+00_JPRB ,&
&  0.84327E+00_JPRB, 0.88807E+00_JPRB, 0.93792E+00_JPRB, 0.98725E+00_JPRB, 0.11045E+01_JPRB, 0.13676E+01_JPRB, 0.16716E+01_JPRB ,&
&  0.20346E+01_JPRB, 0.15850E+01_JPRB, 0.17938E+01_JPRB, 0.17193E+01_JPRB, 0.18691E+01_JPRB, 0.19436E+01_JPRB, 0.18108E+01_JPRB ,&
&  0.15960E+01_JPRB, 0.15139E+01_JPRB, 0.14416E+01_JPRB, 0.13940E+01_JPRB, 0.14704E+01_JPRB, 0.15676E+01_JPRB, 0.16700E+01_JPRB ,&
&  0.17249E+01_JPRB, 0.17363E+01_JPRB, 0.18942E+01_JPRB, 0.17963E+01_JPRB, 0.17367E+01_JPRB, 0.16655E+01_JPRB, 0.15605E+01_JPRB ,&
&  0.14191E+01_JPRB, 0.12582E+01_JPRB, 0.10787E+01_JPRB, 0.89505E+00_JPRB, 0.76975E+00_JPRB, 0.65440E+00_JPRB, 0.55882E+00_JPRB ,&
&  0.49021E+00_JPRB, 0.44813E+00_JPRB, 0.45319E+00_JPRB, 0.45271E+00_JPRB, 0.44208E+00_JPRB, 0.42459E+00_JPRB, 0.39047E+00_JPRB ,&
&  0.36864E+00_JPRB, 0.36992E+00_JPRB, 0.40916E+00_JPRB, 0.44244E+00_JPRB, 0.45542E+00_JPRB, 0.47053E+00_JPRB, 0.52221E+00_JPRB ,&
&  0.47248E+00_JPRB, 0.51738E+00_JPRB, 0.55837E+00_JPRB, 0.53911E+00_JPRB, 0.52463E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.52438E+00_JPRB, 0.50698E+00_JPRB, 0.48884E+00_JPRB, 0.46719E+00_JPRB, 0.45798E+00_JPRB, 0.44447E+00_JPRB, 0.40297E+00_JPRB ,&
&  0.35736E+00_JPRB, 0.33564E+00_JPRB, 0.33781E+00_JPRB, 0.32580E+00_JPRB, 0.31306E+00_JPRB, 0.30895E+00_JPRB, 0.31019E+00_JPRB ,&
&  0.31949E+00_JPRB, 0.34576E+00_JPRB, 0.36943E+00_JPRB, 0.38090E+00_JPRB, 0.38210E+00_JPRB, 0.38696E+00_JPRB, 0.39718E+00_JPRB ,&
&  0.40693E+00_JPRB, 0.41794E+00_JPRB, 0.42738E+00_JPRB, 0.42501E+00_JPRB, 0.42196E+00_JPRB, 0.42444E+00_JPRB, 0.43066E+00_JPRB ,&
&  0.44154E+00_JPRB, 0.44832E+00_JPRB, 0.43877E+00_JPRB, 0.44856E+00_JPRB, 0.43243E+00_JPRB, 0.43535E+00_JPRB, 0.42121E+00_JPRB ,&
&  0.34335E+00_JPRB, 0.35829E+00_JPRB, 0.51040E+00_JPRB, 0.56854E+00_JPRB, 0.60929E+00_JPRB, 0.66706E+00_JPRB, 0.73476E+00_JPRB ,&
&  0.79034E+00_JPRB, 0.83824E+00_JPRB, 0.88474E+00_JPRB, 0.94265E+00_JPRB, 0.10143E+01_JPRB, 0.11441E+01_JPRB, 0.13015E+01_JPRB ,&
&  0.15786E+01_JPRB, 0.13237E+01_JPRB, 0.11796E+01_JPRB, 0.11627E+01_JPRB, 0.13950E+01_JPRB, 0.14093E+01_JPRB, 0.13799E+01_JPRB ,&
&  0.13188E+01_JPRB, 0.11980E+01_JPRB, 0.11338E+01_JPRB, 0.10693E+01_JPRB, 0.10899E+01_JPRB, 0.11557E+01_JPRB, 0.12565E+01_JPRB ,&
&  0.13614E+01_JPRB, 0.14997E+01_JPRB, 0.16255E+01_JPRB, 0.15663E+01_JPRB, 0.15371E+01_JPRB, 0.15082E+01_JPRB, 0.14561E+01_JPRB ,&
&  0.13895E+01_JPRB, 0.12482E+01_JPRB, 0.10328E+01_JPRB, 0.87091E+00_JPRB, 0.73994E+00_JPRB, 0.63749E+00_JPRB, 0.55358E+00_JPRB ,&
&  0.51373E+00_JPRB, 0.50612E+00_JPRB, 0.50957E+00_JPRB, 0.50910E+00_JPRB, 0.48870E+00_JPRB, 0.46228E+00_JPRB, 0.43008E+00_JPRB ,&
&  0.39454E+00_JPRB, 0.35903E+00_JPRB, 0.33798E+00_JPRB, 0.35134E+00_JPRB, 0.38658E+00_JPRB, 0.41805E+00_JPRB, 0.43367E+00_JPRB ,&
&  0.43828E+00_JPRB, 0.44802E+00_JPRB, 0.45408E+00_JPRB, 0.46519E+00_JPRB, 0.48882E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.54614E+00_JPRB, 0.53273E+00_JPRB, 0.50826E+00_JPRB, 0.50558E+00_JPRB, 0.48357E+00_JPRB, 0.46752E+00_JPRB, 0.42379E+00_JPRB ,&
&  0.36805E+00_JPRB, 0.35367E+00_JPRB, 0.37256E+00_JPRB, 0.37644E+00_JPRB, 0.36131E+00_JPRB, 0.35999E+00_JPRB, 0.36846E+00_JPRB ,&
&  0.37995E+00_JPRB, 0.40816E+00_JPRB, 0.41193E+00_JPRB, 0.39917E+00_JPRB, 0.39270E+00_JPRB, 0.39111E+00_JPRB, 0.39667E+00_JPRB ,&
&  0.39886E+00_JPRB, 0.39933E+00_JPRB, 0.39552E+00_JPRB, 0.38573E+00_JPRB, 0.38859E+00_JPRB, 0.39568E+00_JPRB, 0.40102E+00_JPRB ,&
&  0.40472E+00_JPRB, 0.40236E+00_JPRB, 0.41554E+00_JPRB, 0.43244E+00_JPRB, 0.42744E+00_JPRB, 0.41458E+00_JPRB, 0.34937E+00_JPRB ,&
&  0.30852E+00_JPRB, 0.30021E+00_JPRB, 0.33215E+00_JPRB, 0.44161E+00_JPRB, 0.49951E+00_JPRB, 0.55924E+00_JPRB, 0.61947E+00_JPRB ,&
&  0.59301E+00_JPRB, 0.59831E+00_JPRB, 0.64884E+00_JPRB, 0.80660E+00_JPRB, 0.88773E+00_JPRB, 0.97826E+00_JPRB, 0.10522E+01_JPRB ,&
&  0.11333E+01_JPRB, 0.11591E+01_JPRB, 0.11248E+01_JPRB, 0.88791E+00_JPRB, 0.87966E+00_JPRB, 0.10778E+01_JPRB, 0.10372E+01_JPRB ,&
&  0.10881E+01_JPRB, 0.10989E+01_JPRB, 0.11295E+01_JPRB, 0.10100E+01_JPRB, 0.95438E+00_JPRB, 0.93933E+00_JPRB, 0.97793E+00_JPRB ,&
&  0.10615E+01_JPRB, 0.11772E+01_JPRB, 0.13327E+01_JPRB, 0.13506E+01_JPRB, 0.13355E+01_JPRB, 0.13124E+01_JPRB, 0.12790E+01_JPRB ,&
&  0.12464E+01_JPRB, 0.11044E+01_JPRB, 0.92957E+00_JPRB, 0.78851E+00_JPRB, 0.67574E+00_JPRB, 0.59117E+00_JPRB, 0.54224E+00_JPRB ,&
&  0.52653E+00_JPRB, 0.52473E+00_JPRB, 0.53539E+00_JPRB, 0.53129E+00_JPRB, 0.50159E+00_JPRB, 0.46412E+00_JPRB, 0.43549E+00_JPRB ,&
&  0.41512E+00_JPRB, 0.38911E+00_JPRB, 0.36779E+00_JPRB, 0.36331E+00_JPRB, 0.38868E+00_JPRB, 0.42224E+00_JPRB, 0.43292E+00_JPRB ,&
&  0.42145E+00_JPRB, 0.42597E+00_JPRB, 0.45699E+00_JPRB, 0.48961E+00_JPRB, 0.50626E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.50614E+00_JPRB, 0.52038E+00_JPRB, 0.53569E+00_JPRB, 0.53334E+00_JPRB, 0.53274E+00_JPRB, 0.48109E+00_JPRB, 0.42712E+00_JPRB ,&
&  0.39075E+00_JPRB, 0.39632E+00_JPRB, 0.41449E+00_JPRB, 0.43075E+00_JPRB, 0.43343E+00_JPRB, 0.43004E+00_JPRB, 0.43380E+00_JPRB ,&
&  0.44296E+00_JPRB, 0.44306E+00_JPRB, 0.42650E+00_JPRB, 0.41369E+00_JPRB, 0.39553E+00_JPRB, 0.39297E+00_JPRB, 0.39587E+00_JPRB ,&
&  0.39844E+00_JPRB, 0.39679E+00_JPRB, 0.39540E+00_JPRB, 0.39332E+00_JPRB, 0.39599E+00_JPRB, 0.39707E+00_JPRB, 0.39600E+00_JPRB ,&
&  0.38928E+00_JPRB, 0.38265E+00_JPRB, 0.38931E+00_JPRB, 0.41004E+00_JPRB, 0.40581E+00_JPRB, 0.40319E+00_JPRB, 0.33123E+00_JPRB ,&
&  0.30332E+00_JPRB, 0.28673E+00_JPRB, 0.28556E+00_JPRB, 0.29967E+00_JPRB, 0.34522E+00_JPRB, 0.40275E+00_JPRB, 0.48007E+00_JPRB ,&
&  0.54883E+00_JPRB, 0.60408E+00_JPRB, 0.63623E+00_JPRB, 0.66335E+00_JPRB, 0.70452E+00_JPRB, 0.76956E+00_JPRB, 0.84105E+00_JPRB ,&
&  0.87643E+00_JPRB, 0.89096E+00_JPRB, 0.87541E+00_JPRB, 0.73488E+00_JPRB, 0.71455E+00_JPRB, 0.78504E+00_JPRB, 0.87886E+00_JPRB ,&
&  0.96071E+00_JPRB, 0.87304E+00_JPRB, 0.95392E+00_JPRB, 0.99753E+00_JPRB, 0.99667E+00_JPRB, 0.98554E+00_JPRB, 0.97347E+00_JPRB ,&
&  0.10423E+01_JPRB, 0.11006E+01_JPRB, 0.11989E+01_JPRB, 0.12208E+01_JPRB, 0.11566E+01_JPRB, 0.11616E+01_JPRB, 0.11119E+01_JPRB ,&
&  0.10467E+01_JPRB, 0.89185E+00_JPRB, 0.75558E+00_JPRB, 0.68417E+00_JPRB, 0.61353E+00_JPRB, 0.57228E+00_JPRB, 0.55242E+00_JPRB ,&
&  0.53702E+00_JPRB, 0.52428E+00_JPRB, 0.53219E+00_JPRB, 0.52697E+00_JPRB, 0.50738E+00_JPRB, 0.48573E+00_JPRB, 0.46670E+00_JPRB ,&
&  0.45052E+00_JPRB, 0.43986E+00_JPRB, 0.43715E+00_JPRB, 0.42841E+00_JPRB, 0.43093E+00_JPRB, 0.44163E+00_JPRB, 0.44826E+00_JPRB ,&
&  0.45203E+00_JPRB, 0.45571E+00_JPRB, 0.46589E+00_JPRB, 0.47677E+00_JPRB, 0.48679E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.53195E+00_JPRB, 0.54456E+00_JPRB, 0.55147E+00_JPRB, 0.56006E+00_JPRB, 0.57933E+00_JPRB, 0.54497E+00_JPRB, 0.52237E+00_JPRB ,&
&  0.51241E+00_JPRB, 0.52528E+00_JPRB, 0.52438E+00_JPRB, 0.52106E+00_JPRB, 0.51131E+00_JPRB, 0.50306E+00_JPRB, 0.49728E+00_JPRB ,&
&  0.49185E+00_JPRB, 0.47732E+00_JPRB, 0.45191E+00_JPRB, 0.41060E+00_JPRB, 0.39878E+00_JPRB, 0.39639E+00_JPRB, 0.40025E+00_JPRB ,&
&  0.40303E+00_JPRB, 0.39911E+00_JPRB, 0.39938E+00_JPRB, 0.39722E+00_JPRB, 0.39399E+00_JPRB, 0.38137E+00_JPRB, 0.36992E+00_JPRB ,&
&  0.37359E+00_JPRB, 0.37665E+00_JPRB, 0.39802E+00_JPRB, 0.38598E+00_JPRB, 0.37671E+00_JPRB, 0.37491E+00_JPRB, 0.31195E+00_JPRB ,&
&  0.30017E+00_JPRB, 0.28633E+00_JPRB, 0.28217E+00_JPRB, 0.28264E+00_JPRB, 0.29931E+00_JPRB, 0.31785E+00_JPRB, 0.35646E+00_JPRB ,&
&  0.41353E+00_JPRB, 0.50366E+00_JPRB, 0.54859E+00_JPRB, 0.59730E+00_JPRB, 0.63880E+00_JPRB, 0.67955E+00_JPRB, 0.71932E+00_JPRB ,&
&  0.74946E+00_JPRB, 0.78095E+00_JPRB, 0.80647E+00_JPRB, 0.85474E+00_JPRB, 0.89578E+00_JPRB, 0.85125E+00_JPRB, 0.88033E+00_JPRB ,&
&  0.10731E+01_JPRB, 0.99100E+00_JPRB, 0.96946E+00_JPRB, 0.93158E+00_JPRB, 0.93412E+00_JPRB, 0.93780E+00_JPRB, 0.94102E+00_JPRB ,&
&  0.98606E+00_JPRB, 0.10528E+01_JPRB, 0.11527E+01_JPRB, 0.11980E+01_JPRB, 0.11284E+01_JPRB, 0.11186E+01_JPRB, 0.10687E+01_JPRB ,&
&  0.94467E+00_JPRB, 0.82375E+00_JPRB, 0.72103E+00_JPRB, 0.66346E+00_JPRB, 0.60826E+00_JPRB, 0.58058E+00_JPRB, 0.57851E+00_JPRB ,&
&  0.57555E+00_JPRB, 0.56561E+00_JPRB, 0.56059E+00_JPRB, 0.54642E+00_JPRB, 0.52446E+00_JPRB, 0.51100E+00_JPRB, 0.51176E+00_JPRB ,&
&  0.51236E+00_JPRB, 0.51037E+00_JPRB, 0.50991E+00_JPRB, 0.50653E+00_JPRB, 0.49880E+00_JPRB, 0.49017E+00_JPRB, 0.49104E+00_JPRB ,&
&  0.49778E+00_JPRB, 0.50433E+00_JPRB, 0.50973E+00_JPRB, 0.51321E+00_JPRB, 0.51932E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.54849E+00_JPRB, 0.56359E+00_JPRB, 0.56905E+00_JPRB, 0.57170E+00_JPRB, 0.57565E+00_JPRB, 0.56306E+00_JPRB, 0.55246E+00_JPRB ,&
&  0.54517E+00_JPRB, 0.53609E+00_JPRB, 0.52534E+00_JPRB, 0.51506E+00_JPRB, 0.50476E+00_JPRB, 0.49828E+00_JPRB, 0.49198E+00_JPRB ,&
&  0.48591E+00_JPRB, 0.45126E+00_JPRB, 0.43322E+00_JPRB, 0.42107E+00_JPRB, 0.41070E+00_JPRB, 0.40651E+00_JPRB, 0.40520E+00_JPRB ,&
&  0.40106E+00_JPRB, 0.39879E+00_JPRB, 0.39953E+00_JPRB, 0.39829E+00_JPRB, 0.39358E+00_JPRB, 0.38461E+00_JPRB, 0.39075E+00_JPRB ,&
&  0.40950E+00_JPRB, 0.40239E+00_JPRB, 0.38916E+00_JPRB, 0.37646E+00_JPRB, 0.35786E+00_JPRB, 0.33259E+00_JPRB, 0.31271E+00_JPRB ,&
&  0.30484E+00_JPRB, 0.28762E+00_JPRB, 0.28911E+00_JPRB, 0.29276E+00_JPRB, 0.29688E+00_JPRB, 0.31190E+00_JPRB, 0.34519E+00_JPRB ,&
&  0.38529E+00_JPRB, 0.48179E+00_JPRB, 0.52213E+00_JPRB, 0.57009E+00_JPRB, 0.60640E+00_JPRB, 0.63636E+00_JPRB, 0.66211E+00_JPRB ,&
&  0.67833E+00_JPRB, 0.70234E+00_JPRB, 0.72903E+00_JPRB, 0.76891E+00_JPRB, 0.80983E+00_JPRB, 0.83284E+00_JPRB, 0.84918E+00_JPRB ,&
&  0.87960E+00_JPRB, 0.85358E+00_JPRB, 0.85781E+00_JPRB, 0.83945E+00_JPRB, 0.83774E+00_JPRB, 0.86859E+00_JPRB, 0.85660E+00_JPRB ,&
&  0.86145E+00_JPRB, 0.88468E+00_JPRB, 0.95078E+00_JPRB, 0.10073E+01_JPRB, 0.99271E+00_JPRB, 0.10034E+01_JPRB, 0.99056E+00_JPRB ,&
&  0.92813E+00_JPRB, 0.89104E+00_JPRB, 0.85295E+00_JPRB, 0.79208E+00_JPRB, 0.72742E+00_JPRB, 0.68394E+00_JPRB, 0.65703E+00_JPRB ,&
&  0.63042E+00_JPRB, 0.59882E+00_JPRB, 0.57754E+00_JPRB, 0.56059E+00_JPRB, 0.54177E+00_JPRB, 0.52458E+00_JPRB, 0.52469E+00_JPRB ,&
&  0.52666E+00_JPRB, 0.52775E+00_JPRB, 0.52706E+00_JPRB, 0.52560E+00_JPRB, 0.51540E+00_JPRB, 0.50373E+00_JPRB, 0.49338E+00_JPRB ,&
&  0.49549E+00_JPRB, 0.50110E+00_JPRB, 0.51293E+00_JPRB, 0.52247E+00_JPRB, 0.53421E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.55097E+00_JPRB, 0.55349E+00_JPRB, 0.55007E+00_JPRB, 0.54472E+00_JPRB, 0.54060E+00_JPRB, 0.53141E+00_JPRB, 0.52593E+00_JPRB ,&
&  0.52167E+00_JPRB, 0.51751E+00_JPRB, 0.50852E+00_JPRB, 0.49885E+00_JPRB, 0.48597E+00_JPRB, 0.47676E+00_JPRB, 0.46620E+00_JPRB ,&
&  0.45792E+00_JPRB, 0.44777E+00_JPRB, 0.43710E+00_JPRB, 0.42693E+00_JPRB, 0.41939E+00_JPRB, 0.41193E+00_JPRB, 0.40784E+00_JPRB ,&
&  0.40391E+00_JPRB, 0.39578E+00_JPRB, 0.38857E+00_JPRB, 0.38982E+00_JPRB, 0.38706E+00_JPRB, 0.38757E+00_JPRB, 0.39504E+00_JPRB ,&
&  0.40275E+00_JPRB, 0.40098E+00_JPRB, 0.39796E+00_JPRB, 0.38057E+00_JPRB, 0.35471E+00_JPRB, 0.29544E+00_JPRB, 0.30151E+00_JPRB ,&
&  0.29733E+00_JPRB, 0.29008E+00_JPRB, 0.29346E+00_JPRB, 0.30173E+00_JPRB, 0.31055E+00_JPRB, 0.33389E+00_JPRB, 0.34263E+00_JPRB ,&
&  0.34472E+00_JPRB, 0.46440E+00_JPRB, 0.49643E+00_JPRB, 0.52900E+00_JPRB, 0.55595E+00_JPRB, 0.57988E+00_JPRB, 0.60287E+00_JPRB ,&
&  0.61725E+00_JPRB, 0.63869E+00_JPRB, 0.65737E+00_JPRB, 0.69038E+00_JPRB, 0.72242E+00_JPRB, 0.76015E+00_JPRB, 0.78977E+00_JPRB ,&
&  0.79721E+00_JPRB, 0.79497E+00_JPRB, 0.80203E+00_JPRB, 0.80114E+00_JPRB, 0.80509E+00_JPRB, 0.82139E+00_JPRB, 0.84960E+00_JPRB ,&
&  0.89073E+00_JPRB, 0.89363E+00_JPRB, 0.93944E+00_JPRB, 0.96617E+00_JPRB, 0.96148E+00_JPRB, 0.95942E+00_JPRB, 0.95581E+00_JPRB ,&
&  0.91953E+00_JPRB, 0.89938E+00_JPRB, 0.85492E+00_JPRB, 0.78637E+00_JPRB, 0.72044E+00_JPRB, 0.68217E+00_JPRB, 0.64720E+00_JPRB ,&
&  0.62207E+00_JPRB, 0.59523E+00_JPRB, 0.56685E+00_JPRB, 0.54497E+00_JPRB, 0.52767E+00_JPRB, 0.51716E+00_JPRB, 0.51459E+00_JPRB ,&
&  0.51307E+00_JPRB, 0.51225E+00_JPRB, 0.51187E+00_JPRB, 0.51572E+00_JPRB, 0.51407E+00_JPRB, 0.51356E+00_JPRB, 0.51201E+00_JPRB ,&
&  0.51580E+00_JPRB, 0.52042E+00_JPRB, 0.52929E+00_JPRB, 0.53731E+00_JPRB, 0.54241E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.54318E+00_JPRB, 0.54328E+00_JPRB, 0.54304E+00_JPRB, 0.53564E+00_JPRB, 0.53178E+00_JPRB, 0.52657E+00_JPRB, 0.52695E+00_JPRB ,&
&  0.52013E+00_JPRB, 0.51417E+00_JPRB, 0.49898E+00_JPRB, 0.48929E+00_JPRB, 0.47761E+00_JPRB, 0.47095E+00_JPRB, 0.46461E+00_JPRB ,&
&  0.46072E+00_JPRB, 0.45523E+00_JPRB, 0.45245E+00_JPRB, 0.44537E+00_JPRB, 0.43980E+00_JPRB, 0.43049E+00_JPRB, 0.42021E+00_JPRB ,&
&  0.40874E+00_JPRB, 0.39677E+00_JPRB, 0.38496E+00_JPRB, 0.38631E+00_JPRB, 0.38213E+00_JPRB, 0.38095E+00_JPRB, 0.38200E+00_JPRB ,&
&  0.39191E+00_JPRB, 0.35137E+00_JPRB, 0.35334E+00_JPRB, 0.33892E+00_JPRB, 0.32418E+00_JPRB, 0.30889E+00_JPRB, 0.29859E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30342E+00_JPRB, 0.31270E+00_JPRB, 0.32837E+00_JPRB, 0.33681E+00_JPRB, 0.35263E+00_JPRB, 0.36636E+00_JPRB ,&
&  0.37261E+00_JPRB, 0.45723E+00_JPRB, 0.50142E+00_JPRB, 0.52724E+00_JPRB, 0.55007E+00_JPRB, 0.57059E+00_JPRB, 0.58616E+00_JPRB ,&
&  0.60618E+00_JPRB, 0.62690E+00_JPRB, 0.64456E+00_JPRB, 0.66042E+00_JPRB, 0.69211E+00_JPRB, 0.71373E+00_JPRB, 0.73048E+00_JPRB ,&
&  0.73867E+00_JPRB, 0.73355E+00_JPRB, 0.73661E+00_JPRB, 0.74851E+00_JPRB, 0.77405E+00_JPRB, 0.80131E+00_JPRB, 0.83193E+00_JPRB ,&
&  0.86096E+00_JPRB, 0.87904E+00_JPRB, 0.89764E+00_JPRB, 0.91492E+00_JPRB, 0.92445E+00_JPRB, 0.91693E+00_JPRB, 0.90247E+00_JPRB ,&
&  0.86829E+00_JPRB, 0.83691E+00_JPRB, 0.79753E+00_JPRB, 0.75594E+00_JPRB, 0.71247E+00_JPRB, 0.66691E+00_JPRB, 0.62603E+00_JPRB ,&
&  0.60427E+00_JPRB, 0.58407E+00_JPRB, 0.56594E+00_JPRB, 0.54718E+00_JPRB, 0.53165E+00_JPRB, 0.52253E+00_JPRB, 0.52040E+00_JPRB ,&
&  0.51729E+00_JPRB, 0.51633E+00_JPRB, 0.51661E+00_JPRB, 0.51787E+00_JPRB, 0.52117E+00_JPRB, 0.52319E+00_JPRB, 0.52830E+00_JPRB ,&
&  0.53392E+00_JPRB, 0.53910E+00_JPRB, 0.54022E+00_JPRB, 0.54059E+00_JPRB, 0.54142E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.53783E+00_JPRB, 0.53949E+00_JPRB, 0.54567E+00_JPRB, 0.54666E+00_JPRB, 0.54704E+00_JPRB, 0.54267E+00_JPRB, 0.54159E+00_JPRB ,&
&  0.53519E+00_JPRB, 0.52870E+00_JPRB, 0.51341E+00_JPRB, 0.50238E+00_JPRB, 0.49100E+00_JPRB, 0.48455E+00_JPRB, 0.47950E+00_JPRB ,&
&  0.47643E+00_JPRB, 0.47158E+00_JPRB, 0.46643E+00_JPRB, 0.45954E+00_JPRB, 0.45138E+00_JPRB, 0.44135E+00_JPRB, 0.42678E+00_JPRB ,&
&  0.41340E+00_JPRB, 0.40310E+00_JPRB, 0.38794E+00_JPRB, 0.39037E+00_JPRB, 0.38456E+00_JPRB, 0.38087E+00_JPRB, 0.37989E+00_JPRB ,&
&  0.37488E+00_JPRB, 0.36674E+00_JPRB, 0.35546E+00_JPRB, 0.34608E+00_JPRB, 0.33711E+00_JPRB, 0.32787E+00_JPRB, 0.33549E+00_JPRB ,&
&  0.34207E+00_JPRB, 0.36264E+00_JPRB, 0.37321E+00_JPRB, 0.37862E+00_JPRB, 0.38457E+00_JPRB, 0.38823E+00_JPRB, 0.39946E+00_JPRB ,&
&  0.41538E+00_JPRB, 0.43763E+00_JPRB, 0.49283E+00_JPRB, 0.52544E+00_JPRB, 0.54304E+00_JPRB, 0.56139E+00_JPRB, 0.57330E+00_JPRB ,&
&  0.58886E+00_JPRB, 0.59660E+00_JPRB, 0.59948E+00_JPRB, 0.60564E+00_JPRB, 0.61990E+00_JPRB, 0.63868E+00_JPRB, 0.65265E+00_JPRB ,&
&  0.66254E+00_JPRB, 0.66475E+00_JPRB, 0.67204E+00_JPRB, 0.68209E+00_JPRB, 0.70262E+00_JPRB, 0.72353E+00_JPRB, 0.74496E+00_JPRB ,&
&  0.76297E+00_JPRB, 0.78057E+00_JPRB, 0.79040E+00_JPRB, 0.79237E+00_JPRB, 0.78803E+00_JPRB, 0.77932E+00_JPRB, 0.77062E+00_JPRB ,&
&  0.75304E+00_JPRB, 0.73071E+00_JPRB, 0.70486E+00_JPRB, 0.67049E+00_JPRB, 0.64653E+00_JPRB, 0.63174E+00_JPRB, 0.61203E+00_JPRB ,&
&  0.59565E+00_JPRB, 0.58119E+00_JPRB, 0.56750E+00_JPRB, 0.55465E+00_JPRB, 0.54407E+00_JPRB, 0.53473E+00_JPRB, 0.52875E+00_JPRB ,&
&  0.52460E+00_JPRB, 0.52260E+00_JPRB, 0.52338E+00_JPRB, 0.52560E+00_JPRB, 0.53143E+00_JPRB, 0.53581E+00_JPRB, 0.54292E+00_JPRB ,&
&  0.54594E+00_JPRB, 0.54605E+00_JPRB, 0.54222E+00_JPRB, 0.53915E+00_JPRB, 0.53673E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.54214E+00_JPRB, 0.54189E+00_JPRB, 0.54367E+00_JPRB, 0.54485E+00_JPRB, 0.54672E+00_JPRB, 0.54551E+00_JPRB, 0.54454E+00_JPRB ,&
&  0.54096E+00_JPRB, 0.53434E+00_JPRB, 0.52271E+00_JPRB, 0.51231E+00_JPRB, 0.50310E+00_JPRB, 0.49710E+00_JPRB, 0.49190E+00_JPRB ,&
&  0.48696E+00_JPRB, 0.48280E+00_JPRB, 0.47840E+00_JPRB, 0.47383E+00_JPRB, 0.46794E+00_JPRB, 0.46115E+00_JPRB, 0.45205E+00_JPRB ,&
&  0.44508E+00_JPRB, 0.43824E+00_JPRB, 0.43251E+00_JPRB, 0.42860E+00_JPRB, 0.42720E+00_JPRB, 0.40792E+00_JPRB, 0.40365E+00_JPRB ,&
&  0.40830E+00_JPRB, 0.40109E+00_JPRB, 0.39528E+00_JPRB, 0.38581E+00_JPRB, 0.38924E+00_JPRB, 0.39782E+00_JPRB, 0.40363E+00_JPRB ,&
&  0.39393E+00_JPRB, 0.41424E+00_JPRB, 0.41666E+00_JPRB, 0.41838E+00_JPRB, 0.41819E+00_JPRB, 0.41706E+00_JPRB, 0.43011E+00_JPRB ,&
&  0.45924E+00_JPRB, 0.47406E+00_JPRB, 0.49837E+00_JPRB, 0.52417E+00_JPRB, 0.54077E+00_JPRB, 0.55794E+00_JPRB, 0.57286E+00_JPRB ,&
&  0.58740E+00_JPRB, 0.59540E+00_JPRB, 0.59944E+00_JPRB, 0.60344E+00_JPRB, 0.61221E+00_JPRB, 0.62157E+00_JPRB, 0.62888E+00_JPRB ,&
&  0.63291E+00_JPRB, 0.63335E+00_JPRB, 0.63568E+00_JPRB, 0.63843E+00_JPRB, 0.64496E+00_JPRB, 0.65004E+00_JPRB, 0.65384E+00_JPRB ,&
&  0.65471E+00_JPRB, 0.65525E+00_JPRB, 0.65293E+00_JPRB, 0.64865E+00_JPRB, 0.64306E+00_JPRB, 0.63637E+00_JPRB, 0.63340E+00_JPRB ,&
&  0.62592E+00_JPRB, 0.61718E+00_JPRB, 0.60676E+00_JPRB, 0.59426E+00_JPRB, 0.58421E+00_JPRB, 0.57661E+00_JPRB, 0.57012E+00_JPRB ,&
&  0.56369E+00_JPRB, 0.55812E+00_JPRB, 0.55328E+00_JPRB, 0.54876E+00_JPRB, 0.54417E+00_JPRB, 0.53975E+00_JPRB, 0.53702E+00_JPRB ,&
&  0.53707E+00_JPRB, 0.53750E+00_JPRB, 0.53989E+00_JPRB, 0.54128E+00_JPRB, 0.54500E+00_JPRB, 0.54654E+00_JPRB, 0.54926E+00_JPRB ,&
&  0.55042E+00_JPRB, 0.54944E+00_JPRB, 0.54645E+00_JPRB, 0.54397E+00_JPRB, 0.54177E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.53952E+00_JPRB, 0.53782E+00_JPRB, 0.53674E+00_JPRB, 0.53710E+00_JPRB, 0.53819E+00_JPRB, 0.53860E+00_JPRB, 0.53819E+00_JPRB ,&
&  0.53642E+00_JPRB, 0.53188E+00_JPRB, 0.52654E+00_JPRB, 0.52096E+00_JPRB, 0.51465E+00_JPRB, 0.50881E+00_JPRB, 0.50515E+00_JPRB ,&
&  0.50111E+00_JPRB, 0.49704E+00_JPRB, 0.49286E+00_JPRB, 0.48997E+00_JPRB, 0.48679E+00_JPRB, 0.48373E+00_JPRB, 0.48101E+00_JPRB ,&
&  0.47934E+00_JPRB, 0.47724E+00_JPRB, 0.47482E+00_JPRB, 0.47181E+00_JPRB, 0.46972E+00_JPRB, 0.46715E+00_JPRB, 0.46469E+00_JPRB ,&
&  0.46172E+00_JPRB, 0.46081E+00_JPRB, 0.46032E+00_JPRB, 0.46059E+00_JPRB, 0.46283E+00_JPRB, 0.46591E+00_JPRB, 0.46725E+00_JPRB ,&
&  0.46769E+00_JPRB, 0.46607E+00_JPRB, 0.46766E+00_JPRB, 0.46987E+00_JPRB, 0.47360E+00_JPRB, 0.47935E+00_JPRB, 0.48716E+00_JPRB ,&
&  0.49712E+00_JPRB, 0.50853E+00_JPRB, 0.51996E+00_JPRB, 0.52817E+00_JPRB, 0.53599E+00_JPRB, 0.54308E+00_JPRB, 0.54714E+00_JPRB ,&
&  0.55133E+00_JPRB, 0.55408E+00_JPRB, 0.55792E+00_JPRB, 0.56074E+00_JPRB, 0.56490E+00_JPRB, 0.56845E+00_JPRB, 0.57170E+00_JPRB ,&
&  0.57233E+00_JPRB, 0.57066E+00_JPRB, 0.56892E+00_JPRB, 0.56767E+00_JPRB, 0.56558E+00_JPRB, 0.56392E+00_JPRB, 0.56251E+00_JPRB ,&
&  0.56137E+00_JPRB, 0.56049E+00_JPRB, 0.56037E+00_JPRB, 0.56048E+00_JPRB, 0.56078E+00_JPRB, 0.56045E+00_JPRB, 0.56103E+00_JPRB ,&
&  0.55991E+00_JPRB, 0.55836E+00_JPRB, 0.55600E+00_JPRB, 0.55234E+00_JPRB, 0.54925E+00_JPRB, 0.54699E+00_JPRB, 0.54497E+00_JPRB ,&
&  0.54291E+00_JPRB, 0.54138E+00_JPRB, 0.54037E+00_JPRB, 0.54015E+00_JPRB, 0.53925E+00_JPRB, 0.53850E+00_JPRB, 0.53832E+00_JPRB ,&
&  0.53873E+00_JPRB, 0.53862E+00_JPRB, 0.53969E+00_JPRB, 0.54077E+00_JPRB, 0.54413E+00_JPRB, 0.54562E+00_JPRB, 0.54675E+00_JPRB ,&
&  0.54690E+00_JPRB, 0.54757E+00_JPRB, 0.54648E+00_JPRB, 0.54403E+00_JPRB, 0.54133E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.53989E+00_JPRB, 0.53847E+00_JPRB, 0.53724E+00_JPRB, 0.53645E+00_JPRB, 0.53615E+00_JPRB, 0.53609E+00_JPRB, 0.53536E+00_JPRB ,&
&  0.53424E+00_JPRB, 0.53228E+00_JPRB, 0.53031E+00_JPRB, 0.52788E+00_JPRB, 0.52506E+00_JPRB, 0.52123E+00_JPRB, 0.51800E+00_JPRB ,&
&  0.51508E+00_JPRB, 0.51274E+00_JPRB, 0.51010E+00_JPRB, 0.50842E+00_JPRB, 0.50674E+00_JPRB, 0.50536E+00_JPRB, 0.50405E+00_JPRB ,&
&  0.50318E+00_JPRB, 0.50231E+00_JPRB, 0.50148E+00_JPRB, 0.50070E+00_JPRB, 0.50045E+00_JPRB, 0.50003E+00_JPRB, 0.49951E+00_JPRB ,&
&  0.49910E+00_JPRB, 0.49895E+00_JPRB, 0.49884E+00_JPRB, 0.49885E+00_JPRB, 0.49890E+00_JPRB, 0.49932E+00_JPRB, 0.49975E+00_JPRB ,&
&  0.50042E+00_JPRB, 0.50162E+00_JPRB, 0.50323E+00_JPRB, 0.50474E+00_JPRB, 0.50639E+00_JPRB, 0.50854E+00_JPRB, 0.51107E+00_JPRB ,&
&  0.51330E+00_JPRB, 0.51522E+00_JPRB, 0.51719E+00_JPRB, 0.51942E+00_JPRB, 0.52155E+00_JPRB, 0.52374E+00_JPRB, 0.52548E+00_JPRB ,&
&  0.52711E+00_JPRB, 0.52868E+00_JPRB, 0.53052E+00_JPRB, 0.53097E+00_JPRB, 0.53214E+00_JPRB, 0.53317E+00_JPRB, 0.53456E+00_JPRB ,&
&  0.53491E+00_JPRB, 0.53520E+00_JPRB, 0.53592E+00_JPRB, 0.53669E+00_JPRB, 0.53720E+00_JPRB, 0.53778E+00_JPRB, 0.53850E+00_JPRB ,&
&  0.53949E+00_JPRB, 0.54013E+00_JPRB, 0.54097E+00_JPRB, 0.54175E+00_JPRB, 0.54284E+00_JPRB, 0.54371E+00_JPRB, 0.54466E+00_JPRB ,&
&  0.54491E+00_JPRB, 0.54502E+00_JPRB, 0.54489E+00_JPRB, 0.54461E+00_JPRB, 0.54447E+00_JPRB, 0.54448E+00_JPRB, 0.54455E+00_JPRB ,&
&  0.54470E+00_JPRB, 0.54478E+00_JPRB, 0.54480E+00_JPRB, 0.54463E+00_JPRB, 0.54398E+00_JPRB, 0.54336E+00_JPRB, 0.54294E+00_JPRB ,&
&  0.54296E+00_JPRB, 0.54291E+00_JPRB, 0.54319E+00_JPRB, 0.54371E+00_JPRB, 0.54485E+00_JPRB, 0.54525E+00_JPRB, 0.54553E+00_JPRB ,&
&  0.54533E+00_JPRB, 0.54492E+00_JPRB, 0.54390E+00_JPRB, 0.54262E+00_JPRB, 0.54121E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB ,&
&  0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB, 0.52747E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB ,&
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB ,&
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB ,&
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB ,&
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB ,&
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB ,&
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB ,&
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB ,&
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB ,&
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB ,&
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB ,&
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB ,&
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB ,&
&  0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB, 0.12279E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.12453E+00_JPRB, 0.12516E+00_JPRB, 0.12563E+00_JPRB, 0.12602E+00_JPRB, 0.12593E+00_JPRB, 0.12602E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12617E+00_JPRB, 0.12619E+00_JPRB, 0.12641E+00_JPRB, 0.12663E+00_JPRB, 0.12685E+00_JPRB, 0.12708E+00_JPRB, 0.12732E+00_JPRB ,&
&  0.12743E+00_JPRB, 0.12765E+00_JPRB, 0.12681E+00_JPRB, 0.12614E+00_JPRB, 0.12576E+00_JPRB, 0.12555E+00_JPRB, 0.12546E+00_JPRB ,&
&  0.12538E+00_JPRB, 0.12534E+00_JPRB, 0.12525E+00_JPRB, 0.12589E+00_JPRB, 0.12631E+00_JPRB, 0.12670E+00_JPRB, 0.12701E+00_JPRB ,&
&  0.12715E+00_JPRB, 0.12733E+00_JPRB, 0.12747E+00_JPRB, 0.12763E+00_JPRB, 0.12780E+00_JPRB, 0.12775E+00_JPRB, 0.12763E+00_JPRB ,&
&  0.12744E+00_JPRB, 0.12717E+00_JPRB, 0.12684E+00_JPRB, 0.12653E+00_JPRB, 0.12628E+00_JPRB, 0.12601E+00_JPRB, 0.12570E+00_JPRB ,&
&  0.12541E+00_JPRB, 0.12506E+00_JPRB, 0.12472E+00_JPRB, 0.12441E+00_JPRB, 0.12410E+00_JPRB, 0.12384E+00_JPRB, 0.12356E+00_JPRB ,&
&  0.12336E+00_JPRB, 0.12317E+00_JPRB, 0.12299E+00_JPRB, 0.12285E+00_JPRB, 0.12266E+00_JPRB, 0.12257E+00_JPRB, 0.12239E+00_JPRB ,&
&  0.12232E+00_JPRB, 0.12224E+00_JPRB, 0.12217E+00_JPRB, 0.12209E+00_JPRB, 0.12206E+00_JPRB, 0.12202E+00_JPRB, 0.12200E+00_JPRB ,&
&  0.12195E+00_JPRB, 0.12202E+00_JPRB, 0.12191E+00_JPRB, 0.12177E+00_JPRB, 0.12162E+00_JPRB, 0.12147E+00_JPRB, 0.12133E+00_JPRB ,&
&  0.12120E+00_JPRB, 0.12108E+00_JPRB, 0.12099E+00_JPRB, 0.12099E+00_JPRB, 0.12100E+00_JPRB, 0.12102E+00_JPRB, 0.12107E+00_JPRB ,&
&  0.12113E+00_JPRB, 0.12121E+00_JPRB, 0.12129E+00_JPRB, 0.12127E+00_JPRB, 0.12130E+00_JPRB, 0.12131E+00_JPRB, 0.12133E+00_JPRB ,&
&  0.12136E+00_JPRB, 0.12141E+00_JPRB, 0.12148E+00_JPRB, 0.12156E+00_JPRB, 0.12175E+00_JPRB, 0.12184E+00_JPRB, 0.12197E+00_JPRB ,&
&  0.12215E+00_JPRB, 0.12246E+00_JPRB, 0.12274E+00_JPRB, 0.12305E+00_JPRB, 0.12342E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.13313E+00_JPRB, 0.13497E+00_JPRB, 0.13653E+00_JPRB, 0.13853E+00_JPRB, 0.13595E+00_JPRB, 0.14298E+00_JPRB, 0.14085E+00_JPRB ,&
&  0.13861E+00_JPRB, 0.13396E+00_JPRB, 0.13377E+00_JPRB, 0.13278E+00_JPRB, 0.13152E+00_JPRB, 0.13067E+00_JPRB, 0.13024E+00_JPRB ,&
&  0.13000E+00_JPRB, 0.12959E+00_JPRB, 0.12971E+00_JPRB, 0.12978E+00_JPRB, 0.12984E+00_JPRB, 0.12987E+00_JPRB, 0.12999E+00_JPRB ,&
&  0.13035E+00_JPRB, 0.13107E+00_JPRB, 0.13184E+00_JPRB, 0.13060E+00_JPRB, 0.13057E+00_JPRB, 0.13095E+00_JPRB, 0.13162E+00_JPRB ,&
&  0.13228E+00_JPRB, 0.13329E+00_JPRB, 0.13400E+00_JPRB, 0.13447E+00_JPRB, 0.13561E+00_JPRB, 0.13555E+00_JPRB, 0.13533E+00_JPRB ,&
&  0.13494E+00_JPRB, 0.13423E+00_JPRB, 0.13319E+00_JPRB, 0.13224E+00_JPRB, 0.13124E+00_JPRB, 0.12998E+00_JPRB, 0.12877E+00_JPRB ,&
&  0.12776E+00_JPRB, 0.12693E+00_JPRB, 0.12591E+00_JPRB, 0.12492E+00_JPRB, 0.12401E+00_JPRB, 0.12312E+00_JPRB, 0.12235E+00_JPRB ,&
&  0.12179E+00_JPRB, 0.12132E+00_JPRB, 0.12084E+00_JPRB, 0.12046E+00_JPRB, 0.12006E+00_JPRB, 0.11969E+00_JPRB, 0.11935E+00_JPRB ,&
&  0.11943E+00_JPRB, 0.11950E+00_JPRB, 0.11956E+00_JPRB, 0.11960E+00_JPRB, 0.11986E+00_JPRB, 0.11988E+00_JPRB, 0.11991E+00_JPRB ,&
&  0.11990E+00_JPRB, 0.11979E+00_JPRB, 0.11970E+00_JPRB, 0.11953E+00_JPRB, 0.11934E+00_JPRB, 0.11911E+00_JPRB, 0.11902E+00_JPRB ,&
&  0.11903E+00_JPRB, 0.11912E+00_JPRB, 0.11939E+00_JPRB, 0.11957E+00_JPRB, 0.11974E+00_JPRB, 0.11989E+00_JPRB, 0.12023E+00_JPRB ,&
&  0.12059E+00_JPRB, 0.12097E+00_JPRB, 0.12132E+00_JPRB, 0.12129E+00_JPRB, 0.12151E+00_JPRB, 0.12171E+00_JPRB, 0.12190E+00_JPRB ,&
&  0.12202E+00_JPRB, 0.12242E+00_JPRB, 0.12302E+00_JPRB, 0.12380E+00_JPRB, 0.12521E+00_JPRB, 0.12647E+00_JPRB, 0.12766E+00_JPRB ,&
&  0.12892E+00_JPRB, 0.13140E+00_JPRB, 0.13258E+00_JPRB, 0.13314E+00_JPRB, 0.13342E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.14330E+00_JPRB, 0.14502E+00_JPRB, 0.14609E+00_JPRB, 0.14684E+00_JPRB, 0.14780E+00_JPRB, 0.14820E+00_JPRB, 0.14858E+00_JPRB ,&
&  0.14687E+00_JPRB, 0.14620E+00_JPRB, 0.14513E+00_JPRB, 0.14383E+00_JPRB, 0.14283E+00_JPRB, 0.14083E+00_JPRB, 0.13875E+00_JPRB ,&
&  0.13637E+00_JPRB, 0.13421E+00_JPRB, 0.13273E+00_JPRB, 0.13131E+00_JPRB, 0.12993E+00_JPRB, 0.12878E+00_JPRB, 0.12994E+00_JPRB ,&
&  0.13062E+00_JPRB, 0.13108E+00_JPRB, 0.13210E+00_JPRB, 0.13230E+00_JPRB, 0.13356E+00_JPRB, 0.13551E+00_JPRB, 0.13809E+00_JPRB ,&
&  0.14010E+00_JPRB, 0.14282E+00_JPRB, 0.14445E+00_JPRB, 0.14527E+00_JPRB, 0.14632E+00_JPRB, 0.14699E+00_JPRB, 0.14664E+00_JPRB ,&
&  0.14548E+00_JPRB, 0.14392E+00_JPRB, 0.14217E+00_JPRB, 0.13997E+00_JPRB, 0.13741E+00_JPRB, 0.13488E+00_JPRB, 0.13238E+00_JPRB ,&
&  0.13047E+00_JPRB, 0.12875E+00_JPRB, 0.12701E+00_JPRB, 0.12547E+00_JPRB, 0.12401E+00_JPRB, 0.12269E+00_JPRB, 0.12136E+00_JPRB ,&
&  0.12030E+00_JPRB, 0.11946E+00_JPRB, 0.11844E+00_JPRB, 0.11782E+00_JPRB, 0.11721E+00_JPRB, 0.11691E+00_JPRB, 0.11672E+00_JPRB ,&
&  0.11693E+00_JPRB, 0.11725E+00_JPRB, 0.11759E+00_JPRB, 0.11776E+00_JPRB, 0.11792E+00_JPRB, 0.11798E+00_JPRB, 0.11812E+00_JPRB ,&
&  0.11824E+00_JPRB, 0.11840E+00_JPRB, 0.11808E+00_JPRB, 0.11779E+00_JPRB, 0.11740E+00_JPRB, 0.11698E+00_JPRB, 0.11686E+00_JPRB ,&
&  0.11710E+00_JPRB, 0.11732E+00_JPRB, 0.11726E+00_JPRB, 0.11752E+00_JPRB, 0.11793E+00_JPRB, 0.11837E+00_JPRB, 0.11940E+00_JPRB ,&
&  0.11997E+00_JPRB, 0.12054E+00_JPRB, 0.12108E+00_JPRB, 0.12153E+00_JPRB, 0.12216E+00_JPRB, 0.12270E+00_JPRB, 0.12334E+00_JPRB ,&
&  0.12412E+00_JPRB, 0.12491E+00_JPRB, 0.12599E+00_JPRB, 0.12731E+00_JPRB, 0.13037E+00_JPRB, 0.13260E+00_JPRB, 0.13472E+00_JPRB ,&
&  0.13609E+00_JPRB, 0.13432E+00_JPRB, 0.14148E+00_JPRB, 0.14122E+00_JPRB, 0.14129E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.14864E+00_JPRB, 0.15098E+00_JPRB, 0.14997E+00_JPRB, 0.14998E+00_JPRB, 0.14966E+00_JPRB, 0.14969E+00_JPRB, 0.14899E+00_JPRB ,&
&  0.14877E+00_JPRB, 0.14660E+00_JPRB, 0.14601E+00_JPRB, 0.14431E+00_JPRB, 0.14249E+00_JPRB, 0.14013E+00_JPRB, 0.13778E+00_JPRB ,&
&  0.13538E+00_JPRB, 0.13323E+00_JPRB, 0.13171E+00_JPRB, 0.13087E+00_JPRB, 0.12962E+00_JPRB, 0.12933E+00_JPRB, 0.13058E+00_JPRB ,&
&  0.13086E+00_JPRB, 0.13176E+00_JPRB, 0.13198E+00_JPRB, 0.13322E+00_JPRB, 0.13484E+00_JPRB, 0.13594E+00_JPRB, 0.13837E+00_JPRB ,&
&  0.14040E+00_JPRB, 0.14183E+00_JPRB, 0.14655E+00_JPRB, 0.14938E+00_JPRB, 0.15115E+00_JPRB, 0.15351E+00_JPRB, 0.15311E+00_JPRB ,&
&  0.15203E+00_JPRB, 0.14938E+00_JPRB, 0.15269E+00_JPRB, 0.15127E+00_JPRB, 0.14531E+00_JPRB, 0.14228E+00_JPRB, 0.13728E+00_JPRB ,&
&  0.13392E+00_JPRB, 0.13034E+00_JPRB, 0.12761E+00_JPRB, 0.12556E+00_JPRB, 0.12312E+00_JPRB, 0.12151E+00_JPRB, 0.11972E+00_JPRB ,&
&  0.11800E+00_JPRB, 0.11714E+00_JPRB, 0.11589E+00_JPRB, 0.11560E+00_JPRB, 0.11558E+00_JPRB, 0.11581E+00_JPRB, 0.11625E+00_JPRB ,&
&  0.11689E+00_JPRB, 0.11751E+00_JPRB, 0.11808E+00_JPRB, 0.11859E+00_JPRB, 0.11901E+00_JPRB, 0.11929E+00_JPRB, 0.11921E+00_JPRB ,&
&  0.11928E+00_JPRB, 0.11915E+00_JPRB, 0.11873E+00_JPRB, 0.11846E+00_JPRB, 0.11767E+00_JPRB, 0.11728E+00_JPRB, 0.11689E+00_JPRB ,&
&  0.11672E+00_JPRB, 0.11673E+00_JPRB, 0.11668E+00_JPRB, 0.11686E+00_JPRB, 0.11714E+00_JPRB, 0.11774E+00_JPRB, 0.11826E+00_JPRB ,&
&  0.11868E+00_JPRB, 0.11941E+00_JPRB, 0.11987E+00_JPRB, 0.12049E+00_JPRB, 0.12135E+00_JPRB, 0.12182E+00_JPRB, 0.12285E+00_JPRB ,&
&  0.12379E+00_JPRB, 0.12485E+00_JPRB, 0.12669E+00_JPRB, 0.12856E+00_JPRB, 0.13119E+00_JPRB, 0.13470E+00_JPRB, 0.13655E+00_JPRB ,&
&  0.13845E+00_JPRB, 0.14079E+00_JPRB, 0.14051E+00_JPRB, 0.14463E+00_JPRB, 0.14651E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.15054E+00_JPRB, 0.15138E+00_JPRB, 0.14988E+00_JPRB, 0.15042E+00_JPRB, 0.15125E+00_JPRB, 0.15172E+00_JPRB, 0.15217E+00_JPRB ,&
&  0.14990E+00_JPRB, 0.14885E+00_JPRB, 0.14654E+00_JPRB, 0.14424E+00_JPRB, 0.14166E+00_JPRB, 0.13890E+00_JPRB, 0.13674E+00_JPRB ,&
&  0.13497E+00_JPRB, 0.13414E+00_JPRB, 0.13334E+00_JPRB, 0.13385E+00_JPRB, 0.13390E+00_JPRB, 0.13534E+00_JPRB, 0.13653E+00_JPRB ,&
&  0.13728E+00_JPRB, 0.13817E+00_JPRB, 0.13720E+00_JPRB, 0.13733E+00_JPRB, 0.13912E+00_JPRB, 0.14090E+00_JPRB, 0.14353E+00_JPRB ,&
&  0.14638E+00_JPRB, 0.14927E+00_JPRB, 0.15383E+00_JPRB, 0.15559E+00_JPRB, 0.15755E+00_JPRB, 0.15887E+00_JPRB, 0.15894E+00_JPRB ,&
&  0.15870E+00_JPRB, 0.15899E+00_JPRB, 0.15993E+00_JPRB, 0.15980E+00_JPRB, 0.15342E+00_JPRB, 0.14836E+00_JPRB, 0.14310E+00_JPRB ,&
&  0.13808E+00_JPRB, 0.13339E+00_JPRB, 0.12929E+00_JPRB, 0.12653E+00_JPRB, 0.12388E+00_JPRB, 0.12193E+00_JPRB, 0.12041E+00_JPRB ,&
&  0.11883E+00_JPRB, 0.11783E+00_JPRB, 0.11663E+00_JPRB, 0.11623E+00_JPRB, 0.11640E+00_JPRB, 0.11698E+00_JPRB, 0.11777E+00_JPRB ,&
&  0.11871E+00_JPRB, 0.11939E+00_JPRB, 0.12024E+00_JPRB, 0.12109E+00_JPRB, 0.12170E+00_JPRB, 0.12211E+00_JPRB, 0.12195E+00_JPRB ,&
&  0.12213E+00_JPRB, 0.12231E+00_JPRB, 0.12192E+00_JPRB, 0.12180E+00_JPRB, 0.12083E+00_JPRB, 0.11974E+00_JPRB, 0.11858E+00_JPRB ,&
&  0.11712E+00_JPRB, 0.11598E+00_JPRB, 0.11555E+00_JPRB, 0.11530E+00_JPRB, 0.11584E+00_JPRB, 0.11669E+00_JPRB, 0.11728E+00_JPRB ,&
&  0.11763E+00_JPRB, 0.11762E+00_JPRB, 0.11772E+00_JPRB, 0.11868E+00_JPRB, 0.11969E+00_JPRB, 0.12096E+00_JPRB, 0.12227E+00_JPRB ,&
&  0.12277E+00_JPRB, 0.12388E+00_JPRB, 0.12488E+00_JPRB, 0.12604E+00_JPRB, 0.12978E+00_JPRB, 0.13296E+00_JPRB, 0.13904E+00_JPRB ,&
&  0.14281E+00_JPRB, 0.14642E+00_JPRB, 0.14757E+00_JPRB, 0.15052E+00_JPRB, 0.15020E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.15438E+00_JPRB, 0.15633E+00_JPRB, 0.15826E+00_JPRB, 0.15742E+00_JPRB, 0.15758E+00_JPRB, 0.15766E+00_JPRB, 0.15773E+00_JPRB ,&
&  0.15773E+00_JPRB, 0.15721E+00_JPRB, 0.15738E+00_JPRB, 0.15602E+00_JPRB, 0.15518E+00_JPRB, 0.15144E+00_JPRB, 0.14985E+00_JPRB ,&
&  0.14784E+00_JPRB, 0.14716E+00_JPRB, 0.14730E+00_JPRB, 0.14824E+00_JPRB, 0.14872E+00_JPRB, 0.14985E+00_JPRB, 0.14932E+00_JPRB ,&
&  0.14941E+00_JPRB, 0.14917E+00_JPRB, 0.14804E+00_JPRB, 0.14775E+00_JPRB, 0.14827E+00_JPRB, 0.14893E+00_JPRB, 0.14927E+00_JPRB ,&
&  0.15081E+00_JPRB, 0.15289E+00_JPRB, 0.15494E+00_JPRB, 0.15645E+00_JPRB, 0.16174E+00_JPRB, 0.16333E+00_JPRB, 0.16497E+00_JPRB ,&
&  0.16608E+00_JPRB, 0.16760E+00_JPRB, 0.16855E+00_JPRB, 0.16805E+00_JPRB, 0.16695E+00_JPRB, 0.16391E+00_JPRB, 0.16269E+00_JPRB ,&
&  0.15268E+00_JPRB, 0.14602E+00_JPRB, 0.13941E+00_JPRB, 0.13369E+00_JPRB, 0.12969E+00_JPRB, 0.12693E+00_JPRB, 0.12463E+00_JPRB ,&
&  0.12271E+00_JPRB, 0.12091E+00_JPRB, 0.11901E+00_JPRB, 0.11894E+00_JPRB, 0.11957E+00_JPRB, 0.12052E+00_JPRB, 0.12144E+00_JPRB ,&
&  0.12249E+00_JPRB, 0.12311E+00_JPRB, 0.12343E+00_JPRB, 0.12409E+00_JPRB, 0.12429E+00_JPRB, 0.12465E+00_JPRB, 0.12484E+00_JPRB ,&
&  0.12591E+00_JPRB, 0.12627E+00_JPRB, 0.12634E+00_JPRB, 0.12736E+00_JPRB, 0.12601E+00_JPRB, 0.12377E+00_JPRB, 0.12118E+00_JPRB ,&
&  0.11876E+00_JPRB, 0.11722E+00_JPRB, 0.11595E+00_JPRB, 0.11545E+00_JPRB, 0.11574E+00_JPRB, 0.11632E+00_JPRB, 0.11714E+00_JPRB ,&
&  0.11738E+00_JPRB, 0.11789E+00_JPRB, 0.11830E+00_JPRB, 0.11906E+00_JPRB, 0.11989E+00_JPRB, 0.12080E+00_JPRB, 0.12176E+00_JPRB ,&
&  0.12320E+00_JPRB, 0.12486E+00_JPRB, 0.12655E+00_JPRB, 0.12829E+00_JPRB, 0.13105E+00_JPRB, 0.13449E+00_JPRB, 0.14003E+00_JPRB ,&
&  0.14557E+00_JPRB, 0.15414E+00_JPRB, 0.15394E+00_JPRB, 0.15519E+00_JPRB, 0.15524E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.16062E+00_JPRB, 0.16415E+00_JPRB, 0.16581E+00_JPRB, 0.16591E+00_JPRB, 0.16495E+00_JPRB, 0.16417E+00_JPRB, 0.16405E+00_JPRB ,&
&  0.16400E+00_JPRB, 0.16296E+00_JPRB, 0.16216E+00_JPRB, 0.16146E+00_JPRB, 0.16072E+00_JPRB, 0.16004E+00_JPRB, 0.16031E+00_JPRB ,&
&  0.16144E+00_JPRB, 0.16266E+00_JPRB, 0.16341E+00_JPRB, 0.16353E+00_JPRB, 0.16301E+00_JPRB, 0.16324E+00_JPRB, 0.16403E+00_JPRB ,&
&  0.16399E+00_JPRB, 0.16448E+00_JPRB, 0.16774E+00_JPRB, 0.16846E+00_JPRB, 0.16948E+00_JPRB, 0.16973E+00_JPRB, 0.16982E+00_JPRB ,&
&  0.16569E+00_JPRB, 0.16075E+00_JPRB, 0.15726E+00_JPRB, 0.15905E+00_JPRB, 0.16747E+00_JPRB, 0.16920E+00_JPRB, 0.17202E+00_JPRB ,&
&  0.17395E+00_JPRB, 0.17510E+00_JPRB, 0.17624E+00_JPRB, 0.17737E+00_JPRB, 0.17873E+00_JPRB, 0.17775E+00_JPRB, 0.17564E+00_JPRB ,&
&  0.17321E+00_JPRB, 0.17225E+00_JPRB, 0.16917E+00_JPRB, 0.15606E+00_JPRB, 0.14776E+00_JPRB, 0.14308E+00_JPRB, 0.13827E+00_JPRB ,&
&  0.13395E+00_JPRB, 0.13127E+00_JPRB, 0.13042E+00_JPRB, 0.12938E+00_JPRB, 0.13016E+00_JPRB, 0.13169E+00_JPRB, 0.13235E+00_JPRB ,&
&  0.13206E+00_JPRB, 0.13237E+00_JPRB, 0.13208E+00_JPRB, 0.13125E+00_JPRB, 0.12987E+00_JPRB, 0.12898E+00_JPRB, 0.12845E+00_JPRB ,&
&  0.12884E+00_JPRB, 0.12875E+00_JPRB, 0.13234E+00_JPRB, 0.13388E+00_JPRB, 0.13025E+00_JPRB, 0.12827E+00_JPRB, 0.12603E+00_JPRB ,&
&  0.12178E+00_JPRB, 0.11934E+00_JPRB, 0.11804E+00_JPRB, 0.11753E+00_JPRB, 0.11832E+00_JPRB, 0.11948E+00_JPRB, 0.12015E+00_JPRB ,&
&  0.12040E+00_JPRB, 0.12121E+00_JPRB, 0.12162E+00_JPRB, 0.12210E+00_JPRB, 0.12243E+00_JPRB, 0.12312E+00_JPRB, 0.12413E+00_JPRB ,&
&  0.12543E+00_JPRB, 0.12771E+00_JPRB, 0.13003E+00_JPRB, 0.13266E+00_JPRB, 0.13429E+00_JPRB, 0.13559E+00_JPRB, 0.13840E+00_JPRB ,&
&  0.14050E+00_JPRB, 0.14678E+00_JPRB, 0.15614E+00_JPRB, 0.16056E+00_JPRB, 0.15675E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.17151E+00_JPRB, 0.17342E+00_JPRB, 0.17410E+00_JPRB, 0.17421E+00_JPRB, 0.17416E+00_JPRB, 0.17470E+00_JPRB, 0.17524E+00_JPRB ,&
&  0.17522E+00_JPRB, 0.17453E+00_JPRB, 0.17339E+00_JPRB, 0.17158E+00_JPRB, 0.16946E+00_JPRB, 0.16830E+00_JPRB, 0.16936E+00_JPRB ,&
&  0.17018E+00_JPRB, 0.16983E+00_JPRB, 0.16960E+00_JPRB, 0.16923E+00_JPRB, 0.16861E+00_JPRB, 0.16863E+00_JPRB, 0.16968E+00_JPRB ,&
&  0.17110E+00_JPRB, 0.17216E+00_JPRB, 0.17363E+00_JPRB, 0.17491E+00_JPRB, 0.17694E+00_JPRB, 0.17877E+00_JPRB, 0.17971E+00_JPRB ,&
&  0.17801E+00_JPRB, 0.17040E+00_JPRB, 0.16474E+00_JPRB, 0.16706E+00_JPRB, 0.17592E+00_JPRB, 0.18039E+00_JPRB, 0.18295E+00_JPRB ,&
&  0.18464E+00_JPRB, 0.18556E+00_JPRB, 0.18579E+00_JPRB, 0.18702E+00_JPRB, 0.18886E+00_JPRB, 0.19059E+00_JPRB, 0.19026E+00_JPRB ,&
&  0.18835E+00_JPRB, 0.18661E+00_JPRB, 0.18410E+00_JPRB, 0.17992E+00_JPRB, 0.17436E+00_JPRB, 0.16891E+00_JPRB, 0.16438E+00_JPRB ,&
&  0.16045E+00_JPRB, 0.15659E+00_JPRB, 0.15321E+00_JPRB, 0.15102E+00_JPRB, 0.15025E+00_JPRB, 0.15121E+00_JPRB, 0.15109E+00_JPRB ,&
&  0.15058E+00_JPRB, 0.14891E+00_JPRB, 0.14680E+00_JPRB, 0.14439E+00_JPRB, 0.14226E+00_JPRB, 0.13874E+00_JPRB, 0.13459E+00_JPRB ,&
&  0.13236E+00_JPRB, 0.13422E+00_JPRB, 0.13999E+00_JPRB, 0.14516E+00_JPRB, 0.14787E+00_JPRB, 0.14017E+00_JPRB, 0.13574E+00_JPRB ,&
&  0.13173E+00_JPRB, 0.12812E+00_JPRB, 0.12589E+00_JPRB, 0.12535E+00_JPRB, 0.12548E+00_JPRB, 0.12566E+00_JPRB, 0.12645E+00_JPRB ,&
&  0.12772E+00_JPRB, 0.12887E+00_JPRB, 0.12955E+00_JPRB, 0.13020E+00_JPRB, 0.13046E+00_JPRB, 0.13051E+00_JPRB, 0.13062E+00_JPRB ,&
&  0.13182E+00_JPRB, 0.13447E+00_JPRB, 0.13822E+00_JPRB, 0.14255E+00_JPRB, 0.14596E+00_JPRB, 0.14908E+00_JPRB, 0.15224E+00_JPRB ,&
&  0.15578E+00_JPRB, 0.15875E+00_JPRB, 0.16156E+00_JPRB, 0.16516E+00_JPRB, 0.16848E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.18105E+00_JPRB, 0.18314E+00_JPRB, 0.18400E+00_JPRB, 0.18381E+00_JPRB, 0.18355E+00_JPRB, 0.18453E+00_JPRB, 0.18581E+00_JPRB ,&
&  0.18643E+00_JPRB, 0.18645E+00_JPRB, 0.18569E+00_JPRB, 0.18423E+00_JPRB, 0.18324E+00_JPRB, 0.18365E+00_JPRB, 0.18214E+00_JPRB ,&
&  0.18054E+00_JPRB, 0.18040E+00_JPRB, 0.17885E+00_JPRB, 0.17720E+00_JPRB, 0.17477E+00_JPRB, 0.17322E+00_JPRB, 0.17362E+00_JPRB ,&
&  0.17563E+00_JPRB, 0.17638E+00_JPRB, 0.17724E+00_JPRB, 0.17840E+00_JPRB, 0.18049E+00_JPRB, 0.18286E+00_JPRB, 0.18462E+00_JPRB ,&
&  0.18607E+00_JPRB, 0.18584E+00_JPRB, 0.18405E+00_JPRB, 0.18707E+00_JPRB, 0.18978E+00_JPRB, 0.19314E+00_JPRB, 0.19530E+00_JPRB ,&
&  0.19705E+00_JPRB, 0.19835E+00_JPRB, 0.19891E+00_JPRB, 0.19881E+00_JPRB, 0.19922E+00_JPRB, 0.20085E+00_JPRB, 0.20197E+00_JPRB ,&
&  0.20227E+00_JPRB, 0.20165E+00_JPRB, 0.20013E+00_JPRB, 0.19859E+00_JPRB, 0.19523E+00_JPRB, 0.19034E+00_JPRB, 0.18333E+00_JPRB ,&
&  0.17616E+00_JPRB, 0.17049E+00_JPRB, 0.16617E+00_JPRB, 0.16263E+00_JPRB, 0.16011E+00_JPRB, 0.15964E+00_JPRB, 0.15952E+00_JPRB ,&
&  0.15918E+00_JPRB, 0.15890E+00_JPRB, 0.16034E+00_JPRB, 0.16136E+00_JPRB, 0.15848E+00_JPRB, 0.15297E+00_JPRB, 0.14882E+00_JPRB ,&
&  0.14796E+00_JPRB, 0.14964E+00_JPRB, 0.15242E+00_JPRB, 0.15391E+00_JPRB, 0.15653E+00_JPRB, 0.15721E+00_JPRB, 0.15644E+00_JPRB ,&
&  0.15192E+00_JPRB, 0.14613E+00_JPRB, 0.14293E+00_JPRB, 0.14069E+00_JPRB, 0.13921E+00_JPRB, 0.13878E+00_JPRB, 0.13927E+00_JPRB ,&
&  0.14107E+00_JPRB, 0.14310E+00_JPRB, 0.14520E+00_JPRB, 0.14614E+00_JPRB, 0.14576E+00_JPRB, 0.14511E+00_JPRB, 0.14431E+00_JPRB ,&
&  0.14508E+00_JPRB, 0.14865E+00_JPRB, 0.15445E+00_JPRB, 0.15842E+00_JPRB, 0.16171E+00_JPRB, 0.16390E+00_JPRB, 0.16512E+00_JPRB ,&
&  0.16608E+00_JPRB, 0.16683E+00_JPRB, 0.16783E+00_JPRB, 0.17217E+00_JPRB, 0.17769E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.19225E+00_JPRB, 0.19450E+00_JPRB, 0.19582E+00_JPRB, 0.19598E+00_JPRB, 0.19580E+00_JPRB, 0.19665E+00_JPRB, 0.19714E+00_JPRB ,&
&  0.19762E+00_JPRB, 0.19788E+00_JPRB, 0.19702E+00_JPRB, 0.19631E+00_JPRB, 0.19803E+00_JPRB, 0.19754E+00_JPRB, 0.19698E+00_JPRB ,&
&  0.19536E+00_JPRB, 0.19348E+00_JPRB, 0.19071E+00_JPRB, 0.18753E+00_JPRB, 0.18362E+00_JPRB, 0.18105E+00_JPRB, 0.18116E+00_JPRB ,&
&  0.18250E+00_JPRB, 0.18304E+00_JPRB, 0.18395E+00_JPRB, 0.18528E+00_JPRB, 0.18790E+00_JPRB, 0.19119E+00_JPRB, 0.19372E+00_JPRB ,&
&  0.19582E+00_JPRB, 0.19910E+00_JPRB, 0.20117E+00_JPRB, 0.19969E+00_JPRB, 0.20392E+00_JPRB, 0.20748E+00_JPRB, 0.21031E+00_JPRB ,&
&  0.21229E+00_JPRB, 0.21321E+00_JPRB, 0.21288E+00_JPRB, 0.21182E+00_JPRB, 0.21110E+00_JPRB, 0.21208E+00_JPRB, 0.21248E+00_JPRB ,&
&  0.21272E+00_JPRB, 0.21439E+00_JPRB, 0.21645E+00_JPRB, 0.21683E+00_JPRB, 0.21501E+00_JPRB, 0.21097E+00_JPRB, 0.20487E+00_JPRB ,&
&  0.19718E+00_JPRB, 0.18921E+00_JPRB, 0.18282E+00_JPRB, 0.17772E+00_JPRB, 0.17378E+00_JPRB, 0.17045E+00_JPRB, 0.16774E+00_JPRB ,&
&  0.16483E+00_JPRB, 0.16288E+00_JPRB, 0.16298E+00_JPRB, 0.16374E+00_JPRB, 0.16425E+00_JPRB, 0.16347E+00_JPRB, 0.16291E+00_JPRB ,&
&  0.16230E+00_JPRB, 0.16278E+00_JPRB, 0.16354E+00_JPRB, 0.16385E+00_JPRB, 0.16393E+00_JPRB, 0.16349E+00_JPRB, 0.16308E+00_JPRB ,&
&  0.16370E+00_JPRB, 0.16408E+00_JPRB, 0.16333E+00_JPRB, 0.16174E+00_JPRB, 0.16038E+00_JPRB, 0.16007E+00_JPRB, 0.16010E+00_JPRB ,&
&  0.16041E+00_JPRB, 0.16119E+00_JPRB, 0.16253E+00_JPRB, 0.16399E+00_JPRB, 0.16513E+00_JPRB, 0.16559E+00_JPRB, 0.16565E+00_JPRB ,&
&  0.16570E+00_JPRB, 0.16538E+00_JPRB, 0.16591E+00_JPRB, 0.16822E+00_JPRB, 0.17091E+00_JPRB, 0.17323E+00_JPRB, 0.17466E+00_JPRB ,&
&  0.17451E+00_JPRB, 0.17541E+00_JPRB, 0.17851E+00_JPRB, 0.18435E+00_JPRB, 0.18880E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.20773E+00_JPRB, 0.21021E+00_JPRB, 0.21288E+00_JPRB, 0.21333E+00_JPRB, 0.21200E+00_JPRB, 0.21141E+00_JPRB, 0.21160E+00_JPRB ,&
&  0.21202E+00_JPRB, 0.21236E+00_JPRB, 0.21146E+00_JPRB, 0.21154E+00_JPRB, 0.21170E+00_JPRB, 0.21083E+00_JPRB, 0.21007E+00_JPRB ,&
&  0.20873E+00_JPRB, 0.20588E+00_JPRB, 0.20199E+00_JPRB, 0.19721E+00_JPRB, 0.19361E+00_JPRB, 0.19166E+00_JPRB, 0.19128E+00_JPRB ,&
&  0.19184E+00_JPRB, 0.19230E+00_JPRB, 0.19480E+00_JPRB, 0.19714E+00_JPRB, 0.20018E+00_JPRB, 0.20410E+00_JPRB, 0.20812E+00_JPRB ,&
&  0.21118E+00_JPRB, 0.21430E+00_JPRB, 0.21724E+00_JPRB, 0.21989E+00_JPRB, 0.22322E+00_JPRB, 0.22804E+00_JPRB, 0.23161E+00_JPRB ,&
&  0.23475E+00_JPRB, 0.23558E+00_JPRB, 0.23270E+00_JPRB, 0.23000E+00_JPRB, 0.22986E+00_JPRB, 0.23118E+00_JPRB, 0.23227E+00_JPRB ,&
&  0.23299E+00_JPRB, 0.23387E+00_JPRB, 0.23464E+00_JPRB, 0.23404E+00_JPRB, 0.23132E+00_JPRB, 0.22679E+00_JPRB, 0.22101E+00_JPRB ,&
&  0.21479E+00_JPRB, 0.20868E+00_JPRB, 0.20295E+00_JPRB, 0.19733E+00_JPRB, 0.19256E+00_JPRB, 0.18784E+00_JPRB, 0.18337E+00_JPRB ,&
&  0.17914E+00_JPRB, 0.17513E+00_JPRB, 0.17215E+00_JPRB, 0.17046E+00_JPRB, 0.17044E+00_JPRB, 0.16984E+00_JPRB, 0.17016E+00_JPRB ,&
&  0.17001E+00_JPRB, 0.17009E+00_JPRB, 0.17055E+00_JPRB, 0.17165E+00_JPRB, 0.17314E+00_JPRB, 0.17324E+00_JPRB, 0.17213E+00_JPRB ,&
&  0.17148E+00_JPRB, 0.17251E+00_JPRB, 0.17227E+00_JPRB, 0.17012E+00_JPRB, 0.16742E+00_JPRB, 0.16667E+00_JPRB, 0.16778E+00_JPRB ,&
&  0.17004E+00_JPRB, 0.17221E+00_JPRB, 0.17406E+00_JPRB, 0.17582E+00_JPRB, 0.17729E+00_JPRB, 0.17912E+00_JPRB, 0.18079E+00_JPRB ,&
&  0.18216E+00_JPRB, 0.18272E+00_JPRB, 0.18396E+00_JPRB, 0.18530E+00_JPRB, 0.18676E+00_JPRB, 0.18784E+00_JPRB, 0.18776E+00_JPRB ,&
&  0.18742E+00_JPRB, 0.18886E+00_JPRB, 0.19340E+00_JPRB, 0.20046E+00_JPRB, 0.20495E+00_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.22684E+00_JPRB, 0.22887E+00_JPRB, 0.23163E+00_JPRB, 0.23402E+00_JPRB, 0.23507E+00_JPRB, 0.23475E+00_JPRB, 0.23409E+00_JPRB ,&
&  0.23413E+00_JPRB, 0.23435E+00_JPRB, 0.23315E+00_JPRB, 0.23092E+00_JPRB, 0.22861E+00_JPRB, 0.22609E+00_JPRB, 0.22414E+00_JPRB ,&
&  0.22217E+00_JPRB, 0.22011E+00_JPRB, 0.21664E+00_JPRB, 0.21166E+00_JPRB, 0.20650E+00_JPRB, 0.20349E+00_JPRB, 0.20203E+00_JPRB ,&
&  0.20194E+00_JPRB, 0.20247E+00_JPRB, 0.20505E+00_JPRB, 0.20697E+00_JPRB, 0.20957E+00_JPRB, 0.21375E+00_JPRB, 0.21937E+00_JPRB ,&
&  0.22469E+00_JPRB, 0.22969E+00_JPRB, 0.23467E+00_JPRB, 0.23876E+00_JPRB, 0.24271E+00_JPRB, 0.24716E+00_JPRB, 0.25125E+00_JPRB ,&
&  0.25535E+00_JPRB, 0.25804E+00_JPRB, 0.25830E+00_JPRB, 0.25923E+00_JPRB, 0.26130E+00_JPRB, 0.26346E+00_JPRB, 0.26459E+00_JPRB ,&
&  0.26532E+00_JPRB, 0.26345E+00_JPRB, 0.26082E+00_JPRB, 0.25722E+00_JPRB, 0.25345E+00_JPRB, 0.24931E+00_JPRB, 0.24370E+00_JPRB ,&
&  0.23696E+00_JPRB, 0.22967E+00_JPRB, 0.22281E+00_JPRB, 0.21635E+00_JPRB, 0.21021E+00_JPRB, 0.20452E+00_JPRB, 0.19924E+00_JPRB ,&
&  0.19398E+00_JPRB, 0.18992E+00_JPRB, 0.18604E+00_JPRB, 0.18391E+00_JPRB, 0.18291E+00_JPRB, 0.18162E+00_JPRB, 0.18170E+00_JPRB ,&
&  0.18122E+00_JPRB, 0.18105E+00_JPRB, 0.18104E+00_JPRB, 0.18272E+00_JPRB, 0.18577E+00_JPRB, 0.18755E+00_JPRB, 0.18646E+00_JPRB ,&
&  0.18328E+00_JPRB, 0.18128E+00_JPRB, 0.18019E+00_JPRB, 0.17930E+00_JPRB, 0.17703E+00_JPRB, 0.17657E+00_JPRB, 0.17818E+00_JPRB ,&
&  0.18177E+00_JPRB, 0.18578E+00_JPRB, 0.18944E+00_JPRB, 0.19160E+00_JPRB, 0.19372E+00_JPRB, 0.19527E+00_JPRB, 0.19663E+00_JPRB ,&
&  0.19827E+00_JPRB, 0.19950E+00_JPRB, 0.20048E+00_JPRB, 0.20186E+00_JPRB, 0.20420E+00_JPRB, 0.20657E+00_JPRB, 0.20891E+00_JPRB ,&
&  0.21135E+00_JPRB, 0.21504E+00_JPRB, 0.21880E+00_JPRB, 0.22154E+00_JPRB, 0.22480E+00_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.25402E+00_JPRB, 0.25562E+00_JPRB, 0.25765E+00_JPRB, 0.25820E+00_JPRB, 0.25785E+00_JPRB, 0.25802E+00_JPRB, 0.25717E+00_JPRB ,&
&  0.25513E+00_JPRB, 0.25410E+00_JPRB, 0.25442E+00_JPRB, 0.25319E+00_JPRB, 0.25063E+00_JPRB, 0.24746E+00_JPRB, 0.24396E+00_JPRB ,&
&  0.24120E+00_JPRB, 0.23838E+00_JPRB, 0.23368E+00_JPRB, 0.22734E+00_JPRB, 0.22188E+00_JPRB, 0.21843E+00_JPRB, 0.21676E+00_JPRB ,&
&  0.21621E+00_JPRB, 0.21647E+00_JPRB, 0.21635E+00_JPRB, 0.21717E+00_JPRB, 0.21935E+00_JPRB, 0.22408E+00_JPRB, 0.23104E+00_JPRB ,&
&  0.23811E+00_JPRB, 0.24417E+00_JPRB, 0.25045E+00_JPRB, 0.25714E+00_JPRB, 0.26495E+00_JPRB, 0.27311E+00_JPRB, 0.27932E+00_JPRB ,&
&  0.28470E+00_JPRB, 0.28646E+00_JPRB, 0.28760E+00_JPRB, 0.29135E+00_JPRB, 0.29831E+00_JPRB, 0.30275E+00_JPRB, 0.30554E+00_JPRB ,&
&  0.30702E+00_JPRB, 0.30588E+00_JPRB, 0.30386E+00_JPRB, 0.29968E+00_JPRB, 0.29356E+00_JPRB, 0.28540E+00_JPRB, 0.27655E+00_JPRB ,&
&  0.26782E+00_JPRB, 0.25888E+00_JPRB, 0.24920E+00_JPRB, 0.24004E+00_JPRB, 0.23232E+00_JPRB, 0.22552E+00_JPRB, 0.21853E+00_JPRB ,&
&  0.21184E+00_JPRB, 0.20643E+00_JPRB, 0.20216E+00_JPRB, 0.19938E+00_JPRB, 0.19720E+00_JPRB, 0.19560E+00_JPRB, 0.19547E+00_JPRB ,&
&  0.19607E+00_JPRB, 0.19760E+00_JPRB, 0.19831E+00_JPRB, 0.19967E+00_JPRB, 0.20196E+00_JPRB, 0.20302E+00_JPRB, 0.20215E+00_JPRB ,&
&  0.19854E+00_JPRB, 0.19469E+00_JPRB, 0.19110E+00_JPRB, 0.19034E+00_JPRB, 0.19015E+00_JPRB, 0.19108E+00_JPRB, 0.19252E+00_JPRB ,&
&  0.19575E+00_JPRB, 0.20068E+00_JPRB, 0.20507E+00_JPRB, 0.20802E+00_JPRB, 0.21130E+00_JPRB, 0.21413E+00_JPRB, 0.21705E+00_JPRB ,&
&  0.22050E+00_JPRB, 0.22434E+00_JPRB, 0.22960E+00_JPRB, 0.23486E+00_JPRB, 0.23962E+00_JPRB, 0.24342E+00_JPRB, 0.24623E+00_JPRB ,&
&  0.24806E+00_JPRB, 0.24851E+00_JPRB, 0.24870E+00_JPRB, 0.25007E+00_JPRB, 0.25230E+00_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.28390E+00_JPRB, 0.28429E+00_JPRB, 0.28529E+00_JPRB, 0.28605E+00_JPRB, 0.28394E+00_JPRB, 0.28159E+00_JPRB, 0.27866E+00_JPRB ,&
&  0.27746E+00_JPRB, 0.27654E+00_JPRB, 0.27619E+00_JPRB, 0.27343E+00_JPRB, 0.27129E+00_JPRB, 0.27003E+00_JPRB, 0.26687E+00_JPRB ,&
&  0.26406E+00_JPRB, 0.25738E+00_JPRB, 0.25012E+00_JPRB, 0.24462E+00_JPRB, 0.24111E+00_JPRB, 0.23885E+00_JPRB, 0.23432E+00_JPRB ,&
&  0.23129E+00_JPRB, 0.22897E+00_JPRB, 0.22803E+00_JPRB, 0.22843E+00_JPRB, 0.23061E+00_JPRB, 0.23639E+00_JPRB, 0.24601E+00_JPRB ,&
&  0.25286E+00_JPRB, 0.25953E+00_JPRB, 0.25927E+00_JPRB, 0.27140E+00_JPRB, 0.28247E+00_JPRB, 0.29589E+00_JPRB, 0.30597E+00_JPRB ,&
&  0.31454E+00_JPRB, 0.32268E+00_JPRB, 0.33227E+00_JPRB, 0.34095E+00_JPRB, 0.34470E+00_JPRB, 0.34639E+00_JPRB, 0.34602E+00_JPRB ,&
&  0.34348E+00_JPRB, 0.33906E+00_JPRB, 0.33410E+00_JPRB, 0.32820E+00_JPRB, 0.31996E+00_JPRB, 0.31018E+00_JPRB, 0.30030E+00_JPRB ,&
&  0.29112E+00_JPRB, 0.28214E+00_JPRB, 0.27325E+00_JPRB, 0.26365E+00_JPRB, 0.25412E+00_JPRB, 0.24550E+00_JPRB, 0.23664E+00_JPRB ,&
&  0.22910E+00_JPRB, 0.22314E+00_JPRB, 0.21872E+00_JPRB, 0.21571E+00_JPRB, 0.21358E+00_JPRB, 0.21148E+00_JPRB, 0.21066E+00_JPRB ,&
&  0.21107E+00_JPRB, 0.21276E+00_JPRB, 0.21489E+00_JPRB, 0.21658E+00_JPRB, 0.21861E+00_JPRB, 0.21900E+00_JPRB, 0.21677E+00_JPRB ,&
&  0.21429E+00_JPRB, 0.21292E+00_JPRB, 0.20877E+00_JPRB, 0.20627E+00_JPRB, 0.20576E+00_JPRB, 0.20612E+00_JPRB, 0.20777E+00_JPRB ,&
&  0.21113E+00_JPRB, 0.21588E+00_JPRB, 0.22025E+00_JPRB, 0.22396E+00_JPRB, 0.22703E+00_JPRB, 0.23029E+00_JPRB, 0.23424E+00_JPRB ,&
&  0.23851E+00_JPRB, 0.24269E+00_JPRB, 0.24932E+00_JPRB, 0.25677E+00_JPRB, 0.26319E+00_JPRB, 0.27000E+00_JPRB, 0.27415E+00_JPRB ,&
&  0.27570E+00_JPRB, 0.27635E+00_JPRB, 0.27840E+00_JPRB, 0.28139E+00_JPRB, 0.28344E+00_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.32107E+00_JPRB, 0.31915E+00_JPRB, 0.31672E+00_JPRB, 0.31491E+00_JPRB, 0.31385E+00_JPRB, 0.31106E+00_JPRB, 0.30733E+00_JPRB ,&
&  0.30477E+00_JPRB, 0.30224E+00_JPRB, 0.30039E+00_JPRB, 0.29739E+00_JPRB, 0.29579E+00_JPRB, 0.29486E+00_JPRB, 0.29134E+00_JPRB ,&
&  0.28714E+00_JPRB, 0.28115E+00_JPRB, 0.27526E+00_JPRB, 0.26997E+00_JPRB, 0.26496E+00_JPRB, 0.26042E+00_JPRB, 0.25540E+00_JPRB ,&
&  0.25070E+00_JPRB, 0.24731E+00_JPRB, 0.24569E+00_JPRB, 0.24424E+00_JPRB, 0.24577E+00_JPRB, 0.25172E+00_JPRB, 0.26116E+00_JPRB ,&
&  0.26631E+00_JPRB, 0.26458E+00_JPRB, 0.27383E+00_JPRB, 0.28286E+00_JPRB, 0.29926E+00_JPRB, 0.31890E+00_JPRB, 0.33326E+00_JPRB ,&
&  0.34454E+00_JPRB, 0.35752E+00_JPRB, 0.37024E+00_JPRB, 0.37744E+00_JPRB, 0.37795E+00_JPRB, 0.37357E+00_JPRB, 0.37047E+00_JPRB ,&
&  0.36664E+00_JPRB, 0.35899E+00_JPRB, 0.35061E+00_JPRB, 0.33977E+00_JPRB, 0.32809E+00_JPRB, 0.31857E+00_JPRB, 0.31099E+00_JPRB ,&
&  0.30367E+00_JPRB, 0.29625E+00_JPRB, 0.28943E+00_JPRB, 0.28246E+00_JPRB, 0.27486E+00_JPRB, 0.26695E+00_JPRB, 0.25866E+00_JPRB ,&
&  0.25050E+00_JPRB, 0.24338E+00_JPRB, 0.23766E+00_JPRB, 0.23477E+00_JPRB, 0.23332E+00_JPRB, 0.23226E+00_JPRB, 0.23160E+00_JPRB ,&
&  0.23218E+00_JPRB, 0.23357E+00_JPRB, 0.23528E+00_JPRB, 0.23682E+00_JPRB, 0.23804E+00_JPRB, 0.23808E+00_JPRB, 0.23648E+00_JPRB ,&
&  0.23481E+00_JPRB, 0.23407E+00_JPRB, 0.23381E+00_JPRB, 0.23145E+00_JPRB, 0.22896E+00_JPRB, 0.22933E+00_JPRB, 0.23105E+00_JPRB ,&
&  0.23452E+00_JPRB, 0.23772E+00_JPRB, 0.24054E+00_JPRB, 0.24288E+00_JPRB, 0.24464E+00_JPRB, 0.24780E+00_JPRB, 0.25166E+00_JPRB ,&
&  0.25647E+00_JPRB, 0.26252E+00_JPRB, 0.27111E+00_JPRB, 0.28186E+00_JPRB, 0.29175E+00_JPRB, 0.29990E+00_JPRB, 0.30494E+00_JPRB ,&
&  0.30789E+00_JPRB, 0.31011E+00_JPRB, 0.31248E+00_JPRB, 0.31603E+00_JPRB, 0.31971E+00_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.35879E+00_JPRB, 0.35459E+00_JPRB, 0.34888E+00_JPRB, 0.34418E+00_JPRB, 0.34215E+00_JPRB, 0.34057E+00_JPRB, 0.33693E+00_JPRB ,&
&  0.33348E+00_JPRB, 0.33022E+00_JPRB, 0.32671E+00_JPRB, 0.32365E+00_JPRB, 0.32107E+00_JPRB, 0.31909E+00_JPRB, 0.31571E+00_JPRB ,&
&  0.31061E+00_JPRB, 0.30434E+00_JPRB, 0.29756E+00_JPRB, 0.29221E+00_JPRB, 0.28560E+00_JPRB, 0.27880E+00_JPRB, 0.27385E+00_JPRB ,&
&  0.26972E+00_JPRB, 0.26698E+00_JPRB, 0.26621E+00_JPRB, 0.26495E+00_JPRB, 0.26614E+00_JPRB, 0.26962E+00_JPRB, 0.27756E+00_JPRB ,&
&  0.28549E+00_JPRB, 0.26578E+00_JPRB, 0.27821E+00_JPRB, 0.28833E+00_JPRB, 0.30954E+00_JPRB, 0.33742E+00_JPRB, 0.35711E+00_JPRB ,&
&  0.37187E+00_JPRB, 0.38360E+00_JPRB, 0.39208E+00_JPRB, 0.39532E+00_JPRB, 0.39092E+00_JPRB, 0.38769E+00_JPRB, 0.38535E+00_JPRB ,&
&  0.37936E+00_JPRB, 0.37068E+00_JPRB, 0.35902E+00_JPRB, 0.34646E+00_JPRB, 0.33487E+00_JPRB, 0.32640E+00_JPRB, 0.31968E+00_JPRB ,&
&  0.31431E+00_JPRB, 0.30911E+00_JPRB, 0.30444E+00_JPRB, 0.29867E+00_JPRB, 0.29183E+00_JPRB, 0.28469E+00_JPRB, 0.27850E+00_JPRB ,&
&  0.27257E+00_JPRB, 0.26664E+00_JPRB, 0.26133E+00_JPRB, 0.25676E+00_JPRB, 0.25438E+00_JPRB, 0.25346E+00_JPRB, 0.25348E+00_JPRB ,&
&  0.25423E+00_JPRB, 0.25601E+00_JPRB, 0.25754E+00_JPRB, 0.25846E+00_JPRB, 0.25889E+00_JPRB, 0.25802E+00_JPRB, 0.25575E+00_JPRB ,&
&  0.25565E+00_JPRB, 0.25568E+00_JPRB, 0.25561E+00_JPRB, 0.25602E+00_JPRB, 0.25566E+00_JPRB, 0.25576E+00_JPRB, 0.25760E+00_JPRB ,&
&  0.25955E+00_JPRB, 0.26084E+00_JPRB, 0.26239E+00_JPRB, 0.26421E+00_JPRB, 0.26650E+00_JPRB, 0.27005E+00_JPRB, 0.27503E+00_JPRB ,&
&  0.28069E+00_JPRB, 0.28706E+00_JPRB, 0.29624E+00_JPRB, 0.30836E+00_JPRB, 0.32027E+00_JPRB, 0.33080E+00_JPRB, 0.33967E+00_JPRB ,&
&  0.34324E+00_JPRB, 0.34638E+00_JPRB, 0.35173E+00_JPRB, 0.35552E+00_JPRB, 0.35933E+00_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.40510E+00_JPRB, 0.39801E+00_JPRB, 0.38857E+00_JPRB, 0.38235E+00_JPRB, 0.37989E+00_JPRB, 0.37678E+00_JPRB, 0.37343E+00_JPRB ,&
&  0.36801E+00_JPRB, 0.36179E+00_JPRB, 0.35577E+00_JPRB, 0.35049E+00_JPRB, 0.34622E+00_JPRB, 0.34301E+00_JPRB, 0.33915E+00_JPRB ,&
&  0.33377E+00_JPRB, 0.32788E+00_JPRB, 0.32222E+00_JPRB, 0.31602E+00_JPRB, 0.30773E+00_JPRB, 0.30119E+00_JPRB, 0.29599E+00_JPRB ,&
&  0.29186E+00_JPRB, 0.28856E+00_JPRB, 0.28772E+00_JPRB, 0.28649E+00_JPRB, 0.28513E+00_JPRB, 0.28560E+00_JPRB, 0.29421E+00_JPRB ,&
&  0.28584E+00_JPRB, 0.26165E+00_JPRB, 0.27536E+00_JPRB, 0.29339E+00_JPRB, 0.31890E+00_JPRB, 0.35582E+00_JPRB, 0.38166E+00_JPRB ,&
&  0.39734E+00_JPRB, 0.40586E+00_JPRB, 0.41006E+00_JPRB, 0.40946E+00_JPRB, 0.40676E+00_JPRB, 0.40640E+00_JPRB, 0.40163E+00_JPRB ,&
&  0.39190E+00_JPRB, 0.38017E+00_JPRB, 0.36811E+00_JPRB, 0.35924E+00_JPRB, 0.35042E+00_JPRB, 0.34424E+00_JPRB, 0.33754E+00_JPRB ,&
&  0.33056E+00_JPRB, 0.32444E+00_JPRB, 0.32119E+00_JPRB, 0.31891E+00_JPRB, 0.31529E+00_JPRB, 0.30964E+00_JPRB, 0.30417E+00_JPRB ,&
&  0.30012E+00_JPRB, 0.29711E+00_JPRB, 0.29410E+00_JPRB, 0.29088E+00_JPRB, 0.28933E+00_JPRB, 0.28839E+00_JPRB, 0.28674E+00_JPRB ,&
&  0.28603E+00_JPRB, 0.28574E+00_JPRB, 0.28561E+00_JPRB, 0.28528E+00_JPRB, 0.28444E+00_JPRB, 0.28319E+00_JPRB, 0.28377E+00_JPRB ,&
&  0.28560E+00_JPRB, 0.28495E+00_JPRB, 0.28458E+00_JPRB, 0.28629E+00_JPRB, 0.28813E+00_JPRB, 0.28943E+00_JPRB, 0.28970E+00_JPRB ,&
&  0.28901E+00_JPRB, 0.28927E+00_JPRB, 0.29095E+00_JPRB, 0.29429E+00_JPRB, 0.29826E+00_JPRB, 0.30394E+00_JPRB, 0.31099E+00_JPRB ,&
&  0.31849E+00_JPRB, 0.32775E+00_JPRB, 0.34048E+00_JPRB, 0.35942E+00_JPRB, 0.37433E+00_JPRB, 0.38594E+00_JPRB, 0.39498E+00_JPRB ,&
&  0.39675E+00_JPRB, 0.39683E+00_JPRB, 0.40226E+00_JPRB, 0.40416E+00_JPRB, 0.40610E+00_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.42554E+00_JPRB, 0.42794E+00_JPRB, 0.42402E+00_JPRB, 0.41800E+00_JPRB, 0.41220E+00_JPRB, 0.40568E+00_JPRB, 0.40011E+00_JPRB ,&
&  0.39210E+00_JPRB, 0.38266E+00_JPRB, 0.37599E+00_JPRB, 0.37313E+00_JPRB, 0.37153E+00_JPRB, 0.36871E+00_JPRB, 0.36370E+00_JPRB ,&
&  0.35742E+00_JPRB, 0.35112E+00_JPRB, 0.34541E+00_JPRB, 0.33857E+00_JPRB, 0.33204E+00_JPRB, 0.32623E+00_JPRB, 0.32197E+00_JPRB ,&
&  0.31776E+00_JPRB, 0.31307E+00_JPRB, 0.30845E+00_JPRB, 0.30558E+00_JPRB, 0.30312E+00_JPRB, 0.30194E+00_JPRB, 0.30965E+00_JPRB ,&
&  0.33301E+00_JPRB, 0.28723E+00_JPRB, 0.30117E+00_JPRB, 0.32054E+00_JPRB, 0.34729E+00_JPRB, 0.38066E+00_JPRB, 0.40160E+00_JPRB ,&
&  0.41483E+00_JPRB, 0.42433E+00_JPRB, 0.42823E+00_JPRB, 0.43048E+00_JPRB, 0.43277E+00_JPRB, 0.42664E+00_JPRB, 0.41382E+00_JPRB ,&
&  0.40112E+00_JPRB, 0.38844E+00_JPRB, 0.37799E+00_JPRB, 0.37040E+00_JPRB, 0.36504E+00_JPRB, 0.36229E+00_JPRB, 0.35702E+00_JPRB ,&
&  0.34706E+00_JPRB, 0.34183E+00_JPRB, 0.33937E+00_JPRB, 0.33705E+00_JPRB, 0.33569E+00_JPRB, 0.33364E+00_JPRB, 0.32799E+00_JPRB ,&
&  0.32264E+00_JPRB, 0.31986E+00_JPRB, 0.31907E+00_JPRB, 0.31856E+00_JPRB, 0.31873E+00_JPRB, 0.31763E+00_JPRB, 0.31524E+00_JPRB ,&
&  0.31238E+00_JPRB, 0.30867E+00_JPRB, 0.30638E+00_JPRB, 0.30455E+00_JPRB, 0.30309E+00_JPRB, 0.30478E+00_JPRB, 0.30750E+00_JPRB ,&
&  0.30909E+00_JPRB, 0.31086E+00_JPRB, 0.31364E+00_JPRB, 0.31571E+00_JPRB, 0.31619E+00_JPRB, 0.31568E+00_JPRB, 0.31469E+00_JPRB ,&
&  0.31360E+00_JPRB, 0.31338E+00_JPRB, 0.31515E+00_JPRB, 0.32012E+00_JPRB, 0.32681E+00_JPRB, 0.33473E+00_JPRB, 0.34362E+00_JPRB ,&
&  0.35399E+00_JPRB, 0.36536E+00_JPRB, 0.37918E+00_JPRB, 0.40382E+00_JPRB, 0.41744E+00_JPRB, 0.42989E+00_JPRB, 0.44077E+00_JPRB ,&
&  0.44293E+00_JPRB, 0.44156E+00_JPRB, 0.37446E+00_JPRB, 0.40589E+00_JPRB, 0.41631E+00_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.46208E+00_JPRB, 0.46802E+00_JPRB, 0.46315E+00_JPRB, 0.45207E+00_JPRB, 0.44123E+00_JPRB, 0.43083E+00_JPRB, 0.42236E+00_JPRB ,&
&  0.41144E+00_JPRB, 0.40252E+00_JPRB, 0.39926E+00_JPRB, 0.39866E+00_JPRB, 0.39775E+00_JPRB, 0.39396E+00_JPRB, 0.38876E+00_JPRB ,&
&  0.38350E+00_JPRB, 0.37759E+00_JPRB, 0.37062E+00_JPRB, 0.36370E+00_JPRB, 0.35791E+00_JPRB, 0.35150E+00_JPRB, 0.34710E+00_JPRB ,&
&  0.34327E+00_JPRB, 0.34032E+00_JPRB, 0.33498E+00_JPRB, 0.32784E+00_JPRB, 0.32526E+00_JPRB, 0.32089E+00_JPRB, 0.32950E+00_JPRB ,&
&  0.36013E+00_JPRB, 0.31261E+00_JPRB, 0.32945E+00_JPRB, 0.35980E+00_JPRB, 0.38453E+00_JPRB, 0.41561E+00_JPRB, 0.43382E+00_JPRB ,&
&  0.44891E+00_JPRB, 0.45828E+00_JPRB, 0.46315E+00_JPRB, 0.47064E+00_JPRB, 0.46800E+00_JPRB, 0.45210E+00_JPRB, 0.43132E+00_JPRB ,&
&  0.41733E+00_JPRB, 0.40216E+00_JPRB, 0.39272E+00_JPRB, 0.38404E+00_JPRB, 0.38142E+00_JPRB, 0.38179E+00_JPRB, 0.37801E+00_JPRB ,&
&  0.37140E+00_JPRB, 0.36912E+00_JPRB, 0.36584E+00_JPRB, 0.36589E+00_JPRB, 0.36660E+00_JPRB, 0.36860E+00_JPRB, 0.36648E+00_JPRB ,&
&  0.36363E+00_JPRB, 0.36478E+00_JPRB, 0.36676E+00_JPRB, 0.36569E+00_JPRB, 0.36356E+00_JPRB, 0.36041E+00_JPRB, 0.35601E+00_JPRB ,&
&  0.35175E+00_JPRB, 0.34871E+00_JPRB, 0.34799E+00_JPRB, 0.34709E+00_JPRB, 0.34658E+00_JPRB, 0.34688E+00_JPRB, 0.34767E+00_JPRB ,&
&  0.34913E+00_JPRB, 0.35053E+00_JPRB, 0.35120E+00_JPRB, 0.35089E+00_JPRB, 0.34976E+00_JPRB, 0.34716E+00_JPRB, 0.34647E+00_JPRB ,&
&  0.34708E+00_JPRB, 0.34952E+00_JPRB, 0.35396E+00_JPRB, 0.36036E+00_JPRB, 0.36900E+00_JPRB, 0.37703E+00_JPRB, 0.38658E+00_JPRB ,&
&  0.39706E+00_JPRB, 0.40948E+00_JPRB, 0.42696E+00_JPRB, 0.44298E+00_JPRB, 0.46451E+00_JPRB, 0.47128E+00_JPRB, 0.47843E+00_JPRB ,&
&  0.48199E+00_JPRB, 0.48342E+00_JPRB, 0.48563E+00_JPRB, 0.43214E+00_JPRB, 0.45324E+00_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.50348E+00_JPRB, 0.50892E+00_JPRB, 0.49648E+00_JPRB, 0.47952E+00_JPRB, 0.46268E+00_JPRB, 0.44898E+00_JPRB, 0.43713E+00_JPRB ,&
&  0.42912E+00_JPRB, 0.42381E+00_JPRB, 0.42030E+00_JPRB, 0.41822E+00_JPRB, 0.41523E+00_JPRB, 0.41011E+00_JPRB, 0.40485E+00_JPRB ,&
&  0.40082E+00_JPRB, 0.39611E+00_JPRB, 0.38859E+00_JPRB, 0.38363E+00_JPRB, 0.38096E+00_JPRB, 0.37762E+00_JPRB, 0.37455E+00_JPRB ,&
&  0.37069E+00_JPRB, 0.36188E+00_JPRB, 0.35294E+00_JPRB, 0.34776E+00_JPRB, 0.34398E+00_JPRB, 0.34220E+00_JPRB, 0.35049E+00_JPRB ,&
&  0.38212E+00_JPRB, 0.35060E+00_JPRB, 0.37484E+00_JPRB, 0.39989E+00_JPRB, 0.42522E+00_JPRB, 0.44361E+00_JPRB, 0.46286E+00_JPRB ,&
&  0.47707E+00_JPRB, 0.49021E+00_JPRB, 0.50041E+00_JPRB, 0.51236E+00_JPRB, 0.49686E+00_JPRB, 0.47432E+00_JPRB, 0.45573E+00_JPRB ,&
&  0.43855E+00_JPRB, 0.42129E+00_JPRB, 0.40792E+00_JPRB, 0.39952E+00_JPRB, 0.39860E+00_JPRB, 0.39895E+00_JPRB, 0.39748E+00_JPRB ,&
&  0.39699E+00_JPRB, 0.39363E+00_JPRB, 0.38896E+00_JPRB, 0.39011E+00_JPRB, 0.38891E+00_JPRB, 0.39706E+00_JPRB, 0.40512E+00_JPRB ,&
&  0.40911E+00_JPRB, 0.41395E+00_JPRB, 0.41480E+00_JPRB, 0.41359E+00_JPRB, 0.40883E+00_JPRB, 0.40299E+00_JPRB, 0.39655E+00_JPRB ,&
&  0.39120E+00_JPRB, 0.38836E+00_JPRB, 0.38867E+00_JPRB, 0.38944E+00_JPRB, 0.38978E+00_JPRB, 0.39203E+00_JPRB, 0.39416E+00_JPRB ,&
&  0.39631E+00_JPRB, 0.39696E+00_JPRB, 0.39701E+00_JPRB, 0.39339E+00_JPRB, 0.38818E+00_JPRB, 0.38446E+00_JPRB, 0.38151E+00_JPRB ,&
&  0.38022E+00_JPRB, 0.38214E+00_JPRB, 0.38567E+00_JPRB, 0.39314E+00_JPRB, 0.40105E+00_JPRB, 0.41104E+00_JPRB, 0.41900E+00_JPRB ,&
&  0.42673E+00_JPRB, 0.44014E+00_JPRB, 0.45728E+00_JPRB, 0.47851E+00_JPRB, 0.48771E+00_JPRB, 0.49269E+00_JPRB, 0.49889E+00_JPRB ,&
&  0.50368E+00_JPRB, 0.50697E+00_JPRB, 0.50113E+00_JPRB, 0.49900E+00_JPRB, 0.48925E+00_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.58295E+00_JPRB, 0.56282E+00_JPRB, 0.53392E+00_JPRB, 0.50508E+00_JPRB, 0.47803E+00_JPRB, 0.45690E+00_JPRB, 0.44403E+00_JPRB ,&
&  0.43618E+00_JPRB, 0.43119E+00_JPRB, 0.42804E+00_JPRB, 0.42557E+00_JPRB, 0.42197E+00_JPRB, 0.41898E+00_JPRB, 0.41451E+00_JPRB ,&
&  0.40931E+00_JPRB, 0.40308E+00_JPRB, 0.39742E+00_JPRB, 0.39393E+00_JPRB, 0.39057E+00_JPRB, 0.38917E+00_JPRB, 0.38790E+00_JPRB ,&
&  0.38356E+00_JPRB, 0.37747E+00_JPRB, 0.36914E+00_JPRB, 0.36058E+00_JPRB, 0.35608E+00_JPRB, 0.35598E+00_JPRB, 0.37087E+00_JPRB ,&
&  0.41329E+00_JPRB, 0.38589E+00_JPRB, 0.41988E+00_JPRB, 0.44556E+00_JPRB, 0.46536E+00_JPRB, 0.48797E+00_JPRB, 0.51505E+00_JPRB ,&
&  0.53264E+00_JPRB, 0.54816E+00_JPRB, 0.56728E+00_JPRB, 0.56480E+00_JPRB, 0.53731E+00_JPRB, 0.51358E+00_JPRB, 0.48877E+00_JPRB ,&
&  0.46319E+00_JPRB, 0.43933E+00_JPRB, 0.41908E+00_JPRB, 0.41195E+00_JPRB, 0.41247E+00_JPRB, 0.41625E+00_JPRB, 0.41773E+00_JPRB ,&
&  0.41918E+00_JPRB, 0.41782E+00_JPRB, 0.42021E+00_JPRB, 0.43065E+00_JPRB, 0.44267E+00_JPRB, 0.45777E+00_JPRB, 0.48017E+00_JPRB ,&
&  0.48946E+00_JPRB, 0.48522E+00_JPRB, 0.48461E+00_JPRB, 0.47809E+00_JPRB, 0.46336E+00_JPRB, 0.44984E+00_JPRB, 0.43989E+00_JPRB ,&
&  0.43331E+00_JPRB, 0.43060E+00_JPRB, 0.43114E+00_JPRB, 0.43093E+00_JPRB, 0.43224E+00_JPRB, 0.43451E+00_JPRB, 0.43768E+00_JPRB ,&
&  0.43959E+00_JPRB, 0.44121E+00_JPRB, 0.44093E+00_JPRB, 0.43683E+00_JPRB, 0.43472E+00_JPRB, 0.43311E+00_JPRB, 0.42790E+00_JPRB ,&
&  0.42174E+00_JPRB, 0.41993E+00_JPRB, 0.42390E+00_JPRB, 0.43043E+00_JPRB, 0.43986E+00_JPRB, 0.44955E+00_JPRB, 0.45737E+00_JPRB ,&
&  0.46644E+00_JPRB, 0.48189E+00_JPRB, 0.47945E+00_JPRB, 0.48102E+00_JPRB, 0.50729E+00_JPRB, 0.51057E+00_JPRB, 0.51851E+00_JPRB ,&
&  0.52707E+00_JPRB, 0.53099E+00_JPRB, 0.52645E+00_JPRB, 0.51995E+00_JPRB, 0.57930E+00_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.55478E+00_JPRB, 0.53443E+00_JPRB, 0.50997E+00_JPRB, 0.48719E+00_JPRB, 0.46697E+00_JPRB, 0.45175E+00_JPRB, 0.44272E+00_JPRB ,&
&  0.43585E+00_JPRB, 0.43103E+00_JPRB, 0.43132E+00_JPRB, 0.43425E+00_JPRB, 0.43358E+00_JPRB, 0.42695E+00_JPRB, 0.41788E+00_JPRB ,&
&  0.40897E+00_JPRB, 0.40323E+00_JPRB, 0.39929E+00_JPRB, 0.39873E+00_JPRB, 0.39600E+00_JPRB, 0.39335E+00_JPRB, 0.38907E+00_JPRB ,&
&  0.38654E+00_JPRB, 0.38363E+00_JPRB, 0.37838E+00_JPRB, 0.37312E+00_JPRB, 0.37004E+00_JPRB, 0.37087E+00_JPRB, 0.39259E+00_JPRB ,&
&  0.43589E+00_JPRB, 0.42737E+00_JPRB, 0.46716E+00_JPRB, 0.50364E+00_JPRB, 0.52572E+00_JPRB, 0.55248E+00_JPRB, 0.60374E+00_JPRB ,&
&  0.63696E+00_JPRB, 0.65167E+00_JPRB, 0.66011E+00_JPRB, 0.63980E+00_JPRB, 0.59682E+00_JPRB, 0.56215E+00_JPRB, 0.52110E+00_JPRB ,&
&  0.48083E+00_JPRB, 0.44860E+00_JPRB, 0.42853E+00_JPRB, 0.42113E+00_JPRB, 0.42512E+00_JPRB, 0.43266E+00_JPRB, 0.43681E+00_JPRB ,&
&  0.44086E+00_JPRB, 0.44419E+00_JPRB, 0.45926E+00_JPRB, 0.48482E+00_JPRB, 0.52365E+00_JPRB, 0.55588E+00_JPRB, 0.59617E+00_JPRB ,&
&  0.61156E+00_JPRB, 0.60337E+00_JPRB, 0.58880E+00_JPRB, 0.56596E+00_JPRB, 0.54981E+00_JPRB, 0.53181E+00_JPRB, 0.51759E+00_JPRB ,&
&  0.50135E+00_JPRB, 0.48837E+00_JPRB, 0.47906E+00_JPRB, 0.47404E+00_JPRB, 0.47296E+00_JPRB, 0.47385E+00_JPRB, 0.47642E+00_JPRB ,&
&  0.47619E+00_JPRB, 0.47713E+00_JPRB, 0.47798E+00_JPRB, 0.48273E+00_JPRB, 0.48866E+00_JPRB, 0.48487E+00_JPRB, 0.47566E+00_JPRB ,&
&  0.46524E+00_JPRB, 0.45741E+00_JPRB, 0.45915E+00_JPRB, 0.46688E+00_JPRB, 0.47466E+00_JPRB, 0.47969E+00_JPRB, 0.48348E+00_JPRB ,&
&  0.49560E+00_JPRB, 0.50553E+00_JPRB, 0.51131E+00_JPRB, 0.51150E+00_JPRB, 0.51488E+00_JPRB, 0.54371E+00_JPRB, 0.54908E+00_JPRB ,&
&  0.55354E+00_JPRB, 0.55392E+00_JPRB, 0.54495E+00_JPRB, 0.53530E+00_JPRB, 0.55908E+00_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.54749E+00_JPRB, 0.53278E+00_JPRB, 0.50967E+00_JPRB, 0.48955E+00_JPRB, 0.47435E+00_JPRB, 0.45916E+00_JPRB, 0.44894E+00_JPRB ,&
&  0.44153E+00_JPRB, 0.44222E+00_JPRB, 0.44504E+00_JPRB, 0.44657E+00_JPRB, 0.44322E+00_JPRB, 0.43371E+00_JPRB, 0.42411E+00_JPRB ,&
&  0.41854E+00_JPRB, 0.41908E+00_JPRB, 0.42023E+00_JPRB, 0.41836E+00_JPRB, 0.41436E+00_JPRB, 0.40898E+00_JPRB, 0.40411E+00_JPRB ,&
&  0.39993E+00_JPRB, 0.39591E+00_JPRB, 0.39188E+00_JPRB, 0.38993E+00_JPRB, 0.38758E+00_JPRB, 0.39088E+00_JPRB, 0.41277E+00_JPRB ,&
&  0.45295E+00_JPRB, 0.45457E+00_JPRB, 0.56973E+00_JPRB, 0.59289E+00_JPRB, 0.61544E+00_JPRB, 0.66092E+00_JPRB, 0.76011E+00_JPRB ,&
&  0.82569E+00_JPRB, 0.84159E+00_JPRB, 0.81376E+00_JPRB, 0.75006E+00_JPRB, 0.66330E+00_JPRB, 0.59308E+00_JPRB, 0.53898E+00_JPRB ,&
&  0.49395E+00_JPRB, 0.46459E+00_JPRB, 0.44805E+00_JPRB, 0.44402E+00_JPRB, 0.44851E+00_JPRB, 0.45816E+00_JPRB, 0.46308E+00_JPRB ,&
&  0.47254E+00_JPRB, 0.48247E+00_JPRB, 0.50910E+00_JPRB, 0.55433E+00_JPRB, 0.58077E+00_JPRB, 0.57384E+00_JPRB, 0.64274E+00_JPRB ,&
&  0.76884E+00_JPRB, 0.73658E+00_JPRB, 0.70618E+00_JPRB, 0.67483E+00_JPRB, 0.65167E+00_JPRB, 0.62793E+00_JPRB, 0.59427E+00_JPRB ,&
&  0.56828E+00_JPRB, 0.54552E+00_JPRB, 0.52950E+00_JPRB, 0.52223E+00_JPRB, 0.51811E+00_JPRB, 0.51737E+00_JPRB, 0.51655E+00_JPRB ,&
&  0.51678E+00_JPRB, 0.51837E+00_JPRB, 0.52432E+00_JPRB, 0.52741E+00_JPRB, 0.52535E+00_JPRB, 0.51578E+00_JPRB, 0.50278E+00_JPRB ,&
&  0.49395E+00_JPRB, 0.48980E+00_JPRB, 0.48833E+00_JPRB, 0.48597E+00_JPRB, 0.49657E+00_JPRB, 0.51199E+00_JPRB, 0.52105E+00_JPRB ,&
&  0.52071E+00_JPRB, 0.52386E+00_JPRB, 0.53659E+00_JPRB, 0.53793E+00_JPRB, 0.55254E+00_JPRB, 0.58856E+00_JPRB, 0.59156E+00_JPRB ,&
&  0.59593E+00_JPRB, 0.58135E+00_JPRB, 0.56425E+00_JPRB, 0.55195E+00_JPRB, 0.55260E+00_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.54651E+00_JPRB, 0.52566E+00_JPRB, 0.50916E+00_JPRB, 0.49320E+00_JPRB, 0.48208E+00_JPRB, 0.47140E+00_JPRB, 0.46453E+00_JPRB ,&
&  0.46292E+00_JPRB, 0.46454E+00_JPRB, 0.46431E+00_JPRB, 0.45961E+00_JPRB, 0.45079E+00_JPRB, 0.44423E+00_JPRB, 0.44045E+00_JPRB ,&
&  0.43664E+00_JPRB, 0.43707E+00_JPRB, 0.43649E+00_JPRB, 0.42965E+00_JPRB, 0.42211E+00_JPRB, 0.41726E+00_JPRB, 0.41237E+00_JPRB ,&
&  0.40882E+00_JPRB, 0.40696E+00_JPRB, 0.40558E+00_JPRB, 0.40480E+00_JPRB, 0.40503E+00_JPRB, 0.41170E+00_JPRB, 0.42888E+00_JPRB ,&
&  0.45833E+00_JPRB, 0.40282E+00_JPRB, 0.64104E+00_JPRB, 0.72759E+00_JPRB, 0.77898E+00_JPRB, 0.84716E+00_JPRB, 0.97996E+00_JPRB ,&
&  0.11211E+01_JPRB, 0.11017E+01_JPRB, 0.98949E+00_JPRB, 0.84657E+00_JPRB, 0.71033E+00_JPRB, 0.61230E+00_JPRB, 0.55943E+00_JPRB ,&
&  0.52229E+00_JPRB, 0.49598E+00_JPRB, 0.48481E+00_JPRB, 0.47909E+00_JPRB, 0.48307E+00_JPRB, 0.49085E+00_JPRB, 0.49622E+00_JPRB ,&
&  0.50717E+00_JPRB, 0.51804E+00_JPRB, 0.54805E+00_JPRB, 0.59458E+00_JPRB, 0.59988E+00_JPRB, 0.62272E+00_JPRB, 0.67179E+00_JPRB ,&
&  0.77324E+00_JPRB, 0.85469E+00_JPRB, 0.79875E+00_JPRB, 0.75280E+00_JPRB, 0.72077E+00_JPRB, 0.68884E+00_JPRB, 0.65407E+00_JPRB ,&
&  0.62367E+00_JPRB, 0.59376E+00_JPRB, 0.57469E+00_JPRB, 0.56087E+00_JPRB, 0.55191E+00_JPRB, 0.54725E+00_JPRB, 0.54499E+00_JPRB ,&
&  0.54388E+00_JPRB, 0.54408E+00_JPRB, 0.54674E+00_JPRB, 0.54695E+00_JPRB, 0.54440E+00_JPRB, 0.53327E+00_JPRB, 0.51808E+00_JPRB ,&
&  0.51202E+00_JPRB, 0.51340E+00_JPRB, 0.51174E+00_JPRB, 0.51941E+00_JPRB, 0.53219E+00_JPRB, 0.53904E+00_JPRB, 0.54733E+00_JPRB ,&
&  0.56047E+00_JPRB, 0.56652E+00_JPRB, 0.56963E+00_JPRB, 0.57786E+00_JPRB, 0.57954E+00_JPRB, 0.61676E+00_JPRB, 0.60557E+00_JPRB ,&
&  0.60357E+00_JPRB, 0.59142E+00_JPRB, 0.57245E+00_JPRB, 0.56289E+00_JPRB, 0.55846E+00_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.54709E+00_JPRB, 0.52799E+00_JPRB, 0.51459E+00_JPRB, 0.50415E+00_JPRB, 0.49600E+00_JPRB, 0.48990E+00_JPRB, 0.48983E+00_JPRB ,&
&  0.49025E+00_JPRB, 0.48802E+00_JPRB, 0.47791E+00_JPRB, 0.46527E+00_JPRB, 0.46177E+00_JPRB, 0.46304E+00_JPRB, 0.45644E+00_JPRB ,&
&  0.45567E+00_JPRB, 0.45642E+00_JPRB, 0.45140E+00_JPRB, 0.44273E+00_JPRB, 0.43586E+00_JPRB, 0.43243E+00_JPRB, 0.43230E+00_JPRB ,&
&  0.42675E+00_JPRB, 0.42524E+00_JPRB, 0.42825E+00_JPRB, 0.43048E+00_JPRB, 0.43326E+00_JPRB, 0.44733E+00_JPRB, 0.45248E+00_JPRB ,&
&  0.47350E+00_JPRB, 0.52720E+00_JPRB, 0.58689E+00_JPRB, 0.80285E+00_JPRB, 0.94473E+00_JPRB, 0.10840E+01_JPRB, 0.11648E+01_JPRB ,&
&  0.12315E+01_JPRB, 0.13102E+01_JPRB, 0.11208E+01_JPRB, 0.92503E+00_JPRB, 0.74535E+00_JPRB, 0.64544E+00_JPRB, 0.59580E+00_JPRB ,&
&  0.57046E+00_JPRB, 0.54580E+00_JPRB, 0.52908E+00_JPRB, 0.52521E+00_JPRB, 0.52822E+00_JPRB, 0.52887E+00_JPRB, 0.52789E+00_JPRB ,&
&  0.53598E+00_JPRB, 0.55116E+00_JPRB, 0.59497E+00_JPRB, 0.64392E+00_JPRB, 0.65040E+00_JPRB, 0.71503E+00_JPRB, 0.77500E+00_JPRB ,&
&  0.84030E+00_JPRB, 0.95469E+00_JPRB, 0.90234E+00_JPRB, 0.83664E+00_JPRB, 0.77882E+00_JPRB, 0.72329E+00_JPRB, 0.67769E+00_JPRB ,&
&  0.64297E+00_JPRB, 0.61372E+00_JPRB, 0.59310E+00_JPRB, 0.57783E+00_JPRB, 0.56786E+00_JPRB, 0.56274E+00_JPRB, 0.55689E+00_JPRB ,&
&  0.54901E+00_JPRB, 0.54800E+00_JPRB, 0.55234E+00_JPRB, 0.55580E+00_JPRB, 0.55315E+00_JPRB, 0.54646E+00_JPRB, 0.54015E+00_JPRB ,&
&  0.53993E+00_JPRB, 0.54675E+00_JPRB, 0.54728E+00_JPRB, 0.56799E+00_JPRB, 0.57882E+00_JPRB, 0.56628E+00_JPRB, 0.56412E+00_JPRB ,&
&  0.59041E+00_JPRB, 0.60203E+00_JPRB, 0.60255E+00_JPRB, 0.60253E+00_JPRB, 0.62107E+00_JPRB, 0.66109E+00_JPRB, 0.61576E+00_JPRB ,&
&  0.59277E+00_JPRB, 0.58502E+00_JPRB, 0.57251E+00_JPRB, 0.56492E+00_JPRB, 0.55821E+00_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.55371E+00_JPRB, 0.54122E+00_JPRB, 0.53150E+00_JPRB, 0.52085E+00_JPRB, 0.51276E+00_JPRB, 0.51147E+00_JPRB, 0.51028E+00_JPRB ,&
&  0.50587E+00_JPRB, 0.48972E+00_JPRB, 0.47228E+00_JPRB, 0.46687E+00_JPRB, 0.46709E+00_JPRB, 0.46152E+00_JPRB, 0.46555E+00_JPRB ,&
&  0.46876E+00_JPRB, 0.47178E+00_JPRB, 0.46323E+00_JPRB, 0.45760E+00_JPRB, 0.45421E+00_JPRB, 0.45239E+00_JPRB, 0.45398E+00_JPRB ,&
&  0.45611E+00_JPRB, 0.45776E+00_JPRB, 0.46287E+00_JPRB, 0.46784E+00_JPRB, 0.47438E+00_JPRB, 0.48879E+00_JPRB, 0.49827E+00_JPRB ,&
&  0.55954E+00_JPRB, 0.58094E+00_JPRB, 0.63264E+00_JPRB, 0.10033E+01_JPRB, 0.12227E+01_JPRB, 0.12980E+01_JPRB, 0.12223E+01_JPRB ,&
&  0.12155E+01_JPRB, 0.13204E+01_JPRB, 0.12013E+01_JPRB, 0.98944E+00_JPRB, 0.81723E+00_JPRB, 0.73783E+00_JPRB, 0.68550E+00_JPRB ,&
&  0.64248E+00_JPRB, 0.60182E+00_JPRB, 0.57594E+00_JPRB, 0.56005E+00_JPRB, 0.55967E+00_JPRB, 0.55685E+00_JPRB, 0.56008E+00_JPRB ,&
&  0.57185E+00_JPRB, 0.60221E+00_JPRB, 0.65547E+00_JPRB, 0.68840E+00_JPRB, 0.77707E+00_JPRB, 0.87756E+00_JPRB, 0.91748E+00_JPRB ,&
&  0.92162E+00_JPRB, 0.99461E+00_JPRB, 0.92356E+00_JPRB, 0.85695E+00_JPRB, 0.77830E+00_JPRB, 0.73871E+00_JPRB, 0.68728E+00_JPRB ,&
&  0.64803E+00_JPRB, 0.61524E+00_JPRB, 0.59147E+00_JPRB, 0.58000E+00_JPRB, 0.56998E+00_JPRB, 0.56305E+00_JPRB, 0.55719E+00_JPRB ,&
&  0.54949E+00_JPRB, 0.55578E+00_JPRB, 0.56313E+00_JPRB, 0.56084E+00_JPRB, 0.56028E+00_JPRB, 0.56280E+00_JPRB, 0.56824E+00_JPRB ,&
&  0.57425E+00_JPRB, 0.57905E+00_JPRB, 0.58148E+00_JPRB, 0.60901E+00_JPRB, 0.62104E+00_JPRB, 0.60779E+00_JPRB, 0.60648E+00_JPRB ,&
&  0.62253E+00_JPRB, 0.63387E+00_JPRB, 0.62914E+00_JPRB, 0.61005E+00_JPRB, 0.62278E+00_JPRB, 0.63680E+00_JPRB, 0.61138E+00_JPRB ,&
&  0.60696E+00_JPRB, 0.59863E+00_JPRB, 0.57846E+00_JPRB, 0.56545E+00_JPRB, 0.56053E+00_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.56928E+00_JPRB, 0.55564E+00_JPRB, 0.54030E+00_JPRB, 0.52931E+00_JPRB, 0.52329E+00_JPRB, 0.51769E+00_JPRB, 0.50660E+00_JPRB ,&
&  0.49083E+00_JPRB, 0.48251E+00_JPRB, 0.46554E+00_JPRB, 0.46390E+00_JPRB, 0.47166E+00_JPRB, 0.47713E+00_JPRB, 0.48605E+00_JPRB ,&
&  0.49116E+00_JPRB, 0.49132E+00_JPRB, 0.48617E+00_JPRB, 0.48109E+00_JPRB, 0.48018E+00_JPRB, 0.48163E+00_JPRB, 0.49093E+00_JPRB ,&
&  0.49187E+00_JPRB, 0.49365E+00_JPRB, 0.49941E+00_JPRB, 0.50683E+00_JPRB, 0.52749E+00_JPRB, 0.56238E+00_JPRB, 0.60659E+00_JPRB ,&
&  0.71161E+00_JPRB, 0.71663E+00_JPRB, 0.73767E+00_JPRB, 0.10686E+01_JPRB, 0.12857E+01_JPRB, 0.12418E+01_JPRB, 0.12173E+01_JPRB ,&
&  0.11920E+01_JPRB, 0.11332E+01_JPRB, 0.11564E+01_JPRB, 0.10161E+01_JPRB, 0.87865E+00_JPRB, 0.80281E+00_JPRB, 0.74439E+00_JPRB ,&
&  0.68738E+00_JPRB, 0.64407E+00_JPRB, 0.60947E+00_JPRB, 0.59145E+00_JPRB, 0.59144E+00_JPRB, 0.60130E+00_JPRB, 0.60814E+00_JPRB ,&
&  0.63519E+00_JPRB, 0.68418E+00_JPRB, 0.75865E+00_JPRB, 0.82237E+00_JPRB, 0.91075E+00_JPRB, 0.99992E+00_JPRB, 0.97639E+00_JPRB ,&
&  0.95673E+00_JPRB, 0.95973E+00_JPRB, 0.91970E+00_JPRB, 0.85863E+00_JPRB, 0.74265E+00_JPRB, 0.74283E+00_JPRB, 0.68519E+00_JPRB ,&
&  0.64152E+00_JPRB, 0.60908E+00_JPRB, 0.59193E+00_JPRB, 0.58455E+00_JPRB, 0.57953E+00_JPRB, 0.57156E+00_JPRB, 0.56611E+00_JPRB ,&
&  0.56242E+00_JPRB, 0.56878E+00_JPRB, 0.57327E+00_JPRB, 0.57141E+00_JPRB, 0.57508E+00_JPRB, 0.58290E+00_JPRB, 0.59083E+00_JPRB ,&
&  0.59808E+00_JPRB, 0.61160E+00_JPRB, 0.62345E+00_JPRB, 0.65880E+00_JPRB, 0.67487E+00_JPRB, 0.67458E+00_JPRB, 0.65605E+00_JPRB ,&
&  0.64363E+00_JPRB, 0.64990E+00_JPRB, 0.64962E+00_JPRB, 0.64580E+00_JPRB, 0.64157E+00_JPRB, 0.64260E+00_JPRB, 0.64243E+00_JPRB ,&
&  0.64300E+00_JPRB, 0.62309E+00_JPRB, 0.59802E+00_JPRB, 0.58565E+00_JPRB, 0.57755E+00_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.58070E+00_JPRB, 0.56514E+00_JPRB, 0.54331E+00_JPRB, 0.52091E+00_JPRB, 0.50975E+00_JPRB, 0.49912E+00_JPRB, 0.48581E+00_JPRB ,&
&  0.47949E+00_JPRB, 0.47659E+00_JPRB, 0.47342E+00_JPRB, 0.48567E+00_JPRB, 0.50772E+00_JPRB, 0.51863E+00_JPRB, 0.52385E+00_JPRB ,&
&  0.53000E+00_JPRB, 0.53171E+00_JPRB, 0.51759E+00_JPRB, 0.51413E+00_JPRB, 0.52218E+00_JPRB, 0.53205E+00_JPRB, 0.53721E+00_JPRB ,&
&  0.54110E+00_JPRB, 0.54846E+00_JPRB, 0.55934E+00_JPRB, 0.58021E+00_JPRB, 0.62820E+00_JPRB, 0.69878E+00_JPRB, 0.79131E+00_JPRB ,&
&  0.89497E+00_JPRB, 0.85529E+00_JPRB, 0.54139E+00_JPRB, 0.86187E+00_JPRB, 0.11376E+01_JPRB, 0.11500E+01_JPRB, 0.11129E+01_JPRB ,&
&  0.10890E+01_JPRB, 0.10356E+01_JPRB, 0.10434E+01_JPRB, 0.99158E+00_JPRB, 0.89917E+00_JPRB, 0.81979E+00_JPRB, 0.75170E+00_JPRB ,&
&  0.69322E+00_JPRB, 0.65346E+00_JPRB, 0.63254E+00_JPRB, 0.63553E+00_JPRB, 0.64539E+00_JPRB, 0.64754E+00_JPRB, 0.66692E+00_JPRB ,&
&  0.71351E+00_JPRB, 0.77297E+00_JPRB, 0.87587E+00_JPRB, 0.91252E+00_JPRB, 0.99736E+00_JPRB, 0.99814E+00_JPRB, 0.90700E+00_JPRB ,&
&  0.82066E+00_JPRB, 0.87082E+00_JPRB, 0.89104E+00_JPRB, 0.82556E+00_JPRB, 0.72065E+00_JPRB, 0.67302E+00_JPRB, 0.67040E+00_JPRB ,&
&  0.63066E+00_JPRB, 0.60428E+00_JPRB, 0.59366E+00_JPRB, 0.58742E+00_JPRB, 0.57977E+00_JPRB, 0.57252E+00_JPRB, 0.56694E+00_JPRB ,&
&  0.57167E+00_JPRB, 0.57595E+00_JPRB, 0.57775E+00_JPRB, 0.57903E+00_JPRB, 0.58473E+00_JPRB, 0.59225E+00_JPRB, 0.60400E+00_JPRB ,&
&  0.62425E+00_JPRB, 0.65023E+00_JPRB, 0.69047E+00_JPRB, 0.70874E+00_JPRB, 0.71263E+00_JPRB, 0.71036E+00_JPRB, 0.69694E+00_JPRB ,&
&  0.68504E+00_JPRB, 0.68209E+00_JPRB, 0.67487E+00_JPRB, 0.65668E+00_JPRB, 0.69927E+00_JPRB, 0.71585E+00_JPRB, 0.75371E+00_JPRB ,&
&  0.78840E+00_JPRB, 0.79799E+00_JPRB, 0.74671E+00_JPRB, 0.61914E+00_JPRB, 0.60177E+00_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.54597E+00_JPRB, 0.52810E+00_JPRB, 0.49754E+00_JPRB, 0.49087E+00_JPRB, 0.49027E+00_JPRB, 0.48996E+00_JPRB, 0.50338E+00_JPRB ,&
&  0.51471E+00_JPRB, 0.53157E+00_JPRB, 0.54533E+00_JPRB, 0.56550E+00_JPRB, 0.57689E+00_JPRB, 0.57666E+00_JPRB, 0.58245E+00_JPRB ,&
&  0.58272E+00_JPRB, 0.58176E+00_JPRB, 0.58221E+00_JPRB, 0.58919E+00_JPRB, 0.60010E+00_JPRB, 0.60493E+00_JPRB, 0.61144E+00_JPRB ,&
&  0.62592E+00_JPRB, 0.64198E+00_JPRB, 0.66419E+00_JPRB, 0.71728E+00_JPRB, 0.80700E+00_JPRB, 0.91234E+00_JPRB, 0.10116E+01_JPRB ,&
&  0.10335E+01_JPRB, 0.79421E+00_JPRB, 0.63246E+00_JPRB, 0.10332E+01_JPRB, 0.11462E+01_JPRB, 0.11318E+01_JPRB, 0.10225E+01_JPRB ,&
&  0.10118E+01_JPRB, 0.98988E+00_JPRB, 0.97791E+00_JPRB, 0.95747E+00_JPRB, 0.84976E+00_JPRB, 0.78535E+00_JPRB, 0.73394E+00_JPRB ,&
&  0.70181E+00_JPRB, 0.67937E+00_JPRB, 0.67422E+00_JPRB, 0.68451E+00_JPRB, 0.69712E+00_JPRB, 0.71232E+00_JPRB, 0.75487E+00_JPRB ,&
&  0.80808E+00_JPRB, 0.87200E+00_JPRB, 0.96160E+00_JPRB, 0.92147E+00_JPRB, 0.96423E+00_JPRB, 0.89322E+00_JPRB, 0.83158E+00_JPRB ,&
&  0.76960E+00_JPRB, 0.80662E+00_JPRB, 0.84505E+00_JPRB, 0.79611E+00_JPRB, 0.70889E+00_JPRB, 0.64837E+00_JPRB, 0.65078E+00_JPRB ,&
&  0.61228E+00_JPRB, 0.59535E+00_JPRB, 0.58361E+00_JPRB, 0.57284E+00_JPRB, 0.57235E+00_JPRB, 0.57565E+00_JPRB, 0.57938E+00_JPRB ,&
&  0.58364E+00_JPRB, 0.58221E+00_JPRB, 0.58181E+00_JPRB, 0.58914E+00_JPRB, 0.60098E+00_JPRB, 0.60961E+00_JPRB, 0.62710E+00_JPRB ,&
&  0.65106E+00_JPRB, 0.68188E+00_JPRB, 0.71767E+00_JPRB, 0.74268E+00_JPRB, 0.74032E+00_JPRB, 0.71715E+00_JPRB, 0.72120E+00_JPRB ,&
&  0.72163E+00_JPRB, 0.74190E+00_JPRB, 0.74770E+00_JPRB, 0.76175E+00_JPRB, 0.77157E+00_JPRB, 0.79530E+00_JPRB, 0.80481E+00_JPRB ,&
&  0.80539E+00_JPRB, 0.78166E+00_JPRB, 0.69974E+00_JPRB, 0.59390E+00_JPRB, 0.57080E+00_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.49373E+00_JPRB, 0.48821E+00_JPRB, 0.49095E+00_JPRB, 0.49858E+00_JPRB, 0.51042E+00_JPRB, 0.53435E+00_JPRB, 0.56095E+00_JPRB ,&
&  0.58287E+00_JPRB, 0.61948E+00_JPRB, 0.64430E+00_JPRB, 0.64593E+00_JPRB, 0.64028E+00_JPRB, 0.64118E+00_JPRB, 0.64558E+00_JPRB ,&
&  0.65515E+00_JPRB, 0.66224E+00_JPRB, 0.66705E+00_JPRB, 0.67377E+00_JPRB, 0.68372E+00_JPRB, 0.69474E+00_JPRB, 0.70928E+00_JPRB ,&
&  0.73273E+00_JPRB, 0.77192E+00_JPRB, 0.82653E+00_JPRB, 0.90513E+00_JPRB, 0.10075E+01_JPRB, 0.11116E+01_JPRB, 0.11600E+01_JPRB ,&
&  0.89033E+00_JPRB, 0.53696E+00_JPRB, 0.10119E+01_JPRB, 0.11725E+01_JPRB, 0.10807E+01_JPRB, 0.10414E+01_JPRB, 0.10051E+01_JPRB ,&
&  0.94721E+00_JPRB, 0.94401E+00_JPRB, 0.91521E+00_JPRB, 0.91905E+00_JPRB, 0.82022E+00_JPRB, 0.77103E+00_JPRB, 0.74321E+00_JPRB ,&
&  0.72863E+00_JPRB, 0.71983E+00_JPRB, 0.72177E+00_JPRB, 0.72942E+00_JPRB, 0.75050E+00_JPRB, 0.78208E+00_JPRB, 0.82950E+00_JPRB ,&
&  0.87682E+00_JPRB, 0.92111E+00_JPRB, 0.99354E+00_JPRB, 0.87117E+00_JPRB, 0.87558E+00_JPRB, 0.82374E+00_JPRB, 0.77328E+00_JPRB ,&
&  0.75651E+00_JPRB, 0.74322E+00_JPRB, 0.75107E+00_JPRB, 0.77091E+00_JPRB, 0.70949E+00_JPRB, 0.62762E+00_JPRB, 0.63561E+00_JPRB ,&
&  0.60252E+00_JPRB, 0.59185E+00_JPRB, 0.57988E+00_JPRB, 0.56583E+00_JPRB, 0.56015E+00_JPRB, 0.56489E+00_JPRB, 0.57516E+00_JPRB ,&
&  0.58075E+00_JPRB, 0.58406E+00_JPRB, 0.59141E+00_JPRB, 0.59936E+00_JPRB, 0.61593E+00_JPRB, 0.63107E+00_JPRB, 0.64389E+00_JPRB ,&
&  0.67571E+00_JPRB, 0.70758E+00_JPRB, 0.73897E+00_JPRB, 0.76758E+00_JPRB, 0.78868E+00_JPRB, 0.77131E+00_JPRB, 0.75927E+00_JPRB ,&
&  0.76982E+00_JPRB, 0.78207E+00_JPRB, 0.79819E+00_JPRB, 0.82033E+00_JPRB, 0.82552E+00_JPRB, 0.79666E+00_JPRB, 0.78778E+00_JPRB ,&
&  0.73055E+00_JPRB, 0.66069E+00_JPRB, 0.57910E+00_JPRB, 0.53087E+00_JPRB, 0.51360E+00_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.51964E+00_JPRB, 0.53278E+00_JPRB, 0.54855E+00_JPRB, 0.56841E+00_JPRB, 0.59668E+00_JPRB, 0.62866E+00_JPRB, 0.66927E+00_JPRB ,&
&  0.71009E+00_JPRB, 0.72001E+00_JPRB, 0.72129E+00_JPRB, 0.72158E+00_JPRB, 0.72061E+00_JPRB, 0.72399E+00_JPRB, 0.73115E+00_JPRB ,&
&  0.73904E+00_JPRB, 0.74791E+00_JPRB, 0.75753E+00_JPRB, 0.76764E+00_JPRB, 0.78097E+00_JPRB, 0.80029E+00_JPRB, 0.82744E+00_JPRB ,&
&  0.85574E+00_JPRB, 0.90862E+00_JPRB, 0.97995E+00_JPRB, 0.10591E+01_JPRB, 0.11388E+01_JPRB, 0.11996E+01_JPRB, 0.12049E+01_JPRB ,&
&  0.77349E+00_JPRB, 0.99855E+00_JPRB, 0.11605E+01_JPRB, 0.11063E+01_JPRB, 0.10313E+01_JPRB, 0.97902E+00_JPRB, 0.96230E+00_JPRB ,&
&  0.91895E+00_JPRB, 0.86342E+00_JPRB, 0.84943E+00_JPRB, 0.89190E+00_JPRB, 0.81939E+00_JPRB, 0.77729E+00_JPRB, 0.76610E+00_JPRB ,&
&  0.76469E+00_JPRB, 0.76819E+00_JPRB, 0.77947E+00_JPRB, 0.78953E+00_JPRB, 0.81418E+00_JPRB, 0.85050E+00_JPRB, 0.88724E+00_JPRB ,&
&  0.93427E+00_JPRB, 0.97066E+00_JPRB, 0.10146E+01_JPRB, 0.86514E+00_JPRB, 0.82733E+00_JPRB, 0.77993E+00_JPRB, 0.71921E+00_JPRB ,&
&  0.70113E+00_JPRB, 0.71216E+00_JPRB, 0.73370E+00_JPRB, 0.75548E+00_JPRB, 0.70325E+00_JPRB, 0.65539E+00_JPRB, 0.61876E+00_JPRB ,&
&  0.59700E+00_JPRB, 0.57791E+00_JPRB, 0.55902E+00_JPRB, 0.55134E+00_JPRB, 0.54728E+00_JPRB, 0.55440E+00_JPRB, 0.56976E+00_JPRB ,&
&  0.58031E+00_JPRB, 0.58416E+00_JPRB, 0.59415E+00_JPRB, 0.60654E+00_JPRB, 0.63626E+00_JPRB, 0.65427E+00_JPRB, 0.66553E+00_JPRB ,&
&  0.69290E+00_JPRB, 0.71189E+00_JPRB, 0.73876E+00_JPRB, 0.76721E+00_JPRB, 0.79234E+00_JPRB, 0.80939E+00_JPRB, 0.80758E+00_JPRB ,&
&  0.81098E+00_JPRB, 0.83816E+00_JPRB, 0.86896E+00_JPRB, 0.87736E+00_JPRB, 0.85647E+00_JPRB, 0.78006E+00_JPRB, 0.70799E+00_JPRB ,&
&  0.62751E+00_JPRB, 0.60433E+00_JPRB, 0.56939E+00_JPRB, 0.54438E+00_JPRB, 0.52698E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.61359E+00_JPRB, 0.62810E+00_JPRB, 0.69272E+00_JPRB, 0.71045E+00_JPRB, 0.77107E+00_JPRB, 0.79239E+00_JPRB, 0.81217E+00_JPRB ,&
&  0.81017E+00_JPRB, 0.80857E+00_JPRB, 0.80557E+00_JPRB, 0.80568E+00_JPRB, 0.80501E+00_JPRB, 0.80943E+00_JPRB, 0.81328E+00_JPRB ,&
&  0.82224E+00_JPRB, 0.83565E+00_JPRB, 0.85182E+00_JPRB, 0.86824E+00_JPRB, 0.89062E+00_JPRB, 0.91645E+00_JPRB, 0.94714E+00_JPRB ,&
&  0.98309E+00_JPRB, 0.10278E+01_JPRB, 0.10774E+01_JPRB, 0.11388E+01_JPRB, 0.12033E+01_JPRB, 0.12473E+01_JPRB, 0.12375E+01_JPRB ,&
&  0.94932E+00_JPRB, 0.11561E+01_JPRB, 0.11338E+01_JPRB, 0.10772E+01_JPRB, 0.99759E+00_JPRB, 0.93341E+00_JPRB, 0.90401E+00_JPRB ,&
&  0.88057E+00_JPRB, 0.82812E+00_JPRB, 0.82072E+00_JPRB, 0.84005E+00_JPRB, 0.82885E+00_JPRB, 0.79152E+00_JPRB, 0.79692E+00_JPRB ,&
&  0.81189E+00_JPRB, 0.82951E+00_JPRB, 0.84156E+00_JPRB, 0.84851E+00_JPRB, 0.87756E+00_JPRB, 0.90874E+00_JPRB, 0.94476E+00_JPRB ,&
&  0.99307E+00_JPRB, 0.10284E+01_JPRB, 0.10706E+01_JPRB, 0.94373E+00_JPRB, 0.86098E+00_JPRB, 0.81410E+00_JPRB, 0.71577E+00_JPRB ,&
&  0.69015E+00_JPRB, 0.67577E+00_JPRB, 0.73160E+00_JPRB, 0.76681E+00_JPRB, 0.71168E+00_JPRB, 0.64356E+00_JPRB, 0.60458E+00_JPRB ,&
&  0.57407E+00_JPRB, 0.55306E+00_JPRB, 0.54590E+00_JPRB, 0.53451E+00_JPRB, 0.52877E+00_JPRB, 0.54266E+00_JPRB, 0.55730E+00_JPRB ,&
&  0.58042E+00_JPRB, 0.60869E+00_JPRB, 0.63328E+00_JPRB, 0.66244E+00_JPRB, 0.68121E+00_JPRB, 0.70399E+00_JPRB, 0.71699E+00_JPRB ,&
&  0.73605E+00_JPRB, 0.77022E+00_JPRB, 0.77757E+00_JPRB, 0.80637E+00_JPRB, 0.81430E+00_JPRB, 0.84188E+00_JPRB, 0.86869E+00_JPRB ,&
&  0.89769E+00_JPRB, 0.92407E+00_JPRB, 0.93119E+00_JPRB, 0.91522E+00_JPRB, 0.86437E+00_JPRB, 0.78968E+00_JPRB, 0.63071E+00_JPRB ,&
&  0.60369E+00_JPRB, 0.58502E+00_JPRB, 0.58816E+00_JPRB, 0.59074E+00_JPRB, 0.60678E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.77004E+00_JPRB, 0.81632E+00_JPRB, 0.86208E+00_JPRB, 0.91717E+00_JPRB, 0.95902E+00_JPRB, 0.94059E+00_JPRB, 0.90889E+00_JPRB ,&
&  0.88801E+00_JPRB, 0.87734E+00_JPRB, 0.88061E+00_JPRB, 0.89110E+00_JPRB, 0.89840E+00_JPRB, 0.90025E+00_JPRB, 0.90754E+00_JPRB ,&
&  0.92058E+00_JPRB, 0.93938E+00_JPRB, 0.95873E+00_JPRB, 0.98043E+00_JPRB, 0.99538E+00_JPRB, 0.10168E+01_JPRB, 0.10425E+01_JPRB ,&
&  0.10675E+01_JPRB, 0.11030E+01_JPRB, 0.11336E+01_JPRB, 0.11667E+01_JPRB, 0.11992E+01_JPRB, 0.11892E+01_JPRB, 0.89585E+00_JPRB ,&
&  0.11216E+01_JPRB, 0.11607E+01_JPRB, 0.11670E+01_JPRB, 0.11026E+01_JPRB, 0.99995E+00_JPRB, 0.92690E+00_JPRB, 0.89489E+00_JPRB ,&
&  0.86736E+00_JPRB, 0.83790E+00_JPRB, 0.81391E+00_JPRB, 0.80118E+00_JPRB, 0.85786E+00_JPRB, 0.82120E+00_JPRB, 0.83517E+00_JPRB ,&
&  0.85298E+00_JPRB, 0.86086E+00_JPRB, 0.86430E+00_JPRB, 0.87632E+00_JPRB, 0.91808E+00_JPRB, 0.95265E+00_JPRB, 0.99790E+00_JPRB ,&
&  0.10433E+01_JPRB, 0.10859E+01_JPRB, 0.11296E+01_JPRB, 0.99805E+00_JPRB, 0.98455E+00_JPRB, 0.95072E+00_JPRB, 0.81653E+00_JPRB ,&
&  0.70712E+00_JPRB, 0.69525E+00_JPRB, 0.75866E+00_JPRB, 0.78714E+00_JPRB, 0.72131E+00_JPRB, 0.63690E+00_JPRB, 0.60450E+00_JPRB ,&
&  0.56876E+00_JPRB, 0.55044E+00_JPRB, 0.54542E+00_JPRB, 0.52624E+00_JPRB, 0.51746E+00_JPRB, 0.53823E+00_JPRB, 0.56913E+00_JPRB ,&
&  0.60817E+00_JPRB, 0.65617E+00_JPRB, 0.69351E+00_JPRB, 0.72516E+00_JPRB, 0.76793E+00_JPRB, 0.84023E+00_JPRB, 0.88049E+00_JPRB ,&
&  0.81279E+00_JPRB, 0.79069E+00_JPRB, 0.85324E+00_JPRB, 0.90282E+00_JPRB, 0.95536E+00_JPRB, 0.10346E+01_JPRB, 0.11165E+01_JPRB ,&
&  0.12423E+01_JPRB, 0.13154E+01_JPRB, 0.11195E+01_JPRB, 0.94297E+00_JPRB, 0.12691E+01_JPRB, 0.13491E+01_JPRB, 0.67844E+00_JPRB ,&
&  0.64543E+00_JPRB, 0.66202E+00_JPRB, 0.67062E+00_JPRB, 0.70143E+00_JPRB, 0.73398E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.90144E+00_JPRB, 0.92596E+00_JPRB, 0.95297E+00_JPRB, 0.96679E+00_JPRB, 0.97110E+00_JPRB, 0.95267E+00_JPRB, 0.93658E+00_JPRB ,&
&  0.93303E+00_JPRB, 0.93723E+00_JPRB, 0.94833E+00_JPRB, 0.96104E+00_JPRB, 0.96704E+00_JPRB, 0.97179E+00_JPRB, 0.98579E+00_JPRB ,&
&  0.10040E+01_JPRB, 0.10279E+01_JPRB, 0.10572E+01_JPRB, 0.10743E+01_JPRB, 0.10898E+01_JPRB, 0.10988E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.11290E+01_JPRB, 0.11611E+01_JPRB, 0.11606E+01_JPRB, 0.11776E+01_JPRB, 0.11843E+01_JPRB, 0.12054E+01_JPRB, 0.97784E+00_JPRB ,&
&  0.11371E+01_JPRB, 0.11920E+01_JPRB, 0.11666E+01_JPRB, 0.10941E+01_JPRB, 0.99143E+00_JPRB, 0.93826E+00_JPRB, 0.90227E+00_JPRB ,&
&  0.88385E+00_JPRB, 0.85932E+00_JPRB, 0.82528E+00_JPRB, 0.86168E+00_JPRB, 0.88028E+00_JPRB, 0.83369E+00_JPRB, 0.83586E+00_JPRB ,&
&  0.84224E+00_JPRB, 0.84609E+00_JPRB, 0.85897E+00_JPRB, 0.88990E+00_JPRB, 0.93852E+00_JPRB, 0.97840E+00_JPRB, 0.10240E+01_JPRB ,&
&  0.10724E+01_JPRB, 0.11321E+01_JPRB, 0.11960E+01_JPRB, 0.10908E+01_JPRB, 0.11354E+01_JPRB, 0.11305E+01_JPRB, 0.95784E+00_JPRB ,&
&  0.72925E+00_JPRB, 0.67887E+00_JPRB, 0.72622E+00_JPRB, 0.80893E+00_JPRB, 0.73777E+00_JPRB, 0.64130E+00_JPRB, 0.62446E+00_JPRB ,&
&  0.59193E+00_JPRB, 0.56503E+00_JPRB, 0.55685E+00_JPRB, 0.53346E+00_JPRB, 0.52005E+00_JPRB, 0.54597E+00_JPRB, 0.57187E+00_JPRB ,&
&  0.60563E+00_JPRB, 0.65011E+00_JPRB, 0.69777E+00_JPRB, 0.74502E+00_JPRB, 0.79986E+00_JPRB, 0.81500E+00_JPRB, 0.77074E+00_JPRB ,&
&  0.82032E+00_JPRB, 0.87435E+00_JPRB, 0.94124E+00_JPRB, 0.10538E+01_JPRB, 0.11393E+01_JPRB, 0.13598E+01_JPRB, 0.15997E+01_JPRB ,&
&  0.18220E+01_JPRB, 0.17178E+01_JPRB, 0.20500E+01_JPRB, 0.28566E+01_JPRB, 0.26332E+01_JPRB, 0.21138E+01_JPRB, 0.78066E+00_JPRB ,&
&  0.76879E+00_JPRB, 0.76632E+00_JPRB, 0.79658E+00_JPRB, 0.83446E+00_JPRB, 0.87851E+00_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.89539E+00_JPRB, 0.91525E+00_JPRB, 0.93410E+00_JPRB, 0.95138E+00_JPRB, 0.96322E+00_JPRB, 0.97083E+00_JPRB, 0.97505E+00_JPRB ,&
&  0.98839E+00_JPRB, 0.10025E+01_JPRB, 0.10133E+01_JPRB, 0.10188E+01_JPRB, 0.10237E+01_JPRB, 0.10388E+01_JPRB, 0.10611E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.11015E+01_JPRB, 0.11317E+01_JPRB, 0.11087E+01_JPRB, 0.10850E+01_JPRB, 0.10844E+01_JPRB, 0.11006E+01_JPRB ,&
&  0.10994E+01_JPRB, 0.10855E+01_JPRB, 0.10748E+01_JPRB, 0.10950E+01_JPRB, 0.11203E+01_JPRB, 0.11475E+01_JPRB, 0.95065E+00_JPRB ,&
&  0.10940E+01_JPRB, 0.11776E+01_JPRB, 0.11347E+01_JPRB, 0.10639E+01_JPRB, 0.95485E+00_JPRB, 0.91770E+00_JPRB, 0.90051E+00_JPRB ,&
&  0.88566E+00_JPRB, 0.87468E+00_JPRB, 0.85779E+00_JPRB, 0.84326E+00_JPRB, 0.83537E+00_JPRB, 0.81904E+00_JPRB, 0.81402E+00_JPRB ,&
&  0.81649E+00_JPRB, 0.81926E+00_JPRB, 0.83524E+00_JPRB, 0.86330E+00_JPRB, 0.90774E+00_JPRB, 0.95386E+00_JPRB, 0.98506E+00_JPRB ,&
&  0.10293E+01_JPRB, 0.10832E+01_JPRB, 0.10501E+01_JPRB, 0.10614E+01_JPRB, 0.11602E+01_JPRB, 0.11477E+01_JPRB, 0.97404E+00_JPRB ,&
&  0.72475E+00_JPRB, 0.69923E+00_JPRB, 0.73007E+00_JPRB, 0.81584E+00_JPRB, 0.74805E+00_JPRB, 0.67879E+00_JPRB, 0.65258E+00_JPRB ,&
&  0.59238E+00_JPRB, 0.53234E+00_JPRB, 0.54224E+00_JPRB, 0.53663E+00_JPRB, 0.52524E+00_JPRB, 0.54271E+00_JPRB, 0.56069E+00_JPRB ,&
&  0.57909E+00_JPRB, 0.61652E+00_JPRB, 0.65223E+00_JPRB, 0.68633E+00_JPRB, 0.70020E+00_JPRB, 0.66317E+00_JPRB, 0.75682E+00_JPRB ,&
&  0.76631E+00_JPRB, 0.76243E+00_JPRB, 0.80772E+00_JPRB, 0.70441E+00_JPRB, 0.99826E+00_JPRB, 0.10675E+01_JPRB, 0.11870E+01_JPRB ,&
&  0.11591E+01_JPRB, 0.15715E+01_JPRB, 0.19519E+01_JPRB, 0.15129E+01_JPRB, 0.11745E+01_JPRB, 0.83916E+00_JPRB, 0.78095E+00_JPRB ,&
&  0.78262E+00_JPRB, 0.79392E+00_JPRB, 0.81728E+00_JPRB, 0.84631E+00_JPRB, 0.87198E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.89463E+00_JPRB, 0.91311E+00_JPRB, 0.93074E+00_JPRB, 0.95009E+00_JPRB, 0.96916E+00_JPRB, 0.98208E+00_JPRB, 0.99796E+00_JPRB ,&
&  0.10163E+01_JPRB, 0.10317E+01_JPRB, 0.10398E+01_JPRB, 0.10501E+01_JPRB, 0.10660E+01_JPRB, 0.10977E+01_JPRB, 0.11263E+01_JPRB ,&
&  0.11105E+01_JPRB, 0.11235E+01_JPRB, 0.11306E+01_JPRB, 0.11316E+01_JPRB, 0.11193E+01_JPRB, 0.10904E+01_JPRB, 0.10961E+01_JPRB ,&
&  0.10871E+01_JPRB, 0.10816E+01_JPRB, 0.10771E+01_JPRB, 0.10941E+01_JPRB, 0.11246E+01_JPRB, 0.11848E+01_JPRB, 0.94629E+00_JPRB ,&
&  0.10217E+01_JPRB, 0.10899E+01_JPRB, 0.10892E+01_JPRB, 0.99027E+00_JPRB, 0.87974E+00_JPRB, 0.84337E+00_JPRB, 0.83065E+00_JPRB ,&
&  0.83119E+00_JPRB, 0.80905E+00_JPRB, 0.80146E+00_JPRB, 0.79704E+00_JPRB, 0.79301E+00_JPRB, 0.79701E+00_JPRB, 0.78988E+00_JPRB ,&
&  0.79064E+00_JPRB, 0.79613E+00_JPRB, 0.80726E+00_JPRB, 0.83106E+00_JPRB, 0.85938E+00_JPRB, 0.88598E+00_JPRB, 0.90775E+00_JPRB ,&
&  0.93446E+00_JPRB, 0.95484E+00_JPRB, 0.89382E+00_JPRB, 0.91015E+00_JPRB, 0.97795E+00_JPRB, 0.95483E+00_JPRB, 0.84584E+00_JPRB ,&
&  0.75360E+00_JPRB, 0.68059E+00_JPRB, 0.65486E+00_JPRB, 0.78351E+00_JPRB, 0.75687E+00_JPRB, 0.70398E+00_JPRB, 0.64535E+00_JPRB ,&
&  0.55954E+00_JPRB, 0.50410E+00_JPRB, 0.50293E+00_JPRB, 0.53578E+00_JPRB, 0.52607E+00_JPRB, 0.52142E+00_JPRB, 0.52289E+00_JPRB ,&
&  0.52648E+00_JPRB, 0.52717E+00_JPRB, 0.53900E+00_JPRB, 0.57076E+00_JPRB, 0.59166E+00_JPRB, 0.54205E+00_JPRB, 0.68520E+00_JPRB ,&
&  0.63568E+00_JPRB, 0.52146E+00_JPRB, 0.60148E+00_JPRB, 0.75051E+00_JPRB, 0.78507E+00_JPRB, 0.80458E+00_JPRB, 0.90920E+00_JPRB ,&
&  0.10540E+01_JPRB, 0.11807E+01_JPRB, 0.11848E+01_JPRB, 0.96027E+00_JPRB, 0.83759E+00_JPRB, 0.78315E+00_JPRB, 0.77187E+00_JPRB ,&
&  0.77697E+00_JPRB, 0.79215E+00_JPRB, 0.81794E+00_JPRB, 0.84775E+00_JPRB, 0.87367E+00_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.88324E+00_JPRB, 0.90420E+00_JPRB, 0.92029E+00_JPRB, 0.93230E+00_JPRB, 0.94380E+00_JPRB, 0.95843E+00_JPRB, 0.97233E+00_JPRB ,&
&  0.98649E+00_JPRB, 0.99822E+00_JPRB, 0.10062E+01_JPRB, 0.10209E+01_JPRB, 0.10377E+01_JPRB, 0.10711E+01_JPRB, 0.10876E+01_JPRB ,&
&  0.11179E+01_JPRB, 0.11278E+01_JPRB, 0.11255E+01_JPRB, 0.11380E+01_JPRB, 0.11310E+01_JPRB, 0.11107E+01_JPRB, 0.11049E+01_JPRB ,&
&  0.11052E+01_JPRB, 0.10975E+01_JPRB, 0.10932E+01_JPRB, 0.10929E+01_JPRB, 0.10942E+01_JPRB, 0.10673E+01_JPRB, 0.11252E+01_JPRB ,&
&  0.76668E+00_JPRB, 0.98415E+00_JPRB, 0.10490E+01_JPRB, 0.93215E+00_JPRB, 0.86682E+00_JPRB, 0.78339E+00_JPRB, 0.76746E+00_JPRB ,&
&  0.77051E+00_JPRB, 0.75997E+00_JPRB, 0.76344E+00_JPRB, 0.77575E+00_JPRB, 0.77880E+00_JPRB, 0.78307E+00_JPRB, 0.77705E+00_JPRB ,&
&  0.78321E+00_JPRB, 0.77824E+00_JPRB, 0.78178E+00_JPRB, 0.78907E+00_JPRB, 0.80988E+00_JPRB, 0.80590E+00_JPRB, 0.80319E+00_JPRB ,&
&  0.81604E+00_JPRB, 0.82917E+00_JPRB, 0.67072E+00_JPRB, 0.66087E+00_JPRB, 0.71920E+00_JPRB, 0.72591E+00_JPRB, 0.67193E+00_JPRB ,&
&  0.64925E+00_JPRB, 0.64876E+00_JPRB, 0.70435E+00_JPRB, 0.71321E+00_JPRB, 0.72508E+00_JPRB, 0.70949E+00_JPRB, 0.68624E+00_JPRB ,&
&  0.60984E+00_JPRB, 0.52501E+00_JPRB, 0.49906E+00_JPRB, 0.51750E+00_JPRB, 0.52847E+00_JPRB, 0.53046E+00_JPRB, 0.49354E+00_JPRB ,&
&  0.45818E+00_JPRB, 0.42721E+00_JPRB, 0.41765E+00_JPRB, 0.43038E+00_JPRB, 0.37127E+00_JPRB, 0.50189E+00_JPRB, 0.63159E+00_JPRB ,&
&  0.55990E+00_JPRB, 0.46771E+00_JPRB, 0.46250E+00_JPRB, 0.63316E+00_JPRB, 0.63725E+00_JPRB, 0.60636E+00_JPRB, 0.64578E+00_JPRB ,&
&  0.74564E+00_JPRB, 0.79429E+00_JPRB, 0.80989E+00_JPRB, 0.73869E+00_JPRB, 0.74761E+00_JPRB, 0.72600E+00_JPRB, 0.72590E+00_JPRB ,&
&  0.74071E+00_JPRB, 0.76700E+00_JPRB, 0.79947E+00_JPRB, 0.83090E+00_JPRB, 0.85822E+00_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.83935E+00_JPRB, 0.85662E+00_JPRB, 0.86717E+00_JPRB, 0.87763E+00_JPRB, 0.89302E+00_JPRB, 0.90905E+00_JPRB, 0.91617E+00_JPRB ,&
&  0.93276E+00_JPRB, 0.94817E+00_JPRB, 0.95495E+00_JPRB, 0.97338E+00_JPRB, 0.98142E+00_JPRB, 0.10317E+01_JPRB, 0.10506E+01_JPRB ,&
&  0.10869E+01_JPRB, 0.11058E+01_JPRB, 0.11108E+01_JPRB, 0.11122E+01_JPRB, 0.11248E+01_JPRB, 0.11234E+01_JPRB, 0.11191E+01_JPRB ,&
&  0.11375E+01_JPRB, 0.11757E+01_JPRB, 0.12257E+01_JPRB, 0.12749E+01_JPRB, 0.13061E+01_JPRB, 0.14024E+01_JPRB, 0.15295E+01_JPRB ,&
&  0.10093E+01_JPRB, 0.82363E+00_JPRB, 0.90048E+00_JPRB, 0.79447E+00_JPRB, 0.78292E+00_JPRB, 0.77484E+00_JPRB, 0.73898E+00_JPRB ,&
&  0.72054E+00_JPRB, 0.73926E+00_JPRB, 0.76585E+00_JPRB, 0.77442E+00_JPRB, 0.76746E+00_JPRB, 0.76832E+00_JPRB, 0.75940E+00_JPRB ,&
&  0.75671E+00_JPRB, 0.75456E+00_JPRB, 0.76746E+00_JPRB, 0.78959E+00_JPRB, 0.73037E+00_JPRB, 0.77656E+00_JPRB, 0.78726E+00_JPRB ,&
&  0.79402E+00_JPRB, 0.72363E+00_JPRB, 0.54822E+00_JPRB, 0.52424E+00_JPRB, 0.55472E+00_JPRB, 0.55506E+00_JPRB, 0.53660E+00_JPRB ,&
&  0.57605E+00_JPRB, 0.58375E+00_JPRB, 0.51260E+00_JPRB, 0.62239E+00_JPRB, 0.69124E+00_JPRB, 0.72319E+00_JPRB, 0.70676E+00_JPRB ,&
&  0.67052E+00_JPRB, 0.59650E+00_JPRB, 0.56747E+00_JPRB, 0.55130E+00_JPRB, 0.54077E+00_JPRB, 0.51696E+00_JPRB, 0.50224E+00_JPRB ,&
&  0.46225E+00_JPRB, 0.41246E+00_JPRB, 0.38868E+00_JPRB, 0.37054E+00_JPRB, 0.39880E+00_JPRB, 0.39587E+00_JPRB, 0.56100E+00_JPRB ,&
&  0.45345E+00_JPRB, 0.43901E+00_JPRB, 0.40809E+00_JPRB, 0.54669E+00_JPRB, 0.51264E+00_JPRB, 0.46445E+00_JPRB, 0.51902E+00_JPRB ,&
&  0.57545E+00_JPRB, 0.60402E+00_JPRB, 0.61385E+00_JPRB, 0.62828E+00_JPRB, 0.66962E+00_JPRB, 0.69022E+00_JPRB, 0.70760E+00_JPRB ,&
&  0.70730E+00_JPRB, 0.73549E+00_JPRB, 0.77463E+00_JPRB, 0.80374E+00_JPRB, 0.81429E+00_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.72115E+00_JPRB, 0.73427E+00_JPRB, 0.75139E+00_JPRB, 0.76965E+00_JPRB, 0.77950E+00_JPRB, 0.79633E+00_JPRB, 0.81206E+00_JPRB ,&
&  0.83374E+00_JPRB, 0.86202E+00_JPRB, 0.87796E+00_JPRB, 0.88328E+00_JPRB, 0.89996E+00_JPRB, 0.92064E+00_JPRB, 0.93203E+00_JPRB ,&
&  0.95428E+00_JPRB, 0.96282E+00_JPRB, 0.10130E+01_JPRB, 0.10381E+01_JPRB, 0.10752E+01_JPRB, 0.11256E+01_JPRB, 0.11331E+01_JPRB ,&
&  0.11276E+01_JPRB, 0.11151E+01_JPRB, 0.11268E+01_JPRB, 0.11241E+01_JPRB, 0.11976E+01_JPRB, 0.11928E+01_JPRB, 0.11173E+01_JPRB ,&
&  0.81219E+00_JPRB, 0.68175E+00_JPRB, 0.85878E+00_JPRB, 0.77177E+00_JPRB, 0.76637E+00_JPRB, 0.74536E+00_JPRB, 0.67996E+00_JPRB ,&
&  0.67989E+00_JPRB, 0.70885E+00_JPRB, 0.74299E+00_JPRB, 0.76687E+00_JPRB, 0.79793E+00_JPRB, 0.80044E+00_JPRB, 0.78756E+00_JPRB ,&
&  0.78517E+00_JPRB, 0.79074E+00_JPRB, 0.78846E+00_JPRB, 0.71320E+00_JPRB, 0.63378E+00_JPRB, 0.65857E+00_JPRB, 0.66374E+00_JPRB ,&
&  0.65607E+00_JPRB, 0.62742E+00_JPRB, 0.50926E+00_JPRB, 0.47314E+00_JPRB, 0.48984E+00_JPRB, 0.45666E+00_JPRB, 0.45279E+00_JPRB ,&
&  0.51161E+00_JPRB, 0.47115E+00_JPRB, 0.32549E+00_JPRB, 0.45485E+00_JPRB, 0.58979E+00_JPRB, 0.68610E+00_JPRB, 0.72880E+00_JPRB ,&
&  0.71653E+00_JPRB, 0.65501E+00_JPRB, 0.62204E+00_JPRB, 0.61768E+00_JPRB, 0.61448E+00_JPRB, 0.56180E+00_JPRB, 0.52864E+00_JPRB ,&
&  0.47131E+00_JPRB, 0.41559E+00_JPRB, 0.37514E+00_JPRB, 0.36651E+00_JPRB, 0.35798E+00_JPRB, 0.40620E+00_JPRB, 0.46645E+00_JPRB ,&
&  0.44440E+00_JPRB, 0.41228E+00_JPRB, 0.46697E+00_JPRB, 0.47078E+00_JPRB, 0.35633E+00_JPRB, 0.42462E+00_JPRB, 0.45961E+00_JPRB ,&
&  0.48632E+00_JPRB, 0.50447E+00_JPRB, 0.51888E+00_JPRB, 0.54284E+00_JPRB, 0.57654E+00_JPRB, 0.58850E+00_JPRB, 0.59261E+00_JPRB ,&
&  0.60870E+00_JPRB, 0.63575E+00_JPRB, 0.65558E+00_JPRB, 0.67995E+00_JPRB, 0.70173E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.55950E+00_JPRB, 0.55904E+00_JPRB, 0.56224E+00_JPRB, 0.56861E+00_JPRB, 0.56893E+00_JPRB, 0.56667E+00_JPRB, 0.57786E+00_JPRB ,&
&  0.59884E+00_JPRB, 0.63922E+00_JPRB, 0.69697E+00_JPRB, 0.72878E+00_JPRB, 0.71179E+00_JPRB, 0.66763E+00_JPRB, 0.65385E+00_JPRB ,&
&  0.65422E+00_JPRB, 0.65205E+00_JPRB, 0.67987E+00_JPRB, 0.72735E+00_JPRB, 0.79047E+00_JPRB, 0.81500E+00_JPRB, 0.87492E+00_JPRB ,&
&  0.87677E+00_JPRB, 0.88983E+00_JPRB, 0.96635E+00_JPRB, 0.10286E+01_JPRB, 0.15553E+01_JPRB, 0.14379E+01_JPRB, 0.10277E+01_JPRB ,&
&  0.76733E+00_JPRB, 0.76172E+00_JPRB, 0.85416E+00_JPRB, 0.85484E+00_JPRB, 0.87683E+00_JPRB, 0.84004E+00_JPRB, 0.85201E+00_JPRB ,&
&  0.83951E+00_JPRB, 0.87676E+00_JPRB, 0.90428E+00_JPRB, 0.94180E+00_JPRB, 0.96727E+00_JPRB, 0.97048E+00_JPRB, 0.97943E+00_JPRB ,&
&  0.98429E+00_JPRB, 0.97901E+00_JPRB, 0.95346E+00_JPRB, 0.79213E+00_JPRB, 0.77860E+00_JPRB, 0.75173E+00_JPRB, 0.70836E+00_JPRB ,&
&  0.65568E+00_JPRB, 0.61133E+00_JPRB, 0.61893E+00_JPRB, 0.57416E+00_JPRB, 0.53510E+00_JPRB, 0.48128E+00_JPRB, 0.43738E+00_JPRB ,&
&  0.44154E+00_JPRB, 0.38204E+00_JPRB, 0.27491E+00_JPRB, 0.38060E+00_JPRB, 0.52870E+00_JPRB, 0.58128E+00_JPRB, 0.71240E+00_JPRB ,&
&  0.75319E+00_JPRB, 0.72393E+00_JPRB, 0.69538E+00_JPRB, 0.68741E+00_JPRB, 0.64279E+00_JPRB, 0.60720E+00_JPRB, 0.51849E+00_JPRB ,&
&  0.48908E+00_JPRB, 0.42226E+00_JPRB, 0.37908E+00_JPRB, 0.36392E+00_JPRB, 0.35641E+00_JPRB, 0.36037E+00_JPRB, 0.44238E+00_JPRB ,&
&  0.49064E+00_JPRB, 0.47555E+00_JPRB, 0.50536E+00_JPRB, 0.52209E+00_JPRB, 0.43262E+00_JPRB, 0.40962E+00_JPRB, 0.41597E+00_JPRB ,&
&  0.42291E+00_JPRB, 0.43827E+00_JPRB, 0.46445E+00_JPRB, 0.48284E+00_JPRB, 0.49637E+00_JPRB, 0.50854E+00_JPRB, 0.52288E+00_JPRB ,&
&  0.53204E+00_JPRB, 0.54326E+00_JPRB, 0.54786E+00_JPRB, 0.55265E+00_JPRB, 0.55720E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.58894E+00_JPRB, 0.59028E+00_JPRB, 0.57761E+00_JPRB, 0.59122E+00_JPRB, 0.61594E+00_JPRB, 0.67242E+00_JPRB, 0.73036E+00_JPRB ,&
&  0.80808E+00_JPRB, 0.90369E+00_JPRB, 0.10192E+01_JPRB, 0.11142E+01_JPRB, 0.11333E+01_JPRB, 0.11065E+01_JPRB, 0.10048E+01_JPRB ,&
&  0.87556E+00_JPRB, 0.74595E+00_JPRB, 0.67649E+00_JPRB, 0.65984E+00_JPRB, 0.70686E+00_JPRB, 0.73579E+00_JPRB, 0.80108E+00_JPRB ,&
&  0.89917E+00_JPRB, 0.10614E+01_JPRB, 0.12474E+01_JPRB, 0.16833E+01_JPRB, 0.20267E+01_JPRB, 0.12706E+01_JPRB, 0.99842E+00_JPRB ,&
&  0.95037E+00_JPRB, 0.10547E+01_JPRB, 0.10461E+01_JPRB, 0.10738E+01_JPRB, 0.11650E+01_JPRB, 0.12387E+01_JPRB, 0.13272E+01_JPRB ,&
&  0.13673E+01_JPRB, 0.14046E+01_JPRB, 0.14305E+01_JPRB, 0.14450E+01_JPRB, 0.14627E+01_JPRB, 0.14613E+01_JPRB, 0.14390E+01_JPRB ,&
&  0.14016E+01_JPRB, 0.13324E+01_JPRB, 0.12797E+01_JPRB, 0.12450E+01_JPRB, 0.12053E+01_JPRB, 0.11052E+01_JPRB, 0.97223E+00_JPRB ,&
&  0.86712E+00_JPRB, 0.80054E+00_JPRB, 0.81205E+00_JPRB, 0.77319E+00_JPRB, 0.67587E+00_JPRB, 0.60788E+00_JPRB, 0.50859E+00_JPRB ,&
&  0.45997E+00_JPRB, 0.42616E+00_JPRB, 0.34824E+00_JPRB, 0.50248E+00_JPRB, 0.61219E+00_JPRB, 0.66427E+00_JPRB, 0.69985E+00_JPRB ,&
&  0.76561E+00_JPRB, 0.76162E+00_JPRB, 0.75239E+00_JPRB, 0.71983E+00_JPRB, 0.63914E+00_JPRB, 0.64893E+00_JPRB, 0.49186E+00_JPRB ,&
&  0.49325E+00_JPRB, 0.43099E+00_JPRB, 0.40301E+00_JPRB, 0.40886E+00_JPRB, 0.42009E+00_JPRB, 0.38223E+00_JPRB, 0.43511E+00_JPRB ,&
&  0.47519E+00_JPRB, 0.68415E+00_JPRB, 0.89306E+00_JPRB, 0.12719E+01_JPRB, 0.16915E+01_JPRB, 0.45097E+00_JPRB, 0.43140E+00_JPRB ,&
&  0.42608E+00_JPRB, 0.43343E+00_JPRB, 0.45230E+00_JPRB, 0.46447E+00_JPRB, 0.48374E+00_JPRB, 0.49537E+00_JPRB, 0.49910E+00_JPRB ,&
&  0.50757E+00_JPRB, 0.52767E+00_JPRB, 0.54784E+00_JPRB, 0.55757E+00_JPRB, 0.57686E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.84652E+00_JPRB, 0.88357E+00_JPRB, 0.88679E+00_JPRB, 0.95239E+00_JPRB, 0.10136E+01_JPRB, 0.10374E+01_JPRB, 0.10561E+01_JPRB ,&
&  0.11086E+01_JPRB, 0.11933E+01_JPRB, 0.12587E+01_JPRB, 0.12955E+01_JPRB, 0.13328E+01_JPRB, 0.13925E+01_JPRB, 0.13982E+01_JPRB ,&
&  0.13586E+01_JPRB, 0.13220E+01_JPRB, 0.12945E+01_JPRB, 0.12605E+01_JPRB, 0.12515E+01_JPRB, 0.12776E+01_JPRB, 0.14082E+01_JPRB ,&
&  0.15144E+01_JPRB, 0.15305E+01_JPRB, 0.15620E+01_JPRB, 0.13544E+01_JPRB, 0.91707E+00_JPRB, 0.11190E+01_JPRB, 0.10909E+01_JPRB ,&
&  0.11617E+01_JPRB, 0.12308E+01_JPRB, 0.12814E+01_JPRB, 0.13461E+01_JPRB, 0.14073E+01_JPRB, 0.14871E+01_JPRB, 0.15671E+01_JPRB ,&
&  0.16179E+01_JPRB, 0.16762E+01_JPRB, 0.17416E+01_JPRB, 0.17891E+01_JPRB, 0.18439E+01_JPRB, 0.19220E+01_JPRB, 0.19383E+01_JPRB ,&
&  0.19270E+01_JPRB, 0.18607E+01_JPRB, 0.17932E+01_JPRB, 0.17407E+01_JPRB, 0.15855E+01_JPRB, 0.14125E+01_JPRB, 0.12140E+01_JPRB ,&
&  0.10919E+01_JPRB, 0.10281E+01_JPRB, 0.10180E+01_JPRB, 0.99184E+00_JPRB, 0.91716E+00_JPRB, 0.82361E+00_JPRB, 0.70604E+00_JPRB ,&
&  0.63042E+00_JPRB, 0.58711E+00_JPRB, 0.58408E+00_JPRB, 0.65450E+00_JPRB, 0.51052E+00_JPRB, 0.59075E+00_JPRB, 0.76949E+00_JPRB ,&
&  0.77787E+00_JPRB, 0.74884E+00_JPRB, 0.73006E+00_JPRB, 0.64982E+00_JPRB, 0.60210E+00_JPRB, 0.47881E+00_JPRB, 0.49430E+00_JPRB ,&
&  0.51831E+00_JPRB, 0.45631E+00_JPRB, 0.43805E+00_JPRB, 0.46558E+00_JPRB, 0.48969E+00_JPRB, 0.44704E+00_JPRB, 0.44993E+00_JPRB ,&
&  0.49140E+00_JPRB, 0.60593E+00_JPRB, 0.69954E+00_JPRB, 0.82897E+00_JPRB, 0.91322E+00_JPRB, 0.49523E+00_JPRB, 0.42273E+00_JPRB ,&
&  0.40854E+00_JPRB, 0.42085E+00_JPRB, 0.44238E+00_JPRB, 0.46780E+00_JPRB, 0.49842E+00_JPRB, 0.53653E+00_JPRB, 0.57686E+00_JPRB ,&
&  0.60962E+00_JPRB, 0.63344E+00_JPRB, 0.67296E+00_JPRB, 0.72541E+00_JPRB, 0.79522E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.11433E+01_JPRB, 0.12206E+01_JPRB, 0.12873E+01_JPRB, 0.12680E+01_JPRB, 0.12555E+01_JPRB, 0.12146E+01_JPRB, 0.11808E+01_JPRB ,&
&  0.12146E+01_JPRB, 0.12603E+01_JPRB, 0.13303E+01_JPRB, 0.13823E+01_JPRB, 0.14161E+01_JPRB, 0.14866E+01_JPRB, 0.15662E+01_JPRB ,&
&  0.15894E+01_JPRB, 0.15928E+01_JPRB, 0.15918E+01_JPRB, 0.15826E+01_JPRB, 0.15103E+01_JPRB, 0.14734E+01_JPRB, 0.14191E+01_JPRB ,&
&  0.10714E+01_JPRB, 0.89821E+00_JPRB, 0.10727E+01_JPRB, 0.12536E+01_JPRB, 0.11850E+01_JPRB, 0.12245E+01_JPRB, 0.13013E+01_JPRB ,&
&  0.14268E+01_JPRB, 0.14350E+01_JPRB, 0.14592E+01_JPRB, 0.14632E+01_JPRB, 0.14563E+01_JPRB, 0.15417E+01_JPRB, 0.16353E+01_JPRB ,&
&  0.17035E+01_JPRB, 0.17691E+01_JPRB, 0.18374E+01_JPRB, 0.19093E+01_JPRB, 0.19713E+01_JPRB, 0.20436E+01_JPRB, 0.20741E+01_JPRB ,&
&  0.20753E+01_JPRB, 0.20290E+01_JPRB, 0.19688E+01_JPRB, 0.18121E+01_JPRB, 0.16557E+01_JPRB, 0.15257E+01_JPRB, 0.13869E+01_JPRB ,&
&  0.13330E+01_JPRB, 0.13244E+01_JPRB, 0.13377E+01_JPRB, 0.13167E+01_JPRB, 0.12636E+01_JPRB, 0.11331E+01_JPRB, 0.98815E+00_JPRB ,&
&  0.84957E+00_JPRB, 0.74040E+00_JPRB, 0.75372E+00_JPRB, 0.79286E+00_JPRB, 0.65405E+00_JPRB, 0.66017E+00_JPRB, 0.70317E+00_JPRB ,&
&  0.82255E+00_JPRB, 0.76551E+00_JPRB, 0.69025E+00_JPRB, 0.63474E+00_JPRB, 0.64442E+00_JPRB, 0.55694E+00_JPRB, 0.55906E+00_JPRB ,&
&  0.53334E+00_JPRB, 0.47906E+00_JPRB, 0.44598E+00_JPRB, 0.47294E+00_JPRB, 0.40950E+00_JPRB, 0.41602E+00_JPRB, 0.46680E+00_JPRB ,&
&  0.53612E+00_JPRB, 0.53970E+00_JPRB, 0.58031E+00_JPRB, 0.58184E+00_JPRB, 0.57001E+00_JPRB, 0.45647E+00_JPRB, 0.42064E+00_JPRB ,&
&  0.40666E+00_JPRB, 0.42270E+00_JPRB, 0.45746E+00_JPRB, 0.48751E+00_JPRB, 0.53245E+00_JPRB, 0.58531E+00_JPRB, 0.67726E+00_JPRB ,&
&  0.75904E+00_JPRB, 0.83418E+00_JPRB, 0.89295E+00_JPRB, 0.96581E+00_JPRB, 0.10473E+01_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.11157E+01_JPRB, 0.12344E+01_JPRB, 0.13545E+01_JPRB, 0.14376E+01_JPRB, 0.13980E+01_JPRB, 0.13403E+01_JPRB, 0.12996E+01_JPRB ,&
&  0.12728E+01_JPRB, 0.12924E+01_JPRB, 0.13639E+01_JPRB, 0.14297E+01_JPRB, 0.14764E+01_JPRB, 0.15508E+01_JPRB, 0.16598E+01_JPRB ,&
&  0.17459E+01_JPRB, 0.18061E+01_JPRB, 0.18456E+01_JPRB, 0.18741E+01_JPRB, 0.18058E+01_JPRB, 0.17169E+01_JPRB, 0.13927E+01_JPRB ,&
&  0.91695E+00_JPRB, 0.10084E+01_JPRB, 0.13862E+01_JPRB, 0.13223E+01_JPRB, 0.13183E+01_JPRB, 0.13495E+01_JPRB, 0.14596E+01_JPRB ,&
&  0.16194E+01_JPRB, 0.16526E+01_JPRB, 0.16163E+01_JPRB, 0.15692E+01_JPRB, 0.15303E+01_JPRB, 0.15906E+01_JPRB, 0.16928E+01_JPRB ,&
&  0.17705E+01_JPRB, 0.18107E+01_JPRB, 0.18730E+01_JPRB, 0.19574E+01_JPRB, 0.20729E+01_JPRB, 0.21800E+01_JPRB, 0.22336E+01_JPRB ,&
&  0.21860E+01_JPRB, 0.21713E+01_JPRB, 0.20164E+01_JPRB, 0.17648E+01_JPRB, 0.16638E+01_JPRB, 0.16625E+01_JPRB, 0.16633E+01_JPRB ,&
&  0.17072E+01_JPRB, 0.18020E+01_JPRB, 0.18743E+01_JPRB, 0.17918E+01_JPRB, 0.15809E+01_JPRB, 0.14421E+01_JPRB, 0.12140E+01_JPRB ,&
&  0.11292E+01_JPRB, 0.94857E+00_JPRB, 0.92178E+00_JPRB, 0.80926E+00_JPRB, 0.80190E+00_JPRB, 0.83052E+00_JPRB, 0.88520E+00_JPRB ,&
&  0.93281E+00_JPRB, 0.88552E+00_JPRB, 0.80477E+00_JPRB, 0.72749E+00_JPRB, 0.77790E+00_JPRB, 0.76460E+00_JPRB, 0.75637E+00_JPRB ,&
&  0.62316E+00_JPRB, 0.58006E+00_JPRB, 0.47299E+00_JPRB, 0.44031E+00_JPRB, 0.34723E+00_JPRB, 0.35327E+00_JPRB, 0.46616E+00_JPRB ,&
&  0.59652E+00_JPRB, 0.56003E+00_JPRB, 0.57243E+00_JPRB, 0.55911E+00_JPRB, 0.50695E+00_JPRB, 0.42959E+00_JPRB, 0.39790E+00_JPRB ,&
&  0.40155E+00_JPRB, 0.44398E+00_JPRB, 0.48334E+00_JPRB, 0.51568E+00_JPRB, 0.57177E+00_JPRB, 0.63338E+00_JPRB, 0.69603E+00_JPRB ,&
&  0.73878E+00_JPRB, 0.79570E+00_JPRB, 0.85469E+00_JPRB, 0.91482E+00_JPRB, 0.10035E+01_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.84541E+00_JPRB, 0.89397E+00_JPRB, 0.94070E+00_JPRB, 0.98219E+00_JPRB, 0.10161E+01_JPRB, 0.99390E+00_JPRB, 0.10005E+01_JPRB ,&
&  0.10747E+01_JPRB, 0.11868E+01_JPRB, 0.13032E+01_JPRB, 0.13852E+01_JPRB, 0.14170E+01_JPRB, 0.14865E+01_JPRB, 0.16221E+01_JPRB ,&
&  0.17440E+01_JPRB, 0.18696E+01_JPRB, 0.19780E+01_JPRB, 0.20780E+01_JPRB, 0.20343E+01_JPRB, 0.18860E+01_JPRB, 0.13997E+01_JPRB ,&
&  0.97399E+00_JPRB, 0.16088E+01_JPRB, 0.13521E+01_JPRB, 0.13501E+01_JPRB, 0.14198E+01_JPRB, 0.15157E+01_JPRB, 0.16046E+01_JPRB ,&
&  0.17104E+01_JPRB, 0.17260E+01_JPRB, 0.16868E+01_JPRB, 0.16314E+01_JPRB, 0.15886E+01_JPRB, 0.16171E+01_JPRB, 0.16895E+01_JPRB ,&
&  0.17686E+01_JPRB, 0.18172E+01_JPRB, 0.18934E+01_JPRB, 0.20143E+01_JPRB, 0.21728E+01_JPRB, 0.23365E+01_JPRB, 0.24012E+01_JPRB ,&
&  0.23727E+01_JPRB, 0.23230E+01_JPRB, 0.20848E+01_JPRB, 0.18015E+01_JPRB, 0.17065E+01_JPRB, 0.18405E+01_JPRB, 0.20399E+01_JPRB ,&
&  0.21559E+01_JPRB, 0.22324E+01_JPRB, 0.24858E+01_JPRB, 0.25366E+01_JPRB, 0.23025E+01_JPRB, 0.19869E+01_JPRB, 0.15074E+01_JPRB ,&
&  0.14369E+01_JPRB, 0.11971E+01_JPRB, 0.10758E+01_JPRB, 0.92950E+00_JPRB, 0.89888E+00_JPRB, 0.94715E+00_JPRB, 0.10104E+01_JPRB ,&
&  0.99776E+00_JPRB, 0.10445E+01_JPRB, 0.97816E+00_JPRB, 0.93261E+00_JPRB, 0.95965E+00_JPRB, 0.10031E+01_JPRB, 0.92531E+00_JPRB ,&
&  0.80697E+00_JPRB, 0.59673E+00_JPRB, 0.50058E+00_JPRB, 0.42005E+00_JPRB, 0.33816E+00_JPRB, 0.30251E+00_JPRB, 0.41263E+00_JPRB ,&
&  0.52883E+00_JPRB, 0.56443E+00_JPRB, 0.65140E+00_JPRB, 0.65307E+00_JPRB, 0.56675E+00_JPRB, 0.46072E+00_JPRB, 0.41213E+00_JPRB ,&
&  0.42455E+00_JPRB, 0.47524E+00_JPRB, 0.51089E+00_JPRB, 0.54418E+00_JPRB, 0.59090E+00_JPRB, 0.60992E+00_JPRB, 0.65991E+00_JPRB ,&
&  0.67822E+00_JPRB, 0.71213E+00_JPRB, 0.74593E+00_JPRB, 0.76124E+00_JPRB, 0.79493E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.64662E+00_JPRB, 0.67030E+00_JPRB, 0.71165E+00_JPRB, 0.74226E+00_JPRB, 0.76725E+00_JPRB, 0.79976E+00_JPRB, 0.80767E+00_JPRB ,&
&  0.85493E+00_JPRB, 0.95485E+00_JPRB, 0.10832E+01_JPRB, 0.12240E+01_JPRB, 0.12735E+01_JPRB, 0.12982E+01_JPRB, 0.14159E+01_JPRB ,&
&  0.16026E+01_JPRB, 0.18673E+01_JPRB, 0.21502E+01_JPRB, 0.23867E+01_JPRB, 0.24120E+01_JPRB, 0.22169E+01_JPRB, 0.15565E+01_JPRB ,&
&  0.12892E+01_JPRB, 0.16746E+01_JPRB, 0.15293E+01_JPRB, 0.14823E+01_JPRB, 0.16033E+01_JPRB, 0.17363E+01_JPRB, 0.16650E+01_JPRB ,&
&  0.16452E+01_JPRB, 0.16568E+01_JPRB, 0.16004E+01_JPRB, 0.15747E+01_JPRB, 0.15924E+01_JPRB, 0.16048E+01_JPRB, 0.16504E+01_JPRB ,&
&  0.17154E+01_JPRB, 0.17569E+01_JPRB, 0.18504E+01_JPRB, 0.20347E+01_JPRB, 0.22559E+01_JPRB, 0.24437E+01_JPRB, 0.25456E+01_JPRB ,&
&  0.25755E+01_JPRB, 0.24854E+01_JPRB, 0.23340E+01_JPRB, 0.18951E+01_JPRB, 0.17979E+01_JPRB, 0.19927E+01_JPRB, 0.23915E+01_JPRB ,&
&  0.26289E+01_JPRB, 0.27577E+01_JPRB, 0.31482E+01_JPRB, 0.34433E+01_JPRB, 0.32796E+01_JPRB, 0.27178E+01_JPRB, 0.21101E+01_JPRB ,&
&  0.18191E+01_JPRB, 0.15285E+01_JPRB, 0.12380E+01_JPRB, 0.11460E+01_JPRB, 0.10675E+01_JPRB, 0.10699E+01_JPRB, 0.11653E+01_JPRB ,&
&  0.11903E+01_JPRB, 0.12337E+01_JPRB, 0.11692E+01_JPRB, 0.11220E+01_JPRB, 0.10878E+01_JPRB, 0.10810E+01_JPRB, 0.11324E+01_JPRB ,&
&  0.10594E+01_JPRB, 0.79629E+00_JPRB, 0.62213E+00_JPRB, 0.41292E+00_JPRB, 0.30879E+00_JPRB, 0.26350E+00_JPRB, 0.34753E+00_JPRB ,&
&  0.47710E+00_JPRB, 0.51501E+00_JPRB, 0.57122E+00_JPRB, 0.77043E+00_JPRB, 0.72353E+00_JPRB, 0.54909E+00_JPRB, 0.42651E+00_JPRB ,&
&  0.42950E+00_JPRB, 0.47387E+00_JPRB, 0.49955E+00_JPRB, 0.50940E+00_JPRB, 0.54055E+00_JPRB, 0.56831E+00_JPRB, 0.57637E+00_JPRB ,&
&  0.59915E+00_JPRB, 0.61859E+00_JPRB, 0.62222E+00_JPRB, 0.61737E+00_JPRB, 0.63135E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.54953E+00_JPRB, 0.57126E+00_JPRB, 0.57940E+00_JPRB, 0.58121E+00_JPRB, 0.59374E+00_JPRB, 0.62484E+00_JPRB, 0.63710E+00_JPRB ,&
&  0.68017E+00_JPRB, 0.74999E+00_JPRB, 0.84923E+00_JPRB, 0.99551E+00_JPRB, 0.10925E+01_JPRB, 0.11144E+01_JPRB, 0.11619E+01_JPRB ,&
&  0.12987E+01_JPRB, 0.15869E+01_JPRB, 0.20399E+01_JPRB, 0.24655E+01_JPRB, 0.25227E+01_JPRB, 0.22636E+01_JPRB, 0.20475E+01_JPRB ,&
&  0.18026E+01_JPRB, 0.20020E+01_JPRB, 0.18619E+01_JPRB, 0.20401E+01_JPRB, 0.23236E+01_JPRB, 0.26730E+01_JPRB, 0.17611E+01_JPRB ,&
&  0.15951E+01_JPRB, 0.15592E+01_JPRB, 0.15056E+01_JPRB, 0.15057E+01_JPRB, 0.14920E+01_JPRB, 0.15282E+01_JPRB, 0.15561E+01_JPRB ,&
&  0.15776E+01_JPRB, 0.16672E+01_JPRB, 0.17873E+01_JPRB, 0.20006E+01_JPRB, 0.23291E+01_JPRB, 0.25765E+01_JPRB, 0.27226E+01_JPRB ,&
&  0.27082E+01_JPRB, 0.27245E+01_JPRB, 0.26154E+01_JPRB, 0.21955E+01_JPRB, 0.19251E+01_JPRB, 0.21606E+01_JPRB, 0.27133E+01_JPRB ,&
&  0.33668E+01_JPRB, 0.38768E+01_JPRB, 0.40121E+01_JPRB, 0.39831E+01_JPRB, 0.38406E+01_JPRB, 0.35094E+01_JPRB, 0.28033E+01_JPRB ,&
&  0.22913E+01_JPRB, 0.18715E+01_JPRB, 0.14772E+01_JPRB, 0.14419E+01_JPRB, 0.13903E+01_JPRB, 0.13236E+01_JPRB, 0.11365E+01_JPRB ,&
&  0.10394E+01_JPRB, 0.99875E+00_JPRB, 0.95884E+00_JPRB, 0.10345E+01_JPRB, 0.11663E+01_JPRB, 0.11619E+01_JPRB, 0.12895E+01_JPRB ,&
&  0.10643E+01_JPRB, 0.42596E+00_JPRB, 0.30052E+00_JPRB, 0.34315E+00_JPRB, 0.22834E+00_JPRB, 0.21836E+00_JPRB, 0.34813E+00_JPRB ,&
&  0.50194E+00_JPRB, 0.55413E+00_JPRB, 0.56117E+00_JPRB, 0.63201E+00_JPRB, 0.83159E+00_JPRB, 0.69439E+00_JPRB, 0.45569E+00_JPRB ,&
&  0.46200E+00_JPRB, 0.50550E+00_JPRB, 0.51168E+00_JPRB, 0.49749E+00_JPRB, 0.49840E+00_JPRB, 0.52163E+00_JPRB, 0.49783E+00_JPRB ,&
&  0.52230E+00_JPRB, 0.53708E+00_JPRB, 0.54102E+00_JPRB, 0.54086E+00_JPRB, 0.53844E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.50721E+00_JPRB, 0.53040E+00_JPRB, 0.54441E+00_JPRB, 0.52040E+00_JPRB, 0.50907E+00_JPRB, 0.52182E+00_JPRB, 0.53057E+00_JPRB ,&
&  0.57113E+00_JPRB, 0.61420E+00_JPRB, 0.67091E+00_JPRB, 0.77012E+00_JPRB, 0.87690E+00_JPRB, 0.94383E+00_JPRB, 0.99944E+00_JPRB ,&
&  0.10811E+01_JPRB, 0.12941E+01_JPRB, 0.17347E+01_JPRB, 0.23416E+01_JPRB, 0.26660E+01_JPRB, 0.24894E+01_JPRB, 0.25432E+01_JPRB ,&
&  0.21681E+01_JPRB, 0.25349E+01_JPRB, 0.32946E+01_JPRB, 0.40625E+01_JPRB, 0.41947E+01_JPRB, 0.39192E+01_JPRB, 0.23389E+01_JPRB ,&
&  0.18166E+01_JPRB, 0.17173E+01_JPRB, 0.15471E+01_JPRB, 0.15745E+01_JPRB, 0.14406E+01_JPRB, 0.14822E+01_JPRB, 0.15013E+01_JPRB ,&
&  0.15496E+01_JPRB, 0.16542E+01_JPRB, 0.17551E+01_JPRB, 0.20008E+01_JPRB, 0.23954E+01_JPRB, 0.27381E+01_JPRB, 0.30556E+01_JPRB ,&
&  0.30000E+01_JPRB, 0.30079E+01_JPRB, 0.29110E+01_JPRB, 0.28445E+01_JPRB, 0.19162E+01_JPRB, 0.25924E+01_JPRB, 0.31034E+01_JPRB ,&
&  0.39347E+01_JPRB, 0.44784E+01_JPRB, 0.43065E+01_JPRB, 0.48414E+01_JPRB, 0.47774E+01_JPRB, 0.40894E+01_JPRB, 0.33134E+01_JPRB ,&
&  0.28069E+01_JPRB, 0.21582E+01_JPRB, 0.17964E+01_JPRB, 0.17565E+01_JPRB, 0.16839E+01_JPRB, 0.15908E+01_JPRB, 0.98859E+00_JPRB ,&
&  0.93029E+00_JPRB, 0.10931E+01_JPRB, 0.10398E+01_JPRB, 0.89248E+00_JPRB, 0.11164E+01_JPRB, 0.12223E+01_JPRB, 0.96927E+00_JPRB ,&
&  0.44747E+00_JPRB, 0.27598E+00_JPRB, 0.23303E+00_JPRB, 0.23253E+00_JPRB, 0.21159E+00_JPRB, 0.23724E+00_JPRB, 0.53522E+00_JPRB ,&
&  0.49829E+00_JPRB, 0.60579E+00_JPRB, 0.56441E+00_JPRB, 0.59307E+00_JPRB, 0.84228E+00_JPRB, 0.91183E+00_JPRB, 0.11545E+01_JPRB ,&
&  0.56373E+00_JPRB, 0.53247E+00_JPRB, 0.54718E+00_JPRB, 0.54366E+00_JPRB, 0.55238E+00_JPRB, 0.53221E+00_JPRB, 0.50620E+00_JPRB ,&
&  0.52135E+00_JPRB, 0.54045E+00_JPRB, 0.51962E+00_JPRB, 0.50096E+00_JPRB, 0.49543E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.49100E+00_JPRB, 0.50457E+00_JPRB, 0.50770E+00_JPRB, 0.49710E+00_JPRB, 0.46485E+00_JPRB, 0.46019E+00_JPRB, 0.47659E+00_JPRB ,&
&  0.49952E+00_JPRB, 0.52589E+00_JPRB, 0.55942E+00_JPRB, 0.60737E+00_JPRB, 0.67879E+00_JPRB, 0.75828E+00_JPRB, 0.84165E+00_JPRB ,&
&  0.94565E+00_JPRB, 0.11542E+01_JPRB, 0.15283E+01_JPRB, 0.20013E+01_JPRB, 0.25977E+01_JPRB, 0.25232E+01_JPRB, 0.28140E+01_JPRB ,&
&  0.24735E+01_JPRB, 0.28143E+01_JPRB, 0.40158E+01_JPRB, 0.55564E+01_JPRB, 0.74054E+01_JPRB, 0.70659E+01_JPRB, 0.50584E+01_JPRB ,&
&  0.33089E+01_JPRB, 0.26088E+01_JPRB, 0.22302E+01_JPRB, 0.18791E+01_JPRB, 0.16384E+01_JPRB, 0.15861E+01_JPRB, 0.16346E+01_JPRB ,&
&  0.16003E+01_JPRB, 0.16223E+01_JPRB, 0.16615E+01_JPRB, 0.18554E+01_JPRB, 0.22344E+01_JPRB, 0.26734E+01_JPRB, 0.31992E+01_JPRB ,&
&  0.34510E+01_JPRB, 0.35796E+01_JPRB, 0.34268E+01_JPRB, 0.32573E+01_JPRB, 0.28734E+01_JPRB, 0.27532E+01_JPRB, 0.32895E+01_JPRB ,&
&  0.43307E+01_JPRB, 0.51993E+01_JPRB, 0.50072E+01_JPRB, 0.58894E+01_JPRB, 0.52786E+01_JPRB, 0.44712E+01_JPRB, 0.37576E+01_JPRB ,&
&  0.31177E+01_JPRB, 0.28530E+01_JPRB, 0.20813E+01_JPRB, 0.19254E+01_JPRB, 0.18959E+01_JPRB, 0.14178E+01_JPRB, 0.10803E+01_JPRB ,&
&  0.12203E+01_JPRB, 0.11458E+01_JPRB, 0.10762E+01_JPRB, 0.88535E+00_JPRB, 0.11042E+01_JPRB, 0.10354E+01_JPRB, 0.56309E+00_JPRB ,&
&  0.40243E+00_JPRB, 0.31160E+00_JPRB, 0.23052E+00_JPRB, 0.23016E+00_JPRB, 0.22184E+00_JPRB, 0.25452E+00_JPRB, 0.48798E+00_JPRB ,&
&  0.57236E+00_JPRB, 0.66661E+00_JPRB, 0.62888E+00_JPRB, 0.58120E+00_JPRB, 0.75966E+00_JPRB, 0.96183E+00_JPRB, 0.22839E+01_JPRB ,&
&  0.15227E+01_JPRB, 0.90975E+00_JPRB, 0.63913E+00_JPRB, 0.60028E+00_JPRB, 0.59565E+00_JPRB, 0.57402E+00_JPRB, 0.54384E+00_JPRB ,&
&  0.53145E+00_JPRB, 0.54436E+00_JPRB, 0.53916E+00_JPRB, 0.52526E+00_JPRB, 0.50861E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.50374E+00_JPRB, 0.50598E+00_JPRB, 0.51465E+00_JPRB, 0.51298E+00_JPRB, 0.49079E+00_JPRB, 0.47546E+00_JPRB, 0.47319E+00_JPRB ,&
&  0.48110E+00_JPRB, 0.48773E+00_JPRB, 0.49849E+00_JPRB, 0.51880E+00_JPRB, 0.55274E+00_JPRB, 0.60702E+00_JPRB, 0.68716E+00_JPRB ,&
&  0.82242E+00_JPRB, 0.10633E+01_JPRB, 0.13443E+01_JPRB, 0.16196E+01_JPRB, 0.20892E+01_JPRB, 0.22439E+01_JPRB, 0.27041E+01_JPRB ,&
&  0.27615E+01_JPRB, 0.30452E+01_JPRB, 0.43495E+01_JPRB, 0.66256E+01_JPRB, 0.10192E+02_JPRB, 0.11217E+02_JPRB, 0.10061E+02_JPRB ,&
&  0.76491E+01_JPRB, 0.50792E+01_JPRB, 0.35464E+01_JPRB, 0.25893E+01_JPRB, 0.20189E+01_JPRB, 0.17248E+01_JPRB, 0.17071E+01_JPRB ,&
&  0.16357E+01_JPRB, 0.16195E+01_JPRB, 0.16493E+01_JPRB, 0.16600E+01_JPRB, 0.18912E+01_JPRB, 0.23529E+01_JPRB, 0.30579E+01_JPRB ,&
&  0.36358E+01_JPRB, 0.40535E+01_JPRB, 0.41138E+01_JPRB, 0.35560E+01_JPRB, 0.40344E+01_JPRB, 0.40503E+01_JPRB, 0.37816E+01_JPRB ,&
&  0.40819E+01_JPRB, 0.45622E+01_JPRB, 0.59137E+01_JPRB, 0.57975E+01_JPRB, 0.49949E+01_JPRB, 0.47275E+01_JPRB, 0.39457E+01_JPRB ,&
&  0.33419E+01_JPRB, 0.29359E+01_JPRB, 0.23092E+01_JPRB, 0.21166E+01_JPRB, 0.18112E+01_JPRB, 0.13445E+01_JPRB, 0.13256E+01_JPRB ,&
&  0.14264E+01_JPRB, 0.13885E+01_JPRB, 0.12070E+01_JPRB, 0.97502E+00_JPRB, 0.93894E+00_JPRB, 0.72134E+00_JPRB, 0.50109E+00_JPRB ,&
&  0.42220E+00_JPRB, 0.35700E+00_JPRB, 0.27095E+00_JPRB, 0.30991E+00_JPRB, 0.28858E+00_JPRB, 0.33665E+00_JPRB, 0.51809E+00_JPRB ,&
&  0.70478E+00_JPRB, 0.71118E+00_JPRB, 0.71242E+00_JPRB, 0.61465E+00_JPRB, 0.64744E+00_JPRB, 0.71840E+00_JPRB, 0.24009E+01_JPRB ,&
&  0.16093E+01_JPRB, 0.13551E+01_JPRB, 0.99780E+00_JPRB, 0.74670E+00_JPRB, 0.64792E+00_JPRB, 0.60311E+00_JPRB, 0.58111E+00_JPRB ,&
&  0.55525E+00_JPRB, 0.53702E+00_JPRB, 0.53066E+00_JPRB, 0.51680E+00_JPRB, 0.50535E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.53775E+00_JPRB, 0.54298E+00_JPRB, 0.55547E+00_JPRB, 0.55830E+00_JPRB, 0.55307E+00_JPRB, 0.53104E+00_JPRB, 0.50319E+00_JPRB ,&
&  0.49505E+00_JPRB, 0.49150E+00_JPRB, 0.48862E+00_JPRB, 0.50018E+00_JPRB, 0.50654E+00_JPRB, 0.52935E+00_JPRB, 0.58301E+00_JPRB ,&
&  0.73906E+00_JPRB, 0.10517E+01_JPRB, 0.10819E+01_JPRB, 0.12169E+01_JPRB, 0.16094E+01_JPRB, 0.21920E+01_JPRB, 0.24659E+01_JPRB ,&
&  0.28758E+01_JPRB, 0.30453E+01_JPRB, 0.40238E+01_JPRB, 0.62785E+01_JPRB, 0.12915E+02_JPRB, 0.14963E+02_JPRB, 0.13922E+02_JPRB ,&
&  0.13153E+02_JPRB, 0.94544E+01_JPRB, 0.59686E+01_JPRB, 0.40782E+01_JPRB, 0.28930E+01_JPRB, 0.20923E+01_JPRB, 0.17238E+01_JPRB ,&
&  0.15889E+01_JPRB, 0.15355E+01_JPRB, 0.15271E+01_JPRB, 0.15538E+01_JPRB, 0.16652E+01_JPRB, 0.20313E+01_JPRB, 0.27700E+01_JPRB ,&
&  0.34863E+01_JPRB, 0.41919E+01_JPRB, 0.46253E+01_JPRB, 0.44861E+01_JPRB, 0.37401E+01_JPRB, 0.40081E+01_JPRB, 0.56025E+01_JPRB ,&
&  0.55260E+01_JPRB, 0.59218E+01_JPRB, 0.59061E+01_JPRB, 0.60021E+01_JPRB, 0.49110E+01_JPRB, 0.42130E+01_JPRB, 0.37615E+01_JPRB ,&
&  0.26897E+01_JPRB, 0.22003E+01_JPRB, 0.20816E+01_JPRB, 0.19376E+01_JPRB, 0.17082E+01_JPRB, 0.17366E+01_JPRB, 0.21307E+01_JPRB ,&
&  0.17026E+01_JPRB, 0.17320E+01_JPRB, 0.17427E+01_JPRB, 0.15892E+01_JPRB, 0.10090E+01_JPRB, 0.73003E+00_JPRB, 0.92287E+00_JPRB ,&
&  0.88090E+00_JPRB, 0.53773E+00_JPRB, 0.43625E+00_JPRB, 0.55117E+00_JPRB, 0.41582E+00_JPRB, 0.47879E+00_JPRB, 0.65610E+00_JPRB ,&
&  0.69925E+00_JPRB, 0.69757E+00_JPRB, 0.79818E+00_JPRB, 0.70415E+00_JPRB, 0.69062E+00_JPRB, 0.62314E+00_JPRB, 0.13112E+01_JPRB ,&
&  0.16154E+01_JPRB, 0.14757E+01_JPRB, 0.13201E+01_JPRB, 0.10218E+01_JPRB, 0.81222E+00_JPRB, 0.71297E+00_JPRB, 0.67139E+00_JPRB ,&
&  0.64323E+00_JPRB, 0.61513E+00_JPRB, 0.58870E+00_JPRB, 0.55295E+00_JPRB, 0.54212E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.57533E+00_JPRB, 0.57533E+00_JPRB, 0.58286E+00_JPRB, 0.59312E+00_JPRB, 0.59509E+00_JPRB, 0.56891E+00_JPRB, 0.53428E+00_JPRB ,&
&  0.51069E+00_JPRB, 0.49988E+00_JPRB, 0.49695E+00_JPRB, 0.49594E+00_JPRB, 0.48364E+00_JPRB, 0.49060E+00_JPRB, 0.51891E+00_JPRB ,&
&  0.65327E+00_JPRB, 0.87952E+00_JPRB, 0.94836E+00_JPRB, 0.10578E+01_JPRB, 0.14959E+01_JPRB, 0.21508E+01_JPRB, 0.25843E+01_JPRB ,&
&  0.31605E+01_JPRB, 0.31284E+01_JPRB, 0.37707E+01_JPRB, 0.59856E+01_JPRB, 0.10578E+02_JPRB, 0.13698E+02_JPRB, 0.13339E+02_JPRB ,&
&  0.12880E+02_JPRB, 0.10547E+02_JPRB, 0.79347E+01_JPRB, 0.59117E+01_JPRB, 0.42505E+01_JPRB, 0.30623E+01_JPRB, 0.22341E+01_JPRB ,&
&  0.17952E+01_JPRB, 0.15882E+01_JPRB, 0.15214E+01_JPRB, 0.14883E+01_JPRB, 0.15598E+01_JPRB, 0.17264E+01_JPRB, 0.23099E+01_JPRB ,&
&  0.29550E+01_JPRB, 0.38788E+01_JPRB, 0.49331E+01_JPRB, 0.53974E+01_JPRB, 0.45123E+01_JPRB, 0.47915E+01_JPRB, 0.61705E+01_JPRB ,&
&  0.62188E+01_JPRB, 0.60064E+01_JPRB, 0.58498E+01_JPRB, 0.54262E+01_JPRB, 0.50742E+01_JPRB, 0.43959E+01_JPRB, 0.42290E+01_JPRB ,&
&  0.32323E+01_JPRB, 0.25675E+01_JPRB, 0.21621E+01_JPRB, 0.18538E+01_JPRB, 0.19171E+01_JPRB, 0.24007E+01_JPRB, 0.24530E+01_JPRB ,&
&  0.21949E+01_JPRB, 0.21159E+01_JPRB, 0.19609E+01_JPRB, 0.15946E+01_JPRB, 0.13082E+01_JPRB, 0.10987E+01_JPRB, 0.12556E+01_JPRB ,&
&  0.12345E+01_JPRB, 0.11369E+01_JPRB, 0.95929E+00_JPRB, 0.77546E+00_JPRB, 0.63039E+00_JPRB, 0.72862E+00_JPRB, 0.78177E+00_JPRB ,&
&  0.75329E+00_JPRB, 0.69256E+00_JPRB, 0.73070E+00_JPRB, 0.70912E+00_JPRB, 0.68210E+00_JPRB, 0.55449E+00_JPRB, 0.89719E+00_JPRB ,&
&  0.10912E+01_JPRB, 0.13263E+01_JPRB, 0.13758E+01_JPRB, 0.11953E+01_JPRB, 0.10330E+01_JPRB, 0.89382E+00_JPRB, 0.81517E+00_JPRB ,&
&  0.77478E+00_JPRB, 0.73632E+00_JPRB, 0.68212E+00_JPRB, 0.62759E+00_JPRB, 0.59724E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.57775E+00_JPRB, 0.57462E+00_JPRB, 0.57237E+00_JPRB, 0.56988E+00_JPRB, 0.56037E+00_JPRB, 0.55095E+00_JPRB, 0.53352E+00_JPRB ,&
&  0.51694E+00_JPRB, 0.50314E+00_JPRB, 0.49754E+00_JPRB, 0.49026E+00_JPRB, 0.48630E+00_JPRB, 0.48031E+00_JPRB, 0.48425E+00_JPRB ,&
&  0.56113E+00_JPRB, 0.75454E+00_JPRB, 0.80913E+00_JPRB, 0.96588E+00_JPRB, 0.13636E+01_JPRB, 0.20052E+01_JPRB, 0.28562E+01_JPRB ,&
&  0.33162E+01_JPRB, 0.34689E+01_JPRB, 0.39591E+01_JPRB, 0.56193E+01_JPRB, 0.77878E+01_JPRB, 0.91810E+01_JPRB, 0.93655E+01_JPRB ,&
&  0.80571E+01_JPRB, 0.71139E+01_JPRB, 0.60598E+01_JPRB, 0.51367E+01_JPRB, 0.44401E+01_JPRB, 0.38063E+01_JPRB, 0.30635E+01_JPRB ,&
&  0.24024E+01_JPRB, 0.19704E+01_JPRB, 0.17548E+01_JPRB, 0.16481E+01_JPRB, 0.16131E+01_JPRB, 0.16680E+01_JPRB, 0.19465E+01_JPRB ,&
&  0.24119E+01_JPRB, 0.32015E+01_JPRB, 0.46451E+01_JPRB, 0.62003E+01_JPRB, 0.58737E+01_JPRB, 0.67017E+01_JPRB, 0.65502E+01_JPRB ,&
&  0.70785E+01_JPRB, 0.69079E+01_JPRB, 0.58907E+01_JPRB, 0.55559E+01_JPRB, 0.43743E+01_JPRB, 0.44482E+01_JPRB, 0.46144E+01_JPRB ,&
&  0.45748E+01_JPRB, 0.41448E+01_JPRB, 0.37699E+01_JPRB, 0.29248E+01_JPRB, 0.24116E+01_JPRB, 0.29327E+01_JPRB, 0.27708E+01_JPRB ,&
&  0.24194E+01_JPRB, 0.22597E+01_JPRB, 0.20435E+01_JPRB, 0.18859E+01_JPRB, 0.15354E+01_JPRB, 0.14082E+01_JPRB, 0.12125E+01_JPRB ,&
&  0.11177E+01_JPRB, 0.96054E+00_JPRB, 0.10355E+01_JPRB, 0.91632E+00_JPRB, 0.86466E+00_JPRB, 0.86494E+00_JPRB, 0.85753E+00_JPRB ,&
&  0.80661E+00_JPRB, 0.69714E+00_JPRB, 0.62689E+00_JPRB, 0.67447E+00_JPRB, 0.71137E+00_JPRB, 0.60469E+00_JPRB, 0.68618E+00_JPRB ,&
&  0.88079E+00_JPRB, 0.10236E+01_JPRB, 0.10172E+01_JPRB, 0.10835E+01_JPRB, 0.10679E+01_JPRB, 0.10241E+01_JPRB, 0.93940E+00_JPRB ,&
&  0.82939E+00_JPRB, 0.73131E+00_JPRB, 0.66936E+00_JPRB, 0.61733E+00_JPRB, 0.58384E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.55000E+00_JPRB, 0.55297E+00_JPRB, 0.54350E+00_JPRB, 0.52797E+00_JPRB, 0.52057E+00_JPRB, 0.51370E+00_JPRB, 0.50609E+00_JPRB ,&
&  0.49293E+00_JPRB, 0.48525E+00_JPRB, 0.48303E+00_JPRB, 0.48392E+00_JPRB, 0.48398E+00_JPRB, 0.48231E+00_JPRB, 0.47710E+00_JPRB ,&
&  0.49028E+00_JPRB, 0.67411E+00_JPRB, 0.69810E+00_JPRB, 0.75231E+00_JPRB, 0.10610E+01_JPRB, 0.18587E+01_JPRB, 0.27268E+01_JPRB ,&
&  0.32156E+01_JPRB, 0.34632E+01_JPRB, 0.39257E+01_JPRB, 0.45695E+01_JPRB, 0.53112E+01_JPRB, 0.58195E+01_JPRB, 0.59682E+01_JPRB ,&
&  0.57518E+01_JPRB, 0.44140E+01_JPRB, 0.39693E+01_JPRB, 0.35652E+01_JPRB, 0.33385E+01_JPRB, 0.31072E+01_JPRB, 0.28758E+01_JPRB ,&
&  0.25744E+01_JPRB, 0.22085E+01_JPRB, 0.19287E+01_JPRB, 0.17478E+01_JPRB, 0.16496E+01_JPRB, 0.15622E+01_JPRB, 0.16565E+01_JPRB ,&
&  0.18700E+01_JPRB, 0.21421E+01_JPRB, 0.30682E+01_JPRB, 0.45210E+01_JPRB, 0.66545E+01_JPRB, 0.79013E+01_JPRB, 0.77431E+01_JPRB ,&
&  0.62768E+01_JPRB, 0.56924E+01_JPRB, 0.67970E+01_JPRB, 0.57022E+01_JPRB, 0.52337E+01_JPRB, 0.51888E+01_JPRB, 0.48729E+01_JPRB ,&
&  0.47009E+01_JPRB, 0.43328E+01_JPRB, 0.38063E+01_JPRB, 0.31969E+01_JPRB, 0.30804E+01_JPRB, 0.30787E+01_JPRB, 0.28013E+01_JPRB ,&
&  0.24088E+01_JPRB, 0.22862E+01_JPRB, 0.20968E+01_JPRB, 0.18912E+01_JPRB, 0.17409E+01_JPRB, 0.16948E+01_JPRB, 0.15118E+01_JPRB ,&
&  0.13668E+01_JPRB, 0.14334E+01_JPRB, 0.11870E+01_JPRB, 0.96185E+00_JPRB, 0.83859E+00_JPRB, 0.86905E+00_JPRB, 0.88876E+00_JPRB ,&
&  0.81029E+00_JPRB, 0.67707E+00_JPRB, 0.54822E+00_JPRB, 0.54042E+00_JPRB, 0.57489E+00_JPRB, 0.58194E+00_JPRB, 0.57561E+00_JPRB ,&
&  0.65656E+00_JPRB, 0.81234E+00_JPRB, 0.82721E+00_JPRB, 0.85645E+00_JPRB, 0.97399E+00_JPRB, 0.92910E+00_JPRB, 0.82482E+00_JPRB ,&
&  0.72744E+00_JPRB, 0.65976E+00_JPRB, 0.61519E+00_JPRB, 0.58256E+00_JPRB, 0.55508E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.53358E+00_JPRB, 0.53134E+00_JPRB, 0.52126E+00_JPRB, 0.50604E+00_JPRB, 0.49944E+00_JPRB, 0.49993E+00_JPRB, 0.49339E+00_JPRB ,&
&  0.48850E+00_JPRB, 0.48451E+00_JPRB, 0.48963E+00_JPRB, 0.48717E+00_JPRB, 0.49089E+00_JPRB, 0.50010E+00_JPRB, 0.50452E+00_JPRB ,&
&  0.50285E+00_JPRB, 0.63224E+00_JPRB, 0.60521E+00_JPRB, 0.66159E+00_JPRB, 0.84368E+00_JPRB, 0.14420E+01_JPRB, 0.20834E+01_JPRB ,&
&  0.25739E+01_JPRB, 0.27796E+01_JPRB, 0.30030E+01_JPRB, 0.33630E+01_JPRB, 0.35908E+01_JPRB, 0.35491E+01_JPRB, 0.35014E+01_JPRB ,&
&  0.33264E+01_JPRB, 0.30420E+01_JPRB, 0.25371E+01_JPRB, 0.24870E+01_JPRB, 0.22808E+01_JPRB, 0.19457E+01_JPRB, 0.20621E+01_JPRB ,&
&  0.20887E+01_JPRB, 0.20449E+01_JPRB, 0.19339E+01_JPRB, 0.18332E+01_JPRB, 0.17665E+01_JPRB, 0.16694E+01_JPRB, 0.16290E+01_JPRB ,&
&  0.16590E+01_JPRB, 0.17291E+01_JPRB, 0.19492E+01_JPRB, 0.25439E+01_JPRB, 0.38597E+01_JPRB, 0.54577E+01_JPRB, 0.66913E+01_JPRB ,&
&  0.76747E+01_JPRB, 0.75887E+01_JPRB, 0.60616E+01_JPRB, 0.58853E+01_JPRB, 0.61561E+01_JPRB, 0.53545E+01_JPRB, 0.47623E+01_JPRB ,&
&  0.46757E+01_JPRB, 0.45208E+01_JPRB, 0.39402E+01_JPRB, 0.34779E+01_JPRB, 0.32019E+01_JPRB, 0.30816E+01_JPRB, 0.28407E+01_JPRB ,&
&  0.24009E+01_JPRB, 0.22184E+01_JPRB, 0.21094E+01_JPRB, 0.19415E+01_JPRB, 0.17385E+01_JPRB, 0.15952E+01_JPRB, 0.15114E+01_JPRB ,&
&  0.14075E+01_JPRB, 0.13803E+01_JPRB, 0.11414E+01_JPRB, 0.10717E+01_JPRB, 0.86998E+00_JPRB, 0.85690E+00_JPRB, 0.89853E+00_JPRB ,&
&  0.81205E+00_JPRB, 0.65262E+00_JPRB, 0.50081E+00_JPRB, 0.44615E+00_JPRB, 0.43556E+00_JPRB, 0.45115E+00_JPRB, 0.49288E+00_JPRB ,&
&  0.54952E+00_JPRB, 0.55945E+00_JPRB, 0.67496E+00_JPRB, 0.71075E+00_JPRB, 0.76957E+00_JPRB, 0.75888E+00_JPRB, 0.71335E+00_JPRB ,&
&  0.67349E+00_JPRB, 0.64242E+00_JPRB, 0.61097E+00_JPRB, 0.57749E+00_JPRB, 0.54995E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.51757E+00_JPRB, 0.49925E+00_JPRB, 0.49120E+00_JPRB, 0.48078E+00_JPRB, 0.47667E+00_JPRB, 0.47730E+00_JPRB, 0.48023E+00_JPRB ,&
&  0.48755E+00_JPRB, 0.49234E+00_JPRB, 0.50017E+00_JPRB, 0.49833E+00_JPRB, 0.49927E+00_JPRB, 0.49894E+00_JPRB, 0.52290E+00_JPRB ,&
&  0.53585E+00_JPRB, 0.53501E+00_JPRB, 0.56061E+00_JPRB, 0.60500E+00_JPRB, 0.85554E+00_JPRB, 0.11003E+01_JPRB, 0.14190E+01_JPRB ,&
&  0.16574E+01_JPRB, 0.18537E+01_JPRB, 0.19632E+01_JPRB, 0.20337E+01_JPRB, 0.20598E+01_JPRB, 0.20531E+01_JPRB, 0.19475E+01_JPRB ,&
&  0.17731E+01_JPRB, 0.15719E+01_JPRB, 0.14700E+01_JPRB, 0.14170E+01_JPRB, 0.14350E+01_JPRB, 0.12737E+01_JPRB, 0.13695E+01_JPRB ,&
&  0.14667E+01_JPRB, 0.15583E+01_JPRB, 0.16030E+01_JPRB, 0.16278E+01_JPRB, 0.16325E+01_JPRB, 0.16451E+01_JPRB, 0.16636E+01_JPRB ,&
&  0.17126E+01_JPRB, 0.17191E+01_JPRB, 0.17416E+01_JPRB, 0.19935E+01_JPRB, 0.27216E+01_JPRB, 0.43214E+01_JPRB, 0.58332E+01_JPRB ,&
&  0.56981E+01_JPRB, 0.64679E+01_JPRB, 0.72522E+01_JPRB, 0.73120E+01_JPRB, 0.61360E+01_JPRB, 0.50554E+01_JPRB, 0.45855E+01_JPRB ,&
&  0.45930E+01_JPRB, 0.47098E+01_JPRB, 0.42658E+01_JPRB, 0.35879E+01_JPRB, 0.32189E+01_JPRB, 0.30488E+01_JPRB, 0.28793E+01_JPRB ,&
&  0.26203E+01_JPRB, 0.23566E+01_JPRB, 0.21406E+01_JPRB, 0.19930E+01_JPRB, 0.18838E+01_JPRB, 0.16573E+01_JPRB, 0.14922E+01_JPRB ,&
&  0.13700E+01_JPRB, 0.12321E+01_JPRB, 0.11457E+01_JPRB, 0.11105E+01_JPRB, 0.97165E+00_JPRB, 0.95295E+00_JPRB, 0.92475E+00_JPRB ,&
&  0.83381E+00_JPRB, 0.63892E+00_JPRB, 0.52067E+00_JPRB, 0.46067E+00_JPRB, 0.43718E+00_JPRB, 0.42713E+00_JPRB, 0.43872E+00_JPRB ,&
&  0.47089E+00_JPRB, 0.49147E+00_JPRB, 0.59503E+00_JPRB, 0.64207E+00_JPRB, 0.66361E+00_JPRB, 0.65303E+00_JPRB, 0.63326E+00_JPRB ,&
&  0.60186E+00_JPRB, 0.57643E+00_JPRB, 0.56448E+00_JPRB, 0.55344E+00_JPRB, 0.53981E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.50400E+00_JPRB, 0.49390E+00_JPRB, 0.48773E+00_JPRB, 0.48219E+00_JPRB, 0.47839E+00_JPRB, 0.47523E+00_JPRB, 0.47505E+00_JPRB ,&
&  0.47602E+00_JPRB, 0.48760E+00_JPRB, 0.50448E+00_JPRB, 0.50466E+00_JPRB, 0.50269E+00_JPRB, 0.50712E+00_JPRB, 0.55911E+00_JPRB ,&
&  0.44196E+00_JPRB, 0.45738E+00_JPRB, 0.48349E+00_JPRB, 0.57880E+00_JPRB, 0.82372E+00_JPRB, 0.87830E+00_JPRB, 0.10160E+01_JPRB ,&
&  0.11212E+01_JPRB, 0.11770E+01_JPRB, 0.12150E+01_JPRB, 0.12561E+01_JPRB, 0.12555E+01_JPRB, 0.12504E+01_JPRB, 0.11998E+01_JPRB ,&
&  0.10449E+01_JPRB, 0.93052E+00_JPRB, 0.88009E+00_JPRB, 0.84586E+00_JPRB, 0.88313E+00_JPRB, 0.94137E+00_JPRB, 0.10391E+01_JPRB ,&
&  0.11241E+01_JPRB, 0.12076E+01_JPRB, 0.12737E+01_JPRB, 0.13225E+01_JPRB, 0.13558E+01_JPRB, 0.13928E+01_JPRB, 0.14390E+01_JPRB ,&
&  0.14893E+01_JPRB, 0.15304E+01_JPRB, 0.16002E+01_JPRB, 0.18414E+01_JPRB, 0.23543E+01_JPRB, 0.41431E+01_JPRB, 0.56021E+01_JPRB ,&
&  0.44053E+01_JPRB, 0.40230E+01_JPRB, 0.50797E+01_JPRB, 0.60993E+01_JPRB, 0.53373E+01_JPRB, 0.44348E+01_JPRB, 0.38252E+01_JPRB ,&
&  0.37165E+01_JPRB, 0.37716E+01_JPRB, 0.35817E+01_JPRB, 0.32167E+01_JPRB, 0.27635E+01_JPRB, 0.25556E+01_JPRB, 0.24797E+01_JPRB ,&
&  0.23290E+01_JPRB, 0.21638E+01_JPRB, 0.20224E+01_JPRB, 0.18727E+01_JPRB, 0.18129E+01_JPRB, 0.17631E+01_JPRB, 0.16887E+01_JPRB ,&
&  0.15533E+01_JPRB, 0.13954E+01_JPRB, 0.12476E+01_JPRB, 0.11373E+01_JPRB, 0.10407E+01_JPRB, 0.95732E+00_JPRB, 0.87106E+00_JPRB ,&
&  0.84755E+00_JPRB, 0.68215E+00_JPRB, 0.61780E+00_JPRB, 0.56343E+00_JPRB, 0.53596E+00_JPRB, 0.46965E+00_JPRB, 0.43868E+00_JPRB ,&
&  0.44934E+00_JPRB, 0.48090E+00_JPRB, 0.53914E+00_JPRB, 0.56491E+00_JPRB, 0.57124E+00_JPRB, 0.57903E+00_JPRB, 0.51762E+00_JPRB ,&
&  0.54402E+00_JPRB, 0.52094E+00_JPRB, 0.50950E+00_JPRB, 0.51068E+00_JPRB, 0.50820E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.48740E+00_JPRB, 0.48446E+00_JPRB, 0.48714E+00_JPRB, 0.48035E+00_JPRB, 0.47167E+00_JPRB, 0.47283E+00_JPRB, 0.48021E+00_JPRB ,&
&  0.48356E+00_JPRB, 0.49387E+00_JPRB, 0.50282E+00_JPRB, 0.50403E+00_JPRB, 0.51015E+00_JPRB, 0.54219E+00_JPRB, 0.49976E+00_JPRB ,&
&  0.42166E+00_JPRB, 0.43013E+00_JPRB, 0.43560E+00_JPRB, 0.48287E+00_JPRB, 0.54807E+00_JPRB, 0.58978E+00_JPRB, 0.64366E+00_JPRB ,&
&  0.72304E+00_JPRB, 0.75161E+00_JPRB, 0.79082E+00_JPRB, 0.82374E+00_JPRB, 0.83565E+00_JPRB, 0.83645E+00_JPRB, 0.82362E+00_JPRB ,&
&  0.74142E+00_JPRB, 0.68696E+00_JPRB, 0.65056E+00_JPRB, 0.63150E+00_JPRB, 0.66895E+00_JPRB, 0.73446E+00_JPRB, 0.80631E+00_JPRB ,&
&  0.87864E+00_JPRB, 0.94312E+00_JPRB, 0.10081E+01_JPRB, 0.10592E+01_JPRB, 0.10902E+01_JPRB, 0.11165E+01_JPRB, 0.11540E+01_JPRB ,&
&  0.12033E+01_JPRB, 0.12536E+01_JPRB, 0.12996E+01_JPRB, 0.15001E+01_JPRB, 0.17264E+01_JPRB, 0.26493E+01_JPRB, 0.33649E+01_JPRB ,&
&  0.28906E+01_JPRB, 0.26704E+01_JPRB, 0.28513E+01_JPRB, 0.31499E+01_JPRB, 0.32082E+01_JPRB, 0.29654E+01_JPRB, 0.27560E+01_JPRB ,&
&  0.27022E+01_JPRB, 0.27287E+01_JPRB, 0.27397E+01_JPRB, 0.25740E+01_JPRB, 0.23482E+01_JPRB, 0.22012E+01_JPRB, 0.20782E+01_JPRB ,&
&  0.19611E+01_JPRB, 0.18065E+01_JPRB, 0.18123E+01_JPRB, 0.17682E+01_JPRB, 0.17123E+01_JPRB, 0.16637E+01_JPRB, 0.16633E+01_JPRB ,&
&  0.16320E+01_JPRB, 0.15349E+01_JPRB, 0.13826E+01_JPRB, 0.12607E+01_JPRB, 0.11627E+01_JPRB, 0.11317E+01_JPRB, 0.10586E+01_JPRB ,&
&  0.94780E+00_JPRB, 0.76599E+00_JPRB, 0.73466E+00_JPRB, 0.65935E+00_JPRB, 0.59338E+00_JPRB, 0.52958E+00_JPRB, 0.48327E+00_JPRB ,&
&  0.47495E+00_JPRB, 0.51328E+00_JPRB, 0.52337E+00_JPRB, 0.52747E+00_JPRB, 0.53452E+00_JPRB, 0.53927E+00_JPRB, 0.47223E+00_JPRB ,&
&  0.47229E+00_JPRB, 0.50219E+00_JPRB, 0.49254E+00_JPRB, 0.49165E+00_JPRB, 0.48915E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.47511E+00_JPRB, 0.47977E+00_JPRB, 0.48433E+00_JPRB, 0.49120E+00_JPRB, 0.47811E+00_JPRB, 0.50032E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.51404E+00_JPRB, 0.52000E+00_JPRB, 0.50732E+00_JPRB, 0.52819E+00_JPRB, 0.57793E+00_JPRB, 0.54626E+00_JPRB, 0.41880E+00_JPRB ,&
&  0.37214E+00_JPRB, 0.37539E+00_JPRB, 0.39468E+00_JPRB, 0.41163E+00_JPRB, 0.42860E+00_JPRB, 0.44581E+00_JPRB, 0.47486E+00_JPRB ,&
&  0.51239E+00_JPRB, 0.53621E+00_JPRB, 0.57520E+00_JPRB, 0.60089E+00_JPRB, 0.61173E+00_JPRB, 0.61344E+00_JPRB, 0.62658E+00_JPRB ,&
&  0.59061E+00_JPRB, 0.56662E+00_JPRB, 0.55166E+00_JPRB, 0.51569E+00_JPRB, 0.55566E+00_JPRB, 0.59260E+00_JPRB, 0.64151E+00_JPRB ,&
&  0.69844E+00_JPRB, 0.76115E+00_JPRB, 0.80997E+00_JPRB, 0.85619E+00_JPRB, 0.89082E+00_JPRB, 0.91769E+00_JPRB, 0.94492E+00_JPRB ,&
&  0.98615E+00_JPRB, 0.10310E+01_JPRB, 0.10757E+01_JPRB, 0.12135E+01_JPRB, 0.14123E+01_JPRB, 0.16823E+01_JPRB, 0.20415E+01_JPRB ,&
&  0.21289E+01_JPRB, 0.23442E+01_JPRB, 0.25140E+01_JPRB, 0.21922E+01_JPRB, 0.23213E+01_JPRB, 0.25233E+01_JPRB, 0.23651E+01_JPRB ,&
&  0.22019E+01_JPRB, 0.19662E+01_JPRB, 0.18391E+01_JPRB, 0.17726E+01_JPRB, 0.18006E+01_JPRB, 0.18173E+01_JPRB, 0.17882E+01_JPRB ,&
&  0.16804E+01_JPRB, 0.15618E+01_JPRB, 0.16039E+01_JPRB, 0.16058E+01_JPRB, 0.15819E+01_JPRB, 0.15571E+01_JPRB, 0.15872E+01_JPRB ,&
&  0.16082E+01_JPRB, 0.15802E+01_JPRB, 0.15226E+01_JPRB, 0.14403E+01_JPRB, 0.13625E+01_JPRB, 0.12762E+01_JPRB, 0.11617E+01_JPRB ,&
&  0.10528E+01_JPRB, 0.93573E+00_JPRB, 0.83333E+00_JPRB, 0.76070E+00_JPRB, 0.68388E+00_JPRB, 0.61391E+00_JPRB, 0.55491E+00_JPRB ,&
&  0.49935E+00_JPRB, 0.48987E+00_JPRB, 0.52320E+00_JPRB, 0.51568E+00_JPRB, 0.51649E+00_JPRB, 0.51409E+00_JPRB, 0.47836E+00_JPRB ,&
&  0.46702E+00_JPRB, 0.48464E+00_JPRB, 0.48423E+00_JPRB, 0.47880E+00_JPRB, 0.47499E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.48120E+00_JPRB, 0.47401E+00_JPRB, 0.47118E+00_JPRB, 0.48823E+00_JPRB, 0.49217E+00_JPRB, 0.46765E+00_JPRB, 0.43698E+00_JPRB ,&
&  0.54790E+00_JPRB, 0.50463E+00_JPRB, 0.56891E+00_JPRB, 0.43409E+00_JPRB, 0.37939E+00_JPRB, 0.36997E+00_JPRB, 0.35581E+00_JPRB ,&
&  0.34788E+00_JPRB, 0.34957E+00_JPRB, 0.35815E+00_JPRB, 0.37304E+00_JPRB, 0.38983E+00_JPRB, 0.39859E+00_JPRB, 0.40848E+00_JPRB ,&
&  0.42558E+00_JPRB, 0.44503E+00_JPRB, 0.47792E+00_JPRB, 0.50632E+00_JPRB, 0.51985E+00_JPRB, 0.52239E+00_JPRB, 0.53120E+00_JPRB ,&
&  0.50278E+00_JPRB, 0.48852E+00_JPRB, 0.49006E+00_JPRB, 0.48526E+00_JPRB, 0.49221E+00_JPRB, 0.50857E+00_JPRB, 0.53752E+00_JPRB ,&
&  0.59786E+00_JPRB, 0.56088E+00_JPRB, 0.67047E+00_JPRB, 0.70215E+00_JPRB, 0.73551E+00_JPRB, 0.76261E+00_JPRB, 0.78513E+00_JPRB ,&
&  0.81238E+00_JPRB, 0.85803E+00_JPRB, 0.90652E+00_JPRB, 0.97803E+00_JPRB, 0.11687E+01_JPRB, 0.12931E+01_JPRB, 0.13699E+01_JPRB ,&
&  0.16184E+01_JPRB, 0.14466E+01_JPRB, 0.17003E+01_JPRB, 0.17670E+01_JPRB, 0.20340E+01_JPRB, 0.22214E+01_JPRB, 0.21020E+01_JPRB ,&
&  0.21346E+01_JPRB, 0.18319E+01_JPRB, 0.15744E+01_JPRB, 0.13950E+01_JPRB, 0.13411E+01_JPRB, 0.13361E+01_JPRB, 0.13695E+01_JPRB ,&
&  0.13432E+01_JPRB, 0.12399E+01_JPRB, 0.12286E+01_JPRB, 0.12514E+01_JPRB, 0.12804E+01_JPRB, 0.13351E+01_JPRB, 0.14067E+01_JPRB ,&
&  0.14652E+01_JPRB, 0.14876E+01_JPRB, 0.15026E+01_JPRB, 0.14295E+01_JPRB, 0.13836E+01_JPRB, 0.13578E+01_JPRB, 0.11992E+01_JPRB ,&
&  0.10738E+01_JPRB, 0.10018E+01_JPRB, 0.94784E+00_JPRB, 0.90769E+00_JPRB, 0.82756E+00_JPRB, 0.74598E+00_JPRB, 0.65138E+00_JPRB ,&
&  0.53051E+00_JPRB, 0.46723E+00_JPRB, 0.45829E+00_JPRB, 0.44989E+00_JPRB, 0.44691E+00_JPRB, 0.45206E+00_JPRB, 0.45208E+00_JPRB ,&
&  0.45379E+00_JPRB, 0.45742E+00_JPRB, 0.47270E+00_JPRB, 0.48256E+00_JPRB, 0.48201E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.51360E+00_JPRB, 0.50650E+00_JPRB, 0.49243E+00_JPRB, 0.48950E+00_JPRB, 0.49808E+00_JPRB, 0.47327E+00_JPRB, 0.42180E+00_JPRB ,&
&  0.44051E+00_JPRB, 0.40076E+00_JPRB, 0.38542E+00_JPRB, 0.37233E+00_JPRB, 0.36309E+00_JPRB, 0.34496E+00_JPRB, 0.32534E+00_JPRB ,&
&  0.32696E+00_JPRB, 0.34527E+00_JPRB, 0.37416E+00_JPRB, 0.39495E+00_JPRB, 0.40414E+00_JPRB, 0.40491E+00_JPRB, 0.40256E+00_JPRB ,&
&  0.40786E+00_JPRB, 0.41958E+00_JPRB, 0.43895E+00_JPRB, 0.46898E+00_JPRB, 0.46772E+00_JPRB, 0.46056E+00_JPRB, 0.45599E+00_JPRB ,&
&  0.45064E+00_JPRB, 0.44101E+00_JPRB, 0.42499E+00_JPRB, 0.42995E+00_JPRB, 0.43787E+00_JPRB, 0.45976E+00_JPRB, 0.49577E+00_JPRB ,&
&  0.41539E+00_JPRB, 0.42497E+00_JPRB, 0.54428E+00_JPRB, 0.57468E+00_JPRB, 0.60770E+00_JPRB, 0.64008E+00_JPRB, 0.66725E+00_JPRB ,&
&  0.69604E+00_JPRB, 0.74133E+00_JPRB, 0.79354E+00_JPRB, 0.85327E+00_JPRB, 0.10073E+01_JPRB, 0.11386E+01_JPRB, 0.12050E+01_JPRB ,&
&  0.13424E+01_JPRB, 0.12509E+01_JPRB, 0.12159E+01_JPRB, 0.13288E+01_JPRB, 0.18002E+01_JPRB, 0.17845E+01_JPRB, 0.16783E+01_JPRB ,&
&  0.15713E+01_JPRB, 0.14003E+01_JPRB, 0.12660E+01_JPRB, 0.11926E+01_JPRB, 0.11323E+01_JPRB, 0.11056E+01_JPRB, 0.11051E+01_JPRB ,&
&  0.10381E+01_JPRB, 0.99894E+00_JPRB, 0.98504E+00_JPRB, 0.99830E+00_JPRB, 0.10268E+01_JPRB, 0.10922E+01_JPRB, 0.11911E+01_JPRB ,&
&  0.12980E+01_JPRB, 0.13825E+01_JPRB, 0.14085E+01_JPRB, 0.14088E+01_JPRB, 0.13981E+01_JPRB, 0.13751E+01_JPRB, 0.12870E+01_JPRB ,&
&  0.11261E+01_JPRB, 0.10182E+01_JPRB, 0.96228E+00_JPRB, 0.92066E+00_JPRB, 0.89035E+00_JPRB, 0.81306E+00_JPRB, 0.67653E+00_JPRB ,&
&  0.52792E+00_JPRB, 0.43899E+00_JPRB, 0.42047E+00_JPRB, 0.41452E+00_JPRB, 0.41445E+00_JPRB, 0.41601E+00_JPRB, 0.42271E+00_JPRB ,&
&  0.42520E+00_JPRB, 0.42251E+00_JPRB, 0.42744E+00_JPRB, 0.44541E+00_JPRB, 0.47185E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.52146E+00_JPRB, 0.52027E+00_JPRB, 0.48916E+00_JPRB, 0.50068E+00_JPRB, 0.47782E+00_JPRB, 0.46222E+00_JPRB, 0.43572E+00_JPRB ,&
&  0.42446E+00_JPRB, 0.40179E+00_JPRB, 0.38652E+00_JPRB, 0.36130E+00_JPRB, 0.34634E+00_JPRB, 0.34079E+00_JPRB, 0.35540E+00_JPRB ,&
&  0.37649E+00_JPRB, 0.39516E+00_JPRB, 0.40659E+00_JPRB, 0.41019E+00_JPRB, 0.40917E+00_JPRB, 0.40194E+00_JPRB, 0.39496E+00_JPRB ,&
&  0.39385E+00_JPRB, 0.40170E+00_JPRB, 0.40257E+00_JPRB, 0.41095E+00_JPRB, 0.41382E+00_JPRB, 0.40753E+00_JPRB, 0.40410E+00_JPRB ,&
&  0.39059E+00_JPRB, 0.37593E+00_JPRB, 0.37308E+00_JPRB, 0.39153E+00_JPRB, 0.41267E+00_JPRB, 0.43521E+00_JPRB, 0.40691E+00_JPRB ,&
&  0.36586E+00_JPRB, 0.35825E+00_JPRB, 0.37523E+00_JPRB, 0.50247E+00_JPRB, 0.54741E+00_JPRB, 0.56830E+00_JPRB, 0.60864E+00_JPRB ,&
&  0.59097E+00_JPRB, 0.59663E+00_JPRB, 0.63652E+00_JPRB, 0.74900E+00_JPRB, 0.82881E+00_JPRB, 0.96719E+00_JPRB, 0.10498E+01_JPRB ,&
&  0.11390E+01_JPRB, 0.12515E+01_JPRB, 0.13676E+01_JPRB, 0.11765E+01_JPRB, 0.12092E+01_JPRB, 0.14199E+01_JPRB, 0.12263E+01_JPRB ,&
&  0.11089E+01_JPRB, 0.10286E+01_JPRB, 0.10477E+01_JPRB, 0.10033E+01_JPRB, 0.93883E+00_JPRB, 0.89858E+00_JPRB, 0.84613E+00_JPRB ,&
&  0.76608E+00_JPRB, 0.73952E+00_JPRB, 0.76162E+00_JPRB, 0.79700E+00_JPRB, 0.83466E+00_JPRB, 0.89966E+00_JPRB, 0.97399E+00_JPRB ,&
&  0.10726E+01_JPRB, 0.11695E+01_JPRB, 0.12238E+01_JPRB, 0.12413E+01_JPRB, 0.12674E+01_JPRB, 0.12626E+01_JPRB, 0.11959E+01_JPRB ,&
&  0.10624E+01_JPRB, 0.96688E+00_JPRB, 0.90845E+00_JPRB, 0.87360E+00_JPRB, 0.83399E+00_JPRB, 0.75649E+00_JPRB, 0.63899E+00_JPRB ,&
&  0.54218E+00_JPRB, 0.46839E+00_JPRB, 0.43451E+00_JPRB, 0.41978E+00_JPRB, 0.42288E+00_JPRB, 0.43025E+00_JPRB, 0.43258E+00_JPRB ,&
&  0.42923E+00_JPRB, 0.42507E+00_JPRB, 0.42581E+00_JPRB, 0.44184E+00_JPRB, 0.48764E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.48402E+00_JPRB, 0.50866E+00_JPRB, 0.53333E+00_JPRB, 0.51797E+00_JPRB, 0.49746E+00_JPRB, 0.44843E+00_JPRB, 0.41412E+00_JPRB ,&
&  0.39351E+00_JPRB, 0.39228E+00_JPRB, 0.39401E+00_JPRB, 0.39806E+00_JPRB, 0.40524E+00_JPRB, 0.42062E+00_JPRB, 0.43154E+00_JPRB ,&
&  0.44006E+00_JPRB, 0.43998E+00_JPRB, 0.44066E+00_JPRB, 0.43860E+00_JPRB, 0.43079E+00_JPRB, 0.41596E+00_JPRB, 0.40391E+00_JPRB ,&
&  0.40023E+00_JPRB, 0.40132E+00_JPRB, 0.39746E+00_JPRB, 0.38921E+00_JPRB, 0.38340E+00_JPRB, 0.37371E+00_JPRB, 0.37814E+00_JPRB ,&
&  0.37213E+00_JPRB, 0.36384E+00_JPRB, 0.37162E+00_JPRB, 0.38810E+00_JPRB, 0.40237E+00_JPRB, 0.42413E+00_JPRB, 0.38652E+00_JPRB ,&
&  0.34422E+00_JPRB, 0.32414E+00_JPRB, 0.31586E+00_JPRB, 0.32411E+00_JPRB, 0.37619E+00_JPRB, 0.50651E+00_JPRB, 0.55772E+00_JPRB ,&
&  0.58804E+00_JPRB, 0.61089E+00_JPRB, 0.63862E+00_JPRB, 0.67392E+00_JPRB, 0.69825E+00_JPRB, 0.76841E+00_JPRB, 0.86939E+00_JPRB ,&
&  0.95943E+00_JPRB, 0.10371E+01_JPRB, 0.11146E+01_JPRB, 0.95985E+00_JPRB, 0.85995E+00_JPRB, 0.86117E+00_JPRB, 0.88108E+00_JPRB ,&
&  0.91405E+00_JPRB, 0.86223E+00_JPRB, 0.86412E+00_JPRB, 0.87942E+00_JPRB, 0.84096E+00_JPRB, 0.80126E+00_JPRB, 0.74385E+00_JPRB ,&
&  0.69296E+00_JPRB, 0.66741E+00_JPRB, 0.68450E+00_JPRB, 0.68570E+00_JPRB, 0.71274E+00_JPRB, 0.76588E+00_JPRB, 0.81590E+00_JPRB ,&
&  0.88384E+00_JPRB, 0.93670E+00_JPRB, 0.95555E+00_JPRB, 0.95884E+00_JPRB, 0.96729E+00_JPRB, 0.99330E+00_JPRB, 0.99676E+00_JPRB ,&
&  0.92941E+00_JPRB, 0.87433E+00_JPRB, 0.83193E+00_JPRB, 0.79984E+00_JPRB, 0.75066E+00_JPRB, 0.69857E+00_JPRB, 0.63218E+00_JPRB ,&
&  0.55744E+00_JPRB, 0.49764E+00_JPRB, 0.46626E+00_JPRB, 0.45187E+00_JPRB, 0.45673E+00_JPRB, 0.46941E+00_JPRB, 0.47516E+00_JPRB ,&
&  0.46983E+00_JPRB, 0.45883E+00_JPRB, 0.45498E+00_JPRB, 0.46064E+00_JPRB, 0.47640E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.52358E+00_JPRB, 0.52666E+00_JPRB, 0.52778E+00_JPRB, 0.52410E+00_JPRB, 0.50896E+00_JPRB, 0.48203E+00_JPRB, 0.46970E+00_JPRB ,&
&  0.46370E+00_JPRB, 0.46739E+00_JPRB, 0.46563E+00_JPRB, 0.46209E+00_JPRB, 0.46202E+00_JPRB, 0.46903E+00_JPRB, 0.47761E+00_JPRB ,&
&  0.48231E+00_JPRB, 0.47835E+00_JPRB, 0.46873E+00_JPRB, 0.44758E+00_JPRB, 0.43004E+00_JPRB, 0.41650E+00_JPRB, 0.41038E+00_JPRB ,&
&  0.41139E+00_JPRB, 0.40579E+00_JPRB, 0.39246E+00_JPRB, 0.37480E+00_JPRB, 0.35975E+00_JPRB, 0.34732E+00_JPRB, 0.34974E+00_JPRB ,&
&  0.36013E+00_JPRB, 0.36373E+00_JPRB, 0.37338E+00_JPRB, 0.38033E+00_JPRB, 0.38925E+00_JPRB, 0.41090E+00_JPRB, 0.35840E+00_JPRB ,&
&  0.32177E+00_JPRB, 0.30472E+00_JPRB, 0.29246E+00_JPRB, 0.29051E+00_JPRB, 0.30002E+00_JPRB, 0.32679E+00_JPRB, 0.38677E+00_JPRB ,&
&  0.51667E+00_JPRB, 0.55438E+00_JPRB, 0.57329E+00_JPRB, 0.59582E+00_JPRB, 0.61142E+00_JPRB, 0.62223E+00_JPRB, 0.66735E+00_JPRB ,&
&  0.72350E+00_JPRB, 0.79521E+00_JPRB, 0.85449E+00_JPRB, 0.86440E+00_JPRB, 0.85868E+00_JPRB, 0.82064E+00_JPRB, 0.87182E+00_JPRB ,&
&  0.10791E+01_JPRB, 0.97774E+00_JPRB, 0.86201E+00_JPRB, 0.78088E+00_JPRB, 0.71966E+00_JPRB, 0.67240E+00_JPRB, 0.66139E+00_JPRB ,&
&  0.65785E+00_JPRB, 0.65013E+00_JPRB, 0.65739E+00_JPRB, 0.65817E+00_JPRB, 0.66434E+00_JPRB, 0.68760E+00_JPRB, 0.73102E+00_JPRB ,&
&  0.76702E+00_JPRB, 0.79618E+00_JPRB, 0.84272E+00_JPRB, 0.82604E+00_JPRB, 0.82466E+00_JPRB, 0.84345E+00_JPRB, 0.86328E+00_JPRB ,&
&  0.84588E+00_JPRB, 0.84358E+00_JPRB, 0.79390E+00_JPRB, 0.72838E+00_JPRB, 0.69763E+00_JPRB, 0.64579E+00_JPRB, 0.59476E+00_JPRB ,&
&  0.54382E+00_JPRB, 0.50522E+00_JPRB, 0.48726E+00_JPRB, 0.48097E+00_JPRB, 0.49585E+00_JPRB, 0.51197E+00_JPRB, 0.52180E+00_JPRB ,&
&  0.52293E+00_JPRB, 0.52175E+00_JPRB, 0.52006E+00_JPRB, 0.51909E+00_JPRB, 0.51948E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.53409E+00_JPRB, 0.53249E+00_JPRB, 0.52855E+00_JPRB, 0.52422E+00_JPRB, 0.51609E+00_JPRB, 0.51044E+00_JPRB, 0.50475E+00_JPRB ,&
&  0.50086E+00_JPRB, 0.49807E+00_JPRB, 0.49524E+00_JPRB, 0.49295E+00_JPRB, 0.49070E+00_JPRB, 0.48984E+00_JPRB, 0.48674E+00_JPRB ,&
&  0.47890E+00_JPRB, 0.45006E+00_JPRB, 0.42852E+00_JPRB, 0.41918E+00_JPRB, 0.41227E+00_JPRB, 0.40955E+00_JPRB, 0.40712E+00_JPRB ,&
&  0.39926E+00_JPRB, 0.38896E+00_JPRB, 0.37304E+00_JPRB, 0.35081E+00_JPRB, 0.33962E+00_JPRB, 0.34235E+00_JPRB, 0.35316E+00_JPRB ,&
&  0.36939E+00_JPRB, 0.37614E+00_JPRB, 0.37023E+00_JPRB, 0.37125E+00_JPRB, 0.38713E+00_JPRB, 0.38029E+00_JPRB, 0.32749E+00_JPRB ,&
&  0.30261E+00_JPRB, 0.28681E+00_JPRB, 0.28183E+00_JPRB, 0.28068E+00_JPRB, 0.28645E+00_JPRB, 0.30512E+00_JPRB, 0.35196E+00_JPRB ,&
&  0.43218E+00_JPRB, 0.53228E+00_JPRB, 0.54372E+00_JPRB, 0.54894E+00_JPRB, 0.55540E+00_JPRB, 0.56090E+00_JPRB, 0.57337E+00_JPRB ,&
&  0.59614E+00_JPRB, 0.62731E+00_JPRB, 0.65401E+00_JPRB, 0.68285E+00_JPRB, 0.70731E+00_JPRB, 0.71683E+00_JPRB, 0.71884E+00_JPRB ,&
&  0.72930E+00_JPRB, 0.70483E+00_JPRB, 0.69282E+00_JPRB, 0.68302E+00_JPRB, 0.67884E+00_JPRB, 0.65120E+00_JPRB, 0.64124E+00_JPRB ,&
&  0.64829E+00_JPRB, 0.65498E+00_JPRB, 0.66673E+00_JPRB, 0.67087E+00_JPRB, 0.67848E+00_JPRB, 0.70091E+00_JPRB, 0.71666E+00_JPRB ,&
&  0.72523E+00_JPRB, 0.74503E+00_JPRB, 0.77612E+00_JPRB, 0.78170E+00_JPRB, 0.78696E+00_JPRB, 0.79232E+00_JPRB, 0.79648E+00_JPRB ,&
&  0.79494E+00_JPRB, 0.80982E+00_JPRB, 0.78885E+00_JPRB, 0.74485E+00_JPRB, 0.69909E+00_JPRB, 0.63367E+00_JPRB, 0.58443E+00_JPRB ,&
&  0.54890E+00_JPRB, 0.50844E+00_JPRB, 0.49322E+00_JPRB, 0.49467E+00_JPRB, 0.50934E+00_JPRB, 0.52225E+00_JPRB, 0.52834E+00_JPRB ,&
&  0.53544E+00_JPRB, 0.54045E+00_JPRB, 0.54316E+00_JPRB, 0.54175E+00_JPRB, 0.53810E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.51833E+00_JPRB, 0.51503E+00_JPRB, 0.51415E+00_JPRB, 0.51406E+00_JPRB, 0.50819E+00_JPRB, 0.50202E+00_JPRB, 0.49297E+00_JPRB ,&
&  0.48639E+00_JPRB, 0.48104E+00_JPRB, 0.47698E+00_JPRB, 0.47190E+00_JPRB, 0.46667E+00_JPRB, 0.46055E+00_JPRB, 0.45606E+00_JPRB ,&
&  0.45086E+00_JPRB, 0.44379E+00_JPRB, 0.43160E+00_JPRB, 0.42012E+00_JPRB, 0.41170E+00_JPRB, 0.40674E+00_JPRB, 0.40066E+00_JPRB ,&
&  0.39565E+00_JPRB, 0.38575E+00_JPRB, 0.36439E+00_JPRB, 0.34610E+00_JPRB, 0.33169E+00_JPRB, 0.33130E+00_JPRB, 0.35858E+00_JPRB ,&
&  0.37127E+00_JPRB, 0.37377E+00_JPRB, 0.37565E+00_JPRB, 0.37055E+00_JPRB, 0.36449E+00_JPRB, 0.31380E+00_JPRB, 0.30938E+00_JPRB ,&
&  0.29511E+00_JPRB, 0.28517E+00_JPRB, 0.28294E+00_JPRB, 0.28349E+00_JPRB, 0.28873E+00_JPRB, 0.30192E+00_JPRB, 0.32186E+00_JPRB ,&
&  0.35765E+00_JPRB, 0.46903E+00_JPRB, 0.49742E+00_JPRB, 0.51930E+00_JPRB, 0.52226E+00_JPRB, 0.52279E+00_JPRB, 0.52734E+00_JPRB ,&
&  0.53482E+00_JPRB, 0.54871E+00_JPRB, 0.55867E+00_JPRB, 0.57265E+00_JPRB, 0.58618E+00_JPRB, 0.59438E+00_JPRB, 0.61295E+00_JPRB ,&
&  0.62508E+00_JPRB, 0.63063E+00_JPRB, 0.63043E+00_JPRB, 0.62755E+00_JPRB, 0.63390E+00_JPRB, 0.62294E+00_JPRB, 0.61351E+00_JPRB ,&
&  0.62829E+00_JPRB, 0.65837E+00_JPRB, 0.68196E+00_JPRB, 0.70501E+00_JPRB, 0.71009E+00_JPRB, 0.71963E+00_JPRB, 0.73674E+00_JPRB ,&
&  0.74395E+00_JPRB, 0.76123E+00_JPRB, 0.78368E+00_JPRB, 0.78260E+00_JPRB, 0.78024E+00_JPRB, 0.77697E+00_JPRB, 0.78415E+00_JPRB ,&
&  0.78815E+00_JPRB, 0.79226E+00_JPRB, 0.76994E+00_JPRB, 0.74781E+00_JPRB, 0.71373E+00_JPRB, 0.66902E+00_JPRB, 0.63315E+00_JPRB ,&
&  0.59311E+00_JPRB, 0.56398E+00_JPRB, 0.55338E+00_JPRB, 0.55374E+00_JPRB, 0.55967E+00_JPRB, 0.56684E+00_JPRB, 0.56893E+00_JPRB ,&
&  0.56898E+00_JPRB, 0.56216E+00_JPRB, 0.55084E+00_JPRB, 0.53707E+00_JPRB, 0.52563E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.50361E+00_JPRB, 0.49742E+00_JPRB, 0.49197E+00_JPRB, 0.48566E+00_JPRB, 0.47852E+00_JPRB, 0.47330E+00_JPRB, 0.46862E+00_JPRB ,&
&  0.46578E+00_JPRB, 0.46240E+00_JPRB, 0.45953E+00_JPRB, 0.45515E+00_JPRB, 0.45155E+00_JPRB, 0.44855E+00_JPRB, 0.44630E+00_JPRB ,&
&  0.44491E+00_JPRB, 0.43951E+00_JPRB, 0.43304E+00_JPRB, 0.42482E+00_JPRB, 0.41736E+00_JPRB, 0.41023E+00_JPRB, 0.40224E+00_JPRB ,&
&  0.39554E+00_JPRB, 0.38182E+00_JPRB, 0.36299E+00_JPRB, 0.35284E+00_JPRB, 0.33641E+00_JPRB, 0.32941E+00_JPRB, 0.32897E+00_JPRB ,&
&  0.34685E+00_JPRB, 0.33684E+00_JPRB, 0.34013E+00_JPRB, 0.32687E+00_JPRB, 0.31487E+00_JPRB, 0.30374E+00_JPRB, 0.29673E+00_JPRB ,&
&  0.29121E+00_JPRB, 0.28679E+00_JPRB, 0.28685E+00_JPRB, 0.28968E+00_JPRB, 0.29267E+00_JPRB, 0.30277E+00_JPRB, 0.31787E+00_JPRB ,&
&  0.35394E+00_JPRB, 0.44982E+00_JPRB, 0.46895E+00_JPRB, 0.49021E+00_JPRB, 0.49337E+00_JPRB, 0.49361E+00_JPRB, 0.49523E+00_JPRB ,&
&  0.49884E+00_JPRB, 0.50376E+00_JPRB, 0.50686E+00_JPRB, 0.50634E+00_JPRB, 0.52362E+00_JPRB, 0.53724E+00_JPRB, 0.55428E+00_JPRB ,&
&  0.56238E+00_JPRB, 0.56694E+00_JPRB, 0.56825E+00_JPRB, 0.57467E+00_JPRB, 0.59112E+00_JPRB, 0.59225E+00_JPRB, 0.58528E+00_JPRB ,&
&  0.59189E+00_JPRB, 0.60862E+00_JPRB, 0.64016E+00_JPRB, 0.68240E+00_JPRB, 0.70386E+00_JPRB, 0.72456E+00_JPRB, 0.76005E+00_JPRB ,&
&  0.78404E+00_JPRB, 0.82328E+00_JPRB, 0.85920E+00_JPRB, 0.86696E+00_JPRB, 0.85776E+00_JPRB, 0.83878E+00_JPRB, 0.81800E+00_JPRB ,&
&  0.80448E+00_JPRB, 0.78104E+00_JPRB, 0.76026E+00_JPRB, 0.74375E+00_JPRB, 0.72617E+00_JPRB, 0.70575E+00_JPRB, 0.69216E+00_JPRB ,&
&  0.67444E+00_JPRB, 0.66141E+00_JPRB, 0.64857E+00_JPRB, 0.63492E+00_JPRB, 0.62115E+00_JPRB, 0.60634E+00_JPRB, 0.58897E+00_JPRB ,&
&  0.57085E+00_JPRB, 0.55318E+00_JPRB, 0.53504E+00_JPRB, 0.52119E+00_JPRB, 0.51025E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.48941E+00_JPRB, 0.48001E+00_JPRB, 0.47026E+00_JPRB, 0.46237E+00_JPRB, 0.45624E+00_JPRB, 0.45271E+00_JPRB, 0.45211E+00_JPRB ,&
&  0.45242E+00_JPRB, 0.45164E+00_JPRB, 0.44982E+00_JPRB, 0.44755E+00_JPRB, 0.44535E+00_JPRB, 0.44554E+00_JPRB, 0.44577E+00_JPRB ,&
&  0.44597E+00_JPRB, 0.44342E+00_JPRB, 0.43848E+00_JPRB, 0.43135E+00_JPRB, 0.42344E+00_JPRB, 0.41694E+00_JPRB, 0.40797E+00_JPRB ,&
&  0.39848E+00_JPRB, 0.38769E+00_JPRB, 0.36838E+00_JPRB, 0.35665E+00_JPRB, 0.33880E+00_JPRB, 0.32734E+00_JPRB, 0.32177E+00_JPRB ,&
&  0.31842E+00_JPRB, 0.31977E+00_JPRB, 0.32411E+00_JPRB, 0.32805E+00_JPRB, 0.32442E+00_JPRB, 0.31865E+00_JPRB, 0.31583E+00_JPRB ,&
&  0.31055E+00_JPRB, 0.30787E+00_JPRB, 0.30606E+00_JPRB, 0.30657E+00_JPRB, 0.30959E+00_JPRB, 0.31714E+00_JPRB, 0.33178E+00_JPRB ,&
&  0.36394E+00_JPRB, 0.40879E+00_JPRB, 0.45130E+00_JPRB, 0.47151E+00_JPRB, 0.46976E+00_JPRB, 0.47467E+00_JPRB, 0.47553E+00_JPRB ,&
&  0.47795E+00_JPRB, 0.47957E+00_JPRB, 0.48289E+00_JPRB, 0.48811E+00_JPRB, 0.49767E+00_JPRB, 0.50631E+00_JPRB, 0.51551E+00_JPRB ,&
&  0.51912E+00_JPRB, 0.52232E+00_JPRB, 0.51981E+00_JPRB, 0.52478E+00_JPRB, 0.53490E+00_JPRB, 0.54203E+00_JPRB, 0.54165E+00_JPRB ,&
&  0.54605E+00_JPRB, 0.55555E+00_JPRB, 0.57934E+00_JPRB, 0.61847E+00_JPRB, 0.65793E+00_JPRB, 0.69627E+00_JPRB, 0.73269E+00_JPRB ,&
&  0.77003E+00_JPRB, 0.81323E+00_JPRB, 0.85113E+00_JPRB, 0.86114E+00_JPRB, 0.85556E+00_JPRB, 0.85488E+00_JPRB, 0.83631E+00_JPRB ,&
&  0.81788E+00_JPRB, 0.79697E+00_JPRB, 0.78125E+00_JPRB, 0.76855E+00_JPRB, 0.75747E+00_JPRB, 0.73981E+00_JPRB, 0.72328E+00_JPRB ,&
&  0.70359E+00_JPRB, 0.68102E+00_JPRB, 0.65726E+00_JPRB, 0.63395E+00_JPRB, 0.61189E+00_JPRB, 0.59090E+00_JPRB, 0.57208E+00_JPRB ,&
&  0.55379E+00_JPRB, 0.53809E+00_JPRB, 0.52301E+00_JPRB, 0.51013E+00_JPRB, 0.49957E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.48179E+00_JPRB, 0.47373E+00_JPRB, 0.46649E+00_JPRB, 0.46048E+00_JPRB, 0.45593E+00_JPRB, 0.45346E+00_JPRB, 0.45245E+00_JPRB ,&
&  0.45214E+00_JPRB, 0.45297E+00_JPRB, 0.45345E+00_JPRB, 0.45377E+00_JPRB, 0.45380E+00_JPRB, 0.45538E+00_JPRB, 0.45620E+00_JPRB ,&
&  0.45595E+00_JPRB, 0.45448E+00_JPRB, 0.45264E+00_JPRB, 0.44963E+00_JPRB, 0.44378E+00_JPRB, 0.43647E+00_JPRB, 0.42604E+00_JPRB ,&
&  0.41917E+00_JPRB, 0.40992E+00_JPRB, 0.39899E+00_JPRB, 0.38704E+00_JPRB, 0.37908E+00_JPRB, 0.36075E+00_JPRB, 0.35344E+00_JPRB ,&
&  0.34857E+00_JPRB, 0.34128E+00_JPRB, 0.33801E+00_JPRB, 0.33831E+00_JPRB, 0.35386E+00_JPRB, 0.36538E+00_JPRB, 0.37122E+00_JPRB ,&
&  0.35931E+00_JPRB, 0.36081E+00_JPRB, 0.35807E+00_JPRB, 0.35701E+00_JPRB, 0.35971E+00_JPRB, 0.36641E+00_JPRB, 0.38949E+00_JPRB ,&
&  0.42978E+00_JPRB, 0.44581E+00_JPRB, 0.45463E+00_JPRB, 0.46328E+00_JPRB, 0.46781E+00_JPRB, 0.47041E+00_JPRB, 0.47193E+00_JPRB ,&
&  0.47777E+00_JPRB, 0.48285E+00_JPRB, 0.48683E+00_JPRB, 0.48909E+00_JPRB, 0.49175E+00_JPRB, 0.49457E+00_JPRB, 0.49753E+00_JPRB ,&
&  0.49392E+00_JPRB, 0.49176E+00_JPRB, 0.49101E+00_JPRB, 0.49309E+00_JPRB, 0.49610E+00_JPRB, 0.50205E+00_JPRB, 0.50941E+00_JPRB ,&
&  0.51828E+00_JPRB, 0.52412E+00_JPRB, 0.53713E+00_JPRB, 0.55763E+00_JPRB, 0.58169E+00_JPRB, 0.61254E+00_JPRB, 0.64496E+00_JPRB ,&
&  0.67632E+00_JPRB, 0.70216E+00_JPRB, 0.72520E+00_JPRB, 0.73539E+00_JPRB, 0.74035E+00_JPRB, 0.74358E+00_JPRB, 0.74084E+00_JPRB ,&
&  0.73524E+00_JPRB, 0.72628E+00_JPRB, 0.71723E+00_JPRB, 0.70534E+00_JPRB, 0.69263E+00_JPRB, 0.67903E+00_JPRB, 0.66520E+00_JPRB ,&
&  0.64761E+00_JPRB, 0.63007E+00_JPRB, 0.61231E+00_JPRB, 0.59509E+00_JPRB, 0.57835E+00_JPRB, 0.56317E+00_JPRB, 0.54917E+00_JPRB ,&
&  0.53643E+00_JPRB, 0.52365E+00_JPRB, 0.51252E+00_JPRB, 0.50204E+00_JPRB, 0.49180E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.47318E+00_JPRB, 0.46917E+00_JPRB, 0.46610E+00_JPRB, 0.46349E+00_JPRB, 0.46188E+00_JPRB, 0.46137E+00_JPRB, 0.46128E+00_JPRB ,&
&  0.46131E+00_JPRB, 0.46221E+00_JPRB, 0.46350E+00_JPRB, 0.46482E+00_JPRB, 0.46609E+00_JPRB, 0.46713E+00_JPRB, 0.46776E+00_JPRB ,&
&  0.46824E+00_JPRB, 0.46907E+00_JPRB, 0.46928E+00_JPRB, 0.46927E+00_JPRB, 0.46818E+00_JPRB, 0.46568E+00_JPRB, 0.46202E+00_JPRB ,&
&  0.45789E+00_JPRB, 0.45237E+00_JPRB, 0.44575E+00_JPRB, 0.43850E+00_JPRB, 0.43451E+00_JPRB, 0.42976E+00_JPRB, 0.42528E+00_JPRB ,&
&  0.42198E+00_JPRB, 0.42162E+00_JPRB, 0.42102E+00_JPRB, 0.42037E+00_JPRB, 0.42025E+00_JPRB, 0.42153E+00_JPRB, 0.42233E+00_JPRB ,&
&  0.42319E+00_JPRB, 0.42560E+00_JPRB, 0.43138E+00_JPRB, 0.43743E+00_JPRB, 0.44423E+00_JPRB, 0.45232E+00_JPRB, 0.46188E+00_JPRB ,&
&  0.46889E+00_JPRB, 0.47224E+00_JPRB, 0.47535E+00_JPRB, 0.47863E+00_JPRB, 0.48005E+00_JPRB, 0.48037E+00_JPRB, 0.48033E+00_JPRB ,&
&  0.48120E+00_JPRB, 0.48136E+00_JPRB, 0.48214E+00_JPRB, 0.48353E+00_JPRB, 0.48304E+00_JPRB, 0.48242E+00_JPRB, 0.48142E+00_JPRB ,&
&  0.47935E+00_JPRB, 0.47826E+00_JPRB, 0.47819E+00_JPRB, 0.47785E+00_JPRB, 0.47816E+00_JPRB, 0.48012E+00_JPRB, 0.48323E+00_JPRB ,&
&  0.48745E+00_JPRB, 0.49223E+00_JPRB, 0.50055E+00_JPRB, 0.51068E+00_JPRB, 0.52257E+00_JPRB, 0.53745E+00_JPRB, 0.55028E+00_JPRB ,&
&  0.56264E+00_JPRB, 0.57379E+00_JPRB, 0.58373E+00_JPRB, 0.59100E+00_JPRB, 0.59625E+00_JPRB, 0.60137E+00_JPRB, 0.60163E+00_JPRB ,&
&  0.60073E+00_JPRB, 0.59871E+00_JPRB, 0.59662E+00_JPRB, 0.59121E+00_JPRB, 0.58424E+00_JPRB, 0.57700E+00_JPRB, 0.57030E+00_JPRB ,&
&  0.56104E+00_JPRB, 0.55177E+00_JPRB, 0.54327E+00_JPRB, 0.53582E+00_JPRB, 0.52792E+00_JPRB, 0.52070E+00_JPRB, 0.51333E+00_JPRB ,&
&  0.50585E+00_JPRB, 0.49826E+00_JPRB, 0.49163E+00_JPRB, 0.48524E+00_JPRB, 0.47892E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.47628E+00_JPRB, 0.47612E+00_JPRB, 0.47617E+00_JPRB, 0.47593E+00_JPRB, 0.47543E+00_JPRB, 0.47540E+00_JPRB, 0.47548E+00_JPRB ,&
&  0.47550E+00_JPRB, 0.47586E+00_JPRB, 0.47648E+00_JPRB, 0.47723E+00_JPRB, 0.47832E+00_JPRB, 0.47941E+00_JPRB, 0.48060E+00_JPRB ,&
&  0.48171E+00_JPRB, 0.48282E+00_JPRB, 0.48391E+00_JPRB, 0.48470E+00_JPRB, 0.48511E+00_JPRB, 0.48507E+00_JPRB, 0.48470E+00_JPRB ,&
&  0.48389E+00_JPRB, 0.48297E+00_JPRB, 0.48215E+00_JPRB, 0.48140E+00_JPRB, 0.48028E+00_JPRB, 0.47895E+00_JPRB, 0.47745E+00_JPRB ,&
&  0.47613E+00_JPRB, 0.47531E+00_JPRB, 0.47461E+00_JPRB, 0.47383E+00_JPRB, 0.47339E+00_JPRB, 0.47389E+00_JPRB, 0.47436E+00_JPRB ,&
&  0.47494E+00_JPRB, 0.47579E+00_JPRB, 0.47757E+00_JPRB, 0.47928E+00_JPRB, 0.48087E+00_JPRB, 0.48238E+00_JPRB, 0.48371E+00_JPRB ,&
&  0.48417E+00_JPRB, 0.48427E+00_JPRB, 0.48407E+00_JPRB, 0.48367E+00_JPRB, 0.48261E+00_JPRB, 0.48165E+00_JPRB, 0.48070E+00_JPRB ,&
&  0.47964E+00_JPRB, 0.47872E+00_JPRB, 0.47783E+00_JPRB, 0.47659E+00_JPRB, 0.47407E+00_JPRB, 0.47234E+00_JPRB, 0.47166E+00_JPRB ,&
&  0.47177E+00_JPRB, 0.47179E+00_JPRB, 0.47182E+00_JPRB, 0.47176E+00_JPRB, 0.47205E+00_JPRB, 0.47249E+00_JPRB, 0.47322E+00_JPRB ,&
&  0.47375E+00_JPRB, 0.47468E+00_JPRB, 0.47653E+00_JPRB, 0.47847E+00_JPRB, 0.48069E+00_JPRB, 0.48319E+00_JPRB, 0.48548E+00_JPRB ,&
&  0.48805E+00_JPRB, 0.49101E+00_JPRB, 0.49364E+00_JPRB, 0.49639E+00_JPRB, 0.49933E+00_JPRB, 0.50222E+00_JPRB, 0.50422E+00_JPRB ,&
&  0.50521E+00_JPRB, 0.50600E+00_JPRB, 0.50655E+00_JPRB, 0.50594E+00_JPRB, 0.50457E+00_JPRB, 0.50325E+00_JPRB, 0.50175E+00_JPRB ,&
&  0.50010E+00_JPRB, 0.49817E+00_JPRB, 0.49636E+00_JPRB, 0.49433E+00_JPRB, 0.49200E+00_JPRB, 0.48978E+00_JPRB, 0.48755E+00_JPRB ,&
&  0.48524E+00_JPRB, 0.48291E+00_JPRB, 0.48107E+00_JPRB, 0.47928E+00_JPRB, 0.47762E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB, 0.48775E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB, 0.11615E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.12165E+00_JPRB, 0.12151E+00_JPRB, 0.12137E+00_JPRB, 0.12123E+00_JPRB, 0.12071E+00_JPRB, 0.12051E+00_JPRB, 0.12031E+00_JPRB ,&
&  0.12022E+00_JPRB, 0.12016E+00_JPRB, 0.12022E+00_JPRB, 0.12034E+00_JPRB, 0.12045E+00_JPRB, 0.12054E+00_JPRB, 0.12060E+00_JPRB ,&
&  0.12056E+00_JPRB, 0.12054E+00_JPRB, 0.11999E+00_JPRB, 0.11959E+00_JPRB, 0.11935E+00_JPRB, 0.11918E+00_JPRB, 0.11892E+00_JPRB ,&
&  0.11873E+00_JPRB, 0.11848E+00_JPRB, 0.11817E+00_JPRB, 0.11793E+00_JPRB, 0.11792E+00_JPRB, 0.11779E+00_JPRB, 0.11778E+00_JPRB ,&
&  0.11773E+00_JPRB, 0.11783E+00_JPRB, 0.11783E+00_JPRB, 0.11786E+00_JPRB, 0.11809E+00_JPRB, 0.11828E+00_JPRB, 0.11827E+00_JPRB ,&
&  0.11824E+00_JPRB, 0.11804E+00_JPRB, 0.11786E+00_JPRB, 0.11772E+00_JPRB, 0.11764E+00_JPRB, 0.11761E+00_JPRB, 0.11754E+00_JPRB ,&
&  0.11741E+00_JPRB, 0.11721E+00_JPRB, 0.11698E+00_JPRB, 0.11672E+00_JPRB, 0.11647E+00_JPRB, 0.11631E+00_JPRB, 0.11625E+00_JPRB ,&
&  0.11620E+00_JPRB, 0.11606E+00_JPRB, 0.11586E+00_JPRB, 0.11558E+00_JPRB, 0.11533E+00_JPRB, 0.11509E+00_JPRB, 0.11485E+00_JPRB ,&
&  0.11465E+00_JPRB, 0.11454E+00_JPRB, 0.11443E+00_JPRB, 0.11433E+00_JPRB, 0.11425E+00_JPRB, 0.11416E+00_JPRB, 0.11407E+00_JPRB ,&
&  0.11398E+00_JPRB, 0.11391E+00_JPRB, 0.11384E+00_JPRB, 0.11379E+00_JPRB, 0.11375E+00_JPRB, 0.11373E+00_JPRB, 0.11373E+00_JPRB ,&
&  0.11376E+00_JPRB, 0.11379E+00_JPRB, 0.11385E+00_JPRB, 0.11398E+00_JPRB, 0.11413E+00_JPRB, 0.11431E+00_JPRB, 0.11452E+00_JPRB ,&
&  0.11476E+00_JPRB, 0.11503E+00_JPRB, 0.11532E+00_JPRB, 0.11555E+00_JPRB, 0.11588E+00_JPRB, 0.11620E+00_JPRB, 0.11654E+00_JPRB ,&
&  0.11688E+00_JPRB, 0.11731E+00_JPRB, 0.11777E+00_JPRB, 0.11826E+00_JPRB, 0.11878E+00_JPRB, 0.11909E+00_JPRB, 0.11946E+00_JPRB ,&
&  0.11988E+00_JPRB, 0.12039E+00_JPRB, 0.12069E+00_JPRB, 0.12092E+00_JPRB, 0.12118E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.13140E+00_JPRB, 0.13339E+00_JPRB, 0.13492E+00_JPRB, 0.13680E+00_JPRB, 0.13693E+00_JPRB, 0.14827E+00_JPRB, 0.14642E+00_JPRB ,&
&  0.14533E+00_JPRB, 0.14038E+00_JPRB, 0.13984E+00_JPRB, 0.13820E+00_JPRB, 0.13629E+00_JPRB, 0.13420E+00_JPRB, 0.13228E+00_JPRB ,&
&  0.13052E+00_JPRB, 0.12858E+00_JPRB, 0.12714E+00_JPRB, 0.12595E+00_JPRB, 0.12487E+00_JPRB, 0.12388E+00_JPRB, 0.12308E+00_JPRB ,&
&  0.12299E+00_JPRB, 0.12344E+00_JPRB, 0.12432E+00_JPRB, 0.12422E+00_JPRB, 0.12399E+00_JPRB, 0.12366E+00_JPRB, 0.12350E+00_JPRB ,&
&  0.12301E+00_JPRB, 0.12266E+00_JPRB, 0.12234E+00_JPRB, 0.12186E+00_JPRB, 0.12227E+00_JPRB, 0.12265E+00_JPRB, 0.12304E+00_JPRB ,&
&  0.12341E+00_JPRB, 0.12376E+00_JPRB, 0.12365E+00_JPRB, 0.12347E+00_JPRB, 0.12306E+00_JPRB, 0.12250E+00_JPRB, 0.12169E+00_JPRB ,&
&  0.12090E+00_JPRB, 0.12009E+00_JPRB, 0.11930E+00_JPRB, 0.11872E+00_JPRB, 0.11822E+00_JPRB, 0.11773E+00_JPRB, 0.11733E+00_JPRB ,&
&  0.11712E+00_JPRB, 0.11699E+00_JPRB, 0.11683E+00_JPRB, 0.11657E+00_JPRB, 0.11625E+00_JPRB, 0.11592E+00_JPRB, 0.11563E+00_JPRB ,&
&  0.11540E+00_JPRB, 0.11500E+00_JPRB, 0.11463E+00_JPRB, 0.11425E+00_JPRB, 0.11396E+00_JPRB, 0.11360E+00_JPRB, 0.11329E+00_JPRB ,&
&  0.11302E+00_JPRB, 0.11281E+00_JPRB, 0.11276E+00_JPRB, 0.11274E+00_JPRB, 0.11273E+00_JPRB, 0.11278E+00_JPRB, 0.11290E+00_JPRB ,&
&  0.11307E+00_JPRB, 0.11325E+00_JPRB, 0.11349E+00_JPRB, 0.11366E+00_JPRB, 0.11384E+00_JPRB, 0.11399E+00_JPRB, 0.11434E+00_JPRB ,&
&  0.11476E+00_JPRB, 0.11521E+00_JPRB, 0.11565E+00_JPRB, 0.11591E+00_JPRB, 0.11664E+00_JPRB, 0.11738E+00_JPRB, 0.11818E+00_JPRB ,&
&  0.11902E+00_JPRB, 0.12010E+00_JPRB, 0.12125E+00_JPRB, 0.12247E+00_JPRB, 0.12386E+00_JPRB, 0.12483E+00_JPRB, 0.12586E+00_JPRB ,&
&  0.12722E+00_JPRB, 0.12942E+00_JPRB, 0.13034E+00_JPRB, 0.13079E+00_JPRB, 0.13117E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.15486E+00_JPRB, 0.15547E+00_JPRB, 0.15601E+00_JPRB, 0.15624E+00_JPRB, 0.15628E+00_JPRB, 0.15572E+00_JPRB, 0.15503E+00_JPRB ,&
&  0.15164E+00_JPRB, 0.14911E+00_JPRB, 0.14638E+00_JPRB, 0.14369E+00_JPRB, 0.14133E+00_JPRB, 0.13912E+00_JPRB, 0.13705E+00_JPRB ,&
&  0.13517E+00_JPRB, 0.13380E+00_JPRB, 0.13246E+00_JPRB, 0.13056E+00_JPRB, 0.12812E+00_JPRB, 0.12589E+00_JPRB, 0.12435E+00_JPRB ,&
&  0.12384E+00_JPRB, 0.12403E+00_JPRB, 0.12521E+00_JPRB, 0.12651E+00_JPRB, 0.12791E+00_JPRB, 0.12898E+00_JPRB, 0.13004E+00_JPRB ,&
&  0.12946E+00_JPRB, 0.12906E+00_JPRB, 0.12871E+00_JPRB, 0.12816E+00_JPRB, 0.12857E+00_JPRB, 0.12913E+00_JPRB, 0.12942E+00_JPRB ,&
&  0.12974E+00_JPRB, 0.13073E+00_JPRB, 0.13138E+00_JPRB, 0.13199E+00_JPRB, 0.13175E+00_JPRB, 0.13088E+00_JPRB, 0.12947E+00_JPRB ,&
&  0.12779E+00_JPRB, 0.12610E+00_JPRB, 0.12464E+00_JPRB, 0.12340E+00_JPRB, 0.12245E+00_JPRB, 0.12144E+00_JPRB, 0.12062E+00_JPRB ,&
&  0.12022E+00_JPRB, 0.11985E+00_JPRB, 0.11931E+00_JPRB, 0.11874E+00_JPRB, 0.11788E+00_JPRB, 0.11714E+00_JPRB, 0.11636E+00_JPRB ,&
&  0.11564E+00_JPRB, 0.11494E+00_JPRB, 0.11432E+00_JPRB, 0.11368E+00_JPRB, 0.11324E+00_JPRB, 0.11283E+00_JPRB, 0.11244E+00_JPRB ,&
&  0.11202E+00_JPRB, 0.11187E+00_JPRB, 0.11157E+00_JPRB, 0.11148E+00_JPRB, 0.11132E+00_JPRB, 0.11121E+00_JPRB, 0.11137E+00_JPRB ,&
&  0.11183E+00_JPRB, 0.11229E+00_JPRB, 0.11255E+00_JPRB, 0.11284E+00_JPRB, 0.11312E+00_JPRB, 0.11349E+00_JPRB, 0.11406E+00_JPRB ,&
&  0.11450E+00_JPRB, 0.11508E+00_JPRB, 0.11559E+00_JPRB, 0.11618E+00_JPRB, 0.11723E+00_JPRB, 0.11833E+00_JPRB, 0.11958E+00_JPRB ,&
&  0.12114E+00_JPRB, 0.12279E+00_JPRB, 0.12472E+00_JPRB, 0.12662E+00_JPRB, 0.12910E+00_JPRB, 0.13081E+00_JPRB, 0.13389E+00_JPRB ,&
&  0.13733E+00_JPRB, 0.13905E+00_JPRB, 0.15017E+00_JPRB, 0.15125E+00_JPRB, 0.15275E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.15978E+00_JPRB, 0.16087E+00_JPRB, 0.16082E+00_JPRB, 0.16123E+00_JPRB, 0.16052E+00_JPRB, 0.15931E+00_JPRB, 0.15764E+00_JPRB ,&
&  0.15542E+00_JPRB, 0.15005E+00_JPRB, 0.14640E+00_JPRB, 0.14239E+00_JPRB, 0.13900E+00_JPRB, 0.13609E+00_JPRB, 0.13407E+00_JPRB ,&
&  0.13324E+00_JPRB, 0.13307E+00_JPRB, 0.13302E+00_JPRB, 0.13230E+00_JPRB, 0.13095E+00_JPRB, 0.13005E+00_JPRB, 0.12887E+00_JPRB ,&
&  0.12800E+00_JPRB, 0.12787E+00_JPRB, 0.12857E+00_JPRB, 0.13024E+00_JPRB, 0.13315E+00_JPRB, 0.13546E+00_JPRB, 0.13750E+00_JPRB ,&
&  0.13981E+00_JPRB, 0.13690E+00_JPRB, 0.13657E+00_JPRB, 0.13670E+00_JPRB, 0.13640E+00_JPRB, 0.13549E+00_JPRB, 0.13369E+00_JPRB ,&
&  0.13578E+00_JPRB, 0.13629E+00_JPRB, 0.14043E+00_JPRB, 0.14040E+00_JPRB, 0.13934E+00_JPRB, 0.13981E+00_JPRB, 0.13813E+00_JPRB ,&
&  0.13731E+00_JPRB, 0.13450E+00_JPRB, 0.13244E+00_JPRB, 0.12978E+00_JPRB, 0.12729E+00_JPRB, 0.12604E+00_JPRB, 0.12461E+00_JPRB ,&
&  0.12413E+00_JPRB, 0.12401E+00_JPRB, 0.12292E+00_JPRB, 0.12224E+00_JPRB, 0.12103E+00_JPRB, 0.11966E+00_JPRB, 0.11877E+00_JPRB ,&
&  0.11784E+00_JPRB, 0.11710E+00_JPRB, 0.11668E+00_JPRB, 0.11631E+00_JPRB, 0.11616E+00_JPRB, 0.11611E+00_JPRB, 0.11605E+00_JPRB ,&
&  0.11607E+00_JPRB, 0.11598E+00_JPRB, 0.11552E+00_JPRB, 0.11523E+00_JPRB, 0.11441E+00_JPRB, 0.11384E+00_JPRB, 0.11338E+00_JPRB ,&
&  0.11301E+00_JPRB, 0.11289E+00_JPRB, 0.11283E+00_JPRB, 0.11288E+00_JPRB, 0.11300E+00_JPRB, 0.11338E+00_JPRB, 0.11381E+00_JPRB ,&
&  0.11426E+00_JPRB, 0.11501E+00_JPRB, 0.11561E+00_JPRB, 0.11644E+00_JPRB, 0.11742E+00_JPRB, 0.11826E+00_JPRB, 0.11959E+00_JPRB ,&
&  0.12108E+00_JPRB, 0.12282E+00_JPRB, 0.12538E+00_JPRB, 0.12786E+00_JPRB, 0.13068E+00_JPRB, 0.13469E+00_JPRB, 0.13916E+00_JPRB ,&
&  0.14415E+00_JPRB, 0.15041E+00_JPRB, 0.15128E+00_JPRB, 0.15542E+00_JPRB, 0.15822E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.15982E+00_JPRB, 0.16132E+00_JPRB, 0.16206E+00_JPRB, 0.16310E+00_JPRB, 0.16317E+00_JPRB, 0.16121E+00_JPRB, 0.15928E+00_JPRB ,&
&  0.15457E+00_JPRB, 0.15067E+00_JPRB, 0.14594E+00_JPRB, 0.14145E+00_JPRB, 0.13727E+00_JPRB, 0.13452E+00_JPRB, 0.13299E+00_JPRB ,&
&  0.13321E+00_JPRB, 0.13471E+00_JPRB, 0.13588E+00_JPRB, 0.13653E+00_JPRB, 0.13710E+00_JPRB, 0.13818E+00_JPRB, 0.13851E+00_JPRB ,&
&  0.13824E+00_JPRB, 0.13716E+00_JPRB, 0.13637E+00_JPRB, 0.13729E+00_JPRB, 0.13918E+00_JPRB, 0.14116E+00_JPRB, 0.14281E+00_JPRB ,&
&  0.14433E+00_JPRB, 0.14388E+00_JPRB, 0.14389E+00_JPRB, 0.14313E+00_JPRB, 0.14240E+00_JPRB, 0.14150E+00_JPRB, 0.14099E+00_JPRB ,&
&  0.14131E+00_JPRB, 0.14162E+00_JPRB, 0.14291E+00_JPRB, 0.14370E+00_JPRB, 0.14246E+00_JPRB, 0.14356E+00_JPRB, 0.14315E+00_JPRB ,&
&  0.14207E+00_JPRB, 0.13929E+00_JPRB, 0.13643E+00_JPRB, 0.13424E+00_JPRB, 0.13204E+00_JPRB, 0.13030E+00_JPRB, 0.12888E+00_JPRB ,&
&  0.12754E+00_JPRB, 0.12680E+00_JPRB, 0.12530E+00_JPRB, 0.12404E+00_JPRB, 0.12327E+00_JPRB, 0.12244E+00_JPRB, 0.12191E+00_JPRB ,&
&  0.12152E+00_JPRB, 0.12091E+00_JPRB, 0.12075E+00_JPRB, 0.12045E+00_JPRB, 0.12056E+00_JPRB, 0.12072E+00_JPRB, 0.12106E+00_JPRB ,&
&  0.12175E+00_JPRB, 0.12252E+00_JPRB, 0.12243E+00_JPRB, 0.12226E+00_JPRB, 0.12100E+00_JPRB, 0.11961E+00_JPRB, 0.11837E+00_JPRB ,&
&  0.11688E+00_JPRB, 0.11572E+00_JPRB, 0.11500E+00_JPRB, 0.11433E+00_JPRB, 0.11425E+00_JPRB, 0.11434E+00_JPRB, 0.11438E+00_JPRB ,&
&  0.11459E+00_JPRB, 0.11487E+00_JPRB, 0.11527E+00_JPRB, 0.11606E+00_JPRB, 0.11676E+00_JPRB, 0.11775E+00_JPRB, 0.11883E+00_JPRB ,&
&  0.11973E+00_JPRB, 0.12135E+00_JPRB, 0.12308E+00_JPRB, 0.12518E+00_JPRB, 0.12898E+00_JPRB, 0.13243E+00_JPRB, 0.13879E+00_JPRB ,&
&  0.14426E+00_JPRB, 0.15393E+00_JPRB, 0.15246E+00_JPRB, 0.15519E+00_JPRB, 0.15817E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.16186E+00_JPRB, 0.16273E+00_JPRB, 0.16249E+00_JPRB, 0.16262E+00_JPRB, 0.16222E+00_JPRB, 0.16345E+00_JPRB, 0.16290E+00_JPRB ,&
&  0.16307E+00_JPRB, 0.16237E+00_JPRB, 0.16159E+00_JPRB, 0.15956E+00_JPRB, 0.15851E+00_JPRB, 0.15276E+00_JPRB, 0.15139E+00_JPRB ,&
&  0.15042E+00_JPRB, 0.15134E+00_JPRB, 0.15166E+00_JPRB, 0.15218E+00_JPRB, 0.15320E+00_JPRB, 0.15486E+00_JPRB, 0.15506E+00_JPRB ,&
&  0.15412E+00_JPRB, 0.15210E+00_JPRB, 0.14978E+00_JPRB, 0.14765E+00_JPRB, 0.14800E+00_JPRB, 0.14777E+00_JPRB, 0.14697E+00_JPRB ,&
&  0.14598E+00_JPRB, 0.14650E+00_JPRB, 0.14698E+00_JPRB, 0.14821E+00_JPRB, 0.15306E+00_JPRB, 0.15178E+00_JPRB, 0.15044E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.14970E+00_JPRB, 0.14996E+00_JPRB, 0.15009E+00_JPRB, 0.15153E+00_JPRB, 0.15098E+00_JPRB, 0.15315E+00_JPRB ,&
&  0.15023E+00_JPRB, 0.14832E+00_JPRB, 0.14596E+00_JPRB, 0.14325E+00_JPRB, 0.14011E+00_JPRB, 0.13811E+00_JPRB, 0.13619E+00_JPRB ,&
&  0.13413E+00_JPRB, 0.13242E+00_JPRB, 0.13021E+00_JPRB, 0.12942E+00_JPRB, 0.12836E+00_JPRB, 0.12741E+00_JPRB, 0.12654E+00_JPRB ,&
&  0.12583E+00_JPRB, 0.12486E+00_JPRB, 0.12398E+00_JPRB, 0.12339E+00_JPRB, 0.12318E+00_JPRB, 0.12330E+00_JPRB, 0.12418E+00_JPRB ,&
&  0.12594E+00_JPRB, 0.12745E+00_JPRB, 0.12860E+00_JPRB, 0.12939E+00_JPRB, 0.12807E+00_JPRB, 0.12568E+00_JPRB, 0.12325E+00_JPRB ,&
&  0.12030E+00_JPRB, 0.11863E+00_JPRB, 0.11711E+00_JPRB, 0.11623E+00_JPRB, 0.11603E+00_JPRB, 0.11606E+00_JPRB, 0.11637E+00_JPRB ,&
&  0.11670E+00_JPRB, 0.11714E+00_JPRB, 0.11745E+00_JPRB, 0.11782E+00_JPRB, 0.11830E+00_JPRB, 0.11876E+00_JPRB, 0.11943E+00_JPRB ,&
&  0.12051E+00_JPRB, 0.12191E+00_JPRB, 0.12389E+00_JPRB, 0.12626E+00_JPRB, 0.12895E+00_JPRB, 0.13254E+00_JPRB, 0.13790E+00_JPRB ,&
&  0.14371E+00_JPRB, 0.15406E+00_JPRB, 0.15527E+00_JPRB, 0.15888E+00_JPRB, 0.16116E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.16421E+00_JPRB, 0.16553E+00_JPRB, 0.16601E+00_JPRB, 0.16706E+00_JPRB, 0.16750E+00_JPRB, 0.16880E+00_JPRB, 0.16995E+00_JPRB ,&
&  0.17238E+00_JPRB, 0.17456E+00_JPRB, 0.17510E+00_JPRB, 0.17349E+00_JPRB, 0.17275E+00_JPRB, 0.17065E+00_JPRB, 0.17000E+00_JPRB ,&
&  0.17012E+00_JPRB, 0.17132E+00_JPRB, 0.17160E+00_JPRB, 0.17178E+00_JPRB, 0.17185E+00_JPRB, 0.17190E+00_JPRB, 0.17231E+00_JPRB ,&
&  0.17054E+00_JPRB, 0.17019E+00_JPRB, 0.17227E+00_JPRB, 0.17172E+00_JPRB, 0.17125E+00_JPRB, 0.17015E+00_JPRB, 0.16956E+00_JPRB ,&
&  0.16596E+00_JPRB, 0.16061E+00_JPRB, 0.15374E+00_JPRB, 0.15606E+00_JPRB, 0.16312E+00_JPRB, 0.16137E+00_JPRB, 0.16052E+00_JPRB ,&
&  0.15978E+00_JPRB, 0.15853E+00_JPRB, 0.15796E+00_JPRB, 0.15741E+00_JPRB, 0.15774E+00_JPRB, 0.15809E+00_JPRB, 0.15926E+00_JPRB ,&
&  0.16100E+00_JPRB, 0.16364E+00_JPRB, 0.16349E+00_JPRB, 0.15786E+00_JPRB, 0.15465E+00_JPRB, 0.15343E+00_JPRB, 0.15044E+00_JPRB ,&
&  0.14705E+00_JPRB, 0.14543E+00_JPRB, 0.14468E+00_JPRB, 0.14301E+00_JPRB, 0.14271E+00_JPRB, 0.14235E+00_JPRB, 0.14047E+00_JPRB ,&
&  0.13786E+00_JPRB, 0.13592E+00_JPRB, 0.13296E+00_JPRB, 0.12986E+00_JPRB, 0.12766E+00_JPRB, 0.12660E+00_JPRB, 0.12656E+00_JPRB ,&
&  0.12804E+00_JPRB, 0.12977E+00_JPRB, 0.13506E+00_JPRB, 0.13871E+00_JPRB, 0.13620E+00_JPRB, 0.13253E+00_JPRB, 0.12895E+00_JPRB ,&
&  0.12442E+00_JPRB, 0.12218E+00_JPRB, 0.12046E+00_JPRB, 0.11925E+00_JPRB, 0.11916E+00_JPRB, 0.11990E+00_JPRB, 0.12055E+00_JPRB ,&
&  0.12124E+00_JPRB, 0.12206E+00_JPRB, 0.12245E+00_JPRB, 0.12259E+00_JPRB, 0.12261E+00_JPRB, 0.12279E+00_JPRB, 0.12326E+00_JPRB ,&
&  0.12391E+00_JPRB, 0.12544E+00_JPRB, 0.12770E+00_JPRB, 0.13016E+00_JPRB, 0.13186E+00_JPRB, 0.13315E+00_JPRB, 0.13584E+00_JPRB ,&
&  0.13910E+00_JPRB, 0.14665E+00_JPRB, 0.15749E+00_JPRB, 0.16644E+00_JPRB, 0.16348E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.17597E+00_JPRB, 0.17672E+00_JPRB, 0.17771E+00_JPRB, 0.17893E+00_JPRB, 0.18041E+00_JPRB, 0.18205E+00_JPRB, 0.18342E+00_JPRB ,&
&  0.18488E+00_JPRB, 0.18583E+00_JPRB, 0.18609E+00_JPRB, 0.18517E+00_JPRB, 0.18452E+00_JPRB, 0.18512E+00_JPRB, 0.18660E+00_JPRB ,&
&  0.18517E+00_JPRB, 0.18264E+00_JPRB, 0.18173E+00_JPRB, 0.18182E+00_JPRB, 0.18146E+00_JPRB, 0.18125E+00_JPRB, 0.18195E+00_JPRB ,&
&  0.18286E+00_JPRB, 0.18270E+00_JPRB, 0.18254E+00_JPRB, 0.18180E+00_JPRB, 0.18115E+00_JPRB, 0.18059E+00_JPRB, 0.17978E+00_JPRB ,&
&  0.17726E+00_JPRB, 0.16762E+00_JPRB, 0.16088E+00_JPRB, 0.16345E+00_JPRB, 0.17076E+00_JPRB, 0.17024E+00_JPRB, 0.17012E+00_JPRB ,&
&  0.16952E+00_JPRB, 0.16822E+00_JPRB, 0.16679E+00_JPRB, 0.16544E+00_JPRB, 0.16481E+00_JPRB, 0.16493E+00_JPRB, 0.16548E+00_JPRB ,&
&  0.16602E+00_JPRB, 0.16758E+00_JPRB, 0.16964E+00_JPRB, 0.17069E+00_JPRB, 0.17169E+00_JPRB, 0.17210E+00_JPRB, 0.17226E+00_JPRB ,&
&  0.17214E+00_JPRB, 0.17146E+00_JPRB, 0.17116E+00_JPRB, 0.17087E+00_JPRB, 0.17062E+00_JPRB, 0.16997E+00_JPRB, 0.16719E+00_JPRB ,&
&  0.16192E+00_JPRB, 0.15712E+00_JPRB, 0.15123E+00_JPRB, 0.14533E+00_JPRB, 0.13978E+00_JPRB, 0.13575E+00_JPRB, 0.13262E+00_JPRB ,&
&  0.13200E+00_JPRB, 0.13505E+00_JPRB, 0.14190E+00_JPRB, 0.15082E+00_JPRB, 0.15825E+00_JPRB, 0.14764E+00_JPRB, 0.14131E+00_JPRB ,&
&  0.13581E+00_JPRB, 0.13161E+00_JPRB, 0.12906E+00_JPRB, 0.12774E+00_JPRB, 0.12695E+00_JPRB, 0.12708E+00_JPRB, 0.12770E+00_JPRB ,&
&  0.12879E+00_JPRB, 0.13004E+00_JPRB, 0.13121E+00_JPRB, 0.13186E+00_JPRB, 0.13207E+00_JPRB, 0.13188E+00_JPRB, 0.13150E+00_JPRB ,&
&  0.13174E+00_JPRB, 0.13357E+00_JPRB, 0.13720E+00_JPRB, 0.14159E+00_JPRB, 0.14528E+00_JPRB, 0.14867E+00_JPRB, 0.15352E+00_JPRB ,&
&  0.15967E+00_JPRB, 0.16511E+00_JPRB, 0.16896E+00_JPRB, 0.17283E+00_JPRB, 0.17478E+00_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.19281E+00_JPRB, 0.19359E+00_JPRB, 0.19442E+00_JPRB, 0.19536E+00_JPRB, 0.19715E+00_JPRB, 0.19850E+00_JPRB, 0.20011E+00_JPRB ,&
&  0.20125E+00_JPRB, 0.20000E+00_JPRB, 0.19837E+00_JPRB, 0.19719E+00_JPRB, 0.19686E+00_JPRB, 0.19653E+00_JPRB, 0.19644E+00_JPRB ,&
&  0.19753E+00_JPRB, 0.19717E+00_JPRB, 0.19493E+00_JPRB, 0.19349E+00_JPRB, 0.19084E+00_JPRB, 0.18903E+00_JPRB, 0.18868E+00_JPRB ,&
&  0.19021E+00_JPRB, 0.19015E+00_JPRB, 0.18995E+00_JPRB, 0.18932E+00_JPRB, 0.18834E+00_JPRB, 0.18790E+00_JPRB, 0.18691E+00_JPRB ,&
&  0.18641E+00_JPRB, 0.18575E+00_JPRB, 0.18100E+00_JPRB, 0.18169E+00_JPRB, 0.18117E+00_JPRB, 0.18090E+00_JPRB, 0.18026E+00_JPRB ,&
&  0.17958E+00_JPRB, 0.17859E+00_JPRB, 0.17690E+00_JPRB, 0.17507E+00_JPRB, 0.17353E+00_JPRB, 0.17266E+00_JPRB, 0.17249E+00_JPRB ,&
&  0.17266E+00_JPRB, 0.17328E+00_JPRB, 0.17456E+00_JPRB, 0.17647E+00_JPRB, 0.17728E+00_JPRB, 0.17782E+00_JPRB, 0.17865E+00_JPRB ,&
&  0.17902E+00_JPRB, 0.17909E+00_JPRB, 0.17918E+00_JPRB, 0.17933E+00_JPRB, 0.17937E+00_JPRB, 0.17893E+00_JPRB, 0.17859E+00_JPRB ,&
&  0.17747E+00_JPRB, 0.17552E+00_JPRB, 0.17430E+00_JPRB, 0.17268E+00_JPRB, 0.16810E+00_JPRB, 0.15799E+00_JPRB, 0.15230E+00_JPRB ,&
&  0.15212E+00_JPRB, 0.15655E+00_JPRB, 0.16265E+00_JPRB, 0.16547E+00_JPRB, 0.16967E+00_JPRB, 0.16987E+00_JPRB, 0.16686E+00_JPRB ,&
&  0.15999E+00_JPRB, 0.15375E+00_JPRB, 0.14952E+00_JPRB, 0.14583E+00_JPRB, 0.14351E+00_JPRB, 0.14289E+00_JPRB, 0.14349E+00_JPRB ,&
&  0.14534E+00_JPRB, 0.14827E+00_JPRB, 0.15088E+00_JPRB, 0.15225E+00_JPRB, 0.15227E+00_JPRB, 0.15077E+00_JPRB, 0.14918E+00_JPRB ,&
&  0.14843E+00_JPRB, 0.15180E+00_JPRB, 0.15935E+00_JPRB, 0.16872E+00_JPRB, 0.17241E+00_JPRB, 0.17367E+00_JPRB, 0.17493E+00_JPRB ,&
&  0.17771E+00_JPRB, 0.18169E+00_JPRB, 0.18584E+00_JPRB, 0.18872E+00_JPRB, 0.19105E+00_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.20964E+00_JPRB, 0.21012E+00_JPRB, 0.21080E+00_JPRB, 0.21190E+00_JPRB, 0.21347E+00_JPRB, 0.21429E+00_JPRB, 0.21495E+00_JPRB ,&
&  0.21432E+00_JPRB, 0.21274E+00_JPRB, 0.21097E+00_JPRB, 0.20910E+00_JPRB, 0.20844E+00_JPRB, 0.20870E+00_JPRB, 0.20918E+00_JPRB ,&
&  0.20904E+00_JPRB, 0.20923E+00_JPRB, 0.20789E+00_JPRB, 0.20539E+00_JPRB, 0.20246E+00_JPRB, 0.20057E+00_JPRB, 0.20014E+00_JPRB ,&
&  0.19970E+00_JPRB, 0.19901E+00_JPRB, 0.19886E+00_JPRB, 0.19879E+00_JPRB, 0.19724E+00_JPRB, 0.19640E+00_JPRB, 0.19680E+00_JPRB ,&
&  0.19762E+00_JPRB, 0.19895E+00_JPRB, 0.19927E+00_JPRB, 0.19654E+00_JPRB, 0.19876E+00_JPRB, 0.19840E+00_JPRB, 0.19654E+00_JPRB ,&
&  0.19420E+00_JPRB, 0.19181E+00_JPRB, 0.18966E+00_JPRB, 0.18759E+00_JPRB, 0.18554E+00_JPRB, 0.18398E+00_JPRB, 0.18295E+00_JPRB ,&
&  0.18263E+00_JPRB, 0.18270E+00_JPRB, 0.18299E+00_JPRB, 0.18382E+00_JPRB, 0.18446E+00_JPRB, 0.18481E+00_JPRB, 0.18484E+00_JPRB ,&
&  0.18495E+00_JPRB, 0.18516E+00_JPRB, 0.18553E+00_JPRB, 0.18577E+00_JPRB, 0.18607E+00_JPRB, 0.18641E+00_JPRB, 0.18649E+00_JPRB ,&
&  0.18572E+00_JPRB, 0.18428E+00_JPRB, 0.18227E+00_JPRB, 0.18171E+00_JPRB, 0.18185E+00_JPRB, 0.18077E+00_JPRB, 0.17873E+00_JPRB ,&
&  0.17663E+00_JPRB, 0.17598E+00_JPRB, 0.17678E+00_JPRB, 0.17785E+00_JPRB, 0.17866E+00_JPRB, 0.17888E+00_JPRB, 0.17855E+00_JPRB ,&
&  0.17762E+00_JPRB, 0.17574E+00_JPRB, 0.17325E+00_JPRB, 0.17136E+00_JPRB, 0.17044E+00_JPRB, 0.17045E+00_JPRB, 0.17127E+00_JPRB ,&
&  0.17139E+00_JPRB, 0.17134E+00_JPRB, 0.17227E+00_JPRB, 0.17297E+00_JPRB, 0.17340E+00_JPRB, 0.17403E+00_JPRB, 0.17534E+00_JPRB ,&
&  0.17721E+00_JPRB, 0.18008E+00_JPRB, 0.18356E+00_JPRB, 0.18712E+00_JPRB, 0.18962E+00_JPRB, 0.18872E+00_JPRB, 0.18890E+00_JPRB ,&
&  0.19265E+00_JPRB, 0.19741E+00_JPRB, 0.20148E+00_JPRB, 0.20474E+00_JPRB, 0.20762E+00_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.22873E+00_JPRB, 0.22997E+00_JPRB, 0.23137E+00_JPRB, 0.23307E+00_JPRB, 0.23422E+00_JPRB, 0.23462E+00_JPRB, 0.23327E+00_JPRB ,&
&  0.23081E+00_JPRB, 0.22819E+00_JPRB, 0.22566E+00_JPRB, 0.22353E+00_JPRB, 0.22241E+00_JPRB, 0.22195E+00_JPRB, 0.22185E+00_JPRB ,&
&  0.22187E+00_JPRB, 0.22185E+00_JPRB, 0.22039E+00_JPRB, 0.21804E+00_JPRB, 0.21488E+00_JPRB, 0.21223E+00_JPRB, 0.21173E+00_JPRB ,&
&  0.21045E+00_JPRB, 0.20958E+00_JPRB, 0.20911E+00_JPRB, 0.20854E+00_JPRB, 0.20843E+00_JPRB, 0.20946E+00_JPRB, 0.21115E+00_JPRB ,&
&  0.21307E+00_JPRB, 0.21450E+00_JPRB, 0.21560E+00_JPRB, 0.21649E+00_JPRB, 0.21602E+00_JPRB, 0.21484E+00_JPRB, 0.21345E+00_JPRB ,&
&  0.21182E+00_JPRB, 0.21028E+00_JPRB, 0.20895E+00_JPRB, 0.20725E+00_JPRB, 0.20477E+00_JPRB, 0.20184E+00_JPRB, 0.19977E+00_JPRB ,&
&  0.19822E+00_JPRB, 0.19680E+00_JPRB, 0.19528E+00_JPRB, 0.19450E+00_JPRB, 0.19397E+00_JPRB, 0.19379E+00_JPRB, 0.19391E+00_JPRB ,&
&  0.19433E+00_JPRB, 0.19552E+00_JPRB, 0.19715E+00_JPRB, 0.19886E+00_JPRB, 0.19928E+00_JPRB, 0.19942E+00_JPRB, 0.19970E+00_JPRB ,&
&  0.19917E+00_JPRB, 0.19804E+00_JPRB, 0.19647E+00_JPRB, 0.19481E+00_JPRB, 0.19390E+00_JPRB, 0.19323E+00_JPRB, 0.19244E+00_JPRB ,&
&  0.19083E+00_JPRB, 0.18984E+00_JPRB, 0.18924E+00_JPRB, 0.18911E+00_JPRB, 0.18931E+00_JPRB, 0.18937E+00_JPRB, 0.18872E+00_JPRB ,&
&  0.18793E+00_JPRB, 0.18722E+00_JPRB, 0.18620E+00_JPRB, 0.18511E+00_JPRB, 0.18346E+00_JPRB, 0.18370E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18498E+00_JPRB, 0.18529E+00_JPRB, 0.18552E+00_JPRB, 0.18648E+00_JPRB, 0.18746E+00_JPRB, 0.18888E+00_JPRB, 0.19110E+00_JPRB ,&
&  0.19290E+00_JPRB, 0.19523E+00_JPRB, 0.19882E+00_JPRB, 0.20210E+00_JPRB, 0.20447E+00_JPRB, 0.20551E+00_JPRB, 0.20756E+00_JPRB ,&
&  0.21246E+00_JPRB, 0.21796E+00_JPRB, 0.22161E+00_JPRB, 0.22507E+00_JPRB, 0.22787E+00_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.24719E+00_JPRB, 0.25163E+00_JPRB, 0.25510E+00_JPRB, 0.25705E+00_JPRB, 0.25701E+00_JPRB, 0.25574E+00_JPRB, 0.25334E+00_JPRB ,&
&  0.25003E+00_JPRB, 0.24683E+00_JPRB, 0.24356E+00_JPRB, 0.24089E+00_JPRB, 0.23936E+00_JPRB, 0.23823E+00_JPRB, 0.23750E+00_JPRB ,&
&  0.23632E+00_JPRB, 0.23473E+00_JPRB, 0.23253E+00_JPRB, 0.23056E+00_JPRB, 0.22823E+00_JPRB, 0.22479E+00_JPRB, 0.22239E+00_JPRB ,&
&  0.22079E+00_JPRB, 0.21917E+00_JPRB, 0.21882E+00_JPRB, 0.22009E+00_JPRB, 0.22235E+00_JPRB, 0.22457E+00_JPRB, 0.22704E+00_JPRB ,&
&  0.22877E+00_JPRB, 0.22934E+00_JPRB, 0.22968E+00_JPRB, 0.22946E+00_JPRB, 0.22949E+00_JPRB, 0.22917E+00_JPRB, 0.22837E+00_JPRB ,&
&  0.22745E+00_JPRB, 0.22648E+00_JPRB, 0.22498E+00_JPRB, 0.22465E+00_JPRB, 0.22454E+00_JPRB, 0.22351E+00_JPRB, 0.22172E+00_JPRB ,&
&  0.21884E+00_JPRB, 0.21470E+00_JPRB, 0.21129E+00_JPRB, 0.20947E+00_JPRB, 0.20882E+00_JPRB, 0.20902E+00_JPRB, 0.20982E+00_JPRB ,&
&  0.21093E+00_JPRB, 0.21250E+00_JPRB, 0.21397E+00_JPRB, 0.21467E+00_JPRB, 0.21450E+00_JPRB, 0.21434E+00_JPRB, 0.21318E+00_JPRB ,&
&  0.21204E+00_JPRB, 0.21132E+00_JPRB, 0.21077E+00_JPRB, 0.21016E+00_JPRB, 0.20821E+00_JPRB, 0.20586E+00_JPRB, 0.20554E+00_JPRB ,&
&  0.20441E+00_JPRB, 0.20373E+00_JPRB, 0.20268E+00_JPRB, 0.20234E+00_JPRB, 0.20183E+00_JPRB, 0.20083E+00_JPRB, 0.19966E+00_JPRB ,&
&  0.19861E+00_JPRB, 0.19804E+00_JPRB, 0.19762E+00_JPRB, 0.19663E+00_JPRB, 0.19681E+00_JPRB, 0.19727E+00_JPRB, 0.19856E+00_JPRB ,&
&  0.19975E+00_JPRB, 0.20101E+00_JPRB, 0.20253E+00_JPRB, 0.20382E+00_JPRB, 0.20581E+00_JPRB, 0.20820E+00_JPRB, 0.21094E+00_JPRB ,&
&  0.21323E+00_JPRB, 0.21653E+00_JPRB, 0.22043E+00_JPRB, 0.22407E+00_JPRB, 0.22689E+00_JPRB, 0.22882E+00_JPRB, 0.23135E+00_JPRB ,&
&  0.23563E+00_JPRB, 0.23915E+00_JPRB, 0.24078E+00_JPRB, 0.24251E+00_JPRB, 0.24391E+00_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.27404E+00_JPRB, 0.27851E+00_JPRB, 0.28092E+00_JPRB, 0.28186E+00_JPRB, 0.28191E+00_JPRB, 0.28029E+00_JPRB, 0.27799E+00_JPRB ,&
&  0.27469E+00_JPRB, 0.27157E+00_JPRB, 0.26803E+00_JPRB, 0.26384E+00_JPRB, 0.26009E+00_JPRB, 0.25705E+00_JPRB, 0.25457E+00_JPRB ,&
&  0.25211E+00_JPRB, 0.24988E+00_JPRB, 0.24807E+00_JPRB, 0.24609E+00_JPRB, 0.24380E+00_JPRB, 0.24090E+00_JPRB, 0.23688E+00_JPRB ,&
&  0.23466E+00_JPRB, 0.23299E+00_JPRB, 0.23412E+00_JPRB, 0.23677E+00_JPRB, 0.23878E+00_JPRB, 0.24013E+00_JPRB, 0.24107E+00_JPRB ,&
&  0.24241E+00_JPRB, 0.24292E+00_JPRB, 0.24356E+00_JPRB, 0.24561E+00_JPRB, 0.24765E+00_JPRB, 0.24733E+00_JPRB, 0.24576E+00_JPRB ,&
&  0.24417E+00_JPRB, 0.24391E+00_JPRB, 0.24412E+00_JPRB, 0.24423E+00_JPRB, 0.24389E+00_JPRB, 0.24232E+00_JPRB, 0.23961E+00_JPRB ,&
&  0.23431E+00_JPRB, 0.23025E+00_JPRB, 0.22934E+00_JPRB, 0.23032E+00_JPRB, 0.23198E+00_JPRB, 0.23324E+00_JPRB, 0.23405E+00_JPRB ,&
&  0.23435E+00_JPRB, 0.23390E+00_JPRB, 0.23298E+00_JPRB, 0.23188E+00_JPRB, 0.23035E+00_JPRB, 0.22844E+00_JPRB, 0.22731E+00_JPRB ,&
&  0.22601E+00_JPRB, 0.22479E+00_JPRB, 0.22385E+00_JPRB, 0.22326E+00_JPRB, 0.22217E+00_JPRB, 0.22077E+00_JPRB, 0.22009E+00_JPRB ,&
&  0.21910E+00_JPRB, 0.21864E+00_JPRB, 0.21784E+00_JPRB, 0.21709E+00_JPRB, 0.21602E+00_JPRB, 0.21500E+00_JPRB, 0.21487E+00_JPRB ,&
&  0.21507E+00_JPRB, 0.21401E+00_JPRB, 0.21262E+00_JPRB, 0.21236E+00_JPRB, 0.21268E+00_JPRB, 0.21333E+00_JPRB, 0.21394E+00_JPRB ,&
&  0.21459E+00_JPRB, 0.21540E+00_JPRB, 0.21712E+00_JPRB, 0.21896E+00_JPRB, 0.22099E+00_JPRB, 0.22353E+00_JPRB, 0.22650E+00_JPRB ,&
&  0.23028E+00_JPRB, 0.23493E+00_JPRB, 0.24027E+00_JPRB, 0.24513E+00_JPRB, 0.24918E+00_JPRB, 0.25180E+00_JPRB, 0.25533E+00_JPRB ,&
&  0.25961E+00_JPRB, 0.26255E+00_JPRB, 0.26481E+00_JPRB, 0.26725E+00_JPRB, 0.26989E+00_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.30082E+00_JPRB, 0.30240E+00_JPRB, 0.30240E+00_JPRB, 0.30271E+00_JPRB, 0.30224E+00_JPRB, 0.30065E+00_JPRB, 0.29798E+00_JPRB ,&
&  0.29525E+00_JPRB, 0.29199E+00_JPRB, 0.28870E+00_JPRB, 0.28438E+00_JPRB, 0.27986E+00_JPRB, 0.27632E+00_JPRB, 0.27241E+00_JPRB ,&
&  0.26838E+00_JPRB, 0.26467E+00_JPRB, 0.26225E+00_JPRB, 0.26033E+00_JPRB, 0.25849E+00_JPRB, 0.25685E+00_JPRB, 0.25447E+00_JPRB ,&
&  0.25242E+00_JPRB, 0.25042E+00_JPRB, 0.25081E+00_JPRB, 0.25304E+00_JPRB, 0.25483E+00_JPRB, 0.25622E+00_JPRB, 0.25736E+00_JPRB ,&
&  0.25772E+00_JPRB, 0.26109E+00_JPRB, 0.25546E+00_JPRB, 0.26384E+00_JPRB, 0.26549E+00_JPRB, 0.26586E+00_JPRB, 0.26560E+00_JPRB ,&
&  0.26658E+00_JPRB, 0.26805E+00_JPRB, 0.26891E+00_JPRB, 0.26682E+00_JPRB, 0.26390E+00_JPRB, 0.26175E+00_JPRB, 0.25892E+00_JPRB ,&
&  0.25487E+00_JPRB, 0.25340E+00_JPRB, 0.25373E+00_JPRB, 0.25425E+00_JPRB, 0.25440E+00_JPRB, 0.25401E+00_JPRB, 0.25309E+00_JPRB ,&
&  0.25188E+00_JPRB, 0.25010E+00_JPRB, 0.24795E+00_JPRB, 0.24569E+00_JPRB, 0.24427E+00_JPRB, 0.24245E+00_JPRB, 0.24064E+00_JPRB ,&
&  0.23971E+00_JPRB, 0.23973E+00_JPRB, 0.23973E+00_JPRB, 0.23964E+00_JPRB, 0.23910E+00_JPRB, 0.23880E+00_JPRB, 0.23878E+00_JPRB ,&
&  0.23815E+00_JPRB, 0.23734E+00_JPRB, 0.23638E+00_JPRB, 0.23509E+00_JPRB, 0.23374E+00_JPRB, 0.23320E+00_JPRB, 0.23345E+00_JPRB ,&
&  0.23338E+00_JPRB, 0.23211E+00_JPRB, 0.23002E+00_JPRB, 0.22944E+00_JPRB, 0.22917E+00_JPRB, 0.22927E+00_JPRB, 0.22965E+00_JPRB ,&
&  0.23063E+00_JPRB, 0.23243E+00_JPRB, 0.23434E+00_JPRB, 0.23606E+00_JPRB, 0.23773E+00_JPRB, 0.24041E+00_JPRB, 0.24389E+00_JPRB ,&
&  0.24809E+00_JPRB, 0.25268E+00_JPRB, 0.25714E+00_JPRB, 0.26142E+00_JPRB, 0.26524E+00_JPRB, 0.26994E+00_JPRB, 0.27622E+00_JPRB ,&
&  0.28162E+00_JPRB, 0.28574E+00_JPRB, 0.28912E+00_JPRB, 0.29308E+00_JPRB, 0.29735E+00_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.33183E+00_JPRB, 0.33015E+00_JPRB, 0.32712E+00_JPRB, 0.32531E+00_JPRB, 0.32302E+00_JPRB, 0.32013E+00_JPRB, 0.31701E+00_JPRB ,&
&  0.31456E+00_JPRB, 0.31214E+00_JPRB, 0.30967E+00_JPRB, 0.30608E+00_JPRB, 0.30116E+00_JPRB, 0.29647E+00_JPRB, 0.29083E+00_JPRB ,&
&  0.28558E+00_JPRB, 0.28102E+00_JPRB, 0.27808E+00_JPRB, 0.27662E+00_JPRB, 0.27533E+00_JPRB, 0.27438E+00_JPRB, 0.27316E+00_JPRB ,&
&  0.27265E+00_JPRB, 0.27265E+00_JPRB, 0.27255E+00_JPRB, 0.27286E+00_JPRB, 0.27358E+00_JPRB, 0.27609E+00_JPRB, 0.27766E+00_JPRB ,&
&  0.27930E+00_JPRB, 0.27251E+00_JPRB, 0.27883E+00_JPRB, 0.28325E+00_JPRB, 0.29018E+00_JPRB, 0.29463E+00_JPRB, 0.29691E+00_JPRB ,&
&  0.29828E+00_JPRB, 0.29943E+00_JPRB, 0.29741E+00_JPRB, 0.29451E+00_JPRB, 0.29186E+00_JPRB, 0.29013E+00_JPRB, 0.28638E+00_JPRB ,&
&  0.28244E+00_JPRB, 0.27969E+00_JPRB, 0.27752E+00_JPRB, 0.27607E+00_JPRB, 0.27443E+00_JPRB, 0.27193E+00_JPRB, 0.26915E+00_JPRB ,&
&  0.26547E+00_JPRB, 0.26280E+00_JPRB, 0.26061E+00_JPRB, 0.25978E+00_JPRB, 0.25999E+00_JPRB, 0.26096E+00_JPRB, 0.26237E+00_JPRB ,&
&  0.26452E+00_JPRB, 0.26643E+00_JPRB, 0.26738E+00_JPRB, 0.26781E+00_JPRB, 0.26777E+00_JPRB, 0.26770E+00_JPRB, 0.26704E+00_JPRB ,&
&  0.26567E+00_JPRB, 0.26406E+00_JPRB, 0.26247E+00_JPRB, 0.26071E+00_JPRB, 0.25886E+00_JPRB, 0.25763E+00_JPRB, 0.25650E+00_JPRB ,&
&  0.25515E+00_JPRB, 0.25338E+00_JPRB, 0.25137E+00_JPRB, 0.25024E+00_JPRB, 0.24939E+00_JPRB, 0.24888E+00_JPRB, 0.24952E+00_JPRB ,&
&  0.25131E+00_JPRB, 0.25340E+00_JPRB, 0.25557E+00_JPRB, 0.25735E+00_JPRB, 0.25903E+00_JPRB, 0.26071E+00_JPRB, 0.26328E+00_JPRB ,&
&  0.26784E+00_JPRB, 0.27280E+00_JPRB, 0.27787E+00_JPRB, 0.28299E+00_JPRB, 0.28938E+00_JPRB, 0.29780E+00_JPRB, 0.30690E+00_JPRB ,&
&  0.31335E+00_JPRB, 0.31749E+00_JPRB, 0.32171E+00_JPRB, 0.32611E+00_JPRB, 0.33039E+00_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.35346E+00_JPRB, 0.35106E+00_JPRB, 0.34881E+00_JPRB, 0.34614E+00_JPRB, 0.34243E+00_JPRB, 0.33716E+00_JPRB, 0.33324E+00_JPRB ,&
&  0.33079E+00_JPRB, 0.32780E+00_JPRB, 0.32533E+00_JPRB, 0.32197E+00_JPRB, 0.31787E+00_JPRB, 0.31298E+00_JPRB, 0.30655E+00_JPRB ,&
&  0.30086E+00_JPRB, 0.29563E+00_JPRB, 0.29241E+00_JPRB, 0.29103E+00_JPRB, 0.29129E+00_JPRB, 0.29165E+00_JPRB, 0.29055E+00_JPRB ,&
&  0.28993E+00_JPRB, 0.28991E+00_JPRB, 0.28946E+00_JPRB, 0.28972E+00_JPRB, 0.29041E+00_JPRB, 0.29318E+00_JPRB, 0.29638E+00_JPRB ,&
&  0.29889E+00_JPRB, 0.27494E+00_JPRB, 0.28355E+00_JPRB, 0.28563E+00_JPRB, 0.29410E+00_JPRB, 0.30719E+00_JPRB, 0.32083E+00_JPRB ,&
&  0.32989E+00_JPRB, 0.33098E+00_JPRB, 0.32888E+00_JPRB, 0.32365E+00_JPRB, 0.31903E+00_JPRB, 0.31486E+00_JPRB, 0.31033E+00_JPRB ,&
&  0.30578E+00_JPRB, 0.30303E+00_JPRB, 0.30059E+00_JPRB, 0.29800E+00_JPRB, 0.29401E+00_JPRB, 0.28874E+00_JPRB, 0.28344E+00_JPRB ,&
&  0.27959E+00_JPRB, 0.27810E+00_JPRB, 0.27856E+00_JPRB, 0.28003E+00_JPRB, 0.28199E+00_JPRB, 0.28529E+00_JPRB, 0.28927E+00_JPRB ,&
&  0.29308E+00_JPRB, 0.29485E+00_JPRB, 0.29578E+00_JPRB, 0.29685E+00_JPRB, 0.29735E+00_JPRB, 0.29692E+00_JPRB, 0.29549E+00_JPRB ,&
&  0.29344E+00_JPRB, 0.29078E+00_JPRB, 0.28801E+00_JPRB, 0.28493E+00_JPRB, 0.28196E+00_JPRB, 0.27890E+00_JPRB, 0.27606E+00_JPRB ,&
&  0.27361E+00_JPRB, 0.27091E+00_JPRB, 0.26806E+00_JPRB, 0.26586E+00_JPRB, 0.26531E+00_JPRB, 0.26588E+00_JPRB, 0.26724E+00_JPRB ,&
&  0.26970E+00_JPRB, 0.27218E+00_JPRB, 0.27373E+00_JPRB, 0.27553E+00_JPRB, 0.27748E+00_JPRB, 0.27965E+00_JPRB, 0.28315E+00_JPRB ,&
&  0.28833E+00_JPRB, 0.29265E+00_JPRB, 0.29706E+00_JPRB, 0.30377E+00_JPRB, 0.31303E+00_JPRB, 0.32371E+00_JPRB, 0.33359E+00_JPRB ,&
&  0.34015E+00_JPRB, 0.34316E+00_JPRB, 0.34664E+00_JPRB, 0.34921E+00_JPRB, 0.35368E+00_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.38963E+00_JPRB, 0.38863E+00_JPRB, 0.38384E+00_JPRB, 0.37419E+00_JPRB, 0.36372E+00_JPRB, 0.35510E+00_JPRB, 0.35031E+00_JPRB ,&
&  0.34762E+00_JPRB, 0.34313E+00_JPRB, 0.33926E+00_JPRB, 0.33662E+00_JPRB, 0.33299E+00_JPRB, 0.32909E+00_JPRB, 0.32370E+00_JPRB ,&
&  0.31962E+00_JPRB, 0.31658E+00_JPRB, 0.31429E+00_JPRB, 0.31290E+00_JPRB, 0.31205E+00_JPRB, 0.31107E+00_JPRB, 0.30937E+00_JPRB ,&
&  0.30734E+00_JPRB, 0.30563E+00_JPRB, 0.30438E+00_JPRB, 0.30485E+00_JPRB, 0.30565E+00_JPRB, 0.30856E+00_JPRB, 0.31369E+00_JPRB ,&
&  0.29922E+00_JPRB, 0.27392E+00_JPRB, 0.28434E+00_JPRB, 0.29440E+00_JPRB, 0.30677E+00_JPRB, 0.32710E+00_JPRB, 0.34891E+00_JPRB ,&
&  0.36226E+00_JPRB, 0.37180E+00_JPRB, 0.36961E+00_JPRB, 0.36332E+00_JPRB, 0.35534E+00_JPRB, 0.34790E+00_JPRB, 0.34218E+00_JPRB ,&
&  0.33598E+00_JPRB, 0.33194E+00_JPRB, 0.32812E+00_JPRB, 0.32189E+00_JPRB, 0.31409E+00_JPRB, 0.30770E+00_JPRB, 0.30392E+00_JPRB ,&
&  0.30258E+00_JPRB, 0.30304E+00_JPRB, 0.30375E+00_JPRB, 0.30487E+00_JPRB, 0.30698E+00_JPRB, 0.31133E+00_JPRB, 0.31666E+00_JPRB ,&
&  0.32100E+00_JPRB, 0.32518E+00_JPRB, 0.32981E+00_JPRB, 0.33414E+00_JPRB, 0.33791E+00_JPRB, 0.33883E+00_JPRB, 0.33789E+00_JPRB ,&
&  0.33420E+00_JPRB, 0.32825E+00_JPRB, 0.32211E+00_JPRB, 0.31648E+00_JPRB, 0.31113E+00_JPRB, 0.30552E+00_JPRB, 0.29991E+00_JPRB ,&
&  0.29523E+00_JPRB, 0.29069E+00_JPRB, 0.28705E+00_JPRB, 0.28573E+00_JPRB, 0.28599E+00_JPRB, 0.28798E+00_JPRB, 0.29024E+00_JPRB ,&
&  0.29219E+00_JPRB, 0.29419E+00_JPRB, 0.29563E+00_JPRB, 0.29878E+00_JPRB, 0.30248E+00_JPRB, 0.30715E+00_JPRB, 0.31228E+00_JPRB ,&
&  0.31825E+00_JPRB, 0.32396E+00_JPRB, 0.33080E+00_JPRB, 0.34359E+00_JPRB, 0.35609E+00_JPRB, 0.36743E+00_JPRB, 0.37542E+00_JPRB ,&
&  0.37782E+00_JPRB, 0.37945E+00_JPRB, 0.38242E+00_JPRB, 0.38403E+00_JPRB, 0.38992E+00_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.43662E+00_JPRB, 0.43453E+00_JPRB, 0.42192E+00_JPRB, 0.40876E+00_JPRB, 0.39610E+00_JPRB, 0.38443E+00_JPRB, 0.37668E+00_JPRB ,&
&  0.37195E+00_JPRB, 0.36476E+00_JPRB, 0.35860E+00_JPRB, 0.35473E+00_JPRB, 0.35024E+00_JPRB, 0.34600E+00_JPRB, 0.34090E+00_JPRB ,&
&  0.33708E+00_JPRB, 0.33509E+00_JPRB, 0.33391E+00_JPRB, 0.33222E+00_JPRB, 0.32985E+00_JPRB, 0.32640E+00_JPRB, 0.32260E+00_JPRB ,&
&  0.31867E+00_JPRB, 0.31574E+00_JPRB, 0.31309E+00_JPRB, 0.31213E+00_JPRB, 0.31468E+00_JPRB, 0.31970E+00_JPRB, 0.32454E+00_JPRB ,&
&  0.33991E+00_JPRB, 0.29435E+00_JPRB, 0.30461E+00_JPRB, 0.31816E+00_JPRB, 0.33459E+00_JPRB, 0.35984E+00_JPRB, 0.38194E+00_JPRB ,&
&  0.39680E+00_JPRB, 0.41148E+00_JPRB, 0.41185E+00_JPRB, 0.40696E+00_JPRB, 0.39492E+00_JPRB, 0.38170E+00_JPRB, 0.36983E+00_JPRB ,&
&  0.36198E+00_JPRB, 0.35637E+00_JPRB, 0.34919E+00_JPRB, 0.34081E+00_JPRB, 0.33277E+00_JPRB, 0.32712E+00_JPRB, 0.32505E+00_JPRB ,&
&  0.32558E+00_JPRB, 0.32625E+00_JPRB, 0.32634E+00_JPRB, 0.32874E+00_JPRB, 0.33349E+00_JPRB, 0.33960E+00_JPRB, 0.34657E+00_JPRB ,&
&  0.35471E+00_JPRB, 0.36481E+00_JPRB, 0.37378E+00_JPRB, 0.38122E+00_JPRB, 0.38442E+00_JPRB, 0.38199E+00_JPRB, 0.37223E+00_JPRB ,&
&  0.35974E+00_JPRB, 0.34845E+00_JPRB, 0.34080E+00_JPRB, 0.33638E+00_JPRB, 0.33285E+00_JPRB, 0.33018E+00_JPRB, 0.32725E+00_JPRB ,&
&  0.32311E+00_JPRB, 0.31871E+00_JPRB, 0.31574E+00_JPRB, 0.31442E+00_JPRB, 0.31358E+00_JPRB, 0.31416E+00_JPRB, 0.31553E+00_JPRB ,&
&  0.31723E+00_JPRB, 0.31925E+00_JPRB, 0.32203E+00_JPRB, 0.32653E+00_JPRB, 0.33208E+00_JPRB, 0.33820E+00_JPRB, 0.34326E+00_JPRB ,&
&  0.34907E+00_JPRB, 0.35523E+00_JPRB, 0.36219E+00_JPRB, 0.37740E+00_JPRB, 0.39153E+00_JPRB, 0.40162E+00_JPRB, 0.41058E+00_JPRB ,&
&  0.41578E+00_JPRB, 0.42489E+00_JPRB, 0.36864E+00_JPRB, 0.40561E+00_JPRB, 0.42809E+00_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.50199E+00_JPRB, 0.49274E+00_JPRB, 0.47192E+00_JPRB, 0.44940E+00_JPRB, 0.42979E+00_JPRB, 0.41212E+00_JPRB, 0.39889E+00_JPRB ,&
&  0.39096E+00_JPRB, 0.38581E+00_JPRB, 0.38062E+00_JPRB, 0.37596E+00_JPRB, 0.37048E+00_JPRB, 0.36582E+00_JPRB, 0.35983E+00_JPRB ,&
&  0.35584E+00_JPRB, 0.35362E+00_JPRB, 0.35176E+00_JPRB, 0.34873E+00_JPRB, 0.34511E+00_JPRB, 0.34095E+00_JPRB, 0.33555E+00_JPRB ,&
&  0.33093E+00_JPRB, 0.32850E+00_JPRB, 0.32477E+00_JPRB, 0.32381E+00_JPRB, 0.32956E+00_JPRB, 0.33622E+00_JPRB, 0.34420E+00_JPRB ,&
&  0.36462E+00_JPRB, 0.32178E+00_JPRB, 0.32958E+00_JPRB, 0.35777E+00_JPRB, 0.37671E+00_JPRB, 0.40444E+00_JPRB, 0.42759E+00_JPRB ,&
&  0.43960E+00_JPRB, 0.44999E+00_JPRB, 0.44885E+00_JPRB, 0.44179E+00_JPRB, 0.42536E+00_JPRB, 0.40942E+00_JPRB, 0.39621E+00_JPRB ,&
&  0.38472E+00_JPRB, 0.37487E+00_JPRB, 0.36648E+00_JPRB, 0.35938E+00_JPRB, 0.35268E+00_JPRB, 0.34988E+00_JPRB, 0.35097E+00_JPRB ,&
&  0.35286E+00_JPRB, 0.35537E+00_JPRB, 0.35918E+00_JPRB, 0.36539E+00_JPRB, 0.37127E+00_JPRB, 0.37783E+00_JPRB, 0.39132E+00_JPRB ,&
&  0.41178E+00_JPRB, 0.42682E+00_JPRB, 0.43734E+00_JPRB, 0.43881E+00_JPRB, 0.43193E+00_JPRB, 0.41952E+00_JPRB, 0.40231E+00_JPRB ,&
&  0.38501E+00_JPRB, 0.37176E+00_JPRB, 0.36297E+00_JPRB, 0.35740E+00_JPRB, 0.35395E+00_JPRB, 0.35065E+00_JPRB, 0.34808E+00_JPRB ,&
&  0.34546E+00_JPRB, 0.34337E+00_JPRB, 0.34350E+00_JPRB, 0.34601E+00_JPRB, 0.34941E+00_JPRB, 0.35256E+00_JPRB, 0.35452E+00_JPRB ,&
&  0.35579E+00_JPRB, 0.35705E+00_JPRB, 0.36064E+00_JPRB, 0.36577E+00_JPRB, 0.37125E+00_JPRB, 0.37494E+00_JPRB, 0.37940E+00_JPRB ,&
&  0.38521E+00_JPRB, 0.39203E+00_JPRB, 0.40357E+00_JPRB, 0.41634E+00_JPRB, 0.43725E+00_JPRB, 0.44958E+00_JPRB, 0.45901E+00_JPRB ,&
&  0.46995E+00_JPRB, 0.47439E+00_JPRB, 0.48157E+00_JPRB, 0.43055E+00_JPRB, 0.49116E+00_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.57621E+00_JPRB, 0.54381E+00_JPRB, 0.50079E+00_JPRB, 0.47083E+00_JPRB, 0.44943E+00_JPRB, 0.42724E+00_JPRB, 0.41234E+00_JPRB ,&
&  0.40243E+00_JPRB, 0.39804E+00_JPRB, 0.39451E+00_JPRB, 0.38956E+00_JPRB, 0.38464E+00_JPRB, 0.38194E+00_JPRB, 0.37835E+00_JPRB ,&
&  0.37420E+00_JPRB, 0.37005E+00_JPRB, 0.36569E+00_JPRB, 0.36241E+00_JPRB, 0.35766E+00_JPRB, 0.35370E+00_JPRB, 0.34970E+00_JPRB ,&
&  0.34478E+00_JPRB, 0.34051E+00_JPRB, 0.33575E+00_JPRB, 0.33978E+00_JPRB, 0.34753E+00_JPRB, 0.35302E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.38133E+00_JPRB, 0.35194E+00_JPRB, 0.37623E+00_JPRB, 0.40108E+00_JPRB, 0.42075E+00_JPRB, 0.44356E+00_JPRB, 0.46088E+00_JPRB ,&
&  0.46541E+00_JPRB, 0.47884E+00_JPRB, 0.48196E+00_JPRB, 0.47222E+00_JPRB, 0.46392E+00_JPRB, 0.44282E+00_JPRB, 0.42325E+00_JPRB ,&
&  0.40899E+00_JPRB, 0.39954E+00_JPRB, 0.39092E+00_JPRB, 0.38078E+00_JPRB, 0.37455E+00_JPRB, 0.37338E+00_JPRB, 0.37553E+00_JPRB ,&
&  0.37862E+00_JPRB, 0.38409E+00_JPRB, 0.38960E+00_JPRB, 0.39509E+00_JPRB, 0.39939E+00_JPRB, 0.41659E+00_JPRB, 0.44992E+00_JPRB ,&
&  0.47701E+00_JPRB, 0.49320E+00_JPRB, 0.49924E+00_JPRB, 0.49925E+00_JPRB, 0.48751E+00_JPRB, 0.47184E+00_JPRB, 0.45381E+00_JPRB ,&
&  0.43614E+00_JPRB, 0.41804E+00_JPRB, 0.40123E+00_JPRB, 0.38833E+00_JPRB, 0.37786E+00_JPRB, 0.37061E+00_JPRB, 0.36693E+00_JPRB ,&
&  0.36670E+00_JPRB, 0.36932E+00_JPRB, 0.37366E+00_JPRB, 0.37828E+00_JPRB, 0.38146E+00_JPRB, 0.38225E+00_JPRB, 0.38202E+00_JPRB ,&
&  0.38176E+00_JPRB, 0.38201E+00_JPRB, 0.38440E+00_JPRB, 0.38869E+00_JPRB, 0.39410E+00_JPRB, 0.39960E+00_JPRB, 0.40553E+00_JPRB ,&
&  0.41215E+00_JPRB, 0.41908E+00_JPRB, 0.42999E+00_JPRB, 0.45782E+00_JPRB, 0.47186E+00_JPRB, 0.48973E+00_JPRB, 0.50528E+00_JPRB ,&
&  0.52569E+00_JPRB, 0.53644E+00_JPRB, 0.53382E+00_JPRB, 0.52248E+00_JPRB, 0.55085E+00_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.64211E+00_JPRB, 0.55651E+00_JPRB, 0.50423E+00_JPRB, 0.47826E+00_JPRB, 0.46870E+00_JPRB, 0.45324E+00_JPRB, 0.43778E+00_JPRB ,&
&  0.42412E+00_JPRB, 0.41533E+00_JPRB, 0.41073E+00_JPRB, 0.40487E+00_JPRB, 0.39872E+00_JPRB, 0.39387E+00_JPRB, 0.38839E+00_JPRB ,&
&  0.38219E+00_JPRB, 0.37854E+00_JPRB, 0.37538E+00_JPRB, 0.37325E+00_JPRB, 0.37157E+00_JPRB, 0.36750E+00_JPRB, 0.36254E+00_JPRB ,&
&  0.35876E+00_JPRB, 0.35398E+00_JPRB, 0.35363E+00_JPRB, 0.35978E+00_JPRB, 0.36479E+00_JPRB, 0.36694E+00_JPRB, 0.37734E+00_JPRB ,&
&  0.40062E+00_JPRB, 0.37134E+00_JPRB, 0.40423E+00_JPRB, 0.43284E+00_JPRB, 0.46292E+00_JPRB, 0.51336E+00_JPRB, 0.54865E+00_JPRB ,&
&  0.55910E+00_JPRB, 0.57718E+00_JPRB, 0.57694E+00_JPRB, 0.55155E+00_JPRB, 0.51917E+00_JPRB, 0.49702E+00_JPRB, 0.48017E+00_JPRB ,&
&  0.46576E+00_JPRB, 0.44849E+00_JPRB, 0.43061E+00_JPRB, 0.41639E+00_JPRB, 0.40872E+00_JPRB, 0.40639E+00_JPRB, 0.40626E+00_JPRB ,&
&  0.40750E+00_JPRB, 0.41175E+00_JPRB, 0.41548E+00_JPRB, 0.42104E+00_JPRB, 0.44134E+00_JPRB, 0.49370E+00_JPRB, 0.56663E+00_JPRB ,&
&  0.60112E+00_JPRB, 0.61361E+00_JPRB, 0.61784E+00_JPRB, 0.61374E+00_JPRB, 0.58712E+00_JPRB, 0.55863E+00_JPRB, 0.53081E+00_JPRB ,&
&  0.50459E+00_JPRB, 0.47695E+00_JPRB, 0.45411E+00_JPRB, 0.43589E+00_JPRB, 0.42107E+00_JPRB, 0.41213E+00_JPRB, 0.40878E+00_JPRB ,&
&  0.40797E+00_JPRB, 0.40779E+00_JPRB, 0.40756E+00_JPRB, 0.40589E+00_JPRB, 0.40255E+00_JPRB, 0.40146E+00_JPRB, 0.40100E+00_JPRB ,&
&  0.40116E+00_JPRB, 0.40286E+00_JPRB, 0.40690E+00_JPRB, 0.41260E+00_JPRB, 0.42075E+00_JPRB, 0.42639E+00_JPRB, 0.43402E+00_JPRB ,&
&  0.44307E+00_JPRB, 0.45605E+00_JPRB, 0.45576E+00_JPRB, 0.47326E+00_JPRB, 0.52420E+00_JPRB, 0.54446E+00_JPRB, 0.56858E+00_JPRB ,&
&  0.59410E+00_JPRB, 0.59323E+00_JPRB, 0.57535E+00_JPRB, 0.57823E+00_JPRB, 0.70589E+00_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.52342E+00_JPRB, 0.50082E+00_JPRB, 0.48647E+00_JPRB, 0.48097E+00_JPRB, 0.47850E+00_JPRB, 0.46880E+00_JPRB, 0.45234E+00_JPRB ,&
&  0.43658E+00_JPRB, 0.42325E+00_JPRB, 0.41363E+00_JPRB, 0.40663E+00_JPRB, 0.40224E+00_JPRB, 0.39788E+00_JPRB, 0.39298E+00_JPRB ,&
&  0.38764E+00_JPRB, 0.38553E+00_JPRB, 0.38417E+00_JPRB, 0.38146E+00_JPRB, 0.37444E+00_JPRB, 0.36866E+00_JPRB, 0.36688E+00_JPRB ,&
&  0.36585E+00_JPRB, 0.36687E+00_JPRB, 0.37207E+00_JPRB, 0.37787E+00_JPRB, 0.38015E+00_JPRB, 0.38152E+00_JPRB, 0.39033E+00_JPRB ,&
&  0.40997E+00_JPRB, 0.40403E+00_JPRB, 0.42589E+00_JPRB, 0.47086E+00_JPRB, 0.52540E+00_JPRB, 0.62104E+00_JPRB, 0.72072E+00_JPRB ,&
&  0.75361E+00_JPRB, 0.73435E+00_JPRB, 0.68431E+00_JPRB, 0.62649E+00_JPRB, 0.58008E+00_JPRB, 0.55338E+00_JPRB, 0.53237E+00_JPRB ,&
&  0.51105E+00_JPRB, 0.48960E+00_JPRB, 0.47147E+00_JPRB, 0.45787E+00_JPRB, 0.44874E+00_JPRB, 0.44538E+00_JPRB, 0.44513E+00_JPRB ,&
&  0.44484E+00_JPRB, 0.44358E+00_JPRB, 0.44446E+00_JPRB, 0.46226E+00_JPRB, 0.54025E+00_JPRB, 0.65186E+00_JPRB, 0.74963E+00_JPRB ,&
&  0.78906E+00_JPRB, 0.81948E+00_JPRB, 0.81282E+00_JPRB, 0.77596E+00_JPRB, 0.71814E+00_JPRB, 0.66028E+00_JPRB, 0.59958E+00_JPRB ,&
&  0.55533E+00_JPRB, 0.52201E+00_JPRB, 0.49818E+00_JPRB, 0.47849E+00_JPRB, 0.46052E+00_JPRB, 0.44705E+00_JPRB, 0.43840E+00_JPRB ,&
&  0.43230E+00_JPRB, 0.42699E+00_JPRB, 0.42172E+00_JPRB, 0.41668E+00_JPRB, 0.41240E+00_JPRB, 0.40947E+00_JPRB, 0.41135E+00_JPRB ,&
&  0.41355E+00_JPRB, 0.41510E+00_JPRB, 0.42559E+00_JPRB, 0.44085E+00_JPRB, 0.45589E+00_JPRB, 0.46570E+00_JPRB, 0.47194E+00_JPRB ,&
&  0.47882E+00_JPRB, 0.48924E+00_JPRB, 0.48562E+00_JPRB, 0.49203E+00_JPRB, 0.53692E+00_JPRB, 0.59073E+00_JPRB, 0.60557E+00_JPRB ,&
&  0.60377E+00_JPRB, 0.58705E+00_JPRB, 0.57324E+00_JPRB, 0.56511E+00_JPRB, 0.55989E+00_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.52878E+00_JPRB, 0.51061E+00_JPRB, 0.50156E+00_JPRB, 0.49284E+00_JPRB, 0.48444E+00_JPRB, 0.47249E+00_JPRB, 0.45652E+00_JPRB ,&
&  0.44517E+00_JPRB, 0.43616E+00_JPRB, 0.42548E+00_JPRB, 0.41611E+00_JPRB, 0.40990E+00_JPRB, 0.40523E+00_JPRB, 0.39939E+00_JPRB ,&
&  0.39573E+00_JPRB, 0.39371E+00_JPRB, 0.38523E+00_JPRB, 0.37681E+00_JPRB, 0.37346E+00_JPRB, 0.37326E+00_JPRB, 0.37688E+00_JPRB ,&
&  0.38082E+00_JPRB, 0.38594E+00_JPRB, 0.38991E+00_JPRB, 0.39307E+00_JPRB, 0.39312E+00_JPRB, 0.39529E+00_JPRB, 0.40734E+00_JPRB ,&
&  0.42956E+00_JPRB, 0.42855E+00_JPRB, 0.50862E+00_JPRB, 0.54870E+00_JPRB, 0.62163E+00_JPRB, 0.74215E+00_JPRB, 0.87201E+00_JPRB ,&
&  0.89052E+00_JPRB, 0.82898E+00_JPRB, 0.74893E+00_JPRB, 0.68922E+00_JPRB, 0.64459E+00_JPRB, 0.61038E+00_JPRB, 0.57308E+00_JPRB ,&
&  0.54322E+00_JPRB, 0.52413E+00_JPRB, 0.50610E+00_JPRB, 0.48969E+00_JPRB, 0.48340E+00_JPRB, 0.48172E+00_JPRB, 0.47858E+00_JPRB ,&
&  0.47380E+00_JPRB, 0.47038E+00_JPRB, 0.47978E+00_JPRB, 0.52731E+00_JPRB, 0.63389E+00_JPRB, 0.73232E+00_JPRB, 0.88894E+00_JPRB ,&
&  0.10630E+01_JPRB, 0.10787E+01_JPRB, 0.10185E+01_JPRB, 0.90520E+00_JPRB, 0.78160E+00_JPRB, 0.67500E+00_JPRB, 0.60992E+00_JPRB ,&
&  0.56846E+00_JPRB, 0.53193E+00_JPRB, 0.50900E+00_JPRB, 0.49403E+00_JPRB, 0.48287E+00_JPRB, 0.47369E+00_JPRB, 0.46201E+00_JPRB ,&
&  0.44895E+00_JPRB, 0.43931E+00_JPRB, 0.43333E+00_JPRB, 0.42811E+00_JPRB, 0.42496E+00_JPRB, 0.42405E+00_JPRB, 0.42680E+00_JPRB ,&
&  0.43295E+00_JPRB, 0.44553E+00_JPRB, 0.46560E+00_JPRB, 0.48110E+00_JPRB, 0.50234E+00_JPRB, 0.52708E+00_JPRB, 0.54157E+00_JPRB ,&
&  0.52683E+00_JPRB, 0.51518E+00_JPRB, 0.50407E+00_JPRB, 0.50294E+00_JPRB, 0.55886E+00_JPRB, 0.62313E+00_JPRB, 0.61376E+00_JPRB ,&
&  0.60436E+00_JPRB, 0.59073E+00_JPRB, 0.57441E+00_JPRB, 0.55442E+00_JPRB, 0.55446E+00_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.54225E+00_JPRB, 0.52441E+00_JPRB, 0.51009E+00_JPRB, 0.49407E+00_JPRB, 0.48237E+00_JPRB, 0.47424E+00_JPRB, 0.46358E+00_JPRB ,&
&  0.45384E+00_JPRB, 0.44481E+00_JPRB, 0.43543E+00_JPRB, 0.42615E+00_JPRB, 0.41850E+00_JPRB, 0.40907E+00_JPRB, 0.40165E+00_JPRB ,&
&  0.39452E+00_JPRB, 0.38470E+00_JPRB, 0.37806E+00_JPRB, 0.37640E+00_JPRB, 0.38033E+00_JPRB, 0.38560E+00_JPRB, 0.39215E+00_JPRB ,&
&  0.39996E+00_JPRB, 0.40525E+00_JPRB, 0.40657E+00_JPRB, 0.40882E+00_JPRB, 0.40708E+00_JPRB, 0.40801E+00_JPRB, 0.42126E+00_JPRB ,&
&  0.44861E+00_JPRB, 0.41655E+00_JPRB, 0.58879E+00_JPRB, 0.68577E+00_JPRB, 0.80784E+00_JPRB, 0.92144E+00_JPRB, 0.99131E+00_JPRB ,&
&  0.10197E+01_JPRB, 0.92974E+00_JPRB, 0.83429E+00_JPRB, 0.76877E+00_JPRB, 0.71472E+00_JPRB, 0.67070E+00_JPRB, 0.63523E+00_JPRB ,&
&  0.60193E+00_JPRB, 0.57676E+00_JPRB, 0.55740E+00_JPRB, 0.52926E+00_JPRB, 0.50688E+00_JPRB, 0.49475E+00_JPRB, 0.48807E+00_JPRB ,&
&  0.48160E+00_JPRB, 0.48616E+00_JPRB, 0.51077E+00_JPRB, 0.58201E+00_JPRB, 0.73232E+00_JPRB, 0.87740E+00_JPRB, 0.95889E+00_JPRB ,&
&  0.10416E+01_JPRB, 0.11496E+01_JPRB, 0.10838E+01_JPRB, 0.94377E+00_JPRB, 0.80565E+00_JPRB, 0.70197E+00_JPRB, 0.62127E+00_JPRB ,&
&  0.57492E+00_JPRB, 0.54860E+00_JPRB, 0.53360E+00_JPRB, 0.52212E+00_JPRB, 0.50842E+00_JPRB, 0.49142E+00_JPRB, 0.47261E+00_JPRB ,&
&  0.45884E+00_JPRB, 0.45211E+00_JPRB, 0.44775E+00_JPRB, 0.44668E+00_JPRB, 0.44628E+00_JPRB, 0.44694E+00_JPRB, 0.45222E+00_JPRB ,&
&  0.46325E+00_JPRB, 0.48432E+00_JPRB, 0.50132E+00_JPRB, 0.51679E+00_JPRB, 0.53415E+00_JPRB, 0.55066E+00_JPRB, 0.56433E+00_JPRB ,&
&  0.56540E+00_JPRB, 0.55200E+00_JPRB, 0.53389E+00_JPRB, 0.53611E+00_JPRB, 0.55861E+00_JPRB, 0.61091E+00_JPRB, 0.61544E+00_JPRB ,&
&  0.61386E+00_JPRB, 0.60143E+00_JPRB, 0.57704E+00_JPRB, 0.56589E+00_JPRB, 0.56239E+00_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.55087E+00_JPRB, 0.53856E+00_JPRB, 0.52224E+00_JPRB, 0.50626E+00_JPRB, 0.49943E+00_JPRB, 0.49435E+00_JPRB, 0.48192E+00_JPRB ,&
&  0.47148E+00_JPRB, 0.46419E+00_JPRB, 0.45545E+00_JPRB, 0.44387E+00_JPRB, 0.43026E+00_JPRB, 0.41239E+00_JPRB, 0.39601E+00_JPRB ,&
&  0.38820E+00_JPRB, 0.38168E+00_JPRB, 0.38470E+00_JPRB, 0.38982E+00_JPRB, 0.39802E+00_JPRB, 0.40478E+00_JPRB, 0.41250E+00_JPRB ,&
&  0.41807E+00_JPRB, 0.42063E+00_JPRB, 0.42444E+00_JPRB, 0.42739E+00_JPRB, 0.42796E+00_JPRB, 0.43152E+00_JPRB, 0.44763E+00_JPRB ,&
&  0.49081E+00_JPRB, 0.61706E+00_JPRB, 0.56747E+00_JPRB, 0.86696E+00_JPRB, 0.10679E+01_JPRB, 0.11751E+01_JPRB, 0.11366E+01_JPRB ,&
&  0.11122E+01_JPRB, 0.10589E+01_JPRB, 0.94482E+00_JPRB, 0.85075E+00_JPRB, 0.77659E+00_JPRB, 0.72738E+00_JPRB, 0.69262E+00_JPRB ,&
&  0.66662E+00_JPRB, 0.63531E+00_JPRB, 0.59656E+00_JPRB, 0.55772E+00_JPRB, 0.53194E+00_JPRB, 0.51501E+00_JPRB, 0.50735E+00_JPRB ,&
&  0.50433E+00_JPRB, 0.51843E+00_JPRB, 0.56382E+00_JPRB, 0.71039E+00_JPRB, 0.87704E+00_JPRB, 0.99710E+00_JPRB, 0.97797E+00_JPRB ,&
&  0.93778E+00_JPRB, 0.10601E+01_JPRB, 0.10056E+01_JPRB, 0.86732E+00_JPRB, 0.76266E+00_JPRB, 0.67396E+00_JPRB, 0.61180E+00_JPRB ,&
&  0.57403E+00_JPRB, 0.55119E+00_JPRB, 0.53803E+00_JPRB, 0.52772E+00_JPRB, 0.51478E+00_JPRB, 0.50071E+00_JPRB, 0.48986E+00_JPRB ,&
&  0.48273E+00_JPRB, 0.47684E+00_JPRB, 0.47261E+00_JPRB, 0.47248E+00_JPRB, 0.47111E+00_JPRB, 0.47353E+00_JPRB, 0.48012E+00_JPRB ,&
&  0.49277E+00_JPRB, 0.51846E+00_JPRB, 0.53022E+00_JPRB, 0.54565E+00_JPRB, 0.55082E+00_JPRB, 0.55158E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.57774E+00_JPRB, 0.57665E+00_JPRB, 0.56676E+00_JPRB, 0.56649E+00_JPRB, 0.58205E+00_JPRB, 0.61648E+00_JPRB, 0.61977E+00_JPRB ,&
&  0.62049E+00_JPRB, 0.60932E+00_JPRB, 0.59183E+00_JPRB, 0.57665E+00_JPRB, 0.56644E+00_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.56010E+00_JPRB, 0.54609E+00_JPRB, 0.53441E+00_JPRB, 0.53000E+00_JPRB, 0.52360E+00_JPRB, 0.51716E+00_JPRB, 0.50407E+00_JPRB ,&
&  0.48996E+00_JPRB, 0.48156E+00_JPRB, 0.47008E+00_JPRB, 0.45932E+00_JPRB, 0.43765E+00_JPRB, 0.41086E+00_JPRB, 0.40081E+00_JPRB ,&
&  0.39482E+00_JPRB, 0.39632E+00_JPRB, 0.40565E+00_JPRB, 0.40950E+00_JPRB, 0.41385E+00_JPRB, 0.42269E+00_JPRB, 0.43133E+00_JPRB ,&
&  0.43748E+00_JPRB, 0.43765E+00_JPRB, 0.44225E+00_JPRB, 0.44839E+00_JPRB, 0.45349E+00_JPRB, 0.47129E+00_JPRB, 0.49242E+00_JPRB ,&
&  0.55559E+00_JPRB, 0.67469E+00_JPRB, 0.58593E+00_JPRB, 0.11099E+01_JPRB, 0.13587E+01_JPRB, 0.14002E+01_JPRB, 0.13173E+01_JPRB ,&
&  0.12628E+01_JPRB, 0.12309E+01_JPRB, 0.10461E+01_JPRB, 0.91168E+00_JPRB, 0.81324E+00_JPRB, 0.76009E+00_JPRB, 0.72635E+00_JPRB ,&
&  0.69830E+00_JPRB, 0.66638E+00_JPRB, 0.62337E+00_JPRB, 0.58448E+00_JPRB, 0.55929E+00_JPRB, 0.54193E+00_JPRB, 0.53873E+00_JPRB ,&
&  0.54075E+00_JPRB, 0.56568E+00_JPRB, 0.65088E+00_JPRB, 0.84239E+00_JPRB, 0.10059E+01_JPRB, 0.10405E+01_JPRB, 0.94792E+00_JPRB ,&
&  0.91737E+00_JPRB, 0.98312E+00_JPRB, 0.91713E+00_JPRB, 0.78820E+00_JPRB, 0.69847E+00_JPRB, 0.64630E+00_JPRB, 0.59414E+00_JPRB ,&
&  0.56177E+00_JPRB, 0.54004E+00_JPRB, 0.53029E+00_JPRB, 0.52328E+00_JPRB, 0.51751E+00_JPRB, 0.51518E+00_JPRB, 0.51316E+00_JPRB ,&
&  0.51085E+00_JPRB, 0.50420E+00_JPRB, 0.49853E+00_JPRB, 0.49074E+00_JPRB, 0.49324E+00_JPRB, 0.49797E+00_JPRB, 0.50504E+00_JPRB ,&
&  0.52598E+00_JPRB, 0.56199E+00_JPRB, 0.56539E+00_JPRB, 0.57937E+00_JPRB, 0.57625E+00_JPRB, 0.56196E+00_JPRB, 0.56594E+00_JPRB ,&
&  0.58384E+00_JPRB, 0.58694E+00_JPRB, 0.59528E+00_JPRB, 0.58679E+00_JPRB, 0.59815E+00_JPRB, 0.62184E+00_JPRB, 0.62686E+00_JPRB ,&
&  0.63390E+00_JPRB, 0.62663E+00_JPRB, 0.61289E+00_JPRB, 0.58960E+00_JPRB, 0.57478E+00_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.55797E+00_JPRB, 0.54214E+00_JPRB, 0.53704E+00_JPRB, 0.53612E+00_JPRB, 0.53192E+00_JPRB, 0.52259E+00_JPRB, 0.50695E+00_JPRB ,&
&  0.48808E+00_JPRB, 0.47506E+00_JPRB, 0.46071E+00_JPRB, 0.44329E+00_JPRB, 0.41749E+00_JPRB, 0.39295E+00_JPRB, 0.39276E+00_JPRB ,&
&  0.40762E+00_JPRB, 0.41669E+00_JPRB, 0.42485E+00_JPRB, 0.43350E+00_JPRB, 0.44267E+00_JPRB, 0.45427E+00_JPRB, 0.46279E+00_JPRB ,&
&  0.46508E+00_JPRB, 0.46942E+00_JPRB, 0.47957E+00_JPRB, 0.49226E+00_JPRB, 0.50856E+00_JPRB, 0.53275E+00_JPRB, 0.56369E+00_JPRB ,&
&  0.64164E+00_JPRB, 0.72332E+00_JPRB, 0.69852E+00_JPRB, 0.12010E+01_JPRB, 0.15175E+01_JPRB, 0.14435E+01_JPRB, 0.14060E+01_JPRB ,&
&  0.13037E+01_JPRB, 0.11090E+01_JPRB, 0.10613E+01_JPRB, 0.91551E+00_JPRB, 0.82062E+00_JPRB, 0.77139E+00_JPRB, 0.74003E+00_JPRB ,&
&  0.71454E+00_JPRB, 0.68358E+00_JPRB, 0.64377E+00_JPRB, 0.61856E+00_JPRB, 0.59543E+00_JPRB, 0.58278E+00_JPRB, 0.58347E+00_JPRB ,&
&  0.59170E+00_JPRB, 0.63528E+00_JPRB, 0.77739E+00_JPRB, 0.10007E+01_JPRB, 0.10389E+01_JPRB, 0.98717E+00_JPRB, 0.86552E+00_JPRB ,&
&  0.86553E+00_JPRB, 0.86243E+00_JPRB, 0.80845E+00_JPRB, 0.71424E+00_JPRB, 0.63244E+00_JPRB, 0.63089E+00_JPRB, 0.57957E+00_JPRB ,&
&  0.55189E+00_JPRB, 0.53468E+00_JPRB, 0.52799E+00_JPRB, 0.52433E+00_JPRB, 0.52484E+00_JPRB, 0.52586E+00_JPRB, 0.52552E+00_JPRB ,&
&  0.52620E+00_JPRB, 0.52746E+00_JPRB, 0.52162E+00_JPRB, 0.52175E+00_JPRB, 0.52540E+00_JPRB, 0.53174E+00_JPRB, 0.54603E+00_JPRB ,&
&  0.57926E+00_JPRB, 0.61542E+00_JPRB, 0.62163E+00_JPRB, 0.62961E+00_JPRB, 0.61611E+00_JPRB, 0.59967E+00_JPRB, 0.59645E+00_JPRB ,&
&  0.59649E+00_JPRB, 0.61674E+00_JPRB, 0.63119E+00_JPRB, 0.62614E+00_JPRB, 0.62919E+00_JPRB, 0.63770E+00_JPRB, 0.64888E+00_JPRB ,&
&  0.66101E+00_JPRB, 0.65572E+00_JPRB, 0.65470E+00_JPRB, 0.60037E+00_JPRB, 0.58112E+00_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.53728E+00_JPRB, 0.52477E+00_JPRB, 0.51101E+00_JPRB, 0.50690E+00_JPRB, 0.50208E+00_JPRB, 0.48790E+00_JPRB, 0.46989E+00_JPRB ,&
&  0.44932E+00_JPRB, 0.43399E+00_JPRB, 0.41435E+00_JPRB, 0.39813E+00_JPRB, 0.39768E+00_JPRB, 0.40942E+00_JPRB, 0.43273E+00_JPRB ,&
&  0.45614E+00_JPRB, 0.46993E+00_JPRB, 0.48055E+00_JPRB, 0.49105E+00_JPRB, 0.50061E+00_JPRB, 0.50561E+00_JPRB, 0.50537E+00_JPRB ,&
&  0.51072E+00_JPRB, 0.52415E+00_JPRB, 0.54427E+00_JPRB, 0.56671E+00_JPRB, 0.59333E+00_JPRB, 0.62941E+00_JPRB, 0.68388E+00_JPRB ,&
&  0.78509E+00_JPRB, 0.82349E+00_JPRB, 0.50336E+00_JPRB, 0.94236E+00_JPRB, 0.13491E+01_JPRB, 0.13369E+01_JPRB, 0.12558E+01_JPRB ,&
&  0.11435E+01_JPRB, 0.96876E+00_JPRB, 0.93220E+00_JPRB, 0.88700E+00_JPRB, 0.80271E+00_JPRB, 0.76267E+00_JPRB, 0.74050E+00_JPRB ,&
&  0.72702E+00_JPRB, 0.70376E+00_JPRB, 0.68415E+00_JPRB, 0.66906E+00_JPRB, 0.64861E+00_JPRB, 0.63527E+00_JPRB, 0.63368E+00_JPRB ,&
&  0.65323E+00_JPRB, 0.73483E+00_JPRB, 0.93463E+00_JPRB, 0.10241E+01_JPRB, 0.10342E+01_JPRB, 0.90957E+00_JPRB, 0.76539E+00_JPRB ,&
&  0.71858E+00_JPRB, 0.75496E+00_JPRB, 0.73297E+00_JPRB, 0.66945E+00_JPRB, 0.60432E+00_JPRB, 0.57041E+00_JPRB, 0.57461E+00_JPRB ,&
&  0.54886E+00_JPRB, 0.53192E+00_JPRB, 0.52336E+00_JPRB, 0.51861E+00_JPRB, 0.51670E+00_JPRB, 0.51728E+00_JPRB, 0.52093E+00_JPRB ,&
&  0.53004E+00_JPRB, 0.53452E+00_JPRB, 0.54166E+00_JPRB, 0.55264E+00_JPRB, 0.56331E+00_JPRB, 0.57783E+00_JPRB, 0.60488E+00_JPRB ,&
&  0.64452E+00_JPRB, 0.67479E+00_JPRB, 0.69782E+00_JPRB, 0.68651E+00_JPRB, 0.66231E+00_JPRB, 0.64884E+00_JPRB, 0.65581E+00_JPRB ,&
&  0.65565E+00_JPRB, 0.66730E+00_JPRB, 0.64814E+00_JPRB, 0.63509E+00_JPRB, 0.66290E+00_JPRB, 0.67094E+00_JPRB, 0.71160E+00_JPRB ,&
&  0.76373E+00_JPRB, 0.84423E+00_JPRB, 0.86584E+00_JPRB, 0.60253E+00_JPRB, 0.56438E+00_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.49917E+00_JPRB, 0.48132E+00_JPRB, 0.45592E+00_JPRB, 0.44620E+00_JPRB, 0.45057E+00_JPRB, 0.44462E+00_JPRB, 0.43884E+00_JPRB ,&
&  0.43301E+00_JPRB, 0.42866E+00_JPRB, 0.42531E+00_JPRB, 0.43484E+00_JPRB, 0.46020E+00_JPRB, 0.49454E+00_JPRB, 0.52558E+00_JPRB ,&
&  0.54718E+00_JPRB, 0.55745E+00_JPRB, 0.56041E+00_JPRB, 0.56520E+00_JPRB, 0.56876E+00_JPRB, 0.57210E+00_JPRB, 0.57694E+00_JPRB ,&
&  0.59576E+00_JPRB, 0.62497E+00_JPRB, 0.66152E+00_JPRB, 0.69923E+00_JPRB, 0.74049E+00_JPRB, 0.79301E+00_JPRB, 0.87518E+00_JPRB ,&
&  0.10001E+01_JPRB, 0.77482E+00_JPRB, 0.59802E+00_JPRB, 0.12107E+01_JPRB, 0.13686E+01_JPRB, 0.13305E+01_JPRB, 0.11347E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.92668E+00_JPRB, 0.85982E+00_JPRB, 0.87277E+00_JPRB, 0.78838E+00_JPRB, 0.75438E+00_JPRB, 0.74546E+00_JPRB ,&
&  0.74012E+00_JPRB, 0.73632E+00_JPRB, 0.72880E+00_JPRB, 0.71599E+00_JPRB, 0.69958E+00_JPRB, 0.69095E+00_JPRB, 0.70089E+00_JPRB ,&
&  0.75747E+00_JPRB, 0.88190E+00_JPRB, 0.10773E+01_JPRB, 0.10293E+01_JPRB, 0.10272E+01_JPRB, 0.84906E+00_JPRB, 0.72465E+00_JPRB ,&
&  0.67863E+00_JPRB, 0.69326E+00_JPRB, 0.68909E+00_JPRB, 0.65818E+00_JPRB, 0.60598E+00_JPRB, 0.55755E+00_JPRB, 0.57004E+00_JPRB ,&
&  0.54181E+00_JPRB, 0.52401E+00_JPRB, 0.50972E+00_JPRB, 0.50409E+00_JPRB, 0.50812E+00_JPRB, 0.51514E+00_JPRB, 0.51827E+00_JPRB ,&
&  0.52893E+00_JPRB, 0.53696E+00_JPRB, 0.55605E+00_JPRB, 0.57642E+00_JPRB, 0.59768E+00_JPRB, 0.62434E+00_JPRB, 0.66434E+00_JPRB ,&
&  0.70015E+00_JPRB, 0.72502E+00_JPRB, 0.74072E+00_JPRB, 0.74499E+00_JPRB, 0.72489E+00_JPRB, 0.69835E+00_JPRB, 0.70070E+00_JPRB ,&
&  0.69145E+00_JPRB, 0.69456E+00_JPRB, 0.68283E+00_JPRB, 0.69318E+00_JPRB, 0.70621E+00_JPRB, 0.72177E+00_JPRB, 0.75833E+00_JPRB ,&
&  0.80360E+00_JPRB, 0.81333E+00_JPRB, 0.74120E+00_JPRB, 0.56073E+00_JPRB, 0.51947E+00_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.48290E+00_JPRB, 0.45973E+00_JPRB, 0.44905E+00_JPRB, 0.44801E+00_JPRB, 0.45420E+00_JPRB, 0.45749E+00_JPRB, 0.45681E+00_JPRB ,&
&  0.45894E+00_JPRB, 0.46985E+00_JPRB, 0.48340E+00_JPRB, 0.52305E+00_JPRB, 0.56251E+00_JPRB, 0.59014E+00_JPRB, 0.60707E+00_JPRB ,&
&  0.62280E+00_JPRB, 0.63149E+00_JPRB, 0.63505E+00_JPRB, 0.64053E+00_JPRB, 0.64526E+00_JPRB, 0.65770E+00_JPRB, 0.68221E+00_JPRB ,&
&  0.71937E+00_JPRB, 0.75790E+00_JPRB, 0.80460E+00_JPRB, 0.85964E+00_JPRB, 0.92474E+00_JPRB, 0.98196E+00_JPRB, 0.10310E+01_JPRB ,&
&  0.83097E+00_JPRB, 0.51845E+00_JPRB, 0.12153E+01_JPRB, 0.14314E+01_JPRB, 0.12598E+01_JPRB, 0.11731E+01_JPRB, 0.10635E+01_JPRB ,&
&  0.93290E+00_JPRB, 0.88025E+00_JPRB, 0.84143E+00_JPRB, 0.85454E+00_JPRB, 0.77860E+00_JPRB, 0.75267E+00_JPRB, 0.74544E+00_JPRB ,&
&  0.74655E+00_JPRB, 0.75457E+00_JPRB, 0.75324E+00_JPRB, 0.74410E+00_JPRB, 0.74123E+00_JPRB, 0.75524E+00_JPRB, 0.80596E+00_JPRB ,&
&  0.90708E+00_JPRB, 0.10460E+01_JPRB, 0.11606E+01_JPRB, 0.10028E+01_JPRB, 0.10197E+01_JPRB, 0.87173E+00_JPRB, 0.72435E+00_JPRB ,&
&  0.68011E+00_JPRB, 0.65302E+00_JPRB, 0.64470E+00_JPRB, 0.66391E+00_JPRB, 0.62779E+00_JPRB, 0.56786E+00_JPRB, 0.57077E+00_JPRB ,&
&  0.53799E+00_JPRB, 0.51380E+00_JPRB, 0.50473E+00_JPRB, 0.50733E+00_JPRB, 0.51576E+00_JPRB, 0.52607E+00_JPRB, 0.53631E+00_JPRB ,&
&  0.54094E+00_JPRB, 0.55365E+00_JPRB, 0.56852E+00_JPRB, 0.59453E+00_JPRB, 0.62414E+00_JPRB, 0.66928E+00_JPRB, 0.71018E+00_JPRB ,&
&  0.73767E+00_JPRB, 0.76253E+00_JPRB, 0.78672E+00_JPRB, 0.80581E+00_JPRB, 0.82180E+00_JPRB, 0.79210E+00_JPRB, 0.76095E+00_JPRB ,&
&  0.75385E+00_JPRB, 0.75414E+00_JPRB, 0.76952E+00_JPRB, 0.78956E+00_JPRB, 0.75961E+00_JPRB, 0.74751E+00_JPRB, 0.77383E+00_JPRB ,&
&  0.77744E+00_JPRB, 0.74265E+00_JPRB, 0.64546E+00_JPRB, 0.54033E+00_JPRB, 0.50374E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.52578E+00_JPRB, 0.51308E+00_JPRB, 0.49883E+00_JPRB, 0.50069E+00_JPRB, 0.50307E+00_JPRB, 0.51964E+00_JPRB, 0.51996E+00_JPRB ,&
&  0.53267E+00_JPRB, 0.57111E+00_JPRB, 0.61638E+00_JPRB, 0.63810E+00_JPRB, 0.65696E+00_JPRB, 0.67587E+00_JPRB, 0.68979E+00_JPRB ,&
&  0.70029E+00_JPRB, 0.71070E+00_JPRB, 0.72141E+00_JPRB, 0.73433E+00_JPRB, 0.75271E+00_JPRB, 0.77757E+00_JPRB, 0.81539E+00_JPRB ,&
&  0.85053E+00_JPRB, 0.88959E+00_JPRB, 0.94040E+00_JPRB, 0.10053E+01_JPRB, 0.10757E+01_JPRB, 0.11127E+01_JPRB, 0.11384E+01_JPRB ,&
&  0.75840E+00_JPRB, 0.11816E+01_JPRB, 0.13726E+01_JPRB, 0.12773E+01_JPRB, 0.11523E+01_JPRB, 0.10713E+01_JPRB, 0.10059E+01_JPRB ,&
&  0.92541E+00_JPRB, 0.85041E+00_JPRB, 0.81886E+00_JPRB, 0.84612E+00_JPRB, 0.77856E+00_JPRB, 0.76389E+00_JPRB, 0.75978E+00_JPRB ,&
&  0.77952E+00_JPRB, 0.79158E+00_JPRB, 0.79158E+00_JPRB, 0.80203E+00_JPRB, 0.82292E+00_JPRB, 0.87504E+00_JPRB, 0.95804E+00_JPRB ,&
&  0.10636E+01_JPRB, 0.11572E+01_JPRB, 0.12019E+01_JPRB, 0.10092E+01_JPRB, 0.10296E+01_JPRB, 0.92450E+00_JPRB, 0.76264E+00_JPRB ,&
&  0.66395E+00_JPRB, 0.64893E+00_JPRB, 0.66825E+00_JPRB, 0.69386E+00_JPRB, 0.65097E+00_JPRB, 0.60897E+00_JPRB, 0.58046E+00_JPRB ,&
&  0.54665E+00_JPRB, 0.52398E+00_JPRB, 0.52332E+00_JPRB, 0.52254E+00_JPRB, 0.52992E+00_JPRB, 0.53616E+00_JPRB, 0.54892E+00_JPRB ,&
&  0.55583E+00_JPRB, 0.57302E+00_JPRB, 0.59474E+00_JPRB, 0.61831E+00_JPRB, 0.64941E+00_JPRB, 0.68731E+00_JPRB, 0.72984E+00_JPRB ,&
&  0.76167E+00_JPRB, 0.79278E+00_JPRB, 0.81995E+00_JPRB, 0.84291E+00_JPRB, 0.85216E+00_JPRB, 0.85136E+00_JPRB, 0.82660E+00_JPRB ,&
&  0.80940E+00_JPRB, 0.81860E+00_JPRB, 0.82462E+00_JPRB, 0.81861E+00_JPRB, 0.79133E+00_JPRB, 0.77023E+00_JPRB, 0.75121E+00_JPRB ,&
&  0.70825E+00_JPRB, 0.63363E+00_JPRB, 0.56264E+00_JPRB, 0.54145E+00_JPRB, 0.52895E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.59953E+00_JPRB, 0.60561E+00_JPRB, 0.60527E+00_JPRB, 0.63645E+00_JPRB, 0.64166E+00_JPRB, 0.65198E+00_JPRB, 0.66187E+00_JPRB ,&
&  0.66818E+00_JPRB, 0.68386E+00_JPRB, 0.70327E+00_JPRB, 0.72532E+00_JPRB, 0.74253E+00_JPRB, 0.75525E+00_JPRB, 0.76514E+00_JPRB ,&
&  0.77797E+00_JPRB, 0.79314E+00_JPRB, 0.81173E+00_JPRB, 0.83390E+00_JPRB, 0.86803E+00_JPRB, 0.90402E+00_JPRB, 0.94054E+00_JPRB ,&
&  0.97293E+00_JPRB, 0.10231E+01_JPRB, 0.10897E+01_JPRB, 0.11726E+01_JPRB, 0.12593E+01_JPRB, 0.13224E+01_JPRB, 0.13448E+01_JPRB ,&
&  0.11469E+01_JPRB, 0.13823E+01_JPRB, 0.13364E+01_JPRB, 0.12045E+01_JPRB, 0.10885E+01_JPRB, 0.10193E+01_JPRB, 0.95593E+00_JPRB ,&
&  0.90121E+00_JPRB, 0.82722E+00_JPRB, 0.79557E+00_JPRB, 0.79409E+00_JPRB, 0.79154E+00_JPRB, 0.78666E+00_JPRB, 0.80091E+00_JPRB ,&
&  0.83188E+00_JPRB, 0.85739E+00_JPRB, 0.88790E+00_JPRB, 0.91238E+00_JPRB, 0.95135E+00_JPRB, 0.10170E+01_JPRB, 0.10870E+01_JPRB ,&
&  0.11587E+01_JPRB, 0.12058E+01_JPRB, 0.12640E+01_JPRB, 0.11274E+01_JPRB, 0.10885E+01_JPRB, 0.10430E+01_JPRB, 0.80716E+00_JPRB ,&
&  0.66848E+00_JPRB, 0.64404E+00_JPRB, 0.70040E+00_JPRB, 0.74669E+00_JPRB, 0.70100E+00_JPRB, 0.64307E+00_JPRB, 0.59917E+00_JPRB ,&
&  0.56161E+00_JPRB, 0.53345E+00_JPRB, 0.53868E+00_JPRB, 0.52863E+00_JPRB, 0.52803E+00_JPRB, 0.54401E+00_JPRB, 0.56033E+00_JPRB ,&
&  0.59560E+00_JPRB, 0.60983E+00_JPRB, 0.64248E+00_JPRB, 0.67111E+00_JPRB, 0.70500E+00_JPRB, 0.73874E+00_JPRB, 0.77197E+00_JPRB ,&
&  0.79798E+00_JPRB, 0.80903E+00_JPRB, 0.81078E+00_JPRB, 0.83696E+00_JPRB, 0.84369E+00_JPRB, 0.85782E+00_JPRB, 0.88238E+00_JPRB ,&
&  0.91098E+00_JPRB, 0.92993E+00_JPRB, 0.91810E+00_JPRB, 0.90406E+00_JPRB, 0.87501E+00_JPRB, 0.84609E+00_JPRB, 0.64737E+00_JPRB ,&
&  0.61720E+00_JPRB, 0.58237E+00_JPRB, 0.59609E+00_JPRB, 0.61340E+00_JPRB, 0.60470E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.73320E+00_JPRB, 0.76187E+00_JPRB, 0.76339E+00_JPRB, 0.78294E+00_JPRB, 0.77639E+00_JPRB, 0.76095E+00_JPRB, 0.76450E+00_JPRB ,&
&  0.77573E+00_JPRB, 0.78251E+00_JPRB, 0.80101E+00_JPRB, 0.81989E+00_JPRB, 0.83350E+00_JPRB, 0.83631E+00_JPRB, 0.84719E+00_JPRB ,&
&  0.86662E+00_JPRB, 0.89301E+00_JPRB, 0.92291E+00_JPRB, 0.96225E+00_JPRB, 0.98658E+00_JPRB, 0.99690E+00_JPRB, 0.10251E+01_JPRB ,&
&  0.10441E+01_JPRB, 0.10981E+01_JPRB, 0.11627E+01_JPRB, 0.12272E+01_JPRB, 0.12669E+01_JPRB, 0.12828E+01_JPRB, 0.10495E+01_JPRB ,&
&  0.14341E+01_JPRB, 0.14819E+01_JPRB, 0.13921E+01_JPRB, 0.11765E+01_JPRB, 0.10639E+01_JPRB, 0.98808E+00_JPRB, 0.92445E+00_JPRB ,&
&  0.87388E+00_JPRB, 0.81932E+00_JPRB, 0.77673E+00_JPRB, 0.75772E+00_JPRB, 0.80987E+00_JPRB, 0.81923E+00_JPRB, 0.85245E+00_JPRB ,&
&  0.89152E+00_JPRB, 0.92815E+00_JPRB, 0.96149E+00_JPRB, 0.98962E+00_JPRB, 0.10375E+01_JPRB, 0.10927E+01_JPRB, 0.11425E+01_JPRB ,&
&  0.11829E+01_JPRB, 0.12097E+01_JPRB, 0.12897E+01_JPRB, 0.11520E+01_JPRB, 0.11640E+01_JPRB, 0.11662E+01_JPRB, 0.95769E+00_JPRB ,&
&  0.70608E+00_JPRB, 0.67550E+00_JPRB, 0.73988E+00_JPRB, 0.77046E+00_JPRB, 0.73590E+00_JPRB, 0.66155E+00_JPRB, 0.61333E+00_JPRB ,&
&  0.55925E+00_JPRB, 0.55334E+00_JPRB, 0.54973E+00_JPRB, 0.53785E+00_JPRB, 0.52929E+00_JPRB, 0.55490E+00_JPRB, 0.57724E+00_JPRB ,&
&  0.62592E+00_JPRB, 0.65923E+00_JPRB, 0.70329E+00_JPRB, 0.74369E+00_JPRB, 0.80364E+00_JPRB, 0.87876E+00_JPRB, 0.95176E+00_JPRB ,&
&  0.83867E+00_JPRB, 0.78542E+00_JPRB, 0.82677E+00_JPRB, 0.88422E+00_JPRB, 0.92501E+00_JPRB, 0.10127E+01_JPRB, 0.11178E+01_JPRB ,&
&  0.12425E+01_JPRB, 0.13440E+01_JPRB, 0.12328E+01_JPRB, 0.10362E+01_JPRB, 0.14194E+01_JPRB, 0.14714E+01_JPRB, 0.67221E+00_JPRB ,&
&  0.66054E+00_JPRB, 0.69011E+00_JPRB, 0.70574E+00_JPRB, 0.72468E+00_JPRB, 0.74273E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.79973E+00_JPRB, 0.81016E+00_JPRB, 0.82013E+00_JPRB, 0.82632E+00_JPRB, 0.82463E+00_JPRB, 0.83254E+00_JPRB, 0.84972E+00_JPRB ,&
&  0.86619E+00_JPRB, 0.88204E+00_JPRB, 0.89736E+00_JPRB, 0.90827E+00_JPRB, 0.92073E+00_JPRB, 0.92499E+00_JPRB, 0.94210E+00_JPRB ,&
&  0.96783E+00_JPRB, 0.99145E+00_JPRB, 0.99849E+00_JPRB, 0.10206E+01_JPRB, 0.10269E+01_JPRB, 0.10230E+01_JPRB, 0.10461E+01_JPRB ,&
&  0.10725E+01_JPRB, 0.11142E+01_JPRB, 0.11430E+01_JPRB, 0.11576E+01_JPRB, 0.11515E+01_JPRB, 0.12082E+01_JPRB, 0.10663E+01_JPRB ,&
&  0.13653E+01_JPRB, 0.15269E+01_JPRB, 0.13953E+01_JPRB, 0.11222E+01_JPRB, 0.10119E+01_JPRB, 0.95293E+00_JPRB, 0.89790E+00_JPRB ,&
&  0.86107E+00_JPRB, 0.82009E+00_JPRB, 0.77620E+00_JPRB, 0.80021E+00_JPRB, 0.82400E+00_JPRB, 0.85396E+00_JPRB, 0.89627E+00_JPRB ,&
&  0.92891E+00_JPRB, 0.96278E+00_JPRB, 0.99593E+00_JPRB, 0.10238E+01_JPRB, 0.10745E+01_JPRB, 0.11172E+01_JPRB, 0.11526E+01_JPRB ,&
&  0.11799E+01_JPRB, 0.12188E+01_JPRB, 0.12726E+01_JPRB, 0.11748E+01_JPRB, 0.12199E+01_JPRB, 0.12134E+01_JPRB, 0.10468E+01_JPRB ,&
&  0.77714E+00_JPRB, 0.68704E+00_JPRB, 0.72750E+00_JPRB, 0.78038E+00_JPRB, 0.75817E+00_JPRB, 0.66517E+00_JPRB, 0.64403E+00_JPRB ,&
&  0.57487E+00_JPRB, 0.53363E+00_JPRB, 0.53726E+00_JPRB, 0.52974E+00_JPRB, 0.52184E+00_JPRB, 0.55229E+00_JPRB, 0.57580E+00_JPRB ,&
&  0.61262E+00_JPRB, 0.66685E+00_JPRB, 0.70589E+00_JPRB, 0.72785E+00_JPRB, 0.79246E+00_JPRB, 0.85483E+00_JPRB, 0.81116E+00_JPRB ,&
&  0.86245E+00_JPRB, 0.90064E+00_JPRB, 0.93788E+00_JPRB, 0.10526E+01_JPRB, 0.10451E+01_JPRB, 0.12474E+01_JPRB, 0.14694E+01_JPRB ,&
&  0.17701E+01_JPRB, 0.17183E+01_JPRB, 0.20301E+01_JPRB, 0.30640E+01_JPRB, 0.26296E+01_JPRB, 0.22076E+01_JPRB, 0.80143E+00_JPRB ,&
&  0.78609E+00_JPRB, 0.79581E+00_JPRB, 0.81325E+00_JPRB, 0.81074E+00_JPRB, 0.80766E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.82493E+00_JPRB, 0.83709E+00_JPRB, 0.84536E+00_JPRB, 0.85971E+00_JPRB, 0.87759E+00_JPRB, 0.89906E+00_JPRB, 0.91464E+00_JPRB ,&
&  0.93250E+00_JPRB, 0.94644E+00_JPRB, 0.95922E+00_JPRB, 0.97368E+00_JPRB, 0.98320E+00_JPRB, 0.99764E+00_JPRB, 0.99883E+00_JPRB ,&
&  0.10146E+01_JPRB, 0.10074E+01_JPRB, 0.10119E+01_JPRB, 0.10130E+01_JPRB, 0.10122E+01_JPRB, 0.10111E+01_JPRB, 0.10166E+01_JPRB ,&
&  0.10202E+01_JPRB, 0.10135E+01_JPRB, 0.10196E+01_JPRB, 0.10301E+01_JPRB, 0.10331E+01_JPRB, 0.10867E+01_JPRB, 0.92825E+00_JPRB ,&
&  0.12838E+01_JPRB, 0.14295E+01_JPRB, 0.13350E+01_JPRB, 0.10546E+01_JPRB, 0.93942E+00_JPRB, 0.89568E+00_JPRB, 0.86557E+00_JPRB ,&
&  0.83757E+00_JPRB, 0.82126E+00_JPRB, 0.79879E+00_JPRB, 0.80353E+00_JPRB, 0.82512E+00_JPRB, 0.85641E+00_JPRB, 0.88399E+00_JPRB ,&
&  0.90273E+00_JPRB, 0.92337E+00_JPRB, 0.95039E+00_JPRB, 0.98107E+00_JPRB, 0.10376E+01_JPRB, 0.10705E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.10859E+01_JPRB, 0.11026E+01_JPRB, 0.10498E+01_JPRB, 0.10527E+01_JPRB, 0.11212E+01_JPRB, 0.10087E+01_JPRB, 0.89248E+00_JPRB ,&
&  0.76760E+00_JPRB, 0.72284E+00_JPRB, 0.75482E+00_JPRB, 0.82980E+00_JPRB, 0.77092E+00_JPRB, 0.70530E+00_JPRB, 0.67221E+00_JPRB ,&
&  0.59387E+00_JPRB, 0.55245E+00_JPRB, 0.54226E+00_JPRB, 0.51864E+00_JPRB, 0.51987E+00_JPRB, 0.53896E+00_JPRB, 0.58274E+00_JPRB ,&
&  0.60330E+00_JPRB, 0.62523E+00_JPRB, 0.65154E+00_JPRB, 0.65926E+00_JPRB, 0.68909E+00_JPRB, 0.65803E+00_JPRB, 0.78336E+00_JPRB ,&
&  0.79011E+00_JPRB, 0.79119E+00_JPRB, 0.84452E+00_JPRB, 0.73829E+00_JPRB, 0.10002E+01_JPRB, 0.10849E+01_JPRB, 0.12374E+01_JPRB ,&
&  0.12840E+01_JPRB, 0.17441E+01_JPRB, 0.21183E+01_JPRB, 0.17716E+01_JPRB, 0.12793E+01_JPRB, 0.86784E+00_JPRB, 0.77281E+00_JPRB ,&
&  0.76688E+00_JPRB, 0.78360E+00_JPRB, 0.80093E+00_JPRB, 0.80787E+00_JPRB, 0.81602E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.82037E+00_JPRB, 0.83202E+00_JPRB, 0.84430E+00_JPRB, 0.86657E+00_JPRB, 0.89094E+00_JPRB, 0.91455E+00_JPRB, 0.93270E+00_JPRB ,&
&  0.95102E+00_JPRB, 0.96425E+00_JPRB, 0.98174E+00_JPRB, 0.99783E+00_JPRB, 0.10093E+01_JPRB, 0.10147E+01_JPRB, 0.10140E+01_JPRB ,&
&  0.10161E+01_JPRB, 0.10185E+01_JPRB, 0.10289E+01_JPRB, 0.10350E+01_JPRB, 0.10283E+01_JPRB, 0.10326E+01_JPRB, 0.10464E+01_JPRB ,&
&  0.10570E+01_JPRB, 0.10665E+01_JPRB, 0.10936E+01_JPRB, 0.11333E+01_JPRB, 0.11868E+01_JPRB, 0.12888E+01_JPRB, 0.11243E+01_JPRB ,&
&  0.11825E+01_JPRB, 0.12788E+01_JPRB, 0.12569E+01_JPRB, 0.98017E+00_JPRB, 0.86867E+00_JPRB, 0.82764E+00_JPRB, 0.80184E+00_JPRB ,&
&  0.78527E+00_JPRB, 0.76946E+00_JPRB, 0.76473E+00_JPRB, 0.77636E+00_JPRB, 0.79741E+00_JPRB, 0.82174E+00_JPRB, 0.83298E+00_JPRB ,&
&  0.84862E+00_JPRB, 0.86521E+00_JPRB, 0.88145E+00_JPRB, 0.89643E+00_JPRB, 0.92543E+00_JPRB, 0.93777E+00_JPRB, 0.93328E+00_JPRB ,&
&  0.91526E+00_JPRB, 0.90680E+00_JPRB, 0.81383E+00_JPRB, 0.77313E+00_JPRB, 0.80626E+00_JPRB, 0.76003E+00_JPRB, 0.70582E+00_JPRB ,&
&  0.72669E+00_JPRB, 0.71423E+00_JPRB, 0.70325E+00_JPRB, 0.85243E+00_JPRB, 0.82620E+00_JPRB, 0.71458E+00_JPRB, 0.69390E+00_JPRB ,&
&  0.62242E+00_JPRB, 0.55466E+00_JPRB, 0.52484E+00_JPRB, 0.52359E+00_JPRB, 0.52975E+00_JPRB, 0.55617E+00_JPRB, 0.55643E+00_JPRB ,&
&  0.56462E+00_JPRB, 0.57818E+00_JPRB, 0.57907E+00_JPRB, 0.57081E+00_JPRB, 0.61174E+00_JPRB, 0.53175E+00_JPRB, 0.69199E+00_JPRB ,&
&  0.66059E+00_JPRB, 0.56163E+00_JPRB, 0.66446E+00_JPRB, 0.83638E+00_JPRB, 0.90054E+00_JPRB, 0.98767E+00_JPRB, 0.11448E+01_JPRB ,&
&  0.13214E+01_JPRB, 0.14519E+01_JPRB, 0.13020E+01_JPRB, 0.10957E+01_JPRB, 0.88297E+00_JPRB, 0.79194E+00_JPRB, 0.75500E+00_JPRB ,&
&  0.74870E+00_JPRB, 0.77118E+00_JPRB, 0.78823E+00_JPRB, 0.79610E+00_JPRB, 0.80774E+00_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.78448E+00_JPRB, 0.79839E+00_JPRB, 0.81425E+00_JPRB, 0.83391E+00_JPRB, 0.85222E+00_JPRB, 0.86850E+00_JPRB, 0.88738E+00_JPRB ,&
&  0.91236E+00_JPRB, 0.93155E+00_JPRB, 0.95132E+00_JPRB, 0.96363E+00_JPRB, 0.96983E+00_JPRB, 0.98211E+00_JPRB, 0.98587E+00_JPRB ,&
&  0.98718E+00_JPRB, 0.98818E+00_JPRB, 0.10033E+01_JPRB, 0.10131E+01_JPRB, 0.10110E+01_JPRB, 0.10067E+01_JPRB, 0.10188E+01_JPRB ,&
&  0.10179E+01_JPRB, 0.10440E+01_JPRB, 0.10672E+01_JPRB, 0.10962E+01_JPRB, 0.11207E+01_JPRB, 0.11287E+01_JPRB, 0.11995E+01_JPRB ,&
&  0.84170E+00_JPRB, 0.11614E+01_JPRB, 0.11553E+01_JPRB, 0.90324E+00_JPRB, 0.83012E+00_JPRB, 0.77210E+00_JPRB, 0.74310E+00_JPRB ,&
&  0.72585E+00_JPRB, 0.71512E+00_JPRB, 0.71759E+00_JPRB, 0.73304E+00_JPRB, 0.75133E+00_JPRB, 0.77457E+00_JPRB, 0.78750E+00_JPRB ,&
&  0.79582E+00_JPRB, 0.80222E+00_JPRB, 0.80449E+00_JPRB, 0.80205E+00_JPRB, 0.80200E+00_JPRB, 0.79108E+00_JPRB, 0.78063E+00_JPRB ,&
&  0.75536E+00_JPRB, 0.73934E+00_JPRB, 0.55728E+00_JPRB, 0.52854E+00_JPRB, 0.56286E+00_JPRB, 0.56589E+00_JPRB, 0.52766E+00_JPRB ,&
&  0.61882E+00_JPRB, 0.72277E+00_JPRB, 0.79137E+00_JPRB, 0.83994E+00_JPRB, 0.82350E+00_JPRB, 0.75088E+00_JPRB, 0.73085E+00_JPRB ,&
&  0.69469E+00_JPRB, 0.63039E+00_JPRB, 0.60101E+00_JPRB, 0.57846E+00_JPRB, 0.57823E+00_JPRB, 0.56687E+00_JPRB, 0.55256E+00_JPRB ,&
&  0.52720E+00_JPRB, 0.51065E+00_JPRB, 0.48975E+00_JPRB, 0.45300E+00_JPRB, 0.37001E+00_JPRB, 0.48877E+00_JPRB, 0.62644E+00_JPRB ,&
&  0.57212E+00_JPRB, 0.50515E+00_JPRB, 0.51002E+00_JPRB, 0.69577E+00_JPRB, 0.72674E+00_JPRB, 0.77207E+00_JPRB, 0.89084E+00_JPRB ,&
&  0.97937E+00_JPRB, 0.98542E+00_JPRB, 0.88766E+00_JPRB, 0.77553E+00_JPRB, 0.73259E+00_JPRB, 0.71652E+00_JPRB, 0.69425E+00_JPRB ,&
&  0.69440E+00_JPRB, 0.71704E+00_JPRB, 0.73796E+00_JPRB, 0.75314E+00_JPRB, 0.76896E+00_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.75060E+00_JPRB, 0.74066E+00_JPRB, 0.74945E+00_JPRB, 0.76391E+00_JPRB, 0.77707E+00_JPRB, 0.79086E+00_JPRB, 0.81463E+00_JPRB ,&
&  0.84451E+00_JPRB, 0.86899E+00_JPRB, 0.89379E+00_JPRB, 0.91129E+00_JPRB, 0.91711E+00_JPRB, 0.93032E+00_JPRB, 0.94119E+00_JPRB ,&
&  0.95386E+00_JPRB, 0.94971E+00_JPRB, 0.96643E+00_JPRB, 0.99540E+00_JPRB, 0.10276E+01_JPRB, 0.10394E+01_JPRB, 0.10483E+01_JPRB ,&
&  0.10609E+01_JPRB, 0.10977E+01_JPRB, 0.11422E+01_JPRB, 0.11951E+01_JPRB, 0.12854E+01_JPRB, 0.14448E+01_JPRB, 0.16122E+01_JPRB ,&
&  0.10585E+01_JPRB, 0.95397E+00_JPRB, 0.97298E+00_JPRB, 0.79617E+00_JPRB, 0.72059E+00_JPRB, 0.72335E+00_JPRB, 0.68835E+00_JPRB ,&
&  0.65768E+00_JPRB, 0.66308E+00_JPRB, 0.66212E+00_JPRB, 0.66358E+00_JPRB, 0.67614E+00_JPRB, 0.70663E+00_JPRB, 0.71519E+00_JPRB ,&
&  0.72748E+00_JPRB, 0.73098E+00_JPRB, 0.72077E+00_JPRB, 0.71158E+00_JPRB, 0.63876E+00_JPRB, 0.69508E+00_JPRB, 0.68198E+00_JPRB ,&
&  0.67540E+00_JPRB, 0.60808E+00_JPRB, 0.43546E+00_JPRB, 0.41331E+00_JPRB, 0.44353E+00_JPRB, 0.43380E+00_JPRB, 0.43184E+00_JPRB ,&
&  0.55859E+00_JPRB, 0.65975E+00_JPRB, 0.58947E+00_JPRB, 0.75291E+00_JPRB, 0.81536E+00_JPRB, 0.83202E+00_JPRB, 0.82573E+00_JPRB ,&
&  0.80709E+00_JPRB, 0.75679E+00_JPRB, 0.71431E+00_JPRB, 0.66950E+00_JPRB, 0.64475E+00_JPRB, 0.61371E+00_JPRB, 0.57399E+00_JPRB ,&
&  0.53617E+00_JPRB, 0.50193E+00_JPRB, 0.45155E+00_JPRB, 0.41051E+00_JPRB, 0.43560E+00_JPRB, 0.40054E+00_JPRB, 0.56793E+00_JPRB ,&
&  0.52131E+00_JPRB, 0.51509E+00_JPRB, 0.44526E+00_JPRB, 0.57874E+00_JPRB, 0.58063E+00_JPRB, 0.56800E+00_JPRB, 0.64226E+00_JPRB ,&
&  0.68723E+00_JPRB, 0.71863E+00_JPRB, 0.70090E+00_JPRB, 0.66925E+00_JPRB, 0.65599E+00_JPRB, 0.65644E+00_JPRB, 0.63692E+00_JPRB ,&
&  0.63073E+00_JPRB, 0.64916E+00_JPRB, 0.68242E+00_JPRB, 0.69313E+00_JPRB, 0.72278E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.66644E+00_JPRB, 0.66968E+00_JPRB, 0.67340E+00_JPRB, 0.68064E+00_JPRB, 0.69578E+00_JPRB, 0.71203E+00_JPRB, 0.72119E+00_JPRB ,&
&  0.74274E+00_JPRB, 0.76088E+00_JPRB, 0.78133E+00_JPRB, 0.78712E+00_JPRB, 0.80244E+00_JPRB, 0.81881E+00_JPRB, 0.82456E+00_JPRB ,&
&  0.84422E+00_JPRB, 0.86726E+00_JPRB, 0.86918E+00_JPRB, 0.87902E+00_JPRB, 0.90776E+00_JPRB, 0.92678E+00_JPRB, 0.90964E+00_JPRB ,&
&  0.88725E+00_JPRB, 0.85919E+00_JPRB, 0.87688E+00_JPRB, 0.91355E+00_JPRB, 0.10897E+01_JPRB, 0.11839E+01_JPRB, 0.11577E+01_JPRB ,&
&  0.88638E+00_JPRB, 0.82493E+00_JPRB, 0.96536E+00_JPRB, 0.79995E+00_JPRB, 0.71911E+00_JPRB, 0.66482E+00_JPRB, 0.61195E+00_JPRB ,&
&  0.61799E+00_JPRB, 0.63236E+00_JPRB, 0.63583E+00_JPRB, 0.64273E+00_JPRB, 0.66325E+00_JPRB, 0.67470E+00_JPRB, 0.70086E+00_JPRB ,&
&  0.70405E+00_JPRB, 0.71316E+00_JPRB, 0.69963E+00_JPRB, 0.64049E+00_JPRB, 0.59170E+00_JPRB, 0.63843E+00_JPRB, 0.62984E+00_JPRB ,&
&  0.62834E+00_JPRB, 0.62462E+00_JPRB, 0.53831E+00_JPRB, 0.50712E+00_JPRB, 0.49517E+00_JPRB, 0.44544E+00_JPRB, 0.42698E+00_JPRB ,&
&  0.47336E+00_JPRB, 0.50259E+00_JPRB, 0.40393E+00_JPRB, 0.59453E+00_JPRB, 0.75060E+00_JPRB, 0.83561E+00_JPRB, 0.90574E+00_JPRB ,&
&  0.91476E+00_JPRB, 0.86851E+00_JPRB, 0.79901E+00_JPRB, 0.77862E+00_JPRB, 0.75939E+00_JPRB, 0.67095E+00_JPRB, 0.62970E+00_JPRB ,&
&  0.56144E+00_JPRB, 0.51223E+00_JPRB, 0.46584E+00_JPRB, 0.43825E+00_JPRB, 0.40483E+00_JPRB, 0.42021E+00_JPRB, 0.46426E+00_JPRB ,&
&  0.46116E+00_JPRB, 0.46116E+00_JPRB, 0.50510E+00_JPRB, 0.55726E+00_JPRB, 0.43827E+00_JPRB, 0.45938E+00_JPRB, 0.50738E+00_JPRB ,&
&  0.52766E+00_JPRB, 0.53064E+00_JPRB, 0.53356E+00_JPRB, 0.53610E+00_JPRB, 0.54142E+00_JPRB, 0.55939E+00_JPRB, 0.55612E+00_JPRB ,&
&  0.55427E+00_JPRB, 0.55462E+00_JPRB, 0.57361E+00_JPRB, 0.59724E+00_JPRB, 0.63729E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.52821E+00_JPRB, 0.52910E+00_JPRB, 0.51744E+00_JPRB, 0.50935E+00_JPRB, 0.50839E+00_JPRB, 0.51234E+00_JPRB, 0.51848E+00_JPRB ,&
&  0.52963E+00_JPRB, 0.54791E+00_JPRB, 0.56870E+00_JPRB, 0.59867E+00_JPRB, 0.60762E+00_JPRB, 0.58365E+00_JPRB, 0.59334E+00_JPRB ,&
&  0.59230E+00_JPRB, 0.60680E+00_JPRB, 0.60030E+00_JPRB, 0.67375E+00_JPRB, 0.70097E+00_JPRB, 0.71253E+00_JPRB, 0.79693E+00_JPRB ,&
&  0.78330E+00_JPRB, 0.85902E+00_JPRB, 0.91889E+00_JPRB, 0.10285E+01_JPRB, 0.17147E+01_JPRB, 0.15406E+01_JPRB, 0.95482E+00_JPRB ,&
&  0.78884E+00_JPRB, 0.89121E+00_JPRB, 0.87267E+00_JPRB, 0.78387E+00_JPRB, 0.73843E+00_JPRB, 0.66272E+00_JPRB, 0.63644E+00_JPRB ,&
&  0.65751E+00_JPRB, 0.68202E+00_JPRB, 0.73883E+00_JPRB, 0.78744E+00_JPRB, 0.82359E+00_JPRB, 0.86509E+00_JPRB, 0.91370E+00_JPRB ,&
&  0.94820E+00_JPRB, 0.97381E+00_JPRB, 0.92721E+00_JPRB, 0.81064E+00_JPRB, 0.82745E+00_JPRB, 0.85206E+00_JPRB, 0.85788E+00_JPRB ,&
&  0.86079E+00_JPRB, 0.86343E+00_JPRB, 0.87732E+00_JPRB, 0.81418E+00_JPRB, 0.72842E+00_JPRB, 0.67297E+00_JPRB, 0.58342E+00_JPRB ,&
&  0.52732E+00_JPRB, 0.48096E+00_JPRB, 0.42330E+00_JPRB, 0.60217E+00_JPRB, 0.75690E+00_JPRB, 0.77418E+00_JPRB, 0.93277E+00_JPRB ,&
&  0.99138E+00_JPRB, 0.10113E+01_JPRB, 0.95935E+00_JPRB, 0.88105E+00_JPRB, 0.78948E+00_JPRB, 0.74479E+00_JPRB, 0.64351E+00_JPRB ,&
&  0.60800E+00_JPRB, 0.54607E+00_JPRB, 0.49707E+00_JPRB, 0.46049E+00_JPRB, 0.42030E+00_JPRB, 0.39511E+00_JPRB, 0.42177E+00_JPRB ,&
&  0.43279E+00_JPRB, 0.42069E+00_JPRB, 0.48239E+00_JPRB, 0.67584E+00_JPRB, 0.76603E+00_JPRB, 0.50769E+00_JPRB, 0.43536E+00_JPRB ,&
&  0.43290E+00_JPRB, 0.44050E+00_JPRB, 0.44539E+00_JPRB, 0.45129E+00_JPRB, 0.46671E+00_JPRB, 0.47951E+00_JPRB, 0.47439E+00_JPRB ,&
&  0.47911E+00_JPRB, 0.47690E+00_JPRB, 0.47573E+00_JPRB, 0.48346E+00_JPRB, 0.50023E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.45476E+00_JPRB, 0.45912E+00_JPRB, 0.47037E+00_JPRB, 0.50717E+00_JPRB, 0.52852E+00_JPRB, 0.53769E+00_JPRB, 0.56308E+00_JPRB ,&
&  0.59130E+00_JPRB, 0.64321E+00_JPRB, 0.68063E+00_JPRB, 0.69682E+00_JPRB, 0.67688E+00_JPRB, 0.65059E+00_JPRB, 0.63470E+00_JPRB ,&
&  0.62831E+00_JPRB, 0.58676E+00_JPRB, 0.56178E+00_JPRB, 0.58484E+00_JPRB, 0.64595E+00_JPRB, 0.75063E+00_JPRB, 0.86696E+00_JPRB ,&
&  0.10098E+01_JPRB, 0.11755E+01_JPRB, 0.13982E+01_JPRB, 0.19588E+01_JPRB, 0.21824E+01_JPRB, 0.92936E+00_JPRB, 0.83889E+00_JPRB ,&
&  0.95864E+00_JPRB, 0.10659E+01_JPRB, 0.97524E+00_JPRB, 0.85321E+00_JPRB, 0.78270E+00_JPRB, 0.74477E+00_JPRB, 0.74099E+00_JPRB ,&
&  0.78165E+00_JPRB, 0.81553E+00_JPRB, 0.86467E+00_JPRB, 0.92852E+00_JPRB, 0.99912E+00_JPRB, 0.10693E+01_JPRB, 0.11557E+01_JPRB ,&
&  0.12252E+01_JPRB, 0.12328E+01_JPRB, 0.12097E+01_JPRB, 0.11731E+01_JPRB, 0.11807E+01_JPRB, 0.12090E+01_JPRB, 0.12292E+01_JPRB ,&
&  0.12235E+01_JPRB, 0.12010E+01_JPRB, 0.11900E+01_JPRB, 0.11503E+01_JPRB, 0.10169E+01_JPRB, 0.91289E+00_JPRB, 0.78604E+00_JPRB ,&
&  0.70855E+00_JPRB, 0.65287E+00_JPRB, 0.60262E+00_JPRB, 0.83679E+00_JPRB, 0.96239E+00_JPRB, 0.97490E+00_JPRB, 0.99420E+00_JPRB ,&
&  0.10799E+01_JPRB, 0.10702E+01_JPRB, 0.10196E+01_JPRB, 0.87333E+00_JPRB, 0.76601E+00_JPRB, 0.77358E+00_JPRB, 0.62312E+00_JPRB ,&
&  0.64501E+00_JPRB, 0.59801E+00_JPRB, 0.54735E+00_JPRB, 0.49427E+00_JPRB, 0.44093E+00_JPRB, 0.36883E+00_JPRB, 0.36871E+00_JPRB ,&
&  0.35135E+00_JPRB, 0.42687E+00_JPRB, 0.48942E+00_JPRB, 0.79117E+00_JPRB, 0.17447E+01_JPRB, 0.88102E+00_JPRB, 0.54372E+00_JPRB ,&
&  0.41905E+00_JPRB, 0.40924E+00_JPRB, 0.41281E+00_JPRB, 0.42256E+00_JPRB, 0.43661E+00_JPRB, 0.44752E+00_JPRB, 0.45087E+00_JPRB ,&
&  0.46303E+00_JPRB, 0.46467E+00_JPRB, 0.47144E+00_JPRB, 0.46498E+00_JPRB, 0.44614E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.57048E+00_JPRB, 0.60049E+00_JPRB, 0.64244E+00_JPRB, 0.68796E+00_JPRB, 0.71174E+00_JPRB, 0.72776E+00_JPRB, 0.72042E+00_JPRB ,&
&  0.73540E+00_JPRB, 0.74998E+00_JPRB, 0.77927E+00_JPRB, 0.80837E+00_JPRB, 0.82282E+00_JPRB, 0.85781E+00_JPRB, 0.88141E+00_JPRB ,&
&  0.87941E+00_JPRB, 0.86275E+00_JPRB, 0.89252E+00_JPRB, 0.94690E+00_JPRB, 0.10809E+01_JPRB, 0.12941E+01_JPRB, 0.15024E+01_JPRB ,&
&  0.16913E+01_JPRB, 0.17489E+01_JPRB, 0.17436E+01_JPRB, 0.11554E+01_JPRB, 0.76253E+00_JPRB, 0.82601E+00_JPRB, 0.89904E+00_JPRB ,&
&  0.95150E+00_JPRB, 0.96486E+00_JPRB, 0.91180E+00_JPRB, 0.84714E+00_JPRB, 0.80006E+00_JPRB, 0.79261E+00_JPRB, 0.83185E+00_JPRB ,&
&  0.89692E+00_JPRB, 0.96612E+00_JPRB, 0.10317E+01_JPRB, 0.10739E+01_JPRB, 0.11267E+01_JPRB, 0.12109E+01_JPRB, 0.13164E+01_JPRB ,&
&  0.14034E+01_JPRB, 0.14450E+01_JPRB, 0.14418E+01_JPRB, 0.14587E+01_JPRB, 0.14553E+01_JPRB, 0.14757E+01_JPRB, 0.14826E+01_JPRB ,&
&  0.14777E+01_JPRB, 0.14513E+01_JPRB, 0.14035E+01_JPRB, 0.13721E+01_JPRB, 0.12968E+01_JPRB, 0.11290E+01_JPRB, 0.98561E+00_JPRB ,&
&  0.91856E+00_JPRB, 0.86399E+00_JPRB, 0.87370E+00_JPRB, 0.99456E+00_JPRB, 0.76579E+00_JPRB, 0.87846E+00_JPRB, 0.11208E+01_JPRB ,&
&  0.11184E+01_JPRB, 0.10833E+01_JPRB, 0.10023E+01_JPRB, 0.90033E+00_JPRB, 0.85889E+00_JPRB, 0.71131E+00_JPRB, 0.62424E+00_JPRB ,&
&  0.66582E+00_JPRB, 0.64102E+00_JPRB, 0.57968E+00_JPRB, 0.48169E+00_JPRB, 0.43573E+00_JPRB, 0.36651E+00_JPRB, 0.35462E+00_JPRB ,&
&  0.39501E+00_JPRB, 0.43984E+00_JPRB, 0.44161E+00_JPRB, 0.55754E+00_JPRB, 0.83442E+00_JPRB, 0.59569E+00_JPRB, 0.48048E+00_JPRB ,&
&  0.41235E+00_JPRB, 0.40305E+00_JPRB, 0.40526E+00_JPRB, 0.39997E+00_JPRB, 0.41291E+00_JPRB, 0.42969E+00_JPRB, 0.43282E+00_JPRB ,&
&  0.46901E+00_JPRB, 0.49675E+00_JPRB, 0.52217E+00_JPRB, 0.54736E+00_JPRB, 0.54976E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.68980E+00_JPRB, 0.78417E+00_JPRB, 0.85555E+00_JPRB, 0.92128E+00_JPRB, 0.95397E+00_JPRB, 0.10037E+01_JPRB, 0.98462E+00_JPRB ,&
&  0.90384E+00_JPRB, 0.84288E+00_JPRB, 0.89810E+00_JPRB, 0.97329E+00_JPRB, 0.10412E+01_JPRB, 0.10891E+01_JPRB, 0.11290E+01_JPRB ,&
&  0.11703E+01_JPRB, 0.12042E+01_JPRB, 0.12321E+01_JPRB, 0.12353E+01_JPRB, 0.12563E+01_JPRB, 0.12935E+01_JPRB, 0.12616E+01_JPRB ,&
&  0.10210E+01_JPRB, 0.90395E+00_JPRB, 0.11443E+01_JPRB, 0.12224E+01_JPRB, 0.93094E+00_JPRB, 0.93040E+00_JPRB, 0.97652E+00_JPRB ,&
&  0.99160E+00_JPRB, 0.96432E+00_JPRB, 0.96427E+00_JPRB, 0.95304E+00_JPRB, 0.82464E+00_JPRB, 0.85800E+00_JPRB, 0.95008E+00_JPRB ,&
&  0.10475E+01_JPRB, 0.11320E+01_JPRB, 0.11912E+01_JPRB, 0.12249E+01_JPRB, 0.12653E+01_JPRB, 0.13417E+01_JPRB, 0.14231E+01_JPRB ,&
&  0.15018E+01_JPRB, 0.15512E+01_JPRB, 0.15685E+01_JPRB, 0.15902E+01_JPRB, 0.16339E+01_JPRB, 0.16943E+01_JPRB, 0.17150E+01_JPRB ,&
&  0.17261E+01_JPRB, 0.17302E+01_JPRB, 0.17172E+01_JPRB, 0.16988E+01_JPRB, 0.16411E+01_JPRB, 0.13911E+01_JPRB, 0.12291E+01_JPRB ,&
&  0.11329E+01_JPRB, 0.10194E+01_JPRB, 0.10314E+01_JPRB, 0.11122E+01_JPRB, 0.88410E+00_JPRB, 0.94287E+00_JPRB, 0.10332E+01_JPRB ,&
&  0.11971E+01_JPRB, 0.11255E+01_JPRB, 0.10727E+01_JPRB, 0.10612E+01_JPRB, 0.10179E+01_JPRB, 0.89557E+00_JPRB, 0.74505E+00_JPRB ,&
&  0.62707E+00_JPRB, 0.61521E+00_JPRB, 0.55275E+00_JPRB, 0.49110E+00_JPRB, 0.36624E+00_JPRB, 0.35071E+00_JPRB, 0.38852E+00_JPRB ,&
&  0.48964E+00_JPRB, 0.43703E+00_JPRB, 0.43457E+00_JPRB, 0.49249E+00_JPRB, 0.55727E+00_JPRB, 0.45734E+00_JPRB, 0.42410E+00_JPRB ,&
&  0.40767E+00_JPRB, 0.41658E+00_JPRB, 0.41903E+00_JPRB, 0.41174E+00_JPRB, 0.41086E+00_JPRB, 0.42729E+00_JPRB, 0.44391E+00_JPRB ,&
&  0.47615E+00_JPRB, 0.51435E+00_JPRB, 0.54746E+00_JPRB, 0.59001E+00_JPRB, 0.64594E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.68709E+00_JPRB, 0.76370E+00_JPRB, 0.87958E+00_JPRB, 0.10271E+01_JPRB, 0.12184E+01_JPRB, 0.13965E+01_JPRB, 0.14347E+01_JPRB ,&
&  0.11778E+01_JPRB, 0.93292E+00_JPRB, 0.10149E+01_JPRB, 0.11007E+01_JPRB, 0.11750E+01_JPRB, 0.12519E+01_JPRB, 0.13153E+01_JPRB ,&
&  0.13897E+01_JPRB, 0.14663E+01_JPRB, 0.15345E+01_JPRB, 0.15748E+01_JPRB, 0.15632E+01_JPRB, 0.15048E+01_JPRB, 0.12965E+01_JPRB ,&
&  0.92995E+00_JPRB, 0.10499E+01_JPRB, 0.15074E+01_JPRB, 0.12156E+01_JPRB, 0.10144E+01_JPRB, 0.10269E+01_JPRB, 0.10681E+01_JPRB ,&
&  0.10737E+01_JPRB, 0.10272E+01_JPRB, 0.10400E+01_JPRB, 0.10129E+01_JPRB, 0.92453E+00_JPRB, 0.98662E+00_JPRB, 0.11105E+01_JPRB ,&
&  0.11802E+01_JPRB, 0.12586E+01_JPRB, 0.13326E+01_JPRB, 0.13819E+01_JPRB, 0.13943E+01_JPRB, 0.14382E+01_JPRB, 0.14765E+01_JPRB ,&
&  0.15069E+01_JPRB, 0.15451E+01_JPRB, 0.15663E+01_JPRB, 0.16070E+01_JPRB, 0.17270E+01_JPRB, 0.18764E+01_JPRB, 0.19962E+01_JPRB ,&
&  0.20644E+01_JPRB, 0.22352E+01_JPRB, 0.22610E+01_JPRB, 0.21139E+01_JPRB, 0.18470E+01_JPRB, 0.18111E+01_JPRB, 0.14679E+01_JPRB ,&
&  0.13600E+01_JPRB, 0.11688E+01_JPRB, 0.12011E+01_JPRB, 0.10439E+01_JPRB, 0.10571E+01_JPRB, 0.11230E+01_JPRB, 0.11959E+01_JPRB ,&
&  0.12337E+01_JPRB, 0.12246E+01_JPRB, 0.11574E+01_JPRB, 0.11326E+01_JPRB, 0.11916E+01_JPRB, 0.10833E+01_JPRB, 0.98575E+00_JPRB ,&
&  0.66995E+00_JPRB, 0.64462E+00_JPRB, 0.52448E+00_JPRB, 0.43012E+00_JPRB, 0.33110E+00_JPRB, 0.34096E+00_JPRB, 0.42952E+00_JPRB ,&
&  0.55306E+00_JPRB, 0.55210E+00_JPRB, 0.55344E+00_JPRB, 0.56268E+00_JPRB, 0.52066E+00_JPRB, 0.44927E+00_JPRB, 0.41050E+00_JPRB ,&
&  0.39396E+00_JPRB, 0.40790E+00_JPRB, 0.42832E+00_JPRB, 0.43150E+00_JPRB, 0.42553E+00_JPRB, 0.43788E+00_JPRB, 0.45682E+00_JPRB ,&
&  0.48507E+00_JPRB, 0.51688E+00_JPRB, 0.55934E+00_JPRB, 0.58324E+00_JPRB, 0.63363E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.59047E+00_JPRB, 0.65078E+00_JPRB, 0.72659E+00_JPRB, 0.83035E+00_JPRB, 0.93185E+00_JPRB, 0.99769E+00_JPRB, 0.94006E+00_JPRB ,&
&  0.88889E+00_JPRB, 0.92377E+00_JPRB, 0.98474E+00_JPRB, 0.10622E+01_JPRB, 0.11452E+01_JPRB, 0.12351E+01_JPRB, 0.13167E+01_JPRB ,&
&  0.13975E+01_JPRB, 0.14997E+01_JPRB, 0.16190E+01_JPRB, 0.17230E+01_JPRB, 0.17395E+01_JPRB, 0.16258E+01_JPRB, 0.11895E+01_JPRB ,&
&  0.98715E+00_JPRB, 0.17763E+01_JPRB, 0.15003E+01_JPRB, 0.13005E+01_JPRB, 0.10967E+01_JPRB, 0.10884E+01_JPRB, 0.10865E+01_JPRB ,&
&  0.10546E+01_JPRB, 0.10708E+01_JPRB, 0.11160E+01_JPRB, 0.11120E+01_JPRB, 0.11269E+01_JPRB, 0.11837E+01_JPRB, 0.12424E+01_JPRB ,&
&  0.12675E+01_JPRB, 0.13309E+01_JPRB, 0.14092E+01_JPRB, 0.14755E+01_JPRB, 0.14788E+01_JPRB, 0.14817E+01_JPRB, 0.14838E+01_JPRB ,&
&  0.14833E+01_JPRB, 0.14828E+01_JPRB, 0.14899E+01_JPRB, 0.16171E+01_JPRB, 0.17703E+01_JPRB, 0.19782E+01_JPRB, 0.22321E+01_JPRB ,&
&  0.23679E+01_JPRB, 0.24850E+01_JPRB, 0.27762E+01_JPRB, 0.27228E+01_JPRB, 0.25207E+01_JPRB, 0.23567E+01_JPRB, 0.17519E+01_JPRB ,&
&  0.16310E+01_JPRB, 0.13252E+01_JPRB, 0.13168E+01_JPRB, 0.11320E+01_JPRB, 0.11403E+01_JPRB, 0.12322E+01_JPRB, 0.12920E+01_JPRB ,&
&  0.12518E+01_JPRB, 0.13284E+01_JPRB, 0.12743E+01_JPRB, 0.12184E+01_JPRB, 0.11998E+01_JPRB, 0.11777E+01_JPRB, 0.11425E+01_JPRB ,&
&  0.91713E+00_JPRB, 0.63509E+00_JPRB, 0.53556E+00_JPRB, 0.41085E+00_JPRB, 0.34158E+00_JPRB, 0.31128E+00_JPRB, 0.39586E+00_JPRB ,&
&  0.55667E+00_JPRB, 0.70553E+00_JPRB, 0.84528E+00_JPRB, 0.76452E+00_JPRB, 0.68368E+00_JPRB, 0.60475E+00_JPRB, 0.51213E+00_JPRB ,&
&  0.41854E+00_JPRB, 0.41937E+00_JPRB, 0.45382E+00_JPRB, 0.46388E+00_JPRB, 0.45560E+00_JPRB, 0.46696E+00_JPRB, 0.47111E+00_JPRB ,&
&  0.47045E+00_JPRB, 0.48101E+00_JPRB, 0.50960E+00_JPRB, 0.51635E+00_JPRB, 0.54538E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.51178E+00_JPRB, 0.54846E+00_JPRB, 0.61970E+00_JPRB, 0.68415E+00_JPRB, 0.72000E+00_JPRB, 0.73607E+00_JPRB, 0.74395E+00_JPRB ,&
&  0.78593E+00_JPRB, 0.84411E+00_JPRB, 0.88848E+00_JPRB, 0.92738E+00_JPRB, 0.98767E+00_JPRB, 0.10852E+01_JPRB, 0.12078E+01_JPRB ,&
&  0.13474E+01_JPRB, 0.15182E+01_JPRB, 0.17334E+01_JPRB, 0.19507E+01_JPRB, 0.21042E+01_JPRB, 0.20239E+01_JPRB, 0.12990E+01_JPRB ,&
&  0.12693E+01_JPRB, 0.17972E+01_JPRB, 0.17697E+01_JPRB, 0.16536E+01_JPRB, 0.13778E+01_JPRB, 0.12493E+01_JPRB, 0.11471E+01_JPRB ,&
&  0.11723E+01_JPRB, 0.11882E+01_JPRB, 0.12353E+01_JPRB, 0.12869E+01_JPRB, 0.13112E+01_JPRB, 0.13158E+01_JPRB, 0.12799E+01_JPRB ,&
&  0.12433E+01_JPRB, 0.12502E+01_JPRB, 0.12799E+01_JPRB, 0.13675E+01_JPRB, 0.14522E+01_JPRB, 0.15035E+01_JPRB, 0.15190E+01_JPRB ,&
&  0.15052E+01_JPRB, 0.14760E+01_JPRB, 0.14693E+01_JPRB, 0.14965E+01_JPRB, 0.17746E+01_JPRB, 0.19921E+01_JPRB, 0.22960E+01_JPRB ,&
&  0.25383E+01_JPRB, 0.26978E+01_JPRB, 0.32564E+01_JPRB, 0.36153E+01_JPRB, 0.36111E+01_JPRB, 0.30076E+01_JPRB, 0.22797E+01_JPRB ,&
&  0.19014E+01_JPRB, 0.15634E+01_JPRB, 0.13530E+01_JPRB, 0.12429E+01_JPRB, 0.12526E+01_JPRB, 0.13134E+01_JPRB, 0.14628E+01_JPRB ,&
&  0.14230E+01_JPRB, 0.14099E+01_JPRB, 0.13825E+01_JPRB, 0.12424E+01_JPRB, 0.11945E+01_JPRB, 0.11537E+01_JPRB, 0.12831E+01_JPRB ,&
&  0.12841E+01_JPRB, 0.98228E+00_JPRB, 0.66908E+00_JPRB, 0.44111E+00_JPRB, 0.35120E+00_JPRB, 0.27431E+00_JPRB, 0.32049E+00_JPRB ,&
&  0.43975E+00_JPRB, 0.61457E+00_JPRB, 0.84471E+00_JPRB, 0.10464E+01_JPRB, 0.92828E+00_JPRB, 0.88067E+00_JPRB, 0.81268E+00_JPRB ,&
&  0.55003E+00_JPRB, 0.48568E+00_JPRB, 0.49197E+00_JPRB, 0.49716E+00_JPRB, 0.48158E+00_JPRB, 0.49883E+00_JPRB, 0.51604E+00_JPRB ,&
&  0.49649E+00_JPRB, 0.45106E+00_JPRB, 0.45266E+00_JPRB, 0.45924E+00_JPRB, 0.47881E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.45742E+00_JPRB, 0.46621E+00_JPRB, 0.51212E+00_JPRB, 0.55002E+00_JPRB, 0.58419E+00_JPRB, 0.61393E+00_JPRB, 0.65620E+00_JPRB ,&
&  0.71585E+00_JPRB, 0.76025E+00_JPRB, 0.79040E+00_JPRB, 0.79350E+00_JPRB, 0.79739E+00_JPRB, 0.84403E+00_JPRB, 0.93229E+00_JPRB ,&
&  0.10805E+01_JPRB, 0.13167E+01_JPRB, 0.16506E+01_JPRB, 0.19394E+01_JPRB, 0.20643E+01_JPRB, 0.18562E+01_JPRB, 0.17705E+01_JPRB ,&
&  0.18423E+01_JPRB, 0.21998E+01_JPRB, 0.23061E+01_JPRB, 0.24004E+01_JPRB, 0.22825E+01_JPRB, 0.21287E+01_JPRB, 0.14189E+01_JPRB ,&
&  0.13971E+01_JPRB, 0.14544E+01_JPRB, 0.14944E+01_JPRB, 0.15639E+01_JPRB, 0.15815E+01_JPRB, 0.15392E+01_JPRB, 0.14710E+01_JPRB ,&
&  0.13355E+01_JPRB, 0.12276E+01_JPRB, 0.11681E+01_JPRB, 0.11999E+01_JPRB, 0.13202E+01_JPRB, 0.14364E+01_JPRB, 0.15523E+01_JPRB ,&
&  0.16174E+01_JPRB, 0.16034E+01_JPRB, 0.15561E+01_JPRB, 0.14838E+01_JPRB, 0.16939E+01_JPRB, 0.21168E+01_JPRB, 0.24800E+01_JPRB ,&
&  0.28111E+01_JPRB, 0.33096E+01_JPRB, 0.37466E+01_JPRB, 0.40449E+01_JPRB, 0.41888E+01_JPRB, 0.37757E+01_JPRB, 0.29811E+01_JPRB ,&
&  0.23159E+01_JPRB, 0.17735E+01_JPRB, 0.13890E+01_JPRB, 0.13607E+01_JPRB, 0.14566E+01_JPRB, 0.14680E+01_JPRB, 0.13655E+01_JPRB ,&
&  0.12845E+01_JPRB, 0.12177E+01_JPRB, 0.12216E+01_JPRB, 0.12030E+01_JPRB, 0.12130E+01_JPRB, 0.11930E+01_JPRB, 0.14455E+01_JPRB ,&
&  0.13244E+01_JPRB, 0.71335E+00_JPRB, 0.49384E+00_JPRB, 0.52857E+00_JPRB, 0.36187E+00_JPRB, 0.26824E+00_JPRB, 0.33270E+00_JPRB ,&
&  0.40613E+00_JPRB, 0.63294E+00_JPRB, 0.87598E+00_JPRB, 0.90170E+00_JPRB, 0.11865E+01_JPRB, 0.12637E+01_JPRB, 0.13423E+01_JPRB ,&
&  0.81500E+00_JPRB, 0.65034E+00_JPRB, 0.53538E+00_JPRB, 0.50691E+00_JPRB, 0.49038E+00_JPRB, 0.50185E+00_JPRB, 0.50179E+00_JPRB ,&
&  0.47777E+00_JPRB, 0.43786E+00_JPRB, 0.43433E+00_JPRB, 0.43835E+00_JPRB, 0.44336E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.47815E+00_JPRB, 0.45209E+00_JPRB, 0.45609E+00_JPRB, 0.46263E+00_JPRB, 0.51431E+00_JPRB, 0.57321E+00_JPRB, 0.60475E+00_JPRB ,&
&  0.66625E+00_JPRB, 0.71727E+00_JPRB, 0.72887E+00_JPRB, 0.71067E+00_JPRB, 0.68969E+00_JPRB, 0.69492E+00_JPRB, 0.72926E+00_JPRB ,&
&  0.81495E+00_JPRB, 0.10021E+01_JPRB, 0.13828E+01_JPRB, 0.17739E+01_JPRB, 0.19934E+01_JPRB, 0.19498E+01_JPRB, 0.21240E+01_JPRB ,&
&  0.21885E+01_JPRB, 0.26812E+01_JPRB, 0.34587E+01_JPRB, 0.37161E+01_JPRB, 0.29179E+01_JPRB, 0.25420E+01_JPRB, 0.18948E+01_JPRB ,&
&  0.17715E+01_JPRB, 0.17927E+01_JPRB, 0.18688E+01_JPRB, 0.19810E+01_JPRB, 0.19988E+01_JPRB, 0.19072E+01_JPRB, 0.18242E+01_JPRB ,&
&  0.16388E+01_JPRB, 0.14555E+01_JPRB, 0.12491E+01_JPRB, 0.11718E+01_JPRB, 0.12487E+01_JPRB, 0.14086E+01_JPRB, 0.15835E+01_JPRB ,&
&  0.17377E+01_JPRB, 0.17859E+01_JPRB, 0.17295E+01_JPRB, 0.17519E+01_JPRB, 0.14433E+01_JPRB, 0.21504E+01_JPRB, 0.27861E+01_JPRB ,&
&  0.32296E+01_JPRB, 0.36708E+01_JPRB, 0.37633E+01_JPRB, 0.44802E+01_JPRB, 0.48301E+01_JPRB, 0.42413E+01_JPRB, 0.34285E+01_JPRB ,&
&  0.27737E+01_JPRB, 0.20469E+01_JPRB, 0.16495E+01_JPRB, 0.16178E+01_JPRB, 0.16188E+01_JPRB, 0.15883E+01_JPRB, 0.11526E+01_JPRB ,&
&  0.11165E+01_JPRB, 0.13895E+01_JPRB, 0.13794E+01_JPRB, 0.11694E+01_JPRB, 0.12096E+01_JPRB, 0.12001E+01_JPRB, 0.11063E+01_JPRB ,&
&  0.60296E+00_JPRB, 0.43569E+00_JPRB, 0.40675E+00_JPRB, 0.37461E+00_JPRB, 0.33241E+00_JPRB, 0.29805E+00_JPRB, 0.44890E+00_JPRB ,&
&  0.45583E+00_JPRB, 0.73944E+00_JPRB, 0.96498E+00_JPRB, 0.10360E+01_JPRB, 0.13171E+01_JPRB, 0.16397E+01_JPRB, 0.22636E+01_JPRB ,&
&  0.11445E+01_JPRB, 0.84940E+00_JPRB, 0.62210E+00_JPRB, 0.52831E+00_JPRB, 0.51125E+00_JPRB, 0.50233E+00_JPRB, 0.48353E+00_JPRB ,&
&  0.46701E+00_JPRB, 0.45917E+00_JPRB, 0.45702E+00_JPRB, 0.44907E+00_JPRB, 0.47191E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.54497E+00_JPRB, 0.50237E+00_JPRB, 0.47834E+00_JPRB, 0.48589E+00_JPRB, 0.52272E+00_JPRB, 0.54513E+00_JPRB, 0.56492E+00_JPRB ,&
&  0.61798E+00_JPRB, 0.67854E+00_JPRB, 0.69918E+00_JPRB, 0.65984E+00_JPRB, 0.63007E+00_JPRB, 0.62899E+00_JPRB, 0.63520E+00_JPRB ,&
&  0.68118E+00_JPRB, 0.80721E+00_JPRB, 0.11399E+01_JPRB, 0.13952E+01_JPRB, 0.17155E+01_JPRB, 0.17846E+01_JPRB, 0.21639E+01_JPRB ,&
&  0.23672E+01_JPRB, 0.28256E+01_JPRB, 0.35054E+01_JPRB, 0.36411E+01_JPRB, 0.38250E+01_JPRB, 0.30940E+01_JPRB, 0.26952E+01_JPRB ,&
&  0.22011E+01_JPRB, 0.20211E+01_JPRB, 0.19933E+01_JPRB, 0.21772E+01_JPRB, 0.23139E+01_JPRB, 0.22867E+01_JPRB, 0.21442E+01_JPRB ,&
&  0.18356E+01_JPRB, 0.16301E+01_JPRB, 0.13467E+01_JPRB, 0.12402E+01_JPRB, 0.11901E+01_JPRB, 0.12818E+01_JPRB, 0.14893E+01_JPRB ,&
&  0.18183E+01_JPRB, 0.20414E+01_JPRB, 0.19635E+01_JPRB, 0.18938E+01_JPRB, 0.17247E+01_JPRB, 0.18776E+01_JPRB, 0.28856E+01_JPRB ,&
&  0.37545E+01_JPRB, 0.43724E+01_JPRB, 0.45473E+01_JPRB, 0.54659E+01_JPRB, 0.53143E+01_JPRB, 0.44728E+01_JPRB, 0.39279E+01_JPRB ,&
&  0.31902E+01_JPRB, 0.26129E+01_JPRB, 0.19700E+01_JPRB, 0.18086E+01_JPRB, 0.17046E+01_JPRB, 0.13125E+01_JPRB, 0.11725E+01_JPRB ,&
&  0.13584E+01_JPRB, 0.14267E+01_JPRB, 0.13761E+01_JPRB, 0.11614E+01_JPRB, 0.12501E+01_JPRB, 0.10774E+01_JPRB, 0.67072E+00_JPRB ,&
&  0.57134E+00_JPRB, 0.51640E+00_JPRB, 0.42472E+00_JPRB, 0.37715E+00_JPRB, 0.33877E+00_JPRB, 0.32326E+00_JPRB, 0.44717E+00_JPRB ,&
&  0.56045E+00_JPRB, 0.88112E+00_JPRB, 0.10742E+01_JPRB, 0.11556E+01_JPRB, 0.13224E+01_JPRB, 0.18128E+01_JPRB, 0.22715E+01_JPRB ,&
&  0.15820E+01_JPRB, 0.12551E+01_JPRB, 0.71297E+00_JPRB, 0.56349E+00_JPRB, 0.52592E+00_JPRB, 0.50978E+00_JPRB, 0.49162E+00_JPRB ,&
&  0.47460E+00_JPRB, 0.48031E+00_JPRB, 0.48693E+00_JPRB, 0.49541E+00_JPRB, 0.51175E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.56926E+00_JPRB, 0.54792E+00_JPRB, 0.51898E+00_JPRB, 0.53480E+00_JPRB, 0.55636E+00_JPRB, 0.58244E+00_JPRB, 0.57660E+00_JPRB ,&
&  0.58883E+00_JPRB, 0.61870E+00_JPRB, 0.64260E+00_JPRB, 0.62865E+00_JPRB, 0.58679E+00_JPRB, 0.59357E+00_JPRB, 0.59993E+00_JPRB ,&
&  0.64869E+00_JPRB, 0.79099E+00_JPRB, 0.99299E+00_JPRB, 0.10936E+01_JPRB, 0.12216E+01_JPRB, 0.13491E+01_JPRB, 0.17591E+01_JPRB ,&
&  0.23348E+01_JPRB, 0.27900E+01_JPRB, 0.32596E+01_JPRB, 0.37965E+01_JPRB, 0.43160E+01_JPRB, 0.37819E+01_JPRB, 0.36796E+01_JPRB ,&
&  0.30249E+01_JPRB, 0.24916E+01_JPRB, 0.23074E+01_JPRB, 0.23123E+01_JPRB, 0.24672E+01_JPRB, 0.25117E+01_JPRB, 0.23824E+01_JPRB ,&
&  0.20546E+01_JPRB, 0.16858E+01_JPRB, 0.14230E+01_JPRB, 0.12519E+01_JPRB, 0.11725E+01_JPRB, 0.12164E+01_JPRB, 0.14365E+01_JPRB ,&
&  0.19448E+01_JPRB, 0.24314E+01_JPRB, 0.24039E+01_JPRB, 0.21326E+01_JPRB, 0.23541E+01_JPRB, 0.25543E+01_JPRB, 0.31844E+01_JPRB ,&
&  0.36194E+01_JPRB, 0.38524E+01_JPRB, 0.53067E+01_JPRB, 0.58061E+01_JPRB, 0.52903E+01_JPRB, 0.46268E+01_JPRB, 0.38835E+01_JPRB ,&
&  0.32860E+01_JPRB, 0.27081E+01_JPRB, 0.21158E+01_JPRB, 0.19141E+01_JPRB, 0.15966E+01_JPRB, 0.12909E+01_JPRB, 0.14281E+01_JPRB ,&
&  0.15531E+01_JPRB, 0.15757E+01_JPRB, 0.14352E+01_JPRB, 0.11675E+01_JPRB, 0.11049E+01_JPRB, 0.86030E+00_JPRB, 0.63850E+00_JPRB ,&
&  0.56756E+00_JPRB, 0.53416E+00_JPRB, 0.46045E+00_JPRB, 0.45798E+00_JPRB, 0.40885E+00_JPRB, 0.40768E+00_JPRB, 0.50198E+00_JPRB ,&
&  0.67310E+00_JPRB, 0.93041E+00_JPRB, 0.10394E+01_JPRB, 0.10800E+01_JPRB, 0.11978E+01_JPRB, 0.13407E+01_JPRB, 0.19999E+01_JPRB ,&
&  0.13604E+01_JPRB, 0.14114E+01_JPRB, 0.96982E+00_JPRB, 0.68641E+00_JPRB, 0.57903E+00_JPRB, 0.53500E+00_JPRB, 0.52431E+00_JPRB ,&
&  0.52390E+00_JPRB, 0.52350E+00_JPRB, 0.52825E+00_JPRB, 0.53205E+00_JPRB, 0.54285E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.56981E+00_JPRB, 0.58329E+00_JPRB, 0.58003E+00_JPRB, 0.58961E+00_JPRB, 0.60069E+00_JPRB, 0.60932E+00_JPRB, 0.59892E+00_JPRB ,&
&  0.58589E+00_JPRB, 0.59804E+00_JPRB, 0.59609E+00_JPRB, 0.58150E+00_JPRB, 0.55976E+00_JPRB, 0.56626E+00_JPRB, 0.58247E+00_JPRB ,&
&  0.65229E+00_JPRB, 0.86739E+00_JPRB, 0.72926E+00_JPRB, 0.75710E+00_JPRB, 0.89899E+00_JPRB, 0.10825E+01_JPRB, 0.13296E+01_JPRB ,&
&  0.20721E+01_JPRB, 0.27329E+01_JPRB, 0.32014E+01_JPRB, 0.40780E+01_JPRB, 0.53646E+01_JPRB, 0.50998E+01_JPRB, 0.47658E+01_JPRB ,&
&  0.40287E+01_JPRB, 0.32923E+01_JPRB, 0.29976E+01_JPRB, 0.28080E+01_JPRB, 0.27593E+01_JPRB, 0.27154E+01_JPRB, 0.25332E+01_JPRB ,&
&  0.21991E+01_JPRB, 0.17537E+01_JPRB, 0.14192E+01_JPRB, 0.13263E+01_JPRB, 0.12539E+01_JPRB, 0.12969E+01_JPRB, 0.14607E+01_JPRB ,&
&  0.20328E+01_JPRB, 0.27167E+01_JPRB, 0.29194E+01_JPRB, 0.27566E+01_JPRB, 0.23909E+01_JPRB, 0.28873E+01_JPRB, 0.47374E+01_JPRB ,&
&  0.49279E+01_JPRB, 0.52442E+01_JPRB, 0.56332E+01_JPRB, 0.66168E+01_JPRB, 0.52834E+01_JPRB, 0.39961E+01_JPRB, 0.36400E+01_JPRB ,&
&  0.26632E+01_JPRB, 0.20468E+01_JPRB, 0.18762E+01_JPRB, 0.17203E+01_JPRB, 0.15473E+01_JPRB, 0.16395E+01_JPRB, 0.21957E+01_JPRB ,&
&  0.17939E+01_JPRB, 0.19094E+01_JPRB, 0.19600E+01_JPRB, 0.17154E+01_JPRB, 0.11139E+01_JPRB, 0.95096E+00_JPRB, 0.12460E+01_JPRB ,&
&  0.12913E+01_JPRB, 0.87242E+00_JPRB, 0.59850E+00_JPRB, 0.71989E+00_JPRB, 0.54883E+00_JPRB, 0.56793E+00_JPRB, 0.67690E+00_JPRB ,&
&  0.75090E+00_JPRB, 0.86014E+00_JPRB, 0.11590E+01_JPRB, 0.11667E+01_JPRB, 0.11393E+01_JPRB, 0.11452E+01_JPRB, 0.14310E+01_JPRB ,&
&  0.14207E+01_JPRB, 0.13957E+01_JPRB, 0.11858E+01_JPRB, 0.88851E+00_JPRB, 0.71653E+00_JPRB, 0.63674E+00_JPRB, 0.60326E+00_JPRB ,&
&  0.59482E+00_JPRB, 0.60943E+00_JPRB, 0.60456E+00_JPRB, 0.58316E+00_JPRB, 0.57493E+00_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.62962E+00_JPRB, 0.63966E+00_JPRB, 0.64322E+00_JPRB, 0.65078E+00_JPRB, 0.65967E+00_JPRB, 0.62757E+00_JPRB, 0.61303E+00_JPRB ,&
&  0.59156E+00_JPRB, 0.57893E+00_JPRB, 0.57856E+00_JPRB, 0.58115E+00_JPRB, 0.56500E+00_JPRB, 0.56415E+00_JPRB, 0.57768E+00_JPRB ,&
&  0.64402E+00_JPRB, 0.71551E+00_JPRB, 0.65492E+00_JPRB, 0.65746E+00_JPRB, 0.80924E+00_JPRB, 0.94162E+00_JPRB, 0.12212E+01_JPRB ,&
&  0.18576E+01_JPRB, 0.25888E+01_JPRB, 0.33804E+01_JPRB, 0.43281E+01_JPRB, 0.50686E+01_JPRB, 0.56665E+01_JPRB, 0.58599E+01_JPRB ,&
&  0.56000E+01_JPRB, 0.46284E+01_JPRB, 0.40236E+01_JPRB, 0.35852E+01_JPRB, 0.31985E+01_JPRB, 0.28811E+01_JPRB, 0.26247E+01_JPRB ,&
&  0.24132E+01_JPRB, 0.20528E+01_JPRB, 0.16584E+01_JPRB, 0.14036E+01_JPRB, 0.13455E+01_JPRB, 0.13464E+01_JPRB, 0.14803E+01_JPRB ,&
&  0.19936E+01_JPRB, 0.26839E+01_JPRB, 0.33717E+01_JPRB, 0.35269E+01_JPRB, 0.28727E+01_JPRB, 0.31211E+01_JPRB, 0.45040E+01_JPRB ,&
&  0.53366E+01_JPRB, 0.54011E+01_JPRB, 0.61692E+01_JPRB, 0.64595E+01_JPRB, 0.54094E+01_JPRB, 0.41320E+01_JPRB, 0.40544E+01_JPRB ,&
&  0.31364E+01_JPRB, 0.23557E+01_JPRB, 0.18856E+01_JPRB, 0.15051E+01_JPRB, 0.15989E+01_JPRB, 0.22235E+01_JPRB, 0.22126E+01_JPRB ,&
&  0.21450E+01_JPRB, 0.21349E+01_JPRB, 0.20032E+01_JPRB, 0.16818E+01_JPRB, 0.13234E+01_JPRB, 0.12261E+01_JPRB, 0.14999E+01_JPRB ,&
&  0.15161E+01_JPRB, 0.14749E+01_JPRB, 0.12368E+01_JPRB, 0.10354E+01_JPRB, 0.84745E+00_JPRB, 0.89282E+00_JPRB, 0.88247E+00_JPRB ,&
&  0.87827E+00_JPRB, 0.84807E+00_JPRB, 0.10863E+01_JPRB, 0.11595E+01_JPRB, 0.10567E+01_JPRB, 0.95314E+00_JPRB, 0.11353E+01_JPRB ,&
&  0.11107E+01_JPRB, 0.12862E+01_JPRB, 0.11445E+01_JPRB, 0.94273E+00_JPRB, 0.82490E+00_JPRB, 0.76648E+00_JPRB, 0.72055E+00_JPRB ,&
&  0.71613E+00_JPRB, 0.72508E+00_JPRB, 0.71510E+00_JPRB, 0.67650E+00_JPRB, 0.65318E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.69311E+00_JPRB, 0.69941E+00_JPRB, 0.70023E+00_JPRB, 0.70046E+00_JPRB, 0.69793E+00_JPRB, 0.65496E+00_JPRB, 0.62181E+00_JPRB ,&
&  0.59899E+00_JPRB, 0.58492E+00_JPRB, 0.58888E+00_JPRB, 0.58459E+00_JPRB, 0.57441E+00_JPRB, 0.56873E+00_JPRB, 0.58116E+00_JPRB ,&
&  0.63909E+00_JPRB, 0.70467E+00_JPRB, 0.63117E+00_JPRB, 0.69239E+00_JPRB, 0.77684E+00_JPRB, 0.90919E+00_JPRB, 0.13193E+01_JPRB ,&
&  0.17520E+01_JPRB, 0.23028E+01_JPRB, 0.30380E+01_JPRB, 0.38880E+01_JPRB, 0.45597E+01_JPRB, 0.53188E+01_JPRB, 0.60573E+01_JPRB ,&
&  0.56504E+01_JPRB, 0.55524E+01_JPRB, 0.48156E+01_JPRB, 0.40658E+01_JPRB, 0.34440E+01_JPRB, 0.29702E+01_JPRB, 0.27200E+01_JPRB ,&
&  0.25186E+01_JPRB, 0.23789E+01_JPRB, 0.20942E+01_JPRB, 0.17702E+01_JPRB, 0.15825E+01_JPRB, 0.15059E+01_JPRB, 0.15288E+01_JPRB ,&
&  0.17079E+01_JPRB, 0.24275E+01_JPRB, 0.35181E+01_JPRB, 0.44247E+01_JPRB, 0.34963E+01_JPRB, 0.38459E+01_JPRB, 0.44725E+01_JPRB ,&
&  0.65057E+01_JPRB, 0.68172E+01_JPRB, 0.63419E+01_JPRB, 0.63198E+01_JPRB, 0.45865E+01_JPRB, 0.40278E+01_JPRB, 0.40240E+01_JPRB ,&
&  0.38429E+01_JPRB, 0.34006E+01_JPRB, 0.32410E+01_JPRB, 0.22530E+01_JPRB, 0.18506E+01_JPRB, 0.23298E+01_JPRB, 0.21413E+01_JPRB ,&
&  0.19898E+01_JPRB, 0.19303E+01_JPRB, 0.18217E+01_JPRB, 0.17575E+01_JPRB, 0.15387E+01_JPRB, 0.14266E+01_JPRB, 0.12335E+01_JPRB ,&
&  0.12027E+01_JPRB, 0.10594E+01_JPRB, 0.11215E+01_JPRB, 0.10204E+01_JPRB, 0.95688E+00_JPRB, 0.94259E+00_JPRB, 0.92148E+00_JPRB ,&
&  0.96052E+00_JPRB, 0.94222E+00_JPRB, 0.10662E+01_JPRB, 0.11485E+01_JPRB, 0.10450E+01_JPRB, 0.91536E+00_JPRB, 0.88743E+00_JPRB ,&
&  0.95820E+00_JPRB, 0.10665E+01_JPRB, 0.10047E+01_JPRB, 0.91679E+00_JPRB, 0.85385E+00_JPRB, 0.81669E+00_JPRB, 0.79118E+00_JPRB ,&
&  0.76189E+00_JPRB, 0.74869E+00_JPRB, 0.74062E+00_JPRB, 0.72801E+00_JPRB, 0.71249E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.68680E+00_JPRB, 0.68972E+00_JPRB, 0.73136E+00_JPRB, 0.71190E+00_JPRB, 0.71759E+00_JPRB, 0.68186E+00_JPRB, 0.65404E+00_JPRB ,&
&  0.63214E+00_JPRB, 0.62045E+00_JPRB, 0.60754E+00_JPRB, 0.59282E+00_JPRB, 0.57798E+00_JPRB, 0.56629E+00_JPRB, 0.57105E+00_JPRB ,&
&  0.59269E+00_JPRB, 0.74300E+00_JPRB, 0.67749E+00_JPRB, 0.65631E+00_JPRB, 0.70979E+00_JPRB, 0.94275E+00_JPRB, 0.12933E+01_JPRB ,&
&  0.16418E+01_JPRB, 0.19733E+01_JPRB, 0.25399E+01_JPRB, 0.30920E+01_JPRB, 0.36855E+01_JPRB, 0.43133E+01_JPRB, 0.50642E+01_JPRB ,&
&  0.55274E+01_JPRB, 0.46835E+01_JPRB, 0.43885E+01_JPRB, 0.38221E+01_JPRB, 0.33698E+01_JPRB, 0.29874E+01_JPRB, 0.27188E+01_JPRB ,&
&  0.25050E+01_JPRB, 0.23516E+01_JPRB, 0.21552E+01_JPRB, 0.19244E+01_JPRB, 0.16900E+01_JPRB, 0.15467E+01_JPRB, 0.15042E+01_JPRB ,&
&  0.15796E+01_JPRB, 0.19527E+01_JPRB, 0.29238E+01_JPRB, 0.43413E+01_JPRB, 0.60375E+01_JPRB, 0.62336E+01_JPRB, 0.59177E+01_JPRB ,&
&  0.52003E+01_JPRB, 0.51841E+01_JPRB, 0.68343E+01_JPRB, 0.55759E+01_JPRB, 0.51117E+01_JPRB, 0.47789E+01_JPRB, 0.43355E+01_JPRB ,&
&  0.40449E+01_JPRB, 0.33804E+01_JPRB, 0.30783E+01_JPRB, 0.24324E+01_JPRB, 0.24310E+01_JPRB, 0.21986E+01_JPRB, 0.20024E+01_JPRB ,&
&  0.18220E+01_JPRB, 0.18035E+01_JPRB, 0.17244E+01_JPRB, 0.16515E+01_JPRB, 0.16241E+01_JPRB, 0.16893E+01_JPRB, 0.16200E+01_JPRB ,&
&  0.14410E+01_JPRB, 0.14198E+01_JPRB, 0.12211E+01_JPRB, 0.10269E+01_JPRB, 0.87990E+00_JPRB, 0.87822E+00_JPRB, 0.89047E+00_JPRB ,&
&  0.92096E+00_JPRB, 0.95817E+00_JPRB, 0.10596E+01_JPRB, 0.10752E+01_JPRB, 0.98815E+00_JPRB, 0.90483E+00_JPRB, 0.84236E+00_JPRB ,&
&  0.84968E+00_JPRB, 0.91067E+00_JPRB, 0.94187E+00_JPRB, 0.86173E+00_JPRB, 0.86086E+00_JPRB, 0.82852E+00_JPRB, 0.79945E+00_JPRB ,&
&  0.76159E+00_JPRB, 0.72584E+00_JPRB, 0.71289E+00_JPRB, 0.70634E+00_JPRB, 0.70085E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.69089E+00_JPRB, 0.68184E+00_JPRB, 0.71961E+00_JPRB, 0.70211E+00_JPRB, 0.69127E+00_JPRB, 0.67278E+00_JPRB, 0.65396E+00_JPRB ,&
&  0.63555E+00_JPRB, 0.62757E+00_JPRB, 0.61869E+00_JPRB, 0.59672E+00_JPRB, 0.57803E+00_JPRB, 0.56629E+00_JPRB, 0.56645E+00_JPRB ,&
&  0.56819E+00_JPRB, 0.72508E+00_JPRB, 0.69846E+00_JPRB, 0.75079E+00_JPRB, 0.78161E+00_JPRB, 0.96736E+00_JPRB, 0.11626E+01_JPRB ,&
&  0.14075E+01_JPRB, 0.16022E+01_JPRB, 0.18326E+01_JPRB, 0.21461E+01_JPRB, 0.24822E+01_JPRB, 0.28503E+01_JPRB, 0.34534E+01_JPRB ,&
&  0.36284E+01_JPRB, 0.35759E+01_JPRB, 0.33753E+01_JPRB, 0.34268E+01_JPRB, 0.32859E+01_JPRB, 0.28251E+01_JPRB, 0.26367E+01_JPRB ,&
&  0.23679E+01_JPRB, 0.21792E+01_JPRB, 0.20296E+01_JPRB, 0.19324E+01_JPRB, 0.18358E+01_JPRB, 0.17089E+01_JPRB, 0.16192E+01_JPRB ,&
&  0.15517E+01_JPRB, 0.16328E+01_JPRB, 0.21030E+01_JPRB, 0.32729E+01_JPRB, 0.51398E+01_JPRB, 0.63709E+01_JPRB, 0.64474E+01_JPRB ,&
&  0.68417E+01_JPRB, 0.66572E+01_JPRB, 0.50238E+01_JPRB, 0.53277E+01_JPRB, 0.60537E+01_JPRB, 0.52635E+01_JPRB, 0.44638E+01_JPRB ,&
&  0.43152E+01_JPRB, 0.39185E+01_JPRB, 0.36247E+01_JPRB, 0.30696E+01_JPRB, 0.25194E+01_JPRB, 0.22613E+01_JPRB, 0.20757E+01_JPRB ,&
&  0.19095E+01_JPRB, 0.18596E+01_JPRB, 0.18210E+01_JPRB, 0.17192E+01_JPRB, 0.16289E+01_JPRB, 0.15722E+01_JPRB, 0.15944E+01_JPRB ,&
&  0.14976E+01_JPRB, 0.13511E+01_JPRB, 0.10709E+01_JPRB, 0.10295E+01_JPRB, 0.81224E+00_JPRB, 0.79379E+00_JPRB, 0.85401E+00_JPRB ,&
&  0.84399E+00_JPRB, 0.92561E+00_JPRB, 0.98757E+00_JPRB, 0.89610E+00_JPRB, 0.84841E+00_JPRB, 0.81436E+00_JPRB, 0.81721E+00_JPRB ,&
&  0.81567E+00_JPRB, 0.77609E+00_JPRB, 0.84323E+00_JPRB, 0.81383E+00_JPRB, 0.82253E+00_JPRB, 0.79763E+00_JPRB, 0.76573E+00_JPRB ,&
&  0.75979E+00_JPRB, 0.74070E+00_JPRB, 0.72491E+00_JPRB, 0.72296E+00_JPRB, 0.71034E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.69674E+00_JPRB, 0.68671E+00_JPRB, 0.70565E+00_JPRB, 0.66855E+00_JPRB, 0.64046E+00_JPRB, 0.61957E+00_JPRB, 0.60556E+00_JPRB ,&
&  0.59753E+00_JPRB, 0.59004E+00_JPRB, 0.58285E+00_JPRB, 0.57080E+00_JPRB, 0.56072E+00_JPRB, 0.54924E+00_JPRB, 0.54779E+00_JPRB ,&
&  0.55945E+00_JPRB, 0.58179E+00_JPRB, 0.61274E+00_JPRB, 0.62484E+00_JPRB, 0.76225E+00_JPRB, 0.82033E+00_JPRB, 0.91944E+00_JPRB ,&
&  0.10364E+01_JPRB, 0.11702E+01_JPRB, 0.12370E+01_JPRB, 0.13035E+01_JPRB, 0.14374E+01_JPRB, 0.17181E+01_JPRB, 0.20883E+01_JPRB ,&
&  0.22297E+01_JPRB, 0.21533E+01_JPRB, 0.21565E+01_JPRB, 0.22610E+01_JPRB, 0.24378E+01_JPRB, 0.21474E+01_JPRB, 0.20680E+01_JPRB ,&
&  0.19143E+01_JPRB, 0.17893E+01_JPRB, 0.16996E+01_JPRB, 0.16446E+01_JPRB, 0.16390E+01_JPRB, 0.16526E+01_JPRB, 0.16714E+01_JPRB ,&
&  0.16557E+01_JPRB, 0.15928E+01_JPRB, 0.16982E+01_JPRB, 0.24655E+01_JPRB, 0.47029E+01_JPRB, 0.72381E+01_JPRB, 0.88741E+01_JPRB ,&
&  0.80487E+01_JPRB, 0.76133E+01_JPRB, 0.76093E+01_JPRB, 0.75161E+01_JPRB, 0.65725E+01_JPRB, 0.55511E+01_JPRB, 0.48638E+01_JPRB ,&
&  0.43999E+01_JPRB, 0.41206E+01_JPRB, 0.39587E+01_JPRB, 0.35410E+01_JPRB, 0.30237E+01_JPRB, 0.27712E+01_JPRB, 0.25574E+01_JPRB ,&
&  0.23296E+01_JPRB, 0.22431E+01_JPRB, 0.21950E+01_JPRB, 0.20171E+01_JPRB, 0.18879E+01_JPRB, 0.17382E+01_JPRB, 0.17380E+01_JPRB ,&
&  0.16915E+01_JPRB, 0.12876E+01_JPRB, 0.10863E+01_JPRB, 0.11605E+01_JPRB, 0.90929E+00_JPRB, 0.88362E+00_JPRB, 0.95960E+00_JPRB ,&
&  0.90149E+00_JPRB, 0.87128E+00_JPRB, 0.86844E+00_JPRB, 0.78213E+00_JPRB, 0.76170E+00_JPRB, 0.76534E+00_JPRB, 0.75355E+00_JPRB ,&
&  0.74010E+00_JPRB, 0.73189E+00_JPRB, 0.80824E+00_JPRB, 0.75594E+00_JPRB, 0.75635E+00_JPRB, 0.74624E+00_JPRB, 0.72734E+00_JPRB ,&
&  0.72199E+00_JPRB, 0.70981E+00_JPRB, 0.69439E+00_JPRB, 0.69129E+00_JPRB, 0.69963E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.66002E+00_JPRB, 0.66069E+00_JPRB, 0.66878E+00_JPRB, 0.66365E+00_JPRB, 0.64062E+00_JPRB, 0.61326E+00_JPRB, 0.59142E+00_JPRB ,&
&  0.57910E+00_JPRB, 0.57114E+00_JPRB, 0.56153E+00_JPRB, 0.54995E+00_JPRB, 0.54025E+00_JPRB, 0.53031E+00_JPRB, 0.53969E+00_JPRB ,&
&  0.47319E+00_JPRB, 0.47768E+00_JPRB, 0.49145E+00_JPRB, 0.53904E+00_JPRB, 0.66655E+00_JPRB, 0.68335E+00_JPRB, 0.72623E+00_JPRB ,&
&  0.77838E+00_JPRB, 0.83317E+00_JPRB, 0.86447E+00_JPRB, 0.90930E+00_JPRB, 0.98880E+00_JPRB, 0.11505E+01_JPRB, 0.13512E+01_JPRB ,&
&  0.13668E+01_JPRB, 0.12372E+01_JPRB, 0.11979E+01_JPRB, 0.11651E+01_JPRB, 0.12186E+01_JPRB, 0.12874E+01_JPRB, 0.13901E+01_JPRB ,&
&  0.13811E+01_JPRB, 0.13590E+01_JPRB, 0.13506E+01_JPRB, 0.13510E+01_JPRB, 0.13554E+01_JPRB, 0.13741E+01_JPRB, 0.14034E+01_JPRB ,&
&  0.14358E+01_JPRB, 0.14313E+01_JPRB, 0.14601E+01_JPRB, 0.18768E+01_JPRB, 0.38499E+01_JPRB, 0.74134E+01_JPRB, 0.11414E+02_JPRB ,&
&  0.97054E+01_JPRB, 0.83429E+01_JPRB, 0.80693E+01_JPRB, 0.80272E+01_JPRB, 0.68850E+01_JPRB, 0.56559E+01_JPRB, 0.46533E+01_JPRB ,&
&  0.41056E+01_JPRB, 0.37451E+01_JPRB, 0.36347E+01_JPRB, 0.35138E+01_JPRB, 0.31995E+01_JPRB, 0.30161E+01_JPRB, 0.28902E+01_JPRB ,&
&  0.26888E+01_JPRB, 0.25420E+01_JPRB, 0.23571E+01_JPRB, 0.21610E+01_JPRB, 0.20565E+01_JPRB, 0.20501E+01_JPRB, 0.20065E+01_JPRB ,&
&  0.19576E+01_JPRB, 0.17710E+01_JPRB, 0.15055E+01_JPRB, 0.14499E+01_JPRB, 0.11815E+01_JPRB, 0.11351E+01_JPRB, 0.12283E+01_JPRB ,&
&  0.10531E+01_JPRB, 0.89214E+00_JPRB, 0.83232E+00_JPRB, 0.80711E+00_JPRB, 0.80008E+00_JPRB, 0.81085E+00_JPRB, 0.78217E+00_JPRB ,&
&  0.73536E+00_JPRB, 0.76602E+00_JPRB, 0.79263E+00_JPRB, 0.70390E+00_JPRB, 0.68443E+00_JPRB, 0.66553E+00_JPRB, 0.61106E+00_JPRB ,&
&  0.64251E+00_JPRB, 0.63519E+00_JPRB, 0.63363E+00_JPRB, 0.63818E+00_JPRB, 0.64950E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.60511E+00_JPRB, 0.61289E+00_JPRB, 0.62166E+00_JPRB, 0.62084E+00_JPRB, 0.60529E+00_JPRB, 0.58672E+00_JPRB, 0.57635E+00_JPRB ,&
&  0.57194E+00_JPRB, 0.55904E+00_JPRB, 0.54353E+00_JPRB, 0.52605E+00_JPRB, 0.51469E+00_JPRB, 0.51689E+00_JPRB, 0.48575E+00_JPRB ,&
&  0.43967E+00_JPRB, 0.44668E+00_JPRB, 0.45090E+00_JPRB, 0.47757E+00_JPRB, 0.51737E+00_JPRB, 0.54289E+00_JPRB, 0.57127E+00_JPRB ,&
&  0.63255E+00_JPRB, 0.68555E+00_JPRB, 0.71779E+00_JPRB, 0.75406E+00_JPRB, 0.78526E+00_JPRB, 0.82669E+00_JPRB, 0.86965E+00_JPRB ,&
&  0.84427E+00_JPRB, 0.77699E+00_JPRB, 0.72565E+00_JPRB, 0.68087E+00_JPRB, 0.71785E+00_JPRB, 0.78689E+00_JPRB, 0.84109E+00_JPRB ,&
&  0.87741E+00_JPRB, 0.91138E+00_JPRB, 0.95083E+00_JPRB, 0.98358E+00_JPRB, 0.10039E+01_JPRB, 0.10303E+01_JPRB, 0.10563E+01_JPRB ,&
&  0.10938E+01_JPRB, 0.11234E+01_JPRB, 0.11611E+01_JPRB, 0.13431E+01_JPRB, 0.27224E+01_JPRB, 0.54843E+01_JPRB, 0.76591E+01_JPRB ,&
&  0.66281E+01_JPRB, 0.62664E+01_JPRB, 0.60169E+01_JPRB, 0.57313E+01_JPRB, 0.52435E+01_JPRB, 0.44824E+01_JPRB, 0.38322E+01_JPRB ,&
&  0.35409E+01_JPRB, 0.32322E+01_JPRB, 0.31157E+01_JPRB, 0.29674E+01_JPRB, 0.28557E+01_JPRB, 0.27259E+01_JPRB, 0.25029E+01_JPRB ,&
&  0.23329E+01_JPRB, 0.20905E+01_JPRB, 0.20193E+01_JPRB, 0.19146E+01_JPRB, 0.18665E+01_JPRB, 0.18592E+01_JPRB, 0.18472E+01_JPRB ,&
&  0.18259E+01_JPRB, 0.17803E+01_JPRB, 0.16266E+01_JPRB, 0.15090E+01_JPRB, 0.14118E+01_JPRB, 0.14650E+01_JPRB, 0.13353E+01_JPRB ,&
&  0.11260E+01_JPRB, 0.91059E+00_JPRB, 0.88353E+00_JPRB, 0.83778E+00_JPRB, 0.80278E+00_JPRB, 0.78401E+00_JPRB, 0.76236E+00_JPRB ,&
&  0.78109E+00_JPRB, 0.81313E+00_JPRB, 0.76182E+00_JPRB, 0.67111E+00_JPRB, 0.63191E+00_JPRB, 0.60179E+00_JPRB, 0.53074E+00_JPRB ,&
&  0.52959E+00_JPRB, 0.57684E+00_JPRB, 0.58314E+00_JPRB, 0.58765E+00_JPRB, 0.59269E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.56958E+00_JPRB, 0.57619E+00_JPRB, 0.57732E+00_JPRB, 0.58030E+00_JPRB, 0.58129E+00_JPRB, 0.58141E+00_JPRB, 0.55745E+00_JPRB ,&
&  0.58573E+00_JPRB, 0.57177E+00_JPRB, 0.54373E+00_JPRB, 0.52677E+00_JPRB, 0.52512E+00_JPRB, 0.49157E+00_JPRB, 0.42080E+00_JPRB ,&
&  0.40319E+00_JPRB, 0.41089E+00_JPRB, 0.42984E+00_JPRB, 0.44585E+00_JPRB, 0.45957E+00_JPRB, 0.46942E+00_JPRB, 0.48711E+00_JPRB ,&
&  0.51154E+00_JPRB, 0.53549E+00_JPRB, 0.56549E+00_JPRB, 0.59525E+00_JPRB, 0.59878E+00_JPRB, 0.59785E+00_JPRB, 0.61103E+00_JPRB ,&
&  0.59355E+00_JPRB, 0.56355E+00_JPRB, 0.54784E+00_JPRB, 0.50161E+00_JPRB, 0.53784E+00_JPRB, 0.55581E+00_JPRB, 0.57568E+00_JPRB ,&
&  0.60316E+00_JPRB, 0.63270E+00_JPRB, 0.66616E+00_JPRB, 0.69429E+00_JPRB, 0.71551E+00_JPRB, 0.74029E+00_JPRB, 0.76641E+00_JPRB ,&
&  0.79377E+00_JPRB, 0.83260E+00_JPRB, 0.88863E+00_JPRB, 0.98386E+00_JPRB, 0.17234E+01_JPRB, 0.34374E+01_JPRB, 0.45878E+01_JPRB ,&
&  0.42794E+01_JPRB, 0.45537E+01_JPRB, 0.45632E+01_JPRB, 0.39645E+01_JPRB, 0.39083E+01_JPRB, 0.38167E+01_JPRB, 0.34685E+01_JPRB ,&
&  0.33739E+01_JPRB, 0.31603E+01_JPRB, 0.28508E+01_JPRB, 0.25834E+01_JPRB, 0.23555E+01_JPRB, 0.21467E+01_JPRB, 0.19487E+01_JPRB ,&
&  0.18066E+01_JPRB, 0.16769E+01_JPRB, 0.16593E+01_JPRB, 0.16879E+01_JPRB, 0.16866E+01_JPRB, 0.16943E+01_JPRB, 0.17060E+01_JPRB ,&
&  0.16960E+01_JPRB, 0.16797E+01_JPRB, 0.16331E+01_JPRB, 0.15399E+01_JPRB, 0.14527E+01_JPRB, 0.13639E+01_JPRB, 0.12648E+01_JPRB ,&
&  0.11646E+01_JPRB, 0.10241E+01_JPRB, 0.93302E+00_JPRB, 0.89521E+00_JPRB, 0.83988E+00_JPRB, 0.77968E+00_JPRB, 0.76468E+00_JPRB ,&
&  0.75052E+00_JPRB, 0.69254E+00_JPRB, 0.68351E+00_JPRB, 0.62072E+00_JPRB, 0.58067E+00_JPRB, 0.55837E+00_JPRB, 0.50928E+00_JPRB ,&
&  0.48868E+00_JPRB, 0.52955E+00_JPRB, 0.54973E+00_JPRB, 0.56228E+00_JPRB, 0.56546E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.54506E+00_JPRB, 0.55392E+00_JPRB, 0.55881E+00_JPRB, 0.56428E+00_JPRB, 0.56441E+00_JPRB, 0.54269E+00_JPRB, 0.52429E+00_JPRB ,&
&  0.58495E+00_JPRB, 0.54669E+00_JPRB, 0.56044E+00_JPRB, 0.47173E+00_JPRB, 0.43162E+00_JPRB, 0.42309E+00_JPRB, 0.40730E+00_JPRB ,&
&  0.40230E+00_JPRB, 0.40797E+00_JPRB, 0.41475E+00_JPRB, 0.42163E+00_JPRB, 0.43128E+00_JPRB, 0.43322E+00_JPRB, 0.43427E+00_JPRB ,&
&  0.43942E+00_JPRB, 0.44950E+00_JPRB, 0.47012E+00_JPRB, 0.48088E+00_JPRB, 0.48793E+00_JPRB, 0.49307E+00_JPRB, 0.50279E+00_JPRB ,&
&  0.47945E+00_JPRB, 0.46423E+00_JPRB, 0.46792E+00_JPRB, 0.46449E+00_JPRB, 0.46313E+00_JPRB, 0.46738E+00_JPRB, 0.48112E+00_JPRB ,&
&  0.49995E+00_JPRB, 0.47749E+00_JPRB, 0.52501E+00_JPRB, 0.55169E+00_JPRB, 0.57500E+00_JPRB, 0.59613E+00_JPRB, 0.61704E+00_JPRB ,&
&  0.63813E+00_JPRB, 0.66329E+00_JPRB, 0.70158E+00_JPRB, 0.77848E+00_JPRB, 0.11248E+01_JPRB, 0.21108E+01_JPRB, 0.27927E+01_JPRB ,&
&  0.30593E+01_JPRB, 0.24134E+01_JPRB, 0.27774E+01_JPRB, 0.28712E+01_JPRB, 0.30024E+01_JPRB, 0.30070E+01_JPRB, 0.29246E+01_JPRB ,&
&  0.28945E+01_JPRB, 0.27791E+01_JPRB, 0.24847E+01_JPRB, 0.21085E+01_JPRB, 0.18209E+01_JPRB, 0.16497E+01_JPRB, 0.15385E+01_JPRB ,&
&  0.14332E+01_JPRB, 0.12857E+01_JPRB, 0.13575E+01_JPRB, 0.14318E+01_JPRB, 0.14653E+01_JPRB, 0.14935E+01_JPRB, 0.15434E+01_JPRB ,&
&  0.15792E+01_JPRB, 0.15588E+01_JPRB, 0.15204E+01_JPRB, 0.13999E+01_JPRB, 0.13415E+01_JPRB, 0.12881E+01_JPRB, 0.12118E+01_JPRB ,&
&  0.11550E+01_JPRB, 0.11008E+01_JPRB, 0.10506E+01_JPRB, 0.10428E+01_JPRB, 0.93867E+00_JPRB, 0.86707E+00_JPRB, 0.81512E+00_JPRB ,&
&  0.71730E+00_JPRB, 0.59929E+00_JPRB, 0.54755E+00_JPRB, 0.51398E+00_JPRB, 0.48592E+00_JPRB, 0.48594E+00_JPRB, 0.49376E+00_JPRB ,&
&  0.48002E+00_JPRB, 0.49097E+00_JPRB, 0.52066E+00_JPRB, 0.53185E+00_JPRB, 0.53474E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.52002E+00_JPRB, 0.53864E+00_JPRB, 0.55377E+00_JPRB, 0.55737E+00_JPRB, 0.54902E+00_JPRB, 0.52263E+00_JPRB, 0.48508E+00_JPRB ,&
&  0.47947E+00_JPRB, 0.45981E+00_JPRB, 0.43983E+00_JPRB, 0.42555E+00_JPRB, 0.41629E+00_JPRB, 0.40587E+00_JPRB, 0.39135E+00_JPRB ,&
&  0.39027E+00_JPRB, 0.39943E+00_JPRB, 0.41355E+00_JPRB, 0.41630E+00_JPRB, 0.41420E+00_JPRB, 0.41130E+00_JPRB, 0.40671E+00_JPRB ,&
&  0.40177E+00_JPRB, 0.40232E+00_JPRB, 0.41076E+00_JPRB, 0.42530E+00_JPRB, 0.43326E+00_JPRB, 0.43503E+00_JPRB, 0.43712E+00_JPRB ,&
&  0.43222E+00_JPRB, 0.42538E+00_JPRB, 0.42196E+00_JPRB, 0.42361E+00_JPRB, 0.42064E+00_JPRB, 0.43083E+00_JPRB, 0.43646E+00_JPRB ,&
&  0.39469E+00_JPRB, 0.40532E+00_JPRB, 0.46962E+00_JPRB, 0.49376E+00_JPRB, 0.51485E+00_JPRB, 0.53248E+00_JPRB, 0.54971E+00_JPRB ,&
&  0.56785E+00_JPRB, 0.59962E+00_JPRB, 0.64097E+00_JPRB, 0.69975E+00_JPRB, 0.88887E+00_JPRB, 0.13587E+01_JPRB, 0.18033E+01_JPRB ,&
&  0.22771E+01_JPRB, 0.19868E+01_JPRB, 0.18647E+01_JPRB, 0.19711E+01_JPRB, 0.24011E+01_JPRB, 0.23786E+01_JPRB, 0.23720E+01_JPRB ,&
&  0.23622E+01_JPRB, 0.22216E+01_JPRB, 0.20609E+01_JPRB, 0.18180E+01_JPRB, 0.15289E+01_JPRB, 0.13307E+01_JPRB, 0.12020E+01_JPRB ,&
&  0.10899E+01_JPRB, 0.10121E+01_JPRB, 0.98064E+00_JPRB, 0.10236E+01_JPRB, 0.10992E+01_JPRB, 0.11593E+01_JPRB, 0.12094E+01_JPRB ,&
&  0.12772E+01_JPRB, 0.13165E+01_JPRB, 0.12971E+01_JPRB, 0.12526E+01_JPRB, 0.11929E+01_JPRB, 0.11555E+01_JPRB, 0.11377E+01_JPRB ,&
&  0.11180E+01_JPRB, 0.10869E+01_JPRB, 0.10377E+01_JPRB, 0.98957E+00_JPRB, 0.91243E+00_JPRB, 0.84039E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.61241E+00_JPRB, 0.52410E+00_JPRB, 0.46188E+00_JPRB, 0.44331E+00_JPRB, 0.43996E+00_JPRB, 0.43651E+00_JPRB, 0.43675E+00_JPRB ,&
&  0.44514E+00_JPRB, 0.45308E+00_JPRB, 0.46018E+00_JPRB, 0.47289E+00_JPRB, 0.49244E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.51599E+00_JPRB, 0.53164E+00_JPRB, 0.53798E+00_JPRB, 0.54232E+00_JPRB, 0.51772E+00_JPRB, 0.49455E+00_JPRB, 0.47228E+00_JPRB ,&
&  0.46499E+00_JPRB, 0.45376E+00_JPRB, 0.44338E+00_JPRB, 0.42800E+00_JPRB, 0.41567E+00_JPRB, 0.41119E+00_JPRB, 0.41413E+00_JPRB ,&
&  0.41967E+00_JPRB, 0.42054E+00_JPRB, 0.41795E+00_JPRB, 0.40698E+00_JPRB, 0.39681E+00_JPRB, 0.38767E+00_JPRB, 0.38028E+00_JPRB ,&
&  0.37366E+00_JPRB, 0.37407E+00_JPRB, 0.37935E+00_JPRB, 0.39011E+00_JPRB, 0.39677E+00_JPRB, 0.40157E+00_JPRB, 0.40016E+00_JPRB ,&
&  0.39201E+00_JPRB, 0.38253E+00_JPRB, 0.38568E+00_JPRB, 0.39255E+00_JPRB, 0.39501E+00_JPRB, 0.40271E+00_JPRB, 0.38437E+00_JPRB ,&
&  0.36705E+00_JPRB, 0.36595E+00_JPRB, 0.38347E+00_JPRB, 0.45606E+00_JPRB, 0.48280E+00_JPRB, 0.49775E+00_JPRB, 0.51667E+00_JPRB ,&
&  0.50892E+00_JPRB, 0.51868E+00_JPRB, 0.55617E+00_JPRB, 0.65167E+00_JPRB, 0.76154E+00_JPRB, 0.94962E+00_JPRB, 0.12096E+01_JPRB ,&
&  0.14708E+01_JPRB, 0.16879E+01_JPRB, 0.18323E+01_JPRB, 0.16409E+01_JPRB, 0.15993E+01_JPRB, 0.17700E+01_JPRB, 0.17942E+01_JPRB ,&
&  0.17606E+01_JPRB, 0.16868E+01_JPRB, 0.16820E+01_JPRB, 0.14886E+01_JPRB, 0.12475E+01_JPRB, 0.10870E+01_JPRB, 0.96347E+00_JPRB ,&
&  0.85099E+00_JPRB, 0.77405E+00_JPRB, 0.72755E+00_JPRB, 0.69975E+00_JPRB, 0.70339E+00_JPRB, 0.75299E+00_JPRB, 0.82958E+00_JPRB ,&
&  0.90902E+00_JPRB, 0.95214E+00_JPRB, 0.97291E+00_JPRB, 0.96454E+00_JPRB, 0.95433E+00_JPRB, 0.94090E+00_JPRB, 0.93498E+00_JPRB ,&
&  0.92644E+00_JPRB, 0.92432E+00_JPRB, 0.91243E+00_JPRB, 0.89355E+00_JPRB, 0.82198E+00_JPRB, 0.71045E+00_JPRB, 0.60270E+00_JPRB ,&
&  0.52721E+00_JPRB, 0.48106E+00_JPRB, 0.44769E+00_JPRB, 0.43665E+00_JPRB, 0.43627E+00_JPRB, 0.43487E+00_JPRB, 0.42289E+00_JPRB ,&
&  0.41927E+00_JPRB, 0.42887E+00_JPRB, 0.44417E+00_JPRB, 0.46086E+00_JPRB, 0.48561E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.50607E+00_JPRB, 0.51165E+00_JPRB, 0.50871E+00_JPRB, 0.49914E+00_JPRB, 0.49055E+00_JPRB, 0.45884E+00_JPRB, 0.43512E+00_JPRB ,&
&  0.42467E+00_JPRB, 0.42241E+00_JPRB, 0.42458E+00_JPRB, 0.42616E+00_JPRB, 0.42917E+00_JPRB, 0.43395E+00_JPRB, 0.43439E+00_JPRB ,&
&  0.42901E+00_JPRB, 0.42376E+00_JPRB, 0.41812E+00_JPRB, 0.40896E+00_JPRB, 0.39632E+00_JPRB, 0.38158E+00_JPRB, 0.37012E+00_JPRB ,&
&  0.36340E+00_JPRB, 0.36273E+00_JPRB, 0.37018E+00_JPRB, 0.37792E+00_JPRB, 0.38295E+00_JPRB, 0.38390E+00_JPRB, 0.38139E+00_JPRB ,&
&  0.37317E+00_JPRB, 0.36775E+00_JPRB, 0.37359E+00_JPRB, 0.37591E+00_JPRB, 0.37945E+00_JPRB, 0.38396E+00_JPRB, 0.37078E+00_JPRB ,&
&  0.35670E+00_JPRB, 0.34486E+00_JPRB, 0.34237E+00_JPRB, 0.35234E+00_JPRB, 0.39036E+00_JPRB, 0.45739E+00_JPRB, 0.47883E+00_JPRB ,&
&  0.49893E+00_JPRB, 0.52141E+00_JPRB, 0.54787E+00_JPRB, 0.58506E+00_JPRB, 0.63523E+00_JPRB, 0.70492E+00_JPRB, 0.83070E+00_JPRB ,&
&  0.99125E+00_JPRB, 0.11337E+01_JPRB, 0.12748E+01_JPRB, 0.11671E+01_JPRB, 0.10833E+01_JPRB, 0.10815E+01_JPRB, 0.11122E+01_JPRB ,&
&  0.11258E+01_JPRB, 0.10608E+01_JPRB, 0.10835E+01_JPRB, 0.10899E+01_JPRB, 0.10288E+01_JPRB, 0.94674E+00_JPRB, 0.84604E+00_JPRB ,&
&  0.76635E+00_JPRB, 0.70239E+00_JPRB, 0.65934E+00_JPRB, 0.61691E+00_JPRB, 0.59006E+00_JPRB, 0.58270E+00_JPRB, 0.59448E+00_JPRB ,&
&  0.62246E+00_JPRB, 0.65669E+00_JPRB, 0.65796E+00_JPRB, 0.65048E+00_JPRB, 0.67190E+00_JPRB, 0.70680E+00_JPRB, 0.73730E+00_JPRB ,&
&  0.74849E+00_JPRB, 0.75643E+00_JPRB, 0.76893E+00_JPRB, 0.76367E+00_JPRB, 0.71891E+00_JPRB, 0.67329E+00_JPRB, 0.61881E+00_JPRB ,&
&  0.56283E+00_JPRB, 0.51133E+00_JPRB, 0.48401E+00_JPRB, 0.47307E+00_JPRB, 0.46659E+00_JPRB, 0.45983E+00_JPRB, 0.45014E+00_JPRB ,&
&  0.44338E+00_JPRB, 0.44533E+00_JPRB, 0.45690E+00_JPRB, 0.47210E+00_JPRB, 0.49175E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.50807E+00_JPRB, 0.48802E+00_JPRB, 0.47428E+00_JPRB, 0.46848E+00_JPRB, 0.46569E+00_JPRB, 0.45154E+00_JPRB, 0.44194E+00_JPRB ,&
&  0.43730E+00_JPRB, 0.44219E+00_JPRB, 0.44135E+00_JPRB, 0.43743E+00_JPRB, 0.43601E+00_JPRB, 0.43515E+00_JPRB, 0.43473E+00_JPRB ,&
&  0.43296E+00_JPRB, 0.42776E+00_JPRB, 0.41720E+00_JPRB, 0.39905E+00_JPRB, 0.38509E+00_JPRB, 0.37517E+00_JPRB, 0.36946E+00_JPRB ,&
&  0.36814E+00_JPRB, 0.36984E+00_JPRB, 0.37520E+00_JPRB, 0.37778E+00_JPRB, 0.37421E+00_JPRB, 0.36870E+00_JPRB, 0.36198E+00_JPRB ,&
&  0.36269E+00_JPRB, 0.37191E+00_JPRB, 0.36631E+00_JPRB, 0.36663E+00_JPRB, 0.37526E+00_JPRB, 0.37138E+00_JPRB, 0.35722E+00_JPRB ,&
&  0.34497E+00_JPRB, 0.33263E+00_JPRB, 0.32496E+00_JPRB, 0.32317E+00_JPRB, 0.33390E+00_JPRB, 0.35226E+00_JPRB, 0.38768E+00_JPRB ,&
&  0.44627E+00_JPRB, 0.47362E+00_JPRB, 0.49199E+00_JPRB, 0.51423E+00_JPRB, 0.53829E+00_JPRB, 0.56664E+00_JPRB, 0.61083E+00_JPRB ,&
&  0.67761E+00_JPRB, 0.74982E+00_JPRB, 0.81850E+00_JPRB, 0.86432E+00_JPRB, 0.89013E+00_JPRB, 0.83180E+00_JPRB, 0.81058E+00_JPRB ,&
&  0.87190E+00_JPRB, 0.87170E+00_JPRB, 0.83892E+00_JPRB, 0.79764E+00_JPRB, 0.75308E+00_JPRB, 0.71162E+00_JPRB, 0.67692E+00_JPRB ,&
&  0.65306E+00_JPRB, 0.63545E+00_JPRB, 0.61513E+00_JPRB, 0.57956E+00_JPRB, 0.54510E+00_JPRB, 0.52618E+00_JPRB, 0.52042E+00_JPRB ,&
&  0.52235E+00_JPRB, 0.53741E+00_JPRB, 0.53389E+00_JPRB, 0.49828E+00_JPRB, 0.50445E+00_JPRB, 0.52726E+00_JPRB, 0.55812E+00_JPRB ,&
&  0.59452E+00_JPRB, 0.62432E+00_JPRB, 0.64604E+00_JPRB, 0.64557E+00_JPRB, 0.63463E+00_JPRB, 0.61568E+00_JPRB, 0.60230E+00_JPRB ,&
&  0.58925E+00_JPRB, 0.57545E+00_JPRB, 0.56586E+00_JPRB, 0.56275E+00_JPRB, 0.55671E+00_JPRB, 0.55119E+00_JPRB, 0.54541E+00_JPRB ,&
&  0.53935E+00_JPRB, 0.53823E+00_JPRB, 0.53927E+00_JPRB, 0.53646E+00_JPRB, 0.52949E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.46640E+00_JPRB, 0.45969E+00_JPRB, 0.45439E+00_JPRB, 0.45117E+00_JPRB, 0.44808E+00_JPRB, 0.44490E+00_JPRB, 0.44248E+00_JPRB ,&
&  0.44103E+00_JPRB, 0.44043E+00_JPRB, 0.43795E+00_JPRB, 0.43510E+00_JPRB, 0.43225E+00_JPRB, 0.42977E+00_JPRB, 0.42750E+00_JPRB ,&
&  0.42178E+00_JPRB, 0.40573E+00_JPRB, 0.39915E+00_JPRB, 0.39228E+00_JPRB, 0.38345E+00_JPRB, 0.37866E+00_JPRB, 0.37627E+00_JPRB ,&
&  0.37279E+00_JPRB, 0.37090E+00_JPRB, 0.37136E+00_JPRB, 0.36678E+00_JPRB, 0.36300E+00_JPRB, 0.36522E+00_JPRB, 0.36893E+00_JPRB ,&
&  0.36279E+00_JPRB, 0.36237E+00_JPRB, 0.37520E+00_JPRB, 0.37300E+00_JPRB, 0.35847E+00_JPRB, 0.35655E+00_JPRB, 0.34487E+00_JPRB ,&
&  0.33701E+00_JPRB, 0.32146E+00_JPRB, 0.31597E+00_JPRB, 0.31214E+00_JPRB, 0.31791E+00_JPRB, 0.32801E+00_JPRB, 0.35232E+00_JPRB ,&
&  0.37933E+00_JPRB, 0.42756E+00_JPRB, 0.44658E+00_JPRB, 0.46138E+00_JPRB, 0.47542E+00_JPRB, 0.49010E+00_JPRB, 0.50735E+00_JPRB ,&
&  0.52920E+00_JPRB, 0.55590E+00_JPRB, 0.58836E+00_JPRB, 0.61573E+00_JPRB, 0.63710E+00_JPRB, 0.64274E+00_JPRB, 0.64064E+00_JPRB ,&
&  0.64979E+00_JPRB, 0.64272E+00_JPRB, 0.62514E+00_JPRB, 0.60517E+00_JPRB, 0.58158E+00_JPRB, 0.55759E+00_JPRB, 0.53170E+00_JPRB ,&
&  0.52233E+00_JPRB, 0.52654E+00_JPRB, 0.51623E+00_JPRB, 0.50600E+00_JPRB, 0.49363E+00_JPRB, 0.48795E+00_JPRB, 0.48157E+00_JPRB ,&
&  0.47062E+00_JPRB, 0.46679E+00_JPRB, 0.47528E+00_JPRB, 0.47238E+00_JPRB, 0.47649E+00_JPRB, 0.48267E+00_JPRB, 0.49305E+00_JPRB ,&
&  0.50536E+00_JPRB, 0.52180E+00_JPRB, 0.53795E+00_JPRB, 0.55073E+00_JPRB, 0.56255E+00_JPRB, 0.56430E+00_JPRB, 0.57124E+00_JPRB ,&
&  0.57028E+00_JPRB, 0.56855E+00_JPRB, 0.56504E+00_JPRB, 0.56351E+00_JPRB, 0.55986E+00_JPRB, 0.55422E+00_JPRB, 0.54606E+00_JPRB ,&
&  0.53620E+00_JPRB, 0.52351E+00_JPRB, 0.50884E+00_JPRB, 0.49357E+00_JPRB, 0.47809E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.43445E+00_JPRB, 0.43173E+00_JPRB, 0.42986E+00_JPRB, 0.42782E+00_JPRB, 0.42658E+00_JPRB, 0.42526E+00_JPRB, 0.42412E+00_JPRB ,&
&  0.42349E+00_JPRB, 0.42196E+00_JPRB, 0.41944E+00_JPRB, 0.41619E+00_JPRB, 0.41335E+00_JPRB, 0.41053E+00_JPRB, 0.40754E+00_JPRB ,&
&  0.40279E+00_JPRB, 0.39727E+00_JPRB, 0.39139E+00_JPRB, 0.38651E+00_JPRB, 0.38656E+00_JPRB, 0.38572E+00_JPRB, 0.38487E+00_JPRB ,&
&  0.38127E+00_JPRB, 0.37542E+00_JPRB, 0.36691E+00_JPRB, 0.36500E+00_JPRB, 0.36531E+00_JPRB, 0.36118E+00_JPRB, 0.36490E+00_JPRB ,&
&  0.37109E+00_JPRB, 0.36978E+00_JPRB, 0.36839E+00_JPRB, 0.36677E+00_JPRB, 0.35176E+00_JPRB, 0.32118E+00_JPRB, 0.32952E+00_JPRB ,&
&  0.32615E+00_JPRB, 0.31688E+00_JPRB, 0.31320E+00_JPRB, 0.31092E+00_JPRB, 0.31435E+00_JPRB, 0.32296E+00_JPRB, 0.33276E+00_JPRB ,&
&  0.33949E+00_JPRB, 0.39171E+00_JPRB, 0.41220E+00_JPRB, 0.42570E+00_JPRB, 0.43398E+00_JPRB, 0.44080E+00_JPRB, 0.44856E+00_JPRB ,&
&  0.45556E+00_JPRB, 0.46718E+00_JPRB, 0.48211E+00_JPRB, 0.49610E+00_JPRB, 0.50814E+00_JPRB, 0.51605E+00_JPRB, 0.52367E+00_JPRB ,&
&  0.53252E+00_JPRB, 0.53307E+00_JPRB, 0.52581E+00_JPRB, 0.51761E+00_JPRB, 0.50602E+00_JPRB, 0.49362E+00_JPRB, 0.48123E+00_JPRB ,&
&  0.47388E+00_JPRB, 0.46672E+00_JPRB, 0.46645E+00_JPRB, 0.46403E+00_JPRB, 0.45971E+00_JPRB, 0.45663E+00_JPRB, 0.45526E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45271E+00_JPRB, 0.45067E+00_JPRB, 0.45439E+00_JPRB, 0.45976E+00_JPRB, 0.46319E+00_JPRB, 0.46612E+00_JPRB ,&
&  0.46703E+00_JPRB, 0.47002E+00_JPRB, 0.47118E+00_JPRB, 0.47682E+00_JPRB, 0.48444E+00_JPRB, 0.49051E+00_JPRB, 0.49606E+00_JPRB ,&
&  0.50068E+00_JPRB, 0.50344E+00_JPRB, 0.50298E+00_JPRB, 0.49891E+00_JPRB, 0.49213E+00_JPRB, 0.48283E+00_JPRB, 0.47148E+00_JPRB ,&
&  0.45985E+00_JPRB, 0.45082E+00_JPRB, 0.44351E+00_JPRB, 0.43920E+00_JPRB, 0.43603E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.41678E+00_JPRB, 0.41326E+00_JPRB, 0.41109E+00_JPRB, 0.40903E+00_JPRB, 0.40817E+00_JPRB, 0.40718E+00_JPRB, 0.40606E+00_JPRB ,&
&  0.40456E+00_JPRB, 0.40327E+00_JPRB, 0.40164E+00_JPRB, 0.40065E+00_JPRB, 0.39914E+00_JPRB, 0.39802E+00_JPRB, 0.39642E+00_JPRB ,&
&  0.39574E+00_JPRB, 0.39420E+00_JPRB, 0.39410E+00_JPRB, 0.39214E+00_JPRB, 0.39279E+00_JPRB, 0.39128E+00_JPRB, 0.38710E+00_JPRB ,&
&  0.38290E+00_JPRB, 0.37671E+00_JPRB, 0.36525E+00_JPRB, 0.36762E+00_JPRB, 0.36548E+00_JPRB, 0.36673E+00_JPRB, 0.36195E+00_JPRB ,&
&  0.36140E+00_JPRB, 0.34650E+00_JPRB, 0.35305E+00_JPRB, 0.34625E+00_JPRB, 0.34187E+00_JPRB, 0.33584E+00_JPRB, 0.32779E+00_JPRB ,&
&  0.32533E+00_JPRB, 0.32063E+00_JPRB, 0.32047E+00_JPRB, 0.32258E+00_JPRB, 0.32587E+00_JPRB, 0.33090E+00_JPRB, 0.33653E+00_JPRB ,&
&  0.33960E+00_JPRB, 0.37140E+00_JPRB, 0.38951E+00_JPRB, 0.40234E+00_JPRB, 0.41273E+00_JPRB, 0.42012E+00_JPRB, 0.42693E+00_JPRB ,&
&  0.43324E+00_JPRB, 0.43990E+00_JPRB, 0.44702E+00_JPRB, 0.44944E+00_JPRB, 0.45760E+00_JPRB, 0.46074E+00_JPRB, 0.46421E+00_JPRB ,&
&  0.46713E+00_JPRB, 0.46647E+00_JPRB, 0.46507E+00_JPRB, 0.46195E+00_JPRB, 0.46049E+00_JPRB, 0.45694E+00_JPRB, 0.45570E+00_JPRB ,&
&  0.45283E+00_JPRB, 0.45086E+00_JPRB, 0.45110E+00_JPRB, 0.45087E+00_JPRB, 0.44930E+00_JPRB, 0.44785E+00_JPRB, 0.44688E+00_JPRB ,&
&  0.44645E+00_JPRB, 0.44811E+00_JPRB, 0.44951E+00_JPRB, 0.45326E+00_JPRB, 0.45723E+00_JPRB, 0.45945E+00_JPRB, 0.46167E+00_JPRB ,&
&  0.46394E+00_JPRB, 0.45981E+00_JPRB, 0.45676E+00_JPRB, 0.45519E+00_JPRB, 0.45559E+00_JPRB, 0.45697E+00_JPRB, 0.45894E+00_JPRB ,&
&  0.45933E+00_JPRB, 0.45654E+00_JPRB, 0.45287E+00_JPRB, 0.44791E+00_JPRB, 0.44290E+00_JPRB, 0.43876E+00_JPRB, 0.43463E+00_JPRB ,&
&  0.43136E+00_JPRB, 0.43071E+00_JPRB, 0.42798E+00_JPRB, 0.42409E+00_JPRB, 0.41998E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.40724E+00_JPRB, 0.40281E+00_JPRB, 0.39939E+00_JPRB, 0.39727E+00_JPRB, 0.39557E+00_JPRB, 0.39449E+00_JPRB, 0.39377E+00_JPRB ,&
&  0.39293E+00_JPRB, 0.39221E+00_JPRB, 0.39182E+00_JPRB, 0.39161E+00_JPRB, 0.39127E+00_JPRB, 0.39119E+00_JPRB, 0.39081E+00_JPRB ,&
&  0.39183E+00_JPRB, 0.39128E+00_JPRB, 0.39221E+00_JPRB, 0.39115E+00_JPRB, 0.39051E+00_JPRB, 0.38913E+00_JPRB, 0.38580E+00_JPRB ,&
&  0.38438E+00_JPRB, 0.38172E+00_JPRB, 0.36732E+00_JPRB, 0.37125E+00_JPRB, 0.36639E+00_JPRB, 0.36454E+00_JPRB, 0.36358E+00_JPRB ,&
&  0.35821E+00_JPRB, 0.35567E+00_JPRB, 0.35169E+00_JPRB, 0.35054E+00_JPRB, 0.35062E+00_JPRB, 0.34698E+00_JPRB, 0.34691E+00_JPRB ,&
&  0.34318E+00_JPRB, 0.34210E+00_JPRB, 0.34248E+00_JPRB, 0.34298E+00_JPRB, 0.34607E+00_JPRB, 0.34722E+00_JPRB, 0.35094E+00_JPRB ,&
&  0.35917E+00_JPRB, 0.37125E+00_JPRB, 0.39242E+00_JPRB, 0.40287E+00_JPRB, 0.41355E+00_JPRB, 0.42109E+00_JPRB, 0.42843E+00_JPRB ,&
&  0.43263E+00_JPRB, 0.43578E+00_JPRB, 0.43811E+00_JPRB, 0.43999E+00_JPRB, 0.44038E+00_JPRB, 0.43838E+00_JPRB, 0.43828E+00_JPRB ,&
&  0.43857E+00_JPRB, 0.43921E+00_JPRB, 0.43934E+00_JPRB, 0.43920E+00_JPRB, 0.44100E+00_JPRB, 0.44292E+00_JPRB, 0.44605E+00_JPRB ,&
&  0.44861E+00_JPRB, 0.45067E+00_JPRB, 0.44989E+00_JPRB, 0.45040E+00_JPRB, 0.44894E+00_JPRB, 0.44786E+00_JPRB, 0.44670E+00_JPRB ,&
&  0.44580E+00_JPRB, 0.44605E+00_JPRB, 0.44725E+00_JPRB, 0.44341E+00_JPRB, 0.44280E+00_JPRB, 0.44898E+00_JPRB, 0.45031E+00_JPRB ,&
&  0.45206E+00_JPRB, 0.45296E+00_JPRB, 0.45309E+00_JPRB, 0.45250E+00_JPRB, 0.45180E+00_JPRB, 0.45184E+00_JPRB, 0.45158E+00_JPRB ,&
&  0.45100E+00_JPRB, 0.44705E+00_JPRB, 0.44279E+00_JPRB, 0.43864E+00_JPRB, 0.43409E+00_JPRB, 0.43044E+00_JPRB, 0.42664E+00_JPRB ,&
&  0.42362E+00_JPRB, 0.42265E+00_JPRB, 0.42147E+00_JPRB, 0.41823E+00_JPRB, 0.41295E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.39967E+00_JPRB, 0.39628E+00_JPRB, 0.39402E+00_JPRB, 0.39246E+00_JPRB, 0.39102E+00_JPRB, 0.39014E+00_JPRB, 0.39012E+00_JPRB ,&
&  0.39047E+00_JPRB, 0.38994E+00_JPRB, 0.38985E+00_JPRB, 0.38973E+00_JPRB, 0.38917E+00_JPRB, 0.38965E+00_JPRB, 0.38981E+00_JPRB ,&
&  0.39045E+00_JPRB, 0.39016E+00_JPRB, 0.39142E+00_JPRB, 0.39091E+00_JPRB, 0.39100E+00_JPRB, 0.39141E+00_JPRB, 0.39108E+00_JPRB ,&
&  0.39177E+00_JPRB, 0.39138E+00_JPRB, 0.38796E+00_JPRB, 0.38420E+00_JPRB, 0.38101E+00_JPRB, 0.36631E+00_JPRB, 0.36642E+00_JPRB ,&
&  0.37028E+00_JPRB, 0.36521E+00_JPRB, 0.36369E+00_JPRB, 0.36223E+00_JPRB, 0.36267E+00_JPRB, 0.36243E+00_JPRB, 0.36514E+00_JPRB ,&
&  0.35854E+00_JPRB, 0.36574E+00_JPRB, 0.36588E+00_JPRB, 0.36594E+00_JPRB, 0.36695E+00_JPRB, 0.36648E+00_JPRB, 0.37121E+00_JPRB ,&
&  0.38858E+00_JPRB, 0.39243E+00_JPRB, 0.39970E+00_JPRB, 0.40602E+00_JPRB, 0.41346E+00_JPRB, 0.41998E+00_JPRB, 0.42619E+00_JPRB ,&
&  0.43069E+00_JPRB, 0.43345E+00_JPRB, 0.43505E+00_JPRB, 0.43504E+00_JPRB, 0.43524E+00_JPRB, 0.43496E+00_JPRB, 0.43520E+00_JPRB ,&
&  0.43547E+00_JPRB, 0.43646E+00_JPRB, 0.43819E+00_JPRB, 0.43976E+00_JPRB, 0.44181E+00_JPRB, 0.44297E+00_JPRB, 0.44414E+00_JPRB ,&
&  0.44479E+00_JPRB, 0.44492E+00_JPRB, 0.44427E+00_JPRB, 0.44403E+00_JPRB, 0.44294E+00_JPRB, 0.44275E+00_JPRB, 0.44198E+00_JPRB ,&
&  0.44202E+00_JPRB, 0.44138E+00_JPRB, 0.44154E+00_JPRB, 0.44107E+00_JPRB, 0.44093E+00_JPRB, 0.44154E+00_JPRB, 0.44165E+00_JPRB ,&
&  0.44269E+00_JPRB, 0.44317E+00_JPRB, 0.44366E+00_JPRB, 0.44405E+00_JPRB, 0.44369E+00_JPRB, 0.44391E+00_JPRB, 0.44360E+00_JPRB ,&
&  0.44408E+00_JPRB, 0.44159E+00_JPRB, 0.43849E+00_JPRB, 0.43449E+00_JPRB, 0.43083E+00_JPRB, 0.42727E+00_JPRB, 0.42359E+00_JPRB ,&
&  0.41998E+00_JPRB, 0.41607E+00_JPRB, 0.41218E+00_JPRB, 0.40798E+00_JPRB, 0.40347E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.39947E+00_JPRB, 0.39746E+00_JPRB, 0.39604E+00_JPRB, 0.39522E+00_JPRB, 0.39479E+00_JPRB, 0.39443E+00_JPRB, 0.39414E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39415E+00_JPRB, 0.39418E+00_JPRB, 0.39424E+00_JPRB, 0.39417E+00_JPRB, 0.39453E+00_JPRB, 0.39442E+00_JPRB ,&
&  0.39422E+00_JPRB, 0.39347E+00_JPRB, 0.39306E+00_JPRB, 0.39245E+00_JPRB, 0.39292E+00_JPRB, 0.39362E+00_JPRB, 0.39531E+00_JPRB ,&
&  0.39577E+00_JPRB, 0.39599E+00_JPRB, 0.39552E+00_JPRB, 0.39307E+00_JPRB, 0.39053E+00_JPRB, 0.38813E+00_JPRB, 0.38618E+00_JPRB ,&
&  0.38456E+00_JPRB, 0.38356E+00_JPRB, 0.38341E+00_JPRB, 0.38372E+00_JPRB, 0.38546E+00_JPRB, 0.38570E+00_JPRB, 0.38556E+00_JPRB ,&
&  0.38489E+00_JPRB, 0.38277E+00_JPRB, 0.38301E+00_JPRB, 0.38359E+00_JPRB, 0.38471E+00_JPRB, 0.38744E+00_JPRB, 0.38993E+00_JPRB ,&
&  0.39314E+00_JPRB, 0.39693E+00_JPRB, 0.40266E+00_JPRB, 0.40675E+00_JPRB, 0.41100E+00_JPRB, 0.41453E+00_JPRB, 0.41628E+00_JPRB ,&
&  0.41767E+00_JPRB, 0.41880E+00_JPRB, 0.42006E+00_JPRB, 0.42098E+00_JPRB, 0.42187E+00_JPRB, 0.42289E+00_JPRB, 0.42429E+00_JPRB ,&
&  0.42541E+00_JPRB, 0.42656E+00_JPRB, 0.42736E+00_JPRB, 0.42816E+00_JPRB, 0.42912E+00_JPRB, 0.43005E+00_JPRB, 0.43116E+00_JPRB ,&
&  0.43212E+00_JPRB, 0.43208E+00_JPRB, 0.43183E+00_JPRB, 0.43222E+00_JPRB, 0.43264E+00_JPRB, 0.43284E+00_JPRB, 0.43223E+00_JPRB ,&
&  0.43198E+00_JPRB, 0.43165E+00_JPRB, 0.43148E+00_JPRB, 0.43124E+00_JPRB, 0.43088E+00_JPRB, 0.43046E+00_JPRB, 0.42976E+00_JPRB ,&
&  0.42893E+00_JPRB, 0.42789E+00_JPRB, 0.42699E+00_JPRB, 0.42673E+00_JPRB, 0.42613E+00_JPRB, 0.42614E+00_JPRB, 0.42640E+00_JPRB ,&
&  0.42684E+00_JPRB, 0.42585E+00_JPRB, 0.42451E+00_JPRB, 0.42252E+00_JPRB, 0.42035E+00_JPRB, 0.41783E+00_JPRB, 0.41534E+00_JPRB ,&
&  0.41274E+00_JPRB, 0.41005E+00_JPRB, 0.40704E+00_JPRB, 0.40423E+00_JPRB, 0.40169E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.40888E+00_JPRB, 0.40786E+00_JPRB, 0.40695E+00_JPRB, 0.40607E+00_JPRB, 0.40521E+00_JPRB, 0.40458E+00_JPRB, 0.40406E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40319E+00_JPRB, 0.40268E+00_JPRB, 0.40231E+00_JPRB, 0.40211E+00_JPRB, 0.40187E+00_JPRB, 0.40163E+00_JPRB ,&
&  0.40144E+00_JPRB, 0.40137E+00_JPRB, 0.40140E+00_JPRB, 0.40120E+00_JPRB, 0.40113E+00_JPRB, 0.40103E+00_JPRB, 0.40062E+00_JPRB ,&
&  0.40003E+00_JPRB, 0.39949E+00_JPRB, 0.39886E+00_JPRB, 0.39837E+00_JPRB, 0.39800E+00_JPRB, 0.39737E+00_JPRB, 0.39659E+00_JPRB ,&
&  0.39588E+00_JPRB, 0.39543E+00_JPRB, 0.39520E+00_JPRB, 0.39521E+00_JPRB, 0.39539E+00_JPRB, 0.39545E+00_JPRB, 0.39548E+00_JPRB ,&
&  0.39558E+00_JPRB, 0.39579E+00_JPRB, 0.39614E+00_JPRB, 0.39673E+00_JPRB, 0.39752E+00_JPRB, 0.39853E+00_JPRB, 0.40006E+00_JPRB ,&
&  0.40183E+00_JPRB, 0.40366E+00_JPRB, 0.40551E+00_JPRB, 0.40714E+00_JPRB, 0.40857E+00_JPRB, 0.40959E+00_JPRB, 0.41028E+00_JPRB ,&
&  0.41103E+00_JPRB, 0.41190E+00_JPRB, 0.41284E+00_JPRB, 0.41342E+00_JPRB, 0.41393E+00_JPRB, 0.41436E+00_JPRB, 0.41491E+00_JPRB ,&
&  0.41519E+00_JPRB, 0.41548E+00_JPRB, 0.41585E+00_JPRB, 0.41631E+00_JPRB, 0.41664E+00_JPRB, 0.41688E+00_JPRB, 0.41723E+00_JPRB ,&
&  0.41776E+00_JPRB, 0.41829E+00_JPRB, 0.41860E+00_JPRB, 0.41892E+00_JPRB, 0.41938E+00_JPRB, 0.41997E+00_JPRB, 0.42054E+00_JPRB ,&
&  0.42102E+00_JPRB, 0.42146E+00_JPRB, 0.42187E+00_JPRB, 0.42219E+00_JPRB, 0.42237E+00_JPRB, 0.42232E+00_JPRB, 0.42185E+00_JPRB ,&
&  0.42126E+00_JPRB, 0.42055E+00_JPRB, 0.42006E+00_JPRB, 0.41985E+00_JPRB, 0.41956E+00_JPRB, 0.41937E+00_JPRB, 0.41932E+00_JPRB ,&
&  0.41944E+00_JPRB, 0.41955E+00_JPRB, 0.41954E+00_JPRB, 0.41943E+00_JPRB, 0.41910E+00_JPRB, 0.41843E+00_JPRB, 0.41753E+00_JPRB ,&
&  0.41644E+00_JPRB, 0.41503E+00_JPRB, 0.41355E+00_JPRB, 0.41195E+00_JPRB, 0.41031E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB ,&
&  0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB, 0.41236E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB, 0.12353E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.12857E+00_JPRB, 0.12875E+00_JPRB, 0.12886E+00_JPRB, 0.12893E+00_JPRB, 0.12886E+00_JPRB, 0.12889E+00_JPRB, 0.12891E+00_JPRB ,&
&  0.12892E+00_JPRB, 0.12883E+00_JPRB, 0.12900E+00_JPRB, 0.12910E+00_JPRB, 0.12916E+00_JPRB, 0.12918E+00_JPRB, 0.12908E+00_JPRB ,&
&  0.12887E+00_JPRB, 0.12860E+00_JPRB, 0.12788E+00_JPRB, 0.12777E+00_JPRB, 0.12782E+00_JPRB, 0.12803E+00_JPRB, 0.12816E+00_JPRB ,&
&  0.12837E+00_JPRB, 0.12857E+00_JPRB, 0.12873E+00_JPRB, 0.12916E+00_JPRB, 0.12936E+00_JPRB, 0.12959E+00_JPRB, 0.12982E+00_JPRB ,&
&  0.13004E+00_JPRB, 0.13028E+00_JPRB, 0.13046E+00_JPRB, 0.13065E+00_JPRB, 0.13080E+00_JPRB, 0.13074E+00_JPRB, 0.13056E+00_JPRB ,&
&  0.13028E+00_JPRB, 0.12985E+00_JPRB, 0.12941E+00_JPRB, 0.12901E+00_JPRB, 0.12866E+00_JPRB, 0.12829E+00_JPRB, 0.12793E+00_JPRB ,&
&  0.12754E+00_JPRB, 0.12710E+00_JPRB, 0.12662E+00_JPRB, 0.12619E+00_JPRB, 0.12575E+00_JPRB, 0.12538E+00_JPRB, 0.12495E+00_JPRB ,&
&  0.12461E+00_JPRB, 0.12427E+00_JPRB, 0.12395E+00_JPRB, 0.12361E+00_JPRB, 0.12332E+00_JPRB, 0.12303E+00_JPRB, 0.12274E+00_JPRB ,&
&  0.12244E+00_JPRB, 0.12227E+00_JPRB, 0.12204E+00_JPRB, 0.12184E+00_JPRB, 0.12166E+00_JPRB, 0.12152E+00_JPRB, 0.12134E+00_JPRB ,&
&  0.12117E+00_JPRB, 0.12101E+00_JPRB, 0.12093E+00_JPRB, 0.12081E+00_JPRB, 0.12076E+00_JPRB, 0.12074E+00_JPRB, 0.12079E+00_JPRB ,&
&  0.12084E+00_JPRB, 0.12093E+00_JPRB, 0.12107E+00_JPRB, 0.12129E+00_JPRB, 0.12149E+00_JPRB, 0.12171E+00_JPRB, 0.12195E+00_JPRB ,&
&  0.12221E+00_JPRB, 0.12246E+00_JPRB, 0.12274E+00_JPRB, 0.12300E+00_JPRB, 0.12332E+00_JPRB, 0.12364E+00_JPRB, 0.12400E+00_JPRB ,&
&  0.12432E+00_JPRB, 0.12467E+00_JPRB, 0.12496E+00_JPRB, 0.12530E+00_JPRB, 0.12556E+00_JPRB, 0.12573E+00_JPRB, 0.12588E+00_JPRB ,&
&  0.12617E+00_JPRB, 0.12645E+00_JPRB, 0.12680E+00_JPRB, 0.12712E+00_JPRB, 0.12769E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.14172E+00_JPRB, 0.14409E+00_JPRB, 0.14588E+00_JPRB, 0.14791E+00_JPRB, 0.14751E+00_JPRB, 0.15884E+00_JPRB, 0.15704E+00_JPRB ,&
&  0.15680E+00_JPRB, 0.15142E+00_JPRB, 0.15142E+00_JPRB, 0.15044E+00_JPRB, 0.14895E+00_JPRB, 0.14730E+00_JPRB, 0.14555E+00_JPRB ,&
&  0.14385E+00_JPRB, 0.14176E+00_JPRB, 0.13995E+00_JPRB, 0.13845E+00_JPRB, 0.13708E+00_JPRB, 0.13588E+00_JPRB, 0.13465E+00_JPRB ,&
&  0.13439E+00_JPRB, 0.13490E+00_JPRB, 0.13578E+00_JPRB, 0.13530E+00_JPRB, 0.13572E+00_JPRB, 0.13653E+00_JPRB, 0.13762E+00_JPRB ,&
&  0.13857E+00_JPRB, 0.13980E+00_JPRB, 0.14056E+00_JPRB, 0.14073E+00_JPRB, 0.14077E+00_JPRB, 0.14008E+00_JPRB, 0.13961E+00_JPRB ,&
&  0.13901E+00_JPRB, 0.13819E+00_JPRB, 0.13716E+00_JPRB, 0.13629E+00_JPRB, 0.13529E+00_JPRB, 0.13398E+00_JPRB, 0.13261E+00_JPRB ,&
&  0.13136E+00_JPRB, 0.13026E+00_JPRB, 0.12914E+00_JPRB, 0.12821E+00_JPRB, 0.12731E+00_JPRB, 0.12642E+00_JPRB, 0.12561E+00_JPRB ,&
&  0.12498E+00_JPRB, 0.12441E+00_JPRB, 0.12380E+00_JPRB, 0.12313E+00_JPRB, 0.12237E+00_JPRB, 0.12164E+00_JPRB, 0.12094E+00_JPRB ,&
&  0.12056E+00_JPRB, 0.12022E+00_JPRB, 0.11987E+00_JPRB, 0.11951E+00_JPRB, 0.11933E+00_JPRB, 0.11917E+00_JPRB, 0.11898E+00_JPRB ,&
&  0.11879E+00_JPRB, 0.11854E+00_JPRB, 0.11844E+00_JPRB, 0.11835E+00_JPRB, 0.11827E+00_JPRB, 0.11821E+00_JPRB, 0.11829E+00_JPRB ,&
&  0.11845E+00_JPRB, 0.11867E+00_JPRB, 0.11909E+00_JPRB, 0.11954E+00_JPRB, 0.12002E+00_JPRB, 0.12050E+00_JPRB, 0.12119E+00_JPRB ,&
&  0.12193E+00_JPRB, 0.12265E+00_JPRB, 0.12328E+00_JPRB, 0.12380E+00_JPRB, 0.12458E+00_JPRB, 0.12532E+00_JPRB, 0.12603E+00_JPRB ,&
&  0.12682E+00_JPRB, 0.12783E+00_JPRB, 0.12887E+00_JPRB, 0.12994E+00_JPRB, 0.13116E+00_JPRB, 0.13225E+00_JPRB, 0.13332E+00_JPRB ,&
&  0.13458E+00_JPRB, 0.13686E+00_JPRB, 0.13851E+00_JPRB, 0.13966E+00_JPRB, 0.14096E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.16254E+00_JPRB, 0.16371E+00_JPRB, 0.16473E+00_JPRB, 0.16524E+00_JPRB, 0.16570E+00_JPRB, 0.16574E+00_JPRB, 0.16570E+00_JPRB ,&
&  0.16316E+00_JPRB, 0.16201E+00_JPRB, 0.16040E+00_JPRB, 0.15858E+00_JPRB, 0.15699E+00_JPRB, 0.15525E+00_JPRB, 0.15339E+00_JPRB ,&
&  0.15160E+00_JPRB, 0.14946E+00_JPRB, 0.14708E+00_JPRB, 0.14431E+00_JPRB, 0.14158E+00_JPRB, 0.13900E+00_JPRB, 0.13739E+00_JPRB ,&
&  0.13654E+00_JPRB, 0.13663E+00_JPRB, 0.13787E+00_JPRB, 0.13894E+00_JPRB, 0.14052E+00_JPRB, 0.14256E+00_JPRB, 0.14589E+00_JPRB ,&
&  0.14853E+00_JPRB, 0.15173E+00_JPRB, 0.15391E+00_JPRB, 0.15467E+00_JPRB, 0.15396E+00_JPRB, 0.15358E+00_JPRB, 0.15274E+00_JPRB ,&
&  0.15116E+00_JPRB, 0.14949E+00_JPRB, 0.14802E+00_JPRB, 0.14646E+00_JPRB, 0.14468E+00_JPRB, 0.14240E+00_JPRB, 0.13963E+00_JPRB ,&
&  0.13709E+00_JPRB, 0.13487E+00_JPRB, 0.13284E+00_JPRB, 0.13128E+00_JPRB, 0.12980E+00_JPRB, 0.12848E+00_JPRB, 0.12712E+00_JPRB ,&
&  0.12608E+00_JPRB, 0.12516E+00_JPRB, 0.12400E+00_JPRB, 0.12300E+00_JPRB, 0.12187E+00_JPRB, 0.12093E+00_JPRB, 0.11992E+00_JPRB ,&
&  0.11910E+00_JPRB, 0.11837E+00_JPRB, 0.11775E+00_JPRB, 0.11717E+00_JPRB, 0.11678E+00_JPRB, 0.11639E+00_JPRB, 0.11595E+00_JPRB ,&
&  0.11555E+00_JPRB, 0.11542E+00_JPRB, 0.11534E+00_JPRB, 0.11528E+00_JPRB, 0.11531E+00_JPRB, 0.11503E+00_JPRB, 0.11507E+00_JPRB ,&
&  0.11537E+00_JPRB, 0.11570E+00_JPRB, 0.11589E+00_JPRB, 0.11634E+00_JPRB, 0.11700E+00_JPRB, 0.11780E+00_JPRB, 0.11891E+00_JPRB ,&
&  0.11979E+00_JPRB, 0.12066E+00_JPRB, 0.12148E+00_JPRB, 0.12232E+00_JPRB, 0.12337E+00_JPRB, 0.12451E+00_JPRB, 0.12572E+00_JPRB ,&
&  0.12715E+00_JPRB, 0.12850E+00_JPRB, 0.12990E+00_JPRB, 0.13138E+00_JPRB, 0.13368E+00_JPRB, 0.13606E+00_JPRB, 0.13967E+00_JPRB ,&
&  0.14337E+00_JPRB, 0.14563E+00_JPRB, 0.15794E+00_JPRB, 0.15899E+00_JPRB, 0.16044E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.16301E+00_JPRB, 0.16444E+00_JPRB, 0.16539E+00_JPRB, 0.16656E+00_JPRB, 0.16722E+00_JPRB, 0.16880E+00_JPRB, 0.16921E+00_JPRB ,&
&  0.16936E+00_JPRB, 0.16543E+00_JPRB, 0.16304E+00_JPRB, 0.16046E+00_JPRB, 0.15823E+00_JPRB, 0.15575E+00_JPRB, 0.15332E+00_JPRB ,&
&  0.15140E+00_JPRB, 0.14976E+00_JPRB, 0.14803E+00_JPRB, 0.14559E+00_JPRB, 0.14314E+00_JPRB, 0.14127E+00_JPRB, 0.13981E+00_JPRB ,&
&  0.13837E+00_JPRB, 0.13873E+00_JPRB, 0.13991E+00_JPRB, 0.14257E+00_JPRB, 0.14637E+00_JPRB, 0.14977E+00_JPRB, 0.15419E+00_JPRB ,&
&  0.16030E+00_JPRB, 0.16213E+00_JPRB, 0.16381E+00_JPRB, 0.16466E+00_JPRB, 0.16438E+00_JPRB, 0.16473E+00_JPRB, 0.16219E+00_JPRB ,&
&  0.16244E+00_JPRB, 0.16086E+00_JPRB, 0.16532E+00_JPRB, 0.16317E+00_JPRB, 0.15722E+00_JPRB, 0.15415E+00_JPRB, 0.14839E+00_JPRB ,&
&  0.14378E+00_JPRB, 0.13906E+00_JPRB, 0.13578E+00_JPRB, 0.13388E+00_JPRB, 0.13136E+00_JPRB, 0.12995E+00_JPRB, 0.12787E+00_JPRB ,&
&  0.12597E+00_JPRB, 0.12524E+00_JPRB, 0.12397E+00_JPRB, 0.12336E+00_JPRB, 0.12264E+00_JPRB, 0.12169E+00_JPRB, 0.12067E+00_JPRB ,&
&  0.11996E+00_JPRB, 0.11932E+00_JPRB, 0.11894E+00_JPRB, 0.11845E+00_JPRB, 0.11779E+00_JPRB, 0.11712E+00_JPRB, 0.11641E+00_JPRB ,&
&  0.11590E+00_JPRB, 0.11557E+00_JPRB, 0.11528E+00_JPRB, 0.11510E+00_JPRB, 0.11492E+00_JPRB, 0.11464E+00_JPRB, 0.11466E+00_JPRB ,&
&  0.11487E+00_JPRB, 0.11539E+00_JPRB, 0.11588E+00_JPRB, 0.11652E+00_JPRB, 0.11714E+00_JPRB, 0.11797E+00_JPRB, 0.11864E+00_JPRB ,&
&  0.11933E+00_JPRB, 0.12020E+00_JPRB, 0.12095E+00_JPRB, 0.12184E+00_JPRB, 0.12282E+00_JPRB, 0.12368E+00_JPRB, 0.12498E+00_JPRB ,&
&  0.12630E+00_JPRB, 0.12761E+00_JPRB, 0.12937E+00_JPRB, 0.13126E+00_JPRB, 0.13366E+00_JPRB, 0.13785E+00_JPRB, 0.14273E+00_JPRB ,&
&  0.14882E+00_JPRB, 0.15812E+00_JPRB, 0.15853E+00_JPRB, 0.16057E+00_JPRB, 0.16191E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.16782E+00_JPRB, 0.16973E+00_JPRB, 0.17147E+00_JPRB, 0.17304E+00_JPRB, 0.17340E+00_JPRB, 0.17375E+00_JPRB, 0.17453E+00_JPRB ,&
&  0.17096E+00_JPRB, 0.16826E+00_JPRB, 0.16381E+00_JPRB, 0.15945E+00_JPRB, 0.15533E+00_JPRB, 0.15176E+00_JPRB, 0.14962E+00_JPRB ,&
&  0.14856E+00_JPRB, 0.14858E+00_JPRB, 0.14905E+00_JPRB, 0.14986E+00_JPRB, 0.14955E+00_JPRB, 0.14882E+00_JPRB, 0.14801E+00_JPRB ,&
&  0.14683E+00_JPRB, 0.14676E+00_JPRB, 0.14703E+00_JPRB, 0.15006E+00_JPRB, 0.15437E+00_JPRB, 0.15934E+00_JPRB, 0.16282E+00_JPRB ,&
&  0.16535E+00_JPRB, 0.16722E+00_JPRB, 0.16937E+00_JPRB, 0.17027E+00_JPRB, 0.17095E+00_JPRB, 0.17209E+00_JPRB, 0.17272E+00_JPRB ,&
&  0.17406E+00_JPRB, 0.17440E+00_JPRB, 0.17449E+00_JPRB, 0.17509E+00_JPRB, 0.16735E+00_JPRB, 0.16139E+00_JPRB, 0.15541E+00_JPRB ,&
&  0.14952E+00_JPRB, 0.14362E+00_JPRB, 0.13835E+00_JPRB, 0.13529E+00_JPRB, 0.13245E+00_JPRB, 0.13077E+00_JPRB, 0.12930E+00_JPRB ,&
&  0.12823E+00_JPRB, 0.12711E+00_JPRB, 0.12555E+00_JPRB, 0.12402E+00_JPRB, 0.12315E+00_JPRB, 0.12240E+00_JPRB, 0.12148E+00_JPRB ,&
&  0.12138E+00_JPRB, 0.12081E+00_JPRB, 0.12068E+00_JPRB, 0.12028E+00_JPRB, 0.11998E+00_JPRB, 0.11973E+00_JPRB, 0.11974E+00_JPRB ,&
&  0.12006E+00_JPRB, 0.12067E+00_JPRB, 0.12084E+00_JPRB, 0.12104E+00_JPRB, 0.12084E+00_JPRB, 0.12027E+00_JPRB, 0.11974E+00_JPRB ,&
&  0.11924E+00_JPRB, 0.11883E+00_JPRB, 0.11880E+00_JPRB, 0.11874E+00_JPRB, 0.11920E+00_JPRB, 0.11983E+00_JPRB, 0.12048E+00_JPRB ,&
&  0.12134E+00_JPRB, 0.12203E+00_JPRB, 0.12271E+00_JPRB, 0.12352E+00_JPRB, 0.12425E+00_JPRB, 0.12518E+00_JPRB, 0.12626E+00_JPRB ,&
&  0.12713E+00_JPRB, 0.12871E+00_JPRB, 0.13002E+00_JPRB, 0.13161E+00_JPRB, 0.13444E+00_JPRB, 0.13786E+00_JPRB, 0.14549E+00_JPRB ,&
&  0.15348E+00_JPRB, 0.17064E+00_JPRB, 0.17230E+00_JPRB, 0.16648E+00_JPRB, 0.16677E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.17565E+00_JPRB, 0.17795E+00_JPRB, 0.18048E+00_JPRB, 0.17941E+00_JPRB, 0.17951E+00_JPRB, 0.17984E+00_JPRB, 0.18107E+00_JPRB ,&
&  0.18169E+00_JPRB, 0.18142E+00_JPRB, 0.18110E+00_JPRB, 0.17995E+00_JPRB, 0.17850E+00_JPRB, 0.17146E+00_JPRB, 0.16827E+00_JPRB ,&
&  0.16587E+00_JPRB, 0.16617E+00_JPRB, 0.16664E+00_JPRB, 0.16858E+00_JPRB, 0.16899E+00_JPRB, 0.16842E+00_JPRB, 0.16576E+00_JPRB ,&
&  0.16339E+00_JPRB, 0.16168E+00_JPRB, 0.16002E+00_JPRB, 0.15997E+00_JPRB, 0.16173E+00_JPRB, 0.16376E+00_JPRB, 0.16487E+00_JPRB ,&
&  0.16500E+00_JPRB, 0.16660E+00_JPRB, 0.16791E+00_JPRB, 0.17041E+00_JPRB, 0.17764E+00_JPRB, 0.17940E+00_JPRB, 0.17999E+00_JPRB ,&
&  0.18083E+00_JPRB, 0.18222E+00_JPRB, 0.18316E+00_JPRB, 0.18325E+00_JPRB, 0.18179E+00_JPRB, 0.17778E+00_JPRB, 0.17674E+00_JPRB ,&
&  0.16406E+00_JPRB, 0.15658E+00_JPRB, 0.14931E+00_JPRB, 0.14383E+00_JPRB, 0.13927E+00_JPRB, 0.13674E+00_JPRB, 0.13393E+00_JPRB ,&
&  0.13158E+00_JPRB, 0.12906E+00_JPRB, 0.12713E+00_JPRB, 0.12619E+00_JPRB, 0.12593E+00_JPRB, 0.12570E+00_JPRB, 0.12520E+00_JPRB ,&
&  0.12526E+00_JPRB, 0.12487E+00_JPRB, 0.12457E+00_JPRB, 0.12426E+00_JPRB, 0.12407E+00_JPRB, 0.12415E+00_JPRB, 0.12479E+00_JPRB ,&
&  0.12649E+00_JPRB, 0.12819E+00_JPRB, 0.12982E+00_JPRB, 0.13114E+00_JPRB, 0.13061E+00_JPRB, 0.12910E+00_JPRB, 0.12678E+00_JPRB ,&
&  0.12485E+00_JPRB, 0.12361E+00_JPRB, 0.12228E+00_JPRB, 0.12195E+00_JPRB, 0.12247E+00_JPRB, 0.12315E+00_JPRB, 0.12429E+00_JPRB ,&
&  0.12529E+00_JPRB, 0.12610E+00_JPRB, 0.12664E+00_JPRB, 0.12704E+00_JPRB, 0.12767E+00_JPRB, 0.12831E+00_JPRB, 0.12934E+00_JPRB ,&
&  0.13049E+00_JPRB, 0.13192E+00_JPRB, 0.13326E+00_JPRB, 0.13502E+00_JPRB, 0.13755E+00_JPRB, 0.14154E+00_JPRB, 0.14833E+00_JPRB ,&
&  0.15553E+00_JPRB, 0.17448E+00_JPRB, 0.17666E+00_JPRB, 0.17409E+00_JPRB, 0.17512E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.18271E+00_JPRB, 0.18680E+00_JPRB, 0.18930E+00_JPRB, 0.18893E+00_JPRB, 0.18863E+00_JPRB, 0.18936E+00_JPRB, 0.19027E+00_JPRB ,&
&  0.19141E+00_JPRB, 0.19104E+00_JPRB, 0.18973E+00_JPRB, 0.18836E+00_JPRB, 0.18685E+00_JPRB, 0.18485E+00_JPRB, 0.18341E+00_JPRB ,&
&  0.18228E+00_JPRB, 0.18212E+00_JPRB, 0.18154E+00_JPRB, 0.18220E+00_JPRB, 0.18272E+00_JPRB, 0.18536E+00_JPRB, 0.18427E+00_JPRB ,&
&  0.18113E+00_JPRB, 0.17954E+00_JPRB, 0.18171E+00_JPRB, 0.18212E+00_JPRB, 0.18264E+00_JPRB, 0.18345E+00_JPRB, 0.18466E+00_JPRB ,&
&  0.18243E+00_JPRB, 0.17920E+00_JPRB, 0.17409E+00_JPRB, 0.17849E+00_JPRB, 0.18804E+00_JPRB, 0.18858E+00_JPRB, 0.18765E+00_JPRB ,&
&  0.18888E+00_JPRB, 0.19155E+00_JPRB, 0.19286E+00_JPRB, 0.19330E+00_JPRB, 0.19185E+00_JPRB, 0.19109E+00_JPRB, 0.19007E+00_JPRB ,&
&  0.18754E+00_JPRB, 0.18485E+00_JPRB, 0.17717E+00_JPRB, 0.16220E+00_JPRB, 0.15546E+00_JPRB, 0.15205E+00_JPRB, 0.14809E+00_JPRB ,&
&  0.14328E+00_JPRB, 0.14068E+00_JPRB, 0.14027E+00_JPRB, 0.13909E+00_JPRB, 0.13937E+00_JPRB, 0.13888E+00_JPRB, 0.13811E+00_JPRB ,&
&  0.13704E+00_JPRB, 0.13625E+00_JPRB, 0.13480E+00_JPRB, 0.13255E+00_JPRB, 0.13091E+00_JPRB, 0.13048E+00_JPRB, 0.13110E+00_JPRB ,&
&  0.13284E+00_JPRB, 0.13573E+00_JPRB, 0.14358E+00_JPRB, 0.14818E+00_JPRB, 0.14621E+00_JPRB, 0.14185E+00_JPRB, 0.13610E+00_JPRB ,&
&  0.13140E+00_JPRB, 0.12988E+00_JPRB, 0.12883E+00_JPRB, 0.12797E+00_JPRB, 0.12826E+00_JPRB, 0.12923E+00_JPRB, 0.13021E+00_JPRB ,&
&  0.13127E+00_JPRB, 0.13241E+00_JPRB, 0.13309E+00_JPRB, 0.13345E+00_JPRB, 0.13381E+00_JPRB, 0.13403E+00_JPRB, 0.13449E+00_JPRB ,&
&  0.13472E+00_JPRB, 0.13599E+00_JPRB, 0.13720E+00_JPRB, 0.13938E+00_JPRB, 0.14091E+00_JPRB, 0.14273E+00_JPRB, 0.14634E+00_JPRB ,&
&  0.14975E+00_JPRB, 0.15952E+00_JPRB, 0.17374E+00_JPRB, 0.18242E+00_JPRB, 0.17996E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.19271E+00_JPRB, 0.19383E+00_JPRB, 0.19401E+00_JPRB, 0.19320E+00_JPRB, 0.19206E+00_JPRB, 0.19236E+00_JPRB, 0.19381E+00_JPRB ,&
&  0.19670E+00_JPRB, 0.19890E+00_JPRB, 0.19953E+00_JPRB, 0.19968E+00_JPRB, 0.19879E+00_JPRB, 0.19670E+00_JPRB, 0.19488E+00_JPRB ,&
&  0.19341E+00_JPRB, 0.19249E+00_JPRB, 0.19176E+00_JPRB, 0.19148E+00_JPRB, 0.19149E+00_JPRB, 0.19271E+00_JPRB, 0.19403E+00_JPRB ,&
&  0.19797E+00_JPRB, 0.19826E+00_JPRB, 0.19765E+00_JPRB, 0.19742E+00_JPRB, 0.19714E+00_JPRB, 0.19702E+00_JPRB, 0.19693E+00_JPRB ,&
&  0.19481E+00_JPRB, 0.18698E+00_JPRB, 0.17964E+00_JPRB, 0.18442E+00_JPRB, 0.19734E+00_JPRB, 0.19776E+00_JPRB, 0.19765E+00_JPRB ,&
&  0.19893E+00_JPRB, 0.20181E+00_JPRB, 0.20371E+00_JPRB, 0.20474E+00_JPRB, 0.20353E+00_JPRB, 0.20166E+00_JPRB, 0.20107E+00_JPRB ,&
&  0.20116E+00_JPRB, 0.19932E+00_JPRB, 0.19445E+00_JPRB, 0.18906E+00_JPRB, 0.18360E+00_JPRB, 0.18140E+00_JPRB, 0.17962E+00_JPRB ,&
&  0.17728E+00_JPRB, 0.17596E+00_JPRB, 0.17542E+00_JPRB, 0.17491E+00_JPRB, 0.17530E+00_JPRB, 0.17357E+00_JPRB, 0.17145E+00_JPRB ,&
&  0.16744E+00_JPRB, 0.16363E+00_JPRB, 0.15875E+00_JPRB, 0.15356E+00_JPRB, 0.14898E+00_JPRB, 0.14437E+00_JPRB, 0.14120E+00_JPRB ,&
&  0.14047E+00_JPRB, 0.14380E+00_JPRB, 0.15267E+00_JPRB, 0.16290E+00_JPRB, 0.17227E+00_JPRB, 0.16004E+00_JPRB, 0.15263E+00_JPRB ,&
&  0.14550E+00_JPRB, 0.14020E+00_JPRB, 0.13726E+00_JPRB, 0.13651E+00_JPRB, 0.13665E+00_JPRB, 0.13744E+00_JPRB, 0.13880E+00_JPRB ,&
&  0.14049E+00_JPRB, 0.14271E+00_JPRB, 0.14445E+00_JPRB, 0.14557E+00_JPRB, 0.14631E+00_JPRB, 0.14652E+00_JPRB, 0.14634E+00_JPRB ,&
&  0.14610E+00_JPRB, 0.14719E+00_JPRB, 0.14956E+00_JPRB, 0.15356E+00_JPRB, 0.15667E+00_JPRB, 0.16054E+00_JPRB, 0.16611E+00_JPRB ,&
&  0.17399E+00_JPRB, 0.18348E+00_JPRB, 0.18613E+00_JPRB, 0.18778E+00_JPRB, 0.19046E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.20211E+00_JPRB, 0.20333E+00_JPRB, 0.20347E+00_JPRB, 0.20378E+00_JPRB, 0.20430E+00_JPRB, 0.20609E+00_JPRB, 0.21013E+00_JPRB ,&
&  0.21407E+00_JPRB, 0.21592E+00_JPRB, 0.21458E+00_JPRB, 0.21253E+00_JPRB, 0.21147E+00_JPRB, 0.21082E+00_JPRB, 0.21053E+00_JPRB ,&
&  0.21026E+00_JPRB, 0.20921E+00_JPRB, 0.20641E+00_JPRB, 0.20264E+00_JPRB, 0.20002E+00_JPRB, 0.19962E+00_JPRB, 0.20032E+00_JPRB ,&
&  0.20430E+00_JPRB, 0.20712E+00_JPRB, 0.20978E+00_JPRB, 0.21189E+00_JPRB, 0.21225E+00_JPRB, 0.21175E+00_JPRB, 0.21300E+00_JPRB ,&
&  0.21408E+00_JPRB, 0.21217E+00_JPRB, 0.20696E+00_JPRB, 0.21136E+00_JPRB, 0.21327E+00_JPRB, 0.21445E+00_JPRB, 0.21372E+00_JPRB ,&
&  0.21443E+00_JPRB, 0.21650E+00_JPRB, 0.21834E+00_JPRB, 0.21983E+00_JPRB, 0.21908E+00_JPRB, 0.21577E+00_JPRB, 0.21446E+00_JPRB ,&
&  0.21436E+00_JPRB, 0.21320E+00_JPRB, 0.21066E+00_JPRB, 0.20839E+00_JPRB, 0.20589E+00_JPRB, 0.20482E+00_JPRB, 0.20459E+00_JPRB ,&
&  0.20364E+00_JPRB, 0.20148E+00_JPRB, 0.19715E+00_JPRB, 0.19296E+00_JPRB, 0.19001E+00_JPRB, 0.18725E+00_JPRB, 0.18642E+00_JPRB ,&
&  0.18708E+00_JPRB, 0.18840E+00_JPRB, 0.18859E+00_JPRB, 0.18751E+00_JPRB, 0.18368E+00_JPRB, 0.17299E+00_JPRB, 0.16600E+00_JPRB ,&
&  0.16479E+00_JPRB, 0.16959E+00_JPRB, 0.17813E+00_JPRB, 0.17874E+00_JPRB, 0.18047E+00_JPRB, 0.18184E+00_JPRB, 0.18327E+00_JPRB ,&
&  0.18259E+00_JPRB, 0.16899E+00_JPRB, 0.16297E+00_JPRB, 0.15981E+00_JPRB, 0.15667E+00_JPRB, 0.15594E+00_JPRB, 0.15723E+00_JPRB ,&
&  0.15994E+00_JPRB, 0.16392E+00_JPRB, 0.16648E+00_JPRB, 0.16783E+00_JPRB, 0.16810E+00_JPRB, 0.16747E+00_JPRB, 0.16671E+00_JPRB ,&
&  0.16663E+00_JPRB, 0.17018E+00_JPRB, 0.17766E+00_JPRB, 0.18670E+00_JPRB, 0.18765E+00_JPRB, 0.18928E+00_JPRB, 0.19004E+00_JPRB ,&
&  0.19069E+00_JPRB, 0.19247E+00_JPRB, 0.19359E+00_JPRB, 0.19627E+00_JPRB, 0.19952E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.21212E+00_JPRB, 0.21511E+00_JPRB, 0.21814E+00_JPRB, 0.21998E+00_JPRB, 0.22186E+00_JPRB, 0.22371E+00_JPRB, 0.22694E+00_JPRB ,&
&  0.23068E+00_JPRB, 0.23178E+00_JPRB, 0.23012E+00_JPRB, 0.22748E+00_JPRB, 0.22575E+00_JPRB, 0.22492E+00_JPRB, 0.22460E+00_JPRB ,&
&  0.22437E+00_JPRB, 0.22421E+00_JPRB, 0.22349E+00_JPRB, 0.22186E+00_JPRB, 0.21957E+00_JPRB, 0.21735E+00_JPRB, 0.21580E+00_JPRB ,&
&  0.21586E+00_JPRB, 0.21756E+00_JPRB, 0.22014E+00_JPRB, 0.22070E+00_JPRB, 0.22103E+00_JPRB, 0.22229E+00_JPRB, 0.22418E+00_JPRB ,&
&  0.22797E+00_JPRB, 0.22999E+00_JPRB, 0.23230E+00_JPRB, 0.22811E+00_JPRB, 0.23286E+00_JPRB, 0.23452E+00_JPRB, 0.23509E+00_JPRB ,&
&  0.23639E+00_JPRB, 0.23737E+00_JPRB, 0.23820E+00_JPRB, 0.23741E+00_JPRB, 0.23298E+00_JPRB, 0.23070E+00_JPRB, 0.23072E+00_JPRB ,&
&  0.23083E+00_JPRB, 0.22918E+00_JPRB, 0.22640E+00_JPRB, 0.22381E+00_JPRB, 0.22174E+00_JPRB, 0.22032E+00_JPRB, 0.21915E+00_JPRB ,&
&  0.21853E+00_JPRB, 0.21789E+00_JPRB, 0.21617E+00_JPRB, 0.21126E+00_JPRB, 0.20558E+00_JPRB, 0.20192E+00_JPRB, 0.19949E+00_JPRB ,&
&  0.19974E+00_JPRB, 0.20172E+00_JPRB, 0.20338E+00_JPRB, 0.20363E+00_JPRB, 0.20268E+00_JPRB, 0.20093E+00_JPRB, 0.19803E+00_JPRB ,&
&  0.19333E+00_JPRB, 0.19115E+00_JPRB, 0.19027E+00_JPRB, 0.18926E+00_JPRB, 0.18963E+00_JPRB, 0.19129E+00_JPRB, 0.19284E+00_JPRB ,&
&  0.19466E+00_JPRB, 0.19468E+00_JPRB, 0.19335E+00_JPRB, 0.19108E+00_JPRB, 0.18870E+00_JPRB, 0.18851E+00_JPRB, 0.18895E+00_JPRB ,&
&  0.18926E+00_JPRB, 0.18946E+00_JPRB, 0.19001E+00_JPRB, 0.19089E+00_JPRB, 0.19218E+00_JPRB, 0.19426E+00_JPRB, 0.19607E+00_JPRB ,&
&  0.19604E+00_JPRB, 0.19497E+00_JPRB, 0.19593E+00_JPRB, 0.19737E+00_JPRB, 0.19807E+00_JPRB, 0.19855E+00_JPRB, 0.19887E+00_JPRB ,&
&  0.19860E+00_JPRB, 0.19879E+00_JPRB, 0.20070E+00_JPRB, 0.20521E+00_JPRB, 0.20936E+00_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.22341E+00_JPRB, 0.22705E+00_JPRB, 0.23211E+00_JPRB, 0.23361E+00_JPRB, 0.23648E+00_JPRB, 0.23928E+00_JPRB, 0.24277E+00_JPRB ,&
&  0.24488E+00_JPRB, 0.24490E+00_JPRB, 0.24332E+00_JPRB, 0.24179E+00_JPRB, 0.24088E+00_JPRB, 0.24040E+00_JPRB, 0.23970E+00_JPRB ,&
&  0.23940E+00_JPRB, 0.23950E+00_JPRB, 0.23951E+00_JPRB, 0.23950E+00_JPRB, 0.23861E+00_JPRB, 0.23720E+00_JPRB, 0.23564E+00_JPRB ,&
&  0.23281E+00_JPRB, 0.23107E+00_JPRB, 0.23077E+00_JPRB, 0.23096E+00_JPRB, 0.23067E+00_JPRB, 0.23147E+00_JPRB, 0.23332E+00_JPRB ,&
&  0.23711E+00_JPRB, 0.24017E+00_JPRB, 0.24407E+00_JPRB, 0.24813E+00_JPRB, 0.25060E+00_JPRB, 0.25236E+00_JPRB, 0.25441E+00_JPRB ,&
&  0.25567E+00_JPRB, 0.25591E+00_JPRB, 0.25577E+00_JPRB, 0.25553E+00_JPRB, 0.25241E+00_JPRB, 0.25077E+00_JPRB, 0.24937E+00_JPRB ,&
&  0.24762E+00_JPRB, 0.24495E+00_JPRB, 0.24139E+00_JPRB, 0.23828E+00_JPRB, 0.23519E+00_JPRB, 0.23303E+00_JPRB, 0.23129E+00_JPRB ,&
&  0.23006E+00_JPRB, 0.22964E+00_JPRB, 0.22992E+00_JPRB, 0.22884E+00_JPRB, 0.22415E+00_JPRB, 0.21945E+00_JPRB, 0.21680E+00_JPRB ,&
&  0.21558E+00_JPRB, 0.21541E+00_JPRB, 0.21682E+00_JPRB, 0.21585E+00_JPRB, 0.21269E+00_JPRB, 0.21031E+00_JPRB, 0.20933E+00_JPRB ,&
&  0.20783E+00_JPRB, 0.20701E+00_JPRB, 0.20751E+00_JPRB, 0.20808E+00_JPRB, 0.20872E+00_JPRB, 0.20902E+00_JPRB, 0.20998E+00_JPRB ,&
&  0.21066E+00_JPRB, 0.21181E+00_JPRB, 0.21148E+00_JPRB, 0.20884E+00_JPRB, 0.20525E+00_JPRB, 0.20257E+00_JPRB, 0.20189E+00_JPRB ,&
&  0.20308E+00_JPRB, 0.20428E+00_JPRB, 0.20448E+00_JPRB, 0.20447E+00_JPRB, 0.20543E+00_JPRB, 0.20779E+00_JPRB, 0.21088E+00_JPRB ,&
&  0.21216E+00_JPRB, 0.21340E+00_JPRB, 0.21517E+00_JPRB, 0.21695E+00_JPRB, 0.21808E+00_JPRB, 0.21864E+00_JPRB, 0.21869E+00_JPRB ,&
&  0.21798E+00_JPRB, 0.21863E+00_JPRB, 0.22009E+00_JPRB, 0.22187E+00_JPRB, 0.22317E+00_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.24137E+00_JPRB, 0.24393E+00_JPRB, 0.24731E+00_JPRB, 0.24824E+00_JPRB, 0.24891E+00_JPRB, 0.25061E+00_JPRB, 0.25370E+00_JPRB ,&
&  0.25581E+00_JPRB, 0.25636E+00_JPRB, 0.25616E+00_JPRB, 0.25578E+00_JPRB, 0.25586E+00_JPRB, 0.25707E+00_JPRB, 0.25750E+00_JPRB ,&
&  0.25676E+00_JPRB, 0.25616E+00_JPRB, 0.25543E+00_JPRB, 0.25502E+00_JPRB, 0.25388E+00_JPRB, 0.25211E+00_JPRB, 0.25074E+00_JPRB ,&
&  0.24873E+00_JPRB, 0.24698E+00_JPRB, 0.24583E+00_JPRB, 0.24507E+00_JPRB, 0.24490E+00_JPRB, 0.24537E+00_JPRB, 0.24723E+00_JPRB ,&
&  0.24942E+00_JPRB, 0.25206E+00_JPRB, 0.25521E+00_JPRB, 0.25878E+00_JPRB, 0.26238E+00_JPRB, 0.26645E+00_JPRB, 0.27001E+00_JPRB ,&
&  0.27277E+00_JPRB, 0.27494E+00_JPRB, 0.27615E+00_JPRB, 0.27294E+00_JPRB, 0.26888E+00_JPRB, 0.26698E+00_JPRB, 0.26550E+00_JPRB ,&
&  0.26269E+00_JPRB, 0.25934E+00_JPRB, 0.25481E+00_JPRB, 0.25214E+00_JPRB, 0.25126E+00_JPRB, 0.25016E+00_JPRB, 0.24885E+00_JPRB ,&
&  0.24801E+00_JPRB, 0.24755E+00_JPRB, 0.24672E+00_JPRB, 0.24512E+00_JPRB, 0.24224E+00_JPRB, 0.23806E+00_JPRB, 0.23377E+00_JPRB ,&
&  0.23103E+00_JPRB, 0.22916E+00_JPRB, 0.22941E+00_JPRB, 0.22775E+00_JPRB, 0.22418E+00_JPRB, 0.22102E+00_JPRB, 0.22082E+00_JPRB ,&
&  0.22089E+00_JPRB, 0.22120E+00_JPRB, 0.22092E+00_JPRB, 0.21981E+00_JPRB, 0.22013E+00_JPRB, 0.22120E+00_JPRB, 0.22265E+00_JPRB ,&
&  0.22615E+00_JPRB, 0.22855E+00_JPRB, 0.22819E+00_JPRB, 0.22698E+00_JPRB, 0.22503E+00_JPRB, 0.22450E+00_JPRB, 0.22484E+00_JPRB ,&
&  0.22515E+00_JPRB, 0.22513E+00_JPRB, 0.22520E+00_JPRB, 0.22460E+00_JPRB, 0.22472E+00_JPRB, 0.22620E+00_JPRB, 0.22802E+00_JPRB ,&
&  0.23019E+00_JPRB, 0.23288E+00_JPRB, 0.23522E+00_JPRB, 0.23761E+00_JPRB, 0.23976E+00_JPRB, 0.24073E+00_JPRB, 0.24159E+00_JPRB ,&
&  0.24248E+00_JPRB, 0.24257E+00_JPRB, 0.24221E+00_JPRB, 0.24210E+00_JPRB, 0.24203E+00_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.26545E+00_JPRB, 0.26584E+00_JPRB, 0.26704E+00_JPRB, 0.26776E+00_JPRB, 0.26783E+00_JPRB, 0.26848E+00_JPRB, 0.27101E+00_JPRB ,&
&  0.27321E+00_JPRB, 0.27441E+00_JPRB, 0.27520E+00_JPRB, 0.27520E+00_JPRB, 0.27479E+00_JPRB, 0.27586E+00_JPRB, 0.27674E+00_JPRB ,&
&  0.27674E+00_JPRB, 0.27565E+00_JPRB, 0.27455E+00_JPRB, 0.27326E+00_JPRB, 0.27155E+00_JPRB, 0.26983E+00_JPRB, 0.26861E+00_JPRB ,&
&  0.26688E+00_JPRB, 0.26497E+00_JPRB, 0.26315E+00_JPRB, 0.26289E+00_JPRB, 0.26350E+00_JPRB, 0.26421E+00_JPRB, 0.26552E+00_JPRB ,&
&  0.26682E+00_JPRB, 0.26948E+00_JPRB, 0.27259E+00_JPRB, 0.27608E+00_JPRB, 0.27955E+00_JPRB, 0.28416E+00_JPRB, 0.28920E+00_JPRB ,&
&  0.29392E+00_JPRB, 0.29623E+00_JPRB, 0.29752E+00_JPRB, 0.29504E+00_JPRB, 0.29344E+00_JPRB, 0.29181E+00_JPRB, 0.29065E+00_JPRB ,&
&  0.28940E+00_JPRB, 0.28490E+00_JPRB, 0.28106E+00_JPRB, 0.28040E+00_JPRB, 0.28075E+00_JPRB, 0.28084E+00_JPRB, 0.28051E+00_JPRB ,&
&  0.27937E+00_JPRB, 0.27790E+00_JPRB, 0.27547E+00_JPRB, 0.27210E+00_JPRB, 0.26856E+00_JPRB, 0.26406E+00_JPRB, 0.25730E+00_JPRB ,&
&  0.25086E+00_JPRB, 0.24564E+00_JPRB, 0.24178E+00_JPRB, 0.23849E+00_JPRB, 0.23638E+00_JPRB, 0.23405E+00_JPRB, 0.23347E+00_JPRB ,&
&  0.23385E+00_JPRB, 0.23566E+00_JPRB, 0.23677E+00_JPRB, 0.23618E+00_JPRB, 0.23511E+00_JPRB, 0.23531E+00_JPRB, 0.23719E+00_JPRB ,&
&  0.24069E+00_JPRB, 0.24335E+00_JPRB, 0.24407E+00_JPRB, 0.24343E+00_JPRB, 0.24245E+00_JPRB, 0.24296E+00_JPRB, 0.24439E+00_JPRB ,&
&  0.24502E+00_JPRB, 0.24542E+00_JPRB, 0.24655E+00_JPRB, 0.24648E+00_JPRB, 0.24842E+00_JPRB, 0.25165E+00_JPRB, 0.25518E+00_JPRB ,&
&  0.25857E+00_JPRB, 0.26276E+00_JPRB, 0.26693E+00_JPRB, 0.27016E+00_JPRB, 0.27292E+00_JPRB, 0.27290E+00_JPRB, 0.27315E+00_JPRB ,&
&  0.27317E+00_JPRB, 0.27203E+00_JPRB, 0.27090E+00_JPRB, 0.26915E+00_JPRB, 0.26688E+00_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.29271E+00_JPRB, 0.29201E+00_JPRB, 0.29147E+00_JPRB, 0.29149E+00_JPRB, 0.29135E+00_JPRB, 0.29102E+00_JPRB, 0.29097E+00_JPRB ,&
&  0.29148E+00_JPRB, 0.29141E+00_JPRB, 0.29353E+00_JPRB, 0.29393E+00_JPRB, 0.29353E+00_JPRB, 0.29405E+00_JPRB, 0.29449E+00_JPRB ,&
&  0.29314E+00_JPRB, 0.29211E+00_JPRB, 0.29117E+00_JPRB, 0.28931E+00_JPRB, 0.28618E+00_JPRB, 0.28484E+00_JPRB, 0.28389E+00_JPRB ,&
&  0.28202E+00_JPRB, 0.27964E+00_JPRB, 0.27770E+00_JPRB, 0.27714E+00_JPRB, 0.27811E+00_JPRB, 0.27854E+00_JPRB, 0.27984E+00_JPRB ,&
&  0.28081E+00_JPRB, 0.28355E+00_JPRB, 0.27223E+00_JPRB, 0.28172E+00_JPRB, 0.29088E+00_JPRB, 0.29980E+00_JPRB, 0.30921E+00_JPRB ,&
&  0.31573E+00_JPRB, 0.32145E+00_JPRB, 0.32686E+00_JPRB, 0.32703E+00_JPRB, 0.32473E+00_JPRB, 0.32103E+00_JPRB, 0.31918E+00_JPRB ,&
&  0.31816E+00_JPRB, 0.31578E+00_JPRB, 0.31382E+00_JPRB, 0.31323E+00_JPRB, 0.31305E+00_JPRB, 0.31152E+00_JPRB, 0.30931E+00_JPRB ,&
&  0.30741E+00_JPRB, 0.30551E+00_JPRB, 0.30245E+00_JPRB, 0.29894E+00_JPRB, 0.29497E+00_JPRB, 0.28964E+00_JPRB, 0.28206E+00_JPRB ,&
&  0.27317E+00_JPRB, 0.26612E+00_JPRB, 0.26024E+00_JPRB, 0.25656E+00_JPRB, 0.25500E+00_JPRB, 0.25328E+00_JPRB, 0.25159E+00_JPRB ,&
&  0.25125E+00_JPRB, 0.25188E+00_JPRB, 0.25235E+00_JPRB, 0.25243E+00_JPRB, 0.25333E+00_JPRB, 0.25392E+00_JPRB, 0.25607E+00_JPRB ,&
&  0.25836E+00_JPRB, 0.25888E+00_JPRB, 0.25849E+00_JPRB, 0.25835E+00_JPRB, 0.25696E+00_JPRB, 0.25737E+00_JPRB, 0.25891E+00_JPRB ,&
&  0.26074E+00_JPRB, 0.26332E+00_JPRB, 0.26526E+00_JPRB, 0.26695E+00_JPRB, 0.26916E+00_JPRB, 0.27256E+00_JPRB, 0.27770E+00_JPRB ,&
&  0.28255E+00_JPRB, 0.28795E+00_JPRB, 0.29401E+00_JPRB, 0.29906E+00_JPRB, 0.30232E+00_JPRB, 0.30369E+00_JPRB, 0.30470E+00_JPRB ,&
&  0.30399E+00_JPRB, 0.30082E+00_JPRB, 0.29759E+00_JPRB, 0.29549E+00_JPRB, 0.29388E+00_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.33041E+00_JPRB, 0.32813E+00_JPRB, 0.32496E+00_JPRB, 0.32256E+00_JPRB, 0.32003E+00_JPRB, 0.31694E+00_JPRB, 0.31524E+00_JPRB ,&
&  0.31449E+00_JPRB, 0.31435E+00_JPRB, 0.31496E+00_JPRB, 0.31474E+00_JPRB, 0.31441E+00_JPRB, 0.31488E+00_JPRB, 0.31445E+00_JPRB ,&
&  0.31361E+00_JPRB, 0.31240E+00_JPRB, 0.31089E+00_JPRB, 0.30866E+00_JPRB, 0.30579E+00_JPRB, 0.30318E+00_JPRB, 0.29967E+00_JPRB ,&
&  0.29746E+00_JPRB, 0.29653E+00_JPRB, 0.29627E+00_JPRB, 0.29667E+00_JPRB, 0.29839E+00_JPRB, 0.30062E+00_JPRB, 0.30282E+00_JPRB ,&
&  0.30367E+00_JPRB, 0.29222E+00_JPRB, 0.29746E+00_JPRB, 0.30091E+00_JPRB, 0.30949E+00_JPRB, 0.32372E+00_JPRB, 0.33666E+00_JPRB ,&
&  0.34448E+00_JPRB, 0.35152E+00_JPRB, 0.35641E+00_JPRB, 0.35826E+00_JPRB, 0.35573E+00_JPRB, 0.35310E+00_JPRB, 0.35316E+00_JPRB ,&
&  0.35330E+00_JPRB, 0.35222E+00_JPRB, 0.35061E+00_JPRB, 0.34847E+00_JPRB, 0.34547E+00_JPRB, 0.34256E+00_JPRB, 0.33816E+00_JPRB ,&
&  0.33395E+00_JPRB, 0.33126E+00_JPRB, 0.32867E+00_JPRB, 0.32601E+00_JPRB, 0.32157E+00_JPRB, 0.31603E+00_JPRB, 0.30905E+00_JPRB ,&
&  0.30132E+00_JPRB, 0.29584E+00_JPRB, 0.29238E+00_JPRB, 0.28886E+00_JPRB, 0.28526E+00_JPRB, 0.28276E+00_JPRB, 0.27989E+00_JPRB ,&
&  0.27890E+00_JPRB, 0.27975E+00_JPRB, 0.27938E+00_JPRB, 0.27918E+00_JPRB, 0.27975E+00_JPRB, 0.27928E+00_JPRB, 0.27911E+00_JPRB ,&
&  0.27748E+00_JPRB, 0.27568E+00_JPRB, 0.27454E+00_JPRB, 0.27390E+00_JPRB, 0.27345E+00_JPRB, 0.27481E+00_JPRB, 0.27718E+00_JPRB ,&
&  0.27978E+00_JPRB, 0.28349E+00_JPRB, 0.28748E+00_JPRB, 0.29043E+00_JPRB, 0.29298E+00_JPRB, 0.29717E+00_JPRB, 0.30256E+00_JPRB ,&
&  0.30878E+00_JPRB, 0.31592E+00_JPRB, 0.32280E+00_JPRB, 0.32896E+00_JPRB, 0.33218E+00_JPRB, 0.33397E+00_JPRB, 0.33560E+00_JPRB ,&
&  0.33612E+00_JPRB, 0.33316E+00_JPRB, 0.33269E+00_JPRB, 0.33286E+00_JPRB, 0.33192E+00_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.35638E+00_JPRB, 0.35597E+00_JPRB, 0.35384E+00_JPRB, 0.35085E+00_JPRB, 0.34719E+00_JPRB, 0.34521E+00_JPRB, 0.34419E+00_JPRB ,&
&  0.34501E+00_JPRB, 0.34406E+00_JPRB, 0.34447E+00_JPRB, 0.34227E+00_JPRB, 0.34006E+00_JPRB, 0.33850E+00_JPRB, 0.33661E+00_JPRB ,&
&  0.33427E+00_JPRB, 0.33130E+00_JPRB, 0.32843E+00_JPRB, 0.32599E+00_JPRB, 0.32298E+00_JPRB, 0.31887E+00_JPRB, 0.31514E+00_JPRB ,&
&  0.31256E+00_JPRB, 0.31176E+00_JPRB, 0.31156E+00_JPRB, 0.31166E+00_JPRB, 0.31192E+00_JPRB, 0.31378E+00_JPRB, 0.31558E+00_JPRB ,&
&  0.31897E+00_JPRB, 0.29051E+00_JPRB, 0.29999E+00_JPRB, 0.30356E+00_JPRB, 0.31093E+00_JPRB, 0.32742E+00_JPRB, 0.34663E+00_JPRB ,&
&  0.36407E+00_JPRB, 0.37687E+00_JPRB, 0.38402E+00_JPRB, 0.38386E+00_JPRB, 0.37881E+00_JPRB, 0.37495E+00_JPRB, 0.37448E+00_JPRB ,&
&  0.37301E+00_JPRB, 0.36990E+00_JPRB, 0.36711E+00_JPRB, 0.36400E+00_JPRB, 0.35963E+00_JPRB, 0.35301E+00_JPRB, 0.34602E+00_JPRB ,&
&  0.34227E+00_JPRB, 0.34044E+00_JPRB, 0.33897E+00_JPRB, 0.33735E+00_JPRB, 0.33377E+00_JPRB, 0.33058E+00_JPRB, 0.32789E+00_JPRB ,&
&  0.32537E+00_JPRB, 0.32110E+00_JPRB, 0.31850E+00_JPRB, 0.31670E+00_JPRB, 0.31373E+00_JPRB, 0.31173E+00_JPRB, 0.31027E+00_JPRB ,&
&  0.30878E+00_JPRB, 0.30681E+00_JPRB, 0.30536E+00_JPRB, 0.30444E+00_JPRB, 0.30179E+00_JPRB, 0.29906E+00_JPRB, 0.29605E+00_JPRB ,&
&  0.29450E+00_JPRB, 0.29349E+00_JPRB, 0.29238E+00_JPRB, 0.29225E+00_JPRB, 0.29299E+00_JPRB, 0.29515E+00_JPRB, 0.29820E+00_JPRB ,&
&  0.30189E+00_JPRB, 0.30611E+00_JPRB, 0.30988E+00_JPRB, 0.31276E+00_JPRB, 0.31666E+00_JPRB, 0.31981E+00_JPRB, 0.32418E+00_JPRB ,&
&  0.32973E+00_JPRB, 0.33509E+00_JPRB, 0.34022E+00_JPRB, 0.34623E+00_JPRB, 0.35174E+00_JPRB, 0.35540E+00_JPRB, 0.35871E+00_JPRB ,&
&  0.36156E+00_JPRB, 0.35935E+00_JPRB, 0.35831E+00_JPRB, 0.35646E+00_JPRB, 0.35526E+00_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.38922E+00_JPRB, 0.38825E+00_JPRB, 0.38612E+00_JPRB, 0.38237E+00_JPRB, 0.37860E+00_JPRB, 0.37564E+00_JPRB, 0.37406E+00_JPRB ,&
&  0.37425E+00_JPRB, 0.37499E+00_JPRB, 0.37397E+00_JPRB, 0.37004E+00_JPRB, 0.36643E+00_JPRB, 0.36385E+00_JPRB, 0.36234E+00_JPRB ,&
&  0.36006E+00_JPRB, 0.35633E+00_JPRB, 0.35124E+00_JPRB, 0.34632E+00_JPRB, 0.34080E+00_JPRB, 0.33631E+00_JPRB, 0.33274E+00_JPRB ,&
&  0.32999E+00_JPRB, 0.32918E+00_JPRB, 0.32893E+00_JPRB, 0.32911E+00_JPRB, 0.33060E+00_JPRB, 0.33143E+00_JPRB, 0.33752E+00_JPRB ,&
&  0.31447E+00_JPRB, 0.28393E+00_JPRB, 0.29408E+00_JPRB, 0.30397E+00_JPRB, 0.31436E+00_JPRB, 0.33699E+00_JPRB, 0.36469E+00_JPRB ,&
&  0.39211E+00_JPRB, 0.41482E+00_JPRB, 0.42676E+00_JPRB, 0.42444E+00_JPRB, 0.41514E+00_JPRB, 0.40790E+00_JPRB, 0.40448E+00_JPRB ,&
&  0.39856E+00_JPRB, 0.39139E+00_JPRB, 0.38548E+00_JPRB, 0.37839E+00_JPRB, 0.37076E+00_JPRB, 0.36336E+00_JPRB, 0.35730E+00_JPRB ,&
&  0.35230E+00_JPRB, 0.34948E+00_JPRB, 0.34934E+00_JPRB, 0.34863E+00_JPRB, 0.34823E+00_JPRB, 0.34958E+00_JPRB, 0.34992E+00_JPRB ,&
&  0.35013E+00_JPRB, 0.34905E+00_JPRB, 0.34835E+00_JPRB, 0.34750E+00_JPRB, 0.34735E+00_JPRB, 0.34578E+00_JPRB, 0.34348E+00_JPRB ,&
&  0.34085E+00_JPRB, 0.33871E+00_JPRB, 0.33730E+00_JPRB, 0.33422E+00_JPRB, 0.32753E+00_JPRB, 0.32132E+00_JPRB, 0.31785E+00_JPRB ,&
&  0.31680E+00_JPRB, 0.31669E+00_JPRB, 0.31648E+00_JPRB, 0.31724E+00_JPRB, 0.31862E+00_JPRB, 0.32068E+00_JPRB, 0.32392E+00_JPRB ,&
&  0.32843E+00_JPRB, 0.33318E+00_JPRB, 0.33670E+00_JPRB, 0.33955E+00_JPRB, 0.34416E+00_JPRB, 0.34851E+00_JPRB, 0.35306E+00_JPRB ,&
&  0.35786E+00_JPRB, 0.36287E+00_JPRB, 0.36848E+00_JPRB, 0.37672E+00_JPRB, 0.38601E+00_JPRB, 0.39326E+00_JPRB, 0.39579E+00_JPRB ,&
&  0.39439E+00_JPRB, 0.38847E+00_JPRB, 0.38980E+00_JPRB, 0.38779E+00_JPRB, 0.38757E+00_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.40400E+00_JPRB, 0.40724E+00_JPRB, 0.40640E+00_JPRB, 0.40407E+00_JPRB, 0.40205E+00_JPRB, 0.39860E+00_JPRB, 0.39572E+00_JPRB ,&
&  0.39193E+00_JPRB, 0.38996E+00_JPRB, 0.38575E+00_JPRB, 0.38318E+00_JPRB, 0.38137E+00_JPRB, 0.38011E+00_JPRB, 0.37801E+00_JPRB ,&
&  0.37393E+00_JPRB, 0.36742E+00_JPRB, 0.36078E+00_JPRB, 0.35550E+00_JPRB, 0.35206E+00_JPRB, 0.34862E+00_JPRB, 0.34582E+00_JPRB ,&
&  0.34343E+00_JPRB, 0.34135E+00_JPRB, 0.33942E+00_JPRB, 0.34052E+00_JPRB, 0.34203E+00_JPRB, 0.34267E+00_JPRB, 0.34199E+00_JPRB ,&
&  0.34996E+00_JPRB, 0.30143E+00_JPRB, 0.30809E+00_JPRB, 0.31739E+00_JPRB, 0.32771E+00_JPRB, 0.35304E+00_JPRB, 0.38963E+00_JPRB ,&
&  0.41970E+00_JPRB, 0.44390E+00_JPRB, 0.45184E+00_JPRB, 0.44533E+00_JPRB, 0.43536E+00_JPRB, 0.42872E+00_JPRB, 0.42354E+00_JPRB ,&
&  0.41244E+00_JPRB, 0.40219E+00_JPRB, 0.39294E+00_JPRB, 0.38423E+00_JPRB, 0.37581E+00_JPRB, 0.36901E+00_JPRB, 0.36270E+00_JPRB ,&
&  0.35909E+00_JPRB, 0.35832E+00_JPRB, 0.35766E+00_JPRB, 0.35704E+00_JPRB, 0.35820E+00_JPRB, 0.36144E+00_JPRB, 0.36326E+00_JPRB ,&
&  0.36410E+00_JPRB, 0.36598E+00_JPRB, 0.36627E+00_JPRB, 0.36626E+00_JPRB, 0.36711E+00_JPRB, 0.37073E+00_JPRB, 0.36975E+00_JPRB ,&
&  0.36523E+00_JPRB, 0.35843E+00_JPRB, 0.35221E+00_JPRB, 0.34775E+00_JPRB, 0.34400E+00_JPRB, 0.34028E+00_JPRB, 0.33894E+00_JPRB ,&
&  0.33714E+00_JPRB, 0.33713E+00_JPRB, 0.33735E+00_JPRB, 0.33899E+00_JPRB, 0.34080E+00_JPRB, 0.34340E+00_JPRB, 0.34728E+00_JPRB ,&
&  0.35204E+00_JPRB, 0.35667E+00_JPRB, 0.36219E+00_JPRB, 0.36772E+00_JPRB, 0.37233E+00_JPRB, 0.37543E+00_JPRB, 0.37745E+00_JPRB ,&
&  0.37937E+00_JPRB, 0.38325E+00_JPRB, 0.38562E+00_JPRB, 0.39689E+00_JPRB, 0.40607E+00_JPRB, 0.41574E+00_JPRB, 0.41627E+00_JPRB ,&
&  0.41289E+00_JPRB, 0.40957E+00_JPRB, 0.35239E+00_JPRB, 0.38199E+00_JPRB, 0.39511E+00_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.43978E+00_JPRB, 0.44114E+00_JPRB, 0.43888E+00_JPRB, 0.43701E+00_JPRB, 0.43175E+00_JPRB, 0.42593E+00_JPRB, 0.42068E+00_JPRB ,&
&  0.41376E+00_JPRB, 0.40687E+00_JPRB, 0.40057E+00_JPRB, 0.39847E+00_JPRB, 0.39580E+00_JPRB, 0.39236E+00_JPRB, 0.38592E+00_JPRB ,&
&  0.37777E+00_JPRB, 0.36981E+00_JPRB, 0.36265E+00_JPRB, 0.35850E+00_JPRB, 0.35797E+00_JPRB, 0.35679E+00_JPRB, 0.35459E+00_JPRB ,&
&  0.35383E+00_JPRB, 0.35571E+00_JPRB, 0.35604E+00_JPRB, 0.35622E+00_JPRB, 0.35604E+00_JPRB, 0.35195E+00_JPRB, 0.35272E+00_JPRB ,&
&  0.36232E+00_JPRB, 0.31759E+00_JPRB, 0.33170E+00_JPRB, 0.35220E+00_JPRB, 0.36618E+00_JPRB, 0.39195E+00_JPRB, 0.43080E+00_JPRB ,&
&  0.46852E+00_JPRB, 0.48455E+00_JPRB, 0.48439E+00_JPRB, 0.46872E+00_JPRB, 0.45644E+00_JPRB, 0.44704E+00_JPRB, 0.43110E+00_JPRB ,&
&  0.41488E+00_JPRB, 0.40455E+00_JPRB, 0.39829E+00_JPRB, 0.39323E+00_JPRB, 0.39044E+00_JPRB, 0.38809E+00_JPRB, 0.38627E+00_JPRB ,&
&  0.38309E+00_JPRB, 0.37961E+00_JPRB, 0.37617E+00_JPRB, 0.37468E+00_JPRB, 0.37691E+00_JPRB, 0.38210E+00_JPRB, 0.38878E+00_JPRB ,&
&  0.39389E+00_JPRB, 0.39792E+00_JPRB, 0.39903E+00_JPRB, 0.39986E+00_JPRB, 0.40120E+00_JPRB, 0.40156E+00_JPRB, 0.39742E+00_JPRB ,&
&  0.38858E+00_JPRB, 0.37948E+00_JPRB, 0.37354E+00_JPRB, 0.37022E+00_JPRB, 0.36899E+00_JPRB, 0.36911E+00_JPRB, 0.37043E+00_JPRB ,&
&  0.36991E+00_JPRB, 0.36879E+00_JPRB, 0.36751E+00_JPRB, 0.36797E+00_JPRB, 0.36903E+00_JPRB, 0.37246E+00_JPRB, 0.37738E+00_JPRB ,&
&  0.38267E+00_JPRB, 0.38775E+00_JPRB, 0.39341E+00_JPRB, 0.39897E+00_JPRB, 0.40197E+00_JPRB, 0.40575E+00_JPRB, 0.40698E+00_JPRB ,&
&  0.40728E+00_JPRB, 0.40947E+00_JPRB, 0.41385E+00_JPRB, 0.41720E+00_JPRB, 0.43962E+00_JPRB, 0.44853E+00_JPRB, 0.44057E+00_JPRB ,&
&  0.43344E+00_JPRB, 0.42277E+00_JPRB, 0.42564E+00_JPRB, 0.39315E+00_JPRB, 0.42757E+00_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.48280E+00_JPRB, 0.47878E+00_JPRB, 0.47135E+00_JPRB, 0.46446E+00_JPRB, 0.45485E+00_JPRB, 0.44746E+00_JPRB, 0.43779E+00_JPRB ,&
&  0.42916E+00_JPRB, 0.42044E+00_JPRB, 0.41237E+00_JPRB, 0.40997E+00_JPRB, 0.40423E+00_JPRB, 0.39556E+00_JPRB, 0.38245E+00_JPRB ,&
&  0.37135E+00_JPRB, 0.36235E+00_JPRB, 0.35711E+00_JPRB, 0.35557E+00_JPRB, 0.35500E+00_JPRB, 0.35313E+00_JPRB, 0.35262E+00_JPRB ,&
&  0.35399E+00_JPRB, 0.35413E+00_JPRB, 0.35703E+00_JPRB, 0.35744E+00_JPRB, 0.36160E+00_JPRB, 0.35831E+00_JPRB, 0.35628E+00_JPRB ,&
&  0.36761E+00_JPRB, 0.33860E+00_JPRB, 0.36707E+00_JPRB, 0.40188E+00_JPRB, 0.43272E+00_JPRB, 0.45561E+00_JPRB, 0.48530E+00_JPRB ,&
&  0.50936E+00_JPRB, 0.51325E+00_JPRB, 0.50574E+00_JPRB, 0.48488E+00_JPRB, 0.46940E+00_JPRB, 0.44966E+00_JPRB, 0.42881E+00_JPRB ,&
&  0.41874E+00_JPRB, 0.41496E+00_JPRB, 0.41162E+00_JPRB, 0.40818E+00_JPRB, 0.40385E+00_JPRB, 0.40245E+00_JPRB, 0.40353E+00_JPRB ,&
&  0.40144E+00_JPRB, 0.39802E+00_JPRB, 0.39299E+00_JPRB, 0.39205E+00_JPRB, 0.39229E+00_JPRB, 0.40071E+00_JPRB, 0.41214E+00_JPRB ,&
&  0.41876E+00_JPRB, 0.42576E+00_JPRB, 0.43040E+00_JPRB, 0.43554E+00_JPRB, 0.43438E+00_JPRB, 0.43168E+00_JPRB, 0.42104E+00_JPRB ,&
&  0.41297E+00_JPRB, 0.40801E+00_JPRB, 0.40582E+00_JPRB, 0.40463E+00_JPRB, 0.40412E+00_JPRB, 0.40216E+00_JPRB, 0.40212E+00_JPRB ,&
&  0.40003E+00_JPRB, 0.39728E+00_JPRB, 0.39676E+00_JPRB, 0.39744E+00_JPRB, 0.39829E+00_JPRB, 0.39914E+00_JPRB, 0.40043E+00_JPRB ,&
&  0.40227E+00_JPRB, 0.40343E+00_JPRB, 0.40595E+00_JPRB, 0.40870E+00_JPRB, 0.41004E+00_JPRB, 0.41242E+00_JPRB, 0.41617E+00_JPRB ,&
&  0.41864E+00_JPRB, 0.42393E+00_JPRB, 0.42955E+00_JPRB, 0.45092E+00_JPRB, 0.46409E+00_JPRB, 0.46125E+00_JPRB, 0.45089E+00_JPRB ,&
&  0.44192E+00_JPRB, 0.43803E+00_JPRB, 0.44069E+00_JPRB, 0.45855E+00_JPRB, 0.45980E+00_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.57880E+00_JPRB, 0.54191E+00_JPRB, 0.50791E+00_JPRB, 0.48795E+00_JPRB, 0.47580E+00_JPRB, 0.46762E+00_JPRB, 0.45713E+00_JPRB ,&
&  0.44711E+00_JPRB, 0.43610E+00_JPRB, 0.42727E+00_JPRB, 0.42143E+00_JPRB, 0.41444E+00_JPRB, 0.40075E+00_JPRB, 0.38252E+00_JPRB ,&
&  0.36997E+00_JPRB, 0.36805E+00_JPRB, 0.36662E+00_JPRB, 0.36262E+00_JPRB, 0.35900E+00_JPRB, 0.35870E+00_JPRB, 0.35714E+00_JPRB ,&
&  0.35739E+00_JPRB, 0.35752E+00_JPRB, 0.36046E+00_JPRB, 0.36223E+00_JPRB, 0.36126E+00_JPRB, 0.36817E+00_JPRB, 0.36783E+00_JPRB ,&
&  0.37319E+00_JPRB, 0.34708E+00_JPRB, 0.40276E+00_JPRB, 0.46820E+00_JPRB, 0.50809E+00_JPRB, 0.53628E+00_JPRB, 0.54810E+00_JPRB ,&
&  0.54656E+00_JPRB, 0.53992E+00_JPRB, 0.52569E+00_JPRB, 0.50767E+00_JPRB, 0.48601E+00_JPRB, 0.45992E+00_JPRB, 0.44387E+00_JPRB ,&
&  0.44100E+00_JPRB, 0.43672E+00_JPRB, 0.43084E+00_JPRB, 0.42219E+00_JPRB, 0.41312E+00_JPRB, 0.41226E+00_JPRB, 0.41611E+00_JPRB ,&
&  0.41529E+00_JPRB, 0.41388E+00_JPRB, 0.41196E+00_JPRB, 0.41531E+00_JPRB, 0.42698E+00_JPRB, 0.44925E+00_JPRB, 0.46793E+00_JPRB ,&
&  0.47662E+00_JPRB, 0.48670E+00_JPRB, 0.48964E+00_JPRB, 0.48925E+00_JPRB, 0.48131E+00_JPRB, 0.47192E+00_JPRB, 0.45968E+00_JPRB ,&
&  0.44989E+00_JPRB, 0.44297E+00_JPRB, 0.44046E+00_JPRB, 0.43745E+00_JPRB, 0.43423E+00_JPRB, 0.43161E+00_JPRB, 0.42890E+00_JPRB ,&
&  0.42826E+00_JPRB, 0.42817E+00_JPRB, 0.42925E+00_JPRB, 0.42794E+00_JPRB, 0.42623E+00_JPRB, 0.42346E+00_JPRB, 0.42049E+00_JPRB ,&
&  0.41502E+00_JPRB, 0.41134E+00_JPRB, 0.41546E+00_JPRB, 0.42617E+00_JPRB, 0.43845E+00_JPRB, 0.44682E+00_JPRB, 0.45227E+00_JPRB ,&
&  0.45492E+00_JPRB, 0.46253E+00_JPRB, 0.45319E+00_JPRB, 0.47123E+00_JPRB, 0.51587E+00_JPRB, 0.50456E+00_JPRB, 0.49478E+00_JPRB ,&
&  0.47932E+00_JPRB, 0.47181E+00_JPRB, 0.47261E+00_JPRB, 0.49220E+00_JPRB, 0.57206E+00_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.53159E+00_JPRB, 0.49937E+00_JPRB, 0.48386E+00_JPRB, 0.47557E+00_JPRB, 0.47266E+00_JPRB, 0.47126E+00_JPRB, 0.46349E+00_JPRB ,&
&  0.45146E+00_JPRB, 0.44033E+00_JPRB, 0.43256E+00_JPRB, 0.42686E+00_JPRB, 0.41472E+00_JPRB, 0.39604E+00_JPRB, 0.38242E+00_JPRB ,&
&  0.38057E+00_JPRB, 0.37857E+00_JPRB, 0.37745E+00_JPRB, 0.37301E+00_JPRB, 0.36830E+00_JPRB, 0.36671E+00_JPRB, 0.36443E+00_JPRB ,&
&  0.36104E+00_JPRB, 0.35728E+00_JPRB, 0.35806E+00_JPRB, 0.36157E+00_JPRB, 0.36550E+00_JPRB, 0.36982E+00_JPRB, 0.37388E+00_JPRB ,&
&  0.37412E+00_JPRB, 0.38172E+00_JPRB, 0.48494E+00_JPRB, 0.62007E+00_JPRB, 0.65459E+00_JPRB, 0.65597E+00_JPRB, 0.65556E+00_JPRB ,&
&  0.63117E+00_JPRB, 0.60037E+00_JPRB, 0.57057E+00_JPRB, 0.53863E+00_JPRB, 0.50062E+00_JPRB, 0.48094E+00_JPRB, 0.47244E+00_JPRB ,&
&  0.46561E+00_JPRB, 0.45570E+00_JPRB, 0.44442E+00_JPRB, 0.43249E+00_JPRB, 0.42498E+00_JPRB, 0.42942E+00_JPRB, 0.43908E+00_JPRB ,&
&  0.43859E+00_JPRB, 0.44058E+00_JPRB, 0.44233E+00_JPRB, 0.45445E+00_JPRB, 0.48957E+00_JPRB, 0.52556E+00_JPRB, 0.55369E+00_JPRB ,&
&  0.56291E+00_JPRB, 0.58940E+00_JPRB, 0.57329E+00_JPRB, 0.55441E+00_JPRB, 0.53416E+00_JPRB, 0.51598E+00_JPRB, 0.50060E+00_JPRB ,&
&  0.49052E+00_JPRB, 0.48309E+00_JPRB, 0.47755E+00_JPRB, 0.47238E+00_JPRB, 0.46697E+00_JPRB, 0.46173E+00_JPRB, 0.46169E+00_JPRB ,&
&  0.46237E+00_JPRB, 0.45758E+00_JPRB, 0.45334E+00_JPRB, 0.44839E+00_JPRB, 0.43952E+00_JPRB, 0.43107E+00_JPRB, 0.42450E+00_JPRB ,&
&  0.41790E+00_JPRB, 0.41788E+00_JPRB, 0.43666E+00_JPRB, 0.45877E+00_JPRB, 0.47383E+00_JPRB, 0.48577E+00_JPRB, 0.49323E+00_JPRB ,&
&  0.49577E+00_JPRB, 0.49899E+00_JPRB, 0.49293E+00_JPRB, 0.50032E+00_JPRB, 0.52142E+00_JPRB, 0.55150E+00_JPRB, 0.53197E+00_JPRB ,&
&  0.51702E+00_JPRB, 0.51016E+00_JPRB, 0.50911E+00_JPRB, 0.51082E+00_JPRB, 0.56164E+00_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.50842E+00_JPRB, 0.48895E+00_JPRB, 0.48028E+00_JPRB, 0.47515E+00_JPRB, 0.47181E+00_JPRB, 0.46941E+00_JPRB, 0.46192E+00_JPRB ,&
&  0.44976E+00_JPRB, 0.44250E+00_JPRB, 0.43424E+00_JPRB, 0.42179E+00_JPRB, 0.40430E+00_JPRB, 0.39479E+00_JPRB, 0.39525E+00_JPRB ,&
&  0.39469E+00_JPRB, 0.39165E+00_JPRB, 0.38631E+00_JPRB, 0.38451E+00_JPRB, 0.38086E+00_JPRB, 0.37534E+00_JPRB, 0.37260E+00_JPRB ,&
&  0.36471E+00_JPRB, 0.36098E+00_JPRB, 0.36242E+00_JPRB, 0.36724E+00_JPRB, 0.37176E+00_JPRB, 0.37857E+00_JPRB, 0.38371E+00_JPRB ,&
&  0.39299E+00_JPRB, 0.44036E+00_JPRB, 0.67710E+00_JPRB, 0.84747E+00_JPRB, 0.90052E+00_JPRB, 0.87508E+00_JPRB, 0.87434E+00_JPRB ,&
&  0.79893E+00_JPRB, 0.74065E+00_JPRB, 0.67917E+00_JPRB, 0.60936E+00_JPRB, 0.55600E+00_JPRB, 0.52980E+00_JPRB, 0.51422E+00_JPRB ,&
&  0.49534E+00_JPRB, 0.48019E+00_JPRB, 0.46617E+00_JPRB, 0.45498E+00_JPRB, 0.45276E+00_JPRB, 0.46132E+00_JPRB, 0.47081E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47878E+00_JPRB, 0.48768E+00_JPRB, 0.50841E+00_JPRB, 0.52908E+00_JPRB, 0.54566E+00_JPRB, 0.64124E+00_JPRB ,&
&  0.76209E+00_JPRB, 0.75444E+00_JPRB, 0.70549E+00_JPRB, 0.65910E+00_JPRB, 0.61731E+00_JPRB, 0.58322E+00_JPRB, 0.55701E+00_JPRB ,&
&  0.54358E+00_JPRB, 0.53212E+00_JPRB, 0.52476E+00_JPRB, 0.51598E+00_JPRB, 0.50835E+00_JPRB, 0.50468E+00_JPRB, 0.50089E+00_JPRB ,&
&  0.49363E+00_JPRB, 0.48623E+00_JPRB, 0.47686E+00_JPRB, 0.46246E+00_JPRB, 0.44614E+00_JPRB, 0.43700E+00_JPRB, 0.43364E+00_JPRB ,&
&  0.43426E+00_JPRB, 0.45024E+00_JPRB, 0.47258E+00_JPRB, 0.48836E+00_JPRB, 0.51607E+00_JPRB, 0.53674E+00_JPRB, 0.55154E+00_JPRB ,&
&  0.53067E+00_JPRB, 0.52451E+00_JPRB, 0.53451E+00_JPRB, 0.54700E+00_JPRB, 0.57058E+00_JPRB, 0.60481E+00_JPRB, 0.57700E+00_JPRB ,&
&  0.56304E+00_JPRB, 0.56086E+00_JPRB, 0.55189E+00_JPRB, 0.53534E+00_JPRB, 0.53512E+00_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.50562E+00_JPRB, 0.49418E+00_JPRB, 0.48435E+00_JPRB, 0.48023E+00_JPRB, 0.47659E+00_JPRB, 0.46737E+00_JPRB, 0.45870E+00_JPRB ,&
&  0.45013E+00_JPRB, 0.44058E+00_JPRB, 0.42749E+00_JPRB, 0.40785E+00_JPRB, 0.39549E+00_JPRB, 0.39379E+00_JPRB, 0.40034E+00_JPRB ,&
&  0.40197E+00_JPRB, 0.39550E+00_JPRB, 0.39238E+00_JPRB, 0.39233E+00_JPRB, 0.39182E+00_JPRB, 0.38639E+00_JPRB, 0.37896E+00_JPRB ,&
&  0.37253E+00_JPRB, 0.37288E+00_JPRB, 0.37509E+00_JPRB, 0.37948E+00_JPRB, 0.38322E+00_JPRB, 0.38879E+00_JPRB, 0.39702E+00_JPRB ,&
&  0.42346E+00_JPRB, 0.41745E+00_JPRB, 0.83378E+00_JPRB, 0.11400E+01_JPRB, 0.12011E+01_JPRB, 0.11676E+01_JPRB, 0.11319E+01_JPRB ,&
&  0.10528E+01_JPRB, 0.94814E+00_JPRB, 0.83854E+00_JPRB, 0.72864E+00_JPRB, 0.64573E+00_JPRB, 0.59194E+00_JPRB, 0.55415E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.50909E+00_JPRB, 0.49238E+00_JPRB, 0.48518E+00_JPRB, 0.48988E+00_JPRB, 0.49414E+00_JPRB, 0.49605E+00_JPRB ,&
&  0.50150E+00_JPRB, 0.50993E+00_JPRB, 0.52853E+00_JPRB, 0.56099E+00_JPRB, 0.59008E+00_JPRB, 0.66139E+00_JPRB, 0.80113E+00_JPRB ,&
&  0.93150E+00_JPRB, 0.97281E+00_JPRB, 0.87050E+00_JPRB, 0.77641E+00_JPRB, 0.71456E+00_JPRB, 0.67404E+00_JPRB, 0.64336E+00_JPRB ,&
&  0.61766E+00_JPRB, 0.59820E+00_JPRB, 0.58484E+00_JPRB, 0.56402E+00_JPRB, 0.55101E+00_JPRB, 0.54319E+00_JPRB, 0.52756E+00_JPRB ,&
&  0.51083E+00_JPRB, 0.49556E+00_JPRB, 0.47932E+00_JPRB, 0.46445E+00_JPRB, 0.45175E+00_JPRB, 0.45062E+00_JPRB, 0.45449E+00_JPRB ,&
&  0.46310E+00_JPRB, 0.48342E+00_JPRB, 0.49791E+00_JPRB, 0.51676E+00_JPRB, 0.54351E+00_JPRB, 0.56182E+00_JPRB, 0.57063E+00_JPRB ,&
&  0.55412E+00_JPRB, 0.55991E+00_JPRB, 0.57758E+00_JPRB, 0.59861E+00_JPRB, 0.57764E+00_JPRB, 0.64040E+00_JPRB, 0.61981E+00_JPRB ,&
&  0.61443E+00_JPRB, 0.60176E+00_JPRB, 0.56371E+00_JPRB, 0.54550E+00_JPRB, 0.52643E+00_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.51670E+00_JPRB, 0.50153E+00_JPRB, 0.48893E+00_JPRB, 0.47841E+00_JPRB, 0.47043E+00_JPRB, 0.46584E+00_JPRB, 0.45917E+00_JPRB ,&
&  0.45099E+00_JPRB, 0.43924E+00_JPRB, 0.42837E+00_JPRB, 0.41327E+00_JPRB, 0.40732E+00_JPRB, 0.41020E+00_JPRB, 0.41818E+00_JPRB ,&
&  0.41376E+00_JPRB, 0.40176E+00_JPRB, 0.40075E+00_JPRB, 0.40145E+00_JPRB, 0.39828E+00_JPRB, 0.39133E+00_JPRB, 0.38439E+00_JPRB ,&
&  0.38668E+00_JPRB, 0.38897E+00_JPRB, 0.39101E+00_JPRB, 0.39400E+00_JPRB, 0.39765E+00_JPRB, 0.40599E+00_JPRB, 0.42230E+00_JPRB ,&
&  0.47246E+00_JPRB, 0.63490E+00_JPRB, 0.76888E+00_JPRB, 0.13963E+01_JPRB, 0.15889E+01_JPRB, 0.15565E+01_JPRB, 0.13858E+01_JPRB ,&
&  0.12187E+01_JPRB, 0.11327E+01_JPRB, 0.99563E+00_JPRB, 0.86734E+00_JPRB, 0.74542E+00_JPRB, 0.65867E+00_JPRB, 0.60766E+00_JPRB ,&
&  0.57393E+00_JPRB, 0.54814E+00_JPRB, 0.53364E+00_JPRB, 0.53044E+00_JPRB, 0.52762E+00_JPRB, 0.52580E+00_JPRB, 0.52666E+00_JPRB ,&
&  0.53297E+00_JPRB, 0.54815E+00_JPRB, 0.57674E+00_JPRB, 0.62672E+00_JPRB, 0.63978E+00_JPRB, 0.78831E+00_JPRB, 0.99035E+00_JPRB ,&
&  0.10897E+01_JPRB, 0.11576E+01_JPRB, 0.10145E+01_JPRB, 0.88066E+00_JPRB, 0.80421E+00_JPRB, 0.75032E+00_JPRB, 0.70452E+00_JPRB ,&
&  0.67279E+00_JPRB, 0.64792E+00_JPRB, 0.62087E+00_JPRB, 0.58722E+00_JPRB, 0.57068E+00_JPRB, 0.55741E+00_JPRB, 0.53300E+00_JPRB ,&
&  0.50721E+00_JPRB, 0.49056E+00_JPRB, 0.48078E+00_JPRB, 0.47386E+00_JPRB, 0.47092E+00_JPRB, 0.47658E+00_JPRB, 0.48575E+00_JPRB ,&
&  0.49828E+00_JPRB, 0.52405E+00_JPRB, 0.52864E+00_JPRB, 0.53974E+00_JPRB, 0.56444E+00_JPRB, 0.58859E+00_JPRB, 0.57938E+00_JPRB ,&
&  0.57671E+00_JPRB, 0.58557E+00_JPRB, 0.60379E+00_JPRB, 0.61973E+00_JPRB, 0.60605E+00_JPRB, 0.66727E+00_JPRB, 0.64472E+00_JPRB ,&
&  0.64547E+00_JPRB, 0.60547E+00_JPRB, 0.57034E+00_JPRB, 0.55745E+00_JPRB, 0.53984E+00_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.52175E+00_JPRB, 0.49594E+00_JPRB, 0.47842E+00_JPRB, 0.46959E+00_JPRB, 0.46536E+00_JPRB, 0.46454E+00_JPRB, 0.45721E+00_JPRB ,&
&  0.45214E+00_JPRB, 0.44136E+00_JPRB, 0.42757E+00_JPRB, 0.41634E+00_JPRB, 0.42259E+00_JPRB, 0.42691E+00_JPRB, 0.42694E+00_JPRB ,&
&  0.42028E+00_JPRB, 0.41575E+00_JPRB, 0.41368E+00_JPRB, 0.41040E+00_JPRB, 0.40502E+00_JPRB, 0.40019E+00_JPRB, 0.39902E+00_JPRB ,&
&  0.40440E+00_JPRB, 0.40421E+00_JPRB, 0.40353E+00_JPRB, 0.40764E+00_JPRB, 0.41718E+00_JPRB, 0.43865E+00_JPRB, 0.46777E+00_JPRB ,&
&  0.54814E+00_JPRB, 0.63426E+00_JPRB, 0.78058E+00_JPRB, 0.15796E+01_JPRB, 0.19613E+01_JPRB, 0.18918E+01_JPRB, 0.16197E+01_JPRB ,&
&  0.13534E+01_JPRB, 0.13025E+01_JPRB, 0.11456E+01_JPRB, 0.96488E+00_JPRB, 0.80605E+00_JPRB, 0.71660E+00_JPRB, 0.65564E+00_JPRB ,&
&  0.61345E+00_JPRB, 0.59069E+00_JPRB, 0.58350E+00_JPRB, 0.58143E+00_JPRB, 0.57585E+00_JPRB, 0.57763E+00_JPRB, 0.58044E+00_JPRB ,&
&  0.58404E+00_JPRB, 0.59958E+00_JPRB, 0.62691E+00_JPRB, 0.63587E+00_JPRB, 0.72279E+00_JPRB, 0.98538E+00_JPRB, 0.11875E+01_JPRB ,&
&  0.12459E+01_JPRB, 0.12298E+01_JPRB, 0.10923E+01_JPRB, 0.93814E+00_JPRB, 0.84031E+00_JPRB, 0.79630E+00_JPRB, 0.74687E+00_JPRB ,&
&  0.69494E+00_JPRB, 0.66101E+00_JPRB, 0.61763E+00_JPRB, 0.59490E+00_JPRB, 0.58884E+00_JPRB, 0.55200E+00_JPRB, 0.52313E+00_JPRB ,&
&  0.50945E+00_JPRB, 0.50413E+00_JPRB, 0.50103E+00_JPRB, 0.49884E+00_JPRB, 0.50276E+00_JPRB, 0.51073E+00_JPRB, 0.52429E+00_JPRB ,&
&  0.54399E+00_JPRB, 0.57459E+00_JPRB, 0.57020E+00_JPRB, 0.58117E+00_JPRB, 0.61343E+00_JPRB, 0.63545E+00_JPRB, 0.62538E+00_JPRB ,&
&  0.61530E+00_JPRB, 0.61679E+00_JPRB, 0.62447E+00_JPRB, 0.61056E+00_JPRB, 0.60647E+00_JPRB, 0.66317E+00_JPRB, 0.66653E+00_JPRB ,&
&  0.67756E+00_JPRB, 0.62951E+00_JPRB, 0.60391E+00_JPRB, 0.56916E+00_JPRB, 0.54749E+00_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.50771E+00_JPRB, 0.48279E+00_JPRB, 0.47773E+00_JPRB, 0.47447E+00_JPRB, 0.46434E+00_JPRB, 0.45596E+00_JPRB, 0.44677E+00_JPRB ,&
&  0.43983E+00_JPRB, 0.42695E+00_JPRB, 0.41860E+00_JPRB, 0.42593E+00_JPRB, 0.44236E+00_JPRB, 0.45285E+00_JPRB, 0.44526E+00_JPRB ,&
&  0.43644E+00_JPRB, 0.43021E+00_JPRB, 0.43321E+00_JPRB, 0.43385E+00_JPRB, 0.42973E+00_JPRB, 0.43101E+00_JPRB, 0.42925E+00_JPRB ,&
&  0.42996E+00_JPRB, 0.42708E+00_JPRB, 0.42974E+00_JPRB, 0.44207E+00_JPRB, 0.46369E+00_JPRB, 0.50045E+00_JPRB, 0.55246E+00_JPRB ,&
&  0.68359E+00_JPRB, 0.71001E+00_JPRB, 0.82062E+00_JPRB, 0.14413E+01_JPRB, 0.19003E+01_JPRB, 0.17906E+01_JPRB, 0.16317E+01_JPRB ,&
&  0.13982E+01_JPRB, 0.10967E+01_JPRB, 0.10658E+01_JPRB, 0.98093E+00_JPRB, 0.84625E+00_JPRB, 0.74399E+00_JPRB, 0.68328E+00_JPRB ,&
&  0.65075E+00_JPRB, 0.62871E+00_JPRB, 0.62398E+00_JPRB, 0.63032E+00_JPRB, 0.64206E+00_JPRB, 0.65591E+00_JPRB, 0.65972E+00_JPRB ,&
&  0.65942E+00_JPRB, 0.66329E+00_JPRB, 0.68546E+00_JPRB, 0.70890E+00_JPRB, 0.88439E+00_JPRB, 0.12119E+01_JPRB, 0.13484E+01_JPRB ,&
&  0.13377E+01_JPRB, 0.12500E+01_JPRB, 0.10874E+01_JPRB, 0.93370E+00_JPRB, 0.78673E+00_JPRB, 0.79658E+00_JPRB, 0.74223E+00_JPRB ,&
&  0.68413E+00_JPRB, 0.64354E+00_JPRB, 0.62350E+00_JPRB, 0.60221E+00_JPRB, 0.57151E+00_JPRB, 0.54148E+00_JPRB, 0.52578E+00_JPRB ,&
&  0.52703E+00_JPRB, 0.53382E+00_JPRB, 0.53504E+00_JPRB, 0.53147E+00_JPRB, 0.54220E+00_JPRB, 0.54943E+00_JPRB, 0.56840E+00_JPRB ,&
&  0.58911E+00_JPRB, 0.61662E+00_JPRB, 0.62160E+00_JPRB, 0.65006E+00_JPRB, 0.68338E+00_JPRB, 0.70284E+00_JPRB, 0.68948E+00_JPRB ,&
&  0.65539E+00_JPRB, 0.66671E+00_JPRB, 0.67199E+00_JPRB, 0.65502E+00_JPRB, 0.65777E+00_JPRB, 0.66486E+00_JPRB, 0.66922E+00_JPRB ,&
&  0.70090E+00_JPRB, 0.69608E+00_JPRB, 0.66131E+00_JPRB, 0.57394E+00_JPRB, 0.53791E+00_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.49992E+00_JPRB, 0.48267E+00_JPRB, 0.48207E+00_JPRB, 0.47510E+00_JPRB, 0.46345E+00_JPRB, 0.45671E+00_JPRB, 0.44935E+00_JPRB ,&
&  0.44270E+00_JPRB, 0.44305E+00_JPRB, 0.44707E+00_JPRB, 0.47521E+00_JPRB, 0.48757E+00_JPRB, 0.48014E+00_JPRB, 0.47116E+00_JPRB ,&
&  0.46516E+00_JPRB, 0.46505E+00_JPRB, 0.46670E+00_JPRB, 0.46754E+00_JPRB, 0.46676E+00_JPRB, 0.47135E+00_JPRB, 0.47776E+00_JPRB ,&
&  0.47664E+00_JPRB, 0.47892E+00_JPRB, 0.49007E+00_JPRB, 0.51351E+00_JPRB, 0.55731E+00_JPRB, 0.62478E+00_JPRB, 0.72817E+00_JPRB ,&
&  0.91241E+00_JPRB, 0.85377E+00_JPRB, 0.46287E+00_JPRB, 0.91485E+00_JPRB, 0.14930E+01_JPRB, 0.15722E+01_JPRB, 0.14977E+01_JPRB ,&
&  0.13242E+01_JPRB, 0.10153E+01_JPRB, 0.93827E+00_JPRB, 0.95311E+00_JPRB, 0.83150E+00_JPRB, 0.74210E+00_JPRB, 0.69414E+00_JPRB ,&
&  0.66620E+00_JPRB, 0.66494E+00_JPRB, 0.69205E+00_JPRB, 0.71505E+00_JPRB, 0.73692E+00_JPRB, 0.74337E+00_JPRB, 0.73759E+00_JPRB ,&
&  0.73100E+00_JPRB, 0.73881E+00_JPRB, 0.78657E+00_JPRB, 0.83371E+00_JPRB, 0.11453E+01_JPRB, 0.13200E+01_JPRB, 0.13449E+01_JPRB ,&
&  0.11072E+01_JPRB, 0.10579E+01_JPRB, 0.98209E+00_JPRB, 0.88144E+00_JPRB, 0.77505E+00_JPRB, 0.70739E+00_JPRB, 0.71314E+00_JPRB ,&
&  0.65207E+00_JPRB, 0.61149E+00_JPRB, 0.60205E+00_JPRB, 0.58687E+00_JPRB, 0.56807E+00_JPRB, 0.55697E+00_JPRB, 0.55317E+00_JPRB ,&
&  0.56342E+00_JPRB, 0.57352E+00_JPRB, 0.57891E+00_JPRB, 0.57925E+00_JPRB, 0.58438E+00_JPRB, 0.59302E+00_JPRB, 0.61187E+00_JPRB ,&
&  0.62815E+00_JPRB, 0.66111E+00_JPRB, 0.70238E+00_JPRB, 0.70693E+00_JPRB, 0.72566E+00_JPRB, 0.76626E+00_JPRB, 0.75234E+00_JPRB ,&
&  0.72045E+00_JPRB, 0.74034E+00_JPRB, 0.71320E+00_JPRB, 0.66190E+00_JPRB, 0.67617E+00_JPRB, 0.66974E+00_JPRB, 0.68854E+00_JPRB ,&
&  0.75213E+00_JPRB, 0.89012E+00_JPRB, 0.86786E+00_JPRB, 0.56777E+00_JPRB, 0.51772E+00_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.46725E+00_JPRB, 0.45677E+00_JPRB, 0.45110E+00_JPRB, 0.45672E+00_JPRB, 0.45995E+00_JPRB, 0.46457E+00_JPRB, 0.45198E+00_JPRB ,&
&  0.46191E+00_JPRB, 0.49189E+00_JPRB, 0.51704E+00_JPRB, 0.53135E+00_JPRB, 0.52634E+00_JPRB, 0.51671E+00_JPRB, 0.51319E+00_JPRB ,&
&  0.51237E+00_JPRB, 0.51567E+00_JPRB, 0.52667E+00_JPRB, 0.53138E+00_JPRB, 0.53366E+00_JPRB, 0.54066E+00_JPRB, 0.54707E+00_JPRB ,&
&  0.55400E+00_JPRB, 0.56869E+00_JPRB, 0.59516E+00_JPRB, 0.64459E+00_JPRB, 0.72184E+00_JPRB, 0.83354E+00_JPRB, 0.96198E+00_JPRB ,&
&  0.11265E+01_JPRB, 0.78714E+00_JPRB, 0.48758E+00_JPRB, 0.11876E+01_JPRB, 0.15109E+01_JPRB, 0.15058E+01_JPRB, 0.13015E+01_JPRB ,&
&  0.11644E+01_JPRB, 0.97453E+00_JPRB, 0.88107E+00_JPRB, 0.91452E+00_JPRB, 0.79012E+00_JPRB, 0.74295E+00_JPRB, 0.72697E+00_JPRB ,&
&  0.73595E+00_JPRB, 0.76276E+00_JPRB, 0.78868E+00_JPRB, 0.80430E+00_JPRB, 0.80373E+00_JPRB, 0.79126E+00_JPRB, 0.78419E+00_JPRB ,&
&  0.80477E+00_JPRB, 0.84765E+00_JPRB, 0.93753E+00_JPRB, 0.10398E+01_JPRB, 0.13138E+01_JPRB, 0.12329E+01_JPRB, 0.11948E+01_JPRB ,&
&  0.93039E+00_JPRB, 0.90358E+00_JPRB, 0.88632E+00_JPRB, 0.84149E+00_JPRB, 0.74823E+00_JPRB, 0.65224E+00_JPRB, 0.66337E+00_JPRB ,&
&  0.61599E+00_JPRB, 0.59192E+00_JPRB, 0.58897E+00_JPRB, 0.57661E+00_JPRB, 0.56592E+00_JPRB, 0.57077E+00_JPRB, 0.58729E+00_JPRB ,&
&  0.61228E+00_JPRB, 0.62050E+00_JPRB, 0.62203E+00_JPRB, 0.61906E+00_JPRB, 0.62202E+00_JPRB, 0.62981E+00_JPRB, 0.63918E+00_JPRB ,&
&  0.66112E+00_JPRB, 0.69425E+00_JPRB, 0.73026E+00_JPRB, 0.74414E+00_JPRB, 0.74622E+00_JPRB, 0.76793E+00_JPRB, 0.80201E+00_JPRB ,&
&  0.76732E+00_JPRB, 0.77025E+00_JPRB, 0.72686E+00_JPRB, 0.70824E+00_JPRB, 0.67210E+00_JPRB, 0.67029E+00_JPRB, 0.67147E+00_JPRB ,&
&  0.65811E+00_JPRB, 0.60667E+00_JPRB, 0.54637E+00_JPRB, 0.47848E+00_JPRB, 0.46870E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.45894E+00_JPRB, 0.44941E+00_JPRB, 0.44523E+00_JPRB, 0.45994E+00_JPRB, 0.45689E+00_JPRB, 0.46687E+00_JPRB, 0.48061E+00_JPRB ,&
&  0.52040E+00_JPRB, 0.55725E+00_JPRB, 0.58722E+00_JPRB, 0.57928E+00_JPRB, 0.57824E+00_JPRB, 0.56732E+00_JPRB, 0.57222E+00_JPRB ,&
&  0.58209E+00_JPRB, 0.59923E+00_JPRB, 0.61791E+00_JPRB, 0.62480E+00_JPRB, 0.63004E+00_JPRB, 0.63777E+00_JPRB, 0.65036E+00_JPRB ,&
&  0.66942E+00_JPRB, 0.69662E+00_JPRB, 0.74325E+00_JPRB, 0.82415E+00_JPRB, 0.94628E+00_JPRB, 0.10563E+01_JPRB, 0.11300E+01_JPRB ,&
&  0.78876E+00_JPRB, 0.39441E+00_JPRB, 0.11274E+01_JPRB, 0.14996E+01_JPRB, 0.14568E+01_JPRB, 0.13697E+01_JPRB, 0.12731E+01_JPRB ,&
&  0.10530E+01_JPRB, 0.93156E+00_JPRB, 0.84156E+00_JPRB, 0.89618E+00_JPRB, 0.80333E+00_JPRB, 0.78452E+00_JPRB, 0.79621E+00_JPRB ,&
&  0.81842E+00_JPRB, 0.83767E+00_JPRB, 0.85272E+00_JPRB, 0.84797E+00_JPRB, 0.83310E+00_JPRB, 0.82967E+00_JPRB, 0.84576E+00_JPRB ,&
&  0.89135E+00_JPRB, 0.98033E+00_JPRB, 0.10826E+01_JPRB, 0.10647E+01_JPRB, 0.11567E+01_JPRB, 0.10768E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.89236E+00_JPRB, 0.78563E+00_JPRB, 0.75733E+00_JPRB, 0.79575E+00_JPRB, 0.73219E+00_JPRB, 0.61815E+00_JPRB, 0.62186E+00_JPRB ,&
&  0.59204E+00_JPRB, 0.58169E+00_JPRB, 0.58937E+00_JPRB, 0.58207E+00_JPRB, 0.57280E+00_JPRB, 0.58896E+00_JPRB, 0.61148E+00_JPRB ,&
&  0.63641E+00_JPRB, 0.64655E+00_JPRB, 0.63761E+00_JPRB, 0.63235E+00_JPRB, 0.63607E+00_JPRB, 0.63781E+00_JPRB, 0.64110E+00_JPRB ,&
&  0.66599E+00_JPRB, 0.69440E+00_JPRB, 0.72998E+00_JPRB, 0.76754E+00_JPRB, 0.81315E+00_JPRB, 0.81461E+00_JPRB, 0.79652E+00_JPRB ,&
&  0.76353E+00_JPRB, 0.74778E+00_JPRB, 0.71438E+00_JPRB, 0.70227E+00_JPRB, 0.68806E+00_JPRB, 0.66501E+00_JPRB, 0.64216E+00_JPRB ,&
&  0.59717E+00_JPRB, 0.52314E+00_JPRB, 0.48190E+00_JPRB, 0.47547E+00_JPRB, 0.46538E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.48480E+00_JPRB, 0.47587E+00_JPRB, 0.47891E+00_JPRB, 0.48635E+00_JPRB, 0.49896E+00_JPRB, 0.52032E+00_JPRB, 0.55556E+00_JPRB ,&
&  0.59664E+00_JPRB, 0.61750E+00_JPRB, 0.63499E+00_JPRB, 0.62262E+00_JPRB, 0.62319E+00_JPRB, 0.63657E+00_JPRB, 0.64345E+00_JPRB ,&
&  0.66707E+00_JPRB, 0.69260E+00_JPRB, 0.70616E+00_JPRB, 0.70789E+00_JPRB, 0.71412E+00_JPRB, 0.73520E+00_JPRB, 0.76493E+00_JPRB ,&
&  0.80094E+00_JPRB, 0.84877E+00_JPRB, 0.91471E+00_JPRB, 0.10083E+01_JPRB, 0.11157E+01_JPRB, 0.11503E+01_JPRB, 0.11397E+01_JPRB ,&
&  0.61437E+00_JPRB, 0.11385E+01_JPRB, 0.15024E+01_JPRB, 0.14738E+01_JPRB, 0.13815E+01_JPRB, 0.12880E+01_JPRB, 0.11901E+01_JPRB ,&
&  0.10185E+01_JPRB, 0.88730E+00_JPRB, 0.85360E+00_JPRB, 0.90986E+00_JPRB, 0.84535E+00_JPRB, 0.84461E+00_JPRB, 0.85826E+00_JPRB ,&
&  0.87680E+00_JPRB, 0.88967E+00_JPRB, 0.88781E+00_JPRB, 0.88038E+00_JPRB, 0.88808E+00_JPRB, 0.91183E+00_JPRB, 0.94667E+00_JPRB ,&
&  0.99470E+00_JPRB, 0.10710E+01_JPRB, 0.11402E+01_JPRB, 0.10475E+01_JPRB, 0.10424E+01_JPRB, 0.10211E+01_JPRB, 0.10384E+01_JPRB ,&
&  0.85830E+00_JPRB, 0.72410E+00_JPRB, 0.72714E+00_JPRB, 0.76408E+00_JPRB, 0.69797E+00_JPRB, 0.63486E+00_JPRB, 0.60354E+00_JPRB ,&
&  0.58830E+00_JPRB, 0.56817E+00_JPRB, 0.56108E+00_JPRB, 0.55603E+00_JPRB, 0.57190E+00_JPRB, 0.58952E+00_JPRB, 0.61421E+00_JPRB ,&
&  0.62179E+00_JPRB, 0.62050E+00_JPRB, 0.60919E+00_JPRB, 0.60289E+00_JPRB, 0.60830E+00_JPRB, 0.61700E+00_JPRB, 0.61756E+00_JPRB ,&
&  0.63687E+00_JPRB, 0.64580E+00_JPRB, 0.67243E+00_JPRB, 0.71908E+00_JPRB, 0.77917E+00_JPRB, 0.80315E+00_JPRB, 0.79734E+00_JPRB ,&
&  0.74151E+00_JPRB, 0.73519E+00_JPRB, 0.74162E+00_JPRB, 0.72655E+00_JPRB, 0.71358E+00_JPRB, 0.66755E+00_JPRB, 0.57896E+00_JPRB ,&
&  0.53933E+00_JPRB, 0.50239E+00_JPRB, 0.48971E+00_JPRB, 0.48493E+00_JPRB, 0.49645E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.55246E+00_JPRB, 0.53972E+00_JPRB, 0.54612E+00_JPRB, 0.56173E+00_JPRB, 0.57348E+00_JPRB, 0.59828E+00_JPRB, 0.62980E+00_JPRB ,&
&  0.67066E+00_JPRB, 0.69526E+00_JPRB, 0.70289E+00_JPRB, 0.70079E+00_JPRB, 0.70675E+00_JPRB, 0.70493E+00_JPRB, 0.71485E+00_JPRB ,&
&  0.73563E+00_JPRB, 0.75746E+00_JPRB, 0.78533E+00_JPRB, 0.80702E+00_JPRB, 0.81913E+00_JPRB, 0.85569E+00_JPRB, 0.89376E+00_JPRB ,&
&  0.94398E+00_JPRB, 0.99795E+00_JPRB, 0.10683E+01_JPRB, 0.11501E+01_JPRB, 0.12396E+01_JPRB, 0.12763E+01_JPRB, 0.12687E+01_JPRB ,&
&  0.10594E+01_JPRB, 0.15303E+01_JPRB, 0.15408E+01_JPRB, 0.13728E+01_JPRB, 0.12163E+01_JPRB, 0.11495E+01_JPRB, 0.10793E+01_JPRB ,&
&  0.10094E+01_JPRB, 0.90949E+00_JPRB, 0.87691E+00_JPRB, 0.87844E+00_JPRB, 0.89343E+00_JPRB, 0.88785E+00_JPRB, 0.90823E+00_JPRB ,&
&  0.92963E+00_JPRB, 0.93894E+00_JPRB, 0.94157E+00_JPRB, 0.94523E+00_JPRB, 0.95948E+00_JPRB, 0.98850E+00_JPRB, 0.10311E+01_JPRB ,&
&  0.10669E+01_JPRB, 0.11106E+01_JPRB, 0.11680E+01_JPRB, 0.10923E+01_JPRB, 0.10562E+01_JPRB, 0.11243E+01_JPRB, 0.10936E+01_JPRB ,&
&  0.84468E+00_JPRB, 0.67522E+00_JPRB, 0.70301E+00_JPRB, 0.73945E+00_JPRB, 0.68340E+00_JPRB, 0.62545E+00_JPRB, 0.58958E+00_JPRB ,&
&  0.56353E+00_JPRB, 0.53509E+00_JPRB, 0.53841E+00_JPRB, 0.54410E+00_JPRB, 0.55160E+00_JPRB, 0.55552E+00_JPRB, 0.56956E+00_JPRB ,&
&  0.58187E+00_JPRB, 0.57336E+00_JPRB, 0.56220E+00_JPRB, 0.55459E+00_JPRB, 0.56375E+00_JPRB, 0.58123E+00_JPRB, 0.60313E+00_JPRB ,&
&  0.65584E+00_JPRB, 0.62001E+00_JPRB, 0.63627E+00_JPRB, 0.68340E+00_JPRB, 0.73229E+00_JPRB, 0.78893E+00_JPRB, 0.84003E+00_JPRB ,&
&  0.87768E+00_JPRB, 0.91576E+00_JPRB, 0.96989E+00_JPRB, 0.98225E+00_JPRB, 0.91505E+00_JPRB, 0.83146E+00_JPRB, 0.59462E+00_JPRB ,&
&  0.58302E+00_JPRB, 0.57306E+00_JPRB, 0.57696E+00_JPRB, 0.57585E+00_JPRB, 0.57336E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.67055E+00_JPRB, 0.66676E+00_JPRB, 0.68894E+00_JPRB, 0.71096E+00_JPRB, 0.73000E+00_JPRB, 0.74783E+00_JPRB, 0.76029E+00_JPRB ,&
&  0.77847E+00_JPRB, 0.79156E+00_JPRB, 0.79077E+00_JPRB, 0.78276E+00_JPRB, 0.79267E+00_JPRB, 0.78831E+00_JPRB, 0.80625E+00_JPRB ,&
&  0.83360E+00_JPRB, 0.86622E+00_JPRB, 0.90742E+00_JPRB, 0.92807E+00_JPRB, 0.94399E+00_JPRB, 0.97638E+00_JPRB, 0.10040E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.10822E+01_JPRB, 0.11267E+01_JPRB, 0.11811E+01_JPRB, 0.12291E+01_JPRB, 0.12156E+01_JPRB, 0.95985E+00_JPRB ,&
&  0.14031E+01_JPRB, 0.16284E+01_JPRB, 0.15137E+01_JPRB, 0.12729E+01_JPRB, 0.11314E+01_JPRB, 0.10702E+01_JPRB, 0.10479E+01_JPRB ,&
&  0.10195E+01_JPRB, 0.95551E+00_JPRB, 0.89446E+00_JPRB, 0.86981E+00_JPRB, 0.94579E+00_JPRB, 0.92013E+00_JPRB, 0.94279E+00_JPRB ,&
&  0.97105E+00_JPRB, 0.98875E+00_JPRB, 0.10049E+01_JPRB, 0.10184E+01_JPRB, 0.10249E+01_JPRB, 0.10366E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.10714E+01_JPRB, 0.10917E+01_JPRB, 0.11553E+01_JPRB, 0.10563E+01_JPRB, 0.10435E+01_JPRB, 0.11113E+01_JPRB, 0.10894E+01_JPRB ,&
&  0.85797E+00_JPRB, 0.68254E+00_JPRB, 0.69284E+00_JPRB, 0.71881E+00_JPRB, 0.68188E+00_JPRB, 0.61844E+00_JPRB, 0.58911E+00_JPRB ,&
&  0.56667E+00_JPRB, 0.54809E+00_JPRB, 0.54269E+00_JPRB, 0.53165E+00_JPRB, 0.53592E+00_JPRB, 0.54901E+00_JPRB, 0.55328E+00_JPRB ,&
&  0.57176E+00_JPRB, 0.55816E+00_JPRB, 0.55215E+00_JPRB, 0.55234E+00_JPRB, 0.57697E+00_JPRB, 0.62380E+00_JPRB, 0.72908E+00_JPRB ,&
&  0.84792E+00_JPRB, 0.55013E+00_JPRB, 0.61476E+00_JPRB, 0.66841E+00_JPRB, 0.74638E+00_JPRB, 0.87380E+00_JPRB, 0.10722E+01_JPRB ,&
&  0.14098E+01_JPRB, 0.18845E+01_JPRB, 0.18443E+01_JPRB, 0.14655E+01_JPRB, 0.18716E+01_JPRB, 0.15623E+01_JPRB, 0.68027E+00_JPRB ,&
&  0.68400E+00_JPRB, 0.66589E+00_JPRB, 0.67101E+00_JPRB, 0.67630E+00_JPRB, 0.67739E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.75626E+00_JPRB, 0.77277E+00_JPRB, 0.79150E+00_JPRB, 0.80818E+00_JPRB, 0.81703E+00_JPRB, 0.82480E+00_JPRB, 0.83508E+00_JPRB ,&
&  0.83865E+00_JPRB, 0.83688E+00_JPRB, 0.84483E+00_JPRB, 0.85888E+00_JPRB, 0.87723E+00_JPRB, 0.89583E+00_JPRB, 0.92098E+00_JPRB ,&
&  0.95475E+00_JPRB, 0.98478E+00_JPRB, 0.10226E+01_JPRB, 0.10438E+01_JPRB, 0.10504E+01_JPRB, 0.10579E+01_JPRB, 0.10602E+01_JPRB ,&
&  0.10791E+01_JPRB, 0.10896E+01_JPRB, 0.11030E+01_JPRB, 0.11418E+01_JPRB, 0.11672E+01_JPRB, 0.11868E+01_JPRB, 0.99426E+00_JPRB ,&
&  0.13122E+01_JPRB, 0.15457E+01_JPRB, 0.13839E+01_JPRB, 0.11698E+01_JPRB, 0.10587E+01_JPRB, 0.10342E+01_JPRB, 0.10194E+01_JPRB ,&
&  0.10149E+01_JPRB, 0.99161E+00_JPRB, 0.90661E+00_JPRB, 0.92042E+00_JPRB, 0.93474E+00_JPRB, 0.92149E+00_JPRB, 0.94612E+00_JPRB ,&
&  0.97479E+00_JPRB, 0.99407E+00_JPRB, 0.10225E+01_JPRB, 0.10528E+01_JPRB, 0.10641E+01_JPRB, 0.10800E+01_JPRB, 0.10701E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.10714E+01_JPRB, 0.11045E+01_JPRB, 0.10103E+01_JPRB, 0.10015E+01_JPRB, 0.10013E+01_JPRB, 0.88962E+00_JPRB ,&
&  0.78908E+00_JPRB, 0.69901E+00_JPRB, 0.66923E+00_JPRB, 0.74395E+00_JPRB, 0.72263E+00_JPRB, 0.65984E+00_JPRB, 0.61843E+00_JPRB ,&
&  0.58637E+00_JPRB, 0.55652E+00_JPRB, 0.56594E+00_JPRB, 0.55417E+00_JPRB, 0.51548E+00_JPRB, 0.53004E+00_JPRB, 0.54849E+00_JPRB ,&
&  0.53371E+00_JPRB, 0.54358E+00_JPRB, 0.56081E+00_JPRB, 0.55222E+00_JPRB, 0.55303E+00_JPRB, 0.56697E+00_JPRB, 0.54773E+00_JPRB ,&
&  0.60797E+00_JPRB, 0.58748E+00_JPRB, 0.61453E+00_JPRB, 0.67961E+00_JPRB, 0.69034E+00_JPRB, 0.90356E+00_JPRB, 0.12540E+01_JPRB ,&
&  0.18422E+01_JPRB, 0.20176E+01_JPRB, 0.26437E+01_JPRB, 0.41349E+01_JPRB, 0.31321E+01_JPRB, 0.20662E+01_JPRB, 0.77153E+00_JPRB ,&
&  0.76921E+00_JPRB, 0.75937E+00_JPRB, 0.77056E+00_JPRB, 0.76358E+00_JPRB, 0.76133E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.76177E+00_JPRB, 0.78393E+00_JPRB, 0.80153E+00_JPRB, 0.81882E+00_JPRB, 0.83358E+00_JPRB, 0.84699E+00_JPRB, 0.86289E+00_JPRB ,&
&  0.87267E+00_JPRB, 0.88836E+00_JPRB, 0.90798E+00_JPRB, 0.92755E+00_JPRB, 0.94271E+00_JPRB, 0.96661E+00_JPRB, 0.98605E+00_JPRB ,&
&  0.10152E+01_JPRB, 0.10337E+01_JPRB, 0.10486E+01_JPRB, 0.10462E+01_JPRB, 0.10387E+01_JPRB, 0.10416E+01_JPRB, 0.10401E+01_JPRB ,&
&  0.10290E+01_JPRB, 0.10265E+01_JPRB, 0.10333E+01_JPRB, 0.10513E+01_JPRB, 0.10571E+01_JPRB, 0.10879E+01_JPRB, 0.92120E+00_JPRB ,&
&  0.12062E+01_JPRB, 0.13414E+01_JPRB, 0.12356E+01_JPRB, 0.10745E+01_JPRB, 0.98451E+00_JPRB, 0.96966E+00_JPRB, 0.96453E+00_JPRB ,&
&  0.96971E+00_JPRB, 0.99366E+00_JPRB, 0.90926E+00_JPRB, 0.86738E+00_JPRB, 0.85470E+00_JPRB, 0.85343E+00_JPRB, 0.87016E+00_JPRB ,&
&  0.89181E+00_JPRB, 0.91646E+00_JPRB, 0.94679E+00_JPRB, 0.97322E+00_JPRB, 0.98289E+00_JPRB, 0.98183E+00_JPRB, 0.96190E+00_JPRB ,&
&  0.92935E+00_JPRB, 0.90628E+00_JPRB, 0.85646E+00_JPRB, 0.85052E+00_JPRB, 0.84095E+00_JPRB, 0.74850E+00_JPRB, 0.63979E+00_JPRB ,&
&  0.59934E+00_JPRB, 0.66760E+00_JPRB, 0.71272E+00_JPRB, 0.83789E+00_JPRB, 0.79137E+00_JPRB, 0.72803E+00_JPRB, 0.67362E+00_JPRB ,&
&  0.62196E+00_JPRB, 0.58079E+00_JPRB, 0.57375E+00_JPRB, 0.54900E+00_JPRB, 0.53502E+00_JPRB, 0.52979E+00_JPRB, 0.52274E+00_JPRB ,&
&  0.53050E+00_JPRB, 0.53575E+00_JPRB, 0.54653E+00_JPRB, 0.54817E+00_JPRB, 0.53397E+00_JPRB, 0.47254E+00_JPRB, 0.57353E+00_JPRB ,&
&  0.56896E+00_JPRB, 0.52595E+00_JPRB, 0.62182E+00_JPRB, 0.50104E+00_JPRB, 0.61631E+00_JPRB, 0.68177E+00_JPRB, 0.81270E+00_JPRB ,&
&  0.90336E+00_JPRB, 0.12765E+01_JPRB, 0.17679E+01_JPRB, 0.14003E+01_JPRB, 0.11127E+01_JPRB, 0.82760E+00_JPRB, 0.73356E+00_JPRB ,&
&  0.72825E+00_JPRB, 0.72879E+00_JPRB, 0.73478E+00_JPRB, 0.73465E+00_JPRB, 0.74205E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.76542E+00_JPRB, 0.78474E+00_JPRB, 0.79906E+00_JPRB, 0.81599E+00_JPRB, 0.83314E+00_JPRB, 0.84902E+00_JPRB, 0.86454E+00_JPRB ,&
&  0.87742E+00_JPRB, 0.88802E+00_JPRB, 0.89681E+00_JPRB, 0.90999E+00_JPRB, 0.92720E+00_JPRB, 0.94106E+00_JPRB, 0.95892E+00_JPRB ,&
&  0.98668E+00_JPRB, 0.10078E+01_JPRB, 0.10295E+01_JPRB, 0.10234E+01_JPRB, 0.10194E+01_JPRB, 0.10339E+01_JPRB, 0.10490E+01_JPRB ,&
&  0.10537E+01_JPRB, 0.10662E+01_JPRB, 0.10882E+01_JPRB, 0.11127E+01_JPRB, 0.11432E+01_JPRB, 0.11927E+01_JPRB, 0.11010E+01_JPRB ,&
&  0.11305E+01_JPRB, 0.12013E+01_JPRB, 0.11206E+01_JPRB, 0.97141E+00_JPRB, 0.90184E+00_JPRB, 0.87002E+00_JPRB, 0.85584E+00_JPRB ,&
&  0.84601E+00_JPRB, 0.81167E+00_JPRB, 0.78564E+00_JPRB, 0.75940E+00_JPRB, 0.74912E+00_JPRB, 0.75171E+00_JPRB, 0.75652E+00_JPRB ,&
&  0.76313E+00_JPRB, 0.77117E+00_JPRB, 0.79062E+00_JPRB, 0.81356E+00_JPRB, 0.82958E+00_JPRB, 0.81090E+00_JPRB, 0.78433E+00_JPRB ,&
&  0.74296E+00_JPRB, 0.70694E+00_JPRB, 0.59440E+00_JPRB, 0.56345E+00_JPRB, 0.54270E+00_JPRB, 0.49780E+00_JPRB, 0.44972E+00_JPRB ,&
&  0.49848E+00_JPRB, 0.58240E+00_JPRB, 0.72525E+00_JPRB, 0.88358E+00_JPRB, 0.82199E+00_JPRB, 0.76797E+00_JPRB, 0.71002E+00_JPRB ,&
&  0.64635E+00_JPRB, 0.59863E+00_JPRB, 0.57683E+00_JPRB, 0.54774E+00_JPRB, 0.53908E+00_JPRB, 0.54584E+00_JPRB, 0.52194E+00_JPRB ,&
&  0.52005E+00_JPRB, 0.51182E+00_JPRB, 0.50953E+00_JPRB, 0.50350E+00_JPRB, 0.51223E+00_JPRB, 0.44432E+00_JPRB, 0.53077E+00_JPRB ,&
&  0.54213E+00_JPRB, 0.43251E+00_JPRB, 0.48132E+00_JPRB, 0.57707E+00_JPRB, 0.57120E+00_JPRB, 0.59045E+00_JPRB, 0.67340E+00_JPRB ,&
&  0.82753E+00_JPRB, 0.93754E+00_JPRB, 0.95322E+00_JPRB, 0.89354E+00_JPRB, 0.79673E+00_JPRB, 0.73932E+00_JPRB, 0.71000E+00_JPRB ,&
&  0.71727E+00_JPRB, 0.72451E+00_JPRB, 0.73428E+00_JPRB, 0.73577E+00_JPRB, 0.74453E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.75024E+00_JPRB, 0.77117E+00_JPRB, 0.78315E+00_JPRB, 0.79536E+00_JPRB, 0.81115E+00_JPRB, 0.81838E+00_JPRB, 0.82443E+00_JPRB ,&
&  0.82600E+00_JPRB, 0.82241E+00_JPRB, 0.82833E+00_JPRB, 0.84088E+00_JPRB, 0.84693E+00_JPRB, 0.84491E+00_JPRB, 0.85024E+00_JPRB ,&
&  0.86208E+00_JPRB, 0.88136E+00_JPRB, 0.90102E+00_JPRB, 0.89273E+00_JPRB, 0.88588E+00_JPRB, 0.89499E+00_JPRB, 0.90917E+00_JPRB ,&
&  0.90825E+00_JPRB, 0.90448E+00_JPRB, 0.91089E+00_JPRB, 0.91882E+00_JPRB, 0.93462E+00_JPRB, 0.93684E+00_JPRB, 0.10350E+01_JPRB ,&
&  0.84216E+00_JPRB, 0.10855E+01_JPRB, 0.10305E+01_JPRB, 0.87552E+00_JPRB, 0.84500E+00_JPRB, 0.77975E+00_JPRB, 0.75774E+00_JPRB ,&
&  0.74099E+00_JPRB, 0.71536E+00_JPRB, 0.68811E+00_JPRB, 0.66474E+00_JPRB, 0.65756E+00_JPRB, 0.66228E+00_JPRB, 0.65677E+00_JPRB ,&
&  0.65558E+00_JPRB, 0.66561E+00_JPRB, 0.67607E+00_JPRB, 0.68484E+00_JPRB, 0.68377E+00_JPRB, 0.67070E+00_JPRB, 0.64216E+00_JPRB ,&
&  0.61893E+00_JPRB, 0.58198E+00_JPRB, 0.44880E+00_JPRB, 0.42282E+00_JPRB, 0.42498E+00_JPRB, 0.38627E+00_JPRB, 0.37280E+00_JPRB ,&
&  0.46426E+00_JPRB, 0.56374E+00_JPRB, 0.77956E+00_JPRB, 0.86512E+00_JPRB, 0.84527E+00_JPRB, 0.85636E+00_JPRB, 0.77166E+00_JPRB ,&
&  0.68650E+00_JPRB, 0.62315E+00_JPRB, 0.60476E+00_JPRB, 0.56631E+00_JPRB, 0.55270E+00_JPRB, 0.54008E+00_JPRB, 0.52069E+00_JPRB ,&
&  0.51083E+00_JPRB, 0.49434E+00_JPRB, 0.48142E+00_JPRB, 0.47922E+00_JPRB, 0.40292E+00_JPRB, 0.43857E+00_JPRB, 0.51041E+00_JPRB ,&
&  0.50287E+00_JPRB, 0.45023E+00_JPRB, 0.41146E+00_JPRB, 0.52363E+00_JPRB, 0.51820E+00_JPRB, 0.52688E+00_JPRB, 0.58991E+00_JPRB ,&
&  0.70253E+00_JPRB, 0.74099E+00_JPRB, 0.74079E+00_JPRB, 0.72707E+00_JPRB, 0.70162E+00_JPRB, 0.67875E+00_JPRB, 0.66927E+00_JPRB ,&
&  0.68307E+00_JPRB, 0.69628E+00_JPRB, 0.71126E+00_JPRB, 0.71829E+00_JPRB, 0.72972E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.72842E+00_JPRB, 0.74133E+00_JPRB, 0.74087E+00_JPRB, 0.75482E+00_JPRB, 0.77210E+00_JPRB, 0.77569E+00_JPRB, 0.77733E+00_JPRB ,&
&  0.76778E+00_JPRB, 0.77032E+00_JPRB, 0.76577E+00_JPRB, 0.78136E+00_JPRB, 0.78148E+00_JPRB, 0.78312E+00_JPRB, 0.78521E+00_JPRB ,&
&  0.79646E+00_JPRB, 0.80486E+00_JPRB, 0.83120E+00_JPRB, 0.83590E+00_JPRB, 0.83286E+00_JPRB, 0.83852E+00_JPRB, 0.85594E+00_JPRB ,&
&  0.87671E+00_JPRB, 0.90265E+00_JPRB, 0.93665E+00_JPRB, 0.96451E+00_JPRB, 0.10601E+01_JPRB, 0.12099E+01_JPRB, 0.14674E+01_JPRB ,&
&  0.11292E+01_JPRB, 0.93532E+00_JPRB, 0.91873E+00_JPRB, 0.78759E+00_JPRB, 0.75868E+00_JPRB, 0.74734E+00_JPRB, 0.70700E+00_JPRB ,&
&  0.66129E+00_JPRB, 0.63582E+00_JPRB, 0.61436E+00_JPRB, 0.60344E+00_JPRB, 0.60051E+00_JPRB, 0.59133E+00_JPRB, 0.59483E+00_JPRB ,&
&  0.58712E+00_JPRB, 0.58509E+00_JPRB, 0.63959E+00_JPRB, 0.67589E+00_JPRB, 0.67743E+00_JPRB, 0.72579E+00_JPRB, 0.68511E+00_JPRB ,&
&  0.67712E+00_JPRB, 0.63218E+00_JPRB, 0.51231E+00_JPRB, 0.47233E+00_JPRB, 0.45607E+00_JPRB, 0.43951E+00_JPRB, 0.43155E+00_JPRB ,&
&  0.48540E+00_JPRB, 0.56913E+00_JPRB, 0.59586E+00_JPRB, 0.79147E+00_JPRB, 0.87256E+00_JPRB, 0.93549E+00_JPRB, 0.79737E+00_JPRB ,&
&  0.68939E+00_JPRB, 0.62579E+00_JPRB, 0.61022E+00_JPRB, 0.57673E+00_JPRB, 0.56556E+00_JPRB, 0.53299E+00_JPRB, 0.50192E+00_JPRB ,&
&  0.49784E+00_JPRB, 0.47832E+00_JPRB, 0.45940E+00_JPRB, 0.43497E+00_JPRB, 0.43944E+00_JPRB, 0.38024E+00_JPRB, 0.43673E+00_JPRB ,&
&  0.43786E+00_JPRB, 0.47055E+00_JPRB, 0.43185E+00_JPRB, 0.50671E+00_JPRB, 0.47025E+00_JPRB, 0.46563E+00_JPRB, 0.50658E+00_JPRB ,&
&  0.57487E+00_JPRB, 0.60984E+00_JPRB, 0.62274E+00_JPRB, 0.64761E+00_JPRB, 0.65286E+00_JPRB, 0.64414E+00_JPRB, 0.65045E+00_JPRB ,&
&  0.68498E+00_JPRB, 0.68984E+00_JPRB, 0.69480E+00_JPRB, 0.69552E+00_JPRB, 0.71171E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.67201E+00_JPRB, 0.68430E+00_JPRB, 0.67845E+00_JPRB, 0.68485E+00_JPRB, 0.68679E+00_JPRB, 0.69915E+00_JPRB, 0.69734E+00_JPRB ,&
&  0.69344E+00_JPRB, 0.70001E+00_JPRB, 0.68066E+00_JPRB, 0.68447E+00_JPRB, 0.67917E+00_JPRB, 0.68263E+00_JPRB, 0.67308E+00_JPRB ,&
&  0.67423E+00_JPRB, 0.68718E+00_JPRB, 0.69365E+00_JPRB, 0.70271E+00_JPRB, 0.70139E+00_JPRB, 0.68534E+00_JPRB, 0.66467E+00_JPRB ,&
&  0.69587E+00_JPRB, 0.78278E+00_JPRB, 0.87928E+00_JPRB, 0.94183E+00_JPRB, 0.10909E+01_JPRB, 0.10487E+01_JPRB, 0.11160E+01_JPRB ,&
&  0.96068E+00_JPRB, 0.85504E+00_JPRB, 0.94198E+00_JPRB, 0.83727E+00_JPRB, 0.80401E+00_JPRB, 0.76009E+00_JPRB, 0.72805E+00_JPRB ,&
&  0.72469E+00_JPRB, 0.73547E+00_JPRB, 0.75448E+00_JPRB, 0.77955E+00_JPRB, 0.80096E+00_JPRB, 0.79574E+00_JPRB, 0.77623E+00_JPRB ,&
&  0.76512E+00_JPRB, 0.79550E+00_JPRB, 0.87652E+00_JPRB, 0.86026E+00_JPRB, 0.86862E+00_JPRB, 0.89055E+00_JPRB, 0.88056E+00_JPRB ,&
&  0.85948E+00_JPRB, 0.82191E+00_JPRB, 0.69871E+00_JPRB, 0.63118E+00_JPRB, 0.63373E+00_JPRB, 0.59830E+00_JPRB, 0.60155E+00_JPRB ,&
&  0.64834E+00_JPRB, 0.66710E+00_JPRB, 0.55772E+00_JPRB, 0.67112E+00_JPRB, 0.78416E+00_JPRB, 0.81411E+00_JPRB, 0.80346E+00_JPRB ,&
&  0.70647E+00_JPRB, 0.65149E+00_JPRB, 0.62475E+00_JPRB, 0.60690E+00_JPRB, 0.58220E+00_JPRB, 0.54481E+00_JPRB, 0.52325E+00_JPRB ,&
&  0.49320E+00_JPRB, 0.46797E+00_JPRB, 0.43710E+00_JPRB, 0.41167E+00_JPRB, 0.40049E+00_JPRB, 0.42902E+00_JPRB, 0.40818E+00_JPRB ,&
&  0.42108E+00_JPRB, 0.45018E+00_JPRB, 0.53163E+00_JPRB, 0.49562E+00_JPRB, 0.37832E+00_JPRB, 0.42110E+00_JPRB, 0.43611E+00_JPRB ,&
&  0.46029E+00_JPRB, 0.49231E+00_JPRB, 0.51585E+00_JPRB, 0.54611E+00_JPRB, 0.56077E+00_JPRB, 0.56075E+00_JPRB, 0.56865E+00_JPRB ,&
&  0.59209E+00_JPRB, 0.61587E+00_JPRB, 0.64958E+00_JPRB, 0.65458E+00_JPRB, 0.65650E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.57681E+00_JPRB, 0.57904E+00_JPRB, 0.58245E+00_JPRB, 0.57899E+00_JPRB, 0.57527E+00_JPRB, 0.57395E+00_JPRB, 0.57528E+00_JPRB ,&
&  0.57481E+00_JPRB, 0.56591E+00_JPRB, 0.55739E+00_JPRB, 0.55051E+00_JPRB, 0.53754E+00_JPRB, 0.51293E+00_JPRB, 0.47836E+00_JPRB ,&
&  0.48341E+00_JPRB, 0.49213E+00_JPRB, 0.50677E+00_JPRB, 0.54300E+00_JPRB, 0.61963E+00_JPRB, 0.73378E+00_JPRB, 0.87907E+00_JPRB ,&
&  0.11601E+01_JPRB, 0.14659E+01_JPRB, 0.16660E+01_JPRB, 0.17126E+01_JPRB, 0.18632E+01_JPRB, 0.12256E+01_JPRB, 0.94683E+00_JPRB ,&
&  0.97915E+00_JPRB, 0.92662E+00_JPRB, 0.90932E+00_JPRB, 0.87041E+00_JPRB, 0.84550E+00_JPRB, 0.82895E+00_JPRB, 0.85686E+00_JPRB ,&
&  0.91628E+00_JPRB, 0.97002E+00_JPRB, 0.10163E+01_JPRB, 0.10606E+01_JPRB, 0.10919E+01_JPRB, 0.11218E+01_JPRB, 0.11173E+01_JPRB ,&
&  0.11041E+01_JPRB, 0.10922E+01_JPRB, 0.11106E+01_JPRB, 0.10358E+01_JPRB, 0.10468E+01_JPRB, 0.10514E+01_JPRB, 0.10371E+01_JPRB ,&
&  0.99266E+00_JPRB, 0.92361E+00_JPRB, 0.88313E+00_JPRB, 0.82837E+00_JPRB, 0.79955E+00_JPRB, 0.76792E+00_JPRB, 0.74099E+00_JPRB ,&
&  0.73358E+00_JPRB, 0.71525E+00_JPRB, 0.60251E+00_JPRB, 0.70031E+00_JPRB, 0.76844E+00_JPRB, 0.69544E+00_JPRB, 0.77880E+00_JPRB ,&
&  0.70838E+00_JPRB, 0.68089E+00_JPRB, 0.64317E+00_JPRB, 0.61417E+00_JPRB, 0.59498E+00_JPRB, 0.61442E+00_JPRB, 0.52078E+00_JPRB ,&
&  0.50486E+00_JPRB, 0.47162E+00_JPRB, 0.41911E+00_JPRB, 0.40511E+00_JPRB, 0.40488E+00_JPRB, 0.42783E+00_JPRB, 0.43152E+00_JPRB ,&
&  0.44420E+00_JPRB, 0.46623E+00_JPRB, 0.55873E+00_JPRB, 0.60900E+00_JPRB, 0.47806E+00_JPRB, 0.39410E+00_JPRB, 0.40049E+00_JPRB ,&
&  0.40339E+00_JPRB, 0.40985E+00_JPRB, 0.41835E+00_JPRB, 0.44027E+00_JPRB, 0.46083E+00_JPRB, 0.47731E+00_JPRB, 0.49010E+00_JPRB ,&
&  0.49791E+00_JPRB, 0.51883E+00_JPRB, 0.53936E+00_JPRB, 0.55654E+00_JPRB, 0.57230E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.48949E+00_JPRB, 0.49115E+00_JPRB, 0.50034E+00_JPRB, 0.50036E+00_JPRB, 0.51290E+00_JPRB, 0.51217E+00_JPRB, 0.51231E+00_JPRB ,&
&  0.51899E+00_JPRB, 0.53258E+00_JPRB, 0.52308E+00_JPRB, 0.50087E+00_JPRB, 0.49580E+00_JPRB, 0.50707E+00_JPRB, 0.54067E+00_JPRB ,&
&  0.61419E+00_JPRB, 0.68615E+00_JPRB, 0.75106E+00_JPRB, 0.85608E+00_JPRB, 0.10520E+01_JPRB, 0.13698E+01_JPRB, 0.16745E+01_JPRB ,&
&  0.18843E+01_JPRB, 0.20112E+01_JPRB, 0.19839E+01_JPRB, 0.20447E+01_JPRB, 0.16028E+01_JPRB, 0.94331E+00_JPRB, 0.10067E+01_JPRB ,&
&  0.10625E+01_JPRB, 0.10894E+01_JPRB, 0.10210E+01_JPRB, 0.96599E+00_JPRB, 0.92876E+00_JPRB, 0.94978E+00_JPRB, 0.10027E+01_JPRB ,&
&  0.10628E+01_JPRB, 0.11255E+01_JPRB, 0.11740E+01_JPRB, 0.12064E+01_JPRB, 0.12192E+01_JPRB, 0.12373E+01_JPRB, 0.12414E+01_JPRB ,&
&  0.12659E+01_JPRB, 0.12616E+01_JPRB, 0.12466E+01_JPRB, 0.12057E+01_JPRB, 0.11765E+01_JPRB, 0.11853E+01_JPRB, 0.11747E+01_JPRB ,&
&  0.11418E+01_JPRB, 0.10555E+01_JPRB, 0.10139E+01_JPRB, 0.97125E+00_JPRB, 0.89548E+00_JPRB, 0.81709E+00_JPRB, 0.79380E+00_JPRB ,&
&  0.77981E+00_JPRB, 0.78816E+00_JPRB, 0.65753E+00_JPRB, 0.79851E+00_JPRB, 0.89217E+00_JPRB, 0.81924E+00_JPRB, 0.78072E+00_JPRB ,&
&  0.71461E+00_JPRB, 0.69094E+00_JPRB, 0.63323E+00_JPRB, 0.62259E+00_JPRB, 0.63301E+00_JPRB, 0.68745E+00_JPRB, 0.52870E+00_JPRB ,&
&  0.55827E+00_JPRB, 0.48043E+00_JPRB, 0.46751E+00_JPRB, 0.46391E+00_JPRB, 0.46039E+00_JPRB, 0.43693E+00_JPRB, 0.43103E+00_JPRB ,&
&  0.43113E+00_JPRB, 0.54575E+00_JPRB, 0.72810E+00_JPRB, 0.11308E+01_JPRB, 0.15440E+01_JPRB, 0.41568E+00_JPRB, 0.39361E+00_JPRB ,&
&  0.40021E+00_JPRB, 0.40067E+00_JPRB, 0.39819E+00_JPRB, 0.40979E+00_JPRB, 0.42215E+00_JPRB, 0.43244E+00_JPRB, 0.44850E+00_JPRB ,&
&  0.45908E+00_JPRB, 0.46606E+00_JPRB, 0.47357E+00_JPRB, 0.47284E+00_JPRB, 0.48213E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.45524E+00_JPRB, 0.46464E+00_JPRB, 0.49201E+00_JPRB, 0.54151E+00_JPRB, 0.60821E+00_JPRB, 0.63962E+00_JPRB, 0.63116E+00_JPRB ,&
&  0.59944E+00_JPRB, 0.60042E+00_JPRB, 0.62286E+00_JPRB, 0.63052E+00_JPRB, 0.67214E+00_JPRB, 0.75635E+00_JPRB, 0.84078E+00_JPRB ,&
&  0.95582E+00_JPRB, 0.10589E+01_JPRB, 0.11567E+01_JPRB, 0.13096E+01_JPRB, 0.14563E+01_JPRB, 0.16269E+01_JPRB, 0.17246E+01_JPRB ,&
&  0.16985E+01_JPRB, 0.14863E+01_JPRB, 0.13788E+01_JPRB, 0.10581E+01_JPRB, 0.86700E+00_JPRB, 0.97819E+00_JPRB, 0.97682E+00_JPRB ,&
&  0.98164E+00_JPRB, 0.10058E+01_JPRB, 0.10019E+01_JPRB, 0.10169E+01_JPRB, 0.10510E+01_JPRB, 0.10831E+01_JPRB, 0.11334E+01_JPRB ,&
&  0.12015E+01_JPRB, 0.12654E+01_JPRB, 0.13142E+01_JPRB, 0.13489E+01_JPRB, 0.13431E+01_JPRB, 0.13465E+01_JPRB, 0.13770E+01_JPRB ,&
&  0.14245E+01_JPRB, 0.14175E+01_JPRB, 0.14022E+01_JPRB, 0.13774E+01_JPRB, 0.13038E+01_JPRB, 0.12861E+01_JPRB, 0.12613E+01_JPRB ,&
&  0.12377E+01_JPRB, 0.11953E+01_JPRB, 0.11599E+01_JPRB, 0.11331E+01_JPRB, 0.10711E+01_JPRB, 0.92426E+00_JPRB, 0.86847E+00_JPRB ,&
&  0.84969E+00_JPRB, 0.81166E+00_JPRB, 0.75085E+00_JPRB, 0.85402E+00_JPRB, 0.63971E+00_JPRB, 0.73844E+00_JPRB, 0.81988E+00_JPRB ,&
&  0.79001E+00_JPRB, 0.75249E+00_JPRB, 0.68684E+00_JPRB, 0.66455E+00_JPRB, 0.65295E+00_JPRB, 0.57285E+00_JPRB, 0.58096E+00_JPRB ,&
&  0.62421E+00_JPRB, 0.56902E+00_JPRB, 0.54430E+00_JPRB, 0.53329E+00_JPRB, 0.54125E+00_JPRB, 0.50288E+00_JPRB, 0.50041E+00_JPRB ,&
&  0.49257E+00_JPRB, 0.58467E+00_JPRB, 0.59581E+00_JPRB, 0.72145E+00_JPRB, 0.74931E+00_JPRB, 0.40122E+00_JPRB, 0.40260E+00_JPRB ,&
&  0.41379E+00_JPRB, 0.42329E+00_JPRB, 0.42582E+00_JPRB, 0.44807E+00_JPRB, 0.46119E+00_JPRB, 0.45891E+00_JPRB, 0.46508E+00_JPRB ,&
&  0.47180E+00_JPRB, 0.47086E+00_JPRB, 0.46883E+00_JPRB, 0.46724E+00_JPRB, 0.46442E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.50398E+00_JPRB, 0.50005E+00_JPRB, 0.52453E+00_JPRB, 0.62041E+00_JPRB, 0.71824E+00_JPRB, 0.77662E+00_JPRB, 0.75281E+00_JPRB ,&
&  0.70209E+00_JPRB, 0.65579E+00_JPRB, 0.69136E+00_JPRB, 0.73632E+00_JPRB, 0.80959E+00_JPRB, 0.89805E+00_JPRB, 0.99437E+00_JPRB ,&
&  0.11098E+01_JPRB, 0.12039E+01_JPRB, 0.12740E+01_JPRB, 0.13514E+01_JPRB, 0.14512E+01_JPRB, 0.15527E+01_JPRB, 0.16031E+01_JPRB ,&
&  0.13054E+01_JPRB, 0.10047E+01_JPRB, 0.12055E+01_JPRB, 0.10939E+01_JPRB, 0.10526E+01_JPRB, 0.10459E+01_JPRB, 0.10599E+01_JPRB ,&
&  0.10537E+01_JPRB, 0.10670E+01_JPRB, 0.11020E+01_JPRB, 0.11365E+01_JPRB, 0.11343E+01_JPRB, 0.11750E+01_JPRB, 0.12396E+01_JPRB ,&
&  0.13219E+01_JPRB, 0.13796E+01_JPRB, 0.14057E+01_JPRB, 0.14245E+01_JPRB, 0.14278E+01_JPRB, 0.14411E+01_JPRB, 0.14697E+01_JPRB ,&
&  0.14924E+01_JPRB, 0.15056E+01_JPRB, 0.14780E+01_JPRB, 0.14179E+01_JPRB, 0.13815E+01_JPRB, 0.13628E+01_JPRB, 0.13241E+01_JPRB ,&
&  0.13062E+01_JPRB, 0.12771E+01_JPRB, 0.13022E+01_JPRB, 0.13221E+01_JPRB, 0.13004E+01_JPRB, 0.11122E+01_JPRB, 0.10512E+01_JPRB ,&
&  0.97107E+00_JPRB, 0.89019E+00_JPRB, 0.85674E+00_JPRB, 0.91999E+00_JPRB, 0.71784E+00_JPRB, 0.77391E+00_JPRB, 0.78391E+00_JPRB ,&
&  0.84648E+00_JPRB, 0.81694E+00_JPRB, 0.78141E+00_JPRB, 0.73580E+00_JPRB, 0.73942E+00_JPRB, 0.63945E+00_JPRB, 0.61936E+00_JPRB ,&
&  0.61572E+00_JPRB, 0.65513E+00_JPRB, 0.60181E+00_JPRB, 0.58191E+00_JPRB, 0.54190E+00_JPRB, 0.56437E+00_JPRB, 0.60197E+00_JPRB ,&
&  0.69586E+00_JPRB, 0.67467E+00_JPRB, 0.63615E+00_JPRB, 0.59103E+00_JPRB, 0.48816E+00_JPRB, 0.42092E+00_JPRB, 0.42779E+00_JPRB ,&
&  0.44490E+00_JPRB, 0.46165E+00_JPRB, 0.46842E+00_JPRB, 0.48766E+00_JPRB, 0.50230E+00_JPRB, 0.49852E+00_JPRB, 0.49276E+00_JPRB ,&
&  0.49445E+00_JPRB, 0.50963E+00_JPRB, 0.50364E+00_JPRB, 0.50486E+00_JPRB, 0.50914E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.57615E+00_JPRB, 0.57541E+00_JPRB, 0.59625E+00_JPRB, 0.68451E+00_JPRB, 0.80903E+00_JPRB, 0.91619E+00_JPRB, 0.94018E+00_JPRB ,&
&  0.93988E+00_JPRB, 0.76544E+00_JPRB, 0.79209E+00_JPRB, 0.80879E+00_JPRB, 0.89338E+00_JPRB, 0.98801E+00_JPRB, 0.10873E+01_JPRB ,&
&  0.11840E+01_JPRB, 0.12865E+01_JPRB, 0.13570E+01_JPRB, 0.14181E+01_JPRB, 0.14819E+01_JPRB, 0.15402E+01_JPRB, 0.13194E+01_JPRB ,&
&  0.10143E+01_JPRB, 0.11961E+01_JPRB, 0.17822E+01_JPRB, 0.13466E+01_JPRB, 0.11255E+01_JPRB, 0.11208E+01_JPRB, 0.11572E+01_JPRB ,&
&  0.11910E+01_JPRB, 0.11715E+01_JPRB, 0.12032E+01_JPRB, 0.12541E+01_JPRB, 0.12582E+01_JPRB, 0.13118E+01_JPRB, 0.13903E+01_JPRB ,&
&  0.14320E+01_JPRB, 0.14735E+01_JPRB, 0.14805E+01_JPRB, 0.14700E+01_JPRB, 0.14666E+01_JPRB, 0.14804E+01_JPRB, 0.15078E+01_JPRB ,&
&  0.15289E+01_JPRB, 0.15379E+01_JPRB, 0.15116E+01_JPRB, 0.14315E+01_JPRB, 0.14164E+01_JPRB, 0.14284E+01_JPRB, 0.14213E+01_JPRB ,&
&  0.14045E+01_JPRB, 0.14584E+01_JPRB, 0.14699E+01_JPRB, 0.14003E+01_JPRB, 0.12768E+01_JPRB, 0.13562E+01_JPRB, 0.11833E+01_JPRB ,&
&  0.11068E+01_JPRB, 0.97235E+00_JPRB, 0.98418E+00_JPRB, 0.80589E+00_JPRB, 0.82126E+00_JPRB, 0.85203E+00_JPRB, 0.84381E+00_JPRB ,&
&  0.83091E+00_JPRB, 0.84204E+00_JPRB, 0.82436E+00_JPRB, 0.80540E+00_JPRB, 0.79237E+00_JPRB, 0.70293E+00_JPRB, 0.68618E+00_JPRB ,&
&  0.65411E+00_JPRB, 0.70495E+00_JPRB, 0.60464E+00_JPRB, 0.54546E+00_JPRB, 0.48802E+00_JPRB, 0.51705E+00_JPRB, 0.59794E+00_JPRB ,&
&  0.72785E+00_JPRB, 0.78873E+00_JPRB, 0.77276E+00_JPRB, 0.71052E+00_JPRB, 0.59930E+00_JPRB, 0.51603E+00_JPRB, 0.47103E+00_JPRB ,&
&  0.46992E+00_JPRB, 0.47774E+00_JPRB, 0.49457E+00_JPRB, 0.51062E+00_JPRB, 0.52722E+00_JPRB, 0.54133E+00_JPRB, 0.54905E+00_JPRB ,&
&  0.55256E+00_JPRB, 0.55269E+00_JPRB, 0.55005E+00_JPRB, 0.57372E+00_JPRB, 0.58807E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.62523E+00_JPRB, 0.61045E+00_JPRB, 0.59615E+00_JPRB, 0.62512E+00_JPRB, 0.68656E+00_JPRB, 0.71068E+00_JPRB, 0.71559E+00_JPRB ,&
&  0.70808E+00_JPRB, 0.69681E+00_JPRB, 0.72654E+00_JPRB, 0.73922E+00_JPRB, 0.82039E+00_JPRB, 0.92083E+00_JPRB, 0.10039E+01_JPRB ,&
&  0.10994E+01_JPRB, 0.11871E+01_JPRB, 0.12817E+01_JPRB, 0.13913E+01_JPRB, 0.14322E+01_JPRB, 0.14761E+01_JPRB, 0.11550E+01_JPRB ,&
&  0.93456E+00_JPRB, 0.19950E+01_JPRB, 0.17607E+01_JPRB, 0.15629E+01_JPRB, 0.12699E+01_JPRB, 0.12254E+01_JPRB, 0.12184E+01_JPRB ,&
&  0.12560E+01_JPRB, 0.12074E+01_JPRB, 0.12570E+01_JPRB, 0.13093E+01_JPRB, 0.13330E+01_JPRB, 0.13721E+01_JPRB, 0.14197E+01_JPRB ,&
&  0.14522E+01_JPRB, 0.14922E+01_JPRB, 0.14967E+01_JPRB, 0.14927E+01_JPRB, 0.14901E+01_JPRB, 0.14755E+01_JPRB, 0.15023E+01_JPRB ,&
&  0.15449E+01_JPRB, 0.15725E+01_JPRB, 0.15604E+01_JPRB, 0.15495E+01_JPRB, 0.15057E+01_JPRB, 0.15524E+01_JPRB, 0.16244E+01_JPRB ,&
&  0.15825E+01_JPRB, 0.15417E+01_JPRB, 0.16643E+01_JPRB, 0.16200E+01_JPRB, 0.15225E+01_JPRB, 0.15317E+01_JPRB, 0.13026E+01_JPRB ,&
&  0.12713E+01_JPRB, 0.10620E+01_JPRB, 0.10519E+01_JPRB, 0.84464E+00_JPRB, 0.85882E+00_JPRB, 0.91545E+00_JPRB, 0.88382E+00_JPRB ,&
&  0.83945E+00_JPRB, 0.87593E+00_JPRB, 0.85790E+00_JPRB, 0.85799E+00_JPRB, 0.83575E+00_JPRB, 0.78355E+00_JPRB, 0.72107E+00_JPRB ,&
&  0.72425E+00_JPRB, 0.72308E+00_JPRB, 0.66679E+00_JPRB, 0.51916E+00_JPRB, 0.46675E+00_JPRB, 0.44916E+00_JPRB, 0.56942E+00_JPRB ,&
&  0.69891E+00_JPRB, 0.76416E+00_JPRB, 0.82747E+00_JPRB, 0.87102E+00_JPRB, 0.81058E+00_JPRB, 0.73265E+00_JPRB, 0.62530E+00_JPRB ,&
&  0.52499E+00_JPRB, 0.48898E+00_JPRB, 0.51081E+00_JPRB, 0.50753E+00_JPRB, 0.52344E+00_JPRB, 0.54217E+00_JPRB, 0.55944E+00_JPRB ,&
&  0.57521E+00_JPRB, 0.59156E+00_JPRB, 0.60607E+00_JPRB, 0.61060E+00_JPRB, 0.62292E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.61798E+00_JPRB, 0.61036E+00_JPRB, 0.60000E+00_JPRB, 0.60694E+00_JPRB, 0.66955E+00_JPRB, 0.68890E+00_JPRB, 0.65430E+00_JPRB ,&
&  0.62637E+00_JPRB, 0.62364E+00_JPRB, 0.64964E+00_JPRB, 0.63518E+00_JPRB, 0.66940E+00_JPRB, 0.76844E+00_JPRB, 0.84136E+00_JPRB ,&
&  0.91649E+00_JPRB, 0.10194E+01_JPRB, 0.11626E+01_JPRB, 0.13222E+01_JPRB, 0.14316E+01_JPRB, 0.15561E+01_JPRB, 0.10099E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.17728E+01_JPRB, 0.18828E+01_JPRB, 0.18630E+01_JPRB, 0.16301E+01_JPRB, 0.14860E+01_JPRB, 0.13495E+01_JPRB ,&
&  0.13551E+01_JPRB, 0.12830E+01_JPRB, 0.12333E+01_JPRB, 0.12188E+01_JPRB, 0.12125E+01_JPRB, 0.12742E+01_JPRB, 0.12911E+01_JPRB ,&
&  0.12645E+01_JPRB, 0.12737E+01_JPRB, 0.13418E+01_JPRB, 0.13901E+01_JPRB, 0.14168E+01_JPRB, 0.14372E+01_JPRB, 0.14631E+01_JPRB ,&
&  0.15236E+01_JPRB, 0.15899E+01_JPRB, 0.16845E+01_JPRB, 0.16387E+01_JPRB, 0.16608E+01_JPRB, 0.17124E+01_JPRB, 0.18294E+01_JPRB ,&
&  0.17889E+01_JPRB, 0.17469E+01_JPRB, 0.19636E+01_JPRB, 0.21492E+01_JPRB, 0.20765E+01_JPRB, 0.17912E+01_JPRB, 0.14969E+01_JPRB ,&
&  0.13774E+01_JPRB, 0.11902E+01_JPRB, 0.10363E+01_JPRB, 0.89963E+00_JPRB, 0.93274E+00_JPRB, 0.10022E+01_JPRB, 0.10164E+01_JPRB ,&
&  0.99160E+00_JPRB, 0.94123E+00_JPRB, 0.91029E+00_JPRB, 0.89173E+00_JPRB, 0.87253E+00_JPRB, 0.81584E+00_JPRB, 0.80306E+00_JPRB ,&
&  0.87710E+00_JPRB, 0.95671E+00_JPRB, 0.82995E+00_JPRB, 0.60989E+00_JPRB, 0.49182E+00_JPRB, 0.38219E+00_JPRB, 0.42994E+00_JPRB ,&
&  0.63415E+00_JPRB, 0.69885E+00_JPRB, 0.74799E+00_JPRB, 0.94647E+00_JPRB, 0.96850E+00_JPRB, 0.95580E+00_JPRB, 0.86228E+00_JPRB ,&
&  0.69552E+00_JPRB, 0.61410E+00_JPRB, 0.57143E+00_JPRB, 0.53607E+00_JPRB, 0.53574E+00_JPRB, 0.54141E+00_JPRB, 0.55676E+00_JPRB ,&
&  0.57281E+00_JPRB, 0.58264E+00_JPRB, 0.58939E+00_JPRB, 0.59539E+00_JPRB, 0.61261E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.61553E+00_JPRB, 0.61092E+00_JPRB, 0.61826E+00_JPRB, 0.61627E+00_JPRB, 0.65631E+00_JPRB, 0.68060E+00_JPRB, 0.61348E+00_JPRB ,&
&  0.59431E+00_JPRB, 0.58770E+00_JPRB, 0.60265E+00_JPRB, 0.59721E+00_JPRB, 0.59683E+00_JPRB, 0.63502E+00_JPRB, 0.68661E+00_JPRB ,&
&  0.75524E+00_JPRB, 0.84006E+00_JPRB, 0.99551E+00_JPRB, 0.11707E+01_JPRB, 0.12330E+01_JPRB, 0.11787E+01_JPRB, 0.12601E+01_JPRB ,&
&  0.15593E+01_JPRB, 0.19273E+01_JPRB, 0.20571E+01_JPRB, 0.22849E+01_JPRB, 0.22801E+01_JPRB, 0.23791E+01_JPRB, 0.18734E+01_JPRB ,&
&  0.17109E+01_JPRB, 0.15705E+01_JPRB, 0.13923E+01_JPRB, 0.12703E+01_JPRB, 0.11562E+01_JPRB, 0.11406E+01_JPRB, 0.11806E+01_JPRB ,&
&  0.11577E+01_JPRB, 0.11587E+01_JPRB, 0.11895E+01_JPRB, 0.12118E+01_JPRB, 0.12671E+01_JPRB, 0.13106E+01_JPRB, 0.13642E+01_JPRB ,&
&  0.14532E+01_JPRB, 0.15613E+01_JPRB, 0.16822E+01_JPRB, 0.17339E+01_JPRB, 0.17459E+01_JPRB, 0.18127E+01_JPRB, 0.19944E+01_JPRB ,&
&  0.20858E+01_JPRB, 0.21817E+01_JPRB, 0.22702E+01_JPRB, 0.24546E+01_JPRB, 0.24698E+01_JPRB, 0.22226E+01_JPRB, 0.18223E+01_JPRB ,&
&  0.15429E+01_JPRB, 0.12954E+01_JPRB, 0.10321E+01_JPRB, 0.97698E+00_JPRB, 0.10365E+01_JPRB, 0.11298E+01_JPRB, 0.10300E+01_JPRB ,&
&  0.91614E+00_JPRB, 0.83451E+00_JPRB, 0.83125E+00_JPRB, 0.87535E+00_JPRB, 0.93466E+00_JPRB, 0.91860E+00_JPRB, 0.93874E+00_JPRB ,&
&  0.90829E+00_JPRB, 0.61243E+00_JPRB, 0.51573E+00_JPRB, 0.60318E+00_JPRB, 0.47138E+00_JPRB, 0.36219E+00_JPRB, 0.44205E+00_JPRB ,&
&  0.57389E+00_JPRB, 0.71616E+00_JPRB, 0.76344E+00_JPRB, 0.80078E+00_JPRB, 0.10063E+01_JPRB, 0.11091E+01_JPRB, 0.11419E+01_JPRB ,&
&  0.81292E+00_JPRB, 0.71827E+00_JPRB, 0.65735E+00_JPRB, 0.60872E+00_JPRB, 0.57346E+00_JPRB, 0.56155E+00_JPRB, 0.55939E+00_JPRB ,&
&  0.55111E+00_JPRB, 0.55501E+00_JPRB, 0.56161E+00_JPRB, 0.57499E+00_JPRB, 0.59849E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.58939E+00_JPRB, 0.59799E+00_JPRB, 0.61561E+00_JPRB, 0.62907E+00_JPRB, 0.61817E+00_JPRB, 0.63365E+00_JPRB, 0.61974E+00_JPRB ,&
&  0.58807E+00_JPRB, 0.56314E+00_JPRB, 0.56124E+00_JPRB, 0.56990E+00_JPRB, 0.57608E+00_JPRB, 0.57889E+00_JPRB, 0.60087E+00_JPRB ,&
&  0.64127E+00_JPRB, 0.71348E+00_JPRB, 0.87388E+00_JPRB, 0.10204E+01_JPRB, 0.10827E+01_JPRB, 0.10124E+01_JPRB, 0.13760E+01_JPRB ,&
&  0.16726E+01_JPRB, 0.20539E+01_JPRB, 0.25318E+01_JPRB, 0.27304E+01_JPRB, 0.26620E+01_JPRB, 0.26228E+01_JPRB, 0.22737E+01_JPRB ,&
&  0.19858E+01_JPRB, 0.17908E+01_JPRB, 0.16122E+01_JPRB, 0.14625E+01_JPRB, 0.13548E+01_JPRB, 0.12402E+01_JPRB, 0.11956E+01_JPRB ,&
&  0.11492E+01_JPRB, 0.11213E+01_JPRB, 0.11404E+01_JPRB, 0.11393E+01_JPRB, 0.11710E+01_JPRB, 0.12054E+01_JPRB, 0.12646E+01_JPRB ,&
&  0.13630E+01_JPRB, 0.15020E+01_JPRB, 0.16560E+01_JPRB, 0.18635E+01_JPRB, 0.15552E+01_JPRB, 0.18365E+01_JPRB, 0.19341E+01_JPRB ,&
&  0.20827E+01_JPRB, 0.21597E+01_JPRB, 0.21345E+01_JPRB, 0.27553E+01_JPRB, 0.29539E+01_JPRB, 0.25373E+01_JPRB, 0.20861E+01_JPRB ,&
&  0.17567E+01_JPRB, 0.14203E+01_JPRB, 0.11810E+01_JPRB, 0.11517E+01_JPRB, 0.11775E+01_JPRB, 0.12114E+01_JPRB, 0.90927E+00_JPRB ,&
&  0.82824E+00_JPRB, 0.93375E+00_JPRB, 0.92626E+00_JPRB, 0.81341E+00_JPRB, 0.89958E+00_JPRB, 0.10239E+01_JPRB, 0.77295E+00_JPRB ,&
&  0.43740E+00_JPRB, 0.39157E+00_JPRB, 0.41262E+00_JPRB, 0.41565E+00_JPRB, 0.39570E+00_JPRB, 0.39217E+00_JPRB, 0.60608E+00_JPRB ,&
&  0.54104E+00_JPRB, 0.70969E+00_JPRB, 0.70550E+00_JPRB, 0.74665E+00_JPRB, 0.89428E+00_JPRB, 0.11357E+01_JPRB, 0.14525E+01_JPRB ,&
&  0.91097E+00_JPRB, 0.79621E+00_JPRB, 0.69460E+00_JPRB, 0.64094E+00_JPRB, 0.59778E+00_JPRB, 0.57326E+00_JPRB, 0.56149E+00_JPRB ,&
&  0.54975E+00_JPRB, 0.54193E+00_JPRB, 0.54076E+00_JPRB, 0.55366E+00_JPRB, 0.57120E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.59600E+00_JPRB, 0.60114E+00_JPRB, 0.60289E+00_JPRB, 0.60459E+00_JPRB, 0.58683E+00_JPRB, 0.59415E+00_JPRB, 0.61731E+00_JPRB ,&
&  0.60081E+00_JPRB, 0.56692E+00_JPRB, 0.55152E+00_JPRB, 0.55615E+00_JPRB, 0.56861E+00_JPRB, 0.57524E+00_JPRB, 0.57404E+00_JPRB ,&
&  0.59958E+00_JPRB, 0.66373E+00_JPRB, 0.87620E+00_JPRB, 0.89652E+00_JPRB, 0.96660E+00_JPRB, 0.92454E+00_JPRB, 0.12207E+01_JPRB ,&
&  0.16589E+01_JPRB, 0.21598E+01_JPRB, 0.26053E+01_JPRB, 0.28120E+01_JPRB, 0.31842E+01_JPRB, 0.32383E+01_JPRB, 0.30770E+01_JPRB ,&
&  0.25530E+01_JPRB, 0.21762E+01_JPRB, 0.18935E+01_JPRB, 0.17012E+01_JPRB, 0.15470E+01_JPRB, 0.13747E+01_JPRB, 0.12781E+01_JPRB ,&
&  0.11873E+01_JPRB, 0.11310E+01_JPRB, 0.10841E+01_JPRB, 0.10936E+01_JPRB, 0.11103E+01_JPRB, 0.11149E+01_JPRB, 0.11580E+01_JPRB ,&
&  0.12355E+01_JPRB, 0.14052E+01_JPRB, 0.15977E+01_JPRB, 0.18482E+01_JPRB, 0.16296E+01_JPRB, 0.14663E+01_JPRB, 0.17158E+01_JPRB ,&
&  0.19886E+01_JPRB, 0.22240E+01_JPRB, 0.23312E+01_JPRB, 0.32383E+01_JPRB, 0.32705E+01_JPRB, 0.27410E+01_JPRB, 0.24222E+01_JPRB ,&
&  0.19777E+01_JPRB, 0.17454E+01_JPRB, 0.13763E+01_JPRB, 0.13394E+01_JPRB, 0.13440E+01_JPRB, 0.10742E+01_JPRB, 0.95183E+00_JPRB ,&
&  0.10262E+01_JPRB, 0.99860E+00_JPRB, 0.93544E+00_JPRB, 0.79667E+00_JPRB, 0.88602E+00_JPRB, 0.86228E+00_JPRB, 0.52800E+00_JPRB ,&
&  0.46995E+00_JPRB, 0.45786E+00_JPRB, 0.44964E+00_JPRB, 0.43230E+00_JPRB, 0.42330E+00_JPRB, 0.42344E+00_JPRB, 0.54540E+00_JPRB ,&
&  0.53253E+00_JPRB, 0.66271E+00_JPRB, 0.68971E+00_JPRB, 0.71037E+00_JPRB, 0.77269E+00_JPRB, 0.10498E+01_JPRB, 0.12549E+01_JPRB ,&
&  0.96082E+00_JPRB, 0.84097E+00_JPRB, 0.70137E+00_JPRB, 0.64431E+00_JPRB, 0.59528E+00_JPRB, 0.56657E+00_JPRB, 0.57514E+00_JPRB ,&
&  0.56632E+00_JPRB, 0.54794E+00_JPRB, 0.54464E+00_JPRB, 0.55758E+00_JPRB, 0.57518E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.59574E+00_JPRB, 0.60335E+00_JPRB, 0.60862E+00_JPRB, 0.62203E+00_JPRB, 0.61334E+00_JPRB, 0.60253E+00_JPRB, 0.62131E+00_JPRB ,&
&  0.60599E+00_JPRB, 0.58555E+00_JPRB, 0.56540E+00_JPRB, 0.56449E+00_JPRB, 0.56015E+00_JPRB, 0.57164E+00_JPRB, 0.57532E+00_JPRB ,&
&  0.59528E+00_JPRB, 0.69170E+00_JPRB, 0.84313E+00_JPRB, 0.81777E+00_JPRB, 0.79106E+00_JPRB, 0.78285E+00_JPRB, 0.95920E+00_JPRB ,&
&  0.15382E+01_JPRB, 0.21683E+01_JPRB, 0.26172E+01_JPRB, 0.30597E+01_JPRB, 0.34676E+01_JPRB, 0.34134E+01_JPRB, 0.35094E+01_JPRB ,&
&  0.30817E+01_JPRB, 0.25779E+01_JPRB, 0.22338E+01_JPRB, 0.19620E+01_JPRB, 0.18225E+01_JPRB, 0.16870E+01_JPRB, 0.15452E+01_JPRB ,&
&  0.13569E+01_JPRB, 0.12115E+01_JPRB, 0.11257E+01_JPRB, 0.10629E+01_JPRB, 0.10419E+01_JPRB, 0.10512E+01_JPRB, 0.10527E+01_JPRB ,&
&  0.11199E+01_JPRB, 0.12837E+01_JPRB, 0.14730E+01_JPRB, 0.17460E+01_JPRB, 0.18768E+01_JPRB, 0.18478E+01_JPRB, 0.17825E+01_JPRB ,&
&  0.18101E+01_JPRB, 0.18739E+01_JPRB, 0.27007E+01_JPRB, 0.34589E+01_JPRB, 0.32460E+01_JPRB, 0.28558E+01_JPRB, 0.25214E+01_JPRB ,&
&  0.20918E+01_JPRB, 0.18241E+01_JPRB, 0.14876E+01_JPRB, 0.14886E+01_JPRB, 0.13152E+01_JPRB, 0.10519E+01_JPRB, 0.11066E+01_JPRB ,&
&  0.11683E+01_JPRB, 0.10900E+01_JPRB, 0.94762E+00_JPRB, 0.80108E+00_JPRB, 0.72163E+00_JPRB, 0.62918E+00_JPRB, 0.52603E+00_JPRB ,&
&  0.49715E+00_JPRB, 0.48793E+00_JPRB, 0.46403E+00_JPRB, 0.45974E+00_JPRB, 0.44988E+00_JPRB, 0.46475E+00_JPRB, 0.52456E+00_JPRB ,&
&  0.59819E+00_JPRB, 0.63178E+00_JPRB, 0.67245E+00_JPRB, 0.69434E+00_JPRB, 0.70351E+00_JPRB, 0.77006E+00_JPRB, 0.10170E+01_JPRB ,&
&  0.86602E+00_JPRB, 0.82961E+00_JPRB, 0.72346E+00_JPRB, 0.64311E+00_JPRB, 0.60828E+00_JPRB, 0.57797E+00_JPRB, 0.56095E+00_JPRB ,&
&  0.55736E+00_JPRB, 0.56848E+00_JPRB, 0.57505E+00_JPRB, 0.57292E+00_JPRB, 0.56802E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.59384E+00_JPRB, 0.61037E+00_JPRB, 0.62133E+00_JPRB, 0.62525E+00_JPRB, 0.61683E+00_JPRB, 0.61204E+00_JPRB, 0.62095E+00_JPRB ,&
&  0.61116E+00_JPRB, 0.59952E+00_JPRB, 0.58686E+00_JPRB, 0.58255E+00_JPRB, 0.57337E+00_JPRB, 0.57370E+00_JPRB, 0.56872E+00_JPRB ,&
&  0.58834E+00_JPRB, 0.73626E+00_JPRB, 0.64932E+00_JPRB, 0.65512E+00_JPRB, 0.70498E+00_JPRB, 0.74460E+00_JPRB, 0.83503E+00_JPRB ,&
&  0.14157E+01_JPRB, 0.21816E+01_JPRB, 0.27810E+01_JPRB, 0.36080E+01_JPRB, 0.43909E+01_JPRB, 0.40499E+01_JPRB, 0.38351E+01_JPRB ,&
&  0.36525E+01_JPRB, 0.30346E+01_JPRB, 0.26389E+01_JPRB, 0.23357E+01_JPRB, 0.20971E+01_JPRB, 0.19158E+01_JPRB, 0.17046E+01_JPRB ,&
&  0.14799E+01_JPRB, 0.13170E+01_JPRB, 0.11991E+01_JPRB, 0.11101E+01_JPRB, 0.10488E+01_JPRB, 0.10717E+01_JPRB, 0.10665E+01_JPRB ,&
&  0.10401E+01_JPRB, 0.11477E+01_JPRB, 0.13483E+01_JPRB, 0.15791E+01_JPRB, 0.16882E+01_JPRB, 0.17819E+01_JPRB, 0.24147E+01_JPRB ,&
&  0.24469E+01_JPRB, 0.25175E+01_JPRB, 0.29713E+01_JPRB, 0.43716E+01_JPRB, 0.33841E+01_JPRB, 0.26532E+01_JPRB, 0.25392E+01_JPRB ,&
&  0.17986E+01_JPRB, 0.14952E+01_JPRB, 0.14273E+01_JPRB, 0.14090E+01_JPRB, 0.12170E+01_JPRB, 0.12484E+01_JPRB, 0.16328E+01_JPRB ,&
&  0.13210E+01_JPRB, 0.12226E+01_JPRB, 0.11887E+01_JPRB, 0.11091E+01_JPRB, 0.65051E+00_JPRB, 0.55256E+00_JPRB, 0.72148E+00_JPRB ,&
&  0.75190E+00_JPRB, 0.56015E+00_JPRB, 0.47769E+00_JPRB, 0.55726E+00_JPRB, 0.47644E+00_JPRB, 0.50499E+00_JPRB, 0.59030E+00_JPRB ,&
&  0.59297E+00_JPRB, 0.59291E+00_JPRB, 0.67675E+00_JPRB, 0.67300E+00_JPRB, 0.66425E+00_JPRB, 0.66182E+00_JPRB, 0.80361E+00_JPRB ,&
&  0.85561E+00_JPRB, 0.86605E+00_JPRB, 0.81957E+00_JPRB, 0.71205E+00_JPRB, 0.66370E+00_JPRB, 0.62985E+00_JPRB, 0.60675E+00_JPRB ,&
&  0.57700E+00_JPRB, 0.56940E+00_JPRB, 0.58371E+00_JPRB, 0.58988E+00_JPRB, 0.57346E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.59522E+00_JPRB, 0.59002E+00_JPRB, 0.60056E+00_JPRB, 0.60162E+00_JPRB, 0.60884E+00_JPRB, 0.61346E+00_JPRB, 0.62025E+00_JPRB ,&
&  0.61018E+00_JPRB, 0.59742E+00_JPRB, 0.60163E+00_JPRB, 0.60354E+00_JPRB, 0.59126E+00_JPRB, 0.57789E+00_JPRB, 0.56855E+00_JPRB ,&
&  0.58917E+00_JPRB, 0.63008E+00_JPRB, 0.57414E+00_JPRB, 0.58295E+00_JPRB, 0.69042E+00_JPRB, 0.72367E+00_JPRB, 0.84274E+00_JPRB ,&
&  0.13600E+01_JPRB, 0.21054E+01_JPRB, 0.28560E+01_JPRB, 0.38534E+01_JPRB, 0.45036E+01_JPRB, 0.44447E+01_JPRB, 0.40934E+01_JPRB ,&
&  0.40395E+01_JPRB, 0.32817E+01_JPRB, 0.28541E+01_JPRB, 0.25866E+01_JPRB, 0.22834E+01_JPRB, 0.19990E+01_JPRB, 0.17615E+01_JPRB ,&
&  0.15438E+01_JPRB, 0.13576E+01_JPRB, 0.12448E+01_JPRB, 0.11541E+01_JPRB, 0.11233E+01_JPRB, 0.11206E+01_JPRB, 0.10917E+01_JPRB ,&
&  0.10197E+01_JPRB, 0.10153E+01_JPRB, 0.12413E+01_JPRB, 0.14790E+01_JPRB, 0.15517E+01_JPRB, 0.16527E+01_JPRB, 0.21422E+01_JPRB ,&
&  0.25344E+01_JPRB, 0.26019E+01_JPRB, 0.31575E+01_JPRB, 0.38620E+01_JPRB, 0.36646E+01_JPRB, 0.29142E+01_JPRB, 0.28895E+01_JPRB ,&
&  0.21071E+01_JPRB, 0.17025E+01_JPRB, 0.14442E+01_JPRB, 0.12071E+01_JPRB, 0.11721E+01_JPRB, 0.16189E+01_JPRB, 0.15693E+01_JPRB ,&
&  0.14341E+01_JPRB, 0.12993E+01_JPRB, 0.12325E+01_JPRB, 0.10396E+01_JPRB, 0.80278E+00_JPRB, 0.59181E+00_JPRB, 0.69318E+00_JPRB ,&
&  0.77752E+00_JPRB, 0.80872E+00_JPRB, 0.74404E+00_JPRB, 0.66536E+00_JPRB, 0.57508E+00_JPRB, 0.62188E+00_JPRB, 0.61338E+00_JPRB ,&
&  0.59467E+00_JPRB, 0.56830E+00_JPRB, 0.59791E+00_JPRB, 0.62832E+00_JPRB, 0.64282E+00_JPRB, 0.60786E+00_JPRB, 0.69626E+00_JPRB ,&
&  0.74310E+00_JPRB, 0.84560E+00_JPRB, 0.85118E+00_JPRB, 0.76532E+00_JPRB, 0.70434E+00_JPRB, 0.67878E+00_JPRB, 0.65802E+00_JPRB ,&
&  0.63749E+00_JPRB, 0.60350E+00_JPRB, 0.59671E+00_JPRB, 0.58796E+00_JPRB, 0.58065E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.59668E+00_JPRB, 0.59666E+00_JPRB, 0.59167E+00_JPRB, 0.58378E+00_JPRB, 0.57676E+00_JPRB, 0.57654E+00_JPRB, 0.58462E+00_JPRB ,&
&  0.58194E+00_JPRB, 0.56652E+00_JPRB, 0.56812E+00_JPRB, 0.57099E+00_JPRB, 0.56336E+00_JPRB, 0.54972E+00_JPRB, 0.54633E+00_JPRB ,&
&  0.57031E+00_JPRB, 0.58757E+00_JPRB, 0.52482E+00_JPRB, 0.58953E+00_JPRB, 0.65330E+00_JPRB, 0.68557E+00_JPRB, 0.84470E+00_JPRB ,&
&  0.10964E+01_JPRB, 0.16329E+01_JPRB, 0.22675E+01_JPRB, 0.31937E+01_JPRB, 0.40225E+01_JPRB, 0.44029E+01_JPRB, 0.43988E+01_JPRB ,&
&  0.39028E+01_JPRB, 0.35302E+01_JPRB, 0.30315E+01_JPRB, 0.26337E+01_JPRB, 0.23008E+01_JPRB, 0.20513E+01_JPRB, 0.18043E+01_JPRB ,&
&  0.16053E+01_JPRB, 0.14425E+01_JPRB, 0.13333E+01_JPRB, 0.12327E+01_JPRB, 0.11892E+01_JPRB, 0.11772E+01_JPRB, 0.11513E+01_JPRB ,&
&  0.10632E+01_JPRB, 0.98143E+00_JPRB, 0.11229E+01_JPRB, 0.13917E+01_JPRB, 0.14576E+01_JPRB, 0.16084E+01_JPRB, 0.18109E+01_JPRB ,&
&  0.26304E+01_JPRB, 0.29833E+01_JPRB, 0.30738E+01_JPRB, 0.41590E+01_JPRB, 0.27807E+01_JPRB, 0.31061E+01_JPRB, 0.33408E+01_JPRB ,&
&  0.30853E+01_JPRB, 0.26461E+01_JPRB, 0.23913E+01_JPRB, 0.17980E+01_JPRB, 0.13705E+01_JPRB, 0.17607E+01_JPRB, 0.15653E+01_JPRB ,&
&  0.13746E+01_JPRB, 0.13397E+01_JPRB, 0.12854E+01_JPRB, 0.12501E+01_JPRB, 0.88744E+00_JPRB, 0.70460E+00_JPRB, 0.59518E+00_JPRB ,&
&  0.63366E+00_JPRB, 0.63952E+00_JPRB, 0.72162E+00_JPRB, 0.66774E+00_JPRB, 0.65449E+00_JPRB, 0.62835E+00_JPRB, 0.60020E+00_JPRB ,&
&  0.59810E+00_JPRB, 0.57946E+00_JPRB, 0.56573E+00_JPRB, 0.60238E+00_JPRB, 0.63881E+00_JPRB, 0.59778E+00_JPRB, 0.63662E+00_JPRB ,&
&  0.68189E+00_JPRB, 0.75260E+00_JPRB, 0.78283E+00_JPRB, 0.78065E+00_JPRB, 0.72201E+00_JPRB, 0.68540E+00_JPRB, 0.66923E+00_JPRB ,&
&  0.66426E+00_JPRB, 0.64390E+00_JPRB, 0.61560E+00_JPRB, 0.61028E+00_JPRB, 0.59233E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.59852E+00_JPRB, 0.58837E+00_JPRB, 0.56906E+00_JPRB, 0.56372E+00_JPRB, 0.55776E+00_JPRB, 0.55505E+00_JPRB, 0.54846E+00_JPRB ,&
&  0.53606E+00_JPRB, 0.53096E+00_JPRB, 0.52924E+00_JPRB, 0.52773E+00_JPRB, 0.52515E+00_JPRB, 0.51597E+00_JPRB, 0.51635E+00_JPRB ,&
&  0.52593E+00_JPRB, 0.64829E+00_JPRB, 0.57341E+00_JPRB, 0.52362E+00_JPRB, 0.50809E+00_JPRB, 0.60194E+00_JPRB, 0.73236E+00_JPRB ,&
&  0.93072E+00_JPRB, 0.11995E+01_JPRB, 0.16638E+01_JPRB, 0.20978E+01_JPRB, 0.26615E+01_JPRB, 0.31935E+01_JPRB, 0.35382E+01_JPRB ,&
&  0.35919E+01_JPRB, 0.27338E+01_JPRB, 0.24929E+01_JPRB, 0.21480E+01_JPRB, 0.19378E+01_JPRB, 0.17191E+01_JPRB, 0.15633E+01_JPRB ,&
&  0.14265E+01_JPRB, 0.13358E+01_JPRB, 0.12659E+01_JPRB, 0.11925E+01_JPRB, 0.11478E+01_JPRB, 0.11202E+01_JPRB, 0.11114E+01_JPRB ,&
&  0.10731E+01_JPRB, 0.10006E+01_JPRB, 0.10435E+01_JPRB, 0.12416E+01_JPRB, 0.16222E+01_JPRB, 0.19406E+01_JPRB, 0.21537E+01_JPRB ,&
&  0.19691E+01_JPRB, 0.19333E+01_JPRB, 0.32013E+01_JPRB, 0.37168E+01_JPRB, 0.35914E+01_JPRB, 0.40466E+01_JPRB, 0.40545E+01_JPRB ,&
&  0.35880E+01_JPRB, 0.28220E+01_JPRB, 0.23237E+01_JPRB, 0.18908E+01_JPRB, 0.19554E+01_JPRB, 0.17840E+01_JPRB, 0.16032E+01_JPRB ,&
&  0.14298E+01_JPRB, 0.14468E+01_JPRB, 0.14063E+01_JPRB, 0.13676E+01_JPRB, 0.12518E+01_JPRB, 0.11093E+01_JPRB, 0.88987E+00_JPRB ,&
&  0.78443E+00_JPRB, 0.88901E+00_JPRB, 0.76435E+00_JPRB, 0.64034E+00_JPRB, 0.56747E+00_JPRB, 0.57948E+00_JPRB, 0.60714E+00_JPRB ,&
&  0.62483E+00_JPRB, 0.62538E+00_JPRB, 0.61398E+00_JPRB, 0.62175E+00_JPRB, 0.63051E+00_JPRB, 0.61279E+00_JPRB, 0.61705E+00_JPRB ,&
&  0.62347E+00_JPRB, 0.67755E+00_JPRB, 0.70649E+00_JPRB, 0.73255E+00_JPRB, 0.74055E+00_JPRB, 0.70684E+00_JPRB, 0.68694E+00_JPRB ,&
&  0.68285E+00_JPRB, 0.68105E+00_JPRB, 0.65567E+00_JPRB, 0.62781E+00_JPRB, 0.60412E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.58874E+00_JPRB, 0.57333E+00_JPRB, 0.55541E+00_JPRB, 0.55391E+00_JPRB, 0.55411E+00_JPRB, 0.55417E+00_JPRB, 0.54118E+00_JPRB ,&
&  0.51502E+00_JPRB, 0.50012E+00_JPRB, 0.49675E+00_JPRB, 0.50058E+00_JPRB, 0.50134E+00_JPRB, 0.50032E+00_JPRB, 0.50805E+00_JPRB ,&
&  0.50634E+00_JPRB, 0.72940E+00_JPRB, 0.63784E+00_JPRB, 0.60511E+00_JPRB, 0.54140E+00_JPRB, 0.59996E+00_JPRB, 0.69025E+00_JPRB ,&
&  0.82353E+00_JPRB, 0.89990E+00_JPRB, 0.10728E+01_JPRB, 0.13364E+01_JPRB, 0.16140E+01_JPRB, 0.18482E+01_JPRB, 0.22210E+01_JPRB ,&
&  0.22014E+01_JPRB, 0.20233E+01_JPRB, 0.16927E+01_JPRB, 0.16331E+01_JPRB, 0.15523E+01_JPRB, 0.13477E+01_JPRB, 0.13353E+01_JPRB ,&
&  0.12533E+01_JPRB, 0.12025E+01_JPRB, 0.11555E+01_JPRB, 0.11163E+01_JPRB, 0.10768E+01_JPRB, 0.10229E+01_JPRB, 0.99313E+00_JPRB ,&
&  0.97488E+00_JPRB, 0.95757E+00_JPRB, 0.10039E+01_JPRB, 0.10716E+01_JPRB, 0.13289E+01_JPRB, 0.16698E+01_JPRB, 0.19395E+01_JPRB ,&
&  0.22806E+01_JPRB, 0.20713E+01_JPRB, 0.19497E+01_JPRB, 0.27427E+01_JPRB, 0.39159E+01_JPRB, 0.41897E+01_JPRB, 0.38469E+01_JPRB ,&
&  0.38601E+01_JPRB, 0.33960E+01_JPRB, 0.28333E+01_JPRB, 0.24388E+01_JPRB, 0.21647E+01_JPRB, 0.19427E+01_JPRB, 0.18102E+01_JPRB ,&
&  0.16334E+01_JPRB, 0.15598E+01_JPRB, 0.15838E+01_JPRB, 0.15136E+01_JPRB, 0.13872E+01_JPRB, 0.12406E+01_JPRB, 0.11586E+01_JPRB ,&
&  0.10572E+01_JPRB, 0.87827E+00_JPRB, 0.65174E+00_JPRB, 0.62844E+00_JPRB, 0.53649E+00_JPRB, 0.54262E+00_JPRB, 0.60287E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.62813E+00_JPRB, 0.62836E+00_JPRB, 0.60140E+00_JPRB, 0.60677E+00_JPRB, 0.59119E+00_JPRB, 0.59608E+00_JPRB ,&
&  0.60034E+00_JPRB, 0.57002E+00_JPRB, 0.60480E+00_JPRB, 0.65093E+00_JPRB, 0.69404E+00_JPRB, 0.69750E+00_JPRB, 0.68700E+00_JPRB ,&
&  0.68400E+00_JPRB, 0.67874E+00_JPRB, 0.66294E+00_JPRB, 0.63260E+00_JPRB, 0.60738E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.58512E+00_JPRB, 0.57114E+00_JPRB, 0.55587E+00_JPRB, 0.54647E+00_JPRB, 0.54118E+00_JPRB, 0.54055E+00_JPRB, 0.52911E+00_JPRB ,&
&  0.50860E+00_JPRB, 0.49314E+00_JPRB, 0.48590E+00_JPRB, 0.48800E+00_JPRB, 0.48754E+00_JPRB, 0.48050E+00_JPRB, 0.49166E+00_JPRB ,&
&  0.54958E+00_JPRB, 0.65292E+00_JPRB, 0.60125E+00_JPRB, 0.53331E+00_JPRB, 0.61168E+00_JPRB, 0.59589E+00_JPRB, 0.63093E+00_JPRB ,&
&  0.65187E+00_JPRB, 0.75205E+00_JPRB, 0.82574E+00_JPRB, 0.93310E+00_JPRB, 0.10373E+01_JPRB, 0.11587E+01_JPRB, 0.13435E+01_JPRB ,&
&  0.13857E+01_JPRB, 0.12537E+01_JPRB, 0.11649E+01_JPRB, 0.11738E+01_JPRB, 0.12427E+01_JPRB, 0.11289E+01_JPRB, 0.11180E+01_JPRB ,&
&  0.10738E+01_JPRB, 0.10353E+01_JPRB, 0.10047E+01_JPRB, 0.98562E+00_JPRB, 0.96678E+00_JPRB, 0.93934E+00_JPRB, 0.91941E+00_JPRB ,&
&  0.91502E+00_JPRB, 0.91335E+00_JPRB, 0.96187E+00_JPRB, 0.10116E+01_JPRB, 0.11661E+01_JPRB, 0.15536E+01_JPRB, 0.22185E+01_JPRB ,&
&  0.24541E+01_JPRB, 0.25330E+01_JPRB, 0.27646E+01_JPRB, 0.31229E+01_JPRB, 0.33071E+01_JPRB, 0.34536E+01_JPRB, 0.35853E+01_JPRB ,&
&  0.34909E+01_JPRB, 0.33657E+01_JPRB, 0.32124E+01_JPRB, 0.29540E+01_JPRB, 0.25956E+01_JPRB, 0.23127E+01_JPRB, 0.21019E+01_JPRB ,&
&  0.18682E+01_JPRB, 0.17362E+01_JPRB, 0.17102E+01_JPRB, 0.15885E+01_JPRB, 0.14946E+01_JPRB, 0.13285E+01_JPRB, 0.13081E+01_JPRB ,&
&  0.11417E+01_JPRB, 0.72458E+00_JPRB, 0.60317E+00_JPRB, 0.64026E+00_JPRB, 0.55378E+00_JPRB, 0.57346E+00_JPRB, 0.62534E+00_JPRB ,&
&  0.60571E+00_JPRB, 0.59453E+00_JPRB, 0.59798E+00_JPRB, 0.58210E+00_JPRB, 0.57084E+00_JPRB, 0.58169E+00_JPRB, 0.56796E+00_JPRB ,&
&  0.53342E+00_JPRB, 0.50807E+00_JPRB, 0.52415E+00_JPRB, 0.56119E+00_JPRB, 0.61196E+00_JPRB, 0.63412E+00_JPRB, 0.63742E+00_JPRB ,&
&  0.64374E+00_JPRB, 0.64640E+00_JPRB, 0.64539E+00_JPRB, 0.62938E+00_JPRB, 0.60210E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.59448E+00_JPRB, 0.58306E+00_JPRB, 0.57124E+00_JPRB, 0.56120E+00_JPRB, 0.54582E+00_JPRB, 0.53115E+00_JPRB, 0.51317E+00_JPRB ,&
&  0.49960E+00_JPRB, 0.48374E+00_JPRB, 0.47400E+00_JPRB, 0.46903E+00_JPRB, 0.46701E+00_JPRB, 0.46053E+00_JPRB, 0.48329E+00_JPRB ,&
&  0.45875E+00_JPRB, 0.51517E+00_JPRB, 0.48744E+00_JPRB, 0.51903E+00_JPRB, 0.62030E+00_JPRB, 0.57794E+00_JPRB, 0.59156E+00_JPRB ,&
&  0.61386E+00_JPRB, 0.66117E+00_JPRB, 0.68438E+00_JPRB, 0.71440E+00_JPRB, 0.73680E+00_JPRB, 0.76714E+00_JPRB, 0.85879E+00_JPRB ,&
&  0.88790E+00_JPRB, 0.83019E+00_JPRB, 0.82420E+00_JPRB, 0.85353E+00_JPRB, 0.91148E+00_JPRB, 0.97442E+00_JPRB, 0.10152E+01_JPRB ,&
&  0.98448E+00_JPRB, 0.95627E+00_JPRB, 0.93178E+00_JPRB, 0.90674E+00_JPRB, 0.88054E+00_JPRB, 0.85505E+00_JPRB, 0.83310E+00_JPRB ,&
&  0.82387E+00_JPRB, 0.83251E+00_JPRB, 0.88764E+00_JPRB, 0.96960E+00_JPRB, 0.10443E+01_JPRB, 0.14698E+01_JPRB, 0.24575E+01_JPRB ,&
&  0.28141E+01_JPRB, 0.27969E+01_JPRB, 0.30150E+01_JPRB, 0.34574E+01_JPRB, 0.34573E+01_JPRB, 0.33888E+01_JPRB, 0.32708E+01_JPRB ,&
&  0.31790E+01_JPRB, 0.30795E+01_JPRB, 0.29195E+01_JPRB, 0.28923E+01_JPRB, 0.27733E+01_JPRB, 0.26303E+01_JPRB, 0.24049E+01_JPRB ,&
&  0.20745E+01_JPRB, 0.18763E+01_JPRB, 0.17958E+01_JPRB, 0.16612E+01_JPRB, 0.15273E+01_JPRB, 0.14232E+01_JPRB, 0.13068E+01_JPRB ,&
&  0.11378E+01_JPRB, 0.95575E+00_JPRB, 0.73892E+00_JPRB, 0.68673E+00_JPRB, 0.57917E+00_JPRB, 0.59175E+00_JPRB, 0.64273E+00_JPRB ,&
&  0.60843E+00_JPRB, 0.55661E+00_JPRB, 0.53799E+00_JPRB, 0.53281E+00_JPRB, 0.53078E+00_JPRB, 0.53863E+00_JPRB, 0.50724E+00_JPRB ,&
&  0.47644E+00_JPRB, 0.47861E+00_JPRB, 0.48449E+00_JPRB, 0.50646E+00_JPRB, 0.54052E+00_JPRB, 0.57285E+00_JPRB, 0.53954E+00_JPRB ,&
&  0.57780E+00_JPRB, 0.59773E+00_JPRB, 0.60849E+00_JPRB, 0.61342E+00_JPRB, 0.60712E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.57354E+00_JPRB, 0.56620E+00_JPRB, 0.55967E+00_JPRB, 0.55231E+00_JPRB, 0.53611E+00_JPRB, 0.50965E+00_JPRB, 0.48894E+00_JPRB ,&
&  0.48447E+00_JPRB, 0.47373E+00_JPRB, 0.46225E+00_JPRB, 0.45182E+00_JPRB, 0.44443E+00_JPRB, 0.43734E+00_JPRB, 0.43281E+00_JPRB ,&
&  0.42438E+00_JPRB, 0.45875E+00_JPRB, 0.46297E+00_JPRB, 0.48391E+00_JPRB, 0.50394E+00_JPRB, 0.49872E+00_JPRB, 0.50802E+00_JPRB ,&
&  0.54515E+00_JPRB, 0.56731E+00_JPRB, 0.57914E+00_JPRB, 0.58042E+00_JPRB, 0.57946E+00_JPRB, 0.60146E+00_JPRB, 0.65851E+00_JPRB ,&
&  0.65823E+00_JPRB, 0.65404E+00_JPRB, 0.65668E+00_JPRB, 0.68727E+00_JPRB, 0.77289E+00_JPRB, 0.87153E+00_JPRB, 0.89765E+00_JPRB ,&
&  0.88017E+00_JPRB, 0.86103E+00_JPRB, 0.83590E+00_JPRB, 0.81273E+00_JPRB, 0.79234E+00_JPRB, 0.77312E+00_JPRB, 0.75387E+00_JPRB ,&
&  0.74736E+00_JPRB, 0.76134E+00_JPRB, 0.80895E+00_JPRB, 0.89186E+00_JPRB, 0.98531E+00_JPRB, 0.12773E+01_JPRB, 0.17510E+01_JPRB ,&
&  0.19498E+01_JPRB, 0.20929E+01_JPRB, 0.22791E+01_JPRB, 0.24936E+01_JPRB, 0.25955E+01_JPRB, 0.25734E+01_JPRB, 0.25183E+01_JPRB ,&
&  0.25120E+01_JPRB, 0.24591E+01_JPRB, 0.23565E+01_JPRB, 0.22414E+01_JPRB, 0.21460E+01_JPRB, 0.21314E+01_JPRB, 0.20555E+01_JPRB ,&
&  0.18890E+01_JPRB, 0.16460E+01_JPRB, 0.16284E+01_JPRB, 0.15658E+01_JPRB, 0.14874E+01_JPRB, 0.13871E+01_JPRB, 0.12765E+01_JPRB ,&
&  0.11387E+01_JPRB, 0.10051E+01_JPRB, 0.85869E+00_JPRB, 0.77033E+00_JPRB, 0.69273E+00_JPRB, 0.68748E+00_JPRB, 0.62849E+00_JPRB ,&
&  0.56888E+00_JPRB, 0.50687E+00_JPRB, 0.50044E+00_JPRB, 0.49719E+00_JPRB, 0.49399E+00_JPRB, 0.47541E+00_JPRB, 0.44778E+00_JPRB ,&
&  0.44135E+00_JPRB, 0.45373E+00_JPRB, 0.45960E+00_JPRB, 0.46828E+00_JPRB, 0.48941E+00_JPRB, 0.52328E+00_JPRB, 0.47397E+00_JPRB ,&
&  0.48676E+00_JPRB, 0.53493E+00_JPRB, 0.55829E+00_JPRB, 0.57091E+00_JPRB, 0.57507E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.53968E+00_JPRB, 0.53670E+00_JPRB, 0.52945E+00_JPRB, 0.52290E+00_JPRB, 0.50795E+00_JPRB, 0.48758E+00_JPRB, 0.46077E+00_JPRB ,&
&  0.47474E+00_JPRB, 0.46609E+00_JPRB, 0.45558E+00_JPRB, 0.44209E+00_JPRB, 0.42856E+00_JPRB, 0.40720E+00_JPRB, 0.37570E+00_JPRB ,&
&  0.37603E+00_JPRB, 0.39659E+00_JPRB, 0.42626E+00_JPRB, 0.44389E+00_JPRB, 0.44766E+00_JPRB, 0.45092E+00_JPRB, 0.46226E+00_JPRB ,&
&  0.47581E+00_JPRB, 0.48742E+00_JPRB, 0.50222E+00_JPRB, 0.51149E+00_JPRB, 0.51318E+00_JPRB, 0.51784E+00_JPRB, 0.54048E+00_JPRB ,&
&  0.52509E+00_JPRB, 0.51774E+00_JPRB, 0.53755E+00_JPRB, 0.52425E+00_JPRB, 0.61915E+00_JPRB, 0.67795E+00_JPRB, 0.70528E+00_JPRB ,&
&  0.69902E+00_JPRB, 0.67675E+00_JPRB, 0.66130E+00_JPRB, 0.66103E+00_JPRB, 0.66136E+00_JPRB, 0.66152E+00_JPRB, 0.66385E+00_JPRB ,&
&  0.67071E+00_JPRB, 0.68748E+00_JPRB, 0.72790E+00_JPRB, 0.79487E+00_JPRB, 0.88131E+00_JPRB, 0.10782E+01_JPRB, 0.12785E+01_JPRB ,&
&  0.14501E+01_JPRB, 0.16892E+01_JPRB, 0.19125E+01_JPRB, 0.19175E+01_JPRB, 0.20686E+01_JPRB, 0.21299E+01_JPRB, 0.20608E+01_JPRB ,&
&  0.20298E+01_JPRB, 0.19353E+01_JPRB, 0.19151E+01_JPRB, 0.18599E+01_JPRB, 0.17970E+01_JPRB, 0.17581E+01_JPRB, 0.17206E+01_JPRB ,&
&  0.16463E+01_JPRB, 0.14640E+01_JPRB, 0.14360E+01_JPRB, 0.13921E+01_JPRB, 0.13313E+01_JPRB, 0.12687E+01_JPRB, 0.12038E+01_JPRB ,&
&  0.11293E+01_JPRB, 0.10365E+01_JPRB, 0.93892E+00_JPRB, 0.85607E+00_JPRB, 0.78865E+00_JPRB, 0.71764E+00_JPRB, 0.65441E+00_JPRB ,&
&  0.59879E+00_JPRB, 0.53246E+00_JPRB, 0.49580E+00_JPRB, 0.48778E+00_JPRB, 0.47040E+00_JPRB, 0.43932E+00_JPRB, 0.41990E+00_JPRB ,&
&  0.41069E+00_JPRB, 0.42477E+00_JPRB, 0.44030E+00_JPRB, 0.44691E+00_JPRB, 0.45791E+00_JPRB, 0.47313E+00_JPRB, 0.45481E+00_JPRB ,&
&  0.44548E+00_JPRB, 0.48579E+00_JPRB, 0.50861E+00_JPRB, 0.52378E+00_JPRB, 0.53455E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.50066E+00_JPRB, 0.50228E+00_JPRB, 0.49552E+00_JPRB, 0.48890E+00_JPRB, 0.48496E+00_JPRB, 0.46520E+00_JPRB, 0.44946E+00_JPRB ,&
&  0.48366E+00_JPRB, 0.44920E+00_JPRB, 0.44466E+00_JPRB, 0.40299E+00_JPRB, 0.38206E+00_JPRB, 0.37484E+00_JPRB, 0.36845E+00_JPRB ,&
&  0.37214E+00_JPRB, 0.38576E+00_JPRB, 0.39843E+00_JPRB, 0.40918E+00_JPRB, 0.42126E+00_JPRB, 0.42531E+00_JPRB, 0.42995E+00_JPRB ,&
&  0.43512E+00_JPRB, 0.44191E+00_JPRB, 0.45092E+00_JPRB, 0.45185E+00_JPRB, 0.45159E+00_JPRB, 0.45270E+00_JPRB, 0.45624E+00_JPRB ,&
&  0.44674E+00_JPRB, 0.44617E+00_JPRB, 0.46927E+00_JPRB, 0.48619E+00_JPRB, 0.52111E+00_JPRB, 0.54337E+00_JPRB, 0.54614E+00_JPRB ,&
&  0.53120E+00_JPRB, 0.50322E+00_JPRB, 0.53037E+00_JPRB, 0.54624E+00_JPRB, 0.56036E+00_JPRB, 0.56980E+00_JPRB, 0.57723E+00_JPRB ,&
&  0.58106E+00_JPRB, 0.58858E+00_JPRB, 0.60587E+00_JPRB, 0.63700E+00_JPRB, 0.68871E+00_JPRB, 0.79629E+00_JPRB, 0.89792E+00_JPRB ,&
&  0.11081E+01_JPRB, 0.10542E+01_JPRB, 0.13112E+01_JPRB, 0.14671E+01_JPRB, 0.16324E+01_JPRB, 0.16610E+01_JPRB, 0.16023E+01_JPRB ,&
&  0.14950E+01_JPRB, 0.14446E+01_JPRB, 0.14309E+01_JPRB, 0.13824E+01_JPRB, 0.13376E+01_JPRB, 0.13065E+01_JPRB, 0.13028E+01_JPRB ,&
&  0.12661E+01_JPRB, 0.11360E+01_JPRB, 0.11393E+01_JPRB, 0.10962E+01_JPRB, 0.10618E+01_JPRB, 0.10395E+01_JPRB, 0.10231E+01_JPRB ,&
&  0.99952E+00_JPRB, 0.95325E+00_JPRB, 0.87980E+00_JPRB, 0.77205E+00_JPRB, 0.72094E+00_JPRB, 0.67472E+00_JPRB, 0.62100E+00_JPRB ,&
&  0.58640E+00_JPRB, 0.55063E+00_JPRB, 0.51765E+00_JPRB, 0.50534E+00_JPRB, 0.46974E+00_JPRB, 0.44276E+00_JPRB, 0.42037E+00_JPRB ,&
&  0.39624E+00_JPRB, 0.39642E+00_JPRB, 0.40758E+00_JPRB, 0.41532E+00_JPRB, 0.41459E+00_JPRB, 0.41570E+00_JPRB, 0.42057E+00_JPRB ,&
&  0.42290E+00_JPRB, 0.44362E+00_JPRB, 0.46873E+00_JPRB, 0.48046E+00_JPRB, 0.49063E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.46951E+00_JPRB, 0.47578E+00_JPRB, 0.47160E+00_JPRB, 0.46635E+00_JPRB, 0.46174E+00_JPRB, 0.44976E+00_JPRB, 0.42551E+00_JPRB ,&
&  0.42489E+00_JPRB, 0.40386E+00_JPRB, 0.38179E+00_JPRB, 0.37611E+00_JPRB, 0.37468E+00_JPRB, 0.37123E+00_JPRB, 0.36379E+00_JPRB ,&
&  0.36596E+00_JPRB, 0.37787E+00_JPRB, 0.39331E+00_JPRB, 0.40239E+00_JPRB, 0.40997E+00_JPRB, 0.41207E+00_JPRB, 0.41623E+00_JPRB ,&
&  0.42067E+00_JPRB, 0.41923E+00_JPRB, 0.41783E+00_JPRB, 0.41439E+00_JPRB, 0.41067E+00_JPRB, 0.41161E+00_JPRB, 0.41200E+00_JPRB ,&
&  0.41786E+00_JPRB, 0.42357E+00_JPRB, 0.43434E+00_JPRB, 0.45317E+00_JPRB, 0.46968E+00_JPRB, 0.48480E+00_JPRB, 0.46424E+00_JPRB ,&
&  0.40560E+00_JPRB, 0.41582E+00_JPRB, 0.45194E+00_JPRB, 0.47145E+00_JPRB, 0.49248E+00_JPRB, 0.50748E+00_JPRB, 0.51655E+00_JPRB ,&
&  0.51821E+00_JPRB, 0.51573E+00_JPRB, 0.51869E+00_JPRB, 0.53480E+00_JPRB, 0.56715E+00_JPRB, 0.62113E+00_JPRB, 0.67804E+00_JPRB ,&
&  0.79627E+00_JPRB, 0.80294E+00_JPRB, 0.83964E+00_JPRB, 0.95557E+00_JPRB, 0.12032E+01_JPRB, 0.12312E+01_JPRB, 0.12269E+01_JPRB ,&
&  0.12058E+01_JPRB, 0.11783E+01_JPRB, 0.11446E+01_JPRB, 0.10741E+01_JPRB, 0.99546E+00_JPRB, 0.93938E+00_JPRB, 0.89788E+00_JPRB ,&
&  0.87653E+00_JPRB, 0.86412E+00_JPRB, 0.83896E+00_JPRB, 0.82513E+00_JPRB, 0.79206E+00_JPRB, 0.77307E+00_JPRB, 0.76506E+00_JPRB ,&
&  0.76222E+00_JPRB, 0.75157E+00_JPRB, 0.71307E+00_JPRB, 0.66347E+00_JPRB, 0.61268E+00_JPRB, 0.57764E+00_JPRB, 0.55238E+00_JPRB ,&
&  0.52881E+00_JPRB, 0.51040E+00_JPRB, 0.49415E+00_JPRB, 0.47667E+00_JPRB, 0.45551E+00_JPRB, 0.43426E+00_JPRB, 0.40923E+00_JPRB ,&
&  0.38701E+00_JPRB, 0.37517E+00_JPRB, 0.37962E+00_JPRB, 0.38289E+00_JPRB, 0.38441E+00_JPRB, 0.38859E+00_JPRB, 0.38574E+00_JPRB ,&
&  0.39294E+00_JPRB, 0.40920E+00_JPRB, 0.41996E+00_JPRB, 0.42859E+00_JPRB, 0.44547E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.44496E+00_JPRB, 0.45313E+00_JPRB, 0.45159E+00_JPRB, 0.45289E+00_JPRB, 0.44469E+00_JPRB, 0.43328E+00_JPRB, 0.42042E+00_JPRB ,&
&  0.41854E+00_JPRB, 0.40927E+00_JPRB, 0.39494E+00_JPRB, 0.38048E+00_JPRB, 0.37289E+00_JPRB, 0.37034E+00_JPRB, 0.37421E+00_JPRB ,&
&  0.38243E+00_JPRB, 0.39266E+00_JPRB, 0.40070E+00_JPRB, 0.40390E+00_JPRB, 0.40247E+00_JPRB, 0.40165E+00_JPRB, 0.40476E+00_JPRB ,&
&  0.40630E+00_JPRB, 0.40322E+00_JPRB, 0.40172E+00_JPRB, 0.39787E+00_JPRB, 0.39613E+00_JPRB, 0.39591E+00_JPRB, 0.39477E+00_JPRB ,&
&  0.39479E+00_JPRB, 0.39572E+00_JPRB, 0.40708E+00_JPRB, 0.42328E+00_JPRB, 0.43498E+00_JPRB, 0.44005E+00_JPRB, 0.40608E+00_JPRB ,&
&  0.37187E+00_JPRB, 0.36578E+00_JPRB, 0.39769E+00_JPRB, 0.41867E+00_JPRB, 0.43854E+00_JPRB, 0.45146E+00_JPRB, 0.45656E+00_JPRB ,&
&  0.44890E+00_JPRB, 0.44364E+00_JPRB, 0.45282E+00_JPRB, 0.47790E+00_JPRB, 0.49755E+00_JPRB, 0.52768E+00_JPRB, 0.56132E+00_JPRB ,&
&  0.60832E+00_JPRB, 0.66749E+00_JPRB, 0.72917E+00_JPRB, 0.71896E+00_JPRB, 0.76408E+00_JPRB, 0.97112E+00_JPRB, 0.10001E+01_JPRB ,&
&  0.96302E+00_JPRB, 0.93087E+00_JPRB, 0.94016E+00_JPRB, 0.87716E+00_JPRB, 0.80877E+00_JPRB, 0.75328E+00_JPRB, 0.70284E+00_JPRB ,&
&  0.65898E+00_JPRB, 0.63721E+00_JPRB, 0.62354E+00_JPRB, 0.62523E+00_JPRB, 0.61421E+00_JPRB, 0.59456E+00_JPRB, 0.58339E+00_JPRB ,&
&  0.57688E+00_JPRB, 0.56264E+00_JPRB, 0.54458E+00_JPRB, 0.51290E+00_JPRB, 0.48909E+00_JPRB, 0.47679E+00_JPRB, 0.47323E+00_JPRB ,&
&  0.47031E+00_JPRB, 0.46767E+00_JPRB, 0.46451E+00_JPRB, 0.45577E+00_JPRB, 0.43418E+00_JPRB, 0.41080E+00_JPRB, 0.39390E+00_JPRB ,&
&  0.38097E+00_JPRB, 0.36914E+00_JPRB, 0.36790E+00_JPRB, 0.36953E+00_JPRB, 0.37409E+00_JPRB, 0.37705E+00_JPRB, 0.37371E+00_JPRB ,&
&  0.37422E+00_JPRB, 0.38048E+00_JPRB, 0.39684E+00_JPRB, 0.41378E+00_JPRB, 0.42854E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.40918E+00_JPRB, 0.42521E+00_JPRB, 0.43764E+00_JPRB, 0.44282E+00_JPRB, 0.44230E+00_JPRB, 0.41915E+00_JPRB, 0.39709E+00_JPRB ,&
&  0.38540E+00_JPRB, 0.38351E+00_JPRB, 0.38585E+00_JPRB, 0.38859E+00_JPRB, 0.39229E+00_JPRB, 0.39703E+00_JPRB, 0.40094E+00_JPRB ,&
&  0.40238E+00_JPRB, 0.40328E+00_JPRB, 0.40543E+00_JPRB, 0.40848E+00_JPRB, 0.40740E+00_JPRB, 0.40535E+00_JPRB, 0.40630E+00_JPRB ,&
&  0.40644E+00_JPRB, 0.40240E+00_JPRB, 0.40027E+00_JPRB, 0.39475E+00_JPRB, 0.39079E+00_JPRB, 0.38780E+00_JPRB, 0.38250E+00_JPRB ,&
&  0.37660E+00_JPRB, 0.37660E+00_JPRB, 0.38607E+00_JPRB, 0.39866E+00_JPRB, 0.40928E+00_JPRB, 0.40402E+00_JPRB, 0.37841E+00_JPRB ,&
&  0.35467E+00_JPRB, 0.34234E+00_JPRB, 0.34630E+00_JPRB, 0.36941E+00_JPRB, 0.39217E+00_JPRB, 0.38941E+00_JPRB, 0.39998E+00_JPRB ,&
&  0.40497E+00_JPRB, 0.40929E+00_JPRB, 0.41619E+00_JPRB, 0.43015E+00_JPRB, 0.44713E+00_JPRB, 0.46597E+00_JPRB, 0.48476E+00_JPRB ,&
&  0.50707E+00_JPRB, 0.53329E+00_JPRB, 0.56490E+00_JPRB, 0.56395E+00_JPRB, 0.55147E+00_JPRB, 0.58622E+00_JPRB, 0.63072E+00_JPRB ,&
&  0.67160E+00_JPRB, 0.66998E+00_JPRB, 0.70158E+00_JPRB, 0.70713E+00_JPRB, 0.69754E+00_JPRB, 0.67119E+00_JPRB, 0.63662E+00_JPRB ,&
&  0.60691E+00_JPRB, 0.57208E+00_JPRB, 0.54397E+00_JPRB, 0.53443E+00_JPRB, 0.52562E+00_JPRB, 0.50967E+00_JPRB, 0.49624E+00_JPRB ,&
&  0.48330E+00_JPRB, 0.46238E+00_JPRB, 0.42994E+00_JPRB, 0.40975E+00_JPRB, 0.41308E+00_JPRB, 0.42959E+00_JPRB, 0.44535E+00_JPRB ,&
&  0.44966E+00_JPRB, 0.44888E+00_JPRB, 0.44860E+00_JPRB, 0.44225E+00_JPRB, 0.42207E+00_JPRB, 0.40705E+00_JPRB, 0.39710E+00_JPRB ,&
&  0.39115E+00_JPRB, 0.38218E+00_JPRB, 0.38177E+00_JPRB, 0.37927E+00_JPRB, 0.37938E+00_JPRB, 0.37911E+00_JPRB, 0.37396E+00_JPRB ,&
&  0.37205E+00_JPRB, 0.37342E+00_JPRB, 0.38052E+00_JPRB, 0.38826E+00_JPRB, 0.39727E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.40758E+00_JPRB, 0.41548E+00_JPRB, 0.42517E+00_JPRB, 0.43220E+00_JPRB, 0.43417E+00_JPRB, 0.42496E+00_JPRB, 0.41638E+00_JPRB ,&
&  0.41133E+00_JPRB, 0.41237E+00_JPRB, 0.41252E+00_JPRB, 0.41261E+00_JPRB, 0.41358E+00_JPRB, 0.41455E+00_JPRB, 0.41574E+00_JPRB ,&
&  0.41706E+00_JPRB, 0.41792E+00_JPRB, 0.41705E+00_JPRB, 0.41164E+00_JPRB, 0.40699E+00_JPRB, 0.40614E+00_JPRB, 0.40828E+00_JPRB ,&
&  0.40951E+00_JPRB, 0.40614E+00_JPRB, 0.40344E+00_JPRB, 0.39601E+00_JPRB, 0.38658E+00_JPRB, 0.37735E+00_JPRB, 0.36923E+00_JPRB ,&
&  0.36846E+00_JPRB, 0.37138E+00_JPRB, 0.37101E+00_JPRB, 0.37671E+00_JPRB, 0.38635E+00_JPRB, 0.37388E+00_JPRB, 0.35560E+00_JPRB ,&
&  0.33917E+00_JPRB, 0.32718E+00_JPRB, 0.32177E+00_JPRB, 0.32230E+00_JPRB, 0.33695E+00_JPRB, 0.35869E+00_JPRB, 0.37002E+00_JPRB ,&
&  0.36082E+00_JPRB, 0.37404E+00_JPRB, 0.39003E+00_JPRB, 0.40077E+00_JPRB, 0.41139E+00_JPRB, 0.42455E+00_JPRB, 0.43682E+00_JPRB ,&
&  0.44846E+00_JPRB, 0.45886E+00_JPRB, 0.46759E+00_JPRB, 0.48371E+00_JPRB, 0.49286E+00_JPRB, 0.48226E+00_JPRB, 0.48796E+00_JPRB ,&
&  0.55505E+00_JPRB, 0.59029E+00_JPRB, 0.62699E+00_JPRB, 0.63546E+00_JPRB, 0.62862E+00_JPRB, 0.60642E+00_JPRB, 0.58376E+00_JPRB ,&
&  0.56386E+00_JPRB, 0.54361E+00_JPRB, 0.52309E+00_JPRB, 0.50921E+00_JPRB, 0.49905E+00_JPRB, 0.48424E+00_JPRB, 0.46754E+00_JPRB ,&
&  0.44446E+00_JPRB, 0.41420E+00_JPRB, 0.41851E+00_JPRB, 0.43024E+00_JPRB, 0.44185E+00_JPRB, 0.45088E+00_JPRB, 0.45164E+00_JPRB ,&
&  0.44995E+00_JPRB, 0.44824E+00_JPRB, 0.44684E+00_JPRB, 0.44312E+00_JPRB, 0.43181E+00_JPRB, 0.42085E+00_JPRB, 0.41441E+00_JPRB ,&
&  0.41074E+00_JPRB, 0.40567E+00_JPRB, 0.40136E+00_JPRB, 0.39775E+00_JPRB, 0.39409E+00_JPRB, 0.39043E+00_JPRB, 0.38290E+00_JPRB ,&
&  0.37770E+00_JPRB, 0.37995E+00_JPRB, 0.38583E+00_JPRB, 0.39487E+00_JPRB, 0.40125E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.40409E+00_JPRB, 0.41110E+00_JPRB, 0.41783E+00_JPRB, 0.42332E+00_JPRB, 0.42612E+00_JPRB, 0.42615E+00_JPRB, 0.42435E+00_JPRB ,&
&  0.42243E+00_JPRB, 0.42216E+00_JPRB, 0.42184E+00_JPRB, 0.42219E+00_JPRB, 0.42258E+00_JPRB, 0.42307E+00_JPRB, 0.42313E+00_JPRB ,&
&  0.42406E+00_JPRB, 0.41540E+00_JPRB, 0.40991E+00_JPRB, 0.40958E+00_JPRB, 0.40928E+00_JPRB, 0.41027E+00_JPRB, 0.41246E+00_JPRB ,&
&  0.40980E+00_JPRB, 0.40236E+00_JPRB, 0.39631E+00_JPRB, 0.38841E+00_JPRB, 0.38028E+00_JPRB, 0.37933E+00_JPRB, 0.37831E+00_JPRB ,&
&  0.37209E+00_JPRB, 0.36663E+00_JPRB, 0.37090E+00_JPRB, 0.37347E+00_JPRB, 0.36319E+00_JPRB, 0.35070E+00_JPRB, 0.32792E+00_JPRB ,&
&  0.32363E+00_JPRB, 0.31464E+00_JPRB, 0.31303E+00_JPRB, 0.31376E+00_JPRB, 0.32047E+00_JPRB, 0.32913E+00_JPRB, 0.34422E+00_JPRB ,&
&  0.35128E+00_JPRB, 0.35661E+00_JPRB, 0.36729E+00_JPRB, 0.37905E+00_JPRB, 0.38730E+00_JPRB, 0.39461E+00_JPRB, 0.39872E+00_JPRB ,&
&  0.40663E+00_JPRB, 0.41588E+00_JPRB, 0.42397E+00_JPRB, 0.43089E+00_JPRB, 0.43915E+00_JPRB, 0.44232E+00_JPRB, 0.44695E+00_JPRB ,&
&  0.46926E+00_JPRB, 0.48642E+00_JPRB, 0.50004E+00_JPRB, 0.51240E+00_JPRB, 0.51919E+00_JPRB, 0.51937E+00_JPRB, 0.50848E+00_JPRB ,&
&  0.49886E+00_JPRB, 0.49214E+00_JPRB, 0.48378E+00_JPRB, 0.47692E+00_JPRB, 0.47107E+00_JPRB, 0.46528E+00_JPRB, 0.45615E+00_JPRB ,&
&  0.43731E+00_JPRB, 0.42045E+00_JPRB, 0.41592E+00_JPRB, 0.42397E+00_JPRB, 0.42272E+00_JPRB, 0.41605E+00_JPRB, 0.41125E+00_JPRB ,&
&  0.40861E+00_JPRB, 0.41035E+00_JPRB, 0.41317E+00_JPRB, 0.41720E+00_JPRB, 0.42096E+00_JPRB, 0.42050E+00_JPRB, 0.42246E+00_JPRB ,&
&  0.42055E+00_JPRB, 0.41660E+00_JPRB, 0.41300E+00_JPRB, 0.40947E+00_JPRB, 0.40613E+00_JPRB, 0.40258E+00_JPRB, 0.39294E+00_JPRB ,&
&  0.38217E+00_JPRB, 0.37684E+00_JPRB, 0.37984E+00_JPRB, 0.38661E+00_JPRB, 0.39580E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.39881E+00_JPRB, 0.40335E+00_JPRB, 0.40824E+00_JPRB, 0.41015E+00_JPRB, 0.41236E+00_JPRB, 0.41222E+00_JPRB, 0.41268E+00_JPRB ,&
&  0.41252E+00_JPRB, 0.41322E+00_JPRB, 0.41354E+00_JPRB, 0.41418E+00_JPRB, 0.41451E+00_JPRB, 0.41520E+00_JPRB, 0.41541E+00_JPRB ,&
&  0.41505E+00_JPRB, 0.41446E+00_JPRB, 0.41287E+00_JPRB, 0.41153E+00_JPRB, 0.41016E+00_JPRB, 0.41111E+00_JPRB, 0.41059E+00_JPRB ,&
&  0.40792E+00_JPRB, 0.40182E+00_JPRB, 0.38954E+00_JPRB, 0.38258E+00_JPRB, 0.38169E+00_JPRB, 0.37577E+00_JPRB, 0.37696E+00_JPRB ,&
&  0.37469E+00_JPRB, 0.37404E+00_JPRB, 0.37087E+00_JPRB, 0.37024E+00_JPRB, 0.36239E+00_JPRB, 0.31720E+00_JPRB, 0.31270E+00_JPRB ,&
&  0.30921E+00_JPRB, 0.30438E+00_JPRB, 0.30409E+00_JPRB, 0.30537E+00_JPRB, 0.30991E+00_JPRB, 0.32300E+00_JPRB, 0.33556E+00_JPRB ,&
&  0.33728E+00_JPRB, 0.35413E+00_JPRB, 0.35879E+00_JPRB, 0.36313E+00_JPRB, 0.37132E+00_JPRB, 0.37860E+00_JPRB, 0.38015E+00_JPRB ,&
&  0.38391E+00_JPRB, 0.38791E+00_JPRB, 0.39357E+00_JPRB, 0.40042E+00_JPRB, 0.40631E+00_JPRB, 0.40963E+00_JPRB, 0.41381E+00_JPRB ,&
&  0.41801E+00_JPRB, 0.42203E+00_JPRB, 0.42641E+00_JPRB, 0.43012E+00_JPRB, 0.43229E+00_JPRB, 0.43402E+00_JPRB, 0.43687E+00_JPRB ,&
&  0.43983E+00_JPRB, 0.44111E+00_JPRB, 0.44309E+00_JPRB, 0.43644E+00_JPRB, 0.43299E+00_JPRB, 0.42864E+00_JPRB, 0.42825E+00_JPRB ,&
&  0.42326E+00_JPRB, 0.42179E+00_JPRB, 0.41643E+00_JPRB, 0.41811E+00_JPRB, 0.41366E+00_JPRB, 0.41083E+00_JPRB, 0.40823E+00_JPRB ,&
&  0.40835E+00_JPRB, 0.40701E+00_JPRB, 0.40386E+00_JPRB, 0.40784E+00_JPRB, 0.41302E+00_JPRB, 0.41891E+00_JPRB, 0.42287E+00_JPRB ,&
&  0.42438E+00_JPRB, 0.42279E+00_JPRB, 0.42053E+00_JPRB, 0.41651E+00_JPRB, 0.41345E+00_JPRB, 0.41063E+00_JPRB, 0.40161E+00_JPRB ,&
&  0.39140E+00_JPRB, 0.38221E+00_JPRB, 0.38111E+00_JPRB, 0.38360E+00_JPRB, 0.39049E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.39175E+00_JPRB, 0.39400E+00_JPRB, 0.39756E+00_JPRB, 0.39808E+00_JPRB, 0.40026E+00_JPRB, 0.40068E+00_JPRB, 0.40151E+00_JPRB ,&
&  0.40137E+00_JPRB, 0.40184E+00_JPRB, 0.40166E+00_JPRB, 0.40201E+00_JPRB, 0.40189E+00_JPRB, 0.40262E+00_JPRB, 0.40289E+00_JPRB ,&
&  0.40352E+00_JPRB, 0.40381E+00_JPRB, 0.40445E+00_JPRB, 0.40456E+00_JPRB, 0.40521E+00_JPRB, 0.40570E+00_JPRB, 0.40471E+00_JPRB ,&
&  0.40231E+00_JPRB, 0.39617E+00_JPRB, 0.38022E+00_JPRB, 0.37648E+00_JPRB, 0.37323E+00_JPRB, 0.37324E+00_JPRB, 0.37233E+00_JPRB ,&
&  0.37150E+00_JPRB, 0.34018E+00_JPRB, 0.34092E+00_JPRB, 0.33294E+00_JPRB, 0.32783E+00_JPRB, 0.32285E+00_JPRB, 0.31546E+00_JPRB ,&
&  0.31276E+00_JPRB, 0.31022E+00_JPRB, 0.31012E+00_JPRB, 0.31392E+00_JPRB, 0.31829E+00_JPRB, 0.32520E+00_JPRB, 0.33349E+00_JPRB ,&
&  0.33503E+00_JPRB, 0.34782E+00_JPRB, 0.35318E+00_JPRB, 0.35446E+00_JPRB, 0.36329E+00_JPRB, 0.37003E+00_JPRB, 0.37194E+00_JPRB ,&
&  0.37456E+00_JPRB, 0.37392E+00_JPRB, 0.37631E+00_JPRB, 0.38184E+00_JPRB, 0.38773E+00_JPRB, 0.38944E+00_JPRB, 0.39076E+00_JPRB ,&
&  0.39142E+00_JPRB, 0.39123E+00_JPRB, 0.39069E+00_JPRB, 0.38870E+00_JPRB, 0.38723E+00_JPRB, 0.38783E+00_JPRB, 0.38995E+00_JPRB ,&
&  0.39295E+00_JPRB, 0.39587E+00_JPRB, 0.40031E+00_JPRB, 0.40186E+00_JPRB, 0.40246E+00_JPRB, 0.40376E+00_JPRB, 0.40470E+00_JPRB ,&
&  0.40485E+00_JPRB, 0.40479E+00_JPRB, 0.40264E+00_JPRB, 0.40160E+00_JPRB, 0.40232E+00_JPRB, 0.40176E+00_JPRB, 0.40249E+00_JPRB ,&
&  0.40592E+00_JPRB, 0.40632E+00_JPRB, 0.40569E+00_JPRB, 0.40644E+00_JPRB, 0.40981E+00_JPRB, 0.41441E+00_JPRB, 0.41800E+00_JPRB ,&
&  0.42048E+00_JPRB, 0.42037E+00_JPRB, 0.41893E+00_JPRB, 0.41491E+00_JPRB, 0.41086E+00_JPRB, 0.40877E+00_JPRB, 0.40355E+00_JPRB ,&
&  0.39848E+00_JPRB, 0.39193E+00_JPRB, 0.38737E+00_JPRB, 0.38503E+00_JPRB, 0.38769E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.38592E+00_JPRB, 0.38752E+00_JPRB, 0.39055E+00_JPRB, 0.39231E+00_JPRB, 0.39505E+00_JPRB, 0.39592E+00_JPRB, 0.39735E+00_JPRB ,&
&  0.39748E+00_JPRB, 0.39795E+00_JPRB, 0.39811E+00_JPRB, 0.39802E+00_JPRB, 0.39787E+00_JPRB, 0.39755E+00_JPRB, 0.39752E+00_JPRB ,&
&  0.39797E+00_JPRB, 0.39801E+00_JPRB, 0.39909E+00_JPRB, 0.39957E+00_JPRB, 0.40011E+00_JPRB, 0.40057E+00_JPRB, 0.39839E+00_JPRB ,&
&  0.39547E+00_JPRB, 0.39296E+00_JPRB, 0.37074E+00_JPRB, 0.36842E+00_JPRB, 0.36383E+00_JPRB, 0.36154E+00_JPRB, 0.36199E+00_JPRB ,&
&  0.35960E+00_JPRB, 0.35675E+00_JPRB, 0.34819E+00_JPRB, 0.33892E+00_JPRB, 0.33116E+00_JPRB, 0.32159E+00_JPRB, 0.31932E+00_JPRB ,&
&  0.31720E+00_JPRB, 0.31914E+00_JPRB, 0.32159E+00_JPRB, 0.32388E+00_JPRB, 0.32993E+00_JPRB, 0.33419E+00_JPRB, 0.34095E+00_JPRB ,&
&  0.34922E+00_JPRB, 0.35528E+00_JPRB, 0.35544E+00_JPRB, 0.35018E+00_JPRB, 0.35895E+00_JPRB, 0.36366E+00_JPRB, 0.36843E+00_JPRB ,&
&  0.37104E+00_JPRB, 0.37163E+00_JPRB, 0.37382E+00_JPRB, 0.37473E+00_JPRB, 0.37539E+00_JPRB, 0.37720E+00_JPRB, 0.37705E+00_JPRB ,&
&  0.37740E+00_JPRB, 0.37573E+00_JPRB, 0.37318E+00_JPRB, 0.36958E+00_JPRB, 0.36564E+00_JPRB, 0.36333E+00_JPRB, 0.36270E+00_JPRB ,&
&  0.36347E+00_JPRB, 0.36575E+00_JPRB, 0.36865E+00_JPRB, 0.37230E+00_JPRB, 0.37539E+00_JPRB, 0.37804E+00_JPRB, 0.38063E+00_JPRB ,&
&  0.38258E+00_JPRB, 0.38404E+00_JPRB, 0.38746E+00_JPRB, 0.38601E+00_JPRB, 0.38711E+00_JPRB, 0.39187E+00_JPRB, 0.39172E+00_JPRB ,&
&  0.39413E+00_JPRB, 0.39800E+00_JPRB, 0.40019E+00_JPRB, 0.39986E+00_JPRB, 0.40064E+00_JPRB, 0.40285E+00_JPRB, 0.40543E+00_JPRB ,&
&  0.40737E+00_JPRB, 0.40756E+00_JPRB, 0.40630E+00_JPRB, 0.40425E+00_JPRB, 0.40216E+00_JPRB, 0.40015E+00_JPRB, 0.39685E+00_JPRB ,&
&  0.39489E+00_JPRB, 0.39251E+00_JPRB, 0.38960E+00_JPRB, 0.38601E+00_JPRB, 0.38537E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.38565E+00_JPRB, 0.38512E+00_JPRB, 0.38537E+00_JPRB, 0.38624E+00_JPRB, 0.38763E+00_JPRB, 0.38864E+00_JPRB, 0.38994E+00_JPRB ,&
&  0.39081E+00_JPRB, 0.39137E+00_JPRB, 0.39116E+00_JPRB, 0.39149E+00_JPRB, 0.39147E+00_JPRB, 0.39177E+00_JPRB, 0.39142E+00_JPRB ,&
&  0.39140E+00_JPRB, 0.39082E+00_JPRB, 0.39167E+00_JPRB, 0.39212E+00_JPRB, 0.39288E+00_JPRB, 0.39380E+00_JPRB, 0.39209E+00_JPRB ,&
&  0.39042E+00_JPRB, 0.38935E+00_JPRB, 0.38580E+00_JPRB, 0.38248E+00_JPRB, 0.38010E+00_JPRB, 0.36142E+00_JPRB, 0.35561E+00_JPRB ,&
&  0.36024E+00_JPRB, 0.36314E+00_JPRB, 0.36648E+00_JPRB, 0.36914E+00_JPRB, 0.36396E+00_JPRB, 0.35599E+00_JPRB, 0.35161E+00_JPRB ,&
&  0.33647E+00_JPRB, 0.33889E+00_JPRB, 0.34059E+00_JPRB, 0.34211E+00_JPRB, 0.34578E+00_JPRB, 0.34685E+00_JPRB, 0.35033E+00_JPRB ,&
&  0.36007E+00_JPRB, 0.36234E+00_JPRB, 0.36521E+00_JPRB, 0.36682E+00_JPRB, 0.36696E+00_JPRB, 0.36761E+00_JPRB, 0.37077E+00_JPRB ,&
&  0.37199E+00_JPRB, 0.37312E+00_JPRB, 0.37377E+00_JPRB, 0.37343E+00_JPRB, 0.37345E+00_JPRB, 0.37268E+00_JPRB, 0.37139E+00_JPRB ,&
&  0.36936E+00_JPRB, 0.36813E+00_JPRB, 0.36669E+00_JPRB, 0.36488E+00_JPRB, 0.36354E+00_JPRB, 0.36148E+00_JPRB, 0.35998E+00_JPRB ,&
&  0.35929E+00_JPRB, 0.35938E+00_JPRB, 0.35998E+00_JPRB, 0.36107E+00_JPRB, 0.36267E+00_JPRB, 0.36483E+00_JPRB, 0.36706E+00_JPRB ,&
&  0.36946E+00_JPRB, 0.37177E+00_JPRB, 0.37375E+00_JPRB, 0.37554E+00_JPRB, 0.37718E+00_JPRB, 0.37889E+00_JPRB, 0.38052E+00_JPRB ,&
&  0.38276E+00_JPRB, 0.38547E+00_JPRB, 0.38713E+00_JPRB, 0.38738E+00_JPRB, 0.38810E+00_JPRB, 0.38890E+00_JPRB, 0.39010E+00_JPRB ,&
&  0.39123E+00_JPRB, 0.39263E+00_JPRB, 0.39313E+00_JPRB, 0.39306E+00_JPRB, 0.39262E+00_JPRB, 0.39253E+00_JPRB, 0.39202E+00_JPRB ,&
&  0.39214E+00_JPRB, 0.39247E+00_JPRB, 0.39205E+00_JPRB, 0.39008E+00_JPRB, 0.38766E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.38953E+00_JPRB, 0.38858E+00_JPRB, 0.38776E+00_JPRB, 0.38724E+00_JPRB, 0.38672E+00_JPRB, 0.38663E+00_JPRB, 0.38668E+00_JPRB ,&
&  0.38682E+00_JPRB, 0.38675E+00_JPRB, 0.38640E+00_JPRB, 0.38634E+00_JPRB, 0.38659E+00_JPRB, 0.38699E+00_JPRB, 0.38680E+00_JPRB ,&
&  0.38676E+00_JPRB, 0.38665E+00_JPRB, 0.38692E+00_JPRB, 0.38680E+00_JPRB, 0.38661E+00_JPRB, 0.38628E+00_JPRB, 0.38615E+00_JPRB ,&
&  0.38604E+00_JPRB, 0.38563E+00_JPRB, 0.38487E+00_JPRB, 0.38331E+00_JPRB, 0.38177E+00_JPRB, 0.37998E+00_JPRB, 0.37805E+00_JPRB ,&
&  0.37582E+00_JPRB, 0.37398E+00_JPRB, 0.37241E+00_JPRB, 0.37093E+00_JPRB, 0.37206E+00_JPRB, 0.37261E+00_JPRB, 0.37285E+00_JPRB ,&
&  0.37263E+00_JPRB, 0.37100E+00_JPRB, 0.37052E+00_JPRB, 0.37016E+00_JPRB, 0.36992E+00_JPRB, 0.37126E+00_JPRB, 0.37217E+00_JPRB ,&
&  0.37281E+00_JPRB, 0.37333E+00_JPRB, 0.37479E+00_JPRB, 0.37552E+00_JPRB, 0.37592E+00_JPRB, 0.37577E+00_JPRB, 0.37523E+00_JPRB ,&
&  0.37538E+00_JPRB, 0.37552E+00_JPRB, 0.37542E+00_JPRB, 0.37496E+00_JPRB, 0.37472E+00_JPRB, 0.37427E+00_JPRB, 0.37325E+00_JPRB ,&
&  0.37210E+00_JPRB, 0.37133E+00_JPRB, 0.37037E+00_JPRB, 0.36920E+00_JPRB, 0.36795E+00_JPRB, 0.36713E+00_JPRB, 0.36619E+00_JPRB ,&
&  0.36533E+00_JPRB, 0.36470E+00_JPRB, 0.36425E+00_JPRB, 0.36379E+00_JPRB, 0.36334E+00_JPRB, 0.36336E+00_JPRB, 0.36408E+00_JPRB ,&
&  0.36504E+00_JPRB, 0.36633E+00_JPRB, 0.36757E+00_JPRB, 0.36902E+00_JPRB, 0.37021E+00_JPRB, 0.37141E+00_JPRB, 0.37311E+00_JPRB ,&
&  0.37504E+00_JPRB, 0.37659E+00_JPRB, 0.37798E+00_JPRB, 0.37943E+00_JPRB, 0.38108E+00_JPRB, 0.38204E+00_JPRB, 0.38274E+00_JPRB ,&
&  0.38341E+00_JPRB, 0.38485E+00_JPRB, 0.38618E+00_JPRB, 0.38751E+00_JPRB, 0.38870E+00_JPRB, 0.38968E+00_JPRB, 0.39029E+00_JPRB ,&
&  0.39078E+00_JPRB, 0.39141E+00_JPRB, 0.39177E+00_JPRB, 0.39137E+00_JPRB, 0.39069E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.38970E+00_JPRB, 0.38966E+00_JPRB, 0.38949E+00_JPRB, 0.38931E+00_JPRB, 0.38905E+00_JPRB, 0.38885E+00_JPRB, 0.38858E+00_JPRB ,&
&  0.38841E+00_JPRB, 0.38821E+00_JPRB, 0.38816E+00_JPRB, 0.38803E+00_JPRB, 0.38797E+00_JPRB, 0.38784E+00_JPRB, 0.38769E+00_JPRB ,&
&  0.38750E+00_JPRB, 0.38743E+00_JPRB, 0.38719E+00_JPRB, 0.38699E+00_JPRB, 0.38665E+00_JPRB, 0.38641E+00_JPRB, 0.38604E+00_JPRB ,&
&  0.38578E+00_JPRB, 0.38535E+00_JPRB, 0.38504E+00_JPRB, 0.38441E+00_JPRB, 0.38399E+00_JPRB, 0.38348E+00_JPRB, 0.38306E+00_JPRB ,&
&  0.38260E+00_JPRB, 0.38231E+00_JPRB, 0.38198E+00_JPRB, 0.38157E+00_JPRB, 0.38129E+00_JPRB, 0.38140E+00_JPRB, 0.38151E+00_JPRB ,&
&  0.38158E+00_JPRB, 0.38170E+00_JPRB, 0.38199E+00_JPRB, 0.38211E+00_JPRB, 0.38189E+00_JPRB, 0.38172E+00_JPRB, 0.38163E+00_JPRB ,&
&  0.38151E+00_JPRB, 0.38125E+00_JPRB, 0.38099E+00_JPRB, 0.38083E+00_JPRB, 0.38055E+00_JPRB, 0.38014E+00_JPRB, 0.37981E+00_JPRB ,&
&  0.37955E+00_JPRB, 0.37932E+00_JPRB, 0.37901E+00_JPRB, 0.37855E+00_JPRB, 0.37814E+00_JPRB, 0.37769E+00_JPRB, 0.37717E+00_JPRB ,&
&  0.37661E+00_JPRB, 0.37603E+00_JPRB, 0.37540E+00_JPRB, 0.37471E+00_JPRB, 0.37400E+00_JPRB, 0.37347E+00_JPRB, 0.37307E+00_JPRB ,&
&  0.37277E+00_JPRB, 0.37263E+00_JPRB, 0.37256E+00_JPRB, 0.37253E+00_JPRB, 0.37252E+00_JPRB, 0.37264E+00_JPRB, 0.37285E+00_JPRB ,&
&  0.37309E+00_JPRB, 0.37338E+00_JPRB, 0.37382E+00_JPRB, 0.37453E+00_JPRB, 0.37518E+00_JPRB, 0.37584E+00_JPRB, 0.37673E+00_JPRB ,&
&  0.37757E+00_JPRB, 0.37825E+00_JPRB, 0.37878E+00_JPRB, 0.37948E+00_JPRB, 0.38051E+00_JPRB, 0.38128E+00_JPRB, 0.38191E+00_JPRB ,&
&  0.38252E+00_JPRB, 0.38322E+00_JPRB, 0.38388E+00_JPRB, 0.38461E+00_JPRB, 0.38524E+00_JPRB, 0.38593E+00_JPRB, 0.38665E+00_JPRB ,&
&  0.38732E+00_JPRB, 0.38803E+00_JPRB, 0.38869E+00_JPRB, 0.38921E+00_JPRB, 0.38957E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB, 0.38402E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB, 0.15313E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.16620E+00_JPRB, 0.16650E+00_JPRB, 0.16706E+00_JPRB, 0.16774E+00_JPRB, 0.16847E+00_JPRB, 0.16933E+00_JPRB, 0.16968E+00_JPRB ,&
&  0.16984E+00_JPRB, 0.16985E+00_JPRB, 0.17018E+00_JPRB, 0.17058E+00_JPRB, 0.17083E+00_JPRB, 0.17083E+00_JPRB, 0.17045E+00_JPRB ,&
&  0.16981E+00_JPRB, 0.16907E+00_JPRB, 0.16809E+00_JPRB, 0.16763E+00_JPRB, 0.16728E+00_JPRB, 0.16689E+00_JPRB, 0.16612E+00_JPRB ,&
&  0.16545E+00_JPRB, 0.16425E+00_JPRB, 0.16258E+00_JPRB, 0.16114E+00_JPRB, 0.16077E+00_JPRB, 0.16084E+00_JPRB, 0.16123E+00_JPRB ,&
&  0.16154E+00_JPRB, 0.16170E+00_JPRB, 0.16171E+00_JPRB, 0.16177E+00_JPRB, 0.16191E+00_JPRB, 0.16174E+00_JPRB, 0.16146E+00_JPRB ,&
&  0.16113E+00_JPRB, 0.16061E+00_JPRB, 0.16013E+00_JPRB, 0.15969E+00_JPRB, 0.15927E+00_JPRB, 0.15881E+00_JPRB, 0.15827E+00_JPRB ,&
&  0.15765E+00_JPRB, 0.15689E+00_JPRB, 0.15601E+00_JPRB, 0.15529E+00_JPRB, 0.15455E+00_JPRB, 0.15397E+00_JPRB, 0.15327E+00_JPRB ,&
&  0.15268E+00_JPRB, 0.15214E+00_JPRB, 0.15160E+00_JPRB, 0.15108E+00_JPRB, 0.15059E+00_JPRB, 0.15012E+00_JPRB, 0.14964E+00_JPRB ,&
&  0.14919E+00_JPRB, 0.14882E+00_JPRB, 0.14840E+00_JPRB, 0.14800E+00_JPRB, 0.14774E+00_JPRB, 0.14752E+00_JPRB, 0.14733E+00_JPRB ,&
&  0.14716E+00_JPRB, 0.14710E+00_JPRB, 0.14707E+00_JPRB, 0.14704E+00_JPRB, 0.14702E+00_JPRB, 0.14712E+00_JPRB, 0.14723E+00_JPRB ,&
&  0.14738E+00_JPRB, 0.14749E+00_JPRB, 0.14763E+00_JPRB, 0.14782E+00_JPRB, 0.14803E+00_JPRB, 0.14826E+00_JPRB, 0.14859E+00_JPRB ,&
&  0.14899E+00_JPRB, 0.14947E+00_JPRB, 0.14996E+00_JPRB, 0.15032E+00_JPRB, 0.15097E+00_JPRB, 0.15168E+00_JPRB, 0.15251E+00_JPRB ,&
&  0.15342E+00_JPRB, 0.15447E+00_JPRB, 0.15558E+00_JPRB, 0.15680E+00_JPRB, 0.15803E+00_JPRB, 0.15885E+00_JPRB, 0.15968E+00_JPRB ,&
&  0.16056E+00_JPRB, 0.16157E+00_JPRB, 0.16255E+00_JPRB, 0.16355E+00_JPRB, 0.16479E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.18811E+00_JPRB, 0.19515E+00_JPRB, 0.20238E+00_JPRB, 0.21262E+00_JPRB, 0.22100E+00_JPRB, 0.25613E+00_JPRB, 0.25101E+00_JPRB ,&
&  0.24732E+00_JPRB, 0.23280E+00_JPRB, 0.23090E+00_JPRB, 0.22711E+00_JPRB, 0.22252E+00_JPRB, 0.21748E+00_JPRB, 0.21248E+00_JPRB ,&
&  0.20779E+00_JPRB, 0.20232E+00_JPRB, 0.19749E+00_JPRB, 0.19373E+00_JPRB, 0.19035E+00_JPRB, 0.18705E+00_JPRB, 0.18415E+00_JPRB ,&
&  0.18282E+00_JPRB, 0.18256E+00_JPRB, 0.18171E+00_JPRB, 0.17805E+00_JPRB, 0.17477E+00_JPRB, 0.17271E+00_JPRB, 0.17200E+00_JPRB ,&
&  0.17186E+00_JPRB, 0.17318E+00_JPRB, 0.17384E+00_JPRB, 0.17428E+00_JPRB, 0.17505E+00_JPRB, 0.17422E+00_JPRB, 0.17339E+00_JPRB ,&
&  0.17228E+00_JPRB, 0.17100E+00_JPRB, 0.16963E+00_JPRB, 0.16848E+00_JPRB, 0.16730E+00_JPRB, 0.16559E+00_JPRB, 0.16407E+00_JPRB ,&
&  0.16252E+00_JPRB, 0.16065E+00_JPRB, 0.15832E+00_JPRB, 0.15676E+00_JPRB, 0.15499E+00_JPRB, 0.15347E+00_JPRB, 0.15197E+00_JPRB ,&
&  0.15087E+00_JPRB, 0.14985E+00_JPRB, 0.14862E+00_JPRB, 0.14712E+00_JPRB, 0.14561E+00_JPRB, 0.14426E+00_JPRB, 0.14296E+00_JPRB ,&
&  0.14223E+00_JPRB, 0.14149E+00_JPRB, 0.14076E+00_JPRB, 0.13997E+00_JPRB, 0.13957E+00_JPRB, 0.13918E+00_JPRB, 0.13884E+00_JPRB ,&
&  0.13854E+00_JPRB, 0.13842E+00_JPRB, 0.13847E+00_JPRB, 0.13857E+00_JPRB, 0.13867E+00_JPRB, 0.13903E+00_JPRB, 0.13945E+00_JPRB ,&
&  0.14002E+00_JPRB, 0.14072E+00_JPRB, 0.14166E+00_JPRB, 0.14262E+00_JPRB, 0.14350E+00_JPRB, 0.14424E+00_JPRB, 0.14521E+00_JPRB ,&
&  0.14676E+00_JPRB, 0.14856E+00_JPRB, 0.15052E+00_JPRB, 0.15210E+00_JPRB, 0.15418E+00_JPRB, 0.15606E+00_JPRB, 0.15793E+00_JPRB ,&
&  0.15977E+00_JPRB, 0.16188E+00_JPRB, 0.16415E+00_JPRB, 0.16653E+00_JPRB, 0.16919E+00_JPRB, 0.17179E+00_JPRB, 0.17419E+00_JPRB ,&
&  0.17664E+00_JPRB, 0.17992E+00_JPRB, 0.18204E+00_JPRB, 0.18346E+00_JPRB, 0.18534E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.26080E+00_JPRB, 0.26544E+00_JPRB, 0.26902E+00_JPRB, 0.27129E+00_JPRB, 0.27335E+00_JPRB, 0.27488E+00_JPRB, 0.27540E+00_JPRB ,&
&  0.26901E+00_JPRB, 0.26398E+00_JPRB, 0.25712E+00_JPRB, 0.24897E+00_JPRB, 0.24157E+00_JPRB, 0.23530E+00_JPRB, 0.22765E+00_JPRB ,&
&  0.22140E+00_JPRB, 0.21550E+00_JPRB, 0.20913E+00_JPRB, 0.20253E+00_JPRB, 0.19557E+00_JPRB, 0.18858E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18333E+00_JPRB, 0.18397E+00_JPRB, 0.18539E+00_JPRB, 0.18692E+00_JPRB, 0.18785E+00_JPRB, 0.18854E+00_JPRB, 0.18744E+00_JPRB ,&
&  0.18392E+00_JPRB, 0.18433E+00_JPRB, 0.18489E+00_JPRB, 0.18522E+00_JPRB, 0.18646E+00_JPRB, 0.18757E+00_JPRB, 0.18728E+00_JPRB ,&
&  0.18562E+00_JPRB, 0.18335E+00_JPRB, 0.18022E+00_JPRB, 0.17734E+00_JPRB, 0.17605E+00_JPRB, 0.17379E+00_JPRB, 0.17122E+00_JPRB ,&
&  0.16891E+00_JPRB, 0.16508E+00_JPRB, 0.16142E+00_JPRB, 0.15884E+00_JPRB, 0.15568E+00_JPRB, 0.15347E+00_JPRB, 0.15062E+00_JPRB ,&
&  0.14829E+00_JPRB, 0.14611E+00_JPRB, 0.14372E+00_JPRB, 0.14182E+00_JPRB, 0.14002E+00_JPRB, 0.13868E+00_JPRB, 0.13736E+00_JPRB ,&
&  0.13657E+00_JPRB, 0.13587E+00_JPRB, 0.13543E+00_JPRB, 0.13495E+00_JPRB, 0.13470E+00_JPRB, 0.13444E+00_JPRB, 0.13424E+00_JPRB ,&
&  0.13413E+00_JPRB, 0.13443E+00_JPRB, 0.13465E+00_JPRB, 0.13504E+00_JPRB, 0.13549E+00_JPRB, 0.13582E+00_JPRB, 0.13635E+00_JPRB ,&
&  0.13733E+00_JPRB, 0.13838E+00_JPRB, 0.13914E+00_JPRB, 0.14010E+00_JPRB, 0.14116E+00_JPRB, 0.14247E+00_JPRB, 0.14419E+00_JPRB ,&
&  0.14601E+00_JPRB, 0.14803E+00_JPRB, 0.15029E+00_JPRB, 0.15254E+00_JPRB, 0.15536E+00_JPRB, 0.15826E+00_JPRB, 0.16129E+00_JPRB ,&
&  0.16485E+00_JPRB, 0.16866E+00_JPRB, 0.17252E+00_JPRB, 0.17676E+00_JPRB, 0.18286E+00_JPRB, 0.18895E+00_JPRB, 0.19656E+00_JPRB ,&
&  0.20416E+00_JPRB, 0.21109E+00_JPRB, 0.24250E+00_JPRB, 0.24735E+00_JPRB, 0.25397E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.26718E+00_JPRB, 0.27780E+00_JPRB, 0.28660E+00_JPRB, 0.29207E+00_JPRB, 0.29787E+00_JPRB, 0.30022E+00_JPRB, 0.30898E+00_JPRB ,&
&  0.30350E+00_JPRB, 0.28522E+00_JPRB, 0.27404E+00_JPRB, 0.25773E+00_JPRB, 0.23745E+00_JPRB, 0.22106E+00_JPRB, 0.21299E+00_JPRB ,&
&  0.20828E+00_JPRB, 0.20467E+00_JPRB, 0.20022E+00_JPRB, 0.19531E+00_JPRB, 0.19030E+00_JPRB, 0.18662E+00_JPRB, 0.18412E+00_JPRB ,&
&  0.18189E+00_JPRB, 0.18219E+00_JPRB, 0.18437E+00_JPRB, 0.18833E+00_JPRB, 0.19391E+00_JPRB, 0.19818E+00_JPRB, 0.20141E+00_JPRB ,&
&  0.20422E+00_JPRB, 0.19618E+00_JPRB, 0.19689E+00_JPRB, 0.19808E+00_JPRB, 0.19650E+00_JPRB, 0.19854E+00_JPRB, 0.20162E+00_JPRB ,&
&  0.19999E+00_JPRB, 0.19498E+00_JPRB, 0.19802E+00_JPRB, 0.19437E+00_JPRB, 0.18626E+00_JPRB, 0.18387E+00_JPRB, 0.17813E+00_JPRB ,&
&  0.17440E+00_JPRB, 0.16725E+00_JPRB, 0.16224E+00_JPRB, 0.15897E+00_JPRB, 0.15436E+00_JPRB, 0.15236E+00_JPRB, 0.14929E+00_JPRB ,&
&  0.14602E+00_JPRB, 0.14359E+00_JPRB, 0.14073E+00_JPRB, 0.13918E+00_JPRB, 0.13789E+00_JPRB, 0.13678E+00_JPRB, 0.13592E+00_JPRB ,&
&  0.13545E+00_JPRB, 0.13502E+00_JPRB, 0.13506E+00_JPRB, 0.13511E+00_JPRB, 0.13520E+00_JPRB, 0.13556E+00_JPRB, 0.13584E+00_JPRB ,&
&  0.13646E+00_JPRB, 0.13713E+00_JPRB, 0.13767E+00_JPRB, 0.13849E+00_JPRB, 0.13890E+00_JPRB, 0.13941E+00_JPRB, 0.13982E+00_JPRB ,&
&  0.14028E+00_JPRB, 0.14096E+00_JPRB, 0.14140E+00_JPRB, 0.14212E+00_JPRB, 0.14296E+00_JPRB, 0.14444E+00_JPRB, 0.14592E+00_JPRB ,&
&  0.14770E+00_JPRB, 0.14968E+00_JPRB, 0.15184E+00_JPRB, 0.15410E+00_JPRB, 0.15688E+00_JPRB, 0.15951E+00_JPRB, 0.16290E+00_JPRB ,&
&  0.16656E+00_JPRB, 0.17003E+00_JPRB, 0.17386E+00_JPRB, 0.17888E+00_JPRB, 0.18537E+00_JPRB, 0.19471E+00_JPRB, 0.20386E+00_JPRB ,&
&  0.21441E+00_JPRB, 0.24139E+00_JPRB, 0.24778E+00_JPRB, 0.25069E+00_JPRB, 0.25886E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.29355E+00_JPRB, 0.30281E+00_JPRB, 0.31686E+00_JPRB, 0.32581E+00_JPRB, 0.33313E+00_JPRB, 0.32899E+00_JPRB, 0.32512E+00_JPRB ,&
&  0.32114E+00_JPRB, 0.30685E+00_JPRB, 0.28979E+00_JPRB, 0.26058E+00_JPRB, 0.23593E+00_JPRB, 0.21704E+00_JPRB, 0.20685E+00_JPRB ,&
&  0.20101E+00_JPRB, 0.19921E+00_JPRB, 0.19769E+00_JPRB, 0.19749E+00_JPRB, 0.19670E+00_JPRB, 0.19856E+00_JPRB, 0.20014E+00_JPRB ,&
&  0.19990E+00_JPRB, 0.19871E+00_JPRB, 0.19796E+00_JPRB, 0.20099E+00_JPRB, 0.20679E+00_JPRB, 0.21140E+00_JPRB, 0.21578E+00_JPRB ,&
&  0.21911E+00_JPRB, 0.21671E+00_JPRB, 0.21754E+00_JPRB, 0.21535E+00_JPRB, 0.21261E+00_JPRB, 0.21220E+00_JPRB, 0.21265E+00_JPRB ,&
&  0.21310E+00_JPRB, 0.21392E+00_JPRB, 0.21342E+00_JPRB, 0.21083E+00_JPRB, 0.19971E+00_JPRB, 0.19382E+00_JPRB, 0.18645E+00_JPRB ,&
&  0.17847E+00_JPRB, 0.16925E+00_JPRB, 0.16201E+00_JPRB, 0.15845E+00_JPRB, 0.15454E+00_JPRB, 0.15286E+00_JPRB, 0.14983E+00_JPRB ,&
&  0.14638E+00_JPRB, 0.14305E+00_JPRB, 0.14064E+00_JPRB, 0.13941E+00_JPRB, 0.13895E+00_JPRB, 0.13812E+00_JPRB, 0.13778E+00_JPRB ,&
&  0.13796E+00_JPRB, 0.13822E+00_JPRB, 0.13889E+00_JPRB, 0.13962E+00_JPRB, 0.14047E+00_JPRB, 0.14130E+00_JPRB, 0.14235E+00_JPRB ,&
&  0.14391E+00_JPRB, 0.14570E+00_JPRB, 0.14653E+00_JPRB, 0.14719E+00_JPRB, 0.14699E+00_JPRB, 0.14666E+00_JPRB, 0.14609E+00_JPRB ,&
&  0.14564E+00_JPRB, 0.14553E+00_JPRB, 0.14554E+00_JPRB, 0.14552E+00_JPRB, 0.14600E+00_JPRB, 0.14689E+00_JPRB, 0.14789E+00_JPRB ,&
&  0.14942E+00_JPRB, 0.15082E+00_JPRB, 0.15239E+00_JPRB, 0.15414E+00_JPRB, 0.15622E+00_JPRB, 0.15862E+00_JPRB, 0.16134E+00_JPRB ,&
&  0.16392E+00_JPRB, 0.16709E+00_JPRB, 0.16977E+00_JPRB, 0.17190E+00_JPRB, 0.17634E+00_JPRB, 0.18312E+00_JPRB, 0.19649E+00_JPRB ,&
&  0.21262E+00_JPRB, 0.25366E+00_JPRB, 0.29673E+00_JPRB, 0.28111E+00_JPRB, 0.28566E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.29247E+00_JPRB, 0.30435E+00_JPRB, 0.31967E+00_JPRB, 0.32900E+00_JPRB, 0.33402E+00_JPRB, 0.33651E+00_JPRB, 0.33518E+00_JPRB ,&
&  0.33176E+00_JPRB, 0.33138E+00_JPRB, 0.34260E+00_JPRB, 0.32525E+00_JPRB, 0.30734E+00_JPRB, 0.27368E+00_JPRB, 0.25727E+00_JPRB ,&
&  0.24524E+00_JPRB, 0.24321E+00_JPRB, 0.24318E+00_JPRB, 0.24800E+00_JPRB, 0.25211E+00_JPRB, 0.25540E+00_JPRB, 0.25205E+00_JPRB ,&
&  0.24707E+00_JPRB, 0.23804E+00_JPRB, 0.23047E+00_JPRB, 0.22326E+00_JPRB, 0.22323E+00_JPRB, 0.22217E+00_JPRB, 0.21961E+00_JPRB ,&
&  0.21655E+00_JPRB, 0.21447E+00_JPRB, 0.21416E+00_JPRB, 0.21390E+00_JPRB, 0.22550E+00_JPRB, 0.22513E+00_JPRB, 0.22462E+00_JPRB ,&
&  0.22401E+00_JPRB, 0.22393E+00_JPRB, 0.22299E+00_JPRB, 0.22129E+00_JPRB, 0.21672E+00_JPRB, 0.20564E+00_JPRB, 0.20563E+00_JPRB ,&
&  0.19565E+00_JPRB, 0.18493E+00_JPRB, 0.17481E+00_JPRB, 0.16781E+00_JPRB, 0.16353E+00_JPRB, 0.16139E+00_JPRB, 0.15681E+00_JPRB ,&
&  0.15194E+00_JPRB, 0.14648E+00_JPRB, 0.14375E+00_JPRB, 0.14328E+00_JPRB, 0.14372E+00_JPRB, 0.14425E+00_JPRB, 0.14474E+00_JPRB ,&
&  0.14582E+00_JPRB, 0.14639E+00_JPRB, 0.14669E+00_JPRB, 0.14726E+00_JPRB, 0.14783E+00_JPRB, 0.14876E+00_JPRB, 0.15035E+00_JPRB ,&
&  0.15324E+00_JPRB, 0.15646E+00_JPRB, 0.15937E+00_JPRB, 0.16147E+00_JPRB, 0.16051E+00_JPRB, 0.15769E+00_JPRB, 0.15382E+00_JPRB ,&
&  0.15108E+00_JPRB, 0.15028E+00_JPRB, 0.14955E+00_JPRB, 0.14961E+00_JPRB, 0.14991E+00_JPRB, 0.15040E+00_JPRB, 0.15107E+00_JPRB ,&
&  0.15187E+00_JPRB, 0.15255E+00_JPRB, 0.15338E+00_JPRB, 0.15442E+00_JPRB, 0.15595E+00_JPRB, 0.15734E+00_JPRB, 0.15930E+00_JPRB ,&
&  0.16223E+00_JPRB, 0.16542E+00_JPRB, 0.16996E+00_JPRB, 0.17401E+00_JPRB, 0.17914E+00_JPRB, 0.18704E+00_JPRB, 0.20007E+00_JPRB ,&
&  0.21436E+00_JPRB, 0.24941E+00_JPRB, 0.26589E+00_JPRB, 0.27527E+00_JPRB, 0.28221E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.28983E+00_JPRB, 0.29857E+00_JPRB, 0.31075E+00_JPRB, 0.32108E+00_JPRB, 0.32988E+00_JPRB, 0.33831E+00_JPRB, 0.34546E+00_JPRB ,&
&  0.34904E+00_JPRB, 0.35112E+00_JPRB, 0.35183E+00_JPRB, 0.35523E+00_JPRB, 0.35233E+00_JPRB, 0.34599E+00_JPRB, 0.34062E+00_JPRB ,&
&  0.33628E+00_JPRB, 0.33276E+00_JPRB, 0.32759E+00_JPRB, 0.32488E+00_JPRB, 0.32468E+00_JPRB, 0.32488E+00_JPRB, 0.31932E+00_JPRB ,&
&  0.30922E+00_JPRB, 0.30883E+00_JPRB, 0.31380E+00_JPRB, 0.30881E+00_JPRB, 0.30340E+00_JPRB, 0.29837E+00_JPRB, 0.29271E+00_JPRB ,&
&  0.27758E+00_JPRB, 0.25811E+00_JPRB, 0.23078E+00_JPRB, 0.22538E+00_JPRB, 0.24125E+00_JPRB, 0.23570E+00_JPRB, 0.23821E+00_JPRB ,&
&  0.23838E+00_JPRB, 0.23887E+00_JPRB, 0.23738E+00_JPRB, 0.23599E+00_JPRB, 0.23244E+00_JPRB, 0.23101E+00_JPRB, 0.22973E+00_JPRB ,&
&  0.22488E+00_JPRB, 0.22024E+00_JPRB, 0.21288E+00_JPRB, 0.19590E+00_JPRB, 0.19005E+00_JPRB, 0.18747E+00_JPRB, 0.18215E+00_JPRB ,&
&  0.17449E+00_JPRB, 0.16815E+00_JPRB, 0.16518E+00_JPRB, 0.16211E+00_JPRB, 0.16413E+00_JPRB, 0.16558E+00_JPRB, 0.16560E+00_JPRB ,&
&  0.16559E+00_JPRB, 0.16527E+00_JPRB, 0.16301E+00_JPRB, 0.15934E+00_JPRB, 0.15650E+00_JPRB, 0.15535E+00_JPRB, 0.15568E+00_JPRB ,&
&  0.15776E+00_JPRB, 0.16225E+00_JPRB, 0.17249E+00_JPRB, 0.18067E+00_JPRB, 0.18004E+00_JPRB, 0.17518E+00_JPRB, 0.16662E+00_JPRB ,&
&  0.16000E+00_JPRB, 0.15737E+00_JPRB, 0.15521E+00_JPRB, 0.15416E+00_JPRB, 0.15461E+00_JPRB, 0.15603E+00_JPRB, 0.15696E+00_JPRB ,&
&  0.15797E+00_JPRB, 0.15886E+00_JPRB, 0.15966E+00_JPRB, 0.16023E+00_JPRB, 0.16114E+00_JPRB, 0.16215E+00_JPRB, 0.16396E+00_JPRB ,&
&  0.16668E+00_JPRB, 0.17095E+00_JPRB, 0.17660E+00_JPRB, 0.18262E+00_JPRB, 0.18689E+00_JPRB, 0.18892E+00_JPRB, 0.19209E+00_JPRB ,&
&  0.19770E+00_JPRB, 0.21164E+00_JPRB, 0.23960E+00_JPRB, 0.28559E+00_JPRB, 0.29018E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.29257E+00_JPRB, 0.29692E+00_JPRB, 0.29974E+00_JPRB, 0.30324E+00_JPRB, 0.31077E+00_JPRB, 0.32158E+00_JPRB, 0.33382E+00_JPRB ,&
&  0.34480E+00_JPRB, 0.35346E+00_JPRB, 0.35765E+00_JPRB, 0.36273E+00_JPRB, 0.36768E+00_JPRB, 0.37014E+00_JPRB, 0.37545E+00_JPRB ,&
&  0.37623E+00_JPRB, 0.37293E+00_JPRB, 0.36693E+00_JPRB, 0.35891E+00_JPRB, 0.35471E+00_JPRB, 0.35613E+00_JPRB, 0.35923E+00_JPRB ,&
&  0.36134E+00_JPRB, 0.36300E+00_JPRB, 0.36321E+00_JPRB, 0.36000E+00_JPRB, 0.35681E+00_JPRB, 0.35497E+00_JPRB, 0.35126E+00_JPRB ,&
&  0.34339E+00_JPRB, 0.29442E+00_JPRB, 0.25657E+00_JPRB, 0.25156E+00_JPRB, 0.26913E+00_JPRB, 0.27166E+00_JPRB, 0.26613E+00_JPRB ,&
&  0.26021E+00_JPRB, 0.25569E+00_JPRB, 0.25314E+00_JPRB, 0.25106E+00_JPRB, 0.24842E+00_JPRB, 0.24623E+00_JPRB, 0.24726E+00_JPRB ,&
&  0.24858E+00_JPRB, 0.24637E+00_JPRB, 0.24325E+00_JPRB, 0.23706E+00_JPRB, 0.23251E+00_JPRB, 0.23013E+00_JPRB, 0.22810E+00_JPRB ,&
&  0.22384E+00_JPRB, 0.21975E+00_JPRB, 0.21937E+00_JPRB, 0.22016E+00_JPRB, 0.22159E+00_JPRB, 0.22259E+00_JPRB, 0.22015E+00_JPRB ,&
&  0.21328E+00_JPRB, 0.20702E+00_JPRB, 0.19871E+00_JPRB, 0.18894E+00_JPRB, 0.17948E+00_JPRB, 0.17305E+00_JPRB, 0.16812E+00_JPRB ,&
&  0.16688E+00_JPRB, 0.17140E+00_JPRB, 0.18327E+00_JPRB, 0.19880E+00_JPRB, 0.22110E+00_JPRB, 0.20845E+00_JPRB, 0.19664E+00_JPRB ,&
&  0.18648E+00_JPRB, 0.17875E+00_JPRB, 0.17297E+00_JPRB, 0.17025E+00_JPRB, 0.16923E+00_JPRB, 0.16936E+00_JPRB, 0.17042E+00_JPRB ,&
&  0.17230E+00_JPRB, 0.17487E+00_JPRB, 0.17729E+00_JPRB, 0.17847E+00_JPRB, 0.17923E+00_JPRB, 0.17996E+00_JPRB, 0.18011E+00_JPRB ,&
&  0.17948E+00_JPRB, 0.18153E+00_JPRB, 0.18796E+00_JPRB, 0.19627E+00_JPRB, 0.20459E+00_JPRB, 0.21217E+00_JPRB, 0.22502E+00_JPRB ,&
&  0.24082E+00_JPRB, 0.26122E+00_JPRB, 0.27270E+00_JPRB, 0.28129E+00_JPRB, 0.28900E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.31716E+00_JPRB, 0.31861E+00_JPRB, 0.31604E+00_JPRB, 0.31306E+00_JPRB, 0.31260E+00_JPRB, 0.31449E+00_JPRB, 0.32053E+00_JPRB ,&
&  0.33037E+00_JPRB, 0.34266E+00_JPRB, 0.35437E+00_JPRB, 0.36418E+00_JPRB, 0.37314E+00_JPRB, 0.37718E+00_JPRB, 0.38330E+00_JPRB ,&
&  0.38980E+00_JPRB, 0.39571E+00_JPRB, 0.39648E+00_JPRB, 0.38625E+00_JPRB, 0.37628E+00_JPRB, 0.37102E+00_JPRB, 0.37107E+00_JPRB ,&
&  0.37298E+00_JPRB, 0.37580E+00_JPRB, 0.38218E+00_JPRB, 0.38472E+00_JPRB, 0.37905E+00_JPRB, 0.37785E+00_JPRB, 0.37589E+00_JPRB ,&
&  0.37062E+00_JPRB, 0.36014E+00_JPRB, 0.32821E+00_JPRB, 0.32329E+00_JPRB, 0.31544E+00_JPRB, 0.30659E+00_JPRB, 0.29751E+00_JPRB ,&
&  0.28730E+00_JPRB, 0.27723E+00_JPRB, 0.27040E+00_JPRB, 0.26647E+00_JPRB, 0.26733E+00_JPRB, 0.26854E+00_JPRB, 0.26793E+00_JPRB ,&
&  0.26987E+00_JPRB, 0.27269E+00_JPRB, 0.27481E+00_JPRB, 0.27487E+00_JPRB, 0.27422E+00_JPRB, 0.27269E+00_JPRB, 0.26770E+00_JPRB ,&
&  0.26314E+00_JPRB, 0.26048E+00_JPRB, 0.25838E+00_JPRB, 0.25652E+00_JPRB, 0.25595E+00_JPRB, 0.25615E+00_JPRB, 0.25615E+00_JPRB ,&
&  0.25407E+00_JPRB, 0.25044E+00_JPRB, 0.24635E+00_JPRB, 0.23968E+00_JPRB, 0.22886E+00_JPRB, 0.21269E+00_JPRB, 0.20327E+00_JPRB ,&
&  0.20155E+00_JPRB, 0.21269E+00_JPRB, 0.23479E+00_JPRB, 0.24226E+00_JPRB, 0.24946E+00_JPRB, 0.25362E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25543E+00_JPRB, 0.23026E+00_JPRB, 0.21668E+00_JPRB, 0.21003E+00_JPRB, 0.20431E+00_JPRB, 0.19960E+00_JPRB, 0.19941E+00_JPRB ,&
&  0.20239E+00_JPRB, 0.20913E+00_JPRB, 0.21542E+00_JPRB, 0.21847E+00_JPRB, 0.21788E+00_JPRB, 0.21437E+00_JPRB, 0.21101E+00_JPRB ,&
&  0.20775E+00_JPRB, 0.21185E+00_JPRB, 0.22640E+00_JPRB, 0.25023E+00_JPRB, 0.25816E+00_JPRB, 0.26419E+00_JPRB, 0.26987E+00_JPRB ,&
&  0.27587E+00_JPRB, 0.28498E+00_JPRB, 0.29625E+00_JPRB, 0.30531E+00_JPRB, 0.31337E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.33213E+00_JPRB, 0.33184E+00_JPRB, 0.33323E+00_JPRB, 0.33206E+00_JPRB, 0.32950E+00_JPRB, 0.32785E+00_JPRB, 0.33183E+00_JPRB ,&
&  0.33895E+00_JPRB, 0.35043E+00_JPRB, 0.36552E+00_JPRB, 0.37880E+00_JPRB, 0.38691E+00_JPRB, 0.39199E+00_JPRB, 0.39977E+00_JPRB ,&
&  0.40691E+00_JPRB, 0.41480E+00_JPRB, 0.41491E+00_JPRB, 0.40600E+00_JPRB, 0.39842E+00_JPRB, 0.39439E+00_JPRB, 0.39379E+00_JPRB ,&
&  0.39317E+00_JPRB, 0.39507E+00_JPRB, 0.39845E+00_JPRB, 0.40210E+00_JPRB, 0.40169E+00_JPRB, 0.40010E+00_JPRB, 0.40407E+00_JPRB ,&
&  0.40260E+00_JPRB, 0.39362E+00_JPRB, 0.38470E+00_JPRB, 0.36009E+00_JPRB, 0.35995E+00_JPRB, 0.35006E+00_JPRB, 0.34231E+00_JPRB ,&
&  0.33188E+00_JPRB, 0.31913E+00_JPRB, 0.30996E+00_JPRB, 0.30408E+00_JPRB, 0.30150E+00_JPRB, 0.29888E+00_JPRB, 0.29672E+00_JPRB ,&
&  0.29528E+00_JPRB, 0.29482E+00_JPRB, 0.29587E+00_JPRB, 0.29361E+00_JPRB, 0.29246E+00_JPRB, 0.29332E+00_JPRB, 0.29496E+00_JPRB ,&
&  0.29435E+00_JPRB, 0.29344E+00_JPRB, 0.29268E+00_JPRB, 0.29131E+00_JPRB, 0.28948E+00_JPRB, 0.28702E+00_JPRB, 0.28458E+00_JPRB ,&
&  0.28341E+00_JPRB, 0.28417E+00_JPRB, 0.28425E+00_JPRB, 0.28034E+00_JPRB, 0.27270E+00_JPRB, 0.26396E+00_JPRB, 0.25831E+00_JPRB ,&
&  0.25582E+00_JPRB, 0.25766E+00_JPRB, 0.26521E+00_JPRB, 0.27609E+00_JPRB, 0.28640E+00_JPRB, 0.29241E+00_JPRB, 0.29597E+00_JPRB ,&
&  0.30043E+00_JPRB, 0.29923E+00_JPRB, 0.29575E+00_JPRB, 0.29247E+00_JPRB, 0.28938E+00_JPRB, 0.28540E+00_JPRB, 0.28528E+00_JPRB ,&
&  0.28753E+00_JPRB, 0.28962E+00_JPRB, 0.29011E+00_JPRB, 0.28879E+00_JPRB, 0.28660E+00_JPRB, 0.28220E+00_JPRB, 0.27602E+00_JPRB ,&
&  0.26856E+00_JPRB, 0.25997E+00_JPRB, 0.25762E+00_JPRB, 0.25785E+00_JPRB, 0.26157E+00_JPRB, 0.26943E+00_JPRB, 0.28010E+00_JPRB ,&
&  0.29139E+00_JPRB, 0.30087E+00_JPRB, 0.31102E+00_JPRB, 0.32037E+00_JPRB, 0.32861E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.35622E+00_JPRB, 0.35821E+00_JPRB, 0.36059E+00_JPRB, 0.36364E+00_JPRB, 0.36704E+00_JPRB, 0.36928E+00_JPRB, 0.37343E+00_JPRB ,&
&  0.37573E+00_JPRB, 0.38201E+00_JPRB, 0.39244E+00_JPRB, 0.40307E+00_JPRB, 0.41097E+00_JPRB, 0.41778E+00_JPRB, 0.42431E+00_JPRB ,&
&  0.42954E+00_JPRB, 0.43651E+00_JPRB, 0.43730E+00_JPRB, 0.43473E+00_JPRB, 0.43189E+00_JPRB, 0.42966E+00_JPRB, 0.42769E+00_JPRB ,&
&  0.42517E+00_JPRB, 0.42508E+00_JPRB, 0.42894E+00_JPRB, 0.43336E+00_JPRB, 0.43798E+00_JPRB, 0.43957E+00_JPRB, 0.43873E+00_JPRB ,&
&  0.44303E+00_JPRB, 0.43430E+00_JPRB, 0.42888E+00_JPRB, 0.42229E+00_JPRB, 0.41642E+00_JPRB, 0.40536E+00_JPRB, 0.39731E+00_JPRB ,&
&  0.38778E+00_JPRB, 0.37956E+00_JPRB, 0.37615E+00_JPRB, 0.37245E+00_JPRB, 0.36296E+00_JPRB, 0.35168E+00_JPRB, 0.34309E+00_JPRB ,&
&  0.33733E+00_JPRB, 0.33154E+00_JPRB, 0.32477E+00_JPRB, 0.31935E+00_JPRB, 0.31499E+00_JPRB, 0.31356E+00_JPRB, 0.31550E+00_JPRB ,&
&  0.31952E+00_JPRB, 0.32290E+00_JPRB, 0.32458E+00_JPRB, 0.32472E+00_JPRB, 0.32305E+00_JPRB, 0.32038E+00_JPRB, 0.31868E+00_JPRB ,&
&  0.31896E+00_JPRB, 0.32008E+00_JPRB, 0.32230E+00_JPRB, 0.32749E+00_JPRB, 0.32805E+00_JPRB, 0.32653E+00_JPRB, 0.32101E+00_JPRB ,&
&  0.31825E+00_JPRB, 0.31673E+00_JPRB, 0.31815E+00_JPRB, 0.32305E+00_JPRB, 0.33116E+00_JPRB, 0.33900E+00_JPRB, 0.34469E+00_JPRB ,&
&  0.34765E+00_JPRB, 0.34963E+00_JPRB, 0.35281E+00_JPRB, 0.35633E+00_JPRB, 0.35751E+00_JPRB, 0.35652E+00_JPRB, 0.35471E+00_JPRB ,&
&  0.35718E+00_JPRB, 0.36423E+00_JPRB, 0.37171E+00_JPRB, 0.37623E+00_JPRB, 0.37820E+00_JPRB, 0.37684E+00_JPRB, 0.36984E+00_JPRB ,&
&  0.35734E+00_JPRB, 0.34308E+00_JPRB, 0.32921E+00_JPRB, 0.31739E+00_JPRB, 0.30922E+00_JPRB, 0.30541E+00_JPRB, 0.30840E+00_JPRB ,&
&  0.31630E+00_JPRB, 0.32642E+00_JPRB, 0.33590E+00_JPRB, 0.34307E+00_JPRB, 0.35082E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.37996E+00_JPRB, 0.38021E+00_JPRB, 0.38324E+00_JPRB, 0.38976E+00_JPRB, 0.39416E+00_JPRB, 0.39829E+00_JPRB, 0.40609E+00_JPRB ,&
&  0.41302E+00_JPRB, 0.41914E+00_JPRB, 0.42658E+00_JPRB, 0.43491E+00_JPRB, 0.44142E+00_JPRB, 0.45062E+00_JPRB, 0.45916E+00_JPRB ,&
&  0.46458E+00_JPRB, 0.46668E+00_JPRB, 0.46776E+00_JPRB, 0.46865E+00_JPRB, 0.46825E+00_JPRB, 0.46553E+00_JPRB, 0.46496E+00_JPRB ,&
&  0.46315E+00_JPRB, 0.46122E+00_JPRB, 0.46353E+00_JPRB, 0.46409E+00_JPRB, 0.46619E+00_JPRB, 0.46957E+00_JPRB, 0.47044E+00_JPRB ,&
&  0.46924E+00_JPRB, 0.45711E+00_JPRB, 0.45308E+00_JPRB, 0.45073E+00_JPRB, 0.45542E+00_JPRB, 0.45697E+00_JPRB, 0.45599E+00_JPRB ,&
&  0.45013E+00_JPRB, 0.44674E+00_JPRB, 0.44888E+00_JPRB, 0.45143E+00_JPRB, 0.44585E+00_JPRB, 0.43299E+00_JPRB, 0.42246E+00_JPRB ,&
&  0.41398E+00_JPRB, 0.40158E+00_JPRB, 0.38399E+00_JPRB, 0.37389E+00_JPRB, 0.36973E+00_JPRB, 0.36485E+00_JPRB, 0.36014E+00_JPRB ,&
&  0.35757E+00_JPRB, 0.35676E+00_JPRB, 0.35827E+00_JPRB, 0.35735E+00_JPRB, 0.35357E+00_JPRB, 0.34892E+00_JPRB, 0.34695E+00_JPRB ,&
&  0.34932E+00_JPRB, 0.35137E+00_JPRB, 0.35471E+00_JPRB, 0.35720E+00_JPRB, 0.35658E+00_JPRB, 0.35696E+00_JPRB, 0.36046E+00_JPRB ,&
&  0.36562E+00_JPRB, 0.37039E+00_JPRB, 0.37575E+00_JPRB, 0.37793E+00_JPRB, 0.38061E+00_JPRB, 0.38682E+00_JPRB, 0.38708E+00_JPRB ,&
&  0.38571E+00_JPRB, 0.38791E+00_JPRB, 0.39204E+00_JPRB, 0.39957E+00_JPRB, 0.41104E+00_JPRB, 0.41855E+00_JPRB, 0.41953E+00_JPRB ,&
&  0.41946E+00_JPRB, 0.42158E+00_JPRB, 0.42506E+00_JPRB, 0.43370E+00_JPRB, 0.44313E+00_JPRB, 0.45000E+00_JPRB, 0.45294E+00_JPRB ,&
&  0.44999E+00_JPRB, 0.44346E+00_JPRB, 0.43150E+00_JPRB, 0.41741E+00_JPRB, 0.40479E+00_JPRB, 0.39492E+00_JPRB, 0.38685E+00_JPRB ,&
&  0.37897E+00_JPRB, 0.37579E+00_JPRB, 0.37571E+00_JPRB, 0.37781E+00_JPRB, 0.37915E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.41168E+00_JPRB, 0.41249E+00_JPRB, 0.41735E+00_JPRB, 0.42230E+00_JPRB, 0.42592E+00_JPRB, 0.43101E+00_JPRB, 0.44058E+00_JPRB ,&
&  0.45028E+00_JPRB, 0.45967E+00_JPRB, 0.46610E+00_JPRB, 0.47392E+00_JPRB, 0.48473E+00_JPRB, 0.49197E+00_JPRB, 0.49775E+00_JPRB ,&
&  0.50013E+00_JPRB, 0.50033E+00_JPRB, 0.50104E+00_JPRB, 0.50393E+00_JPRB, 0.50339E+00_JPRB, 0.49975E+00_JPRB, 0.49847E+00_JPRB ,&
&  0.49846E+00_JPRB, 0.49699E+00_JPRB, 0.49708E+00_JPRB, 0.49870E+00_JPRB, 0.49714E+00_JPRB, 0.49421E+00_JPRB, 0.49461E+00_JPRB ,&
&  0.49589E+00_JPRB, 0.48298E+00_JPRB, 0.47618E+00_JPRB, 0.46979E+00_JPRB, 0.47402E+00_JPRB, 0.48599E+00_JPRB, 0.49894E+00_JPRB ,&
&  0.50680E+00_JPRB, 0.51213E+00_JPRB, 0.51182E+00_JPRB, 0.51095E+00_JPRB, 0.50649E+00_JPRB, 0.49509E+00_JPRB, 0.48509E+00_JPRB ,&
&  0.47824E+00_JPRB, 0.46874E+00_JPRB, 0.44656E+00_JPRB, 0.43408E+00_JPRB, 0.43106E+00_JPRB, 0.43362E+00_JPRB, 0.43728E+00_JPRB ,&
&  0.43897E+00_JPRB, 0.43681E+00_JPRB, 0.43122E+00_JPRB, 0.42418E+00_JPRB, 0.41436E+00_JPRB, 0.40286E+00_JPRB, 0.39276E+00_JPRB ,&
&  0.38685E+00_JPRB, 0.38238E+00_JPRB, 0.37866E+00_JPRB, 0.38094E+00_JPRB, 0.38021E+00_JPRB, 0.37640E+00_JPRB, 0.37803E+00_JPRB ,&
&  0.38863E+00_JPRB, 0.40190E+00_JPRB, 0.41420E+00_JPRB, 0.42098E+00_JPRB, 0.42367E+00_JPRB, 0.42676E+00_JPRB, 0.43061E+00_JPRB ,&
&  0.43645E+00_JPRB, 0.44246E+00_JPRB, 0.44732E+00_JPRB, 0.45319E+00_JPRB, 0.46060E+00_JPRB, 0.46742E+00_JPRB, 0.47025E+00_JPRB ,&
&  0.47273E+00_JPRB, 0.47402E+00_JPRB, 0.47457E+00_JPRB, 0.47893E+00_JPRB, 0.48617E+00_JPRB, 0.49138E+00_JPRB, 0.49574E+00_JPRB ,&
&  0.50043E+00_JPRB, 0.50267E+00_JPRB, 0.50141E+00_JPRB, 0.49744E+00_JPRB, 0.49192E+00_JPRB, 0.48077E+00_JPRB, 0.46712E+00_JPRB ,&
&  0.45151E+00_JPRB, 0.43756E+00_JPRB, 0.42748E+00_JPRB, 0.41885E+00_JPRB, 0.41302E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.45399E+00_JPRB, 0.45207E+00_JPRB, 0.45268E+00_JPRB, 0.45605E+00_JPRB, 0.45980E+00_JPRB, 0.46286E+00_JPRB, 0.46895E+00_JPRB ,&
&  0.47908E+00_JPRB, 0.48700E+00_JPRB, 0.49650E+00_JPRB, 0.50603E+00_JPRB, 0.51321E+00_JPRB, 0.51804E+00_JPRB, 0.52067E+00_JPRB ,&
&  0.51896E+00_JPRB, 0.51621E+00_JPRB, 0.51879E+00_JPRB, 0.52323E+00_JPRB, 0.52458E+00_JPRB, 0.52416E+00_JPRB, 0.52424E+00_JPRB ,&
&  0.52400E+00_JPRB, 0.52222E+00_JPRB, 0.52008E+00_JPRB, 0.51679E+00_JPRB, 0.51275E+00_JPRB, 0.51142E+00_JPRB, 0.51008E+00_JPRB ,&
&  0.50452E+00_JPRB, 0.50404E+00_JPRB, 0.45458E+00_JPRB, 0.47023E+00_JPRB, 0.48361E+00_JPRB, 0.50351E+00_JPRB, 0.52258E+00_JPRB ,&
&  0.53641E+00_JPRB, 0.54561E+00_JPRB, 0.55043E+00_JPRB, 0.54936E+00_JPRB, 0.53753E+00_JPRB, 0.52956E+00_JPRB, 0.52041E+00_JPRB ,&
&  0.50908E+00_JPRB, 0.50097E+00_JPRB, 0.48812E+00_JPRB, 0.47456E+00_JPRB, 0.47004E+00_JPRB, 0.47053E+00_JPRB, 0.47059E+00_JPRB ,&
&  0.46751E+00_JPRB, 0.46477E+00_JPRB, 0.46673E+00_JPRB, 0.46814E+00_JPRB, 0.46826E+00_JPRB, 0.46315E+00_JPRB, 0.45647E+00_JPRB ,&
&  0.44596E+00_JPRB, 0.43554E+00_JPRB, 0.42831E+00_JPRB, 0.42272E+00_JPRB, 0.41772E+00_JPRB, 0.41228E+00_JPRB, 0.40694E+00_JPRB ,&
&  0.40984E+00_JPRB, 0.42026E+00_JPRB, 0.43002E+00_JPRB, 0.43828E+00_JPRB, 0.44377E+00_JPRB, 0.44757E+00_JPRB, 0.45280E+00_JPRB ,&
&  0.45883E+00_JPRB, 0.46884E+00_JPRB, 0.47546E+00_JPRB, 0.48129E+00_JPRB, 0.48572E+00_JPRB, 0.48993E+00_JPRB, 0.49489E+00_JPRB ,&
&  0.49879E+00_JPRB, 0.50490E+00_JPRB, 0.50902E+00_JPRB, 0.51420E+00_JPRB, 0.51690E+00_JPRB, 0.52127E+00_JPRB, 0.52691E+00_JPRB ,&
&  0.53337E+00_JPRB, 0.53917E+00_JPRB, 0.54142E+00_JPRB, 0.53957E+00_JPRB, 0.53193E+00_JPRB, 0.52162E+00_JPRB, 0.51177E+00_JPRB ,&
&  0.49942E+00_JPRB, 0.48549E+00_JPRB, 0.47183E+00_JPRB, 0.46337E+00_JPRB, 0.45641E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.51022E+00_JPRB, 0.50548E+00_JPRB, 0.50046E+00_JPRB, 0.50006E+00_JPRB, 0.49959E+00_JPRB, 0.50231E+00_JPRB, 0.50591E+00_JPRB ,&
&  0.51317E+00_JPRB, 0.51781E+00_JPRB, 0.52694E+00_JPRB, 0.53417E+00_JPRB, 0.53754E+00_JPRB, 0.54155E+00_JPRB, 0.54417E+00_JPRB ,&
&  0.54525E+00_JPRB, 0.54673E+00_JPRB, 0.54756E+00_JPRB, 0.55192E+00_JPRB, 0.55280E+00_JPRB, 0.55224E+00_JPRB, 0.55156E+00_JPRB ,&
&  0.55026E+00_JPRB, 0.54909E+00_JPRB, 0.54519E+00_JPRB, 0.53941E+00_JPRB, 0.53189E+00_JPRB, 0.52924E+00_JPRB, 0.52408E+00_JPRB ,&
&  0.52617E+00_JPRB, 0.49366E+00_JPRB, 0.50518E+00_JPRB, 0.50390E+00_JPRB, 0.51674E+00_JPRB, 0.53700E+00_JPRB, 0.56664E+00_JPRB ,&
&  0.58791E+00_JPRB, 0.60000E+00_JPRB, 0.60294E+00_JPRB, 0.60229E+00_JPRB, 0.58947E+00_JPRB, 0.57566E+00_JPRB, 0.55996E+00_JPRB ,&
&  0.54178E+00_JPRB, 0.53165E+00_JPRB, 0.51857E+00_JPRB, 0.50905E+00_JPRB, 0.50525E+00_JPRB, 0.50277E+00_JPRB, 0.49865E+00_JPRB ,&
&  0.49235E+00_JPRB, 0.48709E+00_JPRB, 0.48835E+00_JPRB, 0.49234E+00_JPRB, 0.49637E+00_JPRB, 0.49893E+00_JPRB, 0.49810E+00_JPRB ,&
&  0.49193E+00_JPRB, 0.48616E+00_JPRB, 0.48284E+00_JPRB, 0.48175E+00_JPRB, 0.47678E+00_JPRB, 0.46866E+00_JPRB, 0.46032E+00_JPRB ,&
&  0.45820E+00_JPRB, 0.46085E+00_JPRB, 0.46460E+00_JPRB, 0.46943E+00_JPRB, 0.47314E+00_JPRB, 0.47756E+00_JPRB, 0.47965E+00_JPRB ,&
&  0.48440E+00_JPRB, 0.48901E+00_JPRB, 0.49822E+00_JPRB, 0.50704E+00_JPRB, 0.50938E+00_JPRB, 0.51795E+00_JPRB, 0.52665E+00_JPRB ,&
&  0.53490E+00_JPRB, 0.54417E+00_JPRB, 0.55082E+00_JPRB, 0.55470E+00_JPRB, 0.55888E+00_JPRB, 0.56119E+00_JPRB, 0.56608E+00_JPRB ,&
&  0.57304E+00_JPRB, 0.57424E+00_JPRB, 0.57891E+00_JPRB, 0.57825E+00_JPRB, 0.57198E+00_JPRB, 0.56248E+00_JPRB, 0.55115E+00_JPRB ,&
&  0.53925E+00_JPRB, 0.52445E+00_JPRB, 0.51732E+00_JPRB, 0.51342E+00_JPRB, 0.51165E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.54943E+00_JPRB, 0.54314E+00_JPRB, 0.53317E+00_JPRB, 0.52787E+00_JPRB, 0.52647E+00_JPRB, 0.53072E+00_JPRB, 0.53606E+00_JPRB ,&
&  0.54134E+00_JPRB, 0.54345E+00_JPRB, 0.55075E+00_JPRB, 0.55791E+00_JPRB, 0.56203E+00_JPRB, 0.56474E+00_JPRB, 0.56388E+00_JPRB ,&
&  0.56398E+00_JPRB, 0.56418E+00_JPRB, 0.56019E+00_JPRB, 0.55733E+00_JPRB, 0.55289E+00_JPRB, 0.55409E+00_JPRB, 0.55483E+00_JPRB ,&
&  0.55464E+00_JPRB, 0.55280E+00_JPRB, 0.54795E+00_JPRB, 0.54072E+00_JPRB, 0.53805E+00_JPRB, 0.53856E+00_JPRB, 0.53709E+00_JPRB ,&
&  0.54451E+00_JPRB, 0.46478E+00_JPRB, 0.49957E+00_JPRB, 0.51033E+00_JPRB, 0.52525E+00_JPRB, 0.54902E+00_JPRB, 0.57851E+00_JPRB ,&
&  0.60895E+00_JPRB, 0.63371E+00_JPRB, 0.64155E+00_JPRB, 0.64107E+00_JPRB, 0.63008E+00_JPRB, 0.61345E+00_JPRB, 0.59445E+00_JPRB ,&
&  0.57810E+00_JPRB, 0.56519E+00_JPRB, 0.54885E+00_JPRB, 0.53957E+00_JPRB, 0.53220E+00_JPRB, 0.52263E+00_JPRB, 0.51498E+00_JPRB ,&
&  0.51263E+00_JPRB, 0.51164E+00_JPRB, 0.51238E+00_JPRB, 0.51481E+00_JPRB, 0.51824E+00_JPRB, 0.52068E+00_JPRB, 0.52080E+00_JPRB ,&
&  0.51953E+00_JPRB, 0.51662E+00_JPRB, 0.51169E+00_JPRB, 0.50927E+00_JPRB, 0.50824E+00_JPRB, 0.50943E+00_JPRB, 0.50554E+00_JPRB ,&
&  0.49920E+00_JPRB, 0.50083E+00_JPRB, 0.50426E+00_JPRB, 0.50664E+00_JPRB, 0.51003E+00_JPRB, 0.51366E+00_JPRB, 0.51442E+00_JPRB ,&
&  0.51505E+00_JPRB, 0.51923E+00_JPRB, 0.52370E+00_JPRB, 0.53060E+00_JPRB, 0.53543E+00_JPRB, 0.54436E+00_JPRB, 0.55157E+00_JPRB ,&
&  0.55822E+00_JPRB, 0.56840E+00_JPRB, 0.57751E+00_JPRB, 0.58293E+00_JPRB, 0.58602E+00_JPRB, 0.58977E+00_JPRB, 0.59400E+00_JPRB ,&
&  0.59691E+00_JPRB, 0.59909E+00_JPRB, 0.60098E+00_JPRB, 0.59907E+00_JPRB, 0.59280E+00_JPRB, 0.58638E+00_JPRB, 0.58183E+00_JPRB ,&
&  0.57214E+00_JPRB, 0.56069E+00_JPRB, 0.55664E+00_JPRB, 0.55390E+00_JPRB, 0.55089E+00_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.60871E+00_JPRB, 0.60178E+00_JPRB, 0.58982E+00_JPRB, 0.58012E+00_JPRB, 0.57277E+00_JPRB, 0.57128E+00_JPRB, 0.57078E+00_JPRB ,&
&  0.57600E+00_JPRB, 0.58386E+00_JPRB, 0.59092E+00_JPRB, 0.59444E+00_JPRB, 0.59671E+00_JPRB, 0.59663E+00_JPRB, 0.59335E+00_JPRB ,&
&  0.59087E+00_JPRB, 0.58029E+00_JPRB, 0.57280E+00_JPRB, 0.57366E+00_JPRB, 0.57135E+00_JPRB, 0.56889E+00_JPRB, 0.56617E+00_JPRB ,&
&  0.56390E+00_JPRB, 0.56275E+00_JPRB, 0.55600E+00_JPRB, 0.55364E+00_JPRB, 0.55396E+00_JPRB, 0.55701E+00_JPRB, 0.56824E+00_JPRB ,&
&  0.49655E+00_JPRB, 0.41724E+00_JPRB, 0.46491E+00_JPRB, 0.51031E+00_JPRB, 0.53517E+00_JPRB, 0.57152E+00_JPRB, 0.60638E+00_JPRB ,&
&  0.65147E+00_JPRB, 0.67917E+00_JPRB, 0.69257E+00_JPRB, 0.68686E+00_JPRB, 0.67101E+00_JPRB, 0.65505E+00_JPRB, 0.63889E+00_JPRB ,&
&  0.61924E+00_JPRB, 0.59626E+00_JPRB, 0.58230E+00_JPRB, 0.57665E+00_JPRB, 0.57184E+00_JPRB, 0.56596E+00_JPRB, 0.56344E+00_JPRB ,&
&  0.56381E+00_JPRB, 0.56424E+00_JPRB, 0.56346E+00_JPRB, 0.56311E+00_JPRB, 0.56529E+00_JPRB, 0.56768E+00_JPRB, 0.57000E+00_JPRB ,&
&  0.56912E+00_JPRB, 0.56693E+00_JPRB, 0.56316E+00_JPRB, 0.55670E+00_JPRB, 0.55197E+00_JPRB, 0.55217E+00_JPRB, 0.55040E+00_JPRB ,&
&  0.54940E+00_JPRB, 0.54836E+00_JPRB, 0.54886E+00_JPRB, 0.55018E+00_JPRB, 0.55048E+00_JPRB, 0.55117E+00_JPRB, 0.55433E+00_JPRB ,&
&  0.56044E+00_JPRB, 0.56122E+00_JPRB, 0.56451E+00_JPRB, 0.56999E+00_JPRB, 0.57539E+00_JPRB, 0.58021E+00_JPRB, 0.58278E+00_JPRB ,&
&  0.58751E+00_JPRB, 0.59824E+00_JPRB, 0.60561E+00_JPRB, 0.61002E+00_JPRB, 0.61392E+00_JPRB, 0.61863E+00_JPRB, 0.62546E+00_JPRB ,&
&  0.63053E+00_JPRB, 0.63465E+00_JPRB, 0.64005E+00_JPRB, 0.64733E+00_JPRB, 0.64945E+00_JPRB, 0.64599E+00_JPRB, 0.63908E+00_JPRB ,&
&  0.62779E+00_JPRB, 0.61227E+00_JPRB, 0.61350E+00_JPRB, 0.61158E+00_JPRB, 0.60948E+00_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.62675E+00_JPRB, 0.62865E+00_JPRB, 0.62270E+00_JPRB, 0.61371E+00_JPRB, 0.60723E+00_JPRB, 0.60570E+00_JPRB, 0.60609E+00_JPRB ,&
&  0.61103E+00_JPRB, 0.61524E+00_JPRB, 0.61635E+00_JPRB, 0.61334E+00_JPRB, 0.60854E+00_JPRB, 0.60105E+00_JPRB, 0.59532E+00_JPRB ,&
&  0.58362E+00_JPRB, 0.57084E+00_JPRB, 0.56736E+00_JPRB, 0.56811E+00_JPRB, 0.56639E+00_JPRB, 0.56095E+00_JPRB, 0.55912E+00_JPRB ,&
&  0.56006E+00_JPRB, 0.56452E+00_JPRB, 0.56063E+00_JPRB, 0.55892E+00_JPRB, 0.55591E+00_JPRB, 0.55913E+00_JPRB, 0.56185E+00_JPRB ,&
&  0.56234E+00_JPRB, 0.43586E+00_JPRB, 0.48087E+00_JPRB, 0.52275E+00_JPRB, 0.54373E+00_JPRB, 0.58770E+00_JPRB, 0.63774E+00_JPRB ,&
&  0.67457E+00_JPRB, 0.69288E+00_JPRB, 0.68877E+00_JPRB, 0.67741E+00_JPRB, 0.65841E+00_JPRB, 0.63795E+00_JPRB, 0.61889E+00_JPRB ,&
&  0.60043E+00_JPRB, 0.58898E+00_JPRB, 0.58183E+00_JPRB, 0.57655E+00_JPRB, 0.57550E+00_JPRB, 0.57428E+00_JPRB, 0.57290E+00_JPRB ,&
&  0.57717E+00_JPRB, 0.58653E+00_JPRB, 0.58892E+00_JPRB, 0.59235E+00_JPRB, 0.59592E+00_JPRB, 0.59767E+00_JPRB, 0.60447E+00_JPRB ,&
&  0.60721E+00_JPRB, 0.60495E+00_JPRB, 0.60284E+00_JPRB, 0.59853E+00_JPRB, 0.58952E+00_JPRB, 0.58655E+00_JPRB, 0.58262E+00_JPRB ,&
&  0.57912E+00_JPRB, 0.57250E+00_JPRB, 0.56722E+00_JPRB, 0.56613E+00_JPRB, 0.56763E+00_JPRB, 0.56882E+00_JPRB, 0.57570E+00_JPRB ,&
&  0.58653E+00_JPRB, 0.59001E+00_JPRB, 0.59311E+00_JPRB, 0.59826E+00_JPRB, 0.60205E+00_JPRB, 0.60379E+00_JPRB, 0.60627E+00_JPRB ,&
&  0.61357E+00_JPRB, 0.62660E+00_JPRB, 0.63862E+00_JPRB, 0.64326E+00_JPRB, 0.64489E+00_JPRB, 0.65163E+00_JPRB, 0.65873E+00_JPRB ,&
&  0.66617E+00_JPRB, 0.66789E+00_JPRB, 0.66783E+00_JPRB, 0.68734E+00_JPRB, 0.69472E+00_JPRB, 0.68923E+00_JPRB, 0.67972E+00_JPRB ,&
&  0.66146E+00_JPRB, 0.65535E+00_JPRB, 0.52834E+00_JPRB, 0.58625E+00_JPRB, 0.60629E+00_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.66200E+00_JPRB, 0.66813E+00_JPRB, 0.66295E+00_JPRB, 0.65296E+00_JPRB, 0.64966E+00_JPRB, 0.64918E+00_JPRB, 0.64973E+00_JPRB ,&
&  0.64501E+00_JPRB, 0.63695E+00_JPRB, 0.62743E+00_JPRB, 0.61612E+00_JPRB, 0.60437E+00_JPRB, 0.59495E+00_JPRB, 0.58543E+00_JPRB ,&
&  0.56862E+00_JPRB, 0.55979E+00_JPRB, 0.56078E+00_JPRB, 0.55859E+00_JPRB, 0.55633E+00_JPRB, 0.55316E+00_JPRB, 0.55521E+00_JPRB ,&
&  0.55981E+00_JPRB, 0.56207E+00_JPRB, 0.56337E+00_JPRB, 0.55855E+00_JPRB, 0.55532E+00_JPRB, 0.55675E+00_JPRB, 0.55451E+00_JPRB ,&
&  0.55280E+00_JPRB, 0.44243E+00_JPRB, 0.48066E+00_JPRB, 0.54649E+00_JPRB, 0.57379E+00_JPRB, 0.61922E+00_JPRB, 0.68832E+00_JPRB ,&
&  0.73074E+00_JPRB, 0.72408E+00_JPRB, 0.70335E+00_JPRB, 0.68399E+00_JPRB, 0.65956E+00_JPRB, 0.63527E+00_JPRB, 0.61426E+00_JPRB ,&
&  0.60217E+00_JPRB, 0.59738E+00_JPRB, 0.59540E+00_JPRB, 0.59224E+00_JPRB, 0.58848E+00_JPRB, 0.58953E+00_JPRB, 0.59125E+00_JPRB ,&
&  0.59479E+00_JPRB, 0.60152E+00_JPRB, 0.60565E+00_JPRB, 0.61377E+00_JPRB, 0.62055E+00_JPRB, 0.63165E+00_JPRB, 0.64627E+00_JPRB ,&
&  0.64818E+00_JPRB, 0.65513E+00_JPRB, 0.66770E+00_JPRB, 0.66774E+00_JPRB, 0.65528E+00_JPRB, 0.64856E+00_JPRB, 0.64185E+00_JPRB ,&
&  0.63273E+00_JPRB, 0.61609E+00_JPRB, 0.61401E+00_JPRB, 0.61186E+00_JPRB, 0.60890E+00_JPRB, 0.60835E+00_JPRB, 0.61787E+00_JPRB ,&
&  0.62722E+00_JPRB, 0.63048E+00_JPRB, 0.63616E+00_JPRB, 0.64492E+00_JPRB, 0.64905E+00_JPRB, 0.64498E+00_JPRB, 0.64294E+00_JPRB ,&
&  0.64823E+00_JPRB, 0.65804E+00_JPRB, 0.66264E+00_JPRB, 0.66336E+00_JPRB, 0.67072E+00_JPRB, 0.68279E+00_JPRB, 0.69015E+00_JPRB ,&
&  0.69488E+00_JPRB, 0.69408E+00_JPRB, 0.69968E+00_JPRB, 0.67582E+00_JPRB, 0.71765E+00_JPRB, 0.71271E+00_JPRB, 0.70236E+00_JPRB ,&
&  0.69275E+00_JPRB, 0.68100E+00_JPRB, 0.68113E+00_JPRB, 0.58530E+00_JPRB, 0.62816E+00_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.70136E+00_JPRB, 0.71117E+00_JPRB, 0.69934E+00_JPRB, 0.68590E+00_JPRB, 0.67904E+00_JPRB, 0.67104E+00_JPRB, 0.65284E+00_JPRB ,&
&  0.63201E+00_JPRB, 0.61333E+00_JPRB, 0.60324E+00_JPRB, 0.58954E+00_JPRB, 0.57668E+00_JPRB, 0.56971E+00_JPRB, 0.56112E+00_JPRB ,&
&  0.54668E+00_JPRB, 0.54405E+00_JPRB, 0.54521E+00_JPRB, 0.54408E+00_JPRB, 0.54062E+00_JPRB, 0.53947E+00_JPRB, 0.54787E+00_JPRB ,&
&  0.54962E+00_JPRB, 0.54757E+00_JPRB, 0.54759E+00_JPRB, 0.54504E+00_JPRB, 0.54171E+00_JPRB, 0.54276E+00_JPRB, 0.53354E+00_JPRB ,&
&  0.52951E+00_JPRB, 0.46825E+00_JPRB, 0.52446E+00_JPRB, 0.59291E+00_JPRB, 0.64655E+00_JPRB, 0.68600E+00_JPRB, 0.72983E+00_JPRB ,&
&  0.74984E+00_JPRB, 0.72828E+00_JPRB, 0.69912E+00_JPRB, 0.67729E+00_JPRB, 0.64946E+00_JPRB, 0.62703E+00_JPRB, 0.61007E+00_JPRB ,&
&  0.59997E+00_JPRB, 0.59742E+00_JPRB, 0.59605E+00_JPRB, 0.59509E+00_JPRB, 0.59348E+00_JPRB, 0.59261E+00_JPRB, 0.59475E+00_JPRB ,&
&  0.59507E+00_JPRB, 0.59804E+00_JPRB, 0.60125E+00_JPRB, 0.60008E+00_JPRB, 0.60599E+00_JPRB, 0.62863E+00_JPRB, 0.65375E+00_JPRB ,&
&  0.65965E+00_JPRB, 0.67381E+00_JPRB, 0.69107E+00_JPRB, 0.69531E+00_JPRB, 0.68599E+00_JPRB, 0.68231E+00_JPRB, 0.67630E+00_JPRB ,&
&  0.67193E+00_JPRB, 0.66151E+00_JPRB, 0.66141E+00_JPRB, 0.65743E+00_JPRB, 0.65037E+00_JPRB, 0.64725E+00_JPRB, 0.64776E+00_JPRB ,&
&  0.64923E+00_JPRB, 0.65086E+00_JPRB, 0.65757E+00_JPRB, 0.66518E+00_JPRB, 0.66213E+00_JPRB, 0.65456E+00_JPRB, 0.65386E+00_JPRB ,&
&  0.65698E+00_JPRB, 0.65324E+00_JPRB, 0.64464E+00_JPRB, 0.65399E+00_JPRB, 0.67256E+00_JPRB, 0.68276E+00_JPRB, 0.68778E+00_JPRB ,&
&  0.69247E+00_JPRB, 0.69761E+00_JPRB, 0.70779E+00_JPRB, 0.73640E+00_JPRB, 0.73878E+00_JPRB, 0.72433E+00_JPRB, 0.71616E+00_JPRB ,&
&  0.71246E+00_JPRB, 0.70374E+00_JPRB, 0.69533E+00_JPRB, 0.69183E+00_JPRB, 0.64676E+00_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.83652E+00_JPRB, 0.78322E+00_JPRB, 0.73922E+00_JPRB, 0.70837E+00_JPRB, 0.68879E+00_JPRB, 0.66235E+00_JPRB, 0.62936E+00_JPRB ,&
&  0.60591E+00_JPRB, 0.59859E+00_JPRB, 0.59055E+00_JPRB, 0.57783E+00_JPRB, 0.56958E+00_JPRB, 0.55904E+00_JPRB, 0.54976E+00_JPRB ,&
&  0.53659E+00_JPRB, 0.53292E+00_JPRB, 0.52902E+00_JPRB, 0.53366E+00_JPRB, 0.53556E+00_JPRB, 0.54182E+00_JPRB, 0.55409E+00_JPRB ,&
&  0.55603E+00_JPRB, 0.53562E+00_JPRB, 0.53277E+00_JPRB, 0.53641E+00_JPRB, 0.53937E+00_JPRB, 0.54214E+00_JPRB, 0.53106E+00_JPRB ,&
&  0.52858E+00_JPRB, 0.49496E+00_JPRB, 0.60453E+00_JPRB, 0.70528E+00_JPRB, 0.76093E+00_JPRB, 0.81033E+00_JPRB, 0.81430E+00_JPRB ,&
&  0.80770E+00_JPRB, 0.77453E+00_JPRB, 0.73220E+00_JPRB, 0.68528E+00_JPRB, 0.64266E+00_JPRB, 0.61712E+00_JPRB, 0.60676E+00_JPRB ,&
&  0.60199E+00_JPRB, 0.60370E+00_JPRB, 0.60281E+00_JPRB, 0.60380E+00_JPRB, 0.60477E+00_JPRB, 0.60389E+00_JPRB, 0.60900E+00_JPRB ,&
&  0.61746E+00_JPRB, 0.60825E+00_JPRB, 0.60360E+00_JPRB, 0.61455E+00_JPRB, 0.62970E+00_JPRB, 0.64669E+00_JPRB, 0.68430E+00_JPRB ,&
&  0.70451E+00_JPRB, 0.72712E+00_JPRB, 0.74788E+00_JPRB, 0.75388E+00_JPRB, 0.74949E+00_JPRB, 0.74752E+00_JPRB, 0.73681E+00_JPRB ,&
&  0.73338E+00_JPRB, 0.72658E+00_JPRB, 0.71709E+00_JPRB, 0.70607E+00_JPRB, 0.69422E+00_JPRB, 0.68781E+00_JPRB, 0.68383E+00_JPRB ,&
&  0.67959E+00_JPRB, 0.68788E+00_JPRB, 0.69314E+00_JPRB, 0.69249E+00_JPRB, 0.68106E+00_JPRB, 0.67340E+00_JPRB, 0.66945E+00_JPRB ,&
&  0.65851E+00_JPRB, 0.64195E+00_JPRB, 0.63649E+00_JPRB, 0.64750E+00_JPRB, 0.67181E+00_JPRB, 0.68111E+00_JPRB, 0.68373E+00_JPRB ,&
&  0.69664E+00_JPRB, 0.70730E+00_JPRB, 0.69512E+00_JPRB, 0.71265E+00_JPRB, 0.77547E+00_JPRB, 0.76887E+00_JPRB, 0.75834E+00_JPRB ,&
&  0.74273E+00_JPRB, 0.73678E+00_JPRB, 0.72237E+00_JPRB, 0.71049E+00_JPRB, 0.81516E+00_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.74298E+00_JPRB, 0.71705E+00_JPRB, 0.69431E+00_JPRB, 0.66609E+00_JPRB, 0.64767E+00_JPRB, 0.62754E+00_JPRB, 0.60781E+00_JPRB ,&
&  0.59042E+00_JPRB, 0.58073E+00_JPRB, 0.57198E+00_JPRB, 0.56361E+00_JPRB, 0.56569E+00_JPRB, 0.56147E+00_JPRB, 0.55063E+00_JPRB ,&
&  0.53434E+00_JPRB, 0.52219E+00_JPRB, 0.51919E+00_JPRB, 0.52305E+00_JPRB, 0.53662E+00_JPRB, 0.54750E+00_JPRB, 0.55653E+00_JPRB ,&
&  0.55974E+00_JPRB, 0.53838E+00_JPRB, 0.52996E+00_JPRB, 0.53385E+00_JPRB, 0.53714E+00_JPRB, 0.54227E+00_JPRB, 0.52522E+00_JPRB ,&
&  0.51513E+00_JPRB, 0.50157E+00_JPRB, 0.67620E+00_JPRB, 0.91996E+00_JPRB, 0.96426E+00_JPRB, 0.94626E+00_JPRB, 0.91403E+00_JPRB ,&
&  0.88299E+00_JPRB, 0.84308E+00_JPRB, 0.78349E+00_JPRB, 0.71788E+00_JPRB, 0.67172E+00_JPRB, 0.64510E+00_JPRB, 0.63398E+00_JPRB ,&
&  0.63638E+00_JPRB, 0.64107E+00_JPRB, 0.63771E+00_JPRB, 0.63726E+00_JPRB, 0.63466E+00_JPRB, 0.63305E+00_JPRB, 0.63418E+00_JPRB ,&
&  0.63175E+00_JPRB, 0.62628E+00_JPRB, 0.62027E+00_JPRB, 0.61976E+00_JPRB, 0.64896E+00_JPRB, 0.70107E+00_JPRB, 0.77188E+00_JPRB ,&
&  0.81302E+00_JPRB, 0.84226E+00_JPRB, 0.85720E+00_JPRB, 0.85563E+00_JPRB, 0.84394E+00_JPRB, 0.83349E+00_JPRB, 0.82689E+00_JPRB ,&
&  0.81222E+00_JPRB, 0.78304E+00_JPRB, 0.76269E+00_JPRB, 0.74710E+00_JPRB, 0.73134E+00_JPRB, 0.71879E+00_JPRB, 0.70754E+00_JPRB ,&
&  0.70475E+00_JPRB, 0.71100E+00_JPRB, 0.71144E+00_JPRB, 0.70596E+00_JPRB, 0.68950E+00_JPRB, 0.67241E+00_JPRB, 0.65801E+00_JPRB ,&
&  0.63720E+00_JPRB, 0.62562E+00_JPRB, 0.63342E+00_JPRB, 0.65567E+00_JPRB, 0.67849E+00_JPRB, 0.68446E+00_JPRB, 0.69186E+00_JPRB ,&
&  0.70640E+00_JPRB, 0.71896E+00_JPRB, 0.71863E+00_JPRB, 0.71509E+00_JPRB, 0.70936E+00_JPRB, 0.76493E+00_JPRB, 0.75962E+00_JPRB ,&
&  0.74096E+00_JPRB, 0.72370E+00_JPRB, 0.71494E+00_JPRB, 0.70534E+00_JPRB, 0.75901E+00_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.71035E+00_JPRB, 0.70216E+00_JPRB, 0.67372E+00_JPRB, 0.64635E+00_JPRB, 0.62856E+00_JPRB, 0.61621E+00_JPRB, 0.60469E+00_JPRB ,&
&  0.58741E+00_JPRB, 0.56529E+00_JPRB, 0.55277E+00_JPRB, 0.55118E+00_JPRB, 0.55367E+00_JPRB, 0.55317E+00_JPRB, 0.53860E+00_JPRB ,&
&  0.52050E+00_JPRB, 0.50863E+00_JPRB, 0.51308E+00_JPRB, 0.52198E+00_JPRB, 0.53779E+00_JPRB, 0.53816E+00_JPRB, 0.54861E+00_JPRB ,&
&  0.55360E+00_JPRB, 0.55052E+00_JPRB, 0.54794E+00_JPRB, 0.54676E+00_JPRB, 0.54297E+00_JPRB, 0.54011E+00_JPRB, 0.51916E+00_JPRB ,&
&  0.51815E+00_JPRB, 0.52744E+00_JPRB, 0.82307E+00_JPRB, 0.11276E+01_JPRB, 0.11794E+01_JPRB, 0.10861E+01_JPRB, 0.10227E+01_JPRB ,&
&  0.97256E+00_JPRB, 0.92294E+00_JPRB, 0.84661E+00_JPRB, 0.79201E+00_JPRB, 0.74633E+00_JPRB, 0.71647E+00_JPRB, 0.70125E+00_JPRB ,&
&  0.69636E+00_JPRB, 0.68759E+00_JPRB, 0.66931E+00_JPRB, 0.65227E+00_JPRB, 0.64394E+00_JPRB, 0.63783E+00_JPRB, 0.63843E+00_JPRB ,&
&  0.64253E+00_JPRB, 0.64144E+00_JPRB, 0.64102E+00_JPRB, 0.64032E+00_JPRB, 0.65449E+00_JPRB, 0.67238E+00_JPRB, 0.86228E+00_JPRB ,&
&  0.11407E+01_JPRB, 0.11314E+01_JPRB, 0.10983E+01_JPRB, 0.10407E+01_JPRB, 0.98147E+00_JPRB, 0.93922E+00_JPRB, 0.90216E+00_JPRB ,&
&  0.86814E+00_JPRB, 0.83496E+00_JPRB, 0.80830E+00_JPRB, 0.79187E+00_JPRB, 0.77865E+00_JPRB, 0.76664E+00_JPRB, 0.76374E+00_JPRB ,&
&  0.75032E+00_JPRB, 0.73790E+00_JPRB, 0.72468E+00_JPRB, 0.70905E+00_JPRB, 0.68475E+00_JPRB, 0.66141E+00_JPRB, 0.64358E+00_JPRB ,&
&  0.62513E+00_JPRB, 0.62521E+00_JPRB, 0.64152E+00_JPRB, 0.65593E+00_JPRB, 0.69412E+00_JPRB, 0.72499E+00_JPRB, 0.72395E+00_JPRB ,&
&  0.70505E+00_JPRB, 0.70681E+00_JPRB, 0.70917E+00_JPRB, 0.69296E+00_JPRB, 0.68411E+00_JPRB, 0.74487E+00_JPRB, 0.73510E+00_JPRB ,&
&  0.71635E+00_JPRB, 0.70766E+00_JPRB, 0.70208E+00_JPRB, 0.69289E+00_JPRB, 0.71093E+00_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.68348E+00_JPRB, 0.65838E+00_JPRB, 0.63658E+00_JPRB, 0.62230E+00_JPRB, 0.60922E+00_JPRB, 0.59897E+00_JPRB, 0.58621E+00_JPRB ,&
&  0.56576E+00_JPRB, 0.54764E+00_JPRB, 0.54524E+00_JPRB, 0.54448E+00_JPRB, 0.54125E+00_JPRB, 0.53792E+00_JPRB, 0.52082E+00_JPRB ,&
&  0.50458E+00_JPRB, 0.49125E+00_JPRB, 0.50346E+00_JPRB, 0.51725E+00_JPRB, 0.53222E+00_JPRB, 0.54399E+00_JPRB, 0.55869E+00_JPRB ,&
&  0.55438E+00_JPRB, 0.55509E+00_JPRB, 0.55738E+00_JPRB, 0.55795E+00_JPRB, 0.54854E+00_JPRB, 0.54926E+00_JPRB, 0.51910E+00_JPRB ,&
&  0.54793E+00_JPRB, 0.55636E+00_JPRB, 0.98708E+00_JPRB, 0.14175E+01_JPRB, 0.13531E+01_JPRB, 0.12263E+01_JPRB, 0.10935E+01_JPRB ,&
&  0.10712E+01_JPRB, 0.99008E+00_JPRB, 0.90266E+00_JPRB, 0.84345E+00_JPRB, 0.79127E+00_JPRB, 0.75110E+00_JPRB, 0.72706E+00_JPRB ,&
&  0.70886E+00_JPRB, 0.68082E+00_JPRB, 0.66076E+00_JPRB, 0.64473E+00_JPRB, 0.64295E+00_JPRB, 0.64332E+00_JPRB, 0.64976E+00_JPRB ,&
&  0.65596E+00_JPRB, 0.65525E+00_JPRB, 0.65464E+00_JPRB, 0.66511E+00_JPRB, 0.68851E+00_JPRB, 0.81902E+00_JPRB, 0.97989E+00_JPRB ,&
&  0.11581E+01_JPRB, 0.13688E+01_JPRB, 0.12778E+01_JPRB, 0.11570E+01_JPRB, 0.10662E+01_JPRB, 0.10004E+01_JPRB, 0.95735E+00_JPRB ,&
&  0.91401E+00_JPRB, 0.88337E+00_JPRB, 0.86510E+00_JPRB, 0.85480E+00_JPRB, 0.84660E+00_JPRB, 0.83584E+00_JPRB, 0.81155E+00_JPRB ,&
&  0.77762E+00_JPRB, 0.74537E+00_JPRB, 0.71704E+00_JPRB, 0.69152E+00_JPRB, 0.66480E+00_JPRB, 0.65621E+00_JPRB, 0.65242E+00_JPRB ,&
&  0.64366E+00_JPRB, 0.65341E+00_JPRB, 0.66289E+00_JPRB, 0.66396E+00_JPRB, 0.69153E+00_JPRB, 0.70231E+00_JPRB, 0.70320E+00_JPRB ,&
&  0.69969E+00_JPRB, 0.69898E+00_JPRB, 0.68704E+00_JPRB, 0.69688E+00_JPRB, 0.67522E+00_JPRB, 0.73766E+00_JPRB, 0.72215E+00_JPRB ,&
&  0.69792E+00_JPRB, 0.70053E+00_JPRB, 0.69507E+00_JPRB, 0.69507E+00_JPRB, 0.69613E+00_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.65696E+00_JPRB, 0.62570E+00_JPRB, 0.60567E+00_JPRB, 0.59899E+00_JPRB, 0.59348E+00_JPRB, 0.58152E+00_JPRB, 0.57180E+00_JPRB ,&
&  0.55657E+00_JPRB, 0.54098E+00_JPRB, 0.53428E+00_JPRB, 0.52734E+00_JPRB, 0.53172E+00_JPRB, 0.52711E+00_JPRB, 0.51581E+00_JPRB ,&
&  0.50176E+00_JPRB, 0.49976E+00_JPRB, 0.50385E+00_JPRB, 0.51467E+00_JPRB, 0.53592E+00_JPRB, 0.54610E+00_JPRB, 0.55918E+00_JPRB ,&
&  0.56806E+00_JPRB, 0.57651E+00_JPRB, 0.57603E+00_JPRB, 0.57063E+00_JPRB, 0.56386E+00_JPRB, 0.55300E+00_JPRB, 0.53491E+00_JPRB ,&
&  0.58577E+00_JPRB, 0.10512E+01_JPRB, 0.97837E+00_JPRB, 0.16947E+01_JPRB, 0.15287E+01_JPRB, 0.13823E+01_JPRB, 0.10766E+01_JPRB ,&
&  0.97205E+00_JPRB, 0.10087E+01_JPRB, 0.95706E+00_JPRB, 0.87673E+00_JPRB, 0.79762E+00_JPRB, 0.72813E+00_JPRB, 0.68577E+00_JPRB ,&
&  0.66895E+00_JPRB, 0.66719E+00_JPRB, 0.66270E+00_JPRB, 0.65271E+00_JPRB, 0.65691E+00_JPRB, 0.65580E+00_JPRB, 0.66229E+00_JPRB ,&
&  0.66820E+00_JPRB, 0.67094E+00_JPRB, 0.67714E+00_JPRB, 0.72861E+00_JPRB, 0.80523E+00_JPRB, 0.10231E+01_JPRB, 0.11636E+01_JPRB ,&
&  0.12009E+01_JPRB, 0.14208E+01_JPRB, 0.12880E+01_JPRB, 0.11407E+01_JPRB, 0.10579E+01_JPRB, 0.98885E+00_JPRB, 0.93965E+00_JPRB ,&
&  0.89757E+00_JPRB, 0.87530E+00_JPRB, 0.87164E+00_JPRB, 0.86581E+00_JPRB, 0.85487E+00_JPRB, 0.82396E+00_JPRB, 0.77771E+00_JPRB ,&
&  0.73514E+00_JPRB, 0.71189E+00_JPRB, 0.68752E+00_JPRB, 0.67427E+00_JPRB, 0.66843E+00_JPRB, 0.67075E+00_JPRB, 0.68009E+00_JPRB ,&
&  0.68235E+00_JPRB, 0.69395E+00_JPRB, 0.68326E+00_JPRB, 0.66462E+00_JPRB, 0.68205E+00_JPRB, 0.67760E+00_JPRB, 0.66214E+00_JPRB ,&
&  0.68169E+00_JPRB, 0.69008E+00_JPRB, 0.68769E+00_JPRB, 0.69738E+00_JPRB, 0.69219E+00_JPRB, 0.76376E+00_JPRB, 0.72446E+00_JPRB ,&
&  0.70071E+00_JPRB, 0.70575E+00_JPRB, 0.71537E+00_JPRB, 0.69911E+00_JPRB, 0.68514E+00_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.63688E+00_JPRB, 0.61868E+00_JPRB, 0.60736E+00_JPRB, 0.61355E+00_JPRB, 0.59233E+00_JPRB, 0.57803E+00_JPRB, 0.57609E+00_JPRB ,&
&  0.55117E+00_JPRB, 0.53673E+00_JPRB, 0.52948E+00_JPRB, 0.53319E+00_JPRB, 0.53810E+00_JPRB, 0.53195E+00_JPRB, 0.53063E+00_JPRB ,&
&  0.52038E+00_JPRB, 0.51220E+00_JPRB, 0.51534E+00_JPRB, 0.53581E+00_JPRB, 0.56243E+00_JPRB, 0.56971E+00_JPRB, 0.58386E+00_JPRB ,&
&  0.59843E+00_JPRB, 0.59241E+00_JPRB, 0.58214E+00_JPRB, 0.58222E+00_JPRB, 0.58732E+00_JPRB, 0.58323E+00_JPRB, 0.59414E+00_JPRB ,&
&  0.61816E+00_JPRB, 0.10471E+01_JPRB, 0.11485E+01_JPRB, 0.18977E+01_JPRB, 0.17003E+01_JPRB, 0.14366E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.96885E+00_JPRB, 0.10322E+01_JPRB, 0.94654E+00_JPRB, 0.85040E+00_JPRB, 0.74455E+00_JPRB, 0.66719E+00_JPRB, 0.64676E+00_JPRB ,&
&  0.66258E+00_JPRB, 0.67825E+00_JPRB, 0.69319E+00_JPRB, 0.68186E+00_JPRB, 0.67000E+00_JPRB, 0.66991E+00_JPRB, 0.67474E+00_JPRB ,&
&  0.68101E+00_JPRB, 0.69095E+00_JPRB, 0.73457E+00_JPRB, 0.82081E+00_JPRB, 0.10684E+01_JPRB, 0.12195E+01_JPRB, 0.11952E+01_JPRB ,&
&  0.11659E+01_JPRB, 0.12575E+01_JPRB, 0.11503E+01_JPRB, 0.10388E+01_JPRB, 0.95720E+00_JPRB, 0.93872E+00_JPRB, 0.90555E+00_JPRB ,&
&  0.88376E+00_JPRB, 0.87581E+00_JPRB, 0.84965E+00_JPRB, 0.81612E+00_JPRB, 0.78728E+00_JPRB, 0.75551E+00_JPRB, 0.72062E+00_JPRB ,&
&  0.70054E+00_JPRB, 0.69075E+00_JPRB, 0.68195E+00_JPRB, 0.68466E+00_JPRB, 0.69880E+00_JPRB, 0.71307E+00_JPRB, 0.72843E+00_JPRB ,&
&  0.73025E+00_JPRB, 0.73716E+00_JPRB, 0.70693E+00_JPRB, 0.69006E+00_JPRB, 0.70340E+00_JPRB, 0.68606E+00_JPRB, 0.67187E+00_JPRB ,&
&  0.69449E+00_JPRB, 0.71848E+00_JPRB, 0.73268E+00_JPRB, 0.72890E+00_JPRB, 0.73860E+00_JPRB, 0.79006E+00_JPRB, 0.75208E+00_JPRB ,&
&  0.72547E+00_JPRB, 0.75600E+00_JPRB, 0.76869E+00_JPRB, 0.70576E+00_JPRB, 0.67079E+00_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.64932E+00_JPRB, 0.64743E+00_JPRB, 0.63404E+00_JPRB, 0.62962E+00_JPRB, 0.59770E+00_JPRB, 0.57058E+00_JPRB, 0.54825E+00_JPRB ,&
&  0.53813E+00_JPRB, 0.53638E+00_JPRB, 0.54940E+00_JPRB, 0.55386E+00_JPRB, 0.55523E+00_JPRB, 0.54754E+00_JPRB, 0.54140E+00_JPRB ,&
&  0.52797E+00_JPRB, 0.53021E+00_JPRB, 0.54231E+00_JPRB, 0.56240E+00_JPRB, 0.58484E+00_JPRB, 0.59435E+00_JPRB, 0.60489E+00_JPRB ,&
&  0.60501E+00_JPRB, 0.60111E+00_JPRB, 0.60363E+00_JPRB, 0.61375E+00_JPRB, 0.62962E+00_JPRB, 0.64481E+00_JPRB, 0.66062E+00_JPRB ,&
&  0.67461E+00_JPRB, 0.10118E+01_JPRB, 0.14649E+01_JPRB, 0.17309E+01_JPRB, 0.15498E+01_JPRB, 0.12778E+01_JPRB, 0.10984E+01_JPRB ,&
&  0.92317E+00_JPRB, 0.78363E+00_JPRB, 0.87052E+00_JPRB, 0.79712E+00_JPRB, 0.71111E+00_JPRB, 0.67338E+00_JPRB, 0.67540E+00_JPRB ,&
&  0.69874E+00_JPRB, 0.70150E+00_JPRB, 0.70421E+00_JPRB, 0.71129E+00_JPRB, 0.70651E+00_JPRB, 0.70878E+00_JPRB, 0.71613E+00_JPRB ,&
&  0.74537E+00_JPRB, 0.80841E+00_JPRB, 0.91168E+00_JPRB, 0.10796E+01_JPRB, 0.12904E+01_JPRB, 0.11943E+01_JPRB, 0.10886E+01_JPRB ,&
&  0.10747E+01_JPRB, 0.10504E+01_JPRB, 0.99932E+00_JPRB, 0.95469E+00_JPRB, 0.83856E+00_JPRB, 0.91276E+00_JPRB, 0.88045E+00_JPRB ,&
&  0.85255E+00_JPRB, 0.82905E+00_JPRB, 0.79781E+00_JPRB, 0.77370E+00_JPRB, 0.75270E+00_JPRB, 0.72796E+00_JPRB, 0.71139E+00_JPRB ,&
&  0.70706E+00_JPRB, 0.70935E+00_JPRB, 0.71375E+00_JPRB, 0.72162E+00_JPRB, 0.73467E+00_JPRB, 0.75159E+00_JPRB, 0.76310E+00_JPRB ,&
&  0.76626E+00_JPRB, 0.77569E+00_JPRB, 0.76309E+00_JPRB, 0.76949E+00_JPRB, 0.77073E+00_JPRB, 0.74879E+00_JPRB, 0.72496E+00_JPRB ,&
&  0.73639E+00_JPRB, 0.77117E+00_JPRB, 0.78871E+00_JPRB, 0.79102E+00_JPRB, 0.79920E+00_JPRB, 0.82425E+00_JPRB, 0.77932E+00_JPRB ,&
&  0.76930E+00_JPRB, 0.83226E+00_JPRB, 0.79946E+00_JPRB, 0.71631E+00_JPRB, 0.67505E+00_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.66454E+00_JPRB, 0.66682E+00_JPRB, 0.65729E+00_JPRB, 0.62275E+00_JPRB, 0.59121E+00_JPRB, 0.55862E+00_JPRB, 0.53638E+00_JPRB ,&
&  0.53472E+00_JPRB, 0.54564E+00_JPRB, 0.57736E+00_JPRB, 0.59482E+00_JPRB, 0.58456E+00_JPRB, 0.56744E+00_JPRB, 0.56127E+00_JPRB ,&
&  0.56473E+00_JPRB, 0.57625E+00_JPRB, 0.58217E+00_JPRB, 0.59567E+00_JPRB, 0.61341E+00_JPRB, 0.63073E+00_JPRB, 0.64911E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.64163E+00_JPRB, 0.65606E+00_JPRB, 0.66453E+00_JPRB, 0.67883E+00_JPRB, 0.70796E+00_JPRB, 0.72241E+00_JPRB ,&
&  0.79409E+00_JPRB, 0.11161E+01_JPRB, 0.73922E+00_JPRB, 0.10370E+01_JPRB, 0.12197E+01_JPRB, 0.12151E+01_JPRB, 0.10801E+01_JPRB ,&
&  0.92529E+00_JPRB, 0.78154E+00_JPRB, 0.77900E+00_JPRB, 0.79778E+00_JPRB, 0.73968E+00_JPRB, 0.73431E+00_JPRB, 0.73687E+00_JPRB ,&
&  0.72186E+00_JPRB, 0.73191E+00_JPRB, 0.73380E+00_JPRB, 0.74382E+00_JPRB, 0.75196E+00_JPRB, 0.77072E+00_JPRB, 0.81904E+00_JPRB ,&
&  0.90777E+00_JPRB, 0.99984E+00_JPRB, 0.11536E+01_JPRB, 0.12212E+01_JPRB, 0.13671E+01_JPRB, 0.11281E+01_JPRB, 0.97508E+00_JPRB ,&
&  0.86946E+00_JPRB, 0.91534E+00_JPRB, 0.95409E+00_JPRB, 0.94634E+00_JPRB, 0.86890E+00_JPRB, 0.80646E+00_JPRB, 0.84381E+00_JPRB ,&
&  0.81307E+00_JPRB, 0.78869E+00_JPRB, 0.78238E+00_JPRB, 0.77124E+00_JPRB, 0.75450E+00_JPRB, 0.74252E+00_JPRB, 0.73670E+00_JPRB ,&
&  0.73309E+00_JPRB, 0.73906E+00_JPRB, 0.75082E+00_JPRB, 0.75611E+00_JPRB, 0.76704E+00_JPRB, 0.77798E+00_JPRB, 0.79425E+00_JPRB ,&
&  0.81105E+00_JPRB, 0.83411E+00_JPRB, 0.86135E+00_JPRB, 0.86912E+00_JPRB, 0.86309E+00_JPRB, 0.83868E+00_JPRB, 0.81942E+00_JPRB ,&
&  0.81649E+00_JPRB, 0.83616E+00_JPRB, 0.83160E+00_JPRB, 0.82575E+00_JPRB, 0.87801E+00_JPRB, 0.85643E+00_JPRB, 0.82334E+00_JPRB ,&
&  0.87427E+00_JPRB, 0.10299E+01_JPRB, 0.10834E+01_JPRB, 0.72484E+00_JPRB, 0.68170E+00_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.63419E+00_JPRB, 0.61737E+00_JPRB, 0.58996E+00_JPRB, 0.55617E+00_JPRB, 0.55233E+00_JPRB, 0.54927E+00_JPRB, 0.56096E+00_JPRB ,&
&  0.59556E+00_JPRB, 0.64343E+00_JPRB, 0.65819E+00_JPRB, 0.64973E+00_JPRB, 0.63314E+00_JPRB, 0.62692E+00_JPRB, 0.62248E+00_JPRB ,&
&  0.62362E+00_JPRB, 0.62137E+00_JPRB, 0.63587E+00_JPRB, 0.65536E+00_JPRB, 0.67811E+00_JPRB, 0.69442E+00_JPRB, 0.69493E+00_JPRB ,&
&  0.69891E+00_JPRB, 0.70341E+00_JPRB, 0.71897E+00_JPRB, 0.73531E+00_JPRB, 0.76856E+00_JPRB, 0.82951E+00_JPRB, 0.91461E+00_JPRB ,&
&  0.11357E+01_JPRB, 0.87932E+00_JPRB, 0.52734E+00_JPRB, 0.10307E+01_JPRB, 0.11945E+01_JPRB, 0.12603E+01_JPRB, 0.10644E+01_JPRB ,&
&  0.96563E+00_JPRB, 0.85214E+00_JPRB, 0.80467E+00_JPRB, 0.85776E+00_JPRB, 0.83261E+00_JPRB, 0.83047E+00_JPRB, 0.82951E+00_JPRB ,&
&  0.80164E+00_JPRB, 0.80073E+00_JPRB, 0.79542E+00_JPRB, 0.80631E+00_JPRB, 0.82891E+00_JPRB, 0.87586E+00_JPRB, 0.95839E+00_JPRB ,&
&  0.10506E+01_JPRB, 0.11637E+01_JPRB, 0.13083E+01_JPRB, 0.13031E+01_JPRB, 0.13320E+01_JPRB, 0.10438E+01_JPRB, 0.92380E+00_JPRB ,&
&  0.83225E+00_JPRB, 0.87223E+00_JPRB, 0.94160E+00_JPRB, 0.95746E+00_JPRB, 0.89689E+00_JPRB, 0.78612E+00_JPRB, 0.83117E+00_JPRB ,&
&  0.82014E+00_JPRB, 0.80238E+00_JPRB, 0.79785E+00_JPRB, 0.78128E+00_JPRB, 0.76614E+00_JPRB, 0.76044E+00_JPRB, 0.76158E+00_JPRB ,&
&  0.76194E+00_JPRB, 0.77173E+00_JPRB, 0.77557E+00_JPRB, 0.77696E+00_JPRB, 0.78477E+00_JPRB, 0.79024E+00_JPRB, 0.81707E+00_JPRB ,&
&  0.84225E+00_JPRB, 0.86240E+00_JPRB, 0.89035E+00_JPRB, 0.91469E+00_JPRB, 0.90467E+00_JPRB, 0.86850E+00_JPRB, 0.86038E+00_JPRB ,&
&  0.84730E+00_JPRB, 0.85599E+00_JPRB, 0.85079E+00_JPRB, 0.88116E+00_JPRB, 0.86192E+00_JPRB, 0.82165E+00_JPRB, 0.77389E+00_JPRB ,&
&  0.77880E+00_JPRB, 0.77702E+00_JPRB, 0.73674E+00_JPRB, 0.67375E+00_JPRB, 0.64467E+00_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.60644E+00_JPRB, 0.58576E+00_JPRB, 0.57267E+00_JPRB, 0.57843E+00_JPRB, 0.58712E+00_JPRB, 0.60622E+00_JPRB, 0.65297E+00_JPRB ,&
&  0.68926E+00_JPRB, 0.70595E+00_JPRB, 0.72616E+00_JPRB, 0.72361E+00_JPRB, 0.72878E+00_JPRB, 0.70627E+00_JPRB, 0.69538E+00_JPRB ,&
&  0.67782E+00_JPRB, 0.68554E+00_JPRB, 0.70147E+00_JPRB, 0.71603E+00_JPRB, 0.72751E+00_JPRB, 0.73589E+00_JPRB, 0.74639E+00_JPRB ,&
&  0.76381E+00_JPRB, 0.78887E+00_JPRB, 0.82643E+00_JPRB, 0.87533E+00_JPRB, 0.95703E+00_JPRB, 0.10595E+01_JPRB, 0.11072E+01_JPRB ,&
&  0.80503E+00_JPRB, 0.42195E+00_JPRB, 0.80802E+00_JPRB, 0.12052E+01_JPRB, 0.12579E+01_JPRB, 0.12650E+01_JPRB, 0.11710E+01_JPRB ,&
&  0.10257E+01_JPRB, 0.93258E+00_JPRB, 0.86670E+00_JPRB, 0.94154E+00_JPRB, 0.91742E+00_JPRB, 0.90827E+00_JPRB, 0.90102E+00_JPRB ,&
&  0.88557E+00_JPRB, 0.87855E+00_JPRB, 0.87810E+00_JPRB, 0.89631E+00_JPRB, 0.93062E+00_JPRB, 0.98739E+00_JPRB, 0.10606E+01_JPRB ,&
&  0.11595E+01_JPRB, 0.13089E+01_JPRB, 0.14257E+01_JPRB, 0.12851E+01_JPRB, 0.12660E+01_JPRB, 0.10771E+01_JPRB, 0.94038E+00_JPRB ,&
&  0.86662E+00_JPRB, 0.82581E+00_JPRB, 0.85908E+00_JPRB, 0.97598E+00_JPRB, 0.92969E+00_JPRB, 0.79657E+00_JPRB, 0.84514E+00_JPRB ,&
&  0.83906E+00_JPRB, 0.81942E+00_JPRB, 0.81239E+00_JPRB, 0.79176E+00_JPRB, 0.78413E+00_JPRB, 0.78365E+00_JPRB, 0.79034E+00_JPRB ,&
&  0.78949E+00_JPRB, 0.79129E+00_JPRB, 0.79137E+00_JPRB, 0.78710E+00_JPRB, 0.77326E+00_JPRB, 0.77845E+00_JPRB, 0.80187E+00_JPRB ,&
&  0.82181E+00_JPRB, 0.82796E+00_JPRB, 0.85269E+00_JPRB, 0.87739E+00_JPRB, 0.90563E+00_JPRB, 0.88646E+00_JPRB, 0.87497E+00_JPRB ,&
&  0.85889E+00_JPRB, 0.86048E+00_JPRB, 0.87582E+00_JPRB, 0.88409E+00_JPRB, 0.88739E+00_JPRB, 0.83971E+00_JPRB, 0.76556E+00_JPRB ,&
&  0.74021E+00_JPRB, 0.71084E+00_JPRB, 0.68599E+00_JPRB, 0.66261E+00_JPRB, 0.63646E+00_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.66598E+00_JPRB, 0.67241E+00_JPRB, 0.66495E+00_JPRB, 0.68521E+00_JPRB, 0.70390E+00_JPRB, 0.73239E+00_JPRB, 0.75736E+00_JPRB ,&
&  0.77909E+00_JPRB, 0.80036E+00_JPRB, 0.82655E+00_JPRB, 0.79948E+00_JPRB, 0.80220E+00_JPRB, 0.78336E+00_JPRB, 0.75405E+00_JPRB ,&
&  0.74505E+00_JPRB, 0.75542E+00_JPRB, 0.76687E+00_JPRB, 0.77674E+00_JPRB, 0.78449E+00_JPRB, 0.80396E+00_JPRB, 0.83249E+00_JPRB ,&
&  0.86935E+00_JPRB, 0.91664E+00_JPRB, 0.98147E+00_JPRB, 0.10318E+01_JPRB, 0.10879E+01_JPRB, 0.11009E+01_JPRB, 0.10299E+01_JPRB ,&
&  0.52392E+00_JPRB, 0.73115E+00_JPRB, 0.11597E+01_JPRB, 0.13109E+01_JPRB, 0.13654E+01_JPRB, 0.13342E+01_JPRB, 0.12505E+01_JPRB ,&
&  0.11094E+01_JPRB, 0.95252E+00_JPRB, 0.91633E+00_JPRB, 0.10143E+01_JPRB, 0.98030E+00_JPRB, 0.97483E+00_JPRB, 0.95976E+00_JPRB ,&
&  0.95189E+00_JPRB, 0.96253E+00_JPRB, 0.98109E+00_JPRB, 0.99581E+00_JPRB, 0.10347E+01_JPRB, 0.10874E+01_JPRB, 0.11650E+01_JPRB ,&
&  0.12689E+01_JPRB, 0.13831E+01_JPRB, 0.14901E+01_JPRB, 0.13003E+01_JPRB, 0.12492E+01_JPRB, 0.11271E+01_JPRB, 0.10199E+01_JPRB ,&
&  0.86214E+00_JPRB, 0.81283E+00_JPRB, 0.87391E+00_JPRB, 0.98880E+00_JPRB, 0.93844E+00_JPRB, 0.88650E+00_JPRB, 0.86594E+00_JPRB ,&
&  0.87100E+00_JPRB, 0.83570E+00_JPRB, 0.82531E+00_JPRB, 0.80459E+00_JPRB, 0.79087E+00_JPRB, 0.79617E+00_JPRB, 0.81058E+00_JPRB ,&
&  0.80211E+00_JPRB, 0.78414E+00_JPRB, 0.77977E+00_JPRB, 0.78431E+00_JPRB, 0.76824E+00_JPRB, 0.75387E+00_JPRB, 0.75241E+00_JPRB ,&
&  0.76086E+00_JPRB, 0.75005E+00_JPRB, 0.76124E+00_JPRB, 0.79446E+00_JPRB, 0.83857E+00_JPRB, 0.86639E+00_JPRB, 0.88049E+00_JPRB ,&
&  0.87413E+00_JPRB, 0.90003E+00_JPRB, 0.10410E+01_JPRB, 0.10986E+01_JPRB, 0.11066E+01_JPRB, 0.93548E+00_JPRB, 0.77025E+00_JPRB ,&
&  0.74926E+00_JPRB, 0.71951E+00_JPRB, 0.69151E+00_JPRB, 0.68067E+00_JPRB, 0.66885E+00_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.73529E+00_JPRB, 0.74660E+00_JPRB, 0.76574E+00_JPRB, 0.79749E+00_JPRB, 0.83236E+00_JPRB, 0.85586E+00_JPRB, 0.87865E+00_JPRB ,&
&  0.88631E+00_JPRB, 0.89610E+00_JPRB, 0.90299E+00_JPRB, 0.85373E+00_JPRB, 0.85926E+00_JPRB, 0.83653E+00_JPRB, 0.82463E+00_JPRB ,&
&  0.81443E+00_JPRB, 0.82667E+00_JPRB, 0.84209E+00_JPRB, 0.85345E+00_JPRB, 0.87245E+00_JPRB, 0.89880E+00_JPRB, 0.94126E+00_JPRB ,&
&  0.99502E+00_JPRB, 0.10426E+01_JPRB, 0.11055E+01_JPRB, 0.11381E+01_JPRB, 0.11735E+01_JPRB, 0.11896E+01_JPRB, 0.11535E+01_JPRB ,&
&  0.73565E+00_JPRB, 0.12315E+01_JPRB, 0.13470E+01_JPRB, 0.13986E+01_JPRB, 0.14038E+01_JPRB, 0.12764E+01_JPRB, 0.12153E+01_JPRB ,&
&  0.11128E+01_JPRB, 0.99356E+00_JPRB, 0.97284E+00_JPRB, 0.10166E+01_JPRB, 0.10708E+01_JPRB, 0.10558E+01_JPRB, 0.10484E+01_JPRB ,&
&  0.10454E+01_JPRB, 0.10628E+01_JPRB, 0.10844E+01_JPRB, 0.11076E+01_JPRB, 0.11448E+01_JPRB, 0.11872E+01_JPRB, 0.12568E+01_JPRB ,&
&  0.13288E+01_JPRB, 0.14063E+01_JPRB, 0.14721E+01_JPRB, 0.13308E+01_JPRB, 0.11901E+01_JPRB, 0.11417E+01_JPRB, 0.10556E+01_JPRB ,&
&  0.84011E+00_JPRB, 0.75068E+00_JPRB, 0.83669E+00_JPRB, 0.97073E+00_JPRB, 0.93095E+00_JPRB, 0.85506E+00_JPRB, 0.80402E+00_JPRB ,&
&  0.79306E+00_JPRB, 0.76065E+00_JPRB, 0.77241E+00_JPRB, 0.74004E+00_JPRB, 0.76100E+00_JPRB, 0.75546E+00_JPRB, 0.76340E+00_JPRB ,&
&  0.76856E+00_JPRB, 0.74018E+00_JPRB, 0.72745E+00_JPRB, 0.72230E+00_JPRB, 0.71383E+00_JPRB, 0.70458E+00_JPRB, 0.69776E+00_JPRB ,&
&  0.70688E+00_JPRB, 0.67242E+00_JPRB, 0.65337E+00_JPRB, 0.67284E+00_JPRB, 0.71327E+00_JPRB, 0.76213E+00_JPRB, 0.85804E+00_JPRB ,&
&  0.10176E+01_JPRB, 0.11871E+01_JPRB, 0.14800E+01_JPRB, 0.15755E+01_JPRB, 0.15326E+01_JPRB, 0.11489E+01_JPRB, 0.83671E+00_JPRB ,&
&  0.82807E+00_JPRB, 0.78235E+00_JPRB, 0.74602E+00_JPRB, 0.72485E+00_JPRB, 0.72578E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.87714E+00_JPRB, 0.89610E+00_JPRB, 0.91750E+00_JPRB, 0.93488E+00_JPRB, 0.93575E+00_JPRB, 0.93430E+00_JPRB, 0.93057E+00_JPRB ,&
&  0.95303E+00_JPRB, 0.92772E+00_JPRB, 0.94932E+00_JPRB, 0.92199E+00_JPRB, 0.93934E+00_JPRB, 0.91658E+00_JPRB, 0.91530E+00_JPRB ,&
&  0.92051E+00_JPRB, 0.94177E+00_JPRB, 0.96844E+00_JPRB, 0.97426E+00_JPRB, 0.96486E+00_JPRB, 0.98550E+00_JPRB, 0.10202E+01_JPRB ,&
&  0.10626E+01_JPRB, 0.10820E+01_JPRB, 0.11170E+01_JPRB, 0.11286E+01_JPRB, 0.11019E+01_JPRB, 0.10765E+01_JPRB, 0.77200E+00_JPRB ,&
&  0.11387E+01_JPRB, 0.13445E+01_JPRB, 0.14430E+01_JPRB, 0.13799E+01_JPRB, 0.12534E+01_JPRB, 0.11577E+01_JPRB, 0.11343E+01_JPRB ,&
&  0.11145E+01_JPRB, 0.10721E+01_JPRB, 0.10247E+01_JPRB, 0.10236E+01_JPRB, 0.11341E+01_JPRB, 0.11030E+01_JPRB, 0.11116E+01_JPRB ,&
&  0.11176E+01_JPRB, 0.11311E+01_JPRB, 0.11518E+01_JPRB, 0.11790E+01_JPRB, 0.11996E+01_JPRB, 0.12211E+01_JPRB, 0.12568E+01_JPRB ,&
&  0.12782E+01_JPRB, 0.12805E+01_JPRB, 0.13157E+01_JPRB, 0.11860E+01_JPRB, 0.11208E+01_JPRB, 0.10847E+01_JPRB, 0.97736E+00_JPRB ,&
&  0.75854E+00_JPRB, 0.71427E+00_JPRB, 0.81665E+00_JPRB, 0.92125E+00_JPRB, 0.89327E+00_JPRB, 0.80370E+00_JPRB, 0.75356E+00_JPRB ,&
&  0.72542E+00_JPRB, 0.70444E+00_JPRB, 0.67904E+00_JPRB, 0.68633E+00_JPRB, 0.68428E+00_JPRB, 0.69583E+00_JPRB, 0.70822E+00_JPRB ,&
&  0.72129E+00_JPRB, 0.70271E+00_JPRB, 0.68154E+00_JPRB, 0.67091E+00_JPRB, 0.67548E+00_JPRB, 0.72487E+00_JPRB, 0.78360E+00_JPRB ,&
&  0.80478E+00_JPRB, 0.52235E+00_JPRB, 0.57646E+00_JPRB, 0.57002E+00_JPRB, 0.59473E+00_JPRB, 0.66981E+00_JPRB, 0.85516E+00_JPRB ,&
&  0.13166E+01_JPRB, 0.20539E+01_JPRB, 0.23278E+01_JPRB, 0.19931E+01_JPRB, 0.24608E+01_JPRB, 0.17504E+01_JPRB, 0.95873E+00_JPRB ,&
&  0.93505E+00_JPRB, 0.90230E+00_JPRB, 0.86232E+00_JPRB, 0.84906E+00_JPRB, 0.85538E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.92839E+00_JPRB, 0.93972E+00_JPRB, 0.93496E+00_JPRB, 0.94624E+00_JPRB, 0.92285E+00_JPRB, 0.93780E+00_JPRB, 0.93606E+00_JPRB ,&
&  0.95179E+00_JPRB, 0.95828E+00_JPRB, 0.97325E+00_JPRB, 0.98495E+00_JPRB, 0.10072E+01_JPRB, 0.10261E+01_JPRB, 0.10384E+01_JPRB ,&
&  0.10579E+01_JPRB, 0.10770E+01_JPRB, 0.10923E+01_JPRB, 0.10988E+01_JPRB, 0.10713E+01_JPRB, 0.10664E+01_JPRB, 0.10544E+01_JPRB ,&
&  0.10662E+01_JPRB, 0.10543E+01_JPRB, 0.10618E+01_JPRB, 0.10596E+01_JPRB, 0.10363E+01_JPRB, 0.10356E+01_JPRB, 0.75762E+00_JPRB ,&
&  0.11745E+01_JPRB, 0.13271E+01_JPRB, 0.13364E+01_JPRB, 0.12314E+01_JPRB, 0.11331E+01_JPRB, 0.11005E+01_JPRB, 0.10928E+01_JPRB ,&
&  0.11124E+01_JPRB, 0.11215E+01_JPRB, 0.10642E+01_JPRB, 0.10894E+01_JPRB, 0.11030E+01_JPRB, 0.10739E+01_JPRB, 0.10802E+01_JPRB ,&
&  0.10834E+01_JPRB, 0.10895E+01_JPRB, 0.11009E+01_JPRB, 0.11010E+01_JPRB, 0.10847E+01_JPRB, 0.10938E+01_JPRB, 0.10967E+01_JPRB ,&
&  0.10784E+01_JPRB, 0.10828E+01_JPRB, 0.11071E+01_JPRB, 0.10438E+01_JPRB, 0.10097E+01_JPRB, 0.90026E+00_JPRB, 0.81776E+00_JPRB ,&
&  0.65851E+00_JPRB, 0.64995E+00_JPRB, 0.73706E+00_JPRB, 0.89625E+00_JPRB, 0.86351E+00_JPRB, 0.80316E+00_JPRB, 0.74664E+00_JPRB ,&
&  0.68410E+00_JPRB, 0.66667E+00_JPRB, 0.63984E+00_JPRB, 0.62903E+00_JPRB, 0.62595E+00_JPRB, 0.64126E+00_JPRB, 0.64967E+00_JPRB ,&
&  0.66119E+00_JPRB, 0.67103E+00_JPRB, 0.65771E+00_JPRB, 0.64071E+00_JPRB, 0.62700E+00_JPRB, 0.64141E+00_JPRB, 0.56038E+00_JPRB ,&
&  0.63329E+00_JPRB, 0.53980E+00_JPRB, 0.54131E+00_JPRB, 0.52677E+00_JPRB, 0.50134E+00_JPRB, 0.54597E+00_JPRB, 0.73123E+00_JPRB ,&
&  0.12279E+01_JPRB, 0.16007E+01_JPRB, 0.23097E+01_JPRB, 0.39319E+01_JPRB, 0.27602E+01_JPRB, 0.20254E+01_JPRB, 0.10196E+01_JPRB ,&
&  0.94846E+00_JPRB, 0.92114E+00_JPRB, 0.90523E+00_JPRB, 0.89309E+00_JPRB, 0.90894E+00_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.89106E+00_JPRB, 0.89953E+00_JPRB, 0.90720E+00_JPRB, 0.91531E+00_JPRB, 0.92115E+00_JPRB, 0.93653E+00_JPRB, 0.94119E+00_JPRB ,&
&  0.95131E+00_JPRB, 0.95612E+00_JPRB, 0.97027E+00_JPRB, 0.98894E+00_JPRB, 0.10018E+01_JPRB, 0.10225E+01_JPRB, 0.10475E+01_JPRB ,&
&  0.10759E+01_JPRB, 0.10922E+01_JPRB, 0.11131E+01_JPRB, 0.11158E+01_JPRB, 0.10987E+01_JPRB, 0.10957E+01_JPRB, 0.10889E+01_JPRB ,&
&  0.10710E+01_JPRB, 0.10606E+01_JPRB, 0.10668E+01_JPRB, 0.10760E+01_JPRB, 0.10646E+01_JPRB, 0.10514E+01_JPRB, 0.78968E+00_JPRB ,&
&  0.11260E+01_JPRB, 0.12557E+01_JPRB, 0.12371E+01_JPRB, 0.11123E+01_JPRB, 0.10352E+01_JPRB, 0.10229E+01_JPRB, 0.10331E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.11111E+01_JPRB, 0.10677E+01_JPRB, 0.10015E+01_JPRB, 0.97337E+00_JPRB, 0.96689E+00_JPRB, 0.95867E+00_JPRB ,&
&  0.95786E+00_JPRB, 0.96015E+00_JPRB, 0.95356E+00_JPRB, 0.93129E+00_JPRB, 0.90184E+00_JPRB, 0.87804E+00_JPRB, 0.84859E+00_JPRB ,&
&  0.81777E+00_JPRB, 0.80363E+00_JPRB, 0.73759E+00_JPRB, 0.76692E+00_JPRB, 0.74394E+00_JPRB, 0.66687E+00_JPRB, 0.57652E+00_JPRB ,&
&  0.49401E+00_JPRB, 0.53864E+00_JPRB, 0.66880E+00_JPRB, 0.88157E+00_JPRB, 0.84734E+00_JPRB, 0.78612E+00_JPRB, 0.75294E+00_JPRB ,&
&  0.70447E+00_JPRB, 0.65721E+00_JPRB, 0.62135E+00_JPRB, 0.59968E+00_JPRB, 0.58999E+00_JPRB, 0.60266E+00_JPRB, 0.61508E+00_JPRB ,&
&  0.61883E+00_JPRB, 0.62250E+00_JPRB, 0.60994E+00_JPRB, 0.60197E+00_JPRB, 0.58180E+00_JPRB, 0.51743E+00_JPRB, 0.56922E+00_JPRB ,&
&  0.55511E+00_JPRB, 0.45082E+00_JPRB, 0.52199E+00_JPRB, 0.37161E+00_JPRB, 0.48128E+00_JPRB, 0.51916E+00_JPRB, 0.62845E+00_JPRB ,&
&  0.75941E+00_JPRB, 0.10795E+01_JPRB, 0.14999E+01_JPRB, 0.13011E+01_JPRB, 0.11607E+01_JPRB, 0.10450E+01_JPRB, 0.94972E+00_JPRB ,&
&  0.90880E+00_JPRB, 0.89222E+00_JPRB, 0.88520E+00_JPRB, 0.87780E+00_JPRB, 0.88201E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.86704E+00_JPRB, 0.87443E+00_JPRB, 0.88102E+00_JPRB, 0.89235E+00_JPRB, 0.90313E+00_JPRB, 0.91765E+00_JPRB, 0.92775E+00_JPRB ,&
&  0.93807E+00_JPRB, 0.94499E+00_JPRB, 0.94409E+00_JPRB, 0.95672E+00_JPRB, 0.96657E+00_JPRB, 0.98673E+00_JPRB, 0.10096E+01_JPRB ,&
&  0.10335E+01_JPRB, 0.10586E+01_JPRB, 0.10866E+01_JPRB, 0.10989E+01_JPRB, 0.11079E+01_JPRB, 0.11212E+01_JPRB, 0.11327E+01_JPRB ,&
&  0.11387E+01_JPRB, 0.11426E+01_JPRB, 0.11555E+01_JPRB, 0.11788E+01_JPRB, 0.12065E+01_JPRB, 0.12345E+01_JPRB, 0.10312E+01_JPRB ,&
&  0.11086E+01_JPRB, 0.11682E+01_JPRB, 0.11002E+01_JPRB, 0.98245E+00_JPRB, 0.93828E+00_JPRB, 0.90795E+00_JPRB, 0.89687E+00_JPRB ,&
&  0.91070E+00_JPRB, 0.88991E+00_JPRB, 0.86624E+00_JPRB, 0.82324E+00_JPRB, 0.80275E+00_JPRB, 0.81734E+00_JPRB, 0.83840E+00_JPRB ,&
&  0.84410E+00_JPRB, 0.84526E+00_JPRB, 0.83233E+00_JPRB, 0.81155E+00_JPRB, 0.79054E+00_JPRB, 0.77024E+00_JPRB, 0.73763E+00_JPRB ,&
&  0.69845E+00_JPRB, 0.68738E+00_JPRB, 0.60205E+00_JPRB, 0.60551E+00_JPRB, 0.61945E+00_JPRB, 0.59136E+00_JPRB, 0.52291E+00_JPRB ,&
&  0.45411E+00_JPRB, 0.44766E+00_JPRB, 0.51090E+00_JPRB, 0.80255E+00_JPRB, 0.85549E+00_JPRB, 0.80892E+00_JPRB, 0.76642E+00_JPRB ,&
&  0.72358E+00_JPRB, 0.69171E+00_JPRB, 0.63382E+00_JPRB, 0.59384E+00_JPRB, 0.56955E+00_JPRB, 0.57725E+00_JPRB, 0.58329E+00_JPRB ,&
&  0.58083E+00_JPRB, 0.57085E+00_JPRB, 0.55746E+00_JPRB, 0.54110E+00_JPRB, 0.54676E+00_JPRB, 0.49121E+00_JPRB, 0.54147E+00_JPRB ,&
&  0.52194E+00_JPRB, 0.38935E+00_JPRB, 0.41624E+00_JPRB, 0.52852E+00_JPRB, 0.51736E+00_JPRB, 0.58831E+00_JPRB, 0.70508E+00_JPRB ,&
&  0.90787E+00_JPRB, 0.10231E+01_JPRB, 0.99696E+00_JPRB, 0.95667E+00_JPRB, 0.89583E+00_JPRB, 0.89106E+00_JPRB, 0.86761E+00_JPRB ,&
&  0.84884E+00_JPRB, 0.84491E+00_JPRB, 0.84602E+00_JPRB, 0.84907E+00_JPRB, 0.85702E+00_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.82364E+00_JPRB, 0.83793E+00_JPRB, 0.84901E+00_JPRB, 0.85607E+00_JPRB, 0.85911E+00_JPRB, 0.86218E+00_JPRB, 0.86814E+00_JPRB ,&
&  0.87057E+00_JPRB, 0.86807E+00_JPRB, 0.86576E+00_JPRB, 0.87971E+00_JPRB, 0.89686E+00_JPRB, 0.90347E+00_JPRB, 0.92165E+00_JPRB ,&
&  0.91519E+00_JPRB, 0.92456E+00_JPRB, 0.93984E+00_JPRB, 0.94910E+00_JPRB, 0.96877E+00_JPRB, 0.96866E+00_JPRB, 0.95362E+00_JPRB ,&
&  0.94723E+00_JPRB, 0.93874E+00_JPRB, 0.93751E+00_JPRB, 0.93665E+00_JPRB, 0.94525E+00_JPRB, 0.95801E+00_JPRB, 0.11096E+01_JPRB ,&
&  0.97454E+00_JPRB, 0.11021E+01_JPRB, 0.99348E+00_JPRB, 0.86779E+00_JPRB, 0.85719E+00_JPRB, 0.80043E+00_JPRB, 0.77470E+00_JPRB ,&
&  0.74078E+00_JPRB, 0.70149E+00_JPRB, 0.67433E+00_JPRB, 0.65179E+00_JPRB, 0.66060E+00_JPRB, 0.68683E+00_JPRB, 0.71376E+00_JPRB ,&
&  0.74872E+00_JPRB, 0.75630E+00_JPRB, 0.75392E+00_JPRB, 0.74691E+00_JPRB, 0.73348E+00_JPRB, 0.74688E+00_JPRB, 0.72157E+00_JPRB ,&
&  0.72158E+00_JPRB, 0.76254E+00_JPRB, 0.69583E+00_JPRB, 0.72452E+00_JPRB, 0.73322E+00_JPRB, 0.71672E+00_JPRB, 0.66170E+00_JPRB ,&
&  0.58034E+00_JPRB, 0.54813E+00_JPRB, 0.60679E+00_JPRB, 0.72498E+00_JPRB, 0.80629E+00_JPRB, 0.84872E+00_JPRB, 0.79292E+00_JPRB ,&
&  0.74284E+00_JPRB, 0.71310E+00_JPRB, 0.66693E+00_JPRB, 0.61223E+00_JPRB, 0.59161E+00_JPRB, 0.58532E+00_JPRB, 0.58043E+00_JPRB ,&
&  0.56479E+00_JPRB, 0.55060E+00_JPRB, 0.53324E+00_JPRB, 0.52435E+00_JPRB, 0.47112E+00_JPRB, 0.53957E+00_JPRB, 0.56149E+00_JPRB ,&
&  0.50735E+00_JPRB, 0.44540E+00_JPRB, 0.40246E+00_JPRB, 0.53854E+00_JPRB, 0.56608E+00_JPRB, 0.62949E+00_JPRB, 0.70306E+00_JPRB ,&
&  0.79498E+00_JPRB, 0.86288E+00_JPRB, 0.88036E+00_JPRB, 0.89767E+00_JPRB, 0.78740E+00_JPRB, 0.76396E+00_JPRB, 0.75450E+00_JPRB ,&
&  0.75737E+00_JPRB, 0.76707E+00_JPRB, 0.77873E+00_JPRB, 0.79149E+00_JPRB, 0.80787E+00_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.75345E+00_JPRB, 0.77150E+00_JPRB, 0.79243E+00_JPRB, 0.79416E+00_JPRB, 0.80023E+00_JPRB, 0.79642E+00_JPRB, 0.79782E+00_JPRB ,&
&  0.79004E+00_JPRB, 0.78929E+00_JPRB, 0.79145E+00_JPRB, 0.79629E+00_JPRB, 0.80337E+00_JPRB, 0.81282E+00_JPRB, 0.83969E+00_JPRB ,&
&  0.83917E+00_JPRB, 0.84395E+00_JPRB, 0.85080E+00_JPRB, 0.86425E+00_JPRB, 0.88059E+00_JPRB, 0.88400E+00_JPRB, 0.86514E+00_JPRB ,&
&  0.85122E+00_JPRB, 0.85132E+00_JPRB, 0.86245E+00_JPRB, 0.88064E+00_JPRB, 0.10142E+01_JPRB, 0.12933E+01_JPRB, 0.17183E+01_JPRB ,&
&  0.13998E+01_JPRB, 0.10171E+01_JPRB, 0.93137E+00_JPRB, 0.80645E+00_JPRB, 0.79837E+00_JPRB, 0.81759E+00_JPRB, 0.78500E+00_JPRB ,&
&  0.76391E+00_JPRB, 0.72807E+00_JPRB, 0.70885E+00_JPRB, 0.70530E+00_JPRB, 0.70429E+00_JPRB, 0.68475E+00_JPRB, 0.68013E+00_JPRB ,&
&  0.66098E+00_JPRB, 0.65834E+00_JPRB, 0.67186E+00_JPRB, 0.69108E+00_JPRB, 0.75742E+00_JPRB, 0.82313E+00_JPRB, 0.81248E+00_JPRB ,&
&  0.87640E+00_JPRB, 0.90247E+00_JPRB, 0.87447E+00_JPRB, 0.89485E+00_JPRB, 0.90749E+00_JPRB, 0.88177E+00_JPRB, 0.84391E+00_JPRB ,&
&  0.85580E+00_JPRB, 0.74960E+00_JPRB, 0.59876E+00_JPRB, 0.66839E+00_JPRB, 0.74284E+00_JPRB, 0.85940E+00_JPRB, 0.81775E+00_JPRB ,&
&  0.75761E+00_JPRB, 0.72236E+00_JPRB, 0.68260E+00_JPRB, 0.63425E+00_JPRB, 0.61213E+00_JPRB, 0.60730E+00_JPRB, 0.59590E+00_JPRB ,&
&  0.56805E+00_JPRB, 0.56333E+00_JPRB, 0.55395E+00_JPRB, 0.54018E+00_JPRB, 0.55253E+00_JPRB, 0.48713E+00_JPRB, 0.56481E+00_JPRB ,&
&  0.52002E+00_JPRB, 0.51670E+00_JPRB, 0.45139E+00_JPRB, 0.57485E+00_JPRB, 0.60952E+00_JPRB, 0.63583E+00_JPRB, 0.66571E+00_JPRB ,&
&  0.69910E+00_JPRB, 0.71443E+00_JPRB, 0.73459E+00_JPRB, 0.74856E+00_JPRB, 0.73834E+00_JPRB, 0.72459E+00_JPRB, 0.72915E+00_JPRB ,&
&  0.73254E+00_JPRB, 0.72738E+00_JPRB, 0.72553E+00_JPRB, 0.72661E+00_JPRB, 0.73955E+00_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.65088E+00_JPRB, 0.66075E+00_JPRB, 0.67656E+00_JPRB, 0.68789E+00_JPRB, 0.71009E+00_JPRB, 0.70498E+00_JPRB, 0.70209E+00_JPRB ,&
&  0.67897E+00_JPRB, 0.67618E+00_JPRB, 0.67156E+00_JPRB, 0.67009E+00_JPRB, 0.67351E+00_JPRB, 0.69427E+00_JPRB, 0.73362E+00_JPRB ,&
&  0.74541E+00_JPRB, 0.75053E+00_JPRB, 0.74710E+00_JPRB, 0.74071E+00_JPRB, 0.71611E+00_JPRB, 0.69613E+00_JPRB, 0.68588E+00_JPRB ,&
&  0.71094E+00_JPRB, 0.88711E+00_JPRB, 0.10916E+01_JPRB, 0.12449E+01_JPRB, 0.14034E+01_JPRB, 0.12329E+01_JPRB, 0.13698E+01_JPRB ,&
&  0.12722E+01_JPRB, 0.10140E+01_JPRB, 0.10467E+01_JPRB, 0.94582E+00_JPRB, 0.93670E+00_JPRB, 0.91745E+00_JPRB, 0.89090E+00_JPRB ,&
&  0.89686E+00_JPRB, 0.88698E+00_JPRB, 0.86118E+00_JPRB, 0.84567E+00_JPRB, 0.85140E+00_JPRB, 0.86912E+00_JPRB, 0.87931E+00_JPRB ,&
&  0.89260E+00_JPRB, 0.87619E+00_JPRB, 0.87836E+00_JPRB, 0.90691E+00_JPRB, 0.93468E+00_JPRB, 0.95382E+00_JPRB, 0.98841E+00_JPRB ,&
&  0.10345E+01_JPRB, 0.10313E+01_JPRB, 0.92983E+00_JPRB, 0.93303E+00_JPRB, 0.98682E+00_JPRB, 0.90080E+00_JPRB, 0.90591E+00_JPRB ,&
&  0.97977E+00_JPRB, 0.86891E+00_JPRB, 0.60414E+00_JPRB, 0.65192E+00_JPRB, 0.72107E+00_JPRB, 0.77914E+00_JPRB, 0.87281E+00_JPRB ,&
&  0.80333E+00_JPRB, 0.77822E+00_JPRB, 0.70063E+00_JPRB, 0.65868E+00_JPRB, 0.64755E+00_JPRB, 0.66958E+00_JPRB, 0.64840E+00_JPRB ,&
&  0.65707E+00_JPRB, 0.58395E+00_JPRB, 0.57106E+00_JPRB, 0.57191E+00_JPRB, 0.56513E+00_JPRB, 0.56434E+00_JPRB, 0.55889E+00_JPRB ,&
&  0.56547E+00_JPRB, 0.55110E+00_JPRB, 0.55255E+00_JPRB, 0.56691E+00_JPRB, 0.53888E+00_JPRB, 0.64946E+00_JPRB, 0.65872E+00_JPRB ,&
&  0.66268E+00_JPRB, 0.65316E+00_JPRB, 0.63824E+00_JPRB, 0.63561E+00_JPRB, 0.62408E+00_JPRB, 0.61445E+00_JPRB, 0.62476E+00_JPRB ,&
&  0.64162E+00_JPRB, 0.63784E+00_JPRB, 0.63239E+00_JPRB, 0.63209E+00_JPRB, 0.63884E+00_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.55257E+00_JPRB, 0.55618E+00_JPRB, 0.55908E+00_JPRB, 0.56683E+00_JPRB, 0.58003E+00_JPRB, 0.58626E+00_JPRB, 0.58289E+00_JPRB ,&
&  0.57015E+00_JPRB, 0.56441E+00_JPRB, 0.54598E+00_JPRB, 0.52281E+00_JPRB, 0.52312E+00_JPRB, 0.56685E+00_JPRB, 0.61358E+00_JPRB ,&
&  0.64605E+00_JPRB, 0.66465E+00_JPRB, 0.67197E+00_JPRB, 0.68281E+00_JPRB, 0.72854E+00_JPRB, 0.76938E+00_JPRB, 0.97448E+00_JPRB ,&
&  0.13204E+01_JPRB, 0.17738E+01_JPRB, 0.20516E+01_JPRB, 0.21228E+01_JPRB, 0.18903E+01_JPRB, 0.14008E+01_JPRB, 0.13889E+01_JPRB ,&
&  0.12858E+01_JPRB, 0.10911E+01_JPRB, 0.10377E+01_JPRB, 0.10283E+01_JPRB, 0.10113E+01_JPRB, 0.95980E+00_JPRB, 0.93260E+00_JPRB ,&
&  0.92540E+00_JPRB, 0.92658E+00_JPRB, 0.92958E+00_JPRB, 0.93835E+00_JPRB, 0.94771E+00_JPRB, 0.96550E+00_JPRB, 0.98401E+00_JPRB ,&
&  0.10001E+01_JPRB, 0.98893E+00_JPRB, 0.10332E+01_JPRB, 0.93297E+00_JPRB, 0.93356E+00_JPRB, 0.94513E+00_JPRB, 0.95649E+00_JPRB ,&
&  0.93842E+00_JPRB, 0.87928E+00_JPRB, 0.88725E+00_JPRB, 0.90745E+00_JPRB, 0.93073E+00_JPRB, 0.88156E+00_JPRB, 0.84641E+00_JPRB ,&
&  0.87813E+00_JPRB, 0.84802E+00_JPRB, 0.66084E+00_JPRB, 0.69024E+00_JPRB, 0.77291E+00_JPRB, 0.72373E+00_JPRB, 0.88998E+00_JPRB ,&
&  0.86379E+00_JPRB, 0.82248E+00_JPRB, 0.77151E+00_JPRB, 0.72185E+00_JPRB, 0.72143E+00_JPRB, 0.72738E+00_JPRB, 0.68971E+00_JPRB ,&
&  0.75834E+00_JPRB, 0.65077E+00_JPRB, 0.61638E+00_JPRB, 0.61686E+00_JPRB, 0.60464E+00_JPRB, 0.59462E+00_JPRB, 0.59345E+00_JPRB ,&
&  0.62464E+00_JPRB, 0.63251E+00_JPRB, 0.60785E+00_JPRB, 0.64145E+00_JPRB, 0.58623E+00_JPRB, 0.64028E+00_JPRB, 0.64796E+00_JPRB ,&
&  0.64475E+00_JPRB, 0.62461E+00_JPRB, 0.61877E+00_JPRB, 0.61217E+00_JPRB, 0.60922E+00_JPRB, 0.59326E+00_JPRB, 0.59613E+00_JPRB ,&
&  0.59123E+00_JPRB, 0.58533E+00_JPRB, 0.57611E+00_JPRB, 0.57366E+00_JPRB, 0.56120E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.58332E+00_JPRB, 0.57757E+00_JPRB, 0.56578E+00_JPRB, 0.55144E+00_JPRB, 0.53721E+00_JPRB, 0.51611E+00_JPRB, 0.52140E+00_JPRB ,&
&  0.51476E+00_JPRB, 0.52428E+00_JPRB, 0.54658E+00_JPRB, 0.56548E+00_JPRB, 0.60079E+00_JPRB, 0.65746E+00_JPRB, 0.71453E+00_JPRB ,&
&  0.76065E+00_JPRB, 0.78887E+00_JPRB, 0.85855E+00_JPRB, 0.96553E+00_JPRB, 0.11380E+01_JPRB, 0.14162E+01_JPRB, 0.18443E+01_JPRB ,&
&  0.20277E+01_JPRB, 0.21173E+01_JPRB, 0.20301E+01_JPRB, 0.17626E+01_JPRB, 0.14602E+01_JPRB, 0.12293E+01_JPRB, 0.12951E+01_JPRB ,&
&  0.13553E+01_JPRB, 0.13018E+01_JPRB, 0.11856E+01_JPRB, 0.11253E+01_JPRB, 0.10643E+01_JPRB, 0.10141E+01_JPRB, 0.98607E+00_JPRB ,&
&  0.97999E+00_JPRB, 0.97593E+00_JPRB, 0.96187E+00_JPRB, 0.96840E+00_JPRB, 0.96963E+00_JPRB, 0.97339E+00_JPRB, 0.98020E+00_JPRB ,&
&  0.97045E+00_JPRB, 0.95075E+00_JPRB, 0.94066E+00_JPRB, 0.89347E+00_JPRB, 0.88110E+00_JPRB, 0.87177E+00_JPRB, 0.84975E+00_JPRB ,&
&  0.82693E+00_JPRB, 0.78070E+00_JPRB, 0.79167E+00_JPRB, 0.81382E+00_JPRB, 0.80376E+00_JPRB, 0.75795E+00_JPRB, 0.76032E+00_JPRB ,&
&  0.79030E+00_JPRB, 0.82899E+00_JPRB, 0.68145E+00_JPRB, 0.78819E+00_JPRB, 0.91690E+00_JPRB, 0.88693E+00_JPRB, 0.89865E+00_JPRB ,&
&  0.92474E+00_JPRB, 0.88714E+00_JPRB, 0.86398E+00_JPRB, 0.80261E+00_JPRB, 0.81584E+00_JPRB, 0.80829E+00_JPRB, 0.73138E+00_JPRB ,&
&  0.84792E+00_JPRB, 0.73153E+00_JPRB, 0.70286E+00_JPRB, 0.67702E+00_JPRB, 0.64523E+00_JPRB, 0.61274E+00_JPRB, 0.59613E+00_JPRB ,&
&  0.59233E+00_JPRB, 0.77702E+00_JPRB, 0.89067E+00_JPRB, 0.10785E+01_JPRB, 0.10824E+01_JPRB, 0.65251E+00_JPRB, 0.67245E+00_JPRB ,&
&  0.66580E+00_JPRB, 0.65998E+00_JPRB, 0.65818E+00_JPRB, 0.65134E+00_JPRB, 0.64235E+00_JPRB, 0.63676E+00_JPRB, 0.63680E+00_JPRB ,&
&  0.62797E+00_JPRB, 0.62000E+00_JPRB, 0.61115E+00_JPRB, 0.60476E+00_JPRB, 0.59493E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.66889E+00_JPRB, 0.67268E+00_JPRB, 0.66317E+00_JPRB, 0.63849E+00_JPRB, 0.60499E+00_JPRB, 0.57003E+00_JPRB, 0.55049E+00_JPRB ,&
&  0.52957E+00_JPRB, 0.54986E+00_JPRB, 0.59284E+00_JPRB, 0.65102E+00_JPRB, 0.72360E+00_JPRB, 0.80258E+00_JPRB, 0.88214E+00_JPRB ,&
&  0.93762E+00_JPRB, 0.99784E+00_JPRB, 0.10909E+01_JPRB, 0.12920E+01_JPRB, 0.15101E+01_JPRB, 0.16671E+01_JPRB, 0.17500E+01_JPRB ,&
&  0.18051E+01_JPRB, 0.18385E+01_JPRB, 0.17489E+01_JPRB, 0.13275E+01_JPRB, 0.10919E+01_JPRB, 0.11968E+01_JPRB, 0.10965E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.11055E+01_JPRB, 0.11094E+01_JPRB, 0.11025E+01_JPRB, 0.10520E+01_JPRB, 0.10071E+01_JPRB, 0.98400E+00_JPRB ,&
&  0.97036E+00_JPRB, 0.97415E+00_JPRB, 0.97558E+00_JPRB, 0.96030E+00_JPRB, 0.96187E+00_JPRB, 0.97392E+00_JPRB, 0.96503E+00_JPRB ,&
&  0.93014E+00_JPRB, 0.91282E+00_JPRB, 0.89592E+00_JPRB, 0.89413E+00_JPRB, 0.87234E+00_JPRB, 0.85155E+00_JPRB, 0.81919E+00_JPRB ,&
&  0.79063E+00_JPRB, 0.76933E+00_JPRB, 0.77168E+00_JPRB, 0.78783E+00_JPRB, 0.78844E+00_JPRB, 0.73221E+00_JPRB, 0.74101E+00_JPRB ,&
&  0.77318E+00_JPRB, 0.79092E+00_JPRB, 0.74732E+00_JPRB, 0.84658E+00_JPRB, 0.65535E+00_JPRB, 0.76594E+00_JPRB, 0.93426E+00_JPRB ,&
&  0.96476E+00_JPRB, 0.89537E+00_JPRB, 0.84492E+00_JPRB, 0.81766E+00_JPRB, 0.77765E+00_JPRB, 0.74128E+00_JPRB, 0.81055E+00_JPRB ,&
&  0.84452E+00_JPRB, 0.79615E+00_JPRB, 0.75169E+00_JPRB, 0.70897E+00_JPRB, 0.69408E+00_JPRB, 0.66625E+00_JPRB, 0.64949E+00_JPRB ,&
&  0.63253E+00_JPRB, 0.77013E+00_JPRB, 0.81232E+00_JPRB, 0.84602E+00_JPRB, 0.85244E+00_JPRB, 0.64154E+00_JPRB, 0.66576E+00_JPRB ,&
&  0.66723E+00_JPRB, 0.65430E+00_JPRB, 0.65416E+00_JPRB, 0.65048E+00_JPRB, 0.63486E+00_JPRB, 0.63254E+00_JPRB, 0.64114E+00_JPRB ,&
&  0.64041E+00_JPRB, 0.64291E+00_JPRB, 0.64798E+00_JPRB, 0.65079E+00_JPRB, 0.65929E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.66404E+00_JPRB, 0.68203E+00_JPRB, 0.70643E+00_JPRB, 0.72129E+00_JPRB, 0.72481E+00_JPRB, 0.70973E+00_JPRB, 0.66281E+00_JPRB ,&
&  0.61413E+00_JPRB, 0.59257E+00_JPRB, 0.62544E+00_JPRB, 0.66681E+00_JPRB, 0.71731E+00_JPRB, 0.77966E+00_JPRB, 0.86835E+00_JPRB ,&
&  0.98084E+00_JPRB, 0.10839E+01_JPRB, 0.11903E+01_JPRB, 0.13575E+01_JPRB, 0.15367E+01_JPRB, 0.15545E+01_JPRB, 0.15685E+01_JPRB ,&
&  0.13904E+01_JPRB, 0.11743E+01_JPRB, 0.16883E+01_JPRB, 0.18812E+01_JPRB, 0.16394E+01_JPRB, 0.12490E+01_JPRB, 0.10546E+01_JPRB ,&
&  0.10456E+01_JPRB, 0.10937E+01_JPRB, 0.11123E+01_JPRB, 0.11107E+01_JPRB, 0.10206E+01_JPRB, 0.10017E+01_JPRB, 0.97824E+00_JPRB ,&
&  0.96275E+00_JPRB, 0.93437E+00_JPRB, 0.93436E+00_JPRB, 0.93520E+00_JPRB, 0.92034E+00_JPRB, 0.92591E+00_JPRB, 0.92314E+00_JPRB ,&
&  0.90274E+00_JPRB, 0.88136E+00_JPRB, 0.88094E+00_JPRB, 0.88028E+00_JPRB, 0.88323E+00_JPRB, 0.86758E+00_JPRB, 0.83245E+00_JPRB ,&
&  0.80311E+00_JPRB, 0.77374E+00_JPRB, 0.78652E+00_JPRB, 0.80859E+00_JPRB, 0.82209E+00_JPRB, 0.77226E+00_JPRB, 0.79115E+00_JPRB ,&
&  0.79360E+00_JPRB, 0.78545E+00_JPRB, 0.78267E+00_JPRB, 0.84378E+00_JPRB, 0.70620E+00_JPRB, 0.77594E+00_JPRB, 0.81231E+00_JPRB ,&
&  0.91546E+00_JPRB, 0.88786E+00_JPRB, 0.83114E+00_JPRB, 0.79100E+00_JPRB, 0.77088E+00_JPRB, 0.68913E+00_JPRB, 0.70782E+00_JPRB ,&
&  0.72952E+00_JPRB, 0.73194E+00_JPRB, 0.70218E+00_JPRB, 0.69367E+00_JPRB, 0.65500E+00_JPRB, 0.64741E+00_JPRB, 0.66995E+00_JPRB ,&
&  0.75754E+00_JPRB, 0.77947E+00_JPRB, 0.75925E+00_JPRB, 0.71183E+00_JPRB, 0.69730E+00_JPRB, 0.61580E+00_JPRB, 0.64722E+00_JPRB ,&
&  0.65185E+00_JPRB, 0.64397E+00_JPRB, 0.63053E+00_JPRB, 0.63074E+00_JPRB, 0.63157E+00_JPRB, 0.62650E+00_JPRB, 0.63241E+00_JPRB ,&
&  0.63581E+00_JPRB, 0.63960E+00_JPRB, 0.64240E+00_JPRB, 0.64646E+00_JPRB, 0.65220E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.64009E+00_JPRB, 0.67009E+00_JPRB, 0.71638E+00_JPRB, 0.78852E+00_JPRB, 0.86186E+00_JPRB, 0.96275E+00_JPRB, 0.10459E+01_JPRB ,&
&  0.92330E+00_JPRB, 0.59389E+00_JPRB, 0.61405E+00_JPRB, 0.63409E+00_JPRB, 0.66763E+00_JPRB, 0.71578E+00_JPRB, 0.78033E+00_JPRB ,&
&  0.88523E+00_JPRB, 0.10283E+01_JPRB, 0.11984E+01_JPRB, 0.13393E+01_JPRB, 0.14608E+01_JPRB, 0.14763E+01_JPRB, 0.13603E+01_JPRB ,&
&  0.10743E+01_JPRB, 0.13230E+01_JPRB, 0.22214E+01_JPRB, 0.20118E+01_JPRB, 0.17784E+01_JPRB, 0.14329E+01_JPRB, 0.11044E+01_JPRB ,&
&  0.10132E+01_JPRB, 0.10251E+01_JPRB, 0.10218E+01_JPRB, 0.99855E+00_JPRB, 0.95202E+00_JPRB, 0.94385E+00_JPRB, 0.94237E+00_JPRB ,&
&  0.92903E+00_JPRB, 0.90132E+00_JPRB, 0.87251E+00_JPRB, 0.87460E+00_JPRB, 0.86120E+00_JPRB, 0.85149E+00_JPRB, 0.85110E+00_JPRB ,&
&  0.86076E+00_JPRB, 0.85866E+00_JPRB, 0.85583E+00_JPRB, 0.84675E+00_JPRB, 0.86876E+00_JPRB, 0.87913E+00_JPRB, 0.84857E+00_JPRB ,&
&  0.81238E+00_JPRB, 0.80718E+00_JPRB, 0.81266E+00_JPRB, 0.79527E+00_JPRB, 0.75829E+00_JPRB, 0.80186E+00_JPRB, 0.78731E+00_JPRB ,&
&  0.80844E+00_JPRB, 0.78678E+00_JPRB, 0.82289E+00_JPRB, 0.73983E+00_JPRB, 0.76565E+00_JPRB, 0.83299E+00_JPRB, 0.84956E+00_JPRB ,&
&  0.85764E+00_JPRB, 0.88380E+00_JPRB, 0.83332E+00_JPRB, 0.81910E+00_JPRB, 0.78860E+00_JPRB, 0.70134E+00_JPRB, 0.70636E+00_JPRB ,&
&  0.67423E+00_JPRB, 0.70572E+00_JPRB, 0.67955E+00_JPRB, 0.64507E+00_JPRB, 0.57812E+00_JPRB, 0.55226E+00_JPRB, 0.56581E+00_JPRB ,&
&  0.65289E+00_JPRB, 0.75502E+00_JPRB, 0.77262E+00_JPRB, 0.74536E+00_JPRB, 0.68927E+00_JPRB, 0.63409E+00_JPRB, 0.64823E+00_JPRB ,&
&  0.64036E+00_JPRB, 0.62958E+00_JPRB, 0.62374E+00_JPRB, 0.62678E+00_JPRB, 0.62919E+00_JPRB, 0.62508E+00_JPRB, 0.61929E+00_JPRB ,&
&  0.61675E+00_JPRB, 0.61494E+00_JPRB, 0.61277E+00_JPRB, 0.61618E+00_JPRB, 0.62539E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.58774E+00_JPRB, 0.60485E+00_JPRB, 0.63495E+00_JPRB, 0.67106E+00_JPRB, 0.70754E+00_JPRB, 0.74987E+00_JPRB, 0.73955E+00_JPRB ,&
&  0.64708E+00_JPRB, 0.59115E+00_JPRB, 0.60697E+00_JPRB, 0.60789E+00_JPRB, 0.61542E+00_JPRB, 0.64480E+00_JPRB, 0.69451E+00_JPRB ,&
&  0.75993E+00_JPRB, 0.88465E+00_JPRB, 0.10568E+01_JPRB, 0.12836E+01_JPRB, 0.13684E+01_JPRB, 0.13944E+01_JPRB, 0.12991E+01_JPRB ,&
&  0.10805E+01_JPRB, 0.22809E+01_JPRB, 0.20747E+01_JPRB, 0.21171E+01_JPRB, 0.19313E+01_JPRB, 0.17930E+01_JPRB, 0.13619E+01_JPRB ,&
&  0.10602E+01_JPRB, 0.99359E+00_JPRB, 0.99039E+00_JPRB, 0.92448E+00_JPRB, 0.88491E+00_JPRB, 0.87997E+00_JPRB, 0.88648E+00_JPRB ,&
&  0.87297E+00_JPRB, 0.85796E+00_JPRB, 0.83264E+00_JPRB, 0.82849E+00_JPRB, 0.82293E+00_JPRB, 0.81770E+00_JPRB, 0.81882E+00_JPRB ,&
&  0.82843E+00_JPRB, 0.83273E+00_JPRB, 0.82173E+00_JPRB, 0.82473E+00_JPRB, 0.83916E+00_JPRB, 0.87003E+00_JPRB, 0.87044E+00_JPRB ,&
&  0.82403E+00_JPRB, 0.78469E+00_JPRB, 0.81556E+00_JPRB, 0.81654E+00_JPRB, 0.80916E+00_JPRB, 0.84164E+00_JPRB, 0.79167E+00_JPRB ,&
&  0.82594E+00_JPRB, 0.78599E+00_JPRB, 0.81458E+00_JPRB, 0.72439E+00_JPRB, 0.75664E+00_JPRB, 0.85346E+00_JPRB, 0.88053E+00_JPRB ,&
&  0.85520E+00_JPRB, 0.89514E+00_JPRB, 0.84981E+00_JPRB, 0.83442E+00_JPRB, 0.80600E+00_JPRB, 0.74340E+00_JPRB, 0.69396E+00_JPRB ,&
&  0.69019E+00_JPRB, 0.70751E+00_JPRB, 0.69042E+00_JPRB, 0.60776E+00_JPRB, 0.56726E+00_JPRB, 0.52932E+00_JPRB, 0.53507E+00_JPRB ,&
&  0.58722E+00_JPRB, 0.66578E+00_JPRB, 0.74187E+00_JPRB, 0.78658E+00_JPRB, 0.77002E+00_JPRB, 0.73062E+00_JPRB, 0.69988E+00_JPRB ,&
&  0.66774E+00_JPRB, 0.63763E+00_JPRB, 0.62497E+00_JPRB, 0.62052E+00_JPRB, 0.62021E+00_JPRB, 0.62107E+00_JPRB, 0.61299E+00_JPRB ,&
&  0.60724E+00_JPRB, 0.60139E+00_JPRB, 0.59624E+00_JPRB, 0.59119E+00_JPRB, 0.58845E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.57973E+00_JPRB, 0.57183E+00_JPRB, 0.58788E+00_JPRB, 0.62147E+00_JPRB, 0.63674E+00_JPRB, 0.64812E+00_JPRB, 0.62687E+00_JPRB ,&
&  0.56222E+00_JPRB, 0.54606E+00_JPRB, 0.54753E+00_JPRB, 0.55290E+00_JPRB, 0.56464E+00_JPRB, 0.59713E+00_JPRB, 0.64418E+00_JPRB ,&
&  0.70320E+00_JPRB, 0.78887E+00_JPRB, 0.87593E+00_JPRB, 0.10387E+01_JPRB, 0.11782E+01_JPRB, 0.13328E+01_JPRB, 0.93797E+00_JPRB ,&
&  0.11767E+01_JPRB, 0.21391E+01_JPRB, 0.22096E+01_JPRB, 0.22415E+01_JPRB, 0.22044E+01_JPRB, 0.21283E+01_JPRB, 0.16958E+01_JPRB ,&
&  0.13526E+01_JPRB, 0.11203E+01_JPRB, 0.10022E+01_JPRB, 0.90346E+00_JPRB, 0.82796E+00_JPRB, 0.83841E+00_JPRB, 0.84504E+00_JPRB ,&
&  0.85058E+00_JPRB, 0.86975E+00_JPRB, 0.81902E+00_JPRB, 0.80280E+00_JPRB, 0.78928E+00_JPRB, 0.77237E+00_JPRB, 0.78644E+00_JPRB ,&
&  0.80155E+00_JPRB, 0.80455E+00_JPRB, 0.80873E+00_JPRB, 0.78413E+00_JPRB, 0.82088E+00_JPRB, 0.85008E+00_JPRB, 0.86739E+00_JPRB ,&
&  0.81144E+00_JPRB, 0.77087E+00_JPRB, 0.82592E+00_JPRB, 0.89286E+00_JPRB, 0.93208E+00_JPRB, 0.92564E+00_JPRB, 0.86644E+00_JPRB ,&
&  0.84804E+00_JPRB, 0.79971E+00_JPRB, 0.76514E+00_JPRB, 0.70396E+00_JPRB, 0.73980E+00_JPRB, 0.84169E+00_JPRB, 0.92747E+00_JPRB ,&
&  0.93722E+00_JPRB, 0.89992E+00_JPRB, 0.87566E+00_JPRB, 0.83268E+00_JPRB, 0.81322E+00_JPRB, 0.77069E+00_JPRB, 0.77308E+00_JPRB ,&
&  0.82648E+00_JPRB, 0.87453E+00_JPRB, 0.75051E+00_JPRB, 0.63959E+00_JPRB, 0.60001E+00_JPRB, 0.52175E+00_JPRB, 0.51157E+00_JPRB ,&
&  0.54092E+00_JPRB, 0.56598E+00_JPRB, 0.60250E+00_JPRB, 0.71944E+00_JPRB, 0.80408E+00_JPRB, 0.84156E+00_JPRB, 0.79284E+00_JPRB ,&
&  0.69652E+00_JPRB, 0.67978E+00_JPRB, 0.65481E+00_JPRB, 0.63424E+00_JPRB, 0.63076E+00_JPRB, 0.62423E+00_JPRB, 0.61954E+00_JPRB ,&
&  0.60883E+00_JPRB, 0.60194E+00_JPRB, 0.59830E+00_JPRB, 0.59733E+00_JPRB, 0.59338E+00_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.59530E+00_JPRB, 0.58419E+00_JPRB, 0.57268E+00_JPRB, 0.57787E+00_JPRB, 0.60203E+00_JPRB, 0.59746E+00_JPRB, 0.57542E+00_JPRB ,&
&  0.52955E+00_JPRB, 0.50644E+00_JPRB, 0.48427E+00_JPRB, 0.47777E+00_JPRB, 0.48940E+00_JPRB, 0.52034E+00_JPRB, 0.57226E+00_JPRB ,&
&  0.64574E+00_JPRB, 0.71417E+00_JPRB, 0.78622E+00_JPRB, 0.83832E+00_JPRB, 0.92474E+00_JPRB, 0.88296E+00_JPRB, 0.98207E+00_JPRB ,&
&  0.14925E+01_JPRB, 0.19913E+01_JPRB, 0.21735E+01_JPRB, 0.22472E+01_JPRB, 0.23423E+01_JPRB, 0.24836E+01_JPRB, 0.21135E+01_JPRB ,&
&  0.17557E+01_JPRB, 0.14506E+01_JPRB, 0.11615E+01_JPRB, 0.99140E+00_JPRB, 0.87310E+00_JPRB, 0.80966E+00_JPRB, 0.80833E+00_JPRB ,&
&  0.81121E+00_JPRB, 0.82252E+00_JPRB, 0.79670E+00_JPRB, 0.77128E+00_JPRB, 0.75526E+00_JPRB, 0.73792E+00_JPRB, 0.74137E+00_JPRB ,&
&  0.75475E+00_JPRB, 0.76335E+00_JPRB, 0.76467E+00_JPRB, 0.75779E+00_JPRB, 0.78300E+00_JPRB, 0.81802E+00_JPRB, 0.85969E+00_JPRB ,&
&  0.84069E+00_JPRB, 0.84857E+00_JPRB, 0.88154E+00_JPRB, 0.91092E+00_JPRB, 0.99185E+00_JPRB, 0.10346E+01_JPRB, 0.10054E+01_JPRB ,&
&  0.92077E+00_JPRB, 0.80410E+00_JPRB, 0.69949E+00_JPRB, 0.69210E+00_JPRB, 0.75313E+00_JPRB, 0.82475E+00_JPRB, 0.82535E+00_JPRB ,&
&  0.78960E+00_JPRB, 0.79662E+00_JPRB, 0.80932E+00_JPRB, 0.86540E+00_JPRB, 0.90797E+00_JPRB, 0.90180E+00_JPRB, 0.97818E+00_JPRB ,&
&  0.87376E+00_JPRB, 0.57342E+00_JPRB, 0.50993E+00_JPRB, 0.55986E+00_JPRB, 0.48429E+00_JPRB, 0.46397E+00_JPRB, 0.51661E+00_JPRB ,&
&  0.53042E+00_JPRB, 0.55823E+00_JPRB, 0.58111E+00_JPRB, 0.58882E+00_JPRB, 0.70056E+00_JPRB, 0.82286E+00_JPRB, 0.85461E+00_JPRB ,&
&  0.74845E+00_JPRB, 0.72829E+00_JPRB, 0.70491E+00_JPRB, 0.68715E+00_JPRB, 0.67384E+00_JPRB, 0.65134E+00_JPRB, 0.63772E+00_JPRB ,&
&  0.62322E+00_JPRB, 0.61104E+00_JPRB, 0.60362E+00_JPRB, 0.59863E+00_JPRB, 0.60017E+00_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.61385E+00_JPRB, 0.61033E+00_JPRB, 0.60294E+00_JPRB, 0.59079E+00_JPRB, 0.59142E+00_JPRB, 0.59014E+00_JPRB, 0.56696E+00_JPRB ,&
&  0.52248E+00_JPRB, 0.49253E+00_JPRB, 0.46443E+00_JPRB, 0.44696E+00_JPRB, 0.45248E+00_JPRB, 0.46766E+00_JPRB, 0.50694E+00_JPRB ,&
&  0.57521E+00_JPRB, 0.68453E+00_JPRB, 0.81099E+00_JPRB, 0.82959E+00_JPRB, 0.82045E+00_JPRB, 0.77077E+00_JPRB, 0.98865E+00_JPRB ,&
&  0.13396E+01_JPRB, 0.16562E+01_JPRB, 0.21810E+01_JPRB, 0.24371E+01_JPRB, 0.24932E+01_JPRB, 0.24949E+01_JPRB, 0.23500E+01_JPRB ,&
&  0.20994E+01_JPRB, 0.17943E+01_JPRB, 0.14074E+01_JPRB, 0.11330E+01_JPRB, 0.96051E+00_JPRB, 0.84945E+00_JPRB, 0.78663E+00_JPRB ,&
&  0.79274E+00_JPRB, 0.80299E+00_JPRB, 0.78358E+00_JPRB, 0.74580E+00_JPRB, 0.72279E+00_JPRB, 0.71784E+00_JPRB, 0.72433E+00_JPRB ,&
&  0.72527E+00_JPRB, 0.72641E+00_JPRB, 0.72356E+00_JPRB, 0.74328E+00_JPRB, 0.65953E+00_JPRB, 0.77136E+00_JPRB, 0.83022E+00_JPRB ,&
&  0.88226E+00_JPRB, 0.90340E+00_JPRB, 0.88247E+00_JPRB, 0.96704E+00_JPRB, 0.11021E+01_JPRB, 0.11386E+01_JPRB, 0.11272E+01_JPRB ,&
&  0.10384E+01_JPRB, 0.86561E+00_JPRB, 0.72317E+00_JPRB, 0.71921E+00_JPRB, 0.77549E+00_JPRB, 0.81026E+00_JPRB, 0.66843E+00_JPRB ,&
&  0.63772E+00_JPRB, 0.76893E+00_JPRB, 0.80151E+00_JPRB, 0.78039E+00_JPRB, 0.94228E+00_JPRB, 0.11159E+01_JPRB, 0.83343E+00_JPRB ,&
&  0.44218E+00_JPRB, 0.39329E+00_JPRB, 0.40642E+00_JPRB, 0.39150E+00_JPRB, 0.39466E+00_JPRB, 0.41358E+00_JPRB, 0.55772E+00_JPRB ,&
&  0.51310E+00_JPRB, 0.55879E+00_JPRB, 0.56465E+00_JPRB, 0.58524E+00_JPRB, 0.66570E+00_JPRB, 0.81293E+00_JPRB, 0.99383E+00_JPRB ,&
&  0.76219E+00_JPRB, 0.74500E+00_JPRB, 0.68618E+00_JPRB, 0.66439E+00_JPRB, 0.65889E+00_JPRB, 0.65964E+00_JPRB, 0.64772E+00_JPRB ,&
&  0.64056E+00_JPRB, 0.63670E+00_JPRB, 0.63068E+00_JPRB, 0.62451E+00_JPRB, 0.61983E+00_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.61499E+00_JPRB, 0.61800E+00_JPRB, 0.62177E+00_JPRB, 0.61926E+00_JPRB, 0.61151E+00_JPRB, 0.59894E+00_JPRB, 0.57964E+00_JPRB ,&
&  0.54345E+00_JPRB, 0.50345E+00_JPRB, 0.47116E+00_JPRB, 0.45245E+00_JPRB, 0.44805E+00_JPRB, 0.44824E+00_JPRB, 0.46788E+00_JPRB ,&
&  0.52105E+00_JPRB, 0.65789E+00_JPRB, 0.90596E+00_JPRB, 0.84439E+00_JPRB, 0.86466E+00_JPRB, 0.80850E+00_JPRB, 0.90719E+00_JPRB ,&
&  0.12051E+01_JPRB, 0.14632E+01_JPRB, 0.18699E+01_JPRB, 0.22637E+01_JPRB, 0.26973E+01_JPRB, 0.27313E+01_JPRB, 0.26599E+01_JPRB ,&
&  0.24117E+01_JPRB, 0.21196E+01_JPRB, 0.16990E+01_JPRB, 0.13326E+01_JPRB, 0.10595E+01_JPRB, 0.90116E+00_JPRB, 0.81066E+00_JPRB ,&
&  0.79189E+00_JPRB, 0.80160E+00_JPRB, 0.78354E+00_JPRB, 0.73173E+00_JPRB, 0.70240E+00_JPRB, 0.69960E+00_JPRB, 0.72129E+00_JPRB ,&
&  0.72235E+00_JPRB, 0.71715E+00_JPRB, 0.69732E+00_JPRB, 0.70820E+00_JPRB, 0.64238E+00_JPRB, 0.61219E+00_JPRB, 0.76037E+00_JPRB ,&
&  0.88902E+00_JPRB, 0.10095E+01_JPRB, 0.10051E+01_JPRB, 0.11413E+01_JPRB, 0.12902E+01_JPRB, 0.13227E+01_JPRB, 0.13564E+01_JPRB ,&
&  0.12127E+01_JPRB, 0.11023E+01_JPRB, 0.82907E+00_JPRB, 0.79495E+00_JPRB, 0.82983E+00_JPRB, 0.71713E+00_JPRB, 0.66684E+00_JPRB ,&
&  0.71136E+00_JPRB, 0.69993E+00_JPRB, 0.70098E+00_JPRB, 0.63694E+00_JPRB, 0.82802E+00_JPRB, 0.90865E+00_JPRB, 0.48117E+00_JPRB ,&
&  0.41825E+00_JPRB, 0.40069E+00_JPRB, 0.38222E+00_JPRB, 0.36886E+00_JPRB, 0.37681E+00_JPRB, 0.38961E+00_JPRB, 0.46352E+00_JPRB ,&
&  0.49381E+00_JPRB, 0.55035E+00_JPRB, 0.55817E+00_JPRB, 0.57622E+00_JPRB, 0.62151E+00_JPRB, 0.74974E+00_JPRB, 0.10116E+01_JPRB ,&
&  0.89631E+00_JPRB, 0.85486E+00_JPRB, 0.74424E+00_JPRB, 0.68230E+00_JPRB, 0.65722E+00_JPRB, 0.64405E+00_JPRB, 0.63121E+00_JPRB ,&
&  0.62223E+00_JPRB, 0.62086E+00_JPRB, 0.61860E+00_JPRB, 0.61441E+00_JPRB, 0.61373E+00_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.60904E+00_JPRB, 0.61485E+00_JPRB, 0.62102E+00_JPRB, 0.62526E+00_JPRB, 0.62445E+00_JPRB, 0.61423E+00_JPRB, 0.59449E+00_JPRB ,&
&  0.56743E+00_JPRB, 0.53566E+00_JPRB, 0.50245E+00_JPRB, 0.47968E+00_JPRB, 0.46272E+00_JPRB, 0.45219E+00_JPRB, 0.45358E+00_JPRB ,&
&  0.48314E+00_JPRB, 0.64012E+00_JPRB, 0.79495E+00_JPRB, 0.72634E+00_JPRB, 0.70594E+00_JPRB, 0.70881E+00_JPRB, 0.74171E+00_JPRB ,&
&  0.10575E+01_JPRB, 0.12847E+01_JPRB, 0.15884E+01_JPRB, 0.21404E+01_JPRB, 0.26519E+01_JPRB, 0.26553E+01_JPRB, 0.28646E+01_JPRB ,&
&  0.26478E+01_JPRB, 0.23219E+01_JPRB, 0.19422E+01_JPRB, 0.15416E+01_JPRB, 0.12249E+01_JPRB, 0.99328E+00_JPRB, 0.87986E+00_JPRB ,&
&  0.82323E+00_JPRB, 0.81494E+00_JPRB, 0.79487E+00_JPRB, 0.74561E+00_JPRB, 0.72158E+00_JPRB, 0.70700E+00_JPRB, 0.70195E+00_JPRB ,&
&  0.69327E+00_JPRB, 0.70324E+00_JPRB, 0.70221E+00_JPRB, 0.72402E+00_JPRB, 0.76935E+00_JPRB, 0.75913E+00_JPRB, 0.78903E+00_JPRB ,&
&  0.85672E+00_JPRB, 0.95214E+00_JPRB, 0.11250E+01_JPRB, 0.12890E+01_JPRB, 0.15039E+01_JPRB, 0.16643E+01_JPRB, 0.15842E+01_JPRB ,&
&  0.14120E+01_JPRB, 0.12114E+01_JPRB, 0.95751E+00_JPRB, 0.93220E+00_JPRB, 0.80762E+00_JPRB, 0.66242E+00_JPRB, 0.69895E+00_JPRB ,&
&  0.71878E+00_JPRB, 0.69340E+00_JPRB, 0.62364E+00_JPRB, 0.53647E+00_JPRB, 0.56842E+00_JPRB, 0.50998E+00_JPRB, 0.41400E+00_JPRB ,&
&  0.39698E+00_JPRB, 0.38173E+00_JPRB, 0.36853E+00_JPRB, 0.37394E+00_JPRB, 0.36947E+00_JPRB, 0.38446E+00_JPRB, 0.43273E+00_JPRB ,&
&  0.49038E+00_JPRB, 0.53906E+00_JPRB, 0.55153E+00_JPRB, 0.57550E+00_JPRB, 0.61243E+00_JPRB, 0.66203E+00_JPRB, 0.87204E+00_JPRB ,&
&  0.86081E+00_JPRB, 0.89857E+00_JPRB, 0.78458E+00_JPRB, 0.71903E+00_JPRB, 0.68115E+00_JPRB, 0.65525E+00_JPRB, 0.63557E+00_JPRB ,&
&  0.62671E+00_JPRB, 0.61998E+00_JPRB, 0.61352E+00_JPRB, 0.61057E+00_JPRB, 0.60651E+00_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.60848E+00_JPRB, 0.61142E+00_JPRB, 0.61646E+00_JPRB, 0.61982E+00_JPRB, 0.62044E+00_JPRB, 0.61508E+00_JPRB, 0.60548E+00_JPRB ,&
&  0.58276E+00_JPRB, 0.55669E+00_JPRB, 0.52235E+00_JPRB, 0.50070E+00_JPRB, 0.47891E+00_JPRB, 0.46396E+00_JPRB, 0.45618E+00_JPRB ,&
&  0.47167E+00_JPRB, 0.63514E+00_JPRB, 0.56949E+00_JPRB, 0.56610E+00_JPRB, 0.63084E+00_JPRB, 0.69394E+00_JPRB, 0.63463E+00_JPRB ,&
&  0.89767E+00_JPRB, 0.11275E+01_JPRB, 0.14391E+01_JPRB, 0.20906E+01_JPRB, 0.30615E+01_JPRB, 0.29071E+01_JPRB, 0.29647E+01_JPRB ,&
&  0.29188E+01_JPRB, 0.24418E+01_JPRB, 0.20992E+01_JPRB, 0.17290E+01_JPRB, 0.13722E+01_JPRB, 0.11269E+01_JPRB, 0.94263E+00_JPRB ,&
&  0.87146E+00_JPRB, 0.84244E+00_JPRB, 0.81467E+00_JPRB, 0.77961E+00_JPRB, 0.75721E+00_JPRB, 0.74003E+00_JPRB, 0.71411E+00_JPRB ,&
&  0.69707E+00_JPRB, 0.68663E+00_JPRB, 0.67765E+00_JPRB, 0.73101E+00_JPRB, 0.72912E+00_JPRB, 0.72530E+00_JPRB, 0.95287E+00_JPRB ,&
&  0.10658E+01_JPRB, 0.11648E+01_JPRB, 0.11763E+01_JPRB, 0.14777E+01_JPRB, 0.18056E+01_JPRB, 0.18484E+01_JPRB, 0.18545E+01_JPRB ,&
&  0.14158E+01_JPRB, 0.11049E+01_JPRB, 0.97696E+00_JPRB, 0.92384E+00_JPRB, 0.74323E+00_JPRB, 0.75957E+00_JPRB, 0.97955E+00_JPRB ,&
&  0.78399E+00_JPRB, 0.79207E+00_JPRB, 0.80962E+00_JPRB, 0.72043E+00_JPRB, 0.49633E+00_JPRB, 0.43215E+00_JPRB, 0.52230E+00_JPRB ,&
&  0.53853E+00_JPRB, 0.42107E+00_JPRB, 0.38102E+00_JPRB, 0.43745E+00_JPRB, 0.38231E+00_JPRB, 0.39899E+00_JPRB, 0.45022E+00_JPRB ,&
&  0.48413E+00_JPRB, 0.50976E+00_JPRB, 0.55093E+00_JPRB, 0.58378E+00_JPRB, 0.61788E+00_JPRB, 0.64365E+00_JPRB, 0.72052E+00_JPRB ,&
&  0.77097E+00_JPRB, 0.83534E+00_JPRB, 0.83903E+00_JPRB, 0.78180E+00_JPRB, 0.73755E+00_JPRB, 0.69215E+00_JPRB, 0.65664E+00_JPRB ,&
&  0.63186E+00_JPRB, 0.61628E+00_JPRB, 0.60755E+00_JPRB, 0.60239E+00_JPRB, 0.60622E+00_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.61979E+00_JPRB, 0.61593E+00_JPRB, 0.62101E+00_JPRB, 0.63083E+00_JPRB, 0.63645E+00_JPRB, 0.63083E+00_JPRB, 0.61947E+00_JPRB ,&
&  0.60878E+00_JPRB, 0.58067E+00_JPRB, 0.54664E+00_JPRB, 0.51875E+00_JPRB, 0.50097E+00_JPRB, 0.48275E+00_JPRB, 0.46694E+00_JPRB ,&
&  0.46434E+00_JPRB, 0.49884E+00_JPRB, 0.47129E+00_JPRB, 0.47417E+00_JPRB, 0.60918E+00_JPRB, 0.61394E+00_JPRB, 0.64827E+00_JPRB ,&
&  0.77620E+00_JPRB, 0.96060E+00_JPRB, 0.13035E+01_JPRB, 0.18401E+01_JPRB, 0.23350E+01_JPRB, 0.27838E+01_JPRB, 0.29870E+01_JPRB ,&
&  0.29898E+01_JPRB, 0.25693E+01_JPRB, 0.21516E+01_JPRB, 0.18103E+01_JPRB, 0.14744E+01_JPRB, 0.12198E+01_JPRB, 0.10406E+01_JPRB ,&
&  0.94609E+00_JPRB, 0.90259E+00_JPRB, 0.87114E+00_JPRB, 0.83520E+00_JPRB, 0.81907E+00_JPRB, 0.78222E+00_JPRB, 0.74777E+00_JPRB ,&
&  0.73203E+00_JPRB, 0.72654E+00_JPRB, 0.70372E+00_JPRB, 0.72020E+00_JPRB, 0.70645E+00_JPRB, 0.73225E+00_JPRB, 0.83229E+00_JPRB ,&
&  0.97555E+00_JPRB, 0.10885E+01_JPRB, 0.11779E+01_JPRB, 0.14947E+01_JPRB, 0.19043E+01_JPRB, 0.18386E+01_JPRB, 0.19588E+01_JPRB ,&
&  0.16806E+01_JPRB, 0.13662E+01_JPRB, 0.10842E+01_JPRB, 0.83795E+00_JPRB, 0.83771E+00_JPRB, 0.11183E+01_JPRB, 0.11524E+01_JPRB ,&
&  0.10333E+01_JPRB, 0.98564E+00_JPRB, 0.93207E+00_JPRB, 0.78363E+00_JPRB, 0.58802E+00_JPRB, 0.48291E+00_JPRB, 0.55622E+00_JPRB ,&
&  0.59502E+00_JPRB, 0.60247E+00_JPRB, 0.57448E+00_JPRB, 0.52595E+00_JPRB, 0.46506E+00_JPRB, 0.50707E+00_JPRB, 0.50847E+00_JPRB ,&
&  0.50004E+00_JPRB, 0.48997E+00_JPRB, 0.51881E+00_JPRB, 0.57706E+00_JPRB, 0.59093E+00_JPRB, 0.59587E+00_JPRB, 0.63043E+00_JPRB ,&
&  0.66490E+00_JPRB, 0.72370E+00_JPRB, 0.74558E+00_JPRB, 0.74264E+00_JPRB, 0.73540E+00_JPRB, 0.71327E+00_JPRB, 0.69227E+00_JPRB ,&
&  0.67736E+00_JPRB, 0.66740E+00_JPRB, 0.65303E+00_JPRB, 0.63938E+00_JPRB, 0.62746E+00_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.61719E+00_JPRB, 0.61110E+00_JPRB, 0.61215E+00_JPRB, 0.62295E+00_JPRB, 0.64216E+00_JPRB, 0.64543E+00_JPRB, 0.63718E+00_JPRB ,&
&  0.62267E+00_JPRB, 0.60566E+00_JPRB, 0.57940E+00_JPRB, 0.54665E+00_JPRB, 0.52398E+00_JPRB, 0.49817E+00_JPRB, 0.47778E+00_JPRB ,&
&  0.46711E+00_JPRB, 0.47500E+00_JPRB, 0.43080E+00_JPRB, 0.47251E+00_JPRB, 0.52908E+00_JPRB, 0.52481E+00_JPRB, 0.59204E+00_JPRB ,&
&  0.64690E+00_JPRB, 0.79014E+00_JPRB, 0.10242E+01_JPRB, 0.13401E+01_JPRB, 0.16910E+01_JPRB, 0.21674E+01_JPRB, 0.24983E+01_JPRB ,&
&  0.24725E+01_JPRB, 0.24039E+01_JPRB, 0.21843E+01_JPRB, 0.19540E+01_JPRB, 0.16433E+01_JPRB, 0.13927E+01_JPRB, 0.12187E+01_JPRB ,&
&  0.10876E+01_JPRB, 0.10092E+01_JPRB, 0.96605E+00_JPRB, 0.92921E+00_JPRB, 0.89842E+00_JPRB, 0.86538E+00_JPRB, 0.83767E+00_JPRB ,&
&  0.79445E+00_JPRB, 0.78548E+00_JPRB, 0.76246E+00_JPRB, 0.76730E+00_JPRB, 0.72808E+00_JPRB, 0.75859E+00_JPRB, 0.83120E+00_JPRB ,&
&  0.10576E+01_JPRB, 0.12186E+01_JPRB, 0.12747E+01_JPRB, 0.17020E+01_JPRB, 0.16111E+01_JPRB, 0.16040E+01_JPRB, 0.18357E+01_JPRB ,&
&  0.20649E+01_JPRB, 0.20587E+01_JPRB, 0.19141E+01_JPRB, 0.14533E+01_JPRB, 0.11346E+01_JPRB, 0.14622E+01_JPRB, 0.14032E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.11284E+01_JPRB, 0.10083E+01_JPRB, 0.92813E+00_JPRB, 0.72456E+00_JPRB, 0.62253E+00_JPRB, 0.55189E+00_JPRB ,&
&  0.57518E+00_JPRB, 0.55247E+00_JPRB, 0.59815E+00_JPRB, 0.56186E+00_JPRB, 0.53667E+00_JPRB, 0.52878E+00_JPRB, 0.50884E+00_JPRB ,&
&  0.49862E+00_JPRB, 0.48980E+00_JPRB, 0.48803E+00_JPRB, 0.52560E+00_JPRB, 0.55144E+00_JPRB, 0.54164E+00_JPRB, 0.56510E+00_JPRB ,&
&  0.59428E+00_JPRB, 0.63830E+00_JPRB, 0.67839E+00_JPRB, 0.70921E+00_JPRB, 0.72073E+00_JPRB, 0.71910E+00_JPRB, 0.70610E+00_JPRB ,&
&  0.68927E+00_JPRB, 0.67882E+00_JPRB, 0.66377E+00_JPRB, 0.64560E+00_JPRB, 0.62885E+00_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.59123E+00_JPRB, 0.58922E+00_JPRB, 0.59221E+00_JPRB, 0.60899E+00_JPRB, 0.63667E+00_JPRB, 0.65168E+00_JPRB, 0.64293E+00_JPRB ,&
&  0.63180E+00_JPRB, 0.61745E+00_JPRB, 0.60855E+00_JPRB, 0.57945E+00_JPRB, 0.54989E+00_JPRB, 0.51517E+00_JPRB, 0.48852E+00_JPRB ,&
&  0.48368E+00_JPRB, 0.54861E+00_JPRB, 0.47493E+00_JPRB, 0.43293E+00_JPRB, 0.40866E+00_JPRB, 0.45982E+00_JPRB, 0.51558E+00_JPRB ,&
&  0.58741E+00_JPRB, 0.65109E+00_JPRB, 0.82982E+00_JPRB, 0.95304E+00_JPRB, 0.11020E+01_JPRB, 0.14368E+01_JPRB, 0.18486E+01_JPRB ,&
&  0.22766E+01_JPRB, 0.19304E+01_JPRB, 0.19584E+01_JPRB, 0.18231E+01_JPRB, 0.16930E+01_JPRB, 0.15229E+01_JPRB, 0.13546E+01_JPRB ,&
&  0.12097E+01_JPRB, 0.10843E+01_JPRB, 0.10129E+01_JPRB, 0.95641E+00_JPRB, 0.90692E+00_JPRB, 0.86141E+00_JPRB, 0.82330E+00_JPRB ,&
&  0.80136E+00_JPRB, 0.81504E+00_JPRB, 0.83922E+00_JPRB, 0.84019E+00_JPRB, 0.87562E+00_JPRB, 0.94411E+00_JPRB, 0.99582E+00_JPRB ,&
&  0.10314E+01_JPRB, 0.10570E+01_JPRB, 0.14427E+01_JPRB, 0.16750E+01_JPRB, 0.18547E+01_JPRB, 0.19372E+01_JPRB, 0.20509E+01_JPRB ,&
&  0.21793E+01_JPRB, 0.21930E+01_JPRB, 0.21229E+01_JPRB, 0.17695E+01_JPRB, 0.17860E+01_JPRB, 0.17430E+01_JPRB, 0.16198E+01_JPRB ,&
&  0.14674E+01_JPRB, 0.13598E+01_JPRB, 0.11913E+01_JPRB, 0.10585E+01_JPRB, 0.96625E+00_JPRB, 0.90333E+00_JPRB, 0.77477E+00_JPRB ,&
&  0.68554E+00_JPRB, 0.69853E+00_JPRB, 0.64133E+00_JPRB, 0.56566E+00_JPRB, 0.49953E+00_JPRB, 0.49843E+00_JPRB, 0.49520E+00_JPRB ,&
&  0.48994E+00_JPRB, 0.48261E+00_JPRB, 0.46911E+00_JPRB, 0.48076E+00_JPRB, 0.50568E+00_JPRB, 0.50745E+00_JPRB, 0.50505E+00_JPRB ,&
&  0.51362E+00_JPRB, 0.55717E+00_JPRB, 0.58383E+00_JPRB, 0.59971E+00_JPRB, 0.64360E+00_JPRB, 0.65197E+00_JPRB, 0.64085E+00_JPRB ,&
&  0.62204E+00_JPRB, 0.60937E+00_JPRB, 0.60612E+00_JPRB, 0.60079E+00_JPRB, 0.59597E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.55338E+00_JPRB, 0.55880E+00_JPRB, 0.56894E+00_JPRB, 0.59095E+00_JPRB, 0.62529E+00_JPRB, 0.64959E+00_JPRB, 0.64574E+00_JPRB ,&
&  0.64016E+00_JPRB, 0.62794E+00_JPRB, 0.61602E+00_JPRB, 0.59865E+00_JPRB, 0.56660E+00_JPRB, 0.53114E+00_JPRB, 0.50736E+00_JPRB ,&
&  0.51060E+00_JPRB, 0.67475E+00_JPRB, 0.55388E+00_JPRB, 0.51168E+00_JPRB, 0.45461E+00_JPRB, 0.48054E+00_JPRB, 0.51107E+00_JPRB ,&
&  0.58265E+00_JPRB, 0.59905E+00_JPRB, 0.65315E+00_JPRB, 0.72470E+00_JPRB, 0.82257E+00_JPRB, 0.10294E+01_JPRB, 0.13618E+01_JPRB ,&
&  0.15572E+01_JPRB, 0.16948E+01_JPRB, 0.16135E+01_JPRB, 0.17240E+01_JPRB, 0.15927E+01_JPRB, 0.14039E+01_JPRB, 0.13961E+01_JPRB ,&
&  0.12765E+01_JPRB, 0.11619E+01_JPRB, 0.10456E+01_JPRB, 0.96989E+00_JPRB, 0.91535E+00_JPRB, 0.86405E+00_JPRB, 0.82170E+00_JPRB ,&
&  0.80064E+00_JPRB, 0.81140E+00_JPRB, 0.85269E+00_JPRB, 0.88151E+00_JPRB, 0.89373E+00_JPRB, 0.98894E+00_JPRB, 0.10947E+01_JPRB ,&
&  0.13988E+01_JPRB, 0.14104E+01_JPRB, 0.13182E+01_JPRB, 0.17721E+01_JPRB, 0.22556E+01_JPRB, 0.22613E+01_JPRB, 0.21242E+01_JPRB ,&
&  0.21813E+01_JPRB, 0.21562E+01_JPRB, 0.20907E+01_JPRB, 0.19994E+01_JPRB, 0.18934E+01_JPRB, 0.17922E+01_JPRB, 0.16849E+01_JPRB ,&
&  0.15907E+01_JPRB, 0.14915E+01_JPRB, 0.13591E+01_JPRB, 0.11613E+01_JPRB, 0.10248E+01_JPRB, 0.94216E+00_JPRB, 0.89632E+00_JPRB ,&
&  0.80701E+00_JPRB, 0.69966E+00_JPRB, 0.55411E+00_JPRB, 0.54792E+00_JPRB, 0.46800E+00_JPRB, 0.45637E+00_JPRB, 0.47264E+00_JPRB ,&
&  0.46168E+00_JPRB, 0.46633E+00_JPRB, 0.45888E+00_JPRB, 0.44750E+00_JPRB, 0.46204E+00_JPRB, 0.47853E+00_JPRB, 0.47363E+00_JPRB ,&
&  0.47335E+00_JPRB, 0.47708E+00_JPRB, 0.51183E+00_JPRB, 0.51849E+00_JPRB, 0.52746E+00_JPRB, 0.53760E+00_JPRB, 0.54121E+00_JPRB ,&
&  0.53311E+00_JPRB, 0.52291E+00_JPRB, 0.52197E+00_JPRB, 0.53157E+00_JPRB, 0.54273E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.49967E+00_JPRB, 0.51372E+00_JPRB, 0.53218E+00_JPRB, 0.55772E+00_JPRB, 0.59513E+00_JPRB, 0.62565E+00_JPRB, 0.63073E+00_JPRB ,&
&  0.62800E+00_JPRB, 0.62232E+00_JPRB, 0.61045E+00_JPRB, 0.59795E+00_JPRB, 0.57456E+00_JPRB, 0.53861E+00_JPRB, 0.51741E+00_JPRB ,&
&  0.56992E+00_JPRB, 0.59940E+00_JPRB, 0.51855E+00_JPRB, 0.44650E+00_JPRB, 0.50036E+00_JPRB, 0.48789E+00_JPRB, 0.50242E+00_JPRB ,&
&  0.49822E+00_JPRB, 0.54467E+00_JPRB, 0.57439E+00_JPRB, 0.58964E+00_JPRB, 0.63230E+00_JPRB, 0.78319E+00_JPRB, 0.97698E+00_JPRB ,&
&  0.11331E+01_JPRB, 0.12284E+01_JPRB, 0.12872E+01_JPRB, 0.13187E+01_JPRB, 0.13776E+01_JPRB, 0.12381E+01_JPRB, 0.12250E+01_JPRB ,&
&  0.11543E+01_JPRB, 0.10760E+01_JPRB, 0.99399E+00_JPRB, 0.92942E+00_JPRB, 0.87340E+00_JPRB, 0.82499E+00_JPRB, 0.79627E+00_JPRB ,&
&  0.80619E+00_JPRB, 0.82021E+00_JPRB, 0.85030E+00_JPRB, 0.89300E+00_JPRB, 0.92254E+00_JPRB, 0.11179E+01_JPRB, 0.15255E+01_JPRB ,&
&  0.18846E+01_JPRB, 0.20923E+01_JPRB, 0.23801E+01_JPRB, 0.26495E+01_JPRB, 0.26334E+01_JPRB, 0.25821E+01_JPRB, 0.25329E+01_JPRB ,&
&  0.23985E+01_JPRB, 0.22468E+01_JPRB, 0.20928E+01_JPRB, 0.19709E+01_JPRB, 0.18897E+01_JPRB, 0.18342E+01_JPRB, 0.17500E+01_JPRB ,&
&  0.15644E+01_JPRB, 0.14351E+01_JPRB, 0.13757E+01_JPRB, 0.12067E+01_JPRB, 0.10810E+01_JPRB, 0.95490E+00_JPRB, 0.93360E+00_JPRB ,&
&  0.81316E+00_JPRB, 0.56204E+00_JPRB, 0.49458E+00_JPRB, 0.53284E+00_JPRB, 0.46922E+00_JPRB, 0.45772E+00_JPRB, 0.46819E+00_JPRB ,&
&  0.45677E+00_JPRB, 0.44621E+00_JPRB, 0.44483E+00_JPRB, 0.43273E+00_JPRB, 0.43911E+00_JPRB, 0.46809E+00_JPRB, 0.44676E+00_JPRB ,&
&  0.42451E+00_JPRB, 0.43363E+00_JPRB, 0.45399E+00_JPRB, 0.45071E+00_JPRB, 0.44851E+00_JPRB, 0.45562E+00_JPRB, 0.45630E+00_JPRB ,&
&  0.45088E+00_JPRB, 0.45128E+00_JPRB, 0.45268E+00_JPRB, 0.46319E+00_JPRB, 0.48298E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.47101E+00_JPRB, 0.49269E+00_JPRB, 0.50943E+00_JPRB, 0.53209E+00_JPRB, 0.56482E+00_JPRB, 0.60161E+00_JPRB, 0.61511E+00_JPRB ,&
&  0.61555E+00_JPRB, 0.61201E+00_JPRB, 0.60355E+00_JPRB, 0.58924E+00_JPRB, 0.57398E+00_JPRB, 0.54526E+00_JPRB, 0.53156E+00_JPRB ,&
&  0.45255E+00_JPRB, 0.47753E+00_JPRB, 0.41742E+00_JPRB, 0.42700E+00_JPRB, 0.50806E+00_JPRB, 0.48050E+00_JPRB, 0.47043E+00_JPRB ,&
&  0.46253E+00_JPRB, 0.48430E+00_JPRB, 0.51463E+00_JPRB, 0.54057E+00_JPRB, 0.54010E+00_JPRB, 0.62174E+00_JPRB, 0.77318E+00_JPRB ,&
&  0.83607E+00_JPRB, 0.84642E+00_JPRB, 0.88807E+00_JPRB, 0.90063E+00_JPRB, 0.93955E+00_JPRB, 0.10285E+01_JPRB, 0.10623E+01_JPRB ,&
&  0.10096E+01_JPRB, 0.94902E+00_JPRB, 0.89090E+00_JPRB, 0.84730E+00_JPRB, 0.80775E+00_JPRB, 0.76435E+00_JPRB, 0.73766E+00_JPRB ,&
&  0.72793E+00_JPRB, 0.72690E+00_JPRB, 0.77282E+00_JPRB, 0.85412E+00_JPRB, 0.91952E+00_JPRB, 0.12646E+01_JPRB, 0.19915E+01_JPRB ,&
&  0.22113E+01_JPRB, 0.23210E+01_JPRB, 0.25820E+01_JPRB, 0.29006E+01_JPRB, 0.28606E+01_JPRB, 0.27737E+01_JPRB, 0.26258E+01_JPRB ,&
&  0.25145E+01_JPRB, 0.23892E+01_JPRB, 0.21863E+01_JPRB, 0.20697E+01_JPRB, 0.18856E+01_JPRB, 0.17885E+01_JPRB, 0.16316E+01_JPRB ,&
&  0.13858E+01_JPRB, 0.12625E+01_JPRB, 0.12501E+01_JPRB, 0.11769E+01_JPRB, 0.11150E+01_JPRB, 0.10634E+01_JPRB, 0.10097E+01_JPRB ,&
&  0.89378E+00_JPRB, 0.74210E+00_JPRB, 0.58893E+00_JPRB, 0.54724E+00_JPRB, 0.46593E+00_JPRB, 0.45934E+00_JPRB, 0.47449E+00_JPRB ,&
&  0.46147E+00_JPRB, 0.43431E+00_JPRB, 0.42173E+00_JPRB, 0.41557E+00_JPRB, 0.41726E+00_JPRB, 0.43735E+00_JPRB, 0.41462E+00_JPRB ,&
&  0.39045E+00_JPRB, 0.39485E+00_JPRB, 0.39949E+00_JPRB, 0.40718E+00_JPRB, 0.41683E+00_JPRB, 0.41905E+00_JPRB, 0.40894E+00_JPRB ,&
&  0.41936E+00_JPRB, 0.41649E+00_JPRB, 0.41467E+00_JPRB, 0.42399E+00_JPRB, 0.44464E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.45310E+00_JPRB, 0.47743E+00_JPRB, 0.49632E+00_JPRB, 0.51214E+00_JPRB, 0.53560E+00_JPRB, 0.57003E+00_JPRB, 0.58642E+00_JPRB ,&
&  0.58934E+00_JPRB, 0.58915E+00_JPRB, 0.58590E+00_JPRB, 0.57604E+00_JPRB, 0.55629E+00_JPRB, 0.52950E+00_JPRB, 0.46976E+00_JPRB ,&
&  0.39972E+00_JPRB, 0.42946E+00_JPRB, 0.41582E+00_JPRB, 0.42749E+00_JPRB, 0.43281E+00_JPRB, 0.42629E+00_JPRB, 0.41841E+00_JPRB ,&
&  0.43160E+00_JPRB, 0.44821E+00_JPRB, 0.48213E+00_JPRB, 0.52543E+00_JPRB, 0.53441E+00_JPRB, 0.56749E+00_JPRB, 0.67117E+00_JPRB ,&
&  0.70240E+00_JPRB, 0.71761E+00_JPRB, 0.69778E+00_JPRB, 0.69046E+00_JPRB, 0.72743E+00_JPRB, 0.78906E+00_JPRB, 0.83707E+00_JPRB ,&
&  0.83419E+00_JPRB, 0.78866E+00_JPRB, 0.74592E+00_JPRB, 0.72038E+00_JPRB, 0.70183E+00_JPRB, 0.68049E+00_JPRB, 0.65909E+00_JPRB ,&
&  0.63674E+00_JPRB, 0.62878E+00_JPRB, 0.64897E+00_JPRB, 0.69256E+00_JPRB, 0.77176E+00_JPRB, 0.10688E+01_JPRB, 0.13541E+01_JPRB ,&
&  0.14417E+01_JPRB, 0.15821E+01_JPRB, 0.17824E+01_JPRB, 0.19752E+01_JPRB, 0.20937E+01_JPRB, 0.21515E+01_JPRB, 0.21647E+01_JPRB ,&
&  0.21890E+01_JPRB, 0.20937E+01_JPRB, 0.20139E+01_JPRB, 0.19070E+01_JPRB, 0.17261E+01_JPRB, 0.15735E+01_JPRB, 0.14020E+01_JPRB ,&
&  0.12350E+01_JPRB, 0.10616E+01_JPRB, 0.10932E+01_JPRB, 0.10917E+01_JPRB, 0.10771E+01_JPRB, 0.10451E+01_JPRB, 0.99726E+00_JPRB ,&
&  0.91193E+00_JPRB, 0.81431E+00_JPRB, 0.69006E+00_JPRB, 0.60350E+00_JPRB, 0.53477E+00_JPRB, 0.52191E+00_JPRB, 0.48369E+00_JPRB ,&
&  0.44081E+00_JPRB, 0.40544E+00_JPRB, 0.40480E+00_JPRB, 0.40331E+00_JPRB, 0.40301E+00_JPRB, 0.39186E+00_JPRB, 0.38426E+00_JPRB ,&
&  0.38484E+00_JPRB, 0.38698E+00_JPRB, 0.38017E+00_JPRB, 0.38108E+00_JPRB, 0.39327E+00_JPRB, 0.39056E+00_JPRB, 0.38011E+00_JPRB ,&
&  0.38173E+00_JPRB, 0.38549E+00_JPRB, 0.38547E+00_JPRB, 0.39895E+00_JPRB, 0.42306E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.43373E+00_JPRB, 0.45659E+00_JPRB, 0.47589E+00_JPRB, 0.49273E+00_JPRB, 0.51297E+00_JPRB, 0.53439E+00_JPRB, 0.53479E+00_JPRB ,&
&  0.55101E+00_JPRB, 0.55719E+00_JPRB, 0.55495E+00_JPRB, 0.54444E+00_JPRB, 0.52256E+00_JPRB, 0.45689E+00_JPRB, 0.37434E+00_JPRB ,&
&  0.35464E+00_JPRB, 0.36921E+00_JPRB, 0.38965E+00_JPRB, 0.39857E+00_JPRB, 0.40207E+00_JPRB, 0.40665E+00_JPRB, 0.40623E+00_JPRB ,&
&  0.41150E+00_JPRB, 0.42347E+00_JPRB, 0.44980E+00_JPRB, 0.49097E+00_JPRB, 0.52878E+00_JPRB, 0.54583E+00_JPRB, 0.58988E+00_JPRB ,&
&  0.59786E+00_JPRB, 0.62241E+00_JPRB, 0.62648E+00_JPRB, 0.57582E+00_JPRB, 0.59882E+00_JPRB, 0.60476E+00_JPRB, 0.62969E+00_JPRB ,&
&  0.65125E+00_JPRB, 0.64021E+00_JPRB, 0.61462E+00_JPRB, 0.59977E+00_JPRB, 0.59285E+00_JPRB, 0.58302E+00_JPRB, 0.57370E+00_JPRB ,&
&  0.55942E+00_JPRB, 0.55200E+00_JPRB, 0.56698E+00_JPRB, 0.59297E+00_JPRB, 0.63476E+00_JPRB, 0.83986E+00_JPRB, 0.95746E+00_JPRB ,&
&  0.10524E+01_JPRB, 0.12255E+01_JPRB, 0.14918E+01_JPRB, 0.15683E+01_JPRB, 0.17921E+01_JPRB, 0.18935E+01_JPRB, 0.19980E+01_JPRB ,&
&  0.20285E+01_JPRB, 0.18724E+01_JPRB, 0.18177E+01_JPRB, 0.16888E+01_JPRB, 0.15534E+01_JPRB, 0.14018E+01_JPRB, 0.12459E+01_JPRB ,&
&  0.11264E+01_JPRB, 0.10075E+01_JPRB, 0.99942E+00_JPRB, 0.97922E+00_JPRB, 0.94004E+00_JPRB, 0.91239E+00_JPRB, 0.88250E+00_JPRB ,&
&  0.84250E+00_JPRB, 0.78026E+00_JPRB, 0.70036E+00_JPRB, 0.61984E+00_JPRB, 0.55588E+00_JPRB, 0.50667E+00_JPRB, 0.46894E+00_JPRB ,&
&  0.43941E+00_JPRB, 0.40877E+00_JPRB, 0.39952E+00_JPRB, 0.39798E+00_JPRB, 0.39374E+00_JPRB, 0.38041E+00_JPRB, 0.37532E+00_JPRB ,&
&  0.37267E+00_JPRB, 0.37129E+00_JPRB, 0.36831E+00_JPRB, 0.36846E+00_JPRB, 0.37527E+00_JPRB, 0.37897E+00_JPRB, 0.37115E+00_JPRB ,&
&  0.36566E+00_JPRB, 0.37033E+00_JPRB, 0.37669E+00_JPRB, 0.38996E+00_JPRB, 0.41024E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.42031E+00_JPRB, 0.44693E+00_JPRB, 0.46124E+00_JPRB, 0.47425E+00_JPRB, 0.48768E+00_JPRB, 0.48499E+00_JPRB, 0.48482E+00_JPRB ,&
&  0.51369E+00_JPRB, 0.48125E+00_JPRB, 0.51908E+00_JPRB, 0.41269E+00_JPRB, 0.37203E+00_JPRB, 0.36408E+00_JPRB, 0.35438E+00_JPRB ,&
&  0.35596E+00_JPRB, 0.36599E+00_JPRB, 0.37602E+00_JPRB, 0.38295E+00_JPRB, 0.39377E+00_JPRB, 0.39575E+00_JPRB, 0.39641E+00_JPRB ,&
&  0.40015E+00_JPRB, 0.40906E+00_JPRB, 0.43139E+00_JPRB, 0.44538E+00_JPRB, 0.46050E+00_JPRB, 0.47485E+00_JPRB, 0.48948E+00_JPRB ,&
&  0.49214E+00_JPRB, 0.51592E+00_JPRB, 0.55380E+00_JPRB, 0.54296E+00_JPRB, 0.52641E+00_JPRB, 0.51116E+00_JPRB, 0.51815E+00_JPRB ,&
&  0.51694E+00_JPRB, 0.46810E+00_JPRB, 0.48076E+00_JPRB, 0.48121E+00_JPRB, 0.48290E+00_JPRB, 0.48555E+00_JPRB, 0.48853E+00_JPRB ,&
&  0.49094E+00_JPRB, 0.49428E+00_JPRB, 0.51026E+00_JPRB, 0.52950E+00_JPRB, 0.56721E+00_JPRB, 0.64706E+00_JPRB, 0.70610E+00_JPRB ,&
&  0.84598E+00_JPRB, 0.84770E+00_JPRB, 0.11424E+01_JPRB, 0.14018E+01_JPRB, 0.17442E+01_JPRB, 0.18235E+01_JPRB, 0.19143E+01_JPRB ,&
&  0.19102E+01_JPRB, 0.17428E+01_JPRB, 0.16554E+01_JPRB, 0.15039E+01_JPRB, 0.13626E+01_JPRB, 0.12346E+01_JPRB, 0.11308E+01_JPRB ,&
&  0.10360E+01_JPRB, 0.91317E+00_JPRB, 0.89500E+00_JPRB, 0.84336E+00_JPRB, 0.79006E+00_JPRB, 0.75572E+00_JPRB, 0.73328E+00_JPRB ,&
&  0.70296E+00_JPRB, 0.66228E+00_JPRB, 0.60504E+00_JPRB, 0.53062E+00_JPRB, 0.49743E+00_JPRB, 0.46886E+00_JPRB, 0.43905E+00_JPRB ,&
&  0.42526E+00_JPRB, 0.41056E+00_JPRB, 0.40217E+00_JPRB, 0.39822E+00_JPRB, 0.39022E+00_JPRB, 0.37774E+00_JPRB, 0.37092E+00_JPRB ,&
&  0.35954E+00_JPRB, 0.35391E+00_JPRB, 0.35161E+00_JPRB, 0.35247E+00_JPRB, 0.35383E+00_JPRB, 0.35735E+00_JPRB, 0.35818E+00_JPRB ,&
&  0.35873E+00_JPRB, 0.36241E+00_JPRB, 0.36736E+00_JPRB, 0.37565E+00_JPRB, 0.39418E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.39463E+00_JPRB, 0.42856E+00_JPRB, 0.44470E+00_JPRB, 0.45386E+00_JPRB, 0.46165E+00_JPRB, 0.45964E+00_JPRB, 0.43495E+00_JPRB ,&
&  0.43583E+00_JPRB, 0.39317E+00_JPRB, 0.37372E+00_JPRB, 0.35942E+00_JPRB, 0.35355E+00_JPRB, 0.34778E+00_JPRB, 0.33953E+00_JPRB ,&
&  0.34144E+00_JPRB, 0.35091E+00_JPRB, 0.36399E+00_JPRB, 0.37261E+00_JPRB, 0.37999E+00_JPRB, 0.38527E+00_JPRB, 0.38939E+00_JPRB ,&
&  0.39513E+00_JPRB, 0.40245E+00_JPRB, 0.41288E+00_JPRB, 0.42126E+00_JPRB, 0.42614E+00_JPRB, 0.42899E+00_JPRB, 0.42911E+00_JPRB ,&
&  0.43865E+00_JPRB, 0.45901E+00_JPRB, 0.48844E+00_JPRB, 0.50375E+00_JPRB, 0.50199E+00_JPRB, 0.48459E+00_JPRB, 0.45455E+00_JPRB ,&
&  0.38358E+00_JPRB, 0.37826E+00_JPRB, 0.40165E+00_JPRB, 0.41124E+00_JPRB, 0.42113E+00_JPRB, 0.43057E+00_JPRB, 0.43842E+00_JPRB ,&
&  0.44507E+00_JPRB, 0.45473E+00_JPRB, 0.47447E+00_JPRB, 0.49665E+00_JPRB, 0.53260E+00_JPRB, 0.58989E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.70462E+00_JPRB, 0.71191E+00_JPRB, 0.83874E+00_JPRB, 0.11121E+01_JPRB, 0.16668E+01_JPRB, 0.16583E+01_JPRB, 0.16694E+01_JPRB ,&
&  0.16478E+01_JPRB, 0.15798E+01_JPRB, 0.15224E+01_JPRB, 0.14004E+01_JPRB, 0.12507E+01_JPRB, 0.11155E+01_JPRB, 0.10090E+01_JPRB ,&
&  0.90646E+00_JPRB, 0.83798E+00_JPRB, 0.77091E+00_JPRB, 0.73751E+00_JPRB, 0.69951E+00_JPRB, 0.66890E+00_JPRB, 0.63807E+00_JPRB ,&
&  0.60124E+00_JPRB, 0.56370E+00_JPRB, 0.51571E+00_JPRB, 0.47142E+00_JPRB, 0.44196E+00_JPRB, 0.42808E+00_JPRB, 0.41472E+00_JPRB ,&
&  0.40789E+00_JPRB, 0.40436E+00_JPRB, 0.39793E+00_JPRB, 0.39450E+00_JPRB, 0.38289E+00_JPRB, 0.36535E+00_JPRB, 0.35510E+00_JPRB ,&
&  0.34543E+00_JPRB, 0.33629E+00_JPRB, 0.33207E+00_JPRB, 0.33010E+00_JPRB, 0.33181E+00_JPRB, 0.33371E+00_JPRB, 0.33813E+00_JPRB ,&
&  0.34283E+00_JPRB, 0.34317E+00_JPRB, 0.34516E+00_JPRB, 0.34921E+00_JPRB, 0.36663E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.36682E+00_JPRB, 0.39063E+00_JPRB, 0.41089E+00_JPRB, 0.42598E+00_JPRB, 0.42905E+00_JPRB, 0.42834E+00_JPRB, 0.41599E+00_JPRB ,&
&  0.41720E+00_JPRB, 0.39693E+00_JPRB, 0.38271E+00_JPRB, 0.36712E+00_JPRB, 0.35516E+00_JPRB, 0.34712E+00_JPRB, 0.34760E+00_JPRB ,&
&  0.35234E+00_JPRB, 0.35847E+00_JPRB, 0.36427E+00_JPRB, 0.36785E+00_JPRB, 0.37313E+00_JPRB, 0.37958E+00_JPRB, 0.38745E+00_JPRB ,&
&  0.39522E+00_JPRB, 0.39991E+00_JPRB, 0.40403E+00_JPRB, 0.40793E+00_JPRB, 0.41260E+00_JPRB, 0.41563E+00_JPRB, 0.41166E+00_JPRB ,&
&  0.41586E+00_JPRB, 0.42488E+00_JPRB, 0.44895E+00_JPRB, 0.47175E+00_JPRB, 0.48383E+00_JPRB, 0.46601E+00_JPRB, 0.39681E+00_JPRB ,&
&  0.35270E+00_JPRB, 0.33663E+00_JPRB, 0.34061E+00_JPRB, 0.36703E+00_JPRB, 0.37920E+00_JPRB, 0.39046E+00_JPRB, 0.40137E+00_JPRB ,&
&  0.40080E+00_JPRB, 0.40448E+00_JPRB, 0.42505E+00_JPRB, 0.45558E+00_JPRB, 0.48557E+00_JPRB, 0.53900E+00_JPRB, 0.58681E+00_JPRB ,&
&  0.65791E+00_JPRB, 0.76921E+00_JPRB, 0.97150E+00_JPRB, 0.10189E+01_JPRB, 0.11287E+01_JPRB, 0.13951E+01_JPRB, 0.13530E+01_JPRB ,&
&  0.13317E+01_JPRB, 0.13297E+01_JPRB, 0.13335E+01_JPRB, 0.12658E+01_JPRB, 0.11361E+01_JPRB, 0.10198E+01_JPRB, 0.90805E+00_JPRB ,&
&  0.79995E+00_JPRB, 0.72934E+00_JPRB, 0.67139E+00_JPRB, 0.64845E+00_JPRB, 0.62017E+00_JPRB, 0.59055E+00_JPRB, 0.56821E+00_JPRB ,&
&  0.53290E+00_JPRB, 0.49110E+00_JPRB, 0.44983E+00_JPRB, 0.42034E+00_JPRB, 0.40613E+00_JPRB, 0.39963E+00_JPRB, 0.39584E+00_JPRB ,&
&  0.39246E+00_JPRB, 0.38878E+00_JPRB, 0.38424E+00_JPRB, 0.37364E+00_JPRB, 0.35800E+00_JPRB, 0.34065E+00_JPRB, 0.33478E+00_JPRB ,&
&  0.33113E+00_JPRB, 0.32726E+00_JPRB, 0.32625E+00_JPRB, 0.32274E+00_JPRB, 0.32290E+00_JPRB, 0.32267E+00_JPRB, 0.32070E+00_JPRB ,&
&  0.32376E+00_JPRB, 0.32468E+00_JPRB, 0.32283E+00_JPRB, 0.32725E+00_JPRB, 0.34458E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.34857E+00_JPRB, 0.35710E+00_JPRB, 0.36873E+00_JPRB, 0.38381E+00_JPRB, 0.39181E+00_JPRB, 0.38584E+00_JPRB, 0.37150E+00_JPRB ,&
&  0.36324E+00_JPRB, 0.35956E+00_JPRB, 0.36171E+00_JPRB, 0.36543E+00_JPRB, 0.36746E+00_JPRB, 0.36801E+00_JPRB, 0.36774E+00_JPRB ,&
&  0.36592E+00_JPRB, 0.36595E+00_JPRB, 0.36833E+00_JPRB, 0.37280E+00_JPRB, 0.37871E+00_JPRB, 0.38459E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39611E+00_JPRB, 0.39763E+00_JPRB, 0.39765E+00_JPRB, 0.39580E+00_JPRB, 0.39694E+00_JPRB, 0.40012E+00_JPRB, 0.40044E+00_JPRB ,&
&  0.39760E+00_JPRB, 0.40021E+00_JPRB, 0.41249E+00_JPRB, 0.43056E+00_JPRB, 0.45165E+00_JPRB, 0.44013E+00_JPRB, 0.39188E+00_JPRB ,&
&  0.34902E+00_JPRB, 0.31899E+00_JPRB, 0.30911E+00_JPRB, 0.31230E+00_JPRB, 0.32599E+00_JPRB, 0.34793E+00_JPRB, 0.36367E+00_JPRB ,&
&  0.37649E+00_JPRB, 0.38989E+00_JPRB, 0.40321E+00_JPRB, 0.41442E+00_JPRB, 0.43254E+00_JPRB, 0.46158E+00_JPRB, 0.49789E+00_JPRB ,&
&  0.55728E+00_JPRB, 0.63800E+00_JPRB, 0.74551E+00_JPRB, 0.75557E+00_JPRB, 0.79360E+00_JPRB, 0.88374E+00_JPRB, 0.10031E+01_JPRB ,&
&  0.10560E+01_JPRB, 0.99115E+00_JPRB, 0.99874E+00_JPRB, 0.98946E+00_JPRB, 0.96100E+00_JPRB, 0.90847E+00_JPRB, 0.82097E+00_JPRB ,&
&  0.74425E+00_JPRB, 0.67629E+00_JPRB, 0.62043E+00_JPRB, 0.58629E+00_JPRB, 0.56673E+00_JPRB, 0.54493E+00_JPRB, 0.52050E+00_JPRB ,&
&  0.49165E+00_JPRB, 0.45474E+00_JPRB, 0.41569E+00_JPRB, 0.39458E+00_JPRB, 0.38888E+00_JPRB, 0.38759E+00_JPRB, 0.38478E+00_JPRB ,&
&  0.37849E+00_JPRB, 0.37197E+00_JPRB, 0.36589E+00_JPRB, 0.35382E+00_JPRB, 0.34629E+00_JPRB, 0.33464E+00_JPRB, 0.32833E+00_JPRB ,&
&  0.32555E+00_JPRB, 0.32507E+00_JPRB, 0.32947E+00_JPRB, 0.32522E+00_JPRB, 0.32453E+00_JPRB, 0.32697E+00_JPRB, 0.32462E+00_JPRB ,&
&  0.32340E+00_JPRB, 0.32422E+00_JPRB, 0.32318E+00_JPRB, 0.32849E+00_JPRB, 0.33654E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.34690E+00_JPRB, 0.35526E+00_JPRB, 0.36182E+00_JPRB, 0.36477E+00_JPRB, 0.36240E+00_JPRB, 0.36179E+00_JPRB, 0.35617E+00_JPRB ,&
&  0.35351E+00_JPRB, 0.35537E+00_JPRB, 0.35416E+00_JPRB, 0.35584E+00_JPRB, 0.35809E+00_JPRB, 0.36060E+00_JPRB, 0.36336E+00_JPRB ,&
&  0.36657E+00_JPRB, 0.37017E+00_JPRB, 0.37392E+00_JPRB, 0.37809E+00_JPRB, 0.38264E+00_JPRB, 0.38760E+00_JPRB, 0.39235E+00_JPRB ,&
&  0.39477E+00_JPRB, 0.39427E+00_JPRB, 0.39520E+00_JPRB, 0.39333E+00_JPRB, 0.39017E+00_JPRB, 0.38487E+00_JPRB, 0.38013E+00_JPRB ,&
&  0.38167E+00_JPRB, 0.38881E+00_JPRB, 0.39109E+00_JPRB, 0.39900E+00_JPRB, 0.41424E+00_JPRB, 0.40605E+00_JPRB, 0.37227E+00_JPRB ,&
&  0.33448E+00_JPRB, 0.30357E+00_JPRB, 0.28973E+00_JPRB, 0.28514E+00_JPRB, 0.29087E+00_JPRB, 0.30279E+00_JPRB, 0.31753E+00_JPRB ,&
&  0.33255E+00_JPRB, 0.35204E+00_JPRB, 0.37107E+00_JPRB, 0.38380E+00_JPRB, 0.39534E+00_JPRB, 0.40942E+00_JPRB, 0.42051E+00_JPRB ,&
&  0.43627E+00_JPRB, 0.46331E+00_JPRB, 0.51151E+00_JPRB, 0.57029E+00_JPRB, 0.62397E+00_JPRB, 0.65298E+00_JPRB, 0.73107E+00_JPRB ,&
&  0.87748E+00_JPRB, 0.80839E+00_JPRB, 0.78094E+00_JPRB, 0.74087E+00_JPRB, 0.69921E+00_JPRB, 0.66250E+00_JPRB, 0.63606E+00_JPRB ,&
&  0.61994E+00_JPRB, 0.60622E+00_JPRB, 0.58048E+00_JPRB, 0.54869E+00_JPRB, 0.52156E+00_JPRB, 0.50046E+00_JPRB, 0.48800E+00_JPRB ,&
&  0.47463E+00_JPRB, 0.45512E+00_JPRB, 0.43091E+00_JPRB, 0.39342E+00_JPRB, 0.37811E+00_JPRB, 0.37018E+00_JPRB, 0.36343E+00_JPRB ,&
&  0.35873E+00_JPRB, 0.35567E+00_JPRB, 0.35100E+00_JPRB, 0.34340E+00_JPRB, 0.34070E+00_JPRB, 0.33557E+00_JPRB, 0.33417E+00_JPRB ,&
&  0.33363E+00_JPRB, 0.33440E+00_JPRB, 0.33843E+00_JPRB, 0.34061E+00_JPRB, 0.33815E+00_JPRB, 0.33782E+00_JPRB, 0.33728E+00_JPRB ,&
&  0.33387E+00_JPRB, 0.33182E+00_JPRB, 0.33238E+00_JPRB, 0.33628E+00_JPRB, 0.34035E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.35853E+00_JPRB, 0.36263E+00_JPRB, 0.36454E+00_JPRB, 0.36663E+00_JPRB, 0.36478E+00_JPRB, 0.36440E+00_JPRB, 0.36031E+00_JPRB ,&
&  0.35687E+00_JPRB, 0.35508E+00_JPRB, 0.35451E+00_JPRB, 0.35681E+00_JPRB, 0.36095E+00_JPRB, 0.36578E+00_JPRB, 0.37021E+00_JPRB ,&
&  0.37442E+00_JPRB, 0.37446E+00_JPRB, 0.37725E+00_JPRB, 0.38216E+00_JPRB, 0.38461E+00_JPRB, 0.38825E+00_JPRB, 0.39034E+00_JPRB ,&
&  0.39081E+00_JPRB, 0.38842E+00_JPRB, 0.38660E+00_JPRB, 0.38171E+00_JPRB, 0.37840E+00_JPRB, 0.37650E+00_JPRB, 0.37764E+00_JPRB ,&
&  0.37161E+00_JPRB, 0.36876E+00_JPRB, 0.38057E+00_JPRB, 0.38838E+00_JPRB, 0.37528E+00_JPRB, 0.36574E+00_JPRB, 0.34105E+00_JPRB ,&
&  0.31520E+00_JPRB, 0.29014E+00_JPRB, 0.28117E+00_JPRB, 0.27630E+00_JPRB, 0.27952E+00_JPRB, 0.28541E+00_JPRB, 0.29688E+00_JPRB ,&
&  0.30827E+00_JPRB, 0.32430E+00_JPRB, 0.33899E+00_JPRB, 0.35477E+00_JPRB, 0.36518E+00_JPRB, 0.37721E+00_JPRB, 0.38593E+00_JPRB ,&
&  0.39705E+00_JPRB, 0.41239E+00_JPRB, 0.43490E+00_JPRB, 0.46264E+00_JPRB, 0.48795E+00_JPRB, 0.51800E+00_JPRB, 0.55991E+00_JPRB ,&
&  0.62291E+00_JPRB, 0.61155E+00_JPRB, 0.61076E+00_JPRB, 0.59480E+00_JPRB, 0.57149E+00_JPRB, 0.55213E+00_JPRB, 0.53495E+00_JPRB ,&
&  0.52706E+00_JPRB, 0.52641E+00_JPRB, 0.51350E+00_JPRB, 0.50123E+00_JPRB, 0.48613E+00_JPRB, 0.47098E+00_JPRB, 0.46089E+00_JPRB ,&
&  0.45284E+00_JPRB, 0.44328E+00_JPRB, 0.42879E+00_JPRB, 0.40180E+00_JPRB, 0.38503E+00_JPRB, 0.37156E+00_JPRB, 0.36338E+00_JPRB ,&
&  0.35780E+00_JPRB, 0.35388E+00_JPRB, 0.34988E+00_JPRB, 0.34618E+00_JPRB, 0.34531E+00_JPRB, 0.34174E+00_JPRB, 0.34293E+00_JPRB ,&
&  0.34468E+00_JPRB, 0.34689E+00_JPRB, 0.35005E+00_JPRB, 0.35092E+00_JPRB, 0.34592E+00_JPRB, 0.34159E+00_JPRB, 0.34108E+00_JPRB ,&
&  0.34039E+00_JPRB, 0.34297E+00_JPRB, 0.34637E+00_JPRB, 0.35084E+00_JPRB, 0.35446E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.37212E+00_JPRB, 0.37565E+00_JPRB, 0.37768E+00_JPRB, 0.37726E+00_JPRB, 0.37530E+00_JPRB, 0.37401E+00_JPRB, 0.37205E+00_JPRB ,&
&  0.37065E+00_JPRB, 0.36938E+00_JPRB, 0.36896E+00_JPRB, 0.37039E+00_JPRB, 0.37406E+00_JPRB, 0.37739E+00_JPRB, 0.37948E+00_JPRB ,&
&  0.38056E+00_JPRB, 0.38211E+00_JPRB, 0.38240E+00_JPRB, 0.38352E+00_JPRB, 0.38434E+00_JPRB, 0.38695E+00_JPRB, 0.38780E+00_JPRB ,&
&  0.38907E+00_JPRB, 0.38607E+00_JPRB, 0.37824E+00_JPRB, 0.37430E+00_JPRB, 0.37501E+00_JPRB, 0.36656E+00_JPRB, 0.36832E+00_JPRB ,&
&  0.37274E+00_JPRB, 0.37414E+00_JPRB, 0.37072E+00_JPRB, 0.36970E+00_JPRB, 0.36130E+00_JPRB, 0.31491E+00_JPRB, 0.31447E+00_JPRB ,&
&  0.30451E+00_JPRB, 0.28962E+00_JPRB, 0.28244E+00_JPRB, 0.27684E+00_JPRB, 0.27547E+00_JPRB, 0.28192E+00_JPRB, 0.29196E+00_JPRB ,&
&  0.29785E+00_JPRB, 0.31792E+00_JPRB, 0.32665E+00_JPRB, 0.33728E+00_JPRB, 0.34675E+00_JPRB, 0.35914E+00_JPRB, 0.36753E+00_JPRB ,&
&  0.37872E+00_JPRB, 0.38991E+00_JPRB, 0.40752E+00_JPRB, 0.42929E+00_JPRB, 0.45095E+00_JPRB, 0.47100E+00_JPRB, 0.49100E+00_JPRB ,&
&  0.52873E+00_JPRB, 0.52894E+00_JPRB, 0.53158E+00_JPRB, 0.52911E+00_JPRB, 0.51389E+00_JPRB, 0.50168E+00_JPRB, 0.48768E+00_JPRB ,&
&  0.48329E+00_JPRB, 0.48173E+00_JPRB, 0.48027E+00_JPRB, 0.47254E+00_JPRB, 0.46453E+00_JPRB, 0.45738E+00_JPRB, 0.45138E+00_JPRB ,&
&  0.44480E+00_JPRB, 0.43604E+00_JPRB, 0.42501E+00_JPRB, 0.40778E+00_JPRB, 0.39859E+00_JPRB, 0.38697E+00_JPRB, 0.37910E+00_JPRB ,&
&  0.37223E+00_JPRB, 0.36539E+00_JPRB, 0.35745E+00_JPRB, 0.35695E+00_JPRB, 0.35719E+00_JPRB, 0.35815E+00_JPRB, 0.35874E+00_JPRB ,&
&  0.35845E+00_JPRB, 0.35893E+00_JPRB, 0.35943E+00_JPRB, 0.35819E+00_JPRB, 0.35377E+00_JPRB, 0.35009E+00_JPRB, 0.35126E+00_JPRB ,&
&  0.35394E+00_JPRB, 0.35708E+00_JPRB, 0.35993E+00_JPRB, 0.36382E+00_JPRB, 0.36736E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.37307E+00_JPRB, 0.37452E+00_JPRB, 0.37572E+00_JPRB, 0.37594E+00_JPRB, 0.37639E+00_JPRB, 0.37725E+00_JPRB, 0.37834E+00_JPRB ,&
&  0.37868E+00_JPRB, 0.37853E+00_JPRB, 0.37845E+00_JPRB, 0.37826E+00_JPRB, 0.37982E+00_JPRB, 0.38214E+00_JPRB, 0.38503E+00_JPRB ,&
&  0.38588E+00_JPRB, 0.38613E+00_JPRB, 0.38525E+00_JPRB, 0.38436E+00_JPRB, 0.38364E+00_JPRB, 0.38372E+00_JPRB, 0.38274E+00_JPRB ,&
&  0.38293E+00_JPRB, 0.38023E+00_JPRB, 0.37092E+00_JPRB, 0.36939E+00_JPRB, 0.36602E+00_JPRB, 0.36467E+00_JPRB, 0.36238E+00_JPRB ,&
&  0.35714E+00_JPRB, 0.33624E+00_JPRB, 0.34537E+00_JPRB, 0.33655E+00_JPRB, 0.32762E+00_JPRB, 0.31979E+00_JPRB, 0.30974E+00_JPRB ,&
&  0.30390E+00_JPRB, 0.29600E+00_JPRB, 0.29042E+00_JPRB, 0.28869E+00_JPRB, 0.28934E+00_JPRB, 0.29478E+00_JPRB, 0.30176E+00_JPRB ,&
&  0.30522E+00_JPRB, 0.31677E+00_JPRB, 0.32365E+00_JPRB, 0.33182E+00_JPRB, 0.34181E+00_JPRB, 0.35394E+00_JPRB, 0.36186E+00_JPRB ,&
&  0.37315E+00_JPRB, 0.37891E+00_JPRB, 0.39139E+00_JPRB, 0.40371E+00_JPRB, 0.42386E+00_JPRB, 0.44116E+00_JPRB, 0.45848E+00_JPRB ,&
&  0.48124E+00_JPRB, 0.49144E+00_JPRB, 0.49696E+00_JPRB, 0.49710E+00_JPRB, 0.48814E+00_JPRB, 0.47901E+00_JPRB, 0.46873E+00_JPRB ,&
&  0.46095E+00_JPRB, 0.45448E+00_JPRB, 0.45070E+00_JPRB, 0.44678E+00_JPRB, 0.44134E+00_JPRB, 0.43809E+00_JPRB, 0.43530E+00_JPRB ,&
&  0.43189E+00_JPRB, 0.42419E+00_JPRB, 0.41746E+00_JPRB, 0.40614E+00_JPRB, 0.39766E+00_JPRB, 0.38598E+00_JPRB, 0.37663E+00_JPRB ,&
&  0.37006E+00_JPRB, 0.36264E+00_JPRB, 0.35646E+00_JPRB, 0.35344E+00_JPRB, 0.35299E+00_JPRB, 0.35471E+00_JPRB, 0.35809E+00_JPRB ,&
&  0.36185E+00_JPRB, 0.36602E+00_JPRB, 0.36959E+00_JPRB, 0.36775E+00_JPRB, 0.36172E+00_JPRB, 0.35717E+00_JPRB, 0.35626E+00_JPRB ,&
&  0.35817E+00_JPRB, 0.36151E+00_JPRB, 0.36506E+00_JPRB, 0.36957E+00_JPRB, 0.37219E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.37321E+00_JPRB, 0.37204E+00_JPRB, 0.37067E+00_JPRB, 0.36893E+00_JPRB, 0.36823E+00_JPRB, 0.36798E+00_JPRB, 0.36973E+00_JPRB ,&
&  0.37164E+00_JPRB, 0.37416E+00_JPRB, 0.37587E+00_JPRB, 0.37680E+00_JPRB, 0.37784E+00_JPRB, 0.37802E+00_JPRB, 0.37924E+00_JPRB ,&
&  0.38126E+00_JPRB, 0.38326E+00_JPRB, 0.38438E+00_JPRB, 0.38435E+00_JPRB, 0.38365E+00_JPRB, 0.38307E+00_JPRB, 0.37917E+00_JPRB ,&
&  0.37623E+00_JPRB, 0.37331E+00_JPRB, 0.35994E+00_JPRB, 0.36231E+00_JPRB, 0.35914E+00_JPRB, 0.35772E+00_JPRB, 0.35655E+00_JPRB ,&
&  0.34765E+00_JPRB, 0.34395E+00_JPRB, 0.33399E+00_JPRB, 0.32552E+00_JPRB, 0.32608E+00_JPRB, 0.31905E+00_JPRB, 0.31960E+00_JPRB ,&
&  0.31776E+00_JPRB, 0.31545E+00_JPRB, 0.31499E+00_JPRB, 0.31515E+00_JPRB, 0.31827E+00_JPRB, 0.32055E+00_JPRB, 0.32296E+00_JPRB ,&
&  0.32700E+00_JPRB, 0.33159E+00_JPRB, 0.33931E+00_JPRB, 0.34202E+00_JPRB, 0.35250E+00_JPRB, 0.36411E+00_JPRB, 0.37630E+00_JPRB ,&
&  0.38510E+00_JPRB, 0.38586E+00_JPRB, 0.39000E+00_JPRB, 0.39579E+00_JPRB, 0.40221E+00_JPRB, 0.41073E+00_JPRB, 0.42194E+00_JPRB ,&
&  0.43698E+00_JPRB, 0.44903E+00_JPRB, 0.45674E+00_JPRB, 0.45763E+00_JPRB, 0.45597E+00_JPRB, 0.45130E+00_JPRB, 0.44693E+00_JPRB ,&
&  0.44321E+00_JPRB, 0.43939E+00_JPRB, 0.43577E+00_JPRB, 0.43170E+00_JPRB, 0.42673E+00_JPRB, 0.42276E+00_JPRB, 0.41836E+00_JPRB ,&
&  0.41313E+00_JPRB, 0.40654E+00_JPRB, 0.40336E+00_JPRB, 0.39617E+00_JPRB, 0.39136E+00_JPRB, 0.38584E+00_JPRB, 0.37698E+00_JPRB ,&
&  0.36887E+00_JPRB, 0.36120E+00_JPRB, 0.35445E+00_JPRB, 0.34936E+00_JPRB, 0.34652E+00_JPRB, 0.34683E+00_JPRB, 0.34961E+00_JPRB ,&
&  0.35484E+00_JPRB, 0.36142E+00_JPRB, 0.36760E+00_JPRB, 0.36924E+00_JPRB, 0.36935E+00_JPRB, 0.36800E+00_JPRB, 0.36721E+00_JPRB ,&
&  0.36686E+00_JPRB, 0.36823E+00_JPRB, 0.36977E+00_JPRB, 0.37184E+00_JPRB, 0.37373E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.35763E+00_JPRB, 0.35796E+00_JPRB, 0.35856E+00_JPRB, 0.35925E+00_JPRB, 0.36049E+00_JPRB, 0.36206E+00_JPRB, 0.36378E+00_JPRB ,&
&  0.36566E+00_JPRB, 0.36798E+00_JPRB, 0.37013E+00_JPRB, 0.37187E+00_JPRB, 0.37310E+00_JPRB, 0.37454E+00_JPRB, 0.37569E+00_JPRB ,&
&  0.37582E+00_JPRB, 0.37594E+00_JPRB, 0.37667E+00_JPRB, 0.37741E+00_JPRB, 0.37788E+00_JPRB, 0.37824E+00_JPRB, 0.37565E+00_JPRB ,&
&  0.37355E+00_JPRB, 0.37161E+00_JPRB, 0.36921E+00_JPRB, 0.36552E+00_JPRB, 0.36318E+00_JPRB, 0.35142E+00_JPRB, 0.35311E+00_JPRB ,&
&  0.36398E+00_JPRB, 0.36310E+00_JPRB, 0.36094E+00_JPRB, 0.35670E+00_JPRB, 0.34248E+00_JPRB, 0.33570E+00_JPRB, 0.33438E+00_JPRB ,&
&  0.32450E+00_JPRB, 0.33266E+00_JPRB, 0.33534E+00_JPRB, 0.33672E+00_JPRB, 0.33959E+00_JPRB, 0.33803E+00_JPRB, 0.33915E+00_JPRB ,&
&  0.34688E+00_JPRB, 0.34842E+00_JPRB, 0.35417E+00_JPRB, 0.36130E+00_JPRB, 0.36906E+00_JPRB, 0.37610E+00_JPRB, 0.38325E+00_JPRB ,&
&  0.38854E+00_JPRB, 0.39188E+00_JPRB, 0.39369E+00_JPRB, 0.39389E+00_JPRB, 0.39502E+00_JPRB, 0.39756E+00_JPRB, 0.40253E+00_JPRB ,&
&  0.41101E+00_JPRB, 0.42077E+00_JPRB, 0.42903E+00_JPRB, 0.43369E+00_JPRB, 0.43925E+00_JPRB, 0.44272E+00_JPRB, 0.44548E+00_JPRB ,&
&  0.44687E+00_JPRB, 0.44575E+00_JPRB, 0.44106E+00_JPRB, 0.43663E+00_JPRB, 0.43220E+00_JPRB, 0.42673E+00_JPRB, 0.41954E+00_JPRB ,&
&  0.41200E+00_JPRB, 0.40413E+00_JPRB, 0.39678E+00_JPRB, 0.39023E+00_JPRB, 0.38444E+00_JPRB, 0.37876E+00_JPRB, 0.37385E+00_JPRB ,&
&  0.36854E+00_JPRB, 0.36364E+00_JPRB, 0.35846E+00_JPRB, 0.35352E+00_JPRB, 0.34922E+00_JPRB, 0.34671E+00_JPRB, 0.34538E+00_JPRB ,&
&  0.34607E+00_JPRB, 0.34792E+00_JPRB, 0.34943E+00_JPRB, 0.35054E+00_JPRB, 0.35237E+00_JPRB, 0.35455E+00_JPRB, 0.35609E+00_JPRB ,&
&  0.35720E+00_JPRB, 0.35730E+00_JPRB, 0.35721E+00_JPRB, 0.35730E+00_JPRB, 0.35746E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.34645E+00_JPRB, 0.34705E+00_JPRB, 0.34768E+00_JPRB, 0.34840E+00_JPRB, 0.34948E+00_JPRB, 0.35071E+00_JPRB, 0.35196E+00_JPRB ,&
&  0.35319E+00_JPRB, 0.35473E+00_JPRB, 0.35632E+00_JPRB, 0.35774E+00_JPRB, 0.35889E+00_JPRB, 0.36027E+00_JPRB, 0.36156E+00_JPRB ,&
&  0.36255E+00_JPRB, 0.36329E+00_JPRB, 0.36371E+00_JPRB, 0.36424E+00_JPRB, 0.36483E+00_JPRB, 0.36533E+00_JPRB, 0.36652E+00_JPRB ,&
&  0.36767E+00_JPRB, 0.36837E+00_JPRB, 0.36849E+00_JPRB, 0.36606E+00_JPRB, 0.36442E+00_JPRB, 0.36290E+00_JPRB, 0.36160E+00_JPRB ,&
&  0.35970E+00_JPRB, 0.35952E+00_JPRB, 0.35959E+00_JPRB, 0.35964E+00_JPRB, 0.36133E+00_JPRB, 0.36113E+00_JPRB, 0.36070E+00_JPRB ,&
&  0.35985E+00_JPRB, 0.35714E+00_JPRB, 0.35763E+00_JPRB, 0.35842E+00_JPRB, 0.35920E+00_JPRB, 0.36119E+00_JPRB, 0.36301E+00_JPRB ,&
&  0.36514E+00_JPRB, 0.36744E+00_JPRB, 0.37124E+00_JPRB, 0.37545E+00_JPRB, 0.37919E+00_JPRB, 0.38258E+00_JPRB, 0.38502E+00_JPRB ,&
&  0.38710E+00_JPRB, 0.38925E+00_JPRB, 0.39149E+00_JPRB, 0.39364E+00_JPRB, 0.39556E+00_JPRB, 0.39834E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40447E+00_JPRB, 0.40787E+00_JPRB, 0.41332E+00_JPRB, 0.41990E+00_JPRB, 0.42679E+00_JPRB, 0.43125E+00_JPRB, 0.43454E+00_JPRB ,&
&  0.43663E+00_JPRB, 0.43713E+00_JPRB, 0.43531E+00_JPRB, 0.43261E+00_JPRB, 0.42927E+00_JPRB, 0.42449E+00_JPRB, 0.41910E+00_JPRB ,&
&  0.41370E+00_JPRB, 0.40834E+00_JPRB, 0.40267E+00_JPRB, 0.39654E+00_JPRB, 0.39086E+00_JPRB, 0.38541E+00_JPRB, 0.38064E+00_JPRB ,&
&  0.37552E+00_JPRB, 0.37092E+00_JPRB, 0.36651E+00_JPRB, 0.36263E+00_JPRB, 0.35860E+00_JPRB, 0.35514E+00_JPRB, 0.35212E+00_JPRB ,&
&  0.34997E+00_JPRB, 0.34781E+00_JPRB, 0.34572E+00_JPRB, 0.34425E+00_JPRB, 0.34384E+00_JPRB, 0.34400E+00_JPRB, 0.34433E+00_JPRB ,&
&  0.34468E+00_JPRB, 0.34507E+00_JPRB, 0.34555E+00_JPRB, 0.34586E+00_JPRB, 0.34603E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.35659E+00_JPRB, 0.35689E+00_JPRB, 0.35709E+00_JPRB, 0.35731E+00_JPRB, 0.35750E+00_JPRB, 0.35777E+00_JPRB, 0.35802E+00_JPRB ,&
&  0.35825E+00_JPRB, 0.35860E+00_JPRB, 0.35910E+00_JPRB, 0.35956E+00_JPRB, 0.36004E+00_JPRB, 0.36043E+00_JPRB, 0.36086E+00_JPRB ,&
&  0.36117E+00_JPRB, 0.36138E+00_JPRB, 0.36158E+00_JPRB, 0.36206E+00_JPRB, 0.36251E+00_JPRB, 0.36289E+00_JPRB, 0.36335E+00_JPRB ,&
&  0.36360E+00_JPRB, 0.36350E+00_JPRB, 0.36333E+00_JPRB, 0.36360E+00_JPRB, 0.36381E+00_JPRB, 0.36404E+00_JPRB, 0.36426E+00_JPRB ,&
&  0.36444E+00_JPRB, 0.36466E+00_JPRB, 0.36479E+00_JPRB, 0.36488E+00_JPRB, 0.36521E+00_JPRB, 0.36570E+00_JPRB, 0.36626E+00_JPRB ,&
&  0.36690E+00_JPRB, 0.36758E+00_JPRB, 0.36845E+00_JPRB, 0.36932E+00_JPRB, 0.37023E+00_JPRB, 0.37080E+00_JPRB, 0.37148E+00_JPRB ,&
&  0.37210E+00_JPRB, 0.37273E+00_JPRB, 0.37353E+00_JPRB, 0.37446E+00_JPRB, 0.37500E+00_JPRB, 0.37570E+00_JPRB, 0.37613E+00_JPRB ,&
&  0.37720E+00_JPRB, 0.37861E+00_JPRB, 0.38019E+00_JPRB, 0.38185E+00_JPRB, 0.38341E+00_JPRB, 0.38511E+00_JPRB, 0.38705E+00_JPRB ,&
&  0.38893E+00_JPRB, 0.39069E+00_JPRB, 0.39290E+00_JPRB, 0.39546E+00_JPRB, 0.39815E+00_JPRB, 0.40002E+00_JPRB, 0.40182E+00_JPRB ,&
&  0.40358E+00_JPRB, 0.40513E+00_JPRB, 0.40512E+00_JPRB, 0.40496E+00_JPRB, 0.40450E+00_JPRB, 0.40342E+00_JPRB, 0.40176E+00_JPRB ,&
&  0.39983E+00_JPRB, 0.39776E+00_JPRB, 0.39490E+00_JPRB, 0.39169E+00_JPRB, 0.38873E+00_JPRB, 0.38592E+00_JPRB, 0.38314E+00_JPRB ,&
&  0.38041E+00_JPRB, 0.37801E+00_JPRB, 0.37575E+00_JPRB, 0.37353E+00_JPRB, 0.37116E+00_JPRB, 0.36892E+00_JPRB, 0.36668E+00_JPRB ,&
&  0.36476E+00_JPRB, 0.36292E+00_JPRB, 0.36123E+00_JPRB, 0.35967E+00_JPRB, 0.35857E+00_JPRB, 0.35786E+00_JPRB, 0.35727E+00_JPRB ,&
&  0.35679E+00_JPRB, 0.35659E+00_JPRB, 0.35657E+00_JPRB, 0.35653E+00_JPRB, 0.35645E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB, 0.37318E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB, 0.20923E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.23515E+00_JPRB, 0.23521E+00_JPRB, 0.23526E+00_JPRB, 0.23491E+00_JPRB, 0.23347E+00_JPRB, 0.23242E+00_JPRB, 0.23153E+00_JPRB ,&
&  0.23114E+00_JPRB, 0.23039E+00_JPRB, 0.23001E+00_JPRB, 0.22989E+00_JPRB, 0.22958E+00_JPRB, 0.22910E+00_JPRB, 0.22847E+00_JPRB ,&
&  0.22766E+00_JPRB, 0.22658E+00_JPRB, 0.22557E+00_JPRB, 0.22551E+00_JPRB, 0.22580E+00_JPRB, 0.22610E+00_JPRB, 0.22636E+00_JPRB ,&
&  0.22651E+00_JPRB, 0.22660E+00_JPRB, 0.22622E+00_JPRB, 0.22626E+00_JPRB, 0.22630E+00_JPRB, 0.22631E+00_JPRB, 0.22646E+00_JPRB ,&
&  0.22672E+00_JPRB, 0.22691E+00_JPRB, 0.22665E+00_JPRB, 0.22617E+00_JPRB, 0.22523E+00_JPRB, 0.22357E+00_JPRB, 0.22099E+00_JPRB ,&
&  0.21806E+00_JPRB, 0.21512E+00_JPRB, 0.21276E+00_JPRB, 0.21059E+00_JPRB, 0.20860E+00_JPRB, 0.20678E+00_JPRB, 0.20526E+00_JPRB ,&
&  0.20348E+00_JPRB, 0.20171E+00_JPRB, 0.20010E+00_JPRB, 0.19898E+00_JPRB, 0.19777E+00_JPRB, 0.19672E+00_JPRB, 0.19581E+00_JPRB ,&
&  0.19519E+00_JPRB, 0.19465E+00_JPRB, 0.19417E+00_JPRB, 0.19393E+00_JPRB, 0.19374E+00_JPRB, 0.19355E+00_JPRB, 0.19334E+00_JPRB ,&
&  0.19314E+00_JPRB, 0.19299E+00_JPRB, 0.19285E+00_JPRB, 0.19278E+00_JPRB, 0.19282E+00_JPRB, 0.19267E+00_JPRB, 0.19249E+00_JPRB ,&
&  0.19242E+00_JPRB, 0.19263E+00_JPRB, 0.19280E+00_JPRB, 0.19310E+00_JPRB, 0.19357E+00_JPRB, 0.19423E+00_JPRB, 0.19495E+00_JPRB ,&
&  0.19568E+00_JPRB, 0.19652E+00_JPRB, 0.19745E+00_JPRB, 0.19857E+00_JPRB, 0.19976E+00_JPRB, 0.20095E+00_JPRB, 0.20209E+00_JPRB ,&
&  0.20332E+00_JPRB, 0.20445E+00_JPRB, 0.20550E+00_JPRB, 0.20629E+00_JPRB, 0.20732E+00_JPRB, 0.20835E+00_JPRB, 0.20954E+00_JPRB ,&
&  0.21089E+00_JPRB, 0.21264E+00_JPRB, 0.21498E+00_JPRB, 0.21774E+00_JPRB, 0.22107E+00_JPRB, 0.22361E+00_JPRB, 0.22624E+00_JPRB ,&
&  0.22820E+00_JPRB, 0.22965E+00_JPRB, 0.23071E+00_JPRB, 0.23209E+00_JPRB, 0.23363E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.27051E+00_JPRB, 0.27629E+00_JPRB, 0.28046E+00_JPRB, 0.28968E+00_JPRB, 0.30450E+00_JPRB, 0.39974E+00_JPRB, 0.39470E+00_JPRB ,&
&  0.39354E+00_JPRB, 0.35085E+00_JPRB, 0.34251E+00_JPRB, 0.33023E+00_JPRB, 0.31488E+00_JPRB, 0.29585E+00_JPRB, 0.28400E+00_JPRB ,&
&  0.27414E+00_JPRB, 0.26278E+00_JPRB, 0.25201E+00_JPRB, 0.24579E+00_JPRB, 0.24083E+00_JPRB, 0.23657E+00_JPRB, 0.23426E+00_JPRB ,&
&  0.23369E+00_JPRB, 0.23449E+00_JPRB, 0.23564E+00_JPRB, 0.23753E+00_JPRB, 0.24113E+00_JPRB, 0.24497E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.25090E+00_JPRB, 0.25484E+00_JPRB, 0.25542E+00_JPRB, 0.25216E+00_JPRB, 0.24668E+00_JPRB, 0.23965E+00_JPRB, 0.23254E+00_JPRB ,&
&  0.22520E+00_JPRB, 0.21828E+00_JPRB, 0.21260E+00_JPRB, 0.20741E+00_JPRB, 0.20162E+00_JPRB, 0.19631E+00_JPRB, 0.19298E+00_JPRB ,&
&  0.19004E+00_JPRB, 0.18709E+00_JPRB, 0.18481E+00_JPRB, 0.18364E+00_JPRB, 0.18231E+00_JPRB, 0.18144E+00_JPRB, 0.18082E+00_JPRB ,&
&  0.18065E+00_JPRB, 0.18057E+00_JPRB, 0.18064E+00_JPRB, 0.18094E+00_JPRB, 0.18092E+00_JPRB, 0.18086E+00_JPRB, 0.18083E+00_JPRB ,&
&  0.18136E+00_JPRB, 0.18137E+00_JPRB, 0.18126E+00_JPRB, 0.18100E+00_JPRB, 0.18151E+00_JPRB, 0.18160E+00_JPRB, 0.18185E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18316E+00_JPRB, 0.18378E+00_JPRB, 0.18452E+00_JPRB, 0.18523E+00_JPRB, 0.18605E+00_JPRB, 0.18701E+00_JPRB ,&
&  0.18796E+00_JPRB, 0.18937E+00_JPRB, 0.19082E+00_JPRB, 0.19212E+00_JPRB, 0.19326E+00_JPRB, 0.19483E+00_JPRB, 0.19659E+00_JPRB ,&
&  0.19926E+00_JPRB, 0.20188E+00_JPRB, 0.20457E+00_JPRB, 0.20635E+00_JPRB, 0.20906E+00_JPRB, 0.21131E+00_JPRB, 0.21364E+00_JPRB ,&
&  0.21628E+00_JPRB, 0.22036E+00_JPRB, 0.22605E+00_JPRB, 0.23207E+00_JPRB, 0.23850E+00_JPRB, 0.24473E+00_JPRB, 0.24890E+00_JPRB ,&
&  0.25269E+00_JPRB, 0.25672E+00_JPRB, 0.26288E+00_JPRB, 0.26620E+00_JPRB, 0.26891E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.48761E+00_JPRB, 0.48956E+00_JPRB, 0.49174E+00_JPRB, 0.49547E+00_JPRB, 0.49939E+00_JPRB, 0.50353E+00_JPRB, 0.50733E+00_JPRB ,&
&  0.48930E+00_JPRB, 0.46615E+00_JPRB, 0.44727E+00_JPRB, 0.42490E+00_JPRB, 0.40451E+00_JPRB, 0.38820E+00_JPRB, 0.36818E+00_JPRB ,&
&  0.34833E+00_JPRB, 0.32842E+00_JPRB, 0.30586E+00_JPRB, 0.28738E+00_JPRB, 0.26919E+00_JPRB, 0.25444E+00_JPRB, 0.24509E+00_JPRB ,&
&  0.24029E+00_JPRB, 0.23911E+00_JPRB, 0.24078E+00_JPRB, 0.24447E+00_JPRB, 0.24780E+00_JPRB, 0.25276E+00_JPRB, 0.25946E+00_JPRB ,&
&  0.26872E+00_JPRB, 0.28269E+00_JPRB, 0.29533E+00_JPRB, 0.30015E+00_JPRB, 0.29706E+00_JPRB, 0.28905E+00_JPRB, 0.28219E+00_JPRB ,&
&  0.27375E+00_JPRB, 0.26144E+00_JPRB, 0.24685E+00_JPRB, 0.23248E+00_JPRB, 0.21912E+00_JPRB, 0.20937E+00_JPRB, 0.20147E+00_JPRB ,&
&  0.19488E+00_JPRB, 0.18916E+00_JPRB, 0.18484E+00_JPRB, 0.18190E+00_JPRB, 0.17939E+00_JPRB, 0.17731E+00_JPRB, 0.17540E+00_JPRB ,&
&  0.17436E+00_JPRB, 0.17326E+00_JPRB, 0.17258E+00_JPRB, 0.17220E+00_JPRB, 0.17176E+00_JPRB, 0.17166E+00_JPRB, 0.17158E+00_JPRB ,&
&  0.17129E+00_JPRB, 0.17070E+00_JPRB, 0.17036E+00_JPRB, 0.17012E+00_JPRB, 0.17044E+00_JPRB, 0.17069E+00_JPRB, 0.17112E+00_JPRB ,&
&  0.17184E+00_JPRB, 0.17344E+00_JPRB, 0.17450E+00_JPRB, 0.17546E+00_JPRB, 0.17597E+00_JPRB, 0.17637E+00_JPRB, 0.17691E+00_JPRB ,&
&  0.17782E+00_JPRB, 0.17897E+00_JPRB, 0.18004E+00_JPRB, 0.18145E+00_JPRB, 0.18290E+00_JPRB, 0.18490E+00_JPRB, 0.18700E+00_JPRB ,&
&  0.18952E+00_JPRB, 0.19266E+00_JPRB, 0.19661E+00_JPRB, 0.20029E+00_JPRB, 0.20574E+00_JPRB, 0.21121E+00_JPRB, 0.21726E+00_JPRB ,&
&  0.22336E+00_JPRB, 0.22992E+00_JPRB, 0.23832E+00_JPRB, 0.24823E+00_JPRB, 0.26035E+00_JPRB, 0.26923E+00_JPRB, 0.28128E+00_JPRB ,&
&  0.29955E+00_JPRB, 0.33827E+00_JPRB, 0.47278E+00_JPRB, 0.48619E+00_JPRB, 0.49042E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.49984E+00_JPRB, 0.50861E+00_JPRB, 0.52300E+00_JPRB, 0.53464E+00_JPRB, 0.55928E+00_JPRB, 0.57309E+00_JPRB, 0.61992E+00_JPRB ,&
&  0.57398E+00_JPRB, 0.49453E+00_JPRB, 0.46265E+00_JPRB, 0.42833E+00_JPRB, 0.38882E+00_JPRB, 0.36056E+00_JPRB, 0.34055E+00_JPRB ,&
&  0.32878E+00_JPRB, 0.31876E+00_JPRB, 0.30920E+00_JPRB, 0.29714E+00_JPRB, 0.28467E+00_JPRB, 0.27052E+00_JPRB, 0.25965E+00_JPRB ,&
&  0.25305E+00_JPRB, 0.25095E+00_JPRB, 0.25063E+00_JPRB, 0.25513E+00_JPRB, 0.26135E+00_JPRB, 0.26973E+00_JPRB, 0.27920E+00_JPRB ,&
&  0.30006E+00_JPRB, 0.30556E+00_JPRB, 0.30488E+00_JPRB, 0.30517E+00_JPRB, 0.30657E+00_JPRB, 0.30861E+00_JPRB, 0.29895E+00_JPRB ,&
&  0.30965E+00_JPRB, 0.33285E+00_JPRB, 0.33909E+00_JPRB, 0.32128E+00_JPRB, 0.26682E+00_JPRB, 0.23372E+00_JPRB, 0.21128E+00_JPRB ,&
&  0.20242E+00_JPRB, 0.19309E+00_JPRB, 0.18727E+00_JPRB, 0.18191E+00_JPRB, 0.17871E+00_JPRB, 0.17690E+00_JPRB, 0.17517E+00_JPRB ,&
&  0.17381E+00_JPRB, 0.17262E+00_JPRB, 0.17068E+00_JPRB, 0.16974E+00_JPRB, 0.16880E+00_JPRB, 0.16832E+00_JPRB, 0.16802E+00_JPRB ,&
&  0.16788E+00_JPRB, 0.16756E+00_JPRB, 0.16771E+00_JPRB, 0.16820E+00_JPRB, 0.16907E+00_JPRB, 0.16976E+00_JPRB, 0.17040E+00_JPRB ,&
&  0.17112E+00_JPRB, 0.17176E+00_JPRB, 0.17184E+00_JPRB, 0.17215E+00_JPRB, 0.17245E+00_JPRB, 0.17340E+00_JPRB, 0.17377E+00_JPRB ,&
&  0.17420E+00_JPRB, 0.17450E+00_JPRB, 0.17443E+00_JPRB, 0.17486E+00_JPRB, 0.17565E+00_JPRB, 0.17730E+00_JPRB, 0.17924E+00_JPRB ,&
&  0.18176E+00_JPRB, 0.18466E+00_JPRB, 0.18792E+00_JPRB, 0.19114E+00_JPRB, 0.19543E+00_JPRB, 0.20043E+00_JPRB, 0.20672E+00_JPRB ,&
&  0.21365E+00_JPRB, 0.22193E+00_JPRB, 0.23282E+00_JPRB, 0.24637E+00_JPRB, 0.26275E+00_JPRB, 0.28087E+00_JPRB, 0.30248E+00_JPRB ,&
&  0.33851E+00_JPRB, 0.44000E+00_JPRB, 0.50007E+00_JPRB, 0.48525E+00_JPRB, 0.49458E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.57688E+00_JPRB, 0.59283E+00_JPRB, 0.61832E+00_JPRB, 0.61098E+00_JPRB, 0.63145E+00_JPRB, 0.61796E+00_JPRB, 0.59561E+00_JPRB ,&
&  0.54831E+00_JPRB, 0.49089E+00_JPRB, 0.44461E+00_JPRB, 0.39249E+00_JPRB, 0.35238E+00_JPRB, 0.32276E+00_JPRB, 0.30846E+00_JPRB ,&
&  0.30186E+00_JPRB, 0.29987E+00_JPRB, 0.29920E+00_JPRB, 0.29861E+00_JPRB, 0.29454E+00_JPRB, 0.29154E+00_JPRB, 0.28614E+00_JPRB ,&
&  0.28021E+00_JPRB, 0.27531E+00_JPRB, 0.27200E+00_JPRB, 0.27700E+00_JPRB, 0.28335E+00_JPRB, 0.29014E+00_JPRB, 0.29409E+00_JPRB ,&
&  0.29784E+00_JPRB, 0.29916E+00_JPRB, 0.30172E+00_JPRB, 0.30001E+00_JPRB, 0.31090E+00_JPRB, 0.33249E+00_JPRB, 0.35149E+00_JPRB ,&
&  0.36987E+00_JPRB, 0.38533E+00_JPRB, 0.38169E+00_JPRB, 0.35839E+00_JPRB, 0.29307E+00_JPRB, 0.25878E+00_JPRB, 0.22930E+00_JPRB ,&
&  0.21096E+00_JPRB, 0.19744E+00_JPRB, 0.18855E+00_JPRB, 0.18342E+00_JPRB, 0.17960E+00_JPRB, 0.17728E+00_JPRB, 0.17482E+00_JPRB ,&
&  0.17242E+00_JPRB, 0.17085E+00_JPRB, 0.16921E+00_JPRB, 0.16844E+00_JPRB, 0.16761E+00_JPRB, 0.16651E+00_JPRB, 0.16558E+00_JPRB ,&
&  0.16563E+00_JPRB, 0.16541E+00_JPRB, 0.16640E+00_JPRB, 0.16733E+00_JPRB, 0.16918E+00_JPRB, 0.17076E+00_JPRB, 0.17288E+00_JPRB ,&
&  0.17453E+00_JPRB, 0.17584E+00_JPRB, 0.17566E+00_JPRB, 0.17507E+00_JPRB, 0.17416E+00_JPRB, 0.17355E+00_JPRB, 0.17257E+00_JPRB ,&
&  0.17145E+00_JPRB, 0.17148E+00_JPRB, 0.17155E+00_JPRB, 0.17176E+00_JPRB, 0.17266E+00_JPRB, 0.17405E+00_JPRB, 0.17631E+00_JPRB ,&
&  0.17927E+00_JPRB, 0.18240E+00_JPRB, 0.18501E+00_JPRB, 0.18770E+00_JPRB, 0.19122E+00_JPRB, 0.19532E+00_JPRB, 0.20050E+00_JPRB ,&
&  0.20539E+00_JPRB, 0.21332E+00_JPRB, 0.22408E+00_JPRB, 0.23507E+00_JPRB, 0.25044E+00_JPRB, 0.26706E+00_JPRB, 0.29486E+00_JPRB ,&
&  0.34506E+00_JPRB, 0.52092E+00_JPRB, 0.80558E+00_JPRB, 0.57719E+00_JPRB, 0.56367E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.61313E+00_JPRB, 0.62363E+00_JPRB, 0.63761E+00_JPRB, 0.64033E+00_JPRB, 0.65036E+00_JPRB, 0.64923E+00_JPRB, 0.63234E+00_JPRB ,&
&  0.60413E+00_JPRB, 0.57791E+00_JPRB, 0.56346E+00_JPRB, 0.54705E+00_JPRB, 0.51980E+00_JPRB, 0.45904E+00_JPRB, 0.43567E+00_JPRB ,&
&  0.41740E+00_JPRB, 0.41719E+00_JPRB, 0.40830E+00_JPRB, 0.40861E+00_JPRB, 0.39785E+00_JPRB, 0.38667E+00_JPRB, 0.36858E+00_JPRB ,&
&  0.35169E+00_JPRB, 0.33430E+00_JPRB, 0.32247E+00_JPRB, 0.31633E+00_JPRB, 0.31427E+00_JPRB, 0.31022E+00_JPRB, 0.29881E+00_JPRB ,&
&  0.29052E+00_JPRB, 0.28892E+00_JPRB, 0.29172E+00_JPRB, 0.30268E+00_JPRB, 0.34753E+00_JPRB, 0.40330E+00_JPRB, 0.40336E+00_JPRB ,&
&  0.42669E+00_JPRB, 0.44071E+00_JPRB, 0.45429E+00_JPRB, 0.44386E+00_JPRB, 0.41177E+00_JPRB, 0.37426E+00_JPRB, 0.31978E+00_JPRB ,&
&  0.26790E+00_JPRB, 0.23938E+00_JPRB, 0.21628E+00_JPRB, 0.20049E+00_JPRB, 0.19006E+00_JPRB, 0.18563E+00_JPRB, 0.18005E+00_JPRB ,&
&  0.17629E+00_JPRB, 0.17351E+00_JPRB, 0.17234E+00_JPRB, 0.17163E+00_JPRB, 0.17091E+00_JPRB, 0.16899E+00_JPRB, 0.16841E+00_JPRB ,&
&  0.16889E+00_JPRB, 0.16914E+00_JPRB, 0.16908E+00_JPRB, 0.16984E+00_JPRB, 0.17082E+00_JPRB, 0.17217E+00_JPRB, 0.17332E+00_JPRB ,&
&  0.17499E+00_JPRB, 0.17590E+00_JPRB, 0.17742E+00_JPRB, 0.17901E+00_JPRB, 0.17811E+00_JPRB, 0.17687E+00_JPRB, 0.17406E+00_JPRB ,&
&  0.17104E+00_JPRB, 0.17050E+00_JPRB, 0.16986E+00_JPRB, 0.17034E+00_JPRB, 0.17173E+00_JPRB, 0.17372E+00_JPRB, 0.17638E+00_JPRB ,&
&  0.17896E+00_JPRB, 0.18162E+00_JPRB, 0.18370E+00_JPRB, 0.18574E+00_JPRB, 0.18910E+00_JPRB, 0.19274E+00_JPRB, 0.19737E+00_JPRB ,&
&  0.20219E+00_JPRB, 0.20761E+00_JPRB, 0.21519E+00_JPRB, 0.22500E+00_JPRB, 0.23866E+00_JPRB, 0.25448E+00_JPRB, 0.27902E+00_JPRB ,&
&  0.30314E+00_JPRB, 0.40516E+00_JPRB, 0.64245E+00_JPRB, 0.56828E+00_JPRB, 0.58590E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.68771E+00_JPRB, 0.69855E+00_JPRB, 0.70807E+00_JPRB, 0.70965E+00_JPRB, 0.70885E+00_JPRB, 0.70392E+00_JPRB, 0.69192E+00_JPRB ,&
&  0.67229E+00_JPRB, 0.64606E+00_JPRB, 0.62062E+00_JPRB, 0.60033E+00_JPRB, 0.58771E+00_JPRB, 0.59128E+00_JPRB, 0.60034E+00_JPRB ,&
&  0.61835E+00_JPRB, 0.64435E+00_JPRB, 0.65001E+00_JPRB, 0.64384E+00_JPRB, 0.60839E+00_JPRB, 0.57150E+00_JPRB, 0.53702E+00_JPRB ,&
&  0.50658E+00_JPRB, 0.47823E+00_JPRB, 0.46745E+00_JPRB, 0.44882E+00_JPRB, 0.43562E+00_JPRB, 0.41920E+00_JPRB, 0.39628E+00_JPRB ,&
&  0.35882E+00_JPRB, 0.32798E+00_JPRB, 0.30948E+00_JPRB, 0.32441E+00_JPRB, 0.41505E+00_JPRB, 0.47566E+00_JPRB, 0.48414E+00_JPRB ,&
&  0.51912E+00_JPRB, 0.54439E+00_JPRB, 0.53992E+00_JPRB, 0.52047E+00_JPRB, 0.49928E+00_JPRB, 0.47605E+00_JPRB, 0.45715E+00_JPRB ,&
&  0.43837E+00_JPRB, 0.41200E+00_JPRB, 0.36300E+00_JPRB, 0.29355E+00_JPRB, 0.25775E+00_JPRB, 0.24422E+00_JPRB, 0.22976E+00_JPRB ,&
&  0.21886E+00_JPRB, 0.20966E+00_JPRB, 0.20615E+00_JPRB, 0.20320E+00_JPRB, 0.20241E+00_JPRB, 0.19836E+00_JPRB, 0.19500E+00_JPRB ,&
&  0.19110E+00_JPRB, 0.18828E+00_JPRB, 0.18386E+00_JPRB, 0.17946E+00_JPRB, 0.17639E+00_JPRB, 0.17588E+00_JPRB, 0.17557E+00_JPRB ,&
&  0.17650E+00_JPRB, 0.18042E+00_JPRB, 0.19121E+00_JPRB, 0.19726E+00_JPRB, 0.19761E+00_JPRB, 0.19250E+00_JPRB, 0.18539E+00_JPRB ,&
&  0.17919E+00_JPRB, 0.17709E+00_JPRB, 0.17563E+00_JPRB, 0.17607E+00_JPRB, 0.17858E+00_JPRB, 0.18204E+00_JPRB, 0.18501E+00_JPRB ,&
&  0.18738E+00_JPRB, 0.18957E+00_JPRB, 0.19169E+00_JPRB, 0.19396E+00_JPRB, 0.19667E+00_JPRB, 0.19990E+00_JPRB, 0.20429E+00_JPRB ,&
&  0.20909E+00_JPRB, 0.21577E+00_JPRB, 0.22559E+00_JPRB, 0.23971E+00_JPRB, 0.25198E+00_JPRB, 0.25955E+00_JPRB, 0.26950E+00_JPRB ,&
&  0.28513E+00_JPRB, 0.33137E+00_JPRB, 0.46503E+00_JPRB, 0.69294E+00_JPRB, 0.72493E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.74732E+00_JPRB, 0.76505E+00_JPRB, 0.78582E+00_JPRB, 0.79682E+00_JPRB, 0.79782E+00_JPRB, 0.79485E+00_JPRB, 0.78616E+00_JPRB ,&
&  0.77125E+00_JPRB, 0.75327E+00_JPRB, 0.74009E+00_JPRB, 0.73126E+00_JPRB, 0.72756E+00_JPRB, 0.73080E+00_JPRB, 0.73670E+00_JPRB ,&
&  0.73662E+00_JPRB, 0.73305E+00_JPRB, 0.72138E+00_JPRB, 0.70614E+00_JPRB, 0.69004E+00_JPRB, 0.67751E+00_JPRB, 0.66784E+00_JPRB ,&
&  0.65824E+00_JPRB, 0.64353E+00_JPRB, 0.62052E+00_JPRB, 0.58531E+00_JPRB, 0.54377E+00_JPRB, 0.50569E+00_JPRB, 0.47710E+00_JPRB ,&
&  0.45354E+00_JPRB, 0.38585E+00_JPRB, 0.34823E+00_JPRB, 0.36129E+00_JPRB, 0.45596E+00_JPRB, 0.48556E+00_JPRB, 0.50932E+00_JPRB ,&
&  0.54697E+00_JPRB, 0.58697E+00_JPRB, 0.60603E+00_JPRB, 0.60670E+00_JPRB, 0.60133E+00_JPRB, 0.58884E+00_JPRB, 0.56932E+00_JPRB ,&
&  0.54450E+00_JPRB, 0.51406E+00_JPRB, 0.48828E+00_JPRB, 0.45973E+00_JPRB, 0.43017E+00_JPRB, 0.40826E+00_JPRB, 0.39292E+00_JPRB ,&
&  0.38231E+00_JPRB, 0.37289E+00_JPRB, 0.36552E+00_JPRB, 0.35587E+00_JPRB, 0.34670E+00_JPRB, 0.32569E+00_JPRB, 0.30426E+00_JPRB ,&
&  0.28037E+00_JPRB, 0.26329E+00_JPRB, 0.24469E+00_JPRB, 0.22664E+00_JPRB, 0.21184E+00_JPRB, 0.20229E+00_JPRB, 0.19542E+00_JPRB ,&
&  0.19309E+00_JPRB, 0.19834E+00_JPRB, 0.21629E+00_JPRB, 0.24937E+00_JPRB, 0.28923E+00_JPRB, 0.25738E+00_JPRB, 0.23613E+00_JPRB ,&
&  0.21606E+00_JPRB, 0.20423E+00_JPRB, 0.19981E+00_JPRB, 0.19929E+00_JPRB, 0.20056E+00_JPRB, 0.20290E+00_JPRB, 0.20519E+00_JPRB ,&
&  0.20815E+00_JPRB, 0.21089E+00_JPRB, 0.21385E+00_JPRB, 0.21593E+00_JPRB, 0.21728E+00_JPRB, 0.21986E+00_JPRB, 0.22439E+00_JPRB ,&
&  0.23121E+00_JPRB, 0.24267E+00_JPRB, 0.26208E+00_JPRB, 0.28733E+00_JPRB, 0.31324E+00_JPRB, 0.33691E+00_JPRB, 0.38983E+00_JPRB ,&
&  0.47722E+00_JPRB, 0.59004E+00_JPRB, 0.64547E+00_JPRB, 0.69980E+00_JPRB, 0.72650E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.80733E+00_JPRB, 0.83335E+00_JPRB, 0.86241E+00_JPRB, 0.87905E+00_JPRB, 0.87977E+00_JPRB, 0.87648E+00_JPRB, 0.86138E+00_JPRB ,&
&  0.83032E+00_JPRB, 0.80465E+00_JPRB, 0.80017E+00_JPRB, 0.80390E+00_JPRB, 0.81566E+00_JPRB, 0.83441E+00_JPRB, 0.83651E+00_JPRB ,&
&  0.81083E+00_JPRB, 0.79207E+00_JPRB, 0.78287E+00_JPRB, 0.77649E+00_JPRB, 0.76028E+00_JPRB, 0.73625E+00_JPRB, 0.72071E+00_JPRB ,&
&  0.71651E+00_JPRB, 0.71683E+00_JPRB, 0.70807E+00_JPRB, 0.69408E+00_JPRB, 0.66262E+00_JPRB, 0.62392E+00_JPRB, 0.59383E+00_JPRB ,&
&  0.58413E+00_JPRB, 0.57636E+00_JPRB, 0.53519E+00_JPRB, 0.55318E+00_JPRB, 0.55673E+00_JPRB, 0.55700E+00_JPRB, 0.55497E+00_JPRB ,&
&  0.56810E+00_JPRB, 0.60170E+00_JPRB, 0.63467E+00_JPRB, 0.64450E+00_JPRB, 0.65411E+00_JPRB, 0.65594E+00_JPRB, 0.65763E+00_JPRB ,&
&  0.66237E+00_JPRB, 0.65439E+00_JPRB, 0.62476E+00_JPRB, 0.58092E+00_JPRB, 0.53767E+00_JPRB, 0.50401E+00_JPRB, 0.47528E+00_JPRB ,&
&  0.45534E+00_JPRB, 0.44288E+00_JPRB, 0.43320E+00_JPRB, 0.42976E+00_JPRB, 0.43717E+00_JPRB, 0.45194E+00_JPRB, 0.44689E+00_JPRB ,&
&  0.42906E+00_JPRB, 0.41292E+00_JPRB, 0.39131E+00_JPRB, 0.36193E+00_JPRB, 0.33237E+00_JPRB, 0.29320E+00_JPRB, 0.27343E+00_JPRB ,&
&  0.27776E+00_JPRB, 0.31672E+00_JPRB, 0.36826E+00_JPRB, 0.38680E+00_JPRB, 0.41359E+00_JPRB, 0.43955E+00_JPRB, 0.42904E+00_JPRB ,&
&  0.38315E+00_JPRB, 0.31297E+00_JPRB, 0.29004E+00_JPRB, 0.27810E+00_JPRB, 0.27129E+00_JPRB, 0.26623E+00_JPRB, 0.26557E+00_JPRB ,&
&  0.26962E+00_JPRB, 0.28104E+00_JPRB, 0.29015E+00_JPRB, 0.28673E+00_JPRB, 0.27628E+00_JPRB, 0.26947E+00_JPRB, 0.26975E+00_JPRB ,&
&  0.27806E+00_JPRB, 0.30557E+00_JPRB, 0.36921E+00_JPRB, 0.48631E+00_JPRB, 0.52083E+00_JPRB, 0.55306E+00_JPRB, 0.59062E+00_JPRB ,&
&  0.62661E+00_JPRB, 0.65933E+00_JPRB, 0.68744E+00_JPRB, 0.72224E+00_JPRB, 0.76887E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.80202E+00_JPRB, 0.82319E+00_JPRB, 0.84166E+00_JPRB, 0.85417E+00_JPRB, 0.85571E+00_JPRB, 0.84881E+00_JPRB, 0.83302E+00_JPRB ,&
&  0.80319E+00_JPRB, 0.78628E+00_JPRB, 0.79533E+00_JPRB, 0.81004E+00_JPRB, 0.81904E+00_JPRB, 0.81898E+00_JPRB, 0.78761E+00_JPRB ,&
&  0.73392E+00_JPRB, 0.71371E+00_JPRB, 0.71086E+00_JPRB, 0.72067E+00_JPRB, 0.73935E+00_JPRB, 0.74697E+00_JPRB, 0.73559E+00_JPRB ,&
&  0.72383E+00_JPRB, 0.72589E+00_JPRB, 0.72899E+00_JPRB, 0.73600E+00_JPRB, 0.72520E+00_JPRB, 0.70432E+00_JPRB, 0.69558E+00_JPRB ,&
&  0.69336E+00_JPRB, 0.68651E+00_JPRB, 0.67938E+00_JPRB, 0.63045E+00_JPRB, 0.65602E+00_JPRB, 0.64462E+00_JPRB, 0.62673E+00_JPRB ,&
&  0.61061E+00_JPRB, 0.61357E+00_JPRB, 0.65507E+00_JPRB, 0.70119E+00_JPRB, 0.71252E+00_JPRB, 0.71533E+00_JPRB, 0.71086E+00_JPRB ,&
&  0.70773E+00_JPRB, 0.70811E+00_JPRB, 0.70944E+00_JPRB, 0.70122E+00_JPRB, 0.68077E+00_JPRB, 0.65448E+00_JPRB, 0.62555E+00_JPRB ,&
&  0.60786E+00_JPRB, 0.58264E+00_JPRB, 0.53897E+00_JPRB, 0.51863E+00_JPRB, 0.51653E+00_JPRB, 0.52432E+00_JPRB, 0.52813E+00_JPRB ,&
&  0.52508E+00_JPRB, 0.51785E+00_JPRB, 0.52237E+00_JPRB, 0.51991E+00_JPRB, 0.49841E+00_JPRB, 0.48569E+00_JPRB, 0.47988E+00_JPRB ,&
&  0.46825E+00_JPRB, 0.46664E+00_JPRB, 0.47085E+00_JPRB, 0.48200E+00_JPRB, 0.49182E+00_JPRB, 0.50502E+00_JPRB, 0.51014E+00_JPRB ,&
&  0.51210E+00_JPRB, 0.50899E+00_JPRB, 0.50354E+00_JPRB, 0.49186E+00_JPRB, 0.47375E+00_JPRB, 0.46689E+00_JPRB, 0.46828E+00_JPRB ,&
&  0.47683E+00_JPRB, 0.48231E+00_JPRB, 0.48130E+00_JPRB, 0.47990E+00_JPRB, 0.46827E+00_JPRB, 0.45558E+00_JPRB, 0.44566E+00_JPRB ,&
&  0.44078E+00_JPRB, 0.44297E+00_JPRB, 0.45738E+00_JPRB, 0.48146E+00_JPRB, 0.51271E+00_JPRB, 0.55052E+00_JPRB, 0.59489E+00_JPRB ,&
&  0.62662E+00_JPRB, 0.64562E+00_JPRB, 0.67002E+00_JPRB, 0.71713E+00_JPRB, 0.76218E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.76403E+00_JPRB, 0.77211E+00_JPRB, 0.78179E+00_JPRB, 0.79471E+00_JPRB, 0.80625E+00_JPRB, 0.80276E+00_JPRB, 0.79029E+00_JPRB ,&
&  0.76670E+00_JPRB, 0.75818E+00_JPRB, 0.76313E+00_JPRB, 0.76561E+00_JPRB, 0.76733E+00_JPRB, 0.76497E+00_JPRB, 0.74895E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.68306E+00_JPRB, 0.66901E+00_JPRB, 0.66177E+00_JPRB, 0.65953E+00_JPRB, 0.66684E+00_JPRB, 0.67593E+00_JPRB ,&
&  0.69373E+00_JPRB, 0.70469E+00_JPRB, 0.71725E+00_JPRB, 0.72689E+00_JPRB, 0.72966E+00_JPRB, 0.73370E+00_JPRB, 0.73509E+00_JPRB ,&
&  0.72146E+00_JPRB, 0.72213E+00_JPRB, 0.72421E+00_JPRB, 0.71509E+00_JPRB, 0.69441E+00_JPRB, 0.67361E+00_JPRB, 0.64780E+00_JPRB ,&
&  0.63086E+00_JPRB, 0.62250E+00_JPRB, 0.64807E+00_JPRB, 0.70950E+00_JPRB, 0.75244E+00_JPRB, 0.76048E+00_JPRB, 0.75294E+00_JPRB ,&
&  0.75531E+00_JPRB, 0.76283E+00_JPRB, 0.76630E+00_JPRB, 0.75464E+00_JPRB, 0.73480E+00_JPRB, 0.71738E+00_JPRB, 0.70054E+00_JPRB ,&
&  0.69449E+00_JPRB, 0.69318E+00_JPRB, 0.67525E+00_JPRB, 0.64280E+00_JPRB, 0.61525E+00_JPRB, 0.59584E+00_JPRB, 0.58573E+00_JPRB ,&
&  0.57988E+00_JPRB, 0.57075E+00_JPRB, 0.56537E+00_JPRB, 0.57067E+00_JPRB, 0.55109E+00_JPRB, 0.53320E+00_JPRB, 0.53702E+00_JPRB ,&
&  0.55338E+00_JPRB, 0.55857E+00_JPRB, 0.56735E+00_JPRB, 0.56985E+00_JPRB, 0.56609E+00_JPRB, 0.56169E+00_JPRB, 0.56234E+00_JPRB ,&
&  0.57043E+00_JPRB, 0.57971E+00_JPRB, 0.58648E+00_JPRB, 0.59130E+00_JPRB, 0.59629E+00_JPRB, 0.58475E+00_JPRB, 0.56377E+00_JPRB ,&
&  0.54596E+00_JPRB, 0.54229E+00_JPRB, 0.54648E+00_JPRB, 0.54560E+00_JPRB, 0.54384E+00_JPRB, 0.54377E+00_JPRB, 0.54207E+00_JPRB ,&
&  0.53889E+00_JPRB, 0.53112E+00_JPRB, 0.52205E+00_JPRB, 0.52240E+00_JPRB, 0.53388E+00_JPRB, 0.55606E+00_JPRB, 0.59064E+00_JPRB ,&
&  0.64254E+00_JPRB, 0.68200E+00_JPRB, 0.71947E+00_JPRB, 0.73418E+00_JPRB, 0.73698E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.73361E+00_JPRB, 0.74164E+00_JPRB, 0.74732E+00_JPRB, 0.75130E+00_JPRB, 0.75942E+00_JPRB, 0.76965E+00_JPRB, 0.76920E+00_JPRB ,&
&  0.75452E+00_JPRB, 0.74106E+00_JPRB, 0.73182E+00_JPRB, 0.72928E+00_JPRB, 0.73095E+00_JPRB, 0.73311E+00_JPRB, 0.72744E+00_JPRB ,&
&  0.70722E+00_JPRB, 0.68547E+00_JPRB, 0.67213E+00_JPRB, 0.65883E+00_JPRB, 0.65075E+00_JPRB, 0.64574E+00_JPRB, 0.65301E+00_JPRB ,&
&  0.66369E+00_JPRB, 0.68028E+00_JPRB, 0.69392E+00_JPRB, 0.70394E+00_JPRB, 0.70825E+00_JPRB, 0.69740E+00_JPRB, 0.69920E+00_JPRB ,&
&  0.70686E+00_JPRB, 0.70953E+00_JPRB, 0.72513E+00_JPRB, 0.72504E+00_JPRB, 0.71234E+00_JPRB, 0.70230E+00_JPRB, 0.69216E+00_JPRB ,&
&  0.68952E+00_JPRB, 0.70088E+00_JPRB, 0.72628E+00_JPRB, 0.76259E+00_JPRB, 0.78315E+00_JPRB, 0.79073E+00_JPRB, 0.78789E+00_JPRB ,&
&  0.79098E+00_JPRB, 0.80298E+00_JPRB, 0.80961E+00_JPRB, 0.80121E+00_JPRB, 0.78273E+00_JPRB, 0.76745E+00_JPRB, 0.75387E+00_JPRB ,&
&  0.74362E+00_JPRB, 0.73138E+00_JPRB, 0.72318E+00_JPRB, 0.71315E+00_JPRB, 0.69156E+00_JPRB, 0.66592E+00_JPRB, 0.64367E+00_JPRB ,&
&  0.62738E+00_JPRB, 0.61140E+00_JPRB, 0.59184E+00_JPRB, 0.58601E+00_JPRB, 0.59046E+00_JPRB, 0.56751E+00_JPRB, 0.54254E+00_JPRB ,&
&  0.54602E+00_JPRB, 0.57689E+00_JPRB, 0.60082E+00_JPRB, 0.61006E+00_JPRB, 0.61026E+00_JPRB, 0.61404E+00_JPRB, 0.61693E+00_JPRB ,&
&  0.62281E+00_JPRB, 0.63087E+00_JPRB, 0.63700E+00_JPRB, 0.64773E+00_JPRB, 0.65900E+00_JPRB, 0.65755E+00_JPRB, 0.64375E+00_JPRB ,&
&  0.62834E+00_JPRB, 0.61930E+00_JPRB, 0.61511E+00_JPRB, 0.61693E+00_JPRB, 0.62006E+00_JPRB, 0.61779E+00_JPRB, 0.61185E+00_JPRB ,&
&  0.60024E+00_JPRB, 0.58172E+00_JPRB, 0.56599E+00_JPRB, 0.55764E+00_JPRB, 0.55763E+00_JPRB, 0.56451E+00_JPRB, 0.58068E+00_JPRB ,&
&  0.61187E+00_JPRB, 0.65941E+00_JPRB, 0.69403E+00_JPRB, 0.71281E+00_JPRB, 0.71693E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.70601E+00_JPRB, 0.71211E+00_JPRB, 0.72083E+00_JPRB, 0.73349E+00_JPRB, 0.74658E+00_JPRB, 0.75486E+00_JPRB, 0.75400E+00_JPRB ,&
&  0.74301E+00_JPRB, 0.72996E+00_JPRB, 0.72585E+00_JPRB, 0.72551E+00_JPRB, 0.72281E+00_JPRB, 0.72166E+00_JPRB, 0.71754E+00_JPRB ,&
&  0.70718E+00_JPRB, 0.69101E+00_JPRB, 0.68356E+00_JPRB, 0.67022E+00_JPRB, 0.65763E+00_JPRB, 0.64747E+00_JPRB, 0.64806E+00_JPRB ,&
&  0.65720E+00_JPRB, 0.66760E+00_JPRB, 0.68040E+00_JPRB, 0.69519E+00_JPRB, 0.70373E+00_JPRB, 0.70316E+00_JPRB, 0.70065E+00_JPRB ,&
&  0.69875E+00_JPRB, 0.69645E+00_JPRB, 0.70814E+00_JPRB, 0.71762E+00_JPRB, 0.72397E+00_JPRB, 0.73530E+00_JPRB, 0.74435E+00_JPRB ,&
&  0.75167E+00_JPRB, 0.76247E+00_JPRB, 0.78063E+00_JPRB, 0.79400E+00_JPRB, 0.80333E+00_JPRB, 0.81350E+00_JPRB, 0.81743E+00_JPRB ,&
&  0.82385E+00_JPRB, 0.83492E+00_JPRB, 0.83711E+00_JPRB, 0.82791E+00_JPRB, 0.81327E+00_JPRB, 0.80360E+00_JPRB, 0.79789E+00_JPRB ,&
&  0.78821E+00_JPRB, 0.76886E+00_JPRB, 0.75444E+00_JPRB, 0.74316E+00_JPRB, 0.72480E+00_JPRB, 0.70505E+00_JPRB, 0.68177E+00_JPRB ,&
&  0.65515E+00_JPRB, 0.63531E+00_JPRB, 0.61992E+00_JPRB, 0.60608E+00_JPRB, 0.60297E+00_JPRB, 0.59464E+00_JPRB, 0.58803E+00_JPRB ,&
&  0.59257E+00_JPRB, 0.61051E+00_JPRB, 0.62371E+00_JPRB, 0.62779E+00_JPRB, 0.62970E+00_JPRB, 0.64486E+00_JPRB, 0.66338E+00_JPRB ,&
&  0.67768E+00_JPRB, 0.68334E+00_JPRB, 0.67882E+00_JPRB, 0.68064E+00_JPRB, 0.68543E+00_JPRB, 0.68968E+00_JPRB, 0.68390E+00_JPRB ,&
&  0.67550E+00_JPRB, 0.66993E+00_JPRB, 0.66873E+00_JPRB, 0.66519E+00_JPRB, 0.66672E+00_JPRB, 0.66661E+00_JPRB, 0.66379E+00_JPRB ,&
&  0.65800E+00_JPRB, 0.64845E+00_JPRB, 0.63634E+00_JPRB, 0.62947E+00_JPRB, 0.61949E+00_JPRB, 0.60449E+00_JPRB, 0.60587E+00_JPRB ,&
&  0.61739E+00_JPRB, 0.63576E+00_JPRB, 0.65283E+00_JPRB, 0.67265E+00_JPRB, 0.69553E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.68417E+00_JPRB, 0.69617E+00_JPRB, 0.71106E+00_JPRB, 0.72626E+00_JPRB, 0.73554E+00_JPRB, 0.74114E+00_JPRB, 0.73873E+00_JPRB ,&
&  0.72806E+00_JPRB, 0.71747E+00_JPRB, 0.71917E+00_JPRB, 0.71920E+00_JPRB, 0.71852E+00_JPRB, 0.71724E+00_JPRB, 0.71121E+00_JPRB ,&
&  0.70252E+00_JPRB, 0.69529E+00_JPRB, 0.69245E+00_JPRB, 0.68988E+00_JPRB, 0.68045E+00_JPRB, 0.67330E+00_JPRB, 0.66565E+00_JPRB ,&
&  0.66609E+00_JPRB, 0.66936E+00_JPRB, 0.67304E+00_JPRB, 0.67423E+00_JPRB, 0.68318E+00_JPRB, 0.69434E+00_JPRB, 0.69942E+00_JPRB ,&
&  0.69467E+00_JPRB, 0.70072E+00_JPRB, 0.61991E+00_JPRB, 0.68985E+00_JPRB, 0.71546E+00_JPRB, 0.73054E+00_JPRB, 0.74513E+00_JPRB ,&
&  0.75435E+00_JPRB, 0.76492E+00_JPRB, 0.77201E+00_JPRB, 0.77547E+00_JPRB, 0.79155E+00_JPRB, 0.80572E+00_JPRB, 0.80973E+00_JPRB ,&
&  0.81093E+00_JPRB, 0.81012E+00_JPRB, 0.81130E+00_JPRB, 0.81080E+00_JPRB, 0.81458E+00_JPRB, 0.81377E+00_JPRB, 0.80744E+00_JPRB ,&
&  0.79929E+00_JPRB, 0.78791E+00_JPRB, 0.77119E+00_JPRB, 0.75206E+00_JPRB, 0.72151E+00_JPRB, 0.69487E+00_JPRB, 0.67794E+00_JPRB ,&
&  0.66365E+00_JPRB, 0.65926E+00_JPRB, 0.65381E+00_JPRB, 0.64798E+00_JPRB, 0.64559E+00_JPRB, 0.64776E+00_JPRB, 0.64651E+00_JPRB ,&
&  0.65314E+00_JPRB, 0.66195E+00_JPRB, 0.66635E+00_JPRB, 0.67354E+00_JPRB, 0.68274E+00_JPRB, 0.69630E+00_JPRB, 0.70898E+00_JPRB ,&
&  0.72082E+00_JPRB, 0.72868E+00_JPRB, 0.72942E+00_JPRB, 0.72913E+00_JPRB, 0.72684E+00_JPRB, 0.72589E+00_JPRB, 0.71899E+00_JPRB ,&
&  0.71345E+00_JPRB, 0.71057E+00_JPRB, 0.70447E+00_JPRB, 0.69291E+00_JPRB, 0.68613E+00_JPRB, 0.68043E+00_JPRB, 0.67918E+00_JPRB ,&
&  0.68036E+00_JPRB, 0.68151E+00_JPRB, 0.67892E+00_JPRB, 0.67978E+00_JPRB, 0.67373E+00_JPRB, 0.65400E+00_JPRB, 0.63862E+00_JPRB ,&
&  0.63806E+00_JPRB, 0.64345E+00_JPRB, 0.64850E+00_JPRB, 0.65777E+00_JPRB, 0.67233E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.72071E+00_JPRB, 0.72576E+00_JPRB, 0.73407E+00_JPRB, 0.74366E+00_JPRB, 0.74637E+00_JPRB, 0.74546E+00_JPRB, 0.74304E+00_JPRB ,&
&  0.74381E+00_JPRB, 0.73585E+00_JPRB, 0.73009E+00_JPRB, 0.72691E+00_JPRB, 0.72672E+00_JPRB, 0.72436E+00_JPRB, 0.71646E+00_JPRB ,&
&  0.71007E+00_JPRB, 0.70830E+00_JPRB, 0.71376E+00_JPRB, 0.71599E+00_JPRB, 0.70157E+00_JPRB, 0.69721E+00_JPRB, 0.68997E+00_JPRB ,&
&  0.68922E+00_JPRB, 0.69015E+00_JPRB, 0.69391E+00_JPRB, 0.70098E+00_JPRB, 0.71822E+00_JPRB, 0.73067E+00_JPRB, 0.73762E+00_JPRB ,&
&  0.73730E+00_JPRB, 0.65703E+00_JPRB, 0.68457E+00_JPRB, 0.69770E+00_JPRB, 0.72071E+00_JPRB, 0.74299E+00_JPRB, 0.76148E+00_JPRB ,&
&  0.78530E+00_JPRB, 0.79750E+00_JPRB, 0.80145E+00_JPRB, 0.80123E+00_JPRB, 0.80249E+00_JPRB, 0.80282E+00_JPRB, 0.79667E+00_JPRB ,&
&  0.79227E+00_JPRB, 0.79893E+00_JPRB, 0.81457E+00_JPRB, 0.82199E+00_JPRB, 0.82565E+00_JPRB, 0.82760E+00_JPRB, 0.82422E+00_JPRB ,&
&  0.81747E+00_JPRB, 0.80382E+00_JPRB, 0.78108E+00_JPRB, 0.75394E+00_JPRB, 0.72321E+00_JPRB, 0.70240E+00_JPRB, 0.69090E+00_JPRB ,&
&  0.68300E+00_JPRB, 0.68263E+00_JPRB, 0.68228E+00_JPRB, 0.68241E+00_JPRB, 0.68477E+00_JPRB, 0.69238E+00_JPRB, 0.69515E+00_JPRB ,&
&  0.70256E+00_JPRB, 0.71019E+00_JPRB, 0.72242E+00_JPRB, 0.74209E+00_JPRB, 0.76197E+00_JPRB, 0.77326E+00_JPRB, 0.77409E+00_JPRB ,&
&  0.77959E+00_JPRB, 0.78195E+00_JPRB, 0.77801E+00_JPRB, 0.77784E+00_JPRB, 0.77413E+00_JPRB, 0.77543E+00_JPRB, 0.77263E+00_JPRB ,&
&  0.76413E+00_JPRB, 0.75805E+00_JPRB, 0.75081E+00_JPRB, 0.73634E+00_JPRB, 0.72404E+00_JPRB, 0.71536E+00_JPRB, 0.70819E+00_JPRB ,&
&  0.70782E+00_JPRB, 0.71139E+00_JPRB, 0.71659E+00_JPRB, 0.71175E+00_JPRB, 0.69782E+00_JPRB, 0.68965E+00_JPRB, 0.67994E+00_JPRB ,&
&  0.67344E+00_JPRB, 0.67603E+00_JPRB, 0.67678E+00_JPRB, 0.68486E+00_JPRB, 0.70390E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.74835E+00_JPRB, 0.75151E+00_JPRB, 0.75587E+00_JPRB, 0.75623E+00_JPRB, 0.75233E+00_JPRB, 0.75018E+00_JPRB, 0.75213E+00_JPRB ,&
&  0.75354E+00_JPRB, 0.74587E+00_JPRB, 0.73815E+00_JPRB, 0.73275E+00_JPRB, 0.73147E+00_JPRB, 0.72475E+00_JPRB, 0.71418E+00_JPRB ,&
&  0.70705E+00_JPRB, 0.70770E+00_JPRB, 0.71452E+00_JPRB, 0.71880E+00_JPRB, 0.71149E+00_JPRB, 0.70338E+00_JPRB, 0.70112E+00_JPRB ,&
&  0.70278E+00_JPRB, 0.70952E+00_JPRB, 0.72050E+00_JPRB, 0.72758E+00_JPRB, 0.73733E+00_JPRB, 0.75244E+00_JPRB, 0.75481E+00_JPRB ,&
&  0.75472E+00_JPRB, 0.61978E+00_JPRB, 0.65653E+00_JPRB, 0.67124E+00_JPRB, 0.69987E+00_JPRB, 0.73991E+00_JPRB, 0.77153E+00_JPRB ,&
&  0.80064E+00_JPRB, 0.81687E+00_JPRB, 0.82424E+00_JPRB, 0.81775E+00_JPRB, 0.80337E+00_JPRB, 0.79051E+00_JPRB, 0.78665E+00_JPRB ,&
&  0.78468E+00_JPRB, 0.79120E+00_JPRB, 0.80558E+00_JPRB, 0.81591E+00_JPRB, 0.81992E+00_JPRB, 0.82135E+00_JPRB, 0.82422E+00_JPRB ,&
&  0.82168E+00_JPRB, 0.80674E+00_JPRB, 0.78557E+00_JPRB, 0.76006E+00_JPRB, 0.74072E+00_JPRB, 0.73268E+00_JPRB, 0.72116E+00_JPRB ,&
&  0.70473E+00_JPRB, 0.69395E+00_JPRB, 0.69104E+00_JPRB, 0.69291E+00_JPRB, 0.70095E+00_JPRB, 0.71017E+00_JPRB, 0.71838E+00_JPRB ,&
&  0.72798E+00_JPRB, 0.74324E+00_JPRB, 0.75952E+00_JPRB, 0.77534E+00_JPRB, 0.79224E+00_JPRB, 0.80845E+00_JPRB, 0.82135E+00_JPRB ,&
&  0.83010E+00_JPRB, 0.82884E+00_JPRB, 0.81751E+00_JPRB, 0.80841E+00_JPRB, 0.80451E+00_JPRB, 0.80832E+00_JPRB, 0.80856E+00_JPRB ,&
&  0.80067E+00_JPRB, 0.79449E+00_JPRB, 0.79230E+00_JPRB, 0.78284E+00_JPRB, 0.77209E+00_JPRB, 0.76261E+00_JPRB, 0.75275E+00_JPRB ,&
&  0.74496E+00_JPRB, 0.74203E+00_JPRB, 0.73946E+00_JPRB, 0.73398E+00_JPRB, 0.72526E+00_JPRB, 0.72207E+00_JPRB, 0.71977E+00_JPRB ,&
&  0.71730E+00_JPRB, 0.71349E+00_JPRB, 0.71482E+00_JPRB, 0.72462E+00_JPRB, 0.73874E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.75943E+00_JPRB, 0.76613E+00_JPRB, 0.77037E+00_JPRB, 0.77506E+00_JPRB, 0.77567E+00_JPRB, 0.77422E+00_JPRB, 0.77825E+00_JPRB ,&
&  0.78263E+00_JPRB, 0.77807E+00_JPRB, 0.77058E+00_JPRB, 0.76879E+00_JPRB, 0.76112E+00_JPRB, 0.74950E+00_JPRB, 0.74097E+00_JPRB ,&
&  0.72986E+00_JPRB, 0.72694E+00_JPRB, 0.72569E+00_JPRB, 0.72411E+00_JPRB, 0.71906E+00_JPRB, 0.71219E+00_JPRB, 0.71672E+00_JPRB ,&
&  0.72400E+00_JPRB, 0.73159E+00_JPRB, 0.73433E+00_JPRB, 0.73945E+00_JPRB, 0.74662E+00_JPRB, 0.75466E+00_JPRB, 0.76867E+00_JPRB ,&
&  0.66105E+00_JPRB, 0.53778E+00_JPRB, 0.58737E+00_JPRB, 0.63745E+00_JPRB, 0.67781E+00_JPRB, 0.73846E+00_JPRB, 0.78086E+00_JPRB ,&
&  0.81283E+00_JPRB, 0.84193E+00_JPRB, 0.84800E+00_JPRB, 0.82822E+00_JPRB, 0.80768E+00_JPRB, 0.80022E+00_JPRB, 0.79918E+00_JPRB ,&
&  0.79705E+00_JPRB, 0.79886E+00_JPRB, 0.80565E+00_JPRB, 0.81232E+00_JPRB, 0.81861E+00_JPRB, 0.82587E+00_JPRB, 0.82874E+00_JPRB ,&
&  0.82716E+00_JPRB, 0.81561E+00_JPRB, 0.80416E+00_JPRB, 0.78631E+00_JPRB, 0.77099E+00_JPRB, 0.76713E+00_JPRB, 0.75855E+00_JPRB ,&
&  0.74349E+00_JPRB, 0.73299E+00_JPRB, 0.73105E+00_JPRB, 0.73478E+00_JPRB, 0.74448E+00_JPRB, 0.75568E+00_JPRB, 0.77389E+00_JPRB ,&
&  0.79422E+00_JPRB, 0.80882E+00_JPRB, 0.81874E+00_JPRB, 0.83099E+00_JPRB, 0.83790E+00_JPRB, 0.84716E+00_JPRB, 0.86423E+00_JPRB ,&
&  0.87849E+00_JPRB, 0.86979E+00_JPRB, 0.86135E+00_JPRB, 0.85228E+00_JPRB, 0.84869E+00_JPRB, 0.84575E+00_JPRB, 0.83663E+00_JPRB ,&
&  0.82814E+00_JPRB, 0.82345E+00_JPRB, 0.82155E+00_JPRB, 0.82578E+00_JPRB, 0.82426E+00_JPRB, 0.81948E+00_JPRB, 0.81816E+00_JPRB ,&
&  0.81789E+00_JPRB, 0.81139E+00_JPRB, 0.80323E+00_JPRB, 0.79997E+00_JPRB, 0.79243E+00_JPRB, 0.78791E+00_JPRB, 0.78237E+00_JPRB ,&
&  0.77720E+00_JPRB, 0.76168E+00_JPRB, 0.76523E+00_JPRB, 0.75848E+00_JPRB, 0.75497E+00_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.74936E+00_JPRB, 0.77287E+00_JPRB, 0.77951E+00_JPRB, 0.79262E+00_JPRB, 0.80056E+00_JPRB, 0.80532E+00_JPRB, 0.81322E+00_JPRB ,&
&  0.81844E+00_JPRB, 0.81482E+00_JPRB, 0.80900E+00_JPRB, 0.79706E+00_JPRB, 0.77693E+00_JPRB, 0.75522E+00_JPRB, 0.73814E+00_JPRB ,&
&  0.72729E+00_JPRB, 0.72882E+00_JPRB, 0.72441E+00_JPRB, 0.71749E+00_JPRB, 0.71359E+00_JPRB, 0.71192E+00_JPRB, 0.71537E+00_JPRB ,&
&  0.72432E+00_JPRB, 0.72931E+00_JPRB, 0.72536E+00_JPRB, 0.72538E+00_JPRB, 0.73538E+00_JPRB, 0.75276E+00_JPRB, 0.76163E+00_JPRB ,&
&  0.76539E+00_JPRB, 0.57243E+00_JPRB, 0.60925E+00_JPRB, 0.65111E+00_JPRB, 0.68084E+00_JPRB, 0.72783E+00_JPRB, 0.78206E+00_JPRB ,&
&  0.81697E+00_JPRB, 0.84027E+00_JPRB, 0.84477E+00_JPRB, 0.83950E+00_JPRB, 0.82430E+00_JPRB, 0.81063E+00_JPRB, 0.79597E+00_JPRB ,&
&  0.78446E+00_JPRB, 0.78389E+00_JPRB, 0.79130E+00_JPRB, 0.79863E+00_JPRB, 0.80321E+00_JPRB, 0.80913E+00_JPRB, 0.80525E+00_JPRB ,&
&  0.80429E+00_JPRB, 0.80124E+00_JPRB, 0.79895E+00_JPRB, 0.79832E+00_JPRB, 0.79624E+00_JPRB, 0.79420E+00_JPRB, 0.78686E+00_JPRB ,&
&  0.77120E+00_JPRB, 0.76016E+00_JPRB, 0.75467E+00_JPRB, 0.75223E+00_JPRB, 0.75904E+00_JPRB, 0.77275E+00_JPRB, 0.79252E+00_JPRB ,&
&  0.81316E+00_JPRB, 0.81709E+00_JPRB, 0.82286E+00_JPRB, 0.83018E+00_JPRB, 0.84136E+00_JPRB, 0.85249E+00_JPRB, 0.86662E+00_JPRB ,&
&  0.87080E+00_JPRB, 0.86356E+00_JPRB, 0.85749E+00_JPRB, 0.84680E+00_JPRB, 0.84599E+00_JPRB, 0.83273E+00_JPRB, 0.81642E+00_JPRB ,&
&  0.81509E+00_JPRB, 0.81616E+00_JPRB, 0.81790E+00_JPRB, 0.82705E+00_JPRB, 0.83386E+00_JPRB, 0.83458E+00_JPRB, 0.84254E+00_JPRB ,&
&  0.85480E+00_JPRB, 0.86415E+00_JPRB, 0.86374E+00_JPRB, 0.86682E+00_JPRB, 0.85977E+00_JPRB, 0.85303E+00_JPRB, 0.83410E+00_JPRB ,&
&  0.80942E+00_JPRB, 0.79172E+00_JPRB, 0.61389E+00_JPRB, 0.69517E+00_JPRB, 0.71910E+00_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.77614E+00_JPRB, 0.81484E+00_JPRB, 0.83453E+00_JPRB, 0.83836E+00_JPRB, 0.84309E+00_JPRB, 0.84314E+00_JPRB, 0.83798E+00_JPRB ,&
&  0.83528E+00_JPRB, 0.83040E+00_JPRB, 0.81850E+00_JPRB, 0.80075E+00_JPRB, 0.78311E+00_JPRB, 0.76457E+00_JPRB, 0.75175E+00_JPRB ,&
&  0.75153E+00_JPRB, 0.75368E+00_JPRB, 0.74612E+00_JPRB, 0.73877E+00_JPRB, 0.73095E+00_JPRB, 0.72464E+00_JPRB, 0.72324E+00_JPRB ,&
&  0.72046E+00_JPRB, 0.72021E+00_JPRB, 0.72698E+00_JPRB, 0.73560E+00_JPRB, 0.74626E+00_JPRB, 0.75777E+00_JPRB, 0.77293E+00_JPRB ,&
&  0.77419E+00_JPRB, 0.58632E+00_JPRB, 0.60695E+00_JPRB, 0.67145E+00_JPRB, 0.70686E+00_JPRB, 0.75513E+00_JPRB, 0.80494E+00_JPRB ,&
&  0.82874E+00_JPRB, 0.85804E+00_JPRB, 0.86032E+00_JPRB, 0.85473E+00_JPRB, 0.86040E+00_JPRB, 0.85108E+00_JPRB, 0.82125E+00_JPRB ,&
&  0.79875E+00_JPRB, 0.79972E+00_JPRB, 0.81413E+00_JPRB, 0.82561E+00_JPRB, 0.82834E+00_JPRB, 0.82490E+00_JPRB, 0.82249E+00_JPRB ,&
&  0.81782E+00_JPRB, 0.81138E+00_JPRB, 0.80304E+00_JPRB, 0.79156E+00_JPRB, 0.79311E+00_JPRB, 0.80448E+00_JPRB, 0.80952E+00_JPRB ,&
&  0.80637E+00_JPRB, 0.79703E+00_JPRB, 0.78980E+00_JPRB, 0.78667E+00_JPRB, 0.79156E+00_JPRB, 0.80957E+00_JPRB, 0.82861E+00_JPRB ,&
&  0.83818E+00_JPRB, 0.84222E+00_JPRB, 0.85011E+00_JPRB, 0.85082E+00_JPRB, 0.85533E+00_JPRB, 0.86059E+00_JPRB, 0.87024E+00_JPRB ,&
&  0.87387E+00_JPRB, 0.86623E+00_JPRB, 0.85729E+00_JPRB, 0.84506E+00_JPRB, 0.82914E+00_JPRB, 0.81998E+00_JPRB, 0.81796E+00_JPRB ,&
&  0.81530E+00_JPRB, 0.82173E+00_JPRB, 0.83770E+00_JPRB, 0.85747E+00_JPRB, 0.86579E+00_JPRB, 0.86867E+00_JPRB, 0.87291E+00_JPRB ,&
&  0.88249E+00_JPRB, 0.88821E+00_JPRB, 0.89637E+00_JPRB, 0.85043E+00_JPRB, 0.89162E+00_JPRB, 0.89302E+00_JPRB, 0.87520E+00_JPRB ,&
&  0.85251E+00_JPRB, 0.82400E+00_JPRB, 0.81410E+00_JPRB, 0.67972E+00_JPRB, 0.74352E+00_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.83272E+00_JPRB, 0.86790E+00_JPRB, 0.86804E+00_JPRB, 0.86634E+00_JPRB, 0.86165E+00_JPRB, 0.84471E+00_JPRB, 0.82420E+00_JPRB ,&
&  0.80411E+00_JPRB, 0.79435E+00_JPRB, 0.77990E+00_JPRB, 0.76078E+00_JPRB, 0.75022E+00_JPRB, 0.74602E+00_JPRB, 0.74261E+00_JPRB ,&
&  0.74436E+00_JPRB, 0.74554E+00_JPRB, 0.74745E+00_JPRB, 0.74833E+00_JPRB, 0.74393E+00_JPRB, 0.74004E+00_JPRB, 0.73102E+00_JPRB ,&
&  0.72591E+00_JPRB, 0.72784E+00_JPRB, 0.73049E+00_JPRB, 0.73591E+00_JPRB, 0.74400E+00_JPRB, 0.75274E+00_JPRB, 0.75863E+00_JPRB ,&
&  0.75065E+00_JPRB, 0.62960E+00_JPRB, 0.64677E+00_JPRB, 0.69609E+00_JPRB, 0.74864E+00_JPRB, 0.80187E+00_JPRB, 0.82615E+00_JPRB ,&
&  0.84725E+00_JPRB, 0.86144E+00_JPRB, 0.86571E+00_JPRB, 0.86357E+00_JPRB, 0.85570E+00_JPRB, 0.85192E+00_JPRB, 0.83190E+00_JPRB ,&
&  0.82155E+00_JPRB, 0.82198E+00_JPRB, 0.83237E+00_JPRB, 0.83917E+00_JPRB, 0.83922E+00_JPRB, 0.83434E+00_JPRB, 0.83226E+00_JPRB ,&
&  0.82423E+00_JPRB, 0.80126E+00_JPRB, 0.77533E+00_JPRB, 0.75482E+00_JPRB, 0.74756E+00_JPRB, 0.77028E+00_JPRB, 0.79686E+00_JPRB ,&
&  0.81955E+00_JPRB, 0.81473E+00_JPRB, 0.80049E+00_JPRB, 0.79357E+00_JPRB, 0.79619E+00_JPRB, 0.81285E+00_JPRB, 0.83727E+00_JPRB ,&
&  0.84526E+00_JPRB, 0.84671E+00_JPRB, 0.85806E+00_JPRB, 0.85903E+00_JPRB, 0.86014E+00_JPRB, 0.85759E+00_JPRB, 0.85480E+00_JPRB ,&
&  0.84773E+00_JPRB, 0.83988E+00_JPRB, 0.83548E+00_JPRB, 0.82978E+00_JPRB, 0.82878E+00_JPRB, 0.82143E+00_JPRB, 0.81033E+00_JPRB ,&
&  0.79784E+00_JPRB, 0.80798E+00_JPRB, 0.83903E+00_JPRB, 0.86774E+00_JPRB, 0.86774E+00_JPRB, 0.85949E+00_JPRB, 0.85544E+00_JPRB ,&
&  0.86227E+00_JPRB, 0.85787E+00_JPRB, 0.85344E+00_JPRB, 0.89454E+00_JPRB, 0.89168E+00_JPRB, 0.88280E+00_JPRB, 0.87775E+00_JPRB ,&
&  0.87659E+00_JPRB, 0.86312E+00_JPRB, 0.84888E+00_JPRB, 0.83273E+00_JPRB, 0.77461E+00_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.91285E+00_JPRB, 0.90824E+00_JPRB, 0.89800E+00_JPRB, 0.89096E+00_JPRB, 0.87456E+00_JPRB, 0.84486E+00_JPRB, 0.81632E+00_JPRB ,&
&  0.79894E+00_JPRB, 0.77931E+00_JPRB, 0.74955E+00_JPRB, 0.73644E+00_JPRB, 0.73530E+00_JPRB, 0.73866E+00_JPRB, 0.73758E+00_JPRB ,&
&  0.73571E+00_JPRB, 0.74318E+00_JPRB, 0.74224E+00_JPRB, 0.74810E+00_JPRB, 0.74639E+00_JPRB, 0.74968E+00_JPRB, 0.73686E+00_JPRB ,&
&  0.73038E+00_JPRB, 0.72589E+00_JPRB, 0.72559E+00_JPRB, 0.73283E+00_JPRB, 0.73884E+00_JPRB, 0.74327E+00_JPRB, 0.74686E+00_JPRB ,&
&  0.72313E+00_JPRB, 0.61655E+00_JPRB, 0.67093E+00_JPRB, 0.75203E+00_JPRB, 0.80205E+00_JPRB, 0.86176E+00_JPRB, 0.86008E+00_JPRB ,&
&  0.86687E+00_JPRB, 0.86957E+00_JPRB, 0.89038E+00_JPRB, 0.88885E+00_JPRB, 0.86944E+00_JPRB, 0.85534E+00_JPRB, 0.85037E+00_JPRB ,&
&  0.85873E+00_JPRB, 0.86133E+00_JPRB, 0.86659E+00_JPRB, 0.86617E+00_JPRB, 0.86043E+00_JPRB, 0.85272E+00_JPRB, 0.85248E+00_JPRB ,&
&  0.83887E+00_JPRB, 0.80537E+00_JPRB, 0.77132E+00_JPRB, 0.74779E+00_JPRB, 0.75261E+00_JPRB, 0.77614E+00_JPRB, 0.81043E+00_JPRB ,&
&  0.83546E+00_JPRB, 0.83909E+00_JPRB, 0.83329E+00_JPRB, 0.82953E+00_JPRB, 0.83044E+00_JPRB, 0.84489E+00_JPRB, 0.85290E+00_JPRB ,&
&  0.86004E+00_JPRB, 0.85689E+00_JPRB, 0.85538E+00_JPRB, 0.85010E+00_JPRB, 0.84610E+00_JPRB, 0.84529E+00_JPRB, 0.84073E+00_JPRB ,&
&  0.83458E+00_JPRB, 0.83600E+00_JPRB, 0.82709E+00_JPRB, 0.82521E+00_JPRB, 0.81435E+00_JPRB, 0.79769E+00_JPRB, 0.78404E+00_JPRB ,&
&  0.79153E+00_JPRB, 0.81730E+00_JPRB, 0.83588E+00_JPRB, 0.86044E+00_JPRB, 0.86762E+00_JPRB, 0.85808E+00_JPRB, 0.86053E+00_JPRB ,&
&  0.85476E+00_JPRB, 0.84759E+00_JPRB, 0.80453E+00_JPRB, 0.83041E+00_JPRB, 0.92526E+00_JPRB, 0.91772E+00_JPRB, 0.91087E+00_JPRB ,&
&  0.90885E+00_JPRB, 0.89406E+00_JPRB, 0.86038E+00_JPRB, 0.83356E+00_JPRB, 0.89138E+00_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.84392E+00_JPRB, 0.83336E+00_JPRB, 0.84280E+00_JPRB, 0.84757E+00_JPRB, 0.83550E+00_JPRB, 0.80850E+00_JPRB, 0.78778E+00_JPRB ,&
&  0.78278E+00_JPRB, 0.75828E+00_JPRB, 0.73570E+00_JPRB, 0.72166E+00_JPRB, 0.72674E+00_JPRB, 0.73033E+00_JPRB, 0.72912E+00_JPRB ,&
&  0.71957E+00_JPRB, 0.72013E+00_JPRB, 0.71217E+00_JPRB, 0.70945E+00_JPRB, 0.71193E+00_JPRB, 0.71752E+00_JPRB, 0.71564E+00_JPRB ,&
&  0.72491E+00_JPRB, 0.72895E+00_JPRB, 0.73477E+00_JPRB, 0.74255E+00_JPRB, 0.74020E+00_JPRB, 0.74416E+00_JPRB, 0.73410E+00_JPRB ,&
&  0.69893E+00_JPRB, 0.64870E+00_JPRB, 0.74850E+00_JPRB, 0.88837E+00_JPRB, 0.92436E+00_JPRB, 0.93375E+00_JPRB, 0.91548E+00_JPRB ,&
&  0.89688E+00_JPRB, 0.89811E+00_JPRB, 0.92118E+00_JPRB, 0.91546E+00_JPRB, 0.90442E+00_JPRB, 0.90429E+00_JPRB, 0.90231E+00_JPRB ,&
&  0.89718E+00_JPRB, 0.88711E+00_JPRB, 0.88340E+00_JPRB, 0.86878E+00_JPRB, 0.84740E+00_JPRB, 0.83861E+00_JPRB, 0.84025E+00_JPRB ,&
&  0.80093E+00_JPRB, 0.77266E+00_JPRB, 0.74974E+00_JPRB, 0.72868E+00_JPRB, 0.75725E+00_JPRB, 0.81990E+00_JPRB, 0.87121E+00_JPRB ,&
&  0.85647E+00_JPRB, 0.86691E+00_JPRB, 0.86716E+00_JPRB, 0.86483E+00_JPRB, 0.87935E+00_JPRB, 0.86289E+00_JPRB, 0.85641E+00_JPRB ,&
&  0.83903E+00_JPRB, 0.84514E+00_JPRB, 0.85174E+00_JPRB, 0.86300E+00_JPRB, 0.86225E+00_JPRB, 0.85962E+00_JPRB, 0.83156E+00_JPRB ,&
&  0.81746E+00_JPRB, 0.80579E+00_JPRB, 0.79871E+00_JPRB, 0.79290E+00_JPRB, 0.79064E+00_JPRB, 0.77619E+00_JPRB, 0.76545E+00_JPRB ,&
&  0.77508E+00_JPRB, 0.78944E+00_JPRB, 0.83640E+00_JPRB, 0.89650E+00_JPRB, 0.90065E+00_JPRB, 0.87740E+00_JPRB, 0.86125E+00_JPRB ,&
&  0.85079E+00_JPRB, 0.83166E+00_JPRB, 0.80813E+00_JPRB, 0.85178E+00_JPRB, 0.88384E+00_JPRB, 0.95886E+00_JPRB, 0.94171E+00_JPRB ,&
&  0.90780E+00_JPRB, 0.86051E+00_JPRB, 0.83580E+00_JPRB, 0.82411E+00_JPRB, 0.84884E+00_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.80020E+00_JPRB, 0.80123E+00_JPRB, 0.81411E+00_JPRB, 0.82178E+00_JPRB, 0.80948E+00_JPRB, 0.78521E+00_JPRB, 0.76299E+00_JPRB ,&
&  0.75730E+00_JPRB, 0.75760E+00_JPRB, 0.75642E+00_JPRB, 0.73879E+00_JPRB, 0.72332E+00_JPRB, 0.71405E+00_JPRB, 0.71012E+00_JPRB ,&
&  0.71042E+00_JPRB, 0.70182E+00_JPRB, 0.69127E+00_JPRB, 0.68112E+00_JPRB, 0.68767E+00_JPRB, 0.70126E+00_JPRB, 0.72148E+00_JPRB ,&
&  0.72283E+00_JPRB, 0.72450E+00_JPRB, 0.73172E+00_JPRB, 0.74003E+00_JPRB, 0.75042E+00_JPRB, 0.74298E+00_JPRB, 0.72728E+00_JPRB ,&
&  0.69861E+00_JPRB, 0.70584E+00_JPRB, 0.90998E+00_JPRB, 0.10342E+01_JPRB, 0.10321E+01_JPRB, 0.97147E+00_JPRB, 0.93983E+00_JPRB ,&
&  0.91227E+00_JPRB, 0.93686E+00_JPRB, 0.98012E+00_JPRB, 0.97322E+00_JPRB, 0.95220E+00_JPRB, 0.93621E+00_JPRB, 0.92813E+00_JPRB ,&
&  0.90917E+00_JPRB, 0.88316E+00_JPRB, 0.85385E+00_JPRB, 0.84158E+00_JPRB, 0.83180E+00_JPRB, 0.83066E+00_JPRB, 0.80954E+00_JPRB ,&
&  0.78429E+00_JPRB, 0.76264E+00_JPRB, 0.74417E+00_JPRB, 0.72823E+00_JPRB, 0.74147E+00_JPRB, 0.73327E+00_JPRB, 0.81694E+00_JPRB ,&
&  0.98491E+00_JPRB, 0.93260E+00_JPRB, 0.91283E+00_JPRB, 0.91748E+00_JPRB, 0.91890E+00_JPRB, 0.90627E+00_JPRB, 0.89357E+00_JPRB ,&
&  0.89453E+00_JPRB, 0.90839E+00_JPRB, 0.94321E+00_JPRB, 0.94856E+00_JPRB, 0.91410E+00_JPRB, 0.86486E+00_JPRB, 0.84200E+00_JPRB ,&
&  0.83488E+00_JPRB, 0.82637E+00_JPRB, 0.81825E+00_JPRB, 0.80627E+00_JPRB, 0.79514E+00_JPRB, 0.77549E+00_JPRB, 0.76955E+00_JPRB ,&
&  0.77566E+00_JPRB, 0.80539E+00_JPRB, 0.87357E+00_JPRB, 0.90313E+00_JPRB, 0.92970E+00_JPRB, 0.93554E+00_JPRB, 0.88371E+00_JPRB ,&
&  0.82089E+00_JPRB, 0.79100E+00_JPRB, 0.79564E+00_JPRB, 0.82918E+00_JPRB, 0.88815E+00_JPRB, 0.96447E+00_JPRB, 0.92941E+00_JPRB ,&
&  0.87544E+00_JPRB, 0.82502E+00_JPRB, 0.79941E+00_JPRB, 0.78707E+00_JPRB, 0.80872E+00_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.77315E+00_JPRB, 0.76586E+00_JPRB, 0.76766E+00_JPRB, 0.77962E+00_JPRB, 0.76894E+00_JPRB, 0.75267E+00_JPRB, 0.73325E+00_JPRB ,&
&  0.75635E+00_JPRB, 0.77066E+00_JPRB, 0.77800E+00_JPRB, 0.74764E+00_JPRB, 0.72630E+00_JPRB, 0.70145E+00_JPRB, 0.68617E+00_JPRB ,&
&  0.68482E+00_JPRB, 0.67753E+00_JPRB, 0.66804E+00_JPRB, 0.66565E+00_JPRB, 0.67464E+00_JPRB, 0.68608E+00_JPRB, 0.70660E+00_JPRB ,&
&  0.70129E+00_JPRB, 0.70947E+00_JPRB, 0.73454E+00_JPRB, 0.74982E+00_JPRB, 0.76221E+00_JPRB, 0.74472E+00_JPRB, 0.72715E+00_JPRB ,&
&  0.71931E+00_JPRB, 0.72082E+00_JPRB, 0.10391E+01_JPRB, 0.12462E+01_JPRB, 0.10684E+01_JPRB, 0.95297E+00_JPRB, 0.88448E+00_JPRB ,&
&  0.92742E+00_JPRB, 0.97034E+00_JPRB, 0.10114E+01_JPRB, 0.99731E+00_JPRB, 0.98498E+00_JPRB, 0.95318E+00_JPRB, 0.92960E+00_JPRB ,&
&  0.89675E+00_JPRB, 0.87174E+00_JPRB, 0.85571E+00_JPRB, 0.85166E+00_JPRB, 0.83984E+00_JPRB, 0.82110E+00_JPRB, 0.79404E+00_JPRB ,&
&  0.78014E+00_JPRB, 0.76790E+00_JPRB, 0.74933E+00_JPRB, 0.73978E+00_JPRB, 0.76022E+00_JPRB, 0.82845E+00_JPRB, 0.81699E+00_JPRB ,&
&  0.83345E+00_JPRB, 0.99468E+00_JPRB, 0.96967E+00_JPRB, 0.97649E+00_JPRB, 0.96659E+00_JPRB, 0.96194E+00_JPRB, 0.93354E+00_JPRB ,&
&  0.92013E+00_JPRB, 0.94795E+00_JPRB, 0.94448E+00_JPRB, 0.91013E+00_JPRB, 0.87374E+00_JPRB, 0.85810E+00_JPRB, 0.83945E+00_JPRB ,&
&  0.83567E+00_JPRB, 0.84623E+00_JPRB, 0.84419E+00_JPRB, 0.82429E+00_JPRB, 0.80674E+00_JPRB, 0.78994E+00_JPRB, 0.78830E+00_JPRB ,&
&  0.80466E+00_JPRB, 0.82656E+00_JPRB, 0.88326E+00_JPRB, 0.93612E+00_JPRB, 0.92360E+00_JPRB, 0.89002E+00_JPRB, 0.85145E+00_JPRB ,&
&  0.81788E+00_JPRB, 0.79548E+00_JPRB, 0.79026E+00_JPRB, 0.84324E+00_JPRB, 0.86314E+00_JPRB, 0.92312E+00_JPRB, 0.88395E+00_JPRB ,&
&  0.82181E+00_JPRB, 0.78065E+00_JPRB, 0.76317E+00_JPRB, 0.76248E+00_JPRB, 0.77495E+00_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.76840E+00_JPRB, 0.74820E+00_JPRB, 0.75216E+00_JPRB, 0.75658E+00_JPRB, 0.75368E+00_JPRB, 0.73700E+00_JPRB, 0.72731E+00_JPRB ,&
&  0.74861E+00_JPRB, 0.75879E+00_JPRB, 0.75801E+00_JPRB, 0.73089E+00_JPRB, 0.71453E+00_JPRB, 0.68845E+00_JPRB, 0.66825E+00_JPRB ,&
&  0.66778E+00_JPRB, 0.67046E+00_JPRB, 0.66510E+00_JPRB, 0.66710E+00_JPRB, 0.67380E+00_JPRB, 0.69307E+00_JPRB, 0.70653E+00_JPRB ,&
&  0.71959E+00_JPRB, 0.73214E+00_JPRB, 0.75257E+00_JPRB, 0.76573E+00_JPRB, 0.76264E+00_JPRB, 0.74733E+00_JPRB, 0.73450E+00_JPRB ,&
&  0.76880E+00_JPRB, 0.14936E+01_JPRB, 0.10103E+01_JPRB, 0.13678E+01_JPRB, 0.10560E+01_JPRB, 0.90501E+00_JPRB, 0.76064E+00_JPRB ,&
&  0.80727E+00_JPRB, 0.96942E+00_JPRB, 0.10475E+01_JPRB, 0.10201E+01_JPRB, 0.10058E+01_JPRB, 0.97920E+00_JPRB, 0.92713E+00_JPRB ,&
&  0.89148E+00_JPRB, 0.88609E+00_JPRB, 0.86067E+00_JPRB, 0.84989E+00_JPRB, 0.83172E+00_JPRB, 0.81215E+00_JPRB, 0.80107E+00_JPRB ,&
&  0.78970E+00_JPRB, 0.77910E+00_JPRB, 0.76094E+00_JPRB, 0.79672E+00_JPRB, 0.80131E+00_JPRB, 0.89390E+00_JPRB, 0.88780E+00_JPRB ,&
&  0.80859E+00_JPRB, 0.10399E+01_JPRB, 0.10081E+01_JPRB, 0.98529E+00_JPRB, 0.99919E+00_JPRB, 0.97734E+00_JPRB, 0.95865E+00_JPRB ,&
&  0.93254E+00_JPRB, 0.93966E+00_JPRB, 0.89849E+00_JPRB, 0.87822E+00_JPRB, 0.88143E+00_JPRB, 0.87519E+00_JPRB, 0.84768E+00_JPRB ,&
&  0.81963E+00_JPRB, 0.82342E+00_JPRB, 0.83576E+00_JPRB, 0.83094E+00_JPRB, 0.81558E+00_JPRB, 0.79061E+00_JPRB, 0.78793E+00_JPRB ,&
&  0.80919E+00_JPRB, 0.82393E+00_JPRB, 0.85747E+00_JPRB, 0.92904E+00_JPRB, 0.90548E+00_JPRB, 0.84641E+00_JPRB, 0.80678E+00_JPRB ,&
&  0.81769E+00_JPRB, 0.80629E+00_JPRB, 0.80762E+00_JPRB, 0.83630E+00_JPRB, 0.85610E+00_JPRB, 0.88287E+00_JPRB, 0.81612E+00_JPRB ,&
&  0.78325E+00_JPRB, 0.75693E+00_JPRB, 0.74785E+00_JPRB, 0.75187E+00_JPRB, 0.76900E+00_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.79420E+00_JPRB, 0.76632E+00_JPRB, 0.74313E+00_JPRB, 0.74421E+00_JPRB, 0.74674E+00_JPRB, 0.73809E+00_JPRB, 0.72241E+00_JPRB ,&
&  0.71966E+00_JPRB, 0.71424E+00_JPRB, 0.72821E+00_JPRB, 0.71647E+00_JPRB, 0.70763E+00_JPRB, 0.69254E+00_JPRB, 0.68063E+00_JPRB ,&
&  0.67952E+00_JPRB, 0.68468E+00_JPRB, 0.67134E+00_JPRB, 0.67680E+00_JPRB, 0.70612E+00_JPRB, 0.73176E+00_JPRB, 0.74157E+00_JPRB ,&
&  0.75379E+00_JPRB, 0.76801E+00_JPRB, 0.77335E+00_JPRB, 0.77566E+00_JPRB, 0.76446E+00_JPRB, 0.76233E+00_JPRB, 0.75970E+00_JPRB ,&
&  0.81247E+00_JPRB, 0.16380E+01_JPRB, 0.12619E+01_JPRB, 0.14131E+01_JPRB, 0.10721E+01_JPRB, 0.88034E+00_JPRB, 0.71140E+00_JPRB ,&
&  0.73788E+00_JPRB, 0.92317E+00_JPRB, 0.10258E+01_JPRB, 0.10711E+01_JPRB, 0.10243E+01_JPRB, 0.97132E+00_JPRB, 0.91987E+00_JPRB ,&
&  0.88827E+00_JPRB, 0.88409E+00_JPRB, 0.86435E+00_JPRB, 0.84643E+00_JPRB, 0.82020E+00_JPRB, 0.81648E+00_JPRB, 0.81075E+00_JPRB ,&
&  0.80165E+00_JPRB, 0.79831E+00_JPRB, 0.80155E+00_JPRB, 0.80385E+00_JPRB, 0.89773E+00_JPRB, 0.95119E+00_JPRB, 0.89749E+00_JPRB ,&
&  0.86089E+00_JPRB, 0.10522E+01_JPRB, 0.10465E+01_JPRB, 0.10053E+01_JPRB, 0.99885E+00_JPRB, 0.95875E+00_JPRB, 0.95175E+00_JPRB ,&
&  0.91977E+00_JPRB, 0.89790E+00_JPRB, 0.87540E+00_JPRB, 0.86676E+00_JPRB, 0.88046E+00_JPRB, 0.87750E+00_JPRB, 0.83830E+00_JPRB ,&
&  0.81379E+00_JPRB, 0.82337E+00_JPRB, 0.83152E+00_JPRB, 0.83398E+00_JPRB, 0.82068E+00_JPRB, 0.80411E+00_JPRB, 0.81002E+00_JPRB ,&
&  0.83025E+00_JPRB, 0.85135E+00_JPRB, 0.86331E+00_JPRB, 0.91312E+00_JPRB, 0.88867E+00_JPRB, 0.84285E+00_JPRB, 0.79884E+00_JPRB ,&
&  0.80697E+00_JPRB, 0.81726E+00_JPRB, 0.82255E+00_JPRB, 0.82384E+00_JPRB, 0.83889E+00_JPRB, 0.83103E+00_JPRB, 0.78555E+00_JPRB ,&
&  0.78711E+00_JPRB, 0.79281E+00_JPRB, 0.80381E+00_JPRB, 0.79249E+00_JPRB, 0.81337E+00_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.78186E+00_JPRB, 0.74081E+00_JPRB, 0.73035E+00_JPRB, 0.73906E+00_JPRB, 0.72821E+00_JPRB, 0.71251E+00_JPRB, 0.68741E+00_JPRB ,&
&  0.66708E+00_JPRB, 0.68965E+00_JPRB, 0.72046E+00_JPRB, 0.73595E+00_JPRB, 0.74209E+00_JPRB, 0.74668E+00_JPRB, 0.72387E+00_JPRB ,&
&  0.69365E+00_JPRB, 0.69328E+00_JPRB, 0.70827E+00_JPRB, 0.73685E+00_JPRB, 0.75696E+00_JPRB, 0.77718E+00_JPRB, 0.80873E+00_JPRB ,&
&  0.81847E+00_JPRB, 0.81638E+00_JPRB, 0.80638E+00_JPRB, 0.79579E+00_JPRB, 0.79079E+00_JPRB, 0.79970E+00_JPRB, 0.81441E+00_JPRB ,&
&  0.94743E+00_JPRB, 0.21038E+01_JPRB, 0.21077E+01_JPRB, 0.14462E+01_JPRB, 0.11050E+01_JPRB, 0.89033E+00_JPRB, 0.76517E+00_JPRB ,&
&  0.72559E+00_JPRB, 0.73247E+00_JPRB, 0.10508E+01_JPRB, 0.10850E+01_JPRB, 0.10129E+01_JPRB, 0.96279E+00_JPRB, 0.95918E+00_JPRB ,&
&  0.92799E+00_JPRB, 0.92093E+00_JPRB, 0.88485E+00_JPRB, 0.85840E+00_JPRB, 0.83622E+00_JPRB, 0.83892E+00_JPRB, 0.84538E+00_JPRB ,&
&  0.84325E+00_JPRB, 0.85287E+00_JPRB, 0.90134E+00_JPRB, 0.93286E+00_JPRB, 0.90895E+00_JPRB, 0.91633E+00_JPRB, 0.86380E+00_JPRB ,&
&  0.89869E+00_JPRB, 0.10141E+01_JPRB, 0.10328E+01_JPRB, 0.10207E+01_JPRB, 0.93898E+00_JPRB, 0.97740E+00_JPRB, 0.94239E+00_JPRB ,&
&  0.91618E+00_JPRB, 0.87239E+00_JPRB, 0.87193E+00_JPRB, 0.88478E+00_JPRB, 0.89446E+00_JPRB, 0.89433E+00_JPRB, 0.87151E+00_JPRB ,&
&  0.85338E+00_JPRB, 0.85056E+00_JPRB, 0.84681E+00_JPRB, 0.84276E+00_JPRB, 0.83787E+00_JPRB, 0.83855E+00_JPRB, 0.86358E+00_JPRB ,&
&  0.88037E+00_JPRB, 0.91213E+00_JPRB, 0.92133E+00_JPRB, 0.95993E+00_JPRB, 0.93907E+00_JPRB, 0.87780E+00_JPRB, 0.82224E+00_JPRB ,&
&  0.81618E+00_JPRB, 0.84436E+00_JPRB, 0.85784E+00_JPRB, 0.85579E+00_JPRB, 0.84052E+00_JPRB, 0.81430E+00_JPRB, 0.81243E+00_JPRB ,&
&  0.83063E+00_JPRB, 0.86037E+00_JPRB, 0.86001E+00_JPRB, 0.81402E+00_JPRB, 0.82730E+00_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.75964E+00_JPRB, 0.72270E+00_JPRB, 0.71282E+00_JPRB, 0.70268E+00_JPRB, 0.68405E+00_JPRB, 0.67521E+00_JPRB, 0.65390E+00_JPRB ,&
&  0.67296E+00_JPRB, 0.73264E+00_JPRB, 0.80189E+00_JPRB, 0.82772E+00_JPRB, 0.84107E+00_JPRB, 0.83688E+00_JPRB, 0.80561E+00_JPRB ,&
&  0.76624E+00_JPRB, 0.76802E+00_JPRB, 0.78246E+00_JPRB, 0.79232E+00_JPRB, 0.80943E+00_JPRB, 0.82340E+00_JPRB, 0.84898E+00_JPRB ,&
&  0.87015E+00_JPRB, 0.85147E+00_JPRB, 0.82652E+00_JPRB, 0.81679E+00_JPRB, 0.84946E+00_JPRB, 0.89210E+00_JPRB, 0.93946E+00_JPRB ,&
&  0.12725E+01_JPRB, 0.23632E+01_JPRB, 0.13223E+01_JPRB, 0.11521E+01_JPRB, 0.10587E+01_JPRB, 0.93525E+00_JPRB, 0.83951E+00_JPRB ,&
&  0.82105E+00_JPRB, 0.76263E+00_JPRB, 0.90974E+00_JPRB, 0.10666E+01_JPRB, 0.10229E+01_JPRB, 0.98325E+00_JPRB, 0.97827E+00_JPRB ,&
&  0.96832E+00_JPRB, 0.95450E+00_JPRB, 0.94023E+00_JPRB, 0.90187E+00_JPRB, 0.89063E+00_JPRB, 0.88457E+00_JPRB, 0.89044E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.91131E+00_JPRB, 0.97284E+00_JPRB, 0.92919E+00_JPRB, 0.94376E+00_JPRB, 0.88250E+00_JPRB, 0.75406E+00_JPRB ,&
&  0.71151E+00_JPRB, 0.86324E+00_JPRB, 0.98957E+00_JPRB, 0.10752E+01_JPRB, 0.10476E+01_JPRB, 0.90983E+00_JPRB, 0.97090E+00_JPRB ,&
&  0.89898E+00_JPRB, 0.88832E+00_JPRB, 0.90073E+00_JPRB, 0.90346E+00_JPRB, 0.92528E+00_JPRB, 0.92365E+00_JPRB, 0.92612E+00_JPRB ,&
&  0.90814E+00_JPRB, 0.89132E+00_JPRB, 0.87952E+00_JPRB, 0.86946E+00_JPRB, 0.86365E+00_JPRB, 0.87862E+00_JPRB, 0.90834E+00_JPRB ,&
&  0.93525E+00_JPRB, 0.98059E+00_JPRB, 0.10314E+01_JPRB, 0.10475E+01_JPRB, 0.10282E+01_JPRB, 0.96902E+00_JPRB, 0.89920E+00_JPRB ,&
&  0.88100E+00_JPRB, 0.89390E+00_JPRB, 0.87035E+00_JPRB, 0.84039E+00_JPRB, 0.85098E+00_JPRB, 0.85180E+00_JPRB, 0.90820E+00_JPRB ,&
&  0.99610E+00_JPRB, 0.11369E+01_JPRB, 0.12403E+01_JPRB, 0.80135E+00_JPRB, 0.79590E+00_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.69046E+00_JPRB, 0.68764E+00_JPRB, 0.66159E+00_JPRB, 0.64251E+00_JPRB, 0.63531E+00_JPRB, 0.65683E+00_JPRB, 0.70182E+00_JPRB ,&
&  0.77257E+00_JPRB, 0.86081E+00_JPRB, 0.93669E+00_JPRB, 0.95955E+00_JPRB, 0.94527E+00_JPRB, 0.94607E+00_JPRB, 0.89123E+00_JPRB ,&
&  0.83377E+00_JPRB, 0.82221E+00_JPRB, 0.83347E+00_JPRB, 0.84321E+00_JPRB, 0.85583E+00_JPRB, 0.87559E+00_JPRB, 0.89904E+00_JPRB ,&
&  0.91129E+00_JPRB, 0.91692E+00_JPRB, 0.91872E+00_JPRB, 0.96212E+00_JPRB, 0.10306E+01_JPRB, 0.11361E+01_JPRB, 0.13224E+01_JPRB ,&
&  0.17633E+01_JPRB, 0.16667E+01_JPRB, 0.92479E+00_JPRB, 0.98355E+00_JPRB, 0.87303E+00_JPRB, 0.89339E+00_JPRB, 0.86195E+00_JPRB ,&
&  0.90717E+00_JPRB, 0.90541E+00_JPRB, 0.10025E+01_JPRB, 0.11030E+01_JPRB, 0.10781E+01_JPRB, 0.10637E+01_JPRB, 0.10406E+01_JPRB ,&
&  0.10317E+01_JPRB, 0.10055E+01_JPRB, 0.98135E+00_JPRB, 0.94958E+00_JPRB, 0.93464E+00_JPRB, 0.93562E+00_JPRB, 0.93783E+00_JPRB ,&
&  0.96917E+00_JPRB, 0.98307E+00_JPRB, 0.10554E+01_JPRB, 0.95357E+00_JPRB, 0.98130E+00_JPRB, 0.86275E+00_JPRB, 0.70596E+00_JPRB ,&
&  0.66721E+00_JPRB, 0.78675E+00_JPRB, 0.97060E+00_JPRB, 0.11214E+01_JPRB, 0.11093E+01_JPRB, 0.91172E+00_JPRB, 0.95357E+00_JPRB ,&
&  0.89937E+00_JPRB, 0.89992E+00_JPRB, 0.90200E+00_JPRB, 0.91790E+00_JPRB, 0.94909E+00_JPRB, 0.95182E+00_JPRB, 0.95670E+00_JPRB ,&
&  0.92851E+00_JPRB, 0.90506E+00_JPRB, 0.89490E+00_JPRB, 0.89161E+00_JPRB, 0.89715E+00_JPRB, 0.92922E+00_JPRB, 0.94880E+00_JPRB ,&
&  0.97329E+00_JPRB, 0.10108E+01_JPRB, 0.10571E+01_JPRB, 0.11084E+01_JPRB, 0.11002E+01_JPRB, 0.10417E+01_JPRB, 0.96677E+00_JPRB ,&
&  0.94630E+00_JPRB, 0.95162E+00_JPRB, 0.88660E+00_JPRB, 0.84567E+00_JPRB, 0.81614E+00_JPRB, 0.83859E+00_JPRB, 0.83476E+00_JPRB ,&
&  0.83679E+00_JPRB, 0.83943E+00_JPRB, 0.80059E+00_JPRB, 0.69389E+00_JPRB, 0.69447E+00_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.66662E+00_JPRB, 0.66931E+00_JPRB, 0.66124E+00_JPRB, 0.67695E+00_JPRB, 0.67834E+00_JPRB, 0.73880E+00_JPRB, 0.80689E+00_JPRB ,&
&  0.90967E+00_JPRB, 0.98374E+00_JPRB, 0.10270E+01_JPRB, 0.10308E+01_JPRB, 0.10073E+01_JPRB, 0.94378E+00_JPRB, 0.88456E+00_JPRB ,&
&  0.87832E+00_JPRB, 0.87873E+00_JPRB, 0.89017E+00_JPRB, 0.90856E+00_JPRB, 0.92473E+00_JPRB, 0.95107E+00_JPRB, 0.97599E+00_JPRB ,&
&  0.10060E+01_JPRB, 0.10496E+01_JPRB, 0.10980E+01_JPRB, 0.11689E+01_JPRB, 0.12562E+01_JPRB, 0.13757E+01_JPRB, 0.14840E+01_JPRB ,&
&  0.10994E+01_JPRB, 0.56959E+00_JPRB, 0.81160E+00_JPRB, 0.87221E+00_JPRB, 0.82427E+00_JPRB, 0.93559E+00_JPRB, 0.10853E+01_JPRB ,&
&  0.10041E+01_JPRB, 0.97991E+00_JPRB, 0.98385E+00_JPRB, 0.11319E+01_JPRB, 0.11159E+01_JPRB, 0.11028E+01_JPRB, 0.10902E+01_JPRB ,&
&  0.10670E+01_JPRB, 0.10379E+01_JPRB, 0.10091E+01_JPRB, 0.10021E+01_JPRB, 0.99029E+00_JPRB, 0.99907E+00_JPRB, 0.10048E+01_JPRB ,&
&  0.10254E+01_JPRB, 0.10656E+01_JPRB, 0.11479E+01_JPRB, 0.94890E+00_JPRB, 0.97103E+00_JPRB, 0.89304E+00_JPRB, 0.68569E+00_JPRB ,&
&  0.68233E+00_JPRB, 0.72312E+00_JPRB, 0.81993E+00_JPRB, 0.11128E+01_JPRB, 0.11426E+01_JPRB, 0.92580E+00_JPRB, 0.95692E+00_JPRB ,&
&  0.91530E+00_JPRB, 0.94205E+00_JPRB, 0.91829E+00_JPRB, 0.96753E+00_JPRB, 0.98860E+00_JPRB, 0.99941E+00_JPRB, 0.96794E+00_JPRB ,&
&  0.93554E+00_JPRB, 0.90238E+00_JPRB, 0.88971E+00_JPRB, 0.89649E+00_JPRB, 0.90423E+00_JPRB, 0.93559E+00_JPRB, 0.94384E+00_JPRB ,&
&  0.95180E+00_JPRB, 0.97857E+00_JPRB, 0.10271E+01_JPRB, 0.10776E+01_JPRB, 0.11068E+01_JPRB, 0.10566E+01_JPRB, 0.10107E+01_JPRB ,&
&  0.97678E+00_JPRB, 0.94345E+00_JPRB, 0.86905E+00_JPRB, 0.77825E+00_JPRB, 0.77105E+00_JPRB, 0.73032E+00_JPRB, 0.67858E+00_JPRB ,&
&  0.68761E+00_JPRB, 0.68421E+00_JPRB, 0.66789E+00_JPRB, 0.66480E+00_JPRB, 0.66954E+00_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.75012E+00_JPRB, 0.74163E+00_JPRB, 0.73340E+00_JPRB, 0.73526E+00_JPRB, 0.74294E+00_JPRB, 0.78839E+00_JPRB, 0.86183E+00_JPRB ,&
&  0.93036E+00_JPRB, 0.97002E+00_JPRB, 0.10058E+01_JPRB, 0.10182E+01_JPRB, 0.10140E+01_JPRB, 0.96339E+00_JPRB, 0.95133E+00_JPRB ,&
&  0.93625E+00_JPRB, 0.92860E+00_JPRB, 0.94306E+00_JPRB, 0.96881E+00_JPRB, 0.98824E+00_JPRB, 0.10090E+01_JPRB, 0.10287E+01_JPRB ,&
&  0.10544E+01_JPRB, 0.10972E+01_JPRB, 0.11365E+01_JPRB, 0.12039E+01_JPRB, 0.12418E+01_JPRB, 0.12418E+01_JPRB, 0.11582E+01_JPRB ,&
&  0.52702E+00_JPRB, 0.64092E+00_JPRB, 0.10248E+01_JPRB, 0.10183E+01_JPRB, 0.10662E+01_JPRB, 0.11228E+01_JPRB, 0.11832E+01_JPRB ,&
&  0.10990E+01_JPRB, 0.97238E+00_JPRB, 0.99053E+00_JPRB, 0.11594E+01_JPRB, 0.11329E+01_JPRB, 0.11437E+01_JPRB, 0.11346E+01_JPRB ,&
&  0.11094E+01_JPRB, 0.10816E+01_JPRB, 0.10559E+01_JPRB, 0.10449E+01_JPRB, 0.10461E+01_JPRB, 0.10521E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.10724E+01_JPRB, 0.11045E+01_JPRB, 0.11604E+01_JPRB, 0.94406E+00_JPRB, 0.93687E+00_JPRB, 0.90777E+00_JPRB, 0.70181E+00_JPRB ,&
&  0.59751E+00_JPRB, 0.62547E+00_JPRB, 0.77373E+00_JPRB, 0.10748E+01_JPRB, 0.10940E+01_JPRB, 0.11351E+01_JPRB, 0.95763E+00_JPRB ,&
&  0.94265E+00_JPRB, 0.89468E+00_JPRB, 0.90389E+00_JPRB, 0.95104E+00_JPRB, 0.98464E+00_JPRB, 0.10010E+01_JPRB, 0.97042E+00_JPRB ,&
&  0.91370E+00_JPRB, 0.88076E+00_JPRB, 0.86467E+00_JPRB, 0.86831E+00_JPRB, 0.88612E+00_JPRB, 0.89888E+00_JPRB, 0.89290E+00_JPRB ,&
&  0.88776E+00_JPRB, 0.89418E+00_JPRB, 0.93652E+00_JPRB, 0.96518E+00_JPRB, 0.99954E+00_JPRB, 0.10385E+01_JPRB, 0.10740E+01_JPRB ,&
&  0.10452E+01_JPRB, 0.10426E+01_JPRB, 0.11474E+01_JPRB, 0.10719E+01_JPRB, 0.97613E+00_JPRB, 0.84074E+00_JPRB, 0.70658E+00_JPRB ,&
&  0.72177E+00_JPRB, 0.72967E+00_JPRB, 0.72504E+00_JPRB, 0.73079E+00_JPRB, 0.74757E+00_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.83691E+00_JPRB, 0.81962E+00_JPRB, 0.80670E+00_JPRB, 0.79472E+00_JPRB, 0.80600E+00_JPRB, 0.83834E+00_JPRB, 0.87469E+00_JPRB ,&
&  0.90905E+00_JPRB, 0.94861E+00_JPRB, 0.10032E+01_JPRB, 0.10151E+01_JPRB, 0.10460E+01_JPRB, 0.10361E+01_JPRB, 0.10146E+01_JPRB ,&
&  0.98147E+00_JPRB, 0.98640E+00_JPRB, 0.10001E+01_JPRB, 0.10156E+01_JPRB, 0.10131E+01_JPRB, 0.10245E+01_JPRB, 0.10395E+01_JPRB ,&
&  0.10664E+01_JPRB, 0.10910E+01_JPRB, 0.11293E+01_JPRB, 0.12039E+01_JPRB, 0.12122E+01_JPRB, 0.11904E+01_JPRB, 0.11678E+01_JPRB ,&
&  0.63408E+00_JPRB, 0.10601E+01_JPRB, 0.11481E+01_JPRB, 0.11164E+01_JPRB, 0.11563E+01_JPRB, 0.11270E+01_JPRB, 0.10888E+01_JPRB ,&
&  0.10954E+01_JPRB, 0.10570E+01_JPRB, 0.10434E+01_JPRB, 0.11056E+01_JPRB, 0.11613E+01_JPRB, 0.11730E+01_JPRB, 0.11655E+01_JPRB ,&
&  0.11409E+01_JPRB, 0.11126E+01_JPRB, 0.11050E+01_JPRB, 0.11229E+01_JPRB, 0.11253E+01_JPRB, 0.11327E+01_JPRB, 0.11480E+01_JPRB ,&
&  0.11367E+01_JPRB, 0.11298E+01_JPRB, 0.11796E+01_JPRB, 0.10552E+01_JPRB, 0.96801E+00_JPRB, 0.93899E+00_JPRB, 0.77968E+00_JPRB ,&
&  0.57995E+00_JPRB, 0.52604E+00_JPRB, 0.72130E+00_JPRB, 0.96561E+00_JPRB, 0.90096E+00_JPRB, 0.92494E+00_JPRB, 0.87248E+00_JPRB ,&
&  0.78113E+00_JPRB, 0.75539E+00_JPRB, 0.82419E+00_JPRB, 0.88666E+00_JPRB, 0.93721E+00_JPRB, 0.97438E+00_JPRB, 0.95264E+00_JPRB ,&
&  0.90037E+00_JPRB, 0.85711E+00_JPRB, 0.82459E+00_JPRB, 0.81738E+00_JPRB, 0.83928E+00_JPRB, 0.84964E+00_JPRB, 0.83541E+00_JPRB ,&
&  0.83178E+00_JPRB, 0.82430E+00_JPRB, 0.86863E+00_JPRB, 0.90240E+00_JPRB, 0.95129E+00_JPRB, 0.10107E+01_JPRB, 0.11557E+01_JPRB ,&
&  0.13968E+01_JPRB, 0.16216E+01_JPRB, 0.18997E+01_JPRB, 0.18721E+01_JPRB, 0.17088E+01_JPRB, 0.12218E+01_JPRB, 0.81377E+00_JPRB ,&
&  0.83007E+00_JPRB, 0.84874E+00_JPRB, 0.84995E+00_JPRB, 0.85669E+00_JPRB, 0.84650E+00_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.96767E+00_JPRB, 0.95086E+00_JPRB, 0.94636E+00_JPRB, 0.94159E+00_JPRB, 0.93885E+00_JPRB, 0.94575E+00_JPRB, 0.96719E+00_JPRB ,&
&  0.97580E+00_JPRB, 0.10085E+01_JPRB, 0.10575E+01_JPRB, 0.10855E+01_JPRB, 0.11220E+01_JPRB, 0.11159E+01_JPRB, 0.10742E+01_JPRB ,&
&  0.10606E+01_JPRB, 0.10857E+01_JPRB, 0.11080E+01_JPRB, 0.11122E+01_JPRB, 0.10682E+01_JPRB, 0.10359E+01_JPRB, 0.10313E+01_JPRB ,&
&  0.10502E+01_JPRB, 0.10658E+01_JPRB, 0.10818E+01_JPRB, 0.11139E+01_JPRB, 0.11044E+01_JPRB, 0.10247E+01_JPRB, 0.68560E+00_JPRB ,&
&  0.10130E+01_JPRB, 0.12111E+01_JPRB, 0.12236E+01_JPRB, 0.11543E+01_JPRB, 0.11166E+01_JPRB, 0.10445E+01_JPRB, 0.10379E+01_JPRB ,&
&  0.10750E+01_JPRB, 0.11152E+01_JPRB, 0.10921E+01_JPRB, 0.10350E+01_JPRB, 0.11314E+01_JPRB, 0.11320E+01_JPRB, 0.11304E+01_JPRB ,&
&  0.11227E+01_JPRB, 0.11186E+01_JPRB, 0.11199E+01_JPRB, 0.11241E+01_JPRB, 0.11191E+01_JPRB, 0.11238E+01_JPRB, 0.11196E+01_JPRB ,&
&  0.10945E+01_JPRB, 0.10670E+01_JPRB, 0.10849E+01_JPRB, 0.10059E+01_JPRB, 0.97974E+00_JPRB, 0.93550E+00_JPRB, 0.80632E+00_JPRB ,&
&  0.54775E+00_JPRB, 0.49347E+00_JPRB, 0.71471E+00_JPRB, 0.92429E+00_JPRB, 0.82347E+00_JPRB, 0.78812E+00_JPRB, 0.80329E+00_JPRB ,&
&  0.76747E+00_JPRB, 0.71288E+00_JPRB, 0.69891E+00_JPRB, 0.74795E+00_JPRB, 0.81895E+00_JPRB, 0.87378E+00_JPRB, 0.88775E+00_JPRB ,&
&  0.85431E+00_JPRB, 0.82596E+00_JPRB, 0.81993E+00_JPRB, 0.83107E+00_JPRB, 0.85672E+00_JPRB, 0.92141E+00_JPRB, 0.10134E+01_JPRB ,&
&  0.95214E+00_JPRB, 0.77572E+00_JPRB, 0.90130E+00_JPRB, 0.97101E+00_JPRB, 0.10962E+01_JPRB, 0.12667E+01_JPRB, 0.15653E+01_JPRB ,&
&  0.20389E+01_JPRB, 0.25540E+01_JPRB, 0.25284E+01_JPRB, 0.21171E+01_JPRB, 0.26837E+01_JPRB, 0.17862E+01_JPRB, 0.93725E+00_JPRB ,&
&  0.96339E+00_JPRB, 0.98052E+00_JPRB, 0.99541E+00_JPRB, 0.98714E+00_JPRB, 0.97986E+00_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.98858E+00_JPRB, 0.98881E+00_JPRB, 0.97902E+00_JPRB, 0.10066E+01_JPRB, 0.10017E+01_JPRB, 0.10145E+01_JPRB, 0.10136E+01_JPRB ,&
&  0.10344E+01_JPRB, 0.10510E+01_JPRB, 0.10704E+01_JPRB, 0.11044E+01_JPRB, 0.11089E+01_JPRB, 0.11209E+01_JPRB, 0.11460E+01_JPRB ,&
&  0.11704E+01_JPRB, 0.12024E+01_JPRB, 0.11968E+01_JPRB, 0.11774E+01_JPRB, 0.11368E+01_JPRB, 0.11063E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.10326E+01_JPRB, 0.10308E+01_JPRB, 0.10368E+01_JPRB, 0.10359E+01_JPRB, 0.10408E+01_JPRB, 0.69908E+00_JPRB ,&
&  0.11193E+01_JPRB, 0.12587E+01_JPRB, 0.11768E+01_JPRB, 0.11016E+01_JPRB, 0.10146E+01_JPRB, 0.10100E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.10636E+01_JPRB, 0.11059E+01_JPRB, 0.10912E+01_JPRB, 0.10571E+01_JPRB, 0.10296E+01_JPRB, 0.10376E+01_JPRB, 0.10271E+01_JPRB ,&
&  0.10208E+01_JPRB, 0.10141E+01_JPRB, 0.10038E+01_JPRB, 0.98805E+00_JPRB, 0.97408E+00_JPRB, 0.96904E+00_JPRB, 0.94880E+00_JPRB ,&
&  0.91993E+00_JPRB, 0.89203E+00_JPRB, 0.89882E+00_JPRB, 0.89573E+00_JPRB, 0.88627E+00_JPRB, 0.78465E+00_JPRB, 0.63859E+00_JPRB ,&
&  0.44929E+00_JPRB, 0.47368E+00_JPRB, 0.66299E+00_JPRB, 0.91634E+00_JPRB, 0.83798E+00_JPRB, 0.76542E+00_JPRB, 0.72010E+00_JPRB ,&
&  0.71260E+00_JPRB, 0.67223E+00_JPRB, 0.62104E+00_JPRB, 0.61352E+00_JPRB, 0.63532E+00_JPRB, 0.68589E+00_JPRB, 0.69131E+00_JPRB ,&
&  0.70398E+00_JPRB, 0.69840E+00_JPRB, 0.67446E+00_JPRB, 0.65513E+00_JPRB, 0.68134E+00_JPRB, 0.67538E+00_JPRB, 0.59562E+00_JPRB ,&
&  0.69815E+00_JPRB, 0.66652E+00_JPRB, 0.71909E+00_JPRB, 0.81295E+00_JPRB, 0.89877E+00_JPRB, 0.10878E+01_JPRB, 0.13846E+01_JPRB ,&
&  0.18358E+01_JPRB, 0.18191E+01_JPRB, 0.24336E+01_JPRB, 0.37322E+01_JPRB, 0.29225E+01_JPRB, 0.20632E+01_JPRB, 0.10912E+01_JPRB ,&
&  0.10718E+01_JPRB, 0.10396E+01_JPRB, 0.10431E+01_JPRB, 0.10192E+01_JPRB, 0.10002E+01_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.90380E+00_JPRB, 0.91220E+00_JPRB, 0.92800E+00_JPRB, 0.95203E+00_JPRB, 0.96356E+00_JPRB, 0.98354E+00_JPRB, 0.99075E+00_JPRB ,&
&  0.10002E+01_JPRB, 0.10274E+01_JPRB, 0.10526E+01_JPRB, 0.10804E+01_JPRB, 0.11042E+01_JPRB, 0.11280E+01_JPRB, 0.11500E+01_JPRB ,&
&  0.11689E+01_JPRB, 0.11846E+01_JPRB, 0.11891E+01_JPRB, 0.11789E+01_JPRB, 0.11554E+01_JPRB, 0.11416E+01_JPRB, 0.11185E+01_JPRB ,&
&  0.10998E+01_JPRB, 0.10984E+01_JPRB, 0.11031E+01_JPRB, 0.11045E+01_JPRB, 0.11063E+01_JPRB, 0.11363E+01_JPRB, 0.79039E+00_JPRB ,&
&  0.11638E+01_JPRB, 0.12331E+01_JPRB, 0.11645E+01_JPRB, 0.10125E+01_JPRB, 0.93598E+00_JPRB, 0.92338E+00_JPRB, 0.93306E+00_JPRB ,&
&  0.96855E+00_JPRB, 0.10216E+01_JPRB, 0.10491E+01_JPRB, 0.96493E+00_JPRB, 0.88916E+00_JPRB, 0.89388E+00_JPRB, 0.87915E+00_JPRB ,&
&  0.87965E+00_JPRB, 0.87325E+00_JPRB, 0.86359E+00_JPRB, 0.84685E+00_JPRB, 0.82894E+00_JPRB, 0.80790E+00_JPRB, 0.78246E+00_JPRB ,&
&  0.74837E+00_JPRB, 0.71275E+00_JPRB, 0.65637E+00_JPRB, 0.65301E+00_JPRB, 0.64452E+00_JPRB, 0.57043E+00_JPRB, 0.45957E+00_JPRB ,&
&  0.39226E+00_JPRB, 0.52717E+00_JPRB, 0.67200E+00_JPRB, 0.91899E+00_JPRB, 0.85281E+00_JPRB, 0.80704E+00_JPRB, 0.74942E+00_JPRB ,&
&  0.65564E+00_JPRB, 0.61299E+00_JPRB, 0.57375E+00_JPRB, 0.55862E+00_JPRB, 0.55653E+00_JPRB, 0.56977E+00_JPRB, 0.56456E+00_JPRB ,&
&  0.54581E+00_JPRB, 0.51478E+00_JPRB, 0.49930E+00_JPRB, 0.48758E+00_JPRB, 0.49050E+00_JPRB, 0.42725E+00_JPRB, 0.53334E+00_JPRB ,&
&  0.53970E+00_JPRB, 0.48277E+00_JPRB, 0.62887E+00_JPRB, 0.47813E+00_JPRB, 0.63226E+00_JPRB, 0.72426E+00_JPRB, 0.84764E+00_JPRB ,&
&  0.88968E+00_JPRB, 0.11865E+01_JPRB, 0.14947E+01_JPRB, 0.13177E+01_JPRB, 0.11638E+01_JPRB, 0.10394E+01_JPRB, 0.98882E+00_JPRB ,&
&  0.97195E+00_JPRB, 0.95592E+00_JPRB, 0.94559E+00_JPRB, 0.92995E+00_JPRB, 0.91119E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.85175E+00_JPRB, 0.85770E+00_JPRB, 0.86786E+00_JPRB, 0.88256E+00_JPRB, 0.89786E+00_JPRB, 0.91183E+00_JPRB, 0.92287E+00_JPRB ,&
&  0.93137E+00_JPRB, 0.94469E+00_JPRB, 0.96547E+00_JPRB, 0.99092E+00_JPRB, 0.10005E+01_JPRB, 0.10162E+01_JPRB, 0.10195E+01_JPRB ,&
&  0.10401E+01_JPRB, 0.10626E+01_JPRB, 0.10710E+01_JPRB, 0.10799E+01_JPRB, 0.10863E+01_JPRB, 0.10990E+01_JPRB, 0.11152E+01_JPRB ,&
&  0.11299E+01_JPRB, 0.11657E+01_JPRB, 0.12182E+01_JPRB, 0.12608E+01_JPRB, 0.13185E+01_JPRB, 0.14083E+01_JPRB, 0.11689E+01_JPRB ,&
&  0.12254E+01_JPRB, 0.12480E+01_JPRB, 0.11207E+01_JPRB, 0.94392E+00_JPRB, 0.88506E+00_JPRB, 0.84369E+00_JPRB, 0.81845E+00_JPRB ,&
&  0.81275E+00_JPRB, 0.77466E+00_JPRB, 0.77953E+00_JPRB, 0.77175E+00_JPRB, 0.76506E+00_JPRB, 0.78903E+00_JPRB, 0.78255E+00_JPRB ,&
&  0.78170E+00_JPRB, 0.78512E+00_JPRB, 0.78896E+00_JPRB, 0.78578E+00_JPRB, 0.78823E+00_JPRB, 0.77517E+00_JPRB, 0.75254E+00_JPRB ,&
&  0.71680E+00_JPRB, 0.67784E+00_JPRB, 0.61887E+00_JPRB, 0.59645E+00_JPRB, 0.58566E+00_JPRB, 0.53704E+00_JPRB, 0.49041E+00_JPRB ,&
&  0.51789E+00_JPRB, 0.57239E+00_JPRB, 0.62263E+00_JPRB, 0.85899E+00_JPRB, 0.86352E+00_JPRB, 0.86243E+00_JPRB, 0.73400E+00_JPRB ,&
&  0.67786E+00_JPRB, 0.59906E+00_JPRB, 0.56088E+00_JPRB, 0.56008E+00_JPRB, 0.55459E+00_JPRB, 0.54612E+00_JPRB, 0.52087E+00_JPRB ,&
&  0.47534E+00_JPRB, 0.43736E+00_JPRB, 0.42626E+00_JPRB, 0.42068E+00_JPRB, 0.43168E+00_JPRB, 0.41278E+00_JPRB, 0.48673E+00_JPRB ,&
&  0.46396E+00_JPRB, 0.34280E+00_JPRB, 0.41397E+00_JPRB, 0.52684E+00_JPRB, 0.53946E+00_JPRB, 0.60653E+00_JPRB, 0.73845E+00_JPRB ,&
&  0.94844E+00_JPRB, 0.10125E+01_JPRB, 0.10271E+01_JPRB, 0.95823E+00_JPRB, 0.93801E+00_JPRB, 0.92990E+00_JPRB, 0.90659E+00_JPRB ,&
&  0.89446E+00_JPRB, 0.88641E+00_JPRB, 0.87851E+00_JPRB, 0.86418E+00_JPRB, 0.85349E+00_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.81364E+00_JPRB, 0.81461E+00_JPRB, 0.81867E+00_JPRB, 0.82627E+00_JPRB, 0.83555E+00_JPRB, 0.84591E+00_JPRB, 0.85137E+00_JPRB ,&
&  0.84818E+00_JPRB, 0.85133E+00_JPRB, 0.86044E+00_JPRB, 0.87496E+00_JPRB, 0.88764E+00_JPRB, 0.89852E+00_JPRB, 0.90992E+00_JPRB ,&
&  0.90733E+00_JPRB, 0.90749E+00_JPRB, 0.92077E+00_JPRB, 0.93810E+00_JPRB, 0.94793E+00_JPRB, 0.94714E+00_JPRB, 0.94898E+00_JPRB ,&
&  0.94434E+00_JPRB, 0.95218E+00_JPRB, 0.97538E+00_JPRB, 0.98834E+00_JPRB, 0.10090E+01_JPRB, 0.10659E+01_JPRB, 0.12396E+01_JPRB ,&
&  0.11003E+01_JPRB, 0.11928E+01_JPRB, 0.10313E+01_JPRB, 0.89428E+00_JPRB, 0.87490E+00_JPRB, 0.83692E+00_JPRB, 0.80865E+00_JPRB ,&
&  0.77731E+00_JPRB, 0.74661E+00_JPRB, 0.70450E+00_JPRB, 0.69634E+00_JPRB, 0.67599E+00_JPRB, 0.69382E+00_JPRB, 0.70756E+00_JPRB ,&
&  0.73354E+00_JPRB, 0.74531E+00_JPRB, 0.76460E+00_JPRB, 0.77695E+00_JPRB, 0.81082E+00_JPRB, 0.80710E+00_JPRB, 0.77079E+00_JPRB ,&
&  0.76120E+00_JPRB, 0.78712E+00_JPRB, 0.74849E+00_JPRB, 0.76407E+00_JPRB, 0.74824E+00_JPRB, 0.71285E+00_JPRB, 0.69431E+00_JPRB ,&
&  0.67658E+00_JPRB, 0.67634E+00_JPRB, 0.76970E+00_JPRB, 0.78412E+00_JPRB, 0.85581E+00_JPRB, 0.88558E+00_JPRB, 0.80628E+00_JPRB ,&
&  0.74068E+00_JPRB, 0.66722E+00_JPRB, 0.65773E+00_JPRB, 0.64928E+00_JPRB, 0.66836E+00_JPRB, 0.65383E+00_JPRB, 0.63635E+00_JPRB ,&
&  0.57537E+00_JPRB, 0.51418E+00_JPRB, 0.50408E+00_JPRB, 0.47980E+00_JPRB, 0.44946E+00_JPRB, 0.49635E+00_JPRB, 0.52942E+00_JPRB ,&
&  0.50283E+00_JPRB, 0.43878E+00_JPRB, 0.40434E+00_JPRB, 0.56546E+00_JPRB, 0.60237E+00_JPRB, 0.66720E+00_JPRB, 0.74501E+00_JPRB ,&
&  0.85550E+00_JPRB, 0.94957E+00_JPRB, 0.10038E+01_JPRB, 0.95028E+00_JPRB, 0.86943E+00_JPRB, 0.84851E+00_JPRB, 0.83273E+00_JPRB ,&
&  0.83056E+00_JPRB, 0.82750E+00_JPRB, 0.82342E+00_JPRB, 0.81730E+00_JPRB, 0.80827E+00_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.75552E+00_JPRB, 0.75631E+00_JPRB, 0.76919E+00_JPRB, 0.76957E+00_JPRB, 0.76966E+00_JPRB, 0.77773E+00_JPRB, 0.78162E+00_JPRB ,&
&  0.77759E+00_JPRB, 0.78603E+00_JPRB, 0.79717E+00_JPRB, 0.80904E+00_JPRB, 0.82253E+00_JPRB, 0.83416E+00_JPRB, 0.83998E+00_JPRB ,&
&  0.83059E+00_JPRB, 0.82039E+00_JPRB, 0.84026E+00_JPRB, 0.85849E+00_JPRB, 0.86138E+00_JPRB, 0.86514E+00_JPRB, 0.89342E+00_JPRB ,&
&  0.94673E+00_JPRB, 0.97386E+00_JPRB, 0.96969E+00_JPRB, 0.93538E+00_JPRB, 0.94801E+00_JPRB, 0.11280E+01_JPRB, 0.17615E+01_JPRB ,&
&  0.15689E+01_JPRB, 0.10506E+01_JPRB, 0.98034E+00_JPRB, 0.88883E+00_JPRB, 0.88477E+00_JPRB, 0.89909E+00_JPRB, 0.88179E+00_JPRB ,&
&  0.82931E+00_JPRB, 0.82091E+00_JPRB, 0.81124E+00_JPRB, 0.79980E+00_JPRB, 0.80332E+00_JPRB, 0.80943E+00_JPRB, 0.80854E+00_JPRB ,&
&  0.79367E+00_JPRB, 0.79474E+00_JPRB, 0.80868E+00_JPRB, 0.81379E+00_JPRB, 0.84061E+00_JPRB, 0.94710E+00_JPRB, 0.94523E+00_JPRB ,&
&  0.94075E+00_JPRB, 0.90512E+00_JPRB, 0.86354E+00_JPRB, 0.86727E+00_JPRB, 0.87630E+00_JPRB, 0.84300E+00_JPRB, 0.80296E+00_JPRB ,&
&  0.79963E+00_JPRB, 0.76752E+00_JPRB, 0.66129E+00_JPRB, 0.74497E+00_JPRB, 0.83249E+00_JPRB, 0.92138E+00_JPRB, 0.88854E+00_JPRB ,&
&  0.85492E+00_JPRB, 0.78249E+00_JPRB, 0.76028E+00_JPRB, 0.79349E+00_JPRB, 0.84030E+00_JPRB, 0.82790E+00_JPRB, 0.79621E+00_JPRB ,&
&  0.71708E+00_JPRB, 0.64815E+00_JPRB, 0.62892E+00_JPRB, 0.57792E+00_JPRB, 0.55434E+00_JPRB, 0.50126E+00_JPRB, 0.56051E+00_JPRB ,&
&  0.56748E+00_JPRB, 0.58624E+00_JPRB, 0.48830E+00_JPRB, 0.64051E+00_JPRB, 0.68168E+00_JPRB, 0.70180E+00_JPRB, 0.72279E+00_JPRB ,&
&  0.74247E+00_JPRB, 0.76718E+00_JPRB, 0.76921E+00_JPRB, 0.84176E+00_JPRB, 0.82550E+00_JPRB, 0.81008E+00_JPRB, 0.80721E+00_JPRB ,&
&  0.77055E+00_JPRB, 0.75753E+00_JPRB, 0.75295E+00_JPRB, 0.75788E+00_JPRB, 0.75680E+00_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.68232E+00_JPRB, 0.67917E+00_JPRB, 0.67893E+00_JPRB, 0.69409E+00_JPRB, 0.70782E+00_JPRB, 0.70102E+00_JPRB, 0.71006E+00_JPRB ,&
&  0.69234E+00_JPRB, 0.67838E+00_JPRB, 0.69017E+00_JPRB, 0.69715E+00_JPRB, 0.71384E+00_JPRB, 0.72787E+00_JPRB, 0.73773E+00_JPRB ,&
&  0.73970E+00_JPRB, 0.74018E+00_JPRB, 0.71807E+00_JPRB, 0.73612E+00_JPRB, 0.82640E+00_JPRB, 0.97022E+00_JPRB, 0.12236E+01_JPRB ,&
&  0.14853E+01_JPRB, 0.15850E+01_JPRB, 0.16363E+01_JPRB, 0.15894E+01_JPRB, 0.14317E+01_JPRB, 0.12177E+01_JPRB, 0.15122E+01_JPRB ,&
&  0.13374E+01_JPRB, 0.10297E+01_JPRB, 0.10792E+01_JPRB, 0.10203E+01_JPRB, 0.10197E+01_JPRB, 0.98684E+00_JPRB, 0.96467E+00_JPRB ,&
&  0.92401E+00_JPRB, 0.91324E+00_JPRB, 0.91665E+00_JPRB, 0.94002E+00_JPRB, 0.96687E+00_JPRB, 0.99695E+00_JPRB, 0.10111E+01_JPRB ,&
&  0.10089E+01_JPRB, 0.10089E+01_JPRB, 0.10424E+01_JPRB, 0.10052E+01_JPRB, 0.97270E+00_JPRB, 0.99662E+00_JPRB, 0.10162E+01_JPRB ,&
&  0.10270E+01_JPRB, 0.10175E+01_JPRB, 0.88471E+00_JPRB, 0.87135E+00_JPRB, 0.90433E+00_JPRB, 0.83141E+00_JPRB, 0.82005E+00_JPRB ,&
&  0.85643E+00_JPRB, 0.80974E+00_JPRB, 0.62599E+00_JPRB, 0.66422E+00_JPRB, 0.74282E+00_JPRB, 0.85836E+00_JPRB, 0.95741E+00_JPRB ,&
&  0.92908E+00_JPRB, 0.87911E+00_JPRB, 0.87083E+00_JPRB, 0.91065E+00_JPRB, 0.93747E+00_JPRB, 0.93076E+00_JPRB, 0.92128E+00_JPRB ,&
&  0.90350E+00_JPRB, 0.77459E+00_JPRB, 0.71340E+00_JPRB, 0.64632E+00_JPRB, 0.61570E+00_JPRB, 0.60535E+00_JPRB, 0.58603E+00_JPRB ,&
&  0.60380E+00_JPRB, 0.61911E+00_JPRB, 0.62124E+00_JPRB, 0.62353E+00_JPRB, 0.60780E+00_JPRB, 0.74017E+00_JPRB, 0.75848E+00_JPRB ,&
&  0.76070E+00_JPRB, 0.76032E+00_JPRB, 0.76993E+00_JPRB, 0.75703E+00_JPRB, 0.73475E+00_JPRB, 0.71328E+00_JPRB, 0.69281E+00_JPRB ,&
&  0.70091E+00_JPRB, 0.70978E+00_JPRB, 0.71218E+00_JPRB, 0.69732E+00_JPRB, 0.68546E+00_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.65584E+00_JPRB, 0.66004E+00_JPRB, 0.65958E+00_JPRB, 0.67321E+00_JPRB, 0.67092E+00_JPRB, 0.68583E+00_JPRB, 0.67627E+00_JPRB ,&
&  0.66628E+00_JPRB, 0.66786E+00_JPRB, 0.67639E+00_JPRB, 0.68504E+00_JPRB, 0.71128E+00_JPRB, 0.76260E+00_JPRB, 0.82841E+00_JPRB ,&
&  0.89854E+00_JPRB, 0.92853E+00_JPRB, 0.98573E+00_JPRB, 0.11047E+01_JPRB, 0.13288E+01_JPRB, 0.15343E+01_JPRB, 0.15593E+01_JPRB ,&
&  0.16190E+01_JPRB, 0.17191E+01_JPRB, 0.17793E+01_JPRB, 0.18028E+01_JPRB, 0.17115E+01_JPRB, 0.15629E+01_JPRB, 0.13806E+01_JPRB ,&
&  0.12078E+01_JPRB, 0.10466E+01_JPRB, 0.10463E+01_JPRB, 0.10589E+01_JPRB, 0.10647E+01_JPRB, 0.10308E+01_JPRB, 0.99916E+00_JPRB ,&
&  0.97311E+00_JPRB, 0.96558E+00_JPRB, 0.95794E+00_JPRB, 0.96741E+00_JPRB, 0.99424E+00_JPRB, 0.10321E+01_JPRB, 0.10376E+01_JPRB ,&
&  0.10206E+01_JPRB, 0.10087E+01_JPRB, 0.10094E+01_JPRB, 0.92346E+00_JPRB, 0.94473E+00_JPRB, 0.95115E+00_JPRB, 0.94014E+00_JPRB ,&
&  0.92007E+00_JPRB, 0.87633E+00_JPRB, 0.87795E+00_JPRB, 0.86236E+00_JPRB, 0.84225E+00_JPRB, 0.79569E+00_JPRB, 0.77885E+00_JPRB ,&
&  0.79461E+00_JPRB, 0.77066E+00_JPRB, 0.64275E+00_JPRB, 0.68707E+00_JPRB, 0.75547E+00_JPRB, 0.78208E+00_JPRB, 0.99568E+00_JPRB ,&
&  0.97146E+00_JPRB, 0.93611E+00_JPRB, 0.93158E+00_JPRB, 0.91631E+00_JPRB, 0.94329E+00_JPRB, 0.94315E+00_JPRB, 0.95528E+00_JPRB ,&
&  0.10064E+01_JPRB, 0.87411E+00_JPRB, 0.78885E+00_JPRB, 0.73088E+00_JPRB, 0.68060E+00_JPRB, 0.64581E+00_JPRB, 0.63433E+00_JPRB ,&
&  0.66151E+00_JPRB, 0.69019E+00_JPRB, 0.68647E+00_JPRB, 0.67882E+00_JPRB, 0.65320E+00_JPRB, 0.74765E+00_JPRB, 0.75281E+00_JPRB ,&
&  0.77208E+00_JPRB, 0.76080E+00_JPRB, 0.74053E+00_JPRB, 0.71504E+00_JPRB, 0.69885E+00_JPRB, 0.68760E+00_JPRB, 0.68321E+00_JPRB ,&
&  0.68759E+00_JPRB, 0.69138E+00_JPRB, 0.68264E+00_JPRB, 0.67276E+00_JPRB, 0.66114E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.63133E+00_JPRB, 0.62883E+00_JPRB, 0.62953E+00_JPRB, 0.65877E+00_JPRB, 0.65788E+00_JPRB, 0.64995E+00_JPRB, 0.66301E+00_JPRB ,&
&  0.67991E+00_JPRB, 0.70122E+00_JPRB, 0.72668E+00_JPRB, 0.75129E+00_JPRB, 0.78489E+00_JPRB, 0.83558E+00_JPRB, 0.89402E+00_JPRB ,&
&  0.96174E+00_JPRB, 0.10440E+01_JPRB, 0.11795E+01_JPRB, 0.13525E+01_JPRB, 0.15524E+01_JPRB, 0.16556E+01_JPRB, 0.16497E+01_JPRB ,&
&  0.15304E+01_JPRB, 0.15586E+01_JPRB, 0.15935E+01_JPRB, 0.15628E+01_JPRB, 0.12989E+01_JPRB, 0.12739E+01_JPRB, 0.12374E+01_JPRB ,&
&  0.12059E+01_JPRB, 0.11299E+01_JPRB, 0.10797E+01_JPRB, 0.10358E+01_JPRB, 0.99460E+00_JPRB, 0.99315E+00_JPRB, 0.98644E+00_JPRB ,&
&  0.96357E+00_JPRB, 0.95370E+00_JPRB, 0.94671E+00_JPRB, 0.94711E+00_JPRB, 0.96276E+00_JPRB, 0.99682E+00_JPRB, 0.10002E+01_JPRB ,&
&  0.99074E+00_JPRB, 0.99102E+00_JPRB, 0.96399E+00_JPRB, 0.91305E+00_JPRB, 0.90911E+00_JPRB, 0.90075E+00_JPRB, 0.88794E+00_JPRB ,&
&  0.89643E+00_JPRB, 0.86492E+00_JPRB, 0.86605E+00_JPRB, 0.85898E+00_JPRB, 0.81625E+00_JPRB, 0.74989E+00_JPRB, 0.73969E+00_JPRB ,&
&  0.75934E+00_JPRB, 0.79383E+00_JPRB, 0.66802E+00_JPRB, 0.80308E+00_JPRB, 0.89613E+00_JPRB, 0.86655E+00_JPRB, 0.91097E+00_JPRB ,&
&  0.91017E+00_JPRB, 0.91753E+00_JPRB, 0.92641E+00_JPRB, 0.92812E+00_JPRB, 0.92356E+00_JPRB, 0.91985E+00_JPRB, 0.88060E+00_JPRB ,&
&  0.96482E+00_JPRB, 0.87125E+00_JPRB, 0.79506E+00_JPRB, 0.76491E+00_JPRB, 0.73372E+00_JPRB, 0.68302E+00_JPRB, 0.64140E+00_JPRB ,&
&  0.63455E+00_JPRB, 0.83526E+00_JPRB, 0.91655E+00_JPRB, 0.10417E+01_JPRB, 0.11061E+01_JPRB, 0.75296E+00_JPRB, 0.78916E+00_JPRB ,&
&  0.77984E+00_JPRB, 0.76487E+00_JPRB, 0.73814E+00_JPRB, 0.70943E+00_JPRB, 0.70396E+00_JPRB, 0.70316E+00_JPRB, 0.69940E+00_JPRB ,&
&  0.69226E+00_JPRB, 0.67730E+00_JPRB, 0.66741E+00_JPRB, 0.64787E+00_JPRB, 0.63818E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.64707E+00_JPRB, 0.64290E+00_JPRB, 0.64274E+00_JPRB, 0.65778E+00_JPRB, 0.67088E+00_JPRB, 0.67338E+00_JPRB, 0.66806E+00_JPRB ,&
&  0.67635E+00_JPRB, 0.70212E+00_JPRB, 0.73307E+00_JPRB, 0.74870E+00_JPRB, 0.75422E+00_JPRB, 0.76372E+00_JPRB, 0.80638E+00_JPRB ,&
&  0.87991E+00_JPRB, 0.98965E+00_JPRB, 0.11276E+01_JPRB, 0.12402E+01_JPRB, 0.13411E+01_JPRB, 0.14224E+01_JPRB, 0.15517E+01_JPRB ,&
&  0.15664E+01_JPRB, 0.16120E+01_JPRB, 0.15546E+01_JPRB, 0.11987E+01_JPRB, 0.10827E+01_JPRB, 0.12583E+01_JPRB, 0.11433E+01_JPRB ,&
&  0.10934E+01_JPRB, 0.10703E+01_JPRB, 0.10462E+01_JPRB, 0.10019E+01_JPRB, 0.95761E+00_JPRB, 0.96097E+00_JPRB, 0.95077E+00_JPRB ,&
&  0.92248E+00_JPRB, 0.91006E+00_JPRB, 0.90616E+00_JPRB, 0.90984E+00_JPRB, 0.91896E+00_JPRB, 0.92560E+00_JPRB, 0.91607E+00_JPRB ,&
&  0.92444E+00_JPRB, 0.91799E+00_JPRB, 0.91995E+00_JPRB, 0.91447E+00_JPRB, 0.89734E+00_JPRB, 0.90351E+00_JPRB, 0.89397E+00_JPRB ,&
&  0.89337E+00_JPRB, 0.87579E+00_JPRB, 0.86211E+00_JPRB, 0.86072E+00_JPRB, 0.84397E+00_JPRB, 0.75323E+00_JPRB, 0.73491E+00_JPRB ,&
&  0.75183E+00_JPRB, 0.76060E+00_JPRB, 0.71847E+00_JPRB, 0.80086E+00_JPRB, 0.61209E+00_JPRB, 0.71344E+00_JPRB, 0.84762E+00_JPRB ,&
&  0.87986E+00_JPRB, 0.89171E+00_JPRB, 0.89471E+00_JPRB, 0.89675E+00_JPRB, 0.89302E+00_JPRB, 0.83343E+00_JPRB, 0.83627E+00_JPRB ,&
&  0.83848E+00_JPRB, 0.80564E+00_JPRB, 0.77462E+00_JPRB, 0.76552E+00_JPRB, 0.74957E+00_JPRB, 0.72021E+00_JPRB, 0.68293E+00_JPRB ,&
&  0.63633E+00_JPRB, 0.78502E+00_JPRB, 0.81442E+00_JPRB, 0.83611E+00_JPRB, 0.83185E+00_JPRB, 0.72516E+00_JPRB, 0.76016E+00_JPRB ,&
&  0.77909E+00_JPRB, 0.75169E+00_JPRB, 0.71912E+00_JPRB, 0.71306E+00_JPRB, 0.70709E+00_JPRB, 0.70527E+00_JPRB, 0.69075E+00_JPRB ,&
&  0.68766E+00_JPRB, 0.68369E+00_JPRB, 0.67781E+00_JPRB, 0.66661E+00_JPRB, 0.65564E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.64562E+00_JPRB, 0.64504E+00_JPRB, 0.65064E+00_JPRB, 0.66571E+00_JPRB, 0.69304E+00_JPRB, 0.71628E+00_JPRB, 0.67721E+00_JPRB ,&
&  0.66443E+00_JPRB, 0.65804E+00_JPRB, 0.68556E+00_JPRB, 0.69681E+00_JPRB, 0.70523E+00_JPRB, 0.71365E+00_JPRB, 0.75270E+00_JPRB ,&
&  0.82440E+00_JPRB, 0.92838E+00_JPRB, 0.10310E+01_JPRB, 0.10990E+01_JPRB, 0.11715E+01_JPRB, 0.12710E+01_JPRB, 0.13964E+01_JPRB ,&
&  0.12603E+01_JPRB, 0.11042E+01_JPRB, 0.14968E+01_JPRB, 0.15525E+01_JPRB, 0.13898E+01_JPRB, 0.12903E+01_JPRB, 0.12704E+01_JPRB ,&
&  0.11207E+01_JPRB, 0.11038E+01_JPRB, 0.11006E+01_JPRB, 0.10430E+01_JPRB, 0.93631E+00_JPRB, 0.91931E+00_JPRB, 0.90836E+00_JPRB ,&
&  0.88966E+00_JPRB, 0.86695E+00_JPRB, 0.85993E+00_JPRB, 0.86427E+00_JPRB, 0.87259E+00_JPRB, 0.86085E+00_JPRB, 0.84301E+00_JPRB ,&
&  0.83631E+00_JPRB, 0.83467E+00_JPRB, 0.84143E+00_JPRB, 0.83907E+00_JPRB, 0.84592E+00_JPRB, 0.86084E+00_JPRB, 0.86970E+00_JPRB ,&
&  0.87562E+00_JPRB, 0.86400E+00_JPRB, 0.87445E+00_JPRB, 0.89051E+00_JPRB, 0.89952E+00_JPRB, 0.82002E+00_JPRB, 0.81057E+00_JPRB ,&
&  0.78686E+00_JPRB, 0.75677E+00_JPRB, 0.73636E+00_JPRB, 0.77023E+00_JPRB, 0.63059E+00_JPRB, 0.69797E+00_JPRB, 0.74064E+00_JPRB ,&
&  0.85060E+00_JPRB, 0.85776E+00_JPRB, 0.86903E+00_JPRB, 0.89790E+00_JPRB, 0.91853E+00_JPRB, 0.85544E+00_JPRB, 0.82392E+00_JPRB ,&
&  0.80621E+00_JPRB, 0.79919E+00_JPRB, 0.77648E+00_JPRB, 0.76588E+00_JPRB, 0.72600E+00_JPRB, 0.72374E+00_JPRB, 0.73648E+00_JPRB ,&
&  0.76910E+00_JPRB, 0.78530E+00_JPRB, 0.79972E+00_JPRB, 0.79494E+00_JPRB, 0.75721E+00_JPRB, 0.73046E+00_JPRB, 0.74000E+00_JPRB ,&
&  0.75293E+00_JPRB, 0.76465E+00_JPRB, 0.71389E+00_JPRB, 0.68233E+00_JPRB, 0.67520E+00_JPRB, 0.68908E+00_JPRB, 0.67607E+00_JPRB ,&
&  0.67367E+00_JPRB, 0.66426E+00_JPRB, 0.66377E+00_JPRB, 0.65807E+00_JPRB, 0.65392E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.64825E+00_JPRB, 0.66695E+00_JPRB, 0.69732E+00_JPRB, 0.73453E+00_JPRB, 0.77323E+00_JPRB, 0.83146E+00_JPRB, 0.79799E+00_JPRB ,&
&  0.76464E+00_JPRB, 0.59952E+00_JPRB, 0.61539E+00_JPRB, 0.62756E+00_JPRB, 0.64352E+00_JPRB, 0.64515E+00_JPRB, 0.66631E+00_JPRB ,&
&  0.73863E+00_JPRB, 0.84599E+00_JPRB, 0.93514E+00_JPRB, 0.10061E+01_JPRB, 0.10795E+01_JPRB, 0.11787E+01_JPRB, 0.11127E+01_JPRB ,&
&  0.88627E+00_JPRB, 0.10801E+01_JPRB, 0.17162E+01_JPRB, 0.16569E+01_JPRB, 0.15270E+01_JPRB, 0.14073E+01_JPRB, 0.12877E+01_JPRB ,&
&  0.11740E+01_JPRB, 0.11189E+01_JPRB, 0.11367E+01_JPRB, 0.10369E+01_JPRB, 0.91665E+00_JPRB, 0.89324E+00_JPRB, 0.88257E+00_JPRB ,&
&  0.85637E+00_JPRB, 0.83182E+00_JPRB, 0.81472E+00_JPRB, 0.81273E+00_JPRB, 0.81108E+00_JPRB, 0.80001E+00_JPRB, 0.78655E+00_JPRB ,&
&  0.78102E+00_JPRB, 0.78223E+00_JPRB, 0.78341E+00_JPRB, 0.78046E+00_JPRB, 0.79976E+00_JPRB, 0.82192E+00_JPRB, 0.83521E+00_JPRB ,&
&  0.84363E+00_JPRB, 0.86592E+00_JPRB, 0.88495E+00_JPRB, 0.88185E+00_JPRB, 0.84306E+00_JPRB, 0.89398E+00_JPRB, 0.84505E+00_JPRB ,&
&  0.82833E+00_JPRB, 0.78648E+00_JPRB, 0.79543E+00_JPRB, 0.69626E+00_JPRB, 0.68888E+00_JPRB, 0.73117E+00_JPRB, 0.77358E+00_JPRB ,&
&  0.80079E+00_JPRB, 0.83133E+00_JPRB, 0.84247E+00_JPRB, 0.88582E+00_JPRB, 0.93162E+00_JPRB, 0.91184E+00_JPRB, 0.89615E+00_JPRB ,&
&  0.83782E+00_JPRB, 0.85945E+00_JPRB, 0.80421E+00_JPRB, 0.75294E+00_JPRB, 0.68818E+00_JPRB, 0.67002E+00_JPRB, 0.69728E+00_JPRB ,&
&  0.75154E+00_JPRB, 0.78764E+00_JPRB, 0.80918E+00_JPRB, 0.82783E+00_JPRB, 0.81597E+00_JPRB, 0.78892E+00_JPRB, 0.73051E+00_JPRB ,&
&  0.71626E+00_JPRB, 0.67840E+00_JPRB, 0.66551E+00_JPRB, 0.66384E+00_JPRB, 0.65336E+00_JPRB, 0.66367E+00_JPRB, 0.65590E+00_JPRB ,&
&  0.64206E+00_JPRB, 0.64128E+00_JPRB, 0.63975E+00_JPRB, 0.63703E+00_JPRB, 0.63841E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.60544E+00_JPRB, 0.62154E+00_JPRB, 0.64895E+00_JPRB, 0.66386E+00_JPRB, 0.65991E+00_JPRB, 0.67754E+00_JPRB, 0.70986E+00_JPRB ,&
&  0.70614E+00_JPRB, 0.56724E+00_JPRB, 0.55004E+00_JPRB, 0.55810E+00_JPRB, 0.57891E+00_JPRB, 0.59225E+00_JPRB, 0.60169E+00_JPRB ,&
&  0.63613E+00_JPRB, 0.72499E+00_JPRB, 0.86135E+00_JPRB, 0.91372E+00_JPRB, 0.95174E+00_JPRB, 0.10551E+01_JPRB, 0.98512E+00_JPRB ,&
&  0.82231E+00_JPRB, 0.14978E+01_JPRB, 0.14928E+01_JPRB, 0.16674E+01_JPRB, 0.16202E+01_JPRB, 0.15032E+01_JPRB, 0.13461E+01_JPRB ,&
&  0.11615E+01_JPRB, 0.10594E+01_JPRB, 0.10353E+01_JPRB, 0.98235E+00_JPRB, 0.93257E+00_JPRB, 0.88184E+00_JPRB, 0.86689E+00_JPRB ,&
&  0.83644E+00_JPRB, 0.79563E+00_JPRB, 0.76573E+00_JPRB, 0.76157E+00_JPRB, 0.75277E+00_JPRB, 0.74334E+00_JPRB, 0.73744E+00_JPRB ,&
&  0.74275E+00_JPRB, 0.74826E+00_JPRB, 0.74711E+00_JPRB, 0.75853E+00_JPRB, 0.77409E+00_JPRB, 0.80202E+00_JPRB, 0.82401E+00_JPRB ,&
&  0.81362E+00_JPRB, 0.80255E+00_JPRB, 0.85985E+00_JPRB, 0.88489E+00_JPRB, 0.89016E+00_JPRB, 0.92793E+00_JPRB, 0.86632E+00_JPRB ,&
&  0.86930E+00_JPRB, 0.80240E+00_JPRB, 0.81228E+00_JPRB, 0.69072E+00_JPRB, 0.69238E+00_JPRB, 0.73296E+00_JPRB, 0.76293E+00_JPRB ,&
&  0.76235E+00_JPRB, 0.82169E+00_JPRB, 0.84359E+00_JPRB, 0.88886E+00_JPRB, 0.93228E+00_JPRB, 0.98323E+00_JPRB, 0.94081E+00_JPRB ,&
&  0.91615E+00_JPRB, 0.91826E+00_JPRB, 0.90686E+00_JPRB, 0.78534E+00_JPRB, 0.71283E+00_JPRB, 0.65374E+00_JPRB, 0.67633E+00_JPRB ,&
&  0.72489E+00_JPRB, 0.75012E+00_JPRB, 0.78328E+00_JPRB, 0.82790E+00_JPRB, 0.84916E+00_JPRB, 0.87415E+00_JPRB, 0.83555E+00_JPRB ,&
&  0.74620E+00_JPRB, 0.70118E+00_JPRB, 0.64597E+00_JPRB, 0.64308E+00_JPRB, 0.64486E+00_JPRB, 0.63451E+00_JPRB, 0.63581E+00_JPRB ,&
&  0.62829E+00_JPRB, 0.61585E+00_JPRB, 0.61026E+00_JPRB, 0.60341E+00_JPRB, 0.59627E+00_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.57049E+00_JPRB, 0.58148E+00_JPRB, 0.59985E+00_JPRB, 0.61072E+00_JPRB, 0.61205E+00_JPRB, 0.60295E+00_JPRB, 0.63124E+00_JPRB ,&
&  0.65712E+00_JPRB, 0.55548E+00_JPRB, 0.51234E+00_JPRB, 0.49761E+00_JPRB, 0.50872E+00_JPRB, 0.53749E+00_JPRB, 0.56239E+00_JPRB ,&
&  0.58474E+00_JPRB, 0.62112E+00_JPRB, 0.75983E+00_JPRB, 0.83694E+00_JPRB, 0.88121E+00_JPRB, 0.10098E+01_JPRB, 0.74232E+00_JPRB ,&
&  0.81667E+00_JPRB, 0.12545E+01_JPRB, 0.14411E+01_JPRB, 0.16263E+01_JPRB, 0.17392E+01_JPRB, 0.16640E+01_JPRB, 0.14730E+01_JPRB ,&
&  0.12276E+01_JPRB, 0.10708E+01_JPRB, 0.98091E+00_JPRB, 0.92455E+00_JPRB, 0.88412E+00_JPRB, 0.85576E+00_JPRB, 0.82496E+00_JPRB ,&
&  0.80588E+00_JPRB, 0.77293E+00_JPRB, 0.73177E+00_JPRB, 0.71943E+00_JPRB, 0.70778E+00_JPRB, 0.68859E+00_JPRB, 0.69551E+00_JPRB ,&
&  0.71225E+00_JPRB, 0.72454E+00_JPRB, 0.73630E+00_JPRB, 0.72533E+00_JPRB, 0.75091E+00_JPRB, 0.77715E+00_JPRB, 0.80721E+00_JPRB ,&
&  0.78656E+00_JPRB, 0.76937E+00_JPRB, 0.86868E+00_JPRB, 0.96299E+00_JPRB, 0.10000E+01_JPRB, 0.95820E+00_JPRB, 0.90042E+00_JPRB ,&
&  0.88562E+00_JPRB, 0.83470E+00_JPRB, 0.77570E+00_JPRB, 0.68590E+00_JPRB, 0.68720E+00_JPRB, 0.73324E+00_JPRB, 0.76826E+00_JPRB ,&
&  0.80475E+00_JPRB, 0.83300E+00_JPRB, 0.86750E+00_JPRB, 0.88232E+00_JPRB, 0.91900E+00_JPRB, 0.99148E+00_JPRB, 0.10009E+01_JPRB ,&
&  0.10782E+01_JPRB, 0.11832E+01_JPRB, 0.10510E+01_JPRB, 0.90118E+00_JPRB, 0.77856E+00_JPRB, 0.67557E+00_JPRB, 0.64931E+00_JPRB ,&
&  0.68190E+00_JPRB, 0.69058E+00_JPRB, 0.69127E+00_JPRB, 0.74410E+00_JPRB, 0.80308E+00_JPRB, 0.93818E+00_JPRB, 0.98242E+00_JPRB ,&
&  0.84548E+00_JPRB, 0.78050E+00_JPRB, 0.70307E+00_JPRB, 0.66488E+00_JPRB, 0.64319E+00_JPRB, 0.64090E+00_JPRB, 0.63406E+00_JPRB ,&
&  0.62894E+00_JPRB, 0.61647E+00_JPRB, 0.59906E+00_JPRB, 0.58691E+00_JPRB, 0.57437E+00_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.56600E+00_JPRB, 0.55759E+00_JPRB, 0.55830E+00_JPRB, 0.56224E+00_JPRB, 0.57642E+00_JPRB, 0.57550E+00_JPRB, 0.57672E+00_JPRB ,&
&  0.61446E+00_JPRB, 0.53142E+00_JPRB, 0.49792E+00_JPRB, 0.47419E+00_JPRB, 0.48060E+00_JPRB, 0.50819E+00_JPRB, 0.53996E+00_JPRB ,&
&  0.56323E+00_JPRB, 0.58908E+00_JPRB, 0.67971E+00_JPRB, 0.78110E+00_JPRB, 0.77586E+00_JPRB, 0.70028E+00_JPRB, 0.72612E+00_JPRB ,&
&  0.93465E+00_JPRB, 0.12263E+01_JPRB, 0.14239E+01_JPRB, 0.16289E+01_JPRB, 0.18662E+01_JPRB, 0.19686E+01_JPRB, 0.16921E+01_JPRB ,&
&  0.14670E+01_JPRB, 0.12218E+01_JPRB, 0.10333E+01_JPRB, 0.92915E+00_JPRB, 0.88060E+00_JPRB, 0.83337E+00_JPRB, 0.79820E+00_JPRB ,&
&  0.76257E+00_JPRB, 0.73177E+00_JPRB, 0.70768E+00_JPRB, 0.68797E+00_JPRB, 0.67987E+00_JPRB, 0.65620E+00_JPRB, 0.65207E+00_JPRB ,&
&  0.66711E+00_JPRB, 0.68422E+00_JPRB, 0.70486E+00_JPRB, 0.69809E+00_JPRB, 0.71987E+00_JPRB, 0.75071E+00_JPRB, 0.80110E+00_JPRB ,&
&  0.82313E+00_JPRB, 0.87079E+00_JPRB, 0.92350E+00_JPRB, 0.95763E+00_JPRB, 0.10163E+01_JPRB, 0.10285E+01_JPRB, 0.97464E+00_JPRB ,&
&  0.91761E+00_JPRB, 0.84041E+00_JPRB, 0.74057E+00_JPRB, 0.71196E+00_JPRB, 0.72704E+00_JPRB, 0.76268E+00_JPRB, 0.69613E+00_JPRB ,&
&  0.67747E+00_JPRB, 0.67660E+00_JPRB, 0.74003E+00_JPRB, 0.82024E+00_JPRB, 0.88068E+00_JPRB, 0.96859E+00_JPRB, 0.11147E+01_JPRB ,&
&  0.11820E+01_JPRB, 0.80352E+00_JPRB, 0.69431E+00_JPRB, 0.77372E+00_JPRB, 0.67602E+00_JPRB, 0.56667E+00_JPRB, 0.59890E+00_JPRB ,&
&  0.62642E+00_JPRB, 0.66548E+00_JPRB, 0.68666E+00_JPRB, 0.65579E+00_JPRB, 0.70950E+00_JPRB, 0.88626E+00_JPRB, 0.11024E+01_JPRB ,&
&  0.91310E+00_JPRB, 0.84822E+00_JPRB, 0.78986E+00_JPRB, 0.73164E+00_JPRB, 0.68457E+00_JPRB, 0.65385E+00_JPRB, 0.63576E+00_JPRB ,&
&  0.62910E+00_JPRB, 0.62360E+00_JPRB, 0.61436E+00_JPRB, 0.59822E+00_JPRB, 0.58057E+00_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.57769E+00_JPRB, 0.56201E+00_JPRB, 0.55195E+00_JPRB, 0.54446E+00_JPRB, 0.54676E+00_JPRB, 0.54563E+00_JPRB, 0.53865E+00_JPRB ,&
&  0.57338E+00_JPRB, 0.52819E+00_JPRB, 0.49078E+00_JPRB, 0.46990E+00_JPRB, 0.47036E+00_JPRB, 0.48765E+00_JPRB, 0.52025E+00_JPRB ,&
&  0.56200E+00_JPRB, 0.59409E+00_JPRB, 0.67841E+00_JPRB, 0.76068E+00_JPRB, 0.73132E+00_JPRB, 0.62579E+00_JPRB, 0.72009E+00_JPRB ,&
&  0.82095E+00_JPRB, 0.11092E+01_JPRB, 0.15850E+01_JPRB, 0.18714E+01_JPRB, 0.19389E+01_JPRB, 0.20261E+01_JPRB, 0.18472E+01_JPRB ,&
&  0.17098E+01_JPRB, 0.14763E+01_JPRB, 0.12008E+01_JPRB, 0.10260E+01_JPRB, 0.90944E+00_JPRB, 0.85616E+00_JPRB, 0.80200E+00_JPRB ,&
&  0.77428E+00_JPRB, 0.72773E+00_JPRB, 0.68738E+00_JPRB, 0.66308E+00_JPRB, 0.64753E+00_JPRB, 0.63678E+00_JPRB, 0.62419E+00_JPRB ,&
&  0.62651E+00_JPRB, 0.63917E+00_JPRB, 0.66601E+00_JPRB, 0.68930E+00_JPRB, 0.62018E+00_JPRB, 0.71526E+00_JPRB, 0.77529E+00_JPRB ,&
&  0.84129E+00_JPRB, 0.90568E+00_JPRB, 0.90450E+00_JPRB, 0.10219E+01_JPRB, 0.10677E+01_JPRB, 0.10271E+01_JPRB, 0.99654E+00_JPRB ,&
&  0.98772E+00_JPRB, 0.88770E+00_JPRB, 0.79417E+00_JPRB, 0.80366E+00_JPRB, 0.79498E+00_JPRB, 0.78948E+00_JPRB, 0.61345E+00_JPRB ,&
&  0.59437E+00_JPRB, 0.66979E+00_JPRB, 0.69911E+00_JPRB, 0.69778E+00_JPRB, 0.77473E+00_JPRB, 0.90078E+00_JPRB, 0.83248E+00_JPRB ,&
&  0.49468E+00_JPRB, 0.43859E+00_JPRB, 0.45877E+00_JPRB, 0.46239E+00_JPRB, 0.44967E+00_JPRB, 0.46091E+00_JPRB, 0.58230E+00_JPRB ,&
&  0.57404E+00_JPRB, 0.62206E+00_JPRB, 0.65038E+00_JPRB, 0.64266E+00_JPRB, 0.65195E+00_JPRB, 0.72602E+00_JPRB, 0.11410E+01_JPRB ,&
&  0.10075E+01_JPRB, 0.88440E+00_JPRB, 0.81565E+00_JPRB, 0.77339E+00_JPRB, 0.72381E+00_JPRB, 0.68085E+00_JPRB, 0.64628E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.61263E+00_JPRB, 0.60859E+00_JPRB, 0.60299E+00_JPRB, 0.59207E+00_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.56823E+00_JPRB, 0.55364E+00_JPRB, 0.54414E+00_JPRB, 0.53722E+00_JPRB, 0.53121E+00_JPRB, 0.53181E+00_JPRB, 0.52155E+00_JPRB ,&
&  0.53456E+00_JPRB, 0.52874E+00_JPRB, 0.49562E+00_JPRB, 0.47384E+00_JPRB, 0.46731E+00_JPRB, 0.48211E+00_JPRB, 0.52216E+00_JPRB ,&
&  0.55386E+00_JPRB, 0.61459E+00_JPRB, 0.75555E+00_JPRB, 0.72094E+00_JPRB, 0.73069E+00_JPRB, 0.63973E+00_JPRB, 0.67942E+00_JPRB ,&
&  0.77063E+00_JPRB, 0.95579E+00_JPRB, 0.13254E+01_JPRB, 0.16353E+01_JPRB, 0.19890E+01_JPRB, 0.21132E+01_JPRB, 0.20585E+01_JPRB ,&
&  0.19060E+01_JPRB, 0.17440E+01_JPRB, 0.14932E+01_JPRB, 0.12345E+01_JPRB, 0.10552E+01_JPRB, 0.93417E+00_JPRB, 0.85730E+00_JPRB ,&
&  0.81703E+00_JPRB, 0.78299E+00_JPRB, 0.70992E+00_JPRB, 0.66690E+00_JPRB, 0.63870E+00_JPRB, 0.62291E+00_JPRB, 0.60027E+00_JPRB ,&
&  0.59352E+00_JPRB, 0.60385E+00_JPRB, 0.62883E+00_JPRB, 0.65507E+00_JPRB, 0.61380E+00_JPRB, 0.58813E+00_JPRB, 0.71099E+00_JPRB ,&
&  0.83826E+00_JPRB, 0.98693E+00_JPRB, 0.10472E+01_JPRB, 0.12366E+01_JPRB, 0.11821E+01_JPRB, 0.10895E+01_JPRB, 0.10874E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.10764E+01_JPRB, 0.88393E+00_JPRB, 0.89668E+00_JPRB, 0.90647E+00_JPRB, 0.72250E+00_JPRB, 0.63161E+00_JPRB ,&
&  0.67941E+00_JPRB, 0.66276E+00_JPRB, 0.65666E+00_JPRB, 0.62150E+00_JPRB, 0.71356E+00_JPRB, 0.74393E+00_JPRB, 0.48767E+00_JPRB ,&
&  0.42923E+00_JPRB, 0.41674E+00_JPRB, 0.40303E+00_JPRB, 0.38766E+00_JPRB, 0.38529E+00_JPRB, 0.39553E+00_JPRB, 0.47915E+00_JPRB ,&
&  0.50182E+00_JPRB, 0.54850E+00_JPRB, 0.58461E+00_JPRB, 0.61725E+00_JPRB, 0.62259E+00_JPRB, 0.68770E+00_JPRB, 0.86236E+00_JPRB ,&
&  0.98863E+00_JPRB, 0.96108E+00_JPRB, 0.87153E+00_JPRB, 0.80327E+00_JPRB, 0.74116E+00_JPRB, 0.69295E+00_JPRB, 0.65839E+00_JPRB ,&
&  0.63290E+00_JPRB, 0.61898E+00_JPRB, 0.60892E+00_JPRB, 0.59993E+00_JPRB, 0.58407E+00_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.57445E+00_JPRB, 0.55781E+00_JPRB, 0.53868E+00_JPRB, 0.52615E+00_JPRB, 0.52113E+00_JPRB, 0.51577E+00_JPRB, 0.50826E+00_JPRB ,&
&  0.50398E+00_JPRB, 0.51827E+00_JPRB, 0.49818E+00_JPRB, 0.48199E+00_JPRB, 0.47013E+00_JPRB, 0.47614E+00_JPRB, 0.50699E+00_JPRB ,&
&  0.55074E+00_JPRB, 0.63973E+00_JPRB, 0.71054E+00_JPRB, 0.65486E+00_JPRB, 0.62229E+00_JPRB, 0.58511E+00_JPRB, 0.57984E+00_JPRB ,&
&  0.71553E+00_JPRB, 0.83844E+00_JPRB, 0.10592E+01_JPRB, 0.14517E+01_JPRB, 0.19374E+01_JPRB, 0.21040E+01_JPRB, 0.22926E+01_JPRB ,&
&  0.21338E+01_JPRB, 0.19231E+01_JPRB, 0.17445E+01_JPRB, 0.15168E+01_JPRB, 0.12629E+01_JPRB, 0.10961E+01_JPRB, 0.98705E+00_JPRB ,&
&  0.91116E+00_JPRB, 0.85472E+00_JPRB, 0.78568E+00_JPRB, 0.71259E+00_JPRB, 0.65503E+00_JPRB, 0.62142E+00_JPRB, 0.59473E+00_JPRB ,&
&  0.57981E+00_JPRB, 0.59004E+00_JPRB, 0.61168E+00_JPRB, 0.64692E+00_JPRB, 0.68577E+00_JPRB, 0.69787E+00_JPRB, 0.71752E+00_JPRB ,&
&  0.81157E+00_JPRB, 0.91236E+00_JPRB, 0.11880E+01_JPRB, 0.13066E+01_JPRB, 0.13092E+01_JPRB, 0.12984E+01_JPRB, 0.12413E+01_JPRB ,&
&  0.12270E+01_JPRB, 0.12566E+01_JPRB, 0.10539E+01_JPRB, 0.10938E+01_JPRB, 0.97920E+00_JPRB, 0.75801E+00_JPRB, 0.75979E+00_JPRB ,&
&  0.77481E+00_JPRB, 0.72617E+00_JPRB, 0.66037E+00_JPRB, 0.56663E+00_JPRB, 0.57020E+00_JPRB, 0.50466E+00_JPRB, 0.39826E+00_JPRB ,&
&  0.37998E+00_JPRB, 0.36858E+00_JPRB, 0.35809E+00_JPRB, 0.35771E+00_JPRB, 0.35340E+00_JPRB, 0.37434E+00_JPRB, 0.42527E+00_JPRB ,&
&  0.47015E+00_JPRB, 0.50151E+00_JPRB, 0.53299E+00_JPRB, 0.56556E+00_JPRB, 0.58711E+00_JPRB, 0.60113E+00_JPRB, 0.79763E+00_JPRB ,&
&  0.81464E+00_JPRB, 0.85069E+00_JPRB, 0.84787E+00_JPRB, 0.80398E+00_JPRB, 0.75014E+00_JPRB, 0.70339E+00_JPRB, 0.66338E+00_JPRB ,&
&  0.63759E+00_JPRB, 0.62629E+00_JPRB, 0.61843E+00_JPRB, 0.60628E+00_JPRB, 0.59157E+00_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.57165E+00_JPRB, 0.55530E+00_JPRB, 0.53975E+00_JPRB, 0.52954E+00_JPRB, 0.52185E+00_JPRB, 0.51450E+00_JPRB, 0.50342E+00_JPRB ,&
&  0.48906E+00_JPRB, 0.49475E+00_JPRB, 0.49232E+00_JPRB, 0.47006E+00_JPRB, 0.46053E+00_JPRB, 0.46938E+00_JPRB, 0.49834E+00_JPRB ,&
&  0.54488E+00_JPRB, 0.69272E+00_JPRB, 0.59378E+00_JPRB, 0.55245E+00_JPRB, 0.57813E+00_JPRB, 0.58351E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.67462E+00_JPRB, 0.79559E+00_JPRB, 0.98075E+00_JPRB, 0.14293E+01_JPRB, 0.22064E+01_JPRB, 0.23453E+01_JPRB, 0.24788E+01_JPRB ,&
&  0.25329E+01_JPRB, 0.22735E+01_JPRB, 0.20429E+01_JPRB, 0.17847E+01_JPRB, 0.14811E+01_JPRB, 0.12514E+01_JPRB, 0.10991E+01_JPRB ,&
&  0.98555E+00_JPRB, 0.91651E+00_JPRB, 0.83160E+00_JPRB, 0.75581E+00_JPRB, 0.69661E+00_JPRB, 0.65104E+00_JPRB, 0.61837E+00_JPRB ,&
&  0.60017E+00_JPRB, 0.59812E+00_JPRB, 0.60989E+00_JPRB, 0.64479E+00_JPRB, 0.68612E+00_JPRB, 0.70412E+00_JPRB, 0.92843E+00_JPRB ,&
&  0.11058E+01_JPRB, 0.11832E+01_JPRB, 0.12278E+01_JPRB, 0.15053E+01_JPRB, 0.16745E+01_JPRB, 0.16398E+01_JPRB, 0.17098E+01_JPRB ,&
&  0.14417E+01_JPRB, 0.13347E+01_JPRB, 0.13108E+01_JPRB, 0.12610E+01_JPRB, 0.93250E+00_JPRB, 0.90387E+00_JPRB, 0.10756E+01_JPRB ,&
&  0.84326E+00_JPRB, 0.82600E+00_JPRB, 0.84044E+00_JPRB, 0.75471E+00_JPRB, 0.44311E+00_JPRB, 0.39965E+00_JPRB, 0.44596E+00_JPRB ,&
&  0.44956E+00_JPRB, 0.37451E+00_JPRB, 0.34602E+00_JPRB, 0.36825E+00_JPRB, 0.34036E+00_JPRB, 0.36682E+00_JPRB, 0.42387E+00_JPRB ,&
&  0.43267E+00_JPRB, 0.44413E+00_JPRB, 0.49406E+00_JPRB, 0.52211E+00_JPRB, 0.55431E+00_JPRB, 0.56566E+00_JPRB, 0.64259E+00_JPRB ,&
&  0.69630E+00_JPRB, 0.83464E+00_JPRB, 0.87575E+00_JPRB, 0.82589E+00_JPRB, 0.75965E+00_JPRB, 0.70188E+00_JPRB, 0.65964E+00_JPRB ,&
&  0.62805E+00_JPRB, 0.61147E+00_JPRB, 0.60280E+00_JPRB, 0.59097E+00_JPRB, 0.58105E+00_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.53647E+00_JPRB, 0.52540E+00_JPRB, 0.51519E+00_JPRB, 0.50694E+00_JPRB, 0.50410E+00_JPRB, 0.50165E+00_JPRB, 0.49719E+00_JPRB ,&
&  0.48621E+00_JPRB, 0.47859E+00_JPRB, 0.48434E+00_JPRB, 0.46540E+00_JPRB, 0.45119E+00_JPRB, 0.45481E+00_JPRB, 0.49256E+00_JPRB ,&
&  0.53679E+00_JPRB, 0.58740E+00_JPRB, 0.49379E+00_JPRB, 0.46594E+00_JPRB, 0.55708E+00_JPRB, 0.54035E+00_JPRB, 0.56339E+00_JPRB ,&
&  0.66147E+00_JPRB, 0.76151E+00_JPRB, 0.92830E+00_JPRB, 0.13172E+01_JPRB, 0.17299E+01_JPRB, 0.21733E+01_JPRB, 0.24929E+01_JPRB ,&
&  0.25985E+01_JPRB, 0.24912E+01_JPRB, 0.22774E+01_JPRB, 0.20293E+01_JPRB, 0.16741E+01_JPRB, 0.14009E+01_JPRB, 0.12100E+01_JPRB ,&
&  0.10658E+01_JPRB, 0.94747E+00_JPRB, 0.85445E+00_JPRB, 0.74247E+00_JPRB, 0.68499E+00_JPRB, 0.64212E+00_JPRB, 0.61397E+00_JPRB ,&
&  0.60255E+00_JPRB, 0.60779E+00_JPRB, 0.61813E+00_JPRB, 0.65546E+00_JPRB, 0.63448E+00_JPRB, 0.66367E+00_JPRB, 0.84220E+00_JPRB ,&
&  0.11943E+01_JPRB, 0.12403E+01_JPRB, 0.13341E+01_JPRB, 0.15669E+01_JPRB, 0.17683E+01_JPRB, 0.17100E+01_JPRB, 0.18231E+01_JPRB ,&
&  0.14245E+01_JPRB, 0.11658E+01_JPRB, 0.99925E+00_JPRB, 0.79708E+00_JPRB, 0.77797E+00_JPRB, 0.10519E+01_JPRB, 0.10608E+01_JPRB ,&
&  0.97545E+00_JPRB, 0.91433E+00_JPRB, 0.85768E+00_JPRB, 0.65985E+00_JPRB, 0.48063E+00_JPRB, 0.40050E+00_JPRB, 0.42645E+00_JPRB ,&
&  0.43982E+00_JPRB, 0.44833E+00_JPRB, 0.42094E+00_JPRB, 0.39619E+00_JPRB, 0.37391E+00_JPRB, 0.41633E+00_JPRB, 0.41960E+00_JPRB ,&
&  0.40776E+00_JPRB, 0.40424E+00_JPRB, 0.43099E+00_JPRB, 0.46527E+00_JPRB, 0.50066E+00_JPRB, 0.49566E+00_JPRB, 0.55433E+00_JPRB ,&
&  0.58039E+00_JPRB, 0.63202E+00_JPRB, 0.66029E+00_JPRB, 0.66102E+00_JPRB, 0.64443E+00_JPRB, 0.61749E+00_JPRB, 0.59442E+00_JPRB ,&
&  0.58057E+00_JPRB, 0.57056E+00_JPRB, 0.56078E+00_JPRB, 0.55205E+00_JPRB, 0.54528E+00_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.49860E+00_JPRB, 0.49449E+00_JPRB, 0.49017E+00_JPRB, 0.48860E+00_JPRB, 0.48695E+00_JPRB, 0.48813E+00_JPRB, 0.48892E+00_JPRB ,&
&  0.48318E+00_JPRB, 0.47102E+00_JPRB, 0.47673E+00_JPRB, 0.46897E+00_JPRB, 0.45477E+00_JPRB, 0.45107E+00_JPRB, 0.47188E+00_JPRB ,&
&  0.51361E+00_JPRB, 0.51269E+00_JPRB, 0.41926E+00_JPRB, 0.43888E+00_JPRB, 0.49014E+00_JPRB, 0.47784E+00_JPRB, 0.52834E+00_JPRB ,&
&  0.60102E+00_JPRB, 0.69721E+00_JPRB, 0.79925E+00_JPRB, 0.10235E+01_JPRB, 0.13014E+01_JPRB, 0.16569E+01_JPRB, 0.19795E+01_JPRB ,&
&  0.20669E+01_JPRB, 0.22144E+01_JPRB, 0.22590E+01_JPRB, 0.20774E+01_JPRB, 0.18394E+01_JPRB, 0.15928E+01_JPRB, 0.13787E+01_JPRB ,&
&  0.12238E+01_JPRB, 0.10903E+01_JPRB, 0.93580E+00_JPRB, 0.81055E+00_JPRB, 0.73067E+00_JPRB, 0.68457E+00_JPRB, 0.65208E+00_JPRB ,&
&  0.63051E+00_JPRB, 0.61628E+00_JPRB, 0.61395E+00_JPRB, 0.65221E+00_JPRB, 0.63325E+00_JPRB, 0.71084E+00_JPRB, 0.88720E+00_JPRB ,&
&  0.13699E+01_JPRB, 0.14490E+01_JPRB, 0.14656E+01_JPRB, 0.17993E+01_JPRB, 0.14681E+01_JPRB, 0.14411E+01_JPRB, 0.16502E+01_JPRB ,&
&  0.17414E+01_JPRB, 0.16585E+01_JPRB, 0.15614E+01_JPRB, 0.11172E+01_JPRB, 0.85143E+00_JPRB, 0.11325E+01_JPRB, 0.10803E+01_JPRB ,&
&  0.97098E+00_JPRB, 0.90305E+00_JPRB, 0.82982E+00_JPRB, 0.78899E+00_JPRB, 0.53760E+00_JPRB, 0.44328E+00_JPRB, 0.39782E+00_JPRB ,&
&  0.40460E+00_JPRB, 0.41011E+00_JPRB, 0.43451E+00_JPRB, 0.43058E+00_JPRB, 0.42299E+00_JPRB, 0.42506E+00_JPRB, 0.42127E+00_JPRB ,&
&  0.42300E+00_JPRB, 0.41600E+00_JPRB, 0.41274E+00_JPRB, 0.43337E+00_JPRB, 0.46442E+00_JPRB, 0.45997E+00_JPRB, 0.48282E+00_JPRB ,&
&  0.49985E+00_JPRB, 0.53092E+00_JPRB, 0.56111E+00_JPRB, 0.56661E+00_JPRB, 0.56574E+00_JPRB, 0.55626E+00_JPRB, 0.53866E+00_JPRB ,&
&  0.53073E+00_JPRB, 0.52744E+00_JPRB, 0.52087E+00_JPRB, 0.51332E+00_JPRB, 0.50558E+00_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.44932E+00_JPRB, 0.44889E+00_JPRB, 0.45128E+00_JPRB, 0.45651E+00_JPRB, 0.46245E+00_JPRB, 0.46797E+00_JPRB, 0.47216E+00_JPRB ,&
&  0.46857E+00_JPRB, 0.45661E+00_JPRB, 0.45686E+00_JPRB, 0.45954E+00_JPRB, 0.44791E+00_JPRB, 0.44454E+00_JPRB, 0.45838E+00_JPRB ,&
&  0.49405E+00_JPRB, 0.53424E+00_JPRB, 0.45131E+00_JPRB, 0.39862E+00_JPRB, 0.38472E+00_JPRB, 0.43314E+00_JPRB, 0.48014E+00_JPRB ,&
&  0.56947E+00_JPRB, 0.64069E+00_JPRB, 0.74020E+00_JPRB, 0.86015E+00_JPRB, 0.10364E+01_JPRB, 0.12619E+01_JPRB, 0.15867E+01_JPRB ,&
&  0.20103E+01_JPRB, 0.18627E+01_JPRB, 0.19820E+01_JPRB, 0.19052E+01_JPRB, 0.18340E+01_JPRB, 0.16534E+01_JPRB, 0.14635E+01_JPRB ,&
&  0.12807E+01_JPRB, 0.11455E+01_JPRB, 0.10056E+01_JPRB, 0.88024E+00_JPRB, 0.78760E+00_JPRB, 0.71936E+00_JPRB, 0.67924E+00_JPRB ,&
&  0.65951E+00_JPRB, 0.63782E+00_JPRB, 0.61534E+00_JPRB, 0.62724E+00_JPRB, 0.72735E+00_JPRB, 0.86796E+00_JPRB, 0.10926E+01_JPRB ,&
&  0.12601E+01_JPRB, 0.12166E+01_JPRB, 0.15859E+01_JPRB, 0.17993E+01_JPRB, 0.18684E+01_JPRB, 0.18320E+01_JPRB, 0.18414E+01_JPRB ,&
&  0.19296E+01_JPRB, 0.18071E+01_JPRB, 0.16625E+01_JPRB, 0.13330E+01_JPRB, 0.13813E+01_JPRB, 0.12773E+01_JPRB, 0.11329E+01_JPRB ,&
&  0.99464E+00_JPRB, 0.92071E+00_JPRB, 0.82528E+00_JPRB, 0.74968E+00_JPRB, 0.67280E+00_JPRB, 0.60992E+00_JPRB, 0.49406E+00_JPRB ,&
&  0.43991E+00_JPRB, 0.45956E+00_JPRB, 0.44566E+00_JPRB, 0.43217E+00_JPRB, 0.41171E+00_JPRB, 0.42382E+00_JPRB, 0.43120E+00_JPRB ,&
&  0.43858E+00_JPRB, 0.43801E+00_JPRB, 0.43319E+00_JPRB, 0.44369E+00_JPRB, 0.45898E+00_JPRB, 0.46643E+00_JPRB, 0.46112E+00_JPRB ,&
&  0.45093E+00_JPRB, 0.45995E+00_JPRB, 0.47060E+00_JPRB, 0.46919E+00_JPRB, 0.50515E+00_JPRB, 0.49758E+00_JPRB, 0.48441E+00_JPRB ,&
&  0.47547E+00_JPRB, 0.47088E+00_JPRB, 0.46654E+00_JPRB, 0.45968E+00_JPRB, 0.45241E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.40735E+00_JPRB, 0.41145E+00_JPRB, 0.41552E+00_JPRB, 0.42227E+00_JPRB, 0.42947E+00_JPRB, 0.44006E+00_JPRB, 0.44765E+00_JPRB ,&
&  0.44961E+00_JPRB, 0.44172E+00_JPRB, 0.43749E+00_JPRB, 0.44494E+00_JPRB, 0.44150E+00_JPRB, 0.43900E+00_JPRB, 0.45076E+00_JPRB ,&
&  0.48584E+00_JPRB, 0.60433E+00_JPRB, 0.50058E+00_JPRB, 0.45971E+00_JPRB, 0.41844E+00_JPRB, 0.44475E+00_JPRB, 0.47587E+00_JPRB ,&
&  0.57407E+00_JPRB, 0.61134E+00_JPRB, 0.63788E+00_JPRB, 0.69325E+00_JPRB, 0.82926E+00_JPRB, 0.10339E+01_JPRB, 0.12783E+01_JPRB ,&
&  0.14138E+01_JPRB, 0.15814E+01_JPRB, 0.16113E+01_JPRB, 0.18028E+01_JPRB, 0.17161E+01_JPRB, 0.14785E+01_JPRB, 0.14348E+01_JPRB ,&
&  0.12816E+01_JPRB, 0.11361E+01_JPRB, 0.99520E+00_JPRB, 0.88649E+00_JPRB, 0.80215E+00_JPRB, 0.73878E+00_JPRB, 0.69107E+00_JPRB ,&
&  0.65580E+00_JPRB, 0.62412E+00_JPRB, 0.58964E+00_JPRB, 0.57213E+00_JPRB, 0.62850E+00_JPRB, 0.86040E+00_JPRB, 0.11700E+01_JPRB ,&
&  0.17035E+01_JPRB, 0.15652E+01_JPRB, 0.13571E+01_JPRB, 0.18060E+01_JPRB, 0.22115E+01_JPRB, 0.21553E+01_JPRB, 0.19956E+01_JPRB ,&
&  0.20991E+01_JPRB, 0.20480E+01_JPRB, 0.18900E+01_JPRB, 0.17652E+01_JPRB, 0.15987E+01_JPRB, 0.14013E+01_JPRB, 0.11916E+01_JPRB ,&
&  0.97947E+00_JPRB, 0.85587E+00_JPRB, 0.79247E+00_JPRB, 0.70315E+00_JPRB, 0.62926E+00_JPRB, 0.57881E+00_JPRB, 0.54433E+00_JPRB ,&
&  0.49999E+00_JPRB, 0.46924E+00_JPRB, 0.41661E+00_JPRB, 0.43094E+00_JPRB, 0.39339E+00_JPRB, 0.40247E+00_JPRB, 0.42647E+00_JPRB ,&
&  0.42953E+00_JPRB, 0.44544E+00_JPRB, 0.44631E+00_JPRB, 0.43827E+00_JPRB, 0.46072E+00_JPRB, 0.45403E+00_JPRB, 0.43821E+00_JPRB ,&
&  0.42041E+00_JPRB, 0.40042E+00_JPRB, 0.40826E+00_JPRB, 0.41314E+00_JPRB, 0.41479E+00_JPRB, 0.41252E+00_JPRB, 0.40943E+00_JPRB ,&
&  0.40787E+00_JPRB, 0.40785E+00_JPRB, 0.40931E+00_JPRB, 0.40675E+00_JPRB, 0.40533E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.37273E+00_JPRB, 0.37664E+00_JPRB, 0.37874E+00_JPRB, 0.38580E+00_JPRB, 0.39565E+00_JPRB, 0.40816E+00_JPRB, 0.42112E+00_JPRB ,&
&  0.43061E+00_JPRB, 0.43058E+00_JPRB, 0.42536E+00_JPRB, 0.42802E+00_JPRB, 0.42903E+00_JPRB, 0.42900E+00_JPRB, 0.43802E+00_JPRB ,&
&  0.47963E+00_JPRB, 0.52208E+00_JPRB, 0.46477E+00_JPRB, 0.41274E+00_JPRB, 0.43045E+00_JPRB, 0.43780E+00_JPRB, 0.46896E+00_JPRB ,&
&  0.49961E+00_JPRB, 0.55154E+00_JPRB, 0.57814E+00_JPRB, 0.60199E+00_JPRB, 0.66096E+00_JPRB, 0.81356E+00_JPRB, 0.10159E+01_JPRB ,&
&  0.11264E+01_JPRB, 0.11956E+01_JPRB, 0.13045E+01_JPRB, 0.14089E+01_JPRB, 0.14288E+01_JPRB, 0.12793E+01_JPRB, 0.12632E+01_JPRB ,&
&  0.11677E+01_JPRB, 0.10492E+01_JPRB, 0.91984E+00_JPRB, 0.81315E+00_JPRB, 0.73828E+00_JPRB, 0.68764E+00_JPRB, 0.65511E+00_JPRB ,&
&  0.63320E+00_JPRB, 0.60724E+00_JPRB, 0.57197E+00_JPRB, 0.56290E+00_JPRB, 0.60631E+00_JPRB, 0.97338E+00_JPRB, 0.16549E+01_JPRB ,&
&  0.21294E+01_JPRB, 0.22219E+01_JPRB, 0.23541E+01_JPRB, 0.25559E+01_JPRB, 0.25980E+01_JPRB, 0.26401E+01_JPRB, 0.26136E+01_JPRB ,&
&  0.24405E+01_JPRB, 0.22545E+01_JPRB, 0.19750E+01_JPRB, 0.16762E+01_JPRB, 0.14654E+01_JPRB, 0.13106E+01_JPRB, 0.11304E+01_JPRB ,&
&  0.92357E+00_JPRB, 0.80271E+00_JPRB, 0.75557E+00_JPRB, 0.68511E+00_JPRB, 0.63974E+00_JPRB, 0.58733E+00_JPRB, 0.57655E+00_JPRB ,&
&  0.52481E+00_JPRB, 0.41129E+00_JPRB, 0.38926E+00_JPRB, 0.41936E+00_JPRB, 0.39280E+00_JPRB, 0.40516E+00_JPRB, 0.42711E+00_JPRB ,&
&  0.42776E+00_JPRB, 0.42936E+00_JPRB, 0.43569E+00_JPRB, 0.42457E+00_JPRB, 0.42983E+00_JPRB, 0.43010E+00_JPRB, 0.40971E+00_JPRB ,&
&  0.39189E+00_JPRB, 0.38501E+00_JPRB, 0.39020E+00_JPRB, 0.38393E+00_JPRB, 0.37588E+00_JPRB, 0.37064E+00_JPRB, 0.36789E+00_JPRB ,&
&  0.36640E+00_JPRB, 0.36774E+00_JPRB, 0.37134E+00_JPRB, 0.36928E+00_JPRB, 0.36809E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.34543E+00_JPRB, 0.34682E+00_JPRB, 0.35104E+00_JPRB, 0.35981E+00_JPRB, 0.37099E+00_JPRB, 0.38555E+00_JPRB, 0.40082E+00_JPRB ,&
&  0.41529E+00_JPRB, 0.42138E+00_JPRB, 0.41622E+00_JPRB, 0.41329E+00_JPRB, 0.41490E+00_JPRB, 0.41796E+00_JPRB, 0.43317E+00_JPRB ,&
&  0.39998E+00_JPRB, 0.41381E+00_JPRB, 0.37872E+00_JPRB, 0.38397E+00_JPRB, 0.42168E+00_JPRB, 0.42845E+00_JPRB, 0.44281E+00_JPRB ,&
&  0.46027E+00_JPRB, 0.49089E+00_JPRB, 0.51195E+00_JPRB, 0.53293E+00_JPRB, 0.56896E+00_JPRB, 0.65638E+00_JPRB, 0.82378E+00_JPRB ,&
&  0.88582E+00_JPRB, 0.87843E+00_JPRB, 0.93576E+00_JPRB, 0.99648E+00_JPRB, 0.99773E+00_JPRB, 0.10488E+01_JPRB, 0.10791E+01_JPRB ,&
&  0.10201E+01_JPRB, 0.94576E+00_JPRB, 0.86469E+00_JPRB, 0.78313E+00_JPRB, 0.71338E+00_JPRB, 0.65399E+00_JPRB, 0.61301E+00_JPRB ,&
&  0.58289E+00_JPRB, 0.56032E+00_JPRB, 0.54299E+00_JPRB, 0.57447E+00_JPRB, 0.63593E+00_JPRB, 0.11828E+01_JPRB, 0.22148E+01_JPRB ,&
&  0.24527E+01_JPRB, 0.24868E+01_JPRB, 0.26714E+01_JPRB, 0.30143E+01_JPRB, 0.29414E+01_JPRB, 0.28174E+01_JPRB, 0.26152E+01_JPRB ,&
&  0.23683E+01_JPRB, 0.20853E+01_JPRB, 0.17857E+01_JPRB, 0.15474E+01_JPRB, 0.12926E+01_JPRB, 0.11468E+01_JPRB, 0.98426E+00_JPRB ,&
&  0.81618E+00_JPRB, 0.73442E+00_JPRB, 0.70834E+00_JPRB, 0.66810E+00_JPRB, 0.63718E+00_JPRB, 0.62892E+00_JPRB, 0.60737E+00_JPRB ,&
&  0.55268E+00_JPRB, 0.49000E+00_JPRB, 0.42711E+00_JPRB, 0.42193E+00_JPRB, 0.39362E+00_JPRB, 0.40710E+00_JPRB, 0.42833E+00_JPRB ,&
&  0.41980E+00_JPRB, 0.40187E+00_JPRB, 0.39365E+00_JPRB, 0.39052E+00_JPRB, 0.39475E+00_JPRB, 0.40500E+00_JPRB, 0.38722E+00_JPRB ,&
&  0.37522E+00_JPRB, 0.37912E+00_JPRB, 0.37944E+00_JPRB, 0.37383E+00_JPRB, 0.36640E+00_JPRB, 0.35758E+00_JPRB, 0.34988E+00_JPRB ,&
&  0.35098E+00_JPRB, 0.35244E+00_JPRB, 0.35028E+00_JPRB, 0.34472E+00_JPRB, 0.34339E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.32722E+00_JPRB, 0.32852E+00_JPRB, 0.33203E+00_JPRB, 0.34183E+00_JPRB, 0.35455E+00_JPRB, 0.36730E+00_JPRB, 0.38230E+00_JPRB ,&
&  0.39812E+00_JPRB, 0.40788E+00_JPRB, 0.40682E+00_JPRB, 0.40188E+00_JPRB, 0.40053E+00_JPRB, 0.40453E+00_JPRB, 0.38601E+00_JPRB ,&
&  0.35002E+00_JPRB, 0.36527E+00_JPRB, 0.35959E+00_JPRB, 0.36597E+00_JPRB, 0.38510E+00_JPRB, 0.39165E+00_JPRB, 0.40063E+00_JPRB ,&
&  0.42004E+00_JPRB, 0.43295E+00_JPRB, 0.44894E+00_JPRB, 0.47018E+00_JPRB, 0.49859E+00_JPRB, 0.56167E+00_JPRB, 0.67168E+00_JPRB ,&
&  0.70759E+00_JPRB, 0.72926E+00_JPRB, 0.74677E+00_JPRB, 0.78427E+00_JPRB, 0.83195E+00_JPRB, 0.86421E+00_JPRB, 0.85232E+00_JPRB ,&
&  0.81591E+00_JPRB, 0.77299E+00_JPRB, 0.74084E+00_JPRB, 0.71141E+00_JPRB, 0.66842E+00_JPRB, 0.62055E+00_JPRB, 0.58217E+00_JPRB ,&
&  0.55079E+00_JPRB, 0.53161E+00_JPRB, 0.53039E+00_JPRB, 0.55206E+00_JPRB, 0.64382E+00_JPRB, 0.10727E+01_JPRB, 0.16831E+01_JPRB ,&
&  0.18896E+01_JPRB, 0.20727E+01_JPRB, 0.21206E+01_JPRB, 0.21673E+01_JPRB, 0.21982E+01_JPRB, 0.21622E+01_JPRB, 0.20606E+01_JPRB ,&
&  0.18872E+01_JPRB, 0.16406E+01_JPRB, 0.14040E+01_JPRB, 0.12431E+01_JPRB, 0.10909E+01_JPRB, 0.97350E+00_JPRB, 0.83695E+00_JPRB ,&
&  0.72028E+00_JPRB, 0.62650E+00_JPRB, 0.62893E+00_JPRB, 0.61749E+00_JPRB, 0.60190E+00_JPRB, 0.59253E+00_JPRB, 0.57525E+00_JPRB ,&
&  0.54881E+00_JPRB, 0.51540E+00_JPRB, 0.46807E+00_JPRB, 0.44030E+00_JPRB, 0.41703E+00_JPRB, 0.42366E+00_JPRB, 0.40069E+00_JPRB ,&
&  0.37331E+00_JPRB, 0.35411E+00_JPRB, 0.35737E+00_JPRB, 0.36190E+00_JPRB, 0.36952E+00_JPRB, 0.37400E+00_JPRB, 0.36780E+00_JPRB ,&
&  0.36658E+00_JPRB, 0.37018E+00_JPRB, 0.36454E+00_JPRB, 0.35867E+00_JPRB, 0.35228E+00_JPRB, 0.34074E+00_JPRB, 0.33344E+00_JPRB ,&
&  0.33179E+00_JPRB, 0.33011E+00_JPRB, 0.32779E+00_JPRB, 0.32393E+00_JPRB, 0.32460E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.32045E+00_JPRB, 0.32779E+00_JPRB, 0.33178E+00_JPRB, 0.33781E+00_JPRB, 0.34666E+00_JPRB, 0.35513E+00_JPRB, 0.36157E+00_JPRB ,&
&  0.38018E+00_JPRB, 0.38883E+00_JPRB, 0.39221E+00_JPRB, 0.39150E+00_JPRB, 0.38708E+00_JPRB, 0.35780E+00_JPRB, 0.31025E+00_JPRB ,&
&  0.30072E+00_JPRB, 0.31304E+00_JPRB, 0.33038E+00_JPRB, 0.34070E+00_JPRB, 0.35066E+00_JPRB, 0.35945E+00_JPRB, 0.36525E+00_JPRB ,&
&  0.37001E+00_JPRB, 0.37621E+00_JPRB, 0.39089E+00_JPRB, 0.41371E+00_JPRB, 0.43870E+00_JPRB, 0.47417E+00_JPRB, 0.53946E+00_JPRB ,&
&  0.56481E+00_JPRB, 0.59894E+00_JPRB, 0.63857E+00_JPRB, 0.61978E+00_JPRB, 0.68985E+00_JPRB, 0.70596E+00_JPRB, 0.70214E+00_JPRB ,&
&  0.69120E+00_JPRB, 0.66447E+00_JPRB, 0.63253E+00_JPRB, 0.60655E+00_JPRB, 0.58281E+00_JPRB, 0.56205E+00_JPRB, 0.53183E+00_JPRB ,&
&  0.50865E+00_JPRB, 0.49398E+00_JPRB, 0.49216E+00_JPRB, 0.50319E+00_JPRB, 0.54972E+00_JPRB, 0.82474E+00_JPRB, 0.10718E+01_JPRB ,&
&  0.12526E+01_JPRB, 0.14094E+01_JPRB, 0.14445E+01_JPRB, 0.14216E+01_JPRB, 0.16199E+01_JPRB, 0.17596E+01_JPRB, 0.16786E+01_JPRB ,&
&  0.15566E+01_JPRB, 0.13339E+01_JPRB, 0.11934E+01_JPRB, 0.10678E+01_JPRB, 0.96263E+00_JPRB, 0.84980E+00_JPRB, 0.73394E+00_JPRB ,&
&  0.64178E+00_JPRB, 0.56632E+00_JPRB, 0.54614E+00_JPRB, 0.53402E+00_JPRB, 0.52200E+00_JPRB, 0.51214E+00_JPRB, 0.50137E+00_JPRB ,&
&  0.49094E+00_JPRB, 0.47407E+00_JPRB, 0.45093E+00_JPRB, 0.42837E+00_JPRB, 0.41000E+00_JPRB, 0.39291E+00_JPRB, 0.37779E+00_JPRB ,&
&  0.36205E+00_JPRB, 0.34460E+00_JPRB, 0.34206E+00_JPRB, 0.34821E+00_JPRB, 0.35180E+00_JPRB, 0.35091E+00_JPRB, 0.35107E+00_JPRB ,&
&  0.34633E+00_JPRB, 0.35392E+00_JPRB, 0.34983E+00_JPRB, 0.34163E+00_JPRB, 0.33075E+00_JPRB, 0.32014E+00_JPRB, 0.31357E+00_JPRB ,&
&  0.30916E+00_JPRB, 0.30794E+00_JPRB, 0.31029E+00_JPRB, 0.31130E+00_JPRB, 0.31384E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.30951E+00_JPRB, 0.32098E+00_JPRB, 0.32704E+00_JPRB, 0.32995E+00_JPRB, 0.33537E+00_JPRB, 0.33830E+00_JPRB, 0.34204E+00_JPRB ,&
&  0.38583E+00_JPRB, 0.36307E+00_JPRB, 0.37527E+00_JPRB, 0.33051E+00_JPRB, 0.31340E+00_JPRB, 0.30847E+00_JPRB, 0.29991E+00_JPRB ,&
&  0.30087E+00_JPRB, 0.30797E+00_JPRB, 0.31509E+00_JPRB, 0.32164E+00_JPRB, 0.33188E+00_JPRB, 0.33693E+00_JPRB, 0.33985E+00_JPRB ,&
&  0.34467E+00_JPRB, 0.35362E+00_JPRB, 0.36738E+00_JPRB, 0.37993E+00_JPRB, 0.39956E+00_JPRB, 0.42779E+00_JPRB, 0.47433E+00_JPRB ,&
&  0.49806E+00_JPRB, 0.54347E+00_JPRB, 0.61607E+00_JPRB, 0.64215E+00_JPRB, 0.65740E+00_JPRB, 0.65108E+00_JPRB, 0.63810E+00_JPRB ,&
&  0.61762E+00_JPRB, 0.53521E+00_JPRB, 0.55241E+00_JPRB, 0.53379E+00_JPRB, 0.51550E+00_JPRB, 0.49634E+00_JPRB, 0.47830E+00_JPRB ,&
&  0.46321E+00_JPRB, 0.45355E+00_JPRB, 0.45137E+00_JPRB, 0.45196E+00_JPRB, 0.46256E+00_JPRB, 0.54109E+00_JPRB, 0.64664E+00_JPRB ,&
&  0.85512E+00_JPRB, 0.81423E+00_JPRB, 0.10386E+01_JPRB, 0.11160E+01_JPRB, 0.13258E+01_JPRB, 0.14117E+01_JPRB, 0.13552E+01_JPRB ,&
&  0.12140E+01_JPRB, 0.10528E+01_JPRB, 0.92698E+00_JPRB, 0.84488E+00_JPRB, 0.80628E+00_JPRB, 0.74316E+00_JPRB, 0.67366E+00_JPRB ,&
&  0.59567E+00_JPRB, 0.51935E+00_JPRB, 0.49963E+00_JPRB, 0.48186E+00_JPRB, 0.46737E+00_JPRB, 0.45623E+00_JPRB, 0.44754E+00_JPRB ,&
&  0.43613E+00_JPRB, 0.42102E+00_JPRB, 0.40288E+00_JPRB, 0.38215E+00_JPRB, 0.37596E+00_JPRB, 0.36746E+00_JPRB, 0.35834E+00_JPRB ,&
&  0.35410E+00_JPRB, 0.34640E+00_JPRB, 0.34346E+00_JPRB, 0.34513E+00_JPRB, 0.34365E+00_JPRB, 0.34361E+00_JPRB, 0.34027E+00_JPRB ,&
&  0.33109E+00_JPRB, 0.33135E+00_JPRB, 0.32696E+00_JPRB, 0.32222E+00_JPRB, 0.31493E+00_JPRB, 0.31088E+00_JPRB, 0.29886E+00_JPRB ,&
&  0.30091E+00_JPRB, 0.30176E+00_JPRB, 0.29976E+00_JPRB, 0.29919E+00_JPRB, 0.30319E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.30654E+00_JPRB, 0.31472E+00_JPRB, 0.31897E+00_JPRB, 0.31955E+00_JPRB, 0.32172E+00_JPRB, 0.32424E+00_JPRB, 0.32135E+00_JPRB ,&
&  0.33840E+00_JPRB, 0.31454E+00_JPRB, 0.30071E+00_JPRB, 0.29550E+00_JPRB, 0.29586E+00_JPRB, 0.29411E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29195E+00_JPRB, 0.29893E+00_JPRB, 0.30782E+00_JPRB, 0.31449E+00_JPRB, 0.32041E+00_JPRB, 0.32450E+00_JPRB, 0.32800E+00_JPRB ,&
&  0.33351E+00_JPRB, 0.34107E+00_JPRB, 0.35093E+00_JPRB, 0.36507E+00_JPRB, 0.38479E+00_JPRB, 0.41116E+00_JPRB, 0.43756E+00_JPRB ,&
&  0.48141E+00_JPRB, 0.54329E+00_JPRB, 0.59492E+00_JPRB, 0.64014E+00_JPRB, 0.65155E+00_JPRB, 0.63373E+00_JPRB, 0.59484E+00_JPRB ,&
&  0.42113E+00_JPRB, 0.39752E+00_JPRB, 0.46396E+00_JPRB, 0.46033E+00_JPRB, 0.45138E+00_JPRB, 0.43870E+00_JPRB, 0.43141E+00_JPRB ,&
&  0.42758E+00_JPRB, 0.42316E+00_JPRB, 0.42207E+00_JPRB, 0.42083E+00_JPRB, 0.42352E+00_JPRB, 0.43895E+00_JPRB, 0.47718E+00_JPRB ,&
&  0.57841E+00_JPRB, 0.59291E+00_JPRB, 0.63616E+00_JPRB, 0.71549E+00_JPRB, 0.96099E+00_JPRB, 0.94736E+00_JPRB, 0.89476E+00_JPRB ,&
&  0.87825E+00_JPRB, 0.81601E+00_JPRB, 0.76734E+00_JPRB, 0.71828E+00_JPRB, 0.68823E+00_JPRB, 0.66048E+00_JPRB, 0.62483E+00_JPRB ,&
&  0.57907E+00_JPRB, 0.52400E+00_JPRB, 0.47301E+00_JPRB, 0.45364E+00_JPRB, 0.44071E+00_JPRB, 0.42736E+00_JPRB, 0.41905E+00_JPRB ,&
&  0.41063E+00_JPRB, 0.39777E+00_JPRB, 0.38043E+00_JPRB, 0.36443E+00_JPRB, 0.35372E+00_JPRB, 0.34834E+00_JPRB, 0.34490E+00_JPRB ,&
&  0.34287E+00_JPRB, 0.34126E+00_JPRB, 0.33998E+00_JPRB, 0.33981E+00_JPRB, 0.33786E+00_JPRB, 0.33199E+00_JPRB, 0.32350E+00_JPRB ,&
&  0.32052E+00_JPRB, 0.30996E+00_JPRB, 0.30693E+00_JPRB, 0.30097E+00_JPRB, 0.29620E+00_JPRB, 0.29385E+00_JPRB, 0.29751E+00_JPRB ,&
&  0.29469E+00_JPRB, 0.28844E+00_JPRB, 0.29062E+00_JPRB, 0.29202E+00_JPRB, 0.29968E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.30412E+00_JPRB, 0.30905E+00_JPRB, 0.31056E+00_JPRB, 0.30790E+00_JPRB, 0.30628E+00_JPRB, 0.30811E+00_JPRB, 0.30978E+00_JPRB ,&
&  0.32020E+00_JPRB, 0.30989E+00_JPRB, 0.29901E+00_JPRB, 0.29070E+00_JPRB, 0.28637E+00_JPRB, 0.28522E+00_JPRB, 0.28797E+00_JPRB ,&
&  0.29274E+00_JPRB, 0.29741E+00_JPRB, 0.30153E+00_JPRB, 0.30584E+00_JPRB, 0.31004E+00_JPRB, 0.31315E+00_JPRB, 0.31685E+00_JPRB ,&
&  0.32237E+00_JPRB, 0.32785E+00_JPRB, 0.33317E+00_JPRB, 0.34322E+00_JPRB, 0.35971E+00_JPRB, 0.38515E+00_JPRB, 0.40448E+00_JPRB ,&
&  0.43135E+00_JPRB, 0.47097E+00_JPRB, 0.52793E+00_JPRB, 0.60325E+00_JPRB, 0.63701E+00_JPRB, 0.60987E+00_JPRB, 0.46378E+00_JPRB ,&
&  0.35784E+00_JPRB, 0.32724E+00_JPRB, 0.33992E+00_JPRB, 0.39116E+00_JPRB, 0.39354E+00_JPRB, 0.39344E+00_JPRB, 0.39468E+00_JPRB ,&
&  0.38180E+00_JPRB, 0.37514E+00_JPRB, 0.38038E+00_JPRB, 0.39388E+00_JPRB, 0.39502E+00_JPRB, 0.39820E+00_JPRB, 0.41729E+00_JPRB ,&
&  0.45506E+00_JPRB, 0.50139E+00_JPRB, 0.55204E+00_JPRB, 0.54270E+00_JPRB, 0.59444E+00_JPRB, 0.77511E+00_JPRB, 0.76855E+00_JPRB ,&
&  0.71528E+00_JPRB, 0.66215E+00_JPRB, 0.64168E+00_JPRB, 0.59568E+00_JPRB, 0.57357E+00_JPRB, 0.56026E+00_JPRB, 0.53593E+00_JPRB ,&
&  0.50612E+00_JPRB, 0.46924E+00_JPRB, 0.42926E+00_JPRB, 0.41195E+00_JPRB, 0.40567E+00_JPRB, 0.40014E+00_JPRB, 0.39819E+00_JPRB ,&
&  0.39259E+00_JPRB, 0.37589E+00_JPRB, 0.35530E+00_JPRB, 0.33854E+00_JPRB, 0.33050E+00_JPRB, 0.32673E+00_JPRB, 0.32520E+00_JPRB ,&
&  0.32428E+00_JPRB, 0.32451E+00_JPRB, 0.32622E+00_JPRB, 0.32329E+00_JPRB, 0.31563E+00_JPRB, 0.30772E+00_JPRB, 0.30555E+00_JPRB ,&
&  0.30481E+00_JPRB, 0.29867E+00_JPRB, 0.29496E+00_JPRB, 0.28837E+00_JPRB, 0.28637E+00_JPRB, 0.28352E+00_JPRB, 0.28276E+00_JPRB ,&
&  0.28714E+00_JPRB, 0.28492E+00_JPRB, 0.28454E+00_JPRB, 0.28594E+00_JPRB, 0.29486E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.30393E+00_JPRB, 0.30915E+00_JPRB, 0.30659E+00_JPRB, 0.30247E+00_JPRB, 0.29390E+00_JPRB, 0.28940E+00_JPRB, 0.28684E+00_JPRB ,&
&  0.28847E+00_JPRB, 0.28762E+00_JPRB, 0.28716E+00_JPRB, 0.28775E+00_JPRB, 0.28894E+00_JPRB, 0.29121E+00_JPRB, 0.29318E+00_JPRB ,&
&  0.29478E+00_JPRB, 0.29702E+00_JPRB, 0.29950E+00_JPRB, 0.30174E+00_JPRB, 0.30418E+00_JPRB, 0.30800E+00_JPRB, 0.31224E+00_JPRB ,&
&  0.31660E+00_JPRB, 0.31833E+00_JPRB, 0.31942E+00_JPRB, 0.32317E+00_JPRB, 0.33284E+00_JPRB, 0.34987E+00_JPRB, 0.37146E+00_JPRB ,&
&  0.39547E+00_JPRB, 0.42053E+00_JPRB, 0.46855E+00_JPRB, 0.54866E+00_JPRB, 0.58927E+00_JPRB, 0.55613E+00_JPRB, 0.43110E+00_JPRB ,&
&  0.34412E+00_JPRB, 0.30026E+00_JPRB, 0.28901E+00_JPRB, 0.29811E+00_JPRB, 0.31995E+00_JPRB, 0.34586E+00_JPRB, 0.35205E+00_JPRB ,&
&  0.35603E+00_JPRB, 0.35831E+00_JPRB, 0.36058E+00_JPRB, 0.36346E+00_JPRB, 0.36498E+00_JPRB, 0.36810E+00_JPRB, 0.38033E+00_JPRB ,&
&  0.40221E+00_JPRB, 0.42801E+00_JPRB, 0.45274E+00_JPRB, 0.46901E+00_JPRB, 0.49013E+00_JPRB, 0.53799E+00_JPRB, 0.57868E+00_JPRB ,&
&  0.55178E+00_JPRB, 0.49382E+00_JPRB, 0.50912E+00_JPRB, 0.51288E+00_JPRB, 0.51446E+00_JPRB, 0.50319E+00_JPRB, 0.48006E+00_JPRB ,&
&  0.45621E+00_JPRB, 0.42432E+00_JPRB, 0.39391E+00_JPRB, 0.37445E+00_JPRB, 0.36944E+00_JPRB, 0.37347E+00_JPRB, 0.37924E+00_JPRB ,&
&  0.38152E+00_JPRB, 0.36717E+00_JPRB, 0.34067E+00_JPRB, 0.32514E+00_JPRB, 0.32004E+00_JPRB, 0.32061E+00_JPRB, 0.32097E+00_JPRB ,&
&  0.31915E+00_JPRB, 0.31756E+00_JPRB, 0.31763E+00_JPRB, 0.31170E+00_JPRB, 0.30604E+00_JPRB, 0.30206E+00_JPRB, 0.29862E+00_JPRB ,&
&  0.29420E+00_JPRB, 0.29004E+00_JPRB, 0.29253E+00_JPRB, 0.28799E+00_JPRB, 0.28538E+00_JPRB, 0.28429E+00_JPRB, 0.28195E+00_JPRB ,&
&  0.28157E+00_JPRB, 0.28324E+00_JPRB, 0.28335E+00_JPRB, 0.28691E+00_JPRB, 0.29372E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.29876E+00_JPRB, 0.30240E+00_JPRB, 0.30471E+00_JPRB, 0.30106E+00_JPRB, 0.29336E+00_JPRB, 0.28762E+00_JPRB, 0.28440E+00_JPRB ,&
&  0.28515E+00_JPRB, 0.28779E+00_JPRB, 0.28770E+00_JPRB, 0.28901E+00_JPRB, 0.29058E+00_JPRB, 0.29241E+00_JPRB, 0.29469E+00_JPRB ,&
&  0.29734E+00_JPRB, 0.29965E+00_JPRB, 0.30086E+00_JPRB, 0.30147E+00_JPRB, 0.30365E+00_JPRB, 0.30734E+00_JPRB, 0.31095E+00_JPRB ,&
&  0.31254E+00_JPRB, 0.30981E+00_JPRB, 0.31061E+00_JPRB, 0.31414E+00_JPRB, 0.32033E+00_JPRB, 0.32849E+00_JPRB, 0.33938E+00_JPRB ,&
&  0.35925E+00_JPRB, 0.38899E+00_JPRB, 0.44138E+00_JPRB, 0.50206E+00_JPRB, 0.52927E+00_JPRB, 0.48929E+00_JPRB, 0.39614E+00_JPRB ,&
&  0.32994E+00_JPRB, 0.29060E+00_JPRB, 0.27459E+00_JPRB, 0.27161E+00_JPRB, 0.27832E+00_JPRB, 0.29288E+00_JPRB, 0.31282E+00_JPRB ,&
&  0.32599E+00_JPRB, 0.33151E+00_JPRB, 0.33927E+00_JPRB, 0.34563E+00_JPRB, 0.35023E+00_JPRB, 0.35289E+00_JPRB, 0.35692E+00_JPRB ,&
&  0.36322E+00_JPRB, 0.37240E+00_JPRB, 0.38849E+00_JPRB, 0.41311E+00_JPRB, 0.44487E+00_JPRB, 0.43613E+00_JPRB, 0.44457E+00_JPRB ,&
&  0.46102E+00_JPRB, 0.43822E+00_JPRB, 0.44822E+00_JPRB, 0.44262E+00_JPRB, 0.43780E+00_JPRB, 0.42501E+00_JPRB, 0.41477E+00_JPRB ,&
&  0.40353E+00_JPRB, 0.38892E+00_JPRB, 0.37196E+00_JPRB, 0.35475E+00_JPRB, 0.34206E+00_JPRB, 0.33713E+00_JPRB, 0.33917E+00_JPRB ,&
&  0.34518E+00_JPRB, 0.34157E+00_JPRB, 0.33670E+00_JPRB, 0.32034E+00_JPRB, 0.31794E+00_JPRB, 0.31907E+00_JPRB, 0.31698E+00_JPRB ,&
&  0.31486E+00_JPRB, 0.31331E+00_JPRB, 0.31148E+00_JPRB, 0.30702E+00_JPRB, 0.30414E+00_JPRB, 0.30172E+00_JPRB, 0.30036E+00_JPRB ,&
&  0.29697E+00_JPRB, 0.29340E+00_JPRB, 0.29112E+00_JPRB, 0.29002E+00_JPRB, 0.28935E+00_JPRB, 0.28978E+00_JPRB, 0.28936E+00_JPRB ,&
&  0.28774E+00_JPRB, 0.28884E+00_JPRB, 0.29177E+00_JPRB, 0.29527E+00_JPRB, 0.29757E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.30458E+00_JPRB, 0.30472E+00_JPRB, 0.30148E+00_JPRB, 0.29936E+00_JPRB, 0.29470E+00_JPRB, 0.29130E+00_JPRB, 0.28767E+00_JPRB ,&
&  0.28604E+00_JPRB, 0.28631E+00_JPRB, 0.28642E+00_JPRB, 0.28805E+00_JPRB, 0.28983E+00_JPRB, 0.29126E+00_JPRB, 0.29347E+00_JPRB ,&
&  0.29553E+00_JPRB, 0.29503E+00_JPRB, 0.29665E+00_JPRB, 0.29933E+00_JPRB, 0.30156E+00_JPRB, 0.30357E+00_JPRB, 0.30442E+00_JPRB ,&
&  0.30269E+00_JPRB, 0.29958E+00_JPRB, 0.30146E+00_JPRB, 0.30597E+00_JPRB, 0.31190E+00_JPRB, 0.32028E+00_JPRB, 0.33701E+00_JPRB ,&
&  0.36033E+00_JPRB, 0.39309E+00_JPRB, 0.42899E+00_JPRB, 0.46175E+00_JPRB, 0.46520E+00_JPRB, 0.43286E+00_JPRB, 0.35389E+00_JPRB ,&
&  0.31263E+00_JPRB, 0.28726E+00_JPRB, 0.27661E+00_JPRB, 0.27230E+00_JPRB, 0.27577E+00_JPRB, 0.28472E+00_JPRB, 0.30018E+00_JPRB ,&
&  0.31045E+00_JPRB, 0.31658E+00_JPRB, 0.32079E+00_JPRB, 0.32434E+00_JPRB, 0.32671E+00_JPRB, 0.32924E+00_JPRB, 0.33096E+00_JPRB ,&
&  0.33492E+00_JPRB, 0.33570E+00_JPRB, 0.33495E+00_JPRB, 0.33654E+00_JPRB, 0.34097E+00_JPRB, 0.34275E+00_JPRB, 0.34612E+00_JPRB ,&
&  0.35816E+00_JPRB, 0.35777E+00_JPRB, 0.35870E+00_JPRB, 0.35809E+00_JPRB, 0.36151E+00_JPRB, 0.35728E+00_JPRB, 0.35386E+00_JPRB ,&
&  0.34894E+00_JPRB, 0.34761E+00_JPRB, 0.34180E+00_JPRB, 0.33585E+00_JPRB, 0.32938E+00_JPRB, 0.32325E+00_JPRB, 0.32001E+00_JPRB ,&
&  0.31713E+00_JPRB, 0.31736E+00_JPRB, 0.31805E+00_JPRB, 0.31385E+00_JPRB, 0.30679E+00_JPRB, 0.29975E+00_JPRB, 0.29458E+00_JPRB ,&
&  0.29155E+00_JPRB, 0.28936E+00_JPRB, 0.28787E+00_JPRB, 0.28701E+00_JPRB, 0.28747E+00_JPRB, 0.28722E+00_JPRB, 0.28778E+00_JPRB ,&
&  0.28654E+00_JPRB, 0.28466E+00_JPRB, 0.28336E+00_JPRB, 0.28389E+00_JPRB, 0.28630E+00_JPRB, 0.28882E+00_JPRB, 0.29044E+00_JPRB ,&
&  0.29179E+00_JPRB, 0.29581E+00_JPRB, 0.30174E+00_JPRB, 0.30546E+00_JPRB, 0.30560E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.30422E+00_JPRB, 0.30444E+00_JPRB, 0.30188E+00_JPRB, 0.29855E+00_JPRB, 0.29444E+00_JPRB, 0.29181E+00_JPRB, 0.28960E+00_JPRB ,&
&  0.28906E+00_JPRB, 0.28913E+00_JPRB, 0.28910E+00_JPRB, 0.28936E+00_JPRB, 0.28989E+00_JPRB, 0.29026E+00_JPRB, 0.29125E+00_JPRB ,&
&  0.29251E+00_JPRB, 0.29419E+00_JPRB, 0.29542E+00_JPRB, 0.29650E+00_JPRB, 0.29728E+00_JPRB, 0.29862E+00_JPRB, 0.29857E+00_JPRB ,&
&  0.29874E+00_JPRB, 0.29620E+00_JPRB, 0.29702E+00_JPRB, 0.30666E+00_JPRB, 0.31610E+00_JPRB, 0.31813E+00_JPRB, 0.33728E+00_JPRB ,&
&  0.36066E+00_JPRB, 0.39246E+00_JPRB, 0.41298E+00_JPRB, 0.42275E+00_JPRB, 0.41750E+00_JPRB, 0.33797E+00_JPRB, 0.32091E+00_JPRB ,&
&  0.30315E+00_JPRB, 0.28956E+00_JPRB, 0.28322E+00_JPRB, 0.27932E+00_JPRB, 0.27857E+00_JPRB, 0.28359E+00_JPRB, 0.29188E+00_JPRB ,&
&  0.29608E+00_JPRB, 0.31296E+00_JPRB, 0.31304E+00_JPRB, 0.31086E+00_JPRB, 0.31032E+00_JPRB, 0.31079E+00_JPRB, 0.30815E+00_JPRB ,&
&  0.30750E+00_JPRB, 0.30496E+00_JPRB, 0.30145E+00_JPRB, 0.30109E+00_JPRB, 0.30033E+00_JPRB, 0.29726E+00_JPRB, 0.29654E+00_JPRB ,&
&  0.29655E+00_JPRB, 0.29536E+00_JPRB, 0.29399E+00_JPRB, 0.29345E+00_JPRB, 0.29481E+00_JPRB, 0.29662E+00_JPRB, 0.30071E+00_JPRB ,&
&  0.29876E+00_JPRB, 0.30275E+00_JPRB, 0.30455E+00_JPRB, 0.30655E+00_JPRB, 0.30818E+00_JPRB, 0.30797E+00_JPRB, 0.30885E+00_JPRB ,&
&  0.30596E+00_JPRB, 0.30432E+00_JPRB, 0.30336E+00_JPRB, 0.30485E+00_JPRB, 0.30256E+00_JPRB, 0.29871E+00_JPRB, 0.29634E+00_JPRB ,&
&  0.29532E+00_JPRB, 0.29441E+00_JPRB, 0.29001E+00_JPRB, 0.28733E+00_JPRB, 0.28511E+00_JPRB, 0.28411E+00_JPRB, 0.28271E+00_JPRB ,&
&  0.28257E+00_JPRB, 0.28218E+00_JPRB, 0.28217E+00_JPRB, 0.28329E+00_JPRB, 0.28777E+00_JPRB, 0.29212E+00_JPRB, 0.29455E+00_JPRB ,&
&  0.29691E+00_JPRB, 0.29994E+00_JPRB, 0.30319E+00_JPRB, 0.30465E+00_JPRB, 0.30385E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.30030E+00_JPRB, 0.29852E+00_JPRB, 0.29687E+00_JPRB, 0.29400E+00_JPRB, 0.29100E+00_JPRB, 0.28901E+00_JPRB, 0.28719E+00_JPRB ,&
&  0.28674E+00_JPRB, 0.28740E+00_JPRB, 0.28847E+00_JPRB, 0.28967E+00_JPRB, 0.29084E+00_JPRB, 0.29168E+00_JPRB, 0.29304E+00_JPRB ,&
&  0.29346E+00_JPRB, 0.29446E+00_JPRB, 0.29489E+00_JPRB, 0.29538E+00_JPRB, 0.29528E+00_JPRB, 0.29521E+00_JPRB, 0.29447E+00_JPRB ,&
&  0.29290E+00_JPRB, 0.29023E+00_JPRB, 0.29387E+00_JPRB, 0.30725E+00_JPRB, 0.31159E+00_JPRB, 0.31869E+00_JPRB, 0.32910E+00_JPRB ,&
&  0.35147E+00_JPRB, 0.34566E+00_JPRB, 0.35370E+00_JPRB, 0.33883E+00_JPRB, 0.32655E+00_JPRB, 0.30896E+00_JPRB, 0.29495E+00_JPRB ,&
&  0.28952E+00_JPRB, 0.28313E+00_JPRB, 0.27984E+00_JPRB, 0.27991E+00_JPRB, 0.28169E+00_JPRB, 0.28714E+00_JPRB, 0.29378E+00_JPRB ,&
&  0.29639E+00_JPRB, 0.30737E+00_JPRB, 0.30750E+00_JPRB, 0.30536E+00_JPRB, 0.30503E+00_JPRB, 0.30411E+00_JPRB, 0.29982E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29391E+00_JPRB, 0.29024E+00_JPRB, 0.28922E+00_JPRB, 0.29091E+00_JPRB, 0.29033E+00_JPRB, 0.28942E+00_JPRB ,&
&  0.28821E+00_JPRB, 0.28554E+00_JPRB, 0.28182E+00_JPRB, 0.27889E+00_JPRB, 0.27684E+00_JPRB, 0.27598E+00_JPRB, 0.27557E+00_JPRB ,&
&  0.27649E+00_JPRB, 0.27896E+00_JPRB, 0.28083E+00_JPRB, 0.28557E+00_JPRB, 0.28983E+00_JPRB, 0.29623E+00_JPRB, 0.30116E+00_JPRB ,&
&  0.30424E+00_JPRB, 0.30641E+00_JPRB, 0.30763E+00_JPRB, 0.30966E+00_JPRB, 0.31173E+00_JPRB, 0.30955E+00_JPRB, 0.30485E+00_JPRB ,&
&  0.30199E+00_JPRB, 0.29762E+00_JPRB, 0.29382E+00_JPRB, 0.28999E+00_JPRB, 0.28670E+00_JPRB, 0.28478E+00_JPRB, 0.28300E+00_JPRB ,&
&  0.28228E+00_JPRB, 0.28163E+00_JPRB, 0.28209E+00_JPRB, 0.28409E+00_JPRB, 0.28895E+00_JPRB, 0.29268E+00_JPRB, 0.29523E+00_JPRB ,&
&  0.29717E+00_JPRB, 0.30007E+00_JPRB, 0.30195E+00_JPRB, 0.30230E+00_JPRB, 0.30159E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.29713E+00_JPRB, 0.29457E+00_JPRB, 0.29229E+00_JPRB, 0.28949E+00_JPRB, 0.28666E+00_JPRB, 0.28393E+00_JPRB, 0.28229E+00_JPRB ,&
&  0.28127E+00_JPRB, 0.28138E+00_JPRB, 0.28231E+00_JPRB, 0.28334E+00_JPRB, 0.28493E+00_JPRB, 0.28626E+00_JPRB, 0.28792E+00_JPRB ,&
&  0.28908E+00_JPRB, 0.28998E+00_JPRB, 0.29072E+00_JPRB, 0.29095E+00_JPRB, 0.29080E+00_JPRB, 0.29035E+00_JPRB, 0.28881E+00_JPRB ,&
&  0.28773E+00_JPRB, 0.28869E+00_JPRB, 0.28723E+00_JPRB, 0.29696E+00_JPRB, 0.29980E+00_JPRB, 0.30225E+00_JPRB, 0.30715E+00_JPRB ,&
&  0.30964E+00_JPRB, 0.31483E+00_JPRB, 0.31673E+00_JPRB, 0.31402E+00_JPRB, 0.31140E+00_JPRB, 0.30138E+00_JPRB, 0.29682E+00_JPRB ,&
&  0.29136E+00_JPRB, 0.28848E+00_JPRB, 0.28818E+00_JPRB, 0.28836E+00_JPRB, 0.29252E+00_JPRB, 0.29572E+00_JPRB, 0.29924E+00_JPRB ,&
&  0.30362E+00_JPRB, 0.30627E+00_JPRB, 0.30875E+00_JPRB, 0.30359E+00_JPRB, 0.30223E+00_JPRB, 0.30027E+00_JPRB, 0.29868E+00_JPRB ,&
&  0.29737E+00_JPRB, 0.29511E+00_JPRB, 0.29366E+00_JPRB, 0.29211E+00_JPRB, 0.29136E+00_JPRB, 0.29149E+00_JPRB, 0.29159E+00_JPRB ,&
&  0.29178E+00_JPRB, 0.29113E+00_JPRB, 0.28959E+00_JPRB, 0.28691E+00_JPRB, 0.28416E+00_JPRB, 0.28170E+00_JPRB, 0.28018E+00_JPRB ,&
&  0.28016E+00_JPRB, 0.28056E+00_JPRB, 0.28190E+00_JPRB, 0.28296E+00_JPRB, 0.28397E+00_JPRB, 0.28527E+00_JPRB, 0.28708E+00_JPRB ,&
&  0.29006E+00_JPRB, 0.29287E+00_JPRB, 0.29680E+00_JPRB, 0.29808E+00_JPRB, 0.30153E+00_JPRB, 0.30559E+00_JPRB, 0.30709E+00_JPRB ,&
&  0.30557E+00_JPRB, 0.30339E+00_JPRB, 0.30070E+00_JPRB, 0.29816E+00_JPRB, 0.29547E+00_JPRB, 0.29229E+00_JPRB, 0.28939E+00_JPRB ,&
&  0.28664E+00_JPRB, 0.28437E+00_JPRB, 0.28354E+00_JPRB, 0.28440E+00_JPRB, 0.28705E+00_JPRB, 0.28954E+00_JPRB, 0.29195E+00_JPRB ,&
&  0.29454E+00_JPRB, 0.29753E+00_JPRB, 0.29981E+00_JPRB, 0.30031E+00_JPRB, 0.29948E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.29092E+00_JPRB, 0.29035E+00_JPRB, 0.28922E+00_JPRB, 0.28755E+00_JPRB, 0.28604E+00_JPRB, 0.28497E+00_JPRB, 0.28437E+00_JPRB ,&
&  0.28360E+00_JPRB, 0.28204E+00_JPRB, 0.28148E+00_JPRB, 0.28175E+00_JPRB, 0.28228E+00_JPRB, 0.28321E+00_JPRB, 0.28437E+00_JPRB ,&
&  0.28558E+00_JPRB, 0.28651E+00_JPRB, 0.28771E+00_JPRB, 0.28847E+00_JPRB, 0.28936E+00_JPRB, 0.28985E+00_JPRB, 0.28907E+00_JPRB ,&
&  0.28846E+00_JPRB, 0.28807E+00_JPRB, 0.28773E+00_JPRB, 0.28838E+00_JPRB, 0.29038E+00_JPRB, 0.28490E+00_JPRB, 0.28740E+00_JPRB ,&
&  0.29598E+00_JPRB, 0.30083E+00_JPRB, 0.30669E+00_JPRB, 0.31019E+00_JPRB, 0.30849E+00_JPRB, 0.30645E+00_JPRB, 0.30621E+00_JPRB ,&
&  0.29760E+00_JPRB, 0.30227E+00_JPRB, 0.30258E+00_JPRB, 0.30284E+00_JPRB, 0.30446E+00_JPRB, 0.30318E+00_JPRB, 0.30398E+00_JPRB ,&
&  0.30988E+00_JPRB, 0.30846E+00_JPRB, 0.30713E+00_JPRB, 0.30503E+00_JPRB, 0.30261E+00_JPRB, 0.30039E+00_JPRB, 0.29844E+00_JPRB ,&
&  0.29635E+00_JPRB, 0.29469E+00_JPRB, 0.29368E+00_JPRB, 0.29194E+00_JPRB, 0.29077E+00_JPRB, 0.29023E+00_JPRB, 0.29008E+00_JPRB ,&
&  0.28947E+00_JPRB, 0.28915E+00_JPRB, 0.28897E+00_JPRB, 0.28825E+00_JPRB, 0.28747E+00_JPRB, 0.28628E+00_JPRB, 0.28546E+00_JPRB ,&
&  0.28446E+00_JPRB, 0.28324E+00_JPRB, 0.28313E+00_JPRB, 0.28294E+00_JPRB, 0.28265E+00_JPRB, 0.28194E+00_JPRB, 0.28227E+00_JPRB ,&
&  0.28249E+00_JPRB, 0.28249E+00_JPRB, 0.28199E+00_JPRB, 0.28170E+00_JPRB, 0.28216E+00_JPRB, 0.28332E+00_JPRB, 0.28602E+00_JPRB ,&
&  0.28819E+00_JPRB, 0.28993E+00_JPRB, 0.29013E+00_JPRB, 0.29004E+00_JPRB, 0.28978E+00_JPRB, 0.28844E+00_JPRB, 0.28668E+00_JPRB ,&
&  0.28463E+00_JPRB, 0.28259E+00_JPRB, 0.28121E+00_JPRB, 0.28020E+00_JPRB, 0.27965E+00_JPRB, 0.27937E+00_JPRB, 0.28020E+00_JPRB ,&
&  0.28264E+00_JPRB, 0.28548E+00_JPRB, 0.28842E+00_JPRB, 0.28994E+00_JPRB, 0.29096E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.28551E+00_JPRB, 0.28590E+00_JPRB, 0.28592E+00_JPRB, 0.28566E+00_JPRB, 0.28506E+00_JPRB, 0.28433E+00_JPRB, 0.28363E+00_JPRB ,&
&  0.28286E+00_JPRB, 0.28216E+00_JPRB, 0.28143E+00_JPRB, 0.28081E+00_JPRB, 0.28035E+00_JPRB, 0.28020E+00_JPRB, 0.28045E+00_JPRB ,&
&  0.28137E+00_JPRB, 0.28257E+00_JPRB, 0.28420E+00_JPRB, 0.28522E+00_JPRB, 0.28615E+00_JPRB, 0.28667E+00_JPRB, 0.28728E+00_JPRB ,&
&  0.28747E+00_JPRB, 0.28740E+00_JPRB, 0.28720E+00_JPRB, 0.28638E+00_JPRB, 0.28590E+00_JPRB, 0.28577E+00_JPRB, 0.28602E+00_JPRB ,&
&  0.28633E+00_JPRB, 0.28656E+00_JPRB, 0.28716E+00_JPRB, 0.28832E+00_JPRB, 0.29142E+00_JPRB, 0.29360E+00_JPRB, 0.29524E+00_JPRB ,&
&  0.29664E+00_JPRB, 0.29677E+00_JPRB, 0.29677E+00_JPRB, 0.29684E+00_JPRB, 0.29704E+00_JPRB, 0.29715E+00_JPRB, 0.29634E+00_JPRB ,&
&  0.29563E+00_JPRB, 0.29491E+00_JPRB, 0.29411E+00_JPRB, 0.29308E+00_JPRB, 0.29212E+00_JPRB, 0.29142E+00_JPRB, 0.28939E+00_JPRB ,&
&  0.28795E+00_JPRB, 0.28736E+00_JPRB, 0.28736E+00_JPRB, 0.28742E+00_JPRB, 0.28763E+00_JPRB, 0.28785E+00_JPRB, 0.28806E+00_JPRB ,&
&  0.28821E+00_JPRB, 0.28880E+00_JPRB, 0.28942E+00_JPRB, 0.28983E+00_JPRB, 0.28997E+00_JPRB, 0.29017E+00_JPRB, 0.28993E+00_JPRB ,&
&  0.28932E+00_JPRB, 0.28861E+00_JPRB, 0.28829E+00_JPRB, 0.28783E+00_JPRB, 0.28711E+00_JPRB, 0.28640E+00_JPRB, 0.28627E+00_JPRB ,&
&  0.28585E+00_JPRB, 0.28525E+00_JPRB, 0.28457E+00_JPRB, 0.28481E+00_JPRB, 0.28491E+00_JPRB, 0.28482E+00_JPRB, 0.28439E+00_JPRB ,&
&  0.28497E+00_JPRB, 0.28580E+00_JPRB, 0.28692E+00_JPRB, 0.28846E+00_JPRB, 0.28936E+00_JPRB, 0.28924E+00_JPRB, 0.28842E+00_JPRB ,&
&  0.28675E+00_JPRB, 0.28474E+00_JPRB, 0.28281E+00_JPRB, 0.28114E+00_JPRB, 0.28000E+00_JPRB, 0.27966E+00_JPRB, 0.27991E+00_JPRB ,&
&  0.28055E+00_JPRB, 0.28130E+00_JPRB, 0.28250E+00_JPRB, 0.28376E+00_JPRB, 0.28496E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.28112E+00_JPRB, 0.28129E+00_JPRB, 0.28138E+00_JPRB, 0.28136E+00_JPRB, 0.28120E+00_JPRB, 0.28095E+00_JPRB, 0.28072E+00_JPRB ,&
&  0.28045E+00_JPRB, 0.28031E+00_JPRB, 0.28004E+00_JPRB, 0.27975E+00_JPRB, 0.27948E+00_JPRB, 0.27928E+00_JPRB, 0.27912E+00_JPRB ,&
&  0.27897E+00_JPRB, 0.27885E+00_JPRB, 0.27894E+00_JPRB, 0.27907E+00_JPRB, 0.27919E+00_JPRB, 0.27934E+00_JPRB, 0.27950E+00_JPRB ,&
&  0.27974E+00_JPRB, 0.27993E+00_JPRB, 0.28009E+00_JPRB, 0.28016E+00_JPRB, 0.28039E+00_JPRB, 0.28045E+00_JPRB, 0.28063E+00_JPRB ,&
&  0.28075E+00_JPRB, 0.28098E+00_JPRB, 0.28116E+00_JPRB, 0.28141E+00_JPRB, 0.28151E+00_JPRB, 0.28182E+00_JPRB, 0.28197E+00_JPRB ,&
&  0.28219E+00_JPRB, 0.28222E+00_JPRB, 0.28223E+00_JPRB, 0.28223E+00_JPRB, 0.28231E+00_JPRB, 0.28221E+00_JPRB, 0.28213E+00_JPRB ,&
&  0.28208E+00_JPRB, 0.28214E+00_JPRB, 0.28220E+00_JPRB, 0.28219E+00_JPRB, 0.28212E+00_JPRB, 0.28212E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28259E+00_JPRB, 0.28336E+00_JPRB, 0.28420E+00_JPRB, 0.28513E+00_JPRB, 0.28611E+00_JPRB, 0.28691E+00_JPRB, 0.28764E+00_JPRB ,&
&  0.28818E+00_JPRB, 0.28867E+00_JPRB, 0.28926E+00_JPRB, 0.28994E+00_JPRB, 0.29044E+00_JPRB, 0.29092E+00_JPRB, 0.29132E+00_JPRB ,&
&  0.29161E+00_JPRB, 0.29155E+00_JPRB, 0.29156E+00_JPRB, 0.29154E+00_JPRB, 0.29157E+00_JPRB, 0.29153E+00_JPRB, 0.29153E+00_JPRB ,&
&  0.29141E+00_JPRB, 0.29140E+00_JPRB, 0.29138E+00_JPRB, 0.29131E+00_JPRB, 0.29114E+00_JPRB, 0.29095E+00_JPRB, 0.29063E+00_JPRB ,&
&  0.29033E+00_JPRB, 0.28996E+00_JPRB, 0.28954E+00_JPRB, 0.28883E+00_JPRB, 0.28795E+00_JPRB, 0.28702E+00_JPRB, 0.28602E+00_JPRB ,&
&  0.28502E+00_JPRB, 0.28388E+00_JPRB, 0.28280E+00_JPRB, 0.28184E+00_JPRB, 0.28094E+00_JPRB, 0.28030E+00_JPRB, 0.27987E+00_JPRB ,&
&  0.27967E+00_JPRB, 0.27972E+00_JPRB, 0.27998E+00_JPRB, 0.28038E+00_JPRB, 0.28081E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB, 0.28203E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1920:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_OBS1920

