!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_OBS1930 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS1930* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB, 0.13164E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.16342E+00_JPRB, 0.16575E+00_JPRB, 0.16729E+00_JPRB, 0.16826E+00_JPRB, 0.16806E+00_JPRB, 0.16897E+00_JPRB, 0.16995E+00_JPRB ,&
&  0.17122E+00_JPRB, 0.17274E+00_JPRB, 0.17376E+00_JPRB, 0.17473E+00_JPRB, 0.17521E+00_JPRB, 0.17519E+00_JPRB, 0.17517E+00_JPRB ,&
&  0.17499E+00_JPRB, 0.17475E+00_JPRB, 0.17213E+00_JPRB, 0.16976E+00_JPRB, 0.16812E+00_JPRB, 0.16729E+00_JPRB, 0.16649E+00_JPRB ,&
&  0.16579E+00_JPRB, 0.16471E+00_JPRB, 0.16333E+00_JPRB, 0.16117E+00_JPRB, 0.15803E+00_JPRB, 0.15499E+00_JPRB, 0.15190E+00_JPRB ,&
&  0.14912E+00_JPRB, 0.14679E+00_JPRB, 0.14464E+00_JPRB, 0.14244E+00_JPRB, 0.14102E+00_JPRB, 0.13944E+00_JPRB, 0.13785E+00_JPRB ,&
&  0.13593E+00_JPRB, 0.13389E+00_JPRB, 0.13192E+00_JPRB, 0.12992E+00_JPRB, 0.12800E+00_JPRB, 0.12608E+00_JPRB, 0.12467E+00_JPRB ,&
&  0.12319E+00_JPRB, 0.12204E+00_JPRB, 0.12079E+00_JPRB, 0.11965E+00_JPRB, 0.11846E+00_JPRB, 0.11733E+00_JPRB, 0.11612E+00_JPRB ,&
&  0.11529E+00_JPRB, 0.11442E+00_JPRB, 0.11355E+00_JPRB, 0.11274E+00_JPRB, 0.11194E+00_JPRB, 0.11116E+00_JPRB, 0.11027E+00_JPRB ,&
&  0.10947E+00_JPRB, 0.10895E+00_JPRB, 0.10841E+00_JPRB, 0.10780E+00_JPRB, 0.10723E+00_JPRB, 0.10683E+00_JPRB, 0.10648E+00_JPRB ,&
&  0.10616E+00_JPRB, 0.10612E+00_JPRB, 0.10626E+00_JPRB, 0.10640E+00_JPRB, 0.10656E+00_JPRB, 0.10690E+00_JPRB, 0.10743E+00_JPRB ,&
&  0.10806E+00_JPRB, 0.10874E+00_JPRB, 0.10962E+00_JPRB, 0.11087E+00_JPRB, 0.11226E+00_JPRB, 0.11385E+00_JPRB, 0.11559E+00_JPRB ,&
&  0.11747E+00_JPRB, 0.11949E+00_JPRB, 0.12167E+00_JPRB, 0.12388E+00_JPRB, 0.12620E+00_JPRB, 0.12855E+00_JPRB, 0.13081E+00_JPRB ,&
&  0.13315E+00_JPRB, 0.13543E+00_JPRB, 0.13770E+00_JPRB, 0.13976E+00_JPRB, 0.14276E+00_JPRB, 0.14471E+00_JPRB, 0.14685E+00_JPRB ,&
&  0.14887E+00_JPRB, 0.15121E+00_JPRB, 0.15358E+00_JPRB, 0.15653E+00_JPRB, 0.15944E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.18975E+00_JPRB, 0.19017E+00_JPRB, 0.19142E+00_JPRB, 0.19849E+00_JPRB, 0.21372E+00_JPRB, 0.29053E+00_JPRB, 0.28246E+00_JPRB ,&
&  0.27578E+00_JPRB, 0.24717E+00_JPRB, 0.24308E+00_JPRB, 0.23564E+00_JPRB, 0.22588E+00_JPRB, 0.21751E+00_JPRB, 0.21235E+00_JPRB ,&
&  0.20688E+00_JPRB, 0.20101E+00_JPRB, 0.19728E+00_JPRB, 0.19384E+00_JPRB, 0.19055E+00_JPRB, 0.18699E+00_JPRB, 0.18429E+00_JPRB ,&
&  0.18208E+00_JPRB, 0.18057E+00_JPRB, 0.17891E+00_JPRB, 0.17580E+00_JPRB, 0.17436E+00_JPRB, 0.17140E+00_JPRB, 0.16694E+00_JPRB ,&
&  0.16121E+00_JPRB, 0.15792E+00_JPRB, 0.15427E+00_JPRB, 0.15096E+00_JPRB, 0.15304E+00_JPRB, 0.15174E+00_JPRB, 0.14943E+00_JPRB ,&
&  0.14602E+00_JPRB, 0.14250E+00_JPRB, 0.13802E+00_JPRB, 0.13377E+00_JPRB, 0.13012E+00_JPRB, 0.12712E+00_JPRB, 0.12447E+00_JPRB ,&
&  0.12235E+00_JPRB, 0.12054E+00_JPRB, 0.11827E+00_JPRB, 0.11621E+00_JPRB, 0.11403E+00_JPRB, 0.11209E+00_JPRB, 0.11021E+00_JPRB ,&
&  0.10871E+00_JPRB, 0.10729E+00_JPRB, 0.10582E+00_JPRB, 0.10428E+00_JPRB, 0.10306E+00_JPRB, 0.10148E+00_JPRB, 0.10028E+00_JPRB ,&
&  0.99202E-01_JPRB, 0.97914E-01_JPRB, 0.96970E-01_JPRB, 0.95799E-01_JPRB, 0.94934E-01_JPRB, 0.94230E-01_JPRB, 0.93543E-01_JPRB ,&
&  0.92607E-01_JPRB, 0.91987E-01_JPRB, 0.91722E-01_JPRB, 0.91502E-01_JPRB, 0.91168E-01_JPRB, 0.91158E-01_JPRB, 0.91400E-01_JPRB ,&
&  0.91761E-01_JPRB, 0.92187E-01_JPRB, 0.92941E-01_JPRB, 0.93855E-01_JPRB, 0.95203E-01_JPRB, 0.96988E-01_JPRB, 0.99840E-01_JPRB ,&
&  0.10295E+00_JPRB, 0.10641E+00_JPRB, 0.11008E+00_JPRB, 0.11446E+00_JPRB, 0.11882E+00_JPRB, 0.12331E+00_JPRB, 0.12847E+00_JPRB ,&
&  0.13405E+00_JPRB, 0.13827E+00_JPRB, 0.14204E+00_JPRB, 0.14604E+00_JPRB, 0.15385E+00_JPRB, 0.15988E+00_JPRB, 0.16469E+00_JPRB ,&
&  0.17002E+00_JPRB, 0.17712E+00_JPRB, 0.18209E+00_JPRB, 0.18567E+00_JPRB, 0.18805E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.44690E+00_JPRB, 0.45825E+00_JPRB, 0.47327E+00_JPRB, 0.48235E+00_JPRB, 0.49185E+00_JPRB, 0.45908E+00_JPRB, 0.43330E+00_JPRB ,&
&  0.38089E+00_JPRB, 0.35107E+00_JPRB, 0.32633E+00_JPRB, 0.30360E+00_JPRB, 0.28346E+00_JPRB, 0.26621E+00_JPRB, 0.24907E+00_JPRB ,&
&  0.23806E+00_JPRB, 0.23124E+00_JPRB, 0.22197E+00_JPRB, 0.21185E+00_JPRB, 0.20116E+00_JPRB, 0.19184E+00_JPRB, 0.18659E+00_JPRB ,&
&  0.18220E+00_JPRB, 0.17902E+00_JPRB, 0.17774E+00_JPRB, 0.17826E+00_JPRB, 0.18027E+00_JPRB, 0.18082E+00_JPRB, 0.18176E+00_JPRB ,&
&  0.17948E+00_JPRB, 0.18642E+00_JPRB, 0.19546E+00_JPRB, 0.19959E+00_JPRB, 0.20697E+00_JPRB, 0.20539E+00_JPRB, 0.20062E+00_JPRB ,&
&  0.19407E+00_JPRB, 0.18594E+00_JPRB, 0.17704E+00_JPRB, 0.16601E+00_JPRB, 0.15554E+00_JPRB, 0.14758E+00_JPRB, 0.13987E+00_JPRB ,&
&  0.13509E+00_JPRB, 0.12976E+00_JPRB, 0.12392E+00_JPRB, 0.11899E+00_JPRB, 0.11472E+00_JPRB, 0.11107E+00_JPRB, 0.10783E+00_JPRB ,&
&  0.10528E+00_JPRB, 0.10365E+00_JPRB, 0.10206E+00_JPRB, 0.10134E+00_JPRB, 0.10013E+00_JPRB, 0.99021E-01_JPRB, 0.97951E-01_JPRB ,&
&  0.97304E-01_JPRB, 0.96179E-01_JPRB, 0.95333E-01_JPRB, 0.94571E-01_JPRB, 0.93465E-01_JPRB, 0.92504E-01_JPRB, 0.91400E-01_JPRB ,&
&  0.90414E-01_JPRB, 0.89850E-01_JPRB, 0.89526E-01_JPRB, 0.89398E-01_JPRB, 0.89562E-01_JPRB, 0.89473E-01_JPRB, 0.89620E-01_JPRB ,&
&  0.89835E-01_JPRB, 0.90173E-01_JPRB, 0.90178E-01_JPRB, 0.90408E-01_JPRB, 0.91132E-01_JPRB, 0.92140E-01_JPRB, 0.94569E-01_JPRB ,&
&  0.97407E-01_JPRB, 0.10109E+00_JPRB, 0.10517E+00_JPRB, 0.11005E+00_JPRB, 0.11498E+00_JPRB, 0.12006E+00_JPRB, 0.12564E+00_JPRB ,&
&  0.13286E+00_JPRB, 0.13934E+00_JPRB, 0.14666E+00_JPRB, 0.15767E+00_JPRB, 0.18000E+00_JPRB, 0.19337E+00_JPRB, 0.20923E+00_JPRB ,&
&  0.23717E+00_JPRB, 0.27682E+00_JPRB, 0.37976E+00_JPRB, 0.41291E+00_JPRB, 0.42996E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.62363E+00_JPRB, 0.65468E+00_JPRB, 0.67946E+00_JPRB, 0.68189E+00_JPRB, 0.68105E+00_JPRB, 0.64396E+00_JPRB, 0.58648E+00_JPRB ,&
&  0.47927E+00_JPRB, 0.40953E+00_JPRB, 0.36596E+00_JPRB, 0.32494E+00_JPRB, 0.28760E+00_JPRB, 0.26280E+00_JPRB, 0.24569E+00_JPRB ,&
&  0.23269E+00_JPRB, 0.22559E+00_JPRB, 0.22015E+00_JPRB, 0.21555E+00_JPRB, 0.21067E+00_JPRB, 0.20595E+00_JPRB, 0.19722E+00_JPRB ,&
&  0.19206E+00_JPRB, 0.18841E+00_JPRB, 0.18714E+00_JPRB, 0.18960E+00_JPRB, 0.19646E+00_JPRB, 0.20385E+00_JPRB, 0.21466E+00_JPRB ,&
&  0.22881E+00_JPRB, 0.24232E+00_JPRB, 0.24838E+00_JPRB, 0.25141E+00_JPRB, 0.25427E+00_JPRB, 0.25220E+00_JPRB, 0.22924E+00_JPRB ,&
&  0.23445E+00_JPRB, 0.25833E+00_JPRB, 0.27679E+00_JPRB, 0.28823E+00_JPRB, 0.24506E+00_JPRB, 0.20391E+00_JPRB, 0.16352E+00_JPRB ,&
&  0.14372E+00_JPRB, 0.12888E+00_JPRB, 0.12133E+00_JPRB, 0.11576E+00_JPRB, 0.11196E+00_JPRB, 0.10946E+00_JPRB, 0.10725E+00_JPRB ,&
&  0.10513E+00_JPRB, 0.10407E+00_JPRB, 0.10258E+00_JPRB, 0.10178E+00_JPRB, 0.10072E+00_JPRB, 0.10043E+00_JPRB, 0.99228E-01_JPRB ,&
&  0.97998E-01_JPRB, 0.97055E-01_JPRB, 0.96470E-01_JPRB, 0.96181E-01_JPRB, 0.95507E-01_JPRB, 0.94821E-01_JPRB, 0.93301E-01_JPRB ,&
&  0.92178E-01_JPRB, 0.91044E-01_JPRB, 0.90164E-01_JPRB, 0.90080E-01_JPRB, 0.90384E-01_JPRB, 0.91728E-01_JPRB, 0.92481E-01_JPRB ,&
&  0.92638E-01_JPRB, 0.92671E-01_JPRB, 0.92722E-01_JPRB, 0.93331E-01_JPRB, 0.94269E-01_JPRB, 0.95929E-01_JPRB, 0.98240E-01_JPRB ,&
&  0.10102E+00_JPRB, 0.10522E+00_JPRB, 0.10959E+00_JPRB, 0.11454E+00_JPRB, 0.11931E+00_JPRB, 0.12423E+00_JPRB, 0.13047E+00_JPRB ,&
&  0.13695E+00_JPRB, 0.14430E+00_JPRB, 0.15492E+00_JPRB, 0.16655E+00_JPRB, 0.18722E+00_JPRB, 0.20670E+00_JPRB, 0.23928E+00_JPRB ,&
&  0.31090E+00_JPRB, 0.45903E+00_JPRB, 0.54397E+00_JPRB, 0.55369E+00_JPRB, 0.58623E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.74389E+00_JPRB, 0.76231E+00_JPRB, 0.77882E+00_JPRB, 0.77624E+00_JPRB, 0.75905E+00_JPRB, 0.70136E+00_JPRB, 0.63799E+00_JPRB ,&
&  0.55931E+00_JPRB, 0.49285E+00_JPRB, 0.42368E+00_JPRB, 0.36347E+00_JPRB, 0.31325E+00_JPRB, 0.27475E+00_JPRB, 0.25244E+00_JPRB ,&
&  0.23687E+00_JPRB, 0.23014E+00_JPRB, 0.22802E+00_JPRB, 0.23265E+00_JPRB, 0.23536E+00_JPRB, 0.24080E+00_JPRB, 0.23529E+00_JPRB ,&
&  0.22842E+00_JPRB, 0.22019E+00_JPRB, 0.21827E+00_JPRB, 0.22705E+00_JPRB, 0.24000E+00_JPRB, 0.25264E+00_JPRB, 0.26896E+00_JPRB ,&
&  0.28716E+00_JPRB, 0.29295E+00_JPRB, 0.30655E+00_JPRB, 0.31037E+00_JPRB, 0.29550E+00_JPRB, 0.29702E+00_JPRB, 0.30285E+00_JPRB ,&
&  0.31593E+00_JPRB, 0.33451E+00_JPRB, 0.34524E+00_JPRB, 0.36568E+00_JPRB, 0.31410E+00_JPRB, 0.25398E+00_JPRB, 0.20281E+00_JPRB ,&
&  0.16493E+00_JPRB, 0.14354E+00_JPRB, 0.12911E+00_JPRB, 0.12171E+00_JPRB, 0.11497E+00_JPRB, 0.11126E+00_JPRB, 0.10723E+00_JPRB ,&
&  0.10543E+00_JPRB, 0.10521E+00_JPRB, 0.10459E+00_JPRB, 0.10409E+00_JPRB, 0.10423E+00_JPRB, 0.10460E+00_JPRB, 0.10394E+00_JPRB ,&
&  0.10311E+00_JPRB, 0.10276E+00_JPRB, 0.10229E+00_JPRB, 0.10199E+00_JPRB, 0.10125E+00_JPRB, 0.10113E+00_JPRB, 0.99886E-01_JPRB ,&
&  0.99240E-01_JPRB, 0.98067E-01_JPRB, 0.97370E-01_JPRB, 0.97193E-01_JPRB, 0.97440E-01_JPRB, 0.97712E-01_JPRB, 0.98176E-01_JPRB ,&
&  0.99138E-01_JPRB, 0.10002E+00_JPRB, 0.10154E+00_JPRB, 0.10313E+00_JPRB, 0.10562E+00_JPRB, 0.10800E+00_JPRB, 0.11134E+00_JPRB ,&
&  0.11488E+00_JPRB, 0.11916E+00_JPRB, 0.12395E+00_JPRB, 0.12930E+00_JPRB, 0.13411E+00_JPRB, 0.13887E+00_JPRB, 0.14412E+00_JPRB ,&
&  0.14835E+00_JPRB, 0.15491E+00_JPRB, 0.16232E+00_JPRB, 0.17460E+00_JPRB, 0.20017E+00_JPRB, 0.22414E+00_JPRB, 0.27346E+00_JPRB ,&
&  0.35899E+00_JPRB, 0.61455E+00_JPRB, 0.84479E+00_JPRB, 0.69467E+00_JPRB, 0.72551E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.75202E+00_JPRB, 0.76466E+00_JPRB, 0.78188E+00_JPRB, 0.75835E+00_JPRB, 0.74297E+00_JPRB, 0.74636E+00_JPRB, 0.74717E+00_JPRB ,&
&  0.72424E+00_JPRB, 0.68959E+00_JPRB, 0.64089E+00_JPRB, 0.60128E+00_JPRB, 0.53510E+00_JPRB, 0.45035E+00_JPRB, 0.41279E+00_JPRB ,&
&  0.37890E+00_JPRB, 0.37551E+00_JPRB, 0.37793E+00_JPRB, 0.39391E+00_JPRB, 0.40283E+00_JPRB, 0.39941E+00_JPRB, 0.37800E+00_JPRB ,&
&  0.35099E+00_JPRB, 0.32367E+00_JPRB, 0.30627E+00_JPRB, 0.29219E+00_JPRB, 0.29190E+00_JPRB, 0.29012E+00_JPRB, 0.28360E+00_JPRB ,&
&  0.27474E+00_JPRB, 0.27862E+00_JPRB, 0.29570E+00_JPRB, 0.32471E+00_JPRB, 0.37520E+00_JPRB, 0.35520E+00_JPRB, 0.36071E+00_JPRB ,&
&  0.39053E+00_JPRB, 0.41199E+00_JPRB, 0.42053E+00_JPRB, 0.42361E+00_JPRB, 0.41725E+00_JPRB, 0.44306E+00_JPRB, 0.38658E+00_JPRB ,&
&  0.27926E+00_JPRB, 0.22052E+00_JPRB, 0.17885E+00_JPRB, 0.15252E+00_JPRB, 0.13551E+00_JPRB, 0.12578E+00_JPRB, 0.11533E+00_JPRB ,&
&  0.11062E+00_JPRB, 0.10727E+00_JPRB, 0.10638E+00_JPRB, 0.10750E+00_JPRB, 0.10964E+00_JPRB, 0.11237E+00_JPRB, 0.11330E+00_JPRB ,&
&  0.11479E+00_JPRB, 0.11483E+00_JPRB, 0.11401E+00_JPRB, 0.11373E+00_JPRB, 0.11281E+00_JPRB, 0.11342E+00_JPRB, 0.11350E+00_JPRB ,&
&  0.11376E+00_JPRB, 0.11402E+00_JPRB, 0.11554E+00_JPRB, 0.11592E+00_JPRB, 0.11677E+00_JPRB, 0.11391E+00_JPRB, 0.11203E+00_JPRB ,&
&  0.11196E+00_JPRB, 0.11373E+00_JPRB, 0.11677E+00_JPRB, 0.12033E+00_JPRB, 0.12521E+00_JPRB, 0.12921E+00_JPRB, 0.13378E+00_JPRB ,&
&  0.13779E+00_JPRB, 0.14170E+00_JPRB, 0.14552E+00_JPRB, 0.14943E+00_JPRB, 0.15355E+00_JPRB, 0.15769E+00_JPRB, 0.16207E+00_JPRB ,&
&  0.16652E+00_JPRB, 0.17435E+00_JPRB, 0.18406E+00_JPRB, 0.20111E+00_JPRB, 0.22425E+00_JPRB, 0.25172E+00_JPRB, 0.29958E+00_JPRB ,&
&  0.36171E+00_JPRB, 0.59175E+00_JPRB, 0.88937E+00_JPRB, 0.71479E+00_JPRB, 0.73290E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.80138E+00_JPRB, 0.79942E+00_JPRB, 0.80768E+00_JPRB, 0.79736E+00_JPRB, 0.77825E+00_JPRB, 0.76912E+00_JPRB, 0.78607E+00_JPRB ,&
&  0.80587E+00_JPRB, 0.77658E+00_JPRB, 0.75513E+00_JPRB, 0.73601E+00_JPRB, 0.72262E+00_JPRB, 0.70901E+00_JPRB, 0.68664E+00_JPRB ,&
&  0.66654E+00_JPRB, 0.65926E+00_JPRB, 0.65750E+00_JPRB, 0.65251E+00_JPRB, 0.64225E+00_JPRB, 0.63218E+00_JPRB, 0.61442E+00_JPRB ,&
&  0.58213E+00_JPRB, 0.54831E+00_JPRB, 0.53028E+00_JPRB, 0.50672E+00_JPRB, 0.48987E+00_JPRB, 0.47049E+00_JPRB, 0.44281E+00_JPRB ,&
&  0.39837E+00_JPRB, 0.34035E+00_JPRB, 0.30299E+00_JPRB, 0.33887E+00_JPRB, 0.43545E+00_JPRB, 0.42997E+00_JPRB, 0.44853E+00_JPRB ,&
&  0.48303E+00_JPRB, 0.50429E+00_JPRB, 0.51462E+00_JPRB, 0.51787E+00_JPRB, 0.52689E+00_JPRB, 0.53201E+00_JPRB, 0.52846E+00_JPRB ,&
&  0.51428E+00_JPRB, 0.46216E+00_JPRB, 0.42037E+00_JPRB, 0.31847E+00_JPRB, 0.24655E+00_JPRB, 0.20884E+00_JPRB, 0.18043E+00_JPRB ,&
&  0.16231E+00_JPRB, 0.15137E+00_JPRB, 0.14891E+00_JPRB, 0.14940E+00_JPRB, 0.15453E+00_JPRB, 0.15929E+00_JPRB, 0.15924E+00_JPRB ,&
&  0.15586E+00_JPRB, 0.14858E+00_JPRB, 0.13774E+00_JPRB, 0.13100E+00_JPRB, 0.12765E+00_JPRB, 0.12783E+00_JPRB, 0.12977E+00_JPRB ,&
&  0.13171E+00_JPRB, 0.13636E+00_JPRB, 0.14873E+00_JPRB, 0.15986E+00_JPRB, 0.17331E+00_JPRB, 0.15459E+00_JPRB, 0.13973E+00_JPRB ,&
&  0.13474E+00_JPRB, 0.13851E+00_JPRB, 0.14455E+00_JPRB, 0.14916E+00_JPRB, 0.15408E+00_JPRB, 0.15844E+00_JPRB, 0.16289E+00_JPRB ,&
&  0.16702E+00_JPRB, 0.16992E+00_JPRB, 0.17156E+00_JPRB, 0.17328E+00_JPRB, 0.17488E+00_JPRB, 0.17722E+00_JPRB, 0.18020E+00_JPRB ,&
&  0.18522E+00_JPRB, 0.19423E+00_JPRB, 0.20606E+00_JPRB, 0.22552E+00_JPRB, 0.24381E+00_JPRB, 0.26218E+00_JPRB, 0.28303E+00_JPRB ,&
&  0.30440E+00_JPRB, 0.38397E+00_JPRB, 0.57806E+00_JPRB, 0.90044E+00_JPRB, 0.89370E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.79471E+00_JPRB, 0.81474E+00_JPRB, 0.82581E+00_JPRB, 0.83989E+00_JPRB, 0.84012E+00_JPRB, 0.80980E+00_JPRB, 0.79499E+00_JPRB ,&
&  0.79967E+00_JPRB, 0.79492E+00_JPRB, 0.78439E+00_JPRB, 0.77808E+00_JPRB, 0.76925E+00_JPRB, 0.75937E+00_JPRB, 0.75078E+00_JPRB ,&
&  0.74246E+00_JPRB, 0.73757E+00_JPRB, 0.74473E+00_JPRB, 0.75890E+00_JPRB, 0.77061E+00_JPRB, 0.77713E+00_JPRB, 0.78217E+00_JPRB ,&
&  0.78162E+00_JPRB, 0.74842E+00_JPRB, 0.70339E+00_JPRB, 0.67045E+00_JPRB, 0.65363E+00_JPRB, 0.63969E+00_JPRB, 0.60695E+00_JPRB ,&
&  0.54074E+00_JPRB, 0.41902E+00_JPRB, 0.36150E+00_JPRB, 0.40219E+00_JPRB, 0.52261E+00_JPRB, 0.48637E+00_JPRB, 0.50647E+00_JPRB ,&
&  0.53665E+00_JPRB, 0.55225E+00_JPRB, 0.54804E+00_JPRB, 0.53760E+00_JPRB, 0.53416E+00_JPRB, 0.53768E+00_JPRB, 0.53410E+00_JPRB ,&
&  0.53039E+00_JPRB, 0.51422E+00_JPRB, 0.50470E+00_JPRB, 0.49442E+00_JPRB, 0.47734E+00_JPRB, 0.45179E+00_JPRB, 0.43662E+00_JPRB ,&
&  0.39922E+00_JPRB, 0.36231E+00_JPRB, 0.35197E+00_JPRB, 0.35342E+00_JPRB, 0.32295E+00_JPRB, 0.31075E+00_JPRB, 0.29660E+00_JPRB ,&
&  0.25604E+00_JPRB, 0.23458E+00_JPRB, 0.20196E+00_JPRB, 0.18133E+00_JPRB, 0.16608E+00_JPRB, 0.15273E+00_JPRB, 0.15092E+00_JPRB ,&
&  0.15521E+00_JPRB, 0.16700E+00_JPRB, 0.18884E+00_JPRB, 0.22437E+00_JPRB, 0.25720E+00_JPRB, 0.21107E+00_JPRB, 0.19208E+00_JPRB ,&
&  0.17843E+00_JPRB, 0.17543E+00_JPRB, 0.18026E+00_JPRB, 0.18609E+00_JPRB, 0.19225E+00_JPRB, 0.19958E+00_JPRB, 0.20643E+00_JPRB ,&
&  0.21141E+00_JPRB, 0.21450E+00_JPRB, 0.21565E+00_JPRB, 0.21510E+00_JPRB, 0.21450E+00_JPRB, 0.21562E+00_JPRB, 0.21919E+00_JPRB ,&
&  0.22497E+00_JPRB, 0.23689E+00_JPRB, 0.26055E+00_JPRB, 0.29557E+00_JPRB, 0.34171E+00_JPRB, 0.40186E+00_JPRB, 0.49447E+00_JPRB ,&
&  0.61963E+00_JPRB, 0.78445E+00_JPRB, 0.82765E+00_JPRB, 0.83653E+00_JPRB, 0.80620E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.83303E+00_JPRB, 0.84374E+00_JPRB, 0.84164E+00_JPRB, 0.84064E+00_JPRB, 0.84250E+00_JPRB, 0.83550E+00_JPRB, 0.82113E+00_JPRB ,&
&  0.80685E+00_JPRB, 0.78036E+00_JPRB, 0.75016E+00_JPRB, 0.73674E+00_JPRB, 0.74455E+00_JPRB, 0.74464E+00_JPRB, 0.72590E+00_JPRB ,&
&  0.72410E+00_JPRB, 0.73802E+00_JPRB, 0.76109E+00_JPRB, 0.78687E+00_JPRB, 0.82171E+00_JPRB, 0.85785E+00_JPRB, 0.88497E+00_JPRB ,&
&  0.89560E+00_JPRB, 0.89347E+00_JPRB, 0.88567E+00_JPRB, 0.85884E+00_JPRB, 0.82311E+00_JPRB, 0.79570E+00_JPRB, 0.77490E+00_JPRB ,&
&  0.74930E+00_JPRB, 0.70234E+00_JPRB, 0.63225E+00_JPRB, 0.62678E+00_JPRB, 0.56192E+00_JPRB, 0.54070E+00_JPRB, 0.56259E+00_JPRB ,&
&  0.57794E+00_JPRB, 0.55999E+00_JPRB, 0.53642E+00_JPRB, 0.51685E+00_JPRB, 0.51326E+00_JPRB, 0.51196E+00_JPRB, 0.50687E+00_JPRB ,&
&  0.50293E+00_JPRB, 0.49617E+00_JPRB, 0.48213E+00_JPRB, 0.46911E+00_JPRB, 0.45686E+00_JPRB, 0.44464E+00_JPRB, 0.43646E+00_JPRB ,&
&  0.40896E+00_JPRB, 0.36675E+00_JPRB, 0.33549E+00_JPRB, 0.32221E+00_JPRB, 0.31499E+00_JPRB, 0.31874E+00_JPRB, 0.33113E+00_JPRB ,&
&  0.34083E+00_JPRB, 0.34091E+00_JPRB, 0.33510E+00_JPRB, 0.32307E+00_JPRB, 0.30545E+00_JPRB, 0.25923E+00_JPRB, 0.23324E+00_JPRB ,&
&  0.24844E+00_JPRB, 0.28299E+00_JPRB, 0.31866E+00_JPRB, 0.32606E+00_JPRB, 0.33576E+00_JPRB, 0.33856E+00_JPRB, 0.32443E+00_JPRB ,&
&  0.29682E+00_JPRB, 0.26452E+00_JPRB, 0.25764E+00_JPRB, 0.25959E+00_JPRB, 0.26336E+00_JPRB, 0.27085E+00_JPRB, 0.28646E+00_JPRB ,&
&  0.31021E+00_JPRB, 0.34940E+00_JPRB, 0.37128E+00_JPRB, 0.37384E+00_JPRB, 0.36652E+00_JPRB, 0.35481E+00_JPRB, 0.35304E+00_JPRB ,&
&  0.36668E+00_JPRB, 0.42793E+00_JPRB, 0.55033E+00_JPRB, 0.69881E+00_JPRB, 0.69660E+00_JPRB, 0.74027E+00_JPRB, 0.77508E+00_JPRB ,&
&  0.80473E+00_JPRB, 0.81793E+00_JPRB, 0.80444E+00_JPRB, 0.80595E+00_JPRB, 0.81287E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.84551E+00_JPRB, 0.85023E+00_JPRB, 0.84093E+00_JPRB, 0.83688E+00_JPRB, 0.82638E+00_JPRB, 0.81295E+00_JPRB, 0.79056E+00_JPRB ,&
&  0.76406E+00_JPRB, 0.73579E+00_JPRB, 0.70929E+00_JPRB, 0.68763E+00_JPRB, 0.69179E+00_JPRB, 0.72459E+00_JPRB, 0.75216E+00_JPRB ,&
&  0.74110E+00_JPRB, 0.73713E+00_JPRB, 0.75430E+00_JPRB, 0.78503E+00_JPRB, 0.81857E+00_JPRB, 0.84958E+00_JPRB, 0.87215E+00_JPRB ,&
&  0.89058E+00_JPRB, 0.87236E+00_JPRB, 0.87566E+00_JPRB, 0.87731E+00_JPRB, 0.87098E+00_JPRB, 0.85299E+00_JPRB, 0.82418E+00_JPRB ,&
&  0.79548E+00_JPRB, 0.76827E+00_JPRB, 0.73451E+00_JPRB, 0.67485E+00_JPRB, 0.74809E+00_JPRB, 0.67004E+00_JPRB, 0.64100E+00_JPRB ,&
&  0.61396E+00_JPRB, 0.59602E+00_JPRB, 0.57591E+00_JPRB, 0.55404E+00_JPRB, 0.54649E+00_JPRB, 0.54177E+00_JPRB, 0.53379E+00_JPRB ,&
&  0.52726E+00_JPRB, 0.52047E+00_JPRB, 0.51019E+00_JPRB, 0.49679E+00_JPRB, 0.48412E+00_JPRB, 0.47048E+00_JPRB, 0.45799E+00_JPRB ,&
&  0.44006E+00_JPRB, 0.41394E+00_JPRB, 0.38651E+00_JPRB, 0.36655E+00_JPRB, 0.35633E+00_JPRB, 0.36549E+00_JPRB, 0.39740E+00_JPRB ,&
&  0.43033E+00_JPRB, 0.43579E+00_JPRB, 0.44522E+00_JPRB, 0.44756E+00_JPRB, 0.43898E+00_JPRB, 0.42929E+00_JPRB, 0.41186E+00_JPRB ,&
&  0.39811E+00_JPRB, 0.39553E+00_JPRB, 0.39623E+00_JPRB, 0.39949E+00_JPRB, 0.40481E+00_JPRB, 0.41235E+00_JPRB, 0.42289E+00_JPRB ,&
&  0.43264E+00_JPRB, 0.43586E+00_JPRB, 0.44798E+00_JPRB, 0.47463E+00_JPRB, 0.50521E+00_JPRB, 0.53614E+00_JPRB, 0.56228E+00_JPRB ,&
&  0.57974E+00_JPRB, 0.59169E+00_JPRB, 0.59971E+00_JPRB, 0.61479E+00_JPRB, 0.63035E+00_JPRB, 0.64624E+00_JPRB, 0.65459E+00_JPRB ,&
&  0.65230E+00_JPRB, 0.65566E+00_JPRB, 0.67224E+00_JPRB, 0.70016E+00_JPRB, 0.71982E+00_JPRB, 0.75360E+00_JPRB, 0.77473E+00_JPRB ,&
&  0.79724E+00_JPRB, 0.80317E+00_JPRB, 0.79788E+00_JPRB, 0.81053E+00_JPRB, 0.82754E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.80433E+00_JPRB, 0.82234E+00_JPRB, 0.83477E+00_JPRB, 0.83609E+00_JPRB, 0.82093E+00_JPRB, 0.79435E+00_JPRB, 0.77144E+00_JPRB ,&
&  0.74526E+00_JPRB, 0.71981E+00_JPRB, 0.69563E+00_JPRB, 0.66291E+00_JPRB, 0.64528E+00_JPRB, 0.64857E+00_JPRB, 0.69746E+00_JPRB ,&
&  0.76260E+00_JPRB, 0.76742E+00_JPRB, 0.76140E+00_JPRB, 0.78685E+00_JPRB, 0.83971E+00_JPRB, 0.87180E+00_JPRB, 0.89305E+00_JPRB ,&
&  0.89682E+00_JPRB, 0.86431E+00_JPRB, 0.83100E+00_JPRB, 0.81701E+00_JPRB, 0.83990E+00_JPRB, 0.85744E+00_JPRB, 0.86249E+00_JPRB ,&
&  0.85783E+00_JPRB, 0.83598E+00_JPRB, 0.81592E+00_JPRB, 0.79876E+00_JPRB, 0.80211E+00_JPRB, 0.77681E+00_JPRB, 0.72430E+00_JPRB ,&
&  0.65802E+00_JPRB, 0.63178E+00_JPRB, 0.62092E+00_JPRB, 0.60668E+00_JPRB, 0.59576E+00_JPRB, 0.59566E+00_JPRB, 0.58617E+00_JPRB ,&
&  0.57265E+00_JPRB, 0.55726E+00_JPRB, 0.54270E+00_JPRB, 0.52533E+00_JPRB, 0.51178E+00_JPRB, 0.49266E+00_JPRB, 0.47478E+00_JPRB ,&
&  0.45821E+00_JPRB, 0.43813E+00_JPRB, 0.41981E+00_JPRB, 0.40729E+00_JPRB, 0.40207E+00_JPRB, 0.41214E+00_JPRB, 0.45144E+00_JPRB ,&
&  0.48252E+00_JPRB, 0.47943E+00_JPRB, 0.47682E+00_JPRB, 0.48028E+00_JPRB, 0.47265E+00_JPRB, 0.46173E+00_JPRB, 0.45582E+00_JPRB ,&
&  0.45068E+00_JPRB, 0.44653E+00_JPRB, 0.44451E+00_JPRB, 0.44626E+00_JPRB, 0.44840E+00_JPRB, 0.44662E+00_JPRB, 0.44940E+00_JPRB ,&
&  0.45452E+00_JPRB, 0.45990E+00_JPRB, 0.47694E+00_JPRB, 0.50139E+00_JPRB, 0.52171E+00_JPRB, 0.53578E+00_JPRB, 0.54875E+00_JPRB ,&
&  0.56920E+00_JPRB, 0.58177E+00_JPRB, 0.56893E+00_JPRB, 0.55659E+00_JPRB, 0.54995E+00_JPRB, 0.55485E+00_JPRB, 0.56524E+00_JPRB ,&
&  0.58597E+00_JPRB, 0.59908E+00_JPRB, 0.60528E+00_JPRB, 0.60686E+00_JPRB, 0.61027E+00_JPRB, 0.61121E+00_JPRB, 0.62375E+00_JPRB ,&
&  0.65735E+00_JPRB, 0.68640E+00_JPRB, 0.69403E+00_JPRB, 0.72697E+00_JPRB, 0.76910E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.73505E+00_JPRB, 0.76703E+00_JPRB, 0.79190E+00_JPRB, 0.80161E+00_JPRB, 0.80016E+00_JPRB, 0.79160E+00_JPRB, 0.76895E+00_JPRB ,&
&  0.74430E+00_JPRB, 0.72668E+00_JPRB, 0.70794E+00_JPRB, 0.68124E+00_JPRB, 0.65905E+00_JPRB, 0.65429E+00_JPRB, 0.66670E+00_JPRB ,&
&  0.71288E+00_JPRB, 0.75392E+00_JPRB, 0.77066E+00_JPRB, 0.78462E+00_JPRB, 0.83490E+00_JPRB, 0.87717E+00_JPRB, 0.88681E+00_JPRB ,&
&  0.88272E+00_JPRB, 0.88499E+00_JPRB, 0.84867E+00_JPRB, 0.81536E+00_JPRB, 0.81512E+00_JPRB, 0.82011E+00_JPRB, 0.80160E+00_JPRB ,&
&  0.82724E+00_JPRB, 0.84385E+00_JPRB, 0.83655E+00_JPRB, 0.80950E+00_JPRB, 0.80052E+00_JPRB, 0.78975E+00_JPRB, 0.77803E+00_JPRB ,&
&  0.75618E+00_JPRB, 0.70807E+00_JPRB, 0.67919E+00_JPRB, 0.67833E+00_JPRB, 0.67132E+00_JPRB, 0.66370E+00_JPRB, 0.64976E+00_JPRB ,&
&  0.60968E+00_JPRB, 0.57396E+00_JPRB, 0.54735E+00_JPRB, 0.52910E+00_JPRB, 0.51784E+00_JPRB, 0.50924E+00_JPRB, 0.50044E+00_JPRB ,&
&  0.49295E+00_JPRB, 0.46696E+00_JPRB, 0.44037E+00_JPRB, 0.43570E+00_JPRB, 0.44244E+00_JPRB, 0.44922E+00_JPRB, 0.47552E+00_JPRB ,&
&  0.50044E+00_JPRB, 0.49881E+00_JPRB, 0.49497E+00_JPRB, 0.49682E+00_JPRB, 0.50498E+00_JPRB, 0.50443E+00_JPRB, 0.49381E+00_JPRB ,&
&  0.48834E+00_JPRB, 0.48556E+00_JPRB, 0.47602E+00_JPRB, 0.46917E+00_JPRB, 0.46853E+00_JPRB, 0.47279E+00_JPRB, 0.48768E+00_JPRB ,&
&  0.49691E+00_JPRB, 0.49844E+00_JPRB, 0.51221E+00_JPRB, 0.52952E+00_JPRB, 0.53975E+00_JPRB, 0.54765E+00_JPRB, 0.55308E+00_JPRB ,&
&  0.56622E+00_JPRB, 0.58556E+00_JPRB, 0.60054E+00_JPRB, 0.60248E+00_JPRB, 0.57936E+00_JPRB, 0.55497E+00_JPRB, 0.53576E+00_JPRB ,&
&  0.52761E+00_JPRB, 0.53225E+00_JPRB, 0.53970E+00_JPRB, 0.54667E+00_JPRB, 0.55306E+00_JPRB, 0.55716E+00_JPRB, 0.55617E+00_JPRB ,&
&  0.57558E+00_JPRB, 0.60676E+00_JPRB, 0.63221E+00_JPRB, 0.66272E+00_JPRB, 0.69445E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.69329E+00_JPRB, 0.70765E+00_JPRB, 0.71937E+00_JPRB, 0.74089E+00_JPRB, 0.76068E+00_JPRB, 0.75828E+00_JPRB, 0.75104E+00_JPRB ,&
&  0.74548E+00_JPRB, 0.74028E+00_JPRB, 0.73260E+00_JPRB, 0.72260E+00_JPRB, 0.71587E+00_JPRB, 0.71522E+00_JPRB, 0.71617E+00_JPRB ,&
&  0.71809E+00_JPRB, 0.73851E+00_JPRB, 0.77204E+00_JPRB, 0.79851E+00_JPRB, 0.82736E+00_JPRB, 0.87131E+00_JPRB, 0.89227E+00_JPRB ,&
&  0.87650E+00_JPRB, 0.87618E+00_JPRB, 0.86550E+00_JPRB, 0.85100E+00_JPRB, 0.85274E+00_JPRB, 0.84327E+00_JPRB, 0.81489E+00_JPRB ,&
&  0.80867E+00_JPRB, 0.80093E+00_JPRB, 0.79691E+00_JPRB, 0.82344E+00_JPRB, 0.84331E+00_JPRB, 0.84132E+00_JPRB, 0.83549E+00_JPRB ,&
&  0.81399E+00_JPRB, 0.77703E+00_JPRB, 0.74384E+00_JPRB, 0.72866E+00_JPRB, 0.72730E+00_JPRB, 0.70720E+00_JPRB, 0.67922E+00_JPRB ,&
&  0.62186E+00_JPRB, 0.57487E+00_JPRB, 0.54902E+00_JPRB, 0.53387E+00_JPRB, 0.52941E+00_JPRB, 0.52735E+00_JPRB, 0.52361E+00_JPRB ,&
&  0.52949E+00_JPRB, 0.52502E+00_JPRB, 0.49858E+00_JPRB, 0.47123E+00_JPRB, 0.46761E+00_JPRB, 0.47658E+00_JPRB, 0.49540E+00_JPRB ,&
&  0.51049E+00_JPRB, 0.50672E+00_JPRB, 0.51361E+00_JPRB, 0.52075E+00_JPRB, 0.53709E+00_JPRB, 0.55235E+00_JPRB, 0.55264E+00_JPRB ,&
&  0.55178E+00_JPRB, 0.54605E+00_JPRB, 0.53255E+00_JPRB, 0.52418E+00_JPRB, 0.51340E+00_JPRB, 0.50963E+00_JPRB, 0.52164E+00_JPRB ,&
&  0.53923E+00_JPRB, 0.54897E+00_JPRB, 0.56018E+00_JPRB, 0.57363E+00_JPRB, 0.58381E+00_JPRB, 0.59189E+00_JPRB, 0.59868E+00_JPRB ,&
&  0.60898E+00_JPRB, 0.62352E+00_JPRB, 0.63812E+00_JPRB, 0.65078E+00_JPRB, 0.65061E+00_JPRB, 0.63650E+00_JPRB, 0.61125E+00_JPRB ,&
&  0.59424E+00_JPRB, 0.58077E+00_JPRB, 0.57717E+00_JPRB, 0.58170E+00_JPRB, 0.57629E+00_JPRB, 0.55661E+00_JPRB, 0.54699E+00_JPRB ,&
&  0.55808E+00_JPRB, 0.58197E+00_JPRB, 0.60500E+00_JPRB, 0.63635E+00_JPRB, 0.66933E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.64693E+00_JPRB, 0.65888E+00_JPRB, 0.67226E+00_JPRB, 0.68520E+00_JPRB, 0.69845E+00_JPRB, 0.69810E+00_JPRB, 0.69908E+00_JPRB ,&
&  0.71759E+00_JPRB, 0.71986E+00_JPRB, 0.71854E+00_JPRB, 0.70971E+00_JPRB, 0.70338E+00_JPRB, 0.70352E+00_JPRB, 0.69506E+00_JPRB ,&
&  0.69128E+00_JPRB, 0.69849E+00_JPRB, 0.71806E+00_JPRB, 0.74744E+00_JPRB, 0.77601E+00_JPRB, 0.81300E+00_JPRB, 0.84025E+00_JPRB ,&
&  0.84496E+00_JPRB, 0.83470E+00_JPRB, 0.83497E+00_JPRB, 0.83820E+00_JPRB, 0.83852E+00_JPRB, 0.82167E+00_JPRB, 0.81706E+00_JPRB ,&
&  0.79771E+00_JPRB, 0.80317E+00_JPRB, 0.67336E+00_JPRB, 0.74135E+00_JPRB, 0.78374E+00_JPRB, 0.81577E+00_JPRB, 0.82125E+00_JPRB ,&
&  0.81268E+00_JPRB, 0.80031E+00_JPRB, 0.77912E+00_JPRB, 0.75503E+00_JPRB, 0.74617E+00_JPRB, 0.72600E+00_JPRB, 0.70340E+00_JPRB ,&
&  0.65511E+00_JPRB, 0.59362E+00_JPRB, 0.56240E+00_JPRB, 0.55117E+00_JPRB, 0.54603E+00_JPRB, 0.54822E+00_JPRB, 0.55466E+00_JPRB ,&
&  0.55932E+00_JPRB, 0.56072E+00_JPRB, 0.55050E+00_JPRB, 0.53508E+00_JPRB, 0.51735E+00_JPRB, 0.50815E+00_JPRB, 0.50534E+00_JPRB ,&
&  0.50166E+00_JPRB, 0.48961E+00_JPRB, 0.49297E+00_JPRB, 0.52240E+00_JPRB, 0.54681E+00_JPRB, 0.56278E+00_JPRB, 0.56839E+00_JPRB ,&
&  0.57203E+00_JPRB, 0.56478E+00_JPRB, 0.56026E+00_JPRB, 0.55147E+00_JPRB, 0.52917E+00_JPRB, 0.52359E+00_JPRB, 0.54468E+00_JPRB ,&
&  0.56819E+00_JPRB, 0.58158E+00_JPRB, 0.58944E+00_JPRB, 0.60121E+00_JPRB, 0.61320E+00_JPRB, 0.61992E+00_JPRB, 0.62635E+00_JPRB ,&
&  0.63156E+00_JPRB, 0.63646E+00_JPRB, 0.65002E+00_JPRB, 0.66457E+00_JPRB, 0.67118E+00_JPRB, 0.67205E+00_JPRB, 0.67139E+00_JPRB ,&
&  0.66780E+00_JPRB, 0.65976E+00_JPRB, 0.65065E+00_JPRB, 0.64149E+00_JPRB, 0.62080E+00_JPRB, 0.59573E+00_JPRB, 0.57799E+00_JPRB ,&
&  0.57016E+00_JPRB, 0.58119E+00_JPRB, 0.58954E+00_JPRB, 0.60254E+00_JPRB, 0.62961E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.64780E+00_JPRB, 0.64512E+00_JPRB, 0.64456E+00_JPRB, 0.65526E+00_JPRB, 0.67195E+00_JPRB, 0.67535E+00_JPRB, 0.67579E+00_JPRB ,&
&  0.68699E+00_JPRB, 0.69533E+00_JPRB, 0.69452E+00_JPRB, 0.69320E+00_JPRB, 0.68776E+00_JPRB, 0.67500E+00_JPRB, 0.66569E+00_JPRB ,&
&  0.67237E+00_JPRB, 0.67956E+00_JPRB, 0.68376E+00_JPRB, 0.70956E+00_JPRB, 0.74287E+00_JPRB, 0.78178E+00_JPRB, 0.80831E+00_JPRB ,&
&  0.81811E+00_JPRB, 0.81124E+00_JPRB, 0.80776E+00_JPRB, 0.80249E+00_JPRB, 0.79714E+00_JPRB, 0.79883E+00_JPRB, 0.79323E+00_JPRB ,&
&  0.78431E+00_JPRB, 0.68929E+00_JPRB, 0.71761E+00_JPRB, 0.72564E+00_JPRB, 0.73933E+00_JPRB, 0.76075E+00_JPRB, 0.78287E+00_JPRB ,&
&  0.78017E+00_JPRB, 0.77646E+00_JPRB, 0.76903E+00_JPRB, 0.75177E+00_JPRB, 0.73551E+00_JPRB, 0.72597E+00_JPRB, 0.71007E+00_JPRB ,&
&  0.68341E+00_JPRB, 0.64701E+00_JPRB, 0.61459E+00_JPRB, 0.59282E+00_JPRB, 0.58332E+00_JPRB, 0.58187E+00_JPRB, 0.57906E+00_JPRB ,&
&  0.57893E+00_JPRB, 0.57906E+00_JPRB, 0.56989E+00_JPRB, 0.56161E+00_JPRB, 0.55328E+00_JPRB, 0.54846E+00_JPRB, 0.53665E+00_JPRB ,&
&  0.52890E+00_JPRB, 0.51855E+00_JPRB, 0.51818E+00_JPRB, 0.53761E+00_JPRB, 0.55718E+00_JPRB, 0.55995E+00_JPRB, 0.57100E+00_JPRB ,&
&  0.58556E+00_JPRB, 0.59076E+00_JPRB, 0.59118E+00_JPRB, 0.58786E+00_JPRB, 0.58034E+00_JPRB, 0.57981E+00_JPRB, 0.58712E+00_JPRB ,&
&  0.59314E+00_JPRB, 0.60224E+00_JPRB, 0.60482E+00_JPRB, 0.60886E+00_JPRB, 0.61509E+00_JPRB, 0.61982E+00_JPRB, 0.63105E+00_JPRB ,&
&  0.63692E+00_JPRB, 0.64123E+00_JPRB, 0.65303E+00_JPRB, 0.66456E+00_JPRB, 0.66896E+00_JPRB, 0.67078E+00_JPRB, 0.67896E+00_JPRB ,&
&  0.68140E+00_JPRB, 0.67858E+00_JPRB, 0.67018E+00_JPRB, 0.65758E+00_JPRB, 0.64757E+00_JPRB, 0.63325E+00_JPRB, 0.61345E+00_JPRB ,&
&  0.59827E+00_JPRB, 0.59756E+00_JPRB, 0.60810E+00_JPRB, 0.61944E+00_JPRB, 0.63757E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.64261E+00_JPRB, 0.63586E+00_JPRB, 0.62996E+00_JPRB, 0.63692E+00_JPRB, 0.65008E+00_JPRB, 0.66015E+00_JPRB, 0.65955E+00_JPRB ,&
&  0.65923E+00_JPRB, 0.66404E+00_JPRB, 0.67261E+00_JPRB, 0.67585E+00_JPRB, 0.67067E+00_JPRB, 0.65645E+00_JPRB, 0.65068E+00_JPRB ,&
&  0.65176E+00_JPRB, 0.65592E+00_JPRB, 0.65837E+00_JPRB, 0.66490E+00_JPRB, 0.68655E+00_JPRB, 0.71894E+00_JPRB, 0.74941E+00_JPRB ,&
&  0.76474E+00_JPRB, 0.75992E+00_JPRB, 0.75409E+00_JPRB, 0.74901E+00_JPRB, 0.74760E+00_JPRB, 0.75221E+00_JPRB, 0.75266E+00_JPRB ,&
&  0.74551E+00_JPRB, 0.60013E+00_JPRB, 0.65268E+00_JPRB, 0.67025E+00_JPRB, 0.69016E+00_JPRB, 0.71365E+00_JPRB, 0.74074E+00_JPRB ,&
&  0.74133E+00_JPRB, 0.73347E+00_JPRB, 0.71807E+00_JPRB, 0.70824E+00_JPRB, 0.69549E+00_JPRB, 0.69344E+00_JPRB, 0.69291E+00_JPRB ,&
&  0.69208E+00_JPRB, 0.68968E+00_JPRB, 0.66804E+00_JPRB, 0.63321E+00_JPRB, 0.61488E+00_JPRB, 0.60497E+00_JPRB, 0.59733E+00_JPRB ,&
&  0.59045E+00_JPRB, 0.58044E+00_JPRB, 0.57037E+00_JPRB, 0.56188E+00_JPRB, 0.56470E+00_JPRB, 0.56428E+00_JPRB, 0.55297E+00_JPRB ,&
&  0.54887E+00_JPRB, 0.54761E+00_JPRB, 0.55552E+00_JPRB, 0.56520E+00_JPRB, 0.56834E+00_JPRB, 0.56197E+00_JPRB, 0.56908E+00_JPRB ,&
&  0.58702E+00_JPRB, 0.59775E+00_JPRB, 0.60718E+00_JPRB, 0.62026E+00_JPRB, 0.62607E+00_JPRB, 0.62371E+00_JPRB, 0.62276E+00_JPRB ,&
&  0.62680E+00_JPRB, 0.62393E+00_JPRB, 0.61604E+00_JPRB, 0.60984E+00_JPRB, 0.61087E+00_JPRB, 0.61173E+00_JPRB, 0.62217E+00_JPRB ,&
&  0.63175E+00_JPRB, 0.63485E+00_JPRB, 0.63101E+00_JPRB, 0.63380E+00_JPRB, 0.63944E+00_JPRB, 0.64827E+00_JPRB, 0.65401E+00_JPRB ,&
&  0.66005E+00_JPRB, 0.65826E+00_JPRB, 0.64630E+00_JPRB, 0.64531E+00_JPRB, 0.64998E+00_JPRB, 0.64602E+00_JPRB, 0.63481E+00_JPRB ,&
&  0.62627E+00_JPRB, 0.61923E+00_JPRB, 0.62366E+00_JPRB, 0.62840E+00_JPRB, 0.63616E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.66237E+00_JPRB, 0.66035E+00_JPRB, 0.65391E+00_JPRB, 0.65348E+00_JPRB, 0.65413E+00_JPRB, 0.65172E+00_JPRB, 0.65077E+00_JPRB ,&
&  0.64918E+00_JPRB, 0.64703E+00_JPRB, 0.65652E+00_JPRB, 0.66572E+00_JPRB, 0.66889E+00_JPRB, 0.66593E+00_JPRB, 0.66400E+00_JPRB ,&
&  0.66371E+00_JPRB, 0.65983E+00_JPRB, 0.65506E+00_JPRB, 0.65476E+00_JPRB, 0.65969E+00_JPRB, 0.67550E+00_JPRB, 0.69839E+00_JPRB ,&
&  0.71553E+00_JPRB, 0.71580E+00_JPRB, 0.71989E+00_JPRB, 0.73087E+00_JPRB, 0.73293E+00_JPRB, 0.73384E+00_JPRB, 0.74004E+00_JPRB ,&
&  0.62363E+00_JPRB, 0.49867E+00_JPRB, 0.55828E+00_JPRB, 0.61015E+00_JPRB, 0.64624E+00_JPRB, 0.68705E+00_JPRB, 0.73169E+00_JPRB ,&
&  0.74537E+00_JPRB, 0.73518E+00_JPRB, 0.72682E+00_JPRB, 0.71538E+00_JPRB, 0.69864E+00_JPRB, 0.68833E+00_JPRB, 0.67994E+00_JPRB ,&
&  0.67942E+00_JPRB, 0.66995E+00_JPRB, 0.65456E+00_JPRB, 0.64240E+00_JPRB, 0.63304E+00_JPRB, 0.62094E+00_JPRB, 0.60918E+00_JPRB ,&
&  0.59691E+00_JPRB, 0.58666E+00_JPRB, 0.58513E+00_JPRB, 0.58560E+00_JPRB, 0.58990E+00_JPRB, 0.59153E+00_JPRB, 0.58816E+00_JPRB ,&
&  0.58535E+00_JPRB, 0.58750E+00_JPRB, 0.59667E+00_JPRB, 0.60420E+00_JPRB, 0.60580E+00_JPRB, 0.60258E+00_JPRB, 0.60897E+00_JPRB ,&
&  0.61155E+00_JPRB, 0.61871E+00_JPRB, 0.62826E+00_JPRB, 0.64768E+00_JPRB, 0.66162E+00_JPRB, 0.66161E+00_JPRB, 0.65984E+00_JPRB ,&
&  0.66156E+00_JPRB, 0.65476E+00_JPRB, 0.63960E+00_JPRB, 0.62958E+00_JPRB, 0.62497E+00_JPRB, 0.61845E+00_JPRB, 0.61692E+00_JPRB ,&
&  0.62196E+00_JPRB, 0.62345E+00_JPRB, 0.61775E+00_JPRB, 0.61958E+00_JPRB, 0.62836E+00_JPRB, 0.64185E+00_JPRB, 0.65445E+00_JPRB ,&
&  0.64778E+00_JPRB, 0.65550E+00_JPRB, 0.65569E+00_JPRB, 0.66401E+00_JPRB, 0.67458E+00_JPRB, 0.66575E+00_JPRB, 0.65988E+00_JPRB ,&
&  0.65450E+00_JPRB, 0.63576E+00_JPRB, 0.64045E+00_JPRB, 0.64514E+00_JPRB, 0.65052E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.64332E+00_JPRB, 0.67467E+00_JPRB, 0.68334E+00_JPRB, 0.67119E+00_JPRB, 0.64972E+00_JPRB, 0.63842E+00_JPRB, 0.63272E+00_JPRB ,&
&  0.63005E+00_JPRB, 0.62769E+00_JPRB, 0.63778E+00_JPRB, 0.64601E+00_JPRB, 0.65035E+00_JPRB, 0.64593E+00_JPRB, 0.64129E+00_JPRB ,&
&  0.63200E+00_JPRB, 0.61952E+00_JPRB, 0.61141E+00_JPRB, 0.60441E+00_JPRB, 0.60570E+00_JPRB, 0.61567E+00_JPRB, 0.63375E+00_JPRB ,&
&  0.65627E+00_JPRB, 0.67210E+00_JPRB, 0.68463E+00_JPRB, 0.69712E+00_JPRB, 0.69904E+00_JPRB, 0.69847E+00_JPRB, 0.70051E+00_JPRB ,&
&  0.70679E+00_JPRB, 0.52377E+00_JPRB, 0.56004E+00_JPRB, 0.60719E+00_JPRB, 0.64422E+00_JPRB, 0.69974E+00_JPRB, 0.74867E+00_JPRB ,&
&  0.76891E+00_JPRB, 0.75290E+00_JPRB, 0.74139E+00_JPRB, 0.72491E+00_JPRB, 0.70247E+00_JPRB, 0.68130E+00_JPRB, 0.65558E+00_JPRB ,&
&  0.63613E+00_JPRB, 0.62125E+00_JPRB, 0.61508E+00_JPRB, 0.61006E+00_JPRB, 0.60471E+00_JPRB, 0.59712E+00_JPRB, 0.58779E+00_JPRB ,&
&  0.57850E+00_JPRB, 0.57505E+00_JPRB, 0.57402E+00_JPRB, 0.57274E+00_JPRB, 0.57616E+00_JPRB, 0.58254E+00_JPRB, 0.58843E+00_JPRB ,&
&  0.59744E+00_JPRB, 0.60425E+00_JPRB, 0.60510E+00_JPRB, 0.60300E+00_JPRB, 0.60030E+00_JPRB, 0.60087E+00_JPRB, 0.61276E+00_JPRB ,&
&  0.61664E+00_JPRB, 0.62466E+00_JPRB, 0.63333E+00_JPRB, 0.65468E+00_JPRB, 0.67423E+00_JPRB, 0.67748E+00_JPRB, 0.67495E+00_JPRB ,&
&  0.67271E+00_JPRB, 0.66411E+00_JPRB, 0.65081E+00_JPRB, 0.63641E+00_JPRB, 0.62699E+00_JPRB, 0.62481E+00_JPRB, 0.61708E+00_JPRB ,&
&  0.61460E+00_JPRB, 0.61621E+00_JPRB, 0.61616E+00_JPRB, 0.62495E+00_JPRB, 0.64056E+00_JPRB, 0.65295E+00_JPRB, 0.65086E+00_JPRB ,&
&  0.64956E+00_JPRB, 0.65359E+00_JPRB, 0.65453E+00_JPRB, 0.67480E+00_JPRB, 0.68200E+00_JPRB, 0.68204E+00_JPRB, 0.67249E+00_JPRB ,&
&  0.66468E+00_JPRB, 0.65924E+00_JPRB, 0.49773E+00_JPRB, 0.58307E+00_JPRB, 0.60925E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.66471E+00_JPRB, 0.69839E+00_JPRB, 0.69986E+00_JPRB, 0.67174E+00_JPRB, 0.64200E+00_JPRB, 0.62408E+00_JPRB, 0.61372E+00_JPRB ,&
&  0.61234E+00_JPRB, 0.60760E+00_JPRB, 0.60971E+00_JPRB, 0.61983E+00_JPRB, 0.63122E+00_JPRB, 0.63445E+00_JPRB, 0.62022E+00_JPRB ,&
&  0.60241E+00_JPRB, 0.59021E+00_JPRB, 0.58690E+00_JPRB, 0.58724E+00_JPRB, 0.58572E+00_JPRB, 0.58574E+00_JPRB, 0.59614E+00_JPRB ,&
&  0.60846E+00_JPRB, 0.62264E+00_JPRB, 0.63703E+00_JPRB, 0.64820E+00_JPRB, 0.65581E+00_JPRB, 0.66225E+00_JPRB, 0.67932E+00_JPRB ,&
&  0.68691E+00_JPRB, 0.51667E+00_JPRB, 0.53598E+00_JPRB, 0.62128E+00_JPRB, 0.68082E+00_JPRB, 0.75656E+00_JPRB, 0.80258E+00_JPRB ,&
&  0.81246E+00_JPRB, 0.80260E+00_JPRB, 0.78074E+00_JPRB, 0.75264E+00_JPRB, 0.71834E+00_JPRB, 0.68453E+00_JPRB, 0.65043E+00_JPRB ,&
&  0.62760E+00_JPRB, 0.61007E+00_JPRB, 0.60078E+00_JPRB, 0.58840E+00_JPRB, 0.58179E+00_JPRB, 0.57883E+00_JPRB, 0.57595E+00_JPRB ,&
&  0.57419E+00_JPRB, 0.57058E+00_JPRB, 0.57400E+00_JPRB, 0.58107E+00_JPRB, 0.58279E+00_JPRB, 0.59086E+00_JPRB, 0.59313E+00_JPRB ,&
&  0.60225E+00_JPRB, 0.61542E+00_JPRB, 0.61848E+00_JPRB, 0.61513E+00_JPRB, 0.61570E+00_JPRB, 0.62370E+00_JPRB, 0.63536E+00_JPRB ,&
&  0.64215E+00_JPRB, 0.65921E+00_JPRB, 0.67153E+00_JPRB, 0.68683E+00_JPRB, 0.69632E+00_JPRB, 0.69264E+00_JPRB, 0.67980E+00_JPRB ,&
&  0.68066E+00_JPRB, 0.67412E+00_JPRB, 0.66606E+00_JPRB, 0.65617E+00_JPRB, 0.64534E+00_JPRB, 0.63501E+00_JPRB, 0.62605E+00_JPRB ,&
&  0.62269E+00_JPRB, 0.62908E+00_JPRB, 0.63574E+00_JPRB, 0.65517E+00_JPRB, 0.66708E+00_JPRB, 0.66953E+00_JPRB, 0.67222E+00_JPRB ,&
&  0.67679E+00_JPRB, 0.67848E+00_JPRB, 0.68068E+00_JPRB, 0.64662E+00_JPRB, 0.69414E+00_JPRB, 0.70271E+00_JPRB, 0.69606E+00_JPRB ,&
&  0.68785E+00_JPRB, 0.66292E+00_JPRB, 0.65821E+00_JPRB, 0.54032E+00_JPRB, 0.62089E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.75134E+00_JPRB, 0.74511E+00_JPRB, 0.71249E+00_JPRB, 0.67145E+00_JPRB, 0.63607E+00_JPRB, 0.60854E+00_JPRB, 0.59511E+00_JPRB ,&
&  0.58365E+00_JPRB, 0.57451E+00_JPRB, 0.57741E+00_JPRB, 0.58587E+00_JPRB, 0.59582E+00_JPRB, 0.59670E+00_JPRB, 0.59001E+00_JPRB ,&
&  0.58011E+00_JPRB, 0.56713E+00_JPRB, 0.56137E+00_JPRB, 0.55926E+00_JPRB, 0.55386E+00_JPRB, 0.55337E+00_JPRB, 0.55452E+00_JPRB ,&
&  0.56009E+00_JPRB, 0.56717E+00_JPRB, 0.57161E+00_JPRB, 0.58131E+00_JPRB, 0.59605E+00_JPRB, 0.61750E+00_JPRB, 0.63859E+00_JPRB ,&
&  0.63645E+00_JPRB, 0.53301E+00_JPRB, 0.57548E+00_JPRB, 0.66846E+00_JPRB, 0.75469E+00_JPRB, 0.80283E+00_JPRB, 0.82212E+00_JPRB ,&
&  0.81814E+00_JPRB, 0.79897E+00_JPRB, 0.77025E+00_JPRB, 0.73661E+00_JPRB, 0.70073E+00_JPRB, 0.66745E+00_JPRB, 0.63447E+00_JPRB ,&
&  0.61031E+00_JPRB, 0.59278E+00_JPRB, 0.58381E+00_JPRB, 0.57796E+00_JPRB, 0.57418E+00_JPRB, 0.56293E+00_JPRB, 0.55614E+00_JPRB ,&
&  0.55734E+00_JPRB, 0.55980E+00_JPRB, 0.56439E+00_JPRB, 0.56935E+00_JPRB, 0.58324E+00_JPRB, 0.60286E+00_JPRB, 0.61085E+00_JPRB ,&
&  0.61226E+00_JPRB, 0.62030E+00_JPRB, 0.62526E+00_JPRB, 0.61977E+00_JPRB, 0.61777E+00_JPRB, 0.64069E+00_JPRB, 0.65934E+00_JPRB ,&
&  0.67831E+00_JPRB, 0.70126E+00_JPRB, 0.72881E+00_JPRB, 0.74139E+00_JPRB, 0.73752E+00_JPRB, 0.72407E+00_JPRB, 0.69914E+00_JPRB ,&
&  0.68647E+00_JPRB, 0.67028E+00_JPRB, 0.65549E+00_JPRB, 0.64742E+00_JPRB, 0.64023E+00_JPRB, 0.63051E+00_JPRB, 0.62009E+00_JPRB ,&
&  0.61375E+00_JPRB, 0.60845E+00_JPRB, 0.61375E+00_JPRB, 0.63441E+00_JPRB, 0.65491E+00_JPRB, 0.66697E+00_JPRB, 0.67012E+00_JPRB ,&
&  0.66770E+00_JPRB, 0.66605E+00_JPRB, 0.67099E+00_JPRB, 0.71471E+00_JPRB, 0.70455E+00_JPRB, 0.70448E+00_JPRB, 0.69571E+00_JPRB ,&
&  0.68702E+00_JPRB, 0.67452E+00_JPRB, 0.65419E+00_JPRB, 0.64780E+00_JPRB, 0.67743E+00_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.81243E+00_JPRB, 0.76042E+00_JPRB, 0.71219E+00_JPRB, 0.66191E+00_JPRB, 0.62684E+00_JPRB, 0.61289E+00_JPRB, 0.60398E+00_JPRB ,&
&  0.58807E+00_JPRB, 0.57295E+00_JPRB, 0.57194E+00_JPRB, 0.57422E+00_JPRB, 0.57182E+00_JPRB, 0.56522E+00_JPRB, 0.55286E+00_JPRB ,&
&  0.54792E+00_JPRB, 0.55332E+00_JPRB, 0.55056E+00_JPRB, 0.54439E+00_JPRB, 0.53772E+00_JPRB, 0.53244E+00_JPRB, 0.52884E+00_JPRB ,&
&  0.52764E+00_JPRB, 0.53251E+00_JPRB, 0.53594E+00_JPRB, 0.54773E+00_JPRB, 0.56711E+00_JPRB, 0.58913E+00_JPRB, 0.61541E+00_JPRB ,&
&  0.61065E+00_JPRB, 0.52822E+00_JPRB, 0.63965E+00_JPRB, 0.77401E+00_JPRB, 0.86872E+00_JPRB, 0.88789E+00_JPRB, 0.84975E+00_JPRB ,&
&  0.82480E+00_JPRB, 0.77462E+00_JPRB, 0.73527E+00_JPRB, 0.70659E+00_JPRB, 0.67958E+00_JPRB, 0.65666E+00_JPRB, 0.63910E+00_JPRB ,&
&  0.61755E+00_JPRB, 0.60078E+00_JPRB, 0.58437E+00_JPRB, 0.56853E+00_JPRB, 0.56728E+00_JPRB, 0.56382E+00_JPRB, 0.55748E+00_JPRB ,&
&  0.55387E+00_JPRB, 0.55580E+00_JPRB, 0.55931E+00_JPRB, 0.56606E+00_JPRB, 0.57839E+00_JPRB, 0.62018E+00_JPRB, 0.65011E+00_JPRB ,&
&  0.65907E+00_JPRB, 0.67174E+00_JPRB, 0.67223E+00_JPRB, 0.66916E+00_JPRB, 0.67801E+00_JPRB, 0.69309E+00_JPRB, 0.72185E+00_JPRB ,&
&  0.75692E+00_JPRB, 0.77458E+00_JPRB, 0.78092E+00_JPRB, 0.78691E+00_JPRB, 0.77548E+00_JPRB, 0.76105E+00_JPRB, 0.74226E+00_JPRB ,&
&  0.72413E+00_JPRB, 0.69900E+00_JPRB, 0.67930E+00_JPRB, 0.66624E+00_JPRB, 0.65037E+00_JPRB, 0.63864E+00_JPRB, 0.62673E+00_JPRB ,&
&  0.61578E+00_JPRB, 0.60540E+00_JPRB, 0.61734E+00_JPRB, 0.63357E+00_JPRB, 0.64406E+00_JPRB, 0.64659E+00_JPRB, 0.65584E+00_JPRB ,&
&  0.66180E+00_JPRB, 0.67648E+00_JPRB, 0.66213E+00_JPRB, 0.70198E+00_JPRB, 0.75257E+00_JPRB, 0.74017E+00_JPRB, 0.72910E+00_JPRB ,&
&  0.71132E+00_JPRB, 0.69255E+00_JPRB, 0.66881E+00_JPRB, 0.66913E+00_JPRB, 0.80861E+00_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.69431E+00_JPRB, 0.65874E+00_JPRB, 0.62117E+00_JPRB, 0.59783E+00_JPRB, 0.57906E+00_JPRB, 0.56303E+00_JPRB, 0.56146E+00_JPRB ,&
&  0.55813E+00_JPRB, 0.55067E+00_JPRB, 0.55703E+00_JPRB, 0.55515E+00_JPRB, 0.54816E+00_JPRB, 0.53185E+00_JPRB, 0.53438E+00_JPRB ,&
&  0.53901E+00_JPRB, 0.53524E+00_JPRB, 0.52887E+00_JPRB, 0.52298E+00_JPRB, 0.51840E+00_JPRB, 0.51048E+00_JPRB, 0.50591E+00_JPRB ,&
&  0.50205E+00_JPRB, 0.50415E+00_JPRB, 0.51301E+00_JPRB, 0.52751E+00_JPRB, 0.55086E+00_JPRB, 0.57906E+00_JPRB, 0.60660E+00_JPRB ,&
&  0.59369E+00_JPRB, 0.60865E+00_JPRB, 0.77518E+00_JPRB, 0.10199E+01_JPRB, 0.10564E+01_JPRB, 0.94407E+00_JPRB, 0.85633E+00_JPRB ,&
&  0.79549E+00_JPRB, 0.73461E+00_JPRB, 0.69911E+00_JPRB, 0.67786E+00_JPRB, 0.65678E+00_JPRB, 0.64679E+00_JPRB, 0.63515E+00_JPRB ,&
&  0.62101E+00_JPRB, 0.60173E+00_JPRB, 0.57750E+00_JPRB, 0.56359E+00_JPRB, 0.55844E+00_JPRB, 0.55963E+00_JPRB, 0.55632E+00_JPRB ,&
&  0.55382E+00_JPRB, 0.56293E+00_JPRB, 0.56806E+00_JPRB, 0.57202E+00_JPRB, 0.61202E+00_JPRB, 0.68858E+00_JPRB, 0.76631E+00_JPRB ,&
&  0.75353E+00_JPRB, 0.74485E+00_JPRB, 0.74486E+00_JPRB, 0.74647E+00_JPRB, 0.76241E+00_JPRB, 0.77261E+00_JPRB, 0.79119E+00_JPRB ,&
&  0.81051E+00_JPRB, 0.80358E+00_JPRB, 0.78212E+00_JPRB, 0.76577E+00_JPRB, 0.74787E+00_JPRB, 0.74191E+00_JPRB, 0.71866E+00_JPRB ,&
&  0.69753E+00_JPRB, 0.68186E+00_JPRB, 0.67358E+00_JPRB, 0.65176E+00_JPRB, 0.63856E+00_JPRB, 0.62570E+00_JPRB, 0.61763E+00_JPRB ,&
&  0.60365E+00_JPRB, 0.59735E+00_JPRB, 0.62474E+00_JPRB, 0.63926E+00_JPRB, 0.64192E+00_JPRB, 0.64348E+00_JPRB, 0.65540E+00_JPRB ,&
&  0.67177E+00_JPRB, 0.68676E+00_JPRB, 0.69583E+00_JPRB, 0.71721E+00_JPRB, 0.70601E+00_JPRB, 0.75937E+00_JPRB, 0.74172E+00_JPRB ,&
&  0.72329E+00_JPRB, 0.68929E+00_JPRB, 0.65062E+00_JPRB, 0.63958E+00_JPRB, 0.70606E+00_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.64467E+00_JPRB, 0.60190E+00_JPRB, 0.56871E+00_JPRB, 0.54994E+00_JPRB, 0.53046E+00_JPRB, 0.52314E+00_JPRB, 0.51876E+00_JPRB ,&
&  0.52714E+00_JPRB, 0.53381E+00_JPRB, 0.54763E+00_JPRB, 0.54454E+00_JPRB, 0.52385E+00_JPRB, 0.51481E+00_JPRB, 0.52223E+00_JPRB ,&
&  0.51571E+00_JPRB, 0.51620E+00_JPRB, 0.52147E+00_JPRB, 0.51670E+00_JPRB, 0.51126E+00_JPRB, 0.50342E+00_JPRB, 0.49328E+00_JPRB ,&
&  0.49769E+00_JPRB, 0.50492E+00_JPRB, 0.51914E+00_JPRB, 0.54030E+00_JPRB, 0.56759E+00_JPRB, 0.59531E+00_JPRB, 0.61503E+00_JPRB ,&
&  0.62060E+00_JPRB, 0.76818E+00_JPRB, 0.10526E+01_JPRB, 0.12402E+01_JPRB, 0.11441E+01_JPRB, 0.94191E+00_JPRB, 0.84136E+00_JPRB ,&
&  0.76685E+00_JPRB, 0.70621E+00_JPRB, 0.68240E+00_JPRB, 0.66804E+00_JPRB, 0.64821E+00_JPRB, 0.63897E+00_JPRB, 0.64077E+00_JPRB ,&
&  0.62832E+00_JPRB, 0.60665E+00_JPRB, 0.58432E+00_JPRB, 0.56696E+00_JPRB, 0.55714E+00_JPRB, 0.55884E+00_JPRB, 0.56352E+00_JPRB ,&
&  0.56998E+00_JPRB, 0.57857E+00_JPRB, 0.59153E+00_JPRB, 0.59818E+00_JPRB, 0.61352E+00_JPRB, 0.60615E+00_JPRB, 0.75167E+00_JPRB ,&
&  0.94006E+00_JPRB, 0.91390E+00_JPRB, 0.88975E+00_JPRB, 0.87235E+00_JPRB, 0.85585E+00_JPRB, 0.83211E+00_JPRB, 0.82833E+00_JPRB ,&
&  0.81213E+00_JPRB, 0.77966E+00_JPRB, 0.75001E+00_JPRB, 0.73248E+00_JPRB, 0.70966E+00_JPRB, 0.69327E+00_JPRB, 0.66521E+00_JPRB ,&
&  0.65009E+00_JPRB, 0.63773E+00_JPRB, 0.62867E+00_JPRB, 0.62353E+00_JPRB, 0.62361E+00_JPRB, 0.61424E+00_JPRB, 0.61268E+00_JPRB ,&
&  0.60695E+00_JPRB, 0.62244E+00_JPRB, 0.64572E+00_JPRB, 0.63322E+00_JPRB, 0.63900E+00_JPRB, 0.66102E+00_JPRB, 0.67242E+00_JPRB ,&
&  0.65516E+00_JPRB, 0.66265E+00_JPRB, 0.69603E+00_JPRB, 0.70668E+00_JPRB, 0.69283E+00_JPRB, 0.75271E+00_JPRB, 0.72729E+00_JPRB ,&
&  0.69780E+00_JPRB, 0.66457E+00_JPRB, 0.64319E+00_JPRB, 0.62587E+00_JPRB, 0.66134E+00_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.59825E+00_JPRB, 0.56062E+00_JPRB, 0.53753E+00_JPRB, 0.51885E+00_JPRB, 0.52410E+00_JPRB, 0.51910E+00_JPRB, 0.51339E+00_JPRB ,&
&  0.52722E+00_JPRB, 0.53556E+00_JPRB, 0.54697E+00_JPRB, 0.53625E+00_JPRB, 0.51530E+00_JPRB, 0.51011E+00_JPRB, 0.50443E+00_JPRB ,&
&  0.49591E+00_JPRB, 0.51582E+00_JPRB, 0.53491E+00_JPRB, 0.54540E+00_JPRB, 0.53846E+00_JPRB, 0.51609E+00_JPRB, 0.50413E+00_JPRB ,&
&  0.51132E+00_JPRB, 0.52870E+00_JPRB, 0.55479E+00_JPRB, 0.58356E+00_JPRB, 0.60932E+00_JPRB, 0.63391E+00_JPRB, 0.66484E+00_JPRB ,&
&  0.74301E+00_JPRB, 0.88070E+00_JPRB, 0.12073E+01_JPRB, 0.14625E+01_JPRB, 0.11829E+01_JPRB, 0.92726E+00_JPRB, 0.76729E+00_JPRB ,&
&  0.74914E+00_JPRB, 0.69802E+00_JPRB, 0.68993E+00_JPRB, 0.68771E+00_JPRB, 0.67316E+00_JPRB, 0.65907E+00_JPRB, 0.65261E+00_JPRB ,&
&  0.63120E+00_JPRB, 0.61412E+00_JPRB, 0.60219E+00_JPRB, 0.59314E+00_JPRB, 0.58891E+00_JPRB, 0.58957E+00_JPRB, 0.59294E+00_JPRB ,&
&  0.60638E+00_JPRB, 0.61527E+00_JPRB, 0.62461E+00_JPRB, 0.63405E+00_JPRB, 0.61874E+00_JPRB, 0.65640E+00_JPRB, 0.74360E+00_JPRB ,&
&  0.85233E+00_JPRB, 0.99116E+00_JPRB, 0.93871E+00_JPRB, 0.90306E+00_JPRB, 0.87339E+00_JPRB, 0.85316E+00_JPRB, 0.82266E+00_JPRB ,&
&  0.80011E+00_JPRB, 0.76511E+00_JPRB, 0.73491E+00_JPRB, 0.70665E+00_JPRB, 0.66974E+00_JPRB, 0.64738E+00_JPRB, 0.64003E+00_JPRB ,&
&  0.62638E+00_JPRB, 0.62480E+00_JPRB, 0.61405E+00_JPRB, 0.62047E+00_JPRB, 0.61906E+00_JPRB, 0.62026E+00_JPRB, 0.62398E+00_JPRB ,&
&  0.63940E+00_JPRB, 0.68167E+00_JPRB, 0.69127E+00_JPRB, 0.64304E+00_JPRB, 0.62146E+00_JPRB, 0.62922E+00_JPRB, 0.64420E+00_JPRB ,&
&  0.65599E+00_JPRB, 0.68223E+00_JPRB, 0.69647E+00_JPRB, 0.71357E+00_JPRB, 0.67885E+00_JPRB, 0.73341E+00_JPRB, 0.70492E+00_JPRB ,&
&  0.67170E+00_JPRB, 0.65218E+00_JPRB, 0.64425E+00_JPRB, 0.63679E+00_JPRB, 0.63980E+00_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.60272E+00_JPRB, 0.57218E+00_JPRB, 0.53329E+00_JPRB, 0.51711E+00_JPRB, 0.52566E+00_JPRB, 0.51176E+00_JPRB, 0.51558E+00_JPRB ,&
&  0.52864E+00_JPRB, 0.52605E+00_JPRB, 0.52447E+00_JPRB, 0.51660E+00_JPRB, 0.51183E+00_JPRB, 0.50486E+00_JPRB, 0.50393E+00_JPRB ,&
&  0.51593E+00_JPRB, 0.54486E+00_JPRB, 0.54948E+00_JPRB, 0.53815E+00_JPRB, 0.53214E+00_JPRB, 0.51847E+00_JPRB, 0.51354E+00_JPRB ,&
&  0.53716E+00_JPRB, 0.58092E+00_JPRB, 0.62551E+00_JPRB, 0.65077E+00_JPRB, 0.67166E+00_JPRB, 0.69490E+00_JPRB, 0.77878E+00_JPRB ,&
&  0.98876E+00_JPRB, 0.21898E+01_JPRB, 0.99504E+00_JPRB, 0.15839E+01_JPRB, 0.11504E+01_JPRB, 0.87594E+00_JPRB, 0.68616E+00_JPRB ,&
&  0.66642E+00_JPRB, 0.70858E+00_JPRB, 0.71079E+00_JPRB, 0.73326E+00_JPRB, 0.70971E+00_JPRB, 0.66777E+00_JPRB, 0.64865E+00_JPRB ,&
&  0.63328E+00_JPRB, 0.62998E+00_JPRB, 0.63615E+00_JPRB, 0.64154E+00_JPRB, 0.64302E+00_JPRB, 0.64120E+00_JPRB, 0.64157E+00_JPRB ,&
&  0.64976E+00_JPRB, 0.64883E+00_JPRB, 0.66064E+00_JPRB, 0.66757E+00_JPRB, 0.64039E+00_JPRB, 0.69106E+00_JPRB, 0.77686E+00_JPRB ,&
&  0.81071E+00_JPRB, 0.97111E+00_JPRB, 0.89836E+00_JPRB, 0.87368E+00_JPRB, 0.85678E+00_JPRB, 0.82344E+00_JPRB, 0.77858E+00_JPRB ,&
&  0.73780E+00_JPRB, 0.71558E+00_JPRB, 0.69094E+00_JPRB, 0.66800E+00_JPRB, 0.64026E+00_JPRB, 0.63006E+00_JPRB, 0.61581E+00_JPRB ,&
&  0.61857E+00_JPRB, 0.63668E+00_JPRB, 0.63048E+00_JPRB, 0.63420E+00_JPRB, 0.63550E+00_JPRB, 0.65009E+00_JPRB, 0.66553E+00_JPRB ,&
&  0.69445E+00_JPRB, 0.73880E+00_JPRB, 0.71607E+00_JPRB, 0.62936E+00_JPRB, 0.59694E+00_JPRB, 0.60880E+00_JPRB, 0.62198E+00_JPRB ,&
&  0.66570E+00_JPRB, 0.69524E+00_JPRB, 0.69495E+00_JPRB, 0.69609E+00_JPRB, 0.69095E+00_JPRB, 0.72724E+00_JPRB, 0.69298E+00_JPRB ,&
&  0.65813E+00_JPRB, 0.65186E+00_JPRB, 0.65223E+00_JPRB, 0.63984E+00_JPRB, 0.64157E+00_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.60663E+00_JPRB, 0.58973E+00_JPRB, 0.55303E+00_JPRB, 0.52138E+00_JPRB, 0.52103E+00_JPRB, 0.52558E+00_JPRB, 0.53143E+00_JPRB ,&
&  0.52911E+00_JPRB, 0.52466E+00_JPRB, 0.51963E+00_JPRB, 0.51541E+00_JPRB, 0.52434E+00_JPRB, 0.52350E+00_JPRB, 0.55068E+00_JPRB ,&
&  0.56916E+00_JPRB, 0.56162E+00_JPRB, 0.56385E+00_JPRB, 0.54242E+00_JPRB, 0.53941E+00_JPRB, 0.53508E+00_JPRB, 0.55605E+00_JPRB ,&
&  0.60850E+00_JPRB, 0.66527E+00_JPRB, 0.69897E+00_JPRB, 0.71293E+00_JPRB, 0.73428E+00_JPRB, 0.77139E+00_JPRB, 0.89686E+00_JPRB ,&
&  0.12219E+01_JPRB, 0.24305E+01_JPRB, 0.12514E+01_JPRB, 0.16052E+01_JPRB, 0.10339E+01_JPRB, 0.75877E+00_JPRB, 0.62615E+00_JPRB ,&
&  0.62276E+00_JPRB, 0.70217E+00_JPRB, 0.71968E+00_JPRB, 0.74600E+00_JPRB, 0.67645E+00_JPRB, 0.63691E+00_JPRB, 0.62494E+00_JPRB ,&
&  0.63164E+00_JPRB, 0.66720E+00_JPRB, 0.69564E+00_JPRB, 0.70978E+00_JPRB, 0.70214E+00_JPRB, 0.68708E+00_JPRB, 0.68966E+00_JPRB ,&
&  0.69066E+00_JPRB, 0.67834E+00_JPRB, 0.69140E+00_JPRB, 0.66745E+00_JPRB, 0.70400E+00_JPRB, 0.73923E+00_JPRB, 0.76006E+00_JPRB ,&
&  0.78194E+00_JPRB, 0.87132E+00_JPRB, 0.85835E+00_JPRB, 0.85847E+00_JPRB, 0.82472E+00_JPRB, 0.78398E+00_JPRB, 0.73480E+00_JPRB ,&
&  0.70859E+00_JPRB, 0.68893E+00_JPRB, 0.66837E+00_JPRB, 0.63966E+00_JPRB, 0.62687E+00_JPRB, 0.63158E+00_JPRB, 0.62311E+00_JPRB ,&
&  0.64482E+00_JPRB, 0.67021E+00_JPRB, 0.67001E+00_JPRB, 0.67579E+00_JPRB, 0.68591E+00_JPRB, 0.70268E+00_JPRB, 0.72030E+00_JPRB ,&
&  0.74826E+00_JPRB, 0.78122E+00_JPRB, 0.72441E+00_JPRB, 0.62725E+00_JPRB, 0.61510E+00_JPRB, 0.62411E+00_JPRB, 0.62293E+00_JPRB ,&
&  0.66046E+00_JPRB, 0.68902E+00_JPRB, 0.69239E+00_JPRB, 0.67242E+00_JPRB, 0.68104E+00_JPRB, 0.71221E+00_JPRB, 0.68327E+00_JPRB ,&
&  0.66943E+00_JPRB, 0.67066E+00_JPRB, 0.68202E+00_JPRB, 0.68087E+00_JPRB, 0.65370E+00_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.56993E+00_JPRB, 0.55832E+00_JPRB, 0.54364E+00_JPRB, 0.52945E+00_JPRB, 0.52228E+00_JPRB, 0.52964E+00_JPRB, 0.53410E+00_JPRB ,&
&  0.52605E+00_JPRB, 0.52184E+00_JPRB, 0.51556E+00_JPRB, 0.52169E+00_JPRB, 0.54887E+00_JPRB, 0.59723E+00_JPRB, 0.64176E+00_JPRB ,&
&  0.63709E+00_JPRB, 0.60086E+00_JPRB, 0.59451E+00_JPRB, 0.58795E+00_JPRB, 0.60196E+00_JPRB, 0.62034E+00_JPRB, 0.65653E+00_JPRB ,&
&  0.70509E+00_JPRB, 0.73958E+00_JPRB, 0.75942E+00_JPRB, 0.77503E+00_JPRB, 0.80333E+00_JPRB, 0.83372E+00_JPRB, 0.10190E+01_JPRB ,&
&  0.14165E+01_JPRB, 0.28196E+01_JPRB, 0.23056E+01_JPRB, 0.13663E+01_JPRB, 0.87037E+00_JPRB, 0.69371E+00_JPRB, 0.64544E+00_JPRB ,&
&  0.57358E+00_JPRB, 0.50249E+00_JPRB, 0.71038E+00_JPRB, 0.74174E+00_JPRB, 0.68065E+00_JPRB, 0.65184E+00_JPRB, 0.66492E+00_JPRB ,&
&  0.70588E+00_JPRB, 0.73968E+00_JPRB, 0.75840E+00_JPRB, 0.75527E+00_JPRB, 0.73484E+00_JPRB, 0.72767E+00_JPRB, 0.73281E+00_JPRB ,&
&  0.73156E+00_JPRB, 0.73365E+00_JPRB, 0.78721E+00_JPRB, 0.75984E+00_JPRB, 0.72637E+00_JPRB, 0.71052E+00_JPRB, 0.69823E+00_JPRB ,&
&  0.73490E+00_JPRB, 0.78469E+00_JPRB, 0.83959E+00_JPRB, 0.84742E+00_JPRB, 0.75023E+00_JPRB, 0.77974E+00_JPRB, 0.72341E+00_JPRB ,&
&  0.71702E+00_JPRB, 0.69399E+00_JPRB, 0.66797E+00_JPRB, 0.63273E+00_JPRB, 0.64818E+00_JPRB, 0.66045E+00_JPRB, 0.65020E+00_JPRB ,&
&  0.68071E+00_JPRB, 0.71445E+00_JPRB, 0.70366E+00_JPRB, 0.71345E+00_JPRB, 0.72123E+00_JPRB, 0.73906E+00_JPRB, 0.75380E+00_JPRB ,&
&  0.76376E+00_JPRB, 0.79127E+00_JPRB, 0.73953E+00_JPRB, 0.68712E+00_JPRB, 0.66650E+00_JPRB, 0.63881E+00_JPRB, 0.63156E+00_JPRB ,&
&  0.65364E+00_JPRB, 0.69729E+00_JPRB, 0.69997E+00_JPRB, 0.70600E+00_JPRB, 0.69229E+00_JPRB, 0.69488E+00_JPRB, 0.68568E+00_JPRB ,&
&  0.69448E+00_JPRB, 0.70206E+00_JPRB, 0.75603E+00_JPRB, 0.69950E+00_JPRB, 0.62103E+00_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.52084E+00_JPRB, 0.51835E+00_JPRB, 0.51206E+00_JPRB, 0.51581E+00_JPRB, 0.51990E+00_JPRB, 0.51608E+00_JPRB, 0.51917E+00_JPRB ,&
&  0.52737E+00_JPRB, 0.53668E+00_JPRB, 0.53510E+00_JPRB, 0.56609E+00_JPRB, 0.63449E+00_JPRB, 0.70630E+00_JPRB, 0.70425E+00_JPRB ,&
&  0.68765E+00_JPRB, 0.67037E+00_JPRB, 0.66169E+00_JPRB, 0.67114E+00_JPRB, 0.67808E+00_JPRB, 0.71116E+00_JPRB, 0.74488E+00_JPRB ,&
&  0.77561E+00_JPRB, 0.80664E+00_JPRB, 0.83605E+00_JPRB, 0.87303E+00_JPRB, 0.92955E+00_JPRB, 0.10038E+01_JPRB, 0.11901E+01_JPRB ,&
&  0.15267E+01_JPRB, 0.26748E+01_JPRB, 0.88319E+00_JPRB, 0.88053E+00_JPRB, 0.80655E+00_JPRB, 0.66743E+00_JPRB, 0.56850E+00_JPRB ,&
&  0.48680E+00_JPRB, 0.43374E+00_JPRB, 0.62318E+00_JPRB, 0.74197E+00_JPRB, 0.70394E+00_JPRB, 0.71348E+00_JPRB, 0.74246E+00_JPRB ,&
&  0.78718E+00_JPRB, 0.80662E+00_JPRB, 0.80955E+00_JPRB, 0.78933E+00_JPRB, 0.77166E+00_JPRB, 0.76971E+00_JPRB, 0.79151E+00_JPRB ,&
&  0.81821E+00_JPRB, 0.85484E+00_JPRB, 0.86866E+00_JPRB, 0.75422E+00_JPRB, 0.72481E+00_JPRB, 0.68086E+00_JPRB, 0.64980E+00_JPRB ,&
&  0.62255E+00_JPRB, 0.70717E+00_JPRB, 0.82024E+00_JPRB, 0.84825E+00_JPRB, 0.74627E+00_JPRB, 0.69553E+00_JPRB, 0.73132E+00_JPRB ,&
&  0.71317E+00_JPRB, 0.67424E+00_JPRB, 0.64565E+00_JPRB, 0.62215E+00_JPRB, 0.63454E+00_JPRB, 0.66543E+00_JPRB, 0.65874E+00_JPRB ,&
&  0.68743E+00_JPRB, 0.72173E+00_JPRB, 0.70159E+00_JPRB, 0.72623E+00_JPRB, 0.74481E+00_JPRB, 0.75518E+00_JPRB, 0.75834E+00_JPRB ,&
&  0.76614E+00_JPRB, 0.79328E+00_JPRB, 0.80635E+00_JPRB, 0.78292E+00_JPRB, 0.72515E+00_JPRB, 0.68657E+00_JPRB, 0.69192E+00_JPRB ,&
&  0.69505E+00_JPRB, 0.71574E+00_JPRB, 0.70733E+00_JPRB, 0.67920E+00_JPRB, 0.70143E+00_JPRB, 0.71027E+00_JPRB, 0.74186E+00_JPRB ,&
&  0.78504E+00_JPRB, 0.87564E+00_JPRB, 0.93008E+00_JPRB, 0.65321E+00_JPRB, 0.56306E+00_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.49604E+00_JPRB, 0.49500E+00_JPRB, 0.48438E+00_JPRB, 0.49502E+00_JPRB, 0.49523E+00_JPRB, 0.50357E+00_JPRB, 0.51961E+00_JPRB ,&
&  0.54651E+00_JPRB, 0.58078E+00_JPRB, 0.61136E+00_JPRB, 0.65095E+00_JPRB, 0.70862E+00_JPRB, 0.75908E+00_JPRB, 0.75514E+00_JPRB ,&
&  0.73669E+00_JPRB, 0.72570E+00_JPRB, 0.72301E+00_JPRB, 0.72329E+00_JPRB, 0.74217E+00_JPRB, 0.76927E+00_JPRB, 0.79436E+00_JPRB ,&
&  0.82932E+00_JPRB, 0.87723E+00_JPRB, 0.92354E+00_JPRB, 0.98322E+00_JPRB, 0.10518E+01_JPRB, 0.11598E+01_JPRB, 0.12864E+01_JPRB ,&
&  0.16547E+01_JPRB, 0.14761E+01_JPRB, 0.65188E+00_JPRB, 0.85739E+00_JPRB, 0.74065E+00_JPRB, 0.59216E+00_JPRB, 0.47118E+00_JPRB ,&
&  0.40525E+00_JPRB, 0.47364E+00_JPRB, 0.65285E+00_JPRB, 0.75926E+00_JPRB, 0.74741E+00_JPRB, 0.77967E+00_JPRB, 0.81259E+00_JPRB ,&
&  0.83988E+00_JPRB, 0.84811E+00_JPRB, 0.84026E+00_JPRB, 0.82751E+00_JPRB, 0.82885E+00_JPRB, 0.84674E+00_JPRB, 0.88456E+00_JPRB ,&
&  0.92264E+00_JPRB, 0.93992E+00_JPRB, 0.92585E+00_JPRB, 0.78287E+00_JPRB, 0.75272E+00_JPRB, 0.66816E+00_JPRB, 0.62042E+00_JPRB ,&
&  0.56358E+00_JPRB, 0.62119E+00_JPRB, 0.73587E+00_JPRB, 0.79384E+00_JPRB, 0.72236E+00_JPRB, 0.66223E+00_JPRB, 0.73164E+00_JPRB ,&
&  0.69596E+00_JPRB, 0.64608E+00_JPRB, 0.63221E+00_JPRB, 0.62589E+00_JPRB, 0.63202E+00_JPRB, 0.65166E+00_JPRB, 0.67215E+00_JPRB ,&
&  0.69224E+00_JPRB, 0.68772E+00_JPRB, 0.66252E+00_JPRB, 0.68016E+00_JPRB, 0.70123E+00_JPRB, 0.71941E+00_JPRB, 0.73273E+00_JPRB ,&
&  0.75630E+00_JPRB, 0.78357E+00_JPRB, 0.80424E+00_JPRB, 0.82085E+00_JPRB, 0.78674E+00_JPRB, 0.74168E+00_JPRB, 0.74686E+00_JPRB ,&
&  0.70350E+00_JPRB, 0.69447E+00_JPRB, 0.69508E+00_JPRB, 0.69341E+00_JPRB, 0.68683E+00_JPRB, 0.73229E+00_JPRB, 0.76430E+00_JPRB ,&
&  0.80887E+00_JPRB, 0.84859E+00_JPRB, 0.77673E+00_JPRB, 0.58808E+00_JPRB, 0.52307E+00_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.50807E+00_JPRB, 0.51098E+00_JPRB, 0.50145E+00_JPRB, 0.49891E+00_JPRB, 0.49211E+00_JPRB, 0.49612E+00_JPRB, 0.50974E+00_JPRB ,&
&  0.53958E+00_JPRB, 0.58950E+00_JPRB, 0.64961E+00_JPRB, 0.70254E+00_JPRB, 0.76691E+00_JPRB, 0.81335E+00_JPRB, 0.80792E+00_JPRB ,&
&  0.78360E+00_JPRB, 0.77185E+00_JPRB, 0.75074E+00_JPRB, 0.76617E+00_JPRB, 0.77971E+00_JPRB, 0.80485E+00_JPRB, 0.83281E+00_JPRB ,&
&  0.86545E+00_JPRB, 0.90609E+00_JPRB, 0.95423E+00_JPRB, 0.10102E+01_JPRB, 0.10914E+01_JPRB, 0.11717E+01_JPRB, 0.12239E+01_JPRB ,&
&  0.78033E+00_JPRB, 0.39915E+00_JPRB, 0.69421E+00_JPRB, 0.82315E+00_JPRB, 0.62701E+00_JPRB, 0.52558E+00_JPRB, 0.42717E+00_JPRB ,&
&  0.39721E+00_JPRB, 0.54566E+00_JPRB, 0.67555E+00_JPRB, 0.78402E+00_JPRB, 0.79524E+00_JPRB, 0.82658E+00_JPRB, 0.85500E+00_JPRB ,&
&  0.86820E+00_JPRB, 0.87594E+00_JPRB, 0.88026E+00_JPRB, 0.88280E+00_JPRB, 0.89793E+00_JPRB, 0.91976E+00_JPRB, 0.95267E+00_JPRB ,&
&  0.98917E+00_JPRB, 0.98436E+00_JPRB, 0.96702E+00_JPRB, 0.79871E+00_JPRB, 0.78118E+00_JPRB, 0.70786E+00_JPRB, 0.65473E+00_JPRB ,&
&  0.60512E+00_JPRB, 0.55121E+00_JPRB, 0.58106E+00_JPRB, 0.77478E+00_JPRB, 0.77872E+00_JPRB, 0.64139E+00_JPRB, 0.72387E+00_JPRB ,&
&  0.67126E+00_JPRB, 0.62253E+00_JPRB, 0.61200E+00_JPRB, 0.61462E+00_JPRB, 0.60565E+00_JPRB, 0.63773E+00_JPRB, 0.65938E+00_JPRB ,&
&  0.67890E+00_JPRB, 0.65480E+00_JPRB, 0.62792E+00_JPRB, 0.63686E+00_JPRB, 0.65519E+00_JPRB, 0.67703E+00_JPRB, 0.69356E+00_JPRB ,&
&  0.71930E+00_JPRB, 0.73830E+00_JPRB, 0.76139E+00_JPRB, 0.79098E+00_JPRB, 0.81818E+00_JPRB, 0.77738E+00_JPRB, 0.75754E+00_JPRB ,&
&  0.68901E+00_JPRB, 0.66269E+00_JPRB, 0.65079E+00_JPRB, 0.64679E+00_JPRB, 0.65592E+00_JPRB, 0.68832E+00_JPRB, 0.72264E+00_JPRB ,&
&  0.75353E+00_JPRB, 0.75929E+00_JPRB, 0.69162E+00_JPRB, 0.58022E+00_JPRB, 0.52413E+00_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.49998E+00_JPRB, 0.50303E+00_JPRB, 0.50002E+00_JPRB, 0.50785E+00_JPRB, 0.50507E+00_JPRB, 0.50368E+00_JPRB, 0.49968E+00_JPRB ,&
&  0.52215E+00_JPRB, 0.56605E+00_JPRB, 0.63307E+00_JPRB, 0.70794E+00_JPRB, 0.76154E+00_JPRB, 0.82895E+00_JPRB, 0.85502E+00_JPRB ,&
&  0.83851E+00_JPRB, 0.80782E+00_JPRB, 0.79413E+00_JPRB, 0.80737E+00_JPRB, 0.81421E+00_JPRB, 0.83458E+00_JPRB, 0.85623E+00_JPRB ,&
&  0.87873E+00_JPRB, 0.91349E+00_JPRB, 0.95126E+00_JPRB, 0.99406E+00_JPRB, 0.10484E+01_JPRB, 0.10618E+01_JPRB, 0.10120E+01_JPRB ,&
&  0.37501E+00_JPRB, 0.52911E+00_JPRB, 0.75630E+00_JPRB, 0.78247E+00_JPRB, 0.59654E+00_JPRB, 0.50699E+00_JPRB, 0.43063E+00_JPRB ,&
&  0.46713E+00_JPRB, 0.57470E+00_JPRB, 0.68115E+00_JPRB, 0.80479E+00_JPRB, 0.80268E+00_JPRB, 0.83206E+00_JPRB, 0.86684E+00_JPRB ,&
&  0.87869E+00_JPRB, 0.88814E+00_JPRB, 0.91174E+00_JPRB, 0.93964E+00_JPRB, 0.95159E+00_JPRB, 0.98026E+00_JPRB, 0.10013E+01_JPRB ,&
&  0.10285E+01_JPRB, 0.10112E+01_JPRB, 0.10046E+01_JPRB, 0.83248E+00_JPRB, 0.79304E+00_JPRB, 0.75483E+00_JPRB, 0.73429E+00_JPRB ,&
&  0.61526E+00_JPRB, 0.53279E+00_JPRB, 0.56966E+00_JPRB, 0.74671E+00_JPRB, 0.79258E+00_JPRB, 0.79829E+00_JPRB, 0.73332E+00_JPRB ,&
&  0.66383E+00_JPRB, 0.59938E+00_JPRB, 0.56023E+00_JPRB, 0.54680E+00_JPRB, 0.56230E+00_JPRB, 0.61490E+00_JPRB, 0.65482E+00_JPRB ,&
&  0.63678E+00_JPRB, 0.61462E+00_JPRB, 0.60319E+00_JPRB, 0.59872E+00_JPRB, 0.62290E+00_JPRB, 0.64951E+00_JPRB, 0.65971E+00_JPRB ,&
&  0.66044E+00_JPRB, 0.66456E+00_JPRB, 0.68218E+00_JPRB, 0.70754E+00_JPRB, 0.73333E+00_JPRB, 0.75251E+00_JPRB, 0.76375E+00_JPRB ,&
&  0.68044E+00_JPRB, 0.64025E+00_JPRB, 0.64995E+00_JPRB, 0.65871E+00_JPRB, 0.70253E+00_JPRB, 0.74182E+00_JPRB, 0.69703E+00_JPRB ,&
&  0.66896E+00_JPRB, 0.65593E+00_JPRB, 0.62663E+00_JPRB, 0.56542E+00_JPRB, 0.50925E+00_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.51349E+00_JPRB, 0.52784E+00_JPRB, 0.54639E+00_JPRB, 0.56827E+00_JPRB, 0.56353E+00_JPRB, 0.56237E+00_JPRB, 0.54878E+00_JPRB ,&
&  0.54530E+00_JPRB, 0.56187E+00_JPRB, 0.60422E+00_JPRB, 0.67501E+00_JPRB, 0.72690E+00_JPRB, 0.79948E+00_JPRB, 0.89480E+00_JPRB ,&
&  0.91332E+00_JPRB, 0.88324E+00_JPRB, 0.86088E+00_JPRB, 0.85259E+00_JPRB, 0.86234E+00_JPRB, 0.88180E+00_JPRB, 0.89732E+00_JPRB ,&
&  0.91160E+00_JPRB, 0.94906E+00_JPRB, 0.97618E+00_JPRB, 0.10194E+01_JPRB, 0.10599E+01_JPRB, 0.10610E+01_JPRB, 0.10545E+01_JPRB ,&
&  0.61735E+00_JPRB, 0.84221E+00_JPRB, 0.79563E+00_JPRB, 0.76339E+00_JPRB, 0.64621E+00_JPRB, 0.55854E+00_JPRB, 0.52267E+00_JPRB ,&
&  0.54410E+00_JPRB, 0.57791E+00_JPRB, 0.66958E+00_JPRB, 0.71520E+00_JPRB, 0.76541E+00_JPRB, 0.79703E+00_JPRB, 0.84887E+00_JPRB ,&
&  0.86997E+00_JPRB, 0.88921E+00_JPRB, 0.91334E+00_JPRB, 0.93703E+00_JPRB, 0.95087E+00_JPRB, 0.98567E+00_JPRB, 0.10147E+01_JPRB ,&
&  0.10396E+01_JPRB, 0.10339E+01_JPRB, 0.10139E+01_JPRB, 0.88216E+00_JPRB, 0.74888E+00_JPRB, 0.72857E+00_JPRB, 0.68234E+00_JPRB ,&
&  0.55388E+00_JPRB, 0.46934E+00_JPRB, 0.56571E+00_JPRB, 0.75119E+00_JPRB, 0.77963E+00_JPRB, 0.77464E+00_JPRB, 0.72023E+00_JPRB ,&
&  0.64674E+00_JPRB, 0.57247E+00_JPRB, 0.53527E+00_JPRB, 0.50309E+00_JPRB, 0.50261E+00_JPRB, 0.54401E+00_JPRB, 0.58810E+00_JPRB ,&
&  0.57957E+00_JPRB, 0.56199E+00_JPRB, 0.55418E+00_JPRB, 0.56766E+00_JPRB, 0.59580E+00_JPRB, 0.61889E+00_JPRB, 0.63572E+00_JPRB ,&
&  0.61093E+00_JPRB, 0.57814E+00_JPRB, 0.57078E+00_JPRB, 0.58529E+00_JPRB, 0.61214E+00_JPRB, 0.64238E+00_JPRB, 0.67857E+00_JPRB ,&
&  0.70302E+00_JPRB, 0.71546E+00_JPRB, 0.81378E+00_JPRB, 0.10092E+01_JPRB, 0.11487E+01_JPRB, 0.12375E+01_JPRB, 0.10190E+01_JPRB ,&
&  0.80967E+00_JPRB, 0.67659E+00_JPRB, 0.59665E+00_JPRB, 0.54345E+00_JPRB, 0.50280E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.51053E+00_JPRB, 0.55703E+00_JPRB, 0.59340E+00_JPRB, 0.64518E+00_JPRB, 0.66097E+00_JPRB, 0.66219E+00_JPRB, 0.65824E+00_JPRB ,&
&  0.64578E+00_JPRB, 0.65187E+00_JPRB, 0.67299E+00_JPRB, 0.72320E+00_JPRB, 0.77592E+00_JPRB, 0.82655E+00_JPRB, 0.87898E+00_JPRB ,&
&  0.94457E+00_JPRB, 0.97670E+00_JPRB, 0.95345E+00_JPRB, 0.94855E+00_JPRB, 0.96491E+00_JPRB, 0.98711E+00_JPRB, 0.98791E+00_JPRB ,&
&  0.97529E+00_JPRB, 0.97631E+00_JPRB, 0.97919E+00_JPRB, 0.10043E+01_JPRB, 0.10368E+01_JPRB, 0.10474E+01_JPRB, 0.71846E+00_JPRB ,&
&  0.88464E+00_JPRB, 0.81519E+00_JPRB, 0.77814E+00_JPRB, 0.77579E+00_JPRB, 0.73460E+00_JPRB, 0.66908E+00_JPRB, 0.62693E+00_JPRB ,&
&  0.61354E+00_JPRB, 0.61463E+00_JPRB, 0.63374E+00_JPRB, 0.65217E+00_JPRB, 0.70496E+00_JPRB, 0.72844E+00_JPRB, 0.77996E+00_JPRB ,&
&  0.82735E+00_JPRB, 0.83888E+00_JPRB, 0.84685E+00_JPRB, 0.85590E+00_JPRB, 0.86435E+00_JPRB, 0.88402E+00_JPRB, 0.90443E+00_JPRB ,&
&  0.91775E+00_JPRB, 0.91946E+00_JPRB, 0.89519E+00_JPRB, 0.83633E+00_JPRB, 0.74277E+00_JPRB, 0.75564E+00_JPRB, 0.62998E+00_JPRB ,&
&  0.47155E+00_JPRB, 0.43172E+00_JPRB, 0.59962E+00_JPRB, 0.78317E+00_JPRB, 0.76705E+00_JPRB, 0.74360E+00_JPRB, 0.68273E+00_JPRB ,&
&  0.59842E+00_JPRB, 0.55459E+00_JPRB, 0.53010E+00_JPRB, 0.50798E+00_JPRB, 0.49169E+00_JPRB, 0.50549E+00_JPRB, 0.52867E+00_JPRB ,&
&  0.53542E+00_JPRB, 0.53150E+00_JPRB, 0.52699E+00_JPRB, 0.53880E+00_JPRB, 0.55907E+00_JPRB, 0.64513E+00_JPRB, 0.82112E+00_JPRB ,&
&  0.99217E+00_JPRB, 0.49383E+00_JPRB, 0.46323E+00_JPRB, 0.48753E+00_JPRB, 0.53591E+00_JPRB, 0.58500E+00_JPRB, 0.65252E+00_JPRB ,&
&  0.81223E+00_JPRB, 0.10633E+01_JPRB, 0.10078E+01_JPRB, 0.12060E+01_JPRB, 0.24941E+01_JPRB, 0.27300E+01_JPRB, 0.15679E+01_JPRB ,&
&  0.97957E+00_JPRB, 0.68859E+00_JPRB, 0.55530E+00_JPRB, 0.51213E+00_JPRB, 0.50090E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.53820E+00_JPRB, 0.59848E+00_JPRB, 0.66482E+00_JPRB, 0.70503E+00_JPRB, 0.67653E+00_JPRB, 0.66803E+00_JPRB, 0.67877E+00_JPRB ,&
&  0.69835E+00_JPRB, 0.72715E+00_JPRB, 0.75842E+00_JPRB, 0.80042E+00_JPRB, 0.82951E+00_JPRB, 0.86670E+00_JPRB, 0.91625E+00_JPRB ,&
&  0.97092E+00_JPRB, 0.97430E+00_JPRB, 0.98828E+00_JPRB, 0.10035E+01_JPRB, 0.10142E+01_JPRB, 0.10287E+01_JPRB, 0.10381E+01_JPRB ,&
&  0.10373E+01_JPRB, 0.10420E+01_JPRB, 0.10351E+01_JPRB, 0.10508E+01_JPRB, 0.10474E+01_JPRB, 0.11193E+01_JPRB, 0.79676E+00_JPRB ,&
&  0.93254E+00_JPRB, 0.86400E+00_JPRB, 0.78615E+00_JPRB, 0.78238E+00_JPRB, 0.76486E+00_JPRB, 0.74336E+00_JPRB, 0.71736E+00_JPRB ,&
&  0.71433E+00_JPRB, 0.69538E+00_JPRB, 0.64526E+00_JPRB, 0.65338E+00_JPRB, 0.64943E+00_JPRB, 0.66398E+00_JPRB, 0.69506E+00_JPRB ,&
&  0.72835E+00_JPRB, 0.75379E+00_JPRB, 0.75804E+00_JPRB, 0.77238E+00_JPRB, 0.77574E+00_JPRB, 0.76767E+00_JPRB, 0.75427E+00_JPRB ,&
&  0.73772E+00_JPRB, 0.72919E+00_JPRB, 0.72388E+00_JPRB, 0.69045E+00_JPRB, 0.69779E+00_JPRB, 0.63865E+00_JPRB, 0.54633E+00_JPRB ,&
&  0.43874E+00_JPRB, 0.44738E+00_JPRB, 0.66367E+00_JPRB, 0.83994E+00_JPRB, 0.77430E+00_JPRB, 0.74914E+00_JPRB, 0.66531E+00_JPRB ,&
&  0.60886E+00_JPRB, 0.54775E+00_JPRB, 0.52731E+00_JPRB, 0.49373E+00_JPRB, 0.48069E+00_JPRB, 0.49190E+00_JPRB, 0.52349E+00_JPRB ,&
&  0.53771E+00_JPRB, 0.52928E+00_JPRB, 0.52204E+00_JPRB, 0.49995E+00_JPRB, 0.49878E+00_JPRB, 0.51538E+00_JPRB, 0.48064E+00_JPRB ,&
&  0.55488E+00_JPRB, 0.45136E+00_JPRB, 0.46103E+00_JPRB, 0.52466E+00_JPRB, 0.49865E+00_JPRB, 0.54564E+00_JPRB, 0.61451E+00_JPRB ,&
&  0.80052E+00_JPRB, 0.87980E+00_JPRB, 0.12159E+01_JPRB, 0.34131E+01_JPRB, 0.39369E+01_JPRB, 0.49378E+01_JPRB, 0.19617E+01_JPRB ,&
&  0.90536E+00_JPRB, 0.59682E+00_JPRB, 0.52743E+00_JPRB, 0.52539E+00_JPRB, 0.51844E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.55258E+00_JPRB, 0.63229E+00_JPRB, 0.69782E+00_JPRB, 0.62375E+00_JPRB, 0.62810E+00_JPRB, 0.64291E+00_JPRB, 0.66787E+00_JPRB ,&
&  0.69695E+00_JPRB, 0.73274E+00_JPRB, 0.77085E+00_JPRB, 0.81630E+00_JPRB, 0.85197E+00_JPRB, 0.88499E+00_JPRB, 0.92111E+00_JPRB ,&
&  0.95943E+00_JPRB, 0.96980E+00_JPRB, 0.99947E+00_JPRB, 0.10007E+01_JPRB, 0.10221E+01_JPRB, 0.10175E+01_JPRB, 0.10342E+01_JPRB ,&
&  0.10615E+01_JPRB, 0.10761E+01_JPRB, 0.10907E+01_JPRB, 0.10984E+01_JPRB, 0.10797E+01_JPRB, 0.10927E+01_JPRB, 0.81480E+00_JPRB ,&
&  0.99562E+00_JPRB, 0.90908E+00_JPRB, 0.82407E+00_JPRB, 0.80340E+00_JPRB, 0.77140E+00_JPRB, 0.76557E+00_JPRB, 0.75614E+00_JPRB ,&
&  0.74811E+00_JPRB, 0.74394E+00_JPRB, 0.72605E+00_JPRB, 0.68027E+00_JPRB, 0.65008E+00_JPRB, 0.66156E+00_JPRB, 0.67232E+00_JPRB ,&
&  0.69166E+00_JPRB, 0.71889E+00_JPRB, 0.74113E+00_JPRB, 0.75819E+00_JPRB, 0.77077E+00_JPRB, 0.77003E+00_JPRB, 0.76639E+00_JPRB ,&
&  0.75639E+00_JPRB, 0.75604E+00_JPRB, 0.71512E+00_JPRB, 0.70893E+00_JPRB, 0.74338E+00_JPRB, 0.66345E+00_JPRB, 0.54250E+00_JPRB ,&
&  0.44820E+00_JPRB, 0.57955E+00_JPRB, 0.76749E+00_JPRB, 0.91934E+00_JPRB, 0.83000E+00_JPRB, 0.77405E+00_JPRB, 0.71557E+00_JPRB ,&
&  0.64021E+00_JPRB, 0.58060E+00_JPRB, 0.55690E+00_JPRB, 0.51251E+00_JPRB, 0.49456E+00_JPRB, 0.49289E+00_JPRB, 0.49320E+00_JPRB ,&
&  0.48827E+00_JPRB, 0.48318E+00_JPRB, 0.47156E+00_JPRB, 0.46055E+00_JPRB, 0.42025E+00_JPRB, 0.36511E+00_JPRB, 0.51449E+00_JPRB ,&
&  0.44531E+00_JPRB, 0.36298E+00_JPRB, 0.53220E+00_JPRB, 0.40322E+00_JPRB, 0.53630E+00_JPRB, 0.59499E+00_JPRB, 0.64271E+00_JPRB ,&
&  0.60263E+00_JPRB, 0.76275E+00_JPRB, 0.11115E+01_JPRB, 0.11380E+01_JPRB, 0.11959E+01_JPRB, 0.12691E+01_JPRB, 0.74542E+00_JPRB ,&
&  0.59372E+00_JPRB, 0.57976E+00_JPRB, 0.57922E+00_JPRB, 0.53358E+00_JPRB, 0.53011E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.55567E+00_JPRB, 0.61242E+00_JPRB, 0.69987E+00_JPRB, 0.62731E+00_JPRB, 0.62336E+00_JPRB, 0.63556E+00_JPRB, 0.65510E+00_JPRB ,&
&  0.67534E+00_JPRB, 0.70265E+00_JPRB, 0.73203E+00_JPRB, 0.77329E+00_JPRB, 0.80921E+00_JPRB, 0.84137E+00_JPRB, 0.87180E+00_JPRB ,&
&  0.89960E+00_JPRB, 0.92309E+00_JPRB, 0.94579E+00_JPRB, 0.95524E+00_JPRB, 0.96695E+00_JPRB, 0.98541E+00_JPRB, 0.10096E+01_JPRB ,&
&  0.10507E+01_JPRB, 0.10681E+01_JPRB, 0.10916E+01_JPRB, 0.11340E+01_JPRB, 0.11631E+01_JPRB, 0.11844E+01_JPRB, 0.97370E+00_JPRB ,&
&  0.10426E+01_JPRB, 0.93988E+00_JPRB, 0.86641E+00_JPRB, 0.81436E+00_JPRB, 0.77692E+00_JPRB, 0.75868E+00_JPRB, 0.75888E+00_JPRB ,&
&  0.78065E+00_JPRB, 0.73890E+00_JPRB, 0.73561E+00_JPRB, 0.70826E+00_JPRB, 0.69392E+00_JPRB, 0.71602E+00_JPRB, 0.73080E+00_JPRB ,&
&  0.77072E+00_JPRB, 0.82489E+00_JPRB, 0.86975E+00_JPRB, 0.90246E+00_JPRB, 0.93838E+00_JPRB, 0.93967E+00_JPRB, 0.96425E+00_JPRB ,&
&  0.10015E+01_JPRB, 0.10180E+01_JPRB, 0.97477E+00_JPRB, 0.97252E+00_JPRB, 0.95843E+00_JPRB, 0.89435E+00_JPRB, 0.73895E+00_JPRB ,&
&  0.63820E+00_JPRB, 0.71974E+00_JPRB, 0.76538E+00_JPRB, 0.90191E+00_JPRB, 0.86098E+00_JPRB, 0.79574E+00_JPRB, 0.75853E+00_JPRB ,&
&  0.70081E+00_JPRB, 0.65492E+00_JPRB, 0.62457E+00_JPRB, 0.57555E+00_JPRB, 0.58188E+00_JPRB, 0.58671E+00_JPRB, 0.58606E+00_JPRB ,&
&  0.55491E+00_JPRB, 0.52222E+00_JPRB, 0.49052E+00_JPRB, 0.46889E+00_JPRB, 0.48720E+00_JPRB, 0.46593E+00_JPRB, 0.50818E+00_JPRB ,&
&  0.51534E+00_JPRB, 0.36678E+00_JPRB, 0.46883E+00_JPRB, 0.62320E+00_JPRB, 0.66514E+00_JPRB, 0.71270E+00_JPRB, 0.77201E+00_JPRB ,&
&  0.84092E+00_JPRB, 0.83336E+00_JPRB, 0.79583E+00_JPRB, 0.78561E+00_JPRB, 0.77239E+00_JPRB, 0.76583E+00_JPRB, 0.68321E+00_JPRB ,&
&  0.65221E+00_JPRB, 0.59634E+00_JPRB, 0.56531E+00_JPRB, 0.55250E+00_JPRB, 0.52778E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.59886E+00_JPRB, 0.61753E+00_JPRB, 0.68197E+00_JPRB, 0.63965E+00_JPRB, 0.63743E+00_JPRB, 0.65066E+00_JPRB, 0.66646E+00_JPRB ,&
&  0.66559E+00_JPRB, 0.67375E+00_JPRB, 0.69146E+00_JPRB, 0.72326E+00_JPRB, 0.75440E+00_JPRB, 0.77878E+00_JPRB, 0.80550E+00_JPRB ,&
&  0.82622E+00_JPRB, 0.83920E+00_JPRB, 0.84683E+00_JPRB, 0.85531E+00_JPRB, 0.88090E+00_JPRB, 0.90995E+00_JPRB, 0.94929E+00_JPRB ,&
&  0.99521E+00_JPRB, 0.10018E+01_JPRB, 0.99202E+00_JPRB, 0.10070E+01_JPRB, 0.10215E+01_JPRB, 0.10996E+01_JPRB, 0.12076E+01_JPRB ,&
&  0.92076E+00_JPRB, 0.95982E+00_JPRB, 0.89473E+00_JPRB, 0.81310E+00_JPRB, 0.80914E+00_JPRB, 0.78088E+00_JPRB, 0.79224E+00_JPRB ,&
&  0.80749E+00_JPRB, 0.79420E+00_JPRB, 0.80612E+00_JPRB, 0.81361E+00_JPRB, 0.81502E+00_JPRB, 0.83259E+00_JPRB, 0.85636E+00_JPRB ,&
&  0.89199E+00_JPRB, 0.94638E+00_JPRB, 0.99334E+00_JPRB, 0.10638E+01_JPRB, 0.11320E+01_JPRB, 0.11952E+01_JPRB, 0.12436E+01_JPRB ,&
&  0.12729E+01_JPRB, 0.12822E+01_JPRB, 0.11840E+01_JPRB, 0.11743E+01_JPRB, 0.11768E+01_JPRB, 0.11317E+01_JPRB, 0.10142E+01_JPRB ,&
&  0.90192E+00_JPRB, 0.85837E+00_JPRB, 0.85809E+00_JPRB, 0.82651E+00_JPRB, 0.83722E+00_JPRB, 0.84576E+00_JPRB, 0.80808E+00_JPRB ,&
&  0.78978E+00_JPRB, 0.73973E+00_JPRB, 0.70249E+00_JPRB, 0.67380E+00_JPRB, 0.67252E+00_JPRB, 0.69140E+00_JPRB, 0.68577E+00_JPRB ,&
&  0.65643E+00_JPRB, 0.62068E+00_JPRB, 0.58880E+00_JPRB, 0.55835E+00_JPRB, 0.52947E+00_JPRB, 0.62422E+00_JPRB, 0.58600E+00_JPRB ,&
&  0.62675E+00_JPRB, 0.56902E+00_JPRB, 0.54312E+00_JPRB, 0.74330E+00_JPRB, 0.80684E+00_JPRB, 0.80269E+00_JPRB, 0.82934E+00_JPRB ,&
&  0.83510E+00_JPRB, 0.82481E+00_JPRB, 0.84262E+00_JPRB, 0.79496E+00_JPRB, 0.75819E+00_JPRB, 0.73718E+00_JPRB, 0.72434E+00_JPRB ,&
&  0.67050E+00_JPRB, 0.61969E+00_JPRB, 0.57301E+00_JPRB, 0.57436E+00_JPRB, 0.54820E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.58977E+00_JPRB, 0.59396E+00_JPRB, 0.61730E+00_JPRB, 0.65489E+00_JPRB, 0.63388E+00_JPRB, 0.65113E+00_JPRB, 0.64645E+00_JPRB ,&
&  0.64502E+00_JPRB, 0.63834E+00_JPRB, 0.64881E+00_JPRB, 0.66012E+00_JPRB, 0.67984E+00_JPRB, 0.69918E+00_JPRB, 0.70506E+00_JPRB ,&
&  0.71662E+00_JPRB, 0.72928E+00_JPRB, 0.74939E+00_JPRB, 0.78235E+00_JPRB, 0.83954E+00_JPRB, 0.93022E+00_JPRB, 0.11049E+01_JPRB ,&
&  0.12712E+01_JPRB, 0.13321E+01_JPRB, 0.13207E+01_JPRB, 0.12734E+01_JPRB, 0.13768E+01_JPRB, 0.17008E+01_JPRB, 0.19293E+01_JPRB ,&
&  0.11692E+01_JPRB, 0.96490E+00_JPRB, 0.88993E+00_JPRB, 0.80502E+00_JPRB, 0.80782E+00_JPRB, 0.85104E+00_JPRB, 0.85674E+00_JPRB ,&
&  0.84878E+00_JPRB, 0.86523E+00_JPRB, 0.89782E+00_JPRB, 0.93887E+00_JPRB, 0.96783E+00_JPRB, 0.98651E+00_JPRB, 0.10136E+01_JPRB ,&
&  0.10518E+01_JPRB, 0.11074E+01_JPRB, 0.11716E+01_JPRB, 0.12466E+01_JPRB, 0.12212E+01_JPRB, 0.13207E+01_JPRB, 0.13402E+01_JPRB ,&
&  0.13494E+01_JPRB, 0.12636E+01_JPRB, 0.11053E+01_JPRB, 0.11138E+01_JPRB, 0.11947E+01_JPRB, 0.11934E+01_JPRB, 0.10918E+01_JPRB ,&
&  0.11168E+01_JPRB, 0.97489E+00_JPRB, 0.76157E+00_JPRB, 0.76102E+00_JPRB, 0.79508E+00_JPRB, 0.86237E+00_JPRB, 0.85744E+00_JPRB ,&
&  0.86888E+00_JPRB, 0.82500E+00_JPRB, 0.79130E+00_JPRB, 0.77516E+00_JPRB, 0.77301E+00_JPRB, 0.79024E+00_JPRB, 0.79297E+00_JPRB ,&
&  0.73316E+00_JPRB, 0.70491E+00_JPRB, 0.66424E+00_JPRB, 0.64852E+00_JPRB, 0.65530E+00_JPRB, 0.59552E+00_JPRB, 0.66321E+00_JPRB ,&
&  0.69039E+00_JPRB, 0.71269E+00_JPRB, 0.65071E+00_JPRB, 0.81469E+00_JPRB, 0.83729E+00_JPRB, 0.81674E+00_JPRB, 0.81983E+00_JPRB ,&
&  0.83972E+00_JPRB, 0.79568E+00_JPRB, 0.76896E+00_JPRB, 0.74767E+00_JPRB, 0.73475E+00_JPRB, 0.71097E+00_JPRB, 0.66283E+00_JPRB ,&
&  0.62835E+00_JPRB, 0.60495E+00_JPRB, 0.59147E+00_JPRB, 0.58115E+00_JPRB, 0.58360E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.57273E+00_JPRB, 0.59248E+00_JPRB, 0.60614E+00_JPRB, 0.60129E+00_JPRB, 0.60637E+00_JPRB, 0.61231E+00_JPRB, 0.60042E+00_JPRB ,&
&  0.59150E+00_JPRB, 0.58589E+00_JPRB, 0.59910E+00_JPRB, 0.59983E+00_JPRB, 0.60325E+00_JPRB, 0.61771E+00_JPRB, 0.63595E+00_JPRB ,&
&  0.65660E+00_JPRB, 0.67341E+00_JPRB, 0.73843E+00_JPRB, 0.86828E+00_JPRB, 0.11034E+01_JPRB, 0.13967E+01_JPRB, 0.15877E+01_JPRB ,&
&  0.16731E+01_JPRB, 0.17403E+01_JPRB, 0.17762E+01_JPRB, 0.17813E+01_JPRB, 0.15881E+01_JPRB, 0.15311E+01_JPRB, 0.15678E+01_JPRB ,&
&  0.11386E+01_JPRB, 0.87058E+00_JPRB, 0.93712E+00_JPRB, 0.88530E+00_JPRB, 0.88011E+00_JPRB, 0.88592E+00_JPRB, 0.88231E+00_JPRB ,&
&  0.88541E+00_JPRB, 0.90434E+00_JPRB, 0.93753E+00_JPRB, 0.99147E+00_JPRB, 0.10407E+01_JPRB, 0.10712E+01_JPRB, 0.10899E+01_JPRB ,&
&  0.11099E+01_JPRB, 0.11342E+01_JPRB, 0.11811E+01_JPRB, 0.11631E+01_JPRB, 0.11385E+01_JPRB, 0.11603E+01_JPRB, 0.11586E+01_JPRB ,&
&  0.11501E+01_JPRB, 0.11035E+01_JPRB, 0.97750E+00_JPRB, 0.97299E+00_JPRB, 0.10474E+01_JPRB, 0.99625E+00_JPRB, 0.96898E+00_JPRB ,&
&  0.10307E+01_JPRB, 0.98562E+00_JPRB, 0.69581E+00_JPRB, 0.67727E+00_JPRB, 0.73487E+00_JPRB, 0.82426E+00_JPRB, 0.91260E+00_JPRB ,&
&  0.92653E+00_JPRB, 0.90638E+00_JPRB, 0.87207E+00_JPRB, 0.86568E+00_JPRB, 0.85151E+00_JPRB, 0.85360E+00_JPRB, 0.84979E+00_JPRB ,&
&  0.78800E+00_JPRB, 0.76373E+00_JPRB, 0.73460E+00_JPRB, 0.70836E+00_JPRB, 0.70600E+00_JPRB, 0.70167E+00_JPRB, 0.71241E+00_JPRB ,&
&  0.74529E+00_JPRB, 0.74726E+00_JPRB, 0.75837E+00_JPRB, 0.76860E+00_JPRB, 0.76355E+00_JPRB, 0.81447E+00_JPRB, 0.79387E+00_JPRB ,&
&  0.80394E+00_JPRB, 0.77705E+00_JPRB, 0.74946E+00_JPRB, 0.72324E+00_JPRB, 0.70201E+00_JPRB, 0.67726E+00_JPRB, 0.64676E+00_JPRB ,&
&  0.61798E+00_JPRB, 0.59667E+00_JPRB, 0.58273E+00_JPRB, 0.57052E+00_JPRB, 0.56093E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.55993E+00_JPRB, 0.57525E+00_JPRB, 0.59469E+00_JPRB, 0.58911E+00_JPRB, 0.56942E+00_JPRB, 0.55936E+00_JPRB, 0.55292E+00_JPRB ,&
&  0.55630E+00_JPRB, 0.57262E+00_JPRB, 0.58300E+00_JPRB, 0.58404E+00_JPRB, 0.58531E+00_JPRB, 0.60172E+00_JPRB, 0.64409E+00_JPRB ,&
&  0.69968E+00_JPRB, 0.77089E+00_JPRB, 0.89870E+00_JPRB, 0.11520E+01_JPRB, 0.14541E+01_JPRB, 0.16809E+01_JPRB, 0.16043E+01_JPRB ,&
&  0.14680E+01_JPRB, 0.14471E+01_JPRB, 0.14111E+01_JPRB, 0.13689E+01_JPRB, 0.13402E+01_JPRB, 0.13036E+01_JPRB, 0.12216E+01_JPRB ,&
&  0.10373E+01_JPRB, 0.88318E+00_JPRB, 0.88086E+00_JPRB, 0.88694E+00_JPRB, 0.88807E+00_JPRB, 0.87408E+00_JPRB, 0.87959E+00_JPRB ,&
&  0.88424E+00_JPRB, 0.89970E+00_JPRB, 0.92813E+00_JPRB, 0.97886E+00_JPRB, 0.10370E+01_JPRB, 0.10706E+01_JPRB, 0.10807E+01_JPRB ,&
&  0.10823E+01_JPRB, 0.10777E+01_JPRB, 0.10951E+01_JPRB, 0.10089E+01_JPRB, 0.10257E+01_JPRB, 0.10284E+01_JPRB, 0.10202E+01_JPRB ,&
&  0.99346E+00_JPRB, 0.94316E+00_JPRB, 0.97692E+00_JPRB, 0.97161E+00_JPRB, 0.96033E+00_JPRB, 0.93766E+00_JPRB, 0.93076E+00_JPRB ,&
&  0.94872E+00_JPRB, 0.91811E+00_JPRB, 0.71249E+00_JPRB, 0.70989E+00_JPRB, 0.74888E+00_JPRB, 0.74333E+00_JPRB, 0.89155E+00_JPRB ,&
&  0.90719E+00_JPRB, 0.91993E+00_JPRB, 0.92413E+00_JPRB, 0.93673E+00_JPRB, 0.94883E+00_JPRB, 0.96213E+00_JPRB, 0.88715E+00_JPRB ,&
&  0.85678E+00_JPRB, 0.81566E+00_JPRB, 0.80554E+00_JPRB, 0.79180E+00_JPRB, 0.79977E+00_JPRB, 0.80135E+00_JPRB, 0.79770E+00_JPRB ,&
&  0.80669E+00_JPRB, 0.82287E+00_JPRB, 0.83921E+00_JPRB, 0.86155E+00_JPRB, 0.81906E+00_JPRB, 0.78145E+00_JPRB, 0.76490E+00_JPRB ,&
&  0.75055E+00_JPRB, 0.71373E+00_JPRB, 0.70125E+00_JPRB, 0.68523E+00_JPRB, 0.67200E+00_JPRB, 0.65922E+00_JPRB, 0.64632E+00_JPRB ,&
&  0.62702E+00_JPRB, 0.60971E+00_JPRB, 0.59203E+00_JPRB, 0.57282E+00_JPRB, 0.55970E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.56415E+00_JPRB, 0.57180E+00_JPRB, 0.59027E+00_JPRB, 0.60005E+00_JPRB, 0.58000E+00_JPRB, 0.56733E+00_JPRB, 0.56425E+00_JPRB ,&
&  0.57398E+00_JPRB, 0.59010E+00_JPRB, 0.60403E+00_JPRB, 0.61719E+00_JPRB, 0.64674E+00_JPRB, 0.66541E+00_JPRB, 0.69696E+00_JPRB ,&
&  0.75353E+00_JPRB, 0.84229E+00_JPRB, 0.10054E+01_JPRB, 0.12586E+01_JPRB, 0.14862E+01_JPRB, 0.16975E+01_JPRB, 0.15702E+01_JPRB ,&
&  0.14475E+01_JPRB, 0.13873E+01_JPRB, 0.13488E+01_JPRB, 0.12738E+01_JPRB, 0.10982E+01_JPRB, 0.10913E+01_JPRB, 0.10444E+01_JPRB ,&
&  0.10157E+01_JPRB, 0.95383E+00_JPRB, 0.92833E+00_JPRB, 0.88955E+00_JPRB, 0.85155E+00_JPRB, 0.84381E+00_JPRB, 0.85488E+00_JPRB ,&
&  0.86937E+00_JPRB, 0.88332E+00_JPRB, 0.90956E+00_JPRB, 0.94572E+00_JPRB, 0.99055E+00_JPRB, 0.10343E+01_JPRB, 0.10522E+01_JPRB ,&
&  0.10385E+01_JPRB, 0.10227E+01_JPRB, 0.10073E+01_JPRB, 0.96583E+00_JPRB, 0.94992E+00_JPRB, 0.94946E+00_JPRB, 0.94997E+00_JPRB ,&
&  0.95892E+00_JPRB, 0.92443E+00_JPRB, 0.94331E+00_JPRB, 0.96241E+00_JPRB, 0.92266E+00_JPRB, 0.86566E+00_JPRB, 0.85720E+00_JPRB ,&
&  0.87179E+00_JPRB, 0.92719E+00_JPRB, 0.75337E+00_JPRB, 0.82925E+00_JPRB, 0.90184E+00_JPRB, 0.83839E+00_JPRB, 0.84305E+00_JPRB ,&
&  0.89282E+00_JPRB, 0.92348E+00_JPRB, 0.96066E+00_JPRB, 0.99333E+00_JPRB, 0.10143E+01_JPRB, 0.10353E+01_JPRB, 0.89343E+00_JPRB ,&
&  0.89741E+00_JPRB, 0.84880E+00_JPRB, 0.84030E+00_JPRB, 0.83912E+00_JPRB, 0.84937E+00_JPRB, 0.84667E+00_JPRB, 0.81143E+00_JPRB ,&
&  0.78877E+00_JPRB, 0.89847E+00_JPRB, 0.10527E+01_JPRB, 0.12389E+01_JPRB, 0.13029E+01_JPRB, 0.76461E+00_JPRB, 0.77341E+00_JPRB ,&
&  0.74926E+00_JPRB, 0.71638E+00_JPRB, 0.70027E+00_JPRB, 0.68542E+00_JPRB, 0.67475E+00_JPRB, 0.66272E+00_JPRB, 0.65499E+00_JPRB ,&
&  0.64164E+00_JPRB, 0.62360E+00_JPRB, 0.60514E+00_JPRB, 0.58789E+00_JPRB, 0.57331E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.58076E+00_JPRB, 0.57903E+00_JPRB, 0.59216E+00_JPRB, 0.60376E+00_JPRB, 0.60573E+00_JPRB, 0.59284E+00_JPRB, 0.58941E+00_JPRB ,&
&  0.58602E+00_JPRB, 0.59733E+00_JPRB, 0.60787E+00_JPRB, 0.63693E+00_JPRB, 0.67208E+00_JPRB, 0.73106E+00_JPRB, 0.79018E+00_JPRB ,&
&  0.85715E+00_JPRB, 0.94412E+00_JPRB, 0.11064E+01_JPRB, 0.13097E+01_JPRB, 0.14673E+01_JPRB, 0.15562E+01_JPRB, 0.14817E+01_JPRB ,&
&  0.13877E+01_JPRB, 0.13976E+01_JPRB, 0.13670E+01_JPRB, 0.10648E+01_JPRB, 0.95493E+00_JPRB, 0.11082E+01_JPRB, 0.95839E+00_JPRB ,&
&  0.90021E+00_JPRB, 0.88539E+00_JPRB, 0.86059E+00_JPRB, 0.82918E+00_JPRB, 0.79443E+00_JPRB, 0.79415E+00_JPRB, 0.80699E+00_JPRB ,&
&  0.83145E+00_JPRB, 0.85372E+00_JPRB, 0.87940E+00_JPRB, 0.90677E+00_JPRB, 0.93751E+00_JPRB, 0.99011E+00_JPRB, 0.10230E+01_JPRB ,&
&  0.10210E+01_JPRB, 0.10060E+01_JPRB, 0.98670E+00_JPRB, 0.96565E+00_JPRB, 0.92599E+00_JPRB, 0.91453E+00_JPRB, 0.91690E+00_JPRB ,&
&  0.91872E+00_JPRB, 0.91363E+00_JPRB, 0.92050E+00_JPRB, 0.93818E+00_JPRB, 0.93886E+00_JPRB, 0.85468E+00_JPRB, 0.83396E+00_JPRB ,&
&  0.84878E+00_JPRB, 0.88460E+00_JPRB, 0.82610E+00_JPRB, 0.90475E+00_JPRB, 0.66742E+00_JPRB, 0.72191E+00_JPRB, 0.81338E+00_JPRB ,&
&  0.85181E+00_JPRB, 0.89169E+00_JPRB, 0.92894E+00_JPRB, 0.97354E+00_JPRB, 0.99579E+00_JPRB, 0.92417E+00_JPRB, 0.94812E+00_JPRB ,&
&  0.94279E+00_JPRB, 0.88196E+00_JPRB, 0.86952E+00_JPRB, 0.85729E+00_JPRB, 0.86067E+00_JPRB, 0.85447E+00_JPRB, 0.80664E+00_JPRB ,&
&  0.76107E+00_JPRB, 0.86630E+00_JPRB, 0.98594E+00_JPRB, 0.10860E+01_JPRB, 0.10351E+01_JPRB, 0.79100E+00_JPRB, 0.77993E+00_JPRB ,&
&  0.76975E+00_JPRB, 0.73072E+00_JPRB, 0.70692E+00_JPRB, 0.68951E+00_JPRB, 0.67734E+00_JPRB, 0.66180E+00_JPRB, 0.64988E+00_JPRB ,&
&  0.63897E+00_JPRB, 0.62292E+00_JPRB, 0.61066E+00_JPRB, 0.59914E+00_JPRB, 0.58808E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.58658E+00_JPRB, 0.59023E+00_JPRB, 0.60638E+00_JPRB, 0.60941E+00_JPRB, 0.61913E+00_JPRB, 0.62708E+00_JPRB, 0.61115E+00_JPRB ,&
&  0.58062E+00_JPRB, 0.55369E+00_JPRB, 0.56384E+00_JPRB, 0.59692E+00_JPRB, 0.63934E+00_JPRB, 0.70519E+00_JPRB, 0.79019E+00_JPRB ,&
&  0.89664E+00_JPRB, 0.98721E+00_JPRB, 0.11279E+01_JPRB, 0.12512E+01_JPRB, 0.13385E+01_JPRB, 0.13929E+01_JPRB, 0.13743E+01_JPRB ,&
&  0.11071E+01_JPRB, 0.92438E+00_JPRB, 0.13148E+01_JPRB, 0.13973E+01_JPRB, 0.12416E+01_JPRB, 0.11438E+01_JPRB, 0.97259E+00_JPRB ,&
&  0.88100E+00_JPRB, 0.87498E+00_JPRB, 0.86681E+00_JPRB, 0.82110E+00_JPRB, 0.75111E+00_JPRB, 0.74582E+00_JPRB, 0.76063E+00_JPRB ,&
&  0.78808E+00_JPRB, 0.81284E+00_JPRB, 0.84396E+00_JPRB, 0.88124E+00_JPRB, 0.90787E+00_JPRB, 0.95072E+00_JPRB, 0.98472E+00_JPRB ,&
&  0.10014E+01_JPRB, 0.10001E+01_JPRB, 0.99471E+00_JPRB, 0.96748E+00_JPRB, 0.94291E+00_JPRB, 0.92110E+00_JPRB, 0.90043E+00_JPRB ,&
&  0.90160E+00_JPRB, 0.89544E+00_JPRB, 0.91544E+00_JPRB, 0.92684E+00_JPRB, 0.95067E+00_JPRB, 0.88994E+00_JPRB, 0.89826E+00_JPRB ,&
&  0.87442E+00_JPRB, 0.86748E+00_JPRB, 0.85519E+00_JPRB, 0.87676E+00_JPRB, 0.68546E+00_JPRB, 0.72593E+00_JPRB, 0.72954E+00_JPRB ,&
&  0.81843E+00_JPRB, 0.85876E+00_JPRB, 0.89460E+00_JPRB, 0.92892E+00_JPRB, 0.95905E+00_JPRB, 0.90707E+00_JPRB, 0.93876E+00_JPRB ,&
&  0.95331E+00_JPRB, 0.94407E+00_JPRB, 0.89746E+00_JPRB, 0.88415E+00_JPRB, 0.82983E+00_JPRB, 0.86121E+00_JPRB, 0.84465E+00_JPRB ,&
&  0.86134E+00_JPRB, 0.87763E+00_JPRB, 0.93251E+00_JPRB, 0.99936E+00_JPRB, 0.95916E+00_JPRB, 0.85212E+00_JPRB, 0.81901E+00_JPRB ,&
&  0.79758E+00_JPRB, 0.76982E+00_JPRB, 0.73131E+00_JPRB, 0.70188E+00_JPRB, 0.68143E+00_JPRB, 0.66155E+00_JPRB, 0.64214E+00_JPRB ,&
&  0.62275E+00_JPRB, 0.60676E+00_JPRB, 0.59577E+00_JPRB, 0.58659E+00_JPRB, 0.58387E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.56873E+00_JPRB, 0.58805E+00_JPRB, 0.61639E+00_JPRB, 0.64903E+00_JPRB, 0.76229E+00_JPRB, 0.88815E+00_JPRB, 0.95907E+00_JPRB ,&
&  0.90648E+00_JPRB, 0.54063E+00_JPRB, 0.52748E+00_JPRB, 0.54627E+00_JPRB, 0.57704E+00_JPRB, 0.63313E+00_JPRB, 0.69663E+00_JPRB ,&
&  0.78782E+00_JPRB, 0.88759E+00_JPRB, 0.10205E+01_JPRB, 0.11842E+01_JPRB, 0.12750E+01_JPRB, 0.13506E+01_JPRB, 0.12402E+01_JPRB ,&
&  0.86377E+00_JPRB, 0.95222E+00_JPRB, 0.14913E+01_JPRB, 0.14289E+01_JPRB, 0.13179E+01_JPRB, 0.12058E+01_JPRB, 0.10577E+01_JPRB ,&
&  0.94509E+00_JPRB, 0.85682E+00_JPRB, 0.83177E+00_JPRB, 0.79699E+00_JPRB, 0.72045E+00_JPRB, 0.69340E+00_JPRB, 0.69565E+00_JPRB ,&
&  0.71718E+00_JPRB, 0.74975E+00_JPRB, 0.79303E+00_JPRB, 0.83821E+00_JPRB, 0.88585E+00_JPRB, 0.91572E+00_JPRB, 0.94929E+00_JPRB ,&
&  0.97791E+00_JPRB, 0.98984E+00_JPRB, 0.97062E+00_JPRB, 0.94310E+00_JPRB, 0.95001E+00_JPRB, 0.94742E+00_JPRB, 0.93235E+00_JPRB ,&
&  0.92822E+00_JPRB, 0.96403E+00_JPRB, 0.97949E+00_JPRB, 0.92166E+00_JPRB, 0.85109E+00_JPRB, 0.91007E+00_JPRB, 0.87771E+00_JPRB ,&
&  0.87851E+00_JPRB, 0.83872E+00_JPRB, 0.89574E+00_JPRB, 0.76166E+00_JPRB, 0.72057E+00_JPRB, 0.76138E+00_JPRB, 0.76401E+00_JPRB ,&
&  0.76703E+00_JPRB, 0.81233E+00_JPRB, 0.85010E+00_JPRB, 0.88632E+00_JPRB, 0.90694E+00_JPRB, 0.86156E+00_JPRB, 0.91992E+00_JPRB ,&
&  0.89628E+00_JPRB, 0.98545E+00_JPRB, 0.94470E+00_JPRB, 0.91517E+00_JPRB, 0.81863E+00_JPRB, 0.81713E+00_JPRB, 0.82952E+00_JPRB ,&
&  0.86706E+00_JPRB, 0.88457E+00_JPRB, 0.89713E+00_JPRB, 0.90691E+00_JPRB, 0.88166E+00_JPRB, 0.87206E+00_JPRB, 0.86819E+00_JPRB ,&
&  0.83720E+00_JPRB, 0.79378E+00_JPRB, 0.76104E+00_JPRB, 0.72272E+00_JPRB, 0.69368E+00_JPRB, 0.66359E+00_JPRB, 0.63155E+00_JPRB ,&
&  0.60690E+00_JPRB, 0.58997E+00_JPRB, 0.57671E+00_JPRB, 0.56352E+00_JPRB, 0.55920E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.53908E+00_JPRB, 0.55460E+00_JPRB, 0.57861E+00_JPRB, 0.61653E+00_JPRB, 0.72861E+00_JPRB, 0.77353E+00_JPRB, 0.77578E+00_JPRB ,&
&  0.75378E+00_JPRB, 0.51360E+00_JPRB, 0.48759E+00_JPRB, 0.50372E+00_JPRB, 0.53775E+00_JPRB, 0.57876E+00_JPRB, 0.64196E+00_JPRB ,&
&  0.69791E+00_JPRB, 0.78319E+00_JPRB, 0.91702E+00_JPRB, 0.10734E+01_JPRB, 0.11860E+01_JPRB, 0.12003E+01_JPRB, 0.10532E+01_JPRB ,&
&  0.77116E+00_JPRB, 0.14723E+01_JPRB, 0.15327E+01_JPRB, 0.15433E+01_JPRB, 0.14889E+01_JPRB, 0.13587E+01_JPRB, 0.11165E+01_JPRB ,&
&  0.98183E+00_JPRB, 0.89740E+00_JPRB, 0.82625E+00_JPRB, 0.77602E+00_JPRB, 0.72092E+00_JPRB, 0.66496E+00_JPRB, 0.64673E+00_JPRB ,&
&  0.65612E+00_JPRB, 0.69233E+00_JPRB, 0.73903E+00_JPRB, 0.78765E+00_JPRB, 0.84618E+00_JPRB, 0.89743E+00_JPRB, 0.93135E+00_JPRB ,&
&  0.97379E+00_JPRB, 0.98295E+00_JPRB, 0.94473E+00_JPRB, 0.94390E+00_JPRB, 0.95230E+00_JPRB, 0.97570E+00_JPRB, 0.98385E+00_JPRB ,&
&  0.94505E+00_JPRB, 0.92346E+00_JPRB, 0.10224E+01_JPRB, 0.10179E+01_JPRB, 0.94918E+00_JPRB, 0.95405E+00_JPRB, 0.85931E+00_JPRB ,&
&  0.88660E+00_JPRB, 0.81548E+00_JPRB, 0.84033E+00_JPRB, 0.73016E+00_JPRB, 0.71983E+00_JPRB, 0.77212E+00_JPRB, 0.76886E+00_JPRB ,&
&  0.73379E+00_JPRB, 0.77303E+00_JPRB, 0.78683E+00_JPRB, 0.82394E+00_JPRB, 0.84882E+00_JPRB, 0.85421E+00_JPRB, 0.84698E+00_JPRB ,&
&  0.86936E+00_JPRB, 0.91911E+00_JPRB, 0.98290E+00_JPRB, 0.88346E+00_JPRB, 0.83826E+00_JPRB, 0.79788E+00_JPRB, 0.83622E+00_JPRB ,&
&  0.85782E+00_JPRB, 0.86880E+00_JPRB, 0.92851E+00_JPRB, 0.92910E+00_JPRB, 0.84845E+00_JPRB, 0.84663E+00_JPRB, 0.85657E+00_JPRB ,&
&  0.83859E+00_JPRB, 0.80185E+00_JPRB, 0.77850E+00_JPRB, 0.73104E+00_JPRB, 0.67544E+00_JPRB, 0.64201E+00_JPRB, 0.61562E+00_JPRB ,&
&  0.59298E+00_JPRB, 0.57691E+00_JPRB, 0.55998E+00_JPRB, 0.54331E+00_JPRB, 0.53506E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.52154E+00_JPRB, 0.53897E+00_JPRB, 0.56108E+00_JPRB, 0.59300E+00_JPRB, 0.66279E+00_JPRB, 0.68528E+00_JPRB, 0.63339E+00_JPRB ,&
&  0.56715E+00_JPRB, 0.46182E+00_JPRB, 0.44907E+00_JPRB, 0.46463E+00_JPRB, 0.49370E+00_JPRB, 0.52883E+00_JPRB, 0.58922E+00_JPRB ,&
&  0.66204E+00_JPRB, 0.72389E+00_JPRB, 0.85729E+00_JPRB, 0.97636E+00_JPRB, 0.10974E+01_JPRB, 0.11881E+01_JPRB, 0.81077E+00_JPRB ,&
&  0.83919E+00_JPRB, 0.14441E+01_JPRB, 0.15423E+01_JPRB, 0.15763E+01_JPRB, 0.15961E+01_JPRB, 0.15948E+01_JPRB, 0.12865E+01_JPRB ,&
&  0.11139E+01_JPRB, 0.10067E+01_JPRB, 0.88862E+00_JPRB, 0.78465E+00_JPRB, 0.73344E+00_JPRB, 0.69028E+00_JPRB, 0.63644E+00_JPRB ,&
&  0.62217E+00_JPRB, 0.64525E+00_JPRB, 0.69169E+00_JPRB, 0.74403E+00_JPRB, 0.80488E+00_JPRB, 0.87432E+00_JPRB, 0.91948E+00_JPRB ,&
&  0.96547E+00_JPRB, 0.98123E+00_JPRB, 0.97006E+00_JPRB, 0.91575E+00_JPRB, 0.95056E+00_JPRB, 0.98138E+00_JPRB, 0.10134E+01_JPRB ,&
&  0.97067E+00_JPRB, 0.96276E+00_JPRB, 0.11358E+01_JPRB, 0.12508E+01_JPRB, 0.12037E+01_JPRB, 0.10866E+01_JPRB, 0.94361E+00_JPRB ,&
&  0.90348E+00_JPRB, 0.83538E+00_JPRB, 0.75630E+00_JPRB, 0.69098E+00_JPRB, 0.71308E+00_JPRB, 0.76198E+00_JPRB, 0.76843E+00_JPRB ,&
&  0.74124E+00_JPRB, 0.74216E+00_JPRB, 0.73421E+00_JPRB, 0.73565E+00_JPRB, 0.75376E+00_JPRB, 0.77023E+00_JPRB, 0.80337E+00_JPRB ,&
&  0.86869E+00_JPRB, 0.93212E+00_JPRB, 0.95198E+00_JPRB, 0.85603E+00_JPRB, 0.82582E+00_JPRB, 0.73053E+00_JPRB, 0.76437E+00_JPRB ,&
&  0.81781E+00_JPRB, 0.81426E+00_JPRB, 0.83505E+00_JPRB, 0.91669E+00_JPRB, 0.86875E+00_JPRB, 0.86775E+00_JPRB, 0.88529E+00_JPRB ,&
&  0.83175E+00_JPRB, 0.77741E+00_JPRB, 0.75322E+00_JPRB, 0.72980E+00_JPRB, 0.68555E+00_JPRB, 0.64118E+00_JPRB, 0.61372E+00_JPRB ,&
&  0.58581E+00_JPRB, 0.56848E+00_JPRB, 0.54748E+00_JPRB, 0.52902E+00_JPRB, 0.51883E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.50410E+00_JPRB, 0.51480E+00_JPRB, 0.53663E+00_JPRB, 0.55634E+00_JPRB, 0.57822E+00_JPRB, 0.58703E+00_JPRB, 0.53877E+00_JPRB ,&
&  0.47329E+00_JPRB, 0.43148E+00_JPRB, 0.41560E+00_JPRB, 0.42095E+00_JPRB, 0.44990E+00_JPRB, 0.49138E+00_JPRB, 0.53191E+00_JPRB ,&
&  0.61548E+00_JPRB, 0.69222E+00_JPRB, 0.79495E+00_JPRB, 0.89176E+00_JPRB, 0.88834E+00_JPRB, 0.76453E+00_JPRB, 0.76624E+00_JPRB ,&
&  0.11307E+01_JPRB, 0.15656E+01_JPRB, 0.15517E+01_JPRB, 0.15654E+01_JPRB, 0.16546E+01_JPRB, 0.17698E+01_JPRB, 0.14631E+01_JPRB ,&
&  0.12598E+01_JPRB, 0.11229E+01_JPRB, 0.97218E+00_JPRB, 0.86372E+00_JPRB, 0.77242E+00_JPRB, 0.73329E+00_JPRB, 0.67728E+00_JPRB ,&
&  0.62387E+00_JPRB, 0.62992E+00_JPRB, 0.68191E+00_JPRB, 0.73545E+00_JPRB, 0.79517E+00_JPRB, 0.85677E+00_JPRB, 0.92507E+00_JPRB ,&
&  0.96239E+00_JPRB, 0.96837E+00_JPRB, 0.95370E+00_JPRB, 0.91709E+00_JPRB, 0.94018E+00_JPRB, 0.98606E+00_JPRB, 0.10880E+01_JPRB ,&
&  0.11457E+01_JPRB, 0.12593E+01_JPRB, 0.13413E+01_JPRB, 0.14159E+01_JPRB, 0.13532E+01_JPRB, 0.12822E+01_JPRB, 0.11317E+01_JPRB ,&
&  0.10114E+01_JPRB, 0.88369E+00_JPRB, 0.71758E+00_JPRB, 0.68922E+00_JPRB, 0.74875E+00_JPRB, 0.77348E+00_JPRB, 0.70499E+00_JPRB ,&
&  0.65050E+00_JPRB, 0.61833E+00_JPRB, 0.61586E+00_JPRB, 0.63573E+00_JPRB, 0.67862E+00_JPRB, 0.71185E+00_JPRB, 0.77284E+00_JPRB ,&
&  0.78374E+00_JPRB, 0.59709E+00_JPRB, 0.55062E+00_JPRB, 0.63724E+00_JPRB, 0.59127E+00_JPRB, 0.57717E+00_JPRB, 0.67924E+00_JPRB ,&
&  0.75772E+00_JPRB, 0.80372E+00_JPRB, 0.81721E+00_JPRB, 0.78679E+00_JPRB, 0.85883E+00_JPRB, 0.91284E+00_JPRB, 0.99171E+00_JPRB ,&
&  0.86439E+00_JPRB, 0.78319E+00_JPRB, 0.73550E+00_JPRB, 0.70844E+00_JPRB, 0.67692E+00_JPRB, 0.64233E+00_JPRB, 0.61774E+00_JPRB ,&
&  0.59222E+00_JPRB, 0.56537E+00_JPRB, 0.54185E+00_JPRB, 0.52033E+00_JPRB, 0.50484E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.48795E+00_JPRB, 0.49023E+00_JPRB, 0.51107E+00_JPRB, 0.52750E+00_JPRB, 0.52449E+00_JPRB, 0.50747E+00_JPRB, 0.47842E+00_JPRB ,&
&  0.44590E+00_JPRB, 0.41986E+00_JPRB, 0.40139E+00_JPRB, 0.40035E+00_JPRB, 0.41476E+00_JPRB, 0.45637E+00_JPRB, 0.50648E+00_JPRB ,&
&  0.57972E+00_JPRB, 0.68796E+00_JPRB, 0.78526E+00_JPRB, 0.85397E+00_JPRB, 0.77239E+00_JPRB, 0.63061E+00_JPRB, 0.73216E+00_JPRB ,&
&  0.10102E+01_JPRB, 0.14483E+01_JPRB, 0.16773E+01_JPRB, 0.16992E+01_JPRB, 0.17017E+01_JPRB, 0.17168E+01_JPRB, 0.16119E+01_JPRB ,&
&  0.14159E+01_JPRB, 0.12129E+01_JPRB, 0.10472E+01_JPRB, 0.93044E+00_JPRB, 0.82374E+00_JPRB, 0.74493E+00_JPRB, 0.71150E+00_JPRB ,&
&  0.65985E+00_JPRB, 0.62109E+00_JPRB, 0.65610E+00_JPRB, 0.72445E+00_JPRB, 0.77922E+00_JPRB, 0.83609E+00_JPRB, 0.89880E+00_JPRB ,&
&  0.93131E+00_JPRB, 0.95055E+00_JPRB, 0.95027E+00_JPRB, 0.95345E+00_JPRB, 0.80465E+00_JPRB, 0.95677E+00_JPRB, 0.10235E+01_JPRB ,&
&  0.11535E+01_JPRB, 0.13078E+01_JPRB, 0.13913E+01_JPRB, 0.16501E+01_JPRB, 0.15196E+01_JPRB, 0.13865E+01_JPRB, 0.12529E+01_JPRB ,&
&  0.11545E+01_JPRB, 0.98004E+00_JPRB, 0.79868E+00_JPRB, 0.76364E+00_JPRB, 0.79165E+00_JPRB, 0.77174E+00_JPRB, 0.58271E+00_JPRB ,&
&  0.54474E+00_JPRB, 0.60960E+00_JPRB, 0.58075E+00_JPRB, 0.54654E+00_JPRB, 0.61110E+00_JPRB, 0.66580E+00_JPRB, 0.57752E+00_JPRB ,&
&  0.39671E+00_JPRB, 0.35650E+00_JPRB, 0.36054E+00_JPRB, 0.36894E+00_JPRB, 0.36568E+00_JPRB, 0.41097E+00_JPRB, 0.65428E+00_JPRB ,&
&  0.69537E+00_JPRB, 0.76602E+00_JPRB, 0.76373E+00_JPRB, 0.74591E+00_JPRB, 0.79438E+00_JPRB, 0.89684E+00_JPRB, 0.10283E+01_JPRB ,&
&  0.87208E+00_JPRB, 0.80741E+00_JPRB, 0.74649E+00_JPRB, 0.70960E+00_JPRB, 0.67818E+00_JPRB, 0.64815E+00_JPRB, 0.61666E+00_JPRB ,&
&  0.59243E+00_JPRB, 0.56398E+00_JPRB, 0.53718E+00_JPRB, 0.51237E+00_JPRB, 0.49349E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.46608E+00_JPRB, 0.46244E+00_JPRB, 0.47163E+00_JPRB, 0.47530E+00_JPRB, 0.48267E+00_JPRB, 0.45919E+00_JPRB, 0.45915E+00_JPRB ,&
&  0.43273E+00_JPRB, 0.42038E+00_JPRB, 0.39459E+00_JPRB, 0.38533E+00_JPRB, 0.38858E+00_JPRB, 0.42556E+00_JPRB, 0.48849E+00_JPRB ,&
&  0.55921E+00_JPRB, 0.67282E+00_JPRB, 0.86219E+00_JPRB, 0.82381E+00_JPRB, 0.79904E+00_JPRB, 0.71346E+00_JPRB, 0.67264E+00_JPRB ,&
&  0.86915E+00_JPRB, 0.12336E+01_JPRB, 0.14520E+01_JPRB, 0.15736E+01_JPRB, 0.18220E+01_JPRB, 0.18793E+01_JPRB, 0.17877E+01_JPRB ,&
&  0.16432E+01_JPRB, 0.13968E+01_JPRB, 0.11995E+01_JPRB, 0.10486E+01_JPRB, 0.91123E+00_JPRB, 0.79774E+00_JPRB, 0.72131E+00_JPRB ,&
&  0.69352E+00_JPRB, 0.64496E+00_JPRB, 0.64135E+00_JPRB, 0.69950E+00_JPRB, 0.76620E+00_JPRB, 0.80473E+00_JPRB, 0.85206E+00_JPRB ,&
&  0.89506E+00_JPRB, 0.91161E+00_JPRB, 0.91969E+00_JPRB, 0.90955E+00_JPRB, 0.77864E+00_JPRB, 0.70131E+00_JPRB, 0.88051E+00_JPRB ,&
&  0.10667E+01_JPRB, 0.13305E+01_JPRB, 0.15505E+01_JPRB, 0.18901E+01_JPRB, 0.15834E+01_JPRB, 0.14426E+01_JPRB, 0.13914E+01_JPRB ,&
&  0.12499E+01_JPRB, 0.12063E+01_JPRB, 0.87761E+00_JPRB, 0.82384E+00_JPRB, 0.83624E+00_JPRB, 0.66867E+00_JPRB, 0.58398E+00_JPRB ,&
&  0.62220E+00_JPRB, 0.58877E+00_JPRB, 0.53642E+00_JPRB, 0.47603E+00_JPRB, 0.55180E+00_JPRB, 0.55496E+00_JPRB, 0.37581E+00_JPRB ,&
&  0.33567E+00_JPRB, 0.32492E+00_JPRB, 0.31185E+00_JPRB, 0.30243E+00_JPRB, 0.29671E+00_JPRB, 0.32093E+00_JPRB, 0.44764E+00_JPRB ,&
&  0.55186E+00_JPRB, 0.69709E+00_JPRB, 0.73080E+00_JPRB, 0.72668E+00_JPRB, 0.76235E+00_JPRB, 0.90367E+00_JPRB, 0.11582E+01_JPRB ,&
&  0.90593E+00_JPRB, 0.83295E+00_JPRB, 0.74553E+00_JPRB, 0.70007E+00_JPRB, 0.66868E+00_JPRB, 0.64315E+00_JPRB, 0.61684E+00_JPRB ,&
&  0.58630E+00_JPRB, 0.55807E+00_JPRB, 0.53002E+00_JPRB, 0.49993E+00_JPRB, 0.47747E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.44369E+00_JPRB, 0.43507E+00_JPRB, 0.42313E+00_JPRB, 0.42960E+00_JPRB, 0.45094E+00_JPRB, 0.43422E+00_JPRB, 0.43553E+00_JPRB ,&
&  0.42619E+00_JPRB, 0.41649E+00_JPRB, 0.39978E+00_JPRB, 0.38503E+00_JPRB, 0.37899E+00_JPRB, 0.40612E+00_JPRB, 0.45542E+00_JPRB ,&
&  0.52851E+00_JPRB, 0.63552E+00_JPRB, 0.77289E+00_JPRB, 0.67246E+00_JPRB, 0.59801E+00_JPRB, 0.53893E+00_JPRB, 0.50378E+00_JPRB ,&
&  0.73983E+00_JPRB, 0.99431E+00_JPRB, 0.12129E+01_JPRB, 0.14677E+01_JPRB, 0.18042E+01_JPRB, 0.18692E+01_JPRB, 0.20072E+01_JPRB ,&
&  0.19524E+01_JPRB, 0.16482E+01_JPRB, 0.13906E+01_JPRB, 0.11755E+01_JPRB, 0.10157E+01_JPRB, 0.86762E+00_JPRB, 0.76001E+00_JPRB ,&
&  0.69508E+00_JPRB, 0.65903E+00_JPRB, 0.62864E+00_JPRB, 0.66158E+00_JPRB, 0.72438E+00_JPRB, 0.76941E+00_JPRB, 0.80894E+00_JPRB ,&
&  0.84091E+00_JPRB, 0.85923E+00_JPRB, 0.87071E+00_JPRB, 0.88303E+00_JPRB, 0.91261E+00_JPRB, 0.87643E+00_JPRB, 0.83041E+00_JPRB ,&
&  0.84575E+00_JPRB, 0.98191E+00_JPRB, 0.15021E+01_JPRB, 0.17275E+01_JPRB, 0.15352E+01_JPRB, 0.15082E+01_JPRB, 0.14681E+01_JPRB ,&
&  0.13582E+01_JPRB, 0.13179E+01_JPRB, 0.98491E+00_JPRB, 0.96250E+00_JPRB, 0.82306E+00_JPRB, 0.59624E+00_JPRB, 0.61729E+00_JPRB ,&
&  0.64533E+00_JPRB, 0.60873E+00_JPRB, 0.52003E+00_JPRB, 0.43032E+00_JPRB, 0.43754E+00_JPRB, 0.37631E+00_JPRB, 0.31417E+00_JPRB ,&
&  0.30527E+00_JPRB, 0.29436E+00_JPRB, 0.27589E+00_JPRB, 0.27637E+00_JPRB, 0.27060E+00_JPRB, 0.29984E+00_JPRB, 0.38579E+00_JPRB ,&
&  0.49717E+00_JPRB, 0.62000E+00_JPRB, 0.68771E+00_JPRB, 0.69608E+00_JPRB, 0.69914E+00_JPRB, 0.72899E+00_JPRB, 0.93417E+00_JPRB ,&
&  0.79894E+00_JPRB, 0.82564E+00_JPRB, 0.71047E+00_JPRB, 0.67318E+00_JPRB, 0.63966E+00_JPRB, 0.61176E+00_JPRB, 0.58048E+00_JPRB ,&
&  0.55023E+00_JPRB, 0.52747E+00_JPRB, 0.50602E+00_JPRB, 0.47913E+00_JPRB, 0.45839E+00_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.41827E+00_JPRB, 0.40643E+00_JPRB, 0.38853E+00_JPRB, 0.39390E+00_JPRB, 0.41942E+00_JPRB, 0.41622E+00_JPRB, 0.41121E+00_JPRB ,&
&  0.41722E+00_JPRB, 0.40488E+00_JPRB, 0.39425E+00_JPRB, 0.38051E+00_JPRB, 0.37195E+00_JPRB, 0.38905E+00_JPRB, 0.43100E+00_JPRB ,&
&  0.51177E+00_JPRB, 0.64345E+00_JPRB, 0.55859E+00_JPRB, 0.49623E+00_JPRB, 0.49730E+00_JPRB, 0.49617E+00_JPRB, 0.43849E+00_JPRB ,&
&  0.62772E+00_JPRB, 0.78642E+00_JPRB, 0.96036E+00_JPRB, 0.13407E+01_JPRB, 0.20203E+01_JPRB, 0.21756E+01_JPRB, 0.22391E+01_JPRB ,&
&  0.21954E+01_JPRB, 0.20194E+01_JPRB, 0.16645E+01_JPRB, 0.13368E+01_JPRB, 0.11019E+01_JPRB, 0.94353E+00_JPRB, 0.79719E+00_JPRB ,&
&  0.69485E+00_JPRB, 0.65808E+00_JPRB, 0.64534E+00_JPRB, 0.63955E+00_JPRB, 0.67389E+00_JPRB, 0.70746E+00_JPRB, 0.72954E+00_JPRB ,&
&  0.76791E+00_JPRB, 0.80140E+00_JPRB, 0.81071E+00_JPRB, 0.81062E+00_JPRB, 0.80579E+00_JPRB, 0.75590E+00_JPRB, 0.10351E+01_JPRB ,&
&  0.11017E+01_JPRB, 0.11821E+01_JPRB, 0.12515E+01_JPRB, 0.16523E+01_JPRB, 0.16352E+01_JPRB, 0.14721E+01_JPRB, 0.16317E+01_JPRB ,&
&  0.12010E+01_JPRB, 0.10054E+01_JPRB, 0.91827E+00_JPRB, 0.84226E+00_JPRB, 0.64431E+00_JPRB, 0.64232E+00_JPRB, 0.79403E+00_JPRB ,&
&  0.63776E+00_JPRB, 0.66290E+00_JPRB, 0.67033E+00_JPRB, 0.63005E+00_JPRB, 0.37462E+00_JPRB, 0.32553E+00_JPRB, 0.42242E+00_JPRB ,&
&  0.44406E+00_JPRB, 0.34601E+00_JPRB, 0.28897E+00_JPRB, 0.32713E+00_JPRB, 0.27835E+00_JPRB, 0.30384E+00_JPRB, 0.38759E+00_JPRB ,&
&  0.44276E+00_JPRB, 0.50828E+00_JPRB, 0.63801E+00_JPRB, 0.65481E+00_JPRB, 0.63737E+00_JPRB, 0.65071E+00_JPRB, 0.74419E+00_JPRB ,&
&  0.75583E+00_JPRB, 0.80915E+00_JPRB, 0.74662E+00_JPRB, 0.64933E+00_JPRB, 0.58650E+00_JPRB, 0.55482E+00_JPRB, 0.53531E+00_JPRB ,&
&  0.51384E+00_JPRB, 0.49062E+00_JPRB, 0.46835E+00_JPRB, 0.44754E+00_JPRB, 0.43101E+00_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.38010E+00_JPRB, 0.37398E+00_JPRB, 0.35964E+00_JPRB, 0.36445E+00_JPRB, 0.38546E+00_JPRB, 0.39403E+00_JPRB, 0.39416E+00_JPRB ,&
&  0.39679E+00_JPRB, 0.38834E+00_JPRB, 0.38255E+00_JPRB, 0.37294E+00_JPRB, 0.36568E+00_JPRB, 0.37754E+00_JPRB, 0.40645E+00_JPRB ,&
&  0.46546E+00_JPRB, 0.49268E+00_JPRB, 0.43683E+00_JPRB, 0.40822E+00_JPRB, 0.49218E+00_JPRB, 0.46036E+00_JPRB, 0.46882E+00_JPRB ,&
&  0.56308E+00_JPRB, 0.64542E+00_JPRB, 0.78072E+00_JPRB, 0.10860E+01_JPRB, 0.14685E+01_JPRB, 0.20377E+01_JPRB, 0.23117E+01_JPRB ,&
&  0.23227E+01_JPRB, 0.22665E+01_JPRB, 0.19628E+01_JPRB, 0.15952E+01_JPRB, 0.12554E+01_JPRB, 0.10430E+01_JPRB, 0.88814E+00_JPRB ,&
&  0.75907E+00_JPRB, 0.66832E+00_JPRB, 0.63861E+00_JPRB, 0.64159E+00_JPRB, 0.65293E+00_JPRB, 0.66544E+00_JPRB, 0.65913E+00_JPRB ,&
&  0.67112E+00_JPRB, 0.71023E+00_JPRB, 0.73056E+00_JPRB, 0.73168E+00_JPRB, 0.69673E+00_JPRB, 0.63483E+00_JPRB, 0.76899E+00_JPRB ,&
&  0.10742E+01_JPRB, 0.11215E+01_JPRB, 0.12638E+01_JPRB, 0.15365E+01_JPRB, 0.16755E+01_JPRB, 0.15039E+01_JPRB, 0.18469E+01_JPRB ,&
&  0.14689E+01_JPRB, 0.11292E+01_JPRB, 0.88941E+00_JPRB, 0.62952E+00_JPRB, 0.63401E+00_JPRB, 0.82382E+00_JPRB, 0.83124E+00_JPRB ,&
&  0.77315E+00_JPRB, 0.73499E+00_JPRB, 0.69783E+00_JPRB, 0.59854E+00_JPRB, 0.47565E+00_JPRB, 0.37763E+00_JPRB, 0.41527E+00_JPRB ,&
&  0.43279E+00_JPRB, 0.44444E+00_JPRB, 0.42611E+00_JPRB, 0.38878E+00_JPRB, 0.33351E+00_JPRB, 0.37241E+00_JPRB, 0.39876E+00_JPRB ,&
&  0.42701E+00_JPRB, 0.42632E+00_JPRB, 0.48391E+00_JPRB, 0.57057E+00_JPRB, 0.59235E+00_JPRB, 0.58298E+00_JPRB, 0.64599E+00_JPRB ,&
&  0.62999E+00_JPRB, 0.65700E+00_JPRB, 0.59749E+00_JPRB, 0.53942E+00_JPRB, 0.50928E+00_JPRB, 0.49108E+00_JPRB, 0.48011E+00_JPRB ,&
&  0.45991E+00_JPRB, 0.44238E+00_JPRB, 0.42456E+00_JPRB, 0.40282E+00_JPRB, 0.38685E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.35126E+00_JPRB, 0.34666E+00_JPRB, 0.33310E+00_JPRB, 0.33524E+00_JPRB, 0.35151E+00_JPRB, 0.36854E+00_JPRB, 0.37869E+00_JPRB ,&
&  0.37924E+00_JPRB, 0.37835E+00_JPRB, 0.37178E+00_JPRB, 0.36656E+00_JPRB, 0.35970E+00_JPRB, 0.36059E+00_JPRB, 0.38462E+00_JPRB ,&
&  0.43468E+00_JPRB, 0.43063E+00_JPRB, 0.37093E+00_JPRB, 0.40496E+00_JPRB, 0.44992E+00_JPRB, 0.42243E+00_JPRB, 0.46899E+00_JPRB ,&
&  0.51541E+00_JPRB, 0.57535E+00_JPRB, 0.64412E+00_JPRB, 0.81810E+00_JPRB, 0.10740E+01_JPRB, 0.14492E+01_JPRB, 0.18282E+01_JPRB ,&
&  0.19625E+01_JPRB, 0.20845E+01_JPRB, 0.21021E+01_JPRB, 0.18918E+01_JPRB, 0.15229E+01_JPRB, 0.12234E+01_JPRB, 0.10481E+01_JPRB ,&
&  0.88851E+00_JPRB, 0.74602E+00_JPRB, 0.66491E+00_JPRB, 0.63922E+00_JPRB, 0.64424E+00_JPRB, 0.65829E+00_JPRB, 0.66861E+00_JPRB ,&
&  0.64128E+00_JPRB, 0.63246E+00_JPRB, 0.64122E+00_JPRB, 0.71006E+00_JPRB, 0.60497E+00_JPRB, 0.61277E+00_JPRB, 0.70180E+00_JPRB ,&
&  0.11899E+01_JPRB, 0.13543E+01_JPRB, 0.14286E+01_JPRB, 0.18531E+01_JPRB, 0.16102E+01_JPRB, 0.16408E+01_JPRB, 0.19290E+01_JPRB ,&
&  0.20825E+01_JPRB, 0.20025E+01_JPRB, 0.18538E+01_JPRB, 0.10088E+01_JPRB, 0.71429E+00_JPRB, 0.92481E+00_JPRB, 0.88033E+00_JPRB ,&
&  0.80000E+00_JPRB, 0.76735E+00_JPRB, 0.70855E+00_JPRB, 0.69688E+00_JPRB, 0.53423E+00_JPRB, 0.43707E+00_JPRB, 0.35421E+00_JPRB ,&
&  0.34863E+00_JPRB, 0.34168E+00_JPRB, 0.37971E+00_JPRB, 0.36741E+00_JPRB, 0.36223E+00_JPRB, 0.36634E+00_JPRB, 0.36485E+00_JPRB ,&
&  0.38073E+00_JPRB, 0.38125E+00_JPRB, 0.39255E+00_JPRB, 0.46412E+00_JPRB, 0.52477E+00_JPRB, 0.50302E+00_JPRB, 0.54905E+00_JPRB ,&
&  0.55229E+00_JPRB, 0.52933E+00_JPRB, 0.50123E+00_JPRB, 0.47090E+00_JPRB, 0.45779E+00_JPRB, 0.44499E+00_JPRB, 0.42938E+00_JPRB ,&
&  0.41015E+00_JPRB, 0.39512E+00_JPRB, 0.37988E+00_JPRB, 0.36621E+00_JPRB, 0.35510E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.33605E+00_JPRB, 0.33272E+00_JPRB, 0.31519E+00_JPRB, 0.31447E+00_JPRB, 0.32469E+00_JPRB, 0.34096E+00_JPRB, 0.35484E+00_JPRB ,&
&  0.36217E+00_JPRB, 0.36239E+00_JPRB, 0.35883E+00_JPRB, 0.35732E+00_JPRB, 0.35055E+00_JPRB, 0.34960E+00_JPRB, 0.37345E+00_JPRB ,&
&  0.41673E+00_JPRB, 0.47712E+00_JPRB, 0.40926E+00_JPRB, 0.36414E+00_JPRB, 0.33629E+00_JPRB, 0.38701E+00_JPRB, 0.43457E+00_JPRB ,&
&  0.50943E+00_JPRB, 0.54947E+00_JPRB, 0.62353E+00_JPRB, 0.70562E+00_JPRB, 0.78256E+00_JPRB, 0.99188E+00_JPRB, 0.13354E+01_JPRB ,&
&  0.18400E+01_JPRB, 0.18341E+01_JPRB, 0.19655E+01_JPRB, 0.18392E+01_JPRB, 0.16403E+01_JPRB, 0.14094E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.10204E+01_JPRB, 0.85234E+00_JPRB, 0.72823E+00_JPRB, 0.64984E+00_JPRB, 0.62679E+00_JPRB, 0.62363E+00_JPRB, 0.64962E+00_JPRB ,&
&  0.65298E+00_JPRB, 0.63224E+00_JPRB, 0.60749E+00_JPRB, 0.61707E+00_JPRB, 0.67965E+00_JPRB, 0.74078E+00_JPRB, 0.91820E+00_JPRB ,&
&  0.11204E+01_JPRB, 0.11453E+01_JPRB, 0.15679E+01_JPRB, 0.18808E+01_JPRB, 0.22395E+01_JPRB, 0.23156E+01_JPRB, 0.21830E+01_JPRB ,&
&  0.21344E+01_JPRB, 0.19134E+01_JPRB, 0.17167E+01_JPRB, 0.12037E+01_JPRB, 0.12398E+01_JPRB, 0.11590E+01_JPRB, 0.10077E+01_JPRB ,&
&  0.88387E+00_JPRB, 0.78958E+00_JPRB, 0.72531E+00_JPRB, 0.68749E+00_JPRB, 0.65623E+00_JPRB, 0.62064E+00_JPRB, 0.47241E+00_JPRB ,&
&  0.38225E+00_JPRB, 0.40029E+00_JPRB, 0.37655E+00_JPRB, 0.34531E+00_JPRB, 0.31870E+00_JPRB, 0.33237E+00_JPRB, 0.33968E+00_JPRB ,&
&  0.34601E+00_JPRB, 0.34167E+00_JPRB, 0.35005E+00_JPRB, 0.40138E+00_JPRB, 0.46822E+00_JPRB, 0.49209E+00_JPRB, 0.49757E+00_JPRB ,&
&  0.46781E+00_JPRB, 0.46842E+00_JPRB, 0.45612E+00_JPRB, 0.42015E+00_JPRB, 0.42846E+00_JPRB, 0.41146E+00_JPRB, 0.38911E+00_JPRB ,&
&  0.37036E+00_JPRB, 0.36062E+00_JPRB, 0.35161E+00_JPRB, 0.34404E+00_JPRB, 0.33598E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.32210E+00_JPRB, 0.32307E+00_JPRB, 0.31079E+00_JPRB, 0.30846E+00_JPRB, 0.31480E+00_JPRB, 0.32529E+00_JPRB, 0.33573E+00_JPRB ,&
&  0.34396E+00_JPRB, 0.34357E+00_JPRB, 0.34051E+00_JPRB, 0.34142E+00_JPRB, 0.34134E+00_JPRB, 0.34525E+00_JPRB, 0.37516E+00_JPRB ,&
&  0.43921E+00_JPRB, 0.58844E+00_JPRB, 0.46609E+00_JPRB, 0.42906E+00_JPRB, 0.36989E+00_JPRB, 0.38875E+00_JPRB, 0.42602E+00_JPRB ,&
&  0.54160E+00_JPRB, 0.53984E+00_JPRB, 0.53533E+00_JPRB, 0.56592E+00_JPRB, 0.62401E+00_JPRB, 0.74104E+00_JPRB, 0.97171E+00_JPRB ,&
&  0.12027E+01_JPRB, 0.15103E+01_JPRB, 0.16184E+01_JPRB, 0.17448E+01_JPRB, 0.15929E+01_JPRB, 0.13258E+01_JPRB, 0.12534E+01_JPRB ,&
&  0.11010E+01_JPRB, 0.94025E+00_JPRB, 0.80007E+00_JPRB, 0.71123E+00_JPRB, 0.65896E+00_JPRB, 0.62403E+00_JPRB, 0.62246E+00_JPRB ,&
&  0.62861E+00_JPRB, 0.62295E+00_JPRB, 0.60873E+00_JPRB, 0.60415E+00_JPRB, 0.66940E+00_JPRB, 0.85577E+00_JPRB, 0.10306E+01_JPRB ,&
&  0.15584E+01_JPRB, 0.16802E+01_JPRB, 0.14590E+01_JPRB, 0.19925E+01_JPRB, 0.26633E+01_JPRB, 0.25169E+01_JPRB, 0.20601E+01_JPRB ,&
&  0.19884E+01_JPRB, 0.17778E+01_JPRB, 0.16066E+01_JPRB, 0.14680E+01_JPRB, 0.13407E+01_JPRB, 0.12086E+01_JPRB, 0.10530E+01_JPRB ,&
&  0.89294E+00_JPRB, 0.78244E+00_JPRB, 0.70896E+00_JPRB, 0.67470E+00_JPRB, 0.61868E+00_JPRB, 0.56599E+00_JPRB, 0.52090E+00_JPRB ,&
&  0.44593E+00_JPRB, 0.38613E+00_JPRB, 0.31010E+00_JPRB, 0.32056E+00_JPRB, 0.28179E+00_JPRB, 0.28923E+00_JPRB, 0.31563E+00_JPRB ,&
&  0.31642E+00_JPRB, 0.33144E+00_JPRB, 0.34429E+00_JPRB, 0.35717E+00_JPRB, 0.41353E+00_JPRB, 0.43791E+00_JPRB, 0.44512E+00_JPRB ,&
&  0.43310E+00_JPRB, 0.39754E+00_JPRB, 0.41131E+00_JPRB, 0.40141E+00_JPRB, 0.39990E+00_JPRB, 0.37233E+00_JPRB, 0.34515E+00_JPRB ,&
&  0.32881E+00_JPRB, 0.32337E+00_JPRB, 0.32071E+00_JPRB, 0.31895E+00_JPRB, 0.31810E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.29919E+00_JPRB, 0.29938E+00_JPRB, 0.29567E+00_JPRB, 0.29953E+00_JPRB, 0.30370E+00_JPRB, 0.30923E+00_JPRB, 0.31937E+00_JPRB ,&
&  0.33060E+00_JPRB, 0.33105E+00_JPRB, 0.32489E+00_JPRB, 0.32360E+00_JPRB, 0.32636E+00_JPRB, 0.33550E+00_JPRB, 0.37855E+00_JPRB ,&
&  0.48022E+00_JPRB, 0.51461E+00_JPRB, 0.42586E+00_JPRB, 0.35383E+00_JPRB, 0.39515E+00_JPRB, 0.38371E+00_JPRB, 0.41311E+00_JPRB ,&
&  0.40134E+00_JPRB, 0.43357E+00_JPRB, 0.42403E+00_JPRB, 0.42373E+00_JPRB, 0.44238E+00_JPRB, 0.52798E+00_JPRB, 0.66702E+00_JPRB ,&
&  0.80386E+00_JPRB, 0.10075E+01_JPRB, 0.12208E+01_JPRB, 0.12463E+01_JPRB, 0.12937E+01_JPRB, 0.11550E+01_JPRB, 0.11224E+01_JPRB ,&
&  0.10212E+01_JPRB, 0.89325E+00_JPRB, 0.78007E+00_JPRB, 0.70875E+00_JPRB, 0.66245E+00_JPRB, 0.63048E+00_JPRB, 0.61861E+00_JPRB ,&
&  0.62161E+00_JPRB, 0.61902E+00_JPRB, 0.60280E+00_JPRB, 0.61160E+00_JPRB, 0.72956E+00_JPRB, 0.10254E+01_JPRB, 0.14243E+01_JPRB ,&
&  0.19535E+01_JPRB, 0.22065E+01_JPRB, 0.22943E+01_JPRB, 0.24676E+01_JPRB, 0.24695E+01_JPRB, 0.23899E+01_JPRB, 0.21709E+01_JPRB ,&
&  0.18546E+01_JPRB, 0.15836E+01_JPRB, 0.14134E+01_JPRB, 0.12643E+01_JPRB, 0.11536E+01_JPRB, 0.10938E+01_JPRB, 0.99694E+00_JPRB ,&
&  0.84205E+00_JPRB, 0.74650E+00_JPRB, 0.68624E+00_JPRB, 0.60588E+00_JPRB, 0.56072E+00_JPRB, 0.51570E+00_JPRB, 0.50590E+00_JPRB ,&
&  0.50000E+00_JPRB, 0.33216E+00_JPRB, 0.29129E+00_JPRB, 0.31962E+00_JPRB, 0.28890E+00_JPRB, 0.29846E+00_JPRB, 0.32889E+00_JPRB ,&
&  0.32866E+00_JPRB, 0.33121E+00_JPRB, 0.34954E+00_JPRB, 0.35760E+00_JPRB, 0.37560E+00_JPRB, 0.40990E+00_JPRB, 0.40093E+00_JPRB ,&
&  0.38104E+00_JPRB, 0.36779E+00_JPRB, 0.38590E+00_JPRB, 0.38317E+00_JPRB, 0.36820E+00_JPRB, 0.33089E+00_JPRB, 0.30542E+00_JPRB ,&
&  0.29553E+00_JPRB, 0.29416E+00_JPRB, 0.29387E+00_JPRB, 0.29362E+00_JPRB, 0.29414E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.28231E+00_JPRB, 0.28189E+00_JPRB, 0.28136E+00_JPRB, 0.28926E+00_JPRB, 0.29841E+00_JPRB, 0.30247E+00_JPRB, 0.31193E+00_JPRB ,&
&  0.32146E+00_JPRB, 0.32342E+00_JPRB, 0.32004E+00_JPRB, 0.31584E+00_JPRB, 0.31653E+00_JPRB, 0.32854E+00_JPRB, 0.37756E+00_JPRB ,&
&  0.35134E+00_JPRB, 0.36570E+00_JPRB, 0.32166E+00_JPRB, 0.33078E+00_JPRB, 0.38948E+00_JPRB, 0.35205E+00_JPRB, 0.34370E+00_JPRB ,&
&  0.33191E+00_JPRB, 0.32565E+00_JPRB, 0.31256E+00_JPRB, 0.31043E+00_JPRB, 0.32201E+00_JPRB, 0.36756E+00_JPRB, 0.46508E+00_JPRB ,&
&  0.52878E+00_JPRB, 0.64052E+00_JPRB, 0.76764E+00_JPRB, 0.78649E+00_JPRB, 0.85447E+00_JPRB, 0.90335E+00_JPRB, 0.93803E+00_JPRB ,&
&  0.89543E+00_JPRB, 0.79252E+00_JPRB, 0.70201E+00_JPRB, 0.64999E+00_JPRB, 0.61527E+00_JPRB, 0.59609E+00_JPRB, 0.58127E+00_JPRB ,&
&  0.56888E+00_JPRB, 0.57082E+00_JPRB, 0.59566E+00_JPRB, 0.65561E+00_JPRB, 0.75631E+00_JPRB, 0.11565E+01_JPRB, 0.17861E+01_JPRB ,&
&  0.21502E+01_JPRB, 0.22631E+01_JPRB, 0.23505E+01_JPRB, 0.25135E+01_JPRB, 0.24437E+01_JPRB, 0.22037E+01_JPRB, 0.18291E+01_JPRB ,&
&  0.15232E+01_JPRB, 0.13103E+01_JPRB, 0.11981E+01_JPRB, 0.11185E+01_JPRB, 0.98729E+00_JPRB, 0.97977E+00_JPRB, 0.90299E+00_JPRB ,&
&  0.78259E+00_JPRB, 0.71531E+00_JPRB, 0.66445E+00_JPRB, 0.63192E+00_JPRB, 0.59134E+00_JPRB, 0.58609E+00_JPRB, 0.55929E+00_JPRB ,&
&  0.53620E+00_JPRB, 0.46565E+00_JPRB, 0.37497E+00_JPRB, 0.35615E+00_JPRB, 0.30954E+00_JPRB, 0.32073E+00_JPRB, 0.36474E+00_JPRB ,&
&  0.36247E+00_JPRB, 0.35574E+00_JPRB, 0.35359E+00_JPRB, 0.35321E+00_JPRB, 0.35256E+00_JPRB, 0.37726E+00_JPRB, 0.37929E+00_JPRB ,&
&  0.35514E+00_JPRB, 0.36309E+00_JPRB, 0.36652E+00_JPRB, 0.34915E+00_JPRB, 0.32602E+00_JPRB, 0.29331E+00_JPRB, 0.27301E+00_JPRB ,&
&  0.27246E+00_JPRB, 0.27171E+00_JPRB, 0.27548E+00_JPRB, 0.27733E+00_JPRB, 0.27894E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.27095E+00_JPRB, 0.27173E+00_JPRB, 0.27251E+00_JPRB, 0.27883E+00_JPRB, 0.29146E+00_JPRB, 0.29769E+00_JPRB, 0.30736E+00_JPRB ,&
&  0.31407E+00_JPRB, 0.31411E+00_JPRB, 0.31686E+00_JPRB, 0.31152E+00_JPRB, 0.31178E+00_JPRB, 0.32127E+00_JPRB, 0.31994E+00_JPRB ,&
&  0.28945E+00_JPRB, 0.30367E+00_JPRB, 0.29398E+00_JPRB, 0.29419E+00_JPRB, 0.29456E+00_JPRB, 0.28113E+00_JPRB, 0.27026E+00_JPRB ,&
&  0.27556E+00_JPRB, 0.27015E+00_JPRB, 0.26986E+00_JPRB, 0.27791E+00_JPRB, 0.29228E+00_JPRB, 0.32345E+00_JPRB, 0.38222E+00_JPRB ,&
&  0.41896E+00_JPRB, 0.47949E+00_JPRB, 0.53824E+00_JPRB, 0.54955E+00_JPRB, 0.63546E+00_JPRB, 0.70783E+00_JPRB, 0.72991E+00_JPRB ,&
&  0.73220E+00_JPRB, 0.68775E+00_JPRB, 0.63303E+00_JPRB, 0.58997E+00_JPRB, 0.55020E+00_JPRB, 0.52874E+00_JPRB, 0.51966E+00_JPRB ,&
&  0.51321E+00_JPRB, 0.51439E+00_JPRB, 0.53048E+00_JPRB, 0.57009E+00_JPRB, 0.64810E+00_JPRB, 0.94539E+00_JPRB, 0.12779E+01_JPRB ,&
&  0.14382E+01_JPRB, 0.15430E+01_JPRB, 0.16201E+01_JPRB, 0.16326E+01_JPRB, 0.16126E+01_JPRB, 0.14671E+01_JPRB, 0.12532E+01_JPRB ,&
&  0.11474E+01_JPRB, 0.10297E+01_JPRB, 0.10631E+01_JPRB, 0.98916E+00_JPRB, 0.90519E+00_JPRB, 0.87055E+00_JPRB, 0.80501E+00_JPRB ,&
&  0.75515E+00_JPRB, 0.69818E+00_JPRB, 0.66638E+00_JPRB, 0.63295E+00_JPRB, 0.60250E+00_JPRB, 0.59885E+00_JPRB, 0.55913E+00_JPRB ,&
&  0.54312E+00_JPRB, 0.52490E+00_JPRB, 0.46291E+00_JPRB, 0.41981E+00_JPRB, 0.38131E+00_JPRB, 0.39237E+00_JPRB, 0.37569E+00_JPRB ,&
&  0.35565E+00_JPRB, 0.31993E+00_JPRB, 0.33077E+00_JPRB, 0.33762E+00_JPRB, 0.34219E+00_JPRB, 0.34267E+00_JPRB, 0.34383E+00_JPRB ,&
&  0.34786E+00_JPRB, 0.35637E+00_JPRB, 0.32911E+00_JPRB, 0.30436E+00_JPRB, 0.28499E+00_JPRB, 0.26615E+00_JPRB, 0.24717E+00_JPRB ,&
&  0.24430E+00_JPRB, 0.25400E+00_JPRB, 0.26071E+00_JPRB, 0.26580E+00_JPRB, 0.26958E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.26457E+00_JPRB, 0.26829E+00_JPRB, 0.27125E+00_JPRB, 0.27415E+00_JPRB, 0.28520E+00_JPRB, 0.29221E+00_JPRB, 0.29327E+00_JPRB ,&
&  0.30802E+00_JPRB, 0.30692E+00_JPRB, 0.30766E+00_JPRB, 0.30578E+00_JPRB, 0.30481E+00_JPRB, 0.28126E+00_JPRB, 0.23831E+00_JPRB ,&
&  0.22805E+00_JPRB, 0.23712E+00_JPRB, 0.25326E+00_JPRB, 0.25928E+00_JPRB, 0.25518E+00_JPRB, 0.24689E+00_JPRB, 0.24045E+00_JPRB ,&
&  0.24079E+00_JPRB, 0.24343E+00_JPRB, 0.25261E+00_JPRB, 0.26304E+00_JPRB, 0.26963E+00_JPRB, 0.28027E+00_JPRB, 0.30669E+00_JPRB ,&
&  0.33040E+00_JPRB, 0.36734E+00_JPRB, 0.41932E+00_JPRB, 0.40313E+00_JPRB, 0.47922E+00_JPRB, 0.53458E+00_JPRB, 0.53436E+00_JPRB ,&
&  0.55024E+00_JPRB, 0.53915E+00_JPRB, 0.51549E+00_JPRB, 0.50411E+00_JPRB, 0.48029E+00_JPRB, 0.45708E+00_JPRB, 0.44506E+00_JPRB ,&
&  0.44059E+00_JPRB, 0.44255E+00_JPRB, 0.45561E+00_JPRB, 0.47898E+00_JPRB, 0.53590E+00_JPRB, 0.70274E+00_JPRB, 0.83485E+00_JPRB ,&
&  0.89314E+00_JPRB, 0.98775E+00_JPRB, 0.11163E+01_JPRB, 0.10888E+01_JPRB, 0.11091E+01_JPRB, 0.10836E+01_JPRB, 0.10076E+01_JPRB ,&
&  0.97937E+00_JPRB, 0.88463E+00_JPRB, 0.86647E+00_JPRB, 0.81755E+00_JPRB, 0.78287E+00_JPRB, 0.74105E+00_JPRB, 0.69344E+00_JPRB ,&
&  0.67909E+00_JPRB, 0.67287E+00_JPRB, 0.59886E+00_JPRB, 0.55590E+00_JPRB, 0.51675E+00_JPRB, 0.50463E+00_JPRB, 0.48327E+00_JPRB ,&
&  0.47802E+00_JPRB, 0.47747E+00_JPRB, 0.46434E+00_JPRB, 0.44714E+00_JPRB, 0.41923E+00_JPRB, 0.39238E+00_JPRB, 0.36883E+00_JPRB ,&
&  0.35411E+00_JPRB, 0.32806E+00_JPRB, 0.32029E+00_JPRB, 0.32767E+00_JPRB, 0.32877E+00_JPRB, 0.31011E+00_JPRB, 0.31275E+00_JPRB ,&
&  0.31974E+00_JPRB, 0.31170E+00_JPRB, 0.29335E+00_JPRB, 0.27839E+00_JPRB, 0.26971E+00_JPRB, 0.25733E+00_JPRB, 0.24257E+00_JPRB ,&
&  0.23254E+00_JPRB, 0.24064E+00_JPRB, 0.24691E+00_JPRB, 0.25242E+00_JPRB, 0.25949E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.25964E+00_JPRB, 0.26802E+00_JPRB, 0.27062E+00_JPRB, 0.27087E+00_JPRB, 0.27670E+00_JPRB, 0.27484E+00_JPRB, 0.27694E+00_JPRB ,&
&  0.29650E+00_JPRB, 0.28691E+00_JPRB, 0.30191E+00_JPRB, 0.25699E+00_JPRB, 0.23514E+00_JPRB, 0.23018E+00_JPRB, 0.22288E+00_JPRB ,&
&  0.22302E+00_JPRB, 0.22873E+00_JPRB, 0.23289E+00_JPRB, 0.23543E+00_JPRB, 0.23751E+00_JPRB, 0.23157E+00_JPRB, 0.22745E+00_JPRB ,&
&  0.22821E+00_JPRB, 0.23412E+00_JPRB, 0.24361E+00_JPRB, 0.24506E+00_JPRB, 0.24402E+00_JPRB, 0.24501E+00_JPRB, 0.25243E+00_JPRB ,&
&  0.25709E+00_JPRB, 0.27863E+00_JPRB, 0.32122E+00_JPRB, 0.34081E+00_JPRB, 0.38268E+00_JPRB, 0.43258E+00_JPRB, 0.43032E+00_JPRB ,&
&  0.42344E+00_JPRB, 0.37314E+00_JPRB, 0.39541E+00_JPRB, 0.40262E+00_JPRB, 0.40367E+00_JPRB, 0.39398E+00_JPRB, 0.38296E+00_JPRB ,&
&  0.37675E+00_JPRB, 0.37773E+00_JPRB, 0.38605E+00_JPRB, 0.40131E+00_JPRB, 0.43074E+00_JPRB, 0.48687E+00_JPRB, 0.53265E+00_JPRB ,&
&  0.59969E+00_JPRB, 0.58646E+00_JPRB, 0.71931E+00_JPRB, 0.73858E+00_JPRB, 0.79685E+00_JPRB, 0.83026E+00_JPRB, 0.86451E+00_JPRB ,&
&  0.90901E+00_JPRB, 0.82496E+00_JPRB, 0.78780E+00_JPRB, 0.71108E+00_JPRB, 0.66513E+00_JPRB, 0.62026E+00_JPRB, 0.58564E+00_JPRB ,&
&  0.56875E+00_JPRB, 0.52329E+00_JPRB, 0.50244E+00_JPRB, 0.45913E+00_JPRB, 0.43064E+00_JPRB, 0.42182E+00_JPRB, 0.41675E+00_JPRB ,&
&  0.41642E+00_JPRB, 0.41445E+00_JPRB, 0.40146E+00_JPRB, 0.36654E+00_JPRB, 0.35568E+00_JPRB, 0.34855E+00_JPRB, 0.33622E+00_JPRB ,&
&  0.34029E+00_JPRB, 0.33672E+00_JPRB, 0.33160E+00_JPRB, 0.34127E+00_JPRB, 0.32436E+00_JPRB, 0.31308E+00_JPRB, 0.30601E+00_JPRB ,&
&  0.30293E+00_JPRB, 0.28146E+00_JPRB, 0.26841E+00_JPRB, 0.26265E+00_JPRB, 0.25357E+00_JPRB, 0.24415E+00_JPRB, 0.23561E+00_JPRB ,&
&  0.22883E+00_JPRB, 0.23139E+00_JPRB, 0.23786E+00_JPRB, 0.24341E+00_JPRB, 0.25061E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.25215E+00_JPRB, 0.26362E+00_JPRB, 0.26745E+00_JPRB, 0.26797E+00_JPRB, 0.27068E+00_JPRB, 0.26747E+00_JPRB, 0.25744E+00_JPRB ,&
&  0.25972E+00_JPRB, 0.24981E+00_JPRB, 0.23715E+00_JPRB, 0.22914E+00_JPRB, 0.22588E+00_JPRB, 0.22203E+00_JPRB, 0.21428E+00_JPRB ,&
&  0.21239E+00_JPRB, 0.21554E+00_JPRB, 0.22152E+00_JPRB, 0.22293E+00_JPRB, 0.22159E+00_JPRB, 0.21900E+00_JPRB, 0.21841E+00_JPRB ,&
&  0.22174E+00_JPRB, 0.22796E+00_JPRB, 0.23362E+00_JPRB, 0.23446E+00_JPRB, 0.23289E+00_JPRB, 0.23406E+00_JPRB, 0.23819E+00_JPRB ,&
&  0.23955E+00_JPRB, 0.24191E+00_JPRB, 0.25671E+00_JPRB, 0.28283E+00_JPRB, 0.33186E+00_JPRB, 0.37663E+00_JPRB, 0.37129E+00_JPRB ,&
&  0.27115E+00_JPRB, 0.26045E+00_JPRB, 0.31315E+00_JPRB, 0.32495E+00_JPRB, 0.33397E+00_JPRB, 0.33225E+00_JPRB, 0.33071E+00_JPRB ,&
&  0.32888E+00_JPRB, 0.32897E+00_JPRB, 0.32817E+00_JPRB, 0.33465E+00_JPRB, 0.35239E+00_JPRB, 0.37749E+00_JPRB, 0.39494E+00_JPRB ,&
&  0.43155E+00_JPRB, 0.42242E+00_JPRB, 0.43744E+00_JPRB, 0.46142E+00_JPRB, 0.58094E+00_JPRB, 0.65024E+00_JPRB, 0.70627E+00_JPRB ,&
&  0.72956E+00_JPRB, 0.72196E+00_JPRB, 0.73680E+00_JPRB, 0.69953E+00_JPRB, 0.66414E+00_JPRB, 0.60244E+00_JPRB, 0.55466E+00_JPRB ,&
&  0.52132E+00_JPRB, 0.48519E+00_JPRB, 0.45594E+00_JPRB, 0.42615E+00_JPRB, 0.39094E+00_JPRB, 0.37220E+00_JPRB, 0.36395E+00_JPRB ,&
&  0.36429E+00_JPRB, 0.36384E+00_JPRB, 0.34893E+00_JPRB, 0.33127E+00_JPRB, 0.31971E+00_JPRB, 0.31903E+00_JPRB, 0.32169E+00_JPRB ,&
&  0.32425E+00_JPRB, 0.33014E+00_JPRB, 0.33109E+00_JPRB, 0.32953E+00_JPRB, 0.31712E+00_JPRB, 0.30037E+00_JPRB, 0.29042E+00_JPRB ,&
&  0.28281E+00_JPRB, 0.26062E+00_JPRB, 0.24830E+00_JPRB, 0.24531E+00_JPRB, 0.24689E+00_JPRB, 0.24395E+00_JPRB, 0.23368E+00_JPRB ,&
&  0.22687E+00_JPRB, 0.22447E+00_JPRB, 0.22393E+00_JPRB, 0.22955E+00_JPRB, 0.23907E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.23840E+00_JPRB, 0.24635E+00_JPRB, 0.25301E+00_JPRB, 0.25752E+00_JPRB, 0.25622E+00_JPRB, 0.25187E+00_JPRB, 0.24621E+00_JPRB ,&
&  0.24823E+00_JPRB, 0.24478E+00_JPRB, 0.23814E+00_JPRB, 0.22804E+00_JPRB, 0.22049E+00_JPRB, 0.21403E+00_JPRB, 0.21328E+00_JPRB ,&
&  0.21578E+00_JPRB, 0.21836E+00_JPRB, 0.21774E+00_JPRB, 0.21428E+00_JPRB, 0.21125E+00_JPRB, 0.21047E+00_JPRB, 0.21334E+00_JPRB ,&
&  0.21962E+00_JPRB, 0.22552E+00_JPRB, 0.22817E+00_JPRB, 0.22911E+00_JPRB, 0.23132E+00_JPRB, 0.23488E+00_JPRB, 0.23625E+00_JPRB ,&
&  0.23282E+00_JPRB, 0.23202E+00_JPRB, 0.23602E+00_JPRB, 0.24949E+00_JPRB, 0.28726E+00_JPRB, 0.32346E+00_JPRB, 0.28579E+00_JPRB ,&
&  0.22513E+00_JPRB, 0.20639E+00_JPRB, 0.21352E+00_JPRB, 0.25712E+00_JPRB, 0.27036E+00_JPRB, 0.27872E+00_JPRB, 0.28702E+00_JPRB ,&
&  0.27957E+00_JPRB, 0.27194E+00_JPRB, 0.27531E+00_JPRB, 0.29437E+00_JPRB, 0.30417E+00_JPRB, 0.31762E+00_JPRB, 0.33229E+00_JPRB ,&
&  0.35156E+00_JPRB, 0.37195E+00_JPRB, 0.39053E+00_JPRB, 0.36941E+00_JPRB, 0.41944E+00_JPRB, 0.54362E+00_JPRB, 0.57044E+00_JPRB ,&
&  0.58748E+00_JPRB, 0.59417E+00_JPRB, 0.65668E+00_JPRB, 0.64457E+00_JPRB, 0.60971E+00_JPRB, 0.56869E+00_JPRB, 0.53447E+00_JPRB ,&
&  0.49833E+00_JPRB, 0.46353E+00_JPRB, 0.43440E+00_JPRB, 0.41000E+00_JPRB, 0.38221E+00_JPRB, 0.35837E+00_JPRB, 0.33920E+00_JPRB ,&
&  0.33289E+00_JPRB, 0.32848E+00_JPRB, 0.31570E+00_JPRB, 0.30102E+00_JPRB, 0.29458E+00_JPRB, 0.29411E+00_JPRB, 0.30100E+00_JPRB ,&
&  0.30842E+00_JPRB, 0.31799E+00_JPRB, 0.32542E+00_JPRB, 0.32887E+00_JPRB, 0.30811E+00_JPRB, 0.28282E+00_JPRB, 0.27040E+00_JPRB ,&
&  0.26731E+00_JPRB, 0.25729E+00_JPRB, 0.24758E+00_JPRB, 0.24968E+00_JPRB, 0.25518E+00_JPRB, 0.25623E+00_JPRB, 0.24602E+00_JPRB ,&
&  0.23257E+00_JPRB, 0.22535E+00_JPRB, 0.22235E+00_JPRB, 0.22660E+00_JPRB, 0.23234E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.21794E+00_JPRB, 0.22415E+00_JPRB, 0.23466E+00_JPRB, 0.24357E+00_JPRB, 0.24770E+00_JPRB, 0.23675E+00_JPRB, 0.22458E+00_JPRB ,&
&  0.21813E+00_JPRB, 0.21632E+00_JPRB, 0.21791E+00_JPRB, 0.21889E+00_JPRB, 0.21934E+00_JPRB, 0.21992E+00_JPRB, 0.22020E+00_JPRB ,&
&  0.21818E+00_JPRB, 0.21491E+00_JPRB, 0.21185E+00_JPRB, 0.20943E+00_JPRB, 0.20867E+00_JPRB, 0.21057E+00_JPRB, 0.21573E+00_JPRB ,&
&  0.22277E+00_JPRB, 0.22663E+00_JPRB, 0.22832E+00_JPRB, 0.22820E+00_JPRB, 0.23049E+00_JPRB, 0.23421E+00_JPRB, 0.23403E+00_JPRB ,&
&  0.22894E+00_JPRB, 0.22871E+00_JPRB, 0.23108E+00_JPRB, 0.23945E+00_JPRB, 0.25928E+00_JPRB, 0.28670E+00_JPRB, 0.25486E+00_JPRB ,&
&  0.20933E+00_JPRB, 0.18380E+00_JPRB, 0.17545E+00_JPRB, 0.18265E+00_JPRB, 0.20377E+00_JPRB, 0.23988E+00_JPRB, 0.24950E+00_JPRB ,&
&  0.25739E+00_JPRB, 0.26638E+00_JPRB, 0.27290E+00_JPRB, 0.27997E+00_JPRB, 0.28586E+00_JPRB, 0.28972E+00_JPRB, 0.29144E+00_JPRB ,&
&  0.29668E+00_JPRB, 0.30780E+00_JPRB, 0.32090E+00_JPRB, 0.31319E+00_JPRB, 0.31345E+00_JPRB, 0.36313E+00_JPRB, 0.41235E+00_JPRB ,&
&  0.44872E+00_JPRB, 0.43983E+00_JPRB, 0.46217E+00_JPRB, 0.49334E+00_JPRB, 0.53281E+00_JPRB, 0.52101E+00_JPRB, 0.49857E+00_JPRB ,&
&  0.47276E+00_JPRB, 0.43857E+00_JPRB, 0.41914E+00_JPRB, 0.39939E+00_JPRB, 0.37915E+00_JPRB, 0.36240E+00_JPRB, 0.34392E+00_JPRB ,&
&  0.32956E+00_JPRB, 0.31788E+00_JPRB, 0.30303E+00_JPRB, 0.29156E+00_JPRB, 0.29004E+00_JPRB, 0.29584E+00_JPRB, 0.30426E+00_JPRB ,&
&  0.30781E+00_JPRB, 0.31159E+00_JPRB, 0.31864E+00_JPRB, 0.32450E+00_JPRB, 0.30609E+00_JPRB, 0.29036E+00_JPRB, 0.27959E+00_JPRB ,&
&  0.27693E+00_JPRB, 0.27668E+00_JPRB, 0.27738E+00_JPRB, 0.28144E+00_JPRB, 0.27757E+00_JPRB, 0.27667E+00_JPRB, 0.27015E+00_JPRB ,&
&  0.25312E+00_JPRB, 0.23771E+00_JPRB, 0.22797E+00_JPRB, 0.22202E+00_JPRB, 0.21909E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.22362E+00_JPRB, 0.22276E+00_JPRB, 0.22459E+00_JPRB, 0.22903E+00_JPRB, 0.23405E+00_JPRB, 0.23186E+00_JPRB, 0.22827E+00_JPRB ,&
&  0.22569E+00_JPRB, 0.22588E+00_JPRB, 0.22515E+00_JPRB, 0.22338E+00_JPRB, 0.22189E+00_JPRB, 0.22031E+00_JPRB, 0.21854E+00_JPRB ,&
&  0.21659E+00_JPRB, 0.21464E+00_JPRB, 0.21276E+00_JPRB, 0.21069E+00_JPRB, 0.21109E+00_JPRB, 0.21501E+00_JPRB, 0.22244E+00_JPRB ,&
&  0.22910E+00_JPRB, 0.23154E+00_JPRB, 0.23306E+00_JPRB, 0.23360E+00_JPRB, 0.23217E+00_JPRB, 0.23006E+00_JPRB, 0.22860E+00_JPRB ,&
&  0.22425E+00_JPRB, 0.22349E+00_JPRB, 0.22770E+00_JPRB, 0.23556E+00_JPRB, 0.24817E+00_JPRB, 0.25922E+00_JPRB, 0.22846E+00_JPRB ,&
&  0.19281E+00_JPRB, 0.16993E+00_JPRB, 0.15905E+00_JPRB, 0.15702E+00_JPRB, 0.16395E+00_JPRB, 0.17902E+00_JPRB, 0.20229E+00_JPRB ,&
&  0.23932E+00_JPRB, 0.26416E+00_JPRB, 0.27456E+00_JPRB, 0.28540E+00_JPRB, 0.28851E+00_JPRB, 0.28775E+00_JPRB, 0.28675E+00_JPRB ,&
&  0.29106E+00_JPRB, 0.29823E+00_JPRB, 0.30600E+00_JPRB, 0.30513E+00_JPRB, 0.30645E+00_JPRB, 0.30351E+00_JPRB, 0.31704E+00_JPRB ,&
&  0.36409E+00_JPRB, 0.36132E+00_JPRB, 0.34634E+00_JPRB, 0.37895E+00_JPRB, 0.43485E+00_JPRB, 0.45845E+00_JPRB, 0.46656E+00_JPRB ,&
&  0.45128E+00_JPRB, 0.42062E+00_JPRB, 0.39645E+00_JPRB, 0.38289E+00_JPRB, 0.36997E+00_JPRB, 0.35856E+00_JPRB, 0.35244E+00_JPRB ,&
&  0.35094E+00_JPRB, 0.35999E+00_JPRB, 0.40083E+00_JPRB, 0.34994E+00_JPRB, 0.32673E+00_JPRB, 0.31504E+00_JPRB, 0.30802E+00_JPRB ,&
&  0.30615E+00_JPRB, 0.30874E+00_JPRB, 0.31590E+00_JPRB, 0.32387E+00_JPRB, 0.31709E+00_JPRB, 0.30721E+00_JPRB, 0.29951E+00_JPRB ,&
&  0.29731E+00_JPRB, 0.29474E+00_JPRB, 0.29535E+00_JPRB, 0.30007E+00_JPRB, 0.30014E+00_JPRB, 0.29777E+00_JPRB, 0.29790E+00_JPRB ,&
&  0.28491E+00_JPRB, 0.26410E+00_JPRB, 0.25201E+00_JPRB, 0.24141E+00_JPRB, 0.23064E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.22492E+00_JPRB, 0.22125E+00_JPRB, 0.21955E+00_JPRB, 0.21984E+00_JPRB, 0.21983E+00_JPRB, 0.22047E+00_JPRB, 0.21955E+00_JPRB ,&
&  0.21843E+00_JPRB, 0.21733E+00_JPRB, 0.21602E+00_JPRB, 0.21479E+00_JPRB, 0.21397E+00_JPRB, 0.21310E+00_JPRB, 0.21261E+00_JPRB ,&
&  0.21233E+00_JPRB, 0.21010E+00_JPRB, 0.21050E+00_JPRB, 0.21397E+00_JPRB, 0.21895E+00_JPRB, 0.22421E+00_JPRB, 0.23054E+00_JPRB ,&
&  0.23324E+00_JPRB, 0.23340E+00_JPRB, 0.23496E+00_JPRB, 0.23467E+00_JPRB, 0.23169E+00_JPRB, 0.22783E+00_JPRB, 0.22498E+00_JPRB ,&
&  0.22425E+00_JPRB, 0.22234E+00_JPRB, 0.22806E+00_JPRB, 0.23194E+00_JPRB, 0.23032E+00_JPRB, 0.23902E+00_JPRB, 0.20200E+00_JPRB ,&
&  0.17383E+00_JPRB, 0.15424E+00_JPRB, 0.14778E+00_JPRB, 0.14501E+00_JPRB, 0.14881E+00_JPRB, 0.15649E+00_JPRB, 0.17422E+00_JPRB ,&
&  0.20317E+00_JPRB, 0.25488E+00_JPRB, 0.26825E+00_JPRB, 0.27358E+00_JPRB, 0.27825E+00_JPRB, 0.27708E+00_JPRB, 0.27695E+00_JPRB ,&
&  0.28104E+00_JPRB, 0.28123E+00_JPRB, 0.28057E+00_JPRB, 0.27674E+00_JPRB, 0.27441E+00_JPRB, 0.27317E+00_JPRB, 0.26908E+00_JPRB ,&
&  0.27441E+00_JPRB, 0.27930E+00_JPRB, 0.28898E+00_JPRB, 0.31473E+00_JPRB, 0.36165E+00_JPRB, 0.40142E+00_JPRB, 0.41180E+00_JPRB ,&
&  0.39604E+00_JPRB, 0.37828E+00_JPRB, 0.35873E+00_JPRB, 0.34882E+00_JPRB, 0.34053E+00_JPRB, 0.33451E+00_JPRB, 0.33227E+00_JPRB ,&
&  0.33445E+00_JPRB, 0.34321E+00_JPRB, 0.36327E+00_JPRB, 0.34599E+00_JPRB, 0.33601E+00_JPRB, 0.32427E+00_JPRB, 0.31575E+00_JPRB ,&
&  0.30939E+00_JPRB, 0.30939E+00_JPRB, 0.31395E+00_JPRB, 0.31655E+00_JPRB, 0.31711E+00_JPRB, 0.31436E+00_JPRB, 0.31374E+00_JPRB ,&
&  0.31118E+00_JPRB, 0.30760E+00_JPRB, 0.30538E+00_JPRB, 0.30863E+00_JPRB, 0.31187E+00_JPRB, 0.31532E+00_JPRB, 0.31743E+00_JPRB ,&
&  0.30507E+00_JPRB, 0.27990E+00_JPRB, 0.25914E+00_JPRB, 0.24404E+00_JPRB, 0.23252E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.23403E+00_JPRB, 0.22677E+00_JPRB, 0.22135E+00_JPRB, 0.21847E+00_JPRB, 0.21633E+00_JPRB, 0.21581E+00_JPRB, 0.21497E+00_JPRB ,&
&  0.21472E+00_JPRB, 0.21434E+00_JPRB, 0.21401E+00_JPRB, 0.21377E+00_JPRB, 0.21380E+00_JPRB, 0.21417E+00_JPRB, 0.21516E+00_JPRB ,&
&  0.21664E+00_JPRB, 0.21922E+00_JPRB, 0.22255E+00_JPRB, 0.22543E+00_JPRB, 0.22941E+00_JPRB, 0.23298E+00_JPRB, 0.23645E+00_JPRB ,&
&  0.23788E+00_JPRB, 0.23950E+00_JPRB, 0.23902E+00_JPRB, 0.23707E+00_JPRB, 0.23254E+00_JPRB, 0.22480E+00_JPRB, 0.22538E+00_JPRB ,&
&  0.22522E+00_JPRB, 0.22350E+00_JPRB, 0.21937E+00_JPRB, 0.22050E+00_JPRB, 0.22049E+00_JPRB, 0.19044E+00_JPRB, 0.18212E+00_JPRB ,&
&  0.16723E+00_JPRB, 0.15196E+00_JPRB, 0.14625E+00_JPRB, 0.14254E+00_JPRB, 0.14318E+00_JPRB, 0.14848E+00_JPRB, 0.15769E+00_JPRB ,&
&  0.17796E+00_JPRB, 0.23569E+00_JPRB, 0.24796E+00_JPRB, 0.25152E+00_JPRB, 0.24712E+00_JPRB, 0.24741E+00_JPRB, 0.24867E+00_JPRB ,&
&  0.25256E+00_JPRB, 0.25353E+00_JPRB, 0.25372E+00_JPRB, 0.25262E+00_JPRB, 0.25315E+00_JPRB, 0.25088E+00_JPRB, 0.25187E+00_JPRB ,&
&  0.25788E+00_JPRB, 0.26845E+00_JPRB, 0.27695E+00_JPRB, 0.28783E+00_JPRB, 0.30411E+00_JPRB, 0.32086E+00_JPRB, 0.33914E+00_JPRB ,&
&  0.33595E+00_JPRB, 0.33027E+00_JPRB, 0.32610E+00_JPRB, 0.32110E+00_JPRB, 0.31793E+00_JPRB, 0.31528E+00_JPRB, 0.31445E+00_JPRB ,&
&  0.31634E+00_JPRB, 0.32264E+00_JPRB, 0.32347E+00_JPRB, 0.31487E+00_JPRB, 0.31162E+00_JPRB, 0.30498E+00_JPRB, 0.30505E+00_JPRB ,&
&  0.30643E+00_JPRB, 0.31142E+00_JPRB, 0.31370E+00_JPRB, 0.31471E+00_JPRB, 0.31522E+00_JPRB, 0.31248E+00_JPRB, 0.31139E+00_JPRB ,&
&  0.31011E+00_JPRB, 0.30923E+00_JPRB, 0.31146E+00_JPRB, 0.31589E+00_JPRB, 0.31968E+00_JPRB, 0.32214E+00_JPRB, 0.32277E+00_JPRB ,&
&  0.31258E+00_JPRB, 0.29330E+00_JPRB, 0.27503E+00_JPRB, 0.25672E+00_JPRB, 0.24327E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.25177E+00_JPRB, 0.24154E+00_JPRB, 0.23350E+00_JPRB, 0.22791E+00_JPRB, 0.22444E+00_JPRB, 0.22274E+00_JPRB, 0.22153E+00_JPRB ,&
&  0.22123E+00_JPRB, 0.22057E+00_JPRB, 0.22047E+00_JPRB, 0.22017E+00_JPRB, 0.22034E+00_JPRB, 0.22092E+00_JPRB, 0.22232E+00_JPRB ,&
&  0.22460E+00_JPRB, 0.22715E+00_JPRB, 0.23072E+00_JPRB, 0.23328E+00_JPRB, 0.23653E+00_JPRB, 0.23878E+00_JPRB, 0.23974E+00_JPRB ,&
&  0.24106E+00_JPRB, 0.24223E+00_JPRB, 0.24065E+00_JPRB, 0.23972E+00_JPRB, 0.23023E+00_JPRB, 0.22285E+00_JPRB, 0.21791E+00_JPRB ,&
&  0.22023E+00_JPRB, 0.20570E+00_JPRB, 0.20639E+00_JPRB, 0.19597E+00_JPRB, 0.18544E+00_JPRB, 0.17553E+00_JPRB, 0.16643E+00_JPRB ,&
&  0.16067E+00_JPRB, 0.15236E+00_JPRB, 0.14868E+00_JPRB, 0.14785E+00_JPRB, 0.14871E+00_JPRB, 0.15451E+00_JPRB, 0.16298E+00_JPRB ,&
&  0.17800E+00_JPRB, 0.21935E+00_JPRB, 0.23158E+00_JPRB, 0.23604E+00_JPRB, 0.23442E+00_JPRB, 0.23515E+00_JPRB, 0.23495E+00_JPRB ,&
&  0.24117E+00_JPRB, 0.24810E+00_JPRB, 0.25112E+00_JPRB, 0.25181E+00_JPRB, 0.25594E+00_JPRB, 0.25736E+00_JPRB, 0.25803E+00_JPRB ,&
&  0.25978E+00_JPRB, 0.26352E+00_JPRB, 0.26700E+00_JPRB, 0.26875E+00_JPRB, 0.27164E+00_JPRB, 0.27327E+00_JPRB, 0.27805E+00_JPRB ,&
&  0.28040E+00_JPRB, 0.28854E+00_JPRB, 0.29146E+00_JPRB, 0.29579E+00_JPRB, 0.29714E+00_JPRB, 0.29652E+00_JPRB, 0.29755E+00_JPRB ,&
&  0.29931E+00_JPRB, 0.30293E+00_JPRB, 0.30484E+00_JPRB, 0.30644E+00_JPRB, 0.30761E+00_JPRB, 0.30574E+00_JPRB, 0.30610E+00_JPRB ,&
&  0.30879E+00_JPRB, 0.31075E+00_JPRB, 0.31231E+00_JPRB, 0.31282E+00_JPRB, 0.31158E+00_JPRB, 0.30925E+00_JPRB, 0.30991E+00_JPRB ,&
&  0.31106E+00_JPRB, 0.31434E+00_JPRB, 0.31893E+00_JPRB, 0.32087E+00_JPRB, 0.32187E+00_JPRB, 0.32111E+00_JPRB, 0.31661E+00_JPRB ,&
&  0.30699E+00_JPRB, 0.29719E+00_JPRB, 0.28616E+00_JPRB, 0.27527E+00_JPRB, 0.26354E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.26852E+00_JPRB, 0.26084E+00_JPRB, 0.25224E+00_JPRB, 0.24536E+00_JPRB, 0.24040E+00_JPRB, 0.23732E+00_JPRB, 0.23490E+00_JPRB ,&
&  0.23329E+00_JPRB, 0.23201E+00_JPRB, 0.23135E+00_JPRB, 0.23089E+00_JPRB, 0.23075E+00_JPRB, 0.23091E+00_JPRB, 0.23169E+00_JPRB ,&
&  0.23345E+00_JPRB, 0.23573E+00_JPRB, 0.23910E+00_JPRB, 0.24241E+00_JPRB, 0.24536E+00_JPRB, 0.24611E+00_JPRB, 0.24405E+00_JPRB ,&
&  0.24512E+00_JPRB, 0.24581E+00_JPRB, 0.23848E+00_JPRB, 0.23950E+00_JPRB, 0.22977E+00_JPRB, 0.22207E+00_JPRB, 0.21864E+00_JPRB ,&
&  0.21274E+00_JPRB, 0.21080E+00_JPRB, 0.20527E+00_JPRB, 0.19959E+00_JPRB, 0.19533E+00_JPRB, 0.18668E+00_JPRB, 0.18084E+00_JPRB ,&
&  0.17463E+00_JPRB, 0.16704E+00_JPRB, 0.16320E+00_JPRB, 0.16108E+00_JPRB, 0.16268E+00_JPRB, 0.16786E+00_JPRB, 0.17595E+00_JPRB ,&
&  0.18754E+00_JPRB, 0.20211E+00_JPRB, 0.22353E+00_JPRB, 0.23081E+00_JPRB, 0.23296E+00_JPRB, 0.23723E+00_JPRB, 0.24001E+00_JPRB ,&
&  0.24095E+00_JPRB, 0.24639E+00_JPRB, 0.24913E+00_JPRB, 0.24988E+00_JPRB, 0.24966E+00_JPRB, 0.24877E+00_JPRB, 0.24680E+00_JPRB ,&
&  0.24573E+00_JPRB, 0.24620E+00_JPRB, 0.24558E+00_JPRB, 0.24420E+00_JPRB, 0.24337E+00_JPRB, 0.24299E+00_JPRB, 0.24569E+00_JPRB ,&
&  0.24867E+00_JPRB, 0.25508E+00_JPRB, 0.26042E+00_JPRB, 0.26882E+00_JPRB, 0.27323E+00_JPRB, 0.27821E+00_JPRB, 0.28051E+00_JPRB ,&
&  0.28281E+00_JPRB, 0.28503E+00_JPRB, 0.28627E+00_JPRB, 0.28463E+00_JPRB, 0.28465E+00_JPRB, 0.28761E+00_JPRB, 0.28847E+00_JPRB ,&
&  0.28995E+00_JPRB, 0.29149E+00_JPRB, 0.29264E+00_JPRB, 0.29379E+00_JPRB, 0.29518E+00_JPRB, 0.29818E+00_JPRB, 0.30149E+00_JPRB ,&
&  0.30787E+00_JPRB, 0.31225E+00_JPRB, 0.31599E+00_JPRB, 0.31536E+00_JPRB, 0.31389E+00_JPRB, 0.30946E+00_JPRB, 0.30305E+00_JPRB ,&
&  0.29705E+00_JPRB, 0.29200E+00_JPRB, 0.28586E+00_JPRB, 0.28092E+00_JPRB, 0.27507E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.26211E+00_JPRB, 0.25891E+00_JPRB, 0.25502E+00_JPRB, 0.25136E+00_JPRB, 0.24875E+00_JPRB, 0.24726E+00_JPRB, 0.24572E+00_JPRB ,&
&  0.24473E+00_JPRB, 0.24389E+00_JPRB, 0.24383E+00_JPRB, 0.24424E+00_JPRB, 0.24471E+00_JPRB, 0.24617E+00_JPRB, 0.24774E+00_JPRB ,&
&  0.24915E+00_JPRB, 0.25026E+00_JPRB, 0.25202E+00_JPRB, 0.25340E+00_JPRB, 0.25409E+00_JPRB, 0.25359E+00_JPRB, 0.25168E+00_JPRB ,&
&  0.25069E+00_JPRB, 0.24910E+00_JPRB, 0.24759E+00_JPRB, 0.24641E+00_JPRB, 0.24496E+00_JPRB, 0.22937E+00_JPRB, 0.22635E+00_JPRB ,&
&  0.22744E+00_JPRB, 0.22560E+00_JPRB, 0.22472E+00_JPRB, 0.22454E+00_JPRB, 0.22667E+00_JPRB, 0.22412E+00_JPRB, 0.22131E+00_JPRB ,&
&  0.20653E+00_JPRB, 0.20307E+00_JPRB, 0.19890E+00_JPRB, 0.19482E+00_JPRB, 0.19345E+00_JPRB, 0.19469E+00_JPRB, 0.19861E+00_JPRB ,&
&  0.20989E+00_JPRB, 0.21397E+00_JPRB, 0.21938E+00_JPRB, 0.22575E+00_JPRB, 0.23098E+00_JPRB, 0.23426E+00_JPRB, 0.23737E+00_JPRB ,&
&  0.23903E+00_JPRB, 0.24037E+00_JPRB, 0.24099E+00_JPRB, 0.24039E+00_JPRB, 0.23900E+00_JPRB, 0.23770E+00_JPRB, 0.23630E+00_JPRB ,&
&  0.23511E+00_JPRB, 0.23359E+00_JPRB, 0.23233E+00_JPRB, 0.23103E+00_JPRB, 0.22967E+00_JPRB, 0.22930E+00_JPRB, 0.23069E+00_JPRB ,&
&  0.23304E+00_JPRB, 0.23665E+00_JPRB, 0.24013E+00_JPRB, 0.24534E+00_JPRB, 0.25039E+00_JPRB, 0.25599E+00_JPRB, 0.25838E+00_JPRB ,&
&  0.26123E+00_JPRB, 0.26357E+00_JPRB, 0.26639E+00_JPRB, 0.26737E+00_JPRB, 0.26831E+00_JPRB, 0.26942E+00_JPRB, 0.27143E+00_JPRB ,&
&  0.27257E+00_JPRB, 0.27401E+00_JPRB, 0.27546E+00_JPRB, 0.27838E+00_JPRB, 0.28000E+00_JPRB, 0.28331E+00_JPRB, 0.28606E+00_JPRB ,&
&  0.29100E+00_JPRB, 0.29149E+00_JPRB, 0.29267E+00_JPRB, 0.29047E+00_JPRB, 0.28856E+00_JPRB, 0.28373E+00_JPRB, 0.27930E+00_JPRB ,&
&  0.27536E+00_JPRB, 0.27219E+00_JPRB, 0.26956E+00_JPRB, 0.26706E+00_JPRB, 0.26468E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.25180E+00_JPRB, 0.25017E+00_JPRB, 0.24891E+00_JPRB, 0.24823E+00_JPRB, 0.24768E+00_JPRB, 0.24748E+00_JPRB, 0.24742E+00_JPRB ,&
&  0.24741E+00_JPRB, 0.24739E+00_JPRB, 0.24738E+00_JPRB, 0.24768E+00_JPRB, 0.24821E+00_JPRB, 0.24900E+00_JPRB, 0.24959E+00_JPRB ,&
&  0.25013E+00_JPRB, 0.25040E+00_JPRB, 0.25064E+00_JPRB, 0.25020E+00_JPRB, 0.24973E+00_JPRB, 0.24884E+00_JPRB, 0.24796E+00_JPRB ,&
&  0.24646E+00_JPRB, 0.24473E+00_JPRB, 0.24324E+00_JPRB, 0.24209E+00_JPRB, 0.24074E+00_JPRB, 0.23904E+00_JPRB, 0.23793E+00_JPRB ,&
&  0.23735E+00_JPRB, 0.23626E+00_JPRB, 0.23534E+00_JPRB, 0.23444E+00_JPRB, 0.23287E+00_JPRB, 0.23072E+00_JPRB, 0.22874E+00_JPRB ,&
&  0.22623E+00_JPRB, 0.22341E+00_JPRB, 0.22149E+00_JPRB, 0.21979E+00_JPRB, 0.21844E+00_JPRB, 0.21758E+00_JPRB, 0.21755E+00_JPRB ,&
&  0.21927E+00_JPRB, 0.22182E+00_JPRB, 0.22472E+00_JPRB, 0.22676E+00_JPRB, 0.22833E+00_JPRB, 0.22936E+00_JPRB, 0.23047E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23098E+00_JPRB, 0.23085E+00_JPRB, 0.23052E+00_JPRB, 0.22984E+00_JPRB, 0.22952E+00_JPRB, 0.22940E+00_JPRB ,&
&  0.22929E+00_JPRB, 0.22796E+00_JPRB, 0.22691E+00_JPRB, 0.22612E+00_JPRB, 0.22564E+00_JPRB, 0.22572E+00_JPRB, 0.22651E+00_JPRB ,&
&  0.22782E+00_JPRB, 0.22974E+00_JPRB, 0.23193E+00_JPRB, 0.23476E+00_JPRB, 0.23809E+00_JPRB, 0.24164E+00_JPRB, 0.24450E+00_JPRB ,&
&  0.24758E+00_JPRB, 0.25088E+00_JPRB, 0.25419E+00_JPRB, 0.25630E+00_JPRB, 0.25851E+00_JPRB, 0.26067E+00_JPRB, 0.26305E+00_JPRB ,&
&  0.26421E+00_JPRB, 0.26592E+00_JPRB, 0.26777E+00_JPRB, 0.27017E+00_JPRB, 0.27093E+00_JPRB, 0.27216E+00_JPRB, 0.27317E+00_JPRB ,&
&  0.27478E+00_JPRB, 0.27327E+00_JPRB, 0.27236E+00_JPRB, 0.27060E+00_JPRB, 0.26903E+00_JPRB, 0.26571E+00_JPRB, 0.26289E+00_JPRB ,&
&  0.26052E+00_JPRB, 0.25868E+00_JPRB, 0.25670E+00_JPRB, 0.25490E+00_JPRB, 0.25347E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.24339E+00_JPRB, 0.24273E+00_JPRB, 0.24208E+00_JPRB, 0.24146E+00_JPRB, 0.24099E+00_JPRB, 0.24073E+00_JPRB, 0.24058E+00_JPRB ,&
&  0.24038E+00_JPRB, 0.24011E+00_JPRB, 0.23988E+00_JPRB, 0.23974E+00_JPRB, 0.23964E+00_JPRB, 0.23952E+00_JPRB, 0.23933E+00_JPRB ,&
&  0.23909E+00_JPRB, 0.23895E+00_JPRB, 0.23875E+00_JPRB, 0.23834E+00_JPRB, 0.23788E+00_JPRB, 0.23739E+00_JPRB, 0.23679E+00_JPRB ,&
&  0.23607E+00_JPRB, 0.23531E+00_JPRB, 0.23454E+00_JPRB, 0.23354E+00_JPRB, 0.23244E+00_JPRB, 0.23111E+00_JPRB, 0.23002E+00_JPRB ,&
&  0.22907E+00_JPRB, 0.22809E+00_JPRB, 0.22708E+00_JPRB, 0.22617E+00_JPRB, 0.22531E+00_JPRB, 0.22452E+00_JPRB, 0.22365E+00_JPRB ,&
&  0.22295E+00_JPRB, 0.22230E+00_JPRB, 0.22191E+00_JPRB, 0.22164E+00_JPRB, 0.22149E+00_JPRB, 0.22163E+00_JPRB, 0.22201E+00_JPRB ,&
&  0.22226E+00_JPRB, 0.22254E+00_JPRB, 0.22298E+00_JPRB, 0.22348E+00_JPRB, 0.22404E+00_JPRB, 0.22461E+00_JPRB, 0.22516E+00_JPRB ,&
&  0.22558E+00_JPRB, 0.22605E+00_JPRB, 0.22668E+00_JPRB, 0.22729E+00_JPRB, 0.22771E+00_JPRB, 0.22816E+00_JPRB, 0.22860E+00_JPRB ,&
&  0.22894E+00_JPRB, 0.22911E+00_JPRB, 0.22929E+00_JPRB, 0.22953E+00_JPRB, 0.22982E+00_JPRB, 0.23011E+00_JPRB, 0.23060E+00_JPRB ,&
&  0.23116E+00_JPRB, 0.23172E+00_JPRB, 0.23253E+00_JPRB, 0.23356E+00_JPRB, 0.23477E+00_JPRB, 0.23624E+00_JPRB, 0.23764E+00_JPRB ,&
&  0.23909E+00_JPRB, 0.24060E+00_JPRB, 0.24216E+00_JPRB, 0.24359E+00_JPRB, 0.24509E+00_JPRB, 0.24648E+00_JPRB, 0.24776E+00_JPRB ,&
&  0.24858E+00_JPRB, 0.24955E+00_JPRB, 0.25060E+00_JPRB, 0.25150E+00_JPRB, 0.25152E+00_JPRB, 0.25164E+00_JPRB, 0.25172E+00_JPRB ,&
&  0.25207E+00_JPRB, 0.25140E+00_JPRB, 0.25103E+00_JPRB, 0.25047E+00_JPRB, 0.24990E+00_JPRB, 0.24881E+00_JPRB, 0.24796E+00_JPRB ,&
&  0.24709E+00_JPRB, 0.24635E+00_JPRB, 0.24552E+00_JPRB, 0.24470E+00_JPRB, 0.24410E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB, 0.23235E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB ,&
&  0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB, 0.22668E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.24802E+00_JPRB, 0.25057E+00_JPRB, 0.25282E+00_JPRB, 0.25456E+00_JPRB, 0.25517E+00_JPRB, 0.25713E+00_JPRB, 0.25777E+00_JPRB ,&
&  0.25822E+00_JPRB, 0.25829E+00_JPRB, 0.25908E+00_JPRB, 0.25990E+00_JPRB, 0.26045E+00_JPRB, 0.26072E+00_JPRB, 0.26076E+00_JPRB ,&
&  0.26034E+00_JPRB, 0.25945E+00_JPRB, 0.25619E+00_JPRB, 0.25438E+00_JPRB, 0.25337E+00_JPRB, 0.25299E+00_JPRB, 0.25283E+00_JPRB ,&
&  0.25227E+00_JPRB, 0.25154E+00_JPRB, 0.25027E+00_JPRB, 0.25003E+00_JPRB, 0.24922E+00_JPRB, 0.24817E+00_JPRB, 0.24671E+00_JPRB ,&
&  0.24528E+00_JPRB, 0.24412E+00_JPRB, 0.24267E+00_JPRB, 0.24118E+00_JPRB, 0.23932E+00_JPRB, 0.23712E+00_JPRB, 0.23445E+00_JPRB ,&
&  0.23166E+00_JPRB, 0.22881E+00_JPRB, 0.22645E+00_JPRB, 0.22426E+00_JPRB, 0.22238E+00_JPRB, 0.22089E+00_JPRB, 0.21987E+00_JPRB ,&
&  0.21875E+00_JPRB, 0.21750E+00_JPRB, 0.21623E+00_JPRB, 0.21517E+00_JPRB, 0.21412E+00_JPRB, 0.21322E+00_JPRB, 0.21281E+00_JPRB ,&
&  0.21256E+00_JPRB, 0.21226E+00_JPRB, 0.21188E+00_JPRB, 0.21158E+00_JPRB, 0.21116E+00_JPRB, 0.21078E+00_JPRB, 0.21039E+00_JPRB ,&
&  0.21008E+00_JPRB, 0.20987E+00_JPRB, 0.20954E+00_JPRB, 0.20919E+00_JPRB, 0.20897E+00_JPRB, 0.20860E+00_JPRB, 0.20819E+00_JPRB ,&
&  0.20769E+00_JPRB, 0.20737E+00_JPRB, 0.20702E+00_JPRB, 0.20660E+00_JPRB, 0.20621E+00_JPRB, 0.20594E+00_JPRB, 0.20593E+00_JPRB ,&
&  0.20588E+00_JPRB, 0.20577E+00_JPRB, 0.20581E+00_JPRB, 0.20648E+00_JPRB, 0.20719E+00_JPRB, 0.20804E+00_JPRB, 0.20900E+00_JPRB ,&
&  0.21027E+00_JPRB, 0.21160E+00_JPRB, 0.21307E+00_JPRB, 0.21447E+00_JPRB, 0.21632E+00_JPRB, 0.21815E+00_JPRB, 0.21997E+00_JPRB ,&
&  0.22170E+00_JPRB, 0.22348E+00_JPRB, 0.22523E+00_JPRB, 0.22697E+00_JPRB, 0.22920E+00_JPRB, 0.23124E+00_JPRB, 0.23354E+00_JPRB ,&
&  0.23574E+00_JPRB, 0.23810E+00_JPRB, 0.24059E+00_JPRB, 0.24276E+00_JPRB, 0.24504E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.29080E+00_JPRB, 0.30134E+00_JPRB, 0.31042E+00_JPRB, 0.31932E+00_JPRB, 0.32416E+00_JPRB, 0.38881E+00_JPRB, 0.36969E+00_JPRB ,&
&  0.36041E+00_JPRB, 0.33187E+00_JPRB, 0.33241E+00_JPRB, 0.32774E+00_JPRB, 0.32101E+00_JPRB, 0.31316E+00_JPRB, 0.30653E+00_JPRB ,&
&  0.30003E+00_JPRB, 0.29215E+00_JPRB, 0.28697E+00_JPRB, 0.28080E+00_JPRB, 0.27498E+00_JPRB, 0.26925E+00_JPRB, 0.26586E+00_JPRB ,&
&  0.26334E+00_JPRB, 0.26187E+00_JPRB, 0.26075E+00_JPRB, 0.26033E+00_JPRB, 0.26311E+00_JPRB, 0.26465E+00_JPRB, 0.26306E+00_JPRB ,&
&  0.25977E+00_JPRB, 0.25660E+00_JPRB, 0.25249E+00_JPRB, 0.24825E+00_JPRB, 0.24493E+00_JPRB, 0.24205E+00_JPRB, 0.23908E+00_JPRB ,&
&  0.23525E+00_JPRB, 0.23099E+00_JPRB, 0.22679E+00_JPRB, 0.22354E+00_JPRB, 0.22054E+00_JPRB, 0.21754E+00_JPRB, 0.21479E+00_JPRB ,&
&  0.21198E+00_JPRB, 0.20921E+00_JPRB, 0.20644E+00_JPRB, 0.20460E+00_JPRB, 0.20306E+00_JPRB, 0.20161E+00_JPRB, 0.20031E+00_JPRB ,&
&  0.19971E+00_JPRB, 0.19901E+00_JPRB, 0.19814E+00_JPRB, 0.19708E+00_JPRB, 0.19603E+00_JPRB, 0.19490E+00_JPRB, 0.19397E+00_JPRB ,&
&  0.19380E+00_JPRB, 0.19323E+00_JPRB, 0.19264E+00_JPRB, 0.19187E+00_JPRB, 0.19149E+00_JPRB, 0.19075E+00_JPRB, 0.18983E+00_JPRB ,&
&  0.18897E+00_JPRB, 0.18819E+00_JPRB, 0.18798E+00_JPRB, 0.18769E+00_JPRB, 0.18738E+00_JPRB, 0.18718E+00_JPRB, 0.18753E+00_JPRB ,&
&  0.18796E+00_JPRB, 0.18850E+00_JPRB, 0.18939E+00_JPRB, 0.19051E+00_JPRB, 0.19148E+00_JPRB, 0.19244E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19564E+00_JPRB, 0.19750E+00_JPRB, 0.19955E+00_JPRB, 0.20154E+00_JPRB, 0.20477E+00_JPRB, 0.20824E+00_JPRB, 0.21203E+00_JPRB ,&
&  0.21671E+00_JPRB, 0.22308E+00_JPRB, 0.23026E+00_JPRB, 0.23664E+00_JPRB, 0.24233E+00_JPRB, 0.24920E+00_JPRB, 0.25483E+00_JPRB ,&
&  0.25985E+00_JPRB, 0.26473E+00_JPRB, 0.27079E+00_JPRB, 0.27722E+00_JPRB, 0.28423E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.41762E+00_JPRB, 0.41822E+00_JPRB, 0.41836E+00_JPRB, 0.41461E+00_JPRB, 0.41294E+00_JPRB, 0.41280E+00_JPRB, 0.40999E+00_JPRB ,&
&  0.40216E+00_JPRB, 0.39957E+00_JPRB, 0.39478E+00_JPRB, 0.38802E+00_JPRB, 0.38148E+00_JPRB, 0.37101E+00_JPRB, 0.35487E+00_JPRB ,&
&  0.33315E+00_JPRB, 0.31415E+00_JPRB, 0.29964E+00_JPRB, 0.28652E+00_JPRB, 0.27363E+00_JPRB, 0.26105E+00_JPRB, 0.25559E+00_JPRB ,&
&  0.25102E+00_JPRB, 0.24900E+00_JPRB, 0.24825E+00_JPRB, 0.25190E+00_JPRB, 0.25951E+00_JPRB, 0.26904E+00_JPRB, 0.27941E+00_JPRB ,&
&  0.29147E+00_JPRB, 0.30265E+00_JPRB, 0.30418E+00_JPRB, 0.29728E+00_JPRB, 0.29036E+00_JPRB, 0.28598E+00_JPRB, 0.28262E+00_JPRB ,&
&  0.27873E+00_JPRB, 0.27034E+00_JPRB, 0.26098E+00_JPRB, 0.25127E+00_JPRB, 0.24149E+00_JPRB, 0.23434E+00_JPRB, 0.22594E+00_JPRB ,&
&  0.21983E+00_JPRB, 0.21437E+00_JPRB, 0.20872E+00_JPRB, 0.20388E+00_JPRB, 0.20061E+00_JPRB, 0.19784E+00_JPRB, 0.19510E+00_JPRB ,&
&  0.19361E+00_JPRB, 0.19212E+00_JPRB, 0.19044E+00_JPRB, 0.18875E+00_JPRB, 0.18672E+00_JPRB, 0.18511E+00_JPRB, 0.18364E+00_JPRB ,&
&  0.18249E+00_JPRB, 0.18152E+00_JPRB, 0.18092E+00_JPRB, 0.18032E+00_JPRB, 0.17985E+00_JPRB, 0.17898E+00_JPRB, 0.17794E+00_JPRB ,&
&  0.17721E+00_JPRB, 0.17677E+00_JPRB, 0.17601E+00_JPRB, 0.17554E+00_JPRB, 0.17517E+00_JPRB, 0.17447E+00_JPRB, 0.17485E+00_JPRB ,&
&  0.17584E+00_JPRB, 0.17698E+00_JPRB, 0.17735E+00_JPRB, 0.17841E+00_JPRB, 0.17971E+00_JPRB, 0.18132E+00_JPRB, 0.18373E+00_JPRB ,&
&  0.18563E+00_JPRB, 0.18759E+00_JPRB, 0.19001E+00_JPRB, 0.19219E+00_JPRB, 0.19546E+00_JPRB, 0.19949E+00_JPRB, 0.20444E+00_JPRB ,&
&  0.21118E+00_JPRB, 0.21930E+00_JPRB, 0.23043E+00_JPRB, 0.24393E+00_JPRB, 0.26293E+00_JPRB, 0.27147E+00_JPRB, 0.28008E+00_JPRB ,&
&  0.28785E+00_JPRB, 0.30333E+00_JPRB, 0.38532E+00_JPRB, 0.40047E+00_JPRB, 0.41521E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.41562E+00_JPRB, 0.42773E+00_JPRB, 0.44539E+00_JPRB, 0.45624E+00_JPRB, 0.47079E+00_JPRB, 0.47369E+00_JPRB, 0.46900E+00_JPRB ,&
&  0.44973E+00_JPRB, 0.41928E+00_JPRB, 0.40256E+00_JPRB, 0.38239E+00_JPRB, 0.36119E+00_JPRB, 0.34003E+00_JPRB, 0.33072E+00_JPRB ,&
&  0.31956E+00_JPRB, 0.30726E+00_JPRB, 0.29452E+00_JPRB, 0.28369E+00_JPRB, 0.27331E+00_JPRB, 0.26358E+00_JPRB, 0.25776E+00_JPRB ,&
&  0.25180E+00_JPRB, 0.25098E+00_JPRB, 0.25030E+00_JPRB, 0.25451E+00_JPRB, 0.26553E+00_JPRB, 0.28462E+00_JPRB, 0.31046E+00_JPRB ,&
&  0.34958E+00_JPRB, 0.35639E+00_JPRB, 0.34141E+00_JPRB, 0.33500E+00_JPRB, 0.32774E+00_JPRB, 0.32124E+00_JPRB, 0.30649E+00_JPRB ,&
&  0.31372E+00_JPRB, 0.32744E+00_JPRB, 0.34337E+00_JPRB, 0.33386E+00_JPRB, 0.28902E+00_JPRB, 0.26837E+00_JPRB, 0.24495E+00_JPRB ,&
&  0.23199E+00_JPRB, 0.22167E+00_JPRB, 0.21374E+00_JPRB, 0.20651E+00_JPRB, 0.19968E+00_JPRB, 0.19527E+00_JPRB, 0.19043E+00_JPRB ,&
&  0.18675E+00_JPRB, 0.18500E+00_JPRB, 0.18259E+00_JPRB, 0.18227E+00_JPRB, 0.18192E+00_JPRB, 0.18104E+00_JPRB, 0.18009E+00_JPRB ,&
&  0.17865E+00_JPRB, 0.17713E+00_JPRB, 0.17632E+00_JPRB, 0.17671E+00_JPRB, 0.17760E+00_JPRB, 0.17849E+00_JPRB, 0.17688E+00_JPRB ,&
&  0.17560E+00_JPRB, 0.17431E+00_JPRB, 0.17411E+00_JPRB, 0.17496E+00_JPRB, 0.17520E+00_JPRB, 0.17578E+00_JPRB, 0.17580E+00_JPRB ,&
&  0.17581E+00_JPRB, 0.17624E+00_JPRB, 0.17596E+00_JPRB, 0.17658E+00_JPRB, 0.17689E+00_JPRB, 0.17797E+00_JPRB, 0.17883E+00_JPRB ,&
&  0.17975E+00_JPRB, 0.18163E+00_JPRB, 0.18362E+00_JPRB, 0.18636E+00_JPRB, 0.19011E+00_JPRB, 0.19288E+00_JPRB, 0.19718E+00_JPRB ,&
&  0.20255E+00_JPRB, 0.21097E+00_JPRB, 0.22620E+00_JPRB, 0.24331E+00_JPRB, 0.26419E+00_JPRB, 0.27601E+00_JPRB, 0.28585E+00_JPRB ,&
&  0.30087E+00_JPRB, 0.33635E+00_JPRB, 0.38671E+00_JPRB, 0.40534E+00_JPRB, 0.40916E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.47238E+00_JPRB, 0.47985E+00_JPRB, 0.49133E+00_JPRB, 0.49609E+00_JPRB, 0.50443E+00_JPRB, 0.49282E+00_JPRB, 0.48605E+00_JPRB ,&
&  0.45594E+00_JPRB, 0.43303E+00_JPRB, 0.40174E+00_JPRB, 0.36198E+00_JPRB, 0.33550E+00_JPRB, 0.31492E+00_JPRB, 0.30821E+00_JPRB ,&
&  0.30190E+00_JPRB, 0.29771E+00_JPRB, 0.29406E+00_JPRB, 0.29272E+00_JPRB, 0.29235E+00_JPRB, 0.29382E+00_JPRB, 0.29403E+00_JPRB ,&
&  0.29074E+00_JPRB, 0.28512E+00_JPRB, 0.27866E+00_JPRB, 0.28195E+00_JPRB, 0.29487E+00_JPRB, 0.30999E+00_JPRB, 0.32757E+00_JPRB ,&
&  0.34772E+00_JPRB, 0.35878E+00_JPRB, 0.36895E+00_JPRB, 0.37319E+00_JPRB, 0.35792E+00_JPRB, 0.36045E+00_JPRB, 0.36274E+00_JPRB ,&
&  0.37083E+00_JPRB, 0.37848E+00_JPRB, 0.37604E+00_JPRB, 0.37260E+00_JPRB, 0.32253E+00_JPRB, 0.28903E+00_JPRB, 0.26055E+00_JPRB ,&
&  0.23910E+00_JPRB, 0.22522E+00_JPRB, 0.21427E+00_JPRB, 0.20506E+00_JPRB, 0.19624E+00_JPRB, 0.19043E+00_JPRB, 0.18605E+00_JPRB ,&
&  0.18241E+00_JPRB, 0.18067E+00_JPRB, 0.17895E+00_JPRB, 0.17829E+00_JPRB, 0.17834E+00_JPRB, 0.17764E+00_JPRB, 0.17730E+00_JPRB ,&
&  0.17679E+00_JPRB, 0.17581E+00_JPRB, 0.17545E+00_JPRB, 0.17620E+00_JPRB, 0.17786E+00_JPRB, 0.18076E+00_JPRB, 0.18193E+00_JPRB ,&
&  0.18268E+00_JPRB, 0.18303E+00_JPRB, 0.18302E+00_JPRB, 0.18466E+00_JPRB, 0.18549E+00_JPRB, 0.18567E+00_JPRB, 0.18408E+00_JPRB ,&
&  0.18247E+00_JPRB, 0.18128E+00_JPRB, 0.18095E+00_JPRB, 0.18127E+00_JPRB, 0.18240E+00_JPRB, 0.18431E+00_JPRB, 0.18482E+00_JPRB ,&
&  0.18563E+00_JPRB, 0.18539E+00_JPRB, 0.18522E+00_JPRB, 0.18671E+00_JPRB, 0.18849E+00_JPRB, 0.19050E+00_JPRB, 0.19327E+00_JPRB ,&
&  0.19516E+00_JPRB, 0.20175E+00_JPRB, 0.21367E+00_JPRB, 0.23171E+00_JPRB, 0.25813E+00_JPRB, 0.27432E+00_JPRB, 0.31345E+00_JPRB ,&
&  0.37944E+00_JPRB, 0.52609E+00_JPRB, 0.59010E+00_JPRB, 0.44350E+00_JPRB, 0.45731E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.50036E+00_JPRB, 0.51189E+00_JPRB, 0.52472E+00_JPRB, 0.51685E+00_JPRB, 0.50307E+00_JPRB, 0.49535E+00_JPRB, 0.49495E+00_JPRB ,&
&  0.49433E+00_JPRB, 0.49151E+00_JPRB, 0.47921E+00_JPRB, 0.46743E+00_JPRB, 0.44968E+00_JPRB, 0.40112E+00_JPRB, 0.38221E+00_JPRB ,&
&  0.36465E+00_JPRB, 0.36707E+00_JPRB, 0.37206E+00_JPRB, 0.37909E+00_JPRB, 0.39376E+00_JPRB, 0.40503E+00_JPRB, 0.40770E+00_JPRB ,&
&  0.39834E+00_JPRB, 0.38032E+00_JPRB, 0.36324E+00_JPRB, 0.34969E+00_JPRB, 0.34937E+00_JPRB, 0.34980E+00_JPRB, 0.34323E+00_JPRB ,&
&  0.32963E+00_JPRB, 0.32325E+00_JPRB, 0.32272E+00_JPRB, 0.34287E+00_JPRB, 0.39664E+00_JPRB, 0.41718E+00_JPRB, 0.39123E+00_JPRB ,&
&  0.39548E+00_JPRB, 0.39096E+00_JPRB, 0.38719E+00_JPRB, 0.38895E+00_JPRB, 0.37942E+00_JPRB, 0.36209E+00_JPRB, 0.32840E+00_JPRB ,&
&  0.28365E+00_JPRB, 0.26041E+00_JPRB, 0.23946E+00_JPRB, 0.22043E+00_JPRB, 0.20646E+00_JPRB, 0.19818E+00_JPRB, 0.19109E+00_JPRB ,&
&  0.18643E+00_JPRB, 0.18260E+00_JPRB, 0.18000E+00_JPRB, 0.17958E+00_JPRB, 0.18053E+00_JPRB, 0.18088E+00_JPRB, 0.18141E+00_JPRB ,&
&  0.18236E+00_JPRB, 0.18272E+00_JPRB, 0.18281E+00_JPRB, 0.18364E+00_JPRB, 0.18476E+00_JPRB, 0.18698E+00_JPRB, 0.19028E+00_JPRB ,&
&  0.19436E+00_JPRB, 0.19803E+00_JPRB, 0.20165E+00_JPRB, 0.20724E+00_JPRB, 0.20771E+00_JPRB, 0.20668E+00_JPRB, 0.19918E+00_JPRB ,&
&  0.19383E+00_JPRB, 0.19133E+00_JPRB, 0.18883E+00_JPRB, 0.18808E+00_JPRB, 0.18898E+00_JPRB, 0.19061E+00_JPRB, 0.19223E+00_JPRB ,&
&  0.19332E+00_JPRB, 0.19337E+00_JPRB, 0.19271E+00_JPRB, 0.19276E+00_JPRB, 0.19343E+00_JPRB, 0.19365E+00_JPRB, 0.19690E+00_JPRB ,&
&  0.20392E+00_JPRB, 0.21440E+00_JPRB, 0.22605E+00_JPRB, 0.24234E+00_JPRB, 0.25740E+00_JPRB, 0.27363E+00_JPRB, 0.30090E+00_JPRB ,&
&  0.34108E+00_JPRB, 0.45057E+00_JPRB, 0.52076E+00_JPRB, 0.45634E+00_JPRB, 0.47981E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.53066E+00_JPRB, 0.53449E+00_JPRB, 0.53904E+00_JPRB, 0.54168E+00_JPRB, 0.53644E+00_JPRB, 0.52936E+00_JPRB, 0.52476E+00_JPRB ,&
&  0.52987E+00_JPRB, 0.52965E+00_JPRB, 0.50605E+00_JPRB, 0.48741E+00_JPRB, 0.47662E+00_JPRB, 0.47306E+00_JPRB, 0.46984E+00_JPRB ,&
&  0.46381E+00_JPRB, 0.46187E+00_JPRB, 0.46336E+00_JPRB, 0.47498E+00_JPRB, 0.48460E+00_JPRB, 0.49298E+00_JPRB, 0.50051E+00_JPRB ,&
&  0.49657E+00_JPRB, 0.51002E+00_JPRB, 0.53195E+00_JPRB, 0.53156E+00_JPRB, 0.52917E+00_JPRB, 0.52230E+00_JPRB, 0.50818E+00_JPRB ,&
&  0.46098E+00_JPRB, 0.39708E+00_JPRB, 0.33404E+00_JPRB, 0.34484E+00_JPRB, 0.42401E+00_JPRB, 0.43482E+00_JPRB, 0.41830E+00_JPRB ,&
&  0.42847E+00_JPRB, 0.42486E+00_JPRB, 0.40719E+00_JPRB, 0.39441E+00_JPRB, 0.37912E+00_JPRB, 0.36914E+00_JPRB, 0.36234E+00_JPRB ,&
&  0.35962E+00_JPRB, 0.35799E+00_JPRB, 0.34407E+00_JPRB, 0.29678E+00_JPRB, 0.26828E+00_JPRB, 0.25125E+00_JPRB, 0.23387E+00_JPRB ,&
&  0.22044E+00_JPRB, 0.21156E+00_JPRB, 0.20972E+00_JPRB, 0.20988E+00_JPRB, 0.21391E+00_JPRB, 0.21526E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21586E+00_JPRB, 0.21454E+00_JPRB, 0.21043E+00_JPRB, 0.20586E+00_JPRB, 0.20273E+00_JPRB, 0.20207E+00_JPRB, 0.20185E+00_JPRB ,&
&  0.20433E+00_JPRB, 0.20835E+00_JPRB, 0.22429E+00_JPRB, 0.24168E+00_JPRB, 0.24220E+00_JPRB, 0.22700E+00_JPRB, 0.21178E+00_JPRB ,&
&  0.20250E+00_JPRB, 0.19769E+00_JPRB, 0.19472E+00_JPRB, 0.19417E+00_JPRB, 0.19666E+00_JPRB, 0.20012E+00_JPRB, 0.20107E+00_JPRB ,&
&  0.20102E+00_JPRB, 0.20179E+00_JPRB, 0.20200E+00_JPRB, 0.20260E+00_JPRB, 0.20337E+00_JPRB, 0.20578E+00_JPRB, 0.21025E+00_JPRB ,&
&  0.21557E+00_JPRB, 0.22511E+00_JPRB, 0.23602E+00_JPRB, 0.25072E+00_JPRB, 0.26139E+00_JPRB, 0.26899E+00_JPRB, 0.27808E+00_JPRB ,&
&  0.29497E+00_JPRB, 0.33238E+00_JPRB, 0.40119E+00_JPRB, 0.52813E+00_JPRB, 0.52383E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.58619E+00_JPRB, 0.57473E+00_JPRB, 0.56713E+00_JPRB, 0.56183E+00_JPRB, 0.55534E+00_JPRB, 0.55440E+00_JPRB, 0.55393E+00_JPRB ,&
&  0.54831E+00_JPRB, 0.54001E+00_JPRB, 0.54078E+00_JPRB, 0.52329E+00_JPRB, 0.49852E+00_JPRB, 0.48195E+00_JPRB, 0.47971E+00_JPRB ,&
&  0.48376E+00_JPRB, 0.49110E+00_JPRB, 0.50044E+00_JPRB, 0.50973E+00_JPRB, 0.51490E+00_JPRB, 0.52675E+00_JPRB, 0.54783E+00_JPRB ,&
&  0.57241E+00_JPRB, 0.58633E+00_JPRB, 0.59080E+00_JPRB, 0.58313E+00_JPRB, 0.57086E+00_JPRB, 0.55804E+00_JPRB, 0.54739E+00_JPRB ,&
&  0.54027E+00_JPRB, 0.43816E+00_JPRB, 0.36885E+00_JPRB, 0.37461E+00_JPRB, 0.44634E+00_JPRB, 0.43056E+00_JPRB, 0.42801E+00_JPRB ,&
&  0.44047E+00_JPRB, 0.44135E+00_JPRB, 0.42860E+00_JPRB, 0.41189E+00_JPRB, 0.39609E+00_JPRB, 0.38452E+00_JPRB, 0.37577E+00_JPRB ,&
&  0.37148E+00_JPRB, 0.37781E+00_JPRB, 0.38621E+00_JPRB, 0.39928E+00_JPRB, 0.40298E+00_JPRB, 0.39078E+00_JPRB, 0.37609E+00_JPRB ,&
&  0.36180E+00_JPRB, 0.35679E+00_JPRB, 0.35893E+00_JPRB, 0.36474E+00_JPRB, 0.37237E+00_JPRB, 0.37834E+00_JPRB, 0.37652E+00_JPRB ,&
&  0.34719E+00_JPRB, 0.32449E+00_JPRB, 0.29647E+00_JPRB, 0.27013E+00_JPRB, 0.24717E+00_JPRB, 0.23164E+00_JPRB, 0.22382E+00_JPRB ,&
&  0.22165E+00_JPRB, 0.22814E+00_JPRB, 0.24897E+00_JPRB, 0.29776E+00_JPRB, 0.36768E+00_JPRB, 0.28915E+00_JPRB, 0.24978E+00_JPRB ,&
&  0.22780E+00_JPRB, 0.21486E+00_JPRB, 0.21004E+00_JPRB, 0.21055E+00_JPRB, 0.21415E+00_JPRB, 0.21961E+00_JPRB, 0.22388E+00_JPRB ,&
&  0.22729E+00_JPRB, 0.23073E+00_JPRB, 0.23313E+00_JPRB, 0.23439E+00_JPRB, 0.23611E+00_JPRB, 0.23721E+00_JPRB, 0.23617E+00_JPRB ,&
&  0.23438E+00_JPRB, 0.23966E+00_JPRB, 0.25607E+00_JPRB, 0.28223E+00_JPRB, 0.31373E+00_JPRB, 0.34761E+00_JPRB, 0.39903E+00_JPRB ,&
&  0.46469E+00_JPRB, 0.54139E+00_JPRB, 0.55820E+00_JPRB, 0.56953E+00_JPRB, 0.57958E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.62518E+00_JPRB, 0.60009E+00_JPRB, 0.58334E+00_JPRB, 0.57009E+00_JPRB, 0.55184E+00_JPRB, 0.53801E+00_JPRB, 0.53248E+00_JPRB ,&
&  0.54078E+00_JPRB, 0.53965E+00_JPRB, 0.52999E+00_JPRB, 0.52183E+00_JPRB, 0.50584E+00_JPRB, 0.49152E+00_JPRB, 0.48568E+00_JPRB ,&
&  0.49444E+00_JPRB, 0.51102E+00_JPRB, 0.53122E+00_JPRB, 0.53998E+00_JPRB, 0.53970E+00_JPRB, 0.54823E+00_JPRB, 0.56746E+00_JPRB ,&
&  0.59584E+00_JPRB, 0.61565E+00_JPRB, 0.62300E+00_JPRB, 0.61037E+00_JPRB, 0.59323E+00_JPRB, 0.58526E+00_JPRB, 0.57082E+00_JPRB ,&
&  0.55409E+00_JPRB, 0.55795E+00_JPRB, 0.48987E+00_JPRB, 0.48240E+00_JPRB, 0.45889E+00_JPRB, 0.44477E+00_JPRB, 0.44132E+00_JPRB ,&
&  0.45480E+00_JPRB, 0.45938E+00_JPRB, 0.45168E+00_JPRB, 0.44015E+00_JPRB, 0.43191E+00_JPRB, 0.42059E+00_JPRB, 0.40987E+00_JPRB ,&
&  0.40211E+00_JPRB, 0.40438E+00_JPRB, 0.41339E+00_JPRB, 0.42267E+00_JPRB, 0.43386E+00_JPRB, 0.44449E+00_JPRB, 0.44090E+00_JPRB ,&
&  0.42925E+00_JPRB, 0.42712E+00_JPRB, 0.43365E+00_JPRB, 0.44763E+00_JPRB, 0.46292E+00_JPRB, 0.47025E+00_JPRB, 0.47454E+00_JPRB ,&
&  0.46751E+00_JPRB, 0.45106E+00_JPRB, 0.43687E+00_JPRB, 0.42513E+00_JPRB, 0.40050E+00_JPRB, 0.33679E+00_JPRB, 0.31378E+00_JPRB ,&
&  0.32408E+00_JPRB, 0.38321E+00_JPRB, 0.45440E+00_JPRB, 0.44521E+00_JPRB, 0.43974E+00_JPRB, 0.42575E+00_JPRB, 0.39039E+00_JPRB ,&
&  0.35089E+00_JPRB, 0.28908E+00_JPRB, 0.26659E+00_JPRB, 0.26257E+00_JPRB, 0.26116E+00_JPRB, 0.26581E+00_JPRB, 0.27754E+00_JPRB ,&
&  0.29321E+00_JPRB, 0.32091E+00_JPRB, 0.33871E+00_JPRB, 0.34569E+00_JPRB, 0.34452E+00_JPRB, 0.32846E+00_JPRB, 0.31477E+00_JPRB ,&
&  0.30522E+00_JPRB, 0.32931E+00_JPRB, 0.38688E+00_JPRB, 0.46972E+00_JPRB, 0.46731E+00_JPRB, 0.49051E+00_JPRB, 0.51454E+00_JPRB ,&
&  0.53750E+00_JPRB, 0.57649E+00_JPRB, 0.60245E+00_JPRB, 0.60828E+00_JPRB, 0.62664E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.64851E+00_JPRB, 0.62136E+00_JPRB, 0.59276E+00_JPRB, 0.56795E+00_JPRB, 0.55566E+00_JPRB, 0.53776E+00_JPRB, 0.52291E+00_JPRB ,&
&  0.52200E+00_JPRB, 0.52998E+00_JPRB, 0.52561E+00_JPRB, 0.51993E+00_JPRB, 0.51067E+00_JPRB, 0.50298E+00_JPRB, 0.49376E+00_JPRB ,&
&  0.48998E+00_JPRB, 0.50201E+00_JPRB, 0.52810E+00_JPRB, 0.54139E+00_JPRB, 0.54801E+00_JPRB, 0.57876E+00_JPRB, 0.61004E+00_JPRB ,&
&  0.61886E+00_JPRB, 0.63253E+00_JPRB, 0.63693E+00_JPRB, 0.61244E+00_JPRB, 0.59645E+00_JPRB, 0.59028E+00_JPRB, 0.59416E+00_JPRB ,&
&  0.58935E+00_JPRB, 0.59360E+00_JPRB, 0.58579E+00_JPRB, 0.51498E+00_JPRB, 0.51430E+00_JPRB, 0.48354E+00_JPRB, 0.47332E+00_JPRB ,&
&  0.46863E+00_JPRB, 0.47463E+00_JPRB, 0.47171E+00_JPRB, 0.46343E+00_JPRB, 0.46020E+00_JPRB, 0.46335E+00_JPRB, 0.46421E+00_JPRB ,&
&  0.45904E+00_JPRB, 0.44800E+00_JPRB, 0.45230E+00_JPRB, 0.45873E+00_JPRB, 0.46253E+00_JPRB, 0.46962E+00_JPRB, 0.48013E+00_JPRB ,&
&  0.46994E+00_JPRB, 0.46112E+00_JPRB, 0.45258E+00_JPRB, 0.45268E+00_JPRB, 0.45900E+00_JPRB, 0.47345E+00_JPRB, 0.47874E+00_JPRB ,&
&  0.47679E+00_JPRB, 0.47552E+00_JPRB, 0.47637E+00_JPRB, 0.48258E+00_JPRB, 0.49769E+00_JPRB, 0.51284E+00_JPRB, 0.51414E+00_JPRB ,&
&  0.51507E+00_JPRB, 0.51939E+00_JPRB, 0.52382E+00_JPRB, 0.52599E+00_JPRB, 0.52230E+00_JPRB, 0.50734E+00_JPRB, 0.48987E+00_JPRB ,&
&  0.47331E+00_JPRB, 0.45707E+00_JPRB, 0.44306E+00_JPRB, 0.43826E+00_JPRB, 0.45085E+00_JPRB, 0.47954E+00_JPRB, 0.49543E+00_JPRB ,&
&  0.49901E+00_JPRB, 0.50098E+00_JPRB, 0.50468E+00_JPRB, 0.50622E+00_JPRB, 0.50864E+00_JPRB, 0.50704E+00_JPRB, 0.50087E+00_JPRB ,&
&  0.48978E+00_JPRB, 0.47823E+00_JPRB, 0.47464E+00_JPRB, 0.47787E+00_JPRB, 0.48197E+00_JPRB, 0.49281E+00_JPRB, 0.50829E+00_JPRB ,&
&  0.52948E+00_JPRB, 0.56718E+00_JPRB, 0.59933E+00_JPRB, 0.62440E+00_JPRB, 0.64269E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.66604E+00_JPRB, 0.66412E+00_JPRB, 0.63574E+00_JPRB, 0.60427E+00_JPRB, 0.58523E+00_JPRB, 0.56396E+00_JPRB, 0.54681E+00_JPRB ,&
&  0.54526E+00_JPRB, 0.54272E+00_JPRB, 0.54092E+00_JPRB, 0.53874E+00_JPRB, 0.53715E+00_JPRB, 0.53401E+00_JPRB, 0.52191E+00_JPRB ,&
&  0.51763E+00_JPRB, 0.52440E+00_JPRB, 0.54699E+00_JPRB, 0.57264E+00_JPRB, 0.59332E+00_JPRB, 0.61992E+00_JPRB, 0.63092E+00_JPRB ,&
&  0.64227E+00_JPRB, 0.65445E+00_JPRB, 0.65688E+00_JPRB, 0.62478E+00_JPRB, 0.59834E+00_JPRB, 0.59959E+00_JPRB, 0.61058E+00_JPRB ,&
&  0.61085E+00_JPRB, 0.61408E+00_JPRB, 0.61808E+00_JPRB, 0.61413E+00_JPRB, 0.59316E+00_JPRB, 0.55506E+00_JPRB, 0.52531E+00_JPRB ,&
&  0.50119E+00_JPRB, 0.48267E+00_JPRB, 0.47753E+00_JPRB, 0.46707E+00_JPRB, 0.46620E+00_JPRB, 0.47643E+00_JPRB, 0.48896E+00_JPRB ,&
&  0.50172E+00_JPRB, 0.50723E+00_JPRB, 0.51410E+00_JPRB, 0.51085E+00_JPRB, 0.50640E+00_JPRB, 0.50884E+00_JPRB, 0.50844E+00_JPRB ,&
&  0.50746E+00_JPRB, 0.49831E+00_JPRB, 0.49575E+00_JPRB, 0.48887E+00_JPRB, 0.48508E+00_JPRB, 0.48573E+00_JPRB, 0.48896E+00_JPRB ,&
&  0.49024E+00_JPRB, 0.49565E+00_JPRB, 0.49724E+00_JPRB, 0.49928E+00_JPRB, 0.50861E+00_JPRB, 0.52513E+00_JPRB, 0.54177E+00_JPRB ,&
&  0.55424E+00_JPRB, 0.56404E+00_JPRB, 0.57190E+00_JPRB, 0.57771E+00_JPRB, 0.57748E+00_JPRB, 0.57295E+00_JPRB, 0.55914E+00_JPRB ,&
&  0.52852E+00_JPRB, 0.49561E+00_JPRB, 0.47076E+00_JPRB, 0.45081E+00_JPRB, 0.44675E+00_JPRB, 0.45724E+00_JPRB, 0.47472E+00_JPRB ,&
&  0.48795E+00_JPRB, 0.49215E+00_JPRB, 0.48652E+00_JPRB, 0.48506E+00_JPRB, 0.49051E+00_JPRB, 0.50028E+00_JPRB, 0.50784E+00_JPRB ,&
&  0.50711E+00_JPRB, 0.50373E+00_JPRB, 0.50699E+00_JPRB, 0.51315E+00_JPRB, 0.52620E+00_JPRB, 0.54292E+00_JPRB, 0.56116E+00_JPRB ,&
&  0.57958E+00_JPRB, 0.59411E+00_JPRB, 0.61317E+00_JPRB, 0.63262E+00_JPRB, 0.65806E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.67075E+00_JPRB, 0.66254E+00_JPRB, 0.64822E+00_JPRB, 0.62228E+00_JPRB, 0.60539E+00_JPRB, 0.57841E+00_JPRB, 0.56295E+00_JPRB ,&
&  0.56503E+00_JPRB, 0.56609E+00_JPRB, 0.57110E+00_JPRB, 0.58062E+00_JPRB, 0.57885E+00_JPRB, 0.57450E+00_JPRB, 0.57034E+00_JPRB ,&
&  0.57376E+00_JPRB, 0.57433E+00_JPRB, 0.58288E+00_JPRB, 0.59863E+00_JPRB, 0.61525E+00_JPRB, 0.62446E+00_JPRB, 0.63382E+00_JPRB ,&
&  0.64096E+00_JPRB, 0.63464E+00_JPRB, 0.63041E+00_JPRB, 0.60045E+00_JPRB, 0.58309E+00_JPRB, 0.58841E+00_JPRB, 0.59576E+00_JPRB ,&
&  0.60582E+00_JPRB, 0.61638E+00_JPRB, 0.62714E+00_JPRB, 0.63245E+00_JPRB, 0.61862E+00_JPRB, 0.61520E+00_JPRB, 0.60475E+00_JPRB ,&
&  0.56298E+00_JPRB, 0.52201E+00_JPRB, 0.49998E+00_JPRB, 0.49318E+00_JPRB, 0.48905E+00_JPRB, 0.49532E+00_JPRB, 0.50231E+00_JPRB ,&
&  0.51467E+00_JPRB, 0.52609E+00_JPRB, 0.52913E+00_JPRB, 0.53257E+00_JPRB, 0.53415E+00_JPRB, 0.53316E+00_JPRB, 0.53312E+00_JPRB ,&
&  0.52974E+00_JPRB, 0.52744E+00_JPRB, 0.53443E+00_JPRB, 0.53506E+00_JPRB, 0.52658E+00_JPRB, 0.52453E+00_JPRB, 0.52298E+00_JPRB ,&
&  0.52070E+00_JPRB, 0.51832E+00_JPRB, 0.51529E+00_JPRB, 0.52521E+00_JPRB, 0.55036E+00_JPRB, 0.57941E+00_JPRB, 0.60466E+00_JPRB ,&
&  0.61890E+00_JPRB, 0.63037E+00_JPRB, 0.63339E+00_JPRB, 0.63549E+00_JPRB, 0.63505E+00_JPRB, 0.62648E+00_JPRB, 0.62344E+00_JPRB ,&
&  0.61564E+00_JPRB, 0.59667E+00_JPRB, 0.57132E+00_JPRB, 0.53189E+00_JPRB, 0.49359E+00_JPRB, 0.47178E+00_JPRB, 0.47136E+00_JPRB ,&
&  0.48347E+00_JPRB, 0.49580E+00_JPRB, 0.50219E+00_JPRB, 0.51039E+00_JPRB, 0.52540E+00_JPRB, 0.54331E+00_JPRB, 0.56195E+00_JPRB ,&
&  0.57347E+00_JPRB, 0.57306E+00_JPRB, 0.56998E+00_JPRB, 0.57729E+00_JPRB, 0.58737E+00_JPRB, 0.60438E+00_JPRB, 0.62318E+00_JPRB ,&
&  0.63985E+00_JPRB, 0.63935E+00_JPRB, 0.63973E+00_JPRB, 0.64087E+00_JPRB, 0.66040E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.67366E+00_JPRB, 0.67472E+00_JPRB, 0.65952E+00_JPRB, 0.64879E+00_JPRB, 0.63288E+00_JPRB, 0.61214E+00_JPRB, 0.59360E+00_JPRB ,&
&  0.59471E+00_JPRB, 0.60932E+00_JPRB, 0.62353E+00_JPRB, 0.63401E+00_JPRB, 0.64956E+00_JPRB, 0.65291E+00_JPRB, 0.65986E+00_JPRB ,&
&  0.65637E+00_JPRB, 0.61974E+00_JPRB, 0.61409E+00_JPRB, 0.62194E+00_JPRB, 0.62383E+00_JPRB, 0.62950E+00_JPRB, 0.63976E+00_JPRB ,&
&  0.64928E+00_JPRB, 0.64439E+00_JPRB, 0.63624E+00_JPRB, 0.60690E+00_JPRB, 0.60026E+00_JPRB, 0.60517E+00_JPRB, 0.61001E+00_JPRB ,&
&  0.62027E+00_JPRB, 0.63659E+00_JPRB, 0.64159E+00_JPRB, 0.64947E+00_JPRB, 0.65914E+00_JPRB, 0.66388E+00_JPRB, 0.66073E+00_JPRB ,&
&  0.63541E+00_JPRB, 0.60994E+00_JPRB, 0.59970E+00_JPRB, 0.58279E+00_JPRB, 0.56168E+00_JPRB, 0.54677E+00_JPRB, 0.53874E+00_JPRB ,&
&  0.53419E+00_JPRB, 0.53900E+00_JPRB, 0.54517E+00_JPRB, 0.55907E+00_JPRB, 0.56420E+00_JPRB, 0.56829E+00_JPRB, 0.57334E+00_JPRB ,&
&  0.57248E+00_JPRB, 0.56785E+00_JPRB, 0.56753E+00_JPRB, 0.57726E+00_JPRB, 0.58346E+00_JPRB, 0.58170E+00_JPRB, 0.57784E+00_JPRB ,&
&  0.56555E+00_JPRB, 0.55170E+00_JPRB, 0.55901E+00_JPRB, 0.57368E+00_JPRB, 0.58012E+00_JPRB, 0.60476E+00_JPRB, 0.63681E+00_JPRB ,&
&  0.66394E+00_JPRB, 0.68184E+00_JPRB, 0.68153E+00_JPRB, 0.67844E+00_JPRB, 0.67719E+00_JPRB, 0.68460E+00_JPRB, 0.69710E+00_JPRB ,&
&  0.68156E+00_JPRB, 0.67252E+00_JPRB, 0.67061E+00_JPRB, 0.65170E+00_JPRB, 0.61592E+00_JPRB, 0.58555E+00_JPRB, 0.56296E+00_JPRB ,&
&  0.54481E+00_JPRB, 0.53433E+00_JPRB, 0.53783E+00_JPRB, 0.54163E+00_JPRB, 0.55477E+00_JPRB, 0.59030E+00_JPRB, 0.62704E+00_JPRB ,&
&  0.66677E+00_JPRB, 0.68699E+00_JPRB, 0.69291E+00_JPRB, 0.70034E+00_JPRB, 0.70549E+00_JPRB, 0.69404E+00_JPRB, 0.68450E+00_JPRB ,&
&  0.68159E+00_JPRB, 0.67324E+00_JPRB, 0.66395E+00_JPRB, 0.65407E+00_JPRB, 0.66097E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.66754E+00_JPRB, 0.68890E+00_JPRB, 0.65613E+00_JPRB, 0.63836E+00_JPRB, 0.61847E+00_JPRB, 0.61451E+00_JPRB, 0.60554E+00_JPRB ,&
&  0.61560E+00_JPRB, 0.63212E+00_JPRB, 0.65565E+00_JPRB, 0.66360E+00_JPRB, 0.67057E+00_JPRB, 0.67358E+00_JPRB, 0.69101E+00_JPRB ,&
&  0.68693E+00_JPRB, 0.67909E+00_JPRB, 0.68254E+00_JPRB, 0.67243E+00_JPRB, 0.66379E+00_JPRB, 0.66831E+00_JPRB, 0.67650E+00_JPRB ,&
&  0.68608E+00_JPRB, 0.67735E+00_JPRB, 0.66008E+00_JPRB, 0.63356E+00_JPRB, 0.62098E+00_JPRB, 0.61837E+00_JPRB, 0.62569E+00_JPRB ,&
&  0.63101E+00_JPRB, 0.65368E+00_JPRB, 0.59316E+00_JPRB, 0.64864E+00_JPRB, 0.65854E+00_JPRB, 0.65858E+00_JPRB, 0.65740E+00_JPRB ,&
&  0.66220E+00_JPRB, 0.65201E+00_JPRB, 0.64314E+00_JPRB, 0.63465E+00_JPRB, 0.61257E+00_JPRB, 0.59462E+00_JPRB, 0.57524E+00_JPRB ,&
&  0.55804E+00_JPRB, 0.54761E+00_JPRB, 0.55254E+00_JPRB, 0.56954E+00_JPRB, 0.58698E+00_JPRB, 0.59690E+00_JPRB, 0.60306E+00_JPRB ,&
&  0.60639E+00_JPRB, 0.59909E+00_JPRB, 0.58919E+00_JPRB, 0.58944E+00_JPRB, 0.59627E+00_JPRB, 0.60167E+00_JPRB, 0.60556E+00_JPRB ,&
&  0.60689E+00_JPRB, 0.58385E+00_JPRB, 0.57081E+00_JPRB, 0.58682E+00_JPRB, 0.60372E+00_JPRB, 0.62184E+00_JPRB, 0.64556E+00_JPRB ,&
&  0.66342E+00_JPRB, 0.67019E+00_JPRB, 0.66910E+00_JPRB, 0.67592E+00_JPRB, 0.68299E+00_JPRB, 0.69148E+00_JPRB, 0.69827E+00_JPRB ,&
&  0.68893E+00_JPRB, 0.68733E+00_JPRB, 0.68848E+00_JPRB, 0.68883E+00_JPRB, 0.68054E+00_JPRB, 0.65852E+00_JPRB, 0.62712E+00_JPRB ,&
&  0.61280E+00_JPRB, 0.59789E+00_JPRB, 0.58671E+00_JPRB, 0.58416E+00_JPRB, 0.58688E+00_JPRB, 0.61260E+00_JPRB, 0.64899E+00_JPRB ,&
&  0.68502E+00_JPRB, 0.71417E+00_JPRB, 0.73919E+00_JPRB, 0.76207E+00_JPRB, 0.75948E+00_JPRB, 0.73706E+00_JPRB, 0.72683E+00_JPRB ,&
&  0.71150E+00_JPRB, 0.69528E+00_JPRB, 0.68471E+00_JPRB, 0.67605E+00_JPRB, 0.66609E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.70395E+00_JPRB, 0.70489E+00_JPRB, 0.69468E+00_JPRB, 0.65665E+00_JPRB, 0.62796E+00_JPRB, 0.63807E+00_JPRB, 0.64768E+00_JPRB ,&
&  0.64714E+00_JPRB, 0.66167E+00_JPRB, 0.68677E+00_JPRB, 0.71237E+00_JPRB, 0.72959E+00_JPRB, 0.72981E+00_JPRB, 0.73435E+00_JPRB ,&
&  0.73246E+00_JPRB, 0.73076E+00_JPRB, 0.74101E+00_JPRB, 0.74602E+00_JPRB, 0.73545E+00_JPRB, 0.71433E+00_JPRB, 0.71145E+00_JPRB ,&
&  0.71714E+00_JPRB, 0.71286E+00_JPRB, 0.69447E+00_JPRB, 0.68191E+00_JPRB, 0.67176E+00_JPRB, 0.67681E+00_JPRB, 0.68285E+00_JPRB ,&
&  0.67791E+00_JPRB, 0.62433E+00_JPRB, 0.66235E+00_JPRB, 0.67419E+00_JPRB, 0.68333E+00_JPRB, 0.67453E+00_JPRB, 0.67988E+00_JPRB ,&
&  0.69158E+00_JPRB, 0.69897E+00_JPRB, 0.67899E+00_JPRB, 0.65519E+00_JPRB, 0.63388E+00_JPRB, 0.61649E+00_JPRB, 0.58888E+00_JPRB ,&
&  0.57840E+00_JPRB, 0.57555E+00_JPRB, 0.58873E+00_JPRB, 0.60260E+00_JPRB, 0.61859E+00_JPRB, 0.62509E+00_JPRB, 0.63544E+00_JPRB ,&
&  0.63794E+00_JPRB, 0.63293E+00_JPRB, 0.62847E+00_JPRB, 0.62540E+00_JPRB, 0.62546E+00_JPRB, 0.62019E+00_JPRB, 0.61249E+00_JPRB ,&
&  0.60694E+00_JPRB, 0.60547E+00_JPRB, 0.60986E+00_JPRB, 0.61915E+00_JPRB, 0.63031E+00_JPRB, 0.64685E+00_JPRB, 0.65929E+00_JPRB ,&
&  0.67496E+00_JPRB, 0.67951E+00_JPRB, 0.68364E+00_JPRB, 0.69647E+00_JPRB, 0.70505E+00_JPRB, 0.70358E+00_JPRB, 0.70068E+00_JPRB ,&
&  0.69838E+00_JPRB, 0.69349E+00_JPRB, 0.69874E+00_JPRB, 0.69845E+00_JPRB, 0.69965E+00_JPRB, 0.69207E+00_JPRB, 0.67779E+00_JPRB ,&
&  0.67250E+00_JPRB, 0.66839E+00_JPRB, 0.65979E+00_JPRB, 0.65709E+00_JPRB, 0.65497E+00_JPRB, 0.67228E+00_JPRB, 0.70269E+00_JPRB ,&
&  0.71875E+00_JPRB, 0.74044E+00_JPRB, 0.77193E+00_JPRB, 0.79249E+00_JPRB, 0.79090E+00_JPRB, 0.78904E+00_JPRB, 0.77668E+00_JPRB ,&
&  0.75687E+00_JPRB, 0.74289E+00_JPRB, 0.73057E+00_JPRB, 0.72679E+00_JPRB, 0.72351E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.73607E+00_JPRB, 0.72973E+00_JPRB, 0.71535E+00_JPRB, 0.68903E+00_JPRB, 0.65587E+00_JPRB, 0.66515E+00_JPRB, 0.67838E+00_JPRB ,&
&  0.67306E+00_JPRB, 0.66409E+00_JPRB, 0.67451E+00_JPRB, 0.69949E+00_JPRB, 0.72442E+00_JPRB, 0.73910E+00_JPRB, 0.74235E+00_JPRB ,&
&  0.73844E+00_JPRB, 0.74147E+00_JPRB, 0.75047E+00_JPRB, 0.76772E+00_JPRB, 0.76679E+00_JPRB, 0.74869E+00_JPRB, 0.73737E+00_JPRB ,&
&  0.73383E+00_JPRB, 0.73168E+00_JPRB, 0.72856E+00_JPRB, 0.71151E+00_JPRB, 0.70014E+00_JPRB, 0.69740E+00_JPRB, 0.69801E+00_JPRB ,&
&  0.69600E+00_JPRB, 0.55830E+00_JPRB, 0.65462E+00_JPRB, 0.66522E+00_JPRB, 0.67737E+00_JPRB, 0.67608E+00_JPRB, 0.67840E+00_JPRB ,&
&  0.68433E+00_JPRB, 0.70056E+00_JPRB, 0.69295E+00_JPRB, 0.66623E+00_JPRB, 0.63797E+00_JPRB, 0.60976E+00_JPRB, 0.59094E+00_JPRB ,&
&  0.58190E+00_JPRB, 0.58504E+00_JPRB, 0.59502E+00_JPRB, 0.60540E+00_JPRB, 0.61660E+00_JPRB, 0.61859E+00_JPRB, 0.62277E+00_JPRB ,&
&  0.62602E+00_JPRB, 0.63206E+00_JPRB, 0.63572E+00_JPRB, 0.64273E+00_JPRB, 0.64618E+00_JPRB, 0.64398E+00_JPRB, 0.63757E+00_JPRB ,&
&  0.62841E+00_JPRB, 0.62276E+00_JPRB, 0.62242E+00_JPRB, 0.62487E+00_JPRB, 0.63165E+00_JPRB, 0.64276E+00_JPRB, 0.65330E+00_JPRB ,&
&  0.66562E+00_JPRB, 0.67357E+00_JPRB, 0.67779E+00_JPRB, 0.68008E+00_JPRB, 0.68947E+00_JPRB, 0.69938E+00_JPRB, 0.69973E+00_JPRB ,&
&  0.69415E+00_JPRB, 0.68799E+00_JPRB, 0.68716E+00_JPRB, 0.68519E+00_JPRB, 0.68669E+00_JPRB, 0.68840E+00_JPRB, 0.68498E+00_JPRB ,&
&  0.67729E+00_JPRB, 0.67563E+00_JPRB, 0.67682E+00_JPRB, 0.66600E+00_JPRB, 0.66727E+00_JPRB, 0.67677E+00_JPRB, 0.69458E+00_JPRB ,&
&  0.71673E+00_JPRB, 0.74620E+00_JPRB, 0.77555E+00_JPRB, 0.80096E+00_JPRB, 0.81616E+00_JPRB, 0.82117E+00_JPRB, 0.81578E+00_JPRB ,&
&  0.79853E+00_JPRB, 0.77369E+00_JPRB, 0.75561E+00_JPRB, 0.74693E+00_JPRB, 0.74931E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.76265E+00_JPRB, 0.75365E+00_JPRB, 0.74119E+00_JPRB, 0.72435E+00_JPRB, 0.70964E+00_JPRB, 0.70217E+00_JPRB, 0.70863E+00_JPRB ,&
&  0.70796E+00_JPRB, 0.69649E+00_JPRB, 0.70606E+00_JPRB, 0.71940E+00_JPRB, 0.73074E+00_JPRB, 0.74177E+00_JPRB, 0.74701E+00_JPRB ,&
&  0.75279E+00_JPRB, 0.75168E+00_JPRB, 0.75608E+00_JPRB, 0.76870E+00_JPRB, 0.78723E+00_JPRB, 0.78868E+00_JPRB, 0.78559E+00_JPRB ,&
&  0.76267E+00_JPRB, 0.75081E+00_JPRB, 0.74002E+00_JPRB, 0.72708E+00_JPRB, 0.71584E+00_JPRB, 0.71276E+00_JPRB, 0.71646E+00_JPRB ,&
&  0.60608E+00_JPRB, 0.51008E+00_JPRB, 0.60813E+00_JPRB, 0.69126E+00_JPRB, 0.69832E+00_JPRB, 0.70488E+00_JPRB, 0.72256E+00_JPRB ,&
&  0.72934E+00_JPRB, 0.73249E+00_JPRB, 0.73006E+00_JPRB, 0.71621E+00_JPRB, 0.68397E+00_JPRB, 0.65147E+00_JPRB, 0.63279E+00_JPRB ,&
&  0.62116E+00_JPRB, 0.62580E+00_JPRB, 0.62789E+00_JPRB, 0.63553E+00_JPRB, 0.63618E+00_JPRB, 0.62960E+00_JPRB, 0.62897E+00_JPRB ,&
&  0.63105E+00_JPRB, 0.64246E+00_JPRB, 0.65372E+00_JPRB, 0.66248E+00_JPRB, 0.66668E+00_JPRB, 0.66881E+00_JPRB, 0.66744E+00_JPRB ,&
&  0.66677E+00_JPRB, 0.66565E+00_JPRB, 0.66232E+00_JPRB, 0.66325E+00_JPRB, 0.66498E+00_JPRB, 0.67021E+00_JPRB, 0.67970E+00_JPRB ,&
&  0.68341E+00_JPRB, 0.68242E+00_JPRB, 0.68282E+00_JPRB, 0.68398E+00_JPRB, 0.69418E+00_JPRB, 0.70284E+00_JPRB, 0.70172E+00_JPRB ,&
&  0.69825E+00_JPRB, 0.69675E+00_JPRB, 0.70403E+00_JPRB, 0.71002E+00_JPRB, 0.71721E+00_JPRB, 0.72069E+00_JPRB, 0.71686E+00_JPRB ,&
&  0.70840E+00_JPRB, 0.69440E+00_JPRB, 0.68395E+00_JPRB, 0.68269E+00_JPRB, 0.69102E+00_JPRB, 0.69683E+00_JPRB, 0.71167E+00_JPRB ,&
&  0.74018E+00_JPRB, 0.77572E+00_JPRB, 0.80618E+00_JPRB, 0.84316E+00_JPRB, 0.86509E+00_JPRB, 0.87613E+00_JPRB, 0.87107E+00_JPRB ,&
&  0.85377E+00_JPRB, 0.81527E+00_JPRB, 0.80395E+00_JPRB, 0.78267E+00_JPRB, 0.77202E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.74165E+00_JPRB, 0.74717E+00_JPRB, 0.75045E+00_JPRB, 0.74074E+00_JPRB, 0.72786E+00_JPRB, 0.72297E+00_JPRB, 0.72939E+00_JPRB ,&
&  0.72512E+00_JPRB, 0.71181E+00_JPRB, 0.71690E+00_JPRB, 0.72171E+00_JPRB, 0.73043E+00_JPRB, 0.73594E+00_JPRB, 0.74244E+00_JPRB ,&
&  0.74909E+00_JPRB, 0.75494E+00_JPRB, 0.75444E+00_JPRB, 0.76374E+00_JPRB, 0.75709E+00_JPRB, 0.75195E+00_JPRB, 0.74915E+00_JPRB ,&
&  0.74025E+00_JPRB, 0.73073E+00_JPRB, 0.72307E+00_JPRB, 0.71298E+00_JPRB, 0.70617E+00_JPRB, 0.69869E+00_JPRB, 0.68412E+00_JPRB ,&
&  0.69766E+00_JPRB, 0.54324E+00_JPRB, 0.60493E+00_JPRB, 0.70935E+00_JPRB, 0.71506E+00_JPRB, 0.72038E+00_JPRB, 0.73575E+00_JPRB ,&
&  0.75584E+00_JPRB, 0.76760E+00_JPRB, 0.75855E+00_JPRB, 0.73579E+00_JPRB, 0.69626E+00_JPRB, 0.67504E+00_JPRB, 0.66158E+00_JPRB ,&
&  0.64721E+00_JPRB, 0.63748E+00_JPRB, 0.63228E+00_JPRB, 0.63464E+00_JPRB, 0.63483E+00_JPRB, 0.63214E+00_JPRB, 0.63052E+00_JPRB ,&
&  0.63594E+00_JPRB, 0.64760E+00_JPRB, 0.65604E+00_JPRB, 0.66436E+00_JPRB, 0.67236E+00_JPRB, 0.67143E+00_JPRB, 0.67364E+00_JPRB ,&
&  0.67443E+00_JPRB, 0.67577E+00_JPRB, 0.67435E+00_JPRB, 0.67287E+00_JPRB, 0.67143E+00_JPRB, 0.68307E+00_JPRB, 0.69189E+00_JPRB ,&
&  0.68994E+00_JPRB, 0.68723E+00_JPRB, 0.69044E+00_JPRB, 0.69632E+00_JPRB, 0.70140E+00_JPRB, 0.70521E+00_JPRB, 0.70269E+00_JPRB ,&
&  0.70049E+00_JPRB, 0.69593E+00_JPRB, 0.69697E+00_JPRB, 0.69824E+00_JPRB, 0.69518E+00_JPRB, 0.69518E+00_JPRB, 0.69905E+00_JPRB ,&
&  0.69398E+00_JPRB, 0.68276E+00_JPRB, 0.68465E+00_JPRB, 0.69726E+00_JPRB, 0.70679E+00_JPRB, 0.71091E+00_JPRB, 0.72287E+00_JPRB ,&
&  0.75050E+00_JPRB, 0.78210E+00_JPRB, 0.80802E+00_JPRB, 0.84041E+00_JPRB, 0.85869E+00_JPRB, 0.86818E+00_JPRB, 0.86743E+00_JPRB ,&
&  0.85315E+00_JPRB, 0.83396E+00_JPRB, 0.60239E+00_JPRB, 0.70653E+00_JPRB, 0.73102E+00_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.74324E+00_JPRB, 0.75893E+00_JPRB, 0.75845E+00_JPRB, 0.75099E+00_JPRB, 0.74468E+00_JPRB, 0.74043E+00_JPRB, 0.74361E+00_JPRB ,&
&  0.74102E+00_JPRB, 0.73455E+00_JPRB, 0.73747E+00_JPRB, 0.74147E+00_JPRB, 0.74644E+00_JPRB, 0.74733E+00_JPRB, 0.75565E+00_JPRB ,&
&  0.75939E+00_JPRB, 0.76333E+00_JPRB, 0.76843E+00_JPRB, 0.76382E+00_JPRB, 0.74572E+00_JPRB, 0.73299E+00_JPRB, 0.71948E+00_JPRB ,&
&  0.71801E+00_JPRB, 0.71752E+00_JPRB, 0.71794E+00_JPRB, 0.71177E+00_JPRB, 0.69324E+00_JPRB, 0.67790E+00_JPRB, 0.66829E+00_JPRB ,&
&  0.66922E+00_JPRB, 0.51999E+00_JPRB, 0.55968E+00_JPRB, 0.70364E+00_JPRB, 0.76919E+00_JPRB, 0.77056E+00_JPRB, 0.77750E+00_JPRB ,&
&  0.79300E+00_JPRB, 0.79275E+00_JPRB, 0.77186E+00_JPRB, 0.74402E+00_JPRB, 0.71014E+00_JPRB, 0.70231E+00_JPRB, 0.68779E+00_JPRB ,&
&  0.67169E+00_JPRB, 0.66599E+00_JPRB, 0.66161E+00_JPRB, 0.66337E+00_JPRB, 0.65629E+00_JPRB, 0.64759E+00_JPRB, 0.65145E+00_JPRB ,&
&  0.66201E+00_JPRB, 0.66404E+00_JPRB, 0.67039E+00_JPRB, 0.67768E+00_JPRB, 0.67648E+00_JPRB, 0.69531E+00_JPRB, 0.70437E+00_JPRB ,&
&  0.70808E+00_JPRB, 0.70752E+00_JPRB, 0.70636E+00_JPRB, 0.70227E+00_JPRB, 0.70299E+00_JPRB, 0.71840E+00_JPRB, 0.72873E+00_JPRB ,&
&  0.72125E+00_JPRB, 0.71114E+00_JPRB, 0.71854E+00_JPRB, 0.72213E+00_JPRB, 0.71668E+00_JPRB, 0.70551E+00_JPRB, 0.69255E+00_JPRB ,&
&  0.68955E+00_JPRB, 0.68858E+00_JPRB, 0.68954E+00_JPRB, 0.68696E+00_JPRB, 0.68139E+00_JPRB, 0.67720E+00_JPRB, 0.67604E+00_JPRB ,&
&  0.67613E+00_JPRB, 0.67663E+00_JPRB, 0.68289E+00_JPRB, 0.69598E+00_JPRB, 0.71195E+00_JPRB, 0.72968E+00_JPRB, 0.74815E+00_JPRB ,&
&  0.77246E+00_JPRB, 0.80094E+00_JPRB, 0.83090E+00_JPRB, 0.79608E+00_JPRB, 0.85299E+00_JPRB, 0.86778E+00_JPRB, 0.87803E+00_JPRB ,&
&  0.87272E+00_JPRB, 0.83914E+00_JPRB, 0.79613E+00_JPRB, 0.63195E+00_JPRB, 0.70763E+00_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.76301E+00_JPRB, 0.79392E+00_JPRB, 0.79090E+00_JPRB, 0.76909E+00_JPRB, 0.75696E+00_JPRB, 0.75212E+00_JPRB, 0.75080E+00_JPRB ,&
&  0.74635E+00_JPRB, 0.73590E+00_JPRB, 0.74099E+00_JPRB, 0.74885E+00_JPRB, 0.74928E+00_JPRB, 0.74124E+00_JPRB, 0.73633E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.74213E+00_JPRB, 0.74857E+00_JPRB, 0.73997E+00_JPRB, 0.72106E+00_JPRB, 0.70878E+00_JPRB, 0.69770E+00_JPRB ,&
&  0.69626E+00_JPRB, 0.69631E+00_JPRB, 0.69435E+00_JPRB, 0.69640E+00_JPRB, 0.67710E+00_JPRB, 0.65578E+00_JPRB, 0.63756E+00_JPRB ,&
&  0.62130E+00_JPRB, 0.52647E+00_JPRB, 0.60462E+00_JPRB, 0.75132E+00_JPRB, 0.83230E+00_JPRB, 0.85304E+00_JPRB, 0.83900E+00_JPRB ,&
&  0.83094E+00_JPRB, 0.80290E+00_JPRB, 0.76853E+00_JPRB, 0.73798E+00_JPRB, 0.72314E+00_JPRB, 0.71502E+00_JPRB, 0.71096E+00_JPRB ,&
&  0.70678E+00_JPRB, 0.70143E+00_JPRB, 0.69902E+00_JPRB, 0.68452E+00_JPRB, 0.67296E+00_JPRB, 0.67304E+00_JPRB, 0.67717E+00_JPRB ,&
&  0.68328E+00_JPRB, 0.68161E+00_JPRB, 0.68536E+00_JPRB, 0.69114E+00_JPRB, 0.67853E+00_JPRB, 0.68072E+00_JPRB, 0.68274E+00_JPRB ,&
&  0.68629E+00_JPRB, 0.69812E+00_JPRB, 0.69835E+00_JPRB, 0.72309E+00_JPRB, 0.72959E+00_JPRB, 0.73594E+00_JPRB, 0.73736E+00_JPRB ,&
&  0.73204E+00_JPRB, 0.73267E+00_JPRB, 0.72893E+00_JPRB, 0.72760E+00_JPRB, 0.71995E+00_JPRB, 0.70031E+00_JPRB, 0.68250E+00_JPRB ,&
&  0.68151E+00_JPRB, 0.68530E+00_JPRB, 0.67872E+00_JPRB, 0.68041E+00_JPRB, 0.68400E+00_JPRB, 0.67982E+00_JPRB, 0.67749E+00_JPRB ,&
&  0.67623E+00_JPRB, 0.67926E+00_JPRB, 0.68160E+00_JPRB, 0.69166E+00_JPRB, 0.70839E+00_JPRB, 0.72756E+00_JPRB, 0.74725E+00_JPRB ,&
&  0.76635E+00_JPRB, 0.77956E+00_JPRB, 0.80226E+00_JPRB, 0.85285E+00_JPRB, 0.85515E+00_JPRB, 0.85184E+00_JPRB, 0.86273E+00_JPRB ,&
&  0.85499E+00_JPRB, 0.82953E+00_JPRB, 0.80373E+00_JPRB, 0.76461E+00_JPRB, 0.70788E+00_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.85295E+00_JPRB, 0.84383E+00_JPRB, 0.82930E+00_JPRB, 0.80143E+00_JPRB, 0.78843E+00_JPRB, 0.77380E+00_JPRB, 0.75790E+00_JPRB ,&
&  0.75015E+00_JPRB, 0.74279E+00_JPRB, 0.73746E+00_JPRB, 0.72514E+00_JPRB, 0.71335E+00_JPRB, 0.70682E+00_JPRB, 0.69806E+00_JPRB ,&
&  0.69981E+00_JPRB, 0.70131E+00_JPRB, 0.69474E+00_JPRB, 0.68811E+00_JPRB, 0.68659E+00_JPRB, 0.68534E+00_JPRB, 0.69092E+00_JPRB ,&
&  0.68374E+00_JPRB, 0.67498E+00_JPRB, 0.67489E+00_JPRB, 0.66835E+00_JPRB, 0.65706E+00_JPRB, 0.64684E+00_JPRB, 0.61646E+00_JPRB ,&
&  0.60102E+00_JPRB, 0.57370E+00_JPRB, 0.73491E+00_JPRB, 0.92384E+00_JPRB, 0.92736E+00_JPRB, 0.93684E+00_JPRB, 0.90728E+00_JPRB ,&
&  0.88295E+00_JPRB, 0.83607E+00_JPRB, 0.78433E+00_JPRB, 0.75283E+00_JPRB, 0.74781E+00_JPRB, 0.74070E+00_JPRB, 0.72839E+00_JPRB ,&
&  0.71049E+00_JPRB, 0.69253E+00_JPRB, 0.67701E+00_JPRB, 0.67995E+00_JPRB, 0.67613E+00_JPRB, 0.67982E+00_JPRB, 0.68700E+00_JPRB ,&
&  0.68855E+00_JPRB, 0.69004E+00_JPRB, 0.68329E+00_JPRB, 0.68408E+00_JPRB, 0.68680E+00_JPRB, 0.69844E+00_JPRB, 0.71368E+00_JPRB ,&
&  0.73349E+00_JPRB, 0.73192E+00_JPRB, 0.73810E+00_JPRB, 0.74345E+00_JPRB, 0.75314E+00_JPRB, 0.75313E+00_JPRB, 0.74535E+00_JPRB ,&
&  0.73805E+00_JPRB, 0.73684E+00_JPRB, 0.71602E+00_JPRB, 0.72513E+00_JPRB, 0.71644E+00_JPRB, 0.70354E+00_JPRB, 0.69631E+00_JPRB ,&
&  0.68686E+00_JPRB, 0.69095E+00_JPRB, 0.68484E+00_JPRB, 0.68789E+00_JPRB, 0.69366E+00_JPRB, 0.69472E+00_JPRB, 0.69234E+00_JPRB ,&
&  0.69853E+00_JPRB, 0.69698E+00_JPRB, 0.70223E+00_JPRB, 0.72344E+00_JPRB, 0.74838E+00_JPRB, 0.75965E+00_JPRB, 0.77153E+00_JPRB ,&
&  0.78324E+00_JPRB, 0.80422E+00_JPRB, 0.77782E+00_JPRB, 0.79068E+00_JPRB, 0.88838E+00_JPRB, 0.89452E+00_JPRB, 0.86115E+00_JPRB ,&
&  0.84020E+00_JPRB, 0.82108E+00_JPRB, 0.81372E+00_JPRB, 0.79804E+00_JPRB, 0.82313E+00_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.82626E+00_JPRB, 0.80075E+00_JPRB, 0.78432E+00_JPRB, 0.76353E+00_JPRB, 0.75694E+00_JPRB, 0.74406E+00_JPRB, 0.72497E+00_JPRB ,&
&  0.71367E+00_JPRB, 0.71263E+00_JPRB, 0.70640E+00_JPRB, 0.69158E+00_JPRB, 0.67856E+00_JPRB, 0.67369E+00_JPRB, 0.67327E+00_JPRB ,&
&  0.67189E+00_JPRB, 0.67058E+00_JPRB, 0.66870E+00_JPRB, 0.66282E+00_JPRB, 0.66795E+00_JPRB, 0.67865E+00_JPRB, 0.69026E+00_JPRB ,&
&  0.67632E+00_JPRB, 0.66510E+00_JPRB, 0.65451E+00_JPRB, 0.64376E+00_JPRB, 0.64069E+00_JPRB, 0.62196E+00_JPRB, 0.59589E+00_JPRB ,&
&  0.62739E+00_JPRB, 0.71166E+00_JPRB, 0.93132E+00_JPRB, 0.11263E+01_JPRB, 0.10134E+01_JPRB, 0.91678E+00_JPRB, 0.88128E+00_JPRB ,&
&  0.85961E+00_JPRB, 0.80952E+00_JPRB, 0.77896E+00_JPRB, 0.75208E+00_JPRB, 0.74435E+00_JPRB, 0.73513E+00_JPRB, 0.70478E+00_JPRB ,&
&  0.68119E+00_JPRB, 0.66603E+00_JPRB, 0.65480E+00_JPRB, 0.65208E+00_JPRB, 0.64386E+00_JPRB, 0.65240E+00_JPRB, 0.65726E+00_JPRB ,&
&  0.66156E+00_JPRB, 0.66255E+00_JPRB, 0.66352E+00_JPRB, 0.66336E+00_JPRB, 0.67845E+00_JPRB, 0.72956E+00_JPRB, 0.77495E+00_JPRB ,&
&  0.76716E+00_JPRB, 0.76797E+00_JPRB, 0.76425E+00_JPRB, 0.76692E+00_JPRB, 0.76948E+00_JPRB, 0.74920E+00_JPRB, 0.74026E+00_JPRB ,&
&  0.72784E+00_JPRB, 0.72836E+00_JPRB, 0.70889E+00_JPRB, 0.70859E+00_JPRB, 0.71169E+00_JPRB, 0.70749E+00_JPRB, 0.70863E+00_JPRB ,&
&  0.70578E+00_JPRB, 0.70407E+00_JPRB, 0.70253E+00_JPRB, 0.71209E+00_JPRB, 0.71341E+00_JPRB, 0.70517E+00_JPRB, 0.70082E+00_JPRB ,&
&  0.70300E+00_JPRB, 0.69325E+00_JPRB, 0.70271E+00_JPRB, 0.73474E+00_JPRB, 0.75997E+00_JPRB, 0.76791E+00_JPRB, 0.77890E+00_JPRB ,&
&  0.79156E+00_JPRB, 0.79057E+00_JPRB, 0.77338E+00_JPRB, 0.78647E+00_JPRB, 0.80600E+00_JPRB, 0.88793E+00_JPRB, 0.84198E+00_JPRB ,&
&  0.81668E+00_JPRB, 0.80463E+00_JPRB, 0.79713E+00_JPRB, 0.78071E+00_JPRB, 0.83856E+00_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.78311E+00_JPRB, 0.76619E+00_JPRB, 0.75801E+00_JPRB, 0.75094E+00_JPRB, 0.73430E+00_JPRB, 0.71499E+00_JPRB, 0.71093E+00_JPRB ,&
&  0.70882E+00_JPRB, 0.69121E+00_JPRB, 0.67029E+00_JPRB, 0.64308E+00_JPRB, 0.63603E+00_JPRB, 0.63893E+00_JPRB, 0.64183E+00_JPRB ,&
&  0.63912E+00_JPRB, 0.63925E+00_JPRB, 0.63862E+00_JPRB, 0.63909E+00_JPRB, 0.64057E+00_JPRB, 0.66539E+00_JPRB, 0.67685E+00_JPRB ,&
&  0.67103E+00_JPRB, 0.67421E+00_JPRB, 0.65454E+00_JPRB, 0.63309E+00_JPRB, 0.62296E+00_JPRB, 0.60763E+00_JPRB, 0.59360E+00_JPRB ,&
&  0.69768E+00_JPRB, 0.89284E+00_JPRB, 0.12675E+01_JPRB, 0.12194E+01_JPRB, 0.10193E+01_JPRB, 0.86706E+00_JPRB, 0.82989E+00_JPRB ,&
&  0.82541E+00_JPRB, 0.79676E+00_JPRB, 0.76559E+00_JPRB, 0.73782E+00_JPRB, 0.71325E+00_JPRB, 0.70396E+00_JPRB, 0.68920E+00_JPRB ,&
&  0.68080E+00_JPRB, 0.67353E+00_JPRB, 0.64827E+00_JPRB, 0.63521E+00_JPRB, 0.62840E+00_JPRB, 0.62890E+00_JPRB, 0.62696E+00_JPRB ,&
&  0.63284E+00_JPRB, 0.63977E+00_JPRB, 0.64583E+00_JPRB, 0.63763E+00_JPRB, 0.61533E+00_JPRB, 0.58423E+00_JPRB, 0.70121E+00_JPRB ,&
&  0.91394E+00_JPRB, 0.87602E+00_JPRB, 0.81110E+00_JPRB, 0.78788E+00_JPRB, 0.77560E+00_JPRB, 0.74538E+00_JPRB, 0.74272E+00_JPRB ,&
&  0.73895E+00_JPRB, 0.74588E+00_JPRB, 0.73574E+00_JPRB, 0.74943E+00_JPRB, 0.75315E+00_JPRB, 0.74934E+00_JPRB, 0.74239E+00_JPRB ,&
&  0.72560E+00_JPRB, 0.72401E+00_JPRB, 0.72508E+00_JPRB, 0.72620E+00_JPRB, 0.71580E+00_JPRB, 0.71626E+00_JPRB, 0.71557E+00_JPRB ,&
&  0.71370E+00_JPRB, 0.69647E+00_JPRB, 0.69967E+00_JPRB, 0.72060E+00_JPRB, 0.78024E+00_JPRB, 0.81714E+00_JPRB, 0.79964E+00_JPRB ,&
&  0.75726E+00_JPRB, 0.72978E+00_JPRB, 0.73145E+00_JPRB, 0.73957E+00_JPRB, 0.75939E+00_JPRB, 0.85061E+00_JPRB, 0.79812E+00_JPRB ,&
&  0.77123E+00_JPRB, 0.77178E+00_JPRB, 0.76569E+00_JPRB, 0.75781E+00_JPRB, 0.78304E+00_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.77146E+00_JPRB, 0.76089E+00_JPRB, 0.74915E+00_JPRB, 0.73048E+00_JPRB, 0.70470E+00_JPRB, 0.68940E+00_JPRB, 0.68087E+00_JPRB ,&
&  0.67777E+00_JPRB, 0.65430E+00_JPRB, 0.63559E+00_JPRB, 0.63470E+00_JPRB, 0.63848E+00_JPRB, 0.62625E+00_JPRB, 0.61925E+00_JPRB ,&
&  0.62211E+00_JPRB, 0.62389E+00_JPRB, 0.61603E+00_JPRB, 0.60957E+00_JPRB, 0.61522E+00_JPRB, 0.65128E+00_JPRB, 0.66590E+00_JPRB ,&
&  0.67281E+00_JPRB, 0.66682E+00_JPRB, 0.64675E+00_JPRB, 0.61882E+00_JPRB, 0.61076E+00_JPRB, 0.60991E+00_JPRB, 0.62316E+00_JPRB ,&
&  0.81031E+00_JPRB, 0.91324E+00_JPRB, 0.14381E+01_JPRB, 0.14103E+01_JPRB, 0.10360E+01_JPRB, 0.85308E+00_JPRB, 0.75476E+00_JPRB ,&
&  0.78814E+00_JPRB, 0.76967E+00_JPRB, 0.74881E+00_JPRB, 0.72211E+00_JPRB, 0.69949E+00_JPRB, 0.67888E+00_JPRB, 0.66474E+00_JPRB ,&
&  0.66285E+00_JPRB, 0.64865E+00_JPRB, 0.63440E+00_JPRB, 0.63191E+00_JPRB, 0.62348E+00_JPRB, 0.62185E+00_JPRB, 0.62123E+00_JPRB ,&
&  0.62658E+00_JPRB, 0.62639E+00_JPRB, 0.62331E+00_JPRB, 0.60915E+00_JPRB, 0.58378E+00_JPRB, 0.63327E+00_JPRB, 0.71511E+00_JPRB ,&
&  0.78447E+00_JPRB, 0.94466E+00_JPRB, 0.85292E+00_JPRB, 0.80882E+00_JPRB, 0.77236E+00_JPRB, 0.74550E+00_JPRB, 0.75550E+00_JPRB ,&
&  0.76538E+00_JPRB, 0.78202E+00_JPRB, 0.77906E+00_JPRB, 0.78524E+00_JPRB, 0.78988E+00_JPRB, 0.76759E+00_JPRB, 0.75455E+00_JPRB ,&
&  0.75095E+00_JPRB, 0.74788E+00_JPRB, 0.73917E+00_JPRB, 0.72870E+00_JPRB, 0.71310E+00_JPRB, 0.70547E+00_JPRB, 0.70426E+00_JPRB ,&
&  0.70274E+00_JPRB, 0.69035E+00_JPRB, 0.69939E+00_JPRB, 0.72005E+00_JPRB, 0.75235E+00_JPRB, 0.75762E+00_JPRB, 0.74670E+00_JPRB ,&
&  0.72112E+00_JPRB, 0.70730E+00_JPRB, 0.70000E+00_JPRB, 0.72532E+00_JPRB, 0.72141E+00_JPRB, 0.78630E+00_JPRB, 0.74819E+00_JPRB ,&
&  0.74238E+00_JPRB, 0.74295E+00_JPRB, 0.73099E+00_JPRB, 0.73223E+00_JPRB, 0.75855E+00_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.73586E+00_JPRB, 0.70771E+00_JPRB, 0.68982E+00_JPRB, 0.67458E+00_JPRB, 0.66100E+00_JPRB, 0.65084E+00_JPRB, 0.66229E+00_JPRB ,&
&  0.66285E+00_JPRB, 0.63968E+00_JPRB, 0.62204E+00_JPRB, 0.62405E+00_JPRB, 0.61999E+00_JPRB, 0.61696E+00_JPRB, 0.61366E+00_JPRB ,&
&  0.60468E+00_JPRB, 0.59194E+00_JPRB, 0.58892E+00_JPRB, 0.59701E+00_JPRB, 0.62768E+00_JPRB, 0.66492E+00_JPRB, 0.67181E+00_JPRB ,&
&  0.66535E+00_JPRB, 0.66023E+00_JPRB, 0.64106E+00_JPRB, 0.62247E+00_JPRB, 0.61633E+00_JPRB, 0.62178E+00_JPRB, 0.64972E+00_JPRB ,&
&  0.92316E+00_JPRB, 0.21015E+01_JPRB, 0.10797E+01_JPRB, 0.14398E+01_JPRB, 0.10231E+01_JPRB, 0.80790E+00_JPRB, 0.65311E+00_JPRB ,&
&  0.66277E+00_JPRB, 0.72494E+00_JPRB, 0.72896E+00_JPRB, 0.70991E+00_JPRB, 0.66889E+00_JPRB, 0.64989E+00_JPRB, 0.64569E+00_JPRB ,&
&  0.66707E+00_JPRB, 0.64479E+00_JPRB, 0.63231E+00_JPRB, 0.62806E+00_JPRB, 0.61703E+00_JPRB, 0.61581E+00_JPRB, 0.62464E+00_JPRB ,&
&  0.62224E+00_JPRB, 0.61750E+00_JPRB, 0.61070E+00_JPRB, 0.63225E+00_JPRB, 0.59863E+00_JPRB, 0.69353E+00_JPRB, 0.79173E+00_JPRB ,&
&  0.76114E+00_JPRB, 0.97919E+00_JPRB, 0.87079E+00_JPRB, 0.81257E+00_JPRB, 0.78043E+00_JPRB, 0.77241E+00_JPRB, 0.78561E+00_JPRB ,&
&  0.77746E+00_JPRB, 0.76898E+00_JPRB, 0.76059E+00_JPRB, 0.76459E+00_JPRB, 0.77895E+00_JPRB, 0.75979E+00_JPRB, 0.75935E+00_JPRB ,&
&  0.77058E+00_JPRB, 0.74176E+00_JPRB, 0.73298E+00_JPRB, 0.72439E+00_JPRB, 0.73153E+00_JPRB, 0.73444E+00_JPRB, 0.72672E+00_JPRB ,&
&  0.71587E+00_JPRB, 0.70588E+00_JPRB, 0.70392E+00_JPRB, 0.70862E+00_JPRB, 0.71212E+00_JPRB, 0.70881E+00_JPRB, 0.68699E+00_JPRB ,&
&  0.69708E+00_JPRB, 0.69931E+00_JPRB, 0.69430E+00_JPRB, 0.71251E+00_JPRB, 0.71527E+00_JPRB, 0.74976E+00_JPRB, 0.72001E+00_JPRB ,&
&  0.73220E+00_JPRB, 0.73218E+00_JPRB, 0.71518E+00_JPRB, 0.72786E+00_JPRB, 0.75436E+00_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.69559E+00_JPRB, 0.67327E+00_JPRB, 0.65225E+00_JPRB, 0.62849E+00_JPRB, 0.61317E+00_JPRB, 0.62156E+00_JPRB, 0.64492E+00_JPRB ,&
&  0.63023E+00_JPRB, 0.60697E+00_JPRB, 0.60118E+00_JPRB, 0.61613E+00_JPRB, 0.63332E+00_JPRB, 0.62626E+00_JPRB, 0.61731E+00_JPRB ,&
&  0.59917E+00_JPRB, 0.59266E+00_JPRB, 0.58218E+00_JPRB, 0.59224E+00_JPRB, 0.64269E+00_JPRB, 0.63883E+00_JPRB, 0.62877E+00_JPRB ,&
&  0.64637E+00_JPRB, 0.64307E+00_JPRB, 0.62176E+00_JPRB, 0.61835E+00_JPRB, 0.62091E+00_JPRB, 0.64401E+00_JPRB, 0.68891E+00_JPRB ,&
&  0.10103E+01_JPRB, 0.22005E+01_JPRB, 0.12750E+01_JPRB, 0.14702E+01_JPRB, 0.98404E+00_JPRB, 0.74506E+00_JPRB, 0.57565E+00_JPRB ,&
&  0.57105E+00_JPRB, 0.69996E+00_JPRB, 0.71085E+00_JPRB, 0.69042E+00_JPRB, 0.63405E+00_JPRB, 0.61256E+00_JPRB, 0.63674E+00_JPRB ,&
&  0.65078E+00_JPRB, 0.63314E+00_JPRB, 0.62898E+00_JPRB, 0.63999E+00_JPRB, 0.64146E+00_JPRB, 0.63689E+00_JPRB, 0.64135E+00_JPRB ,&
&  0.63958E+00_JPRB, 0.64256E+00_JPRB, 0.64196E+00_JPRB, 0.62431E+00_JPRB, 0.65355E+00_JPRB, 0.75411E+00_JPRB, 0.84264E+00_JPRB ,&
&  0.81666E+00_JPRB, 0.94872E+00_JPRB, 0.85931E+00_JPRB, 0.81558E+00_JPRB, 0.77593E+00_JPRB, 0.77607E+00_JPRB, 0.76954E+00_JPRB ,&
&  0.74745E+00_JPRB, 0.73659E+00_JPRB, 0.71559E+00_JPRB, 0.71640E+00_JPRB, 0.75543E+00_JPRB, 0.76940E+00_JPRB, 0.75433E+00_JPRB ,&
&  0.77243E+00_JPRB, 0.77447E+00_JPRB, 0.77545E+00_JPRB, 0.75478E+00_JPRB, 0.75868E+00_JPRB, 0.75338E+00_JPRB, 0.74130E+00_JPRB ,&
&  0.73161E+00_JPRB, 0.73043E+00_JPRB, 0.71110E+00_JPRB, 0.70682E+00_JPRB, 0.70424E+00_JPRB, 0.69801E+00_JPRB, 0.67172E+00_JPRB ,&
&  0.68420E+00_JPRB, 0.69541E+00_JPRB, 0.70574E+00_JPRB, 0.69862E+00_JPRB, 0.69453E+00_JPRB, 0.72273E+00_JPRB, 0.73127E+00_JPRB ,&
&  0.75044E+00_JPRB, 0.75157E+00_JPRB, 0.71809E+00_JPRB, 0.71895E+00_JPRB, 0.72814E+00_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.66810E+00_JPRB, 0.64176E+00_JPRB, 0.61687E+00_JPRB, 0.57438E+00_JPRB, 0.58345E+00_JPRB, 0.59638E+00_JPRB, 0.57908E+00_JPRB ,&
&  0.53770E+00_JPRB, 0.53310E+00_JPRB, 0.54752E+00_JPRB, 0.57069E+00_JPRB, 0.58951E+00_JPRB, 0.58271E+00_JPRB, 0.58823E+00_JPRB ,&
&  0.57733E+00_JPRB, 0.57380E+00_JPRB, 0.59325E+00_JPRB, 0.62523E+00_JPRB, 0.62764E+00_JPRB, 0.62449E+00_JPRB, 0.63485E+00_JPRB ,&
&  0.63962E+00_JPRB, 0.62678E+00_JPRB, 0.61692E+00_JPRB, 0.61909E+00_JPRB, 0.64564E+00_JPRB, 0.68983E+00_JPRB, 0.75837E+00_JPRB ,&
&  0.10802E+01_JPRB, 0.23733E+01_JPRB, 0.18551E+01_JPRB, 0.13460E+01_JPRB, 0.90316E+00_JPRB, 0.66111E+00_JPRB, 0.53656E+00_JPRB ,&
&  0.50085E+00_JPRB, 0.52558E+00_JPRB, 0.69925E+00_JPRB, 0.66516E+00_JPRB, 0.61186E+00_JPRB, 0.58483E+00_JPRB, 0.63216E+00_JPRB ,&
&  0.66482E+00_JPRB, 0.66073E+00_JPRB, 0.66757E+00_JPRB, 0.67430E+00_JPRB, 0.67841E+00_JPRB, 0.67471E+00_JPRB, 0.66501E+00_JPRB ,&
&  0.66158E+00_JPRB, 0.68116E+00_JPRB, 0.69507E+00_JPRB, 0.67819E+00_JPRB, 0.68928E+00_JPRB, 0.77276E+00_JPRB, 0.86072E+00_JPRB ,&
&  0.84830E+00_JPRB, 0.87162E+00_JPRB, 0.83645E+00_JPRB, 0.83311E+00_JPRB, 0.74155E+00_JPRB, 0.77835E+00_JPRB, 0.74820E+00_JPRB ,&
&  0.72441E+00_JPRB, 0.69490E+00_JPRB, 0.69126E+00_JPRB, 0.70467E+00_JPRB, 0.72180E+00_JPRB, 0.73782E+00_JPRB, 0.74202E+00_JPRB ,&
&  0.75444E+00_JPRB, 0.76539E+00_JPRB, 0.76753E+00_JPRB, 0.75322E+00_JPRB, 0.75077E+00_JPRB, 0.75317E+00_JPRB, 0.76430E+00_JPRB ,&
&  0.76267E+00_JPRB, 0.76988E+00_JPRB, 0.73888E+00_JPRB, 0.73006E+00_JPRB, 0.72490E+00_JPRB, 0.71222E+00_JPRB, 0.67506E+00_JPRB ,&
&  0.67940E+00_JPRB, 0.70996E+00_JPRB, 0.71700E+00_JPRB, 0.70680E+00_JPRB, 0.70631E+00_JPRB, 0.72224E+00_JPRB, 0.72934E+00_JPRB ,&
&  0.75374E+00_JPRB, 0.75978E+00_JPRB, 0.72149E+00_JPRB, 0.70015E+00_JPRB, 0.70781E+00_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.66385E+00_JPRB, 0.64074E+00_JPRB, 0.58979E+00_JPRB, 0.56547E+00_JPRB, 0.56363E+00_JPRB, 0.55772E+00_JPRB, 0.54525E+00_JPRB ,&
&  0.52313E+00_JPRB, 0.51687E+00_JPRB, 0.54649E+00_JPRB, 0.57506E+00_JPRB, 0.60092E+00_JPRB, 0.59755E+00_JPRB, 0.59583E+00_JPRB ,&
&  0.59638E+00_JPRB, 0.61626E+00_JPRB, 0.65505E+00_JPRB, 0.66135E+00_JPRB, 0.64538E+00_JPRB, 0.63591E+00_JPRB, 0.63262E+00_JPRB ,&
&  0.63437E+00_JPRB, 0.63597E+00_JPRB, 0.65120E+00_JPRB, 0.67060E+00_JPRB, 0.70695E+00_JPRB, 0.76982E+00_JPRB, 0.90431E+00_JPRB ,&
&  0.12325E+01_JPRB, 0.20548E+01_JPRB, 0.62462E+00_JPRB, 0.76396E+00_JPRB, 0.69338E+00_JPRB, 0.58668E+00_JPRB, 0.51501E+00_JPRB ,&
&  0.46475E+00_JPRB, 0.47520E+00_JPRB, 0.59345E+00_JPRB, 0.64478E+00_JPRB, 0.58768E+00_JPRB, 0.62149E+00_JPRB, 0.68707E+00_JPRB ,&
&  0.73599E+00_JPRB, 0.73404E+00_JPRB, 0.72547E+00_JPRB, 0.71489E+00_JPRB, 0.71344E+00_JPRB, 0.70721E+00_JPRB, 0.70622E+00_JPRB ,&
&  0.71705E+00_JPRB, 0.74803E+00_JPRB, 0.76782E+00_JPRB, 0.69691E+00_JPRB, 0.77251E+00_JPRB, 0.82676E+00_JPRB, 0.82001E+00_JPRB ,&
&  0.74518E+00_JPRB, 0.79903E+00_JPRB, 0.83323E+00_JPRB, 0.86222E+00_JPRB, 0.72822E+00_JPRB, 0.65059E+00_JPRB, 0.73333E+00_JPRB ,&
&  0.72713E+00_JPRB, 0.70628E+00_JPRB, 0.68608E+00_JPRB, 0.67295E+00_JPRB, 0.66826E+00_JPRB, 0.70689E+00_JPRB, 0.73091E+00_JPRB ,&
&  0.72640E+00_JPRB, 0.74949E+00_JPRB, 0.75391E+00_JPRB, 0.74200E+00_JPRB, 0.74677E+00_JPRB, 0.75746E+00_JPRB, 0.78844E+00_JPRB ,&
&  0.79853E+00_JPRB, 0.80938E+00_JPRB, 0.81541E+00_JPRB, 0.79569E+00_JPRB, 0.79521E+00_JPRB, 0.75848E+00_JPRB, 0.71154E+00_JPRB ,&
&  0.70809E+00_JPRB, 0.72207E+00_JPRB, 0.70387E+00_JPRB, 0.67691E+00_JPRB, 0.70562E+00_JPRB, 0.71107E+00_JPRB, 0.71730E+00_JPRB ,&
&  0.77054E+00_JPRB, 0.80221E+00_JPRB, 0.79703E+00_JPRB, 0.68464E+00_JPRB, 0.69880E+00_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.65715E+00_JPRB, 0.60876E+00_JPRB, 0.57473E+00_JPRB, 0.56952E+00_JPRB, 0.55811E+00_JPRB, 0.56365E+00_JPRB, 0.57132E+00_JPRB ,&
&  0.56757E+00_JPRB, 0.57915E+00_JPRB, 0.58197E+00_JPRB, 0.59252E+00_JPRB, 0.61140E+00_JPRB, 0.61316E+00_JPRB, 0.63031E+00_JPRB ,&
&  0.64985E+00_JPRB, 0.67531E+00_JPRB, 0.69432E+00_JPRB, 0.65447E+00_JPRB, 0.64134E+00_JPRB, 0.63621E+00_JPRB, 0.63891E+00_JPRB ,&
&  0.65487E+00_JPRB, 0.68038E+00_JPRB, 0.71028E+00_JPRB, 0.75078E+00_JPRB, 0.81157E+00_JPRB, 0.92304E+00_JPRB, 0.11313E+01_JPRB ,&
&  0.16637E+01_JPRB, 0.12758E+01_JPRB, 0.48335E+00_JPRB, 0.59391E+00_JPRB, 0.57450E+00_JPRB, 0.54347E+00_JPRB, 0.46419E+00_JPRB ,&
&  0.44416E+00_JPRB, 0.49034E+00_JPRB, 0.55857E+00_JPRB, 0.62389E+00_JPRB, 0.65658E+00_JPRB, 0.69323E+00_JPRB, 0.73961E+00_JPRB ,&
&  0.76296E+00_JPRB, 0.76846E+00_JPRB, 0.76043E+00_JPRB, 0.75908E+00_JPRB, 0.76012E+00_JPRB, 0.76349E+00_JPRB, 0.77018E+00_JPRB ,&
&  0.78865E+00_JPRB, 0.81874E+00_JPRB, 0.84682E+00_JPRB, 0.75321E+00_JPRB, 0.83098E+00_JPRB, 0.82240E+00_JPRB, 0.79243E+00_JPRB ,&
&  0.72656E+00_JPRB, 0.76705E+00_JPRB, 0.80296E+00_JPRB, 0.81568E+00_JPRB, 0.66006E+00_JPRB, 0.59226E+00_JPRB, 0.70182E+00_JPRB ,&
&  0.75520E+00_JPRB, 0.71027E+00_JPRB, 0.69005E+00_JPRB, 0.67881E+00_JPRB, 0.63880E+00_JPRB, 0.67624E+00_JPRB, 0.72891E+00_JPRB ,&
&  0.72647E+00_JPRB, 0.72916E+00_JPRB, 0.75253E+00_JPRB, 0.74179E+00_JPRB, 0.73794E+00_JPRB, 0.75033E+00_JPRB, 0.78238E+00_JPRB ,&
&  0.80231E+00_JPRB, 0.82113E+00_JPRB, 0.82419E+00_JPRB, 0.85303E+00_JPRB, 0.87862E+00_JPRB, 0.81371E+00_JPRB, 0.72945E+00_JPRB ,&
&  0.70842E+00_JPRB, 0.71657E+00_JPRB, 0.70313E+00_JPRB, 0.69702E+00_JPRB, 0.69368E+00_JPRB, 0.69454E+00_JPRB, 0.71545E+00_JPRB ,&
&  0.81132E+00_JPRB, 0.76298E+00_JPRB, 0.71599E+00_JPRB, 0.65248E+00_JPRB, 0.67522E+00_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.59676E+00_JPRB, 0.57088E+00_JPRB, 0.56231E+00_JPRB, 0.54779E+00_JPRB, 0.56118E+00_JPRB, 0.58523E+00_JPRB, 0.60368E+00_JPRB ,&
&  0.60566E+00_JPRB, 0.60356E+00_JPRB, 0.60391E+00_JPRB, 0.61975E+00_JPRB, 0.64226E+00_JPRB, 0.66508E+00_JPRB, 0.69414E+00_JPRB ,&
&  0.71163E+00_JPRB, 0.72086E+00_JPRB, 0.68997E+00_JPRB, 0.64666E+00_JPRB, 0.64591E+00_JPRB, 0.65353E+00_JPRB, 0.66951E+00_JPRB ,&
&  0.69569E+00_JPRB, 0.72452E+00_JPRB, 0.76384E+00_JPRB, 0.83832E+00_JPRB, 0.95463E+00_JPRB, 0.10901E+01_JPRB, 0.12243E+01_JPRB ,&
&  0.92987E+00_JPRB, 0.44830E+00_JPRB, 0.45370E+00_JPRB, 0.51543E+00_JPRB, 0.47101E+00_JPRB, 0.47094E+00_JPRB, 0.46618E+00_JPRB ,&
&  0.45681E+00_JPRB, 0.50283E+00_JPRB, 0.54544E+00_JPRB, 0.66100E+00_JPRB, 0.73146E+00_JPRB, 0.75145E+00_JPRB, 0.77354E+00_JPRB ,&
&  0.79005E+00_JPRB, 0.80012E+00_JPRB, 0.79720E+00_JPRB, 0.79332E+00_JPRB, 0.79783E+00_JPRB, 0.80671E+00_JPRB, 0.81873E+00_JPRB ,&
&  0.84418E+00_JPRB, 0.87414E+00_JPRB, 0.92278E+00_JPRB, 0.78308E+00_JPRB, 0.82107E+00_JPRB, 0.82517E+00_JPRB, 0.79538E+00_JPRB ,&
&  0.71751E+00_JPRB, 0.71693E+00_JPRB, 0.70547E+00_JPRB, 0.78583E+00_JPRB, 0.72552E+00_JPRB, 0.60632E+00_JPRB, 0.72324E+00_JPRB ,&
&  0.76756E+00_JPRB, 0.77832E+00_JPRB, 0.76760E+00_JPRB, 0.69480E+00_JPRB, 0.62677E+00_JPRB, 0.61558E+00_JPRB, 0.66472E+00_JPRB ,&
&  0.68376E+00_JPRB, 0.69662E+00_JPRB, 0.70061E+00_JPRB, 0.70268E+00_JPRB, 0.68641E+00_JPRB, 0.69612E+00_JPRB, 0.72754E+00_JPRB ,&
&  0.76069E+00_JPRB, 0.79253E+00_JPRB, 0.80586E+00_JPRB, 0.83141E+00_JPRB, 0.86750E+00_JPRB, 0.81557E+00_JPRB, 0.73685E+00_JPRB ,&
&  0.71656E+00_JPRB, 0.70956E+00_JPRB, 0.68576E+00_JPRB, 0.68119E+00_JPRB, 0.69406E+00_JPRB, 0.75263E+00_JPRB, 0.87067E+00_JPRB ,&
&  0.86563E+00_JPRB, 0.76633E+00_JPRB, 0.65943E+00_JPRB, 0.62951E+00_JPRB, 0.63793E+00_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.50221E+00_JPRB, 0.49438E+00_JPRB, 0.49497E+00_JPRB, 0.49301E+00_JPRB, 0.52454E+00_JPRB, 0.55869E+00_JPRB, 0.56831E+00_JPRB ,&
&  0.56137E+00_JPRB, 0.57681E+00_JPRB, 0.59180E+00_JPRB, 0.61981E+00_JPRB, 0.67868E+00_JPRB, 0.70089E+00_JPRB, 0.72751E+00_JPRB ,&
&  0.75382E+00_JPRB, 0.74464E+00_JPRB, 0.70655E+00_JPRB, 0.67994E+00_JPRB, 0.68812E+00_JPRB, 0.68900E+00_JPRB, 0.70434E+00_JPRB ,&
&  0.73296E+00_JPRB, 0.76413E+00_JPRB, 0.80247E+00_JPRB, 0.85380E+00_JPRB, 0.91319E+00_JPRB, 0.95085E+00_JPRB, 0.90478E+00_JPRB ,&
&  0.39439E+00_JPRB, 0.38472E+00_JPRB, 0.49133E+00_JPRB, 0.49439E+00_JPRB, 0.45556E+00_JPRB, 0.45689E+00_JPRB, 0.46871E+00_JPRB ,&
&  0.48496E+00_JPRB, 0.48519E+00_JPRB, 0.54167E+00_JPRB, 0.73047E+00_JPRB, 0.74193E+00_JPRB, 0.78207E+00_JPRB, 0.77802E+00_JPRB ,&
&  0.79165E+00_JPRB, 0.80584E+00_JPRB, 0.81156E+00_JPRB, 0.81205E+00_JPRB, 0.81885E+00_JPRB, 0.82901E+00_JPRB, 0.84399E+00_JPRB ,&
&  0.86406E+00_JPRB, 0.92069E+00_JPRB, 0.98086E+00_JPRB, 0.81726E+00_JPRB, 0.79147E+00_JPRB, 0.81094E+00_JPRB, 0.80103E+00_JPRB ,&
&  0.65309E+00_JPRB, 0.66657E+00_JPRB, 0.66930E+00_JPRB, 0.73702E+00_JPRB, 0.74769E+00_JPRB, 0.78795E+00_JPRB, 0.80320E+00_JPRB ,&
&  0.78087E+00_JPRB, 0.76013E+00_JPRB, 0.72482E+00_JPRB, 0.66113E+00_JPRB, 0.59376E+00_JPRB, 0.58663E+00_JPRB, 0.58604E+00_JPRB ,&
&  0.61560E+00_JPRB, 0.62499E+00_JPRB, 0.63358E+00_JPRB, 0.64358E+00_JPRB, 0.60273E+00_JPRB, 0.60315E+00_JPRB, 0.62996E+00_JPRB ,&
&  0.65996E+00_JPRB, 0.68372E+00_JPRB, 0.69150E+00_JPRB, 0.69990E+00_JPRB, 0.71804E+00_JPRB, 0.72888E+00_JPRB, 0.70012E+00_JPRB ,&
&  0.66648E+00_JPRB, 0.64627E+00_JPRB, 0.66422E+00_JPRB, 0.67096E+00_JPRB, 0.84914E+00_JPRB, 0.11602E+01_JPRB, 0.12063E+01_JPRB ,&
&  0.10261E+01_JPRB, 0.78099E+00_JPRB, 0.62046E+00_JPRB, 0.57089E+00_JPRB, 0.55046E+00_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.46110E+00_JPRB, 0.45062E+00_JPRB, 0.45207E+00_JPRB, 0.45200E+00_JPRB, 0.48593E+00_JPRB, 0.50856E+00_JPRB, 0.52872E+00_JPRB ,&
&  0.55825E+00_JPRB, 0.57665E+00_JPRB, 0.60752E+00_JPRB, 0.66234E+00_JPRB, 0.71911E+00_JPRB, 0.76069E+00_JPRB, 0.77417E+00_JPRB ,&
&  0.77698E+00_JPRB, 0.76668E+00_JPRB, 0.74287E+00_JPRB, 0.71289E+00_JPRB, 0.71134E+00_JPRB, 0.70048E+00_JPRB, 0.71024E+00_JPRB ,&
&  0.73303E+00_JPRB, 0.75679E+00_JPRB, 0.78818E+00_JPRB, 0.81668E+00_JPRB, 0.84017E+00_JPRB, 0.84249E+00_JPRB, 0.78103E+00_JPRB ,&
&  0.39157E+00_JPRB, 0.56351E+00_JPRB, 0.59444E+00_JPRB, 0.57762E+00_JPRB, 0.52875E+00_JPRB, 0.49049E+00_JPRB, 0.48844E+00_JPRB ,&
&  0.50648E+00_JPRB, 0.50340E+00_JPRB, 0.54659E+00_JPRB, 0.62260E+00_JPRB, 0.71218E+00_JPRB, 0.74622E+00_JPRB, 0.75848E+00_JPRB ,&
&  0.77110E+00_JPRB, 0.77599E+00_JPRB, 0.79049E+00_JPRB, 0.80534E+00_JPRB, 0.82181E+00_JPRB, 0.83408E+00_JPRB, 0.85363E+00_JPRB ,&
&  0.87607E+00_JPRB, 0.92052E+00_JPRB, 0.99287E+00_JPRB, 0.89113E+00_JPRB, 0.78622E+00_JPRB, 0.85888E+00_JPRB, 0.81520E+00_JPRB ,&
&  0.60066E+00_JPRB, 0.55593E+00_JPRB, 0.62266E+00_JPRB, 0.75776E+00_JPRB, 0.79616E+00_JPRB, 0.81443E+00_JPRB, 0.80720E+00_JPRB ,&
&  0.74414E+00_JPRB, 0.70703E+00_JPRB, 0.65440E+00_JPRB, 0.61336E+00_JPRB, 0.57538E+00_JPRB, 0.55288E+00_JPRB, 0.53811E+00_JPRB ,&
&  0.53449E+00_JPRB, 0.55083E+00_JPRB, 0.56467E+00_JPRB, 0.55633E+00_JPRB, 0.52399E+00_JPRB, 0.52118E+00_JPRB, 0.55277E+00_JPRB ,&
&  0.61290E+00_JPRB, 0.54627E+00_JPRB, 0.50556E+00_JPRB, 0.50585E+00_JPRB, 0.50847E+00_JPRB, 0.51115E+00_JPRB, 0.50907E+00_JPRB ,&
&  0.52414E+00_JPRB, 0.52864E+00_JPRB, 0.63770E+00_JPRB, 0.10139E+01_JPRB, 0.14192E+01_JPRB, 0.16747E+01_JPRB, 0.13765E+01_JPRB ,&
&  0.96343E+00_JPRB, 0.70342E+00_JPRB, 0.55705E+00_JPRB, 0.51836E+00_JPRB, 0.49747E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.48320E+00_JPRB, 0.46650E+00_JPRB, 0.45123E+00_JPRB, 0.46367E+00_JPRB, 0.47199E+00_JPRB, 0.47697E+00_JPRB, 0.50175E+00_JPRB ,&
&  0.55392E+00_JPRB, 0.56694E+00_JPRB, 0.59332E+00_JPRB, 0.63462E+00_JPRB, 0.69405E+00_JPRB, 0.73617E+00_JPRB, 0.77414E+00_JPRB ,&
&  0.81206E+00_JPRB, 0.82926E+00_JPRB, 0.81021E+00_JPRB, 0.78452E+00_JPRB, 0.77044E+00_JPRB, 0.76382E+00_JPRB, 0.76469E+00_JPRB ,&
&  0.75646E+00_JPRB, 0.75808E+00_JPRB, 0.77158E+00_JPRB, 0.78891E+00_JPRB, 0.79704E+00_JPRB, 0.76363E+00_JPRB, 0.47077E+00_JPRB ,&
&  0.64002E+00_JPRB, 0.68622E+00_JPRB, 0.70561E+00_JPRB, 0.67401E+00_JPRB, 0.61812E+00_JPRB, 0.55608E+00_JPRB, 0.52721E+00_JPRB ,&
&  0.52817E+00_JPRB, 0.53844E+00_JPRB, 0.54565E+00_JPRB, 0.57131E+00_JPRB, 0.66012E+00_JPRB, 0.69306E+00_JPRB, 0.71491E+00_JPRB ,&
&  0.72573E+00_JPRB, 0.74626E+00_JPRB, 0.75902E+00_JPRB, 0.77109E+00_JPRB, 0.78826E+00_JPRB, 0.80018E+00_JPRB, 0.80780E+00_JPRB ,&
&  0.81337E+00_JPRB, 0.80785E+00_JPRB, 0.81851E+00_JPRB, 0.77477E+00_JPRB, 0.70372E+00_JPRB, 0.77657E+00_JPRB, 0.65514E+00_JPRB ,&
&  0.48205E+00_JPRB, 0.44218E+00_JPRB, 0.58863E+00_JPRB, 0.80323E+00_JPRB, 0.80366E+00_JPRB, 0.81290E+00_JPRB, 0.76493E+00_JPRB ,&
&  0.69096E+00_JPRB, 0.62569E+00_JPRB, 0.59428E+00_JPRB, 0.57360E+00_JPRB, 0.55148E+00_JPRB, 0.52223E+00_JPRB, 0.49997E+00_JPRB ,&
&  0.47579E+00_JPRB, 0.46017E+00_JPRB, 0.47413E+00_JPRB, 0.46444E+00_JPRB, 0.47096E+00_JPRB, 0.51066E+00_JPRB, 0.62670E+00_JPRB ,&
&  0.94878E+00_JPRB, 0.50665E+00_JPRB, 0.43761E+00_JPRB, 0.42285E+00_JPRB, 0.40613E+00_JPRB, 0.39582E+00_JPRB, 0.39184E+00_JPRB ,&
&  0.40698E+00_JPRB, 0.46889E+00_JPRB, 0.62081E+00_JPRB, 0.13432E+01_JPRB, 0.25017E+01_JPRB, 0.25664E+01_JPRB, 0.17109E+01_JPRB ,&
&  0.10176E+01_JPRB, 0.69125E+00_JPRB, 0.55170E+00_JPRB, 0.51545E+00_JPRB, 0.48932E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.52641E+00_JPRB, 0.49670E+00_JPRB, 0.48887E+00_JPRB, 0.48677E+00_JPRB, 0.47768E+00_JPRB, 0.49589E+00_JPRB, 0.50968E+00_JPRB ,&
&  0.52896E+00_JPRB, 0.55475E+00_JPRB, 0.58661E+00_JPRB, 0.61443E+00_JPRB, 0.65540E+00_JPRB, 0.69410E+00_JPRB, 0.72309E+00_JPRB ,&
&  0.74462E+00_JPRB, 0.75870E+00_JPRB, 0.76508E+00_JPRB, 0.76460E+00_JPRB, 0.75734E+00_JPRB, 0.74956E+00_JPRB, 0.73868E+00_JPRB ,&
&  0.72833E+00_JPRB, 0.73137E+00_JPRB, 0.74958E+00_JPRB, 0.77029E+00_JPRB, 0.77152E+00_JPRB, 0.75454E+00_JPRB, 0.48151E+00_JPRB ,&
&  0.79127E+00_JPRB, 0.81700E+00_JPRB, 0.78580E+00_JPRB, 0.71309E+00_JPRB, 0.65215E+00_JPRB, 0.60987E+00_JPRB, 0.57177E+00_JPRB ,&
&  0.54801E+00_JPRB, 0.54446E+00_JPRB, 0.50365E+00_JPRB, 0.53087E+00_JPRB, 0.59631E+00_JPRB, 0.61464E+00_JPRB, 0.64645E+00_JPRB ,&
&  0.67032E+00_JPRB, 0.69344E+00_JPRB, 0.71201E+00_JPRB, 0.73296E+00_JPRB, 0.74604E+00_JPRB, 0.76173E+00_JPRB, 0.75772E+00_JPRB ,&
&  0.75116E+00_JPRB, 0.73141E+00_JPRB, 0.69791E+00_JPRB, 0.64517E+00_JPRB, 0.63967E+00_JPRB, 0.57675E+00_JPRB, 0.54139E+00_JPRB ,&
&  0.42099E+00_JPRB, 0.44362E+00_JPRB, 0.67079E+00_JPRB, 0.87349E+00_JPRB, 0.81024E+00_JPRB, 0.79403E+00_JPRB, 0.72073E+00_JPRB ,&
&  0.65291E+00_JPRB, 0.60947E+00_JPRB, 0.58881E+00_JPRB, 0.57204E+00_JPRB, 0.53976E+00_JPRB, 0.51562E+00_JPRB, 0.50505E+00_JPRB ,&
&  0.50017E+00_JPRB, 0.45642E+00_JPRB, 0.43698E+00_JPRB, 0.43308E+00_JPRB, 0.41605E+00_JPRB, 0.43554E+00_JPRB, 0.40520E+00_JPRB ,&
&  0.52229E+00_JPRB, 0.49267E+00_JPRB, 0.43801E+00_JPRB, 0.45310E+00_JPRB, 0.42905E+00_JPRB, 0.42715E+00_JPRB, 0.43656E+00_JPRB ,&
&  0.46724E+00_JPRB, 0.42352E+00_JPRB, 0.91573E+00_JPRB, 0.36882E+01_JPRB, 0.36511E+01_JPRB, 0.46416E+01_JPRB, 0.23978E+01_JPRB ,&
&  0.11123E+01_JPRB, 0.66810E+00_JPRB, 0.56924E+00_JPRB, 0.55841E+00_JPRB, 0.53942E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.54077E+00_JPRB, 0.53321E+00_JPRB, 0.51738E+00_JPRB, 0.49076E+00_JPRB, 0.48065E+00_JPRB, 0.49896E+00_JPRB, 0.52350E+00_JPRB ,&
&  0.54083E+00_JPRB, 0.56278E+00_JPRB, 0.58618E+00_JPRB, 0.61148E+00_JPRB, 0.63962E+00_JPRB, 0.66551E+00_JPRB, 0.68547E+00_JPRB ,&
&  0.69886E+00_JPRB, 0.71681E+00_JPRB, 0.73148E+00_JPRB, 0.74204E+00_JPRB, 0.74211E+00_JPRB, 0.74145E+00_JPRB, 0.73713E+00_JPRB ,&
&  0.76394E+00_JPRB, 0.78751E+00_JPRB, 0.81732E+00_JPRB, 0.84481E+00_JPRB, 0.85315E+00_JPRB, 0.81063E+00_JPRB, 0.61241E+00_JPRB ,&
&  0.90387E+00_JPRB, 0.91239E+00_JPRB, 0.82277E+00_JPRB, 0.73226E+00_JPRB, 0.66840E+00_JPRB, 0.62690E+00_JPRB, 0.58754E+00_JPRB ,&
&  0.55921E+00_JPRB, 0.57094E+00_JPRB, 0.54119E+00_JPRB, 0.51470E+00_JPRB, 0.51690E+00_JPRB, 0.53872E+00_JPRB, 0.57126E+00_JPRB ,&
&  0.61701E+00_JPRB, 0.66122E+00_JPRB, 0.70267E+00_JPRB, 0.72994E+00_JPRB, 0.75020E+00_JPRB, 0.76878E+00_JPRB, 0.74391E+00_JPRB ,&
&  0.73661E+00_JPRB, 0.73524E+00_JPRB, 0.67978E+00_JPRB, 0.65754E+00_JPRB, 0.66944E+00_JPRB, 0.62167E+00_JPRB, 0.57478E+00_JPRB ,&
&  0.47814E+00_JPRB, 0.57321E+00_JPRB, 0.78521E+00_JPRB, 0.89592E+00_JPRB, 0.78781E+00_JPRB, 0.76795E+00_JPRB, 0.71375E+00_JPRB ,&
&  0.65011E+00_JPRB, 0.63013E+00_JPRB, 0.62325E+00_JPRB, 0.60871E+00_JPRB, 0.56831E+00_JPRB, 0.54153E+00_JPRB, 0.53887E+00_JPRB ,&
&  0.54398E+00_JPRB, 0.49813E+00_JPRB, 0.46138E+00_JPRB, 0.46324E+00_JPRB, 0.44715E+00_JPRB, 0.38479E+00_JPRB, 0.47535E+00_JPRB ,&
&  0.45361E+00_JPRB, 0.37549E+00_JPRB, 0.45939E+00_JPRB, 0.36256E+00_JPRB, 0.46982E+00_JPRB, 0.47813E+00_JPRB, 0.50683E+00_JPRB ,&
&  0.44057E+00_JPRB, 0.50858E+00_JPRB, 0.86236E+00_JPRB, 0.11892E+01_JPRB, 0.13698E+01_JPRB, 0.13520E+01_JPRB, 0.74540E+00_JPRB ,&
&  0.64585E+00_JPRB, 0.61526E+00_JPRB, 0.59274E+00_JPRB, 0.56976E+00_JPRB, 0.54478E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.55790E+00_JPRB, 0.55840E+00_JPRB, 0.54073E+00_JPRB, 0.51698E+00_JPRB, 0.50577E+00_JPRB, 0.51629E+00_JPRB, 0.53197E+00_JPRB ,&
&  0.54602E+00_JPRB, 0.56516E+00_JPRB, 0.58735E+00_JPRB, 0.60866E+00_JPRB, 0.62744E+00_JPRB, 0.64739E+00_JPRB, 0.67084E+00_JPRB ,&
&  0.69289E+00_JPRB, 0.72028E+00_JPRB, 0.74726E+00_JPRB, 0.77336E+00_JPRB, 0.79482E+00_JPRB, 0.83368E+00_JPRB, 0.85766E+00_JPRB ,&
&  0.89949E+00_JPRB, 0.94133E+00_JPRB, 0.99077E+00_JPRB, 0.10348E+01_JPRB, 0.10861E+01_JPRB, 0.11168E+01_JPRB, 0.90511E+00_JPRB ,&
&  0.98970E+00_JPRB, 0.94970E+00_JPRB, 0.84031E+00_JPRB, 0.74975E+00_JPRB, 0.69806E+00_JPRB, 0.66324E+00_JPRB, 0.63218E+00_JPRB ,&
&  0.62059E+00_JPRB, 0.59228E+00_JPRB, 0.56025E+00_JPRB, 0.54150E+00_JPRB, 0.53214E+00_JPRB, 0.55118E+00_JPRB, 0.54629E+00_JPRB ,&
&  0.58338E+00_JPRB, 0.65070E+00_JPRB, 0.69749E+00_JPRB, 0.76794E+00_JPRB, 0.83288E+00_JPRB, 0.88254E+00_JPRB, 0.88529E+00_JPRB ,&
&  0.88794E+00_JPRB, 0.91652E+00_JPRB, 0.88739E+00_JPRB, 0.89383E+00_JPRB, 0.88698E+00_JPRB, 0.87406E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.71906E+00_JPRB, 0.73154E+00_JPRB, 0.80153E+00_JPRB, 0.87006E+00_JPRB, 0.78740E+00_JPRB, 0.74771E+00_JPRB, 0.72762E+00_JPRB ,&
&  0.66543E+00_JPRB, 0.65429E+00_JPRB, 0.65768E+00_JPRB, 0.63574E+00_JPRB, 0.61997E+00_JPRB, 0.60111E+00_JPRB, 0.59801E+00_JPRB ,&
&  0.60468E+00_JPRB, 0.54696E+00_JPRB, 0.51724E+00_JPRB, 0.50446E+00_JPRB, 0.51128E+00_JPRB, 0.48192E+00_JPRB, 0.48839E+00_JPRB ,&
&  0.50256E+00_JPRB, 0.37272E+00_JPRB, 0.41664E+00_JPRB, 0.53870E+00_JPRB, 0.54592E+00_JPRB, 0.57413E+00_JPRB, 0.61800E+00_JPRB ,&
&  0.66474E+00_JPRB, 0.68606E+00_JPRB, 0.67202E+00_JPRB, 0.68835E+00_JPRB, 0.71487E+00_JPRB, 0.69790E+00_JPRB, 0.63860E+00_JPRB ,&
&  0.64295E+00_JPRB, 0.60677E+00_JPRB, 0.58604E+00_JPRB, 0.57661E+00_JPRB, 0.56847E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.56950E+00_JPRB, 0.57761E+00_JPRB, 0.56840E+00_JPRB, 0.55140E+00_JPRB, 0.53810E+00_JPRB, 0.53414E+00_JPRB, 0.54241E+00_JPRB ,&
&  0.55184E+00_JPRB, 0.57114E+00_JPRB, 0.58607E+00_JPRB, 0.60246E+00_JPRB, 0.61066E+00_JPRB, 0.61702E+00_JPRB, 0.63013E+00_JPRB ,&
&  0.64741E+00_JPRB, 0.66627E+00_JPRB, 0.68904E+00_JPRB, 0.72061E+00_JPRB, 0.74728E+00_JPRB, 0.77969E+00_JPRB, 0.78861E+00_JPRB ,&
&  0.78181E+00_JPRB, 0.79785E+00_JPRB, 0.80456E+00_JPRB, 0.82610E+00_JPRB, 0.97062E+00_JPRB, 0.11117E+01_JPRB, 0.12345E+01_JPRB ,&
&  0.93582E+00_JPRB, 0.94169E+00_JPRB, 0.82933E+00_JPRB, 0.74693E+00_JPRB, 0.74697E+00_JPRB, 0.72843E+00_JPRB, 0.72909E+00_JPRB ,&
&  0.73287E+00_JPRB, 0.72352E+00_JPRB, 0.72088E+00_JPRB, 0.72409E+00_JPRB, 0.73320E+00_JPRB, 0.76037E+00_JPRB, 0.73398E+00_JPRB ,&
&  0.72147E+00_JPRB, 0.75141E+00_JPRB, 0.78729E+00_JPRB, 0.88221E+00_JPRB, 0.10105E+01_JPRB, 0.11123E+01_JPRB, 0.11601E+01_JPRB ,&
&  0.11771E+01_JPRB, 0.12214E+01_JPRB, 0.11580E+01_JPRB, 0.11507E+01_JPRB, 0.11406E+01_JPRB, 0.11256E+01_JPRB, 0.11178E+01_JPRB ,&
&  0.10332E+01_JPRB, 0.97566E+00_JPRB, 0.92281E+00_JPRB, 0.80862E+00_JPRB, 0.77074E+00_JPRB, 0.76404E+00_JPRB, 0.74512E+00_JPRB ,&
&  0.71857E+00_JPRB, 0.71085E+00_JPRB, 0.70917E+00_JPRB, 0.68714E+00_JPRB, 0.68185E+00_JPRB, 0.67368E+00_JPRB, 0.67654E+00_JPRB ,&
&  0.67080E+00_JPRB, 0.61894E+00_JPRB, 0.58648E+00_JPRB, 0.56194E+00_JPRB, 0.53394E+00_JPRB, 0.67100E+00_JPRB, 0.58507E+00_JPRB ,&
&  0.58173E+00_JPRB, 0.49442E+00_JPRB, 0.44828E+00_JPRB, 0.62915E+00_JPRB, 0.63570E+00_JPRB, 0.64358E+00_JPRB, 0.65852E+00_JPRB ,&
&  0.67978E+00_JPRB, 0.69245E+00_JPRB, 0.69949E+00_JPRB, 0.68309E+00_JPRB, 0.67766E+00_JPRB, 0.67275E+00_JPRB, 0.65427E+00_JPRB ,&
&  0.64014E+00_JPRB, 0.62304E+00_JPRB, 0.61882E+00_JPRB, 0.58813E+00_JPRB, 0.57177E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.57006E+00_JPRB, 0.56658E+00_JPRB, 0.57873E+00_JPRB, 0.56645E+00_JPRB, 0.58888E+00_JPRB, 0.58038E+00_JPRB, 0.60756E+00_JPRB ,&
&  0.58400E+00_JPRB, 0.57788E+00_JPRB, 0.57740E+00_JPRB, 0.58680E+00_JPRB, 0.59744E+00_JPRB, 0.61370E+00_JPRB, 0.62566E+00_JPRB ,&
&  0.64680E+00_JPRB, 0.68733E+00_JPRB, 0.74178E+00_JPRB, 0.80247E+00_JPRB, 0.88111E+00_JPRB, 0.95537E+00_JPRB, 0.10143E+01_JPRB ,&
&  0.10953E+01_JPRB, 0.11586E+01_JPRB, 0.11801E+01_JPRB, 0.12691E+01_JPRB, 0.15772E+01_JPRB, 0.19656E+01_JPRB, 0.19632E+01_JPRB ,&
&  0.12501E+01_JPRB, 0.86193E+00_JPRB, 0.79635E+00_JPRB, 0.73691E+00_JPRB, 0.75183E+00_JPRB, 0.81054E+00_JPRB, 0.82375E+00_JPRB ,&
&  0.82958E+00_JPRB, 0.84131E+00_JPRB, 0.86732E+00_JPRB, 0.89381E+00_JPRB, 0.93463E+00_JPRB, 0.99098E+00_JPRB, 0.10075E+01_JPRB ,&
&  0.10012E+01_JPRB, 0.10214E+01_JPRB, 0.10346E+01_JPRB, 0.11137E+01_JPRB, 0.11893E+01_JPRB, 0.12842E+01_JPRB, 0.14078E+01_JPRB ,&
&  0.14729E+01_JPRB, 0.14048E+01_JPRB, 0.11537E+01_JPRB, 0.11392E+01_JPRB, 0.12205E+01_JPRB, 0.12159E+01_JPRB, 0.11463E+01_JPRB ,&
&  0.12379E+01_JPRB, 0.12097E+01_JPRB, 0.85959E+00_JPRB, 0.76869E+00_JPRB, 0.75561E+00_JPRB, 0.78466E+00_JPRB, 0.76708E+00_JPRB ,&
&  0.77501E+00_JPRB, 0.77399E+00_JPRB, 0.77707E+00_JPRB, 0.76349E+00_JPRB, 0.74770E+00_JPRB, 0.73685E+00_JPRB, 0.74381E+00_JPRB ,&
&  0.70128E+00_JPRB, 0.67970E+00_JPRB, 0.66702E+00_JPRB, 0.64336E+00_JPRB, 0.68439E+00_JPRB, 0.62622E+00_JPRB, 0.61920E+00_JPRB ,&
&  0.58399E+00_JPRB, 0.59656E+00_JPRB, 0.54160E+00_JPRB, 0.65839E+00_JPRB, 0.71171E+00_JPRB, 0.69236E+00_JPRB, 0.70845E+00_JPRB ,&
&  0.71732E+00_JPRB, 0.73272E+00_JPRB, 0.72038E+00_JPRB, 0.71673E+00_JPRB, 0.72189E+00_JPRB, 0.71259E+00_JPRB, 0.69870E+00_JPRB ,&
&  0.68458E+00_JPRB, 0.66929E+00_JPRB, 0.64097E+00_JPRB, 0.61349E+00_JPRB, 0.58852E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.61074E+00_JPRB, 0.61075E+00_JPRB, 0.61841E+00_JPRB, 0.63019E+00_JPRB, 0.62910E+00_JPRB, 0.62683E+00_JPRB, 0.62163E+00_JPRB ,&
&  0.59538E+00_JPRB, 0.59240E+00_JPRB, 0.58198E+00_JPRB, 0.58816E+00_JPRB, 0.61077E+00_JPRB, 0.63248E+00_JPRB, 0.67058E+00_JPRB ,&
&  0.70991E+00_JPRB, 0.76192E+00_JPRB, 0.84118E+00_JPRB, 0.10077E+01_JPRB, 0.11883E+01_JPRB, 0.13106E+01_JPRB, 0.14535E+01_JPRB ,&
&  0.14886E+01_JPRB, 0.15135E+01_JPRB, 0.15879E+01_JPRB, 0.16837E+01_JPRB, 0.16688E+01_JPRB, 0.16269E+01_JPRB, 0.15086E+01_JPRB ,&
&  0.10931E+01_JPRB, 0.80896E+00_JPRB, 0.84598E+00_JPRB, 0.80188E+00_JPRB, 0.80658E+00_JPRB, 0.81216E+00_JPRB, 0.82808E+00_JPRB ,&
&  0.86272E+00_JPRB, 0.89176E+00_JPRB, 0.92142E+00_JPRB, 0.94810E+00_JPRB, 0.99407E+00_JPRB, 0.10462E+01_JPRB, 0.10604E+01_JPRB ,&
&  0.10729E+01_JPRB, 0.10944E+01_JPRB, 0.11224E+01_JPRB, 0.10972E+01_JPRB, 0.11195E+01_JPRB, 0.11479E+01_JPRB, 0.11939E+01_JPRB ,&
&  0.12237E+01_JPRB, 0.11636E+01_JPRB, 0.95689E+00_JPRB, 0.97076E+00_JPRB, 0.10988E+01_JPRB, 0.10239E+01_JPRB, 0.10194E+01_JPRB ,&
&  0.11647E+01_JPRB, 0.12654E+01_JPRB, 0.81378E+00_JPRB, 0.68249E+00_JPRB, 0.69890E+00_JPRB, 0.73569E+00_JPRB, 0.78872E+00_JPRB ,&
&  0.78140E+00_JPRB, 0.79106E+00_JPRB, 0.81496E+00_JPRB, 0.82187E+00_JPRB, 0.82622E+00_JPRB, 0.80684E+00_JPRB, 0.78891E+00_JPRB ,&
&  0.72964E+00_JPRB, 0.69827E+00_JPRB, 0.68810E+00_JPRB, 0.67939E+00_JPRB, 0.68230E+00_JPRB, 0.69052E+00_JPRB, 0.66842E+00_JPRB ,&
&  0.64865E+00_JPRB, 0.64134E+00_JPRB, 0.68697E+00_JPRB, 0.67036E+00_JPRB, 0.67280E+00_JPRB, 0.74018E+00_JPRB, 0.72556E+00_JPRB ,&
&  0.74698E+00_JPRB, 0.74992E+00_JPRB, 0.74625E+00_JPRB, 0.73918E+00_JPRB, 0.73393E+00_JPRB, 0.72102E+00_JPRB, 0.70699E+00_JPRB ,&
&  0.69073E+00_JPRB, 0.67145E+00_JPRB, 0.65135E+00_JPRB, 0.63401E+00_JPRB, 0.61964E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.61082E+00_JPRB, 0.61713E+00_JPRB, 0.63037E+00_JPRB, 0.63605E+00_JPRB, 0.63524E+00_JPRB, 0.64311E+00_JPRB, 0.64086E+00_JPRB ,&
&  0.62944E+00_JPRB, 0.61940E+00_JPRB, 0.60656E+00_JPRB, 0.61713E+00_JPRB, 0.63566E+00_JPRB, 0.67316E+00_JPRB, 0.72170E+00_JPRB ,&
&  0.79307E+00_JPRB, 0.90007E+00_JPRB, 0.10392E+01_JPRB, 0.11445E+01_JPRB, 0.12821E+01_JPRB, 0.13349E+01_JPRB, 0.13544E+01_JPRB ,&
&  0.13965E+01_JPRB, 0.14432E+01_JPRB, 0.14621E+01_JPRB, 0.14481E+01_JPRB, 0.13667E+01_JPRB, 0.12914E+01_JPRB, 0.12386E+01_JPRB ,&
&  0.10103E+01_JPRB, 0.82203E+00_JPRB, 0.80163E+00_JPRB, 0.79587E+00_JPRB, 0.78389E+00_JPRB, 0.77066E+00_JPRB, 0.79045E+00_JPRB ,&
&  0.82507E+00_JPRB, 0.86180E+00_JPRB, 0.90837E+00_JPRB, 0.94594E+00_JPRB, 0.98191E+00_JPRB, 0.10061E+01_JPRB, 0.10147E+01_JPRB ,&
&  0.10392E+01_JPRB, 0.10600E+01_JPRB, 0.11037E+01_JPRB, 0.10421E+01_JPRB, 0.10854E+01_JPRB, 0.10530E+01_JPRB, 0.10208E+01_JPRB ,&
&  0.97337E+00_JPRB, 0.88272E+00_JPRB, 0.89244E+00_JPRB, 0.89659E+00_JPRB, 0.90816E+00_JPRB, 0.89411E+00_JPRB, 0.89391E+00_JPRB ,&
&  0.97434E+00_JPRB, 0.10225E+01_JPRB, 0.82054E+00_JPRB, 0.70828E+00_JPRB, 0.71705E+00_JPRB, 0.67227E+00_JPRB, 0.77491E+00_JPRB ,&
&  0.79574E+00_JPRB, 0.80382E+00_JPRB, 0.81731E+00_JPRB, 0.85699E+00_JPRB, 0.89105E+00_JPRB, 0.91644E+00_JPRB, 0.85747E+00_JPRB ,&
&  0.81533E+00_JPRB, 0.78969E+00_JPRB, 0.77555E+00_JPRB, 0.76573E+00_JPRB, 0.77590E+00_JPRB, 0.78868E+00_JPRB, 0.79341E+00_JPRB ,&
&  0.75745E+00_JPRB, 0.73644E+00_JPRB, 0.76485E+00_JPRB, 0.75617E+00_JPRB, 0.73565E+00_JPRB, 0.79075E+00_JPRB, 0.74838E+00_JPRB ,&
&  0.73837E+00_JPRB, 0.73789E+00_JPRB, 0.72557E+00_JPRB, 0.70592E+00_JPRB, 0.69537E+00_JPRB, 0.67696E+00_JPRB, 0.66799E+00_JPRB ,&
&  0.66290E+00_JPRB, 0.64881E+00_JPRB, 0.63700E+00_JPRB, 0.63241E+00_JPRB, 0.61663E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.61227E+00_JPRB, 0.61135E+00_JPRB, 0.62473E+00_JPRB, 0.62794E+00_JPRB, 0.62616E+00_JPRB, 0.63194E+00_JPRB, 0.63703E+00_JPRB ,&
&  0.63461E+00_JPRB, 0.63502E+00_JPRB, 0.63160E+00_JPRB, 0.63120E+00_JPRB, 0.63374E+00_JPRB, 0.65603E+00_JPRB, 0.70146E+00_JPRB ,&
&  0.76190E+00_JPRB, 0.86111E+00_JPRB, 0.99427E+00_JPRB, 0.11176E+01_JPRB, 0.12537E+01_JPRB, 0.13234E+01_JPRB, 0.12935E+01_JPRB ,&
&  0.13379E+01_JPRB, 0.13863E+01_JPRB, 0.13639E+01_JPRB, 0.13236E+01_JPRB, 0.10530E+01_JPRB, 0.10527E+01_JPRB, 0.10649E+01_JPRB ,&
&  0.99366E+00_JPRB, 0.87877E+00_JPRB, 0.82255E+00_JPRB, 0.79242E+00_JPRB, 0.75450E+00_JPRB, 0.74011E+00_JPRB, 0.74228E+00_JPRB ,&
&  0.76071E+00_JPRB, 0.79257E+00_JPRB, 0.83830E+00_JPRB, 0.89935E+00_JPRB, 0.94015E+00_JPRB, 0.95279E+00_JPRB, 0.96194E+00_JPRB ,&
&  0.97106E+00_JPRB, 0.96744E+00_JPRB, 0.96687E+00_JPRB, 0.94961E+00_JPRB, 0.93510E+00_JPRB, 0.90555E+00_JPRB, 0.89070E+00_JPRB ,&
&  0.88229E+00_JPRB, 0.82627E+00_JPRB, 0.82122E+00_JPRB, 0.82297E+00_JPRB, 0.78623E+00_JPRB, 0.75000E+00_JPRB, 0.77127E+00_JPRB ,&
&  0.83591E+00_JPRB, 0.95047E+00_JPRB, 0.81454E+00_JPRB, 0.80647E+00_JPRB, 0.85873E+00_JPRB, 0.77333E+00_JPRB, 0.74063E+00_JPRB ,&
&  0.78261E+00_JPRB, 0.80872E+00_JPRB, 0.82762E+00_JPRB, 0.86320E+00_JPRB, 0.91536E+00_JPRB, 0.98275E+00_JPRB, 0.88960E+00_JPRB ,&
&  0.88658E+00_JPRB, 0.85092E+00_JPRB, 0.85591E+00_JPRB, 0.85225E+00_JPRB, 0.83720E+00_JPRB, 0.80680E+00_JPRB, 0.80607E+00_JPRB ,&
&  0.75668E+00_JPRB, 0.80980E+00_JPRB, 0.90635E+00_JPRB, 0.10290E+01_JPRB, 0.12056E+01_JPRB, 0.82077E+00_JPRB, 0.80884E+00_JPRB ,&
&  0.76514E+00_JPRB, 0.74853E+00_JPRB, 0.73080E+00_JPRB, 0.71128E+00_JPRB, 0.69439E+00_JPRB, 0.67906E+00_JPRB, 0.67096E+00_JPRB ,&
&  0.66254E+00_JPRB, 0.64836E+00_JPRB, 0.63900E+00_JPRB, 0.63502E+00_JPRB, 0.61927E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.62668E+00_JPRB, 0.62406E+00_JPRB, 0.63124E+00_JPRB, 0.64242E+00_JPRB, 0.65259E+00_JPRB, 0.65136E+00_JPRB, 0.64200E+00_JPRB ,&
&  0.62058E+00_JPRB, 0.61384E+00_JPRB, 0.61623E+00_JPRB, 0.62014E+00_JPRB, 0.61004E+00_JPRB, 0.60483E+00_JPRB, 0.62079E+00_JPRB ,&
&  0.66393E+00_JPRB, 0.73820E+00_JPRB, 0.84327E+00_JPRB, 0.97057E+00_JPRB, 0.11374E+01_JPRB, 0.12612E+01_JPRB, 0.12964E+01_JPRB ,&
&  0.13914E+01_JPRB, 0.14369E+01_JPRB, 0.12938E+01_JPRB, 0.92847E+00_JPRB, 0.80346E+00_JPRB, 0.91330E+00_JPRB, 0.88297E+00_JPRB ,&
&  0.84350E+00_JPRB, 0.83151E+00_JPRB, 0.81829E+00_JPRB, 0.79897E+00_JPRB, 0.76812E+00_JPRB, 0.74510E+00_JPRB, 0.72461E+00_JPRB ,&
&  0.71841E+00_JPRB, 0.72695E+00_JPRB, 0.75721E+00_JPRB, 0.81661E+00_JPRB, 0.87566E+00_JPRB, 0.90874E+00_JPRB, 0.92263E+00_JPRB ,&
&  0.92322E+00_JPRB, 0.91132E+00_JPRB, 0.90174E+00_JPRB, 0.88834E+00_JPRB, 0.86314E+00_JPRB, 0.85535E+00_JPRB, 0.84176E+00_JPRB ,&
&  0.82804E+00_JPRB, 0.80830E+00_JPRB, 0.79146E+00_JPRB, 0.77959E+00_JPRB, 0.75914E+00_JPRB, 0.68844E+00_JPRB, 0.69334E+00_JPRB ,&
&  0.74742E+00_JPRB, 0.83468E+00_JPRB, 0.83097E+00_JPRB, 0.90332E+00_JPRB, 0.63642E+00_JPRB, 0.66563E+00_JPRB, 0.71633E+00_JPRB ,&
&  0.72171E+00_JPRB, 0.76439E+00_JPRB, 0.80043E+00_JPRB, 0.84000E+00_JPRB, 0.88673E+00_JPRB, 0.88428E+00_JPRB, 0.96061E+00_JPRB ,&
&  0.94452E+00_JPRB, 0.89664E+00_JPRB, 0.89241E+00_JPRB, 0.91182E+00_JPRB, 0.89800E+00_JPRB, 0.83531E+00_JPRB, 0.80264E+00_JPRB ,&
&  0.75417E+00_JPRB, 0.80690E+00_JPRB, 0.81633E+00_JPRB, 0.86457E+00_JPRB, 0.93166E+00_JPRB, 0.75675E+00_JPRB, 0.76720E+00_JPRB ,&
&  0.76884E+00_JPRB, 0.75561E+00_JPRB, 0.74821E+00_JPRB, 0.72535E+00_JPRB, 0.71029E+00_JPRB, 0.69664E+00_JPRB, 0.68617E+00_JPRB ,&
&  0.68188E+00_JPRB, 0.67333E+00_JPRB, 0.66069E+00_JPRB, 0.64849E+00_JPRB, 0.63621E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.64633E+00_JPRB, 0.64757E+00_JPRB, 0.65735E+00_JPRB, 0.66330E+00_JPRB, 0.67478E+00_JPRB, 0.68574E+00_JPRB, 0.67565E+00_JPRB ,&
&  0.62445E+00_JPRB, 0.58450E+00_JPRB, 0.57733E+00_JPRB, 0.58334E+00_JPRB, 0.58626E+00_JPRB, 0.58243E+00_JPRB, 0.58647E+00_JPRB ,&
&  0.60234E+00_JPRB, 0.63597E+00_JPRB, 0.69278E+00_JPRB, 0.81702E+00_JPRB, 0.99362E+00_JPRB, 0.11728E+01_JPRB, 0.13070E+01_JPRB ,&
&  0.12752E+01_JPRB, 0.96761E+00_JPRB, 0.11816E+01_JPRB, 0.11607E+01_JPRB, 0.98415E+00_JPRB, 0.90571E+00_JPRB, 0.88425E+00_JPRB ,&
&  0.86591E+00_JPRB, 0.86721E+00_JPRB, 0.85869E+00_JPRB, 0.83830E+00_JPRB, 0.78627E+00_JPRB, 0.75927E+00_JPRB, 0.72943E+00_JPRB ,&
&  0.71375E+00_JPRB, 0.70588E+00_JPRB, 0.71357E+00_JPRB, 0.75033E+00_JPRB, 0.79740E+00_JPRB, 0.84954E+00_JPRB, 0.89323E+00_JPRB ,&
&  0.89596E+00_JPRB, 0.87012E+00_JPRB, 0.85757E+00_JPRB, 0.82925E+00_JPRB, 0.83411E+00_JPRB, 0.83625E+00_JPRB, 0.82364E+00_JPRB ,&
&  0.81345E+00_JPRB, 0.79794E+00_JPRB, 0.80640E+00_JPRB, 0.80698E+00_JPRB, 0.79707E+00_JPRB, 0.72612E+00_JPRB, 0.72668E+00_JPRB ,&
&  0.72208E+00_JPRB, 0.74978E+00_JPRB, 0.79010E+00_JPRB, 0.84124E+00_JPRB, 0.66434E+00_JPRB, 0.68141E+00_JPRB, 0.64529E+00_JPRB ,&
&  0.67437E+00_JPRB, 0.69048E+00_JPRB, 0.72929E+00_JPRB, 0.76188E+00_JPRB, 0.81926E+00_JPRB, 0.84477E+00_JPRB, 0.97628E+00_JPRB ,&
&  0.97105E+00_JPRB, 0.95657E+00_JPRB, 0.92926E+00_JPRB, 0.95210E+00_JPRB, 0.89730E+00_JPRB, 0.86052E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.87368E+00_JPRB, 0.85583E+00_JPRB, 0.84287E+00_JPRB, 0.84140E+00_JPRB, 0.88235E+00_JPRB, 0.77880E+00_JPRB, 0.75323E+00_JPRB ,&
&  0.73868E+00_JPRB, 0.74926E+00_JPRB, 0.74698E+00_JPRB, 0.73819E+00_JPRB, 0.72876E+00_JPRB, 0.71284E+00_JPRB, 0.69701E+00_JPRB ,&
&  0.68993E+00_JPRB, 0.68172E+00_JPRB, 0.67361E+00_JPRB, 0.66663E+00_JPRB, 0.65664E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.65939E+00_JPRB, 0.65727E+00_JPRB, 0.66860E+00_JPRB, 0.69024E+00_JPRB, 0.71295E+00_JPRB, 0.76841E+00_JPRB, 0.82231E+00_JPRB ,&
&  0.95378E+00_JPRB, 0.66071E+00_JPRB, 0.57566E+00_JPRB, 0.55075E+00_JPRB, 0.55226E+00_JPRB, 0.55842E+00_JPRB, 0.56661E+00_JPRB ,&
&  0.57604E+00_JPRB, 0.59632E+00_JPRB, 0.62119E+00_JPRB, 0.66136E+00_JPRB, 0.81137E+00_JPRB, 0.10106E+01_JPRB, 0.10909E+01_JPRB ,&
&  0.10154E+01_JPRB, 0.11180E+01_JPRB, 0.13959E+01_JPRB, 0.12062E+01_JPRB, 0.10954E+01_JPRB, 0.10705E+01_JPRB, 0.10114E+01_JPRB ,&
&  0.95487E+00_JPRB, 0.90630E+00_JPRB, 0.87408E+00_JPRB, 0.82936E+00_JPRB, 0.80238E+00_JPRB, 0.77860E+00_JPRB, 0.75657E+00_JPRB ,&
&  0.73110E+00_JPRB, 0.71279E+00_JPRB, 0.70604E+00_JPRB, 0.72318E+00_JPRB, 0.74953E+00_JPRB, 0.78828E+00_JPRB, 0.84845E+00_JPRB ,&
&  0.87930E+00_JPRB, 0.85270E+00_JPRB, 0.82783E+00_JPRB, 0.79507E+00_JPRB, 0.81239E+00_JPRB, 0.82328E+00_JPRB, 0.81979E+00_JPRB ,&
&  0.81287E+00_JPRB, 0.83586E+00_JPRB, 0.85026E+00_JPRB, 0.82111E+00_JPRB, 0.76247E+00_JPRB, 0.80362E+00_JPRB, 0.74617E+00_JPRB ,&
&  0.73609E+00_JPRB, 0.70242E+00_JPRB, 0.77079E+00_JPRB, 0.68041E+00_JPRB, 0.70340E+00_JPRB, 0.71168E+00_JPRB, 0.68318E+00_JPRB ,&
&  0.65087E+00_JPRB, 0.64900E+00_JPRB, 0.65225E+00_JPRB, 0.67181E+00_JPRB, 0.73321E+00_JPRB, 0.76945E+00_JPRB, 0.90419E+00_JPRB ,&
&  0.91984E+00_JPRB, 0.10177E+01_JPRB, 0.98085E+00_JPRB, 0.94296E+00_JPRB, 0.85707E+00_JPRB, 0.80885E+00_JPRB, 0.81160E+00_JPRB ,&
&  0.88143E+00_JPRB, 0.90233E+00_JPRB, 0.89183E+00_JPRB, 0.85584E+00_JPRB, 0.84431E+00_JPRB, 0.80973E+00_JPRB, 0.79475E+00_JPRB ,&
&  0.78048E+00_JPRB, 0.76763E+00_JPRB, 0.75393E+00_JPRB, 0.74924E+00_JPRB, 0.73466E+00_JPRB, 0.71779E+00_JPRB, 0.69346E+00_JPRB ,&
&  0.68087E+00_JPRB, 0.67288E+00_JPRB, 0.66733E+00_JPRB, 0.66630E+00_JPRB, 0.66547E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.65313E+00_JPRB, 0.64499E+00_JPRB, 0.63323E+00_JPRB, 0.63829E+00_JPRB, 0.64499E+00_JPRB, 0.66545E+00_JPRB, 0.70476E+00_JPRB ,&
&  0.79060E+00_JPRB, 0.65400E+00_JPRB, 0.58545E+00_JPRB, 0.53907E+00_JPRB, 0.52474E+00_JPRB, 0.53105E+00_JPRB, 0.53656E+00_JPRB ,&
&  0.54961E+00_JPRB, 0.56236E+00_JPRB, 0.57816E+00_JPRB, 0.57103E+00_JPRB, 0.64753E+00_JPRB, 0.80267E+00_JPRB, 0.78972E+00_JPRB ,&
&  0.81922E+00_JPRB, 0.14150E+01_JPRB, 0.13663E+01_JPRB, 0.12919E+01_JPRB, 0.12091E+01_JPRB, 0.12226E+01_JPRB, 0.10928E+01_JPRB ,&
&  0.99050E+00_JPRB, 0.94559E+00_JPRB, 0.92496E+00_JPRB, 0.87409E+00_JPRB, 0.84054E+00_JPRB, 0.79822E+00_JPRB, 0.77812E+00_JPRB ,&
&  0.75568E+00_JPRB, 0.73880E+00_JPRB, 0.71766E+00_JPRB, 0.71818E+00_JPRB, 0.73313E+00_JPRB, 0.75837E+00_JPRB, 0.80257E+00_JPRB ,&
&  0.84859E+00_JPRB, 0.84923E+00_JPRB, 0.81667E+00_JPRB, 0.81290E+00_JPRB, 0.82207E+00_JPRB, 0.83970E+00_JPRB, 0.84549E+00_JPRB ,&
&  0.82003E+00_JPRB, 0.80455E+00_JPRB, 0.88823E+00_JPRB, 0.90791E+00_JPRB, 0.88857E+00_JPRB, 0.89985E+00_JPRB, 0.79245E+00_JPRB ,&
&  0.79980E+00_JPRB, 0.71115E+00_JPRB, 0.73178E+00_JPRB, 0.64622E+00_JPRB, 0.68716E+00_JPRB, 0.71958E+00_JPRB, 0.70301E+00_JPRB ,&
&  0.65471E+00_JPRB, 0.65673E+00_JPRB, 0.64027E+00_JPRB, 0.64048E+00_JPRB, 0.67587E+00_JPRB, 0.72242E+00_JPRB, 0.78895E+00_JPRB ,&
&  0.86052E+00_JPRB, 0.95640E+00_JPRB, 0.10295E+01_JPRB, 0.87681E+00_JPRB, 0.81598E+00_JPRB, 0.76846E+00_JPRB, 0.76515E+00_JPRB ,&
&  0.85517E+00_JPRB, 0.88239E+00_JPRB, 0.91947E+00_JPRB, 0.91100E+00_JPRB, 0.84692E+00_JPRB, 0.83103E+00_JPRB, 0.83103E+00_JPRB ,&
&  0.82134E+00_JPRB, 0.81337E+00_JPRB, 0.77907E+00_JPRB, 0.74451E+00_JPRB, 0.72077E+00_JPRB, 0.70771E+00_JPRB, 0.67900E+00_JPRB ,&
&  0.66938E+00_JPRB, 0.65851E+00_JPRB, 0.65020E+00_JPRB, 0.65213E+00_JPRB, 0.65222E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.63053E+00_JPRB, 0.62278E+00_JPRB, 0.61983E+00_JPRB, 0.61220E+00_JPRB, 0.59736E+00_JPRB, 0.60717E+00_JPRB, 0.66998E+00_JPRB ,&
&  0.67896E+00_JPRB, 0.60256E+00_JPRB, 0.56257E+00_JPRB, 0.53329E+00_JPRB, 0.51510E+00_JPRB, 0.51001E+00_JPRB, 0.51678E+00_JPRB ,&
&  0.52288E+00_JPRB, 0.53652E+00_JPRB, 0.54076E+00_JPRB, 0.54469E+00_JPRB, 0.58071E+00_JPRB, 0.67952E+00_JPRB, 0.54282E+00_JPRB ,&
&  0.71761E+00_JPRB, 0.12388E+01_JPRB, 0.13936E+01_JPRB, 0.14110E+01_JPRB, 0.13942E+01_JPRB, 0.13728E+01_JPRB, 0.12544E+01_JPRB ,&
&  0.11246E+01_JPRB, 0.10424E+01_JPRB, 0.96924E+00_JPRB, 0.93340E+00_JPRB, 0.90884E+00_JPRB, 0.86067E+00_JPRB, 0.81360E+00_JPRB ,&
&  0.78113E+00_JPRB, 0.75107E+00_JPRB, 0.72446E+00_JPRB, 0.71407E+00_JPRB, 0.71796E+00_JPRB, 0.73656E+00_JPRB, 0.77050E+00_JPRB ,&
&  0.81266E+00_JPRB, 0.84385E+00_JPRB, 0.85013E+00_JPRB, 0.80397E+00_JPRB, 0.82160E+00_JPRB, 0.83314E+00_JPRB, 0.85451E+00_JPRB ,&
&  0.82142E+00_JPRB, 0.80660E+00_JPRB, 0.95352E+00_JPRB, 0.10932E+01_JPRB, 0.11313E+01_JPRB, 0.10498E+01_JPRB, 0.91929E+00_JPRB ,&
&  0.87030E+00_JPRB, 0.79068E+00_JPRB, 0.72152E+00_JPRB, 0.64975E+00_JPRB, 0.67527E+00_JPRB, 0.71606E+00_JPRB, 0.73165E+00_JPRB ,&
&  0.69576E+00_JPRB, 0.66543E+00_JPRB, 0.64585E+00_JPRB, 0.62677E+00_JPRB, 0.64030E+00_JPRB, 0.67015E+00_JPRB, 0.75853E+00_JPRB ,&
&  0.85326E+00_JPRB, 0.97177E+00_JPRB, 0.98077E+00_JPRB, 0.83058E+00_JPRB, 0.77019E+00_JPRB, 0.71653E+00_JPRB, 0.67350E+00_JPRB ,&
&  0.76992E+00_JPRB, 0.79182E+00_JPRB, 0.77022E+00_JPRB, 0.87471E+00_JPRB, 0.80972E+00_JPRB, 0.81612E+00_JPRB, 0.86208E+00_JPRB ,&
&  0.84715E+00_JPRB, 0.82446E+00_JPRB, 0.79614E+00_JPRB, 0.77553E+00_JPRB, 0.73383E+00_JPRB, 0.69059E+00_JPRB, 0.66353E+00_JPRB ,&
&  0.65062E+00_JPRB, 0.64677E+00_JPRB, 0.64012E+00_JPRB, 0.63716E+00_JPRB, 0.63409E+00_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.59621E+00_JPRB, 0.59190E+00_JPRB, 0.58656E+00_JPRB, 0.58209E+00_JPRB, 0.56753E+00_JPRB, 0.55967E+00_JPRB, 0.59271E+00_JPRB ,&
&  0.59346E+00_JPRB, 0.56556E+00_JPRB, 0.52185E+00_JPRB, 0.50898E+00_JPRB, 0.50097E+00_JPRB, 0.49422E+00_JPRB, 0.49795E+00_JPRB ,&
&  0.50299E+00_JPRB, 0.51969E+00_JPRB, 0.54742E+00_JPRB, 0.55045E+00_JPRB, 0.51277E+00_JPRB, 0.46819E+00_JPRB, 0.50995E+00_JPRB ,&
&  0.89700E+00_JPRB, 0.13350E+01_JPRB, 0.14454E+01_JPRB, 0.15585E+01_JPRB, 0.16467E+01_JPRB, 0.17486E+01_JPRB, 0.15657E+01_JPRB ,&
&  0.13756E+01_JPRB, 0.12550E+01_JPRB, 0.11418E+01_JPRB, 0.10747E+01_JPRB, 0.10077E+01_JPRB, 0.93990E+00_JPRB, 0.88971E+00_JPRB ,&
&  0.84044E+00_JPRB, 0.80564E+00_JPRB, 0.76585E+00_JPRB, 0.72259E+00_JPRB, 0.72104E+00_JPRB, 0.73761E+00_JPRB, 0.75472E+00_JPRB ,&
&  0.77800E+00_JPRB, 0.82373E+00_JPRB, 0.83930E+00_JPRB, 0.79986E+00_JPRB, 0.79959E+00_JPRB, 0.81557E+00_JPRB, 0.87441E+00_JPRB ,&
&  0.89841E+00_JPRB, 0.97383E+00_JPRB, 0.10710E+01_JPRB, 0.11728E+01_JPRB, 0.12585E+01_JPRB, 0.12461E+01_JPRB, 0.11183E+01_JPRB ,&
&  0.99187E+00_JPRB, 0.85082E+00_JPRB, 0.71129E+00_JPRB, 0.68510E+00_JPRB, 0.72020E+00_JPRB, 0.75771E+00_JPRB, 0.69503E+00_JPRB ,&
&  0.62758E+00_JPRB, 0.59199E+00_JPRB, 0.58823E+00_JPRB, 0.59566E+00_JPRB, 0.63492E+00_JPRB, 0.65800E+00_JPRB, 0.74773E+00_JPRB ,&
&  0.74680E+00_JPRB, 0.58276E+00_JPRB, 0.53071E+00_JPRB, 0.62062E+00_JPRB, 0.57712E+00_JPRB, 0.54626E+00_JPRB, 0.62325E+00_JPRB ,&
&  0.68441E+00_JPRB, 0.73694E+00_JPRB, 0.72042E+00_JPRB, 0.69586E+00_JPRB, 0.73362E+00_JPRB, 0.76452E+00_JPRB, 0.89815E+00_JPRB ,&
&  0.84320E+00_JPRB, 0.79485E+00_JPRB, 0.78038E+00_JPRB, 0.76873E+00_JPRB, 0.72988E+00_JPRB, 0.67954E+00_JPRB, 0.63898E+00_JPRB ,&
&  0.62132E+00_JPRB, 0.61277E+00_JPRB, 0.60408E+00_JPRB, 0.60035E+00_JPRB, 0.59715E+00_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.55834E+00_JPRB, 0.56073E+00_JPRB, 0.55599E+00_JPRB, 0.55136E+00_JPRB, 0.54113E+00_JPRB, 0.52979E+00_JPRB, 0.53997E+00_JPRB ,&
&  0.53407E+00_JPRB, 0.51748E+00_JPRB, 0.49596E+00_JPRB, 0.48860E+00_JPRB, 0.48403E+00_JPRB, 0.48215E+00_JPRB, 0.48659E+00_JPRB ,&
&  0.49202E+00_JPRB, 0.50920E+00_JPRB, 0.56055E+00_JPRB, 0.57400E+00_JPRB, 0.50573E+00_JPRB, 0.42944E+00_JPRB, 0.52544E+00_JPRB ,&
&  0.96202E+00_JPRB, 0.14961E+01_JPRB, 0.17534E+01_JPRB, 0.19532E+01_JPRB, 0.19726E+01_JPRB, 0.20025E+01_JPRB, 0.18895E+01_JPRB ,&
&  0.17219E+01_JPRB, 0.15703E+01_JPRB, 0.14127E+01_JPRB, 0.12365E+01_JPRB, 0.10894E+01_JPRB, 0.99407E+00_JPRB, 0.92066E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.84461E+00_JPRB, 0.80756E+00_JPRB, 0.76001E+00_JPRB, 0.73496E+00_JPRB, 0.74087E+00_JPRB, 0.74789E+00_JPRB ,&
&  0.75360E+00_JPRB, 0.79724E+00_JPRB, 0.82571E+00_JPRB, 0.82025E+00_JPRB, 0.69106E+00_JPRB, 0.79304E+00_JPRB, 0.84376E+00_JPRB ,&
&  0.92402E+00_JPRB, 0.10131E+01_JPRB, 0.10330E+01_JPRB, 0.12890E+01_JPRB, 0.14240E+01_JPRB, 0.13421E+01_JPRB, 0.12184E+01_JPRB ,&
&  0.11148E+01_JPRB, 0.93964E+00_JPRB, 0.78825E+00_JPRB, 0.78385E+00_JPRB, 0.80892E+00_JPRB, 0.79945E+00_JPRB, 0.60134E+00_JPRB ,&
&  0.56535E+00_JPRB, 0.62295E+00_JPRB, 0.60770E+00_JPRB, 0.55540E+00_JPRB, 0.62401E+00_JPRB, 0.64986E+00_JPRB, 0.57447E+00_JPRB ,&
&  0.38823E+00_JPRB, 0.34888E+00_JPRB, 0.35849E+00_JPRB, 0.37440E+00_JPRB, 0.36217E+00_JPRB, 0.41983E+00_JPRB, 0.63670E+00_JPRB ,&
&  0.61065E+00_JPRB, 0.69436E+00_JPRB, 0.68642E+00_JPRB, 0.66531E+00_JPRB, 0.68144E+00_JPRB, 0.73921E+00_JPRB, 0.98603E+00_JPRB ,&
&  0.86731E+00_JPRB, 0.81810E+00_JPRB, 0.76894E+00_JPRB, 0.73814E+00_JPRB, 0.70305E+00_JPRB, 0.65906E+00_JPRB, 0.62584E+00_JPRB ,&
&  0.60413E+00_JPRB, 0.59369E+00_JPRB, 0.58045E+00_JPRB, 0.57024E+00_JPRB, 0.55930E+00_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.53078E+00_JPRB, 0.52527E+00_JPRB, 0.51999E+00_JPRB, 0.51523E+00_JPRB, 0.50560E+00_JPRB, 0.49739E+00_JPRB, 0.49920E+00_JPRB ,&
&  0.49109E+00_JPRB, 0.47530E+00_JPRB, 0.46195E+00_JPRB, 0.45597E+00_JPRB, 0.46041E+00_JPRB, 0.47209E+00_JPRB, 0.47931E+00_JPRB ,&
&  0.49121E+00_JPRB, 0.52783E+00_JPRB, 0.63950E+00_JPRB, 0.61805E+00_JPRB, 0.59606E+00_JPRB, 0.57031E+00_JPRB, 0.60661E+00_JPRB ,&
&  0.89835E+00_JPRB, 0.13926E+01_JPRB, 0.17845E+01_JPRB, 0.20659E+01_JPRB, 0.23832E+01_JPRB, 0.24762E+01_JPRB, 0.23771E+01_JPRB ,&
&  0.21143E+01_JPRB, 0.18755E+01_JPRB, 0.16297E+01_JPRB, 0.13669E+01_JPRB, 0.11708E+01_JPRB, 0.10447E+01_JPRB, 0.95333E+00_JPRB ,&
&  0.88135E+00_JPRB, 0.85345E+00_JPRB, 0.81929E+00_JPRB, 0.78635E+00_JPRB, 0.75896E+00_JPRB, 0.76094E+00_JPRB, 0.76893E+00_JPRB ,&
&  0.76561E+00_JPRB, 0.77410E+00_JPRB, 0.79522E+00_JPRB, 0.79069E+00_JPRB, 0.69576E+00_JPRB, 0.63189E+00_JPRB, 0.74306E+00_JPRB ,&
&  0.87250E+00_JPRB, 0.10842E+01_JPRB, 0.11858E+01_JPRB, 0.15374E+01_JPRB, 0.15448E+01_JPRB, 0.13884E+01_JPRB, 0.13189E+01_JPRB ,&
&  0.11958E+01_JPRB, 0.11380E+01_JPRB, 0.86369E+00_JPRB, 0.86533E+00_JPRB, 0.91588E+00_JPRB, 0.74101E+00_JPRB, 0.65055E+00_JPRB ,&
&  0.69921E+00_JPRB, 0.66250E+00_JPRB, 0.61619E+00_JPRB, 0.53573E+00_JPRB, 0.62112E+00_JPRB, 0.58235E+00_JPRB, 0.38279E+00_JPRB ,&
&  0.34066E+00_JPRB, 0.32961E+00_JPRB, 0.31958E+00_JPRB, 0.31481E+00_JPRB, 0.30878E+00_JPRB, 0.33736E+00_JPRB, 0.43333E+00_JPRB ,&
&  0.47416E+00_JPRB, 0.57910E+00_JPRB, 0.63512E+00_JPRB, 0.66023E+00_JPRB, 0.68545E+00_JPRB, 0.81311E+00_JPRB, 0.10964E+01_JPRB ,&
&  0.92419E+00_JPRB, 0.90348E+00_JPRB, 0.78548E+00_JPRB, 0.72036E+00_JPRB, 0.67468E+00_JPRB, 0.63896E+00_JPRB, 0.60892E+00_JPRB ,&
&  0.58370E+00_JPRB, 0.56652E+00_JPRB, 0.55269E+00_JPRB, 0.54784E+00_JPRB, 0.53850E+00_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.48991E+00_JPRB, 0.48226E+00_JPRB, 0.47627E+00_JPRB, 0.47119E+00_JPRB, 0.46259E+00_JPRB, 0.45356E+00_JPRB, 0.46057E+00_JPRB ,&
&  0.45916E+00_JPRB, 0.44275E+00_JPRB, 0.43285E+00_JPRB, 0.42985E+00_JPRB, 0.44229E+00_JPRB, 0.46496E+00_JPRB, 0.47592E+00_JPRB ,&
&  0.49150E+00_JPRB, 0.56807E+00_JPRB, 0.67941E+00_JPRB, 0.59329E+00_JPRB, 0.52716E+00_JPRB, 0.49600E+00_JPRB, 0.51079E+00_JPRB ,&
&  0.82048E+00_JPRB, 0.11486E+01_JPRB, 0.14737E+01_JPRB, 0.18784E+01_JPRB, 0.22766E+01_JPRB, 0.23108E+01_JPRB, 0.25174E+01_JPRB ,&
&  0.23644E+01_JPRB, 0.20241E+01_JPRB, 0.17108E+01_JPRB, 0.14255E+01_JPRB, 0.12247E+01_JPRB, 0.10744E+01_JPRB, 0.96338E+00_JPRB ,&
&  0.90214E+00_JPRB, 0.86893E+00_JPRB, 0.84011E+00_JPRB, 0.81373E+00_JPRB, 0.79767E+00_JPRB, 0.79047E+00_JPRB, 0.80282E+00_JPRB ,&
&  0.79578E+00_JPRB, 0.77681E+00_JPRB, 0.77307E+00_JPRB, 0.77352E+00_JPRB, 0.78777E+00_JPRB, 0.78900E+00_JPRB, 0.77885E+00_JPRB ,&
&  0.78578E+00_JPRB, 0.89817E+00_JPRB, 0.13017E+01_JPRB, 0.15930E+01_JPRB, 0.15528E+01_JPRB, 0.14628E+01_JPRB, 0.13388E+01_JPRB ,&
&  0.12449E+01_JPRB, 0.12393E+01_JPRB, 0.10240E+01_JPRB, 0.10362E+01_JPRB, 0.91230E+00_JPRB, 0.65691E+00_JPRB, 0.71549E+00_JPRB ,&
&  0.76214E+00_JPRB, 0.73137E+00_JPRB, 0.62903E+00_JPRB, 0.49741E+00_JPRB, 0.49930E+00_JPRB, 0.40581E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.31723E+00_JPRB, 0.30789E+00_JPRB, 0.29088E+00_JPRB, 0.29053E+00_JPRB, 0.28320E+00_JPRB, 0.30598E+00_JPRB, 0.35798E+00_JPRB ,&
&  0.41550E+00_JPRB, 0.48939E+00_JPRB, 0.56655E+00_JPRB, 0.61248E+00_JPRB, 0.62975E+00_JPRB, 0.66340E+00_JPRB, 0.81190E+00_JPRB ,&
&  0.78303E+00_JPRB, 0.83193E+00_JPRB, 0.74031E+00_JPRB, 0.66950E+00_JPRB, 0.61783E+00_JPRB, 0.58610E+00_JPRB, 0.56273E+00_JPRB ,&
&  0.54394E+00_JPRB, 0.52365E+00_JPRB, 0.50930E+00_JPRB, 0.50299E+00_JPRB, 0.49771E+00_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.44881E+00_JPRB, 0.44393E+00_JPRB, 0.44044E+00_JPRB, 0.43483E+00_JPRB, 0.42871E+00_JPRB, 0.42056E+00_JPRB, 0.42674E+00_JPRB ,&
&  0.43429E+00_JPRB, 0.41501E+00_JPRB, 0.39913E+00_JPRB, 0.40186E+00_JPRB, 0.42101E+00_JPRB, 0.44875E+00_JPRB, 0.48003E+00_JPRB ,&
&  0.51751E+00_JPRB, 0.72641E+00_JPRB, 0.55589E+00_JPRB, 0.44242E+00_JPRB, 0.48444E+00_JPRB, 0.50937E+00_JPRB, 0.48385E+00_JPRB ,&
&  0.71503E+00_JPRB, 0.95275E+00_JPRB, 0.12167E+01_JPRB, 0.17241E+01_JPRB, 0.24910E+01_JPRB, 0.25678E+01_JPRB, 0.26999E+01_JPRB ,&
&  0.26819E+01_JPRB, 0.22553E+01_JPRB, 0.18153E+01_JPRB, 0.14720E+01_JPRB, 0.12402E+01_JPRB, 0.10850E+01_JPRB, 0.99211E+00_JPRB ,&
&  0.93469E+00_JPRB, 0.89551E+00_JPRB, 0.86354E+00_JPRB, 0.84799E+00_JPRB, 0.83242E+00_JPRB, 0.81623E+00_JPRB, 0.82324E+00_JPRB ,&
&  0.81392E+00_JPRB, 0.78952E+00_JPRB, 0.78401E+00_JPRB, 0.79031E+00_JPRB, 0.82851E+00_JPRB, 0.80965E+00_JPRB, 0.11336E+01_JPRB ,&
&  0.10794E+01_JPRB, 0.11929E+01_JPRB, 0.13724E+01_JPRB, 0.17978E+01_JPRB, 0.18075E+01_JPRB, 0.15894E+01_JPRB, 0.15962E+01_JPRB ,&
&  0.12344E+01_JPRB, 0.10529E+01_JPRB, 0.99636E+00_JPRB, 0.94294E+00_JPRB, 0.71667E+00_JPRB, 0.72074E+00_JPRB, 0.95409E+00_JPRB ,&
&  0.79622E+00_JPRB, 0.83770E+00_JPRB, 0.84910E+00_JPRB, 0.77727E+00_JPRB, 0.44378E+00_JPRB, 0.36120E+00_JPRB, 0.45068E+00_JPRB ,&
&  0.46894E+00_JPRB, 0.37011E+00_JPRB, 0.30986E+00_JPRB, 0.34585E+00_JPRB, 0.29401E+00_JPRB, 0.31896E+00_JPRB, 0.36921E+00_JPRB ,&
&  0.37755E+00_JPRB, 0.40625E+00_JPRB, 0.51677E+00_JPRB, 0.56554E+00_JPRB, 0.58539E+00_JPRB, 0.58772E+00_JPRB, 0.62550E+00_JPRB ,&
&  0.65079E+00_JPRB, 0.73316E+00_JPRB, 0.72539E+00_JPRB, 0.64168E+00_JPRB, 0.57855E+00_JPRB, 0.53829E+00_JPRB, 0.51189E+00_JPRB ,&
&  0.49298E+00_JPRB, 0.47101E+00_JPRB, 0.45854E+00_JPRB, 0.45567E+00_JPRB, 0.45325E+00_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.41477E+00_JPRB, 0.41384E+00_JPRB, 0.41305E+00_JPRB, 0.40774E+00_JPRB, 0.40064E+00_JPRB, 0.39571E+00_JPRB, 0.39966E+00_JPRB ,&
&  0.41026E+00_JPRB, 0.38937E+00_JPRB, 0.37589E+00_JPRB, 0.37908E+00_JPRB, 0.39336E+00_JPRB, 0.41530E+00_JPRB, 0.45058E+00_JPRB ,&
&  0.49293E+00_JPRB, 0.53135E+00_JPRB, 0.41128E+00_JPRB, 0.37223E+00_JPRB, 0.49844E+00_JPRB, 0.49207E+00_JPRB, 0.53985E+00_JPRB ,&
&  0.67581E+00_JPRB, 0.81328E+00_JPRB, 0.10434E+01_JPRB, 0.15099E+01_JPRB, 0.19568E+01_JPRB, 0.24384E+01_JPRB, 0.27183E+01_JPRB ,&
&  0.28255E+01_JPRB, 0.25565E+01_JPRB, 0.21334E+01_JPRB, 0.17054E+01_JPRB, 0.13771E+01_JPRB, 0.11652E+01_JPRB, 0.10357E+01_JPRB ,&
&  0.95108E+00_JPRB, 0.88532E+00_JPRB, 0.84515E+00_JPRB, 0.80978E+00_JPRB, 0.80840E+00_JPRB, 0.79911E+00_JPRB, 0.81180E+00_JPRB ,&
&  0.82144E+00_JPRB, 0.80616E+00_JPRB, 0.81212E+00_JPRB, 0.81279E+00_JPRB, 0.80998E+00_JPRB, 0.83928E+00_JPRB, 0.98520E+00_JPRB ,&
&  0.11818E+01_JPRB, 0.13484E+01_JPRB, 0.15379E+01_JPRB, 0.16913E+01_JPRB, 0.17001E+01_JPRB, 0.15154E+01_JPRB, 0.16706E+01_JPRB ,&
&  0.13454E+01_JPRB, 0.10559E+01_JPRB, 0.83827E+00_JPRB, 0.65289E+00_JPRB, 0.67196E+00_JPRB, 0.89279E+00_JPRB, 0.91740E+00_JPRB ,&
&  0.90208E+00_JPRB, 0.91895E+00_JPRB, 0.90144E+00_JPRB, 0.75393E+00_JPRB, 0.59649E+00_JPRB, 0.44655E+00_JPRB, 0.48776E+00_JPRB ,&
&  0.49273E+00_JPRB, 0.48511E+00_JPRB, 0.45412E+00_JPRB, 0.41405E+00_JPRB, 0.35209E+00_JPRB, 0.39140E+00_JPRB, 0.39537E+00_JPRB ,&
&  0.37398E+00_JPRB, 0.37056E+00_JPRB, 0.45177E+00_JPRB, 0.52148E+00_JPRB, 0.52453E+00_JPRB, 0.49040E+00_JPRB, 0.52740E+00_JPRB ,&
&  0.53230E+00_JPRB, 0.56001E+00_JPRB, 0.54794E+00_JPRB, 0.51895E+00_JPRB, 0.49321E+00_JPRB, 0.46943E+00_JPRB, 0.44690E+00_JPRB ,&
&  0.43029E+00_JPRB, 0.41885E+00_JPRB, 0.40907E+00_JPRB, 0.41011E+00_JPRB, 0.41288E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.39729E+00_JPRB, 0.39948E+00_JPRB, 0.39995E+00_JPRB, 0.39795E+00_JPRB, 0.39251E+00_JPRB, 0.38559E+00_JPRB, 0.38771E+00_JPRB ,&
&  0.39173E+00_JPRB, 0.37155E+00_JPRB, 0.36432E+00_JPRB, 0.36700E+00_JPRB, 0.37899E+00_JPRB, 0.39658E+00_JPRB, 0.42292E+00_JPRB ,&
&  0.46459E+00_JPRB, 0.43423E+00_JPRB, 0.34246E+00_JPRB, 0.37028E+00_JPRB, 0.43417E+00_JPRB, 0.44179E+00_JPRB, 0.52363E+00_JPRB ,&
&  0.60530E+00_JPRB, 0.69985E+00_JPRB, 0.84083E+00_JPRB, 0.11446E+01_JPRB, 0.14878E+01_JPRB, 0.18381E+01_JPRB, 0.21148E+01_JPRB ,&
&  0.21833E+01_JPRB, 0.22787E+01_JPRB, 0.22115E+01_JPRB, 0.19687E+01_JPRB, 0.16436E+01_JPRB, 0.13665E+01_JPRB, 0.11845E+01_JPRB ,&
&  0.10528E+01_JPRB, 0.92775E+00_JPRB, 0.84300E+00_JPRB, 0.79340E+00_JPRB, 0.76639E+00_JPRB, 0.76535E+00_JPRB, 0.78898E+00_JPRB ,&
&  0.82223E+00_JPRB, 0.81795E+00_JPRB, 0.80689E+00_JPRB, 0.83226E+00_JPRB, 0.81083E+00_JPRB, 0.86737E+00_JPRB, 0.97792E+00_JPRB ,&
&  0.14290E+01_JPRB, 0.16777E+01_JPRB, 0.17445E+01_JPRB, 0.19154E+01_JPRB, 0.15496E+01_JPRB, 0.14165E+01_JPRB, 0.16082E+01_JPRB ,&
&  0.17294E+01_JPRB, 0.17054E+01_JPRB, 0.16669E+01_JPRB, 0.10191E+01_JPRB, 0.73346E+00_JPRB, 0.97482E+00_JPRB, 0.93775E+00_JPRB ,&
&  0.86628E+00_JPRB, 0.87564E+00_JPRB, 0.85111E+00_JPRB, 0.85803E+00_JPRB, 0.64451E+00_JPRB, 0.53133E+00_JPRB, 0.42152E+00_JPRB ,&
&  0.40111E+00_JPRB, 0.37729E+00_JPRB, 0.40864E+00_JPRB, 0.39262E+00_JPRB, 0.37939E+00_JPRB, 0.37793E+00_JPRB, 0.37226E+00_JPRB ,&
&  0.36778E+00_JPRB, 0.36190E+00_JPRB, 0.37838E+00_JPRB, 0.43149E+00_JPRB, 0.46531E+00_JPRB, 0.41567E+00_JPRB, 0.42509E+00_JPRB ,&
&  0.43745E+00_JPRB, 0.45127E+00_JPRB, 0.45996E+00_JPRB, 0.45890E+00_JPRB, 0.46175E+00_JPRB, 0.44684E+00_JPRB, 0.42147E+00_JPRB ,&
&  0.39941E+00_JPRB, 0.39095E+00_JPRB, 0.38849E+00_JPRB, 0.39051E+00_JPRB, 0.39462E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.37820E+00_JPRB, 0.38351E+00_JPRB, 0.38652E+00_JPRB, 0.38804E+00_JPRB, 0.38834E+00_JPRB, 0.38501E+00_JPRB, 0.38530E+00_JPRB ,&
&  0.38283E+00_JPRB, 0.36837E+00_JPRB, 0.35853E+00_JPRB, 0.36008E+00_JPRB, 0.36910E+00_JPRB, 0.37983E+00_JPRB, 0.39686E+00_JPRB ,&
&  0.44109E+00_JPRB, 0.47172E+00_JPRB, 0.37218E+00_JPRB, 0.32748E+00_JPRB, 0.31585E+00_JPRB, 0.38663E+00_JPRB, 0.45619E+00_JPRB ,&
&  0.55409E+00_JPRB, 0.60146E+00_JPRB, 0.72676E+00_JPRB, 0.91388E+00_JPRB, 0.11145E+01_JPRB, 0.13050E+01_JPRB, 0.14735E+01_JPRB ,&
&  0.18204E+01_JPRB, 0.16943E+01_JPRB, 0.17625E+01_JPRB, 0.16803E+01_JPRB, 0.16287E+01_JPRB, 0.14592E+01_JPRB, 0.12608E+01_JPRB ,&
&  0.11048E+01_JPRB, 0.97855E+00_JPRB, 0.86523E+00_JPRB, 0.77785E+00_JPRB, 0.72932E+00_JPRB, 0.71151E+00_JPRB, 0.73328E+00_JPRB ,&
&  0.80459E+00_JPRB, 0.83461E+00_JPRB, 0.83077E+00_JPRB, 0.84605E+00_JPRB, 0.91280E+00_JPRB, 0.10193E+01_JPRB, 0.12040E+01_JPRB ,&
&  0.13332E+01_JPRB, 0.13968E+01_JPRB, 0.19875E+01_JPRB, 0.20890E+01_JPRB, 0.21319E+01_JPRB, 0.19196E+01_JPRB, 0.17325E+01_JPRB ,&
&  0.17850E+01_JPRB, 0.17437E+01_JPRB, 0.17043E+01_JPRB, 0.12208E+01_JPRB, 0.12433E+01_JPRB, 0.11696E+01_JPRB, 0.10263E+01_JPRB ,&
&  0.92302E+00_JPRB, 0.89153E+00_JPRB, 0.82293E+00_JPRB, 0.78583E+00_JPRB, 0.75987E+00_JPRB, 0.72533E+00_JPRB, 0.55934E+00_JPRB ,&
&  0.44752E+00_JPRB, 0.45832E+00_JPRB, 0.42836E+00_JPRB, 0.38036E+00_JPRB, 0.33811E+00_JPRB, 0.34249E+00_JPRB, 0.34591E+00_JPRB ,&
&  0.34775E+00_JPRB, 0.35063E+00_JPRB, 0.34768E+00_JPRB, 0.36715E+00_JPRB, 0.38894E+00_JPRB, 0.37503E+00_JPRB, 0.35913E+00_JPRB ,&
&  0.34061E+00_JPRB, 0.36459E+00_JPRB, 0.37159E+00_JPRB, 0.37099E+00_JPRB, 0.40597E+00_JPRB, 0.40622E+00_JPRB, 0.39097E+00_JPRB ,&
&  0.37187E+00_JPRB, 0.36358E+00_JPRB, 0.36402E+00_JPRB, 0.36691E+00_JPRB, 0.37138E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.34896E+00_JPRB, 0.36262E+00_JPRB, 0.37366E+00_JPRB, 0.38079E+00_JPRB, 0.38569E+00_JPRB, 0.38420E+00_JPRB, 0.38205E+00_JPRB ,&
&  0.37812E+00_JPRB, 0.36573E+00_JPRB, 0.35251E+00_JPRB, 0.35455E+00_JPRB, 0.35886E+00_JPRB, 0.36800E+00_JPRB, 0.38817E+00_JPRB ,&
&  0.43333E+00_JPRB, 0.56209E+00_JPRB, 0.43049E+00_JPRB, 0.39638E+00_JPRB, 0.35986E+00_JPRB, 0.39790E+00_JPRB, 0.43883E+00_JPRB ,&
&  0.53341E+00_JPRB, 0.53658E+00_JPRB, 0.58046E+00_JPRB, 0.67723E+00_JPRB, 0.83911E+00_JPRB, 0.95409E+00_JPRB, 0.11097E+01_JPRB ,&
&  0.11805E+01_JPRB, 0.12689E+01_JPRB, 0.11894E+01_JPRB, 0.12344E+01_JPRB, 0.12598E+01_JPRB, 0.11640E+01_JPRB, 0.11584E+01_JPRB ,&
&  0.10189E+01_JPRB, 0.92205E+00_JPRB, 0.84797E+00_JPRB, 0.78268E+00_JPRB, 0.74163E+00_JPRB, 0.71443E+00_JPRB, 0.71491E+00_JPRB ,&
&  0.77381E+00_JPRB, 0.84410E+00_JPRB, 0.85682E+00_JPRB, 0.86307E+00_JPRB, 0.89670E+00_JPRB, 0.10455E+01_JPRB, 0.12183E+01_JPRB ,&
&  0.16682E+01_JPRB, 0.17354E+01_JPRB, 0.14385E+01_JPRB, 0.19833E+01_JPRB, 0.24370E+01_JPRB, 0.21097E+01_JPRB, 0.17144E+01_JPRB ,&
&  0.17197E+01_JPRB, 0.16780E+01_JPRB, 0.16218E+01_JPRB, 0.14875E+01_JPRB, 0.13684E+01_JPRB, 0.12444E+01_JPRB, 0.10834E+01_JPRB ,&
&  0.92137E+00_JPRB, 0.83310E+00_JPRB, 0.78227E+00_JPRB, 0.74813E+00_JPRB, 0.67933E+00_JPRB, 0.61600E+00_JPRB, 0.56889E+00_JPRB ,&
&  0.49258E+00_JPRB, 0.42816E+00_JPRB, 0.33119E+00_JPRB, 0.34462E+00_JPRB, 0.29903E+00_JPRB, 0.30220E+00_JPRB, 0.32422E+00_JPRB ,&
&  0.32064E+00_JPRB, 0.32359E+00_JPRB, 0.31895E+00_JPRB, 0.31131E+00_JPRB, 0.33287E+00_JPRB, 0.32858E+00_JPRB, 0.32261E+00_JPRB ,&
&  0.30989E+00_JPRB, 0.29908E+00_JPRB, 0.31675E+00_JPRB, 0.32017E+00_JPRB, 0.33495E+00_JPRB, 0.33813E+00_JPRB, 0.33714E+00_JPRB ,&
&  0.33177E+00_JPRB, 0.33062E+00_JPRB, 0.33535E+00_JPRB, 0.33974E+00_JPRB, 0.34140E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.32095E+00_JPRB, 0.34077E+00_JPRB, 0.35585E+00_JPRB, 0.36310E+00_JPRB, 0.37108E+00_JPRB, 0.36656E+00_JPRB, 0.36449E+00_JPRB ,&
&  0.36696E+00_JPRB, 0.35698E+00_JPRB, 0.34292E+00_JPRB, 0.34492E+00_JPRB, 0.34766E+00_JPRB, 0.35612E+00_JPRB, 0.38147E+00_JPRB ,&
&  0.44799E+00_JPRB, 0.47753E+00_JPRB, 0.40553E+00_JPRB, 0.35060E+00_JPRB, 0.41742E+00_JPRB, 0.39548E+00_JPRB, 0.41698E+00_JPRB ,&
&  0.40085E+00_JPRB, 0.42527E+00_JPRB, 0.43931E+00_JPRB, 0.47317E+00_JPRB, 0.56445E+00_JPRB, 0.70298E+00_JPRB, 0.79681E+00_JPRB ,&
&  0.83434E+00_JPRB, 0.85887E+00_JPRB, 0.84177E+00_JPRB, 0.83236E+00_JPRB, 0.85469E+00_JPRB, 0.79348E+00_JPRB, 0.80726E+00_JPRB ,&
&  0.78259E+00_JPRB, 0.73478E+00_JPRB, 0.69501E+00_JPRB, 0.69024E+00_JPRB, 0.68247E+00_JPRB, 0.67423E+00_JPRB, 0.68948E+00_JPRB ,&
&  0.75352E+00_JPRB, 0.84272E+00_JPRB, 0.85887E+00_JPRB, 0.87550E+00_JPRB, 0.94024E+00_JPRB, 0.11940E+01_JPRB, 0.16383E+01_JPRB ,&
&  0.20526E+01_JPRB, 0.21982E+01_JPRB, 0.22806E+01_JPRB, 0.24459E+01_JPRB, 0.23756E+01_JPRB, 0.21828E+01_JPRB, 0.19986E+01_JPRB ,&
&  0.18200E+01_JPRB, 0.16475E+01_JPRB, 0.15034E+01_JPRB, 0.13695E+01_JPRB, 0.12812E+01_JPRB, 0.12139E+01_JPRB, 0.10713E+01_JPRB ,&
&  0.88377E+00_JPRB, 0.77468E+00_JPRB, 0.72228E+00_JPRB, 0.63994E+00_JPRB, 0.58455E+00_JPRB, 0.52955E+00_JPRB, 0.53157E+00_JPRB ,&
&  0.54140E+00_JPRB, 0.34745E+00_JPRB, 0.29713E+00_JPRB, 0.33141E+00_JPRB, 0.30133E+00_JPRB, 0.30564E+00_JPRB, 0.33278E+00_JPRB ,&
&  0.32714E+00_JPRB, 0.31273E+00_JPRB, 0.30167E+00_JPRB, 0.29147E+00_JPRB, 0.29702E+00_JPRB, 0.31477E+00_JPRB, 0.29397E+00_JPRB ,&
&  0.27806E+00_JPRB, 0.27584E+00_JPRB, 0.28549E+00_JPRB, 0.28315E+00_JPRB, 0.29005E+00_JPRB, 0.29380E+00_JPRB, 0.29879E+00_JPRB ,&
&  0.29804E+00_JPRB, 0.29984E+00_JPRB, 0.29935E+00_JPRB, 0.30480E+00_JPRB, 0.31327E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.30507E+00_JPRB, 0.32683E+00_JPRB, 0.34028E+00_JPRB, 0.35055E+00_JPRB, 0.35903E+00_JPRB, 0.35247E+00_JPRB, 0.35089E+00_JPRB ,&
&  0.35400E+00_JPRB, 0.34760E+00_JPRB, 0.33619E+00_JPRB, 0.33599E+00_JPRB, 0.33799E+00_JPRB, 0.33829E+00_JPRB, 0.34288E+00_JPRB ,&
&  0.29393E+00_JPRB, 0.32715E+00_JPRB, 0.29820E+00_JPRB, 0.32370E+00_JPRB, 0.39117E+00_JPRB, 0.36402E+00_JPRB, 0.35875E+00_JPRB ,&
&  0.34697E+00_JPRB, 0.33679E+00_JPRB, 0.32981E+00_JPRB, 0.34547E+00_JPRB, 0.38126E+00_JPRB, 0.50124E+00_JPRB, 0.60911E+00_JPRB ,&
&  0.58495E+00_JPRB, 0.55602E+00_JPRB, 0.54607E+00_JPRB, 0.54756E+00_JPRB, 0.53665E+00_JPRB, 0.53867E+00_JPRB, 0.56130E+00_JPRB ,&
&  0.56580E+00_JPRB, 0.55735E+00_JPRB, 0.56636E+00_JPRB, 0.59207E+00_JPRB, 0.60703E+00_JPRB, 0.60445E+00_JPRB, 0.61639E+00_JPRB ,&
&  0.67736E+00_JPRB, 0.77553E+00_JPRB, 0.81563E+00_JPRB, 0.87867E+00_JPRB, 0.96915E+00_JPRB, 0.14649E+01_JPRB, 0.21068E+01_JPRB ,&
&  0.24036E+01_JPRB, 0.23399E+01_JPRB, 0.24199E+01_JPRB, 0.25533E+01_JPRB, 0.24552E+01_JPRB, 0.22099E+01_JPRB, 0.19181E+01_JPRB ,&
&  0.16661E+01_JPRB, 0.14346E+01_JPRB, 0.12878E+01_JPRB, 0.12052E+01_JPRB, 0.10792E+01_JPRB, 0.10563E+01_JPRB, 0.95745E+00_JPRB ,&
&  0.82263E+00_JPRB, 0.75052E+00_JPRB, 0.71183E+00_JPRB, 0.66791E+00_JPRB, 0.61021E+00_JPRB, 0.59823E+00_JPRB, 0.55922E+00_JPRB ,&
&  0.53332E+00_JPRB, 0.46863E+00_JPRB, 0.37505E+00_JPRB, 0.35447E+00_JPRB, 0.30831E+00_JPRB, 0.31438E+00_JPRB, 0.36003E+00_JPRB ,&
&  0.35020E+00_JPRB, 0.31893E+00_JPRB, 0.29178E+00_JPRB, 0.27768E+00_JPRB, 0.28388E+00_JPRB, 0.30070E+00_JPRB, 0.27614E+00_JPRB ,&
&  0.24925E+00_JPRB, 0.25461E+00_JPRB, 0.25797E+00_JPRB, 0.25732E+00_JPRB, 0.25883E+00_JPRB, 0.26539E+00_JPRB, 0.26781E+00_JPRB ,&
&  0.27063E+00_JPRB, 0.27334E+00_JPRB, 0.27807E+00_JPRB, 0.28044E+00_JPRB, 0.29266E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.29151E+00_JPRB, 0.30911E+00_JPRB, 0.31925E+00_JPRB, 0.33424E+00_JPRB, 0.34531E+00_JPRB, 0.34141E+00_JPRB, 0.33889E+00_JPRB ,&
&  0.33747E+00_JPRB, 0.33462E+00_JPRB, 0.33000E+00_JPRB, 0.32894E+00_JPRB, 0.32925E+00_JPRB, 0.32599E+00_JPRB, 0.29241E+00_JPRB ,&
&  0.25471E+00_JPRB, 0.27161E+00_JPRB, 0.27312E+00_JPRB, 0.28614E+00_JPRB, 0.30985E+00_JPRB, 0.30086E+00_JPRB, 0.28628E+00_JPRB ,&
&  0.28654E+00_JPRB, 0.28218E+00_JPRB, 0.28975E+00_JPRB, 0.30556E+00_JPRB, 0.32487E+00_JPRB, 0.38816E+00_JPRB, 0.50036E+00_JPRB ,&
&  0.46859E+00_JPRB, 0.45312E+00_JPRB, 0.44061E+00_JPRB, 0.44393E+00_JPRB, 0.45283E+00_JPRB, 0.45048E+00_JPRB, 0.44779E+00_JPRB ,&
&  0.44577E+00_JPRB, 0.45568E+00_JPRB, 0.47201E+00_JPRB, 0.49604E+00_JPRB, 0.51801E+00_JPRB, 0.52598E+00_JPRB, 0.52925E+00_JPRB ,&
&  0.56534E+00_JPRB, 0.63631E+00_JPRB, 0.69176E+00_JPRB, 0.76171E+00_JPRB, 0.87176E+00_JPRB, 0.13367E+01_JPRB, 0.17440E+01_JPRB ,&
&  0.18413E+01_JPRB, 0.18704E+01_JPRB, 0.18552E+01_JPRB, 0.18322E+01_JPRB, 0.18348E+01_JPRB, 0.16467E+01_JPRB, 0.14316E+01_JPRB ,&
&  0.12591E+01_JPRB, 0.10915E+01_JPRB, 0.11004E+01_JPRB, 0.10710E+01_JPRB, 0.98059E+00_JPRB, 0.92503E+00_JPRB, 0.83474E+00_JPRB ,&
&  0.77572E+00_JPRB, 0.71600E+00_JPRB, 0.67756E+00_JPRB, 0.64332E+00_JPRB, 0.60369E+00_JPRB, 0.58272E+00_JPRB, 0.53486E+00_JPRB ,&
&  0.50966E+00_JPRB, 0.48145E+00_JPRB, 0.42191E+00_JPRB, 0.38464E+00_JPRB, 0.35072E+00_JPRB, 0.36433E+00_JPRB, 0.34966E+00_JPRB ,&
&  0.32181E+00_JPRB, 0.28437E+00_JPRB, 0.27552E+00_JPRB, 0.27040E+00_JPRB, 0.27663E+00_JPRB, 0.27603E+00_JPRB, 0.25074E+00_JPRB ,&
&  0.23865E+00_JPRB, 0.24740E+00_JPRB, 0.24572E+00_JPRB, 0.24220E+00_JPRB, 0.24184E+00_JPRB, 0.24146E+00_JPRB, 0.23119E+00_JPRB ,&
&  0.23348E+00_JPRB, 0.25470E+00_JPRB, 0.26680E+00_JPRB, 0.26916E+00_JPRB, 0.27869E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.27736E+00_JPRB, 0.29329E+00_JPRB, 0.30520E+00_JPRB, 0.32038E+00_JPRB, 0.32979E+00_JPRB, 0.32892E+00_JPRB, 0.31547E+00_JPRB ,&
&  0.32477E+00_JPRB, 0.32261E+00_JPRB, 0.32049E+00_JPRB, 0.32338E+00_JPRB, 0.32597E+00_JPRB, 0.29235E+00_JPRB, 0.23588E+00_JPRB ,&
&  0.22220E+00_JPRB, 0.23210E+00_JPRB, 0.25310E+00_JPRB, 0.26799E+00_JPRB, 0.26892E+00_JPRB, 0.26131E+00_JPRB, 0.25405E+00_JPRB ,&
&  0.25248E+00_JPRB, 0.25864E+00_JPRB, 0.28020E+00_JPRB, 0.29860E+00_JPRB, 0.30653E+00_JPRB, 0.32434E+00_JPRB, 0.41243E+00_JPRB ,&
&  0.41393E+00_JPRB, 0.40540E+00_JPRB, 0.40620E+00_JPRB, 0.38515E+00_JPRB, 0.40997E+00_JPRB, 0.39136E+00_JPRB, 0.37730E+00_JPRB ,&
&  0.38031E+00_JPRB, 0.39446E+00_JPRB, 0.41631E+00_JPRB, 0.44426E+00_JPRB, 0.46319E+00_JPRB, 0.47036E+00_JPRB, 0.47543E+00_JPRB ,&
&  0.49098E+00_JPRB, 0.53045E+00_JPRB, 0.57949E+00_JPRB, 0.65044E+00_JPRB, 0.74651E+00_JPRB, 0.10460E+01_JPRB, 0.13446E+01_JPRB ,&
&  0.13980E+01_JPRB, 0.14842E+01_JPRB, 0.15232E+01_JPRB, 0.13562E+01_JPRB, 0.14098E+01_JPRB, 0.13200E+01_JPRB, 0.11747E+01_JPRB ,&
&  0.10601E+01_JPRB, 0.92777E+00_JPRB, 0.91897E+00_JPRB, 0.91370E+00_JPRB, 0.89335E+00_JPRB, 0.81669E+00_JPRB, 0.73575E+00_JPRB ,&
&  0.69988E+00_JPRB, 0.68581E+00_JPRB, 0.60866E+00_JPRB, 0.56520E+00_JPRB, 0.52464E+00_JPRB, 0.50431E+00_JPRB, 0.47264E+00_JPRB ,&
&  0.45113E+00_JPRB, 0.43625E+00_JPRB, 0.41680E+00_JPRB, 0.39795E+00_JPRB, 0.37794E+00_JPRB, 0.36118E+00_JPRB, 0.34228E+00_JPRB ,&
&  0.31657E+00_JPRB, 0.28502E+00_JPRB, 0.26920E+00_JPRB, 0.27085E+00_JPRB, 0.27866E+00_JPRB, 0.26219E+00_JPRB, 0.23960E+00_JPRB ,&
&  0.22917E+00_JPRB, 0.23089E+00_JPRB, 0.23902E+00_JPRB, 0.23268E+00_JPRB, 0.22724E+00_JPRB, 0.22584E+00_JPRB, 0.22397E+00_JPRB ,&
&  0.22558E+00_JPRB, 0.24724E+00_JPRB, 0.26012E+00_JPRB, 0.26442E+00_JPRB, 0.26759E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.26973E+00_JPRB, 0.28473E+00_JPRB, 0.29773E+00_JPRB, 0.31020E+00_JPRB, 0.31802E+00_JPRB, 0.30802E+00_JPRB, 0.30897E+00_JPRB ,&
&  0.35024E+00_JPRB, 0.30142E+00_JPRB, 0.31972E+00_JPRB, 0.26207E+00_JPRB, 0.24028E+00_JPRB, 0.23548E+00_JPRB, 0.22691E+00_JPRB ,&
&  0.22721E+00_JPRB, 0.23587E+00_JPRB, 0.24674E+00_JPRB, 0.25459E+00_JPRB, 0.25911E+00_JPRB, 0.24827E+00_JPRB, 0.24095E+00_JPRB ,&
&  0.23759E+00_JPRB, 0.24791E+00_JPRB, 0.27616E+00_JPRB, 0.29763E+00_JPRB, 0.31089E+00_JPRB, 0.32324E+00_JPRB, 0.36340E+00_JPRB ,&
&  0.36576E+00_JPRB, 0.35839E+00_JPRB, 0.36821E+00_JPRB, 0.36978E+00_JPRB, 0.37137E+00_JPRB, 0.35401E+00_JPRB, 0.33818E+00_JPRB ,&
&  0.33495E+00_JPRB, 0.32996E+00_JPRB, 0.37568E+00_JPRB, 0.40540E+00_JPRB, 0.42295E+00_JPRB, 0.42969E+00_JPRB, 0.44080E+00_JPRB ,&
&  0.45379E+00_JPRB, 0.47634E+00_JPRB, 0.51592E+00_JPRB, 0.57607E+00_JPRB, 0.64439E+00_JPRB, 0.77135E+00_JPRB, 0.89738E+00_JPRB ,&
&  0.10779E+01_JPRB, 0.92478E+00_JPRB, 0.11230E+01_JPRB, 0.10984E+01_JPRB, 0.11993E+01_JPRB, 0.11774E+01_JPRB, 0.10953E+01_JPRB ,&
&  0.99933E+00_JPRB, 0.89461E+00_JPRB, 0.85329E+00_JPRB, 0.79584E+00_JPRB, 0.75155E+00_JPRB, 0.70376E+00_JPRB, 0.65166E+00_JPRB ,&
&  0.62131E+00_JPRB, 0.55770E+00_JPRB, 0.53450E+00_JPRB, 0.49896E+00_JPRB, 0.46641E+00_JPRB, 0.44883E+00_JPRB, 0.43007E+00_JPRB ,&
&  0.41539E+00_JPRB, 0.40510E+00_JPRB, 0.39039E+00_JPRB, 0.35947E+00_JPRB, 0.35115E+00_JPRB, 0.34178E+00_JPRB, 0.32029E+00_JPRB ,&
&  0.30874E+00_JPRB, 0.29530E+00_JPRB, 0.28540E+00_JPRB, 0.29462E+00_JPRB, 0.29333E+00_JPRB, 0.27554E+00_JPRB, 0.25096E+00_JPRB ,&
&  0.23058E+00_JPRB, 0.21908E+00_JPRB, 0.21766E+00_JPRB, 0.21448E+00_JPRB, 0.21113E+00_JPRB, 0.21448E+00_JPRB, 0.22453E+00_JPRB ,&
&  0.22862E+00_JPRB, 0.24023E+00_JPRB, 0.25587E+00_JPRB, 0.26162E+00_JPRB, 0.26320E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.26631E+00_JPRB, 0.27779E+00_JPRB, 0.28605E+00_JPRB, 0.29275E+00_JPRB, 0.29813E+00_JPRB, 0.29330E+00_JPRB, 0.27833E+00_JPRB ,&
&  0.28764E+00_JPRB, 0.24924E+00_JPRB, 0.23354E+00_JPRB, 0.22504E+00_JPRB, 0.22478E+00_JPRB, 0.22184E+00_JPRB, 0.21511E+00_JPRB ,&
&  0.21561E+00_JPRB, 0.22394E+00_JPRB, 0.23948E+00_JPRB, 0.24589E+00_JPRB, 0.24471E+00_JPRB, 0.24049E+00_JPRB, 0.23684E+00_JPRB ,&
&  0.23686E+00_JPRB, 0.24884E+00_JPRB, 0.27247E+00_JPRB, 0.30394E+00_JPRB, 0.32005E+00_JPRB, 0.32687E+00_JPRB, 0.33677E+00_JPRB ,&
&  0.34288E+00_JPRB, 0.34401E+00_JPRB, 0.34691E+00_JPRB, 0.35517E+00_JPRB, 0.34687E+00_JPRB, 0.32211E+00_JPRB, 0.29513E+00_JPRB ,&
&  0.25489E+00_JPRB, 0.26045E+00_JPRB, 0.34062E+00_JPRB, 0.37103E+00_JPRB, 0.39082E+00_JPRB, 0.40036E+00_JPRB, 0.41126E+00_JPRB ,&
&  0.42798E+00_JPRB, 0.44910E+00_JPRB, 0.47743E+00_JPRB, 0.51652E+00_JPRB, 0.55513E+00_JPRB, 0.61258E+00_JPRB, 0.66237E+00_JPRB ,&
&  0.77113E+00_JPRB, 0.65351E+00_JPRB, 0.63941E+00_JPRB, 0.69044E+00_JPRB, 0.96587E+00_JPRB, 0.10139E+01_JPRB, 0.98105E+00_JPRB ,&
&  0.92454E+00_JPRB, 0.81680E+00_JPRB, 0.76633E+00_JPRB, 0.72894E+00_JPRB, 0.68541E+00_JPRB, 0.63164E+00_JPRB, 0.57489E+00_JPRB ,&
&  0.53174E+00_JPRB, 0.49951E+00_JPRB, 0.46820E+00_JPRB, 0.45329E+00_JPRB, 0.43426E+00_JPRB, 0.42463E+00_JPRB, 0.41237E+00_JPRB ,&
&  0.39968E+00_JPRB, 0.39083E+00_JPRB, 0.37158E+00_JPRB, 0.35274E+00_JPRB, 0.33991E+00_JPRB, 0.33011E+00_JPRB, 0.31899E+00_JPRB ,&
&  0.30967E+00_JPRB, 0.30699E+00_JPRB, 0.30887E+00_JPRB, 0.31335E+00_JPRB, 0.30266E+00_JPRB, 0.27535E+00_JPRB, 0.24980E+00_JPRB ,&
&  0.22760E+00_JPRB, 0.21013E+00_JPRB, 0.19734E+00_JPRB, 0.19358E+00_JPRB, 0.19713E+00_JPRB, 0.20304E+00_JPRB, 0.20957E+00_JPRB ,&
&  0.21713E+00_JPRB, 0.22530E+00_JPRB, 0.23481E+00_JPRB, 0.24501E+00_JPRB, 0.25356E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.26531E+00_JPRB, 0.27426E+00_JPRB, 0.27453E+00_JPRB, 0.27468E+00_JPRB, 0.27079E+00_JPRB, 0.26537E+00_JPRB, 0.25661E+00_JPRB ,&
&  0.26025E+00_JPRB, 0.24491E+00_JPRB, 0.23280E+00_JPRB, 0.22241E+00_JPRB, 0.21656E+00_JPRB, 0.21280E+00_JPRB, 0.21523E+00_JPRB ,&
&  0.22315E+00_JPRB, 0.23394E+00_JPRB, 0.24089E+00_JPRB, 0.23988E+00_JPRB, 0.23803E+00_JPRB, 0.23644E+00_JPRB, 0.23848E+00_JPRB ,&
&  0.24188E+00_JPRB, 0.25060E+00_JPRB, 0.26627E+00_JPRB, 0.28913E+00_JPRB, 0.31155E+00_JPRB, 0.32463E+00_JPRB, 0.32737E+00_JPRB ,&
&  0.32623E+00_JPRB, 0.32735E+00_JPRB, 0.33300E+00_JPRB, 0.33858E+00_JPRB, 0.32047E+00_JPRB, 0.28275E+00_JPRB, 0.24763E+00_JPRB ,&
&  0.22056E+00_JPRB, 0.21666E+00_JPRB, 0.24343E+00_JPRB, 0.32516E+00_JPRB, 0.34715E+00_JPRB, 0.36207E+00_JPRB, 0.37557E+00_JPRB ,&
&  0.36285E+00_JPRB, 0.36602E+00_JPRB, 0.39445E+00_JPRB, 0.45148E+00_JPRB, 0.46780E+00_JPRB, 0.50025E+00_JPRB, 0.52487E+00_JPRB ,&
&  0.55581E+00_JPRB, 0.56485E+00_JPRB, 0.60129E+00_JPRB, 0.57456E+00_JPRB, 0.67360E+00_JPRB, 0.90090E+00_JPRB, 0.84807E+00_JPRB ,&
&  0.79597E+00_JPRB, 0.71971E+00_JPRB, 0.70278E+00_JPRB, 0.65279E+00_JPRB, 0.61057E+00_JPRB, 0.57268E+00_JPRB, 0.52666E+00_JPRB ,&
&  0.47878E+00_JPRB, 0.44880E+00_JPRB, 0.43111E+00_JPRB, 0.42952E+00_JPRB, 0.42702E+00_JPRB, 0.42058E+00_JPRB, 0.41287E+00_JPRB ,&
&  0.40352E+00_JPRB, 0.39124E+00_JPRB, 0.36847E+00_JPRB, 0.34333E+00_JPRB, 0.33281E+00_JPRB, 0.32204E+00_JPRB, 0.31264E+00_JPRB ,&
&  0.30640E+00_JPRB, 0.30886E+00_JPRB, 0.31553E+00_JPRB, 0.31518E+00_JPRB, 0.29127E+00_JPRB, 0.25751E+00_JPRB, 0.23189E+00_JPRB ,&
&  0.21370E+00_JPRB, 0.20174E+00_JPRB, 0.19245E+00_JPRB, 0.18973E+00_JPRB, 0.19333E+00_JPRB, 0.19754E+00_JPRB, 0.19998E+00_JPRB ,&
&  0.20332E+00_JPRB, 0.20874E+00_JPRB, 0.22243E+00_JPRB, 0.23766E+00_JPRB, 0.25151E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.24746E+00_JPRB, 0.26313E+00_JPRB, 0.27170E+00_JPRB, 0.27071E+00_JPRB, 0.26842E+00_JPRB, 0.25428E+00_JPRB, 0.23952E+00_JPRB ,&
&  0.23507E+00_JPRB, 0.23225E+00_JPRB, 0.23224E+00_JPRB, 0.23271E+00_JPRB, 0.23414E+00_JPRB, 0.23700E+00_JPRB, 0.24037E+00_JPRB ,&
&  0.24108E+00_JPRB, 0.24144E+00_JPRB, 0.24310E+00_JPRB, 0.24621E+00_JPRB, 0.24847E+00_JPRB, 0.24884E+00_JPRB, 0.25019E+00_JPRB ,&
&  0.25144E+00_JPRB, 0.25489E+00_JPRB, 0.26415E+00_JPRB, 0.27718E+00_JPRB, 0.29120E+00_JPRB, 0.30545E+00_JPRB, 0.31587E+00_JPRB ,&
&  0.31866E+00_JPRB, 0.31241E+00_JPRB, 0.31134E+00_JPRB, 0.30971E+00_JPRB, 0.28699E+00_JPRB, 0.26191E+00_JPRB, 0.23136E+00_JPRB ,&
&  0.20885E+00_JPRB, 0.19619E+00_JPRB, 0.19263E+00_JPRB, 0.20518E+00_JPRB, 0.24171E+00_JPRB, 0.31700E+00_JPRB, 0.33804E+00_JPRB ,&
&  0.35365E+00_JPRB, 0.36341E+00_JPRB, 0.37216E+00_JPRB, 0.38558E+00_JPRB, 0.40027E+00_JPRB, 0.42243E+00_JPRB, 0.43987E+00_JPRB ,&
&  0.45701E+00_JPRB, 0.46965E+00_JPRB, 0.48773E+00_JPRB, 0.47575E+00_JPRB, 0.47950E+00_JPRB, 0.53780E+00_JPRB, 0.59264E+00_JPRB ,&
&  0.59660E+00_JPRB, 0.54591E+00_JPRB, 0.53636E+00_JPRB, 0.54352E+00_JPRB, 0.54044E+00_JPRB, 0.54585E+00_JPRB, 0.52750E+00_JPRB ,&
&  0.49697E+00_JPRB, 0.46518E+00_JPRB, 0.44143E+00_JPRB, 0.42796E+00_JPRB, 0.42828E+00_JPRB, 0.43089E+00_JPRB, 0.41926E+00_JPRB ,&
&  0.40553E+00_JPRB, 0.39018E+00_JPRB, 0.36491E+00_JPRB, 0.33128E+00_JPRB, 0.32173E+00_JPRB, 0.32148E+00_JPRB, 0.32070E+00_JPRB ,&
&  0.31730E+00_JPRB, 0.31816E+00_JPRB, 0.32143E+00_JPRB, 0.31748E+00_JPRB, 0.29069E+00_JPRB, 0.26040E+00_JPRB, 0.23638E+00_JPRB ,&
&  0.22126E+00_JPRB, 0.21187E+00_JPRB, 0.20508E+00_JPRB, 0.20372E+00_JPRB, 0.20218E+00_JPRB, 0.20186E+00_JPRB, 0.19933E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.20384E+00_JPRB, 0.21249E+00_JPRB, 0.22443E+00_JPRB, 0.23628E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.24958E+00_JPRB, 0.25969E+00_JPRB, 0.26634E+00_JPRB, 0.26847E+00_JPRB, 0.26725E+00_JPRB, 0.26160E+00_JPRB, 0.25692E+00_JPRB ,&
&  0.25630E+00_JPRB, 0.25857E+00_JPRB, 0.25928E+00_JPRB, 0.25894E+00_JPRB, 0.25841E+00_JPRB, 0.25793E+00_JPRB, 0.25829E+00_JPRB ,&
&  0.25923E+00_JPRB, 0.26111E+00_JPRB, 0.26380E+00_JPRB, 0.26305E+00_JPRB, 0.26141E+00_JPRB, 0.26059E+00_JPRB, 0.25960E+00_JPRB ,&
&  0.26017E+00_JPRB, 0.26319E+00_JPRB, 0.26837E+00_JPRB, 0.27542E+00_JPRB, 0.27993E+00_JPRB, 0.28457E+00_JPRB, 0.28528E+00_JPRB ,&
&  0.28316E+00_JPRB, 0.28534E+00_JPRB, 0.29137E+00_JPRB, 0.27799E+00_JPRB, 0.25706E+00_JPRB, 0.23980E+00_JPRB, 0.21414E+00_JPRB ,&
&  0.19351E+00_JPRB, 0.17857E+00_JPRB, 0.16932E+00_JPRB, 0.16742E+00_JPRB, 0.17402E+00_JPRB, 0.19165E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.30729E+00_JPRB, 0.32412E+00_JPRB, 0.33088E+00_JPRB, 0.34398E+00_JPRB, 0.36018E+00_JPRB, 0.37780E+00_JPRB, 0.39383E+00_JPRB ,&
&  0.40380E+00_JPRB, 0.40759E+00_JPRB, 0.41214E+00_JPRB, 0.41982E+00_JPRB, 0.42920E+00_JPRB, 0.42222E+00_JPRB, 0.44594E+00_JPRB ,&
&  0.49089E+00_JPRB, 0.47973E+00_JPRB, 0.43956E+00_JPRB, 0.43555E+00_JPRB, 0.44640E+00_JPRB, 0.46055E+00_JPRB, 0.47787E+00_JPRB ,&
&  0.48076E+00_JPRB, 0.47112E+00_JPRB, 0.45859E+00_JPRB, 0.44348E+00_JPRB, 0.43578E+00_JPRB, 0.42983E+00_JPRB, 0.42067E+00_JPRB ,&
&  0.40726E+00_JPRB, 0.39626E+00_JPRB, 0.40874E+00_JPRB, 0.36350E+00_JPRB, 0.34556E+00_JPRB, 0.34196E+00_JPRB, 0.32987E+00_JPRB ,&
&  0.32317E+00_JPRB, 0.32239E+00_JPRB, 0.32442E+00_JPRB, 0.32163E+00_JPRB, 0.30546E+00_JPRB, 0.28162E+00_JPRB, 0.25736E+00_JPRB ,&
&  0.24095E+00_JPRB, 0.22718E+00_JPRB, 0.21521E+00_JPRB, 0.21058E+00_JPRB, 0.20876E+00_JPRB, 0.20573E+00_JPRB, 0.20187E+00_JPRB ,&
&  0.20284E+00_JPRB, 0.20726E+00_JPRB, 0.21639E+00_JPRB, 0.23130E+00_JPRB, 0.24158E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.24270E+00_JPRB, 0.25267E+00_JPRB, 0.26082E+00_JPRB, 0.26506E+00_JPRB, 0.26586E+00_JPRB, 0.26600E+00_JPRB, 0.26516E+00_JPRB ,&
&  0.26496E+00_JPRB, 0.26517E+00_JPRB, 0.26549E+00_JPRB, 0.26563E+00_JPRB, 0.26574E+00_JPRB, 0.26519E+00_JPRB, 0.26586E+00_JPRB ,&
&  0.26700E+00_JPRB, 0.26716E+00_JPRB, 0.26836E+00_JPRB, 0.27087E+00_JPRB, 0.27026E+00_JPRB, 0.26739E+00_JPRB, 0.26276E+00_JPRB ,&
&  0.26061E+00_JPRB, 0.26300E+00_JPRB, 0.26469E+00_JPRB, 0.26355E+00_JPRB, 0.26137E+00_JPRB, 0.25942E+00_JPRB, 0.26172E+00_JPRB ,&
&  0.26943E+00_JPRB, 0.26633E+00_JPRB, 0.26243E+00_JPRB, 0.24665E+00_JPRB, 0.23070E+00_JPRB, 0.22204E+00_JPRB, 0.19374E+00_JPRB ,&
&  0.17378E+00_JPRB, 0.16205E+00_JPRB, 0.15792E+00_JPRB, 0.15689E+00_JPRB, 0.16074E+00_JPRB, 0.17076E+00_JPRB, 0.19285E+00_JPRB ,&
&  0.23058E+00_JPRB, 0.29156E+00_JPRB, 0.30036E+00_JPRB, 0.31435E+00_JPRB, 0.33318E+00_JPRB, 0.35311E+00_JPRB, 0.36772E+00_JPRB ,&
&  0.37680E+00_JPRB, 0.38078E+00_JPRB, 0.38622E+00_JPRB, 0.38965E+00_JPRB, 0.39852E+00_JPRB, 0.40707E+00_JPRB, 0.40840E+00_JPRB ,&
&  0.40421E+00_JPRB, 0.38376E+00_JPRB, 0.38280E+00_JPRB, 0.38190E+00_JPRB, 0.39415E+00_JPRB, 0.40884E+00_JPRB, 0.40684E+00_JPRB ,&
&  0.40649E+00_JPRB, 0.41803E+00_JPRB, 0.41194E+00_JPRB, 0.40858E+00_JPRB, 0.41399E+00_JPRB, 0.42242E+00_JPRB, 0.42115E+00_JPRB ,&
&  0.40347E+00_JPRB, 0.39201E+00_JPRB, 0.39932E+00_JPRB, 0.38172E+00_JPRB, 0.37209E+00_JPRB, 0.36311E+00_JPRB, 0.34716E+00_JPRB ,&
&  0.33497E+00_JPRB, 0.32676E+00_JPRB, 0.32228E+00_JPRB, 0.31845E+00_JPRB, 0.31168E+00_JPRB, 0.29930E+00_JPRB, 0.28199E+00_JPRB ,&
&  0.26090E+00_JPRB, 0.24249E+00_JPRB, 0.22892E+00_JPRB, 0.22068E+00_JPRB, 0.21525E+00_JPRB, 0.21049E+00_JPRB, 0.20809E+00_JPRB ,&
&  0.20817E+00_JPRB, 0.21267E+00_JPRB, 0.21910E+00_JPRB, 0.22727E+00_JPRB, 0.23504E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.24288E+00_JPRB, 0.24945E+00_JPRB, 0.25490E+00_JPRB, 0.25816E+00_JPRB, 0.25989E+00_JPRB, 0.26096E+00_JPRB, 0.26043E+00_JPRB ,&
&  0.25997E+00_JPRB, 0.25969E+00_JPRB, 0.26153E+00_JPRB, 0.26366E+00_JPRB, 0.26682E+00_JPRB, 0.26932E+00_JPRB, 0.27334E+00_JPRB ,&
&  0.27837E+00_JPRB, 0.28274E+00_JPRB, 0.28224E+00_JPRB, 0.27860E+00_JPRB, 0.27422E+00_JPRB, 0.27036E+00_JPRB, 0.26196E+00_JPRB ,&
&  0.25565E+00_JPRB, 0.25703E+00_JPRB, 0.25625E+00_JPRB, 0.25284E+00_JPRB, 0.25074E+00_JPRB, 0.24784E+00_JPRB, 0.24990E+00_JPRB ,&
&  0.24970E+00_JPRB, 0.24434E+00_JPRB, 0.23688E+00_JPRB, 0.22627E+00_JPRB, 0.21634E+00_JPRB, 0.18446E+00_JPRB, 0.17571E+00_JPRB ,&
&  0.16567E+00_JPRB, 0.15821E+00_JPRB, 0.15584E+00_JPRB, 0.15528E+00_JPRB, 0.15714E+00_JPRB, 0.16309E+00_JPRB, 0.17231E+00_JPRB ,&
&  0.18647E+00_JPRB, 0.26174E+00_JPRB, 0.27443E+00_JPRB, 0.28241E+00_JPRB, 0.29750E+00_JPRB, 0.31614E+00_JPRB, 0.33254E+00_JPRB ,&
&  0.34422E+00_JPRB, 0.34916E+00_JPRB, 0.35396E+00_JPRB, 0.35671E+00_JPRB, 0.36207E+00_JPRB, 0.36871E+00_JPRB, 0.37117E+00_JPRB ,&
&  0.36857E+00_JPRB, 0.36154E+00_JPRB, 0.36737E+00_JPRB, 0.36776E+00_JPRB, 0.36191E+00_JPRB, 0.36737E+00_JPRB, 0.37076E+00_JPRB ,&
&  0.36809E+00_JPRB, 0.36135E+00_JPRB, 0.36248E+00_JPRB, 0.37124E+00_JPRB, 0.38520E+00_JPRB, 0.39614E+00_JPRB, 0.39554E+00_JPRB ,&
&  0.38990E+00_JPRB, 0.37901E+00_JPRB, 0.36927E+00_JPRB, 0.36366E+00_JPRB, 0.35578E+00_JPRB, 0.34696E+00_JPRB, 0.33922E+00_JPRB ,&
&  0.32943E+00_JPRB, 0.32365E+00_JPRB, 0.31746E+00_JPRB, 0.31346E+00_JPRB, 0.30753E+00_JPRB, 0.30008E+00_JPRB, 0.29005E+00_JPRB ,&
&  0.27616E+00_JPRB, 0.26060E+00_JPRB, 0.24875E+00_JPRB, 0.23817E+00_JPRB, 0.23144E+00_JPRB, 0.22423E+00_JPRB, 0.22178E+00_JPRB ,&
&  0.22095E+00_JPRB, 0.22400E+00_JPRB, 0.22604E+00_JPRB, 0.23079E+00_JPRB, 0.23548E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.25245E+00_JPRB, 0.25560E+00_JPRB, 0.25798E+00_JPRB, 0.25952E+00_JPRB, 0.26026E+00_JPRB, 0.26160E+00_JPRB, 0.26269E+00_JPRB ,&
&  0.26398E+00_JPRB, 0.26498E+00_JPRB, 0.26707E+00_JPRB, 0.26913E+00_JPRB, 0.27209E+00_JPRB, 0.27518E+00_JPRB, 0.27899E+00_JPRB ,&
&  0.28290E+00_JPRB, 0.28355E+00_JPRB, 0.28012E+00_JPRB, 0.27639E+00_JPRB, 0.27258E+00_JPRB, 0.26605E+00_JPRB, 0.25674E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24397E+00_JPRB, 0.23806E+00_JPRB, 0.23509E+00_JPRB, 0.22969E+00_JPRB, 0.22827E+00_JPRB, 0.22378E+00_JPRB ,&
&  0.22790E+00_JPRB, 0.20742E+00_JPRB, 0.20234E+00_JPRB, 0.18785E+00_JPRB, 0.17708E+00_JPRB, 0.16807E+00_JPRB, 0.16193E+00_JPRB ,&
&  0.15906E+00_JPRB, 0.15609E+00_JPRB, 0.15562E+00_JPRB, 0.15668E+00_JPRB, 0.15913E+00_JPRB, 0.16417E+00_JPRB, 0.17035E+00_JPRB ,&
&  0.18788E+00_JPRB, 0.25104E+00_JPRB, 0.27024E+00_JPRB, 0.27449E+00_JPRB, 0.27906E+00_JPRB, 0.29164E+00_JPRB, 0.30433E+00_JPRB ,&
&  0.31854E+00_JPRB, 0.32401E+00_JPRB, 0.32628E+00_JPRB, 0.32554E+00_JPRB, 0.33450E+00_JPRB, 0.34330E+00_JPRB, 0.34878E+00_JPRB ,&
&  0.35220E+00_JPRB, 0.35577E+00_JPRB, 0.35894E+00_JPRB, 0.35939E+00_JPRB, 0.35267E+00_JPRB, 0.34688E+00_JPRB, 0.33855E+00_JPRB ,&
&  0.33438E+00_JPRB, 0.33072E+00_JPRB, 0.33301E+00_JPRB, 0.33991E+00_JPRB, 0.35001E+00_JPRB, 0.35844E+00_JPRB, 0.36235E+00_JPRB ,&
&  0.36282E+00_JPRB, 0.35871E+00_JPRB, 0.35611E+00_JPRB, 0.35361E+00_JPRB, 0.34894E+00_JPRB, 0.34245E+00_JPRB, 0.33799E+00_JPRB ,&
&  0.33335E+00_JPRB, 0.32334E+00_JPRB, 0.31481E+00_JPRB, 0.30767E+00_JPRB, 0.30164E+00_JPRB, 0.29564E+00_JPRB, 0.29018E+00_JPRB ,&
&  0.28419E+00_JPRB, 0.27720E+00_JPRB, 0.27006E+00_JPRB, 0.26121E+00_JPRB, 0.25387E+00_JPRB, 0.24673E+00_JPRB, 0.24345E+00_JPRB ,&
&  0.24098E+00_JPRB, 0.24209E+00_JPRB, 0.24246E+00_JPRB, 0.24535E+00_JPRB, 0.24806E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.26539E+00_JPRB, 0.26567E+00_JPRB, 0.26623E+00_JPRB, 0.26647E+00_JPRB, 0.26709E+00_JPRB, 0.26745E+00_JPRB, 0.26908E+00_JPRB ,&
&  0.27074E+00_JPRB, 0.27251E+00_JPRB, 0.27330E+00_JPRB, 0.27387E+00_JPRB, 0.27487E+00_JPRB, 0.27622E+00_JPRB, 0.27768E+00_JPRB ,&
&  0.27861E+00_JPRB, 0.27778E+00_JPRB, 0.27499E+00_JPRB, 0.27155E+00_JPRB, 0.26716E+00_JPRB, 0.26130E+00_JPRB, 0.25364E+00_JPRB ,&
&  0.24422E+00_JPRB, 0.23581E+00_JPRB, 0.22230E+00_JPRB, 0.21769E+00_JPRB, 0.20986E+00_JPRB, 0.20473E+00_JPRB, 0.20137E+00_JPRB ,&
&  0.19775E+00_JPRB, 0.19390E+00_JPRB, 0.18996E+00_JPRB, 0.18447E+00_JPRB, 0.18016E+00_JPRB, 0.17300E+00_JPRB, 0.16861E+00_JPRB ,&
&  0.16515E+00_JPRB, 0.16314E+00_JPRB, 0.16284E+00_JPRB, 0.16342E+00_JPRB, 0.16634E+00_JPRB, 0.16936E+00_JPRB, 0.17747E+00_JPRB ,&
&  0.19661E+00_JPRB, 0.22573E+00_JPRB, 0.26826E+00_JPRB, 0.27569E+00_JPRB, 0.27515E+00_JPRB, 0.28140E+00_JPRB, 0.28756E+00_JPRB ,&
&  0.29425E+00_JPRB, 0.29649E+00_JPRB, 0.29816E+00_JPRB, 0.29922E+00_JPRB, 0.30178E+00_JPRB, 0.30408E+00_JPRB, 0.30562E+00_JPRB ,&
&  0.30673E+00_JPRB, 0.30969E+00_JPRB, 0.31472E+00_JPRB, 0.32002E+00_JPRB, 0.32372E+00_JPRB, 0.32473E+00_JPRB, 0.32410E+00_JPRB ,&
&  0.32331E+00_JPRB, 0.32270E+00_JPRB, 0.32539E+00_JPRB, 0.33032E+00_JPRB, 0.33788E+00_JPRB, 0.34483E+00_JPRB, 0.35070E+00_JPRB ,&
&  0.35411E+00_JPRB, 0.34917E+00_JPRB, 0.34153E+00_JPRB, 0.33142E+00_JPRB, 0.32444E+00_JPRB, 0.32316E+00_JPRB, 0.31951E+00_JPRB ,&
&  0.31662E+00_JPRB, 0.31304E+00_JPRB, 0.30844E+00_JPRB, 0.30265E+00_JPRB, 0.29811E+00_JPRB, 0.29387E+00_JPRB, 0.29087E+00_JPRB ,&
&  0.28789E+00_JPRB, 0.28512E+00_JPRB, 0.28212E+00_JPRB, 0.27809E+00_JPRB, 0.27440E+00_JPRB, 0.27070E+00_JPRB, 0.26840E+00_JPRB ,&
&  0.26572E+00_JPRB, 0.26480E+00_JPRB, 0.26397E+00_JPRB, 0.26454E+00_JPRB, 0.26466E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.28655E+00_JPRB, 0.28621E+00_JPRB, 0.28603E+00_JPRB, 0.28570E+00_JPRB, 0.28526E+00_JPRB, 0.28432E+00_JPRB, 0.28310E+00_JPRB ,&
&  0.28163E+00_JPRB, 0.28009E+00_JPRB, 0.27823E+00_JPRB, 0.27621E+00_JPRB, 0.27393E+00_JPRB, 0.27214E+00_JPRB, 0.27087E+00_JPRB ,&
&  0.27013E+00_JPRB, 0.26864E+00_JPRB, 0.26626E+00_JPRB, 0.26256E+00_JPRB, 0.25948E+00_JPRB, 0.25712E+00_JPRB, 0.25456E+00_JPRB ,&
&  0.24911E+00_JPRB, 0.24348E+00_JPRB, 0.23831E+00_JPRB, 0.23198E+00_JPRB, 0.22686E+00_JPRB, 0.21076E+00_JPRB, 0.20635E+00_JPRB ,&
&  0.20382E+00_JPRB, 0.20161E+00_JPRB, 0.20137E+00_JPRB, 0.20147E+00_JPRB, 0.20393E+00_JPRB, 0.20091E+00_JPRB, 0.19895E+00_JPRB ,&
&  0.19012E+00_JPRB, 0.19044E+00_JPRB, 0.19187E+00_JPRB, 0.19368E+00_JPRB, 0.19851E+00_JPRB, 0.20356E+00_JPRB, 0.21326E+00_JPRB ,&
&  0.23164E+00_JPRB, 0.23962E+00_JPRB, 0.25221E+00_JPRB, 0.26201E+00_JPRB, 0.26722E+00_JPRB, 0.26964E+00_JPRB, 0.27181E+00_JPRB ,&
&  0.27453E+00_JPRB, 0.27856E+00_JPRB, 0.28077E+00_JPRB, 0.28204E+00_JPRB, 0.28126E+00_JPRB, 0.27926E+00_JPRB, 0.27757E+00_JPRB ,&
&  0.27500E+00_JPRB, 0.27383E+00_JPRB, 0.27409E+00_JPRB, 0.27669E+00_JPRB, 0.28093E+00_JPRB, 0.28546E+00_JPRB, 0.29091E+00_JPRB ,&
&  0.29603E+00_JPRB, 0.30233E+00_JPRB, 0.30743E+00_JPRB, 0.31299E+00_JPRB, 0.31993E+00_JPRB, 0.32680E+00_JPRB, 0.33174E+00_JPRB ,&
&  0.33503E+00_JPRB, 0.33307E+00_JPRB, 0.32794E+00_JPRB, 0.32169E+00_JPRB, 0.31487E+00_JPRB, 0.30940E+00_JPRB, 0.30628E+00_JPRB ,&
&  0.30389E+00_JPRB, 0.30250E+00_JPRB, 0.30158E+00_JPRB, 0.29984E+00_JPRB, 0.29848E+00_JPRB, 0.29663E+00_JPRB, 0.29484E+00_JPRB ,&
&  0.29283E+00_JPRB, 0.29127E+00_JPRB, 0.29034E+00_JPRB, 0.28984E+00_JPRB, 0.29002E+00_JPRB, 0.28942E+00_JPRB, 0.28901E+00_JPRB ,&
&  0.28806E+00_JPRB, 0.28823E+00_JPRB, 0.28748E+00_JPRB, 0.28712E+00_JPRB, 0.28659E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.27974E+00_JPRB, 0.27831E+00_JPRB, 0.27692E+00_JPRB, 0.27578E+00_JPRB, 0.27454E+00_JPRB, 0.27302E+00_JPRB, 0.27157E+00_JPRB ,&
&  0.27013E+00_JPRB, 0.26865E+00_JPRB, 0.26681E+00_JPRB, 0.26492E+00_JPRB, 0.26288E+00_JPRB, 0.26140E+00_JPRB, 0.26032E+00_JPRB ,&
&  0.25955E+00_JPRB, 0.25856E+00_JPRB, 0.25773E+00_JPRB, 0.25619E+00_JPRB, 0.25467E+00_JPRB, 0.25348E+00_JPRB, 0.25310E+00_JPRB ,&
&  0.25187E+00_JPRB, 0.25042E+00_JPRB, 0.24883E+00_JPRB, 0.24687E+00_JPRB, 0.24498E+00_JPRB, 0.24301E+00_JPRB, 0.24076E+00_JPRB ,&
&  0.23819E+00_JPRB, 0.23666E+00_JPRB, 0.23535E+00_JPRB, 0.23377E+00_JPRB, 0.23223E+00_JPRB, 0.23230E+00_JPRB, 0.23215E+00_JPRB ,&
&  0.23206E+00_JPRB, 0.23222E+00_JPRB, 0.23427E+00_JPRB, 0.23624E+00_JPRB, 0.23831E+00_JPRB, 0.24028E+00_JPRB, 0.24354E+00_JPRB ,&
&  0.24655E+00_JPRB, 0.25028E+00_JPRB, 0.25491E+00_JPRB, 0.25920E+00_JPRB, 0.26245E+00_JPRB, 0.26438E+00_JPRB, 0.26705E+00_JPRB ,&
&  0.26874E+00_JPRB, 0.27107E+00_JPRB, 0.27247E+00_JPRB, 0.27265E+00_JPRB, 0.27132E+00_JPRB, 0.26993E+00_JPRB, 0.26901E+00_JPRB ,&
&  0.26762E+00_JPRB, 0.26740E+00_JPRB, 0.26720E+00_JPRB, 0.26709E+00_JPRB, 0.26616E+00_JPRB, 0.26611E+00_JPRB, 0.26579E+00_JPRB ,&
&  0.26669E+00_JPRB, 0.26920E+00_JPRB, 0.27153E+00_JPRB, 0.27432E+00_JPRB, 0.27794E+00_JPRB, 0.28284E+00_JPRB, 0.28626E+00_JPRB ,&
&  0.28938E+00_JPRB, 0.29077E+00_JPRB, 0.29105E+00_JPRB, 0.28951E+00_JPRB, 0.28792E+00_JPRB, 0.28643E+00_JPRB, 0.28607E+00_JPRB ,&
&  0.28528E+00_JPRB, 0.28488E+00_JPRB, 0.28474E+00_JPRB, 0.28460E+00_JPRB, 0.28391E+00_JPRB, 0.28332E+00_JPRB, 0.28308E+00_JPRB ,&
&  0.28254E+00_JPRB, 0.28225E+00_JPRB, 0.28219E+00_JPRB, 0.28254E+00_JPRB, 0.28266E+00_JPRB, 0.28276E+00_JPRB, 0.28294E+00_JPRB ,&
&  0.28315E+00_JPRB, 0.28280E+00_JPRB, 0.28227E+00_JPRB, 0.28167E+00_JPRB, 0.28104E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.25891E+00_JPRB, 0.25872E+00_JPRB, 0.25819E+00_JPRB, 0.25797E+00_JPRB, 0.25759E+00_JPRB, 0.25760E+00_JPRB, 0.25731E+00_JPRB ,&
&  0.25712E+00_JPRB, 0.25696E+00_JPRB, 0.25706E+00_JPRB, 0.25670E+00_JPRB, 0.25654E+00_JPRB, 0.25632E+00_JPRB, 0.25621E+00_JPRB ,&
&  0.25590E+00_JPRB, 0.25578E+00_JPRB, 0.25560E+00_JPRB, 0.25540E+00_JPRB, 0.25529E+00_JPRB, 0.25511E+00_JPRB, 0.25453E+00_JPRB ,&
&  0.25328E+00_JPRB, 0.25183E+00_JPRB, 0.25068E+00_JPRB, 0.24990E+00_JPRB, 0.24961E+00_JPRB, 0.24951E+00_JPRB, 0.24940E+00_JPRB ,&
&  0.24950E+00_JPRB, 0.24989E+00_JPRB, 0.25027E+00_JPRB, 0.25053E+00_JPRB, 0.25072E+00_JPRB, 0.25135E+00_JPRB, 0.25179E+00_JPRB ,&
&  0.25211E+00_JPRB, 0.25238E+00_JPRB, 0.25307E+00_JPRB, 0.25370E+00_JPRB, 0.25439E+00_JPRB, 0.25511E+00_JPRB, 0.25617E+00_JPRB ,&
&  0.25734E+00_JPRB, 0.25834E+00_JPRB, 0.25949E+00_JPRB, 0.26070E+00_JPRB, 0.26183E+00_JPRB, 0.26297E+00_JPRB, 0.26432E+00_JPRB ,&
&  0.26559E+00_JPRB, 0.26674E+00_JPRB, 0.26778E+00_JPRB, 0.26841E+00_JPRB, 0.26871E+00_JPRB, 0.26861E+00_JPRB, 0.26831E+00_JPRB ,&
&  0.26769E+00_JPRB, 0.26735E+00_JPRB, 0.26665E+00_JPRB, 0.26596E+00_JPRB, 0.26506E+00_JPRB, 0.26461E+00_JPRB, 0.26407E+00_JPRB ,&
&  0.26389E+00_JPRB, 0.26358E+00_JPRB, 0.26351E+00_JPRB, 0.26332E+00_JPRB, 0.26341E+00_JPRB, 0.26356E+00_JPRB, 0.26365E+00_JPRB ,&
&  0.26384E+00_JPRB, 0.26401E+00_JPRB, 0.26438E+00_JPRB, 0.26399E+00_JPRB, 0.26359E+00_JPRB, 0.26320E+00_JPRB, 0.26315E+00_JPRB ,&
&  0.26297E+00_JPRB, 0.26278E+00_JPRB, 0.26267E+00_JPRB, 0.26259E+00_JPRB, 0.26250E+00_JPRB, 0.26237E+00_JPRB, 0.26235E+00_JPRB ,&
&  0.26216E+00_JPRB, 0.26215E+00_JPRB, 0.26211E+00_JPRB, 0.26219E+00_JPRB, 0.26186E+00_JPRB, 0.26156E+00_JPRB, 0.26105E+00_JPRB ,&
&  0.26083E+00_JPRB, 0.26035E+00_JPRB, 0.26011E+00_JPRB, 0.25970E+00_JPRB, 0.25943E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB, 0.25545E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB ,&
&  0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB, 0.22308E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.23633E+00_JPRB, 0.23665E+00_JPRB, 0.23692E+00_JPRB, 0.23721E+00_JPRB, 0.23660E+00_JPRB, 0.23730E+00_JPRB, 0.23783E+00_JPRB ,&
&  0.23854E+00_JPRB, 0.23955E+00_JPRB, 0.24077E+00_JPRB, 0.24172E+00_JPRB, 0.24248E+00_JPRB, 0.24293E+00_JPRB, 0.24323E+00_JPRB ,&
&  0.24325E+00_JPRB, 0.24337E+00_JPRB, 0.24069E+00_JPRB, 0.23978E+00_JPRB, 0.23954E+00_JPRB, 0.23966E+00_JPRB, 0.23988E+00_JPRB ,&
&  0.24007E+00_JPRB, 0.24014E+00_JPRB, 0.23981E+00_JPRB, 0.24008E+00_JPRB, 0.24036E+00_JPRB, 0.24058E+00_JPRB, 0.24083E+00_JPRB ,&
&  0.24105E+00_JPRB, 0.24122E+00_JPRB, 0.24116E+00_JPRB, 0.24098E+00_JPRB, 0.24078E+00_JPRB, 0.24023E+00_JPRB, 0.23936E+00_JPRB ,&
&  0.23831E+00_JPRB, 0.23702E+00_JPRB, 0.23574E+00_JPRB, 0.23456E+00_JPRB, 0.23353E+00_JPRB, 0.23265E+00_JPRB, 0.23182E+00_JPRB ,&
&  0.23085E+00_JPRB, 0.22961E+00_JPRB, 0.22830E+00_JPRB, 0.22704E+00_JPRB, 0.22584E+00_JPRB, 0.22474E+00_JPRB, 0.22361E+00_JPRB ,&
&  0.22273E+00_JPRB, 0.22198E+00_JPRB, 0.22122E+00_JPRB, 0.22057E+00_JPRB, 0.21995E+00_JPRB, 0.21936E+00_JPRB, 0.21879E+00_JPRB ,&
&  0.21805E+00_JPRB, 0.21761E+00_JPRB, 0.21711E+00_JPRB, 0.21684E+00_JPRB, 0.21683E+00_JPRB, 0.21678E+00_JPRB, 0.21682E+00_JPRB ,&
&  0.21691E+00_JPRB, 0.21707E+00_JPRB, 0.21716E+00_JPRB, 0.21734E+00_JPRB, 0.21761E+00_JPRB, 0.21798E+00_JPRB, 0.21844E+00_JPRB ,&
&  0.21894E+00_JPRB, 0.21948E+00_JPRB, 0.22008E+00_JPRB, 0.22075E+00_JPRB, 0.22146E+00_JPRB, 0.22224E+00_JPRB, 0.22298E+00_JPRB ,&
&  0.22372E+00_JPRB, 0.22441E+00_JPRB, 0.22510E+00_JPRB, 0.22562E+00_JPRB, 0.22635E+00_JPRB, 0.22706E+00_JPRB, 0.22790E+00_JPRB ,&
&  0.22875E+00_JPRB, 0.22965E+00_JPRB, 0.23043E+00_JPRB, 0.23126E+00_JPRB, 0.23215E+00_JPRB, 0.23209E+00_JPRB, 0.23205E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23237E+00_JPRB, 0.23249E+00_JPRB, 0.23285E+00_JPRB, 0.23365E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.26527E+00_JPRB, 0.27179E+00_JPRB, 0.27688E+00_JPRB, 0.28230E+00_JPRB, 0.28098E+00_JPRB, 0.31793E+00_JPRB, 0.31652E+00_JPRB ,&
&  0.31679E+00_JPRB, 0.30184E+00_JPRB, 0.30127E+00_JPRB, 0.29921E+00_JPRB, 0.29585E+00_JPRB, 0.29111E+00_JPRB, 0.28673E+00_JPRB ,&
&  0.28288E+00_JPRB, 0.27801E+00_JPRB, 0.27246E+00_JPRB, 0.26779E+00_JPRB, 0.26400E+00_JPRB, 0.26075E+00_JPRB, 0.25801E+00_JPRB ,&
&  0.25741E+00_JPRB, 0.25862E+00_JPRB, 0.26099E+00_JPRB, 0.26052E+00_JPRB, 0.26196E+00_JPRB, 0.26423E+00_JPRB, 0.26676E+00_JPRB ,&
&  0.26920E+00_JPRB, 0.27117E+00_JPRB, 0.27129E+00_JPRB, 0.26975E+00_JPRB, 0.26820E+00_JPRB, 0.26567E+00_JPRB, 0.26337E+00_JPRB ,&
&  0.26071E+00_JPRB, 0.25775E+00_JPRB, 0.25433E+00_JPRB, 0.25158E+00_JPRB, 0.24864E+00_JPRB, 0.24504E+00_JPRB, 0.24157E+00_JPRB ,&
&  0.23859E+00_JPRB, 0.23593E+00_JPRB, 0.23316E+00_JPRB, 0.23058E+00_JPRB, 0.22815E+00_JPRB, 0.22566E+00_JPRB, 0.22353E+00_JPRB ,&
&  0.22193E+00_JPRB, 0.22057E+00_JPRB, 0.21920E+00_JPRB, 0.21777E+00_JPRB, 0.21634E+00_JPRB, 0.21495E+00_JPRB, 0.21379E+00_JPRB ,&
&  0.21349E+00_JPRB, 0.21278E+00_JPRB, 0.21208E+00_JPRB, 0.21162E+00_JPRB, 0.21154E+00_JPRB, 0.21104E+00_JPRB, 0.21081E+00_JPRB ,&
&  0.21072E+00_JPRB, 0.21072E+00_JPRB, 0.21094E+00_JPRB, 0.21123E+00_JPRB, 0.21163E+00_JPRB, 0.21215E+00_JPRB, 0.21299E+00_JPRB ,&
&  0.21398E+00_JPRB, 0.21519E+00_JPRB, 0.21647E+00_JPRB, 0.21780E+00_JPRB, 0.21916E+00_JPRB, 0.22060E+00_JPRB, 0.22220E+00_JPRB ,&
&  0.22383E+00_JPRB, 0.22543E+00_JPRB, 0.22701E+00_JPRB, 0.22856E+00_JPRB, 0.23095E+00_JPRB, 0.23329E+00_JPRB, 0.23579E+00_JPRB ,&
&  0.23847E+00_JPRB, 0.24161E+00_JPRB, 0.24491E+00_JPRB, 0.24842E+00_JPRB, 0.25225E+00_JPRB, 0.25424E+00_JPRB, 0.25584E+00_JPRB ,&
&  0.25705E+00_JPRB, 0.25986E+00_JPRB, 0.26251E+00_JPRB, 0.26289E+00_JPRB, 0.26303E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.32383E+00_JPRB, 0.32132E+00_JPRB, 0.32140E+00_JPRB, 0.32190E+00_JPRB, 0.32471E+00_JPRB, 0.32936E+00_JPRB, 0.33449E+00_JPRB ,&
&  0.33389E+00_JPRB, 0.33673E+00_JPRB, 0.33606E+00_JPRB, 0.33389E+00_JPRB, 0.33012E+00_JPRB, 0.32696E+00_JPRB, 0.32213E+00_JPRB ,&
&  0.31526E+00_JPRB, 0.30649E+00_JPRB, 0.29790E+00_JPRB, 0.28927E+00_JPRB, 0.28101E+00_JPRB, 0.27242E+00_JPRB, 0.26689E+00_JPRB ,&
&  0.26372E+00_JPRB, 0.26284E+00_JPRB, 0.26481E+00_JPRB, 0.26779E+00_JPRB, 0.27322E+00_JPRB, 0.28059E+00_JPRB, 0.29031E+00_JPRB ,&
&  0.30003E+00_JPRB, 0.31050E+00_JPRB, 0.31588E+00_JPRB, 0.31579E+00_JPRB, 0.31547E+00_JPRB, 0.31332E+00_JPRB, 0.30980E+00_JPRB ,&
&  0.30353E+00_JPRB, 0.29625E+00_JPRB, 0.28800E+00_JPRB, 0.27929E+00_JPRB, 0.27040E+00_JPRB, 0.26172E+00_JPRB, 0.25365E+00_JPRB ,&
&  0.24707E+00_JPRB, 0.24147E+00_JPRB, 0.23644E+00_JPRB, 0.23224E+00_JPRB, 0.22897E+00_JPRB, 0.22651E+00_JPRB, 0.22371E+00_JPRB ,&
&  0.22131E+00_JPRB, 0.21921E+00_JPRB, 0.21674E+00_JPRB, 0.21500E+00_JPRB, 0.21344E+00_JPRB, 0.21241E+00_JPRB, 0.21155E+00_JPRB ,&
&  0.21134E+00_JPRB, 0.21112E+00_JPRB, 0.21148E+00_JPRB, 0.21183E+00_JPRB, 0.21228E+00_JPRB, 0.21209E+00_JPRB, 0.21176E+00_JPRB ,&
&  0.21128E+00_JPRB, 0.21086E+00_JPRB, 0.20969E+00_JPRB, 0.20882E+00_JPRB, 0.20824E+00_JPRB, 0.20760E+00_JPRB, 0.20766E+00_JPRB ,&
&  0.20853E+00_JPRB, 0.20943E+00_JPRB, 0.21022E+00_JPRB, 0.21176E+00_JPRB, 0.21331E+00_JPRB, 0.21551E+00_JPRB, 0.21812E+00_JPRB ,&
&  0.22007E+00_JPRB, 0.22236E+00_JPRB, 0.22494E+00_JPRB, 0.22791E+00_JPRB, 0.23142E+00_JPRB, 0.23510E+00_JPRB, 0.23937E+00_JPRB ,&
&  0.24417E+00_JPRB, 0.24916E+00_JPRB, 0.25480E+00_JPRB, 0.26099E+00_JPRB, 0.26852E+00_JPRB, 0.27608E+00_JPRB, 0.28677E+00_JPRB ,&
&  0.29566E+00_JPRB, 0.29755E+00_JPRB, 0.33779E+00_JPRB, 0.33148E+00_JPRB, 0.32708E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.31924E+00_JPRB, 0.32152E+00_JPRB, 0.32473E+00_JPRB, 0.32858E+00_JPRB, 0.33341E+00_JPRB, 0.33761E+00_JPRB, 0.34305E+00_JPRB ,&
&  0.34601E+00_JPRB, 0.34198E+00_JPRB, 0.34214E+00_JPRB, 0.33828E+00_JPRB, 0.33425E+00_JPRB, 0.32919E+00_JPRB, 0.32333E+00_JPRB ,&
&  0.31692E+00_JPRB, 0.30805E+00_JPRB, 0.29856E+00_JPRB, 0.29008E+00_JPRB, 0.28300E+00_JPRB, 0.27729E+00_JPRB, 0.27257E+00_JPRB ,&
&  0.26935E+00_JPRB, 0.26940E+00_JPRB, 0.27155E+00_JPRB, 0.27753E+00_JPRB, 0.28757E+00_JPRB, 0.30032E+00_JPRB, 0.31942E+00_JPRB ,&
&  0.34664E+00_JPRB, 0.34883E+00_JPRB, 0.35308E+00_JPRB, 0.35202E+00_JPRB, 0.34782E+00_JPRB, 0.34551E+00_JPRB, 0.34095E+00_JPRB ,&
&  0.34137E+00_JPRB, 0.33515E+00_JPRB, 0.34511E+00_JPRB, 0.33001E+00_JPRB, 0.30456E+00_JPRB, 0.28926E+00_JPRB, 0.27122E+00_JPRB ,&
&  0.25748E+00_JPRB, 0.24586E+00_JPRB, 0.23707E+00_JPRB, 0.23303E+00_JPRB, 0.22952E+00_JPRB, 0.22781E+00_JPRB, 0.22449E+00_JPRB ,&
&  0.22094E+00_JPRB, 0.21895E+00_JPRB, 0.21595E+00_JPRB, 0.21551E+00_JPRB, 0.21548E+00_JPRB, 0.21496E+00_JPRB, 0.21504E+00_JPRB ,&
&  0.21540E+00_JPRB, 0.21623E+00_JPRB, 0.21769E+00_JPRB, 0.21905E+00_JPRB, 0.21985E+00_JPRB, 0.22031E+00_JPRB, 0.22019E+00_JPRB ,&
&  0.22054E+00_JPRB, 0.22024E+00_JPRB, 0.21945E+00_JPRB, 0.21820E+00_JPRB, 0.21640E+00_JPRB, 0.21455E+00_JPRB, 0.21281E+00_JPRB ,&
&  0.21145E+00_JPRB, 0.21102E+00_JPRB, 0.21072E+00_JPRB, 0.21105E+00_JPRB, 0.21173E+00_JPRB, 0.21343E+00_JPRB, 0.21505E+00_JPRB ,&
&  0.21688E+00_JPRB, 0.21934E+00_JPRB, 0.22154E+00_JPRB, 0.22420E+00_JPRB, 0.22671E+00_JPRB, 0.22838E+00_JPRB, 0.23226E+00_JPRB ,&
&  0.23719E+00_JPRB, 0.24364E+00_JPRB, 0.25114E+00_JPRB, 0.25860E+00_JPRB, 0.26816E+00_JPRB, 0.28389E+00_JPRB, 0.30555E+00_JPRB ,&
&  0.32808E+00_JPRB, 0.35384E+00_JPRB, 0.34894E+00_JPRB, 0.31945E+00_JPRB, 0.31684E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.32899E+00_JPRB, 0.33538E+00_JPRB, 0.34240E+00_JPRB, 0.34986E+00_JPRB, 0.35917E+00_JPRB, 0.36308E+00_JPRB, 0.36506E+00_JPRB ,&
&  0.36113E+00_JPRB, 0.35617E+00_JPRB, 0.34514E+00_JPRB, 0.33449E+00_JPRB, 0.32481E+00_JPRB, 0.31701E+00_JPRB, 0.31147E+00_JPRB ,&
&  0.30858E+00_JPRB, 0.30634E+00_JPRB, 0.30321E+00_JPRB, 0.30008E+00_JPRB, 0.29756E+00_JPRB, 0.29747E+00_JPRB, 0.29747E+00_JPRB ,&
&  0.29650E+00_JPRB, 0.29642E+00_JPRB, 0.29531E+00_JPRB, 0.30052E+00_JPRB, 0.30954E+00_JPRB, 0.32079E+00_JPRB, 0.33543E+00_JPRB ,&
&  0.35596E+00_JPRB, 0.36737E+00_JPRB, 0.38289E+00_JPRB, 0.38364E+00_JPRB, 0.38173E+00_JPRB, 0.37785E+00_JPRB, 0.37478E+00_JPRB ,&
&  0.37023E+00_JPRB, 0.36372E+00_JPRB, 0.35908E+00_JPRB, 0.35346E+00_JPRB, 0.32501E+00_JPRB, 0.30223E+00_JPRB, 0.28487E+00_JPRB ,&
&  0.26946E+00_JPRB, 0.25494E+00_JPRB, 0.24241E+00_JPRB, 0.23601E+00_JPRB, 0.23116E+00_JPRB, 0.22830E+00_JPRB, 0.22534E+00_JPRB ,&
&  0.22220E+00_JPRB, 0.22085E+00_JPRB, 0.21891E+00_JPRB, 0.21823E+00_JPRB, 0.21914E+00_JPRB, 0.21921E+00_JPRB, 0.22001E+00_JPRB ,&
&  0.22066E+00_JPRB, 0.22155E+00_JPRB, 0.22358E+00_JPRB, 0.22514E+00_JPRB, 0.22679E+00_JPRB, 0.22840E+00_JPRB, 0.22929E+00_JPRB ,&
&  0.23094E+00_JPRB, 0.23229E+00_JPRB, 0.23225E+00_JPRB, 0.23156E+00_JPRB, 0.22991E+00_JPRB, 0.22642E+00_JPRB, 0.22287E+00_JPRB ,&
&  0.21913E+00_JPRB, 0.21641E+00_JPRB, 0.21519E+00_JPRB, 0.21427E+00_JPRB, 0.21501E+00_JPRB, 0.21584E+00_JPRB, 0.21686E+00_JPRB ,&
&  0.21869E+00_JPRB, 0.22038E+00_JPRB, 0.22227E+00_JPRB, 0.22484E+00_JPRB, 0.22691E+00_JPRB, 0.22875E+00_JPRB, 0.23114E+00_JPRB ,&
&  0.23179E+00_JPRB, 0.23587E+00_JPRB, 0.24098E+00_JPRB, 0.24721E+00_JPRB, 0.25736E+00_JPRB, 0.26703E+00_JPRB, 0.28909E+00_JPRB ,&
&  0.32634E+00_JPRB, 0.39730E+00_JPRB, 0.39211E+00_JPRB, 0.32170E+00_JPRB, 0.32350E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.35166E+00_JPRB, 0.36018E+00_JPRB, 0.36782E+00_JPRB, 0.37367E+00_JPRB, 0.37717E+00_JPRB, 0.37772E+00_JPRB, 0.38028E+00_JPRB ,&
&  0.38756E+00_JPRB, 0.39177E+00_JPRB, 0.39454E+00_JPRB, 0.39404E+00_JPRB, 0.39345E+00_JPRB, 0.37427E+00_JPRB, 0.36898E+00_JPRB ,&
&  0.36325E+00_JPRB, 0.35927E+00_JPRB, 0.35399E+00_JPRB, 0.35411E+00_JPRB, 0.35402E+00_JPRB, 0.35591E+00_JPRB, 0.35663E+00_JPRB ,&
&  0.35642E+00_JPRB, 0.35403E+00_JPRB, 0.35073E+00_JPRB, 0.34869E+00_JPRB, 0.35003E+00_JPRB, 0.35170E+00_JPRB, 0.34895E+00_JPRB ,&
&  0.34486E+00_JPRB, 0.34408E+00_JPRB, 0.34745E+00_JPRB, 0.35518E+00_JPRB, 0.39636E+00_JPRB, 0.39162E+00_JPRB, 0.38886E+00_JPRB ,&
&  0.38558E+00_JPRB, 0.38151E+00_JPRB, 0.38111E+00_JPRB, 0.37486E+00_JPRB, 0.36557E+00_JPRB, 0.34556E+00_JPRB, 0.33433E+00_JPRB ,&
&  0.29959E+00_JPRB, 0.28010E+00_JPRB, 0.26290E+00_JPRB, 0.25161E+00_JPRB, 0.24192E+00_JPRB, 0.23632E+00_JPRB, 0.23116E+00_JPRB ,&
&  0.22727E+00_JPRB, 0.22429E+00_JPRB, 0.22164E+00_JPRB, 0.22197E+00_JPRB, 0.22357E+00_JPRB, 0.22461E+00_JPRB, 0.22561E+00_JPRB ,&
&  0.22622E+00_JPRB, 0.22670E+00_JPRB, 0.22734E+00_JPRB, 0.22855E+00_JPRB, 0.23014E+00_JPRB, 0.23224E+00_JPRB, 0.23487E+00_JPRB ,&
&  0.23923E+00_JPRB, 0.24273E+00_JPRB, 0.24429E+00_JPRB, 0.24665E+00_JPRB, 0.24631E+00_JPRB, 0.24122E+00_JPRB, 0.23508E+00_JPRB ,&
&  0.22881E+00_JPRB, 0.22550E+00_JPRB, 0.22233E+00_JPRB, 0.22015E+00_JPRB, 0.21965E+00_JPRB, 0.21973E+00_JPRB, 0.22157E+00_JPRB ,&
&  0.22356E+00_JPRB, 0.22527E+00_JPRB, 0.22644E+00_JPRB, 0.22773E+00_JPRB, 0.22940E+00_JPRB, 0.23074E+00_JPRB, 0.23251E+00_JPRB ,&
&  0.23474E+00_JPRB, 0.23782E+00_JPRB, 0.24238E+00_JPRB, 0.24762E+00_JPRB, 0.25400E+00_JPRB, 0.26392E+00_JPRB, 0.28184E+00_JPRB ,&
&  0.30651E+00_JPRB, 0.36353E+00_JPRB, 0.37014E+00_JPRB, 0.33870E+00_JPRB, 0.34441E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.36406E+00_JPRB, 0.37366E+00_JPRB, 0.37885E+00_JPRB, 0.38394E+00_JPRB, 0.38595E+00_JPRB, 0.38728E+00_JPRB, 0.39509E+00_JPRB ,&
&  0.39984E+00_JPRB, 0.40346E+00_JPRB, 0.40556E+00_JPRB, 0.40600E+00_JPRB, 0.40132E+00_JPRB, 0.39454E+00_JPRB, 0.39207E+00_JPRB ,&
&  0.39083E+00_JPRB, 0.39131E+00_JPRB, 0.39021E+00_JPRB, 0.39027E+00_JPRB, 0.39080E+00_JPRB, 0.39559E+00_JPRB, 0.40225E+00_JPRB ,&
&  0.40379E+00_JPRB, 0.40852E+00_JPRB, 0.42084E+00_JPRB, 0.42860E+00_JPRB, 0.43353E+00_JPRB, 0.43345E+00_JPRB, 0.42802E+00_JPRB ,&
&  0.40170E+00_JPRB, 0.37414E+00_JPRB, 0.34623E+00_JPRB, 0.34787E+00_JPRB, 0.40221E+00_JPRB, 0.40894E+00_JPRB, 0.40786E+00_JPRB ,&
&  0.40626E+00_JPRB, 0.41076E+00_JPRB, 0.41093E+00_JPRB, 0.39652E+00_JPRB, 0.38596E+00_JPRB, 0.37110E+00_JPRB, 0.35857E+00_JPRB ,&
&  0.34826E+00_JPRB, 0.34045E+00_JPRB, 0.32763E+00_JPRB, 0.30091E+00_JPRB, 0.28291E+00_JPRB, 0.27372E+00_JPRB, 0.26010E+00_JPRB ,&
&  0.25000E+00_JPRB, 0.24532E+00_JPRB, 0.24415E+00_JPRB, 0.24411E+00_JPRB, 0.24827E+00_JPRB, 0.25014E+00_JPRB, 0.24928E+00_JPRB ,&
&  0.24665E+00_JPRB, 0.24438E+00_JPRB, 0.24100E+00_JPRB, 0.23607E+00_JPRB, 0.23338E+00_JPRB, 0.23240E+00_JPRB, 0.23262E+00_JPRB ,&
&  0.23595E+00_JPRB, 0.24150E+00_JPRB, 0.25406E+00_JPRB, 0.26314E+00_JPRB, 0.26077E+00_JPRB, 0.25004E+00_JPRB, 0.23949E+00_JPRB ,&
&  0.23299E+00_JPRB, 0.23024E+00_JPRB, 0.22823E+00_JPRB, 0.22622E+00_JPRB, 0.22676E+00_JPRB, 0.22860E+00_JPRB, 0.23013E+00_JPRB ,&
&  0.23055E+00_JPRB, 0.23201E+00_JPRB, 0.23279E+00_JPRB, 0.23379E+00_JPRB, 0.23465E+00_JPRB, 0.23610E+00_JPRB, 0.23814E+00_JPRB ,&
&  0.24023E+00_JPRB, 0.24522E+00_JPRB, 0.25090E+00_JPRB, 0.25719E+00_JPRB, 0.26110E+00_JPRB, 0.26401E+00_JPRB, 0.26947E+00_JPRB ,&
&  0.27582E+00_JPRB, 0.29147E+00_JPRB, 0.32155E+00_JPRB, 0.35964E+00_JPRB, 0.35563E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.37359E+00_JPRB, 0.38052E+00_JPRB, 0.38861E+00_JPRB, 0.39935E+00_JPRB, 0.40175E+00_JPRB, 0.40340E+00_JPRB, 0.41263E+00_JPRB ,&
&  0.41330E+00_JPRB, 0.41050E+00_JPRB, 0.40716E+00_JPRB, 0.40711E+00_JPRB, 0.40883E+00_JPRB, 0.41087E+00_JPRB, 0.41000E+00_JPRB ,&
&  0.40064E+00_JPRB, 0.39519E+00_JPRB, 0.39763E+00_JPRB, 0.40037E+00_JPRB, 0.40474E+00_JPRB, 0.41124E+00_JPRB, 0.41679E+00_JPRB ,&
&  0.41970E+00_JPRB, 0.42311E+00_JPRB, 0.42528E+00_JPRB, 0.43131E+00_JPRB, 0.44085E+00_JPRB, 0.44738E+00_JPRB, 0.45397E+00_JPRB ,&
&  0.44114E+00_JPRB, 0.39376E+00_JPRB, 0.36295E+00_JPRB, 0.36975E+00_JPRB, 0.42090E+00_JPRB, 0.43084E+00_JPRB, 0.43173E+00_JPRB ,&
&  0.43493E+00_JPRB, 0.44267E+00_JPRB, 0.44208E+00_JPRB, 0.43607E+00_JPRB, 0.42442E+00_JPRB, 0.40574E+00_JPRB, 0.38283E+00_JPRB ,&
&  0.36745E+00_JPRB, 0.35702E+00_JPRB, 0.35032E+00_JPRB, 0.34397E+00_JPRB, 0.34058E+00_JPRB, 0.33566E+00_JPRB, 0.32815E+00_JPRB ,&
&  0.32528E+00_JPRB, 0.32373E+00_JPRB, 0.32466E+00_JPRB, 0.32651E+00_JPRB, 0.32876E+00_JPRB, 0.33051E+00_JPRB, 0.32881E+00_JPRB ,&
&  0.31644E+00_JPRB, 0.30128E+00_JPRB, 0.28677E+00_JPRB, 0.27098E+00_JPRB, 0.25731E+00_JPRB, 0.24846E+00_JPRB, 0.24152E+00_JPRB ,&
&  0.23858E+00_JPRB, 0.24350E+00_JPRB, 0.25907E+00_JPRB, 0.27748E+00_JPRB, 0.31104E+00_JPRB, 0.28717E+00_JPRB, 0.26355E+00_JPRB ,&
&  0.25158E+00_JPRB, 0.24584E+00_JPRB, 0.24263E+00_JPRB, 0.24237E+00_JPRB, 0.24377E+00_JPRB, 0.24419E+00_JPRB, 0.24423E+00_JPRB ,&
&  0.24530E+00_JPRB, 0.24744E+00_JPRB, 0.24926E+00_JPRB, 0.25077E+00_JPRB, 0.25313E+00_JPRB, 0.25589E+00_JPRB, 0.25734E+00_JPRB ,&
&  0.25792E+00_JPRB, 0.26218E+00_JPRB, 0.27125E+00_JPRB, 0.28169E+00_JPRB, 0.29234E+00_JPRB, 0.30346E+00_JPRB, 0.32126E+00_JPRB ,&
&  0.34383E+00_JPRB, 0.37180E+00_JPRB, 0.37059E+00_JPRB, 0.36996E+00_JPRB, 0.37027E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.39491E+00_JPRB, 0.40596E+00_JPRB, 0.41907E+00_JPRB, 0.42844E+00_JPRB, 0.42771E+00_JPRB, 0.42787E+00_JPRB, 0.43423E+00_JPRB ,&
&  0.42872E+00_JPRB, 0.42182E+00_JPRB, 0.41584E+00_JPRB, 0.40955E+00_JPRB, 0.40709E+00_JPRB, 0.40549E+00_JPRB, 0.40823E+00_JPRB ,&
&  0.41019E+00_JPRB, 0.40895E+00_JPRB, 0.40714E+00_JPRB, 0.40769E+00_JPRB, 0.41402E+00_JPRB, 0.42205E+00_JPRB, 0.42877E+00_JPRB ,&
&  0.42986E+00_JPRB, 0.43317E+00_JPRB, 0.43863E+00_JPRB, 0.44329E+00_JPRB, 0.44575E+00_JPRB, 0.44738E+00_JPRB, 0.44636E+00_JPRB ,&
&  0.44407E+00_JPRB, 0.44275E+00_JPRB, 0.42593E+00_JPRB, 0.44038E+00_JPRB, 0.44485E+00_JPRB, 0.44586E+00_JPRB, 0.45192E+00_JPRB ,&
&  0.45998E+00_JPRB, 0.46865E+00_JPRB, 0.47252E+00_JPRB, 0.47364E+00_JPRB, 0.46904E+00_JPRB, 0.45987E+00_JPRB, 0.43598E+00_JPRB ,&
&  0.41595E+00_JPRB, 0.39502E+00_JPRB, 0.38198E+00_JPRB, 0.37418E+00_JPRB, 0.36819E+00_JPRB, 0.36287E+00_JPRB, 0.35753E+00_JPRB ,&
&  0.35016E+00_JPRB, 0.34360E+00_JPRB, 0.33964E+00_JPRB, 0.33984E+00_JPRB, 0.34418E+00_JPRB, 0.35517E+00_JPRB, 0.36879E+00_JPRB ,&
&  0.37500E+00_JPRB, 0.36803E+00_JPRB, 0.36029E+00_JPRB, 0.35198E+00_JPRB, 0.33894E+00_JPRB, 0.31092E+00_JPRB, 0.29407E+00_JPRB ,&
&  0.28967E+00_JPRB, 0.30216E+00_JPRB, 0.32367E+00_JPRB, 0.32923E+00_JPRB, 0.33396E+00_JPRB, 0.33173E+00_JPRB, 0.32653E+00_JPRB ,&
&  0.31858E+00_JPRB, 0.29474E+00_JPRB, 0.28134E+00_JPRB, 0.27721E+00_JPRB, 0.27473E+00_JPRB, 0.27448E+00_JPRB, 0.27686E+00_JPRB ,&
&  0.28079E+00_JPRB, 0.28840E+00_JPRB, 0.29373E+00_JPRB, 0.29593E+00_JPRB, 0.29908E+00_JPRB, 0.29913E+00_JPRB, 0.29802E+00_JPRB ,&
&  0.29417E+00_JPRB, 0.30027E+00_JPRB, 0.31801E+00_JPRB, 0.34702E+00_JPRB, 0.35326E+00_JPRB, 0.36055E+00_JPRB, 0.36952E+00_JPRB ,&
&  0.37480E+00_JPRB, 0.37954E+00_JPRB, 0.37965E+00_JPRB, 0.38063E+00_JPRB, 0.38599E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.41286E+00_JPRB, 0.42528E+00_JPRB, 0.44121E+00_JPRB, 0.45269E+00_JPRB, 0.45785E+00_JPRB, 0.46350E+00_JPRB, 0.46609E+00_JPRB ,&
&  0.46139E+00_JPRB, 0.45950E+00_JPRB, 0.45384E+00_JPRB, 0.44553E+00_JPRB, 0.43831E+00_JPRB, 0.43453E+00_JPRB, 0.43470E+00_JPRB ,&
&  0.43677E+00_JPRB, 0.43855E+00_JPRB, 0.43501E+00_JPRB, 0.43318E+00_JPRB, 0.43697E+00_JPRB, 0.44213E+00_JPRB, 0.44545E+00_JPRB ,&
&  0.44664E+00_JPRB, 0.44788E+00_JPRB, 0.45251E+00_JPRB, 0.45554E+00_JPRB, 0.45581E+00_JPRB, 0.44947E+00_JPRB, 0.44491E+00_JPRB ,&
&  0.44023E+00_JPRB, 0.43875E+00_JPRB, 0.44588E+00_JPRB, 0.43773E+00_JPRB, 0.45283E+00_JPRB, 0.45500E+00_JPRB, 0.46079E+00_JPRB ,&
&  0.46922E+00_JPRB, 0.48279E+00_JPRB, 0.49298E+00_JPRB, 0.50070E+00_JPRB, 0.50321E+00_JPRB, 0.50658E+00_JPRB, 0.49703E+00_JPRB ,&
&  0.48351E+00_JPRB, 0.46504E+00_JPRB, 0.44658E+00_JPRB, 0.42688E+00_JPRB, 0.40839E+00_JPRB, 0.39387E+00_JPRB, 0.38658E+00_JPRB ,&
&  0.37775E+00_JPRB, 0.36403E+00_JPRB, 0.35077E+00_JPRB, 0.34548E+00_JPRB, 0.35002E+00_JPRB, 0.36015E+00_JPRB, 0.37576E+00_JPRB ,&
&  0.38525E+00_JPRB, 0.38816E+00_JPRB, 0.38501E+00_JPRB, 0.38057E+00_JPRB, 0.37603E+00_JPRB, 0.36925E+00_JPRB, 0.36539E+00_JPRB ,&
&  0.36212E+00_JPRB, 0.35639E+00_JPRB, 0.35336E+00_JPRB, 0.35520E+00_JPRB, 0.35842E+00_JPRB, 0.36038E+00_JPRB, 0.35851E+00_JPRB ,&
&  0.35518E+00_JPRB, 0.35084E+00_JPRB, 0.34279E+00_JPRB, 0.33801E+00_JPRB, 0.33781E+00_JPRB, 0.34205E+00_JPRB, 0.34971E+00_JPRB ,&
&  0.35222E+00_JPRB, 0.35801E+00_JPRB, 0.36376E+00_JPRB, 0.36600E+00_JPRB, 0.36868E+00_JPRB, 0.37099E+00_JPRB, 0.37128E+00_JPRB ,&
&  0.36963E+00_JPRB, 0.36463E+00_JPRB, 0.35752E+00_JPRB, 0.35408E+00_JPRB, 0.35473E+00_JPRB, 0.36102E+00_JPRB, 0.36992E+00_JPRB ,&
&  0.37882E+00_JPRB, 0.39220E+00_JPRB, 0.40252E+00_JPRB, 0.40487E+00_JPRB, 0.40703E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.44096E+00_JPRB, 0.44601E+00_JPRB, 0.45885E+00_JPRB, 0.46954E+00_JPRB, 0.47311E+00_JPRB, 0.48269E+00_JPRB, 0.49400E+00_JPRB ,&
&  0.50350E+00_JPRB, 0.50552E+00_JPRB, 0.50174E+00_JPRB, 0.49122E+00_JPRB, 0.47874E+00_JPRB, 0.46846E+00_JPRB, 0.46423E+00_JPRB ,&
&  0.46630E+00_JPRB, 0.46936E+00_JPRB, 0.47490E+00_JPRB, 0.47794E+00_JPRB, 0.48152E+00_JPRB, 0.48026E+00_JPRB, 0.48053E+00_JPRB ,&
&  0.48216E+00_JPRB, 0.48308E+00_JPRB, 0.48361E+00_JPRB, 0.48012E+00_JPRB, 0.47744E+00_JPRB, 0.47520E+00_JPRB, 0.46709E+00_JPRB ,&
&  0.45789E+00_JPRB, 0.45164E+00_JPRB, 0.45573E+00_JPRB, 0.46747E+00_JPRB, 0.47495E+00_JPRB, 0.47222E+00_JPRB, 0.47107E+00_JPRB ,&
&  0.47129E+00_JPRB, 0.47590E+00_JPRB, 0.48335E+00_JPRB, 0.49483E+00_JPRB, 0.50323E+00_JPRB, 0.51443E+00_JPRB, 0.51513E+00_JPRB ,&
&  0.50880E+00_JPRB, 0.49967E+00_JPRB, 0.49690E+00_JPRB, 0.49605E+00_JPRB, 0.46827E+00_JPRB, 0.43765E+00_JPRB, 0.42258E+00_JPRB ,&
&  0.41453E+00_JPRB, 0.40445E+00_JPRB, 0.38874E+00_JPRB, 0.37756E+00_JPRB, 0.37615E+00_JPRB, 0.38065E+00_JPRB, 0.39193E+00_JPRB ,&
&  0.40021E+00_JPRB, 0.41232E+00_JPRB, 0.42678E+00_JPRB, 0.43597E+00_JPRB, 0.43602E+00_JPRB, 0.42428E+00_JPRB, 0.41364E+00_JPRB ,&
&  0.40752E+00_JPRB, 0.40604E+00_JPRB, 0.40468E+00_JPRB, 0.40258E+00_JPRB, 0.40053E+00_JPRB, 0.39995E+00_JPRB, 0.39878E+00_JPRB ,&
&  0.39449E+00_JPRB, 0.38708E+00_JPRB, 0.37801E+00_JPRB, 0.37282E+00_JPRB, 0.37141E+00_JPRB, 0.37095E+00_JPRB, 0.37234E+00_JPRB ,&
&  0.38134E+00_JPRB, 0.39251E+00_JPRB, 0.40172E+00_JPRB, 0.40620E+00_JPRB, 0.40356E+00_JPRB, 0.39593E+00_JPRB, 0.39119E+00_JPRB ,&
&  0.38737E+00_JPRB, 0.38462E+00_JPRB, 0.38111E+00_JPRB, 0.37932E+00_JPRB, 0.37875E+00_JPRB, 0.38539E+00_JPRB, 0.40035E+00_JPRB ,&
&  0.41926E+00_JPRB, 0.44445E+00_JPRB, 0.45598E+00_JPRB, 0.45458E+00_JPRB, 0.44726E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.49427E+00_JPRB, 0.47756E+00_JPRB, 0.47789E+00_JPRB, 0.48483E+00_JPRB, 0.48883E+00_JPRB, 0.49676E+00_JPRB, 0.51634E+00_JPRB ,&
&  0.52803E+00_JPRB, 0.53554E+00_JPRB, 0.53843E+00_JPRB, 0.53290E+00_JPRB, 0.53163E+00_JPRB, 0.52427E+00_JPRB, 0.51151E+00_JPRB ,&
&  0.50623E+00_JPRB, 0.50514E+00_JPRB, 0.50671E+00_JPRB, 0.51036E+00_JPRB, 0.51238E+00_JPRB, 0.51492E+00_JPRB, 0.51782E+00_JPRB ,&
&  0.51936E+00_JPRB, 0.51926E+00_JPRB, 0.51845E+00_JPRB, 0.51875E+00_JPRB, 0.51797E+00_JPRB, 0.51078E+00_JPRB, 0.50268E+00_JPRB ,&
&  0.49369E+00_JPRB, 0.48470E+00_JPRB, 0.48244E+00_JPRB, 0.48717E+00_JPRB, 0.49238E+00_JPRB, 0.49556E+00_JPRB, 0.49336E+00_JPRB ,&
&  0.48859E+00_JPRB, 0.48541E+00_JPRB, 0.48718E+00_JPRB, 0.49408E+00_JPRB, 0.50277E+00_JPRB, 0.51243E+00_JPRB, 0.51749E+00_JPRB ,&
&  0.51210E+00_JPRB, 0.49489E+00_JPRB, 0.48452E+00_JPRB, 0.48427E+00_JPRB, 0.48145E+00_JPRB, 0.46803E+00_JPRB, 0.45843E+00_JPRB ,&
&  0.45371E+00_JPRB, 0.44506E+00_JPRB, 0.43585E+00_JPRB, 0.42310E+00_JPRB, 0.41132E+00_JPRB, 0.40690E+00_JPRB, 0.40544E+00_JPRB ,&
&  0.40819E+00_JPRB, 0.42500E+00_JPRB, 0.45646E+00_JPRB, 0.48786E+00_JPRB, 0.50211E+00_JPRB, 0.49625E+00_JPRB, 0.48035E+00_JPRB ,&
&  0.47354E+00_JPRB, 0.47097E+00_JPRB, 0.47254E+00_JPRB, 0.47206E+00_JPRB, 0.46834E+00_JPRB, 0.45972E+00_JPRB, 0.45287E+00_JPRB ,&
&  0.44296E+00_JPRB, 0.43196E+00_JPRB, 0.42742E+00_JPRB, 0.42454E+00_JPRB, 0.42142E+00_JPRB, 0.42128E+00_JPRB, 0.42005E+00_JPRB ,&
&  0.42013E+00_JPRB, 0.42381E+00_JPRB, 0.43545E+00_JPRB, 0.44239E+00_JPRB, 0.44403E+00_JPRB, 0.43388E+00_JPRB, 0.43451E+00_JPRB ,&
&  0.43673E+00_JPRB, 0.43421E+00_JPRB, 0.42956E+00_JPRB, 0.42920E+00_JPRB, 0.42895E+00_JPRB, 0.43055E+00_JPRB, 0.44163E+00_JPRB ,&
&  0.46783E+00_JPRB, 0.49109E+00_JPRB, 0.51028E+00_JPRB, 0.51590E+00_JPRB, 0.51174E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.56568E+00_JPRB, 0.53750E+00_JPRB, 0.52645E+00_JPRB, 0.52823E+00_JPRB, 0.53177E+00_JPRB, 0.52789E+00_JPRB, 0.53541E+00_JPRB ,&
&  0.54676E+00_JPRB, 0.55526E+00_JPRB, 0.55985E+00_JPRB, 0.55610E+00_JPRB, 0.55356E+00_JPRB, 0.55068E+00_JPRB, 0.53615E+00_JPRB ,&
&  0.53139E+00_JPRB, 0.52960E+00_JPRB, 0.53348E+00_JPRB, 0.54014E+00_JPRB, 0.54731E+00_JPRB, 0.55901E+00_JPRB, 0.56765E+00_JPRB ,&
&  0.57340E+00_JPRB, 0.57701E+00_JPRB, 0.58112E+00_JPRB, 0.58134E+00_JPRB, 0.57800E+00_JPRB, 0.56941E+00_JPRB, 0.56290E+00_JPRB ,&
&  0.55921E+00_JPRB, 0.54792E+00_JPRB, 0.53830E+00_JPRB, 0.53715E+00_JPRB, 0.54032E+00_JPRB, 0.54326E+00_JPRB, 0.54233E+00_JPRB ,&
&  0.52895E+00_JPRB, 0.51198E+00_JPRB, 0.50995E+00_JPRB, 0.50993E+00_JPRB, 0.51531E+00_JPRB, 0.52729E+00_JPRB, 0.53074E+00_JPRB ,&
&  0.52797E+00_JPRB, 0.51448E+00_JPRB, 0.49594E+00_JPRB, 0.49273E+00_JPRB, 0.49559E+00_JPRB, 0.49014E+00_JPRB, 0.48439E+00_JPRB ,&
&  0.48329E+00_JPRB, 0.48006E+00_JPRB, 0.47397E+00_JPRB, 0.46578E+00_JPRB, 0.45322E+00_JPRB, 0.43675E+00_JPRB, 0.42801E+00_JPRB ,&
&  0.42676E+00_JPRB, 0.44035E+00_JPRB, 0.46549E+00_JPRB, 0.49458E+00_JPRB, 0.52272E+00_JPRB, 0.53678E+00_JPRB, 0.53675E+00_JPRB ,&
&  0.52590E+00_JPRB, 0.52396E+00_JPRB, 0.52947E+00_JPRB, 0.53374E+00_JPRB, 0.53439E+00_JPRB, 0.52378E+00_JPRB, 0.50739E+00_JPRB ,&
&  0.49574E+00_JPRB, 0.48679E+00_JPRB, 0.48559E+00_JPRB, 0.49038E+00_JPRB, 0.49527E+00_JPRB, 0.49636E+00_JPRB, 0.48997E+00_JPRB ,&
&  0.48303E+00_JPRB, 0.47605E+00_JPRB, 0.47231E+00_JPRB, 0.47549E+00_JPRB, 0.48140E+00_JPRB, 0.48710E+00_JPRB, 0.49936E+00_JPRB ,&
&  0.51693E+00_JPRB, 0.53483E+00_JPRB, 0.53905E+00_JPRB, 0.53500E+00_JPRB, 0.53467E+00_JPRB, 0.53214E+00_JPRB, 0.52962E+00_JPRB ,&
&  0.53940E+00_JPRB, 0.56121E+00_JPRB, 0.57687E+00_JPRB, 0.58585E+00_JPRB, 0.58625E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.61246E+00_JPRB, 0.58823E+00_JPRB, 0.57066E+00_JPRB, 0.56419E+00_JPRB, 0.56077E+00_JPRB, 0.54913E+00_JPRB, 0.54424E+00_JPRB ,&
&  0.55044E+00_JPRB, 0.55241E+00_JPRB, 0.55966E+00_JPRB, 0.55953E+00_JPRB, 0.55880E+00_JPRB, 0.56121E+00_JPRB, 0.55349E+00_JPRB ,&
&  0.55332E+00_JPRB, 0.56205E+00_JPRB, 0.57226E+00_JPRB, 0.58612E+00_JPRB, 0.59778E+00_JPRB, 0.60664E+00_JPRB, 0.61424E+00_JPRB ,&
&  0.61731E+00_JPRB, 0.62389E+00_JPRB, 0.62759E+00_JPRB, 0.62367E+00_JPRB, 0.61339E+00_JPRB, 0.60554E+00_JPRB, 0.60445E+00_JPRB ,&
&  0.60376E+00_JPRB, 0.59771E+00_JPRB, 0.55407E+00_JPRB, 0.57767E+00_JPRB, 0.58452E+00_JPRB, 0.58690E+00_JPRB, 0.58803E+00_JPRB ,&
&  0.58195E+00_JPRB, 0.56645E+00_JPRB, 0.55947E+00_JPRB, 0.54824E+00_JPRB, 0.54716E+00_JPRB, 0.55306E+00_JPRB, 0.55221E+00_JPRB ,&
&  0.54906E+00_JPRB, 0.53987E+00_JPRB, 0.52680E+00_JPRB, 0.51652E+00_JPRB, 0.51424E+00_JPRB, 0.51299E+00_JPRB, 0.50635E+00_JPRB ,&
&  0.50147E+00_JPRB, 0.50047E+00_JPRB, 0.49859E+00_JPRB, 0.49294E+00_JPRB, 0.47854E+00_JPRB, 0.46496E+00_JPRB, 0.46186E+00_JPRB ,&
&  0.46434E+00_JPRB, 0.47022E+00_JPRB, 0.48290E+00_JPRB, 0.49981E+00_JPRB, 0.51845E+00_JPRB, 0.53624E+00_JPRB, 0.55265E+00_JPRB ,&
&  0.55950E+00_JPRB, 0.56362E+00_JPRB, 0.56720E+00_JPRB, 0.57156E+00_JPRB, 0.57221E+00_JPRB, 0.56463E+00_JPRB, 0.54616E+00_JPRB ,&
&  0.52657E+00_JPRB, 0.52178E+00_JPRB, 0.52601E+00_JPRB, 0.53521E+00_JPRB, 0.54002E+00_JPRB, 0.53937E+00_JPRB, 0.53245E+00_JPRB ,&
&  0.52780E+00_JPRB, 0.51920E+00_JPRB, 0.51059E+00_JPRB, 0.50646E+00_JPRB, 0.50639E+00_JPRB, 0.51257E+00_JPRB, 0.52977E+00_JPRB ,&
&  0.55023E+00_JPRB, 0.57310E+00_JPRB, 0.59385E+00_JPRB, 0.60176E+00_JPRB, 0.60593E+00_JPRB, 0.60545E+00_JPRB, 0.59669E+00_JPRB ,&
&  0.60046E+00_JPRB, 0.61870E+00_JPRB, 0.62803E+00_JPRB, 0.62934E+00_JPRB, 0.62721E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.66177E+00_JPRB, 0.64317E+00_JPRB, 0.63214E+00_JPRB, 0.62118E+00_JPRB, 0.61316E+00_JPRB, 0.59551E+00_JPRB, 0.58554E+00_JPRB ,&
&  0.57938E+00_JPRB, 0.57579E+00_JPRB, 0.57583E+00_JPRB, 0.57657E+00_JPRB, 0.58652E+00_JPRB, 0.59257E+00_JPRB, 0.59454E+00_JPRB ,&
&  0.60159E+00_JPRB, 0.61844E+00_JPRB, 0.63383E+00_JPRB, 0.64693E+00_JPRB, 0.65753E+00_JPRB, 0.66551E+00_JPRB, 0.66587E+00_JPRB ,&
&  0.67011E+00_JPRB, 0.67855E+00_JPRB, 0.68290E+00_JPRB, 0.67380E+00_JPRB, 0.66216E+00_JPRB, 0.65612E+00_JPRB, 0.65610E+00_JPRB ,&
&  0.64680E+00_JPRB, 0.60054E+00_JPRB, 0.61408E+00_JPRB, 0.62529E+00_JPRB, 0.63767E+00_JPRB, 0.64240E+00_JPRB, 0.64743E+00_JPRB ,&
&  0.64812E+00_JPRB, 0.63220E+00_JPRB, 0.61315E+00_JPRB, 0.59926E+00_JPRB, 0.60241E+00_JPRB, 0.60002E+00_JPRB, 0.59194E+00_JPRB ,&
&  0.58939E+00_JPRB, 0.58746E+00_JPRB, 0.58112E+00_JPRB, 0.56895E+00_JPRB, 0.55788E+00_JPRB, 0.55080E+00_JPRB, 0.54646E+00_JPRB ,&
&  0.53618E+00_JPRB, 0.52747E+00_JPRB, 0.52268E+00_JPRB, 0.51733E+00_JPRB, 0.51257E+00_JPRB, 0.51614E+00_JPRB, 0.51886E+00_JPRB ,&
&  0.51692E+00_JPRB, 0.51645E+00_JPRB, 0.52315E+00_JPRB, 0.53620E+00_JPRB, 0.54723E+00_JPRB, 0.56023E+00_JPRB, 0.57581E+00_JPRB ,&
&  0.59100E+00_JPRB, 0.60399E+00_JPRB, 0.60801E+00_JPRB, 0.61203E+00_JPRB, 0.60747E+00_JPRB, 0.59993E+00_JPRB, 0.58415E+00_JPRB ,&
&  0.56702E+00_JPRB, 0.56537E+00_JPRB, 0.56776E+00_JPRB, 0.57484E+00_JPRB, 0.57440E+00_JPRB, 0.57205E+00_JPRB, 0.57015E+00_JPRB ,&
&  0.56601E+00_JPRB, 0.56253E+00_JPRB, 0.55773E+00_JPRB, 0.55307E+00_JPRB, 0.55615E+00_JPRB, 0.56184E+00_JPRB, 0.57508E+00_JPRB ,&
&  0.59275E+00_JPRB, 0.61156E+00_JPRB, 0.63139E+00_JPRB, 0.64552E+00_JPRB, 0.65084E+00_JPRB, 0.65413E+00_JPRB, 0.65709E+00_JPRB ,&
&  0.65770E+00_JPRB, 0.65590E+00_JPRB, 0.66184E+00_JPRB, 0.66621E+00_JPRB, 0.67089E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.69166E+00_JPRB, 0.67518E+00_JPRB, 0.66586E+00_JPRB, 0.66211E+00_JPRB, 0.65371E+00_JPRB, 0.63477E+00_JPRB, 0.61914E+00_JPRB ,&
&  0.60511E+00_JPRB, 0.59812E+00_JPRB, 0.59569E+00_JPRB, 0.59519E+00_JPRB, 0.60080E+00_JPRB, 0.60577E+00_JPRB, 0.61181E+00_JPRB ,&
&  0.62240E+00_JPRB, 0.63984E+00_JPRB, 0.65497E+00_JPRB, 0.66938E+00_JPRB, 0.68259E+00_JPRB, 0.69589E+00_JPRB, 0.70000E+00_JPRB ,&
&  0.70340E+00_JPRB, 0.71132E+00_JPRB, 0.71351E+00_JPRB, 0.70228E+00_JPRB, 0.69007E+00_JPRB, 0.69213E+00_JPRB, 0.68861E+00_JPRB ,&
&  0.68177E+00_JPRB, 0.56917E+00_JPRB, 0.61866E+00_JPRB, 0.63964E+00_JPRB, 0.66331E+00_JPRB, 0.68443E+00_JPRB, 0.69543E+00_JPRB ,&
&  0.69216E+00_JPRB, 0.67790E+00_JPRB, 0.66115E+00_JPRB, 0.63626E+00_JPRB, 0.62860E+00_JPRB, 0.62218E+00_JPRB, 0.61470E+00_JPRB ,&
&  0.60990E+00_JPRB, 0.60507E+00_JPRB, 0.59835E+00_JPRB, 0.58843E+00_JPRB, 0.57527E+00_JPRB, 0.56564E+00_JPRB, 0.55722E+00_JPRB ,&
&  0.55563E+00_JPRB, 0.54890E+00_JPRB, 0.54242E+00_JPRB, 0.54303E+00_JPRB, 0.55428E+00_JPRB, 0.55875E+00_JPRB, 0.55756E+00_JPRB ,&
&  0.55557E+00_JPRB, 0.55420E+00_JPRB, 0.55927E+00_JPRB, 0.57056E+00_JPRB, 0.58058E+00_JPRB, 0.58736E+00_JPRB, 0.59675E+00_JPRB ,&
&  0.61213E+00_JPRB, 0.62399E+00_JPRB, 0.62501E+00_JPRB, 0.61664E+00_JPRB, 0.61234E+00_JPRB, 0.61399E+00_JPRB, 0.60643E+00_JPRB ,&
&  0.59378E+00_JPRB, 0.58762E+00_JPRB, 0.58847E+00_JPRB, 0.58997E+00_JPRB, 0.58805E+00_JPRB, 0.58300E+00_JPRB, 0.58250E+00_JPRB ,&
&  0.58402E+00_JPRB, 0.58487E+00_JPRB, 0.58414E+00_JPRB, 0.58667E+00_JPRB, 0.59241E+00_JPRB, 0.60019E+00_JPRB, 0.61109E+00_JPRB ,&
&  0.62699E+00_JPRB, 0.64173E+00_JPRB, 0.65064E+00_JPRB, 0.66331E+00_JPRB, 0.67810E+00_JPRB, 0.68438E+00_JPRB, 0.69780E+00_JPRB ,&
&  0.69784E+00_JPRB, 0.69340E+00_JPRB, 0.69189E+00_JPRB, 0.69801E+00_JPRB, 0.70146E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.72085E+00_JPRB, 0.70491E+00_JPRB, 0.69713E+00_JPRB, 0.69216E+00_JPRB, 0.68809E+00_JPRB, 0.68132E+00_JPRB, 0.66818E+00_JPRB ,&
&  0.65521E+00_JPRB, 0.64893E+00_JPRB, 0.64318E+00_JPRB, 0.63520E+00_JPRB, 0.63622E+00_JPRB, 0.64170E+00_JPRB, 0.65222E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.67927E+00_JPRB, 0.69411E+00_JPRB, 0.71316E+00_JPRB, 0.72621E+00_JPRB, 0.73567E+00_JPRB, 0.74104E+00_JPRB ,&
&  0.74479E+00_JPRB, 0.74555E+00_JPRB, 0.73893E+00_JPRB, 0.73049E+00_JPRB, 0.72801E+00_JPRB, 0.72822E+00_JPRB, 0.72283E+00_JPRB ,&
&  0.62753E+00_JPRB, 0.54000E+00_JPRB, 0.58973E+00_JPRB, 0.64625E+00_JPRB, 0.69807E+00_JPRB, 0.74110E+00_JPRB, 0.75927E+00_JPRB ,&
&  0.75022E+00_JPRB, 0.72637E+00_JPRB, 0.70350E+00_JPRB, 0.67556E+00_JPRB, 0.66533E+00_JPRB, 0.66165E+00_JPRB, 0.65385E+00_JPRB ,&
&  0.64454E+00_JPRB, 0.63366E+00_JPRB, 0.62082E+00_JPRB, 0.60076E+00_JPRB, 0.58706E+00_JPRB, 0.57709E+00_JPRB, 0.57637E+00_JPRB ,&
&  0.57929E+00_JPRB, 0.58338E+00_JPRB, 0.58679E+00_JPRB, 0.58891E+00_JPRB, 0.59493E+00_JPRB, 0.60243E+00_JPRB, 0.61014E+00_JPRB ,&
&  0.61966E+00_JPRB, 0.62560E+00_JPRB, 0.63699E+00_JPRB, 0.64577E+00_JPRB, 0.65417E+00_JPRB, 0.66042E+00_JPRB, 0.66547E+00_JPRB ,&
&  0.66462E+00_JPRB, 0.66133E+00_JPRB, 0.64717E+00_JPRB, 0.63992E+00_JPRB, 0.64111E+00_JPRB, 0.64368E+00_JPRB, 0.64215E+00_JPRB ,&
&  0.63496E+00_JPRB, 0.62637E+00_JPRB, 0.61793E+00_JPRB, 0.61477E+00_JPRB, 0.61738E+00_JPRB, 0.61723E+00_JPRB, 0.61370E+00_JPRB ,&
&  0.61075E+00_JPRB, 0.61104E+00_JPRB, 0.61681E+00_JPRB, 0.62778E+00_JPRB, 0.64216E+00_JPRB, 0.66000E+00_JPRB, 0.67459E+00_JPRB ,&
&  0.68992E+00_JPRB, 0.70549E+00_JPRB, 0.71743E+00_JPRB, 0.73721E+00_JPRB, 0.75574E+00_JPRB, 0.76428E+00_JPRB, 0.76019E+00_JPRB ,&
&  0.75753E+00_JPRB, 0.74547E+00_JPRB, 0.73896E+00_JPRB, 0.73819E+00_JPRB, 0.73281E+00_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.70059E+00_JPRB, 0.70971E+00_JPRB, 0.71689E+00_JPRB, 0.71489E+00_JPRB, 0.71178E+00_JPRB, 0.70407E+00_JPRB, 0.68881E+00_JPRB ,&
&  0.67460E+00_JPRB, 0.66919E+00_JPRB, 0.66925E+00_JPRB, 0.67073E+00_JPRB, 0.67365E+00_JPRB, 0.67998E+00_JPRB, 0.69141E+00_JPRB ,&
&  0.70508E+00_JPRB, 0.71959E+00_JPRB, 0.73682E+00_JPRB, 0.75302E+00_JPRB, 0.75713E+00_JPRB, 0.75448E+00_JPRB, 0.75525E+00_JPRB ,&
&  0.76190E+00_JPRB, 0.76129E+00_JPRB, 0.75573E+00_JPRB, 0.75884E+00_JPRB, 0.76644E+00_JPRB, 0.75723E+00_JPRB, 0.73887E+00_JPRB ,&
&  0.72578E+00_JPRB, 0.57794E+00_JPRB, 0.60936E+00_JPRB, 0.66644E+00_JPRB, 0.72178E+00_JPRB, 0.77919E+00_JPRB, 0.80368E+00_JPRB ,&
&  0.79827E+00_JPRB, 0.77200E+00_JPRB, 0.74940E+00_JPRB, 0.72081E+00_JPRB, 0.70951E+00_JPRB, 0.69983E+00_JPRB, 0.68662E+00_JPRB ,&
&  0.66744E+00_JPRB, 0.64745E+00_JPRB, 0.62550E+00_JPRB, 0.60702E+00_JPRB, 0.59223E+00_JPRB, 0.58634E+00_JPRB, 0.58550E+00_JPRB ,&
&  0.58682E+00_JPRB, 0.59820E+00_JPRB, 0.60401E+00_JPRB, 0.61244E+00_JPRB, 0.62519E+00_JPRB, 0.64203E+00_JPRB, 0.65910E+00_JPRB ,&
&  0.67337E+00_JPRB, 0.68266E+00_JPRB, 0.69211E+00_JPRB, 0.70022E+00_JPRB, 0.71073E+00_JPRB, 0.71883E+00_JPRB, 0.71386E+00_JPRB ,&
&  0.69438E+00_JPRB, 0.67017E+00_JPRB, 0.66875E+00_JPRB, 0.67402E+00_JPRB, 0.67251E+00_JPRB, 0.67326E+00_JPRB, 0.67350E+00_JPRB ,&
&  0.66811E+00_JPRB, 0.66093E+00_JPRB, 0.65841E+00_JPRB, 0.65336E+00_JPRB, 0.65042E+00_JPRB, 0.65121E+00_JPRB, 0.64108E+00_JPRB ,&
&  0.63426E+00_JPRB, 0.63464E+00_JPRB, 0.64325E+00_JPRB, 0.66295E+00_JPRB, 0.68921E+00_JPRB, 0.71344E+00_JPRB, 0.72891E+00_JPRB ,&
&  0.73789E+00_JPRB, 0.74811E+00_JPRB, 0.75600E+00_JPRB, 0.78277E+00_JPRB, 0.79119E+00_JPRB, 0.79429E+00_JPRB, 0.79258E+00_JPRB ,&
&  0.78719E+00_JPRB, 0.78216E+00_JPRB, 0.59800E+00_JPRB, 0.66199E+00_JPRB, 0.67912E+00_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.71633E+00_JPRB, 0.74381E+00_JPRB, 0.75550E+00_JPRB, 0.75784E+00_JPRB, 0.75553E+00_JPRB, 0.74323E+00_JPRB, 0.72880E+00_JPRB ,&
&  0.71137E+00_JPRB, 0.70415E+00_JPRB, 0.70155E+00_JPRB, 0.70621E+00_JPRB, 0.71272E+00_JPRB, 0.72230E+00_JPRB, 0.73548E+00_JPRB ,&
&  0.74799E+00_JPRB, 0.75484E+00_JPRB, 0.76054E+00_JPRB, 0.75969E+00_JPRB, 0.75864E+00_JPRB, 0.76158E+00_JPRB, 0.76636E+00_JPRB ,&
&  0.77592E+00_JPRB, 0.77973E+00_JPRB, 0.77767E+00_JPRB, 0.78630E+00_JPRB, 0.79841E+00_JPRB, 0.78797E+00_JPRB, 0.76792E+00_JPRB ,&
&  0.74997E+00_JPRB, 0.61054E+00_JPRB, 0.62766E+00_JPRB, 0.71795E+00_JPRB, 0.78042E+00_JPRB, 0.85682E+00_JPRB, 0.88383E+00_JPRB ,&
&  0.87131E+00_JPRB, 0.83938E+00_JPRB, 0.80367E+00_JPRB, 0.77215E+00_JPRB, 0.75080E+00_JPRB, 0.72560E+00_JPRB, 0.70894E+00_JPRB ,&
&  0.69749E+00_JPRB, 0.68537E+00_JPRB, 0.66636E+00_JPRB, 0.65148E+00_JPRB, 0.63947E+00_JPRB, 0.62765E+00_JPRB, 0.62182E+00_JPRB ,&
&  0.62513E+00_JPRB, 0.63717E+00_JPRB, 0.64938E+00_JPRB, 0.66282E+00_JPRB, 0.68808E+00_JPRB, 0.71051E+00_JPRB, 0.73105E+00_JPRB ,&
&  0.73879E+00_JPRB, 0.74505E+00_JPRB, 0.75537E+00_JPRB, 0.76463E+00_JPRB, 0.77856E+00_JPRB, 0.78667E+00_JPRB, 0.77278E+00_JPRB ,&
&  0.73735E+00_JPRB, 0.71646E+00_JPRB, 0.70426E+00_JPRB, 0.69709E+00_JPRB, 0.69896E+00_JPRB, 0.70753E+00_JPRB, 0.71148E+00_JPRB ,&
&  0.70831E+00_JPRB, 0.70374E+00_JPRB, 0.69779E+00_JPRB, 0.69168E+00_JPRB, 0.69382E+00_JPRB, 0.69787E+00_JPRB, 0.69581E+00_JPRB ,&
&  0.69260E+00_JPRB, 0.69636E+00_JPRB, 0.70745E+00_JPRB, 0.72837E+00_JPRB, 0.75333E+00_JPRB, 0.77982E+00_JPRB, 0.79511E+00_JPRB ,&
&  0.80553E+00_JPRB, 0.81680E+00_JPRB, 0.83580E+00_JPRB, 0.79206E+00_JPRB, 0.82518E+00_JPRB, 0.83356E+00_JPRB, 0.83364E+00_JPRB ,&
&  0.81779E+00_JPRB, 0.79252E+00_JPRB, 0.76241E+00_JPRB, 0.63204E+00_JPRB, 0.68450E+00_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.75968E+00_JPRB, 0.78840E+00_JPRB, 0.79710E+00_JPRB, 0.79777E+00_JPRB, 0.78708E+00_JPRB, 0.77173E+00_JPRB, 0.75957E+00_JPRB ,&
&  0.75452E+00_JPRB, 0.74121E+00_JPRB, 0.73601E+00_JPRB, 0.73518E+00_JPRB, 0.74022E+00_JPRB, 0.74922E+00_JPRB, 0.75146E+00_JPRB ,&
&  0.74487E+00_JPRB, 0.73349E+00_JPRB, 0.72607E+00_JPRB, 0.72677E+00_JPRB, 0.73729E+00_JPRB, 0.74823E+00_JPRB, 0.75561E+00_JPRB ,&
&  0.75910E+00_JPRB, 0.75376E+00_JPRB, 0.75172E+00_JPRB, 0.76085E+00_JPRB, 0.76799E+00_JPRB, 0.75982E+00_JPRB, 0.73887E+00_JPRB ,&
&  0.72301E+00_JPRB, 0.64273E+00_JPRB, 0.71892E+00_JPRB, 0.80925E+00_JPRB, 0.87900E+00_JPRB, 0.91104E+00_JPRB, 0.92365E+00_JPRB ,&
&  0.90223E+00_JPRB, 0.86315E+00_JPRB, 0.81523E+00_JPRB, 0.78328E+00_JPRB, 0.75870E+00_JPRB, 0.73813E+00_JPRB, 0.72430E+00_JPRB ,&
&  0.70327E+00_JPRB, 0.68404E+00_JPRB, 0.66686E+00_JPRB, 0.65763E+00_JPRB, 0.65463E+00_JPRB, 0.65223E+00_JPRB, 0.65041E+00_JPRB ,&
&  0.64884E+00_JPRB, 0.64667E+00_JPRB, 0.64912E+00_JPRB, 0.66163E+00_JPRB, 0.68138E+00_JPRB, 0.71560E+00_JPRB, 0.75211E+00_JPRB ,&
&  0.76418E+00_JPRB, 0.77937E+00_JPRB, 0.80077E+00_JPRB, 0.82798E+00_JPRB, 0.83592E+00_JPRB, 0.83375E+00_JPRB, 0.81879E+00_JPRB ,&
&  0.79771E+00_JPRB, 0.76514E+00_JPRB, 0.74013E+00_JPRB, 0.72128E+00_JPRB, 0.71253E+00_JPRB, 0.70560E+00_JPRB, 0.71184E+00_JPRB ,&
&  0.71830E+00_JPRB, 0.72675E+00_JPRB, 0.72552E+00_JPRB, 0.72282E+00_JPRB, 0.72507E+00_JPRB, 0.71582E+00_JPRB, 0.71507E+00_JPRB ,&
&  0.71927E+00_JPRB, 0.73527E+00_JPRB, 0.76120E+00_JPRB, 0.78451E+00_JPRB, 0.81266E+00_JPRB, 0.83068E+00_JPRB, 0.83600E+00_JPRB ,&
&  0.83768E+00_JPRB, 0.83673E+00_JPRB, 0.83510E+00_JPRB, 0.86768E+00_JPRB, 0.86033E+00_JPRB, 0.85710E+00_JPRB, 0.84841E+00_JPRB ,&
&  0.83080E+00_JPRB, 0.79676E+00_JPRB, 0.75587E+00_JPRB, 0.74160E+00_JPRB, 0.69968E+00_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.88101E+00_JPRB, 0.87362E+00_JPRB, 0.85307E+00_JPRB, 0.83776E+00_JPRB, 0.82070E+00_JPRB, 0.80804E+00_JPRB, 0.79742E+00_JPRB ,&
&  0.79240E+00_JPRB, 0.77865E+00_JPRB, 0.77226E+00_JPRB, 0.76607E+00_JPRB, 0.76475E+00_JPRB, 0.75927E+00_JPRB, 0.74993E+00_JPRB ,&
&  0.73927E+00_JPRB, 0.73363E+00_JPRB, 0.73284E+00_JPRB, 0.73834E+00_JPRB, 0.74888E+00_JPRB, 0.75529E+00_JPRB, 0.75008E+00_JPRB ,&
&  0.74185E+00_JPRB, 0.72893E+00_JPRB, 0.71290E+00_JPRB, 0.72576E+00_JPRB, 0.74073E+00_JPRB, 0.74448E+00_JPRB, 0.72814E+00_JPRB ,&
&  0.71656E+00_JPRB, 0.69912E+00_JPRB, 0.89563E+00_JPRB, 0.98165E+00_JPRB, 0.99237E+00_JPRB, 0.10311E+01_JPRB, 0.98664E+00_JPRB ,&
&  0.94104E+00_JPRB, 0.86717E+00_JPRB, 0.81388E+00_JPRB, 0.79178E+00_JPRB, 0.77850E+00_JPRB, 0.75044E+00_JPRB, 0.72682E+00_JPRB ,&
&  0.69193E+00_JPRB, 0.67659E+00_JPRB, 0.66645E+00_JPRB, 0.66273E+00_JPRB, 0.66104E+00_JPRB, 0.66291E+00_JPRB, 0.66895E+00_JPRB ,&
&  0.67554E+00_JPRB, 0.67717E+00_JPRB, 0.67974E+00_JPRB, 0.69815E+00_JPRB, 0.71920E+00_JPRB, 0.77313E+00_JPRB, 0.81753E+00_JPRB ,&
&  0.83812E+00_JPRB, 0.85686E+00_JPRB, 0.88566E+00_JPRB, 0.90449E+00_JPRB, 0.89356E+00_JPRB, 0.86807E+00_JPRB, 0.84669E+00_JPRB ,&
&  0.82470E+00_JPRB, 0.80089E+00_JPRB, 0.77446E+00_JPRB, 0.75820E+00_JPRB, 0.74256E+00_JPRB, 0.73639E+00_JPRB, 0.74731E+00_JPRB ,&
&  0.75331E+00_JPRB, 0.76438E+00_JPRB, 0.76316E+00_JPRB, 0.75695E+00_JPRB, 0.73934E+00_JPRB, 0.72558E+00_JPRB, 0.72725E+00_JPRB ,&
&  0.74073E+00_JPRB, 0.76743E+00_JPRB, 0.80168E+00_JPRB, 0.83602E+00_JPRB, 0.86216E+00_JPRB, 0.87340E+00_JPRB, 0.87450E+00_JPRB ,&
&  0.88103E+00_JPRB, 0.88212E+00_JPRB, 0.83031E+00_JPRB, 0.83072E+00_JPRB, 0.91587E+00_JPRB, 0.90735E+00_JPRB, 0.89333E+00_JPRB ,&
&  0.86145E+00_JPRB, 0.80979E+00_JPRB, 0.77308E+00_JPRB, 0.76080E+00_JPRB, 0.83822E+00_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.84101E+00_JPRB, 0.83168E+00_JPRB, 0.81695E+00_JPRB, 0.80961E+00_JPRB, 0.81314E+00_JPRB, 0.80607E+00_JPRB, 0.80142E+00_JPRB ,&
&  0.80022E+00_JPRB, 0.79187E+00_JPRB, 0.78445E+00_JPRB, 0.77175E+00_JPRB, 0.75670E+00_JPRB, 0.74600E+00_JPRB, 0.74487E+00_JPRB ,&
&  0.74814E+00_JPRB, 0.74090E+00_JPRB, 0.74651E+00_JPRB, 0.74949E+00_JPRB, 0.74690E+00_JPRB, 0.73391E+00_JPRB, 0.72104E+00_JPRB ,&
&  0.71497E+00_JPRB, 0.70314E+00_JPRB, 0.70039E+00_JPRB, 0.71987E+00_JPRB, 0.73789E+00_JPRB, 0.73900E+00_JPRB, 0.71862E+00_JPRB ,&
&  0.72167E+00_JPRB, 0.80431E+00_JPRB, 0.11396E+01_JPRB, 0.12472E+01_JPRB, 0.11722E+01_JPRB, 0.11005E+01_JPRB, 0.10080E+01_JPRB ,&
&  0.92935E+00_JPRB, 0.85510E+00_JPRB, 0.82261E+00_JPRB, 0.79562E+00_JPRB, 0.76479E+00_JPRB, 0.73909E+00_JPRB, 0.71528E+00_JPRB ,&
&  0.68558E+00_JPRB, 0.67549E+00_JPRB, 0.67152E+00_JPRB, 0.67350E+00_JPRB, 0.67584E+00_JPRB, 0.67634E+00_JPRB, 0.67868E+00_JPRB ,&
&  0.68069E+00_JPRB, 0.68191E+00_JPRB, 0.69044E+00_JPRB, 0.70039E+00_JPRB, 0.73211E+00_JPRB, 0.82242E+00_JPRB, 0.91827E+00_JPRB ,&
&  0.92439E+00_JPRB, 0.94308E+00_JPRB, 0.92931E+00_JPRB, 0.91983E+00_JPRB, 0.90901E+00_JPRB, 0.88497E+00_JPRB, 0.86232E+00_JPRB ,&
&  0.83523E+00_JPRB, 0.81586E+00_JPRB, 0.80129E+00_JPRB, 0.79376E+00_JPRB, 0.77897E+00_JPRB, 0.78369E+00_JPRB, 0.79751E+00_JPRB ,&
&  0.81350E+00_JPRB, 0.82181E+00_JPRB, 0.80613E+00_JPRB, 0.79008E+00_JPRB, 0.76753E+00_JPRB, 0.76480E+00_JPRB, 0.76777E+00_JPRB ,&
&  0.77325E+00_JPRB, 0.79382E+00_JPRB, 0.83083E+00_JPRB, 0.86746E+00_JPRB, 0.89090E+00_JPRB, 0.89898E+00_JPRB, 0.89511E+00_JPRB ,&
&  0.90161E+00_JPRB, 0.89111E+00_JPRB, 0.87012E+00_JPRB, 0.87295E+00_JPRB, 0.86515E+00_JPRB, 0.92319E+00_JPRB, 0.88729E+00_JPRB ,&
&  0.83271E+00_JPRB, 0.79229E+00_JPRB, 0.77258E+00_JPRB, 0.76776E+00_JPRB, 0.82645E+00_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.82748E+00_JPRB, 0.81117E+00_JPRB, 0.80228E+00_JPRB, 0.79436E+00_JPRB, 0.78400E+00_JPRB, 0.78034E+00_JPRB, 0.77504E+00_JPRB ,&
&  0.78075E+00_JPRB, 0.78444E+00_JPRB, 0.77676E+00_JPRB, 0.75290E+00_JPRB, 0.73439E+00_JPRB, 0.73318E+00_JPRB, 0.74087E+00_JPRB ,&
&  0.73693E+00_JPRB, 0.72723E+00_JPRB, 0.73145E+00_JPRB, 0.71542E+00_JPRB, 0.69384E+00_JPRB, 0.68070E+00_JPRB, 0.67545E+00_JPRB ,&
&  0.68004E+00_JPRB, 0.69083E+00_JPRB, 0.71718E+00_JPRB, 0.74195E+00_JPRB, 0.74485E+00_JPRB, 0.73508E+00_JPRB, 0.71033E+00_JPRB ,&
&  0.75690E+00_JPRB, 0.89026E+00_JPRB, 0.14663E+01_JPRB, 0.14623E+01_JPRB, 0.12962E+01_JPRB, 0.11196E+01_JPRB, 0.10354E+01_JPRB ,&
&  0.93421E+00_JPRB, 0.87042E+00_JPRB, 0.82642E+00_JPRB, 0.77189E+00_JPRB, 0.73018E+00_JPRB, 0.70798E+00_JPRB, 0.69811E+00_JPRB ,&
&  0.68154E+00_JPRB, 0.67159E+00_JPRB, 0.67673E+00_JPRB, 0.68646E+00_JPRB, 0.69321E+00_JPRB, 0.69452E+00_JPRB, 0.68186E+00_JPRB ,&
&  0.67718E+00_JPRB, 0.67394E+00_JPRB, 0.67884E+00_JPRB, 0.67956E+00_JPRB, 0.69495E+00_JPRB, 0.69585E+00_JPRB, 0.83827E+00_JPRB ,&
&  0.10786E+01_JPRB, 0.10377E+01_JPRB, 0.97770E+00_JPRB, 0.95487E+00_JPRB, 0.94195E+00_JPRB, 0.90747E+00_JPRB, 0.88327E+00_JPRB ,&
&  0.85754E+00_JPRB, 0.84195E+00_JPRB, 0.85189E+00_JPRB, 0.85410E+00_JPRB, 0.86143E+00_JPRB, 0.84713E+00_JPRB, 0.83930E+00_JPRB ,&
&  0.84716E+00_JPRB, 0.82635E+00_JPRB, 0.80280E+00_JPRB, 0.78077E+00_JPRB, 0.78922E+00_JPRB, 0.80702E+00_JPRB, 0.81467E+00_JPRB ,&
&  0.82106E+00_JPRB, 0.82601E+00_JPRB, 0.84385E+00_JPRB, 0.85565E+00_JPRB, 0.89639E+00_JPRB, 0.93077E+00_JPRB, 0.92549E+00_JPRB ,&
&  0.87822E+00_JPRB, 0.86506E+00_JPRB, 0.87003E+00_JPRB, 0.84139E+00_JPRB, 0.83341E+00_JPRB, 0.90091E+00_JPRB, 0.84517E+00_JPRB ,&
&  0.79461E+00_JPRB, 0.76528E+00_JPRB, 0.75130E+00_JPRB, 0.76883E+00_JPRB, 0.83017E+00_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.84479E+00_JPRB, 0.82850E+00_JPRB, 0.81651E+00_JPRB, 0.79346E+00_JPRB, 0.78911E+00_JPRB, 0.78079E+00_JPRB, 0.77794E+00_JPRB ,&
&  0.77867E+00_JPRB, 0.76789E+00_JPRB, 0.75504E+00_JPRB, 0.73654E+00_JPRB, 0.72953E+00_JPRB, 0.74015E+00_JPRB, 0.73476E+00_JPRB ,&
&  0.71730E+00_JPRB, 0.69409E+00_JPRB, 0.68731E+00_JPRB, 0.67896E+00_JPRB, 0.67063E+00_JPRB, 0.67296E+00_JPRB, 0.68272E+00_JPRB ,&
&  0.68636E+00_JPRB, 0.70364E+00_JPRB, 0.73221E+00_JPRB, 0.75772E+00_JPRB, 0.75366E+00_JPRB, 0.74114E+00_JPRB, 0.74635E+00_JPRB ,&
&  0.81511E+00_JPRB, 0.78475E+00_JPRB, 0.14865E+01_JPRB, 0.16626E+01_JPRB, 0.14007E+01_JPRB, 0.11904E+01_JPRB, 0.99281E+00_JPRB ,&
&  0.93407E+00_JPRB, 0.85364E+00_JPRB, 0.80985E+00_JPRB, 0.75921E+00_JPRB, 0.72860E+00_JPRB, 0.69153E+00_JPRB, 0.68211E+00_JPRB ,&
&  0.68772E+00_JPRB, 0.68113E+00_JPRB, 0.68266E+00_JPRB, 0.70034E+00_JPRB, 0.70841E+00_JPRB, 0.71631E+00_JPRB, 0.69933E+00_JPRB ,&
&  0.68465E+00_JPRB, 0.67062E+00_JPRB, 0.67664E+00_JPRB, 0.68586E+00_JPRB, 0.69106E+00_JPRB, 0.75081E+00_JPRB, 0.82666E+00_JPRB ,&
&  0.89215E+00_JPRB, 0.10777E+01_JPRB, 0.10017E+01_JPRB, 0.96812E+00_JPRB, 0.93739E+00_JPRB, 0.90123E+00_JPRB, 0.89316E+00_JPRB ,&
&  0.86741E+00_JPRB, 0.86437E+00_JPRB, 0.87052E+00_JPRB, 0.90811E+00_JPRB, 0.89506E+00_JPRB, 0.90335E+00_JPRB, 0.89115E+00_JPRB ,&
&  0.86309E+00_JPRB, 0.83084E+00_JPRB, 0.80255E+00_JPRB, 0.79226E+00_JPRB, 0.81479E+00_JPRB, 0.82853E+00_JPRB, 0.82510E+00_JPRB ,&
&  0.82427E+00_JPRB, 0.84674E+00_JPRB, 0.86441E+00_JPRB, 0.87047E+00_JPRB, 0.89525E+00_JPRB, 0.90163E+00_JPRB, 0.90211E+00_JPRB ,&
&  0.88981E+00_JPRB, 0.88833E+00_JPRB, 0.84483E+00_JPRB, 0.83245E+00_JPRB, 0.79597E+00_JPRB, 0.83319E+00_JPRB, 0.79095E+00_JPRB ,&
&  0.76121E+00_JPRB, 0.74052E+00_JPRB, 0.76671E+00_JPRB, 0.82307E+00_JPRB, 0.85309E+00_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.84002E+00_JPRB, 0.84296E+00_JPRB, 0.80875E+00_JPRB, 0.80593E+00_JPRB, 0.79459E+00_JPRB, 0.79938E+00_JPRB, 0.81539E+00_JPRB ,&
&  0.80923E+00_JPRB, 0.78518E+00_JPRB, 0.75423E+00_JPRB, 0.74183E+00_JPRB, 0.75199E+00_JPRB, 0.76271E+00_JPRB, 0.72974E+00_JPRB ,&
&  0.69166E+00_JPRB, 0.68032E+00_JPRB, 0.67556E+00_JPRB, 0.67965E+00_JPRB, 0.69047E+00_JPRB, 0.70819E+00_JPRB, 0.71691E+00_JPRB ,&
&  0.71175E+00_JPRB, 0.74804E+00_JPRB, 0.78707E+00_JPRB, 0.79839E+00_JPRB, 0.77934E+00_JPRB, 0.76412E+00_JPRB, 0.76394E+00_JPRB ,&
&  0.81680E+00_JPRB, 0.16753E+01_JPRB, 0.12065E+01_JPRB, 0.17683E+01_JPRB, 0.14038E+01_JPRB, 0.11489E+01_JPRB, 0.90468E+00_JPRB ,&
&  0.83576E+00_JPRB, 0.84834E+00_JPRB, 0.82409E+00_JPRB, 0.76543E+00_JPRB, 0.72300E+00_JPRB, 0.68942E+00_JPRB, 0.66319E+00_JPRB ,&
&  0.66035E+00_JPRB, 0.67377E+00_JPRB, 0.68039E+00_JPRB, 0.71449E+00_JPRB, 0.74231E+00_JPRB, 0.74462E+00_JPRB, 0.72054E+00_JPRB ,&
&  0.69596E+00_JPRB, 0.67756E+00_JPRB, 0.68512E+00_JPRB, 0.74300E+00_JPRB, 0.74843E+00_JPRB, 0.80386E+00_JPRB, 0.89818E+00_JPRB ,&
&  0.87621E+00_JPRB, 0.10876E+01_JPRB, 0.99944E+00_JPRB, 0.95600E+00_JPRB, 0.92285E+00_JPRB, 0.88810E+00_JPRB, 0.89185E+00_JPRB ,&
&  0.87355E+00_JPRB, 0.87239E+00_JPRB, 0.87910E+00_JPRB, 0.88165E+00_JPRB, 0.90427E+00_JPRB, 0.92611E+00_JPRB, 0.89904E+00_JPRB ,&
&  0.85913E+00_JPRB, 0.82538E+00_JPRB, 0.82632E+00_JPRB, 0.83844E+00_JPRB, 0.85530E+00_JPRB, 0.84318E+00_JPRB, 0.83384E+00_JPRB ,&
&  0.84302E+00_JPRB, 0.88316E+00_JPRB, 0.88752E+00_JPRB, 0.89433E+00_JPRB, 0.88770E+00_JPRB, 0.86807E+00_JPRB, 0.84995E+00_JPRB ,&
&  0.88167E+00_JPRB, 0.87141E+00_JPRB, 0.81504E+00_JPRB, 0.79728E+00_JPRB, 0.77618E+00_JPRB, 0.79310E+00_JPRB, 0.75399E+00_JPRB ,&
&  0.74437E+00_JPRB, 0.75582E+00_JPRB, 0.83140E+00_JPRB, 0.83920E+00_JPRB, 0.84844E+00_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.83863E+00_JPRB, 0.83017E+00_JPRB, 0.80210E+00_JPRB, 0.80498E+00_JPRB, 0.78881E+00_JPRB, 0.81973E+00_JPRB, 0.82508E+00_JPRB ,&
&  0.79088E+00_JPRB, 0.75914E+00_JPRB, 0.74222E+00_JPRB, 0.76157E+00_JPRB, 0.78118E+00_JPRB, 0.76623E+00_JPRB, 0.72520E+00_JPRB ,&
&  0.71227E+00_JPRB, 0.71134E+00_JPRB, 0.71272E+00_JPRB, 0.71647E+00_JPRB, 0.73190E+00_JPRB, 0.75392E+00_JPRB, 0.75068E+00_JPRB ,&
&  0.74714E+00_JPRB, 0.77620E+00_JPRB, 0.80084E+00_JPRB, 0.79918E+00_JPRB, 0.78662E+00_JPRB, 0.77076E+00_JPRB, 0.78563E+00_JPRB ,&
&  0.80522E+00_JPRB, 0.14829E+01_JPRB, 0.11615E+01_JPRB, 0.17583E+01_JPRB, 0.13766E+01_JPRB, 0.11332E+01_JPRB, 0.86577E+00_JPRB ,&
&  0.76506E+00_JPRB, 0.83306E+00_JPRB, 0.80348E+00_JPRB, 0.73334E+00_JPRB, 0.68208E+00_JPRB, 0.66002E+00_JPRB, 0.64440E+00_JPRB ,&
&  0.64380E+00_JPRB, 0.67614E+00_JPRB, 0.70309E+00_JPRB, 0.74191E+00_JPRB, 0.76642E+00_JPRB, 0.75793E+00_JPRB, 0.74100E+00_JPRB ,&
&  0.72313E+00_JPRB, 0.71086E+00_JPRB, 0.74627E+00_JPRB, 0.78175E+00_JPRB, 0.83378E+00_JPRB, 0.89553E+00_JPRB, 0.93580E+00_JPRB ,&
&  0.93135E+00_JPRB, 0.10569E+01_JPRB, 0.10010E+01_JPRB, 0.96317E+00_JPRB, 0.92038E+00_JPRB, 0.90256E+00_JPRB, 0.89609E+00_JPRB ,&
&  0.86551E+00_JPRB, 0.82943E+00_JPRB, 0.82954E+00_JPRB, 0.84877E+00_JPRB, 0.90970E+00_JPRB, 0.92316E+00_JPRB, 0.88212E+00_JPRB ,&
&  0.84062E+00_JPRB, 0.82366E+00_JPRB, 0.82342E+00_JPRB, 0.83862E+00_JPRB, 0.85297E+00_JPRB, 0.84788E+00_JPRB, 0.84503E+00_JPRB ,&
&  0.87319E+00_JPRB, 0.92026E+00_JPRB, 0.90905E+00_JPRB, 0.90307E+00_JPRB, 0.87036E+00_JPRB, 0.84247E+00_JPRB, 0.82197E+00_JPRB ,&
&  0.84111E+00_JPRB, 0.82750E+00_JPRB, 0.79690E+00_JPRB, 0.76304E+00_JPRB, 0.75364E+00_JPRB, 0.76194E+00_JPRB, 0.75032E+00_JPRB ,&
&  0.76451E+00_JPRB, 0.81973E+00_JPRB, 0.87700E+00_JPRB, 0.84074E+00_JPRB, 0.83500E+00_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.81086E+00_JPRB, 0.78281E+00_JPRB, 0.77273E+00_JPRB, 0.76182E+00_JPRB, 0.75038E+00_JPRB, 0.78187E+00_JPRB, 0.77203E+00_JPRB ,&
&  0.74805E+00_JPRB, 0.72458E+00_JPRB, 0.74420E+00_JPRB, 0.78738E+00_JPRB, 0.78698E+00_JPRB, 0.76466E+00_JPRB, 0.74900E+00_JPRB ,&
&  0.74734E+00_JPRB, 0.76488E+00_JPRB, 0.76876E+00_JPRB, 0.77936E+00_JPRB, 0.77869E+00_JPRB, 0.76786E+00_JPRB, 0.76041E+00_JPRB ,&
&  0.77628E+00_JPRB, 0.78447E+00_JPRB, 0.79083E+00_JPRB, 0.79154E+00_JPRB, 0.80114E+00_JPRB, 0.79222E+00_JPRB, 0.83863E+00_JPRB ,&
&  0.91827E+00_JPRB, 0.13926E+01_JPRB, 0.11730E+01_JPRB, 0.13778E+01_JPRB, 0.12435E+01_JPRB, 0.10412E+01_JPRB, 0.78220E+00_JPRB ,&
&  0.66465E+00_JPRB, 0.61979E+00_JPRB, 0.75014E+00_JPRB, 0.71489E+00_JPRB, 0.69332E+00_JPRB, 0.70727E+00_JPRB, 0.71404E+00_JPRB ,&
&  0.73615E+00_JPRB, 0.74424E+00_JPRB, 0.79660E+00_JPRB, 0.81179E+00_JPRB, 0.81083E+00_JPRB, 0.79827E+00_JPRB, 0.77942E+00_JPRB ,&
&  0.78640E+00_JPRB, 0.79828E+00_JPRB, 0.87063E+00_JPRB, 0.90405E+00_JPRB, 0.88872E+00_JPRB, 0.93247E+00_JPRB, 0.94338E+00_JPRB ,&
&  0.96731E+00_JPRB, 0.10054E+01_JPRB, 0.10098E+01_JPRB, 0.98989E+00_JPRB, 0.90368E+00_JPRB, 0.93575E+00_JPRB, 0.88282E+00_JPRB ,&
&  0.82523E+00_JPRB, 0.79113E+00_JPRB, 0.79733E+00_JPRB, 0.83065E+00_JPRB, 0.87429E+00_JPRB, 0.87364E+00_JPRB, 0.84976E+00_JPRB ,&
&  0.83546E+00_JPRB, 0.84018E+00_JPRB, 0.82559E+00_JPRB, 0.83715E+00_JPRB, 0.84595E+00_JPRB, 0.87164E+00_JPRB, 0.89587E+00_JPRB ,&
&  0.93203E+00_JPRB, 0.96023E+00_JPRB, 0.92762E+00_JPRB, 0.89299E+00_JPRB, 0.84634E+00_JPRB, 0.81453E+00_JPRB, 0.78153E+00_JPRB ,&
&  0.78701E+00_JPRB, 0.80767E+00_JPRB, 0.80693E+00_JPRB, 0.77779E+00_JPRB, 0.76136E+00_JPRB, 0.76150E+00_JPRB, 0.78979E+00_JPRB ,&
&  0.82437E+00_JPRB, 0.88574E+00_JPRB, 0.89483E+00_JPRB, 0.84285E+00_JPRB, 0.83138E+00_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.74991E+00_JPRB, 0.74324E+00_JPRB, 0.72193E+00_JPRB, 0.70804E+00_JPRB, 0.73066E+00_JPRB, 0.76407E+00_JPRB, 0.76354E+00_JPRB ,&
&  0.74144E+00_JPRB, 0.73032E+00_JPRB, 0.76549E+00_JPRB, 0.79903E+00_JPRB, 0.79520E+00_JPRB, 0.80934E+00_JPRB, 0.81651E+00_JPRB ,&
&  0.84542E+00_JPRB, 0.83726E+00_JPRB, 0.81686E+00_JPRB, 0.78855E+00_JPRB, 0.78550E+00_JPRB, 0.77204E+00_JPRB, 0.77037E+00_JPRB ,&
&  0.77671E+00_JPRB, 0.78163E+00_JPRB, 0.79172E+00_JPRB, 0.81042E+00_JPRB, 0.83002E+00_JPRB, 0.88747E+00_JPRB, 0.10102E+01_JPRB ,&
&  0.12039E+01_JPRB, 0.15918E+01_JPRB, 0.62883E+00_JPRB, 0.78735E+00_JPRB, 0.89836E+00_JPRB, 0.79393E+00_JPRB, 0.66721E+00_JPRB ,&
&  0.57249E+00_JPRB, 0.51312E+00_JPRB, 0.60842E+00_JPRB, 0.74414E+00_JPRB, 0.76385E+00_JPRB, 0.78911E+00_JPRB, 0.80509E+00_JPRB ,&
&  0.82495E+00_JPRB, 0.84054E+00_JPRB, 0.87357E+00_JPRB, 0.87598E+00_JPRB, 0.87322E+00_JPRB, 0.86100E+00_JPRB, 0.84595E+00_JPRB ,&
&  0.88023E+00_JPRB, 0.93069E+00_JPRB, 0.10230E+01_JPRB, 0.93974E+00_JPRB, 0.94004E+00_JPRB, 0.91492E+00_JPRB, 0.88383E+00_JPRB ,&
&  0.86195E+00_JPRB, 0.95861E+00_JPRB, 0.10300E+01_JPRB, 0.10007E+01_JPRB, 0.90525E+00_JPRB, 0.85131E+00_JPRB, 0.91925E+00_JPRB ,&
&  0.84293E+00_JPRB, 0.79502E+00_JPRB, 0.77679E+00_JPRB, 0.78634E+00_JPRB, 0.84142E+00_JPRB, 0.83721E+00_JPRB, 0.81902E+00_JPRB ,&
&  0.82573E+00_JPRB, 0.81675E+00_JPRB, 0.82941E+00_JPRB, 0.85495E+00_JPRB, 0.88201E+00_JPRB, 0.93284E+00_JPRB, 0.95668E+00_JPRB ,&
&  0.95412E+00_JPRB, 0.96359E+00_JPRB, 0.95749E+00_JPRB, 0.91510E+00_JPRB, 0.85295E+00_JPRB, 0.81153E+00_JPRB, 0.79893E+00_JPRB ,&
&  0.80878E+00_JPRB, 0.81789E+00_JPRB, 0.78760E+00_JPRB, 0.75111E+00_JPRB, 0.78565E+00_JPRB, 0.81216E+00_JPRB, 0.92251E+00_JPRB ,&
&  0.10364E+01_JPRB, 0.11152E+01_JPRB, 0.10781E+01_JPRB, 0.80956E+00_JPRB, 0.80018E+00_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.70995E+00_JPRB, 0.66933E+00_JPRB, 0.63839E+00_JPRB, 0.64993E+00_JPRB, 0.70017E+00_JPRB, 0.75161E+00_JPRB, 0.75100E+00_JPRB ,&
&  0.73689E+00_JPRB, 0.75551E+00_JPRB, 0.82754E+00_JPRB, 0.88189E+00_JPRB, 0.88875E+00_JPRB, 0.89817E+00_JPRB, 0.91242E+00_JPRB ,&
&  0.85577E+00_JPRB, 0.82096E+00_JPRB, 0.80277E+00_JPRB, 0.78316E+00_JPRB, 0.78168E+00_JPRB, 0.77426E+00_JPRB, 0.77188E+00_JPRB ,&
&  0.77748E+00_JPRB, 0.77693E+00_JPRB, 0.79756E+00_JPRB, 0.83105E+00_JPRB, 0.90004E+00_JPRB, 0.10655E+01_JPRB, 0.13181E+01_JPRB ,&
&  0.17729E+01_JPRB, 0.11663E+01_JPRB, 0.49546E+00_JPRB, 0.65672E+00_JPRB, 0.69913E+00_JPRB, 0.67265E+00_JPRB, 0.59563E+00_JPRB ,&
&  0.55775E+00_JPRB, 0.57431E+00_JPRB, 0.68053E+00_JPRB, 0.80682E+00_JPRB, 0.83953E+00_JPRB, 0.86953E+00_JPRB, 0.88026E+00_JPRB ,&
&  0.88552E+00_JPRB, 0.90627E+00_JPRB, 0.91865E+00_JPRB, 0.92199E+00_JPRB, 0.92723E+00_JPRB, 0.93226E+00_JPRB, 0.95352E+00_JPRB ,&
&  0.10071E+01_JPRB, 0.10610E+01_JPRB, 0.11255E+01_JPRB, 0.96985E+00_JPRB, 0.96828E+00_JPRB, 0.88104E+00_JPRB, 0.85682E+00_JPRB ,&
&  0.84896E+00_JPRB, 0.93168E+00_JPRB, 0.10107E+01_JPRB, 0.96284E+00_JPRB, 0.86877E+00_JPRB, 0.80269E+00_JPRB, 0.91882E+00_JPRB ,&
&  0.83125E+00_JPRB, 0.77807E+00_JPRB, 0.74000E+00_JPRB, 0.74902E+00_JPRB, 0.78270E+00_JPRB, 0.79195E+00_JPRB, 0.79012E+00_JPRB ,&
&  0.79769E+00_JPRB, 0.79824E+00_JPRB, 0.83219E+00_JPRB, 0.85297E+00_JPRB, 0.87023E+00_JPRB, 0.90731E+00_JPRB, 0.90939E+00_JPRB ,&
&  0.90139E+00_JPRB, 0.89483E+00_JPRB, 0.90236E+00_JPRB, 0.91948E+00_JPRB, 0.87213E+00_JPRB, 0.79536E+00_JPRB, 0.78337E+00_JPRB ,&
&  0.77930E+00_JPRB, 0.77836E+00_JPRB, 0.75381E+00_JPRB, 0.77006E+00_JPRB, 0.78620E+00_JPRB, 0.86117E+00_JPRB, 0.94250E+00_JPRB ,&
&  0.97853E+00_JPRB, 0.95524E+00_JPRB, 0.89173E+00_JPRB, 0.76884E+00_JPRB, 0.75528E+00_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.66078E+00_JPRB, 0.63452E+00_JPRB, 0.59495E+00_JPRB, 0.59416E+00_JPRB, 0.65386E+00_JPRB, 0.70094E+00_JPRB, 0.70283E+00_JPRB ,&
&  0.70888E+00_JPRB, 0.77154E+00_JPRB, 0.88274E+00_JPRB, 0.97011E+00_JPRB, 0.99606E+00_JPRB, 0.98520E+00_JPRB, 0.88410E+00_JPRB ,&
&  0.84251E+00_JPRB, 0.81556E+00_JPRB, 0.78671E+00_JPRB, 0.77118E+00_JPRB, 0.76149E+00_JPRB, 0.75789E+00_JPRB, 0.76314E+00_JPRB ,&
&  0.77206E+00_JPRB, 0.79462E+00_JPRB, 0.84619E+00_JPRB, 0.91543E+00_JPRB, 0.10247E+01_JPRB, 0.11455E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.90911E+00_JPRB, 0.41927E+00_JPRB, 0.46593E+00_JPRB, 0.66875E+00_JPRB, 0.62522E+00_JPRB, 0.60996E+00_JPRB, 0.57745E+00_JPRB ,&
&  0.58780E+00_JPRB, 0.65450E+00_JPRB, 0.72027E+00_JPRB, 0.83875E+00_JPRB, 0.88633E+00_JPRB, 0.92827E+00_JPRB, 0.93071E+00_JPRB ,&
&  0.94293E+00_JPRB, 0.96193E+00_JPRB, 0.98512E+00_JPRB, 0.10027E+01_JPRB, 0.10243E+01_JPRB, 0.10481E+01_JPRB, 0.10943E+01_JPRB ,&
&  0.11416E+01_JPRB, 0.11763E+01_JPRB, 0.12015E+01_JPRB, 0.10009E+01_JPRB, 0.99671E+00_JPRB, 0.89521E+00_JPRB, 0.82955E+00_JPRB ,&
&  0.82480E+00_JPRB, 0.82858E+00_JPRB, 0.83945E+00_JPRB, 0.94616E+00_JPRB, 0.89232E+00_JPRB, 0.76124E+00_JPRB, 0.82587E+00_JPRB ,&
&  0.77644E+00_JPRB, 0.75710E+00_JPRB, 0.73923E+00_JPRB, 0.68099E+00_JPRB, 0.73900E+00_JPRB, 0.76319E+00_JPRB, 0.77911E+00_JPRB ,&
&  0.76252E+00_JPRB, 0.77844E+00_JPRB, 0.79899E+00_JPRB, 0.83403E+00_JPRB, 0.82304E+00_JPRB, 0.82048E+00_JPRB, 0.80509E+00_JPRB ,&
&  0.80550E+00_JPRB, 0.80955E+00_JPRB, 0.82310E+00_JPRB, 0.84660E+00_JPRB, 0.85423E+00_JPRB, 0.78638E+00_JPRB, 0.73666E+00_JPRB ,&
&  0.71591E+00_JPRB, 0.71073E+00_JPRB, 0.69159E+00_JPRB, 0.74191E+00_JPRB, 0.76688E+00_JPRB, 0.79205E+00_JPRB, 0.82555E+00_JPRB ,&
&  0.84116E+00_JPRB, 0.84226E+00_JPRB, 0.79516E+00_JPRB, 0.74066E+00_JPRB, 0.70414E+00_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.55764E+00_JPRB, 0.54497E+00_JPRB, 0.53006E+00_JPRB, 0.54791E+00_JPRB, 0.58186E+00_JPRB, 0.62283E+00_JPRB, 0.64015E+00_JPRB ,&
&  0.66208E+00_JPRB, 0.75843E+00_JPRB, 0.88467E+00_JPRB, 0.99478E+00_JPRB, 0.10088E+01_JPRB, 0.97195E+00_JPRB, 0.86544E+00_JPRB ,&
&  0.83032E+00_JPRB, 0.79736E+00_JPRB, 0.77478E+00_JPRB, 0.74840E+00_JPRB, 0.73838E+00_JPRB, 0.74834E+00_JPRB, 0.77231E+00_JPRB ,&
&  0.80426E+00_JPRB, 0.83773E+00_JPRB, 0.87780E+00_JPRB, 0.91775E+00_JPRB, 0.94269E+00_JPRB, 0.91966E+00_JPRB, 0.80209E+00_JPRB ,&
&  0.34674E+00_JPRB, 0.43313E+00_JPRB, 0.63902E+00_JPRB, 0.62475E+00_JPRB, 0.58450E+00_JPRB, 0.56125E+00_JPRB, 0.58157E+00_JPRB ,&
&  0.60802E+00_JPRB, 0.61723E+00_JPRB, 0.67811E+00_JPRB, 0.82166E+00_JPRB, 0.89183E+00_JPRB, 0.96081E+00_JPRB, 0.98252E+00_JPRB ,&
&  0.99802E+00_JPRB, 0.10202E+01_JPRB, 0.10587E+01_JPRB, 0.10887E+01_JPRB, 0.11086E+01_JPRB, 0.11242E+01_JPRB, 0.11532E+01_JPRB ,&
&  0.12107E+01_JPRB, 0.12552E+01_JPRB, 0.12711E+01_JPRB, 0.10716E+01_JPRB, 0.10330E+01_JPRB, 0.93730E+00_JPRB, 0.86304E+00_JPRB ,&
&  0.76562E+00_JPRB, 0.76784E+00_JPRB, 0.82086E+00_JPRB, 0.93504E+00_JPRB, 0.89460E+00_JPRB, 0.85743E+00_JPRB, 0.82477E+00_JPRB ,&
&  0.77521E+00_JPRB, 0.70583E+00_JPRB, 0.70921E+00_JPRB, 0.71318E+00_JPRB, 0.71201E+00_JPRB, 0.75200E+00_JPRB, 0.72719E+00_JPRB ,&
&  0.74456E+00_JPRB, 0.75554E+00_JPRB, 0.74928E+00_JPRB, 0.75316E+00_JPRB, 0.74909E+00_JPRB, 0.74626E+00_JPRB, 0.72914E+00_JPRB ,&
&  0.72518E+00_JPRB, 0.71889E+00_JPRB, 0.72814E+00_JPRB, 0.74640E+00_JPRB, 0.75262E+00_JPRB, 0.74690E+00_JPRB, 0.70810E+00_JPRB ,&
&  0.67240E+00_JPRB, 0.68846E+00_JPRB, 0.79615E+00_JPRB, 0.93643E+00_JPRB, 0.92078E+00_JPRB, 0.79419E+00_JPRB, 0.62408E+00_JPRB ,&
&  0.60919E+00_JPRB, 0.63014E+00_JPRB, 0.63286E+00_JPRB, 0.61268E+00_JPRB, 0.58679E+00_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.45445E+00_JPRB, 0.44321E+00_JPRB, 0.44274E+00_JPRB, 0.45333E+00_JPRB, 0.45995E+00_JPRB, 0.48367E+00_JPRB, 0.51366E+00_JPRB ,&
&  0.52819E+00_JPRB, 0.59526E+00_JPRB, 0.74396E+00_JPRB, 0.86946E+00_JPRB, 0.91649E+00_JPRB, 0.94967E+00_JPRB, 0.89163E+00_JPRB ,&
&  0.86737E+00_JPRB, 0.82499E+00_JPRB, 0.76400E+00_JPRB, 0.74303E+00_JPRB, 0.74408E+00_JPRB, 0.75723E+00_JPRB, 0.77659E+00_JPRB ,&
&  0.79986E+00_JPRB, 0.82516E+00_JPRB, 0.84325E+00_JPRB, 0.85951E+00_JPRB, 0.88026E+00_JPRB, 0.83972E+00_JPRB, 0.75526E+00_JPRB ,&
&  0.41117E+00_JPRB, 0.68816E+00_JPRB, 0.66984E+00_JPRB, 0.61435E+00_JPRB, 0.55740E+00_JPRB, 0.52794E+00_JPRB, 0.56154E+00_JPRB ,&
&  0.59627E+00_JPRB, 0.58641E+00_JPRB, 0.62854E+00_JPRB, 0.73321E+00_JPRB, 0.88249E+00_JPRB, 0.95645E+00_JPRB, 0.10140E+01_JPRB ,&
&  0.10595E+01_JPRB, 0.10826E+01_JPRB, 0.11091E+01_JPRB, 0.11326E+01_JPRB, 0.11383E+01_JPRB, 0.11381E+01_JPRB, 0.11603E+01_JPRB ,&
&  0.11960E+01_JPRB, 0.12626E+01_JPRB, 0.12923E+01_JPRB, 0.11700E+01_JPRB, 0.99212E+00_JPRB, 0.96747E+00_JPRB, 0.84764E+00_JPRB ,&
&  0.72748E+00_JPRB, 0.70928E+00_JPRB, 0.81560E+00_JPRB, 0.89616E+00_JPRB, 0.89239E+00_JPRB, 0.88384E+00_JPRB, 0.85263E+00_JPRB ,&
&  0.80028E+00_JPRB, 0.72781E+00_JPRB, 0.64599E+00_JPRB, 0.65017E+00_JPRB, 0.67829E+00_JPRB, 0.69434E+00_JPRB, 0.69265E+00_JPRB ,&
&  0.70191E+00_JPRB, 0.68921E+00_JPRB, 0.67299E+00_JPRB, 0.64811E+00_JPRB, 0.63998E+00_JPRB, 0.65863E+00_JPRB, 0.66671E+00_JPRB ,&
&  0.68532E+00_JPRB, 0.64071E+00_JPRB, 0.60913E+00_JPRB, 0.61894E+00_JPRB, 0.63115E+00_JPRB, 0.64939E+00_JPRB, 0.65091E+00_JPRB ,&
&  0.70391E+00_JPRB, 0.85828E+00_JPRB, 0.12573E+01_JPRB, 0.16664E+01_JPRB, 0.13696E+01_JPRB, 0.10766E+01_JPRB, 0.60236E+00_JPRB ,&
&  0.47484E+00_JPRB, 0.45812E+00_JPRB, 0.47580E+00_JPRB, 0.47874E+00_JPRB, 0.47651E+00_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.40402E+00_JPRB, 0.39499E+00_JPRB, 0.40171E+00_JPRB, 0.40014E+00_JPRB, 0.40370E+00_JPRB, 0.41045E+00_JPRB, 0.41479E+00_JPRB ,&
&  0.41809E+00_JPRB, 0.43443E+00_JPRB, 0.52027E+00_JPRB, 0.63704E+00_JPRB, 0.74559E+00_JPRB, 0.82587E+00_JPRB, 0.85587E+00_JPRB ,&
&  0.87969E+00_JPRB, 0.87576E+00_JPRB, 0.84281E+00_JPRB, 0.80599E+00_JPRB, 0.79084E+00_JPRB, 0.81284E+00_JPRB, 0.82737E+00_JPRB ,&
&  0.83862E+00_JPRB, 0.84622E+00_JPRB, 0.84981E+00_JPRB, 0.86036E+00_JPRB, 0.86628E+00_JPRB, 0.78443E+00_JPRB, 0.48016E+00_JPRB ,&
&  0.70463E+00_JPRB, 0.74465E+00_JPRB, 0.72621E+00_JPRB, 0.65719E+00_JPRB, 0.59998E+00_JPRB, 0.55191E+00_JPRB, 0.54984E+00_JPRB ,&
&  0.55790E+00_JPRB, 0.55730E+00_JPRB, 0.57096E+00_JPRB, 0.62060E+00_JPRB, 0.83067E+00_JPRB, 0.90915E+00_JPRB, 0.95367E+00_JPRB ,&
&  0.99641E+00_JPRB, 0.10387E+01_JPRB, 0.10618E+01_JPRB, 0.10975E+01_JPRB, 0.11063E+01_JPRB, 0.10924E+01_JPRB, 0.10910E+01_JPRB ,&
&  0.10879E+01_JPRB, 0.11227E+01_JPRB, 0.11215E+01_JPRB, 0.10714E+01_JPRB, 0.94931E+00_JPRB, 0.10043E+01_JPRB, 0.77958E+00_JPRB ,&
&  0.67126E+00_JPRB, 0.66570E+00_JPRB, 0.76321E+00_JPRB, 0.85932E+00_JPRB, 0.86684E+00_JPRB, 0.84108E+00_JPRB, 0.78213E+00_JPRB ,&
&  0.72354E+00_JPRB, 0.67305E+00_JPRB, 0.63448E+00_JPRB, 0.61027E+00_JPRB, 0.61595E+00_JPRB, 0.62860E+00_JPRB, 0.62720E+00_JPRB ,&
&  0.62557E+00_JPRB, 0.60064E+00_JPRB, 0.56851E+00_JPRB, 0.56086E+00_JPRB, 0.56511E+00_JPRB, 0.61771E+00_JPRB, 0.75722E+00_JPRB ,&
&  0.84721E+00_JPRB, 0.52600E+00_JPRB, 0.50736E+00_JPRB, 0.49010E+00_JPRB, 0.50275E+00_JPRB, 0.54394E+00_JPRB, 0.63353E+00_JPRB ,&
&  0.85341E+00_JPRB, 0.15657E+01_JPRB, 0.24729E+01_JPRB, 0.18937E+01_JPRB, 0.21729E+01_JPRB, 0.14844E+01_JPRB, 0.58481E+00_JPRB ,&
&  0.44718E+00_JPRB, 0.43797E+00_JPRB, 0.44297E+00_JPRB, 0.43208E+00_JPRB, 0.41891E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.41201E+00_JPRB, 0.40214E+00_JPRB, 0.40858E+00_JPRB, 0.41767E+00_JPRB, 0.41343E+00_JPRB, 0.41255E+00_JPRB, 0.40755E+00_JPRB ,&
&  0.39810E+00_JPRB, 0.39214E+00_JPRB, 0.41301E+00_JPRB, 0.45277E+00_JPRB, 0.50697E+00_JPRB, 0.57662E+00_JPRB, 0.64267E+00_JPRB ,&
&  0.68014E+00_JPRB, 0.72044E+00_JPRB, 0.75170E+00_JPRB, 0.77419E+00_JPRB, 0.76451E+00_JPRB, 0.75299E+00_JPRB, 0.72841E+00_JPRB ,&
&  0.70796E+00_JPRB, 0.70436E+00_JPRB, 0.72175E+00_JPRB, 0.72921E+00_JPRB, 0.75496E+00_JPRB, 0.78087E+00_JPRB, 0.51305E+00_JPRB ,&
&  0.83291E+00_JPRB, 0.84182E+00_JPRB, 0.77942E+00_JPRB, 0.70120E+00_JPRB, 0.65998E+00_JPRB, 0.60042E+00_JPRB, 0.53872E+00_JPRB ,&
&  0.51108E+00_JPRB, 0.50089E+00_JPRB, 0.49553E+00_JPRB, 0.56596E+00_JPRB, 0.67288E+00_JPRB, 0.75023E+00_JPRB, 0.81521E+00_JPRB ,&
&  0.84293E+00_JPRB, 0.87942E+00_JPRB, 0.90178E+00_JPRB, 0.92760E+00_JPRB, 0.94731E+00_JPRB, 0.93466E+00_JPRB, 0.89382E+00_JPRB ,&
&  0.83692E+00_JPRB, 0.80741E+00_JPRB, 0.80813E+00_JPRB, 0.86785E+00_JPRB, 0.84536E+00_JPRB, 0.81889E+00_JPRB, 0.71822E+00_JPRB ,&
&  0.58196E+00_JPRB, 0.56872E+00_JPRB, 0.63999E+00_JPRB, 0.84132E+00_JPRB, 0.86216E+00_JPRB, 0.82740E+00_JPRB, 0.77723E+00_JPRB ,&
&  0.72932E+00_JPRB, 0.68536E+00_JPRB, 0.64903E+00_JPRB, 0.61725E+00_JPRB, 0.59835E+00_JPRB, 0.58789E+00_JPRB, 0.57697E+00_JPRB ,&
&  0.56165E+00_JPRB, 0.53258E+00_JPRB, 0.50134E+00_JPRB, 0.48771E+00_JPRB, 0.47585E+00_JPRB, 0.47569E+00_JPRB, 0.47597E+00_JPRB ,&
&  0.57417E+00_JPRB, 0.46505E+00_JPRB, 0.43394E+00_JPRB, 0.43711E+00_JPRB, 0.41484E+00_JPRB, 0.46683E+00_JPRB, 0.64458E+00_JPRB ,&
&  0.10655E+01_JPRB, 0.16086E+01_JPRB, 0.27544E+01_JPRB, 0.49841E+01_JPRB, 0.27456E+01_JPRB, 0.21093E+01_JPRB, 0.48392E+00_JPRB ,&
&  0.46102E+00_JPRB, 0.46270E+00_JPRB, 0.45888E+00_JPRB, 0.44097E+00_JPRB, 0.43579E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.45109E+00_JPRB, 0.44858E+00_JPRB, 0.45478E+00_JPRB, 0.46168E+00_JPRB, 0.46270E+00_JPRB, 0.46292E+00_JPRB, 0.46214E+00_JPRB ,&
&  0.46296E+00_JPRB, 0.46138E+00_JPRB, 0.46223E+00_JPRB, 0.46852E+00_JPRB, 0.48233E+00_JPRB, 0.51281E+00_JPRB, 0.54749E+00_JPRB ,&
&  0.57685E+00_JPRB, 0.60265E+00_JPRB, 0.62557E+00_JPRB, 0.64404E+00_JPRB, 0.66190E+00_JPRB, 0.66994E+00_JPRB, 0.70670E+00_JPRB ,&
&  0.75344E+00_JPRB, 0.81074E+00_JPRB, 0.86602E+00_JPRB, 0.89919E+00_JPRB, 0.91435E+00_JPRB, 0.83956E+00_JPRB, 0.62980E+00_JPRB ,&
&  0.10049E+01_JPRB, 0.99797E+00_JPRB, 0.88414E+00_JPRB, 0.78411E+00_JPRB, 0.71856E+00_JPRB, 0.66482E+00_JPRB, 0.60867E+00_JPRB ,&
&  0.55422E+00_JPRB, 0.54939E+00_JPRB, 0.50917E+00_JPRB, 0.50416E+00_JPRB, 0.51776E+00_JPRB, 0.55683E+00_JPRB, 0.64765E+00_JPRB ,&
&  0.70252E+00_JPRB, 0.73752E+00_JPRB, 0.77872E+00_JPRB, 0.83744E+00_JPRB, 0.86450E+00_JPRB, 0.84827E+00_JPRB, 0.76593E+00_JPRB ,&
&  0.71733E+00_JPRB, 0.69586E+00_JPRB, 0.65768E+00_JPRB, 0.69634E+00_JPRB, 0.71862E+00_JPRB, 0.70673E+00_JPRB, 0.68036E+00_JPRB ,&
&  0.56583E+00_JPRB, 0.57944E+00_JPRB, 0.67127E+00_JPRB, 0.89354E+00_JPRB, 0.84552E+00_JPRB, 0.79043E+00_JPRB, 0.77959E+00_JPRB ,&
&  0.73521E+00_JPRB, 0.69722E+00_JPRB, 0.66876E+00_JPRB, 0.63520E+00_JPRB, 0.60665E+00_JPRB, 0.59001E+00_JPRB, 0.56716E+00_JPRB ,&
&  0.53800E+00_JPRB, 0.50144E+00_JPRB, 0.46929E+00_JPRB, 0.44491E+00_JPRB, 0.42944E+00_JPRB, 0.37536E+00_JPRB, 0.45931E+00_JPRB ,&
&  0.46932E+00_JPRB, 0.42026E+00_JPRB, 0.44335E+00_JPRB, 0.34207E+00_JPRB, 0.42890E+00_JPRB, 0.48406E+00_JPRB, 0.60942E+00_JPRB ,&
&  0.78027E+00_JPRB, 0.12572E+01_JPRB, 0.19887E+01_JPRB, 0.17925E+01_JPRB, 0.13599E+01_JPRB, 0.86031E+00_JPRB, 0.49855E+00_JPRB ,&
&  0.47668E+00_JPRB, 0.47144E+00_JPRB, 0.47504E+00_JPRB, 0.46413E+00_JPRB, 0.45949E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.47916E+00_JPRB, 0.47788E+00_JPRB, 0.48980E+00_JPRB, 0.50177E+00_JPRB, 0.50800E+00_JPRB, 0.51125E+00_JPRB, 0.51739E+00_JPRB ,&
&  0.51872E+00_JPRB, 0.52259E+00_JPRB, 0.52761E+00_JPRB, 0.53956E+00_JPRB, 0.55902E+00_JPRB, 0.59021E+00_JPRB, 0.62549E+00_JPRB ,&
&  0.66774E+00_JPRB, 0.71336E+00_JPRB, 0.76301E+00_JPRB, 0.81629E+00_JPRB, 0.88307E+00_JPRB, 0.97467E+00_JPRB, 0.11011E+01_JPRB ,&
&  0.11673E+01_JPRB, 0.12498E+01_JPRB, 0.13447E+01_JPRB, 0.14161E+01_JPRB, 0.14048E+01_JPRB, 0.13408E+01_JPRB, 0.10434E+01_JPRB ,&
&  0.11348E+01_JPRB, 0.10961E+01_JPRB, 0.97931E+00_JPRB, 0.86951E+00_JPRB, 0.79845E+00_JPRB, 0.74288E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.69111E+00_JPRB, 0.66275E+00_JPRB, 0.61975E+00_JPRB, 0.59017E+00_JPRB, 0.57313E+00_JPRB, 0.57816E+00_JPRB, 0.57208E+00_JPRB ,&
&  0.62011E+00_JPRB, 0.71291E+00_JPRB, 0.83230E+00_JPRB, 0.98416E+00_JPRB, 0.10658E+01_JPRB, 0.10512E+01_JPRB, 0.92684E+00_JPRB ,&
&  0.83007E+00_JPRB, 0.75846E+00_JPRB, 0.73834E+00_JPRB, 0.77429E+00_JPRB, 0.79760E+00_JPRB, 0.83457E+00_JPRB, 0.83124E+00_JPRB ,&
&  0.74902E+00_JPRB, 0.67795E+00_JPRB, 0.72418E+00_JPRB, 0.90445E+00_JPRB, 0.83680E+00_JPRB, 0.75656E+00_JPRB, 0.76181E+00_JPRB ,&
&  0.71524E+00_JPRB, 0.67827E+00_JPRB, 0.66192E+00_JPRB, 0.63076E+00_JPRB, 0.59863E+00_JPRB, 0.57747E+00_JPRB, 0.56152E+00_JPRB ,&
&  0.52686E+00_JPRB, 0.48727E+00_JPRB, 0.45380E+00_JPRB, 0.44377E+00_JPRB, 0.46340E+00_JPRB, 0.44359E+00_JPRB, 0.50009E+00_JPRB ,&
&  0.47723E+00_JPRB, 0.38023E+00_JPRB, 0.42865E+00_JPRB, 0.51770E+00_JPRB, 0.51859E+00_JPRB, 0.55806E+00_JPRB, 0.62780E+00_JPRB ,&
&  0.70133E+00_JPRB, 0.76598E+00_JPRB, 0.75818E+00_JPRB, 0.64626E+00_JPRB, 0.60177E+00_JPRB, 0.57715E+00_JPRB, 0.55080E+00_JPRB ,&
&  0.52962E+00_JPRB, 0.49274E+00_JPRB, 0.50336E+00_JPRB, 0.48914E+00_JPRB, 0.48498E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.52365E+00_JPRB, 0.51882E+00_JPRB, 0.51943E+00_JPRB, 0.51869E+00_JPRB, 0.52783E+00_JPRB, 0.52858E+00_JPRB, 0.52721E+00_JPRB ,&
&  0.52435E+00_JPRB, 0.53086E+00_JPRB, 0.53463E+00_JPRB, 0.54138E+00_JPRB, 0.54985E+00_JPRB, 0.57376E+00_JPRB, 0.60069E+00_JPRB ,&
&  0.63292E+00_JPRB, 0.67183E+00_JPRB, 0.71316E+00_JPRB, 0.77125E+00_JPRB, 0.85226E+00_JPRB, 0.90155E+00_JPRB, 0.95805E+00_JPRB ,&
&  0.10288E+01_JPRB, 0.11392E+01_JPRB, 0.12577E+01_JPRB, 0.14185E+01_JPRB, 0.15900E+01_JPRB, 0.15771E+01_JPRB, 0.14470E+01_JPRB ,&
&  0.10834E+01_JPRB, 0.10887E+01_JPRB, 0.10062E+01_JPRB, 0.90277E+00_JPRB, 0.90114E+00_JPRB, 0.87339E+00_JPRB, 0.88114E+00_JPRB ,&
&  0.90218E+00_JPRB, 0.90040E+00_JPRB, 0.89161E+00_JPRB, 0.89674E+00_JPRB, 0.90037E+00_JPRB, 0.90561E+00_JPRB, 0.89110E+00_JPRB ,&
&  0.88503E+00_JPRB, 0.91505E+00_JPRB, 0.10531E+01_JPRB, 0.12272E+01_JPRB, 0.13424E+01_JPRB, 0.13639E+01_JPRB, 0.13297E+01_JPRB ,&
&  0.12399E+01_JPRB, 0.11496E+01_JPRB, 0.10651E+01_JPRB, 0.10511E+01_JPRB, 0.10783E+01_JPRB, 0.10908E+01_JPRB, 0.11513E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.86372E+00_JPRB, 0.84275E+00_JPRB, 0.83693E+00_JPRB, 0.79006E+00_JPRB, 0.75766E+00_JPRB, 0.72550E+00_JPRB ,&
&  0.72246E+00_JPRB, 0.68604E+00_JPRB, 0.66521E+00_JPRB, 0.62996E+00_JPRB, 0.59473E+00_JPRB, 0.57270E+00_JPRB, 0.55802E+00_JPRB ,&
&  0.52975E+00_JPRB, 0.50311E+00_JPRB, 0.48457E+00_JPRB, 0.49188E+00_JPRB, 0.49086E+00_JPRB, 0.59981E+00_JPRB, 0.55405E+00_JPRB ,&
&  0.54039E+00_JPRB, 0.49831E+00_JPRB, 0.45954E+00_JPRB, 0.59417E+00_JPRB, 0.61173E+00_JPRB, 0.62273E+00_JPRB, 0.62892E+00_JPRB ,&
&  0.63409E+00_JPRB, 0.62971E+00_JPRB, 0.60594E+00_JPRB, 0.59934E+00_JPRB, 0.60639E+00_JPRB, 0.61164E+00_JPRB, 0.59957E+00_JPRB ,&
&  0.56341E+00_JPRB, 0.54578E+00_JPRB, 0.56723E+00_JPRB, 0.53325E+00_JPRB, 0.52733E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.59615E+00_JPRB, 0.57052E+00_JPRB, 0.55460E+00_JPRB, 0.56131E+00_JPRB, 0.56908E+00_JPRB, 0.55799E+00_JPRB, 0.58486E+00_JPRB ,&
&  0.56076E+00_JPRB, 0.57938E+00_JPRB, 0.59024E+00_JPRB, 0.58743E+00_JPRB, 0.62203E+00_JPRB, 0.65691E+00_JPRB, 0.70100E+00_JPRB ,&
&  0.76115E+00_JPRB, 0.85733E+00_JPRB, 0.97011E+00_JPRB, 0.11066E+01_JPRB, 0.12700E+01_JPRB, 0.13372E+01_JPRB, 0.14028E+01_JPRB ,&
&  0.14956E+01_JPRB, 0.16173E+01_JPRB, 0.17983E+01_JPRB, 0.20111E+01_JPRB, 0.22751E+01_JPRB, 0.20762E+01_JPRB, 0.17562E+01_JPRB ,&
&  0.13039E+01_JPRB, 0.10249E+01_JPRB, 0.98158E+00_JPRB, 0.90842E+00_JPRB, 0.94176E+00_JPRB, 0.10334E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.10947E+01_JPRB, 0.11221E+01_JPRB, 0.11505E+01_JPRB, 0.11844E+01_JPRB, 0.12245E+01_JPRB, 0.12675E+01_JPRB, 0.12886E+01_JPRB ,&
&  0.13047E+01_JPRB, 0.13215E+01_JPRB, 0.13508E+01_JPRB, 0.14334E+01_JPRB, 0.15090E+01_JPRB, 0.15617E+01_JPRB, 0.16024E+01_JPRB ,&
&  0.16343E+01_JPRB, 0.15283E+01_JPRB, 0.13019E+01_JPRB, 0.12951E+01_JPRB, 0.13289E+01_JPRB, 0.13470E+01_JPRB, 0.13512E+01_JPRB ,&
&  0.14482E+01_JPRB, 0.11680E+01_JPRB, 0.78869E+00_JPRB, 0.77987E+00_JPRB, 0.75557E+00_JPRB, 0.76812E+00_JPRB, 0.70266E+00_JPRB ,&
&  0.70493E+00_JPRB, 0.69874E+00_JPRB, 0.69365E+00_JPRB, 0.65998E+00_JPRB, 0.63994E+00_JPRB, 0.61868E+00_JPRB, 0.58905E+00_JPRB ,&
&  0.54950E+00_JPRB, 0.53719E+00_JPRB, 0.52063E+00_JPRB, 0.51931E+00_JPRB, 0.56742E+00_JPRB, 0.53903E+00_JPRB, 0.57890E+00_JPRB ,&
&  0.59025E+00_JPRB, 0.64701E+00_JPRB, 0.58309E+00_JPRB, 0.68401E+00_JPRB, 0.68141E+00_JPRB, 0.67584E+00_JPRB, 0.66758E+00_JPRB ,&
&  0.68072E+00_JPRB, 0.68963E+00_JPRB, 0.69801E+00_JPRB, 0.68627E+00_JPRB, 0.70134E+00_JPRB, 0.72463E+00_JPRB, 0.72095E+00_JPRB ,&
&  0.67841E+00_JPRB, 0.66165E+00_JPRB, 0.63851E+00_JPRB, 0.61033E+00_JPRB, 0.60544E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.63870E+00_JPRB, 0.64271E+00_JPRB, 0.64261E+00_JPRB, 0.64818E+00_JPRB, 0.65312E+00_JPRB, 0.64108E+00_JPRB, 0.66200E+00_JPRB ,&
&  0.65702E+00_JPRB, 0.67688E+00_JPRB, 0.69961E+00_JPRB, 0.72281E+00_JPRB, 0.76711E+00_JPRB, 0.81168E+00_JPRB, 0.87207E+00_JPRB ,&
&  0.97189E+00_JPRB, 0.11315E+01_JPRB, 0.13256E+01_JPRB, 0.15594E+01_JPRB, 0.16031E+01_JPRB, 0.16159E+01_JPRB, 0.16786E+01_JPRB ,&
&  0.17511E+01_JPRB, 0.17890E+01_JPRB, 0.17555E+01_JPRB, 0.17473E+01_JPRB, 0.17424E+01_JPRB, 0.16205E+01_JPRB, 0.14981E+01_JPRB ,&
&  0.12043E+01_JPRB, 0.99224E+00_JPRB, 0.10791E+01_JPRB, 0.10267E+01_JPRB, 0.10585E+01_JPRB, 0.10872E+01_JPRB, 0.11132E+01_JPRB ,&
&  0.11384E+01_JPRB, 0.11492E+01_JPRB, 0.11635E+01_JPRB, 0.11848E+01_JPRB, 0.12199E+01_JPRB, 0.12498E+01_JPRB, 0.12854E+01_JPRB ,&
&  0.13154E+01_JPRB, 0.13433E+01_JPRB, 0.13794E+01_JPRB, 0.13446E+01_JPRB, 0.13788E+01_JPRB, 0.14174E+01_JPRB, 0.14684E+01_JPRB ,&
&  0.15247E+01_JPRB, 0.14603E+01_JPRB, 0.12621E+01_JPRB, 0.13326E+01_JPRB, 0.14516E+01_JPRB, 0.13787E+01_JPRB, 0.14118E+01_JPRB ,&
&  0.15867E+01_JPRB, 0.14525E+01_JPRB, 0.79482E+00_JPRB, 0.68072E+00_JPRB, 0.68774E+00_JPRB, 0.71005E+00_JPRB, 0.73230E+00_JPRB ,&
&  0.72706E+00_JPRB, 0.72143E+00_JPRB, 0.71759E+00_JPRB, 0.71375E+00_JPRB, 0.69020E+00_JPRB, 0.68102E+00_JPRB, 0.63114E+00_JPRB ,&
&  0.53152E+00_JPRB, 0.52147E+00_JPRB, 0.52642E+00_JPRB, 0.53857E+00_JPRB, 0.57235E+00_JPRB, 0.58774E+00_JPRB, 0.60523E+00_JPRB ,&
&  0.65150E+00_JPRB, 0.70687E+00_JPRB, 0.74985E+00_JPRB, 0.68979E+00_JPRB, 0.64352E+00_JPRB, 0.72592E+00_JPRB, 0.72800E+00_JPRB ,&
&  0.74114E+00_JPRB, 0.74194E+00_JPRB, 0.74991E+00_JPRB, 0.75185E+00_JPRB, 0.75717E+00_JPRB, 0.74178E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.70452E+00_JPRB, 0.68813E+00_JPRB, 0.67707E+00_JPRB, 0.65852E+00_JPRB, 0.64467E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.68757E+00_JPRB, 0.66913E+00_JPRB, 0.65934E+00_JPRB, 0.64808E+00_JPRB, 0.64573E+00_JPRB, 0.64226E+00_JPRB, 0.64225E+00_JPRB ,&
&  0.64351E+00_JPRB, 0.64871E+00_JPRB, 0.66817E+00_JPRB, 0.70215E+00_JPRB, 0.74941E+00_JPRB, 0.81262E+00_JPRB, 0.91829E+00_JPRB ,&
&  0.10212E+01_JPRB, 0.11725E+01_JPRB, 0.13479E+01_JPRB, 0.15434E+01_JPRB, 0.16651E+01_JPRB, 0.16810E+01_JPRB, 0.17502E+01_JPRB ,&
&  0.17822E+01_JPRB, 0.17625E+01_JPRB, 0.16623E+01_JPRB, 0.15196E+01_JPRB, 0.13945E+01_JPRB, 0.13428E+01_JPRB, 0.13364E+01_JPRB ,&
&  0.11747E+01_JPRB, 0.99308E+00_JPRB, 0.99252E+00_JPRB, 0.10179E+01_JPRB, 0.10363E+01_JPRB, 0.10518E+01_JPRB, 0.10830E+01_JPRB ,&
&  0.11080E+01_JPRB, 0.11231E+01_JPRB, 0.11311E+01_JPRB, 0.11408E+01_JPRB, 0.11821E+01_JPRB, 0.11912E+01_JPRB, 0.12121E+01_JPRB ,&
&  0.12265E+01_JPRB, 0.12426E+01_JPRB, 0.12933E+01_JPRB, 0.12306E+01_JPRB, 0.13175E+01_JPRB, 0.13109E+01_JPRB, 0.12754E+01_JPRB ,&
&  0.12331E+01_JPRB, 0.11410E+01_JPRB, 0.11638E+01_JPRB, 0.12415E+01_JPRB, 0.13256E+01_JPRB, 0.12999E+01_JPRB, 0.12354E+01_JPRB ,&
&  0.13488E+01_JPRB, 0.13531E+01_JPRB, 0.82706E+00_JPRB, 0.70305E+00_JPRB, 0.71215E+00_JPRB, 0.66745E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.75273E+00_JPRB, 0.74869E+00_JPRB, 0.74403E+00_JPRB, 0.74861E+00_JPRB, 0.75754E+00_JPRB, 0.82090E+00_JPRB, 0.67760E+00_JPRB ,&
&  0.58326E+00_JPRB, 0.54902E+00_JPRB, 0.53888E+00_JPRB, 0.55033E+00_JPRB, 0.58552E+00_JPRB, 0.61751E+00_JPRB, 0.66522E+00_JPRB ,&
&  0.71050E+00_JPRB, 0.76662E+00_JPRB, 0.82238E+00_JPRB, 0.81681E+00_JPRB, 0.74057E+00_JPRB, 0.78721E+00_JPRB, 0.80414E+00_JPRB ,&
&  0.81389E+00_JPRB, 0.81351E+00_JPRB, 0.80133E+00_JPRB, 0.79845E+00_JPRB, 0.78334E+00_JPRB, 0.77581E+00_JPRB, 0.76698E+00_JPRB ,&
&  0.75506E+00_JPRB, 0.75190E+00_JPRB, 0.74280E+00_JPRB, 0.71457E+00_JPRB, 0.69843E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.73719E+00_JPRB, 0.71472E+00_JPRB, 0.69626E+00_JPRB, 0.68120E+00_JPRB, 0.67273E+00_JPRB, 0.65366E+00_JPRB, 0.62858E+00_JPRB ,&
&  0.61766E+00_JPRB, 0.61540E+00_JPRB, 0.62889E+00_JPRB, 0.65688E+00_JPRB, 0.69836E+00_JPRB, 0.74186E+00_JPRB, 0.81996E+00_JPRB ,&
&  0.91274E+00_JPRB, 0.10480E+01_JPRB, 0.12198E+01_JPRB, 0.14538E+01_JPRB, 0.16394E+01_JPRB, 0.18064E+01_JPRB, 0.18350E+01_JPRB ,&
&  0.19076E+01_JPRB, 0.18995E+01_JPRB, 0.17542E+01_JPRB, 0.15139E+01_JPRB, 0.11471E+01_JPRB, 0.10883E+01_JPRB, 0.11537E+01_JPRB ,&
&  0.11532E+01_JPRB, 0.10353E+01_JPRB, 0.98005E+00_JPRB, 0.97318E+00_JPRB, 0.96484E+00_JPRB, 0.99019E+00_JPRB, 0.10165E+01_JPRB ,&
&  0.10435E+01_JPRB, 0.10653E+01_JPRB, 0.10809E+01_JPRB, 0.11060E+01_JPRB, 0.11276E+01_JPRB, 0.11302E+01_JPRB, 0.11468E+01_JPRB ,&
&  0.11337E+01_JPRB, 0.11284E+01_JPRB, 0.11241E+01_JPRB, 0.11062E+01_JPRB, 0.11032E+01_JPRB, 0.10868E+01_JPRB, 0.10549E+01_JPRB ,&
&  0.10290E+01_JPRB, 0.96096E+00_JPRB, 0.99058E+00_JPRB, 0.10287E+01_JPRB, 0.10297E+01_JPRB, 0.98015E+00_JPRB, 0.98570E+00_JPRB ,&
&  0.10981E+01_JPRB, 0.12599E+01_JPRB, 0.82071E+00_JPRB, 0.80319E+00_JPRB, 0.84952E+00_JPRB, 0.77760E+00_JPRB, 0.75148E+00_JPRB ,&
&  0.76146E+00_JPRB, 0.75475E+00_JPRB, 0.75387E+00_JPRB, 0.75835E+00_JPRB, 0.79056E+00_JPRB, 0.87189E+00_JPRB, 0.70629E+00_JPRB ,&
&  0.65419E+00_JPRB, 0.62191E+00_JPRB, 0.62075E+00_JPRB, 0.63092E+00_JPRB, 0.63239E+00_JPRB, 0.60320E+00_JPRB, 0.60537E+00_JPRB ,&
&  0.62948E+00_JPRB, 0.89993E+00_JPRB, 0.11257E+01_JPRB, 0.13373E+01_JPRB, 0.12777E+01_JPRB, 0.80541E+00_JPRB, 0.86907E+00_JPRB ,&
&  0.88381E+00_JPRB, 0.87774E+00_JPRB, 0.84104E+00_JPRB, 0.82864E+00_JPRB, 0.81820E+00_JPRB, 0.80499E+00_JPRB, 0.80202E+00_JPRB ,&
&  0.79392E+00_JPRB, 0.79682E+00_JPRB, 0.80346E+00_JPRB, 0.78533E+00_JPRB, 0.75851E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.79545E+00_JPRB, 0.77771E+00_JPRB, 0.75710E+00_JPRB, 0.73693E+00_JPRB, 0.71857E+00_JPRB, 0.67412E+00_JPRB, 0.62879E+00_JPRB ,&
&  0.59933E+00_JPRB, 0.58653E+00_JPRB, 0.59213E+00_JPRB, 0.60348E+00_JPRB, 0.63063E+00_JPRB, 0.66109E+00_JPRB, 0.71249E+00_JPRB ,&
&  0.79094E+00_JPRB, 0.90231E+00_JPRB, 0.10579E+01_JPRB, 0.12506E+01_JPRB, 0.14808E+01_JPRB, 0.17461E+01_JPRB, 0.19149E+01_JPRB ,&
&  0.20678E+01_JPRB, 0.20873E+01_JPRB, 0.18370E+01_JPRB, 0.12924E+01_JPRB, 0.11138E+01_JPRB, 0.10635E+01_JPRB, 0.97266E+00_JPRB ,&
&  0.96273E+00_JPRB, 0.94959E+00_JPRB, 0.93225E+00_JPRB, 0.91857E+00_JPRB, 0.91215E+00_JPRB, 0.92807E+00_JPRB, 0.94522E+00_JPRB ,&
&  0.96280E+00_JPRB, 0.97824E+00_JPRB, 0.99571E+00_JPRB, 0.10275E+01_JPRB, 0.10356E+01_JPRB, 0.10510E+01_JPRB, 0.10722E+01_JPRB ,&
&  0.10771E+01_JPRB, 0.10636E+01_JPRB, 0.10208E+01_JPRB, 0.99538E+00_JPRB, 0.96391E+00_JPRB, 0.93486E+00_JPRB, 0.90514E+00_JPRB ,&
&  0.89426E+00_JPRB, 0.87740E+00_JPRB, 0.87198E+00_JPRB, 0.88578E+00_JPRB, 0.88971E+00_JPRB, 0.82137E+00_JPRB, 0.84939E+00_JPRB ,&
&  0.95253E+00_JPRB, 0.10579E+01_JPRB, 0.87866E+00_JPRB, 0.90751E+00_JPRB, 0.64584E+00_JPRB, 0.71246E+00_JPRB, 0.77889E+00_JPRB ,&
&  0.76916E+00_JPRB, 0.76104E+00_JPRB, 0.74711E+00_JPRB, 0.77020E+00_JPRB, 0.80158E+00_JPRB, 0.79922E+00_JPRB, 0.75941E+00_JPRB ,&
&  0.71655E+00_JPRB, 0.68489E+00_JPRB, 0.69680E+00_JPRB, 0.71814E+00_JPRB, 0.71573E+00_JPRB, 0.66512E+00_JPRB, 0.61386E+00_JPRB ,&
&  0.60539E+00_JPRB, 0.80047E+00_JPRB, 0.90732E+00_JPRB, 0.10069E+01_JPRB, 0.92331E+00_JPRB, 0.77811E+00_JPRB, 0.82605E+00_JPRB ,&
&  0.86012E+00_JPRB, 0.87233E+00_JPRB, 0.86156E+00_JPRB, 0.84150E+00_JPRB, 0.81757E+00_JPRB, 0.79689E+00_JPRB, 0.78651E+00_JPRB ,&
&  0.78363E+00_JPRB, 0.78068E+00_JPRB, 0.78765E+00_JPRB, 0.79829E+00_JPRB, 0.80470E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.78773E+00_JPRB, 0.78533E+00_JPRB, 0.78612E+00_JPRB, 0.78737E+00_JPRB, 0.78069E+00_JPRB, 0.75282E+00_JPRB, 0.69383E+00_JPRB ,&
&  0.60713E+00_JPRB, 0.54115E+00_JPRB, 0.53348E+00_JPRB, 0.53749E+00_JPRB, 0.54907E+00_JPRB, 0.57908E+00_JPRB, 0.62750E+00_JPRB ,&
&  0.68334E+00_JPRB, 0.77606E+00_JPRB, 0.90883E+00_JPRB, 0.10771E+01_JPRB, 0.12963E+01_JPRB, 0.16210E+01_JPRB, 0.19673E+01_JPRB ,&
&  0.18992E+01_JPRB, 0.14174E+01_JPRB, 0.17662E+01_JPRB, 0.17665E+01_JPRB, 0.15530E+01_JPRB, 0.12574E+01_JPRB, 0.10498E+01_JPRB ,&
&  0.98458E+00_JPRB, 0.95934E+00_JPRB, 0.94305E+00_JPRB, 0.91613E+00_JPRB, 0.87159E+00_JPRB, 0.87862E+00_JPRB, 0.88981E+00_JPRB ,&
&  0.90332E+00_JPRB, 0.91570E+00_JPRB, 0.92353E+00_JPRB, 0.96058E+00_JPRB, 0.96806E+00_JPRB, 0.97748E+00_JPRB, 0.10001E+01_JPRB ,&
&  0.10036E+01_JPRB, 0.98891E+00_JPRB, 0.95122E+00_JPRB, 0.89273E+00_JPRB, 0.87283E+00_JPRB, 0.85186E+00_JPRB, 0.83723E+00_JPRB ,&
&  0.83947E+00_JPRB, 0.83098E+00_JPRB, 0.83993E+00_JPRB, 0.84429E+00_JPRB, 0.85753E+00_JPRB, 0.79841E+00_JPRB, 0.81790E+00_JPRB ,&
&  0.86138E+00_JPRB, 0.95527E+00_JPRB, 0.93147E+00_JPRB, 0.94513E+00_JPRB, 0.70708E+00_JPRB, 0.74664E+00_JPRB, 0.73897E+00_JPRB ,&
&  0.78885E+00_JPRB, 0.76230E+00_JPRB, 0.74009E+00_JPRB, 0.75149E+00_JPRB, 0.79007E+00_JPRB, 0.80890E+00_JPRB, 0.82208E+00_JPRB ,&
&  0.74875E+00_JPRB, 0.72612E+00_JPRB, 0.72829E+00_JPRB, 0.76239E+00_JPRB, 0.75075E+00_JPRB, 0.72004E+00_JPRB, 0.69557E+00_JPRB ,&
&  0.77436E+00_JPRB, 0.81582E+00_JPRB, 0.87614E+00_JPRB, 0.89984E+00_JPRB, 0.83536E+00_JPRB, 0.75983E+00_JPRB, 0.77590E+00_JPRB ,&
&  0.79731E+00_JPRB, 0.81445E+00_JPRB, 0.81333E+00_JPRB, 0.79662E+00_JPRB, 0.76726E+00_JPRB, 0.74181E+00_JPRB, 0.72862E+00_JPRB ,&
&  0.71841E+00_JPRB, 0.72528E+00_JPRB, 0.73580E+00_JPRB, 0.74987E+00_JPRB, 0.77081E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.69757E+00_JPRB, 0.71711E+00_JPRB, 0.73341E+00_JPRB, 0.77712E+00_JPRB, 0.83081E+00_JPRB, 0.90192E+00_JPRB, 0.93731E+00_JPRB ,&
&  0.79661E+00_JPRB, 0.51390E+00_JPRB, 0.50323E+00_JPRB, 0.49918E+00_JPRB, 0.49996E+00_JPRB, 0.51013E+00_JPRB, 0.55173E+00_JPRB ,&
&  0.61417E+00_JPRB, 0.69971E+00_JPRB, 0.79879E+00_JPRB, 0.91767E+00_JPRB, 0.10778E+01_JPRB, 0.13890E+01_JPRB, 0.16049E+01_JPRB ,&
&  0.13853E+01_JPRB, 0.16878E+01_JPRB, 0.21844E+01_JPRB, 0.19794E+01_JPRB, 0.17593E+01_JPRB, 0.15081E+01_JPRB, 0.11270E+01_JPRB ,&
&  0.10576E+01_JPRB, 0.99679E+00_JPRB, 0.98062E+00_JPRB, 0.98253E+00_JPRB, 0.88072E+00_JPRB, 0.86953E+00_JPRB, 0.87023E+00_JPRB ,&
&  0.87541E+00_JPRB, 0.87773E+00_JPRB, 0.88016E+00_JPRB, 0.92178E+00_JPRB, 0.95218E+00_JPRB, 0.96329E+00_JPRB, 0.96248E+00_JPRB ,&
&  0.95603E+00_JPRB, 0.94617E+00_JPRB, 0.91681E+00_JPRB, 0.84820E+00_JPRB, 0.83818E+00_JPRB, 0.83252E+00_JPRB, 0.84087E+00_JPRB ,&
&  0.85752E+00_JPRB, 0.90345E+00_JPRB, 0.92468E+00_JPRB, 0.88161E+00_JPRB, 0.81650E+00_JPRB, 0.85829E+00_JPRB, 0.79612E+00_JPRB ,&
&  0.80325E+00_JPRB, 0.84352E+00_JPRB, 0.98858E+00_JPRB, 0.80597E+00_JPRB, 0.76835E+00_JPRB, 0.80931E+00_JPRB, 0.81410E+00_JPRB ,&
&  0.80133E+00_JPRB, 0.80937E+00_JPRB, 0.76361E+00_JPRB, 0.73946E+00_JPRB, 0.76319E+00_JPRB, 0.77257E+00_JPRB, 0.87283E+00_JPRB ,&
&  0.80586E+00_JPRB, 0.84505E+00_JPRB, 0.78568E+00_JPRB, 0.77844E+00_JPRB, 0.74324E+00_JPRB, 0.69633E+00_JPRB, 0.69617E+00_JPRB ,&
&  0.82478E+00_JPRB, 0.85706E+00_JPRB, 0.87305E+00_JPRB, 0.88775E+00_JPRB, 0.85887E+00_JPRB, 0.83990E+00_JPRB, 0.80411E+00_JPRB ,&
&  0.78679E+00_JPRB, 0.78594E+00_JPRB, 0.78634E+00_JPRB, 0.75811E+00_JPRB, 0.73263E+00_JPRB, 0.69772E+00_JPRB, 0.66904E+00_JPRB ,&
&  0.65279E+00_JPRB, 0.64578E+00_JPRB, 0.64668E+00_JPRB, 0.65883E+00_JPRB, 0.67607E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.62851E+00_JPRB, 0.63747E+00_JPRB, 0.63685E+00_JPRB, 0.61333E+00_JPRB, 0.59934E+00_JPRB, 0.58724E+00_JPRB, 0.55291E+00_JPRB ,&
&  0.52680E+00_JPRB, 0.50924E+00_JPRB, 0.49301E+00_JPRB, 0.48301E+00_JPRB, 0.48435E+00_JPRB, 0.48083E+00_JPRB, 0.49414E+00_JPRB ,&
&  0.54295E+00_JPRB, 0.61929E+00_JPRB, 0.73863E+00_JPRB, 0.81759E+00_JPRB, 0.91561E+00_JPRB, 0.11164E+01_JPRB, 0.12153E+01_JPRB ,&
&  0.12132E+01_JPRB, 0.23059E+01_JPRB, 0.21940E+01_JPRB, 0.21620E+01_JPRB, 0.20287E+01_JPRB, 0.17946E+01_JPRB, 0.13377E+01_JPRB ,&
&  0.11465E+01_JPRB, 0.10559E+01_JPRB, 0.98652E+00_JPRB, 0.95027E+00_JPRB, 0.93142E+00_JPRB, 0.88765E+00_JPRB, 0.88740E+00_JPRB ,&
&  0.88284E+00_JPRB, 0.87723E+00_JPRB, 0.86435E+00_JPRB, 0.89154E+00_JPRB, 0.93335E+00_JPRB, 0.95585E+00_JPRB, 0.97185E+00_JPRB ,&
&  0.96373E+00_JPRB, 0.95152E+00_JPRB, 0.91601E+00_JPRB, 0.88853E+00_JPRB, 0.88021E+00_JPRB, 0.90095E+00_JPRB, 0.93156E+00_JPRB ,&
&  0.93652E+00_JPRB, 0.93876E+00_JPRB, 0.10194E+01_JPRB, 0.10151E+01_JPRB, 0.97009E+00_JPRB, 0.96668E+00_JPRB, 0.84454E+00_JPRB ,&
&  0.83291E+00_JPRB, 0.78285E+00_JPRB, 0.89323E+00_JPRB, 0.79327E+00_JPRB, 0.79640E+00_JPRB, 0.83401E+00_JPRB, 0.84177E+00_JPRB ,&
&  0.81223E+00_JPRB, 0.84666E+00_JPRB, 0.81088E+00_JPRB, 0.77458E+00_JPRB, 0.75762E+00_JPRB, 0.77051E+00_JPRB, 0.84366E+00_JPRB ,&
&  0.87939E+00_JPRB, 0.88634E+00_JPRB, 0.91257E+00_JPRB, 0.81639E+00_JPRB, 0.78876E+00_JPRB, 0.71680E+00_JPRB, 0.71587E+00_JPRB ,&
&  0.83355E+00_JPRB, 0.85067E+00_JPRB, 0.89850E+00_JPRB, 0.90653E+00_JPRB, 0.88689E+00_JPRB, 0.90342E+00_JPRB, 0.87030E+00_JPRB ,&
&  0.83100E+00_JPRB, 0.80438E+00_JPRB, 0.78622E+00_JPRB, 0.76201E+00_JPRB, 0.72320E+00_JPRB, 0.68218E+00_JPRB, 0.65089E+00_JPRB ,&
&  0.62231E+00_JPRB, 0.60421E+00_JPRB, 0.60117E+00_JPRB, 0.60433E+00_JPRB, 0.61449E+00_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.58991E+00_JPRB, 0.59645E+00_JPRB, 0.61157E+00_JPRB, 0.59867E+00_JPRB, 0.57875E+00_JPRB, 0.56020E+00_JPRB, 0.54899E+00_JPRB ,&
&  0.53114E+00_JPRB, 0.50715E+00_JPRB, 0.48746E+00_JPRB, 0.47684E+00_JPRB, 0.47058E+00_JPRB, 0.47205E+00_JPRB, 0.48132E+00_JPRB ,&
&  0.50842E+00_JPRB, 0.54595E+00_JPRB, 0.62305E+00_JPRB, 0.71217E+00_JPRB, 0.79002E+00_JPRB, 0.92667E+00_JPRB, 0.83300E+00_JPRB ,&
&  0.11420E+01_JPRB, 0.21337E+01_JPRB, 0.22408E+01_JPRB, 0.22758E+01_JPRB, 0.22066E+01_JPRB, 0.21326E+01_JPRB, 0.16632E+01_JPRB ,&
&  0.13787E+01_JPRB, 0.12393E+01_JPRB, 0.10723E+01_JPRB, 0.99375E+00_JPRB, 0.96429E+00_JPRB, 0.94873E+00_JPRB, 0.92997E+00_JPRB ,&
&  0.90687E+00_JPRB, 0.88454E+00_JPRB, 0.87556E+00_JPRB, 0.89992E+00_JPRB, 0.94043E+00_JPRB, 0.98831E+00_JPRB, 0.10030E+01_JPRB ,&
&  0.99137E+00_JPRB, 0.97357E+00_JPRB, 0.97040E+00_JPRB, 0.92056E+00_JPRB, 0.95149E+00_JPRB, 0.97822E+00_JPRB, 0.10112E+01_JPRB ,&
&  0.98456E+00_JPRB, 0.96584E+00_JPRB, 0.11054E+01_JPRB, 0.12083E+01_JPRB, 0.12065E+01_JPRB, 0.10934E+01_JPRB, 0.95840E+00_JPRB ,&
&  0.89482E+00_JPRB, 0.82616E+00_JPRB, 0.83254E+00_JPRB, 0.79724E+00_JPRB, 0.81372E+00_JPRB, 0.83710E+00_JPRB, 0.87329E+00_JPRB ,&
&  0.90004E+00_JPRB, 0.88168E+00_JPRB, 0.84903E+00_JPRB, 0.79033E+00_JPRB, 0.77113E+00_JPRB, 0.75093E+00_JPRB, 0.84113E+00_JPRB ,&
&  0.95385E+00_JPRB, 0.10269E+01_JPRB, 0.10549E+01_JPRB, 0.94091E+00_JPRB, 0.86039E+00_JPRB, 0.72349E+00_JPRB, 0.67465E+00_JPRB ,&
&  0.78084E+00_JPRB, 0.79882E+00_JPRB, 0.76005E+00_JPRB, 0.84391E+00_JPRB, 0.82875E+00_JPRB, 0.85881E+00_JPRB, 0.86948E+00_JPRB ,&
&  0.81615E+00_JPRB, 0.78520E+00_JPRB, 0.75297E+00_JPRB, 0.73659E+00_JPRB, 0.73455E+00_JPRB, 0.70381E+00_JPRB, 0.67347E+00_JPRB ,&
&  0.64380E+00_JPRB, 0.62273E+00_JPRB, 0.60502E+00_JPRB, 0.59407E+00_JPRB, 0.58725E+00_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.58118E+00_JPRB, 0.57607E+00_JPRB, 0.57920E+00_JPRB, 0.58822E+00_JPRB, 0.59033E+00_JPRB, 0.57269E+00_JPRB, 0.55357E+00_JPRB ,&
&  0.53176E+00_JPRB, 0.50642E+00_JPRB, 0.48818E+00_JPRB, 0.47323E+00_JPRB, 0.46750E+00_JPRB, 0.46813E+00_JPRB, 0.47638E+00_JPRB ,&
&  0.49447E+00_JPRB, 0.52160E+00_JPRB, 0.59562E+00_JPRB, 0.65525E+00_JPRB, 0.69859E+00_JPRB, 0.67279E+00_JPRB, 0.78999E+00_JPRB ,&
&  0.14204E+01_JPRB, 0.21367E+01_JPRB, 0.21315E+01_JPRB, 0.22297E+01_JPRB, 0.23385E+01_JPRB, 0.24866E+01_JPRB, 0.19655E+01_JPRB ,&
&  0.16257E+01_JPRB, 0.14689E+01_JPRB, 0.12629E+01_JPRB, 0.11322E+01_JPRB, 0.10455E+01_JPRB, 0.10033E+01_JPRB, 0.98469E+00_JPRB ,&
&  0.95445E+00_JPRB, 0.91977E+00_JPRB, 0.89549E+00_JPRB, 0.91414E+00_JPRB, 0.97786E+00_JPRB, 0.10278E+01_JPRB, 0.10518E+01_JPRB ,&
&  0.10206E+01_JPRB, 0.98119E+00_JPRB, 0.97689E+00_JPRB, 0.96227E+00_JPRB, 0.98790E+00_JPRB, 0.10071E+01_JPRB, 0.10431E+01_JPRB ,&
&  0.10557E+01_JPRB, 0.11333E+01_JPRB, 0.12229E+01_JPRB, 0.12614E+01_JPRB, 0.12696E+01_JPRB, 0.12342E+01_JPRB, 0.11199E+01_JPRB ,&
&  0.10010E+01_JPRB, 0.86582E+00_JPRB, 0.77308E+00_JPRB, 0.79737E+00_JPRB, 0.83800E+00_JPRB, 0.85166E+00_JPRB, 0.81789E+00_JPRB ,&
&  0.79843E+00_JPRB, 0.73899E+00_JPRB, 0.75556E+00_JPRB, 0.78146E+00_JPRB, 0.81798E+00_JPRB, 0.78700E+00_JPRB, 0.89406E+00_JPRB ,&
&  0.88295E+00_JPRB, 0.69115E+00_JPRB, 0.63329E+00_JPRB, 0.74323E+00_JPRB, 0.67243E+00_JPRB, 0.57990E+00_JPRB, 0.60637E+00_JPRB ,&
&  0.67551E+00_JPRB, 0.76537E+00_JPRB, 0.73438E+00_JPRB, 0.68983E+00_JPRB, 0.78651E+00_JPRB, 0.82232E+00_JPRB, 0.85841E+00_JPRB ,&
&  0.75447E+00_JPRB, 0.72859E+00_JPRB, 0.70884E+00_JPRB, 0.68727E+00_JPRB, 0.67373E+00_JPRB, 0.67182E+00_JPRB, 0.66870E+00_JPRB ,&
&  0.65151E+00_JPRB, 0.63508E+00_JPRB, 0.62533E+00_JPRB, 0.61441E+00_JPRB, 0.59689E+00_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.58946E+00_JPRB, 0.57215E+00_JPRB, 0.56562E+00_JPRB, 0.56300E+00_JPRB, 0.56107E+00_JPRB, 0.55411E+00_JPRB, 0.54114E+00_JPRB ,&
&  0.52352E+00_JPRB, 0.50010E+00_JPRB, 0.48008E+00_JPRB, 0.46660E+00_JPRB, 0.45930E+00_JPRB, 0.46079E+00_JPRB, 0.47143E+00_JPRB ,&
&  0.48234E+00_JPRB, 0.51976E+00_JPRB, 0.61902E+00_JPRB, 0.66923E+00_JPRB, 0.66583E+00_JPRB, 0.60151E+00_JPRB, 0.77089E+00_JPRB ,&
&  0.11954E+01_JPRB, 0.18091E+01_JPRB, 0.23174E+01_JPRB, 0.24933E+01_JPRB, 0.25844E+01_JPRB, 0.24776E+01_JPRB, 0.21194E+01_JPRB ,&
&  0.18270E+01_JPRB, 0.16781E+01_JPRB, 0.14951E+01_JPRB, 0.13175E+01_JPRB, 0.11701E+01_JPRB, 0.10847E+01_JPRB, 0.10330E+01_JPRB ,&
&  0.10070E+01_JPRB, 0.94969E+00_JPRB, 0.90224E+00_JPRB, 0.90973E+00_JPRB, 0.96950E+00_JPRB, 0.10407E+01_JPRB, 0.10614E+01_JPRB ,&
&  0.10365E+01_JPRB, 0.98380E+00_JPRB, 0.97638E+00_JPRB, 0.10127E+01_JPRB, 0.85674E+00_JPRB, 0.97087E+00_JPRB, 0.97599E+00_JPRB ,&
&  0.10194E+01_JPRB, 0.10919E+01_JPRB, 0.11104E+01_JPRB, 0.13203E+01_JPRB, 0.13682E+01_JPRB, 0.13095E+01_JPRB, 0.12484E+01_JPRB ,&
&  0.11624E+01_JPRB, 0.96512E+00_JPRB, 0.83097E+00_JPRB, 0.86020E+00_JPRB, 0.89208E+00_JPRB, 0.88548E+00_JPRB, 0.69061E+00_JPRB ,&
&  0.69057E+00_JPRB, 0.79994E+00_JPRB, 0.79744E+00_JPRB, 0.74206E+00_JPRB, 0.83233E+00_JPRB, 0.83531E+00_JPRB, 0.71050E+00_JPRB ,&
&  0.45642E+00_JPRB, 0.39839E+00_JPRB, 0.41081E+00_JPRB, 0.41666E+00_JPRB, 0.40516E+00_JPRB, 0.42886E+00_JPRB, 0.66415E+00_JPRB ,&
&  0.61803E+00_JPRB, 0.73962E+00_JPRB, 0.73302E+00_JPRB, 0.71890E+00_JPRB, 0.76518E+00_JPRB, 0.85349E+00_JPRB, 0.10111E+01_JPRB ,&
&  0.81839E+00_JPRB, 0.76411E+00_JPRB, 0.70437E+00_JPRB, 0.66212E+00_JPRB, 0.65060E+00_JPRB, 0.64542E+00_JPRB, 0.64531E+00_JPRB ,&
&  0.63682E+00_JPRB, 0.62315E+00_JPRB, 0.61698E+00_JPRB, 0.61474E+00_JPRB, 0.60690E+00_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.57595E+00_JPRB, 0.56976E+00_JPRB, 0.56336E+00_JPRB, 0.55625E+00_JPRB, 0.54499E+00_JPRB, 0.53218E+00_JPRB, 0.52146E+00_JPRB ,&
&  0.50978E+00_JPRB, 0.48865E+00_JPRB, 0.46993E+00_JPRB, 0.45668E+00_JPRB, 0.45005E+00_JPRB, 0.45096E+00_JPRB, 0.46097E+00_JPRB ,&
&  0.47696E+00_JPRB, 0.52873E+00_JPRB, 0.72666E+00_JPRB, 0.68841E+00_JPRB, 0.70205E+00_JPRB, 0.65425E+00_JPRB, 0.71215E+00_JPRB ,&
&  0.10311E+01_JPRB, 0.15206E+01_JPRB, 0.20762E+01_JPRB, 0.24115E+01_JPRB, 0.27912E+01_JPRB, 0.28297E+01_JPRB, 0.25965E+01_JPRB ,&
&  0.21649E+01_JPRB, 0.18378E+01_JPRB, 0.15743E+01_JPRB, 0.13608E+01_JPRB, 0.11863E+01_JPRB, 0.10831E+01_JPRB, 0.10285E+01_JPRB ,&
&  0.98913E+00_JPRB, 0.97120E+00_JPRB, 0.91646E+00_JPRB, 0.91126E+00_JPRB, 0.97677E+00_JPRB, 0.10532E+01_JPRB, 0.10791E+01_JPRB ,&
&  0.10452E+01_JPRB, 0.98698E+00_JPRB, 0.95592E+00_JPRB, 0.97196E+00_JPRB, 0.86019E+00_JPRB, 0.77551E+00_JPRB, 0.87478E+00_JPRB ,&
&  0.98743E+00_JPRB, 0.11360E+01_JPRB, 0.11628E+01_JPRB, 0.13848E+01_JPRB, 0.14216E+01_JPRB, 0.13712E+01_JPRB, 0.14589E+01_JPRB ,&
&  0.13745E+01_JPRB, 0.12611E+01_JPRB, 0.93797E+00_JPRB, 0.93618E+00_JPRB, 0.97085E+00_JPRB, 0.79084E+00_JPRB, 0.71905E+00_JPRB ,&
&  0.82754E+00_JPRB, 0.80007E+00_JPRB, 0.76228E+00_JPRB, 0.69091E+00_JPRB, 0.82391E+00_JPRB, 0.73970E+00_JPRB, 0.47281E+00_JPRB ,&
&  0.41742E+00_JPRB, 0.39097E+00_JPRB, 0.37075E+00_JPRB, 0.35076E+00_JPRB, 0.33105E+00_JPRB, 0.35131E+00_JPRB, 0.49594E+00_JPRB ,&
&  0.53938E+00_JPRB, 0.68376E+00_JPRB, 0.71714E+00_JPRB, 0.70425E+00_JPRB, 0.71984E+00_JPRB, 0.84068E+00_JPRB, 0.10537E+01_JPRB ,&
&  0.84552E+00_JPRB, 0.82527E+00_JPRB, 0.74786E+00_JPRB, 0.69321E+00_JPRB, 0.65441E+00_JPRB, 0.63235E+00_JPRB, 0.62793E+00_JPRB ,&
&  0.61907E+00_JPRB, 0.60905E+00_JPRB, 0.60027E+00_JPRB, 0.59246E+00_JPRB, 0.58464E+00_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.55893E+00_JPRB, 0.55293E+00_JPRB, 0.55138E+00_JPRB, 0.54972E+00_JPRB, 0.54262E+00_JPRB, 0.52834E+00_JPRB, 0.51513E+00_JPRB ,&
&  0.50293E+00_JPRB, 0.48069E+00_JPRB, 0.46526E+00_JPRB, 0.45432E+00_JPRB, 0.44619E+00_JPRB, 0.44511E+00_JPRB, 0.45150E+00_JPRB ,&
&  0.46416E+00_JPRB, 0.55425E+00_JPRB, 0.69747E+00_JPRB, 0.61008E+00_JPRB, 0.57421E+00_JPRB, 0.54165E+00_JPRB, 0.54631E+00_JPRB ,&
&  0.89153E+00_JPRB, 0.13045E+01_JPRB, 0.17627E+01_JPRB, 0.23655E+01_JPRB, 0.28378E+01_JPRB, 0.27305E+01_JPRB, 0.28127E+01_JPRB ,&
&  0.24709E+01_JPRB, 0.20306E+01_JPRB, 0.17031E+01_JPRB, 0.14545E+01_JPRB, 0.12409E+01_JPRB, 0.11082E+01_JPRB, 0.10292E+01_JPRB ,&
&  0.99353E+00_JPRB, 0.97114E+00_JPRB, 0.92986E+00_JPRB, 0.91365E+00_JPRB, 0.97014E+00_JPRB, 0.10834E+01_JPRB, 0.11379E+01_JPRB ,&
&  0.10654E+01_JPRB, 0.10007E+01_JPRB, 0.94869E+00_JPRB, 0.93766E+00_JPRB, 0.95261E+00_JPRB, 0.93010E+00_JPRB, 0.86641E+00_JPRB ,&
&  0.85777E+00_JPRB, 0.94199E+00_JPRB, 0.12206E+01_JPRB, 0.13065E+01_JPRB, 0.13776E+01_JPRB, 0.14561E+01_JPRB, 0.15262E+01_JPRB ,&
&  0.14680E+01_JPRB, 0.14136E+01_JPRB, 0.11071E+01_JPRB, 0.10715E+01_JPRB, 0.90544E+00_JPRB, 0.75560E+00_JPRB, 0.85610E+00_JPRB ,&
&  0.91304E+00_JPRB, 0.88753E+00_JPRB, 0.78421E+00_JPRB, 0.63652E+00_JPRB, 0.62082E+00_JPRB, 0.50337E+00_JPRB, 0.39002E+00_JPRB ,&
&  0.37974E+00_JPRB, 0.36631E+00_JPRB, 0.33781E+00_JPRB, 0.33028E+00_JPRB, 0.31593E+00_JPRB, 0.34546E+00_JPRB, 0.43424E+00_JPRB ,&
&  0.53680E+00_JPRB, 0.59877E+00_JPRB, 0.62295E+00_JPRB, 0.60648E+00_JPRB, 0.59836E+00_JPRB, 0.63069E+00_JPRB, 0.75469E+00_JPRB ,&
&  0.70512E+00_JPRB, 0.78698E+00_JPRB, 0.73418E+00_JPRB, 0.67995E+00_JPRB, 0.65275E+00_JPRB, 0.63150E+00_JPRB, 0.62086E+00_JPRB ,&
&  0.61318E+00_JPRB, 0.60604E+00_JPRB, 0.59749E+00_JPRB, 0.58239E+00_JPRB, 0.57051E+00_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.55091E+00_JPRB, 0.54002E+00_JPRB, 0.53715E+00_JPRB, 0.53651E+00_JPRB, 0.53463E+00_JPRB, 0.52650E+00_JPRB, 0.51505E+00_JPRB ,&
&  0.49976E+00_JPRB, 0.48043E+00_JPRB, 0.46824E+00_JPRB, 0.45952E+00_JPRB, 0.44824E+00_JPRB, 0.44680E+00_JPRB, 0.45324E+00_JPRB ,&
&  0.47710E+00_JPRB, 0.67021E+00_JPRB, 0.51721E+00_JPRB, 0.44157E+00_JPRB, 0.49522E+00_JPRB, 0.51275E+00_JPRB, 0.48048E+00_JPRB ,&
&  0.78755E+00_JPRB, 0.12029E+01_JPRB, 0.15623E+01_JPRB, 0.23375E+01_JPRB, 0.31813E+01_JPRB, 0.30064E+01_JPRB, 0.29296E+01_JPRB ,&
&  0.28234E+01_JPRB, 0.22474E+01_JPRB, 0.18639E+01_JPRB, 0.15879E+01_JPRB, 0.13274E+01_JPRB, 0.11458E+01_JPRB, 0.10162E+01_JPRB ,&
&  0.95859E+00_JPRB, 0.95585E+00_JPRB, 0.93130E+00_JPRB, 0.93559E+00_JPRB, 0.96612E+00_JPRB, 0.10534E+01_JPRB, 0.11478E+01_JPRB ,&
&  0.11171E+01_JPRB, 0.10463E+01_JPRB, 0.98467E+00_JPRB, 0.97513E+00_JPRB, 0.94597E+00_JPRB, 0.92210E+00_JPRB, 0.10898E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.11791E+01_JPRB, 0.12308E+01_JPRB, 0.14869E+01_JPRB, 0.16203E+01_JPRB, 0.15746E+01_JPRB, 0.16686E+01_JPRB ,&
&  0.13108E+01_JPRB, 0.11476E+01_JPRB, 0.11245E+01_JPRB, 0.10379E+01_JPRB, 0.83550E+00_JPRB, 0.91612E+00_JPRB, 0.12907E+01_JPRB ,&
&  0.10235E+01_JPRB, 0.10947E+01_JPRB, 0.11211E+01_JPRB, 0.10178E+01_JPRB, 0.56822E+00_JPRB, 0.45307E+00_JPRB, 0.60470E+00_JPRB ,&
&  0.63313E+00_JPRB, 0.45755E+00_JPRB, 0.36437E+00_JPRB, 0.40435E+00_JPRB, 0.34249E+00_JPRB, 0.38022E+00_JPRB, 0.46096E+00_JPRB ,&
&  0.49206E+00_JPRB, 0.52404E+00_JPRB, 0.59298E+00_JPRB, 0.55158E+00_JPRB, 0.55282E+00_JPRB, 0.57223E+00_JPRB, 0.63629E+00_JPRB ,&
&  0.63996E+00_JPRB, 0.73537E+00_JPRB, 0.74635E+00_JPRB, 0.67375E+00_JPRB, 0.63411E+00_JPRB, 0.61055E+00_JPRB, 0.60261E+00_JPRB ,&
&  0.60315E+00_JPRB, 0.60007E+00_JPRB, 0.59495E+00_JPRB, 0.58256E+00_JPRB, 0.56488E+00_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.53538E+00_JPRB, 0.52549E+00_JPRB, 0.51938E+00_JPRB, 0.52043E+00_JPRB, 0.52510E+00_JPRB, 0.52636E+00_JPRB, 0.51766E+00_JPRB ,&
&  0.50279E+00_JPRB, 0.48621E+00_JPRB, 0.47079E+00_JPRB, 0.46324E+00_JPRB, 0.45300E+00_JPRB, 0.45101E+00_JPRB, 0.45808E+00_JPRB ,&
&  0.48452E+00_JPRB, 0.52451E+00_JPRB, 0.39957E+00_JPRB, 0.37601E+00_JPRB, 0.50133E+00_JPRB, 0.49543E+00_JPRB, 0.51701E+00_JPRB ,&
&  0.75728E+00_JPRB, 0.10300E+01_JPRB, 0.13793E+01_JPRB, 0.21215E+01_JPRB, 0.25338E+01_JPRB, 0.29038E+01_JPRB, 0.29310E+01_JPRB ,&
&  0.29538E+01_JPRB, 0.24155E+01_JPRB, 0.19395E+01_JPRB, 0.15837E+01_JPRB, 0.13184E+01_JPRB, 0.11536E+01_JPRB, 0.10451E+01_JPRB ,&
&  0.94919E+00_JPRB, 0.90325E+00_JPRB, 0.91433E+00_JPRB, 0.91671E+00_JPRB, 0.95475E+00_JPRB, 0.10007E+01_JPRB, 0.10799E+01_JPRB ,&
&  0.11125E+01_JPRB, 0.10536E+01_JPRB, 0.97592E+00_JPRB, 0.96523E+00_JPRB, 0.89023E+00_JPRB, 0.84868E+00_JPRB, 0.93500E+00_JPRB ,&
&  0.11200E+01_JPRB, 0.12236E+01_JPRB, 0.13323E+01_JPRB, 0.15167E+01_JPRB, 0.17239E+01_JPRB, 0.16512E+01_JPRB, 0.18765E+01_JPRB ,&
&  0.15806E+01_JPRB, 0.13866E+01_JPRB, 0.11969E+01_JPRB, 0.97376E+00_JPRB, 0.96396E+00_JPRB, 0.14198E+01_JPRB, 0.14621E+01_JPRB ,&
&  0.13478E+01_JPRB, 0.13224E+01_JPRB, 0.12491E+01_JPRB, 0.99264E+00_JPRB, 0.72921E+00_JPRB, 0.54187E+00_JPRB, 0.59840E+00_JPRB ,&
&  0.59700E+00_JPRB, 0.62004E+00_JPRB, 0.55776E+00_JPRB, 0.47131E+00_JPRB, 0.40287E+00_JPRB, 0.45868E+00_JPRB, 0.45838E+00_JPRB ,&
&  0.45567E+00_JPRB, 0.45479E+00_JPRB, 0.50571E+00_JPRB, 0.49874E+00_JPRB, 0.49827E+00_JPRB, 0.48791E+00_JPRB, 0.56712E+00_JPRB ,&
&  0.57052E+00_JPRB, 0.61011E+00_JPRB, 0.63314E+00_JPRB, 0.62162E+00_JPRB, 0.61150E+00_JPRB, 0.59936E+00_JPRB, 0.58494E+00_JPRB ,&
&  0.57722E+00_JPRB, 0.57306E+00_JPRB, 0.56351E+00_JPRB, 0.55611E+00_JPRB, 0.54548E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.51300E+00_JPRB, 0.50443E+00_JPRB, 0.49899E+00_JPRB, 0.49837E+00_JPRB, 0.50486E+00_JPRB, 0.51552E+00_JPRB, 0.51614E+00_JPRB ,&
&  0.50679E+00_JPRB, 0.49217E+00_JPRB, 0.47958E+00_JPRB, 0.46952E+00_JPRB, 0.46193E+00_JPRB, 0.45997E+00_JPRB, 0.46284E+00_JPRB ,&
&  0.48700E+00_JPRB, 0.45432E+00_JPRB, 0.34412E+00_JPRB, 0.36709E+00_JPRB, 0.45023E+00_JPRB, 0.44937E+00_JPRB, 0.55954E+00_JPRB ,&
&  0.69744E+00_JPRB, 0.87166E+00_JPRB, 0.10887E+01_JPRB, 0.16425E+01_JPRB, 0.20439E+01_JPRB, 0.23299E+01_JPRB, 0.24189E+01_JPRB ,&
&  0.22978E+01_JPRB, 0.21213E+01_JPRB, 0.18735E+01_JPRB, 0.16127E+01_JPRB, 0.13332E+01_JPRB, 0.12100E+01_JPRB, 0.11235E+01_JPRB ,&
&  0.10503E+01_JPRB, 0.96085E+00_JPRB, 0.91056E+00_JPRB, 0.91384E+00_JPRB, 0.93223E+00_JPRB, 0.96871E+00_JPRB, 0.10429E+01_JPRB ,&
&  0.10857E+01_JPRB, 0.10683E+01_JPRB, 0.96980E+00_JPRB, 0.93876E+00_JPRB, 0.79399E+00_JPRB, 0.78410E+00_JPRB, 0.86322E+00_JPRB ,&
&  0.13122E+01_JPRB, 0.14268E+01_JPRB, 0.14398E+01_JPRB, 0.17484E+01_JPRB, 0.15799E+01_JPRB, 0.16112E+01_JPRB, 0.19196E+01_JPRB ,&
&  0.21127E+01_JPRB, 0.21251E+01_JPRB, 0.21444E+01_JPRB, 0.15939E+01_JPRB, 0.11957E+01_JPRB, 0.16469E+01_JPRB, 0.15676E+01_JPRB ,&
&  0.14322E+01_JPRB, 0.13103E+01_JPRB, 0.11309E+01_JPRB, 0.10521E+01_JPRB, 0.77735E+00_JPRB, 0.61062E+00_JPRB, 0.48072E+00_JPRB ,&
&  0.45784E+00_JPRB, 0.43824E+00_JPRB, 0.47390E+00_JPRB, 0.45080E+00_JPRB, 0.44721E+00_JPRB, 0.43146E+00_JPRB, 0.41550E+00_JPRB ,&
&  0.41586E+00_JPRB, 0.40771E+00_JPRB, 0.41400E+00_JPRB, 0.44201E+00_JPRB, 0.46548E+00_JPRB, 0.43832E+00_JPRB, 0.48880E+00_JPRB ,&
&  0.53032E+00_JPRB, 0.55854E+00_JPRB, 0.56495E+00_JPRB, 0.58048E+00_JPRB, 0.57766E+00_JPRB, 0.58033E+00_JPRB, 0.57624E+00_JPRB ,&
&  0.56443E+00_JPRB, 0.55363E+00_JPRB, 0.54370E+00_JPRB, 0.53243E+00_JPRB, 0.52259E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.46148E+00_JPRB, 0.46175E+00_JPRB, 0.46542E+00_JPRB, 0.46981E+00_JPRB, 0.47594E+00_JPRB, 0.48666E+00_JPRB, 0.49055E+00_JPRB ,&
&  0.48936E+00_JPRB, 0.47959E+00_JPRB, 0.46715E+00_JPRB, 0.45841E+00_JPRB, 0.45570E+00_JPRB, 0.45818E+00_JPRB, 0.46434E+00_JPRB ,&
&  0.47705E+00_JPRB, 0.49779E+00_JPRB, 0.39067E+00_JPRB, 0.34702E+00_JPRB, 0.34893E+00_JPRB, 0.43148E+00_JPRB, 0.56406E+00_JPRB ,&
&  0.67949E+00_JPRB, 0.73372E+00_JPRB, 0.92547E+00_JPRB, 0.12755E+01_JPRB, 0.15466E+01_JPRB, 0.16905E+01_JPRB, 0.18270E+01_JPRB ,&
&  0.21141E+01_JPRB, 0.16745E+01_JPRB, 0.16080E+01_JPRB, 0.14310E+01_JPRB, 0.13053E+01_JPRB, 0.12150E+01_JPRB, 0.11209E+01_JPRB ,&
&  0.10312E+01_JPRB, 0.97587E+00_JPRB, 0.90176E+00_JPRB, 0.86548E+00_JPRB, 0.84896E+00_JPRB, 0.85117E+00_JPRB, 0.89219E+00_JPRB ,&
&  0.94874E+00_JPRB, 0.99337E+00_JPRB, 0.98190E+00_JPRB, 0.96382E+00_JPRB, 0.96773E+00_JPRB, 0.97738E+00_JPRB, 0.10789E+01_JPRB ,&
&  0.11966E+01_JPRB, 0.11568E+01_JPRB, 0.15386E+01_JPRB, 0.17648E+01_JPRB, 0.20047E+01_JPRB, 0.21009E+01_JPRB, 0.21482E+01_JPRB ,&
&  0.22510E+01_JPRB, 0.21919E+01_JPRB, 0.21574E+01_JPRB, 0.18628E+01_JPRB, 0.19366E+01_JPRB, 0.18459E+01_JPRB, 0.16389E+01_JPRB ,&
&  0.14167E+01_JPRB, 0.12625E+01_JPRB, 0.10582E+01_JPRB, 0.92508E+00_JPRB, 0.85762E+00_JPRB, 0.79905E+00_JPRB, 0.61254E+00_JPRB ,&
&  0.49536E+00_JPRB, 0.51663E+00_JPRB, 0.47852E+00_JPRB, 0.44335E+00_JPRB, 0.40327E+00_JPRB, 0.40869E+00_JPRB, 0.40849E+00_JPRB ,&
&  0.40734E+00_JPRB, 0.40185E+00_JPRB, 0.39242E+00_JPRB, 0.41504E+00_JPRB, 0.45040E+00_JPRB, 0.44407E+00_JPRB, 0.45970E+00_JPRB ,&
&  0.44131E+00_JPRB, 0.47608E+00_JPRB, 0.48604E+00_JPRB, 0.48468E+00_JPRB, 0.53186E+00_JPRB, 0.55007E+00_JPRB, 0.54300E+00_JPRB ,&
&  0.52532E+00_JPRB, 0.50927E+00_JPRB, 0.49317E+00_JPRB, 0.47738E+00_JPRB, 0.46675E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.45018E+00_JPRB, 0.45059E+00_JPRB, 0.45313E+00_JPRB, 0.45769E+00_JPRB, 0.46286E+00_JPRB, 0.47506E+00_JPRB, 0.48313E+00_JPRB ,&
&  0.48427E+00_JPRB, 0.48205E+00_JPRB, 0.47329E+00_JPRB, 0.46394E+00_JPRB, 0.45892E+00_JPRB, 0.45674E+00_JPRB, 0.46259E+00_JPRB ,&
&  0.48605E+00_JPRB, 0.64017E+00_JPRB, 0.46483E+00_JPRB, 0.42311E+00_JPRB, 0.39629E+00_JPRB, 0.48900E+00_JPRB, 0.57239E+00_JPRB ,&
&  0.64256E+00_JPRB, 0.64754E+00_JPRB, 0.68707E+00_JPRB, 0.92983E+00_JPRB, 0.11811E+01_JPRB, 0.12105E+01_JPRB, 0.13594E+01_JPRB ,&
&  0.14171E+01_JPRB, 0.14022E+01_JPRB, 0.12612E+01_JPRB, 0.12909E+01_JPRB, 0.11431E+01_JPRB, 0.99036E+00_JPRB, 0.10131E+01_JPRB ,&
&  0.94700E+00_JPRB, 0.91567E+00_JPRB, 0.86851E+00_JPRB, 0.81077E+00_JPRB, 0.78258E+00_JPRB, 0.76332E+00_JPRB, 0.76678E+00_JPRB ,&
&  0.78949E+00_JPRB, 0.82163E+00_JPRB, 0.86871E+00_JPRB, 0.91205E+00_JPRB, 0.95058E+00_JPRB, 0.10699E+01_JPRB, 0.11642E+01_JPRB ,&
&  0.15476E+01_JPRB, 0.16450E+01_JPRB, 0.14002E+01_JPRB, 0.18456E+01_JPRB, 0.25656E+01_JPRB, 0.25096E+01_JPRB, 0.22862E+01_JPRB ,&
&  0.23510E+01_JPRB, 0.22644E+01_JPRB, 0.21813E+01_JPRB, 0.21570E+01_JPRB, 0.20142E+01_JPRB, 0.18242E+01_JPRB, 0.15774E+01_JPRB ,&
&  0.13108E+01_JPRB, 0.11351E+01_JPRB, 0.10032E+01_JPRB, 0.90027E+00_JPRB, 0.77945E+00_JPRB, 0.70030E+00_JPRB, 0.64318E+00_JPRB ,&
&  0.55994E+00_JPRB, 0.49032E+00_JPRB, 0.40097E+00_JPRB, 0.41168E+00_JPRB, 0.35097E+00_JPRB, 0.35774E+00_JPRB, 0.38162E+00_JPRB ,&
&  0.37719E+00_JPRB, 0.39476E+00_JPRB, 0.40031E+00_JPRB, 0.39016E+00_JPRB, 0.42650E+00_JPRB, 0.43619E+00_JPRB, 0.44610E+00_JPRB ,&
&  0.43745E+00_JPRB, 0.41179E+00_JPRB, 0.43381E+00_JPRB, 0.43398E+00_JPRB, 0.45042E+00_JPRB, 0.45705E+00_JPRB, 0.46196E+00_JPRB ,&
&  0.46277E+00_JPRB, 0.45863E+00_JPRB, 0.45354E+00_JPRB, 0.45053E+00_JPRB, 0.45031E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.41976E+00_JPRB, 0.41848E+00_JPRB, 0.41606E+00_JPRB, 0.42255E+00_JPRB, 0.43595E+00_JPRB, 0.45169E+00_JPRB, 0.46438E+00_JPRB ,&
&  0.46658E+00_JPRB, 0.46805E+00_JPRB, 0.46352E+00_JPRB, 0.45590E+00_JPRB, 0.45115E+00_JPRB, 0.44340E+00_JPRB, 0.45393E+00_JPRB ,&
&  0.51245E+00_JPRB, 0.59727E+00_JPRB, 0.46782E+00_JPRB, 0.39390E+00_JPRB, 0.46693E+00_JPRB, 0.49610E+00_JPRB, 0.55673E+00_JPRB ,&
&  0.56319E+00_JPRB, 0.60169E+00_JPRB, 0.60545E+00_JPRB, 0.66155E+00_JPRB, 0.84846E+00_JPRB, 0.94862E+00_JPRB, 0.99546E+00_JPRB ,&
&  0.10226E+01_JPRB, 0.98269E+00_JPRB, 0.99473E+00_JPRB, 0.10284E+01_JPRB, 0.10562E+01_JPRB, 0.93502E+00_JPRB, 0.90555E+00_JPRB ,&
&  0.84884E+00_JPRB, 0.80760E+00_JPRB, 0.75888E+00_JPRB, 0.73407E+00_JPRB, 0.72249E+00_JPRB, 0.71962E+00_JPRB, 0.73141E+00_JPRB ,&
&  0.74765E+00_JPRB, 0.76535E+00_JPRB, 0.78691E+00_JPRB, 0.83822E+00_JPRB, 0.90445E+00_JPRB, 0.11439E+01_JPRB, 0.15588E+01_JPRB ,&
&  0.20601E+01_JPRB, 0.23413E+01_JPRB, 0.25412E+01_JPRB, 0.28365E+01_JPRB, 0.28117E+01_JPRB, 0.27103E+01_JPRB, 0.25820E+01_JPRB ,&
&  0.24186E+01_JPRB, 0.22580E+01_JPRB, 0.21416E+01_JPRB, 0.20169E+01_JPRB, 0.18805E+01_JPRB, 0.17125E+01_JPRB, 0.14689E+01_JPRB ,&
&  0.12030E+01_JPRB, 0.10452E+01_JPRB, 0.97561E+00_JPRB, 0.86025E+00_JPRB, 0.76208E+00_JPRB, 0.67627E+00_JPRB, 0.64271E+00_JPRB ,&
&  0.61795E+00_JPRB, 0.40757E+00_JPRB, 0.36107E+00_JPRB, 0.39679E+00_JPRB, 0.35407E+00_JPRB, 0.36670E+00_JPRB, 0.39188E+00_JPRB ,&
&  0.38450E+00_JPRB, 0.37973E+00_JPRB, 0.38754E+00_JPRB, 0.37701E+00_JPRB, 0.38627E+00_JPRB, 0.42344E+00_JPRB, 0.40426E+00_JPRB ,&
&  0.38610E+00_JPRB, 0.38470E+00_JPRB, 0.39940E+00_JPRB, 0.40228E+00_JPRB, 0.40596E+00_JPRB, 0.40544E+00_JPRB, 0.39978E+00_JPRB ,&
&  0.39944E+00_JPRB, 0.40585E+00_JPRB, 0.40955E+00_JPRB, 0.41440E+00_JPRB, 0.41812E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.40063E+00_JPRB, 0.40195E+00_JPRB, 0.39788E+00_JPRB, 0.40434E+00_JPRB, 0.42312E+00_JPRB, 0.44225E+00_JPRB, 0.45550E+00_JPRB ,&
&  0.45966E+00_JPRB, 0.46304E+00_JPRB, 0.45874E+00_JPRB, 0.45316E+00_JPRB, 0.44757E+00_JPRB, 0.43355E+00_JPRB, 0.45155E+00_JPRB ,&
&  0.37780E+00_JPRB, 0.44186E+00_JPRB, 0.37236E+00_JPRB, 0.38313E+00_JPRB, 0.50110E+00_JPRB, 0.47601E+00_JPRB, 0.49975E+00_JPRB ,&
&  0.50784E+00_JPRB, 0.52761E+00_JPRB, 0.53041E+00_JPRB, 0.56416E+00_JPRB, 0.68096E+00_JPRB, 0.79015E+00_JPRB, 0.83107E+00_JPRB ,&
&  0.79916E+00_JPRB, 0.75947E+00_JPRB, 0.79242E+00_JPRB, 0.81510E+00_JPRB, 0.82170E+00_JPRB, 0.85353E+00_JPRB, 0.85998E+00_JPRB ,&
&  0.80294E+00_JPRB, 0.75793E+00_JPRB, 0.70548E+00_JPRB, 0.68198E+00_JPRB, 0.67084E+00_JPRB, 0.66484E+00_JPRB, 0.65677E+00_JPRB ,&
&  0.66037E+00_JPRB, 0.67883E+00_JPRB, 0.70530E+00_JPRB, 0.78333E+00_JPRB, 0.86766E+00_JPRB, 0.14436E+01_JPRB, 0.22984E+01_JPRB ,&
&  0.27484E+01_JPRB, 0.27425E+01_JPRB, 0.29008E+01_JPRB, 0.31649E+01_JPRB, 0.30262E+01_JPRB, 0.28085E+01_JPRB, 0.25156E+01_JPRB ,&
&  0.23271E+01_JPRB, 0.21761E+01_JPRB, 0.20260E+01_JPRB, 0.18889E+01_JPRB, 0.16724E+01_JPRB, 0.15333E+01_JPRB, 0.13332E+01_JPRB ,&
&  0.10898E+01_JPRB, 0.96934E+00_JPRB, 0.92745E+00_JPRB, 0.89134E+00_JPRB, 0.83459E+00_JPRB, 0.80641E+00_JPRB, 0.74067E+00_JPRB ,&
&  0.68040E+00_JPRB, 0.57311E+00_JPRB, 0.45275E+00_JPRB, 0.42546E+00_JPRB, 0.37649E+00_JPRB, 0.38453E+00_JPRB, 0.43154E+00_JPRB ,&
&  0.40842E+00_JPRB, 0.37796E+00_JPRB, 0.36278E+00_JPRB, 0.35513E+00_JPRB, 0.35737E+00_JPRB, 0.38418E+00_JPRB, 0.36970E+00_JPRB ,&
&  0.35289E+00_JPRB, 0.36852E+00_JPRB, 0.37015E+00_JPRB, 0.37118E+00_JPRB, 0.36931E+00_JPRB, 0.36841E+00_JPRB, 0.34773E+00_JPRB ,&
&  0.35558E+00_JPRB, 0.36162E+00_JPRB, 0.37298E+00_JPRB, 0.38703E+00_JPRB, 0.39555E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.39426E+00_JPRB, 0.39453E+00_JPRB, 0.39096E+00_JPRB, 0.39900E+00_JPRB, 0.41845E+00_JPRB, 0.43621E+00_JPRB, 0.44886E+00_JPRB ,&
&  0.45254E+00_JPRB, 0.45238E+00_JPRB, 0.45154E+00_JPRB, 0.44989E+00_JPRB, 0.44015E+00_JPRB, 0.42667E+00_JPRB, 0.38475E+00_JPRB ,&
&  0.32908E+00_JPRB, 0.37474E+00_JPRB, 0.35598E+00_JPRB, 0.37354E+00_JPRB, 0.42510E+00_JPRB, 0.41095E+00_JPRB, 0.40803E+00_JPRB ,&
&  0.42993E+00_JPRB, 0.43971E+00_JPRB, 0.46889E+00_JPRB, 0.50001E+00_JPRB, 0.55208E+00_JPRB, 0.65763E+00_JPRB, 0.69413E+00_JPRB ,&
&  0.65536E+00_JPRB, 0.63179E+00_JPRB, 0.66196E+00_JPRB, 0.71818E+00_JPRB, 0.77611E+00_JPRB, 0.82294E+00_JPRB, 0.81405E+00_JPRB ,&
&  0.78157E+00_JPRB, 0.73926E+00_JPRB, 0.67451E+00_JPRB, 0.63156E+00_JPRB, 0.60723E+00_JPRB, 0.59298E+00_JPRB, 0.58407E+00_JPRB ,&
&  0.58065E+00_JPRB, 0.59241E+00_JPRB, 0.62544E+00_JPRB, 0.68444E+00_JPRB, 0.83175E+00_JPRB, 0.13419E+01_JPRB, 0.17740E+01_JPRB ,&
&  0.18797E+01_JPRB, 0.19823E+01_JPRB, 0.21486E+01_JPRB, 0.22753E+01_JPRB, 0.23879E+01_JPRB, 0.23014E+01_JPRB, 0.21913E+01_JPRB ,&
&  0.20975E+01_JPRB, 0.19757E+01_JPRB, 0.19086E+01_JPRB, 0.17387E+01_JPRB, 0.15328E+01_JPRB, 0.13734E+01_JPRB, 0.11820E+01_JPRB ,&
&  0.10271E+01_JPRB, 0.92092E+00_JPRB, 0.88646E+00_JPRB, 0.85138E+00_JPRB, 0.81977E+00_JPRB, 0.79938E+00_JPRB, 0.74445E+00_JPRB ,&
&  0.70061E+00_JPRB, 0.64542E+00_JPRB, 0.54638E+00_JPRB, 0.48463E+00_JPRB, 0.43926E+00_JPRB, 0.44146E+00_JPRB, 0.42609E+00_JPRB ,&
&  0.39740E+00_JPRB, 0.35881E+00_JPRB, 0.35736E+00_JPRB, 0.35432E+00_JPRB, 0.35248E+00_JPRB, 0.34980E+00_JPRB, 0.33680E+00_JPRB ,&
&  0.33885E+00_JPRB, 0.34690E+00_JPRB, 0.33278E+00_JPRB, 0.33014E+00_JPRB, 0.32793E+00_JPRB, 0.32547E+00_JPRB, 0.30196E+00_JPRB ,&
&  0.30637E+00_JPRB, 0.33028E+00_JPRB, 0.34958E+00_JPRB, 0.37038E+00_JPRB, 0.38548E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.40058E+00_JPRB, 0.40338E+00_JPRB, 0.40730E+00_JPRB, 0.40911E+00_JPRB, 0.41576E+00_JPRB, 0.43415E+00_JPRB, 0.42470E+00_JPRB ,&
&  0.45509E+00_JPRB, 0.45785E+00_JPRB, 0.45096E+00_JPRB, 0.44852E+00_JPRB, 0.43219E+00_JPRB, 0.36903E+00_JPRB, 0.29376E+00_JPRB ,&
&  0.27935E+00_JPRB, 0.30266E+00_JPRB, 0.33140E+00_JPRB, 0.34878E+00_JPRB, 0.37137E+00_JPRB, 0.37187E+00_JPRB, 0.37177E+00_JPRB ,&
&  0.37588E+00_JPRB, 0.38142E+00_JPRB, 0.40373E+00_JPRB, 0.44017E+00_JPRB, 0.47632E+00_JPRB, 0.53254E+00_JPRB, 0.59381E+00_JPRB ,&
&  0.55830E+00_JPRB, 0.55359E+00_JPRB, 0.59291E+00_JPRB, 0.58015E+00_JPRB, 0.66877E+00_JPRB, 0.69409E+00_JPRB, 0.69802E+00_JPRB ,&
&  0.68495E+00_JPRB, 0.66026E+00_JPRB, 0.62236E+00_JPRB, 0.59805E+00_JPRB, 0.57317E+00_JPRB, 0.55259E+00_JPRB, 0.54003E+00_JPRB ,&
&  0.54321E+00_JPRB, 0.55425E+00_JPRB, 0.58248E+00_JPRB, 0.63097E+00_JPRB, 0.69849E+00_JPRB, 0.99388E+00_JPRB, 0.12279E+01_JPRB ,&
&  0.13868E+01_JPRB, 0.16366E+01_JPRB, 0.18919E+01_JPRB, 0.18157E+01_JPRB, 0.19407E+01_JPRB, 0.20014E+01_JPRB, 0.19278E+01_JPRB ,&
&  0.19313E+01_JPRB, 0.18046E+01_JPRB, 0.17204E+01_JPRB, 0.15691E+01_JPRB, 0.14073E+01_JPRB, 0.12423E+01_JPRB, 0.10801E+01_JPRB ,&
&  0.95739E+00_JPRB, 0.91404E+00_JPRB, 0.87714E+00_JPRB, 0.82055E+00_JPRB, 0.75199E+00_JPRB, 0.70598E+00_JPRB, 0.65937E+00_JPRB ,&
&  0.62833E+00_JPRB, 0.60349E+00_JPRB, 0.56654E+00_JPRB, 0.53156E+00_JPRB, 0.49553E+00_JPRB, 0.46306E+00_JPRB, 0.43953E+00_JPRB ,&
&  0.41490E+00_JPRB, 0.37829E+00_JPRB, 0.35922E+00_JPRB, 0.35618E+00_JPRB, 0.34855E+00_JPRB, 0.33053E+00_JPRB, 0.32420E+00_JPRB ,&
&  0.32081E+00_JPRB, 0.31005E+00_JPRB, 0.30736E+00_JPRB, 0.30312E+00_JPRB, 0.30444E+00_JPRB, 0.30934E+00_JPRB, 0.30805E+00_JPRB ,&
&  0.30943E+00_JPRB, 0.33889E+00_JPRB, 0.35951E+00_JPRB, 0.37732E+00_JPRB, 0.38980E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.40083E+00_JPRB, 0.41491E+00_JPRB, 0.42411E+00_JPRB, 0.42171E+00_JPRB, 0.41870E+00_JPRB, 0.40899E+00_JPRB, 0.41774E+00_JPRB ,&
&  0.45908E+00_JPRB, 0.41871E+00_JPRB, 0.45421E+00_JPRB, 0.35751E+00_JPRB, 0.31442E+00_JPRB, 0.29605E+00_JPRB, 0.28073E+00_JPRB ,&
&  0.28113E+00_JPRB, 0.29712E+00_JPRB, 0.31162E+00_JPRB, 0.32214E+00_JPRB, 0.34396E+00_JPRB, 0.34655E+00_JPRB, 0.34647E+00_JPRB ,&
&  0.35268E+00_JPRB, 0.35824E+00_JPRB, 0.38112E+00_JPRB, 0.38790E+00_JPRB, 0.40937E+00_JPRB, 0.44418E+00_JPRB, 0.49855E+00_JPRB ,&
&  0.47657E+00_JPRB, 0.48259E+00_JPRB, 0.53180E+00_JPRB, 0.54504E+00_JPRB, 0.56124E+00_JPRB, 0.55699E+00_JPRB, 0.54451E+00_JPRB ,&
&  0.53548E+00_JPRB, 0.45434E+00_JPRB, 0.47821E+00_JPRB, 0.47058E+00_JPRB, 0.46208E+00_JPRB, 0.45121E+00_JPRB, 0.43383E+00_JPRB ,&
&  0.42445E+00_JPRB, 0.43130E+00_JPRB, 0.45311E+00_JPRB, 0.48447E+00_JPRB, 0.53603E+00_JPRB, 0.67123E+00_JPRB, 0.82867E+00_JPRB ,&
&  0.10486E+01_JPRB, 0.10234E+01_JPRB, 0.13235E+01_JPRB, 0.14300E+01_JPRB, 0.15642E+01_JPRB, 0.16930E+01_JPRB, 0.17613E+01_JPRB ,&
&  0.17666E+01_JPRB, 0.15966E+01_JPRB, 0.14606E+01_JPRB, 0.13120E+01_JPRB, 0.11812E+01_JPRB, 0.10665E+01_JPRB, 0.98160E+00_JPRB ,&
&  0.90896E+00_JPRB, 0.80996E+00_JPRB, 0.79266E+00_JPRB, 0.73453E+00_JPRB, 0.67162E+00_JPRB, 0.63265E+00_JPRB, 0.59857E+00_JPRB ,&
&  0.57121E+00_JPRB, 0.55060E+00_JPRB, 0.52504E+00_JPRB, 0.48218E+00_JPRB, 0.47204E+00_JPRB, 0.46189E+00_JPRB, 0.43825E+00_JPRB ,&
&  0.42477E+00_JPRB, 0.40380E+00_JPRB, 0.38367E+00_JPRB, 0.38369E+00_JPRB, 0.36425E+00_JPRB, 0.34867E+00_JPRB, 0.34077E+00_JPRB ,&
&  0.32489E+00_JPRB, 0.29627E+00_JPRB, 0.28465E+00_JPRB, 0.28616E+00_JPRB, 0.28652E+00_JPRB, 0.29319E+00_JPRB, 0.31551E+00_JPRB ,&
&  0.32122E+00_JPRB, 0.33580E+00_JPRB, 0.35774E+00_JPRB, 0.36956E+00_JPRB, 0.38651E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.39325E+00_JPRB, 0.41352E+00_JPRB, 0.42975E+00_JPRB, 0.42632E+00_JPRB, 0.41564E+00_JPRB, 0.40270E+00_JPRB, 0.39847E+00_JPRB ,&
&  0.40920E+00_JPRB, 0.35132E+00_JPRB, 0.32418E+00_JPRB, 0.30910E+00_JPRB, 0.30190E+00_JPRB, 0.28853E+00_JPRB, 0.27522E+00_JPRB ,&
&  0.27395E+00_JPRB, 0.28241E+00_JPRB, 0.29922E+00_JPRB, 0.31214E+00_JPRB, 0.32284E+00_JPRB, 0.32970E+00_JPRB, 0.33423E+00_JPRB ,&
&  0.34044E+00_JPRB, 0.34736E+00_JPRB, 0.35102E+00_JPRB, 0.35550E+00_JPRB, 0.36122E+00_JPRB, 0.37447E+00_JPRB, 0.39328E+00_JPRB ,&
&  0.40885E+00_JPRB, 0.41795E+00_JPRB, 0.40496E+00_JPRB, 0.40668E+00_JPRB, 0.41238E+00_JPRB, 0.41447E+00_JPRB, 0.42344E+00_JPRB ,&
&  0.30585E+00_JPRB, 0.29759E+00_JPRB, 0.35301E+00_JPRB, 0.35484E+00_JPRB, 0.36213E+00_JPRB, 0.36303E+00_JPRB, 0.36334E+00_JPRB ,&
&  0.36989E+00_JPRB, 0.38709E+00_JPRB, 0.41967E+00_JPRB, 0.47400E+00_JPRB, 0.53552E+00_JPRB, 0.61127E+00_JPRB, 0.71360E+00_JPRB ,&
&  0.92057E+00_JPRB, 0.90750E+00_JPRB, 0.96106E+00_JPRB, 0.10789E+01_JPRB, 0.12998E+01_JPRB, 0.14129E+01_JPRB, 0.14352E+01_JPRB ,&
&  0.14024E+01_JPRB, 0.12980E+01_JPRB, 0.12100E+01_JPRB, 0.11490E+01_JPRB, 0.10755E+01_JPRB, 0.98884E+00_JPRB, 0.91042E+00_JPRB ,&
&  0.82453E+00_JPRB, 0.74643E+00_JPRB, 0.69730E+00_JPRB, 0.66055E+00_JPRB, 0.61348E+00_JPRB, 0.58233E+00_JPRB, 0.55951E+00_JPRB ,&
&  0.53673E+00_JPRB, 0.51189E+00_JPRB, 0.48326E+00_JPRB, 0.45891E+00_JPRB, 0.44124E+00_JPRB, 0.43628E+00_JPRB, 0.42892E+00_JPRB ,&
&  0.41768E+00_JPRB, 0.40972E+00_JPRB, 0.40242E+00_JPRB, 0.39734E+00_JPRB, 0.37822E+00_JPRB, 0.35578E+00_JPRB, 0.34105E+00_JPRB ,&
&  0.31718E+00_JPRB, 0.29223E+00_JPRB, 0.26796E+00_JPRB, 0.26293E+00_JPRB, 0.27015E+00_JPRB, 0.27943E+00_JPRB, 0.28909E+00_JPRB ,&
&  0.30247E+00_JPRB, 0.31558E+00_JPRB, 0.32448E+00_JPRB, 0.33485E+00_JPRB, 0.35816E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.37668E+00_JPRB, 0.39120E+00_JPRB, 0.40342E+00_JPRB, 0.41116E+00_JPRB, 0.40187E+00_JPRB, 0.39677E+00_JPRB, 0.39350E+00_JPRB ,&
&  0.39985E+00_JPRB, 0.36243E+00_JPRB, 0.33801E+00_JPRB, 0.31798E+00_JPRB, 0.30486E+00_JPRB, 0.29165E+00_JPRB, 0.28798E+00_JPRB ,&
&  0.29101E+00_JPRB, 0.29698E+00_JPRB, 0.30403E+00_JPRB, 0.31026E+00_JPRB, 0.31531E+00_JPRB, 0.31743E+00_JPRB, 0.32156E+00_JPRB ,&
&  0.32801E+00_JPRB, 0.33168E+00_JPRB, 0.32868E+00_JPRB, 0.32709E+00_JPRB, 0.32873E+00_JPRB, 0.33592E+00_JPRB, 0.33843E+00_JPRB ,&
&  0.33823E+00_JPRB, 0.33448E+00_JPRB, 0.33247E+00_JPRB, 0.33988E+00_JPRB, 0.34231E+00_JPRB, 0.34025E+00_JPRB, 0.30943E+00_JPRB ,&
&  0.26109E+00_JPRB, 0.24412E+00_JPRB, 0.25524E+00_JPRB, 0.31053E+00_JPRB, 0.32584E+00_JPRB, 0.33519E+00_JPRB, 0.34949E+00_JPRB ,&
&  0.34283E+00_JPRB, 0.34713E+00_JPRB, 0.38048E+00_JPRB, 0.46653E+00_JPRB, 0.52908E+00_JPRB, 0.59646E+00_JPRB, 0.67301E+00_JPRB ,&
&  0.80260E+00_JPRB, 0.90649E+00_JPRB, 0.10108E+01_JPRB, 0.95714E+00_JPRB, 0.10598E+01_JPRB, 0.13063E+01_JPRB, 0.12262E+01_JPRB ,&
&  0.11397E+01_JPRB, 0.10775E+01_JPRB, 0.10795E+01_JPRB, 0.10112E+01_JPRB, 0.95366E+00_JPRB, 0.87044E+00_JPRB, 0.79324E+00_JPRB ,&
&  0.72221E+00_JPRB, 0.65816E+00_JPRB, 0.61548E+00_JPRB, 0.59965E+00_JPRB, 0.57529E+00_JPRB, 0.55024E+00_JPRB, 0.53290E+00_JPRB ,&
&  0.50869E+00_JPRB, 0.47780E+00_JPRB, 0.44909E+00_JPRB, 0.42511E+00_JPRB, 0.41340E+00_JPRB, 0.40929E+00_JPRB, 0.40433E+00_JPRB ,&
&  0.39700E+00_JPRB, 0.39527E+00_JPRB, 0.39767E+00_JPRB, 0.39760E+00_JPRB, 0.36912E+00_JPRB, 0.33625E+00_JPRB, 0.31514E+00_JPRB ,&
&  0.29876E+00_JPRB, 0.28583E+00_JPRB, 0.27660E+00_JPRB, 0.27696E+00_JPRB, 0.28091E+00_JPRB, 0.28481E+00_JPRB, 0.28537E+00_JPRB ,&
&  0.28834E+00_JPRB, 0.29290E+00_JPRB, 0.30542E+00_JPRB, 0.32777E+00_JPRB, 0.35276E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.33345E+00_JPRB, 0.35240E+00_JPRB, 0.37687E+00_JPRB, 0.39143E+00_JPRB, 0.39896E+00_JPRB, 0.38367E+00_JPRB, 0.36813E+00_JPRB ,&
&  0.35253E+00_JPRB, 0.33246E+00_JPRB, 0.32272E+00_JPRB, 0.31705E+00_JPRB, 0.31472E+00_JPRB, 0.31413E+00_JPRB, 0.31492E+00_JPRB ,&
&  0.31612E+00_JPRB, 0.31615E+00_JPRB, 0.31763E+00_JPRB, 0.32246E+00_JPRB, 0.32706E+00_JPRB, 0.32726E+00_JPRB, 0.32548E+00_JPRB ,&
&  0.32444E+00_JPRB, 0.32318E+00_JPRB, 0.31803E+00_JPRB, 0.31010E+00_JPRB, 0.30855E+00_JPRB, 0.31706E+00_JPRB, 0.32083E+00_JPRB ,&
&  0.32025E+00_JPRB, 0.31280E+00_JPRB, 0.31252E+00_JPRB, 0.31933E+00_JPRB, 0.31764E+00_JPRB, 0.31068E+00_JPRB, 0.28414E+00_JPRB ,&
&  0.25191E+00_JPRB, 0.22656E+00_JPRB, 0.21761E+00_JPRB, 0.22935E+00_JPRB, 0.25625E+00_JPRB, 0.30233E+00_JPRB, 0.31645E+00_JPRB ,&
&  0.32790E+00_JPRB, 0.34774E+00_JPRB, 0.37917E+00_JPRB, 0.42864E+00_JPRB, 0.49390E+00_JPRB, 0.55960E+00_JPRB, 0.61645E+00_JPRB ,&
&  0.69027E+00_JPRB, 0.76459E+00_JPRB, 0.86700E+00_JPRB, 0.83112E+00_JPRB, 0.85375E+00_JPRB, 0.91663E+00_JPRB, 0.91790E+00_JPRB ,&
&  0.87568E+00_JPRB, 0.84522E+00_JPRB, 0.85083E+00_JPRB, 0.82356E+00_JPRB, 0.77964E+00_JPRB, 0.72586E+00_JPRB, 0.66305E+00_JPRB ,&
&  0.61102E+00_JPRB, 0.57049E+00_JPRB, 0.54255E+00_JPRB, 0.53344E+00_JPRB, 0.53128E+00_JPRB, 0.51714E+00_JPRB, 0.49819E+00_JPRB ,&
&  0.47599E+00_JPRB, 0.44449E+00_JPRB, 0.41185E+00_JPRB, 0.38840E+00_JPRB, 0.38219E+00_JPRB, 0.38588E+00_JPRB, 0.38613E+00_JPRB ,&
&  0.38553E+00_JPRB, 0.38463E+00_JPRB, 0.38212E+00_JPRB, 0.37712E+00_JPRB, 0.35397E+00_JPRB, 0.33285E+00_JPRB, 0.31864E+00_JPRB ,&
&  0.31342E+00_JPRB, 0.30903E+00_JPRB, 0.30809E+00_JPRB, 0.31137E+00_JPRB, 0.31011E+00_JPRB, 0.31070E+00_JPRB, 0.31187E+00_JPRB ,&
&  0.31111E+00_JPRB, 0.30940E+00_JPRB, 0.31240E+00_JPRB, 0.31539E+00_JPRB, 0.32186E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.35069E+00_JPRB, 0.35669E+00_JPRB, 0.36236E+00_JPRB, 0.37189E+00_JPRB, 0.38070E+00_JPRB, 0.37886E+00_JPRB, 0.37226E+00_JPRB ,&
&  0.35968E+00_JPRB, 0.34636E+00_JPRB, 0.33767E+00_JPRB, 0.33280E+00_JPRB, 0.33256E+00_JPRB, 0.33438E+00_JPRB, 0.33630E+00_JPRB ,&
&  0.33774E+00_JPRB, 0.33843E+00_JPRB, 0.33610E+00_JPRB, 0.33023E+00_JPRB, 0.32593E+00_JPRB, 0.32459E+00_JPRB, 0.32457E+00_JPRB ,&
&  0.32524E+00_JPRB, 0.32392E+00_JPRB, 0.32079E+00_JPRB, 0.31541E+00_JPRB, 0.30928E+00_JPRB, 0.31394E+00_JPRB, 0.31180E+00_JPRB ,&
&  0.30584E+00_JPRB, 0.30521E+00_JPRB, 0.31083E+00_JPRB, 0.30912E+00_JPRB, 0.30471E+00_JPRB, 0.29595E+00_JPRB, 0.26576E+00_JPRB ,&
&  0.23781E+00_JPRB, 0.21644E+00_JPRB, 0.20467E+00_JPRB, 0.20220E+00_JPRB, 0.20939E+00_JPRB, 0.22750E+00_JPRB, 0.25656E+00_JPRB ,&
&  0.29858E+00_JPRB, 0.31891E+00_JPRB, 0.34592E+00_JPRB, 0.38376E+00_JPRB, 0.43082E+00_JPRB, 0.48292E+00_JPRB, 0.52862E+00_JPRB ,&
&  0.56772E+00_JPRB, 0.61195E+00_JPRB, 0.67165E+00_JPRB, 0.73426E+00_JPRB, 0.80325E+00_JPRB, 0.76344E+00_JPRB, 0.75168E+00_JPRB ,&
&  0.79472E+00_JPRB, 0.75482E+00_JPRB, 0.72905E+00_JPRB, 0.67400E+00_JPRB, 0.62347E+00_JPRB, 0.56610E+00_JPRB, 0.52273E+00_JPRB ,&
&  0.49862E+00_JPRB, 0.48961E+00_JPRB, 0.48941E+00_JPRB, 0.49167E+00_JPRB, 0.48748E+00_JPRB, 0.47925E+00_JPRB, 0.45723E+00_JPRB ,&
&  0.43475E+00_JPRB, 0.42756E+00_JPRB, 0.47537E+00_JPRB, 0.44780E+00_JPRB, 0.42063E+00_JPRB, 0.40809E+00_JPRB, 0.39199E+00_JPRB ,&
&  0.38347E+00_JPRB, 0.37789E+00_JPRB, 0.36975E+00_JPRB, 0.36342E+00_JPRB, 0.35220E+00_JPRB, 0.33604E+00_JPRB, 0.32442E+00_JPRB ,&
&  0.32217E+00_JPRB, 0.32302E+00_JPRB, 0.32409E+00_JPRB, 0.32584E+00_JPRB, 0.32641E+00_JPRB, 0.32852E+00_JPRB, 0.33194E+00_JPRB ,&
&  0.33245E+00_JPRB, 0.33314E+00_JPRB, 0.33674E+00_JPRB, 0.34010E+00_JPRB, 0.34470E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.34267E+00_JPRB, 0.34470E+00_JPRB, 0.34721E+00_JPRB, 0.34883E+00_JPRB, 0.35126E+00_JPRB, 0.35289E+00_JPRB, 0.35190E+00_JPRB ,&
&  0.34790E+00_JPRB, 0.34559E+00_JPRB, 0.34434E+00_JPRB, 0.34611E+00_JPRB, 0.34834E+00_JPRB, 0.34980E+00_JPRB, 0.35036E+00_JPRB ,&
&  0.34784E+00_JPRB, 0.33918E+00_JPRB, 0.33274E+00_JPRB, 0.32969E+00_JPRB, 0.32767E+00_JPRB, 0.32721E+00_JPRB, 0.32696E+00_JPRB ,&
&  0.32453E+00_JPRB, 0.32033E+00_JPRB, 0.31721E+00_JPRB, 0.31031E+00_JPRB, 0.30562E+00_JPRB, 0.30542E+00_JPRB, 0.30963E+00_JPRB ,&
&  0.31150E+00_JPRB, 0.30175E+00_JPRB, 0.30072E+00_JPRB, 0.29783E+00_JPRB, 0.28790E+00_JPRB, 0.28229E+00_JPRB, 0.24439E+00_JPRB ,&
&  0.22069E+00_JPRB, 0.20567E+00_JPRB, 0.20015E+00_JPRB, 0.19782E+00_JPRB, 0.20335E+00_JPRB, 0.21701E+00_JPRB, 0.24394E+00_JPRB ,&
&  0.27404E+00_JPRB, 0.31345E+00_JPRB, 0.33111E+00_JPRB, 0.35906E+00_JPRB, 0.39100E+00_JPRB, 0.42898E+00_JPRB, 0.46181E+00_JPRB ,&
&  0.47827E+00_JPRB, 0.48847E+00_JPRB, 0.50138E+00_JPRB, 0.50823E+00_JPRB, 0.52079E+00_JPRB, 0.52566E+00_JPRB, 0.51699E+00_JPRB ,&
&  0.51882E+00_JPRB, 0.51785E+00_JPRB, 0.52567E+00_JPRB, 0.51529E+00_JPRB, 0.48843E+00_JPRB, 0.46036E+00_JPRB, 0.44094E+00_JPRB ,&
&  0.42155E+00_JPRB, 0.41776E+00_JPRB, 0.42739E+00_JPRB, 0.44370E+00_JPRB, 0.44409E+00_JPRB, 0.43873E+00_JPRB, 0.42647E+00_JPRB ,&
&  0.40919E+00_JPRB, 0.40734E+00_JPRB, 0.42592E+00_JPRB, 0.41541E+00_JPRB, 0.40787E+00_JPRB, 0.40209E+00_JPRB, 0.39075E+00_JPRB ,&
&  0.37527E+00_JPRB, 0.37410E+00_JPRB, 0.37258E+00_JPRB, 0.36582E+00_JPRB, 0.35832E+00_JPRB, 0.34794E+00_JPRB, 0.34266E+00_JPRB ,&
&  0.34138E+00_JPRB, 0.34343E+00_JPRB, 0.34328E+00_JPRB, 0.34323E+00_JPRB, 0.34272E+00_JPRB, 0.34161E+00_JPRB, 0.34292E+00_JPRB ,&
&  0.34237E+00_JPRB, 0.34312E+00_JPRB, 0.34367E+00_JPRB, 0.34333E+00_JPRB, 0.34303E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.33305E+00_JPRB, 0.33390E+00_JPRB, 0.33431E+00_JPRB, 0.33492E+00_JPRB, 0.33623E+00_JPRB, 0.33922E+00_JPRB, 0.34304E+00_JPRB ,&
&  0.34511E+00_JPRB, 0.34726E+00_JPRB, 0.34927E+00_JPRB, 0.34946E+00_JPRB, 0.34843E+00_JPRB, 0.34634E+00_JPRB, 0.34460E+00_JPRB ,&
&  0.34175E+00_JPRB, 0.33848E+00_JPRB, 0.33439E+00_JPRB, 0.33068E+00_JPRB, 0.32721E+00_JPRB, 0.32531E+00_JPRB, 0.32253E+00_JPRB ,&
&  0.31988E+00_JPRB, 0.31607E+00_JPRB, 0.30959E+00_JPRB, 0.30404E+00_JPRB, 0.30215E+00_JPRB, 0.29863E+00_JPRB, 0.29984E+00_JPRB ,&
&  0.29667E+00_JPRB, 0.29062E+00_JPRB, 0.28204E+00_JPRB, 0.27768E+00_JPRB, 0.27226E+00_JPRB, 0.23561E+00_JPRB, 0.22492E+00_JPRB ,&
&  0.21279E+00_JPRB, 0.20296E+00_JPRB, 0.19996E+00_JPRB, 0.20065E+00_JPRB, 0.20512E+00_JPRB, 0.21744E+00_JPRB, 0.23404E+00_JPRB ,&
&  0.25547E+00_JPRB, 0.31839E+00_JPRB, 0.33815E+00_JPRB, 0.36081E+00_JPRB, 0.37895E+00_JPRB, 0.39940E+00_JPRB, 0.41151E+00_JPRB ,&
&  0.42015E+00_JPRB, 0.42506E+00_JPRB, 0.43593E+00_JPRB, 0.45184E+00_JPRB, 0.46533E+00_JPRB, 0.47068E+00_JPRB, 0.46969E+00_JPRB ,&
&  0.45793E+00_JPRB, 0.44283E+00_JPRB, 0.42192E+00_JPRB, 0.40469E+00_JPRB, 0.39438E+00_JPRB, 0.38588E+00_JPRB, 0.38809E+00_JPRB ,&
&  0.39108E+00_JPRB, 0.38992E+00_JPRB, 0.39460E+00_JPRB, 0.39912E+00_JPRB, 0.39891E+00_JPRB, 0.39716E+00_JPRB, 0.39230E+00_JPRB ,&
&  0.38515E+00_JPRB, 0.38740E+00_JPRB, 0.38654E+00_JPRB, 0.38769E+00_JPRB, 0.38410E+00_JPRB, 0.37980E+00_JPRB, 0.37225E+00_JPRB ,&
&  0.37041E+00_JPRB, 0.37324E+00_JPRB, 0.37260E+00_JPRB, 0.36720E+00_JPRB, 0.36152E+00_JPRB, 0.35342E+00_JPRB, 0.34829E+00_JPRB ,&
&  0.34725E+00_JPRB, 0.34871E+00_JPRB, 0.34819E+00_JPRB, 0.34758E+00_JPRB, 0.34519E+00_JPRB, 0.34319E+00_JPRB, 0.34168E+00_JPRB ,&
&  0.33936E+00_JPRB, 0.33875E+00_JPRB, 0.33689E+00_JPRB, 0.33431E+00_JPRB, 0.33388E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.32586E+00_JPRB, 0.32695E+00_JPRB, 0.32717E+00_JPRB, 0.32783E+00_JPRB, 0.32893E+00_JPRB, 0.33170E+00_JPRB, 0.33441E+00_JPRB ,&
&  0.33631E+00_JPRB, 0.33595E+00_JPRB, 0.33624E+00_JPRB, 0.33523E+00_JPRB, 0.33430E+00_JPRB, 0.33276E+00_JPRB, 0.33237E+00_JPRB ,&
&  0.33235E+00_JPRB, 0.33169E+00_JPRB, 0.33006E+00_JPRB, 0.32710E+00_JPRB, 0.32473E+00_JPRB, 0.32153E+00_JPRB, 0.31818E+00_JPRB ,&
&  0.31423E+00_JPRB, 0.30916E+00_JPRB, 0.29861E+00_JPRB, 0.29235E+00_JPRB, 0.28390E+00_JPRB, 0.28012E+00_JPRB, 0.27541E+00_JPRB ,&
&  0.27954E+00_JPRB, 0.25539E+00_JPRB, 0.24899E+00_JPRB, 0.23688E+00_JPRB, 0.22763E+00_JPRB, 0.22072E+00_JPRB, 0.21327E+00_JPRB ,&
&  0.21005E+00_JPRB, 0.20725E+00_JPRB, 0.20773E+00_JPRB, 0.21236E+00_JPRB, 0.21979E+00_JPRB, 0.23403E+00_JPRB, 0.25033E+00_JPRB ,&
&  0.27132E+00_JPRB, 0.32187E+00_JPRB, 0.33982E+00_JPRB, 0.35839E+00_JPRB, 0.37504E+00_JPRB, 0.39318E+00_JPRB, 0.39744E+00_JPRB ,&
&  0.40616E+00_JPRB, 0.41118E+00_JPRB, 0.42111E+00_JPRB, 0.42759E+00_JPRB, 0.44361E+00_JPRB, 0.44086E+00_JPRB, 0.43408E+00_JPRB ,&
&  0.40876E+00_JPRB, 0.38284E+00_JPRB, 0.36231E+00_JPRB, 0.35578E+00_JPRB, 0.35673E+00_JPRB, 0.35970E+00_JPRB, 0.36316E+00_JPRB ,&
&  0.36609E+00_JPRB, 0.36770E+00_JPRB, 0.36675E+00_JPRB, 0.36944E+00_JPRB, 0.37208E+00_JPRB, 0.37502E+00_JPRB, 0.37379E+00_JPRB ,&
&  0.37316E+00_JPRB, 0.37558E+00_JPRB, 0.37562E+00_JPRB, 0.37794E+00_JPRB, 0.37396E+00_JPRB, 0.37084E+00_JPRB, 0.36785E+00_JPRB ,&
&  0.36973E+00_JPRB, 0.36830E+00_JPRB, 0.36444E+00_JPRB, 0.35962E+00_JPRB, 0.35691E+00_JPRB, 0.35154E+00_JPRB, 0.34833E+00_JPRB ,&
&  0.34742E+00_JPRB, 0.34982E+00_JPRB, 0.35194E+00_JPRB, 0.35203E+00_JPRB, 0.34696E+00_JPRB, 0.34210E+00_JPRB, 0.33818E+00_JPRB ,&
&  0.33401E+00_JPRB, 0.33076E+00_JPRB, 0.32690E+00_JPRB, 0.32479E+00_JPRB, 0.32542E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.32214E+00_JPRB, 0.32316E+00_JPRB, 0.32419E+00_JPRB, 0.32497E+00_JPRB, 0.32487E+00_JPRB, 0.32595E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32776E+00_JPRB, 0.32775E+00_JPRB, 0.32865E+00_JPRB, 0.32985E+00_JPRB, 0.33129E+00_JPRB, 0.33239E+00_JPRB, 0.33306E+00_JPRB ,&
&  0.33441E+00_JPRB, 0.33480E+00_JPRB, 0.33480E+00_JPRB, 0.33280E+00_JPRB, 0.33096E+00_JPRB, 0.32795E+00_JPRB, 0.32315E+00_JPRB ,&
&  0.31728E+00_JPRB, 0.31186E+00_JPRB, 0.29381E+00_JPRB, 0.28657E+00_JPRB, 0.27565E+00_JPRB, 0.26956E+00_JPRB, 0.26667E+00_JPRB ,&
&  0.26475E+00_JPRB, 0.26357E+00_JPRB, 0.25986E+00_JPRB, 0.25482E+00_JPRB, 0.24879E+00_JPRB, 0.24010E+00_JPRB, 0.23449E+00_JPRB ,&
&  0.22924E+00_JPRB, 0.22783E+00_JPRB, 0.22990E+00_JPRB, 0.23569E+00_JPRB, 0.24571E+00_JPRB, 0.25861E+00_JPRB, 0.27306E+00_JPRB ,&
&  0.29123E+00_JPRB, 0.31040E+00_JPRB, 0.33627E+00_JPRB, 0.34688E+00_JPRB, 0.35703E+00_JPRB, 0.36660E+00_JPRB, 0.37496E+00_JPRB ,&
&  0.37550E+00_JPRB, 0.37563E+00_JPRB, 0.38174E+00_JPRB, 0.37876E+00_JPRB, 0.37460E+00_JPRB, 0.36308E+00_JPRB, 0.35263E+00_JPRB ,&
&  0.33965E+00_JPRB, 0.33380E+00_JPRB, 0.32962E+00_JPRB, 0.32987E+00_JPRB, 0.33178E+00_JPRB, 0.33495E+00_JPRB, 0.33910E+00_JPRB ,&
&  0.34332E+00_JPRB, 0.34920E+00_JPRB, 0.35264E+00_JPRB, 0.35566E+00_JPRB, 0.35802E+00_JPRB, 0.36202E+00_JPRB, 0.36494E+00_JPRB ,&
&  0.36674E+00_JPRB, 0.36630E+00_JPRB, 0.36363E+00_JPRB, 0.35760E+00_JPRB, 0.35460E+00_JPRB, 0.35872E+00_JPRB, 0.35759E+00_JPRB ,&
&  0.35727E+00_JPRB, 0.35538E+00_JPRB, 0.35426E+00_JPRB, 0.35340E+00_JPRB, 0.35335E+00_JPRB, 0.35166E+00_JPRB, 0.35055E+00_JPRB ,&
&  0.35027E+00_JPRB, 0.35144E+00_JPRB, 0.35128E+00_JPRB, 0.35079E+00_JPRB, 0.34732E+00_JPRB, 0.34227E+00_JPRB, 0.33764E+00_JPRB ,&
&  0.33338E+00_JPRB, 0.32953E+00_JPRB, 0.32579E+00_JPRB, 0.32335E+00_JPRB, 0.32225E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.31753E+00_JPRB, 0.31724E+00_JPRB, 0.31647E+00_JPRB, 0.31565E+00_JPRB, 0.31454E+00_JPRB, 0.31467E+00_JPRB, 0.31576E+00_JPRB ,&
&  0.31753E+00_JPRB, 0.31824E+00_JPRB, 0.31916E+00_JPRB, 0.32124E+00_JPRB, 0.32440E+00_JPRB, 0.32638E+00_JPRB, 0.32855E+00_JPRB ,&
&  0.33073E+00_JPRB, 0.33216E+00_JPRB, 0.33321E+00_JPRB, 0.33326E+00_JPRB, 0.33211E+00_JPRB, 0.32986E+00_JPRB, 0.32827E+00_JPRB ,&
&  0.32606E+00_JPRB, 0.32400E+00_JPRB, 0.32118E+00_JPRB, 0.31803E+00_JPRB, 0.31457E+00_JPRB, 0.29141E+00_JPRB, 0.28445E+00_JPRB ,&
&  0.28270E+00_JPRB, 0.28293E+00_JPRB, 0.28638E+00_JPRB, 0.29146E+00_JPRB, 0.30087E+00_JPRB, 0.29988E+00_JPRB, 0.30093E+00_JPRB ,&
&  0.28834E+00_JPRB, 0.28882E+00_JPRB, 0.28875E+00_JPRB, 0.28979E+00_JPRB, 0.29353E+00_JPRB, 0.29763E+00_JPRB, 0.30502E+00_JPRB ,&
&  0.32197E+00_JPRB, 0.32559E+00_JPRB, 0.32821E+00_JPRB, 0.33094E+00_JPRB, 0.33331E+00_JPRB, 0.33460E+00_JPRB, 0.33473E+00_JPRB ,&
&  0.33337E+00_JPRB, 0.33139E+00_JPRB, 0.32865E+00_JPRB, 0.32478E+00_JPRB, 0.32178E+00_JPRB, 0.31910E+00_JPRB, 0.31897E+00_JPRB ,&
&  0.31804E+00_JPRB, 0.31919E+00_JPRB, 0.32077E+00_JPRB, 0.32266E+00_JPRB, 0.32389E+00_JPRB, 0.32611E+00_JPRB, 0.32991E+00_JPRB ,&
&  0.33363E+00_JPRB, 0.33738E+00_JPRB, 0.33997E+00_JPRB, 0.34276E+00_JPRB, 0.34445E+00_JPRB, 0.34682E+00_JPRB, 0.34753E+00_JPRB ,&
&  0.34831E+00_JPRB, 0.34754E+00_JPRB, 0.34756E+00_JPRB, 0.34724E+00_JPRB, 0.34683E+00_JPRB, 0.34700E+00_JPRB, 0.34663E+00_JPRB ,&
&  0.34682E+00_JPRB, 0.34642E+00_JPRB, 0.34650E+00_JPRB, 0.34668E+00_JPRB, 0.34656E+00_JPRB, 0.34539E+00_JPRB, 0.34438E+00_JPRB ,&
&  0.34419E+00_JPRB, 0.34450E+00_JPRB, 0.34419E+00_JPRB, 0.34406E+00_JPRB, 0.34251E+00_JPRB, 0.33903E+00_JPRB, 0.33455E+00_JPRB ,&
&  0.33033E+00_JPRB, 0.32661E+00_JPRB, 0.32310E+00_JPRB, 0.32066E+00_JPRB, 0.31876E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.31288E+00_JPRB, 0.31291E+00_JPRB, 0.31317E+00_JPRB, 0.31325E+00_JPRB, 0.31298E+00_JPRB, 0.31319E+00_JPRB, 0.31412E+00_JPRB ,&
&  0.31617E+00_JPRB, 0.31838E+00_JPRB, 0.31940E+00_JPRB, 0.31991E+00_JPRB, 0.32053E+00_JPRB, 0.32121E+00_JPRB, 0.32201E+00_JPRB ,&
&  0.32305E+00_JPRB, 0.32374E+00_JPRB, 0.32331E+00_JPRB, 0.32205E+00_JPRB, 0.32157E+00_JPRB, 0.32165E+00_JPRB, 0.32251E+00_JPRB ,&
&  0.32320E+00_JPRB, 0.32325E+00_JPRB, 0.32273E+00_JPRB, 0.32239E+00_JPRB, 0.32127E+00_JPRB, 0.32013E+00_JPRB, 0.31879E+00_JPRB ,&
&  0.31801E+00_JPRB, 0.31747E+00_JPRB, 0.31714E+00_JPRB, 0.31685E+00_JPRB, 0.31732E+00_JPRB, 0.31768E+00_JPRB, 0.31801E+00_JPRB ,&
&  0.31822E+00_JPRB, 0.31830E+00_JPRB, 0.31892E+00_JPRB, 0.31941E+00_JPRB, 0.31978E+00_JPRB, 0.31956E+00_JPRB, 0.31855E+00_JPRB ,&
&  0.31681E+00_JPRB, 0.31513E+00_JPRB, 0.31421E+00_JPRB, 0.31304E+00_JPRB, 0.31200E+00_JPRB, 0.31094E+00_JPRB, 0.31041E+00_JPRB ,&
&  0.30953E+00_JPRB, 0.30877E+00_JPRB, 0.30860E+00_JPRB, 0.30918E+00_JPRB, 0.30958E+00_JPRB, 0.31047E+00_JPRB, 0.31180E+00_JPRB ,&
&  0.31262E+00_JPRB, 0.31350E+00_JPRB, 0.31445E+00_JPRB, 0.31548E+00_JPRB, 0.31647E+00_JPRB, 0.31757E+00_JPRB, 0.31917E+00_JPRB ,&
&  0.32118E+00_JPRB, 0.32373E+00_JPRB, 0.32528E+00_JPRB, 0.32692E+00_JPRB, 0.32878E+00_JPRB, 0.33173E+00_JPRB, 0.33307E+00_JPRB ,&
&  0.33409E+00_JPRB, 0.33498E+00_JPRB, 0.33599E+00_JPRB, 0.33663E+00_JPRB, 0.33712E+00_JPRB, 0.33760E+00_JPRB, 0.33790E+00_JPRB ,&
&  0.33843E+00_JPRB, 0.33844E+00_JPRB, 0.33825E+00_JPRB, 0.33774E+00_JPRB, 0.33710E+00_JPRB, 0.33614E+00_JPRB, 0.33471E+00_JPRB ,&
&  0.33357E+00_JPRB, 0.33178E+00_JPRB, 0.32971E+00_JPRB, 0.32820E+00_JPRB, 0.32728E+00_JPRB, 0.32528E+00_JPRB, 0.32272E+00_JPRB ,&
&  0.31999E+00_JPRB, 0.31773E+00_JPRB, 0.31619E+00_JPRB, 0.31482E+00_JPRB, 0.31375E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.31278E+00_JPRB, 0.31265E+00_JPRB, 0.31267E+00_JPRB, 0.31285E+00_JPRB, 0.31292E+00_JPRB, 0.31283E+00_JPRB, 0.31262E+00_JPRB ,&
&  0.31255E+00_JPRB, 0.31306E+00_JPRB, 0.31390E+00_JPRB, 0.31476E+00_JPRB, 0.31534E+00_JPRB, 0.31555E+00_JPRB, 0.31549E+00_JPRB ,&
&  0.31530E+00_JPRB, 0.31491E+00_JPRB, 0.31407E+00_JPRB, 0.31337E+00_JPRB, 0.31370E+00_JPRB, 0.31406E+00_JPRB, 0.31420E+00_JPRB ,&
&  0.31430E+00_JPRB, 0.31455E+00_JPRB, 0.31489E+00_JPRB, 0.31509E+00_JPRB, 0.31527E+00_JPRB, 0.31534E+00_JPRB, 0.31548E+00_JPRB ,&
&  0.31554E+00_JPRB, 0.31549E+00_JPRB, 0.31517E+00_JPRB, 0.31496E+00_JPRB, 0.31455E+00_JPRB, 0.31443E+00_JPRB, 0.31420E+00_JPRB ,&
&  0.31416E+00_JPRB, 0.31385E+00_JPRB, 0.31343E+00_JPRB, 0.31287E+00_JPRB, 0.31221E+00_JPRB, 0.31097E+00_JPRB, 0.30983E+00_JPRB ,&
&  0.30886E+00_JPRB, 0.30813E+00_JPRB, 0.30774E+00_JPRB, 0.30777E+00_JPRB, 0.30784E+00_JPRB, 0.30801E+00_JPRB, 0.30801E+00_JPRB ,&
&  0.30788E+00_JPRB, 0.30788E+00_JPRB, 0.30798E+00_JPRB, 0.30803E+00_JPRB, 0.30801E+00_JPRB, 0.30794E+00_JPRB, 0.30797E+00_JPRB ,&
&  0.30784E+00_JPRB, 0.30800E+00_JPRB, 0.30856E+00_JPRB, 0.30922E+00_JPRB, 0.30998E+00_JPRB, 0.31071E+00_JPRB, 0.31156E+00_JPRB ,&
&  0.31253E+00_JPRB, 0.31354E+00_JPRB, 0.31442E+00_JPRB, 0.31535E+00_JPRB, 0.31634E+00_JPRB, 0.31734E+00_JPRB, 0.31813E+00_JPRB ,&
&  0.31891E+00_JPRB, 0.31979E+00_JPRB, 0.32081E+00_JPRB, 0.32128E+00_JPRB, 0.32167E+00_JPRB, 0.32197E+00_JPRB, 0.32214E+00_JPRB ,&
&  0.32209E+00_JPRB, 0.32191E+00_JPRB, 0.32169E+00_JPRB, 0.32138E+00_JPRB, 0.32073E+00_JPRB, 0.31997E+00_JPRB, 0.31911E+00_JPRB ,&
&  0.31830E+00_JPRB, 0.31722E+00_JPRB, 0.31616E+00_JPRB, 0.31544E+00_JPRB, 0.31492E+00_JPRB, 0.31440E+00_JPRB, 0.31384E+00_JPRB ,&
&  0.31325E+00_JPRB, 0.31295E+00_JPRB, 0.31300E+00_JPRB, 0.31306E+00_JPRB, 0.31305E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB ,&
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB ,&
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB ,&
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB ,&
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB ,&
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB ,&
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB ,&
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB ,&
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB ,&
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB ,&
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB ,&
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB ,&
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB ,&
&  0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB, 0.30725E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB, 0.21693E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.22789E+00_JPRB, 0.22819E+00_JPRB, 0.22834E+00_JPRB, 0.22844E+00_JPRB, 0.22708E+00_JPRB, 0.22767E+00_JPRB, 0.22788E+00_JPRB ,&
&  0.22824E+00_JPRB, 0.22855E+00_JPRB, 0.22911E+00_JPRB, 0.22975E+00_JPRB, 0.23036E+00_JPRB, 0.23091E+00_JPRB, 0.23137E+00_JPRB ,&
&  0.23164E+00_JPRB, 0.23199E+00_JPRB, 0.23081E+00_JPRB, 0.23015E+00_JPRB, 0.23010E+00_JPRB, 0.23024E+00_JPRB, 0.23026E+00_JPRB ,&
&  0.23025E+00_JPRB, 0.23018E+00_JPRB, 0.22980E+00_JPRB, 0.22951E+00_JPRB, 0.22938E+00_JPRB, 0.22922E+00_JPRB, 0.22895E+00_JPRB ,&
&  0.22857E+00_JPRB, 0.22829E+00_JPRB, 0.22794E+00_JPRB, 0.22755E+00_JPRB, 0.22739E+00_JPRB, 0.22714E+00_JPRB, 0.22652E+00_JPRB ,&
&  0.22580E+00_JPRB, 0.22479E+00_JPRB, 0.22386E+00_JPRB, 0.22302E+00_JPRB, 0.22235E+00_JPRB, 0.22199E+00_JPRB, 0.22163E+00_JPRB ,&
&  0.22115E+00_JPRB, 0.22051E+00_JPRB, 0.21989E+00_JPRB, 0.21917E+00_JPRB, 0.21855E+00_JPRB, 0.21799E+00_JPRB, 0.21734E+00_JPRB ,&
&  0.21661E+00_JPRB, 0.21588E+00_JPRB, 0.21513E+00_JPRB, 0.21431E+00_JPRB, 0.21352E+00_JPRB, 0.21273E+00_JPRB, 0.21187E+00_JPRB ,&
&  0.21102E+00_JPRB, 0.21064E+00_JPRB, 0.21020E+00_JPRB, 0.20982E+00_JPRB, 0.20954E+00_JPRB, 0.20932E+00_JPRB, 0.20911E+00_JPRB ,&
&  0.20894E+00_JPRB, 0.20876E+00_JPRB, 0.20866E+00_JPRB, 0.20858E+00_JPRB, 0.20857E+00_JPRB, 0.20863E+00_JPRB, 0.20879E+00_JPRB ,&
&  0.20898E+00_JPRB, 0.20921E+00_JPRB, 0.20954E+00_JPRB, 0.21004E+00_JPRB, 0.21055E+00_JPRB, 0.21113E+00_JPRB, 0.21171E+00_JPRB ,&
&  0.21235E+00_JPRB, 0.21298E+00_JPRB, 0.21362E+00_JPRB, 0.21422E+00_JPRB, 0.21502E+00_JPRB, 0.21580E+00_JPRB, 0.21665E+00_JPRB ,&
&  0.21749E+00_JPRB, 0.21840E+00_JPRB, 0.21931E+00_JPRB, 0.22028E+00_JPRB, 0.22137E+00_JPRB, 0.22171E+00_JPRB, 0.22206E+00_JPRB ,&
&  0.22250E+00_JPRB, 0.22314E+00_JPRB, 0.22373E+00_JPRB, 0.22448E+00_JPRB, 0.22574E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.25446E+00_JPRB, 0.26089E+00_JPRB, 0.26539E+00_JPRB, 0.27036E+00_JPRB, 0.27059E+00_JPRB, 0.30835E+00_JPRB, 0.30735E+00_JPRB ,&
&  0.30597E+00_JPRB, 0.29112E+00_JPRB, 0.28831E+00_JPRB, 0.28336E+00_JPRB, 0.27771E+00_JPRB, 0.27160E+00_JPRB, 0.26631E+00_JPRB ,&
&  0.26153E+00_JPRB, 0.25626E+00_JPRB, 0.25192E+00_JPRB, 0.24791E+00_JPRB, 0.24463E+00_JPRB, 0.24171E+00_JPRB, 0.23964E+00_JPRB ,&
&  0.23909E+00_JPRB, 0.24045E+00_JPRB, 0.24339E+00_JPRB, 0.24521E+00_JPRB, 0.24611E+00_JPRB, 0.24687E+00_JPRB, 0.24760E+00_JPRB ,&
&  0.24775E+00_JPRB, 0.24788E+00_JPRB, 0.24694E+00_JPRB, 0.24510E+00_JPRB, 0.24324E+00_JPRB, 0.24165E+00_JPRB, 0.24036E+00_JPRB ,&
&  0.23912E+00_JPRB, 0.23782E+00_JPRB, 0.23644E+00_JPRB, 0.23552E+00_JPRB, 0.23434E+00_JPRB, 0.23281E+00_JPRB, 0.23080E+00_JPRB ,&
&  0.22880E+00_JPRB, 0.22681E+00_JPRB, 0.22450E+00_JPRB, 0.22265E+00_JPRB, 0.22086E+00_JPRB, 0.21905E+00_JPRB, 0.21726E+00_JPRB ,&
&  0.21624E+00_JPRB, 0.21525E+00_JPRB, 0.21402E+00_JPRB, 0.21245E+00_JPRB, 0.21124E+00_JPRB, 0.20995E+00_JPRB, 0.20882E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20728E+00_JPRB, 0.20648E+00_JPRB, 0.20566E+00_JPRB, 0.20505E+00_JPRB, 0.20436E+00_JPRB, 0.20374E+00_JPRB ,&
&  0.20320E+00_JPRB, 0.20260E+00_JPRB, 0.20246E+00_JPRB, 0.20240E+00_JPRB, 0.20248E+00_JPRB, 0.20254E+00_JPRB, 0.20284E+00_JPRB ,&
&  0.20329E+00_JPRB, 0.20394E+00_JPRB, 0.20472E+00_JPRB, 0.20546E+00_JPRB, 0.20619E+00_JPRB, 0.20695E+00_JPRB, 0.20804E+00_JPRB ,&
&  0.20924E+00_JPRB, 0.21054E+00_JPRB, 0.21189E+00_JPRB, 0.21296E+00_JPRB, 0.21506E+00_JPRB, 0.21705E+00_JPRB, 0.21918E+00_JPRB ,&
&  0.22163E+00_JPRB, 0.22451E+00_JPRB, 0.22757E+00_JPRB, 0.23081E+00_JPRB, 0.23502E+00_JPRB, 0.23785E+00_JPRB, 0.23996E+00_JPRB ,&
&  0.24227E+00_JPRB, 0.24605E+00_JPRB, 0.24781E+00_JPRB, 0.24930E+00_JPRB, 0.25150E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.32523E+00_JPRB, 0.32966E+00_JPRB, 0.33349E+00_JPRB, 0.33685E+00_JPRB, 0.33977E+00_JPRB, 0.34041E+00_JPRB, 0.33942E+00_JPRB ,&
&  0.33045E+00_JPRB, 0.32373E+00_JPRB, 0.31561E+00_JPRB, 0.30850E+00_JPRB, 0.30193E+00_JPRB, 0.29452E+00_JPRB, 0.28791E+00_JPRB ,&
&  0.28116E+00_JPRB, 0.27444E+00_JPRB, 0.26799E+00_JPRB, 0.26047E+00_JPRB, 0.25261E+00_JPRB, 0.24510E+00_JPRB, 0.24038E+00_JPRB ,&
&  0.23765E+00_JPRB, 0.23745E+00_JPRB, 0.24020E+00_JPRB, 0.24398E+00_JPRB, 0.24890E+00_JPRB, 0.25525E+00_JPRB, 0.26203E+00_JPRB ,&
&  0.26636E+00_JPRB, 0.26878E+00_JPRB, 0.26844E+00_JPRB, 0.26719E+00_JPRB, 0.26530E+00_JPRB, 0.26361E+00_JPRB, 0.26100E+00_JPRB ,&
&  0.25768E+00_JPRB, 0.25514E+00_JPRB, 0.25400E+00_JPRB, 0.25450E+00_JPRB, 0.25390E+00_JPRB, 0.25181E+00_JPRB, 0.24794E+00_JPRB ,&
&  0.24374E+00_JPRB, 0.23955E+00_JPRB, 0.23534E+00_JPRB, 0.23146E+00_JPRB, 0.22827E+00_JPRB, 0.22497E+00_JPRB, 0.22147E+00_JPRB ,&
&  0.21923E+00_JPRB, 0.21684E+00_JPRB, 0.21421E+00_JPRB, 0.21169E+00_JPRB, 0.20960E+00_JPRB, 0.20799E+00_JPRB, 0.20664E+00_JPRB ,&
&  0.20557E+00_JPRB, 0.20473E+00_JPRB, 0.20405E+00_JPRB, 0.20353E+00_JPRB, 0.20304E+00_JPRB, 0.20232E+00_JPRB, 0.20161E+00_JPRB ,&
&  0.20089E+00_JPRB, 0.20061E+00_JPRB, 0.20017E+00_JPRB, 0.20019E+00_JPRB, 0.20043E+00_JPRB, 0.20058E+00_JPRB, 0.20107E+00_JPRB ,&
&  0.20215E+00_JPRB, 0.20327E+00_JPRB, 0.20393E+00_JPRB, 0.20499E+00_JPRB, 0.20602E+00_JPRB, 0.20736E+00_JPRB, 0.20906E+00_JPRB ,&
&  0.21018E+00_JPRB, 0.21143E+00_JPRB, 0.21288E+00_JPRB, 0.21444E+00_JPRB, 0.21693E+00_JPRB, 0.21964E+00_JPRB, 0.22265E+00_JPRB ,&
&  0.22652E+00_JPRB, 0.23035E+00_JPRB, 0.23469E+00_JPRB, 0.23982E+00_JPRB, 0.24865E+00_JPRB, 0.25525E+00_JPRB, 0.26395E+00_JPRB ,&
&  0.27288E+00_JPRB, 0.27905E+00_JPRB, 0.31226E+00_JPRB, 0.31462E+00_JPRB, 0.31963E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.33362E+00_JPRB, 0.34011E+00_JPRB, 0.34701E+00_JPRB, 0.35201E+00_JPRB, 0.35572E+00_JPRB, 0.35870E+00_JPRB, 0.35773E+00_JPRB ,&
&  0.35159E+00_JPRB, 0.33616E+00_JPRB, 0.32465E+00_JPRB, 0.31366E+00_JPRB, 0.30428E+00_JPRB, 0.29546E+00_JPRB, 0.28719E+00_JPRB ,&
&  0.27937E+00_JPRB, 0.27292E+00_JPRB, 0.26770E+00_JPRB, 0.26318E+00_JPRB, 0.25698E+00_JPRB, 0.25156E+00_JPRB, 0.24721E+00_JPRB ,&
&  0.24360E+00_JPRB, 0.24343E+00_JPRB, 0.24558E+00_JPRB, 0.24985E+00_JPRB, 0.25775E+00_JPRB, 0.26602E+00_JPRB, 0.27530E+00_JPRB ,&
&  0.28967E+00_JPRB, 0.29422E+00_JPRB, 0.29325E+00_JPRB, 0.29256E+00_JPRB, 0.28766E+00_JPRB, 0.28166E+00_JPRB, 0.27226E+00_JPRB ,&
&  0.27230E+00_JPRB, 0.27041E+00_JPRB, 0.27705E+00_JPRB, 0.27741E+00_JPRB, 0.27627E+00_JPRB, 0.27575E+00_JPRB, 0.26820E+00_JPRB ,&
&  0.26226E+00_JPRB, 0.25340E+00_JPRB, 0.24659E+00_JPRB, 0.23986E+00_JPRB, 0.23353E+00_JPRB, 0.22972E+00_JPRB, 0.22482E+00_JPRB ,&
&  0.22112E+00_JPRB, 0.21824E+00_JPRB, 0.21459E+00_JPRB, 0.21218E+00_JPRB, 0.21019E+00_JPRB, 0.20815E+00_JPRB, 0.20712E+00_JPRB ,&
&  0.20634E+00_JPRB, 0.20606E+00_JPRB, 0.20614E+00_JPRB, 0.20633E+00_JPRB, 0.20678E+00_JPRB, 0.20763E+00_JPRB, 0.20828E+00_JPRB ,&
&  0.20899E+00_JPRB, 0.20920E+00_JPRB, 0.20882E+00_JPRB, 0.20836E+00_JPRB, 0.20730E+00_JPRB, 0.20658E+00_JPRB, 0.20567E+00_JPRB ,&
&  0.20515E+00_JPRB, 0.20546E+00_JPRB, 0.20578E+00_JPRB, 0.20645E+00_JPRB, 0.20716E+00_JPRB, 0.20863E+00_JPRB, 0.21002E+00_JPRB ,&
&  0.21151E+00_JPRB, 0.21330E+00_JPRB, 0.21478E+00_JPRB, 0.21664E+00_JPRB, 0.21889E+00_JPRB, 0.22048E+00_JPRB, 0.22313E+00_JPRB ,&
&  0.22579E+00_JPRB, 0.22906E+00_JPRB, 0.23446E+00_JPRB, 0.24057E+00_JPRB, 0.24880E+00_JPRB, 0.25945E+00_JPRB, 0.27311E+00_JPRB ,&
&  0.29071E+00_JPRB, 0.31594E+00_JPRB, 0.31804E+00_JPRB, 0.32402E+00_JPRB, 0.32866E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.33857E+00_JPRB, 0.34108E+00_JPRB, 0.34521E+00_JPRB, 0.35050E+00_JPRB, 0.35711E+00_JPRB, 0.36295E+00_JPRB, 0.36420E+00_JPRB ,&
&  0.35511E+00_JPRB, 0.34216E+00_JPRB, 0.32416E+00_JPRB, 0.30966E+00_JPRB, 0.29588E+00_JPRB, 0.28546E+00_JPRB, 0.27693E+00_JPRB ,&
&  0.27171E+00_JPRB, 0.27114E+00_JPRB, 0.27174E+00_JPRB, 0.27298E+00_JPRB, 0.27219E+00_JPRB, 0.27339E+00_JPRB, 0.27393E+00_JPRB ,&
&  0.27294E+00_JPRB, 0.27286E+00_JPRB, 0.27276E+00_JPRB, 0.27631E+00_JPRB, 0.28309E+00_JPRB, 0.28947E+00_JPRB, 0.29582E+00_JPRB ,&
&  0.30320E+00_JPRB, 0.30570E+00_JPRB, 0.31006E+00_JPRB, 0.30907E+00_JPRB, 0.30560E+00_JPRB, 0.30110E+00_JPRB, 0.29483E+00_JPRB ,&
&  0.29003E+00_JPRB, 0.28591E+00_JPRB, 0.28434E+00_JPRB, 0.28482E+00_JPRB, 0.28503E+00_JPRB, 0.28600E+00_JPRB, 0.27945E+00_JPRB ,&
&  0.27209E+00_JPRB, 0.26196E+00_JPRB, 0.25079E+00_JPRB, 0.24265E+00_JPRB, 0.23520E+00_JPRB, 0.22982E+00_JPRB, 0.22501E+00_JPRB ,&
&  0.22054E+00_JPRB, 0.21753E+00_JPRB, 0.21443E+00_JPRB, 0.21220E+00_JPRB, 0.21152E+00_JPRB, 0.20999E+00_JPRB, 0.20918E+00_JPRB ,&
&  0.20887E+00_JPRB, 0.20932E+00_JPRB, 0.21075E+00_JPRB, 0.21210E+00_JPRB, 0.21390E+00_JPRB, 0.21599E+00_JPRB, 0.21781E+00_JPRB ,&
&  0.22021E+00_JPRB, 0.22210E+00_JPRB, 0.22231E+00_JPRB, 0.22168E+00_JPRB, 0.21968E+00_JPRB, 0.21710E+00_JPRB, 0.21472E+00_JPRB ,&
&  0.21243E+00_JPRB, 0.21079E+00_JPRB, 0.21013E+00_JPRB, 0.20936E+00_JPRB, 0.20976E+00_JPRB, 0.21086E+00_JPRB, 0.21176E+00_JPRB ,&
&  0.21337E+00_JPRB, 0.21469E+00_JPRB, 0.21618E+00_JPRB, 0.21859E+00_JPRB, 0.22095E+00_JPRB, 0.22302E+00_JPRB, 0.22521E+00_JPRB ,&
&  0.22549E+00_JPRB, 0.22715E+00_JPRB, 0.22931E+00_JPRB, 0.23300E+00_JPRB, 0.24356E+00_JPRB, 0.25157E+00_JPRB, 0.27007E+00_JPRB ,&
&  0.28741E+00_JPRB, 0.32389E+00_JPRB, 0.32538E+00_JPRB, 0.32959E+00_JPRB, 0.33514E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.34143E+00_JPRB, 0.34582E+00_JPRB, 0.34684E+00_JPRB, 0.35011E+00_JPRB, 0.35442E+00_JPRB, 0.35951E+00_JPRB, 0.36366E+00_JPRB ,&
&  0.36856E+00_JPRB, 0.36829E+00_JPRB, 0.36313E+00_JPRB, 0.35644E+00_JPRB, 0.35072E+00_JPRB, 0.33387E+00_JPRB, 0.32477E+00_JPRB ,&
&  0.31743E+00_JPRB, 0.31673E+00_JPRB, 0.31735E+00_JPRB, 0.31890E+00_JPRB, 0.32005E+00_JPRB, 0.32502E+00_JPRB, 0.32663E+00_JPRB ,&
&  0.32435E+00_JPRB, 0.31940E+00_JPRB, 0.31445E+00_JPRB, 0.31366E+00_JPRB, 0.31624E+00_JPRB, 0.31830E+00_JPRB, 0.31568E+00_JPRB ,&
&  0.31211E+00_JPRB, 0.31183E+00_JPRB, 0.31253E+00_JPRB, 0.31401E+00_JPRB, 0.32379E+00_JPRB, 0.31984E+00_JPRB, 0.31557E+00_JPRB ,&
&  0.30848E+00_JPRB, 0.30228E+00_JPRB, 0.29790E+00_JPRB, 0.29816E+00_JPRB, 0.30075E+00_JPRB, 0.30115E+00_JPRB, 0.30365E+00_JPRB ,&
&  0.29146E+00_JPRB, 0.28137E+00_JPRB, 0.26682E+00_JPRB, 0.25551E+00_JPRB, 0.24508E+00_JPRB, 0.23794E+00_JPRB, 0.23115E+00_JPRB ,&
&  0.22535E+00_JPRB, 0.22063E+00_JPRB, 0.21611E+00_JPRB, 0.21457E+00_JPRB, 0.21540E+00_JPRB, 0.21564E+00_JPRB, 0.21545E+00_JPRB ,&
&  0.21611E+00_JPRB, 0.21708E+00_JPRB, 0.21786E+00_JPRB, 0.21879E+00_JPRB, 0.22022E+00_JPRB, 0.22263E+00_JPRB, 0.22581E+00_JPRB ,&
&  0.23088E+00_JPRB, 0.23496E+00_JPRB, 0.23720E+00_JPRB, 0.23807E+00_JPRB, 0.23500E+00_JPRB, 0.22993E+00_JPRB, 0.22460E+00_JPRB ,&
&  0.21950E+00_JPRB, 0.21709E+00_JPRB, 0.21502E+00_JPRB, 0.21404E+00_JPRB, 0.21391E+00_JPRB, 0.21479E+00_JPRB, 0.21649E+00_JPRB ,&
&  0.21815E+00_JPRB, 0.22018E+00_JPRB, 0.22152E+00_JPRB, 0.22316E+00_JPRB, 0.22518E+00_JPRB, 0.22695E+00_JPRB, 0.22880E+00_JPRB ,&
&  0.23086E+00_JPRB, 0.23247E+00_JPRB, 0.23520E+00_JPRB, 0.23853E+00_JPRB, 0.24477E+00_JPRB, 0.25249E+00_JPRB, 0.26639E+00_JPRB ,&
&  0.28317E+00_JPRB, 0.31625E+00_JPRB, 0.32805E+00_JPRB, 0.33301E+00_JPRB, 0.33927E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.33870E+00_JPRB, 0.34492E+00_JPRB, 0.34998E+00_JPRB, 0.35999E+00_JPRB, 0.36261E+00_JPRB, 0.36505E+00_JPRB, 0.36695E+00_JPRB ,&
&  0.37265E+00_JPRB, 0.37608E+00_JPRB, 0.37376E+00_JPRB, 0.37303E+00_JPRB, 0.37208E+00_JPRB, 0.37112E+00_JPRB, 0.37184E+00_JPRB ,&
&  0.37166E+00_JPRB, 0.37359E+00_JPRB, 0.37137E+00_JPRB, 0.36852E+00_JPRB, 0.37035E+00_JPRB, 0.37502E+00_JPRB, 0.37820E+00_JPRB ,&
&  0.37332E+00_JPRB, 0.37229E+00_JPRB, 0.38096E+00_JPRB, 0.38342E+00_JPRB, 0.38498E+00_JPRB, 0.38721E+00_JPRB, 0.38718E+00_JPRB ,&
&  0.37391E+00_JPRB, 0.34942E+00_JPRB, 0.32196E+00_JPRB, 0.32235E+00_JPRB, 0.33684E+00_JPRB, 0.33264E+00_JPRB, 0.33122E+00_JPRB ,&
&  0.32838E+00_JPRB, 0.32411E+00_JPRB, 0.31995E+00_JPRB, 0.31823E+00_JPRB, 0.31549E+00_JPRB, 0.31119E+00_JPRB, 0.30958E+00_JPRB ,&
&  0.30880E+00_JPRB, 0.31669E+00_JPRB, 0.31310E+00_JPRB, 0.29292E+00_JPRB, 0.27680E+00_JPRB, 0.26814E+00_JPRB, 0.25717E+00_JPRB ,&
&  0.24464E+00_JPRB, 0.23682E+00_JPRB, 0.23534E+00_JPRB, 0.23399E+00_JPRB, 0.23584E+00_JPRB, 0.23692E+00_JPRB, 0.23806E+00_JPRB ,&
&  0.23911E+00_JPRB, 0.23910E+00_JPRB, 0.23473E+00_JPRB, 0.22901E+00_JPRB, 0.22586E+00_JPRB, 0.22576E+00_JPRB, 0.22805E+00_JPRB ,&
&  0.23329E+00_JPRB, 0.23920E+00_JPRB, 0.25279E+00_JPRB, 0.26044E+00_JPRB, 0.25254E+00_JPRB, 0.24633E+00_JPRB, 0.23930E+00_JPRB ,&
&  0.22976E+00_JPRB, 0.22594E+00_JPRB, 0.22280E+00_JPRB, 0.22104E+00_JPRB, 0.22177E+00_JPRB, 0.22473E+00_JPRB, 0.22645E+00_JPRB ,&
&  0.22872E+00_JPRB, 0.23103E+00_JPRB, 0.23195E+00_JPRB, 0.23274E+00_JPRB, 0.23341E+00_JPRB, 0.23508E+00_JPRB, 0.23680E+00_JPRB ,&
&  0.23833E+00_JPRB, 0.24098E+00_JPRB, 0.24424E+00_JPRB, 0.24861E+00_JPRB, 0.25247E+00_JPRB, 0.25480E+00_JPRB, 0.26115E+00_JPRB ,&
&  0.26892E+00_JPRB, 0.28506E+00_JPRB, 0.31186E+00_JPRB, 0.33705E+00_JPRB, 0.33451E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.34561E+00_JPRB, 0.35042E+00_JPRB, 0.35699E+00_JPRB, 0.36433E+00_JPRB, 0.36950E+00_JPRB, 0.37166E+00_JPRB, 0.37394E+00_JPRB ,&
&  0.37701E+00_JPRB, 0.37830E+00_JPRB, 0.37931E+00_JPRB, 0.38207E+00_JPRB, 0.38757E+00_JPRB, 0.39029E+00_JPRB, 0.39477E+00_JPRB ,&
&  0.39642E+00_JPRB, 0.39436E+00_JPRB, 0.38776E+00_JPRB, 0.38570E+00_JPRB, 0.38905E+00_JPRB, 0.39368E+00_JPRB, 0.39896E+00_JPRB ,&
&  0.40456E+00_JPRB, 0.40973E+00_JPRB, 0.41228E+00_JPRB, 0.41317E+00_JPRB, 0.41354E+00_JPRB, 0.41358E+00_JPRB, 0.41109E+00_JPRB ,&
&  0.40686E+00_JPRB, 0.36834E+00_JPRB, 0.34170E+00_JPRB, 0.34348E+00_JPRB, 0.35553E+00_JPRB, 0.35080E+00_JPRB, 0.34935E+00_JPRB ,&
&  0.34784E+00_JPRB, 0.34464E+00_JPRB, 0.33983E+00_JPRB, 0.33605E+00_JPRB, 0.33118E+00_JPRB, 0.32496E+00_JPRB, 0.31984E+00_JPRB ,&
&  0.31891E+00_JPRB, 0.32345E+00_JPRB, 0.32523E+00_JPRB, 0.32816E+00_JPRB, 0.32784E+00_JPRB, 0.33056E+00_JPRB, 0.33004E+00_JPRB ,&
&  0.32541E+00_JPRB, 0.32003E+00_JPRB, 0.31852E+00_JPRB, 0.31840E+00_JPRB, 0.31945E+00_JPRB, 0.32201E+00_JPRB, 0.32236E+00_JPRB ,&
&  0.30903E+00_JPRB, 0.29766E+00_JPRB, 0.27941E+00_JPRB, 0.26176E+00_JPRB, 0.24874E+00_JPRB, 0.24272E+00_JPRB, 0.23830E+00_JPRB ,&
&  0.23711E+00_JPRB, 0.24214E+00_JPRB, 0.25734E+00_JPRB, 0.27631E+00_JPRB, 0.29191E+00_JPRB, 0.27387E+00_JPRB, 0.25778E+00_JPRB ,&
&  0.24781E+00_JPRB, 0.24262E+00_JPRB, 0.23835E+00_JPRB, 0.23665E+00_JPRB, 0.23677E+00_JPRB, 0.23867E+00_JPRB, 0.24154E+00_JPRB ,&
&  0.24574E+00_JPRB, 0.24980E+00_JPRB, 0.25256E+00_JPRB, 0.25389E+00_JPRB, 0.25457E+00_JPRB, 0.25524E+00_JPRB, 0.25463E+00_JPRB ,&
&  0.25421E+00_JPRB, 0.25694E+00_JPRB, 0.26308E+00_JPRB, 0.27133E+00_JPRB, 0.27927E+00_JPRB, 0.28792E+00_JPRB, 0.30022E+00_JPRB ,&
&  0.31408E+00_JPRB, 0.32632E+00_JPRB, 0.33054E+00_JPRB, 0.33614E+00_JPRB, 0.34111E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.36531E+00_JPRB, 0.36948E+00_JPRB, 0.37329E+00_JPRB, 0.37750E+00_JPRB, 0.38356E+00_JPRB, 0.38650E+00_JPRB, 0.38734E+00_JPRB ,&
&  0.38864E+00_JPRB, 0.38933E+00_JPRB, 0.39100E+00_JPRB, 0.39533E+00_JPRB, 0.40104E+00_JPRB, 0.40496E+00_JPRB, 0.41039E+00_JPRB ,&
&  0.41050E+00_JPRB, 0.40229E+00_JPRB, 0.39465E+00_JPRB, 0.39431E+00_JPRB, 0.39962E+00_JPRB, 0.40522E+00_JPRB, 0.41240E+00_JPRB ,&
&  0.41958E+00_JPRB, 0.42493E+00_JPRB, 0.42803E+00_JPRB, 0.42961E+00_JPRB, 0.43168E+00_JPRB, 0.43153E+00_JPRB, 0.42867E+00_JPRB ,&
&  0.42487E+00_JPRB, 0.41658E+00_JPRB, 0.39720E+00_JPRB, 0.39433E+00_JPRB, 0.38569E+00_JPRB, 0.37811E+00_JPRB, 0.37285E+00_JPRB ,&
&  0.36857E+00_JPRB, 0.36522E+00_JPRB, 0.36158E+00_JPRB, 0.35869E+00_JPRB, 0.35540E+00_JPRB, 0.34915E+00_JPRB, 0.34625E+00_JPRB ,&
&  0.34367E+00_JPRB, 0.34147E+00_JPRB, 0.33947E+00_JPRB, 0.34026E+00_JPRB, 0.34124E+00_JPRB, 0.34460E+00_JPRB, 0.34639E+00_JPRB ,&
&  0.34369E+00_JPRB, 0.34040E+00_JPRB, 0.33780E+00_JPRB, 0.33948E+00_JPRB, 0.34164E+00_JPRB, 0.34314E+00_JPRB, 0.34374E+00_JPRB ,&
&  0.34160E+00_JPRB, 0.33754E+00_JPRB, 0.33338E+00_JPRB, 0.32843E+00_JPRB, 0.32423E+00_JPRB, 0.29850E+00_JPRB, 0.28695E+00_JPRB ,&
&  0.28294E+00_JPRB, 0.29154E+00_JPRB, 0.30435E+00_JPRB, 0.30260E+00_JPRB, 0.30379E+00_JPRB, 0.30363E+00_JPRB, 0.30114E+00_JPRB ,&
&  0.29280E+00_JPRB, 0.27900E+00_JPRB, 0.26925E+00_JPRB, 0.26480E+00_JPRB, 0.26152E+00_JPRB, 0.26203E+00_JPRB, 0.26735E+00_JPRB ,&
&  0.27666E+00_JPRB, 0.28811E+00_JPRB, 0.29690E+00_JPRB, 0.30099E+00_JPRB, 0.30013E+00_JPRB, 0.29707E+00_JPRB, 0.29295E+00_JPRB ,&
&  0.28811E+00_JPRB, 0.29306E+00_JPRB, 0.30642E+00_JPRB, 0.32646E+00_JPRB, 0.32894E+00_JPRB, 0.33340E+00_JPRB, 0.33639E+00_JPRB ,&
&  0.34126E+00_JPRB, 0.34737E+00_JPRB, 0.35277E+00_JPRB, 0.35872E+00_JPRB, 0.36245E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.39039E+00_JPRB, 0.39272E+00_JPRB, 0.39498E+00_JPRB, 0.39647E+00_JPRB, 0.39906E+00_JPRB, 0.40374E+00_JPRB, 0.40561E+00_JPRB ,&
&  0.40386E+00_JPRB, 0.40191E+00_JPRB, 0.40177E+00_JPRB, 0.40496E+00_JPRB, 0.41114E+00_JPRB, 0.41778E+00_JPRB, 0.42542E+00_JPRB ,&
&  0.42179E+00_JPRB, 0.40997E+00_JPRB, 0.40565E+00_JPRB, 0.40758E+00_JPRB, 0.41232E+00_JPRB, 0.41766E+00_JPRB, 0.42433E+00_JPRB ,&
&  0.43068E+00_JPRB, 0.43494E+00_JPRB, 0.43937E+00_JPRB, 0.44328E+00_JPRB, 0.44585E+00_JPRB, 0.44430E+00_JPRB, 0.44009E+00_JPRB ,&
&  0.43506E+00_JPRB, 0.43012E+00_JPRB, 0.42596E+00_JPRB, 0.41013E+00_JPRB, 0.40761E+00_JPRB, 0.39866E+00_JPRB, 0.39112E+00_JPRB ,&
&  0.38582E+00_JPRB, 0.38150E+00_JPRB, 0.37887E+00_JPRB, 0.37616E+00_JPRB, 0.37241E+00_JPRB, 0.36591E+00_JPRB, 0.36457E+00_JPRB ,&
&  0.36434E+00_JPRB, 0.36607E+00_JPRB, 0.36220E+00_JPRB, 0.36045E+00_JPRB, 0.35853E+00_JPRB, 0.35714E+00_JPRB, 0.36192E+00_JPRB ,&
&  0.36737E+00_JPRB, 0.36515E+00_JPRB, 0.36140E+00_JPRB, 0.36102E+00_JPRB, 0.36517E+00_JPRB, 0.36677E+00_JPRB, 0.36783E+00_JPRB ,&
&  0.36981E+00_JPRB, 0.37068E+00_JPRB, 0.36658E+00_JPRB, 0.36182E+00_JPRB, 0.35320E+00_JPRB, 0.34291E+00_JPRB, 0.34045E+00_JPRB ,&
&  0.33760E+00_JPRB, 0.33156E+00_JPRB, 0.32758E+00_JPRB, 0.32630E+00_JPRB, 0.32578E+00_JPRB, 0.32632E+00_JPRB, 0.32609E+00_JPRB ,&
&  0.32566E+00_JPRB, 0.32200E+00_JPRB, 0.31661E+00_JPRB, 0.31175E+00_JPRB, 0.30526E+00_JPRB, 0.30714E+00_JPRB, 0.31599E+00_JPRB ,&
&  0.32728E+00_JPRB, 0.33405E+00_JPRB, 0.33851E+00_JPRB, 0.34198E+00_JPRB, 0.34512E+00_JPRB, 0.34669E+00_JPRB, 0.34908E+00_JPRB ,&
&  0.35226E+00_JPRB, 0.35370E+00_JPRB, 0.35212E+00_JPRB, 0.35135E+00_JPRB, 0.35359E+00_JPRB, 0.35801E+00_JPRB, 0.36362E+00_JPRB ,&
&  0.36937E+00_JPRB, 0.37427E+00_JPRB, 0.37842E+00_JPRB, 0.38370E+00_JPRB, 0.38709E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.42204E+00_JPRB, 0.42209E+00_JPRB, 0.42263E+00_JPRB, 0.42399E+00_JPRB, 0.42490E+00_JPRB, 0.42404E+00_JPRB, 0.42384E+00_JPRB ,&
&  0.42081E+00_JPRB, 0.41631E+00_JPRB, 0.41342E+00_JPRB, 0.41460E+00_JPRB, 0.42329E+00_JPRB, 0.43473E+00_JPRB, 0.44185E+00_JPRB ,&
&  0.43801E+00_JPRB, 0.42622E+00_JPRB, 0.42337E+00_JPRB, 0.42571E+00_JPRB, 0.42791E+00_JPRB, 0.43210E+00_JPRB, 0.43827E+00_JPRB ,&
&  0.44389E+00_JPRB, 0.44774E+00_JPRB, 0.45190E+00_JPRB, 0.45361E+00_JPRB, 0.45400E+00_JPRB, 0.44716E+00_JPRB, 0.43624E+00_JPRB ,&
&  0.42584E+00_JPRB, 0.41845E+00_JPRB, 0.41407E+00_JPRB, 0.41062E+00_JPRB, 0.40565E+00_JPRB, 0.40053E+00_JPRB, 0.39387E+00_JPRB ,&
&  0.39048E+00_JPRB, 0.39005E+00_JPRB, 0.39081E+00_JPRB, 0.38901E+00_JPRB, 0.38671E+00_JPRB, 0.38212E+00_JPRB, 0.37851E+00_JPRB ,&
&  0.37750E+00_JPRB, 0.37932E+00_JPRB, 0.38129E+00_JPRB, 0.38116E+00_JPRB, 0.37853E+00_JPRB, 0.37683E+00_JPRB, 0.37865E+00_JPRB ,&
&  0.38473E+00_JPRB, 0.38467E+00_JPRB, 0.38369E+00_JPRB, 0.38422E+00_JPRB, 0.38614E+00_JPRB, 0.38799E+00_JPRB, 0.38753E+00_JPRB ,&
&  0.38710E+00_JPRB, 0.38740E+00_JPRB, 0.38808E+00_JPRB, 0.39039E+00_JPRB, 0.39080E+00_JPRB, 0.38900E+00_JPRB, 0.38377E+00_JPRB ,&
&  0.37559E+00_JPRB, 0.36511E+00_JPRB, 0.36072E+00_JPRB, 0.35760E+00_JPRB, 0.35456E+00_JPRB, 0.35370E+00_JPRB, 0.35478E+00_JPRB ,&
&  0.35323E+00_JPRB, 0.34702E+00_JPRB, 0.34407E+00_JPRB, 0.34415E+00_JPRB, 0.34241E+00_JPRB, 0.34384E+00_JPRB, 0.34678E+00_JPRB ,&
&  0.35602E+00_JPRB, 0.36100E+00_JPRB, 0.36248E+00_JPRB, 0.36044E+00_JPRB, 0.36087E+00_JPRB, 0.36259E+00_JPRB, 0.36668E+00_JPRB ,&
&  0.37292E+00_JPRB, 0.37958E+00_JPRB, 0.38628E+00_JPRB, 0.39366E+00_JPRB, 0.40320E+00_JPRB, 0.41258E+00_JPRB, 0.41604E+00_JPRB ,&
&  0.41597E+00_JPRB, 0.41593E+00_JPRB, 0.41678E+00_JPRB, 0.41802E+00_JPRB, 0.42081E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.44712E+00_JPRB, 0.44759E+00_JPRB, 0.44659E+00_JPRB, 0.44573E+00_JPRB, 0.44473E+00_JPRB, 0.44382E+00_JPRB, 0.44207E+00_JPRB ,&
&  0.43732E+00_JPRB, 0.43120E+00_JPRB, 0.42767E+00_JPRB, 0.42601E+00_JPRB, 0.43155E+00_JPRB, 0.43986E+00_JPRB, 0.44730E+00_JPRB ,&
&  0.44675E+00_JPRB, 0.43816E+00_JPRB, 0.43571E+00_JPRB, 0.43785E+00_JPRB, 0.44054E+00_JPRB, 0.44346E+00_JPRB, 0.44736E+00_JPRB ,&
&  0.45161E+00_JPRB, 0.45597E+00_JPRB, 0.45784E+00_JPRB, 0.45558E+00_JPRB, 0.44947E+00_JPRB, 0.43618E+00_JPRB, 0.42443E+00_JPRB ,&
&  0.41678E+00_JPRB, 0.41094E+00_JPRB, 0.40767E+00_JPRB, 0.40668E+00_JPRB, 0.40452E+00_JPRB, 0.40070E+00_JPRB, 0.39526E+00_JPRB ,&
&  0.39048E+00_JPRB, 0.38816E+00_JPRB, 0.39012E+00_JPRB, 0.39165E+00_JPRB, 0.39255E+00_JPRB, 0.39255E+00_JPRB, 0.39009E+00_JPRB ,&
&  0.38900E+00_JPRB, 0.38908E+00_JPRB, 0.38940E+00_JPRB, 0.39162E+00_JPRB, 0.39000E+00_JPRB, 0.38861E+00_JPRB, 0.39103E+00_JPRB ,&
&  0.39568E+00_JPRB, 0.39956E+00_JPRB, 0.40295E+00_JPRB, 0.40534E+00_JPRB, 0.40618E+00_JPRB, 0.40812E+00_JPRB, 0.40954E+00_JPRB ,&
&  0.40891E+00_JPRB, 0.40577E+00_JPRB, 0.40285E+00_JPRB, 0.40382E+00_JPRB, 0.40655E+00_JPRB, 0.40954E+00_JPRB, 0.41330E+00_JPRB ,&
&  0.41411E+00_JPRB, 0.40911E+00_JPRB, 0.40400E+00_JPRB, 0.39939E+00_JPRB, 0.39511E+00_JPRB, 0.39166E+00_JPRB, 0.38984E+00_JPRB ,&
&  0.38915E+00_JPRB, 0.38606E+00_JPRB, 0.38017E+00_JPRB, 0.37524E+00_JPRB, 0.37411E+00_JPRB, 0.37250E+00_JPRB, 0.36818E+00_JPRB ,&
&  0.36809E+00_JPRB, 0.37013E+00_JPRB, 0.36888E+00_JPRB, 0.36701E+00_JPRB, 0.36902E+00_JPRB, 0.37411E+00_JPRB, 0.38441E+00_JPRB ,&
&  0.39825E+00_JPRB, 0.41428E+00_JPRB, 0.42914E+00_JPRB, 0.44228E+00_JPRB, 0.45020E+00_JPRB, 0.45213E+00_JPRB, 0.45036E+00_JPRB ,&
&  0.44948E+00_JPRB, 0.44985E+00_JPRB, 0.44850E+00_JPRB, 0.44648E+00_JPRB, 0.44626E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.47243E+00_JPRB, 0.47084E+00_JPRB, 0.46940E+00_JPRB, 0.46776E+00_JPRB, 0.46538E+00_JPRB, 0.46335E+00_JPRB, 0.45941E+00_JPRB ,&
&  0.45497E+00_JPRB, 0.45127E+00_JPRB, 0.44979E+00_JPRB, 0.44959E+00_JPRB, 0.45125E+00_JPRB, 0.45533E+00_JPRB, 0.46198E+00_JPRB ,&
&  0.46224E+00_JPRB, 0.45605E+00_JPRB, 0.45340E+00_JPRB, 0.45373E+00_JPRB, 0.45631E+00_JPRB, 0.45817E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46316E+00_JPRB, 0.46389E+00_JPRB, 0.46346E+00_JPRB, 0.45709E+00_JPRB, 0.44095E+00_JPRB, 0.43111E+00_JPRB, 0.42224E+00_JPRB ,&
&  0.41710E+00_JPRB, 0.41246E+00_JPRB, 0.41017E+00_JPRB, 0.40890E+00_JPRB, 0.40788E+00_JPRB, 0.40598E+00_JPRB, 0.40575E+00_JPRB ,&
&  0.40369E+00_JPRB, 0.40095E+00_JPRB, 0.39934E+00_JPRB, 0.39979E+00_JPRB, 0.40144E+00_JPRB, 0.40314E+00_JPRB, 0.40331E+00_JPRB ,&
&  0.40462E+00_JPRB, 0.40869E+00_JPRB, 0.41018E+00_JPRB, 0.41079E+00_JPRB, 0.41099E+00_JPRB, 0.41080E+00_JPRB, 0.41113E+00_JPRB ,&
&  0.41350E+00_JPRB, 0.41846E+00_JPRB, 0.42253E+00_JPRB, 0.42650E+00_JPRB, 0.42806E+00_JPRB, 0.42882E+00_JPRB, 0.42890E+00_JPRB ,&
&  0.42612E+00_JPRB, 0.42244E+00_JPRB, 0.42146E+00_JPRB, 0.42389E+00_JPRB, 0.42859E+00_JPRB, 0.43482E+00_JPRB, 0.43913E+00_JPRB ,&
&  0.44246E+00_JPRB, 0.44358E+00_JPRB, 0.44128E+00_JPRB, 0.43984E+00_JPRB, 0.43694E+00_JPRB, 0.43221E+00_JPRB, 0.42943E+00_JPRB ,&
&  0.42677E+00_JPRB, 0.42455E+00_JPRB, 0.42286E+00_JPRB, 0.41423E+00_JPRB, 0.40650E+00_JPRB, 0.40143E+00_JPRB, 0.39791E+00_JPRB ,&
&  0.39648E+00_JPRB, 0.39549E+00_JPRB, 0.39509E+00_JPRB, 0.39635E+00_JPRB, 0.39816E+00_JPRB, 0.40380E+00_JPRB, 0.41560E+00_JPRB ,&
&  0.43369E+00_JPRB, 0.45080E+00_JPRB, 0.46031E+00_JPRB, 0.46738E+00_JPRB, 0.47209E+00_JPRB, 0.47467E+00_JPRB, 0.47718E+00_JPRB ,&
&  0.48086E+00_JPRB, 0.48200E+00_JPRB, 0.47975E+00_JPRB, 0.47672E+00_JPRB, 0.47486E+00_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.49543E+00_JPRB, 0.48776E+00_JPRB, 0.48220E+00_JPRB, 0.47788E+00_JPRB, 0.47457E+00_JPRB, 0.47385E+00_JPRB, 0.47265E+00_JPRB ,&
&  0.47119E+00_JPRB, 0.46918E+00_JPRB, 0.46844E+00_JPRB, 0.46601E+00_JPRB, 0.46357E+00_JPRB, 0.46649E+00_JPRB, 0.47193E+00_JPRB ,&
&  0.47083E+00_JPRB, 0.46476E+00_JPRB, 0.45997E+00_JPRB, 0.46021E+00_JPRB, 0.46387E+00_JPRB, 0.46664E+00_JPRB, 0.46991E+00_JPRB ,&
&  0.46991E+00_JPRB, 0.46943E+00_JPRB, 0.46836E+00_JPRB, 0.45352E+00_JPRB, 0.43884E+00_JPRB, 0.43165E+00_JPRB, 0.42746E+00_JPRB ,&
&  0.42652E+00_JPRB, 0.42985E+00_JPRB, 0.41528E+00_JPRB, 0.42399E+00_JPRB, 0.41966E+00_JPRB, 0.41340E+00_JPRB, 0.41012E+00_JPRB ,&
&  0.40807E+00_JPRB, 0.40548E+00_JPRB, 0.40374E+00_JPRB, 0.40569E+00_JPRB, 0.40849E+00_JPRB, 0.40962E+00_JPRB, 0.41170E+00_JPRB ,&
&  0.41177E+00_JPRB, 0.41456E+00_JPRB, 0.41686E+00_JPRB, 0.42201E+00_JPRB, 0.42607E+00_JPRB, 0.42868E+00_JPRB, 0.43193E+00_JPRB ,&
&  0.43632E+00_JPRB, 0.44043E+00_JPRB, 0.44433E+00_JPRB, 0.44761E+00_JPRB, 0.44892E+00_JPRB, 0.44874E+00_JPRB, 0.44481E+00_JPRB ,&
&  0.44170E+00_JPRB, 0.44159E+00_JPRB, 0.44370E+00_JPRB, 0.44776E+00_JPRB, 0.45212E+00_JPRB, 0.45473E+00_JPRB, 0.45698E+00_JPRB ,&
&  0.45929E+00_JPRB, 0.46286E+00_JPRB, 0.46324E+00_JPRB, 0.46160E+00_JPRB, 0.45935E+00_JPRB, 0.45659E+00_JPRB, 0.45201E+00_JPRB ,&
&  0.44626E+00_JPRB, 0.44196E+00_JPRB, 0.43764E+00_JPRB, 0.42725E+00_JPRB, 0.41917E+00_JPRB, 0.41836E+00_JPRB, 0.41969E+00_JPRB ,&
&  0.41910E+00_JPRB, 0.41755E+00_JPRB, 0.41868E+00_JPRB, 0.42408E+00_JPRB, 0.42329E+00_JPRB, 0.42866E+00_JPRB, 0.44120E+00_JPRB ,&
&  0.45445E+00_JPRB, 0.46645E+00_JPRB, 0.47616E+00_JPRB, 0.48645E+00_JPRB, 0.49685E+00_JPRB, 0.50493E+00_JPRB, 0.51215E+00_JPRB ,&
&  0.51587E+00_JPRB, 0.51468E+00_JPRB, 0.50958E+00_JPRB, 0.50618E+00_JPRB, 0.50308E+00_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.53626E+00_JPRB, 0.52400E+00_JPRB, 0.51471E+00_JPRB, 0.50804E+00_JPRB, 0.50417E+00_JPRB, 0.50413E+00_JPRB, 0.50259E+00_JPRB ,&
&  0.50109E+00_JPRB, 0.50015E+00_JPRB, 0.49493E+00_JPRB, 0.48918E+00_JPRB, 0.48901E+00_JPRB, 0.49467E+00_JPRB, 0.49947E+00_JPRB ,&
&  0.49733E+00_JPRB, 0.48789E+00_JPRB, 0.47893E+00_JPRB, 0.47745E+00_JPRB, 0.48081E+00_JPRB, 0.48358E+00_JPRB, 0.48349E+00_JPRB ,&
&  0.48522E+00_JPRB, 0.48619E+00_JPRB, 0.47806E+00_JPRB, 0.46184E+00_JPRB, 0.45409E+00_JPRB, 0.44965E+00_JPRB, 0.45368E+00_JPRB ,&
&  0.45948E+00_JPRB, 0.45045E+00_JPRB, 0.46399E+00_JPRB, 0.46089E+00_JPRB, 0.45915E+00_JPRB, 0.45465E+00_JPRB, 0.44765E+00_JPRB ,&
&  0.44227E+00_JPRB, 0.43697E+00_JPRB, 0.43381E+00_JPRB, 0.43254E+00_JPRB, 0.43325E+00_JPRB, 0.43477E+00_JPRB, 0.43964E+00_JPRB ,&
&  0.44128E+00_JPRB, 0.44005E+00_JPRB, 0.44286E+00_JPRB, 0.45008E+00_JPRB, 0.45895E+00_JPRB, 0.46400E+00_JPRB, 0.46855E+00_JPRB ,&
&  0.47227E+00_JPRB, 0.47442E+00_JPRB, 0.47743E+00_JPRB, 0.47855E+00_JPRB, 0.47530E+00_JPRB, 0.47478E+00_JPRB, 0.47494E+00_JPRB ,&
&  0.47709E+00_JPRB, 0.47977E+00_JPRB, 0.48192E+00_JPRB, 0.48359E+00_JPRB, 0.48464E+00_JPRB, 0.48297E+00_JPRB, 0.48161E+00_JPRB ,&
&  0.48281E+00_JPRB, 0.48586E+00_JPRB, 0.48941E+00_JPRB, 0.49131E+00_JPRB, 0.49021E+00_JPRB, 0.48370E+00_JPRB, 0.47611E+00_JPRB ,&
&  0.46822E+00_JPRB, 0.46005E+00_JPRB, 0.45271E+00_JPRB, 0.44840E+00_JPRB, 0.44523E+00_JPRB, 0.44346E+00_JPRB, 0.44255E+00_JPRB ,&
&  0.44238E+00_JPRB, 0.44253E+00_JPRB, 0.44557E+00_JPRB, 0.45127E+00_JPRB, 0.45369E+00_JPRB, 0.46105E+00_JPRB, 0.47087E+00_JPRB ,&
&  0.48119E+00_JPRB, 0.49106E+00_JPRB, 0.50297E+00_JPRB, 0.52017E+00_JPRB, 0.53938E+00_JPRB, 0.55377E+00_JPRB, 0.56496E+00_JPRB ,&
&  0.57223E+00_JPRB, 0.57159E+00_JPRB, 0.56459E+00_JPRB, 0.55684E+00_JPRB, 0.54793E+00_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.57690E+00_JPRB, 0.55874E+00_JPRB, 0.54413E+00_JPRB, 0.53408E+00_JPRB, 0.52877E+00_JPRB, 0.52857E+00_JPRB, 0.52664E+00_JPRB ,&
&  0.52565E+00_JPRB, 0.52318E+00_JPRB, 0.52161E+00_JPRB, 0.52421E+00_JPRB, 0.52574E+00_JPRB, 0.52673E+00_JPRB, 0.52881E+00_JPRB ,&
&  0.52438E+00_JPRB, 0.51282E+00_JPRB, 0.50347E+00_JPRB, 0.50130E+00_JPRB, 0.50285E+00_JPRB, 0.49939E+00_JPRB, 0.49824E+00_JPRB ,&
&  0.50125E+00_JPRB, 0.49350E+00_JPRB, 0.48063E+00_JPRB, 0.47238E+00_JPRB, 0.47284E+00_JPRB, 0.47060E+00_JPRB, 0.46744E+00_JPRB ,&
&  0.48569E+00_JPRB, 0.43607E+00_JPRB, 0.47945E+00_JPRB, 0.48369E+00_JPRB, 0.48664E+00_JPRB, 0.48803E+00_JPRB, 0.48620E+00_JPRB ,&
&  0.48190E+00_JPRB, 0.47684E+00_JPRB, 0.47144E+00_JPRB, 0.46674E+00_JPRB, 0.46446E+00_JPRB, 0.46117E+00_JPRB, 0.45837E+00_JPRB ,&
&  0.46051E+00_JPRB, 0.46354E+00_JPRB, 0.46980E+00_JPRB, 0.47646E+00_JPRB, 0.48458E+00_JPRB, 0.48769E+00_JPRB, 0.49041E+00_JPRB ,&
&  0.49336E+00_JPRB, 0.49521E+00_JPRB, 0.49546E+00_JPRB, 0.49469E+00_JPRB, 0.49346E+00_JPRB, 0.49833E+00_JPRB, 0.50453E+00_JPRB ,&
&  0.50894E+00_JPRB, 0.51195E+00_JPRB, 0.51564E+00_JPRB, 0.51581E+00_JPRB, 0.51563E+00_JPRB, 0.51288E+00_JPRB, 0.50833E+00_JPRB ,&
&  0.50448E+00_JPRB, 0.50330E+00_JPRB, 0.50279E+00_JPRB, 0.49823E+00_JPRB, 0.49163E+00_JPRB, 0.48568E+00_JPRB, 0.47868E+00_JPRB ,&
&  0.46969E+00_JPRB, 0.46252E+00_JPRB, 0.45899E+00_JPRB, 0.45835E+00_JPRB, 0.45984E+00_JPRB, 0.46091E+00_JPRB, 0.46195E+00_JPRB ,&
&  0.46502E+00_JPRB, 0.46604E+00_JPRB, 0.46787E+00_JPRB, 0.47230E+00_JPRB, 0.47593E+00_JPRB, 0.48187E+00_JPRB, 0.48982E+00_JPRB ,&
&  0.50051E+00_JPRB, 0.51252E+00_JPRB, 0.52388E+00_JPRB, 0.54243E+00_JPRB, 0.56816E+00_JPRB, 0.58715E+00_JPRB, 0.59869E+00_JPRB ,&
&  0.60906E+00_JPRB, 0.61127E+00_JPRB, 0.60599E+00_JPRB, 0.59983E+00_JPRB, 0.58971E+00_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.62750E+00_JPRB, 0.61392E+00_JPRB, 0.59557E+00_JPRB, 0.58267E+00_JPRB, 0.57629E+00_JPRB, 0.57406E+00_JPRB, 0.57142E+00_JPRB ,&
&  0.56712E+00_JPRB, 0.56763E+00_JPRB, 0.56851E+00_JPRB, 0.56936E+00_JPRB, 0.57024E+00_JPRB, 0.56826E+00_JPRB, 0.56199E+00_JPRB ,&
&  0.55453E+00_JPRB, 0.54418E+00_JPRB, 0.54289E+00_JPRB, 0.54261E+00_JPRB, 0.53669E+00_JPRB, 0.53264E+00_JPRB, 0.53242E+00_JPRB ,&
&  0.52622E+00_JPRB, 0.51681E+00_JPRB, 0.49967E+00_JPRB, 0.49412E+00_JPRB, 0.49661E+00_JPRB, 0.49674E+00_JPRB, 0.50244E+00_JPRB ,&
&  0.47974E+00_JPRB, 0.44996E+00_JPRB, 0.49607E+00_JPRB, 0.52218E+00_JPRB, 0.52987E+00_JPRB, 0.53734E+00_JPRB, 0.54633E+00_JPRB ,&
&  0.54139E+00_JPRB, 0.54215E+00_JPRB, 0.53077E+00_JPRB, 0.51212E+00_JPRB, 0.50010E+00_JPRB, 0.49754E+00_JPRB, 0.49952E+00_JPRB ,&
&  0.50751E+00_JPRB, 0.51368E+00_JPRB, 0.51656E+00_JPRB, 0.51794E+00_JPRB, 0.52022E+00_JPRB, 0.51808E+00_JPRB, 0.51666E+00_JPRB ,&
&  0.51852E+00_JPRB, 0.52212E+00_JPRB, 0.52401E+00_JPRB, 0.52809E+00_JPRB, 0.53617E+00_JPRB, 0.54847E+00_JPRB, 0.55962E+00_JPRB ,&
&  0.56753E+00_JPRB, 0.57149E+00_JPRB, 0.57658E+00_JPRB, 0.57815E+00_JPRB, 0.57569E+00_JPRB, 0.56717E+00_JPRB, 0.55681E+00_JPRB ,&
&  0.54549E+00_JPRB, 0.53665E+00_JPRB, 0.53031E+00_JPRB, 0.52613E+00_JPRB, 0.51945E+00_JPRB, 0.51293E+00_JPRB, 0.50486E+00_JPRB ,&
&  0.49738E+00_JPRB, 0.48946E+00_JPRB, 0.48596E+00_JPRB, 0.48508E+00_JPRB, 0.48784E+00_JPRB, 0.49186E+00_JPRB, 0.49720E+00_JPRB ,&
&  0.50110E+00_JPRB, 0.50008E+00_JPRB, 0.50028E+00_JPRB, 0.50476E+00_JPRB, 0.51004E+00_JPRB, 0.51894E+00_JPRB, 0.53056E+00_JPRB ,&
&  0.54451E+00_JPRB, 0.55761E+00_JPRB, 0.57181E+00_JPRB, 0.59977E+00_JPRB, 0.62745E+00_JPRB, 0.65015E+00_JPRB, 0.66192E+00_JPRB ,&
&  0.66756E+00_JPRB, 0.65880E+00_JPRB, 0.65797E+00_JPRB, 0.64891E+00_JPRB, 0.63702E+00_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.62899E+00_JPRB, 0.62730E+00_JPRB, 0.61609E+00_JPRB, 0.60934E+00_JPRB, 0.60439E+00_JPRB, 0.59925E+00_JPRB, 0.59084E+00_JPRB ,&
&  0.58573E+00_JPRB, 0.58512E+00_JPRB, 0.58454E+00_JPRB, 0.58551E+00_JPRB, 0.58720E+00_JPRB, 0.58374E+00_JPRB, 0.57750E+00_JPRB ,&
&  0.57061E+00_JPRB, 0.56374E+00_JPRB, 0.56540E+00_JPRB, 0.56524E+00_JPRB, 0.56076E+00_JPRB, 0.55702E+00_JPRB, 0.55182E+00_JPRB ,&
&  0.53803E+00_JPRB, 0.52474E+00_JPRB, 0.51414E+00_JPRB, 0.51697E+00_JPRB, 0.52338E+00_JPRB, 0.51865E+00_JPRB, 0.53090E+00_JPRB ,&
&  0.56994E+00_JPRB, 0.48794E+00_JPRB, 0.51258E+00_JPRB, 0.55734E+00_JPRB, 0.57468E+00_JPRB, 0.59344E+00_JPRB, 0.60531E+00_JPRB ,&
&  0.61167E+00_JPRB, 0.61382E+00_JPRB, 0.59421E+00_JPRB, 0.57217E+00_JPRB, 0.55570E+00_JPRB, 0.54346E+00_JPRB, 0.53531E+00_JPRB ,&
&  0.53429E+00_JPRB, 0.54110E+00_JPRB, 0.54652E+00_JPRB, 0.54608E+00_JPRB, 0.54524E+00_JPRB, 0.54194E+00_JPRB, 0.53983E+00_JPRB ,&
&  0.53909E+00_JPRB, 0.54109E+00_JPRB, 0.54498E+00_JPRB, 0.55411E+00_JPRB, 0.56304E+00_JPRB, 0.57355E+00_JPRB, 0.58752E+00_JPRB ,&
&  0.60280E+00_JPRB, 0.61533E+00_JPRB, 0.62366E+00_JPRB, 0.62453E+00_JPRB, 0.61980E+00_JPRB, 0.61098E+00_JPRB, 0.59850E+00_JPRB ,&
&  0.58322E+00_JPRB, 0.56752E+00_JPRB, 0.55798E+00_JPRB, 0.55346E+00_JPRB, 0.54744E+00_JPRB, 0.54118E+00_JPRB, 0.53580E+00_JPRB ,&
&  0.53135E+00_JPRB, 0.52619E+00_JPRB, 0.52714E+00_JPRB, 0.52867E+00_JPRB, 0.52996E+00_JPRB, 0.52719E+00_JPRB, 0.52396E+00_JPRB ,&
&  0.52122E+00_JPRB, 0.51953E+00_JPRB, 0.52169E+00_JPRB, 0.52731E+00_JPRB, 0.53598E+00_JPRB, 0.55017E+00_JPRB, 0.56807E+00_JPRB ,&
&  0.58131E+00_JPRB, 0.59273E+00_JPRB, 0.60627E+00_JPRB, 0.63526E+00_JPRB, 0.65460E+00_JPRB, 0.66828E+00_JPRB, 0.67380E+00_JPRB ,&
&  0.67231E+00_JPRB, 0.67360E+00_JPRB, 0.53458E+00_JPRB, 0.59894E+00_JPRB, 0.61828E+00_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.64367E+00_JPRB, 0.64125E+00_JPRB, 0.63481E+00_JPRB, 0.63223E+00_JPRB, 0.63157E+00_JPRB, 0.61973E+00_JPRB, 0.61019E+00_JPRB ,&
&  0.60559E+00_JPRB, 0.60727E+00_JPRB, 0.61159E+00_JPRB, 0.61397E+00_JPRB, 0.61051E+00_JPRB, 0.60253E+00_JPRB, 0.59854E+00_JPRB ,&
&  0.59124E+00_JPRB, 0.58440E+00_JPRB, 0.58326E+00_JPRB, 0.58333E+00_JPRB, 0.57849E+00_JPRB, 0.57112E+00_JPRB, 0.56502E+00_JPRB ,&
&  0.55263E+00_JPRB, 0.54521E+00_JPRB, 0.54389E+00_JPRB, 0.54693E+00_JPRB, 0.55003E+00_JPRB, 0.54860E+00_JPRB, 0.56322E+00_JPRB ,&
&  0.59400E+00_JPRB, 0.51208E+00_JPRB, 0.51814E+00_JPRB, 0.59930E+00_JPRB, 0.64488E+00_JPRB, 0.65900E+00_JPRB, 0.67515E+00_JPRB ,&
&  0.69087E+00_JPRB, 0.68479E+00_JPRB, 0.67171E+00_JPRB, 0.64303E+00_JPRB, 0.62194E+00_JPRB, 0.61092E+00_JPRB, 0.60269E+00_JPRB ,&
&  0.59917E+00_JPRB, 0.59687E+00_JPRB, 0.59897E+00_JPRB, 0.59523E+00_JPRB, 0.59425E+00_JPRB, 0.58690E+00_JPRB, 0.58490E+00_JPRB ,&
&  0.58570E+00_JPRB, 0.59045E+00_JPRB, 0.59546E+00_JPRB, 0.59843E+00_JPRB, 0.60699E+00_JPRB, 0.62068E+00_JPRB, 0.64071E+00_JPRB ,&
&  0.66238E+00_JPRB, 0.68275E+00_JPRB, 0.69093E+00_JPRB, 0.68741E+00_JPRB, 0.67738E+00_JPRB, 0.66558E+00_JPRB, 0.64792E+00_JPRB ,&
&  0.62653E+00_JPRB, 0.60549E+00_JPRB, 0.59125E+00_JPRB, 0.58390E+00_JPRB, 0.58008E+00_JPRB, 0.57795E+00_JPRB, 0.57549E+00_JPRB ,&
&  0.57310E+00_JPRB, 0.57373E+00_JPRB, 0.57582E+00_JPRB, 0.57653E+00_JPRB, 0.57692E+00_JPRB, 0.57110E+00_JPRB, 0.56050E+00_JPRB ,&
&  0.55318E+00_JPRB, 0.55232E+00_JPRB, 0.56031E+00_JPRB, 0.56677E+00_JPRB, 0.57500E+00_JPRB, 0.59164E+00_JPRB, 0.61022E+00_JPRB ,&
&  0.62127E+00_JPRB, 0.63460E+00_JPRB, 0.65870E+00_JPRB, 0.64864E+00_JPRB, 0.68299E+00_JPRB, 0.68321E+00_JPRB, 0.68469E+00_JPRB ,&
&  0.67484E+00_JPRB, 0.66080E+00_JPRB, 0.65549E+00_JPRB, 0.56433E+00_JPRB, 0.62495E+00_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.65480E+00_JPRB, 0.65304E+00_JPRB, 0.64770E+00_JPRB, 0.64475E+00_JPRB, 0.64054E+00_JPRB, 0.62991E+00_JPRB, 0.62230E+00_JPRB ,&
&  0.62107E+00_JPRB, 0.61864E+00_JPRB, 0.61676E+00_JPRB, 0.61191E+00_JPRB, 0.60400E+00_JPRB, 0.59895E+00_JPRB, 0.60026E+00_JPRB ,&
&  0.59878E+00_JPRB, 0.59877E+00_JPRB, 0.59906E+00_JPRB, 0.59541E+00_JPRB, 0.59011E+00_JPRB, 0.58175E+00_JPRB, 0.56849E+00_JPRB ,&
&  0.55278E+00_JPRB, 0.54865E+00_JPRB, 0.55737E+00_JPRB, 0.56401E+00_JPRB, 0.56780E+00_JPRB, 0.56989E+00_JPRB, 0.57938E+00_JPRB ,&
&  0.60173E+00_JPRB, 0.55243E+00_JPRB, 0.60754E+00_JPRB, 0.69865E+00_JPRB, 0.75474E+00_JPRB, 0.75665E+00_JPRB, 0.76075E+00_JPRB ,&
&  0.74665E+00_JPRB, 0.72708E+00_JPRB, 0.70208E+00_JPRB, 0.67130E+00_JPRB, 0.64466E+00_JPRB, 0.63007E+00_JPRB, 0.62754E+00_JPRB ,&
&  0.62749E+00_JPRB, 0.62691E+00_JPRB, 0.62945E+00_JPRB, 0.62544E+00_JPRB, 0.62118E+00_JPRB, 0.61736E+00_JPRB, 0.61879E+00_JPRB ,&
&  0.62096E+00_JPRB, 0.62342E+00_JPRB, 0.62422E+00_JPRB, 0.62609E+00_JPRB, 0.62963E+00_JPRB, 0.63972E+00_JPRB, 0.65866E+00_JPRB ,&
&  0.68120E+00_JPRB, 0.70592E+00_JPRB, 0.71433E+00_JPRB, 0.70719E+00_JPRB, 0.69952E+00_JPRB, 0.69595E+00_JPRB, 0.67665E+00_JPRB ,&
&  0.65238E+00_JPRB, 0.63611E+00_JPRB, 0.62247E+00_JPRB, 0.61128E+00_JPRB, 0.60455E+00_JPRB, 0.60308E+00_JPRB, 0.60265E+00_JPRB ,&
&  0.60249E+00_JPRB, 0.60100E+00_JPRB, 0.59828E+00_JPRB, 0.59735E+00_JPRB, 0.59688E+00_JPRB, 0.59126E+00_JPRB, 0.58475E+00_JPRB ,&
&  0.57752E+00_JPRB, 0.58038E+00_JPRB, 0.59211E+00_JPRB, 0.59874E+00_JPRB, 0.60481E+00_JPRB, 0.61734E+00_JPRB, 0.63305E+00_JPRB ,&
&  0.64182E+00_JPRB, 0.65641E+00_JPRB, 0.66994E+00_JPRB, 0.70799E+00_JPRB, 0.71006E+00_JPRB, 0.69416E+00_JPRB, 0.68434E+00_JPRB ,&
&  0.66522E+00_JPRB, 0.64717E+00_JPRB, 0.63708E+00_JPRB, 0.65042E+00_JPRB, 0.61898E+00_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.73571E+00_JPRB, 0.70301E+00_JPRB, 0.67071E+00_JPRB, 0.64991E+00_JPRB, 0.64130E+00_JPRB, 0.63849E+00_JPRB, 0.63712E+00_JPRB ,&
&  0.63371E+00_JPRB, 0.62945E+00_JPRB, 0.61517E+00_JPRB, 0.60535E+00_JPRB, 0.59909E+00_JPRB, 0.59831E+00_JPRB, 0.59755E+00_JPRB ,&
&  0.59940E+00_JPRB, 0.60001E+00_JPRB, 0.59668E+00_JPRB, 0.59627E+00_JPRB, 0.59822E+00_JPRB, 0.59417E+00_JPRB, 0.57939E+00_JPRB ,&
&  0.56489E+00_JPRB, 0.56748E+00_JPRB, 0.57201E+00_JPRB, 0.57433E+00_JPRB, 0.57944E+00_JPRB, 0.58854E+00_JPRB, 0.60331E+00_JPRB ,&
&  0.61384E+00_JPRB, 0.57933E+00_JPRB, 0.71719E+00_JPRB, 0.81426E+00_JPRB, 0.86340E+00_JPRB, 0.89616E+00_JPRB, 0.87538E+00_JPRB ,&
&  0.84274E+00_JPRB, 0.78889E+00_JPRB, 0.74285E+00_JPRB, 0.69700E+00_JPRB, 0.66960E+00_JPRB, 0.65225E+00_JPRB, 0.64207E+00_JPRB ,&
&  0.63839E+00_JPRB, 0.63723E+00_JPRB, 0.63573E+00_JPRB, 0.63292E+00_JPRB, 0.63155E+00_JPRB, 0.63141E+00_JPRB, 0.63235E+00_JPRB ,&
&  0.63486E+00_JPRB, 0.63983E+00_JPRB, 0.64162E+00_JPRB, 0.64126E+00_JPRB, 0.64019E+00_JPRB, 0.66570E+00_JPRB, 0.70079E+00_JPRB ,&
&  0.73144E+00_JPRB, 0.75210E+00_JPRB, 0.74692E+00_JPRB, 0.74118E+00_JPRB, 0.73620E+00_JPRB, 0.72269E+00_JPRB, 0.70720E+00_JPRB ,&
&  0.69054E+00_JPRB, 0.67090E+00_JPRB, 0.64947E+00_JPRB, 0.63385E+00_JPRB, 0.62628E+00_JPRB, 0.62406E+00_JPRB, 0.62314E+00_JPRB ,&
&  0.62386E+00_JPRB, 0.62699E+00_JPRB, 0.62532E+00_JPRB, 0.62553E+00_JPRB, 0.62262E+00_JPRB, 0.61974E+00_JPRB, 0.62025E+00_JPRB ,&
&  0.61953E+00_JPRB, 0.61854E+00_JPRB, 0.62987E+00_JPRB, 0.63800E+00_JPRB, 0.64543E+00_JPRB, 0.65472E+00_JPRB, 0.66227E+00_JPRB ,&
&  0.67123E+00_JPRB, 0.69043E+00_JPRB, 0.66878E+00_JPRB, 0.67494E+00_JPRB, 0.74126E+00_JPRB, 0.71102E+00_JPRB, 0.67472E+00_JPRB ,&
&  0.64567E+00_JPRB, 0.63572E+00_JPRB, 0.63904E+00_JPRB, 0.67010E+00_JPRB, 0.72906E+00_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.69963E+00_JPRB, 0.66055E+00_JPRB, 0.63986E+00_JPRB, 0.63798E+00_JPRB, 0.64433E+00_JPRB, 0.64193E+00_JPRB, 0.64540E+00_JPRB ,&
&  0.63990E+00_JPRB, 0.62885E+00_JPRB, 0.61624E+00_JPRB, 0.60405E+00_JPRB, 0.59873E+00_JPRB, 0.59477E+00_JPRB, 0.59218E+00_JPRB ,&
&  0.59097E+00_JPRB, 0.59014E+00_JPRB, 0.58646E+00_JPRB, 0.59065E+00_JPRB, 0.59322E+00_JPRB, 0.59254E+00_JPRB, 0.57292E+00_JPRB ,&
&  0.56470E+00_JPRB, 0.56669E+00_JPRB, 0.57082E+00_JPRB, 0.57481E+00_JPRB, 0.58141E+00_JPRB, 0.59318E+00_JPRB, 0.60355E+00_JPRB ,&
&  0.60824E+00_JPRB, 0.62690E+00_JPRB, 0.79779E+00_JPRB, 0.92787E+00_JPRB, 0.95999E+00_JPRB, 0.95466E+00_JPRB, 0.91640E+00_JPRB ,&
&  0.88097E+00_JPRB, 0.84952E+00_JPRB, 0.81823E+00_JPRB, 0.75476E+00_JPRB, 0.69654E+00_JPRB, 0.66760E+00_JPRB, 0.65559E+00_JPRB ,&
&  0.65124E+00_JPRB, 0.65076E+00_JPRB, 0.64917E+00_JPRB, 0.64675E+00_JPRB, 0.64360E+00_JPRB, 0.64243E+00_JPRB, 0.64526E+00_JPRB ,&
&  0.64903E+00_JPRB, 0.64604E+00_JPRB, 0.64539E+00_JPRB, 0.63488E+00_JPRB, 0.66508E+00_JPRB, 0.71183E+00_JPRB, 0.76790E+00_JPRB ,&
&  0.78648E+00_JPRB, 0.80120E+00_JPRB, 0.78953E+00_JPRB, 0.76581E+00_JPRB, 0.74150E+00_JPRB, 0.72013E+00_JPRB, 0.70401E+00_JPRB ,&
&  0.69030E+00_JPRB, 0.67701E+00_JPRB, 0.66505E+00_JPRB, 0.66546E+00_JPRB, 0.67383E+00_JPRB, 0.67005E+00_JPRB, 0.66663E+00_JPRB ,&
&  0.66500E+00_JPRB, 0.66604E+00_JPRB, 0.66424E+00_JPRB, 0.65455E+00_JPRB, 0.64194E+00_JPRB, 0.63571E+00_JPRB, 0.63449E+00_JPRB ,&
&  0.64148E+00_JPRB, 0.64486E+00_JPRB, 0.66722E+00_JPRB, 0.67391E+00_JPRB, 0.67936E+00_JPRB, 0.68129E+00_JPRB, 0.68776E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.69489E+00_JPRB, 0.69268E+00_JPRB, 0.69852E+00_JPRB, 0.69192E+00_JPRB, 0.71328E+00_JPRB, 0.65937E+00_JPRB ,&
&  0.62949E+00_JPRB, 0.62344E+00_JPRB, 0.63062E+00_JPRB, 0.65754E+00_JPRB, 0.70788E+00_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.66494E+00_JPRB, 0.64567E+00_JPRB, 0.63900E+00_JPRB, 0.64166E+00_JPRB, 0.63848E+00_JPRB, 0.64154E+00_JPRB, 0.64668E+00_JPRB ,&
&  0.64589E+00_JPRB, 0.63390E+00_JPRB, 0.62201E+00_JPRB, 0.60818E+00_JPRB, 0.60359E+00_JPRB, 0.59637E+00_JPRB, 0.59498E+00_JPRB ,&
&  0.58911E+00_JPRB, 0.58694E+00_JPRB, 0.58225E+00_JPRB, 0.58161E+00_JPRB, 0.58247E+00_JPRB, 0.57893E+00_JPRB, 0.56166E+00_JPRB ,&
&  0.55896E+00_JPRB, 0.55833E+00_JPRB, 0.56496E+00_JPRB, 0.57516E+00_JPRB, 0.58715E+00_JPRB, 0.59379E+00_JPRB, 0.59351E+00_JPRB ,&
&  0.61206E+00_JPRB, 0.63234E+00_JPRB, 0.94910E+00_JPRB, 0.10399E+01_JPRB, 0.10676E+01_JPRB, 0.10010E+01_JPRB, 0.94169E+00_JPRB ,&
&  0.89693E+00_JPRB, 0.87166E+00_JPRB, 0.86270E+00_JPRB, 0.81255E+00_JPRB, 0.74682E+00_JPRB, 0.69720E+00_JPRB, 0.68347E+00_JPRB ,&
&  0.67592E+00_JPRB, 0.66781E+00_JPRB, 0.66499E+00_JPRB, 0.66478E+00_JPRB, 0.66018E+00_JPRB, 0.65717E+00_JPRB, 0.66153E+00_JPRB ,&
&  0.66274E+00_JPRB, 0.66337E+00_JPRB, 0.65484E+00_JPRB, 0.65506E+00_JPRB, 0.66341E+00_JPRB, 0.63234E+00_JPRB, 0.71059E+00_JPRB ,&
&  0.89107E+00_JPRB, 0.88726E+00_JPRB, 0.83454E+00_JPRB, 0.77021E+00_JPRB, 0.73056E+00_JPRB, 0.71481E+00_JPRB, 0.70469E+00_JPRB ,&
&  0.69734E+00_JPRB, 0.68948E+00_JPRB, 0.68931E+00_JPRB, 0.70501E+00_JPRB, 0.71148E+00_JPRB, 0.70572E+00_JPRB, 0.69693E+00_JPRB ,&
&  0.69556E+00_JPRB, 0.69648E+00_JPRB, 0.68717E+00_JPRB, 0.66697E+00_JPRB, 0.65401E+00_JPRB, 0.65328E+00_JPRB, 0.65170E+00_JPRB ,&
&  0.65090E+00_JPRB, 0.65537E+00_JPRB, 0.66375E+00_JPRB, 0.65462E+00_JPRB, 0.67297E+00_JPRB, 0.70253E+00_JPRB, 0.70994E+00_JPRB ,&
&  0.69136E+00_JPRB, 0.67791E+00_JPRB, 0.69855E+00_JPRB, 0.68927E+00_JPRB, 0.68773E+00_JPRB, 0.70442E+00_JPRB, 0.64757E+00_JPRB ,&
&  0.61896E+00_JPRB, 0.61697E+00_JPRB, 0.63843E+00_JPRB, 0.66668E+00_JPRB, 0.70499E+00_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.64768E+00_JPRB, 0.63772E+00_JPRB, 0.63560E+00_JPRB, 0.63244E+00_JPRB, 0.62410E+00_JPRB, 0.61449E+00_JPRB, 0.61893E+00_JPRB ,&
&  0.62511E+00_JPRB, 0.62934E+00_JPRB, 0.62170E+00_JPRB, 0.60669E+00_JPRB, 0.60388E+00_JPRB, 0.59936E+00_JPRB, 0.59397E+00_JPRB ,&
&  0.58958E+00_JPRB, 0.58937E+00_JPRB, 0.58274E+00_JPRB, 0.57895E+00_JPRB, 0.58182E+00_JPRB, 0.58057E+00_JPRB, 0.57402E+00_JPRB ,&
&  0.56653E+00_JPRB, 0.56100E+00_JPRB, 0.56545E+00_JPRB, 0.57003E+00_JPRB, 0.58101E+00_JPRB, 0.58183E+00_JPRB, 0.57620E+00_JPRB ,&
&  0.60333E+00_JPRB, 0.53465E+00_JPRB, 0.10313E+01_JPRB, 0.12909E+01_JPRB, 0.12659E+01_JPRB, 0.10906E+01_JPRB, 0.95916E+00_JPRB ,&
&  0.94888E+00_JPRB, 0.89609E+00_JPRB, 0.86758E+00_JPRB, 0.82392E+00_JPRB, 0.78246E+00_JPRB, 0.73743E+00_JPRB, 0.71921E+00_JPRB ,&
&  0.70682E+00_JPRB, 0.69371E+00_JPRB, 0.69177E+00_JPRB, 0.68940E+00_JPRB, 0.69013E+00_JPRB, 0.68565E+00_JPRB, 0.68905E+00_JPRB ,&
&  0.68740E+00_JPRB, 0.68320E+00_JPRB, 0.67914E+00_JPRB, 0.69502E+00_JPRB, 0.68548E+00_JPRB, 0.67237E+00_JPRB, 0.70034E+00_JPRB ,&
&  0.78131E+00_JPRB, 0.89692E+00_JPRB, 0.83152E+00_JPRB, 0.75578E+00_JPRB, 0.72035E+00_JPRB, 0.71163E+00_JPRB, 0.70365E+00_JPRB ,&
&  0.69451E+00_JPRB, 0.69634E+00_JPRB, 0.71692E+00_JPRB, 0.72275E+00_JPRB, 0.71103E+00_JPRB, 0.69239E+00_JPRB, 0.69362E+00_JPRB ,&
&  0.69743E+00_JPRB, 0.69197E+00_JPRB, 0.67722E+00_JPRB, 0.67179E+00_JPRB, 0.66796E+00_JPRB, 0.65565E+00_JPRB, 0.65030E+00_JPRB ,&
&  0.64460E+00_JPRB, 0.65686E+00_JPRB, 0.66069E+00_JPRB, 0.66112E+00_JPRB, 0.67013E+00_JPRB, 0.68690E+00_JPRB, 0.69517E+00_JPRB ,&
&  0.69986E+00_JPRB, 0.70289E+00_JPRB, 0.69006E+00_JPRB, 0.69099E+00_JPRB, 0.65766E+00_JPRB, 0.67913E+00_JPRB, 0.63742E+00_JPRB ,&
&  0.62311E+00_JPRB, 0.62436E+00_JPRB, 0.63514E+00_JPRB, 0.64640E+00_JPRB, 0.66642E+00_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.63066E+00_JPRB, 0.62633E+00_JPRB, 0.62776E+00_JPRB, 0.62215E+00_JPRB, 0.60883E+00_JPRB, 0.59770E+00_JPRB, 0.59902E+00_JPRB ,&
&  0.59880E+00_JPRB, 0.60342E+00_JPRB, 0.61050E+00_JPRB, 0.60174E+00_JPRB, 0.60053E+00_JPRB, 0.60197E+00_JPRB, 0.59966E+00_JPRB ,&
&  0.59059E+00_JPRB, 0.58633E+00_JPRB, 0.58265E+00_JPRB, 0.57968E+00_JPRB, 0.57907E+00_JPRB, 0.57141E+00_JPRB, 0.56806E+00_JPRB ,&
&  0.56718E+00_JPRB, 0.57088E+00_JPRB, 0.57470E+00_JPRB, 0.57543E+00_JPRB, 0.58187E+00_JPRB, 0.57934E+00_JPRB, 0.57338E+00_JPRB ,&
&  0.59323E+00_JPRB, 0.95905E+00_JPRB, 0.91392E+00_JPRB, 0.15122E+01_JPRB, 0.13525E+01_JPRB, 0.11105E+01_JPRB, 0.91998E+00_JPRB ,&
&  0.90271E+00_JPRB, 0.92133E+00_JPRB, 0.87568E+00_JPRB, 0.83179E+00_JPRB, 0.79518E+00_JPRB, 0.75595E+00_JPRB, 0.72229E+00_JPRB ,&
&  0.69818E+00_JPRB, 0.68832E+00_JPRB, 0.69233E+00_JPRB, 0.69877E+00_JPRB, 0.70958E+00_JPRB, 0.71835E+00_JPRB, 0.71888E+00_JPRB ,&
&  0.71898E+00_JPRB, 0.72255E+00_JPRB, 0.73488E+00_JPRB, 0.76926E+00_JPRB, 0.71644E+00_JPRB, 0.70147E+00_JPRB, 0.73731E+00_JPRB ,&
&  0.76087E+00_JPRB, 0.87462E+00_JPRB, 0.80434E+00_JPRB, 0.74179E+00_JPRB, 0.72357E+00_JPRB, 0.71403E+00_JPRB, 0.70234E+00_JPRB ,&
&  0.69760E+00_JPRB, 0.70761E+00_JPRB, 0.73133E+00_JPRB, 0.71665E+00_JPRB, 0.69042E+00_JPRB, 0.68401E+00_JPRB, 0.68299E+00_JPRB ,&
&  0.68406E+00_JPRB, 0.67309E+00_JPRB, 0.68880E+00_JPRB, 0.69353E+00_JPRB, 0.69006E+00_JPRB, 0.67858E+00_JPRB, 0.66697E+00_JPRB ,&
&  0.66296E+00_JPRB, 0.67481E+00_JPRB, 0.66483E+00_JPRB, 0.65090E+00_JPRB, 0.65431E+00_JPRB, 0.65711E+00_JPRB, 0.65510E+00_JPRB ,&
&  0.69375E+00_JPRB, 0.70251E+00_JPRB, 0.68623E+00_JPRB, 0.67490E+00_JPRB, 0.66029E+00_JPRB, 0.68008E+00_JPRB, 0.63435E+00_JPRB ,&
&  0.62486E+00_JPRB, 0.62336E+00_JPRB, 0.62907E+00_JPRB, 0.63990E+00_JPRB, 0.64268E+00_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.62774E+00_JPRB, 0.62423E+00_JPRB, 0.62569E+00_JPRB, 0.61196E+00_JPRB, 0.59485E+00_JPRB, 0.58593E+00_JPRB, 0.58565E+00_JPRB ,&
&  0.58921E+00_JPRB, 0.60049E+00_JPRB, 0.60815E+00_JPRB, 0.59405E+00_JPRB, 0.58540E+00_JPRB, 0.59009E+00_JPRB, 0.58878E+00_JPRB ,&
&  0.58505E+00_JPRB, 0.58008E+00_JPRB, 0.57846E+00_JPRB, 0.57870E+00_JPRB, 0.57641E+00_JPRB, 0.56734E+00_JPRB, 0.56772E+00_JPRB ,&
&  0.57636E+00_JPRB, 0.58424E+00_JPRB, 0.58763E+00_JPRB, 0.58849E+00_JPRB, 0.59576E+00_JPRB, 0.59163E+00_JPRB, 0.59097E+00_JPRB ,&
&  0.61056E+00_JPRB, 0.97918E+00_JPRB, 0.10157E+01_JPRB, 0.17334E+01_JPRB, 0.14578E+01_JPRB, 0.11888E+01_JPRB, 0.97892E+00_JPRB ,&
&  0.90176E+00_JPRB, 0.94163E+00_JPRB, 0.87050E+00_JPRB, 0.78809E+00_JPRB, 0.74352E+00_JPRB, 0.73000E+00_JPRB, 0.71364E+00_JPRB ,&
&  0.69589E+00_JPRB, 0.70766E+00_JPRB, 0.72006E+00_JPRB, 0.73463E+00_JPRB, 0.74833E+00_JPRB, 0.75157E+00_JPRB, 0.76229E+00_JPRB ,&
&  0.76715E+00_JPRB, 0.78392E+00_JPRB, 0.81336E+00_JPRB, 0.78657E+00_JPRB, 0.74175E+00_JPRB, 0.74906E+00_JPRB, 0.75323E+00_JPRB ,&
&  0.76405E+00_JPRB, 0.82507E+00_JPRB, 0.77199E+00_JPRB, 0.73415E+00_JPRB, 0.70577E+00_JPRB, 0.70163E+00_JPRB, 0.69266E+00_JPRB ,&
&  0.68441E+00_JPRB, 0.69370E+00_JPRB, 0.69859E+00_JPRB, 0.68989E+00_JPRB, 0.67973E+00_JPRB, 0.67354E+00_JPRB, 0.66469E+00_JPRB ,&
&  0.66569E+00_JPRB, 0.68710E+00_JPRB, 0.69237E+00_JPRB, 0.69670E+00_JPRB, 0.70297E+00_JPRB, 0.70519E+00_JPRB, 0.69464E+00_JPRB ,&
&  0.69103E+00_JPRB, 0.69359E+00_JPRB, 0.66338E+00_JPRB, 0.65233E+00_JPRB, 0.65889E+00_JPRB, 0.65527E+00_JPRB, 0.65528E+00_JPRB ,&
&  0.68071E+00_JPRB, 0.68924E+00_JPRB, 0.67151E+00_JPRB, 0.64252E+00_JPRB, 0.65595E+00_JPRB, 0.67664E+00_JPRB, 0.63729E+00_JPRB ,&
&  0.63290E+00_JPRB, 0.63194E+00_JPRB, 0.63556E+00_JPRB, 0.63570E+00_JPRB, 0.63190E+00_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.60293E+00_JPRB, 0.59921E+00_JPRB, 0.59517E+00_JPRB, 0.56203E+00_JPRB, 0.53613E+00_JPRB, 0.52855E+00_JPRB, 0.54541E+00_JPRB ,&
&  0.57761E+00_JPRB, 0.59960E+00_JPRB, 0.59518E+00_JPRB, 0.57971E+00_JPRB, 0.57170E+00_JPRB, 0.57212E+00_JPRB, 0.57638E+00_JPRB ,&
&  0.57318E+00_JPRB, 0.56160E+00_JPRB, 0.55527E+00_JPRB, 0.56230E+00_JPRB, 0.56425E+00_JPRB, 0.57113E+00_JPRB, 0.58273E+00_JPRB ,&
&  0.59892E+00_JPRB, 0.60696E+00_JPRB, 0.60729E+00_JPRB, 0.60948E+00_JPRB, 0.62536E+00_JPRB, 0.62034E+00_JPRB, 0.62300E+00_JPRB ,&
&  0.68177E+00_JPRB, 0.10723E+01_JPRB, 0.12718E+01_JPRB, 0.15568E+01_JPRB, 0.13620E+01_JPRB, 0.10768E+01_JPRB, 0.93163E+00_JPRB ,&
&  0.81858E+00_JPRB, 0.72878E+00_JPRB, 0.78964E+00_JPRB, 0.73121E+00_JPRB, 0.72657E+00_JPRB, 0.71295E+00_JPRB, 0.71015E+00_JPRB ,&
&  0.72789E+00_JPRB, 0.75575E+00_JPRB, 0.77267E+00_JPRB, 0.78233E+00_JPRB, 0.79063E+00_JPRB, 0.79809E+00_JPRB, 0.81161E+00_JPRB ,&
&  0.82191E+00_JPRB, 0.84279E+00_JPRB, 0.86162E+00_JPRB, 0.80919E+00_JPRB, 0.72921E+00_JPRB, 0.75820E+00_JPRB, 0.76100E+00_JPRB ,&
&  0.77047E+00_JPRB, 0.77446E+00_JPRB, 0.75051E+00_JPRB, 0.71661E+00_JPRB, 0.66173E+00_JPRB, 0.69362E+00_JPRB, 0.67948E+00_JPRB ,&
&  0.65830E+00_JPRB, 0.66969E+00_JPRB, 0.66865E+00_JPRB, 0.67574E+00_JPRB, 0.67519E+00_JPRB, 0.66812E+00_JPRB, 0.66811E+00_JPRB ,&
&  0.67796E+00_JPRB, 0.68523E+00_JPRB, 0.69363E+00_JPRB, 0.70085E+00_JPRB, 0.71268E+00_JPRB, 0.72004E+00_JPRB, 0.71514E+00_JPRB ,&
&  0.70460E+00_JPRB, 0.70746E+00_JPRB, 0.68220E+00_JPRB, 0.68691E+00_JPRB, 0.68496E+00_JPRB, 0.68021E+00_JPRB, 0.65860E+00_JPRB ,&
&  0.65933E+00_JPRB, 0.67078E+00_JPRB, 0.65863E+00_JPRB, 0.64960E+00_JPRB, 0.66114E+00_JPRB, 0.65944E+00_JPRB, 0.63675E+00_JPRB ,&
&  0.63856E+00_JPRB, 0.63464E+00_JPRB, 0.63676E+00_JPRB, 0.62104E+00_JPRB, 0.61071E+00_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.59069E+00_JPRB, 0.58312E+00_JPRB, 0.55997E+00_JPRB, 0.50866E+00_JPRB, 0.49986E+00_JPRB, 0.51778E+00_JPRB, 0.57614E+00_JPRB ,&
&  0.59961E+00_JPRB, 0.60314E+00_JPRB, 0.58877E+00_JPRB, 0.56959E+00_JPRB, 0.55672E+00_JPRB, 0.55244E+00_JPRB, 0.56234E+00_JPRB ,&
&  0.57024E+00_JPRB, 0.55890E+00_JPRB, 0.55789E+00_JPRB, 0.56428E+00_JPRB, 0.57356E+00_JPRB, 0.59011E+00_JPRB, 0.60741E+00_JPRB ,&
&  0.62244E+00_JPRB, 0.62989E+00_JPRB, 0.63168E+00_JPRB, 0.65063E+00_JPRB, 0.67540E+00_JPRB, 0.68303E+00_JPRB, 0.71441E+00_JPRB ,&
&  0.90420E+00_JPRB, 0.12377E+01_JPRB, 0.64830E+00_JPRB, 0.97479E+00_JPRB, 0.10898E+01_JPRB, 0.94622E+00_JPRB, 0.83586E+00_JPRB ,&
&  0.75111E+00_JPRB, 0.66474E+00_JPRB, 0.68044E+00_JPRB, 0.73579E+00_JPRB, 0.73109E+00_JPRB, 0.75052E+00_JPRB, 0.76626E+00_JPRB ,&
&  0.79008E+00_JPRB, 0.80401E+00_JPRB, 0.81362E+00_JPRB, 0.82006E+00_JPRB, 0.82510E+00_JPRB, 0.84022E+00_JPRB, 0.85011E+00_JPRB ,&
&  0.84995E+00_JPRB, 0.86159E+00_JPRB, 0.84759E+00_JPRB, 0.73882E+00_JPRB, 0.71988E+00_JPRB, 0.74653E+00_JPRB, 0.72850E+00_JPRB ,&
&  0.69615E+00_JPRB, 0.72543E+00_JPRB, 0.73140E+00_JPRB, 0.68855E+00_JPRB, 0.63884E+00_JPRB, 0.62173E+00_JPRB, 0.65606E+00_JPRB ,&
&  0.64692E+00_JPRB, 0.64156E+00_JPRB, 0.63792E+00_JPRB, 0.64464E+00_JPRB, 0.64522E+00_JPRB, 0.64666E+00_JPRB, 0.65142E+00_JPRB ,&
&  0.66650E+00_JPRB, 0.67966E+00_JPRB, 0.68988E+00_JPRB, 0.69693E+00_JPRB, 0.70868E+00_JPRB, 0.72041E+00_JPRB, 0.71904E+00_JPRB ,&
&  0.72514E+00_JPRB, 0.72708E+00_JPRB, 0.73027E+00_JPRB, 0.72120E+00_JPRB, 0.69976E+00_JPRB, 0.68707E+00_JPRB, 0.67951E+00_JPRB ,&
&  0.67466E+00_JPRB, 0.66845E+00_JPRB, 0.64893E+00_JPRB, 0.64134E+00_JPRB, 0.67819E+00_JPRB, 0.67304E+00_JPRB, 0.72230E+00_JPRB ,&
&  0.78804E+00_JPRB, 0.84636E+00_JPRB, 0.80757E+00_JPRB, 0.61686E+00_JPRB, 0.59668E+00_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.57284E+00_JPRB, 0.54740E+00_JPRB, 0.49941E+00_JPRB, 0.47057E+00_JPRB, 0.49550E+00_JPRB, 0.55829E+00_JPRB, 0.60574E+00_JPRB ,&
&  0.62134E+00_JPRB, 0.61588E+00_JPRB, 0.60977E+00_JPRB, 0.60430E+00_JPRB, 0.61043E+00_JPRB, 0.59910E+00_JPRB, 0.60032E+00_JPRB ,&
&  0.60431E+00_JPRB, 0.60140E+00_JPRB, 0.60493E+00_JPRB, 0.60832E+00_JPRB, 0.61403E+00_JPRB, 0.62992E+00_JPRB, 0.64817E+00_JPRB ,&
&  0.66296E+00_JPRB, 0.67541E+00_JPRB, 0.69623E+00_JPRB, 0.72552E+00_JPRB, 0.77071E+00_JPRB, 0.85224E+00_JPRB, 0.99623E+00_JPRB ,&
&  0.12722E+01_JPRB, 0.89067E+00_JPRB, 0.55619E+00_JPRB, 0.90713E+00_JPRB, 0.94181E+00_JPRB, 0.87066E+00_JPRB, 0.76417E+00_JPRB ,&
&  0.73109E+00_JPRB, 0.70576E+00_JPRB, 0.72013E+00_JPRB, 0.77655E+00_JPRB, 0.79888E+00_JPRB, 0.83587E+00_JPRB, 0.86234E+00_JPRB ,&
&  0.86237E+00_JPRB, 0.85759E+00_JPRB, 0.85466E+00_JPRB, 0.85442E+00_JPRB, 0.86178E+00_JPRB, 0.87670E+00_JPRB, 0.87487E+00_JPRB ,&
&  0.87073E+00_JPRB, 0.87048E+00_JPRB, 0.84126E+00_JPRB, 0.70365E+00_JPRB, 0.69633E+00_JPRB, 0.69961E+00_JPRB, 0.69079E+00_JPRB ,&
&  0.66047E+00_JPRB, 0.67899E+00_JPRB, 0.69626E+00_JPRB, 0.66859E+00_JPRB, 0.63096E+00_JPRB, 0.60289E+00_JPRB, 0.64375E+00_JPRB ,&
&  0.64952E+00_JPRB, 0.62016E+00_JPRB, 0.59431E+00_JPRB, 0.58119E+00_JPRB, 0.60372E+00_JPRB, 0.62221E+00_JPRB, 0.61854E+00_JPRB ,&
&  0.62953E+00_JPRB, 0.64730E+00_JPRB, 0.65986E+00_JPRB, 0.67213E+00_JPRB, 0.69074E+00_JPRB, 0.71032E+00_JPRB, 0.72018E+00_JPRB ,&
&  0.72881E+00_JPRB, 0.72774E+00_JPRB, 0.73087E+00_JPRB, 0.72449E+00_JPRB, 0.69335E+00_JPRB, 0.66054E+00_JPRB, 0.66447E+00_JPRB ,&
&  0.66459E+00_JPRB, 0.67268E+00_JPRB, 0.66774E+00_JPRB, 0.68076E+00_JPRB, 0.69416E+00_JPRB, 0.72470E+00_JPRB, 0.76053E+00_JPRB ,&
&  0.78986E+00_JPRB, 0.77920E+00_JPRB, 0.70078E+00_JPRB, 0.60789E+00_JPRB, 0.58769E+00_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.54080E+00_JPRB, 0.48501E+00_JPRB, 0.43641E+00_JPRB, 0.45404E+00_JPRB, 0.51581E+00_JPRB, 0.56738E+00_JPRB, 0.59299E+00_JPRB ,&
&  0.62136E+00_JPRB, 0.62629E+00_JPRB, 0.62952E+00_JPRB, 0.63489E+00_JPRB, 0.64845E+00_JPRB, 0.64407E+00_JPRB, 0.64943E+00_JPRB ,&
&  0.64304E+00_JPRB, 0.63036E+00_JPRB, 0.62554E+00_JPRB, 0.63806E+00_JPRB, 0.65569E+00_JPRB, 0.67577E+00_JPRB, 0.69851E+00_JPRB ,&
&  0.72366E+00_JPRB, 0.74833E+00_JPRB, 0.78640E+00_JPRB, 0.83261E+00_JPRB, 0.89274E+00_JPRB, 0.95687E+00_JPRB, 0.10027E+01_JPRB ,&
&  0.73884E+00_JPRB, 0.45684E+00_JPRB, 0.76641E+00_JPRB, 0.91802E+00_JPRB, 0.84961E+00_JPRB, 0.80559E+00_JPRB, 0.77089E+00_JPRB ,&
&  0.73721E+00_JPRB, 0.72804E+00_JPRB, 0.73185E+00_JPRB, 0.85714E+00_JPRB, 0.88714E+00_JPRB, 0.89582E+00_JPRB, 0.89887E+00_JPRB ,&
&  0.90012E+00_JPRB, 0.89929E+00_JPRB, 0.90334E+00_JPRB, 0.91590E+00_JPRB, 0.92708E+00_JPRB, 0.93601E+00_JPRB, 0.93181E+00_JPRB ,&
&  0.92571E+00_JPRB, 0.90967E+00_JPRB, 0.87293E+00_JPRB, 0.68662E+00_JPRB, 0.67275E+00_JPRB, 0.66817E+00_JPRB, 0.65135E+00_JPRB ,&
&  0.64542E+00_JPRB, 0.62569E+00_JPRB, 0.62823E+00_JPRB, 0.66661E+00_JPRB, 0.64624E+00_JPRB, 0.58319E+00_JPRB, 0.59451E+00_JPRB ,&
&  0.61037E+00_JPRB, 0.59820E+00_JPRB, 0.57051E+00_JPRB, 0.55692E+00_JPRB, 0.58394E+00_JPRB, 0.58594E+00_JPRB, 0.59359E+00_JPRB ,&
&  0.58938E+00_JPRB, 0.61329E+00_JPRB, 0.63617E+00_JPRB, 0.65327E+00_JPRB, 0.67249E+00_JPRB, 0.69499E+00_JPRB, 0.69475E+00_JPRB ,&
&  0.69977E+00_JPRB, 0.69538E+00_JPRB, 0.70007E+00_JPRB, 0.68881E+00_JPRB, 0.67809E+00_JPRB, 0.65122E+00_JPRB, 0.64651E+00_JPRB ,&
&  0.65335E+00_JPRB, 0.66211E+00_JPRB, 0.66579E+00_JPRB, 0.66257E+00_JPRB, 0.69335E+00_JPRB, 0.69950E+00_JPRB, 0.71115E+00_JPRB ,&
&  0.71001E+00_JPRB, 0.69051E+00_JPRB, 0.63322E+00_JPRB, 0.58969E+00_JPRB, 0.57241E+00_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.45765E+00_JPRB, 0.40641E+00_JPRB, 0.40120E+00_JPRB, 0.44219E+00_JPRB, 0.48344E+00_JPRB, 0.52518E+00_JPRB, 0.56176E+00_JPRB ,&
&  0.59680E+00_JPRB, 0.61447E+00_JPRB, 0.65134E+00_JPRB, 0.66312E+00_JPRB, 0.66887E+00_JPRB, 0.65908E+00_JPRB, 0.64748E+00_JPRB ,&
&  0.65261E+00_JPRB, 0.65699E+00_JPRB, 0.65869E+00_JPRB, 0.67412E+00_JPRB, 0.69007E+00_JPRB, 0.70925E+00_JPRB, 0.73591E+00_JPRB ,&
&  0.76232E+00_JPRB, 0.79425E+00_JPRB, 0.82518E+00_JPRB, 0.86165E+00_JPRB, 0.88119E+00_JPRB, 0.87916E+00_JPRB, 0.81396E+00_JPRB ,&
&  0.46397E+00_JPRB, 0.72039E+00_JPRB, 0.88052E+00_JPRB, 0.86086E+00_JPRB, 0.82630E+00_JPRB, 0.79019E+00_JPRB, 0.79243E+00_JPRB ,&
&  0.76206E+00_JPRB, 0.72229E+00_JPRB, 0.74187E+00_JPRB, 0.91191E+00_JPRB, 0.90445E+00_JPRB, 0.91019E+00_JPRB, 0.91875E+00_JPRB ,&
&  0.93503E+00_JPRB, 0.94719E+00_JPRB, 0.95766E+00_JPRB, 0.96786E+00_JPRB, 0.99185E+00_JPRB, 0.10106E+01_JPRB, 0.10184E+01_JPRB ,&
&  0.10112E+01_JPRB, 0.99772E+00_JPRB, 0.98651E+00_JPRB, 0.78152E+00_JPRB, 0.75688E+00_JPRB, 0.70674E+00_JPRB, 0.66157E+00_JPRB ,&
&  0.61388E+00_JPRB, 0.61048E+00_JPRB, 0.63220E+00_JPRB, 0.69250E+00_JPRB, 0.65938E+00_JPRB, 0.63038E+00_JPRB, 0.58256E+00_JPRB ,&
&  0.58956E+00_JPRB, 0.58996E+00_JPRB, 0.58738E+00_JPRB, 0.57141E+00_JPRB, 0.57122E+00_JPRB, 0.54887E+00_JPRB, 0.58488E+00_JPRB ,&
&  0.59945E+00_JPRB, 0.58727E+00_JPRB, 0.58733E+00_JPRB, 0.60738E+00_JPRB, 0.63510E+00_JPRB, 0.66303E+00_JPRB, 0.66933E+00_JPRB ,&
&  0.65956E+00_JPRB, 0.64353E+00_JPRB, 0.64497E+00_JPRB, 0.64649E+00_JPRB, 0.64715E+00_JPRB, 0.64759E+00_JPRB, 0.63874E+00_JPRB ,&
&  0.63364E+00_JPRB, 0.65186E+00_JPRB, 0.69401E+00_JPRB, 0.65260E+00_JPRB, 0.70193E+00_JPRB, 0.63828E+00_JPRB, 0.62210E+00_JPRB ,&
&  0.60599E+00_JPRB, 0.58636E+00_JPRB, 0.54015E+00_JPRB, 0.51776E+00_JPRB, 0.49053E+00_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.37215E+00_JPRB, 0.35516E+00_JPRB, 0.36420E+00_JPRB, 0.38756E+00_JPRB, 0.41517E+00_JPRB, 0.45652E+00_JPRB, 0.50500E+00_JPRB ,&
&  0.54528E+00_JPRB, 0.58235E+00_JPRB, 0.64612E+00_JPRB, 0.65316E+00_JPRB, 0.65552E+00_JPRB, 0.68117E+00_JPRB, 0.69029E+00_JPRB ,&
&  0.69846E+00_JPRB, 0.69515E+00_JPRB, 0.69770E+00_JPRB, 0.70166E+00_JPRB, 0.71977E+00_JPRB, 0.74124E+00_JPRB, 0.75464E+00_JPRB ,&
&  0.77854E+00_JPRB, 0.80680E+00_JPRB, 0.85497E+00_JPRB, 0.90320E+00_JPRB, 0.94510E+00_JPRB, 0.93499E+00_JPRB, 0.90737E+00_JPRB ,&
&  0.66321E+00_JPRB, 0.85315E+00_JPRB, 0.84858E+00_JPRB, 0.83121E+00_JPRB, 0.79773E+00_JPRB, 0.76494E+00_JPRB, 0.75884E+00_JPRB ,&
&  0.75405E+00_JPRB, 0.72223E+00_JPRB, 0.73818E+00_JPRB, 0.80761E+00_JPRB, 0.90045E+00_JPRB, 0.91512E+00_JPRB, 0.92748E+00_JPRB ,&
&  0.95306E+00_JPRB, 0.97585E+00_JPRB, 0.99922E+00_JPRB, 0.10291E+01_JPRB, 0.10624E+01_JPRB, 0.10925E+01_JPRB, 0.11182E+01_JPRB ,&
&  0.11343E+01_JPRB, 0.11417E+01_JPRB, 0.11831E+01_JPRB, 0.10173E+01_JPRB, 0.91656E+00_JPRB, 0.82883E+00_JPRB, 0.68941E+00_JPRB ,&
&  0.61728E+00_JPRB, 0.59397E+00_JPRB, 0.64470E+00_JPRB, 0.70585E+00_JPRB, 0.67082E+00_JPRB, 0.63669E+00_JPRB, 0.58779E+00_JPRB ,&
&  0.57988E+00_JPRB, 0.57896E+00_JPRB, 0.58131E+00_JPRB, 0.57256E+00_JPRB, 0.57839E+00_JPRB, 0.56577E+00_JPRB, 0.57072E+00_JPRB ,&
&  0.58493E+00_JPRB, 0.57485E+00_JPRB, 0.56795E+00_JPRB, 0.57594E+00_JPRB, 0.60042E+00_JPRB, 0.63072E+00_JPRB, 0.63211E+00_JPRB ,&
&  0.63675E+00_JPRB, 0.57650E+00_JPRB, 0.55682E+00_JPRB, 0.57426E+00_JPRB, 0.58221E+00_JPRB, 0.59009E+00_JPRB, 0.61575E+00_JPRB ,&
&  0.70268E+00_JPRB, 0.80908E+00_JPRB, 0.90516E+00_JPRB, 0.90571E+00_JPRB, 0.96469E+00_JPRB, 0.75252E+00_JPRB, 0.55284E+00_JPRB ,&
&  0.51770E+00_JPRB, 0.50510E+00_JPRB, 0.46247E+00_JPRB, 0.43360E+00_JPRB, 0.40097E+00_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.36618E+00_JPRB, 0.36010E+00_JPRB, 0.36888E+00_JPRB, 0.38495E+00_JPRB, 0.39652E+00_JPRB, 0.42349E+00_JPRB, 0.45567E+00_JPRB ,&
&  0.49076E+00_JPRB, 0.53314E+00_JPRB, 0.56470E+00_JPRB, 0.58159E+00_JPRB, 0.61530E+00_JPRB, 0.66650E+00_JPRB, 0.68002E+00_JPRB ,&
&  0.71646E+00_JPRB, 0.73171E+00_JPRB, 0.72879E+00_JPRB, 0.72315E+00_JPRB, 0.73328E+00_JPRB, 0.75885E+00_JPRB, 0.78311E+00_JPRB ,&
&  0.79850E+00_JPRB, 0.82532E+00_JPRB, 0.87042E+00_JPRB, 0.89510E+00_JPRB, 0.89812E+00_JPRB, 0.87450E+00_JPRB, 0.68038E+00_JPRB ,&
&  0.85808E+00_JPRB, 0.86405E+00_JPRB, 0.85952E+00_JPRB, 0.82061E+00_JPRB, 0.78792E+00_JPRB, 0.75696E+00_JPRB, 0.74517E+00_JPRB ,&
&  0.73846E+00_JPRB, 0.72345E+00_JPRB, 0.71521E+00_JPRB, 0.73256E+00_JPRB, 0.85227E+00_JPRB, 0.88275E+00_JPRB, 0.91927E+00_JPRB ,&
&  0.93128E+00_JPRB, 0.94700E+00_JPRB, 0.98111E+00_JPRB, 0.10189E+01_JPRB, 0.10592E+01_JPRB, 0.10913E+01_JPRB, 0.11207E+01_JPRB ,&
&  0.11530E+01_JPRB, 0.11823E+01_JPRB, 0.12199E+01_JPRB, 0.10884E+01_JPRB, 0.99923E+00_JPRB, 0.93675E+00_JPRB, 0.75129E+00_JPRB ,&
&  0.62299E+00_JPRB, 0.59366E+00_JPRB, 0.64315E+00_JPRB, 0.69235E+00_JPRB, 0.64842E+00_JPRB, 0.61128E+00_JPRB, 0.57658E+00_JPRB ,&
&  0.57368E+00_JPRB, 0.58150E+00_JPRB, 0.59422E+00_JPRB, 0.59629E+00_JPRB, 0.57766E+00_JPRB, 0.56204E+00_JPRB, 0.55239E+00_JPRB ,&
&  0.56989E+00_JPRB, 0.56647E+00_JPRB, 0.55727E+00_JPRB, 0.57543E+00_JPRB, 0.58012E+00_JPRB, 0.63727E+00_JPRB, 0.72918E+00_JPRB ,&
&  0.73217E+00_JPRB, 0.47841E+00_JPRB, 0.49525E+00_JPRB, 0.51526E+00_JPRB, 0.56044E+00_JPRB, 0.63667E+00_JPRB, 0.79227E+00_JPRB ,&
&  0.11411E+01_JPRB, 0.19078E+01_JPRB, 0.21921E+01_JPRB, 0.14488E+01_JPRB, 0.19801E+01_JPRB, 0.12762E+01_JPRB, 0.49886E+00_JPRB ,&
&  0.47082E+00_JPRB, 0.45620E+00_JPRB, 0.42874E+00_JPRB, 0.39981E+00_JPRB, 0.38053E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.44255E+00_JPRB, 0.44345E+00_JPRB, 0.45002E+00_JPRB, 0.45419E+00_JPRB, 0.45854E+00_JPRB, 0.46104E+00_JPRB, 0.46392E+00_JPRB ,&
&  0.47627E+00_JPRB, 0.48306E+00_JPRB, 0.50076E+00_JPRB, 0.51200E+00_JPRB, 0.52187E+00_JPRB, 0.54709E+00_JPRB, 0.56584E+00_JPRB ,&
&  0.59793E+00_JPRB, 0.63874E+00_JPRB, 0.64976E+00_JPRB, 0.66957E+00_JPRB, 0.67827E+00_JPRB, 0.68470E+00_JPRB, 0.68503E+00_JPRB ,&
&  0.72388E+00_JPRB, 0.80016E+00_JPRB, 0.84450E+00_JPRB, 0.89183E+00_JPRB, 0.87708E+00_JPRB, 0.87965E+00_JPRB, 0.69347E+00_JPRB ,&
&  0.91775E+00_JPRB, 0.90733E+00_JPRB, 0.84061E+00_JPRB, 0.80065E+00_JPRB, 0.77363E+00_JPRB, 0.74347E+00_JPRB, 0.70814E+00_JPRB ,&
&  0.69238E+00_JPRB, 0.71121E+00_JPRB, 0.70409E+00_JPRB, 0.74740E+00_JPRB, 0.80484E+00_JPRB, 0.81258E+00_JPRB, 0.85510E+00_JPRB ,&
&  0.85619E+00_JPRB, 0.85397E+00_JPRB, 0.86603E+00_JPRB, 0.89703E+00_JPRB, 0.92095E+00_JPRB, 0.93626E+00_JPRB, 0.94720E+00_JPRB ,&
&  0.95293E+00_JPRB, 0.98017E+00_JPRB, 0.10462E+01_JPRB, 0.10345E+01_JPRB, 0.97048E+00_JPRB, 0.90466E+00_JPRB, 0.79384E+00_JPRB ,&
&  0.63415E+00_JPRB, 0.57779E+00_JPRB, 0.60051E+00_JPRB, 0.67589E+00_JPRB, 0.64453E+00_JPRB, 0.60896E+00_JPRB, 0.58313E+00_JPRB ,&
&  0.56178E+00_JPRB, 0.56476E+00_JPRB, 0.57218E+00_JPRB, 0.56860E+00_JPRB, 0.55308E+00_JPRB, 0.52877E+00_JPRB, 0.50738E+00_JPRB ,&
&  0.50613E+00_JPRB, 0.49810E+00_JPRB, 0.49393E+00_JPRB, 0.49774E+00_JPRB, 0.49940E+00_JPRB, 0.51770E+00_JPRB, 0.48178E+00_JPRB ,&
&  0.52854E+00_JPRB, 0.43815E+00_JPRB, 0.46406E+00_JPRB, 0.50719E+00_JPRB, 0.57732E+00_JPRB, 0.80631E+00_JPRB, 0.12076E+01_JPRB ,&
&  0.18022E+01_JPRB, 0.20724E+01_JPRB, 0.27609E+01_JPRB, 0.40630E+01_JPRB, 0.25521E+01_JPRB, 0.16186E+01_JPRB, 0.51216E+00_JPRB ,&
&  0.50068E+00_JPRB, 0.47502E+00_JPRB, 0.46734E+00_JPRB, 0.45264E+00_JPRB, 0.44833E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.50368E+00_JPRB, 0.51168E+00_JPRB, 0.52068E+00_JPRB, 0.52853E+00_JPRB, 0.53316E+00_JPRB, 0.54217E+00_JPRB, 0.54835E+00_JPRB ,&
&  0.55570E+00_JPRB, 0.55733E+00_JPRB, 0.56216E+00_JPRB, 0.56573E+00_JPRB, 0.57282E+00_JPRB, 0.58404E+00_JPRB, 0.59495E+00_JPRB ,&
&  0.60237E+00_JPRB, 0.61424E+00_JPRB, 0.62920E+00_JPRB, 0.63695E+00_JPRB, 0.66062E+00_JPRB, 0.70784E+00_JPRB, 0.75933E+00_JPRB ,&
&  0.79470E+00_JPRB, 0.83907E+00_JPRB, 0.87808E+00_JPRB, 0.88433E+00_JPRB, 0.86313E+00_JPRB, 0.84553E+00_JPRB, 0.70466E+00_JPRB ,&
&  0.10394E+01_JPRB, 0.10238E+01_JPRB, 0.91703E+00_JPRB, 0.85381E+00_JPRB, 0.79883E+00_JPRB, 0.74805E+00_JPRB, 0.69429E+00_JPRB ,&
&  0.65616E+00_JPRB, 0.70375E+00_JPRB, 0.70161E+00_JPRB, 0.71022E+00_JPRB, 0.69159E+00_JPRB, 0.69276E+00_JPRB, 0.74920E+00_JPRB ,&
&  0.80361E+00_JPRB, 0.78290E+00_JPRB, 0.72497E+00_JPRB, 0.72803E+00_JPRB, 0.74104E+00_JPRB, 0.73999E+00_JPRB, 0.76092E+00_JPRB ,&
&  0.74232E+00_JPRB, 0.72958E+00_JPRB, 0.69065E+00_JPRB, 0.76813E+00_JPRB, 0.77413E+00_JPRB, 0.74599E+00_JPRB, 0.69121E+00_JPRB ,&
&  0.54787E+00_JPRB, 0.55369E+00_JPRB, 0.60011E+00_JPRB, 0.69685E+00_JPRB, 0.65896E+00_JPRB, 0.62883E+00_JPRB, 0.59282E+00_JPRB ,&
&  0.56382E+00_JPRB, 0.55038E+00_JPRB, 0.52923E+00_JPRB, 0.51943E+00_JPRB, 0.51507E+00_JPRB, 0.50079E+00_JPRB, 0.47505E+00_JPRB ,&
&  0.45389E+00_JPRB, 0.44937E+00_JPRB, 0.44312E+00_JPRB, 0.43636E+00_JPRB, 0.43201E+00_JPRB, 0.38406E+00_JPRB, 0.44865E+00_JPRB ,&
&  0.44005E+00_JPRB, 0.38556E+00_JPRB, 0.47030E+00_JPRB, 0.38178E+00_JPRB, 0.49777E+00_JPRB, 0.61663E+00_JPRB, 0.80858E+00_JPRB ,&
&  0.86025E+00_JPRB, 0.11130E+01_JPRB, 0.14640E+01_JPRB, 0.11547E+01_JPRB, 0.86810E+00_JPRB, 0.57088E+00_JPRB, 0.53431E+00_JPRB ,&
&  0.52255E+00_JPRB, 0.51209E+00_JPRB, 0.51467E+00_JPRB, 0.50752E+00_JPRB, 0.50572E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.57181E+00_JPRB, 0.58306E+00_JPRB, 0.59665E+00_JPRB, 0.61440E+00_JPRB, 0.62461E+00_JPRB, 0.64030E+00_JPRB, 0.65176E+00_JPRB ,&
&  0.66850E+00_JPRB, 0.68027E+00_JPRB, 0.68934E+00_JPRB, 0.69863E+00_JPRB, 0.71221E+00_JPRB, 0.73107E+00_JPRB, 0.75574E+00_JPRB ,&
&  0.78255E+00_JPRB, 0.80920E+00_JPRB, 0.83851E+00_JPRB, 0.87233E+00_JPRB, 0.91168E+00_JPRB, 0.96025E+00_JPRB, 0.10051E+01_JPRB ,&
&  0.10453E+01_JPRB, 0.10842E+01_JPRB, 0.11200E+01_JPRB, 0.11475E+01_JPRB, 0.11523E+01_JPRB, 0.11449E+01_JPRB, 0.97975E+00_JPRB ,&
&  0.12161E+01_JPRB, 0.11967E+01_JPRB, 0.11019E+01_JPRB, 0.99905E+00_JPRB, 0.91560E+00_JPRB, 0.84473E+00_JPRB, 0.77285E+00_JPRB ,&
&  0.76327E+00_JPRB, 0.71367E+00_JPRB, 0.68752E+00_JPRB, 0.68117E+00_JPRB, 0.64429E+00_JPRB, 0.62287E+00_JPRB, 0.62868E+00_JPRB ,&
&  0.65436E+00_JPRB, 0.70600E+00_JPRB, 0.71667E+00_JPRB, 0.74097E+00_JPRB, 0.79800E+00_JPRB, 0.76020E+00_JPRB, 0.77282E+00_JPRB ,&
&  0.72031E+00_JPRB, 0.65713E+00_JPRB, 0.56354E+00_JPRB, 0.63170E+00_JPRB, 0.67902E+00_JPRB, 0.70829E+00_JPRB, 0.70832E+00_JPRB ,&
&  0.59409E+00_JPRB, 0.52510E+00_JPRB, 0.56165E+00_JPRB, 0.74234E+00_JPRB, 0.72233E+00_JPRB, 0.65817E+00_JPRB, 0.60920E+00_JPRB ,&
&  0.56230E+00_JPRB, 0.52848E+00_JPRB, 0.49215E+00_JPRB, 0.47745E+00_JPRB, 0.49157E+00_JPRB, 0.48883E+00_JPRB, 0.46739E+00_JPRB ,&
&  0.43547E+00_JPRB, 0.41953E+00_JPRB, 0.41537E+00_JPRB, 0.41028E+00_JPRB, 0.42178E+00_JPRB, 0.38269E+00_JPRB, 0.41978E+00_JPRB ,&
&  0.42813E+00_JPRB, 0.33293E+00_JPRB, 0.39445E+00_JPRB, 0.52473E+00_JPRB, 0.51906E+00_JPRB, 0.57745E+00_JPRB, 0.61197E+00_JPRB ,&
&  0.64450E+00_JPRB, 0.65373E+00_JPRB, 0.60906E+00_JPRB, 0.60494E+00_JPRB, 0.59268E+00_JPRB, 0.57988E+00_JPRB, 0.56515E+00_JPRB ,&
&  0.55185E+00_JPRB, 0.55238E+00_JPRB, 0.56599E+00_JPRB, 0.56578E+00_JPRB, 0.56636E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.58099E+00_JPRB, 0.59273E+00_JPRB, 0.59254E+00_JPRB, 0.59780E+00_JPRB, 0.60945E+00_JPRB, 0.61674E+00_JPRB, 0.62824E+00_JPRB ,&
&  0.63927E+00_JPRB, 0.64212E+00_JPRB, 0.64431E+00_JPRB, 0.64793E+00_JPRB, 0.64861E+00_JPRB, 0.66278E+00_JPRB, 0.67688E+00_JPRB ,&
&  0.69628E+00_JPRB, 0.71077E+00_JPRB, 0.72703E+00_JPRB, 0.75330E+00_JPRB, 0.78671E+00_JPRB, 0.84033E+00_JPRB, 0.89633E+00_JPRB ,&
&  0.96781E+00_JPRB, 0.10520E+01_JPRB, 0.11516E+01_JPRB, 0.12307E+01_JPRB, 0.12698E+01_JPRB, 0.12792E+01_JPRB, 0.13680E+01_JPRB ,&
&  0.11976E+01_JPRB, 0.12802E+01_JPRB, 0.11969E+01_JPRB, 0.10542E+01_JPRB, 0.10313E+01_JPRB, 0.96177E+00_JPRB, 0.93099E+00_JPRB ,&
&  0.92291E+00_JPRB, 0.87269E+00_JPRB, 0.84798E+00_JPRB, 0.82433E+00_JPRB, 0.80146E+00_JPRB, 0.75390E+00_JPRB, 0.70767E+00_JPRB ,&
&  0.69012E+00_JPRB, 0.71000E+00_JPRB, 0.77645E+00_JPRB, 0.92720E+00_JPRB, 0.10800E+01_JPRB, 0.11298E+01_JPRB, 0.10980E+01_JPRB ,&
&  0.10476E+01_JPRB, 0.10101E+01_JPRB, 0.94499E+00_JPRB, 0.91803E+00_JPRB, 0.96150E+00_JPRB, 0.99794E+00_JPRB, 0.98874E+00_JPRB ,&
&  0.84187E+00_JPRB, 0.71401E+00_JPRB, 0.68015E+00_JPRB, 0.74253E+00_JPRB, 0.75214E+00_JPRB, 0.72818E+00_JPRB, 0.64032E+00_JPRB ,&
&  0.55529E+00_JPRB, 0.50642E+00_JPRB, 0.47710E+00_JPRB, 0.47984E+00_JPRB, 0.51979E+00_JPRB, 0.52753E+00_JPRB, 0.51306E+00_JPRB ,&
&  0.47039E+00_JPRB, 0.44319E+00_JPRB, 0.42575E+00_JPRB, 0.41808E+00_JPRB, 0.38750E+00_JPRB, 0.53198E+00_JPRB, 0.47651E+00_JPRB ,&
&  0.47292E+00_JPRB, 0.44637E+00_JPRB, 0.41572E+00_JPRB, 0.56351E+00_JPRB, 0.59767E+00_JPRB, 0.61569E+00_JPRB, 0.61476E+00_JPRB ,&
&  0.60530E+00_JPRB, 0.60364E+00_JPRB, 0.60011E+00_JPRB, 0.58294E+00_JPRB, 0.56978E+00_JPRB, 0.56405E+00_JPRB, 0.58430E+00_JPRB ,&
&  0.57938E+00_JPRB, 0.57866E+00_JPRB, 0.57579E+00_JPRB, 0.58803E+00_JPRB, 0.58520E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.59548E+00_JPRB, 0.60057E+00_JPRB, 0.61024E+00_JPRB, 0.62748E+00_JPRB, 0.62421E+00_JPRB, 0.62095E+00_JPRB, 0.64552E+00_JPRB ,&
&  0.62818E+00_JPRB, 0.61467E+00_JPRB, 0.61021E+00_JPRB, 0.58315E+00_JPRB, 0.59653E+00_JPRB, 0.60842E+00_JPRB, 0.63867E+00_JPRB ,&
&  0.66993E+00_JPRB, 0.68757E+00_JPRB, 0.70306E+00_JPRB, 0.72767E+00_JPRB, 0.77506E+00_JPRB, 0.80974E+00_JPRB, 0.83515E+00_JPRB ,&
&  0.88547E+00_JPRB, 0.96860E+00_JPRB, 0.10631E+01_JPRB, 0.12937E+01_JPRB, 0.18329E+01_JPRB, 0.22524E+01_JPRB, 0.22372E+01_JPRB ,&
&  0.16744E+01_JPRB, 0.11892E+01_JPRB, 0.11289E+01_JPRB, 0.10219E+01_JPRB, 0.10483E+01_JPRB, 0.11392E+01_JPRB, 0.11547E+01_JPRB ,&
&  0.11319E+01_JPRB, 0.11275E+01_JPRB, 0.11197E+01_JPRB, 0.11161E+01_JPRB, 0.11240E+01_JPRB, 0.11262E+01_JPRB, 0.11120E+01_JPRB ,&
&  0.10750E+01_JPRB, 0.10301E+01_JPRB, 0.10596E+01_JPRB, 0.12512E+01_JPRB, 0.13777E+01_JPRB, 0.16122E+01_JPRB, 0.16159E+01_JPRB ,&
&  0.15696E+01_JPRB, 0.14107E+01_JPRB, 0.13598E+01_JPRB, 0.13482E+01_JPRB, 0.13216E+01_JPRB, 0.13032E+01_JPRB, 0.13194E+01_JPRB ,&
&  0.12952E+01_JPRB, 0.11055E+01_JPRB, 0.73241E+00_JPRB, 0.77210E+00_JPRB, 0.78248E+00_JPRB, 0.80216E+00_JPRB, 0.65537E+00_JPRB ,&
&  0.54668E+00_JPRB, 0.51788E+00_JPRB, 0.51300E+00_JPRB, 0.53255E+00_JPRB, 0.57780E+00_JPRB, 0.61294E+00_JPRB, 0.58491E+00_JPRB ,&
&  0.53340E+00_JPRB, 0.47950E+00_JPRB, 0.44764E+00_JPRB, 0.44333E+00_JPRB, 0.46276E+00_JPRB, 0.46581E+00_JPRB, 0.48424E+00_JPRB ,&
&  0.49626E+00_JPRB, 0.56325E+00_JPRB, 0.52549E+00_JPRB, 0.67878E+00_JPRB, 0.71511E+00_JPRB, 0.71081E+00_JPRB, 0.69841E+00_JPRB ,&
&  0.68943E+00_JPRB, 0.67108E+00_JPRB, 0.65396E+00_JPRB, 0.63340E+00_JPRB, 0.61198E+00_JPRB, 0.63229E+00_JPRB, 0.66130E+00_JPRB ,&
&  0.63345E+00_JPRB, 0.64820E+00_JPRB, 0.63356E+00_JPRB, 0.61076E+00_JPRB, 0.61157E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.62558E+00_JPRB, 0.61841E+00_JPRB, 0.63223E+00_JPRB, 0.65416E+00_JPRB, 0.66538E+00_JPRB, 0.67592E+00_JPRB, 0.68661E+00_JPRB ,&
&  0.68925E+00_JPRB, 0.69014E+00_JPRB, 0.69304E+00_JPRB, 0.69261E+00_JPRB, 0.72173E+00_JPRB, 0.75347E+00_JPRB, 0.79331E+00_JPRB ,&
&  0.85333E+00_JPRB, 0.90614E+00_JPRB, 0.95213E+00_JPRB, 0.10419E+01_JPRB, 0.11002E+01_JPRB, 0.11503E+01_JPRB, 0.12400E+01_JPRB ,&
&  0.13593E+01_JPRB, 0.14783E+01_JPRB, 0.15948E+01_JPRB, 0.16997E+01_JPRB, 0.18166E+01_JPRB, 0.19167E+01_JPRB, 0.18948E+01_JPRB ,&
&  0.14074E+01_JPRB, 0.11383E+01_JPRB, 0.12597E+01_JPRB, 0.12086E+01_JPRB, 0.12385E+01_JPRB, 0.12543E+01_JPRB, 0.12508E+01_JPRB ,&
&  0.12666E+01_JPRB, 0.12716E+01_JPRB, 0.12738E+01_JPRB, 0.12797E+01_JPRB, 0.13186E+01_JPRB, 0.13749E+01_JPRB, 0.14523E+01_JPRB ,&
&  0.14857E+01_JPRB, 0.15146E+01_JPRB, 0.15359E+01_JPRB, 0.15213E+01_JPRB, 0.15731E+01_JPRB, 0.16963E+01_JPRB, 0.18062E+01_JPRB ,&
&  0.18613E+01_JPRB, 0.18438E+01_JPRB, 0.16339E+01_JPRB, 0.16098E+01_JPRB, 0.16104E+01_JPRB, 0.15005E+01_JPRB, 0.15399E+01_JPRB ,&
&  0.16108E+01_JPRB, 0.14001E+01_JPRB, 0.85845E+00_JPRB, 0.79269E+00_JPRB, 0.77109E+00_JPRB, 0.73941E+00_JPRB, 0.66565E+00_JPRB ,&
&  0.55899E+00_JPRB, 0.54137E+00_JPRB, 0.55946E+00_JPRB, 0.62380E+00_JPRB, 0.66555E+00_JPRB, 0.69519E+00_JPRB, 0.64734E+00_JPRB ,&
&  0.56932E+00_JPRB, 0.51338E+00_JPRB, 0.46643E+00_JPRB, 0.46676E+00_JPRB, 0.45902E+00_JPRB, 0.50018E+00_JPRB, 0.51038E+00_JPRB ,&
&  0.55787E+00_JPRB, 0.61605E+00_JPRB, 0.69743E+00_JPRB, 0.71715E+00_JPRB, 0.70599E+00_JPRB, 0.81332E+00_JPRB, 0.78759E+00_JPRB ,&
&  0.76606E+00_JPRB, 0.75838E+00_JPRB, 0.74390E+00_JPRB, 0.73202E+00_JPRB, 0.72046E+00_JPRB, 0.71755E+00_JPRB, 0.69967E+00_JPRB ,&
&  0.69255E+00_JPRB, 0.67869E+00_JPRB, 0.67203E+00_JPRB, 0.64826E+00_JPRB, 0.63709E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.64339E+00_JPRB, 0.64518E+00_JPRB, 0.65047E+00_JPRB, 0.66877E+00_JPRB, 0.69226E+00_JPRB, 0.70690E+00_JPRB, 0.71158E+00_JPRB ,&
&  0.72642E+00_JPRB, 0.74673E+00_JPRB, 0.77733E+00_JPRB, 0.80425E+00_JPRB, 0.83939E+00_JPRB, 0.87181E+00_JPRB, 0.92432E+00_JPRB ,&
&  0.97253E+00_JPRB, 0.10310E+01_JPRB, 0.11352E+01_JPRB, 0.12904E+01_JPRB, 0.14190E+01_JPRB, 0.15249E+01_JPRB, 0.16899E+01_JPRB ,&
&  0.18469E+01_JPRB, 0.19090E+01_JPRB, 0.19866E+01_JPRB, 0.19746E+01_JPRB, 0.18213E+01_JPRB, 0.16526E+01_JPRB, 0.16265E+01_JPRB ,&
&  0.13286E+01_JPRB, 0.11402E+01_JPRB, 0.11824E+01_JPRB, 0.12399E+01_JPRB, 0.12740E+01_JPRB, 0.12709E+01_JPRB, 0.12744E+01_JPRB ,&
&  0.12922E+01_JPRB, 0.13012E+01_JPRB, 0.13089E+01_JPRB, 0.13206E+01_JPRB, 0.13503E+01_JPRB, 0.14113E+01_JPRB, 0.15011E+01_JPRB ,&
&  0.15120E+01_JPRB, 0.15430E+01_JPRB, 0.16218E+01_JPRB, 0.14978E+01_JPRB, 0.15852E+01_JPRB, 0.17017E+01_JPRB, 0.17462E+01_JPRB ,&
&  0.17127E+01_JPRB, 0.15543E+01_JPRB, 0.15333E+01_JPRB, 0.15481E+01_JPRB, 0.15559E+01_JPRB, 0.14790E+01_JPRB, 0.14186E+01_JPRB ,&
&  0.15166E+01_JPRB, 0.14745E+01_JPRB, 0.97181E+00_JPRB, 0.82614E+00_JPRB, 0.80993E+00_JPRB, 0.68219E+00_JPRB, 0.67520E+00_JPRB ,&
&  0.60023E+00_JPRB, 0.58270E+00_JPRB, 0.63642E+00_JPRB, 0.67750E+00_JPRB, 0.70972E+00_JPRB, 0.78601E+00_JPRB, 0.64270E+00_JPRB ,&
&  0.59199E+00_JPRB, 0.55108E+00_JPRB, 0.51677E+00_JPRB, 0.51875E+00_JPRB, 0.53647E+00_JPRB, 0.58283E+00_JPRB, 0.61617E+00_JPRB ,&
&  0.64982E+00_JPRB, 0.70988E+00_JPRB, 0.77024E+00_JPRB, 0.82622E+00_JPRB, 0.80291E+00_JPRB, 0.84404E+00_JPRB, 0.81458E+00_JPRB ,&
&  0.78014E+00_JPRB, 0.77276E+00_JPRB, 0.75787E+00_JPRB, 0.73365E+00_JPRB, 0.72020E+00_JPRB, 0.70394E+00_JPRB, 0.69549E+00_JPRB ,&
&  0.69131E+00_JPRB, 0.68693E+00_JPRB, 0.67874E+00_JPRB, 0.67048E+00_JPRB, 0.65472E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.66821E+00_JPRB, 0.66217E+00_JPRB, 0.66250E+00_JPRB, 0.67616E+00_JPRB, 0.69454E+00_JPRB, 0.70144E+00_JPRB, 0.70896E+00_JPRB ,&
&  0.72770E+00_JPRB, 0.75076E+00_JPRB, 0.77496E+00_JPRB, 0.80909E+00_JPRB, 0.84369E+00_JPRB, 0.88029E+00_JPRB, 0.93800E+00_JPRB ,&
&  0.10074E+01_JPRB, 0.11039E+01_JPRB, 0.12385E+01_JPRB, 0.13814E+01_JPRB, 0.15107E+01_JPRB, 0.16564E+01_JPRB, 0.17974E+01_JPRB ,&
&  0.19582E+01_JPRB, 0.19988E+01_JPRB, 0.18945E+01_JPRB, 0.17097E+01_JPRB, 0.13178E+01_JPRB, 0.12997E+01_JPRB, 0.13900E+01_JPRB ,&
&  0.13962E+01_JPRB, 0.12297E+01_JPRB, 0.12097E+01_JPRB, 0.11971E+01_JPRB, 0.12036E+01_JPRB, 0.12234E+01_JPRB, 0.12302E+01_JPRB ,&
&  0.12505E+01_JPRB, 0.12715E+01_JPRB, 0.12891E+01_JPRB, 0.13030E+01_JPRB, 0.13183E+01_JPRB, 0.13553E+01_JPRB, 0.14404E+01_JPRB ,&
&  0.14494E+01_JPRB, 0.14649E+01_JPRB, 0.14790E+01_JPRB, 0.14384E+01_JPRB, 0.14991E+01_JPRB, 0.15718E+01_JPRB, 0.15046E+01_JPRB ,&
&  0.14343E+01_JPRB, 0.12810E+01_JPRB, 0.12440E+01_JPRB, 0.12763E+01_JPRB, 0.12507E+01_JPRB, 0.11348E+01_JPRB, 0.11212E+01_JPRB ,&
&  0.12413E+01_JPRB, 0.14064E+01_JPRB, 0.89398E+00_JPRB, 0.89665E+00_JPRB, 0.98728E+00_JPRB, 0.85874E+00_JPRB, 0.75026E+00_JPRB ,&
&  0.68540E+00_JPRB, 0.68866E+00_JPRB, 0.74612E+00_JPRB, 0.75416E+00_JPRB, 0.73720E+00_JPRB, 0.80967E+00_JPRB, 0.64381E+00_JPRB ,&
&  0.65285E+00_JPRB, 0.61163E+00_JPRB, 0.59196E+00_JPRB, 0.60609E+00_JPRB, 0.63609E+00_JPRB, 0.65252E+00_JPRB, 0.65398E+00_JPRB ,&
&  0.62765E+00_JPRB, 0.89169E+00_JPRB, 0.11355E+01_JPRB, 0.14108E+01_JPRB, 0.14402E+01_JPRB, 0.84738E+00_JPRB, 0.84038E+00_JPRB ,&
&  0.82830E+00_JPRB, 0.80842E+00_JPRB, 0.78443E+00_JPRB, 0.76304E+00_JPRB, 0.74644E+00_JPRB, 0.72890E+00_JPRB, 0.71462E+00_JPRB ,&
&  0.70466E+00_JPRB, 0.69787E+00_JPRB, 0.69345E+00_JPRB, 0.68338E+00_JPRB, 0.67403E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.68811E+00_JPRB, 0.68633E+00_JPRB, 0.67731E+00_JPRB, 0.68013E+00_JPRB, 0.70766E+00_JPRB, 0.72640E+00_JPRB, 0.71189E+00_JPRB ,&
&  0.71204E+00_JPRB, 0.72211E+00_JPRB, 0.73697E+00_JPRB, 0.75797E+00_JPRB, 0.77932E+00_JPRB, 0.79563E+00_JPRB, 0.84165E+00_JPRB ,&
&  0.89798E+00_JPRB, 0.10081E+01_JPRB, 0.11812E+01_JPRB, 0.13559E+01_JPRB, 0.15164E+01_JPRB, 0.17011E+01_JPRB, 0.18799E+01_JPRB ,&
&  0.21060E+01_JPRB, 0.21626E+01_JPRB, 0.19648E+01_JPRB, 0.12944E+01_JPRB, 0.10617E+01_JPRB, 0.11175E+01_JPRB, 0.11346E+01_JPRB ,&
&  0.11356E+01_JPRB, 0.11248E+01_JPRB, 0.11181E+01_JPRB, 0.11133E+01_JPRB, 0.11245E+01_JPRB, 0.11472E+01_JPRB, 0.11509E+01_JPRB ,&
&  0.11628E+01_JPRB, 0.11852E+01_JPRB, 0.12107E+01_JPRB, 0.12402E+01_JPRB, 0.12692E+01_JPRB, 0.13001E+01_JPRB, 0.13314E+01_JPRB ,&
&  0.13366E+01_JPRB, 0.13609E+01_JPRB, 0.13416E+01_JPRB, 0.12963E+01_JPRB, 0.12812E+01_JPRB, 0.12834E+01_JPRB, 0.12477E+01_JPRB ,&
&  0.12144E+01_JPRB, 0.11171E+01_JPRB, 0.10732E+01_JPRB, 0.10906E+01_JPRB, 0.10986E+01_JPRB, 0.99583E+00_JPRB, 0.10016E+01_JPRB ,&
&  0.10827E+01_JPRB, 0.11528E+01_JPRB, 0.93170E+00_JPRB, 0.99941E+00_JPRB, 0.69295E+00_JPRB, 0.79693E+00_JPRB, 0.85895E+00_JPRB ,&
&  0.81784E+00_JPRB, 0.82801E+00_JPRB, 0.85359E+00_JPRB, 0.87546E+00_JPRB, 0.87766E+00_JPRB, 0.79457E+00_JPRB, 0.75303E+00_JPRB ,&
&  0.73417E+00_JPRB, 0.67111E+00_JPRB, 0.61999E+00_JPRB, 0.65270E+00_JPRB, 0.69673E+00_JPRB, 0.68651E+00_JPRB, 0.71531E+00_JPRB ,&
&  0.63216E+00_JPRB, 0.76755E+00_JPRB, 0.92059E+00_JPRB, 0.11079E+01_JPRB, 0.11129E+01_JPRB, 0.84594E+00_JPRB, 0.85081E+00_JPRB ,&
&  0.85448E+00_JPRB, 0.83067E+00_JPRB, 0.80993E+00_JPRB, 0.77835E+00_JPRB, 0.76357E+00_JPRB, 0.74678E+00_JPRB, 0.73015E+00_JPRB ,&
&  0.72250E+00_JPRB, 0.71648E+00_JPRB, 0.71064E+00_JPRB, 0.69682E+00_JPRB, 0.68714E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.70847E+00_JPRB, 0.70915E+00_JPRB, 0.71546E+00_JPRB, 0.71968E+00_JPRB, 0.72515E+00_JPRB, 0.76454E+00_JPRB, 0.78023E+00_JPRB ,&
&  0.74224E+00_JPRB, 0.68770E+00_JPRB, 0.68629E+00_JPRB, 0.68749E+00_JPRB, 0.69190E+00_JPRB, 0.70457E+00_JPRB, 0.73083E+00_JPRB ,&
&  0.75829E+00_JPRB, 0.80882E+00_JPRB, 0.92920E+00_JPRB, 0.11419E+01_JPRB, 0.13831E+01_JPRB, 0.16214E+01_JPRB, 0.18633E+01_JPRB ,&
&  0.17451E+01_JPRB, 0.12694E+01_JPRB, 0.15901E+01_JPRB, 0.15148E+01_JPRB, 0.12744E+01_JPRB, 0.10919E+01_JPRB, 0.10583E+01_JPRB ,&
&  0.10967E+01_JPRB, 0.10924E+01_JPRB, 0.10792E+01_JPRB, 0.10694E+01_JPRB, 0.10388E+01_JPRB, 0.10506E+01_JPRB, 0.10701E+01_JPRB ,&
&  0.10799E+01_JPRB, 0.10903E+01_JPRB, 0.11023E+01_JPRB, 0.11241E+01_JPRB, 0.11734E+01_JPRB, 0.12326E+01_JPRB, 0.12626E+01_JPRB ,&
&  0.12600E+01_JPRB, 0.12478E+01_JPRB, 0.11771E+01_JPRB, 0.11234E+01_JPRB, 0.11078E+01_JPRB, 0.11087E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.11161E+01_JPRB, 0.10330E+01_JPRB, 0.10294E+01_JPRB, 0.10522E+01_JPRB, 0.10614E+01_JPRB, 0.99257E+00_JPRB, 0.10214E+01_JPRB ,&
&  0.10287E+01_JPRB, 0.10350E+01_JPRB, 0.98424E+00_JPRB, 0.10186E+01_JPRB, 0.77304E+00_JPRB, 0.83801E+00_JPRB, 0.79548E+00_JPRB ,&
&  0.88503E+00_JPRB, 0.89745E+00_JPRB, 0.91346E+00_JPRB, 0.93788E+00_JPRB, 0.99272E+00_JPRB, 0.90710E+00_JPRB, 0.86733E+00_JPRB ,&
&  0.77169E+00_JPRB, 0.73414E+00_JPRB, 0.67315E+00_JPRB, 0.72440E+00_JPRB, 0.73674E+00_JPRB, 0.72149E+00_JPRB, 0.78881E+00_JPRB ,&
&  0.82749E+00_JPRB, 0.77315E+00_JPRB, 0.78126E+00_JPRB, 0.87122E+00_JPRB, 0.90582E+00_JPRB, 0.82312E+00_JPRB, 0.82610E+00_JPRB ,&
&  0.83016E+00_JPRB, 0.83723E+00_JPRB, 0.81073E+00_JPRB, 0.78146E+00_JPRB, 0.75682E+00_JPRB, 0.73878E+00_JPRB, 0.73691E+00_JPRB ,&
&  0.72879E+00_JPRB, 0.72241E+00_JPRB, 0.71761E+00_JPRB, 0.71127E+00_JPRB, 0.70752E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.70206E+00_JPRB, 0.70112E+00_JPRB, 0.70144E+00_JPRB, 0.71947E+00_JPRB, 0.76926E+00_JPRB, 0.82502E+00_JPRB, 0.96581E+00_JPRB ,&
&  0.11859E+01_JPRB, 0.76302E+00_JPRB, 0.66254E+00_JPRB, 0.64339E+00_JPRB, 0.63247E+00_JPRB, 0.62762E+00_JPRB, 0.64480E+00_JPRB ,&
&  0.66630E+00_JPRB, 0.69085E+00_JPRB, 0.74153E+00_JPRB, 0.85536E+00_JPRB, 0.10725E+01_JPRB, 0.13955E+01_JPRB, 0.14585E+01_JPRB ,&
&  0.12379E+01_JPRB, 0.14303E+01_JPRB, 0.17574E+01_JPRB, 0.14674E+01_JPRB, 0.13200E+01_JPRB, 0.11880E+01_JPRB, 0.11171E+01_JPRB ,&
&  0.10777E+01_JPRB, 0.10437E+01_JPRB, 0.10476E+01_JPRB, 0.10358E+01_JPRB, 0.10109E+01_JPRB, 0.10096E+01_JPRB, 0.10256E+01_JPRB ,&
&  0.10390E+01_JPRB, 0.10420E+01_JPRB, 0.10428E+01_JPRB, 0.10474E+01_JPRB, 0.10681E+01_JPRB, 0.11202E+01_JPRB, 0.11640E+01_JPRB ,&
&  0.11926E+01_JPRB, 0.11892E+01_JPRB, 0.11180E+01_JPRB, 0.10566E+01_JPRB, 0.10444E+01_JPRB, 0.10562E+01_JPRB, 0.10590E+01_JPRB ,&
&  0.10701E+01_JPRB, 0.10336E+01_JPRB, 0.10270E+01_JPRB, 0.10153E+01_JPRB, 0.96835E+00_JPRB, 0.10297E+01_JPRB, 0.10039E+01_JPRB ,&
&  0.10209E+01_JPRB, 0.99728E+00_JPRB, 0.10684E+01_JPRB, 0.88590E+00_JPRB, 0.87078E+00_JPRB, 0.90928E+00_JPRB, 0.89716E+00_JPRB ,&
&  0.84850E+00_JPRB, 0.90220E+00_JPRB, 0.96279E+00_JPRB, 0.97984E+00_JPRB, 0.10043E+01_JPRB, 0.94500E+00_JPRB, 0.96519E+00_JPRB ,&
&  0.81858E+00_JPRB, 0.86430E+00_JPRB, 0.75831E+00_JPRB, 0.78160E+00_JPRB, 0.77378E+00_JPRB, 0.73930E+00_JPRB, 0.79433E+00_JPRB ,&
&  0.86144E+00_JPRB, 0.83134E+00_JPRB, 0.80267E+00_JPRB, 0.79465E+00_JPRB, 0.77636E+00_JPRB, 0.76758E+00_JPRB, 0.77712E+00_JPRB ,&
&  0.78975E+00_JPRB, 0.82489E+00_JPRB, 0.80729E+00_JPRB, 0.79254E+00_JPRB, 0.77263E+00_JPRB, 0.75380E+00_JPRB, 0.74397E+00_JPRB ,&
&  0.73697E+00_JPRB, 0.72862E+00_JPRB, 0.72146E+00_JPRB, 0.71395E+00_JPRB, 0.71006E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.68605E+00_JPRB, 0.67195E+00_JPRB, 0.65396E+00_JPRB, 0.64100E+00_JPRB, 0.63786E+00_JPRB, 0.66720E+00_JPRB, 0.73723E+00_JPRB ,&
&  0.10224E+01_JPRB, 0.76328E+00_JPRB, 0.66943E+00_JPRB, 0.63860E+00_JPRB, 0.62478E+00_JPRB, 0.59983E+00_JPRB, 0.58781E+00_JPRB ,&
&  0.59272E+00_JPRB, 0.60542E+00_JPRB, 0.63773E+00_JPRB, 0.68853E+00_JPRB, 0.78791E+00_JPRB, 0.10078E+01_JPRB, 0.10945E+01_JPRB ,&
&  0.10478E+01_JPRB, 0.19573E+01_JPRB, 0.16959E+01_JPRB, 0.15800E+01_JPRB, 0.14016E+01_JPRB, 0.13054E+01_JPRB, 0.11399E+01_JPRB ,&
&  0.10639E+01_JPRB, 0.10484E+01_JPRB, 0.10623E+01_JPRB, 0.10679E+01_JPRB, 0.10647E+01_JPRB, 0.10566E+01_JPRB, 0.10572E+01_JPRB ,&
&  0.10489E+01_JPRB, 0.10418E+01_JPRB, 0.10413E+01_JPRB, 0.10410E+01_JPRB, 0.10420E+01_JPRB, 0.10680E+01_JPRB, 0.10906E+01_JPRB ,&
&  0.11218E+01_JPRB, 0.11684E+01_JPRB, 0.11131E+01_JPRB, 0.10602E+01_JPRB, 0.10420E+01_JPRB, 0.10573E+01_JPRB, 0.10662E+01_JPRB ,&
&  0.10232E+01_JPRB, 0.96883E+00_JPRB, 0.10087E+01_JPRB, 0.10293E+01_JPRB, 0.10303E+01_JPRB, 0.10812E+01_JPRB, 0.10429E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.10182E+01_JPRB, 0.10769E+01_JPRB, 0.92042E+00_JPRB, 0.89622E+00_JPRB, 0.95133E+00_JPRB, 0.93676E+00_JPRB ,&
&  0.85726E+00_JPRB, 0.89012E+00_JPRB, 0.93891E+00_JPRB, 0.98579E+00_JPRB, 0.97916E+00_JPRB, 0.95442E+00_JPRB, 0.92933E+00_JPRB ,&
&  0.88353E+00_JPRB, 0.92364E+00_JPRB, 0.87405E+00_JPRB, 0.77213E+00_JPRB, 0.77434E+00_JPRB, 0.69700E+00_JPRB, 0.77287E+00_JPRB ,&
&  0.84080E+00_JPRB, 0.78926E+00_JPRB, 0.80697E+00_JPRB, 0.83989E+00_JPRB, 0.78233E+00_JPRB, 0.75443E+00_JPRB, 0.74942E+00_JPRB ,&
&  0.77554E+00_JPRB, 0.79808E+00_JPRB, 0.78888E+00_JPRB, 0.76746E+00_JPRB, 0.75381E+00_JPRB, 0.74028E+00_JPRB, 0.72055E+00_JPRB ,&
&  0.70902E+00_JPRB, 0.70196E+00_JPRB, 0.69732E+00_JPRB, 0.69398E+00_JPRB, 0.69708E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.69811E+00_JPRB, 0.69601E+00_JPRB, 0.66756E+00_JPRB, 0.63692E+00_JPRB, 0.62125E+00_JPRB, 0.62767E+00_JPRB, 0.65654E+00_JPRB ,&
&  0.86321E+00_JPRB, 0.75562E+00_JPRB, 0.67434E+00_JPRB, 0.63510E+00_JPRB, 0.61262E+00_JPRB, 0.59577E+00_JPRB, 0.57777E+00_JPRB ,&
&  0.56623E+00_JPRB, 0.56607E+00_JPRB, 0.59685E+00_JPRB, 0.61825E+00_JPRB, 0.66043E+00_JPRB, 0.76563E+00_JPRB, 0.69693E+00_JPRB ,&
&  0.98387E+00_JPRB, 0.18706E+01_JPRB, 0.17709E+01_JPRB, 0.17186E+01_JPRB, 0.15764E+01_JPRB, 0.14998E+01_JPRB, 0.12119E+01_JPRB ,&
&  0.11102E+01_JPRB, 0.11062E+01_JPRB, 0.11130E+01_JPRB, 0.11275E+01_JPRB, 0.11431E+01_JPRB, 0.11487E+01_JPRB, 0.11225E+01_JPRB ,&
&  0.10915E+01_JPRB, 0.10587E+01_JPRB, 0.10550E+01_JPRB, 0.10495E+01_JPRB, 0.10460E+01_JPRB, 0.10592E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.10723E+01_JPRB, 0.11128E+01_JPRB, 0.11387E+01_JPRB, 0.10506E+01_JPRB, 0.10369E+01_JPRB, 0.10428E+01_JPRB, 0.10544E+01_JPRB ,&
&  0.99140E+00_JPRB, 0.93615E+00_JPRB, 0.10075E+01_JPRB, 0.10883E+01_JPRB, 0.11425E+01_JPRB, 0.11577E+01_JPRB, 0.11356E+01_JPRB ,&
&  0.11479E+01_JPRB, 0.10831E+01_JPRB, 0.10299E+01_JPRB, 0.93942E+00_JPRB, 0.94591E+00_JPRB, 0.98169E+00_JPRB, 0.99925E+00_JPRB ,&
&  0.98444E+00_JPRB, 0.92728E+00_JPRB, 0.92586E+00_JPRB, 0.92968E+00_JPRB, 0.94808E+00_JPRB, 0.92679E+00_JPRB, 0.98429E+00_JPRB ,&
&  0.10714E+01_JPRB, 0.11813E+01_JPRB, 0.10995E+01_JPRB, 0.93312E+00_JPRB, 0.77468E+00_JPRB, 0.58898E+00_JPRB, 0.63510E+00_JPRB ,&
&  0.74953E+00_JPRB, 0.72657E+00_JPRB, 0.70619E+00_JPRB, 0.84231E+00_JPRB, 0.79917E+00_JPRB, 0.79108E+00_JPRB, 0.77134E+00_JPRB ,&
&  0.75243E+00_JPRB, 0.77068E+00_JPRB, 0.78224E+00_JPRB, 0.77174E+00_JPRB, 0.75252E+00_JPRB, 0.73453E+00_JPRB, 0.70733E+00_JPRB ,&
&  0.68325E+00_JPRB, 0.67388E+00_JPRB, 0.67653E+00_JPRB, 0.68592E+00_JPRB, 0.69687E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.68851E+00_JPRB, 0.69468E+00_JPRB, 0.66576E+00_JPRB, 0.62438E+00_JPRB, 0.60127E+00_JPRB, 0.59646E+00_JPRB, 0.60318E+00_JPRB ,&
&  0.68490E+00_JPRB, 0.68960E+00_JPRB, 0.65845E+00_JPRB, 0.62583E+00_JPRB, 0.61170E+00_JPRB, 0.59079E+00_JPRB, 0.57626E+00_JPRB ,&
&  0.55346E+00_JPRB, 0.54540E+00_JPRB, 0.59161E+00_JPRB, 0.60938E+00_JPRB, 0.63188E+00_JPRB, 0.54526E+00_JPRB, 0.64037E+00_JPRB ,&
&  0.11423E+01_JPRB, 0.19230E+01_JPRB, 0.18645E+01_JPRB, 0.19011E+01_JPRB, 0.18675E+01_JPRB, 0.18797E+01_JPRB, 0.13543E+01_JPRB ,&
&  0.12085E+01_JPRB, 0.11930E+01_JPRB, 0.11781E+01_JPRB, 0.12167E+01_JPRB, 0.12305E+01_JPRB, 0.12287E+01_JPRB, 0.12163E+01_JPRB ,&
&  0.11690E+01_JPRB, 0.11060E+01_JPRB, 0.10763E+01_JPRB, 0.10636E+01_JPRB, 0.10371E+01_JPRB, 0.10228E+01_JPRB, 0.10428E+01_JPRB ,&
&  0.10621E+01_JPRB, 0.10707E+01_JPRB, 0.11021E+01_JPRB, 0.10515E+01_JPRB, 0.99568E+00_JPRB, 0.98666E+00_JPRB, 0.10164E+01_JPRB ,&
&  0.10104E+01_JPRB, 0.10265E+01_JPRB, 0.10522E+01_JPRB, 0.10752E+01_JPRB, 0.11552E+01_JPRB, 0.12322E+01_JPRB, 0.12661E+01_JPRB ,&
&  0.12683E+01_JPRB, 0.11767E+01_JPRB, 0.10273E+01_JPRB, 0.10094E+01_JPRB, 0.10644E+01_JPRB, 0.10804E+01_JPRB, 0.98988E+00_JPRB ,&
&  0.90223E+00_JPRB, 0.80549E+00_JPRB, 0.79198E+00_JPRB, 0.84400E+00_JPRB, 0.91893E+00_JPRB, 0.96290E+00_JPRB, 0.11242E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.72304E+00_JPRB, 0.63748E+00_JPRB, 0.76739E+00_JPRB, 0.61381E+00_JPRB, 0.49082E+00_JPRB, 0.60337E+00_JPRB ,&
&  0.69501E+00_JPRB, 0.74012E+00_JPRB, 0.73156E+00_JPRB, 0.71588E+00_JPRB, 0.79899E+00_JPRB, 0.85643E+00_JPRB, 0.87692E+00_JPRB ,&
&  0.84092E+00_JPRB, 0.81545E+00_JPRB, 0.78370E+00_JPRB, 0.76236E+00_JPRB, 0.75214E+00_JPRB, 0.73894E+00_JPRB, 0.70768E+00_JPRB ,&
&  0.68705E+00_JPRB, 0.67103E+00_JPRB, 0.66470E+00_JPRB, 0.67607E+00_JPRB, 0.68518E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.68351E+00_JPRB, 0.67491E+00_JPRB, 0.64546E+00_JPRB, 0.60635E+00_JPRB, 0.58595E+00_JPRB, 0.57634E+00_JPRB, 0.57226E+00_JPRB ,&
&  0.59668E+00_JPRB, 0.61835E+00_JPRB, 0.62374E+00_JPRB, 0.61473E+00_JPRB, 0.60530E+00_JPRB, 0.58808E+00_JPRB, 0.57224E+00_JPRB ,&
&  0.55149E+00_JPRB, 0.53399E+00_JPRB, 0.60080E+00_JPRB, 0.65030E+00_JPRB, 0.64421E+00_JPRB, 0.57243E+00_JPRB, 0.72444E+00_JPRB ,&
&  0.11446E+01_JPRB, 0.18804E+01_JPRB, 0.22334E+01_JPRB, 0.22806E+01_JPRB, 0.21094E+01_JPRB, 0.20339E+01_JPRB, 0.15971E+01_JPRB ,&
&  0.13929E+01_JPRB, 0.13561E+01_JPRB, 0.12981E+01_JPRB, 0.13022E+01_JPRB, 0.13268E+01_JPRB, 0.13158E+01_JPRB, 0.12729E+01_JPRB ,&
&  0.12303E+01_JPRB, 0.11631E+01_JPRB, 0.11179E+01_JPRB, 0.10953E+01_JPRB, 0.10614E+01_JPRB, 0.10171E+01_JPRB, 0.10023E+01_JPRB ,&
&  0.10299E+01_JPRB, 0.10408E+01_JPRB, 0.10656E+01_JPRB, 0.10864E+01_JPRB, 0.88834E+00_JPRB, 0.94407E+00_JPRB, 0.96986E+00_JPRB ,&
&  0.10049E+01_JPRB, 0.10472E+01_JPRB, 0.10360E+01_JPRB, 0.11067E+01_JPRB, 0.11980E+01_JPRB, 0.12321E+01_JPRB, 0.13034E+01_JPRB ,&
&  0.13844E+01_JPRB, 0.13012E+01_JPRB, 0.11440E+01_JPRB, 0.11364E+01_JPRB, 0.12021E+01_JPRB, 0.11834E+01_JPRB, 0.85960E+00_JPRB ,&
&  0.80359E+00_JPRB, 0.90454E+00_JPRB, 0.85311E+00_JPRB, 0.76930E+00_JPRB, 0.86638E+00_JPRB, 0.95217E+00_JPRB, 0.85565E+00_JPRB ,&
&  0.49444E+00_JPRB, 0.41133E+00_JPRB, 0.41430E+00_JPRB, 0.42306E+00_JPRB, 0.41563E+00_JPRB, 0.43992E+00_JPRB, 0.70644E+00_JPRB ,&
&  0.68221E+00_JPRB, 0.73771E+00_JPRB, 0.73820E+00_JPRB, 0.72043E+00_JPRB, 0.77859E+00_JPRB, 0.86506E+00_JPRB, 0.10939E+01_JPRB ,&
&  0.98638E+00_JPRB, 0.93050E+00_JPRB, 0.88988E+00_JPRB, 0.82817E+00_JPRB, 0.77769E+00_JPRB, 0.73907E+00_JPRB, 0.71023E+00_JPRB ,&
&  0.68768E+00_JPRB, 0.67258E+00_JPRB, 0.66774E+00_JPRB, 0.67344E+00_JPRB, 0.68259E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.68304E+00_JPRB, 0.65565E+00_JPRB, 0.61722E+00_JPRB, 0.57624E+00_JPRB, 0.55966E+00_JPRB, 0.54830E+00_JPRB, 0.54076E+00_JPRB ,&
&  0.55131E+00_JPRB, 0.56622E+00_JPRB, 0.58193E+00_JPRB, 0.58844E+00_JPRB, 0.58485E+00_JPRB, 0.57357E+00_JPRB, 0.55903E+00_JPRB ,&
&  0.54039E+00_JPRB, 0.52755E+00_JPRB, 0.64284E+00_JPRB, 0.66159E+00_JPRB, 0.69133E+00_JPRB, 0.64861E+00_JPRB, 0.76332E+00_JPRB ,&
&  0.11010E+01_JPRB, 0.17449E+01_JPRB, 0.22623E+01_JPRB, 0.23541E+01_JPRB, 0.24424E+01_JPRB, 0.24111E+01_JPRB, 0.21788E+01_JPRB ,&
&  0.19033E+01_JPRB, 0.17341E+01_JPRB, 0.15795E+01_JPRB, 0.14816E+01_JPRB, 0.14082E+01_JPRB, 0.13660E+01_JPRB, 0.13056E+01_JPRB ,&
&  0.12513E+01_JPRB, 0.12217E+01_JPRB, 0.11885E+01_JPRB, 0.11699E+01_JPRB, 0.11250E+01_JPRB, 0.10692E+01_JPRB, 0.10267E+01_JPRB ,&
&  0.10236E+01_JPRB, 0.10327E+01_JPRB, 0.10425E+01_JPRB, 0.10776E+01_JPRB, 0.95962E+00_JPRB, 0.87083E+00_JPRB, 0.94224E+00_JPRB ,&
&  0.10229E+01_JPRB, 0.11211E+01_JPRB, 0.11232E+01_JPRB, 0.12122E+01_JPRB, 0.12446E+01_JPRB, 0.12558E+01_JPRB, 0.13796E+01_JPRB ,&
&  0.14881E+01_JPRB, 0.15800E+01_JPRB, 0.13038E+01_JPRB, 0.12827E+01_JPRB, 0.13643E+01_JPRB, 0.10813E+01_JPRB, 0.94226E+00_JPRB ,&
&  0.10134E+01_JPRB, 0.92289E+00_JPRB, 0.85120E+00_JPRB, 0.72845E+00_JPRB, 0.82340E+00_JPRB, 0.78797E+00_JPRB, 0.49796E+00_JPRB ,&
&  0.41592E+00_JPRB, 0.39016E+00_JPRB, 0.37491E+00_JPRB, 0.37510E+00_JPRB, 0.37104E+00_JPRB, 0.41154E+00_JPRB, 0.58149E+00_JPRB ,&
&  0.63281E+00_JPRB, 0.70015E+00_JPRB, 0.72975E+00_JPRB, 0.73576E+00_JPRB, 0.75639E+00_JPRB, 0.79413E+00_JPRB, 0.10965E+01_JPRB ,&
&  0.12152E+01_JPRB, 0.11578E+01_JPRB, 0.10334E+01_JPRB, 0.95298E+00_JPRB, 0.91303E+00_JPRB, 0.85136E+00_JPRB, 0.78392E+00_JPRB ,&
&  0.72457E+00_JPRB, 0.69369E+00_JPRB, 0.68095E+00_JPRB, 0.68503E+00_JPRB, 0.69966E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.69119E+00_JPRB, 0.66925E+00_JPRB, 0.62368E+00_JPRB, 0.57431E+00_JPRB, 0.54881E+00_JPRB, 0.52804E+00_JPRB, 0.51630E+00_JPRB ,&
&  0.52064E+00_JPRB, 0.53048E+00_JPRB, 0.54035E+00_JPRB, 0.54868E+00_JPRB, 0.54695E+00_JPRB, 0.54072E+00_JPRB, 0.53367E+00_JPRB ,&
&  0.51813E+00_JPRB, 0.53800E+00_JPRB, 0.60787E+00_JPRB, 0.56645E+00_JPRB, 0.57409E+00_JPRB, 0.58111E+00_JPRB, 0.66680E+00_JPRB ,&
&  0.10517E+01_JPRB, 0.16513E+01_JPRB, 0.21852E+01_JPRB, 0.24889E+01_JPRB, 0.27374E+01_JPRB, 0.26578E+01_JPRB, 0.28728E+01_JPRB ,&
&  0.25751E+01_JPRB, 0.22512E+01_JPRB, 0.19691E+01_JPRB, 0.17707E+01_JPRB, 0.15824E+01_JPRB, 0.14573E+01_JPRB, 0.13929E+01_JPRB ,&
&  0.13114E+01_JPRB, 0.12523E+01_JPRB, 0.12287E+01_JPRB, 0.12049E+01_JPRB, 0.11697E+01_JPRB, 0.11298E+01_JPRB, 0.10932E+01_JPRB ,&
&  0.10815E+01_JPRB, 0.10775E+01_JPRB, 0.10691E+01_JPRB, 0.10574E+01_JPRB, 0.10991E+01_JPRB, 0.10517E+01_JPRB, 0.10143E+01_JPRB ,&
&  0.10120E+01_JPRB, 0.10514E+01_JPRB, 0.12011E+01_JPRB, 0.12124E+01_JPRB, 0.12488E+01_JPRB, 0.13276E+01_JPRB, 0.14355E+01_JPRB ,&
&  0.15643E+01_JPRB, 0.16874E+01_JPRB, 0.14808E+01_JPRB, 0.14822E+01_JPRB, 0.13590E+01_JPRB, 0.10940E+01_JPRB, 0.11085E+01_JPRB ,&
&  0.11601E+01_JPRB, 0.10329E+01_JPRB, 0.87546E+00_JPRB, 0.70623E+00_JPRB, 0.68492E+00_JPRB, 0.54927E+00_JPRB, 0.40720E+00_JPRB ,&
&  0.39386E+00_JPRB, 0.38414E+00_JPRB, 0.36595E+00_JPRB, 0.39526E+00_JPRB, 0.39014E+00_JPRB, 0.43791E+00_JPRB, 0.56093E+00_JPRB ,&
&  0.64924E+00_JPRB, 0.65361E+00_JPRB, 0.69229E+00_JPRB, 0.72151E+00_JPRB, 0.73873E+00_JPRB, 0.75657E+00_JPRB, 0.10206E+01_JPRB ,&
&  0.10480E+01_JPRB, 0.10927E+01_JPRB, 0.10635E+01_JPRB, 0.96783E+00_JPRB, 0.93427E+00_JPRB, 0.91493E+00_JPRB, 0.87261E+00_JPRB ,&
&  0.80747E+00_JPRB, 0.74241E+00_JPRB, 0.71476E+00_JPRB, 0.70964E+00_JPRB, 0.70665E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.70967E+00_JPRB, 0.69452E+00_JPRB, 0.65144E+00_JPRB, 0.60075E+00_JPRB, 0.56330E+00_JPRB, 0.53552E+00_JPRB, 0.52151E+00_JPRB ,&
&  0.52505E+00_JPRB, 0.52727E+00_JPRB, 0.53109E+00_JPRB, 0.53244E+00_JPRB, 0.52827E+00_JPRB, 0.52003E+00_JPRB, 0.51895E+00_JPRB ,&
&  0.50644E+00_JPRB, 0.63056E+00_JPRB, 0.47510E+00_JPRB, 0.41365E+00_JPRB, 0.47453E+00_JPRB, 0.54892E+00_JPRB, 0.56971E+00_JPRB ,&
&  0.96455E+00_JPRB, 0.15708E+01_JPRB, 0.20664E+01_JPRB, 0.26930E+01_JPRB, 0.35743E+01_JPRB, 0.34051E+01_JPRB, 0.33919E+01_JPRB ,&
&  0.32184E+01_JPRB, 0.28321E+01_JPRB, 0.24440E+01_JPRB, 0.20751E+01_JPRB, 0.17574E+01_JPRB, 0.15690E+01_JPRB, 0.14583E+01_JPRB ,&
&  0.13591E+01_JPRB, 0.12831E+01_JPRB, 0.12394E+01_JPRB, 0.12243E+01_JPRB, 0.11888E+01_JPRB, 0.11577E+01_JPRB, 0.11407E+01_JPRB ,&
&  0.11342E+01_JPRB, 0.11277E+01_JPRB, 0.11145E+01_JPRB, 0.11188E+01_JPRB, 0.10635E+01_JPRB, 0.10191E+01_JPRB, 0.12128E+01_JPRB ,&
&  0.12161E+01_JPRB, 0.11993E+01_JPRB, 0.11863E+01_JPRB, 0.12941E+01_JPRB, 0.13602E+01_JPRB, 0.14065E+01_JPRB, 0.15602E+01_JPRB ,&
&  0.14891E+01_JPRB, 0.14693E+01_JPRB, 0.14788E+01_JPRB, 0.14516E+01_JPRB, 0.13080E+01_JPRB, 0.13499E+01_JPRB, 0.16122E+01_JPRB ,&
&  0.13118E+01_JPRB, 0.12804E+01_JPRB, 0.12383E+01_JPRB, 0.10995E+01_JPRB, 0.67597E+00_JPRB, 0.53691E+00_JPRB, 0.70134E+00_JPRB ,&
&  0.73419E+00_JPRB, 0.53120E+00_JPRB, 0.44823E+00_JPRB, 0.56074E+00_JPRB, 0.47482E+00_JPRB, 0.51185E+00_JPRB, 0.62545E+00_JPRB ,&
&  0.62481E+00_JPRB, 0.58032E+00_JPRB, 0.65821E+00_JPRB, 0.73575E+00_JPRB, 0.78697E+00_JPRB, 0.75946E+00_JPRB, 0.92247E+00_JPRB ,&
&  0.99285E+00_JPRB, 0.10067E+01_JPRB, 0.10576E+01_JPRB, 0.99017E+00_JPRB, 0.93468E+00_JPRB, 0.90242E+00_JPRB, 0.87437E+00_JPRB ,&
&  0.83333E+00_JPRB, 0.78413E+00_JPRB, 0.75459E+00_JPRB, 0.74522E+00_JPRB, 0.72622E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.72055E+00_JPRB, 0.70658E+00_JPRB, 0.66328E+00_JPRB, 0.61853E+00_JPRB, 0.56561E+00_JPRB, 0.52309E+00_JPRB, 0.51094E+00_JPRB ,&
&  0.51874E+00_JPRB, 0.52763E+00_JPRB, 0.53033E+00_JPRB, 0.52659E+00_JPRB, 0.51949E+00_JPRB, 0.51208E+00_JPRB, 0.50672E+00_JPRB ,&
&  0.49794E+00_JPRB, 0.48172E+00_JPRB, 0.37713E+00_JPRB, 0.34151E+00_JPRB, 0.46730E+00_JPRB, 0.49995E+00_JPRB, 0.56411E+00_JPRB ,&
&  0.84852E+00_JPRB, 0.13345E+01_JPRB, 0.19215E+01_JPRB, 0.26128E+01_JPRB, 0.33067E+01_JPRB, 0.36355E+01_JPRB, 0.36116E+01_JPRB ,&
&  0.36476E+01_JPRB, 0.32552E+01_JPRB, 0.27368E+01_JPRB, 0.23080E+01_JPRB, 0.20119E+01_JPRB, 0.18111E+01_JPRB, 0.16131E+01_JPRB ,&
&  0.14396E+01_JPRB, 0.13248E+01_JPRB, 0.12624E+01_JPRB, 0.12204E+01_JPRB, 0.11962E+01_JPRB, 0.11593E+01_JPRB, 0.11249E+01_JPRB ,&
&  0.11332E+01_JPRB, 0.11605E+01_JPRB, 0.11811E+01_JPRB, 0.11755E+01_JPRB, 0.10245E+01_JPRB, 0.98088E+00_JPRB, 0.11407E+01_JPRB ,&
&  0.12803E+01_JPRB, 0.12337E+01_JPRB, 0.12413E+01_JPRB, 0.13174E+01_JPRB, 0.14116E+01_JPRB, 0.14050E+01_JPRB, 0.16341E+01_JPRB ,&
&  0.15818E+01_JPRB, 0.14868E+01_JPRB, 0.14016E+01_JPRB, 0.12213E+01_JPRB, 0.12386E+01_JPRB, 0.16396E+01_JPRB, 0.17037E+01_JPRB ,&
&  0.15937E+01_JPRB, 0.14885E+01_JPRB, 0.13614E+01_JPRB, 0.11455E+01_JPRB, 0.89551E+00_JPRB, 0.71533E+00_JPRB, 0.81591E+00_JPRB ,&
&  0.87238E+00_JPRB, 0.88369E+00_JPRB, 0.83869E+00_JPRB, 0.74477E+00_JPRB, 0.65361E+00_JPRB, 0.72247E+00_JPRB, 0.66591E+00_JPRB ,&
&  0.56597E+00_JPRB, 0.51915E+00_JPRB, 0.58104E+00_JPRB, 0.69243E+00_JPRB, 0.74457E+00_JPRB, 0.71148E+00_JPRB, 0.86015E+00_JPRB ,&
&  0.90675E+00_JPRB, 0.95538E+00_JPRB, 0.96672E+00_JPRB, 0.93484E+00_JPRB, 0.89784E+00_JPRB, 0.86685E+00_JPRB, 0.82910E+00_JPRB ,&
&  0.80390E+00_JPRB, 0.78591E+00_JPRB, 0.76489E+00_JPRB, 0.75309E+00_JPRB, 0.73261E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.73620E+00_JPRB, 0.71515E+00_JPRB, 0.66945E+00_JPRB, 0.62263E+00_JPRB, 0.56450E+00_JPRB, 0.50841E+00_JPRB, 0.49300E+00_JPRB ,&
&  0.49998E+00_JPRB, 0.50881E+00_JPRB, 0.50938E+00_JPRB, 0.50255E+00_JPRB, 0.49819E+00_JPRB, 0.49162E+00_JPRB, 0.48822E+00_JPRB ,&
&  0.48693E+00_JPRB, 0.43826E+00_JPRB, 0.34320E+00_JPRB, 0.34157E+00_JPRB, 0.40881E+00_JPRB, 0.45273E+00_JPRB, 0.56431E+00_JPRB ,&
&  0.73841E+00_JPRB, 0.11803E+01_JPRB, 0.16135E+01_JPRB, 0.21418E+01_JPRB, 0.26981E+01_JPRB, 0.31188E+01_JPRB, 0.31441E+01_JPRB ,&
&  0.29470E+01_JPRB, 0.29270E+01_JPRB, 0.26960E+01_JPRB, 0.23933E+01_JPRB, 0.21264E+01_JPRB, 0.19225E+01_JPRB, 0.17401E+01_JPRB ,&
&  0.15659E+01_JPRB, 0.14019E+01_JPRB, 0.13038E+01_JPRB, 0.12424E+01_JPRB, 0.12010E+01_JPRB, 0.11661E+01_JPRB, 0.11396E+01_JPRB ,&
&  0.11364E+01_JPRB, 0.11617E+01_JPRB, 0.11768E+01_JPRB, 0.11924E+01_JPRB, 0.10175E+01_JPRB, 0.10553E+01_JPRB, 0.12202E+01_JPRB ,&
&  0.15278E+01_JPRB, 0.14253E+01_JPRB, 0.14092E+01_JPRB, 0.15956E+01_JPRB, 0.14555E+01_JPRB, 0.14932E+01_JPRB, 0.17980E+01_JPRB ,&
&  0.20568E+01_JPRB, 0.20829E+01_JPRB, 0.20672E+01_JPRB, 0.16244E+01_JPRB, 0.13397E+01_JPRB, 0.18374E+01_JPRB, 0.17682E+01_JPRB ,&
&  0.16412E+01_JPRB, 0.15388E+01_JPRB, 0.14153E+01_JPRB, 0.13349E+01_JPRB, 0.10913E+01_JPRB, 0.93345E+00_JPRB, 0.77330E+00_JPRB ,&
&  0.81070E+00_JPRB, 0.78217E+00_JPRB, 0.85273E+00_JPRB, 0.80789E+00_JPRB, 0.75194E+00_JPRB, 0.68376E+00_JPRB, 0.57614E+00_JPRB ,&
&  0.50986E+00_JPRB, 0.46560E+00_JPRB, 0.46387E+00_JPRB, 0.54491E+00_JPRB, 0.61611E+00_JPRB, 0.59487E+00_JPRB, 0.70176E+00_JPRB ,&
&  0.81448E+00_JPRB, 0.89916E+00_JPRB, 0.87309E+00_JPRB, 0.89852E+00_JPRB, 0.89148E+00_JPRB, 0.87469E+00_JPRB, 0.83727E+00_JPRB ,&
&  0.80371E+00_JPRB, 0.78206E+00_JPRB, 0.76889E+00_JPRB, 0.75722E+00_JPRB, 0.74340E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.73431E+00_JPRB, 0.71291E+00_JPRB, 0.66854E+00_JPRB, 0.62271E+00_JPRB, 0.55580E+00_JPRB, 0.49095E+00_JPRB, 0.47811E+00_JPRB ,&
&  0.49086E+00_JPRB, 0.49442E+00_JPRB, 0.48766E+00_JPRB, 0.46952E+00_JPRB, 0.46607E+00_JPRB, 0.47328E+00_JPRB, 0.47719E+00_JPRB ,&
&  0.47990E+00_JPRB, 0.55361E+00_JPRB, 0.43977E+00_JPRB, 0.36386E+00_JPRB, 0.35763E+00_JPRB, 0.42624E+00_JPRB, 0.56167E+00_JPRB ,&
&  0.71083E+00_JPRB, 0.98498E+00_JPRB, 0.12906E+01_JPRB, 0.15579E+01_JPRB, 0.19130E+01_JPRB, 0.21810E+01_JPRB, 0.23120E+01_JPRB ,&
&  0.24177E+01_JPRB, 0.20583E+01_JPRB, 0.20639E+01_JPRB, 0.19045E+01_JPRB, 0.17551E+01_JPRB, 0.16287E+01_JPRB, 0.15247E+01_JPRB ,&
&  0.13982E+01_JPRB, 0.13030E+01_JPRB, 0.11948E+01_JPRB, 0.11205E+01_JPRB, 0.10795E+01_JPRB, 0.10425E+01_JPRB, 0.10116E+01_JPRB ,&
&  0.10365E+01_JPRB, 0.10903E+01_JPRB, 0.11528E+01_JPRB, 0.11741E+01_JPRB, 0.12333E+01_JPRB, 0.13512E+01_JPRB, 0.15177E+01_JPRB ,&
&  0.15244E+01_JPRB, 0.14145E+01_JPRB, 0.17232E+01_JPRB, 0.19001E+01_JPRB, 0.20947E+01_JPRB, 0.21739E+01_JPRB, 0.22448E+01_JPRB ,&
&  0.24410E+01_JPRB, 0.23542E+01_JPRB, 0.22268E+01_JPRB, 0.19199E+01_JPRB, 0.19721E+01_JPRB, 0.19345E+01_JPRB, 0.18124E+01_JPRB ,&
&  0.16488E+01_JPRB, 0.15967E+01_JPRB, 0.14787E+01_JPRB, 0.13735E+01_JPRB, 0.13050E+01_JPRB, 0.12775E+01_JPRB, 0.11381E+01_JPRB ,&
&  0.97200E+00_JPRB, 0.10068E+01_JPRB, 0.91488E+00_JPRB, 0.78488E+00_JPRB, 0.62973E+00_JPRB, 0.56089E+00_JPRB, 0.49938E+00_JPRB ,&
&  0.46235E+00_JPRB, 0.43846E+00_JPRB, 0.43006E+00_JPRB, 0.46537E+00_JPRB, 0.51277E+00_JPRB, 0.52900E+00_JPRB, 0.54982E+00_JPRB ,&
&  0.58072E+00_JPRB, 0.71798E+00_JPRB, 0.75763E+00_JPRB, 0.76715E+00_JPRB, 0.81199E+00_JPRB, 0.83989E+00_JPRB, 0.83796E+00_JPRB ,&
&  0.80838E+00_JPRB, 0.78515E+00_JPRB, 0.76904E+00_JPRB, 0.75784E+00_JPRB, 0.74401E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.72241E+00_JPRB, 0.70380E+00_JPRB, 0.66852E+00_JPRB, 0.62148E+00_JPRB, 0.55155E+00_JPRB, 0.48915E+00_JPRB, 0.47407E+00_JPRB ,&
&  0.47303E+00_JPRB, 0.47081E+00_JPRB, 0.46288E+00_JPRB, 0.45799E+00_JPRB, 0.46027E+00_JPRB, 0.46494E+00_JPRB, 0.48377E+00_JPRB ,&
&  0.52619E+00_JPRB, 0.81552E+00_JPRB, 0.61140E+00_JPRB, 0.49965E+00_JPRB, 0.43422E+00_JPRB, 0.50973E+00_JPRB, 0.59889E+00_JPRB ,&
&  0.74271E+00_JPRB, 0.86830E+00_JPRB, 0.96220E+00_JPRB, 0.11610E+01_JPRB, 0.13682E+01_JPRB, 0.14310E+01_JPRB, 0.15600E+01_JPRB ,&
&  0.15674E+01_JPRB, 0.15666E+01_JPRB, 0.14291E+01_JPRB, 0.14302E+01_JPRB, 0.13671E+01_JPRB, 0.12283E+01_JPRB, 0.12517E+01_JPRB ,&
&  0.11622E+01_JPRB, 0.10913E+01_JPRB, 0.10187E+01_JPRB, 0.97864E+00_JPRB, 0.97750E+00_JPRB, 0.97306E+00_JPRB, 0.94194E+00_JPRB ,&
&  0.95810E+00_JPRB, 0.10051E+01_JPRB, 0.10815E+01_JPRB, 0.11546E+01_JPRB, 0.12504E+01_JPRB, 0.14322E+01_JPRB, 0.15866E+01_JPRB ,&
&  0.19914E+01_JPRB, 0.19181E+01_JPRB, 0.16242E+01_JPRB, 0.19964E+01_JPRB, 0.26035E+01_JPRB, 0.26521E+01_JPRB, 0.24273E+01_JPRB ,&
&  0.25738E+01_JPRB, 0.24713E+01_JPRB, 0.22713E+01_JPRB, 0.20848E+01_JPRB, 0.19314E+01_JPRB, 0.18645E+01_JPRB, 0.18132E+01_JPRB ,&
&  0.16286E+01_JPRB, 0.15259E+01_JPRB, 0.14618E+01_JPRB, 0.13517E+01_JPRB, 0.12570E+01_JPRB, 0.12052E+01_JPRB, 0.11716E+01_JPRB ,&
&  0.10577E+01_JPRB, 0.88433E+00_JPRB, 0.65020E+00_JPRB, 0.62411E+00_JPRB, 0.48281E+00_JPRB, 0.42685E+00_JPRB, 0.43450E+00_JPRB ,&
&  0.41954E+00_JPRB, 0.43783E+00_JPRB, 0.45832E+00_JPRB, 0.44314E+00_JPRB, 0.45019E+00_JPRB, 0.43692E+00_JPRB, 0.45483E+00_JPRB ,&
&  0.47430E+00_JPRB, 0.49646E+00_JPRB, 0.61650E+00_JPRB, 0.65212E+00_JPRB, 0.69472E+00_JPRB, 0.72742E+00_JPRB, 0.74591E+00_JPRB ,&
&  0.75116E+00_JPRB, 0.74567E+00_JPRB, 0.74113E+00_JPRB, 0.73393E+00_JPRB, 0.73179E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.65662E+00_JPRB, 0.64042E+00_JPRB, 0.61571E+00_JPRB, 0.57850E+00_JPRB, 0.52726E+00_JPRB, 0.47554E+00_JPRB, 0.45195E+00_JPRB ,&
&  0.44923E+00_JPRB, 0.44154E+00_JPRB, 0.43740E+00_JPRB, 0.44049E+00_JPRB, 0.44825E+00_JPRB, 0.45950E+00_JPRB, 0.49704E+00_JPRB ,&
&  0.59285E+00_JPRB, 0.73546E+00_JPRB, 0.60636E+00_JPRB, 0.50961E+00_JPRB, 0.59468E+00_JPRB, 0.58205E+00_JPRB, 0.63424E+00_JPRB ,&
&  0.68120E+00_JPRB, 0.81470E+00_JPRB, 0.83376E+00_JPRB, 0.86821E+00_JPRB, 0.98756E+00_JPRB, 0.10613E+01_JPRB, 0.11049E+01_JPRB ,&
&  0.10887E+01_JPRB, 0.10262E+01_JPRB, 0.98174E+00_JPRB, 0.99925E+00_JPRB, 0.10673E+01_JPRB, 0.10180E+01_JPRB, 0.10244E+01_JPRB ,&
&  0.96882E+00_JPRB, 0.92032E+00_JPRB, 0.85619E+00_JPRB, 0.83587E+00_JPRB, 0.85454E+00_JPRB, 0.89501E+00_JPRB, 0.91826E+00_JPRB ,&
&  0.96673E+00_JPRB, 0.10227E+01_JPRB, 0.10721E+01_JPRB, 0.11673E+01_JPRB, 0.13010E+01_JPRB, 0.16215E+01_JPRB, 0.20611E+01_JPRB ,&
&  0.25605E+01_JPRB, 0.26998E+01_JPRB, 0.27915E+01_JPRB, 0.30506E+01_JPRB, 0.30526E+01_JPRB, 0.28968E+01_JPRB, 0.27176E+01_JPRB ,&
&  0.24942E+01_JPRB, 0.23409E+01_JPRB, 0.22188E+01_JPRB, 0.20491E+01_JPRB, 0.18924E+01_JPRB, 0.18130E+01_JPRB, 0.17165E+01_JPRB ,&
&  0.14742E+01_JPRB, 0.13431E+01_JPRB, 0.13233E+01_JPRB, 0.12296E+01_JPRB, 0.11578E+01_JPRB, 0.10763E+01_JPRB, 0.10565E+01_JPRB ,&
&  0.98149E+00_JPRB, 0.62976E+00_JPRB, 0.51706E+00_JPRB, 0.53561E+00_JPRB, 0.44338E+00_JPRB, 0.41233E+00_JPRB, 0.43725E+00_JPRB ,&
&  0.42959E+00_JPRB, 0.43423E+00_JPRB, 0.44632E+00_JPRB, 0.42102E+00_JPRB, 0.40162E+00_JPRB, 0.39756E+00_JPRB, 0.38195E+00_JPRB ,&
&  0.38559E+00_JPRB, 0.40628E+00_JPRB, 0.50286E+00_JPRB, 0.56786E+00_JPRB, 0.61114E+00_JPRB, 0.63380E+00_JPRB, 0.65379E+00_JPRB ,&
&  0.66299E+00_JPRB, 0.66866E+00_JPRB, 0.67235E+00_JPRB, 0.67830E+00_JPRB, 0.67930E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.59103E+00_JPRB, 0.58554E+00_JPRB, 0.56963E+00_JPRB, 0.54720E+00_JPRB, 0.51653E+00_JPRB, 0.48119E+00_JPRB, 0.44583E+00_JPRB ,&
&  0.44308E+00_JPRB, 0.43664E+00_JPRB, 0.43412E+00_JPRB, 0.43661E+00_JPRB, 0.44367E+00_JPRB, 0.45317E+00_JPRB, 0.47230E+00_JPRB ,&
&  0.41253E+00_JPRB, 0.48183E+00_JPRB, 0.44306E+00_JPRB, 0.50998E+00_JPRB, 0.69519E+00_JPRB, 0.58363E+00_JPRB, 0.61879E+00_JPRB ,&
&  0.66140E+00_JPRB, 0.73517E+00_JPRB, 0.76667E+00_JPRB, 0.77730E+00_JPRB, 0.81788E+00_JPRB, 0.84496E+00_JPRB, 0.83555E+00_JPRB ,&
&  0.77604E+00_JPRB, 0.69172E+00_JPRB, 0.67308E+00_JPRB, 0.67898E+00_JPRB, 0.75766E+00_JPRB, 0.82109E+00_JPRB, 0.84007E+00_JPRB ,&
&  0.81434E+00_JPRB, 0.78767E+00_JPRB, 0.76193E+00_JPRB, 0.77529E+00_JPRB, 0.80724E+00_JPRB, 0.83444E+00_JPRB, 0.88513E+00_JPRB ,&
&  0.95582E+00_JPRB, 0.10334E+01_JPRB, 0.11252E+01_JPRB, 0.12477E+01_JPRB, 0.12657E+01_JPRB, 0.19323E+01_JPRB, 0.30815E+01_JPRB ,&
&  0.35770E+01_JPRB, 0.33942E+01_JPRB, 0.34695E+01_JPRB, 0.38192E+01_JPRB, 0.36719E+01_JPRB, 0.32191E+01_JPRB, 0.26637E+01_JPRB ,&
&  0.22633E+01_JPRB, 0.20736E+01_JPRB, 0.20156E+01_JPRB, 0.18949E+01_JPRB, 0.16721E+01_JPRB, 0.16081E+01_JPRB, 0.15373E+01_JPRB ,&
&  0.13659E+01_JPRB, 0.12373E+01_JPRB, 0.11847E+01_JPRB, 0.11303E+01_JPRB, 0.10764E+01_JPRB, 0.10689E+01_JPRB, 0.10294E+01_JPRB ,&
&  0.95929E+00_JPRB, 0.79506E+00_JPRB, 0.59484E+00_JPRB, 0.50505E+00_JPRB, 0.42348E+00_JPRB, 0.42093E+00_JPRB, 0.46640E+00_JPRB ,&
&  0.45242E+00_JPRB, 0.42947E+00_JPRB, 0.40019E+00_JPRB, 0.37167E+00_JPRB, 0.35440E+00_JPRB, 0.35606E+00_JPRB, 0.34022E+00_JPRB ,&
&  0.33571E+00_JPRB, 0.36748E+00_JPRB, 0.41938E+00_JPRB, 0.47725E+00_JPRB, 0.52561E+00_JPRB, 0.56726E+00_JPRB, 0.54012E+00_JPRB ,&
&  0.60261E+00_JPRB, 0.61080E+00_JPRB, 0.61993E+00_JPRB, 0.63152E+00_JPRB, 0.61910E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.51666E+00_JPRB, 0.52676E+00_JPRB, 0.51511E+00_JPRB, 0.51043E+00_JPRB, 0.48906E+00_JPRB, 0.47228E+00_JPRB, 0.44265E+00_JPRB ,&
&  0.44920E+00_JPRB, 0.44543E+00_JPRB, 0.43581E+00_JPRB, 0.43278E+00_JPRB, 0.44176E+00_JPRB, 0.44112E+00_JPRB, 0.39212E+00_JPRB ,&
&  0.35704E+00_JPRB, 0.38315E+00_JPRB, 0.37814E+00_JPRB, 0.42514E+00_JPRB, 0.47381E+00_JPRB, 0.46432E+00_JPRB, 0.49643E+00_JPRB ,&
&  0.58393E+00_JPRB, 0.62560E+00_JPRB, 0.65679E+00_JPRB, 0.67375E+00_JPRB, 0.68484E+00_JPRB, 0.69802E+00_JPRB, 0.70404E+00_JPRB ,&
&  0.65149E+00_JPRB, 0.60765E+00_JPRB, 0.57430E+00_JPRB, 0.57088E+00_JPRB, 0.64352E+00_JPRB, 0.68540E+00_JPRB, 0.67342E+00_JPRB ,&
&  0.65679E+00_JPRB, 0.65611E+00_JPRB, 0.66923E+00_JPRB, 0.72022E+00_JPRB, 0.75960E+00_JPRB, 0.78379E+00_JPRB, 0.83507E+00_JPRB ,&
&  0.91056E+00_JPRB, 0.10150E+01_JPRB, 0.11230E+01_JPRB, 0.12422E+01_JPRB, 0.13035E+01_JPRB, 0.19784E+01_JPRB, 0.28508E+01_JPRB ,&
&  0.30105E+01_JPRB, 0.31305E+01_JPRB, 0.32225E+01_JPRB, 0.32408E+01_JPRB, 0.31671E+01_JPRB, 0.26738E+01_JPRB, 0.21276E+01_JPRB ,&
&  0.18281E+01_JPRB, 0.16746E+01_JPRB, 0.17913E+01_JPRB, 0.16854E+01_JPRB, 0.15291E+01_JPRB, 0.14030E+01_JPRB, 0.13156E+01_JPRB ,&
&  0.12216E+01_JPRB, 0.11123E+01_JPRB, 0.10462E+01_JPRB, 0.99496E+00_JPRB, 0.96196E+00_JPRB, 0.93918E+00_JPRB, 0.89200E+00_JPRB ,&
&  0.83845E+00_JPRB, 0.74956E+00_JPRB, 0.61648E+00_JPRB, 0.52394E+00_JPRB, 0.45848E+00_JPRB, 0.46274E+00_JPRB, 0.43479E+00_JPRB ,&
&  0.39304E+00_JPRB, 0.35337E+00_JPRB, 0.34351E+00_JPRB, 0.32995E+00_JPRB, 0.32295E+00_JPRB, 0.32467E+00_JPRB, 0.32093E+00_JPRB ,&
&  0.33703E+00_JPRB, 0.36958E+00_JPRB, 0.37823E+00_JPRB, 0.40718E+00_JPRB, 0.44669E+00_JPRB, 0.50240E+00_JPRB, 0.44164E+00_JPRB ,&
&  0.46796E+00_JPRB, 0.52697E+00_JPRB, 0.54085E+00_JPRB, 0.54231E+00_JPRB, 0.52508E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.48402E+00_JPRB, 0.49600E+00_JPRB, 0.48461E+00_JPRB, 0.49130E+00_JPRB, 0.48013E+00_JPRB, 0.47600E+00_JPRB, 0.42571E+00_JPRB ,&
&  0.45423E+00_JPRB, 0.44318E+00_JPRB, 0.42727E+00_JPRB, 0.42464E+00_JPRB, 0.44039E+00_JPRB, 0.38134E+00_JPRB, 0.32546E+00_JPRB ,&
&  0.30745E+00_JPRB, 0.32032E+00_JPRB, 0.36047E+00_JPRB, 0.38299E+00_JPRB, 0.39308E+00_JPRB, 0.40436E+00_JPRB, 0.41046E+00_JPRB ,&
&  0.46133E+00_JPRB, 0.48828E+00_JPRB, 0.53602E+00_JPRB, 0.56592E+00_JPRB, 0.56614E+00_JPRB, 0.57807E+00_JPRB, 0.60393E+00_JPRB ,&
&  0.57275E+00_JPRB, 0.55458E+00_JPRB, 0.54889E+00_JPRB, 0.50109E+00_JPRB, 0.56537E+00_JPRB, 0.54859E+00_JPRB, 0.52657E+00_JPRB ,&
&  0.53228E+00_JPRB, 0.56383E+00_JPRB, 0.60006E+00_JPRB, 0.63936E+00_JPRB, 0.67027E+00_JPRB, 0.70551E+00_JPRB, 0.75241E+00_JPRB ,&
&  0.81936E+00_JPRB, 0.94852E+00_JPRB, 0.10914E+01_JPRB, 0.12312E+01_JPRB, 0.14125E+01_JPRB, 0.19286E+01_JPRB, 0.25514E+01_JPRB ,&
&  0.27166E+01_JPRB, 0.28437E+01_JPRB, 0.29353E+01_JPRB, 0.25260E+01_JPRB, 0.25813E+01_JPRB, 0.24152E+01_JPRB, 0.19809E+01_JPRB ,&
&  0.16990E+01_JPRB, 0.14277E+01_JPRB, 0.13138E+01_JPRB, 0.12653E+01_JPRB, 0.12379E+01_JPRB, 0.11276E+01_JPRB, 0.10419E+01_JPRB ,&
&  0.98315E+00_JPRB, 0.97097E+00_JPRB, 0.89896E+00_JPRB, 0.86082E+00_JPRB, 0.80612E+00_JPRB, 0.76055E+00_JPRB, 0.71120E+00_JPRB ,&
&  0.66679E+00_JPRB, 0.60659E+00_JPRB, 0.55034E+00_JPRB, 0.50692E+00_JPRB, 0.46575E+00_JPRB, 0.43340E+00_JPRB, 0.41181E+00_JPRB ,&
&  0.38822E+00_JPRB, 0.35605E+00_JPRB, 0.34225E+00_JPRB, 0.33227E+00_JPRB, 0.32709E+00_JPRB, 0.32357E+00_JPRB, 0.32918E+00_JPRB ,&
&  0.34709E+00_JPRB, 0.37501E+00_JPRB, 0.39830E+00_JPRB, 0.40203E+00_JPRB, 0.41906E+00_JPRB, 0.44960E+00_JPRB, 0.43386E+00_JPRB ,&
&  0.41819E+00_JPRB, 0.47710E+00_JPRB, 0.48401E+00_JPRB, 0.48677E+00_JPRB, 0.48194E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.47507E+00_JPRB, 0.47954E+00_JPRB, 0.47336E+00_JPRB, 0.47547E+00_JPRB, 0.48069E+00_JPRB, 0.44558E+00_JPRB, 0.41143E+00_JPRB ,&
&  0.47495E+00_JPRB, 0.40955E+00_JPRB, 0.44037E+00_JPRB, 0.36166E+00_JPRB, 0.33682E+00_JPRB, 0.33357E+00_JPRB, 0.32098E+00_JPRB ,&
&  0.31036E+00_JPRB, 0.32984E+00_JPRB, 0.35205E+00_JPRB, 0.36081E+00_JPRB, 0.37855E+00_JPRB, 0.38955E+00_JPRB, 0.39182E+00_JPRB ,&
&  0.40338E+00_JPRB, 0.42126E+00_JPRB, 0.45422E+00_JPRB, 0.45857E+00_JPRB, 0.44801E+00_JPRB, 0.45440E+00_JPRB, 0.46993E+00_JPRB ,&
&  0.45548E+00_JPRB, 0.44071E+00_JPRB, 0.45587E+00_JPRB, 0.46105E+00_JPRB, 0.46182E+00_JPRB, 0.44089E+00_JPRB, 0.43378E+00_JPRB ,&
&  0.44859E+00_JPRB, 0.43523E+00_JPRB, 0.50130E+00_JPRB, 0.53092E+00_JPRB, 0.56706E+00_JPRB, 0.60928E+00_JPRB, 0.65111E+00_JPRB ,&
&  0.71953E+00_JPRB, 0.83884E+00_JPRB, 0.98493E+00_JPRB, 0.11066E+01_JPRB, 0.12925E+01_JPRB, 0.17854E+01_JPRB, 0.21540E+01_JPRB ,&
&  0.24790E+01_JPRB, 0.18013E+01_JPRB, 0.21236E+01_JPRB, 0.20966E+01_JPRB, 0.22071E+01_JPRB, 0.20711E+01_JPRB, 0.18248E+01_JPRB ,&
&  0.15187E+01_JPRB, 0.12142E+01_JPRB, 0.10390E+01_JPRB, 0.93367E+00_JPRB, 0.89291E+00_JPRB, 0.89669E+00_JPRB, 0.89004E+00_JPRB ,&
&  0.87229E+00_JPRB, 0.80185E+00_JPRB, 0.72821E+00_JPRB, 0.67261E+00_JPRB, 0.63365E+00_JPRB, 0.60170E+00_JPRB, 0.57186E+00_JPRB ,&
&  0.54327E+00_JPRB, 0.51047E+00_JPRB, 0.47703E+00_JPRB, 0.44188E+00_JPRB, 0.42599E+00_JPRB, 0.41032E+00_JPRB, 0.39970E+00_JPRB ,&
&  0.39254E+00_JPRB, 0.37453E+00_JPRB, 0.35623E+00_JPRB, 0.34570E+00_JPRB, 0.33897E+00_JPRB, 0.34169E+00_JPRB, 0.35172E+00_JPRB ,&
&  0.35284E+00_JPRB, 0.34729E+00_JPRB, 0.35502E+00_JPRB, 0.36402E+00_JPRB, 0.37151E+00_JPRB, 0.38544E+00_JPRB, 0.42990E+00_JPRB ,&
&  0.41482E+00_JPRB, 0.43432E+00_JPRB, 0.46545E+00_JPRB, 0.47048E+00_JPRB, 0.47240E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.47269E+00_JPRB, 0.47340E+00_JPRB, 0.46706E+00_JPRB, 0.45829E+00_JPRB, 0.46280E+00_JPRB, 0.44525E+00_JPRB, 0.39004E+00_JPRB ,&
&  0.39609E+00_JPRB, 0.35716E+00_JPRB, 0.33302E+00_JPRB, 0.32401E+00_JPRB, 0.32656E+00_JPRB, 0.32385E+00_JPRB, 0.31978E+00_JPRB ,&
&  0.32138E+00_JPRB, 0.33436E+00_JPRB, 0.35326E+00_JPRB, 0.35999E+00_JPRB, 0.36926E+00_JPRB, 0.37710E+00_JPRB, 0.37764E+00_JPRB ,&
&  0.38436E+00_JPRB, 0.39411E+00_JPRB, 0.40365E+00_JPRB, 0.40998E+00_JPRB, 0.40054E+00_JPRB, 0.39245E+00_JPRB, 0.38674E+00_JPRB ,&
&  0.38260E+00_JPRB, 0.38349E+00_JPRB, 0.38897E+00_JPRB, 0.41043E+00_JPRB, 0.40090E+00_JPRB, 0.39149E+00_JPRB, 0.39876E+00_JPRB ,&
&  0.32749E+00_JPRB, 0.34185E+00_JPRB, 0.42710E+00_JPRB, 0.45656E+00_JPRB, 0.48455E+00_JPRB, 0.51821E+00_JPRB, 0.57006E+00_JPRB ,&
&  0.63337E+00_JPRB, 0.72548E+00_JPRB, 0.83658E+00_JPRB, 0.94119E+00_JPRB, 0.10834E+01_JPRB, 0.13882E+01_JPRB, 0.16810E+01_JPRB ,&
&  0.19382E+01_JPRB, 0.15378E+01_JPRB, 0.14482E+01_JPRB, 0.15244E+01_JPRB, 0.17185E+01_JPRB, 0.14574E+01_JPRB, 0.12545E+01_JPRB ,&
&  0.10632E+01_JPRB, 0.92687E+00_JPRB, 0.85790E+00_JPRB, 0.87151E+00_JPRB, 0.85483E+00_JPRB, 0.86208E+00_JPRB, 0.84651E+00_JPRB ,&
&  0.74450E+00_JPRB, 0.66435E+00_JPRB, 0.60232E+00_JPRB, 0.57564E+00_JPRB, 0.55639E+00_JPRB, 0.53909E+00_JPRB, 0.52299E+00_JPRB ,&
&  0.50798E+00_JPRB, 0.48592E+00_JPRB, 0.45719E+00_JPRB, 0.42664E+00_JPRB, 0.40790E+00_JPRB, 0.39602E+00_JPRB, 0.38670E+00_JPRB ,&
&  0.37721E+00_JPRB, 0.36772E+00_JPRB, 0.36263E+00_JPRB, 0.36272E+00_JPRB, 0.36500E+00_JPRB, 0.36459E+00_JPRB, 0.36747E+00_JPRB ,&
&  0.35539E+00_JPRB, 0.33445E+00_JPRB, 0.32098E+00_JPRB, 0.32818E+00_JPRB, 0.34493E+00_JPRB, 0.36189E+00_JPRB, 0.37736E+00_JPRB ,&
&  0.39623E+00_JPRB, 0.41235E+00_JPRB, 0.41093E+00_JPRB, 0.42190E+00_JPRB, 0.44044E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.47120E+00_JPRB, 0.46980E+00_JPRB, 0.45563E+00_JPRB, 0.45308E+00_JPRB, 0.44259E+00_JPRB, 0.43654E+00_JPRB, 0.41600E+00_JPRB ,&
&  0.41131E+00_JPRB, 0.38929E+00_JPRB, 0.36436E+00_JPRB, 0.33857E+00_JPRB, 0.32734E+00_JPRB, 0.32391E+00_JPRB, 0.32673E+00_JPRB ,&
&  0.33450E+00_JPRB, 0.34400E+00_JPRB, 0.35391E+00_JPRB, 0.35503E+00_JPRB, 0.36143E+00_JPRB, 0.36702E+00_JPRB, 0.36825E+00_JPRB ,&
&  0.36906E+00_JPRB, 0.37106E+00_JPRB, 0.36807E+00_JPRB, 0.36123E+00_JPRB, 0.35866E+00_JPRB, 0.36036E+00_JPRB, 0.35911E+00_JPRB ,&
&  0.35240E+00_JPRB, 0.35074E+00_JPRB, 0.36191E+00_JPRB, 0.39044E+00_JPRB, 0.37509E+00_JPRB, 0.37895E+00_JPRB, 0.33682E+00_JPRB ,&
&  0.29633E+00_JPRB, 0.29081E+00_JPRB, 0.31670E+00_JPRB, 0.40203E+00_JPRB, 0.42758E+00_JPRB, 0.46236E+00_JPRB, 0.49197E+00_JPRB ,&
&  0.49835E+00_JPRB, 0.53607E+00_JPRB, 0.62872E+00_JPRB, 0.81238E+00_JPRB, 0.91140E+00_JPRB, 0.11130E+01_JPRB, 0.12984E+01_JPRB ,&
&  0.14272E+01_JPRB, 0.14579E+01_JPRB, 0.14633E+01_JPRB, 0.12007E+01_JPRB, 0.10907E+01_JPRB, 0.11702E+01_JPRB, 0.10074E+01_JPRB ,&
&  0.90638E+00_JPRB, 0.81245E+00_JPRB, 0.78554E+00_JPRB, 0.79108E+00_JPRB, 0.73771E+00_JPRB, 0.73321E+00_JPRB, 0.69439E+00_JPRB ,&
&  0.62778E+00_JPRB, 0.58796E+00_JPRB, 0.55993E+00_JPRB, 0.54214E+00_JPRB, 0.52953E+00_JPRB, 0.52250E+00_JPRB, 0.52134E+00_JPRB ,&
&  0.52639E+00_JPRB, 0.51585E+00_JPRB, 0.47100E+00_JPRB, 0.41599E+00_JPRB, 0.39251E+00_JPRB, 0.38088E+00_JPRB, 0.37209E+00_JPRB ,&
&  0.36403E+00_JPRB, 0.36138E+00_JPRB, 0.36608E+00_JPRB, 0.36963E+00_JPRB, 0.36620E+00_JPRB, 0.35336E+00_JPRB, 0.34266E+00_JPRB ,&
&  0.33721E+00_JPRB, 0.32941E+00_JPRB, 0.32492E+00_JPRB, 0.33916E+00_JPRB, 0.35659E+00_JPRB, 0.36839E+00_JPRB, 0.36944E+00_JPRB ,&
&  0.37121E+00_JPRB, 0.38201E+00_JPRB, 0.39407E+00_JPRB, 0.41795E+00_JPRB, 0.44138E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.40566E+00_JPRB, 0.42750E+00_JPRB, 0.45132E+00_JPRB, 0.45131E+00_JPRB, 0.45362E+00_JPRB, 0.42978E+00_JPRB, 0.40258E+00_JPRB ,&
&  0.38366E+00_JPRB, 0.37300E+00_JPRB, 0.36920E+00_JPRB, 0.36272E+00_JPRB, 0.35841E+00_JPRB, 0.35725E+00_JPRB, 0.35760E+00_JPRB ,&
&  0.35907E+00_JPRB, 0.36039E+00_JPRB, 0.36171E+00_JPRB, 0.36276E+00_JPRB, 0.36655E+00_JPRB, 0.37181E+00_JPRB, 0.37147E+00_JPRB ,&
&  0.36686E+00_JPRB, 0.36469E+00_JPRB, 0.35645E+00_JPRB, 0.34524E+00_JPRB, 0.34117E+00_JPRB, 0.34603E+00_JPRB, 0.35559E+00_JPRB ,&
&  0.35478E+00_JPRB, 0.34864E+00_JPRB, 0.35475E+00_JPRB, 0.36817E+00_JPRB, 0.36195E+00_JPRB, 0.36924E+00_JPRB, 0.33436E+00_JPRB ,&
&  0.29464E+00_JPRB, 0.26801E+00_JPRB, 0.25827E+00_JPRB, 0.28068E+00_JPRB, 0.33074E+00_JPRB, 0.41015E+00_JPRB, 0.44657E+00_JPRB ,&
&  0.48174E+00_JPRB, 0.52681E+00_JPRB, 0.58179E+00_JPRB, 0.65658E+00_JPRB, 0.75601E+00_JPRB, 0.91776E+00_JPRB, 0.10602E+01_JPRB ,&
&  0.11558E+01_JPRB, 0.11690E+01_JPRB, 0.11405E+01_JPRB, 0.91629E+00_JPRB, 0.80221E+00_JPRB, 0.75451E+00_JPRB, 0.79338E+00_JPRB ,&
&  0.80309E+00_JPRB, 0.69501E+00_JPRB, 0.63972E+00_JPRB, 0.62142E+00_JPRB, 0.60670E+00_JPRB, 0.60820E+00_JPRB, 0.60734E+00_JPRB ,&
&  0.59130E+00_JPRB, 0.56374E+00_JPRB, 0.53570E+00_JPRB, 0.51835E+00_JPRB, 0.50940E+00_JPRB, 0.50778E+00_JPRB, 0.50482E+00_JPRB ,&
&  0.51027E+00_JPRB, 0.52725E+00_JPRB, 0.48839E+00_JPRB, 0.41002E+00_JPRB, 0.39018E+00_JPRB, 0.38340E+00_JPRB, 0.37800E+00_JPRB ,&
&  0.36895E+00_JPRB, 0.36434E+00_JPRB, 0.36685E+00_JPRB, 0.36971E+00_JPRB, 0.36731E+00_JPRB, 0.36267E+00_JPRB, 0.36103E+00_JPRB ,&
&  0.36298E+00_JPRB, 0.36556E+00_JPRB, 0.37085E+00_JPRB, 0.38676E+00_JPRB, 0.39396E+00_JPRB, 0.39395E+00_JPRB, 0.38654E+00_JPRB ,&
&  0.38144E+00_JPRB, 0.38293E+00_JPRB, 0.38472E+00_JPRB, 0.38479E+00_JPRB, 0.39123E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.41287E+00_JPRB, 0.42099E+00_JPRB, 0.43340E+00_JPRB, 0.44256E+00_JPRB, 0.45352E+00_JPRB, 0.44812E+00_JPRB, 0.44163E+00_JPRB ,&
&  0.43949E+00_JPRB, 0.43712E+00_JPRB, 0.42550E+00_JPRB, 0.41283E+00_JPRB, 0.39708E+00_JPRB, 0.38565E+00_JPRB, 0.38277E+00_JPRB ,&
&  0.38445E+00_JPRB, 0.38550E+00_JPRB, 0.38631E+00_JPRB, 0.38391E+00_JPRB, 0.38214E+00_JPRB, 0.38336E+00_JPRB, 0.37963E+00_JPRB ,&
&  0.37250E+00_JPRB, 0.36663E+00_JPRB, 0.36017E+00_JPRB, 0.35379E+00_JPRB, 0.34483E+00_JPRB, 0.34314E+00_JPRB, 0.34242E+00_JPRB ,&
&  0.34100E+00_JPRB, 0.34313E+00_JPRB, 0.35136E+00_JPRB, 0.34699E+00_JPRB, 0.35099E+00_JPRB, 0.35927E+00_JPRB, 0.32059E+00_JPRB ,&
&  0.28340E+00_JPRB, 0.25854E+00_JPRB, 0.24557E+00_JPRB, 0.24661E+00_JPRB, 0.26304E+00_JPRB, 0.29899E+00_JPRB, 0.34975E+00_JPRB ,&
&  0.41364E+00_JPRB, 0.45633E+00_JPRB, 0.49657E+00_JPRB, 0.53714E+00_JPRB, 0.58321E+00_JPRB, 0.68077E+00_JPRB, 0.76478E+00_JPRB ,&
&  0.84529E+00_JPRB, 0.92229E+00_JPRB, 0.96122E+00_JPRB, 0.95937E+00_JPRB, 0.91083E+00_JPRB, 0.78115E+00_JPRB, 0.76039E+00_JPRB ,&
&  0.85808E+00_JPRB, 0.69420E+00_JPRB, 0.57453E+00_JPRB, 0.51957E+00_JPRB, 0.50733E+00_JPRB, 0.50824E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.51362E+00_JPRB, 0.51250E+00_JPRB, 0.50496E+00_JPRB, 0.49638E+00_JPRB, 0.49135E+00_JPRB, 0.48780E+00_JPRB, 0.48527E+00_JPRB ,&
&  0.48121E+00_JPRB, 0.50891E+00_JPRB, 0.55987E+00_JPRB, 0.45559E+00_JPRB, 0.41945E+00_JPRB, 0.40139E+00_JPRB, 0.38378E+00_JPRB ,&
&  0.37677E+00_JPRB, 0.37568E+00_JPRB, 0.37981E+00_JPRB, 0.38196E+00_JPRB, 0.38316E+00_JPRB, 0.38666E+00_JPRB, 0.39363E+00_JPRB ,&
&  0.40485E+00_JPRB, 0.41376E+00_JPRB, 0.41728E+00_JPRB, 0.41851E+00_JPRB, 0.41299E+00_JPRB, 0.40286E+00_JPRB, 0.39677E+00_JPRB ,&
&  0.39787E+00_JPRB, 0.40101E+00_JPRB, 0.40490E+00_JPRB, 0.40808E+00_JPRB, 0.40976E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.41810E+00_JPRB, 0.42143E+00_JPRB, 0.42865E+00_JPRB, 0.43479E+00_JPRB, 0.44338E+00_JPRB, 0.44994E+00_JPRB, 0.45291E+00_JPRB ,&
&  0.46063E+00_JPRB, 0.45702E+00_JPRB, 0.44410E+00_JPRB, 0.42801E+00_JPRB, 0.41219E+00_JPRB, 0.40259E+00_JPRB, 0.39879E+00_JPRB ,&
&  0.40101E+00_JPRB, 0.39949E+00_JPRB, 0.39853E+00_JPRB, 0.39946E+00_JPRB, 0.40061E+00_JPRB, 0.40204E+00_JPRB, 0.39711E+00_JPRB ,&
&  0.38273E+00_JPRB, 0.37264E+00_JPRB, 0.36173E+00_JPRB, 0.34889E+00_JPRB, 0.33679E+00_JPRB, 0.32953E+00_JPRB, 0.33388E+00_JPRB ,&
&  0.34129E+00_JPRB, 0.33749E+00_JPRB, 0.33659E+00_JPRB, 0.33815E+00_JPRB, 0.34216E+00_JPRB, 0.34016E+00_JPRB, 0.28208E+00_JPRB ,&
&  0.26053E+00_JPRB, 0.24674E+00_JPRB, 0.24186E+00_JPRB, 0.24258E+00_JPRB, 0.25514E+00_JPRB, 0.27855E+00_JPRB, 0.32104E+00_JPRB ,&
&  0.36489E+00_JPRB, 0.40761E+00_JPRB, 0.43646E+00_JPRB, 0.46957E+00_JPRB, 0.52014E+00_JPRB, 0.58374E+00_JPRB, 0.63996E+00_JPRB ,&
&  0.69304E+00_JPRB, 0.75594E+00_JPRB, 0.79279E+00_JPRB, 0.80812E+00_JPRB, 0.78423E+00_JPRB, 0.73432E+00_JPRB, 0.68721E+00_JPRB ,&
&  0.65076E+00_JPRB, 0.57761E+00_JPRB, 0.54484E+00_JPRB, 0.52063E+00_JPRB, 0.50579E+00_JPRB, 0.49614E+00_JPRB, 0.48384E+00_JPRB ,&
&  0.47796E+00_JPRB, 0.47841E+00_JPRB, 0.47367E+00_JPRB, 0.47240E+00_JPRB, 0.46917E+00_JPRB, 0.46273E+00_JPRB, 0.45328E+00_JPRB ,&
&  0.44542E+00_JPRB, 0.43734E+00_JPRB, 0.43205E+00_JPRB, 0.42255E+00_JPRB, 0.41056E+00_JPRB, 0.39894E+00_JPRB, 0.39349E+00_JPRB ,&
&  0.39300E+00_JPRB, 0.39658E+00_JPRB, 0.40235E+00_JPRB, 0.40353E+00_JPRB, 0.40616E+00_JPRB, 0.40958E+00_JPRB, 0.41940E+00_JPRB ,&
&  0.42336E+00_JPRB, 0.42697E+00_JPRB, 0.42775E+00_JPRB, 0.42547E+00_JPRB, 0.41876E+00_JPRB, 0.40991E+00_JPRB, 0.40387E+00_JPRB ,&
&  0.40274E+00_JPRB, 0.40359E+00_JPRB, 0.40499E+00_JPRB, 0.40873E+00_JPRB, 0.41391E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.41502E+00_JPRB, 0.41714E+00_JPRB, 0.42251E+00_JPRB, 0.42873E+00_JPRB, 0.43535E+00_JPRB, 0.44201E+00_JPRB, 0.44535E+00_JPRB ,&
&  0.44734E+00_JPRB, 0.44511E+00_JPRB, 0.44129E+00_JPRB, 0.43033E+00_JPRB, 0.42276E+00_JPRB, 0.41949E+00_JPRB, 0.41568E+00_JPRB ,&
&  0.41168E+00_JPRB, 0.41043E+00_JPRB, 0.41022E+00_JPRB, 0.41148E+00_JPRB, 0.40997E+00_JPRB, 0.41047E+00_JPRB, 0.40559E+00_JPRB ,&
&  0.39317E+00_JPRB, 0.38319E+00_JPRB, 0.36565E+00_JPRB, 0.35018E+00_JPRB, 0.33960E+00_JPRB, 0.33247E+00_JPRB, 0.33436E+00_JPRB ,&
&  0.33322E+00_JPRB, 0.33328E+00_JPRB, 0.33418E+00_JPRB, 0.33574E+00_JPRB, 0.34057E+00_JPRB, 0.28059E+00_JPRB, 0.26390E+00_JPRB ,&
&  0.25211E+00_JPRB, 0.24465E+00_JPRB, 0.24321E+00_JPRB, 0.24414E+00_JPRB, 0.25044E+00_JPRB, 0.26805E+00_JPRB, 0.29523E+00_JPRB ,&
&  0.31928E+00_JPRB, 0.38746E+00_JPRB, 0.40444E+00_JPRB, 0.42631E+00_JPRB, 0.46325E+00_JPRB, 0.50463E+00_JPRB, 0.54228E+00_JPRB ,&
&  0.57376E+00_JPRB, 0.61133E+00_JPRB, 0.64244E+00_JPRB, 0.66339E+00_JPRB, 0.65541E+00_JPRB, 0.64284E+00_JPRB, 0.61951E+00_JPRB ,&
&  0.59885E+00_JPRB, 0.57022E+00_JPRB, 0.55168E+00_JPRB, 0.53441E+00_JPRB, 0.52121E+00_JPRB, 0.50624E+00_JPRB, 0.49533E+00_JPRB ,&
&  0.48748E+00_JPRB, 0.47778E+00_JPRB, 0.47188E+00_JPRB, 0.46303E+00_JPRB, 0.45268E+00_JPRB, 0.44459E+00_JPRB, 0.43629E+00_JPRB ,&
&  0.42729E+00_JPRB, 0.41848E+00_JPRB, 0.41002E+00_JPRB, 0.40433E+00_JPRB, 0.39959E+00_JPRB, 0.39620E+00_JPRB, 0.39545E+00_JPRB ,&
&  0.39673E+00_JPRB, 0.40307E+00_JPRB, 0.40760E+00_JPRB, 0.41247E+00_JPRB, 0.41649E+00_JPRB, 0.41671E+00_JPRB, 0.41782E+00_JPRB ,&
&  0.41933E+00_JPRB, 0.42225E+00_JPRB, 0.42216E+00_JPRB, 0.41986E+00_JPRB, 0.41412E+00_JPRB, 0.40708E+00_JPRB, 0.40233E+00_JPRB ,&
&  0.40115E+00_JPRB, 0.40259E+00_JPRB, 0.40193E+00_JPRB, 0.40497E+00_JPRB, 0.40988E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.40987E+00_JPRB, 0.40966E+00_JPRB, 0.41032E+00_JPRB, 0.41460E+00_JPRB, 0.42212E+00_JPRB, 0.43061E+00_JPRB, 0.43033E+00_JPRB ,&
&  0.43031E+00_JPRB, 0.42791E+00_JPRB, 0.42884E+00_JPRB, 0.42782E+00_JPRB, 0.42591E+00_JPRB, 0.42250E+00_JPRB, 0.41944E+00_JPRB ,&
&  0.41676E+00_JPRB, 0.41130E+00_JPRB, 0.40459E+00_JPRB, 0.40331E+00_JPRB, 0.40497E+00_JPRB, 0.40654E+00_JPRB, 0.40669E+00_JPRB ,&
&  0.40437E+00_JPRB, 0.39696E+00_JPRB, 0.37538E+00_JPRB, 0.35903E+00_JPRB, 0.34389E+00_JPRB, 0.34015E+00_JPRB, 0.33320E+00_JPRB ,&
&  0.34314E+00_JPRB, 0.31253E+00_JPRB, 0.31014E+00_JPRB, 0.29544E+00_JPRB, 0.28236E+00_JPRB, 0.26864E+00_JPRB, 0.25863E+00_JPRB ,&
&  0.25514E+00_JPRB, 0.25192E+00_JPRB, 0.25223E+00_JPRB, 0.25759E+00_JPRB, 0.26547E+00_JPRB, 0.28017E+00_JPRB, 0.29872E+00_JPRB ,&
&  0.31683E+00_JPRB, 0.36544E+00_JPRB, 0.38160E+00_JPRB, 0.39402E+00_JPRB, 0.41510E+00_JPRB, 0.43643E+00_JPRB, 0.45728E+00_JPRB ,&
&  0.47522E+00_JPRB, 0.49319E+00_JPRB, 0.50990E+00_JPRB, 0.51480E+00_JPRB, 0.52138E+00_JPRB, 0.51345E+00_JPRB, 0.51016E+00_JPRB ,&
&  0.50848E+00_JPRB, 0.50462E+00_JPRB, 0.49820E+00_JPRB, 0.48931E+00_JPRB, 0.48042E+00_JPRB, 0.46985E+00_JPRB, 0.46244E+00_JPRB ,&
&  0.45305E+00_JPRB, 0.44445E+00_JPRB, 0.43440E+00_JPRB, 0.42603E+00_JPRB, 0.41723E+00_JPRB, 0.41112E+00_JPRB, 0.40498E+00_JPRB ,&
&  0.40010E+00_JPRB, 0.39646E+00_JPRB, 0.39232E+00_JPRB, 0.38950E+00_JPRB, 0.38794E+00_JPRB, 0.38643E+00_JPRB, 0.38686E+00_JPRB ,&
&  0.39168E+00_JPRB, 0.39607E+00_JPRB, 0.40211E+00_JPRB, 0.40759E+00_JPRB, 0.41175E+00_JPRB, 0.41255E+00_JPRB, 0.41138E+00_JPRB ,&
&  0.41190E+00_JPRB, 0.41306E+00_JPRB, 0.41277E+00_JPRB, 0.40992E+00_JPRB, 0.40540E+00_JPRB, 0.40050E+00_JPRB, 0.39838E+00_JPRB ,&
&  0.39743E+00_JPRB, 0.39857E+00_JPRB, 0.39796E+00_JPRB, 0.40171E+00_JPRB, 0.40590E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.39795E+00_JPRB, 0.39853E+00_JPRB, 0.39862E+00_JPRB, 0.40077E+00_JPRB, 0.40361E+00_JPRB, 0.40819E+00_JPRB, 0.40941E+00_JPRB ,&
&  0.41357E+00_JPRB, 0.41720E+00_JPRB, 0.42086E+00_JPRB, 0.42200E+00_JPRB, 0.42246E+00_JPRB, 0.42119E+00_JPRB, 0.41866E+00_JPRB ,&
&  0.41601E+00_JPRB, 0.41235E+00_JPRB, 0.40930E+00_JPRB, 0.40598E+00_JPRB, 0.40286E+00_JPRB, 0.40147E+00_JPRB, 0.40109E+00_JPRB ,&
&  0.40157E+00_JPRB, 0.40149E+00_JPRB, 0.37354E+00_JPRB, 0.36273E+00_JPRB, 0.34782E+00_JPRB, 0.34048E+00_JPRB, 0.33720E+00_JPRB ,&
&  0.33401E+00_JPRB, 0.33066E+00_JPRB, 0.32682E+00_JPRB, 0.31887E+00_JPRB, 0.30740E+00_JPRB, 0.29151E+00_JPRB, 0.28291E+00_JPRB ,&
&  0.27539E+00_JPRB, 0.27330E+00_JPRB, 0.27464E+00_JPRB, 0.27981E+00_JPRB, 0.28951E+00_JPRB, 0.30066E+00_JPRB, 0.31431E+00_JPRB ,&
&  0.33295E+00_JPRB, 0.35400E+00_JPRB, 0.36757E+00_JPRB, 0.37117E+00_JPRB, 0.38546E+00_JPRB, 0.39486E+00_JPRB, 0.40295E+00_JPRB ,&
&  0.40953E+00_JPRB, 0.41525E+00_JPRB, 0.42507E+00_JPRB, 0.42870E+00_JPRB, 0.42712E+00_JPRB, 0.42628E+00_JPRB, 0.42926E+00_JPRB ,&
&  0.43061E+00_JPRB, 0.43051E+00_JPRB, 0.42731E+00_JPRB, 0.42416E+00_JPRB, 0.42032E+00_JPRB, 0.41552E+00_JPRB, 0.41112E+00_JPRB ,&
&  0.40664E+00_JPRB, 0.40247E+00_JPRB, 0.39668E+00_JPRB, 0.39070E+00_JPRB, 0.38671E+00_JPRB, 0.38325E+00_JPRB, 0.38019E+00_JPRB ,&
&  0.37791E+00_JPRB, 0.37639E+00_JPRB, 0.37480E+00_JPRB, 0.37082E+00_JPRB, 0.37012E+00_JPRB, 0.37362E+00_JPRB, 0.37488E+00_JPRB ,&
&  0.37923E+00_JPRB, 0.38317E+00_JPRB, 0.38802E+00_JPRB, 0.39181E+00_JPRB, 0.39513E+00_JPRB, 0.39697E+00_JPRB, 0.39697E+00_JPRB ,&
&  0.39735E+00_JPRB, 0.39761E+00_JPRB, 0.39726E+00_JPRB, 0.39472E+00_JPRB, 0.39206E+00_JPRB, 0.38928E+00_JPRB, 0.38867E+00_JPRB ,&
&  0.38789E+00_JPRB, 0.38833E+00_JPRB, 0.38805E+00_JPRB, 0.39148E+00_JPRB, 0.39458E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.38945E+00_JPRB, 0.39113E+00_JPRB, 0.39263E+00_JPRB, 0.39442E+00_JPRB, 0.39699E+00_JPRB, 0.40018E+00_JPRB, 0.40231E+00_JPRB ,&
&  0.40513E+00_JPRB, 0.40814E+00_JPRB, 0.41162E+00_JPRB, 0.41351E+00_JPRB, 0.41472E+00_JPRB, 0.41487E+00_JPRB, 0.41435E+00_JPRB ,&
&  0.41295E+00_JPRB, 0.41154E+00_JPRB, 0.41005E+00_JPRB, 0.40896E+00_JPRB, 0.40792E+00_JPRB, 0.40685E+00_JPRB, 0.40496E+00_JPRB ,&
&  0.40309E+00_JPRB, 0.40274E+00_JPRB, 0.39961E+00_JPRB, 0.39515E+00_JPRB, 0.39137E+00_JPRB, 0.36089E+00_JPRB, 0.35044E+00_JPRB ,&
&  0.34797E+00_JPRB, 0.34502E+00_JPRB, 0.34581E+00_JPRB, 0.34714E+00_JPRB, 0.35452E+00_JPRB, 0.35268E+00_JPRB, 0.35189E+00_JPRB ,&
&  0.33523E+00_JPRB, 0.33382E+00_JPRB, 0.33039E+00_JPRB, 0.32897E+00_JPRB, 0.33150E+00_JPRB, 0.33435E+00_JPRB, 0.34223E+00_JPRB ,&
&  0.36056E+00_JPRB, 0.36673E+00_JPRB, 0.36987E+00_JPRB, 0.37336E+00_JPRB, 0.37767E+00_JPRB, 0.38115E+00_JPRB, 0.38415E+00_JPRB ,&
&  0.38658E+00_JPRB, 0.38849E+00_JPRB, 0.38956E+00_JPRB, 0.38780E+00_JPRB, 0.38564E+00_JPRB, 0.38503E+00_JPRB, 0.38463E+00_JPRB ,&
&  0.38381E+00_JPRB, 0.38377E+00_JPRB, 0.38404E+00_JPRB, 0.38415E+00_JPRB, 0.38417E+00_JPRB, 0.38349E+00_JPRB, 0.38262E+00_JPRB ,&
&  0.38151E+00_JPRB, 0.38008E+00_JPRB, 0.37869E+00_JPRB, 0.37703E+00_JPRB, 0.37625E+00_JPRB, 0.37449E+00_JPRB, 0.37337E+00_JPRB ,&
&  0.37206E+00_JPRB, 0.37141E+00_JPRB, 0.37038E+00_JPRB, 0.37048E+00_JPRB, 0.37026E+00_JPRB, 0.37157E+00_JPRB, 0.37286E+00_JPRB ,&
&  0.37540E+00_JPRB, 0.37735E+00_JPRB, 0.37923E+00_JPRB, 0.38043E+00_JPRB, 0.38178E+00_JPRB, 0.38278E+00_JPRB, 0.38289E+00_JPRB ,&
&  0.38282E+00_JPRB, 0.38228E+00_JPRB, 0.38154E+00_JPRB, 0.38025E+00_JPRB, 0.37932E+00_JPRB, 0.37859E+00_JPRB, 0.37861E+00_JPRB ,&
&  0.37848E+00_JPRB, 0.37970E+00_JPRB, 0.38190E+00_JPRB, 0.38471E+00_JPRB, 0.38720E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.39462E+00_JPRB, 0.39620E+00_JPRB, 0.39707E+00_JPRB, 0.39772E+00_JPRB, 0.39828E+00_JPRB, 0.39961E+00_JPRB, 0.40079E+00_JPRB ,&
&  0.40202E+00_JPRB, 0.40346E+00_JPRB, 0.40546E+00_JPRB, 0.40666E+00_JPRB, 0.40759E+00_JPRB, 0.40807E+00_JPRB, 0.40863E+00_JPRB ,&
&  0.40881E+00_JPRB, 0.40872E+00_JPRB, 0.40817E+00_JPRB, 0.40790E+00_JPRB, 0.40772E+00_JPRB, 0.40773E+00_JPRB, 0.40756E+00_JPRB ,&
&  0.40675E+00_JPRB, 0.40566E+00_JPRB, 0.40445E+00_JPRB, 0.40253E+00_JPRB, 0.40000E+00_JPRB, 0.39727E+00_JPRB, 0.39411E+00_JPRB ,&
&  0.39097E+00_JPRB, 0.38854E+00_JPRB, 0.38612E+00_JPRB, 0.38387E+00_JPRB, 0.38302E+00_JPRB, 0.38205E+00_JPRB, 0.38107E+00_JPRB ,&
&  0.38010E+00_JPRB, 0.37885E+00_JPRB, 0.37872E+00_JPRB, 0.37903E+00_JPRB, 0.37972E+00_JPRB, 0.38108E+00_JPRB, 0.38237E+00_JPRB ,&
&  0.38335E+00_JPRB, 0.38353E+00_JPRB, 0.38318E+00_JPRB, 0.38329E+00_JPRB, 0.38366E+00_JPRB, 0.38376E+00_JPRB, 0.38328E+00_JPRB ,&
&  0.38299E+00_JPRB, 0.38215E+00_JPRB, 0.38083E+00_JPRB, 0.37933E+00_JPRB, 0.37833E+00_JPRB, 0.37730E+00_JPRB, 0.37619E+00_JPRB ,&
&  0.37552E+00_JPRB, 0.37514E+00_JPRB, 0.37496E+00_JPRB, 0.37483E+00_JPRB, 0.37514E+00_JPRB, 0.37543E+00_JPRB, 0.37568E+00_JPRB ,&
&  0.37592E+00_JPRB, 0.37674E+00_JPRB, 0.37758E+00_JPRB, 0.37811E+00_JPRB, 0.37864E+00_JPRB, 0.37893E+00_JPRB, 0.37928E+00_JPRB ,&
&  0.37935E+00_JPRB, 0.37917E+00_JPRB, 0.37830E+00_JPRB, 0.37800E+00_JPRB, 0.37785E+00_JPRB, 0.37825E+00_JPRB, 0.37840E+00_JPRB ,&
&  0.37926E+00_JPRB, 0.37979E+00_JPRB, 0.38019E+00_JPRB, 0.38084E+00_JPRB, 0.38146E+00_JPRB, 0.38191E+00_JPRB, 0.38205E+00_JPRB ,&
&  0.38211E+00_JPRB, 0.38244E+00_JPRB, 0.38277E+00_JPRB, 0.38312E+00_JPRB, 0.38391E+00_JPRB, 0.38478E+00_JPRB, 0.38572E+00_JPRB ,&
&  0.38684E+00_JPRB, 0.38857E+00_JPRB, 0.39043E+00_JPRB, 0.39190E+00_JPRB, 0.39316E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.39855E+00_JPRB, 0.39873E+00_JPRB, 0.39879E+00_JPRB, 0.39875E+00_JPRB, 0.39883E+00_JPRB, 0.39910E+00_JPRB, 0.39930E+00_JPRB ,&
&  0.39956E+00_JPRB, 0.39983E+00_JPRB, 0.40040E+00_JPRB, 0.40074E+00_JPRB, 0.40111E+00_JPRB, 0.40134E+00_JPRB, 0.40167E+00_JPRB ,&
&  0.40193E+00_JPRB, 0.40220E+00_JPRB, 0.40239E+00_JPRB, 0.40268E+00_JPRB, 0.40301E+00_JPRB, 0.40348E+00_JPRB, 0.40386E+00_JPRB ,&
&  0.40393E+00_JPRB, 0.40369E+00_JPRB, 0.40355E+00_JPRB, 0.40329E+00_JPRB, 0.40296E+00_JPRB, 0.40219E+00_JPRB, 0.40122E+00_JPRB ,&
&  0.40004E+00_JPRB, 0.39890E+00_JPRB, 0.39802E+00_JPRB, 0.39727E+00_JPRB, 0.39677E+00_JPRB, 0.39632E+00_JPRB, 0.39603E+00_JPRB ,&
&  0.39560E+00_JPRB, 0.39537E+00_JPRB, 0.39515E+00_JPRB, 0.39501E+00_JPRB, 0.39462E+00_JPRB, 0.39417E+00_JPRB, 0.39361E+00_JPRB ,&
&  0.39301E+00_JPRB, 0.39225E+00_JPRB, 0.39159E+00_JPRB, 0.39092E+00_JPRB, 0.39014E+00_JPRB, 0.38900E+00_JPRB, 0.38775E+00_JPRB ,&
&  0.38645E+00_JPRB, 0.38509E+00_JPRB, 0.38370E+00_JPRB, 0.38243E+00_JPRB, 0.38151E+00_JPRB, 0.38067E+00_JPRB, 0.37978E+00_JPRB ,&
&  0.37940E+00_JPRB, 0.37949E+00_JPRB, 0.37964E+00_JPRB, 0.37964E+00_JPRB, 0.38001E+00_JPRB, 0.38031E+00_JPRB, 0.38069E+00_JPRB ,&
&  0.38100E+00_JPRB, 0.38158E+00_JPRB, 0.38233E+00_JPRB, 0.38304E+00_JPRB, 0.38382E+00_JPRB, 0.38463E+00_JPRB, 0.38559E+00_JPRB ,&
&  0.38654E+00_JPRB, 0.38751E+00_JPRB, 0.38855E+00_JPRB, 0.38958E+00_JPRB, 0.39012E+00_JPRB, 0.39051E+00_JPRB, 0.39057E+00_JPRB ,&
&  0.39077E+00_JPRB, 0.39089E+00_JPRB, 0.39119E+00_JPRB, 0.39152E+00_JPRB, 0.39201E+00_JPRB, 0.39244E+00_JPRB, 0.39291E+00_JPRB ,&
&  0.39348E+00_JPRB, 0.39416E+00_JPRB, 0.39472E+00_JPRB, 0.39523E+00_JPRB, 0.39572E+00_JPRB, 0.39632E+00_JPRB, 0.39680E+00_JPRB ,&
&  0.39720E+00_JPRB, 0.39768E+00_JPRB, 0.39812E+00_JPRB, 0.39831E+00_JPRB, 0.39843E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB, 0.39466E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB, 0.18568E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.18905E+00_JPRB, 0.18935E+00_JPRB, 0.18948E+00_JPRB, 0.18953E+00_JPRB, 0.18829E+00_JPRB, 0.18790E+00_JPRB, 0.18748E+00_JPRB ,&
&  0.18717E+00_JPRB, 0.18701E+00_JPRB, 0.18737E+00_JPRB, 0.18782E+00_JPRB, 0.18828E+00_JPRB, 0.18872E+00_JPRB, 0.18906E+00_JPRB ,&
&  0.18919E+00_JPRB, 0.18932E+00_JPRB, 0.18842E+00_JPRB, 0.18801E+00_JPRB, 0.18791E+00_JPRB, 0.18799E+00_JPRB, 0.18800E+00_JPRB ,&
&  0.18813E+00_JPRB, 0.18824E+00_JPRB, 0.18825E+00_JPRB, 0.18853E+00_JPRB, 0.18904E+00_JPRB, 0.18948E+00_JPRB, 0.18993E+00_JPRB ,&
&  0.19037E+00_JPRB, 0.19097E+00_JPRB, 0.19152E+00_JPRB, 0.19203E+00_JPRB, 0.19263E+00_JPRB, 0.19317E+00_JPRB, 0.19344E+00_JPRB ,&
&  0.19361E+00_JPRB, 0.19357E+00_JPRB, 0.19360E+00_JPRB, 0.19362E+00_JPRB, 0.19375E+00_JPRB, 0.19393E+00_JPRB, 0.19392E+00_JPRB ,&
&  0.19380E+00_JPRB, 0.19361E+00_JPRB, 0.19336E+00_JPRB, 0.19286E+00_JPRB, 0.19234E+00_JPRB, 0.19184E+00_JPRB, 0.19136E+00_JPRB ,&
&  0.19093E+00_JPRB, 0.19046E+00_JPRB, 0.18995E+00_JPRB, 0.18940E+00_JPRB, 0.18882E+00_JPRB, 0.18826E+00_JPRB, 0.18770E+00_JPRB ,&
&  0.18712E+00_JPRB, 0.18675E+00_JPRB, 0.18634E+00_JPRB, 0.18599E+00_JPRB, 0.18573E+00_JPRB, 0.18547E+00_JPRB, 0.18522E+00_JPRB ,&
&  0.18497E+00_JPRB, 0.18466E+00_JPRB, 0.18439E+00_JPRB, 0.18407E+00_JPRB, 0.18379E+00_JPRB, 0.18348E+00_JPRB, 0.18324E+00_JPRB ,&
&  0.18300E+00_JPRB, 0.18280E+00_JPRB, 0.18263E+00_JPRB, 0.18255E+00_JPRB, 0.18251E+00_JPRB, 0.18256E+00_JPRB, 0.18261E+00_JPRB ,&
&  0.18268E+00_JPRB, 0.18273E+00_JPRB, 0.18277E+00_JPRB, 0.18269E+00_JPRB, 0.18287E+00_JPRB, 0.18307E+00_JPRB, 0.18334E+00_JPRB ,&
&  0.18359E+00_JPRB, 0.18399E+00_JPRB, 0.18438E+00_JPRB, 0.18481E+00_JPRB, 0.18546E+00_JPRB, 0.18569E+00_JPRB, 0.18590E+00_JPRB ,&
&  0.18614E+00_JPRB, 0.18663E+00_JPRB, 0.18706E+00_JPRB, 0.18730E+00_JPRB, 0.18782E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.20194E+00_JPRB, 0.20695E+00_JPRB, 0.21095E+00_JPRB, 0.21497E+00_JPRB, 0.21473E+00_JPRB, 0.23392E+00_JPRB, 0.23182E+00_JPRB ,&
&  0.23128E+00_JPRB, 0.22392E+00_JPRB, 0.22447E+00_JPRB, 0.22293E+00_JPRB, 0.22048E+00_JPRB, 0.21742E+00_JPRB, 0.21415E+00_JPRB ,&
&  0.21082E+00_JPRB, 0.20691E+00_JPRB, 0.20326E+00_JPRB, 0.19979E+00_JPRB, 0.19693E+00_JPRB, 0.19445E+00_JPRB, 0.19305E+00_JPRB ,&
&  0.19302E+00_JPRB, 0.19436E+00_JPRB, 0.19669E+00_JPRB, 0.19817E+00_JPRB, 0.19973E+00_JPRB, 0.20134E+00_JPRB, 0.20304E+00_JPRB ,&
&  0.20456E+00_JPRB, 0.20646E+00_JPRB, 0.20750E+00_JPRB, 0.20723E+00_JPRB, 0.20600E+00_JPRB, 0.20525E+00_JPRB, 0.20482E+00_JPRB ,&
&  0.20497E+00_JPRB, 0.20491E+00_JPRB, 0.20476E+00_JPRB, 0.20479E+00_JPRB, 0.20478E+00_JPRB, 0.20413E+00_JPRB, 0.20292E+00_JPRB ,&
&  0.20192E+00_JPRB, 0.20111E+00_JPRB, 0.19967E+00_JPRB, 0.19829E+00_JPRB, 0.19699E+00_JPRB, 0.19564E+00_JPRB, 0.19426E+00_JPRB ,&
&  0.19325E+00_JPRB, 0.19223E+00_JPRB, 0.19097E+00_JPRB, 0.18951E+00_JPRB, 0.18810E+00_JPRB, 0.18684E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18555E+00_JPRB, 0.18500E+00_JPRB, 0.18452E+00_JPRB, 0.18405E+00_JPRB, 0.18390E+00_JPRB, 0.18353E+00_JPRB, 0.18329E+00_JPRB ,&
&  0.18312E+00_JPRB, 0.18290E+00_JPRB, 0.18267E+00_JPRB, 0.18246E+00_JPRB, 0.18232E+00_JPRB, 0.18210E+00_JPRB, 0.18190E+00_JPRB ,&
&  0.18179E+00_JPRB, 0.18171E+00_JPRB, 0.18160E+00_JPRB, 0.18132E+00_JPRB, 0.18102E+00_JPRB, 0.18065E+00_JPRB, 0.18068E+00_JPRB ,&
&  0.18084E+00_JPRB, 0.18113E+00_JPRB, 0.18162E+00_JPRB, 0.18187E+00_JPRB, 0.18264E+00_JPRB, 0.18339E+00_JPRB, 0.18426E+00_JPRB ,&
&  0.18519E+00_JPRB, 0.18659E+00_JPRB, 0.18810E+00_JPRB, 0.18973E+00_JPRB, 0.19221E+00_JPRB, 0.19327E+00_JPRB, 0.19420E+00_JPRB ,&
&  0.19531E+00_JPRB, 0.19844E+00_JPRB, 0.19972E+00_JPRB, 0.20003E+00_JPRB, 0.20108E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.24423E+00_JPRB, 0.24699E+00_JPRB, 0.24891E+00_JPRB, 0.25019E+00_JPRB, 0.25138E+00_JPRB, 0.25071E+00_JPRB, 0.24929E+00_JPRB ,&
&  0.24535E+00_JPRB, 0.24393E+00_JPRB, 0.24126E+00_JPRB, 0.23852E+00_JPRB, 0.23531E+00_JPRB, 0.23204E+00_JPRB, 0.22806E+00_JPRB ,&
&  0.22369E+00_JPRB, 0.21898E+00_JPRB, 0.21326E+00_JPRB, 0.20718E+00_JPRB, 0.20163E+00_JPRB, 0.19647E+00_JPRB, 0.19314E+00_JPRB ,&
&  0.19156E+00_JPRB, 0.19212E+00_JPRB, 0.19464E+00_JPRB, 0.19796E+00_JPRB, 0.20201E+00_JPRB, 0.20662E+00_JPRB, 0.21236E+00_JPRB ,&
&  0.21738E+00_JPRB, 0.22346E+00_JPRB, 0.22718E+00_JPRB, 0.22884E+00_JPRB, 0.22745E+00_JPRB, 0.22724E+00_JPRB, 0.22619E+00_JPRB ,&
&  0.22479E+00_JPRB, 0.22247E+00_JPRB, 0.22126E+00_JPRB, 0.22092E+00_JPRB, 0.21967E+00_JPRB, 0.21744E+00_JPRB, 0.21463E+00_JPRB ,&
&  0.21215E+00_JPRB, 0.21002E+00_JPRB, 0.20756E+00_JPRB, 0.20546E+00_JPRB, 0.20385E+00_JPRB, 0.20197E+00_JPRB, 0.19958E+00_JPRB ,&
&  0.19737E+00_JPRB, 0.19514E+00_JPRB, 0.19250E+00_JPRB, 0.19018E+00_JPRB, 0.18789E+00_JPRB, 0.18621E+00_JPRB, 0.18471E+00_JPRB ,&
&  0.18402E+00_JPRB, 0.18332E+00_JPRB, 0.18307E+00_JPRB, 0.18293E+00_JPRB, 0.18328E+00_JPRB, 0.18305E+00_JPRB, 0.18272E+00_JPRB ,&
&  0.18230E+00_JPRB, 0.18220E+00_JPRB, 0.18175E+00_JPRB, 0.18160E+00_JPRB, 0.18145E+00_JPRB, 0.18094E+00_JPRB, 0.18072E+00_JPRB ,&
&  0.18089E+00_JPRB, 0.18092E+00_JPRB, 0.18057E+00_JPRB, 0.18062E+00_JPRB, 0.18042E+00_JPRB, 0.18008E+00_JPRB, 0.18037E+00_JPRB ,&
&  0.18025E+00_JPRB, 0.18048E+00_JPRB, 0.18081E+00_JPRB, 0.18132E+00_JPRB, 0.18236E+00_JPRB, 0.18370E+00_JPRB, 0.18549E+00_JPRB ,&
&  0.18816E+00_JPRB, 0.19060E+00_JPRB, 0.19310E+00_JPRB, 0.19604E+00_JPRB, 0.20128E+00_JPRB, 0.20453E+00_JPRB, 0.21001E+00_JPRB ,&
&  0.21544E+00_JPRB, 0.21645E+00_JPRB, 0.23878E+00_JPRB, 0.23973E+00_JPRB, 0.24119E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.25704E+00_JPRB, 0.25897E+00_JPRB, 0.26020E+00_JPRB, 0.26223E+00_JPRB, 0.26454E+00_JPRB, 0.26542E+00_JPRB, 0.26645E+00_JPRB ,&
&  0.26373E+00_JPRB, 0.25676E+00_JPRB, 0.25175E+00_JPRB, 0.24542E+00_JPRB, 0.23897E+00_JPRB, 0.23249E+00_JPRB, 0.22646E+00_JPRB ,&
&  0.22045E+00_JPRB, 0.21506E+00_JPRB, 0.20969E+00_JPRB, 0.20520E+00_JPRB, 0.20219E+00_JPRB, 0.19972E+00_JPRB, 0.19690E+00_JPRB ,&
&  0.19445E+00_JPRB, 0.19519E+00_JPRB, 0.19745E+00_JPRB, 0.20201E+00_JPRB, 0.20927E+00_JPRB, 0.21626E+00_JPRB, 0.22385E+00_JPRB ,&
&  0.23766E+00_JPRB, 0.24203E+00_JPRB, 0.24179E+00_JPRB, 0.24348E+00_JPRB, 0.24273E+00_JPRB, 0.24316E+00_JPRB, 0.23291E+00_JPRB ,&
&  0.23405E+00_JPRB, 0.23603E+00_JPRB, 0.24289E+00_JPRB, 0.24631E+00_JPRB, 0.24166E+00_JPRB, 0.23703E+00_JPRB, 0.22905E+00_JPRB ,&
&  0.22203E+00_JPRB, 0.21669E+00_JPRB, 0.21314E+00_JPRB, 0.21096E+00_JPRB, 0.20706E+00_JPRB, 0.20417E+00_JPRB, 0.19935E+00_JPRB ,&
&  0.19520E+00_JPRB, 0.19230E+00_JPRB, 0.18871E+00_JPRB, 0.18641E+00_JPRB, 0.18457E+00_JPRB, 0.18285E+00_JPRB, 0.18155E+00_JPRB ,&
&  0.18083E+00_JPRB, 0.18025E+00_JPRB, 0.18083E+00_JPRB, 0.18154E+00_JPRB, 0.18288E+00_JPRB, 0.18396E+00_JPRB, 0.18427E+00_JPRB ,&
&  0.18455E+00_JPRB, 0.18490E+00_JPRB, 0.18495E+00_JPRB, 0.18539E+00_JPRB, 0.18489E+00_JPRB, 0.18415E+00_JPRB, 0.18322E+00_JPRB ,&
&  0.18229E+00_JPRB, 0.18199E+00_JPRB, 0.18144E+00_JPRB, 0.18113E+00_JPRB, 0.17981E+00_JPRB, 0.17889E+00_JPRB, 0.17839E+00_JPRB ,&
&  0.17792E+00_JPRB, 0.17828E+00_JPRB, 0.17842E+00_JPRB, 0.17918E+00_JPRB, 0.18037E+00_JPRB, 0.18153E+00_JPRB, 0.18347E+00_JPRB ,&
&  0.18596E+00_JPRB, 0.18883E+00_JPRB, 0.19351E+00_JPRB, 0.19867E+00_JPRB, 0.20435E+00_JPRB, 0.21214E+00_JPRB, 0.22008E+00_JPRB ,&
&  0.23097E+00_JPRB, 0.24746E+00_JPRB, 0.24966E+00_JPRB, 0.25118E+00_JPRB, 0.25488E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.26646E+00_JPRB, 0.26744E+00_JPRB, 0.26895E+00_JPRB, 0.27128E+00_JPRB, 0.27419E+00_JPRB, 0.27537E+00_JPRB, 0.27567E+00_JPRB ,&
&  0.26664E+00_JPRB, 0.25882E+00_JPRB, 0.25206E+00_JPRB, 0.24169E+00_JPRB, 0.23346E+00_JPRB, 0.22595E+00_JPRB, 0.22048E+00_JPRB ,&
&  0.21610E+00_JPRB, 0.21364E+00_JPRB, 0.21135E+00_JPRB, 0.21059E+00_JPRB, 0.21083E+00_JPRB, 0.21168E+00_JPRB, 0.21048E+00_JPRB ,&
&  0.20890E+00_JPRB, 0.20976E+00_JPRB, 0.21036E+00_JPRB, 0.21481E+00_JPRB, 0.22121E+00_JPRB, 0.22853E+00_JPRB, 0.23565E+00_JPRB ,&
&  0.24472E+00_JPRB, 0.24880E+00_JPRB, 0.25186E+00_JPRB, 0.25261E+00_JPRB, 0.25106E+00_JPRB, 0.25016E+00_JPRB, 0.24788E+00_JPRB ,&
&  0.24706E+00_JPRB, 0.24635E+00_JPRB, 0.24723E+00_JPRB, 0.24812E+00_JPRB, 0.24693E+00_JPRB, 0.24395E+00_JPRB, 0.23796E+00_JPRB ,&
&  0.22961E+00_JPRB, 0.22274E+00_JPRB, 0.21681E+00_JPRB, 0.21225E+00_JPRB, 0.20650E+00_JPRB, 0.20136E+00_JPRB, 0.19622E+00_JPRB ,&
&  0.19164E+00_JPRB, 0.18808E+00_JPRB, 0.18399E+00_JPRB, 0.18098E+00_JPRB, 0.17959E+00_JPRB, 0.17799E+00_JPRB, 0.17729E+00_JPRB ,&
&  0.17723E+00_JPRB, 0.17753E+00_JPRB, 0.17929E+00_JPRB, 0.18082E+00_JPRB, 0.18320E+00_JPRB, 0.18595E+00_JPRB, 0.18776E+00_JPRB ,&
&  0.19071E+00_JPRB, 0.19326E+00_JPRB, 0.19462E+00_JPRB, 0.19553E+00_JPRB, 0.19460E+00_JPRB, 0.19211E+00_JPRB, 0.18945E+00_JPRB ,&
&  0.18617E+00_JPRB, 0.18379E+00_JPRB, 0.18225E+00_JPRB, 0.18060E+00_JPRB, 0.17981E+00_JPRB, 0.17915E+00_JPRB, 0.17850E+00_JPRB ,&
&  0.17803E+00_JPRB, 0.17785E+00_JPRB, 0.17764E+00_JPRB, 0.17852E+00_JPRB, 0.17952E+00_JPRB, 0.18131E+00_JPRB, 0.18350E+00_JPRB ,&
&  0.18560E+00_JPRB, 0.18876E+00_JPRB, 0.19224E+00_JPRB, 0.19693E+00_JPRB, 0.20500E+00_JPRB, 0.21223E+00_JPRB, 0.22776E+00_JPRB ,&
&  0.24186E+00_JPRB, 0.27156E+00_JPRB, 0.28318E+00_JPRB, 0.26929E+00_JPRB, 0.26715E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.28048E+00_JPRB, 0.27762E+00_JPRB, 0.27511E+00_JPRB, 0.27485E+00_JPRB, 0.27539E+00_JPRB, 0.27847E+00_JPRB, 0.27636E+00_JPRB ,&
&  0.27208E+00_JPRB, 0.27059E+00_JPRB, 0.27203E+00_JPRB, 0.27261E+00_JPRB, 0.27170E+00_JPRB, 0.26084E+00_JPRB, 0.25468E+00_JPRB ,&
&  0.24798E+00_JPRB, 0.24542E+00_JPRB, 0.24346E+00_JPRB, 0.24407E+00_JPRB, 0.24650E+00_JPRB, 0.24802E+00_JPRB, 0.24410E+00_JPRB ,&
&  0.23977E+00_JPRB, 0.23549E+00_JPRB, 0.23124E+00_JPRB, 0.22946E+00_JPRB, 0.23078E+00_JPRB, 0.23237E+00_JPRB, 0.23276E+00_JPRB ,&
&  0.23347E+00_JPRB, 0.23610E+00_JPRB, 0.23841E+00_JPRB, 0.24168E+00_JPRB, 0.25435E+00_JPRB, 0.25322E+00_JPRB, 0.25194E+00_JPRB ,&
&  0.25211E+00_JPRB, 0.25235E+00_JPRB, 0.25314E+00_JPRB, 0.25348E+00_JPRB, 0.25682E+00_JPRB, 0.26006E+00_JPRB, 0.26219E+00_JPRB ,&
&  0.24824E+00_JPRB, 0.23948E+00_JPRB, 0.23140E+00_JPRB, 0.22256E+00_JPRB, 0.21330E+00_JPRB, 0.20558E+00_JPRB, 0.19843E+00_JPRB ,&
&  0.19231E+00_JPRB, 0.18656E+00_JPRB, 0.18153E+00_JPRB, 0.17952E+00_JPRB, 0.17884E+00_JPRB, 0.17787E+00_JPRB, 0.17788E+00_JPRB ,&
&  0.17842E+00_JPRB, 0.17911E+00_JPRB, 0.17962E+00_JPRB, 0.18101E+00_JPRB, 0.18311E+00_JPRB, 0.18636E+00_JPRB, 0.18984E+00_JPRB ,&
&  0.19514E+00_JPRB, 0.19933E+00_JPRB, 0.20412E+00_JPRB, 0.20775E+00_JPRB, 0.20629E+00_JPRB, 0.20162E+00_JPRB, 0.19649E+00_JPRB ,&
&  0.19072E+00_JPRB, 0.18736E+00_JPRB, 0.18403E+00_JPRB, 0.18163E+00_JPRB, 0.18031E+00_JPRB, 0.17960E+00_JPRB, 0.18002E+00_JPRB ,&
&  0.18032E+00_JPRB, 0.18086E+00_JPRB, 0.18057E+00_JPRB, 0.18082E+00_JPRB, 0.18161E+00_JPRB, 0.18323E+00_JPRB, 0.18533E+00_JPRB ,&
&  0.18854E+00_JPRB, 0.19229E+00_JPRB, 0.19684E+00_JPRB, 0.20189E+00_JPRB, 0.20904E+00_JPRB, 0.21884E+00_JPRB, 0.23325E+00_JPRB ,&
&  0.25038E+00_JPRB, 0.28456E+00_JPRB, 0.29841E+00_JPRB, 0.28704E+00_JPRB, 0.28357E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.29657E+00_JPRB, 0.29295E+00_JPRB, 0.29112E+00_JPRB, 0.29144E+00_JPRB, 0.29166E+00_JPRB, 0.29454E+00_JPRB, 0.29423E+00_JPRB ,&
&  0.29010E+00_JPRB, 0.28551E+00_JPRB, 0.28395E+00_JPRB, 0.28355E+00_JPRB, 0.28234E+00_JPRB, 0.28119E+00_JPRB, 0.28167E+00_JPRB ,&
&  0.28012E+00_JPRB, 0.28156E+00_JPRB, 0.28401E+00_JPRB, 0.28523E+00_JPRB, 0.28597E+00_JPRB, 0.28619E+00_JPRB, 0.28624E+00_JPRB ,&
&  0.28027E+00_JPRB, 0.27883E+00_JPRB, 0.28429E+00_JPRB, 0.28182E+00_JPRB, 0.27819E+00_JPRB, 0.27509E+00_JPRB, 0.27409E+00_JPRB ,&
&  0.26643E+00_JPRB, 0.25562E+00_JPRB, 0.24260E+00_JPRB, 0.24674E+00_JPRB, 0.26364E+00_JPRB, 0.26236E+00_JPRB, 0.26017E+00_JPRB ,&
&  0.25984E+00_JPRB, 0.25924E+00_JPRB, 0.26003E+00_JPRB, 0.26048E+00_JPRB, 0.26111E+00_JPRB, 0.26098E+00_JPRB, 0.26175E+00_JPRB ,&
&  0.26288E+00_JPRB, 0.26781E+00_JPRB, 0.27174E+00_JPRB, 0.25426E+00_JPRB, 0.23831E+00_JPRB, 0.22608E+00_JPRB, 0.21461E+00_JPRB ,&
&  0.20443E+00_JPRB, 0.20103E+00_JPRB, 0.19780E+00_JPRB, 0.19286E+00_JPRB, 0.19349E+00_JPRB, 0.19393E+00_JPRB, 0.19330E+00_JPRB ,&
&  0.19173E+00_JPRB, 0.19076E+00_JPRB, 0.18938E+00_JPRB, 0.18714E+00_JPRB, 0.18622E+00_JPRB, 0.18726E+00_JPRB, 0.19023E+00_JPRB ,&
&  0.19511E+00_JPRB, 0.20104E+00_JPRB, 0.21438E+00_JPRB, 0.22612E+00_JPRB, 0.22359E+00_JPRB, 0.21519E+00_JPRB, 0.20756E+00_JPRB ,&
&  0.19839E+00_JPRB, 0.19369E+00_JPRB, 0.18914E+00_JPRB, 0.18551E+00_JPRB, 0.18392E+00_JPRB, 0.18430E+00_JPRB, 0.18530E+00_JPRB ,&
&  0.18661E+00_JPRB, 0.18842E+00_JPRB, 0.18917E+00_JPRB, 0.18874E+00_JPRB, 0.18797E+00_JPRB, 0.18828E+00_JPRB, 0.18959E+00_JPRB ,&
&  0.19216E+00_JPRB, 0.19670E+00_JPRB, 0.20235E+00_JPRB, 0.20924E+00_JPRB, 0.21512E+00_JPRB, 0.22057E+00_JPRB, 0.22879E+00_JPRB ,&
&  0.23642E+00_JPRB, 0.25325E+00_JPRB, 0.27862E+00_JPRB, 0.30671E+00_JPRB, 0.30365E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.29777E+00_JPRB, 0.29487E+00_JPRB, 0.29521E+00_JPRB, 0.29566E+00_JPRB, 0.29472E+00_JPRB, 0.29487E+00_JPRB, 0.29445E+00_JPRB ,&
&  0.29450E+00_JPRB, 0.29274E+00_JPRB, 0.29071E+00_JPRB, 0.29083E+00_JPRB, 0.29262E+00_JPRB, 0.29468E+00_JPRB, 0.29503E+00_JPRB ,&
&  0.29458E+00_JPRB, 0.29559E+00_JPRB, 0.29858E+00_JPRB, 0.30200E+00_JPRB, 0.30281E+00_JPRB, 0.30312E+00_JPRB, 0.30486E+00_JPRB ,&
&  0.30715E+00_JPRB, 0.30794E+00_JPRB, 0.31055E+00_JPRB, 0.30847E+00_JPRB, 0.30421E+00_JPRB, 0.30235E+00_JPRB, 0.30073E+00_JPRB ,&
&  0.29523E+00_JPRB, 0.27286E+00_JPRB, 0.25769E+00_JPRB, 0.26105E+00_JPRB, 0.27750E+00_JPRB, 0.27767E+00_JPRB, 0.27577E+00_JPRB ,&
&  0.27387E+00_JPRB, 0.27140E+00_JPRB, 0.26922E+00_JPRB, 0.26839E+00_JPRB, 0.26829E+00_JPRB, 0.26795E+00_JPRB, 0.26621E+00_JPRB ,&
&  0.26620E+00_JPRB, 0.27002E+00_JPRB, 0.27590E+00_JPRB, 0.27471E+00_JPRB, 0.27186E+00_JPRB, 0.26594E+00_JPRB, 0.26216E+00_JPRB ,&
&  0.25856E+00_JPRB, 0.25446E+00_JPRB, 0.25227E+00_JPRB, 0.25146E+00_JPRB, 0.25113E+00_JPRB, 0.24935E+00_JPRB, 0.24636E+00_JPRB ,&
&  0.23819E+00_JPRB, 0.22962E+00_JPRB, 0.22037E+00_JPRB, 0.21130E+00_JPRB, 0.20419E+00_JPRB, 0.19936E+00_JPRB, 0.19588E+00_JPRB ,&
&  0.19698E+00_JPRB, 0.20644E+00_JPRB, 0.22339E+00_JPRB, 0.24249E+00_JPRB, 0.26500E+00_JPRB, 0.24193E+00_JPRB, 0.23055E+00_JPRB ,&
&  0.21985E+00_JPRB, 0.21077E+00_JPRB, 0.20455E+00_JPRB, 0.20042E+00_JPRB, 0.19756E+00_JPRB, 0.19608E+00_JPRB, 0.19620E+00_JPRB ,&
&  0.19887E+00_JPRB, 0.20257E+00_JPRB, 0.20587E+00_JPRB, 0.20637E+00_JPRB, 0.20538E+00_JPRB, 0.20359E+00_JPRB, 0.20258E+00_JPRB ,&
&  0.20337E+00_JPRB, 0.20792E+00_JPRB, 0.21593E+00_JPRB, 0.22742E+00_JPRB, 0.23822E+00_JPRB, 0.24954E+00_JPRB, 0.26324E+00_JPRB ,&
&  0.27833E+00_JPRB, 0.29368E+00_JPRB, 0.29741E+00_JPRB, 0.30034E+00_JPRB, 0.30034E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.30579E+00_JPRB, 0.30381E+00_JPRB, 0.30414E+00_JPRB, 0.30522E+00_JPRB, 0.30518E+00_JPRB, 0.30486E+00_JPRB, 0.30265E+00_JPRB ,&
&  0.30062E+00_JPRB, 0.30105E+00_JPRB, 0.30236E+00_JPRB, 0.30471E+00_JPRB, 0.30882E+00_JPRB, 0.31314E+00_JPRB, 0.31235E+00_JPRB ,&
&  0.30912E+00_JPRB, 0.30757E+00_JPRB, 0.30951E+00_JPRB, 0.31174E+00_JPRB, 0.31225E+00_JPRB, 0.31285E+00_JPRB, 0.31627E+00_JPRB ,&
&  0.32069E+00_JPRB, 0.32262E+00_JPRB, 0.32538E+00_JPRB, 0.32494E+00_JPRB, 0.32143E+00_JPRB, 0.32163E+00_JPRB, 0.31991E+00_JPRB ,&
&  0.31515E+00_JPRB, 0.30970E+00_JPRB, 0.30097E+00_JPRB, 0.30212E+00_JPRB, 0.29859E+00_JPRB, 0.29579E+00_JPRB, 0.29170E+00_JPRB ,&
&  0.28731E+00_JPRB, 0.28386E+00_JPRB, 0.27995E+00_JPRB, 0.27674E+00_JPRB, 0.27500E+00_JPRB, 0.27494E+00_JPRB, 0.27280E+00_JPRB ,&
&  0.27128E+00_JPRB, 0.27236E+00_JPRB, 0.27383E+00_JPRB, 0.27220E+00_JPRB, 0.27305E+00_JPRB, 0.27979E+00_JPRB, 0.28479E+00_JPRB ,&
&  0.28641E+00_JPRB, 0.28623E+00_JPRB, 0.28382E+00_JPRB, 0.28322E+00_JPRB, 0.28214E+00_JPRB, 0.28018E+00_JPRB, 0.27695E+00_JPRB ,&
&  0.27234E+00_JPRB, 0.27010E+00_JPRB, 0.26859E+00_JPRB, 0.26547E+00_JPRB, 0.26110E+00_JPRB, 0.24373E+00_JPRB, 0.23416E+00_JPRB ,&
&  0.23317E+00_JPRB, 0.24176E+00_JPRB, 0.25713E+00_JPRB, 0.26290E+00_JPRB, 0.26725E+00_JPRB, 0.27191E+00_JPRB, 0.27302E+00_JPRB ,&
&  0.27565E+00_JPRB, 0.25498E+00_JPRB, 0.24384E+00_JPRB, 0.23575E+00_JPRB, 0.22843E+00_JPRB, 0.22341E+00_JPRB, 0.22216E+00_JPRB ,&
&  0.22479E+00_JPRB, 0.23079E+00_JPRB, 0.23555E+00_JPRB, 0.23768E+00_JPRB, 0.23717E+00_JPRB, 0.23368E+00_JPRB, 0.23034E+00_JPRB ,&
&  0.22948E+00_JPRB, 0.23515E+00_JPRB, 0.24700E+00_JPRB, 0.26015E+00_JPRB, 0.26859E+00_JPRB, 0.27749E+00_JPRB, 0.28515E+00_JPRB ,&
&  0.29038E+00_JPRB, 0.29526E+00_JPRB, 0.30147E+00_JPRB, 0.30389E+00_JPRB, 0.30518E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.30603E+00_JPRB, 0.30605E+00_JPRB, 0.30923E+00_JPRB, 0.31272E+00_JPRB, 0.31369E+00_JPRB, 0.31374E+00_JPRB, 0.31152E+00_JPRB ,&
&  0.31002E+00_JPRB, 0.31123E+00_JPRB, 0.31300E+00_JPRB, 0.31422E+00_JPRB, 0.31607E+00_JPRB, 0.31845E+00_JPRB, 0.32027E+00_JPRB ,&
&  0.32023E+00_JPRB, 0.31855E+00_JPRB, 0.31560E+00_JPRB, 0.31648E+00_JPRB, 0.31652E+00_JPRB, 0.31834E+00_JPRB, 0.32542E+00_JPRB ,&
&  0.33046E+00_JPRB, 0.33396E+00_JPRB, 0.33693E+00_JPRB, 0.33733E+00_JPRB, 0.33691E+00_JPRB, 0.33842E+00_JPRB, 0.33528E+00_JPRB ,&
&  0.32934E+00_JPRB, 0.32376E+00_JPRB, 0.31919E+00_JPRB, 0.31224E+00_JPRB, 0.31467E+00_JPRB, 0.31211E+00_JPRB, 0.30901E+00_JPRB ,&
&  0.30507E+00_JPRB, 0.30032E+00_JPRB, 0.29492E+00_JPRB, 0.28990E+00_JPRB, 0.28625E+00_JPRB, 0.28294E+00_JPRB, 0.27829E+00_JPRB ,&
&  0.27676E+00_JPRB, 0.27837E+00_JPRB, 0.28402E+00_JPRB, 0.28721E+00_JPRB, 0.28546E+00_JPRB, 0.28675E+00_JPRB, 0.29343E+00_JPRB ,&
&  0.29994E+00_JPRB, 0.30283E+00_JPRB, 0.30262E+00_JPRB, 0.30317E+00_JPRB, 0.30380E+00_JPRB, 0.30510E+00_JPRB, 0.30570E+00_JPRB ,&
&  0.30474E+00_JPRB, 0.30323E+00_JPRB, 0.30083E+00_JPRB, 0.29492E+00_JPRB, 0.28780E+00_JPRB, 0.28204E+00_JPRB, 0.27966E+00_JPRB ,&
&  0.28212E+00_JPRB, 0.28162E+00_JPRB, 0.28114E+00_JPRB, 0.28236E+00_JPRB, 0.28274E+00_JPRB, 0.28190E+00_JPRB, 0.28083E+00_JPRB ,&
&  0.28191E+00_JPRB, 0.28145E+00_JPRB, 0.28330E+00_JPRB, 0.28061E+00_JPRB, 0.27615E+00_JPRB, 0.27189E+00_JPRB, 0.26790E+00_JPRB ,&
&  0.26547E+00_JPRB, 0.26447E+00_JPRB, 0.26638E+00_JPRB, 0.26696E+00_JPRB, 0.26867E+00_JPRB, 0.27067E+00_JPRB, 0.27237E+00_JPRB ,&
&  0.27329E+00_JPRB, 0.27437E+00_JPRB, 0.27648E+00_JPRB, 0.28103E+00_JPRB, 0.28590E+00_JPRB, 0.29016E+00_JPRB, 0.29469E+00_JPRB ,&
&  0.29900E+00_JPRB, 0.30213E+00_JPRB, 0.30794E+00_JPRB, 0.30934E+00_JPRB, 0.30884E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.31064E+00_JPRB, 0.31270E+00_JPRB, 0.31474E+00_JPRB, 0.31659E+00_JPRB, 0.31785E+00_JPRB, 0.31884E+00_JPRB, 0.31807E+00_JPRB ,&
&  0.31792E+00_JPRB, 0.31951E+00_JPRB, 0.32159E+00_JPRB, 0.32328E+00_JPRB, 0.32420E+00_JPRB, 0.32595E+00_JPRB, 0.32758E+00_JPRB ,&
&  0.32864E+00_JPRB, 0.32782E+00_JPRB, 0.32496E+00_JPRB, 0.32393E+00_JPRB, 0.32292E+00_JPRB, 0.32765E+00_JPRB, 0.33584E+00_JPRB ,&
&  0.33945E+00_JPRB, 0.34177E+00_JPRB, 0.34466E+00_JPRB, 0.34541E+00_JPRB, 0.34592E+00_JPRB, 0.34576E+00_JPRB, 0.34139E+00_JPRB ,&
&  0.33664E+00_JPRB, 0.33221E+00_JPRB, 0.32973E+00_JPRB, 0.32646E+00_JPRB, 0.32598E+00_JPRB, 0.32325E+00_JPRB, 0.32114E+00_JPRB ,&
&  0.31969E+00_JPRB, 0.31738E+00_JPRB, 0.31487E+00_JPRB, 0.31050E+00_JPRB, 0.30487E+00_JPRB, 0.29763E+00_JPRB, 0.29281E+00_JPRB ,&
&  0.29097E+00_JPRB, 0.29432E+00_JPRB, 0.30238E+00_JPRB, 0.30647E+00_JPRB, 0.30395E+00_JPRB, 0.30382E+00_JPRB, 0.30724E+00_JPRB ,&
&  0.31191E+00_JPRB, 0.31691E+00_JPRB, 0.31847E+00_JPRB, 0.31758E+00_JPRB, 0.31580E+00_JPRB, 0.31739E+00_JPRB, 0.32144E+00_JPRB ,&
&  0.32505E+00_JPRB, 0.32712E+00_JPRB, 0.32661E+00_JPRB, 0.32586E+00_JPRB, 0.32333E+00_JPRB, 0.31731E+00_JPRB, 0.30916E+00_JPRB ,&
&  0.30178E+00_JPRB, 0.29723E+00_JPRB, 0.29385E+00_JPRB, 0.29069E+00_JPRB, 0.28961E+00_JPRB, 0.29008E+00_JPRB, 0.29226E+00_JPRB ,&
&  0.29314E+00_JPRB, 0.29500E+00_JPRB, 0.29514E+00_JPRB, 0.29105E+00_JPRB, 0.28776E+00_JPRB, 0.28894E+00_JPRB, 0.29037E+00_JPRB ,&
&  0.29280E+00_JPRB, 0.29510E+00_JPRB, 0.29751E+00_JPRB, 0.29861E+00_JPRB, 0.30020E+00_JPRB, 0.30269E+00_JPRB, 0.30406E+00_JPRB ,&
&  0.30475E+00_JPRB, 0.30533E+00_JPRB, 0.30708E+00_JPRB, 0.30989E+00_JPRB, 0.31229E+00_JPRB, 0.31406E+00_JPRB, 0.31318E+00_JPRB ,&
&  0.31193E+00_JPRB, 0.31078E+00_JPRB, 0.31144E+00_JPRB, 0.31189E+00_JPRB, 0.31055E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.33032E+00_JPRB, 0.33047E+00_JPRB, 0.33002E+00_JPRB, 0.32916E+00_JPRB, 0.32835E+00_JPRB, 0.32772E+00_JPRB, 0.32797E+00_JPRB ,&
&  0.32832E+00_JPRB, 0.32908E+00_JPRB, 0.32997E+00_JPRB, 0.33072E+00_JPRB, 0.33236E+00_JPRB, 0.33419E+00_JPRB, 0.33558E+00_JPRB ,&
&  0.33713E+00_JPRB, 0.33653E+00_JPRB, 0.33449E+00_JPRB, 0.33245E+00_JPRB, 0.33325E+00_JPRB, 0.33846E+00_JPRB, 0.34434E+00_JPRB ,&
&  0.34601E+00_JPRB, 0.34730E+00_JPRB, 0.34802E+00_JPRB, 0.34720E+00_JPRB, 0.34617E+00_JPRB, 0.34592E+00_JPRB, 0.34235E+00_JPRB ,&
&  0.33813E+00_JPRB, 0.33675E+00_JPRB, 0.33653E+00_JPRB, 0.33356E+00_JPRB, 0.33250E+00_JPRB, 0.33386E+00_JPRB, 0.33205E+00_JPRB ,&
&  0.33079E+00_JPRB, 0.32916E+00_JPRB, 0.32652E+00_JPRB, 0.32447E+00_JPRB, 0.32203E+00_JPRB, 0.31707E+00_JPRB, 0.31121E+00_JPRB ,&
&  0.30642E+00_JPRB, 0.30668E+00_JPRB, 0.31179E+00_JPRB, 0.31736E+00_JPRB, 0.31696E+00_JPRB, 0.31650E+00_JPRB, 0.31820E+00_JPRB ,&
&  0.32175E+00_JPRB, 0.32605E+00_JPRB, 0.32876E+00_JPRB, 0.32795E+00_JPRB, 0.32690E+00_JPRB, 0.32877E+00_JPRB, 0.33139E+00_JPRB ,&
&  0.33274E+00_JPRB, 0.33430E+00_JPRB, 0.33540E+00_JPRB, 0.33684E+00_JPRB, 0.33706E+00_JPRB, 0.33479E+00_JPRB, 0.33271E+00_JPRB ,&
&  0.32905E+00_JPRB, 0.32462E+00_JPRB, 0.31962E+00_JPRB, 0.31437E+00_JPRB, 0.31096E+00_JPRB, 0.30922E+00_JPRB, 0.30981E+00_JPRB ,&
&  0.30964E+00_JPRB, 0.31139E+00_JPRB, 0.31187E+00_JPRB, 0.30962E+00_JPRB, 0.30659E+00_JPRB, 0.30653E+00_JPRB, 0.30991E+00_JPRB ,&
&  0.31454E+00_JPRB, 0.31758E+00_JPRB, 0.32289E+00_JPRB, 0.32568E+00_JPRB, 0.32772E+00_JPRB, 0.33012E+00_JPRB, 0.33073E+00_JPRB ,&
&  0.32817E+00_JPRB, 0.32613E+00_JPRB, 0.32576E+00_JPRB, 0.32853E+00_JPRB, 0.32925E+00_JPRB, 0.32846E+00_JPRB, 0.32638E+00_JPRB ,&
&  0.32607E+00_JPRB, 0.32804E+00_JPRB, 0.32878E+00_JPRB, 0.32855E+00_JPRB, 0.32909E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.35424E+00_JPRB, 0.35276E+00_JPRB, 0.35141E+00_JPRB, 0.34939E+00_JPRB, 0.34726E+00_JPRB, 0.34541E+00_JPRB, 0.34378E+00_JPRB ,&
&  0.34281E+00_JPRB, 0.34284E+00_JPRB, 0.34355E+00_JPRB, 0.34382E+00_JPRB, 0.34362E+00_JPRB, 0.34438E+00_JPRB, 0.34466E+00_JPRB ,&
&  0.34613E+00_JPRB, 0.34646E+00_JPRB, 0.34474E+00_JPRB, 0.34477E+00_JPRB, 0.34750E+00_JPRB, 0.35235E+00_JPRB, 0.35451E+00_JPRB ,&
&  0.35262E+00_JPRB, 0.35280E+00_JPRB, 0.35302E+00_JPRB, 0.35142E+00_JPRB, 0.35038E+00_JPRB, 0.34960E+00_JPRB, 0.34573E+00_JPRB ,&
&  0.34135E+00_JPRB, 0.34118E+00_JPRB, 0.34321E+00_JPRB, 0.34533E+00_JPRB, 0.34395E+00_JPRB, 0.34100E+00_JPRB, 0.34104E+00_JPRB ,&
&  0.33920E+00_JPRB, 0.33906E+00_JPRB, 0.33895E+00_JPRB, 0.33965E+00_JPRB, 0.33817E+00_JPRB, 0.33610E+00_JPRB, 0.33242E+00_JPRB ,&
&  0.32452E+00_JPRB, 0.32224E+00_JPRB, 0.32445E+00_JPRB, 0.32863E+00_JPRB, 0.32835E+00_JPRB, 0.32935E+00_JPRB, 0.33246E+00_JPRB ,&
&  0.33629E+00_JPRB, 0.33912E+00_JPRB, 0.34093E+00_JPRB, 0.34229E+00_JPRB, 0.34409E+00_JPRB, 0.34581E+00_JPRB, 0.34686E+00_JPRB ,&
&  0.34817E+00_JPRB, 0.35024E+00_JPRB, 0.35249E+00_JPRB, 0.35454E+00_JPRB, 0.35483E+00_JPRB, 0.35401E+00_JPRB, 0.35235E+00_JPRB ,&
&  0.34984E+00_JPRB, 0.34572E+00_JPRB, 0.34141E+00_JPRB, 0.33600E+00_JPRB, 0.33102E+00_JPRB, 0.32872E+00_JPRB, 0.32799E+00_JPRB ,&
&  0.32543E+00_JPRB, 0.32323E+00_JPRB, 0.32164E+00_JPRB, 0.32158E+00_JPRB, 0.32204E+00_JPRB, 0.32344E+00_JPRB, 0.32585E+00_JPRB ,&
&  0.32779E+00_JPRB, 0.33056E+00_JPRB, 0.33669E+00_JPRB, 0.34105E+00_JPRB, 0.34332E+00_JPRB, 0.34637E+00_JPRB, 0.34783E+00_JPRB ,&
&  0.34626E+00_JPRB, 0.34476E+00_JPRB, 0.34365E+00_JPRB, 0.34557E+00_JPRB, 0.34769E+00_JPRB, 0.34892E+00_JPRB, 0.35014E+00_JPRB ,&
&  0.35310E+00_JPRB, 0.35606E+00_JPRB, 0.35655E+00_JPRB, 0.35619E+00_JPRB, 0.35578E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.37579E+00_JPRB, 0.37223E+00_JPRB, 0.36857E+00_JPRB, 0.36538E+00_JPRB, 0.36264E+00_JPRB, 0.35973E+00_JPRB, 0.35708E+00_JPRB ,&
&  0.35440E+00_JPRB, 0.35289E+00_JPRB, 0.35186E+00_JPRB, 0.35094E+00_JPRB, 0.34799E+00_JPRB, 0.34672E+00_JPRB, 0.34759E+00_JPRB ,&
&  0.35006E+00_JPRB, 0.35251E+00_JPRB, 0.35469E+00_JPRB, 0.35480E+00_JPRB, 0.35741E+00_JPRB, 0.35991E+00_JPRB, 0.36043E+00_JPRB ,&
&  0.36008E+00_JPRB, 0.35981E+00_JPRB, 0.35830E+00_JPRB, 0.35607E+00_JPRB, 0.35635E+00_JPRB, 0.35676E+00_JPRB, 0.35306E+00_JPRB ,&
&  0.34682E+00_JPRB, 0.34687E+00_JPRB, 0.33975E+00_JPRB, 0.35044E+00_JPRB, 0.35233E+00_JPRB, 0.34772E+00_JPRB, 0.34234E+00_JPRB ,&
&  0.34123E+00_JPRB, 0.34266E+00_JPRB, 0.34584E+00_JPRB, 0.34767E+00_JPRB, 0.34795E+00_JPRB, 0.34782E+00_JPRB, 0.34779E+00_JPRB ,&
&  0.34107E+00_JPRB, 0.33740E+00_JPRB, 0.33751E+00_JPRB, 0.33868E+00_JPRB, 0.33900E+00_JPRB, 0.33992E+00_JPRB, 0.34201E+00_JPRB ,&
&  0.34359E+00_JPRB, 0.34475E+00_JPRB, 0.34612E+00_JPRB, 0.34900E+00_JPRB, 0.35120E+00_JPRB, 0.35243E+00_JPRB, 0.35454E+00_JPRB ,&
&  0.35756E+00_JPRB, 0.36099E+00_JPRB, 0.36353E+00_JPRB, 0.36471E+00_JPRB, 0.36655E+00_JPRB, 0.36813E+00_JPRB, 0.36867E+00_JPRB ,&
&  0.36752E+00_JPRB, 0.36517E+00_JPRB, 0.36247E+00_JPRB, 0.35798E+00_JPRB, 0.35205E+00_JPRB, 0.34760E+00_JPRB, 0.34443E+00_JPRB ,&
&  0.33933E+00_JPRB, 0.33736E+00_JPRB, 0.33672E+00_JPRB, 0.33531E+00_JPRB, 0.33414E+00_JPRB, 0.33400E+00_JPRB, 0.33465E+00_JPRB ,&
&  0.33672E+00_JPRB, 0.34038E+00_JPRB, 0.34297E+00_JPRB, 0.34621E+00_JPRB, 0.35036E+00_JPRB, 0.35449E+00_JPRB, 0.35506E+00_JPRB ,&
&  0.35401E+00_JPRB, 0.35360E+00_JPRB, 0.35540E+00_JPRB, 0.36053E+00_JPRB, 0.36655E+00_JPRB, 0.37255E+00_JPRB, 0.37842E+00_JPRB ,&
&  0.38287E+00_JPRB, 0.38475E+00_JPRB, 0.38370E+00_JPRB, 0.38228E+00_JPRB, 0.37939E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.40460E+00_JPRB, 0.39794E+00_JPRB, 0.39101E+00_JPRB, 0.38581E+00_JPRB, 0.38211E+00_JPRB, 0.37814E+00_JPRB, 0.37332E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36491E+00_JPRB, 0.36135E+00_JPRB, 0.35804E+00_JPRB, 0.35524E+00_JPRB, 0.35445E+00_JPRB, 0.35551E+00_JPRB ,&
&  0.35815E+00_JPRB, 0.36352E+00_JPRB, 0.36693E+00_JPRB, 0.36622E+00_JPRB, 0.36835E+00_JPRB, 0.37079E+00_JPRB, 0.37159E+00_JPRB ,&
&  0.36983E+00_JPRB, 0.36726E+00_JPRB, 0.36437E+00_JPRB, 0.36274E+00_JPRB, 0.36287E+00_JPRB, 0.36387E+00_JPRB, 0.36058E+00_JPRB ,&
&  0.35565E+00_JPRB, 0.34594E+00_JPRB, 0.35523E+00_JPRB, 0.35855E+00_JPRB, 0.35900E+00_JPRB, 0.35770E+00_JPRB, 0.35516E+00_JPRB ,&
&  0.35064E+00_JPRB, 0.34785E+00_JPRB, 0.34847E+00_JPRB, 0.35011E+00_JPRB, 0.35217E+00_JPRB, 0.35281E+00_JPRB, 0.35357E+00_JPRB ,&
&  0.35052E+00_JPRB, 0.34914E+00_JPRB, 0.34797E+00_JPRB, 0.34754E+00_JPRB, 0.34864E+00_JPRB, 0.34986E+00_JPRB, 0.35060E+00_JPRB ,&
&  0.34985E+00_JPRB, 0.35083E+00_JPRB, 0.35217E+00_JPRB, 0.35194E+00_JPRB, 0.35386E+00_JPRB, 0.35650E+00_JPRB, 0.35980E+00_JPRB ,&
&  0.36362E+00_JPRB, 0.36833E+00_JPRB, 0.37307E+00_JPRB, 0.37857E+00_JPRB, 0.38449E+00_JPRB, 0.38880E+00_JPRB, 0.39138E+00_JPRB ,&
&  0.39189E+00_JPRB, 0.38997E+00_JPRB, 0.38600E+00_JPRB, 0.37954E+00_JPRB, 0.37390E+00_JPRB, 0.36956E+00_JPRB, 0.36369E+00_JPRB ,&
&  0.35590E+00_JPRB, 0.35136E+00_JPRB, 0.34893E+00_JPRB, 0.34808E+00_JPRB, 0.34683E+00_JPRB, 0.34650E+00_JPRB, 0.34787E+00_JPRB ,&
&  0.35082E+00_JPRB, 0.35382E+00_JPRB, 0.35576E+00_JPRB, 0.35715E+00_JPRB, 0.35911E+00_JPRB, 0.35892E+00_JPRB, 0.35905E+00_JPRB ,&
&  0.36154E+00_JPRB, 0.36681E+00_JPRB, 0.37577E+00_JPRB, 0.39034E+00_JPRB, 0.40308E+00_JPRB, 0.41210E+00_JPRB, 0.41770E+00_JPRB ,&
&  0.42005E+00_JPRB, 0.42017E+00_JPRB, 0.41762E+00_JPRB, 0.41459E+00_JPRB, 0.41071E+00_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.42308E+00_JPRB, 0.41405E+00_JPRB, 0.40460E+00_JPRB, 0.39848E+00_JPRB, 0.39429E+00_JPRB, 0.38997E+00_JPRB, 0.38471E+00_JPRB ,&
&  0.37869E+00_JPRB, 0.37346E+00_JPRB, 0.36858E+00_JPRB, 0.36375E+00_JPRB, 0.36166E+00_JPRB, 0.36092E+00_JPRB, 0.36203E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36986E+00_JPRB, 0.37257E+00_JPRB, 0.37305E+00_JPRB, 0.37403E+00_JPRB, 0.37409E+00_JPRB, 0.37279E+00_JPRB ,&
&  0.36924E+00_JPRB, 0.36485E+00_JPRB, 0.36265E+00_JPRB, 0.36378E+00_JPRB, 0.36421E+00_JPRB, 0.36476E+00_JPRB, 0.36402E+00_JPRB ,&
&  0.36710E+00_JPRB, 0.33274E+00_JPRB, 0.35336E+00_JPRB, 0.36260E+00_JPRB, 0.36083E+00_JPRB, 0.36108E+00_JPRB, 0.35874E+00_JPRB ,&
&  0.35325E+00_JPRB, 0.35017E+00_JPRB, 0.34845E+00_JPRB, 0.34965E+00_JPRB, 0.35218E+00_JPRB, 0.35321E+00_JPRB, 0.35324E+00_JPRB ,&
&  0.35060E+00_JPRB, 0.35079E+00_JPRB, 0.35071E+00_JPRB, 0.35118E+00_JPRB, 0.35225E+00_JPRB, 0.35327E+00_JPRB, 0.35358E+00_JPRB ,&
&  0.35269E+00_JPRB, 0.35024E+00_JPRB, 0.34775E+00_JPRB, 0.34793E+00_JPRB, 0.34983E+00_JPRB, 0.35374E+00_JPRB, 0.35999E+00_JPRB ,&
&  0.36778E+00_JPRB, 0.37613E+00_JPRB, 0.38469E+00_JPRB, 0.39299E+00_JPRB, 0.40110E+00_JPRB, 0.40750E+00_JPRB, 0.41206E+00_JPRB ,&
&  0.41147E+00_JPRB, 0.40821E+00_JPRB, 0.40158E+00_JPRB, 0.39313E+00_JPRB, 0.38668E+00_JPRB, 0.37995E+00_JPRB, 0.37308E+00_JPRB ,&
&  0.36773E+00_JPRB, 0.36279E+00_JPRB, 0.35976E+00_JPRB, 0.35940E+00_JPRB, 0.35980E+00_JPRB, 0.35948E+00_JPRB, 0.35992E+00_JPRB ,&
&  0.36077E+00_JPRB, 0.36173E+00_JPRB, 0.36265E+00_JPRB, 0.36328E+00_JPRB, 0.36272E+00_JPRB, 0.36209E+00_JPRB, 0.36362E+00_JPRB ,&
&  0.36782E+00_JPRB, 0.37550E+00_JPRB, 0.38844E+00_JPRB, 0.40676E+00_JPRB, 0.42540E+00_JPRB, 0.43924E+00_JPRB, 0.44570E+00_JPRB ,&
&  0.44496E+00_JPRB, 0.44136E+00_JPRB, 0.43539E+00_JPRB, 0.43208E+00_JPRB, 0.42977E+00_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.43699E+00_JPRB, 0.43258E+00_JPRB, 0.42759E+00_JPRB, 0.42221E+00_JPRB, 0.41620E+00_JPRB, 0.40899E+00_JPRB, 0.40115E+00_JPRB ,&
&  0.39489E+00_JPRB, 0.38950E+00_JPRB, 0.38427E+00_JPRB, 0.38049E+00_JPRB, 0.37731E+00_JPRB, 0.37675E+00_JPRB, 0.37738E+00_JPRB ,&
&  0.37849E+00_JPRB, 0.37854E+00_JPRB, 0.37824E+00_JPRB, 0.37915E+00_JPRB, 0.37930E+00_JPRB, 0.37751E+00_JPRB, 0.37478E+00_JPRB ,&
&  0.36997E+00_JPRB, 0.36587E+00_JPRB, 0.36471E+00_JPRB, 0.36752E+00_JPRB, 0.36696E+00_JPRB, 0.36871E+00_JPRB, 0.37501E+00_JPRB ,&
&  0.35804E+00_JPRB, 0.32767E+00_JPRB, 0.35205E+00_JPRB, 0.37666E+00_JPRB, 0.37151E+00_JPRB, 0.36651E+00_JPRB, 0.36392E+00_JPRB ,&
&  0.36038E+00_JPRB, 0.35874E+00_JPRB, 0.35738E+00_JPRB, 0.35719E+00_JPRB, 0.35853E+00_JPRB, 0.35818E+00_JPRB, 0.35654E+00_JPRB ,&
&  0.35540E+00_JPRB, 0.35767E+00_JPRB, 0.36006E+00_JPRB, 0.36141E+00_JPRB, 0.36143E+00_JPRB, 0.36084E+00_JPRB, 0.35799E+00_JPRB ,&
&  0.35539E+00_JPRB, 0.35406E+00_JPRB, 0.35459E+00_JPRB, 0.35828E+00_JPRB, 0.36412E+00_JPRB, 0.37292E+00_JPRB, 0.38493E+00_JPRB ,&
&  0.39747E+00_JPRB, 0.40912E+00_JPRB, 0.41980E+00_JPRB, 0.42828E+00_JPRB, 0.43523E+00_JPRB, 0.43999E+00_JPRB, 0.44186E+00_JPRB ,&
&  0.43654E+00_JPRB, 0.42662E+00_JPRB, 0.41473E+00_JPRB, 0.40609E+00_JPRB, 0.40004E+00_JPRB, 0.39600E+00_JPRB, 0.39353E+00_JPRB ,&
&  0.39281E+00_JPRB, 0.39185E+00_JPRB, 0.39022E+00_JPRB, 0.38939E+00_JPRB, 0.38906E+00_JPRB, 0.38781E+00_JPRB, 0.38704E+00_JPRB ,&
&  0.38646E+00_JPRB, 0.38511E+00_JPRB, 0.38270E+00_JPRB, 0.38027E+00_JPRB, 0.37838E+00_JPRB, 0.37891E+00_JPRB, 0.38136E+00_JPRB ,&
&  0.38729E+00_JPRB, 0.39893E+00_JPRB, 0.41402E+00_JPRB, 0.44138E+00_JPRB, 0.46064E+00_JPRB, 0.47424E+00_JPRB, 0.47947E+00_JPRB ,&
&  0.47406E+00_JPRB, 0.45968E+00_JPRB, 0.44850E+00_JPRB, 0.44262E+00_JPRB, 0.44293E+00_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.44575E+00_JPRB, 0.44540E+00_JPRB, 0.44110E+00_JPRB, 0.43531E+00_JPRB, 0.43000E+00_JPRB, 0.42319E+00_JPRB, 0.41462E+00_JPRB ,&
&  0.40665E+00_JPRB, 0.39893E+00_JPRB, 0.39450E+00_JPRB, 0.39081E+00_JPRB, 0.38513E+00_JPRB, 0.38254E+00_JPRB, 0.38376E+00_JPRB ,&
&  0.38352E+00_JPRB, 0.37974E+00_JPRB, 0.37501E+00_JPRB, 0.37485E+00_JPRB, 0.37367E+00_JPRB, 0.37173E+00_JPRB, 0.36870E+00_JPRB ,&
&  0.36537E+00_JPRB, 0.36462E+00_JPRB, 0.36578E+00_JPRB, 0.36758E+00_JPRB, 0.36504E+00_JPRB, 0.36652E+00_JPRB, 0.37465E+00_JPRB ,&
&  0.40975E+00_JPRB, 0.34524E+00_JPRB, 0.37354E+00_JPRB, 0.40560E+00_JPRB, 0.39723E+00_JPRB, 0.38917E+00_JPRB, 0.38138E+00_JPRB ,&
&  0.37664E+00_JPRB, 0.37598E+00_JPRB, 0.37611E+00_JPRB, 0.37347E+00_JPRB, 0.37015E+00_JPRB, 0.36592E+00_JPRB, 0.36412E+00_JPRB ,&
&  0.36393E+00_JPRB, 0.36775E+00_JPRB, 0.37026E+00_JPRB, 0.37073E+00_JPRB, 0.36872E+00_JPRB, 0.36676E+00_JPRB, 0.36543E+00_JPRB ,&
&  0.36552E+00_JPRB, 0.36747E+00_JPRB, 0.36996E+00_JPRB, 0.37301E+00_JPRB, 0.37815E+00_JPRB, 0.38804E+00_JPRB, 0.40223E+00_JPRB ,&
&  0.41878E+00_JPRB, 0.43294E+00_JPRB, 0.44502E+00_JPRB, 0.45496E+00_JPRB, 0.46176E+00_JPRB, 0.46435E+00_JPRB, 0.46069E+00_JPRB ,&
&  0.45235E+00_JPRB, 0.43806E+00_JPRB, 0.42853E+00_JPRB, 0.42116E+00_JPRB, 0.41357E+00_JPRB, 0.40843E+00_JPRB, 0.40547E+00_JPRB ,&
&  0.40413E+00_JPRB, 0.40318E+00_JPRB, 0.40166E+00_JPRB, 0.39898E+00_JPRB, 0.39813E+00_JPRB, 0.39785E+00_JPRB, 0.39924E+00_JPRB ,&
&  0.39920E+00_JPRB, 0.39822E+00_JPRB, 0.39492E+00_JPRB, 0.39044E+00_JPRB, 0.38973E+00_JPRB, 0.39092E+00_JPRB, 0.39362E+00_JPRB ,&
&  0.40013E+00_JPRB, 0.41201E+00_JPRB, 0.42748E+00_JPRB, 0.45638E+00_JPRB, 0.47171E+00_JPRB, 0.48297E+00_JPRB, 0.48530E+00_JPRB ,&
&  0.47693E+00_JPRB, 0.46291E+00_JPRB, 0.37813E+00_JPRB, 0.43181E+00_JPRB, 0.44839E+00_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.48527E+00_JPRB, 0.47916E+00_JPRB, 0.46855E+00_JPRB, 0.45230E+00_JPRB, 0.44058E+00_JPRB, 0.43181E+00_JPRB, 0.42273E+00_JPRB ,&
&  0.41461E+00_JPRB, 0.40825E+00_JPRB, 0.40335E+00_JPRB, 0.39737E+00_JPRB, 0.39061E+00_JPRB, 0.39003E+00_JPRB, 0.39090E+00_JPRB ,&
&  0.38904E+00_JPRB, 0.38325E+00_JPRB, 0.37754E+00_JPRB, 0.37671E+00_JPRB, 0.37655E+00_JPRB, 0.37663E+00_JPRB, 0.37415E+00_JPRB ,&
&  0.36971E+00_JPRB, 0.36945E+00_JPRB, 0.37355E+00_JPRB, 0.37532E+00_JPRB, 0.37212E+00_JPRB, 0.37422E+00_JPRB, 0.38279E+00_JPRB ,&
&  0.42397E+00_JPRB, 0.36733E+00_JPRB, 0.39217E+00_JPRB, 0.44722E+00_JPRB, 0.44383E+00_JPRB, 0.44350E+00_JPRB, 0.43397E+00_JPRB ,&
&  0.42462E+00_JPRB, 0.42073E+00_JPRB, 0.41699E+00_JPRB, 0.40604E+00_JPRB, 0.39354E+00_JPRB, 0.38623E+00_JPRB, 0.38465E+00_JPRB ,&
&  0.38689E+00_JPRB, 0.38896E+00_JPRB, 0.39061E+00_JPRB, 0.38816E+00_JPRB, 0.38725E+00_JPRB, 0.38708E+00_JPRB, 0.38780E+00_JPRB ,&
&  0.38862E+00_JPRB, 0.39133E+00_JPRB, 0.39420E+00_JPRB, 0.39492E+00_JPRB, 0.39976E+00_JPRB, 0.41532E+00_JPRB, 0.43681E+00_JPRB ,&
&  0.45678E+00_JPRB, 0.47373E+00_JPRB, 0.48443E+00_JPRB, 0.49129E+00_JPRB, 0.49213E+00_JPRB, 0.49147E+00_JPRB, 0.48596E+00_JPRB ,&
&  0.47240E+00_JPRB, 0.45817E+00_JPRB, 0.45021E+00_JPRB, 0.44141E+00_JPRB, 0.43266E+00_JPRB, 0.42637E+00_JPRB, 0.42109E+00_JPRB ,&
&  0.41695E+00_JPRB, 0.41378E+00_JPRB, 0.41149E+00_JPRB, 0.40707E+00_JPRB, 0.40427E+00_JPRB, 0.40538E+00_JPRB, 0.40637E+00_JPRB ,&
&  0.40648E+00_JPRB, 0.40805E+00_JPRB, 0.41014E+00_JPRB, 0.41254E+00_JPRB, 0.41501E+00_JPRB, 0.41847E+00_JPRB, 0.42257E+00_JPRB ,&
&  0.43391E+00_JPRB, 0.45130E+00_JPRB, 0.47162E+00_JPRB, 0.48138E+00_JPRB, 0.49985E+00_JPRB, 0.50274E+00_JPRB, 0.49398E+00_JPRB ,&
&  0.48010E+00_JPRB, 0.46409E+00_JPRB, 0.45691E+00_JPRB, 0.41333E+00_JPRB, 0.48639E+00_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.54100E+00_JPRB, 0.52111E+00_JPRB, 0.49006E+00_JPRB, 0.46050E+00_JPRB, 0.44006E+00_JPRB, 0.42712E+00_JPRB, 0.41971E+00_JPRB ,&
&  0.41448E+00_JPRB, 0.41052E+00_JPRB, 0.40595E+00_JPRB, 0.39837E+00_JPRB, 0.39398E+00_JPRB, 0.39270E+00_JPRB, 0.39217E+00_JPRB ,&
&  0.38984E+00_JPRB, 0.38216E+00_JPRB, 0.38000E+00_JPRB, 0.37806E+00_JPRB, 0.37735E+00_JPRB, 0.37658E+00_JPRB, 0.37285E+00_JPRB ,&
&  0.36788E+00_JPRB, 0.36581E+00_JPRB, 0.37026E+00_JPRB, 0.37318E+00_JPRB, 0.37579E+00_JPRB, 0.38114E+00_JPRB, 0.39405E+00_JPRB ,&
&  0.43674E+00_JPRB, 0.41352E+00_JPRB, 0.45916E+00_JPRB, 0.49349E+00_JPRB, 0.48757E+00_JPRB, 0.48136E+00_JPRB, 0.47828E+00_JPRB ,&
&  0.47128E+00_JPRB, 0.46561E+00_JPRB, 0.45110E+00_JPRB, 0.42877E+00_JPRB, 0.41034E+00_JPRB, 0.40695E+00_JPRB, 0.40300E+00_JPRB ,&
&  0.39987E+00_JPRB, 0.39964E+00_JPRB, 0.40059E+00_JPRB, 0.40222E+00_JPRB, 0.40359E+00_JPRB, 0.40388E+00_JPRB, 0.40416E+00_JPRB ,&
&  0.40532E+00_JPRB, 0.40735E+00_JPRB, 0.40796E+00_JPRB, 0.41147E+00_JPRB, 0.42371E+00_JPRB, 0.44305E+00_JPRB, 0.46387E+00_JPRB ,&
&  0.47956E+00_JPRB, 0.49196E+00_JPRB, 0.50592E+00_JPRB, 0.51469E+00_JPRB, 0.51637E+00_JPRB, 0.51153E+00_JPRB, 0.50159E+00_JPRB ,&
&  0.48987E+00_JPRB, 0.48528E+00_JPRB, 0.47250E+00_JPRB, 0.45839E+00_JPRB, 0.44957E+00_JPRB, 0.44435E+00_JPRB, 0.44078E+00_JPRB ,&
&  0.43839E+00_JPRB, 0.43465E+00_JPRB, 0.42927E+00_JPRB, 0.42452E+00_JPRB, 0.42295E+00_JPRB, 0.41996E+00_JPRB, 0.41574E+00_JPRB ,&
&  0.41506E+00_JPRB, 0.41885E+00_JPRB, 0.42570E+00_JPRB, 0.43069E+00_JPRB, 0.43546E+00_JPRB, 0.44266E+00_JPRB, 0.44834E+00_JPRB ,&
&  0.45804E+00_JPRB, 0.47464E+00_JPRB, 0.48963E+00_JPRB, 0.51514E+00_JPRB, 0.50881E+00_JPRB, 0.50427E+00_JPRB, 0.48870E+00_JPRB ,&
&  0.47320E+00_JPRB, 0.46056E+00_JPRB, 0.44909E+00_JPRB, 0.47239E+00_JPRB, 0.53404E+00_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.58343E+00_JPRB, 0.53368E+00_JPRB, 0.49265E+00_JPRB, 0.45963E+00_JPRB, 0.43967E+00_JPRB, 0.42815E+00_JPRB, 0.41994E+00_JPRB ,&
&  0.41651E+00_JPRB, 0.41556E+00_JPRB, 0.41058E+00_JPRB, 0.40663E+00_JPRB, 0.40423E+00_JPRB, 0.40387E+00_JPRB, 0.40408E+00_JPRB ,&
&  0.39726E+00_JPRB, 0.39639E+00_JPRB, 0.39685E+00_JPRB, 0.39328E+00_JPRB, 0.38830E+00_JPRB, 0.38330E+00_JPRB, 0.37671E+00_JPRB ,&
&  0.37261E+00_JPRB, 0.37175E+00_JPRB, 0.37465E+00_JPRB, 0.37910E+00_JPRB, 0.38320E+00_JPRB, 0.39432E+00_JPRB, 0.41473E+00_JPRB ,&
&  0.46161E+00_JPRB, 0.45996E+00_JPRB, 0.53033E+00_JPRB, 0.55545E+00_JPRB, 0.51118E+00_JPRB, 0.50979E+00_JPRB, 0.51549E+00_JPRB ,&
&  0.52122E+00_JPRB, 0.52289E+00_JPRB, 0.50485E+00_JPRB, 0.47812E+00_JPRB, 0.45469E+00_JPRB, 0.44206E+00_JPRB, 0.43477E+00_JPRB ,&
&  0.42775E+00_JPRB, 0.42336E+00_JPRB, 0.41982E+00_JPRB, 0.42332E+00_JPRB, 0.42645E+00_JPRB, 0.42604E+00_JPRB, 0.42521E+00_JPRB ,&
&  0.42403E+00_JPRB, 0.42339E+00_JPRB, 0.42416E+00_JPRB, 0.43556E+00_JPRB, 0.45814E+00_JPRB, 0.48364E+00_JPRB, 0.50693E+00_JPRB ,&
&  0.52069E+00_JPRB, 0.53356E+00_JPRB, 0.54561E+00_JPRB, 0.55214E+00_JPRB, 0.55005E+00_JPRB, 0.54370E+00_JPRB, 0.53689E+00_JPRB ,&
&  0.53010E+00_JPRB, 0.51466E+00_JPRB, 0.49279E+00_JPRB, 0.47673E+00_JPRB, 0.46921E+00_JPRB, 0.46521E+00_JPRB, 0.46488E+00_JPRB ,&
&  0.46516E+00_JPRB, 0.46522E+00_JPRB, 0.46244E+00_JPRB, 0.45667E+00_JPRB, 0.44754E+00_JPRB, 0.44122E+00_JPRB, 0.43528E+00_JPRB ,&
&  0.43226E+00_JPRB, 0.43346E+00_JPRB, 0.43841E+00_JPRB, 0.44698E+00_JPRB, 0.45430E+00_JPRB, 0.46155E+00_JPRB, 0.46800E+00_JPRB ,&
&  0.47962E+00_JPRB, 0.50285E+00_JPRB, 0.49177E+00_JPRB, 0.49496E+00_JPRB, 0.53190E+00_JPRB, 0.51276E+00_JPRB, 0.49112E+00_JPRB ,&
&  0.47315E+00_JPRB, 0.46062E+00_JPRB, 0.46066E+00_JPRB, 0.48967E+00_JPRB, 0.61198E+00_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.49506E+00_JPRB, 0.47840E+00_JPRB, 0.46225E+00_JPRB, 0.44563E+00_JPRB, 0.43064E+00_JPRB, 0.42326E+00_JPRB, 0.42146E+00_JPRB ,&
&  0.42382E+00_JPRB, 0.42600E+00_JPRB, 0.42439E+00_JPRB, 0.41875E+00_JPRB, 0.41554E+00_JPRB, 0.41276E+00_JPRB, 0.41091E+00_JPRB ,&
&  0.40564E+00_JPRB, 0.40655E+00_JPRB, 0.40345E+00_JPRB, 0.39840E+00_JPRB, 0.39287E+00_JPRB, 0.38609E+00_JPRB, 0.37955E+00_JPRB ,&
&  0.37587E+00_JPRB, 0.37538E+00_JPRB, 0.37742E+00_JPRB, 0.38230E+00_JPRB, 0.39423E+00_JPRB, 0.40570E+00_JPRB, 0.41792E+00_JPRB ,&
&  0.46229E+00_JPRB, 0.52093E+00_JPRB, 0.63235E+00_JPRB, 0.69347E+00_JPRB, 0.62289E+00_JPRB, 0.59183E+00_JPRB, 0.58377E+00_JPRB ,&
&  0.57841E+00_JPRB, 0.56717E+00_JPRB, 0.54579E+00_JPRB, 0.52594E+00_JPRB, 0.50497E+00_JPRB, 0.49676E+00_JPRB, 0.48662E+00_JPRB ,&
&  0.47525E+00_JPRB, 0.46600E+00_JPRB, 0.45470E+00_JPRB, 0.45227E+00_JPRB, 0.45101E+00_JPRB, 0.44789E+00_JPRB, 0.44384E+00_JPRB ,&
&  0.44187E+00_JPRB, 0.44221E+00_JPRB, 0.44587E+00_JPRB, 0.46346E+00_JPRB, 0.49635E+00_JPRB, 0.53016E+00_JPRB, 0.56822E+00_JPRB ,&
&  0.58009E+00_JPRB, 0.58731E+00_JPRB, 0.59225E+00_JPRB, 0.59022E+00_JPRB, 0.57602E+00_JPRB, 0.56543E+00_JPRB, 0.56476E+00_JPRB ,&
&  0.55792E+00_JPRB, 0.53258E+00_JPRB, 0.50510E+00_JPRB, 0.48766E+00_JPRB, 0.47834E+00_JPRB, 0.47664E+00_JPRB, 0.47705E+00_JPRB ,&
&  0.48059E+00_JPRB, 0.48290E+00_JPRB, 0.47709E+00_JPRB, 0.46578E+00_JPRB, 0.45433E+00_JPRB, 0.44728E+00_JPRB, 0.44413E+00_JPRB ,&
&  0.44100E+00_JPRB, 0.44186E+00_JPRB, 0.44928E+00_JPRB, 0.45458E+00_JPRB, 0.45731E+00_JPRB, 0.46266E+00_JPRB, 0.47259E+00_JPRB ,&
&  0.48814E+00_JPRB, 0.49638E+00_JPRB, 0.49238E+00_JPRB, 0.49637E+00_JPRB, 0.49295E+00_JPRB, 0.50699E+00_JPRB, 0.49023E+00_JPRB ,&
&  0.47094E+00_JPRB, 0.46662E+00_JPRB, 0.48006E+00_JPRB, 0.49690E+00_JPRB, 0.50998E+00_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.47591E+00_JPRB, 0.46314E+00_JPRB, 0.45156E+00_JPRB, 0.43303E+00_JPRB, 0.42161E+00_JPRB, 0.41545E+00_JPRB, 0.41759E+00_JPRB ,&
&  0.42345E+00_JPRB, 0.43089E+00_JPRB, 0.42857E+00_JPRB, 0.42588E+00_JPRB, 0.42114E+00_JPRB, 0.41863E+00_JPRB, 0.41243E+00_JPRB ,&
&  0.40934E+00_JPRB, 0.40611E+00_JPRB, 0.40087E+00_JPRB, 0.39580E+00_JPRB, 0.38758E+00_JPRB, 0.38435E+00_JPRB, 0.37967E+00_JPRB ,&
&  0.37506E+00_JPRB, 0.37350E+00_JPRB, 0.37626E+00_JPRB, 0.38547E+00_JPRB, 0.39673E+00_JPRB, 0.40935E+00_JPRB, 0.42226E+00_JPRB ,&
&  0.48294E+00_JPRB, 0.51400E+00_JPRB, 0.79406E+00_JPRB, 0.82101E+00_JPRB, 0.76796E+00_JPRB, 0.71293E+00_JPRB, 0.71453E+00_JPRB ,&
&  0.70721E+00_JPRB, 0.66482E+00_JPRB, 0.62742E+00_JPRB, 0.60204E+00_JPRB, 0.57944E+00_JPRB, 0.56377E+00_JPRB, 0.54321E+00_JPRB ,&
&  0.52458E+00_JPRB, 0.50416E+00_JPRB, 0.47896E+00_JPRB, 0.46394E+00_JPRB, 0.45495E+00_JPRB, 0.45209E+00_JPRB, 0.45556E+00_JPRB ,&
&  0.45999E+00_JPRB, 0.46200E+00_JPRB, 0.47315E+00_JPRB, 0.49600E+00_JPRB, 0.50945E+00_JPRB, 0.49668E+00_JPRB, 0.54302E+00_JPRB ,&
&  0.64898E+00_JPRB, 0.64489E+00_JPRB, 0.62274E+00_JPRB, 0.60724E+00_JPRB, 0.57786E+00_JPRB, 0.56705E+00_JPRB, 0.56502E+00_JPRB ,&
&  0.55556E+00_JPRB, 0.53763E+00_JPRB, 0.51765E+00_JPRB, 0.50165E+00_JPRB, 0.49068E+00_JPRB, 0.48496E+00_JPRB, 0.48869E+00_JPRB ,&
&  0.48991E+00_JPRB, 0.48242E+00_JPRB, 0.46648E+00_JPRB, 0.45328E+00_JPRB, 0.44655E+00_JPRB, 0.44594E+00_JPRB, 0.44810E+00_JPRB ,&
&  0.45000E+00_JPRB, 0.45274E+00_JPRB, 0.45399E+00_JPRB, 0.44358E+00_JPRB, 0.45124E+00_JPRB, 0.46810E+00_JPRB, 0.47357E+00_JPRB ,&
&  0.46358E+00_JPRB, 0.46406E+00_JPRB, 0.47588E+00_JPRB, 0.47894E+00_JPRB, 0.47983E+00_JPRB, 0.50916E+00_JPRB, 0.49310E+00_JPRB ,&
&  0.47343E+00_JPRB, 0.47449E+00_JPRB, 0.49125E+00_JPRB, 0.49921E+00_JPRB, 0.49952E+00_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.46042E+00_JPRB, 0.44708E+00_JPRB, 0.43818E+00_JPRB, 0.42564E+00_JPRB, 0.41422E+00_JPRB, 0.40766E+00_JPRB, 0.41019E+00_JPRB ,&
&  0.42273E+00_JPRB, 0.43452E+00_JPRB, 0.42970E+00_JPRB, 0.42810E+00_JPRB, 0.42809E+00_JPRB, 0.42323E+00_JPRB, 0.41479E+00_JPRB ,&
&  0.40627E+00_JPRB, 0.39816E+00_JPRB, 0.38907E+00_JPRB, 0.38007E+00_JPRB, 0.37371E+00_JPRB, 0.37454E+00_JPRB, 0.37335E+00_JPRB ,&
&  0.37165E+00_JPRB, 0.37495E+00_JPRB, 0.37994E+00_JPRB, 0.38890E+00_JPRB, 0.39723E+00_JPRB, 0.40891E+00_JPRB, 0.42382E+00_JPRB ,&
&  0.48391E+00_JPRB, 0.40061E+00_JPRB, 0.82375E+00_JPRB, 0.99133E+00_JPRB, 0.10302E+01_JPRB, 0.96268E+00_JPRB, 0.90172E+00_JPRB ,&
&  0.87425E+00_JPRB, 0.78553E+00_JPRB, 0.72235E+00_JPRB, 0.66605E+00_JPRB, 0.60151E+00_JPRB, 0.56318E+00_JPRB, 0.54286E+00_JPRB ,&
&  0.52770E+00_JPRB, 0.51152E+00_JPRB, 0.49685E+00_JPRB, 0.48445E+00_JPRB, 0.47449E+00_JPRB, 0.47156E+00_JPRB, 0.47841E+00_JPRB ,&
&  0.48677E+00_JPRB, 0.49165E+00_JPRB, 0.51189E+00_JPRB, 0.53664E+00_JPRB, 0.52255E+00_JPRB, 0.52105E+00_JPRB, 0.53801E+00_JPRB ,&
&  0.59655E+00_JPRB, 0.67003E+00_JPRB, 0.62609E+00_JPRB, 0.58125E+00_JPRB, 0.55424E+00_JPRB, 0.54919E+00_JPRB, 0.55155E+00_JPRB ,&
&  0.54528E+00_JPRB, 0.53619E+00_JPRB, 0.52473E+00_JPRB, 0.51079E+00_JPRB, 0.49869E+00_JPRB, 0.49302E+00_JPRB, 0.48896E+00_JPRB ,&
&  0.47985E+00_JPRB, 0.46479E+00_JPRB, 0.45513E+00_JPRB, 0.45027E+00_JPRB, 0.44841E+00_JPRB, 0.44863E+00_JPRB, 0.45312E+00_JPRB ,&
&  0.46170E+00_JPRB, 0.46305E+00_JPRB, 0.45302E+00_JPRB, 0.43870E+00_JPRB, 0.43567E+00_JPRB, 0.44030E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.45305E+00_JPRB, 0.46062E+00_JPRB, 0.46536E+00_JPRB, 0.47410E+00_JPRB, 0.46103E+00_JPRB, 0.49800E+00_JPRB, 0.48572E+00_JPRB ,&
&  0.47750E+00_JPRB, 0.47873E+00_JPRB, 0.48738E+00_JPRB, 0.49026E+00_JPRB, 0.48028E+00_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.44128E+00_JPRB, 0.43211E+00_JPRB, 0.42364E+00_JPRB, 0.41949E+00_JPRB, 0.41177E+00_JPRB, 0.40778E+00_JPRB, 0.41140E+00_JPRB ,&
&  0.42153E+00_JPRB, 0.42494E+00_JPRB, 0.42346E+00_JPRB, 0.42135E+00_JPRB, 0.41751E+00_JPRB, 0.41231E+00_JPRB, 0.40462E+00_JPRB ,&
&  0.39794E+00_JPRB, 0.38731E+00_JPRB, 0.37377E+00_JPRB, 0.36443E+00_JPRB, 0.36934E+00_JPRB, 0.37418E+00_JPRB, 0.37351E+00_JPRB ,&
&  0.37228E+00_JPRB, 0.37791E+00_JPRB, 0.38027E+00_JPRB, 0.39015E+00_JPRB, 0.39711E+00_JPRB, 0.41018E+00_JPRB, 0.42775E+00_JPRB ,&
&  0.48271E+00_JPRB, 0.68076E+00_JPRB, 0.73680E+00_JPRB, 0.11194E+01_JPRB, 0.12797E+01_JPRB, 0.12143E+01_JPRB, 0.10295E+01_JPRB ,&
&  0.93699E+00_JPRB, 0.87716E+00_JPRB, 0.77754E+00_JPRB, 0.69533E+00_JPRB, 0.63918E+00_JPRB, 0.59716E+00_JPRB, 0.55952E+00_JPRB ,&
&  0.52903E+00_JPRB, 0.50945E+00_JPRB, 0.49805E+00_JPRB, 0.48772E+00_JPRB, 0.48561E+00_JPRB, 0.48959E+00_JPRB, 0.50092E+00_JPRB ,&
&  0.51202E+00_JPRB, 0.51880E+00_JPRB, 0.54866E+00_JPRB, 0.57646E+00_JPRB, 0.53752E+00_JPRB, 0.54720E+00_JPRB, 0.59052E+00_JPRB ,&
&  0.61425E+00_JPRB, 0.68671E+00_JPRB, 0.60809E+00_JPRB, 0.55780E+00_JPRB, 0.54178E+00_JPRB, 0.54803E+00_JPRB, 0.54957E+00_JPRB ,&
&  0.54067E+00_JPRB, 0.53337E+00_JPRB, 0.52214E+00_JPRB, 0.51057E+00_JPRB, 0.50181E+00_JPRB, 0.49470E+00_JPRB, 0.48094E+00_JPRB ,&
&  0.46997E+00_JPRB, 0.46704E+00_JPRB, 0.46714E+00_JPRB, 0.46740E+00_JPRB, 0.46913E+00_JPRB, 0.47296E+00_JPRB, 0.47623E+00_JPRB ,&
&  0.47888E+00_JPRB, 0.47046E+00_JPRB, 0.44698E+00_JPRB, 0.43362E+00_JPRB, 0.43410E+00_JPRB, 0.43728E+00_JPRB, 0.43592E+00_JPRB ,&
&  0.45756E+00_JPRB, 0.46745E+00_JPRB, 0.46593E+00_JPRB, 0.46992E+00_JPRB, 0.47552E+00_JPRB, 0.52406E+00_JPRB, 0.49013E+00_JPRB ,&
&  0.48139E+00_JPRB, 0.47788E+00_JPRB, 0.48069E+00_JPRB, 0.48407E+00_JPRB, 0.46644E+00_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.42593E+00_JPRB, 0.41504E+00_JPRB, 0.40918E+00_JPRB, 0.40887E+00_JPRB, 0.40747E+00_JPRB, 0.40622E+00_JPRB, 0.41080E+00_JPRB ,&
&  0.41283E+00_JPRB, 0.41301E+00_JPRB, 0.41139E+00_JPRB, 0.40910E+00_JPRB, 0.40356E+00_JPRB, 0.39779E+00_JPRB, 0.39136E+00_JPRB ,&
&  0.38818E+00_JPRB, 0.37392E+00_JPRB, 0.36675E+00_JPRB, 0.37519E+00_JPRB, 0.37947E+00_JPRB, 0.37842E+00_JPRB, 0.37865E+00_JPRB ,&
&  0.37837E+00_JPRB, 0.38164E+00_JPRB, 0.38176E+00_JPRB, 0.38688E+00_JPRB, 0.39613E+00_JPRB, 0.41611E+00_JPRB, 0.43799E+00_JPRB ,&
&  0.51237E+00_JPRB, 0.65365E+00_JPRB, 0.73090E+00_JPRB, 0.12558E+01_JPRB, 0.13751E+01_JPRB, 0.12996E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.96362E+00_JPRB, 0.95453E+00_JPRB, 0.85050E+00_JPRB, 0.75892E+00_JPRB, 0.69997E+00_JPRB, 0.64517E+00_JPRB, 0.58219E+00_JPRB ,&
&  0.53023E+00_JPRB, 0.50239E+00_JPRB, 0.49375E+00_JPRB, 0.49765E+00_JPRB, 0.50694E+00_JPRB, 0.51438E+00_JPRB, 0.52464E+00_JPRB ,&
&  0.53346E+00_JPRB, 0.55654E+00_JPRB, 0.59240E+00_JPRB, 0.57452E+00_JPRB, 0.54461E+00_JPRB, 0.56476E+00_JPRB, 0.58762E+00_JPRB ,&
&  0.61819E+00_JPRB, 0.63595E+00_JPRB, 0.57378E+00_JPRB, 0.54172E+00_JPRB, 0.53126E+00_JPRB, 0.54635E+00_JPRB, 0.54497E+00_JPRB ,&
&  0.53393E+00_JPRB, 0.52511E+00_JPRB, 0.51253E+00_JPRB, 0.50489E+00_JPRB, 0.49920E+00_JPRB, 0.48823E+00_JPRB, 0.48190E+00_JPRB ,&
&  0.48230E+00_JPRB, 0.48406E+00_JPRB, 0.48995E+00_JPRB, 0.49242E+00_JPRB, 0.49427E+00_JPRB, 0.49332E+00_JPRB, 0.49053E+00_JPRB ,&
&  0.48219E+00_JPRB, 0.46397E+00_JPRB, 0.44460E+00_JPRB, 0.44573E+00_JPRB, 0.45764E+00_JPRB, 0.45842E+00_JPRB, 0.45567E+00_JPRB ,&
&  0.47100E+00_JPRB, 0.47980E+00_JPRB, 0.48018E+00_JPRB, 0.47125E+00_JPRB, 0.48265E+00_JPRB, 0.50947E+00_JPRB, 0.48520E+00_JPRB ,&
&  0.48434E+00_JPRB, 0.47998E+00_JPRB, 0.47874E+00_JPRB, 0.47708E+00_JPRB, 0.45327E+00_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.43137E+00_JPRB, 0.41361E+00_JPRB, 0.40510E+00_JPRB, 0.39944E+00_JPRB, 0.39811E+00_JPRB, 0.40116E+00_JPRB, 0.40367E+00_JPRB ,&
&  0.40161E+00_JPRB, 0.40076E+00_JPRB, 0.40196E+00_JPRB, 0.39906E+00_JPRB, 0.39030E+00_JPRB, 0.38263E+00_JPRB, 0.37628E+00_JPRB ,&
&  0.37753E+00_JPRB, 0.37998E+00_JPRB, 0.39060E+00_JPRB, 0.39756E+00_JPRB, 0.39895E+00_JPRB, 0.39691E+00_JPRB, 0.39707E+00_JPRB ,&
&  0.39922E+00_JPRB, 0.39747E+00_JPRB, 0.39456E+00_JPRB, 0.40266E+00_JPRB, 0.41677E+00_JPRB, 0.44348E+00_JPRB, 0.48109E+00_JPRB ,&
&  0.58663E+00_JPRB, 0.68309E+00_JPRB, 0.76493E+00_JPRB, 0.11043E+01_JPRB, 0.13071E+01_JPRB, 0.12251E+01_JPRB, 0.11002E+01_JPRB ,&
&  0.95669E+00_JPRB, 0.83935E+00_JPRB, 0.87761E+00_JPRB, 0.79234E+00_JPRB, 0.71972E+00_JPRB, 0.65720E+00_JPRB, 0.60367E+00_JPRB ,&
&  0.56036E+00_JPRB, 0.53594E+00_JPRB, 0.53943E+00_JPRB, 0.55054E+00_JPRB, 0.55576E+00_JPRB, 0.55457E+00_JPRB, 0.56078E+00_JPRB ,&
&  0.57715E+00_JPRB, 0.61344E+00_JPRB, 0.64009E+00_JPRB, 0.60631E+00_JPRB, 0.54163E+00_JPRB, 0.56667E+00_JPRB, 0.58309E+00_JPRB ,&
&  0.61179E+00_JPRB, 0.59073E+00_JPRB, 0.55928E+00_JPRB, 0.53964E+00_JPRB, 0.51717E+00_JPRB, 0.54295E+00_JPRB, 0.53514E+00_JPRB ,&
&  0.52774E+00_JPRB, 0.51602E+00_JPRB, 0.50410E+00_JPRB, 0.49810E+00_JPRB, 0.49304E+00_JPRB, 0.49070E+00_JPRB, 0.48779E+00_JPRB ,&
&  0.48742E+00_JPRB, 0.48846E+00_JPRB, 0.49603E+00_JPRB, 0.49859E+00_JPRB, 0.49656E+00_JPRB, 0.48633E+00_JPRB, 0.47106E+00_JPRB ,&
&  0.46050E+00_JPRB, 0.46320E+00_JPRB, 0.45904E+00_JPRB, 0.46969E+00_JPRB, 0.47888E+00_JPRB, 0.48181E+00_JPRB, 0.47277E+00_JPRB ,&
&  0.47421E+00_JPRB, 0.48759E+00_JPRB, 0.49320E+00_JPRB, 0.49652E+00_JPRB, 0.49973E+00_JPRB, 0.50241E+00_JPRB, 0.48339E+00_JPRB ,&
&  0.47684E+00_JPRB, 0.47155E+00_JPRB, 0.48222E+00_JPRB, 0.47725E+00_JPRB, 0.45031E+00_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.43965E+00_JPRB, 0.43222E+00_JPRB, 0.39513E+00_JPRB, 0.39565E+00_JPRB, 0.39465E+00_JPRB, 0.39785E+00_JPRB, 0.39202E+00_JPRB ,&
&  0.38806E+00_JPRB, 0.38968E+00_JPRB, 0.39260E+00_JPRB, 0.38689E+00_JPRB, 0.36840E+00_JPRB, 0.36032E+00_JPRB, 0.36433E+00_JPRB ,&
&  0.37459E+00_JPRB, 0.39374E+00_JPRB, 0.41703E+00_JPRB, 0.41999E+00_JPRB, 0.42331E+00_JPRB, 0.42157E+00_JPRB, 0.41814E+00_JPRB ,&
&  0.41932E+00_JPRB, 0.42276E+00_JPRB, 0.42872E+00_JPRB, 0.44418E+00_JPRB, 0.47052E+00_JPRB, 0.50895E+00_JPRB, 0.56721E+00_JPRB ,&
&  0.70678E+00_JPRB, 0.74084E+00_JPRB, 0.41203E+00_JPRB, 0.70748E+00_JPRB, 0.10174E+01_JPRB, 0.10562E+01_JPRB, 0.98878E+00_JPRB ,&
&  0.90868E+00_JPRB, 0.79747E+00_JPRB, 0.78469E+00_JPRB, 0.78109E+00_JPRB, 0.71165E+00_JPRB, 0.67041E+00_JPRB, 0.63063E+00_JPRB ,&
&  0.59602E+00_JPRB, 0.58778E+00_JPRB, 0.59846E+00_JPRB, 0.61082E+00_JPRB, 0.60828E+00_JPRB, 0.60956E+00_JPRB, 0.62446E+00_JPRB ,&
&  0.64917E+00_JPRB, 0.67566E+00_JPRB, 0.67910E+00_JPRB, 0.59434E+00_JPRB, 0.55787E+00_JPRB, 0.57336E+00_JPRB, 0.56849E+00_JPRB ,&
&  0.55364E+00_JPRB, 0.56468E+00_JPRB, 0.55984E+00_JPRB, 0.53624E+00_JPRB, 0.51335E+00_JPRB, 0.50783E+00_JPRB, 0.52395E+00_JPRB ,&
&  0.51596E+00_JPRB, 0.49998E+00_JPRB, 0.48934E+00_JPRB, 0.48465E+00_JPRB, 0.48033E+00_JPRB, 0.47959E+00_JPRB, 0.47649E+00_JPRB ,&
&  0.47648E+00_JPRB, 0.48210E+00_JPRB, 0.48650E+00_JPRB, 0.48373E+00_JPRB, 0.47489E+00_JPRB, 0.46221E+00_JPRB, 0.45198E+00_JPRB ,&
&  0.45608E+00_JPRB, 0.47009E+00_JPRB, 0.48817E+00_JPRB, 0.49119E+00_JPRB, 0.49102E+00_JPRB, 0.49614E+00_JPRB, 0.49520E+00_JPRB ,&
&  0.49753E+00_JPRB, 0.50658E+00_JPRB, 0.49976E+00_JPRB, 0.49480E+00_JPRB, 0.51162E+00_JPRB, 0.50504E+00_JPRB, 0.50566E+00_JPRB ,&
&  0.54837E+00_JPRB, 0.63399E+00_JPRB, 0.77082E+00_JPRB, 0.51888E+00_JPRB, 0.46860E+00_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.44994E+00_JPRB, 0.41893E+00_JPRB, 0.38492E+00_JPRB, 0.38306E+00_JPRB, 0.39749E+00_JPRB, 0.39921E+00_JPRB, 0.39442E+00_JPRB ,&
&  0.38377E+00_JPRB, 0.37436E+00_JPRB, 0.36302E+00_JPRB, 0.35598E+00_JPRB, 0.35902E+00_JPRB, 0.36667E+00_JPRB, 0.39318E+00_JPRB ,&
&  0.41972E+00_JPRB, 0.44261E+00_JPRB, 0.46155E+00_JPRB, 0.46553E+00_JPRB, 0.45914E+00_JPRB, 0.46070E+00_JPRB, 0.46689E+00_JPRB ,&
&  0.46891E+00_JPRB, 0.48192E+00_JPRB, 0.50015E+00_JPRB, 0.52640E+00_JPRB, 0.56391E+00_JPRB, 0.62170E+00_JPRB, 0.71157E+00_JPRB ,&
&  0.84893E+00_JPRB, 0.60518E+00_JPRB, 0.46025E+00_JPRB, 0.83056E+00_JPRB, 0.99047E+00_JPRB, 0.10094E+01_JPRB, 0.85660E+00_JPRB ,&
&  0.80774E+00_JPRB, 0.76222E+00_JPRB, 0.75317E+00_JPRB, 0.77617E+00_JPRB, 0.70785E+00_JPRB, 0.68304E+00_JPRB, 0.65733E+00_JPRB ,&
&  0.64160E+00_JPRB, 0.64470E+00_JPRB, 0.65899E+00_JPRB, 0.67122E+00_JPRB, 0.67611E+00_JPRB, 0.69365E+00_JPRB, 0.71347E+00_JPRB ,&
&  0.73434E+00_JPRB, 0.73360E+00_JPRB, 0.71858E+00_JPRB, 0.60588E+00_JPRB, 0.57460E+00_JPRB, 0.57244E+00_JPRB, 0.56941E+00_JPRB ,&
&  0.54600E+00_JPRB, 0.54612E+00_JPRB, 0.54589E+00_JPRB, 0.52700E+00_JPRB, 0.51002E+00_JPRB, 0.49594E+00_JPRB, 0.51040E+00_JPRB ,&
&  0.49450E+00_JPRB, 0.47610E+00_JPRB, 0.46277E+00_JPRB, 0.45593E+00_JPRB, 0.45571E+00_JPRB, 0.45815E+00_JPRB, 0.45428E+00_JPRB ,&
&  0.45911E+00_JPRB, 0.46081E+00_JPRB, 0.46501E+00_JPRB, 0.45855E+00_JPRB, 0.45440E+00_JPRB, 0.45295E+00_JPRB, 0.44447E+00_JPRB ,&
&  0.46277E+00_JPRB, 0.48754E+00_JPRB, 0.49386E+00_JPRB, 0.50573E+00_JPRB, 0.50276E+00_JPRB, 0.49418E+00_JPRB, 0.50148E+00_JPRB ,&
&  0.50196E+00_JPRB, 0.51412E+00_JPRB, 0.51018E+00_JPRB, 0.51743E+00_JPRB, 0.51405E+00_JPRB, 0.51912E+00_JPRB, 0.53782E+00_JPRB ,&
&  0.58842E+00_JPRB, 0.64926E+00_JPRB, 0.65395E+00_JPRB, 0.51127E+00_JPRB, 0.46969E+00_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.43016E+00_JPRB, 0.40638E+00_JPRB, 0.37009E+00_JPRB, 0.35666E+00_JPRB, 0.37481E+00_JPRB, 0.38812E+00_JPRB, 0.38044E+00_JPRB ,&
&  0.37330E+00_JPRB, 0.36522E+00_JPRB, 0.36092E+00_JPRB, 0.36663E+00_JPRB, 0.39333E+00_JPRB, 0.42900E+00_JPRB, 0.45228E+00_JPRB ,&
&  0.47631E+00_JPRB, 0.49506E+00_JPRB, 0.51158E+00_JPRB, 0.51782E+00_JPRB, 0.51656E+00_JPRB, 0.52261E+00_JPRB, 0.52966E+00_JPRB ,&
&  0.54309E+00_JPRB, 0.56507E+00_JPRB, 0.59549E+00_JPRB, 0.63886E+00_JPRB, 0.69295E+00_JPRB, 0.75688E+00_JPRB, 0.81822E+00_JPRB ,&
&  0.61084E+00_JPRB, 0.36589E+00_JPRB, 0.78260E+00_JPRB, 0.91227E+00_JPRB, 0.85950E+00_JPRB, 0.84109E+00_JPRB, 0.78035E+00_JPRB ,&
&  0.73273E+00_JPRB, 0.73400E+00_JPRB, 0.70756E+00_JPRB, 0.74009E+00_JPRB, 0.68919E+00_JPRB, 0.68851E+00_JPRB, 0.69322E+00_JPRB ,&
&  0.70511E+00_JPRB, 0.71743E+00_JPRB, 0.73390E+00_JPRB, 0.74882E+00_JPRB, 0.76475E+00_JPRB, 0.79106E+00_JPRB, 0.80216E+00_JPRB ,&
&  0.81401E+00_JPRB, 0.80805E+00_JPRB, 0.78308E+00_JPRB, 0.63527E+00_JPRB, 0.59203E+00_JPRB, 0.57483E+00_JPRB, 0.56207E+00_JPRB ,&
&  0.54515E+00_JPRB, 0.51632E+00_JPRB, 0.50367E+00_JPRB, 0.51607E+00_JPRB, 0.50934E+00_JPRB, 0.48395E+00_JPRB, 0.49346E+00_JPRB ,&
&  0.46848E+00_JPRB, 0.44362E+00_JPRB, 0.42856E+00_JPRB, 0.41808E+00_JPRB, 0.42504E+00_JPRB, 0.42980E+00_JPRB, 0.43261E+00_JPRB ,&
&  0.42708E+00_JPRB, 0.42909E+00_JPRB, 0.44003E+00_JPRB, 0.44267E+00_JPRB, 0.44834E+00_JPRB, 0.45676E+00_JPRB, 0.45994E+00_JPRB ,&
&  0.48604E+00_JPRB, 0.49361E+00_JPRB, 0.50231E+00_JPRB, 0.51448E+00_JPRB, 0.51903E+00_JPRB, 0.50321E+00_JPRB, 0.49738E+00_JPRB ,&
&  0.50083E+00_JPRB, 0.50733E+00_JPRB, 0.50830E+00_JPRB, 0.51866E+00_JPRB, 0.52436E+00_JPRB, 0.53633E+00_JPRB, 0.55845E+00_JPRB ,&
&  0.58630E+00_JPRB, 0.59556E+00_JPRB, 0.56624E+00_JPRB, 0.49189E+00_JPRB, 0.45856E+00_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.41359E+00_JPRB, 0.39188E+00_JPRB, 0.35772E+00_JPRB, 0.32520E+00_JPRB, 0.34368E+00_JPRB, 0.34064E+00_JPRB, 0.35168E+00_JPRB ,&
&  0.35441E+00_JPRB, 0.37496E+00_JPRB, 0.39381E+00_JPRB, 0.42212E+00_JPRB, 0.45404E+00_JPRB, 0.48397E+00_JPRB, 0.50688E+00_JPRB ,&
&  0.52632E+00_JPRB, 0.54703E+00_JPRB, 0.56329E+00_JPRB, 0.56797E+00_JPRB, 0.57695E+00_JPRB, 0.59187E+00_JPRB, 0.60730E+00_JPRB ,&
&  0.62739E+00_JPRB, 0.65687E+00_JPRB, 0.69960E+00_JPRB, 0.75596E+00_JPRB, 0.82178E+00_JPRB, 0.84664E+00_JPRB, 0.84732E+00_JPRB ,&
&  0.52626E+00_JPRB, 0.76297E+00_JPRB, 0.86414E+00_JPRB, 0.81370E+00_JPRB, 0.78444E+00_JPRB, 0.76162E+00_JPRB, 0.74291E+00_JPRB ,&
&  0.71440E+00_JPRB, 0.67447E+00_JPRB, 0.65876E+00_JPRB, 0.70929E+00_JPRB, 0.70627E+00_JPRB, 0.73690E+00_JPRB, 0.76083E+00_JPRB ,&
&  0.78315E+00_JPRB, 0.80237E+00_JPRB, 0.81995E+00_JPRB, 0.84249E+00_JPRB, 0.86289E+00_JPRB, 0.87915E+00_JPRB, 0.88723E+00_JPRB ,&
&  0.90061E+00_JPRB, 0.90163E+00_JPRB, 0.87624E+00_JPRB, 0.71358E+00_JPRB, 0.66062E+00_JPRB, 0.61816E+00_JPRB, 0.58206E+00_JPRB ,&
&  0.52250E+00_JPRB, 0.49869E+00_JPRB, 0.49744E+00_JPRB, 0.52007E+00_JPRB, 0.50840E+00_JPRB, 0.49144E+00_JPRB, 0.47185E+00_JPRB ,&
&  0.44919E+00_JPRB, 0.42446E+00_JPRB, 0.41230E+00_JPRB, 0.40939E+00_JPRB, 0.42216E+00_JPRB, 0.42311E+00_JPRB, 0.41154E+00_JPRB ,&
&  0.39641E+00_JPRB, 0.39960E+00_JPRB, 0.42400E+00_JPRB, 0.44269E+00_JPRB, 0.47000E+00_JPRB, 0.48669E+00_JPRB, 0.49601E+00_JPRB ,&
&  0.50804E+00_JPRB, 0.49985E+00_JPRB, 0.50925E+00_JPRB, 0.51836E+00_JPRB, 0.51630E+00_JPRB, 0.51072E+00_JPRB, 0.50020E+00_JPRB ,&
&  0.49639E+00_JPRB, 0.51107E+00_JPRB, 0.52462E+00_JPRB, 0.53743E+00_JPRB, 0.54082E+00_JPRB, 0.54067E+00_JPRB, 0.54587E+00_JPRB ,&
&  0.53824E+00_JPRB, 0.52049E+00_JPRB, 0.49848E+00_JPRB, 0.46696E+00_JPRB, 0.43812E+00_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.40619E+00_JPRB, 0.39461E+00_JPRB, 0.37902E+00_JPRB, 0.36943E+00_JPRB, 0.36060E+00_JPRB, 0.35093E+00_JPRB, 0.35904E+00_JPRB ,&
&  0.38230E+00_JPRB, 0.40206E+00_JPRB, 0.43539E+00_JPRB, 0.48471E+00_JPRB, 0.52518E+00_JPRB, 0.54197E+00_JPRB, 0.56175E+00_JPRB ,&
&  0.57758E+00_JPRB, 0.59729E+00_JPRB, 0.61272E+00_JPRB, 0.62432E+00_JPRB, 0.64087E+00_JPRB, 0.66194E+00_JPRB, 0.69050E+00_JPRB ,&
&  0.72731E+00_JPRB, 0.76442E+00_JPRB, 0.81433E+00_JPRB, 0.88115E+00_JPRB, 0.94367E+00_JPRB, 0.95785E+00_JPRB, 0.96791E+00_JPRB ,&
&  0.74087E+00_JPRB, 0.85755E+00_JPRB, 0.83707E+00_JPRB, 0.79835E+00_JPRB, 0.75689E+00_JPRB, 0.72211E+00_JPRB, 0.70288E+00_JPRB ,&
&  0.69143E+00_JPRB, 0.65520E+00_JPRB, 0.65621E+00_JPRB, 0.69941E+00_JPRB, 0.76609E+00_JPRB, 0.79504E+00_JPRB, 0.82538E+00_JPRB ,&
&  0.85454E+00_JPRB, 0.88182E+00_JPRB, 0.90575E+00_JPRB, 0.92917E+00_JPRB, 0.94596E+00_JPRB, 0.97258E+00_JPRB, 0.98119E+00_JPRB ,&
&  0.99907E+00_JPRB, 0.10009E+01_JPRB, 0.99544E+00_JPRB, 0.84502E+00_JPRB, 0.74860E+00_JPRB, 0.68023E+00_JPRB, 0.58835E+00_JPRB ,&
&  0.52423E+00_JPRB, 0.49510E+00_JPRB, 0.51225E+00_JPRB, 0.53782E+00_JPRB, 0.52155E+00_JPRB, 0.49198E+00_JPRB, 0.45983E+00_JPRB ,&
&  0.44971E+00_JPRB, 0.41611E+00_JPRB, 0.40539E+00_JPRB, 0.41195E+00_JPRB, 0.42950E+00_JPRB, 0.41334E+00_JPRB, 0.39224E+00_JPRB ,&
&  0.37705E+00_JPRB, 0.37037E+00_JPRB, 0.40012E+00_JPRB, 0.43419E+00_JPRB, 0.47858E+00_JPRB, 0.49752E+00_JPRB, 0.48980E+00_JPRB ,&
&  0.52600E+00_JPRB, 0.49346E+00_JPRB, 0.49592E+00_JPRB, 0.50574E+00_JPRB, 0.51068E+00_JPRB, 0.51643E+00_JPRB, 0.52511E+00_JPRB ,&
&  0.55888E+00_JPRB, 0.62196E+00_JPRB, 0.64665E+00_JPRB, 0.60194E+00_JPRB, 0.56762E+00_JPRB, 0.53047E+00_JPRB, 0.48728E+00_JPRB ,&
&  0.49078E+00_JPRB, 0.48012E+00_JPRB, 0.46896E+00_JPRB, 0.45290E+00_JPRB, 0.43033E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.45942E+00_JPRB, 0.44755E+00_JPRB, 0.44024E+00_JPRB, 0.43383E+00_JPRB, 0.41862E+00_JPRB, 0.41843E+00_JPRB, 0.41481E+00_JPRB ,&
&  0.42985E+00_JPRB, 0.44813E+00_JPRB, 0.48043E+00_JPRB, 0.51793E+00_JPRB, 0.56487E+00_JPRB, 0.59335E+00_JPRB, 0.62480E+00_JPRB ,&
&  0.63886E+00_JPRB, 0.65935E+00_JPRB, 0.68717E+00_JPRB, 0.70537E+00_JPRB, 0.72943E+00_JPRB, 0.73960E+00_JPRB, 0.76941E+00_JPRB ,&
&  0.79459E+00_JPRB, 0.82456E+00_JPRB, 0.87079E+00_JPRB, 0.91047E+00_JPRB, 0.94548E+00_JPRB, 0.93464E+00_JPRB, 0.73215E+00_JPRB ,&
&  0.87174E+00_JPRB, 0.86161E+00_JPRB, 0.83584E+00_JPRB, 0.78589E+00_JPRB, 0.73162E+00_JPRB, 0.68622E+00_JPRB, 0.67166E+00_JPRB ,&
&  0.66595E+00_JPRB, 0.66151E+00_JPRB, 0.67229E+00_JPRB, 0.69587E+00_JPRB, 0.79430E+00_JPRB, 0.84366E+00_JPRB, 0.87327E+00_JPRB ,&
&  0.90693E+00_JPRB, 0.93920E+00_JPRB, 0.96850E+00_JPRB, 0.99897E+00_JPRB, 0.10241E+01_JPRB, 0.10576E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.11192E+01_JPRB, 0.11291E+01_JPRB, 0.10926E+01_JPRB, 0.91126E+00_JPRB, 0.84689E+00_JPRB, 0.76894E+00_JPRB, 0.63839E+00_JPRB ,&
&  0.55730E+00_JPRB, 0.53320E+00_JPRB, 0.55366E+00_JPRB, 0.57814E+00_JPRB, 0.54788E+00_JPRB, 0.50764E+00_JPRB, 0.48343E+00_JPRB ,&
&  0.44967E+00_JPRB, 0.44557E+00_JPRB, 0.43515E+00_JPRB, 0.44961E+00_JPRB, 0.45542E+00_JPRB, 0.43753E+00_JPRB, 0.42610E+00_JPRB ,&
&  0.40751E+00_JPRB, 0.39228E+00_JPRB, 0.40847E+00_JPRB, 0.43933E+00_JPRB, 0.51003E+00_JPRB, 0.57370E+00_JPRB, 0.58380E+00_JPRB ,&
&  0.63094E+00_JPRB, 0.43253E+00_JPRB, 0.46121E+00_JPRB, 0.48752E+00_JPRB, 0.53213E+00_JPRB, 0.57576E+00_JPRB, 0.66432E+00_JPRB ,&
&  0.85536E+00_JPRB, 0.11966E+01_JPRB, 0.10676E+01_JPRB, 0.81141E+00_JPRB, 0.90201E+00_JPRB, 0.82702E+00_JPRB, 0.44047E+00_JPRB ,&
&  0.44246E+00_JPRB, 0.44404E+00_JPRB, 0.45918E+00_JPRB, 0.46348E+00_JPRB, 0.46641E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.53103E+00_JPRB, 0.52536E+00_JPRB, 0.52446E+00_JPRB, 0.51514E+00_JPRB, 0.50982E+00_JPRB, 0.51024E+00_JPRB, 0.49757E+00_JPRB ,&
&  0.50682E+00_JPRB, 0.52208E+00_JPRB, 0.52738E+00_JPRB, 0.54262E+00_JPRB, 0.55070E+00_JPRB, 0.58019E+00_JPRB, 0.62281E+00_JPRB ,&
&  0.64761E+00_JPRB, 0.70840E+00_JPRB, 0.73149E+00_JPRB, 0.76179E+00_JPRB, 0.78477E+00_JPRB, 0.80759E+00_JPRB, 0.82581E+00_JPRB ,&
&  0.85580E+00_JPRB, 0.88962E+00_JPRB, 0.92907E+00_JPRB, 0.95368E+00_JPRB, 0.94224E+00_JPRB, 0.99859E+00_JPRB, 0.78051E+00_JPRB ,&
&  0.89152E+00_JPRB, 0.87213E+00_JPRB, 0.79305E+00_JPRB, 0.74598E+00_JPRB, 0.70145E+00_JPRB, 0.67045E+00_JPRB, 0.64862E+00_JPRB ,&
&  0.64342E+00_JPRB, 0.64293E+00_JPRB, 0.65647E+00_JPRB, 0.70789E+00_JPRB, 0.77593E+00_JPRB, 0.83628E+00_JPRB, 0.87250E+00_JPRB ,&
&  0.90970E+00_JPRB, 0.93238E+00_JPRB, 0.97218E+00_JPRB, 0.10178E+01_JPRB, 0.10686E+01_JPRB, 0.11027E+01_JPRB, 0.11404E+01_JPRB ,&
&  0.11520E+01_JPRB, 0.11660E+01_JPRB, 0.10964E+01_JPRB, 0.10082E+01_JPRB, 0.96635E+00_JPRB, 0.88324E+00_JPRB, 0.74121E+00_JPRB ,&
&  0.60390E+00_JPRB, 0.56940E+00_JPRB, 0.58215E+00_JPRB, 0.62654E+00_JPRB, 0.58655E+00_JPRB, 0.55294E+00_JPRB, 0.53689E+00_JPRB ,&
&  0.51317E+00_JPRB, 0.47521E+00_JPRB, 0.48685E+00_JPRB, 0.51580E+00_JPRB, 0.52290E+00_JPRB, 0.51365E+00_JPRB, 0.49527E+00_JPRB ,&
&  0.47446E+00_JPRB, 0.44511E+00_JPRB, 0.43881E+00_JPRB, 0.43885E+00_JPRB, 0.44722E+00_JPRB, 0.46969E+00_JPRB, 0.41955E+00_JPRB ,&
&  0.47305E+00_JPRB, 0.43094E+00_JPRB, 0.45424E+00_JPRB, 0.50292E+00_JPRB, 0.59279E+00_JPRB, 0.76529E+00_JPRB, 0.10910E+01_JPRB ,&
&  0.16749E+01_JPRB, 0.19377E+01_JPRB, 0.21968E+01_JPRB, 0.27529E+01_JPRB, 0.24009E+01_JPRB, 0.17610E+01_JPRB, 0.49600E+00_JPRB ,&
&  0.49250E+00_JPRB, 0.49615E+00_JPRB, 0.51578E+00_JPRB, 0.52379E+00_JPRB, 0.52824E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.56744E+00_JPRB, 0.57526E+00_JPRB, 0.58617E+00_JPRB, 0.59769E+00_JPRB, 0.61179E+00_JPRB, 0.62587E+00_JPRB, 0.63674E+00_JPRB ,&
&  0.64797E+00_JPRB, 0.66210E+00_JPRB, 0.67560E+00_JPRB, 0.68457E+00_JPRB, 0.69335E+00_JPRB, 0.71117E+00_JPRB, 0.73213E+00_JPRB ,&
&  0.73811E+00_JPRB, 0.77165E+00_JPRB, 0.78172E+00_JPRB, 0.80986E+00_JPRB, 0.83341E+00_JPRB, 0.86376E+00_JPRB, 0.88333E+00_JPRB ,&
&  0.89850E+00_JPRB, 0.92463E+00_JPRB, 0.92469E+00_JPRB, 0.90838E+00_JPRB, 0.90346E+00_JPRB, 0.93089E+00_JPRB, 0.72565E+00_JPRB ,&
&  0.94310E+00_JPRB, 0.90777E+00_JPRB, 0.79090E+00_JPRB, 0.72950E+00_JPRB, 0.68158E+00_JPRB, 0.65363E+00_JPRB, 0.63602E+00_JPRB ,&
&  0.62522E+00_JPRB, 0.63568E+00_JPRB, 0.61673E+00_JPRB, 0.62428E+00_JPRB, 0.69533E+00_JPRB, 0.78240E+00_JPRB, 0.84309E+00_JPRB ,&
&  0.84918E+00_JPRB, 0.88337E+00_JPRB, 0.89135E+00_JPRB, 0.90508E+00_JPRB, 0.94234E+00_JPRB, 0.97494E+00_JPRB, 0.99884E+00_JPRB ,&
&  0.10003E+01_JPRB, 0.10015E+01_JPRB, 0.95592E+00_JPRB, 0.10130E+01_JPRB, 0.10379E+01_JPRB, 0.95977E+00_JPRB, 0.80438E+00_JPRB ,&
&  0.59769E+00_JPRB, 0.61008E+00_JPRB, 0.65072E+00_JPRB, 0.71403E+00_JPRB, 0.68714E+00_JPRB, 0.65850E+00_JPRB, 0.59713E+00_JPRB ,&
&  0.57106E+00_JPRB, 0.55834E+00_JPRB, 0.57650E+00_JPRB, 0.60141E+00_JPRB, 0.62281E+00_JPRB, 0.61482E+00_JPRB, 0.57882E+00_JPRB ,&
&  0.54407E+00_JPRB, 0.49967E+00_JPRB, 0.47236E+00_JPRB, 0.45791E+00_JPRB, 0.43414E+00_JPRB, 0.37735E+00_JPRB, 0.41751E+00_JPRB ,&
&  0.40941E+00_JPRB, 0.39118E+00_JPRB, 0.44790E+00_JPRB, 0.37566E+00_JPRB, 0.56175E+00_JPRB, 0.72715E+00_JPRB, 0.99098E+00_JPRB ,&
&  0.11997E+01_JPRB, 0.17088E+01_JPRB, 0.21656E+01_JPRB, 0.16389E+01_JPRB, 0.11174E+01_JPRB, 0.62030E+00_JPRB, 0.53013E+00_JPRB ,&
&  0.53651E+00_JPRB, 0.54116E+00_JPRB, 0.54900E+00_JPRB, 0.54834E+00_JPRB, 0.55856E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.58279E+00_JPRB, 0.59382E+00_JPRB, 0.60916E+00_JPRB, 0.62993E+00_JPRB, 0.64948E+00_JPRB, 0.66391E+00_JPRB, 0.67281E+00_JPRB ,&
&  0.68907E+00_JPRB, 0.71111E+00_JPRB, 0.72335E+00_JPRB, 0.72931E+00_JPRB, 0.74281E+00_JPRB, 0.75552E+00_JPRB, 0.77900E+00_JPRB ,&
&  0.79316E+00_JPRB, 0.82620E+00_JPRB, 0.85012E+00_JPRB, 0.87462E+00_JPRB, 0.89311E+00_JPRB, 0.91625E+00_JPRB, 0.93780E+00_JPRB ,&
&  0.95504E+00_JPRB, 0.99129E+00_JPRB, 0.10245E+01_JPRB, 0.10507E+01_JPRB, 0.10886E+01_JPRB, 0.11018E+01_JPRB, 0.87916E+00_JPRB ,&
&  0.10504E+01_JPRB, 0.99155E+00_JPRB, 0.85722E+00_JPRB, 0.75498E+00_JPRB, 0.69043E+00_JPRB, 0.63890E+00_JPRB, 0.60917E+00_JPRB ,&
&  0.60248E+00_JPRB, 0.60321E+00_JPRB, 0.60330E+00_JPRB, 0.58565E+00_JPRB, 0.56975E+00_JPRB, 0.60039E+00_JPRB, 0.68799E+00_JPRB ,&
&  0.73948E+00_JPRB, 0.76534E+00_JPRB, 0.78687E+00_JPRB, 0.77852E+00_JPRB, 0.80009E+00_JPRB, 0.83637E+00_JPRB, 0.86653E+00_JPRB ,&
&  0.86327E+00_JPRB, 0.83472E+00_JPRB, 0.78313E+00_JPRB, 0.93208E+00_JPRB, 0.97567E+00_JPRB, 0.98733E+00_JPRB, 0.89603E+00_JPRB ,&
&  0.74628E+00_JPRB, 0.67133E+00_JPRB, 0.69779E+00_JPRB, 0.78457E+00_JPRB, 0.77262E+00_JPRB, 0.75780E+00_JPRB, 0.71290E+00_JPRB ,&
&  0.68763E+00_JPRB, 0.66753E+00_JPRB, 0.67208E+00_JPRB, 0.69091E+00_JPRB, 0.70301E+00_JPRB, 0.69823E+00_JPRB, 0.64749E+00_JPRB ,&
&  0.60173E+00_JPRB, 0.55526E+00_JPRB, 0.52263E+00_JPRB, 0.49239E+00_JPRB, 0.46179E+00_JPRB, 0.40482E+00_JPRB, 0.42899E+00_JPRB ,&
&  0.40266E+00_JPRB, 0.33290E+00_JPRB, 0.36791E+00_JPRB, 0.44915E+00_JPRB, 0.49269E+00_JPRB, 0.57521E+00_JPRB, 0.73739E+00_JPRB ,&
&  0.94696E+00_JPRB, 0.10909E+01_JPRB, 0.10668E+01_JPRB, 0.80322E+00_JPRB, 0.62028E+00_JPRB, 0.56150E+00_JPRB, 0.55969E+00_JPRB ,&
&  0.56688E+00_JPRB, 0.56489E+00_JPRB, 0.56705E+00_JPRB, 0.57084E+00_JPRB, 0.57788E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.56634E+00_JPRB, 0.56453E+00_JPRB, 0.57027E+00_JPRB, 0.58270E+00_JPRB, 0.58753E+00_JPRB, 0.59012E+00_JPRB, 0.59079E+00_JPRB ,&
&  0.60517E+00_JPRB, 0.62192E+00_JPRB, 0.63738E+00_JPRB, 0.65301E+00_JPRB, 0.66554E+00_JPRB, 0.67695E+00_JPRB, 0.70785E+00_JPRB ,&
&  0.72221E+00_JPRB, 0.74327E+00_JPRB, 0.75751E+00_JPRB, 0.77182E+00_JPRB, 0.78363E+00_JPRB, 0.77648E+00_JPRB, 0.77851E+00_JPRB ,&
&  0.78564E+00_JPRB, 0.80845E+00_JPRB, 0.83338E+00_JPRB, 0.87465E+00_JPRB, 0.92962E+00_JPRB, 0.93880E+00_JPRB, 0.10415E+01_JPRB ,&
&  0.97058E+00_JPRB, 0.11047E+01_JPRB, 0.95215E+00_JPRB, 0.80514E+00_JPRB, 0.76722E+00_JPRB, 0.69073E+00_JPRB, 0.65044E+00_JPRB ,&
&  0.64847E+00_JPRB, 0.62294E+00_JPRB, 0.59519E+00_JPRB, 0.56254E+00_JPRB, 0.52708E+00_JPRB, 0.52870E+00_JPRB, 0.54192E+00_JPRB ,&
&  0.60392E+00_JPRB, 0.63995E+00_JPRB, 0.70357E+00_JPRB, 0.76816E+00_JPRB, 0.84614E+00_JPRB, 0.87427E+00_JPRB, 0.89238E+00_JPRB ,&
&  0.89355E+00_JPRB, 0.87665E+00_JPRB, 0.83029E+00_JPRB, 0.94161E+00_JPRB, 0.10299E+01_JPRB, 0.10816E+01_JPRB, 0.10770E+01_JPRB ,&
&  0.10009E+01_JPRB, 0.88324E+00_JPRB, 0.86873E+00_JPRB, 0.86289E+00_JPRB, 0.84533E+00_JPRB, 0.85278E+00_JPRB, 0.81821E+00_JPRB ,&
&  0.78885E+00_JPRB, 0.75601E+00_JPRB, 0.73884E+00_JPRB, 0.74261E+00_JPRB, 0.74012E+00_JPRB, 0.72400E+00_JPRB, 0.65568E+00_JPRB ,&
&  0.61302E+00_JPRB, 0.57738E+00_JPRB, 0.55571E+00_JPRB, 0.53091E+00_JPRB, 0.46326E+00_JPRB, 0.62471E+00_JPRB, 0.52269E+00_JPRB ,&
&  0.44188E+00_JPRB, 0.42683E+00_JPRB, 0.35767E+00_JPRB, 0.46078E+00_JPRB, 0.50263E+00_JPRB, 0.55362E+00_JPRB, 0.62054E+00_JPRB ,&
&  0.63507E+00_JPRB, 0.62299E+00_JPRB, 0.56109E+00_JPRB, 0.50089E+00_JPRB, 0.51372E+00_JPRB, 0.51476E+00_JPRB, 0.51754E+00_JPRB ,&
&  0.53134E+00_JPRB, 0.53982E+00_JPRB, 0.55021E+00_JPRB, 0.56886E+00_JPRB, 0.56316E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.55794E+00_JPRB, 0.54666E+00_JPRB, 0.53543E+00_JPRB, 0.52420E+00_JPRB, 0.52903E+00_JPRB, 0.52843E+00_JPRB, 0.53568E+00_JPRB ,&
&  0.54397E+00_JPRB, 0.54101E+00_JPRB, 0.54300E+00_JPRB, 0.54304E+00_JPRB, 0.55398E+00_JPRB, 0.56374E+00_JPRB, 0.59059E+00_JPRB ,&
&  0.61533E+00_JPRB, 0.62753E+00_JPRB, 0.64526E+00_JPRB, 0.66201E+00_JPRB, 0.67467E+00_JPRB, 0.68733E+00_JPRB, 0.67059E+00_JPRB ,&
&  0.65203E+00_JPRB, 0.65178E+00_JPRB, 0.68441E+00_JPRB, 0.79267E+00_JPRB, 0.10019E+01_JPRB, 0.12781E+01_JPRB, 0.18201E+01_JPRB ,&
&  0.16809E+01_JPRB, 0.10976E+01_JPRB, 0.98390E+00_JPRB, 0.86228E+00_JPRB, 0.89406E+00_JPRB, 0.95967E+00_JPRB, 0.95317E+00_JPRB ,&
&  0.94119E+00_JPRB, 0.92952E+00_JPRB, 0.91249E+00_JPRB, 0.88808E+00_JPRB, 0.83719E+00_JPRB, 0.77049E+00_JPRB, 0.71769E+00_JPRB ,&
&  0.69081E+00_JPRB, 0.65999E+00_JPRB, 0.73560E+00_JPRB, 0.93782E+00_JPRB, 0.10719E+01_JPRB, 0.13079E+01_JPRB, 0.12676E+01_JPRB ,&
&  0.12783E+01_JPRB, 0.12285E+01_JPRB, 0.11914E+01_JPRB, 0.12874E+01_JPRB, 0.13193E+01_JPRB, 0.13684E+01_JPRB, 0.13596E+01_JPRB ,&
&  0.13924E+01_JPRB, 0.11970E+01_JPRB, 0.85520E+00_JPRB, 0.93642E+00_JPRB, 0.92200E+00_JPRB, 0.95171E+00_JPRB, 0.91217E+00_JPRB ,&
&  0.87499E+00_JPRB, 0.82404E+00_JPRB, 0.79351E+00_JPRB, 0.77401E+00_JPRB, 0.74955E+00_JPRB, 0.70819E+00_JPRB, 0.63056E+00_JPRB ,&
&  0.59674E+00_JPRB, 0.57199E+00_JPRB, 0.55802E+00_JPRB, 0.53176E+00_JPRB, 0.52504E+00_JPRB, 0.50484E+00_JPRB, 0.50324E+00_JPRB ,&
&  0.46978E+00_JPRB, 0.50401E+00_JPRB, 0.42525E+00_JPRB, 0.49435E+00_JPRB, 0.52606E+00_JPRB, 0.51617E+00_JPRB, 0.52016E+00_JPRB ,&
&  0.49877E+00_JPRB, 0.48036E+00_JPRB, 0.46647E+00_JPRB, 0.46928E+00_JPRB, 0.48343E+00_JPRB, 0.50580E+00_JPRB, 0.51282E+00_JPRB ,&
&  0.52353E+00_JPRB, 0.53850E+00_JPRB, 0.55759E+00_JPRB, 0.57829E+00_JPRB, 0.57590E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.58371E+00_JPRB, 0.58414E+00_JPRB, 0.58477E+00_JPRB, 0.59109E+00_JPRB, 0.59098E+00_JPRB, 0.58571E+00_JPRB, 0.57258E+00_JPRB ,&
&  0.56397E+00_JPRB, 0.54832E+00_JPRB, 0.53769E+00_JPRB, 0.52991E+00_JPRB, 0.53952E+00_JPRB, 0.56639E+00_JPRB, 0.60969E+00_JPRB ,&
&  0.62209E+00_JPRB, 0.62018E+00_JPRB, 0.59858E+00_JPRB, 0.58763E+00_JPRB, 0.61047E+00_JPRB, 0.61563E+00_JPRB, 0.59305E+00_JPRB ,&
&  0.57230E+00_JPRB, 0.62342E+00_JPRB, 0.76252E+00_JPRB, 0.94948E+00_JPRB, 0.12763E+01_JPRB, 0.14052E+01_JPRB, 0.17368E+01_JPRB ,&
&  0.15084E+01_JPRB, 0.11552E+01_JPRB, 0.12499E+01_JPRB, 0.12047E+01_JPRB, 0.12786E+01_JPRB, 0.13187E+01_JPRB, 0.13315E+01_JPRB ,&
&  0.13650E+01_JPRB, 0.13877E+01_JPRB, 0.14088E+01_JPRB, 0.14382E+01_JPRB, 0.14498E+01_JPRB, 0.14305E+01_JPRB, 0.14088E+01_JPRB ,&
&  0.13887E+01_JPRB, 0.13464E+01_JPRB, 0.13517E+01_JPRB, 0.13948E+01_JPRB, 0.15126E+01_JPRB, 0.16394E+01_JPRB, 0.16420E+01_JPRB ,&
&  0.16337E+01_JPRB, 0.16749E+01_JPRB, 0.16010E+01_JPRB, 0.16966E+01_JPRB, 0.17218E+01_JPRB, 0.16617E+01_JPRB, 0.17689E+01_JPRB ,&
&  0.18167E+01_JPRB, 0.15135E+01_JPRB, 0.95584E+00_JPRB, 0.98371E+00_JPRB, 0.94718E+00_JPRB, 0.93373E+00_JPRB, 0.97152E+00_JPRB ,&
&  0.91837E+00_JPRB, 0.87846E+00_JPRB, 0.84825E+00_JPRB, 0.82673E+00_JPRB, 0.77662E+00_JPRB, 0.71005E+00_JPRB, 0.66445E+00_JPRB ,&
&  0.61087E+00_JPRB, 0.57441E+00_JPRB, 0.55027E+00_JPRB, 0.52076E+00_JPRB, 0.48215E+00_JPRB, 0.50481E+00_JPRB, 0.51061E+00_JPRB ,&
&  0.52268E+00_JPRB, 0.52037E+00_JPRB, 0.52832E+00_JPRB, 0.50653E+00_JPRB, 0.49246E+00_JPRB, 0.55155E+00_JPRB, 0.53858E+00_JPRB ,&
&  0.53513E+00_JPRB, 0.52934E+00_JPRB, 0.52700E+00_JPRB, 0.52186E+00_JPRB, 0.54248E+00_JPRB, 0.57321E+00_JPRB, 0.58021E+00_JPRB ,&
&  0.59389E+00_JPRB, 0.58302E+00_JPRB, 0.58918E+00_JPRB, 0.59786E+00_JPRB, 0.60258E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.68343E+00_JPRB, 0.68270E+00_JPRB, 0.67066E+00_JPRB, 0.67934E+00_JPRB, 0.69008E+00_JPRB, 0.68503E+00_JPRB, 0.69723E+00_JPRB ,&
&  0.71503E+00_JPRB, 0.73176E+00_JPRB, 0.74092E+00_JPRB, 0.74603E+00_JPRB, 0.78784E+00_JPRB, 0.84093E+00_JPRB, 0.90351E+00_JPRB ,&
&  0.93936E+00_JPRB, 0.97448E+00_JPRB, 0.98530E+00_JPRB, 0.97646E+00_JPRB, 0.97353E+00_JPRB, 0.94491E+00_JPRB, 0.90503E+00_JPRB ,&
&  0.96145E+00_JPRB, 0.12061E+01_JPRB, 0.15936E+01_JPRB, 0.19158E+01_JPRB, 0.21736E+01_JPRB, 0.17483E+01_JPRB, 0.17029E+01_JPRB ,&
&  0.14640E+01_JPRB, 0.12344E+01_JPRB, 0.12919E+01_JPRB, 0.14040E+01_JPRB, 0.14891E+01_JPRB, 0.14920E+01_JPRB, 0.15263E+01_JPRB ,&
&  0.15445E+01_JPRB, 0.15596E+01_JPRB, 0.15850E+01_JPRB, 0.16505E+01_JPRB, 0.17018E+01_JPRB, 0.17389E+01_JPRB, 0.17882E+01_JPRB ,&
&  0.18236E+01_JPRB, 0.18186E+01_JPRB, 0.18044E+01_JPRB, 0.16528E+01_JPRB, 0.16843E+01_JPRB, 0.17660E+01_JPRB, 0.18356E+01_JPRB ,&
&  0.18540E+01_JPRB, 0.18211E+01_JPRB, 0.19195E+01_JPRB, 0.19429E+01_JPRB, 0.19317E+01_JPRB, 0.18750E+01_JPRB, 0.18750E+01_JPRB ,&
&  0.18702E+01_JPRB, 0.16766E+01_JPRB, 0.10955E+01_JPRB, 0.11203E+01_JPRB, 0.10575E+01_JPRB, 0.92384E+00_JPRB, 0.98285E+00_JPRB ,&
&  0.94791E+00_JPRB, 0.88239E+00_JPRB, 0.84840E+00_JPRB, 0.83474E+00_JPRB, 0.80959E+00_JPRB, 0.78808E+00_JPRB, 0.74199E+00_JPRB ,&
&  0.69048E+00_JPRB, 0.62521E+00_JPRB, 0.54637E+00_JPRB, 0.51761E+00_JPRB, 0.50023E+00_JPRB, 0.55718E+00_JPRB, 0.66530E+00_JPRB ,&
&  0.65752E+00_JPRB, 0.61393E+00_JPRB, 0.63343E+00_JPRB, 0.61837E+00_JPRB, 0.57764E+00_JPRB, 0.61082E+00_JPRB, 0.61106E+00_JPRB ,&
&  0.61735E+00_JPRB, 0.62557E+00_JPRB, 0.63117E+00_JPRB, 0.63214E+00_JPRB, 0.64200E+00_JPRB, 0.65943E+00_JPRB, 0.67591E+00_JPRB ,&
&  0.69080E+00_JPRB, 0.68719E+00_JPRB, 0.68486E+00_JPRB, 0.68259E+00_JPRB, 0.67888E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.72850E+00_JPRB, 0.70473E+00_JPRB, 0.68488E+00_JPRB, 0.67482E+00_JPRB, 0.67270E+00_JPRB, 0.68174E+00_JPRB, 0.69914E+00_JPRB ,&
&  0.72942E+00_JPRB, 0.77329E+00_JPRB, 0.81801E+00_JPRB, 0.85555E+00_JPRB, 0.91360E+00_JPRB, 0.99084E+00_JPRB, 0.10901E+01_JPRB ,&
&  0.11983E+01_JPRB, 0.13152E+01_JPRB, 0.13795E+01_JPRB, 0.14119E+01_JPRB, 0.14768E+01_JPRB, 0.15819E+01_JPRB, 0.18098E+01_JPRB ,&
&  0.21158E+01_JPRB, 0.23562E+01_JPRB, 0.24572E+01_JPRB, 0.21749E+01_JPRB, 0.16078E+01_JPRB, 0.14730E+01_JPRB, 0.15623E+01_JPRB ,&
&  0.15883E+01_JPRB, 0.14961E+01_JPRB, 0.14403E+01_JPRB, 0.14363E+01_JPRB, 0.14447E+01_JPRB, 0.14872E+01_JPRB, 0.15124E+01_JPRB ,&
&  0.15247E+01_JPRB, 0.15445E+01_JPRB, 0.15973E+01_JPRB, 0.16537E+01_JPRB, 0.17446E+01_JPRB, 0.18396E+01_JPRB, 0.19486E+01_JPRB ,&
&  0.19813E+01_JPRB, 0.20276E+01_JPRB, 0.19946E+01_JPRB, 0.19100E+01_JPRB, 0.18725E+01_JPRB, 0.18947E+01_JPRB, 0.19442E+01_JPRB ,&
&  0.19869E+01_JPRB, 0.19085E+01_JPRB, 0.18998E+01_JPRB, 0.18995E+01_JPRB, 0.18059E+01_JPRB, 0.15969E+01_JPRB, 0.15148E+01_JPRB ,&
&  0.15281E+01_JPRB, 0.16589E+01_JPRB, 0.10823E+01_JPRB, 0.12158E+01_JPRB, 0.13108E+01_JPRB, 0.11948E+01_JPRB, 0.10563E+01_JPRB ,&
&  0.10316E+01_JPRB, 0.95643E+00_JPRB, 0.92909E+00_JPRB, 0.94892E+00_JPRB, 0.92012E+00_JPRB, 0.91076E+00_JPRB, 0.79422E+00_JPRB ,&
&  0.80024E+00_JPRB, 0.70443E+00_JPRB, 0.58475E+00_JPRB, 0.49935E+00_JPRB, 0.52275E+00_JPRB, 0.59691E+00_JPRB, 0.68375E+00_JPRB ,&
&  0.66955E+00_JPRB, 0.77590E+00_JPRB, 0.90510E+00_JPRB, 0.11060E+01_JPRB, 0.12641E+01_JPRB, 0.62365E+00_JPRB, 0.63474E+00_JPRB ,&
&  0.64777E+00_JPRB, 0.67683E+00_JPRB, 0.68563E+00_JPRB, 0.69105E+00_JPRB, 0.69807E+00_JPRB, 0.71870E+00_JPRB, 0.75131E+00_JPRB ,&
&  0.76900E+00_JPRB, 0.77717E+00_JPRB, 0.78500E+00_JPRB, 0.77395E+00_JPRB, 0.74849E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.76324E+00_JPRB, 0.76005E+00_JPRB, 0.75473E+00_JPRB, 0.73976E+00_JPRB, 0.71378E+00_JPRB, 0.69558E+00_JPRB, 0.69722E+00_JPRB ,&
&  0.70082E+00_JPRB, 0.73409E+00_JPRB, 0.79260E+00_JPRB, 0.86548E+00_JPRB, 0.96172E+00_JPRB, 0.10704E+01_JPRB, 0.11847E+01_JPRB ,&
&  0.13350E+01_JPRB, 0.14889E+01_JPRB, 0.15933E+01_JPRB, 0.16811E+01_JPRB, 0.17831E+01_JPRB, 0.19545E+01_JPRB, 0.21544E+01_JPRB ,&
&  0.23027E+01_JPRB, 0.22981E+01_JPRB, 0.21506E+01_JPRB, 0.15389E+01_JPRB, 0.12276E+01_JPRB, 0.13999E+01_JPRB, 0.14242E+01_JPRB ,&
&  0.14267E+01_JPRB, 0.14117E+01_JPRB, 0.14073E+01_JPRB, 0.13875E+01_JPRB, 0.13964E+01_JPRB, 0.14359E+01_JPRB, 0.14641E+01_JPRB ,&
&  0.14687E+01_JPRB, 0.14797E+01_JPRB, 0.15224E+01_JPRB, 0.16129E+01_JPRB, 0.17438E+01_JPRB, 0.18045E+01_JPRB, 0.18781E+01_JPRB ,&
&  0.19416E+01_JPRB, 0.20482E+01_JPRB, 0.20902E+01_JPRB, 0.20815E+01_JPRB, 0.20065E+01_JPRB, 0.19800E+01_JPRB, 0.19865E+01_JPRB ,&
&  0.19572E+01_JPRB, 0.18775E+01_JPRB, 0.18500E+01_JPRB, 0.18402E+01_JPRB, 0.17868E+01_JPRB, 0.15073E+01_JPRB, 0.13800E+01_JPRB ,&
&  0.13409E+01_JPRB, 0.13099E+01_JPRB, 0.11191E+01_JPRB, 0.13404E+01_JPRB, 0.93377E+00_JPRB, 0.10740E+01_JPRB, 0.11330E+01_JPRB ,&
&  0.11080E+01_JPRB, 0.10968E+01_JPRB, 0.11111E+01_JPRB, 0.11064E+01_JPRB, 0.10785E+01_JPRB, 0.89244E+00_JPRB, 0.85192E+00_JPRB ,&
&  0.84802E+00_JPRB, 0.71918E+00_JPRB, 0.61641E+00_JPRB, 0.48360E+00_JPRB, 0.53117E+00_JPRB, 0.60473E+00_JPRB, 0.64772E+00_JPRB ,&
&  0.61714E+00_JPRB, 0.76149E+00_JPRB, 0.92170E+00_JPRB, 0.10021E+01_JPRB, 0.10381E+01_JPRB, 0.62582E+00_JPRB, 0.62524E+00_JPRB ,&
&  0.64833E+00_JPRB, 0.66612E+00_JPRB, 0.68329E+00_JPRB, 0.68256E+00_JPRB, 0.69978E+00_JPRB, 0.73191E+00_JPRB, 0.76182E+00_JPRB ,&
&  0.77458E+00_JPRB, 0.77114E+00_JPRB, 0.77659E+00_JPRB, 0.77174E+00_JPRB, 0.76674E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.76264E+00_JPRB, 0.77139E+00_JPRB, 0.79869E+00_JPRB, 0.81493E+00_JPRB, 0.82349E+00_JPRB, 0.77987E+00_JPRB, 0.74566E+00_JPRB ,&
&  0.70010E+00_JPRB, 0.65155E+00_JPRB, 0.70757E+00_JPRB, 0.77605E+00_JPRB, 0.85715E+00_JPRB, 0.94394E+00_JPRB, 0.10346E+01_JPRB ,&
&  0.11550E+01_JPRB, 0.13055E+01_JPRB, 0.14899E+01_JPRB, 0.16272E+01_JPRB, 0.17458E+01_JPRB, 0.19006E+01_JPRB, 0.20397E+01_JPRB ,&
&  0.16893E+01_JPRB, 0.12146E+01_JPRB, 0.15554E+01_JPRB, 0.16773E+01_JPRB, 0.13861E+01_JPRB, 0.12974E+01_JPRB, 0.12910E+01_JPRB ,&
&  0.13206E+01_JPRB, 0.13519E+01_JPRB, 0.13714E+01_JPRB, 0.13820E+01_JPRB, 0.13467E+01_JPRB, 0.13764E+01_JPRB, 0.14174E+01_JPRB ,&
&  0.14420E+01_JPRB, 0.14456E+01_JPRB, 0.14545E+01_JPRB, 0.15247E+01_JPRB, 0.16477E+01_JPRB, 0.16724E+01_JPRB, 0.17725E+01_JPRB ,&
&  0.18780E+01_JPRB, 0.20217E+01_JPRB, 0.20448E+01_JPRB, 0.19597E+01_JPRB, 0.19532E+01_JPRB, 0.19780E+01_JPRB, 0.19762E+01_JPRB ,&
&  0.19606E+01_JPRB, 0.19201E+01_JPRB, 0.19167E+01_JPRB, 0.19257E+01_JPRB, 0.18697E+01_JPRB, 0.16068E+01_JPRB, 0.15101E+01_JPRB ,&
&  0.13953E+01_JPRB, 0.12597E+01_JPRB, 0.11588E+01_JPRB, 0.12845E+01_JPRB, 0.10160E+01_JPRB, 0.10872E+01_JPRB, 0.10438E+01_JPRB ,&
&  0.11621E+01_JPRB, 0.11851E+01_JPRB, 0.11589E+01_JPRB, 0.11093E+01_JPRB, 0.10642E+01_JPRB, 0.90764E+00_JPRB, 0.84929E+00_JPRB ,&
&  0.76425E+00_JPRB, 0.74224E+00_JPRB, 0.61742E+00_JPRB, 0.51034E+00_JPRB, 0.45655E+00_JPRB, 0.49916E+00_JPRB, 0.57376E+00_JPRB ,&
&  0.66372E+00_JPRB, 0.67683E+00_JPRB, 0.81921E+00_JPRB, 0.87029E+00_JPRB, 0.87644E+00_JPRB, 0.68065E+00_JPRB, 0.65521E+00_JPRB ,&
&  0.66238E+00_JPRB, 0.66673E+00_JPRB, 0.64226E+00_JPRB, 0.65014E+00_JPRB, 0.67514E+00_JPRB, 0.71445E+00_JPRB, 0.73236E+00_JPRB ,&
&  0.73128E+00_JPRB, 0.71543E+00_JPRB, 0.71377E+00_JPRB, 0.72367E+00_JPRB, 0.75069E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.69084E+00_JPRB, 0.71517E+00_JPRB, 0.74431E+00_JPRB, 0.79400E+00_JPRB, 0.86642E+00_JPRB, 0.95786E+00_JPRB, 0.97656E+00_JPRB ,&
&  0.83650E+00_JPRB, 0.60240E+00_JPRB, 0.62027E+00_JPRB, 0.64805E+00_JPRB, 0.69893E+00_JPRB, 0.74748E+00_JPRB, 0.80254E+00_JPRB ,&
&  0.88676E+00_JPRB, 0.99935E+00_JPRB, 0.11930E+01_JPRB, 0.14084E+01_JPRB, 0.15899E+01_JPRB, 0.18073E+01_JPRB, 0.16946E+01_JPRB ,&
&  0.12111E+01_JPRB, 0.12555E+01_JPRB, 0.15375E+01_JPRB, 0.14953E+01_JPRB, 0.13274E+01_JPRB, 0.13323E+01_JPRB, 0.12932E+01_JPRB ,&
&  0.12628E+01_JPRB, 0.12825E+01_JPRB, 0.13049E+01_JPRB, 0.12777E+01_JPRB, 0.12490E+01_JPRB, 0.12770E+01_JPRB, 0.13291E+01_JPRB ,&
&  0.13814E+01_JPRB, 0.14225E+01_JPRB, 0.14483E+01_JPRB, 0.14759E+01_JPRB, 0.15314E+01_JPRB, 0.15539E+01_JPRB, 0.16506E+01_JPRB ,&
&  0.17893E+01_JPRB, 0.19090E+01_JPRB, 0.19303E+01_JPRB, 0.18645E+01_JPRB, 0.18916E+01_JPRB, 0.19824E+01_JPRB, 0.20433E+01_JPRB ,&
&  0.20783E+01_JPRB, 0.21477E+01_JPRB, 0.21693E+01_JPRB, 0.20847E+01_JPRB, 0.18623E+01_JPRB, 0.19118E+01_JPRB, 0.16401E+01_JPRB ,&
&  0.15323E+01_JPRB, 0.13327E+01_JPRB, 0.13356E+01_JPRB, 0.10633E+01_JPRB, 0.11220E+01_JPRB, 0.11484E+01_JPRB, 0.11436E+01_JPRB ,&
&  0.11628E+01_JPRB, 0.12475E+01_JPRB, 0.12307E+01_JPRB, 0.11288E+01_JPRB, 0.11044E+01_JPRB, 0.93267E+00_JPRB, 0.87846E+00_JPRB ,&
&  0.71829E+00_JPRB, 0.76961E+00_JPRB, 0.63348E+00_JPRB, 0.48877E+00_JPRB, 0.38758E+00_JPRB, 0.38705E+00_JPRB, 0.45732E+00_JPRB ,&
&  0.61600E+00_JPRB, 0.68968E+00_JPRB, 0.72927E+00_JPRB, 0.75528E+00_JPRB, 0.76730E+00_JPRB, 0.68518E+00_JPRB, 0.68488E+00_JPRB ,&
&  0.69840E+00_JPRB, 0.67782E+00_JPRB, 0.63122E+00_JPRB, 0.63193E+00_JPRB, 0.65415E+00_JPRB, 0.69043E+00_JPRB, 0.68661E+00_JPRB ,&
&  0.69026E+00_JPRB, 0.68143E+00_JPRB, 0.66658E+00_JPRB, 0.65677E+00_JPRB, 0.66722E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.61712E+00_JPRB, 0.61807E+00_JPRB, 0.60925E+00_JPRB, 0.60502E+00_JPRB, 0.60724E+00_JPRB, 0.61754E+00_JPRB, 0.64166E+00_JPRB ,&
&  0.58808E+00_JPRB, 0.57864E+00_JPRB, 0.58360E+00_JPRB, 0.58069E+00_JPRB, 0.59112E+00_JPRB, 0.60319E+00_JPRB, 0.62763E+00_JPRB ,&
&  0.65798E+00_JPRB, 0.68638E+00_JPRB, 0.80168E+00_JPRB, 0.10302E+01_JPRB, 0.12571E+01_JPRB, 0.15132E+01_JPRB, 0.13904E+01_JPRB ,&
&  0.11063E+01_JPRB, 0.17756E+01_JPRB, 0.13787E+01_JPRB, 0.14312E+01_JPRB, 0.13968E+01_JPRB, 0.14124E+01_JPRB, 0.12828E+01_JPRB ,&
&  0.12238E+01_JPRB, 0.12180E+01_JPRB, 0.12494E+01_JPRB, 0.12292E+01_JPRB, 0.12314E+01_JPRB, 0.12429E+01_JPRB, 0.12697E+01_JPRB ,&
&  0.13185E+01_JPRB, 0.13653E+01_JPRB, 0.13966E+01_JPRB, 0.14272E+01_JPRB, 0.14692E+01_JPRB, 0.14898E+01_JPRB, 0.15313E+01_JPRB ,&
&  0.16375E+01_JPRB, 0.17789E+01_JPRB, 0.18202E+01_JPRB, 0.18626E+01_JPRB, 0.19177E+01_JPRB, 0.20746E+01_JPRB, 0.21865E+01_JPRB ,&
&  0.21505E+01_JPRB, 0.21357E+01_JPRB, 0.23429E+01_JPRB, 0.23596E+01_JPRB, 0.22703E+01_JPRB, 0.22451E+01_JPRB, 0.18291E+01_JPRB ,&
&  0.17249E+01_JPRB, 0.14550E+01_JPRB, 0.14427E+01_JPRB, 0.11557E+01_JPRB, 0.11805E+01_JPRB, 0.12287E+01_JPRB, 0.11545E+01_JPRB ,&
&  0.11234E+01_JPRB, 0.12387E+01_JPRB, 0.12233E+01_JPRB, 0.11822E+01_JPRB, 0.11000E+01_JPRB, 0.10088E+01_JPRB, 0.89729E+00_JPRB ,&
&  0.79008E+00_JPRB, 0.75875E+00_JPRB, 0.73832E+00_JPRB, 0.51812E+00_JPRB, 0.42862E+00_JPRB, 0.40084E+00_JPRB, 0.42901E+00_JPRB ,&
&  0.55568E+00_JPRB, 0.64174E+00_JPRB, 0.78899E+00_JPRB, 0.76490E+00_JPRB, 0.73170E+00_JPRB, 0.70027E+00_JPRB, 0.70854E+00_JPRB ,&
&  0.69530E+00_JPRB, 0.69349E+00_JPRB, 0.65657E+00_JPRB, 0.64337E+00_JPRB, 0.66395E+00_JPRB, 0.68767E+00_JPRB, 0.67780E+00_JPRB ,&
&  0.68301E+00_JPRB, 0.68148E+00_JPRB, 0.66566E+00_JPRB, 0.64076E+00_JPRB, 0.62517E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.62101E+00_JPRB, 0.60591E+00_JPRB, 0.60360E+00_JPRB, 0.59509E+00_JPRB, 0.58857E+00_JPRB, 0.58026E+00_JPRB, 0.58853E+00_JPRB ,&
&  0.57141E+00_JPRB, 0.55963E+00_JPRB, 0.56018E+00_JPRB, 0.54973E+00_JPRB, 0.55231E+00_JPRB, 0.53918E+00_JPRB, 0.54777E+00_JPRB ,&
&  0.55574E+00_JPRB, 0.53075E+00_JPRB, 0.56544E+00_JPRB, 0.70270E+00_JPRB, 0.89970E+00_JPRB, 0.12203E+01_JPRB, 0.93753E+00_JPRB ,&
&  0.11699E+01_JPRB, 0.17204E+01_JPRB, 0.14784E+01_JPRB, 0.14954E+01_JPRB, 0.15538E+01_JPRB, 0.15624E+01_JPRB, 0.13006E+01_JPRB ,&
&  0.11817E+01_JPRB, 0.11474E+01_JPRB, 0.11682E+01_JPRB, 0.12124E+01_JPRB, 0.12705E+01_JPRB, 0.13315E+01_JPRB, 0.13318E+01_JPRB ,&
&  0.13500E+01_JPRB, 0.13830E+01_JPRB, 0.14016E+01_JPRB, 0.14068E+01_JPRB, 0.14266E+01_JPRB, 0.14530E+01_JPRB, 0.14700E+01_JPRB ,&
&  0.15133E+01_JPRB, 0.16089E+01_JPRB, 0.17792E+01_JPRB, 0.17808E+01_JPRB, 0.19066E+01_JPRB, 0.21376E+01_JPRB, 0.22786E+01_JPRB ,&
&  0.21578E+01_JPRB, 0.21134E+01_JPRB, 0.24733E+01_JPRB, 0.28057E+01_JPRB, 0.28472E+01_JPRB, 0.26478E+01_JPRB, 0.21725E+01_JPRB ,&
&  0.19080E+01_JPRB, 0.16456E+01_JPRB, 0.14920E+01_JPRB, 0.12605E+01_JPRB, 0.12427E+01_JPRB, 0.12874E+01_JPRB, 0.12687E+01_JPRB ,&
&  0.13340E+01_JPRB, 0.13089E+01_JPRB, 0.12897E+01_JPRB, 0.11949E+01_JPRB, 0.10924E+01_JPRB, 0.10165E+01_JPRB, 0.10097E+01_JPRB ,&
&  0.98989E+00_JPRB, 0.10008E+01_JPRB, 0.95404E+00_JPRB, 0.64401E+00_JPRB, 0.50476E+00_JPRB, 0.39604E+00_JPRB, 0.42423E+00_JPRB ,&
&  0.54641E+00_JPRB, 0.59697E+00_JPRB, 0.62518E+00_JPRB, 0.86921E+00_JPRB, 0.80916E+00_JPRB, 0.77391E+00_JPRB, 0.77284E+00_JPRB ,&
&  0.74335E+00_JPRB, 0.76192E+00_JPRB, 0.74454E+00_JPRB, 0.71022E+00_JPRB, 0.71094E+00_JPRB, 0.71494E+00_JPRB, 0.71603E+00_JPRB ,&
&  0.70240E+00_JPRB, 0.69710E+00_JPRB, 0.69498E+00_JPRB, 0.67844E+00_JPRB, 0.64829E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.62911E+00_JPRB, 0.61748E+00_JPRB, 0.59640E+00_JPRB, 0.58526E+00_JPRB, 0.58619E+00_JPRB, 0.57004E+00_JPRB, 0.55679E+00_JPRB ,&
&  0.55790E+00_JPRB, 0.54421E+00_JPRB, 0.53080E+00_JPRB, 0.52649E+00_JPRB, 0.52558E+00_JPRB, 0.51873E+00_JPRB, 0.51781E+00_JPRB ,&
&  0.51144E+00_JPRB, 0.48360E+00_JPRB, 0.51933E+00_JPRB, 0.57889E+00_JPRB, 0.70163E+00_JPRB, 0.77570E+00_JPRB, 0.86784E+00_JPRB ,&
&  0.14275E+01_JPRB, 0.18509E+01_JPRB, 0.16299E+01_JPRB, 0.17559E+01_JPRB, 0.19714E+01_JPRB, 0.22230E+01_JPRB, 0.16333E+01_JPRB ,&
&  0.13884E+01_JPRB, 0.12475E+01_JPRB, 0.11515E+01_JPRB, 0.12075E+01_JPRB, 0.13205E+01_JPRB, 0.14204E+01_JPRB, 0.14479E+01_JPRB ,&
&  0.14012E+01_JPRB, 0.13766E+01_JPRB, 0.13877E+01_JPRB, 0.13922E+01_JPRB, 0.13936E+01_JPRB, 0.14346E+01_JPRB, 0.14538E+01_JPRB ,&
&  0.14689E+01_JPRB, 0.15232E+01_JPRB, 0.16629E+01_JPRB, 0.17127E+01_JPRB, 0.18697E+01_JPRB, 0.21092E+01_JPRB, 0.23985E+01_JPRB ,&
&  0.24244E+01_JPRB, 0.25168E+01_JPRB, 0.27629E+01_JPRB, 0.30759E+01_JPRB, 0.32763E+01_JPRB, 0.31934E+01_JPRB, 0.26448E+01_JPRB ,&
&  0.22279E+01_JPRB, 0.17976E+01_JPRB, 0.14708E+01_JPRB, 0.13684E+01_JPRB, 0.13810E+01_JPRB, 0.14278E+01_JPRB, 0.12704E+01_JPRB ,&
&  0.12158E+01_JPRB, 0.10880E+01_JPRB, 0.10569E+01_JPRB, 0.11336E+01_JPRB, 0.11272E+01_JPRB, 0.10787E+01_JPRB, 0.12007E+01_JPRB ,&
&  0.10671E+01_JPRB, 0.67867E+00_JPRB, 0.59380E+00_JPRB, 0.67198E+00_JPRB, 0.48448E+00_JPRB, 0.37079E+00_JPRB, 0.47033E+00_JPRB ,&
&  0.55787E+00_JPRB, 0.63879E+00_JPRB, 0.64908E+00_JPRB, 0.68625E+00_JPRB, 0.84063E+00_JPRB, 0.81687E+00_JPRB, 0.86836E+00_JPRB ,&
&  0.79012E+00_JPRB, 0.80078E+00_JPRB, 0.80075E+00_JPRB, 0.74986E+00_JPRB, 0.72614E+00_JPRB, 0.74597E+00_JPRB, 0.75432E+00_JPRB ,&
&  0.74399E+00_JPRB, 0.73255E+00_JPRB, 0.70725E+00_JPRB, 0.68111E+00_JPRB, 0.65032E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.66447E+00_JPRB, 0.64800E+00_JPRB, 0.63750E+00_JPRB, 0.61309E+00_JPRB, 0.59793E+00_JPRB, 0.57859E+00_JPRB, 0.55373E+00_JPRB ,&
&  0.53912E+00_JPRB, 0.52994E+00_JPRB, 0.52301E+00_JPRB, 0.51435E+00_JPRB, 0.51031E+00_JPRB, 0.50796E+00_JPRB, 0.51456E+00_JPRB ,&
&  0.49831E+00_JPRB, 0.48006E+00_JPRB, 0.56708E+00_JPRB, 0.62651E+00_JPRB, 0.66469E+00_JPRB, 0.67989E+00_JPRB, 0.90557E+00_JPRB ,&
&  0.14680E+01_JPRB, 0.20248E+01_JPRB, 0.24074E+01_JPRB, 0.25413E+01_JPRB, 0.27012E+01_JPRB, 0.29240E+01_JPRB, 0.24897E+01_JPRB ,&
&  0.19939E+01_JPRB, 0.16087E+01_JPRB, 0.13401E+01_JPRB, 0.13233E+01_JPRB, 0.14315E+01_JPRB, 0.15268E+01_JPRB, 0.15177E+01_JPRB ,&
&  0.14777E+01_JPRB, 0.14021E+01_JPRB, 0.13609E+01_JPRB, 0.13354E+01_JPRB, 0.13531E+01_JPRB, 0.14041E+01_JPRB, 0.14375E+01_JPRB ,&
&  0.14507E+01_JPRB, 0.14508E+01_JPRB, 0.15406E+01_JPRB, 0.17297E+01_JPRB, 0.16255E+01_JPRB, 0.20603E+01_JPRB, 0.24168E+01_JPRB ,&
&  0.26595E+01_JPRB, 0.27649E+01_JPRB, 0.28107E+01_JPRB, 0.34549E+01_JPRB, 0.38324E+01_JPRB, 0.36364E+01_JPRB, 0.30589E+01_JPRB ,&
&  0.26848E+01_JPRB, 0.21373E+01_JPRB, 0.16994E+01_JPRB, 0.15409E+01_JPRB, 0.15050E+01_JPRB, 0.15163E+01_JPRB, 0.10745E+01_JPRB ,&
&  0.10715E+01_JPRB, 0.12708E+01_JPRB, 0.11615E+01_JPRB, 0.10500E+01_JPRB, 0.11583E+01_JPRB, 0.11568E+01_JPRB, 0.90950E+00_JPRB ,&
&  0.51151E+00_JPRB, 0.43297E+00_JPRB, 0.43197E+00_JPRB, 0.42055E+00_JPRB, 0.37780E+00_JPRB, 0.37203E+00_JPRB, 0.63142E+00_JPRB ,&
&  0.58948E+00_JPRB, 0.69714E+00_JPRB, 0.68907E+00_JPRB, 0.71269E+00_JPRB, 0.81626E+00_JPRB, 0.83159E+00_JPRB, 0.10677E+01_JPRB ,&
&  0.91427E+00_JPRB, 0.86920E+00_JPRB, 0.84450E+00_JPRB, 0.80575E+00_JPRB, 0.78855E+00_JPRB, 0.77272E+00_JPRB, 0.77882E+00_JPRB ,&
&  0.78756E+00_JPRB, 0.79166E+00_JPRB, 0.77035E+00_JPRB, 0.73172E+00_JPRB, 0.69589E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.71489E+00_JPRB, 0.67851E+00_JPRB, 0.65548E+00_JPRB, 0.64652E+00_JPRB, 0.63348E+00_JPRB, 0.60158E+00_JPRB, 0.55911E+00_JPRB ,&
&  0.53382E+00_JPRB, 0.51987E+00_JPRB, 0.51176E+00_JPRB, 0.50771E+00_JPRB, 0.51171E+00_JPRB, 0.51419E+00_JPRB, 0.51306E+00_JPRB ,&
&  0.48797E+00_JPRB, 0.49566E+00_JPRB, 0.62623E+00_JPRB, 0.67339E+00_JPRB, 0.70335E+00_JPRB, 0.70739E+00_JPRB, 0.91136E+00_JPRB ,&
&  0.14483E+01_JPRB, 0.20455E+01_JPRB, 0.24693E+01_JPRB, 0.25624E+01_JPRB, 0.33935E+01_JPRB, 0.38706E+01_JPRB, 0.38341E+01_JPRB ,&
&  0.31092E+01_JPRB, 0.24509E+01_JPRB, 0.19090E+01_JPRB, 0.16370E+01_JPRB, 0.16359E+01_JPRB, 0.16668E+01_JPRB, 0.16478E+01_JPRB ,&
&  0.15620E+01_JPRB, 0.14802E+01_JPRB, 0.14080E+01_JPRB, 0.13660E+01_JPRB, 0.13661E+01_JPRB, 0.14018E+01_JPRB, 0.14553E+01_JPRB ,&
&  0.14715E+01_JPRB, 0.14315E+01_JPRB, 0.14288E+01_JPRB, 0.16238E+01_JPRB, 0.17440E+01_JPRB, 0.18585E+01_JPRB, 0.22769E+01_JPRB ,&
&  0.27055E+01_JPRB, 0.30252E+01_JPRB, 0.30605E+01_JPRB, 0.40529E+01_JPRB, 0.42877E+01_JPRB, 0.39698E+01_JPRB, 0.35888E+01_JPRB ,&
&  0.31626E+01_JPRB, 0.28139E+01_JPRB, 0.19764E+01_JPRB, 0.17028E+01_JPRB, 0.16476E+01_JPRB, 0.13461E+01_JPRB, 0.11955E+01_JPRB ,&
&  0.13747E+01_JPRB, 0.12790E+01_JPRB, 0.11258E+01_JPRB, 0.96783E+00_JPRB, 0.11609E+01_JPRB, 0.10269E+01_JPRB, 0.56349E+00_JPRB ,&
&  0.47318E+00_JPRB, 0.44797E+00_JPRB, 0.40518E+00_JPRB, 0.38808E+00_JPRB, 0.36695E+00_JPRB, 0.37966E+00_JPRB, 0.53959E+00_JPRB ,&
&  0.58947E+00_JPRB, 0.68292E+00_JPRB, 0.69980E+00_JPRB, 0.73643E+00_JPRB, 0.81620E+00_JPRB, 0.92162E+00_JPRB, 0.14687E+01_JPRB ,&
&  0.11818E+01_JPRB, 0.10604E+01_JPRB, 0.94345E+00_JPRB, 0.88444E+00_JPRB, 0.83883E+00_JPRB, 0.80513E+00_JPRB, 0.79843E+00_JPRB ,&
&  0.79658E+00_JPRB, 0.79820E+00_JPRB, 0.78355E+00_JPRB, 0.76200E+00_JPRB, 0.73971E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.74190E+00_JPRB, 0.71182E+00_JPRB, 0.67701E+00_JPRB, 0.65012E+00_JPRB, 0.63513E+00_JPRB, 0.61782E+00_JPRB, 0.58858E+00_JPRB ,&
&  0.55468E+00_JPRB, 0.53670E+00_JPRB, 0.52692E+00_JPRB, 0.51404E+00_JPRB, 0.51346E+00_JPRB, 0.51946E+00_JPRB, 0.51690E+00_JPRB ,&
&  0.49645E+00_JPRB, 0.52702E+00_JPRB, 0.60984E+00_JPRB, 0.60973E+00_JPRB, 0.62055E+00_JPRB, 0.63582E+00_JPRB, 0.79768E+00_JPRB ,&
&  0.13405E+01_JPRB, 0.19573E+01_JPRB, 0.23622E+01_JPRB, 0.27787E+01_JPRB, 0.37238E+01_JPRB, 0.38654E+01_JPRB, 0.44011E+01_JPRB ,&
&  0.39167E+01_JPRB, 0.32786E+01_JPRB, 0.27182E+01_JPRB, 0.23485E+01_JPRB, 0.21183E+01_JPRB, 0.19390E+01_JPRB, 0.18406E+01_JPRB ,&
&  0.17331E+01_JPRB, 0.15984E+01_JPRB, 0.15412E+01_JPRB, 0.14859E+01_JPRB, 0.14636E+01_JPRB, 0.14572E+01_JPRB, 0.15237E+01_JPRB ,&
&  0.15309E+01_JPRB, 0.14787E+01_JPRB, 0.13886E+01_JPRB, 0.14778E+01_JPRB, 0.18535E+01_JPRB, 0.21395E+01_JPRB, 0.22525E+01_JPRB ,&
&  0.23896E+01_JPRB, 0.25285E+01_JPRB, 0.31660E+01_JPRB, 0.39860E+01_JPRB, 0.44945E+01_JPRB, 0.45060E+01_JPRB, 0.38415E+01_JPRB ,&
&  0.33836E+01_JPRB, 0.29326E+01_JPRB, 0.21244E+01_JPRB, 0.18258E+01_JPRB, 0.15676E+01_JPRB, 0.12963E+01_JPRB, 0.14155E+01_JPRB ,&
&  0.15235E+01_JPRB, 0.14124E+01_JPRB, 0.11753E+01_JPRB, 0.93177E+00_JPRB, 0.91574E+00_JPRB, 0.70549E+00_JPRB, 0.52640E+00_JPRB ,&
&  0.49752E+00_JPRB, 0.46336E+00_JPRB, 0.42319E+00_JPRB, 0.42973E+00_JPRB, 0.40741E+00_JPRB, 0.43778E+00_JPRB, 0.56232E+00_JPRB ,&
&  0.66207E+00_JPRB, 0.70653E+00_JPRB, 0.75957E+00_JPRB, 0.77018E+00_JPRB, 0.78508E+00_JPRB, 0.76252E+00_JPRB, 0.12885E+01_JPRB ,&
&  0.96290E+00_JPRB, 0.96873E+00_JPRB, 0.98661E+00_JPRB, 0.95829E+00_JPRB, 0.93825E+00_JPRB, 0.88842E+00_JPRB, 0.83873E+00_JPRB ,&
&  0.81113E+00_JPRB, 0.79058E+00_JPRB, 0.77497E+00_JPRB, 0.76899E+00_JPRB, 0.75978E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.74146E+00_JPRB, 0.70673E+00_JPRB, 0.67666E+00_JPRB, 0.64828E+00_JPRB, 0.62491E+00_JPRB, 0.60752E+00_JPRB, 0.59429E+00_JPRB ,&
&  0.57537E+00_JPRB, 0.56076E+00_JPRB, 0.54246E+00_JPRB, 0.52352E+00_JPRB, 0.51424E+00_JPRB, 0.51342E+00_JPRB, 0.50531E+00_JPRB ,&
&  0.48896E+00_JPRB, 0.56518E+00_JPRB, 0.44896E+00_JPRB, 0.43096E+00_JPRB, 0.51827E+00_JPRB, 0.59239E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.10860E+01_JPRB, 0.17316E+01_JPRB, 0.21700E+01_JPRB, 0.30243E+01_JPRB, 0.48033E+01_JPRB, 0.45890E+01_JPRB, 0.46156E+01_JPRB ,&
&  0.45899E+01_JPRB, 0.38034E+01_JPRB, 0.32455E+01_JPRB, 0.27665E+01_JPRB, 0.24161E+01_JPRB, 0.21457E+01_JPRB, 0.19136E+01_JPRB ,&
&  0.17220E+01_JPRB, 0.15908E+01_JPRB, 0.15239E+01_JPRB, 0.15210E+01_JPRB, 0.15372E+01_JPRB, 0.15626E+01_JPRB, 0.15888E+01_JPRB ,&
&  0.15899E+01_JPRB, 0.15518E+01_JPRB, 0.14560E+01_JPRB, 0.14865E+01_JPRB, 0.16735E+01_JPRB, 0.19749E+01_JPRB, 0.27470E+01_JPRB ,&
&  0.27337E+01_JPRB, 0.28862E+01_JPRB, 0.28761E+01_JPRB, 0.41694E+01_JPRB, 0.47888E+01_JPRB, 0.44221E+01_JPRB, 0.40076E+01_JPRB ,&
&  0.29514E+01_JPRB, 0.23679E+01_JPRB, 0.19980E+01_JPRB, 0.17029E+01_JPRB, 0.14978E+01_JPRB, 0.15825E+01_JPRB, 0.20770E+01_JPRB ,&
&  0.17024E+01_JPRB, 0.17651E+01_JPRB, 0.17460E+01_JPRB, 0.14793E+01_JPRB, 0.88068E+00_JPRB, 0.70575E+00_JPRB, 0.93812E+00_JPRB ,&
&  0.97535E+00_JPRB, 0.65009E+00_JPRB, 0.51733E+00_JPRB, 0.60259E+00_JPRB, 0.49290E+00_JPRB, 0.51987E+00_JPRB, 0.61876E+00_JPRB ,&
&  0.64958E+00_JPRB, 0.68193E+00_JPRB, 0.82779E+00_JPRB, 0.83415E+00_JPRB, 0.82112E+00_JPRB, 0.73536E+00_JPRB, 0.98199E+00_JPRB ,&
&  0.89480E+00_JPRB, 0.86546E+00_JPRB, 0.90736E+00_JPRB, 0.88294E+00_JPRB, 0.88010E+00_JPRB, 0.86151E+00_JPRB, 0.82460E+00_JPRB ,&
&  0.80104E+00_JPRB, 0.78049E+00_JPRB, 0.76905E+00_JPRB, 0.76431E+00_JPRB, 0.76255E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.73540E+00_JPRB, 0.70061E+00_JPRB, 0.66385E+00_JPRB, 0.63799E+00_JPRB, 0.61636E+00_JPRB, 0.59371E+00_JPRB, 0.57425E+00_JPRB ,&
&  0.56262E+00_JPRB, 0.55111E+00_JPRB, 0.53033E+00_JPRB, 0.51556E+00_JPRB, 0.50548E+00_JPRB, 0.50771E+00_JPRB, 0.50533E+00_JPRB ,&
&  0.49162E+00_JPRB, 0.46148E+00_JPRB, 0.38620E+00_JPRB, 0.37367E+00_JPRB, 0.48511E+00_JPRB, 0.51466E+00_JPRB, 0.57879E+00_JPRB ,&
&  0.85723E+00_JPRB, 0.14579E+01_JPRB, 0.21384E+01_JPRB, 0.30437E+01_JPRB, 0.43385E+01_JPRB, 0.50579E+01_JPRB, 0.49067E+01_JPRB ,&
&  0.49445E+01_JPRB, 0.41834E+01_JPRB, 0.35624E+01_JPRB, 0.30621E+01_JPRB, 0.25983E+01_JPRB, 0.21674E+01_JPRB, 0.18329E+01_JPRB ,&
&  0.16435E+01_JPRB, 0.15142E+01_JPRB, 0.14429E+01_JPRB, 0.13761E+01_JPRB, 0.13997E+01_JPRB, 0.14676E+01_JPRB, 0.15398E+01_JPRB ,&
&  0.15655E+01_JPRB, 0.15629E+01_JPRB, 0.15327E+01_JPRB, 0.15105E+01_JPRB, 0.17201E+01_JPRB, 0.20533E+01_JPRB, 0.23951E+01_JPRB ,&
&  0.26791E+01_JPRB, 0.27823E+01_JPRB, 0.29780E+01_JPRB, 0.35185E+01_JPRB, 0.41924E+01_JPRB, 0.39079E+01_JPRB, 0.38429E+01_JPRB ,&
&  0.29935E+01_JPRB, 0.24512E+01_JPRB, 0.19994E+01_JPRB, 0.14864E+01_JPRB, 0.15089E+01_JPRB, 0.21365E+01_JPRB, 0.22600E+01_JPRB ,&
&  0.21583E+01_JPRB, 0.21282E+01_JPRB, 0.19266E+01_JPRB, 0.14841E+01_JPRB, 0.11629E+01_JPRB, 0.92656E+00_JPRB, 0.10556E+01_JPRB ,&
&  0.10580E+01_JPRB, 0.10310E+01_JPRB, 0.94035E+00_JPRB, 0.81038E+00_JPRB, 0.69340E+00_JPRB, 0.72918E+00_JPRB, 0.66636E+00_JPRB ,&
&  0.62124E+00_JPRB, 0.62147E+00_JPRB, 0.76769E+00_JPRB, 0.83777E+00_JPRB, 0.81510E+00_JPRB, 0.68745E+00_JPRB, 0.87723E+00_JPRB ,&
&  0.82962E+00_JPRB, 0.83491E+00_JPRB, 0.79496E+00_JPRB, 0.77739E+00_JPRB, 0.78764E+00_JPRB, 0.79277E+00_JPRB, 0.79139E+00_JPRB ,&
&  0.77149E+00_JPRB, 0.76144E+00_JPRB, 0.75877E+00_JPRB, 0.75678E+00_JPRB, 0.75091E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.72279E+00_JPRB, 0.69079E+00_JPRB, 0.65938E+00_JPRB, 0.63583E+00_JPRB, 0.61325E+00_JPRB, 0.59091E+00_JPRB, 0.56615E+00_JPRB ,&
&  0.54807E+00_JPRB, 0.53720E+00_JPRB, 0.52227E+00_JPRB, 0.51047E+00_JPRB, 0.50654E+00_JPRB, 0.50950E+00_JPRB, 0.51439E+00_JPRB ,&
&  0.50980E+00_JPRB, 0.47680E+00_JPRB, 0.37500E+00_JPRB, 0.37465E+00_JPRB, 0.41995E+00_JPRB, 0.43157E+00_JPRB, 0.52566E+00_JPRB ,&
&  0.71981E+00_JPRB, 0.11592E+01_JPRB, 0.17377E+01_JPRB, 0.25417E+01_JPRB, 0.34388E+01_JPRB, 0.41220E+01_JPRB, 0.42275E+01_JPRB ,&
&  0.37010E+01_JPRB, 0.36046E+01_JPRB, 0.30928E+01_JPRB, 0.26436E+01_JPRB, 0.22001E+01_JPRB, 0.18741E+01_JPRB, 0.16501E+01_JPRB ,&
&  0.14595E+01_JPRB, 0.13402E+01_JPRB, 0.12837E+01_JPRB, 0.12671E+01_JPRB, 0.12909E+01_JPRB, 0.13441E+01_JPRB, 0.14167E+01_JPRB ,&
&  0.14440E+01_JPRB, 0.14562E+01_JPRB, 0.14475E+01_JPRB, 0.15282E+01_JPRB, 0.15821E+01_JPRB, 0.20773E+01_JPRB, 0.22748E+01_JPRB ,&
&  0.28606E+01_JPRB, 0.31152E+01_JPRB, 0.31434E+01_JPRB, 0.39254E+01_JPRB, 0.32754E+01_JPRB, 0.34994E+01_JPRB, 0.36220E+01_JPRB ,&
&  0.33647E+01_JPRB, 0.30553E+01_JPRB, 0.28320E+01_JPRB, 0.22422E+01_JPRB, 0.18010E+01_JPRB, 0.23291E+01_JPRB, 0.22712E+01_JPRB ,&
&  0.20670E+01_JPRB, 0.19799E+01_JPRB, 0.17757E+01_JPRB, 0.15918E+01_JPRB, 0.13241E+01_JPRB, 0.11323E+01_JPRB, 0.90849E+00_JPRB ,&
&  0.91281E+00_JPRB, 0.85732E+00_JPRB, 0.90729E+00_JPRB, 0.84726E+00_JPRB, 0.79855E+00_JPRB, 0.75278E+00_JPRB, 0.67837E+00_JPRB ,&
&  0.62034E+00_JPRB, 0.59439E+00_JPRB, 0.63837E+00_JPRB, 0.68568E+00_JPRB, 0.71168E+00_JPRB, 0.61657E+00_JPRB, 0.69720E+00_JPRB ,&
&  0.77073E+00_JPRB, 0.79356E+00_JPRB, 0.74135E+00_JPRB, 0.77270E+00_JPRB, 0.78633E+00_JPRB, 0.77538E+00_JPRB, 0.77035E+00_JPRB ,&
&  0.76055E+00_JPRB, 0.75124E+00_JPRB, 0.74973E+00_JPRB, 0.75603E+00_JPRB, 0.74588E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.68297E+00_JPRB, 0.65966E+00_JPRB, 0.63509E+00_JPRB, 0.61612E+00_JPRB, 0.60415E+00_JPRB, 0.59060E+00_JPRB, 0.56690E+00_JPRB ,&
&  0.53959E+00_JPRB, 0.52268E+00_JPRB, 0.51320E+00_JPRB, 0.50592E+00_JPRB, 0.50322E+00_JPRB, 0.50857E+00_JPRB, 0.51592E+00_JPRB ,&
&  0.51911E+00_JPRB, 0.61430E+00_JPRB, 0.46767E+00_JPRB, 0.37686E+00_JPRB, 0.34130E+00_JPRB, 0.40245E+00_JPRB, 0.51438E+00_JPRB ,&
&  0.68502E+00_JPRB, 0.98634E+00_JPRB, 0.14943E+01_JPRB, 0.19317E+01_JPRB, 0.24211E+01_JPRB, 0.28911E+01_JPRB, 0.30347E+01_JPRB ,&
&  0.31867E+01_JPRB, 0.25129E+01_JPRB, 0.22532E+01_JPRB, 0.18368E+01_JPRB, 0.15873E+01_JPRB, 0.14224E+01_JPRB, 0.12802E+01_JPRB ,&
&  0.11616E+01_JPRB, 0.11028E+01_JPRB, 0.10647E+01_JPRB, 0.10736E+01_JPRB, 0.10860E+01_JPRB, 0.11196E+01_JPRB, 0.11778E+01_JPRB ,&
&  0.12397E+01_JPRB, 0.12726E+01_JPRB, 0.12731E+01_JPRB, 0.13530E+01_JPRB, 0.16537E+01_JPRB, 0.20499E+01_JPRB, 0.23763E+01_JPRB ,&
&  0.23146E+01_JPRB, 0.22764E+01_JPRB, 0.36257E+01_JPRB, 0.41930E+01_JPRB, 0.42801E+01_JPRB, 0.42346E+01_JPRB, 0.38495E+01_JPRB ,&
&  0.34810E+01_JPRB, 0.31808E+01_JPRB, 0.29011E+01_JPRB, 0.24600E+01_JPRB, 0.25060E+01_JPRB, 0.24359E+01_JPRB, 0.23358E+01_JPRB ,&
&  0.20855E+01_JPRB, 0.19036E+01_JPRB, 0.17289E+01_JPRB, 0.15948E+01_JPRB, 0.14729E+01_JPRB, 0.13951E+01_JPRB, 0.12578E+01_JPRB ,&
&  0.10950E+01_JPRB, 0.11260E+01_JPRB, 0.10077E+01_JPRB, 0.86987E+00_JPRB, 0.74952E+00_JPRB, 0.70560E+00_JPRB, 0.64793E+00_JPRB ,&
&  0.59579E+00_JPRB, 0.55379E+00_JPRB, 0.56239E+00_JPRB, 0.58355E+00_JPRB, 0.60055E+00_JPRB, 0.59127E+00_JPRB, 0.57346E+00_JPRB ,&
&  0.56875E+00_JPRB, 0.69570E+00_JPRB, 0.76084E+00_JPRB, 0.77900E+00_JPRB, 0.80346E+00_JPRB, 0.77602E+00_JPRB, 0.76087E+00_JPRB ,&
&  0.74305E+00_JPRB, 0.73218E+00_JPRB, 0.72640E+00_JPRB, 0.71857E+00_JPRB, 0.70401E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.62854E+00_JPRB, 0.61904E+00_JPRB, 0.60988E+00_JPRB, 0.59758E+00_JPRB, 0.59057E+00_JPRB, 0.58019E+00_JPRB, 0.56203E+00_JPRB ,&
&  0.54511E+00_JPRB, 0.52869E+00_JPRB, 0.51430E+00_JPRB, 0.50496E+00_JPRB, 0.50101E+00_JPRB, 0.50337E+00_JPRB, 0.50380E+00_JPRB ,&
&  0.51698E+00_JPRB, 0.80957E+00_JPRB, 0.58948E+00_JPRB, 0.48134E+00_JPRB, 0.41405E+00_JPRB, 0.46684E+00_JPRB, 0.54855E+00_JPRB ,&
&  0.71826E+00_JPRB, 0.83193E+00_JPRB, 0.10496E+01_JPRB, 0.13783E+01_JPRB, 0.16838E+01_JPRB, 0.18927E+01_JPRB, 0.20440E+01_JPRB ,&
&  0.18728E+01_JPRB, 0.16981E+01_JPRB, 0.13231E+01_JPRB, 0.12419E+01_JPRB, 0.11174E+01_JPRB, 0.95969E+00_JPRB, 0.98991E+00_JPRB ,&
&  0.95767E+00_JPRB, 0.95977E+00_JPRB, 0.95698E+00_JPRB, 0.97453E+00_JPRB, 0.99382E+00_JPRB, 0.10194E+01_JPRB, 0.10539E+01_JPRB ,&
&  0.10867E+01_JPRB, 0.11225E+01_JPRB, 0.11528E+01_JPRB, 0.12083E+01_JPRB, 0.13945E+01_JPRB, 0.17933E+01_JPRB, 0.20933E+01_JPRB ,&
&  0.26233E+01_JPRB, 0.25614E+01_JPRB, 0.24485E+01_JPRB, 0.34012E+01_JPRB, 0.44929E+01_JPRB, 0.42312E+01_JPRB, 0.35179E+01_JPRB ,&
&  0.31900E+01_JPRB, 0.30029E+01_JPRB, 0.28097E+01_JPRB, 0.25746E+01_JPRB, 0.22639E+01_JPRB, 0.21532E+01_JPRB, 0.20299E+01_JPRB ,&
&  0.18264E+01_JPRB, 0.16637E+01_JPRB, 0.15514E+01_JPRB, 0.14562E+01_JPRB, 0.13682E+01_JPRB, 0.13228E+01_JPRB, 0.12900E+01_JPRB ,&
&  0.12179E+01_JPRB, 0.10635E+01_JPRB, 0.77312E+00_JPRB, 0.75374E+00_JPRB, 0.61899E+00_JPRB, 0.55860E+00_JPRB, 0.54089E+00_JPRB ,&
&  0.48916E+00_JPRB, 0.50529E+00_JPRB, 0.52319E+00_JPRB, 0.51122E+00_JPRB, 0.56482E+00_JPRB, 0.52882E+00_JPRB, 0.50318E+00_JPRB ,&
&  0.48734E+00_JPRB, 0.48816E+00_JPRB, 0.65959E+00_JPRB, 0.72981E+00_JPRB, 0.75301E+00_JPRB, 0.74549E+00_JPRB, 0.72756E+00_JPRB ,&
&  0.68892E+00_JPRB, 0.66391E+00_JPRB, 0.65573E+00_JPRB, 0.65271E+00_JPRB, 0.64058E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.56781E+00_JPRB, 0.57038E+00_JPRB, 0.56725E+00_JPRB, 0.55309E+00_JPRB, 0.54210E+00_JPRB, 0.53881E+00_JPRB, 0.54232E+00_JPRB ,&
&  0.54049E+00_JPRB, 0.52586E+00_JPRB, 0.51428E+00_JPRB, 0.50051E+00_JPRB, 0.49294E+00_JPRB, 0.48955E+00_JPRB, 0.49087E+00_JPRB ,&
&  0.55921E+00_JPRB, 0.74565E+00_JPRB, 0.57682E+00_JPRB, 0.46272E+00_JPRB, 0.54547E+00_JPRB, 0.52892E+00_JPRB, 0.56893E+00_JPRB ,&
&  0.60154E+00_JPRB, 0.72173E+00_JPRB, 0.77938E+00_JPRB, 0.89506E+00_JPRB, 0.10433E+01_JPRB, 0.12374E+01_JPRB, 0.13076E+01_JPRB ,&
&  0.11626E+01_JPRB, 0.92206E+00_JPRB, 0.76563E+00_JPRB, 0.73834E+00_JPRB, 0.78742E+00_JPRB, 0.71287E+00_JPRB, 0.76164E+00_JPRB ,&
&  0.78450E+00_JPRB, 0.81271E+00_JPRB, 0.83821E+00_JPRB, 0.85065E+00_JPRB, 0.87678E+00_JPRB, 0.91497E+00_JPRB, 0.95986E+00_JPRB ,&
&  0.10159E+01_JPRB, 0.10685E+01_JPRB, 0.11006E+01_JPRB, 0.11501E+01_JPRB, 0.13368E+01_JPRB, 0.18711E+01_JPRB, 0.23953E+01_JPRB ,&
&  0.29517E+01_JPRB, 0.32083E+01_JPRB, 0.36614E+01_JPRB, 0.43088E+01_JPRB, 0.43525E+01_JPRB, 0.41502E+01_JPRB, 0.36445E+01_JPRB ,&
&  0.30685E+01_JPRB, 0.25894E+01_JPRB, 0.22790E+01_JPRB, 0.19717E+01_JPRB, 0.17593E+01_JPRB, 0.16981E+01_JPRB, 0.16155E+01_JPRB ,&
&  0.14763E+01_JPRB, 0.13826E+01_JPRB, 0.13416E+01_JPRB, 0.12425E+01_JPRB, 0.11768E+01_JPRB, 0.11011E+01_JPRB, 0.10868E+01_JPRB ,&
&  0.10534E+01_JPRB, 0.74423E+00_JPRB, 0.62927E+00_JPRB, 0.65300E+00_JPRB, 0.56020E+00_JPRB, 0.50624E+00_JPRB, 0.49996E+00_JPRB ,&
&  0.45019E+00_JPRB, 0.45647E+00_JPRB, 0.48285E+00_JPRB, 0.45270E+00_JPRB, 0.47405E+00_JPRB, 0.49007E+00_JPRB, 0.43506E+00_JPRB ,&
&  0.41532E+00_JPRB, 0.42437E+00_JPRB, 0.52821E+00_JPRB, 0.60789E+00_JPRB, 0.64792E+00_JPRB, 0.64247E+00_JPRB, 0.61941E+00_JPRB ,&
&  0.57671E+00_JPRB, 0.56064E+00_JPRB, 0.56320E+00_JPRB, 0.56420E+00_JPRB, 0.56642E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.50834E+00_JPRB, 0.51791E+00_JPRB, 0.52224E+00_JPRB, 0.52292E+00_JPRB, 0.52272E+00_JPRB, 0.51969E+00_JPRB, 0.52356E+00_JPRB ,&
&  0.52768E+00_JPRB, 0.51964E+00_JPRB, 0.50572E+00_JPRB, 0.49323E+00_JPRB, 0.47958E+00_JPRB, 0.47362E+00_JPRB, 0.49393E+00_JPRB ,&
&  0.42119E+00_JPRB, 0.51441E+00_JPRB, 0.42958E+00_JPRB, 0.45014E+00_JPRB, 0.65660E+00_JPRB, 0.58338E+00_JPRB, 0.56796E+00_JPRB ,&
&  0.53953E+00_JPRB, 0.54868E+00_JPRB, 0.55792E+00_JPRB, 0.63453E+00_JPRB, 0.69963E+00_JPRB, 0.80051E+00_JPRB, 0.85818E+00_JPRB ,&
&  0.73523E+00_JPRB, 0.56577E+00_JPRB, 0.49848E+00_JPRB, 0.48968E+00_JPRB, 0.53770E+00_JPRB, 0.57450E+00_JPRB, 0.64455E+00_JPRB ,&
&  0.68335E+00_JPRB, 0.71405E+00_JPRB, 0.74054E+00_JPRB, 0.77777E+00_JPRB, 0.79877E+00_JPRB, 0.82404E+00_JPRB, 0.86119E+00_JPRB ,&
&  0.90614E+00_JPRB, 0.96712E+00_JPRB, 0.10284E+01_JPRB, 0.11520E+01_JPRB, 0.12809E+01_JPRB, 0.20178E+01_JPRB, 0.32432E+01_JPRB ,&
&  0.36762E+01_JPRB, 0.37628E+01_JPRB, 0.42137E+01_JPRB, 0.49949E+01_JPRB, 0.49106E+01_JPRB, 0.44090E+01_JPRB, 0.36496E+01_JPRB ,&
&  0.29519E+01_JPRB, 0.23644E+01_JPRB, 0.20358E+01_JPRB, 0.17663E+01_JPRB, 0.15138E+01_JPRB, 0.14583E+01_JPRB, 0.13675E+01_JPRB ,&
&  0.12325E+01_JPRB, 0.11756E+01_JPRB, 0.11431E+01_JPRB, 0.11083E+01_JPRB, 0.10488E+01_JPRB, 0.10173E+01_JPRB, 0.97791E+00_JPRB ,&
&  0.93340E+00_JPRB, 0.82672E+00_JPRB, 0.65617E+00_JPRB, 0.58017E+00_JPRB, 0.51512E+00_JPRB, 0.48181E+00_JPRB, 0.51024E+00_JPRB ,&
&  0.45745E+00_JPRB, 0.43343E+00_JPRB, 0.41990E+00_JPRB, 0.40040E+00_JPRB, 0.40157E+00_JPRB, 0.42362E+00_JPRB, 0.38986E+00_JPRB ,&
&  0.37636E+00_JPRB, 0.41926E+00_JPRB, 0.46063E+00_JPRB, 0.50900E+00_JPRB, 0.53533E+00_JPRB, 0.53809E+00_JPRB, 0.46355E+00_JPRB ,&
&  0.48577E+00_JPRB, 0.47871E+00_JPRB, 0.48656E+00_JPRB, 0.48614E+00_JPRB, 0.49411E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.46620E+00_JPRB, 0.47677E+00_JPRB, 0.48638E+00_JPRB, 0.49910E+00_JPRB, 0.51296E+00_JPRB, 0.51055E+00_JPRB, 0.51037E+00_JPRB ,&
&  0.51614E+00_JPRB, 0.51140E+00_JPRB, 0.49453E+00_JPRB, 0.48250E+00_JPRB, 0.46710E+00_JPRB, 0.46605E+00_JPRB, 0.41385E+00_JPRB ,&
&  0.36815E+00_JPRB, 0.41928E+00_JPRB, 0.40782E+00_JPRB, 0.44328E+00_JPRB, 0.50869E+00_JPRB, 0.50295E+00_JPRB, 0.51150E+00_JPRB ,&
&  0.50950E+00_JPRB, 0.48509E+00_JPRB, 0.47788E+00_JPRB, 0.49542E+00_JPRB, 0.51364E+00_JPRB, 0.56272E+00_JPRB, 0.62487E+00_JPRB ,&
&  0.56106E+00_JPRB, 0.45903E+00_JPRB, 0.41588E+00_JPRB, 0.42130E+00_JPRB, 0.47920E+00_JPRB, 0.53527E+00_JPRB, 0.57287E+00_JPRB ,&
&  0.60539E+00_JPRB, 0.63915E+00_JPRB, 0.67947E+00_JPRB, 0.72756E+00_JPRB, 0.75315E+00_JPRB, 0.77558E+00_JPRB, 0.80060E+00_JPRB ,&
&  0.82978E+00_JPRB, 0.87928E+00_JPRB, 0.94855E+00_JPRB, 0.10758E+01_JPRB, 0.12226E+01_JPRB, 0.19390E+01_JPRB, 0.28520E+01_JPRB ,&
&  0.32162E+01_JPRB, 0.34828E+01_JPRB, 0.38080E+01_JPRB, 0.40786E+01_JPRB, 0.40305E+01_JPRB, 0.35359E+01_JPRB, 0.29774E+01_JPRB ,&
&  0.25449E+01_JPRB, 0.21050E+01_JPRB, 0.18851E+01_JPRB, 0.15866E+01_JPRB, 0.13644E+01_JPRB, 0.12714E+01_JPRB, 0.11682E+01_JPRB ,&
&  0.10745E+01_JPRB, 0.10244E+01_JPRB, 0.97721E+00_JPRB, 0.94874E+00_JPRB, 0.88653E+00_JPRB, 0.86194E+00_JPRB, 0.82005E+00_JPRB ,&
&  0.80495E+00_JPRB, 0.77259E+00_JPRB, 0.67140E+00_JPRB, 0.58907E+00_JPRB, 0.51704E+00_JPRB, 0.49381E+00_JPRB, 0.45736E+00_JPRB ,&
&  0.43500E+00_JPRB, 0.41202E+00_JPRB, 0.38774E+00_JPRB, 0.36225E+00_JPRB, 0.35059E+00_JPRB, 0.35719E+00_JPRB, 0.35605E+00_JPRB ,&
&  0.38519E+00_JPRB, 0.43103E+00_JPRB, 0.43013E+00_JPRB, 0.43431E+00_JPRB, 0.44294E+00_JPRB, 0.46284E+00_JPRB, 0.37766E+00_JPRB ,&
&  0.38773E+00_JPRB, 0.44221E+00_JPRB, 0.44355E+00_JPRB, 0.44436E+00_JPRB, 0.44984E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.43938E+00_JPRB, 0.45225E+00_JPRB, 0.46585E+00_JPRB, 0.48841E+00_JPRB, 0.50726E+00_JPRB, 0.51362E+00_JPRB, 0.47805E+00_JPRB ,&
&  0.51736E+00_JPRB, 0.49660E+00_JPRB, 0.47852E+00_JPRB, 0.47640E+00_JPRB, 0.46954E+00_JPRB, 0.40295E+00_JPRB, 0.33433E+00_JPRB ,&
&  0.32382E+00_JPRB, 0.34581E+00_JPRB, 0.38499E+00_JPRB, 0.41099E+00_JPRB, 0.44091E+00_JPRB, 0.44776E+00_JPRB, 0.45002E+00_JPRB ,&
&  0.44739E+00_JPRB, 0.43182E+00_JPRB, 0.43483E+00_JPRB, 0.43918E+00_JPRB, 0.44119E+00_JPRB, 0.45001E+00_JPRB, 0.48063E+00_JPRB ,&
&  0.44816E+00_JPRB, 0.42220E+00_JPRB, 0.42542E+00_JPRB, 0.41374E+00_JPRB, 0.47260E+00_JPRB, 0.49789E+00_JPRB, 0.53357E+00_JPRB ,&
&  0.57295E+00_JPRB, 0.61268E+00_JPRB, 0.67783E+00_JPRB, 0.72659E+00_JPRB, 0.75943E+00_JPRB, 0.77988E+00_JPRB, 0.81102E+00_JPRB ,&
&  0.83194E+00_JPRB, 0.87214E+00_JPRB, 0.94730E+00_JPRB, 0.10905E+01_JPRB, 0.13310E+01_JPRB, 0.17811E+01_JPRB, 0.25888E+01_JPRB ,&
&  0.28872E+01_JPRB, 0.32857E+01_JPRB, 0.36014E+01_JPRB, 0.31408E+01_JPRB, 0.32054E+01_JPRB, 0.32182E+01_JPRB, 0.28946E+01_JPRB ,&
&  0.26152E+01_JPRB, 0.21217E+01_JPRB, 0.17784E+01_JPRB, 0.15293E+01_JPRB, 0.13203E+01_JPRB, 0.11098E+01_JPRB, 0.96237E+00_JPRB ,&
&  0.91867E+00_JPRB, 0.90994E+00_JPRB, 0.76180E+00_JPRB, 0.70354E+00_JPRB, 0.66073E+00_JPRB, 0.64620E+00_JPRB, 0.62677E+00_JPRB ,&
&  0.61531E+00_JPRB, 0.60591E+00_JPRB, 0.57441E+00_JPRB, 0.53650E+00_JPRB, 0.48374E+00_JPRB, 0.46214E+00_JPRB, 0.44300E+00_JPRB ,&
&  0.43617E+00_JPRB, 0.41424E+00_JPRB, 0.37449E+00_JPRB, 0.35588E+00_JPRB, 0.34224E+00_JPRB, 0.33380E+00_JPRB, 0.33592E+00_JPRB ,&
&  0.35390E+00_JPRB, 0.36936E+00_JPRB, 0.41750E+00_JPRB, 0.42127E+00_JPRB, 0.42671E+00_JPRB, 0.45164E+00_JPRB, 0.40219E+00_JPRB ,&
&  0.38188E+00_JPRB, 0.44294E+00_JPRB, 0.43160E+00_JPRB, 0.42449E+00_JPRB, 0.42753E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.42132E+00_JPRB, 0.44431E+00_JPRB, 0.46206E+00_JPRB, 0.48330E+00_JPRB, 0.50654E+00_JPRB, 0.46693E+00_JPRB, 0.43852E+00_JPRB ,&
&  0.50176E+00_JPRB, 0.42708E+00_JPRB, 0.47392E+00_JPRB, 0.38613E+00_JPRB, 0.34646E+00_JPRB, 0.33489E+00_JPRB, 0.32757E+00_JPRB ,&
&  0.32327E+00_JPRB, 0.34562E+00_JPRB, 0.36700E+00_JPRB, 0.38344E+00_JPRB, 0.41714E+00_JPRB, 0.42091E+00_JPRB, 0.41364E+00_JPRB ,&
&  0.40079E+00_JPRB, 0.38749E+00_JPRB, 0.39111E+00_JPRB, 0.39159E+00_JPRB, 0.40199E+00_JPRB, 0.41347E+00_JPRB, 0.42109E+00_JPRB ,&
&  0.39937E+00_JPRB, 0.40050E+00_JPRB, 0.42650E+00_JPRB, 0.44628E+00_JPRB, 0.46103E+00_JPRB, 0.47247E+00_JPRB, 0.49967E+00_JPRB ,&
&  0.53276E+00_JPRB, 0.54833E+00_JPRB, 0.70020E+00_JPRB, 0.73943E+00_JPRB, 0.78239E+00_JPRB, 0.80927E+00_JPRB, 0.84168E+00_JPRB ,&
&  0.88271E+00_JPRB, 0.93310E+00_JPRB, 0.98527E+00_JPRB, 0.10635E+01_JPRB, 0.12301E+01_JPRB, 0.16103E+01_JPRB, 0.20542E+01_JPRB ,&
&  0.27961E+01_JPRB, 0.21651E+01_JPRB, 0.26614E+01_JPRB, 0.25400E+01_JPRB, 0.27168E+01_JPRB, 0.28247E+01_JPRB, 0.26649E+01_JPRB ,&
&  0.24629E+01_JPRB, 0.19911E+01_JPRB, 0.15865E+01_JPRB, 0.12690E+01_JPRB, 0.10464E+01_JPRB, 0.94144E+00_JPRB, 0.84536E+00_JPRB ,&
&  0.78368E+00_JPRB, 0.71965E+00_JPRB, 0.62794E+00_JPRB, 0.58171E+00_JPRB, 0.57393E+00_JPRB, 0.56926E+00_JPRB, 0.53831E+00_JPRB ,&
&  0.51684E+00_JPRB, 0.49976E+00_JPRB, 0.47478E+00_JPRB, 0.45323E+00_JPRB, 0.43767E+00_JPRB, 0.42642E+00_JPRB, 0.41768E+00_JPRB ,&
&  0.41636E+00_JPRB, 0.40841E+00_JPRB, 0.38199E+00_JPRB, 0.36928E+00_JPRB, 0.36039E+00_JPRB, 0.34516E+00_JPRB, 0.33539E+00_JPRB ,&
&  0.32831E+00_JPRB, 0.32103E+00_JPRB, 0.33474E+00_JPRB, 0.36057E+00_JPRB, 0.36715E+00_JPRB, 0.37445E+00_JPRB, 0.41504E+00_JPRB ,&
&  0.39380E+00_JPRB, 0.41138E+00_JPRB, 0.41388E+00_JPRB, 0.39980E+00_JPRB, 0.40191E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.40106E+00_JPRB, 0.42880E+00_JPRB, 0.45641E+00_JPRB, 0.47748E+00_JPRB, 0.49956E+00_JPRB, 0.45464E+00_JPRB, 0.41075E+00_JPRB ,&
&  0.43777E+00_JPRB, 0.38039E+00_JPRB, 0.34954E+00_JPRB, 0.33668E+00_JPRB, 0.33440E+00_JPRB, 0.32759E+00_JPRB, 0.31869E+00_JPRB ,&
&  0.31690E+00_JPRB, 0.32681E+00_JPRB, 0.34696E+00_JPRB, 0.36211E+00_JPRB, 0.37590E+00_JPRB, 0.38184E+00_JPRB, 0.37713E+00_JPRB ,&
&  0.37070E+00_JPRB, 0.36649E+00_JPRB, 0.36452E+00_JPRB, 0.36661E+00_JPRB, 0.37120E+00_JPRB, 0.37925E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38515E+00_JPRB, 0.39054E+00_JPRB, 0.40420E+00_JPRB, 0.42910E+00_JPRB, 0.43451E+00_JPRB, 0.43933E+00_JPRB, 0.44529E+00_JPRB ,&
&  0.36966E+00_JPRB, 0.43203E+00_JPRB, 0.66680E+00_JPRB, 0.73265E+00_JPRB, 0.77339E+00_JPRB, 0.82950E+00_JPRB, 0.88202E+00_JPRB ,&
&  0.93424E+00_JPRB, 0.10032E+01_JPRB, 0.10895E+01_JPRB, 0.11850E+01_JPRB, 0.12918E+01_JPRB, 0.14625E+01_JPRB, 0.17120E+01_JPRB ,&
&  0.23048E+01_JPRB, 0.19056E+01_JPRB, 0.17803E+01_JPRB, 0.17992E+01_JPRB, 0.24458E+01_JPRB, 0.23678E+01_JPRB, 0.19917E+01_JPRB ,&
&  0.17082E+01_JPRB, 0.15061E+01_JPRB, 0.13325E+01_JPRB, 0.11519E+01_JPRB, 0.10192E+01_JPRB, 0.95171E+00_JPRB, 0.85833E+00_JPRB ,&
&  0.74756E+00_JPRB, 0.67847E+00_JPRB, 0.60409E+00_JPRB, 0.58037E+00_JPRB, 0.57994E+00_JPRB, 0.54956E+00_JPRB, 0.50771E+00_JPRB ,&
&  0.47098E+00_JPRB, 0.44305E+00_JPRB, 0.42179E+00_JPRB, 0.40539E+00_JPRB, 0.39330E+00_JPRB, 0.38788E+00_JPRB, 0.38712E+00_JPRB ,&
&  0.38899E+00_JPRB, 0.39065E+00_JPRB, 0.38248E+00_JPRB, 0.37800E+00_JPRB, 0.36802E+00_JPRB, 0.34865E+00_JPRB, 0.33166E+00_JPRB ,&
&  0.32448E+00_JPRB, 0.31374E+00_JPRB, 0.30890E+00_JPRB, 0.31396E+00_JPRB, 0.32127E+00_JPRB, 0.33445E+00_JPRB, 0.35204E+00_JPRB ,&
&  0.37023E+00_JPRB, 0.37617E+00_JPRB, 0.35877E+00_JPRB, 0.35361E+00_JPRB, 0.36430E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.37789E+00_JPRB, 0.41853E+00_JPRB, 0.45078E+00_JPRB, 0.47639E+00_JPRB, 0.47533E+00_JPRB, 0.45622E+00_JPRB, 0.40693E+00_JPRB ,&
&  0.39704E+00_JPRB, 0.37225E+00_JPRB, 0.36001E+00_JPRB, 0.34571E+00_JPRB, 0.33584E+00_JPRB, 0.32855E+00_JPRB, 0.32683E+00_JPRB ,&
&  0.33202E+00_JPRB, 0.34154E+00_JPRB, 0.35052E+00_JPRB, 0.35533E+00_JPRB, 0.35864E+00_JPRB, 0.35875E+00_JPRB, 0.35772E+00_JPRB ,&
&  0.35599E+00_JPRB, 0.35448E+00_JPRB, 0.35430E+00_JPRB, 0.35627E+00_JPRB, 0.36243E+00_JPRB, 0.36983E+00_JPRB, 0.37095E+00_JPRB ,&
&  0.36635E+00_JPRB, 0.36267E+00_JPRB, 0.37777E+00_JPRB, 0.40152E+00_JPRB, 0.40054E+00_JPRB, 0.40279E+00_JPRB, 0.36786E+00_JPRB ,&
&  0.33390E+00_JPRB, 0.35135E+00_JPRB, 0.44126E+00_JPRB, 0.64344E+00_JPRB, 0.70721E+00_JPRB, 0.77100E+00_JPRB, 0.81575E+00_JPRB ,&
&  0.77661E+00_JPRB, 0.80933E+00_JPRB, 0.91590E+00_JPRB, 0.11713E+01_JPRB, 0.12416E+01_JPRB, 0.13562E+01_JPRB, 0.14746E+01_JPRB ,&
&  0.17290E+01_JPRB, 0.19000E+01_JPRB, 0.19603E+01_JPRB, 0.15379E+01_JPRB, 0.16410E+01_JPRB, 0.19268E+01_JPRB, 0.15994E+01_JPRB ,&
&  0.14108E+01_JPRB, 0.12885E+01_JPRB, 0.12497E+01_JPRB, 0.10950E+01_JPRB, 0.96328E+00_JPRB, 0.86956E+00_JPRB, 0.80865E+00_JPRB ,&
&  0.75344E+00_JPRB, 0.69956E+00_JPRB, 0.60945E+00_JPRB, 0.55731E+00_JPRB, 0.54700E+00_JPRB, 0.53085E+00_JPRB, 0.51341E+00_JPRB ,&
&  0.48612E+00_JPRB, 0.44578E+00_JPRB, 0.40776E+00_JPRB, 0.37395E+00_JPRB, 0.35258E+00_JPRB, 0.35365E+00_JPRB, 0.36556E+00_JPRB ,&
&  0.37631E+00_JPRB, 0.37821E+00_JPRB, 0.37692E+00_JPRB, 0.37135E+00_JPRB, 0.35769E+00_JPRB, 0.33794E+00_JPRB, 0.32482E+00_JPRB ,&
&  0.31510E+00_JPRB, 0.30570E+00_JPRB, 0.30239E+00_JPRB, 0.30668E+00_JPRB, 0.31324E+00_JPRB, 0.32388E+00_JPRB, 0.33586E+00_JPRB ,&
&  0.34143E+00_JPRB, 0.34813E+00_JPRB, 0.34490E+00_JPRB, 0.34530E+00_JPRB, 0.34902E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.34446E+00_JPRB, 0.38300E+00_JPRB, 0.43930E+00_JPRB, 0.47205E+00_JPRB, 0.47724E+00_JPRB, 0.44118E+00_JPRB, 0.40249E+00_JPRB ,&
&  0.38147E+00_JPRB, 0.36891E+00_JPRB, 0.36642E+00_JPRB, 0.36450E+00_JPRB, 0.36396E+00_JPRB, 0.36450E+00_JPRB, 0.36586E+00_JPRB ,&
&  0.36543E+00_JPRB, 0.36373E+00_JPRB, 0.36224E+00_JPRB, 0.36245E+00_JPRB, 0.36481E+00_JPRB, 0.36467E+00_JPRB, 0.36386E+00_JPRB ,&
&  0.36352E+00_JPRB, 0.36313E+00_JPRB, 0.36122E+00_JPRB, 0.36011E+00_JPRB, 0.36216E+00_JPRB, 0.36778E+00_JPRB, 0.37022E+00_JPRB ,&
&  0.36345E+00_JPRB, 0.35393E+00_JPRB, 0.36255E+00_JPRB, 0.38199E+00_JPRB, 0.38172E+00_JPRB, 0.38135E+00_JPRB, 0.35180E+00_JPRB ,&
&  0.32688E+00_JPRB, 0.31077E+00_JPRB, 0.32099E+00_JPRB, 0.37165E+00_JPRB, 0.46406E+00_JPRB, 0.63812E+00_JPRB, 0.72311E+00_JPRB ,&
&  0.80629E+00_JPRB, 0.87246E+00_JPRB, 0.93404E+00_JPRB, 0.99618E+00_JPRB, 0.10784E+01_JPRB, 0.11569E+01_JPRB, 0.12046E+01_JPRB ,&
&  0.12767E+01_JPRB, 0.13704E+01_JPRB, 0.15110E+01_JPRB, 0.11425E+01_JPRB, 0.10470E+01_JPRB, 0.11072E+01_JPRB, 0.11629E+01_JPRB ,&
&  0.11373E+01_JPRB, 0.10122E+01_JPRB, 0.98054E+00_JPRB, 0.95972E+00_JPRB, 0.86739E+00_JPRB, 0.78780E+00_JPRB, 0.73305E+00_JPRB ,&
&  0.68960E+00_JPRB, 0.66242E+00_JPRB, 0.60096E+00_JPRB, 0.52702E+00_JPRB, 0.50021E+00_JPRB, 0.49486E+00_JPRB, 0.49382E+00_JPRB ,&
&  0.49797E+00_JPRB, 0.48491E+00_JPRB, 0.42657E+00_JPRB, 0.35674E+00_JPRB, 0.34305E+00_JPRB, 0.34800E+00_JPRB, 0.36258E+00_JPRB ,&
&  0.37124E+00_JPRB, 0.37011E+00_JPRB, 0.36976E+00_JPRB, 0.36505E+00_JPRB, 0.35680E+00_JPRB, 0.34268E+00_JPRB, 0.32985E+00_JPRB ,&
&  0.31878E+00_JPRB, 0.31620E+00_JPRB, 0.32259E+00_JPRB, 0.32603E+00_JPRB, 0.33703E+00_JPRB, 0.34762E+00_JPRB, 0.35864E+00_JPRB ,&
&  0.35642E+00_JPRB, 0.35340E+00_JPRB, 0.34909E+00_JPRB, 0.33754E+00_JPRB, 0.33064E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.35556E+00_JPRB, 0.38758E+00_JPRB, 0.43049E+00_JPRB, 0.46082E+00_JPRB, 0.46580E+00_JPRB, 0.44318E+00_JPRB, 0.42382E+00_JPRB ,&
&  0.41307E+00_JPRB, 0.41021E+00_JPRB, 0.40789E+00_JPRB, 0.40433E+00_JPRB, 0.40066E+00_JPRB, 0.39676E+00_JPRB, 0.39290E+00_JPRB ,&
&  0.39006E+00_JPRB, 0.38825E+00_JPRB, 0.38592E+00_JPRB, 0.38073E+00_JPRB, 0.37706E+00_JPRB, 0.37655E+00_JPRB, 0.37879E+00_JPRB ,&
&  0.38213E+00_JPRB, 0.38163E+00_JPRB, 0.38081E+00_JPRB, 0.37657E+00_JPRB, 0.36716E+00_JPRB, 0.35633E+00_JPRB, 0.34758E+00_JPRB ,&
&  0.34242E+00_JPRB, 0.34615E+00_JPRB, 0.35477E+00_JPRB, 0.37056E+00_JPRB, 0.37000E+00_JPRB, 0.36759E+00_JPRB, 0.33959E+00_JPRB ,&
&  0.32035E+00_JPRB, 0.29860E+00_JPRB, 0.29019E+00_JPRB, 0.29895E+00_JPRB, 0.32974E+00_JPRB, 0.37905E+00_JPRB, 0.46069E+00_JPRB ,&
&  0.61015E+00_JPRB, 0.67040E+00_JPRB, 0.71884E+00_JPRB, 0.77578E+00_JPRB, 0.83112E+00_JPRB, 0.88332E+00_JPRB, 0.92501E+00_JPRB ,&
&  0.97912E+00_JPRB, 0.10406E+01_JPRB, 0.11043E+01_JPRB, 0.10778E+01_JPRB, 0.10673E+01_JPRB, 0.96966E+00_JPRB, 0.10399E+01_JPRB ,&
&  0.12425E+01_JPRB, 0.10793E+01_JPRB, 0.93805E+00_JPRB, 0.83603E+00_JPRB, 0.76185E+00_JPRB, 0.69686E+00_JPRB, 0.64655E+00_JPRB ,&
&  0.60626E+00_JPRB, 0.58306E+00_JPRB, 0.56143E+00_JPRB, 0.51300E+00_JPRB, 0.47233E+00_JPRB, 0.45618E+00_JPRB, 0.45612E+00_JPRB ,&
&  0.47446E+00_JPRB, 0.49181E+00_JPRB, 0.54073E+00_JPRB, 0.43029E+00_JPRB, 0.40741E+00_JPRB, 0.40041E+00_JPRB, 0.40091E+00_JPRB ,&
&  0.39562E+00_JPRB, 0.38666E+00_JPRB, 0.38327E+00_JPRB, 0.37882E+00_JPRB, 0.37163E+00_JPRB, 0.36266E+00_JPRB, 0.35542E+00_JPRB ,&
&  0.35069E+00_JPRB, 0.35398E+00_JPRB, 0.36218E+00_JPRB, 0.36899E+00_JPRB, 0.37303E+00_JPRB, 0.37462E+00_JPRB, 0.37662E+00_JPRB ,&
&  0.37554E+00_JPRB, 0.37158E+00_JPRB, 0.36671E+00_JPRB, 0.35595E+00_JPRB, 0.34865E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.37401E+00_JPRB, 0.39820E+00_JPRB, 0.42647E+00_JPRB, 0.44551E+00_JPRB, 0.45581E+00_JPRB, 0.44532E+00_JPRB, 0.43377E+00_JPRB ,&
&  0.42434E+00_JPRB, 0.41731E+00_JPRB, 0.41278E+00_JPRB, 0.40953E+00_JPRB, 0.40543E+00_JPRB, 0.40155E+00_JPRB, 0.39788E+00_JPRB ,&
&  0.39506E+00_JPRB, 0.38731E+00_JPRB, 0.38319E+00_JPRB, 0.38358E+00_JPRB, 0.38507E+00_JPRB, 0.38834E+00_JPRB, 0.39241E+00_JPRB ,&
&  0.39214E+00_JPRB, 0.38754E+00_JPRB, 0.38387E+00_JPRB, 0.37176E+00_JPRB, 0.35736E+00_JPRB, 0.34876E+00_JPRB, 0.35166E+00_JPRB ,&
&  0.35118E+00_JPRB, 0.34268E+00_JPRB, 0.35761E+00_JPRB, 0.37129E+00_JPRB, 0.36379E+00_JPRB, 0.35693E+00_JPRB, 0.31030E+00_JPRB ,&
&  0.29696E+00_JPRB, 0.28439E+00_JPRB, 0.28403E+00_JPRB, 0.28861E+00_JPRB, 0.30803E+00_JPRB, 0.34613E+00_JPRB, 0.40376E+00_JPRB ,&
&  0.46581E+00_JPRB, 0.56809E+00_JPRB, 0.59422E+00_JPRB, 0.64371E+00_JPRB, 0.69436E+00_JPRB, 0.74270E+00_JPRB, 0.78192E+00_JPRB ,&
&  0.83609E+00_JPRB, 0.90736E+00_JPRB, 0.95211E+00_JPRB, 0.96295E+00_JPRB, 0.97814E+00_JPRB, 0.98209E+00_JPRB, 0.99750E+00_JPRB ,&
&  0.10195E+01_JPRB, 0.86521E+00_JPRB, 0.76212E+00_JPRB, 0.68710E+00_JPRB, 0.63763E+00_JPRB, 0.61420E+00_JPRB, 0.56658E+00_JPRB ,&
&  0.54376E+00_JPRB, 0.53092E+00_JPRB, 0.51401E+00_JPRB, 0.50488E+00_JPRB, 0.49283E+00_JPRB, 0.47419E+00_JPRB, 0.44576E+00_JPRB ,&
&  0.41782E+00_JPRB, 0.40271E+00_JPRB, 0.42274E+00_JPRB, 0.45531E+00_JPRB, 0.46317E+00_JPRB, 0.44863E+00_JPRB, 0.42417E+00_JPRB ,&
&  0.39900E+00_JPRB, 0.38509E+00_JPRB, 0.38226E+00_JPRB, 0.38152E+00_JPRB, 0.38121E+00_JPRB, 0.37987E+00_JPRB, 0.38459E+00_JPRB ,&
&  0.38424E+00_JPRB, 0.38967E+00_JPRB, 0.39591E+00_JPRB, 0.39917E+00_JPRB, 0.39778E+00_JPRB, 0.39323E+00_JPRB, 0.38911E+00_JPRB ,&
&  0.38486E+00_JPRB, 0.38026E+00_JPRB, 0.37607E+00_JPRB, 0.37058E+00_JPRB, 0.36726E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.38974E+00_JPRB, 0.40258E+00_JPRB, 0.42182E+00_JPRB, 0.43640E+00_JPRB, 0.44755E+00_JPRB, 0.44548E+00_JPRB, 0.43701E+00_JPRB ,&
&  0.42860E+00_JPRB, 0.42201E+00_JPRB, 0.41599E+00_JPRB, 0.40994E+00_JPRB, 0.40286E+00_JPRB, 0.39724E+00_JPRB, 0.39307E+00_JPRB ,&
&  0.39105E+00_JPRB, 0.39047E+00_JPRB, 0.39073E+00_JPRB, 0.39098E+00_JPRB, 0.39180E+00_JPRB, 0.39324E+00_JPRB, 0.39468E+00_JPRB ,&
&  0.39643E+00_JPRB, 0.39273E+00_JPRB, 0.37875E+00_JPRB, 0.36278E+00_JPRB, 0.35578E+00_JPRB, 0.34890E+00_JPRB, 0.35511E+00_JPRB ,&
&  0.35661E+00_JPRB, 0.35979E+00_JPRB, 0.36113E+00_JPRB, 0.36378E+00_JPRB, 0.35430E+00_JPRB, 0.29631E+00_JPRB, 0.29679E+00_JPRB ,&
&  0.28927E+00_JPRB, 0.28222E+00_JPRB, 0.28482E+00_JPRB, 0.28969E+00_JPRB, 0.30030E+00_JPRB, 0.32372E+00_JPRB, 0.36329E+00_JPRB ,&
&  0.40196E+00_JPRB, 0.53722E+00_JPRB, 0.57234E+00_JPRB, 0.60898E+00_JPRB, 0.64168E+00_JPRB, 0.68384E+00_JPRB, 0.73537E+00_JPRB ,&
&  0.77177E+00_JPRB, 0.81286E+00_JPRB, 0.84232E+00_JPRB, 0.86578E+00_JPRB, 0.86881E+00_JPRB, 0.85894E+00_JPRB, 0.84585E+00_JPRB ,&
&  0.81426E+00_JPRB, 0.72823E+00_JPRB, 0.66362E+00_JPRB, 0.62072E+00_JPRB, 0.58096E+00_JPRB, 0.55302E+00_JPRB, 0.53186E+00_JPRB ,&
&  0.52537E+00_JPRB, 0.50080E+00_JPRB, 0.48874E+00_JPRB, 0.46894E+00_JPRB, 0.45395E+00_JPRB, 0.43790E+00_JPRB, 0.41885E+00_JPRB ,&
&  0.40268E+00_JPRB, 0.39846E+00_JPRB, 0.40592E+00_JPRB, 0.42510E+00_JPRB, 0.43351E+00_JPRB, 0.43079E+00_JPRB, 0.42443E+00_JPRB ,&
&  0.41111E+00_JPRB, 0.40771E+00_JPRB, 0.40602E+00_JPRB, 0.40724E+00_JPRB, 0.40721E+00_JPRB, 0.40973E+00_JPRB, 0.41315E+00_JPRB ,&
&  0.41648E+00_JPRB, 0.41732E+00_JPRB, 0.41748E+00_JPRB, 0.41799E+00_JPRB, 0.41733E+00_JPRB, 0.41490E+00_JPRB, 0.41042E+00_JPRB ,&
&  0.40556E+00_JPRB, 0.40231E+00_JPRB, 0.39601E+00_JPRB, 0.39030E+00_JPRB, 0.38729E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.40396E+00_JPRB, 0.40810E+00_JPRB, 0.41502E+00_JPRB, 0.42034E+00_JPRB, 0.42391E+00_JPRB, 0.42331E+00_JPRB, 0.41744E+00_JPRB ,&
&  0.41188E+00_JPRB, 0.40670E+00_JPRB, 0.40256E+00_JPRB, 0.39940E+00_JPRB, 0.39790E+00_JPRB, 0.39528E+00_JPRB, 0.39220E+00_JPRB ,&
&  0.38989E+00_JPRB, 0.38868E+00_JPRB, 0.38849E+00_JPRB, 0.38828E+00_JPRB, 0.38949E+00_JPRB, 0.38932E+00_JPRB, 0.39023E+00_JPRB ,&
&  0.39097E+00_JPRB, 0.38412E+00_JPRB, 0.36464E+00_JPRB, 0.35417E+00_JPRB, 0.34662E+00_JPRB, 0.34787E+00_JPRB, 0.35090E+00_JPRB ,&
&  0.35575E+00_JPRB, 0.31515E+00_JPRB, 0.31759E+00_JPRB, 0.30733E+00_JPRB, 0.29953E+00_JPRB, 0.29852E+00_JPRB, 0.29361E+00_JPRB ,&
&  0.29100E+00_JPRB, 0.28793E+00_JPRB, 0.29183E+00_JPRB, 0.29967E+00_JPRB, 0.30653E+00_JPRB, 0.32132E+00_JPRB, 0.34555E+00_JPRB ,&
&  0.38408E+00_JPRB, 0.50078E+00_JPRB, 0.55083E+00_JPRB, 0.58478E+00_JPRB, 0.61003E+00_JPRB, 0.64084E+00_JPRB, 0.67143E+00_JPRB ,&
&  0.69521E+00_JPRB, 0.72522E+00_JPRB, 0.73949E+00_JPRB, 0.73450E+00_JPRB, 0.74287E+00_JPRB, 0.73020E+00_JPRB, 0.70334E+00_JPRB ,&
&  0.66161E+00_JPRB, 0.60980E+00_JPRB, 0.57059E+00_JPRB, 0.54367E+00_JPRB, 0.52067E+00_JPRB, 0.50114E+00_JPRB, 0.48737E+00_JPRB ,&
&  0.48521E+00_JPRB, 0.47147E+00_JPRB, 0.45496E+00_JPRB, 0.43177E+00_JPRB, 0.41340E+00_JPRB, 0.40163E+00_JPRB, 0.39624E+00_JPRB ,&
&  0.39504E+00_JPRB, 0.40089E+00_JPRB, 0.41042E+00_JPRB, 0.42103E+00_JPRB, 0.42349E+00_JPRB, 0.42213E+00_JPRB, 0.42414E+00_JPRB ,&
&  0.42306E+00_JPRB, 0.42202E+00_JPRB, 0.42148E+00_JPRB, 0.42076E+00_JPRB, 0.41785E+00_JPRB, 0.41443E+00_JPRB, 0.41332E+00_JPRB ,&
&  0.41378E+00_JPRB, 0.41417E+00_JPRB, 0.41484E+00_JPRB, 0.41640E+00_JPRB, 0.41985E+00_JPRB, 0.42281E+00_JPRB, 0.42345E+00_JPRB ,&
&  0.41704E+00_JPRB, 0.41073E+00_JPRB, 0.40467E+00_JPRB, 0.40312E+00_JPRB, 0.40269E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.38971E+00_JPRB, 0.39047E+00_JPRB, 0.39169E+00_JPRB, 0.39252E+00_JPRB, 0.39268E+00_JPRB, 0.39274E+00_JPRB, 0.39239E+00_JPRB ,&
&  0.39219E+00_JPRB, 0.39258E+00_JPRB, 0.39270E+00_JPRB, 0.39218E+00_JPRB, 0.39245E+00_JPRB, 0.39315E+00_JPRB, 0.39318E+00_JPRB ,&
&  0.39096E+00_JPRB, 0.38802E+00_JPRB, 0.38537E+00_JPRB, 0.38211E+00_JPRB, 0.37926E+00_JPRB, 0.37724E+00_JPRB, 0.37420E+00_JPRB ,&
&  0.37231E+00_JPRB, 0.36714E+00_JPRB, 0.34363E+00_JPRB, 0.34247E+00_JPRB, 0.33741E+00_JPRB, 0.33666E+00_JPRB, 0.34009E+00_JPRB ,&
&  0.33680E+00_JPRB, 0.33762E+00_JPRB, 0.33198E+00_JPRB, 0.32124E+00_JPRB, 0.31369E+00_JPRB, 0.30289E+00_JPRB, 0.30204E+00_JPRB ,&
&  0.30048E+00_JPRB, 0.30388E+00_JPRB, 0.31278E+00_JPRB, 0.31847E+00_JPRB, 0.32305E+00_JPRB, 0.32918E+00_JPRB, 0.34468E+00_JPRB ,&
&  0.37704E+00_JPRB, 0.42286E+00_JPRB, 0.48130E+00_JPRB, 0.51021E+00_JPRB, 0.53405E+00_JPRB, 0.55843E+00_JPRB, 0.58103E+00_JPRB ,&
&  0.59807E+00_JPRB, 0.61203E+00_JPRB, 0.61410E+00_JPRB, 0.61256E+00_JPRB, 0.59977E+00_JPRB, 0.57663E+00_JPRB, 0.54989E+00_JPRB ,&
&  0.52265E+00_JPRB, 0.50281E+00_JPRB, 0.48696E+00_JPRB, 0.47576E+00_JPRB, 0.46602E+00_JPRB, 0.45451E+00_JPRB, 0.44456E+00_JPRB ,&
&  0.43286E+00_JPRB, 0.41529E+00_JPRB, 0.39851E+00_JPRB, 0.38322E+00_JPRB, 0.37480E+00_JPRB, 0.36982E+00_JPRB, 0.37015E+00_JPRB ,&
&  0.37229E+00_JPRB, 0.37802E+00_JPRB, 0.38447E+00_JPRB, 0.38749E+00_JPRB, 0.39166E+00_JPRB, 0.39707E+00_JPRB, 0.39767E+00_JPRB ,&
&  0.39874E+00_JPRB, 0.39990E+00_JPRB, 0.39939E+00_JPRB, 0.39801E+00_JPRB, 0.39482E+00_JPRB, 0.39105E+00_JPRB, 0.38757E+00_JPRB ,&
&  0.38566E+00_JPRB, 0.38529E+00_JPRB, 0.38783E+00_JPRB, 0.39078E+00_JPRB, 0.39471E+00_JPRB, 0.39635E+00_JPRB, 0.39460E+00_JPRB ,&
&  0.38956E+00_JPRB, 0.38694E+00_JPRB, 0.38503E+00_JPRB, 0.38656E+00_JPRB, 0.38827E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.37530E+00_JPRB, 0.37722E+00_JPRB, 0.37850E+00_JPRB, 0.37914E+00_JPRB, 0.37959E+00_JPRB, 0.38044E+00_JPRB, 0.38112E+00_JPRB ,&
&  0.38183E+00_JPRB, 0.38232E+00_JPRB, 0.38297E+00_JPRB, 0.38336E+00_JPRB, 0.38344E+00_JPRB, 0.38608E+00_JPRB, 0.38680E+00_JPRB ,&
&  0.38622E+00_JPRB, 0.38493E+00_JPRB, 0.38361E+00_JPRB, 0.38195E+00_JPRB, 0.37988E+00_JPRB, 0.37710E+00_JPRB, 0.37199E+00_JPRB ,&
&  0.36930E+00_JPRB, 0.36593E+00_JPRB, 0.36194E+00_JPRB, 0.36145E+00_JPRB, 0.36025E+00_JPRB, 0.34404E+00_JPRB, 0.34337E+00_JPRB ,&
&  0.35376E+00_JPRB, 0.35934E+00_JPRB, 0.36276E+00_JPRB, 0.36695E+00_JPRB, 0.35950E+00_JPRB, 0.34332E+00_JPRB, 0.33185E+00_JPRB ,&
&  0.31219E+00_JPRB, 0.32691E+00_JPRB, 0.33722E+00_JPRB, 0.34208E+00_JPRB, 0.34481E+00_JPRB, 0.34927E+00_JPRB, 0.37185E+00_JPRB ,&
&  0.41786E+00_JPRB, 0.42794E+00_JPRB, 0.43906E+00_JPRB, 0.45129E+00_JPRB, 0.46385E+00_JPRB, 0.47299E+00_JPRB, 0.48303E+00_JPRB ,&
&  0.48928E+00_JPRB, 0.49351E+00_JPRB, 0.49273E+00_JPRB, 0.48958E+00_JPRB, 0.48038E+00_JPRB, 0.47124E+00_JPRB, 0.46167E+00_JPRB ,&
&  0.45363E+00_JPRB, 0.44583E+00_JPRB, 0.43734E+00_JPRB, 0.42716E+00_JPRB, 0.41648E+00_JPRB, 0.40598E+00_JPRB, 0.39543E+00_JPRB ,&
&  0.38424E+00_JPRB, 0.37281E+00_JPRB, 0.36463E+00_JPRB, 0.35791E+00_JPRB, 0.35529E+00_JPRB, 0.35397E+00_JPRB, 0.35542E+00_JPRB ,&
&  0.35675E+00_JPRB, 0.35931E+00_JPRB, 0.36219E+00_JPRB, 0.36653E+00_JPRB, 0.36978E+00_JPRB, 0.37259E+00_JPRB, 0.37516E+00_JPRB ,&
&  0.37759E+00_JPRB, 0.37952E+00_JPRB, 0.38099E+00_JPRB, 0.38182E+00_JPRB, 0.38218E+00_JPRB, 0.38233E+00_JPRB, 0.38176E+00_JPRB ,&
&  0.38120E+00_JPRB, 0.38053E+00_JPRB, 0.37985E+00_JPRB, 0.37882E+00_JPRB, 0.37730E+00_JPRB, 0.37449E+00_JPRB, 0.37164E+00_JPRB ,&
&  0.36916E+00_JPRB, 0.36852E+00_JPRB, 0.36865E+00_JPRB, 0.37047E+00_JPRB, 0.37277E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.37814E+00_JPRB, 0.37938E+00_JPRB, 0.38025E+00_JPRB, 0.38067E+00_JPRB, 0.38095E+00_JPRB, 0.38115E+00_JPRB, 0.38087E+00_JPRB ,&
&  0.38005E+00_JPRB, 0.37944E+00_JPRB, 0.37930E+00_JPRB, 0.37895E+00_JPRB, 0.37818E+00_JPRB, 0.37909E+00_JPRB, 0.38122E+00_JPRB ,&
&  0.38136E+00_JPRB, 0.38120E+00_JPRB, 0.38106E+00_JPRB, 0.38122E+00_JPRB, 0.38126E+00_JPRB, 0.38102E+00_JPRB, 0.38103E+00_JPRB ,&
&  0.38154E+00_JPRB, 0.38181E+00_JPRB, 0.38182E+00_JPRB, 0.38129E+00_JPRB, 0.38219E+00_JPRB, 0.38301E+00_JPRB, 0.38338E+00_JPRB ,&
&  0.38303E+00_JPRB, 0.38213E+00_JPRB, 0.38134E+00_JPRB, 0.38127E+00_JPRB, 0.38377E+00_JPRB, 0.38704E+00_JPRB, 0.39151E+00_JPRB ,&
&  0.39696E+00_JPRB, 0.40336E+00_JPRB, 0.40965E+00_JPRB, 0.41539E+00_JPRB, 0.41989E+00_JPRB, 0.42505E+00_JPRB, 0.42874E+00_JPRB ,&
&  0.43000E+00_JPRB, 0.42911E+00_JPRB, 0.42778E+00_JPRB, 0.42771E+00_JPRB, 0.42773E+00_JPRB, 0.42741E+00_JPRB, 0.42719E+00_JPRB ,&
&  0.42699E+00_JPRB, 0.42666E+00_JPRB, 0.42632E+00_JPRB, 0.42486E+00_JPRB, 0.42232E+00_JPRB, 0.41952E+00_JPRB, 0.41622E+00_JPRB ,&
&  0.41258E+00_JPRB, 0.40809E+00_JPRB, 0.40320E+00_JPRB, 0.39763E+00_JPRB, 0.39177E+00_JPRB, 0.38584E+00_JPRB, 0.38035E+00_JPRB ,&
&  0.37622E+00_JPRB, 0.37373E+00_JPRB, 0.37201E+00_JPRB, 0.36963E+00_JPRB, 0.36768E+00_JPRB, 0.36639E+00_JPRB, 0.36663E+00_JPRB ,&
&  0.36708E+00_JPRB, 0.36851E+00_JPRB, 0.37026E+00_JPRB, 0.37268E+00_JPRB, 0.37454E+00_JPRB, 0.37617E+00_JPRB, 0.37748E+00_JPRB ,&
&  0.37852E+00_JPRB, 0.37938E+00_JPRB, 0.38018E+00_JPRB, 0.38095E+00_JPRB, 0.38108E+00_JPRB, 0.38090E+00_JPRB, 0.38023E+00_JPRB ,&
&  0.37939E+00_JPRB, 0.37841E+00_JPRB, 0.37715E+00_JPRB, 0.37578E+00_JPRB, 0.37455E+00_JPRB, 0.37319E+00_JPRB, 0.37264E+00_JPRB ,&
&  0.37244E+00_JPRB, 0.37282E+00_JPRB, 0.37356E+00_JPRB, 0.37512E+00_JPRB, 0.37679E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.38099E+00_JPRB, 0.38218E+00_JPRB, 0.38334E+00_JPRB, 0.38432E+00_JPRB, 0.38512E+00_JPRB, 0.38577E+00_JPRB, 0.38630E+00_JPRB ,&
&  0.38650E+00_JPRB, 0.38710E+00_JPRB, 0.38791E+00_JPRB, 0.38868E+00_JPRB, 0.38939E+00_JPRB, 0.39007E+00_JPRB, 0.39068E+00_JPRB ,&
&  0.39159E+00_JPRB, 0.39380E+00_JPRB, 0.39556E+00_JPRB, 0.39676E+00_JPRB, 0.39802E+00_JPRB, 0.39930E+00_JPRB, 0.40052E+00_JPRB ,&
&  0.40176E+00_JPRB, 0.40299E+00_JPRB, 0.40419E+00_JPRB, 0.40525E+00_JPRB, 0.40646E+00_JPRB, 0.40759E+00_JPRB, 0.40869E+00_JPRB ,&
&  0.40987E+00_JPRB, 0.41141E+00_JPRB, 0.41316E+00_JPRB, 0.41502E+00_JPRB, 0.41690E+00_JPRB, 0.41913E+00_JPRB, 0.42122E+00_JPRB ,&
&  0.42301E+00_JPRB, 0.42430E+00_JPRB, 0.42424E+00_JPRB, 0.42328E+00_JPRB, 0.42159E+00_JPRB, 0.41938E+00_JPRB, 0.41675E+00_JPRB ,&
&  0.41372E+00_JPRB, 0.41099E+00_JPRB, 0.40857E+00_JPRB, 0.40631E+00_JPRB, 0.40423E+00_JPRB, 0.40246E+00_JPRB, 0.40064E+00_JPRB ,&
&  0.39892E+00_JPRB, 0.39719E+00_JPRB, 0.39562E+00_JPRB, 0.39402E+00_JPRB, 0.39238E+00_JPRB, 0.39066E+00_JPRB, 0.38896E+00_JPRB ,&
&  0.38743E+00_JPRB, 0.38580E+00_JPRB, 0.38441E+00_JPRB, 0.38297E+00_JPRB, 0.38164E+00_JPRB, 0.38057E+00_JPRB, 0.37964E+00_JPRB ,&
&  0.37875E+00_JPRB, 0.37811E+00_JPRB, 0.37767E+00_JPRB, 0.37704E+00_JPRB, 0.37634E+00_JPRB, 0.37565E+00_JPRB, 0.37509E+00_JPRB ,&
&  0.37468E+00_JPRB, 0.37463E+00_JPRB, 0.37457E+00_JPRB, 0.37481E+00_JPRB, 0.37492E+00_JPRB, 0.37500E+00_JPRB, 0.37524E+00_JPRB ,&
&  0.37543E+00_JPRB, 0.37571E+00_JPRB, 0.37593E+00_JPRB, 0.37646E+00_JPRB, 0.37677E+00_JPRB, 0.37684E+00_JPRB, 0.37668E+00_JPRB ,&
&  0.37648E+00_JPRB, 0.37626E+00_JPRB, 0.37613E+00_JPRB, 0.37596E+00_JPRB, 0.37605E+00_JPRB, 0.37639E+00_JPRB, 0.37677E+00_JPRB ,&
&  0.37714E+00_JPRB, 0.37763E+00_JPRB, 0.37820E+00_JPRB, 0.37906E+00_JPRB, 0.37990E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB, 0.39126E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB, 0.14303E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.14805E+00_JPRB, 0.14808E+00_JPRB, 0.14800E+00_JPRB, 0.14748E+00_JPRB, 0.14574E+00_JPRB, 0.14545E+00_JPRB, 0.14545E+00_JPRB ,&
&  0.14562E+00_JPRB, 0.14572E+00_JPRB, 0.14643E+00_JPRB, 0.14721E+00_JPRB, 0.14796E+00_JPRB, 0.14853E+00_JPRB, 0.14901E+00_JPRB ,&
&  0.14927E+00_JPRB, 0.14953E+00_JPRB, 0.14865E+00_JPRB, 0.14822E+00_JPRB, 0.14810E+00_JPRB, 0.14812E+00_JPRB, 0.14812E+00_JPRB ,&
&  0.14807E+00_JPRB, 0.14806E+00_JPRB, 0.14793E+00_JPRB, 0.14808E+00_JPRB, 0.14832E+00_JPRB, 0.14852E+00_JPRB, 0.14877E+00_JPRB ,&
&  0.14894E+00_JPRB, 0.14911E+00_JPRB, 0.14920E+00_JPRB, 0.14927E+00_JPRB, 0.14950E+00_JPRB, 0.14960E+00_JPRB, 0.14949E+00_JPRB ,&
&  0.14930E+00_JPRB, 0.14896E+00_JPRB, 0.14858E+00_JPRB, 0.14819E+00_JPRB, 0.14790E+00_JPRB, 0.14762E+00_JPRB, 0.14736E+00_JPRB ,&
&  0.14710E+00_JPRB, 0.14681E+00_JPRB, 0.14650E+00_JPRB, 0.14616E+00_JPRB, 0.14584E+00_JPRB, 0.14560E+00_JPRB, 0.14534E+00_JPRB ,&
&  0.14513E+00_JPRB, 0.14494E+00_JPRB, 0.14472E+00_JPRB, 0.14446E+00_JPRB, 0.14418E+00_JPRB, 0.14391E+00_JPRB, 0.14361E+00_JPRB ,&
&  0.14323E+00_JPRB, 0.14297E+00_JPRB, 0.14271E+00_JPRB, 0.14247E+00_JPRB, 0.14228E+00_JPRB, 0.14208E+00_JPRB, 0.14190E+00_JPRB ,&
&  0.14170E+00_JPRB, 0.14151E+00_JPRB, 0.14130E+00_JPRB, 0.14110E+00_JPRB, 0.14092E+00_JPRB, 0.14077E+00_JPRB, 0.14065E+00_JPRB ,&
&  0.14053E+00_JPRB, 0.14042E+00_JPRB, 0.14033E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14040E+00_JPRB, 0.14043E+00_JPRB ,&
&  0.14048E+00_JPRB, 0.14051E+00_JPRB, 0.14053E+00_JPRB, 0.14046E+00_JPRB, 0.14059E+00_JPRB, 0.14077E+00_JPRB, 0.14104E+00_JPRB ,&
&  0.14135E+00_JPRB, 0.14175E+00_JPRB, 0.14217E+00_JPRB, 0.14261E+00_JPRB, 0.14311E+00_JPRB, 0.14353E+00_JPRB, 0.14400E+00_JPRB ,&
&  0.14454E+00_JPRB, 0.14510E+00_JPRB, 0.14551E+00_JPRB, 0.14598E+00_JPRB, 0.14671E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.15858E+00_JPRB, 0.16140E+00_JPRB, 0.16334E+00_JPRB, 0.16521E+00_JPRB, 0.16120E+00_JPRB, 0.17226E+00_JPRB, 0.16914E+00_JPRB ,&
&  0.16851E+00_JPRB, 0.16653E+00_JPRB, 0.16765E+00_JPRB, 0.16739E+00_JPRB, 0.16658E+00_JPRB, 0.16548E+00_JPRB, 0.16417E+00_JPRB ,&
&  0.16296E+00_JPRB, 0.16150E+00_JPRB, 0.16048E+00_JPRB, 0.15942E+00_JPRB, 0.15857E+00_JPRB, 0.15775E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15713E+00_JPRB, 0.15778E+00_JPRB, 0.15898E+00_JPRB, 0.15851E+00_JPRB, 0.15860E+00_JPRB, 0.15904E+00_JPRB, 0.15965E+00_JPRB ,&
&  0.15996E+00_JPRB, 0.16015E+00_JPRB, 0.15996E+00_JPRB, 0.15906E+00_JPRB, 0.15893E+00_JPRB, 0.15882E+00_JPRB, 0.15878E+00_JPRB ,&
&  0.15860E+00_JPRB, 0.15822E+00_JPRB, 0.15736E+00_JPRB, 0.15656E+00_JPRB, 0.15574E+00_JPRB, 0.15479E+00_JPRB, 0.15369E+00_JPRB ,&
&  0.15278E+00_JPRB, 0.15206E+00_JPRB, 0.15123E+00_JPRB, 0.15043E+00_JPRB, 0.14973E+00_JPRB, 0.14906E+00_JPRB, 0.14845E+00_JPRB ,&
&  0.14805E+00_JPRB, 0.14776E+00_JPRB, 0.14744E+00_JPRB, 0.14697E+00_JPRB, 0.14643E+00_JPRB, 0.14591E+00_JPRB, 0.14544E+00_JPRB ,&
&  0.14512E+00_JPRB, 0.14462E+00_JPRB, 0.14413E+00_JPRB, 0.14360E+00_JPRB, 0.14319E+00_JPRB, 0.14253E+00_JPRB, 0.14196E+00_JPRB ,&
&  0.14141E+00_JPRB, 0.14091E+00_JPRB, 0.14055E+00_JPRB, 0.14029E+00_JPRB, 0.14006E+00_JPRB, 0.13991E+00_JPRB, 0.13984E+00_JPRB ,&
&  0.13981E+00_JPRB, 0.13982E+00_JPRB, 0.13990E+00_JPRB, 0.13992E+00_JPRB, 0.13992E+00_JPRB, 0.13996E+00_JPRB, 0.14024E+00_JPRB ,&
&  0.14049E+00_JPRB, 0.14067E+00_JPRB, 0.14081E+00_JPRB, 0.14061E+00_JPRB, 0.14097E+00_JPRB, 0.14128E+00_JPRB, 0.14166E+00_JPRB ,&
&  0.14201E+00_JPRB, 0.14271E+00_JPRB, 0.14354E+00_JPRB, 0.14453E+00_JPRB, 0.14638E+00_JPRB, 0.14807E+00_JPRB, 0.14984E+00_JPRB ,&
&  0.15184E+00_JPRB, 0.15469E+00_JPRB, 0.15601E+00_JPRB, 0.15693E+00_JPRB, 0.15775E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.18307E+00_JPRB, 0.18440E+00_JPRB, 0.18569E+00_JPRB, 0.18689E+00_JPRB, 0.18817E+00_JPRB, 0.18863E+00_JPRB, 0.18849E+00_JPRB ,&
&  0.18622E+00_JPRB, 0.18554E+00_JPRB, 0.18398E+00_JPRB, 0.18215E+00_JPRB, 0.18068E+00_JPRB, 0.17822E+00_JPRB, 0.17565E+00_JPRB ,&
&  0.17301E+00_JPRB, 0.17046E+00_JPRB, 0.16814E+00_JPRB, 0.16498E+00_JPRB, 0.16193E+00_JPRB, 0.15896E+00_JPRB, 0.15791E+00_JPRB ,&
&  0.15743E+00_JPRB, 0.15779E+00_JPRB, 0.15925E+00_JPRB, 0.16056E+00_JPRB, 0.16245E+00_JPRB, 0.16449E+00_JPRB, 0.16759E+00_JPRB ,&
&  0.17071E+00_JPRB, 0.17305E+00_JPRB, 0.17346E+00_JPRB, 0.17246E+00_JPRB, 0.17146E+00_JPRB, 0.17106E+00_JPRB, 0.17055E+00_JPRB ,&
&  0.16968E+00_JPRB, 0.16903E+00_JPRB, 0.16830E+00_JPRB, 0.16763E+00_JPRB, 0.16615E+00_JPRB, 0.16372E+00_JPRB, 0.16131E+00_JPRB ,&
&  0.15909E+00_JPRB, 0.15713E+00_JPRB, 0.15534E+00_JPRB, 0.15379E+00_JPRB, 0.15288E+00_JPRB, 0.15198E+00_JPRB, 0.15092E+00_JPRB ,&
&  0.15028E+00_JPRB, 0.14961E+00_JPRB, 0.14882E+00_JPRB, 0.14822E+00_JPRB, 0.14753E+00_JPRB, 0.14703E+00_JPRB, 0.14646E+00_JPRB ,&
&  0.14600E+00_JPRB, 0.14534E+00_JPRB, 0.14476E+00_JPRB, 0.14414E+00_JPRB, 0.14385E+00_JPRB, 0.14318E+00_JPRB, 0.14255E+00_JPRB ,&
&  0.14177E+00_JPRB, 0.14129E+00_JPRB, 0.14042E+00_JPRB, 0.13991E+00_JPRB, 0.13932E+00_JPRB, 0.13880E+00_JPRB, 0.13847E+00_JPRB ,&
&  0.13859E+00_JPRB, 0.13867E+00_JPRB, 0.13842E+00_JPRB, 0.13844E+00_JPRB, 0.13855E+00_JPRB, 0.13897E+00_JPRB, 0.14012E+00_JPRB ,&
&  0.14086E+00_JPRB, 0.14169E+00_JPRB, 0.14251E+00_JPRB, 0.14316E+00_JPRB, 0.14421E+00_JPRB, 0.14524E+00_JPRB, 0.14628E+00_JPRB ,&
&  0.14749E+00_JPRB, 0.14855E+00_JPRB, 0.14980E+00_JPRB, 0.15128E+00_JPRB, 0.15473E+00_JPRB, 0.15737E+00_JPRB, 0.16028E+00_JPRB ,&
&  0.16260E+00_JPRB, 0.16227E+00_JPRB, 0.17557E+00_JPRB, 0.17820E+00_JPRB, 0.18011E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.19116E+00_JPRB, 0.19197E+00_JPRB, 0.19204E+00_JPRB, 0.19297E+00_JPRB, 0.19381E+00_JPRB, 0.19480E+00_JPRB, 0.19514E+00_JPRB ,&
&  0.19442E+00_JPRB, 0.18998E+00_JPRB, 0.18747E+00_JPRB, 0.18425E+00_JPRB, 0.18108E+00_JPRB, 0.17787E+00_JPRB, 0.17519E+00_JPRB ,&
&  0.17344E+00_JPRB, 0.17210E+00_JPRB, 0.17088E+00_JPRB, 0.16921E+00_JPRB, 0.16723E+00_JPRB, 0.16524E+00_JPRB, 0.16342E+00_JPRB ,&
&  0.16180E+00_JPRB, 0.16178E+00_JPRB, 0.16283E+00_JPRB, 0.16487E+00_JPRB, 0.16859E+00_JPRB, 0.17169E+00_JPRB, 0.17565E+00_JPRB ,&
&  0.18211E+00_JPRB, 0.17973E+00_JPRB, 0.17983E+00_JPRB, 0.17991E+00_JPRB, 0.17952E+00_JPRB, 0.17922E+00_JPRB, 0.17588E+00_JPRB ,&
&  0.17846E+00_JPRB, 0.17863E+00_JPRB, 0.18177E+00_JPRB, 0.18126E+00_JPRB, 0.17715E+00_JPRB, 0.17347E+00_JPRB, 0.16867E+00_JPRB ,&
&  0.16500E+00_JPRB, 0.16079E+00_JPRB, 0.15798E+00_JPRB, 0.15554E+00_JPRB, 0.15288E+00_JPRB, 0.15178E+00_JPRB, 0.14995E+00_JPRB ,&
&  0.14880E+00_JPRB, 0.14854E+00_JPRB, 0.14770E+00_JPRB, 0.14764E+00_JPRB, 0.14696E+00_JPRB, 0.14599E+00_JPRB, 0.14532E+00_JPRB ,&
&  0.14471E+00_JPRB, 0.14407E+00_JPRB, 0.14384E+00_JPRB, 0.14362E+00_JPRB, 0.14375E+00_JPRB, 0.14381E+00_JPRB, 0.14393E+00_JPRB ,&
&  0.14417E+00_JPRB, 0.14432E+00_JPRB, 0.14399E+00_JPRB, 0.14382E+00_JPRB, 0.14303E+00_JPRB, 0.14234E+00_JPRB, 0.14129E+00_JPRB ,&
&  0.14066E+00_JPRB, 0.14032E+00_JPRB, 0.13984E+00_JPRB, 0.13952E+00_JPRB, 0.13948E+00_JPRB, 0.13995E+00_JPRB, 0.14054E+00_JPRB ,&
&  0.14120E+00_JPRB, 0.14233E+00_JPRB, 0.14328E+00_JPRB, 0.14453E+00_JPRB, 0.14594E+00_JPRB, 0.14708E+00_JPRB, 0.14868E+00_JPRB ,&
&  0.15028E+00_JPRB, 0.15190E+00_JPRB, 0.15459E+00_JPRB, 0.15765E+00_JPRB, 0.16172E+00_JPRB, 0.16664E+00_JPRB, 0.16981E+00_JPRB ,&
&  0.17349E+00_JPRB, 0.18098E+00_JPRB, 0.18352E+00_JPRB, 0.18951E+00_JPRB, 0.19043E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.19605E+00_JPRB, 0.19591E+00_JPRB, 0.19533E+00_JPRB, 0.19485E+00_JPRB, 0.19527E+00_JPRB, 0.19600E+00_JPRB, 0.19682E+00_JPRB ,&
&  0.19341E+00_JPRB, 0.19061E+00_JPRB, 0.18630E+00_JPRB, 0.18151E+00_JPRB, 0.17736E+00_JPRB, 0.17387E+00_JPRB, 0.17180E+00_JPRB ,&
&  0.17152E+00_JPRB, 0.17327E+00_JPRB, 0.17484E+00_JPRB, 0.17655E+00_JPRB, 0.17675E+00_JPRB, 0.17694E+00_JPRB, 0.17577E+00_JPRB ,&
&  0.17448E+00_JPRB, 0.17393E+00_JPRB, 0.17420E+00_JPRB, 0.17604E+00_JPRB, 0.17875E+00_JPRB, 0.18070E+00_JPRB, 0.18294E+00_JPRB ,&
&  0.18589E+00_JPRB, 0.18548E+00_JPRB, 0.18625E+00_JPRB, 0.18576E+00_JPRB, 0.18585E+00_JPRB, 0.18646E+00_JPRB, 0.18671E+00_JPRB ,&
&  0.18726E+00_JPRB, 0.18788E+00_JPRB, 0.18761E+00_JPRB, 0.18698E+00_JPRB, 0.18305E+00_JPRB, 0.18017E+00_JPRB, 0.17554E+00_JPRB ,&
&  0.17017E+00_JPRB, 0.16550E+00_JPRB, 0.16045E+00_JPRB, 0.15697E+00_JPRB, 0.15308E+00_JPRB, 0.15075E+00_JPRB, 0.14897E+00_JPRB ,&
&  0.14728E+00_JPRB, 0.14661E+00_JPRB, 0.14570E+00_JPRB, 0.14476E+00_JPRB, 0.14459E+00_JPRB, 0.14370E+00_JPRB, 0.14362E+00_JPRB ,&
&  0.14382E+00_JPRB, 0.14401E+00_JPRB, 0.14482E+00_JPRB, 0.14526E+00_JPRB, 0.14603E+00_JPRB, 0.14677E+00_JPRB, 0.14742E+00_JPRB ,&
&  0.14852E+00_JPRB, 0.14980E+00_JPRB, 0.14986E+00_JPRB, 0.15005E+00_JPRB, 0.14926E+00_JPRB, 0.14769E+00_JPRB, 0.14594E+00_JPRB ,&
&  0.14413E+00_JPRB, 0.14271E+00_JPRB, 0.14227E+00_JPRB, 0.14166E+00_JPRB, 0.14182E+00_JPRB, 0.14217E+00_JPRB, 0.14239E+00_JPRB ,&
&  0.14303E+00_JPRB, 0.14360E+00_JPRB, 0.14435E+00_JPRB, 0.14594E+00_JPRB, 0.14711E+00_JPRB, 0.14859E+00_JPRB, 0.15012E+00_JPRB ,&
&  0.15135E+00_JPRB, 0.15351E+00_JPRB, 0.15542E+00_JPRB, 0.15804E+00_JPRB, 0.16398E+00_JPRB, 0.16954E+00_JPRB, 0.17943E+00_JPRB ,&
&  0.18519E+00_JPRB, 0.19845E+00_JPRB, 0.19734E+00_JPRB, 0.19571E+00_JPRB, 0.19646E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.20199E+00_JPRB, 0.20073E+00_JPRB, 0.19973E+00_JPRB, 0.19854E+00_JPRB, 0.19926E+00_JPRB, 0.20030E+00_JPRB, 0.20159E+00_JPRB ,&
&  0.20308E+00_JPRB, 0.20344E+00_JPRB, 0.20311E+00_JPRB, 0.20125E+00_JPRB, 0.20001E+00_JPRB, 0.19361E+00_JPRB, 0.19120E+00_JPRB ,&
&  0.18941E+00_JPRB, 0.19089E+00_JPRB, 0.19310E+00_JPRB, 0.19648E+00_JPRB, 0.19813E+00_JPRB, 0.19982E+00_JPRB, 0.19910E+00_JPRB ,&
&  0.19708E+00_JPRB, 0.19459E+00_JPRB, 0.19231E+00_JPRB, 0.19047E+00_JPRB, 0.18996E+00_JPRB, 0.18864E+00_JPRB, 0.18716E+00_JPRB ,&
&  0.18706E+00_JPRB, 0.18831E+00_JPRB, 0.18910E+00_JPRB, 0.18991E+00_JPRB, 0.19596E+00_JPRB, 0.19687E+00_JPRB, 0.19869E+00_JPRB ,&
&  0.19906E+00_JPRB, 0.19940E+00_JPRB, 0.19856E+00_JPRB, 0.19718E+00_JPRB, 0.19593E+00_JPRB, 0.19149E+00_JPRB, 0.18935E+00_JPRB ,&
&  0.18199E+00_JPRB, 0.17756E+00_JPRB, 0.17110E+00_JPRB, 0.16548E+00_JPRB, 0.15969E+00_JPRB, 0.15536E+00_JPRB, 0.15181E+00_JPRB ,&
&  0.14900E+00_JPRB, 0.14693E+00_JPRB, 0.14518E+00_JPRB, 0.14496E+00_JPRB, 0.14509E+00_JPRB, 0.14486E+00_JPRB, 0.14501E+00_JPRB ,&
&  0.14568E+00_JPRB, 0.14642E+00_JPRB, 0.14638E+00_JPRB, 0.14692E+00_JPRB, 0.14744E+00_JPRB, 0.14819E+00_JPRB, 0.14956E+00_JPRB ,&
&  0.15166E+00_JPRB, 0.15359E+00_JPRB, 0.15451E+00_JPRB, 0.15601E+00_JPRB, 0.15534E+00_JPRB, 0.15286E+00_JPRB, 0.15038E+00_JPRB ,&
&  0.14779E+00_JPRB, 0.14664E+00_JPRB, 0.14547E+00_JPRB, 0.14481E+00_JPRB, 0.14476E+00_JPRB, 0.14497E+00_JPRB, 0.14570E+00_JPRB ,&
&  0.14647E+00_JPRB, 0.14726E+00_JPRB, 0.14760E+00_JPRB, 0.14807E+00_JPRB, 0.14849E+00_JPRB, 0.14926E+00_JPRB, 0.15064E+00_JPRB ,&
&  0.15249E+00_JPRB, 0.15461E+00_JPRB, 0.15667E+00_JPRB, 0.15922E+00_JPRB, 0.16346E+00_JPRB, 0.16932E+00_JPRB, 0.17825E+00_JPRB ,&
&  0.18585E+00_JPRB, 0.19932E+00_JPRB, 0.20062E+00_JPRB, 0.20111E+00_JPRB, 0.20222E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.21028E+00_JPRB, 0.21042E+00_JPRB, 0.20695E+00_JPRB, 0.20378E+00_JPRB, 0.20230E+00_JPRB, 0.20180E+00_JPRB, 0.20282E+00_JPRB ,&
&  0.20532E+00_JPRB, 0.20720E+00_JPRB, 0.20908E+00_JPRB, 0.21073E+00_JPRB, 0.21143E+00_JPRB, 0.21142E+00_JPRB, 0.21233E+00_JPRB ,&
&  0.21320E+00_JPRB, 0.21540E+00_JPRB, 0.21638E+00_JPRB, 0.21849E+00_JPRB, 0.21874E+00_JPRB, 0.21949E+00_JPRB, 0.22066E+00_JPRB ,&
&  0.22032E+00_JPRB, 0.22113E+00_JPRB, 0.22272E+00_JPRB, 0.22084E+00_JPRB, 0.21954E+00_JPRB, 0.21749E+00_JPRB, 0.21570E+00_JPRB ,&
&  0.21001E+00_JPRB, 0.20391E+00_JPRB, 0.19703E+00_JPRB, 0.19968E+00_JPRB, 0.20779E+00_JPRB, 0.21011E+00_JPRB, 0.21357E+00_JPRB ,&
&  0.21439E+00_JPRB, 0.21284E+00_JPRB, 0.21109E+00_JPRB, 0.20813E+00_JPRB, 0.20519E+00_JPRB, 0.20200E+00_JPRB, 0.19952E+00_JPRB ,&
&  0.19687E+00_JPRB, 0.19730E+00_JPRB, 0.19407E+00_JPRB, 0.18175E+00_JPRB, 0.17482E+00_JPRB, 0.17058E+00_JPRB, 0.16445E+00_JPRB ,&
&  0.15853E+00_JPRB, 0.15651E+00_JPRB, 0.15600E+00_JPRB, 0.15426E+00_JPRB, 0.15438E+00_JPRB, 0.15471E+00_JPRB, 0.15472E+00_JPRB ,&
&  0.15367E+00_JPRB, 0.15347E+00_JPRB, 0.15223E+00_JPRB, 0.15045E+00_JPRB, 0.14884E+00_JPRB, 0.14821E+00_JPRB, 0.14858E+00_JPRB ,&
&  0.14995E+00_JPRB, 0.15164E+00_JPRB, 0.15683E+00_JPRB, 0.16038E+00_JPRB, 0.15676E+00_JPRB, 0.15360E+00_JPRB, 0.15304E+00_JPRB ,&
&  0.14968E+00_JPRB, 0.14894E+00_JPRB, 0.14890E+00_JPRB, 0.14885E+00_JPRB, 0.14917E+00_JPRB, 0.14998E+00_JPRB, 0.15007E+00_JPRB ,&
&  0.15065E+00_JPRB, 0.15167E+00_JPRB, 0.15191E+00_JPRB, 0.15191E+00_JPRB, 0.15173E+00_JPRB, 0.15213E+00_JPRB, 0.15318E+00_JPRB ,&
&  0.15447E+00_JPRB, 0.15676E+00_JPRB, 0.15921E+00_JPRB, 0.16269E+00_JPRB, 0.16548E+00_JPRB, 0.16765E+00_JPRB, 0.17144E+00_JPRB ,&
&  0.17505E+00_JPRB, 0.18326E+00_JPRB, 0.19751E+00_JPRB, 0.20889E+00_JPRB, 0.20688E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.21536E+00_JPRB, 0.21521E+00_JPRB, 0.21394E+00_JPRB, 0.21275E+00_JPRB, 0.21121E+00_JPRB, 0.20968E+00_JPRB, 0.21017E+00_JPRB ,&
&  0.21224E+00_JPRB, 0.21500E+00_JPRB, 0.21767E+00_JPRB, 0.21975E+00_JPRB, 0.22142E+00_JPRB, 0.22225E+00_JPRB, 0.22359E+00_JPRB ,&
&  0.22520E+00_JPRB, 0.22786E+00_JPRB, 0.22853E+00_JPRB, 0.22914E+00_JPRB, 0.22860E+00_JPRB, 0.22810E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.23249E+00_JPRB, 0.23418E+00_JPRB, 0.23358E+00_JPRB, 0.23176E+00_JPRB, 0.23229E+00_JPRB, 0.23394E+00_JPRB, 0.23305E+00_JPRB ,&
&  0.22720E+00_JPRB, 0.21453E+00_JPRB, 0.20523E+00_JPRB, 0.20717E+00_JPRB, 0.21611E+00_JPRB, 0.22018E+00_JPRB, 0.22265E+00_JPRB ,&
&  0.22421E+00_JPRB, 0.22447E+00_JPRB, 0.22377E+00_JPRB, 0.22248E+00_JPRB, 0.22020E+00_JPRB, 0.21627E+00_JPRB, 0.21150E+00_JPRB ,&
&  0.20713E+00_JPRB, 0.20471E+00_JPRB, 0.20211E+00_JPRB, 0.19765E+00_JPRB, 0.19310E+00_JPRB, 0.19016E+00_JPRB, 0.18712E+00_JPRB ,&
&  0.18408E+00_JPRB, 0.18152E+00_JPRB, 0.18089E+00_JPRB, 0.18068E+00_JPRB, 0.18083E+00_JPRB, 0.18063E+00_JPRB, 0.17903E+00_JPRB ,&
&  0.17545E+00_JPRB, 0.17150E+00_JPRB, 0.16597E+00_JPRB, 0.16108E+00_JPRB, 0.15752E+00_JPRB, 0.15422E+00_JPRB, 0.15047E+00_JPRB ,&
&  0.14853E+00_JPRB, 0.15052E+00_JPRB, 0.15796E+00_JPRB, 0.16681E+00_JPRB, 0.17411E+00_JPRB, 0.16254E+00_JPRB, 0.15774E+00_JPRB ,&
&  0.15616E+00_JPRB, 0.15562E+00_JPRB, 0.15577E+00_JPRB, 0.15689E+00_JPRB, 0.15751E+00_JPRB, 0.15744E+00_JPRB, 0.15794E+00_JPRB ,&
&  0.15952E+00_JPRB, 0.16122E+00_JPRB, 0.16215E+00_JPRB, 0.16305E+00_JPRB, 0.16357E+00_JPRB, 0.16335E+00_JPRB, 0.16305E+00_JPRB ,&
&  0.16355E+00_JPRB, 0.16599E+00_JPRB, 0.17024E+00_JPRB, 0.17633E+00_JPRB, 0.18105E+00_JPRB, 0.18539E+00_JPRB, 0.19113E+00_JPRB ,&
&  0.19838E+00_JPRB, 0.20602E+00_JPRB, 0.20956E+00_JPRB, 0.21270E+00_JPRB, 0.21437E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.22113E+00_JPRB, 0.22232E+00_JPRB, 0.22423E+00_JPRB, 0.22616E+00_JPRB, 0.22678E+00_JPRB, 0.22616E+00_JPRB, 0.22549E+00_JPRB ,&
&  0.22573E+00_JPRB, 0.22735E+00_JPRB, 0.22907E+00_JPRB, 0.22928E+00_JPRB, 0.22936E+00_JPRB, 0.22964E+00_JPRB, 0.23129E+00_JPRB ,&
&  0.23385E+00_JPRB, 0.23682E+00_JPRB, 0.23683E+00_JPRB, 0.23650E+00_JPRB, 0.23638E+00_JPRB, 0.23567E+00_JPRB, 0.23695E+00_JPRB ,&
&  0.23998E+00_JPRB, 0.24037E+00_JPRB, 0.24042E+00_JPRB, 0.23978E+00_JPRB, 0.24122E+00_JPRB, 0.24403E+00_JPRB, 0.24539E+00_JPRB ,&
&  0.24361E+00_JPRB, 0.23705E+00_JPRB, 0.22916E+00_JPRB, 0.22887E+00_JPRB, 0.23053E+00_JPRB, 0.23228E+00_JPRB, 0.23259E+00_JPRB ,&
&  0.23217E+00_JPRB, 0.23137E+00_JPRB, 0.23033E+00_JPRB, 0.22938E+00_JPRB, 0.22838E+00_JPRB, 0.22530E+00_JPRB, 0.22171E+00_JPRB ,&
&  0.21888E+00_JPRB, 0.21595E+00_JPRB, 0.21297E+00_JPRB, 0.20915E+00_JPRB, 0.20519E+00_JPRB, 0.20238E+00_JPRB, 0.19934E+00_JPRB ,&
&  0.19553E+00_JPRB, 0.19269E+00_JPRB, 0.19188E+00_JPRB, 0.19205E+00_JPRB, 0.19242E+00_JPRB, 0.19188E+00_JPRB, 0.19121E+00_JPRB ,&
&  0.18991E+00_JPRB, 0.18887E+00_JPRB, 0.18751E+00_JPRB, 0.18694E+00_JPRB, 0.18299E+00_JPRB, 0.17456E+00_JPRB, 0.16884E+00_JPRB ,&
&  0.16788E+00_JPRB, 0.17064E+00_JPRB, 0.17486E+00_JPRB, 0.17529E+00_JPRB, 0.17662E+00_JPRB, 0.17752E+00_JPRB, 0.17728E+00_JPRB ,&
&  0.17535E+00_JPRB, 0.17089E+00_JPRB, 0.17017E+00_JPRB, 0.17085E+00_JPRB, 0.17113E+00_JPRB, 0.17197E+00_JPRB, 0.17343E+00_JPRB ,&
&  0.17606E+00_JPRB, 0.17850E+00_JPRB, 0.18055E+00_JPRB, 0.18248E+00_JPRB, 0.18356E+00_JPRB, 0.18281E+00_JPRB, 0.18080E+00_JPRB ,&
&  0.18040E+00_JPRB, 0.18519E+00_JPRB, 0.19422E+00_JPRB, 0.20512E+00_JPRB, 0.20722E+00_JPRB, 0.20920E+00_JPRB, 0.21112E+00_JPRB ,&
&  0.21201E+00_JPRB, 0.21477E+00_JPRB, 0.21718E+00_JPRB, 0.21859E+00_JPRB, 0.21975E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.23103E+00_JPRB, 0.23262E+00_JPRB, 0.23341E+00_JPRB, 0.23439E+00_JPRB, 0.23626E+00_JPRB, 0.23759E+00_JPRB, 0.23779E+00_JPRB ,&
&  0.23784E+00_JPRB, 0.23875E+00_JPRB, 0.23933E+00_JPRB, 0.23956E+00_JPRB, 0.23727E+00_JPRB, 0.23704E+00_JPRB, 0.23869E+00_JPRB ,&
&  0.24086E+00_JPRB, 0.24332E+00_JPRB, 0.24364E+00_JPRB, 0.24366E+00_JPRB, 0.24345E+00_JPRB, 0.24276E+00_JPRB, 0.24448E+00_JPRB ,&
&  0.24589E+00_JPRB, 0.24657E+00_JPRB, 0.24797E+00_JPRB, 0.24921E+00_JPRB, 0.25182E+00_JPRB, 0.25481E+00_JPRB, 0.25619E+00_JPRB ,&
&  0.25558E+00_JPRB, 0.25379E+00_JPRB, 0.24787E+00_JPRB, 0.24149E+00_JPRB, 0.24222E+00_JPRB, 0.24355E+00_JPRB, 0.24359E+00_JPRB ,&
&  0.24161E+00_JPRB, 0.23894E+00_JPRB, 0.23710E+00_JPRB, 0.23564E+00_JPRB, 0.23397E+00_JPRB, 0.23132E+00_JPRB, 0.22847E+00_JPRB ,&
&  0.22594E+00_JPRB, 0.22508E+00_JPRB, 0.22374E+00_JPRB, 0.22165E+00_JPRB, 0.21869E+00_JPRB, 0.21640E+00_JPRB, 0.21406E+00_JPRB ,&
&  0.20975E+00_JPRB, 0.20543E+00_JPRB, 0.20314E+00_JPRB, 0.20273E+00_JPRB, 0.20333E+00_JPRB, 0.20315E+00_JPRB, 0.20177E+00_JPRB ,&
&  0.20015E+00_JPRB, 0.19789E+00_JPRB, 0.19654E+00_JPRB, 0.19615E+00_JPRB, 0.19720E+00_JPRB, 0.19639E+00_JPRB, 0.19253E+00_JPRB ,&
&  0.19021E+00_JPRB, 0.18851E+00_JPRB, 0.18832E+00_JPRB, 0.18730E+00_JPRB, 0.18636E+00_JPRB, 0.18507E+00_JPRB, 0.18533E+00_JPRB ,&
&  0.18617E+00_JPRB, 0.18656E+00_JPRB, 0.18886E+00_JPRB, 0.19292E+00_JPRB, 0.19665E+00_JPRB, 0.19884E+00_JPRB, 0.20011E+00_JPRB ,&
&  0.20056E+00_JPRB, 0.19980E+00_JPRB, 0.19906E+00_JPRB, 0.19935E+00_JPRB, 0.20096E+00_JPRB, 0.20321E+00_JPRB, 0.20547E+00_JPRB ,&
&  0.20618E+00_JPRB, 0.20715E+00_JPRB, 0.20928E+00_JPRB, 0.21253E+00_JPRB, 0.21451E+00_JPRB, 0.21624E+00_JPRB, 0.21783E+00_JPRB ,&
&  0.21869E+00_JPRB, 0.22009E+00_JPRB, 0.22355E+00_JPRB, 0.22579E+00_JPRB, 0.22816E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.23988E+00_JPRB, 0.24066E+00_JPRB, 0.24166E+00_JPRB, 0.24185E+00_JPRB, 0.24270E+00_JPRB, 0.24344E+00_JPRB, 0.24470E+00_JPRB ,&
&  0.24511E+00_JPRB, 0.24629E+00_JPRB, 0.24811E+00_JPRB, 0.24801E+00_JPRB, 0.24768E+00_JPRB, 0.24755E+00_JPRB, 0.24824E+00_JPRB ,&
&  0.25054E+00_JPRB, 0.25207E+00_JPRB, 0.25183E+00_JPRB, 0.25172E+00_JPRB, 0.25140E+00_JPRB, 0.25119E+00_JPRB, 0.25343E+00_JPRB ,&
&  0.25405E+00_JPRB, 0.25488E+00_JPRB, 0.25666E+00_JPRB, 0.25805E+00_JPRB, 0.26127E+00_JPRB, 0.26500E+00_JPRB, 0.26701E+00_JPRB ,&
&  0.26605E+00_JPRB, 0.26314E+00_JPRB, 0.25879E+00_JPRB, 0.25538E+00_JPRB, 0.25497E+00_JPRB, 0.25596E+00_JPRB, 0.25812E+00_JPRB ,&
&  0.25723E+00_JPRB, 0.25185E+00_JPRB, 0.24507E+00_JPRB, 0.24144E+00_JPRB, 0.24025E+00_JPRB, 0.23885E+00_JPRB, 0.23632E+00_JPRB ,&
&  0.23425E+00_JPRB, 0.23382E+00_JPRB, 0.23391E+00_JPRB, 0.23319E+00_JPRB, 0.23181E+00_JPRB, 0.22947E+00_JPRB, 0.22639E+00_JPRB ,&
&  0.22376E+00_JPRB, 0.22105E+00_JPRB, 0.21938E+00_JPRB, 0.21811E+00_JPRB, 0.21804E+00_JPRB, 0.21839E+00_JPRB, 0.21781E+00_JPRB ,&
&  0.21624E+00_JPRB, 0.21529E+00_JPRB, 0.21393E+00_JPRB, 0.21106E+00_JPRB, 0.20898E+00_JPRB, 0.20836E+00_JPRB, 0.20731E+00_JPRB ,&
&  0.20580E+00_JPRB, 0.20514E+00_JPRB, 0.20478E+00_JPRB, 0.20415E+00_JPRB, 0.20479E+00_JPRB, 0.20630E+00_JPRB, 0.20385E+00_JPRB ,&
&  0.20182E+00_JPRB, 0.20047E+00_JPRB, 0.20044E+00_JPRB, 0.20169E+00_JPRB, 0.20387E+00_JPRB, 0.20631E+00_JPRB, 0.20839E+00_JPRB ,&
&  0.20928E+00_JPRB, 0.20952E+00_JPRB, 0.20986E+00_JPRB, 0.20987E+00_JPRB, 0.21065E+00_JPRB, 0.21247E+00_JPRB, 0.21480E+00_JPRB ,&
&  0.21604E+00_JPRB, 0.21840E+00_JPRB, 0.22080E+00_JPRB, 0.22381E+00_JPRB, 0.22546E+00_JPRB, 0.22700E+00_JPRB, 0.22858E+00_JPRB ,&
&  0.22986E+00_JPRB, 0.23148E+00_JPRB, 0.23381E+00_JPRB, 0.23582E+00_JPRB, 0.23801E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.25141E+00_JPRB, 0.25071E+00_JPRB, 0.25052E+00_JPRB, 0.25117E+00_JPRB, 0.25328E+00_JPRB, 0.25388E+00_JPRB, 0.25378E+00_JPRB ,&
&  0.25341E+00_JPRB, 0.25499E+00_JPRB, 0.25819E+00_JPRB, 0.26010E+00_JPRB, 0.26022E+00_JPRB, 0.25998E+00_JPRB, 0.25974E+00_JPRB ,&
&  0.25988E+00_JPRB, 0.26001E+00_JPRB, 0.25920E+00_JPRB, 0.25935E+00_JPRB, 0.25975E+00_JPRB, 0.25936E+00_JPRB, 0.26052E+00_JPRB ,&
&  0.26085E+00_JPRB, 0.26135E+00_JPRB, 0.26353E+00_JPRB, 0.26522E+00_JPRB, 0.26851E+00_JPRB, 0.27329E+00_JPRB, 0.27491E+00_JPRB ,&
&  0.27346E+00_JPRB, 0.26976E+00_JPRB, 0.26693E+00_JPRB, 0.26763E+00_JPRB, 0.26910E+00_JPRB, 0.27195E+00_JPRB, 0.27472E+00_JPRB ,&
&  0.27467E+00_JPRB, 0.26842E+00_JPRB, 0.25981E+00_JPRB, 0.25447E+00_JPRB, 0.25164E+00_JPRB, 0.25015E+00_JPRB, 0.24899E+00_JPRB ,&
&  0.24707E+00_JPRB, 0.24555E+00_JPRB, 0.24487E+00_JPRB, 0.24489E+00_JPRB, 0.24481E+00_JPRB, 0.24248E+00_JPRB, 0.23988E+00_JPRB ,&
&  0.23631E+00_JPRB, 0.23385E+00_JPRB, 0.23178E+00_JPRB, 0.23110E+00_JPRB, 0.23127E+00_JPRB, 0.23117E+00_JPRB, 0.23036E+00_JPRB ,&
&  0.22926E+00_JPRB, 0.22898E+00_JPRB, 0.22874E+00_JPRB, 0.22777E+00_JPRB, 0.22657E+00_JPRB, 0.22634E+00_JPRB, 0.22536E+00_JPRB ,&
&  0.22315E+00_JPRB, 0.22169E+00_JPRB, 0.22146E+00_JPRB, 0.22215E+00_JPRB, 0.22199E+00_JPRB, 0.22262E+00_JPRB, 0.22399E+00_JPRB ,&
&  0.22306E+00_JPRB, 0.22069E+00_JPRB, 0.21953E+00_JPRB, 0.22008E+00_JPRB, 0.22176E+00_JPRB, 0.22396E+00_JPRB, 0.22501E+00_JPRB ,&
&  0.22596E+00_JPRB, 0.22588E+00_JPRB, 0.22671E+00_JPRB, 0.22791E+00_JPRB, 0.22922E+00_JPRB, 0.22982E+00_JPRB, 0.23002E+00_JPRB ,&
&  0.23037E+00_JPRB, 0.23080E+00_JPRB, 0.23256E+00_JPRB, 0.23544E+00_JPRB, 0.23797E+00_JPRB, 0.24086E+00_JPRB, 0.24309E+00_JPRB ,&
&  0.24456E+00_JPRB, 0.24615E+00_JPRB, 0.24728E+00_JPRB, 0.24826E+00_JPRB, 0.24974E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.26978E+00_JPRB, 0.26791E+00_JPRB, 0.26675E+00_JPRB, 0.26722E+00_JPRB, 0.26946E+00_JPRB, 0.27006E+00_JPRB, 0.26881E+00_JPRB ,&
&  0.26747E+00_JPRB, 0.26864E+00_JPRB, 0.27133E+00_JPRB, 0.27250E+00_JPRB, 0.27279E+00_JPRB, 0.27341E+00_JPRB, 0.27253E+00_JPRB ,&
&  0.26976E+00_JPRB, 0.26678E+00_JPRB, 0.26723E+00_JPRB, 0.26841E+00_JPRB, 0.26963E+00_JPRB, 0.26871E+00_JPRB, 0.26749E+00_JPRB ,&
&  0.26833E+00_JPRB, 0.26845E+00_JPRB, 0.27017E+00_JPRB, 0.27218E+00_JPRB, 0.27529E+00_JPRB, 0.27968E+00_JPRB, 0.28269E+00_JPRB ,&
&  0.28283E+00_JPRB, 0.28109E+00_JPRB, 0.28076E+00_JPRB, 0.28258E+00_JPRB, 0.28440E+00_JPRB, 0.28894E+00_JPRB, 0.29298E+00_JPRB ,&
&  0.29353E+00_JPRB, 0.28978E+00_JPRB, 0.28028E+00_JPRB, 0.27324E+00_JPRB, 0.26978E+00_JPRB, 0.26663E+00_JPRB, 0.26362E+00_JPRB ,&
&  0.25998E+00_JPRB, 0.25717E+00_JPRB, 0.25550E+00_JPRB, 0.25481E+00_JPRB, 0.25466E+00_JPRB, 0.25426E+00_JPRB, 0.25246E+00_JPRB ,&
&  0.25008E+00_JPRB, 0.24826E+00_JPRB, 0.24682E+00_JPRB, 0.24568E+00_JPRB, 0.24460E+00_JPRB, 0.24416E+00_JPRB, 0.24366E+00_JPRB ,&
&  0.24345E+00_JPRB, 0.24395E+00_JPRB, 0.24485E+00_JPRB, 0.24505E+00_JPRB, 0.24508E+00_JPRB, 0.24454E+00_JPRB, 0.24378E+00_JPRB ,&
&  0.24234E+00_JPRB, 0.23979E+00_JPRB, 0.23760E+00_JPRB, 0.23679E+00_JPRB, 0.23630E+00_JPRB, 0.23651E+00_JPRB, 0.23689E+00_JPRB ,&
&  0.23665E+00_JPRB, 0.23656E+00_JPRB, 0.23736E+00_JPRB, 0.23851E+00_JPRB, 0.23941E+00_JPRB, 0.23979E+00_JPRB, 0.24033E+00_JPRB ,&
&  0.24178E+00_JPRB, 0.24347E+00_JPRB, 0.24462E+00_JPRB, 0.24538E+00_JPRB, 0.24574E+00_JPRB, 0.24666E+00_JPRB, 0.24656E+00_JPRB ,&
&  0.24662E+00_JPRB, 0.24753E+00_JPRB, 0.25009E+00_JPRB, 0.25387E+00_JPRB, 0.25755E+00_JPRB, 0.26001E+00_JPRB, 0.26250E+00_JPRB ,&
&  0.26452E+00_JPRB, 0.26617E+00_JPRB, 0.26715E+00_JPRB, 0.26783E+00_JPRB, 0.26905E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.28874E+00_JPRB, 0.28731E+00_JPRB, 0.28639E+00_JPRB, 0.28533E+00_JPRB, 0.28609E+00_JPRB, 0.28559E+00_JPRB, 0.28349E+00_JPRB ,&
&  0.28196E+00_JPRB, 0.28324E+00_JPRB, 0.28554E+00_JPRB, 0.28633E+00_JPRB, 0.28555E+00_JPRB, 0.28426E+00_JPRB, 0.28172E+00_JPRB ,&
&  0.27819E+00_JPRB, 0.27644E+00_JPRB, 0.27614E+00_JPRB, 0.27668E+00_JPRB, 0.27676E+00_JPRB, 0.27555E+00_JPRB, 0.27478E+00_JPRB ,&
&  0.27613E+00_JPRB, 0.27761E+00_JPRB, 0.27788E+00_JPRB, 0.28014E+00_JPRB, 0.28337E+00_JPRB, 0.28880E+00_JPRB, 0.29398E+00_JPRB ,&
&  0.29506E+00_JPRB, 0.29593E+00_JPRB, 0.28999E+00_JPRB, 0.29903E+00_JPRB, 0.30319E+00_JPRB, 0.30851E+00_JPRB, 0.31035E+00_JPRB ,&
&  0.31029E+00_JPRB, 0.30633E+00_JPRB, 0.29707E+00_JPRB, 0.29021E+00_JPRB, 0.28662E+00_JPRB, 0.28394E+00_JPRB, 0.27957E+00_JPRB ,&
&  0.27484E+00_JPRB, 0.27043E+00_JPRB, 0.26787E+00_JPRB, 0.26679E+00_JPRB, 0.26734E+00_JPRB, 0.26810E+00_JPRB, 0.26674E+00_JPRB ,&
&  0.26528E+00_JPRB, 0.26379E+00_JPRB, 0.26328E+00_JPRB, 0.26262E+00_JPRB, 0.26154E+00_JPRB, 0.26072E+00_JPRB, 0.26062E+00_JPRB ,&
&  0.26086E+00_JPRB, 0.26103E+00_JPRB, 0.26146E+00_JPRB, 0.26241E+00_JPRB, 0.26298E+00_JPRB, 0.26234E+00_JPRB, 0.26065E+00_JPRB ,&
&  0.25864E+00_JPRB, 0.25675E+00_JPRB, 0.25496E+00_JPRB, 0.25321E+00_JPRB, 0.25174E+00_JPRB, 0.25077E+00_JPRB, 0.25005E+00_JPRB ,&
&  0.24926E+00_JPRB, 0.24963E+00_JPRB, 0.25101E+00_JPRB, 0.25179E+00_JPRB, 0.25210E+00_JPRB, 0.25297E+00_JPRB, 0.25465E+00_JPRB ,&
&  0.25633E+00_JPRB, 0.25817E+00_JPRB, 0.25943E+00_JPRB, 0.25976E+00_JPRB, 0.25895E+00_JPRB, 0.25864E+00_JPRB, 0.25904E+00_JPRB ,&
&  0.26047E+00_JPRB, 0.26294E+00_JPRB, 0.26672E+00_JPRB, 0.27180E+00_JPRB, 0.27715E+00_JPRB, 0.28090E+00_JPRB, 0.28440E+00_JPRB ,&
&  0.28719E+00_JPRB, 0.28797E+00_JPRB, 0.28792E+00_JPRB, 0.28902E+00_JPRB, 0.28968E+00_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.30798E+00_JPRB, 0.30664E+00_JPRB, 0.30571E+00_JPRB, 0.30327E+00_JPRB, 0.30186E+00_JPRB, 0.30112E+00_JPRB, 0.29986E+00_JPRB ,&
&  0.29899E+00_JPRB, 0.29978E+00_JPRB, 0.29905E+00_JPRB, 0.29873E+00_JPRB, 0.29814E+00_JPRB, 0.29583E+00_JPRB, 0.29279E+00_JPRB ,&
&  0.29111E+00_JPRB, 0.28949E+00_JPRB, 0.28804E+00_JPRB, 0.28650E+00_JPRB, 0.28520E+00_JPRB, 0.28329E+00_JPRB, 0.28372E+00_JPRB ,&
&  0.28530E+00_JPRB, 0.28816E+00_JPRB, 0.29099E+00_JPRB, 0.29433E+00_JPRB, 0.29873E+00_JPRB, 0.30448E+00_JPRB, 0.30994E+00_JPRB ,&
&  0.31550E+00_JPRB, 0.30500E+00_JPRB, 0.31434E+00_JPRB, 0.31932E+00_JPRB, 0.32745E+00_JPRB, 0.33283E+00_JPRB, 0.33148E+00_JPRB ,&
&  0.32758E+00_JPRB, 0.31832E+00_JPRB, 0.30898E+00_JPRB, 0.30197E+00_JPRB, 0.29919E+00_JPRB, 0.29790E+00_JPRB, 0.29515E+00_JPRB ,&
&  0.29211E+00_JPRB, 0.28818E+00_JPRB, 0.28501E+00_JPRB, 0.28323E+00_JPRB, 0.28282E+00_JPRB, 0.28305E+00_JPRB, 0.28220E+00_JPRB ,&
&  0.28136E+00_JPRB, 0.28039E+00_JPRB, 0.28000E+00_JPRB, 0.27986E+00_JPRB, 0.27963E+00_JPRB, 0.28004E+00_JPRB, 0.28094E+00_JPRB ,&
&  0.28173E+00_JPRB, 0.28225E+00_JPRB, 0.28270E+00_JPRB, 0.28349E+00_JPRB, 0.28364E+00_JPRB, 0.28279E+00_JPRB, 0.28134E+00_JPRB ,&
&  0.27930E+00_JPRB, 0.27743E+00_JPRB, 0.27514E+00_JPRB, 0.27352E+00_JPRB, 0.27174E+00_JPRB, 0.26965E+00_JPRB, 0.26784E+00_JPRB ,&
&  0.26695E+00_JPRB, 0.26700E+00_JPRB, 0.26738E+00_JPRB, 0.26782E+00_JPRB, 0.26850E+00_JPRB, 0.26963E+00_JPRB, 0.27092E+00_JPRB ,&
&  0.27227E+00_JPRB, 0.27342E+00_JPRB, 0.27369E+00_JPRB, 0.27355E+00_JPRB, 0.27277E+00_JPRB, 0.27266E+00_JPRB, 0.27379E+00_JPRB ,&
&  0.27684E+00_JPRB, 0.28082E+00_JPRB, 0.28656E+00_JPRB, 0.29537E+00_JPRB, 0.30316E+00_JPRB, 0.30795E+00_JPRB, 0.31155E+00_JPRB ,&
&  0.31210E+00_JPRB, 0.31027E+00_JPRB, 0.30849E+00_JPRB, 0.30883E+00_JPRB, 0.30926E+00_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.32173E+00_JPRB, 0.32095E+00_JPRB, 0.32026E+00_JPRB, 0.31890E+00_JPRB, 0.31779E+00_JPRB, 0.31707E+00_JPRB, 0.31491E+00_JPRB ,&
&  0.31477E+00_JPRB, 0.31246E+00_JPRB, 0.31006E+00_JPRB, 0.30944E+00_JPRB, 0.30832E+00_JPRB, 0.30629E+00_JPRB, 0.30454E+00_JPRB ,&
&  0.30248E+00_JPRB, 0.30033E+00_JPRB, 0.29915E+00_JPRB, 0.29739E+00_JPRB, 0.29460E+00_JPRB, 0.29255E+00_JPRB, 0.29326E+00_JPRB ,&
&  0.29445E+00_JPRB, 0.29716E+00_JPRB, 0.30420E+00_JPRB, 0.31136E+00_JPRB, 0.31810E+00_JPRB, 0.32397E+00_JPRB, 0.33012E+00_JPRB ,&
&  0.33842E+00_JPRB, 0.30272E+00_JPRB, 0.31801E+00_JPRB, 0.32733E+00_JPRB, 0.34095E+00_JPRB, 0.35269E+00_JPRB, 0.35076E+00_JPRB ,&
&  0.34239E+00_JPRB, 0.33127E+00_JPRB, 0.32075E+00_JPRB, 0.31296E+00_JPRB, 0.31077E+00_JPRB, 0.31001E+00_JPRB, 0.30861E+00_JPRB ,&
&  0.30749E+00_JPRB, 0.30448E+00_JPRB, 0.30108E+00_JPRB, 0.29870E+00_JPRB, 0.29741E+00_JPRB, 0.29643E+00_JPRB, 0.29529E+00_JPRB ,&
&  0.29486E+00_JPRB, 0.29513E+00_JPRB, 0.29542E+00_JPRB, 0.29561E+00_JPRB, 0.29637E+00_JPRB, 0.29800E+00_JPRB, 0.29993E+00_JPRB ,&
&  0.30175E+00_JPRB, 0.30273E+00_JPRB, 0.30377E+00_JPRB, 0.30432E+00_JPRB, 0.30471E+00_JPRB, 0.30480E+00_JPRB, 0.30416E+00_JPRB ,&
&  0.30235E+00_JPRB, 0.30008E+00_JPRB, 0.29789E+00_JPRB, 0.29578E+00_JPRB, 0.29342E+00_JPRB, 0.29063E+00_JPRB, 0.28844E+00_JPRB ,&
&  0.28662E+00_JPRB, 0.28524E+00_JPRB, 0.28271E+00_JPRB, 0.28229E+00_JPRB, 0.28310E+00_JPRB, 0.28394E+00_JPRB, 0.28492E+00_JPRB ,&
&  0.28574E+00_JPRB, 0.28559E+00_JPRB, 0.28492E+00_JPRB, 0.28431E+00_JPRB, 0.28399E+00_JPRB, 0.28444E+00_JPRB, 0.28599E+00_JPRB ,&
&  0.28885E+00_JPRB, 0.29404E+00_JPRB, 0.30111E+00_JPRB, 0.31135E+00_JPRB, 0.32001E+00_JPRB, 0.32507E+00_JPRB, 0.32771E+00_JPRB ,&
&  0.32639E+00_JPRB, 0.32349E+00_JPRB, 0.32126E+00_JPRB, 0.32006E+00_JPRB, 0.32130E+00_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.34171E+00_JPRB, 0.33975E+00_JPRB, 0.33870E+00_JPRB, 0.33671E+00_JPRB, 0.33482E+00_JPRB, 0.33287E+00_JPRB, 0.33055E+00_JPRB ,&
&  0.32760E+00_JPRB, 0.32592E+00_JPRB, 0.32420E+00_JPRB, 0.32176E+00_JPRB, 0.31920E+00_JPRB, 0.31701E+00_JPRB, 0.31465E+00_JPRB ,&
&  0.31213E+00_JPRB, 0.30949E+00_JPRB, 0.30796E+00_JPRB, 0.30704E+00_JPRB, 0.30574E+00_JPRB, 0.30546E+00_JPRB, 0.30664E+00_JPRB ,&
&  0.30709E+00_JPRB, 0.30954E+00_JPRB, 0.31862E+00_JPRB, 0.32858E+00_JPRB, 0.33890E+00_JPRB, 0.34701E+00_JPRB, 0.35928E+00_JPRB ,&
&  0.34435E+00_JPRB, 0.30734E+00_JPRB, 0.32407E+00_JPRB, 0.34119E+00_JPRB, 0.35777E+00_JPRB, 0.37272E+00_JPRB, 0.37137E+00_JPRB ,&
&  0.36766E+00_JPRB, 0.35933E+00_JPRB, 0.34937E+00_JPRB, 0.34082E+00_JPRB, 0.33699E+00_JPRB, 0.33387E+00_JPRB, 0.33066E+00_JPRB ,&
&  0.32753E+00_JPRB, 0.32318E+00_JPRB, 0.31927E+00_JPRB, 0.31589E+00_JPRB, 0.31376E+00_JPRB, 0.31189E+00_JPRB, 0.31014E+00_JPRB ,&
&  0.30975E+00_JPRB, 0.31015E+00_JPRB, 0.31062E+00_JPRB, 0.31116E+00_JPRB, 0.31218E+00_JPRB, 0.31473E+00_JPRB, 0.31830E+00_JPRB ,&
&  0.32237E+00_JPRB, 0.32682E+00_JPRB, 0.33128E+00_JPRB, 0.33474E+00_JPRB, 0.33732E+00_JPRB, 0.33810E+00_JPRB, 0.33630E+00_JPRB ,&
&  0.33358E+00_JPRB, 0.32979E+00_JPRB, 0.32581E+00_JPRB, 0.32130E+00_JPRB, 0.31593E+00_JPRB, 0.31093E+00_JPRB, 0.30700E+00_JPRB ,&
&  0.30405E+00_JPRB, 0.30114E+00_JPRB, 0.29891E+00_JPRB, 0.29848E+00_JPRB, 0.29919E+00_JPRB, 0.29964E+00_JPRB, 0.29962E+00_JPRB ,&
&  0.29934E+00_JPRB, 0.29862E+00_JPRB, 0.29819E+00_JPRB, 0.29848E+00_JPRB, 0.29943E+00_JPRB, 0.30152E+00_JPRB, 0.30401E+00_JPRB ,&
&  0.30646E+00_JPRB, 0.31162E+00_JPRB, 0.31880E+00_JPRB, 0.33119E+00_JPRB, 0.33991E+00_JPRB, 0.34415E+00_JPRB, 0.34520E+00_JPRB ,&
&  0.34376E+00_JPRB, 0.34151E+00_JPRB, 0.34116E+00_JPRB, 0.34036E+00_JPRB, 0.34123E+00_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.36459E+00_JPRB, 0.36445E+00_JPRB, 0.36095E+00_JPRB, 0.35373E+00_JPRB, 0.34835E+00_JPRB, 0.34459E+00_JPRB, 0.34225E+00_JPRB ,&
&  0.34112E+00_JPRB, 0.34017E+00_JPRB, 0.33851E+00_JPRB, 0.33481E+00_JPRB, 0.33171E+00_JPRB, 0.32755E+00_JPRB, 0.32316E+00_JPRB ,&
&  0.31960E+00_JPRB, 0.31637E+00_JPRB, 0.31478E+00_JPRB, 0.31242E+00_JPRB, 0.31136E+00_JPRB, 0.31269E+00_JPRB, 0.31478E+00_JPRB ,&
&  0.31752E+00_JPRB, 0.32028E+00_JPRB, 0.32868E+00_JPRB, 0.33793E+00_JPRB, 0.35377E+00_JPRB, 0.36417E+00_JPRB, 0.37684E+00_JPRB ,&
&  0.40566E+00_JPRB, 0.33838E+00_JPRB, 0.35417E+00_JPRB, 0.37535E+00_JPRB, 0.38501E+00_JPRB, 0.39829E+00_JPRB, 0.39571E+00_JPRB ,&
&  0.39390E+00_JPRB, 0.39119E+00_JPRB, 0.38386E+00_JPRB, 0.37620E+00_JPRB, 0.37147E+00_JPRB, 0.36402E+00_JPRB, 0.35539E+00_JPRB ,&
&  0.34550E+00_JPRB, 0.33866E+00_JPRB, 0.33350E+00_JPRB, 0.32893E+00_JPRB, 0.32529E+00_JPRB, 0.32321E+00_JPRB, 0.32241E+00_JPRB ,&
&  0.32282E+00_JPRB, 0.32327E+00_JPRB, 0.32332E+00_JPRB, 0.32395E+00_JPRB, 0.32568E+00_JPRB, 0.32949E+00_JPRB, 0.33545E+00_JPRB ,&
&  0.34494E+00_JPRB, 0.35535E+00_JPRB, 0.36387E+00_JPRB, 0.36915E+00_JPRB, 0.37178E+00_JPRB, 0.36999E+00_JPRB, 0.36568E+00_JPRB ,&
&  0.35924E+00_JPRB, 0.35136E+00_JPRB, 0.34444E+00_JPRB, 0.33833E+00_JPRB, 0.33289E+00_JPRB, 0.32796E+00_JPRB, 0.32271E+00_JPRB ,&
&  0.31889E+00_JPRB, 0.31673E+00_JPRB, 0.31643E+00_JPRB, 0.31649E+00_JPRB, 0.31622E+00_JPRB, 0.31586E+00_JPRB, 0.31605E+00_JPRB ,&
&  0.31589E+00_JPRB, 0.31574E+00_JPRB, 0.31595E+00_JPRB, 0.31722E+00_JPRB, 0.31941E+00_JPRB, 0.32148E+00_JPRB, 0.32322E+00_JPRB ,&
&  0.32506E+00_JPRB, 0.32943E+00_JPRB, 0.33478E+00_JPRB, 0.34873E+00_JPRB, 0.35402E+00_JPRB, 0.35664E+00_JPRB, 0.35892E+00_JPRB ,&
&  0.36052E+00_JPRB, 0.36041E+00_JPRB, 0.32416E+00_JPRB, 0.34771E+00_JPRB, 0.35917E+00_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.39482E+00_JPRB, 0.39143E+00_JPRB, 0.38258E+00_JPRB, 0.37312E+00_JPRB, 0.36768E+00_JPRB, 0.36475E+00_JPRB, 0.36299E+00_JPRB ,&
&  0.36053E+00_JPRB, 0.35723E+00_JPRB, 0.35358E+00_JPRB, 0.35149E+00_JPRB, 0.34870E+00_JPRB, 0.34449E+00_JPRB, 0.33927E+00_JPRB ,&
&  0.33448E+00_JPRB, 0.32992E+00_JPRB, 0.32654E+00_JPRB, 0.32252E+00_JPRB, 0.31931E+00_JPRB, 0.31984E+00_JPRB, 0.32173E+00_JPRB ,&
&  0.32715E+00_JPRB, 0.33467E+00_JPRB, 0.34169E+00_JPRB, 0.35509E+00_JPRB, 0.36952E+00_JPRB, 0.38808E+00_JPRB, 0.40441E+00_JPRB ,&
&  0.43400E+00_JPRB, 0.36649E+00_JPRB, 0.37538E+00_JPRB, 0.40452E+00_JPRB, 0.41574E+00_JPRB, 0.42838E+00_JPRB, 0.43096E+00_JPRB ,&
&  0.43247E+00_JPRB, 0.43422E+00_JPRB, 0.43131E+00_JPRB, 0.42421E+00_JPRB, 0.41210E+00_JPRB, 0.39192E+00_JPRB, 0.37263E+00_JPRB ,&
&  0.35854E+00_JPRB, 0.35383E+00_JPRB, 0.35182E+00_JPRB, 0.34916E+00_JPRB, 0.34671E+00_JPRB, 0.34623E+00_JPRB, 0.34778E+00_JPRB ,&
&  0.34875E+00_JPRB, 0.34906E+00_JPRB, 0.34865E+00_JPRB, 0.34820E+00_JPRB, 0.34958E+00_JPRB, 0.35721E+00_JPRB, 0.37166E+00_JPRB ,&
&  0.39027E+00_JPRB, 0.40623E+00_JPRB, 0.41653E+00_JPRB, 0.42082E+00_JPRB, 0.41826E+00_JPRB, 0.40979E+00_JPRB, 0.39595E+00_JPRB ,&
&  0.38095E+00_JPRB, 0.36885E+00_JPRB, 0.36123E+00_JPRB, 0.35542E+00_JPRB, 0.35069E+00_JPRB, 0.34667E+00_JPRB, 0.34433E+00_JPRB ,&
&  0.34364E+00_JPRB, 0.34362E+00_JPRB, 0.34329E+00_JPRB, 0.34176E+00_JPRB, 0.34101E+00_JPRB, 0.34059E+00_JPRB, 0.34085E+00_JPRB ,&
&  0.34115E+00_JPRB, 0.34137E+00_JPRB, 0.34247E+00_JPRB, 0.34401E+00_JPRB, 0.34501E+00_JPRB, 0.34518E+00_JPRB, 0.34589E+00_JPRB ,&
&  0.34941E+00_JPRB, 0.35548E+00_JPRB, 0.36539E+00_JPRB, 0.37907E+00_JPRB, 0.38896E+00_JPRB, 0.38474E+00_JPRB, 0.38159E+00_JPRB ,&
&  0.38152E+00_JPRB, 0.38099E+00_JPRB, 0.38463E+00_JPRB, 0.36479E+00_JPRB, 0.39306E+00_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.41851E+00_JPRB, 0.41856E+00_JPRB, 0.40954E+00_JPRB, 0.39837E+00_JPRB, 0.38810E+00_JPRB, 0.38190E+00_JPRB, 0.37710E+00_JPRB ,&
&  0.37428E+00_JPRB, 0.37261E+00_JPRB, 0.37136E+00_JPRB, 0.36964E+00_JPRB, 0.36672E+00_JPRB, 0.36197E+00_JPRB, 0.35571E+00_JPRB ,&
&  0.35014E+00_JPRB, 0.34450E+00_JPRB, 0.33998E+00_JPRB, 0.33468E+00_JPRB, 0.32798E+00_JPRB, 0.32506E+00_JPRB, 0.32798E+00_JPRB ,&
&  0.33371E+00_JPRB, 0.34085E+00_JPRB, 0.34761E+00_JPRB, 0.35792E+00_JPRB, 0.37844E+00_JPRB, 0.39629E+00_JPRB, 0.41243E+00_JPRB ,&
&  0.44597E+00_JPRB, 0.39759E+00_JPRB, 0.41180E+00_JPRB, 0.42849E+00_JPRB, 0.43900E+00_JPRB, 0.44847E+00_JPRB, 0.46391E+00_JPRB ,&
&  0.47887E+00_JPRB, 0.48714E+00_JPRB, 0.48758E+00_JPRB, 0.47189E+00_JPRB, 0.44328E+00_JPRB, 0.41895E+00_JPRB, 0.40453E+00_JPRB ,&
&  0.39693E+00_JPRB, 0.39381E+00_JPRB, 0.39091E+00_JPRB, 0.38878E+00_JPRB, 0.38799E+00_JPRB, 0.38733E+00_JPRB, 0.38649E+00_JPRB ,&
&  0.38559E+00_JPRB, 0.38493E+00_JPRB, 0.38483E+00_JPRB, 0.38605E+00_JPRB, 0.39141E+00_JPRB, 0.40331E+00_JPRB, 0.41847E+00_JPRB ,&
&  0.43849E+00_JPRB, 0.45195E+00_JPRB, 0.45499E+00_JPRB, 0.45540E+00_JPRB, 0.45349E+00_JPRB, 0.44325E+00_JPRB, 0.42287E+00_JPRB ,&
&  0.40465E+00_JPRB, 0.38986E+00_JPRB, 0.37985E+00_JPRB, 0.37336E+00_JPRB, 0.36946E+00_JPRB, 0.36795E+00_JPRB, 0.36862E+00_JPRB ,&
&  0.36958E+00_JPRB, 0.36945E+00_JPRB, 0.36979E+00_JPRB, 0.36955E+00_JPRB, 0.36993E+00_JPRB, 0.36964E+00_JPRB, 0.36752E+00_JPRB ,&
&  0.36474E+00_JPRB, 0.36345E+00_JPRB, 0.36264E+00_JPRB, 0.36204E+00_JPRB, 0.36204E+00_JPRB, 0.36105E+00_JPRB, 0.36088E+00_JPRB ,&
&  0.36352E+00_JPRB, 0.37500E+00_JPRB, 0.39098E+00_JPRB, 0.41998E+00_JPRB, 0.41634E+00_JPRB, 0.40745E+00_JPRB, 0.39829E+00_JPRB ,&
&  0.39512E+00_JPRB, 0.39768E+00_JPRB, 0.39529E+00_JPRB, 0.40237E+00_JPRB, 0.41765E+00_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.47501E+00_JPRB, 0.46218E+00_JPRB, 0.44713E+00_JPRB, 0.42914E+00_JPRB, 0.41307E+00_JPRB, 0.40155E+00_JPRB, 0.39338E+00_JPRB ,&
&  0.38855E+00_JPRB, 0.38594E+00_JPRB, 0.38437E+00_JPRB, 0.38319E+00_JPRB, 0.38122E+00_JPRB, 0.37899E+00_JPRB, 0.37326E+00_JPRB ,&
&  0.36638E+00_JPRB, 0.35861E+00_JPRB, 0.35271E+00_JPRB, 0.34747E+00_JPRB, 0.33756E+00_JPRB, 0.33242E+00_JPRB, 0.33493E+00_JPRB ,&
&  0.34011E+00_JPRB, 0.34659E+00_JPRB, 0.35167E+00_JPRB, 0.36249E+00_JPRB, 0.38114E+00_JPRB, 0.39756E+00_JPRB, 0.41781E+00_JPRB ,&
&  0.45765E+00_JPRB, 0.41996E+00_JPRB, 0.45060E+00_JPRB, 0.45862E+00_JPRB, 0.45932E+00_JPRB, 0.48304E+00_JPRB, 0.51726E+00_JPRB ,&
&  0.54409E+00_JPRB, 0.57593E+00_JPRB, 0.56984E+00_JPRB, 0.53451E+00_JPRB, 0.50084E+00_JPRB, 0.47548E+00_JPRB, 0.46003E+00_JPRB ,&
&  0.44764E+00_JPRB, 0.43715E+00_JPRB, 0.42795E+00_JPRB, 0.42299E+00_JPRB, 0.41797E+00_JPRB, 0.41305E+00_JPRB, 0.40810E+00_JPRB ,&
&  0.40607E+00_JPRB, 0.40512E+00_JPRB, 0.40543E+00_JPRB, 0.41200E+00_JPRB, 0.42565E+00_JPRB, 0.44474E+00_JPRB, 0.46615E+00_JPRB ,&
&  0.48427E+00_JPRB, 0.49075E+00_JPRB, 0.49548E+00_JPRB, 0.48926E+00_JPRB, 0.47563E+00_JPRB, 0.46146E+00_JPRB, 0.44821E+00_JPRB ,&
&  0.43319E+00_JPRB, 0.41674E+00_JPRB, 0.40541E+00_JPRB, 0.39835E+00_JPRB, 0.39466E+00_JPRB, 0.39375E+00_JPRB, 0.39417E+00_JPRB ,&
&  0.39355E+00_JPRB, 0.39397E+00_JPRB, 0.39761E+00_JPRB, 0.40220E+00_JPRB, 0.40246E+00_JPRB, 0.39889E+00_JPRB, 0.39269E+00_JPRB ,&
&  0.38744E+00_JPRB, 0.38300E+00_JPRB, 0.38199E+00_JPRB, 0.38277E+00_JPRB, 0.38441E+00_JPRB, 0.38703E+00_JPRB, 0.38850E+00_JPRB ,&
&  0.39287E+00_JPRB, 0.41059E+00_JPRB, 0.41819E+00_JPRB, 0.44068E+00_JPRB, 0.46705E+00_JPRB, 0.45510E+00_JPRB, 0.43887E+00_JPRB ,&
&  0.42749E+00_JPRB, 0.42358E+00_JPRB, 0.41748E+00_JPRB, 0.42482E+00_JPRB, 0.49026E+00_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.45311E+00_JPRB, 0.44574E+00_JPRB, 0.43747E+00_JPRB, 0.43020E+00_JPRB, 0.41956E+00_JPRB, 0.40923E+00_JPRB, 0.39903E+00_JPRB ,&
&  0.39174E+00_JPRB, 0.38833E+00_JPRB, 0.38915E+00_JPRB, 0.39054E+00_JPRB, 0.38918E+00_JPRB, 0.38331E+00_JPRB, 0.37199E+00_JPRB ,&
&  0.36475E+00_JPRB, 0.35962E+00_JPRB, 0.35384E+00_JPRB, 0.34653E+00_JPRB, 0.33937E+00_JPRB, 0.33708E+00_JPRB, 0.33851E+00_JPRB ,&
&  0.34343E+00_JPRB, 0.34983E+00_JPRB, 0.35441E+00_JPRB, 0.36618E+00_JPRB, 0.38143E+00_JPRB, 0.39822E+00_JPRB, 0.42113E+00_JPRB ,&
&  0.45316E+00_JPRB, 0.48584E+00_JPRB, 0.51362E+00_JPRB, 0.52021E+00_JPRB, 0.51529E+00_JPRB, 0.53831E+00_JPRB, 0.59199E+00_JPRB ,&
&  0.64506E+00_JPRB, 0.67809E+00_JPRB, 0.66551E+00_JPRB, 0.61287E+00_JPRB, 0.57048E+00_JPRB, 0.54045E+00_JPRB, 0.51046E+00_JPRB ,&
&  0.48109E+00_JPRB, 0.45827E+00_JPRB, 0.44033E+00_JPRB, 0.42804E+00_JPRB, 0.41875E+00_JPRB, 0.41230E+00_JPRB, 0.40941E+00_JPRB ,&
&  0.41039E+00_JPRB, 0.41292E+00_JPRB, 0.41969E+00_JPRB, 0.43192E+00_JPRB, 0.46130E+00_JPRB, 0.49181E+00_JPRB, 0.51681E+00_JPRB ,&
&  0.52820E+00_JPRB, 0.53641E+00_JPRB, 0.54048E+00_JPRB, 0.52880E+00_JPRB, 0.50845E+00_JPRB, 0.48370E+00_JPRB, 0.46389E+00_JPRB ,&
&  0.44994E+00_JPRB, 0.43715E+00_JPRB, 0.42630E+00_JPRB, 0.41635E+00_JPRB, 0.41056E+00_JPRB, 0.40897E+00_JPRB, 0.40958E+00_JPRB ,&
&  0.40894E+00_JPRB, 0.41022E+00_JPRB, 0.41251E+00_JPRB, 0.41385E+00_JPRB, 0.41318E+00_JPRB, 0.41143E+00_JPRB, 0.40819E+00_JPRB ,&
&  0.40416E+00_JPRB, 0.40142E+00_JPRB, 0.40625E+00_JPRB, 0.40996E+00_JPRB, 0.41352E+00_JPRB, 0.41407E+00_JPRB, 0.41231E+00_JPRB ,&
&  0.41529E+00_JPRB, 0.42804E+00_JPRB, 0.43427E+00_JPRB, 0.44774E+00_JPRB, 0.45835E+00_JPRB, 0.47183E+00_JPRB, 0.45742E+00_JPRB ,&
&  0.44915E+00_JPRB, 0.43705E+00_JPRB, 0.42690E+00_JPRB, 0.42887E+00_JPRB, 0.45520E+00_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.44262E+00_JPRB, 0.43836E+00_JPRB, 0.43148E+00_JPRB, 0.42443E+00_JPRB, 0.41566E+00_JPRB, 0.40567E+00_JPRB, 0.39847E+00_JPRB ,&
&  0.39473E+00_JPRB, 0.39535E+00_JPRB, 0.39731E+00_JPRB, 0.39005E+00_JPRB, 0.37904E+00_JPRB, 0.36866E+00_JPRB, 0.36253E+00_JPRB ,&
&  0.35811E+00_JPRB, 0.35384E+00_JPRB, 0.34837E+00_JPRB, 0.34111E+00_JPRB, 0.33820E+00_JPRB, 0.33783E+00_JPRB, 0.33816E+00_JPRB ,&
&  0.34352E+00_JPRB, 0.34873E+00_JPRB, 0.35555E+00_JPRB, 0.36452E+00_JPRB, 0.37762E+00_JPRB, 0.39428E+00_JPRB, 0.41917E+00_JPRB ,&
&  0.46312E+00_JPRB, 0.49216E+00_JPRB, 0.59460E+00_JPRB, 0.57008E+00_JPRB, 0.58689E+00_JPRB, 0.62034E+00_JPRB, 0.70345E+00_JPRB ,&
&  0.77614E+00_JPRB, 0.79431E+00_JPRB, 0.75110E+00_JPRB, 0.68133E+00_JPRB, 0.61548E+00_JPRB, 0.56633E+00_JPRB, 0.53314E+00_JPRB ,&
&  0.50115E+00_JPRB, 0.47193E+00_JPRB, 0.45178E+00_JPRB, 0.43818E+00_JPRB, 0.42886E+00_JPRB, 0.42329E+00_JPRB, 0.42135E+00_JPRB ,&
&  0.42396E+00_JPRB, 0.43148E+00_JPRB, 0.44447E+00_JPRB, 0.46565E+00_JPRB, 0.48615E+00_JPRB, 0.47196E+00_JPRB, 0.49565E+00_JPRB ,&
&  0.56514E+00_JPRB, 0.57047E+00_JPRB, 0.56549E+00_JPRB, 0.54768E+00_JPRB, 0.52444E+00_JPRB, 0.50090E+00_JPRB, 0.47808E+00_JPRB ,&
&  0.46097E+00_JPRB, 0.44819E+00_JPRB, 0.43635E+00_JPRB, 0.42913E+00_JPRB, 0.42649E+00_JPRB, 0.42835E+00_JPRB, 0.43045E+00_JPRB ,&
&  0.43023E+00_JPRB, 0.42960E+00_JPRB, 0.43013E+00_JPRB, 0.43168E+00_JPRB, 0.43337E+00_JPRB, 0.43543E+00_JPRB, 0.43022E+00_JPRB ,&
&  0.42639E+00_JPRB, 0.42696E+00_JPRB, 0.43189E+00_JPRB, 0.42454E+00_JPRB, 0.42712E+00_JPRB, 0.42952E+00_JPRB, 0.42630E+00_JPRB ,&
&  0.41916E+00_JPRB, 0.42167E+00_JPRB, 0.43320E+00_JPRB, 0.44332E+00_JPRB, 0.46438E+00_JPRB, 0.48490E+00_JPRB, 0.46096E+00_JPRB ,&
&  0.45777E+00_JPRB, 0.44723E+00_JPRB, 0.43667E+00_JPRB, 0.43712E+00_JPRB, 0.44874E+00_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.43493E+00_JPRB, 0.42994E+00_JPRB, 0.42396E+00_JPRB, 0.41782E+00_JPRB, 0.41001E+00_JPRB, 0.40248E+00_JPRB, 0.39796E+00_JPRB ,&
&  0.39788E+00_JPRB, 0.40148E+00_JPRB, 0.39959E+00_JPRB, 0.38964E+00_JPRB, 0.37698E+00_JPRB, 0.36549E+00_JPRB, 0.36005E+00_JPRB ,&
&  0.35567E+00_JPRB, 0.34930E+00_JPRB, 0.34082E+00_JPRB, 0.33174E+00_JPRB, 0.32933E+00_JPRB, 0.33209E+00_JPRB, 0.32960E+00_JPRB ,&
&  0.33552E+00_JPRB, 0.34242E+00_JPRB, 0.34954E+00_JPRB, 0.35609E+00_JPRB, 0.36885E+00_JPRB, 0.38660E+00_JPRB, 0.41168E+00_JPRB ,&
&  0.45682E+00_JPRB, 0.39374E+00_JPRB, 0.59602E+00_JPRB, 0.66888E+00_JPRB, 0.72259E+00_JPRB, 0.77037E+00_JPRB, 0.85255E+00_JPRB ,&
&  0.92147E+00_JPRB, 0.89325E+00_JPRB, 0.83420E+00_JPRB, 0.73411E+00_JPRB, 0.64115E+00_JPRB, 0.57640E+00_JPRB, 0.53634E+00_JPRB ,&
&  0.51140E+00_JPRB, 0.49226E+00_JPRB, 0.47430E+00_JPRB, 0.45837E+00_JPRB, 0.44709E+00_JPRB, 0.44038E+00_JPRB, 0.43901E+00_JPRB ,&
&  0.44783E+00_JPRB, 0.46384E+00_JPRB, 0.48032E+00_JPRB, 0.51251E+00_JPRB, 0.50436E+00_JPRB, 0.49902E+00_JPRB, 0.49778E+00_JPRB ,&
&  0.53843E+00_JPRB, 0.59916E+00_JPRB, 0.57756E+00_JPRB, 0.54556E+00_JPRB, 0.52812E+00_JPRB, 0.51004E+00_JPRB, 0.48907E+00_JPRB ,&
&  0.47000E+00_JPRB, 0.45799E+00_JPRB, 0.44947E+00_JPRB, 0.44446E+00_JPRB, 0.44491E+00_JPRB, 0.44496E+00_JPRB, 0.44106E+00_JPRB ,&
&  0.43959E+00_JPRB, 0.44003E+00_JPRB, 0.44211E+00_JPRB, 0.44569E+00_JPRB, 0.44632E+00_JPRB, 0.44707E+00_JPRB, 0.44588E+00_JPRB ,&
&  0.44467E+00_JPRB, 0.44597E+00_JPRB, 0.43941E+00_JPRB, 0.42915E+00_JPRB, 0.42758E+00_JPRB, 0.42458E+00_JPRB, 0.42223E+00_JPRB ,&
&  0.42519E+00_JPRB, 0.42972E+00_JPRB, 0.43552E+00_JPRB, 0.45119E+00_JPRB, 0.45798E+00_JPRB, 0.48006E+00_JPRB, 0.46164E+00_JPRB ,&
&  0.46407E+00_JPRB, 0.45807E+00_JPRB, 0.45275E+00_JPRB, 0.44555E+00_JPRB, 0.44444E+00_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.43492E+00_JPRB, 0.42868E+00_JPRB, 0.42257E+00_JPRB, 0.41519E+00_JPRB, 0.40587E+00_JPRB, 0.40009E+00_JPRB, 0.40223E+00_JPRB ,&
&  0.40799E+00_JPRB, 0.40557E+00_JPRB, 0.40079E+00_JPRB, 0.39130E+00_JPRB, 0.37645E+00_JPRB, 0.37000E+00_JPRB, 0.36618E+00_JPRB ,&
&  0.35905E+00_JPRB, 0.35373E+00_JPRB, 0.34439E+00_JPRB, 0.33956E+00_JPRB, 0.33801E+00_JPRB, 0.33496E+00_JPRB, 0.32897E+00_JPRB ,&
&  0.33203E+00_JPRB, 0.33877E+00_JPRB, 0.34693E+00_JPRB, 0.35402E+00_JPRB, 0.36743E+00_JPRB, 0.38545E+00_JPRB, 0.41113E+00_JPRB ,&
&  0.45795E+00_JPRB, 0.60522E+00_JPRB, 0.55951E+00_JPRB, 0.84788E+00_JPRB, 0.87936E+00_JPRB, 0.92542E+00_JPRB, 0.91827E+00_JPRB ,&
&  0.91685E+00_JPRB, 0.93148E+00_JPRB, 0.90143E+00_JPRB, 0.77536E+00_JPRB, 0.65572E+00_JPRB, 0.58566E+00_JPRB, 0.54199E+00_JPRB ,&
&  0.51198E+00_JPRB, 0.48827E+00_JPRB, 0.46968E+00_JPRB, 0.45621E+00_JPRB, 0.44561E+00_JPRB, 0.44279E+00_JPRB, 0.45827E+00_JPRB ,&
&  0.48193E+00_JPRB, 0.50139E+00_JPRB, 0.52929E+00_JPRB, 0.55617E+00_JPRB, 0.51698E+00_JPRB, 0.51600E+00_JPRB, 0.53143E+00_JPRB ,&
&  0.54779E+00_JPRB, 0.60560E+00_JPRB, 0.55706E+00_JPRB, 0.53240E+00_JPRB, 0.52229E+00_JPRB, 0.51420E+00_JPRB, 0.49761E+00_JPRB ,&
&  0.48059E+00_JPRB, 0.47250E+00_JPRB, 0.46719E+00_JPRB, 0.46497E+00_JPRB, 0.46179E+00_JPRB, 0.45713E+00_JPRB, 0.45151E+00_JPRB ,&
&  0.44659E+00_JPRB, 0.45107E+00_JPRB, 0.45535E+00_JPRB, 0.45732E+00_JPRB, 0.45862E+00_JPRB, 0.45667E+00_JPRB, 0.45480E+00_JPRB ,&
&  0.45567E+00_JPRB, 0.45864E+00_JPRB, 0.43944E+00_JPRB, 0.42560E+00_JPRB, 0.42091E+00_JPRB, 0.41816E+00_JPRB, 0.41663E+00_JPRB ,&
&  0.42706E+00_JPRB, 0.43531E+00_JPRB, 0.44125E+00_JPRB, 0.45423E+00_JPRB, 0.47336E+00_JPRB, 0.49612E+00_JPRB, 0.47583E+00_JPRB ,&
&  0.48444E+00_JPRB, 0.47752E+00_JPRB, 0.46801E+00_JPRB, 0.45233E+00_JPRB, 0.44455E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.43909E+00_JPRB, 0.42993E+00_JPRB, 0.42399E+00_JPRB, 0.41797E+00_JPRB, 0.41219E+00_JPRB, 0.41058E+00_JPRB, 0.41328E+00_JPRB ,&
&  0.41429E+00_JPRB, 0.40896E+00_JPRB, 0.39993E+00_JPRB, 0.39090E+00_JPRB, 0.37586E+00_JPRB, 0.36734E+00_JPRB, 0.36346E+00_JPRB ,&
&  0.36051E+00_JPRB, 0.36128E+00_JPRB, 0.35767E+00_JPRB, 0.35472E+00_JPRB, 0.35212E+00_JPRB, 0.34706E+00_JPRB, 0.34620E+00_JPRB ,&
&  0.34441E+00_JPRB, 0.34718E+00_JPRB, 0.35345E+00_JPRB, 0.36447E+00_JPRB, 0.38040E+00_JPRB, 0.40239E+00_JPRB, 0.42714E+00_JPRB ,&
&  0.48770E+00_JPRB, 0.57603E+00_JPRB, 0.60929E+00_JPRB, 0.11004E+01_JPRB, 0.10896E+01_JPRB, 0.10395E+01_JPRB, 0.93114E+00_JPRB ,&
&  0.88277E+00_JPRB, 0.93378E+00_JPRB, 0.91349E+00_JPRB, 0.80895E+00_JPRB, 0.69504E+00_JPRB, 0.61106E+00_JPRB, 0.54398E+00_JPRB ,&
&  0.50110E+00_JPRB, 0.47296E+00_JPRB, 0.45859E+00_JPRB, 0.45224E+00_JPRB, 0.45035E+00_JPRB, 0.46959E+00_JPRB, 0.50023E+00_JPRB ,&
&  0.51905E+00_JPRB, 0.54216E+00_JPRB, 0.57796E+00_JPRB, 0.55339E+00_JPRB, 0.52069E+00_JPRB, 0.52864E+00_JPRB, 0.53838E+00_JPRB ,&
&  0.55076E+00_JPRB, 0.55846E+00_JPRB, 0.53309E+00_JPRB, 0.51829E+00_JPRB, 0.50232E+00_JPRB, 0.50898E+00_JPRB, 0.50068E+00_JPRB ,&
&  0.48964E+00_JPRB, 0.48422E+00_JPRB, 0.47961E+00_JPRB, 0.47624E+00_JPRB, 0.47040E+00_JPRB, 0.46580E+00_JPRB, 0.45509E+00_JPRB ,&
&  0.45674E+00_JPRB, 0.45982E+00_JPRB, 0.46078E+00_JPRB, 0.46507E+00_JPRB, 0.46043E+00_JPRB, 0.45862E+00_JPRB, 0.45804E+00_JPRB ,&
&  0.46379E+00_JPRB, 0.46363E+00_JPRB, 0.43839E+00_JPRB, 0.42478E+00_JPRB, 0.42182E+00_JPRB, 0.41926E+00_JPRB, 0.41885E+00_JPRB ,&
&  0.43089E+00_JPRB, 0.44461E+00_JPRB, 0.45316E+00_JPRB, 0.45896E+00_JPRB, 0.48295E+00_JPRB, 0.49778E+00_JPRB, 0.50322E+00_JPRB ,&
&  0.49981E+00_JPRB, 0.49462E+00_JPRB, 0.48484E+00_JPRB, 0.46077E+00_JPRB, 0.45025E+00_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.44001E+00_JPRB, 0.42900E+00_JPRB, 0.42020E+00_JPRB, 0.41752E+00_JPRB, 0.41328E+00_JPRB, 0.41030E+00_JPRB, 0.41623E+00_JPRB ,&
&  0.41243E+00_JPRB, 0.40333E+00_JPRB, 0.38850E+00_JPRB, 0.37276E+00_JPRB, 0.35994E+00_JPRB, 0.36154E+00_JPRB, 0.36628E+00_JPRB ,&
&  0.36678E+00_JPRB, 0.36540E+00_JPRB, 0.36393E+00_JPRB, 0.36465E+00_JPRB, 0.36312E+00_JPRB, 0.36245E+00_JPRB, 0.36154E+00_JPRB ,&
&  0.36110E+00_JPRB, 0.36468E+00_JPRB, 0.37658E+00_JPRB, 0.39616E+00_JPRB, 0.41859E+00_JPRB, 0.44999E+00_JPRB, 0.49014E+00_JPRB ,&
&  0.57684E+00_JPRB, 0.62027E+00_JPRB, 0.69160E+00_JPRB, 0.10432E+01_JPRB, 0.11023E+01_JPRB, 0.10006E+01_JPRB, 0.91017E+00_JPRB ,&
&  0.81662E+00_JPRB, 0.76510E+00_JPRB, 0.88293E+00_JPRB, 0.79568E+00_JPRB, 0.69324E+00_JPRB, 0.59987E+00_JPRB, 0.53324E+00_JPRB ,&
&  0.49442E+00_JPRB, 0.46710E+00_JPRB, 0.45951E+00_JPRB, 0.46500E+00_JPRB, 0.48942E+00_JPRB, 0.52826E+00_JPRB, 0.55188E+00_JPRB ,&
&  0.56717E+00_JPRB, 0.59206E+00_JPRB, 0.62741E+00_JPRB, 0.56739E+00_JPRB, 0.50779E+00_JPRB, 0.51719E+00_JPRB, 0.51661E+00_JPRB ,&
&  0.53057E+00_JPRB, 0.52440E+00_JPRB, 0.52412E+00_JPRB, 0.51231E+00_JPRB, 0.49185E+00_JPRB, 0.51727E+00_JPRB, 0.50080E+00_JPRB ,&
&  0.49550E+00_JPRB, 0.49032E+00_JPRB, 0.48491E+00_JPRB, 0.48173E+00_JPRB, 0.47315E+00_JPRB, 0.45806E+00_JPRB, 0.44878E+00_JPRB ,&
&  0.45107E+00_JPRB, 0.45762E+00_JPRB, 0.45629E+00_JPRB, 0.45617E+00_JPRB, 0.45127E+00_JPRB, 0.45276E+00_JPRB, 0.45694E+00_JPRB ,&
&  0.46606E+00_JPRB, 0.46785E+00_JPRB, 0.44991E+00_JPRB, 0.44366E+00_JPRB, 0.44142E+00_JPRB, 0.44033E+00_JPRB, 0.43860E+00_JPRB ,&
&  0.44638E+00_JPRB, 0.46355E+00_JPRB, 0.47611E+00_JPRB, 0.48851E+00_JPRB, 0.50489E+00_JPRB, 0.51637E+00_JPRB, 0.52197E+00_JPRB ,&
&  0.52732E+00_JPRB, 0.54584E+00_JPRB, 0.55271E+00_JPRB, 0.48751E+00_JPRB, 0.45997E+00_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.42245E+00_JPRB, 0.41348E+00_JPRB, 0.40987E+00_JPRB, 0.40954E+00_JPRB, 0.40726E+00_JPRB, 0.40939E+00_JPRB, 0.41338E+00_JPRB ,&
&  0.39670E+00_JPRB, 0.37844E+00_JPRB, 0.36589E+00_JPRB, 0.36130E+00_JPRB, 0.36073E+00_JPRB, 0.37124E+00_JPRB, 0.37472E+00_JPRB ,&
&  0.37140E+00_JPRB, 0.37006E+00_JPRB, 0.37957E+00_JPRB, 0.38096E+00_JPRB, 0.38166E+00_JPRB, 0.38011E+00_JPRB, 0.38061E+00_JPRB ,&
&  0.38818E+00_JPRB, 0.40205E+00_JPRB, 0.42866E+00_JPRB, 0.46761E+00_JPRB, 0.50605E+00_JPRB, 0.56326E+00_JPRB, 0.62099E+00_JPRB ,&
&  0.72045E+00_JPRB, 0.72490E+00_JPRB, 0.41850E+00_JPRB, 0.66319E+00_JPRB, 0.86408E+00_JPRB, 0.87178E+00_JPRB, 0.80896E+00_JPRB ,&
&  0.74618E+00_JPRB, 0.69722E+00_JPRB, 0.75929E+00_JPRB, 0.75716E+00_JPRB, 0.65024E+00_JPRB, 0.58108E+00_JPRB, 0.52617E+00_JPRB ,&
&  0.49350E+00_JPRB, 0.48212E+00_JPRB, 0.48791E+00_JPRB, 0.51406E+00_JPRB, 0.55815E+00_JPRB, 0.59497E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.61697E+00_JPRB, 0.64797E+00_JPRB, 0.65993E+00_JPRB, 0.54508E+00_JPRB, 0.50136E+00_JPRB, 0.49995E+00_JPRB, 0.49032E+00_JPRB ,&
&  0.48218E+00_JPRB, 0.50986E+00_JPRB, 0.53099E+00_JPRB, 0.51359E+00_JPRB, 0.49689E+00_JPRB, 0.48757E+00_JPRB, 0.50153E+00_JPRB ,&
&  0.49520E+00_JPRB, 0.49104E+00_JPRB, 0.48733E+00_JPRB, 0.47515E+00_JPRB, 0.45149E+00_JPRB, 0.42938E+00_JPRB, 0.43401E+00_JPRB ,&
&  0.43664E+00_JPRB, 0.44398E+00_JPRB, 0.44019E+00_JPRB, 0.44568E+00_JPRB, 0.45037E+00_JPRB, 0.45651E+00_JPRB, 0.46132E+00_JPRB ,&
&  0.46918E+00_JPRB, 0.47585E+00_JPRB, 0.48293E+00_JPRB, 0.47633E+00_JPRB, 0.47115E+00_JPRB, 0.46929E+00_JPRB, 0.47261E+00_JPRB ,&
&  0.48253E+00_JPRB, 0.49566E+00_JPRB, 0.50016E+00_JPRB, 0.51132E+00_JPRB, 0.54276E+00_JPRB, 0.56166E+00_JPRB, 0.57302E+00_JPRB ,&
&  0.62734E+00_JPRB, 0.67405E+00_JPRB, 0.62142E+00_JPRB, 0.47185E+00_JPRB, 0.44119E+00_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.41044E+00_JPRB, 0.40883E+00_JPRB, 0.40614E+00_JPRB, 0.40865E+00_JPRB, 0.40475E+00_JPRB, 0.40798E+00_JPRB, 0.39788E+00_JPRB ,&
&  0.37704E+00_JPRB, 0.36909E+00_JPRB, 0.38296E+00_JPRB, 0.38798E+00_JPRB, 0.39064E+00_JPRB, 0.40283E+00_JPRB, 0.41642E+00_JPRB ,&
&  0.41951E+00_JPRB, 0.41728E+00_JPRB, 0.41415E+00_JPRB, 0.41320E+00_JPRB, 0.40957E+00_JPRB, 0.41130E+00_JPRB, 0.42376E+00_JPRB ,&
&  0.44190E+00_JPRB, 0.47528E+00_JPRB, 0.52505E+00_JPRB, 0.58561E+00_JPRB, 0.64535E+00_JPRB, 0.71750E+00_JPRB, 0.76817E+00_JPRB ,&
&  0.86505E+00_JPRB, 0.62486E+00_JPRB, 0.45266E+00_JPRB, 0.74921E+00_JPRB, 0.84435E+00_JPRB, 0.83308E+00_JPRB, 0.73113E+00_JPRB ,&
&  0.69295E+00_JPRB, 0.67319E+00_JPRB, 0.67644E+00_JPRB, 0.65786E+00_JPRB, 0.59693E+00_JPRB, 0.56648E+00_JPRB, 0.53651E+00_JPRB ,&
&  0.51934E+00_JPRB, 0.52431E+00_JPRB, 0.54578E+00_JPRB, 0.58743E+00_JPRB, 0.63142E+00_JPRB, 0.64305E+00_JPRB, 0.64911E+00_JPRB ,&
&  0.67867E+00_JPRB, 0.70044E+00_JPRB, 0.68153E+00_JPRB, 0.54388E+00_JPRB, 0.50483E+00_JPRB, 0.49100E+00_JPRB, 0.48901E+00_JPRB ,&
&  0.48559E+00_JPRB, 0.50979E+00_JPRB, 0.53620E+00_JPRB, 0.52377E+00_JPRB, 0.50714E+00_JPRB, 0.48841E+00_JPRB, 0.50276E+00_JPRB ,&
&  0.49489E+00_JPRB, 0.48691E+00_JPRB, 0.47444E+00_JPRB, 0.45137E+00_JPRB, 0.42579E+00_JPRB, 0.42709E+00_JPRB, 0.42107E+00_JPRB ,&
&  0.42285E+00_JPRB, 0.43276E+00_JPRB, 0.43043E+00_JPRB, 0.44020E+00_JPRB, 0.45108E+00_JPRB, 0.46612E+00_JPRB, 0.47900E+00_JPRB ,&
&  0.48646E+00_JPRB, 0.49623E+00_JPRB, 0.50841E+00_JPRB, 0.50908E+00_JPRB, 0.50246E+00_JPRB, 0.49213E+00_JPRB, 0.50365E+00_JPRB ,&
&  0.51350E+00_JPRB, 0.53050E+00_JPRB, 0.53530E+00_JPRB, 0.55063E+00_JPRB, 0.54438E+00_JPRB, 0.54704E+00_JPRB, 0.52245E+00_JPRB ,&
&  0.52420E+00_JPRB, 0.49611E+00_JPRB, 0.46353E+00_JPRB, 0.41465E+00_JPRB, 0.40936E+00_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.41683E+00_JPRB, 0.41119E+00_JPRB, 0.40373E+00_JPRB, 0.39595E+00_JPRB, 0.38805E+00_JPRB, 0.38840E+00_JPRB, 0.37868E+00_JPRB ,&
&  0.37524E+00_JPRB, 0.39310E+00_JPRB, 0.41818E+00_JPRB, 0.43836E+00_JPRB, 0.45521E+00_JPRB, 0.45993E+00_JPRB, 0.47198E+00_JPRB ,&
&  0.46954E+00_JPRB, 0.46585E+00_JPRB, 0.46363E+00_JPRB, 0.46213E+00_JPRB, 0.46604E+00_JPRB, 0.48177E+00_JPRB, 0.51042E+00_JPRB ,&
&  0.55168E+00_JPRB, 0.60536E+00_JPRB, 0.66890E+00_JPRB, 0.73304E+00_JPRB, 0.78555E+00_JPRB, 0.83449E+00_JPRB, 0.84640E+00_JPRB ,&
&  0.62123E+00_JPRB, 0.40813E+00_JPRB, 0.76441E+00_JPRB, 0.84838E+00_JPRB, 0.78995E+00_JPRB, 0.75770E+00_JPRB, 0.70472E+00_JPRB ,&
&  0.65379E+00_JPRB, 0.64634E+00_JPRB, 0.62764E+00_JPRB, 0.62390E+00_JPRB, 0.57591E+00_JPRB, 0.56350E+00_JPRB, 0.55837E+00_JPRB ,&
&  0.56068E+00_JPRB, 0.57296E+00_JPRB, 0.60376E+00_JPRB, 0.65016E+00_JPRB, 0.67351E+00_JPRB, 0.67649E+00_JPRB, 0.69897E+00_JPRB ,&
&  0.73578E+00_JPRB, 0.74460E+00_JPRB, 0.72053E+00_JPRB, 0.55985E+00_JPRB, 0.50550E+00_JPRB, 0.49604E+00_JPRB, 0.48898E+00_JPRB ,&
&  0.50332E+00_JPRB, 0.50657E+00_JPRB, 0.51421E+00_JPRB, 0.53207E+00_JPRB, 0.52053E+00_JPRB, 0.48762E+00_JPRB, 0.49538E+00_JPRB ,&
&  0.48368E+00_JPRB, 0.45834E+00_JPRB, 0.43147E+00_JPRB, 0.41773E+00_JPRB, 0.41404E+00_JPRB, 0.42420E+00_JPRB, 0.42306E+00_JPRB ,&
&  0.42476E+00_JPRB, 0.43427E+00_JPRB, 0.43557E+00_JPRB, 0.44127E+00_JPRB, 0.46614E+00_JPRB, 0.49108E+00_JPRB, 0.50661E+00_JPRB ,&
&  0.51164E+00_JPRB, 0.52522E+00_JPRB, 0.53639E+00_JPRB, 0.54160E+00_JPRB, 0.54236E+00_JPRB, 0.52775E+00_JPRB, 0.52781E+00_JPRB ,&
&  0.53550E+00_JPRB, 0.54345E+00_JPRB, 0.54328E+00_JPRB, 0.54772E+00_JPRB, 0.53229E+00_JPRB, 0.53251E+00_JPRB, 0.48324E+00_JPRB ,&
&  0.46191E+00_JPRB, 0.42665E+00_JPRB, 0.40033E+00_JPRB, 0.38993E+00_JPRB, 0.39581E+00_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.41919E+00_JPRB, 0.40287E+00_JPRB, 0.39292E+00_JPRB, 0.38490E+00_JPRB, 0.37549E+00_JPRB, 0.39478E+00_JPRB, 0.39820E+00_JPRB ,&
&  0.41698E+00_JPRB, 0.44898E+00_JPRB, 0.48484E+00_JPRB, 0.49928E+00_JPRB, 0.49709E+00_JPRB, 0.51134E+00_JPRB, 0.52360E+00_JPRB ,&
&  0.52929E+00_JPRB, 0.53308E+00_JPRB, 0.53994E+00_JPRB, 0.54931E+00_JPRB, 0.56768E+00_JPRB, 0.59646E+00_JPRB, 0.63964E+00_JPRB ,&
&  0.68834E+00_JPRB, 0.73934E+00_JPRB, 0.79285E+00_JPRB, 0.83968E+00_JPRB, 0.86438E+00_JPRB, 0.88649E+00_JPRB, 0.85791E+00_JPRB ,&
&  0.56526E+00_JPRB, 0.74105E+00_JPRB, 0.83754E+00_JPRB, 0.78902E+00_JPRB, 0.74431E+00_JPRB, 0.71135E+00_JPRB, 0.68747E+00_JPRB ,&
&  0.65908E+00_JPRB, 0.62265E+00_JPRB, 0.61025E+00_JPRB, 0.63261E+00_JPRB, 0.59288E+00_JPRB, 0.58026E+00_JPRB, 0.58384E+00_JPRB ,&
&  0.59236E+00_JPRB, 0.61173E+00_JPRB, 0.65232E+00_JPRB, 0.69477E+00_JPRB, 0.71029E+00_JPRB, 0.72103E+00_JPRB, 0.74959E+00_JPRB ,&
&  0.78676E+00_JPRB, 0.80030E+00_JPRB, 0.78953E+00_JPRB, 0.62563E+00_JPRB, 0.56194E+00_JPRB, 0.55001E+00_JPRB, 0.53608E+00_JPRB ,&
&  0.50474E+00_JPRB, 0.51751E+00_JPRB, 0.52773E+00_JPRB, 0.54745E+00_JPRB, 0.51408E+00_JPRB, 0.48985E+00_JPRB, 0.46444E+00_JPRB ,&
&  0.43491E+00_JPRB, 0.40447E+00_JPRB, 0.38562E+00_JPRB, 0.40714E+00_JPRB, 0.42941E+00_JPRB, 0.44154E+00_JPRB, 0.43406E+00_JPRB ,&
&  0.45397E+00_JPRB, 0.44281E+00_JPRB, 0.44125E+00_JPRB, 0.45150E+00_JPRB, 0.49721E+00_JPRB, 0.51458E+00_JPRB, 0.51939E+00_JPRB ,&
&  0.52569E+00_JPRB, 0.53263E+00_JPRB, 0.53790E+00_JPRB, 0.54554E+00_JPRB, 0.55062E+00_JPRB, 0.55094E+00_JPRB, 0.54766E+00_JPRB ,&
&  0.54479E+00_JPRB, 0.55317E+00_JPRB, 0.55067E+00_JPRB, 0.54738E+00_JPRB, 0.54051E+00_JPRB, 0.56790E+00_JPRB, 0.50369E+00_JPRB ,&
&  0.45859E+00_JPRB, 0.42517E+00_JPRB, 0.39987E+00_JPRB, 0.39316E+00_JPRB, 0.39894E+00_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.42426E+00_JPRB, 0.41224E+00_JPRB, 0.39511E+00_JPRB, 0.40208E+00_JPRB, 0.40358E+00_JPRB, 0.41810E+00_JPRB, 0.43468E+00_JPRB ,&
&  0.45613E+00_JPRB, 0.49591E+00_JPRB, 0.52982E+00_JPRB, 0.55499E+00_JPRB, 0.56501E+00_JPRB, 0.58148E+00_JPRB, 0.59781E+00_JPRB ,&
&  0.60419E+00_JPRB, 0.61349E+00_JPRB, 0.62550E+00_JPRB, 0.64642E+00_JPRB, 0.67420E+00_JPRB, 0.70541E+00_JPRB, 0.74431E+00_JPRB ,&
&  0.78741E+00_JPRB, 0.83585E+00_JPRB, 0.89976E+00_JPRB, 0.91011E+00_JPRB, 0.92262E+00_JPRB, 0.94465E+00_JPRB, 0.91543E+00_JPRB ,&
&  0.68559E+00_JPRB, 0.79892E+00_JPRB, 0.77530E+00_JPRB, 0.74113E+00_JPRB, 0.71536E+00_JPRB, 0.68180E+00_JPRB, 0.65394E+00_JPRB ,&
&  0.63207E+00_JPRB, 0.59233E+00_JPRB, 0.57624E+00_JPRB, 0.57220E+00_JPRB, 0.60753E+00_JPRB, 0.59303E+00_JPRB, 0.60220E+00_JPRB ,&
&  0.61985E+00_JPRB, 0.65263E+00_JPRB, 0.69117E+00_JPRB, 0.72422E+00_JPRB, 0.74929E+00_JPRB, 0.77104E+00_JPRB, 0.80950E+00_JPRB ,&
&  0.85308E+00_JPRB, 0.87090E+00_JPRB, 0.87225E+00_JPRB, 0.76039E+00_JPRB, 0.65497E+00_JPRB, 0.61863E+00_JPRB, 0.57027E+00_JPRB ,&
&  0.52513E+00_JPRB, 0.51648E+00_JPRB, 0.54732E+00_JPRB, 0.56849E+00_JPRB, 0.52710E+00_JPRB, 0.48326E+00_JPRB, 0.44663E+00_JPRB ,&
&  0.42020E+00_JPRB, 0.39648E+00_JPRB, 0.41136E+00_JPRB, 0.45783E+00_JPRB, 0.48064E+00_JPRB, 0.49829E+00_JPRB, 0.49848E+00_JPRB ,&
&  0.49178E+00_JPRB, 0.46703E+00_JPRB, 0.45091E+00_JPRB, 0.46208E+00_JPRB, 0.49614E+00_JPRB, 0.52855E+00_JPRB, 0.52825E+00_JPRB ,&
&  0.51402E+00_JPRB, 0.50958E+00_JPRB, 0.52023E+00_JPRB, 0.54307E+00_JPRB, 0.56201E+00_JPRB, 0.58879E+00_JPRB, 0.61157E+00_JPRB ,&
&  0.64282E+00_JPRB, 0.68870E+00_JPRB, 0.68243E+00_JPRB, 0.68603E+00_JPRB, 0.78723E+00_JPRB, 0.84366E+00_JPRB, 0.53353E+00_JPRB ,&
&  0.45077E+00_JPRB, 0.41749E+00_JPRB, 0.41231E+00_JPRB, 0.40507E+00_JPRB, 0.43168E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.45797E+00_JPRB, 0.47473E+00_JPRB, 0.46459E+00_JPRB, 0.46655E+00_JPRB, 0.47280E+00_JPRB, 0.48232E+00_JPRB, 0.50264E+00_JPRB ,&
&  0.50654E+00_JPRB, 0.53060E+00_JPRB, 0.55348E+00_JPRB, 0.57103E+00_JPRB, 0.59769E+00_JPRB, 0.63553E+00_JPRB, 0.68050E+00_JPRB ,&
&  0.72357E+00_JPRB, 0.74578E+00_JPRB, 0.75922E+00_JPRB, 0.76499E+00_JPRB, 0.78327E+00_JPRB, 0.81382E+00_JPRB, 0.82209E+00_JPRB ,&
&  0.86599E+00_JPRB, 0.89457E+00_JPRB, 0.93272E+00_JPRB, 0.99042E+00_JPRB, 0.93946E+00_JPRB, 0.93336E+00_JPRB, 0.68475E+00_JPRB ,&
&  0.76651E+00_JPRB, 0.76706E+00_JPRB, 0.74728E+00_JPRB, 0.72661E+00_JPRB, 0.69830E+00_JPRB, 0.65480E+00_JPRB, 0.62454E+00_JPRB ,&
&  0.60230E+00_JPRB, 0.58253E+00_JPRB, 0.56425E+00_JPRB, 0.55846E+00_JPRB, 0.60812E+00_JPRB, 0.59893E+00_JPRB, 0.61842E+00_JPRB ,&
&  0.65197E+00_JPRB, 0.68786E+00_JPRB, 0.71283E+00_JPRB, 0.73817E+00_JPRB, 0.77265E+00_JPRB, 0.81808E+00_JPRB, 0.86588E+00_JPRB ,&
&  0.90380E+00_JPRB, 0.94271E+00_JPRB, 0.94084E+00_JPRB, 0.83269E+00_JPRB, 0.79749E+00_JPRB, 0.69304E+00_JPRB, 0.60419E+00_JPRB ,&
&  0.55546E+00_JPRB, 0.54623E+00_JPRB, 0.58345E+00_JPRB, 0.60858E+00_JPRB, 0.54936E+00_JPRB, 0.49265E+00_JPRB, 0.45300E+00_JPRB ,&
&  0.43181E+00_JPRB, 0.42906E+00_JPRB, 0.45337E+00_JPRB, 0.47935E+00_JPRB, 0.51395E+00_JPRB, 0.54601E+00_JPRB, 0.55146E+00_JPRB ,&
&  0.53972E+00_JPRB, 0.50884E+00_JPRB, 0.49522E+00_JPRB, 0.51195E+00_JPRB, 0.56862E+00_JPRB, 0.62884E+00_JPRB, 0.67742E+00_JPRB ,&
&  0.56103E+00_JPRB, 0.48039E+00_JPRB, 0.51144E+00_JPRB, 0.55093E+00_JPRB, 0.60550E+00_JPRB, 0.68117E+00_JPRB, 0.79588E+00_JPRB ,&
&  0.96596E+00_JPRB, 0.13175E+01_JPRB, 0.13021E+01_JPRB, 0.10890E+01_JPRB, 0.16116E+01_JPRB, 0.13319E+01_JPRB, 0.47521E+00_JPRB ,&
&  0.45520E+00_JPRB, 0.44702E+00_JPRB, 0.44873E+00_JPRB, 0.44941E+00_JPRB, 0.45738E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.54254E+00_JPRB, 0.54934E+00_JPRB, 0.56429E+00_JPRB, 0.56328E+00_JPRB, 0.57013E+00_JPRB, 0.58679E+00_JPRB, 0.60329E+00_JPRB ,&
&  0.62182E+00_JPRB, 0.62781E+00_JPRB, 0.63828E+00_JPRB, 0.65025E+00_JPRB, 0.66416E+00_JPRB, 0.69724E+00_JPRB, 0.74888E+00_JPRB ,&
&  0.77990E+00_JPRB, 0.80768E+00_JPRB, 0.84488E+00_JPRB, 0.84489E+00_JPRB, 0.85897E+00_JPRB, 0.89051E+00_JPRB, 0.90185E+00_JPRB ,&
&  0.93373E+00_JPRB, 0.94839E+00_JPRB, 0.96211E+00_JPRB, 0.10053E+01_JPRB, 0.96427E+00_JPRB, 0.97221E+00_JPRB, 0.75064E+00_JPRB ,&
&  0.78782E+00_JPRB, 0.76354E+00_JPRB, 0.71615E+00_JPRB, 0.69156E+00_JPRB, 0.66883E+00_JPRB, 0.63679E+00_JPRB, 0.61191E+00_JPRB ,&
&  0.60054E+00_JPRB, 0.57867E+00_JPRB, 0.55775E+00_JPRB, 0.57788E+00_JPRB, 0.58822E+00_JPRB, 0.60663E+00_JPRB, 0.63847E+00_JPRB ,&
&  0.67116E+00_JPRB, 0.69984E+00_JPRB, 0.71889E+00_JPRB, 0.75225E+00_JPRB, 0.80545E+00_JPRB, 0.85907E+00_JPRB, 0.91720E+00_JPRB ,&
&  0.95527E+00_JPRB, 0.99514E+00_JPRB, 0.10133E+01_JPRB, 0.92405E+00_JPRB, 0.99338E+00_JPRB, 0.87226E+00_JPRB, 0.68821E+00_JPRB ,&
&  0.57893E+00_JPRB, 0.54721E+00_JPRB, 0.57640E+00_JPRB, 0.63759E+00_JPRB, 0.58207E+00_JPRB, 0.53627E+00_JPRB, 0.49347E+00_JPRB ,&
&  0.47441E+00_JPRB, 0.47064E+00_JPRB, 0.47550E+00_JPRB, 0.49846E+00_JPRB, 0.53303E+00_JPRB, 0.56988E+00_JPRB, 0.59126E+00_JPRB ,&
&  0.58715E+00_JPRB, 0.55311E+00_JPRB, 0.51937E+00_JPRB, 0.49617E+00_JPRB, 0.50017E+00_JPRB, 0.52275E+00_JPRB, 0.49771E+00_JPRB ,&
&  0.51401E+00_JPRB, 0.49874E+00_JPRB, 0.51024E+00_JPRB, 0.57260E+00_JPRB, 0.67991E+00_JPRB, 0.89045E+00_JPRB, 0.12900E+01_JPRB ,&
&  0.18800E+01_JPRB, 0.20426E+01_JPRB, 0.25414E+01_JPRB, 0.34136E+01_JPRB, 0.25118E+01_JPRB, 0.17545E+01_JPRB, 0.52326E+00_JPRB ,&
&  0.53565E+00_JPRB, 0.52940E+00_JPRB, 0.52652E+00_JPRB, 0.52329E+00_JPRB, 0.52778E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.56993E+00_JPRB, 0.58231E+00_JPRB, 0.59720E+00_JPRB, 0.60998E+00_JPRB, 0.62744E+00_JPRB, 0.65318E+00_JPRB, 0.67849E+00_JPRB ,&
&  0.69909E+00_JPRB, 0.71266E+00_JPRB, 0.72050E+00_JPRB, 0.73527E+00_JPRB, 0.75435E+00_JPRB, 0.77866E+00_JPRB, 0.81119E+00_JPRB ,&
&  0.81584E+00_JPRB, 0.84160E+00_JPRB, 0.84622E+00_JPRB, 0.86655E+00_JPRB, 0.86490E+00_JPRB, 0.89185E+00_JPRB, 0.90758E+00_JPRB ,&
&  0.91535E+00_JPRB, 0.91800E+00_JPRB, 0.91702E+00_JPRB, 0.92803E+00_JPRB, 0.90317E+00_JPRB, 0.91060E+00_JPRB, 0.68996E+00_JPRB ,&
&  0.77278E+00_JPRB, 0.74162E+00_JPRB, 0.66838E+00_JPRB, 0.63417E+00_JPRB, 0.60529E+00_JPRB, 0.58331E+00_JPRB, 0.56819E+00_JPRB ,&
&  0.55203E+00_JPRB, 0.55678E+00_JPRB, 0.55126E+00_JPRB, 0.55573E+00_JPRB, 0.58505E+00_JPRB, 0.61767E+00_JPRB, 0.64101E+00_JPRB ,&
&  0.67630E+00_JPRB, 0.70437E+00_JPRB, 0.72734E+00_JPRB, 0.75963E+00_JPRB, 0.80730E+00_JPRB, 0.85553E+00_JPRB, 0.90376E+00_JPRB ,&
&  0.92080E+00_JPRB, 0.94932E+00_JPRB, 0.90310E+00_JPRB, 0.94063E+00_JPRB, 0.10642E+01_JPRB, 0.99675E+00_JPRB, 0.78917E+00_JPRB ,&
&  0.54050E+00_JPRB, 0.53801E+00_JPRB, 0.59083E+00_JPRB, 0.68655E+00_JPRB, 0.64653E+00_JPRB, 0.60589E+00_JPRB, 0.56112E+00_JPRB ,&
&  0.53853E+00_JPRB, 0.51715E+00_JPRB, 0.51883E+00_JPRB, 0.54216E+00_JPRB, 0.57854E+00_JPRB, 0.61388E+00_JPRB, 0.62953E+00_JPRB ,&
&  0.61830E+00_JPRB, 0.58455E+00_JPRB, 0.53823E+00_JPRB, 0.49474E+00_JPRB, 0.46333E+00_JPRB, 0.43291E+00_JPRB, 0.48959E+00_JPRB ,&
&  0.48049E+00_JPRB, 0.46188E+00_JPRB, 0.50047E+00_JPRB, 0.46866E+00_JPRB, 0.65319E+00_JPRB, 0.80798E+00_JPRB, 0.10548E+01_JPRB ,&
&  0.11903E+01_JPRB, 0.16312E+01_JPRB, 0.20557E+01_JPRB, 0.14651E+01_JPRB, 0.10785E+01_JPRB, 0.64860E+00_JPRB, 0.57432E+00_JPRB ,&
&  0.56998E+00_JPRB, 0.55202E+00_JPRB, 0.54819E+00_JPRB, 0.55373E+00_JPRB, 0.56028E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.59994E+00_JPRB, 0.61523E+00_JPRB, 0.63062E+00_JPRB, 0.64514E+00_JPRB, 0.65908E+00_JPRB, 0.67116E+00_JPRB, 0.68466E+00_JPRB ,&
&  0.69888E+00_JPRB, 0.71410E+00_JPRB, 0.72857E+00_JPRB, 0.74653E+00_JPRB, 0.76360E+00_JPRB, 0.77153E+00_JPRB, 0.78103E+00_JPRB ,&
&  0.78418E+00_JPRB, 0.80565E+00_JPRB, 0.81070E+00_JPRB, 0.82955E+00_JPRB, 0.84652E+00_JPRB, 0.85331E+00_JPRB, 0.87559E+00_JPRB ,&
&  0.88561E+00_JPRB, 0.89357E+00_JPRB, 0.90309E+00_JPRB, 0.92271E+00_JPRB, 0.92487E+00_JPRB, 0.96268E+00_JPRB, 0.75086E+00_JPRB ,&
&  0.73559E+00_JPRB, 0.68989E+00_JPRB, 0.62733E+00_JPRB, 0.57705E+00_JPRB, 0.53825E+00_JPRB, 0.49792E+00_JPRB, 0.47325E+00_JPRB ,&
&  0.46780E+00_JPRB, 0.46950E+00_JPRB, 0.47982E+00_JPRB, 0.49810E+00_JPRB, 0.52953E+00_JPRB, 0.57103E+00_JPRB, 0.60431E+00_JPRB ,&
&  0.63532E+00_JPRB, 0.67272E+00_JPRB, 0.70477E+00_JPRB, 0.73796E+00_JPRB, 0.77613E+00_JPRB, 0.80585E+00_JPRB, 0.82248E+00_JPRB ,&
&  0.81836E+00_JPRB, 0.81734E+00_JPRB, 0.74672E+00_JPRB, 0.82820E+00_JPRB, 0.93910E+00_JPRB, 0.93988E+00_JPRB, 0.79512E+00_JPRB ,&
&  0.59855E+00_JPRB, 0.54805E+00_JPRB, 0.57984E+00_JPRB, 0.71651E+00_JPRB, 0.73980E+00_JPRB, 0.71479E+00_JPRB, 0.68086E+00_JPRB ,&
&  0.65233E+00_JPRB, 0.62189E+00_JPRB, 0.60608E+00_JPRB, 0.62044E+00_JPRB, 0.65150E+00_JPRB, 0.66473E+00_JPRB, 0.63199E+00_JPRB ,&
&  0.60327E+00_JPRB, 0.58325E+00_JPRB, 0.54803E+00_JPRB, 0.49375E+00_JPRB, 0.46101E+00_JPRB, 0.40666E+00_JPRB, 0.44727E+00_JPRB ,&
&  0.43447E+00_JPRB, 0.38188E+00_JPRB, 0.41605E+00_JPRB, 0.51087E+00_JPRB, 0.57449E+00_JPRB, 0.65914E+00_JPRB, 0.82815E+00_JPRB ,&
&  0.97029E+00_JPRB, 0.10147E+01_JPRB, 0.94541E+00_JPRB, 0.76778E+00_JPRB, 0.62840E+00_JPRB, 0.57330E+00_JPRB, 0.57065E+00_JPRB ,&
&  0.56928E+00_JPRB, 0.56287E+00_JPRB, 0.56680E+00_JPRB, 0.57784E+00_JPRB, 0.58686E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.57992E+00_JPRB, 0.59472E+00_JPRB, 0.60977E+00_JPRB, 0.62503E+00_JPRB, 0.63813E+00_JPRB, 0.65010E+00_JPRB, 0.66296E+00_JPRB ,&
&  0.67591E+00_JPRB, 0.69129E+00_JPRB, 0.70776E+00_JPRB, 0.72514E+00_JPRB, 0.73791E+00_JPRB, 0.74115E+00_JPRB, 0.74808E+00_JPRB ,&
&  0.75139E+00_JPRB, 0.74866E+00_JPRB, 0.73874E+00_JPRB, 0.75490E+00_JPRB, 0.77175E+00_JPRB, 0.78351E+00_JPRB, 0.80299E+00_JPRB ,&
&  0.82602E+00_JPRB, 0.81710E+00_JPRB, 0.81447E+00_JPRB, 0.83297E+00_JPRB, 0.84247E+00_JPRB, 0.79647E+00_JPRB, 0.85626E+00_JPRB ,&
&  0.60560E+00_JPRB, 0.71678E+00_JPRB, 0.65851E+00_JPRB, 0.57184E+00_JPRB, 0.54771E+00_JPRB, 0.48648E+00_JPRB, 0.45057E+00_JPRB ,&
&  0.46638E+00_JPRB, 0.44771E+00_JPRB, 0.43600E+00_JPRB, 0.43186E+00_JPRB, 0.45121E+00_JPRB, 0.49243E+00_JPRB, 0.52795E+00_JPRB ,&
&  0.57137E+00_JPRB, 0.60793E+00_JPRB, 0.65529E+00_JPRB, 0.70938E+00_JPRB, 0.77093E+00_JPRB, 0.80220E+00_JPRB, 0.76258E+00_JPRB ,&
&  0.71216E+00_JPRB, 0.68953E+00_JPRB, 0.57260E+00_JPRB, 0.65199E+00_JPRB, 0.75801E+00_JPRB, 0.77531E+00_JPRB, 0.69620E+00_JPRB ,&
&  0.58873E+00_JPRB, 0.57879E+00_JPRB, 0.68542E+00_JPRB, 0.78741E+00_JPRB, 0.83125E+00_JPRB, 0.83936E+00_JPRB, 0.79903E+00_JPRB ,&
&  0.76230E+00_JPRB, 0.71446E+00_JPRB, 0.68081E+00_JPRB, 0.69805E+00_JPRB, 0.70088E+00_JPRB, 0.67606E+00_JPRB, 0.60057E+00_JPRB ,&
&  0.55848E+00_JPRB, 0.55193E+00_JPRB, 0.54553E+00_JPRB, 0.52723E+00_JPRB, 0.48515E+00_JPRB, 0.56757E+00_JPRB, 0.45191E+00_JPRB ,&
&  0.47477E+00_JPRB, 0.42930E+00_JPRB, 0.36278E+00_JPRB, 0.46459E+00_JPRB, 0.50671E+00_JPRB, 0.53782E+00_JPRB, 0.58892E+00_JPRB ,&
&  0.61959E+00_JPRB, 0.57775E+00_JPRB, 0.56859E+00_JPRB, 0.54532E+00_JPRB, 0.54397E+00_JPRB, 0.53884E+00_JPRB, 0.55048E+00_JPRB ,&
&  0.55685E+00_JPRB, 0.55498E+00_JPRB, 0.55961E+00_JPRB, 0.56681E+00_JPRB, 0.57095E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.55227E+00_JPRB, 0.55827E+00_JPRB, 0.56250E+00_JPRB, 0.57703E+00_JPRB, 0.59919E+00_JPRB, 0.60100E+00_JPRB, 0.61157E+00_JPRB ,&
&  0.61662E+00_JPRB, 0.63060E+00_JPRB, 0.64577E+00_JPRB, 0.66430E+00_JPRB, 0.67266E+00_JPRB, 0.67460E+00_JPRB, 0.68867E+00_JPRB ,&
&  0.69920E+00_JPRB, 0.69186E+00_JPRB, 0.69815E+00_JPRB, 0.71252E+00_JPRB, 0.71572E+00_JPRB, 0.73978E+00_JPRB, 0.73825E+00_JPRB ,&
&  0.76550E+00_JPRB, 0.77307E+00_JPRB, 0.79722E+00_JPRB, 0.86406E+00_JPRB, 0.94474E+00_JPRB, 0.11187E+01_JPRB, 0.14247E+01_JPRB ,&
&  0.11018E+01_JPRB, 0.79562E+00_JPRB, 0.75661E+00_JPRB, 0.64248E+00_JPRB, 0.63717E+00_JPRB, 0.65463E+00_JPRB, 0.64961E+00_JPRB ,&
&  0.62297E+00_JPRB, 0.57832E+00_JPRB, 0.54558E+00_JPRB, 0.51627E+00_JPRB, 0.50121E+00_JPRB, 0.51059E+00_JPRB, 0.50848E+00_JPRB ,&
&  0.48623E+00_JPRB, 0.52655E+00_JPRB, 0.58454E+00_JPRB, 0.67639E+00_JPRB, 0.74731E+00_JPRB, 0.88795E+00_JPRB, 0.87658E+00_JPRB ,&
&  0.83265E+00_JPRB, 0.79419E+00_JPRB, 0.71548E+00_JPRB, 0.75112E+00_JPRB, 0.80864E+00_JPRB, 0.78527E+00_JPRB, 0.69501E+00_JPRB ,&
&  0.66031E+00_JPRB, 0.60285E+00_JPRB, 0.63428E+00_JPRB, 0.85764E+00_JPRB, 0.94234E+00_JPRB, 0.98172E+00_JPRB, 0.94378E+00_JPRB ,&
&  0.88978E+00_JPRB, 0.84338E+00_JPRB, 0.78150E+00_JPRB, 0.73739E+00_JPRB, 0.71611E+00_JPRB, 0.67421E+00_JPRB, 0.59893E+00_JPRB ,&
&  0.53171E+00_JPRB, 0.52240E+00_JPRB, 0.53017E+00_JPRB, 0.51960E+00_JPRB, 0.54796E+00_JPRB, 0.49271E+00_JPRB, 0.46168E+00_JPRB ,&
&  0.39235E+00_JPRB, 0.41116E+00_JPRB, 0.35723E+00_JPRB, 0.41866E+00_JPRB, 0.41524E+00_JPRB, 0.40640E+00_JPRB, 0.41911E+00_JPRB ,&
&  0.42971E+00_JPRB, 0.43896E+00_JPRB, 0.45299E+00_JPRB, 0.46507E+00_JPRB, 0.47994E+00_JPRB, 0.50414E+00_JPRB, 0.53042E+00_JPRB ,&
&  0.55059E+00_JPRB, 0.53761E+00_JPRB, 0.53717E+00_JPRB, 0.54041E+00_JPRB, 0.55243E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.51408E+00_JPRB, 0.51007E+00_JPRB, 0.50235E+00_JPRB, 0.50239E+00_JPRB, 0.50643E+00_JPRB, 0.51844E+00_JPRB, 0.52573E+00_JPRB ,&
&  0.52329E+00_JPRB, 0.52707E+00_JPRB, 0.51051E+00_JPRB, 0.51023E+00_JPRB, 0.52286E+00_JPRB, 0.52958E+00_JPRB, 0.53578E+00_JPRB ,&
&  0.56372E+00_JPRB, 0.57541E+00_JPRB, 0.57402E+00_JPRB, 0.57695E+00_JPRB, 0.57273E+00_JPRB, 0.61287E+00_JPRB, 0.61603E+00_JPRB ,&
&  0.60242E+00_JPRB, 0.62561E+00_JPRB, 0.72347E+00_JPRB, 0.96604E+00_JPRB, 0.11673E+01_JPRB, 0.10972E+01_JPRB, 0.12093E+01_JPRB ,&
&  0.99719E+00_JPRB, 0.87406E+00_JPRB, 0.10017E+01_JPRB, 0.91074E+00_JPRB, 0.93990E+00_JPRB, 0.96965E+00_JPRB, 0.95609E+00_JPRB ,&
&  0.96345E+00_JPRB, 0.94066E+00_JPRB, 0.93000E+00_JPRB, 0.92349E+00_JPRB, 0.90415E+00_JPRB, 0.88165E+00_JPRB, 0.84452E+00_JPRB ,&
&  0.78132E+00_JPRB, 0.76107E+00_JPRB, 0.79209E+00_JPRB, 0.90064E+00_JPRB, 0.10287E+01_JPRB, 0.11320E+01_JPRB, 0.11460E+01_JPRB ,&
&  0.11587E+01_JPRB, 0.11683E+01_JPRB, 0.10985E+01_JPRB, 0.11161E+01_JPRB, 0.11598E+01_JPRB, 0.10669E+01_JPRB, 0.97413E+00_JPRB ,&
&  0.89213E+00_JPRB, 0.69201E+00_JPRB, 0.65413E+00_JPRB, 0.91707E+00_JPRB, 0.10055E+01_JPRB, 0.10264E+01_JPRB, 0.10568E+01_JPRB ,&
&  0.10139E+01_JPRB, 0.99619E+00_JPRB, 0.91271E+00_JPRB, 0.81662E+00_JPRB, 0.73029E+00_JPRB, 0.65835E+00_JPRB, 0.62012E+00_JPRB ,&
&  0.54023E+00_JPRB, 0.49968E+00_JPRB, 0.49921E+00_JPRB, 0.48957E+00_JPRB, 0.46830E+00_JPRB, 0.48258E+00_JPRB, 0.42151E+00_JPRB ,&
&  0.36994E+00_JPRB, 0.34943E+00_JPRB, 0.37396E+00_JPRB, 0.36657E+00_JPRB, 0.32059E+00_JPRB, 0.35460E+00_JPRB, 0.34923E+00_JPRB ,&
&  0.36325E+00_JPRB, 0.38309E+00_JPRB, 0.39058E+00_JPRB, 0.40076E+00_JPRB, 0.41966E+00_JPRB, 0.43650E+00_JPRB, 0.46050E+00_JPRB ,&
&  0.47627E+00_JPRB, 0.48898E+00_JPRB, 0.50254E+00_JPRB, 0.51616E+00_JPRB, 0.51226E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.56168E+00_JPRB, 0.58768E+00_JPRB, 0.60235E+00_JPRB, 0.62063E+00_JPRB, 0.62737E+00_JPRB, 0.62062E+00_JPRB, 0.59390E+00_JPRB ,&
&  0.57245E+00_JPRB, 0.54936E+00_JPRB, 0.53949E+00_JPRB, 0.52372E+00_JPRB, 0.53796E+00_JPRB, 0.56755E+00_JPRB, 0.59384E+00_JPRB ,&
&  0.65864E+00_JPRB, 0.62419E+00_JPRB, 0.62991E+00_JPRB, 0.66080E+00_JPRB, 0.67210E+00_JPRB, 0.71396E+00_JPRB, 0.83195E+00_JPRB ,&
&  0.10227E+01_JPRB, 0.12364E+01_JPRB, 0.15171E+01_JPRB, 0.19154E+01_JPRB, 0.19364E+01_JPRB, 0.13784E+01_JPRB, 0.12233E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.10331E+01_JPRB, 0.11199E+01_JPRB, 0.11819E+01_JPRB, 0.12398E+01_JPRB, 0.12295E+01_JPRB, 0.12428E+01_JPRB ,&
&  0.12605E+01_JPRB, 0.12842E+01_JPRB, 0.12959E+01_JPRB, 0.12951E+01_JPRB, 0.13033E+01_JPRB, 0.13239E+01_JPRB, 0.13576E+01_JPRB ,&
&  0.13796E+01_JPRB, 0.13739E+01_JPRB, 0.13702E+01_JPRB, 0.13402E+01_JPRB, 0.13964E+01_JPRB, 0.14327E+01_JPRB, 0.14627E+01_JPRB ,&
&  0.14693E+01_JPRB, 0.14731E+01_JPRB, 0.15470E+01_JPRB, 0.15332E+01_JPRB, 0.14872E+01_JPRB, 0.14345E+01_JPRB, 0.13227E+01_JPRB ,&
&  0.11932E+01_JPRB, 0.95806E+00_JPRB, 0.84831E+00_JPRB, 0.12214E+01_JPRB, 0.12194E+01_JPRB, 0.10692E+01_JPRB, 0.11610E+01_JPRB ,&
&  0.11144E+01_JPRB, 0.11108E+01_JPRB, 0.10715E+01_JPRB, 0.93699E+00_JPRB, 0.81823E+00_JPRB, 0.72182E+00_JPRB, 0.61456E+00_JPRB ,&
&  0.60117E+00_JPRB, 0.53075E+00_JPRB, 0.48691E+00_JPRB, 0.47382E+00_JPRB, 0.45103E+00_JPRB, 0.44401E+00_JPRB, 0.46086E+00_JPRB ,&
&  0.43740E+00_JPRB, 0.39459E+00_JPRB, 0.40576E+00_JPRB, 0.44816E+00_JPRB, 0.46767E+00_JPRB, 0.36374E+00_JPRB, 0.34827E+00_JPRB ,&
&  0.36319E+00_JPRB, 0.37715E+00_JPRB, 0.39058E+00_JPRB, 0.40469E+00_JPRB, 0.41620E+00_JPRB, 0.43654E+00_JPRB, 0.45679E+00_JPRB ,&
&  0.48003E+00_JPRB, 0.51265E+00_JPRB, 0.52781E+00_JPRB, 0.54335E+00_JPRB, 0.55847E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.78632E+00_JPRB, 0.82289E+00_JPRB, 0.85662E+00_JPRB, 0.87548E+00_JPRB, 0.87709E+00_JPRB, 0.88506E+00_JPRB, 0.88639E+00_JPRB ,&
&  0.89635E+00_JPRB, 0.89797E+00_JPRB, 0.92706E+00_JPRB, 0.98901E+00_JPRB, 0.11114E+01_JPRB, 0.11770E+01_JPRB, 0.12617E+01_JPRB ,&
&  0.12686E+01_JPRB, 0.12954E+01_JPRB, 0.13263E+01_JPRB, 0.13669E+01_JPRB, 0.14445E+01_JPRB, 0.15593E+01_JPRB, 0.17885E+01_JPRB ,&
&  0.22079E+01_JPRB, 0.24591E+01_JPRB, 0.23640E+01_JPRB, 0.19421E+01_JPRB, 0.14687E+01_JPRB, 0.11572E+01_JPRB, 0.12320E+01_JPRB ,&
&  0.12791E+01_JPRB, 0.13071E+01_JPRB, 0.13462E+01_JPRB, 0.13584E+01_JPRB, 0.13524E+01_JPRB, 0.13561E+01_JPRB, 0.13529E+01_JPRB ,&
&  0.13598E+01_JPRB, 0.13723E+01_JPRB, 0.13726E+01_JPRB, 0.13699E+01_JPRB, 0.13956E+01_JPRB, 0.14498E+01_JPRB, 0.15294E+01_JPRB ,&
&  0.16502E+01_JPRB, 0.17389E+01_JPRB, 0.17594E+01_JPRB, 0.17493E+01_JPRB, 0.17211E+01_JPRB, 0.16948E+01_JPRB, 0.16976E+01_JPRB ,&
&  0.17139E+01_JPRB, 0.16919E+01_JPRB, 0.17715E+01_JPRB, 0.18179E+01_JPRB, 0.17343E+01_JPRB, 0.16329E+01_JPRB, 0.15462E+01_JPRB ,&
&  0.14819E+01_JPRB, 0.14098E+01_JPRB, 0.11992E+01_JPRB, 0.15265E+01_JPRB, 0.15651E+01_JPRB, 0.14014E+01_JPRB, 0.12634E+01_JPRB ,&
&  0.12232E+01_JPRB, 0.12068E+01_JPRB, 0.11406E+01_JPRB, 0.99093E+00_JPRB, 0.86888E+00_JPRB, 0.81715E+00_JPRB, 0.64800E+00_JPRB ,&
&  0.67085E+00_JPRB, 0.59371E+00_JPRB, 0.52189E+00_JPRB, 0.49093E+00_JPRB, 0.47589E+00_JPRB, 0.46013E+00_JPRB, 0.47162E+00_JPRB ,&
&  0.43793E+00_JPRB, 0.48445E+00_JPRB, 0.58094E+00_JPRB, 0.92996E+00_JPRB, 0.17003E+01_JPRB, 0.43364E+00_JPRB, 0.36371E+00_JPRB ,&
&  0.38456E+00_JPRB, 0.41322E+00_JPRB, 0.44552E+00_JPRB, 0.47079E+00_JPRB, 0.49095E+00_JPRB, 0.52371E+00_JPRB, 0.57554E+00_JPRB ,&
&  0.61209E+00_JPRB, 0.64116E+00_JPRB, 0.67526E+00_JPRB, 0.72967E+00_JPRB, 0.75617E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.92498E+00_JPRB, 0.95929E+00_JPRB, 0.98057E+00_JPRB, 0.10213E+01_JPRB, 0.10622E+01_JPRB, 0.11067E+01_JPRB, 0.11700E+01_JPRB ,&
&  0.12547E+01_JPRB, 0.13478E+01_JPRB, 0.14364E+01_JPRB, 0.14997E+01_JPRB, 0.15581E+01_JPRB, 0.16021E+01_JPRB, 0.16408E+01_JPRB ,&
&  0.16693E+01_JPRB, 0.17370E+01_JPRB, 0.18139E+01_JPRB, 0.19156E+01_JPRB, 0.20346E+01_JPRB, 0.21728E+01_JPRB, 0.23157E+01_JPRB ,&
&  0.24018E+01_JPRB, 0.22793E+01_JPRB, 0.19531E+01_JPRB, 0.13470E+01_JPRB, 0.10758E+01_JPRB, 0.11734E+01_JPRB, 0.12236E+01_JPRB ,&
&  0.12909E+01_JPRB, 0.13420E+01_JPRB, 0.13450E+01_JPRB, 0.13786E+01_JPRB, 0.14013E+01_JPRB, 0.14162E+01_JPRB, 0.14176E+01_JPRB ,&
&  0.14287E+01_JPRB, 0.14465E+01_JPRB, 0.14523E+01_JPRB, 0.14544E+01_JPRB, 0.14851E+01_JPRB, 0.15727E+01_JPRB, 0.17147E+01_JPRB ,&
&  0.18911E+01_JPRB, 0.20119E+01_JPRB, 0.20906E+01_JPRB, 0.21313E+01_JPRB, 0.20358E+01_JPRB, 0.19382E+01_JPRB, 0.18781E+01_JPRB ,&
&  0.18785E+01_JPRB, 0.18987E+01_JPRB, 0.19674E+01_JPRB, 0.20183E+01_JPRB, 0.19992E+01_JPRB, 0.18161E+01_JPRB, 0.17110E+01_JPRB ,&
&  0.16964E+01_JPRB, 0.16094E+01_JPRB, 0.15245E+01_JPRB, 0.17538E+01_JPRB, 0.11515E+01_JPRB, 0.11818E+01_JPRB, 0.12448E+01_JPRB ,&
&  0.13561E+01_JPRB, 0.14256E+01_JPRB, 0.12633E+01_JPRB, 0.10365E+01_JPRB, 0.91754E+00_JPRB, 0.67975E+00_JPRB, 0.65489E+00_JPRB ,&
&  0.73096E+00_JPRB, 0.69475E+00_JPRB, 0.58875E+00_JPRB, 0.52747E+00_JPRB, 0.54268E+00_JPRB, 0.49602E+00_JPRB, 0.49646E+00_JPRB ,&
&  0.47754E+00_JPRB, 0.50376E+00_JPRB, 0.56852E+00_JPRB, 0.71194E+00_JPRB, 0.88661E+00_JPRB, 0.44510E+00_JPRB, 0.38946E+00_JPRB ,&
&  0.40852E+00_JPRB, 0.44963E+00_JPRB, 0.49886E+00_JPRB, 0.54305E+00_JPRB, 0.58493E+00_JPRB, 0.63061E+00_JPRB, 0.68045E+00_JPRB ,&
&  0.73676E+00_JPRB, 0.80452E+00_JPRB, 0.84887E+00_JPRB, 0.87805E+00_JPRB, 0.89758E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.90993E+00_JPRB, 0.94779E+00_JPRB, 0.99694E+00_JPRB, 0.10659E+01_JPRB, 0.11378E+01_JPRB, 0.11596E+01_JPRB, 0.11517E+01_JPRB ,&
&  0.11861E+01_JPRB, 0.12471E+01_JPRB, 0.13236E+01_JPRB, 0.13800E+01_JPRB, 0.14038E+01_JPRB, 0.14533E+01_JPRB, 0.15129E+01_JPRB ,&
&  0.15718E+01_JPRB, 0.16386E+01_JPRB, 0.17219E+01_JPRB, 0.18227E+01_JPRB, 0.19586E+01_JPRB, 0.21269E+01_JPRB, 0.22038E+01_JPRB ,&
&  0.17286E+01_JPRB, 0.12326E+01_JPRB, 0.16224E+01_JPRB, 0.15604E+01_JPRB, 0.13216E+01_JPRB, 0.11672E+01_JPRB, 0.11682E+01_JPRB ,&
&  0.12821E+01_JPRB, 0.13557E+01_JPRB, 0.13700E+01_JPRB, 0.13719E+01_JPRB, 0.13568E+01_JPRB, 0.13899E+01_JPRB, 0.14229E+01_JPRB ,&
&  0.14507E+01_JPRB, 0.14725E+01_JPRB, 0.14887E+01_JPRB, 0.15020E+01_JPRB, 0.15500E+01_JPRB, 0.16545E+01_JPRB, 0.18281E+01_JPRB ,&
&  0.20146E+01_JPRB, 0.21349E+01_JPRB, 0.22432E+01_JPRB, 0.22664E+01_JPRB, 0.21645E+01_JPRB, 0.20736E+01_JPRB, 0.20434E+01_JPRB ,&
&  0.20701E+01_JPRB, 0.21073E+01_JPRB, 0.21505E+01_JPRB, 0.21637E+01_JPRB, 0.21914E+01_JPRB, 0.19590E+01_JPRB, 0.18896E+01_JPRB ,&
&  0.18770E+01_JPRB, 0.17485E+01_JPRB, 0.16994E+01_JPRB, 0.17507E+01_JPRB, 0.12102E+01_JPRB, 0.11850E+01_JPRB, 0.11371E+01_JPRB ,&
&  0.13771E+01_JPRB, 0.15098E+01_JPRB, 0.14471E+01_JPRB, 0.12504E+01_JPRB, 0.10342E+01_JPRB, 0.71454E+00_JPRB, 0.68742E+00_JPRB ,&
&  0.74766E+00_JPRB, 0.79746E+00_JPRB, 0.66596E+00_JPRB, 0.61144E+00_JPRB, 0.50066E+00_JPRB, 0.44998E+00_JPRB, 0.46582E+00_JPRB ,&
&  0.50397E+00_JPRB, 0.50114E+00_JPRB, 0.53561E+00_JPRB, 0.58633E+00_JPRB, 0.61346E+00_JPRB, 0.45877E+00_JPRB, 0.40742E+00_JPRB ,&
&  0.42566E+00_JPRB, 0.46596E+00_JPRB, 0.52257E+00_JPRB, 0.58669E+00_JPRB, 0.62081E+00_JPRB, 0.65607E+00_JPRB, 0.69983E+00_JPRB ,&
&  0.76347E+00_JPRB, 0.81503E+00_JPRB, 0.85043E+00_JPRB, 0.85935E+00_JPRB, 0.88600E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.82734E+00_JPRB, 0.87551E+00_JPRB, 0.94326E+00_JPRB, 0.10469E+01_JPRB, 0.11940E+01_JPRB, 0.13352E+01_JPRB, 0.12342E+01_JPRB ,&
&  0.11149E+01_JPRB, 0.97905E+00_JPRB, 0.10221E+01_JPRB, 0.10642E+01_JPRB, 0.11044E+01_JPRB, 0.11727E+01_JPRB, 0.12649E+01_JPRB ,&
&  0.13643E+01_JPRB, 0.14571E+01_JPRB, 0.15465E+01_JPRB, 0.16206E+01_JPRB, 0.16629E+01_JPRB, 0.18067E+01_JPRB, 0.16839E+01_JPRB ,&
&  0.11519E+01_JPRB, 0.11031E+01_JPRB, 0.15867E+01_JPRB, 0.13501E+01_JPRB, 0.13370E+01_JPRB, 0.13199E+01_JPRB, 0.12339E+01_JPRB ,&
&  0.12579E+01_JPRB, 0.12995E+01_JPRB, 0.12873E+01_JPRB, 0.12771E+01_JPRB, 0.12507E+01_JPRB, 0.13135E+01_JPRB, 0.13791E+01_JPRB ,&
&  0.14167E+01_JPRB, 0.14398E+01_JPRB, 0.14615E+01_JPRB, 0.14794E+01_JPRB, 0.15230E+01_JPRB, 0.16249E+01_JPRB, 0.18292E+01_JPRB ,&
&  0.20061E+01_JPRB, 0.21714E+01_JPRB, 0.23372E+01_JPRB, 0.23519E+01_JPRB, 0.22733E+01_JPRB, 0.22531E+01_JPRB, 0.23036E+01_JPRB ,&
&  0.23627E+01_JPRB, 0.24669E+01_JPRB, 0.24891E+01_JPRB, 0.24111E+01_JPRB, 0.21888E+01_JPRB, 0.22903E+01_JPRB, 0.19938E+01_JPRB ,&
&  0.19898E+01_JPRB, 0.18473E+01_JPRB, 0.19562E+01_JPRB, 0.14760E+01_JPRB, 0.13490E+01_JPRB, 0.13357E+01_JPRB, 0.13183E+01_JPRB ,&
&  0.13191E+01_JPRB, 0.14767E+01_JPRB, 0.14993E+01_JPRB, 0.13291E+01_JPRB, 0.11323E+01_JPRB, 0.84429E+00_JPRB, 0.85214E+00_JPRB ,&
&  0.83314E+00_JPRB, 0.91923E+00_JPRB, 0.73529E+00_JPRB, 0.58022E+00_JPRB, 0.42677E+00_JPRB, 0.36639E+00_JPRB, 0.40251E+00_JPRB ,&
&  0.48385E+00_JPRB, 0.53297E+00_JPRB, 0.52132E+00_JPRB, 0.51105E+00_JPRB, 0.52202E+00_JPRB, 0.47879E+00_JPRB, 0.43473E+00_JPRB ,&
&  0.45519E+00_JPRB, 0.47925E+00_JPRB, 0.51524E+00_JPRB, 0.57464E+00_JPRB, 0.61348E+00_JPRB, 0.65793E+00_JPRB, 0.66706E+00_JPRB ,&
&  0.69974E+00_JPRB, 0.73023E+00_JPRB, 0.74466E+00_JPRB, 0.75979E+00_JPRB, 0.78597E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.67807E+00_JPRB, 0.69019E+00_JPRB, 0.72045E+00_JPRB, 0.77010E+00_JPRB, 0.81476E+00_JPRB, 0.85603E+00_JPRB, 0.86399E+00_JPRB ,&
&  0.84064E+00_JPRB, 0.81494E+00_JPRB, 0.81323E+00_JPRB, 0.79192E+00_JPRB, 0.78540E+00_JPRB, 0.79616E+00_JPRB, 0.85202E+00_JPRB ,&
&  0.97949E+00_JPRB, 0.11584E+01_JPRB, 0.13340E+01_JPRB, 0.14459E+01_JPRB, 0.14828E+01_JPRB, 0.15640E+01_JPRB, 0.13343E+01_JPRB ,&
&  0.92274E+00_JPRB, 0.15990E+01_JPRB, 0.13172E+01_JPRB, 0.13346E+01_JPRB, 0.13691E+01_JPRB, 0.14362E+01_JPRB, 0.12412E+01_JPRB ,&
&  0.11938E+01_JPRB, 0.12168E+01_JPRB, 0.12119E+01_JPRB, 0.12158E+01_JPRB, 0.12013E+01_JPRB, 0.12464E+01_JPRB, 0.13106E+01_JPRB ,&
&  0.13746E+01_JPRB, 0.14153E+01_JPRB, 0.14364E+01_JPRB, 0.14397E+01_JPRB, 0.14612E+01_JPRB, 0.15118E+01_JPRB, 0.16897E+01_JPRB ,&
&  0.18811E+01_JPRB, 0.21112E+01_JPRB, 0.23289E+01_JPRB, 0.25225E+01_JPRB, 0.24492E+01_JPRB, 0.24854E+01_JPRB, 0.26403E+01_JPRB ,&
&  0.26491E+01_JPRB, 0.25500E+01_JPRB, 0.27178E+01_JPRB, 0.27074E+01_JPRB, 0.26041E+01_JPRB, 0.26574E+01_JPRB, 0.21848E+01_JPRB ,&
&  0.21880E+01_JPRB, 0.19232E+01_JPRB, 0.20095E+01_JPRB, 0.15938E+01_JPRB, 0.14801E+01_JPRB, 0.14815E+01_JPRB, 0.14360E+01_JPRB ,&
&  0.13220E+01_JPRB, 0.14376E+01_JPRB, 0.14073E+01_JPRB, 0.13172E+01_JPRB, 0.12654E+01_JPRB, 0.11085E+01_JPRB, 0.10701E+01_JPRB ,&
&  0.10326E+01_JPRB, 0.97713E+00_JPRB, 0.83662E+00_JPRB, 0.50126E+00_JPRB, 0.39033E+00_JPRB, 0.32759E+00_JPRB, 0.35300E+00_JPRB ,&
&  0.43597E+00_JPRB, 0.50127E+00_JPRB, 0.55794E+00_JPRB, 0.52779E+00_JPRB, 0.50437E+00_JPRB, 0.48961E+00_JPRB, 0.47397E+00_JPRB ,&
&  0.47664E+00_JPRB, 0.48663E+00_JPRB, 0.49843E+00_JPRB, 0.54457E+00_JPRB, 0.60238E+00_JPRB, 0.64048E+00_JPRB, 0.62210E+00_JPRB ,&
&  0.60207E+00_JPRB, 0.60336E+00_JPRB, 0.62677E+00_JPRB, 0.64465E+00_JPRB, 0.66280E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.59060E+00_JPRB, 0.58104E+00_JPRB, 0.58075E+00_JPRB, 0.59828E+00_JPRB, 0.60254E+00_JPRB, 0.60340E+00_JPRB, 0.63981E+00_JPRB ,&
&  0.64872E+00_JPRB, 0.63684E+00_JPRB, 0.64647E+00_JPRB, 0.64828E+00_JPRB, 0.64186E+00_JPRB, 0.62162E+00_JPRB, 0.61634E+00_JPRB ,&
&  0.64046E+00_JPRB, 0.74548E+00_JPRB, 0.95418E+00_JPRB, 0.12038E+01_JPRB, 0.13657E+01_JPRB, 0.16318E+01_JPRB, 0.11085E+01_JPRB ,&
&  0.10003E+01_JPRB, 0.13563E+01_JPRB, 0.12557E+01_JPRB, 0.13538E+01_JPRB, 0.14355E+01_JPRB, 0.15280E+01_JPRB, 0.13171E+01_JPRB ,&
&  0.12550E+01_JPRB, 0.12490E+01_JPRB, 0.12082E+01_JPRB, 0.12158E+01_JPRB, 0.12286E+01_JPRB, 0.12397E+01_JPRB, 0.12453E+01_JPRB ,&
&  0.12813E+01_JPRB, 0.13408E+01_JPRB, 0.13843E+01_JPRB, 0.13940E+01_JPRB, 0.13999E+01_JPRB, 0.14257E+01_JPRB, 0.15678E+01_JPRB ,&
&  0.17578E+01_JPRB, 0.19682E+01_JPRB, 0.22565E+01_JPRB, 0.24697E+01_JPRB, 0.26152E+01_JPRB, 0.26868E+01_JPRB, 0.29655E+01_JPRB ,&
&  0.29356E+01_JPRB, 0.28072E+01_JPRB, 0.29871E+01_JPRB, 0.30908E+01_JPRB, 0.30749E+01_JPRB, 0.30364E+01_JPRB, 0.25857E+01_JPRB ,&
&  0.23658E+01_JPRB, 0.21926E+01_JPRB, 0.20801E+01_JPRB, 0.17538E+01_JPRB, 0.16081E+01_JPRB, 0.15770E+01_JPRB, 0.15883E+01_JPRB ,&
&  0.15089E+01_JPRB, 0.15156E+01_JPRB, 0.15275E+01_JPRB, 0.14039E+01_JPRB, 0.13375E+01_JPRB, 0.12324E+01_JPRB, 0.13129E+01_JPRB ,&
&  0.12673E+01_JPRB, 0.10952E+01_JPRB, 0.70110E+00_JPRB, 0.39509E+00_JPRB, 0.31848E+00_JPRB, 0.27384E+00_JPRB, 0.30885E+00_JPRB ,&
&  0.39265E+00_JPRB, 0.45657E+00_JPRB, 0.50410E+00_JPRB, 0.61893E+00_JPRB, 0.60286E+00_JPRB, 0.56943E+00_JPRB, 0.53820E+00_JPRB ,&
&  0.52719E+00_JPRB, 0.50941E+00_JPRB, 0.50130E+00_JPRB, 0.51071E+00_JPRB, 0.56063E+00_JPRB, 0.61233E+00_JPRB, 0.63356E+00_JPRB ,&
&  0.60758E+00_JPRB, 0.57784E+00_JPRB, 0.56420E+00_JPRB, 0.56374E+00_JPRB, 0.57304E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.53751E+00_JPRB, 0.53257E+00_JPRB, 0.53312E+00_JPRB, 0.54058E+00_JPRB, 0.54556E+00_JPRB, 0.52509E+00_JPRB, 0.49352E+00_JPRB ,&
&  0.49942E+00_JPRB, 0.48427E+00_JPRB, 0.50095E+00_JPRB, 0.51615E+00_JPRB, 0.53660E+00_JPRB, 0.53877E+00_JPRB, 0.53989E+00_JPRB ,&
&  0.53375E+00_JPRB, 0.56347E+00_JPRB, 0.71526E+00_JPRB, 0.96564E+00_JPRB, 0.11749E+01_JPRB, 0.12995E+01_JPRB, 0.12910E+01_JPRB ,&
&  0.12448E+01_JPRB, 0.13501E+01_JPRB, 0.13646E+01_JPRB, 0.16513E+01_JPRB, 0.18839E+01_JPRB, 0.22428E+01_JPRB, 0.14862E+01_JPRB ,&
&  0.13445E+01_JPRB, 0.13107E+01_JPRB, 0.12858E+01_JPRB, 0.12863E+01_JPRB, 0.13210E+01_JPRB, 0.13017E+01_JPRB, 0.12829E+01_JPRB ,&
&  0.12527E+01_JPRB, 0.12605E+01_JPRB, 0.12872E+01_JPRB, 0.13074E+01_JPRB, 0.13210E+01_JPRB, 0.13479E+01_JPRB, 0.14738E+01_JPRB ,&
&  0.16922E+01_JPRB, 0.19483E+01_JPRB, 0.22333E+01_JPRB, 0.25537E+01_JPRB, 0.27232E+01_JPRB, 0.29178E+01_JPRB, 0.33094E+01_JPRB ,&
&  0.34681E+01_JPRB, 0.35552E+01_JPRB, 0.34611E+01_JPRB, 0.33526E+01_JPRB, 0.32811E+01_JPRB, 0.34619E+01_JPRB, 0.30707E+01_JPRB ,&
&  0.26532E+01_JPRB, 0.23080E+01_JPRB, 0.20152E+01_JPRB, 0.18759E+01_JPRB, 0.17769E+01_JPRB, 0.17469E+01_JPRB, 0.15239E+01_JPRB ,&
&  0.13164E+01_JPRB, 0.12110E+01_JPRB, 0.12238E+01_JPRB, 0.13149E+01_JPRB, 0.14161E+01_JPRB, 0.14163E+01_JPRB, 0.16169E+01_JPRB ,&
&  0.12341E+01_JPRB, 0.63490E+00_JPRB, 0.39651E+00_JPRB, 0.38114E+00_JPRB, 0.26991E+00_JPRB, 0.22319E+00_JPRB, 0.31327E+00_JPRB ,&
&  0.40476E+00_JPRB, 0.49598E+00_JPRB, 0.51845E+00_JPRB, 0.56914E+00_JPRB, 0.67470E+00_JPRB, 0.67477E+00_JPRB, 0.63851E+00_JPRB ,&
&  0.58903E+00_JPRB, 0.55811E+00_JPRB, 0.54546E+00_JPRB, 0.54004E+00_JPRB, 0.57545E+00_JPRB, 0.62167E+00_JPRB, 0.64808E+00_JPRB ,&
&  0.63663E+00_JPRB, 0.60440E+00_JPRB, 0.55891E+00_JPRB, 0.52459E+00_JPRB, 0.52123E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.53298E+00_JPRB, 0.54787E+00_JPRB, 0.56169E+00_JPRB, 0.54829E+00_JPRB, 0.52484E+00_JPRB, 0.51716E+00_JPRB, 0.48023E+00_JPRB ,&
&  0.45599E+00_JPRB, 0.43968E+00_JPRB, 0.43087E+00_JPRB, 0.44001E+00_JPRB, 0.46267E+00_JPRB, 0.47986E+00_JPRB, 0.47749E+00_JPRB ,&
&  0.46887E+00_JPRB, 0.49894E+00_JPRB, 0.59439E+00_JPRB, 0.82641E+00_JPRB, 0.10977E+01_JPRB, 0.13692E+01_JPRB, 0.14206E+01_JPRB ,&
&  0.13044E+01_JPRB, 0.17528E+01_JPRB, 0.24369E+01_JPRB, 0.27928E+01_JPRB, 0.26132E+01_JPRB, 0.27603E+01_JPRB, 0.19675E+01_JPRB ,&
&  0.16405E+01_JPRB, 0.16015E+01_JPRB, 0.15660E+01_JPRB, 0.15412E+01_JPRB, 0.14868E+01_JPRB, 0.14058E+01_JPRB, 0.13621E+01_JPRB ,&
&  0.13233E+01_JPRB, 0.12476E+01_JPRB, 0.12217E+01_JPRB, 0.12200E+01_JPRB, 0.12407E+01_JPRB, 0.12745E+01_JPRB, 0.13935E+01_JPRB ,&
&  0.16172E+01_JPRB, 0.18752E+01_JPRB, 0.21759E+01_JPRB, 0.28795E+01_JPRB, 0.24615E+01_JPRB, 0.32444E+01_JPRB, 0.35937E+01_JPRB ,&
&  0.37447E+01_JPRB, 0.37965E+01_JPRB, 0.34887E+01_JPRB, 0.38490E+01_JPRB, 0.37531E+01_JPRB, 0.37806E+01_JPRB, 0.34389E+01_JPRB ,&
&  0.29842E+01_JPRB, 0.24269E+01_JPRB, 0.21564E+01_JPRB, 0.19845E+01_JPRB, 0.18472E+01_JPRB, 0.18233E+01_JPRB, 0.12735E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.12975E+01_JPRB, 0.12755E+01_JPRB, 0.11730E+01_JPRB, 0.13967E+01_JPRB, 0.15399E+01_JPRB, 0.12019E+01_JPRB ,&
&  0.58277E+00_JPRB, 0.45942E+00_JPRB, 0.40134E+00_JPRB, 0.35146E+00_JPRB, 0.30373E+00_JPRB, 0.28717E+00_JPRB, 0.53892E+00_JPRB ,&
&  0.45690E+00_JPRB, 0.58162E+00_JPRB, 0.59430E+00_JPRB, 0.64988E+00_JPRB, 0.82984E+00_JPRB, 0.89191E+00_JPRB, 0.13399E+01_JPRB ,&
&  0.12083E+01_JPRB, 0.10160E+01_JPRB, 0.86738E+00_JPRB, 0.75848E+00_JPRB, 0.71256E+00_JPRB, 0.69896E+00_JPRB, 0.69255E+00_JPRB ,&
&  0.68162E+00_JPRB, 0.64628E+00_JPRB, 0.61275E+00_JPRB, 0.56587E+00_JPRB, 0.52701E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.56250E+00_JPRB, 0.56555E+00_JPRB, 0.56843E+00_JPRB, 0.56487E+00_JPRB, 0.55170E+00_JPRB, 0.54778E+00_JPRB, 0.52258E+00_JPRB ,&
&  0.48533E+00_JPRB, 0.46023E+00_JPRB, 0.43790E+00_JPRB, 0.43990E+00_JPRB, 0.44993E+00_JPRB, 0.45914E+00_JPRB, 0.45073E+00_JPRB ,&
&  0.43962E+00_JPRB, 0.46636E+00_JPRB, 0.61945E+00_JPRB, 0.73805E+00_JPRB, 0.10532E+01_JPRB, 0.13353E+01_JPRB, 0.15393E+01_JPRB ,&
&  0.14679E+01_JPRB, 0.19211E+01_JPRB, 0.26663E+01_JPRB, 0.30482E+01_JPRB, 0.40375E+01_JPRB, 0.39952E+01_JPRB, 0.34065E+01_JPRB ,&
&  0.25692E+01_JPRB, 0.24424E+01_JPRB, 0.23396E+01_JPRB, 0.21420E+01_JPRB, 0.18899E+01_JPRB, 0.16441E+01_JPRB, 0.14623E+01_JPRB ,&
&  0.13570E+01_JPRB, 0.13037E+01_JPRB, 0.12173E+01_JPRB, 0.11687E+01_JPRB, 0.11774E+01_JPRB, 0.12068E+01_JPRB, 0.13313E+01_JPRB ,&
&  0.15441E+01_JPRB, 0.17881E+01_JPRB, 0.20607E+01_JPRB, 0.27403E+01_JPRB, 0.28109E+01_JPRB, 0.30636E+01_JPRB, 0.36916E+01_JPRB ,&
&  0.39588E+01_JPRB, 0.41301E+01_JPRB, 0.37575E+01_JPRB, 0.45645E+01_JPRB, 0.41252E+01_JPRB, 0.40942E+01_JPRB, 0.39515E+01_JPRB ,&
&  0.33567E+01_JPRB, 0.29117E+01_JPRB, 0.22110E+01_JPRB, 0.19978E+01_JPRB, 0.19185E+01_JPRB, 0.15567E+01_JPRB, 0.12920E+01_JPRB ,&
&  0.13933E+01_JPRB, 0.13187E+01_JPRB, 0.12489E+01_JPRB, 0.11335E+01_JPRB, 0.13739E+01_JPRB, 0.13185E+01_JPRB, 0.70117E+00_JPRB ,&
&  0.54233E+00_JPRB, 0.52251E+00_JPRB, 0.42334E+00_JPRB, 0.37078E+00_JPRB, 0.32704E+00_JPRB, 0.31299E+00_JPRB, 0.43911E+00_JPRB ,&
&  0.46631E+00_JPRB, 0.63583E+00_JPRB, 0.71458E+00_JPRB, 0.84070E+00_JPRB, 0.96811E+00_JPRB, 0.10815E+01_JPRB, 0.18756E+01_JPRB ,&
&  0.18526E+01_JPRB, 0.15595E+01_JPRB, 0.12541E+01_JPRB, 0.10884E+01_JPRB, 0.94952E+00_JPRB, 0.83497E+00_JPRB, 0.75902E+00_JPRB ,&
&  0.72096E+00_JPRB, 0.67222E+00_JPRB, 0.64593E+00_JPRB, 0.62913E+00_JPRB, 0.58683E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.57111E+00_JPRB, 0.55992E+00_JPRB, 0.55814E+00_JPRB, 0.54299E+00_JPRB, 0.52859E+00_JPRB, 0.52424E+00_JPRB, 0.53534E+00_JPRB ,&
&  0.51821E+00_JPRB, 0.49280E+00_JPRB, 0.46757E+00_JPRB, 0.45726E+00_JPRB, 0.46385E+00_JPRB, 0.46830E+00_JPRB, 0.45354E+00_JPRB ,&
&  0.44471E+00_JPRB, 0.47174E+00_JPRB, 0.60199E+00_JPRB, 0.70167E+00_JPRB, 0.88829E+00_JPRB, 0.11514E+01_JPRB, 0.13822E+01_JPRB ,&
&  0.15874E+01_JPRB, 0.20389E+01_JPRB, 0.26596E+01_JPRB, 0.36773E+01_JPRB, 0.49226E+01_JPRB, 0.54769E+01_JPRB, 0.58280E+01_JPRB ,&
&  0.48003E+01_JPRB, 0.38232E+01_JPRB, 0.32344E+01_JPRB, 0.28164E+01_JPRB, 0.23445E+01_JPRB, 0.19555E+01_JPRB, 0.16476E+01_JPRB ,&
&  0.14478E+01_JPRB, 0.12953E+01_JPRB, 0.12544E+01_JPRB, 0.11818E+01_JPRB, 0.11502E+01_JPRB, 0.11619E+01_JPRB, 0.12882E+01_JPRB ,&
&  0.14752E+01_JPRB, 0.17272E+01_JPRB, 0.20846E+01_JPRB, 0.25563E+01_JPRB, 0.31849E+01_JPRB, 0.38450E+01_JPRB, 0.39645E+01_JPRB ,&
&  0.37281E+01_JPRB, 0.35873E+01_JPRB, 0.41754E+01_JPRB, 0.47174E+01_JPRB, 0.43771E+01_JPRB, 0.45285E+01_JPRB, 0.42162E+01_JPRB ,&
&  0.35837E+01_JPRB, 0.30073E+01_JPRB, 0.23126E+01_JPRB, 0.21847E+01_JPRB, 0.17758E+01_JPRB, 0.13980E+01_JPRB, 0.14884E+01_JPRB ,&
&  0.15487E+01_JPRB, 0.14356E+01_JPRB, 0.12563E+01_JPRB, 0.10736E+01_JPRB, 0.10756E+01_JPRB, 0.87873E+00_JPRB, 0.66388E+00_JPRB ,&
&  0.59583E+00_JPRB, 0.52242E+00_JPRB, 0.45490E+00_JPRB, 0.45662E+00_JPRB, 0.39698E+00_JPRB, 0.39090E+00_JPRB, 0.48904E+00_JPRB ,&
&  0.58556E+00_JPRB, 0.65757E+00_JPRB, 0.85130E+00_JPRB, 0.94719E+00_JPRB, 0.96292E+00_JPRB, 0.91234E+00_JPRB, 0.13632E+01_JPRB ,&
&  0.11058E+01_JPRB, 0.11720E+01_JPRB, 0.11565E+01_JPRB, 0.10654E+01_JPRB, 0.98004E+00_JPRB, 0.89375E+00_JPRB, 0.80381E+00_JPRB ,&
&  0.73224E+00_JPRB, 0.67125E+00_JPRB, 0.62975E+00_JPRB, 0.61541E+00_JPRB, 0.59661E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.55740E+00_JPRB, 0.54216E+00_JPRB, 0.51816E+00_JPRB, 0.48198E+00_JPRB, 0.46882E+00_JPRB, 0.47853E+00_JPRB, 0.50226E+00_JPRB ,&
&  0.51024E+00_JPRB, 0.51199E+00_JPRB, 0.50030E+00_JPRB, 0.48563E+00_JPRB, 0.48529E+00_JPRB, 0.47853E+00_JPRB, 0.46071E+00_JPRB ,&
&  0.45395E+00_JPRB, 0.48961E+00_JPRB, 0.43063E+00_JPRB, 0.48007E+00_JPRB, 0.66285E+00_JPRB, 0.10229E+01_JPRB, 0.10572E+01_JPRB ,&
&  0.14953E+01_JPRB, 0.20218E+01_JPRB, 0.25273E+01_JPRB, 0.37237E+01_JPRB, 0.66387E+01_JPRB, 0.68151E+01_JPRB, 0.65363E+01_JPRB ,&
&  0.60227E+01_JPRB, 0.48925E+01_JPRB, 0.38641E+01_JPRB, 0.30566E+01_JPRB, 0.24549E+01_JPRB, 0.20542E+01_JPRB, 0.17513E+01_JPRB ,&
&  0.15504E+01_JPRB, 0.13747E+01_JPRB, 0.12544E+01_JPRB, 0.12244E+01_JPRB, 0.11890E+01_JPRB, 0.11691E+01_JPRB, 0.12664E+01_JPRB ,&
&  0.14126E+01_JPRB, 0.16504E+01_JPRB, 0.21505E+01_JPRB, 0.27342E+01_JPRB, 0.30501E+01_JPRB, 0.35695E+01_JPRB, 0.46646E+01_JPRB ,&
&  0.42321E+01_JPRB, 0.41593E+01_JPRB, 0.40407E+01_JPRB, 0.50006E+01_JPRB, 0.48242E+01_JPRB, 0.45849E+01_JPRB, 0.44445E+01_JPRB ,&
&  0.31958E+01_JPRB, 0.25244E+01_JPRB, 0.22584E+01_JPRB, 0.20732E+01_JPRB, 0.16008E+01_JPRB, 0.16154E+01_JPRB, 0.20693E+01_JPRB ,&
&  0.16531E+01_JPRB, 0.16166E+01_JPRB, 0.16098E+01_JPRB, 0.15672E+01_JPRB, 0.10498E+01_JPRB, 0.76816E+00_JPRB, 0.10343E+01_JPRB ,&
&  0.99156E+00_JPRB, 0.63819E+00_JPRB, 0.51553E+00_JPRB, 0.59754E+00_JPRB, 0.49171E+00_JPRB, 0.52295E+00_JPRB, 0.64657E+00_JPRB ,&
&  0.65088E+00_JPRB, 0.68924E+00_JPRB, 0.97653E+00_JPRB, 0.10261E+01_JPRB, 0.98400E+00_JPRB, 0.87096E+00_JPRB, 0.10262E+01_JPRB ,&
&  0.10671E+01_JPRB, 0.10444E+01_JPRB, 0.10850E+01_JPRB, 0.10015E+01_JPRB, 0.92786E+00_JPRB, 0.87005E+00_JPRB, 0.80526E+00_JPRB ,&
&  0.75147E+00_JPRB, 0.68728E+00_JPRB, 0.62894E+00_JPRB, 0.58545E+00_JPRB, 0.56889E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.54197E+00_JPRB, 0.52657E+00_JPRB, 0.50768E+00_JPRB, 0.48383E+00_JPRB, 0.45859E+00_JPRB, 0.45198E+00_JPRB, 0.45962E+00_JPRB ,&
&  0.47056E+00_JPRB, 0.48541E+00_JPRB, 0.49289E+00_JPRB, 0.49557E+00_JPRB, 0.49513E+00_JPRB, 0.48876E+00_JPRB, 0.46480E+00_JPRB ,&
&  0.45752E+00_JPRB, 0.44919E+00_JPRB, 0.38480E+00_JPRB, 0.39538E+00_JPRB, 0.59334E+00_JPRB, 0.82612E+00_JPRB, 0.93540E+00_JPRB ,&
&  0.13104E+01_JPRB, 0.18511E+01_JPRB, 0.25208E+01_JPRB, 0.38102E+01_JPRB, 0.51647E+01_JPRB, 0.59837E+01_JPRB, 0.56457E+01_JPRB ,&
&  0.55987E+01_JPRB, 0.49908E+01_JPRB, 0.41205E+01_JPRB, 0.34092E+01_JPRB, 0.27687E+01_JPRB, 0.22707E+01_JPRB, 0.19014E+01_JPRB ,&
&  0.16614E+01_JPRB, 0.14939E+01_JPRB, 0.13513E+01_JPRB, 0.12669E+01_JPRB, 0.12509E+01_JPRB, 0.12248E+01_JPRB, 0.12730E+01_JPRB ,&
&  0.13526E+01_JPRB, 0.15309E+01_JPRB, 0.21156E+01_JPRB, 0.30130E+01_JPRB, 0.35156E+01_JPRB, 0.37415E+01_JPRB, 0.40254E+01_JPRB ,&
&  0.41308E+01_JPRB, 0.39792E+01_JPRB, 0.42041E+01_JPRB, 0.43379E+01_JPRB, 0.46057E+01_JPRB, 0.44583E+01_JPRB, 0.46901E+01_JPRB ,&
&  0.37558E+01_JPRB, 0.29929E+01_JPRB, 0.23223E+01_JPRB, 0.18598E+01_JPRB, 0.17444E+01_JPRB, 0.21864E+01_JPRB, 0.21703E+01_JPRB ,&
&  0.19056E+01_JPRB, 0.17877E+01_JPRB, 0.16353E+01_JPRB, 0.14156E+01_JPRB, 0.13202E+01_JPRB, 0.86959E+00_JPRB, 0.94125E+00_JPRB ,&
&  0.97590E+00_JPRB, 0.96667E+00_JPRB, 0.88311E+00_JPRB, 0.76871E+00_JPRB, 0.68917E+00_JPRB, 0.70315E+00_JPRB, 0.70503E+00_JPRB ,&
&  0.70364E+00_JPRB, 0.64878E+00_JPRB, 0.77551E+00_JPRB, 0.91768E+00_JPRB, 0.98382E+00_JPRB, 0.85530E+00_JPRB, 0.96015E+00_JPRB ,&
&  0.93844E+00_JPRB, 0.10056E+01_JPRB, 0.10038E+01_JPRB, 0.95740E+00_JPRB, 0.88502E+00_JPRB, 0.82064E+00_JPRB, 0.77282E+00_JPRB ,&
&  0.72509E+00_JPRB, 0.67183E+00_JPRB, 0.61007E+00_JPRB, 0.56745E+00_JPRB, 0.55237E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.56642E+00_JPRB, 0.54575E+00_JPRB, 0.51934E+00_JPRB, 0.49683E+00_JPRB, 0.47469E+00_JPRB, 0.46501E+00_JPRB, 0.46592E+00_JPRB ,&
&  0.47232E+00_JPRB, 0.47687E+00_JPRB, 0.48743E+00_JPRB, 0.49796E+00_JPRB, 0.50374E+00_JPRB, 0.50051E+00_JPRB, 0.47371E+00_JPRB ,&
&  0.47504E+00_JPRB, 0.47862E+00_JPRB, 0.38001E+00_JPRB, 0.40210E+00_JPRB, 0.51313E+00_JPRB, 0.63589E+00_JPRB, 0.87710E+00_JPRB ,&
&  0.10957E+01_JPRB, 0.15249E+01_JPRB, 0.20696E+01_JPRB, 0.29453E+01_JPRB, 0.35395E+01_JPRB, 0.38559E+01_JPRB, 0.38776E+01_JPRB ,&
&  0.35243E+01_JPRB, 0.34832E+01_JPRB, 0.32374E+01_JPRB, 0.28711E+01_JPRB, 0.25172E+01_JPRB, 0.22477E+01_JPRB, 0.20266E+01_JPRB ,&
&  0.18312E+01_JPRB, 0.16658E+01_JPRB, 0.15441E+01_JPRB, 0.14435E+01_JPRB, 0.13977E+01_JPRB, 0.13699E+01_JPRB, 0.13724E+01_JPRB ,&
&  0.13458E+01_JPRB, 0.13777E+01_JPRB, 0.17586E+01_JPRB, 0.27391E+01_JPRB, 0.32252E+01_JPRB, 0.37628E+01_JPRB, 0.38309E+01_JPRB ,&
&  0.45151E+01_JPRB, 0.46824E+01_JPRB, 0.45852E+01_JPRB, 0.51739E+01_JPRB, 0.44242E+01_JPRB, 0.45409E+01_JPRB, 0.45907E+01_JPRB ,&
&  0.47603E+01_JPRB, 0.42326E+01_JPRB, 0.36240E+01_JPRB, 0.28815E+01_JPRB, 0.21702E+01_JPRB, 0.24782E+01_JPRB, 0.21804E+01_JPRB ,&
&  0.18751E+01_JPRB, 0.18091E+01_JPRB, 0.16723E+01_JPRB, 0.15728E+01_JPRB, 0.13014E+01_JPRB, 0.11260E+01_JPRB, 0.83777E+00_JPRB ,&
&  0.82472E+00_JPRB, 0.74870E+00_JPRB, 0.74667E+00_JPRB, 0.68659E+00_JPRB, 0.66267E+00_JPRB, 0.60793E+00_JPRB, 0.56729E+00_JPRB ,&
&  0.58000E+00_JPRB, 0.53451E+00_JPRB, 0.54703E+00_JPRB, 0.61355E+00_JPRB, 0.78426E+00_JPRB, 0.76294E+00_JPRB, 0.83772E+00_JPRB ,&
&  0.86869E+00_JPRB, 0.90751E+00_JPRB, 0.82071E+00_JPRB, 0.84602E+00_JPRB, 0.79983E+00_JPRB, 0.75538E+00_JPRB, 0.73227E+00_JPRB ,&
&  0.68062E+00_JPRB, 0.64455E+00_JPRB, 0.61320E+00_JPRB, 0.58025E+00_JPRB, 0.57374E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.53923E+00_JPRB, 0.51511E+00_JPRB, 0.50132E+00_JPRB, 0.49363E+00_JPRB, 0.47796E+00_JPRB, 0.46907E+00_JPRB, 0.48186E+00_JPRB ,&
&  0.48739E+00_JPRB, 0.47942E+00_JPRB, 0.47673E+00_JPRB, 0.48161E+00_JPRB, 0.48986E+00_JPRB, 0.49371E+00_JPRB, 0.48259E+00_JPRB ,&
&  0.48495E+00_JPRB, 0.62832E+00_JPRB, 0.46650E+00_JPRB, 0.37860E+00_JPRB, 0.36415E+00_JPRB, 0.48105E+00_JPRB, 0.66184E+00_JPRB ,&
&  0.89535E+00_JPRB, 0.11467E+01_JPRB, 0.15735E+01_JPRB, 0.18985E+01_JPRB, 0.21841E+01_JPRB, 0.23548E+01_JPRB, 0.24573E+01_JPRB ,&
&  0.25485E+01_JPRB, 0.19758E+01_JPRB, 0.18840E+01_JPRB, 0.18068E+01_JPRB, 0.17803E+01_JPRB, 0.17395E+01_JPRB, 0.17306E+01_JPRB ,&
&  0.16821E+01_JPRB, 0.16412E+01_JPRB, 0.15883E+01_JPRB, 0.15281E+01_JPRB, 0.14662E+01_JPRB, 0.14125E+01_JPRB, 0.13657E+01_JPRB ,&
&  0.13578E+01_JPRB, 0.13436E+01_JPRB, 0.14406E+01_JPRB, 0.19477E+01_JPRB, 0.29393E+01_JPRB, 0.37062E+01_JPRB, 0.43077E+01_JPRB ,&
&  0.40952E+01_JPRB, 0.38125E+01_JPRB, 0.52426E+01_JPRB, 0.55279E+01_JPRB, 0.55806E+01_JPRB, 0.57096E+01_JPRB, 0.52355E+01_JPRB ,&
&  0.50351E+01_JPRB, 0.44357E+01_JPRB, 0.37164E+01_JPRB, 0.29746E+01_JPRB, 0.28256E+01_JPRB, 0.26092E+01_JPRB, 0.23278E+01_JPRB ,&
&  0.20542E+01_JPRB, 0.20374E+01_JPRB, 0.18426E+01_JPRB, 0.16060E+01_JPRB, 0.14441E+01_JPRB, 0.13879E+01_JPRB, 0.10896E+01_JPRB ,&
&  0.85998E+00_JPRB, 0.78482E+00_JPRB, 0.62445E+00_JPRB, 0.53389E+00_JPRB, 0.48404E+00_JPRB, 0.43549E+00_JPRB, 0.41354E+00_JPRB ,&
&  0.40949E+00_JPRB, 0.41603E+00_JPRB, 0.43685E+00_JPRB, 0.47437E+00_JPRB, 0.58882E+00_JPRB, 0.65367E+00_JPRB, 0.68021E+00_JPRB ,&
&  0.64025E+00_JPRB, 0.72529E+00_JPRB, 0.75800E+00_JPRB, 0.76251E+00_JPRB, 0.79794E+00_JPRB, 0.72243E+00_JPRB, 0.69227E+00_JPRB ,&
&  0.64596E+00_JPRB, 0.62715E+00_JPRB, 0.62116E+00_JPRB, 0.59370E+00_JPRB, 0.57050E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.54262E+00_JPRB, 0.52549E+00_JPRB, 0.51706E+00_JPRB, 0.50711E+00_JPRB, 0.50122E+00_JPRB, 0.49735E+00_JPRB, 0.50575E+00_JPRB ,&
&  0.50315E+00_JPRB, 0.49846E+00_JPRB, 0.49225E+00_JPRB, 0.49007E+00_JPRB, 0.49138E+00_JPRB, 0.49081E+00_JPRB, 0.48990E+00_JPRB ,&
&  0.50599E+00_JPRB, 0.75621E+00_JPRB, 0.58733E+00_JPRB, 0.49996E+00_JPRB, 0.42308E+00_JPRB, 0.48443E+00_JPRB, 0.58779E+00_JPRB ,&
&  0.76801E+00_JPRB, 0.85900E+00_JPRB, 0.10334E+01_JPRB, 0.12416E+01_JPRB, 0.14452E+01_JPRB, 0.13948E+01_JPRB, 0.14739E+01_JPRB ,&
&  0.13610E+01_JPRB, 0.12583E+01_JPRB, 0.10535E+01_JPRB, 0.11580E+01_JPRB, 0.11636E+01_JPRB, 0.10593E+01_JPRB, 0.12720E+01_JPRB ,&
&  0.13567E+01_JPRB, 0.14232E+01_JPRB, 0.14716E+01_JPRB, 0.14974E+01_JPRB, 0.15063E+01_JPRB, 0.14726E+01_JPRB, 0.14211E+01_JPRB ,&
&  0.13477E+01_JPRB, 0.13337E+01_JPRB, 0.14123E+01_JPRB, 0.15540E+01_JPRB, 0.19813E+01_JPRB, 0.26106E+01_JPRB, 0.38066E+01_JPRB ,&
&  0.48619E+01_JPRB, 0.45223E+01_JPRB, 0.41499E+01_JPRB, 0.50649E+01_JPRB, 0.56323E+01_JPRB, 0.51380E+01_JPRB, 0.45167E+01_JPRB ,&
&  0.43996E+01_JPRB, 0.41890E+01_JPRB, 0.38064E+01_JPRB, 0.34425E+01_JPRB, 0.30954E+01_JPRB, 0.28365E+01_JPRB, 0.25594E+01_JPRB ,&
&  0.22635E+01_JPRB, 0.21643E+01_JPRB, 0.20636E+01_JPRB, 0.17455E+01_JPRB, 0.14402E+01_JPRB, 0.12711E+01_JPRB, 0.11179E+01_JPRB ,&
&  0.96390E+00_JPRB, 0.63726E+00_JPRB, 0.43872E+00_JPRB, 0.34528E+00_JPRB, 0.32034E+00_JPRB, 0.29738E+00_JPRB, 0.30188E+00_JPRB ,&
&  0.29965E+00_JPRB, 0.33370E+00_JPRB, 0.40284E+00_JPRB, 0.43261E+00_JPRB, 0.55761E+00_JPRB, 0.61522E+00_JPRB, 0.57847E+00_JPRB ,&
&  0.51923E+00_JPRB, 0.51845E+00_JPRB, 0.72142E+00_JPRB, 0.75315E+00_JPRB, 0.76356E+00_JPRB, 0.70075E+00_JPRB, 0.68048E+00_JPRB ,&
&  0.65383E+00_JPRB, 0.64355E+00_JPRB, 0.63061E+00_JPRB, 0.59852E+00_JPRB, 0.56458E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.54308E+00_JPRB, 0.53799E+00_JPRB, 0.53229E+00_JPRB, 0.52008E+00_JPRB, 0.51116E+00_JPRB, 0.49774E+00_JPRB, 0.50200E+00_JPRB ,&
&  0.50202E+00_JPRB, 0.50230E+00_JPRB, 0.49903E+00_JPRB, 0.49901E+00_JPRB, 0.50077E+00_JPRB, 0.49946E+00_JPRB, 0.50101E+00_JPRB ,&
&  0.56971E+00_JPRB, 0.73421E+00_JPRB, 0.63435E+00_JPRB, 0.49316E+00_JPRB, 0.56182E+00_JPRB, 0.55252E+00_JPRB, 0.60468E+00_JPRB ,&
&  0.64283E+00_JPRB, 0.72558E+00_JPRB, 0.76204E+00_JPRB, 0.78814E+00_JPRB, 0.86435E+00_JPRB, 0.93224E+00_JPRB, 0.93003E+00_JPRB ,&
&  0.85647E+00_JPRB, 0.76177E+00_JPRB, 0.72058E+00_JPRB, 0.72509E+00_JPRB, 0.82269E+00_JPRB, 0.77054E+00_JPRB, 0.90501E+00_JPRB ,&
&  0.99330E+00_JPRB, 0.10751E+01_JPRB, 0.11539E+01_JPRB, 0.12394E+01_JPRB, 0.13054E+01_JPRB, 0.13608E+01_JPRB, 0.13814E+01_JPRB ,&
&  0.13775E+01_JPRB, 0.13825E+01_JPRB, 0.14040E+01_JPRB, 0.15153E+01_JPRB, 0.17514E+01_JPRB, 0.24468E+01_JPRB, 0.40093E+01_JPRB ,&
&  0.47326E+01_JPRB, 0.45978E+01_JPRB, 0.51573E+01_JPRB, 0.55269E+01_JPRB, 0.50517E+01_JPRB, 0.44958E+01_JPRB, 0.40326E+01_JPRB ,&
&  0.37903E+01_JPRB, 0.36805E+01_JPRB, 0.36215E+01_JPRB, 0.35494E+01_JPRB, 0.33842E+01_JPRB, 0.30410E+01_JPRB, 0.27301E+01_JPRB ,&
&  0.23964E+01_JPRB, 0.22660E+01_JPRB, 0.21741E+01_JPRB, 0.18068E+01_JPRB, 0.14085E+01_JPRB, 0.11399E+01_JPRB, 0.98196E+00_JPRB ,&
&  0.86068E+00_JPRB, 0.51934E+00_JPRB, 0.37409E+00_JPRB, 0.29469E+00_JPRB, 0.27963E+00_JPRB, 0.27322E+00_JPRB, 0.31405E+00_JPRB ,&
&  0.29683E+00_JPRB, 0.34532E+00_JPRB, 0.42641E+00_JPRB, 0.45796E+00_JPRB, 0.48999E+00_JPRB, 0.51689E+00_JPRB, 0.42288E+00_JPRB ,&
&  0.39422E+00_JPRB, 0.46151E+00_JPRB, 0.67159E+00_JPRB, 0.70843E+00_JPRB, 0.72609E+00_JPRB, 0.70180E+00_JPRB, 0.69612E+00_JPRB ,&
&  0.66493E+00_JPRB, 0.64055E+00_JPRB, 0.59879E+00_JPRB, 0.56672E+00_JPRB, 0.55344E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.56726E+00_JPRB, 0.55212E+00_JPRB, 0.54146E+00_JPRB, 0.52966E+00_JPRB, 0.50900E+00_JPRB, 0.49136E+00_JPRB, 0.49456E+00_JPRB ,&
&  0.49299E+00_JPRB, 0.49039E+00_JPRB, 0.49339E+00_JPRB, 0.49502E+00_JPRB, 0.49236E+00_JPRB, 0.49794E+00_JPRB, 0.53002E+00_JPRB ,&
&  0.45262E+00_JPRB, 0.52300E+00_JPRB, 0.45722E+00_JPRB, 0.47295E+00_JPRB, 0.65830E+00_JPRB, 0.59325E+00_JPRB, 0.59378E+00_JPRB ,&
&  0.58729E+00_JPRB, 0.60533E+00_JPRB, 0.59635E+00_JPRB, 0.59524E+00_JPRB, 0.60466E+00_JPRB, 0.65754E+00_JPRB, 0.68285E+00_JPRB ,&
&  0.60288E+00_JPRB, 0.53220E+00_JPRB, 0.52440E+00_JPRB, 0.53332E+00_JPRB, 0.58676E+00_JPRB, 0.67039E+00_JPRB, 0.78240E+00_JPRB ,&
&  0.83455E+00_JPRB, 0.88137E+00_JPRB, 0.94906E+00_JPRB, 0.10197E+01_JPRB, 0.10843E+01_JPRB, 0.11473E+01_JPRB, 0.11773E+01_JPRB ,&
&  0.11860E+01_JPRB, 0.12310E+01_JPRB, 0.13020E+01_JPRB, 0.14587E+01_JPRB, 0.15773E+01_JPRB, 0.24239E+01_JPRB, 0.42881E+01_JPRB ,&
&  0.50089E+01_JPRB, 0.42585E+01_JPRB, 0.42583E+01_JPRB, 0.48754E+01_JPRB, 0.46823E+01_JPRB, 0.42170E+01_JPRB, 0.36543E+01_JPRB ,&
&  0.32831E+01_JPRB, 0.30854E+01_JPRB, 0.29602E+01_JPRB, 0.29420E+01_JPRB, 0.29490E+01_JPRB, 0.28998E+01_JPRB, 0.27758E+01_JPRB ,&
&  0.24999E+01_JPRB, 0.23251E+01_JPRB, 0.21824E+01_JPRB, 0.18561E+01_JPRB, 0.14875E+01_JPRB, 0.11887E+01_JPRB, 0.91680E+00_JPRB ,&
&  0.72569E+00_JPRB, 0.54375E+00_JPRB, 0.40912E+00_JPRB, 0.34767E+00_JPRB, 0.32770E+00_JPRB, 0.33805E+00_JPRB, 0.41901E+00_JPRB ,&
&  0.35670E+00_JPRB, 0.38291E+00_JPRB, 0.39827E+00_JPRB, 0.40423E+00_JPRB, 0.40710E+00_JPRB, 0.39657E+00_JPRB, 0.37102E+00_JPRB ,&
&  0.39206E+00_JPRB, 0.52238E+00_JPRB, 0.63939E+00_JPRB, 0.67098E+00_JPRB, 0.68096E+00_JPRB, 0.65839E+00_JPRB, 0.58252E+00_JPRB ,&
&  0.67299E+00_JPRB, 0.59658E+00_JPRB, 0.56626E+00_JPRB, 0.56517E+00_JPRB, 0.57874E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.56347E+00_JPRB, 0.55383E+00_JPRB, 0.54154E+00_JPRB, 0.52190E+00_JPRB, 0.49871E+00_JPRB, 0.48064E+00_JPRB, 0.47484E+00_JPRB ,&
&  0.47878E+00_JPRB, 0.48174E+00_JPRB, 0.48423E+00_JPRB, 0.48194E+00_JPRB, 0.47270E+00_JPRB, 0.49227E+00_JPRB, 0.44517E+00_JPRB ,&
&  0.39257E+00_JPRB, 0.41851E+00_JPRB, 0.41294E+00_JPRB, 0.44146E+00_JPRB, 0.48458E+00_JPRB, 0.47340E+00_JPRB, 0.47292E+00_JPRB ,&
&  0.48816E+00_JPRB, 0.49032E+00_JPRB, 0.50180E+00_JPRB, 0.50817E+00_JPRB, 0.51131E+00_JPRB, 0.54461E+00_JPRB, 0.59592E+00_JPRB ,&
&  0.55533E+00_JPRB, 0.50930E+00_JPRB, 0.48911E+00_JPRB, 0.50813E+00_JPRB, 0.58980E+00_JPRB, 0.69012E+00_JPRB, 0.74787E+00_JPRB ,&
&  0.77248E+00_JPRB, 0.80069E+00_JPRB, 0.86163E+00_JPRB, 0.90397E+00_JPRB, 0.93459E+00_JPRB, 0.96599E+00_JPRB, 0.99137E+00_JPRB ,&
&  0.10179E+01_JPRB, 0.10795E+01_JPRB, 0.11685E+01_JPRB, 0.13213E+01_JPRB, 0.14573E+01_JPRB, 0.19840E+01_JPRB, 0.27694E+01_JPRB ,&
&  0.30099E+01_JPRB, 0.30349E+01_JPRB, 0.30062E+01_JPRB, 0.30562E+01_JPRB, 0.31695E+01_JPRB, 0.29716E+01_JPRB, 0.27110E+01_JPRB ,&
&  0.26101E+01_JPRB, 0.23674E+01_JPRB, 0.25266E+01_JPRB, 0.25645E+01_JPRB, 0.25309E+01_JPRB, 0.25654E+01_JPRB, 0.25128E+01_JPRB ,&
&  0.23829E+01_JPRB, 0.21906E+01_JPRB, 0.21483E+01_JPRB, 0.18876E+01_JPRB, 0.15525E+01_JPRB, 0.13007E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.80787E+00_JPRB, 0.63334E+00_JPRB, 0.49786E+00_JPRB, 0.44316E+00_JPRB, 0.39861E+00_JPRB, 0.39008E+00_JPRB, 0.38862E+00_JPRB ,&
&  0.37630E+00_JPRB, 0.37377E+00_JPRB, 0.36580E+00_JPRB, 0.37483E+00_JPRB, 0.36421E+00_JPRB, 0.35724E+00_JPRB, 0.39752E+00_JPRB ,&
&  0.47396E+00_JPRB, 0.67042E+00_JPRB, 0.62768E+00_JPRB, 0.64456E+00_JPRB, 0.63401E+00_JPRB, 0.60702E+00_JPRB, 0.45624E+00_JPRB ,&
&  0.50733E+00_JPRB, 0.57711E+00_JPRB, 0.55182E+00_JPRB, 0.55354E+00_JPRB, 0.56062E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.53419E+00_JPRB, 0.53799E+00_JPRB, 0.51945E+00_JPRB, 0.49887E+00_JPRB, 0.48653E+00_JPRB, 0.46215E+00_JPRB, 0.43942E+00_JPRB ,&
&  0.48894E+00_JPRB, 0.47905E+00_JPRB, 0.47979E+00_JPRB, 0.48636E+00_JPRB, 0.48210E+00_JPRB, 0.42680E+00_JPRB, 0.37942E+00_JPRB ,&
&  0.35209E+00_JPRB, 0.34937E+00_JPRB, 0.37378E+00_JPRB, 0.38775E+00_JPRB, 0.39628E+00_JPRB, 0.39173E+00_JPRB, 0.39532E+00_JPRB ,&
&  0.40262E+00_JPRB, 0.41765E+00_JPRB, 0.44742E+00_JPRB, 0.47021E+00_JPRB, 0.47854E+00_JPRB, 0.49488E+00_JPRB, 0.54728E+00_JPRB ,&
&  0.53163E+00_JPRB, 0.54209E+00_JPRB, 0.56202E+00_JPRB, 0.51881E+00_JPRB, 0.63514E+00_JPRB, 0.67457E+00_JPRB, 0.69872E+00_JPRB ,&
&  0.74714E+00_JPRB, 0.79284E+00_JPRB, 0.81929E+00_JPRB, 0.83596E+00_JPRB, 0.86801E+00_JPRB, 0.88337E+00_JPRB, 0.90464E+00_JPRB ,&
&  0.93371E+00_JPRB, 0.97513E+00_JPRB, 0.10291E+01_JPRB, 0.11095E+01_JPRB, 0.13045E+01_JPRB, 0.15026E+01_JPRB, 0.17823E+01_JPRB ,&
&  0.19774E+01_JPRB, 0.21235E+01_JPRB, 0.23127E+01_JPRB, 0.20800E+01_JPRB, 0.25634E+01_JPRB, 0.27445E+01_JPRB, 0.24673E+01_JPRB ,&
&  0.23884E+01_JPRB, 0.20937E+01_JPRB, 0.21102E+01_JPRB, 0.22072E+01_JPRB, 0.22673E+01_JPRB, 0.23051E+01_JPRB, 0.23147E+01_JPRB ,&
&  0.21779E+01_JPRB, 0.20757E+01_JPRB, 0.20321E+01_JPRB, 0.18158E+01_JPRB, 0.15478E+01_JPRB, 0.13437E+01_JPRB, 0.11047E+01_JPRB ,&
&  0.88974E+00_JPRB, 0.70990E+00_JPRB, 0.60109E+00_JPRB, 0.54969E+00_JPRB, 0.47362E+00_JPRB, 0.45243E+00_JPRB, 0.43132E+00_JPRB ,&
&  0.40750E+00_JPRB, 0.38569E+00_JPRB, 0.37737E+00_JPRB, 0.36672E+00_JPRB, 0.35906E+00_JPRB, 0.36787E+00_JPRB, 0.37874E+00_JPRB ,&
&  0.42181E+00_JPRB, 0.49665E+00_JPRB, 0.66103E+00_JPRB, 0.61193E+00_JPRB, 0.58431E+00_JPRB, 0.55861E+00_JPRB, 0.47938E+00_JPRB ,&
&  0.45385E+00_JPRB, 0.53840E+00_JPRB, 0.53158E+00_JPRB, 0.52786E+00_JPRB, 0.52110E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.49852E+00_JPRB, 0.50377E+00_JPRB, 0.49252E+00_JPRB, 0.48031E+00_JPRB, 0.46488E+00_JPRB, 0.41451E+00_JPRB, 0.41179E+00_JPRB ,&
&  0.48068E+00_JPRB, 0.42875E+00_JPRB, 0.47939E+00_JPRB, 0.37917E+00_JPRB, 0.35696E+00_JPRB, 0.34179E+00_JPRB, 0.32512E+00_JPRB ,&
&  0.30732E+00_JPRB, 0.31347E+00_JPRB, 0.33148E+00_JPRB, 0.34436E+00_JPRB, 0.35363E+00_JPRB, 0.35239E+00_JPRB, 0.35981E+00_JPRB ,&
&  0.37368E+00_JPRB, 0.39930E+00_JPRB, 0.43399E+00_JPRB, 0.45257E+00_JPRB, 0.47208E+00_JPRB, 0.50191E+00_JPRB, 0.53111E+00_JPRB ,&
&  0.50384E+00_JPRB, 0.50707E+00_JPRB, 0.52150E+00_JPRB, 0.54638E+00_JPRB, 0.58954E+00_JPRB, 0.65420E+00_JPRB, 0.69639E+00_JPRB ,&
&  0.71687E+00_JPRB, 0.59389E+00_JPRB, 0.70947E+00_JPRB, 0.74048E+00_JPRB, 0.80522E+00_JPRB, 0.84023E+00_JPRB, 0.86256E+00_JPRB ,&
&  0.89590E+00_JPRB, 0.93108E+00_JPRB, 0.96989E+00_JPRB, 0.10176E+01_JPRB, 0.10783E+01_JPRB, 0.12013E+01_JPRB, 0.12923E+01_JPRB ,&
&  0.15120E+01_JPRB, 0.11649E+01_JPRB, 0.13229E+01_JPRB, 0.14293E+01_JPRB, 0.19302E+01_JPRB, 0.20954E+01_JPRB, 0.20292E+01_JPRB ,&
&  0.20379E+01_JPRB, 0.18888E+01_JPRB, 0.18247E+01_JPRB, 0.18047E+01_JPRB, 0.18597E+01_JPRB, 0.19299E+01_JPRB, 0.19601E+01_JPRB ,&
&  0.18909E+01_JPRB, 0.18318E+01_JPRB, 0.18860E+01_JPRB, 0.17338E+01_JPRB, 0.15276E+01_JPRB, 0.13185E+01_JPRB, 0.10862E+01_JPRB ,&
&  0.92343E+00_JPRB, 0.77480E+00_JPRB, 0.64615E+00_JPRB, 0.56223E+00_JPRB, 0.50183E+00_JPRB, 0.46579E+00_JPRB, 0.45077E+00_JPRB ,&
&  0.44833E+00_JPRB, 0.44086E+00_JPRB, 0.43016E+00_JPRB, 0.40338E+00_JPRB, 0.39152E+00_JPRB, 0.38823E+00_JPRB, 0.40510E+00_JPRB ,&
&  0.40513E+00_JPRB, 0.38606E+00_JPRB, 0.41459E+00_JPRB, 0.44538E+00_JPRB, 0.44336E+00_JPRB, 0.45528E+00_JPRB, 0.50726E+00_JPRB ,&
&  0.43832E+00_JPRB, 0.46486E+00_JPRB, 0.51164E+00_JPRB, 0.51532E+00_JPRB, 0.49980E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.47622E+00_JPRB, 0.46752E+00_JPRB, 0.46319E+00_JPRB, 0.45417E+00_JPRB, 0.44060E+00_JPRB, 0.40301E+00_JPRB, 0.37852E+00_JPRB ,&
&  0.39233E+00_JPRB, 0.37822E+00_JPRB, 0.36695E+00_JPRB, 0.34139E+00_JPRB, 0.32682E+00_JPRB, 0.31221E+00_JPRB, 0.29924E+00_JPRB ,&
&  0.29209E+00_JPRB, 0.30732E+00_JPRB, 0.33576E+00_JPRB, 0.34413E+00_JPRB, 0.34639E+00_JPRB, 0.34990E+00_JPRB, 0.36305E+00_JPRB ,&
&  0.38251E+00_JPRB, 0.40567E+00_JPRB, 0.42095E+00_JPRB, 0.43836E+00_JPRB, 0.46058E+00_JPRB, 0.48596E+00_JPRB, 0.48404E+00_JPRB ,&
&  0.47903E+00_JPRB, 0.46251E+00_JPRB, 0.44466E+00_JPRB, 0.48250E+00_JPRB, 0.54934E+00_JPRB, 0.61315E+00_JPRB, 0.59622E+00_JPRB ,&
&  0.41597E+00_JPRB, 0.41723E+00_JPRB, 0.59722E+00_JPRB, 0.65319E+00_JPRB, 0.71020E+00_JPRB, 0.74560E+00_JPRB, 0.79254E+00_JPRB ,&
&  0.83555E+00_JPRB, 0.87662E+00_JPRB, 0.90862E+00_JPRB, 0.94432E+00_JPRB, 0.99469E+00_JPRB, 0.10665E+01_JPRB, 0.10975E+01_JPRB ,&
&  0.12019E+01_JPRB, 0.97664E+00_JPRB, 0.93761E+00_JPRB, 0.10172E+01_JPRB, 0.14477E+01_JPRB, 0.15016E+01_JPRB, 0.14528E+01_JPRB ,&
&  0.14818E+01_JPRB, 0.14818E+01_JPRB, 0.15043E+01_JPRB, 0.15296E+01_JPRB, 0.14892E+01_JPRB, 0.15037E+01_JPRB, 0.15506E+01_JPRB ,&
&  0.16683E+01_JPRB, 0.17576E+01_JPRB, 0.17358E+01_JPRB, 0.16734E+01_JPRB, 0.15564E+01_JPRB, 0.14203E+01_JPRB, 0.12167E+01_JPRB ,&
&  0.97614E+00_JPRB, 0.80360E+00_JPRB, 0.64992E+00_JPRB, 0.55154E+00_JPRB, 0.50328E+00_JPRB, 0.48787E+00_JPRB, 0.47921E+00_JPRB ,&
&  0.49171E+00_JPRB, 0.49892E+00_JPRB, 0.48983E+00_JPRB, 0.44961E+00_JPRB, 0.41855E+00_JPRB, 0.40691E+00_JPRB, 0.39733E+00_JPRB ,&
&  0.38679E+00_JPRB, 0.35269E+00_JPRB, 0.32361E+00_JPRB, 0.33141E+00_JPRB, 0.36275E+00_JPRB, 0.39083E+00_JPRB, 0.40730E+00_JPRB ,&
&  0.40581E+00_JPRB, 0.40625E+00_JPRB, 0.41707E+00_JPRB, 0.43989E+00_JPRB, 0.45613E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.46363E+00_JPRB, 0.43340E+00_JPRB, 0.41598E+00_JPRB, 0.42046E+00_JPRB, 0.38762E+00_JPRB, 0.36020E+00_JPRB, 0.34843E+00_JPRB ,&
&  0.35490E+00_JPRB, 0.35909E+00_JPRB, 0.36164E+00_JPRB, 0.34342E+00_JPRB, 0.32905E+00_JPRB, 0.32109E+00_JPRB, 0.31931E+00_JPRB ,&
&  0.33186E+00_JPRB, 0.34754E+00_JPRB, 0.35924E+00_JPRB, 0.35634E+00_JPRB, 0.35631E+00_JPRB, 0.36357E+00_JPRB, 0.37908E+00_JPRB ,&
&  0.40083E+00_JPRB, 0.41455E+00_JPRB, 0.41554E+00_JPRB, 0.41833E+00_JPRB, 0.44193E+00_JPRB, 0.46717E+00_JPRB, 0.43874E+00_JPRB ,&
&  0.40765E+00_JPRB, 0.38742E+00_JPRB, 0.38552E+00_JPRB, 0.43428E+00_JPRB, 0.50522E+00_JPRB, 0.54167E+00_JPRB, 0.42043E+00_JPRB ,&
&  0.33341E+00_JPRB, 0.31820E+00_JPRB, 0.35559E+00_JPRB, 0.49330E+00_JPRB, 0.53707E+00_JPRB, 0.58558E+00_JPRB, 0.65962E+00_JPRB ,&
&  0.60665E+00_JPRB, 0.60710E+00_JPRB, 0.63020E+00_JPRB, 0.78165E+00_JPRB, 0.83567E+00_JPRB, 0.87885E+00_JPRB, 0.90844E+00_JPRB ,&
&  0.93454E+00_JPRB, 0.10022E+01_JPRB, 0.11829E+01_JPRB, 0.10183E+01_JPRB, 0.11236E+01_JPRB, 0.13535E+01_JPRB, 0.13079E+01_JPRB ,&
&  0.13724E+01_JPRB, 0.14092E+01_JPRB, 0.15886E+01_JPRB, 0.14939E+01_JPRB, 0.13397E+01_JPRB, 0.12619E+01_JPRB, 0.12916E+01_JPRB ,&
&  0.14870E+01_JPRB, 0.16230E+01_JPRB, 0.15472E+01_JPRB, 0.14440E+01_JPRB, 0.13832E+01_JPRB, 0.13011E+01_JPRB, 0.11525E+01_JPRB ,&
&  0.95310E+00_JPRB, 0.80097E+00_JPRB, 0.66720E+00_JPRB, 0.57118E+00_JPRB, 0.53741E+00_JPRB, 0.53588E+00_JPRB, 0.53024E+00_JPRB ,&
&  0.53989E+00_JPRB, 0.54462E+00_JPRB, 0.53596E+00_JPRB, 0.48150E+00_JPRB, 0.43316E+00_JPRB, 0.39394E+00_JPRB, 0.36657E+00_JPRB ,&
&  0.35107E+00_JPRB, 0.32882E+00_JPRB, 0.31559E+00_JPRB, 0.32561E+00_JPRB, 0.36377E+00_JPRB, 0.40619E+00_JPRB, 0.42406E+00_JPRB ,&
&  0.41189E+00_JPRB, 0.40538E+00_JPRB, 0.42684E+00_JPRB, 0.45929E+00_JPRB, 0.47241E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.48541E+00_JPRB, 0.48466E+00_JPRB, 0.47108E+00_JPRB, 0.44221E+00_JPRB, 0.41733E+00_JPRB, 0.38870E+00_JPRB, 0.36991E+00_JPRB ,&
&  0.36135E+00_JPRB, 0.35795E+00_JPRB, 0.35821E+00_JPRB, 0.36146E+00_JPRB, 0.36693E+00_JPRB, 0.37594E+00_JPRB, 0.37980E+00_JPRB ,&
&  0.38489E+00_JPRB, 0.38795E+00_JPRB, 0.39352E+00_JPRB, 0.40222E+00_JPRB, 0.41287E+00_JPRB, 0.42486E+00_JPRB, 0.43651E+00_JPRB ,&
&  0.44601E+00_JPRB, 0.44071E+00_JPRB, 0.42084E+00_JPRB, 0.40877E+00_JPRB, 0.41490E+00_JPRB, 0.43171E+00_JPRB, 0.41683E+00_JPRB ,&
&  0.37873E+00_JPRB, 0.35536E+00_JPRB, 0.35724E+00_JPRB, 0.42092E+00_JPRB, 0.45937E+00_JPRB, 0.48140E+00_JPRB, 0.36654E+00_JPRB ,&
&  0.30416E+00_JPRB, 0.27753E+00_JPRB, 0.27540E+00_JPRB, 0.29897E+00_JPRB, 0.35198E+00_JPRB, 0.43540E+00_JPRB, 0.49712E+00_JPRB ,&
&  0.54365E+00_JPRB, 0.58660E+00_JPRB, 0.60067E+00_JPRB, 0.62651E+00_JPRB, 0.66960E+00_JPRB, 0.71290E+00_JPRB, 0.75688E+00_JPRB ,&
&  0.79308E+00_JPRB, 0.85963E+00_JPRB, 0.10116E+01_JPRB, 0.91957E+00_JPRB, 0.94951E+00_JPRB, 0.99618E+00_JPRB, 0.11004E+01_JPRB ,&
&  0.12803E+01_JPRB, 0.13335E+01_JPRB, 0.14163E+01_JPRB, 0.15450E+01_JPRB, 0.14389E+01_JPRB, 0.13419E+01_JPRB, 0.13099E+01_JPRB ,&
&  0.14430E+01_JPRB, 0.15564E+01_JPRB, 0.14856E+01_JPRB, 0.13184E+01_JPRB, 0.12476E+01_JPRB, 0.12019E+01_JPRB, 0.10597E+01_JPRB ,&
&  0.88796E+00_JPRB, 0.78544E+00_JPRB, 0.67449E+00_JPRB, 0.60606E+00_JPRB, 0.59454E+00_JPRB, 0.59075E+00_JPRB, 0.58992E+00_JPRB ,&
&  0.59124E+00_JPRB, 0.58540E+00_JPRB, 0.57340E+00_JPRB, 0.52255E+00_JPRB, 0.47324E+00_JPRB, 0.42642E+00_JPRB, 0.39524E+00_JPRB ,&
&  0.37675E+00_JPRB, 0.37085E+00_JPRB, 0.37368E+00_JPRB, 0.38795E+00_JPRB, 0.41719E+00_JPRB, 0.44279E+00_JPRB, 0.46026E+00_JPRB ,&
&  0.46172E+00_JPRB, 0.44916E+00_JPRB, 0.44528E+00_JPRB, 0.44740E+00_JPRB, 0.47030E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.54774E+00_JPRB, 0.55867E+00_JPRB, 0.55086E+00_JPRB, 0.52609E+00_JPRB, 0.49409E+00_JPRB, 0.45870E+00_JPRB, 0.43936E+00_JPRB ,&
&  0.43251E+00_JPRB, 0.43399E+00_JPRB, 0.43170E+00_JPRB, 0.42855E+00_JPRB, 0.42911E+00_JPRB, 0.43133E+00_JPRB, 0.43491E+00_JPRB ,&
&  0.44111E+00_JPRB, 0.45070E+00_JPRB, 0.45963E+00_JPRB, 0.46053E+00_JPRB, 0.46639E+00_JPRB, 0.47164E+00_JPRB, 0.47662E+00_JPRB ,&
&  0.47286E+00_JPRB, 0.45124E+00_JPRB, 0.42335E+00_JPRB, 0.40823E+00_JPRB, 0.39266E+00_JPRB, 0.39105E+00_JPRB, 0.37928E+00_JPRB ,&
&  0.35313E+00_JPRB, 0.35122E+00_JPRB, 0.38398E+00_JPRB, 0.41384E+00_JPRB, 0.41162E+00_JPRB, 0.43624E+00_JPRB, 0.32214E+00_JPRB ,&
&  0.28258E+00_JPRB, 0.26207E+00_JPRB, 0.25699E+00_JPRB, 0.26448E+00_JPRB, 0.28680E+00_JPRB, 0.31895E+00_JPRB, 0.35867E+00_JPRB ,&
&  0.42762E+00_JPRB, 0.48688E+00_JPRB, 0.52448E+00_JPRB, 0.56148E+00_JPRB, 0.59949E+00_JPRB, 0.63411E+00_JPRB, 0.65685E+00_JPRB ,&
&  0.67561E+00_JPRB, 0.71689E+00_JPRB, 0.78764E+00_JPRB, 0.88513E+00_JPRB, 0.10852E+01_JPRB, 0.10585E+01_JPRB, 0.10935E+01_JPRB ,&
&  0.13955E+01_JPRB, 0.15789E+01_JPRB, 0.14860E+01_JPRB, 0.13959E+01_JPRB, 0.13749E+01_JPRB, 0.14092E+01_JPRB, 0.14355E+01_JPRB ,&
&  0.15170E+01_JPRB, 0.15675E+01_JPRB, 0.15145E+01_JPRB, 0.13385E+01_JPRB, 0.12423E+01_JPRB, 0.11786E+01_JPRB, 0.10517E+01_JPRB ,&
&  0.90816E+00_JPRB, 0.86125E+00_JPRB, 0.81483E+00_JPRB, 0.71805E+00_JPRB, 0.66866E+00_JPRB, 0.64725E+00_JPRB, 0.63738E+00_JPRB ,&
&  0.62595E+00_JPRB, 0.61070E+00_JPRB, 0.58538E+00_JPRB, 0.55762E+00_JPRB, 0.52827E+00_JPRB, 0.50211E+00_JPRB, 0.48142E+00_JPRB ,&
&  0.46881E+00_JPRB, 0.45943E+00_JPRB, 0.45584E+00_JPRB, 0.45355E+00_JPRB, 0.44775E+00_JPRB, 0.44647E+00_JPRB, 0.45454E+00_JPRB ,&
&  0.46785E+00_JPRB, 0.47746E+00_JPRB, 0.49219E+00_JPRB, 0.50657E+00_JPRB, 0.52596E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.55132E+00_JPRB, 0.56528E+00_JPRB, 0.57410E+00_JPRB, 0.57250E+00_JPRB, 0.55991E+00_JPRB, 0.54340E+00_JPRB, 0.53065E+00_JPRB ,&
&  0.52113E+00_JPRB, 0.51265E+00_JPRB, 0.50314E+00_JPRB, 0.49597E+00_JPRB, 0.49026E+00_JPRB, 0.48821E+00_JPRB, 0.48720E+00_JPRB ,&
&  0.48861E+00_JPRB, 0.48123E+00_JPRB, 0.47983E+00_JPRB, 0.48366E+00_JPRB, 0.48965E+00_JPRB, 0.48947E+00_JPRB, 0.48548E+00_JPRB ,&
&  0.46532E+00_JPRB, 0.43776E+00_JPRB, 0.41238E+00_JPRB, 0.39248E+00_JPRB, 0.37820E+00_JPRB, 0.37168E+00_JPRB, 0.37693E+00_JPRB ,&
&  0.38772E+00_JPRB, 0.38845E+00_JPRB, 0.39140E+00_JPRB, 0.41128E+00_JPRB, 0.42547E+00_JPRB, 0.36763E+00_JPRB, 0.28603E+00_JPRB ,&
&  0.27025E+00_JPRB, 0.25625E+00_JPRB, 0.25521E+00_JPRB, 0.25981E+00_JPRB, 0.27157E+00_JPRB, 0.29194E+00_JPRB, 0.32678E+00_JPRB ,&
&  0.37979E+00_JPRB, 0.49085E+00_JPRB, 0.54875E+00_JPRB, 0.59083E+00_JPRB, 0.62713E+00_JPRB, 0.65692E+00_JPRB, 0.66760E+00_JPRB ,&
&  0.67466E+00_JPRB, 0.69408E+00_JPRB, 0.73402E+00_JPRB, 0.78208E+00_JPRB, 0.85778E+00_JPRB, 0.10085E+01_JPRB, 0.11332E+01_JPRB ,&
&  0.13067E+01_JPRB, 0.13810E+01_JPRB, 0.13497E+01_JPRB, 0.12625E+01_JPRB, 0.12849E+01_JPRB, 0.13514E+01_JPRB, 0.13647E+01_JPRB ,&
&  0.14026E+01_JPRB, 0.14401E+01_JPRB, 0.13843E+01_JPRB, 0.12864E+01_JPRB, 0.12233E+01_JPRB, 0.11725E+01_JPRB, 0.10645E+01_JPRB ,&
&  0.95045E+00_JPRB, 0.87624E+00_JPRB, 0.81737E+00_JPRB, 0.76078E+00_JPRB, 0.72342E+00_JPRB, 0.68051E+00_JPRB, 0.64986E+00_JPRB ,&
&  0.62767E+00_JPRB, 0.60990E+00_JPRB, 0.58034E+00_JPRB, 0.55596E+00_JPRB, 0.54319E+00_JPRB, 0.53429E+00_JPRB, 0.52565E+00_JPRB ,&
&  0.51815E+00_JPRB, 0.50902E+00_JPRB, 0.49891E+00_JPRB, 0.48931E+00_JPRB, 0.47638E+00_JPRB, 0.47046E+00_JPRB, 0.47325E+00_JPRB ,&
&  0.48177E+00_JPRB, 0.49594E+00_JPRB, 0.51106E+00_JPRB, 0.52460E+00_JPRB, 0.53922E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.57556E+00_JPRB, 0.58353E+00_JPRB, 0.58636E+00_JPRB, 0.58061E+00_JPRB, 0.56901E+00_JPRB, 0.55422E+00_JPRB, 0.54066E+00_JPRB ,&
&  0.52943E+00_JPRB, 0.52122E+00_JPRB, 0.51244E+00_JPRB, 0.50655E+00_JPRB, 0.49912E+00_JPRB, 0.49545E+00_JPRB, 0.49053E+00_JPRB ,&
&  0.48969E+00_JPRB, 0.49063E+00_JPRB, 0.48630E+00_JPRB, 0.48046E+00_JPRB, 0.46614E+00_JPRB, 0.45240E+00_JPRB, 0.44164E+00_JPRB ,&
&  0.43141E+00_JPRB, 0.42362E+00_JPRB, 0.40654E+00_JPRB, 0.39305E+00_JPRB, 0.38742E+00_JPRB, 0.38552E+00_JPRB, 0.39507E+00_JPRB ,&
&  0.39764E+00_JPRB, 0.40624E+00_JPRB, 0.41407E+00_JPRB, 0.40931E+00_JPRB, 0.40338E+00_JPRB, 0.31002E+00_JPRB, 0.28202E+00_JPRB ,&
&  0.27301E+00_JPRB, 0.26656E+00_JPRB, 0.26521E+00_JPRB, 0.27048E+00_JPRB, 0.28022E+00_JPRB, 0.30532E+00_JPRB, 0.32963E+00_JPRB ,&
&  0.35566E+00_JPRB, 0.50003E+00_JPRB, 0.55966E+00_JPRB, 0.61523E+00_JPRB, 0.65268E+00_JPRB, 0.68039E+00_JPRB, 0.69737E+00_JPRB ,&
&  0.71413E+00_JPRB, 0.74050E+00_JPRB, 0.75614E+00_JPRB, 0.78905E+00_JPRB, 0.85020E+00_JPRB, 0.95972E+00_JPRB, 0.10535E+01_JPRB ,&
&  0.11571E+01_JPRB, 0.11915E+01_JPRB, 0.12177E+01_JPRB, 0.12094E+01_JPRB, 0.12222E+01_JPRB, 0.12247E+01_JPRB, 0.12184E+01_JPRB ,&
&  0.12170E+01_JPRB, 0.11881E+01_JPRB, 0.11867E+01_JPRB, 0.11606E+01_JPRB, 0.11352E+01_JPRB, 0.11040E+01_JPRB, 0.10241E+01_JPRB ,&
&  0.92624E+00_JPRB, 0.84917E+00_JPRB, 0.78232E+00_JPRB, 0.73405E+00_JPRB, 0.69612E+00_JPRB, 0.66765E+00_JPRB, 0.65605E+00_JPRB ,&
&  0.65355E+00_JPRB, 0.65358E+00_JPRB, 0.63532E+00_JPRB, 0.61296E+00_JPRB, 0.59921E+00_JPRB, 0.59191E+00_JPRB, 0.57302E+00_JPRB ,&
&  0.55620E+00_JPRB, 0.53647E+00_JPRB, 0.52501E+00_JPRB, 0.51251E+00_JPRB, 0.50119E+00_JPRB, 0.49638E+00_JPRB, 0.49645E+00_JPRB ,&
&  0.50403E+00_JPRB, 0.51429E+00_JPRB, 0.53048E+00_JPRB, 0.54755E+00_JPRB, 0.56625E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.60752E+00_JPRB, 0.59540E+00_JPRB, 0.57023E+00_JPRB, 0.53580E+00_JPRB, 0.51173E+00_JPRB, 0.49678E+00_JPRB, 0.49163E+00_JPRB ,&
&  0.48968E+00_JPRB, 0.48984E+00_JPRB, 0.48961E+00_JPRB, 0.48849E+00_JPRB, 0.48489E+00_JPRB, 0.47909E+00_JPRB, 0.47305E+00_JPRB ,&
&  0.46297E+00_JPRB, 0.45331E+00_JPRB, 0.44233E+00_JPRB, 0.43616E+00_JPRB, 0.43053E+00_JPRB, 0.42635E+00_JPRB, 0.42304E+00_JPRB ,&
&  0.41894E+00_JPRB, 0.41391E+00_JPRB, 0.39696E+00_JPRB, 0.38918E+00_JPRB, 0.38249E+00_JPRB, 0.38417E+00_JPRB, 0.38343E+00_JPRB ,&
&  0.40115E+00_JPRB, 0.35664E+00_JPRB, 0.34858E+00_JPRB, 0.33308E+00_JPRB, 0.32017E+00_JPRB, 0.30603E+00_JPRB, 0.29522E+00_JPRB ,&
&  0.28912E+00_JPRB, 0.28398E+00_JPRB, 0.28284E+00_JPRB, 0.28916E+00_JPRB, 0.29751E+00_JPRB, 0.31443E+00_JPRB, 0.33380E+00_JPRB ,&
&  0.37466E+00_JPRB, 0.50692E+00_JPRB, 0.57264E+00_JPRB, 0.62394E+00_JPRB, 0.67212E+00_JPRB, 0.71565E+00_JPRB, 0.74031E+00_JPRB ,&
&  0.76481E+00_JPRB, 0.79817E+00_JPRB, 0.82519E+00_JPRB, 0.83537E+00_JPRB, 0.90704E+00_JPRB, 0.95797E+00_JPRB, 0.10084E+01_JPRB ,&
&  0.10519E+01_JPRB, 0.10835E+01_JPRB, 0.11093E+01_JPRB, 0.11202E+01_JPRB, 0.11303E+01_JPRB, 0.11142E+01_JPRB, 0.10873E+01_JPRB ,&
&  0.10675E+01_JPRB, 0.10571E+01_JPRB, 0.10495E+01_JPRB, 0.10133E+01_JPRB, 0.98711E+00_JPRB, 0.96406E+00_JPRB, 0.92375E+00_JPRB ,&
&  0.87812E+00_JPRB, 0.83362E+00_JPRB, 0.78789E+00_JPRB, 0.74199E+00_JPRB, 0.70224E+00_JPRB, 0.68080E+00_JPRB, 0.67765E+00_JPRB ,&
&  0.68171E+00_JPRB, 0.68202E+00_JPRB, 0.67194E+00_JPRB, 0.65619E+00_JPRB, 0.63919E+00_JPRB, 0.62969E+00_JPRB, 0.61440E+00_JPRB ,&
&  0.59730E+00_JPRB, 0.57455E+00_JPRB, 0.55998E+00_JPRB, 0.54634E+00_JPRB, 0.53908E+00_JPRB, 0.53784E+00_JPRB, 0.54118E+00_JPRB ,&
&  0.54746E+00_JPRB, 0.55719E+00_JPRB, 0.57280E+00_JPRB, 0.58954E+00_JPRB, 0.60291E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.55950E+00_JPRB, 0.52830E+00_JPRB, 0.49596E+00_JPRB, 0.47909E+00_JPRB, 0.47665E+00_JPRB, 0.47471E+00_JPRB, 0.47371E+00_JPRB ,&
&  0.47300E+00_JPRB, 0.47596E+00_JPRB, 0.47907E+00_JPRB, 0.48053E+00_JPRB, 0.47914E+00_JPRB, 0.47584E+00_JPRB, 0.47103E+00_JPRB ,&
&  0.46397E+00_JPRB, 0.45624E+00_JPRB, 0.44854E+00_JPRB, 0.44212E+00_JPRB, 0.43506E+00_JPRB, 0.42822E+00_JPRB, 0.41963E+00_JPRB ,&
&  0.41038E+00_JPRB, 0.40159E+00_JPRB, 0.37050E+00_JPRB, 0.36692E+00_JPRB, 0.35977E+00_JPRB, 0.35849E+00_JPRB, 0.36158E+00_JPRB ,&
&  0.36340E+00_JPRB, 0.35812E+00_JPRB, 0.35217E+00_JPRB, 0.33961E+00_JPRB, 0.32838E+00_JPRB, 0.31617E+00_JPRB, 0.30901E+00_JPRB ,&
&  0.30621E+00_JPRB, 0.30781E+00_JPRB, 0.31345E+00_JPRB, 0.32120E+00_JPRB, 0.33199E+00_JPRB, 0.34537E+00_JPRB, 0.36516E+00_JPRB ,&
&  0.41439E+00_JPRB, 0.48533E+00_JPRB, 0.56716E+00_JPRB, 0.61319E+00_JPRB, 0.66489E+00_JPRB, 0.71351E+00_JPRB, 0.74657E+00_JPRB ,&
&  0.77546E+00_JPRB, 0.80626E+00_JPRB, 0.84050E+00_JPRB, 0.88141E+00_JPRB, 0.91835E+00_JPRB, 0.94629E+00_JPRB, 0.98352E+00_JPRB ,&
&  0.10133E+01_JPRB, 0.10320E+01_JPRB, 0.10350E+01_JPRB, 0.10317E+01_JPRB, 0.10236E+01_JPRB, 0.10056E+01_JPRB, 0.98843E+00_JPRB ,&
&  0.97765E+00_JPRB, 0.96905E+00_JPRB, 0.95630E+00_JPRB, 0.92781E+00_JPRB, 0.90934E+00_JPRB, 0.89500E+00_JPRB, 0.87370E+00_JPRB ,&
&  0.84899E+00_JPRB, 0.82181E+00_JPRB, 0.79702E+00_JPRB, 0.76500E+00_JPRB, 0.74305E+00_JPRB, 0.73606E+00_JPRB, 0.73417E+00_JPRB ,&
&  0.73141E+00_JPRB, 0.72744E+00_JPRB, 0.72134E+00_JPRB, 0.71270E+00_JPRB, 0.69636E+00_JPRB, 0.68066E+00_JPRB, 0.66538E+00_JPRB ,&
&  0.65197E+00_JPRB, 0.63650E+00_JPRB, 0.62249E+00_JPRB, 0.60572E+00_JPRB, 0.59547E+00_JPRB, 0.58644E+00_JPRB, 0.58440E+00_JPRB ,&
&  0.58573E+00_JPRB, 0.59014E+00_JPRB, 0.59252E+00_JPRB, 0.59157E+00_JPRB, 0.58040E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.53363E+00_JPRB, 0.50538E+00_JPRB, 0.49058E+00_JPRB, 0.48745E+00_JPRB, 0.48568E+00_JPRB, 0.48398E+00_JPRB, 0.48232E+00_JPRB ,&
&  0.47883E+00_JPRB, 0.47619E+00_JPRB, 0.47608E+00_JPRB, 0.47609E+00_JPRB, 0.47438E+00_JPRB, 0.47231E+00_JPRB, 0.46985E+00_JPRB ,&
&  0.46712E+00_JPRB, 0.46308E+00_JPRB, 0.45920E+00_JPRB, 0.45431E+00_JPRB, 0.44917E+00_JPRB, 0.44283E+00_JPRB, 0.43439E+00_JPRB ,&
&  0.42657E+00_JPRB, 0.41884E+00_JPRB, 0.40877E+00_JPRB, 0.40424E+00_JPRB, 0.40166E+00_JPRB, 0.38018E+00_JPRB, 0.37477E+00_JPRB ,&
&  0.37794E+00_JPRB, 0.37945E+00_JPRB, 0.38130E+00_JPRB, 0.38354E+00_JPRB, 0.38808E+00_JPRB, 0.38328E+00_JPRB, 0.38296E+00_JPRB ,&
&  0.36798E+00_JPRB, 0.37774E+00_JPRB, 0.38532E+00_JPRB, 0.39364E+00_JPRB, 0.40342E+00_JPRB, 0.41935E+00_JPRB, 0.44620E+00_JPRB ,&
&  0.49720E+00_JPRB, 0.52451E+00_JPRB, 0.55353E+00_JPRB, 0.58312E+00_JPRB, 0.61896E+00_JPRB, 0.65298E+00_JPRB, 0.68100E+00_JPRB ,&
&  0.70396E+00_JPRB, 0.72885E+00_JPRB, 0.75864E+00_JPRB, 0.79963E+00_JPRB, 0.83284E+00_JPRB, 0.86285E+00_JPRB, 0.88375E+00_JPRB ,&
&  0.90887E+00_JPRB, 0.92681E+00_JPRB, 0.93897E+00_JPRB, 0.94693E+00_JPRB, 0.95546E+00_JPRB, 0.95849E+00_JPRB, 0.96157E+00_JPRB ,&
&  0.96162E+00_JPRB, 0.96025E+00_JPRB, 0.95160E+00_JPRB, 0.93693E+00_JPRB, 0.92500E+00_JPRB, 0.91543E+00_JPRB, 0.90246E+00_JPRB ,&
&  0.88837E+00_JPRB, 0.87439E+00_JPRB, 0.85861E+00_JPRB, 0.84107E+00_JPRB, 0.82204E+00_JPRB, 0.80538E+00_JPRB, 0.78703E+00_JPRB ,&
&  0.77305E+00_JPRB, 0.76281E+00_JPRB, 0.75478E+00_JPRB, 0.74460E+00_JPRB, 0.73263E+00_JPRB, 0.72135E+00_JPRB, 0.70828E+00_JPRB ,&
&  0.69618E+00_JPRB, 0.68479E+00_JPRB, 0.67576E+00_JPRB, 0.66502E+00_JPRB, 0.65774E+00_JPRB, 0.64725E+00_JPRB, 0.63802E+00_JPRB ,&
&  0.62747E+00_JPRB, 0.61987E+00_JPRB, 0.60587E+00_JPRB, 0.58565E+00_JPRB, 0.55784E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.51605E+00_JPRB, 0.50404E+00_JPRB, 0.49859E+00_JPRB, 0.49574E+00_JPRB, 0.49369E+00_JPRB, 0.49076E+00_JPRB, 0.48738E+00_JPRB ,&
&  0.48452E+00_JPRB, 0.48127E+00_JPRB, 0.47803E+00_JPRB, 0.47685E+00_JPRB, 0.47513E+00_JPRB, 0.47432E+00_JPRB, 0.47346E+00_JPRB ,&
&  0.47280E+00_JPRB, 0.47170E+00_JPRB, 0.47097E+00_JPRB, 0.46970E+00_JPRB, 0.46770E+00_JPRB, 0.46480E+00_JPRB, 0.46236E+00_JPRB ,&
&  0.46077E+00_JPRB, 0.45964E+00_JPRB, 0.45762E+00_JPRB, 0.45604E+00_JPRB, 0.45620E+00_JPRB, 0.45564E+00_JPRB, 0.45456E+00_JPRB ,&
&  0.45587E+00_JPRB, 0.45705E+00_JPRB, 0.45699E+00_JPRB, 0.45593E+00_JPRB, 0.45461E+00_JPRB, 0.45500E+00_JPRB, 0.45799E+00_JPRB ,&
&  0.46194E+00_JPRB, 0.47270E+00_JPRB, 0.48083E+00_JPRB, 0.48872E+00_JPRB, 0.49677E+00_JPRB, 0.50650E+00_JPRB, 0.51809E+00_JPRB ,&
&  0.52993E+00_JPRB, 0.53945E+00_JPRB, 0.54994E+00_JPRB, 0.56414E+00_JPRB, 0.57857E+00_JPRB, 0.59274E+00_JPRB, 0.60827E+00_JPRB ,&
&  0.62090E+00_JPRB, 0.63444E+00_JPRB, 0.65001E+00_JPRB, 0.66684E+00_JPRB, 0.68081E+00_JPRB, 0.70045E+00_JPRB, 0.72502E+00_JPRB ,&
&  0.75675E+00_JPRB, 0.77251E+00_JPRB, 0.78985E+00_JPRB, 0.80702E+00_JPRB, 0.82861E+00_JPRB, 0.84087E+00_JPRB, 0.85180E+00_JPRB ,&
&  0.85935E+00_JPRB, 0.86643E+00_JPRB, 0.86450E+00_JPRB, 0.86098E+00_JPRB, 0.85794E+00_JPRB, 0.85244E+00_JPRB, 0.84610E+00_JPRB ,&
&  0.84015E+00_JPRB, 0.83528E+00_JPRB, 0.82620E+00_JPRB, 0.81784E+00_JPRB, 0.80904E+00_JPRB, 0.80114E+00_JPRB, 0.78536E+00_JPRB ,&
&  0.77122E+00_JPRB, 0.76092E+00_JPRB, 0.75436E+00_JPRB, 0.74454E+00_JPRB, 0.73500E+00_JPRB, 0.72617E+00_JPRB, 0.71758E+00_JPRB ,&
&  0.70889E+00_JPRB, 0.69857E+00_JPRB, 0.68753E+00_JPRB, 0.67556E+00_JPRB, 0.66511E+00_JPRB, 0.65078E+00_JPRB, 0.63445E+00_JPRB ,&
&  0.61532E+00_JPRB, 0.59886E+00_JPRB, 0.57877E+00_JPRB, 0.55481E+00_JPRB, 0.53093E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.53331E+00_JPRB, 0.52972E+00_JPRB, 0.52638E+00_JPRB, 0.52299E+00_JPRB, 0.51923E+00_JPRB, 0.51502E+00_JPRB, 0.51052E+00_JPRB ,&
&  0.50635E+00_JPRB, 0.50289E+00_JPRB, 0.49976E+00_JPRB, 0.49695E+00_JPRB, 0.49442E+00_JPRB, 0.49245E+00_JPRB, 0.49147E+00_JPRB ,&
&  0.49105E+00_JPRB, 0.49055E+00_JPRB, 0.49035E+00_JPRB, 0.49032E+00_JPRB, 0.49005E+00_JPRB, 0.48958E+00_JPRB, 0.48904E+00_JPRB ,&
&  0.48880E+00_JPRB, 0.48861E+00_JPRB, 0.48834E+00_JPRB, 0.48848E+00_JPRB, 0.48902E+00_JPRB, 0.48958E+00_JPRB, 0.49012E+00_JPRB ,&
&  0.49114E+00_JPRB, 0.49237E+00_JPRB, 0.49400E+00_JPRB, 0.49589E+00_JPRB, 0.49966E+00_JPRB, 0.50391E+00_JPRB, 0.50831E+00_JPRB ,&
&  0.51252E+00_JPRB, 0.51687E+00_JPRB, 0.52048E+00_JPRB, 0.52351E+00_JPRB, 0.52622E+00_JPRB, 0.52942E+00_JPRB, 0.53305E+00_JPRB ,&
&  0.53700E+00_JPRB, 0.54139E+00_JPRB, 0.54647E+00_JPRB, 0.55152E+00_JPRB, 0.55629E+00_JPRB, 0.56132E+00_JPRB, 0.56684E+00_JPRB ,&
&  0.57234E+00_JPRB, 0.57797E+00_JPRB, 0.58369E+00_JPRB, 0.58943E+00_JPRB, 0.59495E+00_JPRB, 0.60149E+00_JPRB, 0.60934E+00_JPRB ,&
&  0.61821E+00_JPRB, 0.62517E+00_JPRB, 0.63318E+00_JPRB, 0.64187E+00_JPRB, 0.65477E+00_JPRB, 0.66130E+00_JPRB, 0.66757E+00_JPRB ,&
&  0.67316E+00_JPRB, 0.68202E+00_JPRB, 0.68740E+00_JPRB, 0.69288E+00_JPRB, 0.69785E+00_JPRB, 0.70229E+00_JPRB, 0.70507E+00_JPRB ,&
&  0.70742E+00_JPRB, 0.70938E+00_JPRB, 0.70966E+00_JPRB, 0.70913E+00_JPRB, 0.70772E+00_JPRB, 0.70583E+00_JPRB, 0.70074E+00_JPRB ,&
&  0.69602E+00_JPRB, 0.69141E+00_JPRB, 0.68721E+00_JPRB, 0.68115E+00_JPRB, 0.67505E+00_JPRB, 0.66874E+00_JPRB, 0.66222E+00_JPRB ,&
&  0.65376E+00_JPRB, 0.64507E+00_JPRB, 0.63569E+00_JPRB, 0.62577E+00_JPRB, 0.61622E+00_JPRB, 0.60631E+00_JPRB, 0.59556E+00_JPRB ,&
&  0.58343E+00_JPRB, 0.57217E+00_JPRB, 0.55950E+00_JPRB, 0.54688E+00_JPRB, 0.53749E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB, 0.55330E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB ,&
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB ,&
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB ,&
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB ,&
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB ,&
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB ,&
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB ,&
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB ,&
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB ,&
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB ,&
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB ,&
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB ,&
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB ,&
&  0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB, 0.12088E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.12798E+00_JPRB, 0.12800E+00_JPRB, 0.12794E+00_JPRB, 0.12771E+00_JPRB, 0.12721E+00_JPRB, 0.12709E+00_JPRB, 0.12693E+00_JPRB ,&
&  0.12670E+00_JPRB, 0.12619E+00_JPRB, 0.12618E+00_JPRB, 0.12611E+00_JPRB, 0.12603E+00_JPRB, 0.12594E+00_JPRB, 0.12576E+00_JPRB ,&
&  0.12539E+00_JPRB, 0.12482E+00_JPRB, 0.12312E+00_JPRB, 0.12230E+00_JPRB, 0.12179E+00_JPRB, 0.12153E+00_JPRB, 0.12132E+00_JPRB ,&
&  0.12120E+00_JPRB, 0.12114E+00_JPRB, 0.12108E+00_JPRB, 0.12143E+00_JPRB, 0.12158E+00_JPRB, 0.12164E+00_JPRB, 0.12169E+00_JPRB ,&
&  0.12170E+00_JPRB, 0.12177E+00_JPRB, 0.12178E+00_JPRB, 0.12178E+00_JPRB, 0.12188E+00_JPRB, 0.12187E+00_JPRB, 0.12172E+00_JPRB ,&
&  0.12154E+00_JPRB, 0.12132E+00_JPRB, 0.12110E+00_JPRB, 0.12084E+00_JPRB, 0.12065E+00_JPRB, 0.12049E+00_JPRB, 0.12042E+00_JPRB ,&
&  0.12028E+00_JPRB, 0.12017E+00_JPRB, 0.12006E+00_JPRB, 0.11999E+00_JPRB, 0.11990E+00_JPRB, 0.11984E+00_JPRB, 0.11974E+00_JPRB ,&
&  0.11979E+00_JPRB, 0.11979E+00_JPRB, 0.11982E+00_JPRB, 0.11981E+00_JPRB, 0.11980E+00_JPRB, 0.11975E+00_JPRB, 0.11966E+00_JPRB ,&
&  0.11950E+00_JPRB, 0.11941E+00_JPRB, 0.11930E+00_JPRB, 0.11921E+00_JPRB, 0.11918E+00_JPRB, 0.11915E+00_JPRB, 0.11913E+00_JPRB ,&
&  0.11913E+00_JPRB, 0.11919E+00_JPRB, 0.11920E+00_JPRB, 0.11924E+00_JPRB, 0.11928E+00_JPRB, 0.11933E+00_JPRB, 0.11937E+00_JPRB ,&
&  0.11944E+00_JPRB, 0.11949E+00_JPRB, 0.11958E+00_JPRB, 0.11970E+00_JPRB, 0.11983E+00_JPRB, 0.12000E+00_JPRB, 0.12023E+00_JPRB ,&
&  0.12048E+00_JPRB, 0.12073E+00_JPRB, 0.12098E+00_JPRB, 0.12123E+00_JPRB, 0.12161E+00_JPRB, 0.12199E+00_JPRB, 0.12243E+00_JPRB ,&
&  0.12286E+00_JPRB, 0.12330E+00_JPRB, 0.12369E+00_JPRB, 0.12411E+00_JPRB, 0.12463E+00_JPRB, 0.12490E+00_JPRB, 0.12514E+00_JPRB ,&
&  0.12549E+00_JPRB, 0.12584E+00_JPRB, 0.12620E+00_JPRB, 0.12653E+00_JPRB, 0.12706E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.13275E+00_JPRB, 0.13315E+00_JPRB, 0.13282E+00_JPRB, 0.13252E+00_JPRB, 0.12777E+00_JPRB, 0.13351E+00_JPRB, 0.13278E+00_JPRB ,&
&  0.13301E+00_JPRB, 0.13248E+00_JPRB, 0.13337E+00_JPRB, 0.13331E+00_JPRB, 0.13287E+00_JPRB, 0.13256E+00_JPRB, 0.13220E+00_JPRB ,&
&  0.13183E+00_JPRB, 0.13113E+00_JPRB, 0.13071E+00_JPRB, 0.13019E+00_JPRB, 0.12969E+00_JPRB, 0.12892E+00_JPRB, 0.12813E+00_JPRB ,&
&  0.12779E+00_JPRB, 0.12787E+00_JPRB, 0.12821E+00_JPRB, 0.12704E+00_JPRB, 0.12659E+00_JPRB, 0.12632E+00_JPRB, 0.12621E+00_JPRB ,&
&  0.12599E+00_JPRB, 0.12597E+00_JPRB, 0.12600E+00_JPRB, 0.12595E+00_JPRB, 0.12692E+00_JPRB, 0.12719E+00_JPRB, 0.12740E+00_JPRB ,&
&  0.12741E+00_JPRB, 0.12724E+00_JPRB, 0.12663E+00_JPRB, 0.12608E+00_JPRB, 0.12554E+00_JPRB, 0.12462E+00_JPRB, 0.12366E+00_JPRB ,&
&  0.12283E+00_JPRB, 0.12218E+00_JPRB, 0.12147E+00_JPRB, 0.12091E+00_JPRB, 0.12045E+00_JPRB, 0.12001E+00_JPRB, 0.11972E+00_JPRB ,&
&  0.11962E+00_JPRB, 0.11959E+00_JPRB, 0.11954E+00_JPRB, 0.11948E+00_JPRB, 0.11940E+00_JPRB, 0.11932E+00_JPRB, 0.11924E+00_JPRB ,&
&  0.11938E+00_JPRB, 0.11930E+00_JPRB, 0.11913E+00_JPRB, 0.11887E+00_JPRB, 0.11871E+00_JPRB, 0.11840E+00_JPRB, 0.11818E+00_JPRB ,&
&  0.11800E+00_JPRB, 0.11779E+00_JPRB, 0.11768E+00_JPRB, 0.11764E+00_JPRB, 0.11762E+00_JPRB, 0.11759E+00_JPRB, 0.11765E+00_JPRB ,&
&  0.11776E+00_JPRB, 0.11792E+00_JPRB, 0.11822E+00_JPRB, 0.11848E+00_JPRB, 0.11871E+00_JPRB, 0.11892E+00_JPRB, 0.11934E+00_JPRB ,&
&  0.11979E+00_JPRB, 0.12024E+00_JPRB, 0.12068E+00_JPRB, 0.12100E+00_JPRB, 0.12163E+00_JPRB, 0.12226E+00_JPRB, 0.12294E+00_JPRB ,&
&  0.12365E+00_JPRB, 0.12455E+00_JPRB, 0.12547E+00_JPRB, 0.12638E+00_JPRB, 0.12757E+00_JPRB, 0.12855E+00_JPRB, 0.12949E+00_JPRB ,&
&  0.13054E+00_JPRB, 0.13246E+00_JPRB, 0.13310E+00_JPRB, 0.13339E+00_JPRB, 0.13321E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.14701E+00_JPRB, 0.14764E+00_JPRB, 0.14794E+00_JPRB, 0.14834E+00_JPRB, 0.14924E+00_JPRB, 0.14926E+00_JPRB, 0.14911E+00_JPRB ,&
&  0.14729E+00_JPRB, 0.14651E+00_JPRB, 0.14531E+00_JPRB, 0.14369E+00_JPRB, 0.14206E+00_JPRB, 0.13952E+00_JPRB, 0.13780E+00_JPRB ,&
&  0.13608E+00_JPRB, 0.13428E+00_JPRB, 0.13304E+00_JPRB, 0.13152E+00_JPRB, 0.13022E+00_JPRB, 0.12881E+00_JPRB, 0.12831E+00_JPRB ,&
&  0.12765E+00_JPRB, 0.12762E+00_JPRB, 0.12832E+00_JPRB, 0.12894E+00_JPRB, 0.13003E+00_JPRB, 0.13119E+00_JPRB, 0.13267E+00_JPRB ,&
&  0.13322E+00_JPRB, 0.13404E+00_JPRB, 0.13478E+00_JPRB, 0.13524E+00_JPRB, 0.13666E+00_JPRB, 0.13770E+00_JPRB, 0.13804E+00_JPRB ,&
&  0.13761E+00_JPRB, 0.13689E+00_JPRB, 0.13542E+00_JPRB, 0.13377E+00_JPRB, 0.13181E+00_JPRB, 0.12961E+00_JPRB, 0.12752E+00_JPRB ,&
&  0.12586E+00_JPRB, 0.12435E+00_JPRB, 0.12319E+00_JPRB, 0.12225E+00_JPRB, 0.12166E+00_JPRB, 0.12118E+00_JPRB, 0.12073E+00_JPRB ,&
&  0.12045E+00_JPRB, 0.12021E+00_JPRB, 0.11987E+00_JPRB, 0.11980E+00_JPRB, 0.11967E+00_JPRB, 0.11966E+00_JPRB, 0.11965E+00_JPRB ,&
&  0.11963E+00_JPRB, 0.11953E+00_JPRB, 0.11937E+00_JPRB, 0.11906E+00_JPRB, 0.11875E+00_JPRB, 0.11842E+00_JPRB, 0.11809E+00_JPRB ,&
&  0.11774E+00_JPRB, 0.11766E+00_JPRB, 0.11739E+00_JPRB, 0.11719E+00_JPRB, 0.11695E+00_JPRB, 0.11658E+00_JPRB, 0.11648E+00_JPRB ,&
&  0.11657E+00_JPRB, 0.11670E+00_JPRB, 0.11663E+00_JPRB, 0.11678E+00_JPRB, 0.11697E+00_JPRB, 0.11726E+00_JPRB, 0.11793E+00_JPRB ,&
&  0.11836E+00_JPRB, 0.11881E+00_JPRB, 0.11925E+00_JPRB, 0.11984E+00_JPRB, 0.12083E+00_JPRB, 0.12182E+00_JPRB, 0.12292E+00_JPRB ,&
&  0.12423E+00_JPRB, 0.12549E+00_JPRB, 0.12673E+00_JPRB, 0.12797E+00_JPRB, 0.12998E+00_JPRB, 0.13149E+00_JPRB, 0.13348E+00_JPRB ,&
&  0.13526E+00_JPRB, 0.13481E+00_JPRB, 0.14221E+00_JPRB, 0.14314E+00_JPRB, 0.14444E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.15294E+00_JPRB, 0.15310E+00_JPRB, 0.15298E+00_JPRB, 0.15383E+00_JPRB, 0.15465E+00_JPRB, 0.15554E+00_JPRB, 0.15538E+00_JPRB ,&
&  0.15519E+00_JPRB, 0.15360E+00_JPRB, 0.15236E+00_JPRB, 0.14959E+00_JPRB, 0.14735E+00_JPRB, 0.14475E+00_JPRB, 0.14249E+00_JPRB ,&
&  0.14049E+00_JPRB, 0.13860E+00_JPRB, 0.13686E+00_JPRB, 0.13538E+00_JPRB, 0.13405E+00_JPRB, 0.13293E+00_JPRB, 0.13189E+00_JPRB ,&
&  0.13029E+00_JPRB, 0.12989E+00_JPRB, 0.12996E+00_JPRB, 0.13109E+00_JPRB, 0.13332E+00_JPRB, 0.13491E+00_JPRB, 0.13680E+00_JPRB ,&
&  0.13920E+00_JPRB, 0.13893E+00_JPRB, 0.14149E+00_JPRB, 0.14399E+00_JPRB, 0.14602E+00_JPRB, 0.14759E+00_JPRB, 0.14794E+00_JPRB ,&
&  0.14855E+00_JPRB, 0.14490E+00_JPRB, 0.14626E+00_JPRB, 0.14308E+00_JPRB, 0.13895E+00_JPRB, 0.13681E+00_JPRB, 0.13309E+00_JPRB ,&
&  0.13111E+00_JPRB, 0.12825E+00_JPRB, 0.12638E+00_JPRB, 0.12487E+00_JPRB, 0.12321E+00_JPRB, 0.12276E+00_JPRB, 0.12202E+00_JPRB ,&
&  0.12157E+00_JPRB, 0.12155E+00_JPRB, 0.12089E+00_JPRB, 0.12063E+00_JPRB, 0.12019E+00_JPRB, 0.11977E+00_JPRB, 0.11997E+00_JPRB ,&
&  0.12034E+00_JPRB, 0.12085E+00_JPRB, 0.12137E+00_JPRB, 0.12143E+00_JPRB, 0.12138E+00_JPRB, 0.12136E+00_JPRB, 0.12129E+00_JPRB ,&
&  0.12147E+00_JPRB, 0.12147E+00_JPRB, 0.12111E+00_JPRB, 0.12071E+00_JPRB, 0.11982E+00_JPRB, 0.11896E+00_JPRB, 0.11810E+00_JPRB ,&
&  0.11737E+00_JPRB, 0.11697E+00_JPRB, 0.11665E+00_JPRB, 0.11658E+00_JPRB, 0.11664E+00_JPRB, 0.11695E+00_JPRB, 0.11726E+00_JPRB ,&
&  0.11754E+00_JPRB, 0.11815E+00_JPRB, 0.11862E+00_JPRB, 0.11936E+00_JPRB, 0.12025E+00_JPRB, 0.12106E+00_JPRB, 0.12229E+00_JPRB ,&
&  0.12362E+00_JPRB, 0.12507E+00_JPRB, 0.12712E+00_JPRB, 0.12895E+00_JPRB, 0.13123E+00_JPRB, 0.13425E+00_JPRB, 0.13658E+00_JPRB ,&
&  0.13913E+00_JPRB, 0.14194E+00_JPRB, 0.14260E+00_JPRB, 0.14875E+00_JPRB, 0.15219E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.15173E+00_JPRB, 0.15294E+00_JPRB, 0.15434E+00_JPRB, 0.15571E+00_JPRB, 0.15693E+00_JPRB, 0.15815E+00_JPRB, 0.15896E+00_JPRB ,&
&  0.15682E+00_JPRB, 0.15547E+00_JPRB, 0.15300E+00_JPRB, 0.15063E+00_JPRB, 0.14799E+00_JPRB, 0.14518E+00_JPRB, 0.14412E+00_JPRB ,&
&  0.14280E+00_JPRB, 0.14250E+00_JPRB, 0.14149E+00_JPRB, 0.14107E+00_JPRB, 0.14029E+00_JPRB, 0.13988E+00_JPRB, 0.13917E+00_JPRB ,&
&  0.13760E+00_JPRB, 0.13676E+00_JPRB, 0.13595E+00_JPRB, 0.13635E+00_JPRB, 0.13770E+00_JPRB, 0.13919E+00_JPRB, 0.14078E+00_JPRB ,&
&  0.14304E+00_JPRB, 0.14423E+00_JPRB, 0.14641E+00_JPRB, 0.14801E+00_JPRB, 0.15046E+00_JPRB, 0.15249E+00_JPRB, 0.15286E+00_JPRB ,&
&  0.15292E+00_JPRB, 0.15175E+00_JPRB, 0.15158E+00_JPRB, 0.15003E+00_JPRB, 0.14470E+00_JPRB, 0.14150E+00_JPRB, 0.13711E+00_JPRB ,&
&  0.13222E+00_JPRB, 0.12853E+00_JPRB, 0.12601E+00_JPRB, 0.12512E+00_JPRB, 0.12433E+00_JPRB, 0.12405E+00_JPRB, 0.12378E+00_JPRB ,&
&  0.12308E+00_JPRB, 0.12226E+00_JPRB, 0.12132E+00_JPRB, 0.12075E+00_JPRB, 0.12099E+00_JPRB, 0.12157E+00_JPRB, 0.12282E+00_JPRB ,&
&  0.12410E+00_JPRB, 0.12481E+00_JPRB, 0.12559E+00_JPRB, 0.12586E+00_JPRB, 0.12597E+00_JPRB, 0.12616E+00_JPRB, 0.12623E+00_JPRB ,&
&  0.12676E+00_JPRB, 0.12690E+00_JPRB, 0.12645E+00_JPRB, 0.12571E+00_JPRB, 0.12390E+00_JPRB, 0.12156E+00_JPRB, 0.11965E+00_JPRB ,&
&  0.11807E+00_JPRB, 0.11703E+00_JPRB, 0.11666E+00_JPRB, 0.11631E+00_JPRB, 0.11653E+00_JPRB, 0.11687E+00_JPRB, 0.11718E+00_JPRB ,&
&  0.11772E+00_JPRB, 0.11818E+00_JPRB, 0.11863E+00_JPRB, 0.11960E+00_JPRB, 0.12040E+00_JPRB, 0.12156E+00_JPRB, 0.12277E+00_JPRB ,&
&  0.12361E+00_JPRB, 0.12490E+00_JPRB, 0.12598E+00_JPRB, 0.12718E+00_JPRB, 0.13070E+00_JPRB, 0.13361E+00_JPRB, 0.13947E+00_JPRB ,&
&  0.14357E+00_JPRB, 0.14595E+00_JPRB, 0.14777E+00_JPRB, 0.15203E+00_JPRB, 0.15161E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.15185E+00_JPRB, 0.15350E+00_JPRB, 0.15545E+00_JPRB, 0.15691E+00_JPRB, 0.15831E+00_JPRB, 0.15950E+00_JPRB, 0.16014E+00_JPRB ,&
&  0.16026E+00_JPRB, 0.15974E+00_JPRB, 0.15980E+00_JPRB, 0.15882E+00_JPRB, 0.15788E+00_JPRB, 0.15380E+00_JPRB, 0.15303E+00_JPRB ,&
&  0.15199E+00_JPRB, 0.15239E+00_JPRB, 0.15199E+00_JPRB, 0.15216E+00_JPRB, 0.15160E+00_JPRB, 0.15134E+00_JPRB, 0.15015E+00_JPRB ,&
&  0.14837E+00_JPRB, 0.14694E+00_JPRB, 0.14549E+00_JPRB, 0.14497E+00_JPRB, 0.14562E+00_JPRB, 0.14616E+00_JPRB, 0.14599E+00_JPRB ,&
&  0.14602E+00_JPRB, 0.14637E+00_JPRB, 0.14688E+00_JPRB, 0.14711E+00_JPRB, 0.15200E+00_JPRB, 0.15397E+00_JPRB, 0.15503E+00_JPRB ,&
&  0.15509E+00_JPRB, 0.15566E+00_JPRB, 0.15594E+00_JPRB, 0.15537E+00_JPRB, 0.15476E+00_JPRB, 0.15152E+00_JPRB, 0.14927E+00_JPRB ,&
&  0.14174E+00_JPRB, 0.13637E+00_JPRB, 0.13226E+00_JPRB, 0.12973E+00_JPRB, 0.12866E+00_JPRB, 0.12847E+00_JPRB, 0.12789E+00_JPRB ,&
&  0.12651E+00_JPRB, 0.12526E+00_JPRB, 0.12448E+00_JPRB, 0.12499E+00_JPRB, 0.12584E+00_JPRB, 0.12683E+00_JPRB, 0.12797E+00_JPRB ,&
&  0.12902E+00_JPRB, 0.12928E+00_JPRB, 0.12928E+00_JPRB, 0.12961E+00_JPRB, 0.12986E+00_JPRB, 0.13024E+00_JPRB, 0.13056E+00_JPRB ,&
&  0.13139E+00_JPRB, 0.13084E+00_JPRB, 0.13133E+00_JPRB, 0.13215E+00_JPRB, 0.13020E+00_JPRB, 0.12706E+00_JPRB, 0.12388E+00_JPRB ,&
&  0.12102E+00_JPRB, 0.11940E+00_JPRB, 0.11814E+00_JPRB, 0.11736E+00_JPRB, 0.11722E+00_JPRB, 0.11738E+00_JPRB, 0.11789E+00_JPRB ,&
&  0.11854E+00_JPRB, 0.11940E+00_JPRB, 0.11999E+00_JPRB, 0.12070E+00_JPRB, 0.12155E+00_JPRB, 0.12242E+00_JPRB, 0.12345E+00_JPRB ,&
&  0.12470E+00_JPRB, 0.12591E+00_JPRB, 0.12675E+00_JPRB, 0.12799E+00_JPRB, 0.13033E+00_JPRB, 0.13355E+00_JPRB, 0.13850E+00_JPRB ,&
&  0.14428E+00_JPRB, 0.14973E+00_JPRB, 0.14836E+00_JPRB, 0.14987E+00_JPRB, 0.15118E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.15709E+00_JPRB, 0.15927E+00_JPRB, 0.16092E+00_JPRB, 0.16242E+00_JPRB, 0.16355E+00_JPRB, 0.16483E+00_JPRB, 0.16540E+00_JPRB ,&
&  0.16592E+00_JPRB, 0.16591E+00_JPRB, 0.16625E+00_JPRB, 0.16615E+00_JPRB, 0.16626E+00_JPRB, 0.16525E+00_JPRB, 0.16462E+00_JPRB ,&
&  0.16419E+00_JPRB, 0.16438E+00_JPRB, 0.16424E+00_JPRB, 0.16399E+00_JPRB, 0.16323E+00_JPRB, 0.16229E+00_JPRB, 0.16090E+00_JPRB ,&
&  0.15818E+00_JPRB, 0.15653E+00_JPRB, 0.15695E+00_JPRB, 0.15667E+00_JPRB, 0.15711E+00_JPRB, 0.15724E+00_JPRB, 0.15771E+00_JPRB ,&
&  0.15422E+00_JPRB, 0.15275E+00_JPRB, 0.15249E+00_JPRB, 0.15177E+00_JPRB, 0.15449E+00_JPRB, 0.15566E+00_JPRB, 0.15735E+00_JPRB ,&
&  0.15799E+00_JPRB, 0.15851E+00_JPRB, 0.15925E+00_JPRB, 0.15900E+00_JPRB, 0.15940E+00_JPRB, 0.15982E+00_JPRB, 0.15879E+00_JPRB ,&
&  0.15623E+00_JPRB, 0.15407E+00_JPRB, 0.15114E+00_JPRB, 0.14375E+00_JPRB, 0.14149E+00_JPRB, 0.14135E+00_JPRB, 0.13980E+00_JPRB ,&
&  0.13725E+00_JPRB, 0.13722E+00_JPRB, 0.13682E+00_JPRB, 0.13562E+00_JPRB, 0.13706E+00_JPRB, 0.13916E+00_JPRB, 0.13934E+00_JPRB ,&
&  0.13841E+00_JPRB, 0.13819E+00_JPRB, 0.13764E+00_JPRB, 0.13658E+00_JPRB, 0.13531E+00_JPRB, 0.13475E+00_JPRB, 0.13426E+00_JPRB ,&
&  0.13406E+00_JPRB, 0.13317E+00_JPRB, 0.13628E+00_JPRB, 0.13732E+00_JPRB, 0.13325E+00_JPRB, 0.13393E+00_JPRB, 0.13163E+00_JPRB ,&
&  0.12615E+00_JPRB, 0.12361E+00_JPRB, 0.12229E+00_JPRB, 0.12123E+00_JPRB, 0.12110E+00_JPRB, 0.12134E+00_JPRB, 0.12138E+00_JPRB ,&
&  0.12221E+00_JPRB, 0.12319E+00_JPRB, 0.12371E+00_JPRB, 0.12430E+00_JPRB, 0.12507E+00_JPRB, 0.12616E+00_JPRB, 0.12757E+00_JPRB ,&
&  0.12876E+00_JPRB, 0.13046E+00_JPRB, 0.13139E+00_JPRB, 0.13290E+00_JPRB, 0.13403E+00_JPRB, 0.13538E+00_JPRB, 0.13850E+00_JPRB ,&
&  0.14127E+00_JPRB, 0.14661E+00_JPRB, 0.15411E+00_JPRB, 0.15705E+00_JPRB, 0.15512E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.16227E+00_JPRB, 0.16436E+00_JPRB, 0.16698E+00_JPRB, 0.16968E+00_JPRB, 0.17170E+00_JPRB, 0.17371E+00_JPRB, 0.17546E+00_JPRB ,&
&  0.17700E+00_JPRB, 0.17803E+00_JPRB, 0.17866E+00_JPRB, 0.17895E+00_JPRB, 0.17910E+00_JPRB, 0.17872E+00_JPRB, 0.17790E+00_JPRB ,&
&  0.17705E+00_JPRB, 0.17565E+00_JPRB, 0.17391E+00_JPRB, 0.17245E+00_JPRB, 0.17146E+00_JPRB, 0.17031E+00_JPRB, 0.16870E+00_JPRB ,&
&  0.16700E+00_JPRB, 0.16553E+00_JPRB, 0.16453E+00_JPRB, 0.16375E+00_JPRB, 0.16389E+00_JPRB, 0.16450E+00_JPRB, 0.16502E+00_JPRB ,&
&  0.16217E+00_JPRB, 0.15729E+00_JPRB, 0.15732E+00_JPRB, 0.15830E+00_JPRB, 0.15800E+00_JPRB, 0.15935E+00_JPRB, 0.16021E+00_JPRB ,&
&  0.16140E+00_JPRB, 0.16285E+00_JPRB, 0.16416E+00_JPRB, 0.16487E+00_JPRB, 0.16537E+00_JPRB, 0.16602E+00_JPRB, 0.16541E+00_JPRB ,&
&  0.16353E+00_JPRB, 0.16186E+00_JPRB, 0.16036E+00_JPRB, 0.15886E+00_JPRB, 0.15846E+00_JPRB, 0.15961E+00_JPRB, 0.16024E+00_JPRB ,&
&  0.15936E+00_JPRB, 0.15812E+00_JPRB, 0.15698E+00_JPRB, 0.15709E+00_JPRB, 0.15769E+00_JPRB, 0.15799E+00_JPRB, 0.15635E+00_JPRB ,&
&  0.15479E+00_JPRB, 0.15229E+00_JPRB, 0.14985E+00_JPRB, 0.14751E+00_JPRB, 0.14614E+00_JPRB, 0.14313E+00_JPRB, 0.14004E+00_JPRB ,&
&  0.13929E+00_JPRB, 0.14250E+00_JPRB, 0.14785E+00_JPRB, 0.15066E+00_JPRB, 0.14435E+00_JPRB, 0.14264E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.13732E+00_JPRB, 0.13293E+00_JPRB, 0.12989E+00_JPRB, 0.12861E+00_JPRB, 0.12818E+00_JPRB, 0.12785E+00_JPRB, 0.12833E+00_JPRB ,&
&  0.12922E+00_JPRB, 0.12984E+00_JPRB, 0.13048E+00_JPRB, 0.13151E+00_JPRB, 0.13248E+00_JPRB, 0.13344E+00_JPRB, 0.13425E+00_JPRB ,&
&  0.13591E+00_JPRB, 0.13796E+00_JPRB, 0.13982E+00_JPRB, 0.14239E+00_JPRB, 0.14515E+00_JPRB, 0.14797E+00_JPRB, 0.15097E+00_JPRB ,&
&  0.15433E+00_JPRB, 0.15674E+00_JPRB, 0.15803E+00_JPRB, 0.16019E+00_JPRB, 0.16118E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.17368E+00_JPRB, 0.17852E+00_JPRB, 0.18285E+00_JPRB, 0.18530E+00_JPRB, 0.18707E+00_JPRB, 0.18958E+00_JPRB, 0.19208E+00_JPRB ,&
&  0.19376E+00_JPRB, 0.19461E+00_JPRB, 0.19490E+00_JPRB, 0.19465E+00_JPRB, 0.19365E+00_JPRB, 0.19210E+00_JPRB, 0.19037E+00_JPRB ,&
&  0.18850E+00_JPRB, 0.18594E+00_JPRB, 0.18360E+00_JPRB, 0.18179E+00_JPRB, 0.18024E+00_JPRB, 0.17841E+00_JPRB, 0.17693E+00_JPRB ,&
&  0.17529E+00_JPRB, 0.17454E+00_JPRB, 0.17382E+00_JPRB, 0.17330E+00_JPRB, 0.17302E+00_JPRB, 0.17240E+00_JPRB, 0.17182E+00_JPRB ,&
&  0.17025E+00_JPRB, 0.16748E+00_JPRB, 0.16586E+00_JPRB, 0.16546E+00_JPRB, 0.16563E+00_JPRB, 0.16526E+00_JPRB, 0.16480E+00_JPRB ,&
&  0.16654E+00_JPRB, 0.16932E+00_JPRB, 0.17125E+00_JPRB, 0.17232E+00_JPRB, 0.17299E+00_JPRB, 0.17387E+00_JPRB, 0.17499E+00_JPRB ,&
&  0.17295E+00_JPRB, 0.17051E+00_JPRB, 0.16872E+00_JPRB, 0.16771E+00_JPRB, 0.16745E+00_JPRB, 0.16801E+00_JPRB, 0.16929E+00_JPRB ,&
&  0.17046E+00_JPRB, 0.17106E+00_JPRB, 0.17066E+00_JPRB, 0.16949E+00_JPRB, 0.16813E+00_JPRB, 0.16748E+00_JPRB, 0.16705E+00_JPRB ,&
&  0.16685E+00_JPRB, 0.16666E+00_JPRB, 0.16637E+00_JPRB, 0.16558E+00_JPRB, 0.16132E+00_JPRB, 0.15726E+00_JPRB, 0.15465E+00_JPRB ,&
&  0.15506E+00_JPRB, 0.15590E+00_JPRB, 0.15626E+00_JPRB, 0.16024E+00_JPRB, 0.16466E+00_JPRB, 0.16107E+00_JPRB, 0.15636E+00_JPRB ,&
&  0.15014E+00_JPRB, 0.14924E+00_JPRB, 0.14659E+00_JPRB, 0.14269E+00_JPRB, 0.14134E+00_JPRB, 0.14123E+00_JPRB, 0.14103E+00_JPRB ,&
&  0.14177E+00_JPRB, 0.14227E+00_JPRB, 0.14346E+00_JPRB, 0.14409E+00_JPRB, 0.14441E+00_JPRB, 0.14494E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14696E+00_JPRB, 0.14999E+00_JPRB, 0.15398E+00_JPRB, 0.15626E+00_JPRB, 0.15843E+00_JPRB, 0.16060E+00_JPRB, 0.16211E+00_JPRB ,&
&  0.16331E+00_JPRB, 0.16493E+00_JPRB, 0.16648E+00_JPRB, 0.16770E+00_JPRB, 0.16988E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.19018E+00_JPRB, 0.19439E+00_JPRB, 0.19693E+00_JPRB, 0.19940E+00_JPRB, 0.20218E+00_JPRB, 0.20663E+00_JPRB, 0.20971E+00_JPRB ,&
&  0.21027E+00_JPRB, 0.21015E+00_JPRB, 0.20960E+00_JPRB, 0.20842E+00_JPRB, 0.20664E+00_JPRB, 0.20506E+00_JPRB, 0.20245E+00_JPRB ,&
&  0.19925E+00_JPRB, 0.19634E+00_JPRB, 0.19375E+00_JPRB, 0.19126E+00_JPRB, 0.18897E+00_JPRB, 0.18687E+00_JPRB, 0.18515E+00_JPRB ,&
&  0.18445E+00_JPRB, 0.18393E+00_JPRB, 0.18287E+00_JPRB, 0.18255E+00_JPRB, 0.18271E+00_JPRB, 0.18242E+00_JPRB, 0.18144E+00_JPRB ,&
&  0.18020E+00_JPRB, 0.17787E+00_JPRB, 0.17547E+00_JPRB, 0.17437E+00_JPRB, 0.17467E+00_JPRB, 0.17414E+00_JPRB, 0.17339E+00_JPRB ,&
&  0.17470E+00_JPRB, 0.17676E+00_JPRB, 0.17895E+00_JPRB, 0.17993E+00_JPRB, 0.18051E+00_JPRB, 0.18141E+00_JPRB, 0.18261E+00_JPRB ,&
&  0.18284E+00_JPRB, 0.18066E+00_JPRB, 0.17868E+00_JPRB, 0.17820E+00_JPRB, 0.17794E+00_JPRB, 0.17752E+00_JPRB, 0.17771E+00_JPRB ,&
&  0.17820E+00_JPRB, 0.17930E+00_JPRB, 0.18007E+00_JPRB, 0.17929E+00_JPRB, 0.17787E+00_JPRB, 0.17685E+00_JPRB, 0.17616E+00_JPRB ,&
&  0.17575E+00_JPRB, 0.17526E+00_JPRB, 0.17439E+00_JPRB, 0.17308E+00_JPRB, 0.17169E+00_JPRB, 0.17050E+00_JPRB, 0.16935E+00_JPRB ,&
&  0.16833E+00_JPRB, 0.16854E+00_JPRB, 0.17039E+00_JPRB, 0.17183E+00_JPRB, 0.17186E+00_JPRB, 0.17064E+00_JPRB, 0.16859E+00_JPRB ,&
&  0.16708E+00_JPRB, 0.16515E+00_JPRB, 0.16365E+00_JPRB, 0.16244E+00_JPRB, 0.16146E+00_JPRB, 0.16011E+00_JPRB, 0.15957E+00_JPRB ,&
&  0.15969E+00_JPRB, 0.16104E+00_JPRB, 0.16171E+00_JPRB, 0.16213E+00_JPRB, 0.16201E+00_JPRB, 0.16189E+00_JPRB, 0.16273E+00_JPRB ,&
&  0.16387E+00_JPRB, 0.16553E+00_JPRB, 0.16724E+00_JPRB, 0.16949E+00_JPRB, 0.17186E+00_JPRB, 0.17500E+00_JPRB, 0.17764E+00_JPRB ,&
&  0.17925E+00_JPRB, 0.17975E+00_JPRB, 0.18090E+00_JPRB, 0.18256E+00_JPRB, 0.18544E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.20549E+00_JPRB, 0.20854E+00_JPRB, 0.21109E+00_JPRB, 0.21454E+00_JPRB, 0.21773E+00_JPRB, 0.22218E+00_JPRB, 0.22395E+00_JPRB ,&
&  0.22363E+00_JPRB, 0.22297E+00_JPRB, 0.22129E+00_JPRB, 0.21881E+00_JPRB, 0.21508E+00_JPRB, 0.21140E+00_JPRB, 0.20800E+00_JPRB ,&
&  0.20547E+00_JPRB, 0.20339E+00_JPRB, 0.20170E+00_JPRB, 0.20014E+00_JPRB, 0.19797E+00_JPRB, 0.19592E+00_JPRB, 0.19431E+00_JPRB ,&
&  0.19345E+00_JPRB, 0.19255E+00_JPRB, 0.19206E+00_JPRB, 0.19178E+00_JPRB, 0.19166E+00_JPRB, 0.19170E+00_JPRB, 0.19187E+00_JPRB ,&
&  0.19103E+00_JPRB, 0.18807E+00_JPRB, 0.18645E+00_JPRB, 0.18600E+00_JPRB, 0.18568E+00_JPRB, 0.18622E+00_JPRB, 0.18615E+00_JPRB ,&
&  0.18614E+00_JPRB, 0.18695E+00_JPRB, 0.18839E+00_JPRB, 0.18990E+00_JPRB, 0.18996E+00_JPRB, 0.19040E+00_JPRB, 0.19143E+00_JPRB ,&
&  0.19187E+00_JPRB, 0.19139E+00_JPRB, 0.18982E+00_JPRB, 0.18917E+00_JPRB, 0.18949E+00_JPRB, 0.18929E+00_JPRB, 0.18956E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19110E+00_JPRB, 0.19152E+00_JPRB, 0.19017E+00_JPRB, 0.18892E+00_JPRB, 0.18848E+00_JPRB, 0.18845E+00_JPRB ,&
&  0.18808E+00_JPRB, 0.18713E+00_JPRB, 0.18528E+00_JPRB, 0.18374E+00_JPRB, 0.18137E+00_JPRB, 0.17946E+00_JPRB, 0.17852E+00_JPRB ,&
&  0.17906E+00_JPRB, 0.17951E+00_JPRB, 0.18077E+00_JPRB, 0.18194E+00_JPRB, 0.18269E+00_JPRB, 0.18286E+00_JPRB, 0.18132E+00_JPRB ,&
&  0.17923E+00_JPRB, 0.17830E+00_JPRB, 0.17739E+00_JPRB, 0.17661E+00_JPRB, 0.17673E+00_JPRB, 0.17662E+00_JPRB, 0.17621E+00_JPRB ,&
&  0.17568E+00_JPRB, 0.17591E+00_JPRB, 0.17629E+00_JPRB, 0.17654E+00_JPRB, 0.17651E+00_JPRB, 0.17602E+00_JPRB, 0.17658E+00_JPRB ,&
&  0.17785E+00_JPRB, 0.17960E+00_JPRB, 0.18164E+00_JPRB, 0.18389E+00_JPRB, 0.18639E+00_JPRB, 0.18871E+00_JPRB, 0.19086E+00_JPRB ,&
&  0.19362E+00_JPRB, 0.19476E+00_JPRB, 0.19644E+00_JPRB, 0.19910E+00_JPRB, 0.20181E+00_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.21802E+00_JPRB, 0.22064E+00_JPRB, 0.22499E+00_JPRB, 0.22983E+00_JPRB, 0.23323E+00_JPRB, 0.23702E+00_JPRB, 0.23749E+00_JPRB ,&
&  0.23652E+00_JPRB, 0.23455E+00_JPRB, 0.23060E+00_JPRB, 0.22592E+00_JPRB, 0.22166E+00_JPRB, 0.21820E+00_JPRB, 0.21588E+00_JPRB ,&
&  0.21455E+00_JPRB, 0.21344E+00_JPRB, 0.21213E+00_JPRB, 0.20992E+00_JPRB, 0.20713E+00_JPRB, 0.20458E+00_JPRB, 0.20249E+00_JPRB ,&
&  0.20155E+00_JPRB, 0.20121E+00_JPRB, 0.20134E+00_JPRB, 0.20114E+00_JPRB, 0.20118E+00_JPRB, 0.20260E+00_JPRB, 0.20428E+00_JPRB ,&
&  0.20340E+00_JPRB, 0.19949E+00_JPRB, 0.19723E+00_JPRB, 0.19713E+00_JPRB, 0.19742E+00_JPRB, 0.19809E+00_JPRB, 0.19898E+00_JPRB ,&
&  0.19945E+00_JPRB, 0.20032E+00_JPRB, 0.20144E+00_JPRB, 0.20279E+00_JPRB, 0.20365E+00_JPRB, 0.20594E+00_JPRB, 0.20715E+00_JPRB ,&
&  0.20731E+00_JPRB, 0.20667E+00_JPRB, 0.20538E+00_JPRB, 0.20284E+00_JPRB, 0.20091E+00_JPRB, 0.20015E+00_JPRB, 0.19940E+00_JPRB ,&
&  0.19996E+00_JPRB, 0.20054E+00_JPRB, 0.20105E+00_JPRB, 0.20039E+00_JPRB, 0.19894E+00_JPRB, 0.19878E+00_JPRB, 0.19922E+00_JPRB ,&
&  0.19910E+00_JPRB, 0.19794E+00_JPRB, 0.19635E+00_JPRB, 0.19548E+00_JPRB, 0.19461E+00_JPRB, 0.19445E+00_JPRB, 0.19445E+00_JPRB ,&
&  0.19436E+00_JPRB, 0.19358E+00_JPRB, 0.19312E+00_JPRB, 0.19300E+00_JPRB, 0.19295E+00_JPRB, 0.19335E+00_JPRB, 0.19305E+00_JPRB ,&
&  0.19297E+00_JPRB, 0.19313E+00_JPRB, 0.19222E+00_JPRB, 0.19200E+00_JPRB, 0.19202E+00_JPRB, 0.19234E+00_JPRB, 0.19244E+00_JPRB ,&
&  0.19218E+00_JPRB, 0.19177E+00_JPRB, 0.19144E+00_JPRB, 0.19150E+00_JPRB, 0.19120E+00_JPRB, 0.19080E+00_JPRB, 0.19159E+00_JPRB ,&
&  0.19450E+00_JPRB, 0.19755E+00_JPRB, 0.20021E+00_JPRB, 0.20200E+00_JPRB, 0.20267E+00_JPRB, 0.20407E+00_JPRB, 0.20660E+00_JPRB ,&
&  0.20887E+00_JPRB, 0.20983E+00_JPRB, 0.21161E+00_JPRB, 0.21316E+00_JPRB, 0.21504E+00_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.23245E+00_JPRB, 0.23534E+00_JPRB, 0.23944E+00_JPRB, 0.24533E+00_JPRB, 0.24869E+00_JPRB, 0.25144E+00_JPRB, 0.25091E+00_JPRB ,&
&  0.24842E+00_JPRB, 0.24491E+00_JPRB, 0.24078E+00_JPRB, 0.23590E+00_JPRB, 0.23089E+00_JPRB, 0.22710E+00_JPRB, 0.22566E+00_JPRB ,&
&  0.22552E+00_JPRB, 0.22478E+00_JPRB, 0.22260E+00_JPRB, 0.21951E+00_JPRB, 0.21600E+00_JPRB, 0.21321E+00_JPRB, 0.21096E+00_JPRB ,&
&  0.21008E+00_JPRB, 0.21022E+00_JPRB, 0.21087E+00_JPRB, 0.21124E+00_JPRB, 0.21161E+00_JPRB, 0.21414E+00_JPRB, 0.21798E+00_JPRB ,&
&  0.21592E+00_JPRB, 0.21350E+00_JPRB, 0.20913E+00_JPRB, 0.20874E+00_JPRB, 0.20997E+00_JPRB, 0.21222E+00_JPRB, 0.21436E+00_JPRB ,&
&  0.21604E+00_JPRB, 0.21850E+00_JPRB, 0.22096E+00_JPRB, 0.22293E+00_JPRB, 0.22439E+00_JPRB, 0.22583E+00_JPRB, 0.22672E+00_JPRB ,&
&  0.22693E+00_JPRB, 0.22583E+00_JPRB, 0.22405E+00_JPRB, 0.22085E+00_JPRB, 0.21671E+00_JPRB, 0.21393E+00_JPRB, 0.21238E+00_JPRB ,&
&  0.21072E+00_JPRB, 0.21035E+00_JPRB, 0.21028E+00_JPRB, 0.20979E+00_JPRB, 0.20905E+00_JPRB, 0.20888E+00_JPRB, 0.20939E+00_JPRB ,&
&  0.20965E+00_JPRB, 0.20921E+00_JPRB, 0.20843E+00_JPRB, 0.20834E+00_JPRB, 0.20885E+00_JPRB, 0.20998E+00_JPRB, 0.21106E+00_JPRB ,&
&  0.21180E+00_JPRB, 0.21165E+00_JPRB, 0.21112E+00_JPRB, 0.20961E+00_JPRB, 0.20850E+00_JPRB, 0.20864E+00_JPRB, 0.20966E+00_JPRB ,&
&  0.21026E+00_JPRB, 0.21026E+00_JPRB, 0.21051E+00_JPRB, 0.20968E+00_JPRB, 0.20900E+00_JPRB, 0.20917E+00_JPRB, 0.20965E+00_JPRB ,&
&  0.20985E+00_JPRB, 0.20972E+00_JPRB, 0.20937E+00_JPRB, 0.20916E+00_JPRB, 0.20953E+00_JPRB, 0.20965E+00_JPRB, 0.21057E+00_JPRB ,&
&  0.21292E+00_JPRB, 0.21560E+00_JPRB, 0.21837E+00_JPRB, 0.22052E+00_JPRB, 0.22185E+00_JPRB, 0.22223E+00_JPRB, 0.22427E+00_JPRB ,&
&  0.22721E+00_JPRB, 0.22915E+00_JPRB, 0.22887E+00_JPRB, 0.22843E+00_JPRB, 0.23009E+00_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.24944E+00_JPRB, 0.25081E+00_JPRB, 0.25535E+00_JPRB, 0.26034E+00_JPRB, 0.26523E+00_JPRB, 0.26596E+00_JPRB, 0.26328E+00_JPRB ,&
&  0.26036E+00_JPRB, 0.25647E+00_JPRB, 0.25101E+00_JPRB, 0.24450E+00_JPRB, 0.23855E+00_JPRB, 0.23677E+00_JPRB, 0.23667E+00_JPRB ,&
&  0.23686E+00_JPRB, 0.23573E+00_JPRB, 0.23307E+00_JPRB, 0.22991E+00_JPRB, 0.22611E+00_JPRB, 0.22277E+00_JPRB, 0.22027E+00_JPRB ,&
&  0.21948E+00_JPRB, 0.21914E+00_JPRB, 0.21957E+00_JPRB, 0.21974E+00_JPRB, 0.22062E+00_JPRB, 0.22376E+00_JPRB, 0.22914E+00_JPRB ,&
&  0.22687E+00_JPRB, 0.22690E+00_JPRB, 0.22111E+00_JPRB, 0.22347E+00_JPRB, 0.22555E+00_JPRB, 0.22985E+00_JPRB, 0.23438E+00_JPRB ,&
&  0.23866E+00_JPRB, 0.24289E+00_JPRB, 0.24566E+00_JPRB, 0.24630E+00_JPRB, 0.24539E+00_JPRB, 0.24529E+00_JPRB, 0.24499E+00_JPRB ,&
&  0.24422E+00_JPRB, 0.24315E+00_JPRB, 0.24132E+00_JPRB, 0.23788E+00_JPRB, 0.23310E+00_JPRB, 0.22883E+00_JPRB, 0.22569E+00_JPRB ,&
&  0.22482E+00_JPRB, 0.22433E+00_JPRB, 0.22303E+00_JPRB, 0.22184E+00_JPRB, 0.22065E+00_JPRB, 0.22000E+00_JPRB, 0.22058E+00_JPRB ,&
&  0.22176E+00_JPRB, 0.22278E+00_JPRB, 0.22359E+00_JPRB, 0.22475E+00_JPRB, 0.22669E+00_JPRB, 0.22849E+00_JPRB, 0.22967E+00_JPRB ,&
&  0.22995E+00_JPRB, 0.22947E+00_JPRB, 0.22907E+00_JPRB, 0.22857E+00_JPRB, 0.22823E+00_JPRB, 0.22775E+00_JPRB, 0.22774E+00_JPRB ,&
&  0.22793E+00_JPRB, 0.22797E+00_JPRB, 0.22743E+00_JPRB, 0.22642E+00_JPRB, 0.22555E+00_JPRB, 0.22483E+00_JPRB, 0.22466E+00_JPRB ,&
&  0.22503E+00_JPRB, 0.22545E+00_JPRB, 0.22540E+00_JPRB, 0.22505E+00_JPRB, 0.22531E+00_JPRB, 0.22582E+00_JPRB, 0.22707E+00_JPRB ,&
&  0.22898E+00_JPRB, 0.23157E+00_JPRB, 0.23498E+00_JPRB, 0.23869E+00_JPRB, 0.23995E+00_JPRB, 0.24119E+00_JPRB, 0.24393E+00_JPRB ,&
&  0.24646E+00_JPRB, 0.24731E+00_JPRB, 0.24759E+00_JPRB, 0.24795E+00_JPRB, 0.24912E+00_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.27057E+00_JPRB, 0.26953E+00_JPRB, 0.27199E+00_JPRB, 0.27657E+00_JPRB, 0.28220E+00_JPRB, 0.28042E+00_JPRB, 0.27757E+00_JPRB ,&
&  0.27200E+00_JPRB, 0.26513E+00_JPRB, 0.25856E+00_JPRB, 0.25242E+00_JPRB, 0.24926E+00_JPRB, 0.24920E+00_JPRB, 0.24875E+00_JPRB ,&
&  0.24761E+00_JPRB, 0.24509E+00_JPRB, 0.24139E+00_JPRB, 0.23765E+00_JPRB, 0.23487E+00_JPRB, 0.23342E+00_JPRB, 0.23216E+00_JPRB ,&
&  0.23138E+00_JPRB, 0.23052E+00_JPRB, 0.23153E+00_JPRB, 0.23169E+00_JPRB, 0.23273E+00_JPRB, 0.23696E+00_JPRB, 0.24022E+00_JPRB ,&
&  0.24206E+00_JPRB, 0.24253E+00_JPRB, 0.24300E+00_JPRB, 0.24370E+00_JPRB, 0.24651E+00_JPRB, 0.25317E+00_JPRB, 0.26055E+00_JPRB ,&
&  0.26689E+00_JPRB, 0.27171E+00_JPRB, 0.27425E+00_JPRB, 0.27374E+00_JPRB, 0.27139E+00_JPRB, 0.26859E+00_JPRB, 0.26659E+00_JPRB ,&
&  0.26457E+00_JPRB, 0.26108E+00_JPRB, 0.25738E+00_JPRB, 0.25429E+00_JPRB, 0.25273E+00_JPRB, 0.25148E+00_JPRB, 0.24984E+00_JPRB ,&
&  0.24779E+00_JPRB, 0.24589E+00_JPRB, 0.24334E+00_JPRB, 0.24073E+00_JPRB, 0.23847E+00_JPRB, 0.23734E+00_JPRB, 0.23775E+00_JPRB ,&
&  0.23945E+00_JPRB, 0.24201E+00_JPRB, 0.24502E+00_JPRB, 0.24849E+00_JPRB, 0.25091E+00_JPRB, 0.25270E+00_JPRB, 0.25303E+00_JPRB ,&
&  0.25283E+00_JPRB, 0.25243E+00_JPRB, 0.25210E+00_JPRB, 0.25180E+00_JPRB, 0.25123E+00_JPRB, 0.25023E+00_JPRB, 0.24906E+00_JPRB ,&
&  0.24803E+00_JPRB, 0.24732E+00_JPRB, 0.24680E+00_JPRB, 0.24635E+00_JPRB, 0.24506E+00_JPRB, 0.24333E+00_JPRB, 0.24221E+00_JPRB ,&
&  0.24168E+00_JPRB, 0.24150E+00_JPRB, 0.24139E+00_JPRB, 0.24094E+00_JPRB, 0.24072E+00_JPRB, 0.24130E+00_JPRB, 0.24283E+00_JPRB ,&
&  0.24542E+00_JPRB, 0.24906E+00_JPRB, 0.25352E+00_JPRB, 0.25869E+00_JPRB, 0.26300E+00_JPRB, 0.26709E+00_JPRB, 0.27097E+00_JPRB ,&
&  0.27310E+00_JPRB, 0.27243E+00_JPRB, 0.27234E+00_JPRB, 0.27304E+00_JPRB, 0.27246E+00_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.29595E+00_JPRB, 0.29229E+00_JPRB, 0.29175E+00_JPRB, 0.29489E+00_JPRB, 0.29774E+00_JPRB, 0.29602E+00_JPRB, 0.29134E+00_JPRB ,&
&  0.28217E+00_JPRB, 0.27410E+00_JPRB, 0.26733E+00_JPRB, 0.26373E+00_JPRB, 0.26438E+00_JPRB, 0.26411E+00_JPRB, 0.26181E+00_JPRB ,&
&  0.25952E+00_JPRB, 0.25566E+00_JPRB, 0.25255E+00_JPRB, 0.24944E+00_JPRB, 0.24711E+00_JPRB, 0.24612E+00_JPRB, 0.24587E+00_JPRB ,&
&  0.24481E+00_JPRB, 0.24370E+00_JPRB, 0.24414E+00_JPRB, 0.24504E+00_JPRB, 0.24762E+00_JPRB, 0.25262E+00_JPRB, 0.25824E+00_JPRB ,&
&  0.26717E+00_JPRB, 0.25222E+00_JPRB, 0.26778E+00_JPRB, 0.27249E+00_JPRB, 0.27558E+00_JPRB, 0.28115E+00_JPRB, 0.28721E+00_JPRB ,&
&  0.29280E+00_JPRB, 0.29681E+00_JPRB, 0.29911E+00_JPRB, 0.29896E+00_JPRB, 0.29650E+00_JPRB, 0.29406E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.28541E+00_JPRB, 0.28363E+00_JPRB, 0.28272E+00_JPRB, 0.28068E+00_JPRB, 0.27833E+00_JPRB, 0.27500E+00_JPRB, 0.27161E+00_JPRB ,&
&  0.26847E+00_JPRB, 0.26550E+00_JPRB, 0.26318E+00_JPRB, 0.26108E+00_JPRB, 0.25959E+00_JPRB, 0.25921E+00_JPRB, 0.26019E+00_JPRB ,&
&  0.26169E+00_JPRB, 0.26393E+00_JPRB, 0.26860E+00_JPRB, 0.27343E+00_JPRB, 0.27657E+00_JPRB, 0.27814E+00_JPRB, 0.27883E+00_JPRB ,&
&  0.27884E+00_JPRB, 0.27854E+00_JPRB, 0.27789E+00_JPRB, 0.27701E+00_JPRB, 0.27555E+00_JPRB, 0.27397E+00_JPRB, 0.27121E+00_JPRB ,&
&  0.26900E+00_JPRB, 0.26734E+00_JPRB, 0.26556E+00_JPRB, 0.26440E+00_JPRB, 0.26285E+00_JPRB, 0.26052E+00_JPRB, 0.25814E+00_JPRB ,&
&  0.25626E+00_JPRB, 0.25492E+00_JPRB, 0.25394E+00_JPRB, 0.25354E+00_JPRB, 0.25371E+00_JPRB, 0.25541E+00_JPRB, 0.25742E+00_JPRB ,&
&  0.26021E+00_JPRB, 0.26455E+00_JPRB, 0.27075E+00_JPRB, 0.27788E+00_JPRB, 0.28548E+00_JPRB, 0.29019E+00_JPRB, 0.29418E+00_JPRB ,&
&  0.29631E+00_JPRB, 0.29582E+00_JPRB, 0.29642E+00_JPRB, 0.29715E+00_JPRB, 0.29734E+00_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.32343E+00_JPRB, 0.32078E+00_JPRB, 0.31720E+00_JPRB, 0.31813E+00_JPRB, 0.31589E+00_JPRB, 0.31129E+00_JPRB, 0.30442E+00_JPRB ,&
&  0.29427E+00_JPRB, 0.28682E+00_JPRB, 0.28096E+00_JPRB, 0.27881E+00_JPRB, 0.28068E+00_JPRB, 0.27889E+00_JPRB, 0.27527E+00_JPRB ,&
&  0.27257E+00_JPRB, 0.26948E+00_JPRB, 0.26733E+00_JPRB, 0.26482E+00_JPRB, 0.26292E+00_JPRB, 0.26239E+00_JPRB, 0.26209E+00_JPRB ,&
&  0.26088E+00_JPRB, 0.25912E+00_JPRB, 0.25798E+00_JPRB, 0.25813E+00_JPRB, 0.26172E+00_JPRB, 0.26858E+00_JPRB, 0.27927E+00_JPRB ,&
&  0.27671E+00_JPRB, 0.25890E+00_JPRB, 0.27454E+00_JPRB, 0.29407E+00_JPRB, 0.30572E+00_JPRB, 0.31520E+00_JPRB, 0.32156E+00_JPRB ,&
&  0.32724E+00_JPRB, 0.32875E+00_JPRB, 0.32960E+00_JPRB, 0.32900E+00_JPRB, 0.32630E+00_JPRB, 0.32138E+00_JPRB, 0.31646E+00_JPRB ,&
&  0.31365E+00_JPRB, 0.31208E+00_JPRB, 0.31013E+00_JPRB, 0.30572E+00_JPRB, 0.30062E+00_JPRB, 0.29486E+00_JPRB, 0.28947E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.28148E+00_JPRB, 0.27848E+00_JPRB, 0.27714E+00_JPRB, 0.27778E+00_JPRB, 0.27983E+00_JPRB, 0.28357E+00_JPRB ,&
&  0.28795E+00_JPRB, 0.29444E+00_JPRB, 0.30288E+00_JPRB, 0.30884E+00_JPRB, 0.31222E+00_JPRB, 0.31293E+00_JPRB, 0.31284E+00_JPRB ,&
&  0.31105E+00_JPRB, 0.30910E+00_JPRB, 0.30816E+00_JPRB, 0.30657E+00_JPRB, 0.30418E+00_JPRB, 0.30075E+00_JPRB, 0.29615E+00_JPRB ,&
&  0.29239E+00_JPRB, 0.28940E+00_JPRB, 0.28652E+00_JPRB, 0.28390E+00_JPRB, 0.28121E+00_JPRB, 0.27825E+00_JPRB, 0.27564E+00_JPRB ,&
&  0.27283E+00_JPRB, 0.26993E+00_JPRB, 0.26798E+00_JPRB, 0.26791E+00_JPRB, 0.27027E+00_JPRB, 0.27333E+00_JPRB, 0.27731E+00_JPRB ,&
&  0.28187E+00_JPRB, 0.28886E+00_JPRB, 0.29683E+00_JPRB, 0.30678E+00_JPRB, 0.31484E+00_JPRB, 0.31731E+00_JPRB, 0.31960E+00_JPRB ,&
&  0.32142E+00_JPRB, 0.31986E+00_JPRB, 0.32190E+00_JPRB, 0.32228E+00_JPRB, 0.32192E+00_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.34886E+00_JPRB, 0.35341E+00_JPRB, 0.35286E+00_JPRB, 0.34638E+00_JPRB, 0.33660E+00_JPRB, 0.32974E+00_JPRB, 0.31973E+00_JPRB ,&
&  0.30910E+00_JPRB, 0.30255E+00_JPRB, 0.29512E+00_JPRB, 0.29198E+00_JPRB, 0.29169E+00_JPRB, 0.28826E+00_JPRB, 0.28566E+00_JPRB ,&
&  0.28351E+00_JPRB, 0.27964E+00_JPRB, 0.27726E+00_JPRB, 0.27487E+00_JPRB, 0.27468E+00_JPRB, 0.27479E+00_JPRB, 0.27476E+00_JPRB ,&
&  0.27403E+00_JPRB, 0.27239E+00_JPRB, 0.27114E+00_JPRB, 0.27079E+00_JPRB, 0.27381E+00_JPRB, 0.28112E+00_JPRB, 0.29450E+00_JPRB ,&
&  0.32206E+00_JPRB, 0.28782E+00_JPRB, 0.29837E+00_JPRB, 0.31203E+00_JPRB, 0.33487E+00_JPRB, 0.34790E+00_JPRB, 0.35219E+00_JPRB ,&
&  0.35712E+00_JPRB, 0.36295E+00_JPRB, 0.36490E+00_JPRB, 0.36622E+00_JPRB, 0.36034E+00_JPRB, 0.35317E+00_JPRB, 0.34901E+00_JPRB ,&
&  0.34403E+00_JPRB, 0.33759E+00_JPRB, 0.33088E+00_JPRB, 0.32337E+00_JPRB, 0.31874E+00_JPRB, 0.31527E+00_JPRB, 0.31161E+00_JPRB ,&
&  0.30690E+00_JPRB, 0.30149E+00_JPRB, 0.29765E+00_JPRB, 0.29658E+00_JPRB, 0.29902E+00_JPRB, 0.30495E+00_JPRB, 0.31568E+00_JPRB ,&
&  0.32574E+00_JPRB, 0.33678E+00_JPRB, 0.34676E+00_JPRB, 0.35202E+00_JPRB, 0.35406E+00_JPRB, 0.35226E+00_JPRB, 0.34891E+00_JPRB ,&
&  0.34447E+00_JPRB, 0.34004E+00_JPRB, 0.33693E+00_JPRB, 0.33260E+00_JPRB, 0.32690E+00_JPRB, 0.32247E+00_JPRB, 0.31920E+00_JPRB ,&
&  0.31693E+00_JPRB, 0.31367E+00_JPRB, 0.30940E+00_JPRB, 0.30514E+00_JPRB, 0.30066E+00_JPRB, 0.29698E+00_JPRB, 0.29381E+00_JPRB ,&
&  0.29078E+00_JPRB, 0.28766E+00_JPRB, 0.28619E+00_JPRB, 0.28784E+00_JPRB, 0.29120E+00_JPRB, 0.29516E+00_JPRB, 0.29892E+00_JPRB ,&
&  0.30448E+00_JPRB, 0.31286E+00_JPRB, 0.32071E+00_JPRB, 0.33332E+00_JPRB, 0.33900E+00_JPRB, 0.34188E+00_JPRB, 0.34468E+00_JPRB ,&
&  0.34492E+00_JPRB, 0.34349E+00_JPRB, 0.31098E+00_JPRB, 0.33381E+00_JPRB, 0.34272E+00_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.38628E+00_JPRB, 0.39022E+00_JPRB, 0.38689E+00_JPRB, 0.37279E+00_JPRB, 0.35799E+00_JPRB, 0.34892E+00_JPRB, 0.33837E+00_JPRB ,&
&  0.32710E+00_JPRB, 0.31785E+00_JPRB, 0.30736E+00_JPRB, 0.30348E+00_JPRB, 0.30086E+00_JPRB, 0.29955E+00_JPRB, 0.29809E+00_JPRB ,&
&  0.29468E+00_JPRB, 0.28908E+00_JPRB, 0.28456E+00_JPRB, 0.28253E+00_JPRB, 0.28284E+00_JPRB, 0.28386E+00_JPRB, 0.28533E+00_JPRB ,&
&  0.28512E+00_JPRB, 0.28529E+00_JPRB, 0.28516E+00_JPRB, 0.28575E+00_JPRB, 0.28807E+00_JPRB, 0.29583E+00_JPRB, 0.31089E+00_JPRB ,&
&  0.34666E+00_JPRB, 0.31345E+00_JPRB, 0.32126E+00_JPRB, 0.33853E+00_JPRB, 0.35628E+00_JPRB, 0.37852E+00_JPRB, 0.39730E+00_JPRB ,&
&  0.40373E+00_JPRB, 0.40009E+00_JPRB, 0.40401E+00_JPRB, 0.41272E+00_JPRB, 0.41036E+00_JPRB, 0.40571E+00_JPRB, 0.39825E+00_JPRB ,&
&  0.38618E+00_JPRB, 0.37225E+00_JPRB, 0.35798E+00_JPRB, 0.34854E+00_JPRB, 0.34447E+00_JPRB, 0.34173E+00_JPRB, 0.34086E+00_JPRB ,&
&  0.33663E+00_JPRB, 0.33395E+00_JPRB, 0.33668E+00_JPRB, 0.34051E+00_JPRB, 0.34404E+00_JPRB, 0.35319E+00_JPRB, 0.36735E+00_JPRB ,&
&  0.37718E+00_JPRB, 0.38781E+00_JPRB, 0.39542E+00_JPRB, 0.39726E+00_JPRB, 0.39829E+00_JPRB, 0.39538E+00_JPRB, 0.38611E+00_JPRB ,&
&  0.37662E+00_JPRB, 0.36691E+00_JPRB, 0.35867E+00_JPRB, 0.35112E+00_JPRB, 0.34564E+00_JPRB, 0.34354E+00_JPRB, 0.34251E+00_JPRB ,&
&  0.34164E+00_JPRB, 0.34105E+00_JPRB, 0.33876E+00_JPRB, 0.33530E+00_JPRB, 0.33229E+00_JPRB, 0.33042E+00_JPRB, 0.32864E+00_JPRB ,&
&  0.32644E+00_JPRB, 0.32407E+00_JPRB, 0.32352E+00_JPRB, 0.32566E+00_JPRB, 0.32831E+00_JPRB, 0.33198E+00_JPRB, 0.33668E+00_JPRB ,&
&  0.34136E+00_JPRB, 0.34973E+00_JPRB, 0.35611E+00_JPRB, 0.36763E+00_JPRB, 0.37402E+00_JPRB, 0.37434E+00_JPRB, 0.37546E+00_JPRB ,&
&  0.37298E+00_JPRB, 0.37063E+00_JPRB, 0.36960E+00_JPRB, 0.35364E+00_JPRB, 0.37817E+00_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.43424E+00_JPRB, 0.43169E+00_JPRB, 0.41540E+00_JPRB, 0.39413E+00_JPRB, 0.37751E+00_JPRB, 0.36273E+00_JPRB, 0.35027E+00_JPRB ,&
&  0.34197E+00_JPRB, 0.33301E+00_JPRB, 0.32329E+00_JPRB, 0.31609E+00_JPRB, 0.31476E+00_JPRB, 0.31224E+00_JPRB, 0.30900E+00_JPRB ,&
&  0.30319E+00_JPRB, 0.29769E+00_JPRB, 0.29402E+00_JPRB, 0.29254E+00_JPRB, 0.29283E+00_JPRB, 0.29448E+00_JPRB, 0.29577E+00_JPRB ,&
&  0.29507E+00_JPRB, 0.29268E+00_JPRB, 0.29364E+00_JPRB, 0.29655E+00_JPRB, 0.30171E+00_JPRB, 0.30921E+00_JPRB, 0.32725E+00_JPRB ,&
&  0.36678E+00_JPRB, 0.34106E+00_JPRB, 0.35608E+00_JPRB, 0.36415E+00_JPRB, 0.37172E+00_JPRB, 0.39363E+00_JPRB, 0.41422E+00_JPRB ,&
&  0.42870E+00_JPRB, 0.43856E+00_JPRB, 0.45740E+00_JPRB, 0.47418E+00_JPRB, 0.47274E+00_JPRB, 0.46112E+00_JPRB, 0.44460E+00_JPRB ,&
&  0.42688E+00_JPRB, 0.41375E+00_JPRB, 0.40253E+00_JPRB, 0.39326E+00_JPRB, 0.38659E+00_JPRB, 0.38359E+00_JPRB, 0.38387E+00_JPRB ,&
&  0.38048E+00_JPRB, 0.37908E+00_JPRB, 0.38214E+00_JPRB, 0.39174E+00_JPRB, 0.40089E+00_JPRB, 0.40975E+00_JPRB, 0.41915E+00_JPRB ,&
&  0.42529E+00_JPRB, 0.42908E+00_JPRB, 0.43052E+00_JPRB, 0.42950E+00_JPRB, 0.42350E+00_JPRB, 0.41671E+00_JPRB, 0.40312E+00_JPRB ,&
&  0.38947E+00_JPRB, 0.37905E+00_JPRB, 0.37398E+00_JPRB, 0.37035E+00_JPRB, 0.36791E+00_JPRB, 0.36674E+00_JPRB, 0.36540E+00_JPRB ,&
&  0.36439E+00_JPRB, 0.36572E+00_JPRB, 0.36722E+00_JPRB, 0.36843E+00_JPRB, 0.36939E+00_JPRB, 0.37001E+00_JPRB, 0.36919E+00_JPRB ,&
&  0.36781E+00_JPRB, 0.36733E+00_JPRB, 0.36762E+00_JPRB, 0.36807E+00_JPRB, 0.36872E+00_JPRB, 0.37000E+00_JPRB, 0.37228E+00_JPRB ,&
&  0.37614E+00_JPRB, 0.38521E+00_JPRB, 0.39020E+00_JPRB, 0.40247E+00_JPRB, 0.40214E+00_JPRB, 0.40108E+00_JPRB, 0.40052E+00_JPRB ,&
&  0.39779E+00_JPRB, 0.39484E+00_JPRB, 0.38862E+00_JPRB, 0.38938E+00_JPRB, 0.41074E+00_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.51435E+00_JPRB, 0.48549E+00_JPRB, 0.44699E+00_JPRB, 0.41676E+00_JPRB, 0.39508E+00_JPRB, 0.37835E+00_JPRB, 0.36794E+00_JPRB ,&
&  0.36363E+00_JPRB, 0.35461E+00_JPRB, 0.34280E+00_JPRB, 0.33303E+00_JPRB, 0.32817E+00_JPRB, 0.32437E+00_JPRB, 0.31924E+00_JPRB ,&
&  0.31469E+00_JPRB, 0.31078E+00_JPRB, 0.30847E+00_JPRB, 0.30748E+00_JPRB, 0.30689E+00_JPRB, 0.30624E+00_JPRB, 0.30402E+00_JPRB ,&
&  0.30041E+00_JPRB, 0.30063E+00_JPRB, 0.30289E+00_JPRB, 0.30941E+00_JPRB, 0.31729E+00_JPRB, 0.32611E+00_JPRB, 0.34330E+00_JPRB ,&
&  0.38820E+00_JPRB, 0.36396E+00_JPRB, 0.39194E+00_JPRB, 0.39911E+00_JPRB, 0.40066E+00_JPRB, 0.41940E+00_JPRB, 0.44174E+00_JPRB ,&
&  0.46660E+00_JPRB, 0.49880E+00_JPRB, 0.52986E+00_JPRB, 0.54424E+00_JPRB, 0.53447E+00_JPRB, 0.51365E+00_JPRB, 0.49366E+00_JPRB ,&
&  0.47480E+00_JPRB, 0.46199E+00_JPRB, 0.45003E+00_JPRB, 0.43897E+00_JPRB, 0.43094E+00_JPRB, 0.42989E+00_JPRB, 0.43349E+00_JPRB ,&
&  0.43441E+00_JPRB, 0.42716E+00_JPRB, 0.43167E+00_JPRB, 0.44130E+00_JPRB, 0.45486E+00_JPRB, 0.46138E+00_JPRB, 0.46654E+00_JPRB ,&
&  0.47046E+00_JPRB, 0.46785E+00_JPRB, 0.46220E+00_JPRB, 0.45358E+00_JPRB, 0.43817E+00_JPRB, 0.42245E+00_JPRB, 0.40678E+00_JPRB ,&
&  0.39108E+00_JPRB, 0.38110E+00_JPRB, 0.37686E+00_JPRB, 0.37486E+00_JPRB, 0.37558E+00_JPRB, 0.37791E+00_JPRB, 0.38228E+00_JPRB ,&
&  0.38915E+00_JPRB, 0.39564E+00_JPRB, 0.40108E+00_JPRB, 0.40403E+00_JPRB, 0.40363E+00_JPRB, 0.40182E+00_JPRB, 0.39812E+00_JPRB ,&
&  0.39356E+00_JPRB, 0.38960E+00_JPRB, 0.38754E+00_JPRB, 0.38819E+00_JPRB, 0.39021E+00_JPRB, 0.39332E+00_JPRB, 0.39719E+00_JPRB ,&
&  0.40232E+00_JPRB, 0.41277E+00_JPRB, 0.40979E+00_JPRB, 0.41909E+00_JPRB, 0.43402E+00_JPRB, 0.43022E+00_JPRB, 0.42550E+00_JPRB ,&
&  0.41849E+00_JPRB, 0.41466E+00_JPRB, 0.41186E+00_JPRB, 0.41780E+00_JPRB, 0.49550E+00_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.46381E+00_JPRB, 0.45420E+00_JPRB, 0.43240E+00_JPRB, 0.41212E+00_JPRB, 0.39875E+00_JPRB, 0.38876E+00_JPRB, 0.38285E+00_JPRB ,&
&  0.37938E+00_JPRB, 0.37227E+00_JPRB, 0.36554E+00_JPRB, 0.35613E+00_JPRB, 0.34806E+00_JPRB, 0.34150E+00_JPRB, 0.33517E+00_JPRB ,&
&  0.32978E+00_JPRB, 0.32675E+00_JPRB, 0.32442E+00_JPRB, 0.32206E+00_JPRB, 0.31803E+00_JPRB, 0.31192E+00_JPRB, 0.30853E+00_JPRB ,&
&  0.30792E+00_JPRB, 0.30954E+00_JPRB, 0.31400E+00_JPRB, 0.32222E+00_JPRB, 0.32980E+00_JPRB, 0.33797E+00_JPRB, 0.35320E+00_JPRB ,&
&  0.38869E+00_JPRB, 0.43423E+00_JPRB, 0.45963E+00_JPRB, 0.46597E+00_JPRB, 0.45352E+00_JPRB, 0.45233E+00_JPRB, 0.48005E+00_JPRB ,&
&  0.52664E+00_JPRB, 0.56964E+00_JPRB, 0.60476E+00_JPRB, 0.61814E+00_JPRB, 0.60676E+00_JPRB, 0.59139E+00_JPRB, 0.56987E+00_JPRB ,&
&  0.54818E+00_JPRB, 0.52405E+00_JPRB, 0.49990E+00_JPRB, 0.47721E+00_JPRB, 0.45645E+00_JPRB, 0.45313E+00_JPRB, 0.45969E+00_JPRB ,&
&  0.47080E+00_JPRB, 0.47676E+00_JPRB, 0.47616E+00_JPRB, 0.47959E+00_JPRB, 0.48543E+00_JPRB, 0.49555E+00_JPRB, 0.51468E+00_JPRB ,&
&  0.52191E+00_JPRB, 0.51042E+00_JPRB, 0.49360E+00_JPRB, 0.47247E+00_JPRB, 0.44658E+00_JPRB, 0.42498E+00_JPRB, 0.40725E+00_JPRB ,&
&  0.39331E+00_JPRB, 0.38355E+00_JPRB, 0.37784E+00_JPRB, 0.37656E+00_JPRB, 0.38077E+00_JPRB, 0.39059E+00_JPRB, 0.40118E+00_JPRB ,&
&  0.41178E+00_JPRB, 0.42084E+00_JPRB, 0.42539E+00_JPRB, 0.42648E+00_JPRB, 0.42277E+00_JPRB, 0.41514E+00_JPRB, 0.40318E+00_JPRB ,&
&  0.39277E+00_JPRB, 0.38842E+00_JPRB, 0.39381E+00_JPRB, 0.39911E+00_JPRB, 0.40327E+00_JPRB, 0.40728E+00_JPRB, 0.41134E+00_JPRB ,&
&  0.41960E+00_JPRB, 0.42637E+00_JPRB, 0.42697E+00_JPRB, 0.42501E+00_JPRB, 0.42382E+00_JPRB, 0.43964E+00_JPRB, 0.43397E+00_JPRB ,&
&  0.42999E+00_JPRB, 0.42632E+00_JPRB, 0.42723E+00_JPRB, 0.43057E+00_JPRB, 0.46002E+00_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.45365E+00_JPRB, 0.44985E+00_JPRB, 0.43662E+00_JPRB, 0.42109E+00_JPRB, 0.41011E+00_JPRB, 0.40146E+00_JPRB, 0.39509E+00_JPRB ,&
&  0.39133E+00_JPRB, 0.38489E+00_JPRB, 0.37936E+00_JPRB, 0.37193E+00_JPRB, 0.36292E+00_JPRB, 0.35566E+00_JPRB, 0.35130E+00_JPRB ,&
&  0.34525E+00_JPRB, 0.34238E+00_JPRB, 0.33917E+00_JPRB, 0.33225E+00_JPRB, 0.32478E+00_JPRB, 0.31864E+00_JPRB, 0.31477E+00_JPRB ,&
&  0.31366E+00_JPRB, 0.31653E+00_JPRB, 0.32225E+00_JPRB, 0.33406E+00_JPRB, 0.34350E+00_JPRB, 0.35210E+00_JPRB, 0.36878E+00_JPRB ,&
&  0.41445E+00_JPRB, 0.48895E+00_JPRB, 0.56893E+00_JPRB, 0.53303E+00_JPRB, 0.50719E+00_JPRB, 0.49571E+00_JPRB, 0.55170E+00_JPRB ,&
&  0.61188E+00_JPRB, 0.67365E+00_JPRB, 0.71096E+00_JPRB, 0.72447E+00_JPRB, 0.69785E+00_JPRB, 0.67642E+00_JPRB, 0.65197E+00_JPRB ,&
&  0.62463E+00_JPRB, 0.59283E+00_JPRB, 0.55182E+00_JPRB, 0.51098E+00_JPRB, 0.48733E+00_JPRB, 0.48696E+00_JPRB, 0.49760E+00_JPRB ,&
&  0.51172E+00_JPRB, 0.51788E+00_JPRB, 0.50608E+00_JPRB, 0.50026E+00_JPRB, 0.48297E+00_JPRB, 0.46627E+00_JPRB, 0.50866E+00_JPRB ,&
&  0.60067E+00_JPRB, 0.58389E+00_JPRB, 0.54527E+00_JPRB, 0.49474E+00_JPRB, 0.46063E+00_JPRB, 0.43370E+00_JPRB, 0.41717E+00_JPRB ,&
&  0.40415E+00_JPRB, 0.39523E+00_JPRB, 0.39089E+00_JPRB, 0.39122E+00_JPRB, 0.39782E+00_JPRB, 0.40765E+00_JPRB, 0.41496E+00_JPRB ,&
&  0.42222E+00_JPRB, 0.42730E+00_JPRB, 0.42929E+00_JPRB, 0.42631E+00_JPRB, 0.41999E+00_JPRB, 0.41080E+00_JPRB, 0.40183E+00_JPRB ,&
&  0.39796E+00_JPRB, 0.40182E+00_JPRB, 0.41318E+00_JPRB, 0.41311E+00_JPRB, 0.41899E+00_JPRB, 0.42933E+00_JPRB, 0.43079E+00_JPRB ,&
&  0.42599E+00_JPRB, 0.42635E+00_JPRB, 0.43355E+00_JPRB, 0.43100E+00_JPRB, 0.43205E+00_JPRB, 0.45664E+00_JPRB, 0.44537E+00_JPRB ,&
&  0.43624E+00_JPRB, 0.43083E+00_JPRB, 0.43371E+00_JPRB, 0.43527E+00_JPRB, 0.44682E+00_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.45456E+00_JPRB, 0.45265E+00_JPRB, 0.44205E+00_JPRB, 0.42827E+00_JPRB, 0.41755E+00_JPRB, 0.40679E+00_JPRB, 0.39886E+00_JPRB ,&
&  0.39351E+00_JPRB, 0.38933E+00_JPRB, 0.38639E+00_JPRB, 0.38261E+00_JPRB, 0.37720E+00_JPRB, 0.36989E+00_JPRB, 0.36292E+00_JPRB ,&
&  0.35532E+00_JPRB, 0.34777E+00_JPRB, 0.34053E+00_JPRB, 0.33293E+00_JPRB, 0.32744E+00_JPRB, 0.32349E+00_JPRB, 0.32095E+00_JPRB ,&
&  0.32124E+00_JPRB, 0.32438E+00_JPRB, 0.33174E+00_JPRB, 0.34656E+00_JPRB, 0.36266E+00_JPRB, 0.36971E+00_JPRB, 0.37785E+00_JPRB ,&
&  0.41291E+00_JPRB, 0.36295E+00_JPRB, 0.55840E+00_JPRB, 0.60404E+00_JPRB, 0.59479E+00_JPRB, 0.61224E+00_JPRB, 0.69393E+00_JPRB ,&
&  0.76713E+00_JPRB, 0.79403E+00_JPRB, 0.79936E+00_JPRB, 0.77402E+00_JPRB, 0.71560E+00_JPRB, 0.66794E+00_JPRB, 0.62864E+00_JPRB ,&
&  0.59568E+00_JPRB, 0.56584E+00_JPRB, 0.53330E+00_JPRB, 0.50243E+00_JPRB, 0.48092E+00_JPRB, 0.47769E+00_JPRB, 0.48372E+00_JPRB ,&
&  0.50444E+00_JPRB, 0.51009E+00_JPRB, 0.50665E+00_JPRB, 0.51110E+00_JPRB, 0.48484E+00_JPRB, 0.49014E+00_JPRB, 0.50397E+00_JPRB ,&
&  0.58439E+00_JPRB, 0.64638E+00_JPRB, 0.58277E+00_JPRB, 0.51224E+00_JPRB, 0.46889E+00_JPRB, 0.44778E+00_JPRB, 0.42805E+00_JPRB ,&
&  0.41750E+00_JPRB, 0.41054E+00_JPRB, 0.40726E+00_JPRB, 0.40690E+00_JPRB, 0.41080E+00_JPRB, 0.41596E+00_JPRB, 0.41793E+00_JPRB ,&
&  0.42024E+00_JPRB, 0.42188E+00_JPRB, 0.42297E+00_JPRB, 0.42215E+00_JPRB, 0.41843E+00_JPRB, 0.41240E+00_JPRB, 0.41052E+00_JPRB ,&
&  0.41514E+00_JPRB, 0.42664E+00_JPRB, 0.43542E+00_JPRB, 0.43667E+00_JPRB, 0.44159E+00_JPRB, 0.44658E+00_JPRB, 0.44859E+00_JPRB ,&
&  0.44758E+00_JPRB, 0.44691E+00_JPRB, 0.44427E+00_JPRB, 0.44818E+00_JPRB, 0.43870E+00_JPRB, 0.46094E+00_JPRB, 0.44732E+00_JPRB ,&
&  0.43869E+00_JPRB, 0.43589E+00_JPRB, 0.43732E+00_JPRB, 0.44126E+00_JPRB, 0.44780E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.45939E+00_JPRB, 0.45747E+00_JPRB, 0.44774E+00_JPRB, 0.43477E+00_JPRB, 0.42110E+00_JPRB, 0.40976E+00_JPRB, 0.40520E+00_JPRB ,&
&  0.40003E+00_JPRB, 0.39423E+00_JPRB, 0.38873E+00_JPRB, 0.38612E+00_JPRB, 0.38536E+00_JPRB, 0.37975E+00_JPRB, 0.36947E+00_JPRB ,&
&  0.35840E+00_JPRB, 0.34770E+00_JPRB, 0.33931E+00_JPRB, 0.33563E+00_JPRB, 0.33152E+00_JPRB, 0.32921E+00_JPRB, 0.32946E+00_JPRB ,&
&  0.33074E+00_JPRB, 0.33620E+00_JPRB, 0.34736E+00_JPRB, 0.36597E+00_JPRB, 0.38000E+00_JPRB, 0.38340E+00_JPRB, 0.38613E+00_JPRB ,&
&  0.41584E+00_JPRB, 0.53384E+00_JPRB, 0.53341E+00_JPRB, 0.70210E+00_JPRB, 0.76488E+00_JPRB, 0.80830E+00_JPRB, 0.84753E+00_JPRB ,&
&  0.86503E+00_JPRB, 0.88258E+00_JPRB, 0.84483E+00_JPRB, 0.78465E+00_JPRB, 0.70325E+00_JPRB, 0.63959E+00_JPRB, 0.58482E+00_JPRB ,&
&  0.54016E+00_JPRB, 0.50569E+00_JPRB, 0.48192E+00_JPRB, 0.46393E+00_JPRB, 0.45364E+00_JPRB, 0.45317E+00_JPRB, 0.46426E+00_JPRB ,&
&  0.48384E+00_JPRB, 0.49901E+00_JPRB, 0.51945E+00_JPRB, 0.54555E+00_JPRB, 0.51471E+00_JPRB, 0.51898E+00_JPRB, 0.54599E+00_JPRB ,&
&  0.58535E+00_JPRB, 0.65003E+00_JPRB, 0.57269E+00_JPRB, 0.50251E+00_JPRB, 0.46938E+00_JPRB, 0.45700E+00_JPRB, 0.44503E+00_JPRB ,&
&  0.43647E+00_JPRB, 0.43088E+00_JPRB, 0.42803E+00_JPRB, 0.42468E+00_JPRB, 0.42270E+00_JPRB, 0.42228E+00_JPRB, 0.42227E+00_JPRB ,&
&  0.42331E+00_JPRB, 0.42509E+00_JPRB, 0.42763E+00_JPRB, 0.42895E+00_JPRB, 0.42712E+00_JPRB, 0.42786E+00_JPRB, 0.43382E+00_JPRB ,&
&  0.44438E+00_JPRB, 0.45442E+00_JPRB, 0.45710E+00_JPRB, 0.45573E+00_JPRB, 0.45963E+00_JPRB, 0.46487E+00_JPRB, 0.46235E+00_JPRB ,&
&  0.46725E+00_JPRB, 0.46716E+00_JPRB, 0.46426E+00_JPRB, 0.46182E+00_JPRB, 0.45879E+00_JPRB, 0.48244E+00_JPRB, 0.45511E+00_JPRB ,&
&  0.44628E+00_JPRB, 0.44572E+00_JPRB, 0.44852E+00_JPRB, 0.44892E+00_JPRB, 0.45453E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.46553E+00_JPRB, 0.45756E+00_JPRB, 0.44785E+00_JPRB, 0.43612E+00_JPRB, 0.42326E+00_JPRB, 0.41483E+00_JPRB, 0.41116E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.39650E+00_JPRB, 0.39121E+00_JPRB, 0.38932E+00_JPRB, 0.38774E+00_JPRB, 0.38148E+00_JPRB, 0.36940E+00_JPRB ,&
&  0.35730E+00_JPRB, 0.34966E+00_JPRB, 0.34612E+00_JPRB, 0.34249E+00_JPRB, 0.33789E+00_JPRB, 0.33538E+00_JPRB, 0.33651E+00_JPRB ,&
&  0.34152E+00_JPRB, 0.34941E+00_JPRB, 0.36531E+00_JPRB, 0.38358E+00_JPRB, 0.39817E+00_JPRB, 0.40883E+00_JPRB, 0.41411E+00_JPRB ,&
&  0.46691E+00_JPRB, 0.53302E+00_JPRB, 0.60703E+00_JPRB, 0.92629E+00_JPRB, 0.99726E+00_JPRB, 0.98162E+00_JPRB, 0.95542E+00_JPRB ,&
&  0.92551E+00_JPRB, 0.95163E+00_JPRB, 0.87246E+00_JPRB, 0.77512E+00_JPRB, 0.67756E+00_JPRB, 0.60890E+00_JPRB, 0.55712E+00_JPRB ,&
&  0.51091E+00_JPRB, 0.47925E+00_JPRB, 0.46422E+00_JPRB, 0.45809E+00_JPRB, 0.45624E+00_JPRB, 0.45914E+00_JPRB, 0.46913E+00_JPRB ,&
&  0.48660E+00_JPRB, 0.51342E+00_JPRB, 0.55898E+00_JPRB, 0.58193E+00_JPRB, 0.55689E+00_JPRB, 0.54536E+00_JPRB, 0.55065E+00_JPRB ,&
&  0.59047E+00_JPRB, 0.61122E+00_JPRB, 0.54627E+00_JPRB, 0.50276E+00_JPRB, 0.47650E+00_JPRB, 0.47453E+00_JPRB, 0.46107E+00_JPRB ,&
&  0.45455E+00_JPRB, 0.44967E+00_JPRB, 0.44631E+00_JPRB, 0.44001E+00_JPRB, 0.43283E+00_JPRB, 0.43036E+00_JPRB, 0.43089E+00_JPRB ,&
&  0.43267E+00_JPRB, 0.43625E+00_JPRB, 0.43828E+00_JPRB, 0.43959E+00_JPRB, 0.44115E+00_JPRB, 0.44705E+00_JPRB, 0.45648E+00_JPRB ,&
&  0.46389E+00_JPRB, 0.47766E+00_JPRB, 0.47572E+00_JPRB, 0.47208E+00_JPRB, 0.48231E+00_JPRB, 0.48980E+00_JPRB, 0.47935E+00_JPRB ,&
&  0.47703E+00_JPRB, 0.48319E+00_JPRB, 0.48216E+00_JPRB, 0.46885E+00_JPRB, 0.47541E+00_JPRB, 0.49068E+00_JPRB, 0.46851E+00_JPRB ,&
&  0.46328E+00_JPRB, 0.46405E+00_JPRB, 0.46954E+00_JPRB, 0.46802E+00_JPRB, 0.46704E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.47113E+00_JPRB, 0.45958E+00_JPRB, 0.45023E+00_JPRB, 0.44290E+00_JPRB, 0.43313E+00_JPRB, 0.42280E+00_JPRB, 0.41498E+00_JPRB ,&
&  0.40223E+00_JPRB, 0.39374E+00_JPRB, 0.38772E+00_JPRB, 0.38095E+00_JPRB, 0.37467E+00_JPRB, 0.36914E+00_JPRB, 0.36450E+00_JPRB ,&
&  0.36167E+00_JPRB, 0.35941E+00_JPRB, 0.35774E+00_JPRB, 0.35406E+00_JPRB, 0.35089E+00_JPRB, 0.35036E+00_JPRB, 0.35264E+00_JPRB ,&
&  0.36400E+00_JPRB, 0.38018E+00_JPRB, 0.39891E+00_JPRB, 0.41840E+00_JPRB, 0.43607E+00_JPRB, 0.45770E+00_JPRB, 0.48153E+00_JPRB ,&
&  0.57076E+00_JPRB, 0.60221E+00_JPRB, 0.74860E+00_JPRB, 0.11218E+01_JPRB, 0.11541E+01_JPRB, 0.10351E+01_JPRB, 0.98432E+00_JPRB ,&
&  0.92487E+00_JPRB, 0.81828E+00_JPRB, 0.83316E+00_JPRB, 0.73033E+00_JPRB, 0.63463E+00_JPRB, 0.58543E+00_JPRB, 0.55134E+00_JPRB ,&
&  0.51670E+00_JPRB, 0.49577E+00_JPRB, 0.49014E+00_JPRB, 0.48757E+00_JPRB, 0.48142E+00_JPRB, 0.48543E+00_JPRB, 0.49710E+00_JPRB ,&
&  0.52309E+00_JPRB, 0.56338E+00_JPRB, 0.62983E+00_JPRB, 0.66417E+00_JPRB, 0.57624E+00_JPRB, 0.55289E+00_JPRB, 0.54739E+00_JPRB ,&
&  0.57856E+00_JPRB, 0.56853E+00_JPRB, 0.53543E+00_JPRB, 0.50842E+00_JPRB, 0.48625E+00_JPRB, 0.49405E+00_JPRB, 0.47545E+00_JPRB ,&
&  0.46914E+00_JPRB, 0.46308E+00_JPRB, 0.45617E+00_JPRB, 0.44513E+00_JPRB, 0.43839E+00_JPRB, 0.43325E+00_JPRB, 0.43386E+00_JPRB ,&
&  0.43959E+00_JPRB, 0.44488E+00_JPRB, 0.44946E+00_JPRB, 0.45482E+00_JPRB, 0.46143E+00_JPRB, 0.46756E+00_JPRB, 0.47440E+00_JPRB ,&
&  0.48126E+00_JPRB, 0.49817E+00_JPRB, 0.49579E+00_JPRB, 0.50561E+00_JPRB, 0.52017E+00_JPRB, 0.52458E+00_JPRB, 0.50224E+00_JPRB ,&
&  0.48791E+00_JPRB, 0.49467E+00_JPRB, 0.49507E+00_JPRB, 0.49144E+00_JPRB, 0.49813E+00_JPRB, 0.50332E+00_JPRB, 0.48706E+00_JPRB ,&
&  0.48982E+00_JPRB, 0.49526E+00_JPRB, 0.50788E+00_JPRB, 0.49591E+00_JPRB, 0.48422E+00_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.48019E+00_JPRB, 0.46299E+00_JPRB, 0.45079E+00_JPRB, 0.43792E+00_JPRB, 0.42401E+00_JPRB, 0.40862E+00_JPRB, 0.39644E+00_JPRB ,&
&  0.38143E+00_JPRB, 0.37798E+00_JPRB, 0.37470E+00_JPRB, 0.36987E+00_JPRB, 0.36444E+00_JPRB, 0.36511E+00_JPRB, 0.37124E+00_JPRB ,&
&  0.37663E+00_JPRB, 0.37816E+00_JPRB, 0.37968E+00_JPRB, 0.38080E+00_JPRB, 0.38473E+00_JPRB, 0.39320E+00_JPRB, 0.40598E+00_JPRB ,&
&  0.42272E+00_JPRB, 0.44188E+00_JPRB, 0.46727E+00_JPRB, 0.49767E+00_JPRB, 0.52629E+00_JPRB, 0.56709E+00_JPRB, 0.61879E+00_JPRB ,&
&  0.71630E+00_JPRB, 0.71206E+00_JPRB, 0.47841E+00_JPRB, 0.76336E+00_JPRB, 0.95739E+00_JPRB, 0.90925E+00_JPRB, 0.85286E+00_JPRB ,&
&  0.82366E+00_JPRB, 0.75853E+00_JPRB, 0.73722E+00_JPRB, 0.68339E+00_JPRB, 0.61380E+00_JPRB, 0.59450E+00_JPRB, 0.57739E+00_JPRB ,&
&  0.55521E+00_JPRB, 0.54010E+00_JPRB, 0.52809E+00_JPRB, 0.52290E+00_JPRB, 0.51354E+00_JPRB, 0.51521E+00_JPRB, 0.54296E+00_JPRB ,&
&  0.58150E+00_JPRB, 0.64568E+00_JPRB, 0.71883E+00_JPRB, 0.67895E+00_JPRB, 0.59084E+00_JPRB, 0.55281E+00_JPRB, 0.53540E+00_JPRB ,&
&  0.52849E+00_JPRB, 0.54404E+00_JPRB, 0.53981E+00_JPRB, 0.51809E+00_JPRB, 0.49765E+00_JPRB, 0.48396E+00_JPRB, 0.48489E+00_JPRB ,&
&  0.47277E+00_JPRB, 0.46337E+00_JPRB, 0.45377E+00_JPRB, 0.43974E+00_JPRB, 0.43555E+00_JPRB, 0.43448E+00_JPRB, 0.44205E+00_JPRB ,&
&  0.45098E+00_JPRB, 0.45759E+00_JPRB, 0.46506E+00_JPRB, 0.47343E+00_JPRB, 0.47750E+00_JPRB, 0.48457E+00_JPRB, 0.49191E+00_JPRB ,&
&  0.50167E+00_JPRB, 0.51947E+00_JPRB, 0.53859E+00_JPRB, 0.54919E+00_JPRB, 0.55001E+00_JPRB, 0.54496E+00_JPRB, 0.53025E+00_JPRB ,&
&  0.52400E+00_JPRB, 0.52231E+00_JPRB, 0.51280E+00_JPRB, 0.50840E+00_JPRB, 0.53119E+00_JPRB, 0.54093E+00_JPRB, 0.55538E+00_JPRB ,&
&  0.59571E+00_JPRB, 0.63168E+00_JPRB, 0.64030E+00_JPRB, 0.53702E+00_JPRB, 0.50334E+00_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.45785E+00_JPRB, 0.43193E+00_JPRB, 0.40543E+00_JPRB, 0.39158E+00_JPRB, 0.38187E+00_JPRB, 0.37747E+00_JPRB, 0.37504E+00_JPRB ,&
&  0.37559E+00_JPRB, 0.37436E+00_JPRB, 0.36998E+00_JPRB, 0.37319E+00_JPRB, 0.38310E+00_JPRB, 0.39353E+00_JPRB, 0.40749E+00_JPRB ,&
&  0.41348E+00_JPRB, 0.41666E+00_JPRB, 0.42290E+00_JPRB, 0.43840E+00_JPRB, 0.45985E+00_JPRB, 0.48140E+00_JPRB, 0.50031E+00_JPRB ,&
&  0.52114E+00_JPRB, 0.54467E+00_JPRB, 0.57580E+00_JPRB, 0.61723E+00_JPRB, 0.67276E+00_JPRB, 0.74312E+00_JPRB, 0.81323E+00_JPRB ,&
&  0.88073E+00_JPRB, 0.68547E+00_JPRB, 0.54206E+00_JPRB, 0.85835E+00_JPRB, 0.92255E+00_JPRB, 0.87089E+00_JPRB, 0.76083E+00_JPRB ,&
&  0.73422E+00_JPRB, 0.72742E+00_JPRB, 0.70157E+00_JPRB, 0.67869E+00_JPRB, 0.62535E+00_JPRB, 0.60923E+00_JPRB, 0.59438E+00_JPRB ,&
&  0.57890E+00_JPRB, 0.56185E+00_JPRB, 0.55846E+00_JPRB, 0.55979E+00_JPRB, 0.55981E+00_JPRB, 0.57497E+00_JPRB, 0.61129E+00_JPRB ,&
&  0.66175E+00_JPRB, 0.72992E+00_JPRB, 0.79779E+00_JPRB, 0.70161E+00_JPRB, 0.62915E+00_JPRB, 0.56403E+00_JPRB, 0.54357E+00_JPRB ,&
&  0.53152E+00_JPRB, 0.54642E+00_JPRB, 0.54961E+00_JPRB, 0.53231E+00_JPRB, 0.51247E+00_JPRB, 0.48802E+00_JPRB, 0.47914E+00_JPRB ,&
&  0.46632E+00_JPRB, 0.45201E+00_JPRB, 0.44209E+00_JPRB, 0.43312E+00_JPRB, 0.43506E+00_JPRB, 0.44415E+00_JPRB, 0.45315E+00_JPRB ,&
&  0.46014E+00_JPRB, 0.46610E+00_JPRB, 0.47679E+00_JPRB, 0.49058E+00_JPRB, 0.49893E+00_JPRB, 0.50600E+00_JPRB, 0.51641E+00_JPRB ,&
&  0.52586E+00_JPRB, 0.53676E+00_JPRB, 0.55667E+00_JPRB, 0.56652E+00_JPRB, 0.56430E+00_JPRB, 0.54574E+00_JPRB, 0.54661E+00_JPRB ,&
&  0.54504E+00_JPRB, 0.55215E+00_JPRB, 0.54936E+00_JPRB, 0.55888E+00_JPRB, 0.56716E+00_JPRB, 0.59661E+00_JPRB, 0.62797E+00_JPRB ,&
&  0.65168E+00_JPRB, 0.65343E+00_JPRB, 0.62611E+00_JPRB, 0.52140E+00_JPRB, 0.48485E+00_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.39143E+00_JPRB, 0.37299E+00_JPRB, 0.37490E+00_JPRB, 0.37989E+00_JPRB, 0.39269E+00_JPRB, 0.40359E+00_JPRB, 0.39792E+00_JPRB ,&
&  0.40121E+00_JPRB, 0.40632E+00_JPRB, 0.41592E+00_JPRB, 0.42607E+00_JPRB, 0.44007E+00_JPRB, 0.45394E+00_JPRB, 0.46028E+00_JPRB ,&
&  0.47455E+00_JPRB, 0.49308E+00_JPRB, 0.51799E+00_JPRB, 0.54270E+00_JPRB, 0.56828E+00_JPRB, 0.58781E+00_JPRB, 0.60528E+00_JPRB ,&
&  0.62698E+00_JPRB, 0.66407E+00_JPRB, 0.71212E+00_JPRB, 0.76673E+00_JPRB, 0.83149E+00_JPRB, 0.90847E+00_JPRB, 0.95081E+00_JPRB ,&
&  0.74689E+00_JPRB, 0.47789E+00_JPRB, 0.88224E+00_JPRB, 0.97315E+00_JPRB, 0.85631E+00_JPRB, 0.79626E+00_JPRB, 0.73875E+00_JPRB ,&
&  0.69191E+00_JPRB, 0.68939E+00_JPRB, 0.66118E+00_JPRB, 0.66602E+00_JPRB, 0.62842E+00_JPRB, 0.61016E+00_JPRB, 0.60266E+00_JPRB ,&
&  0.59300E+00_JPRB, 0.58839E+00_JPRB, 0.60121E+00_JPRB, 0.61118E+00_JPRB, 0.62418E+00_JPRB, 0.64792E+00_JPRB, 0.68753E+00_JPRB ,&
&  0.74316E+00_JPRB, 0.80343E+00_JPRB, 0.84381E+00_JPRB, 0.70976E+00_JPRB, 0.65318E+00_JPRB, 0.59520E+00_JPRB, 0.56224E+00_JPRB ,&
&  0.55790E+00_JPRB, 0.54765E+00_JPRB, 0.54185E+00_JPRB, 0.54489E+00_JPRB, 0.52754E+00_JPRB, 0.48137E+00_JPRB, 0.46976E+00_JPRB ,&
&  0.44878E+00_JPRB, 0.43876E+00_JPRB, 0.43490E+00_JPRB, 0.43194E+00_JPRB, 0.43280E+00_JPRB, 0.43956E+00_JPRB, 0.44658E+00_JPRB ,&
&  0.45724E+00_JPRB, 0.46183E+00_JPRB, 0.47327E+00_JPRB, 0.48769E+00_JPRB, 0.50011E+00_JPRB, 0.51440E+00_JPRB, 0.53080E+00_JPRB ,&
&  0.53767E+00_JPRB, 0.55287E+00_JPRB, 0.56697E+00_JPRB, 0.56926E+00_JPRB, 0.57095E+00_JPRB, 0.55799E+00_JPRB, 0.55428E+00_JPRB ,&
&  0.55428E+00_JPRB, 0.55716E+00_JPRB, 0.55623E+00_JPRB, 0.56541E+00_JPRB, 0.57794E+00_JPRB, 0.58795E+00_JPRB, 0.60843E+00_JPRB ,&
&  0.60350E+00_JPRB, 0.57821E+00_JPRB, 0.52088E+00_JPRB, 0.45477E+00_JPRB, 0.42377E+00_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.38430E+00_JPRB, 0.40142E+00_JPRB, 0.41989E+00_JPRB, 0.43626E+00_JPRB, 0.45004E+00_JPRB, 0.46060E+00_JPRB, 0.47839E+00_JPRB ,&
&  0.49748E+00_JPRB, 0.50813E+00_JPRB, 0.50915E+00_JPRB, 0.51808E+00_JPRB, 0.52852E+00_JPRB, 0.54388E+00_JPRB, 0.55358E+00_JPRB ,&
&  0.56861E+00_JPRB, 0.59149E+00_JPRB, 0.61190E+00_JPRB, 0.63353E+00_JPRB, 0.65386E+00_JPRB, 0.67448E+00_JPRB, 0.70441E+00_JPRB ,&
&  0.74337E+00_JPRB, 0.79405E+00_JPRB, 0.84992E+00_JPRB, 0.89833E+00_JPRB, 0.93764E+00_JPRB, 0.98441E+00_JPRB, 0.98486E+00_JPRB ,&
&  0.67331E+00_JPRB, 0.85716E+00_JPRB, 0.95435E+00_JPRB, 0.87468E+00_JPRB, 0.80449E+00_JPRB, 0.75973E+00_JPRB, 0.73456E+00_JPRB ,&
&  0.69580E+00_JPRB, 0.65226E+00_JPRB, 0.63641E+00_JPRB, 0.66228E+00_JPRB, 0.62368E+00_JPRB, 0.61591E+00_JPRB, 0.61728E+00_JPRB ,&
&  0.62355E+00_JPRB, 0.63481E+00_JPRB, 0.65054E+00_JPRB, 0.66983E+00_JPRB, 0.69736E+00_JPRB, 0.72847E+00_JPRB, 0.77241E+00_JPRB ,&
&  0.82542E+00_JPRB, 0.86916E+00_JPRB, 0.89341E+00_JPRB, 0.75182E+00_JPRB, 0.70197E+00_JPRB, 0.65590E+00_JPRB, 0.61556E+00_JPRB ,&
&  0.56803E+00_JPRB, 0.57173E+00_JPRB, 0.57134E+00_JPRB, 0.57768E+00_JPRB, 0.53621E+00_JPRB, 0.49291E+00_JPRB, 0.45020E+00_JPRB ,&
&  0.43001E+00_JPRB, 0.41487E+00_JPRB, 0.40823E+00_JPRB, 0.40205E+00_JPRB, 0.40079E+00_JPRB, 0.41111E+00_JPRB, 0.42318E+00_JPRB ,&
&  0.43457E+00_JPRB, 0.43593E+00_JPRB, 0.45213E+00_JPRB, 0.46312E+00_JPRB, 0.49742E+00_JPRB, 0.52962E+00_JPRB, 0.54508E+00_JPRB ,&
&  0.55381E+00_JPRB, 0.55036E+00_JPRB, 0.54752E+00_JPRB, 0.54556E+00_JPRB, 0.54387E+00_JPRB, 0.54038E+00_JPRB, 0.53770E+00_JPRB ,&
&  0.53510E+00_JPRB, 0.55142E+00_JPRB, 0.54714E+00_JPRB, 0.54524E+00_JPRB, 0.55722E+00_JPRB, 0.53973E+00_JPRB, 0.52721E+00_JPRB ,&
&  0.50641E+00_JPRB, 0.46127E+00_JPRB, 0.42704E+00_JPRB, 0.40288E+00_JPRB, 0.38818E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.46660E+00_JPRB, 0.50597E+00_JPRB, 0.52128E+00_JPRB, 0.53976E+00_JPRB, 0.55104E+00_JPRB, 0.56641E+00_JPRB, 0.57763E+00_JPRB ,&
&  0.58741E+00_JPRB, 0.59991E+00_JPRB, 0.60215E+00_JPRB, 0.61736E+00_JPRB, 0.62871E+00_JPRB, 0.63842E+00_JPRB, 0.64365E+00_JPRB ,&
&  0.65162E+00_JPRB, 0.66855E+00_JPRB, 0.68735E+00_JPRB, 0.71025E+00_JPRB, 0.73266E+00_JPRB, 0.76284E+00_JPRB, 0.80537E+00_JPRB ,&
&  0.85628E+00_JPRB, 0.90315E+00_JPRB, 0.94102E+00_JPRB, 0.96907E+00_JPRB, 0.97856E+00_JPRB, 0.10101E+01_JPRB, 0.98999E+00_JPRB ,&
&  0.75767E+00_JPRB, 0.89174E+00_JPRB, 0.86152E+00_JPRB, 0.81918E+00_JPRB, 0.78052E+00_JPRB, 0.74150E+00_JPRB, 0.71261E+00_JPRB ,&
&  0.68428E+00_JPRB, 0.63985E+00_JPRB, 0.62724E+00_JPRB, 0.63198E+00_JPRB, 0.63598E+00_JPRB, 0.63511E+00_JPRB, 0.64334E+00_JPRB ,&
&  0.65943E+00_JPRB, 0.67736E+00_JPRB, 0.69889E+00_JPRB, 0.72352E+00_JPRB, 0.76469E+00_JPRB, 0.81902E+00_JPRB, 0.86002E+00_JPRB ,&
&  0.90504E+00_JPRB, 0.94908E+00_JPRB, 0.96839E+00_JPRB, 0.84608E+00_JPRB, 0.75303E+00_JPRB, 0.70751E+00_JPRB, 0.64509E+00_JPRB ,&
&  0.59044E+00_JPRB, 0.57845E+00_JPRB, 0.60567E+00_JPRB, 0.61558E+00_JPRB, 0.55462E+00_JPRB, 0.48638E+00_JPRB, 0.43242E+00_JPRB ,&
&  0.41405E+00_JPRB, 0.38686E+00_JPRB, 0.37421E+00_JPRB, 0.37296E+00_JPRB, 0.37617E+00_JPRB, 0.38263E+00_JPRB, 0.40061E+00_JPRB ,&
&  0.41261E+00_JPRB, 0.42553E+00_JPRB, 0.44638E+00_JPRB, 0.47382E+00_JPRB, 0.51798E+00_JPRB, 0.58072E+00_JPRB, 0.58596E+00_JPRB ,&
&  0.56444E+00_JPRB, 0.53634E+00_JPRB, 0.51278E+00_JPRB, 0.51482E+00_JPRB, 0.51581E+00_JPRB, 0.52089E+00_JPRB, 0.53834E+00_JPRB ,&
&  0.56556E+00_JPRB, 0.58680E+00_JPRB, 0.55887E+00_JPRB, 0.54238E+00_JPRB, 0.55162E+00_JPRB, 0.52362E+00_JPRB, 0.46923E+00_JPRB ,&
&  0.44867E+00_JPRB, 0.43109E+00_JPRB, 0.41542E+00_JPRB, 0.42218E+00_JPRB, 0.44176E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.61092E+00_JPRB, 0.63402E+00_JPRB, 0.64851E+00_JPRB, 0.64645E+00_JPRB, 0.66788E+00_JPRB, 0.67919E+00_JPRB, 0.68273E+00_JPRB ,&
&  0.68604E+00_JPRB, 0.69618E+00_JPRB, 0.69853E+00_JPRB, 0.71366E+00_JPRB, 0.72342E+00_JPRB, 0.73390E+00_JPRB, 0.73814E+00_JPRB ,&
&  0.74422E+00_JPRB, 0.75973E+00_JPRB, 0.78016E+00_JPRB, 0.81180E+00_JPRB, 0.84220E+00_JPRB, 0.88062E+00_JPRB, 0.91152E+00_JPRB ,&
&  0.92945E+00_JPRB, 0.95721E+00_JPRB, 0.99441E+00_JPRB, 0.10132E+01_JPRB, 0.99425E+00_JPRB, 0.97361E+00_JPRB, 0.74298E+00_JPRB ,&
&  0.83866E+00_JPRB, 0.84759E+00_JPRB, 0.83137E+00_JPRB, 0.80306E+00_JPRB, 0.76831E+00_JPRB, 0.72224E+00_JPRB, 0.68792E+00_JPRB ,&
&  0.66556E+00_JPRB, 0.64344E+00_JPRB, 0.62562E+00_JPRB, 0.61152E+00_JPRB, 0.64237E+00_JPRB, 0.64319E+00_JPRB, 0.65543E+00_JPRB ,&
&  0.67638E+00_JPRB, 0.70541E+00_JPRB, 0.73614E+00_JPRB, 0.78405E+00_JPRB, 0.84196E+00_JPRB, 0.89670E+00_JPRB, 0.95408E+00_JPRB ,&
&  0.99358E+00_JPRB, 0.10213E+01_JPRB, 0.10491E+01_JPRB, 0.90055E+00_JPRB, 0.83800E+00_JPRB, 0.75247E+00_JPRB, 0.68442E+00_JPRB ,&
&  0.63519E+00_JPRB, 0.62396E+00_JPRB, 0.66055E+00_JPRB, 0.66348E+00_JPRB, 0.58550E+00_JPRB, 0.49968E+00_JPRB, 0.43793E+00_JPRB ,&
&  0.41091E+00_JPRB, 0.37517E+00_JPRB, 0.36753E+00_JPRB, 0.36564E+00_JPRB, 0.37286E+00_JPRB, 0.37988E+00_JPRB, 0.38940E+00_JPRB ,&
&  0.40249E+00_JPRB, 0.42566E+00_JPRB, 0.46972E+00_JPRB, 0.53340E+00_JPRB, 0.61230E+00_JPRB, 0.69588E+00_JPRB, 0.66634E+00_JPRB ,&
&  0.56362E+00_JPRB, 0.50787E+00_JPRB, 0.52153E+00_JPRB, 0.53845E+00_JPRB, 0.57629E+00_JPRB, 0.63644E+00_JPRB, 0.71085E+00_JPRB ,&
&  0.83638E+00_JPRB, 0.91602E+00_JPRB, 0.83526E+00_JPRB, 0.70135E+00_JPRB, 0.97990E+00_JPRB, 0.95934E+00_JPRB, 0.49384E+00_JPRB ,&
&  0.53737E+00_JPRB, 0.52819E+00_JPRB, 0.54826E+00_JPRB, 0.57175E+00_JPRB, 0.60097E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.73397E+00_JPRB, 0.75443E+00_JPRB, 0.76164E+00_JPRB, 0.75924E+00_JPRB, 0.77017E+00_JPRB, 0.79087E+00_JPRB, 0.79496E+00_JPRB ,&
&  0.79896E+00_JPRB, 0.79043E+00_JPRB, 0.79643E+00_JPRB, 0.80404E+00_JPRB, 0.81204E+00_JPRB, 0.82163E+00_JPRB, 0.83020E+00_JPRB ,&
&  0.83293E+00_JPRB, 0.85714E+00_JPRB, 0.86632E+00_JPRB, 0.89342E+00_JPRB, 0.92133E+00_JPRB, 0.96261E+00_JPRB, 0.97995E+00_JPRB ,&
&  0.97439E+00_JPRB, 0.99339E+00_JPRB, 0.10155E+01_JPRB, 0.10452E+01_JPRB, 0.10289E+01_JPRB, 0.10250E+01_JPRB, 0.80894E+00_JPRB ,&
&  0.87847E+00_JPRB, 0.84604E+00_JPRB, 0.79597E+00_JPRB, 0.76835E+00_JPRB, 0.73462E+00_JPRB, 0.70091E+00_JPRB, 0.67083E+00_JPRB ,&
&  0.65748E+00_JPRB, 0.64186E+00_JPRB, 0.61925E+00_JPRB, 0.63302E+00_JPRB, 0.63724E+00_JPRB, 0.63840E+00_JPRB, 0.66180E+00_JPRB ,&
&  0.68444E+00_JPRB, 0.72442E+00_JPRB, 0.77201E+00_JPRB, 0.83231E+00_JPRB, 0.89513E+00_JPRB, 0.95688E+00_JPRB, 0.10265E+01_JPRB ,&
&  0.10931E+01_JPRB, 0.11593E+01_JPRB, 0.11839E+01_JPRB, 0.10473E+01_JPRB, 0.10153E+01_JPRB, 0.86477E+00_JPRB, 0.74971E+00_JPRB ,&
&  0.66182E+00_JPRB, 0.62617E+00_JPRB, 0.65085E+00_JPRB, 0.70271E+00_JPRB, 0.60541E+00_JPRB, 0.53082E+00_JPRB, 0.46084E+00_JPRB ,&
&  0.43163E+00_JPRB, 0.39835E+00_JPRB, 0.38992E+00_JPRB, 0.38107E+00_JPRB, 0.37482E+00_JPRB, 0.37863E+00_JPRB, 0.38616E+00_JPRB ,&
&  0.38898E+00_JPRB, 0.40484E+00_JPRB, 0.44103E+00_JPRB, 0.47983E+00_JPRB, 0.51082E+00_JPRB, 0.52438E+00_JPRB, 0.50112E+00_JPRB ,&
&  0.53626E+00_JPRB, 0.53800E+00_JPRB, 0.54559E+00_JPRB, 0.60451E+00_JPRB, 0.71134E+00_JPRB, 0.87582E+00_JPRB, 0.10838E+01_JPRB ,&
&  0.13384E+01_JPRB, 0.14080E+01_JPRB, 0.17172E+01_JPRB, 0.22712E+01_JPRB, 0.20498E+01_JPRB, 0.17270E+01_JPRB, 0.61209E+00_JPRB ,&
&  0.64061E+00_JPRB, 0.64588E+00_JPRB, 0.67048E+00_JPRB, 0.69408E+00_JPRB, 0.72387E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.73453E+00_JPRB, 0.75151E+00_JPRB, 0.76381E+00_JPRB, 0.77144E+00_JPRB, 0.78472E+00_JPRB, 0.79771E+00_JPRB, 0.80973E+00_JPRB ,&
&  0.81985E+00_JPRB, 0.82595E+00_JPRB, 0.83645E+00_JPRB, 0.84570E+00_JPRB, 0.85737E+00_JPRB, 0.87181E+00_JPRB, 0.88909E+00_JPRB ,&
&  0.90091E+00_JPRB, 0.91771E+00_JPRB, 0.92352E+00_JPRB, 0.95067E+00_JPRB, 0.98733E+00_JPRB, 0.10012E+01_JPRB, 0.10055E+01_JPRB ,&
&  0.99566E+00_JPRB, 0.10056E+01_JPRB, 0.10144E+01_JPRB, 0.10317E+01_JPRB, 0.10322E+01_JPRB, 0.10609E+01_JPRB, 0.88429E+00_JPRB ,&
&  0.91797E+00_JPRB, 0.85510E+00_JPRB, 0.75593E+00_JPRB, 0.72232E+00_JPRB, 0.69228E+00_JPRB, 0.67710E+00_JPRB, 0.66335E+00_JPRB ,&
&  0.65110E+00_JPRB, 0.64374E+00_JPRB, 0.62918E+00_JPRB, 0.61624E+00_JPRB, 0.61788E+00_JPRB, 0.62176E+00_JPRB, 0.63970E+00_JPRB ,&
&  0.67152E+00_JPRB, 0.71887E+00_JPRB, 0.77523E+00_JPRB, 0.84111E+00_JPRB, 0.91829E+00_JPRB, 0.98603E+00_JPRB, 0.10544E+01_JPRB ,&
&  0.11169E+01_JPRB, 0.11736E+01_JPRB, 0.11160E+01_JPRB, 0.11410E+01_JPRB, 0.12044E+01_JPRB, 0.10076E+01_JPRB, 0.81890E+00_JPRB ,&
&  0.61758E+00_JPRB, 0.61433E+00_JPRB, 0.65706E+00_JPRB, 0.73885E+00_JPRB, 0.64471E+00_JPRB, 0.58263E+00_JPRB, 0.50914E+00_JPRB ,&
&  0.44788E+00_JPRB, 0.41651E+00_JPRB, 0.40966E+00_JPRB, 0.43689E+00_JPRB, 0.41188E+00_JPRB, 0.39651E+00_JPRB, 0.40182E+00_JPRB ,&
&  0.38864E+00_JPRB, 0.38993E+00_JPRB, 0.39797E+00_JPRB, 0.41443E+00_JPRB, 0.42810E+00_JPRB, 0.42266E+00_JPRB, 0.47870E+00_JPRB ,&
&  0.48705E+00_JPRB, 0.48562E+00_JPRB, 0.50504E+00_JPRB, 0.46462E+00_JPRB, 0.64554E+00_JPRB, 0.73951E+00_JPRB, 0.87271E+00_JPRB ,&
&  0.10135E+01_JPRB, 0.14093E+01_JPRB, 0.16637E+01_JPRB, 0.12894E+01_JPRB, 0.97189E+00_JPRB, 0.67249E+00_JPRB, 0.63511E+00_JPRB ,&
&  0.64795E+00_JPRB, 0.65018E+00_JPRB, 0.66666E+00_JPRB, 0.68830E+00_JPRB, 0.71215E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.72852E+00_JPRB, 0.74738E+00_JPRB, 0.76255E+00_JPRB, 0.77438E+00_JPRB, 0.78967E+00_JPRB, 0.80526E+00_JPRB, 0.81814E+00_JPRB ,&
&  0.82981E+00_JPRB, 0.84038E+00_JPRB, 0.85393E+00_JPRB, 0.86604E+00_JPRB, 0.88087E+00_JPRB, 0.89920E+00_JPRB, 0.91657E+00_JPRB ,&
&  0.93304E+00_JPRB, 0.94087E+00_JPRB, 0.96536E+00_JPRB, 0.99479E+00_JPRB, 0.10123E+01_JPRB, 0.10111E+01_JPRB, 0.10051E+01_JPRB ,&
&  0.10007E+01_JPRB, 0.10157E+01_JPRB, 0.10285E+01_JPRB, 0.10414E+01_JPRB, 0.10624E+01_JPRB, 0.11048E+01_JPRB, 0.89434E+00_JPRB ,&
&  0.94693E+00_JPRB, 0.85736E+00_JPRB, 0.72096E+00_JPRB, 0.66252E+00_JPRB, 0.63689E+00_JPRB, 0.62247E+00_JPRB, 0.61504E+00_JPRB ,&
&  0.62085E+00_JPRB, 0.59912E+00_JPRB, 0.58886E+00_JPRB, 0.58122E+00_JPRB, 0.58580E+00_JPRB, 0.59105E+00_JPRB, 0.60411E+00_JPRB ,&
&  0.62861E+00_JPRB, 0.67017E+00_JPRB, 0.72286E+00_JPRB, 0.78373E+00_JPRB, 0.86207E+00_JPRB, 0.92676E+00_JPRB, 0.97261E+00_JPRB ,&
&  0.99768E+00_JPRB, 0.10032E+01_JPRB, 0.91182E+00_JPRB, 0.99467E+00_JPRB, 0.10843E+01_JPRB, 0.10272E+01_JPRB, 0.90611E+00_JPRB ,&
&  0.72918E+00_JPRB, 0.62471E+00_JPRB, 0.65041E+00_JPRB, 0.75737E+00_JPRB, 0.71952E+00_JPRB, 0.63568E+00_JPRB, 0.57699E+00_JPRB ,&
&  0.48763E+00_JPRB, 0.46188E+00_JPRB, 0.45062E+00_JPRB, 0.49436E+00_JPRB, 0.48198E+00_JPRB, 0.44272E+00_JPRB, 0.43161E+00_JPRB ,&
&  0.39763E+00_JPRB, 0.36374E+00_JPRB, 0.34799E+00_JPRB, 0.36492E+00_JPRB, 0.40351E+00_JPRB, 0.38565E+00_JPRB, 0.44288E+00_JPRB ,&
&  0.43329E+00_JPRB, 0.38490E+00_JPRB, 0.41511E+00_JPRB, 0.48658E+00_JPRB, 0.52776E+00_JPRB, 0.57789E+00_JPRB, 0.68369E+00_JPRB ,&
&  0.84777E+00_JPRB, 0.10510E+01_JPRB, 0.10194E+01_JPRB, 0.80575E+00_JPRB, 0.64090E+00_JPRB, 0.60651E+00_JPRB, 0.61874E+00_JPRB ,&
&  0.63699E+00_JPRB, 0.65090E+00_JPRB, 0.66776E+00_JPRB, 0.68641E+00_JPRB, 0.70637E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.71489E+00_JPRB, 0.73327E+00_JPRB, 0.74941E+00_JPRB, 0.76429E+00_JPRB, 0.77947E+00_JPRB, 0.79369E+00_JPRB, 0.80557E+00_JPRB ,&
&  0.81638E+00_JPRB, 0.82989E+00_JPRB, 0.84157E+00_JPRB, 0.85360E+00_JPRB, 0.87165E+00_JPRB, 0.88754E+00_JPRB, 0.89652E+00_JPRB ,&
&  0.91324E+00_JPRB, 0.93452E+00_JPRB, 0.96128E+00_JPRB, 0.98267E+00_JPRB, 0.99115E+00_JPRB, 0.98871E+00_JPRB, 0.98378E+00_JPRB ,&
&  0.97025E+00_JPRB, 0.96292E+00_JPRB, 0.96607E+00_JPRB, 0.97772E+00_JPRB, 0.10036E+01_JPRB, 0.98668E+00_JPRB, 0.10521E+01_JPRB ,&
&  0.75698E+00_JPRB, 0.86071E+00_JPRB, 0.71050E+00_JPRB, 0.60543E+00_JPRB, 0.59554E+00_JPRB, 0.55916E+00_JPRB, 0.55493E+00_JPRB ,&
&  0.56993E+00_JPRB, 0.56990E+00_JPRB, 0.56497E+00_JPRB, 0.55927E+00_JPRB, 0.56029E+00_JPRB, 0.57115E+00_JPRB, 0.57121E+00_JPRB ,&
&  0.58468E+00_JPRB, 0.60665E+00_JPRB, 0.62015E+00_JPRB, 0.65583E+00_JPRB, 0.72255E+00_JPRB, 0.78359E+00_JPRB, 0.81318E+00_JPRB ,&
&  0.81672E+00_JPRB, 0.80060E+00_JPRB, 0.65075E+00_JPRB, 0.72566E+00_JPRB, 0.82958E+00_JPRB, 0.84547E+00_JPRB, 0.79731E+00_JPRB ,&
&  0.77278E+00_JPRB, 0.68712E+00_JPRB, 0.78062E+00_JPRB, 0.81651E+00_JPRB, 0.79181E+00_JPRB, 0.73432E+00_JPRB, 0.65512E+00_JPRB ,&
&  0.58194E+00_JPRB, 0.52327E+00_JPRB, 0.53926E+00_JPRB, 0.59538E+00_JPRB, 0.58555E+00_JPRB, 0.51268E+00_JPRB, 0.47880E+00_JPRB ,&
&  0.42596E+00_JPRB, 0.37202E+00_JPRB, 0.33829E+00_JPRB, 0.34626E+00_JPRB, 0.36126E+00_JPRB, 0.49383E+00_JPRB, 0.44020E+00_JPRB ,&
&  0.45983E+00_JPRB, 0.42304E+00_JPRB, 0.34428E+00_JPRB, 0.42323E+00_JPRB, 0.42942E+00_JPRB, 0.42565E+00_JPRB, 0.47998E+00_JPRB ,&
&  0.59128E+00_JPRB, 0.68508E+00_JPRB, 0.68927E+00_JPRB, 0.62227E+00_JPRB, 0.59642E+00_JPRB, 0.60086E+00_JPRB, 0.61718E+00_JPRB ,&
&  0.63417E+00_JPRB, 0.64961E+00_JPRB, 0.66460E+00_JPRB, 0.68069E+00_JPRB, 0.69687E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.68828E+00_JPRB, 0.71202E+00_JPRB, 0.72743E+00_JPRB, 0.73951E+00_JPRB, 0.74329E+00_JPRB, 0.74800E+00_JPRB, 0.76198E+00_JPRB ,&
&  0.77246E+00_JPRB, 0.78387E+00_JPRB, 0.79665E+00_JPRB, 0.81628E+00_JPRB, 0.84217E+00_JPRB, 0.86139E+00_JPRB, 0.86811E+00_JPRB ,&
&  0.87266E+00_JPRB, 0.90215E+00_JPRB, 0.92121E+00_JPRB, 0.95759E+00_JPRB, 0.97073E+00_JPRB, 0.99331E+00_JPRB, 0.99896E+00_JPRB ,&
&  0.99552E+00_JPRB, 0.99962E+00_JPRB, 0.10081E+01_JPRB, 0.10293E+01_JPRB, 0.10491E+01_JPRB, 0.11527E+01_JPRB, 0.13064E+01_JPRB ,&
&  0.98874E+00_JPRB, 0.81050E+00_JPRB, 0.69494E+00_JPRB, 0.55904E+00_JPRB, 0.54508E+00_JPRB, 0.54269E+00_JPRB, 0.53570E+00_JPRB ,&
&  0.54382E+00_JPRB, 0.55472E+00_JPRB, 0.55852E+00_JPRB, 0.54575E+00_JPRB, 0.55271E+00_JPRB, 0.57764E+00_JPRB, 0.56949E+00_JPRB ,&
&  0.56682E+00_JPRB, 0.56841E+00_JPRB, 0.56141E+00_JPRB, 0.60772E+00_JPRB, 0.58394E+00_JPRB, 0.63718E+00_JPRB, 0.67495E+00_JPRB ,&
&  0.69147E+00_JPRB, 0.64533E+00_JPRB, 0.51356E+00_JPRB, 0.54658E+00_JPRB, 0.64493E+00_JPRB, 0.68721E+00_JPRB, 0.69001E+00_JPRB ,&
&  0.76875E+00_JPRB, 0.77414E+00_JPRB, 0.67366E+00_JPRB, 0.83255E+00_JPRB, 0.88304E+00_JPRB, 0.88765E+00_JPRB, 0.82178E+00_JPRB ,&
&  0.75680E+00_JPRB, 0.66295E+00_JPRB, 0.67834E+00_JPRB, 0.69187E+00_JPRB, 0.67953E+00_JPRB, 0.63989E+00_JPRB, 0.56018E+00_JPRB ,&
&  0.48534E+00_JPRB, 0.42085E+00_JPRB, 0.37295E+00_JPRB, 0.36467E+00_JPRB, 0.42662E+00_JPRB, 0.47173E+00_JPRB, 0.44012E+00_JPRB ,&
&  0.38931E+00_JPRB, 0.40883E+00_JPRB, 0.34209E+00_JPRB, 0.38687E+00_JPRB, 0.38118E+00_JPRB, 0.36990E+00_JPRB, 0.39959E+00_JPRB ,&
&  0.45501E+00_JPRB, 0.49881E+00_JPRB, 0.52871E+00_JPRB, 0.54956E+00_JPRB, 0.56736E+00_JPRB, 0.58858E+00_JPRB, 0.59748E+00_JPRB ,&
&  0.61171E+00_JPRB, 0.62718E+00_JPRB, 0.64220E+00_JPRB, 0.66385E+00_JPRB, 0.67297E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.58779E+00_JPRB, 0.59708E+00_JPRB, 0.61136E+00_JPRB, 0.61965E+00_JPRB, 0.62790E+00_JPRB, 0.63296E+00_JPRB, 0.65601E+00_JPRB ,&
&  0.66968E+00_JPRB, 0.68280E+00_JPRB, 0.70314E+00_JPRB, 0.72386E+00_JPRB, 0.73280E+00_JPRB, 0.74305E+00_JPRB, 0.74640E+00_JPRB ,&
&  0.75961E+00_JPRB, 0.77519E+00_JPRB, 0.77477E+00_JPRB, 0.78197E+00_JPRB, 0.78932E+00_JPRB, 0.84796E+00_JPRB, 0.84918E+00_JPRB ,&
&  0.83998E+00_JPRB, 0.84840E+00_JPRB, 0.85942E+00_JPRB, 0.89969E+00_JPRB, 0.10201E+01_JPRB, 0.10822E+01_JPRB, 0.10600E+01_JPRB ,&
&  0.92415E+00_JPRB, 0.80842E+00_JPRB, 0.87949E+00_JPRB, 0.72951E+00_JPRB, 0.69177E+00_JPRB, 0.68160E+00_JPRB, 0.63655E+00_JPRB ,&
&  0.62527E+00_JPRB, 0.62732E+00_JPRB, 0.64589E+00_JPRB, 0.62788E+00_JPRB, 0.62876E+00_JPRB, 0.61804E+00_JPRB, 0.58901E+00_JPRB ,&
&  0.57991E+00_JPRB, 0.57736E+00_JPRB, 0.57601E+00_JPRB, 0.61360E+00_JPRB, 0.61529E+00_JPRB, 0.64580E+00_JPRB, 0.64765E+00_JPRB ,&
&  0.64372E+00_JPRB, 0.64843E+00_JPRB, 0.60957E+00_JPRB, 0.62883E+00_JPRB, 0.72139E+00_JPRB, 0.74739E+00_JPRB, 0.74471E+00_JPRB ,&
&  0.78929E+00_JPRB, 0.75120E+00_JPRB, 0.54553E+00_JPRB, 0.77878E+00_JPRB, 0.90445E+00_JPRB, 0.91890E+00_JPRB, 0.93022E+00_JPRB ,&
&  0.88739E+00_JPRB, 0.84248E+00_JPRB, 0.81371E+00_JPRB, 0.81286E+00_JPRB, 0.76836E+00_JPRB, 0.71604E+00_JPRB, 0.66375E+00_JPRB ,&
&  0.56333E+00_JPRB, 0.46308E+00_JPRB, 0.38662E+00_JPRB, 0.35835E+00_JPRB, 0.35234E+00_JPRB, 0.44683E+00_JPRB, 0.39585E+00_JPRB ,&
&  0.35487E+00_JPRB, 0.32629E+00_JPRB, 0.35789E+00_JPRB, 0.33420E+00_JPRB, 0.32309E+00_JPRB, 0.35424E+00_JPRB, 0.35841E+00_JPRB ,&
&  0.37930E+00_JPRB, 0.40490E+00_JPRB, 0.42922E+00_JPRB, 0.44782E+00_JPRB, 0.47038E+00_JPRB, 0.48999E+00_JPRB, 0.50536E+00_JPRB ,&
&  0.52042E+00_JPRB, 0.53895E+00_JPRB, 0.55471E+00_JPRB, 0.57219E+00_JPRB, 0.58606E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.57914E+00_JPRB, 0.59072E+00_JPRB, 0.58810E+00_JPRB, 0.59043E+00_JPRB, 0.58313E+00_JPRB, 0.58155E+00_JPRB, 0.56705E+00_JPRB ,&
&  0.55396E+00_JPRB, 0.55928E+00_JPRB, 0.58284E+00_JPRB, 0.62731E+00_JPRB, 0.63475E+00_JPRB, 0.64792E+00_JPRB, 0.64612E+00_JPRB ,&
&  0.66246E+00_JPRB, 0.65754E+00_JPRB, 0.65886E+00_JPRB, 0.64933E+00_JPRB, 0.66352E+00_JPRB, 0.70285E+00_JPRB, 0.78200E+00_JPRB ,&
&  0.88914E+00_JPRB, 0.10164E+01_JPRB, 0.12046E+01_JPRB, 0.14086E+01_JPRB, 0.18410E+01_JPRB, 0.14705E+01_JPRB, 0.10710E+01_JPRB ,&
&  0.10583E+01_JPRB, 0.10013E+01_JPRB, 0.10226E+01_JPRB, 0.10309E+01_JPRB, 0.10050E+01_JPRB, 0.95307E+00_JPRB, 0.97144E+00_JPRB ,&
&  0.98101E+00_JPRB, 0.10059E+01_JPRB, 0.10314E+01_JPRB, 0.10318E+01_JPRB, 0.10317E+01_JPRB, 0.10042E+01_JPRB, 0.95336E+00_JPRB ,&
&  0.88325E+00_JPRB, 0.89241E+00_JPRB, 0.89303E+00_JPRB, 0.92385E+00_JPRB, 0.10045E+01_JPRB, 0.10070E+01_JPRB, 0.99865E+00_JPRB ,&
&  0.97911E+00_JPRB, 0.10112E+01_JPRB, 0.11075E+01_JPRB, 0.11203E+01_JPRB, 0.11305E+01_JPRB, 0.10630E+01_JPRB, 0.98552E+00_JPRB ,&
&  0.91854E+00_JPRB, 0.77867E+00_JPRB, 0.61482E+00_JPRB, 0.90210E+00_JPRB, 0.10070E+01_JPRB, 0.91415E+00_JPRB, 0.10179E+01_JPRB ,&
&  0.97667E+00_JPRB, 0.94436E+00_JPRB, 0.95306E+00_JPRB, 0.92353E+00_JPRB, 0.83571E+00_JPRB, 0.78258E+00_JPRB, 0.68362E+00_JPRB ,&
&  0.63730E+00_JPRB, 0.52308E+00_JPRB, 0.42541E+00_JPRB, 0.37152E+00_JPRB, 0.34776E+00_JPRB, 0.36657E+00_JPRB, 0.39151E+00_JPRB ,&
&  0.38581E+00_JPRB, 0.32121E+00_JPRB, 0.33803E+00_JPRB, 0.38304E+00_JPRB, 0.48701E+00_JPRB, 0.38348E+00_JPRB, 0.35321E+00_JPRB ,&
&  0.37678E+00_JPRB, 0.39751E+00_JPRB, 0.41828E+00_JPRB, 0.43110E+00_JPRB, 0.43689E+00_JPRB, 0.44399E+00_JPRB, 0.47284E+00_JPRB ,&
&  0.49623E+00_JPRB, 0.51743E+00_JPRB, 0.53534E+00_JPRB, 0.55983E+00_JPRB, 0.57301E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.73817E+00_JPRB, 0.75943E+00_JPRB, 0.77324E+00_JPRB, 0.77539E+00_JPRB, 0.76563E+00_JPRB, 0.75507E+00_JPRB, 0.73322E+00_JPRB ,&
&  0.70806E+00_JPRB, 0.71562E+00_JPRB, 0.74782E+00_JPRB, 0.82201E+00_JPRB, 0.86992E+00_JPRB, 0.89837E+00_JPRB, 0.94288E+00_JPRB ,&
&  0.98730E+00_JPRB, 0.10081E+01_JPRB, 0.10105E+01_JPRB, 0.10417E+01_JPRB, 0.10784E+01_JPRB, 0.11249E+01_JPRB, 0.12988E+01_JPRB ,&
&  0.15426E+01_JPRB, 0.18380E+01_JPRB, 0.21391E+01_JPRB, 0.23588E+01_JPRB, 0.19372E+01_JPRB, 0.11799E+01_JPRB, 0.11612E+01_JPRB ,&
&  0.12472E+01_JPRB, 0.12925E+01_JPRB, 0.12857E+01_JPRB, 0.13070E+01_JPRB, 0.13327E+01_JPRB, 0.13985E+01_JPRB, 0.14821E+01_JPRB ,&
&  0.15373E+01_JPRB, 0.15723E+01_JPRB, 0.16077E+01_JPRB, 0.16283E+01_JPRB, 0.16602E+01_JPRB, 0.16871E+01_JPRB, 0.17085E+01_JPRB ,&
&  0.16885E+01_JPRB, 0.16354E+01_JPRB, 0.16026E+01_JPRB, 0.16344E+01_JPRB, 0.15994E+01_JPRB, 0.15491E+01_JPRB, 0.14964E+01_JPRB ,&
&  0.14559E+01_JPRB, 0.14499E+01_JPRB, 0.15569E+01_JPRB, 0.15975E+01_JPRB, 0.14809E+01_JPRB, 0.13553E+01_JPRB, 0.12851E+01_JPRB ,&
&  0.12050E+01_JPRB, 0.10975E+01_JPRB, 0.85974E+00_JPRB, 0.12191E+01_JPRB, 0.12807E+01_JPRB, 0.12012E+01_JPRB, 0.11101E+01_JPRB ,&
&  0.10610E+01_JPRB, 0.10528E+01_JPRB, 0.10263E+01_JPRB, 0.10084E+01_JPRB, 0.89976E+00_JPRB, 0.85333E+00_JPRB, 0.66192E+00_JPRB ,&
&  0.67092E+00_JPRB, 0.57865E+00_JPRB, 0.47457E+00_JPRB, 0.40643E+00_JPRB, 0.38735E+00_JPRB, 0.38389E+00_JPRB, 0.40770E+00_JPRB ,&
&  0.37741E+00_JPRB, 0.37483E+00_JPRB, 0.45128E+00_JPRB, 0.80067E+00_JPRB, 0.18580E+01_JPRB, 0.48123E+00_JPRB, 0.38350E+00_JPRB ,&
&  0.41201E+00_JPRB, 0.44751E+00_JPRB, 0.48130E+00_JPRB, 0.50189E+00_JPRB, 0.51334E+00_JPRB, 0.53345E+00_JPRB, 0.57139E+00_JPRB ,&
&  0.61372E+00_JPRB, 0.64235E+00_JPRB, 0.65499E+00_JPRB, 0.68895E+00_JPRB, 0.71614E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.86285E+00_JPRB, 0.88815E+00_JPRB, 0.91012E+00_JPRB, 0.94362E+00_JPRB, 0.97454E+00_JPRB, 0.98144E+00_JPRB, 0.97439E+00_JPRB ,&
&  0.98757E+00_JPRB, 0.10198E+01_JPRB, 0.10539E+01_JPRB, 0.10835E+01_JPRB, 0.11243E+01_JPRB, 0.11588E+01_JPRB, 0.11964E+01_JPRB ,&
&  0.12365E+01_JPRB, 0.12947E+01_JPRB, 0.13883E+01_JPRB, 0.14904E+01_JPRB, 0.15750E+01_JPRB, 0.17081E+01_JPRB, 0.18498E+01_JPRB ,&
&  0.19878E+01_JPRB, 0.18581E+01_JPRB, 0.18083E+01_JPRB, 0.14401E+01_JPRB, 0.98557E+00_JPRB, 0.12002E+01_JPRB, 0.12572E+01_JPRB ,&
&  0.13240E+01_JPRB, 0.14080E+01_JPRB, 0.14903E+01_JPRB, 0.15652E+01_JPRB, 0.16834E+01_JPRB, 0.17725E+01_JPRB, 0.18210E+01_JPRB ,&
&  0.18702E+01_JPRB, 0.19156E+01_JPRB, 0.19611E+01_JPRB, 0.20173E+01_JPRB, 0.20898E+01_JPRB, 0.21589E+01_JPRB, 0.21915E+01_JPRB ,&
&  0.22307E+01_JPRB, 0.22195E+01_JPRB, 0.22211E+01_JPRB, 0.22194E+01_JPRB, 0.20708E+01_JPRB, 0.19531E+01_JPRB, 0.18609E+01_JPRB ,&
&  0.18087E+01_JPRB, 0.17969E+01_JPRB, 0.18375E+01_JPRB, 0.18744E+01_JPRB, 0.18401E+01_JPRB, 0.16669E+01_JPRB, 0.15357E+01_JPRB ,&
&  0.14415E+01_JPRB, 0.12919E+01_JPRB, 0.12368E+01_JPRB, 0.15462E+01_JPRB, 0.10031E+01_JPRB, 0.11263E+01_JPRB, 0.12100E+01_JPRB ,&
&  0.11766E+01_JPRB, 0.12056E+01_JPRB, 0.11787E+01_JPRB, 0.10669E+01_JPRB, 0.92414E+00_JPRB, 0.64718E+00_JPRB, 0.65103E+00_JPRB ,&
&  0.70899E+00_JPRB, 0.62636E+00_JPRB, 0.51940E+00_JPRB, 0.43439E+00_JPRB, 0.43607E+00_JPRB, 0.42497E+00_JPRB, 0.43115E+00_JPRB ,&
&  0.40996E+00_JPRB, 0.42217E+00_JPRB, 0.46539E+00_JPRB, 0.64295E+00_JPRB, 0.91354E+00_JPRB, 0.46601E+00_JPRB, 0.41680E+00_JPRB ,&
&  0.45435E+00_JPRB, 0.51786E+00_JPRB, 0.58674E+00_JPRB, 0.63615E+00_JPRB, 0.65606E+00_JPRB, 0.67487E+00_JPRB, 0.69523E+00_JPRB ,&
&  0.71935E+00_JPRB, 0.75766E+00_JPRB, 0.80985E+00_JPRB, 0.83268E+00_JPRB, 0.84986E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.88200E+00_JPRB, 0.93006E+00_JPRB, 0.10040E+01_JPRB, 0.10609E+01_JPRB, 0.10929E+01_JPRB, 0.10962E+01_JPRB, 0.10744E+01_JPRB ,&
&  0.10768E+01_JPRB, 0.11067E+01_JPRB, 0.11695E+01_JPRB, 0.12221E+01_JPRB, 0.12763E+01_JPRB, 0.13270E+01_JPRB, 0.13794E+01_JPRB ,&
&  0.14272E+01_JPRB, 0.14708E+01_JPRB, 0.15373E+01_JPRB, 0.16185E+01_JPRB, 0.16912E+01_JPRB, 0.18169E+01_JPRB, 0.18050E+01_JPRB ,&
&  0.14654E+01_JPRB, 0.11519E+01_JPRB, 0.14274E+01_JPRB, 0.14655E+01_JPRB, 0.12903E+01_JPRB, 0.13540E+01_JPRB, 0.14454E+01_JPRB ,&
&  0.15628E+01_JPRB, 0.16495E+01_JPRB, 0.17246E+01_JPRB, 0.17751E+01_JPRB, 0.18137E+01_JPRB, 0.18791E+01_JPRB, 0.19317E+01_JPRB ,&
&  0.20001E+01_JPRB, 0.20828E+01_JPRB, 0.21674E+01_JPRB, 0.22763E+01_JPRB, 0.23886E+01_JPRB, 0.25133E+01_JPRB, 0.25939E+01_JPRB ,&
&  0.26526E+01_JPRB, 0.26692E+01_JPRB, 0.26301E+01_JPRB, 0.25211E+01_JPRB, 0.23581E+01_JPRB, 0.22384E+01_JPRB, 0.21409E+01_JPRB ,&
&  0.21061E+01_JPRB, 0.21027E+01_JPRB, 0.21571E+01_JPRB, 0.21904E+01_JPRB, 0.21659E+01_JPRB, 0.19097E+01_JPRB, 0.17731E+01_JPRB ,&
&  0.16632E+01_JPRB, 0.14988E+01_JPRB, 0.15098E+01_JPRB, 0.15897E+01_JPRB, 0.11649E+01_JPRB, 0.11756E+01_JPRB, 0.10878E+01_JPRB ,&
&  0.12489E+01_JPRB, 0.12775E+01_JPRB, 0.12711E+01_JPRB, 0.12023E+01_JPRB, 0.95800E+00_JPRB, 0.67823E+00_JPRB, 0.69756E+00_JPRB ,&
&  0.76282E+00_JPRB, 0.73949E+00_JPRB, 0.58009E+00_JPRB, 0.49088E+00_JPRB, 0.42619E+00_JPRB, 0.42548E+00_JPRB, 0.43799E+00_JPRB ,&
&  0.42976E+00_JPRB, 0.42270E+00_JPRB, 0.46577E+00_JPRB, 0.54629E+00_JPRB, 0.59819E+00_JPRB, 0.47784E+00_JPRB, 0.46681E+00_JPRB ,&
&  0.51784E+00_JPRB, 0.56814E+00_JPRB, 0.61198E+00_JPRB, 0.64634E+00_JPRB, 0.68104E+00_JPRB, 0.71536E+00_JPRB, 0.73998E+00_JPRB ,&
&  0.78651E+00_JPRB, 0.77841E+00_JPRB, 0.81557E+00_JPRB, 0.83450E+00_JPRB, 0.86216E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.86569E+00_JPRB, 0.92747E+00_JPRB, 0.99933E+00_JPRB, 0.10678E+01_JPRB, 0.10991E+01_JPRB, 0.11266E+01_JPRB, 0.11376E+01_JPRB ,&
&  0.11106E+01_JPRB, 0.10843E+01_JPRB, 0.11599E+01_JPRB, 0.12321E+01_JPRB, 0.13240E+01_JPRB, 0.14023E+01_JPRB, 0.14760E+01_JPRB ,&
&  0.15397E+01_JPRB, 0.16121E+01_JPRB, 0.16993E+01_JPRB, 0.18060E+01_JPRB, 0.18647E+01_JPRB, 0.19249E+01_JPRB, 0.16665E+01_JPRB ,&
&  0.11774E+01_JPRB, 0.12730E+01_JPRB, 0.17712E+01_JPRB, 0.17074E+01_JPRB, 0.16008E+01_JPRB, 0.16858E+01_JPRB, 0.17715E+01_JPRB ,&
&  0.18299E+01_JPRB, 0.18606E+01_JPRB, 0.18428E+01_JPRB, 0.18477E+01_JPRB, 0.18376E+01_JPRB, 0.18591E+01_JPRB, 0.18845E+01_JPRB ,&
&  0.19452E+01_JPRB, 0.20537E+01_JPRB, 0.21852E+01_JPRB, 0.23645E+01_JPRB, 0.25499E+01_JPRB, 0.27247E+01_JPRB, 0.27974E+01_JPRB ,&
&  0.28142E+01_JPRB, 0.28465E+01_JPRB, 0.28361E+01_JPRB, 0.26060E+01_JPRB, 0.24869E+01_JPRB, 0.24720E+01_JPRB, 0.24848E+01_JPRB ,&
&  0.24693E+01_JPRB, 0.25567E+01_JPRB, 0.25309E+01_JPRB, 0.23223E+01_JPRB, 0.20205E+01_JPRB, 0.20295E+01_JPRB, 0.17741E+01_JPRB ,&
&  0.17583E+01_JPRB, 0.16413E+01_JPRB, 0.17225E+01_JPRB, 0.13656E+01_JPRB, 0.12849E+01_JPRB, 0.12979E+01_JPRB, 0.12665E+01_JPRB ,&
&  0.12510E+01_JPRB, 0.12991E+01_JPRB, 0.13233E+01_JPRB, 0.11192E+01_JPRB, 0.10512E+01_JPRB, 0.77571E+00_JPRB, 0.83597E+00_JPRB ,&
&  0.82655E+00_JPRB, 0.90852E+00_JPRB, 0.63942E+00_JPRB, 0.48756E+00_JPRB, 0.40626E+00_JPRB, 0.40659E+00_JPRB, 0.43247E+00_JPRB ,&
&  0.42610E+00_JPRB, 0.43963E+00_JPRB, 0.46024E+00_JPRB, 0.48212E+00_JPRB, 0.48757E+00_JPRB, 0.47100E+00_JPRB, 0.48234E+00_JPRB ,&
&  0.53541E+00_JPRB, 0.58066E+00_JPRB, 0.59250E+00_JPRB, 0.60614E+00_JPRB, 0.63132E+00_JPRB, 0.67624E+00_JPRB, 0.70808E+00_JPRB ,&
&  0.74458E+00_JPRB, 0.76375E+00_JPRB, 0.78749E+00_JPRB, 0.80046E+00_JPRB, 0.81873E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.70131E+00_JPRB, 0.73335E+00_JPRB, 0.75637E+00_JPRB, 0.75531E+00_JPRB, 0.75834E+00_JPRB, 0.77498E+00_JPRB, 0.81698E+00_JPRB ,&
&  0.86967E+00_JPRB, 0.93126E+00_JPRB, 0.10037E+01_JPRB, 0.10926E+01_JPRB, 0.12064E+01_JPRB, 0.13046E+01_JPRB, 0.14181E+01_JPRB ,&
&  0.15310E+01_JPRB, 0.16430E+01_JPRB, 0.17616E+01_JPRB, 0.18845E+01_JPRB, 0.19406E+01_JPRB, 0.19327E+01_JPRB, 0.15372E+01_JPRB ,&
&  0.10863E+01_JPRB, 0.19024E+01_JPRB, 0.18286E+01_JPRB, 0.19314E+01_JPRB, 0.19510E+01_JPRB, 0.20291E+01_JPRB, 0.20188E+01_JPRB ,&
&  0.20195E+01_JPRB, 0.19445E+01_JPRB, 0.18810E+01_JPRB, 0.18530E+01_JPRB, 0.18512E+01_JPRB, 0.18629E+01_JPRB, 0.18495E+01_JPRB ,&
&  0.18523E+01_JPRB, 0.19137E+01_JPRB, 0.20392E+01_JPRB, 0.22289E+01_JPRB, 0.24602E+01_JPRB, 0.27130E+01_JPRB, 0.28679E+01_JPRB ,&
&  0.29723E+01_JPRB, 0.30699E+01_JPRB, 0.30789E+01_JPRB, 0.28716E+01_JPRB, 0.27480E+01_JPRB, 0.28216E+01_JPRB, 0.30272E+01_JPRB ,&
&  0.29768E+01_JPRB, 0.28709E+01_JPRB, 0.29571E+01_JPRB, 0.27567E+01_JPRB, 0.23858E+01_JPRB, 0.22299E+01_JPRB, 0.18311E+01_JPRB ,&
&  0.18610E+01_JPRB, 0.16698E+01_JPRB, 0.17157E+01_JPRB, 0.13821E+01_JPRB, 0.13348E+01_JPRB, 0.14306E+01_JPRB, 0.14198E+01_JPRB ,&
&  0.13142E+01_JPRB, 0.13702E+01_JPRB, 0.13670E+01_JPRB, 0.11962E+01_JPRB, 0.11018E+01_JPRB, 0.93550E+00_JPRB, 0.88152E+00_JPRB ,&
&  0.84167E+00_JPRB, 0.80916E+00_JPRB, 0.66165E+00_JPRB, 0.43521E+00_JPRB, 0.38676E+00_JPRB, 0.39373E+00_JPRB, 0.43431E+00_JPRB ,&
&  0.43666E+00_JPRB, 0.47109E+00_JPRB, 0.51642E+00_JPRB, 0.50640E+00_JPRB, 0.47560E+00_JPRB, 0.47473E+00_JPRB, 0.49386E+00_JPRB ,&
&  0.54425E+00_JPRB, 0.58608E+00_JPRB, 0.59166E+00_JPRB, 0.58623E+00_JPRB, 0.57133E+00_JPRB, 0.60376E+00_JPRB, 0.62437E+00_JPRB ,&
&  0.65574E+00_JPRB, 0.65443E+00_JPRB, 0.67380E+00_JPRB, 0.68951E+00_JPRB, 0.69505E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.61575E+00_JPRB, 0.63534E+00_JPRB, 0.66966E+00_JPRB, 0.67429E+00_JPRB, 0.66130E+00_JPRB, 0.66084E+00_JPRB, 0.67483E+00_JPRB ,&
&  0.69585E+00_JPRB, 0.72394E+00_JPRB, 0.77081E+00_JPRB, 0.83530E+00_JPRB, 0.93555E+00_JPRB, 0.10376E+01_JPRB, 0.11716E+01_JPRB ,&
&  0.13460E+01_JPRB, 0.15454E+01_JPRB, 0.17655E+01_JPRB, 0.20010E+01_JPRB, 0.21640E+01_JPRB, 0.21589E+01_JPRB, 0.13820E+01_JPRB ,&
&  0.12651E+01_JPRB, 0.19140E+01_JPRB, 0.19652E+01_JPRB, 0.21559E+01_JPRB, 0.23192E+01_JPRB, 0.23768E+01_JPRB, 0.22215E+01_JPRB ,&
&  0.21485E+01_JPRB, 0.20508E+01_JPRB, 0.19971E+01_JPRB, 0.19433E+01_JPRB, 0.19152E+01_JPRB, 0.19110E+01_JPRB, 0.18660E+01_JPRB ,&
&  0.18086E+01_JPRB, 0.17861E+01_JPRB, 0.18338E+01_JPRB, 0.19659E+01_JPRB, 0.21654E+01_JPRB, 0.23858E+01_JPRB, 0.26900E+01_JPRB ,&
&  0.29979E+01_JPRB, 0.32684E+01_JPRB, 0.35085E+01_JPRB, 0.32617E+01_JPRB, 0.31887E+01_JPRB, 0.31764E+01_JPRB, 0.33449E+01_JPRB ,&
&  0.32918E+01_JPRB, 0.31261E+01_JPRB, 0.34042E+01_JPRB, 0.35366E+01_JPRB, 0.31530E+01_JPRB, 0.26211E+01_JPRB, 0.20804E+01_JPRB ,&
&  0.19471E+01_JPRB, 0.18458E+01_JPRB, 0.17285E+01_JPRB, 0.15119E+01_JPRB, 0.14456E+01_JPRB, 0.15837E+01_JPRB, 0.17155E+01_JPRB ,&
&  0.16501E+01_JPRB, 0.16293E+01_JPRB, 0.15251E+01_JPRB, 0.13358E+01_JPRB, 0.11784E+01_JPRB, 0.10408E+01_JPRB, 0.10016E+01_JPRB ,&
&  0.91985E+00_JPRB, 0.88153E+00_JPRB, 0.69509E+00_JPRB, 0.43226E+00_JPRB, 0.35746E+00_JPRB, 0.32438E+00_JPRB, 0.37912E+00_JPRB ,&
&  0.43604E+00_JPRB, 0.48001E+00_JPRB, 0.49126E+00_JPRB, 0.59587E+00_JPRB, 0.51852E+00_JPRB, 0.49966E+00_JPRB, 0.50522E+00_JPRB ,&
&  0.54569E+00_JPRB, 0.58351E+00_JPRB, 0.60003E+00_JPRB, 0.57873E+00_JPRB, 0.54771E+00_JPRB, 0.55241E+00_JPRB, 0.55994E+00_JPRB ,&
&  0.58048E+00_JPRB, 0.58822E+00_JPRB, 0.58422E+00_JPRB, 0.59510E+00_JPRB, 0.61263E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.56914E+00_JPRB, 0.58320E+00_JPRB, 0.59772E+00_JPRB, 0.60995E+00_JPRB, 0.61418E+00_JPRB, 0.60846E+00_JPRB, 0.59952E+00_JPRB ,&
&  0.59927E+00_JPRB, 0.60838E+00_JPRB, 0.64041E+00_JPRB, 0.67945E+00_JPRB, 0.73979E+00_JPRB, 0.80150E+00_JPRB, 0.89999E+00_JPRB ,&
&  0.10744E+01_JPRB, 0.13115E+01_JPRB, 0.16378E+01_JPRB, 0.20055E+01_JPRB, 0.21415E+01_JPRB, 0.18878E+01_JPRB, 0.16761E+01_JPRB ,&
&  0.16165E+01_JPRB, 0.19727E+01_JPRB, 0.20401E+01_JPRB, 0.23915E+01_JPRB, 0.27258E+01_JPRB, 0.28618E+01_JPRB, 0.23444E+01_JPRB ,&
&  0.22123E+01_JPRB, 0.20892E+01_JPRB, 0.19698E+01_JPRB, 0.19292E+01_JPRB, 0.19129E+01_JPRB, 0.19276E+01_JPRB, 0.19159E+01_JPRB ,&
&  0.18474E+01_JPRB, 0.17645E+01_JPRB, 0.17240E+01_JPRB, 0.17510E+01_JPRB, 0.18794E+01_JPRB, 0.20381E+01_JPRB, 0.22595E+01_JPRB ,&
&  0.26430E+01_JPRB, 0.31614E+01_JPRB, 0.36876E+01_JPRB, 0.37401E+01_JPRB, 0.35556E+01_JPRB, 0.35244E+01_JPRB, 0.35576E+01_JPRB ,&
&  0.36347E+01_JPRB, 0.39370E+01_JPRB, 0.40207E+01_JPRB, 0.39164E+01_JPRB, 0.34938E+01_JPRB, 0.31293E+01_JPRB, 0.24884E+01_JPRB ,&
&  0.21477E+01_JPRB, 0.19521E+01_JPRB, 0.17666E+01_JPRB, 0.17451E+01_JPRB, 0.17141E+01_JPRB, 0.17966E+01_JPRB, 0.16840E+01_JPRB ,&
&  0.14724E+01_JPRB, 0.13070E+01_JPRB, 0.12350E+01_JPRB, 0.12780E+01_JPRB, 0.12914E+01_JPRB, 0.12385E+01_JPRB, 0.12335E+01_JPRB ,&
&  0.92454E+00_JPRB, 0.39909E+00_JPRB, 0.26477E+00_JPRB, 0.32398E+00_JPRB, 0.24557E+00_JPRB, 0.22871E+00_JPRB, 0.33293E+00_JPRB ,&
&  0.42308E+00_JPRB, 0.50982E+00_JPRB, 0.56187E+00_JPRB, 0.59468E+00_JPRB, 0.62446E+00_JPRB, 0.58068E+00_JPRB, 0.55346E+00_JPRB ,&
&  0.55935E+00_JPRB, 0.58484E+00_JPRB, 0.61882E+00_JPRB, 0.60802E+00_JPRB, 0.56447E+00_JPRB, 0.54763E+00_JPRB, 0.53279E+00_JPRB ,&
&  0.53271E+00_JPRB, 0.55548E+00_JPRB, 0.56921E+00_JPRB, 0.56941E+00_JPRB, 0.56430E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.57249E+00_JPRB, 0.57130E+00_JPRB, 0.57818E+00_JPRB, 0.56708E+00_JPRB, 0.54844E+00_JPRB, 0.53799E+00_JPRB, 0.54445E+00_JPRB ,&
&  0.53838E+00_JPRB, 0.53659E+00_JPRB, 0.55879E+00_JPRB, 0.59298E+00_JPRB, 0.64027E+00_JPRB, 0.69144E+00_JPRB, 0.74005E+00_JPRB ,&
&  0.85027E+00_JPRB, 0.10591E+01_JPRB, 0.14090E+01_JPRB, 0.19103E+01_JPRB, 0.21967E+01_JPRB, 0.20358E+01_JPRB, 0.19701E+01_JPRB ,&
&  0.17727E+01_JPRB, 0.21057E+01_JPRB, 0.25663E+01_JPRB, 0.32719E+01_JPRB, 0.34632E+01_JPRB, 0.32541E+01_JPRB, 0.25920E+01_JPRB ,&
&  0.24669E+01_JPRB, 0.22684E+01_JPRB, 0.20550E+01_JPRB, 0.18509E+01_JPRB, 0.17394E+01_JPRB, 0.17236E+01_JPRB, 0.17628E+01_JPRB ,&
&  0.17820E+01_JPRB, 0.17295E+01_JPRB, 0.16640E+01_JPRB, 0.16406E+01_JPRB, 0.16947E+01_JPRB, 0.18167E+01_JPRB, 0.19694E+01_JPRB ,&
&  0.21867E+01_JPRB, 0.25888E+01_JPRB, 0.34118E+01_JPRB, 0.41890E+01_JPRB, 0.34106E+01_JPRB, 0.39234E+01_JPRB, 0.37389E+01_JPRB ,&
&  0.37096E+01_JPRB, 0.40925E+01_JPRB, 0.41492E+01_JPRB, 0.46892E+01_JPRB, 0.39034E+01_JPRB, 0.32925E+01_JPRB, 0.26957E+01_JPRB ,&
&  0.23672E+01_JPRB, 0.19615E+01_JPRB, 0.18491E+01_JPRB, 0.18987E+01_JPRB, 0.19486E+01_JPRB, 0.19567E+01_JPRB, 0.13605E+01_JPRB ,&
&  0.12848E+01_JPRB, 0.14275E+01_JPRB, 0.13409E+01_JPRB, 0.11667E+01_JPRB, 0.13384E+01_JPRB, 0.14580E+01_JPRB, 0.98178E+00_JPRB ,&
&  0.38707E+00_JPRB, 0.27558E+00_JPRB, 0.23561E+00_JPRB, 0.22285E+00_JPRB, 0.19605E+00_JPRB, 0.21659E+00_JPRB, 0.47067E+00_JPRB ,&
&  0.47361E+00_JPRB, 0.59346E+00_JPRB, 0.65476E+00_JPRB, 0.68969E+00_JPRB, 0.76252E+00_JPRB, 0.70233E+00_JPRB, 0.10338E+01_JPRB ,&
&  0.81215E+00_JPRB, 0.70975E+00_JPRB, 0.68631E+00_JPRB, 0.67914E+00_JPRB, 0.63250E+00_JPRB, 0.60527E+00_JPRB, 0.57487E+00_JPRB ,&
&  0.56475E+00_JPRB, 0.57031E+00_JPRB, 0.56885E+00_JPRB, 0.56539E+00_JPRB, 0.57596E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.59370E+00_JPRB, 0.58819E+00_JPRB, 0.57427E+00_JPRB, 0.53985E+00_JPRB, 0.51286E+00_JPRB, 0.49904E+00_JPRB, 0.49219E+00_JPRB ,&
&  0.48788E+00_JPRB, 0.48784E+00_JPRB, 0.49684E+00_JPRB, 0.52062E+00_JPRB, 0.54835E+00_JPRB, 0.58979E+00_JPRB, 0.64489E+00_JPRB ,&
&  0.74212E+00_JPRB, 0.93020E+00_JPRB, 0.12847E+01_JPRB, 0.16669E+01_JPRB, 0.20731E+01_JPRB, 0.20211E+01_JPRB, 0.21410E+01_JPRB ,&
&  0.19248E+01_JPRB, 0.21132E+01_JPRB, 0.27091E+01_JPRB, 0.37023E+01_JPRB, 0.50989E+01_JPRB, 0.50666E+01_JPRB, 0.43810E+01_JPRB ,&
&  0.37730E+01_JPRB, 0.33140E+01_JPRB, 0.29458E+01_JPRB, 0.25419E+01_JPRB, 0.19468E+01_JPRB, 0.16111E+01_JPRB, 0.15329E+01_JPRB ,&
&  0.15138E+01_JPRB, 0.15510E+01_JPRB, 0.15180E+01_JPRB, 0.15039E+01_JPRB, 0.15748E+01_JPRB, 0.16917E+01_JPRB, 0.18509E+01_JPRB ,&
&  0.19760E+01_JPRB, 0.21484E+01_JPRB, 0.26990E+01_JPRB, 0.38698E+01_JPRB, 0.39883E+01_JPRB, 0.37358E+01_JPRB, 0.37009E+01_JPRB ,&
&  0.36575E+01_JPRB, 0.42130E+01_JPRB, 0.47613E+01_JPRB, 0.56899E+01_JPRB, 0.39921E+01_JPRB, 0.32726E+01_JPRB, 0.28884E+01_JPRB ,&
&  0.25642E+01_JPRB, 0.23719E+01_JPRB, 0.18583E+01_JPRB, 0.18319E+01_JPRB, 0.20004E+01_JPRB, 0.16511E+01_JPRB, 0.14223E+01_JPRB ,&
&  0.15807E+01_JPRB, 0.14252E+01_JPRB, 0.13264E+01_JPRB, 0.11601E+01_JPRB, 0.14492E+01_JPRB, 0.12762E+01_JPRB, 0.55411E+00_JPRB ,&
&  0.35801E+00_JPRB, 0.29164E+00_JPRB, 0.22384E+00_JPRB, 0.21375E+00_JPRB, 0.20256E+00_JPRB, 0.22839E+00_JPRB, 0.40310E+00_JPRB ,&
&  0.46277E+00_JPRB, 0.65387E+00_JPRB, 0.73476E+00_JPRB, 0.79915E+00_JPRB, 0.84681E+00_JPRB, 0.98734E+00_JPRB, 0.23336E+01_JPRB ,&
&  0.21717E+01_JPRB, 0.16906E+01_JPRB, 0.12908E+01_JPRB, 0.10367E+01_JPRB, 0.83819E+00_JPRB, 0.73290E+00_JPRB, 0.66591E+00_JPRB ,&
&  0.63368E+00_JPRB, 0.61631E+00_JPRB, 0.60455E+00_JPRB, 0.59455E+00_JPRB, 0.58631E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.60620E+00_JPRB, 0.60886E+00_JPRB, 0.59541E+00_JPRB, 0.54113E+00_JPRB, 0.50690E+00_JPRB, 0.50269E+00_JPRB, 0.49034E+00_JPRB ,&
&  0.47398E+00_JPRB, 0.46204E+00_JPRB, 0.45970E+00_JPRB, 0.46890E+00_JPRB, 0.47631E+00_JPRB, 0.49331E+00_JPRB, 0.54755E+00_JPRB ,&
&  0.67041E+00_JPRB, 0.88663E+00_JPRB, 0.11854E+01_JPRB, 0.14553E+01_JPRB, 0.17050E+01_JPRB, 0.17653E+01_JPRB, 0.20999E+01_JPRB ,&
&  0.21592E+01_JPRB, 0.21809E+01_JPRB, 0.26015E+01_JPRB, 0.37666E+01_JPRB, 0.56106E+01_JPRB, 0.67069E+01_JPRB, 0.69927E+01_JPRB ,&
&  0.59713E+01_JPRB, 0.48865E+01_JPRB, 0.41485E+01_JPRB, 0.36425E+01_JPRB, 0.29631E+01_JPRB, 0.21426E+01_JPRB, 0.16182E+01_JPRB ,&
&  0.14041E+01_JPRB, 0.14207E+01_JPRB, 0.14534E+01_JPRB, 0.14242E+01_JPRB, 0.14876E+01_JPRB, 0.15973E+01_JPRB, 0.17916E+01_JPRB ,&
&  0.19301E+01_JPRB, 0.20241E+01_JPRB, 0.22597E+01_JPRB, 0.31322E+01_JPRB, 0.43605E+01_JPRB, 0.46728E+01_JPRB, 0.38869E+01_JPRB ,&
&  0.33342E+01_JPRB, 0.34421E+01_JPRB, 0.53132E+01_JPRB, 0.58017E+01_JPRB, 0.40734E+01_JPRB, 0.34540E+01_JPRB, 0.30272E+01_JPRB ,&
&  0.28325E+01_JPRB, 0.26802E+01_JPRB, 0.21528E+01_JPRB, 0.20334E+01_JPRB, 0.17848E+01_JPRB, 0.14901E+01_JPRB, 0.16636E+01_JPRB ,&
&  0.17068E+01_JPRB, 0.15355E+01_JPRB, 0.13105E+01_JPRB, 0.11199E+01_JPRB, 0.11958E+01_JPRB, 0.87069E+00_JPRB, 0.58399E+00_JPRB ,&
&  0.47159E+00_JPRB, 0.37047E+00_JPRB, 0.29729E+00_JPRB, 0.29085E+00_JPRB, 0.25273E+00_JPRB, 0.27245E+00_JPRB, 0.35395E+00_JPRB ,&
&  0.50889E+00_JPRB, 0.63389E+00_JPRB, 0.80595E+00_JPRB, 0.91872E+00_JPRB, 0.90984E+00_JPRB, 0.85121E+00_JPRB, 0.20507E+01_JPRB ,&
&  0.15762E+01_JPRB, 0.15301E+01_JPRB, 0.15597E+01_JPRB, 0.13155E+01_JPRB, 0.10632E+01_JPRB, 0.89376E+00_JPRB, 0.76749E+00_JPRB ,&
&  0.68700E+00_JPRB, 0.63972E+00_JPRB, 0.61368E+00_JPRB, 0.60888E+00_JPRB, 0.61355E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.66002E+00_JPRB, 0.62394E+00_JPRB, 0.60431E+00_JPRB, 0.56775E+00_JPRB, 0.51486E+00_JPRB, 0.49214E+00_JPRB, 0.49795E+00_JPRB ,&
&  0.48690E+00_JPRB, 0.46800E+00_JPRB, 0.45096E+00_JPRB, 0.43752E+00_JPRB, 0.43985E+00_JPRB, 0.45005E+00_JPRB, 0.47711E+00_JPRB ,&
&  0.60919E+00_JPRB, 0.94339E+00_JPRB, 0.98207E+00_JPRB, 0.11140E+01_JPRB, 0.13418E+01_JPRB, 0.17068E+01_JPRB, 0.19006E+01_JPRB ,&
&  0.23085E+01_JPRB, 0.23747E+01_JPRB, 0.25930E+01_JPRB, 0.38079E+01_JPRB, 0.68607E+01_JPRB, 0.81627E+01_JPRB, 0.89301E+01_JPRB ,&
&  0.87153E+01_JPRB, 0.71838E+01_JPRB, 0.57021E+01_JPRB, 0.44714E+01_JPRB, 0.36438E+01_JPRB, 0.30770E+01_JPRB, 0.23356E+01_JPRB ,&
&  0.17888E+01_JPRB, 0.15660E+01_JPRB, 0.15345E+01_JPRB, 0.15064E+01_JPRB, 0.14966E+01_JPRB, 0.15503E+01_JPRB, 0.16961E+01_JPRB ,&
&  0.18862E+01_JPRB, 0.20552E+01_JPRB, 0.22114E+01_JPRB, 0.28555E+01_JPRB, 0.36355E+01_JPRB, 0.40209E+01_JPRB, 0.47676E+01_JPRB ,&
&  0.43368E+01_JPRB, 0.45204E+01_JPRB, 0.53091E+01_JPRB, 0.64474E+01_JPRB, 0.44220E+01_JPRB, 0.34505E+01_JPRB, 0.31190E+01_JPRB ,&
&  0.24651E+01_JPRB, 0.22435E+01_JPRB, 0.22333E+01_JPRB, 0.20131E+01_JPRB, 0.16438E+01_JPRB, 0.18003E+01_JPRB, 0.23430E+01_JPRB ,&
&  0.18023E+01_JPRB, 0.17262E+01_JPRB, 0.16759E+01_JPRB, 0.16306E+01_JPRB, 0.11088E+01_JPRB, 0.78227E+00_JPRB, 0.11012E+01_JPRB ,&
&  0.10993E+01_JPRB, 0.62674E+00_JPRB, 0.46147E+00_JPRB, 0.51463E+00_JPRB, 0.37669E+00_JPRB, 0.37256E+00_JPRB, 0.42304E+00_JPRB ,&
&  0.46016E+00_JPRB, 0.49058E+00_JPRB, 0.78787E+00_JPRB, 0.83963E+00_JPRB, 0.86908E+00_JPRB, 0.75498E+00_JPRB, 0.13522E+01_JPRB ,&
&  0.14965E+01_JPRB, 0.12599E+01_JPRB, 0.13760E+01_JPRB, 0.12994E+01_JPRB, 0.11469E+01_JPRB, 0.10152E+01_JPRB, 0.92512E+00_JPRB ,&
&  0.83598E+00_JPRB, 0.76504E+00_JPRB, 0.72413E+00_JPRB, 0.70425E+00_JPRB, 0.69803E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.63373E+00_JPRB, 0.55994E+00_JPRB, 0.52729E+00_JPRB, 0.52551E+00_JPRB, 0.52064E+00_JPRB, 0.48515E+00_JPRB, 0.47849E+00_JPRB ,&
&  0.49666E+00_JPRB, 0.47645E+00_JPRB, 0.44645E+00_JPRB, 0.43174E+00_JPRB, 0.43467E+00_JPRB, 0.43862E+00_JPRB, 0.43582E+00_JPRB ,&
&  0.51232E+00_JPRB, 0.70937E+00_JPRB, 0.81127E+00_JPRB, 0.92913E+00_JPRB, 0.11963E+01_JPRB, 0.15381E+01_JPRB, 0.17326E+01_JPRB ,&
&  0.21871E+01_JPRB, 0.24591E+01_JPRB, 0.28877E+01_JPRB, 0.42465E+01_JPRB, 0.61050E+01_JPRB, 0.78686E+01_JPRB, 0.84741E+01_JPRB ,&
&  0.92313E+01_JPRB, 0.87519E+01_JPRB, 0.69433E+01_JPRB, 0.56256E+01_JPRB, 0.47284E+01_JPRB, 0.41445E+01_JPRB, 0.34995E+01_JPRB ,&
&  0.27124E+01_JPRB, 0.20940E+01_JPRB, 0.18258E+01_JPRB, 0.17093E+01_JPRB, 0.16823E+01_JPRB, 0.16295E+01_JPRB, 0.16984E+01_JPRB ,&
&  0.18639E+01_JPRB, 0.20942E+01_JPRB, 0.22259E+01_JPRB, 0.27890E+01_JPRB, 0.37523E+01_JPRB, 0.42401E+01_JPRB, 0.47298E+01_JPRB ,&
&  0.48299E+01_JPRB, 0.45506E+01_JPRB, 0.52852E+01_JPRB, 0.54921E+01_JPRB, 0.47451E+01_JPRB, 0.37832E+01_JPRB, 0.35196E+01_JPRB ,&
&  0.27378E+01_JPRB, 0.23465E+01_JPRB, 0.21651E+01_JPRB, 0.18686E+01_JPRB, 0.18971E+01_JPRB, 0.24025E+01_JPRB, 0.23943E+01_JPRB ,&
&  0.21028E+01_JPRB, 0.19264E+01_JPRB, 0.17286E+01_JPRB, 0.14721E+01_JPRB, 0.14024E+01_JPRB, 0.10897E+01_JPRB, 0.12538E+01_JPRB ,&
&  0.12847E+01_JPRB, 0.11910E+01_JPRB, 0.97062E+00_JPRB, 0.79143E+00_JPRB, 0.60520E+00_JPRB, 0.56628E+00_JPRB, 0.49257E+00_JPRB ,&
&  0.46770E+00_JPRB, 0.45489E+00_JPRB, 0.60350E+00_JPRB, 0.71377E+00_JPRB, 0.73691E+00_JPRB, 0.68963E+00_JPRB, 0.10076E+01_JPRB ,&
&  0.11207E+01_JPRB, 0.12429E+01_JPRB, 0.11971E+01_JPRB, 0.10986E+01_JPRB, 0.99646E+00_JPRB, 0.93083E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.85103E+00_JPRB, 0.81612E+00_JPRB, 0.79275E+00_JPRB, 0.75861E+00_JPRB, 0.71223E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.58690E+00_JPRB, 0.51729E+00_JPRB, 0.48305E+00_JPRB, 0.47274E+00_JPRB, 0.47916E+00_JPRB, 0.47998E+00_JPRB, 0.46795E+00_JPRB ,&
&  0.46392E+00_JPRB, 0.47546E+00_JPRB, 0.46088E+00_JPRB, 0.44694E+00_JPRB, 0.44261E+00_JPRB, 0.44062E+00_JPRB, 0.42608E+00_JPRB ,&
&  0.45985E+00_JPRB, 0.58843E+00_JPRB, 0.65261E+00_JPRB, 0.81866E+00_JPRB, 0.10393E+01_JPRB, 0.13835E+01_JPRB, 0.17807E+01_JPRB ,&
&  0.19964E+01_JPRB, 0.22672E+01_JPRB, 0.26711E+01_JPRB, 0.36849E+01_JPRB, 0.48839E+01_JPRB, 0.59376E+01_JPRB, 0.62033E+01_JPRB ,&
&  0.58676E+01_JPRB, 0.59915E+01_JPRB, 0.56165E+01_JPRB, 0.50264E+01_JPRB, 0.45318E+01_JPRB, 0.41539E+01_JPRB, 0.36842E+01_JPRB ,&
&  0.31277E+01_JPRB, 0.25635E+01_JPRB, 0.21644E+01_JPRB, 0.19549E+01_JPRB, 0.18694E+01_JPRB, 0.19272E+01_JPRB, 0.19791E+01_JPRB ,&
&  0.20020E+01_JPRB, 0.21344E+01_JPRB, 0.21549E+01_JPRB, 0.24798E+01_JPRB, 0.31746E+01_JPRB, 0.42019E+01_JPRB, 0.46236E+01_JPRB ,&
&  0.54175E+01_JPRB, 0.51373E+01_JPRB, 0.56227E+01_JPRB, 0.59901E+01_JPRB, 0.43130E+01_JPRB, 0.40855E+01_JPRB, 0.41983E+01_JPRB ,&
&  0.40047E+01_JPRB, 0.35148E+01_JPRB, 0.32029E+01_JPRB, 0.25343E+01_JPRB, 0.21142E+01_JPRB, 0.25634E+01_JPRB, 0.24058E+01_JPRB ,&
&  0.21560E+01_JPRB, 0.20209E+01_JPRB, 0.18119E+01_JPRB, 0.17412E+01_JPRB, 0.15117E+01_JPRB, 0.14298E+01_JPRB, 0.11791E+01_JPRB ,&
&  0.11761E+01_JPRB, 0.10147E+01_JPRB, 0.10466E+01_JPRB, 0.90622E+00_JPRB, 0.79686E+00_JPRB, 0.66688E+00_JPRB, 0.53315E+00_JPRB ,&
&  0.46830E+00_JPRB, 0.44605E+00_JPRB, 0.48401E+00_JPRB, 0.60627E+00_JPRB, 0.69135E+00_JPRB, 0.64377E+00_JPRB, 0.75347E+00_JPRB ,&
&  0.95384E+00_JPRB, 0.10796E+01_JPRB, 0.97570E+00_JPRB, 0.99614E+00_JPRB, 0.90620E+00_JPRB, 0.83237E+00_JPRB, 0.82096E+00_JPRB ,&
&  0.76760E+00_JPRB, 0.75007E+00_JPRB, 0.73456E+00_JPRB, 0.70630E+00_JPRB, 0.66195E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.54329E+00_JPRB, 0.48319E+00_JPRB, 0.44272E+00_JPRB, 0.43664E+00_JPRB, 0.45192E+00_JPRB, 0.46246E+00_JPRB, 0.46607E+00_JPRB ,&
&  0.45845E+00_JPRB, 0.46509E+00_JPRB, 0.46994E+00_JPRB, 0.46429E+00_JPRB, 0.45947E+00_JPRB, 0.45423E+00_JPRB, 0.43199E+00_JPRB ,&
&  0.42672E+00_JPRB, 0.54670E+00_JPRB, 0.56860E+00_JPRB, 0.63826E+00_JPRB, 0.80706E+00_JPRB, 0.12184E+01_JPRB, 0.16203E+01_JPRB ,&
&  0.18777E+01_JPRB, 0.20830E+01_JPRB, 0.24256E+01_JPRB, 0.29299E+01_JPRB, 0.34058E+01_JPRB, 0.36465E+01_JPRB, 0.37105E+01_JPRB ,&
&  0.40676E+01_JPRB, 0.34186E+01_JPRB, 0.34684E+01_JPRB, 0.31422E+01_JPRB, 0.29736E+01_JPRB, 0.28021E+01_JPRB, 0.26410E+01_JPRB ,&
&  0.24695E+01_JPRB, 0.22558E+01_JPRB, 0.20341E+01_JPRB, 0.18909E+01_JPRB, 0.18282E+01_JPRB, 0.18247E+01_JPRB, 0.18779E+01_JPRB ,&
&  0.19388E+01_JPRB, 0.19885E+01_JPRB, 0.20730E+01_JPRB, 0.20345E+01_JPRB, 0.24856E+01_JPRB, 0.34654E+01_JPRB, 0.45128E+01_JPRB ,&
&  0.44985E+01_JPRB, 0.40453E+01_JPRB, 0.61288E+01_JPRB, 0.61048E+01_JPRB, 0.52034E+01_JPRB, 0.49580E+01_JPRB, 0.47816E+01_JPRB ,&
&  0.45490E+01_JPRB, 0.39417E+01_JPRB, 0.32017E+01_JPRB, 0.25581E+01_JPRB, 0.27408E+01_JPRB, 0.26795E+01_JPRB, 0.25059E+01_JPRB ,&
&  0.22918E+01_JPRB, 0.21746E+01_JPRB, 0.19938E+01_JPRB, 0.18258E+01_JPRB, 0.17033E+01_JPRB, 0.16950E+01_JPRB, 0.15342E+01_JPRB ,&
&  0.13892E+01_JPRB, 0.13787E+01_JPRB, 0.11593E+01_JPRB, 0.92947E+00_JPRB, 0.78920E+00_JPRB, 0.68272E+00_JPRB, 0.57414E+00_JPRB ,&
&  0.47057E+00_JPRB, 0.43624E+00_JPRB, 0.44076E+00_JPRB, 0.50131E+00_JPRB, 0.61147E+00_JPRB, 0.58631E+00_JPRB, 0.62689E+00_JPRB ,&
&  0.66138E+00_JPRB, 0.89037E+00_JPRB, 0.89280E+00_JPRB, 0.87768E+00_JPRB, 0.85223E+00_JPRB, 0.78252E+00_JPRB, 0.73045E+00_JPRB ,&
&  0.67568E+00_JPRB, 0.67571E+00_JPRB, 0.67296E+00_JPRB, 0.64492E+00_JPRB, 0.61012E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.51786E+00_JPRB, 0.46343E+00_JPRB, 0.42479E+00_JPRB, 0.42013E+00_JPRB, 0.44380E+00_JPRB, 0.45636E+00_JPRB, 0.46242E+00_JPRB ,&
&  0.46093E+00_JPRB, 0.46044E+00_JPRB, 0.47250E+00_JPRB, 0.47820E+00_JPRB, 0.48357E+00_JPRB, 0.47586E+00_JPRB, 0.44765E+00_JPRB ,&
&  0.44215E+00_JPRB, 0.52482E+00_JPRB, 0.50882E+00_JPRB, 0.57832E+00_JPRB, 0.69270E+00_JPRB, 0.97538E+00_JPRB, 0.12401E+01_JPRB ,&
&  0.14883E+01_JPRB, 0.16189E+01_JPRB, 0.17906E+01_JPRB, 0.20496E+01_JPRB, 0.21902E+01_JPRB, 0.21164E+01_JPRB, 0.20307E+01_JPRB ,&
&  0.19316E+01_JPRB, 0.19036E+01_JPRB, 0.16492E+01_JPRB, 0.17581E+01_JPRB, 0.17203E+01_JPRB, 0.15060E+01_JPRB, 0.16805E+01_JPRB ,&
&  0.16758E+01_JPRB, 0.16856E+01_JPRB, 0.16469E+01_JPRB, 0.16173E+01_JPRB, 0.16141E+01_JPRB, 0.16119E+01_JPRB, 0.16042E+01_JPRB ,&
&  0.16048E+01_JPRB, 0.16313E+01_JPRB, 0.18072E+01_JPRB, 0.18542E+01_JPRB, 0.19425E+01_JPRB, 0.24174E+01_JPRB, 0.36069E+01_JPRB ,&
&  0.49314E+01_JPRB, 0.48195E+01_JPRB, 0.46128E+01_JPRB, 0.55706E+01_JPRB, 0.59105E+01_JPRB, 0.54011E+01_JPRB, 0.48749E+01_JPRB ,&
&  0.46580E+01_JPRB, 0.41624E+01_JPRB, 0.36416E+01_JPRB, 0.31677E+01_JPRB, 0.31115E+01_JPRB, 0.30726E+01_JPRB, 0.28362E+01_JPRB ,&
&  0.24967E+01_JPRB, 0.22505E+01_JPRB, 0.21130E+01_JPRB, 0.19209E+01_JPRB, 0.17209E+01_JPRB, 0.15887E+01_JPRB, 0.15435E+01_JPRB ,&
&  0.14418E+01_JPRB, 0.12427E+01_JPRB, 0.96900E+00_JPRB, 0.84867E+00_JPRB, 0.69595E+00_JPRB, 0.59989E+00_JPRB, 0.55251E+00_JPRB ,&
&  0.46437E+00_JPRB, 0.40760E+00_JPRB, 0.40783E+00_JPRB, 0.42653E+00_JPRB, 0.51386E+00_JPRB, 0.50390E+00_JPRB, 0.54795E+00_JPRB ,&
&  0.54864E+00_JPRB, 0.54549E+00_JPRB, 0.72486E+00_JPRB, 0.78612E+00_JPRB, 0.77858E+00_JPRB, 0.70247E+00_JPRB, 0.63197E+00_JPRB ,&
&  0.59975E+00_JPRB, 0.61934E+00_JPRB, 0.61733E+00_JPRB, 0.60192E+00_JPRB, 0.57711E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.49043E+00_JPRB, 0.43810E+00_JPRB, 0.40510E+00_JPRB, 0.39576E+00_JPRB, 0.41451E+00_JPRB, 0.43591E+00_JPRB, 0.44845E+00_JPRB ,&
&  0.46016E+00_JPRB, 0.46860E+00_JPRB, 0.47594E+00_JPRB, 0.48484E+00_JPRB, 0.49312E+00_JPRB, 0.48098E+00_JPRB, 0.47153E+00_JPRB ,&
&  0.47790E+00_JPRB, 0.45808E+00_JPRB, 0.46408E+00_JPRB, 0.49575E+00_JPRB, 0.71141E+00_JPRB, 0.84541E+00_JPRB, 0.10017E+01_JPRB ,&
&  0.11184E+01_JPRB, 0.12299E+01_JPRB, 0.12463E+01_JPRB, 0.12459E+01_JPRB, 0.12382E+01_JPRB, 0.12315E+01_JPRB, 0.11312E+01_JPRB ,&
&  0.98723E+00_JPRB, 0.91178E+00_JPRB, 0.88168E+00_JPRB, 0.93875E+00_JPRB, 0.10327E+01_JPRB, 0.91278E+00_JPRB, 0.10235E+01_JPRB ,&
&  0.10918E+01_JPRB, 0.11637E+01_JPRB, 0.11993E+01_JPRB, 0.12097E+01_JPRB, 0.12344E+01_JPRB, 0.12687E+01_JPRB, 0.13050E+01_JPRB ,&
&  0.13477E+01_JPRB, 0.13556E+01_JPRB, 0.14299E+01_JPRB, 0.16628E+01_JPRB, 0.18272E+01_JPRB, 0.22923E+01_JPRB, 0.36782E+01_JPRB ,&
&  0.48971E+01_JPRB, 0.49531E+01_JPRB, 0.58368E+01_JPRB, 0.66906E+01_JPRB, 0.60541E+01_JPRB, 0.52067E+01_JPRB, 0.45985E+01_JPRB ,&
&  0.41656E+01_JPRB, 0.38149E+01_JPRB, 0.35098E+01_JPRB, 0.32614E+01_JPRB, 0.30998E+01_JPRB, 0.31256E+01_JPRB, 0.29754E+01_JPRB ,&
&  0.26416E+01_JPRB, 0.23586E+01_JPRB, 0.22065E+01_JPRB, 0.19949E+01_JPRB, 0.18368E+01_JPRB, 0.16505E+01_JPRB, 0.16063E+01_JPRB ,&
&  0.14750E+01_JPRB, 0.10906E+01_JPRB, 0.89245E+00_JPRB, 0.81350E+00_JPRB, 0.62218E+00_JPRB, 0.53197E+00_JPRB, 0.53516E+00_JPRB ,&
&  0.44168E+00_JPRB, 0.38610E+00_JPRB, 0.40874E+00_JPRB, 0.43890E+00_JPRB, 0.45608E+00_JPRB, 0.45086E+00_JPRB, 0.43577E+00_JPRB ,&
&  0.39633E+00_JPRB, 0.42845E+00_JPRB, 0.61083E+00_JPRB, 0.65530E+00_JPRB, 0.66308E+00_JPRB, 0.62753E+00_JPRB, 0.59651E+00_JPRB ,&
&  0.55419E+00_JPRB, 0.56633E+00_JPRB, 0.56623E+00_JPRB, 0.55710E+00_JPRB, 0.54187E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.48963E+00_JPRB, 0.43581E+00_JPRB, 0.39387E+00_JPRB, 0.38414E+00_JPRB, 0.40566E+00_JPRB, 0.42684E+00_JPRB, 0.44148E+00_JPRB ,&
&  0.45968E+00_JPRB, 0.47364E+00_JPRB, 0.48549E+00_JPRB, 0.49637E+00_JPRB, 0.50389E+00_JPRB, 0.49711E+00_JPRB, 0.54813E+00_JPRB ,&
&  0.41543E+00_JPRB, 0.40942E+00_JPRB, 0.39905E+00_JPRB, 0.45878E+00_JPRB, 0.69738E+00_JPRB, 0.71793E+00_JPRB, 0.77481E+00_JPRB ,&
&  0.77369E+00_JPRB, 0.77225E+00_JPRB, 0.74958E+00_JPRB, 0.73471E+00_JPRB, 0.72932E+00_JPRB, 0.74867E+00_JPRB, 0.73281E+00_JPRB ,&
&  0.64385E+00_JPRB, 0.57982E+00_JPRB, 0.56312E+00_JPRB, 0.56688E+00_JPRB, 0.62817E+00_JPRB, 0.67766E+00_JPRB, 0.76789E+00_JPRB ,&
&  0.82106E+00_JPRB, 0.88137E+00_JPRB, 0.94142E+00_JPRB, 0.98880E+00_JPRB, 0.10092E+01_JPRB, 0.10175E+01_JPRB, 0.10278E+01_JPRB ,&
&  0.10476E+01_JPRB, 0.10773E+01_JPRB, 0.11517E+01_JPRB, 0.14062E+01_JPRB, 0.16363E+01_JPRB, 0.24839E+01_JPRB, 0.44375E+01_JPRB ,&
&  0.50754E+01_JPRB, 0.44467E+01_JPRB, 0.49247E+01_JPRB, 0.58790E+01_JPRB, 0.53174E+01_JPRB, 0.44699E+01_JPRB, 0.37108E+01_JPRB ,&
&  0.32122E+01_JPRB, 0.29117E+01_JPRB, 0.27609E+01_JPRB, 0.28294E+01_JPRB, 0.27994E+01_JPRB, 0.28345E+01_JPRB, 0.28141E+01_JPRB ,&
&  0.26473E+01_JPRB, 0.25233E+01_JPRB, 0.24042E+01_JPRB, 0.22212E+01_JPRB, 0.20581E+01_JPRB, 0.19216E+01_JPRB, 0.17818E+01_JPRB ,&
&  0.15772E+01_JPRB, 0.13276E+01_JPRB, 0.10302E+01_JPRB, 0.84759E+00_JPRB, 0.65876E+00_JPRB, 0.59283E+00_JPRB, 0.64651E+00_JPRB ,&
&  0.49290E+00_JPRB, 0.42597E+00_JPRB, 0.42320E+00_JPRB, 0.44362E+00_JPRB, 0.43421E+00_JPRB, 0.42931E+00_JPRB, 0.39736E+00_JPRB ,&
&  0.35848E+00_JPRB, 0.42501E+00_JPRB, 0.58191E+00_JPRB, 0.60679E+00_JPRB, 0.60089E+00_JPRB, 0.59117E+00_JPRB, 0.46418E+00_JPRB ,&
&  0.57333E+00_JPRB, 0.54112E+00_JPRB, 0.53938E+00_JPRB, 0.53059E+00_JPRB, 0.52135E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.50109E+00_JPRB, 0.45026E+00_JPRB, 0.40163E+00_JPRB, 0.39417E+00_JPRB, 0.41577E+00_JPRB, 0.43007E+00_JPRB, 0.44095E+00_JPRB ,&
&  0.45933E+00_JPRB, 0.46872E+00_JPRB, 0.48294E+00_JPRB, 0.49958E+00_JPRB, 0.50643E+00_JPRB, 0.53526E+00_JPRB, 0.48003E+00_JPRB ,&
&  0.39850E+00_JPRB, 0.39595E+00_JPRB, 0.38687E+00_JPRB, 0.41089E+00_JPRB, 0.48181E+00_JPRB, 0.49828E+00_JPRB, 0.50817E+00_JPRB ,&
&  0.52599E+00_JPRB, 0.51342E+00_JPRB, 0.52341E+00_JPRB, 0.52244E+00_JPRB, 0.52024E+00_JPRB, 0.53425E+00_JPRB, 0.55189E+00_JPRB ,&
&  0.50266E+00_JPRB, 0.46554E+00_JPRB, 0.45055E+00_JPRB, 0.44599E+00_JPRB, 0.50825E+00_JPRB, 0.58761E+00_JPRB, 0.63524E+00_JPRB ,&
&  0.67471E+00_JPRB, 0.71872E+00_JPRB, 0.76775E+00_JPRB, 0.81944E+00_JPRB, 0.86734E+00_JPRB, 0.90031E+00_JPRB, 0.91329E+00_JPRB ,&
&  0.92554E+00_JPRB, 0.94884E+00_JPRB, 0.98671E+00_JPRB, 0.11207E+01_JPRB, 0.14033E+01_JPRB, 0.21192E+01_JPRB, 0.31208E+01_JPRB ,&
&  0.33148E+01_JPRB, 0.32525E+01_JPRB, 0.33658E+01_JPRB, 0.35609E+01_JPRB, 0.35014E+01_JPRB, 0.29826E+01_JPRB, 0.25079E+01_JPRB ,&
&  0.23153E+01_JPRB, 0.22364E+01_JPRB, 0.22608E+01_JPRB, 0.23742E+01_JPRB, 0.23915E+01_JPRB, 0.24328E+01_JPRB, 0.24264E+01_JPRB ,&
&  0.24485E+01_JPRB, 0.24613E+01_JPRB, 0.24612E+01_JPRB, 0.23271E+01_JPRB, 0.22093E+01_JPRB, 0.20835E+01_JPRB, 0.19079E+01_JPRB ,&
&  0.16946E+01_JPRB, 0.14497E+01_JPRB, 0.11701E+01_JPRB, 0.94042E+00_JPRB, 0.75167E+00_JPRB, 0.69619E+00_JPRB, 0.62035E+00_JPRB ,&
&  0.48754E+00_JPRB, 0.43282E+00_JPRB, 0.41938E+00_JPRB, 0.43061E+00_JPRB, 0.41265E+00_JPRB, 0.39413E+00_JPRB, 0.40168E+00_JPRB ,&
&  0.44537E+00_JPRB, 0.57367E+00_JPRB, 0.58664E+00_JPRB, 0.58751E+00_JPRB, 0.58812E+00_JPRB, 0.59449E+00_JPRB, 0.41793E+00_JPRB ,&
&  0.46318E+00_JPRB, 0.57202E+00_JPRB, 0.55419E+00_JPRB, 0.54304E+00_JPRB, 0.53047E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.50063E+00_JPRB, 0.46386E+00_JPRB, 0.42613E+00_JPRB, 0.41400E+00_JPRB, 0.42738E+00_JPRB, 0.44362E+00_JPRB, 0.43495E+00_JPRB ,&
&  0.47331E+00_JPRB, 0.46068E+00_JPRB, 0.48673E+00_JPRB, 0.52539E+00_JPRB, 0.56600E+00_JPRB, 0.48915E+00_JPRB, 0.36712E+00_JPRB ,&
&  0.33477E+00_JPRB, 0.34120E+00_JPRB, 0.36082E+00_JPRB, 0.37134E+00_JPRB, 0.38298E+00_JPRB, 0.38446E+00_JPRB, 0.38861E+00_JPRB ,&
&  0.39838E+00_JPRB, 0.41495E+00_JPRB, 0.45268E+00_JPRB, 0.47464E+00_JPRB, 0.48141E+00_JPRB, 0.48078E+00_JPRB, 0.49605E+00_JPRB ,&
&  0.46115E+00_JPRB, 0.44964E+00_JPRB, 0.45342E+00_JPRB, 0.42900E+00_JPRB, 0.49315E+00_JPRB, 0.52348E+00_JPRB, 0.54815E+00_JPRB ,&
&  0.57584E+00_JPRB, 0.61835E+00_JPRB, 0.67137E+00_JPRB, 0.72232E+00_JPRB, 0.76974E+00_JPRB, 0.81098E+00_JPRB, 0.84494E+00_JPRB ,&
&  0.87426E+00_JPRB, 0.90802E+00_JPRB, 0.94807E+00_JPRB, 0.10230E+01_JPRB, 0.12782E+01_JPRB, 0.17139E+01_JPRB, 0.23117E+01_JPRB ,&
&  0.24571E+01_JPRB, 0.26605E+01_JPRB, 0.28490E+01_JPRB, 0.25953E+01_JPRB, 0.25686E+01_JPRB, 0.24442E+01_JPRB, 0.21585E+01_JPRB ,&
&  0.20020E+01_JPRB, 0.18958E+01_JPRB, 0.19080E+01_JPRB, 0.19874E+01_JPRB, 0.21132E+01_JPRB, 0.21974E+01_JPRB, 0.22355E+01_JPRB ,&
&  0.22920E+01_JPRB, 0.23841E+01_JPRB, 0.24531E+01_JPRB, 0.23416E+01_JPRB, 0.22259E+01_JPRB, 0.20916E+01_JPRB, 0.19094E+01_JPRB ,&
&  0.17149E+01_JPRB, 0.15200E+01_JPRB, 0.12940E+01_JPRB, 0.10560E+01_JPRB, 0.85906E+00_JPRB, 0.74854E+00_JPRB, 0.63448E+00_JPRB ,&
&  0.51840E+00_JPRB, 0.44028E+00_JPRB, 0.44166E+00_JPRB, 0.44245E+00_JPRB, 0.42299E+00_JPRB, 0.39818E+00_JPRB, 0.38204E+00_JPRB ,&
&  0.41299E+00_JPRB, 0.47485E+00_JPRB, 0.63541E+00_JPRB, 0.60274E+00_JPRB, 0.59848E+00_JPRB, 0.59068E+00_JPRB, 0.48451E+00_JPRB ,&
&  0.45724E+00_JPRB, 0.59060E+00_JPRB, 0.55134E+00_JPRB, 0.54615E+00_JPRB, 0.53151E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.52211E+00_JPRB, 0.49320E+00_JPRB, 0.46650E+00_JPRB, 0.44535E+00_JPRB, 0.43898E+00_JPRB, 0.43039E+00_JPRB, 0.42540E+00_JPRB ,&
&  0.47510E+00_JPRB, 0.39707E+00_JPRB, 0.47497E+00_JPRB, 0.37407E+00_JPRB, 0.34171E+00_JPRB, 0.33874E+00_JPRB, 0.32645E+00_JPRB ,&
&  0.32133E+00_JPRB, 0.32956E+00_JPRB, 0.34284E+00_JPRB, 0.35415E+00_JPRB, 0.35922E+00_JPRB, 0.36309E+00_JPRB, 0.37545E+00_JPRB ,&
&  0.39367E+00_JPRB, 0.41861E+00_JPRB, 0.45245E+00_JPRB, 0.46824E+00_JPRB, 0.47021E+00_JPRB, 0.46853E+00_JPRB, 0.47737E+00_JPRB ,&
&  0.44599E+00_JPRB, 0.44091E+00_JPRB, 0.45402E+00_JPRB, 0.46247E+00_JPRB, 0.46283E+00_JPRB, 0.45811E+00_JPRB, 0.47875E+00_JPRB ,&
&  0.48124E+00_JPRB, 0.45281E+00_JPRB, 0.58761E+00_JPRB, 0.62998E+00_JPRB, 0.68215E+00_JPRB, 0.73152E+00_JPRB, 0.77898E+00_JPRB ,&
&  0.82155E+00_JPRB, 0.86625E+00_JPRB, 0.91620E+00_JPRB, 0.96466E+00_JPRB, 0.10767E+01_JPRB, 0.13257E+01_JPRB, 0.16119E+01_JPRB ,&
&  0.19827E+01_JPRB, 0.15854E+01_JPRB, 0.18464E+01_JPRB, 0.18008E+01_JPRB, 0.20190E+01_JPRB, 0.21190E+01_JPRB, 0.19984E+01_JPRB ,&
&  0.18049E+01_JPRB, 0.16999E+01_JPRB, 0.16125E+01_JPRB, 0.15562E+01_JPRB, 0.16441E+01_JPRB, 0.17611E+01_JPRB, 0.18862E+01_JPRB ,&
&  0.19718E+01_JPRB, 0.20211E+01_JPRB, 0.22287E+01_JPRB, 0.21297E+01_JPRB, 0.20542E+01_JPRB, 0.19705E+01_JPRB, 0.18468E+01_JPRB ,&
&  0.16825E+01_JPRB, 0.14935E+01_JPRB, 0.12763E+01_JPRB, 0.10463E+01_JPRB, 0.89025E+00_JPRB, 0.75064E+00_JPRB, 0.63538E+00_JPRB ,&
&  0.55397E+00_JPRB, 0.50394E+00_JPRB, 0.50456E+00_JPRB, 0.49841E+00_JPRB, 0.48180E+00_JPRB, 0.45877E+00_JPRB, 0.41824E+00_JPRB ,&
&  0.38929E+00_JPRB, 0.38984E+00_JPRB, 0.42929E+00_JPRB, 0.46538E+00_JPRB, 0.48068E+00_JPRB, 0.49604E+00_JPRB, 0.54780E+00_JPRB ,&
&  0.49730E+00_JPRB, 0.54128E+00_JPRB, 0.58299E+00_JPRB, 0.56249E+00_JPRB, 0.54716E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.54882E+00_JPRB, 0.52872E+00_JPRB, 0.50880E+00_JPRB, 0.48575E+00_JPRB, 0.47477E+00_JPRB, 0.45897E+00_JPRB, 0.41510E+00_JPRB ,&
&  0.36765E+00_JPRB, 0.34491E+00_JPRB, 0.34778E+00_JPRB, 0.33569E+00_JPRB, 0.32281E+00_JPRB, 0.31865E+00_JPRB, 0.32025E+00_JPRB ,&
&  0.33009E+00_JPRB, 0.35743E+00_JPRB, 0.38202E+00_JPRB, 0.39405E+00_JPRB, 0.39497E+00_JPRB, 0.39936E+00_JPRB, 0.40960E+00_JPRB ,&
&  0.41966E+00_JPRB, 0.43080E+00_JPRB, 0.43978E+00_JPRB, 0.43588E+00_JPRB, 0.43176E+00_JPRB, 0.43334E+00_JPRB, 0.43860E+00_JPRB ,&
&  0.44863E+00_JPRB, 0.45476E+00_JPRB, 0.44422E+00_JPRB, 0.45296E+00_JPRB, 0.43599E+00_JPRB, 0.43781E+00_JPRB, 0.42276E+00_JPRB ,&
&  0.34598E+00_JPRB, 0.36050E+00_JPRB, 0.50981E+00_JPRB, 0.56419E+00_JPRB, 0.60149E+00_JPRB, 0.65637E+00_JPRB, 0.72180E+00_JPRB ,&
&  0.77610E+00_JPRB, 0.82326E+00_JPRB, 0.86888E+00_JPRB, 0.92543E+00_JPRB, 0.99436E+00_JPRB, 0.11185E+01_JPRB, 0.12670E+01_JPRB ,&
&  0.15413E+01_JPRB, 0.13092E+01_JPRB, 0.11837E+01_JPRB, 0.11851E+01_JPRB, 0.14711E+01_JPRB, 0.15290E+01_JPRB, 0.15182E+01_JPRB ,&
&  0.14614E+01_JPRB, 0.13307E+01_JPRB, 0.12532E+01_JPRB, 0.11889E+01_JPRB, 0.12139E+01_JPRB, 0.12910E+01_JPRB, 0.14146E+01_JPRB ,&
&  0.15350E+01_JPRB, 0.17059E+01_JPRB, 0.18852E+01_JPRB, 0.18417E+01_JPRB, 0.18015E+01_JPRB, 0.17664E+01_JPRB, 0.17024E+01_JPRB ,&
&  0.16223E+01_JPRB, 0.14582E+01_JPRB, 0.12053E+01_JPRB, 0.10100E+01_JPRB, 0.85011E+00_JPRB, 0.72619E+00_JPRB, 0.62590E+00_JPRB ,&
&  0.57749E+00_JPRB, 0.56625E+00_JPRB, 0.56577E+00_JPRB, 0.55942E+00_JPRB, 0.53191E+00_JPRB, 0.49934E+00_JPRB, 0.46202E+00_JPRB ,&
&  0.41917E+00_JPRB, 0.38128E+00_JPRB, 0.35827E+00_JPRB, 0.37164E+00_JPRB, 0.40912E+00_JPRB, 0.44425E+00_JPRB, 0.46002E+00_JPRB ,&
&  0.46603E+00_JPRB, 0.47381E+00_JPRB, 0.47849E+00_JPRB, 0.49117E+00_JPRB, 0.51586E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.56708E+00_JPRB, 0.55194E+00_JPRB, 0.52622E+00_JPRB, 0.52286E+00_JPRB, 0.50006E+00_JPRB, 0.48282E+00_JPRB, 0.43671E+00_JPRB ,&
&  0.37859E+00_JPRB, 0.36345E+00_JPRB, 0.38325E+00_JPRB, 0.38791E+00_JPRB, 0.37305E+00_JPRB, 0.37193E+00_JPRB, 0.38081E+00_JPRB ,&
&  0.39267E+00_JPRB, 0.42147E+00_JPRB, 0.42520E+00_JPRB, 0.41195E+00_JPRB, 0.40486E+00_JPRB, 0.40271E+00_JPRB, 0.40812E+00_JPRB ,&
&  0.41034E+00_JPRB, 0.41089E+00_JPRB, 0.40666E+00_JPRB, 0.39570E+00_JPRB, 0.39760E+00_JPRB, 0.40373E+00_JPRB, 0.40811E+00_JPRB ,&
&  0.41114E+00_JPRB, 0.40815E+00_JPRB, 0.42066E+00_JPRB, 0.43679E+00_JPRB, 0.43117E+00_JPRB, 0.41767E+00_JPRB, 0.35257E+00_JPRB ,&
&  0.31208E+00_JPRB, 0.30340E+00_JPRB, 0.33488E+00_JPRB, 0.44145E+00_JPRB, 0.49685E+00_JPRB, 0.55416E+00_JPRB, 0.61236E+00_JPRB ,&
&  0.58536E+00_JPRB, 0.59056E+00_JPRB, 0.64013E+00_JPRB, 0.79607E+00_JPRB, 0.87481E+00_JPRB, 0.96444E+00_JPRB, 0.10389E+01_JPRB ,&
&  0.11239E+01_JPRB, 0.11576E+01_JPRB, 0.11353E+01_JPRB, 0.90293E+00_JPRB, 0.92485E+00_JPRB, 0.11582E+01_JPRB, 0.11223E+01_JPRB ,&
&  0.11730E+01_JPRB, 0.11797E+01_JPRB, 0.12174E+01_JPRB, 0.10995E+01_JPRB, 0.10340E+01_JPRB, 0.10198E+01_JPRB, 0.10692E+01_JPRB ,&
&  0.11700E+01_JPRB, 0.13123E+01_JPRB, 0.15239E+01_JPRB, 0.15764E+01_JPRB, 0.15519E+01_JPRB, 0.15212E+01_JPRB, 0.14814E+01_JPRB ,&
&  0.14448E+01_JPRB, 0.12816E+01_JPRB, 0.10796E+01_JPRB, 0.91090E+00_JPRB, 0.77464E+00_JPRB, 0.67028E+00_JPRB, 0.60912E+00_JPRB ,&
&  0.58746E+00_JPRB, 0.58163E+00_JPRB, 0.58968E+00_JPRB, 0.58089E+00_JPRB, 0.54352E+00_JPRB, 0.49855E+00_JPRB, 0.46444E+00_JPRB ,&
&  0.44045E+00_JPRB, 0.41166E+00_JPRB, 0.38816E+00_JPRB, 0.38315E+00_JPRB, 0.40960E+00_JPRB, 0.44470E+00_JPRB, 0.45543E+00_JPRB ,&
&  0.44320E+00_JPRB, 0.44657E+00_JPRB, 0.47795E+00_JPRB, 0.51145E+00_JPRB, 0.52812E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.52612E+00_JPRB, 0.54025E+00_JPRB, 0.55458E+00_JPRB, 0.55129E+00_JPRB, 0.54944E+00_JPRB, 0.49617E+00_JPRB, 0.44065E+00_JPRB ,&
&  0.40323E+00_JPRB, 0.40888E+00_JPRB, 0.42795E+00_JPRB, 0.44497E+00_JPRB, 0.44789E+00_JPRB, 0.44396E+00_JPRB, 0.44732E+00_JPRB ,&
&  0.45669E+00_JPRB, 0.45687E+00_JPRB, 0.43961E+00_JPRB, 0.42614E+00_JPRB, 0.40673E+00_JPRB, 0.40368E+00_JPRB, 0.40654E+00_JPRB ,&
&  0.40928E+00_JPRB, 0.40762E+00_JPRB, 0.40608E+00_JPRB, 0.40295E+00_JPRB, 0.40461E+00_JPRB, 0.40476E+00_JPRB, 0.40265E+00_JPRB ,&
&  0.39487E+00_JPRB, 0.38783E+00_JPRB, 0.39419E+00_JPRB, 0.41431E+00_JPRB, 0.40981E+00_JPRB, 0.40699E+00_JPRB, 0.33499E+00_JPRB ,&
&  0.30698E+00_JPRB, 0.29019E+00_JPRB, 0.28889E+00_JPRB, 0.30269E+00_JPRB, 0.34666E+00_JPRB, 0.40141E+00_JPRB, 0.47720E+00_JPRB ,&
&  0.54521E+00_JPRB, 0.60025E+00_JPRB, 0.63248E+00_JPRB, 0.65946E+00_JPRB, 0.70055E+00_JPRB, 0.76533E+00_JPRB, 0.83830E+00_JPRB ,&
&  0.87735E+00_JPRB, 0.89696E+00_JPRB, 0.88574E+00_JPRB, 0.75618E+00_JPRB, 0.73741E+00_JPRB, 0.81122E+00_JPRB, 0.91069E+00_JPRB ,&
&  0.10004E+01_JPRB, 0.92693E+00_JPRB, 0.99687E+00_JPRB, 0.10486E+01_JPRB, 0.10539E+01_JPRB, 0.10462E+01_JPRB, 0.10375E+01_JPRB ,&
&  0.11224E+01_JPRB, 0.12079E+01_JPRB, 0.13482E+01_JPRB, 0.13944E+01_JPRB, 0.13282E+01_JPRB, 0.13356E+01_JPRB, 0.12771E+01_JPRB ,&
&  0.12065E+01_JPRB, 0.10354E+01_JPRB, 0.88686E+00_JPRB, 0.78659E+00_JPRB, 0.69654E+00_JPRB, 0.64359E+00_JPRB, 0.61716E+00_JPRB ,&
&  0.59570E+00_JPRB, 0.57807E+00_JPRB, 0.58406E+00_JPRB, 0.57528E+00_JPRB, 0.54879E+00_JPRB, 0.52164E+00_JPRB, 0.49877E+00_JPRB ,&
&  0.48007E+00_JPRB, 0.46790E+00_JPRB, 0.46442E+00_JPRB, 0.45424E+00_JPRB, 0.45589E+00_JPRB, 0.46591E+00_JPRB, 0.47116E+00_JPRB ,&
&  0.47343E+00_JPRB, 0.47587E+00_JPRB, 0.48549E+00_JPRB, 0.49635E+00_JPRB, 0.50640E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.55212E+00_JPRB, 0.56475E+00_JPRB, 0.57092E+00_JPRB, 0.57898E+00_JPRB, 0.59722E+00_JPRB, 0.56169E+00_JPRB, 0.53814E+00_JPRB ,&
&  0.52751E+00_JPRB, 0.53954E+00_JPRB, 0.53982E+00_JPRB, 0.53672E+00_JPRB, 0.52691E+00_JPRB, 0.51843E+00_JPRB, 0.51248E+00_JPRB ,&
&  0.50695E+00_JPRB, 0.49197E+00_JPRB, 0.46540E+00_JPRB, 0.42255E+00_JPRB, 0.41013E+00_JPRB, 0.40747E+00_JPRB, 0.41130E+00_JPRB ,&
&  0.41392E+00_JPRB, 0.40956E+00_JPRB, 0.40932E+00_JPRB, 0.40600E+00_JPRB, 0.40199E+00_JPRB, 0.38837E+00_JPRB, 0.37600E+00_JPRB ,&
&  0.37911E+00_JPRB, 0.38159E+00_JPRB, 0.40249E+00_JPRB, 0.39042E+00_JPRB, 0.38096E+00_JPRB, 0.37896E+00_JPRB, 0.31594E+00_JPRB ,&
&  0.30409E+00_JPRB, 0.29012E+00_JPRB, 0.28580E+00_JPRB, 0.28608E+00_JPRB, 0.30228E+00_JPRB, 0.32010E+00_JPRB, 0.35679E+00_JPRB ,&
&  0.41133E+00_JPRB, 0.49994E+00_JPRB, 0.54373E+00_JPRB, 0.59262E+00_JPRB, 0.63524E+00_JPRB, 0.67665E+00_JPRB, 0.71760E+00_JPRB ,&
&  0.74921E+00_JPRB, 0.78307E+00_JPRB, 0.81168E+00_JPRB, 0.86847E+00_JPRB, 0.91975E+00_JPRB, 0.87687E+00_JPRB, 0.90703E+00_JPRB ,&
&  0.11102E+01_JPRB, 0.10340E+01_JPRB, 0.10069E+01_JPRB, 0.97059E+00_JPRB, 0.97710E+00_JPRB, 0.98544E+00_JPRB, 0.99490E+00_JPRB ,&
&  0.10524E+01_JPRB, 0.11393E+01_JPRB, 0.12675E+01_JPRB, 0.13355E+01_JPRB, 0.12758E+01_JPRB, 0.12710E+01_JPRB, 0.12250E+01_JPRB ,&
&  0.10963E+01_JPRB, 0.98540E+00_JPRB, 0.93208E+00_JPRB, 0.80823E+00_JPRB, 0.71541E+00_JPRB, 0.66770E+00_JPRB, 0.65704E+00_JPRB ,&
&  0.64624E+00_JPRB, 0.62918E+00_JPRB, 0.61975E+00_JPRB, 0.60014E+00_JPRB, 0.57129E+00_JPRB, 0.55383E+00_JPRB, 0.55284E+00_JPRB ,&
&  0.55198E+00_JPRB, 0.54808E+00_JPRB, 0.54570E+00_JPRB, 0.54019E+00_JPRB, 0.52977E+00_JPRB, 0.51814E+00_JPRB, 0.51668E+00_JPRB ,&
&  0.52155E+00_JPRB, 0.52622E+00_JPRB, 0.53019E+00_JPRB, 0.53305E+00_JPRB, 0.53914E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.56715E+00_JPRB, 0.58259E+00_JPRB, 0.58760E+00_JPRB, 0.58996E+00_JPRB, 0.59351E+00_JPRB, 0.58044E+00_JPRB, 0.56950E+00_JPRB ,&
&  0.56210E+00_JPRB, 0.55319E+00_JPRB, 0.54245E+00_JPRB, 0.53188E+00_JPRB, 0.52119E+00_JPRB, 0.51449E+00_JPRB, 0.50798E+00_JPRB ,&
&  0.50166E+00_JPRB, 0.46569E+00_JPRB, 0.44687E+00_JPRB, 0.43401E+00_JPRB, 0.42296E+00_JPRB, 0.41836E+00_JPRB, 0.41655E+00_JPRB ,&
&  0.41161E+00_JPRB, 0.40858E+00_JPRB, 0.40882E+00_JPRB, 0.40675E+00_JPRB, 0.40132E+00_JPRB, 0.39168E+00_JPRB, 0.39715E+00_JPRB ,&
&  0.41509E+00_JPRB, 0.40734E+00_JPRB, 0.39383E+00_JPRB, 0.38089E+00_JPRB, 0.36207E+00_JPRB, 0.33691E+00_JPRB, 0.31719E+00_JPRB ,&
&  0.30915E+00_JPRB, 0.29164E+00_JPRB, 0.29291E+00_JPRB, 0.29623E+00_JPRB, 0.30000E+00_JPRB, 0.31469E+00_JPRB, 0.34675E+00_JPRB ,&
&  0.38539E+00_JPRB, 0.47960E+00_JPRB, 0.51823E+00_JPRB, 0.56489E+00_JPRB, 0.60091E+00_JPRB, 0.63088E+00_JPRB, 0.65770E+00_JPRB ,&
&  0.67467E+00_JPRB, 0.70041E+00_JPRB, 0.72952E+00_JPRB, 0.77302E+00_JPRB, 0.81825E+00_JPRB, 0.84472E+00_JPRB, 0.86411E+00_JPRB ,&
&  0.89779E+00_JPRB, 0.87504E+00_JPRB, 0.88044E+00_JPRB, 0.86542E+00_JPRB, 0.86828E+00_JPRB, 0.90526E+00_JPRB, 0.89900E+00_JPRB ,&
&  0.91327E+00_JPRB, 0.94958E+00_JPRB, 0.10313E+01_JPRB, 0.11045E+01_JPRB, 0.11061E+01_JPRB, 0.11284E+01_JPRB, 0.11220E+01_JPRB ,&
&  0.10551E+01_JPRB, 0.10149E+01_JPRB, 0.99165E+00_JPRB, 0.91488E+00_JPRB, 0.83408E+00_JPRB, 0.77733E+00_JPRB, 0.74050E+00_JPRB ,&
&  0.70438E+00_JPRB, 0.66378E+00_JPRB, 0.63635E+00_JPRB, 0.61422E+00_JPRB, 0.58996E+00_JPRB, 0.56800E+00_JPRB, 0.56606E+00_JPRB ,&
&  0.56591E+00_JPRB, 0.56464E+00_JPRB, 0.56124E+00_JPRB, 0.55671E+00_JPRB, 0.54317E+00_JPRB, 0.52847E+00_JPRB, 0.51581E+00_JPRB ,&
&  0.51643E+00_JPRB, 0.52070E+00_JPRB, 0.53165E+00_JPRB, 0.54068E+00_JPRB, 0.55242E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.56883E+00_JPRB, 0.57154E+00_JPRB, 0.56770E+00_JPRB, 0.56181E+00_JPRB, 0.55738E+00_JPRB, 0.54782E+00_JPRB, 0.54239E+00_JPRB ,&
&  0.53834E+00_JPRB, 0.53438E+00_JPRB, 0.52521E+00_JPRB, 0.51524E+00_JPRB, 0.50187E+00_JPRB, 0.49235E+00_JPRB, 0.48139E+00_JPRB ,&
&  0.47273E+00_JPRB, 0.46206E+00_JPRB, 0.45091E+00_JPRB, 0.44021E+00_JPRB, 0.43237E+00_JPRB, 0.42432E+00_JPRB, 0.41926E+00_JPRB ,&
&  0.41431E+00_JPRB, 0.40565E+00_JPRB, 0.39781E+00_JPRB, 0.39849E+00_JPRB, 0.39526E+00_JPRB, 0.39491E+00_JPRB, 0.40164E+00_JPRB ,&
&  0.40861E+00_JPRB, 0.40622E+00_JPRB, 0.40294E+00_JPRB, 0.38563E+00_JPRB, 0.35962E+00_JPRB, 0.29988E+00_JPRB, 0.30605E+00_JPRB ,&
&  0.30167E+00_JPRB, 0.29420E+00_JPRB, 0.29741E+00_JPRB, 0.30550E+00_JPRB, 0.31412E+00_JPRB, 0.33727E+00_JPRB, 0.34532E+00_JPRB ,&
&  0.34645E+00_JPRB, 0.46404E+00_JPRB, 0.49471E+00_JPRB, 0.52623E+00_JPRB, 0.55279E+00_JPRB, 0.57660E+00_JPRB, 0.59989E+00_JPRB ,&
&  0.61511E+00_JPRB, 0.63774E+00_JPRB, 0.65796E+00_JPRB, 0.69282E+00_JPRB, 0.72736E+00_JPRB, 0.76815E+00_JPRB, 0.80048E+00_JPRB ,&
&  0.81063E+00_JPRB, 0.81165E+00_JPRB, 0.82136E+00_JPRB, 0.82402E+00_JPRB, 0.83212E+00_JPRB, 0.85282E+00_JPRB, 0.88684E+00_JPRB ,&
&  0.93353E+00_JPRB, 0.94370E+00_JPRB, 0.99926E+00_JPRB, 0.10360E+01_JPRB, 0.10413E+01_JPRB, 0.10498E+01_JPRB, 0.10555E+01_JPRB ,&
&  0.10223E+01_JPRB, 0.10051E+01_JPRB, 0.96302E+00_JPRB, 0.88627E+00_JPRB, 0.80980E+00_JPRB, 0.76285E+00_JPRB, 0.71983E+00_JPRB ,&
&  0.68785E+00_JPRB, 0.65448E+00_JPRB, 0.61962E+00_JPRB, 0.59231E+00_JPRB, 0.57051E+00_JPRB, 0.55608E+00_JPRB, 0.55051E+00_JPRB ,&
&  0.54573E+00_JPRB, 0.54189E+00_JPRB, 0.53868E+00_JPRB, 0.54040E+00_JPRB, 0.53660E+00_JPRB, 0.53438E+00_JPRB, 0.53162E+00_JPRB ,&
&  0.53458E+00_JPRB, 0.53852E+00_JPRB, 0.54680E+00_JPRB, 0.55460E+00_JPRB, 0.55975E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.56079E+00_JPRB, 0.56111E+00_JPRB, 0.56084E+00_JPRB, 0.55301E+00_JPRB, 0.54892E+00_JPRB, 0.54359E+00_JPRB, 0.54414E+00_JPRB ,&
&  0.53719E+00_JPRB, 0.53109E+00_JPRB, 0.51526E+00_JPRB, 0.50509E+00_JPRB, 0.49297E+00_JPRB, 0.48610E+00_JPRB, 0.47953E+00_JPRB ,&
&  0.47557E+00_JPRB, 0.46990E+00_JPRB, 0.46712E+00_JPRB, 0.45970E+00_JPRB, 0.45374E+00_JPRB, 0.44341E+00_JPRB, 0.43193E+00_JPRB ,&
&  0.41961E+00_JPRB, 0.40687E+00_JPRB, 0.39432E+00_JPRB, 0.39527E+00_JPRB, 0.39064E+00_JPRB, 0.38894E+00_JPRB, 0.38929E+00_JPRB ,&
&  0.39820E+00_JPRB, 0.35714E+00_JPRB, 0.35926E+00_JPRB, 0.34473E+00_JPRB, 0.32977E+00_JPRB, 0.31412E+00_JPRB, 0.30342E+00_JPRB ,&
&  0.30593E+00_JPRB, 0.30787E+00_JPRB, 0.31710E+00_JPRB, 0.33271E+00_JPRB, 0.34103E+00_JPRB, 0.35668E+00_JPRB, 0.36997E+00_JPRB ,&
&  0.37529E+00_JPRB, 0.45837E+00_JPRB, 0.50157E+00_JPRB, 0.52678E+00_JPRB, 0.54925E+00_JPRB, 0.56956E+00_JPRB, 0.58568E+00_JPRB ,&
&  0.60663E+00_JPRB, 0.62854E+00_JPRB, 0.64789E+00_JPRB, 0.66561E+00_JPRB, 0.69939E+00_JPRB, 0.72301E+00_JPRB, 0.74186E+00_JPRB ,&
&  0.75220E+00_JPRB, 0.74985E+00_JPRB, 0.75544E+00_JPRB, 0.77049E+00_JPRB, 0.79763E+00_JPRB, 0.82612E+00_JPRB, 0.85925E+00_JPRB ,&
&  0.89148E+00_JPRB, 0.91577E+00_JPRB, 0.94137E+00_JPRB, 0.96639E+00_JPRB, 0.98334E+00_JPRB, 0.98318E+00_JPRB, 0.97594E+00_JPRB ,&
&  0.94551E+00_JPRB, 0.91568E+00_JPRB, 0.87588E+00_JPRB, 0.83087E+00_JPRB, 0.78201E+00_JPRB, 0.72993E+00_JPRB, 0.68209E+00_JPRB ,&
&  0.65601E+00_JPRB, 0.63055E+00_JPRB, 0.60758E+00_JPRB, 0.58402E+00_JPRB, 0.56457E+00_JPRB, 0.55194E+00_JPRB, 0.54723E+00_JPRB ,&
&  0.54170E+00_JPRB, 0.53882E+00_JPRB, 0.53756E+00_JPRB, 0.53775E+00_JPRB, 0.54032E+00_JPRB, 0.54153E+00_JPRB, 0.54626E+00_JPRB ,&
&  0.55143E+00_JPRB, 0.55644E+00_JPRB, 0.55735E+00_JPRB, 0.55775E+00_JPRB, 0.55877E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.55532E+00_JPRB, 0.55720E+00_JPRB, 0.56367E+00_JPRB, 0.56465E+00_JPRB, 0.56500E+00_JPRB, 0.56050E+00_JPRB, 0.55937E+00_JPRB ,&
&  0.55266E+00_JPRB, 0.54579E+00_JPRB, 0.52973E+00_JPRB, 0.51817E+00_JPRB, 0.50629E+00_JPRB, 0.49945E+00_JPRB, 0.49414E+00_JPRB ,&
&  0.49088E+00_JPRB, 0.48592E+00_JPRB, 0.48065E+00_JPRB, 0.47341E+00_JPRB, 0.46465E+00_JPRB, 0.45365E+00_JPRB, 0.43817E+00_JPRB ,&
&  0.42420E+00_JPRB, 0.41357E+00_JPRB, 0.39786E+00_JPRB, 0.40014E+00_JPRB, 0.39380E+00_JPRB, 0.38963E+00_JPRB, 0.38824E+00_JPRB ,&
&  0.38258E+00_JPRB, 0.37401E+00_JPRB, 0.36196E+00_JPRB, 0.35219E+00_JPRB, 0.34304E+00_JPRB, 0.33345E+00_JPRB, 0.34114E+00_JPRB ,&
&  0.34772E+00_JPRB, 0.36854E+00_JPRB, 0.37912E+00_JPRB, 0.38436E+00_JPRB, 0.39014E+00_JPRB, 0.39342E+00_JPRB, 0.40421E+00_JPRB ,&
&  0.41954E+00_JPRB, 0.44087E+00_JPRB, 0.49520E+00_JPRB, 0.52723E+00_JPRB, 0.54459E+00_JPRB, 0.56316E+00_JPRB, 0.57588E+00_JPRB ,&
&  0.59241E+00_JPRB, 0.60152E+00_JPRB, 0.60608E+00_JPRB, 0.61380E+00_JPRB, 0.62975E+00_JPRB, 0.65037E+00_JPRB, 0.66641E+00_JPRB ,&
&  0.67820E+00_JPRB, 0.68239E+00_JPRB, 0.69114E+00_JPRB, 0.70227E+00_JPRB, 0.72290E+00_JPRB, 0.74412E+00_JPRB, 0.76682E+00_JPRB ,&
&  0.78729E+00_JPRB, 0.80806E+00_JPRB, 0.82002E+00_JPRB, 0.82671E+00_JPRB, 0.82607E+00_JPRB, 0.82101E+00_JPRB, 0.81535E+00_JPRB ,&
&  0.79963E+00_JPRB, 0.77794E+00_JPRB, 0.75168E+00_JPRB, 0.71481E+00_JPRB, 0.68846E+00_JPRB, 0.67221E+00_JPRB, 0.64955E+00_JPRB ,&
&  0.63057E+00_JPRB, 0.61319E+00_JPRB, 0.59701E+00_JPRB, 0.58138E+00_JPRB, 0.56865E+00_JPRB, 0.55715E+00_JPRB, 0.54974E+00_JPRB ,&
&  0.54432E+00_JPRB, 0.54154E+00_JPRB, 0.54175E+00_JPRB, 0.54360E+00_JPRB, 0.54909E+00_JPRB, 0.55321E+00_JPRB, 0.56028E+00_JPRB ,&
&  0.56321E+00_JPRB, 0.56327E+00_JPRB, 0.55939E+00_JPRB, 0.55634E+00_JPRB, 0.55404E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.55948E+00_JPRB, 0.55922E+00_JPRB, 0.56109E+00_JPRB, 0.56228E+00_JPRB, 0.56401E+00_JPRB, 0.56283E+00_JPRB, 0.56182E+00_JPRB ,&
&  0.55810E+00_JPRB, 0.55119E+00_JPRB, 0.53904E+00_JPRB, 0.52802E+00_JPRB, 0.51821E+00_JPRB, 0.51178E+00_JPRB, 0.50624E+00_JPRB ,&
&  0.50104E+00_JPRB, 0.49670E+00_JPRB, 0.49204E+00_JPRB, 0.48710E+00_JPRB, 0.48073E+00_JPRB, 0.47341E+00_JPRB, 0.46391E+00_JPRB ,&
&  0.45665E+00_JPRB, 0.44949E+00_JPRB, 0.44341E+00_JPRB, 0.43915E+00_JPRB, 0.43751E+00_JPRB, 0.41750E+00_JPRB, 0.41303E+00_JPRB ,&
&  0.41783E+00_JPRB, 0.41013E+00_JPRB, 0.40384E+00_JPRB, 0.39373E+00_JPRB, 0.39677E+00_JPRB, 0.40517E+00_JPRB, 0.41086E+00_JPRB ,&
&  0.40065E+00_JPRB, 0.42135E+00_JPRB, 0.42375E+00_JPRB, 0.42531E+00_JPRB, 0.42483E+00_JPRB, 0.42330E+00_JPRB, 0.43603E+00_JPRB ,&
&  0.46466E+00_JPRB, 0.47878E+00_JPRB, 0.50285E+00_JPRB, 0.52886E+00_JPRB, 0.54579E+00_JPRB, 0.56347E+00_JPRB, 0.57933E+00_JPRB ,&
&  0.59497E+00_JPRB, 0.60426E+00_JPRB, 0.60951E+00_JPRB, 0.61477E+00_JPRB, 0.62486E+00_JPRB, 0.63549E+00_JPRB, 0.64411E+00_JPRB ,&
&  0.64920E+00_JPRB, 0.65032E+00_JPRB, 0.65309E+00_JPRB, 0.65615E+00_JPRB, 0.66270E+00_JPRB, 0.66786E+00_JPRB, 0.67218E+00_JPRB ,&
&  0.67386E+00_JPRB, 0.67516E+00_JPRB, 0.67371E+00_JPRB, 0.67033E+00_JPRB, 0.66561E+00_JPRB, 0.65958E+00_JPRB, 0.65726E+00_JPRB ,&
&  0.65018E+00_JPRB, 0.64166E+00_JPRB, 0.63119E+00_JPRB, 0.61826E+00_JPRB, 0.60780E+00_JPRB, 0.59987E+00_JPRB, 0.59279E+00_JPRB ,&
&  0.58580E+00_JPRB, 0.57956E+00_JPRB, 0.57411E+00_JPRB, 0.56880E+00_JPRB, 0.56354E+00_JPRB, 0.55841E+00_JPRB, 0.55510E+00_JPRB ,&
&  0.55455E+00_JPRB, 0.55452E+00_JPRB, 0.55674E+00_JPRB, 0.55802E+00_JPRB, 0.56182E+00_JPRB, 0.56341E+00_JPRB, 0.56633E+00_JPRB ,&
&  0.56761E+00_JPRB, 0.56669E+00_JPRB, 0.56372E+00_JPRB, 0.56124E+00_JPRB, 0.55907E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.55619E+00_JPRB, 0.55451E+00_JPRB, 0.55346E+00_JPRB, 0.55379E+00_JPRB, 0.55481E+00_JPRB, 0.55517E+00_JPRB, 0.55468E+00_JPRB ,&
&  0.55274E+00_JPRB, 0.54791E+00_JPRB, 0.54223E+00_JPRB, 0.53621E+00_JPRB, 0.52942E+00_JPRB, 0.52317E+00_JPRB, 0.51925E+00_JPRB ,&
&  0.51499E+00_JPRB, 0.51072E+00_JPRB, 0.50634E+00_JPRB, 0.50323E+00_JPRB, 0.49976E+00_JPRB, 0.49643E+00_JPRB, 0.49343E+00_JPRB ,&
&  0.49149E+00_JPRB, 0.48910E+00_JPRB, 0.48636E+00_JPRB, 0.48297E+00_JPRB, 0.48061E+00_JPRB, 0.47781E+00_JPRB, 0.47512E+00_JPRB ,&
&  0.47193E+00_JPRB, 0.47082E+00_JPRB, 0.47019E+00_JPRB, 0.47029E+00_JPRB, 0.47244E+00_JPRB, 0.47533E+00_JPRB, 0.47649E+00_JPRB ,&
&  0.47672E+00_JPRB, 0.47486E+00_JPRB, 0.47623E+00_JPRB, 0.47822E+00_JPRB, 0.48172E+00_JPRB, 0.48733E+00_JPRB, 0.49500E+00_JPRB ,&
&  0.50492E+00_JPRB, 0.51640E+00_JPRB, 0.52817E+00_JPRB, 0.53679E+00_JPRB, 0.54512E+00_JPRB, 0.55268E+00_JPRB, 0.55731E+00_JPRB ,&
&  0.56222E+00_JPRB, 0.56560E+00_JPRB, 0.56996E+00_JPRB, 0.57339E+00_JPRB, 0.57824E+00_JPRB, 0.58230E+00_JPRB, 0.58594E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58543E+00_JPRB, 0.58365E+00_JPRB, 0.58228E+00_JPRB, 0.58003E+00_JPRB, 0.57822E+00_JPRB, 0.57683E+00_JPRB ,&
&  0.57582E+00_JPRB, 0.57497E+00_JPRB, 0.57506E+00_JPRB, 0.57543E+00_JPRB, 0.57594E+00_JPRB, 0.57571E+00_JPRB, 0.57648E+00_JPRB ,&
&  0.57551E+00_JPRB, 0.57412E+00_JPRB, 0.57180E+00_JPRB, 0.56812E+00_JPRB, 0.56504E+00_JPRB, 0.56286E+00_JPRB, 0.56076E+00_JPRB ,&
&  0.55866E+00_JPRB, 0.55711E+00_JPRB, 0.55612E+00_JPRB, 0.55573E+00_JPRB, 0.55463E+00_JPRB, 0.55377E+00_JPRB, 0.55351E+00_JPRB ,&
&  0.55394E+00_JPRB, 0.55388E+00_JPRB, 0.55506E+00_JPRB, 0.55630E+00_JPRB, 0.55987E+00_JPRB, 0.56151E+00_JPRB, 0.56283E+00_JPRB ,&
&  0.56315E+00_JPRB, 0.56400E+00_JPRB, 0.56305E+00_JPRB, 0.56069E+00_JPRB, 0.55802E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.55587E+00_JPRB, 0.55448E+00_JPRB, 0.55324E+00_JPRB, 0.55241E+00_JPRB, 0.55203E+00_JPRB, 0.55189E+00_JPRB, 0.55099E+00_JPRB ,&
&  0.54972E+00_JPRB, 0.54757E+00_JPRB, 0.54540E+00_JPRB, 0.54278E+00_JPRB, 0.53973E+00_JPRB, 0.53564E+00_JPRB, 0.53218E+00_JPRB ,&
&  0.52907E+00_JPRB, 0.52656E+00_JPRB, 0.52376E+00_JPRB, 0.52192E+00_JPRB, 0.52008E+00_JPRB, 0.51856E+00_JPRB, 0.51707E+00_JPRB ,&
&  0.51602E+00_JPRB, 0.51495E+00_JPRB, 0.51393E+00_JPRB, 0.51299E+00_JPRB, 0.51264E+00_JPRB, 0.51210E+00_JPRB, 0.51148E+00_JPRB ,&
&  0.51095E+00_JPRB, 0.51071E+00_JPRB, 0.51051E+00_JPRB, 0.51040E+00_JPRB, 0.51038E+00_JPRB, 0.51075E+00_JPRB, 0.51112E+00_JPRB ,&
&  0.51172E+00_JPRB, 0.51289E+00_JPRB, 0.51453E+00_JPRB, 0.51605E+00_JPRB, 0.51772E+00_JPRB, 0.51995E+00_JPRB, 0.52259E+00_JPRB ,&
&  0.52494E+00_JPRB, 0.52699E+00_JPRB, 0.52913E+00_JPRB, 0.53155E+00_JPRB, 0.53384E+00_JPRB, 0.53621E+00_JPRB, 0.53815E+00_JPRB ,&
&  0.53998E+00_JPRB, 0.54173E+00_JPRB, 0.54373E+00_JPRB, 0.54435E+00_JPRB, 0.54567E+00_JPRB, 0.54681E+00_JPRB, 0.54832E+00_JPRB ,&
&  0.54875E+00_JPRB, 0.54909E+00_JPRB, 0.54978E+00_JPRB, 0.55051E+00_JPRB, 0.55102E+00_JPRB, 0.55159E+00_JPRB, 0.55230E+00_JPRB ,&
&  0.55328E+00_JPRB, 0.55393E+00_JPRB, 0.55479E+00_JPRB, 0.55562E+00_JPRB, 0.55675E+00_JPRB, 0.55765E+00_JPRB, 0.55866E+00_JPRB ,&
&  0.55898E+00_JPRB, 0.55917E+00_JPRB, 0.55909E+00_JPRB, 0.55885E+00_JPRB, 0.55874E+00_JPRB, 0.55880E+00_JPRB, 0.55894E+00_JPRB ,&
&  0.55916E+00_JPRB, 0.55931E+00_JPRB, 0.55940E+00_JPRB, 0.55930E+00_JPRB, 0.55870E+00_JPRB, 0.55815E+00_JPRB, 0.55780E+00_JPRB ,&
&  0.55792E+00_JPRB, 0.55797E+00_JPRB, 0.55834E+00_JPRB, 0.55893E+00_JPRB, 0.56016E+00_JPRB, 0.56062E+00_JPRB, 0.56096E+00_JPRB ,&
&  0.56081E+00_JPRB, 0.56052E+00_JPRB, 0.55963E+00_JPRB, 0.55847E+00_JPRB, 0.55713E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB, 0.54164E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB, 0.12449E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.12626E+00_JPRB, 0.12690E+00_JPRB, 0.12737E+00_JPRB, 0.12776E+00_JPRB, 0.12767E+00_JPRB, 0.12776E+00_JPRB, 0.12786E+00_JPRB ,&
&  0.12790E+00_JPRB, 0.12792E+00_JPRB, 0.12814E+00_JPRB, 0.12836E+00_JPRB, 0.12858E+00_JPRB, 0.12882E+00_JPRB, 0.12906E+00_JPRB ,&
&  0.12917E+00_JPRB, 0.12940E+00_JPRB, 0.12854E+00_JPRB, 0.12786E+00_JPRB, 0.12746E+00_JPRB, 0.12725E+00_JPRB, 0.12715E+00_JPRB ,&
&  0.12706E+00_JPRB, 0.12702E+00_JPRB, 0.12694E+00_JPRB, 0.12759E+00_JPRB, 0.12801E+00_JPRB, 0.12841E+00_JPRB, 0.12872E+00_JPRB ,&
&  0.12886E+00_JPRB, 0.12904E+00_JPRB, 0.12919E+00_JPRB, 0.12935E+00_JPRB, 0.12952E+00_JPRB, 0.12948E+00_JPRB, 0.12936E+00_JPRB ,&
&  0.12917E+00_JPRB, 0.12889E+00_JPRB, 0.12856E+00_JPRB, 0.12825E+00_JPRB, 0.12800E+00_JPRB, 0.12772E+00_JPRB, 0.12742E+00_JPRB ,&
&  0.12712E+00_JPRB, 0.12677E+00_JPRB, 0.12643E+00_JPRB, 0.12611E+00_JPRB, 0.12581E+00_JPRB, 0.12555E+00_JPRB, 0.12526E+00_JPRB ,&
&  0.12505E+00_JPRB, 0.12487E+00_JPRB, 0.12469E+00_JPRB, 0.12455E+00_JPRB, 0.12435E+00_JPRB, 0.12426E+00_JPRB, 0.12408E+00_JPRB ,&
&  0.12401E+00_JPRB, 0.12393E+00_JPRB, 0.12386E+00_JPRB, 0.12378E+00_JPRB, 0.12376E+00_JPRB, 0.12372E+00_JPRB, 0.12369E+00_JPRB ,&
&  0.12365E+00_JPRB, 0.12372E+00_JPRB, 0.12361E+00_JPRB, 0.12347E+00_JPRB, 0.12331E+00_JPRB, 0.12316E+00_JPRB, 0.12302E+00_JPRB ,&
&  0.12289E+00_JPRB, 0.12276E+00_JPRB, 0.12268E+00_JPRB, 0.12268E+00_JPRB, 0.12268E+00_JPRB, 0.12271E+00_JPRB, 0.12276E+00_JPRB ,&
&  0.12282E+00_JPRB, 0.12290E+00_JPRB, 0.12298E+00_JPRB, 0.12296E+00_JPRB, 0.12299E+00_JPRB, 0.12301E+00_JPRB, 0.12303E+00_JPRB ,&
&  0.12305E+00_JPRB, 0.12310E+00_JPRB, 0.12317E+00_JPRB, 0.12326E+00_JPRB, 0.12344E+00_JPRB, 0.12354E+00_JPRB, 0.12367E+00_JPRB ,&
&  0.12385E+00_JPRB, 0.12417E+00_JPRB, 0.12445E+00_JPRB, 0.12475E+00_JPRB, 0.12513E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.13490E+00_JPRB, 0.13674E+00_JPRB, 0.13829E+00_JPRB, 0.14029E+00_JPRB, 0.13763E+00_JPRB, 0.14465E+00_JPRB, 0.14249E+00_JPRB ,&
&  0.14022E+00_JPRB, 0.13556E+00_JPRB, 0.13538E+00_JPRB, 0.13440E+00_JPRB, 0.13314E+00_JPRB, 0.13231E+00_JPRB, 0.13189E+00_JPRB ,&
&  0.13167E+00_JPRB, 0.13127E+00_JPRB, 0.13140E+00_JPRB, 0.13149E+00_JPRB, 0.13156E+00_JPRB, 0.13159E+00_JPRB, 0.13173E+00_JPRB ,&
&  0.13209E+00_JPRB, 0.13283E+00_JPRB, 0.13361E+00_JPRB, 0.13234E+00_JPRB, 0.13230E+00_JPRB, 0.13268E+00_JPRB, 0.13335E+00_JPRB ,&
&  0.13400E+00_JPRB, 0.13501E+00_JPRB, 0.13573E+00_JPRB, 0.13621E+00_JPRB, 0.13738E+00_JPRB, 0.13733E+00_JPRB, 0.13712E+00_JPRB ,&
&  0.13672E+00_JPRB, 0.13601E+00_JPRB, 0.13496E+00_JPRB, 0.13400E+00_JPRB, 0.13300E+00_JPRB, 0.13173E+00_JPRB, 0.13051E+00_JPRB ,&
&  0.12949E+00_JPRB, 0.12866E+00_JPRB, 0.12762E+00_JPRB, 0.12663E+00_JPRB, 0.12571E+00_JPRB, 0.12481E+00_JPRB, 0.12403E+00_JPRB ,&
&  0.12347E+00_JPRB, 0.12300E+00_JPRB, 0.12251E+00_JPRB, 0.12213E+00_JPRB, 0.12173E+00_JPRB, 0.12135E+00_JPRB, 0.12101E+00_JPRB ,&
&  0.12109E+00_JPRB, 0.12116E+00_JPRB, 0.12122E+00_JPRB, 0.12127E+00_JPRB, 0.12153E+00_JPRB, 0.12155E+00_JPRB, 0.12159E+00_JPRB ,&
&  0.12159E+00_JPRB, 0.12148E+00_JPRB, 0.12138E+00_JPRB, 0.12122E+00_JPRB, 0.12102E+00_JPRB, 0.12079E+00_JPRB, 0.12070E+00_JPRB ,&
&  0.12070E+00_JPRB, 0.12080E+00_JPRB, 0.12107E+00_JPRB, 0.12126E+00_JPRB, 0.12143E+00_JPRB, 0.12157E+00_JPRB, 0.12192E+00_JPRB ,&
&  0.12229E+00_JPRB, 0.12267E+00_JPRB, 0.12302E+00_JPRB, 0.12299E+00_JPRB, 0.12321E+00_JPRB, 0.12341E+00_JPRB, 0.12360E+00_JPRB ,&
&  0.12372E+00_JPRB, 0.12412E+00_JPRB, 0.12473E+00_JPRB, 0.12553E+00_JPRB, 0.12696E+00_JPRB, 0.12824E+00_JPRB, 0.12943E+00_JPRB ,&
&  0.13071E+00_JPRB, 0.13322E+00_JPRB, 0.13440E+00_JPRB, 0.13495E+00_JPRB, 0.13522E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.14500E+00_JPRB, 0.14673E+00_JPRB, 0.14781E+00_JPRB, 0.14857E+00_JPRB, 0.14955E+00_JPRB, 0.14995E+00_JPRB, 0.15034E+00_JPRB ,&
&  0.14862E+00_JPRB, 0.14796E+00_JPRB, 0.14688E+00_JPRB, 0.14558E+00_JPRB, 0.14458E+00_JPRB, 0.14256E+00_JPRB, 0.14046E+00_JPRB ,&
&  0.13807E+00_JPRB, 0.13590E+00_JPRB, 0.13442E+00_JPRB, 0.13300E+00_JPRB, 0.13162E+00_JPRB, 0.13047E+00_JPRB, 0.13166E+00_JPRB ,&
&  0.13236E+00_JPRB, 0.13283E+00_JPRB, 0.13387E+00_JPRB, 0.13405E+00_JPRB, 0.13531E+00_JPRB, 0.13727E+00_JPRB, 0.13986E+00_JPRB ,&
&  0.14186E+00_JPRB, 0.14459E+00_JPRB, 0.14622E+00_JPRB, 0.14705E+00_JPRB, 0.14811E+00_JPRB, 0.14879E+00_JPRB, 0.14844E+00_JPRB ,&
&  0.14728E+00_JPRB, 0.14571E+00_JPRB, 0.14395E+00_JPRB, 0.14173E+00_JPRB, 0.13915E+00_JPRB, 0.13662E+00_JPRB, 0.13410E+00_JPRB ,&
&  0.13218E+00_JPRB, 0.13046E+00_JPRB, 0.12871E+00_JPRB, 0.12716E+00_JPRB, 0.12568E+00_JPRB, 0.12435E+00_JPRB, 0.12301E+00_JPRB ,&
&  0.12194E+00_JPRB, 0.12109E+00_JPRB, 0.12007E+00_JPRB, 0.11944E+00_JPRB, 0.11883E+00_JPRB, 0.11853E+00_JPRB, 0.11834E+00_JPRB ,&
&  0.11856E+00_JPRB, 0.11889E+00_JPRB, 0.11924E+00_JPRB, 0.11941E+00_JPRB, 0.11958E+00_JPRB, 0.11964E+00_JPRB, 0.11979E+00_JPRB ,&
&  0.11992E+00_JPRB, 0.12008E+00_JPRB, 0.11976E+00_JPRB, 0.11947E+00_JPRB, 0.11908E+00_JPRB, 0.11865E+00_JPRB, 0.11853E+00_JPRB ,&
&  0.11878E+00_JPRB, 0.11900E+00_JPRB, 0.11894E+00_JPRB, 0.11920E+00_JPRB, 0.11961E+00_JPRB, 0.12005E+00_JPRB, 0.12110E+00_JPRB ,&
&  0.12167E+00_JPRB, 0.12224E+00_JPRB, 0.12278E+00_JPRB, 0.12323E+00_JPRB, 0.12387E+00_JPRB, 0.12441E+00_JPRB, 0.12506E+00_JPRB ,&
&  0.12584E+00_JPRB, 0.12665E+00_JPRB, 0.12773E+00_JPRB, 0.12908E+00_JPRB, 0.13218E+00_JPRB, 0.13442E+00_JPRB, 0.13654E+00_JPRB ,&
&  0.13789E+00_JPRB, 0.13604E+00_JPRB, 0.14320E+00_JPRB, 0.14292E+00_JPRB, 0.14297E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.15041E+00_JPRB, 0.15277E+00_JPRB, 0.15173E+00_JPRB, 0.15174E+00_JPRB, 0.15140E+00_JPRB, 0.15142E+00_JPRB, 0.15070E+00_JPRB ,&
&  0.15047E+00_JPRB, 0.14831E+00_JPRB, 0.14773E+00_JPRB, 0.14603E+00_JPRB, 0.14421E+00_JPRB, 0.14183E+00_JPRB, 0.13948E+00_JPRB ,&
&  0.13707E+00_JPRB, 0.13492E+00_JPRB, 0.13340E+00_JPRB, 0.13256E+00_JPRB, 0.13131E+00_JPRB, 0.13103E+00_JPRB, 0.13232E+00_JPRB ,&
&  0.13261E+00_JPRB, 0.13351E+00_JPRB, 0.13373E+00_JPRB, 0.13497E+00_JPRB, 0.13660E+00_JPRB, 0.13767E+00_JPRB, 0.14009E+00_JPRB ,&
&  0.14208E+00_JPRB, 0.14352E+00_JPRB, 0.14830E+00_JPRB, 0.15115E+00_JPRB, 0.15293E+00_JPRB, 0.15532E+00_JPRB, 0.15494E+00_JPRB ,&
&  0.15385E+00_JPRB, 0.15113E+00_JPRB, 0.15444E+00_JPRB, 0.15297E+00_JPRB, 0.14699E+00_JPRB, 0.14397E+00_JPRB, 0.13895E+00_JPRB ,&
&  0.13561E+00_JPRB, 0.13202E+00_JPRB, 0.12930E+00_JPRB, 0.12724E+00_JPRB, 0.12476E+00_JPRB, 0.12314E+00_JPRB, 0.12133E+00_JPRB ,&
&  0.11959E+00_JPRB, 0.11873E+00_JPRB, 0.11748E+00_JPRB, 0.11719E+00_JPRB, 0.11718E+00_JPRB, 0.11742E+00_JPRB, 0.11787E+00_JPRB ,&
&  0.11852E+00_JPRB, 0.11915E+00_JPRB, 0.11974E+00_JPRB, 0.12025E+00_JPRB, 0.12069E+00_JPRB, 0.12097E+00_JPRB, 0.12089E+00_JPRB ,&
&  0.12096E+00_JPRB, 0.12084E+00_JPRB, 0.12042E+00_JPRB, 0.12015E+00_JPRB, 0.11936E+00_JPRB, 0.11897E+00_JPRB, 0.11859E+00_JPRB ,&
&  0.11841E+00_JPRB, 0.11843E+00_JPRB, 0.11838E+00_JPRB, 0.11857E+00_JPRB, 0.11884E+00_JPRB, 0.11944E+00_JPRB, 0.11997E+00_JPRB ,&
&  0.12038E+00_JPRB, 0.12111E+00_JPRB, 0.12157E+00_JPRB, 0.12219E+00_JPRB, 0.12305E+00_JPRB, 0.12352E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12551E+00_JPRB, 0.12658E+00_JPRB, 0.12844E+00_JPRB, 0.13034E+00_JPRB, 0.13300E+00_JPRB, 0.13654E+00_JPRB, 0.13838E+00_JPRB ,&
&  0.14025E+00_JPRB, 0.14252E+00_JPRB, 0.14219E+00_JPRB, 0.14635E+00_JPRB, 0.14826E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.15231E+00_JPRB, 0.15315E+00_JPRB, 0.15163E+00_JPRB, 0.15216E+00_JPRB, 0.15298E+00_JPRB, 0.15345E+00_JPRB, 0.15390E+00_JPRB ,&
&  0.15161E+00_JPRB, 0.15057E+00_JPRB, 0.14825E+00_JPRB, 0.14596E+00_JPRB, 0.14339E+00_JPRB, 0.14063E+00_JPRB, 0.13847E+00_JPRB ,&
&  0.13669E+00_JPRB, 0.13586E+00_JPRB, 0.13506E+00_JPRB, 0.13558E+00_JPRB, 0.13563E+00_JPRB, 0.13707E+00_JPRB, 0.13828E+00_JPRB ,&
&  0.13903E+00_JPRB, 0.13992E+00_JPRB, 0.13894E+00_JPRB, 0.13904E+00_JPRB, 0.14083E+00_JPRB, 0.14260E+00_JPRB, 0.14522E+00_JPRB ,&
&  0.14807E+00_JPRB, 0.15097E+00_JPRB, 0.15557E+00_JPRB, 0.15733E+00_JPRB, 0.15931E+00_JPRB, 0.16064E+00_JPRB, 0.16071E+00_JPRB ,&
&  0.16047E+00_JPRB, 0.16074E+00_JPRB, 0.16169E+00_JPRB, 0.16156E+00_JPRB, 0.15516E+00_JPRB, 0.15009E+00_JPRB, 0.14482E+00_JPRB ,&
&  0.13979E+00_JPRB, 0.13509E+00_JPRB, 0.13098E+00_JPRB, 0.12821E+00_JPRB, 0.12553E+00_JPRB, 0.12356E+00_JPRB, 0.12203E+00_JPRB ,&
&  0.12044E+00_JPRB, 0.11943E+00_JPRB, 0.11822E+00_JPRB, 0.11782E+00_JPRB, 0.11800E+00_JPRB, 0.11859E+00_JPRB, 0.11940E+00_JPRB ,&
&  0.12036E+00_JPRB, 0.12105E+00_JPRB, 0.12192E+00_JPRB, 0.12278E+00_JPRB, 0.12340E+00_JPRB, 0.12382E+00_JPRB, 0.12365E+00_JPRB ,&
&  0.12384E+00_JPRB, 0.12403E+00_JPRB, 0.12364E+00_JPRB, 0.12353E+00_JPRB, 0.12256E+00_JPRB, 0.12146E+00_JPRB, 0.12029E+00_JPRB ,&
&  0.11882E+00_JPRB, 0.11767E+00_JPRB, 0.11723E+00_JPRB, 0.11698E+00_JPRB, 0.11752E+00_JPRB, 0.11837E+00_JPRB, 0.11896E+00_JPRB ,&
&  0.11931E+00_JPRB, 0.11929E+00_JPRB, 0.11939E+00_JPRB, 0.12036E+00_JPRB, 0.12137E+00_JPRB, 0.12266E+00_JPRB, 0.12399E+00_JPRB ,&
&  0.12448E+00_JPRB, 0.12560E+00_JPRB, 0.12661E+00_JPRB, 0.12779E+00_JPRB, 0.13159E+00_JPRB, 0.13481E+00_JPRB, 0.14095E+00_JPRB ,&
&  0.14470E+00_JPRB, 0.14816E+00_JPRB, 0.14920E+00_JPRB, 0.15223E+00_JPRB, 0.15196E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.15619E+00_JPRB, 0.15814E+00_JPRB, 0.16005E+00_JPRB, 0.15920E+00_JPRB, 0.15936E+00_JPRB, 0.15943E+00_JPRB, 0.15951E+00_JPRB ,&
&  0.15950E+00_JPRB, 0.15897E+00_JPRB, 0.15914E+00_JPRB, 0.15774E+00_JPRB, 0.15689E+00_JPRB, 0.15316E+00_JPRB, 0.15159E+00_JPRB ,&
&  0.14959E+00_JPRB, 0.14892E+00_JPRB, 0.14906E+00_JPRB, 0.15002E+00_JPRB, 0.15049E+00_JPRB, 0.15164E+00_JPRB, 0.15110E+00_JPRB ,&
&  0.15119E+00_JPRB, 0.15096E+00_JPRB, 0.14982E+00_JPRB, 0.14953E+00_JPRB, 0.15004E+00_JPRB, 0.15069E+00_JPRB, 0.15103E+00_JPRB ,&
&  0.15259E+00_JPRB, 0.15469E+00_JPRB, 0.15674E+00_JPRB, 0.15824E+00_JPRB, 0.16352E+00_JPRB, 0.16510E+00_JPRB, 0.16677E+00_JPRB ,&
&  0.16789E+00_JPRB, 0.16941E+00_JPRB, 0.17038E+00_JPRB, 0.16987E+00_JPRB, 0.16877E+00_JPRB, 0.16570E+00_JPRB, 0.16447E+00_JPRB ,&
&  0.15446E+00_JPRB, 0.14779E+00_JPRB, 0.14117E+00_JPRB, 0.13543E+00_JPRB, 0.13141E+00_JPRB, 0.12863E+00_JPRB, 0.12631E+00_JPRB ,&
&  0.12438E+00_JPRB, 0.12257E+00_JPRB, 0.12065E+00_JPRB, 0.12058E+00_JPRB, 0.12122E+00_JPRB, 0.12218E+00_JPRB, 0.12312E+00_JPRB ,&
&  0.12419E+00_JPRB, 0.12481E+00_JPRB, 0.12514E+00_JPRB, 0.12581E+00_JPRB, 0.12601E+00_JPRB, 0.12637E+00_JPRB, 0.12657E+00_JPRB ,&
&  0.12765E+00_JPRB, 0.12802E+00_JPRB, 0.12809E+00_JPRB, 0.12914E+00_JPRB, 0.12778E+00_JPRB, 0.12553E+00_JPRB, 0.12293E+00_JPRB ,&
&  0.12048E+00_JPRB, 0.11892E+00_JPRB, 0.11764E+00_JPRB, 0.11712E+00_JPRB, 0.11741E+00_JPRB, 0.11799E+00_JPRB, 0.11881E+00_JPRB ,&
&  0.11905E+00_JPRB, 0.11957E+00_JPRB, 0.11998E+00_JPRB, 0.12074E+00_JPRB, 0.12158E+00_JPRB, 0.12250E+00_JPRB, 0.12347E+00_JPRB ,&
&  0.12493E+00_JPRB, 0.12661E+00_JPRB, 0.12832E+00_JPRB, 0.13008E+00_JPRB, 0.13287E+00_JPRB, 0.13634E+00_JPRB, 0.14192E+00_JPRB ,&
&  0.14746E+00_JPRB, 0.15595E+00_JPRB, 0.15567E+00_JPRB, 0.15700E+00_JPRB, 0.15707E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.16248E+00_JPRB, 0.16601E+00_JPRB, 0.16767E+00_JPRB, 0.16776E+00_JPRB, 0.16680E+00_JPRB, 0.16601E+00_JPRB, 0.16589E+00_JPRB ,&
&  0.16583E+00_JPRB, 0.16478E+00_JPRB, 0.16399E+00_JPRB, 0.16328E+00_JPRB, 0.16254E+00_JPRB, 0.16184E+00_JPRB, 0.16211E+00_JPRB ,&
&  0.16325E+00_JPRB, 0.16448E+00_JPRB, 0.16525E+00_JPRB, 0.16539E+00_JPRB, 0.16487E+00_JPRB, 0.16511E+00_JPRB, 0.16590E+00_JPRB ,&
&  0.16586E+00_JPRB, 0.16634E+00_JPRB, 0.16959E+00_JPRB, 0.17029E+00_JPRB, 0.17131E+00_JPRB, 0.17154E+00_JPRB, 0.17163E+00_JPRB ,&
&  0.16747E+00_JPRB, 0.16251E+00_JPRB, 0.15906E+00_JPRB, 0.16088E+00_JPRB, 0.16928E+00_JPRB, 0.17101E+00_JPRB, 0.17387E+00_JPRB ,&
&  0.17583E+00_JPRB, 0.17699E+00_JPRB, 0.17817E+00_JPRB, 0.17931E+00_JPRB, 0.18068E+00_JPRB, 0.17970E+00_JPRB, 0.17758E+00_JPRB ,&
&  0.17513E+00_JPRB, 0.17417E+00_JPRB, 0.17108E+00_JPRB, 0.15791E+00_JPRB, 0.14959E+00_JPRB, 0.14490E+00_JPRB, 0.14007E+00_JPRB ,&
&  0.13572E+00_JPRB, 0.13305E+00_JPRB, 0.13221E+00_JPRB, 0.13115E+00_JPRB, 0.13194E+00_JPRB, 0.13349E+00_JPRB, 0.13417E+00_JPRB ,&
&  0.13387E+00_JPRB, 0.13419E+00_JPRB, 0.13390E+00_JPRB, 0.13306E+00_JPRB, 0.13165E+00_JPRB, 0.13075E+00_JPRB, 0.13022E+00_JPRB ,&
&  0.13062E+00_JPRB, 0.13053E+00_JPRB, 0.13415E+00_JPRB, 0.13567E+00_JPRB, 0.13197E+00_JPRB, 0.13006E+00_JPRB, 0.12785E+00_JPRB ,&
&  0.12354E+00_JPRB, 0.12109E+00_JPRB, 0.11976E+00_JPRB, 0.11924E+00_JPRB, 0.12003E+00_JPRB, 0.12120E+00_JPRB, 0.12186E+00_JPRB ,&
&  0.12210E+00_JPRB, 0.12293E+00_JPRB, 0.12335E+00_JPRB, 0.12383E+00_JPRB, 0.12417E+00_JPRB, 0.12487E+00_JPRB, 0.12589E+00_JPRB ,&
&  0.12720E+00_JPRB, 0.12952E+00_JPRB, 0.13186E+00_JPRB, 0.13451E+00_JPRB, 0.13614E+00_JPRB, 0.13744E+00_JPRB, 0.14027E+00_JPRB ,&
&  0.14236E+00_JPRB, 0.14869E+00_JPRB, 0.15811E+00_JPRB, 0.16245E+00_JPRB, 0.15857E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.17345E+00_JPRB, 0.17535E+00_JPRB, 0.17603E+00_JPRB, 0.17613E+00_JPRB, 0.17608E+00_JPRB, 0.17662E+00_JPRB, 0.17715E+00_JPRB ,&
&  0.17713E+00_JPRB, 0.17645E+00_JPRB, 0.17531E+00_JPRB, 0.17349E+00_JPRB, 0.17135E+00_JPRB, 0.17017E+00_JPRB, 0.17122E+00_JPRB ,&
&  0.17206E+00_JPRB, 0.17172E+00_JPRB, 0.17150E+00_JPRB, 0.17114E+00_JPRB, 0.17053E+00_JPRB, 0.17057E+00_JPRB, 0.17164E+00_JPRB ,&
&  0.17305E+00_JPRB, 0.17410E+00_JPRB, 0.17555E+00_JPRB, 0.17684E+00_JPRB, 0.17887E+00_JPRB, 0.18070E+00_JPRB, 0.18162E+00_JPRB ,&
&  0.17988E+00_JPRB, 0.17226E+00_JPRB, 0.16662E+00_JPRB, 0.16897E+00_JPRB, 0.17779E+00_JPRB, 0.18231E+00_JPRB, 0.18493E+00_JPRB ,&
&  0.18667E+00_JPRB, 0.18764E+00_JPRB, 0.18790E+00_JPRB, 0.18914E+00_JPRB, 0.19099E+00_JPRB, 0.19272E+00_JPRB, 0.19237E+00_JPRB ,&
&  0.19045E+00_JPRB, 0.18870E+00_JPRB, 0.18618E+00_JPRB, 0.18199E+00_JPRB, 0.17641E+00_JPRB, 0.17094E+00_JPRB, 0.16637E+00_JPRB ,&
&  0.16240E+00_JPRB, 0.15852E+00_JPRB, 0.15512E+00_JPRB, 0.15292E+00_JPRB, 0.15216E+00_JPRB, 0.15314E+00_JPRB, 0.15301E+00_JPRB ,&
&  0.15253E+00_JPRB, 0.15085E+00_JPRB, 0.14873E+00_JPRB, 0.14631E+00_JPRB, 0.14417E+00_JPRB, 0.14062E+00_JPRB, 0.13642E+00_JPRB ,&
&  0.13418E+00_JPRB, 0.13607E+00_JPRB, 0.14190E+00_JPRB, 0.14709E+00_JPRB, 0.14963E+00_JPRB, 0.14200E+00_JPRB, 0.13761E+00_JPRB ,&
&  0.13360E+00_JPRB, 0.12996E+00_JPRB, 0.12770E+00_JPRB, 0.12715E+00_JPRB, 0.12727E+00_JPRB, 0.12744E+00_JPRB, 0.12824E+00_JPRB ,&
&  0.12952E+00_JPRB, 0.13069E+00_JPRB, 0.13138E+00_JPRB, 0.13204E+00_JPRB, 0.13230E+00_JPRB, 0.13235E+00_JPRB, 0.13246E+00_JPRB ,&
&  0.13368E+00_JPRB, 0.13636E+00_JPRB, 0.14014E+00_JPRB, 0.14449E+00_JPRB, 0.14792E+00_JPRB, 0.15106E+00_JPRB, 0.15422E+00_JPRB ,&
&  0.15774E+00_JPRB, 0.16065E+00_JPRB, 0.16349E+00_JPRB, 0.16711E+00_JPRB, 0.17043E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.18306E+00_JPRB, 0.18514E+00_JPRB, 0.18600E+00_JPRB, 0.18579E+00_JPRB, 0.18552E+00_JPRB, 0.18651E+00_JPRB, 0.18779E+00_JPRB ,&
&  0.18842E+00_JPRB, 0.18844E+00_JPRB, 0.18768E+00_JPRB, 0.18621E+00_JPRB, 0.18520E+00_JPRB, 0.18560E+00_JPRB, 0.18409E+00_JPRB ,&
&  0.18250E+00_JPRB, 0.18237E+00_JPRB, 0.18082E+00_JPRB, 0.17917E+00_JPRB, 0.17674E+00_JPRB, 0.17520E+00_JPRB, 0.17562E+00_JPRB ,&
&  0.17764E+00_JPRB, 0.17839E+00_JPRB, 0.17924E+00_JPRB, 0.18039E+00_JPRB, 0.18246E+00_JPRB, 0.18482E+00_JPRB, 0.18657E+00_JPRB ,&
&  0.18800E+00_JPRB, 0.18774E+00_JPRB, 0.18597E+00_JPRB, 0.18898E+00_JPRB, 0.19173E+00_JPRB, 0.19514E+00_JPRB, 0.19738E+00_JPRB ,&
&  0.19925E+00_JPRB, 0.20066E+00_JPRB, 0.20128E+00_JPRB, 0.20118E+00_JPRB, 0.20156E+00_JPRB, 0.20319E+00_JPRB, 0.20429E+00_JPRB ,&
&  0.20457E+00_JPRB, 0.20396E+00_JPRB, 0.20244E+00_JPRB, 0.20091E+00_JPRB, 0.19754E+00_JPRB, 0.19264E+00_JPRB, 0.18557E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17260E+00_JPRB, 0.16824E+00_JPRB, 0.16468E+00_JPRB, 0.16215E+00_JPRB, 0.16169E+00_JPRB, 0.16158E+00_JPRB ,&
&  0.16123E+00_JPRB, 0.16093E+00_JPRB, 0.16236E+00_JPRB, 0.16336E+00_JPRB, 0.16041E+00_JPRB, 0.15493E+00_JPRB, 0.15076E+00_JPRB ,&
&  0.14989E+00_JPRB, 0.15153E+00_JPRB, 0.15424E+00_JPRB, 0.15578E+00_JPRB, 0.15846E+00_JPRB, 0.15914E+00_JPRB, 0.15836E+00_JPRB ,&
&  0.15379E+00_JPRB, 0.14804E+00_JPRB, 0.14484E+00_JPRB, 0.14258E+00_JPRB, 0.14109E+00_JPRB, 0.14067E+00_JPRB, 0.14117E+00_JPRB ,&
&  0.14298E+00_JPRB, 0.14500E+00_JPRB, 0.14712E+00_JPRB, 0.14807E+00_JPRB, 0.14769E+00_JPRB, 0.14705E+00_JPRB, 0.14625E+00_JPRB ,&
&  0.14703E+00_JPRB, 0.15064E+00_JPRB, 0.15648E+00_JPRB, 0.16040E+00_JPRB, 0.16373E+00_JPRB, 0.16594E+00_JPRB, 0.16717E+00_JPRB ,&
&  0.16812E+00_JPRB, 0.16886E+00_JPRB, 0.16983E+00_JPRB, 0.17416E+00_JPRB, 0.17969E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.19433E+00_JPRB, 0.19658E+00_JPRB, 0.19788E+00_JPRB, 0.19803E+00_JPRB, 0.19784E+00_JPRB, 0.19869E+00_JPRB, 0.19919E+00_JPRB ,&
&  0.19969E+00_JPRB, 0.19995E+00_JPRB, 0.19907E+00_JPRB, 0.19836E+00_JPRB, 0.20009E+00_JPRB, 0.19961E+00_JPRB, 0.19906E+00_JPRB ,&
&  0.19744E+00_JPRB, 0.19555E+00_JPRB, 0.19277E+00_JPRB, 0.18957E+00_JPRB, 0.18564E+00_JPRB, 0.18308E+00_JPRB, 0.18320E+00_JPRB ,&
&  0.18455E+00_JPRB, 0.18510E+00_JPRB, 0.18600E+00_JPRB, 0.18732E+00_JPRB, 0.18992E+00_JPRB, 0.19319E+00_JPRB, 0.19572E+00_JPRB ,&
&  0.19780E+00_JPRB, 0.20108E+00_JPRB, 0.20315E+00_JPRB, 0.20171E+00_JPRB, 0.20595E+00_JPRB, 0.20959E+00_JPRB, 0.21259E+00_JPRB ,&
&  0.21481E+00_JPRB, 0.21591E+00_JPRB, 0.21565E+00_JPRB, 0.21461E+00_JPRB, 0.21386E+00_JPRB, 0.21480E+00_JPRB, 0.21515E+00_JPRB ,&
&  0.21536E+00_JPRB, 0.21704E+00_JPRB, 0.21913E+00_JPRB, 0.21952E+00_JPRB, 0.21770E+00_JPRB, 0.21363E+00_JPRB, 0.20747E+00_JPRB ,&
&  0.19970E+00_JPRB, 0.19163E+00_JPRB, 0.18515E+00_JPRB, 0.17999E+00_JPRB, 0.17600E+00_JPRB, 0.17264E+00_JPRB, 0.16992E+00_JPRB ,&
&  0.16697E+00_JPRB, 0.16499E+00_JPRB, 0.16507E+00_JPRB, 0.16582E+00_JPRB, 0.16631E+00_JPRB, 0.16549E+00_JPRB, 0.16490E+00_JPRB ,&
&  0.16425E+00_JPRB, 0.16473E+00_JPRB, 0.16550E+00_JPRB, 0.16581E+00_JPRB, 0.16590E+00_JPRB, 0.16547E+00_JPRB, 0.16508E+00_JPRB ,&
&  0.16571E+00_JPRB, 0.16607E+00_JPRB, 0.16531E+00_JPRB, 0.16371E+00_JPRB, 0.16234E+00_JPRB, 0.16202E+00_JPRB, 0.16206E+00_JPRB ,&
&  0.16239E+00_JPRB, 0.16319E+00_JPRB, 0.16455E+00_JPRB, 0.16602E+00_JPRB, 0.16717E+00_JPRB, 0.16763E+00_JPRB, 0.16768E+00_JPRB ,&
&  0.16773E+00_JPRB, 0.16741E+00_JPRB, 0.16797E+00_JPRB, 0.17031E+00_JPRB, 0.17303E+00_JPRB, 0.17536E+00_JPRB, 0.17679E+00_JPRB ,&
&  0.17661E+00_JPRB, 0.17750E+00_JPRB, 0.18060E+00_JPRB, 0.18645E+00_JPRB, 0.19090E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.20986E+00_JPRB, 0.21233E+00_JPRB, 0.21501E+00_JPRB, 0.21545E+00_JPRB, 0.21413E+00_JPRB, 0.21355E+00_JPRB, 0.21376E+00_JPRB ,&
&  0.21420E+00_JPRB, 0.21454E+00_JPRB, 0.21364E+00_JPRB, 0.21372E+00_JPRB, 0.21388E+00_JPRB, 0.21302E+00_JPRB, 0.21227E+00_JPRB ,&
&  0.21093E+00_JPRB, 0.20808E+00_JPRB, 0.20416E+00_JPRB, 0.19934E+00_JPRB, 0.19572E+00_JPRB, 0.19375E+00_JPRB, 0.19337E+00_JPRB ,&
&  0.19394E+00_JPRB, 0.19441E+00_JPRB, 0.19691E+00_JPRB, 0.19924E+00_JPRB, 0.20226E+00_JPRB, 0.20617E+00_JPRB, 0.21019E+00_JPRB ,&
&  0.21326E+00_JPRB, 0.21638E+00_JPRB, 0.21933E+00_JPRB, 0.22201E+00_JPRB, 0.22543E+00_JPRB, 0.23044E+00_JPRB, 0.23434E+00_JPRB ,&
&  0.23784E+00_JPRB, 0.23896E+00_JPRB, 0.23613E+00_JPRB, 0.23338E+00_JPRB, 0.23320E+00_JPRB, 0.23450E+00_JPRB, 0.23559E+00_JPRB ,&
&  0.23632E+00_JPRB, 0.23719E+00_JPRB, 0.23793E+00_JPRB, 0.23727E+00_JPRB, 0.23449E+00_JPRB, 0.22990E+00_JPRB, 0.22405E+00_JPRB ,&
&  0.21774E+00_JPRB, 0.21155E+00_JPRB, 0.20572E+00_JPRB, 0.19997E+00_JPRB, 0.19510E+00_JPRB, 0.19029E+00_JPRB, 0.18576E+00_JPRB ,&
&  0.18148E+00_JPRB, 0.17742E+00_JPRB, 0.17441E+00_JPRB, 0.17268E+00_JPRB, 0.17261E+00_JPRB, 0.17198E+00_JPRB, 0.17228E+00_JPRB ,&
&  0.17210E+00_JPRB, 0.17216E+00_JPRB, 0.17259E+00_JPRB, 0.17368E+00_JPRB, 0.17517E+00_JPRB, 0.17530E+00_JPRB, 0.17420E+00_JPRB ,&
&  0.17356E+00_JPRB, 0.17459E+00_JPRB, 0.17434E+00_JPRB, 0.17218E+00_JPRB, 0.16946E+00_JPRB, 0.16872E+00_JPRB, 0.16984E+00_JPRB ,&
&  0.17212E+00_JPRB, 0.17432E+00_JPRB, 0.17619E+00_JPRB, 0.17796E+00_JPRB, 0.17944E+00_JPRB, 0.18127E+00_JPRB, 0.18295E+00_JPRB ,&
&  0.18432E+00_JPRB, 0.18488E+00_JPRB, 0.18613E+00_JPRB, 0.18748E+00_JPRB, 0.18895E+00_JPRB, 0.19004E+00_JPRB, 0.18995E+00_JPRB ,&
&  0.18960E+00_JPRB, 0.19104E+00_JPRB, 0.19557E+00_JPRB, 0.20262E+00_JPRB, 0.20710E+00_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.22900E+00_JPRB, 0.23105E+00_JPRB, 0.23385E+00_JPRB, 0.23630E+00_JPRB, 0.23738E+00_JPRB, 0.23710E+00_JPRB, 0.23646E+00_JPRB ,&
&  0.23652E+00_JPRB, 0.23674E+00_JPRB, 0.23554E+00_JPRB, 0.23330E+00_JPRB, 0.23098E+00_JPRB, 0.22845E+00_JPRB, 0.22648E+00_JPRB ,&
&  0.22452E+00_JPRB, 0.22245E+00_JPRB, 0.21895E+00_JPRB, 0.21392E+00_JPRB, 0.20871E+00_JPRB, 0.20567E+00_JPRB, 0.20419E+00_JPRB ,&
&  0.20410E+00_JPRB, 0.20463E+00_JPRB, 0.20721E+00_JPRB, 0.20911E+00_JPRB, 0.21170E+00_JPRB, 0.21589E+00_JPRB, 0.22152E+00_JPRB ,&
&  0.22686E+00_JPRB, 0.23189E+00_JPRB, 0.23692E+00_JPRB, 0.24112E+00_JPRB, 0.24529E+00_JPRB, 0.25016E+00_JPRB, 0.25475E+00_JPRB ,&
&  0.25933E+00_JPRB, 0.26231E+00_JPRB, 0.26264E+00_JPRB, 0.26351E+00_JPRB, 0.26560E+00_JPRB, 0.26781E+00_JPRB, 0.26899E+00_JPRB ,&
&  0.26972E+00_JPRB, 0.26773E+00_JPRB, 0.26496E+00_JPRB, 0.26119E+00_JPRB, 0.25726E+00_JPRB, 0.25299E+00_JPRB, 0.24726E+00_JPRB ,&
&  0.24042E+00_JPRB, 0.23301E+00_JPRB, 0.22600E+00_JPRB, 0.21939E+00_JPRB, 0.21310E+00_JPRB, 0.20725E+00_JPRB, 0.20186E+00_JPRB ,&
&  0.19652E+00_JPRB, 0.19241E+00_JPRB, 0.18848E+00_JPRB, 0.18631E+00_JPRB, 0.18526E+00_JPRB, 0.18393E+00_JPRB, 0.18398E+00_JPRB ,&
&  0.18347E+00_JPRB, 0.18327E+00_JPRB, 0.18325E+00_JPRB, 0.18493E+00_JPRB, 0.18798E+00_JPRB, 0.18976E+00_JPRB, 0.18867E+00_JPRB ,&
&  0.18547E+00_JPRB, 0.18346E+00_JPRB, 0.18236E+00_JPRB, 0.18146E+00_JPRB, 0.17919E+00_JPRB, 0.17874E+00_JPRB, 0.18036E+00_JPRB ,&
&  0.18398E+00_JPRB, 0.18802E+00_JPRB, 0.19169E+00_JPRB, 0.19386E+00_JPRB, 0.19599E+00_JPRB, 0.19755E+00_JPRB, 0.19893E+00_JPRB ,&
&  0.20058E+00_JPRB, 0.20180E+00_JPRB, 0.20277E+00_JPRB, 0.20413E+00_JPRB, 0.20647E+00_JPRB, 0.20883E+00_JPRB, 0.21116E+00_JPRB ,&
&  0.21361E+00_JPRB, 0.21729E+00_JPRB, 0.22103E+00_JPRB, 0.22373E+00_JPRB, 0.22697E+00_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.25633E+00_JPRB, 0.25802E+00_JPRB, 0.26014E+00_JPRB, 0.26075E+00_JPRB, 0.26045E+00_JPRB, 0.26065E+00_JPRB, 0.25981E+00_JPRB ,&
&  0.25776E+00_JPRB, 0.25673E+00_JPRB, 0.25705E+00_JPRB, 0.25581E+00_JPRB, 0.25323E+00_JPRB, 0.25003E+00_JPRB, 0.24650E+00_JPRB ,&
&  0.24371E+00_JPRB, 0.24086E+00_JPRB, 0.23612E+00_JPRB, 0.22974E+00_JPRB, 0.22423E+00_JPRB, 0.22075E+00_JPRB, 0.21905E+00_JPRB ,&
&  0.21848E+00_JPRB, 0.21872E+00_JPRB, 0.21858E+00_JPRB, 0.21938E+00_JPRB, 0.22156E+00_JPRB, 0.22631E+00_JPRB, 0.23330E+00_JPRB ,&
&  0.24039E+00_JPRB, 0.24663E+00_JPRB, 0.25303E+00_JPRB, 0.25993E+00_JPRB, 0.26815E+00_JPRB, 0.27705E+00_JPRB, 0.28409E+00_JPRB ,&
&  0.29016E+00_JPRB, 0.29210E+00_JPRB, 0.29322E+00_JPRB, 0.29701E+00_JPRB, 0.30410E+00_JPRB, 0.30871E+00_JPRB, 0.31161E+00_JPRB ,&
&  0.31304E+00_JPRB, 0.31171E+00_JPRB, 0.30940E+00_JPRB, 0.30488E+00_JPRB, 0.29844E+00_JPRB, 0.29003E+00_JPRB, 0.28096E+00_JPRB ,&
&  0.27202E+00_JPRB, 0.26285E+00_JPRB, 0.25294E+00_JPRB, 0.24354E+00_JPRB, 0.23560E+00_JPRB, 0.22860E+00_JPRB, 0.22144E+00_JPRB ,&
&  0.21463E+00_JPRB, 0.20913E+00_JPRB, 0.20479E+00_JPRB, 0.20195E+00_JPRB, 0.19973E+00_JPRB, 0.19808E+00_JPRB, 0.19792E+00_JPRB ,&
&  0.19850E+00_JPRB, 0.20000E+00_JPRB, 0.20070E+00_JPRB, 0.20206E+00_JPRB, 0.20436E+00_JPRB, 0.20541E+00_JPRB, 0.20451E+00_JPRB ,&
&  0.20087E+00_JPRB, 0.19701E+00_JPRB, 0.19339E+00_JPRB, 0.19263E+00_JPRB, 0.19245E+00_JPRB, 0.19339E+00_JPRB, 0.19485E+00_JPRB ,&
&  0.19811E+00_JPRB, 0.20306E+00_JPRB, 0.20746E+00_JPRB, 0.21043E+00_JPRB, 0.21373E+00_JPRB, 0.21659E+00_JPRB, 0.21952E+00_JPRB ,&
&  0.22297E+00_JPRB, 0.22682E+00_JPRB, 0.23207E+00_JPRB, 0.23731E+00_JPRB, 0.24205E+00_JPRB, 0.24583E+00_JPRB, 0.24862E+00_JPRB ,&
&  0.25043E+00_JPRB, 0.25084E+00_JPRB, 0.25098E+00_JPRB, 0.25231E+00_JPRB, 0.25455E+00_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.28655E+00_JPRB, 0.28708E+00_JPRB, 0.28821E+00_JPRB, 0.28904E+00_JPRB, 0.28696E+00_JPRB, 0.28459E+00_JPRB, 0.28162E+00_JPRB ,&
&  0.28040E+00_JPRB, 0.27946E+00_JPRB, 0.27910E+00_JPRB, 0.27631E+00_JPRB, 0.27414E+00_JPRB, 0.27286E+00_JPRB, 0.26965E+00_JPRB ,&
&  0.26679E+00_JPRB, 0.26006E+00_JPRB, 0.25273E+00_JPRB, 0.24721E+00_JPRB, 0.24367E+00_JPRB, 0.24139E+00_JPRB, 0.23682E+00_JPRB ,&
&  0.23375E+00_JPRB, 0.23138E+00_JPRB, 0.23040E+00_JPRB, 0.23078E+00_JPRB, 0.23295E+00_JPRB, 0.23873E+00_JPRB, 0.24837E+00_JPRB ,&
&  0.25519E+00_JPRB, 0.26272E+00_JPRB, 0.26256E+00_JPRB, 0.27498E+00_JPRB, 0.28667E+00_JPRB, 0.30132E+00_JPRB, 0.31267E+00_JPRB ,&
&  0.32184E+00_JPRB, 0.32999E+00_JPRB, 0.33955E+00_JPRB, 0.34836E+00_JPRB, 0.35207E+00_JPRB, 0.35371E+00_JPRB, 0.35322E+00_JPRB ,&
&  0.35057E+00_JPRB, 0.34601E+00_JPRB, 0.34077E+00_JPRB, 0.33450E+00_JPRB, 0.32583E+00_JPRB, 0.31566E+00_JPRB, 0.30543E+00_JPRB ,&
&  0.29593E+00_JPRB, 0.28662E+00_JPRB, 0.27746E+00_JPRB, 0.26757E+00_JPRB, 0.25779E+00_JPRB, 0.24894E+00_JPRB, 0.23987E+00_JPRB ,&
&  0.23216E+00_JPRB, 0.22609E+00_JPRB, 0.22158E+00_JPRB, 0.21850E+00_JPRB, 0.21631E+00_JPRB, 0.21414E+00_JPRB, 0.21328E+00_JPRB ,&
&  0.21365E+00_JPRB, 0.21532E+00_JPRB, 0.21745E+00_JPRB, 0.21914E+00_JPRB, 0.22118E+00_JPRB, 0.22156E+00_JPRB, 0.21930E+00_JPRB ,&
&  0.21679E+00_JPRB, 0.21543E+00_JPRB, 0.21127E+00_JPRB, 0.20876E+00_JPRB, 0.20824E+00_JPRB, 0.20861E+00_JPRB, 0.21028E+00_JPRB ,&
&  0.21367E+00_JPRB, 0.21845E+00_JPRB, 0.22285E+00_JPRB, 0.22658E+00_JPRB, 0.22967E+00_JPRB, 0.23294E+00_JPRB, 0.23691E+00_JPRB ,&
&  0.24117E+00_JPRB, 0.24535E+00_JPRB, 0.25197E+00_JPRB, 0.25941E+00_JPRB, 0.26579E+00_JPRB, 0.27255E+00_JPRB, 0.27667E+00_JPRB ,&
&  0.27817E+00_JPRB, 0.27875E+00_JPRB, 0.28076E+00_JPRB, 0.28377E+00_JPRB, 0.28593E+00_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.32440E+00_JPRB, 0.32271E+00_JPRB, 0.32040E+00_JPRB, 0.31862E+00_JPRB, 0.31754E+00_JPRB, 0.31467E+00_JPRB, 0.31083E+00_JPRB ,&
&  0.30818E+00_JPRB, 0.30558E+00_JPRB, 0.30368E+00_JPRB, 0.30062E+00_JPRB, 0.29897E+00_JPRB, 0.29800E+00_JPRB, 0.29441E+00_JPRB ,&
&  0.29015E+00_JPRB, 0.28409E+00_JPRB, 0.27814E+00_JPRB, 0.27280E+00_JPRB, 0.26776E+00_JPRB, 0.26318E+00_JPRB, 0.25811E+00_JPRB ,&
&  0.25335E+00_JPRB, 0.24991E+00_JPRB, 0.24825E+00_JPRB, 0.24676E+00_JPRB, 0.24827E+00_JPRB, 0.25421E+00_JPRB, 0.26365E+00_JPRB ,&
&  0.26897E+00_JPRB, 0.26857E+00_JPRB, 0.27816E+00_JPRB, 0.28749E+00_JPRB, 0.30467E+00_JPRB, 0.32605E+00_JPRB, 0.34189E+00_JPRB ,&
&  0.35349E+00_JPRB, 0.36628E+00_JPRB, 0.37880E+00_JPRB, 0.38581E+00_JPRB, 0.38616E+00_JPRB, 0.38157E+00_JPRB, 0.37830E+00_JPRB ,&
&  0.37429E+00_JPRB, 0.36636E+00_JPRB, 0.35773E+00_JPRB, 0.34655E+00_JPRB, 0.33443E+00_JPRB, 0.32443E+00_JPRB, 0.31642E+00_JPRB ,&
&  0.30874E+00_JPRB, 0.30104E+00_JPRB, 0.29396E+00_JPRB, 0.28676E+00_JPRB, 0.27893E+00_JPRB, 0.27078E+00_JPRB, 0.26223E+00_JPRB ,&
&  0.25384E+00_JPRB, 0.24655E+00_JPRB, 0.24069E+00_JPRB, 0.23771E+00_JPRB, 0.23620E+00_JPRB, 0.23508E+00_JPRB, 0.23438E+00_JPRB ,&
&  0.23493E+00_JPRB, 0.23631E+00_JPRB, 0.23802E+00_JPRB, 0.23957E+00_JPRB, 0.24081E+00_JPRB, 0.24086E+00_JPRB, 0.23925E+00_JPRB ,&
&  0.23758E+00_JPRB, 0.23686E+00_JPRB, 0.23661E+00_JPRB, 0.23423E+00_JPRB, 0.23173E+00_JPRB, 0.23210E+00_JPRB, 0.23385E+00_JPRB ,&
&  0.23736E+00_JPRB, 0.24059E+00_JPRB, 0.24344E+00_JPRB, 0.24578E+00_JPRB, 0.24754E+00_JPRB, 0.25069E+00_JPRB, 0.25454E+00_JPRB ,&
&  0.25933E+00_JPRB, 0.26536E+00_JPRB, 0.27393E+00_JPRB, 0.28463E+00_JPRB, 0.29441E+00_JPRB, 0.30249E+00_JPRB, 0.30750E+00_JPRB ,&
&  0.31042E+00_JPRB, 0.31264E+00_JPRB, 0.31508E+00_JPRB, 0.31881E+00_JPRB, 0.32275E+00_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.36345E+00_JPRB, 0.35947E+00_JPRB, 0.35377E+00_JPRB, 0.34896E+00_JPRB, 0.34677E+00_JPRB, 0.34502E+00_JPRB, 0.34117E+00_JPRB ,&
&  0.33754E+00_JPRB, 0.33413E+00_JPRB, 0.33048E+00_JPRB, 0.32729E+00_JPRB, 0.32462E+00_JPRB, 0.32256E+00_JPRB, 0.31908E+00_JPRB ,&
&  0.31391E+00_JPRB, 0.30755E+00_JPRB, 0.30070E+00_JPRB, 0.29529E+00_JPRB, 0.28863E+00_JPRB, 0.28176E+00_JPRB, 0.27676E+00_JPRB ,&
&  0.27258E+00_JPRB, 0.26980E+00_JPRB, 0.26899E+00_JPRB, 0.26770E+00_JPRB, 0.26886E+00_JPRB, 0.27231E+00_JPRB, 0.28027E+00_JPRB ,&
&  0.28890E+00_JPRB, 0.26949E+00_JPRB, 0.28243E+00_JPRB, 0.29312E+00_JPRB, 0.31555E+00_JPRB, 0.34592E+00_JPRB, 0.36739E+00_JPRB ,&
&  0.38263E+00_JPRB, 0.39399E+00_JPRB, 0.40209E+00_JPRB, 0.40494E+00_JPRB, 0.40014E+00_JPRB, 0.39657E+00_JPRB, 0.39398E+00_JPRB ,&
&  0.38766E+00_JPRB, 0.37867E+00_JPRB, 0.36660E+00_JPRB, 0.35362E+00_JPRB, 0.34158E+00_JPRB, 0.33269E+00_JPRB, 0.32556E+00_JPRB ,&
&  0.31985E+00_JPRB, 0.31437E+00_JPRB, 0.30949E+00_JPRB, 0.30350E+00_JPRB, 0.29637E+00_JPRB, 0.28895E+00_JPRB, 0.28246E+00_JPRB ,&
&  0.27625E+00_JPRB, 0.27006E+00_JPRB, 0.26455E+00_JPRB, 0.25984E+00_JPRB, 0.25739E+00_JPRB, 0.25644E+00_JPRB, 0.25644E+00_JPRB ,&
&  0.25717E+00_JPRB, 0.25896E+00_JPRB, 0.26051E+00_JPRB, 0.26148E+00_JPRB, 0.26193E+00_JPRB, 0.26109E+00_JPRB, 0.25882E+00_JPRB ,&
&  0.25871E+00_JPRB, 0.25876E+00_JPRB, 0.25871E+00_JPRB, 0.25915E+00_JPRB, 0.25881E+00_JPRB, 0.25893E+00_JPRB, 0.26081E+00_JPRB ,&
&  0.26277E+00_JPRB, 0.26407E+00_JPRB, 0.26562E+00_JPRB, 0.26743E+00_JPRB, 0.26969E+00_JPRB, 0.27321E+00_JPRB, 0.27816E+00_JPRB ,&
&  0.28378E+00_JPRB, 0.29010E+00_JPRB, 0.29927E+00_JPRB, 0.31127E+00_JPRB, 0.32302E+00_JPRB, 0.33351E+00_JPRB, 0.34242E+00_JPRB ,&
&  0.34610E+00_JPRB, 0.34940E+00_JPRB, 0.35503E+00_JPRB, 0.35924E+00_JPRB, 0.36354E+00_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.41170E+00_JPRB, 0.40465E+00_JPRB, 0.39501E+00_JPRB, 0.38847E+00_JPRB, 0.38571E+00_JPRB, 0.38227E+00_JPRB, 0.37859E+00_JPRB ,&
&  0.37286E+00_JPRB, 0.36634E+00_JPRB, 0.36006E+00_JPRB, 0.35457E+00_JPRB, 0.35014E+00_JPRB, 0.34682E+00_JPRB, 0.34285E+00_JPRB ,&
&  0.33738E+00_JPRB, 0.33140E+00_JPRB, 0.32566E+00_JPRB, 0.31938E+00_JPRB, 0.31099E+00_JPRB, 0.30437E+00_JPRB, 0.29910E+00_JPRB ,&
&  0.29493E+00_JPRB, 0.29160E+00_JPRB, 0.29073E+00_JPRB, 0.28946E+00_JPRB, 0.28807E+00_JPRB, 0.28851E+00_JPRB, 0.29727E+00_JPRB ,&
&  0.28977E+00_JPRB, 0.26571E+00_JPRB, 0.27984E+00_JPRB, 0.29864E+00_JPRB, 0.32571E+00_JPRB, 0.36588E+00_JPRB, 0.39395E+00_JPRB ,&
&  0.41005E+00_JPRB, 0.41824E+00_JPRB, 0.42183E+00_JPRB, 0.42052E+00_JPRB, 0.41719E+00_JPRB, 0.41641E+00_JPRB, 0.41124E+00_JPRB ,&
&  0.40099E+00_JPRB, 0.38877E+00_JPRB, 0.37635E+00_JPRB, 0.36721E+00_JPRB, 0.35806E+00_JPRB, 0.35150E+00_JPRB, 0.34440E+00_JPRB ,&
&  0.33704E+00_JPRB, 0.33065E+00_JPRB, 0.32721E+00_JPRB, 0.32468E+00_JPRB, 0.32079E+00_JPRB, 0.31479E+00_JPRB, 0.30886E+00_JPRB ,&
&  0.30438E+00_JPRB, 0.30103E+00_JPRB, 0.29779E+00_JPRB, 0.29444E+00_JPRB, 0.29279E+00_JPRB, 0.29178E+00_JPRB, 0.29010E+00_JPRB ,&
&  0.28939E+00_JPRB, 0.28913E+00_JPRB, 0.28907E+00_JPRB, 0.28880E+00_JPRB, 0.28802E+00_JPRB, 0.28679E+00_JPRB, 0.28735E+00_JPRB ,&
&  0.28914E+00_JPRB, 0.28845E+00_JPRB, 0.28807E+00_JPRB, 0.28982E+00_JPRB, 0.29171E+00_JPRB, 0.29303E+00_JPRB, 0.29331E+00_JPRB ,&
&  0.29261E+00_JPRB, 0.29285E+00_JPRB, 0.29450E+00_JPRB, 0.29781E+00_JPRB, 0.30174E+00_JPRB, 0.30738E+00_JPRB, 0.31440E+00_JPRB ,&
&  0.32185E+00_JPRB, 0.33106E+00_JPRB, 0.34375E+00_JPRB, 0.36257E+00_JPRB, 0.37741E+00_JPRB, 0.38911E+00_JPRB, 0.39841E+00_JPRB ,&
&  0.40047E+00_JPRB, 0.40082E+00_JPRB, 0.40683E+00_JPRB, 0.40949E+00_JPRB, 0.41217E+00_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.43427E+00_JPRB, 0.43647E+00_JPRB, 0.43192E+00_JPRB, 0.42526E+00_JPRB, 0.41890E+00_JPRB, 0.41184E+00_JPRB, 0.40581E+00_JPRB ,&
&  0.39739E+00_JPRB, 0.38757E+00_JPRB, 0.38060E+00_JPRB, 0.37756E+00_JPRB, 0.37585E+00_JPRB, 0.37290E+00_JPRB, 0.36777E+00_JPRB ,&
&  0.36138E+00_JPRB, 0.35498E+00_JPRB, 0.34920E+00_JPRB, 0.34227E+00_JPRB, 0.33564E+00_JPRB, 0.32975E+00_JPRB, 0.32542E+00_JPRB ,&
&  0.32114E+00_JPRB, 0.31637E+00_JPRB, 0.31167E+00_JPRB, 0.30873E+00_JPRB, 0.30623E+00_JPRB, 0.30504E+00_JPRB, 0.31309E+00_JPRB ,&
&  0.33841E+00_JPRB, 0.29181E+00_JPRB, 0.30635E+00_JPRB, 0.32697E+00_JPRB, 0.35625E+00_JPRB, 0.39348E+00_JPRB, 0.41622E+00_JPRB ,&
&  0.42969E+00_JPRB, 0.43871E+00_JPRB, 0.44167E+00_JPRB, 0.44307E+00_JPRB, 0.44482E+00_JPRB, 0.43817E+00_JPRB, 0.42462E+00_JPRB ,&
&  0.41124E+00_JPRB, 0.39801E+00_JPRB, 0.38728E+00_JPRB, 0.37948E+00_JPRB, 0.37386E+00_JPRB, 0.37087E+00_JPRB, 0.36532E+00_JPRB ,&
&  0.35497E+00_JPRB, 0.34948E+00_JPRB, 0.34676E+00_JPRB, 0.34410E+00_JPRB, 0.34247E+00_JPRB, 0.34003E+00_JPRB, 0.33371E+00_JPRB ,&
&  0.32771E+00_JPRB, 0.32449E+00_JPRB, 0.32349E+00_JPRB, 0.32294E+00_JPRB, 0.32309E+00_JPRB, 0.32194E+00_JPRB, 0.31951E+00_JPRB ,&
&  0.31665E+00_JPRB, 0.31292E+00_JPRB, 0.31067E+00_JPRB, 0.30888E+00_JPRB, 0.30746E+00_JPRB, 0.30914E+00_JPRB, 0.31180E+00_JPRB ,&
&  0.31330E+00_JPRB, 0.31500E+00_JPRB, 0.31773E+00_JPRB, 0.31980E+00_JPRB, 0.32026E+00_JPRB, 0.31972E+00_JPRB, 0.31870E+00_JPRB ,&
&  0.31756E+00_JPRB, 0.31729E+00_JPRB, 0.31903E+00_JPRB, 0.32397E+00_JPRB, 0.33064E+00_JPRB, 0.33853E+00_JPRB, 0.34743E+00_JPRB ,&
&  0.35782E+00_JPRB, 0.36915E+00_JPRB, 0.38277E+00_JPRB, 0.40760E+00_JPRB, 0.42141E+00_JPRB, 0.43412E+00_JPRB, 0.44536E+00_JPRB ,&
&  0.44790E+00_JPRB, 0.44664E+00_JPRB, 0.38078E+00_JPRB, 0.41355E+00_JPRB, 0.42463E+00_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.47332E+00_JPRB, 0.47843E+00_JPRB, 0.47231E+00_JPRB, 0.46019E+00_JPRB, 0.44849E+00_JPRB, 0.43741E+00_JPRB, 0.42839E+00_JPRB ,&
&  0.41700E+00_JPRB, 0.40770E+00_JPRB, 0.40423E+00_JPRB, 0.40350E+00_JPRB, 0.40248E+00_JPRB, 0.39856E+00_JPRB, 0.39321E+00_JPRB ,&
&  0.38781E+00_JPRB, 0.38179E+00_JPRB, 0.37473E+00_JPRB, 0.36773E+00_JPRB, 0.36187E+00_JPRB, 0.35539E+00_JPRB, 0.35091E+00_JPRB ,&
&  0.34699E+00_JPRB, 0.34396E+00_JPRB, 0.33850E+00_JPRB, 0.33124E+00_JPRB, 0.32858E+00_JPRB, 0.32421E+00_JPRB, 0.33329E+00_JPRB ,&
&  0.36640E+00_JPRB, 0.31909E+00_JPRB, 0.33683E+00_JPRB, 0.36950E+00_JPRB, 0.39755E+00_JPRB, 0.43224E+00_JPRB, 0.45165E+00_JPRB ,&
&  0.46656E+00_JPRB, 0.47509E+00_JPRB, 0.47885E+00_JPRB, 0.48547E+00_JPRB, 0.48234E+00_JPRB, 0.46563E+00_JPRB, 0.44402E+00_JPRB ,&
&  0.42935E+00_JPRB, 0.41368E+00_JPRB, 0.40392E+00_JPRB, 0.39486E+00_JPRB, 0.39196E+00_JPRB, 0.39210E+00_JPRB, 0.38800E+00_JPRB ,&
&  0.38093E+00_JPRB, 0.37828E+00_JPRB, 0.37460E+00_JPRB, 0.37427E+00_JPRB, 0.37474E+00_JPRB, 0.37642E+00_JPRB, 0.37363E+00_JPRB ,&
&  0.37025E+00_JPRB, 0.37130E+00_JPRB, 0.37337E+00_JPRB, 0.37223E+00_JPRB, 0.37000E+00_JPRB, 0.36662E+00_JPRB, 0.36213E+00_JPRB ,&
&  0.35789E+00_JPRB, 0.35487E+00_JPRB, 0.35423E+00_JPRB, 0.35333E+00_JPRB, 0.35277E+00_JPRB, 0.35288E+00_JPRB, 0.35343E+00_JPRB ,&
&  0.35463E+00_JPRB, 0.35581E+00_JPRB, 0.35630E+00_JPRB, 0.35584E+00_JPRB, 0.35458E+00_JPRB, 0.35183E+00_JPRB, 0.35106E+00_JPRB ,&
&  0.35161E+00_JPRB, 0.35403E+00_JPRB, 0.35845E+00_JPRB, 0.36482E+00_JPRB, 0.37348E+00_JPRB, 0.38155E+00_JPRB, 0.39114E+00_JPRB ,&
&  0.40163E+00_JPRB, 0.41390E+00_JPRB, 0.43106E+00_JPRB, 0.44845E+00_JPRB, 0.47036E+00_JPRB, 0.47716E+00_JPRB, 0.48448E+00_JPRB ,&
&  0.48820E+00_JPRB, 0.48978E+00_JPRB, 0.49179E+00_JPRB, 0.44133E+00_JPRB, 0.46395E+00_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.51542E+00_JPRB, 0.51995E+00_JPRB, 0.50622E+00_JPRB, 0.48811E+00_JPRB, 0.47037E+00_JPRB, 0.45594E+00_JPRB, 0.44350E+00_JPRB ,&
&  0.43506E+00_JPRB, 0.42946E+00_JPRB, 0.42576E+00_JPRB, 0.42351E+00_JPRB, 0.42036E+00_JPRB, 0.41504E+00_JPRB, 0.40963E+00_JPRB ,&
&  0.40548E+00_JPRB, 0.40066E+00_JPRB, 0.39304E+00_JPRB, 0.38800E+00_JPRB, 0.38527E+00_JPRB, 0.38187E+00_JPRB, 0.37872E+00_JPRB ,&
&  0.37477E+00_JPRB, 0.36581E+00_JPRB, 0.35673E+00_JPRB, 0.35143E+00_JPRB, 0.34754E+00_JPRB, 0.34579E+00_JPRB, 0.35463E+00_JPRB ,&
&  0.38999E+00_JPRB, 0.36293E+00_JPRB, 0.38919E+00_JPRB, 0.41617E+00_JPRB, 0.44506E+00_JPRB, 0.46582E+00_JPRB, 0.48534E+00_JPRB ,&
&  0.49875E+00_JPRB, 0.51072E+00_JPRB, 0.52011E+00_JPRB, 0.53177E+00_JPRB, 0.51515E+00_JPRB, 0.49099E+00_JPRB, 0.47128E+00_JPRB ,&
&  0.45298E+00_JPRB, 0.43486E+00_JPRB, 0.42067E+00_JPRB, 0.41169E+00_JPRB, 0.41041E+00_JPRB, 0.41056E+00_JPRB, 0.40875E+00_JPRB ,&
&  0.40780E+00_JPRB, 0.40382E+00_JPRB, 0.39854E+00_JPRB, 0.39926E+00_JPRB, 0.39762E+00_JPRB, 0.40585E+00_JPRB, 0.41410E+00_JPRB ,&
&  0.41824E+00_JPRB, 0.42327E+00_JPRB, 0.42394E+00_JPRB, 0.42267E+00_JPRB, 0.41763E+00_JPRB, 0.41160E+00_JPRB, 0.40513E+00_JPRB ,&
&  0.39979E+00_JPRB, 0.39694E+00_JPRB, 0.39721E+00_JPRB, 0.39784E+00_JPRB, 0.39792E+00_JPRB, 0.39984E+00_JPRB, 0.40167E+00_JPRB ,&
&  0.40358E+00_JPRB, 0.40404E+00_JPRB, 0.40394E+00_JPRB, 0.40006E+00_JPRB, 0.39450E+00_JPRB, 0.39049E+00_JPRB, 0.38734E+00_JPRB ,&
&  0.38586E+00_JPRB, 0.38770E+00_JPRB, 0.39118E+00_JPRB, 0.39864E+00_JPRB, 0.40659E+00_JPRB, 0.41658E+00_JPRB, 0.42446E+00_JPRB ,&
&  0.43204E+00_JPRB, 0.44527E+00_JPRB, 0.46251E+00_JPRB, 0.48414E+00_JPRB, 0.49371E+00_JPRB, 0.49892E+00_JPRB, 0.50545E+00_JPRB ,&
&  0.51055E+00_JPRB, 0.51409E+00_JPRB, 0.50790E+00_JPRB, 0.50693E+00_JPRB, 0.50055E+00_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.59626E+00_JPRB, 0.57469E+00_JPRB, 0.54411E+00_JPRB, 0.51389E+00_JPRB, 0.48583E+00_JPRB, 0.46390E+00_JPRB, 0.45049E+00_JPRB ,&
&  0.44227E+00_JPRB, 0.43703E+00_JPRB, 0.43371E+00_JPRB, 0.43109E+00_JPRB, 0.42731E+00_JPRB, 0.42419E+00_JPRB, 0.41963E+00_JPRB ,&
&  0.41431E+00_JPRB, 0.40791E+00_JPRB, 0.40211E+00_JPRB, 0.39851E+00_JPRB, 0.39509E+00_JPRB, 0.39365E+00_JPRB, 0.39229E+00_JPRB ,&
&  0.38780E+00_JPRB, 0.38160E+00_JPRB, 0.37314E+00_JPRB, 0.36445E+00_JPRB, 0.35985E+00_JPRB, 0.35981E+00_JPRB, 0.37558E+00_JPRB ,&
&  0.42469E+00_JPRB, 0.40966E+00_JPRB, 0.44496E+00_JPRB, 0.47184E+00_JPRB, 0.49454E+00_JPRB, 0.51866E+00_JPRB, 0.54544E+00_JPRB ,&
&  0.56143E+00_JPRB, 0.57508E+00_JPRB, 0.59408E+00_JPRB, 0.59059E+00_JPRB, 0.56007E+00_JPRB, 0.53363E+00_JPRB, 0.50647E+00_JPRB ,&
&  0.47887E+00_JPRB, 0.45338E+00_JPRB, 0.43187E+00_JPRB, 0.42411E+00_JPRB, 0.42427E+00_JPRB, 0.42794E+00_JPRB, 0.42936E+00_JPRB ,&
&  0.43058E+00_JPRB, 0.42867E+00_JPRB, 0.43057E+00_JPRB, 0.44114E+00_JPRB, 0.45386E+00_JPRB, 0.47009E+00_JPRB, 0.49457E+00_JPRB ,&
&  0.50422E+00_JPRB, 0.49833E+00_JPRB, 0.49732E+00_JPRB, 0.49089E+00_JPRB, 0.47587E+00_JPRB, 0.46212E+00_JPRB, 0.45196E+00_JPRB ,&
&  0.44519E+00_JPRB, 0.44218E+00_JPRB, 0.44245E+00_JPRB, 0.44177E+00_JPRB, 0.44253E+00_JPRB, 0.44437E+00_JPRB, 0.44726E+00_JPRB ,&
&  0.44902E+00_JPRB, 0.45060E+00_JPRB, 0.45018E+00_JPRB, 0.44582E+00_JPRB, 0.44348E+00_JPRB, 0.44159E+00_JPRB, 0.43599E+00_JPRB ,&
&  0.42931E+00_JPRB, 0.42713E+00_JPRB, 0.43081E+00_JPRB, 0.43722E+00_JPRB, 0.44654E+00_JPRB, 0.45606E+00_JPRB, 0.46365E+00_JPRB ,&
&  0.47246E+00_JPRB, 0.48784E+00_JPRB, 0.48563E+00_JPRB, 0.48820E+00_JPRB, 0.51486E+00_JPRB, 0.51822E+00_JPRB, 0.52646E+00_JPRB ,&
&  0.53532E+00_JPRB, 0.53921E+00_JPRB, 0.53403E+00_JPRB, 0.52985E+00_JPRB, 0.59262E+00_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.56641E+00_JPRB, 0.54482E+00_JPRB, 0.51915E+00_JPRB, 0.49542E+00_JPRB, 0.47440E+00_JPRB, 0.45858E+00_JPRB, 0.44910E+00_JPRB ,&
&  0.44187E+00_JPRB, 0.43683E+00_JPRB, 0.43707E+00_JPRB, 0.43991E+00_JPRB, 0.43910E+00_JPRB, 0.43238E+00_JPRB, 0.42318E+00_JPRB ,&
&  0.41410E+00_JPRB, 0.40817E+00_JPRB, 0.40407E+00_JPRB, 0.40340E+00_JPRB, 0.40059E+00_JPRB, 0.39787E+00_JPRB, 0.39347E+00_JPRB ,&
&  0.39084E+00_JPRB, 0.38787E+00_JPRB, 0.38254E+00_JPRB, 0.37718E+00_JPRB, 0.37401E+00_JPRB, 0.37497E+00_JPRB, 0.39869E+00_JPRB ,&
&  0.45118E+00_JPRB, 0.48167E+00_JPRB, 0.51293E+00_JPRB, 0.54568E+00_JPRB, 0.56830E+00_JPRB, 0.59261E+00_JPRB, 0.64425E+00_JPRB ,&
&  0.67466E+00_JPRB, 0.68793E+00_JPRB, 0.69541E+00_JPRB, 0.67172E+00_JPRB, 0.62318E+00_JPRB, 0.58420E+00_JPRB, 0.53962E+00_JPRB ,&
&  0.49665E+00_JPRB, 0.46239E+00_JPRB, 0.44099E+00_JPRB, 0.43298E+00_JPRB, 0.43668E+00_JPRB, 0.44420E+00_JPRB, 0.44836E+00_JPRB ,&
&  0.45237E+00_JPRB, 0.45567E+00_JPRB, 0.47177E+00_JPRB, 0.50026E+00_JPRB, 0.54432E+00_JPRB, 0.57770E+00_JPRB, 0.62062E+00_JPRB ,&
&  0.63706E+00_JPRB, 0.62775E+00_JPRB, 0.61187E+00_JPRB, 0.58721E+00_JPRB, 0.56972E+00_JPRB, 0.55031E+00_JPRB, 0.53507E+00_JPRB ,&
&  0.51747E+00_JPRB, 0.50305E+00_JPRB, 0.49270E+00_JPRB, 0.48697E+00_JPRB, 0.48540E+00_JPRB, 0.48606E+00_JPRB, 0.48861E+00_JPRB ,&
&  0.48821E+00_JPRB, 0.48910E+00_JPRB, 0.48974E+00_JPRB, 0.49427E+00_JPRB, 0.50000E+00_JPRB, 0.49580E+00_JPRB, 0.48594E+00_JPRB ,&
&  0.47483E+00_JPRB, 0.46629E+00_JPRB, 0.46762E+00_JPRB, 0.47504E+00_JPRB, 0.48240E+00_JPRB, 0.48695E+00_JPRB, 0.49039E+00_JPRB ,&
&  0.50238E+00_JPRB, 0.51279E+00_JPRB, 0.51872E+00_JPRB, 0.51904E+00_JPRB, 0.52326E+00_JPRB, 0.55276E+00_JPRB, 0.55835E+00_JPRB ,&
&  0.56286E+00_JPRB, 0.56298E+00_JPRB, 0.55359E+00_JPRB, 0.54494E+00_JPRB, 0.57072E+00_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.55793E+00_JPRB, 0.54247E+00_JPRB, 0.51845E+00_JPRB, 0.49759E+00_JPRB, 0.48174E+00_JPRB, 0.46597E+00_JPRB, 0.45531E+00_JPRB ,&
&  0.44757E+00_JPRB, 0.44812E+00_JPRB, 0.45088E+00_JPRB, 0.45232E+00_JPRB, 0.44886E+00_JPRB, 0.43919E+00_JPRB, 0.42942E+00_JPRB ,&
&  0.42367E+00_JPRB, 0.42406E+00_JPRB, 0.42511E+00_JPRB, 0.42313E+00_JPRB, 0.41900E+00_JPRB, 0.41350E+00_JPRB, 0.40858E+00_JPRB ,&
&  0.40432E+00_JPRB, 0.40028E+00_JPRB, 0.39619E+00_JPRB, 0.39419E+00_JPRB, 0.39176E+00_JPRB, 0.39540E+00_JPRB, 0.42128E+00_JPRB ,&
&  0.48146E+00_JPRB, 0.52741E+00_JPRB, 0.64750E+00_JPRB, 0.65375E+00_JPRB, 0.66931E+00_JPRB, 0.71303E+00_JPRB, 0.81492E+00_JPRB ,&
&  0.87891E+00_JPRB, 0.89202E+00_JPRB, 0.85931E+00_JPRB, 0.78683E+00_JPRB, 0.69066E+00_JPRB, 0.61483E+00_JPRB, 0.55689E+00_JPRB ,&
&  0.50891E+00_JPRB, 0.47775E+00_JPRB, 0.46020E+00_JPRB, 0.45565E+00_JPRB, 0.45993E+00_JPRB, 0.46950E+00_JPRB, 0.47454E+00_JPRB ,&
&  0.48428E+00_JPRB, 0.49537E+00_JPRB, 0.52532E+00_JPRB, 0.57691E+00_JPRB, 0.60818E+00_JPRB, 0.60085E+00_JPRB, 0.67517E+00_JPRB ,&
&  0.80767E+00_JPRB, 0.77230E+00_JPRB, 0.73927E+00_JPRB, 0.70553E+00_JPRB, 0.68019E+00_JPRB, 0.65455E+00_JPRB, 0.61758E+00_JPRB ,&
&  0.58865E+00_JPRB, 0.56332E+00_JPRB, 0.54555E+00_JPRB, 0.53736E+00_JPRB, 0.53274E+00_JPRB, 0.53194E+00_JPRB, 0.53121E+00_JPRB ,&
&  0.53140E+00_JPRB, 0.53276E+00_JPRB, 0.53846E+00_JPRB, 0.54102E+00_JPRB, 0.53825E+00_JPRB, 0.52799E+00_JPRB, 0.51424E+00_JPRB ,&
&  0.50469E+00_JPRB, 0.49977E+00_JPRB, 0.49793E+00_JPRB, 0.49494E+00_JPRB, 0.50496E+00_JPRB, 0.51989E+00_JPRB, 0.52854E+00_JPRB ,&
&  0.52820E+00_JPRB, 0.53176E+00_JPRB, 0.54468E+00_JPRB, 0.54598E+00_JPRB, 0.56178E+00_JPRB, 0.59879E+00_JPRB, 0.60211E+00_JPRB ,&
&  0.60664E+00_JPRB, 0.59148E+00_JPRB, 0.57397E+00_JPRB, 0.56193E+00_JPRB, 0.56312E+00_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.55601E+00_JPRB, 0.53454E+00_JPRB, 0.51735E+00_JPRB, 0.50082E+00_JPRB, 0.48923E+00_JPRB, 0.47809E+00_JPRB, 0.47089E+00_JPRB ,&
&  0.46907E+00_JPRB, 0.47059E+00_JPRB, 0.47026E+00_JPRB, 0.46536E+00_JPRB, 0.45631E+00_JPRB, 0.44955E+00_JPRB, 0.44557E+00_JPRB ,&
&  0.44157E+00_JPRB, 0.44187E+00_JPRB, 0.44122E+00_JPRB, 0.43430E+00_JPRB, 0.42669E+00_JPRB, 0.42176E+00_JPRB, 0.41683E+00_JPRB ,&
&  0.41322E+00_JPRB, 0.41133E+00_JPRB, 0.40992E+00_JPRB, 0.40916E+00_JPRB, 0.40937E+00_JPRB, 0.41723E+00_JPRB, 0.43976E+00_JPRB ,&
&  0.48433E+00_JPRB, 0.42748E+00_JPRB, 0.71606E+00_JPRB, 0.80469E+00_JPRB, 0.85084E+00_JPRB, 0.91638E+00_JPRB, 0.10493E+01_JPRB ,&
&  0.11922E+01_JPRB, 0.11676E+01_JPRB, 0.10423E+01_JPRB, 0.88454E+00_JPRB, 0.73646E+00_JPRB, 0.63166E+00_JPRB, 0.57559E+00_JPRB ,&
&  0.53637E+00_JPRB, 0.50898E+00_JPRB, 0.49718E+00_JPRB, 0.49096E+00_JPRB, 0.49465E+00_JPRB, 0.50252E+00_JPRB, 0.50822E+00_JPRB ,&
&  0.52008E+00_JPRB, 0.53308E+00_JPRB, 0.56704E+00_JPRB, 0.61934E+00_JPRB, 0.62615E+00_JPRB, 0.65171E+00_JPRB, 0.70577E+00_JPRB ,&
&  0.81724E+00_JPRB, 0.90112E+00_JPRB, 0.83927E+00_JPRB, 0.78947E+00_JPRB, 0.75445E+00_JPRB, 0.72007E+00_JPRB, 0.68167E+00_JPRB ,&
&  0.64753E+00_JPRB, 0.61426E+00_JPRB, 0.59315E+00_JPRB, 0.57779E+00_JPRB, 0.56801E+00_JPRB, 0.56316E+00_JPRB, 0.56070E+00_JPRB ,&
&  0.55922E+00_JPRB, 0.55881E+00_JPRB, 0.56086E+00_JPRB, 0.56046E+00_JPRB, 0.55729E+00_JPRB, 0.54551E+00_JPRB, 0.52952E+00_JPRB ,&
&  0.52264E+00_JPRB, 0.52334E+00_JPRB, 0.52114E+00_JPRB, 0.52809E+00_JPRB, 0.54029E+00_JPRB, 0.54668E+00_JPRB, 0.55493E+00_JPRB ,&
&  0.56871E+00_JPRB, 0.57507E+00_JPRB, 0.57860E+00_JPRB, 0.58691E+00_JPRB, 0.58931E+00_JPRB, 0.62814E+00_JPRB, 0.61682E+00_JPRB ,&
&  0.61443E+00_JPRB, 0.60168E+00_JPRB, 0.58211E+00_JPRB, 0.57252E+00_JPRB, 0.56821E+00_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.55570E+00_JPRB, 0.53601E+00_JPRB, 0.52214E+00_JPRB, 0.51134E+00_JPRB, 0.50288E+00_JPRB, 0.49644E+00_JPRB, 0.49615E+00_JPRB ,&
&  0.49638E+00_JPRB, 0.49396E+00_JPRB, 0.48355E+00_JPRB, 0.47059E+00_JPRB, 0.46687E+00_JPRB, 0.46795E+00_JPRB, 0.46114E+00_JPRB ,&
&  0.46022E+00_JPRB, 0.46094E+00_JPRB, 0.45597E+00_JPRB, 0.44731E+00_JPRB, 0.44043E+00_JPRB, 0.43699E+00_JPRB, 0.43684E+00_JPRB ,&
&  0.43118E+00_JPRB, 0.42955E+00_JPRB, 0.43255E+00_JPRB, 0.43491E+00_JPRB, 0.43805E+00_JPRB, 0.45365E+00_JPRB, 0.46304E+00_JPRB ,&
&  0.49364E+00_JPRB, 0.59045E+00_JPRB, 0.64394E+00_JPRB, 0.88706E+00_JPRB, 0.10273E+01_JPRB, 0.11650E+01_JPRB, 0.12349E+01_JPRB ,&
&  0.12974E+01_JPRB, 0.13755E+01_JPRB, 0.11709E+01_JPRB, 0.96008E+00_JPRB, 0.76910E+00_JPRB, 0.66357E+00_JPRB, 0.61153E+00_JPRB ,&
&  0.58492E+00_JPRB, 0.55896E+00_JPRB, 0.54135E+00_JPRB, 0.53721E+00_JPRB, 0.54044E+00_JPRB, 0.54167E+00_JPRB, 0.54149E+00_JPRB ,&
&  0.55111E+00_JPRB, 0.56878E+00_JPRB, 0.61697E+00_JPRB, 0.66942E+00_JPRB, 0.67822E+00_JPRB, 0.75081E+00_JPRB, 0.81739E+00_JPRB ,&
&  0.89134E+00_JPRB, 0.10102E+01_JPRB, 0.95283E+00_JPRB, 0.88147E+00_JPRB, 0.81676E+00_JPRB, 0.75599E+00_JPRB, 0.70519E+00_JPRB ,&
&  0.66677E+00_JPRB, 0.63509E+00_JPRB, 0.61241E+00_JPRB, 0.59570E+00_JPRB, 0.58467E+00_JPRB, 0.57884E+00_JPRB, 0.57196E+00_JPRB ,&
&  0.56299E+00_JPRB, 0.56116E+00_JPRB, 0.56500E+00_JPRB, 0.56803E+00_JPRB, 0.56512E+00_JPRB, 0.55797E+00_JPRB, 0.55103E+00_JPRB ,&
&  0.55007E+00_JPRB, 0.55634E+00_JPRB, 0.55608E+00_JPRB, 0.57609E+00_JPRB, 0.58640E+00_JPRB, 0.57359E+00_JPRB, 0.57181E+00_JPRB ,&
&  0.59900E+00_JPRB, 0.61131E+00_JPRB, 0.61227E+00_JPRB, 0.61230E+00_JPRB, 0.63178E+00_JPRB, 0.67388E+00_JPRB, 0.62670E+00_JPRB ,&
&  0.60264E+00_JPRB, 0.59439E+00_JPRB, 0.58157E+00_JPRB, 0.57390E+00_JPRB, 0.56707E+00_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.56150E+00_JPRB, 0.54861E+00_JPRB, 0.53860E+00_JPRB, 0.52771E+00_JPRB, 0.51934E+00_JPRB, 0.51777E+00_JPRB, 0.51635E+00_JPRB ,&
&  0.51163E+00_JPRB, 0.49508E+00_JPRB, 0.47727E+00_JPRB, 0.47160E+00_JPRB, 0.47166E+00_JPRB, 0.46590E+00_JPRB, 0.46983E+00_JPRB ,&
&  0.47304E+00_JPRB, 0.47614E+00_JPRB, 0.46773E+00_JPRB, 0.46215E+00_JPRB, 0.45876E+00_JPRB, 0.45698E+00_JPRB, 0.45851E+00_JPRB ,&
&  0.46040E+00_JPRB, 0.46193E+00_JPRB, 0.46719E+00_JPRB, 0.47249E+00_JPRB, 0.47988E+00_JPRB, 0.49595E+00_JPRB, 0.50883E+00_JPRB ,&
&  0.57914E+00_JPRB, 0.62202E+00_JPRB, 0.68555E+00_JPRB, 0.10979E+01_JPRB, 0.13146E+01_JPRB, 0.13806E+01_JPRB, 0.12841E+01_JPRB ,&
&  0.12680E+01_JPRB, 0.13734E+01_JPRB, 0.12460E+01_JPRB, 0.10221E+01_JPRB, 0.84145E+00_JPRB, 0.75811E+00_JPRB, 0.70333E+00_JPRB ,&
&  0.65778E+00_JPRB, 0.61511E+00_JPRB, 0.58832E+00_JPRB, 0.57217E+00_JPRB, 0.57244E+00_JPRB, 0.57091E+00_JPRB, 0.57557E+00_JPRB ,&
&  0.58916E+00_JPRB, 0.62213E+00_JPRB, 0.67893E+00_JPRB, 0.71571E+00_JPRB, 0.81865E+00_JPRB, 0.93326E+00_JPRB, 0.97708E+00_JPRB ,&
&  0.97846E+00_JPRB, 0.10542E+01_JPRB, 0.97471E+00_JPRB, 0.90150E+00_JPRB, 0.81369E+00_JPRB, 0.76991E+00_JPRB, 0.71380E+00_JPRB ,&
&  0.67148E+00_JPRB, 0.63626E+00_JPRB, 0.61065E+00_JPRB, 0.59792E+00_JPRB, 0.58676E+00_JPRB, 0.57840E+00_JPRB, 0.57095E+00_JPRB ,&
&  0.56205E+00_JPRB, 0.56780E+00_JPRB, 0.57470E+00_JPRB, 0.57207E+00_JPRB, 0.57124E+00_JPRB, 0.57340E+00_JPRB, 0.57841E+00_JPRB ,&
&  0.58395E+00_JPRB, 0.58819E+00_JPRB, 0.58980E+00_JPRB, 0.61668E+00_JPRB, 0.62856E+00_JPRB, 0.61551E+00_JPRB, 0.61471E+00_JPRB ,&
&  0.63181E+00_JPRB, 0.64414E+00_JPRB, 0.63967E+00_JPRB, 0.62026E+00_JPRB, 0.63320E+00_JPRB, 0.64776E+00_JPRB, 0.62109E+00_JPRB ,&
&  0.61620E+00_JPRB, 0.60750E+00_JPRB, 0.58704E+00_JPRB, 0.57382E+00_JPRB, 0.56864E+00_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.57633E+00_JPRB, 0.56236E+00_JPRB, 0.54673E+00_JPRB, 0.53549E+00_JPRB, 0.52926E+00_JPRB, 0.52338E+00_JPRB, 0.51194E+00_JPRB ,&
&  0.49580E+00_JPRB, 0.48719E+00_JPRB, 0.46989E+00_JPRB, 0.46804E+00_JPRB, 0.47574E+00_JPRB, 0.48119E+00_JPRB, 0.49022E+00_JPRB ,&
&  0.49538E+00_JPRB, 0.49567E+00_JPRB, 0.49068E+00_JPRB, 0.48564E+00_JPRB, 0.48470E+00_JPRB, 0.48617E+00_JPRB, 0.49526E+00_JPRB ,&
&  0.49600E+00_JPRB, 0.49788E+00_JPRB, 0.50409E+00_JPRB, 0.51224E+00_JPRB, 0.53413E+00_JPRB, 0.57133E+00_JPRB, 0.61949E+00_JPRB ,&
&  0.73485E+00_JPRB, 0.75530E+00_JPRB, 0.79956E+00_JPRB, 0.11463E+01_JPRB, 0.13638E+01_JPRB, 0.13048E+01_JPRB, 0.12642E+01_JPRB ,&
&  0.12310E+01_JPRB, 0.11675E+01_JPRB, 0.11906E+01_JPRB, 0.10443E+01_JPRB, 0.90193E+00_JPRB, 0.82273E+00_JPRB, 0.76143E+00_JPRB ,&
&  0.70181E+00_JPRB, 0.65704E+00_JPRB, 0.62193E+00_JPRB, 0.60444E+00_JPRB, 0.60579E+00_JPRB, 0.61778E+00_JPRB, 0.62634E+00_JPRB ,&
&  0.65580E+00_JPRB, 0.70765E+00_JPRB, 0.78876E+00_JPRB, 0.86600E+00_JPRB, 0.97737E+00_JPRB, 0.10797E+01_JPRB, 0.10477E+01_JPRB ,&
&  0.10193E+01_JPRB, 0.10147E+01_JPRB, 0.96700E+00_JPRB, 0.90008E+00_JPRB, 0.77272E+00_JPRB, 0.77170E+00_JPRB, 0.71117E+00_JPRB ,&
&  0.66407E+00_JPRB, 0.62915E+00_JPRB, 0.61052E+00_JPRB, 0.60211E+00_JPRB, 0.59551E+00_JPRB, 0.58565E+00_JPRB, 0.57861E+00_JPRB ,&
&  0.57411E+00_JPRB, 0.57997E+00_JPRB, 0.58412E+00_JPRB, 0.58183E+00_JPRB, 0.58508E+00_JPRB, 0.59268E+00_JPRB, 0.60044E+00_JPRB ,&
&  0.60748E+00_JPRB, 0.62049E+00_JPRB, 0.63168E+00_JPRB, 0.66676E+00_JPRB, 0.68304E+00_JPRB, 0.68320E+00_JPRB, 0.66517E+00_JPRB ,&
&  0.65358E+00_JPRB, 0.66073E+00_JPRB, 0.66023E+00_JPRB, 0.65640E+00_JPRB, 0.65202E+00_JPRB, 0.65275E+00_JPRB, 0.65171E+00_JPRB ,&
&  0.65214E+00_JPRB, 0.63188E+00_JPRB, 0.60599E+00_JPRB, 0.59340E+00_JPRB, 0.58496E+00_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.58688E+00_JPRB, 0.57093E+00_JPRB, 0.54874E+00_JPRB, 0.52602E+00_JPRB, 0.51465E+00_JPRB, 0.50381E+00_JPRB, 0.49025E+00_JPRB ,&
&  0.48377E+00_JPRB, 0.48081E+00_JPRB, 0.47740E+00_JPRB, 0.48961E+00_JPRB, 0.51176E+00_JPRB, 0.52279E+00_JPRB, 0.52816E+00_JPRB ,&
&  0.53437E+00_JPRB, 0.53625E+00_JPRB, 0.52225E+00_JPRB, 0.51881E+00_JPRB, 0.52696E+00_JPRB, 0.53691E+00_JPRB, 0.54192E+00_JPRB ,&
&  0.54582E+00_JPRB, 0.55352E+00_JPRB, 0.56514E+00_JPRB, 0.58716E+00_JPRB, 0.63760E+00_JPRB, 0.71231E+00_JPRB, 0.81116E+00_JPRB ,&
&  0.92979E+00_JPRB, 0.89675E+00_JPRB, 0.55450E+00_JPRB, 0.89112E+00_JPRB, 0.11855E+01_JPRB, 0.11922E+01_JPRB, 0.11427E+01_JPRB ,&
&  0.11155E+01_JPRB, 0.10600E+01_JPRB, 0.10690E+01_JPRB, 0.10151E+01_JPRB, 0.92045E+00_JPRB, 0.83842E+00_JPRB, 0.76774E+00_JPRB ,&
&  0.70742E+00_JPRB, 0.66714E+00_JPRB, 0.64739E+00_JPRB, 0.65209E+00_JPRB, 0.66376E+00_JPRB, 0.66742E+00_JPRB, 0.68933E+00_JPRB ,&
&  0.74004E+00_JPRB, 0.80443E+00_JPRB, 0.92320E+00_JPRB, 0.97889E+00_JPRB, 0.10879E+01_JPRB, 0.10860E+01_JPRB, 0.97417E+00_JPRB ,&
&  0.87026E+00_JPRB, 0.91355E+00_JPRB, 0.93115E+00_JPRB, 0.85992E+00_JPRB, 0.74611E+00_JPRB, 0.69641E+00_JPRB, 0.69379E+00_JPRB ,&
&  0.65136E+00_JPRB, 0.62261E+00_JPRB, 0.61082E+00_JPRB, 0.60307E+00_JPRB, 0.59341E+00_JPRB, 0.58457E+00_JPRB, 0.57815E+00_JPRB ,&
&  0.58231E+00_JPRB, 0.58614E+00_JPRB, 0.58757E+00_JPRB, 0.58850E+00_JPRB, 0.59399E+00_JPRB, 0.60143E+00_JPRB, 0.61309E+00_JPRB ,&
&  0.63315E+00_JPRB, 0.65885E+00_JPRB, 0.69883E+00_JPRB, 0.71725E+00_JPRB, 0.72137E+00_JPRB, 0.71957E+00_JPRB, 0.70703E+00_JPRB ,&
&  0.69595E+00_JPRB, 0.69313E+00_JPRB, 0.68468E+00_JPRB, 0.66645E+00_JPRB, 0.70895E+00_JPRB, 0.72493E+00_JPRB, 0.76218E+00_JPRB ,&
&  0.79659E+00_JPRB, 0.80576E+00_JPRB, 0.75393E+00_JPRB, 0.62608E+00_JPRB, 0.60832E+00_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.55076E+00_JPRB, 0.53263E+00_JPRB, 0.50181E+00_JPRB, 0.49505E+00_JPRB, 0.49443E+00_JPRB, 0.49405E+00_JPRB, 0.50757E+00_JPRB ,&
&  0.51905E+00_JPRB, 0.53623E+00_JPRB, 0.54973E+00_JPRB, 0.57001E+00_JPRB, 0.58144E+00_JPRB, 0.58132E+00_JPRB, 0.58726E+00_JPRB ,&
&  0.58776E+00_JPRB, 0.58701E+00_JPRB, 0.58771E+00_JPRB, 0.59494E+00_JPRB, 0.60620E+00_JPRB, 0.61107E+00_JPRB, 0.61745E+00_JPRB ,&
&  0.63216E+00_JPRB, 0.64884E+00_JPRB, 0.67232E+00_JPRB, 0.72809E+00_JPRB, 0.82278E+00_JPRB, 0.93628E+00_JPRB, 0.10479E+01_JPRB ,&
&  0.10889E+01_JPRB, 0.82414E+00_JPRB, 0.64632E+00_JPRB, 0.10644E+01_JPRB, 0.11837E+01_JPRB, 0.11637E+01_JPRB, 0.10446E+01_JPRB ,&
&  0.10340E+01_JPRB, 0.10118E+01_JPRB, 0.99996E+00_JPRB, 0.97864E+00_JPRB, 0.86790E+00_JPRB, 0.80227E+00_JPRB, 0.75014E+00_JPRB ,&
&  0.71858E+00_JPRB, 0.69795E+00_JPRB, 0.69465E+00_JPRB, 0.70660E+00_JPRB, 0.72060E+00_JPRB, 0.73768E+00_JPRB, 0.78521E+00_JPRB ,&
&  0.84534E+00_JPRB, 0.91866E+00_JPRB, 0.10297E+01_JPRB, 0.10010E+01_JPRB, 0.10597E+01_JPRB, 0.96918E+00_JPRB, 0.88681E+00_JPRB ,&
&  0.80961E+00_JPRB, 0.83984E+00_JPRB, 0.87693E+00_JPRB, 0.82410E+00_JPRB, 0.73059E+00_JPRB, 0.66819E+00_JPRB, 0.67104E+00_JPRB ,&
&  0.62955E+00_JPRB, 0.61100E+00_JPRB, 0.59782E+00_JPRB, 0.58487E+00_JPRB, 0.58296E+00_JPRB, 0.58577E+00_JPRB, 0.58930E+00_JPRB ,&
&  0.59314E+00_JPRB, 0.59127E+00_JPRB, 0.59061E+00_JPRB, 0.59781E+00_JPRB, 0.60967E+00_JPRB, 0.61827E+00_JPRB, 0.63564E+00_JPRB ,&
&  0.65936E+00_JPRB, 0.69000E+00_JPRB, 0.72566E+00_JPRB, 0.75070E+00_JPRB, 0.74835E+00_JPRB, 0.72533E+00_JPRB, 0.73014E+00_JPRB ,&
&  0.73076E+00_JPRB, 0.75108E+00_JPRB, 0.75656E+00_JPRB, 0.77056E+00_JPRB, 0.77982E+00_JPRB, 0.80267E+00_JPRB, 0.81150E+00_JPRB ,&
&  0.81174E+00_JPRB, 0.78767E+00_JPRB, 0.70545E+00_JPRB, 0.59932E+00_JPRB, 0.57591E+00_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.49744E+00_JPRB, 0.49203E+00_JPRB, 0.49485E+00_JPRB, 0.50242E+00_JPRB, 0.51432E+00_JPRB, 0.53838E+00_JPRB, 0.56523E+00_JPRB ,&
&  0.58740E+00_JPRB, 0.62448E+00_JPRB, 0.64964E+00_JPRB, 0.65129E+00_JPRB, 0.64568E+00_JPRB, 0.64682E+00_JPRB, 0.65159E+00_JPRB ,&
&  0.66154E+00_JPRB, 0.66894E+00_JPRB, 0.67400E+00_JPRB, 0.68098E+00_JPRB, 0.69141E+00_JPRB, 0.70256E+00_JPRB, 0.71736E+00_JPRB ,&
&  0.74156E+00_JPRB, 0.78219E+00_JPRB, 0.83966E+00_JPRB, 0.92332E+00_JPRB, 0.10337E+01_JPRB, 0.11483E+01_JPRB, 0.12042E+01_JPRB ,&
&  0.91592E+00_JPRB, 0.54466E+00_JPRB, 0.10346E+01_JPRB, 0.12041E+01_JPRB, 0.11072E+01_JPRB, 0.10650E+01_JPRB, 0.10255E+01_JPRB ,&
&  0.96759E+00_JPRB, 0.96509E+00_JPRB, 0.93575E+00_JPRB, 0.93909E+00_JPRB, 0.83844E+00_JPRB, 0.78962E+00_JPRB, 0.76338E+00_JPRB ,&
&  0.75117E+00_JPRB, 0.74426E+00_JPRB, 0.74751E+00_JPRB, 0.75699E+00_JPRB, 0.78056E+00_JPRB, 0.81667E+00_JPRB, 0.87174E+00_JPRB ,&
&  0.92809E+00_JPRB, 0.98334E+00_JPRB, 0.10722E+01_JPRB, 0.95091E+00_JPRB, 0.96256E+00_JPRB, 0.89092E+00_JPRB, 0.81929E+00_JPRB ,&
&  0.78811E+00_JPRB, 0.76799E+00_JPRB, 0.77512E+00_JPRB, 0.79405E+00_JPRB, 0.72851E+00_JPRB, 0.64398E+00_JPRB, 0.65174E+00_JPRB ,&
&  0.61602E+00_JPRB, 0.60415E+00_JPRB, 0.58987E+00_JPRB, 0.57430E+00_JPRB, 0.56830E+00_JPRB, 0.57337E+00_JPRB, 0.58380E+00_JPRB ,&
&  0.58914E+00_JPRB, 0.59214E+00_JPRB, 0.59931E+00_JPRB, 0.60720E+00_JPRB, 0.62384E+00_JPRB, 0.63894E+00_JPRB, 0.65152E+00_JPRB ,&
&  0.68315E+00_JPRB, 0.71498E+00_JPRB, 0.74626E+00_JPRB, 0.77470E+00_JPRB, 0.79566E+00_JPRB, 0.77837E+00_JPRB, 0.76699E+00_JPRB ,&
&  0.77763E+00_JPRB, 0.78981E+00_JPRB, 0.80553E+00_JPRB, 0.82743E+00_JPRB, 0.83193E+00_JPRB, 0.80217E+00_JPRB, 0.79279E+00_JPRB ,&
&  0.73516E+00_JPRB, 0.66516E+00_JPRB, 0.58332E+00_JPRB, 0.53487E+00_JPRB, 0.51744E+00_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.52289E+00_JPRB, 0.53623E+00_JPRB, 0.55213E+00_JPRB, 0.57212E+00_JPRB, 0.60071E+00_JPRB, 0.63315E+00_JPRB, 0.67437E+00_JPRB ,&
&  0.71583E+00_JPRB, 0.72611E+00_JPRB, 0.72757E+00_JPRB, 0.72795E+00_JPRB, 0.72726E+00_JPRB, 0.73106E+00_JPRB, 0.73863E+00_JPRB ,&
&  0.74686E+00_JPRB, 0.75603E+00_JPRB, 0.76596E+00_JPRB, 0.77664E+00_JPRB, 0.79060E+00_JPRB, 0.81088E+00_JPRB, 0.83925E+00_JPRB ,&
&  0.86936E+00_JPRB, 0.92539E+00_JPRB, 0.10013E+01_JPRB, 0.10863E+01_JPRB, 0.11733E+01_JPRB, 0.12396E+01_JPRB, 0.12460E+01_JPRB ,&
&  0.78846E+00_JPRB, 0.10180E+01_JPRB, 0.11873E+01_JPRB, 0.11310E+01_JPRB, 0.10522E+01_JPRB, 0.99909E+00_JPRB, 0.98252E+00_JPRB ,&
&  0.93964E+00_JPRB, 0.88345E+00_JPRB, 0.86903E+00_JPRB, 0.91327E+00_JPRB, 0.83963E+00_JPRB, 0.79934E+00_JPRB, 0.79058E+00_JPRB ,&
&  0.79166E+00_JPRB, 0.79725E+00_JPRB, 0.81119E+00_JPRB, 0.82474E+00_JPRB, 0.85381E+00_JPRB, 0.89704E+00_JPRB, 0.94226E+00_JPRB ,&
&  0.99896E+00_JPRB, 0.10465E+01_JPRB, 0.11000E+01_JPRB, 0.94629E+00_JPRB, 0.90989E+00_JPRB, 0.85361E+00_JPRB, 0.77191E+00_JPRB ,&
&  0.72469E+00_JPRB, 0.73139E+00_JPRB, 0.75327E+00_JPRB, 0.77417E+00_JPRB, 0.71863E+00_JPRB, 0.66936E+00_JPRB, 0.63103E+00_JPRB ,&
&  0.60744E+00_JPRB, 0.58655E+00_JPRB, 0.56580E+00_JPRB, 0.55774E+00_JPRB, 0.55400E+00_JPRB, 0.56163E+00_JPRB, 0.57716E+00_JPRB ,&
&  0.58775E+00_JPRB, 0.59146E+00_JPRB, 0.60139E+00_JPRB, 0.61385E+00_JPRB, 0.64367E+00_JPRB, 0.66169E+00_JPRB, 0.67273E+00_JPRB ,&
&  0.70008E+00_JPRB, 0.71888E+00_JPRB, 0.74549E+00_JPRB, 0.77375E+00_JPRB, 0.79880E+00_JPRB, 0.81581E+00_JPRB, 0.81406E+00_JPRB ,&
&  0.81714E+00_JPRB, 0.84408E+00_JPRB, 0.87474E+00_JPRB, 0.88273E+00_JPRB, 0.86108E+00_JPRB, 0.78395E+00_JPRB, 0.71152E+00_JPRB ,&
&  0.63078E+00_JPRB, 0.60770E+00_JPRB, 0.57277E+00_JPRB, 0.54771E+00_JPRB, 0.53020E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.61686E+00_JPRB, 0.63155E+00_JPRB, 0.69666E+00_JPRB, 0.71469E+00_JPRB, 0.77605E+00_JPRB, 0.79796E+00_JPRB, 0.81845E+00_JPRB ,&
&  0.81696E+00_JPRB, 0.81572E+00_JPRB, 0.81303E+00_JPRB, 0.81345E+00_JPRB, 0.81318E+00_JPRB, 0.81806E+00_JPRB, 0.82228E+00_JPRB ,&
&  0.83161E+00_JPRB, 0.84554E+00_JPRB, 0.86245E+00_JPRB, 0.87972E+00_JPRB, 0.90333E+00_JPRB, 0.93087E+00_JPRB, 0.96400E+00_JPRB ,&
&  0.10032E+01_JPRB, 0.10516E+01_JPRB, 0.11057E+01_JPRB, 0.11715E+01_JPRB, 0.12404E+01_JPRB, 0.12876E+01_JPRB, 0.12802E+01_JPRB ,&
&  0.96757E+00_JPRB, 0.11783E+01_JPRB, 0.11564E+01_JPRB, 0.10984E+01_JPRB, 0.10170E+01_JPRB, 0.95267E+00_JPRB, 0.92391E+00_JPRB ,&
&  0.90109E+00_JPRB, 0.84775E+00_JPRB, 0.84044E+00_JPRB, 0.86101E+00_JPRB, 0.85113E+00_JPRB, 0.81575E+00_JPRB, 0.82408E+00_JPRB ,&
&  0.84206E+00_JPRB, 0.86375E+00_JPRB, 0.88035E+00_JPRB, 0.89168E+00_JPRB, 0.92659E+00_JPRB, 0.96631E+00_JPRB, 0.10115E+01_JPRB ,&
&  0.10692E+01_JPRB, 0.11146E+01_JPRB, 0.11623E+01_JPRB, 0.10265E+01_JPRB, 0.94018E+00_JPRB, 0.88568E+00_JPRB, 0.74720E+00_JPRB ,&
&  0.70433E+00_JPRB, 0.69020E+00_JPRB, 0.74649E+00_JPRB, 0.78129E+00_JPRB, 0.72373E+00_JPRB, 0.65428E+00_JPRB, 0.61353E+00_JPRB ,&
&  0.58093E+00_JPRB, 0.55834E+00_JPRB, 0.55086E+00_JPRB, 0.53974E+00_JPRB, 0.53442E+00_JPRB, 0.54881E+00_JPRB, 0.56380E+00_JPRB ,&
&  0.58720E+00_JPRB, 0.61578E+00_JPRB, 0.64057E+00_JPRB, 0.66981E+00_JPRB, 0.68843E+00_JPRB, 0.71098E+00_JPRB, 0.72368E+00_JPRB ,&
&  0.74268E+00_JPRB, 0.77694E+00_JPRB, 0.78395E+00_JPRB, 0.81234E+00_JPRB, 0.81983E+00_JPRB, 0.84707E+00_JPRB, 0.87387E+00_JPRB ,&
&  0.90269E+00_JPRB, 0.92894E+00_JPRB, 0.93550E+00_JPRB, 0.91875E+00_JPRB, 0.86721E+00_JPRB, 0.79231E+00_JPRB, 0.63333E+00_JPRB ,&
&  0.60636E+00_JPRB, 0.58772E+00_JPRB, 0.59105E+00_JPRB, 0.59379E+00_JPRB, 0.60994E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.77384E+00_JPRB, 0.82051E+00_JPRB, 0.86672E+00_JPRB, 0.92256E+00_JPRB, 0.96523E+00_JPRB, 0.94752E+00_JPRB, 0.91623E+00_JPRB ,&
&  0.89581E+00_JPRB, 0.88552E+00_JPRB, 0.88926E+00_JPRB, 0.90024E+00_JPRB, 0.90805E+00_JPRB, 0.91044E+00_JPRB, 0.91824E+00_JPRB ,&
&  0.93195E+00_JPRB, 0.95150E+00_JPRB, 0.97192E+00_JPRB, 0.99489E+00_JPRB, 0.10115E+01_JPRB, 0.10353E+01_JPRB, 0.10638E+01_JPRB ,&
&  0.10921E+01_JPRB, 0.11308E+01_JPRB, 0.11640E+01_JPRB, 0.11994E+01_JPRB, 0.12333E+01_JPRB, 0.12231E+01_JPRB, 0.91481E+00_JPRB ,&
&  0.11425E+01_JPRB, 0.11807E+01_JPRB, 0.11884E+01_JPRB, 0.11233E+01_JPRB, 0.10194E+01_JPRB, 0.94630E+00_JPRB, 0.91490E+00_JPRB ,&
&  0.88790E+00_JPRB, 0.85838E+00_JPRB, 0.83404E+00_JPRB, 0.82148E+00_JPRB, 0.88217E+00_JPRB, 0.84685E+00_JPRB, 0.86408E+00_JPRB ,&
&  0.88562E+00_JPRB, 0.89772E+00_JPRB, 0.90535E+00_JPRB, 0.92260E+00_JPRB, 0.97109E+00_JPRB, 0.10139E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.11223E+01_JPRB, 0.11740E+01_JPRB, 0.12217E+01_JPRB, 0.10794E+01_JPRB, 0.10673E+01_JPRB, 0.10326E+01_JPRB, 0.83084E+00_JPRB ,&
&  0.71951E+00_JPRB, 0.70689E+00_JPRB, 0.77027E+00_JPRB, 0.79824E+00_JPRB, 0.73075E+00_JPRB, 0.64494E+00_JPRB, 0.61082E+00_JPRB ,&
&  0.57326E+00_JPRB, 0.55432E+00_JPRB, 0.54939E+00_JPRB, 0.53050E+00_JPRB, 0.52207E+00_JPRB, 0.54340E+00_JPRB, 0.57496E+00_JPRB ,&
&  0.61462E+00_JPRB, 0.66353E+00_JPRB, 0.70170E+00_JPRB, 0.73390E+00_JPRB, 0.77731E+00_JPRB, 0.85097E+00_JPRB, 0.89227E+00_JPRB ,&
&  0.82237E+00_JPRB, 0.79804E+00_JPRB, 0.85903E+00_JPRB, 0.90791E+00_JPRB, 0.96000E+00_JPRB, 0.10391E+01_JPRB, 0.11209E+01_JPRB ,&
&  0.12469E+01_JPRB, 0.13199E+01_JPRB, 0.11231E+01_JPRB, 0.94570E+00_JPRB, 0.12711E+01_JPRB, 0.13515E+01_JPRB, 0.68107E+00_JPRB ,&
&  0.64817E+00_JPRB, 0.66490E+00_JPRB, 0.67374E+00_JPRB, 0.70477E+00_JPRB, 0.73757E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.90660E+00_JPRB, 0.93157E+00_JPRB, 0.95907E+00_JPRB, 0.97352E+00_JPRB, 0.97845E+00_JPRB, 0.96047E+00_JPRB, 0.94480E+00_JPRB ,&
&  0.94173E+00_JPRB, 0.94653E+00_JPRB, 0.95828E+00_JPRB, 0.97158E+00_JPRB, 0.97813E+00_JPRB, 0.98344E+00_JPRB, 0.99799E+00_JPRB ,&
&  0.10169E+01_JPRB, 0.10415E+01_JPRB, 0.10720E+01_JPRB, 0.10905E+01_JPRB, 0.11080E+01_JPRB, 0.11191E+01_JPRB, 0.11392E+01_JPRB ,&
&  0.11539E+01_JPRB, 0.11878E+01_JPRB, 0.11885E+01_JPRB, 0.12067E+01_JPRB, 0.12138E+01_JPRB, 0.12368E+01_JPRB, 0.99977E+00_JPRB ,&
&  0.11578E+01_JPRB, 0.12110E+01_JPRB, 0.11870E+01_JPRB, 0.11145E+01_JPRB, 0.10113E+01_JPRB, 0.95847E+00_JPRB, 0.92293E+00_JPRB ,&
&  0.90541E+00_JPRB, 0.88115E+00_JPRB, 0.84656E+00_JPRB, 0.88514E+00_JPRB, 0.90604E+00_JPRB, 0.86128E+00_JPRB, 0.86667E+00_JPRB ,&
&  0.87674E+00_JPRB, 0.88482E+00_JPRB, 0.90138E+00_JPRB, 0.93702E+00_JPRB, 0.99181E+00_JPRB, 0.10383E+01_JPRB, 0.10910E+01_JPRB ,&
&  0.11475E+01_JPRB, 0.12162E+01_JPRB, 0.12837E+01_JPRB, 0.11707E+01_JPRB, 0.12104E+01_JPRB, 0.11784E+01_JPRB, 0.97269E+00_JPRB ,&
&  0.74051E+00_JPRB, 0.68758E+00_JPRB, 0.73451E+00_JPRB, 0.81769E+00_JPRB, 0.74548E+00_JPRB, 0.64759E+00_JPRB, 0.62926E+00_JPRB ,&
&  0.59553E+00_JPRB, 0.56836E+00_JPRB, 0.56036E+00_JPRB, 0.53709E+00_JPRB, 0.52399E+00_JPRB, 0.55036E+00_JPRB, 0.57710E+00_JPRB ,&
&  0.61200E+00_JPRB, 0.65806E+00_JPRB, 0.70782E+00_JPRB, 0.75704E+00_JPRB, 0.81381E+00_JPRB, 0.82985E+00_JPRB, 0.78358E+00_JPRB ,&
&  0.83079E+00_JPRB, 0.88061E+00_JPRB, 0.94653E+00_JPRB, 0.10589E+01_JPRB, 0.11430E+01_JPRB, 0.13638E+01_JPRB, 0.16036E+01_JPRB ,&
&  0.18259E+01_JPRB, 0.17206E+01_JPRB, 0.20523E+01_JPRB, 0.28591E+01_JPRB, 0.26351E+01_JPRB, 0.21166E+01_JPRB, 0.78374E+00_JPRB ,&
&  0.77221E+00_JPRB, 0.76988E+00_JPRB, 0.80036E+00_JPRB, 0.83857E+00_JPRB, 0.88316E+00_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.90069E+00_JPRB, 0.92097E+00_JPRB, 0.94038E+00_JPRB, 0.95829E+00_JPRB, 0.97073E+00_JPRB, 0.97895E+00_JPRB, 0.98374E+00_JPRB ,&
&  0.99770E+00_JPRB, 0.10125E+01_JPRB, 0.10237E+01_JPRB, 0.10297E+01_JPRB, 0.10349E+01_JPRB, 0.10504E+01_JPRB, 0.10731E+01_JPRB ,&
&  0.11021E+01_JPRB, 0.11144E+01_JPRB, 0.11454E+01_JPRB, 0.11232E+01_JPRB, 0.11002E+01_JPRB, 0.11005E+01_JPRB, 0.11179E+01_JPRB ,&
&  0.11178E+01_JPRB, 0.11044E+01_JPRB, 0.10941E+01_JPRB, 0.11156E+01_JPRB, 0.11414E+01_JPRB, 0.11707E+01_JPRB, 0.97464E+00_JPRB ,&
&  0.11138E+01_JPRB, 0.11952E+01_JPRB, 0.11538E+01_JPRB, 0.10836E+01_JPRB, 0.97427E+00_JPRB, 0.93766E+00_JPRB, 0.92136E+00_JPRB ,&
&  0.90733E+00_JPRB, 0.89716E+00_JPRB, 0.88114E+00_JPRB, 0.86791E+00_JPRB, 0.86243E+00_JPRB, 0.84875E+00_JPRB, 0.84700E+00_JPRB ,&
&  0.85275E+00_JPRB, 0.85854E+00_JPRB, 0.87857E+00_JPRB, 0.91087E+00_JPRB, 0.96074E+00_JPRB, 0.10125E+01_JPRB, 0.10482E+01_JPRB ,&
&  0.10975E+01_JPRB, 0.11560E+01_JPRB, 0.11153E+01_JPRB, 0.11268E+01_JPRB, 0.12230E+01_JPRB, 0.11847E+01_JPRB, 0.99193E+00_JPRB ,&
&  0.73641E+00_JPRB, 0.70606E+00_JPRB, 0.73667E+00_JPRB, 0.82315E+00_JPRB, 0.75479E+00_JPRB, 0.68447E+00_JPRB, 0.65696E+00_JPRB ,&
&  0.59571E+00_JPRB, 0.53528E+00_JPRB, 0.54540E+00_JPRB, 0.53999E+00_JPRB, 0.52889E+00_JPRB, 0.54696E+00_JPRB, 0.56597E+00_JPRB ,&
&  0.58574E+00_JPRB, 0.62528E+00_JPRB, 0.66349E+00_JPRB, 0.70031E+00_JPRB, 0.71652E+00_JPRB, 0.67925E+00_JPRB, 0.77183E+00_JPRB ,&
&  0.77649E+00_JPRB, 0.77106E+00_JPRB, 0.81487E+00_JPRB, 0.70786E+00_JPRB, 0.10010E+01_JPRB, 0.10705E+01_JPRB, 0.11899E+01_JPRB ,&
&  0.11620E+01_JPRB, 0.15746E+01_JPRB, 0.19545E+01_JPRB, 0.15154E+01_JPRB, 0.11773E+01_JPRB, 0.84263E+00_JPRB, 0.78466E+00_JPRB ,&
&  0.78651E+00_JPRB, 0.79790E+00_JPRB, 0.82140E+00_JPRB, 0.85077E+00_JPRB, 0.87687E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.89935E+00_JPRB, 0.91819E+00_JPRB, 0.93628E+00_JPRB, 0.95618E+00_JPRB, 0.97587E+00_JPRB, 0.98932E+00_JPRB, 0.10057E+01_JPRB ,&
&  0.10245E+01_JPRB, 0.10403E+01_JPRB, 0.10484E+01_JPRB, 0.10588E+01_JPRB, 0.10748E+01_JPRB, 0.11069E+01_JPRB, 0.11356E+01_JPRB ,&
&  0.11202E+01_JPRB, 0.11333E+01_JPRB, 0.11407E+01_JPRB, 0.11422E+01_JPRB, 0.11299E+01_JPRB, 0.11009E+01_JPRB, 0.11071E+01_JPRB ,&
&  0.10987E+01_JPRB, 0.10937E+01_JPRB, 0.10896E+01_JPRB, 0.11073E+01_JPRB, 0.11386E+01_JPRB, 0.12015E+01_JPRB, 0.96381E+00_JPRB ,&
&  0.10365E+01_JPRB, 0.11053E+01_JPRB, 0.11072E+01_JPRB, 0.10087E+01_JPRB, 0.89765E+00_JPRB, 0.86191E+00_JPRB, 0.85003E+00_JPRB ,&
&  0.85180E+00_JPRB, 0.83065E+00_JPRB, 0.82454E+00_JPRB, 0.82196E+00_JPRB, 0.82070E+00_JPRB, 0.82825E+00_JPRB, 0.82365E+00_JPRB ,&
&  0.82719E+00_JPRB, 0.83590E+00_JPRB, 0.85086E+00_JPRB, 0.87833E+00_JPRB, 0.91080E+00_JPRB, 0.94120E+00_JPRB, 0.96619E+00_JPRB ,&
&  0.99443E+00_JPRB, 0.10134E+01_JPRB, 0.94399E+00_JPRB, 0.95838E+00_JPRB, 0.10229E+01_JPRB, 0.98346E+00_JPRB, 0.86395E+00_JPRB ,&
&  0.76671E+00_JPRB, 0.68622E+00_JPRB, 0.66039E+00_JPRB, 0.79012E+00_JPRB, 0.76316E+00_JPRB, 0.70961E+00_JPRB, 0.64959E+00_JPRB ,&
&  0.56273E+00_JPRB, 0.50708E+00_JPRB, 0.50608E+00_JPRB, 0.53939E+00_JPRB, 0.53007E+00_JPRB, 0.52605E+00_JPRB, 0.52866E+00_JPRB ,&
&  0.53396E+00_JPRB, 0.53678E+00_JPRB, 0.55065E+00_JPRB, 0.58580E+00_JPRB, 0.61202E+00_JPRB, 0.56216E+00_JPRB, 0.70179E+00_JPRB ,&
&  0.65159E+00_JPRB, 0.53360E+00_JPRB, 0.60847E+00_JPRB, 0.75399E+00_JPRB, 0.78831E+00_JPRB, 0.80654E+00_JPRB, 0.91111E+00_JPRB ,&
&  0.10560E+01_JPRB, 0.11830E+01_JPRB, 0.11873E+01_JPRB, 0.96296E+00_JPRB, 0.84063E+00_JPRB, 0.78639E+00_JPRB, 0.77529E+00_JPRB ,&
&  0.78050E+00_JPRB, 0.79572E+00_JPRB, 0.82173E+00_JPRB, 0.85189E+00_JPRB, 0.87813E+00_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.88708E+00_JPRB, 0.90821E+00_JPRB, 0.92457E+00_JPRB, 0.93699E+00_JPRB, 0.94891E+00_JPRB, 0.96391E+00_JPRB, 0.97816E+00_JPRB ,&
&  0.99261E+00_JPRB, 0.10044E+01_JPRB, 0.10125E+01_JPRB, 0.10274E+01_JPRB, 0.10443E+01_JPRB, 0.10779E+01_JPRB, 0.10944E+01_JPRB ,&
&  0.11253E+01_JPRB, 0.11355E+01_JPRB, 0.11333E+01_JPRB, 0.11460E+01_JPRB, 0.11389E+01_JPRB, 0.11188E+01_JPRB, 0.11131E+01_JPRB ,&
&  0.11140E+01_JPRB, 0.11068E+01_JPRB, 0.11028E+01_JPRB, 0.11030E+01_JPRB, 0.11048E+01_JPRB, 0.10782E+01_JPRB, 0.11376E+01_JPRB ,&
&  0.77510E+00_JPRB, 0.99842E+00_JPRB, 0.10668E+01_JPRB, 0.94940E+00_JPRB, 0.88476E+00_JPRB, 0.80107E+00_JPRB, 0.78603E+00_JPRB ,&
&  0.79055E+00_JPRB, 0.78147E+00_JPRB, 0.78626E+00_JPRB, 0.80037E+00_JPRB, 0.80625E+00_JPRB, 0.81332E+00_JPRB, 0.80934E+00_JPRB ,&
&  0.81846E+00_JPRB, 0.81582E+00_JPRB, 0.82168E+00_JPRB, 0.83101E+00_JPRB, 0.85542E+00_JPRB, 0.85454E+00_JPRB, 0.85296E+00_JPRB ,&
&  0.86546E+00_JPRB, 0.87560E+00_JPRB, 0.70361E+00_JPRB, 0.69110E+00_JPRB, 0.74882E+00_JPRB, 0.74836E+00_JPRB, 0.68847E+00_JPRB ,&
&  0.66171E+00_JPRB, 0.65417E+00_JPRB, 0.71018E+00_JPRB, 0.71958E+00_JPRB, 0.73154E+00_JPRB, 0.71577E+00_JPRB, 0.69161E+00_JPRB ,&
&  0.61406E+00_JPRB, 0.52881E+00_JPRB, 0.50300E+00_JPRB, 0.52222E+00_JPRB, 0.53364E+00_JPRB, 0.53604E+00_JPRB, 0.50029E+00_JPRB ,&
&  0.46722E+00_JPRB, 0.44035E+00_JPRB, 0.43574E+00_JPRB, 0.45606E+00_JPRB, 0.41268E+00_JPRB, 0.60423E+00_JPRB, 0.67479E+00_JPRB ,&
&  0.60178E+00_JPRB, 0.51540E+00_JPRB, 0.46536E+00_JPRB, 0.63550E+00_JPRB, 0.64088E+00_JPRB, 0.60789E+00_JPRB, 0.64714E+00_JPRB ,&
&  0.74722E+00_JPRB, 0.79616E+00_JPRB, 0.81190E+00_JPRB, 0.74095E+00_JPRB, 0.75009E+00_JPRB, 0.72863E+00_JPRB, 0.72861E+00_JPRB ,&
&  0.74351E+00_JPRB, 0.76995E+00_JPRB, 0.80268E+00_JPRB, 0.83435E+00_JPRB, 0.86189E+00_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.84215E+00_JPRB, 0.85950E+00_JPRB, 0.87021E+00_JPRB, 0.88090E+00_JPRB, 0.89662E+00_JPRB, 0.91292E+00_JPRB, 0.92025E+00_JPRB ,&
&  0.93699E+00_JPRB, 0.95251E+00_JPRB, 0.95942E+00_JPRB, 0.97809E+00_JPRB, 0.98639E+00_JPRB, 0.10369E+01_JPRB, 0.10560E+01_JPRB ,&
&  0.10930E+01_JPRB, 0.11124E+01_JPRB, 0.11175E+01_JPRB, 0.11189E+01_JPRB, 0.11312E+01_JPRB, 0.11303E+01_JPRB, 0.11261E+01_JPRB ,&
&  0.11448E+01_JPRB, 0.11837E+01_JPRB, 0.12341E+01_JPRB, 0.12838E+01_JPRB, 0.13150E+01_JPRB, 0.14110E+01_JPRB, 0.15374E+01_JPRB ,&
&  0.10162E+01_JPRB, 0.83671E+00_JPRB, 0.91673E+00_JPRB, 0.80965E+00_JPRB, 0.79966E+00_JPRB, 0.79298E+00_JPRB, 0.75760E+00_JPRB ,&
&  0.73984E+00_JPRB, 0.76022E+00_JPRB, 0.78758E+00_JPRB, 0.79777E+00_JPRB, 0.79317E+00_JPRB, 0.79609E+00_JPRB, 0.78745E+00_JPRB ,&
&  0.78683E+00_JPRB, 0.78682E+00_JPRB, 0.80170E+00_JPRB, 0.82637E+00_JPRB, 0.76746E+00_JPRB, 0.82016E+00_JPRB, 0.83376E+00_JPRB ,&
&  0.84082E+00_JPRB, 0.76384E+00_JPRB, 0.57474E+00_JPRB, 0.54834E+00_JPRB, 0.57962E+00_JPRB, 0.57569E+00_JPRB, 0.55415E+00_JPRB ,&
&  0.59110E+00_JPRB, 0.59083E+00_JPRB, 0.51812E+00_JPRB, 0.62904E+00_JPRB, 0.69860E+00_JPRB, 0.73073E+00_JPRB, 0.71359E+00_JPRB ,&
&  0.67649E+00_JPRB, 0.60170E+00_JPRB, 0.57282E+00_JPRB, 0.55807E+00_JPRB, 0.54864E+00_JPRB, 0.52606E+00_JPRB, 0.51405E+00_JPRB ,&
&  0.47724E+00_JPRB, 0.43152E+00_JPRB, 0.41298E+00_JPRB, 0.40028E+00_JPRB, 0.45797E+00_JPRB, 0.50275E+00_JPRB, 0.63091E+00_JPRB ,&
&  0.48929E+00_JPRB, 0.48323E+00_JPRB, 0.43735E+00_JPRB, 0.56754E+00_JPRB, 0.51568E+00_JPRB, 0.46576E+00_JPRB, 0.52021E+00_JPRB ,&
&  0.57688E+00_JPRB, 0.60557E+00_JPRB, 0.61543E+00_JPRB, 0.62999E+00_JPRB, 0.67147E+00_JPRB, 0.69219E+00_JPRB, 0.70964E+00_JPRB ,&
&  0.70939E+00_JPRB, 0.73766E+00_JPRB, 0.77700E+00_JPRB, 0.80631E+00_JPRB, 0.81700E+00_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.72318E+00_JPRB, 0.73632E+00_JPRB, 0.75355E+00_JPRB, 0.77198E+00_JPRB, 0.78196E+00_JPRB, 0.79890E+00_JPRB, 0.81474E+00_JPRB ,&
&  0.83648E+00_JPRB, 0.86488E+00_JPRB, 0.88121E+00_JPRB, 0.88702E+00_JPRB, 0.90396E+00_JPRB, 0.92494E+00_JPRB, 0.93651E+00_JPRB ,&
&  0.95915E+00_JPRB, 0.96817E+00_JPRB, 0.10188E+01_JPRB, 0.10440E+01_JPRB, 0.10808E+01_JPRB, 0.11312E+01_JPRB, 0.11387E+01_JPRB ,&
&  0.11329E+01_JPRB, 0.11205E+01_JPRB, 0.11326E+01_JPRB, 0.11300E+01_JPRB, 0.12033E+01_JPRB, 0.11985E+01_JPRB, 0.11229E+01_JPRB ,&
&  0.81933E+00_JPRB, 0.69368E+00_JPRB, 0.87697E+00_JPRB, 0.78885E+00_JPRB, 0.78453E+00_JPRB, 0.76450E+00_JPRB, 0.69728E+00_JPRB ,&
&  0.69665E+00_JPRB, 0.72688E+00_JPRB, 0.76271E+00_JPRB, 0.78907E+00_JPRB, 0.82309E+00_JPRB, 0.82683E+00_JPRB, 0.81473E+00_JPRB ,&
&  0.81281E+00_JPRB, 0.81974E+00_JPRB, 0.81891E+00_JPRB, 0.74361E+00_JPRB, 0.66438E+00_JPRB, 0.69347E+00_JPRB, 0.70137E+00_JPRB ,&
&  0.69474E+00_JPRB, 0.66434E+00_JPRB, 0.53711E+00_JPRB, 0.49796E+00_JPRB, 0.51498E+00_JPRB, 0.47706E+00_JPRB, 0.47055E+00_JPRB ,&
&  0.52876E+00_JPRB, 0.48099E+00_JPRB, 0.33099E+00_JPRB, 0.46245E+00_JPRB, 0.59903E+00_JPRB, 0.69539E+00_JPRB, 0.73795E+00_JPRB ,&
&  0.72469E+00_JPRB, 0.66214E+00_JPRB, 0.62948E+00_JPRB, 0.62718E+00_JPRB, 0.62586E+00_JPRB, 0.57434E+00_JPRB, 0.54303E+00_JPRB ,&
&  0.48527E+00_JPRB, 0.43007E+00_JPRB, 0.39082E+00_JPRB, 0.38943E+00_JPRB, 0.38822E+00_JPRB, 0.49015E+00_JPRB, 0.52559E+00_JPRB ,&
&  0.48358E+00_JPRB, 0.43443E+00_JPRB, 0.49525E+00_JPRB, 0.48481E+00_JPRB, 0.35905E+00_JPRB, 0.42603E+00_JPRB, 0.46089E+00_JPRB ,&
&  0.48787E+00_JPRB, 0.50600E+00_JPRB, 0.52038E+00_JPRB, 0.54436E+00_JPRB, 0.57814E+00_JPRB, 0.59018E+00_JPRB, 0.59427E+00_JPRB ,&
&  0.61042E+00_JPRB, 0.63750E+00_JPRB, 0.65737E+00_JPRB, 0.68181E+00_JPRB, 0.70368E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.56058E+00_JPRB, 0.55999E+00_JPRB, 0.56317E+00_JPRB, 0.56957E+00_JPRB, 0.56990E+00_JPRB, 0.56761E+00_JPRB, 0.57865E+00_JPRB ,&
&  0.59936E+00_JPRB, 0.63934E+00_JPRB, 0.69709E+00_JPRB, 0.72929E+00_JPRB, 0.71285E+00_JPRB, 0.66922E+00_JPRB, 0.65560E+00_JPRB ,&
&  0.65646E+00_JPRB, 0.65503E+00_JPRB, 0.68338E+00_JPRB, 0.73121E+00_JPRB, 0.79425E+00_JPRB, 0.81843E+00_JPRB, 0.87799E+00_JPRB ,&
&  0.87898E+00_JPRB, 0.89208E+00_JPRB, 0.96879E+00_JPRB, 0.10315E+01_JPRB, 0.15595E+01_JPRB, 0.14434E+01_JPRB, 0.10351E+01_JPRB ,&
&  0.77932E+00_JPRB, 0.77993E+00_JPRB, 0.87880E+00_JPRB, 0.87995E+00_JPRB, 0.90443E+00_JPRB, 0.86803E+00_JPRB, 0.88224E+00_JPRB ,&
&  0.86934E+00_JPRB, 0.90942E+00_JPRB, 0.93886E+00_JPRB, 0.97874E+00_JPRB, 0.10060E+01_JPRB, 0.10097E+01_JPRB, 0.10204E+01_JPRB ,&
&  0.10258E+01_JPRB, 0.10206E+01_JPRB, 0.99515E+00_JPRB, 0.83086E+00_JPRB, 0.82039E+00_JPRB, 0.79514E+00_JPRB, 0.75230E+00_JPRB ,&
&  0.69942E+00_JPRB, 0.65413E+00_JPRB, 0.66408E+00_JPRB, 0.61554E+00_JPRB, 0.57346E+00_JPRB, 0.51409E+00_JPRB, 0.46454E+00_JPRB ,&
&  0.46361E+00_JPRB, 0.39534E+00_JPRB, 0.28361E+00_JPRB, 0.39570E+00_JPRB, 0.54792E+00_JPRB, 0.59622E+00_JPRB, 0.72669E+00_JPRB ,&
&  0.76540E+00_JPRB, 0.73577E+00_JPRB, 0.70697E+00_JPRB, 0.69953E+00_JPRB, 0.65541E+00_JPRB, 0.62853E+00_JPRB, 0.54723E+00_JPRB ,&
&  0.51047E+00_JPRB, 0.43730E+00_JPRB, 0.39304E+00_JPRB, 0.38065E+00_JPRB, 0.37672E+00_JPRB, 0.39405E+00_JPRB, 0.51956E+00_JPRB ,&
&  0.55025E+00_JPRB, 0.49693E+00_JPRB, 0.51963E+00_JPRB, 0.53113E+00_JPRB, 0.43568E+00_JPRB, 0.41129E+00_JPRB, 0.41801E+00_JPRB ,&
&  0.42502E+00_JPRB, 0.44028E+00_JPRB, 0.46640E+00_JPRB, 0.48469E+00_JPRB, 0.49812E+00_JPRB, 0.51011E+00_JPRB, 0.52434E+00_JPRB ,&
&  0.53339E+00_JPRB, 0.54453E+00_JPRB, 0.54905E+00_JPRB, 0.55381E+00_JPRB, 0.55835E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.58848E+00_JPRB, 0.58945E+00_JPRB, 0.57658E+00_JPRB, 0.58988E+00_JPRB, 0.61415E+00_JPRB, 0.66996E+00_JPRB, 0.72627E+00_JPRB ,&
&  0.80094E+00_JPRB, 0.89330E+00_JPRB, 0.10069E+01_JPRB, 0.11012E+01_JPRB, 0.11214E+01_JPRB, 0.10971E+01_JPRB, 0.99855E+00_JPRB ,&
&  0.87249E+00_JPRB, 0.74593E+00_JPRB, 0.67835E+00_JPRB, 0.66259E+00_JPRB, 0.70976E+00_JPRB, 0.73848E+00_JPRB, 0.80367E+00_JPRB ,&
&  0.90196E+00_JPRB, 0.10666E+01_JPRB, 0.12535E+01_JPRB, 0.16893E+01_JPRB, 0.20311E+01_JPRB, 0.12784E+01_JPRB, 0.10114E+01_JPRB ,&
&  0.97407E+00_JPRB, 0.10847E+01_JPRB, 0.10783E+01_JPRB, 0.11100E+01_JPRB, 0.12103E+01_JPRB, 0.12915E+01_JPRB, 0.13886E+01_JPRB ,&
&  0.14331E+01_JPRB, 0.14737E+01_JPRB, 0.15022E+01_JPRB, 0.15180E+01_JPRB, 0.15372E+01_JPRB, 0.15364E+01_JPRB, 0.15139E+01_JPRB ,&
&  0.14747E+01_JPRB, 0.14021E+01_JPRB, 0.13492E+01_JPRB, 0.13168E+01_JPRB, 0.12793E+01_JPRB, 0.11780E+01_JPRB, 0.10404E+01_JPRB ,&
&  0.93176E+00_JPRB, 0.86408E+00_JPRB, 0.88080E+00_JPRB, 0.84106E+00_JPRB, 0.73647E+00_JPRB, 0.66353E+00_JPRB, 0.55554E+00_JPRB ,&
&  0.49847E+00_JPRB, 0.45528E+00_JPRB, 0.37098E+00_JPRB, 0.54084E+00_JPRB, 0.65593E+00_JPRB, 0.69870E+00_JPRB, 0.72704E+00_JPRB ,&
&  0.78779E+00_JPRB, 0.78061E+00_JPRB, 0.76890E+00_JPRB, 0.73622E+00_JPRB, 0.66024E+00_JPRB, 0.68467E+00_JPRB, 0.53312E+00_JPRB ,&
&  0.52509E+00_JPRB, 0.45044E+00_JPRB, 0.41960E+00_JPRB, 0.42930E+00_JPRB, 0.44428E+00_JPRB, 0.40554E+00_JPRB, 0.48716E+00_JPRB ,&
&  0.51826E+00_JPRB, 0.70712E+00_JPRB, 0.90270E+00_JPRB, 0.12779E+01_JPRB, 0.16954E+01_JPRB, 0.45407E+00_JPRB, 0.43494E+00_JPRB ,&
&  0.42960E+00_JPRB, 0.43675E+00_JPRB, 0.45560E+00_JPRB, 0.46743E+00_JPRB, 0.48635E+00_JPRB, 0.49743E+00_JPRB, 0.50064E+00_JPRB ,&
&  0.50873E+00_JPRB, 0.52847E+00_JPRB, 0.54834E+00_JPRB, 0.55780E+00_JPRB, 0.57677E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.84282E+00_JPRB, 0.87835E+00_JPRB, 0.88052E+00_JPRB, 0.94427E+00_JPRB, 0.10036E+01_JPRB, 0.10255E+01_JPRB, 0.10422E+01_JPRB ,&
&  0.10926E+01_JPRB, 0.11748E+01_JPRB, 0.12393E+01_JPRB, 0.12768E+01_JPRB, 0.13149E+01_JPRB, 0.13755E+01_JPRB, 0.13835E+01_JPRB ,&
&  0.13468E+01_JPRB, 0.13142E+01_JPRB, 0.12923E+01_JPRB, 0.12678E+01_JPRB, 0.12661E+01_JPRB, 0.12957E+01_JPRB, 0.14302E+01_JPRB ,&
&  0.15401E+01_JPRB, 0.15547E+01_JPRB, 0.15790E+01_JPRB, 0.13588E+01_JPRB, 0.92236E+00_JPRB, 0.11304E+01_JPRB, 0.11058E+01_JPRB ,&
&  0.11889E+01_JPRB, 0.12669E+01_JPRB, 0.13232E+01_JPRB, 0.13968E+01_JPRB, 0.14669E+01_JPRB, 0.15540E+01_JPRB, 0.16420E+01_JPRB ,&
&  0.16997E+01_JPRB, 0.17639E+01_JPRB, 0.18352E+01_JPRB, 0.18872E+01_JPRB, 0.19478E+01_JPRB, 0.20325E+01_JPRB, 0.20509E+01_JPRB ,&
&  0.20402E+01_JPRB, 0.19707E+01_JPRB, 0.19011E+01_JPRB, 0.18487E+01_JPRB, 0.16880E+01_JPRB, 0.15096E+01_JPRB, 0.13033E+01_JPRB ,&
&  0.11775E+01_JPRB, 0.11142E+01_JPRB, 0.11093E+01_JPRB, 0.10866E+01_JPRB, 0.10095E+01_JPRB, 0.90917E+00_JPRB, 0.78399E+00_JPRB ,&
&  0.70024E+00_JPRB, 0.64862E+00_JPRB, 0.64661E+00_JPRB, 0.72243E+00_JPRB, 0.55841E+00_JPRB, 0.63880E+00_JPRB, 0.81472E+00_JPRB ,&
&  0.81331E+00_JPRB, 0.77855E+00_JPRB, 0.75582E+00_JPRB, 0.67402E+00_JPRB, 0.63433E+00_JPRB, 0.51906E+00_JPRB, 0.54486E+00_JPRB ,&
&  0.55882E+00_JPRB, 0.48267E+00_JPRB, 0.46124E+00_JPRB, 0.49601E+00_JPRB, 0.52415E+00_JPRB, 0.47527E+00_JPRB, 0.49077E+00_JPRB ,&
&  0.53848E+00_JPRB, 0.62916E+00_JPRB, 0.70942E+00_JPRB, 0.83519E+00_JPRB, 0.91817E+00_JPRB, 0.49979E+00_JPRB, 0.42713E+00_JPRB ,&
&  0.41282E+00_JPRB, 0.42500E+00_JPRB, 0.44630E+00_JPRB, 0.47127E+00_JPRB, 0.50181E+00_JPRB, 0.53942E+00_JPRB, 0.57889E+00_JPRB ,&
&  0.61081E+00_JPRB, 0.63381E+00_JPRB, 0.67263E+00_JPRB, 0.72424E+00_JPRB, 0.79284E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.11351E+01_JPRB, 0.12095E+01_JPRB, 0.12735E+01_JPRB, 0.12525E+01_JPRB, 0.12388E+01_JPRB, 0.11963E+01_JPRB, 0.11606E+01_JPRB ,&
&  0.11923E+01_JPRB, 0.12358E+01_JPRB, 0.13046E+01_JPRB, 0.13571E+01_JPRB, 0.13919E+01_JPRB, 0.14625E+01_JPRB, 0.15431E+01_JPRB ,&
&  0.15691E+01_JPRB, 0.15772E+01_JPRB, 0.15840E+01_JPRB, 0.15879E+01_JPRB, 0.15378E+01_JPRB, 0.15231E+01_JPRB, 0.14877E+01_JPRB ,&
&  0.11289E+01_JPRB, 0.91496E+00_JPRB, 0.10964E+01_JPRB, 0.12690E+01_JPRB, 0.12031E+01_JPRB, 0.12414E+01_JPRB, 0.13158E+01_JPRB ,&
&  0.14502E+01_JPRB, 0.14758E+01_JPRB, 0.15079E+01_JPRB, 0.15167E+01_JPRB, 0.15148E+01_JPRB, 0.16084E+01_JPRB, 0.17102E+01_JPRB ,&
&  0.17876E+01_JPRB, 0.18613E+01_JPRB, 0.19364E+01_JPRB, 0.20154E+01_JPRB, 0.20838E+01_JPRB, 0.21622E+01_JPRB, 0.21982E+01_JPRB ,&
&  0.22035E+01_JPRB, 0.21569E+01_JPRB, 0.20951E+01_JPRB, 0.19325E+01_JPRB, 0.17699E+01_JPRB, 0.16355E+01_JPRB, 0.14916E+01_JPRB ,&
&  0.14390E+01_JPRB, 0.14361E+01_JPRB, 0.14583E+01_JPRB, 0.14448E+01_JPRB, 0.13969E+01_JPRB, 0.12604E+01_JPRB, 0.11078E+01_JPRB ,&
&  0.95770E+00_JPRB, 0.83438E+00_JPRB, 0.85166E+00_JPRB, 0.89255E+00_JPRB, 0.73226E+00_JPRB, 0.73150E+00_JPRB, 0.76560E+00_JPRB ,&
&  0.87844E+00_JPRB, 0.81246E+00_JPRB, 0.73147E+00_JPRB, 0.67848E+00_JPRB, 0.70302E+00_JPRB, 0.61990E+00_JPRB, 0.62571E+00_JPRB ,&
&  0.58849E+00_JPRB, 0.51519E+00_JPRB, 0.47561E+00_JPRB, 0.51070E+00_JPRB, 0.44779E+00_JPRB, 0.44929E+00_JPRB, 0.51422E+00_JPRB ,&
&  0.59535E+00_JPRB, 0.56549E+00_JPRB, 0.59662E+00_JPRB, 0.59201E+00_JPRB, 0.57737E+00_JPRB, 0.46230E+00_JPRB, 0.42601E+00_JPRB ,&
&  0.41161E+00_JPRB, 0.42728E+00_JPRB, 0.46199E+00_JPRB, 0.49147E+00_JPRB, 0.53647E+00_JPRB, 0.58880E+00_JPRB, 0.68032E+00_JPRB ,&
&  0.76082E+00_JPRB, 0.83437E+00_JPRB, 0.89179E+00_JPRB, 0.96282E+00_JPRB, 0.10421E+01_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.11095E+01_JPRB, 0.12258E+01_JPRB, 0.13427E+01_JPRB, 0.14225E+01_JPRB, 0.13805E+01_JPRB, 0.13209E+01_JPRB, 0.12777E+01_JPRB ,&
&  0.12483E+01_JPRB, 0.12641E+01_JPRB, 0.13334E+01_JPRB, 0.13975E+01_JPRB, 0.14441E+01_JPRB, 0.15170E+01_JPRB, 0.16240E+01_JPRB ,&
&  0.17099E+01_JPRB, 0.17723E+01_JPRB, 0.18172E+01_JPRB, 0.18562E+01_JPRB, 0.18167E+01_JPRB, 0.17694E+01_JPRB, 0.14704E+01_JPRB ,&
&  0.98113E+00_JPRB, 0.10437E+01_JPRB, 0.14068E+01_JPRB, 0.13474E+01_JPRB, 0.13499E+01_JPRB, 0.13907E+01_JPRB, 0.14964E+01_JPRB ,&
&  0.16507E+01_JPRB, 0.16766E+01_JPRB, 0.16523E+01_JPRB, 0.16107E+01_JPRB, 0.15790E+01_JPRB, 0.16491E+01_JPRB, 0.17579E+01_JPRB ,&
&  0.18451E+01_JPRB, 0.18963E+01_JPRB, 0.19677E+01_JPRB, 0.20614E+01_JPRB, 0.21880E+01_JPRB, 0.23044E+01_JPRB, 0.23653E+01_JPRB ,&
&  0.23204E+01_JPRB, 0.23089E+01_JPRB, 0.21523E+01_JPRB, 0.18905E+01_JPRB, 0.17872E+01_JPRB, 0.17882E+01_JPRB, 0.17918E+01_JPRB ,&
&  0.18419E+01_JPRB, 0.19494E+01_JPRB, 0.20348E+01_JPRB, 0.19567E+01_JPRB, 0.17413E+01_JPRB, 0.16055E+01_JPRB, 0.13680E+01_JPRB ,&
&  0.12826E+01_JPRB, 0.10852E+01_JPRB, 0.10574E+01_JPRB, 0.92547E+00_JPRB, 0.91463E+00_JPRB, 0.93828E+00_JPRB, 0.98601E+00_JPRB ,&
&  0.10215E+01_JPRB, 0.96572E+00_JPRB, 0.87574E+00_JPRB, 0.80118E+00_JPRB, 0.86681E+00_JPRB, 0.86152E+00_JPRB, 0.85592E+00_JPRB ,&
&  0.70241E+00_JPRB, 0.63619E+00_JPRB, 0.51190E+00_JPRB, 0.48446E+00_JPRB, 0.38424E+00_JPRB, 0.38955E+00_JPRB, 0.53392E+00_JPRB ,&
&  0.67274E+00_JPRB, 0.60402E+00_JPRB, 0.61102E+00_JPRB, 0.58869E+00_JPRB, 0.51990E+00_JPRB, 0.43793E+00_JPRB, 0.40465E+00_JPRB ,&
&  0.40752E+00_JPRB, 0.44990E+00_JPRB, 0.48899E+00_JPRB, 0.52078E+00_JPRB, 0.57682E+00_JPRB, 0.63771E+00_JPRB, 0.70012E+00_JPRB ,&
&  0.74158E+00_JPRB, 0.79689E+00_JPRB, 0.85450E+00_JPRB, 0.91318E+00_JPRB, 0.10000E+01_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.84368E+00_JPRB, 0.89118E+00_JPRB, 0.93584E+00_JPRB, 0.97416E+00_JPRB, 0.10041E+01_JPRB, 0.98067E+00_JPRB, 0.98594E+00_JPRB ,&
&  0.10573E+01_JPRB, 0.11646E+01_JPRB, 0.12751E+01_JPRB, 0.13530E+01_JPRB, 0.13833E+01_JPRB, 0.14491E+01_JPRB, 0.15785E+01_JPRB ,&
&  0.16943E+01_JPRB, 0.18157E+01_JPRB, 0.19196E+01_JPRB, 0.20176E+01_JPRB, 0.19871E+01_JPRB, 0.18731E+01_JPRB, 0.14056E+01_JPRB ,&
&  0.98028E+00_JPRB, 0.16024E+01_JPRB, 0.13489E+01_JPRB, 0.13755E+01_JPRB, 0.14641E+01_JPRB, 0.15788E+01_JPRB, 0.16378E+01_JPRB ,&
&  0.17195E+01_JPRB, 0.17350E+01_JPRB, 0.17002E+01_JPRB, 0.16475E+01_JPRB, 0.16076E+01_JPRB, 0.16461E+01_JPRB, 0.17279E+01_JPRB ,&
&  0.18172E+01_JPRB, 0.18799E+01_JPRB, 0.19708E+01_JPRB, 0.21052E+01_JPRB, 0.22792E+01_JPRB, 0.24574E+01_JPRB, 0.25337E+01_JPRB ,&
&  0.25127E+01_JPRB, 0.24692E+01_JPRB, 0.22269E+01_JPRB, 0.19329E+01_JPRB, 0.18369E+01_JPRB, 0.19837E+01_JPRB, 0.21991E+01_JPRB ,&
&  0.23242E+01_JPRB, 0.24106E+01_JPRB, 0.26902E+01_JPRB, 0.27523E+01_JPRB, 0.25127E+01_JPRB, 0.21915E+01_JPRB, 0.16887E+01_JPRB ,&
&  0.16337E+01_JPRB, 0.13787E+01_JPRB, 0.12483E+01_JPRB, 0.10760E+01_JPRB, 0.10402E+01_JPRB, 0.10938E+01_JPRB, 0.11584E+01_JPRB ,&
&  0.11356E+01_JPRB, 0.11780E+01_JPRB, 0.10965E+01_JPRB, 0.10452E+01_JPRB, 0.10810E+01_JPRB, 0.11363E+01_JPRB, 0.10561E+01_JPRB ,&
&  0.92146E+00_JPRB, 0.67385E+00_JPRB, 0.55422E+00_JPRB, 0.46715E+00_JPRB, 0.37714E+00_JPRB, 0.34039E+00_JPRB, 0.48462E+00_JPRB ,&
&  0.61003E+00_JPRB, 0.63320E+00_JPRB, 0.72350E+00_JPRB, 0.73128E+00_JPRB, 0.59827E+00_JPRB, 0.47520E+00_JPRB, 0.42187E+00_JPRB ,&
&  0.43315E+00_JPRB, 0.48351E+00_JPRB, 0.51854E+00_JPRB, 0.55105E+00_JPRB, 0.59741E+00_JPRB, 0.61531E+00_JPRB, 0.66507E+00_JPRB ,&
&  0.68255E+00_JPRB, 0.71528E+00_JPRB, 0.74788E+00_JPRB, 0.76217E+00_JPRB, 0.79470E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.64902E+00_JPRB, 0.67230E+00_JPRB, 0.71276E+00_JPRB, 0.74201E+00_JPRB, 0.76381E+00_JPRB, 0.79324E+00_JPRB, 0.80072E+00_JPRB ,&
&  0.84674E+00_JPRB, 0.94266E+00_JPRB, 0.10650E+01_JPRB, 0.11991E+01_JPRB, 0.12457E+01_JPRB, 0.12684E+01_JPRB, 0.13787E+01_JPRB ,&
&  0.15529E+01_JPRB, 0.18017E+01_JPRB, 0.20666E+01_JPRB, 0.22889E+01_JPRB, 0.23157E+01_JPRB, 0.21481E+01_JPRB, 0.15113E+01_JPRB ,&
&  0.12569E+01_JPRB, 0.16277E+01_JPRB, 0.14890E+01_JPRB, 0.14729E+01_JPRB, 0.16030E+01_JPRB, 0.17393E+01_JPRB, 0.16737E+01_JPRB ,&
&  0.16478E+01_JPRB, 0.16476E+01_JPRB, 0.15904E+01_JPRB, 0.15641E+01_JPRB, 0.15785E+01_JPRB, 0.15951E+01_JPRB, 0.16460E+01_JPRB ,&
&  0.17208E+01_JPRB, 0.17793E+01_JPRB, 0.18936E+01_JPRB, 0.20943E+01_JPRB, 0.23295E+01_JPRB, 0.25373E+01_JPRB, 0.26614E+01_JPRB ,&
&  0.27127E+01_JPRB, 0.26339E+01_JPRB, 0.24905E+01_JPRB, 0.20339E+01_JPRB, 0.19381E+01_JPRB, 0.21536E+01_JPRB, 0.25851E+01_JPRB ,&
&  0.28364E+01_JPRB, 0.29733E+01_JPRB, 0.34009E+01_JPRB, 0.37307E+01_JPRB, 0.35642E+01_JPRB, 0.29696E+01_JPRB, 0.23283E+01_JPRB ,&
&  0.20437E+01_JPRB, 0.17510E+01_JPRB, 0.14412E+01_JPRB, 0.13338E+01_JPRB, 0.12457E+01_JPRB, 0.12553E+01_JPRB, 0.13614E+01_JPRB ,&
&  0.13683E+01_JPRB, 0.13987E+01_JPRB, 0.13222E+01_JPRB, 0.12668E+01_JPRB, 0.12316E+01_JPRB, 0.12290E+01_JPRB, 0.12984E+01_JPRB ,&
&  0.12203E+01_JPRB, 0.92417E+00_JPRB, 0.70602E+00_JPRB, 0.46210E+00_JPRB, 0.34601E+00_JPRB, 0.30321E+00_JPRB, 0.40920E+00_JPRB ,&
&  0.56642E+00_JPRB, 0.60342E+00_JPRB, 0.65176E+00_JPRB, 0.88749E+00_JPRB, 0.78746E+00_JPRB, 0.57583E+00_JPRB, 0.43951E+00_JPRB ,&
&  0.43995E+00_JPRB, 0.48432E+00_JPRB, 0.50949E+00_JPRB, 0.51777E+00_JPRB, 0.54824E+00_JPRB, 0.57592E+00_JPRB, 0.58283E+00_JPRB ,&
&  0.60498E+00_JPRB, 0.62409E+00_JPRB, 0.62682E+00_JPRB, 0.62107E+00_JPRB, 0.63434E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.55408E+00_JPRB, 0.57564E+00_JPRB, 0.58313E+00_JPRB, 0.58465E+00_JPRB, 0.59643E+00_JPRB, 0.62499E+00_JPRB, 0.63554E+00_JPRB ,&
&  0.67756E+00_JPRB, 0.74542E+00_JPRB, 0.84027E+00_JPRB, 0.97955E+00_JPRB, 0.10716E+01_JPRB, 0.10922E+01_JPRB, 0.11362E+01_JPRB ,&
&  0.12628E+01_JPRB, 0.15307E+01_JPRB, 0.19493E+01_JPRB, 0.23403E+01_JPRB, 0.23904E+01_JPRB, 0.21473E+01_JPRB, 0.19474E+01_JPRB ,&
&  0.17177E+01_JPRB, 0.19040E+01_JPRB, 0.17681E+01_JPRB, 0.19506E+01_JPRB, 0.22242E+01_JPRB, 0.25458E+01_JPRB, 0.17215E+01_JPRB ,&
&  0.15702E+01_JPRB, 0.15350E+01_JPRB, 0.14794E+01_JPRB, 0.14766E+01_JPRB, 0.14572E+01_JPRB, 0.14889E+01_JPRB, 0.15165E+01_JPRB ,&
&  0.15448E+01_JPRB, 0.16448E+01_JPRB, 0.17808E+01_JPRB, 0.20094E+01_JPRB, 0.23521E+01_JPRB, 0.26214E+01_JPRB, 0.27948E+01_JPRB ,&
&  0.28107E+01_JPRB, 0.28611E+01_JPRB, 0.27785E+01_JPRB, 0.23507E+01_JPRB, 0.20725E+01_JPRB, 0.23333E+01_JPRB, 0.29334E+01_JPRB ,&
&  0.36355E+01_JPRB, 0.41830E+01_JPRB, 0.43331E+01_JPRB, 0.43030E+01_JPRB, 0.41609E+01_JPRB, 0.38170E+01_JPRB, 0.30580E+01_JPRB ,&
&  0.25328E+01_JPRB, 0.21139E+01_JPRB, 0.16963E+01_JPRB, 0.16674E+01_JPRB, 0.16167E+01_JPRB, 0.15503E+01_JPRB, 0.13385E+01_JPRB ,&
&  0.12252E+01_JPRB, 0.11707E+01_JPRB, 0.11270E+01_JPRB, 0.12102E+01_JPRB, 0.13447E+01_JPRB, 0.13370E+01_JPRB, 0.14915E+01_JPRB ,&
&  0.12298E+01_JPRB, 0.48353E+00_JPRB, 0.33586E+00_JPRB, 0.38358E+00_JPRB, 0.25102E+00_JPRB, 0.24630E+00_JPRB, 0.41213E+00_JPRB ,&
&  0.60614E+00_JPRB, 0.68287E+00_JPRB, 0.68075E+00_JPRB, 0.74071E+00_JPRB, 0.92426E+00_JPRB, 0.73913E+00_JPRB, 0.47453E+00_JPRB ,&
&  0.47444E+00_JPRB, 0.51869E+00_JPRB, 0.52500E+00_JPRB, 0.50872E+00_JPRB, 0.50831E+00_JPRB, 0.53128E+00_JPRB, 0.50549E+00_JPRB ,&
&  0.52968E+00_JPRB, 0.54414E+00_JPRB, 0.54795E+00_JPRB, 0.54705E+00_JPRB, 0.54346E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.51330E+00_JPRB, 0.53606E+00_JPRB, 0.54972E+00_JPRB, 0.52531E+00_JPRB, 0.51416E+00_JPRB, 0.52643E+00_JPRB, 0.53403E+00_JPRB ,&
&  0.57343E+00_JPRB, 0.61591E+00_JPRB, 0.67083E+00_JPRB, 0.76658E+00_JPRB, 0.86856E+00_JPRB, 0.92995E+00_JPRB, 0.98042E+00_JPRB ,&
&  0.10560E+01_JPRB, 0.12543E+01_JPRB, 0.16589E+01_JPRB, 0.22139E+01_JPRB, 0.25083E+01_JPRB, 0.23440E+01_JPRB, 0.23923E+01_JPRB ,&
&  0.20358E+01_JPRB, 0.23621E+01_JPRB, 0.30474E+01_JPRB, 0.37502E+01_JPRB, 0.38716E+01_JPRB, 0.36162E+01_JPRB, 0.22054E+01_JPRB ,&
&  0.17335E+01_JPRB, 0.16457E+01_JPRB, 0.14880E+01_JPRB, 0.15153E+01_JPRB, 0.13857E+01_JPRB, 0.14185E+01_JPRB, 0.14354E+01_JPRB ,&
&  0.14917E+01_JPRB, 0.16020E+01_JPRB, 0.17092E+01_JPRB, 0.19577E+01_JPRB, 0.23593E+01_JPRB, 0.27231E+01_JPRB, 0.30726E+01_JPRB ,&
&  0.30541E+01_JPRB, 0.31068E+01_JPRB, 0.30538E+01_JPRB, 0.30204E+01_JPRB, 0.20467E+01_JPRB, 0.27834E+01_JPRB, 0.33374E+01_JPRB ,&
&  0.42371E+01_JPRB, 0.48355E+01_JPRB, 0.46511E+01_JPRB, 0.52078E+01_JPRB, 0.51522E+01_JPRB, 0.44322E+01_JPRB, 0.35863E+01_JPRB ,&
&  0.30519E+01_JPRB, 0.23887E+01_JPRB, 0.20221E+01_JPRB, 0.19940E+01_JPRB, 0.19400E+01_JPRB, 0.18772E+01_JPRB, 0.11904E+01_JPRB ,&
&  0.11239E+01_JPRB, 0.13049E+01_JPRB, 0.12381E+01_JPRB, 0.10696E+01_JPRB, 0.13136E+01_JPRB, 0.14287E+01_JPRB, 0.11285E+01_JPRB ,&
&  0.50444E+00_JPRB, 0.30192E+00_JPRB, 0.25414E+00_JPRB, 0.25302E+00_JPRB, 0.22822E+00_JPRB, 0.26697E+00_JPRB, 0.66621E+00_JPRB ,&
&  0.60105E+00_JPRB, 0.75820E+00_JPRB, 0.71331E+00_JPRB, 0.72571E+00_JPRB, 0.95991E+00_JPRB, 0.97734E+00_JPRB, 0.11818E+01_JPRB ,&
&  0.57752E+00_JPRB, 0.54646E+00_JPRB, 0.56386E+00_JPRB, 0.55991E+00_JPRB, 0.56804E+00_JPRB, 0.54595E+00_JPRB, 0.51722E+00_JPRB ,&
&  0.53149E+00_JPRB, 0.55031E+00_JPRB, 0.52868E+00_JPRB, 0.50898E+00_JPRB, 0.50245E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.49805E+00_JPRB, 0.51059E+00_JPRB, 0.51366E+00_JPRB, 0.50305E+00_JPRB, 0.47095E+00_JPRB, 0.46668E+00_JPRB, 0.48332E+00_JPRB ,&
&  0.50583E+00_JPRB, 0.53149E+00_JPRB, 0.56452E+00_JPRB, 0.61254E+00_JPRB, 0.68292E+00_JPRB, 0.75777E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.92822E+00_JPRB, 0.11228E+01_JPRB, 0.14627E+01_JPRB, 0.18886E+01_JPRB, 0.24254E+01_JPRB, 0.23499E+01_JPRB, 0.26292E+01_JPRB ,&
&  0.23061E+01_JPRB, 0.25992E+01_JPRB, 0.36766E+01_JPRB, 0.50655E+01_JPRB, 0.67113E+01_JPRB, 0.63934E+01_JPRB, 0.46054E+01_JPRB ,&
&  0.30447E+01_JPRB, 0.24178E+01_JPRB, 0.20861E+01_JPRB, 0.17724E+01_JPRB, 0.15512E+01_JPRB, 0.15005E+01_JPRB, 0.15424E+01_JPRB ,&
&  0.15157E+01_JPRB, 0.15480E+01_JPRB, 0.15920E+01_JPRB, 0.17847E+01_JPRB, 0.21648E+01_JPRB, 0.26124E+01_JPRB, 0.31625E+01_JPRB ,&
&  0.34677E+01_JPRB, 0.36545E+01_JPRB, 0.35507E+01_JPRB, 0.34187E+01_JPRB, 0.30432E+01_JPRB, 0.29366E+01_JPRB, 0.35139E+01_JPRB ,&
&  0.46605E+01_JPRB, 0.56492E+01_JPRB, 0.54443E+01_JPRB, 0.63286E+01_JPRB, 0.56575E+01_JPRB, 0.48152E+01_JPRB, 0.40399E+01_JPRB ,&
&  0.33457E+01_JPRB, 0.31042E+01_JPRB, 0.22837E+01_JPRB, 0.21471E+01_JPRB, 0.21562E+01_JPRB, 0.16551E+01_JPRB, 0.13074E+01_JPRB ,&
&  0.14863E+01_JPRB, 0.13599E+01_JPRB, 0.12648E+01_JPRB, 0.10516E+01_JPRB, 0.13178E+01_JPRB, 0.12218E+01_JPRB, 0.64323E+00_JPRB ,&
&  0.44652E+00_JPRB, 0.34039E+00_JPRB, 0.24872E+00_JPRB, 0.24867E+00_JPRB, 0.24042E+00_JPRB, 0.28419E+00_JPRB, 0.57669E+00_JPRB ,&
&  0.67870E+00_JPRB, 0.82329E+00_JPRB, 0.79380E+00_JPRB, 0.72651E+00_JPRB, 0.88188E+00_JPRB, 0.10540E+01_JPRB, 0.23221E+01_JPRB ,&
&  0.15412E+01_JPRB, 0.92397E+00_JPRB, 0.65617E+00_JPRB, 0.62047E+00_JPRB, 0.61655E+00_JPRB, 0.59295E+00_JPRB, 0.56015E+00_JPRB ,&
&  0.54610E+00_JPRB, 0.55855E+00_JPRB, 0.55237E+00_JPRB, 0.53719E+00_JPRB, 0.51806E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.51491E+00_JPRB, 0.51523E+00_JPRB, 0.52263E+00_JPRB, 0.52044E+00_JPRB, 0.49812E+00_JPRB, 0.48290E+00_JPRB, 0.48049E+00_JPRB ,&
&  0.48834E+00_JPRB, 0.49510E+00_JPRB, 0.50620E+00_JPRB, 0.52762E+00_JPRB, 0.56217E+00_JPRB, 0.61525E+00_JPRB, 0.68908E+00_JPRB ,&
&  0.81367E+00_JPRB, 0.10363E+01_JPRB, 0.12871E+01_JPRB, 0.15339E+01_JPRB, 0.19564E+01_JPRB, 0.20882E+01_JPRB, 0.25169E+01_JPRB ,&
&  0.25659E+01_JPRB, 0.28010E+01_JPRB, 0.39619E+01_JPRB, 0.60032E+01_JPRB, 0.91738E+01_JPRB, 0.10094E+02_JPRB, 0.90896E+01_JPRB ,&
&  0.69340E+01_JPRB, 0.46238E+01_JPRB, 0.32516E+01_JPRB, 0.23977E+01_JPRB, 0.18848E+01_JPRB, 0.16170E+01_JPRB, 0.16002E+01_JPRB ,&
&  0.15353E+01_JPRB, 0.15251E+01_JPRB, 0.15577E+01_JPRB, 0.15757E+01_JPRB, 0.18099E+01_JPRB, 0.22688E+01_JPRB, 0.29899E+01_JPRB ,&
&  0.36238E+01_JPRB, 0.41186E+01_JPRB, 0.42449E+01_JPRB, 0.36941E+01_JPRB, 0.42514E+01_JPRB, 0.43228E+01_JPRB, 0.40460E+01_JPRB ,&
&  0.43877E+01_JPRB, 0.49588E+01_JPRB, 0.64751E+01_JPRB, 0.62140E+01_JPRB, 0.53618E+01_JPRB, 0.50864E+01_JPRB, 0.42265E+01_JPRB ,&
&  0.35799E+01_JPRB, 0.32100E+01_JPRB, 0.25240E+01_JPRB, 0.23485E+01_JPRB, 0.20368E+01_JPRB, 0.15557E+01_JPRB, 0.15753E+01_JPRB ,&
&  0.17068E+01_JPRB, 0.16339E+01_JPRB, 0.14023E+01_JPRB, 0.11360E+01_JPRB, 0.10947E+01_JPRB, 0.82382E+00_JPRB, 0.55776E+00_JPRB ,&
&  0.46573E+00_JPRB, 0.39036E+00_JPRB, 0.29282E+00_JPRB, 0.33839E+00_JPRB, 0.31272E+00_JPRB, 0.37287E+00_JPRB, 0.59539E+00_JPRB ,&
&  0.83312E+00_JPRB, 0.86081E+00_JPRB, 0.88179E+00_JPRB, 0.75398E+00_JPRB, 0.76700E+00_JPRB, 0.83097E+00_JPRB, 0.24912E+01_JPRB ,&
&  0.16539E+01_JPRB, 0.13882E+01_JPRB, 0.10237E+01_JPRB, 0.77002E+00_JPRB, 0.66998E+00_JPRB, 0.62443E+00_JPRB, 0.60149E+00_JPRB ,&
&  0.57410E+00_JPRB, 0.55477E+00_JPRB, 0.54766E+00_JPRB, 0.53306E+00_JPRB, 0.51912E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.55609E+00_JPRB, 0.55875E+00_JPRB, 0.56815E+00_JPRB, 0.56899E+00_JPRB, 0.56228E+00_JPRB, 0.54006E+00_JPRB, 0.51222E+00_JPRB ,&
&  0.50390E+00_JPRB, 0.50071E+00_JPRB, 0.49875E+00_JPRB, 0.51126E+00_JPRB, 0.51838E+00_JPRB, 0.54124E+00_JPRB, 0.59218E+00_JPRB ,&
&  0.73871E+00_JPRB, 0.10274E+01_JPRB, 0.10456E+01_JPRB, 0.11677E+01_JPRB, 0.15220E+01_JPRB, 0.20442E+01_JPRB, 0.22933E+01_JPRB ,&
&  0.26684E+01_JPRB, 0.28037E+01_JPRB, 0.36670E+01_JPRB, 0.56697E+01_JPRB, 0.11578E+02_JPRB, 0.13487E+02_JPRB, 0.12582E+02_JPRB ,&
&  0.11886E+02_JPRB, 0.85631E+01_JPRB, 0.54308E+01_JPRB, 0.37313E+01_JPRB, 0.26648E+01_JPRB, 0.19407E+01_JPRB, 0.16067E+01_JPRB ,&
&  0.14845E+01_JPRB, 0.14392E+01_JPRB, 0.14327E+01_JPRB, 0.14611E+01_JPRB, 0.15748E+01_JPRB, 0.19306E+01_JPRB, 0.26755E+01_JPRB ,&
&  0.34435E+01_JPRB, 0.42417E+01_JPRB, 0.47795E+01_JPRB, 0.46708E+01_JPRB, 0.39258E+01_JPRB, 0.42961E+01_JPRB, 0.61180E+01_JPRB ,&
&  0.60569E+01_JPRB, 0.65169E+01_JPRB, 0.64547E+01_JPRB, 0.64662E+01_JPRB, 0.53394E+01_JPRB, 0.45491E+01_JPRB, 0.40399E+01_JPRB ,&
&  0.28730E+01_JPRB, 0.23656E+01_JPRB, 0.22827E+01_JPRB, 0.21728E+01_JPRB, 0.19150E+01_JPRB, 0.19931E+01_JPRB, 0.25052E+01_JPRB ,&
&  0.19795E+01_JPRB, 0.20155E+01_JPRB, 0.20285E+01_JPRB, 0.18826E+01_JPRB, 0.11613E+01_JPRB, 0.81823E+00_JPRB, 0.10667E+01_JPRB ,&
&  0.10222E+01_JPRB, 0.60645E+00_JPRB, 0.48240E+00_JPRB, 0.61984E+00_JPRB, 0.45709E+00_JPRB, 0.53321E+00_JPRB, 0.75107E+00_JPRB ,&
&  0.80707E+00_JPRB, 0.81567E+00_JPRB, 0.97934E+00_JPRB, 0.85632E+00_JPRB, 0.82528E+00_JPRB, 0.73822E+00_JPRB, 0.14134E+01_JPRB ,&
&  0.17031E+01_JPRB, 0.15317E+01_JPRB, 0.13610E+01_JPRB, 0.10532E+01_JPRB, 0.83859E+00_JPRB, 0.73870E+00_JPRB, 0.69695E+00_JPRB ,&
&  0.66862E+00_JPRB, 0.63977E+00_JPRB, 0.61221E+00_JPRB, 0.57480E+00_JPRB, 0.56218E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.59927E+00_JPRB, 0.59741E+00_JPRB, 0.60235E+00_JPRB, 0.60964E+00_JPRB, 0.60919E+00_JPRB, 0.58196E+00_JPRB, 0.54632E+00_JPRB ,&
&  0.52203E+00_JPRB, 0.51144E+00_JPRB, 0.50966E+00_JPRB, 0.50954E+00_JPRB, 0.49784E+00_JPRB, 0.50484E+00_JPRB, 0.53196E+00_JPRB ,&
&  0.66052E+00_JPRB, 0.86819E+00_JPRB, 0.92548E+00_JPRB, 0.10261E+01_JPRB, 0.14234E+01_JPRB, 0.20169E+01_JPRB, 0.24071E+01_JPRB ,&
&  0.29306E+01_JPRB, 0.28826E+01_JPRB, 0.34416E+01_JPRB, 0.54077E+01_JPRB, 0.95056E+01_JPRB, 0.12323E+02_JPRB, 0.12040E+02_JPRB ,&
&  0.11604E+02_JPRB, 0.95207E+01_JPRB, 0.71829E+01_JPRB, 0.53655E+01_JPRB, 0.38748E+01_JPRB, 0.28083E+01_JPRB, 0.20618E+01_JPRB ,&
&  0.16673E+01_JPRB, 0.14817E+01_JPRB, 0.14223E+01_JPRB, 0.13952E+01_JPRB, 0.14649E+01_JPRB, 0.16205E+01_JPRB, 0.21884E+01_JPRB ,&
&  0.28726E+01_JPRB, 0.38886E+01_JPRB, 0.50892E+01_JPRB, 0.56465E+01_JPRB, 0.47597E+01_JPRB, 0.51771E+01_JPRB, 0.67689E+01_JPRB ,&
&  0.68747E+01_JPRB, 0.66708E+01_JPRB, 0.65095E+01_JPRB, 0.59379E+01_JPRB, 0.55706E+01_JPRB, 0.47561E+01_JPRB, 0.45193E+01_JPRB ,&
&  0.34374E+01_JPRB, 0.27437E+01_JPRB, 0.23375E+01_JPRB, 0.20592E+01_JPRB, 0.22171E+01_JPRB, 0.28047E+01_JPRB, 0.28160E+01_JPRB ,&
&  0.25168E+01_JPRB, 0.24191E+01_JPRB, 0.22352E+01_JPRB, 0.18135E+01_JPRB, 0.15551E+01_JPRB, 0.12848E+01_JPRB, 0.14778E+01_JPRB ,&
&  0.14457E+01_JPRB, 0.13223E+01_JPRB, 0.11076E+01_JPRB, 0.88289E+00_JPRB, 0.70951E+00_JPRB, 0.82849E+00_JPRB, 0.89009E+00_JPRB ,&
&  0.85902E+00_JPRB, 0.79475E+00_JPRB, 0.87828E+00_JPRB, 0.85908E+00_JPRB, 0.82648E+00_JPRB, 0.65850E+00_JPRB, 0.99675E+00_JPRB ,&
&  0.11754E+01_JPRB, 0.14027E+01_JPRB, 0.14389E+01_JPRB, 0.12416E+01_JPRB, 0.10713E+01_JPRB, 0.92756E+00_JPRB, 0.84704E+00_JPRB ,&
&  0.80641E+00_JPRB, 0.76699E+00_JPRB, 0.71166E+00_JPRB, 0.65515E+00_JPRB, 0.62289E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.60468E+00_JPRB, 0.60041E+00_JPRB, 0.59610E+00_JPRB, 0.59110E+00_JPRB, 0.57933E+00_JPRB, 0.56796E+00_JPRB, 0.54896E+00_JPRB ,&
&  0.53153E+00_JPRB, 0.51740E+00_JPRB, 0.51237E+00_JPRB, 0.50623E+00_JPRB, 0.50278E+00_JPRB, 0.49615E+00_JPRB, 0.49943E+00_JPRB ,&
&  0.57417E+00_JPRB, 0.75580E+00_JPRB, 0.80111E+00_JPRB, 0.94997E+00_JPRB, 0.13176E+01_JPRB, 0.19035E+01_JPRB, 0.26818E+01_JPRB ,&
&  0.30909E+01_JPRB, 0.32051E+01_JPRB, 0.36296E+01_JPRB, 0.51056E+01_JPRB, 0.70446E+01_JPRB, 0.82958E+01_JPRB, 0.84848E+01_JPRB ,&
&  0.72979E+01_JPRB, 0.64470E+01_JPRB, 0.55040E+01_JPRB, 0.46737E+01_JPRB, 0.40458E+01_JPRB, 0.34771E+01_JPRB, 0.28107E+01_JPRB ,&
&  0.22180E+01_JPRB, 0.18314E+01_JPRB, 0.16367E+01_JPRB, 0.15395E+01_JPRB, 0.15073E+01_JPRB, 0.15576E+01_JPRB, 0.18176E+01_JPRB ,&
&  0.22860E+01_JPRB, 0.31400E+01_JPRB, 0.47541E+01_JPRB, 0.64888E+01_JPRB, 0.62132E+01_JPRB, 0.72361E+01_JPRB, 0.71836E+01_JPRB ,&
&  0.79076E+01_JPRB, 0.77847E+01_JPRB, 0.66225E+01_JPRB, 0.61532E+01_JPRB, 0.48000E+01_JPRB, 0.48371E+01_JPRB, 0.49589E+01_JPRB ,&
&  0.48722E+01_JPRB, 0.44507E+01_JPRB, 0.41250E+01_JPRB, 0.32766E+01_JPRB, 0.27757E+01_JPRB, 0.33626E+01_JPRB, 0.31215E+01_JPRB ,&
&  0.27207E+01_JPRB, 0.25469E+01_JPRB, 0.23020E+01_JPRB, 0.21525E+01_JPRB, 0.17591E+01_JPRB, 0.16376E+01_JPRB, 0.14057E+01_JPRB ,&
&  0.12924E+01_JPRB, 0.10979E+01_JPRB, 0.11918E+01_JPRB, 0.10445E+01_JPRB, 0.98132E+00_JPRB, 0.97928E+00_JPRB, 0.97036E+00_JPRB ,&
&  0.91121E+00_JPRB, 0.78900E+00_JPRB, 0.71985E+00_JPRB, 0.78844E+00_JPRB, 0.85242E+00_JPRB, 0.71402E+00_JPRB, 0.79016E+00_JPRB ,&
&  0.96997E+00_JPRB, 0.11041E+01_JPRB, 0.10847E+01_JPRB, 0.11435E+01_JPRB, 0.11198E+01_JPRB, 0.10708E+01_JPRB, 0.98088E+00_JPRB ,&
&  0.86554E+00_JPRB, 0.76313E+00_JPRB, 0.69929E+00_JPRB, 0.64563E+00_JPRB, 0.61121E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.57757E+00_JPRB, 0.58033E+00_JPRB, 0.56901E+00_JPRB, 0.55139E+00_JPRB, 0.54198E+00_JPRB, 0.53331E+00_JPRB, 0.52444E+00_JPRB ,&
&  0.51036E+00_JPRB, 0.50198E+00_JPRB, 0.49962E+00_JPRB, 0.50075E+00_JPRB, 0.50124E+00_JPRB, 0.49957E+00_JPRB, 0.49391E+00_JPRB ,&
&  0.50688E+00_JPRB, 0.68612E+00_JPRB, 0.70680E+00_JPRB, 0.75629E+00_JPRB, 0.10519E+01_JPRB, 0.18036E+01_JPRB, 0.26020E+01_JPRB ,&
&  0.30293E+01_JPRB, 0.32253E+01_JPRB, 0.36215E+01_JPRB, 0.41865E+01_JPRB, 0.48445E+01_JPRB, 0.52952E+01_JPRB, 0.54353E+01_JPRB ,&
&  0.52640E+01_JPRB, 0.40432E+01_JPRB, 0.36429E+01_JPRB, 0.32793E+01_JPRB, 0.30724E+01_JPRB, 0.28626E+01_JPRB, 0.26551E+01_JPRB ,&
&  0.23836E+01_JPRB, 0.20529E+01_JPRB, 0.18000E+01_JPRB, 0.16349E+01_JPRB, 0.15434E+01_JPRB, 0.14617E+01_JPRB, 0.15451E+01_JPRB ,&
&  0.17435E+01_JPRB, 0.20340E+01_JPRB, 0.30501E+01_JPRB, 0.46722E+01_JPRB, 0.69891E+01_JPRB, 0.84264E+01_JPRB, 0.84278E+01_JPRB ,&
&  0.70002E+01_JPRB, 0.63818E+01_JPRB, 0.76449E+01_JPRB, 0.62770E+01_JPRB, 0.57205E+01_JPRB, 0.56664E+01_JPRB, 0.52741E+01_JPRB ,&
&  0.50196E+01_JPRB, 0.46089E+01_JPRB, 0.40950E+01_JPRB, 0.35249E+01_JPRB, 0.35086E+01_JPRB, 0.34676E+01_JPRB, 0.31303E+01_JPRB ,&
&  0.26980E+01_JPRB, 0.25740E+01_JPRB, 0.23738E+01_JPRB, 0.21523E+01_JPRB, 0.19866E+01_JPRB, 0.19633E+01_JPRB, 0.17640E+01_JPRB ,&
&  0.15982E+01_JPRB, 0.16903E+01_JPRB, 0.13847E+01_JPRB, 0.11063E+01_JPRB, 0.94935E+00_JPRB, 0.98270E+00_JPRB, 0.10099E+01_JPRB ,&
&  0.91529E+00_JPRB, 0.76034E+00_JPRB, 0.61482E+00_JPRB, 0.61192E+00_JPRB, 0.66504E+00_JPRB, 0.68029E+00_JPRB, 0.66704E+00_JPRB ,&
&  0.73454E+00_JPRB, 0.88558E+00_JPRB, 0.89753E+00_JPRB, 0.92348E+00_JPRB, 0.10377E+01_JPRB, 0.98424E+00_JPRB, 0.87031E+00_JPRB ,&
&  0.76428E+00_JPRB, 0.69149E+00_JPRB, 0.64489E+00_JPRB, 0.61111E+00_JPRB, 0.58265E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.56240E+00_JPRB, 0.55942E+00_JPRB, 0.54795E+00_JPRB, 0.53081E+00_JPRB, 0.52227E+00_JPRB, 0.52100E+00_JPRB, 0.51321E+00_JPRB ,&
&  0.50766E+00_JPRB, 0.50313E+00_JPRB, 0.50787E+00_JPRB, 0.50519E+00_JPRB, 0.50891E+00_JPRB, 0.51813E+00_JPRB, 0.52245E+00_JPRB ,&
&  0.52153E+00_JPRB, 0.65501E+00_JPRB, 0.63110E+00_JPRB, 0.68807E+00_JPRB, 0.86554E+00_JPRB, 0.14457E+01_JPRB, 0.20313E+01_JPRB ,&
&  0.24502E+01_JPRB, 0.26148E+01_JPRB, 0.27975E+01_JPRB, 0.31122E+01_JPRB, 0.33040E+01_JPRB, 0.32563E+01_JPRB, 0.32127E+01_JPRB ,&
&  0.30623E+01_JPRB, 0.28128E+01_JPRB, 0.23522E+01_JPRB, 0.23105E+01_JPRB, 0.21231E+01_JPRB, 0.18184E+01_JPRB, 0.19364E+01_JPRB ,&
&  0.19550E+01_JPRB, 0.19134E+01_JPRB, 0.18115E+01_JPRB, 0.17183E+01_JPRB, 0.16548E+01_JPRB, 0.15630E+01_JPRB, 0.15232E+01_JPRB ,&
&  0.15482E+01_JPRB, 0.16163E+01_JPRB, 0.18667E+01_JPRB, 0.25329E+01_JPRB, 0.39571E+01_JPRB, 0.57164E+01_JPRB, 0.71567E+01_JPRB ,&
&  0.84672E+01_JPRB, 0.82443E+01_JPRB, 0.65201E+01_JPRB, 0.62578E+01_JPRB, 0.65115E+01_JPRB, 0.57133E+01_JPRB, 0.50923E+01_JPRB ,&
&  0.49989E+01_JPRB, 0.48379E+01_JPRB, 0.42809E+01_JPRB, 0.38292E+01_JPRB, 0.35781E+01_JPRB, 0.34412E+01_JPRB, 0.31810E+01_JPRB ,&
&  0.27011E+01_JPRB, 0.25068E+01_JPRB, 0.24019E+01_JPRB, 0.22386E+01_JPRB, 0.19947E+01_JPRB, 0.18374E+01_JPRB, 0.17640E+01_JPRB ,&
&  0.16575E+01_JPRB, 0.16196E+01_JPRB, 0.13140E+01_JPRB, 0.12334E+01_JPRB, 0.98144E+00_JPRB, 0.96102E+00_JPRB, 0.10219E+01_JPRB ,&
&  0.91027E+00_JPRB, 0.72702E+00_JPRB, 0.55567E+00_JPRB, 0.49292E+00_JPRB, 0.48709E+00_JPRB, 0.51022E+00_JPRB, 0.56354E+00_JPRB ,&
&  0.61917E+00_JPRB, 0.61294E+00_JPRB, 0.73351E+00_JPRB, 0.76573E+00_JPRB, 0.82468E+00_JPRB, 0.81104E+00_JPRB, 0.76038E+00_JPRB ,&
&  0.71613E+00_JPRB, 0.68125E+00_JPRB, 0.64621E+00_JPRB, 0.60954E+00_JPRB, 0.57980E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.54582E+00_JPRB, 0.52546E+00_JPRB, 0.51623E+00_JPRB, 0.50453E+00_JPRB, 0.49905E+00_JPRB, 0.49829E+00_JPRB, 0.50018E+00_JPRB ,&
&  0.50769E+00_JPRB, 0.51254E+00_JPRB, 0.52003E+00_JPRB, 0.51748E+00_JPRB, 0.51769E+00_JPRB, 0.51720E+00_JPRB, 0.54105E+00_JPRB ,&
&  0.55579E+00_JPRB, 0.57758E+00_JPRB, 0.61085E+00_JPRB, 0.65579E+00_JPRB, 0.92546E+00_JPRB, 0.11506E+01_JPRB, 0.14299E+01_JPRB ,&
&  0.16171E+01_JPRB, 0.17765E+01_JPRB, 0.18557E+01_JPRB, 0.19055E+01_JPRB, 0.19170E+01_JPRB, 0.19038E+01_JPRB, 0.18072E+01_JPRB ,&
&  0.16496E+01_JPRB, 0.14658E+01_JPRB, 0.13754E+01_JPRB, 0.13339E+01_JPRB, 0.13568E+01_JPRB, 0.12082E+01_JPRB, 0.12977E+01_JPRB ,&
&  0.13858E+01_JPRB, 0.14709E+01_JPRB, 0.15128E+01_JPRB, 0.15354E+01_JPRB, 0.15368E+01_JPRB, 0.15445E+01_JPRB, 0.15584E+01_JPRB ,&
&  0.16010E+01_JPRB, 0.16042E+01_JPRB, 0.16375E+01_JPRB, 0.19241E+01_JPRB, 0.26856E+01_JPRB, 0.43431E+01_JPRB, 0.59900E+01_JPRB ,&
&  0.61409E+01_JPRB, 0.67857E+01_JPRB, 0.75257E+01_JPRB, 0.75448E+01_JPRB, 0.62893E+01_JPRB, 0.52285E+01_JPRB, 0.47935E+01_JPRB ,&
&  0.48366E+01_JPRB, 0.49795E+01_JPRB, 0.45906E+01_JPRB, 0.39181E+01_JPRB, 0.35772E+01_JPRB, 0.34094E+01_JPRB, 0.32297E+01_JPRB ,&
&  0.29560E+01_JPRB, 0.26730E+01_JPRB, 0.24511E+01_JPRB, 0.22908E+01_JPRB, 0.21724E+01_JPRB, 0.19245E+01_JPRB, 0.17596E+01_JPRB ,&
&  0.16579E+01_JPRB, 0.14501E+01_JPRB, 0.13262E+01_JPRB, 0.13095E+01_JPRB, 0.11139E+01_JPRB, 0.10891E+01_JPRB, 0.10867E+01_JPRB ,&
&  0.94212E+00_JPRB, 0.71231E+00_JPRB, 0.58187E+00_JPRB, 0.50846E+00_JPRB, 0.48034E+00_JPRB, 0.47423E+00_JPRB, 0.48792E+00_JPRB ,&
&  0.52125E+00_JPRB, 0.53799E+00_JPRB, 0.64272E+00_JPRB, 0.68958E+00_JPRB, 0.71140E+00_JPRB, 0.69956E+00_JPRB, 0.67662E+00_JPRB ,&
&  0.64126E+00_JPRB, 0.61322E+00_JPRB, 0.59921E+00_JPRB, 0.58620E+00_JPRB, 0.57065E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.53050E+00_JPRB, 0.51854E+00_JPRB, 0.51088E+00_JPRB, 0.50456E+00_JPRB, 0.50005E+00_JPRB, 0.49608E+00_JPRB, 0.49516E+00_JPRB ,&
&  0.49618E+00_JPRB, 0.50787E+00_JPRB, 0.52470E+00_JPRB, 0.52436E+00_JPRB, 0.52154E+00_JPRB, 0.52530E+00_JPRB, 0.57704E+00_JPRB ,&
&  0.46082E+00_JPRB, 0.48556E+00_JPRB, 0.51754E+00_JPRB, 0.63090E+00_JPRB, 0.90857E+00_JPRB, 0.92983E+00_JPRB, 0.10395E+01_JPRB ,&
&  0.11086E+01_JPRB, 0.11403E+01_JPRB, 0.11617E+01_JPRB, 0.11907E+01_JPRB, 0.11827E+01_JPRB, 0.11739E+01_JPRB, 0.11273E+01_JPRB ,&
&  0.98501E+00_JPRB, 0.88167E+00_JPRB, 0.83864E+00_JPRB, 0.80911E+00_JPRB, 0.84585E+00_JPRB, 0.90085E+00_JPRB, 0.99312E+00_JPRB ,&
&  0.10717E+01_JPRB, 0.11486E+01_JPRB, 0.12101E+01_JPRB, 0.12566E+01_JPRB, 0.12870E+01_JPRB, 0.13182E+01_JPRB, 0.13570E+01_JPRB ,&
&  0.14011E+01_JPRB, 0.14371E+01_JPRB, 0.15016E+01_JPRB, 0.17541E+01_JPRB, 0.22586E+01_JPRB, 0.39993E+01_JPRB, 0.54404E+01_JPRB ,&
&  0.44865E+01_JPRB, 0.40140E+01_JPRB, 0.50548E+01_JPRB, 0.60436E+01_JPRB, 0.53130E+01_JPRB, 0.44576E+01_JPRB, 0.39057E+01_JPRB ,&
&  0.38463E+01_JPRB, 0.39476E+01_JPRB, 0.38345E+01_JPRB, 0.35086E+01_JPRB, 0.30757E+01_JPRB, 0.28850E+01_JPRB, 0.28043E+01_JPRB ,&
&  0.26478E+01_JPRB, 0.24835E+01_JPRB, 0.23320E+01_JPRB, 0.21763E+01_JPRB, 0.21059E+01_JPRB, 0.20707E+01_JPRB, 0.19967E+01_JPRB ,&
&  0.18579E+01_JPRB, 0.16886E+01_JPRB, 0.15073E+01_JPRB, 0.13807E+01_JPRB, 0.12177E+01_JPRB, 0.11345E+01_JPRB, 0.11120E+01_JPRB ,&
&  0.99439E+00_JPRB, 0.77953E+00_JPRB, 0.69027E+00_JPRB, 0.62027E+00_JPRB, 0.58545E+00_JPRB, 0.51394E+00_JPRB, 0.47805E+00_JPRB ,&
&  0.48482E+00_JPRB, 0.51621E+00_JPRB, 0.57734E+00_JPRB, 0.60361E+00_JPRB, 0.61074E+00_JPRB, 0.61863E+00_JPRB, 0.55701E+00_JPRB ,&
&  0.57826E+00_JPRB, 0.55285E+00_JPRB, 0.54022E+00_JPRB, 0.54021E+00_JPRB, 0.53640E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.51121E+00_JPRB, 0.50713E+00_JPRB, 0.50836E+00_JPRB, 0.50044E+00_JPRB, 0.49105E+00_JPRB, 0.49175E+00_JPRB, 0.49906E+00_JPRB ,&
&  0.50299E+00_JPRB, 0.51379E+00_JPRB, 0.52284E+00_JPRB, 0.52393E+00_JPRB, 0.52972E+00_JPRB, 0.56110E+00_JPRB, 0.51758E+00_JPRB ,&
&  0.43969E+00_JPRB, 0.45346E+00_JPRB, 0.46223E+00_JPRB, 0.51708E+00_JPRB, 0.58688E+00_JPRB, 0.62114E+00_JPRB, 0.66490E+00_JPRB ,&
&  0.72892E+00_JPRB, 0.74327E+00_JPRB, 0.77126E+00_JPRB, 0.79494E+00_JPRB, 0.80080E+00_JPRB, 0.79939E+00_JPRB, 0.78742E+00_JPRB ,&
&  0.71122E+00_JPRB, 0.66082E+00_JPRB, 0.62800E+00_JPRB, 0.61039E+00_JPRB, 0.64611E+00_JPRB, 0.70791E+00_JPRB, 0.77649E+00_JPRB ,&
&  0.84572E+00_JPRB, 0.90573E+00_JPRB, 0.96580E+00_JPRB, 0.10132E+01_JPRB, 0.10418E+01_JPRB, 0.10656E+01_JPRB, 0.10988E+01_JPRB ,&
&  0.11427E+01_JPRB, 0.11887E+01_JPRB, 0.12299E+01_JPRB, 0.14296E+01_JPRB, 0.16512E+01_JPRB, 0.25373E+01_JPRB, 0.32431E+01_JPRB ,&
&  0.28268E+01_JPRB, 0.26171E+01_JPRB, 0.28162E+01_JPRB, 0.31238E+01_JPRB, 0.31947E+01_JPRB, 0.29768E+01_JPRB, 0.28042E+01_JPRB ,&
&  0.27924E+01_JPRB, 0.28541E+01_JPRB, 0.29433E+01_JPRB, 0.27939E+01_JPRB, 0.25907E+01_JPRB, 0.24665E+01_JPRB, 0.23450E+01_JPRB ,&
&  0.22444E+01_JPRB, 0.21104E+01_JPRB, 0.21188E+01_JPRB, 0.20655E+01_JPRB, 0.20014E+01_JPRB, 0.19628E+01_JPRB, 0.19583E+01_JPRB ,&
&  0.19329E+01_JPRB, 0.18396E+01_JPRB, 0.16665E+01_JPRB, 0.15269E+01_JPRB, 0.13900E+01_JPRB, 0.13590E+01_JPRB, 0.12574E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.85779E+00_JPRB, 0.81410E+00_JPRB, 0.72351E+00_JPRB, 0.64736E+00_JPRB, 0.57248E+00_JPRB, 0.51846E+00_JPRB ,&
&  0.50634E+00_JPRB, 0.54627E+00_JPRB, 0.55738E+00_JPRB, 0.56295E+00_JPRB, 0.57115E+00_JPRB, 0.57498E+00_JPRB, 0.50500E+00_JPRB ,&
&  0.50313E+00_JPRB, 0.53259E+00_JPRB, 0.52132E+00_JPRB, 0.51827E+00_JPRB, 0.51407E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.49604E+00_JPRB, 0.50017E+00_JPRB, 0.50392E+00_JPRB, 0.50938E+00_JPRB, 0.49505E+00_JPRB, 0.51619E+00_JPRB, 0.46854E+00_JPRB ,&
&  0.53102E+00_JPRB, 0.53801E+00_JPRB, 0.52607E+00_JPRB, 0.54682E+00_JPRB, 0.59595E+00_JPRB, 0.56213E+00_JPRB, 0.43324E+00_JPRB ,&
&  0.38653E+00_JPRB, 0.39131E+00_JPRB, 0.41317E+00_JPRB, 0.43252E+00_JPRB, 0.45048E+00_JPRB, 0.46754E+00_JPRB, 0.49493E+00_JPRB ,&
&  0.52539E+00_JPRB, 0.54229E+00_JPRB, 0.57287E+00_JPRB, 0.59196E+00_JPRB, 0.59910E+00_JPRB, 0.59973E+00_JPRB, 0.61091E+00_JPRB ,&
&  0.57566E+00_JPRB, 0.55239E+00_JPRB, 0.53866E+00_JPRB, 0.50455E+00_JPRB, 0.54328E+00_JPRB, 0.57751E+00_JPRB, 0.62391E+00_JPRB ,&
&  0.67875E+00_JPRB, 0.73874E+00_JPRB, 0.78420E+00_JPRB, 0.82693E+00_JPRB, 0.85893E+00_JPRB, 0.88396E+00_JPRB, 0.90918E+00_JPRB ,&
&  0.94630E+00_JPRB, 0.98706E+00_JPRB, 0.10289E+01_JPRB, 0.11648E+01_JPRB, 0.13581E+01_JPRB, 0.16147E+01_JPRB, 0.19643E+01_JPRB ,&
&  0.20680E+01_JPRB, 0.22956E+01_JPRB, 0.24819E+01_JPRB, 0.21738E+01_JPRB, 0.23104E+01_JPRB, 0.25277E+01_JPRB, 0.23972E+01_JPRB ,&
&  0.22645E+01_JPRB, 0.20502E+01_JPRB, 0.19456E+01_JPRB, 0.18992E+01_JPRB, 0.19603E+01_JPRB, 0.20012E+01_JPRB, 0.19872E+01_JPRB ,&
&  0.18949E+01_JPRB, 0.18287E+01_JPRB, 0.18741E+01_JPRB, 0.18817E+01_JPRB, 0.18571E+01_JPRB, 0.18387E+01_JPRB, 0.18691E+01_JPRB ,&
&  0.18909E+01_JPRB, 0.18673E+01_JPRB, 0.18071E+01_JPRB, 0.17076E+01_JPRB, 0.16037E+01_JPRB, 0.14868E+01_JPRB, 0.13362E+01_JPRB ,&
&  0.11958E+01_JPRB, 0.10478E+01_JPRB, 0.92220E+00_JPRB, 0.83470E+00_JPRB, 0.74449E+00_JPRB, 0.65944E+00_JPRB, 0.59084E+00_JPRB ,&
&  0.52861E+00_JPRB, 0.51808E+00_JPRB, 0.55589E+00_JPRB, 0.55020E+00_JPRB, 0.55208E+00_JPRB, 0.54875E+00_JPRB, 0.50935E+00_JPRB ,&
&  0.49765E+00_JPRB, 0.51521E+00_JPRB, 0.51281E+00_JPRB, 0.50415E+00_JPRB, 0.49761E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.50239E+00_JPRB, 0.49358E+00_JPRB, 0.49020E+00_JPRB, 0.50639E+00_JPRB, 0.50860E+00_JPRB, 0.48239E+00_JPRB, 0.45058E+00_JPRB ,&
&  0.56169E+00_JPRB, 0.51941E+00_JPRB, 0.58304E+00_JPRB, 0.44705E+00_JPRB, 0.39219E+00_JPRB, 0.38242E+00_JPRB, 0.36774E+00_JPRB ,&
&  0.35959E+00_JPRB, 0.36167E+00_JPRB, 0.37113E+00_JPRB, 0.38754E+00_JPRB, 0.40624E+00_JPRB, 0.41602E+00_JPRB, 0.42584E+00_JPRB ,&
&  0.43982E+00_JPRB, 0.45556E+00_JPRB, 0.48292E+00_JPRB, 0.50538E+00_JPRB, 0.51558E+00_JPRB, 0.51737E+00_JPRB, 0.52533E+00_JPRB ,&
&  0.49744E+00_JPRB, 0.48435E+00_JPRB, 0.48665E+00_JPRB, 0.48232E+00_JPRB, 0.48957E+00_JPRB, 0.50396E+00_JPRB, 0.53083E+00_JPRB ,&
&  0.58885E+00_JPRB, 0.55206E+00_JPRB, 0.65801E+00_JPRB, 0.68674E+00_JPRB, 0.71743E+00_JPRB, 0.74255E+00_JPRB, 0.76345E+00_JPRB ,&
&  0.78801E+00_JPRB, 0.82942E+00_JPRB, 0.87440E+00_JPRB, 0.94323E+00_JPRB, 0.11291E+01_JPRB, 0.12486E+01_JPRB, 0.13243E+01_JPRB ,&
&  0.15733E+01_JPRB, 0.14287E+01_JPRB, 0.16936E+01_JPRB, 0.17700E+01_JPRB, 0.20405E+01_JPRB, 0.22397E+01_JPRB, 0.21441E+01_JPRB ,&
&  0.22108E+01_JPRB, 0.19143E+01_JPRB, 0.16618E+01_JPRB, 0.14828E+01_JPRB, 0.14395E+01_JPRB, 0.14519E+01_JPRB, 0.15012E+01_JPRB ,&
&  0.14929E+01_JPRB, 0.14108E+01_JPRB, 0.14209E+01_JPRB, 0.14561E+01_JPRB, 0.15017E+01_JPRB, 0.15754E+01_JPRB, 0.16606E+01_JPRB ,&
&  0.17290E+01_JPRB, 0.17575E+01_JPRB, 0.17726E+01_JPRB, 0.16735E+01_JPRB, 0.16027E+01_JPRB, 0.15526E+01_JPRB, 0.13625E+01_JPRB ,&
&  0.12130E+01_JPRB, 0.11220E+01_JPRB, 0.10512E+01_JPRB, 0.10006E+01_JPRB, 0.89994E+00_JPRB, 0.80105E+00_JPRB, 0.69289E+00_JPRB ,&
&  0.56005E+00_JPRB, 0.49301E+00_JPRB, 0.48673E+00_JPRB, 0.48192E+00_JPRB, 0.47994E+00_JPRB, 0.48593E+00_JPRB, 0.48799E+00_JPRB ,&
&  0.48765E+00_JPRB, 0.48943E+00_JPRB, 0.50542E+00_JPRB, 0.51083E+00_JPRB, 0.50664E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.54137E+00_JPRB, 0.53031E+00_JPRB, 0.51366E+00_JPRB, 0.50918E+00_JPRB, 0.51566E+00_JPRB, 0.48876E+00_JPRB, 0.43542E+00_JPRB ,&
&  0.45341E+00_JPRB, 0.41282E+00_JPRB, 0.39721E+00_JPRB, 0.38347E+00_JPRB, 0.37368E+00_JPRB, 0.35473E+00_JPRB, 0.33484E+00_JPRB ,&
&  0.33693E+00_JPRB, 0.35622E+00_JPRB, 0.38646E+00_JPRB, 0.40820E+00_JPRB, 0.41799E+00_JPRB, 0.41900E+00_JPRB, 0.41648E+00_JPRB ,&
&  0.42083E+00_JPRB, 0.43104E+00_JPRB, 0.44715E+00_JPRB, 0.47161E+00_JPRB, 0.46919E+00_JPRB, 0.46225E+00_JPRB, 0.45790E+00_JPRB ,&
&  0.45255E+00_JPRB, 0.44372E+00_JPRB, 0.42855E+00_JPRB, 0.43421E+00_JPRB, 0.44184E+00_JPRB, 0.46228E+00_JPRB, 0.49599E+00_JPRB ,&
&  0.41647E+00_JPRB, 0.42508E+00_JPRB, 0.54085E+00_JPRB, 0.56822E+00_JPRB, 0.59923E+00_JPRB, 0.62969E+00_JPRB, 0.65496E+00_JPRB ,&
&  0.68208E+00_JPRB, 0.72432E+00_JPRB, 0.77320E+00_JPRB, 0.82937E+00_JPRB, 0.97732E+00_JPRB, 0.11034E+01_JPRB, 0.11695E+01_JPRB ,&
&  0.13058E+01_JPRB, 0.12247E+01_JPRB, 0.12029E+01_JPRB, 0.13228E+01_JPRB, 0.18064E+01_JPRB, 0.18198E+01_JPRB, 0.17325E+01_JPRB ,&
&  0.16372E+01_JPRB, 0.14659E+01_JPRB, 0.13310E+01_JPRB, 0.12641E+01_JPRB, 0.12087E+01_JPRB, 0.11922E+01_JPRB, 0.12068E+01_JPRB ,&
&  0.11459E+01_JPRB, 0.11260E+01_JPRB, 0.11321E+01_JPRB, 0.11589E+01_JPRB, 0.12019E+01_JPRB, 0.12880E+01_JPRB, 0.14073E+01_JPRB ,&
&  0.15338E+01_JPRB, 0.16351E+01_JPRB, 0.16643E+01_JPRB, 0.16495E+01_JPRB, 0.16141E+01_JPRB, 0.15645E+01_JPRB, 0.14523E+01_JPRB ,&
&  0.12647E+01_JPRB, 0.11391E+01_JPRB, 0.10696E+01_JPRB, 0.10150E+01_JPRB, 0.97061E+00_JPRB, 0.87698E+00_JPRB, 0.72369E+00_JPRB ,&
&  0.55975E+00_JPRB, 0.46755E+00_JPRB, 0.44821E+00_JPRB, 0.44284E+00_JPRB, 0.44403E+00_JPRB, 0.44847E+00_JPRB, 0.45622E+00_JPRB ,&
&  0.45997E+00_JPRB, 0.45560E+00_JPRB, 0.45831E+00_JPRB, 0.47700E+00_JPRB, 0.50340E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.54529E+00_JPRB, 0.54249E+00_JPRB, 0.50975E+00_JPRB, 0.51974E+00_JPRB, 0.49544E+00_JPRB, 0.47805E+00_JPRB, 0.44979E+00_JPRB ,&
&  0.43699E+00_JPRB, 0.41281E+00_JPRB, 0.39652E+00_JPRB, 0.37066E+00_JPRB, 0.35571E+00_JPRB, 0.35056E+00_JPRB, 0.36602E+00_JPRB ,&
&  0.38828E+00_JPRB, 0.40762E+00_JPRB, 0.41950E+00_JPRB, 0.42320E+00_JPRB, 0.42211E+00_JPRB, 0.41467E+00_JPRB, 0.40754E+00_JPRB ,&
&  0.40608E+00_JPRB, 0.41325E+00_JPRB, 0.41339E+00_JPRB, 0.41948E+00_JPRB, 0.42030E+00_JPRB, 0.41311E+00_JPRB, 0.40903E+00_JPRB ,&
&  0.39548E+00_JPRB, 0.38185E+00_JPRB, 0.38042E+00_JPRB, 0.39961E+00_JPRB, 0.42033E+00_JPRB, 0.44126E+00_JPRB, 0.41122E+00_JPRB ,&
&  0.36998E+00_JPRB, 0.36196E+00_JPRB, 0.37825E+00_JPRB, 0.50194E+00_JPRB, 0.54410E+00_JPRB, 0.56356E+00_JPRB, 0.60217E+00_JPRB ,&
&  0.58414E+00_JPRB, 0.58863E+00_JPRB, 0.62556E+00_JPRB, 0.73325E+00_JPRB, 0.80930E+00_JPRB, 0.94205E+00_JPRB, 0.10217E+01_JPRB ,&
&  0.11104E+01_JPRB, 0.12240E+01_JPRB, 0.13440E+01_JPRB, 0.11679E+01_JPRB, 0.12260E+01_JPRB, 0.14650E+01_JPRB, 0.12829E+01_JPRB ,&
&  0.11688E+01_JPRB, 0.10826E+01_JPRB, 0.10994E+01_JPRB, 0.10591E+01_JPRB, 0.99504E+00_JPRB, 0.95977E+00_JPRB, 0.91442E+00_JPRB ,&
&  0.83807E+00_JPRB, 0.82250E+00_JPRB, 0.86176E+00_JPRB, 0.91224E+00_JPRB, 0.96475E+00_JPRB, 0.10476E+01_JPRB, 0.11392E+01_JPRB ,&
&  0.12568E+01_JPRB, 0.13691E+01_JPRB, 0.14344E+01_JPRB, 0.14399E+01_JPRB, 0.14497E+01_JPRB, 0.14254E+01_JPRB, 0.13376E+01_JPRB ,&
&  0.11813E+01_JPRB, 0.10700E+01_JPRB, 0.10010E+01_JPRB, 0.95853E+00_JPRB, 0.90719E+00_JPRB, 0.81419E+00_JPRB, 0.68072E+00_JPRB ,&
&  0.57438E+00_JPRB, 0.49661E+00_JPRB, 0.46202E+00_JPRB, 0.44814E+00_JPRB, 0.45174E+00_JPRB, 0.45896E+00_JPRB, 0.45986E+00_JPRB ,&
&  0.45499E+00_JPRB, 0.44985E+00_JPRB, 0.44987E+00_JPRB, 0.46567E+00_JPRB, 0.51260E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.50611E+00_JPRB, 0.53014E+00_JPRB, 0.55391E+00_JPRB, 0.53728E+00_JPRB, 0.51504E+00_JPRB, 0.46434E+00_JPRB, 0.42865E+00_JPRB ,&
&  0.40674E+00_JPRB, 0.40476E+00_JPRB, 0.40620E+00_JPRB, 0.41042E+00_JPRB, 0.41832E+00_JPRB, 0.43450E+00_JPRB, 0.44608E+00_JPRB ,&
&  0.45513E+00_JPRB, 0.45514E+00_JPRB, 0.45565E+00_JPRB, 0.45297E+00_JPRB, 0.44421E+00_JPRB, 0.42841E+00_JPRB, 0.41571E+00_JPRB ,&
&  0.41155E+00_JPRB, 0.41174E+00_JPRB, 0.40745E+00_JPRB, 0.39838E+00_JPRB, 0.39197E+00_JPRB, 0.38177E+00_JPRB, 0.38557E+00_JPRB ,&
&  0.37894E+00_JPRB, 0.37134E+00_JPRB, 0.38030E+00_JPRB, 0.39686E+00_JPRB, 0.41055E+00_JPRB, 0.43067E+00_JPRB, 0.39187E+00_JPRB ,&
&  0.34921E+00_JPRB, 0.32897E+00_JPRB, 0.32057E+00_JPRB, 0.32818E+00_JPRB, 0.37881E+00_JPRB, 0.50643E+00_JPRB, 0.55579E+00_JPRB ,&
&  0.58449E+00_JPRB, 0.60614E+00_JPRB, 0.63224E+00_JPRB, 0.66546E+00_JPRB, 0.68781E+00_JPRB, 0.75490E+00_JPRB, 0.85357E+00_JPRB ,&
&  0.94278E+00_JPRB, 0.10229E+01_JPRB, 0.11029E+01_JPRB, 0.96118E+00_JPRB, 0.86859E+00_JPRB, 0.88199E+00_JPRB, 0.91530E+00_JPRB ,&
&  0.95916E+00_JPRB, 0.90825E+00_JPRB, 0.90464E+00_JPRB, 0.92107E+00_JPRB, 0.88485E+00_JPRB, 0.84764E+00_JPRB, 0.79406E+00_JPRB ,&
&  0.74751E+00_JPRB, 0.73046E+00_JPRB, 0.76151E+00_JPRB, 0.77129E+00_JPRB, 0.81066E+00_JPRB, 0.87954E+00_JPRB, 0.94204E+00_JPRB ,&
&  0.10239E+01_JPRB, 0.10869E+01_JPRB, 0.11126E+01_JPRB, 0.10980E+01_JPRB, 0.10956E+01_JPRB, 0.11165E+01_JPRB, 0.11123E+01_JPRB ,&
&  0.10309E+01_JPRB, 0.96584E+00_JPRB, 0.91487E+00_JPRB, 0.87485E+00_JPRB, 0.81424E+00_JPRB, 0.75183E+00_JPRB, 0.67590E+00_JPRB ,&
&  0.59571E+00_JPRB, 0.53395E+00_JPRB, 0.50251E+00_JPRB, 0.48729E+00_JPRB, 0.49036E+00_JPRB, 0.50053E+00_JPRB, 0.50310E+00_JPRB ,&
&  0.49505E+00_JPRB, 0.48244E+00_JPRB, 0.47757E+00_JPRB, 0.48285E+00_JPRB, 0.49901E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.54624E+00_JPRB, 0.54853E+00_JPRB, 0.54893E+00_JPRB, 0.54435E+00_JPRB, 0.52816E+00_JPRB, 0.50016E+00_JPRB, 0.48675E+00_JPRB ,&
&  0.48014E+00_JPRB, 0.48358E+00_JPRB, 0.48216E+00_JPRB, 0.47919E+00_JPRB, 0.47984E+00_JPRB, 0.48743E+00_JPRB, 0.49617E+00_JPRB ,&
&  0.50058E+00_JPRB, 0.49578E+00_JPRB, 0.48490E+00_JPRB, 0.46215E+00_JPRB, 0.44328E+00_JPRB, 0.42871E+00_JPRB, 0.42181E+00_JPRB ,&
&  0.42214E+00_JPRB, 0.41568E+00_JPRB, 0.40190E+00_JPRB, 0.38357E+00_JPRB, 0.36840E+00_JPRB, 0.35577E+00_JPRB, 0.35796E+00_JPRB ,&
&  0.36840E+00_JPRB, 0.37239E+00_JPRB, 0.38212E+00_JPRB, 0.38900E+00_JPRB, 0.39680E+00_JPRB, 0.41698E+00_JPRB, 0.36375E+00_JPRB ,&
&  0.32678E+00_JPRB, 0.30956E+00_JPRB, 0.29718E+00_JPRB, 0.29502E+00_JPRB, 0.30438E+00_JPRB, 0.33064E+00_JPRB, 0.38930E+00_JPRB ,&
&  0.51812E+00_JPRB, 0.55352E+00_JPRB, 0.57060E+00_JPRB, 0.59187E+00_JPRB, 0.60733E+00_JPRB, 0.61947E+00_JPRB, 0.66490E+00_JPRB ,&
&  0.72124E+00_JPRB, 0.79266E+00_JPRB, 0.85246E+00_JPRB, 0.86576E+00_JPRB, 0.86461E+00_JPRB, 0.83346E+00_JPRB, 0.89448E+00_JPRB ,&
&  0.11207E+01_JPRB, 0.10210E+01_JPRB, 0.89980E+00_JPRB, 0.81573E+00_JPRB, 0.75333E+00_JPRB, 0.70646E+00_JPRB, 0.70022E+00_JPRB ,&
&  0.70330E+00_JPRB, 0.70268E+00_JPRB, 0.71828E+00_JPRB, 0.72613E+00_JPRB, 0.74077E+00_JPRB, 0.77416E+00_JPRB, 0.83187E+00_JPRB ,&
&  0.88459E+00_JPRB, 0.93843E+00_JPRB, 0.10492E+01_JPRB, 0.98107E+00_JPRB, 0.95323E+00_JPRB, 0.96005E+00_JPRB, 0.97253E+00_JPRB ,&
&  0.94637E+00_JPRB, 0.94203E+00_JPRB, 0.88065E+00_JPRB, 0.80072E+00_JPRB, 0.76127E+00_JPRB, 0.70070E+00_JPRB, 0.64413E+00_JPRB ,&
&  0.59027E+00_JPRB, 0.54944E+00_JPRB, 0.52989E+00_JPRB, 0.52146E+00_JPRB, 0.53296E+00_JPRB, 0.54559E+00_JPRB, 0.55202E+00_JPRB ,&
&  0.55101E+00_JPRB, 0.54824E+00_JPRB, 0.54530E+00_JPRB, 0.54338E+00_JPRB, 0.54284E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.55958E+00_JPRB, 0.55657E+00_JPRB, 0.55132E+00_JPRB, 0.54603E+00_JPRB, 0.53719E+00_JPRB, 0.53104E+00_JPRB, 0.52526E+00_JPRB ,&
&  0.52135E+00_JPRB, 0.51888E+00_JPRB, 0.51636E+00_JPRB, 0.51423E+00_JPRB, 0.51165E+00_JPRB, 0.50999E+00_JPRB, 0.50578E+00_JPRB ,&
&  0.49664E+00_JPRB, 0.46585E+00_JPRB, 0.44289E+00_JPRB, 0.43259E+00_JPRB, 0.42471E+00_JPRB, 0.42112E+00_JPRB, 0.41779E+00_JPRB ,&
&  0.40907E+00_JPRB, 0.39819E+00_JPRB, 0.38193E+00_JPRB, 0.35946E+00_JPRB, 0.34837E+00_JPRB, 0.35125E+00_JPRB, 0.36171E+00_JPRB ,&
&  0.37750E+00_JPRB, 0.38448E+00_JPRB, 0.37840E+00_JPRB, 0.37854E+00_JPRB, 0.39347E+00_JPRB, 0.38605E+00_JPRB, 0.33308E+00_JPRB ,&
&  0.30774E+00_JPRB, 0.29147E+00_JPRB, 0.28645E+00_JPRB, 0.28523E+00_JPRB, 0.29095E+00_JPRB, 0.30943E+00_JPRB, 0.35594E+00_JPRB ,&
&  0.43669E+00_JPRB, 0.53609E+00_JPRB, 0.54513E+00_JPRB, 0.55046E+00_JPRB, 0.55835E+00_JPRB, 0.56573E+00_JPRB, 0.57949E+00_JPRB ,&
&  0.60261E+00_JPRB, 0.63324E+00_JPRB, 0.66003E+00_JPRB, 0.68965E+00_JPRB, 0.71594E+00_JPRB, 0.72741E+00_JPRB, 0.73224E+00_JPRB ,&
&  0.74672E+00_JPRB, 0.72623E+00_JPRB, 0.71750E+00_JPRB, 0.71030E+00_JPRB, 0.70752E+00_JPRB, 0.68034E+00_JPRB, 0.67296E+00_JPRB ,&
&  0.68640E+00_JPRB, 0.70065E+00_JPRB, 0.71889E+00_JPRB, 0.72819E+00_JPRB, 0.74194E+00_JPRB, 0.77223E+00_JPRB, 0.79809E+00_JPRB ,&
&  0.81694E+00_JPRB, 0.84893E+00_JPRB, 0.89775E+00_JPRB, 0.89760E+00_JPRB, 0.89631E+00_JPRB, 0.89658E+00_JPRB, 0.89798E+00_JPRB ,&
&  0.89356E+00_JPRB, 0.90669E+00_JPRB, 0.87684E+00_JPRB, 0.82178E+00_JPRB, 0.76561E+00_JPRB, 0.69133E+00_JPRB, 0.63803E+00_JPRB ,&
&  0.60016E+00_JPRB, 0.55567E+00_JPRB, 0.53774E+00_JPRB, 0.53598E+00_JPRB, 0.54659E+00_JPRB, 0.55554E+00_JPRB, 0.55902E+00_JPRB ,&
&  0.56444E+00_JPRB, 0.56868E+00_JPRB, 0.57087E+00_JPRB, 0.56887E+00_JPRB, 0.56456E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.54506E+00_JPRB, 0.54041E+00_JPRB, 0.53787E+00_JPRB, 0.53626E+00_JPRB, 0.52998E+00_JPRB, 0.52340E+00_JPRB, 0.51429E+00_JPRB ,&
&  0.50750E+00_JPRB, 0.50193E+00_JPRB, 0.49738E+00_JPRB, 0.49156E+00_JPRB, 0.48539E+00_JPRB, 0.47820E+00_JPRB, 0.47265E+00_JPRB ,&
&  0.46642E+00_JPRB, 0.45846E+00_JPRB, 0.44524E+00_JPRB, 0.43283E+00_JPRB, 0.42372E+00_JPRB, 0.41813E+00_JPRB, 0.41124E+00_JPRB ,&
&  0.40567E+00_JPRB, 0.39511E+00_JPRB, 0.37344E+00_JPRB, 0.35525E+00_JPRB, 0.34044E+00_JPRB, 0.33963E+00_JPRB, 0.36683E+00_JPRB ,&
&  0.37950E+00_JPRB, 0.38187E+00_JPRB, 0.38317E+00_JPRB, 0.37732E+00_JPRB, 0.37053E+00_JPRB, 0.31903E+00_JPRB, 0.31480E+00_JPRB ,&
&  0.30023E+00_JPRB, 0.28996E+00_JPRB, 0.28767E+00_JPRB, 0.28820E+00_JPRB, 0.29343E+00_JPRB, 0.30683E+00_JPRB, 0.32691E+00_JPRB ,&
&  0.36356E+00_JPRB, 0.47631E+00_JPRB, 0.50364E+00_JPRB, 0.52543E+00_JPRB, 0.52934E+00_JPRB, 0.53115E+00_JPRB, 0.53672E+00_JPRB ,&
&  0.54490E+00_JPRB, 0.55876E+00_JPRB, 0.56896E+00_JPRB, 0.58341E+00_JPRB, 0.59772E+00_JPRB, 0.60689E+00_JPRB, 0.62705E+00_JPRB ,&
&  0.64085E+00_JPRB, 0.64858E+00_JPRB, 0.65077E+00_JPRB, 0.65006E+00_JPRB, 0.65831E+00_JPRB, 0.64843E+00_JPRB, 0.64037E+00_JPRB ,&
&  0.66012E+00_JPRB, 0.69590E+00_JPRB, 0.72463E+00_JPRB, 0.75288E+00_JPRB, 0.76364E+00_JPRB, 0.77930E+00_JPRB, 0.80334E+00_JPRB ,&
&  0.81577E+00_JPRB, 0.83994E+00_JPRB, 0.87049E+00_JPRB, 0.87216E+00_JPRB, 0.86813E+00_JPRB, 0.86269E+00_JPRB, 0.86760E+00_JPRB ,&
&  0.86952E+00_JPRB, 0.87284E+00_JPRB, 0.84681E+00_JPRB, 0.82212E+00_JPRB, 0.78287E+00_JPRB, 0.73291E+00_JPRB, 0.69298E+00_JPRB ,&
&  0.64813E+00_JPRB, 0.61406E+00_JPRB, 0.59911E+00_JPRB, 0.59514E+00_JPRB, 0.59726E+00_JPRB, 0.60131E+00_JPRB, 0.60130E+00_JPRB ,&
&  0.59980E+00_JPRB, 0.59192E+00_JPRB, 0.57973E+00_JPRB, 0.56557E+00_JPRB, 0.55330E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.52999E+00_JPRB, 0.52312E+00_JPRB, 0.51711E+00_JPRB, 0.50979E+00_JPRB, 0.50165E+00_JPRB, 0.49524E+00_JPRB, 0.48983E+00_JPRB ,&
&  0.48631E+00_JPRB, 0.48218E+00_JPRB, 0.47857E+00_JPRB, 0.47340E+00_JPRB, 0.46891E+00_JPRB, 0.46502E+00_JPRB, 0.46190E+00_JPRB ,&
&  0.45971E+00_JPRB, 0.45359E+00_JPRB, 0.44655E+00_JPRB, 0.43777E+00_JPRB, 0.42983E+00_JPRB, 0.42227E+00_JPRB, 0.41352E+00_JPRB ,&
&  0.40632E+00_JPRB, 0.39186E+00_JPRB, 0.37279E+00_JPRB, 0.36270E+00_JPRB, 0.34532E+00_JPRB, 0.33761E+00_JPRB, 0.33650E+00_JPRB ,&
&  0.35423E+00_JPRB, 0.34387E+00_JPRB, 0.34765E+00_JPRB, 0.33391E+00_JPRB, 0.32148E+00_JPRB, 0.30997E+00_JPRB, 0.30245E+00_JPRB ,&
&  0.29673E+00_JPRB, 0.29208E+00_JPRB, 0.29222E+00_JPRB, 0.29525E+00_JPRB, 0.29839E+00_JPRB, 0.30893E+00_JPRB, 0.32446E+00_JPRB ,&
&  0.36159E+00_JPRB, 0.45938E+00_JPRB, 0.47868E+00_JPRB, 0.49971E+00_JPRB, 0.50299E+00_JPRB, 0.50414E+00_JPRB, 0.50648E+00_JPRB ,&
&  0.51092E+00_JPRB, 0.51610E+00_JPRB, 0.51969E+00_JPRB, 0.52000E+00_JPRB, 0.53781E+00_JPRB, 0.55240E+00_JPRB, 0.57045E+00_JPRB ,&
&  0.57914E+00_JPRB, 0.58449E+00_JPRB, 0.58678E+00_JPRB, 0.59460E+00_JPRB, 0.61322E+00_JPRB, 0.61587E+00_JPRB, 0.61024E+00_JPRB ,&
&  0.61965E+00_JPRB, 0.63968E+00_JPRB, 0.67560E+00_JPRB, 0.72252E+00_JPRB, 0.74814E+00_JPRB, 0.77174E+00_JPRB, 0.81127E+00_JPRB ,&
&  0.83759E+00_JPRB, 0.88208E+00_JPRB, 0.92405E+00_JPRB, 0.93467E+00_JPRB, 0.92539E+00_JPRB, 0.90518E+00_JPRB, 0.88275E+00_JPRB ,&
&  0.86896E+00_JPRB, 0.84458E+00_JPRB, 0.82324E+00_JPRB, 0.80630E+00_JPRB, 0.78650E+00_JPRB, 0.76376E+00_JPRB, 0.74701E+00_JPRB ,&
&  0.72562E+00_JPRB, 0.70866E+00_JPRB, 0.69275E+00_JPRB, 0.67561E+00_JPRB, 0.65829E+00_JPRB, 0.64015E+00_JPRB, 0.62047E+00_JPRB ,&
&  0.60020E+00_JPRB, 0.58127E+00_JPRB, 0.56216E+00_JPRB, 0.54818E+00_JPRB, 0.53676E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.51490E+00_JPRB, 0.50466E+00_JPRB, 0.49405E+00_JPRB, 0.48516E+00_JPRB, 0.47821E+00_JPRB, 0.47388E+00_JPRB, 0.47241E+00_JPRB ,&
&  0.47191E+00_JPRB, 0.47030E+00_JPRB, 0.46783E+00_JPRB, 0.46484E+00_JPRB, 0.46195E+00_JPRB, 0.46156E+00_JPRB, 0.46118E+00_JPRB ,&
&  0.46088E+00_JPRB, 0.45784E+00_JPRB, 0.45257E+00_JPRB, 0.44511E+00_JPRB, 0.43681E+00_JPRB, 0.42994E+00_JPRB, 0.42013E+00_JPRB ,&
&  0.41026E+00_JPRB, 0.39897E+00_JPRB, 0.37888E+00_JPRB, 0.36678E+00_JPRB, 0.34793E+00_JPRB, 0.33579E+00_JPRB, 0.32977E+00_JPRB ,&
&  0.32605E+00_JPRB, 0.32724E+00_JPRB, 0.33151E+00_JPRB, 0.33560E+00_JPRB, 0.33191E+00_JPRB, 0.32607E+00_JPRB, 0.32328E+00_JPRB ,&
&  0.31784E+00_JPRB, 0.31509E+00_JPRB, 0.31322E+00_JPRB, 0.31376E+00_JPRB, 0.31687E+00_JPRB, 0.32466E+00_JPRB, 0.33970E+00_JPRB ,&
&  0.37267E+00_JPRB, 0.41820E+00_JPRB, 0.46178E+00_JPRB, 0.48372E+00_JPRB, 0.48277E+00_JPRB, 0.48763E+00_JPRB, 0.48849E+00_JPRB ,&
&  0.49128E+00_JPRB, 0.49330E+00_JPRB, 0.49755E+00_JPRB, 0.50372E+00_JPRB, 0.51380E+00_JPRB, 0.52305E+00_JPRB, 0.53292E+00_JPRB ,&
&  0.53687E+00_JPRB, 0.54037E+00_JPRB, 0.53845E+00_JPRB, 0.54423E+00_JPRB, 0.55598E+00_JPRB, 0.56487E+00_JPRB, 0.56588E+00_JPRB ,&
&  0.57220E+00_JPRB, 0.58396E+00_JPRB, 0.61097E+00_JPRB, 0.65425E+00_JPRB, 0.69855E+00_JPRB, 0.74062E+00_JPRB, 0.78036E+00_JPRB ,&
&  0.82036E+00_JPRB, 0.86704E+00_JPRB, 0.90856E+00_JPRB, 0.91989E+00_JPRB, 0.91422E+00_JPRB, 0.91406E+00_JPRB, 0.89445E+00_JPRB ,&
&  0.87539E+00_JPRB, 0.85378E+00_JPRB, 0.83754E+00_JPRB, 0.82406E+00_JPRB, 0.81097E+00_JPRB, 0.79055E+00_JPRB, 0.77083E+00_JPRB ,&
&  0.74812E+00_JPRB, 0.72268E+00_JPRB, 0.69640E+00_JPRB, 0.67045E+00_JPRB, 0.64607E+00_JPRB, 0.62289E+00_JPRB, 0.60233E+00_JPRB ,&
&  0.58251E+00_JPRB, 0.56581E+00_JPRB, 0.54996E+00_JPRB, 0.53668E+00_JPRB, 0.52561E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.50555E+00_JPRB, 0.49664E+00_JPRB, 0.48863E+00_JPRB, 0.48187E+00_JPRB, 0.47657E+00_JPRB, 0.47333E+00_JPRB, 0.47159E+00_JPRB ,&
&  0.47064E+00_JPRB, 0.47103E+00_JPRB, 0.47110E+00_JPRB, 0.47090E+00_JPRB, 0.47041E+00_JPRB, 0.47157E+00_JPRB, 0.47190E+00_JPRB ,&
&  0.47124E+00_JPRB, 0.46941E+00_JPRB, 0.46742E+00_JPRB, 0.46425E+00_JPRB, 0.45810E+00_JPRB, 0.45042E+00_JPRB, 0.43939E+00_JPRB ,&
&  0.43226E+00_JPRB, 0.42260E+00_JPRB, 0.41113E+00_JPRB, 0.39849E+00_JPRB, 0.39012E+00_JPRB, 0.37071E+00_JPRB, 0.36309E+00_JPRB ,&
&  0.35795E+00_JPRB, 0.35008E+00_JPRB, 0.34634E+00_JPRB, 0.34635E+00_JPRB, 0.36217E+00_JPRB, 0.37407E+00_JPRB, 0.38017E+00_JPRB ,&
&  0.36811E+00_JPRB, 0.37013E+00_JPRB, 0.36723E+00_JPRB, 0.36603E+00_JPRB, 0.36874E+00_JPRB, 0.37556E+00_JPRB, 0.39951E+00_JPRB ,&
&  0.44150E+00_JPRB, 0.45858E+00_JPRB, 0.46831E+00_JPRB, 0.47754E+00_JPRB, 0.48243E+00_JPRB, 0.48561E+00_JPRB, 0.48758E+00_JPRB ,&
&  0.49350E+00_JPRB, 0.49871E+00_JPRB, 0.50325E+00_JPRB, 0.50575E+00_JPRB, 0.50892E+00_JPRB, 0.51226E+00_JPRB, 0.51566E+00_JPRB ,&
&  0.51214E+00_JPRB, 0.51018E+00_JPRB, 0.50986E+00_JPRB, 0.51250E+00_JPRB, 0.51654E+00_JPRB, 0.52406E+00_JPRB, 0.53313E+00_JPRB ,&
&  0.54376E+00_JPRB, 0.55129E+00_JPRB, 0.56647E+00_JPRB, 0.58933E+00_JPRB, 0.61629E+00_JPRB, 0.65034E+00_JPRB, 0.68570E+00_JPRB ,&
&  0.71979E+00_JPRB, 0.74780E+00_JPRB, 0.77287E+00_JPRB, 0.78366E+00_JPRB, 0.78855E+00_JPRB, 0.79163E+00_JPRB, 0.78827E+00_JPRB ,&
&  0.78193E+00_JPRB, 0.77203E+00_JPRB, 0.76206E+00_JPRB, 0.74902E+00_JPRB, 0.73508E+00_JPRB, 0.71990E+00_JPRB, 0.70436E+00_JPRB ,&
&  0.68513E+00_JPRB, 0.66603E+00_JPRB, 0.64685E+00_JPRB, 0.62826E+00_JPRB, 0.61014E+00_JPRB, 0.59359E+00_JPRB, 0.57844E+00_JPRB ,&
&  0.56464E+00_JPRB, 0.55086E+00_JPRB, 0.53881E+00_JPRB, 0.52750E+00_JPRB, 0.51640E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.49511E+00_JPRB, 0.49037E+00_JPRB, 0.48656E+00_JPRB, 0.48326E+00_JPRB, 0.48106E+00_JPRB, 0.48001E+00_JPRB, 0.47942E+00_JPRB ,&
&  0.47901E+00_JPRB, 0.47960E+00_JPRB, 0.48056E+00_JPRB, 0.48150E+00_JPRB, 0.48243E+00_JPRB, 0.48315E+00_JPRB, 0.48346E+00_JPRB ,&
&  0.48364E+00_JPRB, 0.48429E+00_JPRB, 0.48446E+00_JPRB, 0.48444E+00_JPRB, 0.48334E+00_JPRB, 0.48081E+00_JPRB, 0.47710E+00_JPRB ,&
&  0.47287E+00_JPRB, 0.46720E+00_JPRB, 0.46025E+00_JPRB, 0.45253E+00_JPRB, 0.44826E+00_JPRB, 0.44317E+00_JPRB, 0.43833E+00_JPRB ,&
&  0.43474E+00_JPRB, 0.43440E+00_JPRB, 0.43381E+00_JPRB, 0.43313E+00_JPRB, 0.43285E+00_JPRB, 0.43400E+00_JPRB, 0.43471E+00_JPRB ,&
&  0.43553E+00_JPRB, 0.43798E+00_JPRB, 0.44426E+00_JPRB, 0.45081E+00_JPRB, 0.45820E+00_JPRB, 0.46688E+00_JPRB, 0.47691E+00_JPRB ,&
&  0.48431E+00_JPRB, 0.48799E+00_JPRB, 0.49147E+00_JPRB, 0.49499E+00_JPRB, 0.49670E+00_JPRB, 0.49734E+00_JPRB, 0.49769E+00_JPRB ,&
&  0.49898E+00_JPRB, 0.49944E+00_JPRB, 0.50045E+00_JPRB, 0.50184E+00_JPRB, 0.50133E+00_JPRB, 0.50061E+00_JPRB, 0.49949E+00_JPRB ,&
&  0.49726E+00_JPRB, 0.49630E+00_JPRB, 0.49656E+00_JPRB, 0.49665E+00_JPRB, 0.49763E+00_JPRB, 0.50041E+00_JPRB, 0.50438E+00_JPRB ,&
&  0.50969E+00_JPRB, 0.51580E+00_JPRB, 0.52555E+00_JPRB, 0.53715E+00_JPRB, 0.55070E+00_JPRB, 0.56746E+00_JPRB, 0.58163E+00_JPRB ,&
&  0.59507E+00_JPRB, 0.60699E+00_JPRB, 0.61760E+00_JPRB, 0.62511E+00_JPRB, 0.63038E+00_JPRB, 0.63548E+00_JPRB, 0.63532E+00_JPRB ,&
&  0.63395E+00_JPRB, 0.63139E+00_JPRB, 0.62878E+00_JPRB, 0.62275E+00_JPRB, 0.61525E+00_JPRB, 0.60754E+00_JPRB, 0.60039E+00_JPRB ,&
&  0.59058E+00_JPRB, 0.58069E+00_JPRB, 0.57158E+00_JPRB, 0.56347E+00_JPRB, 0.55491E+00_JPRB, 0.54700E+00_JPRB, 0.53900E+00_JPRB ,&
&  0.53092E+00_JPRB, 0.52273E+00_JPRB, 0.51548E+00_JPRB, 0.50849E+00_JPRB, 0.50156E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.49612E+00_JPRB, 0.49566E+00_JPRB, 0.49539E+00_JPRB, 0.49485E+00_JPRB, 0.49402E+00_JPRB, 0.49367E+00_JPRB, 0.49346E+00_JPRB ,&
&  0.49320E+00_JPRB, 0.49336E+00_JPRB, 0.49385E+00_JPRB, 0.49448E+00_JPRB, 0.49546E+00_JPRB, 0.49642E+00_JPRB, 0.49749E+00_JPRB ,&
&  0.49854E+00_JPRB, 0.49959E+00_JPRB, 0.50070E+00_JPRB, 0.50155E+00_JPRB, 0.50204E+00_JPRB, 0.50209E+00_JPRB, 0.50179E+00_JPRB ,&
&  0.50102E+00_JPRB, 0.50016E+00_JPRB, 0.49940E+00_JPRB, 0.49869E+00_JPRB, 0.49757E+00_JPRB, 0.49620E+00_JPRB, 0.49462E+00_JPRB ,&
&  0.49324E+00_JPRB, 0.49236E+00_JPRB, 0.49164E+00_JPRB, 0.49083E+00_JPRB, 0.49037E+00_JPRB, 0.49089E+00_JPRB, 0.49140E+00_JPRB ,&
&  0.49198E+00_JPRB, 0.49286E+00_JPRB, 0.49468E+00_JPRB, 0.49645E+00_JPRB, 0.49809E+00_JPRB, 0.49967E+00_JPRB, 0.50110E+00_JPRB ,&
&  0.50166E+00_JPRB, 0.50187E+00_JPRB, 0.50178E+00_JPRB, 0.50148E+00_JPRB, 0.50056E+00_JPRB, 0.49973E+00_JPRB, 0.49894E+00_JPRB ,&
&  0.49801E+00_JPRB, 0.49721E+00_JPRB, 0.49641E+00_JPRB, 0.49519E+00_JPRB, 0.49258E+00_JPRB, 0.49076E+00_JPRB, 0.49002E+00_JPRB ,&
&  0.49012E+00_JPRB, 0.49019E+00_JPRB, 0.49031E+00_JPRB, 0.49040E+00_JPRB, 0.49093E+00_JPRB, 0.49160E+00_JPRB, 0.49264E+00_JPRB ,&
&  0.49352E+00_JPRB, 0.49484E+00_JPRB, 0.49713E+00_JPRB, 0.49956E+00_JPRB, 0.50232E+00_JPRB, 0.50542E+00_JPRB, 0.50822E+00_JPRB ,&
&  0.51129E+00_JPRB, 0.51472E+00_JPRB, 0.51775E+00_JPRB, 0.52080E+00_JPRB, 0.52402E+00_JPRB, 0.52718E+00_JPRB, 0.52937E+00_JPRB ,&
&  0.53037E+00_JPRB, 0.53115E+00_JPRB, 0.53165E+00_JPRB, 0.53089E+00_JPRB, 0.52927E+00_JPRB, 0.52773E+00_JPRB, 0.52599E+00_JPRB ,&
&  0.52406E+00_JPRB, 0.52181E+00_JPRB, 0.51970E+00_JPRB, 0.51736E+00_JPRB, 0.51467E+00_JPRB, 0.51207E+00_JPRB, 0.50951E+00_JPRB ,&
&  0.50686E+00_JPRB, 0.50416E+00_JPRB, 0.50195E+00_JPRB, 0.49982E+00_JPRB, 0.49780E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB, 0.50725E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB ,&
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB ,&
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB ,&
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB ,&
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB ,&
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB ,&
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB ,&
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB ,&
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB ,&
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB ,&
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB ,&
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB ,&
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB ,&
&  0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB, 0.11762E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.12316E+00_JPRB, 0.12302E+00_JPRB, 0.12288E+00_JPRB, 0.12273E+00_JPRB, 0.12219E+00_JPRB, 0.12199E+00_JPRB, 0.12178E+00_JPRB ,&
&  0.12168E+00_JPRB, 0.12162E+00_JPRB, 0.12168E+00_JPRB, 0.12180E+00_JPRB, 0.12191E+00_JPRB, 0.12200E+00_JPRB, 0.12207E+00_JPRB ,&
&  0.12203E+00_JPRB, 0.12201E+00_JPRB, 0.12146E+00_JPRB, 0.12105E+00_JPRB, 0.12080E+00_JPRB, 0.12063E+00_JPRB, 0.12037E+00_JPRB ,&
&  0.12018E+00_JPRB, 0.11992E+00_JPRB, 0.11961E+00_JPRB, 0.11937E+00_JPRB, 0.11936E+00_JPRB, 0.11923E+00_JPRB, 0.11922E+00_JPRB ,&
&  0.11917E+00_JPRB, 0.11928E+00_JPRB, 0.11928E+00_JPRB, 0.11932E+00_JPRB, 0.11955E+00_JPRB, 0.11974E+00_JPRB, 0.11974E+00_JPRB ,&
&  0.11971E+00_JPRB, 0.11952E+00_JPRB, 0.11933E+00_JPRB, 0.11920E+00_JPRB, 0.11912E+00_JPRB, 0.11908E+00_JPRB, 0.11902E+00_JPRB ,&
&  0.11889E+00_JPRB, 0.11869E+00_JPRB, 0.11846E+00_JPRB, 0.11820E+00_JPRB, 0.11795E+00_JPRB, 0.11779E+00_JPRB, 0.11773E+00_JPRB ,&
&  0.11768E+00_JPRB, 0.11754E+00_JPRB, 0.11734E+00_JPRB, 0.11706E+00_JPRB, 0.11681E+00_JPRB, 0.11656E+00_JPRB, 0.11633E+00_JPRB ,&
&  0.11612E+00_JPRB, 0.11601E+00_JPRB, 0.11589E+00_JPRB, 0.11579E+00_JPRB, 0.11572E+00_JPRB, 0.11562E+00_JPRB, 0.11553E+00_JPRB ,&
&  0.11544E+00_JPRB, 0.11537E+00_JPRB, 0.11530E+00_JPRB, 0.11524E+00_JPRB, 0.11521E+00_JPRB, 0.11518E+00_JPRB, 0.11519E+00_JPRB ,&
&  0.11522E+00_JPRB, 0.11525E+00_JPRB, 0.11530E+00_JPRB, 0.11543E+00_JPRB, 0.11558E+00_JPRB, 0.11576E+00_JPRB, 0.11598E+00_JPRB ,&
&  0.11621E+00_JPRB, 0.11648E+00_JPRB, 0.11678E+00_JPRB, 0.11700E+00_JPRB, 0.11733E+00_JPRB, 0.11766E+00_JPRB, 0.11800E+00_JPRB ,&
&  0.11834E+00_JPRB, 0.11877E+00_JPRB, 0.11924E+00_JPRB, 0.11973E+00_JPRB, 0.12026E+00_JPRB, 0.12058E+00_JPRB, 0.12095E+00_JPRB ,&
&  0.12137E+00_JPRB, 0.12189E+00_JPRB, 0.12219E+00_JPRB, 0.12242E+00_JPRB, 0.12269E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.13295E+00_JPRB, 0.13494E+00_JPRB, 0.13646E+00_JPRB, 0.13834E+00_JPRB, 0.13840E+00_JPRB, 0.14977E+00_JPRB, 0.14791E+00_JPRB ,&
&  0.14682E+00_JPRB, 0.14187E+00_JPRB, 0.14134E+00_JPRB, 0.13971E+00_JPRB, 0.13779E+00_JPRB, 0.13569E+00_JPRB, 0.13377E+00_JPRB ,&
&  0.13200E+00_JPRB, 0.13005E+00_JPRB, 0.12860E+00_JPRB, 0.12741E+00_JPRB, 0.12633E+00_JPRB, 0.12534E+00_JPRB, 0.12455E+00_JPRB ,&
&  0.12445E+00_JPRB, 0.12492E+00_JPRB, 0.12581E+00_JPRB, 0.12570E+00_JPRB, 0.12547E+00_JPRB, 0.12514E+00_JPRB, 0.12498E+00_JPRB ,&
&  0.12449E+00_JPRB, 0.12413E+00_JPRB, 0.12381E+00_JPRB, 0.12333E+00_JPRB, 0.12375E+00_JPRB, 0.12414E+00_JPRB, 0.12454E+00_JPRB ,&
&  0.12492E+00_JPRB, 0.12528E+00_JPRB, 0.12518E+00_JPRB, 0.12500E+00_JPRB, 0.12460E+00_JPRB, 0.12404E+00_JPRB, 0.12322E+00_JPRB ,&
&  0.12243E+00_JPRB, 0.12162E+00_JPRB, 0.12082E+00_JPRB, 0.12024E+00_JPRB, 0.11973E+00_JPRB, 0.11925E+00_JPRB, 0.11884E+00_JPRB ,&
&  0.11863E+00_JPRB, 0.11850E+00_JPRB, 0.11834E+00_JPRB, 0.11808E+00_JPRB, 0.11776E+00_JPRB, 0.11742E+00_JPRB, 0.11713E+00_JPRB ,&
&  0.11690E+00_JPRB, 0.11649E+00_JPRB, 0.11612E+00_JPRB, 0.11573E+00_JPRB, 0.11544E+00_JPRB, 0.11507E+00_JPRB, 0.11476E+00_JPRB ,&
&  0.11448E+00_JPRB, 0.11428E+00_JPRB, 0.11422E+00_JPRB, 0.11420E+00_JPRB, 0.11419E+00_JPRB, 0.11424E+00_JPRB, 0.11436E+00_JPRB ,&
&  0.11452E+00_JPRB, 0.11471E+00_JPRB, 0.11494E+00_JPRB, 0.11512E+00_JPRB, 0.11529E+00_JPRB, 0.11544E+00_JPRB, 0.11580E+00_JPRB ,&
&  0.11622E+00_JPRB, 0.11667E+00_JPRB, 0.11711E+00_JPRB, 0.11737E+00_JPRB, 0.11810E+00_JPRB, 0.11884E+00_JPRB, 0.11965E+00_JPRB ,&
&  0.12050E+00_JPRB, 0.12159E+00_JPRB, 0.12276E+00_JPRB, 0.12399E+00_JPRB, 0.12539E+00_JPRB, 0.12637E+00_JPRB, 0.12741E+00_JPRB ,&
&  0.12878E+00_JPRB, 0.13102E+00_JPRB, 0.13193E+00_JPRB, 0.13238E+00_JPRB, 0.13274E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.15655E+00_JPRB, 0.15716E+00_JPRB, 0.15770E+00_JPRB, 0.15793E+00_JPRB, 0.15797E+00_JPRB, 0.15739E+00_JPRB, 0.15669E+00_JPRB ,&
&  0.15328E+00_JPRB, 0.15073E+00_JPRB, 0.14799E+00_JPRB, 0.14528E+00_JPRB, 0.14290E+00_JPRB, 0.14067E+00_JPRB, 0.13857E+00_JPRB ,&
&  0.13667E+00_JPRB, 0.13529E+00_JPRB, 0.13393E+00_JPRB, 0.13202E+00_JPRB, 0.12957E+00_JPRB, 0.12733E+00_JPRB, 0.12580E+00_JPRB ,&
&  0.12529E+00_JPRB, 0.12548E+00_JPRB, 0.12668E+00_JPRB, 0.12799E+00_JPRB, 0.12940E+00_JPRB, 0.13048E+00_JPRB, 0.13154E+00_JPRB ,&
&  0.13097E+00_JPRB, 0.13057E+00_JPRB, 0.13022E+00_JPRB, 0.12967E+00_JPRB, 0.13009E+00_JPRB, 0.13066E+00_JPRB, 0.13096E+00_JPRB ,&
&  0.13130E+00_JPRB, 0.13231E+00_JPRB, 0.13298E+00_JPRB, 0.13361E+00_JPRB, 0.13338E+00_JPRB, 0.13252E+00_JPRB, 0.13110E+00_JPRB ,&
&  0.12942E+00_JPRB, 0.12772E+00_JPRB, 0.12625E+00_JPRB, 0.12499E+00_JPRB, 0.12403E+00_JPRB, 0.12301E+00_JPRB, 0.12218E+00_JPRB ,&
&  0.12177E+00_JPRB, 0.12139E+00_JPRB, 0.12084E+00_JPRB, 0.12027E+00_JPRB, 0.11940E+00_JPRB, 0.11865E+00_JPRB, 0.11786E+00_JPRB ,&
&  0.11713E+00_JPRB, 0.11643E+00_JPRB, 0.11580E+00_JPRB, 0.11515E+00_JPRB, 0.11471E+00_JPRB, 0.11430E+00_JPRB, 0.11390E+00_JPRB ,&
&  0.11348E+00_JPRB, 0.11332E+00_JPRB, 0.11302E+00_JPRB, 0.11292E+00_JPRB, 0.11276E+00_JPRB, 0.11265E+00_JPRB, 0.11282E+00_JPRB ,&
&  0.11327E+00_JPRB, 0.11374E+00_JPRB, 0.11399E+00_JPRB, 0.11429E+00_JPRB, 0.11457E+00_JPRB, 0.11494E+00_JPRB, 0.11552E+00_JPRB ,&
&  0.11597E+00_JPRB, 0.11655E+00_JPRB, 0.11706E+00_JPRB, 0.11765E+00_JPRB, 0.11871E+00_JPRB, 0.11982E+00_JPRB, 0.12108E+00_JPRB ,&
&  0.12265E+00_JPRB, 0.12432E+00_JPRB, 0.12627E+00_JPRB, 0.12818E+00_JPRB, 0.13069E+00_JPRB, 0.13241E+00_JPRB, 0.13550E+00_JPRB ,&
&  0.13895E+00_JPRB, 0.14064E+00_JPRB, 0.15182E+00_JPRB, 0.15290E+00_JPRB, 0.15441E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.16159E+00_JPRB, 0.16268E+00_JPRB, 0.16261E+00_JPRB, 0.16302E+00_JPRB, 0.16228E+00_JPRB, 0.16104E+00_JPRB, 0.15933E+00_JPRB ,&
&  0.15708E+00_JPRB, 0.15169E+00_JPRB, 0.14803E+00_JPRB, 0.14400E+00_JPRB, 0.14058E+00_JPRB, 0.13766E+00_JPRB, 0.13562E+00_JPRB ,&
&  0.13478E+00_JPRB, 0.13460E+00_JPRB, 0.13453E+00_JPRB, 0.13381E+00_JPRB, 0.13245E+00_JPRB, 0.13154E+00_JPRB, 0.13035E+00_JPRB ,&
&  0.12948E+00_JPRB, 0.12936E+00_JPRB, 0.13006E+00_JPRB, 0.13174E+00_JPRB, 0.13468E+00_JPRB, 0.13698E+00_JPRB, 0.13903E+00_JPRB ,&
&  0.14133E+00_JPRB, 0.13839E+00_JPRB, 0.13807E+00_JPRB, 0.13821E+00_JPRB, 0.13793E+00_JPRB, 0.13704E+00_JPRB, 0.13526E+00_JPRB ,&
&  0.13740E+00_JPRB, 0.13793E+00_JPRB, 0.14211E+00_JPRB, 0.14208E+00_JPRB, 0.14102E+00_JPRB, 0.14153E+00_JPRB, 0.13987E+00_JPRB ,&
&  0.13906E+00_JPRB, 0.13623E+00_JPRB, 0.13416E+00_JPRB, 0.13147E+00_JPRB, 0.12894E+00_JPRB, 0.12767E+00_JPRB, 0.12622E+00_JPRB ,&
&  0.12573E+00_JPRB, 0.12561E+00_JPRB, 0.12450E+00_JPRB, 0.12381E+00_JPRB, 0.12259E+00_JPRB, 0.12120E+00_JPRB, 0.12029E+00_JPRB ,&
&  0.11936E+00_JPRB, 0.11860E+00_JPRB, 0.11818E+00_JPRB, 0.11781E+00_JPRB, 0.11766E+00_JPRB, 0.11760E+00_JPRB, 0.11754E+00_JPRB ,&
&  0.11756E+00_JPRB, 0.11747E+00_JPRB, 0.11700E+00_JPRB, 0.11671E+00_JPRB, 0.11588E+00_JPRB, 0.11531E+00_JPRB, 0.11485E+00_JPRB ,&
&  0.11447E+00_JPRB, 0.11435E+00_JPRB, 0.11428E+00_JPRB, 0.11434E+00_JPRB, 0.11446E+00_JPRB, 0.11484E+00_JPRB, 0.11527E+00_JPRB ,&
&  0.11572E+00_JPRB, 0.11647E+00_JPRB, 0.11708E+00_JPRB, 0.11791E+00_JPRB, 0.11891E+00_JPRB, 0.11976E+00_JPRB, 0.12110E+00_JPRB ,&
&  0.12261E+00_JPRB, 0.12437E+00_JPRB, 0.12696E+00_JPRB, 0.12946E+00_JPRB, 0.13230E+00_JPRB, 0.13634E+00_JPRB, 0.14084E+00_JPRB ,&
&  0.14585E+00_JPRB, 0.15211E+00_JPRB, 0.15297E+00_JPRB, 0.15718E+00_JPRB, 0.16002E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.16166E+00_JPRB, 0.16317E+00_JPRB, 0.16390E+00_JPRB, 0.16493E+00_JPRB, 0.16499E+00_JPRB, 0.16299E+00_JPRB, 0.16103E+00_JPRB ,&
&  0.15628E+00_JPRB, 0.15235E+00_JPRB, 0.14760E+00_JPRB, 0.14309E+00_JPRB, 0.13888E+00_JPRB, 0.13612E+00_JPRB, 0.13458E+00_JPRB ,&
&  0.13478E+00_JPRB, 0.13628E+00_JPRB, 0.13744E+00_JPRB, 0.13809E+00_JPRB, 0.13865E+00_JPRB, 0.13973E+00_JPRB, 0.14005E+00_JPRB ,&
&  0.13977E+00_JPRB, 0.13869E+00_JPRB, 0.13790E+00_JPRB, 0.13882E+00_JPRB, 0.14072E+00_JPRB, 0.14271E+00_JPRB, 0.14436E+00_JPRB ,&
&  0.14588E+00_JPRB, 0.14542E+00_JPRB, 0.14544E+00_JPRB, 0.14468E+00_JPRB, 0.14396E+00_JPRB, 0.14309E+00_JPRB, 0.14260E+00_JPRB ,&
&  0.14295E+00_JPRB, 0.14328E+00_JPRB, 0.14461E+00_JPRB, 0.14542E+00_JPRB, 0.14417E+00_JPRB, 0.14533E+00_JPRB, 0.14495E+00_JPRB ,&
&  0.14388E+00_JPRB, 0.14108E+00_JPRB, 0.13819E+00_JPRB, 0.13597E+00_JPRB, 0.13374E+00_JPRB, 0.13197E+00_JPRB, 0.13053E+00_JPRB ,&
&  0.12917E+00_JPRB, 0.12842E+00_JPRB, 0.12690E+00_JPRB, 0.12563E+00_JPRB, 0.12484E+00_JPRB, 0.12400E+00_JPRB, 0.12346E+00_JPRB ,&
&  0.12307E+00_JPRB, 0.12246E+00_JPRB, 0.12230E+00_JPRB, 0.12200E+00_JPRB, 0.12210E+00_JPRB, 0.12226E+00_JPRB, 0.12260E+00_JPRB ,&
&  0.12329E+00_JPRB, 0.12407E+00_JPRB, 0.12399E+00_JPRB, 0.12382E+00_JPRB, 0.12255E+00_JPRB, 0.12115E+00_JPRB, 0.11990E+00_JPRB ,&
&  0.11839E+00_JPRB, 0.11722E+00_JPRB, 0.11649E+00_JPRB, 0.11581E+00_JPRB, 0.11573E+00_JPRB, 0.11582E+00_JPRB, 0.11585E+00_JPRB ,&
&  0.11606E+00_JPRB, 0.11633E+00_JPRB, 0.11674E+00_JPRB, 0.11754E+00_JPRB, 0.11824E+00_JPRB, 0.11924E+00_JPRB, 0.12034E+00_JPRB ,&
&  0.12125E+00_JPRB, 0.12289E+00_JPRB, 0.12464E+00_JPRB, 0.12676E+00_JPRB, 0.13061E+00_JPRB, 0.13410E+00_JPRB, 0.14053E+00_JPRB ,&
&  0.14601E+00_JPRB, 0.15566E+00_JPRB, 0.15417E+00_JPRB, 0.15697E+00_JPRB, 0.15999E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.16371E+00_JPRB, 0.16460E+00_JPRB, 0.16435E+00_JPRB, 0.16449E+00_JPRB, 0.16408E+00_JPRB, 0.16529E+00_JPRB, 0.16473E+00_JPRB ,&
&  0.16488E+00_JPRB, 0.16415E+00_JPRB, 0.16337E+00_JPRB, 0.16130E+00_JPRB, 0.16023E+00_JPRB, 0.15447E+00_JPRB, 0.15308E+00_JPRB ,&
&  0.15208E+00_JPRB, 0.15300E+00_JPRB, 0.15331E+00_JPRB, 0.15384E+00_JPRB, 0.15485E+00_JPRB, 0.15650E+00_JPRB, 0.15669E+00_JPRB ,&
&  0.15574E+00_JPRB, 0.15372E+00_JPRB, 0.15140E+00_JPRB, 0.14927E+00_JPRB, 0.14962E+00_JPRB, 0.14938E+00_JPRB, 0.14857E+00_JPRB ,&
&  0.14759E+00_JPRB, 0.14811E+00_JPRB, 0.14860E+00_JPRB, 0.14984E+00_JPRB, 0.15470E+00_JPRB, 0.15340E+00_JPRB, 0.15208E+00_JPRB ,&
&  0.15170E+00_JPRB, 0.15140E+00_JPRB, 0.15167E+00_JPRB, 0.15183E+00_JPRB, 0.15331E+00_JPRB, 0.15275E+00_JPRB, 0.15495E+00_JPRB ,&
&  0.15205E+00_JPRB, 0.15016E+00_JPRB, 0.14780E+00_JPRB, 0.14508E+00_JPRB, 0.14190E+00_JPRB, 0.13987E+00_JPRB, 0.13792E+00_JPRB ,&
&  0.13583E+00_JPRB, 0.13410E+00_JPRB, 0.13186E+00_JPRB, 0.13106E+00_JPRB, 0.12998E+00_JPRB, 0.12901E+00_JPRB, 0.12812E+00_JPRB ,&
&  0.12740E+00_JPRB, 0.12642E+00_JPRB, 0.12553E+00_JPRB, 0.12493E+00_JPRB, 0.12473E+00_JPRB, 0.12484E+00_JPRB, 0.12573E+00_JPRB ,&
&  0.12751E+00_JPRB, 0.12903E+00_JPRB, 0.13019E+00_JPRB, 0.13100E+00_JPRB, 0.12968E+00_JPRB, 0.12728E+00_JPRB, 0.12484E+00_JPRB ,&
&  0.12186E+00_JPRB, 0.12017E+00_JPRB, 0.11863E+00_JPRB, 0.11774E+00_JPRB, 0.11753E+00_JPRB, 0.11756E+00_JPRB, 0.11786E+00_JPRB ,&
&  0.11819E+00_JPRB, 0.11863E+00_JPRB, 0.11894E+00_JPRB, 0.11931E+00_JPRB, 0.11979E+00_JPRB, 0.12027E+00_JPRB, 0.12095E+00_JPRB ,&
&  0.12204E+00_JPRB, 0.12346E+00_JPRB, 0.12546E+00_JPRB, 0.12785E+00_JPRB, 0.13057E+00_JPRB, 0.13419E+00_JPRB, 0.13959E+00_JPRB ,&
&  0.14542E+00_JPRB, 0.15576E+00_JPRB, 0.15696E+00_JPRB, 0.16068E+00_JPRB, 0.16301E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.16609E+00_JPRB, 0.16744E+00_JPRB, 0.16793E+00_JPRB, 0.16899E+00_JPRB, 0.16943E+00_JPRB, 0.17073E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17430E+00_JPRB, 0.17648E+00_JPRB, 0.17702E+00_JPRB, 0.17540E+00_JPRB, 0.17464E+00_JPRB, 0.17250E+00_JPRB, 0.17182E+00_JPRB ,&
&  0.17192E+00_JPRB, 0.17311E+00_JPRB, 0.17338E+00_JPRB, 0.17356E+00_JPRB, 0.17362E+00_JPRB, 0.17366E+00_JPRB, 0.17408E+00_JPRB ,&
&  0.17230E+00_JPRB, 0.17192E+00_JPRB, 0.17400E+00_JPRB, 0.17344E+00_JPRB, 0.17296E+00_JPRB, 0.17185E+00_JPRB, 0.17125E+00_JPRB ,&
&  0.16762E+00_JPRB, 0.16224E+00_JPRB, 0.15539E+00_JPRB, 0.15777E+00_JPRB, 0.16484E+00_JPRB, 0.16305E+00_JPRB, 0.16222E+00_JPRB ,&
&  0.16150E+00_JPRB, 0.16027E+00_JPRB, 0.15972E+00_JPRB, 0.15917E+00_JPRB, 0.15952E+00_JPRB, 0.15988E+00_JPRB, 0.16108E+00_JPRB ,&
&  0.16285E+00_JPRB, 0.16553E+00_JPRB, 0.16538E+00_JPRB, 0.15971E+00_JPRB, 0.15650E+00_JPRB, 0.15528E+00_JPRB, 0.15228E+00_JPRB ,&
&  0.14886E+00_JPRB, 0.14725E+00_JPRB, 0.14648E+00_JPRB, 0.14476E+00_JPRB, 0.14443E+00_JPRB, 0.14406E+00_JPRB, 0.14214E+00_JPRB ,&
&  0.13949E+00_JPRB, 0.13754E+00_JPRB, 0.13455E+00_JPRB, 0.13143E+00_JPRB, 0.12921E+00_JPRB, 0.12814E+00_JPRB, 0.12811E+00_JPRB ,&
&  0.12961E+00_JPRB, 0.13136E+00_JPRB, 0.13668E+00_JPRB, 0.14034E+00_JPRB, 0.13780E+00_JPRB, 0.13416E+00_JPRB, 0.13058E+00_JPRB ,&
&  0.12601E+00_JPRB, 0.12376E+00_JPRB, 0.12202E+00_JPRB, 0.12079E+00_JPRB, 0.12070E+00_JPRB, 0.12143E+00_JPRB, 0.12208E+00_JPRB ,&
&  0.12277E+00_JPRB, 0.12360E+00_JPRB, 0.12399E+00_JPRB, 0.12414E+00_JPRB, 0.12416E+00_JPRB, 0.12435E+00_JPRB, 0.12483E+00_JPRB ,&
&  0.12549E+00_JPRB, 0.12704E+00_JPRB, 0.12932E+00_JPRB, 0.13180E+00_JPRB, 0.13351E+00_JPRB, 0.13482E+00_JPRB, 0.13754E+00_JPRB ,&
&  0.14083E+00_JPRB, 0.14845E+00_JPRB, 0.15937E+00_JPRB, 0.16832E+00_JPRB, 0.16533E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.17799E+00_JPRB, 0.17875E+00_JPRB, 0.17975E+00_JPRB, 0.18098E+00_JPRB, 0.18246E+00_JPRB, 0.18411E+00_JPRB, 0.18548E+00_JPRB ,&
&  0.18694E+00_JPRB, 0.18790E+00_JPRB, 0.18815E+00_JPRB, 0.18722E+00_JPRB, 0.18656E+00_JPRB, 0.18713E+00_JPRB, 0.18856E+00_JPRB ,&
&  0.18711E+00_JPRB, 0.18456E+00_JPRB, 0.18363E+00_JPRB, 0.18370E+00_JPRB, 0.18332E+00_JPRB, 0.18311E+00_JPRB, 0.18381E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18455E+00_JPRB, 0.18438E+00_JPRB, 0.18362E+00_JPRB, 0.18295E+00_JPRB, 0.18240E+00_JPRB, 0.18158E+00_JPRB ,&
&  0.17901E+00_JPRB, 0.16935E+00_JPRB, 0.16263E+00_JPRB, 0.16524E+00_JPRB, 0.17254E+00_JPRB, 0.17201E+00_JPRB, 0.17190E+00_JPRB ,&
&  0.17132E+00_JPRB, 0.17003E+00_JPRB, 0.16861E+00_JPRB, 0.16725E+00_JPRB, 0.16662E+00_JPRB, 0.16676E+00_JPRB, 0.16733E+00_JPRB ,&
&  0.16789E+00_JPRB, 0.16948E+00_JPRB, 0.17156E+00_JPRB, 0.17261E+00_JPRB, 0.17361E+00_JPRB, 0.17403E+00_JPRB, 0.17419E+00_JPRB ,&
&  0.17404E+00_JPRB, 0.17334E+00_JPRB, 0.17302E+00_JPRB, 0.17270E+00_JPRB, 0.17241E+00_JPRB, 0.17173E+00_JPRB, 0.16889E+00_JPRB ,&
&  0.16362E+00_JPRB, 0.15879E+00_JPRB, 0.15287E+00_JPRB, 0.14695E+00_JPRB, 0.14139E+00_JPRB, 0.13734E+00_JPRB, 0.13420E+00_JPRB ,&
&  0.13359E+00_JPRB, 0.13669E+00_JPRB, 0.14360E+00_JPRB, 0.15255E+00_JPRB, 0.15988E+00_JPRB, 0.14932E+00_JPRB, 0.14300E+00_JPRB ,&
&  0.13749E+00_JPRB, 0.13326E+00_JPRB, 0.13070E+00_JPRB, 0.12936E+00_JPRB, 0.12855E+00_JPRB, 0.12867E+00_JPRB, 0.12929E+00_JPRB ,&
&  0.13040E+00_JPRB, 0.13165E+00_JPRB, 0.13283E+00_JPRB, 0.13349E+00_JPRB, 0.13371E+00_JPRB, 0.13352E+00_JPRB, 0.13314E+00_JPRB ,&
&  0.13339E+00_JPRB, 0.13525E+00_JPRB, 0.13891E+00_JPRB, 0.14333E+00_JPRB, 0.14705E+00_JPRB, 0.15047E+00_JPRB, 0.15535E+00_JPRB ,&
&  0.16154E+00_JPRB, 0.16698E+00_JPRB, 0.17087E+00_JPRB, 0.17479E+00_JPRB, 0.17677E+00_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.19494E+00_JPRB, 0.19574E+00_JPRB, 0.19660E+00_JPRB, 0.19756E+00_JPRB, 0.19936E+00_JPRB, 0.20072E+00_JPRB, 0.20234E+00_JPRB ,&
&  0.20348E+00_JPRB, 0.20222E+00_JPRB, 0.20058E+00_JPRB, 0.19937E+00_JPRB, 0.19902E+00_JPRB, 0.19866E+00_JPRB, 0.19854E+00_JPRB ,&
&  0.19960E+00_JPRB, 0.19922E+00_JPRB, 0.19695E+00_JPRB, 0.19548E+00_JPRB, 0.19280E+00_JPRB, 0.19097E+00_JPRB, 0.19062E+00_JPRB ,&
&  0.19215E+00_JPRB, 0.19208E+00_JPRB, 0.19186E+00_JPRB, 0.19122E+00_JPRB, 0.19021E+00_JPRB, 0.18977E+00_JPRB, 0.18877E+00_JPRB ,&
&  0.18827E+00_JPRB, 0.18759E+00_JPRB, 0.18285E+00_JPRB, 0.18353E+00_JPRB, 0.18303E+00_JPRB, 0.18277E+00_JPRB, 0.18214E+00_JPRB ,&
&  0.18147E+00_JPRB, 0.18050E+00_JPRB, 0.17880E+00_JPRB, 0.17696E+00_JPRB, 0.17541E+00_JPRB, 0.17455E+00_JPRB, 0.17438E+00_JPRB ,&
&  0.17456E+00_JPRB, 0.17521E+00_JPRB, 0.17651E+00_JPRB, 0.17842E+00_JPRB, 0.17923E+00_JPRB, 0.17979E+00_JPRB, 0.18062E+00_JPRB ,&
&  0.18098E+00_JPRB, 0.18105E+00_JPRB, 0.18113E+00_JPRB, 0.18126E+00_JPRB, 0.18129E+00_JPRB, 0.18083E+00_JPRB, 0.18046E+00_JPRB ,&
&  0.17930E+00_JPRB, 0.17733E+00_JPRB, 0.17608E+00_JPRB, 0.17444E+00_JPRB, 0.16981E+00_JPRB, 0.15969E+00_JPRB, 0.15400E+00_JPRB ,&
&  0.15385E+00_JPRB, 0.15829E+00_JPRB, 0.16437E+00_JPRB, 0.16722E+00_JPRB, 0.17148E+00_JPRB, 0.17167E+00_JPRB, 0.16863E+00_JPRB ,&
&  0.16168E+00_JPRB, 0.15549E+00_JPRB, 0.15128E+00_JPRB, 0.14755E+00_JPRB, 0.14522E+00_JPRB, 0.14460E+00_JPRB, 0.14519E+00_JPRB ,&
&  0.14705E+00_JPRB, 0.14996E+00_JPRB, 0.15259E+00_JPRB, 0.15397E+00_JPRB, 0.15400E+00_JPRB, 0.15251E+00_JPRB, 0.15092E+00_JPRB ,&
&  0.15019E+00_JPRB, 0.15360E+00_JPRB, 0.16119E+00_JPRB, 0.17054E+00_JPRB, 0.17426E+00_JPRB, 0.17554E+00_JPRB, 0.17682E+00_JPRB ,&
&  0.17965E+00_JPRB, 0.18368E+00_JPRB, 0.18787E+00_JPRB, 0.19078E+00_JPRB, 0.19315E+00_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.21192E+00_JPRB, 0.21244E+00_JPRB, 0.21316E+00_JPRB, 0.21429E+00_JPRB, 0.21588E+00_JPRB, 0.21672E+00_JPRB, 0.21739E+00_JPRB ,&
&  0.21677E+00_JPRB, 0.21517E+00_JPRB, 0.21335E+00_JPRB, 0.21145E+00_JPRB, 0.21075E+00_JPRB, 0.21097E+00_JPRB, 0.21142E+00_JPRB ,&
&  0.21125E+00_JPRB, 0.21140E+00_JPRB, 0.21004E+00_JPRB, 0.20750E+00_JPRB, 0.20454E+00_JPRB, 0.20262E+00_JPRB, 0.20217E+00_JPRB ,&
&  0.20173E+00_JPRB, 0.20104E+00_JPRB, 0.20087E+00_JPRB, 0.20078E+00_JPRB, 0.19920E+00_JPRB, 0.19836E+00_JPRB, 0.19876E+00_JPRB ,&
&  0.19958E+00_JPRB, 0.20092E+00_JPRB, 0.20123E+00_JPRB, 0.19852E+00_JPRB, 0.20074E+00_JPRB, 0.20039E+00_JPRB, 0.19854E+00_JPRB ,&
&  0.19621E+00_JPRB, 0.19382E+00_JPRB, 0.19168E+00_JPRB, 0.18962E+00_JPRB, 0.18756E+00_JPRB, 0.18600E+00_JPRB, 0.18497E+00_JPRB ,&
&  0.18466E+00_JPRB, 0.18474E+00_JPRB, 0.18504E+00_JPRB, 0.18586E+00_JPRB, 0.18650E+00_JPRB, 0.18685E+00_JPRB, 0.18688E+00_JPRB ,&
&  0.18698E+00_JPRB, 0.18717E+00_JPRB, 0.18754E+00_JPRB, 0.18777E+00_JPRB, 0.18807E+00_JPRB, 0.18840E+00_JPRB, 0.18846E+00_JPRB ,&
&  0.18767E+00_JPRB, 0.18620E+00_JPRB, 0.18417E+00_JPRB, 0.18360E+00_JPRB, 0.18373E+00_JPRB, 0.18263E+00_JPRB, 0.18057E+00_JPRB ,&
&  0.17846E+00_JPRB, 0.17783E+00_JPRB, 0.17864E+00_JPRB, 0.17972E+00_JPRB, 0.18055E+00_JPRB, 0.18078E+00_JPRB, 0.18046E+00_JPRB ,&
&  0.17953E+00_JPRB, 0.17763E+00_JPRB, 0.17512E+00_JPRB, 0.17321E+00_JPRB, 0.17226E+00_JPRB, 0.17226E+00_JPRB, 0.17309E+00_JPRB ,&
&  0.17320E+00_JPRB, 0.17316E+00_JPRB, 0.17408E+00_JPRB, 0.17480E+00_JPRB, 0.17523E+00_JPRB, 0.17587E+00_JPRB, 0.17719E+00_JPRB ,&
&  0.17907E+00_JPRB, 0.18196E+00_JPRB, 0.18547E+00_JPRB, 0.18907E+00_JPRB, 0.19159E+00_JPRB, 0.19069E+00_JPRB, 0.19090E+00_JPRB ,&
&  0.19470E+00_JPRB, 0.19952E+00_JPRB, 0.20363E+00_JPRB, 0.20693E+00_JPRB, 0.20985E+00_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.23123E+00_JPRB, 0.23254E+00_JPRB, 0.23401E+00_JPRB, 0.23575E+00_JPRB, 0.23695E+00_JPRB, 0.23736E+00_JPRB, 0.23602E+00_JPRB ,&
&  0.23354E+00_JPRB, 0.23087E+00_JPRB, 0.22828E+00_JPRB, 0.22609E+00_JPRB, 0.22492E+00_JPRB, 0.22440E+00_JPRB, 0.22426E+00_JPRB ,&
&  0.22424E+00_JPRB, 0.22417E+00_JPRB, 0.22267E+00_JPRB, 0.22028E+00_JPRB, 0.21708E+00_JPRB, 0.21439E+00_JPRB, 0.21388E+00_JPRB ,&
&  0.21257E+00_JPRB, 0.21169E+00_JPRB, 0.21121E+00_JPRB, 0.21063E+00_JPRB, 0.21051E+00_JPRB, 0.21154E+00_JPRB, 0.21325E+00_JPRB ,&
&  0.21518E+00_JPRB, 0.21662E+00_JPRB, 0.21772E+00_JPRB, 0.21861E+00_JPRB, 0.21814E+00_JPRB, 0.21697E+00_JPRB, 0.21560E+00_JPRB ,&
&  0.21400E+00_JPRB, 0.21250E+00_JPRB, 0.21120E+00_JPRB, 0.20951E+00_JPRB, 0.20704E+00_JPRB, 0.20409E+00_JPRB, 0.20201E+00_JPRB ,&
&  0.20046E+00_JPRB, 0.19905E+00_JPRB, 0.19751E+00_JPRB, 0.19672E+00_JPRB, 0.19617E+00_JPRB, 0.19597E+00_JPRB, 0.19608E+00_JPRB ,&
&  0.19649E+00_JPRB, 0.19768E+00_JPRB, 0.19930E+00_JPRB, 0.20102E+00_JPRB, 0.20144E+00_JPRB, 0.20156E+00_JPRB, 0.20184E+00_JPRB ,&
&  0.20129E+00_JPRB, 0.20015E+00_JPRB, 0.19856E+00_JPRB, 0.19689E+00_JPRB, 0.19596E+00_JPRB, 0.19527E+00_JPRB, 0.19446E+00_JPRB ,&
&  0.19284E+00_JPRB, 0.19184E+00_JPRB, 0.19124E+00_JPRB, 0.19110E+00_JPRB, 0.19131E+00_JPRB, 0.19136E+00_JPRB, 0.19072E+00_JPRB ,&
&  0.18992E+00_JPRB, 0.18921E+00_JPRB, 0.18818E+00_JPRB, 0.18708E+00_JPRB, 0.18542E+00_JPRB, 0.18566E+00_JPRB, 0.18647E+00_JPRB ,&
&  0.18694E+00_JPRB, 0.18724E+00_JPRB, 0.18747E+00_JPRB, 0.18843E+00_JPRB, 0.18942E+00_JPRB, 0.19086E+00_JPRB, 0.19309E+00_JPRB ,&
&  0.19491E+00_JPRB, 0.19726E+00_JPRB, 0.20087E+00_JPRB, 0.20417E+00_JPRB, 0.20655E+00_JPRB, 0.20761E+00_JPRB, 0.20969E+00_JPRB ,&
&  0.21467E+00_JPRB, 0.22024E+00_JPRB, 0.22395E+00_JPRB, 0.22747E+00_JPRB, 0.23033E+00_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.25002E+00_JPRB, 0.25455E+00_JPRB, 0.25811E+00_JPRB, 0.26012E+00_JPRB, 0.26012E+00_JPRB, 0.25888E+00_JPRB, 0.25645E+00_JPRB ,&
&  0.25308E+00_JPRB, 0.24980E+00_JPRB, 0.24645E+00_JPRB, 0.24371E+00_JPRB, 0.24212E+00_JPRB, 0.24092E+00_JPRB, 0.24013E+00_JPRB ,&
&  0.23889E+00_JPRB, 0.23723E+00_JPRB, 0.23498E+00_JPRB, 0.23296E+00_JPRB, 0.23057E+00_JPRB, 0.22708E+00_JPRB, 0.22464E+00_JPRB ,&
&  0.22300E+00_JPRB, 0.22136E+00_JPRB, 0.22100E+00_JPRB, 0.22227E+00_JPRB, 0.22454E+00_JPRB, 0.22677E+00_JPRB, 0.22925E+00_JPRB ,&
&  0.23099E+00_JPRB, 0.23161E+00_JPRB, 0.23198E+00_JPRB, 0.23177E+00_JPRB, 0.23181E+00_JPRB, 0.23150E+00_JPRB, 0.23073E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22898E+00_JPRB, 0.22755E+00_JPRB, 0.22726E+00_JPRB, 0.22716E+00_JPRB, 0.22611E+00_JPRB, 0.22429E+00_JPRB ,&
&  0.22138E+00_JPRB, 0.21721E+00_JPRB, 0.21376E+00_JPRB, 0.21192E+00_JPRB, 0.21125E+00_JPRB, 0.21143E+00_JPRB, 0.21221E+00_JPRB ,&
&  0.21331E+00_JPRB, 0.21488E+00_JPRB, 0.21634E+00_JPRB, 0.21703E+00_JPRB, 0.21685E+00_JPRB, 0.21667E+00_JPRB, 0.21549E+00_JPRB ,&
&  0.21434E+00_JPRB, 0.21361E+00_JPRB, 0.21306E+00_JPRB, 0.21245E+00_JPRB, 0.21049E+00_JPRB, 0.20812E+00_JPRB, 0.20778E+00_JPRB ,&
&  0.20663E+00_JPRB, 0.20593E+00_JPRB, 0.20487E+00_JPRB, 0.20451E+00_JPRB, 0.20397E+00_JPRB, 0.20296E+00_JPRB, 0.20178E+00_JPRB ,&
&  0.20073E+00_JPRB, 0.20015E+00_JPRB, 0.19973E+00_JPRB, 0.19874E+00_JPRB, 0.19892E+00_JPRB, 0.19938E+00_JPRB, 0.20066E+00_JPRB ,&
&  0.20185E+00_JPRB, 0.20310E+00_JPRB, 0.20463E+00_JPRB, 0.20592E+00_JPRB, 0.20793E+00_JPRB, 0.21034E+00_JPRB, 0.21310E+00_JPRB ,&
&  0.21541E+00_JPRB, 0.21873E+00_JPRB, 0.22264E+00_JPRB, 0.22630E+00_JPRB, 0.22913E+00_JPRB, 0.23109E+00_JPRB, 0.23368E+00_JPRB ,&
&  0.23807E+00_JPRB, 0.24169E+00_JPRB, 0.24340E+00_JPRB, 0.24520E+00_JPRB, 0.24666E+00_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.27727E+00_JPRB, 0.28184E+00_JPRB, 0.28432E+00_JPRB, 0.28532E+00_JPRB, 0.28542E+00_JPRB, 0.28382E+00_JPRB, 0.28147E+00_JPRB ,&
&  0.27809E+00_JPRB, 0.27486E+00_JPRB, 0.27122E+00_JPRB, 0.26693E+00_JPRB, 0.26310E+00_JPRB, 0.25998E+00_JPRB, 0.25742E+00_JPRB ,&
&  0.25488E+00_JPRB, 0.25257E+00_JPRB, 0.25069E+00_JPRB, 0.24864E+00_JPRB, 0.24630E+00_JPRB, 0.24333E+00_JPRB, 0.23925E+00_JPRB ,&
&  0.23698E+00_JPRB, 0.23526E+00_JPRB, 0.23638E+00_JPRB, 0.23905E+00_JPRB, 0.24107E+00_JPRB, 0.24242E+00_JPRB, 0.24338E+00_JPRB ,&
&  0.24471E+00_JPRB, 0.24534E+00_JPRB, 0.24608E+00_JPRB, 0.24823E+00_JPRB, 0.25031E+00_JPRB, 0.25002E+00_JPRB, 0.24848E+00_JPRB ,&
&  0.24701E+00_JPRB, 0.24688E+00_JPRB, 0.24719E+00_JPRB, 0.24733E+00_JPRB, 0.24697E+00_JPRB, 0.24534E+00_JPRB, 0.24257E+00_JPRB ,&
&  0.23718E+00_JPRB, 0.23308E+00_JPRB, 0.23213E+00_JPRB, 0.23309E+00_JPRB, 0.23474E+00_JPRB, 0.23599E+00_JPRB, 0.23678E+00_JPRB ,&
&  0.23706E+00_JPRB, 0.23658E+00_JPRB, 0.23562E+00_JPRB, 0.23448E+00_JPRB, 0.23292E+00_JPRB, 0.23099E+00_JPRB, 0.22983E+00_JPRB ,&
&  0.22852E+00_JPRB, 0.22730E+00_JPRB, 0.22637E+00_JPRB, 0.22578E+00_JPRB, 0.22468E+00_JPRB, 0.22326E+00_JPRB, 0.22256E+00_JPRB ,&
&  0.22154E+00_JPRB, 0.22105E+00_JPRB, 0.22024E+00_JPRB, 0.21946E+00_JPRB, 0.21837E+00_JPRB, 0.21733E+00_JPRB, 0.21718E+00_JPRB ,&
&  0.21737E+00_JPRB, 0.21630E+00_JPRB, 0.21491E+00_JPRB, 0.21464E+00_JPRB, 0.21496E+00_JPRB, 0.21561E+00_JPRB, 0.21621E+00_JPRB ,&
&  0.21685E+00_JPRB, 0.21767E+00_JPRB, 0.21939E+00_JPRB, 0.22125E+00_JPRB, 0.22330E+00_JPRB, 0.22586E+00_JPRB, 0.22886E+00_JPRB ,&
&  0.23265E+00_JPRB, 0.23731E+00_JPRB, 0.24267E+00_JPRB, 0.24754E+00_JPRB, 0.25159E+00_JPRB, 0.25425E+00_JPRB, 0.25789E+00_JPRB ,&
&  0.26233E+00_JPRB, 0.26541E+00_JPRB, 0.26776E+00_JPRB, 0.27028E+00_JPRB, 0.27302E+00_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.30455E+00_JPRB, 0.30619E+00_JPRB, 0.30624E+00_JPRB, 0.30659E+00_JPRB, 0.30614E+00_JPRB, 0.30452E+00_JPRB, 0.30176E+00_JPRB ,&
&  0.29893E+00_JPRB, 0.29560E+00_JPRB, 0.29221E+00_JPRB, 0.28777E+00_JPRB, 0.28313E+00_JPRB, 0.27948E+00_JPRB, 0.27546E+00_JPRB ,&
&  0.27133E+00_JPRB, 0.26753E+00_JPRB, 0.26503E+00_JPRB, 0.26304E+00_JPRB, 0.26113E+00_JPRB, 0.25942E+00_JPRB, 0.25698E+00_JPRB ,&
&  0.25488E+00_JPRB, 0.25285E+00_JPRB, 0.25324E+00_JPRB, 0.25549E+00_JPRB, 0.25729E+00_JPRB, 0.25869E+00_JPRB, 0.25985E+00_JPRB ,&
&  0.26018E+00_JPRB, 0.26420E+00_JPRB, 0.25859E+00_JPRB, 0.26708E+00_JPRB, 0.26878E+00_JPRB, 0.26919E+00_JPRB, 0.26904E+00_JPRB ,&
&  0.27020E+00_JPRB, 0.27182E+00_JPRB, 0.27276E+00_JPRB, 0.27056E+00_JPRB, 0.26750E+00_JPRB, 0.26525E+00_JPRB, 0.26234E+00_JPRB ,&
&  0.25821E+00_JPRB, 0.25668E+00_JPRB, 0.25697E+00_JPRB, 0.25745E+00_JPRB, 0.25757E+00_JPRB, 0.25715E+00_JPRB, 0.25619E+00_JPRB ,&
&  0.25493E+00_JPRB, 0.25308E+00_JPRB, 0.25088E+00_JPRB, 0.24858E+00_JPRB, 0.24712E+00_JPRB, 0.24529E+00_JPRB, 0.24347E+00_JPRB ,&
&  0.24255E+00_JPRB, 0.24259E+00_JPRB, 0.24261E+00_JPRB, 0.24251E+00_JPRB, 0.24195E+00_JPRB, 0.24162E+00_JPRB, 0.24157E+00_JPRB ,&
&  0.24091E+00_JPRB, 0.24008E+00_JPRB, 0.23909E+00_JPRB, 0.23778E+00_JPRB, 0.23640E+00_JPRB, 0.23582E+00_JPRB, 0.23604E+00_JPRB ,&
&  0.23595E+00_JPRB, 0.23466E+00_JPRB, 0.23255E+00_JPRB, 0.23196E+00_JPRB, 0.23168E+00_JPRB, 0.23177E+00_JPRB, 0.23215E+00_JPRB ,&
&  0.23313E+00_JPRB, 0.23493E+00_JPRB, 0.23686E+00_JPRB, 0.23859E+00_JPRB, 0.24029E+00_JPRB, 0.24299E+00_JPRB, 0.24649E+00_JPRB ,&
&  0.25069E+00_JPRB, 0.25529E+00_JPRB, 0.25976E+00_JPRB, 0.26402E+00_JPRB, 0.26784E+00_JPRB, 0.27262E+00_JPRB, 0.27913E+00_JPRB ,&
&  0.28476E+00_JPRB, 0.28905E+00_JPRB, 0.29254E+00_JPRB, 0.29662E+00_JPRB, 0.30099E+00_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.33625E+00_JPRB, 0.33466E+00_JPRB, 0.33164E+00_JPRB, 0.32978E+00_JPRB, 0.32742E+00_JPRB, 0.32441E+00_JPRB, 0.32117E+00_JPRB ,&
&  0.31864E+00_JPRB, 0.31614E+00_JPRB, 0.31358E+00_JPRB, 0.30985E+00_JPRB, 0.30475E+00_JPRB, 0.29989E+00_JPRB, 0.29410E+00_JPRB ,&
&  0.28871E+00_JPRB, 0.28404E+00_JPRB, 0.28101E+00_JPRB, 0.27947E+00_JPRB, 0.27813E+00_JPRB, 0.27713E+00_JPRB, 0.27587E+00_JPRB ,&
&  0.27534E+00_JPRB, 0.27533E+00_JPRB, 0.27524E+00_JPRB, 0.27555E+00_JPRB, 0.27626E+00_JPRB, 0.27877E+00_JPRB, 0.28036E+00_JPRB ,&
&  0.28215E+00_JPRB, 0.27678E+00_JPRB, 0.28309E+00_JPRB, 0.28745E+00_JPRB, 0.29431E+00_JPRB, 0.29890E+00_JPRB, 0.30162E+00_JPRB ,&
&  0.30337E+00_JPRB, 0.30462E+00_JPRB, 0.30235E+00_JPRB, 0.29909E+00_JPRB, 0.29624E+00_JPRB, 0.29442E+00_JPRB, 0.29053E+00_JPRB ,&
&  0.28647E+00_JPRB, 0.28363E+00_JPRB, 0.28137E+00_JPRB, 0.27984E+00_JPRB, 0.27812E+00_JPRB, 0.27554E+00_JPRB, 0.27267E+00_JPRB ,&
&  0.26890E+00_JPRB, 0.26616E+00_JPRB, 0.26393E+00_JPRB, 0.26309E+00_JPRB, 0.26331E+00_JPRB, 0.26430E+00_JPRB, 0.26574E+00_JPRB ,&
&  0.26794E+00_JPRB, 0.26989E+00_JPRB, 0.27083E+00_JPRB, 0.27122E+00_JPRB, 0.27115E+00_JPRB, 0.27106E+00_JPRB, 0.27039E+00_JPRB ,&
&  0.26900E+00_JPRB, 0.26735E+00_JPRB, 0.26569E+00_JPRB, 0.26387E+00_JPRB, 0.26194E+00_JPRB, 0.26065E+00_JPRB, 0.25947E+00_JPRB ,&
&  0.25808E+00_JPRB, 0.25627E+00_JPRB, 0.25423E+00_JPRB, 0.25307E+00_JPRB, 0.25220E+00_JPRB, 0.25168E+00_JPRB, 0.25232E+00_JPRB ,&
&  0.25412E+00_JPRB, 0.25623E+00_JPRB, 0.25842E+00_JPRB, 0.26022E+00_JPRB, 0.26192E+00_JPRB, 0.26362E+00_JPRB, 0.26620E+00_JPRB ,&
&  0.27076E+00_JPRB, 0.27573E+00_JPRB, 0.28078E+00_JPRB, 0.28585E+00_JPRB, 0.29230E+00_JPRB, 0.30100E+00_JPRB, 0.31047E+00_JPRB ,&
&  0.31718E+00_JPRB, 0.32144E+00_JPRB, 0.32577E+00_JPRB, 0.33020E+00_JPRB, 0.33469E+00_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.35891E+00_JPRB, 0.35655E+00_JPRB, 0.35418E+00_JPRB, 0.35137E+00_JPRB, 0.34748E+00_JPRB, 0.34205E+00_JPRB, 0.33800E+00_JPRB ,&
&  0.33542E+00_JPRB, 0.33228E+00_JPRB, 0.32965E+00_JPRB, 0.32612E+00_JPRB, 0.32182E+00_JPRB, 0.31672E+00_JPRB, 0.31008E+00_JPRB ,&
&  0.30420E+00_JPRB, 0.29883E+00_JPRB, 0.29551E+00_JPRB, 0.29406E+00_JPRB, 0.29428E+00_JPRB, 0.29462E+00_JPRB, 0.29350E+00_JPRB ,&
&  0.29287E+00_JPRB, 0.29284E+00_JPRB, 0.29238E+00_JPRB, 0.29261E+00_JPRB, 0.29329E+00_JPRB, 0.29611E+00_JPRB, 0.29944E+00_JPRB ,&
&  0.30261E+00_JPRB, 0.27890E+00_JPRB, 0.28785E+00_JPRB, 0.29032E+00_JPRB, 0.29904E+00_JPRB, 0.31267E+00_JPRB, 0.32727E+00_JPRB ,&
&  0.33709E+00_JPRB, 0.33805E+00_JPRB, 0.33539E+00_JPRB, 0.32968E+00_JPRB, 0.32482E+00_JPRB, 0.32041E+00_JPRB, 0.31561E+00_JPRB ,&
&  0.31084E+00_JPRB, 0.30793E+00_JPRB, 0.30534E+00_JPRB, 0.30261E+00_JPRB, 0.29847E+00_JPRB, 0.29303E+00_JPRB, 0.28759E+00_JPRB ,&
&  0.28363E+00_JPRB, 0.28210E+00_JPRB, 0.28256E+00_JPRB, 0.28406E+00_JPRB, 0.28604E+00_JPRB, 0.28940E+00_JPRB, 0.29346E+00_JPRB ,&
&  0.29735E+00_JPRB, 0.29910E+00_JPRB, 0.30001E+00_JPRB, 0.30116E+00_JPRB, 0.30177E+00_JPRB, 0.30143E+00_JPRB, 0.30002E+00_JPRB ,&
&  0.29796E+00_JPRB, 0.29519E+00_JPRB, 0.29227E+00_JPRB, 0.28901E+00_JPRB, 0.28586E+00_JPRB, 0.28261E+00_JPRB, 0.27963E+00_JPRB ,&
&  0.27706E+00_JPRB, 0.27427E+00_JPRB, 0.27134E+00_JPRB, 0.26909E+00_JPRB, 0.26850E+00_JPRB, 0.26906E+00_JPRB, 0.27042E+00_JPRB ,&
&  0.27291E+00_JPRB, 0.27543E+00_JPRB, 0.27702E+00_JPRB, 0.27886E+00_JPRB, 0.28084E+00_JPRB, 0.28302E+00_JPRB, 0.28653E+00_JPRB ,&
&  0.29172E+00_JPRB, 0.29602E+00_JPRB, 0.30038E+00_JPRB, 0.30704E+00_JPRB, 0.31661E+00_JPRB, 0.32776E+00_JPRB, 0.33797E+00_JPRB ,&
&  0.34469E+00_JPRB, 0.34773E+00_JPRB, 0.35116E+00_JPRB, 0.35389E+00_JPRB, 0.35890E+00_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.39679E+00_JPRB, 0.39564E+00_JPRB, 0.39057E+00_JPRB, 0.38053E+00_JPRB, 0.36973E+00_JPRB, 0.36081E+00_JPRB, 0.35579E+00_JPRB ,&
&  0.35288E+00_JPRB, 0.34812E+00_JPRB, 0.34401E+00_JPRB, 0.34115E+00_JPRB, 0.33730E+00_JPRB, 0.33315E+00_JPRB, 0.32752E+00_JPRB ,&
&  0.32324E+00_JPRB, 0.32006E+00_JPRB, 0.31767E+00_JPRB, 0.31622E+00_JPRB, 0.31531E+00_JPRB, 0.31429E+00_JPRB, 0.31256E+00_JPRB ,&
&  0.31051E+00_JPRB, 0.30878E+00_JPRB, 0.30749E+00_JPRB, 0.30795E+00_JPRB, 0.30883E+00_JPRB, 0.31197E+00_JPRB, 0.31738E+00_JPRB ,&
&  0.30341E+00_JPRB, 0.27828E+00_JPRB, 0.28911E+00_JPRB, 0.30044E+00_JPRB, 0.31363E+00_JPRB, 0.33516E+00_JPRB, 0.35800E+00_JPRB ,&
&  0.37168E+00_JPRB, 0.38120E+00_JPRB, 0.37841E+00_JPRB, 0.37181E+00_JPRB, 0.36338E+00_JPRB, 0.35530E+00_JPRB, 0.34898E+00_JPRB ,&
&  0.34232E+00_JPRB, 0.33789E+00_JPRB, 0.33375E+00_JPRB, 0.32721E+00_JPRB, 0.31914E+00_JPRB, 0.31253E+00_JPRB, 0.30863E+00_JPRB ,&
&  0.30724E+00_JPRB, 0.30771E+00_JPRB, 0.30844E+00_JPRB, 0.30958E+00_JPRB, 0.31174E+00_JPRB, 0.31619E+00_JPRB, 0.32165E+00_JPRB ,&
&  0.32608E+00_JPRB, 0.33051E+00_JPRB, 0.33558E+00_JPRB, 0.34031E+00_JPRB, 0.34445E+00_JPRB, 0.34560E+00_JPRB, 0.34474E+00_JPRB ,&
&  0.34094E+00_JPRB, 0.33472E+00_JPRB, 0.32822E+00_JPRB, 0.32225E+00_JPRB, 0.31651E+00_JPRB, 0.31048E+00_JPRB, 0.30445E+00_JPRB ,&
&  0.29941E+00_JPRB, 0.29463E+00_JPRB, 0.29082E+00_JPRB, 0.28944E+00_JPRB, 0.28967E+00_JPRB, 0.29166E+00_JPRB, 0.29397E+00_JPRB ,&
&  0.29598E+00_JPRB, 0.29805E+00_JPRB, 0.29955E+00_JPRB, 0.30275E+00_JPRB, 0.30649E+00_JPRB, 0.31117E+00_JPRB, 0.31632E+00_JPRB ,&
&  0.32230E+00_JPRB, 0.32800E+00_JPRB, 0.33478E+00_JPRB, 0.34771E+00_JPRB, 0.36067E+00_JPRB, 0.37237E+00_JPRB, 0.38056E+00_JPRB ,&
&  0.38304E+00_JPRB, 0.38450E+00_JPRB, 0.38778E+00_JPRB, 0.38995E+00_JPRB, 0.39690E+00_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.44628E+00_JPRB, 0.44367E+00_JPRB, 0.43038E+00_JPRB, 0.41665E+00_JPRB, 0.40354E+00_JPRB, 0.39146E+00_JPRB, 0.38330E+00_JPRB ,&
&  0.37817E+00_JPRB, 0.37053E+00_JPRB, 0.36397E+00_JPRB, 0.35977E+00_JPRB, 0.35496E+00_JPRB, 0.35042E+00_JPRB, 0.34509E+00_JPRB ,&
&  0.34109E+00_JPRB, 0.33897E+00_JPRB, 0.33769E+00_JPRB, 0.33591E+00_JPRB, 0.33345E+00_JPRB, 0.32991E+00_JPRB, 0.32602E+00_JPRB ,&
&  0.32202E+00_JPRB, 0.31904E+00_JPRB, 0.31633E+00_JPRB, 0.31539E+00_JPRB, 0.31808E+00_JPRB, 0.32336E+00_JPRB, 0.32848E+00_JPRB ,&
&  0.34499E+00_JPRB, 0.29949E+00_JPRB, 0.31059E+00_JPRB, 0.32684E+00_JPRB, 0.34509E+00_JPRB, 0.37181E+00_JPRB, 0.39496E+00_JPRB ,&
&  0.40993E+00_JPRB, 0.42445E+00_JPRB, 0.42405E+00_JPRB, 0.41840E+00_JPRB, 0.40524E+00_JPRB, 0.39081E+00_JPRB, 0.37802E+00_JPRB ,&
&  0.36957E+00_JPRB, 0.36346E+00_JPRB, 0.35583E+00_JPRB, 0.34708E+00_JPRB, 0.33871E+00_JPRB, 0.33281E+00_JPRB, 0.33064E+00_JPRB ,&
&  0.33115E+00_JPRB, 0.33182E+00_JPRB, 0.33190E+00_JPRB, 0.33431E+00_JPRB, 0.33915E+00_JPRB, 0.34538E+00_JPRB, 0.35254E+00_JPRB ,&
&  0.36125E+00_JPRB, 0.37234E+00_JPRB, 0.38210E+00_JPRB, 0.39024E+00_JPRB, 0.39375E+00_JPRB, 0.39134E+00_JPRB, 0.38092E+00_JPRB ,&
&  0.36763E+00_JPRB, 0.35562E+00_JPRB, 0.34749E+00_JPRB, 0.34276E+00_JPRB, 0.33901E+00_JPRB, 0.33617E+00_JPRB, 0.33301E+00_JPRB ,&
&  0.32864E+00_JPRB, 0.32402E+00_JPRB, 0.32085E+00_JPRB, 0.31935E+00_JPRB, 0.31839E+00_JPRB, 0.31894E+00_JPRB, 0.32035E+00_JPRB ,&
&  0.32210E+00_JPRB, 0.32416E+00_JPRB, 0.32697E+00_JPRB, 0.33149E+00_JPRB, 0.33703E+00_JPRB, 0.34315E+00_JPRB, 0.34820E+00_JPRB ,&
&  0.35400E+00_JPRB, 0.36013E+00_JPRB, 0.36699E+00_JPRB, 0.38252E+00_JPRB, 0.39707E+00_JPRB, 0.40738E+00_JPRB, 0.41655E+00_JPRB ,&
&  0.42185E+00_JPRB, 0.43104E+00_JPRB, 0.37596E+00_JPRB, 0.41445E+00_JPRB, 0.43819E+00_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.51507E+00_JPRB, 0.50469E+00_JPRB, 0.48259E+00_JPRB, 0.45901E+00_JPRB, 0.43857E+00_JPRB, 0.42014E+00_JPRB, 0.40629E+00_JPRB ,&
&  0.39790E+00_JPRB, 0.39231E+00_JPRB, 0.38671E+00_JPRB, 0.38164E+00_JPRB, 0.37575E+00_JPRB, 0.37074E+00_JPRB, 0.36445E+00_JPRB ,&
&  0.36025E+00_JPRB, 0.35787E+00_JPRB, 0.35588E+00_JPRB, 0.35273E+00_JPRB, 0.34899E+00_JPRB, 0.34472E+00_JPRB, 0.33922E+00_JPRB ,&
&  0.33450E+00_JPRB, 0.33202E+00_JPRB, 0.32822E+00_JPRB, 0.32730E+00_JPRB, 0.33316E+00_JPRB, 0.34004E+00_JPRB, 0.34819E+00_JPRB ,&
&  0.37007E+00_JPRB, 0.33010E+00_JPRB, 0.33893E+00_JPRB, 0.37165E+00_JPRB, 0.39310E+00_JPRB, 0.42244E+00_JPRB, 0.44554E+00_JPRB ,&
&  0.45668E+00_JPRB, 0.46659E+00_JPRB, 0.46433E+00_JPRB, 0.45585E+00_JPRB, 0.43736E+00_JPRB, 0.41992E+00_JPRB, 0.40570E+00_JPRB ,&
&  0.39340E+00_JPRB, 0.38282E+00_JPRB, 0.37399E+00_JPRB, 0.36657E+00_JPRB, 0.35957E+00_JPRB, 0.35658E+00_JPRB, 0.35762E+00_JPRB ,&
&  0.35956E+00_JPRB, 0.36212E+00_JPRB, 0.36595E+00_JPRB, 0.37225E+00_JPRB, 0.37821E+00_JPRB, 0.38481E+00_JPRB, 0.39919E+00_JPRB ,&
&  0.42146E+00_JPRB, 0.43773E+00_JPRB, 0.44919E+00_JPRB, 0.45098E+00_JPRB, 0.44385E+00_JPRB, 0.43081E+00_JPRB, 0.41244E+00_JPRB ,&
&  0.39388E+00_JPRB, 0.37967E+00_JPRB, 0.37023E+00_JPRB, 0.36422E+00_JPRB, 0.36050E+00_JPRB, 0.35702E+00_JPRB, 0.35428E+00_JPRB ,&
&  0.35153E+00_JPRB, 0.34937E+00_JPRB, 0.34953E+00_JPRB, 0.35215E+00_JPRB, 0.35571E+00_JPRB, 0.35898E+00_JPRB, 0.36098E+00_JPRB ,&
&  0.36222E+00_JPRB, 0.36341E+00_JPRB, 0.36690E+00_JPRB, 0.37187E+00_JPRB, 0.37721E+00_JPRB, 0.38081E+00_JPRB, 0.38522E+00_JPRB ,&
&  0.39095E+00_JPRB, 0.39766E+00_JPRB, 0.40893E+00_JPRB, 0.42287E+00_JPRB, 0.44432E+00_JPRB, 0.45677E+00_JPRB, 0.46623E+00_JPRB ,&
&  0.47734E+00_JPRB, 0.48219E+00_JPRB, 0.49034E+00_JPRB, 0.44271E+00_JPRB, 0.50620E+00_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.59154E+00_JPRB, 0.55766E+00_JPRB, 0.51291E+00_JPRB, 0.48156E+00_JPRB, 0.45913E+00_JPRB, 0.43587E+00_JPRB, 0.42011E+00_JPRB ,&
&  0.40966E+00_JPRB, 0.40496E+00_JPRB, 0.40108E+00_JPRB, 0.39574E+00_JPRB, 0.39043E+00_JPRB, 0.38735E+00_JPRB, 0.38345E+00_JPRB ,&
&  0.37904E+00_JPRB, 0.37468E+00_JPRB, 0.37014E+00_JPRB, 0.36672E+00_JPRB, 0.36184E+00_JPRB, 0.35778E+00_JPRB, 0.35370E+00_JPRB ,&
&  0.34871E+00_JPRB, 0.34435E+00_JPRB, 0.33952E+00_JPRB, 0.34359E+00_JPRB, 0.35142E+00_JPRB, 0.35713E+00_JPRB, 0.36500E+00_JPRB ,&
&  0.38896E+00_JPRB, 0.36889E+00_JPRB, 0.39628E+00_JPRB, 0.42456E+00_JPRB, 0.44593E+00_JPRB, 0.46851E+00_JPRB, 0.48355E+00_JPRB ,&
&  0.48536E+00_JPRB, 0.49841E+00_JPRB, 0.49939E+00_JPRB, 0.48746E+00_JPRB, 0.47761E+00_JPRB, 0.45467E+00_JPRB, 0.43367E+00_JPRB ,&
&  0.41841E+00_JPRB, 0.40839E+00_JPRB, 0.39941E+00_JPRB, 0.38891E+00_JPRB, 0.38236E+00_JPRB, 0.38101E+00_JPRB, 0.38311E+00_JPRB ,&
&  0.38631E+00_JPRB, 0.39198E+00_JPRB, 0.39762E+00_JPRB, 0.40314E+00_JPRB, 0.40743E+00_JPRB, 0.42577E+00_JPRB, 0.46224E+00_JPRB ,&
&  0.49176E+00_JPRB, 0.50889E+00_JPRB, 0.51498E+00_JPRB, 0.51495E+00_JPRB, 0.50275E+00_JPRB, 0.48626E+00_JPRB, 0.46714E+00_JPRB ,&
&  0.44821E+00_JPRB, 0.42869E+00_JPRB, 0.41059E+00_JPRB, 0.39665E+00_JPRB, 0.38540E+00_JPRB, 0.37762E+00_JPRB, 0.37367E+00_JPRB ,&
&  0.37341E+00_JPRB, 0.37620E+00_JPRB, 0.38080E+00_JPRB, 0.38567E+00_JPRB, 0.38898E+00_JPRB, 0.38979E+00_JPRB, 0.38949E+00_JPRB ,&
&  0.38912E+00_JPRB, 0.38921E+00_JPRB, 0.39137E+00_JPRB, 0.39544E+00_JPRB, 0.40073E+00_JPRB, 0.40615E+00_JPRB, 0.41192E+00_JPRB ,&
&  0.41834E+00_JPRB, 0.42506E+00_JPRB, 0.43565E+00_JPRB, 0.46431E+00_JPRB, 0.47908E+00_JPRB, 0.49757E+00_JPRB, 0.51367E+00_JPRB ,&
&  0.53490E+00_JPRB, 0.54656E+00_JPRB, 0.54457E+00_JPRB, 0.53508E+00_JPRB, 0.56719E+00_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.65834E+00_JPRB, 0.56974E+00_JPRB, 0.51551E+00_JPRB, 0.48843E+00_JPRB, 0.47826E+00_JPRB, 0.46217E+00_JPRB, 0.44602E+00_JPRB ,&
&  0.43178E+00_JPRB, 0.42260E+00_JPRB, 0.41771E+00_JPRB, 0.41149E+00_JPRB, 0.40493E+00_JPRB, 0.39967E+00_JPRB, 0.39380E+00_JPRB ,&
&  0.38727E+00_JPRB, 0.38342E+00_JPRB, 0.38011E+00_JPRB, 0.37787E+00_JPRB, 0.37610E+00_JPRB, 0.37191E+00_JPRB, 0.36686E+00_JPRB ,&
&  0.36300E+00_JPRB, 0.35813E+00_JPRB, 0.35775E+00_JPRB, 0.36396E+00_JPRB, 0.36902E+00_JPRB, 0.37126E+00_JPRB, 0.38215E+00_JPRB ,&
&  0.41216E+00_JPRB, 0.40298E+00_JPRB, 0.43713E+00_JPRB, 0.46635E+00_JPRB, 0.49737E+00_JPRB, 0.54909E+00_JPRB, 0.58184E+00_JPRB ,&
&  0.58803E+00_JPRB, 0.60327E+00_JPRB, 0.59958E+00_JPRB, 0.57062E+00_JPRB, 0.53541E+00_JPRB, 0.51148E+00_JPRB, 0.49337E+00_JPRB ,&
&  0.47789E+00_JPRB, 0.45970E+00_JPRB, 0.44090E+00_JPRB, 0.42592E+00_JPRB, 0.41767E+00_JPRB, 0.41505E+00_JPRB, 0.41482E+00_JPRB ,&
&  0.41611E+00_JPRB, 0.42051E+00_JPRB, 0.42421E+00_JPRB, 0.42972E+00_JPRB, 0.45162E+00_JPRB, 0.50869E+00_JPRB, 0.58804E+00_JPRB ,&
&  0.62525E+00_JPRB, 0.63769E+00_JPRB, 0.64145E+00_JPRB, 0.63679E+00_JPRB, 0.60844E+00_JPRB, 0.57821E+00_JPRB, 0.54857E+00_JPRB ,&
&  0.52051E+00_JPRB, 0.49096E+00_JPRB, 0.46643E+00_JPRB, 0.44679E+00_JPRB, 0.43084E+00_JPRB, 0.42112E+00_JPRB, 0.41732E+00_JPRB ,&
&  0.41635E+00_JPRB, 0.41614E+00_JPRB, 0.41591E+00_JPRB, 0.41415E+00_JPRB, 0.41062E+00_JPRB, 0.40938E+00_JPRB, 0.40882E+00_JPRB ,&
&  0.40886E+00_JPRB, 0.41042E+00_JPRB, 0.41427E+00_JPRB, 0.41981E+00_JPRB, 0.42787E+00_JPRB, 0.43334E+00_JPRB, 0.44070E+00_JPRB ,&
&  0.44955E+00_JPRB, 0.46232E+00_JPRB, 0.46181E+00_JPRB, 0.48084E+00_JPRB, 0.53341E+00_JPRB, 0.55454E+00_JPRB, 0.57960E+00_JPRB ,&
&  0.60617E+00_JPRB, 0.60600E+00_JPRB, 0.58782E+00_JPRB, 0.59359E+00_JPRB, 0.72465E+00_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.53503E+00_JPRB, 0.51096E+00_JPRB, 0.49595E+00_JPRB, 0.49030E+00_JPRB, 0.48776E+00_JPRB, 0.47776E+00_JPRB, 0.46072E+00_JPRB ,&
&  0.44418E+00_JPRB, 0.43030E+00_JPRB, 0.42035E+00_JPRB, 0.41307E+00_JPRB, 0.40835E+00_JPRB, 0.40368E+00_JPRB, 0.39848E+00_JPRB ,&
&  0.39290E+00_JPRB, 0.39064E+00_JPRB, 0.38915E+00_JPRB, 0.38631E+00_JPRB, 0.37912E+00_JPRB, 0.37320E+00_JPRB, 0.37136E+00_JPRB ,&
&  0.37028E+00_JPRB, 0.37127E+00_JPRB, 0.37649E+00_JPRB, 0.38236E+00_JPRB, 0.38465E+00_JPRB, 0.38609E+00_JPRB, 0.39556E+00_JPRB ,&
&  0.42265E+00_JPRB, 0.46337E+00_JPRB, 0.47412E+00_JPRB, 0.51623E+00_JPRB, 0.57412E+00_JPRB, 0.67236E+00_JPRB, 0.77139E+00_JPRB ,&
&  0.79796E+00_JPRB, 0.77048E+00_JPRB, 0.71306E+00_JPRB, 0.65013E+00_JPRB, 0.60004E+00_JPRB, 0.57103E+00_JPRB, 0.54850E+00_JPRB ,&
&  0.52597E+00_JPRB, 0.50320E+00_JPRB, 0.48381E+00_JPRB, 0.46924E+00_JPRB, 0.45939E+00_JPRB, 0.45573E+00_JPRB, 0.45550E+00_JPRB ,&
&  0.45519E+00_JPRB, 0.45383E+00_JPRB, 0.45455E+00_JPRB, 0.47355E+00_JPRB, 0.55911E+00_JPRB, 0.67975E+00_JPRB, 0.78530E+00_JPRB ,&
&  0.82695E+00_JPRB, 0.85750E+00_JPRB, 0.84894E+00_JPRB, 0.80860E+00_JPRB, 0.74635E+00_JPRB, 0.68440E+00_JPRB, 0.61941E+00_JPRB ,&
&  0.57211E+00_JPRB, 0.53683E+00_JPRB, 0.51156E+00_JPRB, 0.49057E+00_JPRB, 0.47142E+00_JPRB, 0.45705E+00_JPRB, 0.44782E+00_JPRB ,&
&  0.44135E+00_JPRB, 0.43575E+00_JPRB, 0.43019E+00_JPRB, 0.42486E+00_JPRB, 0.42035E+00_JPRB, 0.41730E+00_JPRB, 0.41916E+00_JPRB ,&
&  0.42129E+00_JPRB, 0.42272E+00_JPRB, 0.43320E+00_JPRB, 0.44838E+00_JPRB, 0.46328E+00_JPRB, 0.47290E+00_JPRB, 0.47888E+00_JPRB ,&
&  0.48555E+00_JPRB, 0.49667E+00_JPRB, 0.49263E+00_JPRB, 0.49958E+00_JPRB, 0.54738E+00_JPRB, 0.60284E+00_JPRB, 0.61814E+00_JPRB ,&
&  0.61591E+00_JPRB, 0.59882E+00_JPRB, 0.58472E+00_JPRB, 0.57823E+00_JPRB, 0.57354E+00_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.53860E+00_JPRB, 0.51999E+00_JPRB, 0.51053E+00_JPRB, 0.50160E+00_JPRB, 0.49304E+00_JPRB, 0.48082E+00_JPRB, 0.46429E+00_JPRB ,&
&  0.45245E+00_JPRB, 0.44315E+00_JPRB, 0.43214E+00_JPRB, 0.42244E+00_JPRB, 0.41595E+00_JPRB, 0.41102E+00_JPRB, 0.40493E+00_JPRB ,&
&  0.40109E+00_JPRB, 0.39889E+00_JPRB, 0.39022E+00_JPRB, 0.38161E+00_JPRB, 0.37813E+00_JPRB, 0.37786E+00_JPRB, 0.38148E+00_JPRB ,&
&  0.38544E+00_JPRB, 0.39055E+00_JPRB, 0.39451E+00_JPRB, 0.39770E+00_JPRB, 0.39772E+00_JPRB, 0.40007E+00_JPRB, 0.41345E+00_JPRB ,&
&  0.44917E+00_JPRB, 0.49815E+00_JPRB, 0.57806E+00_JPRB, 0.60664E+00_JPRB, 0.67718E+00_JPRB, 0.79970E+00_JPRB, 0.92878E+00_JPRB ,&
&  0.94084E+00_JPRB, 0.86982E+00_JPRB, 0.78101E+00_JPRB, 0.71686E+00_JPRB, 0.66881E+00_JPRB, 0.63256E+00_JPRB, 0.59330E+00_JPRB ,&
&  0.56152E+00_JPRB, 0.54084E+00_JPRB, 0.52108E+00_JPRB, 0.50313E+00_JPRB, 0.49610E+00_JPRB, 0.49418E+00_JPRB, 0.49097E+00_JPRB ,&
&  0.48603E+00_JPRB, 0.48232E+00_JPRB, 0.49243E+00_JPRB, 0.54509E+00_JPRB, 0.66421E+00_JPRB, 0.77110E+00_JPRB, 0.93764E+00_JPRB ,&
&  0.11188E+01_JPRB, 0.11329E+01_JPRB, 0.10668E+01_JPRB, 0.94492E+00_JPRB, 0.81253E+00_JPRB, 0.69861E+00_JPRB, 0.62905E+00_JPRB ,&
&  0.58489E+00_JPRB, 0.54631E+00_JPRB, 0.52217E+00_JPRB, 0.50649E+00_JPRB, 0.49472E+00_JPRB, 0.48496E+00_JPRB, 0.47256E+00_JPRB ,&
&  0.45864E+00_JPRB, 0.44825E+00_JPRB, 0.44176E+00_JPRB, 0.43619E+00_JPRB, 0.43286E+00_JPRB, 0.43196E+00_JPRB, 0.43478E+00_JPRB ,&
&  0.44102E+00_JPRB, 0.45383E+00_JPRB, 0.47437E+00_JPRB, 0.48919E+00_JPRB, 0.51004E+00_JPRB, 0.53470E+00_JPRB, 0.54927E+00_JPRB ,&
&  0.53447E+00_JPRB, 0.52322E+00_JPRB, 0.51162E+00_JPRB, 0.51085E+00_JPRB, 0.57027E+00_JPRB, 0.63611E+00_JPRB, 0.62603E+00_JPRB ,&
&  0.61590E+00_JPRB, 0.60158E+00_JPRB, 0.58471E+00_JPRB, 0.56507E+00_JPRB, 0.56554E+00_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.55136E+00_JPRB, 0.53302E+00_JPRB, 0.51829E+00_JPRB, 0.50198E+00_JPRB, 0.49011E+00_JPRB, 0.48188E+00_JPRB, 0.47101E+00_JPRB ,&
&  0.46102E+00_JPRB, 0.45180E+00_JPRB, 0.44206E+00_JPRB, 0.43243E+00_JPRB, 0.42448E+00_JPRB, 0.41471E+00_JPRB, 0.40699E+00_JPRB ,&
&  0.39963E+00_JPRB, 0.38961E+00_JPRB, 0.38286E+00_JPRB, 0.38110E+00_JPRB, 0.38498E+00_JPRB, 0.39023E+00_JPRB, 0.39680E+00_JPRB ,&
&  0.40465E+00_JPRB, 0.40994E+00_JPRB, 0.41124E+00_JPRB, 0.41351E+00_JPRB, 0.41168E+00_JPRB, 0.41298E+00_JPRB, 0.42896E+00_JPRB ,&
&  0.46733E+00_JPRB, 0.43856E+00_JPRB, 0.64699E+00_JPRB, 0.74790E+00_JPRB, 0.86905E+00_JPRB, 0.98263E+00_JPRB, 0.10462E+01_JPRB ,&
&  0.10708E+01_JPRB, 0.97169E+00_JPRB, 0.86722E+00_JPRB, 0.79689E+00_JPRB, 0.73941E+00_JPRB, 0.69332E+00_JPRB, 0.65635E+00_JPRB ,&
&  0.62115E+00_JPRB, 0.59439E+00_JPRB, 0.57353E+00_JPRB, 0.54374E+00_JPRB, 0.52036E+00_JPRB, 0.50770E+00_JPRB, 0.50086E+00_JPRB ,&
&  0.49401E+00_JPRB, 0.49894E+00_JPRB, 0.52611E+00_JPRB, 0.60510E+00_JPRB, 0.77002E+00_JPRB, 0.92393E+00_JPRB, 0.10085E+01_JPRB ,&
&  0.10953E+01_JPRB, 0.12038E+01_JPRB, 0.11328E+01_JPRB, 0.98405E+00_JPRB, 0.83687E+00_JPRB, 0.72682E+00_JPRB, 0.64074E+00_JPRB ,&
&  0.59130E+00_JPRB, 0.56344E+00_JPRB, 0.54765E+00_JPRB, 0.53556E+00_JPRB, 0.52120E+00_JPRB, 0.50313E+00_JPRB, 0.48304E+00_JPRB ,&
&  0.46825E+00_JPRB, 0.46096E+00_JPRB, 0.45626E+00_JPRB, 0.45507E+00_JPRB, 0.45474E+00_JPRB, 0.45553E+00_JPRB, 0.46097E+00_JPRB ,&
&  0.47224E+00_JPRB, 0.49341E+00_JPRB, 0.50986E+00_JPRB, 0.52461E+00_JPRB, 0.54175E+00_JPRB, 0.55842E+00_JPRB, 0.57248E+00_JPRB ,&
&  0.57384E+00_JPRB, 0.56036E+00_JPRB, 0.54221E+00_JPRB, 0.54506E+00_JPRB, 0.56898E+00_JPRB, 0.62250E+00_JPRB, 0.62664E+00_JPRB ,&
&  0.62483E+00_JPRB, 0.61167E+00_JPRB, 0.58646E+00_JPRB, 0.57551E+00_JPRB, 0.57205E+00_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.55928E+00_JPRB, 0.54647E+00_JPRB, 0.52976E+00_JPRB, 0.51349E+00_JPRB, 0.50657E+00_JPRB, 0.50149E+00_JPRB, 0.48900E+00_JPRB ,&
&  0.47853E+00_JPRB, 0.47114E+00_JPRB, 0.46208E+00_JPRB, 0.45016E+00_JPRB, 0.43611E+00_JPRB, 0.41775E+00_JPRB, 0.40093E+00_JPRB ,&
&  0.39292E+00_JPRB, 0.38625E+00_JPRB, 0.38923E+00_JPRB, 0.39439E+00_JPRB, 0.40261E+00_JPRB, 0.40937E+00_JPRB, 0.41712E+00_JPRB ,&
&  0.42271E+00_JPRB, 0.42526E+00_JPRB, 0.42907E+00_JPRB, 0.43205E+00_JPRB, 0.43264E+00_JPRB, 0.43684E+00_JPRB, 0.45542E+00_JPRB ,&
&  0.50418E+00_JPRB, 0.68231E+00_JPRB, 0.61501E+00_JPRB, 0.94309E+00_JPRB, 0.11407E+01_JPRB, 0.12401E+01_JPRB, 0.11870E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.10975E+01_JPRB, 0.97553E+00_JPRB, 0.87588E+00_JPRB, 0.79808E+00_JPRB, 0.74672E+00_JPRB, 0.71052E+00_JPRB ,&
&  0.68392E+00_JPRB, 0.65155E+00_JPRB, 0.61155E+00_JPRB, 0.57159E+00_JPRB, 0.54527E+00_JPRB, 0.52816E+00_JPRB, 0.52047E+00_JPRB ,&
&  0.51723E+00_JPRB, 0.53297E+00_JPRB, 0.58362E+00_JPRB, 0.74215E+00_JPRB, 0.92182E+00_JPRB, 0.10470E+01_JPRB, 0.10249E+01_JPRB ,&
&  0.98102E+00_JPRB, 0.11067E+01_JPRB, 0.10489E+01_JPRB, 0.90193E+00_JPRB, 0.79021E+00_JPRB, 0.69640E+00_JPRB, 0.63029E+00_JPRB ,&
&  0.59011E+00_JPRB, 0.56584E+00_JPRB, 0.55175E+00_JPRB, 0.54078E+00_JPRB, 0.52715E+00_JPRB, 0.51219E+00_JPRB, 0.50031E+00_JPRB ,&
&  0.49241E+00_JPRB, 0.48603E+00_JPRB, 0.48159E+00_JPRB, 0.48170E+00_JPRB, 0.48060E+00_JPRB, 0.48301E+00_JPRB, 0.48957E+00_JPRB ,&
&  0.50209E+00_JPRB, 0.52744E+00_JPRB, 0.53821E+00_JPRB, 0.55317E+00_JPRB, 0.55819E+00_JPRB, 0.55925E+00_JPRB, 0.56145E+00_JPRB ,&
&  0.58660E+00_JPRB, 0.58575E+00_JPRB, 0.57596E+00_JPRB, 0.57593E+00_JPRB, 0.59221E+00_JPRB, 0.62761E+00_JPRB, 0.63048E+00_JPRB ,&
&  0.63103E+00_JPRB, 0.61899E+00_JPRB, 0.60094E+00_JPRB, 0.58573E+00_JPRB, 0.57529E+00_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.56784E+00_JPRB, 0.55326E+00_JPRB, 0.54132E+00_JPRB, 0.53675E+00_JPRB, 0.53037E+00_JPRB, 0.52394E+00_JPRB, 0.51075E+00_JPRB ,&
&  0.49657E+00_JPRB, 0.48809E+00_JPRB, 0.47638E+00_JPRB, 0.46535E+00_JPRB, 0.44320E+00_JPRB, 0.41581E+00_JPRB, 0.40541E+00_JPRB ,&
&  0.39925E+00_JPRB, 0.40069E+00_JPRB, 0.41003E+00_JPRB, 0.41390E+00_JPRB, 0.41826E+00_JPRB, 0.42715E+00_JPRB, 0.43586E+00_JPRB ,&
&  0.44205E+00_JPRB, 0.44222E+00_JPRB, 0.44689E+00_JPRB, 0.45303E+00_JPRB, 0.45829E+00_JPRB, 0.47674E+00_JPRB, 0.49929E+00_JPRB ,&
&  0.56729E+00_JPRB, 0.71368E+00_JPRB, 0.63408E+00_JPRB, 0.11993E+01_JPRB, 0.14356E+01_JPRB, 0.14621E+01_JPRB, 0.13622E+01_JPRB ,&
&  0.13005E+01_JPRB, 0.12670E+01_JPRB, 0.10748E+01_JPRB, 0.93498E+00_JPRB, 0.83282E+00_JPRB, 0.77786E+00_JPRB, 0.74319E+00_JPRB ,&
&  0.71467E+00_JPRB, 0.68224E+00_JPRB, 0.63826E+00_JPRB, 0.59871E+00_JPRB, 0.57365E+00_JPRB, 0.55620E+00_JPRB, 0.55264E+00_JPRB ,&
&  0.55517E+00_JPRB, 0.58429E+00_JPRB, 0.67687E+00_JPRB, 0.88177E+00_JPRB, 0.10576E+01_JPRB, 0.10934E+01_JPRB, 0.99231E+00_JPRB ,&
&  0.95773E+00_JPRB, 0.10236E+01_JPRB, 0.95340E+00_JPRB, 0.81630E+00_JPRB, 0.72101E+00_JPRB, 0.66570E+00_JPRB, 0.61054E+00_JPRB ,&
&  0.57638E+00_JPRB, 0.55349E+00_JPRB, 0.54326E+00_JPRB, 0.53585E+00_JPRB, 0.52963E+00_JPRB, 0.52667E+00_JPRB, 0.52379E+00_JPRB ,&
&  0.52079E+00_JPRB, 0.51376E+00_JPRB, 0.50813E+00_JPRB, 0.50048E+00_JPRB, 0.50290E+00_JPRB, 0.50743E+00_JPRB, 0.51423E+00_JPRB ,&
&  0.53493E+00_JPRB, 0.57049E+00_JPRB, 0.57297E+00_JPRB, 0.58646E+00_JPRB, 0.58333E+00_JPRB, 0.56949E+00_JPRB, 0.57408E+00_JPRB ,&
&  0.59281E+00_JPRB, 0.59620E+00_JPRB, 0.60486E+00_JPRB, 0.59662E+00_JPRB, 0.60834E+00_JPRB, 0.63244E+00_JPRB, 0.63739E+00_JPRB ,&
&  0.64404E+00_JPRB, 0.63606E+00_JPRB, 0.62177E+00_JPRB, 0.59821E+00_JPRB, 0.58299E+00_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.56477E+00_JPRB, 0.54851E+00_JPRB, 0.54329E+00_JPRB, 0.54236E+00_JPRB, 0.53820E+00_JPRB, 0.52886E+00_JPRB, 0.51308E+00_JPRB ,&
&  0.49402E+00_JPRB, 0.48076E+00_JPRB, 0.46617E+00_JPRB, 0.44838E+00_JPRB, 0.42207E+00_JPRB, 0.39715E+00_JPRB, 0.39687E+00_JPRB ,&
&  0.41191E+00_JPRB, 0.42097E+00_JPRB, 0.42911E+00_JPRB, 0.43778E+00_JPRB, 0.44707E+00_JPRB, 0.45881E+00_JPRB, 0.46748E+00_JPRB ,&
&  0.46980E+00_JPRB, 0.47415E+00_JPRB, 0.48430E+00_JPRB, 0.49700E+00_JPRB, 0.51358E+00_JPRB, 0.53865E+00_JPRB, 0.57165E+00_JPRB ,&
&  0.65861E+00_JPRB, 0.76359E+00_JPRB, 0.76504E+00_JPRB, 0.12870E+01_JPRB, 0.15784E+01_JPRB, 0.14906E+01_JPRB, 0.14461E+01_JPRB ,&
&  0.13375E+01_JPRB, 0.11366E+01_JPRB, 0.10857E+01_JPRB, 0.93550E+00_JPRB, 0.83871E+00_JPRB, 0.78863E+00_JPRB, 0.75678E+00_JPRB ,&
&  0.73123E+00_JPRB, 0.70021E+00_JPRB, 0.66008E+00_JPRB, 0.63529E+00_JPRB, 0.61257E+00_JPRB, 0.59927E+00_JPRB, 0.59990E+00_JPRB ,&
&  0.61087E+00_JPRB, 0.65996E+00_JPRB, 0.81085E+00_JPRB, 0.10504E+01_JPRB, 0.10961E+01_JPRB, 0.10406E+01_JPRB, 0.90459E+00_JPRB ,&
&  0.90145E+00_JPRB, 0.89457E+00_JPRB, 0.83656E+00_JPRB, 0.73650E+00_JPRB, 0.65010E+00_JPRB, 0.64773E+00_JPRB, 0.59426E+00_JPRB ,&
&  0.56527E+00_JPRB, 0.54706E+00_JPRB, 0.53996E+00_JPRB, 0.53584E+00_JPRB, 0.53592E+00_JPRB, 0.53643E+00_JPRB, 0.53546E+00_JPRB ,&
&  0.53570E+00_JPRB, 0.53687E+00_JPRB, 0.53100E+00_JPRB, 0.53103E+00_JPRB, 0.53447E+00_JPRB, 0.54072E+00_JPRB, 0.55495E+00_JPRB ,&
&  0.58793E+00_JPRB, 0.62360E+00_JPRB, 0.62905E+00_JPRB, 0.63662E+00_JPRB, 0.62341E+00_JPRB, 0.60745E+00_JPRB, 0.60471E+00_JPRB ,&
&  0.60535E+00_JPRB, 0.62637E+00_JPRB, 0.64106E+00_JPRB, 0.63627E+00_JPRB, 0.63949E+00_JPRB, 0.64820E+00_JPRB, 0.65938E+00_JPRB ,&
&  0.67126E+00_JPRB, 0.66538E+00_JPRB, 0.66318E+00_JPRB, 0.60817E+00_JPRB, 0.58844E+00_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.54280E+00_JPRB, 0.53007E+00_JPRB, 0.51622E+00_JPRB, 0.51214E+00_JPRB, 0.50739E+00_JPRB, 0.49315E+00_JPRB, 0.47499E+00_JPRB ,&
&  0.45418E+00_JPRB, 0.43866E+00_JPRB, 0.41876E+00_JPRB, 0.40223E+00_JPRB, 0.40168E+00_JPRB, 0.41356E+00_JPRB, 0.43716E+00_JPRB ,&
&  0.46081E+00_JPRB, 0.47462E+00_JPRB, 0.48524E+00_JPRB, 0.49577E+00_JPRB, 0.50545E+00_JPRB, 0.51057E+00_JPRB, 0.51043E+00_JPRB ,&
&  0.51582E+00_JPRB, 0.52931E+00_JPRB, 0.54957E+00_JPRB, 0.57234E+00_JPRB, 0.59981E+00_JPRB, 0.63809E+00_JPRB, 0.69889E+00_JPRB ,&
&  0.81795E+00_JPRB, 0.87071E+00_JPRB, 0.51566E+00_JPRB, 0.96895E+00_JPRB, 0.13821E+01_JPRB, 0.13674E+01_JPRB, 0.12828E+01_JPRB ,&
&  0.11667E+01_JPRB, 0.98726E+00_JPRB, 0.95072E+00_JPRB, 0.90560E+00_JPRB, 0.82094E+00_JPRB, 0.78102E+00_JPRB, 0.75962E+00_JPRB ,&
&  0.74703E+00_JPRB, 0.72401E+00_JPRB, 0.70498E+00_JPRB, 0.69140E+00_JPRB, 0.67022E+00_JPRB, 0.65562E+00_JPRB, 0.65553E+00_JPRB ,&
&  0.67898E+00_JPRB, 0.76622E+00_JPRB, 0.97994E+00_JPRB, 0.10800E+01_JPRB, 0.10966E+01_JPRB, 0.96067E+00_JPRB, 0.79810E+00_JPRB ,&
&  0.74496E+00_JPRB, 0.77934E+00_JPRB, 0.75464E+00_JPRB, 0.68761E+00_JPRB, 0.61917E+00_JPRB, 0.58462E+00_JPRB, 0.58851E+00_JPRB ,&
&  0.56156E+00_JPRB, 0.54319E+00_JPRB, 0.53364E+00_JPRB, 0.52799E+00_JPRB, 0.52557E+00_JPRB, 0.52592E+00_JPRB, 0.52950E+00_JPRB ,&
&  0.53880E+00_JPRB, 0.54345E+00_JPRB, 0.55055E+00_JPRB, 0.56142E+00_JPRB, 0.57198E+00_JPRB, 0.58634E+00_JPRB, 0.61329E+00_JPRB ,&
&  0.65265E+00_JPRB, 0.68261E+00_JPRB, 0.70536E+00_JPRB, 0.69392E+00_JPRB, 0.67009E+00_JPRB, 0.65697E+00_JPRB, 0.66438E+00_JPRB ,&
&  0.66463E+00_JPRB, 0.67765E+00_JPRB, 0.65757E+00_JPRB, 0.64487E+00_JPRB, 0.67320E+00_JPRB, 0.68124E+00_JPRB, 0.72181E+00_JPRB ,&
&  0.77338E+00_JPRB, 0.85294E+00_JPRB, 0.87318E+00_JPRB, 0.60908E+00_JPRB, 0.57032E+00_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.50325E+00_JPRB, 0.48536E+00_JPRB, 0.45986E+00_JPRB, 0.45020E+00_JPRB, 0.45473E+00_JPRB, 0.44879E+00_JPRB, 0.44309E+00_JPRB ,&
&  0.43736E+00_JPRB, 0.43306E+00_JPRB, 0.42935E+00_JPRB, 0.43890E+00_JPRB, 0.46451E+00_JPRB, 0.49945E+00_JPRB, 0.53087E+00_JPRB ,&
&  0.55280E+00_JPRB, 0.56312E+00_JPRB, 0.56604E+00_JPRB, 0.57079E+00_JPRB, 0.57439E+00_JPRB, 0.57779E+00_JPRB, 0.58276E+00_JPRB ,&
&  0.60184E+00_JPRB, 0.63174E+00_JPRB, 0.66951E+00_JPRB, 0.70917E+00_JPRB, 0.75437E+00_JPRB, 0.81590E+00_JPRB, 0.91291E+00_JPRB ,&
&  0.10618E+01_JPRB, 0.81025E+00_JPRB, 0.60903E+00_JPRB, 0.12359E+01_JPRB, 0.13949E+01_JPRB, 0.13558E+01_JPRB, 0.11555E+01_JPRB ,&
&  0.10606E+01_JPRB, 0.94425E+00_JPRB, 0.87750E+00_JPRB, 0.89247E+00_JPRB, 0.80833E+00_JPRB, 0.77593E+00_JPRB, 0.76882E+00_JPRB ,&
&  0.76498E+00_JPRB, 0.76251E+00_JPRB, 0.75628E+00_JPRB, 0.74314E+00_JPRB, 0.72543E+00_JPRB, 0.71743E+00_JPRB, 0.72937E+00_JPRB ,&
&  0.79068E+00_JPRB, 0.92592E+00_JPRB, 0.11406E+01_JPRB, 0.10930E+01_JPRB, 0.10966E+01_JPRB, 0.89757E+00_JPRB, 0.75378E+00_JPRB ,&
&  0.70007E+00_JPRB, 0.71233E+00_JPRB, 0.70673E+00_JPRB, 0.67389E+00_JPRB, 0.61951E+00_JPRB, 0.57042E+00_JPRB, 0.58272E+00_JPRB ,&
&  0.55278E+00_JPRB, 0.53350E+00_JPRB, 0.51797E+00_JPRB, 0.51155E+00_JPRB, 0.51532E+00_JPRB, 0.52239E+00_JPRB, 0.52575E+00_JPRB ,&
&  0.53666E+00_JPRB, 0.54472E+00_JPRB, 0.56374E+00_JPRB, 0.58410E+00_JPRB, 0.60533E+00_JPRB, 0.63201E+00_JPRB, 0.67194E+00_JPRB ,&
&  0.70752E+00_JPRB, 0.73235E+00_JPRB, 0.74799E+00_JPRB, 0.75228E+00_JPRB, 0.73234E+00_JPRB, 0.70614E+00_JPRB, 0.70912E+00_JPRB ,&
&  0.70020E+00_JPRB, 0.70378E+00_JPRB, 0.69211E+00_JPRB, 0.70307E+00_JPRB, 0.71605E+00_JPRB, 0.73130E+00_JPRB, 0.76750E+00_JPRB ,&
&  0.81193E+00_JPRB, 0.82040E+00_JPRB, 0.74699E+00_JPRB, 0.56574E+00_JPRB, 0.52389E+00_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.48602E+00_JPRB, 0.46288E+00_JPRB, 0.45230E+00_JPRB, 0.45137E+00_JPRB, 0.45775E+00_JPRB, 0.46118E+00_JPRB, 0.46059E+00_JPRB ,&
&  0.46282E+00_JPRB, 0.47383E+00_JPRB, 0.48751E+00_JPRB, 0.52757E+00_JPRB, 0.56768E+00_JPRB, 0.59599E+00_JPRB, 0.61337E+00_JPRB ,&
&  0.62947E+00_JPRB, 0.63818E+00_JPRB, 0.64173E+00_JPRB, 0.64732E+00_JPRB, 0.65220E+00_JPRB, 0.66494E+00_JPRB, 0.69028E+00_JPRB ,&
&  0.72917E+00_JPRB, 0.77005E+00_JPRB, 0.82006E+00_JPRB, 0.88025E+00_JPRB, 0.95283E+00_JPRB, 0.10191E+01_JPRB, 0.10727E+01_JPRB ,&
&  0.85223E+00_JPRB, 0.52368E+00_JPRB, 0.12333E+01_JPRB, 0.14550E+01_JPRB, 0.12812E+01_JPRB, 0.11943E+01_JPRB, 0.10833E+01_JPRB ,&
&  0.95076E+00_JPRB, 0.89827E+00_JPRB, 0.85997E+00_JPRB, 0.87583E+00_JPRB, 0.80180E+00_JPRB, 0.77853E+00_JPRB, 0.77289E+00_JPRB ,&
&  0.77555E+00_JPRB, 0.78571E+00_JPRB, 0.78513E+00_JPRB, 0.77544E+00_JPRB, 0.77285E+00_JPRB, 0.78843E+00_JPRB, 0.84397E+00_JPRB ,&
&  0.95665E+00_JPRB, 0.11114E+01_JPRB, 0.12393E+01_JPRB, 0.10743E+01_JPRB, 0.10980E+01_JPRB, 0.92977E+00_JPRB, 0.75745E+00_JPRB ,&
&  0.69802E+00_JPRB, 0.66817E+00_JPRB, 0.65939E+00_JPRB, 0.67828E+00_JPRB, 0.64051E+00_JPRB, 0.57944E+00_JPRB, 0.58151E+00_JPRB ,&
&  0.54677E+00_JPRB, 0.52133E+00_JPRB, 0.51143E+00_JPRB, 0.51365E+00_JPRB, 0.52204E+00_JPRB, 0.53255E+00_JPRB, 0.54307E+00_JPRB ,&
&  0.54785E+00_JPRB, 0.56064E+00_JPRB, 0.57550E+00_JPRB, 0.60161E+00_JPRB, 0.63128E+00_JPRB, 0.67647E+00_JPRB, 0.71731E+00_JPRB ,&
&  0.74473E+00_JPRB, 0.76954E+00_JPRB, 0.79366E+00_JPRB, 0.81269E+00_JPRB, 0.82874E+00_JPRB, 0.79935E+00_JPRB, 0.76899E+00_JPRB ,&
&  0.76245E+00_JPRB, 0.76309E+00_JPRB, 0.77848E+00_JPRB, 0.79860E+00_JPRB, 0.76830E+00_JPRB, 0.75549E+00_JPRB, 0.78115E+00_JPRB ,&
&  0.78380E+00_JPRB, 0.74762E+00_JPRB, 0.64944E+00_JPRB, 0.54384E+00_JPRB, 0.50697E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.52864E+00_JPRB, 0.51600E+00_JPRB, 0.50180E+00_JPRB, 0.50384E+00_JPRB, 0.50642E+00_JPRB, 0.52331E+00_JPRB, 0.52377E+00_JPRB ,&
&  0.53668E+00_JPRB, 0.57558E+00_JPRB, 0.62145E+00_JPRB, 0.64380E+00_JPRB, 0.66318E+00_JPRB, 0.68268E+00_JPRB, 0.69719E+00_JPRB ,&
&  0.70810E+00_JPRB, 0.71887E+00_JPRB, 0.72998E+00_JPRB, 0.74347E+00_JPRB, 0.76276E+00_JPRB, 0.78921E+00_JPRB, 0.82960E+00_JPRB ,&
&  0.86793E+00_JPRB, 0.91048E+00_JPRB, 0.96537E+00_JPRB, 0.10358E+01_JPRB, 0.11118E+01_JPRB, 0.11513E+01_JPRB, 0.11755E+01_JPRB ,&
&  0.77134E+00_JPRB, 0.11972E+01_JPRB, 0.13938E+01_JPRB, 0.12986E+01_JPRB, 0.11726E+01_JPRB, 0.10921E+01_JPRB, 0.10269E+01_JPRB ,&
&  0.94572E+00_JPRB, 0.87039E+00_JPRB, 0.84016E+00_JPRB, 0.87248E+00_JPRB, 0.80601E+00_JPRB, 0.79309E+00_JPRB, 0.79035E+00_JPRB ,&
&  0.81303E+00_JPRB, 0.82748E+00_JPRB, 0.82875E+00_JPRB, 0.84056E+00_JPRB, 0.86379E+00_JPRB, 0.92106E+00_JPRB, 0.10138E+01_JPRB ,&
&  0.11328E+01_JPRB, 0.12398E+01_JPRB, 0.12892E+01_JPRB, 0.10869E+01_JPRB, 0.11137E+01_JPRB, 0.10030E+01_JPRB, 0.81390E+00_JPRB ,&
&  0.68041E+00_JPRB, 0.66183E+00_JPRB, 0.68166E+00_JPRB, 0.70690E+00_JPRB, 0.66216E+00_JPRB, 0.61876E+00_JPRB, 0.58857E+00_JPRB ,&
&  0.55326E+00_JPRB, 0.52993E+00_JPRB, 0.52884E+00_JPRB, 0.52791E+00_JPRB, 0.53530E+00_JPRB, 0.54174E+00_JPRB, 0.55475E+00_JPRB ,&
&  0.56198E+00_JPRB, 0.57934E+00_JPRB, 0.60141E+00_JPRB, 0.62533E+00_JPRB, 0.65663E+00_JPRB, 0.69459E+00_JPRB, 0.73711E+00_JPRB ,&
&  0.76882E+00_JPRB, 0.79980E+00_JPRB, 0.82690E+00_JPRB, 0.84974E+00_JPRB, 0.85909E+00_JPRB, 0.85865E+00_JPRB, 0.83430E+00_JPRB ,&
&  0.81727E+00_JPRB, 0.82648E+00_JPRB, 0.83239E+00_JPRB, 0.82603E+00_JPRB, 0.79811E+00_JPRB, 0.77605E+00_JPRB, 0.75610E+00_JPRB ,&
&  0.71208E+00_JPRB, 0.63673E+00_JPRB, 0.56536E+00_JPRB, 0.54421E+00_JPRB, 0.53178E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.60253E+00_JPRB, 0.60875E+00_JPRB, 0.60853E+00_JPRB, 0.64011E+00_JPRB, 0.64558E+00_JPRB, 0.65634E+00_JPRB, 0.66654E+00_JPRB ,&
&  0.67333E+00_JPRB, 0.68955E+00_JPRB, 0.70949E+00_JPRB, 0.73203E+00_JPRB, 0.74978E+00_JPRB, 0.76311E+00_JPRB, 0.77360E+00_JPRB ,&
&  0.78708E+00_JPRB, 0.80312E+00_JPRB, 0.82286E+00_JPRB, 0.84662E+00_JPRB, 0.88300E+00_JPRB, 0.92195E+00_JPRB, 0.96206E+00_JPRB ,&
&  0.99837E+00_JPRB, 0.10526E+01_JPRB, 0.11234E+01_JPRB, 0.12109E+01_JPRB, 0.13014E+01_JPRB, 0.13650E+01_JPRB, 0.13894E+01_JPRB ,&
&  0.11669E+01_JPRB, 0.14036E+01_JPRB, 0.13583E+01_JPRB, 0.12259E+01_JPRB, 0.11099E+01_JPRB, 0.10412E+01_JPRB, 0.97831E+00_JPRB ,&
&  0.92445E+00_JPRB, 0.85066E+00_JPRB, 0.82047E+00_JPRB, 0.82171E+00_JPRB, 0.82236E+00_JPRB, 0.81900E+00_JPRB, 0.83611E+00_JPRB ,&
&  0.87100E+00_JPRB, 0.89959E+00_JPRB, 0.93391E+00_JPRB, 0.96243E+00_JPRB, 0.10062E+01_JPRB, 0.10790E+01_JPRB, 0.11582E+01_JPRB ,&
&  0.12393E+01_JPRB, 0.12934E+01_JPRB, 0.13552E+01_JPRB, 0.12099E+01_JPRB, 0.11703E+01_JPRB, 0.11278E+01_JPRB, 0.84304E+00_JPRB ,&
&  0.68078E+00_JPRB, 0.65557E+00_JPRB, 0.71205E+00_JPRB, 0.75817E+00_JPRB, 0.71044E+00_JPRB, 0.65075E+00_JPRB, 0.60519E+00_JPRB ,&
&  0.56656E+00_JPRB, 0.53805E+00_JPRB, 0.54312E+00_JPRB, 0.53307E+00_JPRB, 0.53259E+00_JPRB, 0.54881E+00_JPRB, 0.56542E+00_JPRB ,&
&  0.60119E+00_JPRB, 0.61572E+00_JPRB, 0.64914E+00_JPRB, 0.67834E+00_JPRB, 0.71249E+00_JPRB, 0.74618E+00_JPRB, 0.77930E+00_JPRB ,&
&  0.80517E+00_JPRB, 0.81624E+00_JPRB, 0.81801E+00_JPRB, 0.84423E+00_JPRB, 0.85092E+00_JPRB, 0.86502E+00_JPRB, 0.88961E+00_JPRB ,&
&  0.91815E+00_JPRB, 0.93707E+00_JPRB, 0.92461E+00_JPRB, 0.90975E+00_JPRB, 0.87934E+00_JPRB, 0.84943E+00_JPRB, 0.65006E+00_JPRB ,&
&  0.61965E+00_JPRB, 0.58474E+00_JPRB, 0.59868E+00_JPRB, 0.61625E+00_JPRB, 0.60764E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.73670E+00_JPRB, 0.76575E+00_JPRB, 0.76755E+00_JPRB, 0.78757E+00_JPRB, 0.78136E+00_JPRB, 0.76624E+00_JPRB, 0.77032E+00_JPRB ,&
&  0.78193E+00_JPRB, 0.78914E+00_JPRB, 0.80811E+00_JPRB, 0.82751E+00_JPRB, 0.84175E+00_JPRB, 0.84526E+00_JPRB, 0.85695E+00_JPRB ,&
&  0.87756E+00_JPRB, 0.90540E+00_JPRB, 0.93719E+00_JPRB, 0.97899E+00_JPRB, 0.10057E+01_JPRB, 0.10187E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10726E+01_JPRB, 0.11301E+01_JPRB, 0.11978E+01_JPRB, 0.12653E+01_JPRB, 0.13061E+01_JPRB, 0.13208E+01_JPRB, 0.10725E+01_JPRB ,&
&  0.14617E+01_JPRB, 0.15061E+01_JPRB, 0.14165E+01_JPRB, 0.11993E+01_JPRB, 0.10867E+01_JPRB, 0.10110E+01_JPRB, 0.94810E+00_JPRB ,&
&  0.89880E+00_JPRB, 0.84520E+00_JPRB, 0.80295E+00_JPRB, 0.78438E+00_JPRB, 0.84183E+00_JPRB, 0.85327E+00_JPRB, 0.89020E+00_JPRB ,&
&  0.93337E+00_JPRB, 0.97423E+00_JPRB, 0.10117E+01_JPRB, 0.10449E+01_JPRB, 0.10987E+01_JPRB, 0.11611E+01_JPRB, 0.12177E+01_JPRB ,&
&  0.12643E+01_JPRB, 0.12953E+01_JPRB, 0.13794E+01_JPRB, 0.12309E+01_JPRB, 0.12454E+01_JPRB, 0.12498E+01_JPRB, 0.98069E+00_JPRB ,&
&  0.71725E+00_JPRB, 0.68591E+00_JPRB, 0.75039E+00_JPRB, 0.78025E+00_JPRB, 0.74433E+00_JPRB, 0.66822E+00_JPRB, 0.61868E+00_JPRB ,&
&  0.56351E+00_JPRB, 0.55728E+00_JPRB, 0.55348E+00_JPRB, 0.54164E+00_JPRB, 0.53317E+00_JPRB, 0.55903E+00_JPRB, 0.58166E+00_JPRB ,&
&  0.63113E+00_JPRB, 0.66534E+00_JPRB, 0.71052E+00_JPRB, 0.75204E+00_JPRB, 0.81351E+00_JPRB, 0.89057E+00_JPRB, 0.96576E+00_JPRB ,&
&  0.84967E+00_JPRB, 0.79390E+00_JPRB, 0.83386E+00_JPRB, 0.89108E+00_JPRB, 0.93147E+00_JPRB, 0.10190E+01_JPRB, 0.11241E+01_JPRB ,&
&  0.12490E+01_JPRB, 0.13506E+01_JPRB, 0.12384E+01_JPRB, 0.10405E+01_JPRB, 0.14220E+01_JPRB, 0.14736E+01_JPRB, 0.67437E+00_JPRB ,&
&  0.66294E+00_JPRB, 0.69279E+00_JPRB, 0.70872E+00_JPRB, 0.72789E+00_JPRB, 0.74620E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.80418E+00_JPRB, 0.81497E+00_JPRB, 0.82525E+00_JPRB, 0.83171E+00_JPRB, 0.83031E+00_JPRB, 0.83856E+00_JPRB, 0.85621E+00_JPRB ,&
&  0.87310E+00_JPRB, 0.88941E+00_JPRB, 0.90520E+00_JPRB, 0.91672E+00_JPRB, 0.92987E+00_JPRB, 0.93503E+00_JPRB, 0.95320E+00_JPRB ,&
&  0.98029E+00_JPRB, 0.10053E+01_JPRB, 0.10138E+01_JPRB, 0.10377E+01_JPRB, 0.10461E+01_JPRB, 0.10442E+01_JPRB, 0.10702E+01_JPRB ,&
&  0.10988E+01_JPRB, 0.11420E+01_JPRB, 0.11719E+01_JPRB, 0.11875E+01_JPRB, 0.11806E+01_JPRB, 0.12386E+01_JPRB, 0.10925E+01_JPRB ,&
&  0.13916E+01_JPRB, 0.15526E+01_JPRB, 0.14213E+01_JPRB, 0.11458E+01_JPRB, 0.10360E+01_JPRB, 0.97743E+00_JPRB, 0.92307E+00_JPRB ,&
&  0.88768E+00_JPRB, 0.84750E+00_JPRB, 0.80337E+00_JPRB, 0.83028E+00_JPRB, 0.85608E+00_JPRB, 0.88917E+00_JPRB, 0.93535E+00_JPRB ,&
&  0.97172E+00_JPRB, 0.10097E+01_JPRB, 0.10474E+01_JPRB, 0.10808E+01_JPRB, 0.11376E+01_JPRB, 0.11859E+01_JPRB, 0.12250E+01_JPRB ,&
&  0.12559E+01_JPRB, 0.12971E+01_JPRB, 0.13512E+01_JPRB, 0.12478E+01_JPRB, 0.12903E+01_JPRB, 0.12573E+01_JPRB, 0.10636E+01_JPRB ,&
&  0.78663E+00_JPRB, 0.69556E+00_JPRB, 0.73607E+00_JPRB, 0.78849E+00_JPRB, 0.76603E+00_JPRB, 0.67188E+00_JPRB, 0.64974E+00_JPRB ,&
&  0.57916E+00_JPRB, 0.53727E+00_JPRB, 0.54061E+00_JPRB, 0.53303E+00_JPRB, 0.52519E+00_JPRB, 0.55602E+00_JPRB, 0.58014E+00_JPRB ,&
&  0.61799E+00_JPRB, 0.67357E+00_JPRB, 0.71419E+00_JPRB, 0.73836E+00_JPRB, 0.80625E+00_JPRB, 0.87176E+00_JPRB, 0.82561E+00_JPRB ,&
&  0.87506E+00_JPRB, 0.90851E+00_JPRB, 0.94447E+00_JPRB, 0.10592E+01_JPRB, 0.10499E+01_JPRB, 0.12523E+01_JPRB, 0.14741E+01_JPRB ,&
&  0.17752E+01_JPRB, 0.17226E+01_JPRB, 0.20336E+01_JPRB, 0.30673E+01_JPRB, 0.26316E+01_JPRB, 0.22099E+01_JPRB, 0.80385E+00_JPRB ,&
&  0.78891E+00_JPRB, 0.79899E+00_JPRB, 0.81681E+00_JPRB, 0.81461E+00_JPRB, 0.81186E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.82938E+00_JPRB, 0.84191E+00_JPRB, 0.85050E+00_JPRB, 0.86521E+00_JPRB, 0.88356E+00_JPRB, 0.90557E+00_JPRB, 0.92161E+00_JPRB ,&
&  0.93994E+00_JPRB, 0.95431E+00_JPRB, 0.96753E+00_JPRB, 0.98249E+00_JPRB, 0.99253E+00_JPRB, 0.10076E+01_JPRB, 0.10095E+01_JPRB ,&
&  0.10259E+01_JPRB, 0.10192E+01_JPRB, 0.10242E+01_JPRB, 0.10263E+01_JPRB, 0.10265E+01_JPRB, 0.10267E+01_JPRB, 0.10333E+01_JPRB ,&
&  0.10374E+01_JPRB, 0.10307E+01_JPRB, 0.10371E+01_JPRB, 0.10482E+01_JPRB, 0.10511E+01_JPRB, 0.11063E+01_JPRB, 0.95044E+00_JPRB ,&
&  0.13060E+01_JPRB, 0.14513E+01_JPRB, 0.13594E+01_JPRB, 0.10772E+01_JPRB, 0.96225E+00_JPRB, 0.91944E+00_JPRB, 0.89094E+00_JPRB ,&
&  0.86409E+00_JPRB, 0.84883E+00_JPRB, 0.82761E+00_JPRB, 0.83358E+00_JPRB, 0.85699E+00_JPRB, 0.89103E+00_JPRB, 0.92128E+00_JPRB ,&
&  0.94258E+00_JPRB, 0.96598E+00_JPRB, 0.99686E+00_JPRB, 0.10323E+01_JPRB, 0.10943E+01_JPRB, 0.11309E+01_JPRB, 0.11448E+01_JPRB ,&
&  0.11475E+01_JPRB, 0.11619E+01_JPRB, 0.11016E+01_JPRB, 0.11055E+01_JPRB, 0.11695E+01_JPRB, 0.10347E+01_JPRB, 0.90549E+00_JPRB ,&
&  0.77723E+00_JPRB, 0.72945E+00_JPRB, 0.76164E+00_JPRB, 0.83687E+00_JPRB, 0.77824E+00_JPRB, 0.71249E+00_JPRB, 0.67862E+00_JPRB ,&
&  0.59868E+00_JPRB, 0.55636E+00_JPRB, 0.54563E+00_JPRB, 0.52190E+00_JPRB, 0.52326E+00_JPRB, 0.54274E+00_JPRB, 0.58742E+00_JPRB ,&
&  0.60912E+00_JPRB, 0.63237E+00_JPRB, 0.66050E+00_JPRB, 0.67068E+00_JPRB, 0.70415E+00_JPRB, 0.67424E+00_JPRB, 0.80043E+00_JPRB ,&
&  0.80156E+00_JPRB, 0.80089E+00_JPRB, 0.85269E+00_JPRB, 0.74232E+00_JPRB, 0.10032E+01_JPRB, 0.10880E+01_JPRB, 0.12403E+01_JPRB ,&
&  0.12870E+01_JPRB, 0.17474E+01_JPRB, 0.21212E+01_JPRB, 0.17742E+01_JPRB, 0.12817E+01_JPRB, 0.87044E+00_JPRB, 0.77547E+00_JPRB ,&
&  0.76972E+00_JPRB, 0.78671E+00_JPRB, 0.80428E+00_JPRB, 0.81154E+00_JPRB, 0.82008E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.82431E+00_JPRB, 0.83630E+00_JPRB, 0.84890E+00_JPRB, 0.87157E+00_JPRB, 0.89643E+00_JPRB, 0.92054E+00_JPRB, 0.93915E+00_JPRB ,&
&  0.95794E+00_JPRB, 0.97155E+00_JPRB, 0.98942E+00_JPRB, 0.10058E+01_JPRB, 0.10176E+01_JPRB, 0.10233E+01_JPRB, 0.10229E+01_JPRB ,&
&  0.10252E+01_JPRB, 0.10277E+01_JPRB, 0.10382E+01_JPRB, 0.10448E+01_JPRB, 0.10385E+01_JPRB, 0.10435E+01_JPRB, 0.10579E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10779E+01_JPRB, 0.11053E+01_JPRB, 0.11457E+01_JPRB, 0.11997E+01_JPRB, 0.13036E+01_JPRB, 0.11407E+01_JPRB ,&
&  0.11978E+01_JPRB, 0.12959E+01_JPRB, 0.12786E+01_JPRB, 0.99991E+00_JPRB, 0.88867E+00_JPRB, 0.84861E+00_JPRB, 0.82437E+00_JPRB ,&
&  0.80937E+00_JPRB, 0.79479E+00_JPRB, 0.79145E+00_JPRB, 0.80458E+00_JPRB, 0.82755E+00_JPRB, 0.85397E+00_JPRB, 0.86653E+00_JPRB ,&
&  0.88388E+00_JPRB, 0.90237E+00_JPRB, 0.92084E+00_JPRB, 0.93784E+00_JPRB, 0.96996E+00_JPRB, 0.98348E+00_JPRB, 0.97817E+00_JPRB ,&
&  0.95721E+00_JPRB, 0.94505E+00_JPRB, 0.84430E+00_JPRB, 0.80067E+00_JPRB, 0.83104E+00_JPRB, 0.77550E+00_JPRB, 0.71677E+00_JPRB ,&
&  0.73672E+00_JPRB, 0.71897E+00_JPRB, 0.70851E+00_JPRB, 0.85931E+00_JPRB, 0.83405E+00_JPRB, 0.72254E+00_JPRB, 0.70131E+00_JPRB ,&
&  0.62851E+00_JPRB, 0.55983E+00_JPRB, 0.52909E+00_JPRB, 0.52769E+00_JPRB, 0.53425E+00_JPRB, 0.56135E+00_JPRB, 0.56216E+00_JPRB ,&
&  0.57113E+00_JPRB, 0.58608E+00_JPRB, 0.58877E+00_JPRB, 0.58355E+00_JPRB, 0.63042E+00_JPRB, 0.55001E+00_JPRB, 0.70823E+00_JPRB ,&
&  0.67581E+00_JPRB, 0.57424E+00_JPRB, 0.67201E+00_JPRB, 0.83992E+00_JPRB, 0.90377E+00_JPRB, 0.98955E+00_JPRB, 0.11466E+01_JPRB ,&
&  0.13232E+01_JPRB, 0.14541E+01_JPRB, 0.13046E+01_JPRB, 0.10983E+01_JPRB, 0.88553E+00_JPRB, 0.79448E+00_JPRB, 0.75748E+00_JPRB ,&
&  0.75127E+00_JPRB, 0.77398E+00_JPRB, 0.79126E+00_JPRB, 0.79939E+00_JPRB, 0.81135E+00_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.78789E+00_JPRB, 0.80210E+00_JPRB, 0.81829E+00_JPRB, 0.83833E+00_JPRB, 0.85704E+00_JPRB, 0.87373E+00_JPRB, 0.89301E+00_JPRB ,&
&  0.91843E+00_JPRB, 0.93792E+00_JPRB, 0.95800E+00_JPRB, 0.97049E+00_JPRB, 0.97693E+00_JPRB, 0.98952E+00_JPRB, 0.99345E+00_JPRB ,&
&  0.99486E+00_JPRB, 0.99583E+00_JPRB, 0.10107E+01_JPRB, 0.10207E+01_JPRB, 0.10187E+01_JPRB, 0.10148E+01_JPRB, 0.10274E+01_JPRB ,&
&  0.10269E+01_JPRB, 0.10532E+01_JPRB, 0.10765E+01_JPRB, 0.11058E+01_JPRB, 0.11302E+01_JPRB, 0.11379E+01_JPRB, 0.12094E+01_JPRB ,&
&  0.84977E+00_JPRB, 0.11759E+01_JPRB, 0.11737E+01_JPRB, 0.91990E+00_JPRB, 0.84845E+00_JPRB, 0.79086E+00_JPRB, 0.76309E+00_JPRB ,&
&  0.74738E+00_JPRB, 0.73815E+00_JPRB, 0.74206E+00_JPRB, 0.75903E+00_JPRB, 0.77872E+00_JPRB, 0.80315E+00_JPRB, 0.81639E+00_JPRB ,&
&  0.82544E+00_JPRB, 0.83257E+00_JPRB, 0.83551E+00_JPRB, 0.83369E+00_JPRB, 0.83439E+00_JPRB, 0.82356E+00_JPRB, 0.81145E+00_JPRB ,&
&  0.78325E+00_JPRB, 0.76298E+00_JPRB, 0.57261E+00_JPRB, 0.54193E+00_JPRB, 0.57652E+00_JPRB, 0.57685E+00_JPRB, 0.53669E+00_JPRB ,&
&  0.62849E+00_JPRB, 0.72831E+00_JPRB, 0.79754E+00_JPRB, 0.84845E+00_JPRB, 0.83408E+00_JPRB, 0.76213E+00_JPRB, 0.74146E+00_JPRB ,&
&  0.70368E+00_JPRB, 0.63789E+00_JPRB, 0.60748E+00_JPRB, 0.58464E+00_JPRB, 0.58515E+00_JPRB, 0.57458E+00_JPRB, 0.56059E+00_JPRB ,&
&  0.53544E+00_JPRB, 0.52036E+00_JPRB, 0.50325E+00_JPRB, 0.47357E+00_JPRB, 0.40502E+00_JPRB, 0.59995E+00_JPRB, 0.66698E+00_JPRB ,&
&  0.60478E+00_JPRB, 0.54861E+00_JPRB, 0.51377E+00_JPRB, 0.69789E+00_JPRB, 0.73004E+00_JPRB, 0.77356E+00_JPRB, 0.89230E+00_JPRB ,&
&  0.98106E+00_JPRB, 0.98736E+00_JPRB, 0.88993E+00_JPRB, 0.77802E+00_JPRB, 0.73511E+00_JPRB, 0.71905E+00_JPRB, 0.69670E+00_JPRB ,&
&  0.69680E+00_JPRB, 0.71958E+00_JPRB, 0.74067E+00_JPRB, 0.75603E+00_JPRB, 0.77208E+00_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.75353E+00_JPRB, 0.74382E+00_JPRB, 0.75283E+00_JPRB, 0.76756E+00_JPRB, 0.78100E+00_JPRB, 0.79514E+00_JPRB, 0.81933E+00_JPRB ,&
&  0.84966E+00_JPRB, 0.87448E+00_JPRB, 0.89957E+00_JPRB, 0.91729E+00_JPRB, 0.92329E+00_JPRB, 0.93667E+00_JPRB, 0.94774E+00_JPRB ,&
&  0.96062E+00_JPRB, 0.95646E+00_JPRB, 0.97297E+00_JPRB, 0.10017E+01_JPRB, 0.10342E+01_JPRB, 0.10462E+01_JPRB, 0.10555E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.11054E+01_JPRB, 0.11499E+01_JPRB, 0.12028E+01_JPRB, 0.12931E+01_JPRB, 0.14523E+01_JPRB, 0.16189E+01_JPRB ,&
&  0.10645E+01_JPRB, 0.96491E+00_JPRB, 0.98631E+00_JPRB, 0.80936E+00_JPRB, 0.73513E+00_JPRB, 0.74045E+00_JPRB, 0.70648E+00_JPRB ,&
&  0.67632E+00_JPRB, 0.68283E+00_JPRB, 0.68318E+00_JPRB, 0.68547E+00_JPRB, 0.69873E+00_JPRB, 0.72970E+00_JPRB, 0.73756E+00_JPRB ,&
&  0.75013E+00_JPRB, 0.75370E+00_JPRB, 0.74362E+00_JPRB, 0.73555E+00_JPRB, 0.66231E+00_JPRB, 0.72274E+00_JPRB, 0.70656E+00_JPRB ,&
&  0.69740E+00_JPRB, 0.62845E+00_JPRB, 0.44961E+00_JPRB, 0.42754E+00_JPRB, 0.45943E+00_JPRB, 0.44704E+00_JPRB, 0.44385E+00_JPRB ,&
&  0.57142E+00_JPRB, 0.66802E+00_JPRB, 0.59741E+00_JPRB, 0.76353E+00_JPRB, 0.82894E+00_JPRB, 0.84743E+00_JPRB, 0.84252E+00_JPRB ,&
&  0.82361E+00_JPRB, 0.77012E+00_JPRB, 0.72497E+00_JPRB, 0.67985E+00_JPRB, 0.65568E+00_JPRB, 0.62517E+00_JPRB, 0.58568E+00_JPRB ,&
&  0.54822E+00_JPRB, 0.51524E+00_JPRB, 0.46686E+00_JPRB, 0.43073E+00_JPRB, 0.47360E+00_JPRB, 0.50304E+00_JPRB, 0.63748E+00_JPRB ,&
&  0.55753E+00_JPRB, 0.55810E+00_JPRB, 0.47003E+00_JPRB, 0.60025E+00_JPRB, 0.58362E+00_JPRB, 0.56960E+00_JPRB, 0.64371E+00_JPRB ,&
&  0.68891E+00_JPRB, 0.72045E+00_JPRB, 0.70296E+00_JPRB, 0.67157E+00_JPRB, 0.65851E+00_JPRB, 0.65903E+00_JPRB, 0.63945E+00_JPRB ,&
&  0.63312E+00_JPRB, 0.65148E+00_JPRB, 0.68480E+00_JPRB, 0.69562E+00_JPRB, 0.72547E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.66890E+00_JPRB, 0.67222E+00_JPRB, 0.67607E+00_JPRB, 0.68342E+00_JPRB, 0.69879E+00_JPRB, 0.71530E+00_JPRB, 0.72479E+00_JPRB ,&
&  0.74689E+00_JPRB, 0.76541E+00_JPRB, 0.78617E+00_JPRB, 0.79224E+00_JPRB, 0.80772E+00_JPRB, 0.82413E+00_JPRB, 0.82999E+00_JPRB ,&
&  0.84990E+00_JPRB, 0.87315E+00_JPRB, 0.87478E+00_JPRB, 0.88433E+00_JPRB, 0.91295E+00_JPRB, 0.93187E+00_JPRB, 0.91471E+00_JPRB ,&
&  0.89230E+00_JPRB, 0.86406E+00_JPRB, 0.88152E+00_JPRB, 0.91780E+00_JPRB, 0.10938E+01_JPRB, 0.11882E+01_JPRB, 0.11616E+01_JPRB ,&
&  0.89143E+00_JPRB, 0.83196E+00_JPRB, 0.97750E+00_JPRB, 0.81258E+00_JPRB, 0.73313E+00_JPRB, 0.67978E+00_JPRB, 0.62733E+00_JPRB ,&
&  0.63400E+00_JPRB, 0.64966E+00_JPRB, 0.65553E+00_JPRB, 0.66458E+00_JPRB, 0.68701E+00_JPRB, 0.69912E+00_JPRB, 0.72638E+00_JPRB ,&
&  0.72898E+00_JPRB, 0.73785E+00_JPRB, 0.72467E+00_JPRB, 0.66760E+00_JPRB, 0.62276E+00_JPRB, 0.67371E+00_JPRB, 0.66388E+00_JPRB ,&
&  0.66180E+00_JPRB, 0.66059E+00_JPRB, 0.57125E+00_JPRB, 0.53869E+00_JPRB, 0.52451E+00_JPRB, 0.46989E+00_JPRB, 0.44635E+00_JPRB ,&
&  0.49040E+00_JPRB, 0.51530E+00_JPRB, 0.41673E+00_JPRB, 0.61064E+00_JPRB, 0.76942E+00_JPRB, 0.85459E+00_JPRB, 0.92896E+00_JPRB ,&
&  0.94097E+00_JPRB, 0.89046E+00_JPRB, 0.81526E+00_JPRB, 0.79575E+00_JPRB, 0.77722E+00_JPRB, 0.68583E+00_JPRB, 0.64466E+00_JPRB ,&
&  0.57444E+00_JPRB, 0.52478E+00_JPRB, 0.47891E+00_JPRB, 0.45307E+00_JPRB, 0.42130E+00_JPRB, 0.47267E+00_JPRB, 0.51999E+00_JPRB ,&
&  0.50074E+00_JPRB, 0.49201E+00_JPRB, 0.53023E+00_JPRB, 0.56930E+00_JPRB, 0.44098E+00_JPRB, 0.46152E+00_JPRB, 0.50922E+00_JPRB ,&
&  0.52962E+00_JPRB, 0.53268E+00_JPRB, 0.53572E+00_JPRB, 0.53833E+00_JPRB, 0.54374E+00_JPRB, 0.56178E+00_JPRB, 0.55851E+00_JPRB ,&
&  0.55660E+00_JPRB, 0.55683E+00_JPRB, 0.57578E+00_JPRB, 0.59944E+00_JPRB, 0.63960E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.53046E+00_JPRB, 0.53132E+00_JPRB, 0.51971E+00_JPRB, 0.51168E+00_JPRB, 0.51084E+00_JPRB, 0.51495E+00_JPRB, 0.52136E+00_JPRB ,&
&  0.53301E+00_JPRB, 0.55179E+00_JPRB, 0.57289E+00_JPRB, 0.60289E+00_JPRB, 0.61190E+00_JPRB, 0.58755E+00_JPRB, 0.59711E+00_JPRB ,&
&  0.59613E+00_JPRB, 0.61072E+00_JPRB, 0.60414E+00_JPRB, 0.67765E+00_JPRB, 0.70470E+00_JPRB, 0.71594E+00_JPRB, 0.80043E+00_JPRB ,&
&  0.78660E+00_JPRB, 0.86251E+00_JPRB, 0.92248E+00_JPRB, 0.10318E+01_JPRB, 0.17176E+01_JPRB, 0.15432E+01_JPRB, 0.95739E+00_JPRB ,&
&  0.79312E+00_JPRB, 0.89629E+00_JPRB, 0.88114E+00_JPRB, 0.79657E+00_JPRB, 0.75409E+00_JPRB, 0.67831E+00_JPRB, 0.65266E+00_JPRB ,&
&  0.67648E+00_JPRB, 0.70473E+00_JPRB, 0.76711E+00_JPRB, 0.82150E+00_JPRB, 0.86236E+00_JPRB, 0.90791E+00_JPRB, 0.96108E+00_JPRB ,&
&  0.99782E+00_JPRB, 0.10246E+01_JPRB, 0.97561E+00_JPRB, 0.85914E+00_JPRB, 0.88413E+00_JPRB, 0.91490E+00_JPRB, 0.92423E+00_JPRB ,&
&  0.93012E+00_JPRB, 0.93770E+00_JPRB, 0.95537E+00_JPRB, 0.88623E+00_JPRB, 0.79102E+00_JPRB, 0.72954E+00_JPRB, 0.62662E+00_JPRB ,&
&  0.56008E+00_JPRB, 0.50598E+00_JPRB, 0.44758E+00_JPRB, 0.63425E+00_JPRB, 0.79122E+00_JPRB, 0.80353E+00_JPRB, 0.96809E+00_JPRB ,&
&  0.10280E+01_JPRB, 0.10489E+01_JPRB, 0.98893E+00_JPRB, 0.90662E+00_JPRB, 0.80990E+00_JPRB, 0.76708E+00_JPRB, 0.66940E+00_JPRB ,&
&  0.62728E+00_JPRB, 0.55966E+00_JPRB, 0.50899E+00_JPRB, 0.47205E+00_JPRB, 0.43214E+00_JPRB, 0.41310E+00_JPRB, 0.48782E+00_JPRB ,&
&  0.49559E+00_JPRB, 0.45768E+00_JPRB, 0.50319E+00_JPRB, 0.68713E+00_JPRB, 0.77023E+00_JPRB, 0.51089E+00_JPRB, 0.43800E+00_JPRB ,&
&  0.43561E+00_JPRB, 0.44330E+00_JPRB, 0.44816E+00_JPRB, 0.45401E+00_JPRB, 0.46944E+00_JPRB, 0.48234E+00_JPRB, 0.47716E+00_JPRB ,&
&  0.48181E+00_JPRB, 0.47949E+00_JPRB, 0.47824E+00_JPRB, 0.48581E+00_JPRB, 0.50246E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.45653E+00_JPRB, 0.46072E+00_JPRB, 0.47198E+00_JPRB, 0.50908E+00_JPRB, 0.53048E+00_JPRB, 0.53981E+00_JPRB, 0.56533E+00_JPRB ,&
&  0.59391E+00_JPRB, 0.64651E+00_JPRB, 0.68444E+00_JPRB, 0.70064E+00_JPRB, 0.68038E+00_JPRB, 0.65414E+00_JPRB, 0.63881E+00_JPRB ,&
&  0.63318E+00_JPRB, 0.59215E+00_JPRB, 0.56747E+00_JPRB, 0.59107E+00_JPRB, 0.65158E+00_JPRB, 0.75627E+00_JPRB, 0.87388E+00_JPRB ,&
&  0.10188E+01_JPRB, 0.11844E+01_JPRB, 0.14058E+01_JPRB, 0.19642E+01_JPRB, 0.21849E+01_JPRB, 0.93259E+00_JPRB, 0.84204E+00_JPRB ,&
&  0.96370E+00_JPRB, 0.10716E+01_JPRB, 0.98429E+00_JPRB, 0.86645E+00_JPRB, 0.80022E+00_JPRB, 0.76495E+00_JPRB, 0.76379E+00_JPRB ,&
&  0.80955E+00_JPRB, 0.84865E+00_JPRB, 0.90257E+00_JPRB, 0.97299E+00_JPRB, 0.10512E+01_JPRB, 0.11281E+01_JPRB, 0.12228E+01_JPRB ,&
&  0.12990E+01_JPRB, 0.13085E+01_JPRB, 0.12862E+01_JPRB, 0.12534E+01_JPRB, 0.12687E+01_JPRB, 0.13057E+01_JPRB, 0.13328E+01_JPRB ,&
&  0.13315E+01_JPRB, 0.13122E+01_JPRB, 0.13046E+01_JPRB, 0.12632E+01_JPRB, 0.11154E+01_JPRB, 0.10008E+01_JPRB, 0.85716E+00_JPRB ,&
&  0.76826E+00_JPRB, 0.70193E+00_JPRB, 0.64846E+00_JPRB, 0.89738E+00_JPRB, 0.10246E+01_JPRB, 0.10281E+01_JPRB, 0.10497E+01_JPRB ,&
&  0.11344E+01_JPRB, 0.11236E+01_JPRB, 0.10652E+01_JPRB, 0.90784E+00_JPRB, 0.79294E+00_JPRB, 0.80743E+00_JPRB, 0.66154E+00_JPRB ,&
&  0.67459E+00_JPRB, 0.61662E+00_JPRB, 0.56154E+00_JPRB, 0.50729E+00_JPRB, 0.45394E+00_JPRB, 0.38389E+00_JPRB, 0.41916E+00_JPRB ,&
&  0.40275E+00_JPRB, 0.47147E+00_JPRB, 0.50969E+00_JPRB, 0.80447E+00_JPRB, 0.17511E+01_JPRB, 0.88589E+00_JPRB, 0.54763E+00_JPRB ,&
&  0.42274E+00_JPRB, 0.41314E+00_JPRB, 0.41684E+00_JPRB, 0.42652E+00_JPRB, 0.44043E+00_JPRB, 0.45148E+00_JPRB, 0.45479E+00_JPRB ,&
&  0.46686E+00_JPRB, 0.46815E+00_JPRB, 0.47466E+00_JPRB, 0.46768E+00_JPRB, 0.44825E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.57165E+00_JPRB, 0.60117E+00_JPRB, 0.64259E+00_JPRB, 0.68821E+00_JPRB, 0.71251E+00_JPRB, 0.72910E+00_JPRB, 0.72179E+00_JPRB ,&
&  0.73660E+00_JPRB, 0.75155E+00_JPRB, 0.78164E+00_JPRB, 0.81115E+00_JPRB, 0.82603E+00_JPRB, 0.86215E+00_JPRB, 0.88701E+00_JPRB ,&
&  0.88729E+00_JPRB, 0.87298E+00_JPRB, 0.90630E+00_JPRB, 0.96438E+00_JPRB, 0.10990E+01_JPRB, 0.13123E+01_JPRB, 0.15228E+01_JPRB ,&
&  0.17120E+01_JPRB, 0.17607E+01_JPRB, 0.17510E+01_JPRB, 0.11573E+01_JPRB, 0.76251E+00_JPRB, 0.82760E+00_JPRB, 0.89927E+00_JPRB ,&
&  0.95314E+00_JPRB, 0.97029E+00_JPRB, 0.92118E+00_JPRB, 0.85993E+00_JPRB, 0.81706E+00_JPRB, 0.81430E+00_JPRB, 0.85867E+00_JPRB ,&
&  0.92968E+00_JPRB, 0.10064E+01_JPRB, 0.10801E+01_JPRB, 0.11286E+01_JPRB, 0.11884E+01_JPRB, 0.12821E+01_JPRB, 0.13986E+01_JPRB ,&
&  0.14953E+01_JPRB, 0.15428E+01_JPRB, 0.15424E+01_JPRB, 0.15646E+01_JPRB, 0.15655E+01_JPRB, 0.15929E+01_JPRB, 0.16048E+01_JPRB ,&
&  0.16044E+01_JPRB, 0.15820E+01_JPRB, 0.15378E+01_JPRB, 0.15089E+01_JPRB, 0.14272E+01_JPRB, 0.12428E+01_JPRB, 0.10821E+01_JPRB ,&
&  0.10061E+01_JPRB, 0.94138E+00_JPRB, 0.95401E+00_JPRB, 0.10820E+01_JPRB, 0.82858E+00_JPRB, 0.94284E+00_JPRB, 0.11935E+01_JPRB ,&
&  0.11913E+01_JPRB, 0.11565E+01_JPRB, 0.10709E+01_JPRB, 0.95883E+00_JPRB, 0.91424E+00_JPRB, 0.76629E+00_JPRB, 0.68004E+00_JPRB ,&
&  0.70740E+00_JPRB, 0.66616E+00_JPRB, 0.59766E+00_JPRB, 0.49725E+00_JPRB, 0.45577E+00_JPRB, 0.39267E+00_JPRB, 0.39139E+00_JPRB ,&
&  0.45331E+00_JPRB, 0.47853E+00_JPRB, 0.46020E+00_JPRB, 0.56866E+00_JPRB, 0.84146E+00_JPRB, 0.60090E+00_JPRB, 0.48472E+00_JPRB ,&
&  0.41631E+00_JPRB, 0.40720E+00_JPRB, 0.40958E+00_JPRB, 0.40424E+00_JPRB, 0.41724E+00_JPRB, 0.43435E+00_JPRB, 0.43737E+00_JPRB ,&
&  0.47326E+00_JPRB, 0.50022E+00_JPRB, 0.52512E+00_JPRB, 0.54981E+00_JPRB, 0.55158E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.69095E+00_JPRB, 0.78440E+00_JPRB, 0.85509E+00_JPRB, 0.92032E+00_JPRB, 0.95253E+00_JPRB, 0.10016E+01_JPRB, 0.98249E+00_JPRB ,&
&  0.90169E+00_JPRB, 0.84089E+00_JPRB, 0.89666E+00_JPRB, 0.97244E+00_JPRB, 0.10416E+01_JPRB, 0.10901E+01_JPRB, 0.11311E+01_JPRB ,&
&  0.11742E+01_JPRB, 0.12112E+01_JPRB, 0.12443E+01_JPRB, 0.12559E+01_JPRB, 0.12870E+01_JPRB, 0.13302E+01_JPRB, 0.13058E+01_JPRB ,&
&  0.10586E+01_JPRB, 0.91004E+00_JPRB, 0.11462E+01_JPRB, 0.12300E+01_JPRB, 0.93664E+00_JPRB, 0.93202E+00_JPRB, 0.97802E+00_JPRB ,&
&  0.99361E+00_JPRB, 0.96938E+00_JPRB, 0.97483E+00_JPRB, 0.96764E+00_JPRB, 0.83826E+00_JPRB, 0.87680E+00_JPRB, 0.97793E+00_JPRB ,&
&  0.10820E+01_JPRB, 0.11736E+01_JPRB, 0.12418E+01_JPRB, 0.12849E+01_JPRB, 0.13352E+01_JPRB, 0.14222E+01_JPRB, 0.15146E+01_JPRB ,&
&  0.16035E+01_JPRB, 0.16614E+01_JPRB, 0.16834E+01_JPRB, 0.17094E+01_JPRB, 0.17576E+01_JPRB, 0.18247E+01_JPRB, 0.18488E+01_JPRB ,&
&  0.18642E+01_JPRB, 0.18758E+01_JPRB, 0.18722E+01_JPRB, 0.18625E+01_JPRB, 0.18061E+01_JPRB, 0.15339E+01_JPRB, 0.13579E+01_JPRB ,&
&  0.12521E+01_JPRB, 0.11253E+01_JPRB, 0.11404E+01_JPRB, 0.12260E+01_JPRB, 0.97006E+00_JPRB, 0.10263E+01_JPRB, 0.11182E+01_JPRB ,&
&  0.12911E+01_JPRB, 0.12199E+01_JPRB, 0.11658E+01_JPRB, 0.11586E+01_JPRB, 0.11187E+01_JPRB, 0.99156E+00_JPRB, 0.83260E+00_JPRB ,&
&  0.68328E+00_JPRB, 0.64529E+00_JPRB, 0.57438E+00_JPRB, 0.51629E+00_JPRB, 0.39713E+00_JPRB, 0.38734E+00_JPRB, 0.43577E+00_JPRB ,&
&  0.55797E+00_JPRB, 0.46937E+00_JPRB, 0.45745E+00_JPRB, 0.50957E+00_JPRB, 0.56350E+00_JPRB, 0.46206E+00_JPRB, 0.42828E+00_JPRB ,&
&  0.41168E+00_JPRB, 0.42093E+00_JPRB, 0.42360E+00_JPRB, 0.41637E+00_JPRB, 0.41542E+00_JPRB, 0.43217E+00_JPRB, 0.44927E+00_JPRB ,&
&  0.48064E+00_JPRB, 0.51815E+00_JPRB, 0.55094E+00_JPRB, 0.59298E+00_JPRB, 0.64817E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.69031E+00_JPRB, 0.76554E+00_JPRB, 0.88002E+00_JPRB, 0.10262E+01_JPRB, 0.12164E+01_JPRB, 0.13932E+01_JPRB, 0.14306E+01_JPRB ,&
&  0.11730E+01_JPRB, 0.92732E+00_JPRB, 0.10073E+01_JPRB, 0.10909E+01_JPRB, 0.11640E+01_JPRB, 0.12391E+01_JPRB, 0.13008E+01_JPRB ,&
&  0.13740E+01_JPRB, 0.14499E+01_JPRB, 0.15199E+01_JPRB, 0.15679E+01_JPRB, 0.15713E+01_JPRB, 0.15326E+01_JPRB, 0.13335E+01_JPRB ,&
&  0.95656E+00_JPRB, 0.10449E+01_JPRB, 0.14803E+01_JPRB, 0.12245E+01_JPRB, 0.10338E+01_JPRB, 0.10511E+01_JPRB, 0.10888E+01_JPRB ,&
&  0.10847E+01_JPRB, 0.10262E+01_JPRB, 0.10440E+01_JPRB, 0.10188E+01_JPRB, 0.93233E+00_JPRB, 0.99946E+00_JPRB, 0.11323E+01_JPRB ,&
&  0.12101E+01_JPRB, 0.12946E+01_JPRB, 0.13771E+01_JPRB, 0.14395E+01_JPRB, 0.14640E+01_JPRB, 0.15195E+01_JPRB, 0.15680E+01_JPRB ,&
&  0.16069E+01_JPRB, 0.16542E+01_JPRB, 0.16814E+01_JPRB, 0.17274E+01_JPRB, 0.18570E+01_JPRB, 0.20172E+01_JPRB, 0.21452E+01_JPRB ,&
&  0.22193E+01_JPRB, 0.24073E+01_JPRB, 0.24394E+01_JPRB, 0.22908E+01_JPRB, 0.20109E+01_JPRB, 0.19845E+01_JPRB, 0.16211E+01_JPRB ,&
&  0.15098E+01_JPRB, 0.13039E+01_JPRB, 0.13423E+01_JPRB, 0.11623E+01_JPRB, 0.11757E+01_JPRB, 0.12423E+01_JPRB, 0.13143E+01_JPRB ,&
&  0.13438E+01_JPRB, 0.13436E+01_JPRB, 0.12723E+01_JPRB, 0.12494E+01_JPRB, 0.13225E+01_JPRB, 0.12072E+01_JPRB, 0.11118E+01_JPRB ,&
&  0.74432E+00_JPRB, 0.68707E+00_JPRB, 0.55253E+00_JPRB, 0.46290E+00_JPRB, 0.36526E+00_JPRB, 0.38385E+00_JPRB, 0.50049E+00_JPRB ,&
&  0.63460E+00_JPRB, 0.60788E+00_JPRB, 0.60221E+00_JPRB, 0.60238E+00_JPRB, 0.52910E+00_JPRB, 0.45713E+00_JPRB, 0.41621E+00_JPRB ,&
&  0.39882E+00_JPRB, 0.41273E+00_JPRB, 0.43364E+00_JPRB, 0.43699E+00_JPRB, 0.43093E+00_JPRB, 0.44372E+00_JPRB, 0.46354E+00_JPRB ,&
&  0.49137E+00_JPRB, 0.52228E+00_JPRB, 0.56463E+00_JPRB, 0.58802E+00_JPRB, 0.63794E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.59613E+00_JPRB, 0.65513E+00_JPRB, 0.72953E+00_JPRB, 0.83165E+00_JPRB, 0.93174E+00_JPRB, 0.99662E+00_JPRB, 0.93809E+00_JPRB ,&
&  0.88530E+00_JPRB, 0.91798E+00_JPRB, 0.97603E+00_JPRB, 0.10490E+01_JPRB, 0.11274E+01_JPRB, 0.12122E+01_JPRB, 0.12885E+01_JPRB ,&
&  0.13631E+01_JPRB, 0.14576E+01_JPRB, 0.15678E+01_JPRB, 0.16658E+01_JPRB, 0.16868E+01_JPRB, 0.15891E+01_JPRB, 0.11624E+01_JPRB ,&
&  0.95764E+00_JPRB, 0.17120E+01_JPRB, 0.14453E+01_JPRB, 0.13003E+01_JPRB, 0.11394E+01_JPRB, 0.11525E+01_JPRB, 0.11082E+01_JPRB ,&
&  0.10521E+01_JPRB, 0.10610E+01_JPRB, 0.11022E+01_JPRB, 0.10988E+01_JPRB, 0.11161E+01_JPRB, 0.11789E+01_JPRB, 0.12487E+01_JPRB ,&
&  0.12845E+01_JPRB, 0.13555E+01_JPRB, 0.14406E+01_JPRB, 0.15187E+01_JPRB, 0.15366E+01_JPRB, 0.15515E+01_JPRB, 0.15615E+01_JPRB ,&
&  0.15688E+01_JPRB, 0.15779E+01_JPRB, 0.15922E+01_JPRB, 0.17351E+01_JPRB, 0.19005E+01_JPRB, 0.21230E+01_JPRB, 0.23936E+01_JPRB ,&
&  0.25350E+01_JPRB, 0.26598E+01_JPRB, 0.29702E+01_JPRB, 0.29192E+01_JPRB, 0.27150E+01_JPRB, 0.25564E+01_JPRB, 0.19146E+01_JPRB ,&
&  0.18005E+01_JPRB, 0.14818E+01_JPRB, 0.14838E+01_JPRB, 0.12717E+01_JPRB, 0.12790E+01_JPRB, 0.13814E+01_JPRB, 0.14441E+01_JPRB ,&
&  0.13909E+01_JPRB, 0.14701E+01_JPRB, 0.14069E+01_JPRB, 0.13504E+01_JPRB, 0.13362E+01_JPRB, 0.13199E+01_JPRB, 0.12930E+01_JPRB ,&
&  0.10412E+01_JPRB, 0.70135E+00_JPRB, 0.57872E+00_JPRB, 0.44795E+00_JPRB, 0.37979E+00_JPRB, 0.35492E+00_JPRB, 0.47437E+00_JPRB ,&
&  0.65530E+00_JPRB, 0.80107E+00_JPRB, 0.94316E+00_JPRB, 0.83152E+00_JPRB, 0.70309E+00_JPRB, 0.62136E+00_JPRB, 0.52312E+00_JPRB ,&
&  0.42593E+00_JPRB, 0.42562E+00_JPRB, 0.46035E+00_JPRB, 0.47070E+00_JPRB, 0.46265E+00_JPRB, 0.47470E+00_JPRB, 0.47980E+00_JPRB ,&
&  0.47859E+00_JPRB, 0.48809E+00_JPRB, 0.51672E+00_JPRB, 0.52306E+00_JPRB, 0.55181E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.51919E+00_JPRB, 0.55461E+00_JPRB, 0.62472E+00_JPRB, 0.68788E+00_JPRB, 0.72240E+00_JPRB, 0.73714E+00_JPRB, 0.74407E+00_JPRB ,&
&  0.78514E+00_JPRB, 0.84159E+00_JPRB, 0.88366E+00_JPRB, 0.91968E+00_JPRB, 0.97663E+00_JPRB, 0.10685E+01_JPRB, 0.11818E+01_JPRB ,&
&  0.13118E+01_JPRB, 0.14697E+01_JPRB, 0.16664E+01_JPRB, 0.18645E+01_JPRB, 0.20088E+01_JPRB, 0.19413E+01_JPRB, 0.12432E+01_JPRB ,&
&  0.12077E+01_JPRB, 0.17006E+01_JPRB, 0.16733E+01_JPRB, 0.15857E+01_JPRB, 0.13656E+01_JPRB, 0.12735E+01_JPRB, 0.11584E+01_JPRB ,&
&  0.11617E+01_JPRB, 0.11636E+01_JPRB, 0.11999E+01_JPRB, 0.12457E+01_JPRB, 0.12723E+01_JPRB, 0.12886E+01_JPRB, 0.12640E+01_JPRB ,&
&  0.12382E+01_JPRB, 0.12557E+01_JPRB, 0.12928E+01_JPRB, 0.13887E+01_JPRB, 0.14872E+01_JPRB, 0.15510E+01_JPRB, 0.15765E+01_JPRB ,&
&  0.15703E+01_JPRB, 0.15499E+01_JPRB, 0.15555E+01_JPRB, 0.15942E+01_JPRB, 0.19007E+01_JPRB, 0.21350E+01_JPRB, 0.24610E+01_JPRB ,&
&  0.27139E+01_JPRB, 0.28796E+01_JPRB, 0.34772E+01_JPRB, 0.38654E+01_JPRB, 0.38735E+01_JPRB, 0.32376E+01_JPRB, 0.24611E+01_JPRB ,&
&  0.20729E+01_JPRB, 0.17365E+01_JPRB, 0.15277E+01_JPRB, 0.14038E+01_JPRB, 0.14099E+01_JPRB, 0.14850E+01_JPRB, 0.16560E+01_JPRB ,&
&  0.15957E+01_JPRB, 0.15634E+01_JPRB, 0.15246E+01_JPRB, 0.13743E+01_JPRB, 0.13317E+01_JPRB, 0.13011E+01_JPRB, 0.14613E+01_JPRB ,&
&  0.14771E+01_JPRB, 0.11288E+01_JPRB, 0.74604E+00_JPRB, 0.48720E+00_JPRB, 0.39140E+00_JPRB, 0.31842E+00_JPRB, 0.37891E+00_JPRB ,&
&  0.52727E+00_JPRB, 0.71326E+00_JPRB, 0.94687E+00_JPRB, 0.11393E+01_JPRB, 0.97301E+00_JPRB, 0.91144E+00_JPRB, 0.83557E+00_JPRB ,&
&  0.56557E+00_JPRB, 0.49677E+00_JPRB, 0.50178E+00_JPRB, 0.50608E+00_JPRB, 0.49073E+00_JPRB, 0.50917E+00_JPRB, 0.52797E+00_JPRB ,&
&  0.50717E+00_JPRB, 0.45951E+00_JPRB, 0.46094E+00_JPRB, 0.46733E+00_JPRB, 0.48669E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.46664E+00_JPRB, 0.47373E+00_JPRB, 0.51859E+00_JPRB, 0.55536E+00_JPRB, 0.58849E+00_JPRB, 0.61644E+00_JPRB, 0.65719E+00_JPRB ,&
&  0.71576E+00_JPRB, 0.75928E+00_JPRB, 0.78894E+00_JPRB, 0.79157E+00_JPRB, 0.79451E+00_JPRB, 0.83864E+00_JPRB, 0.92130E+00_JPRB ,&
&  0.10592E+01_JPRB, 0.12783E+01_JPRB, 0.15858E+01_JPRB, 0.18462E+01_JPRB, 0.19586E+01_JPRB, 0.17620E+01_JPRB, 0.16726E+01_JPRB ,&
&  0.17271E+01_JPRB, 0.20517E+01_JPRB, 0.21482E+01_JPRB, 0.22382E+01_JPRB, 0.21424E+01_JPRB, 0.20202E+01_JPRB, 0.13880E+01_JPRB ,&
&  0.13662E+01_JPRB, 0.14050E+01_JPRB, 0.14304E+01_JPRB, 0.14870E+01_JPRB, 0.15039E+01_JPRB, 0.14763E+01_JPRB, 0.14179E+01_JPRB ,&
&  0.12949E+01_JPRB, 0.12054E+01_JPRB, 0.11607E+01_JPRB, 0.12003E+01_JPRB, 0.13299E+01_JPRB, 0.14548E+01_JPRB, 0.15810E+01_JPRB ,&
&  0.16627E+01_JPRB, 0.16628E+01_JPRB, 0.16303E+01_JPRB, 0.15664E+01_JPRB, 0.18038E+01_JPRB, 0.22614E+01_JPRB, 0.26539E+01_JPRB ,&
&  0.30079E+01_JPRB, 0.35445E+01_JPRB, 0.40102E+01_JPRB, 0.43265E+01_JPRB, 0.44941E+01_JPRB, 0.40631E+01_JPRB, 0.32049E+01_JPRB ,&
&  0.24971E+01_JPRB, 0.19425E+01_JPRB, 0.15546E+01_JPRB, 0.15365E+01_JPRB, 0.16432E+01_JPRB, 0.16649E+01_JPRB, 0.15496E+01_JPRB ,&
&  0.14469E+01_JPRB, 0.13550E+01_JPRB, 0.13473E+01_JPRB, 0.13386E+01_JPRB, 0.13607E+01_JPRB, 0.13492E+01_JPRB, 0.16531E+01_JPRB ,&
&  0.15205E+01_JPRB, 0.80002E+00_JPRB, 0.54533E+00_JPRB, 0.58718E+00_JPRB, 0.39773E+00_JPRB, 0.30232E+00_JPRB, 0.39405E+00_JPRB ,&
&  0.48960E+00_JPRB, 0.75483E+00_JPRB, 0.10100E+01_JPRB, 0.99462E+00_JPRB, 0.12618E+01_JPRB, 0.13180E+01_JPRB, 0.13825E+01_JPRB ,&
&  0.84543E+00_JPRB, 0.67075E+00_JPRB, 0.54986E+00_JPRB, 0.51952E+00_JPRB, 0.50261E+00_JPRB, 0.51436E+00_JPRB, 0.51466E+00_JPRB ,&
&  0.48982E+00_JPRB, 0.44822E+00_JPRB, 0.44437E+00_JPRB, 0.44819E+00_JPRB, 0.45286E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.49023E+00_JPRB, 0.46231E+00_JPRB, 0.46464E+00_JPRB, 0.47012E+00_JPRB, 0.52172E+00_JPRB, 0.57944E+00_JPRB, 0.60812E+00_JPRB ,&
&  0.66753E+00_JPRB, 0.71722E+00_JPRB, 0.72954E+00_JPRB, 0.71251E+00_JPRB, 0.69314E+00_JPRB, 0.69827E+00_JPRB, 0.73066E+00_JPRB ,&
&  0.81060E+00_JPRB, 0.98397E+00_JPRB, 0.13318E+01_JPRB, 0.16898E+01_JPRB, 0.18861E+01_JPRB, 0.18458E+01_JPRB, 0.19986E+01_JPRB ,&
&  0.20382E+01_JPRB, 0.24783E+01_JPRB, 0.31827E+01_JPRB, 0.34233E+01_JPRB, 0.27026E+01_JPRB, 0.23715E+01_JPRB, 0.17932E+01_JPRB ,&
&  0.16836E+01_JPRB, 0.17004E+01_JPRB, 0.17609E+01_JPRB, 0.18555E+01_JPRB, 0.18703E+01_JPRB, 0.17935E+01_JPRB, 0.17204E+01_JPRB ,&
&  0.15481E+01_JPRB, 0.13876E+01_JPRB, 0.12117E+01_JPRB, 0.11510E+01_JPRB, 0.12349E+01_JPRB, 0.13999E+01_JPRB, 0.15791E+01_JPRB ,&
&  0.17557E+01_JPRB, 0.18320E+01_JPRB, 0.17966E+01_JPRB, 0.18470E+01_JPRB, 0.15160E+01_JPRB, 0.22945E+01_JPRB, 0.29797E+01_JPRB ,&
&  0.34588E+01_JPRB, 0.39413E+01_JPRB, 0.40415E+01_JPRB, 0.48142E+01_JPRB, 0.51948E+01_JPRB, 0.45635E+01_JPRB, 0.36786E+01_JPRB ,&
&  0.29778E+01_JPRB, 0.22140E+01_JPRB, 0.18233E+01_JPRB, 0.18042E+01_JPRB, 0.18113E+01_JPRB, 0.17893E+01_JPRB, 0.12919E+01_JPRB ,&
&  0.12549E+01_JPRB, 0.15566E+01_JPRB, 0.15278E+01_JPRB, 0.12971E+01_JPRB, 0.13581E+01_JPRB, 0.13562E+01_JPRB, 0.12454E+01_JPRB ,&
&  0.65920E+00_JPRB, 0.46782E+00_JPRB, 0.43687E+00_JPRB, 0.40190E+00_JPRB, 0.35509E+00_JPRB, 0.33068E+00_JPRB, 0.55312E+00_JPRB ,&
&  0.53899E+00_JPRB, 0.88252E+00_JPRB, 0.11242E+01_JPRB, 0.11571E+01_JPRB, 0.14131E+01_JPRB, 0.17176E+01_JPRB, 0.23207E+01_JPRB ,&
&  0.11906E+01_JPRB, 0.88038E+00_JPRB, 0.64142E+00_JPRB, 0.54420E+00_JPRB, 0.52718E+00_JPRB, 0.51743E+00_JPRB, 0.49731E+00_JPRB ,&
&  0.48007E+00_JPRB, 0.47144E+00_JPRB, 0.46920E+00_JPRB, 0.46036E+00_JPRB, 0.48385E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.56022E+00_JPRB, 0.51588E+00_JPRB, 0.48965E+00_JPRB, 0.49637E+00_JPRB, 0.53375E+00_JPRB, 0.55623E+00_JPRB, 0.57488E+00_JPRB ,&
&  0.62502E+00_JPRB, 0.68264E+00_JPRB, 0.70281E+00_JPRB, 0.66632E+00_JPRB, 0.63933E+00_JPRB, 0.63930E+00_JPRB, 0.64472E+00_JPRB ,&
&  0.68763E+00_JPRB, 0.80382E+00_JPRB, 0.11026E+01_JPRB, 0.13326E+01_JPRB, 0.16236E+01_JPRB, 0.16782E+01_JPRB, 0.20269E+01_JPRB ,&
&  0.21995E+01_JPRB, 0.26031E+01_JPRB, 0.32161E+01_JPRB, 0.33389E+01_JPRB, 0.35004E+01_JPRB, 0.28475E+01_JPRB, 0.25020E+01_JPRB ,&
&  0.20571E+01_JPRB, 0.18952E+01_JPRB, 0.18664E+01_JPRB, 0.20254E+01_JPRB, 0.21465E+01_JPRB, 0.21253E+01_JPRB, 0.19976E+01_JPRB ,&
&  0.17144E+01_JPRB, 0.15303E+01_JPRB, 0.12811E+01_JPRB, 0.11924E+01_JPRB, 0.11535E+01_JPRB, 0.12495E+01_JPRB, 0.14608E+01_JPRB ,&
&  0.18051E+01_JPRB, 0.20628E+01_JPRB, 0.20252E+01_JPRB, 0.19886E+01_JPRB, 0.18256E+01_JPRB, 0.19958E+01_JPRB, 0.30846E+01_JPRB ,&
&  0.40232E+01_JPRB, 0.47103E+01_JPRB, 0.49112E+01_JPRB, 0.59184E+01_JPRB, 0.57141E+01_JPRB, 0.48088E+01_JPRB, 0.42120E+01_JPRB ,&
&  0.34144E+01_JPRB, 0.28257E+01_JPRB, 0.21447E+01_JPRB, 0.19892E+01_JPRB, 0.18930E+01_JPRB, 0.14571E+01_JPRB, 0.13093E+01_JPRB ,&
&  0.15358E+01_JPRB, 0.15898E+01_JPRB, 0.15151E+01_JPRB, 0.12768E+01_JPRB, 0.14030E+01_JPRB, 0.12002E+01_JPRB, 0.72287E+00_JPRB ,&
&  0.60958E+00_JPRB, 0.54847E+00_JPRB, 0.44897E+00_JPRB, 0.39807E+00_JPRB, 0.35939E+00_JPRB, 0.35432E+00_JPRB, 0.50884E+00_JPRB ,&
&  0.64498E+00_JPRB, 0.10457E+01_JPRB, 0.12582E+01_JPRB, 0.13068E+01_JPRB, 0.14315E+01_JPRB, 0.19147E+01_JPRB, 0.23468E+01_JPRB ,&
&  0.16453E+01_JPRB, 0.13005E+01_JPRB, 0.73758E+00_JPRB, 0.58180E+00_JPRB, 0.54384E+00_JPRB, 0.52684E+00_JPRB, 0.50722E+00_JPRB ,&
&  0.48887E+00_JPRB, 0.49425E+00_JPRB, 0.50138E+00_JPRB, 0.50961E+00_JPRB, 0.52541E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.58633E+00_JPRB, 0.56396E+00_JPRB, 0.53270E+00_JPRB, 0.54783E+00_JPRB, 0.56983E+00_JPRB, 0.59651E+00_JPRB, 0.59061E+00_JPRB ,&
&  0.60211E+00_JPRB, 0.63025E+00_JPRB, 0.65307E+00_JPRB, 0.63961E+00_JPRB, 0.59973E+00_JPRB, 0.60757E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.65938E+00_JPRB, 0.78922E+00_JPRB, 0.96115E+00_JPRB, 0.10474E+01_JPRB, 0.11633E+01_JPRB, 0.12764E+01_JPRB, 0.16505E+01_JPRB ,&
&  0.21673E+01_JPRB, 0.25687E+01_JPRB, 0.29889E+01_JPRB, 0.34704E+01_JPRB, 0.39296E+01_JPRB, 0.34509E+01_JPRB, 0.33717E+01_JPRB ,&
&  0.27933E+01_JPRB, 0.23186E+01_JPRB, 0.21514E+01_JPRB, 0.21480E+01_JPRB, 0.22836E+01_JPRB, 0.23263E+01_JPRB, 0.22126E+01_JPRB ,&
&  0.19110E+01_JPRB, 0.15765E+01_JPRB, 0.13431E+01_JPRB, 0.11897E+01_JPRB, 0.11181E+01_JPRB, 0.11649E+01_JPRB, 0.13824E+01_JPRB ,&
&  0.18912E+01_JPRB, 0.24054E+01_JPRB, 0.24457E+01_JPRB, 0.22233E+01_JPRB, 0.25263E+01_JPRB, 0.27567E+01_JPRB, 0.34189E+01_JPRB ,&
&  0.38836E+01_JPRB, 0.41556E+01_JPRB, 0.57870E+01_JPRB, 0.62954E+01_JPRB, 0.57117E+01_JPRB, 0.49877E+01_JPRB, 0.41681E+01_JPRB ,&
&  0.35229E+01_JPRB, 0.29692E+01_JPRB, 0.23184E+01_JPRB, 0.21162E+01_JPRB, 0.17604E+01_JPRB, 0.14200E+01_JPRB, 0.15888E+01_JPRB ,&
&  0.17477E+01_JPRB, 0.17581E+01_JPRB, 0.15777E+01_JPRB, 0.12844E+01_JPRB, 0.12164E+01_JPRB, 0.92343E+00_JPRB, 0.67222E+00_JPRB ,&
&  0.59512E+00_JPRB, 0.56094E+00_JPRB, 0.48222E+00_JPRB, 0.48503E+00_JPRB, 0.43310E+00_JPRB, 0.44002E+00_JPRB, 0.55648E+00_JPRB ,&
&  0.78142E+00_JPRB, 0.10960E+01_JPRB, 0.12156E+01_JPRB, 0.12273E+01_JPRB, 0.13168E+01_JPRB, 0.14494E+01_JPRB, 0.21131E+01_JPRB ,&
&  0.14414E+01_JPRB, 0.14730E+01_JPRB, 0.10082E+01_JPRB, 0.71218E+00_JPRB, 0.59996E+00_JPRB, 0.55394E+00_JPRB, 0.54320E+00_JPRB ,&
&  0.54209E+00_JPRB, 0.54098E+00_JPRB, 0.54570E+00_JPRB, 0.54938E+00_JPRB, 0.55912E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.58796E+00_JPRB, 0.60111E+00_JPRB, 0.59597E+00_JPRB, 0.60520E+00_JPRB, 0.61743E+00_JPRB, 0.62654E+00_JPRB, 0.61588E+00_JPRB ,&
&  0.60215E+00_JPRB, 0.61371E+00_JPRB, 0.61112E+00_JPRB, 0.59579E+00_JPRB, 0.57439E+00_JPRB, 0.58132E+00_JPRB, 0.59714E+00_JPRB ,&
&  0.66426E+00_JPRB, 0.85932E+00_JPRB, 0.72062E+00_JPRB, 0.74002E+00_JPRB, 0.86546E+00_JPRB, 0.10285E+01_JPRB, 0.12497E+01_JPRB ,&
&  0.19233E+01_JPRB, 0.25157E+01_JPRB, 0.29360E+01_JPRB, 0.37206E+01_JPRB, 0.48661E+01_JPRB, 0.46399E+01_JPRB, 0.43479E+01_JPRB ,&
&  0.36974E+01_JPRB, 0.30395E+01_JPRB, 0.27713E+01_JPRB, 0.25968E+01_JPRB, 0.25527E+01_JPRB, 0.25155E+01_JPRB, 0.23518E+01_JPRB ,&
&  0.20457E+01_JPRB, 0.16375E+01_JPRB, 0.13359E+01_JPRB, 0.12538E+01_JPRB, 0.11869E+01_JPRB, 0.12283E+01_JPRB, 0.13855E+01_JPRB ,&
&  0.19432E+01_JPRB, 0.26383E+01_JPRB, 0.29243E+01_JPRB, 0.28693E+01_JPRB, 0.25678E+01_JPRB, 0.31073E+01_JPRB, 0.51237E+01_JPRB ,&
&  0.53410E+01_JPRB, 0.57073E+01_JPRB, 0.61589E+01_JPRB, 0.71631E+01_JPRB, 0.57502E+01_JPRB, 0.43284E+01_JPRB, 0.39281E+01_JPRB ,&
&  0.28578E+01_JPRB, 0.22101E+01_JPRB, 0.20727E+01_JPRB, 0.19223E+01_JPRB, 0.17022E+01_JPRB, 0.18092E+01_JPRB, 0.24681E+01_JPRB ,&
&  0.20057E+01_JPRB, 0.21330E+01_JPRB, 0.21851E+01_JPRB, 0.19649E+01_JPRB, 0.12356E+01_JPRB, 0.10270E+01_JPRB, 0.13782E+01_JPRB ,&
&  0.14380E+01_JPRB, 0.94773E+00_JPRB, 0.63732E+00_JPRB, 0.78636E+00_JPRB, 0.59052E+00_JPRB, 0.61640E+00_JPRB, 0.75057E+00_JPRB ,&
&  0.84444E+00_JPRB, 0.98113E+00_JPRB, 0.13502E+01_JPRB, 0.13310E+01_JPRB, 0.12699E+01_JPRB, 0.12700E+01_JPRB, 0.15418E+01_JPRB ,&
&  0.15251E+01_JPRB, 0.14708E+01_JPRB, 0.12429E+01_JPRB, 0.92784E+00_JPRB, 0.74450E+00_JPRB, 0.66161E+00_JPRB, 0.62815E+00_JPRB ,&
&  0.61927E+00_JPRB, 0.63355E+00_JPRB, 0.62764E+00_JPRB, 0.60438E+00_JPRB, 0.59422E+00_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.65094E+00_JPRB, 0.66031E+00_JPRB, 0.66189E+00_JPRB, 0.66993E+00_JPRB, 0.68039E+00_JPRB, 0.64806E+00_JPRB, 0.63240E+00_JPRB ,&
&  0.60949E+00_JPRB, 0.59601E+00_JPRB, 0.59553E+00_JPRB, 0.59726E+00_JPRB, 0.58080E+00_JPRB, 0.57989E+00_JPRB, 0.59314E+00_JPRB ,&
&  0.65690E+00_JPRB, 0.71800E+00_JPRB, 0.65460E+00_JPRB, 0.65187E+00_JPRB, 0.78456E+00_JPRB, 0.89816E+00_JPRB, 0.11470E+01_JPRB ,&
&  0.17255E+01_JPRB, 0.23869E+01_JPRB, 0.30989E+01_JPRB, 0.39448E+01_JPRB, 0.46101E+01_JPRB, 0.51687E+01_JPRB, 0.53636E+01_JPRB ,&
&  0.51376E+01_JPRB, 0.42510E+01_JPRB, 0.36907E+01_JPRB, 0.32911E+01_JPRB, 0.29424E+01_JPRB, 0.26588E+01_JPRB, 0.24303E+01_JPRB ,&
&  0.22405E+01_JPRB, 0.19121E+01_JPRB, 0.15522E+01_JPRB, 0.13203E+01_JPRB, 0.12688E+01_JPRB, 0.12709E+01_JPRB, 0.13961E+01_JPRB ,&
&  0.18846E+01_JPRB, 0.25656E+01_JPRB, 0.33138E+01_JPRB, 0.36470E+01_JPRB, 0.30883E+01_JPRB, 0.33690E+01_JPRB, 0.48892E+01_JPRB ,&
&  0.58398E+01_JPRB, 0.59117E+01_JPRB, 0.67734E+01_JPRB, 0.70092E+01_JPRB, 0.59108E+01_JPRB, 0.44755E+01_JPRB, 0.43675E+01_JPRB ,&
&  0.33634E+01_JPRB, 0.25318E+01_JPRB, 0.20551E+01_JPRB, 0.16815E+01_JPRB, 0.18182E+01_JPRB, 0.25296E+01_JPRB, 0.24842E+01_JPRB ,&
&  0.24046E+01_JPRB, 0.23742E+01_JPRB, 0.22186E+01_JPRB, 0.18642E+01_JPRB, 0.15264E+01_JPRB, 0.13997E+01_JPRB, 0.17185E+01_JPRB ,&
&  0.17239E+01_JPRB, 0.16637E+01_JPRB, 0.13894E+01_JPRB, 0.11532E+01_JPRB, 0.93342E+00_JPRB, 0.98668E+00_JPRB, 0.97486E+00_JPRB ,&
&  0.97105E+00_JPRB, 0.94347E+00_JPRB, 0.12421E+01_JPRB, 0.13188E+01_JPRB, 0.11888E+01_JPRB, 0.10602E+01_JPRB, 0.12471E+01_JPRB ,&
&  0.12064E+01_JPRB, 0.13721E+01_JPRB, 0.12121E+01_JPRB, 0.99343E+00_JPRB, 0.86576E+00_JPRB, 0.80388E+00_JPRB, 0.75643E+00_JPRB ,&
&  0.75131E+00_JPRB, 0.75878E+00_JPRB, 0.74558E+00_JPRB, 0.70345E+00_JPRB, 0.67711E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.72025E+00_JPRB, 0.72453E+00_JPRB, 0.72365E+00_JPRB, 0.72368E+00_JPRB, 0.72206E+00_JPRB, 0.67883E+00_JPRB, 0.64458E+00_JPRB ,&
&  0.62006E+00_JPRB, 0.60435E+00_JPRB, 0.60739E+00_JPRB, 0.60210E+00_JPRB, 0.59101E+00_JPRB, 0.58491E+00_JPRB, 0.59745E+00_JPRB ,&
&  0.65474E+00_JPRB, 0.71158E+00_JPRB, 0.63562E+00_JPRB, 0.68890E+00_JPRB, 0.75766E+00_JPRB, 0.86977E+00_JPRB, 0.12383E+01_JPRB ,&
&  0.16308E+01_JPRB, 0.21284E+01_JPRB, 0.27916E+01_JPRB, 0.35630E+01_JPRB, 0.41736E+01_JPRB, 0.48701E+01_JPRB, 0.55744E+01_JPRB ,&
&  0.52088E+01_JPRB, 0.51080E+01_JPRB, 0.44271E+01_JPRB, 0.37402E+01_JPRB, 0.31702E+01_JPRB, 0.27375E+01_JPRB, 0.25119E+01_JPRB ,&
&  0.23323E+01_JPRB, 0.22077E+01_JPRB, 0.19501E+01_JPRB, 0.16543E+01_JPRB, 0.14847E+01_JPRB, 0.14166E+01_JPRB, 0.14397E+01_JPRB ,&
&  0.16089E+01_JPRB, 0.22925E+01_JPRB, 0.33856E+01_JPRB, 0.44786E+01_JPRB, 0.37279E+01_JPRB, 0.41769E+01_JPRB, 0.48865E+01_JPRB ,&
&  0.71853E+01_JPRB, 0.74782E+01_JPRB, 0.70246E+01_JPRB, 0.68994E+01_JPRB, 0.49546E+01_JPRB, 0.43449E+01_JPRB, 0.43245E+01_JPRB ,&
&  0.41129E+01_JPRB, 0.36549E+01_JPRB, 0.35453E+01_JPRB, 0.25296E+01_JPRB, 0.21031E+01_JPRB, 0.26185E+01_JPRB, 0.23690E+01_JPRB ,&
&  0.21936E+01_JPRB, 0.21204E+01_JPRB, 0.19926E+01_JPRB, 0.19560E+01_JPRB, 0.17272E+01_JPRB, 0.16259E+01_JPRB, 0.13902E+01_JPRB ,&
&  0.13494E+01_JPRB, 0.11738E+01_JPRB, 0.12520E+01_JPRB, 0.11305E+01_JPRB, 0.10562E+01_JPRB, 0.10335E+01_JPRB, 0.10052E+01_JPRB ,&
&  0.10498E+01_JPRB, 0.10324E+01_JPRB, 0.11736E+01_JPRB, 0.12727E+01_JPRB, 0.11694E+01_JPRB, 0.10099E+01_JPRB, 0.98244E+00_JPRB ,&
&  0.10510E+01_JPRB, 0.11534E+01_JPRB, 0.10729E+01_JPRB, 0.97061E+00_JPRB, 0.90173E+00_JPRB, 0.86402E+00_JPRB, 0.83768E+00_JPRB ,&
&  0.80571E+00_JPRB, 0.78938E+00_JPRB, 0.77753E+00_JPRB, 0.76224E+00_JPRB, 0.74396E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.71753E+00_JPRB, 0.71732E+00_JPRB, 0.75903E+00_JPRB, 0.73901E+00_JPRB, 0.74434E+00_JPRB, 0.70771E+00_JPRB, 0.67934E+00_JPRB ,&
&  0.65620E+00_JPRB, 0.64283E+00_JPRB, 0.62818E+00_JPRB, 0.61186E+00_JPRB, 0.59559E+00_JPRB, 0.58304E+00_JPRB, 0.58780E+00_JPRB ,&
&  0.61152E+00_JPRB, 0.75745E+00_JPRB, 0.68907E+00_JPRB, 0.66093E+00_JPRB, 0.70182E+00_JPRB, 0.90966E+00_JPRB, 0.12235E+01_JPRB ,&
&  0.15340E+01_JPRB, 0.18320E+01_JPRB, 0.23467E+01_JPRB, 0.28495E+01_JPRB, 0.33930E+01_JPRB, 0.39732E+01_JPRB, 0.46727E+01_JPRB ,&
&  0.51421E+01_JPRB, 0.43562E+01_JPRB, 0.40793E+01_JPRB, 0.35559E+01_JPRB, 0.31331E+01_JPRB, 0.27773E+01_JPRB, 0.25274E+01_JPRB ,&
&  0.23295E+01_JPRB, 0.21878E+01_JPRB, 0.20080E+01_JPRB, 0.17965E+01_JPRB, 0.15818E+01_JPRB, 0.14526E+01_JPRB, 0.14153E+01_JPRB ,&
&  0.14855E+01_JPRB, 0.18335E+01_JPRB, 0.27626E+01_JPRB, 0.42027E+01_JPRB, 0.61861E+01_JPRB, 0.66467E+01_JPRB, 0.64523E+01_JPRB ,&
&  0.57359E+01_JPRB, 0.56712E+01_JPRB, 0.76498E+01_JPRB, 0.61120E+01_JPRB, 0.55397E+01_JPRB, 0.51786E+01_JPRB, 0.46643E+01_JPRB ,&
&  0.43116E+01_JPRB, 0.35882E+01_JPRB, 0.32824E+01_JPRB, 0.26338E+01_JPRB, 0.26932E+01_JPRB, 0.24097E+01_JPRB, 0.21751E+01_JPRB ,&
&  0.19765E+01_JPRB, 0.19591E+01_JPRB, 0.18750E+01_JPRB, 0.18065E+01_JPRB, 0.17946E+01_JPRB, 0.18994E+01_JPRB, 0.18338E+01_JPRB ,&
&  0.16261E+01_JPRB, 0.16154E+01_JPRB, 0.13722E+01_JPRB, 0.11380E+01_JPRB, 0.96241E+00_JPRB, 0.95815E+00_JPRB, 0.97106E+00_JPRB ,&
&  0.10047E+01_JPRB, 0.10468E+01_JPRB, 0.11599E+01_JPRB, 0.11812E+01_JPRB, 0.10910E+01_JPRB, 0.99562E+00_JPRB, 0.92826E+00_JPRB ,&
&  0.92638E+00_JPRB, 0.99153E+00_JPRB, 0.10172E+01_JPRB, 0.91801E+00_JPRB, 0.91083E+00_JPRB, 0.87929E+00_JPRB, 0.84954E+00_JPRB ,&
&  0.80822E+00_JPRB, 0.76835E+00_JPRB, 0.75191E+00_JPRB, 0.74365E+00_JPRB, 0.73632E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.72404E+00_JPRB, 0.70997E+00_JPRB, 0.74780E+00_JPRB, 0.73039E+00_JPRB, 0.71866E+00_JPRB, 0.69914E+00_JPRB, 0.67939E+00_JPRB ,&
&  0.65970E+00_JPRB, 0.65059E+00_JPRB, 0.64044E+00_JPRB, 0.61694E+00_JPRB, 0.59686E+00_JPRB, 0.58424E+00_JPRB, 0.58415E+00_JPRB ,&
&  0.58800E+00_JPRB, 0.75079E+00_JPRB, 0.72487E+00_JPRB, 0.76996E+00_JPRB, 0.78868E+00_JPRB, 0.95316E+00_JPRB, 0.11207E+01_JPRB ,&
&  0.13339E+01_JPRB, 0.15067E+01_JPRB, 0.17140E+01_JPRB, 0.20027E+01_JPRB, 0.23135E+01_JPRB, 0.26579E+01_JPRB, 0.32129E+01_JPRB ,&
&  0.33863E+01_JPRB, 0.33528E+01_JPRB, 0.31680E+01_JPRB, 0.32188E+01_JPRB, 0.30788E+01_JPRB, 0.26464E+01_JPRB, 0.24741E+01_JPRB ,&
&  0.22190E+01_JPRB, 0.20404E+01_JPRB, 0.18997E+01_JPRB, 0.18087E+01_JPRB, 0.17189E+01_JPRB, 0.16021E+01_JPRB, 0.15205E+01_JPRB ,&
&  0.14592E+01_JPRB, 0.15337E+01_JPRB, 0.19704E+01_JPRB, 0.30971E+01_JPRB, 0.50211E+01_JPRB, 0.65337E+01_JPRB, 0.68228E+01_JPRB ,&
&  0.76085E+01_JPRB, 0.72679E+01_JPRB, 0.53838E+01_JPRB, 0.56535E+01_JPRB, 0.64204E+01_JPRB, 0.55956E+01_JPRB, 0.47433E+01_JPRB ,&
&  0.45792E+01_JPRB, 0.41594E+01_JPRB, 0.38575E+01_JPRB, 0.32783E+01_JPRB, 0.27081E+01_JPRB, 0.24306E+01_JPRB, 0.22319E+01_JPRB ,&
&  0.20580E+01_JPRB, 0.20084E+01_JPRB, 0.19746E+01_JPRB, 0.18828E+01_JPRB, 0.17833E+01_JPRB, 0.17328E+01_JPRB, 0.17780E+01_JPRB ,&
&  0.16844E+01_JPRB, 0.15207E+01_JPRB, 0.11848E+01_JPRB, 0.11391E+01_JPRB, 0.88293E+00_JPRB, 0.85903E+00_JPRB, 0.93122E+00_JPRB ,&
&  0.91719E+00_JPRB, 0.10088E+01_JPRB, 0.10794E+01_JPRB, 0.97491E+00_JPRB, 0.92714E+00_JPRB, 0.88901E+00_JPRB, 0.89495E+00_JPRB ,&
&  0.89494E+00_JPRB, 0.83989E+00_JPRB, 0.91449E+00_JPRB, 0.86963E+00_JPRB, 0.87176E+00_JPRB, 0.84777E+00_JPRB, 0.81585E+00_JPRB ,&
&  0.80785E+00_JPRB, 0.78575E+00_JPRB, 0.76674E+00_JPRB, 0.76269E+00_JPRB, 0.74804E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.73055E+00_JPRB, 0.71572E+00_JPRB, 0.73333E+00_JPRB, 0.69621E+00_JPRB, 0.66739E+00_JPRB, 0.64543E+00_JPRB, 0.63046E+00_JPRB ,&
&  0.62170E+00_JPRB, 0.61341E+00_JPRB, 0.60523E+00_JPRB, 0.59176E+00_JPRB, 0.58037E+00_JPRB, 0.56801E+00_JPRB, 0.56577E+00_JPRB ,&
&  0.57938E+00_JPRB, 0.62101E+00_JPRB, 0.65780E+00_JPRB, 0.66362E+00_JPRB, 0.80684E+00_JPRB, 0.84863E+00_JPRB, 0.93017E+00_JPRB ,&
&  0.10252E+01_JPRB, 0.11474E+01_JPRB, 0.11999E+01_JPRB, 0.12576E+01_JPRB, 0.13779E+01_JPRB, 0.16344E+01_JPRB, 0.19737E+01_JPRB ,&
&  0.21052E+01_JPRB, 0.20382E+01_JPRB, 0.20451E+01_JPRB, 0.21476E+01_JPRB, 0.23126E+01_JPRB, 0.20390E+01_JPRB, 0.19642E+01_JPRB ,&
&  0.18153E+01_JPRB, 0.16935E+01_JPRB, 0.16057E+01_JPRB, 0.15503E+01_JPRB, 0.15419E+01_JPRB, 0.15519E+01_JPRB, 0.15683E+01_JPRB ,&
&  0.15535E+01_JPRB, 0.14955E+01_JPRB, 0.15927E+01_JPRB, 0.23059E+01_JPRB, 0.44400E+01_JPRB, 0.70552E+01_JPRB, 0.89398E+01_JPRB ,&
&  0.89193E+01_JPRB, 0.83148E+01_JPRB, 0.81025E+01_JPRB, 0.78756E+01_JPRB, 0.68160E+01_JPRB, 0.57453E+01_JPRB, 0.50511E+01_JPRB ,&
&  0.45871E+01_JPRB, 0.43132E+01_JPRB, 0.41671E+01_JPRB, 0.37431E+01_JPRB, 0.32160E+01_JPRB, 0.29550E+01_JPRB, 0.27333E+01_JPRB ,&
&  0.25019E+01_JPRB, 0.24206E+01_JPRB, 0.23774E+01_JPRB, 0.21930E+01_JPRB, 0.20609E+01_JPRB, 0.19009E+01_JPRB, 0.19135E+01_JPRB ,&
&  0.19023E+01_JPRB, 0.14340E+01_JPRB, 0.11967E+01_JPRB, 0.12895E+01_JPRB, 0.99656E+00_JPRB, 0.96561E+00_JPRB, 0.10623E+01_JPRB ,&
&  0.99010E+00_JPRB, 0.95081E+00_JPRB, 0.94948E+00_JPRB, 0.85106E+00_JPRB, 0.82681E+00_JPRB, 0.83546E+00_JPRB, 0.82150E+00_JPRB ,&
&  0.80675E+00_JPRB, 0.79331E+00_JPRB, 0.87466E+00_JPRB, 0.81002E+00_JPRB, 0.80348E+00_JPRB, 0.79304E+00_JPRB, 0.77358E+00_JPRB ,&
&  0.76648E+00_JPRB, 0.75221E+00_JPRB, 0.73401E+00_JPRB, 0.72872E+00_JPRB, 0.73597E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.69108E+00_JPRB, 0.69030E+00_JPRB, 0.69693E+00_JPRB, 0.69104E+00_JPRB, 0.66723E+00_JPRB, 0.63899E+00_JPRB, 0.61623E+00_JPRB ,&
&  0.60300E+00_JPRB, 0.59402E+00_JPRB, 0.58312E+00_JPRB, 0.57002E+00_JPRB, 0.55908E+00_JPRB, 0.54812E+00_JPRB, 0.55637E+00_JPRB ,&
&  0.48916E+00_JPRB, 0.50113E+00_JPRB, 0.51992E+00_JPRB, 0.57512E+00_JPRB, 0.72510E+00_JPRB, 0.72742E+00_JPRB, 0.76351E+00_JPRB ,&
&  0.80830E+00_JPRB, 0.85824E+00_JPRB, 0.88234E+00_JPRB, 0.92044E+00_JPRB, 0.99127E+00_JPRB, 0.11393E+01_JPRB, 0.13234E+01_JPRB ,&
&  0.13286E+01_JPRB, 0.12049E+01_JPRB, 0.11707E+01_JPRB, 0.11439E+01_JPRB, 0.11965E+01_JPRB, 0.12606E+01_JPRB, 0.13558E+01_JPRB ,&
&  0.13413E+01_JPRB, 0.13128E+01_JPRB, 0.12978E+01_JPRB, 0.12915E+01_JPRB, 0.12900E+01_JPRB, 0.13025E+01_JPRB, 0.13267E+01_JPRB ,&
&  0.13554E+01_JPRB, 0.13504E+01_JPRB, 0.13767E+01_JPRB, 0.17633E+01_JPRB, 0.35978E+01_JPRB, 0.70306E+01_JPRB, 0.10829E+02_JPRB ,&
&  0.10163E+02_JPRB, 0.86742E+01_JPRB, 0.83200E+01_JPRB, 0.81414E+01_JPRB, 0.69677E+01_JPRB, 0.57165E+01_JPRB, 0.47253E+01_JPRB ,&
&  0.41974E+01_JPRB, 0.38518E+01_JPRB, 0.37713E+01_JPRB, 0.36782E+01_JPRB, 0.33750E+01_JPRB, 0.32031E+01_JPRB, 0.30783E+01_JPRB ,&
&  0.28808E+01_JPRB, 0.27462E+01_JPRB, 0.25571E+01_JPRB, 0.23594E+01_JPRB, 0.22506E+01_JPRB, 0.22534E+01_JPRB, 0.22064E+01_JPRB ,&
&  0.21660E+01_JPRB, 0.19754E+01_JPRB, 0.16828E+01_JPRB, 0.16225E+01_JPRB, 0.13063E+01_JPRB, 0.12506E+01_JPRB, 0.13802E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.98340E+00_JPRB, 0.91126E+00_JPRB, 0.88010E+00_JPRB, 0.86911E+00_JPRB, 0.88229E+00_JPRB, 0.84664E+00_JPRB ,&
&  0.78900E+00_JPRB, 0.82236E+00_JPRB, 0.85132E+00_JPRB, 0.75128E+00_JPRB, 0.72515E+00_JPRB, 0.70431E+00_JPRB, 0.64829E+00_JPRB ,&
&  0.67923E+00_JPRB, 0.67030E+00_JPRB, 0.66705E+00_JPRB, 0.67004E+00_JPRB, 0.68074E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.63319E+00_JPRB, 0.64093E+00_JPRB, 0.64914E+00_JPRB, 0.64774E+00_JPRB, 0.63126E+00_JPRB, 0.61153E+00_JPRB, 0.59972E+00_JPRB ,&
&  0.59396E+00_JPRB, 0.57967E+00_JPRB, 0.56282E+00_JPRB, 0.54412E+00_JPRB, 0.53198E+00_JPRB, 0.53314E+00_JPRB, 0.50029E+00_JPRB ,&
&  0.45401E+00_JPRB, 0.46456E+00_JPRB, 0.47114E+00_JPRB, 0.50221E+00_JPRB, 0.54908E+00_JPRB, 0.57320E+00_JPRB, 0.60148E+00_JPRB ,&
&  0.66589E+00_JPRB, 0.71871E+00_JPRB, 0.75167E+00_JPRB, 0.78738E+00_JPRB, 0.81576E+00_JPRB, 0.85291E+00_JPRB, 0.89283E+00_JPRB ,&
&  0.86141E+00_JPRB, 0.78997E+00_JPRB, 0.73678E+00_JPRB, 0.69196E+00_JPRB, 0.72866E+00_JPRB, 0.79533E+00_JPRB, 0.84602E+00_JPRB ,&
&  0.87812E+00_JPRB, 0.90559E+00_JPRB, 0.93843E+00_JPRB, 0.96450E+00_JPRB, 0.97837E+00_JPRB, 0.99751E+00_JPRB, 0.10169E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.10745E+01_JPRB, 0.11089E+01_JPRB, 0.12792E+01_JPRB, 0.25699E+01_JPRB, 0.52052E+01_JPRB, 0.73242E+01_JPRB ,&
&  0.65779E+01_JPRB, 0.62779E+01_JPRB, 0.60773E+01_JPRB, 0.57786E+01_JPRB, 0.52902E+01_JPRB, 0.45324E+01_JPRB, 0.38858E+01_JPRB ,&
&  0.36085E+01_JPRB, 0.33017E+01_JPRB, 0.32258E+01_JPRB, 0.30902E+01_JPRB, 0.29938E+01_JPRB, 0.28835E+01_JPRB, 0.26714E+01_JPRB ,&
&  0.25162E+01_JPRB, 0.22926E+01_JPRB, 0.22256E+01_JPRB, 0.21145E+01_JPRB, 0.20624E+01_JPRB, 0.20590E+01_JPRB, 0.20464E+01_JPRB ,&
&  0.20290E+01_JPRB, 0.19869E+01_JPRB, 0.18188E+01_JPRB, 0.16905E+01_JPRB, 0.15777E+01_JPRB, 0.16391E+01_JPRB, 0.14906E+01_JPRB ,&
&  0.12518E+01_JPRB, 0.99715E+00_JPRB, 0.96435E+00_JPRB, 0.91020E+00_JPRB, 0.87019E+00_JPRB, 0.84501E+00_JPRB, 0.81430E+00_JPRB ,&
&  0.83181E+00_JPRB, 0.86890E+00_JPRB, 0.81249E+00_JPRB, 0.71238E+00_JPRB, 0.66813E+00_JPRB, 0.63474E+00_JPRB, 0.55870E+00_JPRB ,&
&  0.55618E+00_JPRB, 0.60552E+00_JPRB, 0.61231E+00_JPRB, 0.61612E+00_JPRB, 0.62056E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.59749E+00_JPRB, 0.60357E+00_JPRB, 0.60404E+00_JPRB, 0.60639E+00_JPRB, 0.60617E+00_JPRB, 0.60431E+00_JPRB, 0.57834E+00_JPRB ,&
&  0.60521E+00_JPRB, 0.59006E+00_JPRB, 0.56124E+00_JPRB, 0.54319E+00_JPRB, 0.54052E+00_JPRB, 0.50513E+00_JPRB, 0.43269E+00_JPRB ,&
&  0.41509E+00_JPRB, 0.42399E+00_JPRB, 0.44454E+00_JPRB, 0.46304E+00_JPRB, 0.47920E+00_JPRB, 0.48990E+00_JPRB, 0.50903E+00_JPRB ,&
&  0.53584E+00_JPRB, 0.56191E+00_JPRB, 0.59422E+00_JPRB, 0.62530E+00_JPRB, 0.62727E+00_JPRB, 0.62436E+00_JPRB, 0.63533E+00_JPRB ,&
&  0.61393E+00_JPRB, 0.58073E+00_JPRB, 0.56271E+00_JPRB, 0.51446E+00_JPRB, 0.55095E+00_JPRB, 0.56877E+00_JPRB, 0.58824E+00_JPRB ,&
&  0.61434E+00_JPRB, 0.64147E+00_JPRB, 0.67223E+00_JPRB, 0.69739E+00_JPRB, 0.71490E+00_JPRB, 0.73525E+00_JPRB, 0.75663E+00_JPRB ,&
&  0.77966E+00_JPRB, 0.81436E+00_JPRB, 0.86602E+00_JPRB, 0.95519E+00_JPRB, 0.16590E+01_JPRB, 0.33020E+01_JPRB, 0.44279E+01_JPRB ,&
&  0.41830E+01_JPRB, 0.44897E+01_JPRB, 0.45692E+01_JPRB, 0.39981E+01_JPRB, 0.39521E+01_JPRB, 0.38746E+01_JPRB, 0.35306E+01_JPRB ,&
&  0.34456E+01_JPRB, 0.32342E+01_JPRB, 0.29382E+01_JPRB, 0.26846E+01_JPRB, 0.24703E+01_JPRB, 0.22709E+01_JPRB, 0.20828E+01_JPRB ,&
&  0.19546E+01_JPRB, 0.18620E+01_JPRB, 0.18484E+01_JPRB, 0.18807E+01_JPRB, 0.18777E+01_JPRB, 0.18863E+01_JPRB, 0.18997E+01_JPRB ,&
&  0.18904E+01_JPRB, 0.18771E+01_JPRB, 0.18295E+01_JPRB, 0.17293E+01_JPRB, 0.16321E+01_JPRB, 0.15308E+01_JPRB, 0.14174E+01_JPRB ,&
&  0.13056E+01_JPRB, 0.11421E+01_JPRB, 0.10304E+01_JPRB, 0.98122E+00_JPRB, 0.91497E+00_JPRB, 0.84062E+00_JPRB, 0.81931E+00_JPRB ,&
&  0.80052E+00_JPRB, 0.73598E+00_JPRB, 0.72578E+00_JPRB, 0.65760E+00_JPRB, 0.61371E+00_JPRB, 0.58859E+00_JPRB, 0.53551E+00_JPRB ,&
&  0.51353E+00_JPRB, 0.55607E+00_JPRB, 0.57801E+00_JPRB, 0.59115E+00_JPRB, 0.59398E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.57323E+00_JPRB, 0.58125E+00_JPRB, 0.58521E+00_JPRB, 0.58972E+00_JPRB, 0.58754E+00_JPRB, 0.56288E+00_JPRB, 0.54190E+00_JPRB ,&
&  0.60115E+00_JPRB, 0.56215E+00_JPRB, 0.57505E+00_JPRB, 0.48428E+00_JPRB, 0.44383E+00_JPRB, 0.43500E+00_JPRB, 0.41872E+00_JPRB ,&
&  0.41365E+00_JPRB, 0.41979E+00_JPRB, 0.42663E+00_JPRB, 0.43408E+00_JPRB, 0.44471E+00_JPRB, 0.44767E+00_JPRB, 0.44976E+00_JPRB ,&
&  0.45599E+00_JPRB, 0.46687E+00_JPRB, 0.48883E+00_JPRB, 0.50001E+00_JPRB, 0.50634E+00_JPRB, 0.51026E+00_JPRB, 0.51846E+00_JPRB ,&
&  0.49352E+00_JPRB, 0.47803E+00_JPRB, 0.48100E+00_JPRB, 0.47602E+00_JPRB, 0.47441E+00_JPRB, 0.47877E+00_JPRB, 0.49217E+00_JPRB ,&
&  0.51010E+00_JPRB, 0.48690E+00_JPRB, 0.53388E+00_JPRB, 0.55949E+00_JPRB, 0.58113E+00_JPRB, 0.60032E+00_JPRB, 0.61888E+00_JPRB ,&
&  0.63778E+00_JPRB, 0.66146E+00_JPRB, 0.69782E+00_JPRB, 0.77104E+00_JPRB, 0.11072E+01_JPRB, 0.20554E+01_JPRB, 0.27224E+01_JPRB ,&
&  0.30067E+01_JPRB, 0.24323E+01_JPRB, 0.28279E+01_JPRB, 0.29358E+01_JPRB, 0.30822E+01_JPRB, 0.31038E+01_JPRB, 0.30319E+01_JPRB ,&
&  0.30263E+01_JPRB, 0.29132E+01_JPRB, 0.26049E+01_JPRB, 0.22165E+01_JPRB, 0.19242E+01_JPRB, 0.17526E+01_JPRB, 0.16425E+01_JPRB ,&
&  0.15462E+01_JPRB, 0.14115E+01_JPRB, 0.15068E+01_JPRB, 0.15997E+01_JPRB, 0.16412E+01_JPRB, 0.16747E+01_JPRB, 0.17295E+01_JPRB ,&
&  0.17677E+01_JPRB, 0.17455E+01_JPRB, 0.17050E+01_JPRB, 0.15700E+01_JPRB, 0.15018E+01_JPRB, 0.14418E+01_JPRB, 0.13551E+01_JPRB ,&
&  0.12929E+01_JPRB, 0.12340E+01_JPRB, 0.11730E+01_JPRB, 0.11623E+01_JPRB, 0.10362E+01_JPRB, 0.94935E+00_JPRB, 0.88613E+00_JPRB ,&
&  0.77063E+00_JPRB, 0.63666E+00_JPRB, 0.57969E+00_JPRB, 0.54401E+00_JPRB, 0.51469E+00_JPRB, 0.51383E+00_JPRB, 0.52174E+00_JPRB ,&
&  0.50697E+00_JPRB, 0.51808E+00_JPRB, 0.55026E+00_JPRB, 0.56118E+00_JPRB, 0.56386E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.54879E+00_JPRB, 0.56660E+00_JPRB, 0.58071E+00_JPRB, 0.58283E+00_JPRB, 0.57156E+00_JPRB, 0.54149E+00_JPRB, 0.50046E+00_JPRB ,&
&  0.49279E+00_JPRB, 0.47123E+00_JPRB, 0.45136E+00_JPRB, 0.43663E+00_JPRB, 0.42722E+00_JPRB, 0.41633E+00_JPRB, 0.40137E+00_JPRB ,&
&  0.40023E+00_JPRB, 0.40972E+00_JPRB, 0.42418E+00_JPRB, 0.42697E+00_JPRB, 0.42494E+00_JPRB, 0.42245E+00_JPRB, 0.41839E+00_JPRB ,&
&  0.41390E+00_JPRB, 0.41486E+00_JPRB, 0.42392E+00_JPRB, 0.43914E+00_JPRB, 0.44684E+00_JPRB, 0.44796E+00_JPRB, 0.44954E+00_JPRB ,&
&  0.44418E+00_JPRB, 0.43651E+00_JPRB, 0.43210E+00_JPRB, 0.43356E+00_JPRB, 0.43071E+00_JPRB, 0.44062E+00_JPRB, 0.44556E+00_JPRB ,&
&  0.40302E+00_JPRB, 0.41367E+00_JPRB, 0.47742E+00_JPRB, 0.50108E+00_JPRB, 0.52135E+00_JPRB, 0.53828E+00_JPRB, 0.55469E+00_JPRB ,&
&  0.57186E+00_JPRB, 0.60222E+00_JPRB, 0.64190E+00_JPRB, 0.69867E+00_JPRB, 0.88490E+00_JPRB, 0.13491E+01_JPRB, 0.17863E+01_JPRB ,&
&  0.22491E+01_JPRB, 0.19843E+01_JPRB, 0.18830E+01_JPRB, 0.19964E+01_JPRB, 0.24520E+01_JPRB, 0.24712E+01_JPRB, 0.24877E+01_JPRB ,&
&  0.24915E+01_JPRB, 0.23443E+01_JPRB, 0.21663E+01_JPRB, 0.19155E+01_JPRB, 0.16203E+01_JPRB, 0.14155E+01_JPRB, 0.12864E+01_JPRB ,&
&  0.11716E+01_JPRB, 0.10996E+01_JPRB, 0.10809E+01_JPRB, 0.11420E+01_JPRB, 0.12356E+01_JPRB, 0.13102E+01_JPRB, 0.13671E+01_JPRB ,&
&  0.14405E+01_JPRB, 0.14812E+01_JPRB, 0.14574E+01_JPRB, 0.14073E+01_JPRB, 0.13365E+01_JPRB, 0.12928E+01_JPRB, 0.12727E+01_JPRB ,&
&  0.12496E+01_JPRB, 0.12168E+01_JPRB, 0.11617E+01_JPRB, 0.11035E+01_JPRB, 0.10106E+01_JPRB, 0.92485E+00_JPRB, 0.81364E+00_JPRB ,&
&  0.65912E+00_JPRB, 0.55933E+00_JPRB, 0.48983E+00_JPRB, 0.46840E+00_JPRB, 0.46469E+00_JPRB, 0.46256E+00_JPRB, 0.46244E+00_JPRB ,&
&  0.47269E+00_JPRB, 0.48157E+00_JPRB, 0.48773E+00_JPRB, 0.50158E+00_JPRB, 0.52175E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.54075E+00_JPRB, 0.55610E+00_JPRB, 0.56136E+00_JPRB, 0.56453E+00_JPRB, 0.53728E+00_JPRB, 0.51162E+00_JPRB, 0.48686E+00_JPRB ,&
&  0.47772E+00_JPRB, 0.46465E+00_JPRB, 0.45362E+00_JPRB, 0.43785E+00_JPRB, 0.42530E+00_JPRB, 0.42068E+00_JPRB, 0.42362E+00_JPRB ,&
&  0.42946E+00_JPRB, 0.43031E+00_JPRB, 0.42773E+00_JPRB, 0.41671E+00_JPRB, 0.40653E+00_JPRB, 0.39748E+00_JPRB, 0.39027E+00_JPRB ,&
&  0.38398E+00_JPRB, 0.38488E+00_JPRB, 0.39064E+00_JPRB, 0.40169E+00_JPRB, 0.40819E+00_JPRB, 0.41277E+00_JPRB, 0.41112E+00_JPRB ,&
&  0.40233E+00_JPRB, 0.39175E+00_JPRB, 0.39409E+00_JPRB, 0.40125E+00_JPRB, 0.40387E+00_JPRB, 0.41108E+00_JPRB, 0.39197E+00_JPRB ,&
&  0.37446E+00_JPRB, 0.37305E+00_JPRB, 0.39031E+00_JPRB, 0.46261E+00_JPRB, 0.48922E+00_JPRB, 0.50389E+00_JPRB, 0.52224E+00_JPRB ,&
&  0.51415E+00_JPRB, 0.52363E+00_JPRB, 0.56060E+00_JPRB, 0.65506E+00_JPRB, 0.76479E+00_JPRB, 0.95226E+00_JPRB, 0.12113E+01_JPRB ,&
&  0.14659E+01_JPRB, 0.16860E+01_JPRB, 0.18458E+01_JPRB, 0.16695E+01_JPRB, 0.16648E+01_JPRB, 0.18992E+01_JPRB, 0.19278E+01_JPRB ,&
&  0.18934E+01_JPRB, 0.18061E+01_JPRB, 0.17899E+01_JPRB, 0.15862E+01_JPRB, 0.13291E+01_JPRB, 0.11576E+01_JPRB, 0.10282E+01_JPRB ,&
&  0.90949E+00_JPRB, 0.82973E+00_JPRB, 0.78540E+00_JPRB, 0.76291E+00_JPRB, 0.77343E+00_JPRB, 0.83502E+00_JPRB, 0.92591E+00_JPRB ,&
&  0.10184E+01_JPRB, 0.10701E+01_JPRB, 0.10956E+01_JPRB, 0.10833E+01_JPRB, 0.10689E+01_JPRB, 0.10505E+01_JPRB, 0.10410E+01_JPRB ,&
&  0.10283E+01_JPRB, 0.10247E+01_JPRB, 0.10117E+01_JPRB, 0.99140E+00_JPRB, 0.90800E+00_JPRB, 0.77792E+00_JPRB, 0.65212E+00_JPRB ,&
&  0.56434E+00_JPRB, 0.51124E+00_JPRB, 0.47324E+00_JPRB, 0.46109E+00_JPRB, 0.46054E+00_JPRB, 0.45902E+00_JPRB, 0.44545E+00_JPRB ,&
&  0.44100E+00_JPRB, 0.45084E+00_JPRB, 0.46664E+00_JPRB, 0.48408E+00_JPRB, 0.50965E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.52895E+00_JPRB, 0.53392E+00_JPRB, 0.52945E+00_JPRB, 0.51784E+00_JPRB, 0.50728E+00_JPRB, 0.47367E+00_JPRB, 0.44850E+00_JPRB ,&
&  0.43708E+00_JPRB, 0.43408E+00_JPRB, 0.43581E+00_JPRB, 0.43700E+00_JPRB, 0.43976E+00_JPRB, 0.44430E+00_JPRB, 0.44446E+00_JPRB ,&
&  0.43888E+00_JPRB, 0.43357E+00_JPRB, 0.42803E+00_JPRB, 0.41888E+00_JPRB, 0.40609E+00_JPRB, 0.39116E+00_JPRB, 0.37943E+00_JPRB ,&
&  0.37263E+00_JPRB, 0.37246E+00_JPRB, 0.38056E+00_JPRB, 0.38843E+00_JPRB, 0.39322E+00_JPRB, 0.39381E+00_JPRB, 0.39083E+00_JPRB ,&
&  0.38188E+00_JPRB, 0.37582E+00_JPRB, 0.38137E+00_JPRB, 0.38394E+00_JPRB, 0.38759E+00_JPRB, 0.39152E+00_JPRB, 0.37783E+00_JPRB ,&
&  0.36348E+00_JPRB, 0.35136E+00_JPRB, 0.34874E+00_JPRB, 0.35863E+00_JPRB, 0.39702E+00_JPRB, 0.46419E+00_JPRB, 0.48564E+00_JPRB ,&
&  0.50557E+00_JPRB, 0.52762E+00_JPRB, 0.55380E+00_JPRB, 0.59103E+00_JPRB, 0.64189E+00_JPRB, 0.71309E+00_JPRB, 0.84197E+00_JPRB ,&
&  0.10062E+01_JPRB, 0.11536E+01_JPRB, 0.13094E+01_JPRB, 0.12254E+01_JPRB, 0.11432E+01_JPRB, 0.11563E+01_JPRB, 0.12003E+01_JPRB ,&
&  0.12142E+01_JPRB, 0.11491E+01_JPRB, 0.11619E+01_JPRB, 0.11654E+01_JPRB, 0.10956E+01_JPRB, 0.10052E+01_JPRB, 0.89658E+00_JPRB ,&
&  0.81183E+00_JPRB, 0.74525E+00_JPRB, 0.70256E+00_JPRB, 0.65833E+00_JPRB, 0.63256E+00_JPRB, 0.62925E+00_JPRB, 0.64580E+00_JPRB ,&
&  0.68212E+00_JPRB, 0.72835E+00_JPRB, 0.73821E+00_JPRB, 0.72077E+00_JPRB, 0.74227E+00_JPRB, 0.78141E+00_JPRB, 0.81612E+00_JPRB ,&
&  0.82754E+00_JPRB, 0.83483E+00_JPRB, 0.84789E+00_JPRB, 0.84167E+00_JPRB, 0.78767E+00_JPRB, 0.73376E+00_JPRB, 0.66983E+00_JPRB ,&
&  0.60571E+00_JPRB, 0.54740E+00_JPRB, 0.51589E+00_JPRB, 0.50376E+00_JPRB, 0.49589E+00_JPRB, 0.48745E+00_JPRB, 0.47558E+00_JPRB ,&
&  0.46680E+00_JPRB, 0.46776E+00_JPRB, 0.47943E+00_JPRB, 0.49494E+00_JPRB, 0.51495E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.52972E+00_JPRB, 0.50672E+00_JPRB, 0.49099E+00_JPRB, 0.48411E+00_JPRB, 0.48031E+00_JPRB, 0.46537E+00_JPRB, 0.45505E+00_JPRB ,&
&  0.44979E+00_JPRB, 0.45379E+00_JPRB, 0.45247E+00_JPRB, 0.44822E+00_JPRB, 0.44657E+00_JPRB, 0.44553E+00_JPRB, 0.44504E+00_JPRB ,&
&  0.44328E+00_JPRB, 0.43802E+00_JPRB, 0.42722E+00_JPRB, 0.40892E+00_JPRB, 0.39487E+00_JPRB, 0.38480E+00_JPRB, 0.37885E+00_JPRB ,&
&  0.37725E+00_JPRB, 0.37910E+00_JPRB, 0.38515E+00_JPRB, 0.38739E+00_JPRB, 0.38358E+00_JPRB, 0.37777E+00_JPRB, 0.37065E+00_JPRB ,&
&  0.37087E+00_JPRB, 0.37990E+00_JPRB, 0.37401E+00_JPRB, 0.37475E+00_JPRB, 0.38335E+00_JPRB, 0.37861E+00_JPRB, 0.36413E+00_JPRB ,&
&  0.35152E+00_JPRB, 0.33892E+00_JPRB, 0.33103E+00_JPRB, 0.32913E+00_JPRB, 0.34007E+00_JPRB, 0.35864E+00_JPRB, 0.39457E+00_JPRB ,&
&  0.45350E+00_JPRB, 0.48119E+00_JPRB, 0.49982E+00_JPRB, 0.52258E+00_JPRB, 0.54731E+00_JPRB, 0.57691E+00_JPRB, 0.62346E+00_JPRB ,&
&  0.69464E+00_JPRB, 0.77312E+00_JPRB, 0.85119E+00_JPRB, 0.90821E+00_JPRB, 0.94458E+00_JPRB, 0.88399E+00_JPRB, 0.86227E+00_JPRB ,&
&  0.93379E+00_JPRB, 0.93932E+00_JPRB, 0.89513E+00_JPRB, 0.84548E+00_JPRB, 0.79371E+00_JPRB, 0.74709E+00_JPRB, 0.70942E+00_JPRB ,&
&  0.68466E+00_JPRB, 0.66704E+00_JPRB, 0.64686E+00_JPRB, 0.61002E+00_JPRB, 0.57495E+00_JPRB, 0.55684E+00_JPRB, 0.55310E+00_JPRB ,&
&  0.56081E+00_JPRB, 0.60036E+00_JPRB, 0.67807E+00_JPRB, 0.60134E+00_JPRB, 0.58391E+00_JPRB, 0.59781E+00_JPRB, 0.62377E+00_JPRB ,&
&  0.65951E+00_JPRB, 0.68920E+00_JPRB, 0.71091E+00_JPRB, 0.70848E+00_JPRB, 0.69312E+00_JPRB, 0.66966E+00_JPRB, 0.65288E+00_JPRB ,&
&  0.63711E+00_JPRB, 0.62054E+00_JPRB, 0.60875E+00_JPRB, 0.60445E+00_JPRB, 0.59633E+00_JPRB, 0.58813E+00_JPRB, 0.57996E+00_JPRB ,&
&  0.57184E+00_JPRB, 0.56885E+00_JPRB, 0.56827E+00_JPRB, 0.56353E+00_JPRB, 0.55423E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.48403E+00_JPRB, 0.47584E+00_JPRB, 0.46941E+00_JPRB, 0.46545E+00_JPRB, 0.46179E+00_JPRB, 0.45812E+00_JPRB, 0.45523E+00_JPRB ,&
&  0.45325E+00_JPRB, 0.45217E+00_JPRB, 0.44939E+00_JPRB, 0.44622E+00_JPRB, 0.44318E+00_JPRB, 0.44051E+00_JPRB, 0.43811E+00_JPRB ,&
&  0.43225E+00_JPRB, 0.41590E+00_JPRB, 0.40931E+00_JPRB, 0.40243E+00_JPRB, 0.39344E+00_JPRB, 0.38858E+00_JPRB, 0.38603E+00_JPRB ,&
&  0.38225E+00_JPRB, 0.38026E+00_JPRB, 0.38084E+00_JPRB, 0.37591E+00_JPRB, 0.37189E+00_JPRB, 0.37394E+00_JPRB, 0.37718E+00_JPRB ,&
&  0.37016E+00_JPRB, 0.36977E+00_JPRB, 0.38349E+00_JPRB, 0.38137E+00_JPRB, 0.36580E+00_JPRB, 0.36358E+00_JPRB, 0.35192E+00_JPRB ,&
&  0.34375E+00_JPRB, 0.32756E+00_JPRB, 0.32197E+00_JPRB, 0.31813E+00_JPRB, 0.32410E+00_JPRB, 0.33440E+00_JPRB, 0.35909E+00_JPRB ,&
&  0.38631E+00_JPRB, 0.43525E+00_JPRB, 0.45482E+00_JPRB, 0.47011E+00_JPRB, 0.48482E+00_JPRB, 0.50033E+00_JPRB, 0.51852E+00_JPRB ,&
&  0.54180E+00_JPRB, 0.57065E+00_JPRB, 0.60629E+00_JPRB, 0.63764E+00_JPRB, 0.66299E+00_JPRB, 0.67096E+00_JPRB, 0.66959E+00_JPRB ,&
&  0.68012E+00_JPRB, 0.67222E+00_JPRB, 0.65099E+00_JPRB, 0.62765E+00_JPRB, 0.60183E+00_JPRB, 0.57653E+00_JPRB, 0.54971E+00_JPRB ,&
&  0.54065E+00_JPRB, 0.54623E+00_JPRB, 0.53660E+00_JPRB, 0.52708E+00_JPRB, 0.51531E+00_JPRB, 0.51064E+00_JPRB, 0.50587E+00_JPRB ,&
&  0.49614E+00_JPRB, 0.49817E+00_JPRB, 0.52618E+00_JPRB, 0.52407E+00_JPRB, 0.52709E+00_JPRB, 0.53206E+00_JPRB, 0.54104E+00_JPRB ,&
&  0.55302E+00_JPRB, 0.56974E+00_JPRB, 0.58648E+00_JPRB, 0.59907E+00_JPRB, 0.61064E+00_JPRB, 0.61144E+00_JPRB, 0.61883E+00_JPRB ,&
&  0.61702E+00_JPRB, 0.61387E+00_JPRB, 0.60835E+00_JPRB, 0.60479E+00_JPRB, 0.59901E+00_JPRB, 0.59124E+00_JPRB, 0.58035E+00_JPRB ,&
&  0.56727E+00_JPRB, 0.55134E+00_JPRB, 0.53365E+00_JPRB, 0.51564E+00_JPRB, 0.49772E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.44967E+00_JPRB, 0.44612E+00_JPRB, 0.44357E+00_JPRB, 0.44100E+00_JPRB, 0.43941E+00_JPRB, 0.43772E+00_JPRB, 0.43632E+00_JPRB ,&
&  0.43541E+00_JPRB, 0.43357E+00_JPRB, 0.43083E+00_JPRB, 0.42729E+00_JPRB, 0.42422E+00_JPRB, 0.42124E+00_JPRB, 0.41811E+00_JPRB ,&
&  0.41319E+00_JPRB, 0.40755E+00_JPRB, 0.40157E+00_JPRB, 0.39663E+00_JPRB, 0.39682E+00_JPRB, 0.39603E+00_JPRB, 0.39503E+00_JPRB ,&
&  0.39105E+00_JPRB, 0.38478E+00_JPRB, 0.37601E+00_JPRB, 0.37402E+00_JPRB, 0.37400E+00_JPRB, 0.36930E+00_JPRB, 0.37258E+00_JPRB ,&
&  0.37866E+00_JPRB, 0.37748E+00_JPRB, 0.37621E+00_JPRB, 0.37457E+00_JPRB, 0.35896E+00_JPRB, 0.32762E+00_JPRB, 0.33624E+00_JPRB ,&
&  0.33278E+00_JPRB, 0.32307E+00_JPRB, 0.31932E+00_JPRB, 0.31708E+00_JPRB, 0.32070E+00_JPRB, 0.32966E+00_JPRB, 0.33976E+00_JPRB ,&
&  0.34649E+00_JPRB, 0.39958E+00_JPRB, 0.42049E+00_JPRB, 0.43447E+00_JPRB, 0.44332E+00_JPRB, 0.45086E+00_JPRB, 0.45922E+00_JPRB ,&
&  0.46670E+00_JPRB, 0.47884E+00_JPRB, 0.49454E+00_JPRB, 0.50932E+00_JPRB, 0.52225E+00_JPRB, 0.53057E+00_JPRB, 0.53856E+00_JPRB ,&
&  0.54764E+00_JPRB, 0.54781E+00_JPRB, 0.53990E+00_JPRB, 0.53099E+00_JPRB, 0.51882E+00_JPRB, 0.50603E+00_JPRB, 0.49354E+00_JPRB ,&
&  0.48646E+00_JPRB, 0.47969E+00_JPRB, 0.48035E+00_JPRB, 0.47887E+00_JPRB, 0.47557E+00_JPRB, 0.47396E+00_JPRB, 0.47446E+00_JPRB ,&
&  0.47484E+00_JPRB, 0.47648E+00_JPRB, 0.47664E+00_JPRB, 0.48139E+00_JPRB, 0.48789E+00_JPRB, 0.49206E+00_JPRB, 0.49584E+00_JPRB ,&
&  0.49737E+00_JPRB, 0.50178E+00_JPRB, 0.50367E+00_JPRB, 0.50969E+00_JPRB, 0.51774E+00_JPRB, 0.52377E+00_JPRB, 0.52939E+00_JPRB ,&
&  0.53383E+00_JPRB, 0.53612E+00_JPRB, 0.53479E+00_JPRB, 0.52940E+00_JPRB, 0.52099E+00_JPRB, 0.50994E+00_JPRB, 0.49669E+00_JPRB ,&
&  0.48288E+00_JPRB, 0.47132E+00_JPRB, 0.46217E+00_JPRB, 0.45654E+00_JPRB, 0.45222E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.43094E+00_JPRB, 0.42689E+00_JPRB, 0.42426E+00_JPRB, 0.42180E+00_JPRB, 0.42056E+00_JPRB, 0.41921E+00_JPRB, 0.41777E+00_JPRB ,&
&  0.41604E+00_JPRB, 0.41458E+00_JPRB, 0.41286E+00_JPRB, 0.41171E+00_JPRB, 0.41005E+00_JPRB, 0.40883E+00_JPRB, 0.40714E+00_JPRB ,&
&  0.40641E+00_JPRB, 0.40480E+00_JPRB, 0.40478E+00_JPRB, 0.40280E+00_JPRB, 0.40350E+00_JPRB, 0.40190E+00_JPRB, 0.39737E+00_JPRB ,&
&  0.39279E+00_JPRB, 0.38621E+00_JPRB, 0.37435E+00_JPRB, 0.37674E+00_JPRB, 0.37424E+00_JPRB, 0.37519E+00_JPRB, 0.36987E+00_JPRB ,&
&  0.36889E+00_JPRB, 0.35373E+00_JPRB, 0.36076E+00_JPRB, 0.35384E+00_JPRB, 0.34927E+00_JPRB, 0.34310E+00_JPRB, 0.33469E+00_JPRB ,&
&  0.33219E+00_JPRB, 0.32727E+00_JPRB, 0.32708E+00_JPRB, 0.32926E+00_JPRB, 0.33270E+00_JPRB, 0.33803E+00_JPRB, 0.34392E+00_JPRB ,&
&  0.34698E+00_JPRB, 0.37938E+00_JPRB, 0.39786E+00_JPRB, 0.41100E+00_JPRB, 0.42195E+00_JPRB, 0.42994E+00_JPRB, 0.43716E+00_JPRB ,&
&  0.44379E+00_JPRB, 0.45057E+00_JPRB, 0.45786E+00_JPRB, 0.46023E+00_JPRB, 0.46862E+00_JPRB, 0.47176E+00_JPRB, 0.47538E+00_JPRB ,&
&  0.47851E+00_JPRB, 0.47795E+00_JPRB, 0.47663E+00_JPRB, 0.47343E+00_JPRB, 0.47188E+00_JPRB, 0.46823E+00_JPRB, 0.46695E+00_JPRB ,&
&  0.46423E+00_JPRB, 0.46241E+00_JPRB, 0.46312E+00_JPRB, 0.46342E+00_JPRB, 0.46249E+00_JPRB, 0.46183E+00_JPRB, 0.46161E+00_JPRB ,&
&  0.46203E+00_JPRB, 0.46447E+00_JPRB, 0.46658E+00_JPRB, 0.47136E+00_JPRB, 0.47640E+00_JPRB, 0.47937E+00_JPRB, 0.48237E+00_JPRB ,&
&  0.48562E+00_JPRB, 0.48182E+00_JPRB, 0.47900E+00_JPRB, 0.47774E+00_JPRB, 0.47828E+00_JPRB, 0.47971E+00_JPRB, 0.48169E+00_JPRB ,&
&  0.48207E+00_JPRB, 0.47905E+00_JPRB, 0.47498E+00_JPRB, 0.46943E+00_JPRB, 0.46383E+00_JPRB, 0.45912E+00_JPRB, 0.45445E+00_JPRB ,&
&  0.45035E+00_JPRB, 0.44847E+00_JPRB, 0.44462E+00_JPRB, 0.43974E+00_JPRB, 0.43481E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.42087E+00_JPRB, 0.41608E+00_JPRB, 0.41233E+00_JPRB, 0.40995E+00_JPRB, 0.40794E+00_JPRB, 0.40658E+00_JPRB, 0.40556E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40362E+00_JPRB, 0.40314E+00_JPRB, 0.40280E+00_JPRB, 0.40235E+00_JPRB, 0.40217E+00_JPRB, 0.40170E+00_JPRB ,&
&  0.40268E+00_JPRB, 0.40209E+00_JPRB, 0.40304E+00_JPRB, 0.40195E+00_JPRB, 0.40125E+00_JPRB, 0.39977E+00_JPRB, 0.39611E+00_JPRB ,&
&  0.39441E+00_JPRB, 0.39152E+00_JPRB, 0.37650E+00_JPRB, 0.38050E+00_JPRB, 0.37530E+00_JPRB, 0.37319E+00_JPRB, 0.37193E+00_JPRB ,&
&  0.36611E+00_JPRB, 0.36329E+00_JPRB, 0.35909E+00_JPRB, 0.35793E+00_JPRB, 0.35807E+00_JPRB, 0.35429E+00_JPRB, 0.35429E+00_JPRB ,&
&  0.35050E+00_JPRB, 0.34949E+00_JPRB, 0.34997E+00_JPRB, 0.35057E+00_JPRB, 0.35387E+00_JPRB, 0.35516E+00_JPRB, 0.35906E+00_JPRB ,&
&  0.36744E+00_JPRB, 0.37981E+00_JPRB, 0.40131E+00_JPRB, 0.41172E+00_JPRB, 0.42285E+00_JPRB, 0.43088E+00_JPRB, 0.43860E+00_JPRB ,&
&  0.44309E+00_JPRB, 0.44638E+00_JPRB, 0.44884E+00_JPRB, 0.45076E+00_JPRB, 0.45119E+00_JPRB, 0.44920E+00_JPRB, 0.44922E+00_JPRB ,&
&  0.44966E+00_JPRB, 0.45046E+00_JPRB, 0.45086E+00_JPRB, 0.45079E+00_JPRB, 0.45258E+00_JPRB, 0.45445E+00_JPRB, 0.45760E+00_JPRB ,&
&  0.46010E+00_JPRB, 0.46210E+00_JPRB, 0.46139E+00_JPRB, 0.46197E+00_JPRB, 0.46078E+00_JPRB, 0.46005E+00_JPRB, 0.45924E+00_JPRB ,&
&  0.45871E+00_JPRB, 0.45931E+00_JPRB, 0.46089E+00_JPRB, 0.45719E+00_JPRB, 0.45694E+00_JPRB, 0.46388E+00_JPRB, 0.46568E+00_JPRB ,&
&  0.46797E+00_JPRB, 0.46927E+00_JPRB, 0.46974E+00_JPRB, 0.46953E+00_JPRB, 0.46914E+00_JPRB, 0.46947E+00_JPRB, 0.46943E+00_JPRB ,&
&  0.46906E+00_JPRB, 0.46501E+00_JPRB, 0.46055E+00_JPRB, 0.45619E+00_JPRB, 0.45144E+00_JPRB, 0.44765E+00_JPRB, 0.44365E+00_JPRB ,&
&  0.44036E+00_JPRB, 0.43875E+00_JPRB, 0.43706E+00_JPRB, 0.43302E+00_JPRB, 0.42709E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.41301E+00_JPRB, 0.40934E+00_JPRB, 0.40686E+00_JPRB, 0.40512E+00_JPRB, 0.40344E+00_JPRB, 0.40237E+00_JPRB, 0.40215E+00_JPRB ,&
&  0.40233E+00_JPRB, 0.40165E+00_JPRB, 0.40145E+00_JPRB, 0.40122E+00_JPRB, 0.40055E+00_JPRB, 0.40098E+00_JPRB, 0.40106E+00_JPRB ,&
&  0.40164E+00_JPRB, 0.40128E+00_JPRB, 0.40251E+00_JPRB, 0.40192E+00_JPRB, 0.40193E+00_JPRB, 0.40226E+00_JPRB, 0.40170E+00_JPRB ,&
&  0.40224E+00_JPRB, 0.40166E+00_JPRB, 0.39799E+00_JPRB, 0.39395E+00_JPRB, 0.39059E+00_JPRB, 0.37517E+00_JPRB, 0.37505E+00_JPRB ,&
&  0.37896E+00_JPRB, 0.37364E+00_JPRB, 0.37197E+00_JPRB, 0.37043E+00_JPRB, 0.37075E+00_JPRB, 0.37036E+00_JPRB, 0.37299E+00_JPRB ,&
&  0.36602E+00_JPRB, 0.37354E+00_JPRB, 0.37378E+00_JPRB, 0.37392E+00_JPRB, 0.37506E+00_JPRB, 0.37463E+00_JPRB, 0.37953E+00_JPRB ,&
&  0.39739E+00_JPRB, 0.40141E+00_JPRB, 0.40896E+00_JPRB, 0.41557E+00_JPRB, 0.42331E+00_JPRB, 0.43013E+00_JPRB, 0.43662E+00_JPRB ,&
&  0.44134E+00_JPRB, 0.44428E+00_JPRB, 0.44601E+00_JPRB, 0.44607E+00_JPRB, 0.44634E+00_JPRB, 0.44611E+00_JPRB, 0.44642E+00_JPRB ,&
&  0.44674E+00_JPRB, 0.44783E+00_JPRB, 0.44961E+00_JPRB, 0.45133E+00_JPRB, 0.45342E+00_JPRB, 0.45455E+00_JPRB, 0.45572E+00_JPRB ,&
&  0.45634E+00_JPRB, 0.45642E+00_JPRB, 0.45572E+00_JPRB, 0.45541E+00_JPRB, 0.45430E+00_JPRB, 0.45413E+00_JPRB, 0.45352E+00_JPRB ,&
&  0.45375E+00_JPRB, 0.45334E+00_JPRB, 0.45372E+00_JPRB, 0.45357E+00_JPRB, 0.45377E+00_JPRB, 0.45478E+00_JPRB, 0.45521E+00_JPRB ,&
&  0.45669E+00_JPRB, 0.45750E+00_JPRB, 0.45830E+00_JPRB, 0.45895E+00_JPRB, 0.45882E+00_JPRB, 0.45929E+00_JPRB, 0.45922E+00_JPRB ,&
&  0.45997E+00_JPRB, 0.45750E+00_JPRB, 0.45431E+00_JPRB, 0.45022E+00_JPRB, 0.44647E+00_JPRB, 0.44288E+00_JPRB, 0.43917E+00_JPRB ,&
&  0.43547E+00_JPRB, 0.43120E+00_JPRB, 0.42689E+00_JPRB, 0.42217E+00_JPRB, 0.41718E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.41287E+00_JPRB, 0.41062E+00_JPRB, 0.40899E+00_JPRB, 0.40799E+00_JPRB, 0.40738E+00_JPRB, 0.40688E+00_JPRB, 0.40645E+00_JPRB ,&
&  0.40603E+00_JPRB, 0.40630E+00_JPRB, 0.40624E+00_JPRB, 0.40620E+00_JPRB, 0.40602E+00_JPRB, 0.40632E+00_JPRB, 0.40621E+00_JPRB ,&
&  0.40595E+00_JPRB, 0.40507E+00_JPRB, 0.40450E+00_JPRB, 0.40376E+00_JPRB, 0.40415E+00_JPRB, 0.40476E+00_JPRB, 0.40632E+00_JPRB ,&
&  0.40665E+00_JPRB, 0.40674E+00_JPRB, 0.40615E+00_JPRB, 0.40348E+00_JPRB, 0.40073E+00_JPRB, 0.39813E+00_JPRB, 0.39599E+00_JPRB ,&
&  0.39421E+00_JPRB, 0.39309E+00_JPRB, 0.39284E+00_JPRB, 0.39306E+00_JPRB, 0.39481E+00_JPRB, 0.39507E+00_JPRB, 0.39492E+00_JPRB ,&
&  0.39423E+00_JPRB, 0.39208E+00_JPRB, 0.39237E+00_JPRB, 0.39300E+00_JPRB, 0.39417E+00_JPRB, 0.39701E+00_JPRB, 0.39963E+00_JPRB ,&
&  0.40300E+00_JPRB, 0.40695E+00_JPRB, 0.41292E+00_JPRB, 0.41724E+00_JPRB, 0.42172E+00_JPRB, 0.42543E+00_JPRB, 0.42730E+00_JPRB ,&
&  0.42878E+00_JPRB, 0.42999E+00_JPRB, 0.43129E+00_JPRB, 0.43229E+00_JPRB, 0.43325E+00_JPRB, 0.43434E+00_JPRB, 0.43579E+00_JPRB ,&
&  0.43692E+00_JPRB, 0.43808E+00_JPRB, 0.43889E+00_JPRB, 0.43970E+00_JPRB, 0.44064E+00_JPRB, 0.44160E+00_JPRB, 0.44275E+00_JPRB ,&
&  0.44376E+00_JPRB, 0.44380E+00_JPRB, 0.44361E+00_JPRB, 0.44407E+00_JPRB, 0.44456E+00_JPRB, 0.44484E+00_JPRB, 0.44436E+00_JPRB ,&
&  0.44424E+00_JPRB, 0.44405E+00_JPRB, 0.44403E+00_JPRB, 0.44404E+00_JPRB, 0.44390E+00_JPRB, 0.44368E+00_JPRB, 0.44312E+00_JPRB ,&
&  0.44248E+00_JPRB, 0.44152E+00_JPRB, 0.44067E+00_JPRB, 0.44049E+00_JPRB, 0.44003E+00_JPRB, 0.44020E+00_JPRB, 0.44062E+00_JPRB ,&
&  0.44124E+00_JPRB, 0.44037E+00_JPRB, 0.43908E+00_JPRB, 0.43705E+00_JPRB, 0.43487E+00_JPRB, 0.43235E+00_JPRB, 0.42987E+00_JPRB ,&
&  0.42730E+00_JPRB, 0.42448E+00_JPRB, 0.42126E+00_JPRB, 0.41819E+00_JPRB, 0.41539E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.42194E+00_JPRB, 0.42086E+00_JPRB, 0.41987E+00_JPRB, 0.41891E+00_JPRB, 0.41797E+00_JPRB, 0.41726E+00_JPRB, 0.41667E+00_JPRB ,&
&  0.41614E+00_JPRB, 0.41565E+00_JPRB, 0.41507E+00_JPRB, 0.41460E+00_JPRB, 0.41431E+00_JPRB, 0.41396E+00_JPRB, 0.41365E+00_JPRB ,&
&  0.41339E+00_JPRB, 0.41326E+00_JPRB, 0.41322E+00_JPRB, 0.41295E+00_JPRB, 0.41281E+00_JPRB, 0.41265E+00_JPRB, 0.41216E+00_JPRB ,&
&  0.41149E+00_JPRB, 0.41086E+00_JPRB, 0.41015E+00_JPRB, 0.40957E+00_JPRB, 0.40914E+00_JPRB, 0.40844E+00_JPRB, 0.40758E+00_JPRB ,&
&  0.40680E+00_JPRB, 0.40630E+00_JPRB, 0.40602E+00_JPRB, 0.40599E+00_JPRB, 0.40614E+00_JPRB, 0.40620E+00_JPRB, 0.40623E+00_JPRB ,&
&  0.40633E+00_JPRB, 0.40655E+00_JPRB, 0.40693E+00_JPRB, 0.40755E+00_JPRB, 0.40837E+00_JPRB, 0.40942E+00_JPRB, 0.41104E+00_JPRB ,&
&  0.41288E+00_JPRB, 0.41478E+00_JPRB, 0.41671E+00_JPRB, 0.41844E+00_JPRB, 0.41998E+00_JPRB, 0.42108E+00_JPRB, 0.42183E+00_JPRB ,&
&  0.42266E+00_JPRB, 0.42361E+00_JPRB, 0.42462E+00_JPRB, 0.42526E+00_JPRB, 0.42580E+00_JPRB, 0.42627E+00_JPRB, 0.42685E+00_JPRB ,&
&  0.42716E+00_JPRB, 0.42749E+00_JPRB, 0.42788E+00_JPRB, 0.42837E+00_JPRB, 0.42872E+00_JPRB, 0.42901E+00_JPRB, 0.42941E+00_JPRB ,&
&  0.42999E+00_JPRB, 0.43057E+00_JPRB, 0.43094E+00_JPRB, 0.43131E+00_JPRB, 0.43182E+00_JPRB, 0.43246E+00_JPRB, 0.43308E+00_JPRB ,&
&  0.43361E+00_JPRB, 0.43410E+00_JPRB, 0.43457E+00_JPRB, 0.43495E+00_JPRB, 0.43518E+00_JPRB, 0.43518E+00_JPRB, 0.43474E+00_JPRB ,&
&  0.43416E+00_JPRB, 0.43347E+00_JPRB, 0.43302E+00_JPRB, 0.43285E+00_JPRB, 0.43260E+00_JPRB, 0.43244E+00_JPRB, 0.43243E+00_JPRB ,&
&  0.43261E+00_JPRB, 0.43274E+00_JPRB, 0.43273E+00_JPRB, 0.43261E+00_JPRB, 0.43227E+00_JPRB, 0.43158E+00_JPRB, 0.43068E+00_JPRB ,&
&  0.42961E+00_JPRB, 0.42820E+00_JPRB, 0.42670E+00_JPRB, 0.42508E+00_JPRB, 0.42342E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB, 0.42465E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB, 0.12503E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.13014E+00_JPRB, 0.13032E+00_JPRB, 0.13042E+00_JPRB, 0.13049E+00_JPRB, 0.13041E+00_JPRB, 0.13044E+00_JPRB, 0.13045E+00_JPRB ,&
&  0.13047E+00_JPRB, 0.13037E+00_JPRB, 0.13054E+00_JPRB, 0.13064E+00_JPRB, 0.13070E+00_JPRB, 0.13072E+00_JPRB, 0.13062E+00_JPRB ,&
&  0.13040E+00_JPRB, 0.13013E+00_JPRB, 0.12940E+00_JPRB, 0.12928E+00_JPRB, 0.12933E+00_JPRB, 0.12954E+00_JPRB, 0.12967E+00_JPRB ,&
&  0.12989E+00_JPRB, 0.13009E+00_JPRB, 0.13024E+00_JPRB, 0.13069E+00_JPRB, 0.13089E+00_JPRB, 0.13112E+00_JPRB, 0.13135E+00_JPRB ,&
&  0.13157E+00_JPRB, 0.13183E+00_JPRB, 0.13201E+00_JPRB, 0.13220E+00_JPRB, 0.13236E+00_JPRB, 0.13230E+00_JPRB, 0.13211E+00_JPRB ,&
&  0.13184E+00_JPRB, 0.13140E+00_JPRB, 0.13096E+00_JPRB, 0.13055E+00_JPRB, 0.13020E+00_JPRB, 0.12982E+00_JPRB, 0.12946E+00_JPRB ,&
&  0.12907E+00_JPRB, 0.12862E+00_JPRB, 0.12814E+00_JPRB, 0.12770E+00_JPRB, 0.12727E+00_JPRB, 0.12689E+00_JPRB, 0.12646E+00_JPRB ,&
&  0.12611E+00_JPRB, 0.12577E+00_JPRB, 0.12544E+00_JPRB, 0.12511E+00_JPRB, 0.12481E+00_JPRB, 0.12452E+00_JPRB, 0.12423E+00_JPRB ,&
&  0.12393E+00_JPRB, 0.12375E+00_JPRB, 0.12353E+00_JPRB, 0.12332E+00_JPRB, 0.12314E+00_JPRB, 0.12300E+00_JPRB, 0.12282E+00_JPRB ,&
&  0.12265E+00_JPRB, 0.12249E+00_JPRB, 0.12241E+00_JPRB, 0.12229E+00_JPRB, 0.12224E+00_JPRB, 0.12221E+00_JPRB, 0.12227E+00_JPRB ,&
&  0.12232E+00_JPRB, 0.12240E+00_JPRB, 0.12255E+00_JPRB, 0.12277E+00_JPRB, 0.12297E+00_JPRB, 0.12320E+00_JPRB, 0.12343E+00_JPRB ,&
&  0.12369E+00_JPRB, 0.12395E+00_JPRB, 0.12423E+00_JPRB, 0.12450E+00_JPRB, 0.12482E+00_JPRB, 0.12514E+00_JPRB, 0.12550E+00_JPRB ,&
&  0.12583E+00_JPRB, 0.12619E+00_JPRB, 0.12648E+00_JPRB, 0.12682E+00_JPRB, 0.12709E+00_JPRB, 0.12726E+00_JPRB, 0.12742E+00_JPRB ,&
&  0.12770E+00_JPRB, 0.12799E+00_JPRB, 0.12835E+00_JPRB, 0.12867E+00_JPRB, 0.12925E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.14334E+00_JPRB, 0.14571E+00_JPRB, 0.14749E+00_JPRB, 0.14950E+00_JPRB, 0.14902E+00_JPRB, 0.16036E+00_JPRB, 0.15855E+00_JPRB ,&
&  0.15830E+00_JPRB, 0.15294E+00_JPRB, 0.15296E+00_JPRB, 0.15199E+00_JPRB, 0.15050E+00_JPRB, 0.14886E+00_JPRB, 0.14710E+00_JPRB ,&
&  0.14540E+00_JPRB, 0.14331E+00_JPRB, 0.14150E+00_JPRB, 0.13999E+00_JPRB, 0.13862E+00_JPRB, 0.13742E+00_JPRB, 0.13620E+00_JPRB ,&
&  0.13594E+00_JPRB, 0.13646E+00_JPRB, 0.13735E+00_JPRB, 0.13686E+00_JPRB, 0.13727E+00_JPRB, 0.13809E+00_JPRB, 0.13918E+00_JPRB ,&
&  0.14013E+00_JPRB, 0.14138E+00_JPRB, 0.14214E+00_JPRB, 0.14232E+00_JPRB, 0.14237E+00_JPRB, 0.14169E+00_JPRB, 0.14122E+00_JPRB ,&
&  0.14061E+00_JPRB, 0.13979E+00_JPRB, 0.13875E+00_JPRB, 0.13787E+00_JPRB, 0.13687E+00_JPRB, 0.13555E+00_JPRB, 0.13416E+00_JPRB ,&
&  0.13290E+00_JPRB, 0.13180E+00_JPRB, 0.13067E+00_JPRB, 0.12973E+00_JPRB, 0.12883E+00_JPRB, 0.12793E+00_JPRB, 0.12711E+00_JPRB ,&
&  0.12647E+00_JPRB, 0.12590E+00_JPRB, 0.12528E+00_JPRB, 0.12461E+00_JPRB, 0.12385E+00_JPRB, 0.12311E+00_JPRB, 0.12241E+00_JPRB ,&
&  0.12203E+00_JPRB, 0.12168E+00_JPRB, 0.12134E+00_JPRB, 0.12098E+00_JPRB, 0.12081E+00_JPRB, 0.12064E+00_JPRB, 0.12046E+00_JPRB ,&
&  0.12027E+00_JPRB, 0.12002E+00_JPRB, 0.11992E+00_JPRB, 0.11983E+00_JPRB, 0.11974E+00_JPRB, 0.11968E+00_JPRB, 0.11977E+00_JPRB ,&
&  0.11993E+00_JPRB, 0.12015E+00_JPRB, 0.12057E+00_JPRB, 0.12102E+00_JPRB, 0.12151E+00_JPRB, 0.12199E+00_JPRB, 0.12268E+00_JPRB ,&
&  0.12343E+00_JPRB, 0.12415E+00_JPRB, 0.12478E+00_JPRB, 0.12531E+00_JPRB, 0.12609E+00_JPRB, 0.12684E+00_JPRB, 0.12756E+00_JPRB ,&
&  0.12835E+00_JPRB, 0.12937E+00_JPRB, 0.13042E+00_JPRB, 0.13150E+00_JPRB, 0.13273E+00_JPRB, 0.13384E+00_JPRB, 0.13491E+00_JPRB ,&
&  0.13619E+00_JPRB, 0.13849E+00_JPRB, 0.14015E+00_JPRB, 0.14130E+00_JPRB, 0.14260E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.16418E+00_JPRB, 0.16536E+00_JPRB, 0.16637E+00_JPRB, 0.16688E+00_JPRB, 0.16735E+00_JPRB, 0.16739E+00_JPRB, 0.16734E+00_JPRB ,&
&  0.16479E+00_JPRB, 0.16363E+00_JPRB, 0.16202E+00_JPRB, 0.16018E+00_JPRB, 0.15859E+00_JPRB, 0.15682E+00_JPRB, 0.15494E+00_JPRB ,&
&  0.15313E+00_JPRB, 0.15098E+00_JPRB, 0.14859E+00_JPRB, 0.14581E+00_JPRB, 0.14308E+00_JPRB, 0.14050E+00_JPRB, 0.13890E+00_JPRB ,&
&  0.13805E+00_JPRB, 0.13817E+00_JPRB, 0.13943E+00_JPRB, 0.14051E+00_JPRB, 0.14211E+00_JPRB, 0.14416E+00_JPRB, 0.14751E+00_JPRB ,&
&  0.15015E+00_JPRB, 0.15337E+00_JPRB, 0.15556E+00_JPRB, 0.15631E+00_JPRB, 0.15560E+00_JPRB, 0.15523E+00_JPRB, 0.15440E+00_JPRB ,&
&  0.15280E+00_JPRB, 0.15111E+00_JPRB, 0.14964E+00_JPRB, 0.14807E+00_JPRB, 0.14628E+00_JPRB, 0.14399E+00_JPRB, 0.14121E+00_JPRB ,&
&  0.13867E+00_JPRB, 0.13644E+00_JPRB, 0.13440E+00_JPRB, 0.13284E+00_JPRB, 0.13135E+00_JPRB, 0.13002E+00_JPRB, 0.12865E+00_JPRB ,&
&  0.12759E+00_JPRB, 0.12666E+00_JPRB, 0.12549E+00_JPRB, 0.12448E+00_JPRB, 0.12335E+00_JPRB, 0.12240E+00_JPRB, 0.12138E+00_JPRB ,&
&  0.12056E+00_JPRB, 0.11982E+00_JPRB, 0.11921E+00_JPRB, 0.11863E+00_JPRB, 0.11823E+00_JPRB, 0.11784E+00_JPRB, 0.11741E+00_JPRB ,&
&  0.11701E+00_JPRB, 0.11688E+00_JPRB, 0.11681E+00_JPRB, 0.11676E+00_JPRB, 0.11679E+00_JPRB, 0.11651E+00_JPRB, 0.11655E+00_JPRB ,&
&  0.11685E+00_JPRB, 0.11718E+00_JPRB, 0.11736E+00_JPRB, 0.11781E+00_JPRB, 0.11847E+00_JPRB, 0.11927E+00_JPRB, 0.12039E+00_JPRB ,&
&  0.12126E+00_JPRB, 0.12214E+00_JPRB, 0.12296E+00_JPRB, 0.12381E+00_JPRB, 0.12487E+00_JPRB, 0.12602E+00_JPRB, 0.12724E+00_JPRB ,&
&  0.12867E+00_JPRB, 0.13003E+00_JPRB, 0.13145E+00_JPRB, 0.13293E+00_JPRB, 0.13524E+00_JPRB, 0.13763E+00_JPRB, 0.14125E+00_JPRB ,&
&  0.14496E+00_JPRB, 0.14719E+00_JPRB, 0.15954E+00_JPRB, 0.16059E+00_JPRB, 0.16205E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.16465E+00_JPRB, 0.16607E+00_JPRB, 0.16701E+00_JPRB, 0.16818E+00_JPRB, 0.16882E+00_JPRB, 0.17039E+00_JPRB, 0.17079E+00_JPRB ,&
&  0.17094E+00_JPRB, 0.16701E+00_JPRB, 0.16462E+00_JPRB, 0.16204E+00_JPRB, 0.15981E+00_JPRB, 0.15732E+00_JPRB, 0.15488E+00_JPRB ,&
&  0.15295E+00_JPRB, 0.15130E+00_JPRB, 0.14956E+00_JPRB, 0.14711E+00_JPRB, 0.14464E+00_JPRB, 0.14277E+00_JPRB, 0.14131E+00_JPRB ,&
&  0.13987E+00_JPRB, 0.14026E+00_JPRB, 0.14145E+00_JPRB, 0.14415E+00_JPRB, 0.14798E+00_JPRB, 0.15140E+00_JPRB, 0.15583E+00_JPRB ,&
&  0.16194E+00_JPRB, 0.16377E+00_JPRB, 0.16549E+00_JPRB, 0.16636E+00_JPRB, 0.16608E+00_JPRB, 0.16644E+00_JPRB, 0.16393E+00_JPRB ,&
&  0.16417E+00_JPRB, 0.16254E+00_JPRB, 0.16700E+00_JPRB, 0.16480E+00_JPRB, 0.15881E+00_JPRB, 0.15575E+00_JPRB, 0.14996E+00_JPRB ,&
&  0.14536E+00_JPRB, 0.14064E+00_JPRB, 0.13737E+00_JPRB, 0.13548E+00_JPRB, 0.13293E+00_JPRB, 0.13150E+00_JPRB, 0.12940E+00_JPRB ,&
&  0.12748E+00_JPRB, 0.12674E+00_JPRB, 0.12546E+00_JPRB, 0.12486E+00_JPRB, 0.12413E+00_JPRB, 0.12318E+00_JPRB, 0.12214E+00_JPRB ,&
&  0.12143E+00_JPRB, 0.12079E+00_JPRB, 0.12041E+00_JPRB, 0.11992E+00_JPRB, 0.11925E+00_JPRB, 0.11859E+00_JPRB, 0.11787E+00_JPRB ,&
&  0.11736E+00_JPRB, 0.11703E+00_JPRB, 0.11674E+00_JPRB, 0.11657E+00_JPRB, 0.11640E+00_JPRB, 0.11612E+00_JPRB, 0.11615E+00_JPRB ,&
&  0.11635E+00_JPRB, 0.11688E+00_JPRB, 0.11737E+00_JPRB, 0.11801E+00_JPRB, 0.11863E+00_JPRB, 0.11946E+00_JPRB, 0.12012E+00_JPRB ,&
&  0.12081E+00_JPRB, 0.12168E+00_JPRB, 0.12244E+00_JPRB, 0.12333E+00_JPRB, 0.12432E+00_JPRB, 0.12518E+00_JPRB, 0.12648E+00_JPRB ,&
&  0.12781E+00_JPRB, 0.12912E+00_JPRB, 0.13090E+00_JPRB, 0.13280E+00_JPRB, 0.13521E+00_JPRB, 0.13941E+00_JPRB, 0.14430E+00_JPRB ,&
&  0.15039E+00_JPRB, 0.15969E+00_JPRB, 0.16010E+00_JPRB, 0.16218E+00_JPRB, 0.16353E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.16948E+00_JPRB, 0.17139E+00_JPRB, 0.17311E+00_JPRB, 0.17467E+00_JPRB, 0.17501E+00_JPRB, 0.17536E+00_JPRB, 0.17613E+00_JPRB ,&
&  0.17255E+00_JPRB, 0.16985E+00_JPRB, 0.16540E+00_JPRB, 0.16103E+00_JPRB, 0.15692E+00_JPRB, 0.15336E+00_JPRB, 0.15122E+00_JPRB ,&
&  0.15017E+00_JPRB, 0.15019E+00_JPRB, 0.15064E+00_JPRB, 0.15144E+00_JPRB, 0.15111E+00_JPRB, 0.15036E+00_JPRB, 0.14954E+00_JPRB ,&
&  0.14836E+00_JPRB, 0.14832E+00_JPRB, 0.14860E+00_JPRB, 0.15165E+00_JPRB, 0.15598E+00_JPRB, 0.16097E+00_JPRB, 0.16447E+00_JPRB ,&
&  0.16701E+00_JPRB, 0.16889E+00_JPRB, 0.17107E+00_JPRB, 0.17198E+00_JPRB, 0.17269E+00_JPRB, 0.17384E+00_JPRB, 0.17447E+00_JPRB ,&
&  0.17581E+00_JPRB, 0.17614E+00_JPRB, 0.17622E+00_JPRB, 0.17680E+00_JPRB, 0.16903E+00_JPRB, 0.16305E+00_JPRB, 0.15705E+00_JPRB ,&
&  0.15115E+00_JPRB, 0.14524E+00_JPRB, 0.13996E+00_JPRB, 0.13690E+00_JPRB, 0.13403E+00_JPRB, 0.13234E+00_JPRB, 0.13087E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12865E+00_JPRB, 0.12707E+00_JPRB, 0.12553E+00_JPRB, 0.12465E+00_JPRB, 0.12390E+00_JPRB, 0.12296E+00_JPRB ,&
&  0.12287E+00_JPRB, 0.12229E+00_JPRB, 0.12216E+00_JPRB, 0.12176E+00_JPRB, 0.12147E+00_JPRB, 0.12122E+00_JPRB, 0.12123E+00_JPRB ,&
&  0.12156E+00_JPRB, 0.12218E+00_JPRB, 0.12236E+00_JPRB, 0.12258E+00_JPRB, 0.12237E+00_JPRB, 0.12180E+00_JPRB, 0.12127E+00_JPRB ,&
&  0.12076E+00_JPRB, 0.12034E+00_JPRB, 0.12032E+00_JPRB, 0.12025E+00_JPRB, 0.12071E+00_JPRB, 0.12135E+00_JPRB, 0.12200E+00_JPRB ,&
&  0.12286E+00_JPRB, 0.12355E+00_JPRB, 0.12424E+00_JPRB, 0.12504E+00_JPRB, 0.12578E+00_JPRB, 0.12672E+00_JPRB, 0.12780E+00_JPRB ,&
&  0.12867E+00_JPRB, 0.13026E+00_JPRB, 0.13158E+00_JPRB, 0.13317E+00_JPRB, 0.13603E+00_JPRB, 0.13946E+00_JPRB, 0.14713E+00_JPRB ,&
&  0.15511E+00_JPRB, 0.17223E+00_JPRB, 0.17388E+00_JPRB, 0.16811E+00_JPRB, 0.16842E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.17736E+00_JPRB, 0.17966E+00_JPRB, 0.18217E+00_JPRB, 0.18108E+00_JPRB, 0.18118E+00_JPRB, 0.18151E+00_JPRB, 0.18274E+00_JPRB ,&
&  0.18336E+00_JPRB, 0.18308E+00_JPRB, 0.18277E+00_JPRB, 0.18159E+00_JPRB, 0.18013E+00_JPRB, 0.17311E+00_JPRB, 0.16994E+00_JPRB ,&
&  0.16755E+00_JPRB, 0.16786E+00_JPRB, 0.16833E+00_JPRB, 0.17028E+00_JPRB, 0.17067E+00_JPRB, 0.17007E+00_JPRB, 0.16739E+00_JPRB ,&
&  0.16501E+00_JPRB, 0.16331E+00_JPRB, 0.16165E+00_JPRB, 0.16162E+00_JPRB, 0.16340E+00_JPRB, 0.16544E+00_JPRB, 0.16656E+00_JPRB ,&
&  0.16672E+00_JPRB, 0.16834E+00_JPRB, 0.16968E+00_JPRB, 0.17220E+00_JPRB, 0.17941E+00_JPRB, 0.18113E+00_JPRB, 0.18174E+00_JPRB ,&
&  0.18258E+00_JPRB, 0.18397E+00_JPRB, 0.18489E+00_JPRB, 0.18497E+00_JPRB, 0.18351E+00_JPRB, 0.17945E+00_JPRB, 0.17840E+00_JPRB ,&
&  0.16573E+00_JPRB, 0.15825E+00_JPRB, 0.15100E+00_JPRB, 0.14555E+00_JPRB, 0.14094E+00_JPRB, 0.13839E+00_JPRB, 0.13556E+00_JPRB ,&
&  0.13318E+00_JPRB, 0.13063E+00_JPRB, 0.12868E+00_JPRB, 0.12775E+00_JPRB, 0.12748E+00_JPRB, 0.12724E+00_JPRB, 0.12674E+00_JPRB ,&
&  0.12680E+00_JPRB, 0.12641E+00_JPRB, 0.12610E+00_JPRB, 0.12579E+00_JPRB, 0.12560E+00_JPRB, 0.12568E+00_JPRB, 0.12633E+00_JPRB ,&
&  0.12805E+00_JPRB, 0.12976E+00_JPRB, 0.13141E+00_JPRB, 0.13275E+00_JPRB, 0.13222E+00_JPRB, 0.13070E+00_JPRB, 0.12837E+00_JPRB ,&
&  0.12642E+00_JPRB, 0.12517E+00_JPRB, 0.12382E+00_JPRB, 0.12349E+00_JPRB, 0.12402E+00_JPRB, 0.12470E+00_JPRB, 0.12585E+00_JPRB ,&
&  0.12686E+00_JPRB, 0.12767E+00_JPRB, 0.12822E+00_JPRB, 0.12862E+00_JPRB, 0.12925E+00_JPRB, 0.12989E+00_JPRB, 0.13092E+00_JPRB ,&
&  0.13208E+00_JPRB, 0.13351E+00_JPRB, 0.13485E+00_JPRB, 0.13662E+00_JPRB, 0.13917E+00_JPRB, 0.14317E+00_JPRB, 0.14998E+00_JPRB ,&
&  0.15717E+00_JPRB, 0.17609E+00_JPRB, 0.17826E+00_JPRB, 0.17580E+00_JPRB, 0.17685E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.18446E+00_JPRB, 0.18857E+00_JPRB, 0.19106E+00_JPRB, 0.19069E+00_JPRB, 0.19038E+00_JPRB, 0.19110E+00_JPRB, 0.19200E+00_JPRB ,&
&  0.19313E+00_JPRB, 0.19276E+00_JPRB, 0.19146E+00_JPRB, 0.19009E+00_JPRB, 0.18858E+00_JPRB, 0.18659E+00_JPRB, 0.18516E+00_JPRB ,&
&  0.18404E+00_JPRB, 0.18389E+00_JPRB, 0.18333E+00_JPRB, 0.18400E+00_JPRB, 0.18452E+00_JPRB, 0.18713E+00_JPRB, 0.18602E+00_JPRB ,&
&  0.18285E+00_JPRB, 0.18124E+00_JPRB, 0.18341E+00_JPRB, 0.18381E+00_JPRB, 0.18433E+00_JPRB, 0.18514E+00_JPRB, 0.18637E+00_JPRB ,&
&  0.18412E+00_JPRB, 0.18090E+00_JPRB, 0.17586E+00_JPRB, 0.18032E+00_JPRB, 0.18980E+00_JPRB, 0.19030E+00_JPRB, 0.18939E+00_JPRB ,&
&  0.19063E+00_JPRB, 0.19331E+00_JPRB, 0.19462E+00_JPRB, 0.19506E+00_JPRB, 0.19362E+00_JPRB, 0.19287E+00_JPRB, 0.19184E+00_JPRB ,&
&  0.18929E+00_JPRB, 0.18660E+00_JPRB, 0.17886E+00_JPRB, 0.16383E+00_JPRB, 0.15710E+00_JPRB, 0.15372E+00_JPRB, 0.14976E+00_JPRB ,&
&  0.14493E+00_JPRB, 0.14235E+00_JPRB, 0.14195E+00_JPRB, 0.14074E+00_JPRB, 0.14102E+00_JPRB, 0.14053E+00_JPRB, 0.13974E+00_JPRB ,&
&  0.13866E+00_JPRB, 0.13786E+00_JPRB, 0.13641E+00_JPRB, 0.13414E+00_JPRB, 0.13249E+00_JPRB, 0.13206E+00_JPRB, 0.13269E+00_JPRB ,&
&  0.13445E+00_JPRB, 0.13734E+00_JPRB, 0.14523E+00_JPRB, 0.14985E+00_JPRB, 0.14788E+00_JPRB, 0.14355E+00_JPRB, 0.13778E+00_JPRB ,&
&  0.13303E+00_JPRB, 0.13149E+00_JPRB, 0.13044E+00_JPRB, 0.12957E+00_JPRB, 0.12986E+00_JPRB, 0.13084E+00_JPRB, 0.13183E+00_JPRB ,&
&  0.13290E+00_JPRB, 0.13406E+00_JPRB, 0.13475E+00_JPRB, 0.13512E+00_JPRB, 0.13548E+00_JPRB, 0.13571E+00_JPRB, 0.13617E+00_JPRB ,&
&  0.13639E+00_JPRB, 0.13767E+00_JPRB, 0.13887E+00_JPRB, 0.14106E+00_JPRB, 0.14258E+00_JPRB, 0.14442E+00_JPRB, 0.14806E+00_JPRB ,&
&  0.15149E+00_JPRB, 0.16131E+00_JPRB, 0.17555E+00_JPRB, 0.18415E+00_JPRB, 0.18166E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.19453E+00_JPRB, 0.19566E+00_JPRB, 0.19583E+00_JPRB, 0.19502E+00_JPRB, 0.19387E+00_JPRB, 0.19417E+00_JPRB, 0.19562E+00_JPRB ,&
&  0.19851E+00_JPRB, 0.20071E+00_JPRB, 0.20135E+00_JPRB, 0.20150E+00_JPRB, 0.20061E+00_JPRB, 0.19852E+00_JPRB, 0.19670E+00_JPRB ,&
&  0.19525E+00_JPRB, 0.19434E+00_JPRB, 0.19361E+00_JPRB, 0.19333E+00_JPRB, 0.19334E+00_JPRB, 0.19454E+00_JPRB, 0.19584E+00_JPRB ,&
&  0.19976E+00_JPRB, 0.20003E+00_JPRB, 0.19942E+00_JPRB, 0.19918E+00_JPRB, 0.19891E+00_JPRB, 0.19879E+00_JPRB, 0.19871E+00_JPRB ,&
&  0.19655E+00_JPRB, 0.18872E+00_JPRB, 0.18141E+00_JPRB, 0.18624E+00_JPRB, 0.19910E+00_JPRB, 0.19950E+00_JPRB, 0.19941E+00_JPRB ,&
&  0.20070E+00_JPRB, 0.20359E+00_JPRB, 0.20550E+00_JPRB, 0.20653E+00_JPRB, 0.20534E+00_JPRB, 0.20349E+00_JPRB, 0.20291E+00_JPRB ,&
&  0.20301E+00_JPRB, 0.20117E+00_JPRB, 0.19629E+00_JPRB, 0.19085E+00_JPRB, 0.18536E+00_JPRB, 0.18316E+00_JPRB, 0.18138E+00_JPRB ,&
&  0.17903E+00_JPRB, 0.17771E+00_JPRB, 0.17717E+00_JPRB, 0.17665E+00_JPRB, 0.17704E+00_JPRB, 0.17529E+00_JPRB, 0.17312E+00_JPRB ,&
&  0.16914E+00_JPRB, 0.16531E+00_JPRB, 0.16042E+00_JPRB, 0.15524E+00_JPRB, 0.15066E+00_JPRB, 0.14604E+00_JPRB, 0.14285E+00_JPRB ,&
&  0.14213E+00_JPRB, 0.14550E+00_JPRB, 0.15442E+00_JPRB, 0.16464E+00_JPRB, 0.17388E+00_JPRB, 0.16175E+00_JPRB, 0.15437E+00_JPRB ,&
&  0.14724E+00_JPRB, 0.14191E+00_JPRB, 0.13895E+00_JPRB, 0.13820E+00_JPRB, 0.13834E+00_JPRB, 0.13914E+00_JPRB, 0.14052E+00_JPRB ,&
&  0.14224E+00_JPRB, 0.14448E+00_JPRB, 0.14624E+00_JPRB, 0.14738E+00_JPRB, 0.14814E+00_JPRB, 0.14835E+00_JPRB, 0.14817E+00_JPRB ,&
&  0.14793E+00_JPRB, 0.14902E+00_JPRB, 0.15140E+00_JPRB, 0.15541E+00_JPRB, 0.15851E+00_JPRB, 0.16237E+00_JPRB, 0.16792E+00_JPRB ,&
&  0.17577E+00_JPRB, 0.18522E+00_JPRB, 0.18790E+00_JPRB, 0.18958E+00_JPRB, 0.19227E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.20401E+00_JPRB, 0.20524E+00_JPRB, 0.20538E+00_JPRB, 0.20568E+00_JPRB, 0.20621E+00_JPRB, 0.20800E+00_JPRB, 0.21206E+00_JPRB ,&
&  0.21601E+00_JPRB, 0.21787E+00_JPRB, 0.21652E+00_JPRB, 0.21446E+00_JPRB, 0.21339E+00_JPRB, 0.21276E+00_JPRB, 0.21247E+00_JPRB ,&
&  0.21221E+00_JPRB, 0.21117E+00_JPRB, 0.20836E+00_JPRB, 0.20457E+00_JPRB, 0.20194E+00_JPRB, 0.20152E+00_JPRB, 0.20220E+00_JPRB ,&
&  0.20616E+00_JPRB, 0.20896E+00_JPRB, 0.21160E+00_JPRB, 0.21371E+00_JPRB, 0.21408E+00_JPRB, 0.21359E+00_JPRB, 0.21485E+00_JPRB ,&
&  0.21593E+00_JPRB, 0.21399E+00_JPRB, 0.20877E+00_JPRB, 0.21314E+00_JPRB, 0.21507E+00_JPRB, 0.21625E+00_JPRB, 0.21553E+00_JPRB ,&
&  0.21624E+00_JPRB, 0.21832E+00_JPRB, 0.22018E+00_JPRB, 0.22168E+00_JPRB, 0.22095E+00_JPRB, 0.21767E+00_JPRB, 0.21639E+00_JPRB ,&
&  0.21633E+00_JPRB, 0.21517E+00_JPRB, 0.21262E+00_JPRB, 0.21034E+00_JPRB, 0.20781E+00_JPRB, 0.20674E+00_JPRB, 0.20651E+00_JPRB ,&
&  0.20556E+00_JPRB, 0.20339E+00_JPRB, 0.19906E+00_JPRB, 0.19486E+00_JPRB, 0.19190E+00_JPRB, 0.18912E+00_JPRB, 0.18826E+00_JPRB ,&
&  0.18890E+00_JPRB, 0.19021E+00_JPRB, 0.19038E+00_JPRB, 0.18928E+00_JPRB, 0.18539E+00_JPRB, 0.17473E+00_JPRB, 0.16772E+00_JPRB ,&
&  0.16651E+00_JPRB, 0.17130E+00_JPRB, 0.17981E+00_JPRB, 0.18048E+00_JPRB, 0.18224E+00_JPRB, 0.18362E+00_JPRB, 0.18505E+00_JPRB ,&
&  0.18432E+00_JPRB, 0.17077E+00_JPRB, 0.16477E+00_JPRB, 0.16160E+00_JPRB, 0.15846E+00_JPRB, 0.15775E+00_JPRB, 0.15905E+00_JPRB ,&
&  0.16180E+00_JPRB, 0.16579E+00_JPRB, 0.16837E+00_JPRB, 0.16973E+00_JPRB, 0.17002E+00_JPRB, 0.16941E+00_JPRB, 0.16866E+00_JPRB ,&
&  0.16860E+00_JPRB, 0.17216E+00_JPRB, 0.17966E+00_JPRB, 0.18864E+00_JPRB, 0.18959E+00_JPRB, 0.19123E+00_JPRB, 0.19197E+00_JPRB ,&
&  0.19259E+00_JPRB, 0.19436E+00_JPRB, 0.19547E+00_JPRB, 0.19814E+00_JPRB, 0.20140E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.21416E+00_JPRB, 0.21716E+00_JPRB, 0.22019E+00_JPRB, 0.22202E+00_JPRB, 0.22388E+00_JPRB, 0.22573E+00_JPRB, 0.22896E+00_JPRB ,&
&  0.23272E+00_JPRB, 0.23384E+00_JPRB, 0.23219E+00_JPRB, 0.22955E+00_JPRB, 0.22781E+00_JPRB, 0.22699E+00_JPRB, 0.22669E+00_JPRB ,&
&  0.22646E+00_JPRB, 0.22630E+00_JPRB, 0.22558E+00_JPRB, 0.22394E+00_JPRB, 0.22163E+00_JPRB, 0.21938E+00_JPRB, 0.21780E+00_JPRB ,&
&  0.21783E+00_JPRB, 0.21951E+00_JPRB, 0.22207E+00_JPRB, 0.22261E+00_JPRB, 0.22293E+00_JPRB, 0.22419E+00_JPRB, 0.22610E+00_JPRB ,&
&  0.22990E+00_JPRB, 0.23191E+00_JPRB, 0.23420E+00_JPRB, 0.23001E+00_JPRB, 0.23475E+00_JPRB, 0.23640E+00_JPRB, 0.23697E+00_JPRB ,&
&  0.23828E+00_JPRB, 0.23927E+00_JPRB, 0.24011E+00_JPRB, 0.23934E+00_JPRB, 0.23493E+00_JPRB, 0.23269E+00_JPRB, 0.23278E+00_JPRB ,&
&  0.23294E+00_JPRB, 0.23130E+00_JPRB, 0.22851E+00_JPRB, 0.22591E+00_JPRB, 0.22384E+00_JPRB, 0.22240E+00_JPRB, 0.22123E+00_JPRB ,&
&  0.22061E+00_JPRB, 0.21995E+00_JPRB, 0.21822E+00_JPRB, 0.21329E+00_JPRB, 0.20759E+00_JPRB, 0.20390E+00_JPRB, 0.20145E+00_JPRB ,&
&  0.20168E+00_JPRB, 0.20366E+00_JPRB, 0.20532E+00_JPRB, 0.20555E+00_JPRB, 0.20457E+00_JPRB, 0.20278E+00_JPRB, 0.19984E+00_JPRB ,&
&  0.19513E+00_JPRB, 0.19296E+00_JPRB, 0.19210E+00_JPRB, 0.19110E+00_JPRB, 0.19149E+00_JPRB, 0.19317E+00_JPRB, 0.19473E+00_JPRB ,&
&  0.19658E+00_JPRB, 0.19661E+00_JPRB, 0.19526E+00_JPRB, 0.19299E+00_JPRB, 0.19059E+00_JPRB, 0.19040E+00_JPRB, 0.19085E+00_JPRB ,&
&  0.19118E+00_JPRB, 0.19140E+00_JPRB, 0.19198E+00_JPRB, 0.19288E+00_JPRB, 0.19419E+00_JPRB, 0.19628E+00_JPRB, 0.19809E+00_JPRB ,&
&  0.19807E+00_JPRB, 0.19699E+00_JPRB, 0.19795E+00_JPRB, 0.19942E+00_JPRB, 0.20013E+00_JPRB, 0.20062E+00_JPRB, 0.20094E+00_JPRB ,&
&  0.20066E+00_JPRB, 0.20083E+00_JPRB, 0.20272E+00_JPRB, 0.20723E+00_JPRB, 0.21138E+00_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.22560E+00_JPRB, 0.22927E+00_JPRB, 0.23435E+00_JPRB, 0.23584E+00_JPRB, 0.23867E+00_JPRB, 0.24146E+00_JPRB, 0.24494E+00_JPRB ,&
&  0.24706E+00_JPRB, 0.24710E+00_JPRB, 0.24554E+00_JPRB, 0.24401E+00_JPRB, 0.24311E+00_JPRB, 0.24264E+00_JPRB, 0.24195E+00_JPRB ,&
&  0.24165E+00_JPRB, 0.24176E+00_JPRB, 0.24177E+00_JPRB, 0.24174E+00_JPRB, 0.24082E+00_JPRB, 0.23938E+00_JPRB, 0.23779E+00_JPRB ,&
&  0.23493E+00_JPRB, 0.23316E+00_JPRB, 0.23283E+00_JPRB, 0.23299E+00_JPRB, 0.23267E+00_JPRB, 0.23346E+00_JPRB, 0.23532E+00_JPRB ,&
&  0.23912E+00_JPRB, 0.24218E+00_JPRB, 0.24607E+00_JPRB, 0.25011E+00_JPRB, 0.25258E+00_JPRB, 0.25434E+00_JPRB, 0.25639E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25792E+00_JPRB, 0.25781E+00_JPRB, 0.25759E+00_JPRB, 0.25449E+00_JPRB, 0.25291E+00_JPRB, 0.25160E+00_JPRB ,&
&  0.24991E+00_JPRB, 0.24725E+00_JPRB, 0.24368E+00_JPRB, 0.24057E+00_JPRB, 0.23748E+00_JPRB, 0.23531E+00_JPRB, 0.23356E+00_JPRB ,&
&  0.23232E+00_JPRB, 0.23188E+00_JPRB, 0.23215E+00_JPRB, 0.23104E+00_JPRB, 0.22632E+00_JPRB, 0.22158E+00_JPRB, 0.21892E+00_JPRB ,&
&  0.21769E+00_JPRB, 0.21751E+00_JPRB, 0.21893E+00_JPRB, 0.21793E+00_JPRB, 0.21471E+00_JPRB, 0.21227E+00_JPRB, 0.21127E+00_JPRB ,&
&  0.20976E+00_JPRB, 0.20896E+00_JPRB, 0.20947E+00_JPRB, 0.21004E+00_JPRB, 0.21068E+00_JPRB, 0.21098E+00_JPRB, 0.21197E+00_JPRB ,&
&  0.21268E+00_JPRB, 0.21386E+00_JPRB, 0.21354E+00_JPRB, 0.21089E+00_JPRB, 0.20729E+00_JPRB, 0.20461E+00_JPRB, 0.20394E+00_JPRB ,&
&  0.20516E+00_JPRB, 0.20638E+00_JPRB, 0.20659E+00_JPRB, 0.20659E+00_JPRB, 0.20756E+00_JPRB, 0.20993E+00_JPRB, 0.21302E+00_JPRB ,&
&  0.21430E+00_JPRB, 0.21554E+00_JPRB, 0.21731E+00_JPRB, 0.21910E+00_JPRB, 0.22024E+00_JPRB, 0.22082E+00_JPRB, 0.22090E+00_JPRB ,&
&  0.22019E+00_JPRB, 0.22083E+00_JPRB, 0.22228E+00_JPRB, 0.22405E+00_JPRB, 0.22535E+00_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.24377E+00_JPRB, 0.24635E+00_JPRB, 0.24973E+00_JPRB, 0.25065E+00_JPRB, 0.25130E+00_JPRB, 0.25298E+00_JPRB, 0.25608E+00_JPRB ,&
&  0.25818E+00_JPRB, 0.25874E+00_JPRB, 0.25855E+00_JPRB, 0.25819E+00_JPRB, 0.25828E+00_JPRB, 0.25951E+00_JPRB, 0.25996E+00_JPRB ,&
&  0.25922E+00_JPRB, 0.25862E+00_JPRB, 0.25788E+00_JPRB, 0.25744E+00_JPRB, 0.25628E+00_JPRB, 0.25448E+00_JPRB, 0.25307E+00_JPRB ,&
&  0.25103E+00_JPRB, 0.24924E+00_JPRB, 0.24805E+00_JPRB, 0.24726E+00_JPRB, 0.24705E+00_JPRB, 0.24751E+00_JPRB, 0.24935E+00_JPRB ,&
&  0.25154E+00_JPRB, 0.25421E+00_JPRB, 0.25734E+00_JPRB, 0.26091E+00_JPRB, 0.26451E+00_JPRB, 0.26857E+00_JPRB, 0.27214E+00_JPRB ,&
&  0.27492E+00_JPRB, 0.27711E+00_JPRB, 0.27835E+00_JPRB, 0.27519E+00_JPRB, 0.27117E+00_JPRB, 0.26936E+00_JPRB, 0.26796E+00_JPRB ,&
&  0.26519E+00_JPRB, 0.26182E+00_JPRB, 0.25727E+00_JPRB, 0.25462E+00_JPRB, 0.25375E+00_JPRB, 0.25266E+00_JPRB, 0.25135E+00_JPRB ,&
&  0.25052E+00_JPRB, 0.25005E+00_JPRB, 0.24920E+00_JPRB, 0.24757E+00_JPRB, 0.24465E+00_JPRB, 0.24043E+00_JPRB, 0.23611E+00_JPRB ,&
&  0.23336E+00_JPRB, 0.23148E+00_JPRB, 0.23172E+00_JPRB, 0.23003E+00_JPRB, 0.22640E+00_JPRB, 0.22316E+00_JPRB, 0.22292E+00_JPRB ,&
&  0.22298E+00_JPRB, 0.22331E+00_JPRB, 0.22304E+00_JPRB, 0.22194E+00_JPRB, 0.22226E+00_JPRB, 0.22332E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22832E+00_JPRB, 0.23076E+00_JPRB, 0.23041E+00_JPRB, 0.22920E+00_JPRB, 0.22726E+00_JPRB, 0.22673E+00_JPRB, 0.22709E+00_JPRB ,&
&  0.22742E+00_JPRB, 0.22742E+00_JPRB, 0.22750E+00_JPRB, 0.22690E+00_JPRB, 0.22702E+00_JPRB, 0.22849E+00_JPRB, 0.23031E+00_JPRB ,&
&  0.23248E+00_JPRB, 0.23517E+00_JPRB, 0.23752E+00_JPRB, 0.23993E+00_JPRB, 0.24212E+00_JPRB, 0.24314E+00_JPRB, 0.24404E+00_JPRB ,&
&  0.24494E+00_JPRB, 0.24502E+00_JPRB, 0.24464E+00_JPRB, 0.24452E+00_JPRB, 0.24443E+00_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.26815E+00_JPRB, 0.26854E+00_JPRB, 0.26973E+00_JPRB, 0.27042E+00_JPRB, 0.27047E+00_JPRB, 0.27111E+00_JPRB, 0.27363E+00_JPRB ,&
&  0.27584E+00_JPRB, 0.27705E+00_JPRB, 0.27784E+00_JPRB, 0.27785E+00_JPRB, 0.27745E+00_JPRB, 0.27854E+00_JPRB, 0.27944E+00_JPRB ,&
&  0.27944E+00_JPRB, 0.27834E+00_JPRB, 0.27722E+00_JPRB, 0.27591E+00_JPRB, 0.27416E+00_JPRB, 0.27241E+00_JPRB, 0.27115E+00_JPRB ,&
&  0.26938E+00_JPRB, 0.26742E+00_JPRB, 0.26556E+00_JPRB, 0.26526E+00_JPRB, 0.26584E+00_JPRB, 0.26652E+00_JPRB, 0.26783E+00_JPRB ,&
&  0.26910E+00_JPRB, 0.27183E+00_JPRB, 0.27495E+00_JPRB, 0.27844E+00_JPRB, 0.28192E+00_JPRB, 0.28652E+00_JPRB, 0.29157E+00_JPRB ,&
&  0.29628E+00_JPRB, 0.29860E+00_JPRB, 0.29994E+00_JPRB, 0.29752E+00_JPRB, 0.29604E+00_JPRB, 0.29452E+00_JPRB, 0.29344E+00_JPRB ,&
&  0.29219E+00_JPRB, 0.28767E+00_JPRB, 0.28384E+00_JPRB, 0.28322E+00_JPRB, 0.28362E+00_JPRB, 0.28376E+00_JPRB, 0.28349E+00_JPRB ,&
&  0.28239E+00_JPRB, 0.28095E+00_JPRB, 0.27848E+00_JPRB, 0.27505E+00_JPRB, 0.27144E+00_JPRB, 0.26687E+00_JPRB, 0.26004E+00_JPRB ,&
&  0.25355E+00_JPRB, 0.24829E+00_JPRB, 0.24438E+00_JPRB, 0.24103E+00_JPRB, 0.23884E+00_JPRB, 0.23643E+00_JPRB, 0.23580E+00_JPRB ,&
&  0.23617E+00_JPRB, 0.23798E+00_JPRB, 0.23909E+00_JPRB, 0.23849E+00_JPRB, 0.23742E+00_JPRB, 0.23761E+00_JPRB, 0.23952E+00_JPRB ,&
&  0.24306E+00_JPRB, 0.24576E+00_JPRB, 0.24650E+00_JPRB, 0.24588E+00_JPRB, 0.24491E+00_JPRB, 0.24544E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24754E+00_JPRB, 0.24795E+00_JPRB, 0.24908E+00_JPRB, 0.24902E+00_JPRB, 0.25095E+00_JPRB, 0.25417E+00_JPRB, 0.25770E+00_JPRB ,&
&  0.26109E+00_JPRB, 0.26529E+00_JPRB, 0.26949E+00_JPRB, 0.27279E+00_JPRB, 0.27563E+00_JPRB, 0.27568E+00_JPRB, 0.27597E+00_JPRB ,&
&  0.27599E+00_JPRB, 0.27484E+00_JPRB, 0.27367E+00_JPRB, 0.27189E+00_JPRB, 0.26959E+00_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.29584E+00_JPRB, 0.29512E+00_JPRB, 0.29454E+00_JPRB, 0.29453E+00_JPRB, 0.29436E+00_JPRB, 0.29402E+00_JPRB, 0.29397E+00_JPRB ,&
&  0.29447E+00_JPRB, 0.29440E+00_JPRB, 0.29651E+00_JPRB, 0.29691E+00_JPRB, 0.29651E+00_JPRB, 0.29703E+00_JPRB, 0.29745E+00_JPRB ,&
&  0.29608E+00_JPRB, 0.29504E+00_JPRB, 0.29406E+00_JPRB, 0.29218E+00_JPRB, 0.28902E+00_JPRB, 0.28764E+00_JPRB, 0.28665E+00_JPRB ,&
&  0.28474E+00_JPRB, 0.28231E+00_JPRB, 0.28032E+00_JPRB, 0.27973E+00_JPRB, 0.28067E+00_JPRB, 0.28109E+00_JPRB, 0.28239E+00_JPRB ,&
&  0.28330E+00_JPRB, 0.28635E+00_JPRB, 0.27503E+00_JPRB, 0.28450E+00_JPRB, 0.29362E+00_JPRB, 0.30252E+00_JPRB, 0.31190E+00_JPRB ,&
&  0.31840E+00_JPRB, 0.32412E+00_JPRB, 0.32965E+00_JPRB, 0.33000E+00_JPRB, 0.32782E+00_JPRB, 0.32418E+00_JPRB, 0.32242E+00_JPRB ,&
&  0.32145E+00_JPRB, 0.31908E+00_JPRB, 0.31717E+00_JPRB, 0.31666E+00_JPRB, 0.31655E+00_JPRB, 0.31507E+00_JPRB, 0.31291E+00_JPRB ,&
&  0.31107E+00_JPRB, 0.30914E+00_JPRB, 0.30599E+00_JPRB, 0.30236E+00_JPRB, 0.29828E+00_JPRB, 0.29286E+00_JPRB, 0.28520E+00_JPRB ,&
&  0.27624E+00_JPRB, 0.26913E+00_JPRB, 0.26315E+00_JPRB, 0.25940E+00_JPRB, 0.25774E+00_JPRB, 0.25595E+00_JPRB, 0.25422E+00_JPRB ,&
&  0.25384E+00_JPRB, 0.25445E+00_JPRB, 0.25490E+00_JPRB, 0.25497E+00_JPRB, 0.25587E+00_JPRB, 0.25647E+00_JPRB, 0.25865E+00_JPRB ,&
&  0.26098E+00_JPRB, 0.26154E+00_JPRB, 0.26118E+00_JPRB, 0.26106E+00_JPRB, 0.25968E+00_JPRB, 0.26011E+00_JPRB, 0.26166E+00_JPRB ,&
&  0.26350E+00_JPRB, 0.26607E+00_JPRB, 0.26802E+00_JPRB, 0.26972E+00_JPRB, 0.27194E+00_JPRB, 0.27534E+00_JPRB, 0.28049E+00_JPRB ,&
&  0.28537E+00_JPRB, 0.29080E+00_JPRB, 0.29692E+00_JPRB, 0.30207E+00_JPRB, 0.30542E+00_JPRB, 0.30683E+00_JPRB, 0.30787E+00_JPRB ,&
&  0.30718E+00_JPRB, 0.30400E+00_JPRB, 0.30073E+00_JPRB, 0.29862E+00_JPRB, 0.29699E+00_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.33392E+00_JPRB, 0.33165E+00_JPRB, 0.32846E+00_JPRB, 0.32607E+00_JPRB, 0.32354E+00_JPRB, 0.32044E+00_JPRB, 0.31872E+00_JPRB ,&
&  0.31792E+00_JPRB, 0.31775E+00_JPRB, 0.31833E+00_JPRB, 0.31809E+00_JPRB, 0.31774E+00_JPRB, 0.31817E+00_JPRB, 0.31770E+00_JPRB ,&
&  0.31682E+00_JPRB, 0.31557E+00_JPRB, 0.31402E+00_JPRB, 0.31175E+00_JPRB, 0.30884E+00_JPRB, 0.30620E+00_JPRB, 0.30264E+00_JPRB ,&
&  0.30038E+00_JPRB, 0.29940E+00_JPRB, 0.29912E+00_JPRB, 0.29949E+00_JPRB, 0.30119E+00_JPRB, 0.30341E+00_JPRB, 0.30563E+00_JPRB ,&
&  0.30649E+00_JPRB, 0.29570E+00_JPRB, 0.30085E+00_JPRB, 0.30421E+00_JPRB, 0.31272E+00_JPRB, 0.32693E+00_JPRB, 0.33983E+00_JPRB ,&
&  0.34765E+00_JPRB, 0.35483E+00_JPRB, 0.35995E+00_JPRB, 0.36201E+00_JPRB, 0.35957E+00_JPRB, 0.35702E+00_JPRB, 0.35721E+00_JPRB ,&
&  0.35745E+00_JPRB, 0.35639E+00_JPRB, 0.35485E+00_JPRB, 0.35276E+00_JPRB, 0.34979E+00_JPRB, 0.34688E+00_JPRB, 0.34246E+00_JPRB ,&
&  0.33819E+00_JPRB, 0.33543E+00_JPRB, 0.33273E+00_JPRB, 0.32996E+00_JPRB, 0.32539E+00_JPRB, 0.31971E+00_JPRB, 0.31262E+00_JPRB ,&
&  0.30478E+00_JPRB, 0.29918E+00_JPRB, 0.29559E+00_JPRB, 0.29196E+00_JPRB, 0.28827E+00_JPRB, 0.28571E+00_JPRB, 0.28279E+00_JPRB ,&
&  0.28177E+00_JPRB, 0.28260E+00_JPRB, 0.28221E+00_JPRB, 0.28202E+00_JPRB, 0.28259E+00_JPRB, 0.28214E+00_JPRB, 0.28201E+00_JPRB ,&
&  0.28043E+00_JPRB, 0.27865E+00_JPRB, 0.27753E+00_JPRB, 0.27689E+00_JPRB, 0.27644E+00_JPRB, 0.27780E+00_JPRB, 0.28018E+00_JPRB ,&
&  0.28279E+00_JPRB, 0.28651E+00_JPRB, 0.29053E+00_JPRB, 0.29349E+00_JPRB, 0.29606E+00_JPRB, 0.30028E+00_JPRB, 0.30572E+00_JPRB ,&
&  0.31198E+00_JPRB, 0.31917E+00_JPRB, 0.32609E+00_JPRB, 0.33234E+00_JPRB, 0.33562E+00_JPRB, 0.33736E+00_JPRB, 0.33892E+00_JPRB ,&
&  0.33944E+00_JPRB, 0.33647E+00_JPRB, 0.33603E+00_JPRB, 0.33625E+00_JPRB, 0.33538E+00_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.36032E+00_JPRB, 0.36002E+00_JPRB, 0.35793E+00_JPRB, 0.35494E+00_JPRB, 0.35126E+00_JPRB, 0.34928E+00_JPRB, 0.34824E+00_JPRB ,&
&  0.34904E+00_JPRB, 0.34806E+00_JPRB, 0.34842E+00_JPRB, 0.34614E+00_JPRB, 0.34382E+00_JPRB, 0.34217E+00_JPRB, 0.34020E+00_JPRB ,&
&  0.33781E+00_JPRB, 0.33478E+00_JPRB, 0.33186E+00_JPRB, 0.32937E+00_JPRB, 0.32631E+00_JPRB, 0.32215E+00_JPRB, 0.31837E+00_JPRB ,&
&  0.31573E+00_JPRB, 0.31488E+00_JPRB, 0.31464E+00_JPRB, 0.31471E+00_JPRB, 0.31494E+00_JPRB, 0.31676E+00_JPRB, 0.31850E+00_JPRB ,&
&  0.32198E+00_JPRB, 0.29357E+00_JPRB, 0.30325E+00_JPRB, 0.30707E+00_JPRB, 0.31463E+00_JPRB, 0.33122E+00_JPRB, 0.35053E+00_JPRB ,&
&  0.36827E+00_JPRB, 0.38162E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38407E+00_JPRB, 0.38021E+00_JPRB, 0.37984E+00_JPRB ,&
&  0.37839E+00_JPRB, 0.37522E+00_JPRB, 0.37236E+00_JPRB, 0.36920E+00_JPRB, 0.36474E+00_JPRB, 0.35798E+00_JPRB, 0.35085E+00_JPRB ,&
&  0.34700E+00_JPRB, 0.34508E+00_JPRB, 0.34354E+00_JPRB, 0.34183E+00_JPRB, 0.33815E+00_JPRB, 0.33484E+00_JPRB, 0.33204E+00_JPRB ,&
&  0.32937E+00_JPRB, 0.32491E+00_JPRB, 0.32214E+00_JPRB, 0.32020E+00_JPRB, 0.31714E+00_JPRB, 0.31508E+00_JPRB, 0.31357E+00_JPRB ,&
&  0.31203E+00_JPRB, 0.31005E+00_JPRB, 0.30862E+00_JPRB, 0.30772E+00_JPRB, 0.30510E+00_JPRB, 0.30239E+00_JPRB, 0.29939E+00_JPRB ,&
&  0.29784E+00_JPRB, 0.29684E+00_JPRB, 0.29572E+00_JPRB, 0.29559E+00_JPRB, 0.29635E+00_JPRB, 0.29853E+00_JPRB, 0.30161E+00_JPRB ,&
&  0.30531E+00_JPRB, 0.30956E+00_JPRB, 0.31335E+00_JPRB, 0.31624E+00_JPRB, 0.32018E+00_JPRB, 0.32340E+00_JPRB, 0.32783E+00_JPRB ,&
&  0.33342E+00_JPRB, 0.33878E+00_JPRB, 0.34394E+00_JPRB, 0.35000E+00_JPRB, 0.35555E+00_JPRB, 0.35909E+00_JPRB, 0.36235E+00_JPRB ,&
&  0.36522E+00_JPRB, 0.36291E+00_JPRB, 0.36186E+00_JPRB, 0.36007E+00_JPRB, 0.35901E+00_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.39370E+00_JPRB, 0.39285E+00_JPRB, 0.39077E+00_JPRB, 0.38705E+00_JPRB, 0.38331E+00_JPRB, 0.38034E+00_JPRB, 0.37871E+00_JPRB ,&
&  0.37884E+00_JPRB, 0.37954E+00_JPRB, 0.37845E+00_JPRB, 0.37440E+00_JPRB, 0.37066E+00_JPRB, 0.36799E+00_JPRB, 0.36642E+00_JPRB ,&
&  0.36408E+00_JPRB, 0.36028E+00_JPRB, 0.35512E+00_JPRB, 0.35013E+00_JPRB, 0.34454E+00_JPRB, 0.33997E+00_JPRB, 0.33632E+00_JPRB ,&
&  0.33348E+00_JPRB, 0.33259E+00_JPRB, 0.33225E+00_JPRB, 0.33237E+00_JPRB, 0.33381E+00_JPRB, 0.33459E+00_JPRB, 0.34061E+00_JPRB ,&
&  0.31746E+00_JPRB, 0.28717E+00_JPRB, 0.29804E+00_JPRB, 0.30844E+00_JPRB, 0.31893E+00_JPRB, 0.34181E+00_JPRB, 0.37010E+00_JPRB ,&
&  0.39852E+00_JPRB, 0.42222E+00_JPRB, 0.43484E+00_JPRB, 0.43245E+00_JPRB, 0.42285E+00_JPRB, 0.41545E+00_JPRB, 0.41188E+00_JPRB ,&
&  0.40564E+00_JPRB, 0.39815E+00_JPRB, 0.39203E+00_JPRB, 0.38468E+00_JPRB, 0.37680E+00_JPRB, 0.36909E+00_JPRB, 0.36279E+00_JPRB ,&
&  0.35764E+00_JPRB, 0.35472E+00_JPRB, 0.35452E+00_JPRB, 0.35377E+00_JPRB, 0.35330E+00_JPRB, 0.35457E+00_JPRB, 0.35475E+00_JPRB ,&
&  0.35469E+00_JPRB, 0.35337E+00_JPRB, 0.35255E+00_JPRB, 0.35160E+00_JPRB, 0.35136E+00_JPRB, 0.34975E+00_JPRB, 0.34744E+00_JPRB ,&
&  0.34481E+00_JPRB, 0.34267E+00_JPRB, 0.34129E+00_JPRB, 0.33823E+00_JPRB, 0.33150E+00_JPRB, 0.32526E+00_JPRB, 0.32175E+00_JPRB ,&
&  0.32069E+00_JPRB, 0.32055E+00_JPRB, 0.32034E+00_JPRB, 0.32111E+00_JPRB, 0.32255E+00_JPRB, 0.32466E+00_JPRB, 0.32794E+00_JPRB ,&
&  0.33249E+00_JPRB, 0.33727E+00_JPRB, 0.34081E+00_JPRB, 0.34372E+00_JPRB, 0.34843E+00_JPRB, 0.35289E+00_JPRB, 0.35748E+00_JPRB ,&
&  0.36224E+00_JPRB, 0.36718E+00_JPRB, 0.37278E+00_JPRB, 0.38098E+00_JPRB, 0.39038E+00_JPRB, 0.39770E+00_JPRB, 0.40026E+00_JPRB ,&
&  0.39876E+00_JPRB, 0.39244E+00_JPRB, 0.39377E+00_JPRB, 0.39183E+00_JPRB, 0.39182E+00_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.41011E+00_JPRB, 0.41327E+00_JPRB, 0.41232E+00_JPRB, 0.40983E+00_JPRB, 0.40767E+00_JPRB, 0.40403E+00_JPRB, 0.40100E+00_JPRB ,&
&  0.39705E+00_JPRB, 0.39492E+00_JPRB, 0.39055E+00_JPRB, 0.38786E+00_JPRB, 0.38597E+00_JPRB, 0.38469E+00_JPRB, 0.38260E+00_JPRB ,&
&  0.37846E+00_JPRB, 0.37187E+00_JPRB, 0.36515E+00_JPRB, 0.35977E+00_JPRB, 0.35623E+00_JPRB, 0.35267E+00_JPRB, 0.34972E+00_JPRB ,&
&  0.34719E+00_JPRB, 0.34498E+00_JPRB, 0.34295E+00_JPRB, 0.34395E+00_JPRB, 0.34539E+00_JPRB, 0.34595E+00_JPRB, 0.34520E+00_JPRB ,&
&  0.35322E+00_JPRB, 0.30549E+00_JPRB, 0.31384E+00_JPRB, 0.32377E+00_JPRB, 0.33431E+00_JPRB, 0.36058E+00_JPRB, 0.39867E+00_JPRB ,&
&  0.43037E+00_JPRB, 0.45585E+00_JPRB, 0.46442E+00_JPRB, 0.45748E+00_JPRB, 0.44694E+00_JPRB, 0.43991E+00_JPRB, 0.43411E+00_JPRB ,&
&  0.42209E+00_JPRB, 0.41119E+00_JPRB, 0.40145E+00_JPRB, 0.39221E+00_JPRB, 0.38327E+00_JPRB, 0.37606E+00_JPRB, 0.36941E+00_JPRB ,&
&  0.36554E+00_JPRB, 0.36461E+00_JPRB, 0.36383E+00_JPRB, 0.36315E+00_JPRB, 0.36425E+00_JPRB, 0.36735E+00_JPRB, 0.36880E+00_JPRB ,&
&  0.36927E+00_JPRB, 0.37100E+00_JPRB, 0.37117E+00_JPRB, 0.37101E+00_JPRB, 0.37184E+00_JPRB, 0.37554E+00_JPRB, 0.37464E+00_JPRB ,&
&  0.37015E+00_JPRB, 0.36332E+00_JPRB, 0.35704E+00_JPRB, 0.35248E+00_JPRB, 0.34867E+00_JPRB, 0.34491E+00_JPRB, 0.34356E+00_JPRB ,&
&  0.34176E+00_JPRB, 0.34174E+00_JPRB, 0.34198E+00_JPRB, 0.34368E+00_JPRB, 0.34558E+00_JPRB, 0.34827E+00_JPRB, 0.35223E+00_JPRB ,&
&  0.35705E+00_JPRB, 0.36178E+00_JPRB, 0.36742E+00_JPRB, 0.37312E+00_JPRB, 0.37786E+00_JPRB, 0.38101E+00_JPRB, 0.38296E+00_JPRB ,&
&  0.38471E+00_JPRB, 0.38846E+00_JPRB, 0.39076E+00_JPRB, 0.40218E+00_JPRB, 0.41142E+00_JPRB, 0.42124E+00_JPRB, 0.42165E+00_JPRB ,&
&  0.41787E+00_JPRB, 0.41407E+00_JPRB, 0.35769E+00_JPRB, 0.38756E+00_JPRB, 0.40102E+00_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.44853E+00_JPRB, 0.44919E+00_JPRB, 0.44637E+00_JPRB, 0.44411E+00_JPRB, 0.43839E+00_JPRB, 0.43214E+00_JPRB, 0.42655E+00_JPRB ,&
&  0.41933E+00_JPRB, 0.41217E+00_JPRB, 0.40567E+00_JPRB, 0.40348E+00_JPRB, 0.40080E+00_JPRB, 0.39738E+00_JPRB, 0.39087E+00_JPRB ,&
&  0.38258E+00_JPRB, 0.37448E+00_JPRB, 0.36719E+00_JPRB, 0.36292E+00_JPRB, 0.36225E+00_JPRB, 0.36094E+00_JPRB, 0.35863E+00_JPRB ,&
&  0.35775E+00_JPRB, 0.35952E+00_JPRB, 0.35974E+00_JPRB, 0.35984E+00_JPRB, 0.35957E+00_JPRB, 0.35540E+00_JPRB, 0.35608E+00_JPRB ,&
&  0.36623E+00_JPRB, 0.32515E+00_JPRB, 0.34235E+00_JPRB, 0.36420E+00_JPRB, 0.37915E+00_JPRB, 0.40667E+00_JPRB, 0.44801E+00_JPRB ,&
&  0.48781E+00_JPRB, 0.50385E+00_JPRB, 0.50255E+00_JPRB, 0.48521E+00_JPRB, 0.47174E+00_JPRB, 0.46148E+00_JPRB, 0.44435E+00_JPRB ,&
&  0.42701E+00_JPRB, 0.41581E+00_JPRB, 0.40874E+00_JPRB, 0.40307E+00_JPRB, 0.39978E+00_JPRB, 0.39702E+00_JPRB, 0.39484E+00_JPRB ,&
&  0.39123E+00_JPRB, 0.38734E+00_JPRB, 0.38357E+00_JPRB, 0.38182E+00_JPRB, 0.38385E+00_JPRB, 0.38881E+00_JPRB, 0.39510E+00_JPRB ,&
&  0.40002E+00_JPRB, 0.40394E+00_JPRB, 0.40487E+00_JPRB, 0.40567E+00_JPRB, 0.40720E+00_JPRB, 0.40780E+00_JPRB, 0.40372E+00_JPRB ,&
&  0.39472E+00_JPRB, 0.38538E+00_JPRB, 0.37923E+00_JPRB, 0.37578E+00_JPRB, 0.37450E+00_JPRB, 0.37468E+00_JPRB, 0.37614E+00_JPRB ,&
&  0.37570E+00_JPRB, 0.37458E+00_JPRB, 0.37329E+00_JPRB, 0.37380E+00_JPRB, 0.37491E+00_JPRB, 0.37844E+00_JPRB, 0.38349E+00_JPRB ,&
&  0.38896E+00_JPRB, 0.39428E+00_JPRB, 0.40018E+00_JPRB, 0.40593E+00_JPRB, 0.40891E+00_JPRB, 0.41256E+00_JPRB, 0.41357E+00_JPRB ,&
&  0.41369E+00_JPRB, 0.41584E+00_JPRB, 0.42011E+00_JPRB, 0.42439E+00_JPRB, 0.44681E+00_JPRB, 0.45552E+00_JPRB, 0.44682E+00_JPRB ,&
&  0.43906E+00_JPRB, 0.42814E+00_JPRB, 0.43137E+00_JPRB, 0.40131E+00_JPRB, 0.43644E+00_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.49336E+00_JPRB, 0.48846E+00_JPRB, 0.48028E+00_JPRB, 0.47271E+00_JPRB, 0.46238E+00_JPRB, 0.45441E+00_JPRB, 0.44423E+00_JPRB ,&
&  0.43523E+00_JPRB, 0.42626E+00_JPRB, 0.41803E+00_JPRB, 0.41554E+00_JPRB, 0.40972E+00_JPRB, 0.40091E+00_JPRB, 0.38750E+00_JPRB ,&
&  0.37614E+00_JPRB, 0.36695E+00_JPRB, 0.36158E+00_JPRB, 0.35991E+00_JPRB, 0.35922E+00_JPRB, 0.35726E+00_JPRB, 0.35668E+00_JPRB ,&
&  0.35800E+00_JPRB, 0.35808E+00_JPRB, 0.36090E+00_JPRB, 0.36122E+00_JPRB, 0.36529E+00_JPRB, 0.36195E+00_JPRB, 0.35988E+00_JPRB ,&
&  0.37306E+00_JPRB, 0.35416E+00_JPRB, 0.38703E+00_JPRB, 0.42521E+00_JPRB, 0.45965E+00_JPRB, 0.48410E+00_JPRB, 0.51436E+00_JPRB ,&
&  0.53815E+00_JPRB, 0.53932E+00_JPRB, 0.52863E+00_JPRB, 0.50466E+00_JPRB, 0.48663E+00_JPRB, 0.46479E+00_JPRB, 0.44215E+00_JPRB ,&
&  0.43119E+00_JPRB, 0.42687E+00_JPRB, 0.42309E+00_JPRB, 0.41938E+00_JPRB, 0.41475E+00_JPRB, 0.41304E+00_JPRB, 0.41378E+00_JPRB ,&
&  0.41126E+00_JPRB, 0.40737E+00_JPRB, 0.40187E+00_JPRB, 0.40061E+00_JPRB, 0.40061E+00_JPRB, 0.40911E+00_JPRB, 0.42076E+00_JPRB ,&
&  0.42740E+00_JPRB, 0.43435E+00_JPRB, 0.43900E+00_JPRB, 0.44434E+00_JPRB, 0.44338E+00_JPRB, 0.44054E+00_JPRB, 0.42932E+00_JPRB ,&
&  0.42073E+00_JPRB, 0.41539E+00_JPRB, 0.41301E+00_JPRB, 0.41177E+00_JPRB, 0.41132E+00_JPRB, 0.40944E+00_JPRB, 0.40945E+00_JPRB ,&
&  0.40730E+00_JPRB, 0.40446E+00_JPRB, 0.40390E+00_JPRB, 0.40459E+00_JPRB, 0.40546E+00_JPRB, 0.40631E+00_JPRB, 0.40766E+00_JPRB ,&
&  0.40966E+00_JPRB, 0.41107E+00_JPRB, 0.41375E+00_JPRB, 0.41651E+00_JPRB, 0.41752E+00_JPRB, 0.41969E+00_JPRB, 0.42341E+00_JPRB ,&
&  0.42592E+00_JPRB, 0.43120E+00_JPRB, 0.43666E+00_JPRB, 0.45842E+00_JPRB, 0.47199E+00_JPRB, 0.46859E+00_JPRB, 0.45731E+00_JPRB ,&
&  0.44778E+00_JPRB, 0.44391E+00_JPRB, 0.44690E+00_JPRB, 0.46680E+00_JPRB, 0.47066E+00_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.59239E+00_JPRB, 0.55390E+00_JPRB, 0.51829E+00_JPRB, 0.49713E+00_JPRB, 0.48398E+00_JPRB, 0.47505E+00_JPRB, 0.46399E+00_JPRB ,&
&  0.45355E+00_JPRB, 0.44223E+00_JPRB, 0.43318E+00_JPRB, 0.42716E+00_JPRB, 0.42005E+00_JPRB, 0.40608E+00_JPRB, 0.38749E+00_JPRB ,&
&  0.37470E+00_JPRB, 0.37265E+00_JPRB, 0.37113E+00_JPRB, 0.36703E+00_JPRB, 0.36333E+00_JPRB, 0.36299E+00_JPRB, 0.36140E+00_JPRB ,&
&  0.36161E+00_JPRB, 0.36171E+00_JPRB, 0.36460E+00_JPRB, 0.36629E+00_JPRB, 0.36521E+00_JPRB, 0.37208E+00_JPRB, 0.37195E+00_JPRB ,&
&  0.38408E+00_JPRB, 0.37660E+00_JPRB, 0.43826E+00_JPRB, 0.50968E+00_JPRB, 0.55268E+00_JPRB, 0.57879E+00_JPRB, 0.58621E+00_JPRB ,&
&  0.58088E+00_JPRB, 0.57030E+00_JPRB, 0.55175E+00_JPRB, 0.53030E+00_JPRB, 0.50547E+00_JPRB, 0.47629E+00_JPRB, 0.45846E+00_JPRB ,&
&  0.45483E+00_JPRB, 0.44993E+00_JPRB, 0.44354E+00_JPRB, 0.43429E+00_JPRB, 0.42468E+00_JPRB, 0.42339E+00_JPRB, 0.42695E+00_JPRB ,&
&  0.42586E+00_JPRB, 0.42416E+00_JPRB, 0.42198E+00_JPRB, 0.42530E+00_JPRB, 0.43757E+00_JPRB, 0.46157E+00_JPRB, 0.48152E+00_JPRB ,&
&  0.49032E+00_JPRB, 0.50036E+00_JPRB, 0.50314E+00_JPRB, 0.50217E+00_JPRB, 0.49342E+00_JPRB, 0.48287E+00_JPRB, 0.46968E+00_JPRB ,&
&  0.45921E+00_JPRB, 0.45186E+00_JPRB, 0.44913E+00_JPRB, 0.44611E+00_JPRB, 0.44294E+00_JPRB, 0.44027E+00_JPRB, 0.43745E+00_JPRB ,&
&  0.43668E+00_JPRB, 0.43652E+00_JPRB, 0.43757E+00_JPRB, 0.43620E+00_JPRB, 0.43437E+00_JPRB, 0.43147E+00_JPRB, 0.42843E+00_JPRB ,&
&  0.42298E+00_JPRB, 0.41933E+00_JPRB, 0.42345E+00_JPRB, 0.43422E+00_JPRB, 0.44659E+00_JPRB, 0.45507E+00_JPRB, 0.46053E+00_JPRB ,&
&  0.46310E+00_JPRB, 0.47084E+00_JPRB, 0.46124E+00_JPRB, 0.48037E+00_JPRB, 0.52558E+00_JPRB, 0.51318E+00_JPRB, 0.50236E+00_JPRB ,&
&  0.48644E+00_JPRB, 0.47902E+00_JPRB, 0.48024E+00_JPRB, 0.50269E+00_JPRB, 0.58634E+00_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.54367E+00_JPRB, 0.51007E+00_JPRB, 0.49341E+00_JPRB, 0.48424E+00_JPRB, 0.48068E+00_JPRB, 0.47870E+00_JPRB, 0.47046E+00_JPRB ,&
&  0.45802E+00_JPRB, 0.44658E+00_JPRB, 0.43858E+00_JPRB, 0.43268E+00_JPRB, 0.42030E+00_JPRB, 0.40122E+00_JPRB, 0.38727E+00_JPRB ,&
&  0.38527E+00_JPRB, 0.38320E+00_JPRB, 0.38206E+00_JPRB, 0.37759E+00_JPRB, 0.37292E+00_JPRB, 0.37135E+00_JPRB, 0.36903E+00_JPRB ,&
&  0.36555E+00_JPRB, 0.36166E+00_JPRB, 0.36234E+00_JPRB, 0.36581E+00_JPRB, 0.36962E+00_JPRB, 0.37402E+00_JPRB, 0.37985E+00_JPRB ,&
&  0.39043E+00_JPRB, 0.43682E+00_JPRB, 0.54381E+00_JPRB, 0.68810E+00_JPRB, 0.72126E+00_JPRB, 0.71149E+00_JPRB, 0.70414E+00_JPRB ,&
&  0.67294E+00_JPRB, 0.63409E+00_JPRB, 0.59800E+00_JPRB, 0.56207E+00_JPRB, 0.52028E+00_JPRB, 0.49832E+00_JPRB, 0.48869E+00_JPRB ,&
&  0.48127E+00_JPRB, 0.47058E+00_JPRB, 0.45842E+00_JPRB, 0.44544E+00_JPRB, 0.43712E+00_JPRB, 0.44139E+00_JPRB, 0.45107E+00_JPRB ,&
&  0.45046E+00_JPRB, 0.45225E+00_JPRB, 0.45404E+00_JPRB, 0.46758E+00_JPRB, 0.50609E+00_JPRB, 0.54426E+00_JPRB, 0.57411E+00_JPRB ,&
&  0.58376E+00_JPRB, 0.61054E+00_JPRB, 0.59184E+00_JPRB, 0.57085E+00_JPRB, 0.54879E+00_JPRB, 0.52927E+00_JPRB, 0.51301E+00_JPRB ,&
&  0.50233E+00_JPRB, 0.49428E+00_JPRB, 0.48821E+00_JPRB, 0.48270E+00_JPRB, 0.47706E+00_JPRB, 0.47153E+00_JPRB, 0.47135E+00_JPRB ,&
&  0.47198E+00_JPRB, 0.46704E+00_JPRB, 0.46257E+00_JPRB, 0.45734E+00_JPRB, 0.44813E+00_JPRB, 0.43940E+00_JPRB, 0.43263E+00_JPRB ,&
&  0.42591E+00_JPRB, 0.42595E+00_JPRB, 0.44513E+00_JPRB, 0.46759E+00_JPRB, 0.48282E+00_JPRB, 0.49499E+00_JPRB, 0.50254E+00_JPRB ,&
&  0.50507E+00_JPRB, 0.50875E+00_JPRB, 0.50190E+00_JPRB, 0.50955E+00_JPRB, 0.53195E+00_JPRB, 0.56077E+00_JPRB, 0.54009E+00_JPRB ,&
&  0.52529E+00_JPRB, 0.51892E+00_JPRB, 0.51836E+00_JPRB, 0.52210E+00_JPRB, 0.57499E+00_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.51928E+00_JPRB, 0.49882E+00_JPRB, 0.48935E+00_JPRB, 0.48360E+00_JPRB, 0.47974E+00_JPRB, 0.47686E+00_JPRB, 0.46883E+00_JPRB ,&
&  0.45620E+00_JPRB, 0.44862E+00_JPRB, 0.44006E+00_JPRB, 0.42732E+00_JPRB, 0.40949E+00_JPRB, 0.39973E+00_JPRB, 0.40013E+00_JPRB ,&
&  0.39958E+00_JPRB, 0.39651E+00_JPRB, 0.39112E+00_JPRB, 0.38935E+00_JPRB, 0.38571E+00_JPRB, 0.38012E+00_JPRB, 0.37726E+00_JPRB ,&
&  0.36920E+00_JPRB, 0.36532E+00_JPRB, 0.36668E+00_JPRB, 0.37143E+00_JPRB, 0.37590E+00_JPRB, 0.38336E+00_JPRB, 0.39266E+00_JPRB ,&
&  0.41744E+00_JPRB, 0.51739E+00_JPRB, 0.76938E+00_JPRB, 0.93862E+00_JPRB, 0.98369E+00_JPRB, 0.94559E+00_JPRB, 0.93974E+00_JPRB ,&
&  0.85262E+00_JPRB, 0.78022E+00_JPRB, 0.70985E+00_JPRB, 0.63486E+00_JPRB, 0.57781E+00_JPRB, 0.54955E+00_JPRB, 0.53299E+00_JPRB ,&
&  0.51294E+00_JPRB, 0.49644E+00_JPRB, 0.48109E+00_JPRB, 0.46859E+00_JPRB, 0.46562E+00_JPRB, 0.47416E+00_JPRB, 0.48390E+00_JPRB ,&
&  0.48767E+00_JPRB, 0.49189E+00_JPRB, 0.50155E+00_JPRB, 0.52548E+00_JPRB, 0.55025E+00_JPRB, 0.56915E+00_JPRB, 0.67045E+00_JPRB ,&
&  0.79494E+00_JPRB, 0.78453E+00_JPRB, 0.73084E+00_JPRB, 0.68120E+00_JPRB, 0.63673E+00_JPRB, 0.60044E+00_JPRB, 0.57260E+00_JPRB ,&
&  0.55817E+00_JPRB, 0.54593E+00_JPRB, 0.53790E+00_JPRB, 0.52852E+00_JPRB, 0.52031E+00_JPRB, 0.51620E+00_JPRB, 0.51207E+00_JPRB ,&
&  0.50449E+00_JPRB, 0.49666E+00_JPRB, 0.48678E+00_JPRB, 0.47180E+00_JPRB, 0.45492E+00_JPRB, 0.44547E+00_JPRB, 0.44202E+00_JPRB ,&
&  0.44280E+00_JPRB, 0.45951E+00_JPRB, 0.48227E+00_JPRB, 0.49792E+00_JPRB, 0.52599E+00_JPRB, 0.54690E+00_JPRB, 0.56194E+00_JPRB ,&
&  0.54075E+00_JPRB, 0.53460E+00_JPRB, 0.54417E+00_JPRB, 0.55745E+00_JPRB, 0.58223E+00_JPRB, 0.61478E+00_JPRB, 0.58607E+00_JPRB ,&
&  0.57219E+00_JPRB, 0.57043E+00_JPRB, 0.56192E+00_JPRB, 0.54644E+00_JPRB, 0.54730E+00_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.51514E+00_JPRB, 0.50313E+00_JPRB, 0.49286E+00_JPRB, 0.48831E+00_JPRB, 0.48426E+00_JPRB, 0.47462E+00_JPRB, 0.46555E+00_JPRB ,&
&  0.45657E+00_JPRB, 0.44664E+00_JPRB, 0.43315E+00_JPRB, 0.41307E+00_JPRB, 0.40038E+00_JPRB, 0.39853E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40666E+00_JPRB, 0.40011E+00_JPRB, 0.39697E+00_JPRB, 0.39702E+00_JPRB, 0.39656E+00_JPRB, 0.39108E+00_JPRB, 0.38353E+00_JPRB ,&
&  0.37698E+00_JPRB, 0.37723E+00_JPRB, 0.37935E+00_JPRB, 0.38374E+00_JPRB, 0.38752E+00_JPRB, 0.39467E+00_JPRB, 0.40788E+00_JPRB ,&
&  0.44926E+00_JPRB, 0.44440E+00_JPRB, 0.92351E+00_JPRB, 0.12430E+01_JPRB, 0.12944E+01_JPRB, 0.12501E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.11137E+01_JPRB, 0.99282E+00_JPRB, 0.87404E+00_JPRB, 0.75916E+00_JPRB, 0.67248E+00_JPRB, 0.61606E+00_JPRB, 0.57569E+00_JPRB ,&
&  0.54769E+00_JPRB, 0.52616E+00_JPRB, 0.50794E+00_JPRB, 0.49958E+00_JPRB, 0.50386E+00_JPRB, 0.50799E+00_JPRB, 0.50992E+00_JPRB ,&
&  0.51543E+00_JPRB, 0.52439E+00_JPRB, 0.54483E+00_JPRB, 0.58192E+00_JPRB, 0.61572E+00_JPRB, 0.69088E+00_JPRB, 0.83919E+00_JPRB ,&
&  0.97754E+00_JPRB, 0.10141E+01_JPRB, 0.90426E+00_JPRB, 0.80459E+00_JPRB, 0.73869E+00_JPRB, 0.69543E+00_JPRB, 0.66285E+00_JPRB ,&
&  0.63566E+00_JPRB, 0.61500E+00_JPRB, 0.60056E+00_JPRB, 0.57858E+00_JPRB, 0.56460E+00_JPRB, 0.55605E+00_JPRB, 0.53977E+00_JPRB ,&
&  0.52226E+00_JPRB, 0.50620E+00_JPRB, 0.48926E+00_JPRB, 0.47377E+00_JPRB, 0.46062E+00_JPRB, 0.45951E+00_JPRB, 0.46364E+00_JPRB ,&
&  0.47270E+00_JPRB, 0.49346E+00_JPRB, 0.50766E+00_JPRB, 0.52645E+00_JPRB, 0.55353E+00_JPRB, 0.57200E+00_JPRB, 0.58103E+00_JPRB ,&
&  0.56466E+00_JPRB, 0.57043E+00_JPRB, 0.58807E+00_JPRB, 0.60976E+00_JPRB, 0.58810E+00_JPRB, 0.65096E+00_JPRB, 0.62948E+00_JPRB ,&
&  0.62400E+00_JPRB, 0.61161E+00_JPRB, 0.57360E+00_JPRB, 0.55581E+00_JPRB, 0.53690E+00_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.52486E+00_JPRB, 0.50936E+00_JPRB, 0.49649E+00_JPRB, 0.48561E+00_JPRB, 0.47734E+00_JPRB, 0.47254E+00_JPRB, 0.46569E+00_JPRB ,&
&  0.45717E+00_JPRB, 0.44508E+00_JPRB, 0.43379E+00_JPRB, 0.41824E+00_JPRB, 0.41203E+00_JPRB, 0.41480E+00_JPRB, 0.42276E+00_JPRB ,&
&  0.41818E+00_JPRB, 0.40609E+00_JPRB, 0.40517E+00_JPRB, 0.40600E+00_JPRB, 0.40287E+00_JPRB, 0.39589E+00_JPRB, 0.38886E+00_JPRB ,&
&  0.39111E+00_JPRB, 0.39339E+00_JPRB, 0.39544E+00_JPRB, 0.39855E+00_JPRB, 0.40261E+00_JPRB, 0.41281E+00_JPRB, 0.43314E+00_JPRB ,&
&  0.49209E+00_JPRB, 0.73089E+00_JPRB, 0.84749E+00_JPRB, 0.15112E+01_JPRB, 0.16921E+01_JPRB, 0.16467E+01_JPRB, 0.14587E+01_JPRB ,&
&  0.12750E+01_JPRB, 0.11768E+01_JPRB, 0.10337E+01_JPRB, 0.90326E+00_JPRB, 0.77657E+00_JPRB, 0.68483E+00_JPRB, 0.62974E+00_JPRB ,&
&  0.59324E+00_JPRB, 0.56544E+00_JPRB, 0.54959E+00_JPRB, 0.54555E+00_JPRB, 0.54226E+00_JPRB, 0.54020E+00_JPRB, 0.54112E+00_JPRB ,&
&  0.54777E+00_JPRB, 0.56428E+00_JPRB, 0.59587E+00_JPRB, 0.65034E+00_JPRB, 0.66458E+00_JPRB, 0.82203E+00_JPRB, 0.10361E+01_JPRB ,&
&  0.11412E+01_JPRB, 0.12063E+01_JPRB, 0.10538E+01_JPRB, 0.91260E+00_JPRB, 0.83104E+00_JPRB, 0.77386E+00_JPRB, 0.72561E+00_JPRB ,&
&  0.69220E+00_JPRB, 0.66608E+00_JPRB, 0.63762E+00_JPRB, 0.60211E+00_JPRB, 0.58420E+00_JPRB, 0.56994E+00_JPRB, 0.54463E+00_JPRB ,&
&  0.51792E+00_JPRB, 0.50049E+00_JPRB, 0.49030E+00_JPRB, 0.48310E+00_JPRB, 0.48010E+00_JPRB, 0.48597E+00_JPRB, 0.49551E+00_JPRB ,&
&  0.50840E+00_JPRB, 0.53429E+00_JPRB, 0.53815E+00_JPRB, 0.54906E+00_JPRB, 0.57402E+00_JPRB, 0.59863E+00_JPRB, 0.58968E+00_JPRB ,&
&  0.58762E+00_JPRB, 0.59649E+00_JPRB, 0.61500E+00_JPRB, 0.63161E+00_JPRB, 0.61723E+00_JPRB, 0.67926E+00_JPRB, 0.65483E+00_JPRB ,&
&  0.65517E+00_JPRB, 0.61448E+00_JPRB, 0.57882E+00_JPRB, 0.56603E+00_JPRB, 0.54839E+00_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.52872E+00_JPRB, 0.50266E+00_JPRB, 0.48492E+00_JPRB, 0.47582E+00_JPRB, 0.47139E+00_JPRB, 0.47045E+00_JPRB, 0.46303E+00_JPRB ,&
&  0.45776E+00_JPRB, 0.44662E+00_JPRB, 0.43243E+00_JPRB, 0.42086E+00_JPRB, 0.42710E+00_JPRB, 0.43129E+00_JPRB, 0.43129E+00_JPRB ,&
&  0.42449E+00_JPRB, 0.41997E+00_JPRB, 0.41807E+00_JPRB, 0.41490E+00_JPRB, 0.40954E+00_JPRB, 0.40471E+00_JPRB, 0.40351E+00_JPRB ,&
&  0.40892E+00_JPRB, 0.40877E+00_JPRB, 0.40816E+00_JPRB, 0.41249E+00_JPRB, 0.42264E+00_JPRB, 0.44543E+00_JPRB, 0.47715E+00_JPRB ,&
&  0.56434E+00_JPRB, 0.69237E+00_JPRB, 0.85521E+00_JPRB, 0.17029E+01_JPRB, 0.20666E+01_JPRB, 0.19803E+01_JPRB, 0.16834E+01_JPRB ,&
&  0.13998E+01_JPRB, 0.13441E+01_JPRB, 0.11853E+01_JPRB, 0.10000E+01_JPRB, 0.83446E+00_JPRB, 0.74052E+00_JPRB, 0.67576E+00_JPRB ,&
&  0.63100E+00_JPRB, 0.60675E+00_JPRB, 0.59904E+00_JPRB, 0.59657E+00_JPRB, 0.59090E+00_JPRB, 0.59337E+00_JPRB, 0.59690E+00_JPRB ,&
&  0.60127E+00_JPRB, 0.61856E+00_JPRB, 0.64828E+00_JPRB, 0.65729E+00_JPRB, 0.75069E+00_JPRB, 0.10311E+01_JPRB, 0.12452E+01_JPRB ,&
&  0.13053E+01_JPRB, 0.12822E+01_JPRB, 0.11355E+01_JPRB, 0.97269E+00_JPRB, 0.86785E+00_JPRB, 0.82098E+00_JPRB, 0.76902E+00_JPRB ,&
&  0.71445E+00_JPRB, 0.67876E+00_JPRB, 0.63307E+00_JPRB, 0.60848E+00_JPRB, 0.60136E+00_JPRB, 0.56340E+00_JPRB, 0.53351E+00_JPRB ,&
&  0.51919E+00_JPRB, 0.51361E+00_JPRB, 0.51046E+00_JPRB, 0.50816E+00_JPRB, 0.51213E+00_JPRB, 0.52039E+00_JPRB, 0.53430E+00_JPRB ,&
&  0.55427E+00_JPRB, 0.58486E+00_JPRB, 0.57951E+00_JPRB, 0.59023E+00_JPRB, 0.62296E+00_JPRB, 0.64585E+00_JPRB, 0.63627E+00_JPRB ,&
&  0.62647E+00_JPRB, 0.62801E+00_JPRB, 0.63627E+00_JPRB, 0.62248E+00_JPRB, 0.61745E+00_JPRB, 0.67442E+00_JPRB, 0.67622E+00_JPRB ,&
&  0.68646E+00_JPRB, 0.63716E+00_JPRB, 0.61109E+00_JPRB, 0.57630E+00_JPRB, 0.55460E+00_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.51337E+00_JPRB, 0.48829E+00_JPRB, 0.48311E+00_JPRB, 0.47971E+00_JPRB, 0.46939E+00_JPRB, 0.46087E+00_JPRB, 0.45169E+00_JPRB ,&
&  0.44458E+00_JPRB, 0.43147E+00_JPRB, 0.42285E+00_JPRB, 0.43020E+00_JPRB, 0.44677E+00_JPRB, 0.45729E+00_JPRB, 0.44955E+00_JPRB ,&
&  0.44055E+00_JPRB, 0.43438E+00_JPRB, 0.43764E+00_JPRB, 0.43838E+00_JPRB, 0.43426E+00_JPRB, 0.43556E+00_JPRB, 0.43405E+00_JPRB ,&
&  0.43489E+00_JPRB, 0.43199E+00_JPRB, 0.43472E+00_JPRB, 0.44735E+00_JPRB, 0.46955E+00_JPRB, 0.50756E+00_JPRB, 0.56167E+00_JPRB ,&
&  0.70510E+00_JPRB, 0.76742E+00_JPRB, 0.91846E+00_JPRB, 0.15433E+01_JPRB, 0.19862E+01_JPRB, 0.18598E+01_JPRB, 0.16830E+01_JPRB ,&
&  0.14392E+01_JPRB, 0.11281E+01_JPRB, 0.10942E+01_JPRB, 0.10082E+01_JPRB, 0.87016E+00_JPRB, 0.76451E+00_JPRB, 0.70090E+00_JPRB ,&
&  0.66679E+00_JPRB, 0.64397E+00_JPRB, 0.63958E+00_JPRB, 0.64708E+00_JPRB, 0.66062E+00_JPRB, 0.67649E+00_JPRB, 0.68136E+00_JPRB ,&
&  0.68170E+00_JPRB, 0.68660E+00_JPRB, 0.70990E+00_JPRB, 0.73526E+00_JPRB, 0.92622E+00_JPRB, 0.12788E+01_JPRB, 0.14241E+01_JPRB ,&
&  0.14049E+01_JPRB, 0.13043E+01_JPRB, 0.11290E+01_JPRB, 0.96660E+00_JPRB, 0.81078E+00_JPRB, 0.81970E+00_JPRB, 0.76315E+00_JPRB ,&
&  0.70200E+00_JPRB, 0.65878E+00_JPRB, 0.63672E+00_JPRB, 0.61389E+00_JPRB, 0.58206E+00_JPRB, 0.55123E+00_JPRB, 0.53513E+00_JPRB ,&
&  0.53622E+00_JPRB, 0.54306E+00_JPRB, 0.54418E+00_JPRB, 0.54050E+00_JPRB, 0.55152E+00_JPRB, 0.55910E+00_JPRB, 0.57835E+00_JPRB ,&
&  0.59915E+00_JPRB, 0.62633E+00_JPRB, 0.63057E+00_JPRB, 0.65914E+00_JPRB, 0.69301E+00_JPRB, 0.71326E+00_JPRB, 0.70003E+00_JPRB ,&
&  0.66604E+00_JPRB, 0.67860E+00_JPRB, 0.68451E+00_JPRB, 0.66735E+00_JPRB, 0.66904E+00_JPRB, 0.67527E+00_JPRB, 0.67779E+00_JPRB ,&
&  0.70836E+00_JPRB, 0.70272E+00_JPRB, 0.66723E+00_JPRB, 0.57980E+00_JPRB, 0.54366E+00_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.50421E+00_JPRB, 0.48694E+00_JPRB, 0.48638E+00_JPRB, 0.47941E+00_JPRB, 0.46774E+00_JPRB, 0.46105E+00_JPRB, 0.45372E+00_JPRB ,&
&  0.44703E+00_JPRB, 0.44740E+00_JPRB, 0.45136E+00_JPRB, 0.48002E+00_JPRB, 0.49258E+00_JPRB, 0.48487E+00_JPRB, 0.47564E+00_JPRB ,&
&  0.46951E+00_JPRB, 0.46950E+00_JPRB, 0.47136E+00_JPRB, 0.47226E+00_JPRB, 0.47157E+00_JPRB, 0.47660E+00_JPRB, 0.48340E+00_JPRB ,&
&  0.48225E+00_JPRB, 0.48455E+00_JPRB, 0.49601E+00_JPRB, 0.51982E+00_JPRB, 0.56455E+00_JPRB, 0.63452E+00_JPRB, 0.74551E+00_JPRB ,&
&  0.95900E+00_JPRB, 0.92790E+00_JPRB, 0.48277E+00_JPRB, 0.94599E+00_JPRB, 0.15383E+01_JPRB, 0.16125E+01_JPRB, 0.15296E+01_JPRB ,&
&  0.13497E+01_JPRB, 0.10346E+01_JPRB, 0.95706E+00_JPRB, 0.97576E+00_JPRB, 0.85271E+00_JPRB, 0.76119E+00_JPRB, 0.71122E+00_JPRB ,&
&  0.68294E+00_JPRB, 0.68281E+00_JPRB, 0.71252E+00_JPRB, 0.73855E+00_JPRB, 0.76311E+00_JPRB, 0.77091E+00_JPRB, 0.76523E+00_JPRB ,&
&  0.75890E+00_JPRB, 0.76765E+00_JPRB, 0.81876E+00_JPRB, 0.87227E+00_JPRB, 0.12153E+01_JPRB, 0.14061E+01_JPRB, 0.14292E+01_JPRB ,&
&  0.11609E+01_JPRB, 0.10987E+01_JPRB, 0.10149E+01_JPRB, 0.90854E+00_JPRB, 0.79554E+00_JPRB, 0.72627E+00_JPRB, 0.73162E+00_JPRB ,&
&  0.66741E+00_JPRB, 0.62372E+00_JPRB, 0.61265E+00_JPRB, 0.59647E+00_JPRB, 0.57707E+00_JPRB, 0.56578E+00_JPRB, 0.56200E+00_JPRB ,&
&  0.57248E+00_JPRB, 0.58273E+00_JPRB, 0.58796E+00_JPRB, 0.58848E+00_JPRB, 0.59394E+00_JPRB, 0.60269E+00_JPRB, 0.62155E+00_JPRB ,&
&  0.63744E+00_JPRB, 0.66998E+00_JPRB, 0.71103E+00_JPRB, 0.71553E+00_JPRB, 0.73445E+00_JPRB, 0.77549E+00_JPRB, 0.76187E+00_JPRB ,&
&  0.73108E+00_JPRB, 0.75383E+00_JPRB, 0.72528E+00_JPRB, 0.67274E+00_JPRB, 0.68637E+00_JPRB, 0.67876E+00_JPRB, 0.69554E+00_JPRB ,&
&  0.75831E+00_JPRB, 0.89545E+00_JPRB, 0.87238E+00_JPRB, 0.57204E+00_JPRB, 0.52193E+00_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.47044E+00_JPRB, 0.46003E+00_JPRB, 0.45449E+00_JPRB, 0.46032E+00_JPRB, 0.46376E+00_JPRB, 0.46854E+00_JPRB, 0.45601E+00_JPRB ,&
&  0.46632E+00_JPRB, 0.49690E+00_JPRB, 0.52231E+00_JPRB, 0.53729E+00_JPRB, 0.53215E+00_JPRB, 0.52213E+00_JPRB, 0.51835E+00_JPRB ,&
&  0.51757E+00_JPRB, 0.52107E+00_JPRB, 0.53242E+00_JPRB, 0.53737E+00_JPRB, 0.54007E+00_JPRB, 0.54777E+00_JPRB, 0.55418E+00_JPRB ,&
&  0.56125E+00_JPRB, 0.57621E+00_JPRB, 0.60360E+00_JPRB, 0.65462E+00_JPRB, 0.73660E+00_JPRB, 0.86020E+00_JPRB, 0.10125E+01_JPRB ,&
&  0.12282E+01_JPRB, 0.86041E+00_JPRB, 0.50215E+00_JPRB, 0.12117E+01_JPRB, 0.15434E+01_JPRB, 0.15360E+01_JPRB, 0.13231E+01_JPRB ,&
&  0.11843E+01_JPRB, 0.99384E+00_JPRB, 0.90122E+00_JPRB, 0.93928E+00_JPRB, 0.81344E+00_JPRB, 0.76521E+00_JPRB, 0.74925E+00_JPRB ,&
&  0.75988E+00_JPRB, 0.78933E+00_JPRB, 0.81820E+00_JPRB, 0.83602E+00_JPRB, 0.83617E+00_JPRB, 0.82346E+00_JPRB, 0.81633E+00_JPRB ,&
&  0.83937E+00_JPRB, 0.88552E+00_JPRB, 0.98252E+00_JPRB, 0.11004E+01_JPRB, 0.14078E+01_JPRB, 0.13232E+01_JPRB, 0.12753E+01_JPRB ,&
&  0.97010E+00_JPRB, 0.93270E+00_JPRB, 0.91145E+00_JPRB, 0.86403E+00_JPRB, 0.76581E+00_JPRB, 0.66770E+00_JPRB, 0.67873E+00_JPRB ,&
&  0.62841E+00_JPRB, 0.60187E+00_JPRB, 0.59799E+00_JPRB, 0.58499E+00_JPRB, 0.57395E+00_JPRB, 0.57880E+00_JPRB, 0.59566E+00_JPRB ,&
&  0.62112E+00_JPRB, 0.62949E+00_JPRB, 0.63124E+00_JPRB, 0.62870E+00_JPRB, 0.63168E+00_JPRB, 0.63926E+00_JPRB, 0.64808E+00_JPRB ,&
&  0.66962E+00_JPRB, 0.70254E+00_JPRB, 0.73844E+00_JPRB, 0.75219E+00_JPRB, 0.75409E+00_JPRB, 0.77565E+00_JPRB, 0.81102E+00_JPRB ,&
&  0.77780E+00_JPRB, 0.78161E+00_JPRB, 0.73638E+00_JPRB, 0.71737E+00_JPRB, 0.68028E+00_JPRB, 0.67725E+00_JPRB, 0.67742E+00_JPRB ,&
&  0.66341E+00_JPRB, 0.61108E+00_JPRB, 0.54991E+00_JPRB, 0.48164E+00_JPRB, 0.47182E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.46157E+00_JPRB, 0.45224E+00_JPRB, 0.44827E+00_JPRB, 0.46322E+00_JPRB, 0.46022E+00_JPRB, 0.47048E+00_JPRB, 0.48463E+00_JPRB ,&
&  0.52514E+00_JPRB, 0.56272E+00_JPRB, 0.59352E+00_JPRB, 0.58592E+00_JPRB, 0.58476E+00_JPRB, 0.57352E+00_JPRB, 0.57841E+00_JPRB ,&
&  0.58860E+00_JPRB, 0.60624E+00_JPRB, 0.62550E+00_JPRB, 0.63295E+00_JPRB, 0.63879E+00_JPRB, 0.64702E+00_JPRB, 0.66009E+00_JPRB ,&
&  0.68025E+00_JPRB, 0.70914E+00_JPRB, 0.75924E+00_JPRB, 0.84670E+00_JPRB, 0.98099E+00_JPRB, 0.11059E+01_JPRB, 0.11918E+01_JPRB ,&
&  0.82779E+00_JPRB, 0.40436E+00_JPRB, 0.11462E+01_JPRB, 0.15234E+01_JPRB, 0.14803E+01_JPRB, 0.13911E+01_JPRB, 0.12930E+01_JPRB ,&
&  0.10727E+01_JPRB, 0.95409E+00_JPRB, 0.86480E+00_JPRB, 0.92427E+00_JPRB, 0.83053E+00_JPRB, 0.81264E+00_JPRB, 0.82585E+00_JPRB ,&
&  0.85012E+00_JPRB, 0.87153E+00_JPRB, 0.88851E+00_JPRB, 0.88444E+00_JPRB, 0.86945E+00_JPRB, 0.86659E+00_JPRB, 0.88449E+00_JPRB ,&
&  0.93400E+00_JPRB, 0.10293E+01_JPRB, 0.11411E+01_JPRB, 0.11306E+01_JPRB, 0.12417E+01_JPRB, 0.11615E+01_JPRB, 0.11473E+01_JPRB ,&
&  0.92984E+00_JPRB, 0.80734E+00_JPRB, 0.77718E+00_JPRB, 0.81495E+00_JPRB, 0.74752E+00_JPRB, 0.63112E+00_JPRB, 0.63457E+00_JPRB ,&
&  0.60205E+00_JPRB, 0.59001E+00_JPRB, 0.59692E+00_JPRB, 0.58914E+00_JPRB, 0.57957E+00_JPRB, 0.59600E+00_JPRB, 0.61898E+00_JPRB ,&
&  0.64449E+00_JPRB, 0.65505E+00_JPRB, 0.64651E+00_JPRB, 0.64159E+00_JPRB, 0.64511E+00_JPRB, 0.64634E+00_JPRB, 0.64916E+00_JPRB ,&
&  0.67385E+00_JPRB, 0.70210E+00_JPRB, 0.73740E+00_JPRB, 0.77481E+00_JPRB, 0.82021E+00_JPRB, 0.82178E+00_JPRB, 0.80429E+00_JPRB ,&
&  0.77214E+00_JPRB, 0.75639E+00_JPRB, 0.72174E+00_JPRB, 0.70949E+00_JPRB, 0.69443E+00_JPRB, 0.67049E+00_JPRB, 0.64709E+00_JPRB ,&
&  0.60149E+00_JPRB, 0.52685E+00_JPRB, 0.48501E+00_JPRB, 0.47824E+00_JPRB, 0.46796E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.48736E+00_JPRB, 0.47857E+00_JPRB, 0.48182E+00_JPRB, 0.48942E+00_JPRB, 0.50236E+00_JPRB, 0.52427E+00_JPRB, 0.56018E+00_JPRB ,&
&  0.60201E+00_JPRB, 0.62356E+00_JPRB, 0.64163E+00_JPRB, 0.62964E+00_JPRB, 0.63033E+00_JPRB, 0.64393E+00_JPRB, 0.65115E+00_JPRB ,&
&  0.67540E+00_JPRB, 0.70160E+00_JPRB, 0.71578E+00_JPRB, 0.71814E+00_JPRB, 0.72530E+00_JPRB, 0.74795E+00_JPRB, 0.77978E+00_JPRB ,&
&  0.81874E+00_JPRB, 0.87102E+00_JPRB, 0.94279E+00_JPRB, 0.10436E+01_JPRB, 0.11581E+01_JPRB, 0.11948E+01_JPRB, 0.11818E+01_JPRB ,&
&  0.62826E+00_JPRB, 0.11547E+01_JPRB, 0.15234E+01_JPRB, 0.14947E+01_JPRB, 0.14016E+01_JPRB, 0.13084E+01_JPRB, 0.12127E+01_JPRB ,&
&  0.10424E+01_JPRB, 0.91173E+00_JPRB, 0.87974E+00_JPRB, 0.94173E+00_JPRB, 0.87649E+00_JPRB, 0.87756E+00_JPRB, 0.89286E+00_JPRB ,&
&  0.91347E+00_JPRB, 0.92811E+00_JPRB, 0.92718E+00_JPRB, 0.92047E+00_JPRB, 0.92946E+00_JPRB, 0.95576E+00_JPRB, 0.99380E+00_JPRB ,&
&  0.10463E+01_JPRB, 0.11278E+01_JPRB, 0.12001E+01_JPRB, 0.11084E+01_JPRB, 0.11053E+01_JPRB, 0.11087E+01_JPRB, 0.11398E+01_JPRB ,&
&  0.89595E+00_JPRB, 0.74185E+00_JPRB, 0.74439E+00_JPRB, 0.78063E+00_JPRB, 0.71102E+00_JPRB, 0.64654E+00_JPRB, 0.61369E+00_JPRB ,&
&  0.59588E+00_JPRB, 0.57467E+00_JPRB, 0.56702E+00_JPRB, 0.56179E+00_JPRB, 0.57764E+00_JPRB, 0.59552E+00_JPRB, 0.62077E+00_JPRB ,&
&  0.62890E+00_JPRB, 0.62784E+00_JPRB, 0.61675E+00_JPRB, 0.61077E+00_JPRB, 0.61614E+00_JPRB, 0.62480E+00_JPRB, 0.62519E+00_JPRB ,&
&  0.64454E+00_JPRB, 0.65302E+00_JPRB, 0.67923E+00_JPRB, 0.72568E+00_JPRB, 0.78562E+00_JPRB, 0.80935E+00_JPRB, 0.80347E+00_JPRB ,&
&  0.74789E+00_JPRB, 0.74154E+00_JPRB, 0.74736E+00_JPRB, 0.73213E+00_JPRB, 0.71824E+00_JPRB, 0.67167E+00_JPRB, 0.58258E+00_JPRB ,&
&  0.54269E+00_JPRB, 0.50537E+00_JPRB, 0.49230E+00_JPRB, 0.48737E+00_JPRB, 0.49892E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.55532E+00_JPRB, 0.54271E+00_JPRB, 0.54931E+00_JPRB, 0.56521E+00_JPRB, 0.57739E+00_JPRB, 0.60282E+00_JPRB, 0.63501E+00_JPRB ,&
&  0.67669E+00_JPRB, 0.70212E+00_JPRB, 0.71031E+00_JPRB, 0.70867E+00_JPRB, 0.71500E+00_JPRB, 0.71378E+00_JPRB, 0.72423E+00_JPRB ,&
&  0.74576E+00_JPRB, 0.76843E+00_JPRB, 0.79756E+00_JPRB, 0.82058E+00_JPRB, 0.83469E+00_JPRB, 0.87411E+00_JPRB, 0.91616E+00_JPRB ,&
&  0.97083E+00_JPRB, 0.10293E+01_JPRB, 0.11040E+01_JPRB, 0.11893E+01_JPRB, 0.12805E+01_JPRB, 0.13170E+01_JPRB, 0.13092E+01_JPRB ,&
&  0.10769E+01_JPRB, 0.15519E+01_JPRB, 0.15638E+01_JPRB, 0.13952E+01_JPRB, 0.12384E+01_JPRB, 0.11728E+01_JPRB, 0.11043E+01_JPRB ,&
&  0.10366E+01_JPRB, 0.93679E+00_JPRB, 0.90560E+00_JPRB, 0.90940E+00_JPRB, 0.92730E+00_JPRB, 0.92324E+00_JPRB, 0.94591E+00_JPRB ,&
&  0.96966E+00_JPRB, 0.98095E+00_JPRB, 0.98509E+00_JPRB, 0.99022E+00_JPRB, 0.10066E+01_JPRB, 0.10385E+01_JPRB, 0.10851E+01_JPRB ,&
&  0.11243E+01_JPRB, 0.11707E+01_JPRB, 0.12295E+01_JPRB, 0.11518E+01_JPRB, 0.11133E+01_JPRB, 0.12190E+01_JPRB, 0.11799E+01_JPRB ,&
&  0.87200E+00_JPRB, 0.68975E+00_JPRB, 0.71693E+00_JPRB, 0.75276E+00_JPRB, 0.69418E+00_JPRB, 0.63497E+00_JPRB, 0.59754E+00_JPRB ,&
&  0.56943E+00_JPRB, 0.54001E+00_JPRB, 0.54341E+00_JPRB, 0.54926E+00_JPRB, 0.55678E+00_JPRB, 0.56083E+00_JPRB, 0.57511E+00_JPRB ,&
&  0.58808E+00_JPRB, 0.57951E+00_JPRB, 0.56863E+00_JPRB, 0.56132E+00_JPRB, 0.57092E+00_JPRB, 0.58882E+00_JPRB, 0.61084E+00_JPRB ,&
&  0.66370E+00_JPRB, 0.62769E+00_JPRB, 0.64357E+00_JPRB, 0.69035E+00_JPRB, 0.73876E+00_JPRB, 0.79481E+00_JPRB, 0.84566E+00_JPRB ,&
&  0.88311E+00_JPRB, 0.92118E+00_JPRB, 0.97466E+00_JPRB, 0.98650E+00_JPRB, 0.91831E+00_JPRB, 0.83441E+00_JPRB, 0.59741E+00_JPRB ,&
&  0.58575E+00_JPRB, 0.57564E+00_JPRB, 0.57950E+00_JPRB, 0.57845E+00_JPRB, 0.57609E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.67396E+00_JPRB, 0.67046E+00_JPRB, 0.69304E+00_JPRB, 0.71555E+00_JPRB, 0.73509E+00_JPRB, 0.75352E+00_JPRB, 0.76658E+00_JPRB ,&
&  0.78552E+00_JPRB, 0.79944E+00_JPRB, 0.79934E+00_JPRB, 0.79198E+00_JPRB, 0.80244E+00_JPRB, 0.79893E+00_JPRB, 0.81760E+00_JPRB ,&
&  0.84607E+00_JPRB, 0.87997E+00_JPRB, 0.92303E+00_JPRB, 0.94582E+00_JPRB, 0.96435E+00_JPRB, 0.99984E+00_JPRB, 0.10307E+01_JPRB ,&
&  0.10668E+01_JPRB, 0.11148E+01_JPRB, 0.11611E+01_JPRB, 0.12168E+01_JPRB, 0.12656E+01_JPRB, 0.12504E+01_JPRB, 0.98071E+00_JPRB ,&
&  0.14284E+01_JPRB, 0.16536E+01_JPRB, 0.15398E+01_JPRB, 0.12979E+01_JPRB, 0.11563E+01_JPRB, 0.10956E+01_JPRB, 0.10746E+01_JPRB ,&
&  0.10477E+01_JPRB, 0.98433E+00_JPRB, 0.92376E+00_JPRB, 0.90016E+00_JPRB, 0.98234E+00_JPRB, 0.95687E+00_JPRB, 0.98225E+00_JPRB ,&
&  0.10134E+01_JPRB, 0.10340E+01_JPRB, 0.10524E+01_JPRB, 0.10679E+01_JPRB, 0.10763E+01_JPRB, 0.10905E+01_JPRB, 0.11078E+01_JPRB ,&
&  0.11303E+01_JPRB, 0.11521E+01_JPRB, 0.12174E+01_JPRB, 0.11134E+01_JPRB, 0.11002E+01_JPRB, 0.11927E+01_JPRB, 0.11423E+01_JPRB ,&
&  0.87756E+00_JPRB, 0.69405E+00_JPRB, 0.70442E+00_JPRB, 0.72925E+00_JPRB, 0.69099E+00_JPRB, 0.62635E+00_JPRB, 0.59593E+00_JPRB ,&
&  0.57245E+00_JPRB, 0.55326E+00_JPRB, 0.54766E+00_JPRB, 0.53657E+00_JPRB, 0.54069E+00_JPRB, 0.55400E+00_JPRB, 0.55847E+00_JPRB ,&
&  0.57756E+00_JPRB, 0.56421E+00_JPRB, 0.55861E+00_JPRB, 0.55954E+00_JPRB, 0.58589E+00_JPRB, 0.63669E+00_JPRB, 0.74915E+00_JPRB ,&
&  0.86907E+00_JPRB, 0.56137E+00_JPRB, 0.62293E+00_JPRB, 0.67520E+00_JPRB, 0.75223E+00_JPRB, 0.87941E+00_JPRB, 0.10776E+01_JPRB ,&
&  0.14154E+01_JPRB, 0.18903E+01_JPRB, 0.18486E+01_JPRB, 0.14687E+01_JPRB, 0.18739E+01_JPRB, 0.15647E+01_JPRB, 0.68281E+00_JPRB ,&
&  0.68664E+00_JPRB, 0.66848E+00_JPRB, 0.67376E+00_JPRB, 0.67924E+00_JPRB, 0.68054E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.76068E+00_JPRB, 0.77761E+00_JPRB, 0.79683E+00_JPRB, 0.81399E+00_JPRB, 0.82335E+00_JPRB, 0.83169E+00_JPRB, 0.84266E+00_JPRB ,&
&  0.84689E+00_JPRB, 0.84579E+00_JPRB, 0.85442E+00_JPRB, 0.86935E+00_JPRB, 0.88862E+00_JPRB, 0.90835E+00_JPRB, 0.93448E+00_JPRB ,&
&  0.96955E+00_JPRB, 0.10008E+01_JPRB, 0.10405E+01_JPRB, 0.10637E+01_JPRB, 0.10725E+01_JPRB, 0.10819E+01_JPRB, 0.10863E+01_JPRB ,&
&  0.11067E+01_JPRB, 0.11180E+01_JPRB, 0.11321E+01_JPRB, 0.11719E+01_JPRB, 0.11977E+01_JPRB, 0.12185E+01_JPRB, 0.10187E+01_JPRB ,&
&  0.13377E+01_JPRB, 0.15718E+01_JPRB, 0.14102E+01_JPRB, 0.11948E+01_JPRB, 0.10838E+01_JPRB, 0.10604E+01_JPRB, 0.10465E+01_JPRB ,&
&  0.10430E+01_JPRB, 0.10207E+01_JPRB, 0.93606E+00_JPRB, 0.95348E+00_JPRB, 0.97014E+00_JPRB, 0.95779E+00_JPRB, 0.98543E+00_JPRB ,&
&  0.10172E+01_JPRB, 0.10395E+01_JPRB, 0.10707E+01_JPRB, 0.11033E+01_JPRB, 0.11171E+01_JPRB, 0.11366E+01_JPRB, 0.11282E+01_JPRB ,&
&  0.11236E+01_JPRB, 0.11312E+01_JPRB, 0.11622E+01_JPRB, 0.10627E+01_JPRB, 0.10514E+01_JPRB, 0.10431E+01_JPRB, 0.91366E+00_JPRB ,&
&  0.80152E+00_JPRB, 0.70793E+00_JPRB, 0.67876E+00_JPRB, 0.75390E+00_JPRB, 0.73236E+00_JPRB, 0.66847E+00_JPRB, 0.62577E+00_JPRB ,&
&  0.59260E+00_JPRB, 0.56225E+00_JPRB, 0.57199E+00_JPRB, 0.56005E+00_JPRB, 0.52053E+00_JPRB, 0.53540E+00_JPRB, 0.55440E+00_JPRB ,&
&  0.54020E+00_JPRB, 0.55081E+00_JPRB, 0.56905E+00_JPRB, 0.56189E+00_JPRB, 0.56521E+00_JPRB, 0.58399E+00_JPRB, 0.56811E+00_JPRB ,&
&  0.62827E+00_JPRB, 0.59843E+00_JPRB, 0.62278E+00_JPRB, 0.68640E+00_JPRB, 0.69421E+00_JPRB, 0.90776E+00_JPRB, 0.12582E+01_JPRB ,&
&  0.18466E+01_JPRB, 0.20207E+01_JPRB, 0.26460E+01_JPRB, 0.41372E+01_JPRB, 0.31343E+01_JPRB, 0.20686E+01_JPRB, 0.77422E+00_JPRB ,&
&  0.77209E+00_JPRB, 0.76245E+00_JPRB, 0.77389E+00_JPRB, 0.76720E+00_JPRB, 0.76537E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.76638E+00_JPRB, 0.78895E+00_JPRB, 0.80697E+00_JPRB, 0.82474E+00_JPRB, 0.84003E+00_JPRB, 0.85408E+00_JPRB, 0.87057E+00_JPRB ,&
&  0.88096E+00_JPRB, 0.89737E+00_JPRB, 0.91775E+00_JPRB, 0.93808E+00_JPRB, 0.95407E+00_JPRB, 0.97876E+00_JPRB, 0.99897E+00_JPRB ,&
&  0.10289E+01_JPRB, 0.10482E+01_JPRB, 0.10642E+01_JPRB, 0.10625E+01_JPRB, 0.10558E+01_JPRB, 0.10594E+01_JPRB, 0.10585E+01_JPRB ,&
&  0.10476E+01_JPRB, 0.10452E+01_JPRB, 0.10525E+01_JPRB, 0.10715E+01_JPRB, 0.10771E+01_JPRB, 0.11095E+01_JPRB, 0.94443E+00_JPRB ,&
&  0.12286E+01_JPRB, 0.13638E+01_JPRB, 0.12593E+01_JPRB, 0.10975E+01_JPRB, 0.10077E+01_JPRB, 0.99405E+00_JPRB, 0.99017E+00_JPRB ,&
&  0.99657E+00_JPRB, 0.10222E+01_JPRB, 0.93820E+00_JPRB, 0.89652E+00_JPRB, 0.88436E+00_JPRB, 0.88432E+00_JPRB, 0.90297E+00_JPRB ,&
&  0.92680E+00_JPRB, 0.95356E+00_JPRB, 0.98603E+00_JPRB, 0.10146E+01_JPRB, 0.10272E+01_JPRB, 0.10282E+01_JPRB, 0.10085E+01_JPRB ,&
&  0.97485E+00_JPRB, 0.94926E+00_JPRB, 0.89361E+00_JPRB, 0.88560E+00_JPRB, 0.87046E+00_JPRB, 0.76898E+00_JPRB, 0.65238E+00_JPRB ,&
&  0.60943E+00_JPRB, 0.67438E+00_JPRB, 0.72100E+00_JPRB, 0.84879E+00_JPRB, 0.80374E+00_JPRB, 0.74003E+00_JPRB, 0.68372E+00_JPRB ,&
&  0.63055E+00_JPRB, 0.58848E+00_JPRB, 0.58176E+00_JPRB, 0.55675E+00_JPRB, 0.54235E+00_JPRB, 0.53699E+00_JPRB, 0.53023E+00_JPRB ,&
&  0.53873E+00_JPRB, 0.54510E+00_JPRB, 0.55733E+00_JPRB, 0.56078E+00_JPRB, 0.54932E+00_JPRB, 0.49070E+00_JPRB, 0.60302E+00_JPRB ,&
&  0.58839E+00_JPRB, 0.54041E+00_JPRB, 0.63665E+00_JPRB, 0.50564E+00_JPRB, 0.61958E+00_JPRB, 0.68489E+00_JPRB, 0.81554E+00_JPRB ,&
&  0.90595E+00_JPRB, 0.12792E+01_JPRB, 0.17703E+01_JPRB, 0.14025E+01_JPRB, 0.11152E+01_JPRB, 0.83041E+00_JPRB, 0.73653E+00_JPRB ,&
&  0.73141E+00_JPRB, 0.73214E+00_JPRB, 0.73836E+00_JPRB, 0.73851E+00_JPRB, 0.74629E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.76998E+00_JPRB, 0.78965E+00_JPRB, 0.80430E+00_JPRB, 0.82168E+00_JPRB, 0.83931E+00_JPRB, 0.85572E+00_JPRB, 0.87172E+00_JPRB ,&
&  0.88506E+00_JPRB, 0.89613E+00_JPRB, 0.90534E+00_JPRB, 0.91890E+00_JPRB, 0.93642E+00_JPRB, 0.95054E+00_JPRB, 0.96877E+00_JPRB ,&
&  0.99692E+00_JPRB, 0.10186E+01_JPRB, 0.10406E+01_JPRB, 0.10349E+01_JPRB, 0.10310E+01_JPRB, 0.10456E+01_JPRB, 0.10609E+01_JPRB ,&
&  0.10653E+01_JPRB, 0.10774E+01_JPRB, 0.10997E+01_JPRB, 0.11252E+01_JPRB, 0.11563E+01_JPRB, 0.12075E+01_JPRB, 0.11184E+01_JPRB ,&
&  0.11468E+01_JPRB, 0.12203E+01_JPRB, 0.11416E+01_JPRB, 0.99118E+00_JPRB, 0.92133E+00_JPRB, 0.88989E+00_JPRB, 0.87657E+00_JPRB ,&
&  0.86789E+00_JPRB, 0.83360E+00_JPRB, 0.80770E+00_JPRB, 0.78125E+00_JPRB, 0.77109E+00_JPRB, 0.77421E+00_JPRB, 0.77912E+00_JPRB ,&
&  0.78638E+00_JPRB, 0.79515E+00_JPRB, 0.81576E+00_JPRB, 0.84072E+00_JPRB, 0.85969E+00_JPRB, 0.84142E+00_JPRB, 0.81398E+00_JPRB ,&
&  0.76997E+00_JPRB, 0.72998E+00_JPRB, 0.61037E+00_JPRB, 0.57708E+00_JPRB, 0.55439E+00_JPRB, 0.50651E+00_JPRB, 0.45721E+00_JPRB ,&
&  0.50740E+00_JPRB, 0.58843E+00_JPRB, 0.73258E+00_JPRB, 0.89529E+00_JPRB, 0.83590E+00_JPRB, 0.78226E+00_JPRB, 0.72216E+00_JPRB ,&
&  0.65695E+00_JPRB, 0.60849E+00_JPRB, 0.58714E+00_JPRB, 0.55795E+00_JPRB, 0.54951E+00_JPRB, 0.55660E+00_JPRB, 0.53200E+00_JPRB ,&
&  0.53022E+00_JPRB, 0.52325E+00_JPRB, 0.52261E+00_JPRB, 0.51932E+00_JPRB, 0.53391E+00_JPRB, 0.47627E+00_JPRB, 0.56144E+00_JPRB ,&
&  0.57220E+00_JPRB, 0.45213E+00_JPRB, 0.49291E+00_JPRB, 0.58333E+00_JPRB, 0.57564E+00_JPRB, 0.59280E+00_JPRB, 0.67525E+00_JPRB ,&
&  0.82928E+00_JPRB, 0.93953E+00_JPRB, 0.95550E+00_JPRB, 0.89597E+00_JPRB, 0.79933E+00_JPRB, 0.74206E+00_JPRB, 0.71288E+00_JPRB ,&
&  0.72040E+00_JPRB, 0.72787E+00_JPRB, 0.73789E+00_JPRB, 0.73966E+00_JPRB, 0.74877E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.75460E+00_JPRB, 0.77584E+00_JPRB, 0.78807E+00_JPRB, 0.80061E+00_JPRB, 0.81677E+00_JPRB, 0.82445E+00_JPRB, 0.83088E+00_JPRB ,&
&  0.83276E+00_JPRB, 0.82938E+00_JPRB, 0.83553E+00_JPRB, 0.84822E+00_JPRB, 0.85439E+00_JPRB, 0.85234E+00_JPRB, 0.85779E+00_JPRB ,&
&  0.86976E+00_JPRB, 0.88917E+00_JPRB, 0.90902E+00_JPRB, 0.90074E+00_JPRB, 0.89408E+00_JPRB, 0.90317E+00_JPRB, 0.91750E+00_JPRB ,&
&  0.91630E+00_JPRB, 0.91218E+00_JPRB, 0.91875E+00_JPRB, 0.92717E+00_JPRB, 0.94329E+00_JPRB, 0.94551E+00_JPRB, 0.10442E+01_JPRB ,&
&  0.85151E+00_JPRB, 0.11028E+01_JPRB, 0.10494E+01_JPRB, 0.89226E+00_JPRB, 0.86217E+00_JPRB, 0.79557E+00_JPRB, 0.77363E+00_JPRB ,&
&  0.75706E+00_JPRB, 0.73087E+00_JPRB, 0.70349E+00_JPRB, 0.68022E+00_JPRB, 0.67343E+00_JPRB, 0.67819E+00_JPRB, 0.67094E+00_JPRB ,&
&  0.66966E+00_JPRB, 0.67963E+00_JPRB, 0.69127E+00_JPRB, 0.70197E+00_JPRB, 0.70261E+00_JPRB, 0.68977E+00_JPRB, 0.65929E+00_JPRB ,&
&  0.63444E+00_JPRB, 0.59403E+00_JPRB, 0.45759E+00_JPRB, 0.43159E+00_JPRB, 0.43382E+00_JPRB, 0.39420E+00_JPRB, 0.38046E+00_JPRB ,&
&  0.47494E+00_JPRB, 0.57288E+00_JPRB, 0.78967E+00_JPRB, 0.87923E+00_JPRB, 0.86189E+00_JPRB, 0.87473E+00_JPRB, 0.78724E+00_JPRB ,&
&  0.70007E+00_JPRB, 0.63588E+00_JPRB, 0.61823E+00_JPRB, 0.57990E+00_JPRB, 0.56697E+00_JPRB, 0.55441E+00_JPRB, 0.53395E+00_JPRB ,&
&  0.52333E+00_JPRB, 0.50760E+00_JPRB, 0.49681E+00_JPRB, 0.49837E+00_JPRB, 0.43206E+00_JPRB, 0.54371E+00_JPRB, 0.58969E+00_JPRB ,&
&  0.54922E+00_JPRB, 0.50403E+00_JPRB, 0.42469E+00_JPRB, 0.53175E+00_JPRB, 0.52292E+00_JPRB, 0.52888E+00_JPRB, 0.59160E+00_JPRB ,&
&  0.70434E+00_JPRB, 0.74310E+00_JPRB, 0.74310E+00_JPRB, 0.72952E+00_JPRB, 0.70420E+00_JPRB, 0.68148E+00_JPRB, 0.67213E+00_JPRB ,&
&  0.68609E+00_JPRB, 0.69951E+00_JPRB, 0.71473E+00_JPRB, 0.72202E+00_JPRB, 0.73374E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.73238E+00_JPRB, 0.74552E+00_JPRB, 0.74519E+00_JPRB, 0.75935E+00_JPRB, 0.77697E+00_JPRB, 0.78095E+00_JPRB, 0.78287E+00_JPRB ,&
&  0.77351E+00_JPRB, 0.77617E+00_JPRB, 0.77164E+00_JPRB, 0.78730E+00_JPRB, 0.78748E+00_JPRB, 0.78902E+00_JPRB, 0.79118E+00_JPRB ,&
&  0.80256E+00_JPRB, 0.81075E+00_JPRB, 0.83701E+00_JPRB, 0.84162E+00_JPRB, 0.83875E+00_JPRB, 0.84428E+00_JPRB, 0.86181E+00_JPRB ,&
&  0.88249E+00_JPRB, 0.90819E+00_JPRB, 0.94219E+00_JPRB, 0.97017E+00_JPRB, 0.10661E+01_JPRB, 0.12160E+01_JPRB, 0.14736E+01_JPRB ,&
&  0.11372E+01_JPRB, 0.95060E+00_JPRB, 0.93484E+00_JPRB, 0.80175E+00_JPRB, 0.77317E+00_JPRB, 0.76217E+00_JPRB, 0.72140E+00_JPRB ,&
&  0.67499E+00_JPRB, 0.64906E+00_JPRB, 0.62751E+00_JPRB, 0.61675E+00_JPRB, 0.61438E+00_JPRB, 0.60491E+00_JPRB, 0.60759E+00_JPRB ,&
&  0.59842E+00_JPRB, 0.59633E+00_JPRB, 0.65427E+00_JPRB, 0.69591E+00_JPRB, 0.70137E+00_JPRB, 0.75182E+00_JPRB, 0.70752E+00_JPRB ,&
&  0.69804E+00_JPRB, 0.65459E+00_JPRB, 0.53291E+00_JPRB, 0.49177E+00_JPRB, 0.47455E+00_JPRB, 0.45659E+00_JPRB, 0.44821E+00_JPRB ,&
&  0.50409E+00_JPRB, 0.58897E+00_JPRB, 0.61498E+00_JPRB, 0.81171E+00_JPRB, 0.89549E+00_JPRB, 0.95822E+00_JPRB, 0.81922E+00_JPRB ,&
&  0.70811E+00_JPRB, 0.64306E+00_JPRB, 0.62763E+00_JPRB, 0.59434E+00_JPRB, 0.58355E+00_JPRB, 0.54968E+00_JPRB, 0.51686E+00_JPRB ,&
&  0.51202E+00_JPRB, 0.49225E+00_JPRB, 0.47477E+00_JPRB, 0.45325E+00_JPRB, 0.47372E+00_JPRB, 0.45692E+00_JPRB, 0.50307E+00_JPRB ,&
&  0.47571E+00_JPRB, 0.51342E+00_JPRB, 0.46496E+00_JPRB, 0.53903E+00_JPRB, 0.47668E+00_JPRB, 0.46759E+00_JPRB, 0.50818E+00_JPRB ,&
&  0.57678E+00_JPRB, 0.61196E+00_JPRB, 0.62502E+00_JPRB, 0.65007E+00_JPRB, 0.65553E+00_JPRB, 0.64701E+00_JPRB, 0.65351E+00_JPRB ,&
&  0.68822E+00_JPRB, 0.69320E+00_JPRB, 0.69832E+00_JPRB, 0.69915E+00_JPRB, 0.71548E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.67560E+00_JPRB, 0.68797E+00_JPRB, 0.68214E+00_JPRB, 0.68861E+00_JPRB, 0.69078E+00_JPRB, 0.70344E+00_JPRB, 0.70188E+00_JPRB ,&
&  0.69815E+00_JPRB, 0.70475E+00_JPRB, 0.68543E+00_JPRB, 0.68929E+00_JPRB, 0.68397E+00_JPRB, 0.68741E+00_JPRB, 0.67770E+00_JPRB ,&
&  0.67891E+00_JPRB, 0.69174E+00_JPRB, 0.69787E+00_JPRB, 0.70663E+00_JPRB, 0.70507E+00_JPRB, 0.68867E+00_JPRB, 0.66782E+00_JPRB ,&
&  0.69893E+00_JPRB, 0.78599E+00_JPRB, 0.88288E+00_JPRB, 0.94585E+00_JPRB, 0.10956E+01_JPRB, 0.10542E+01_JPRB, 0.11220E+01_JPRB ,&
&  0.96945E+00_JPRB, 0.86809E+00_JPRB, 0.95896E+00_JPRB, 0.85248E+00_JPRB, 0.82000E+00_JPRB, 0.77638E+00_JPRB, 0.74517E+00_JPRB ,&
&  0.74323E+00_JPRB, 0.75628E+00_JPRB, 0.77756E+00_JPRB, 0.80480E+00_JPRB, 0.82783E+00_JPRB, 0.82278E+00_JPRB, 0.80319E+00_JPRB ,&
&  0.79138E+00_JPRB, 0.82319E+00_JPRB, 0.91061E+00_JPRB, 0.90030E+00_JPRB, 0.91460E+00_JPRB, 0.93851E+00_JPRB, 0.92807E+00_JPRB ,&
&  0.90598E+00_JPRB, 0.86868E+00_JPRB, 0.73940E+00_JPRB, 0.66663E+00_JPRB, 0.66755E+00_JPRB, 0.62942E+00_JPRB, 0.63202E+00_JPRB ,&
&  0.68055E+00_JPRB, 0.69912E+00_JPRB, 0.58578E+00_JPRB, 0.70032E+00_JPRB, 0.81602E+00_JPRB, 0.84419E+00_JPRB, 0.83887E+00_JPRB ,&
&  0.73666E+00_JPRB, 0.67843E+00_JPRB, 0.65022E+00_JPRB, 0.63189E+00_JPRB, 0.60558E+00_JPRB, 0.56577E+00_JPRB, 0.54366E+00_JPRB ,&
&  0.50742E+00_JPRB, 0.47993E+00_JPRB, 0.45016E+00_JPRB, 0.42827E+00_JPRB, 0.42411E+00_JPRB, 0.47489E+00_JPRB, 0.44192E+00_JPRB ,&
&  0.45001E+00_JPRB, 0.47817E+00_JPRB, 0.56839E+00_JPRB, 0.51449E+00_JPRB, 0.38300E+00_JPRB, 0.42339E+00_JPRB, 0.43799E+00_JPRB ,&
&  0.46241E+00_JPRB, 0.49457E+00_JPRB, 0.51823E+00_JPRB, 0.54865E+00_JPRB, 0.56345E+00_JPRB, 0.56359E+00_JPRB, 0.57171E+00_JPRB ,&
&  0.59538E+00_JPRB, 0.61927E+00_JPRB, 0.65317E+00_JPRB, 0.65820E+00_JPRB, 0.66001E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.58009E+00_JPRB, 0.58223E+00_JPRB, 0.58562E+00_JPRB, 0.58212E+00_JPRB, 0.57843E+00_JPRB, 0.57728E+00_JPRB, 0.57878E+00_JPRB ,&
&  0.57850E+00_JPRB, 0.56967E+00_JPRB, 0.56118E+00_JPRB, 0.55436E+00_JPRB, 0.54144E+00_JPRB, 0.51661E+00_JPRB, 0.48160E+00_JPRB ,&
&  0.48655E+00_JPRB, 0.49532E+00_JPRB, 0.50988E+00_JPRB, 0.54605E+00_JPRB, 0.62274E+00_JPRB, 0.73687E+00_JPRB, 0.88219E+00_JPRB ,&
&  0.11636E+01_JPRB, 0.14703E+01_JPRB, 0.16704E+01_JPRB, 0.17173E+01_JPRB, 0.18687E+01_JPRB, 0.12319E+01_JPRB, 0.95436E+00_JPRB ,&
&  0.98899E+00_JPRB, 0.93840E+00_JPRB, 0.92262E+00_JPRB, 0.88513E+00_JPRB, 0.86230E+00_JPRB, 0.84814E+00_JPRB, 0.88024E+00_JPRB ,&
&  0.94474E+00_JPRB, 0.10038E+01_JPRB, 0.10546E+01_JPRB, 0.11029E+01_JPRB, 0.11376E+01_JPRB, 0.11702E+01_JPRB, 0.11678E+01_JPRB ,&
&  0.11550E+01_JPRB, 0.11432E+01_JPRB, 0.11625E+01_JPRB, 0.10902E+01_JPRB, 0.11085E+01_JPRB, 0.11163E+01_JPRB, 0.11031E+01_JPRB ,&
&  0.10572E+01_JPRB, 0.98630E+00_JPRB, 0.94476E+00_JPRB, 0.88516E+00_JPRB, 0.85153E+00_JPRB, 0.81751E+00_JPRB, 0.78758E+00_JPRB ,&
&  0.77739E+00_JPRB, 0.75612E+00_JPRB, 0.63894E+00_JPRB, 0.74278E+00_JPRB, 0.81448E+00_JPRB, 0.73631E+00_JPRB, 0.82457E+00_JPRB ,&
&  0.75019E+00_JPRB, 0.72082E+00_JPRB, 0.68177E+00_JPRB, 0.64948E+00_JPRB, 0.62738E+00_JPRB, 0.65187E+00_JPRB, 0.55618E+00_JPRB ,&
&  0.52426E+00_JPRB, 0.48397E+00_JPRB, 0.43070E+00_JPRB, 0.41905E+00_JPRB, 0.42372E+00_JPRB, 0.45417E+00_JPRB, 0.46216E+00_JPRB ,&
&  0.46530E+00_JPRB, 0.48405E+00_JPRB, 0.58171E+00_JPRB, 0.62262E+00_JPRB, 0.48388E+00_JPRB, 0.39724E+00_JPRB, 0.40331E+00_JPRB ,&
&  0.40627E+00_JPRB, 0.41271E+00_JPRB, 0.42125E+00_JPRB, 0.44326E+00_JPRB, 0.46388E+00_JPRB, 0.48049E+00_JPRB, 0.49340E+00_JPRB ,&
&  0.50140E+00_JPRB, 0.52235E+00_JPRB, 0.54281E+00_JPRB, 0.55998E+00_JPRB, 0.57570E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.49277E+00_JPRB, 0.49417E+00_JPRB, 0.50322E+00_JPRB, 0.50312E+00_JPRB, 0.51562E+00_JPRB, 0.51505E+00_JPRB, 0.51545E+00_JPRB ,&
&  0.52259E+00_JPRB, 0.53662E+00_JPRB, 0.52733E+00_JPRB, 0.50524E+00_JPRB, 0.50039E+00_JPRB, 0.51165E+00_JPRB, 0.54512E+00_JPRB ,&
&  0.61887E+00_JPRB, 0.69147E+00_JPRB, 0.75682E+00_JPRB, 0.86175E+00_JPRB, 0.10574E+01_JPRB, 0.13755E+01_JPRB, 0.16805E+01_JPRB ,&
&  0.18906E+01_JPRB, 0.20177E+01_JPRB, 0.19888E+01_JPRB, 0.20500E+01_JPRB, 0.16075E+01_JPRB, 0.94919E+00_JPRB, 0.10133E+01_JPRB ,&
&  0.10707E+01_JPRB, 0.10997E+01_JPRB, 0.10324E+01_JPRB, 0.97913E+00_JPRB, 0.94447E+00_JPRB, 0.96863E+00_JPRB, 0.10266E+01_JPRB ,&
&  0.10919E+01_JPRB, 0.11607E+01_JPRB, 0.12147E+01_JPRB, 0.12522E+01_JPRB, 0.12689E+01_JPRB, 0.12903E+01_JPRB, 0.12969E+01_JPRB ,&
&  0.13252E+01_JPRB, 0.13209E+01_JPRB, 0.13064E+01_JPRB, 0.12702E+01_JPRB, 0.12457E+01_JPRB, 0.12586E+01_JPRB, 0.12512E+01_JPRB ,&
&  0.12205E+01_JPRB, 0.11316E+01_JPRB, 0.10892E+01_JPRB, 0.10445E+01_JPRB, 0.96130E+00_JPRB, 0.87614E+00_JPRB, 0.85017E+00_JPRB ,&
&  0.83381E+00_JPRB, 0.84011E+00_JPRB, 0.70194E+00_JPRB, 0.85501E+00_JPRB, 0.95765E+00_JPRB, 0.87968E+00_JPRB, 0.83806E+00_JPRB ,&
&  0.76648E+00_JPRB, 0.73925E+00_JPRB, 0.67720E+00_JPRB, 0.66582E+00_JPRB, 0.67703E+00_JPRB, 0.74058E+00_JPRB, 0.56892E+00_JPRB ,&
&  0.58596E+00_JPRB, 0.49754E+00_JPRB, 0.48321E+00_JPRB, 0.48292E+00_JPRB, 0.48367E+00_JPRB, 0.46011E+00_JPRB, 0.45120E+00_JPRB ,&
&  0.44754E+00_JPRB, 0.56117E+00_JPRB, 0.74443E+00_JPRB, 0.11446E+01_JPRB, 0.15512E+01_JPRB, 0.41960E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.40411E+00_JPRB, 0.40471E+00_JPRB, 0.40232E+00_JPRB, 0.41400E+00_JPRB, 0.42605E+00_JPRB, 0.43612E+00_JPRB, 0.45209E+00_JPRB ,&
&  0.46269E+00_JPRB, 0.46972E+00_JPRB, 0.47727E+00_JPRB, 0.47637E+00_JPRB, 0.48563E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.45892E+00_JPRB, 0.46821E+00_JPRB, 0.49532E+00_JPRB, 0.54444E+00_JPRB, 0.61093E+00_JPRB, 0.64252E+00_JPRB, 0.63451E+00_JPRB ,&
&  0.60314E+00_JPRB, 0.60422E+00_JPRB, 0.62688E+00_JPRB, 0.63458E+00_JPRB, 0.67733E+00_JPRB, 0.76247E+00_JPRB, 0.84712E+00_JPRB ,&
&  0.96245E+00_JPRB, 0.10660E+01_JPRB, 0.11637E+01_JPRB, 0.13164E+01_JPRB, 0.14627E+01_JPRB, 0.16329E+01_JPRB, 0.17293E+01_JPRB ,&
&  0.17003E+01_JPRB, 0.14842E+01_JPRB, 0.13819E+01_JPRB, 0.10642E+01_JPRB, 0.87183E+00_JPRB, 0.98414E+00_JPRB, 0.98165E+00_JPRB ,&
&  0.98694E+00_JPRB, 0.10122E+01_JPRB, 0.10102E+01_JPRB, 0.10275E+01_JPRB, 0.10639E+01_JPRB, 0.10992E+01_JPRB, 0.11522E+01_JPRB ,&
&  0.12236E+01_JPRB, 0.12922E+01_JPRB, 0.13470E+01_JPRB, 0.13882E+01_JPRB, 0.13888E+01_JPRB, 0.13991E+01_JPRB, 0.14357E+01_JPRB ,&
&  0.14887E+01_JPRB, 0.14829E+01_JPRB, 0.14701E+01_JPRB, 0.14503E+01_JPRB, 0.13775E+01_JPRB, 0.13631E+01_JPRB, 0.13404E+01_JPRB ,&
&  0.13193E+01_JPRB, 0.12764E+01_JPRB, 0.12412E+01_JPRB, 0.12162E+01_JPRB, 0.11513E+01_JPRB, 0.99209E+00_JPRB, 0.93297E+00_JPRB ,&
&  0.91453E+00_JPRB, 0.87232E+00_JPRB, 0.80814E+00_JPRB, 0.92196E+00_JPRB, 0.68734E+00_JPRB, 0.79654E+00_JPRB, 0.88525E+00_JPRB ,&
&  0.85436E+00_JPRB, 0.81351E+00_JPRB, 0.74196E+00_JPRB, 0.71756E+00_JPRB, 0.70299E+00_JPRB, 0.61600E+00_JPRB, 0.61960E+00_JPRB ,&
&  0.65678E+00_JPRB, 0.59508E+00_JPRB, 0.56955E+00_JPRB, 0.56239E+00_JPRB, 0.57337E+00_JPRB, 0.53273E+00_JPRB, 0.52795E+00_JPRB ,&
&  0.51588E+00_JPRB, 0.60210E+00_JPRB, 0.61010E+00_JPRB, 0.73221E+00_JPRB, 0.75504E+00_JPRB, 0.40571E+00_JPRB, 0.40702E+00_JPRB ,&
&  0.41854E+00_JPRB, 0.42828E+00_JPRB, 0.43075E+00_JPRB, 0.45307E+00_JPRB, 0.46591E+00_JPRB, 0.46326E+00_JPRB, 0.46921E+00_JPRB ,&
&  0.47579E+00_JPRB, 0.47471E+00_JPRB, 0.47302E+00_JPRB, 0.47142E+00_JPRB, 0.46846E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.50920E+00_JPRB, 0.50483E+00_JPRB, 0.52853E+00_JPRB, 0.62365E+00_JPRB, 0.72083E+00_JPRB, 0.77870E+00_JPRB, 0.75493E+00_JPRB ,&
&  0.70463E+00_JPRB, 0.65871E+00_JPRB, 0.69509E+00_JPRB, 0.74061E+00_JPRB, 0.81440E+00_JPRB, 0.90488E+00_JPRB, 0.10028E+01_JPRB ,&
&  0.11181E+01_JPRB, 0.12113E+01_JPRB, 0.12805E+01_JPRB, 0.13599E+01_JPRB, 0.14614E+01_JPRB, 0.15645E+01_JPRB, 0.16150E+01_JPRB ,&
&  0.13122E+01_JPRB, 0.10039E+01_JPRB, 0.12003E+01_JPRB, 0.11088E+01_JPRB, 0.10665E+01_JPRB, 0.10569E+01_JPRB, 0.10674E+01_JPRB ,&
&  0.10588E+01_JPRB, 0.10710E+01_JPRB, 0.11073E+01_JPRB, 0.11411E+01_JPRB, 0.11383E+01_JPRB, 0.11807E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.13287E+01_JPRB, 0.13906E+01_JPRB, 0.14225E+01_JPRB, 0.14492E+01_JPRB, 0.14593E+01_JPRB, 0.14798E+01_JPRB, 0.15179E+01_JPRB ,&
&  0.15483E+01_JPRB, 0.15665E+01_JPRB, 0.15409E+01_JPRB, 0.14807E+01_JPRB, 0.14452E+01_JPRB, 0.14307E+01_JPRB, 0.13933E+01_JPRB ,&
&  0.13763E+01_JPRB, 0.13479E+01_JPRB, 0.13797E+01_JPRB, 0.14085E+01_JPRB, 0.13899E+01_JPRB, 0.11888E+01_JPRB, 0.11300E+01_JPRB ,&
&  0.10464E+01_JPRB, 0.96172E+00_JPRB, 0.92643E+00_JPRB, 0.99915E+00_JPRB, 0.77648E+00_JPRB, 0.83794E+00_JPRB, 0.84969E+00_JPRB ,&
&  0.92046E+00_JPRB, 0.88927E+00_JPRB, 0.85016E+00_JPRB, 0.80108E+00_JPRB, 0.80540E+00_JPRB, 0.69235E+00_JPRB, 0.66308E+00_JPRB ,&
&  0.65531E+00_JPRB, 0.69430E+00_JPRB, 0.63946E+00_JPRB, 0.62075E+00_JPRB, 0.57894E+00_JPRB, 0.60369E+00_JPRB, 0.64269E+00_JPRB ,&
&  0.73499E+00_JPRB, 0.70126E+00_JPRB, 0.65724E+00_JPRB, 0.60443E+00_JPRB, 0.49430E+00_JPRB, 0.42613E+00_JPRB, 0.43294E+00_JPRB ,&
&  0.45027E+00_JPRB, 0.46734E+00_JPRB, 0.47403E+00_JPRB, 0.49332E+00_JPRB, 0.50804E+00_JPRB, 0.50407E+00_JPRB, 0.49788E+00_JPRB ,&
&  0.49928E+00_JPRB, 0.51432E+00_JPRB, 0.50851E+00_JPRB, 0.50990E+00_JPRB, 0.51422E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.58160E+00_JPRB, 0.58046E+00_JPRB, 0.60022E+00_JPRB, 0.68760E+00_JPRB, 0.81154E+00_JPRB, 0.91773E+00_JPRB, 0.94139E+00_JPRB ,&
&  0.94151E+00_JPRB, 0.76688E+00_JPRB, 0.79388E+00_JPRB, 0.81111E+00_JPRB, 0.89583E+00_JPRB, 0.99104E+00_JPRB, 0.10918E+01_JPRB ,&
&  0.11882E+01_JPRB, 0.12909E+01_JPRB, 0.13648E+01_JPRB, 0.14316E+01_JPRB, 0.15043E+01_JPRB, 0.15716E+01_JPRB, 0.13546E+01_JPRB ,&
&  0.10355E+01_JPRB, 0.11828E+01_JPRB, 0.17347E+01_JPRB, 0.13575E+01_JPRB, 0.11523E+01_JPRB, 0.11467E+01_JPRB, 0.11703E+01_JPRB ,&
&  0.11949E+01_JPRB, 0.11614E+01_JPRB, 0.11910E+01_JPRB, 0.12373E+01_JPRB, 0.12388E+01_JPRB, 0.12917E+01_JPRB, 0.13698E+01_JPRB ,&
&  0.14148E+01_JPRB, 0.14625E+01_JPRB, 0.14772E+01_JPRB, 0.14755E+01_JPRB, 0.14812E+01_JPRB, 0.15034E+01_JPRB, 0.15409E+01_JPRB ,&
&  0.15719E+01_JPRB, 0.15894E+01_JPRB, 0.15663E+01_JPRB, 0.14842E+01_JPRB, 0.14701E+01_JPRB, 0.14864E+01_JPRB, 0.14833E+01_JPRB ,&
&  0.14678E+01_JPRB, 0.15260E+01_JPRB, 0.15414E+01_JPRB, 0.14740E+01_JPRB, 0.13479E+01_JPRB, 0.14412E+01_JPRB, 0.12641E+01_JPRB ,&
&  0.11886E+01_JPRB, 0.10501E+01_JPRB, 0.10683E+01_JPRB, 0.87254E+00_JPRB, 0.89305E+00_JPRB, 0.92756E+00_JPRB, 0.91874E+00_JPRB ,&
&  0.90335E+00_JPRB, 0.91809E+00_JPRB, 0.89906E+00_JPRB, 0.87912E+00_JPRB, 0.86649E+00_JPRB, 0.76453E+00_JPRB, 0.74053E+00_JPRB ,&
&  0.70288E+00_JPRB, 0.75708E+00_JPRB, 0.64807E+00_JPRB, 0.58790E+00_JPRB, 0.52653E+00_JPRB, 0.55918E+00_JPRB, 0.64706E+00_JPRB ,&
&  0.77845E+00_JPRB, 0.83533E+00_JPRB, 0.81383E+00_JPRB, 0.74228E+00_JPRB, 0.61105E+00_JPRB, 0.52445E+00_JPRB, 0.47759E+00_JPRB ,&
&  0.47621E+00_JPRB, 0.48437E+00_JPRB, 0.50162E+00_JPRB, 0.51773E+00_JPRB, 0.53466E+00_JPRB, 0.54911E+00_JPRB, 0.55660E+00_JPRB ,&
&  0.55950E+00_JPRB, 0.55870E+00_JPRB, 0.55539E+00_JPRB, 0.57923E+00_JPRB, 0.59351E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.62998E+00_JPRB, 0.61477E+00_JPRB, 0.60017E+00_JPRB, 0.62900E+00_JPRB, 0.69052E+00_JPRB, 0.71424E+00_JPRB, 0.71918E+00_JPRB ,&
&  0.71242E+00_JPRB, 0.70108E+00_JPRB, 0.73161E+00_JPRB, 0.74484E+00_JPRB, 0.82727E+00_JPRB, 0.92787E+00_JPRB, 0.10106E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.11835E+01_JPRB, 0.12762E+01_JPRB, 0.13864E+01_JPRB, 0.14383E+01_JPRB, 0.14896E+01_JPRB, 0.11658E+01_JPRB ,&
&  0.92493E+00_JPRB, 0.19098E+01_JPRB, 0.17008E+01_JPRB, 0.15565E+01_JPRB, 0.13027E+01_JPRB, 0.12731E+01_JPRB, 0.12325E+01_JPRB ,&
&  0.12363E+01_JPRB, 0.11798E+01_JPRB, 0.12221E+01_JPRB, 0.12672E+01_JPRB, 0.12862E+01_JPRB, 0.13222E+01_JPRB, 0.13700E+01_JPRB ,&
&  0.14080E+01_JPRB, 0.14567E+01_JPRB, 0.14715E+01_JPRB, 0.14779E+01_JPRB, 0.14878E+01_JPRB, 0.14874E+01_JPRB, 0.15255E+01_JPRB ,&
&  0.15782E+01_JPRB, 0.16162E+01_JPRB, 0.16085E+01_JPRB, 0.16006E+01_JPRB, 0.15569E+01_JPRB, 0.16085E+01_JPRB, 0.16872E+01_JPRB ,&
&  0.16462E+01_JPRB, 0.16049E+01_JPRB, 0.17379E+01_JPRB, 0.16974E+01_JPRB, 0.15990E+01_JPRB, 0.16158E+01_JPRB, 0.13807E+01_JPRB ,&
&  0.13580E+01_JPRB, 0.11399E+01_JPRB, 0.11409E+01_JPRB, 0.91416E+00_JPRB, 0.93375E+00_JPRB, 0.10001E+01_JPRB, 0.96656E+00_JPRB ,&
&  0.91537E+00_JPRB, 0.95564E+00_JPRB, 0.93466E+00_JPRB, 0.93786E+00_JPRB, 0.91674E+00_JPRB, 0.85708E+00_JPRB, 0.78335E+00_JPRB ,&
&  0.78692E+00_JPRB, 0.78753E+00_JPRB, 0.72122E+00_JPRB, 0.56060E+00_JPRB, 0.50743E+00_JPRB, 0.49227E+00_JPRB, 0.63096E+00_JPRB ,&
&  0.76403E+00_JPRB, 0.82403E+00_JPRB, 0.88628E+00_JPRB, 0.92866E+00_JPRB, 0.83859E+00_JPRB, 0.75382E+00_JPRB, 0.63896E+00_JPRB ,&
&  0.53433E+00_JPRB, 0.49672E+00_JPRB, 0.51975E+00_JPRB, 0.51627E+00_JPRB, 0.53278E+00_JPRB, 0.55221E+00_JPRB, 0.56941E+00_JPRB ,&
&  0.58475E+00_JPRB, 0.59980E+00_JPRB, 0.61313E+00_JPRB, 0.61671E+00_JPRB, 0.62829E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.62238E+00_JPRB, 0.61392E+00_JPRB, 0.60359E+00_JPRB, 0.61095E+00_JPRB, 0.67411E+00_JPRB, 0.69414E+00_JPRB, 0.66070E+00_JPRB ,&
&  0.63336E+00_JPRB, 0.63044E+00_JPRB, 0.65683E+00_JPRB, 0.64296E+00_JPRB, 0.67839E+00_JPRB, 0.77909E+00_JPRB, 0.85377E+00_JPRB ,&
&  0.92848E+00_JPRB, 0.10242E+01_JPRB, 0.11546E+01_JPRB, 0.13059E+01_JPRB, 0.14225E+01_JPRB, 0.15585E+01_JPRB, 0.10093E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.16890E+01_JPRB, 0.17919E+01_JPRB, 0.17976E+01_JPRB, 0.15904E+01_JPRB, 0.14662E+01_JPRB, 0.13255E+01_JPRB ,&
&  0.13117E+01_JPRB, 0.12341E+01_JPRB, 0.11808E+01_JPRB, 0.11654E+01_JPRB, 0.11597E+01_JPRB, 0.12170E+01_JPRB, 0.12344E+01_JPRB ,&
&  0.12131E+01_JPRB, 0.12283E+01_JPRB, 0.13017E+01_JPRB, 0.13589E+01_JPRB, 0.14012E+01_JPRB, 0.14369E+01_JPRB, 0.14804E+01_JPRB ,&
&  0.15539E+01_JPRB, 0.16300E+01_JPRB, 0.17345E+01_JPRB, 0.16876E+01_JPRB, 0.17143E+01_JPRB, 0.17713E+01_JPRB, 0.18960E+01_JPRB ,&
&  0.18562E+01_JPRB, 0.18156E+01_JPRB, 0.20500E+01_JPRB, 0.22519E+01_JPRB, 0.21818E+01_JPRB, 0.18843E+01_JPRB, 0.15785E+01_JPRB ,&
&  0.14603E+01_JPRB, 0.12686E+01_JPRB, 0.11141E+01_JPRB, 0.97106E+00_JPRB, 0.10130E+01_JPRB, 0.10949E+01_JPRB, 0.11178E+01_JPRB ,&
&  0.10900E+01_JPRB, 0.10312E+01_JPRB, 0.99442E+00_JPRB, 0.97554E+00_JPRB, 0.95643E+00_JPRB, 0.89517E+00_JPRB, 0.88134E+00_JPRB ,&
&  0.96860E+00_JPRB, 0.10693E+01_JPRB, 0.91654E+00_JPRB, 0.66533E+00_JPRB, 0.53564E+00_JPRB, 0.42206E+00_JPRB, 0.47982E+00_JPRB ,&
&  0.70858E+00_JPRB, 0.76695E+00_JPRB, 0.80869E+00_JPRB, 0.10118E+01_JPRB, 0.10148E+01_JPRB, 0.99274E+00_JPRB, 0.88852E+00_JPRB ,&
&  0.71500E+00_JPRB, 0.62811E+00_JPRB, 0.58361E+00_JPRB, 0.54679E+00_JPRB, 0.54627E+00_JPRB, 0.55230E+00_JPRB, 0.56815E+00_JPRB ,&
&  0.58441E+00_JPRB, 0.59336E+00_JPRB, 0.59819E+00_JPRB, 0.60233E+00_JPRB, 0.61824E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.62205E+00_JPRB, 0.61568E+00_JPRB, 0.62209E+00_JPRB, 0.61999E+00_JPRB, 0.66061E+00_JPRB, 0.68597E+00_JPRB, 0.62067E+00_JPRB ,&
&  0.60214E+00_JPRB, 0.59506E+00_JPRB, 0.60950E+00_JPRB, 0.60436E+00_JPRB, 0.60544E+00_JPRB, 0.64516E+00_JPRB, 0.69739E+00_JPRB ,&
&  0.76659E+00_JPRB, 0.84899E+00_JPRB, 0.98978E+00_JPRB, 0.11529E+01_JPRB, 0.12180E+01_JPRB, 0.11655E+01_JPRB, 0.12336E+01_JPRB ,&
&  0.14945E+01_JPRB, 0.18213E+01_JPRB, 0.19434E+01_JPRB, 0.21571E+01_JPRB, 0.21530E+01_JPRB, 0.22392E+01_JPRB, 0.17799E+01_JPRB ,&
&  0.16251E+01_JPRB, 0.14863E+01_JPRB, 0.13179E+01_JPRB, 0.12041E+01_JPRB, 0.11002E+01_JPRB, 0.10858E+01_JPRB, 0.11238E+01_JPRB ,&
&  0.11052E+01_JPRB, 0.11077E+01_JPRB, 0.11423E+01_JPRB, 0.11730E+01_JPRB, 0.12402E+01_JPRB, 0.12995E+01_JPRB, 0.13734E+01_JPRB ,&
&  0.14826E+01_JPRB, 0.16056E+01_JPRB, 0.17361E+01_JPRB, 0.17909E+01_JPRB, 0.18059E+01_JPRB, 0.18805E+01_JPRB, 0.20730E+01_JPRB ,&
&  0.21734E+01_JPRB, 0.22814E+01_JPRB, 0.23770E+01_JPRB, 0.25686E+01_JPRB, 0.25927E+01_JPRB, 0.23396E+01_JPRB, 0.19214E+01_JPRB ,&
&  0.16342E+01_JPRB, 0.13733E+01_JPRB, 0.10966E+01_JPRB, 0.10488E+01_JPRB, 0.11216E+01_JPRB, 0.12296E+01_JPRB, 0.11278E+01_JPRB ,&
&  0.10011E+01_JPRB, 0.90510E+00_JPRB, 0.90026E+00_JPRB, 0.95630E+00_JPRB, 0.10253E+01_JPRB, 0.10126E+01_JPRB, 0.10447E+01_JPRB ,&
&  0.10083E+01_JPRB, 0.66413E+00_JPRB, 0.55219E+00_JPRB, 0.65295E+00_JPRB, 0.50233E+00_JPRB, 0.38812E+00_JPRB, 0.49303E+00_JPRB ,&
&  0.64986E+00_JPRB, 0.80508E+00_JPRB, 0.84255E+00_JPRB, 0.86526E+00_JPRB, 0.10656E+01_JPRB, 0.11619E+01_JPRB, 0.11794E+01_JPRB ,&
&  0.84299E+00_JPRB, 0.74156E+00_JPRB, 0.67607E+00_JPRB, 0.62424E+00_JPRB, 0.58644E+00_JPRB, 0.57363E+00_JPRB, 0.57149E+00_JPRB ,&
&  0.56272E+00_JPRB, 0.56654E+00_JPRB, 0.57241E+00_JPRB, 0.58396E+00_JPRB, 0.60636E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.59866E+00_JPRB, 0.60565E+00_JPRB, 0.62145E+00_JPRB, 0.63400E+00_JPRB, 0.62300E+00_JPRB, 0.63924E+00_JPRB, 0.62728E+00_JPRB ,&
&  0.59615E+00_JPRB, 0.57028E+00_JPRB, 0.56750E+00_JPRB, 0.57604E+00_JPRB, 0.58328E+00_JPRB, 0.58766E+00_JPRB, 0.61015E+00_JPRB ,&
&  0.65047E+00_JPRB, 0.71922E+00_JPRB, 0.86559E+00_JPRB, 0.10028E+01_JPRB, 0.10613E+01_JPRB, 0.10010E+01_JPRB, 0.13284E+01_JPRB ,&
&  0.15886E+01_JPRB, 0.19229E+01_JPRB, 0.23563E+01_JPRB, 0.25409E+01_JPRB, 0.24828E+01_JPRB, 0.24464E+01_JPRB, 0.21299E+01_JPRB ,&
&  0.18672E+01_JPRB, 0.16862E+01_JPRB, 0.15196E+01_JPRB, 0.13800E+01_JPRB, 0.12817E+01_JPRB, 0.11763E+01_JPRB, 0.11341E+01_JPRB ,&
&  0.10915E+01_JPRB, 0.10680E+01_JPRB, 0.10858E+01_JPRB, 0.10914E+01_JPRB, 0.11362E+01_JPRB, 0.11861E+01_JPRB, 0.12664E+01_JPRB ,&
&  0.13908E+01_JPRB, 0.15515E+01_JPRB, 0.17205E+01_JPRB, 0.19450E+01_JPRB, 0.16117E+01_JPRB, 0.19200E+01_JPRB, 0.20255E+01_JPRB ,&
&  0.21878E+01_JPRB, 0.22774E+01_JPRB, 0.22528E+01_JPRB, 0.28967E+01_JPRB, 0.31078E+01_JPRB, 0.26733E+01_JPRB, 0.22008E+01_JPRB ,&
&  0.18649E+01_JPRB, 0.15103E+01_JPRB, 0.12529E+01_JPRB, 0.12346E+01_JPRB, 0.12662E+01_JPRB, 0.13083E+01_JPRB, 0.97516E+00_JPRB ,&
&  0.89267E+00_JPRB, 0.10196E+01_JPRB, 0.10058E+01_JPRB, 0.87782E+00_JPRB, 0.98012E+00_JPRB, 0.11244E+01_JPRB, 0.84187E+00_JPRB ,&
&  0.46112E+00_JPRB, 0.40776E+00_JPRB, 0.43054E+00_JPRB, 0.43330E+00_JPRB, 0.40999E+00_JPRB, 0.41588E+00_JPRB, 0.71068E+00_JPRB ,&
&  0.62069E+00_JPRB, 0.81163E+00_JPRB, 0.78812E+00_JPRB, 0.81960E+00_JPRB, 0.95800E+00_JPRB, 0.11941E+01_JPRB, 0.14953E+01_JPRB ,&
&  0.94638E+00_JPRB, 0.82584E+00_JPRB, 0.71930E+00_JPRB, 0.66090E+00_JPRB, 0.61358E+00_JPRB, 0.58653E+00_JPRB, 0.57401E+00_JPRB ,&
&  0.56171E+00_JPRB, 0.55357E+00_JPRB, 0.55215E+00_JPRB, 0.56438E+00_JPRB, 0.58126E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.60709E+00_JPRB, 0.61086E+00_JPRB, 0.61155E+00_JPRB, 0.61249E+00_JPRB, 0.59411E+00_JPRB, 0.60142E+00_JPRB, 0.62569E+00_JPRB ,&
&  0.60985E+00_JPRB, 0.57485E+00_JPRB, 0.55831E+00_JPRB, 0.56225E+00_JPRB, 0.57448E+00_JPRB, 0.58164E+00_JPRB, 0.58094E+00_JPRB ,&
&  0.60614E+00_JPRB, 0.66393E+00_JPRB, 0.85669E+00_JPRB, 0.87256E+00_JPRB, 0.93543E+00_JPRB, 0.89829E+00_JPRB, 0.11702E+01_JPRB ,&
&  0.15673E+01_JPRB, 0.20124E+01_JPRB, 0.24135E+01_JPRB, 0.26033E+01_JPRB, 0.29424E+01_JPRB, 0.29940E+01_JPRB, 0.28561E+01_JPRB ,&
&  0.23796E+01_JPRB, 0.20375E+01_JPRB, 0.17784E+01_JPRB, 0.15997E+01_JPRB, 0.14570E+01_JPRB, 0.12978E+01_JPRB, 0.12086E+01_JPRB ,&
&  0.11241E+01_JPRB, 0.10738E+01_JPRB, 0.10315E+01_JPRB, 0.10399E+01_JPRB, 0.10635E+01_JPRB, 0.10872E+01_JPRB, 0.11528E+01_JPRB ,&
&  0.12596E+01_JPRB, 0.14556E+01_JPRB, 0.16684E+01_JPRB, 0.19457E+01_JPRB, 0.17112E+01_JPRB, 0.15326E+01_JPRB, 0.18081E+01_JPRB ,&
&  0.21115E+01_JPRB, 0.23777E+01_JPRB, 0.24893E+01_JPRB, 0.34248E+01_JPRB, 0.34481E+01_JPRB, 0.29006E+01_JPRB, 0.25685E+01_JPRB ,&
&  0.21023E+01_JPRB, 0.18813E+01_JPRB, 0.14618E+01_JPRB, 0.14298E+01_JPRB, 0.14431E+01_JPRB, 0.11458E+01_JPRB, 0.10242E+01_JPRB ,&
&  0.11210E+01_JPRB, 0.10859E+01_JPRB, 0.10129E+01_JPRB, 0.85668E+00_JPRB, 0.96590E+00_JPRB, 0.93099E+00_JPRB, 0.55201E+00_JPRB ,&
&  0.48753E+00_JPRB, 0.47345E+00_JPRB, 0.46402E+00_JPRB, 0.44537E+00_JPRB, 0.43713E+00_JPRB, 0.44876E+00_JPRB, 0.60366E+00_JPRB ,&
&  0.59524E+00_JPRB, 0.76007E+00_JPRB, 0.77528E+00_JPRB, 0.78854E+00_JPRB, 0.83559E+00_JPRB, 0.11113E+01_JPRB, 0.12968E+01_JPRB ,&
&  0.99890E+00_JPRB, 0.87353E+00_JPRB, 0.72786E+00_JPRB, 0.66640E+00_JPRB, 0.61322E+00_JPRB, 0.58114E+00_JPRB, 0.58819E+00_JPRB ,&
&  0.57895E+00_JPRB, 0.56021E+00_JPRB, 0.55675E+00_JPRB, 0.56935E+00_JPRB, 0.58653E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.60841E+00_JPRB, 0.61479E+00_JPRB, 0.61896E+00_JPRB, 0.63258E+00_JPRB, 0.62391E+00_JPRB, 0.61258E+00_JPRB, 0.63130E+00_JPRB ,&
&  0.61613E+00_JPRB, 0.59490E+00_JPRB, 0.57412E+00_JPRB, 0.57236E+00_JPRB, 0.56707E+00_JPRB, 0.57751E+00_JPRB, 0.58001E+00_JPRB ,&
&  0.59859E+00_JPRB, 0.68372E+00_JPRB, 0.81232E+00_JPRB, 0.78671E+00_JPRB, 0.76252E+00_JPRB, 0.75662E+00_JPRB, 0.92073E+00_JPRB ,&
&  0.14505E+01_JPRB, 0.20162E+01_JPRB, 0.24193E+01_JPRB, 0.28232E+01_JPRB, 0.32023E+01_JPRB, 0.31579E+01_JPRB, 0.32409E+01_JPRB ,&
&  0.28528E+01_JPRB, 0.23982E+01_JPRB, 0.20884E+01_JPRB, 0.18379E+01_JPRB, 0.17080E+01_JPRB, 0.15850E+01_JPRB, 0.14561E+01_JPRB ,&
&  0.12817E+01_JPRB, 0.11465E+01_JPRB, 0.10686E+01_JPRB, 0.10104E+01_JPRB, 0.99274E+00_JPRB, 0.10114E+01_JPRB, 0.10382E+01_JPRB ,&
&  0.11365E+01_JPRB, 0.13283E+01_JPRB, 0.15408E+01_JPRB, 0.18485E+01_JPRB, 0.20087E+01_JPRB, 0.19722E+01_JPRB, 0.18919E+01_JPRB ,&
&  0.19166E+01_JPRB, 0.20024E+01_JPRB, 0.29189E+01_JPRB, 0.36582E+01_JPRB, 0.34522E+01_JPRB, 0.30590E+01_JPRB, 0.26914E+01_JPRB ,&
&  0.22337E+01_JPRB, 0.19836E+01_JPRB, 0.15965E+01_JPRB, 0.16049E+01_JPRB, 0.14098E+01_JPRB, 0.11202E+01_JPRB, 0.12033E+01_JPRB ,&
&  0.12848E+01_JPRB, 0.11930E+01_JPRB, 0.10286E+01_JPRB, 0.86650E+00_JPRB, 0.77771E+00_JPRB, 0.66207E+00_JPRB, 0.54302E+00_JPRB ,&
&  0.51198E+00_JPRB, 0.50276E+00_JPRB, 0.47675E+00_JPRB, 0.47520E+00_JPRB, 0.46538E+00_JPRB, 0.49013E+00_JPRB, 0.56600E+00_JPRB ,&
&  0.66620E+00_JPRB, 0.71772E+00_JPRB, 0.75736E+00_JPRB, 0.77224E+00_JPRB, 0.76781E+00_JPRB, 0.83046E+00_JPRB, 0.10683E+01_JPRB ,&
&  0.90712E+00_JPRB, 0.86460E+00_JPRB, 0.75457E+00_JPRB, 0.66854E+00_JPRB, 0.62924E+00_JPRB, 0.59521E+00_JPRB, 0.57552E+00_JPRB ,&
&  0.57062E+00_JPRB, 0.58164E+00_JPRB, 0.58821E+00_JPRB, 0.58607E+00_JPRB, 0.58070E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.60758E+00_JPRB, 0.62376E+00_JPRB, 0.63397E+00_JPRB, 0.63812E+00_JPRB, 0.63025E+00_JPRB, 0.62527E+00_JPRB, 0.63338E+00_JPRB ,&
&  0.62285E+00_JPRB, 0.61050E+00_JPRB, 0.59729E+00_JPRB, 0.59246E+00_JPRB, 0.58222E+00_JPRB, 0.58102E+00_JPRB, 0.57348E+00_JPRB ,&
&  0.58984E+00_JPRB, 0.72017E+00_JPRB, 0.63017E+00_JPRB, 0.63084E+00_JPRB, 0.67581E+00_JPRB, 0.71290E+00_JPRB, 0.79784E+00_JPRB ,&
&  0.13310E+01_JPRB, 0.20320E+01_JPRB, 0.25761E+01_JPRB, 0.33207E+01_JPRB, 0.40407E+01_JPRB, 0.37464E+01_JPRB, 0.35448E+01_JPRB ,&
&  0.33753E+01_JPRB, 0.28143E+01_JPRB, 0.24584E+01_JPRB, 0.21830E+01_JPRB, 0.19653E+01_JPRB, 0.18025E+01_JPRB, 0.16077E+01_JPRB ,&
&  0.13990E+01_JPRB, 0.12459E+01_JPRB, 0.11362E+01_JPRB, 0.10541E+01_JPRB, 0.99746E+00_JPRB, 0.10222E+01_JPRB, 0.10343E+01_JPRB ,&
&  0.10437E+01_JPRB, 0.11808E+01_JPRB, 0.14085E+01_JPRB, 0.16759E+01_JPRB, 0.18249E+01_JPRB, 0.19230E+01_JPRB, 0.26201E+01_JPRB ,&
&  0.26401E+01_JPRB, 0.27204E+01_JPRB, 0.31886E+01_JPRB, 0.46078E+01_JPRB, 0.36473E+01_JPRB, 0.28668E+01_JPRB, 0.27327E+01_JPRB ,&
&  0.19213E+01_JPRB, 0.15964E+01_JPRB, 0.15388E+01_JPRB, 0.15232E+01_JPRB, 0.13101E+01_JPRB, 0.13548E+01_JPRB, 0.18265E+01_JPRB ,&
&  0.14558E+01_JPRB, 0.13547E+01_JPRB, 0.13304E+01_JPRB, 0.12840E+01_JPRB, 0.71780E+00_JPRB, 0.58745E+00_JPRB, 0.79398E+00_JPRB ,&
&  0.83238E+00_JPRB, 0.60302E+00_JPRB, 0.50248E+00_JPRB, 0.59662E+00_JPRB, 0.50243E+00_JPRB, 0.53807E+00_JPRB, 0.64157E+00_JPRB ,&
&  0.64741E+00_JPRB, 0.65378E+00_JPRB, 0.76272E+00_JPRB, 0.74867E+00_JPRB, 0.72790E+00_JPRB, 0.72331E+00_JPRB, 0.86079E+00_JPRB ,&
&  0.90797E+00_JPRB, 0.90670E+00_JPRB, 0.85734E+00_JPRB, 0.74335E+00_JPRB, 0.69005E+00_JPRB, 0.65212E+00_JPRB, 0.62602E+00_JPRB ,&
&  0.59353E+00_JPRB, 0.58417E+00_JPRB, 0.59759E+00_JPRB, 0.60343E+00_JPRB, 0.58714E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.61053E+00_JPRB, 0.60544E+00_JPRB, 0.61588E+00_JPRB, 0.61694E+00_JPRB, 0.62465E+00_JPRB, 0.62911E+00_JPRB, 0.63539E+00_JPRB ,&
&  0.62411E+00_JPRB, 0.61031E+00_JPRB, 0.61402E+00_JPRB, 0.61544E+00_JPRB, 0.60282E+00_JPRB, 0.58844E+00_JPRB, 0.57593E+00_JPRB ,&
&  0.59226E+00_JPRB, 0.62248E+00_JPRB, 0.56062E+00_JPRB, 0.56442E+00_JPRB, 0.66035E+00_JPRB, 0.68976E+00_JPRB, 0.79969E+00_JPRB ,&
&  0.12772E+01_JPRB, 0.19657E+01_JPRB, 0.26508E+01_JPRB, 0.35522E+01_JPRB, 0.41540E+01_JPRB, 0.41112E+01_JPRB, 0.37910E+01_JPRB ,&
&  0.37399E+01_JPRB, 0.30429E+01_JPRB, 0.26470E+01_JPRB, 0.24084E+01_JPRB, 0.21365E+01_JPRB, 0.18794E+01_JPRB, 0.16615E+01_JPRB ,&
&  0.14611E+01_JPRB, 0.12876E+01_JPRB, 0.11820E+01_JPRB, 0.10972E+01_JPRB, 0.10685E+01_JPRB, 0.10664E+01_JPRB, 0.10468E+01_JPRB ,&
&  0.10033E+01_JPRB, 0.10325E+01_JPRB, 0.12880E+01_JPRB, 0.15643E+01_JPRB, 0.16733E+01_JPRB, 0.17885E+01_JPRB, 0.23138E+01_JPRB ,&
&  0.27421E+01_JPRB, 0.28173E+01_JPRB, 0.33920E+01_JPRB, 0.41091E+01_JPRB, 0.39800E+01_JPRB, 0.31544E+01_JPRB, 0.31272E+01_JPRB ,&
&  0.22722E+01_JPRB, 0.18362E+01_JPRB, 0.15559E+01_JPRB, 0.13007E+01_JPRB, 0.12996E+01_JPRB, 0.18433E+01_JPRB, 0.17568E+01_JPRB ,&
&  0.15975E+01_JPRB, 0.14455E+01_JPRB, 0.13764E+01_JPRB, 0.11665E+01_JPRB, 0.92793E+00_JPRB, 0.67699E+00_JPRB, 0.80586E+00_JPRB ,&
&  0.89025E+00_JPRB, 0.91314E+00_JPRB, 0.83182E+00_JPRB, 0.73402E+00_JPRB, 0.62505E+00_JPRB, 0.68024E+00_JPRB, 0.66985E+00_JPRB ,&
&  0.64824E+00_JPRB, 0.61963E+00_JPRB, 0.66277E+00_JPRB, 0.69613E+00_JPRB, 0.70889E+00_JPRB, 0.66670E+00_JPRB, 0.75908E+00_JPRB ,&
&  0.79913E+00_JPRB, 0.89718E+00_JPRB, 0.89733E+00_JPRB, 0.80314E+00_JPRB, 0.73607E+00_JPRB, 0.70693E+00_JPRB, 0.68303E+00_JPRB ,&
&  0.65910E+00_JPRB, 0.62158E+00_JPRB, 0.61255E+00_JPRB, 0.60258E+00_JPRB, 0.59510E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.61396E+00_JPRB, 0.61417E+00_JPRB, 0.60910E+00_JPRB, 0.60098E+00_JPRB, 0.59395E+00_JPRB, 0.59338E+00_JPRB, 0.60112E+00_JPRB ,&
&  0.59763E+00_JPRB, 0.58113E+00_JPRB, 0.58204E+00_JPRB, 0.58449E+00_JPRB, 0.57666E+00_JPRB, 0.56264E+00_JPRB, 0.55780E+00_JPRB ,&
&  0.57742E+00_JPRB, 0.58536E+00_JPRB, 0.51820E+00_JPRB, 0.57292E+00_JPRB, 0.62771E+00_JPRB, 0.65597E+00_JPRB, 0.80514E+00_JPRB ,&
&  0.10423E+01_JPRB, 0.15372E+01_JPRB, 0.21144E+01_JPRB, 0.29536E+01_JPRB, 0.37121E+01_JPRB, 0.40694E+01_JPRB, 0.40996E+01_JPRB ,&
&  0.36426E+01_JPRB, 0.32998E+01_JPRB, 0.28352E+01_JPRB, 0.24628E+01_JPRB, 0.21542E+01_JPRB, 0.19254E+01_JPRB, 0.16987E+01_JPRB ,&
&  0.15156E+01_JPRB, 0.13681E+01_JPRB, 0.12679E+01_JPRB, 0.11733E+01_JPRB, 0.11313E+01_JPRB, 0.11179E+01_JPRB, 0.10953E+01_JPRB ,&
&  0.10263E+01_JPRB, 0.97719E+00_JPRB, 0.11498E+01_JPRB, 0.14556E+01_JPRB, 0.15555E+01_JPRB, 0.17314E+01_JPRB, 0.19541E+01_JPRB ,&
&  0.28821E+01_JPRB, 0.32711E+01_JPRB, 0.33224E+01_JPRB, 0.44444E+01_JPRB, 0.29700E+01_JPRB, 0.33167E+01_JPRB, 0.35908E+01_JPRB ,&
&  0.33223E+01_JPRB, 0.28675E+01_JPRB, 0.26218E+01_JPRB, 0.19768E+01_JPRB, 0.15199E+01_JPRB, 0.19690E+01_JPRB, 0.17315E+01_JPRB ,&
&  0.15148E+01_JPRB, 0.14797E+01_JPRB, 0.14174E+01_JPRB, 0.14002E+01_JPRB, 0.10074E+01_JPRB, 0.81274E+00_JPRB, 0.67421E+00_JPRB ,&
&  0.71069E+00_JPRB, 0.70346E+00_JPRB, 0.79783E+00_JPRB, 0.73113E+00_JPRB, 0.71504E+00_JPRB, 0.68390E+00_JPRB, 0.65115E+00_JPRB ,&
&  0.64929E+00_JPRB, 0.62840E+00_JPRB, 0.61292E+00_JPRB, 0.65630E+00_JPRB, 0.70328E+00_JPRB, 0.65423E+00_JPRB, 0.69901E+00_JPRB ,&
&  0.74325E+00_JPRB, 0.80989E+00_JPRB, 0.83270E+00_JPRB, 0.82544E+00_JPRB, 0.75916E+00_JPRB, 0.71733E+00_JPRB, 0.69839E+00_JPRB ,&
&  0.69126E+00_JPRB, 0.66735E+00_JPRB, 0.63548E+00_JPRB, 0.62811E+00_JPRB, 0.60899E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.61750E+00_JPRB, 0.60729E+00_JPRB, 0.58722E+00_JPRB, 0.58173E+00_JPRB, 0.57571E+00_JPRB, 0.57232E+00_JPRB, 0.56481E+00_JPRB ,&
&  0.55160E+00_JPRB, 0.54587E+00_JPRB, 0.54334E+00_JPRB, 0.54136E+00_JPRB, 0.53876E+00_JPRB, 0.53000E+00_JPRB, 0.52978E+00_JPRB ,&
&  0.53728E+00_JPRB, 0.64923E+00_JPRB, 0.57303E+00_JPRB, 0.52200E+00_JPRB, 0.50280E+00_JPRB, 0.59061E+00_JPRB, 0.71861E+00_JPRB ,&
&  0.90840E+00_JPRB, 0.11578E+01_JPRB, 0.15813E+01_JPRB, 0.19743E+01_JPRB, 0.24901E+01_JPRB, 0.29803E+01_JPRB, 0.33146E+01_JPRB ,&
&  0.34086E+01_JPRB, 0.26011E+01_JPRB, 0.23722E+01_JPRB, 0.20411E+01_JPRB, 0.18345E+01_JPRB, 0.16280E+01_JPRB, 0.14849E+01_JPRB ,&
&  0.13587E+01_JPRB, 0.12761E+01_JPRB, 0.12109E+01_JPRB, 0.11408E+01_JPRB, 0.10970E+01_JPRB, 0.10686E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.10286E+01_JPRB, 0.97642E+00_JPRB, 0.10459E+01_JPRB, 0.12725E+01_JPRB, 0.16983E+01_JPRB, 0.20595E+01_JPRB, 0.23198E+01_JPRB ,&
&  0.21465E+01_JPRB, 0.21073E+01_JPRB, 0.35278E+01_JPRB, 0.40018E+01_JPRB, 0.38279E+01_JPRB, 0.43089E+01_JPRB, 0.43224E+01_JPRB ,&
&  0.38421E+01_JPRB, 0.30242E+01_JPRB, 0.24988E+01_JPRB, 0.20469E+01_JPRB, 0.21469E+01_JPRB, 0.19581E+01_JPRB, 0.17606E+01_JPRB ,&
&  0.15719E+01_JPRB, 0.15976E+01_JPRB, 0.15501E+01_JPRB, 0.15093E+01_JPRB, 0.13943E+01_JPRB, 0.12561E+01_JPRB, 0.10158E+01_JPRB ,&
&  0.89028E+00_JPRB, 0.10148E+01_JPRB, 0.85895E+00_JPRB, 0.70594E+00_JPRB, 0.61598E+00_JPRB, 0.62868E+00_JPRB, 0.66039E+00_JPRB ,&
&  0.67952E+00_JPRB, 0.68002E+00_JPRB, 0.66560E+00_JPRB, 0.67505E+00_JPRB, 0.68904E+00_JPRB, 0.67275E+00_JPRB, 0.67978E+00_JPRB ,&
&  0.68179E+00_JPRB, 0.73679E+00_JPRB, 0.76200E+00_JPRB, 0.78236E+00_JPRB, 0.78264E+00_JPRB, 0.74216E+00_JPRB, 0.71854E+00_JPRB ,&
&  0.71249E+00_JPRB, 0.70862E+00_JPRB, 0.67998E+00_JPRB, 0.64867E+00_JPRB, 0.62319E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.60847E+00_JPRB, 0.59246E+00_JPRB, 0.57367E+00_JPRB, 0.57214E+00_JPRB, 0.57259E+00_JPRB, 0.57208E+00_JPRB, 0.55783E+00_JPRB ,&
&  0.52990E+00_JPRB, 0.51408E+00_JPRB, 0.51027E+00_JPRB, 0.51415E+00_JPRB, 0.51505E+00_JPRB, 0.51447E+00_JPRB, 0.52299E+00_JPRB ,&
&  0.52203E+00_JPRB, 0.73820E+00_JPRB, 0.64917E+00_JPRB, 0.61896E+00_JPRB, 0.55517E+00_JPRB, 0.61794E+00_JPRB, 0.70796E+00_JPRB ,&
&  0.82308E+00_JPRB, 0.89359E+00_JPRB, 0.10496E+01_JPRB, 0.12881E+01_JPRB, 0.15388E+01_JPRB, 0.17596E+01_JPRB, 0.21166E+01_JPRB ,&
&  0.21104E+01_JPRB, 0.19506E+01_JPRB, 0.16284E+01_JPRB, 0.15635E+01_JPRB, 0.14802E+01_JPRB, 0.12853E+01_JPRB, 0.12814E+01_JPRB ,&
&  0.12031E+01_JPRB, 0.11551E+01_JPRB, 0.11116E+01_JPRB, 0.10732E+01_JPRB, 0.10336E+01_JPRB, 0.98123E+00_JPRB, 0.95245E+00_JPRB ,&
&  0.93671E+00_JPRB, 0.92782E+00_JPRB, 0.99005E+00_JPRB, 0.10778E+01_JPRB, 0.13603E+01_JPRB, 0.17508E+01_JPRB, 0.20782E+01_JPRB ,&
&  0.25019E+01_JPRB, 0.22657E+01_JPRB, 0.20952E+01_JPRB, 0.29060E+01_JPRB, 0.41238E+01_JPRB, 0.44025E+01_JPRB, 0.40412E+01_JPRB ,&
&  0.40931E+01_JPRB, 0.36305E+01_JPRB, 0.30507E+01_JPRB, 0.26388E+01_JPRB, 0.23559E+01_JPRB, 0.21232E+01_JPRB, 0.19857E+01_JPRB ,&
&  0.17954E+01_JPRB, 0.17181E+01_JPRB, 0.17473E+01_JPRB, 0.16892E+01_JPRB, 0.15504E+01_JPRB, 0.13882E+01_JPRB, 0.13061E+01_JPRB ,&
&  0.11963E+01_JPRB, 0.99077E+00_JPRB, 0.71926E+00_JPRB, 0.69295E+00_JPRB, 0.58017E+00_JPRB, 0.58479E+00_JPRB, 0.65708E+00_JPRB ,&
&  0.65463E+00_JPRB, 0.68535E+00_JPRB, 0.68671E+00_JPRB, 0.65406E+00_JPRB, 0.66222E+00_JPRB, 0.64478E+00_JPRB, 0.65188E+00_JPRB ,&
&  0.66034E+00_JPRB, 0.61626E+00_JPRB, 0.65169E+00_JPRB, 0.69590E+00_JPRB, 0.73537E+00_JPRB, 0.73373E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71407E+00_JPRB, 0.70715E+00_JPRB, 0.68907E+00_JPRB, 0.65562E+00_JPRB, 0.62820E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.60533E+00_JPRB, 0.59035E+00_JPRB, 0.57411E+00_JPRB, 0.56438E+00_JPRB, 0.55900E+00_JPRB, 0.55777E+00_JPRB, 0.54514E+00_JPRB ,&
&  0.52318E+00_JPRB, 0.50684E+00_JPRB, 0.49938E+00_JPRB, 0.50158E+00_JPRB, 0.50120E+00_JPRB, 0.49427E+00_JPRB, 0.50737E+00_JPRB ,&
&  0.57296E+00_JPRB, 0.69895E+00_JPRB, 0.64280E+00_JPRB, 0.56728E+00_JPRB, 0.65868E+00_JPRB, 0.63507E+00_JPRB, 0.66696E+00_JPRB ,&
&  0.67462E+00_JPRB, 0.77171E+00_JPRB, 0.83083E+00_JPRB, 0.92064E+00_JPRB, 0.10126E+01_JPRB, 0.11290E+01_JPRB, 0.13064E+01_JPRB ,&
&  0.13463E+01_JPRB, 0.12191E+01_JPRB, 0.11268E+01_JPRB, 0.11315E+01_JPRB, 0.11954E+01_JPRB, 0.10863E+01_JPRB, 0.10773E+01_JPRB ,&
&  0.10348E+01_JPRB, 0.99871E+00_JPRB, 0.96999E+00_JPRB, 0.95160E+00_JPRB, 0.93205E+00_JPRB, 0.90456E+00_JPRB, 0.88502E+00_JPRB ,&
&  0.88039E+00_JPRB, 0.88098E+00_JPRB, 0.93850E+00_JPRB, 0.10004E+01_JPRB, 0.11638E+01_JPRB, 0.15780E+01_JPRB, 0.22801E+01_JPRB ,&
&  0.27210E+01_JPRB, 0.27556E+01_JPRB, 0.29530E+01_JPRB, 0.32818E+01_JPRB, 0.34416E+01_JPRB, 0.35825E+01_JPRB, 0.37321E+01_JPRB ,&
&  0.36682E+01_JPRB, 0.35609E+01_JPRB, 0.34272E+01_JPRB, 0.31711E+01_JPRB, 0.28066E+01_JPRB, 0.25169E+01_JPRB, 0.23035E+01_JPRB ,&
&  0.20581E+01_JPRB, 0.19171E+01_JPRB, 0.18915E+01_JPRB, 0.17650E+01_JPRB, 0.16680E+01_JPRB, 0.14894E+01_JPRB, 0.14735E+01_JPRB ,&
&  0.13208E+01_JPRB, 0.81972E+00_JPRB, 0.66503E+00_JPRB, 0.71184E+00_JPRB, 0.60497E+00_JPRB, 0.62537E+00_JPRB, 0.69178E+00_JPRB ,&
&  0.66806E+00_JPRB, 0.65321E+00_JPRB, 0.65820E+00_JPRB, 0.63712E+00_JPRB, 0.62054E+00_JPRB, 0.63208E+00_JPRB, 0.61403E+00_JPRB ,&
&  0.57491E+00_JPRB, 0.54504E+00_JPRB, 0.56122E+00_JPRB, 0.59766E+00_JPRB, 0.64721E+00_JPRB, 0.66645E+00_JPRB, 0.66691E+00_JPRB ,&
&  0.67150E+00_JPRB, 0.67296E+00_JPRB, 0.67098E+00_JPRB, 0.65321E+00_JPRB, 0.62386E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.61575E+00_JPRB, 0.60321E+00_JPRB, 0.59041E+00_JPRB, 0.57960E+00_JPRB, 0.56332E+00_JPRB, 0.54752E+00_JPRB, 0.52810E+00_JPRB ,&
&  0.51323E+00_JPRB, 0.49658E+00_JPRB, 0.48667E+00_JPRB, 0.48172E+00_JPRB, 0.47960E+00_JPRB, 0.47304E+00_JPRB, 0.49809E+00_JPRB ,&
&  0.47362E+00_JPRB, 0.54761E+00_JPRB, 0.51504E+00_JPRB, 0.55114E+00_JPRB, 0.67367E+00_JPRB, 0.62001E+00_JPRB, 0.62646E+00_JPRB ,&
&  0.64075E+00_JPRB, 0.68212E+00_JPRB, 0.69734E+00_JPRB, 0.71717E+00_JPRB, 0.73304E+00_JPRB, 0.75976E+00_JPRB, 0.84645E+00_JPRB ,&
&  0.87264E+00_JPRB, 0.81476E+00_JPRB, 0.80497E+00_JPRB, 0.82921E+00_JPRB, 0.88202E+00_JPRB, 0.94078E+00_JPRB, 0.97945E+00_JPRB ,&
&  0.94988E+00_JPRB, 0.92339E+00_JPRB, 0.90060E+00_JPRB, 0.87686E+00_JPRB, 0.85049E+00_JPRB, 0.82469E+00_JPRB, 0.80338E+00_JPRB ,&
&  0.79417E+00_JPRB, 0.80243E+00_JPRB, 0.85980E+00_JPRB, 0.94660E+00_JPRB, 0.10260E+01_JPRB, 0.14474E+01_JPRB, 0.23761E+01_JPRB ,&
&  0.29785E+01_JPRB, 0.29273E+01_JPRB, 0.31271E+01_JPRB, 0.35435E+01_JPRB, 0.35395E+01_JPRB, 0.34708E+01_JPRB, 0.33767E+01_JPRB ,&
&  0.33100E+01_JPRB, 0.32205E+01_JPRB, 0.30861E+01_JPRB, 0.30901E+01_JPRB, 0.29827E+01_JPRB, 0.28566E+01_JPRB, 0.26368E+01_JPRB ,&
&  0.23029E+01_JPRB, 0.20990E+01_JPRB, 0.20055E+01_JPRB, 0.18658E+01_JPRB, 0.17168E+01_JPRB, 0.16160E+01_JPRB, 0.14935E+01_JPRB ,&
&  0.13207E+01_JPRB, 0.11208E+01_JPRB, 0.85104E+00_JPRB, 0.78008E+00_JPRB, 0.63863E+00_JPRB, 0.65042E+00_JPRB, 0.72761E+00_JPRB ,&
&  0.68731E+00_JPRB, 0.62180E+00_JPRB, 0.59182E+00_JPRB, 0.57937E+00_JPRB, 0.57243E+00_JPRB, 0.57951E+00_JPRB, 0.54114E+00_JPRB ,&
&  0.50325E+00_JPRB, 0.50628E+00_JPRB, 0.51373E+00_JPRB, 0.53642E+00_JPRB, 0.57060E+00_JPRB, 0.60116E+00_JPRB, 0.56476E+00_JPRB ,&
&  0.60251E+00_JPRB, 0.62188E+00_JPRB, 0.63219E+00_JPRB, 0.63686E+00_JPRB, 0.62965E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.59343E+00_JPRB, 0.58531E+00_JPRB, 0.57798E+00_JPRB, 0.56979E+00_JPRB, 0.55223E+00_JPRB, 0.52413E+00_JPRB, 0.50200E+00_JPRB ,&
&  0.49674E+00_JPRB, 0.48539E+00_JPRB, 0.47364E+00_JPRB, 0.46321E+00_JPRB, 0.45571E+00_JPRB, 0.44857E+00_JPRB, 0.44489E+00_JPRB ,&
&  0.43713E+00_JPRB, 0.47868E+00_JPRB, 0.48406E+00_JPRB, 0.50817E+00_JPRB, 0.53210E+00_JPRB, 0.52217E+00_JPRB, 0.52686E+00_JPRB ,&
&  0.56162E+00_JPRB, 0.57985E+00_JPRB, 0.59018E+00_JPRB, 0.58980E+00_JPRB, 0.58733E+00_JPRB, 0.60643E+00_JPRB, 0.65775E+00_JPRB ,&
&  0.65337E+00_JPRB, 0.64803E+00_JPRB, 0.64836E+00_JPRB, 0.67400E+00_JPRB, 0.75243E+00_JPRB, 0.84390E+00_JPRB, 0.86758E+00_JPRB ,&
&  0.84978E+00_JPRB, 0.83129E+00_JPRB, 0.80765E+00_JPRB, 0.78639E+00_JPRB, 0.76648E+00_JPRB, 0.74626E+00_JPRB, 0.72713E+00_JPRB ,&
&  0.72104E+00_JPRB, 0.73437E+00_JPRB, 0.78132E+00_JPRB, 0.86533E+00_JPRB, 0.95859E+00_JPRB, 0.12431E+01_JPRB, 0.16914E+01_JPRB ,&
&  0.19496E+01_JPRB, 0.21197E+01_JPRB, 0.23367E+01_JPRB, 0.25669E+01_JPRB, 0.26819E+01_JPRB, 0.26716E+01_JPRB, 0.26316E+01_JPRB ,&
&  0.26439E+01_JPRB, 0.25903E+01_JPRB, 0.25356E+01_JPRB, 0.24339E+01_JPRB, 0.23435E+01_JPRB, 0.23455E+01_JPRB, 0.22692E+01_JPRB ,&
&  0.21039E+01_JPRB, 0.18723E+01_JPRB, 0.18546E+01_JPRB, 0.17782E+01_JPRB, 0.16843E+01_JPRB, 0.15785E+01_JPRB, 0.14530E+01_JPRB ,&
&  0.13087E+01_JPRB, 0.11726E+01_JPRB, 0.10007E+01_JPRB, 0.88922E+00_JPRB, 0.78462E+00_JPRB, 0.77808E+00_JPRB, 0.70708E+00_JPRB ,&
&  0.63247E+00_JPRB, 0.54917E+00_JPRB, 0.53905E+00_JPRB, 0.53276E+00_JPRB, 0.52929E+00_JPRB, 0.50636E+00_JPRB, 0.47239E+00_JPRB ,&
&  0.46470E+00_JPRB, 0.47962E+00_JPRB, 0.48559E+00_JPRB, 0.49413E+00_JPRB, 0.51554E+00_JPRB, 0.54894E+00_JPRB, 0.49408E+00_JPRB ,&
&  0.50541E+00_JPRB, 0.55516E+00_JPRB, 0.57875E+00_JPRB, 0.59166E+00_JPRB, 0.59558E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.55751E+00_JPRB, 0.55381E+00_JPRB, 0.54558E+00_JPRB, 0.53801E+00_JPRB, 0.52171E+00_JPRB, 0.49990E+00_JPRB, 0.47180E+00_JPRB ,&
&  0.48506E+00_JPRB, 0.47608E+00_JPRB, 0.46542E+00_JPRB, 0.45182E+00_JPRB, 0.43799E+00_JPRB, 0.41609E+00_JPRB, 0.38386E+00_JPRB ,&
&  0.38490E+00_JPRB, 0.40768E+00_JPRB, 0.44098E+00_JPRB, 0.46044E+00_JPRB, 0.46396E+00_JPRB, 0.46557E+00_JPRB, 0.47438E+00_JPRB ,&
&  0.48559E+00_JPRB, 0.49629E+00_JPRB, 0.51177E+00_JPRB, 0.52162E+00_JPRB, 0.52216E+00_JPRB, 0.52556E+00_JPRB, 0.54628E+00_JPRB ,&
&  0.52902E+00_JPRB, 0.52091E+00_JPRB, 0.53913E+00_JPRB, 0.52306E+00_JPRB, 0.61001E+00_JPRB, 0.66089E+00_JPRB, 0.68494E+00_JPRB ,&
&  0.67742E+00_JPRB, 0.65524E+00_JPRB, 0.63994E+00_JPRB, 0.64013E+00_JPRB, 0.64090E+00_JPRB, 0.64026E+00_JPRB, 0.64160E+00_JPRB ,&
&  0.64865E+00_JPRB, 0.66533E+00_JPRB, 0.70439E+00_JPRB, 0.76979E+00_JPRB, 0.85625E+00_JPRB, 0.10464E+01_JPRB, 0.12353E+01_JPRB ,&
&  0.14210E+01_JPRB, 0.16811E+01_JPRB, 0.19417E+01_JPRB, 0.19581E+01_JPRB, 0.21305E+01_JPRB, 0.22174E+01_JPRB, 0.21604E+01_JPRB ,&
&  0.21571E+01_JPRB, 0.20707E+01_JPRB, 0.20670E+01_JPRB, 0.20204E+01_JPRB, 0.19696E+01_JPRB, 0.19319E+01_JPRB, 0.18905E+01_JPRB ,&
&  0.18214E+01_JPRB, 0.16797E+01_JPRB, 0.16521E+01_JPRB, 0.15897E+01_JPRB, 0.15034E+01_JPRB, 0.14256E+01_JPRB, 0.13469E+01_JPRB ,&
&  0.12663E+01_JPRB, 0.11717E+01_JPRB, 0.10691E+01_JPRB, 0.98004E+00_JPRB, 0.90033E+00_JPRB, 0.81396E+00_JPRB, 0.73524E+00_JPRB ,&
&  0.66377E+00_JPRB, 0.57914E+00_JPRB, 0.53362E+00_JPRB, 0.52252E+00_JPRB, 0.50316E+00_JPRB, 0.46511E+00_JPRB, 0.44191E+00_JPRB ,&
&  0.43155E+00_JPRB, 0.44541E+00_JPRB, 0.46337E+00_JPRB, 0.46977E+00_JPRB, 0.48121E+00_JPRB, 0.49581E+00_JPRB, 0.47455E+00_JPRB ,&
&  0.46230E+00_JPRB, 0.50392E+00_JPRB, 0.52710E+00_JPRB, 0.54236E+00_JPRB, 0.55298E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.51724E+00_JPRB, 0.51798E+00_JPRB, 0.51009E+00_JPRB, 0.50244E+00_JPRB, 0.49741E+00_JPRB, 0.47616E+00_JPRB, 0.45912E+00_JPRB ,&
&  0.49209E+00_JPRB, 0.45715E+00_JPRB, 0.45247E+00_JPRB, 0.41017E+00_JPRB, 0.38897E+00_JPRB, 0.38164E+00_JPRB, 0.37567E+00_JPRB ,&
&  0.38024E+00_JPRB, 0.39528E+00_JPRB, 0.40898E+00_JPRB, 0.42021E+00_JPRB, 0.43298E+00_JPRB, 0.43655E+00_JPRB, 0.43958E+00_JPRB ,&
&  0.44380E+00_JPRB, 0.45059E+00_JPRB, 0.45975E+00_JPRB, 0.46095E+00_JPRB, 0.46159E+00_JPRB, 0.46291E+00_JPRB, 0.46573E+00_JPRB ,&
&  0.45483E+00_JPRB, 0.45352E+00_JPRB, 0.47560E+00_JPRB, 0.48954E+00_JPRB, 0.51767E+00_JPRB, 0.53376E+00_JPRB, 0.53477E+00_JPRB ,&
&  0.51962E+00_JPRB, 0.49251E+00_JPRB, 0.51704E+00_JPRB, 0.53191E+00_JPRB, 0.54520E+00_JPRB, 0.55386E+00_JPRB, 0.56016E+00_JPRB ,&
&  0.56393E+00_JPRB, 0.57177E+00_JPRB, 0.58917E+00_JPRB, 0.61982E+00_JPRB, 0.67138E+00_JPRB, 0.77668E+00_JPRB, 0.87589E+00_JPRB ,&
&  0.10870E+01_JPRB, 0.10588E+01_JPRB, 0.13367E+01_JPRB, 0.15082E+01_JPRB, 0.17040E+01_JPRB, 0.17651E+01_JPRB, 0.17219E+01_JPRB ,&
&  0.16426E+01_JPRB, 0.15936E+01_JPRB, 0.15727E+01_JPRB, 0.15127E+01_JPRB, 0.14585E+01_JPRB, 0.14228E+01_JPRB, 0.14190E+01_JPRB ,&
&  0.13917E+01_JPRB, 0.12707E+01_JPRB, 0.12935E+01_JPRB, 0.12414E+01_JPRB, 0.11908E+01_JPRB, 0.11610E+01_JPRB, 0.11382E+01_JPRB ,&
&  0.11100E+01_JPRB, 0.10590E+01_JPRB, 0.97989E+00_JPRB, 0.85644E+00_JPRB, 0.79926E+00_JPRB, 0.74845E+00_JPRB, 0.68380E+00_JPRB ,&
&  0.64290E+00_JPRB, 0.60008E+00_JPRB, 0.55914E+00_JPRB, 0.54538E+00_JPRB, 0.50270E+00_JPRB, 0.47059E+00_JPRB, 0.44526E+00_JPRB ,&
&  0.41754E+00_JPRB, 0.41489E+00_JPRB, 0.42651E+00_JPRB, 0.43442E+00_JPRB, 0.43407E+00_JPRB, 0.43452E+00_JPRB, 0.44258E+00_JPRB ,&
&  0.44224E+00_JPRB, 0.46152E+00_JPRB, 0.48817E+00_JPRB, 0.49916E+00_JPRB, 0.50822E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.48635E+00_JPRB, 0.49126E+00_JPRB, 0.48570E+00_JPRB, 0.47942E+00_JPRB, 0.47355E+00_JPRB, 0.46032E+00_JPRB, 0.43449E+00_JPRB ,&
&  0.43257E+00_JPRB, 0.40997E+00_JPRB, 0.38810E+00_JPRB, 0.38226E+00_JPRB, 0.38099E+00_JPRB, 0.37757E+00_JPRB, 0.37005E+00_JPRB ,&
&  0.37251E+00_JPRB, 0.38522E+00_JPRB, 0.40161E+00_JPRB, 0.41155E+00_JPRB, 0.42009E+00_JPRB, 0.42215E+00_JPRB, 0.42531E+00_JPRB ,&
&  0.42875E+00_JPRB, 0.42680E+00_JPRB, 0.42586E+00_JPRB, 0.42347E+00_JPRB, 0.42017E+00_JPRB, 0.42139E+00_JPRB, 0.42158E+00_JPRB ,&
&  0.42710E+00_JPRB, 0.43214E+00_JPRB, 0.44094E+00_JPRB, 0.45671E+00_JPRB, 0.46890E+00_JPRB, 0.47957E+00_JPRB, 0.45818E+00_JPRB ,&
&  0.40322E+00_JPRB, 0.41299E+00_JPRB, 0.44591E+00_JPRB, 0.46392E+00_JPRB, 0.48333E+00_JPRB, 0.49671E+00_JPRB, 0.50462E+00_JPRB ,&
&  0.50617E+00_JPRB, 0.50484E+00_JPRB, 0.50905E+00_JPRB, 0.52564E+00_JPRB, 0.55778E+00_JPRB, 0.61146E+00_JPRB, 0.66823E+00_JPRB ,&
&  0.78274E+00_JPRB, 0.79369E+00_JPRB, 0.83826E+00_JPRB, 0.96124E+00_JPRB, 0.12317E+01_JPRB, 0.12994E+01_JPRB, 0.13129E+01_JPRB ,&
&  0.13034E+01_JPRB, 0.12744E+01_JPRB, 0.12330E+01_JPRB, 0.11620E+01_JPRB, 0.10801E+01_JPRB, 0.10200E+01_JPRB, 0.98219E+00_JPRB ,&
&  0.95966E+00_JPRB, 0.95172E+00_JPRB, 0.93087E+00_JPRB, 0.91766E+00_JPRB, 0.88161E+00_JPRB, 0.86307E+00_JPRB, 0.85273E+00_JPRB ,&
&  0.84591E+00_JPRB, 0.82967E+00_JPRB, 0.78263E+00_JPRB, 0.72493E+00_JPRB, 0.66759E+00_JPRB, 0.62963E+00_JPRB, 0.60160E+00_JPRB ,&
&  0.57438E+00_JPRB, 0.55316E+00_JPRB, 0.53389E+00_JPRB, 0.51265E+00_JPRB, 0.48705E+00_JPRB, 0.46231E+00_JPRB, 0.43517E+00_JPRB ,&
&  0.40693E+00_JPRB, 0.39401E+00_JPRB, 0.39642E+00_JPRB, 0.39838E+00_JPRB, 0.40037E+00_JPRB, 0.40634E+00_JPRB, 0.40330E+00_JPRB ,&
&  0.41266E+00_JPRB, 0.42792E+00_JPRB, 0.43691E+00_JPRB, 0.44596E+00_JPRB, 0.46276E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.45909E+00_JPRB, 0.46670E+00_JPRB, 0.46427E+00_JPRB, 0.46511E+00_JPRB, 0.45572E+00_JPRB, 0.44342E+00_JPRB, 0.42955E+00_JPRB ,&
&  0.42665E+00_JPRB, 0.41618E+00_JPRB, 0.40144E+00_JPRB, 0.38664E+00_JPRB, 0.37893E+00_JPRB, 0.37640E+00_JPRB, 0.38040E+00_JPRB ,&
&  0.38900E+00_JPRB, 0.39983E+00_JPRB, 0.40847E+00_JPRB, 0.41297E+00_JPRB, 0.41203E+00_JPRB, 0.41086E+00_JPRB, 0.41308E+00_JPRB ,&
&  0.41353E+00_JPRB, 0.41012E+00_JPRB, 0.40911E+00_JPRB, 0.40607E+00_JPRB, 0.40495E+00_JPRB, 0.40515E+00_JPRB, 0.40397E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.40373E+00_JPRB, 0.41364E+00_JPRB, 0.42795E+00_JPRB, 0.43737E+00_JPRB, 0.43864E+00_JPRB, 0.40459E+00_JPRB ,&
&  0.37176E+00_JPRB, 0.36578E+00_JPRB, 0.39700E+00_JPRB, 0.41568E+00_JPRB, 0.43464E+00_JPRB, 0.44661E+00_JPRB, 0.45048E+00_JPRB ,&
&  0.44276E+00_JPRB, 0.43824E+00_JPRB, 0.44796E+00_JPRB, 0.47280E+00_JPRB, 0.49250E+00_JPRB, 0.52233E+00_JPRB, 0.55591E+00_JPRB ,&
&  0.60233E+00_JPRB, 0.66159E+00_JPRB, 0.72629E+00_JPRB, 0.72710E+00_JPRB, 0.79531E+00_JPRB, 0.10376E+01_JPRB, 0.10741E+01_JPRB ,&
&  0.10380E+01_JPRB, 0.10039E+01_JPRB, 0.10113E+01_JPRB, 0.94942E+00_JPRB, 0.87387E+00_JPRB, 0.81160E+00_JPRB, 0.75897E+00_JPRB ,&
&  0.71233E+00_JPRB, 0.68985E+00_JPRB, 0.67661E+00_JPRB, 0.68031E+00_JPRB, 0.67067E+00_JPRB, 0.65318E+00_JPRB, 0.64231E+00_JPRB ,&
&  0.63479E+00_JPRB, 0.61574E+00_JPRB, 0.59301E+00_JPRB, 0.55633E+00_JPRB, 0.52995E+00_JPRB, 0.51507E+00_JPRB, 0.50937E+00_JPRB ,&
&  0.50400E+00_JPRB, 0.50036E+00_JPRB, 0.49666E+00_JPRB, 0.48732E+00_JPRB, 0.46127E+00_JPRB, 0.43262E+00_JPRB, 0.41162E+00_JPRB ,&
&  0.39640E+00_JPRB, 0.38380E+00_JPRB, 0.38191E+00_JPRB, 0.38353E+00_JPRB, 0.38848E+00_JPRB, 0.39208E+00_JPRB, 0.38794E+00_JPRB ,&
&  0.38833E+00_JPRB, 0.39464E+00_JPRB, 0.41039E+00_JPRB, 0.42770E+00_JPRB, 0.44274E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.42049E+00_JPRB, 0.43683E+00_JPRB, 0.44946E+00_JPRB, 0.45436E+00_JPRB, 0.45323E+00_JPRB, 0.42880E+00_JPRB, 0.40564E+00_JPRB ,&
&  0.39316E+00_JPRB, 0.39076E+00_JPRB, 0.39296E+00_JPRB, 0.39563E+00_JPRB, 0.39927E+00_JPRB, 0.40401E+00_JPRB, 0.40796E+00_JPRB ,&
&  0.40957E+00_JPRB, 0.41088E+00_JPRB, 0.41378E+00_JPRB, 0.41826E+00_JPRB, 0.41745E+00_JPRB, 0.41483E+00_JPRB, 0.41475E+00_JPRB ,&
&  0.41390E+00_JPRB, 0.40960E+00_JPRB, 0.40775E+00_JPRB, 0.40246E+00_JPRB, 0.39890E+00_JPRB, 0.39631E+00_JPRB, 0.39125E+00_JPRB ,&
&  0.38502E+00_JPRB, 0.38424E+00_JPRB, 0.39287E+00_JPRB, 0.40423E+00_JPRB, 0.41319E+00_JPRB, 0.40553E+00_JPRB, 0.37866E+00_JPRB ,&
&  0.35513E+00_JPRB, 0.34298E+00_JPRB, 0.34702E+00_JPRB, 0.36988E+00_JPRB, 0.39200E+00_JPRB, 0.38869E+00_JPRB, 0.39883E+00_JPRB ,&
&  0.40325E+00_JPRB, 0.40718E+00_JPRB, 0.41397E+00_JPRB, 0.42807E+00_JPRB, 0.44538E+00_JPRB, 0.46457E+00_JPRB, 0.48375E+00_JPRB ,&
&  0.50733E+00_JPRB, 0.53616E+00_JPRB, 0.57181E+00_JPRB, 0.58254E+00_JPRB, 0.57268E+00_JPRB, 0.61103E+00_JPRB, 0.66124E+00_JPRB ,&
&  0.70676E+00_JPRB, 0.71197E+00_JPRB, 0.74116E+00_JPRB, 0.74802E+00_JPRB, 0.73712E+00_JPRB, 0.70901E+00_JPRB, 0.67346E+00_JPRB ,&
&  0.64336E+00_JPRB, 0.60862E+00_JPRB, 0.58145E+00_JPRB, 0.57148E+00_JPRB, 0.56621E+00_JPRB, 0.55470E+00_JPRB, 0.54210E+00_JPRB ,&
&  0.52871E+00_JPRB, 0.50533E+00_JPRB, 0.47637E+00_JPRB, 0.44812E+00_JPRB, 0.44714E+00_JPRB, 0.46158E+00_JPRB, 0.47655E+00_JPRB ,&
&  0.47936E+00_JPRB, 0.47729E+00_JPRB, 0.47617E+00_JPRB, 0.46971E+00_JPRB, 0.44494E+00_JPRB, 0.42682E+00_JPRB, 0.41454E+00_JPRB ,&
&  0.40751E+00_JPRB, 0.39811E+00_JPRB, 0.39736E+00_JPRB, 0.39518E+00_JPRB, 0.39531E+00_JPRB, 0.39489E+00_JPRB, 0.38887E+00_JPRB ,&
&  0.38605E+00_JPRB, 0.38668E+00_JPRB, 0.39306E+00_JPRB, 0.40004E+00_JPRB, 0.40879E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.41930E+00_JPRB, 0.42697E+00_JPRB, 0.43639E+00_JPRB, 0.44307E+00_JPRB, 0.44462E+00_JPRB, 0.43477E+00_JPRB, 0.42559E+00_JPRB ,&
&  0.42003E+00_JPRB, 0.42056E+00_JPRB, 0.42069E+00_JPRB, 0.42071E+00_JPRB, 0.42163E+00_JPRB, 0.42258E+00_JPRB, 0.42385E+00_JPRB ,&
&  0.42543E+00_JPRB, 0.42694E+00_JPRB, 0.42703E+00_JPRB, 0.42193E+00_JPRB, 0.41707E+00_JPRB, 0.41567E+00_JPRB, 0.41684E+00_JPRB ,&
&  0.41740E+00_JPRB, 0.41386E+00_JPRB, 0.41123E+00_JPRB, 0.40389E+00_JPRB, 0.39461E+00_JPRB, 0.38532E+00_JPRB, 0.37717E+00_JPRB ,&
&  0.37631E+00_JPRB, 0.37892E+00_JPRB, 0.37811E+00_JPRB, 0.38307E+00_JPRB, 0.39135E+00_JPRB, 0.37698E+00_JPRB, 0.35716E+00_JPRB ,&
&  0.34056E+00_JPRB, 0.32859E+00_JPRB, 0.32314E+00_JPRB, 0.32366E+00_JPRB, 0.33822E+00_JPRB, 0.35992E+00_JPRB, 0.37111E+00_JPRB ,&
&  0.36181E+00_JPRB, 0.37485E+00_JPRB, 0.39078E+00_JPRB, 0.40165E+00_JPRB, 0.41280E+00_JPRB, 0.42667E+00_JPRB, 0.43972E+00_JPRB ,&
&  0.45232E+00_JPRB, 0.46394E+00_JPRB, 0.47439E+00_JPRB, 0.49359E+00_JPRB, 0.50415E+00_JPRB, 0.49391E+00_JPRB, 0.50160E+00_JPRB ,&
&  0.57518E+00_JPRB, 0.61769E+00_JPRB, 0.65467E+00_JPRB, 0.66251E+00_JPRB, 0.65558E+00_JPRB, 0.63282E+00_JPRB, 0.60961E+00_JPRB ,&
&  0.58959E+00_JPRB, 0.57034E+00_JPRB, 0.55236E+00_JPRB, 0.54093E+00_JPRB, 0.53371E+00_JPRB, 0.52158E+00_JPRB, 0.50807E+00_JPRB ,&
&  0.48913E+00_JPRB, 0.47273E+00_JPRB, 0.53757E+00_JPRB, 0.51412E+00_JPRB, 0.50232E+00_JPRB, 0.49968E+00_JPRB, 0.49241E+00_JPRB ,&
&  0.48529E+00_JPRB, 0.48003E+00_JPRB, 0.47676E+00_JPRB, 0.47152E+00_JPRB, 0.45654E+00_JPRB, 0.44325E+00_JPRB, 0.43525E+00_JPRB ,&
&  0.43089E+00_JPRB, 0.42518E+00_JPRB, 0.42021E+00_JPRB, 0.41604E+00_JPRB, 0.41175E+00_JPRB, 0.40726E+00_JPRB, 0.39874E+00_JPRB ,&
&  0.39274E+00_JPRB, 0.39437E+00_JPRB, 0.39937E+00_JPRB, 0.40776E+00_JPRB, 0.41352E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.41493E+00_JPRB, 0.42168E+00_JPRB, 0.42827E+00_JPRB, 0.43362E+00_JPRB, 0.43615E+00_JPRB, 0.43592E+00_JPRB, 0.43375E+00_JPRB ,&
&  0.43149E+00_JPRB, 0.43100E+00_JPRB, 0.43063E+00_JPRB, 0.43102E+00_JPRB, 0.43151E+00_JPRB, 0.43218E+00_JPRB, 0.43257E+00_JPRB ,&
&  0.43392E+00_JPRB, 0.42539E+00_JPRB, 0.42019E+00_JPRB, 0.42013E+00_JPRB, 0.41960E+00_JPRB, 0.42003E+00_JPRB, 0.42134E+00_JPRB ,&
&  0.41806E+00_JPRB, 0.41039E+00_JPRB, 0.40417E+00_JPRB, 0.39622E+00_JPRB, 0.38806E+00_JPRB, 0.38713E+00_JPRB, 0.38600E+00_JPRB ,&
&  0.37985E+00_JPRB, 0.37411E+00_JPRB, 0.37814E+00_JPRB, 0.38050E+00_JPRB, 0.36905E+00_JPRB, 0.35506E+00_JPRB, 0.33137E+00_JPRB ,&
&  0.32667E+00_JPRB, 0.31731E+00_JPRB, 0.31552E+00_JPRB, 0.31619E+00_JPRB, 0.32288E+00_JPRB, 0.33163E+00_JPRB, 0.34685E+00_JPRB ,&
&  0.35405E+00_JPRB, 0.35970E+00_JPRB, 0.37055E+00_JPRB, 0.38245E+00_JPRB, 0.39108E+00_JPRB, 0.39890E+00_JPRB, 0.40351E+00_JPRB ,&
&  0.41185E+00_JPRB, 0.42154E+00_JPRB, 0.43022E+00_JPRB, 0.43799E+00_JPRB, 0.44698E+00_JPRB, 0.45100E+00_JPRB, 0.45650E+00_JPRB ,&
&  0.48111E+00_JPRB, 0.49982E+00_JPRB, 0.51438E+00_JPRB, 0.52738E+00_JPRB, 0.53514E+00_JPRB, 0.53616E+00_JPRB, 0.52531E+00_JPRB ,&
&  0.51607E+00_JPRB, 0.51155E+00_JPRB, 0.50570E+00_JPRB, 0.50172E+00_JPRB, 0.49891E+00_JPRB, 0.49597E+00_JPRB, 0.48943E+00_JPRB ,&
&  0.47312E+00_JPRB, 0.46252E+00_JPRB, 0.47542E+00_JPRB, 0.47904E+00_JPRB, 0.47296E+00_JPRB, 0.46148E+00_JPRB, 0.45138E+00_JPRB ,&
&  0.44465E+00_JPRB, 0.44308E+00_JPRB, 0.44346E+00_JPRB, 0.44524E+00_JPRB, 0.44702E+00_JPRB, 0.44530E+00_JPRB, 0.44663E+00_JPRB ,&
&  0.44399E+00_JPRB, 0.43920E+00_JPRB, 0.43446E+00_JPRB, 0.42979E+00_JPRB, 0.42540E+00_JPRB, 0.42079E+00_JPRB, 0.40969E+00_JPRB ,&
&  0.39743E+00_JPRB, 0.39085E+00_JPRB, 0.39282E+00_JPRB, 0.39871E+00_JPRB, 0.40717E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.40945E+00_JPRB, 0.41358E+00_JPRB, 0.41828E+00_JPRB, 0.42007E+00_JPRB, 0.42213E+00_JPRB, 0.42184E+00_JPRB, 0.42210E+00_JPRB ,&
&  0.42171E+00_JPRB, 0.42229E+00_JPRB, 0.42263E+00_JPRB, 0.42342E+00_JPRB, 0.42398E+00_JPRB, 0.42490E+00_JPRB, 0.42542E+00_JPRB ,&
&  0.42540E+00_JPRB, 0.42515E+00_JPRB, 0.42367E+00_JPRB, 0.42226E+00_JPRB, 0.42050E+00_JPRB, 0.42085E+00_JPRB, 0.41962E+00_JPRB ,&
&  0.41642E+00_JPRB, 0.40995E+00_JPRB, 0.39726E+00_JPRB, 0.39013E+00_JPRB, 0.38914E+00_JPRB, 0.38311E+00_JPRB, 0.38432E+00_JPRB ,&
&  0.38197E+00_JPRB, 0.38120E+00_JPRB, 0.37778E+00_JPRB, 0.37703E+00_JPRB, 0.36865E+00_JPRB, 0.32221E+00_JPRB, 0.31736E+00_JPRB ,&
&  0.31358E+00_JPRB, 0.30850E+00_JPRB, 0.30803E+00_JPRB, 0.30929E+00_JPRB, 0.31384E+00_JPRB, 0.32712E+00_JPRB, 0.33990E+00_JPRB ,&
&  0.34177E+00_JPRB, 0.35919E+00_JPRB, 0.36415E+00_JPRB, 0.36881E+00_JPRB, 0.37743E+00_JPRB, 0.38491E+00_JPRB, 0.38649E+00_JPRB ,&
&  0.39017E+00_JPRB, 0.39419E+00_JPRB, 0.40032E+00_JPRB, 0.40756E+00_JPRB, 0.41385E+00_JPRB, 0.41781E+00_JPRB, 0.42261E+00_JPRB ,&
&  0.42749E+00_JPRB, 0.43211E+00_JPRB, 0.43721E+00_JPRB, 0.44145E+00_JPRB, 0.44423E+00_JPRB, 0.44665E+00_JPRB, 0.45023E+00_JPRB ,&
&  0.45430E+00_JPRB, 0.45689E+00_JPRB, 0.46089E+00_JPRB, 0.45670E+00_JPRB, 0.45586E+00_JPRB, 0.45412E+00_JPRB, 0.45682E+00_JPRB ,&
&  0.45514E+00_JPRB, 0.45729E+00_JPRB, 0.45391E+00_JPRB, 0.45399E+00_JPRB, 0.44769E+00_JPRB, 0.44313E+00_JPRB, 0.43945E+00_JPRB ,&
&  0.43863E+00_JPRB, 0.43759E+00_JPRB, 0.43363E+00_JPRB, 0.43584E+00_JPRB, 0.43960E+00_JPRB, 0.44412E+00_JPRB, 0.44711E+00_JPRB ,&
&  0.44781E+00_JPRB, 0.44545E+00_JPRB, 0.44221E+00_JPRB, 0.43703E+00_JPRB, 0.43284E+00_JPRB, 0.42900E+00_JPRB, 0.41861E+00_JPRB ,&
&  0.40697E+00_JPRB, 0.39636E+00_JPRB, 0.39418E+00_JPRB, 0.39569E+00_JPRB, 0.40181E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.40275E+00_JPRB, 0.40459E+00_JPRB, 0.40790E+00_JPRB, 0.40826E+00_JPRB, 0.41035E+00_JPRB, 0.41065E+00_JPRB, 0.41136E+00_JPRB ,&
&  0.41111E+00_JPRB, 0.41146E+00_JPRB, 0.41124E+00_JPRB, 0.41168E+00_JPRB, 0.41172E+00_JPRB, 0.41264E+00_JPRB, 0.41311E+00_JPRB ,&
&  0.41396E+00_JPRB, 0.41437E+00_JPRB, 0.41507E+00_JPRB, 0.41499E+00_JPRB, 0.41532E+00_JPRB, 0.41529E+00_JPRB, 0.41378E+00_JPRB ,&
&  0.41084E+00_JPRB, 0.40419E+00_JPRB, 0.38764E+00_JPRB, 0.38371E+00_JPRB, 0.38020E+00_JPRB, 0.38012E+00_JPRB, 0.37902E+00_JPRB ,&
&  0.37814E+00_JPRB, 0.34619E+00_JPRB, 0.34693E+00_JPRB, 0.33870E+00_JPRB, 0.33334E+00_JPRB, 0.32813E+00_JPRB, 0.32050E+00_JPRB ,&
&  0.31766E+00_JPRB, 0.31500E+00_JPRB, 0.31483E+00_JPRB, 0.31864E+00_JPRB, 0.32307E+00_JPRB, 0.33019E+00_JPRB, 0.33874E+00_JPRB ,&
&  0.34052E+00_JPRB, 0.35403E+00_JPRB, 0.35978E+00_JPRB, 0.36154E+00_JPRB, 0.37071E+00_JPRB, 0.37777E+00_JPRB, 0.37956E+00_JPRB ,&
&  0.38225E+00_JPRB, 0.38172E+00_JPRB, 0.38449E+00_JPRB, 0.39026E+00_JPRB, 0.39648E+00_JPRB, 0.39846E+00_JPRB, 0.40023E+00_JPRB ,&
&  0.40131E+00_JPRB, 0.40162E+00_JPRB, 0.40158E+00_JPRB, 0.40018E+00_JPRB, 0.39939E+00_JPRB, 0.40082E+00_JPRB, 0.40373E+00_JPRB ,&
&  0.40770E+00_JPRB, 0.41180E+00_JPRB, 0.41742E+00_JPRB, 0.42064E+00_JPRB, 0.42293E+00_JPRB, 0.42569E+00_JPRB, 0.42789E+00_JPRB ,&
&  0.42921E+00_JPRB, 0.42985E+00_JPRB, 0.42827E+00_JPRB, 0.42784E+00_JPRB, 0.42818E+00_JPRB, 0.42719E+00_JPRB, 0.42760E+00_JPRB ,&
&  0.43103E+00_JPRB, 0.43140E+00_JPRB, 0.43069E+00_JPRB, 0.43104E+00_JPRB, 0.43381E+00_JPRB, 0.43755E+00_JPRB, 0.44033E+00_JPRB ,&
&  0.44202E+00_JPRB, 0.44116E+00_JPRB, 0.43896E+00_JPRB, 0.43403E+00_JPRB, 0.42908E+00_JPRB, 0.42628E+00_JPRB, 0.42043E+00_JPRB ,&
&  0.41438E+00_JPRB, 0.40666E+00_JPRB, 0.40086E+00_JPRB, 0.39762E+00_JPRB, 0.39948E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.39786E+00_JPRB, 0.39918E+00_JPRB, 0.40197E+00_JPRB, 0.40354E+00_JPRB, 0.40614E+00_JPRB, 0.40688E+00_JPRB, 0.40821E+00_JPRB ,&
&  0.40818E+00_JPRB, 0.40852E+00_JPRB, 0.40858E+00_JPRB, 0.40848E+00_JPRB, 0.40835E+00_JPRB, 0.40807E+00_JPRB, 0.40810E+00_JPRB ,&
&  0.40860E+00_JPRB, 0.40863E+00_JPRB, 0.40964E+00_JPRB, 0.40991E+00_JPRB, 0.41013E+00_JPRB, 0.41018E+00_JPRB, 0.40750E+00_JPRB ,&
&  0.40407E+00_JPRB, 0.40109E+00_JPRB, 0.37802E+00_JPRB, 0.37542E+00_JPRB, 0.37060E+00_JPRB, 0.36813E+00_JPRB, 0.36860E+00_JPRB ,&
&  0.36620E+00_JPRB, 0.36326E+00_JPRB, 0.35443E+00_JPRB, 0.34488E+00_JPRB, 0.33686E+00_JPRB, 0.32703E+00_JPRB, 0.32463E+00_JPRB ,&
&  0.32239E+00_JPRB, 0.32433E+00_JPRB, 0.32681E+00_JPRB, 0.32919E+00_JPRB, 0.33543E+00_JPRB, 0.33991E+00_JPRB, 0.34696E+00_JPRB ,&
&  0.35555E+00_JPRB, 0.36192E+00_JPRB, 0.36259E+00_JPRB, 0.35786E+00_JPRB, 0.36697E+00_JPRB, 0.37204E+00_JPRB, 0.37702E+00_JPRB ,&
&  0.37976E+00_JPRB, 0.38039E+00_JPRB, 0.38276E+00_JPRB, 0.38377E+00_JPRB, 0.38475E+00_JPRB, 0.38697E+00_JPRB, 0.38729E+00_JPRB ,&
&  0.38806E+00_JPRB, 0.38688E+00_JPRB, 0.38485E+00_JPRB, 0.38182E+00_JPRB, 0.37831E+00_JPRB, 0.37635E+00_JPRB, 0.37607E+00_JPRB ,&
&  0.37727E+00_JPRB, 0.38005E+00_JPRB, 0.38360E+00_JPRB, 0.38796E+00_JPRB, 0.39184E+00_JPRB, 0.39529E+00_JPRB, 0.39841E+00_JPRB ,&
&  0.40082E+00_JPRB, 0.40262E+00_JPRB, 0.40643E+00_JPRB, 0.40536E+00_JPRB, 0.40686E+00_JPRB, 0.41235E+00_JPRB, 0.41241E+00_JPRB ,&
&  0.41485E+00_JPRB, 0.41860E+00_JPRB, 0.42079E+00_JPRB, 0.42064E+00_JPRB, 0.42130E+00_JPRB, 0.42311E+00_JPRB, 0.42517E+00_JPRB ,&
&  0.42657E+00_JPRB, 0.42618E+00_JPRB, 0.42436E+00_JPRB, 0.42167E+00_JPRB, 0.41907E+00_JPRB, 0.41658E+00_JPRB, 0.41289E+00_JPRB ,&
&  0.41017E+00_JPRB, 0.40694E+00_JPRB, 0.40311E+00_JPRB, 0.39883E+00_JPRB, 0.39768E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.39797E+00_JPRB, 0.39728E+00_JPRB, 0.39741E+00_JPRB, 0.39816E+00_JPRB, 0.39944E+00_JPRB, 0.40032E+00_JPRB, 0.40148E+00_JPRB ,&
&  0.40217E+00_JPRB, 0.40257E+00_JPRB, 0.40224E+00_JPRB, 0.40249E+00_JPRB, 0.40241E+00_JPRB, 0.40267E+00_JPRB, 0.40225E+00_JPRB ,&
&  0.40214E+00_JPRB, 0.40143E+00_JPRB, 0.40210E+00_JPRB, 0.40236E+00_JPRB, 0.40285E+00_JPRB, 0.40352E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.39932E+00_JPRB, 0.39794E+00_JPRB, 0.39411E+00_JPRB, 0.39057E+00_JPRB, 0.38805E+00_JPRB, 0.36870E+00_JPRB, 0.36267E+00_JPRB ,&
&  0.36728E+00_JPRB, 0.37029E+00_JPRB, 0.37376E+00_JPRB, 0.37655E+00_JPRB, 0.37160E+00_JPRB, 0.36350E+00_JPRB, 0.35900E+00_JPRB ,&
&  0.34334E+00_JPRB, 0.34574E+00_JPRB, 0.34744E+00_JPRB, 0.34900E+00_JPRB, 0.35271E+00_JPRB, 0.35393E+00_JPRB, 0.35762E+00_JPRB ,&
&  0.36783E+00_JPRB, 0.37019E+00_JPRB, 0.37322E+00_JPRB, 0.37501E+00_JPRB, 0.37526E+00_JPRB, 0.37602E+00_JPRB, 0.37928E+00_JPRB ,&
&  0.38060E+00_JPRB, 0.38183E+00_JPRB, 0.38261E+00_JPRB, 0.38239E+00_JPRB, 0.38256E+00_JPRB, 0.38201E+00_JPRB, 0.38102E+00_JPRB ,&
&  0.37925E+00_JPRB, 0.37830E+00_JPRB, 0.37717E+00_JPRB, 0.37574E+00_JPRB, 0.37477E+00_JPRB, 0.37301E+00_JPRB, 0.37177E+00_JPRB ,&
&  0.37132E+00_JPRB, 0.37167E+00_JPRB, 0.37251E+00_JPRB, 0.37391E+00_JPRB, 0.37587E+00_JPRB, 0.37852E+00_JPRB, 0.38122E+00_JPRB ,&
&  0.38423E+00_JPRB, 0.38714E+00_JPRB, 0.38965E+00_JPRB, 0.39189E+00_JPRB, 0.39385E+00_JPRB, 0.39588E+00_JPRB, 0.39767E+00_JPRB ,&
&  0.40006E+00_JPRB, 0.40278E+00_JPRB, 0.40440E+00_JPRB, 0.40460E+00_JPRB, 0.40541E+00_JPRB, 0.40618E+00_JPRB, 0.40723E+00_JPRB ,&
&  0.40810E+00_JPRB, 0.40918E+00_JPRB, 0.40940E+00_JPRB, 0.40899E+00_JPRB, 0.40827E+00_JPRB, 0.40783E+00_JPRB, 0.40697E+00_JPRB ,&
&  0.40661E+00_JPRB, 0.40642E+00_JPRB, 0.40546E+00_JPRB, 0.40303E+00_JPRB, 0.40024E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.40190E+00_JPRB, 0.40081E+00_JPRB, 0.39986E+00_JPRB, 0.39922E+00_JPRB, 0.39855E+00_JPRB, 0.39834E+00_JPRB, 0.39826E+00_JPRB ,&
&  0.39826E+00_JPRB, 0.39807E+00_JPRB, 0.39761E+00_JPRB, 0.39744E+00_JPRB, 0.39759E+00_JPRB, 0.39790E+00_JPRB, 0.39762E+00_JPRB ,&
&  0.39749E+00_JPRB, 0.39729E+00_JPRB, 0.39740E+00_JPRB, 0.39715E+00_JPRB, 0.39680E+00_JPRB, 0.39630E+00_JPRB, 0.39592E+00_JPRB ,&
&  0.39558E+00_JPRB, 0.39495E+00_JPRB, 0.39402E+00_JPRB, 0.39228E+00_JPRB, 0.39062E+00_JPRB, 0.38874E+00_JPRB, 0.38673E+00_JPRB ,&
&  0.38443E+00_JPRB, 0.38254E+00_JPRB, 0.38092E+00_JPRB, 0.37939E+00_JPRB, 0.38052E+00_JPRB, 0.38108E+00_JPRB, 0.38132E+00_JPRB ,&
&  0.38110E+00_JPRB, 0.37943E+00_JPRB, 0.37892E+00_JPRB, 0.37855E+00_JPRB, 0.37829E+00_JPRB, 0.37967E+00_JPRB, 0.38060E+00_JPRB ,&
&  0.38125E+00_JPRB, 0.38173E+00_JPRB, 0.38318E+00_JPRB, 0.38389E+00_JPRB, 0.38429E+00_JPRB, 0.38413E+00_JPRB, 0.38358E+00_JPRB ,&
&  0.38374E+00_JPRB, 0.38389E+00_JPRB, 0.38382E+00_JPRB, 0.38340E+00_JPRB, 0.38325E+00_JPRB, 0.38290E+00_JPRB, 0.38203E+00_JPRB ,&
&  0.38100E+00_JPRB, 0.38035E+00_JPRB, 0.37953E+00_JPRB, 0.37851E+00_JPRB, 0.37741E+00_JPRB, 0.37675E+00_JPRB, 0.37602E+00_JPRB ,&
&  0.37535E+00_JPRB, 0.37493E+00_JPRB, 0.37470E+00_JPRB, 0.37452E+00_JPRB, 0.37441E+00_JPRB, 0.37478E+00_JPRB, 0.37586E+00_JPRB ,&
&  0.37725E+00_JPRB, 0.37902E+00_JPRB, 0.38076E+00_JPRB, 0.38253E+00_JPRB, 0.38400E+00_JPRB, 0.38544E+00_JPRB, 0.38733E+00_JPRB ,&
&  0.38935E+00_JPRB, 0.39097E+00_JPRB, 0.39244E+00_JPRB, 0.39395E+00_JPRB, 0.39569E+00_JPRB, 0.39670E+00_JPRB, 0.39744E+00_JPRB ,&
&  0.39806E+00_JPRB, 0.39938E+00_JPRB, 0.40061E+00_JPRB, 0.40185E+00_JPRB, 0.40290E+00_JPRB, 0.40369E+00_JPRB, 0.40408E+00_JPRB ,&
&  0.40434E+00_JPRB, 0.40472E+00_JPRB, 0.40480E+00_JPRB, 0.40413E+00_JPRB, 0.40324E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.40144E+00_JPRB, 0.40130E+00_JPRB, 0.40105E+00_JPRB, 0.40079E+00_JPRB, 0.40045E+00_JPRB, 0.40018E+00_JPRB, 0.39984E+00_JPRB ,&
&  0.39960E+00_JPRB, 0.39932E+00_JPRB, 0.39918E+00_JPRB, 0.39898E+00_JPRB, 0.39886E+00_JPRB, 0.39867E+00_JPRB, 0.39846E+00_JPRB ,&
&  0.39821E+00_JPRB, 0.39809E+00_JPRB, 0.39777E+00_JPRB, 0.39749E+00_JPRB, 0.39706E+00_JPRB, 0.39673E+00_JPRB, 0.39627E+00_JPRB ,&
&  0.39592E+00_JPRB, 0.39540E+00_JPRB, 0.39499E+00_JPRB, 0.39427E+00_JPRB, 0.39375E+00_JPRB, 0.39315E+00_JPRB, 0.39265E+00_JPRB ,&
&  0.39212E+00_JPRB, 0.39178E+00_JPRB, 0.39141E+00_JPRB, 0.39098E+00_JPRB, 0.39069E+00_JPRB, 0.39079E+00_JPRB, 0.39089E+00_JPRB ,&
&  0.39094E+00_JPRB, 0.39104E+00_JPRB, 0.39133E+00_JPRB, 0.39145E+00_JPRB, 0.39121E+00_JPRB, 0.39104E+00_JPRB, 0.39094E+00_JPRB ,&
&  0.39078E+00_JPRB, 0.39048E+00_JPRB, 0.39019E+00_JPRB, 0.38999E+00_JPRB, 0.38965E+00_JPRB, 0.38919E+00_JPRB, 0.38882E+00_JPRB ,&
&  0.38860E+00_JPRB, 0.38837E+00_JPRB, 0.38806E+00_JPRB, 0.38763E+00_JPRB, 0.38727E+00_JPRB, 0.38688E+00_JPRB, 0.38642E+00_JPRB ,&
&  0.38594E+00_JPRB, 0.38545E+00_JPRB, 0.38490E+00_JPRB, 0.38428E+00_JPRB, 0.38366E+00_JPRB, 0.38325E+00_JPRB, 0.38297E+00_JPRB ,&
&  0.38278E+00_JPRB, 0.38279E+00_JPRB, 0.38287E+00_JPRB, 0.38299E+00_JPRB, 0.38314E+00_JPRB, 0.38344E+00_JPRB, 0.38383E+00_JPRB ,&
&  0.38424E+00_JPRB, 0.38471E+00_JPRB, 0.38535E+00_JPRB, 0.38622E+00_JPRB, 0.38703E+00_JPRB, 0.38786E+00_JPRB, 0.38890E+00_JPRB ,&
&  0.38986E+00_JPRB, 0.39063E+00_JPRB, 0.39124E+00_JPRB, 0.39202E+00_JPRB, 0.39308E+00_JPRB, 0.39388E+00_JPRB, 0.39454E+00_JPRB ,&
&  0.39514E+00_JPRB, 0.39581E+00_JPRB, 0.39645E+00_JPRB, 0.39716E+00_JPRB, 0.39772E+00_JPRB, 0.39833E+00_JPRB, 0.39897E+00_JPRB ,&
&  0.39955E+00_JPRB, 0.40014E+00_JPRB, 0.40070E+00_JPRB, 0.40113E+00_JPRB, 0.40140E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB, 0.39464E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB, 0.15476E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.16794E+00_JPRB, 0.16823E+00_JPRB, 0.16879E+00_JPRB, 0.16946E+00_JPRB, 0.17017E+00_JPRB, 0.17103E+00_JPRB, 0.17137E+00_JPRB ,&
&  0.17152E+00_JPRB, 0.17153E+00_JPRB, 0.17186E+00_JPRB, 0.17225E+00_JPRB, 0.17251E+00_JPRB, 0.17250E+00_JPRB, 0.17212E+00_JPRB ,&
&  0.17148E+00_JPRB, 0.17074E+00_JPRB, 0.16974E+00_JPRB, 0.16927E+00_JPRB, 0.16892E+00_JPRB, 0.16852E+00_JPRB, 0.16775E+00_JPRB ,&
&  0.16708E+00_JPRB, 0.16588E+00_JPRB, 0.16420E+00_JPRB, 0.16277E+00_JPRB, 0.16241E+00_JPRB, 0.16248E+00_JPRB, 0.16288E+00_JPRB ,&
&  0.16320E+00_JPRB, 0.16337E+00_JPRB, 0.16338E+00_JPRB, 0.16346E+00_JPRB, 0.16360E+00_JPRB, 0.16343E+00_JPRB, 0.16315E+00_JPRB ,&
&  0.16282E+00_JPRB, 0.16231E+00_JPRB, 0.16182E+00_JPRB, 0.16138E+00_JPRB, 0.16097E+00_JPRB, 0.16050E+00_JPRB, 0.15996E+00_JPRB ,&
&  0.15934E+00_JPRB, 0.15857E+00_JPRB, 0.15769E+00_JPRB, 0.15696E+00_JPRB, 0.15622E+00_JPRB, 0.15563E+00_JPRB, 0.15492E+00_JPRB ,&
&  0.15433E+00_JPRB, 0.15379E+00_JPRB, 0.15324E+00_JPRB, 0.15272E+00_JPRB, 0.15223E+00_JPRB, 0.15175E+00_JPRB, 0.15127E+00_JPRB ,&
&  0.15082E+00_JPRB, 0.15044E+00_JPRB, 0.15002E+00_JPRB, 0.14962E+00_JPRB, 0.14935E+00_JPRB, 0.14914E+00_JPRB, 0.14894E+00_JPRB ,&
&  0.14878E+00_JPRB, 0.14871E+00_JPRB, 0.14869E+00_JPRB, 0.14866E+00_JPRB, 0.14864E+00_JPRB, 0.14874E+00_JPRB, 0.14885E+00_JPRB ,&
&  0.14900E+00_JPRB, 0.14911E+00_JPRB, 0.14925E+00_JPRB, 0.14943E+00_JPRB, 0.14964E+00_JPRB, 0.14988E+00_JPRB, 0.15020E+00_JPRB ,&
&  0.15060E+00_JPRB, 0.15108E+00_JPRB, 0.15158E+00_JPRB, 0.15193E+00_JPRB, 0.15258E+00_JPRB, 0.15330E+00_JPRB, 0.15412E+00_JPRB ,&
&  0.15504E+00_JPRB, 0.15610E+00_JPRB, 0.15721E+00_JPRB, 0.15845E+00_JPRB, 0.15969E+00_JPRB, 0.16052E+00_JPRB, 0.16136E+00_JPRB ,&
&  0.16225E+00_JPRB, 0.16327E+00_JPRB, 0.16426E+00_JPRB, 0.16527E+00_JPRB, 0.16652E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.18984E+00_JPRB, 0.19687E+00_JPRB, 0.20406E+00_JPRB, 0.21427E+00_JPRB, 0.22253E+00_JPRB, 0.25769E+00_JPRB, 0.25257E+00_JPRB ,&
&  0.24890E+00_JPRB, 0.23443E+00_JPRB, 0.23256E+00_JPRB, 0.22879E+00_JPRB, 0.22420E+00_JPRB, 0.21917E+00_JPRB, 0.21417E+00_JPRB ,&
&  0.20948E+00_JPRB, 0.20401E+00_JPRB, 0.19919E+00_JPRB, 0.19543E+00_JPRB, 0.19206E+00_JPRB, 0.18877E+00_JPRB, 0.18587E+00_JPRB ,&
&  0.18455E+00_JPRB, 0.18429E+00_JPRB, 0.18344E+00_JPRB, 0.17975E+00_JPRB, 0.17646E+00_JPRB, 0.17439E+00_JPRB, 0.17368E+00_JPRB ,&
&  0.17355E+00_JPRB, 0.17489E+00_JPRB, 0.17556E+00_JPRB, 0.17603E+00_JPRB, 0.17683E+00_JPRB, 0.17600E+00_JPRB, 0.17517E+00_JPRB ,&
&  0.17406E+00_JPRB, 0.17278E+00_JPRB, 0.17141E+00_JPRB, 0.17025E+00_JPRB, 0.16907E+00_JPRB, 0.16736E+00_JPRB, 0.16583E+00_JPRB ,&
&  0.16428E+00_JPRB, 0.16241E+00_JPRB, 0.16006E+00_JPRB, 0.15849E+00_JPRB, 0.15670E+00_JPRB, 0.15517E+00_JPRB, 0.15366E+00_JPRB ,&
&  0.15255E+00_JPRB, 0.15151E+00_JPRB, 0.15028E+00_JPRB, 0.14876E+00_JPRB, 0.14724E+00_JPRB, 0.14589E+00_JPRB, 0.14458E+00_JPRB ,&
&  0.14385E+00_JPRB, 0.14311E+00_JPRB, 0.14237E+00_JPRB, 0.14158E+00_JPRB, 0.14118E+00_JPRB, 0.14079E+00_JPRB, 0.14044E+00_JPRB ,&
&  0.14015E+00_JPRB, 0.14002E+00_JPRB, 0.14007E+00_JPRB, 0.14017E+00_JPRB, 0.14027E+00_JPRB, 0.14063E+00_JPRB, 0.14105E+00_JPRB ,&
&  0.14162E+00_JPRB, 0.14233E+00_JPRB, 0.14327E+00_JPRB, 0.14424E+00_JPRB, 0.14512E+00_JPRB, 0.14585E+00_JPRB, 0.14683E+00_JPRB ,&
&  0.14838E+00_JPRB, 0.15018E+00_JPRB, 0.15214E+00_JPRB, 0.15372E+00_JPRB, 0.15581E+00_JPRB, 0.15769E+00_JPRB, 0.15957E+00_JPRB ,&
&  0.16142E+00_JPRB, 0.16354E+00_JPRB, 0.16583E+00_JPRB, 0.16823E+00_JPRB, 0.17093E+00_JPRB, 0.17354E+00_JPRB, 0.17595E+00_JPRB ,&
&  0.17842E+00_JPRB, 0.18172E+00_JPRB, 0.18384E+00_JPRB, 0.18524E+00_JPRB, 0.18711E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.26260E+00_JPRB, 0.26726E+00_JPRB, 0.27085E+00_JPRB, 0.27312E+00_JPRB, 0.27519E+00_JPRB, 0.27673E+00_JPRB, 0.27725E+00_JPRB ,&
&  0.27086E+00_JPRB, 0.26583E+00_JPRB, 0.25896E+00_JPRB, 0.25080E+00_JPRB, 0.24337E+00_JPRB, 0.23705E+00_JPRB, 0.22937E+00_JPRB ,&
&  0.22311E+00_JPRB, 0.21718E+00_JPRB, 0.21080E+00_JPRB, 0.20419E+00_JPRB, 0.19722E+00_JPRB, 0.19021E+00_JPRB, 0.18639E+00_JPRB ,&
&  0.18502E+00_JPRB, 0.18569E+00_JPRB, 0.18714E+00_JPRB, 0.18868E+00_JPRB, 0.18961E+00_JPRB, 0.19029E+00_JPRB, 0.18918E+00_JPRB ,&
&  0.18564E+00_JPRB, 0.18606E+00_JPRB, 0.18665E+00_JPRB, 0.18699E+00_JPRB, 0.18825E+00_JPRB, 0.18938E+00_JPRB, 0.18909E+00_JPRB ,&
&  0.18742E+00_JPRB, 0.18514E+00_JPRB, 0.18200E+00_JPRB, 0.17910E+00_JPRB, 0.17782E+00_JPRB, 0.17556E+00_JPRB, 0.17300E+00_JPRB ,&
&  0.17071E+00_JPRB, 0.16688E+00_JPRB, 0.16320E+00_JPRB, 0.16061E+00_JPRB, 0.15742E+00_JPRB, 0.15520E+00_JPRB, 0.15232E+00_JPRB ,&
&  0.14996E+00_JPRB, 0.14778E+00_JPRB, 0.14538E+00_JPRB, 0.14347E+00_JPRB, 0.14166E+00_JPRB, 0.14031E+00_JPRB, 0.13899E+00_JPRB ,&
&  0.13820E+00_JPRB, 0.13749E+00_JPRB, 0.13705E+00_JPRB, 0.13657E+00_JPRB, 0.13632E+00_JPRB, 0.13606E+00_JPRB, 0.13586E+00_JPRB ,&
&  0.13576E+00_JPRB, 0.13605E+00_JPRB, 0.13627E+00_JPRB, 0.13667E+00_JPRB, 0.13713E+00_JPRB, 0.13745E+00_JPRB, 0.13798E+00_JPRB ,&
&  0.13897E+00_JPRB, 0.14002E+00_JPRB, 0.14078E+00_JPRB, 0.14173E+00_JPRB, 0.14279E+00_JPRB, 0.14410E+00_JPRB, 0.14582E+00_JPRB ,&
&  0.14764E+00_JPRB, 0.14966E+00_JPRB, 0.15192E+00_JPRB, 0.15418E+00_JPRB, 0.15702E+00_JPRB, 0.15992E+00_JPRB, 0.16297E+00_JPRB ,&
&  0.16655E+00_JPRB, 0.17037E+00_JPRB, 0.17425E+00_JPRB, 0.17852E+00_JPRB, 0.18466E+00_JPRB, 0.19075E+00_JPRB, 0.19836E+00_JPRB ,&
&  0.20594E+00_JPRB, 0.21279E+00_JPRB, 0.24425E+00_JPRB, 0.24910E+00_JPRB, 0.25573E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.26907E+00_JPRB, 0.27971E+00_JPRB, 0.28850E+00_JPRB, 0.29397E+00_JPRB, 0.29975E+00_JPRB, 0.30209E+00_JPRB, 0.31081E+00_JPRB ,&
&  0.30533E+00_JPRB, 0.28706E+00_JPRB, 0.27588E+00_JPRB, 0.25956E+00_JPRB, 0.23928E+00_JPRB, 0.22287E+00_JPRB, 0.21478E+00_JPRB ,&
&  0.21005E+00_JPRB, 0.20643E+00_JPRB, 0.20196E+00_JPRB, 0.19702E+00_JPRB, 0.19200E+00_JPRB, 0.18831E+00_JPRB, 0.18580E+00_JPRB ,&
&  0.18357E+00_JPRB, 0.18390E+00_JPRB, 0.18610E+00_JPRB, 0.19009E+00_JPRB, 0.19571E+00_JPRB, 0.19997E+00_JPRB, 0.20321E+00_JPRB ,&
&  0.20598E+00_JPRB, 0.19788E+00_JPRB, 0.19864E+00_JPRB, 0.19988E+00_JPRB, 0.19832E+00_JPRB, 0.20041E+00_JPRB, 0.20360E+00_JPRB ,&
&  0.20192E+00_JPRB, 0.19681E+00_JPRB, 0.19983E+00_JPRB, 0.19610E+00_JPRB, 0.18797E+00_JPRB, 0.18562E+00_JPRB, 0.17987E+00_JPRB ,&
&  0.17618E+00_JPRB, 0.16903E+00_JPRB, 0.16402E+00_JPRB, 0.16075E+00_JPRB, 0.15607E+00_JPRB, 0.15406E+00_JPRB, 0.15097E+00_JPRB ,&
&  0.14768E+00_JPRB, 0.14525E+00_JPRB, 0.14239E+00_JPRB, 0.14085E+00_JPRB, 0.13956E+00_JPRB, 0.13844E+00_JPRB, 0.13757E+00_JPRB ,&
&  0.13710E+00_JPRB, 0.13666E+00_JPRB, 0.13670E+00_JPRB, 0.13676E+00_JPRB, 0.13685E+00_JPRB, 0.13722E+00_JPRB, 0.13750E+00_JPRB ,&
&  0.13812E+00_JPRB, 0.13880E+00_JPRB, 0.13934E+00_JPRB, 0.14017E+00_JPRB, 0.14059E+00_JPRB, 0.14110E+00_JPRB, 0.14151E+00_JPRB ,&
&  0.14198E+00_JPRB, 0.14265E+00_JPRB, 0.14309E+00_JPRB, 0.14380E+00_JPRB, 0.14463E+00_JPRB, 0.14611E+00_JPRB, 0.14758E+00_JPRB ,&
&  0.14936E+00_JPRB, 0.15135E+00_JPRB, 0.15351E+00_JPRB, 0.15577E+00_JPRB, 0.15857E+00_JPRB, 0.16120E+00_JPRB, 0.16460E+00_JPRB ,&
&  0.16828E+00_JPRB, 0.17175E+00_JPRB, 0.17562E+00_JPRB, 0.18066E+00_JPRB, 0.18719E+00_JPRB, 0.19656E+00_JPRB, 0.20569E+00_JPRB ,&
&  0.21623E+00_JPRB, 0.24315E+00_JPRB, 0.24955E+00_JPRB, 0.25254E+00_JPRB, 0.26074E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.29546E+00_JPRB, 0.30474E+00_JPRB, 0.31879E+00_JPRB, 0.32775E+00_JPRB, 0.33507E+00_JPRB, 0.33093E+00_JPRB, 0.32705E+00_JPRB ,&
&  0.32305E+00_JPRB, 0.30877E+00_JPRB, 0.29170E+00_JPRB, 0.26248E+00_JPRB, 0.23783E+00_JPRB, 0.21895E+00_JPRB, 0.20875E+00_JPRB ,&
&  0.20288E+00_JPRB, 0.20106E+00_JPRB, 0.19951E+00_JPRB, 0.19929E+00_JPRB, 0.19847E+00_JPRB, 0.20031E+00_JPRB, 0.20187E+00_JPRB ,&
&  0.20163E+00_JPRB, 0.20045E+00_JPRB, 0.19970E+00_JPRB, 0.20274E+00_JPRB, 0.20857E+00_JPRB, 0.21318E+00_JPRB, 0.21757E+00_JPRB ,&
&  0.22090E+00_JPRB, 0.21848E+00_JPRB, 0.21934E+00_JPRB, 0.21716E+00_JPRB, 0.21446E+00_JPRB, 0.21410E+00_JPRB, 0.21458E+00_JPRB ,&
&  0.21503E+00_JPRB, 0.21583E+00_JPRB, 0.21530E+00_JPRB, 0.21269E+00_JPRB, 0.20153E+00_JPRB, 0.19563E+00_JPRB, 0.18824E+00_JPRB ,&
&  0.18025E+00_JPRB, 0.17101E+00_JPRB, 0.16376E+00_JPRB, 0.16019E+00_JPRB, 0.15625E+00_JPRB, 0.15457E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.14808E+00_JPRB, 0.14474E+00_JPRB, 0.14234E+00_JPRB, 0.14111E+00_JPRB, 0.14065E+00_JPRB, 0.13980E+00_JPRB, 0.13945E+00_JPRB ,&
&  0.13964E+00_JPRB, 0.13990E+00_JPRB, 0.14059E+00_JPRB, 0.14134E+00_JPRB, 0.14220E+00_JPRB, 0.14304E+00_JPRB, 0.14409E+00_JPRB ,&
&  0.14565E+00_JPRB, 0.14746E+00_JPRB, 0.14829E+00_JPRB, 0.14896E+00_JPRB, 0.14876E+00_JPRB, 0.14842E+00_JPRB, 0.14786E+00_JPRB ,&
&  0.14740E+00_JPRB, 0.14729E+00_JPRB, 0.14729E+00_JPRB, 0.14727E+00_JPRB, 0.14775E+00_JPRB, 0.14863E+00_JPRB, 0.14962E+00_JPRB ,&
&  0.15114E+00_JPRB, 0.15254E+00_JPRB, 0.15411E+00_JPRB, 0.15587E+00_JPRB, 0.15795E+00_JPRB, 0.16036E+00_JPRB, 0.16310E+00_JPRB ,&
&  0.16568E+00_JPRB, 0.16886E+00_JPRB, 0.17155E+00_JPRB, 0.17369E+00_JPRB, 0.17817E+00_JPRB, 0.18497E+00_JPRB, 0.19840E+00_JPRB ,&
&  0.21454E+00_JPRB, 0.25551E+00_JPRB, 0.29853E+00_JPRB, 0.28299E+00_JPRB, 0.28756E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.29440E+00_JPRB, 0.30629E+00_JPRB, 0.32160E+00_JPRB, 0.33095E+00_JPRB, 0.33598E+00_JPRB, 0.33849E+00_JPRB, 0.33717E+00_JPRB ,&
&  0.33375E+00_JPRB, 0.33335E+00_JPRB, 0.34456E+00_JPRB, 0.32717E+00_JPRB, 0.30927E+00_JPRB, 0.27563E+00_JPRB, 0.25922E+00_JPRB ,&
&  0.24716E+00_JPRB, 0.24513E+00_JPRB, 0.24508E+00_JPRB, 0.24990E+00_JPRB, 0.25400E+00_JPRB, 0.25727E+00_JPRB, 0.25390E+00_JPRB ,&
&  0.24890E+00_JPRB, 0.23986E+00_JPRB, 0.23228E+00_JPRB, 0.22508E+00_JPRB, 0.22505E+00_JPRB, 0.22399E+00_JPRB, 0.22143E+00_JPRB ,&
&  0.21838E+00_JPRB, 0.21631E+00_JPRB, 0.21600E+00_JPRB, 0.21577E+00_JPRB, 0.22738E+00_JPRB, 0.22699E+00_JPRB, 0.22652E+00_JPRB ,&
&  0.22592E+00_JPRB, 0.22585E+00_JPRB, 0.22491E+00_JPRB, 0.22320E+00_JPRB, 0.21863E+00_JPRB, 0.20743E+00_JPRB, 0.20739E+00_JPRB ,&
&  0.19744E+00_JPRB, 0.18672E+00_JPRB, 0.17659E+00_JPRB, 0.16958E+00_JPRB, 0.16530E+00_JPRB, 0.16317E+00_JPRB, 0.15858E+00_JPRB ,&
&  0.15371E+00_JPRB, 0.14823E+00_JPRB, 0.14550E+00_JPRB, 0.14505E+00_JPRB, 0.14549E+00_JPRB, 0.14602E+00_JPRB, 0.14650E+00_JPRB ,&
&  0.14759E+00_JPRB, 0.14817E+00_JPRB, 0.14847E+00_JPRB, 0.14906E+00_JPRB, 0.14964E+00_JPRB, 0.15059E+00_JPRB, 0.15219E+00_JPRB ,&
&  0.15510E+00_JPRB, 0.15833E+00_JPRB, 0.16126E+00_JPRB, 0.16340E+00_JPRB, 0.16245E+00_JPRB, 0.15960E+00_JPRB, 0.15570E+00_JPRB ,&
&  0.15293E+00_JPRB, 0.15212E+00_JPRB, 0.15138E+00_JPRB, 0.15142E+00_JPRB, 0.15171E+00_JPRB, 0.15220E+00_JPRB, 0.15286E+00_JPRB ,&
&  0.15367E+00_JPRB, 0.15434E+00_JPRB, 0.15517E+00_JPRB, 0.15621E+00_JPRB, 0.15775E+00_JPRB, 0.15914E+00_JPRB, 0.16110E+00_JPRB ,&
&  0.16404E+00_JPRB, 0.16725E+00_JPRB, 0.17179E+00_JPRB, 0.17585E+00_JPRB, 0.18099E+00_JPRB, 0.18890E+00_JPRB, 0.20194E+00_JPRB ,&
&  0.21624E+00_JPRB, 0.25125E+00_JPRB, 0.26769E+00_JPRB, 0.27718E+00_JPRB, 0.28414E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.29185E+00_JPRB, 0.30060E+00_JPRB, 0.31278E+00_JPRB, 0.32311E+00_JPRB, 0.33193E+00_JPRB, 0.34037E+00_JPRB, 0.34753E+00_JPRB ,&
&  0.35113E+00_JPRB, 0.35322E+00_JPRB, 0.35393E+00_JPRB, 0.35733E+00_JPRB, 0.35442E+00_JPRB, 0.34805E+00_JPRB, 0.34268E+00_JPRB ,&
&  0.33832E+00_JPRB, 0.33480E+00_JPRB, 0.32962E+00_JPRB, 0.32691E+00_JPRB, 0.32670E+00_JPRB, 0.32691E+00_JPRB, 0.32134E+00_JPRB ,&
&  0.31122E+00_JPRB, 0.31080E+00_JPRB, 0.31575E+00_JPRB, 0.31074E+00_JPRB, 0.30531E+00_JPRB, 0.30025E+00_JPRB, 0.29459E+00_JPRB ,&
&  0.27942E+00_JPRB, 0.25991E+00_JPRB, 0.23260E+00_JPRB, 0.22728E+00_JPRB, 0.24316E+00_JPRB, 0.23756E+00_JPRB, 0.24011E+00_JPRB ,&
&  0.24030E+00_JPRB, 0.24078E+00_JPRB, 0.23930E+00_JPRB, 0.23790E+00_JPRB, 0.23435E+00_JPRB, 0.23293E+00_JPRB, 0.23164E+00_JPRB ,&
&  0.22674E+00_JPRB, 0.22208E+00_JPRB, 0.21469E+00_JPRB, 0.19767E+00_JPRB, 0.19186E+00_JPRB, 0.18932E+00_JPRB, 0.18403E+00_JPRB ,&
&  0.17638E+00_JPRB, 0.17006E+00_JPRB, 0.16712E+00_JPRB, 0.16401E+00_JPRB, 0.16603E+00_JPRB, 0.16749E+00_JPRB, 0.16751E+00_JPRB ,&
&  0.16750E+00_JPRB, 0.16716E+00_JPRB, 0.16491E+00_JPRB, 0.16124E+00_JPRB, 0.15840E+00_JPRB, 0.15726E+00_JPRB, 0.15761E+00_JPRB ,&
&  0.15972E+00_JPRB, 0.16420E+00_JPRB, 0.17448E+00_JPRB, 0.18268E+00_JPRB, 0.18205E+00_JPRB, 0.17723E+00_JPRB, 0.16864E+00_JPRB ,&
&  0.16195E+00_JPRB, 0.15931E+00_JPRB, 0.15713E+00_JPRB, 0.15606E+00_JPRB, 0.15649E+00_JPRB, 0.15792E+00_JPRB, 0.15884E+00_JPRB ,&
&  0.15986E+00_JPRB, 0.16076E+00_JPRB, 0.16156E+00_JPRB, 0.16213E+00_JPRB, 0.16304E+00_JPRB, 0.16405E+00_JPRB, 0.16586E+00_JPRB ,&
&  0.16858E+00_JPRB, 0.17285E+00_JPRB, 0.17850E+00_JPRB, 0.18453E+00_JPRB, 0.18881E+00_JPRB, 0.19086E+00_JPRB, 0.19407E+00_JPRB ,&
&  0.19969E+00_JPRB, 0.21368E+00_JPRB, 0.24167E+00_JPRB, 0.28760E+00_JPRB, 0.29215E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.29475E+00_JPRB, 0.29909E+00_JPRB, 0.30190E+00_JPRB, 0.30541E+00_JPRB, 0.31295E+00_JPRB, 0.32375E+00_JPRB, 0.33598E+00_JPRB ,&
&  0.34697E+00_JPRB, 0.35565E+00_JPRB, 0.35985E+00_JPRB, 0.36494E+00_JPRB, 0.36987E+00_JPRB, 0.37233E+00_JPRB, 0.37763E+00_JPRB ,&
&  0.37842E+00_JPRB, 0.37511E+00_JPRB, 0.36910E+00_JPRB, 0.36108E+00_JPRB, 0.35688E+00_JPRB, 0.35829E+00_JPRB, 0.36137E+00_JPRB ,&
&  0.36347E+00_JPRB, 0.36512E+00_JPRB, 0.36531E+00_JPRB, 0.36210E+00_JPRB, 0.35889E+00_JPRB, 0.35705E+00_JPRB, 0.35332E+00_JPRB ,&
&  0.34537E+00_JPRB, 0.29637E+00_JPRB, 0.25853E+00_JPRB, 0.25356E+00_JPRB, 0.27107E+00_JPRB, 0.27359E+00_JPRB, 0.26809E+00_JPRB ,&
&  0.26218E+00_JPRB, 0.25765E+00_JPRB, 0.25510E+00_JPRB, 0.25302E+00_JPRB, 0.25037E+00_JPRB, 0.24817E+00_JPRB, 0.24919E+00_JPRB ,&
&  0.25051E+00_JPRB, 0.24830E+00_JPRB, 0.24519E+00_JPRB, 0.23899E+00_JPRB, 0.23443E+00_JPRB, 0.23207E+00_JPRB, 0.23007E+00_JPRB ,&
&  0.22581E+00_JPRB, 0.22171E+00_JPRB, 0.22134E+00_JPRB, 0.22215E+00_JPRB, 0.22359E+00_JPRB, 0.22460E+00_JPRB, 0.22213E+00_JPRB ,&
&  0.21532E+00_JPRB, 0.20903E+00_JPRB, 0.20073E+00_JPRB, 0.19095E+00_JPRB, 0.18151E+00_JPRB, 0.17507E+00_JPRB, 0.17014E+00_JPRB ,&
&  0.16892E+00_JPRB, 0.17350E+00_JPRB, 0.18543E+00_JPRB, 0.20095E+00_JPRB, 0.22306E+00_JPRB, 0.21055E+00_JPRB, 0.19880E+00_JPRB ,&
&  0.18861E+00_JPRB, 0.18083E+00_JPRB, 0.17502E+00_JPRB, 0.17229E+00_JPRB, 0.17126E+00_JPRB, 0.17138E+00_JPRB, 0.17245E+00_JPRB ,&
&  0.17435E+00_JPRB, 0.17693E+00_JPRB, 0.17936E+00_JPRB, 0.18054E+00_JPRB, 0.18130E+00_JPRB, 0.18202E+00_JPRB, 0.18215E+00_JPRB ,&
&  0.18152E+00_JPRB, 0.18359E+00_JPRB, 0.19003E+00_JPRB, 0.19836E+00_JPRB, 0.20668E+00_JPRB, 0.21428E+00_JPRB, 0.22714E+00_JPRB ,&
&  0.24293E+00_JPRB, 0.26329E+00_JPRB, 0.27482E+00_JPRB, 0.28346E+00_JPRB, 0.29119E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.31944E+00_JPRB, 0.32089E+00_JPRB, 0.31832E+00_JPRB, 0.31536E+00_JPRB, 0.31491E+00_JPRB, 0.31679E+00_JPRB, 0.32281E+00_JPRB ,&
&  0.33264E+00_JPRB, 0.34495E+00_JPRB, 0.35668E+00_JPRB, 0.36650E+00_JPRB, 0.37547E+00_JPRB, 0.37950E+00_JPRB, 0.38561E+00_JPRB ,&
&  0.39211E+00_JPRB, 0.39802E+00_JPRB, 0.39880E+00_JPRB, 0.38856E+00_JPRB, 0.37860E+00_JPRB, 0.37332E+00_JPRB, 0.37336E+00_JPRB ,&
&  0.37526E+00_JPRB, 0.37807E+00_JPRB, 0.38443E+00_JPRB, 0.38696E+00_JPRB, 0.38127E+00_JPRB, 0.38005E+00_JPRB, 0.37807E+00_JPRB ,&
&  0.37279E+00_JPRB, 0.36226E+00_JPRB, 0.33030E+00_JPRB, 0.32534E+00_JPRB, 0.31747E+00_JPRB, 0.30860E+00_JPRB, 0.29953E+00_JPRB ,&
&  0.28932E+00_JPRB, 0.27923E+00_JPRB, 0.27240E+00_JPRB, 0.26846E+00_JPRB, 0.26932E+00_JPRB, 0.27053E+00_JPRB, 0.26994E+00_JPRB ,&
&  0.27188E+00_JPRB, 0.27470E+00_JPRB, 0.27684E+00_JPRB, 0.27691E+00_JPRB, 0.27626E+00_JPRB, 0.27475E+00_JPRB, 0.26978E+00_JPRB ,&
&  0.26524E+00_JPRB, 0.26261E+00_JPRB, 0.26053E+00_JPRB, 0.25870E+00_JPRB, 0.25813E+00_JPRB, 0.25834E+00_JPRB, 0.25834E+00_JPRB ,&
&  0.25625E+00_JPRB, 0.25261E+00_JPRB, 0.24851E+00_JPRB, 0.24182E+00_JPRB, 0.23092E+00_JPRB, 0.21483E+00_JPRB, 0.20542E+00_JPRB ,&
&  0.20372E+00_JPRB, 0.21483E+00_JPRB, 0.23689E+00_JPRB, 0.24444E+00_JPRB, 0.25166E+00_JPRB, 0.25581E+00_JPRB, 0.26064E+00_JPRB ,&
&  0.25752E+00_JPRB, 0.23242E+00_JPRB, 0.21886E+00_JPRB, 0.21219E+00_JPRB, 0.20648E+00_JPRB, 0.20179E+00_JPRB, 0.20161E+00_JPRB ,&
&  0.20460E+00_JPRB, 0.21132E+00_JPRB, 0.21763E+00_JPRB, 0.22069E+00_JPRB, 0.22010E+00_JPRB, 0.21660E+00_JPRB, 0.21324E+00_JPRB ,&
&  0.21001E+00_JPRB, 0.21414E+00_JPRB, 0.22871E+00_JPRB, 0.25244E+00_JPRB, 0.26040E+00_JPRB, 0.26644E+00_JPRB, 0.27213E+00_JPRB ,&
&  0.27814E+00_JPRB, 0.28727E+00_JPRB, 0.29854E+00_JPRB, 0.30760E+00_JPRB, 0.31566E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.33455E+00_JPRB, 0.33427E+00_JPRB, 0.33565E+00_JPRB, 0.33449E+00_JPRB, 0.33194E+00_JPRB, 0.33028E+00_JPRB, 0.33424E+00_JPRB ,&
&  0.34136E+00_JPRB, 0.35285E+00_JPRB, 0.36796E+00_JPRB, 0.38125E+00_JPRB, 0.38937E+00_JPRB, 0.39445E+00_JPRB, 0.40222E+00_JPRB ,&
&  0.40937E+00_JPRB, 0.41727E+00_JPRB, 0.41740E+00_JPRB, 0.40848E+00_JPRB, 0.40090E+00_JPRB, 0.39688E+00_JPRB, 0.39628E+00_JPRB ,&
&  0.39564E+00_JPRB, 0.39751E+00_JPRB, 0.40087E+00_JPRB, 0.40450E+00_JPRB, 0.40407E+00_JPRB, 0.40246E+00_JPRB, 0.40641E+00_JPRB ,&
&  0.40492E+00_JPRB, 0.39590E+00_JPRB, 0.38693E+00_JPRB, 0.36230E+00_JPRB, 0.36214E+00_JPRB, 0.35223E+00_JPRB, 0.34446E+00_JPRB ,&
&  0.33401E+00_JPRB, 0.32124E+00_JPRB, 0.31204E+00_JPRB, 0.30615E+00_JPRB, 0.30356E+00_JPRB, 0.30095E+00_JPRB, 0.29881E+00_JPRB ,&
&  0.29739E+00_JPRB, 0.29695E+00_JPRB, 0.29803E+00_JPRB, 0.29578E+00_JPRB, 0.29464E+00_JPRB, 0.29549E+00_JPRB, 0.29712E+00_JPRB ,&
&  0.29652E+00_JPRB, 0.29564E+00_JPRB, 0.29492E+00_JPRB, 0.29357E+00_JPRB, 0.29175E+00_JPRB, 0.28929E+00_JPRB, 0.28685E+00_JPRB ,&
&  0.28569E+00_JPRB, 0.28645E+00_JPRB, 0.28655E+00_JPRB, 0.28264E+00_JPRB, 0.27502E+00_JPRB, 0.26625E+00_JPRB, 0.26057E+00_JPRB ,&
&  0.25805E+00_JPRB, 0.25990E+00_JPRB, 0.26746E+00_JPRB, 0.27837E+00_JPRB, 0.28869E+00_JPRB, 0.29472E+00_JPRB, 0.29830E+00_JPRB ,&
&  0.30276E+00_JPRB, 0.30152E+00_JPRB, 0.29804E+00_JPRB, 0.29476E+00_JPRB, 0.29168E+00_JPRB, 0.28771E+00_JPRB, 0.28759E+00_JPRB ,&
&  0.28987E+00_JPRB, 0.29199E+00_JPRB, 0.29251E+00_JPRB, 0.29119E+00_JPRB, 0.28902E+00_JPRB, 0.28462E+00_JPRB, 0.27845E+00_JPRB ,&
&  0.27098E+00_JPRB, 0.26238E+00_JPRB, 0.26005E+00_JPRB, 0.26032E+00_JPRB, 0.26405E+00_JPRB, 0.27190E+00_JPRB, 0.28258E+00_JPRB ,&
&  0.29386E+00_JPRB, 0.30333E+00_JPRB, 0.31346E+00_JPRB, 0.32280E+00_JPRB, 0.33104E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.35883E+00_JPRB, 0.36080E+00_JPRB, 0.36317E+00_JPRB, 0.36623E+00_JPRB, 0.36964E+00_JPRB, 0.37188E+00_JPRB, 0.37603E+00_JPRB ,&
&  0.37834E+00_JPRB, 0.38464E+00_JPRB, 0.39509E+00_JPRB, 0.40573E+00_JPRB, 0.41363E+00_JPRB, 0.42045E+00_JPRB, 0.42698E+00_JPRB ,&
&  0.43222E+00_JPRB, 0.43920E+00_JPRB, 0.44000E+00_JPRB, 0.43744E+00_JPRB, 0.43461E+00_JPRB, 0.43238E+00_JPRB, 0.43040E+00_JPRB ,&
&  0.42786E+00_JPRB, 0.42774E+00_JPRB, 0.43158E+00_JPRB, 0.43596E+00_JPRB, 0.44055E+00_JPRB, 0.44212E+00_JPRB, 0.44125E+00_JPRB ,&
&  0.44551E+00_JPRB, 0.43675E+00_JPRB, 0.43129E+00_JPRB, 0.42467E+00_JPRB, 0.41879E+00_JPRB, 0.40771E+00_JPRB, 0.39963E+00_JPRB ,&
&  0.39007E+00_JPRB, 0.38182E+00_JPRB, 0.37840E+00_JPRB, 0.37468E+00_JPRB, 0.36518E+00_JPRB, 0.35391E+00_JPRB, 0.34535E+00_JPRB ,&
&  0.33960E+00_JPRB, 0.33385E+00_JPRB, 0.32710E+00_JPRB, 0.32170E+00_JPRB, 0.31733E+00_JPRB, 0.31590E+00_JPRB, 0.31781E+00_JPRB ,&
&  0.32183E+00_JPRB, 0.32522E+00_JPRB, 0.32693E+00_JPRB, 0.32709E+00_JPRB, 0.32543E+00_JPRB, 0.32277E+00_JPRB, 0.32107E+00_JPRB ,&
&  0.32137E+00_JPRB, 0.32250E+00_JPRB, 0.32471E+00_JPRB, 0.32991E+00_JPRB, 0.33049E+00_JPRB, 0.32898E+00_JPRB, 0.32343E+00_JPRB ,&
&  0.32065E+00_JPRB, 0.31910E+00_JPRB, 0.32052E+00_JPRB, 0.32544E+00_JPRB, 0.33360E+00_JPRB, 0.34149E+00_JPRB, 0.34719E+00_JPRB ,&
&  0.35014E+00_JPRB, 0.35211E+00_JPRB, 0.35528E+00_JPRB, 0.35881E+00_JPRB, 0.36002E+00_JPRB, 0.35906E+00_JPRB, 0.35727E+00_JPRB ,&
&  0.35976E+00_JPRB, 0.36684E+00_JPRB, 0.37435E+00_JPRB, 0.37890E+00_JPRB, 0.38090E+00_JPRB, 0.37956E+00_JPRB, 0.37257E+00_JPRB ,&
&  0.36007E+00_JPRB, 0.34579E+00_JPRB, 0.33192E+00_JPRB, 0.32009E+00_JPRB, 0.31191E+00_JPRB, 0.30810E+00_JPRB, 0.31109E+00_JPRB ,&
&  0.31899E+00_JPRB, 0.32911E+00_JPRB, 0.33858E+00_JPRB, 0.34572E+00_JPRB, 0.35345E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.38280E+00_JPRB, 0.38302E+00_JPRB, 0.38607E+00_JPRB, 0.39261E+00_JPRB, 0.39703E+00_JPRB, 0.40116E+00_JPRB, 0.40897E+00_JPRB ,&
&  0.41593E+00_JPRB, 0.42206E+00_JPRB, 0.42952E+00_JPRB, 0.43784E+00_JPRB, 0.44436E+00_JPRB, 0.45357E+00_JPRB, 0.46212E+00_JPRB ,&
&  0.46755E+00_JPRB, 0.46966E+00_JPRB, 0.47075E+00_JPRB, 0.47166E+00_JPRB, 0.47127E+00_JPRB, 0.46855E+00_JPRB, 0.46797E+00_JPRB ,&
&  0.46613E+00_JPRB, 0.46417E+00_JPRB, 0.46644E+00_JPRB, 0.46696E+00_JPRB, 0.46901E+00_JPRB, 0.47234E+00_JPRB, 0.47317E+00_JPRB ,&
&  0.47190E+00_JPRB, 0.45974E+00_JPRB, 0.45568E+00_JPRB, 0.45331E+00_JPRB, 0.45799E+00_JPRB, 0.45952E+00_JPRB, 0.45852E+00_JPRB ,&
&  0.45263E+00_JPRB, 0.44921E+00_JPRB, 0.45134E+00_JPRB, 0.45388E+00_JPRB, 0.44830E+00_JPRB, 0.43543E+00_JPRB, 0.42493E+00_JPRB ,&
&  0.41648E+00_JPRB, 0.40412E+00_JPRB, 0.38655E+00_JPRB, 0.37644E+00_JPRB, 0.37226E+00_JPRB, 0.36738E+00_JPRB, 0.36267E+00_JPRB ,&
&  0.36011E+00_JPRB, 0.35928E+00_JPRB, 0.36077E+00_JPRB, 0.35985E+00_JPRB, 0.35607E+00_JPRB, 0.35143E+00_JPRB, 0.34948E+00_JPRB ,&
&  0.35186E+00_JPRB, 0.35394E+00_JPRB, 0.35729E+00_JPRB, 0.35979E+00_JPRB, 0.35917E+00_JPRB, 0.35957E+00_JPRB, 0.36307E+00_JPRB ,&
&  0.36824E+00_JPRB, 0.37301E+00_JPRB, 0.37834E+00_JPRB, 0.38053E+00_JPRB, 0.38324E+00_JPRB, 0.38950E+00_JPRB, 0.38978E+00_JPRB ,&
&  0.38841E+00_JPRB, 0.39060E+00_JPRB, 0.39473E+00_JPRB, 0.40228E+00_JPRB, 0.41380E+00_JPRB, 0.42134E+00_JPRB, 0.42235E+00_JPRB ,&
&  0.42230E+00_JPRB, 0.42444E+00_JPRB, 0.42795E+00_JPRB, 0.43663E+00_JPRB, 0.44610E+00_JPRB, 0.45301E+00_JPRB, 0.45595E+00_JPRB ,&
&  0.45299E+00_JPRB, 0.44646E+00_JPRB, 0.43448E+00_JPRB, 0.42038E+00_JPRB, 0.40774E+00_JPRB, 0.39785E+00_JPRB, 0.38976E+00_JPRB ,&
&  0.38188E+00_JPRB, 0.37870E+00_JPRB, 0.37861E+00_JPRB, 0.38069E+00_JPRB, 0.38201E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.41478E+00_JPRB, 0.41564E+00_JPRB, 0.42056E+00_JPRB, 0.42555E+00_JPRB, 0.42918E+00_JPRB, 0.43428E+00_JPRB, 0.44385E+00_JPRB ,&
&  0.45356E+00_JPRB, 0.46295E+00_JPRB, 0.46939E+00_JPRB, 0.47722E+00_JPRB, 0.48805E+00_JPRB, 0.49530E+00_JPRB, 0.50109E+00_JPRB ,&
&  0.50348E+00_JPRB, 0.50368E+00_JPRB, 0.50439E+00_JPRB, 0.50727E+00_JPRB, 0.50673E+00_JPRB, 0.50307E+00_JPRB, 0.50178E+00_JPRB ,&
&  0.50174E+00_JPRB, 0.50022E+00_JPRB, 0.50026E+00_JPRB, 0.50183E+00_JPRB, 0.50020E+00_JPRB, 0.49719E+00_JPRB, 0.49754E+00_JPRB ,&
&  0.49874E+00_JPRB, 0.48584E+00_JPRB, 0.47902E+00_JPRB, 0.47263E+00_JPRB, 0.47683E+00_JPRB, 0.48877E+00_JPRB, 0.50170E+00_JPRB ,&
&  0.50954E+00_JPRB, 0.51486E+00_JPRB, 0.51455E+00_JPRB, 0.51369E+00_JPRB, 0.50921E+00_JPRB, 0.49782E+00_JPRB, 0.48785E+00_JPRB ,&
&  0.48104E+00_JPRB, 0.47158E+00_JPRB, 0.44943E+00_JPRB, 0.43694E+00_JPRB, 0.43389E+00_JPRB, 0.43646E+00_JPRB, 0.44013E+00_JPRB ,&
&  0.44180E+00_JPRB, 0.43961E+00_JPRB, 0.43397E+00_JPRB, 0.42691E+00_JPRB, 0.41707E+00_JPRB, 0.40556E+00_JPRB, 0.39545E+00_JPRB ,&
&  0.38956E+00_JPRB, 0.38510E+00_JPRB, 0.38139E+00_JPRB, 0.38369E+00_JPRB, 0.38298E+00_JPRB, 0.37918E+00_JPRB, 0.38083E+00_JPRB ,&
&  0.39145E+00_JPRB, 0.40474E+00_JPRB, 0.41706E+00_JPRB, 0.42385E+00_JPRB, 0.42654E+00_JPRB, 0.42965E+00_JPRB, 0.43351E+00_JPRB ,&
&  0.43935E+00_JPRB, 0.44534E+00_JPRB, 0.45022E+00_JPRB, 0.45613E+00_JPRB, 0.46358E+00_JPRB, 0.47045E+00_JPRB, 0.47331E+00_JPRB ,&
&  0.47582E+00_JPRB, 0.47714E+00_JPRB, 0.47774E+00_JPRB, 0.48216E+00_JPRB, 0.48944E+00_JPRB, 0.49468E+00_JPRB, 0.49906E+00_JPRB ,&
&  0.50376E+00_JPRB, 0.50599E+00_JPRB, 0.50473E+00_JPRB, 0.50075E+00_JPRB, 0.49521E+00_JPRB, 0.48405E+00_JPRB, 0.47037E+00_JPRB ,&
&  0.45472E+00_JPRB, 0.44075E+00_JPRB, 0.43065E+00_JPRB, 0.42200E+00_JPRB, 0.41612E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.45757E+00_JPRB, 0.45577E+00_JPRB, 0.45646E+00_JPRB, 0.45984E+00_JPRB, 0.46360E+00_JPRB, 0.46664E+00_JPRB, 0.47271E+00_JPRB ,&
&  0.48282E+00_JPRB, 0.49074E+00_JPRB, 0.50024E+00_JPRB, 0.50979E+00_JPRB, 0.51700E+00_JPRB, 0.52183E+00_JPRB, 0.52445E+00_JPRB ,&
&  0.52272E+00_JPRB, 0.51994E+00_JPRB, 0.52251E+00_JPRB, 0.52693E+00_JPRB, 0.52827E+00_JPRB, 0.52783E+00_JPRB, 0.52789E+00_JPRB ,&
&  0.52761E+00_JPRB, 0.52577E+00_JPRB, 0.52355E+00_JPRB, 0.52016E+00_JPRB, 0.51602E+00_JPRB, 0.51461E+00_JPRB, 0.51321E+00_JPRB ,&
&  0.50752E+00_JPRB, 0.50736E+00_JPRB, 0.45794E+00_JPRB, 0.47357E+00_JPRB, 0.48689E+00_JPRB, 0.50682E+00_JPRB, 0.52596E+00_JPRB ,&
&  0.53980E+00_JPRB, 0.54895E+00_JPRB, 0.55372E+00_JPRB, 0.55260E+00_JPRB, 0.54074E+00_JPRB, 0.53281E+00_JPRB, 0.52370E+00_JPRB ,&
&  0.51243E+00_JPRB, 0.50437E+00_JPRB, 0.49154E+00_JPRB, 0.47794E+00_JPRB, 0.47342E+00_JPRB, 0.47390E+00_JPRB, 0.47395E+00_JPRB ,&
&  0.47079E+00_JPRB, 0.46798E+00_JPRB, 0.46987E+00_JPRB, 0.47123E+00_JPRB, 0.47129E+00_JPRB, 0.46615E+00_JPRB, 0.45944E+00_JPRB ,&
&  0.44892E+00_JPRB, 0.43849E+00_JPRB, 0.43127E+00_JPRB, 0.42570E+00_JPRB, 0.42074E+00_JPRB, 0.41534E+00_JPRB, 0.41000E+00_JPRB ,&
&  0.41291E+00_JPRB, 0.42335E+00_JPRB, 0.43313E+00_JPRB, 0.44138E+00_JPRB, 0.44687E+00_JPRB, 0.45065E+00_JPRB, 0.45588E+00_JPRB ,&
&  0.46191E+00_JPRB, 0.47194E+00_JPRB, 0.47860E+00_JPRB, 0.48446E+00_JPRB, 0.48892E+00_JPRB, 0.49316E+00_JPRB, 0.49814E+00_JPRB ,&
&  0.50208E+00_JPRB, 0.50826E+00_JPRB, 0.51245E+00_JPRB, 0.51770E+00_JPRB, 0.52047E+00_JPRB, 0.52489E+00_JPRB, 0.53055E+00_JPRB ,&
&  0.53703E+00_JPRB, 0.54283E+00_JPRB, 0.54508E+00_JPRB, 0.54326E+00_JPRB, 0.53562E+00_JPRB, 0.52529E+00_JPRB, 0.51542E+00_JPRB ,&
&  0.50306E+00_JPRB, 0.48911E+00_JPRB, 0.47541E+00_JPRB, 0.46689E+00_JPRB, 0.45991E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.51467E+00_JPRB, 0.51009E+00_JPRB, 0.50509E+00_JPRB, 0.50467E+00_JPRB, 0.50416E+00_JPRB, 0.50680E+00_JPRB, 0.51034E+00_JPRB ,&
&  0.51756E+00_JPRB, 0.52217E+00_JPRB, 0.53129E+00_JPRB, 0.53853E+00_JPRB, 0.54191E+00_JPRB, 0.54592E+00_JPRB, 0.54850E+00_JPRB ,&
&  0.54954E+00_JPRB, 0.55098E+00_JPRB, 0.55175E+00_JPRB, 0.55606E+00_JPRB, 0.55690E+00_JPRB, 0.55628E+00_JPRB, 0.55555E+00_JPRB ,&
&  0.55420E+00_JPRB, 0.55293E+00_JPRB, 0.54893E+00_JPRB, 0.54304E+00_JPRB, 0.53541E+00_JPRB, 0.53266E+00_JPRB, 0.52742E+00_JPRB ,&
&  0.52943E+00_JPRB, 0.49769E+00_JPRB, 0.50929E+00_JPRB, 0.50788E+00_JPRB, 0.52085E+00_JPRB, 0.54138E+00_JPRB, 0.57122E+00_JPRB ,&
&  0.59255E+00_JPRB, 0.60462E+00_JPRB, 0.60750E+00_JPRB, 0.60676E+00_JPRB, 0.59390E+00_JPRB, 0.58005E+00_JPRB, 0.56431E+00_JPRB ,&
&  0.54608E+00_JPRB, 0.53593E+00_JPRB, 0.52281E+00_JPRB, 0.51323E+00_JPRB, 0.50940E+00_JPRB, 0.50687E+00_JPRB, 0.50266E+00_JPRB ,&
&  0.49624E+00_JPRB, 0.49086E+00_JPRB, 0.49203E+00_JPRB, 0.49595E+00_JPRB, 0.49989E+00_JPRB, 0.50237E+00_JPRB, 0.50147E+00_JPRB ,&
&  0.49527E+00_JPRB, 0.48951E+00_JPRB, 0.48623E+00_JPRB, 0.48521E+00_JPRB, 0.48029E+00_JPRB, 0.47221E+00_JPRB, 0.46387E+00_JPRB ,&
&  0.46174E+00_JPRB, 0.46437E+00_JPRB, 0.46809E+00_JPRB, 0.47289E+00_JPRB, 0.47657E+00_JPRB, 0.48097E+00_JPRB, 0.48305E+00_JPRB ,&
&  0.48780E+00_JPRB, 0.49241E+00_JPRB, 0.50162E+00_JPRB, 0.51046E+00_JPRB, 0.51281E+00_JPRB, 0.52140E+00_JPRB, 0.53013E+00_JPRB ,&
&  0.53844E+00_JPRB, 0.54779E+00_JPRB, 0.55454E+00_JPRB, 0.55854E+00_JPRB, 0.56282E+00_JPRB, 0.56519E+00_JPRB, 0.57013E+00_JPRB ,&
&  0.57711E+00_JPRB, 0.57832E+00_JPRB, 0.58301E+00_JPRB, 0.58240E+00_JPRB, 0.57612E+00_JPRB, 0.56658E+00_JPRB, 0.55528E+00_JPRB ,&
&  0.54344E+00_JPRB, 0.52859E+00_JPRB, 0.52142E+00_JPRB, 0.51754E+00_JPRB, 0.51587E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.55514E+00_JPRB, 0.54891E+00_JPRB, 0.53885E+00_JPRB, 0.53342E+00_JPRB, 0.53192E+00_JPRB, 0.53611E+00_JPRB, 0.54141E+00_JPRB ,&
&  0.54665E+00_JPRB, 0.54872E+00_JPRB, 0.55602E+00_JPRB, 0.56318E+00_JPRB, 0.56728E+00_JPRB, 0.56997E+00_JPRB, 0.56904E+00_JPRB ,&
&  0.56905E+00_JPRB, 0.56914E+00_JPRB, 0.56503E+00_JPRB, 0.56200E+00_JPRB, 0.55743E+00_JPRB, 0.55853E+00_JPRB, 0.55916E+00_JPRB ,&
&  0.55886E+00_JPRB, 0.55691E+00_JPRB, 0.55195E+00_JPRB, 0.54461E+00_JPRB, 0.54184E+00_JPRB, 0.54227E+00_JPRB, 0.54072E+00_JPRB ,&
&  0.54815E+00_JPRB, 0.46856E+00_JPRB, 0.50366E+00_JPRB, 0.51505E+00_JPRB, 0.53096E+00_JPRB, 0.55539E+00_JPRB, 0.58517E+00_JPRB ,&
&  0.61579E+00_JPRB, 0.64066E+00_JPRB, 0.64865E+00_JPRB, 0.64827E+00_JPRB, 0.63722E+00_JPRB, 0.62026E+00_JPRB, 0.60077E+00_JPRB ,&
&  0.58402E+00_JPRB, 0.57086E+00_JPRB, 0.55434E+00_JPRB, 0.54496E+00_JPRB, 0.53750E+00_JPRB, 0.52784E+00_JPRB, 0.52004E+00_JPRB ,&
&  0.51752E+00_JPRB, 0.51638E+00_JPRB, 0.51699E+00_JPRB, 0.51931E+00_JPRB, 0.52263E+00_JPRB, 0.52497E+00_JPRB, 0.52498E+00_JPRB ,&
&  0.52369E+00_JPRB, 0.52082E+00_JPRB, 0.51599E+00_JPRB, 0.51363E+00_JPRB, 0.51263E+00_JPRB, 0.51384E+00_JPRB, 0.50994E+00_JPRB ,&
&  0.50358E+00_JPRB, 0.50514E+00_JPRB, 0.50850E+00_JPRB, 0.51079E+00_JPRB, 0.51407E+00_JPRB, 0.51761E+00_JPRB, 0.51829E+00_JPRB ,&
&  0.51886E+00_JPRB, 0.52299E+00_JPRB, 0.52743E+00_JPRB, 0.53433E+00_JPRB, 0.53918E+00_JPRB, 0.54814E+00_JPRB, 0.55541E+00_JPRB ,&
&  0.56214E+00_JPRB, 0.57243E+00_JPRB, 0.58169E+00_JPRB, 0.58727E+00_JPRB, 0.59047E+00_JPRB, 0.59429E+00_JPRB, 0.59856E+00_JPRB ,&
&  0.60151E+00_JPRB, 0.60372E+00_JPRB, 0.60566E+00_JPRB, 0.60386E+00_JPRB, 0.59747E+00_JPRB, 0.59100E+00_JPRB, 0.58648E+00_JPRB ,&
&  0.57684E+00_JPRB, 0.56537E+00_JPRB, 0.56143E+00_JPRB, 0.55889E+00_JPRB, 0.55624E+00_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.61621E+00_JPRB, 0.60921E+00_JPRB, 0.59704E+00_JPRB, 0.58706E+00_JPRB, 0.57947E+00_JPRB, 0.57781E+00_JPRB, 0.57722E+00_JPRB ,&
&  0.58247E+00_JPRB, 0.59040E+00_JPRB, 0.59745E+00_JPRB, 0.60088E+00_JPRB, 0.60306E+00_JPRB, 0.60283E+00_JPRB, 0.59936E+00_JPRB ,&
&  0.59670E+00_JPRB, 0.58593E+00_JPRB, 0.57824E+00_JPRB, 0.57892E+00_JPRB, 0.57648E+00_JPRB, 0.57386E+00_JPRB, 0.57098E+00_JPRB ,&
&  0.56854E+00_JPRB, 0.56723E+00_JPRB, 0.56033E+00_JPRB, 0.55784E+00_JPRB, 0.55806E+00_JPRB, 0.56105E+00_JPRB, 0.57219E+00_JPRB ,&
&  0.50031E+00_JPRB, 0.42144E+00_JPRB, 0.47040E+00_JPRB, 0.51829E+00_JPRB, 0.54481E+00_JPRB, 0.58201E+00_JPRB, 0.61715E+00_JPRB ,&
&  0.66246E+00_JPRB, 0.69019E+00_JPRB, 0.70360E+00_JPRB, 0.69756E+00_JPRB, 0.68126E+00_JPRB, 0.66476E+00_JPRB, 0.64806E+00_JPRB ,&
&  0.62781E+00_JPRB, 0.60422E+00_JPRB, 0.58980E+00_JPRB, 0.58374E+00_JPRB, 0.57859E+00_JPRB, 0.57237E+00_JPRB, 0.56951E+00_JPRB ,&
&  0.56960E+00_JPRB, 0.56983E+00_JPRB, 0.56891E+00_JPRB, 0.56845E+00_JPRB, 0.57057E+00_JPRB, 0.57289E+00_JPRB, 0.57523E+00_JPRB ,&
&  0.57450E+00_JPRB, 0.57246E+00_JPRB, 0.56880E+00_JPRB, 0.56233E+00_JPRB, 0.55755E+00_JPRB, 0.55774E+00_JPRB, 0.55589E+00_JPRB ,&
&  0.55486E+00_JPRB, 0.55371E+00_JPRB, 0.55408E+00_JPRB, 0.55525E+00_JPRB, 0.55537E+00_JPRB, 0.55587E+00_JPRB, 0.55886E+00_JPRB ,&
&  0.56482E+00_JPRB, 0.56550E+00_JPRB, 0.56874E+00_JPRB, 0.57421E+00_JPRB, 0.57963E+00_JPRB, 0.58452E+00_JPRB, 0.58719E+00_JPRB ,&
&  0.59208E+00_JPRB, 0.60301E+00_JPRB, 0.61057E+00_JPRB, 0.61513E+00_JPRB, 0.61913E+00_JPRB, 0.62391E+00_JPRB, 0.63083E+00_JPRB ,&
&  0.63599E+00_JPRB, 0.64021E+00_JPRB, 0.64569E+00_JPRB, 0.65319E+00_JPRB, 0.65526E+00_JPRB, 0.65185E+00_JPRB, 0.64500E+00_JPRB ,&
&  0.63371E+00_JPRB, 0.61801E+00_JPRB, 0.61959E+00_JPRB, 0.61818E+00_JPRB, 0.61667E+00_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.63711E+00_JPRB, 0.63863E+00_JPRB, 0.63210E+00_JPRB, 0.62253E+00_JPRB, 0.61564E+00_JPRB, 0.61389E+00_JPRB, 0.61417E+00_JPRB ,&
&  0.61899E+00_JPRB, 0.62302E+00_JPRB, 0.62390E+00_JPRB, 0.62063E+00_JPRB, 0.61552E+00_JPRB, 0.60768E+00_JPRB, 0.60163E+00_JPRB ,&
&  0.58966E+00_JPRB, 0.57665E+00_JPRB, 0.57298E+00_JPRB, 0.57358E+00_JPRB, 0.57176E+00_JPRB, 0.56620E+00_JPRB, 0.56426E+00_JPRB ,&
&  0.56508E+00_JPRB, 0.56941E+00_JPRB, 0.56538E+00_JPRB, 0.56354E+00_JPRB, 0.56040E+00_JPRB, 0.56352E+00_JPRB, 0.56610E+00_JPRB ,&
&  0.56658E+00_JPRB, 0.44134E+00_JPRB, 0.48972E+00_JPRB, 0.53509E+00_JPRB, 0.55737E+00_JPRB, 0.60256E+00_JPRB, 0.65328E+00_JPRB ,&
&  0.69029E+00_JPRB, 0.70834E+00_JPRB, 0.70360E+00_JPRB, 0.69131E+00_JPRB, 0.67122E+00_JPRB, 0.65008E+00_JPRB, 0.63053E+00_JPRB ,&
&  0.61170E+00_JPRB, 0.59970E+00_JPRB, 0.59184E+00_JPRB, 0.58584E+00_JPRB, 0.58411E+00_JPRB, 0.58221E+00_JPRB, 0.58025E+00_JPRB ,&
&  0.58407E+00_JPRB, 0.59308E+00_JPRB, 0.59527E+00_JPRB, 0.59860E+00_JPRB, 0.60219E+00_JPRB, 0.60401E+00_JPRB, 0.61106E+00_JPRB ,&
&  0.61419E+00_JPRB, 0.61223E+00_JPRB, 0.61034E+00_JPRB, 0.60606E+00_JPRB, 0.59682E+00_JPRB, 0.59348E+00_JPRB, 0.58920E+00_JPRB ,&
&  0.58543E+00_JPRB, 0.57859E+00_JPRB, 0.57315E+00_JPRB, 0.57192E+00_JPRB, 0.57330E+00_JPRB, 0.57438E+00_JPRB, 0.58118E+00_JPRB ,&
&  0.59193E+00_JPRB, 0.59528E+00_JPRB, 0.59825E+00_JPRB, 0.60332E+00_JPRB, 0.60712E+00_JPRB, 0.60896E+00_JPRB, 0.61166E+00_JPRB ,&
&  0.61927E+00_JPRB, 0.63260E+00_JPRB, 0.64489E+00_JPRB, 0.64971E+00_JPRB, 0.65144E+00_JPRB, 0.65828E+00_JPRB, 0.66549E+00_JPRB ,&
&  0.67303E+00_JPRB, 0.67486E+00_JPRB, 0.67489E+00_JPRB, 0.69489E+00_JPRB, 0.70245E+00_JPRB, 0.69694E+00_JPRB, 0.68745E+00_JPRB ,&
&  0.66913E+00_JPRB, 0.66274E+00_JPRB, 0.53646E+00_JPRB, 0.59548E+00_JPRB, 0.61630E+00_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.67567E+00_JPRB, 0.68093E+00_JPRB, 0.67458E+00_JPRB, 0.66363E+00_JPRB, 0.65982E+00_JPRB, 0.65891E+00_JPRB, 0.65899E+00_JPRB ,&
&  0.65375E+00_JPRB, 0.64518E+00_JPRB, 0.63515E+00_JPRB, 0.62343E+00_JPRB, 0.61124E+00_JPRB, 0.60143E+00_JPRB, 0.59163E+00_JPRB ,&
&  0.57457E+00_JPRB, 0.56551E+00_JPRB, 0.56631E+00_JPRB, 0.56400E+00_JPRB, 0.56168E+00_JPRB, 0.55847E+00_JPRB, 0.56047E+00_JPRB ,&
&  0.56503E+00_JPRB, 0.56723E+00_JPRB, 0.56846E+00_JPRB, 0.56352E+00_JPRB, 0.56016E+00_JPRB, 0.56149E+00_JPRB, 0.55904E+00_JPRB ,&
&  0.55779E+00_JPRB, 0.45090E+00_JPRB, 0.49372E+00_JPRB, 0.56400E+00_JPRB, 0.59370E+00_JPRB, 0.64076E+00_JPRB, 0.71025E+00_JPRB ,&
&  0.75187E+00_JPRB, 0.74416E+00_JPRB, 0.72171E+00_JPRB, 0.70064E+00_JPRB, 0.67493E+00_JPRB, 0.64981E+00_JPRB, 0.62803E+00_JPRB ,&
&  0.61526E+00_JPRB, 0.60981E+00_JPRB, 0.60726E+00_JPRB, 0.60351E+00_JPRB, 0.59911E+00_JPRB, 0.59959E+00_JPRB, 0.60081E+00_JPRB ,&
&  0.60391E+00_JPRB, 0.61025E+00_JPRB, 0.61407E+00_JPRB, 0.62197E+00_JPRB, 0.62865E+00_JPRB, 0.63990E+00_JPRB, 0.65508E+00_JPRB ,&
&  0.65765E+00_JPRB, 0.66506E+00_JPRB, 0.67818E+00_JPRB, 0.67809E+00_JPRB, 0.66496E+00_JPRB, 0.65757E+00_JPRB, 0.65030E+00_JPRB ,&
&  0.64068E+00_JPRB, 0.62353E+00_JPRB, 0.62106E+00_JPRB, 0.61855E+00_JPRB, 0.61540E+00_JPRB, 0.61481E+00_JPRB, 0.62437E+00_JPRB ,&
&  0.63374E+00_JPRB, 0.63703E+00_JPRB, 0.64270E+00_JPRB, 0.65147E+00_JPRB, 0.65565E+00_JPRB, 0.65171E+00_JPRB, 0.64994E+00_JPRB ,&
&  0.65558E+00_JPRB, 0.66573E+00_JPRB, 0.67046E+00_JPRB, 0.67116E+00_JPRB, 0.67854E+00_JPRB, 0.69074E+00_JPRB, 0.69826E+00_JPRB ,&
&  0.70312E+00_JPRB, 0.70259E+00_JPRB, 0.70828E+00_JPRB, 0.68521E+00_JPRB, 0.72769E+00_JPRB, 0.72254E+00_JPRB, 0.71177E+00_JPRB ,&
&  0.70208E+00_JPRB, 0.69020E+00_JPRB, 0.69005E+00_JPRB, 0.59746E+00_JPRB, 0.64209E+00_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.71671E+00_JPRB, 0.72551E+00_JPRB, 0.71235E+00_JPRB, 0.69769E+00_JPRB, 0.68989E+00_JPRB, 0.68099E+00_JPRB, 0.66196E+00_JPRB ,&
&  0.64039E+00_JPRB, 0.62106E+00_JPRB, 0.61051E+00_JPRB, 0.59635E+00_JPRB, 0.58307E+00_JPRB, 0.57577E+00_JPRB, 0.56694E+00_JPRB ,&
&  0.55231E+00_JPRB, 0.54954E+00_JPRB, 0.55060E+00_JPRB, 0.54944E+00_JPRB, 0.54599E+00_JPRB, 0.54483E+00_JPRB, 0.55321E+00_JPRB ,&
&  0.55498E+00_JPRB, 0.55296E+00_JPRB, 0.55296E+00_JPRB, 0.55030E+00_JPRB, 0.54681E+00_JPRB, 0.54772E+00_JPRB, 0.53825E+00_JPRB ,&
&  0.53662E+00_JPRB, 0.48315E+00_JPRB, 0.54546E+00_JPRB, 0.62043E+00_JPRB, 0.67829E+00_JPRB, 0.71896E+00_JPRB, 0.76134E+00_JPRB ,&
&  0.77885E+00_JPRB, 0.75464E+00_JPRB, 0.72250E+00_JPRB, 0.69835E+00_JPRB, 0.66860E+00_JPRB, 0.64462E+00_JPRB, 0.62634E+00_JPRB ,&
&  0.61535E+00_JPRB, 0.61207E+00_JPRB, 0.61008E+00_JPRB, 0.60851E+00_JPRB, 0.60628E+00_JPRB, 0.60485E+00_JPRB, 0.60646E+00_JPRB ,&
&  0.60633E+00_JPRB, 0.60895E+00_JPRB, 0.61196E+00_JPRB, 0.61062E+00_JPRB, 0.61647E+00_JPRB, 0.63958E+00_JPRB, 0.66584E+00_JPRB ,&
&  0.67230E+00_JPRB, 0.68679E+00_JPRB, 0.70474E+00_JPRB, 0.70912E+00_JPRB, 0.69910E+00_JPRB, 0.69460E+00_JPRB, 0.68775E+00_JPRB ,&
&  0.68276E+00_JPRB, 0.67175E+00_JPRB, 0.67111E+00_JPRB, 0.66661E+00_JPRB, 0.65907E+00_JPRB, 0.65556E+00_JPRB, 0.65589E+00_JPRB ,&
&  0.65734E+00_JPRB, 0.65904E+00_JPRB, 0.66591E+00_JPRB, 0.67374E+00_JPRB, 0.67094E+00_JPRB, 0.66366E+00_JPRB, 0.66316E+00_JPRB ,&
&  0.66639E+00_JPRB, 0.66251E+00_JPRB, 0.65374E+00_JPRB, 0.66308E+00_JPRB, 0.68177E+00_JPRB, 0.69215E+00_JPRB, 0.69734E+00_JPRB ,&
&  0.70220E+00_JPRB, 0.70781E+00_JPRB, 0.71835E+00_JPRB, 0.74760E+00_JPRB, 0.75045E+00_JPRB, 0.73569E+00_JPRB, 0.72730E+00_JPRB ,&
&  0.72358E+00_JPRB, 0.71475E+00_JPRB, 0.70605E+00_JPRB, 0.70397E+00_JPRB, 0.66163E+00_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.85522E+00_JPRB, 0.79889E+00_JPRB, 0.75267E+00_JPRB, 0.72007E+00_JPRB, 0.69912E+00_JPRB, 0.67152E+00_JPRB, 0.63761E+00_JPRB ,&
&  0.61345E+00_JPRB, 0.60564E+00_JPRB, 0.59719E+00_JPRB, 0.58408E+00_JPRB, 0.57551E+00_JPRB, 0.56472E+00_JPRB, 0.55523E+00_JPRB ,&
&  0.54190E+00_JPRB, 0.53823E+00_JPRB, 0.53444E+00_JPRB, 0.53916E+00_JPRB, 0.54107E+00_JPRB, 0.54726E+00_JPRB, 0.55954E+00_JPRB ,&
&  0.56161E+00_JPRB, 0.54130E+00_JPRB, 0.53843E+00_JPRB, 0.54192E+00_JPRB, 0.54470E+00_JPRB, 0.54730E+00_JPRB, 0.53624E+00_JPRB ,&
&  0.53981E+00_JPRB, 0.52133E+00_JPRB, 0.64261E+00_JPRB, 0.75443E+00_JPRB, 0.81304E+00_JPRB, 0.85996E+00_JPRB, 0.85874E+00_JPRB ,&
&  0.84751E+00_JPRB, 0.81020E+00_JPRB, 0.76322E+00_JPRB, 0.71225E+00_JPRB, 0.66577E+00_JPRB, 0.63766E+00_JPRB, 0.62550E+00_JPRB ,&
&  0.61958E+00_JPRB, 0.62051E+00_JPRB, 0.61896E+00_JPRB, 0.61934E+00_JPRB, 0.61980E+00_JPRB, 0.61844E+00_JPRB, 0.62313E+00_JPRB ,&
&  0.63130E+00_JPRB, 0.62194E+00_JPRB, 0.61727E+00_JPRB, 0.62813E+00_JPRB, 0.64356E+00_JPRB, 0.66127E+00_JPRB, 0.70162E+00_JPRB ,&
&  0.72301E+00_JPRB, 0.74566E+00_JPRB, 0.76700E+00_JPRB, 0.77294E+00_JPRB, 0.76780E+00_JPRB, 0.76482E+00_JPRB, 0.75291E+00_JPRB ,&
&  0.74843E+00_JPRB, 0.74091E+00_JPRB, 0.73059E+00_JPRB, 0.71875E+00_JPRB, 0.70621E+00_JPRB, 0.69929E+00_JPRB, 0.69503E+00_JPRB ,&
&  0.69062E+00_JPRB, 0.69891E+00_JPRB, 0.70447E+00_JPRB, 0.70415E+00_JPRB, 0.69303E+00_JPRB, 0.68542E+00_JPRB, 0.68125E+00_JPRB ,&
&  0.66983E+00_JPRB, 0.65265E+00_JPRB, 0.64693E+00_JPRB, 0.65782E+00_JPRB, 0.68220E+00_JPRB, 0.69165E+00_JPRB, 0.69433E+00_JPRB ,&
&  0.70736E+00_JPRB, 0.71881E+00_JPRB, 0.70714E+00_JPRB, 0.72594E+00_JPRB, 0.78939E+00_JPRB, 0.78254E+00_JPRB, 0.77161E+00_JPRB ,&
&  0.75541E+00_JPRB, 0.74943E+00_JPRB, 0.73480E+00_JPRB, 0.72570E+00_JPRB, 0.83587E+00_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.75761E+00_JPRB, 0.72995E+00_JPRB, 0.70571E+00_JPRB, 0.67636E+00_JPRB, 0.65708E+00_JPRB, 0.63610E+00_JPRB, 0.61566E+00_JPRB ,&
&  0.59771E+00_JPRB, 0.58752E+00_JPRB, 0.57837E+00_JPRB, 0.56966E+00_JPRB, 0.57152E+00_JPRB, 0.56710E+00_JPRB, 0.55606E+00_JPRB ,&
&  0.53966E+00_JPRB, 0.52757E+00_JPRB, 0.52467E+00_JPRB, 0.52858E+00_JPRB, 0.54213E+00_JPRB, 0.55284E+00_JPRB, 0.56189E+00_JPRB ,&
&  0.56532E+00_JPRB, 0.54416E+00_JPRB, 0.53571E+00_JPRB, 0.53950E+00_JPRB, 0.54263E+00_JPRB, 0.54754E+00_JPRB, 0.53102E+00_JPRB ,&
&  0.52795E+00_JPRB, 0.54906E+00_JPRB, 0.73965E+00_JPRB, 0.10025E+01_JPRB, 0.10447E+01_JPRB, 0.10135E+01_JPRB, 0.97168E+00_JPRB ,&
&  0.93365E+00_JPRB, 0.88708E+00_JPRB, 0.82073E+00_JPRB, 0.75008E+00_JPRB, 0.70029E+00_JPRB, 0.67126E+00_JPRB, 0.65799E+00_JPRB ,&
&  0.65890E+00_JPRB, 0.66226E+00_JPRB, 0.65775E+00_JPRB, 0.65635E+00_JPRB, 0.65303E+00_JPRB, 0.65085E+00_JPRB, 0.65158E+00_JPRB ,&
&  0.64900E+00_JPRB, 0.64342E+00_JPRB, 0.63723E+00_JPRB, 0.63662E+00_JPRB, 0.66845E+00_JPRB, 0.72192E+00_JPRB, 0.79776E+00_JPRB ,&
&  0.84119E+00_JPRB, 0.87052E+00_JPRB, 0.88651E+00_JPRB, 0.88430E+00_JPRB, 0.87111E+00_JPRB, 0.85924E+00_JPRB, 0.85108E+00_JPRB ,&
&  0.83432E+00_JPRB, 0.80309E+00_JPRB, 0.78125E+00_JPRB, 0.76442E+00_JPRB, 0.74751E+00_JPRB, 0.73415E+00_JPRB, 0.72244E+00_JPRB ,&
&  0.71957E+00_JPRB, 0.72604E+00_JPRB, 0.72680E+00_JPRB, 0.72143E+00_JPRB, 0.70474E+00_JPRB, 0.68714E+00_JPRB, 0.67197E+00_JPRB ,&
&  0.65020E+00_JPRB, 0.63776E+00_JPRB, 0.64514E+00_JPRB, 0.66695E+00_JPRB, 0.68981E+00_JPRB, 0.69572E+00_JPRB, 0.70316E+00_JPRB ,&
&  0.71811E+00_JPRB, 0.73168E+00_JPRB, 0.73105E+00_JPRB, 0.72688E+00_JPRB, 0.72180E+00_JPRB, 0.77823E+00_JPRB, 0.77300E+00_JPRB ,&
&  0.75395E+00_JPRB, 0.73650E+00_JPRB, 0.72788E+00_JPRB, 0.71996E+00_JPRB, 0.77588E+00_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.72264E+00_JPRB, 0.71369E+00_JPRB, 0.68448E+00_JPRB, 0.65633E+00_JPRB, 0.63780E+00_JPRB, 0.62474E+00_JPRB, 0.61264E+00_JPRB ,&
&  0.59485E+00_JPRB, 0.57222E+00_JPRB, 0.55918E+00_JPRB, 0.55715E+00_JPRB, 0.55932E+00_JPRB, 0.55864E+00_JPRB, 0.54398E+00_JPRB ,&
&  0.52585E+00_JPRB, 0.51396E+00_JPRB, 0.51839E+00_JPRB, 0.52733E+00_JPRB, 0.54300E+00_JPRB, 0.54319E+00_JPRB, 0.55374E+00_JPRB ,&
&  0.55895E+00_JPRB, 0.55603E+00_JPRB, 0.55351E+00_JPRB, 0.55226E+00_JPRB, 0.54835E+00_JPRB, 0.54522E+00_JPRB, 0.52548E+00_JPRB ,&
&  0.53507E+00_JPRB, 0.60059E+00_JPRB, 0.92906E+00_JPRB, 0.12352E+01_JPRB, 0.12747E+01_JPRB, 0.11636E+01_JPRB, 0.10885E+01_JPRB ,&
&  0.10291E+01_JPRB, 0.97227E+00_JPRB, 0.88722E+00_JPRB, 0.82851E+00_JPRB, 0.77970E+00_JPRB, 0.74774E+00_JPRB, 0.73004E+00_JPRB ,&
&  0.72299E+00_JPRB, 0.71266E+00_JPRB, 0.69312E+00_JPRB, 0.67505E+00_JPRB, 0.66591E+00_JPRB, 0.65898E+00_JPRB, 0.65896E+00_JPRB ,&
&  0.66269E+00_JPRB, 0.66130E+00_JPRB, 0.66052E+00_JPRB, 0.66072E+00_JPRB, 0.68000E+00_JPRB, 0.70107E+00_JPRB, 0.90182E+00_JPRB ,&
&  0.11895E+01_JPRB, 0.11778E+01_JPRB, 0.11434E+01_JPRB, 0.10830E+01_JPRB, 0.10200E+01_JPRB, 0.97476E+00_JPRB, 0.93418E+00_JPRB ,&
&  0.89692E+00_JPRB, 0.86116E+00_JPRB, 0.83225E+00_JPRB, 0.81427E+00_JPRB, 0.80015E+00_JPRB, 0.78757E+00_JPRB, 0.78416E+00_JPRB ,&
&  0.77054E+00_JPRB, 0.75788E+00_JPRB, 0.74407E+00_JPRB, 0.72742E+00_JPRB, 0.70198E+00_JPRB, 0.67757E+00_JPRB, 0.65881E+00_JPRB ,&
&  0.63937E+00_JPRB, 0.63851E+00_JPRB, 0.65445E+00_JPRB, 0.66762E+00_JPRB, 0.70553E+00_JPRB, 0.73640E+00_JPRB, 0.73539E+00_JPRB ,&
&  0.71681E+00_JPRB, 0.71912E+00_JPRB, 0.72090E+00_JPRB, 0.70422E+00_JPRB, 0.69601E+00_JPRB, 0.75772E+00_JPRB, 0.74742E+00_JPRB ,&
&  0.72843E+00_JPRB, 0.71995E+00_JPRB, 0.71470E+00_JPRB, 0.70601E+00_JPRB, 0.72435E+00_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.69455E+00_JPRB, 0.66903E+00_JPRB, 0.64662E+00_JPRB, 0.63152E+00_JPRB, 0.61782E+00_JPRB, 0.60713E+00_JPRB, 0.59407E+00_JPRB ,&
&  0.57311E+00_JPRB, 0.55449E+00_JPRB, 0.55142E+00_JPRB, 0.55007E+00_JPRB, 0.54653E+00_JPRB, 0.54303E+00_JPRB, 0.52594E+00_JPRB ,&
&  0.50972E+00_JPRB, 0.49627E+00_JPRB, 0.50841E+00_JPRB, 0.52223E+00_JPRB, 0.53703E+00_JPRB, 0.54865E+00_JPRB, 0.56357E+00_JPRB ,&
&  0.55945E+00_JPRB, 0.56025E+00_JPRB, 0.56263E+00_JPRB, 0.56317E+00_JPRB, 0.55358E+00_JPRB, 0.55443E+00_JPRB, 0.52654E+00_JPRB ,&
&  0.56687E+00_JPRB, 0.58889E+00_JPRB, 0.10858E+01_JPRB, 0.15343E+01_JPRB, 0.14490E+01_JPRB, 0.13048E+01_JPRB, 0.11606E+01_JPRB ,&
&  0.11314E+01_JPRB, 0.10423E+01_JPRB, 0.94409E+00_JPRB, 0.88015E+00_JPRB, 0.82553E+00_JPRB, 0.78409E+00_JPRB, 0.75841E+00_JPRB ,&
&  0.73805E+00_JPRB, 0.70820E+00_JPRB, 0.68667E+00_JPRB, 0.66905E+00_JPRB, 0.66593E+00_JPRB, 0.66531E+00_JPRB, 0.67106E+00_JPRB ,&
&  0.67677E+00_JPRB, 0.67573E+00_JPRB, 0.67534E+00_JPRB, 0.68934E+00_JPRB, 0.72071E+00_JPRB, 0.86141E+00_JPRB, 0.10300E+01_JPRB ,&
&  0.12139E+01_JPRB, 0.14271E+01_JPRB, 0.13315E+01_JPRB, 0.12056E+01_JPRB, 0.11098E+01_JPRB, 0.10397E+01_JPRB, 0.99385E+00_JPRB ,&
&  0.94787E+00_JPRB, 0.91487E+00_JPRB, 0.89465E+00_JPRB, 0.88380E+00_JPRB, 0.87535E+00_JPRB, 0.86403E+00_JPRB, 0.83861E+00_JPRB ,&
&  0.80330E+00_JPRB, 0.76891E+00_JPRB, 0.73815E+00_JPRB, 0.71039E+00_JPRB, 0.68228E+00_JPRB, 0.67307E+00_JPRB, 0.66898E+00_JPRB ,&
&  0.65920E+00_JPRB, 0.66771E+00_JPRB, 0.67599E+00_JPRB, 0.67571E+00_JPRB, 0.70251E+00_JPRB, 0.71297E+00_JPRB, 0.71387E+00_JPRB ,&
&  0.71099E+00_JPRB, 0.71020E+00_JPRB, 0.69830E+00_JPRB, 0.70921E+00_JPRB, 0.68719E+00_JPRB, 0.74971E+00_JPRB, 0.73379E+00_JPRB ,&
&  0.70942E+00_JPRB, 0.71225E+00_JPRB, 0.70699E+00_JPRB, 0.70714E+00_JPRB, 0.70779E+00_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.66658E+00_JPRB, 0.63473E+00_JPRB, 0.61410E+00_JPRB, 0.60705E+00_JPRB, 0.60138E+00_JPRB, 0.58926E+00_JPRB, 0.57927E+00_JPRB ,&
&  0.56344E+00_JPRB, 0.54724E+00_JPRB, 0.53988E+00_JPRB, 0.53244E+00_JPRB, 0.53652E+00_JPRB, 0.53182E+00_JPRB, 0.52060E+00_JPRB ,&
&  0.50666E+00_JPRB, 0.50462E+00_JPRB, 0.50866E+00_JPRB, 0.51955E+00_JPRB, 0.54069E+00_JPRB, 0.55080E+00_JPRB, 0.56400E+00_JPRB ,&
&  0.57287E+00_JPRB, 0.58135E+00_JPRB, 0.58095E+00_JPRB, 0.57549E+00_JPRB, 0.56863E+00_JPRB, 0.55846E+00_JPRB, 0.54249E+00_JPRB ,&
&  0.60192E+00_JPRB, 0.11768E+01_JPRB, 0.10654E+01_JPRB, 0.18354E+01_JPRB, 0.16351E+01_JPRB, 0.14655E+01_JPRB, 0.11372E+01_JPRB ,&
&  0.10243E+01_JPRB, 0.10589E+01_JPRB, 0.99856E+00_JPRB, 0.91222E+00_JPRB, 0.83049E+00_JPRB, 0.75899E+00_JPRB, 0.71422E+00_JPRB ,&
&  0.69660E+00_JPRB, 0.69442E+00_JPRB, 0.68898E+00_JPRB, 0.67749E+00_JPRB, 0.68026E+00_JPRB, 0.67811E+00_JPRB, 0.68387E+00_JPRB ,&
&  0.68918E+00_JPRB, 0.69200E+00_JPRB, 0.70032E+00_JPRB, 0.75829E+00_JPRB, 0.84564E+00_JPRB, 0.10788E+01_JPRB, 0.12220E+01_JPRB ,&
&  0.12542E+01_JPRB, 0.14757E+01_JPRB, 0.13371E+01_JPRB, 0.11839E+01_JPRB, 0.10960E+01_JPRB, 0.10243E+01_JPRB, 0.97252E+00_JPRB ,&
&  0.92802E+00_JPRB, 0.90475E+00_JPRB, 0.90206E+00_JPRB, 0.89752E+00_JPRB, 0.88677E+00_JPRB, 0.85367E+00_JPRB, 0.80402E+00_JPRB ,&
&  0.75795E+00_JPRB, 0.73203E+00_JPRB, 0.70584E+00_JPRB, 0.69145E+00_JPRB, 0.68520E+00_JPRB, 0.68773E+00_JPRB, 0.69700E+00_JPRB ,&
&  0.69818E+00_JPRB, 0.70875E+00_JPRB, 0.69678E+00_JPRB, 0.67616E+00_JPRB, 0.69249E+00_JPRB, 0.68773E+00_JPRB, 0.67225E+00_JPRB ,&
&  0.69230E+00_JPRB, 0.70086E+00_JPRB, 0.69892E+00_JPRB, 0.70955E+00_JPRB, 0.70370E+00_JPRB, 0.77586E+00_JPRB, 0.73538E+00_JPRB ,&
&  0.71143E+00_JPRB, 0.71650E+00_JPRB, 0.72596E+00_JPRB, 0.70970E+00_JPRB, 0.69524E+00_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.64488E+00_JPRB, 0.62640E+00_JPRB, 0.61480E+00_JPRB, 0.62068E+00_JPRB, 0.59944E+00_JPRB, 0.58498E+00_JPRB, 0.58266E+00_JPRB ,&
&  0.55713E+00_JPRB, 0.54214E+00_JPRB, 0.53435E+00_JPRB, 0.53766E+00_JPRB, 0.54241E+00_JPRB, 0.53617E+00_JPRB, 0.53490E+00_JPRB ,&
&  0.52483E+00_JPRB, 0.51681E+00_JPRB, 0.52004E+00_JPRB, 0.54066E+00_JPRB, 0.56748E+00_JPRB, 0.57469E+00_JPRB, 0.58873E+00_JPRB ,&
&  0.60316E+00_JPRB, 0.59718E+00_JPRB, 0.58693E+00_JPRB, 0.58699E+00_JPRB, 0.59249E+00_JPRB, 0.58945E+00_JPRB, 0.60233E+00_JPRB ,&
&  0.63265E+00_JPRB, 0.11316E+01_JPRB, 0.12525E+01_JPRB, 0.20650E+01_JPRB, 0.18177E+01_JPRB, 0.15183E+01_JPRB, 0.11332E+01_JPRB ,&
&  0.10157E+01_JPRB, 0.10761E+01_JPRB, 0.98236E+00_JPRB, 0.88037E+00_JPRB, 0.77143E+00_JPRB, 0.69212E+00_JPRB, 0.67122E+00_JPRB ,&
&  0.68764E+00_JPRB, 0.70414E+00_JPRB, 0.71900E+00_JPRB, 0.70651E+00_JPRB, 0.69319E+00_JPRB, 0.69195E+00_JPRB, 0.69627E+00_JPRB ,&
&  0.70267E+00_JPRB, 0.71453E+00_JPRB, 0.76387E+00_JPRB, 0.85930E+00_JPRB, 0.11268E+01_JPRB, 0.12880E+01_JPRB, 0.12513E+01_JPRB ,&
&  0.12124E+01_JPRB, 0.13017E+01_JPRB, 0.11903E+01_JPRB, 0.10741E+01_JPRB, 0.98659E+00_JPRB, 0.96884E+00_JPRB, 0.93468E+00_JPRB ,&
&  0.91214E+00_JPRB, 0.90522E+00_JPRB, 0.87986E+00_JPRB, 0.84538E+00_JPRB, 0.81387E+00_JPRB, 0.77867E+00_JPRB, 0.74049E+00_JPRB ,&
&  0.71802E+00_JPRB, 0.70698E+00_JPRB, 0.69769E+00_JPRB, 0.70066E+00_JPRB, 0.71544E+00_JPRB, 0.73022E+00_JPRB, 0.74531E+00_JPRB ,&
&  0.74604E+00_JPRB, 0.75184E+00_JPRB, 0.72000E+00_JPRB, 0.70145E+00_JPRB, 0.71389E+00_JPRB, 0.69626E+00_JPRB, 0.68227E+00_JPRB ,&
&  0.70548E+00_JPRB, 0.72966E+00_JPRB, 0.74413E+00_JPRB, 0.74032E+00_JPRB, 0.74983E+00_JPRB, 0.80147E+00_JPRB, 0.76193E+00_JPRB ,&
&  0.73477E+00_JPRB, 0.76491E+00_JPRB, 0.77737E+00_JPRB, 0.71425E+00_JPRB, 0.67909E+00_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.65579E+00_JPRB, 0.65398E+00_JPRB, 0.64047E+00_JPRB, 0.63593E+00_JPRB, 0.60387E+00_JPRB, 0.57632E+00_JPRB, 0.55349E+00_JPRB ,&
&  0.54297E+00_JPRB, 0.54089E+00_JPRB, 0.55354E+00_JPRB, 0.55775E+00_JPRB, 0.55905E+00_JPRB, 0.55134E+00_JPRB, 0.54532E+00_JPRB ,&
&  0.53220E+00_JPRB, 0.53463E+00_JPRB, 0.54686E+00_JPRB, 0.56735E+00_JPRB, 0.59022E+00_JPRB, 0.59969E+00_JPRB, 0.61009E+00_JPRB ,&
&  0.61024E+00_JPRB, 0.60635E+00_JPRB, 0.60885E+00_JPRB, 0.61918E+00_JPRB, 0.63596E+00_JPRB, 0.65248E+00_JPRB, 0.67086E+00_JPRB ,&
&  0.69164E+00_JPRB, 0.11005E+01_JPRB, 0.16369E+01_JPRB, 0.18849E+01_JPRB, 0.16456E+01_JPRB, 0.13397E+01_JPRB, 0.11409E+01_JPRB ,&
&  0.95508E+00_JPRB, 0.80908E+00_JPRB, 0.89539E+00_JPRB, 0.82085E+00_JPRB, 0.73302E+00_JPRB, 0.69542E+00_JPRB, 0.69808E+00_JPRB ,&
&  0.72266E+00_JPRB, 0.72652E+00_JPRB, 0.72953E+00_JPRB, 0.73622E+00_JPRB, 0.73049E+00_JPRB, 0.73214E+00_JPRB, 0.73990E+00_JPRB ,&
&  0.77178E+00_JPRB, 0.84101E+00_JPRB, 0.95479E+00_JPRB, 0.11378E+01_JPRB, 0.13690E+01_JPRB, 0.12647E+01_JPRB, 0.11369E+01_JPRB ,&
&  0.11142E+01_JPRB, 0.10834E+01_JPRB, 0.10287E+01_JPRB, 0.98194E+00_JPRB, 0.86102E+00_JPRB, 0.93984E+00_JPRB, 0.90694E+00_JPRB ,&
&  0.87841E+00_JPRB, 0.85458E+00_JPRB, 0.82198E+00_JPRB, 0.79587E+00_JPRB, 0.77275E+00_JPRB, 0.74590E+00_JPRB, 0.72766E+00_JPRB ,&
&  0.72280E+00_JPRB, 0.72496E+00_JPRB, 0.72946E+00_JPRB, 0.73767E+00_JPRB, 0.75110E+00_JPRB, 0.76796E+00_JPRB, 0.77867E+00_JPRB ,&
&  0.78060E+00_JPRB, 0.78902E+00_JPRB, 0.77556E+00_JPRB, 0.78120E+00_JPRB, 0.78162E+00_JPRB, 0.75948E+00_JPRB, 0.73607E+00_JPRB ,&
&  0.74786E+00_JPRB, 0.78243E+00_JPRB, 0.79963E+00_JPRB, 0.80163E+00_JPRB, 0.80944E+00_JPRB, 0.83412E+00_JPRB, 0.78747E+00_JPRB ,&
&  0.77639E+00_JPRB, 0.83885E+00_JPRB, 0.80532E+00_JPRB, 0.72229E+00_JPRB, 0.68140E+00_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.66940E+00_JPRB, 0.67197E+00_JPRB, 0.66249E+00_JPRB, 0.62757E+00_JPRB, 0.59571E+00_JPRB, 0.56284E+00_JPRB, 0.54041E+00_JPRB ,&
&  0.53866E+00_JPRB, 0.54947E+00_JPRB, 0.58103E+00_JPRB, 0.59860E+00_JPRB, 0.58837E+00_JPRB, 0.57118E+00_JPRB, 0.56519E+00_JPRB ,&
&  0.56899E+00_JPRB, 0.58088E+00_JPRB, 0.58722E+00_JPRB, 0.60133E+00_JPRB, 0.61950E+00_JPRB, 0.63701E+00_JPRB, 0.65562E+00_JPRB ,&
&  0.64331E+00_JPRB, 0.64816E+00_JPRB, 0.66262E+00_JPRB, 0.67198E+00_JPRB, 0.68751E+00_JPRB, 0.71980E+00_JPRB, 0.74003E+00_JPRB ,&
&  0.83474E+00_JPRB, 0.12303E+01_JPRB, 0.78775E+00_JPRB, 0.10945E+01_JPRB, 0.12673E+01_JPRB, 0.12538E+01_JPRB, 0.11086E+01_JPRB ,&
&  0.94906E+00_JPRB, 0.80053E+00_JPRB, 0.79946E+00_JPRB, 0.82228E+00_JPRB, 0.76412E+00_JPRB, 0.76009E+00_JPRB, 0.76302E+00_JPRB ,&
&  0.74852E+00_JPRB, 0.75923E+00_JPRB, 0.76168E+00_JPRB, 0.77176E+00_JPRB, 0.77952E+00_JPRB, 0.79873E+00_JPRB, 0.84966E+00_JPRB ,&
&  0.94453E+00_JPRB, 0.10454E+01_JPRB, 0.12131E+01_JPRB, 0.12925E+01_JPRB, 0.14601E+01_JPRB, 0.11991E+01_JPRB, 0.10172E+01_JPRB ,&
&  0.89662E+00_JPRB, 0.93851E+00_JPRB, 0.97626E+00_JPRB, 0.96784E+00_JPRB, 0.88834E+00_JPRB, 0.82691E+00_JPRB, 0.86506E+00_JPRB ,&
&  0.83318E+00_JPRB, 0.80745E+00_JPRB, 0.80034E+00_JPRB, 0.78836E+00_JPRB, 0.77054E+00_JPRB, 0.75779E+00_JPRB, 0.75152E+00_JPRB ,&
&  0.74758E+00_JPRB, 0.75332E+00_JPRB, 0.76502E+00_JPRB, 0.77058E+00_JPRB, 0.78172E+00_JPRB, 0.79230E+00_JPRB, 0.80783E+00_JPRB ,&
&  0.82397E+00_JPRB, 0.84683E+00_JPRB, 0.87384E+00_JPRB, 0.88114E+00_JPRB, 0.87433E+00_JPRB, 0.84994E+00_JPRB, 0.83139E+00_JPRB ,&
&  0.82855E+00_JPRB, 0.84773E+00_JPRB, 0.84170E+00_JPRB, 0.83533E+00_JPRB, 0.88683E+00_JPRB, 0.86424E+00_JPRB, 0.82918E+00_JPRB ,&
&  0.87905E+00_JPRB, 0.10340E+01_JPRB, 0.10871E+01_JPRB, 0.72864E+00_JPRB, 0.68607E+00_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.63799E+00_JPRB, 0.62134E+00_JPRB, 0.59380E+00_JPRB, 0.55966E+00_JPRB, 0.55575E+00_JPRB, 0.55270E+00_JPRB, 0.56444E+00_JPRB ,&
&  0.59940E+00_JPRB, 0.64752E+00_JPRB, 0.66202E+00_JPRB, 0.65378E+00_JPRB, 0.63710E+00_JPRB, 0.63125E+00_JPRB, 0.62738E+00_JPRB ,&
&  0.62916E+00_JPRB, 0.62745E+00_JPRB, 0.64230E+00_JPRB, 0.66233E+00_JPRB, 0.68546E+00_JPRB, 0.70225E+00_JPRB, 0.70325E+00_JPRB ,&
&  0.70770E+00_JPRB, 0.71264E+00_JPRB, 0.72921E+00_JPRB, 0.74834E+00_JPRB, 0.78740E+00_JPRB, 0.86139E+00_JPRB, 0.97137E+00_JPRB ,&
&  0.12484E+01_JPRB, 0.97557E+00_JPRB, 0.55333E+00_JPRB, 0.10591E+01_JPRB, 0.12203E+01_JPRB, 0.12870E+01_JPRB, 0.10879E+01_JPRB ,&
&  0.98957E+00_JPRB, 0.87433E+00_JPRB, 0.82929E+00_JPRB, 0.88845E+00_JPRB, 0.86548E+00_JPRB, 0.86407E+00_JPRB, 0.86243E+00_JPRB ,&
&  0.83379E+00_JPRB, 0.83285E+00_JPRB, 0.82791E+00_JPRB, 0.83874E+00_JPRB, 0.86197E+00_JPRB, 0.91148E+00_JPRB, 0.99950E+00_JPRB ,&
&  0.10989E+01_JPRB, 0.12210E+01_JPRB, 0.13793E+01_JPRB, 0.13832E+01_JPRB, 0.14288E+01_JPRB, 0.11156E+01_JPRB, 0.96456E+00_JPRB ,&
&  0.85331E+00_JPRB, 0.89023E+00_JPRB, 0.95944E+00_JPRB, 0.97497E+00_JPRB, 0.91268E+00_JPRB, 0.80180E+00_JPRB, 0.84774E+00_JPRB ,&
&  0.83583E+00_JPRB, 0.81727E+00_JPRB, 0.81209E+00_JPRB, 0.79492E+00_JPRB, 0.77950E+00_JPRB, 0.77376E+00_JPRB, 0.77460E+00_JPRB ,&
&  0.77493E+00_JPRB, 0.78474E+00_JPRB, 0.78871E+00_JPRB, 0.79028E+00_JPRB, 0.79777E+00_JPRB, 0.80283E+00_JPRB, 0.82912E+00_JPRB ,&
&  0.85406E+00_JPRB, 0.87409E+00_JPRB, 0.90180E+00_JPRB, 0.92579E+00_JPRB, 0.91503E+00_JPRB, 0.87855E+00_JPRB, 0.87094E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.86583E+00_JPRB, 0.85960E+00_JPRB, 0.88908E+00_JPRB, 0.86866E+00_JPRB, 0.82715E+00_JPRB, 0.77802E+00_JPRB ,&
&  0.78225E+00_JPRB, 0.78005E+00_JPRB, 0.73952E+00_JPRB, 0.67680E+00_JPRB, 0.64810E+00_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.60969E+00_JPRB, 0.58923E+00_JPRB, 0.57612E+00_JPRB, 0.58191E+00_JPRB, 0.59066E+00_JPRB, 0.60979E+00_JPRB, 0.65680E+00_JPRB ,&
&  0.69345E+00_JPRB, 0.71021E+00_JPRB, 0.73056E+00_JPRB, 0.72835E+00_JPRB, 0.73399E+00_JPRB, 0.71227E+00_JPRB, 0.70230E+00_JPRB ,&
&  0.68540E+00_JPRB, 0.69344E+00_JPRB, 0.70984E+00_JPRB, 0.72521E+00_JPRB, 0.73744E+00_JPRB, 0.74711E+00_JPRB, 0.75955E+00_JPRB ,&
&  0.77890E+00_JPRB, 0.80662E+00_JPRB, 0.84901E+00_JPRB, 0.90690E+00_JPRB, 0.10026E+01_JPRB, 0.11213E+01_JPRB, 0.11822E+01_JPRB ,&
&  0.86298E+00_JPRB, 0.43755E+00_JPRB, 0.83101E+00_JPRB, 0.12285E+01_JPRB, 0.12809E+01_JPRB, 0.12908E+01_JPRB, 0.11984E+01_JPRB ,&
&  0.10550E+01_JPRB, 0.96219E+00_JPRB, 0.89871E+00_JPRB, 0.98077E+00_JPRB, 0.95709E+00_JPRB, 0.94748E+00_JPRB, 0.93935E+00_JPRB ,&
&  0.92332E+00_JPRB, 0.91618E+00_JPRB, 0.91568E+00_JPRB, 0.93470E+00_JPRB, 0.97137E+00_JPRB, 0.10325E+01_JPRB, 0.11117E+01_JPRB ,&
&  0.12187E+01_JPRB, 0.13779E+01_JPRB, 0.15053E+01_JPRB, 0.13627E+01_JPRB, 0.13581E+01_JPRB, 0.11697E+01_JPRB, 0.10001E+01_JPRB ,&
&  0.88705E+00_JPRB, 0.84032E+00_JPRB, 0.87344E+00_JPRB, 0.99070E+00_JPRB, 0.94284E+00_JPRB, 0.80902E+00_JPRB, 0.85821E+00_JPRB ,&
&  0.85103E+00_JPRB, 0.83066E+00_JPRB, 0.82308E+00_JPRB, 0.80217E+00_JPRB, 0.79463E+00_JPRB, 0.79436E+00_JPRB, 0.80116E+00_JPRB ,&
&  0.80054E+00_JPRB, 0.80292E+00_JPRB, 0.80336E+00_JPRB, 0.79903E+00_JPRB, 0.78482E+00_JPRB, 0.78945E+00_JPRB, 0.81240E+00_JPRB ,&
&  0.83227E+00_JPRB, 0.83831E+00_JPRB, 0.86274E+00_JPRB, 0.88709E+00_JPRB, 0.91459E+00_JPRB, 0.89490E+00_JPRB, 0.88357E+00_JPRB ,&
&  0.86732E+00_JPRB, 0.86823E+00_JPRB, 0.88241E+00_JPRB, 0.88965E+00_JPRB, 0.89219E+00_JPRB, 0.84347E+00_JPRB, 0.76861E+00_JPRB ,&
&  0.74301E+00_JPRB, 0.71338E+00_JPRB, 0.68847E+00_JPRB, 0.66527E+00_JPRB, 0.63939E+00_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.66933E+00_JPRB, 0.67617E+00_JPRB, 0.66881E+00_JPRB, 0.68933E+00_JPRB, 0.70818E+00_JPRB, 0.73688E+00_JPRB, 0.76226E+00_JPRB ,&
&  0.78433E+00_JPRB, 0.80589E+00_JPRB, 0.83255E+00_JPRB, 0.80617E+00_JPRB, 0.80968E+00_JPRB, 0.79187E+00_JPRB, 0.76352E+00_JPRB ,&
&  0.75530E+00_JPRB, 0.76637E+00_JPRB, 0.77885E+00_JPRB, 0.79021E+00_JPRB, 0.80021E+00_JPRB, 0.82253E+00_JPRB, 0.85412E+00_JPRB ,&
&  0.89490E+00_JPRB, 0.94684E+00_JPRB, 0.10176E+01_JPRB, 0.10742E+01_JPRB, 0.11348E+01_JPRB, 0.11484E+01_JPRB, 0.10745E+01_JPRB ,&
&  0.53899E+00_JPRB, 0.74576E+00_JPRB, 0.11804E+01_JPRB, 0.13340E+01_JPRB, 0.13923E+01_JPRB, 0.13650E+01_JPRB, 0.12848E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.98746E+00_JPRB, 0.95365E+00_JPRB, 0.10584E+01_JPRB, 0.10227E+01_JPRB, 0.10165E+01_JPRB, 0.10011E+01_JPRB ,&
&  0.99308E+00_JPRB, 0.10043E+01_JPRB, 0.10243E+01_JPRB, 0.10414E+01_JPRB, 0.10840E+01_JPRB, 0.11419E+01_JPRB, 0.12265E+01_JPRB ,&
&  0.13388E+01_JPRB, 0.14612E+01_JPRB, 0.15761E+01_JPRB, 0.13809E+01_JPRB, 0.13359E+01_JPRB, 0.12420E+01_JPRB, 0.11257E+01_JPRB ,&
&  0.88589E+00_JPRB, 0.82449E+00_JPRB, 0.88566E+00_JPRB, 0.10005E+01_JPRB, 0.94884E+00_JPRB, 0.89657E+00_JPRB, 0.87546E+00_JPRB ,&
&  0.87910E+00_JPRB, 0.84302E+00_JPRB, 0.83255E+00_JPRB, 0.81180E+00_JPRB, 0.79849E+00_JPRB, 0.80430E+00_JPRB, 0.81942E+00_JPRB ,&
&  0.81164E+00_JPRB, 0.79450E+00_JPRB, 0.79036E+00_JPRB, 0.79475E+00_JPRB, 0.77833E+00_JPRB, 0.76347E+00_JPRB, 0.76177E+00_JPRB ,&
&  0.77003E+00_JPRB, 0.75880E+00_JPRB, 0.76961E+00_JPRB, 0.80266E+00_JPRB, 0.84604E+00_JPRB, 0.87281E+00_JPRB, 0.88655E+00_JPRB ,&
&  0.87977E+00_JPRB, 0.90513E+00_JPRB, 0.10456E+01_JPRB, 0.11027E+01_JPRB, 0.11098E+01_JPRB, 0.93817E+00_JPRB, 0.77284E+00_JPRB ,&
&  0.75189E+00_JPRB, 0.72211E+00_JPRB, 0.69415E+00_JPRB, 0.68347E+00_JPRB, 0.67190E+00_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.73915E+00_JPRB, 0.75087E+00_JPRB, 0.77032E+00_JPRB, 0.80241E+00_JPRB, 0.83772E+00_JPRB, 0.86165E+00_JPRB, 0.88493E+00_JPRB ,&
&  0.89297E+00_JPRB, 0.90341E+00_JPRB, 0.91105E+00_JPRB, 0.86282E+00_JPRB, 0.86917E+00_JPRB, 0.84754E+00_JPRB, 0.83671E+00_JPRB ,&
&  0.82766E+00_JPRB, 0.84105E+00_JPRB, 0.85819E+00_JPRB, 0.87160E+00_JPRB, 0.89326E+00_JPRB, 0.92265E+00_JPRB, 0.96856E+00_JPRB ,&
&  0.10263E+01_JPRB, 0.10772E+01_JPRB, 0.11421E+01_JPRB, 0.11762E+01_JPRB, 0.12115E+01_JPRB, 0.12259E+01_JPRB, 0.11884E+01_JPRB ,&
&  0.74984E+00_JPRB, 0.12532E+01_JPRB, 0.13721E+01_JPRB, 0.14275E+01_JPRB, 0.14360E+01_JPRB, 0.13097E+01_JPRB, 0.12518E+01_JPRB ,&
&  0.11507E+01_JPRB, 0.10317E+01_JPRB, 0.10132E+01_JPRB, 0.10607E+01_JPRB, 0.11174E+01_JPRB, 0.11017E+01_JPRB, 0.10949E+01_JPRB ,&
&  0.10928E+01_JPRB, 0.11120E+01_JPRB, 0.11364E+01_JPRB, 0.11629E+01_JPRB, 0.12044E+01_JPRB, 0.12522E+01_JPRB, 0.13285E+01_JPRB ,&
&  0.14071E+01_JPRB, 0.14904E+01_JPRB, 0.15617E+01_JPRB, 0.14142E+01_JPRB, 0.12707E+01_JPRB, 0.12691E+01_JPRB, 0.11552E+01_JPRB ,&
&  0.85811E+00_JPRB, 0.75985E+00_JPRB, 0.84564E+00_JPRB, 0.97940E+00_JPRB, 0.93904E+00_JPRB, 0.86266E+00_JPRB, 0.81068E+00_JPRB ,&
&  0.79863E+00_JPRB, 0.76584E+00_JPRB, 0.77813E+00_JPRB, 0.74627E+00_JPRB, 0.76790E+00_JPRB, 0.76281E+00_JPRB, 0.77108E+00_JPRB ,&
&  0.77665E+00_JPRB, 0.74871E+00_JPRB, 0.73601E+00_JPRB, 0.73101E+00_JPRB, 0.72218E+00_JPRB, 0.71260E+00_JPRB, 0.70560E+00_JPRB ,&
&  0.71489E+00_JPRB, 0.68053E+00_JPRB, 0.66138E+00_JPRB, 0.68044E+00_JPRB, 0.71944E+00_JPRB, 0.76658E+00_JPRB, 0.86208E+00_JPRB ,&
&  0.10214E+01_JPRB, 0.11912E+01_JPRB, 0.14840E+01_JPRB, 0.15790E+01_JPRB, 0.15350E+01_JPRB, 0.11514E+01_JPRB, 0.83931E+00_JPRB ,&
&  0.83088E+00_JPRB, 0.78529E+00_JPRB, 0.74909E+00_JPRB, 0.72807E+00_JPRB, 0.72926E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.88195E+00_JPRB, 0.90143E+00_JPRB, 0.92338E+00_JPRB, 0.94138E+00_JPRB, 0.94300E+00_JPRB, 0.94208E+00_JPRB, 0.93881E+00_JPRB ,&
&  0.96169E+00_JPRB, 0.93706E+00_JPRB, 0.95921E+00_JPRB, 0.93265E+00_JPRB, 0.95069E+00_JPRB, 0.92903E+00_JPRB, 0.92894E+00_JPRB ,&
&  0.93562E+00_JPRB, 0.95823E+00_JPRB, 0.98638E+00_JPRB, 0.99372E+00_JPRB, 0.98628E+00_JPRB, 0.10092E+01_JPRB, 0.10466E+01_JPRB ,&
&  0.10920E+01_JPRB, 0.11128E+01_JPRB, 0.11483E+01_JPRB, 0.11608E+01_JPRB, 0.11334E+01_JPRB, 0.11060E+01_JPRB, 0.79068E+00_JPRB ,&
&  0.11618E+01_JPRB, 0.13700E+01_JPRB, 0.14740E+01_JPRB, 0.14134E+01_JPRB, 0.12870E+01_JPRB, 0.11908E+01_JPRB, 0.11693E+01_JPRB ,&
&  0.11518E+01_JPRB, 0.11113E+01_JPRB, 0.10659E+01_JPRB, 0.10677E+01_JPRB, 0.11849E+01_JPRB, 0.11530E+01_JPRB, 0.11635E+01_JPRB ,&
&  0.11716E+01_JPRB, 0.11873E+01_JPRB, 0.12108E+01_JPRB, 0.12412E+01_JPRB, 0.12655E+01_JPRB, 0.12912E+01_JPRB, 0.13307E+01_JPRB ,&
&  0.13552E+01_JPRB, 0.13587E+01_JPRB, 0.13937E+01_JPRB, 0.12577E+01_JPRB, 0.11910E+01_JPRB, 0.11941E+01_JPRB, 0.10324E+01_JPRB ,&
&  0.77056E+00_JPRB, 0.72220E+00_JPRB, 0.82419E+00_JPRB, 0.92764E+00_JPRB, 0.89970E+00_JPRB, 0.80959E+00_JPRB, 0.75902E+00_JPRB ,&
&  0.73043E+00_JPRB, 0.70951E+00_JPRB, 0.68463E+00_JPRB, 0.69270E+00_JPRB, 0.69144E+00_JPRB, 0.70360E+00_JPRB, 0.71599E+00_JPRB ,&
&  0.72916E+00_JPRB, 0.71045E+00_JPRB, 0.68882E+00_JPRB, 0.67818E+00_JPRB, 0.68396E+00_JPRB, 0.73713E+00_JPRB, 0.80502E+00_JPRB ,&
&  0.83271E+00_JPRB, 0.53671E+00_JPRB, 0.58816E+00_JPRB, 0.57863E+00_JPRB, 0.60085E+00_JPRB, 0.67424E+00_JPRB, 0.85863E+00_JPRB ,&
&  0.13201E+01_JPRB, 0.20577E+01_JPRB, 0.23307E+01_JPRB, 0.19956E+01_JPRB, 0.24630E+01_JPRB, 0.17532E+01_JPRB, 0.96176E+00_JPRB ,&
&  0.93845E+00_JPRB, 0.90595E+00_JPRB, 0.86618E+00_JPRB, 0.85314E+00_JPRB, 0.85972E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.93455E+00_JPRB, 0.94644E+00_JPRB, 0.94225E+00_JPRB, 0.95408E+00_JPRB, 0.93121E+00_JPRB, 0.94658E+00_JPRB, 0.94524E+00_JPRB ,&
&  0.96142E+00_JPRB, 0.96849E+00_JPRB, 0.98402E+00_JPRB, 0.99635E+00_JPRB, 0.10193E+01_JPRB, 0.10389E+01_JPRB, 0.10521E+01_JPRB ,&
&  0.10726E+01_JPRB, 0.10927E+01_JPRB, 0.11093E+01_JPRB, 0.11173E+01_JPRB, 0.10913E+01_JPRB, 0.10879E+01_JPRB, 0.10776E+01_JPRB ,&
&  0.10906E+01_JPRB, 0.10783E+01_JPRB, 0.10859E+01_JPRB, 0.10846E+01_JPRB, 0.10622E+01_JPRB, 0.10642E+01_JPRB, 0.77724E+00_JPRB ,&
&  0.11999E+01_JPRB, 0.13533E+01_JPRB, 0.13675E+01_JPRB, 0.12635E+01_JPRB, 0.11651E+01_JPRB, 0.11334E+01_JPRB, 0.11269E+01_JPRB ,&
&  0.11487E+01_JPRB, 0.11599E+01_JPRB, 0.11051E+01_JPRB, 0.11346E+01_JPRB, 0.11499E+01_JPRB, 0.11199E+01_JPRB, 0.11281E+01_JPRB ,&
&  0.11331E+01_JPRB, 0.11409E+01_JPRB, 0.11534E+01_JPRB, 0.11545E+01_JPRB, 0.11394E+01_JPRB, 0.11510E+01_JPRB, 0.11547E+01_JPRB ,&
&  0.11361E+01_JPRB, 0.11404E+01_JPRB, 0.11623E+01_JPRB, 0.10978E+01_JPRB, 0.10604E+01_JPRB, 0.94022E+00_JPRB, 0.83899E+00_JPRB ,&
&  0.66830E+00_JPRB, 0.65613E+00_JPRB, 0.74374E+00_JPRB, 0.90206E+00_JPRB, 0.87016E+00_JPRB, 0.80983E+00_JPRB, 0.75306E+00_JPRB ,&
&  0.69015E+00_JPRB, 0.67288E+00_JPRB, 0.64647E+00_JPRB, 0.63631E+00_JPRB, 0.63400E+00_JPRB, 0.64969E+00_JPRB, 0.65857E+00_JPRB ,&
&  0.67010E+00_JPRB, 0.67973E+00_JPRB, 0.66581E+00_JPRB, 0.64843E+00_JPRB, 0.63613E+00_JPRB, 0.65478E+00_JPRB, 0.57986E+00_JPRB ,&
&  0.65694E+00_JPRB, 0.55597E+00_JPRB, 0.55465E+00_JPRB, 0.53812E+00_JPRB, 0.50769E+00_JPRB, 0.54988E+00_JPRB, 0.73360E+00_JPRB ,&
&  0.12302E+01_JPRB, 0.16028E+01_JPRB, 0.23116E+01_JPRB, 0.39342E+01_JPRB, 0.27626E+01_JPRB, 0.20285E+01_JPRB, 0.10233E+01_JPRB ,&
&  0.95259E+00_JPRB, 0.92571E+00_JPRB, 0.91015E+00_JPRB, 0.89834E+00_JPRB, 0.91462E+00_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.89711E+00_JPRB, 0.90605E+00_JPRB, 0.91424E+00_JPRB, 0.92292E+00_JPRB, 0.92927E+00_JPRB, 0.94516E+00_JPRB, 0.95018E+00_JPRB ,&
&  0.96061E+00_JPRB, 0.96567E+00_JPRB, 0.98005E+00_JPRB, 0.99888E+00_JPRB, 0.10121E+01_JPRB, 0.10333E+01_JPRB, 0.10589E+01_JPRB ,&
&  0.10879E+01_JPRB, 0.11050E+01_JPRB, 0.11268E+01_JPRB, 0.11306E+01_JPRB, 0.11142E+01_JPRB, 0.11118E+01_JPRB, 0.11058E+01_JPRB ,&
&  0.10881E+01_JPRB, 0.10775E+01_JPRB, 0.10844E+01_JPRB, 0.10946E+01_JPRB, 0.10842E+01_JPRB, 0.10743E+01_JPRB, 0.81175E+00_JPRB ,&
&  0.11495E+01_JPRB, 0.12799E+01_JPRB, 0.12658E+01_JPRB, 0.11411E+01_JPRB, 0.10642E+01_JPRB, 0.10538E+01_JPRB, 0.10660E+01_JPRB ,&
&  0.11040E+01_JPRB, 0.11481E+01_JPRB, 0.11056E+01_JPRB, 0.10380E+01_JPRB, 0.10074E+01_JPRB, 0.10012E+01_JPRB, 0.99394E+00_JPRB ,&
&  0.99384E+00_JPRB, 0.99653E+00_JPRB, 0.98988E+00_JPRB, 0.96723E+00_JPRB, 0.93816E+00_JPRB, 0.91434E+00_JPRB, 0.88333E+00_JPRB ,&
&  0.85125E+00_JPRB, 0.83512E+00_JPRB, 0.76378E+00_JPRB, 0.79566E+00_JPRB, 0.76874E+00_JPRB, 0.68479E+00_JPRB, 0.58852E+00_JPRB ,&
&  0.50351E+00_JPRB, 0.54344E+00_JPRB, 0.67462E+00_JPRB, 0.88754E+00_JPRB, 0.85559E+00_JPRB, 0.79543E+00_JPRB, 0.76239E+00_JPRB ,&
&  0.71313E+00_JPRB, 0.66533E+00_JPRB, 0.62924E+00_JPRB, 0.60793E+00_JPRB, 0.59893E+00_JPRB, 0.61232E+00_JPRB, 0.62528E+00_JPRB ,&
&  0.62949E+00_JPRB, 0.63344E+00_JPRB, 0.62105E+00_JPRB, 0.61278E+00_JPRB, 0.59564E+00_JPRB, 0.53815E+00_JPRB, 0.60594E+00_JPRB ,&
&  0.58196E+00_JPRB, 0.47249E+00_JPRB, 0.55256E+00_JPRB, 0.38019E+00_JPRB, 0.48788E+00_JPRB, 0.52255E+00_JPRB, 0.63054E+00_JPRB ,&
&  0.76140E+00_JPRB, 0.10819E+01_JPRB, 0.15022E+01_JPRB, 0.13036E+01_JPRB, 0.11638E+01_JPRB, 0.10485E+01_JPRB, 0.95361E+00_JPRB ,&
&  0.91308E+00_JPRB, 0.89688E+00_JPRB, 0.89021E+00_JPRB, 0.88310E+00_JPRB, 0.88766E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.87282E+00_JPRB, 0.88052E+00_JPRB, 0.88746E+00_JPRB, 0.89919E+00_JPRB, 0.91031E+00_JPRB, 0.92522E+00_JPRB, 0.93558E+00_JPRB ,&
&  0.94604E+00_JPRB, 0.95306E+00_JPRB, 0.95224E+00_JPRB, 0.96491E+00_JPRB, 0.97487E+00_JPRB, 0.99516E+00_JPRB, 0.10184E+01_JPRB ,&
&  0.10424E+01_JPRB, 0.10682E+01_JPRB, 0.10966E+01_JPRB, 0.11096E+01_JPRB, 0.11189E+01_JPRB, 0.11325E+01_JPRB, 0.11442E+01_JPRB ,&
&  0.11505E+01_JPRB, 0.11539E+01_JPRB, 0.11669E+01_JPRB, 0.11910E+01_JPRB, 0.12201E+01_JPRB, 0.12511E+01_JPRB, 0.10488E+01_JPRB ,&
&  0.11266E+01_JPRB, 0.11887E+01_JPRB, 0.11240E+01_JPRB, 0.10060E+01_JPRB, 0.96199E+00_JPRB, 0.93239E+00_JPRB, 0.92255E+00_JPRB ,&
&  0.93778E+00_JPRB, 0.91599E+00_JPRB, 0.89105E+00_JPRB, 0.84581E+00_JPRB, 0.82429E+00_JPRB, 0.83958E+00_JPRB, 0.86170E+00_JPRB ,&
&  0.86788E+00_JPRB, 0.86977E+00_JPRB, 0.85692E+00_JPRB, 0.83532E+00_JPRB, 0.81465E+00_JPRB, 0.79436E+00_JPRB, 0.76105E+00_JPRB ,&
&  0.72085E+00_JPRB, 0.70917E+00_JPRB, 0.61949E+00_JPRB, 0.62276E+00_JPRB, 0.63667E+00_JPRB, 0.60565E+00_JPRB, 0.53602E+00_JPRB ,&
&  0.46368E+00_JPRB, 0.45286E+00_JPRB, 0.51693E+00_JPRB, 0.81078E+00_JPRB, 0.86742E+00_JPRB, 0.82300E+00_JPRB, 0.77987E+00_JPRB ,&
&  0.73622E+00_JPRB, 0.70312E+00_JPRB, 0.64403E+00_JPRB, 0.60327E+00_JPRB, 0.57901E+00_JPRB, 0.58755E+00_JPRB, 0.59408E+00_JPRB ,&
&  0.59229E+00_JPRB, 0.58337E+00_JPRB, 0.57128E+00_JPRB, 0.55811E+00_JPRB, 0.57381E+00_JPRB, 0.54095E+00_JPRB, 0.58546E+00_JPRB ,&
&  0.55652E+00_JPRB, 0.41633E+00_JPRB, 0.44028E+00_JPRB, 0.54517E+00_JPRB, 0.52508E+00_JPRB, 0.59083E+00_JPRB, 0.70680E+00_JPRB ,&
&  0.90963E+00_JPRB, 0.10251E+01_JPRB, 0.99946E+00_JPRB, 0.95970E+00_JPRB, 0.89930E+00_JPRB, 0.89487E+00_JPRB, 0.87173E+00_JPRB ,&
&  0.85331E+00_JPRB, 0.84971E+00_JPRB, 0.85113E+00_JPRB, 0.85439E+00_JPRB, 0.86256E+00_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.82920E+00_JPRB, 0.84363E+00_JPRB, 0.85491E+00_JPRB, 0.86223E+00_JPRB, 0.86549E+00_JPRB, 0.86871E+00_JPRB, 0.87476E+00_JPRB ,&
&  0.87720E+00_JPRB, 0.87470E+00_JPRB, 0.87239E+00_JPRB, 0.88638E+00_JPRB, 0.90366E+00_JPRB, 0.91040E+00_JPRB, 0.92880E+00_JPRB ,&
&  0.92236E+00_JPRB, 0.93194E+00_JPRB, 0.94734E+00_JPRB, 0.95699E+00_JPRB, 0.97673E+00_JPRB, 0.97641E+00_JPRB, 0.96102E+00_JPRB ,&
&  0.95451E+00_JPRB, 0.94574E+00_JPRB, 0.94461E+00_JPRB, 0.94408E+00_JPRB, 0.95337E+00_JPRB, 0.96695E+00_JPRB, 0.11188E+01_JPRB ,&
&  0.98638E+00_JPRB, 0.11202E+01_JPRB, 0.10124E+01_JPRB, 0.88484E+00_JPRB, 0.87417E+00_JPRB, 0.81595E+00_JPRB, 0.78949E+00_JPRB ,&
&  0.75403E+00_JPRB, 0.71288E+00_JPRB, 0.68530E+00_JPRB, 0.66283E+00_JPRB, 0.67300E+00_JPRB, 0.70042E+00_JPRB, 0.72679E+00_JPRB ,&
&  0.76236E+00_JPRB, 0.77070E+00_JPRB, 0.76921E+00_JPRB, 0.76224E+00_JPRB, 0.74859E+00_JPRB, 0.76368E+00_JPRB, 0.73838E+00_JPRB ,&
&  0.74082E+00_JPRB, 0.78278E+00_JPRB, 0.71504E+00_JPRB, 0.74452E+00_JPRB, 0.75569E+00_JPRB, 0.73945E+00_JPRB, 0.68159E+00_JPRB ,&
&  0.59534E+00_JPRB, 0.55746E+00_JPRB, 0.61858E+00_JPRB, 0.73984E+00_JPRB, 0.82334E+00_JPRB, 0.86683E+00_JPRB, 0.80943E+00_JPRB ,&
&  0.75733E+00_JPRB, 0.72626E+00_JPRB, 0.67906E+00_JPRB, 0.62332E+00_JPRB, 0.60182E+00_JPRB, 0.59557E+00_JPRB, 0.59026E+00_JPRB ,&
&  0.57484E+00_JPRB, 0.56210E+00_JPRB, 0.54733E+00_JPRB, 0.54437E+00_JPRB, 0.50490E+00_JPRB, 0.65402E+00_JPRB, 0.64718E+00_JPRB ,&
&  0.55265E+00_JPRB, 0.50317E+00_JPRB, 0.42882E+00_JPRB, 0.55782E+00_JPRB, 0.57083E+00_JPRB, 0.63157E+00_JPRB, 0.70495E+00_JPRB ,&
&  0.79690E+00_JPRB, 0.86490E+00_JPRB, 0.88273E+00_JPRB, 0.90057E+00_JPRB, 0.79085E+00_JPRB, 0.76774E+00_JPRB, 0.75861E+00_JPRB ,&
&  0.76186E+00_JPRB, 0.77192E+00_JPRB, 0.78388E+00_JPRB, 0.79677E+00_JPRB, 0.81330E+00_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.75862E+00_JPRB, 0.77676E+00_JPRB, 0.79783E+00_JPRB, 0.79968E+00_JPRB, 0.80582E+00_JPRB, 0.80200E+00_JPRB, 0.80333E+00_JPRB ,&
&  0.79550E+00_JPRB, 0.79467E+00_JPRB, 0.79677E+00_JPRB, 0.80155E+00_JPRB, 0.80871E+00_JPRB, 0.81829E+00_JPRB, 0.84534E+00_JPRB ,&
&  0.84494E+00_JPRB, 0.84977E+00_JPRB, 0.85657E+00_JPRB, 0.87011E+00_JPRB, 0.88631E+00_JPRB, 0.88931E+00_JPRB, 0.86970E+00_JPRB ,&
&  0.85528E+00_JPRB, 0.85524E+00_JPRB, 0.86637E+00_JPRB, 0.88467E+00_JPRB, 0.10187E+01_JPRB, 0.12989E+01_JPRB, 0.17250E+01_JPRB ,&
&  0.14100E+01_JPRB, 0.10314E+01_JPRB, 0.94475E+00_JPRB, 0.81750E+00_JPRB, 0.80913E+00_JPRB, 0.82907E+00_JPRB, 0.79627E+00_JPRB ,&
&  0.77530E+00_JPRB, 0.73946E+00_JPRB, 0.72077E+00_JPRB, 0.71805E+00_JPRB, 0.71766E+00_JPRB, 0.69817E+00_JPRB, 0.69281E+00_JPRB ,&
&  0.67165E+00_JPRB, 0.66795E+00_JPRB, 0.68192E+00_JPRB, 0.70311E+00_JPRB, 0.77323E+00_JPRB, 0.84039E+00_JPRB, 0.82839E+00_JPRB ,&
&  0.89586E+00_JPRB, 0.92689E+00_JPRB, 0.89846E+00_JPRB, 0.91927E+00_JPRB, 0.93545E+00_JPRB, 0.90877E+00_JPRB, 0.87040E+00_JPRB ,&
&  0.88016E+00_JPRB, 0.76851E+00_JPRB, 0.61709E+00_JPRB, 0.69120E+00_JPRB, 0.76716E+00_JPRB, 0.88353E+00_JPRB, 0.83866E+00_JPRB ,&
&  0.77435E+00_JPRB, 0.73625E+00_JPRB, 0.69488E+00_JPRB, 0.64483E+00_JPRB, 0.62170E+00_JPRB, 0.61785E+00_JPRB, 0.60632E+00_JPRB ,&
&  0.57809E+00_JPRB, 0.57506E+00_JPRB, 0.56865E+00_JPRB, 0.56098E+00_JPRB, 0.58805E+00_JPRB, 0.54220E+00_JPRB, 0.61598E+00_JPRB ,&
&  0.55021E+00_JPRB, 0.54929E+00_JPRB, 0.48241E+00_JPRB, 0.59855E+00_JPRB, 0.61349E+00_JPRB, 0.63806E+00_JPRB, 0.66773E+00_JPRB ,&
&  0.70111E+00_JPRB, 0.71647E+00_JPRB, 0.73679E+00_JPRB, 0.75106E+00_JPRB, 0.74116E+00_JPRB, 0.72778E+00_JPRB, 0.73278E+00_JPRB ,&
&  0.73668E+00_JPRB, 0.73197E+00_JPRB, 0.73050E+00_JPRB, 0.73167E+00_JPRB, 0.74467E+00_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.65553E+00_JPRB, 0.66540E+00_JPRB, 0.68123E+00_JPRB, 0.69267E+00_JPRB, 0.71489E+00_JPRB, 0.70970E+00_JPRB, 0.70657E+00_JPRB ,&
&  0.68318E+00_JPRB, 0.68026E+00_JPRB, 0.67544E+00_JPRB, 0.67393E+00_JPRB, 0.67740E+00_JPRB, 0.69819E+00_JPRB, 0.73773E+00_JPRB ,&
&  0.74957E+00_JPRB, 0.75470E+00_JPRB, 0.75108E+00_JPRB, 0.74451E+00_JPRB, 0.71956E+00_JPRB, 0.69893E+00_JPRB, 0.68664E+00_JPRB ,&
&  0.70973E+00_JPRB, 0.88461E+00_JPRB, 0.10898E+01_JPRB, 0.12474E+01_JPRB, 0.14079E+01_JPRB, 0.12386E+01_JPRB, 0.13768E+01_JPRB ,&
&  0.12813E+01_JPRB, 0.10233E+01_JPRB, 0.10579E+01_JPRB, 0.95489E+00_JPRB, 0.94666E+00_JPRB, 0.92874E+00_JPRB, 0.90451E+00_JPRB ,&
&  0.91295E+00_JPRB, 0.90498E+00_JPRB, 0.88061E+00_JPRB, 0.86617E+00_JPRB, 0.87298E+00_JPRB, 0.89152E+00_JPRB, 0.90196E+00_JPRB ,&
&  0.91461E+00_JPRB, 0.89615E+00_JPRB, 0.89735E+00_JPRB, 0.92861E+00_JPRB, 0.95864E+00_JPRB, 0.97901E+00_JPRB, 0.10145E+01_JPRB ,&
&  0.10631E+01_JPRB, 0.10623E+01_JPRB, 0.95885E+00_JPRB, 0.96103E+00_JPRB, 0.10170E+01_JPRB, 0.92968E+00_JPRB, 0.93516E+00_JPRB ,&
&  0.10085E+01_JPRB, 0.89390E+00_JPRB, 0.62412E+00_JPRB, 0.67726E+00_JPRB, 0.74963E+00_JPRB, 0.80511E+00_JPRB, 0.89835E+00_JPRB ,&
&  0.82367E+00_JPRB, 0.79468E+00_JPRB, 0.71310E+00_JPRB, 0.66997E+00_JPRB, 0.65960E+00_JPRB, 0.68369E+00_JPRB, 0.66255E+00_JPRB ,&
&  0.66853E+00_JPRB, 0.59658E+00_JPRB, 0.58807E+00_JPRB, 0.59343E+00_JPRB, 0.59033E+00_JPRB, 0.59319E+00_JPRB, 0.58676E+00_JPRB ,&
&  0.58552E+00_JPRB, 0.56860E+00_JPRB, 0.56520E+00_JPRB, 0.57074E+00_JPRB, 0.54137E+00_JPRB, 0.65159E+00_JPRB, 0.66089E+00_JPRB ,&
&  0.66503E+00_JPRB, 0.65564E+00_JPRB, 0.64085E+00_JPRB, 0.63849E+00_JPRB, 0.62725E+00_JPRB, 0.61781E+00_JPRB, 0.62846E+00_JPRB ,&
&  0.64570E+00_JPRB, 0.64206E+00_JPRB, 0.63683E+00_JPRB, 0.63664E+00_JPRB, 0.64345E+00_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.55704E+00_JPRB, 0.56036E+00_JPRB, 0.56301E+00_JPRB, 0.57055E+00_JPRB, 0.58359E+00_JPRB, 0.58951E+00_JPRB, 0.58589E+00_JPRB ,&
&  0.57291E+00_JPRB, 0.56696E+00_JPRB, 0.54829E+00_JPRB, 0.52499E+00_JPRB, 0.52512E+00_JPRB, 0.56855E+00_JPRB, 0.61494E+00_JPRB ,&
&  0.64686E+00_JPRB, 0.66481E+00_JPRB, 0.67154E+00_JPRB, 0.68141E+00_JPRB, 0.72588E+00_JPRB, 0.76489E+00_JPRB, 0.96567E+00_JPRB ,&
&  0.13069E+01_JPRB, 0.17600E+01_JPRB, 0.20434E+01_JPRB, 0.21253E+01_JPRB, 0.18944E+01_JPRB, 0.14062E+01_JPRB, 0.13952E+01_JPRB ,&
&  0.12916E+01_JPRB, 0.10971E+01_JPRB, 0.10445E+01_JPRB, 0.10366E+01_JPRB, 0.10223E+01_JPRB, 0.97271E+00_JPRB, 0.94846E+00_JPRB ,&
&  0.94395E+00_JPRB, 0.94790E+00_JPRB, 0.95309E+00_JPRB, 0.96349E+00_JPRB, 0.97412E+00_JPRB, 0.99304E+00_JPRB, 0.10123E+01_JPRB ,&
&  0.10288E+01_JPRB, 0.10178E+01_JPRB, 0.10622E+01_JPRB, 0.95940E+00_JPRB, 0.96086E+00_JPRB, 0.97428E+00_JPRB, 0.98725E+00_JPRB ,&
&  0.97022E+00_JPRB, 0.91121E+00_JPRB, 0.92089E+00_JPRB, 0.94017E+00_JPRB, 0.96264E+00_JPRB, 0.91389E+00_JPRB, 0.87804E+00_JPRB ,&
&  0.90866E+00_JPRB, 0.87634E+00_JPRB, 0.68343E+00_JPRB, 0.71776E+00_JPRB, 0.80564E+00_JPRB, 0.75027E+00_JPRB, 0.91851E+00_JPRB ,&
&  0.88714E+00_JPRB, 0.84259E+00_JPRB, 0.78930E+00_JPRB, 0.74014E+00_JPRB, 0.74290E+00_JPRB, 0.75247E+00_JPRB, 0.71042E+00_JPRB ,&
&  0.77470E+00_JPRB, 0.66978E+00_JPRB, 0.64047E+00_JPRB, 0.64554E+00_JPRB, 0.63756E+00_JPRB, 0.63255E+00_JPRB, 0.63187E+00_JPRB ,&
&  0.64246E+00_JPRB, 0.64440E+00_JPRB, 0.61390E+00_JPRB, 0.64374E+00_JPRB, 0.58843E+00_JPRB, 0.64254E+00_JPRB, 0.65063E+00_JPRB ,&
&  0.64784E+00_JPRB, 0.62800E+00_JPRB, 0.62246E+00_JPRB, 0.61618E+00_JPRB, 0.61358E+00_JPRB, 0.59789E+00_JPRB, 0.60098E+00_JPRB ,&
&  0.59630E+00_JPRB, 0.59033E+00_JPRB, 0.58107E+00_JPRB, 0.57866E+00_JPRB, 0.56600E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.58840E+00_JPRB, 0.58209E+00_JPRB, 0.56973E+00_JPRB, 0.55479E+00_JPRB, 0.54006E+00_JPRB, 0.51846E+00_JPRB, 0.52337E+00_JPRB ,&
&  0.51642E+00_JPRB, 0.52570E+00_JPRB, 0.54774E+00_JPRB, 0.56632E+00_JPRB, 0.60116E+00_JPRB, 0.65721E+00_JPRB, 0.71355E+00_JPRB ,&
&  0.75882E+00_JPRB, 0.78565E+00_JPRB, 0.85360E+00_JPRB, 0.95815E+00_JPRB, 0.11280E+01_JPRB, 0.14018E+01_JPRB, 0.18232E+01_JPRB ,&
&  0.20018E+01_JPRB, 0.20920E+01_JPRB, 0.20112E+01_JPRB, 0.17584E+01_JPRB, 0.14615E+01_JPRB, 0.12340E+01_JPRB, 0.12984E+01_JPRB ,&
&  0.13596E+01_JPRB, 0.13074E+01_JPRB, 0.11928E+01_JPRB, 0.11340E+01_JPRB, 0.10755E+01_JPRB, 0.10280E+01_JPRB, 0.10029E+01_JPRB ,&
&  0.99924E+00_JPRB, 0.99755E+00_JPRB, 0.98597E+00_JPRB, 0.99488E+00_JPRB, 0.99748E+00_JPRB, 0.10021E+01_JPRB, 0.10098E+01_JPRB ,&
&  0.10001E+01_JPRB, 0.98016E+00_JPRB, 0.96917E+00_JPRB, 0.92172E+00_JPRB, 0.91007E+00_JPRB, 0.90221E+00_JPRB, 0.88169E+00_JPRB ,&
&  0.85968E+00_JPRB, 0.81218E+00_JPRB, 0.82406E+00_JPRB, 0.84711E+00_JPRB, 0.83552E+00_JPRB, 0.78782E+00_JPRB, 0.79036E+00_JPRB ,&
&  0.82123E+00_JPRB, 0.86078E+00_JPRB, 0.70479E+00_JPRB, 0.82172E+00_JPRB, 0.95866E+00_JPRB, 0.92509E+00_JPRB, 0.93220E+00_JPRB ,&
&  0.95309E+00_JPRB, 0.91269E+00_JPRB, 0.89036E+00_JPRB, 0.83029E+00_JPRB, 0.84580E+00_JPRB, 0.83875E+00_JPRB, 0.75820E+00_JPRB ,&
&  0.87587E+00_JPRB, 0.76324E+00_JPRB, 0.74031E+00_JPRB, 0.71881E+00_JPRB, 0.68811E+00_JPRB, 0.65344E+00_JPRB, 0.63214E+00_JPRB ,&
&  0.61476E+00_JPRB, 0.79207E+00_JPRB, 0.89744E+00_JPRB, 0.10812E+01_JPRB, 0.10852E+01_JPRB, 0.65554E+00_JPRB, 0.67612E+00_JPRB ,&
&  0.67012E+00_JPRB, 0.66485E+00_JPRB, 0.66358E+00_JPRB, 0.65713E+00_JPRB, 0.64863E+00_JPRB, 0.64331E+00_JPRB, 0.64361E+00_JPRB ,&
&  0.63479E+00_JPRB, 0.62648E+00_JPRB, 0.61732E+00_JPRB, 0.61070E+00_JPRB, 0.60055E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.67537E+00_JPRB, 0.67839E+00_JPRB, 0.66811E+00_JPRB, 0.64260E+00_JPRB, 0.60836E+00_JPRB, 0.57271E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.53125E+00_JPRB, 0.55113E+00_JPRB, 0.59368E+00_JPRB, 0.65133E+00_JPRB, 0.72297E+00_JPRB, 0.80100E+00_JPRB, 0.87974E+00_JPRB ,&
&  0.93425E+00_JPRB, 0.99322E+00_JPRB, 0.10840E+01_JPRB, 0.12816E+01_JPRB, 0.14944E+01_JPRB, 0.16445E+01_JPRB, 0.17190E+01_JPRB ,&
&  0.17661E+01_JPRB, 0.17942E+01_JPRB, 0.17082E+01_JPRB, 0.13095E+01_JPRB, 0.10818E+01_JPRB, 0.11968E+01_JPRB, 0.10985E+01_JPRB ,&
&  0.11136E+01_JPRB, 0.11090E+01_JPRB, 0.11151E+01_JPRB, 0.11097E+01_JPRB, 0.10609E+01_JPRB, 0.10183E+01_JPRB, 0.99844E+00_JPRB ,&
&  0.98757E+00_JPRB, 0.99371E+00_JPRB, 0.99795E+00_JPRB, 0.98535E+00_JPRB, 0.98962E+00_JPRB, 0.10033E+01_JPRB, 0.99490E+00_JPRB ,&
&  0.95987E+00_JPRB, 0.94278E+00_JPRB, 0.92517E+00_JPRB, 0.92354E+00_JPRB, 0.90098E+00_JPRB, 0.88062E+00_JPRB, 0.84885E+00_JPRB ,&
&  0.82094E+00_JPRB, 0.80004E+00_JPRB, 0.80294E+00_JPRB, 0.82081E+00_JPRB, 0.82152E+00_JPRB, 0.76046E+00_JPRB, 0.76995E+00_JPRB ,&
&  0.80440E+00_JPRB, 0.82280E+00_JPRB, 0.77650E+00_JPRB, 0.88675E+00_JPRB, 0.68049E+00_JPRB, 0.80007E+00_JPRB, 0.97284E+00_JPRB ,&
&  0.10007E+01_JPRB, 0.92846E+00_JPRB, 0.87782E+00_JPRB, 0.85164E+00_JPRB, 0.81148E+00_JPRB, 0.77484E+00_JPRB, 0.84942E+00_JPRB ,&
&  0.88775E+00_JPRB, 0.84263E+00_JPRB, 0.80286E+00_JPRB, 0.76220E+00_JPRB, 0.74789E+00_JPRB, 0.71706E+00_JPRB, 0.69757E+00_JPRB ,&
&  0.67029E+00_JPRB, 0.79342E+00_JPRB, 0.82473E+00_JPRB, 0.85107E+00_JPRB, 0.85575E+00_JPRB, 0.64518E+00_JPRB, 0.67009E+00_JPRB ,&
&  0.67239E+00_JPRB, 0.66017E+00_JPRB, 0.66077E+00_JPRB, 0.65774E+00_JPRB, 0.64260E+00_JPRB, 0.64050E+00_JPRB, 0.64920E+00_JPRB ,&
&  0.64837E+00_JPRB, 0.65059E+00_JPRB, 0.65540E+00_JPRB, 0.65795E+00_JPRB, 0.66620E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.67151E+00_JPRB, 0.68898E+00_JPRB, 0.71275E+00_JPRB, 0.72673E+00_JPRB, 0.72943E+00_JPRB, 0.71372E+00_JPRB, 0.66618E+00_JPRB ,&
&  0.61709E+00_JPRB, 0.59503E+00_JPRB, 0.62722E+00_JPRB, 0.66792E+00_JPRB, 0.71754E+00_JPRB, 0.77908E+00_JPRB, 0.86673E+00_JPRB ,&
&  0.97778E+00_JPRB, 0.10790E+01_JPRB, 0.11831E+01_JPRB, 0.13475E+01_JPRB, 0.15225E+01_JPRB, 0.15358E+01_JPRB, 0.15443E+01_JPRB ,&
&  0.13635E+01_JPRB, 0.11458E+01_JPRB, 0.16268E+01_JPRB, 0.18232E+01_JPRB, 0.16020E+01_JPRB, 0.12458E+01_JPRB, 0.10559E+01_JPRB ,&
&  0.10450E+01_JPRB, 0.10952E+01_JPRB, 0.11151E+01_JPRB, 0.11148E+01_JPRB, 0.10251E+01_JPRB, 0.10082E+01_JPRB, 0.98727E+00_JPRB ,&
&  0.97494E+00_JPRB, 0.94923E+00_JPRB, 0.95190E+00_JPRB, 0.95525E+00_JPRB, 0.94292E+00_JPRB, 0.95153E+00_JPRB, 0.95112E+00_JPRB ,&
&  0.93166E+00_JPRB, 0.91031E+00_JPRB, 0.90977E+00_JPRB, 0.90797E+00_JPRB, 0.91126E+00_JPRB, 0.89595E+00_JPRB, 0.86036E+00_JPRB ,&
&  0.83119E+00_JPRB, 0.80201E+00_JPRB, 0.81727E+00_JPRB, 0.84301E+00_JPRB, 0.85916E+00_JPRB, 0.80454E+00_JPRB, 0.82582E+00_JPRB ,&
&  0.82705E+00_JPRB, 0.81835E+00_JPRB, 0.81597E+00_JPRB, 0.88482E+00_JPRB, 0.73638E+00_JPRB, 0.81278E+00_JPRB, 0.84948E+00_JPRB ,&
&  0.95837E+00_JPRB, 0.93053E+00_JPRB, 0.87371E+00_JPRB, 0.83553E+00_JPRB, 0.81767E+00_JPRB, 0.73223E+00_JPRB, 0.75574E+00_JPRB ,&
&  0.78279E+00_JPRB, 0.78984E+00_JPRB, 0.76220E+00_JPRB, 0.75638E+00_JPRB, 0.71301E+00_JPRB, 0.70555E+00_JPRB, 0.72961E+00_JPRB ,&
&  0.81578E+00_JPRB, 0.81856E+00_JPRB, 0.78264E+00_JPRB, 0.72312E+00_JPRB, 0.70181E+00_JPRB, 0.62003E+00_JPRB, 0.65184E+00_JPRB ,&
&  0.65740E+00_JPRB, 0.65058E+00_JPRB, 0.63803E+00_JPRB, 0.63893E+00_JPRB, 0.64007E+00_JPRB, 0.63517E+00_JPRB, 0.64123E+00_JPRB ,&
&  0.64459E+00_JPRB, 0.64829E+00_JPRB, 0.65081E+00_JPRB, 0.65460E+00_JPRB, 0.66005E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.64833E+00_JPRB, 0.67782E+00_JPRB, 0.72348E+00_JPRB, 0.79491E+00_JPRB, 0.86740E+00_JPRB, 0.96778E+00_JPRB, 0.10506E+01_JPRB ,&
&  0.92792E+00_JPRB, 0.59809E+00_JPRB, 0.61759E+00_JPRB, 0.63708E+00_JPRB, 0.66977E+00_JPRB, 0.71725E+00_JPRB, 0.78141E+00_JPRB ,&
&  0.88589E+00_JPRB, 0.10278E+01_JPRB, 0.11956E+01_JPRB, 0.13358E+01_JPRB, 0.14564E+01_JPRB, 0.14718E+01_JPRB, 0.13615E+01_JPRB ,&
&  0.10776E+01_JPRB, 0.12869E+01_JPRB, 0.21055E+01_JPRB, 0.19264E+01_JPRB, 0.17239E+01_JPRB, 0.14248E+01_JPRB, 0.11252E+01_JPRB ,&
&  0.10287E+01_JPRB, 0.10232E+01_JPRB, 0.10216E+01_JPRB, 0.99914E+00_JPRB, 0.95278E+00_JPRB, 0.94567E+00_JPRB, 0.94552E+00_JPRB ,&
&  0.93460E+00_JPRB, 0.90978E+00_JPRB, 0.88404E+00_JPRB, 0.88870E+00_JPRB, 0.87787E+00_JPRB, 0.87119E+00_JPRB, 0.87424E+00_JPRB ,&
&  0.88719E+00_JPRB, 0.88710E+00_JPRB, 0.88464E+00_JPRB, 0.87389E+00_JPRB, 0.89687E+00_JPRB, 0.90832E+00_JPRB, 0.87684E+00_JPRB ,&
&  0.84044E+00_JPRB, 0.83729E+00_JPRB, 0.84429E+00_JPRB, 0.82643E+00_JPRB, 0.78769E+00_JPRB, 0.83737E+00_JPRB, 0.82138E+00_JPRB ,&
&  0.84404E+00_JPRB, 0.82015E+00_JPRB, 0.86035E+00_JPRB, 0.76897E+00_JPRB, 0.80033E+00_JPRB, 0.87412E+00_JPRB, 0.89428E+00_JPRB ,&
&  0.90345E+00_JPRB, 0.93689E+00_JPRB, 0.88928E+00_JPRB, 0.87859E+00_JPRB, 0.85075E+00_JPRB, 0.75722E+00_JPRB, 0.76688E+00_JPRB ,&
&  0.73388E+00_JPRB, 0.77536E+00_JPRB, 0.74680E+00_JPRB, 0.71046E+00_JPRB, 0.63091E+00_JPRB, 0.60315E+00_JPRB, 0.62060E+00_JPRB ,&
&  0.71153E+00_JPRB, 0.81009E+00_JPRB, 0.81391E+00_JPRB, 0.77180E+00_JPRB, 0.69819E+00_JPRB, 0.64037E+00_JPRB, 0.65341E+00_JPRB ,&
&  0.64585E+00_JPRB, 0.63613E+00_JPRB, 0.63138E+00_JPRB, 0.63540E+00_JPRB, 0.63827E+00_JPRB, 0.63427E+00_JPRB, 0.62852E+00_JPRB ,&
&  0.62611E+00_JPRB, 0.62442E+00_JPRB, 0.62215E+00_JPRB, 0.62534E+00_JPRB, 0.63415E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.59747E+00_JPRB, 0.61364E+00_JPRB, 0.64298E+00_JPRB, 0.67823E+00_JPRB, 0.71361E+00_JPRB, 0.75512E+00_JPRB, 0.74453E+00_JPRB ,&
&  0.65270E+00_JPRB, 0.59649E+00_JPRB, 0.61162E+00_JPRB, 0.61129E+00_JPRB, 0.61747E+00_JPRB, 0.64499E+00_JPRB, 0.69205E+00_JPRB ,&
&  0.75587E+00_JPRB, 0.87976E+00_JPRB, 0.10522E+01_JPRB, 0.12802E+01_JPRB, 0.13651E+01_JPRB, 0.13892E+01_JPRB, 0.12952E+01_JPRB ,&
&  0.10643E+01_JPRB, 0.21700E+01_JPRB, 0.19655E+01_JPRB, 0.20122E+01_JPRB, 0.18573E+01_JPRB, 0.17549E+01_JPRB, 0.13551E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.98990E+00_JPRB, 0.98699E+00_JPRB, 0.92230E+00_JPRB, 0.88348E+00_JPRB, 0.87967E+00_JPRB, 0.88674E+00_JPRB ,&
&  0.87486E+00_JPRB, 0.86166E+00_JPRB, 0.83893E+00_JPRB, 0.83737E+00_JPRB, 0.83421E+00_JPRB, 0.83260E+00_JPRB, 0.83794E+00_JPRB ,&
&  0.85196E+00_JPRB, 0.85946E+00_JPRB, 0.84950E+00_JPRB, 0.85389E+00_JPRB, 0.86900E+00_JPRB, 0.90178E+00_JPRB, 0.90238E+00_JPRB ,&
&  0.85342E+00_JPRB, 0.81282E+00_JPRB, 0.84766E+00_JPRB, 0.85000E+00_JPRB, 0.84302E+00_JPRB, 0.87883E+00_JPRB, 0.82523E+00_JPRB ,&
&  0.86446E+00_JPRB, 0.81949E+00_JPRB, 0.85059E+00_JPRB, 0.75223E+00_JPRB, 0.78751E+00_JPRB, 0.89440E+00_JPRB, 0.92700E+00_JPRB ,&
&  0.90186E+00_JPRB, 0.95586E+00_JPRB, 0.91581E+00_JPRB, 0.90564E+00_JPRB, 0.87939E+00_JPRB, 0.81213E+00_JPRB, 0.75734E+00_JPRB ,&
&  0.75752E+00_JPRB, 0.78197E+00_JPRB, 0.76612E+00_JPRB, 0.66995E+00_JPRB, 0.62428E+00_JPRB, 0.57932E+00_JPRB, 0.59003E+00_JPRB ,&
&  0.64954E+00_JPRB, 0.72946E+00_JPRB, 0.79823E+00_JPRB, 0.83112E+00_JPRB, 0.78937E+00_JPRB, 0.74318E+00_JPRB, 0.70791E+00_JPRB ,&
&  0.67437E+00_JPRB, 0.64429E+00_JPRB, 0.63235E+00_JPRB, 0.62907E+00_JPRB, 0.62979E+00_JPRB, 0.63098E+00_JPRB, 0.62296E+00_JPRB ,&
&  0.61732E+00_JPRB, 0.61182E+00_JPRB, 0.60695E+00_JPRB, 0.60187E+00_JPRB, 0.59882E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.59128E+00_JPRB, 0.58217E+00_JPRB, 0.59711E+00_JPRB, 0.62958E+00_JPRB, 0.64384E+00_JPRB, 0.65439E+00_JPRB, 0.63277E+00_JPRB ,&
&  0.56823E+00_JPRB, 0.55191E+00_JPRB, 0.55271E+00_JPRB, 0.55693E+00_JPRB, 0.56681E+00_JPRB, 0.59757E+00_JPRB, 0.64146E+00_JPRB ,&
&  0.69642E+00_JPRB, 0.77751E+00_JPRB, 0.86429E+00_JPRB, 0.10323E+01_JPRB, 0.11747E+01_JPRB, 0.13322E+01_JPRB, 0.93954E+00_JPRB ,&
&  0.11536E+01_JPRB, 0.20317E+01_JPRB, 0.20868E+01_JPRB, 0.21189E+01_JPRB, 0.20913E+01_JPRB, 0.20282E+01_JPRB, 0.16149E+01_JPRB ,&
&  0.13070E+01_JPRB, 0.11008E+01_JPRB, 0.99044E+00_JPRB, 0.89600E+00_JPRB, 0.82305E+00_JPRB, 0.83583E+00_JPRB, 0.84398E+00_JPRB ,&
&  0.85015E+00_JPRB, 0.86937E+00_JPRB, 0.81990E+00_JPRB, 0.80635E+00_JPRB, 0.79534E+00_JPRB, 0.78178E+00_JPRB, 0.80126E+00_JPRB ,&
&  0.82232E+00_JPRB, 0.83035E+00_JPRB, 0.83815E+00_JPRB, 0.81308E+00_JPRB, 0.85431E+00_JPRB, 0.88517E+00_JPRB, 0.90318E+00_JPRB ,&
&  0.84182E+00_JPRB, 0.79880E+00_JPRB, 0.86008E+00_JPRB, 0.93472E+00_JPRB, 0.97766E+00_JPRB, 0.96811E+00_JPRB, 0.90443E+00_JPRB ,&
&  0.88704E+00_JPRB, 0.83598E+00_JPRB, 0.79652E+00_JPRB, 0.72969E+00_JPRB, 0.76831E+00_JPRB, 0.88112E+00_JPRB, 0.97896E+00_JPRB ,&
&  0.99526E+00_JPRB, 0.96357E+00_JPRB, 0.94497E+00_JPRB, 0.90269E+00_JPRB, 0.88584E+00_JPRB, 0.84368E+00_JPRB, 0.85235E+00_JPRB ,&
&  0.91982E+00_JPRB, 0.98216E+00_JPRB, 0.83506E+00_JPRB, 0.70487E+00_JPRB, 0.66284E+00_JPRB, 0.57325E+00_JPRB, 0.55858E+00_JPRB ,&
&  0.59671E+00_JPRB, 0.62232E+00_JPRB, 0.65388E+00_JPRB, 0.76827E+00_JPRB, 0.83614E+00_JPRB, 0.86462E+00_JPRB, 0.80860E+00_JPRB ,&
&  0.70800E+00_JPRB, 0.68963E+00_JPRB, 0.66446E+00_JPRB, 0.64414E+00_JPRB, 0.64140E+00_JPRB, 0.63531E+00_JPRB, 0.63084E+00_JPRB ,&
&  0.62018E+00_JPRB, 0.61371E+00_JPRB, 0.61056E+00_JPRB, 0.60990E+00_JPRB, 0.60567E+00_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.60892E+00_JPRB, 0.59688E+00_JPRB, 0.58402E+00_JPRB, 0.58757E+00_JPRB, 0.61027E+00_JPRB, 0.60441E+00_JPRB, 0.58178E+00_JPRB ,&
&  0.53596E+00_JPRB, 0.51268E+00_JPRB, 0.49000E+00_JPRB, 0.48287E+00_JPRB, 0.49317E+00_JPRB, 0.52228E+00_JPRB, 0.57101E+00_JPRB ,&
&  0.63923E+00_JPRB, 0.70232E+00_JPRB, 0.76852E+00_JPRB, 0.82652E+00_JPRB, 0.91943E+00_JPRB, 0.87929E+00_JPRB, 0.97213E+00_JPRB ,&
&  0.14394E+01_JPRB, 0.18823E+01_JPRB, 0.20392E+01_JPRB, 0.21085E+01_JPRB, 0.22076E+01_JPRB, 0.23377E+01_JPRB, 0.19894E+01_JPRB ,&
&  0.16553E+01_JPRB, 0.13845E+01_JPRB, 0.11245E+01_JPRB, 0.96950E+00_JPRB, 0.85972E+00_JPRB, 0.80167E+00_JPRB, 0.80390E+00_JPRB ,&
&  0.80769E+00_JPRB, 0.81888E+00_JPRB, 0.79386E+00_JPRB, 0.77033E+00_JPRB, 0.75646E+00_JPRB, 0.74129E+00_JPRB, 0.74903E+00_JPRB ,&
&  0.76951E+00_JPRB, 0.78510E+00_JPRB, 0.79184E+00_JPRB, 0.78693E+00_JPRB, 0.81772E+00_JPRB, 0.85595E+00_JPRB, 0.90136E+00_JPRB ,&
&  0.88029E+00_JPRB, 0.88868E+00_JPRB, 0.92338E+00_JPRB, 0.95613E+00_JPRB, 0.10421E+01_JPRB, 0.10858E+01_JPRB, 0.10535E+01_JPRB ,&
&  0.96625E+00_JPRB, 0.84150E+00_JPRB, 0.72409E+00_JPRB, 0.71722E+00_JPRB, 0.78478E+00_JPRB, 0.86706E+00_JPRB, 0.86986E+00_JPRB ,&
&  0.82898E+00_JPRB, 0.83486E+00_JPRB, 0.85883E+00_JPRB, 0.93418E+00_JPRB, 0.99179E+00_JPRB, 0.98928E+00_JPRB, 0.10884E+01_JPRB ,&
&  0.96597E+00_JPRB, 0.61179E+00_JPRB, 0.53871E+00_JPRB, 0.60000E+00_JPRB, 0.51336E+00_JPRB, 0.49472E+00_JPRB, 0.56513E+00_JPRB ,&
&  0.58220E+00_JPRB, 0.61814E+00_JPRB, 0.64024E+00_JPRB, 0.63464E+00_JPRB, 0.73965E+00_JPRB, 0.85420E+00_JPRB, 0.87827E+00_JPRB ,&
&  0.76831E+00_JPRB, 0.74516E+00_JPRB, 0.71998E+00_JPRB, 0.70173E+00_JPRB, 0.68830E+00_JPRB, 0.66574E+00_JPRB, 0.65235E+00_JPRB ,&
&  0.63799E+00_JPRB, 0.62585E+00_JPRB, 0.61850E+00_JPRB, 0.61337E+00_JPRB, 0.61453E+00_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.62993E+00_JPRB, 0.62528E+00_JPRB, 0.61672E+00_JPRB, 0.60291E+00_JPRB, 0.60173E+00_JPRB, 0.59867E+00_JPRB, 0.57409E+00_JPRB ,&
&  0.52923E+00_JPRB, 0.49929E+00_JPRB, 0.47105E+00_JPRB, 0.45330E+00_JPRB, 0.45800E+00_JPRB, 0.47186E+00_JPRB, 0.50886E+00_JPRB ,&
&  0.57242E+00_JPRB, 0.67119E+00_JPRB, 0.78439E+00_JPRB, 0.80884E+00_JPRB, 0.80759E+00_JPRB, 0.76227E+00_JPRB, 0.96554E+00_JPRB ,&
&  0.12837E+01_JPRB, 0.15642E+01_JPRB, 0.20381E+01_JPRB, 0.22662E+01_JPRB, 0.23261E+01_JPRB, 0.23350E+01_JPRB, 0.22039E+01_JPRB ,&
&  0.19684E+01_JPRB, 0.16886E+01_JPRB, 0.13368E+01_JPRB, 0.10905E+01_JPRB, 0.93475E+00_JPRB, 0.83294E+00_JPRB, 0.77621E+00_JPRB ,&
&  0.78368E+00_JPRB, 0.79472E+00_JPRB, 0.77704E+00_JPRB, 0.74242E+00_JPRB, 0.72208E+00_JPRB, 0.71748E+00_JPRB, 0.72388E+00_JPRB ,&
&  0.73045E+00_JPRB, 0.74072E+00_JPRB, 0.74672E+00_JPRB, 0.77332E+00_JPRB, 0.68423E+00_JPRB, 0.81149E+00_JPRB, 0.87577E+00_JPRB ,&
&  0.93296E+00_JPRB, 0.95717E+00_JPRB, 0.93164E+00_JPRB, 0.10226E+01_JPRB, 0.11624E+01_JPRB, 0.11942E+01_JPRB, 0.11808E+01_JPRB ,&
&  0.10948E+01_JPRB, 0.91375E+00_JPRB, 0.75349E+00_JPRB, 0.75036E+00_JPRB, 0.81295E+00_JPRB, 0.85810E+00_JPRB, 0.69690E+00_JPRB ,&
&  0.66471E+00_JPRB, 0.81944E+00_JPRB, 0.85462E+00_JPRB, 0.83101E+00_JPRB, 0.10259E+01_JPRB, 0.12291E+01_JPRB, 0.90152E+00_JPRB ,&
&  0.45581E+00_JPRB, 0.40172E+00_JPRB, 0.41720E+00_JPRB, 0.40187E+00_JPRB, 0.40491E+00_JPRB, 0.43284E+00_JPRB, 0.62842E+00_JPRB ,&
&  0.56279E+00_JPRB, 0.61817E+00_JPRB, 0.62246E+00_JPRB, 0.64181E+00_JPRB, 0.71397E+00_JPRB, 0.85197E+00_JPRB, 0.10240E+01_JPRB ,&
&  0.78950E+00_JPRB, 0.76949E+00_JPRB, 0.70698E+00_JPRB, 0.68357E+00_JPRB, 0.67754E+00_JPRB, 0.67823E+00_JPRB, 0.66627E+00_JPRB ,&
&  0.65945E+00_JPRB, 0.65584E+00_JPRB, 0.64967E+00_JPRB, 0.64295E+00_JPRB, 0.63728E+00_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.63329E+00_JPRB, 0.63503E+00_JPRB, 0.63750E+00_JPRB, 0.63358E+00_JPRB, 0.62412E+00_JPRB, 0.60973E+00_JPRB, 0.58847E+00_JPRB ,&
&  0.55095E+00_JPRB, 0.51061E+00_JPRB, 0.47870E+00_JPRB, 0.45987E+00_JPRB, 0.45513E+00_JPRB, 0.45471E+00_JPRB, 0.47341E+00_JPRB ,&
&  0.52308E+00_JPRB, 0.64562E+00_JPRB, 0.86711E+00_JPRB, 0.81455E+00_JPRB, 0.83572E+00_JPRB, 0.78363E+00_JPRB, 0.87203E+00_JPRB ,&
&  0.11447E+01_JPRB, 0.13790E+01_JPRB, 0.17425E+01_JPRB, 0.21012E+01_JPRB, 0.25009E+01_JPRB, 0.25402E+01_JPRB, 0.24857E+01_JPRB ,&
&  0.22587E+01_JPRB, 0.19880E+01_JPRB, 0.16015E+01_JPRB, 0.12686E+01_JPRB, 0.10203E+01_JPRB, 0.87721E+00_JPRB, 0.79437E+00_JPRB ,&
&  0.77852E+00_JPRB, 0.78827E+00_JPRB, 0.77169E+00_JPRB, 0.72431E+00_JPRB, 0.69828E+00_JPRB, 0.69635E+00_JPRB, 0.71439E+00_JPRB ,&
&  0.71772E+00_JPRB, 0.71922E+00_JPRB, 0.71061E+00_JPRB, 0.73274E+00_JPRB, 0.66630E+00_JPRB, 0.63433E+00_JPRB, 0.80264E+00_JPRB ,&
&  0.94883E+00_JPRB, 0.10845E+01_JPRB, 0.10800E+01_JPRB, 0.12165E+01_JPRB, 0.13590E+01_JPRB, 0.13833E+01_JPRB, 0.14191E+01_JPRB ,&
&  0.12753E+01_JPRB, 0.11853E+01_JPRB, 0.87282E+00_JPRB, 0.83621E+00_JPRB, 0.88173E+00_JPRB, 0.75380E+00_JPRB, 0.70243E+00_JPRB ,&
&  0.76048E+00_JPRB, 0.74346E+00_JPRB, 0.74635E+00_JPRB, 0.67595E+00_JPRB, 0.90044E+00_JPRB, 0.97738E+00_JPRB, 0.49587E+00_JPRB ,&
&  0.42738E+00_JPRB, 0.40882E+00_JPRB, 0.39007E+00_JPRB, 0.37651E+00_JPRB, 0.38714E+00_JPRB, 0.40918E+00_JPRB, 0.49785E+00_JPRB ,&
&  0.53644E+00_JPRB, 0.60754E+00_JPRB, 0.61450E+00_JPRB, 0.63489E+00_JPRB, 0.67379E+00_JPRB, 0.79891E+00_JPRB, 0.10480E+01_JPRB ,&
&  0.93273E+00_JPRB, 0.88952E+00_JPRB, 0.77451E+00_JPRB, 0.70864E+00_JPRB, 0.68131E+00_JPRB, 0.66706E+00_JPRB, 0.65331E+00_JPRB ,&
&  0.64356E+00_JPRB, 0.64217E+00_JPRB, 0.63973E+00_JPRB, 0.63493E+00_JPRB, 0.63321E+00_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.62829E+00_JPRB, 0.63308E+00_JPRB, 0.63805E+00_JPRB, 0.64107E+00_JPRB, 0.63900E+00_JPRB, 0.62721E+00_JPRB, 0.60559E+00_JPRB ,&
&  0.57672E+00_JPRB, 0.54387E+00_JPRB, 0.51057E+00_JPRB, 0.48804E+00_JPRB, 0.47087E+00_JPRB, 0.45988E+00_JPRB, 0.46151E+00_JPRB ,&
&  0.49027E+00_JPRB, 0.63029E+00_JPRB, 0.76409E+00_JPRB, 0.70231E+00_JPRB, 0.68344E+00_JPRB, 0.68416E+00_JPRB, 0.70917E+00_JPRB ,&
&  0.10011E+01_JPRB, 0.12143E+01_JPRB, 0.14859E+01_JPRB, 0.19863E+01_JPRB, 0.24617E+01_JPRB, 0.24778E+01_JPRB, 0.26698E+01_JPRB ,&
&  0.24761E+01_JPRB, 0.21790E+01_JPRB, 0.18299E+01_JPRB, 0.14628E+01_JPRB, 0.11733E+01_JPRB, 0.96117E+00_JPRB, 0.85825E+00_JPRB ,&
&  0.80641E+00_JPRB, 0.79913E+00_JPRB, 0.77876E+00_JPRB, 0.73237E+00_JPRB, 0.71061E+00_JPRB, 0.69707E+00_JPRB, 0.69133E+00_JPRB ,&
&  0.68500E+00_JPRB, 0.69839E+00_JPRB, 0.70457E+00_JPRB, 0.73862E+00_JPRB, 0.80142E+00_JPRB, 0.80258E+00_JPRB, 0.83633E+00_JPRB ,&
&  0.91036E+00_JPRB, 0.10168E+01_JPRB, 0.12166E+01_JPRB, 0.13669E+01_JPRB, 0.15783E+01_JPRB, 0.17338E+01_JPRB, 0.16503E+01_JPRB ,&
&  0.14811E+01_JPRB, 0.13091E+01_JPRB, 0.10179E+01_JPRB, 0.99375E+00_JPRB, 0.85770E+00_JPRB, 0.69614E+00_JPRB, 0.74740E+00_JPRB ,&
&  0.77332E+00_JPRB, 0.74534E+00_JPRB, 0.66574E+00_JPRB, 0.56841E+00_JPRB, 0.60336E+00_JPRB, 0.52840E+00_JPRB, 0.42298E+00_JPRB ,&
&  0.40534E+00_JPRB, 0.39019E+00_JPRB, 0.37684E+00_JPRB, 0.38471E+00_JPRB, 0.38124E+00_JPRB, 0.40190E+00_JPRB, 0.45955E+00_JPRB ,&
&  0.53424E+00_JPRB, 0.59501E+00_JPRB, 0.60974E+00_JPRB, 0.63551E+00_JPRB, 0.67132E+00_JPRB, 0.71977E+00_JPRB, 0.92926E+00_JPRB ,&
&  0.90590E+00_JPRB, 0.93889E+00_JPRB, 0.82276E+00_JPRB, 0.75287E+00_JPRB, 0.71107E+00_JPRB, 0.68228E+00_JPRB, 0.66047E+00_JPRB ,&
&  0.65023E+00_JPRB, 0.64239E+00_JPRB, 0.63527E+00_JPRB, 0.63149E+00_JPRB, 0.62648E+00_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.62785E+00_JPRB, 0.62984E+00_JPRB, 0.63395E+00_JPRB, 0.63615E+00_JPRB, 0.63559E+00_JPRB, 0.62912E+00_JPRB, 0.61804E+00_JPRB ,&
&  0.59353E+00_JPRB, 0.56615E+00_JPRB, 0.53122E+00_JPRB, 0.50973E+00_JPRB, 0.48793E+00_JPRB, 0.47256E+00_JPRB, 0.46502E+00_JPRB ,&
&  0.48082E+00_JPRB, 0.62691E+00_JPRB, 0.56039E+00_JPRB, 0.55497E+00_JPRB, 0.61239E+00_JPRB, 0.66628E+00_JPRB, 0.60661E+00_JPRB ,&
&  0.85053E+00_JPRB, 0.10686E+01_JPRB, 0.13542E+01_JPRB, 0.19409E+01_JPRB, 0.28240E+01_JPRB, 0.27050E+01_JPRB, 0.27627E+01_JPRB ,&
&  0.27209E+01_JPRB, 0.22905E+01_JPRB, 0.19811E+01_JPRB, 0.16421E+01_JPRB, 0.13112E+01_JPRB, 0.10856E+01_JPRB, 0.91581E+00_JPRB ,&
&  0.85152E+00_JPRB, 0.82460E+00_JPRB, 0.79670E+00_JPRB, 0.76187E+00_JPRB, 0.74021E+00_JPRB, 0.72333E+00_JPRB, 0.69842E+00_JPRB ,&
&  0.68374E+00_JPRB, 0.67937E+00_JPRB, 0.67961E+00_JPRB, 0.73936E+00_JPRB, 0.75769E+00_JPRB, 0.76344E+00_JPRB, 0.10177E+01_JPRB ,&
&  0.11415E+01_JPRB, 0.12563E+01_JPRB, 0.12643E+01_JPRB, 0.15624E+01_JPRB, 0.18860E+01_JPRB, 0.19146E+01_JPRB, 0.19251E+01_JPRB ,&
&  0.14665E+01_JPRB, 0.11485E+01_JPRB, 0.10272E+01_JPRB, 0.97637E+00_JPRB, 0.78495E+00_JPRB, 0.81033E+00_JPRB, 0.10825E+01_JPRB ,&
&  0.84900E+00_JPRB, 0.86669E+00_JPRB, 0.89839E+00_JPRB, 0.83190E+00_JPRB, 0.54196E+00_JPRB, 0.45514E+00_JPRB, 0.57757E+00_JPRB ,&
&  0.59534E+00_JPRB, 0.44969E+00_JPRB, 0.39907E+00_JPRB, 0.46457E+00_JPRB, 0.40094E+00_JPRB, 0.42191E+00_JPRB, 0.48258E+00_JPRB ,&
&  0.52236E+00_JPRB, 0.55342E+00_JPRB, 0.61047E+00_JPRB, 0.65197E+00_JPRB, 0.68741E+00_JPRB, 0.71632E+00_JPRB, 0.79084E+00_JPRB ,&
&  0.83309E+00_JPRB, 0.88369E+00_JPRB, 0.88569E+00_JPRB, 0.82226E+00_JPRB, 0.77306E+00_JPRB, 0.72300E+00_JPRB, 0.68407E+00_JPRB ,&
&  0.65687E+00_JPRB, 0.63941E+00_JPRB, 0.62936E+00_JPRB, 0.62315E+00_JPRB, 0.62630E+00_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.63969E+00_JPRB, 0.63446E+00_JPRB, 0.63834E+00_JPRB, 0.64729E+00_JPRB, 0.65202E+00_JPRB, 0.64530E+00_JPRB, 0.63284E+00_JPRB ,&
&  0.62071E+00_JPRB, 0.59126E+00_JPRB, 0.55648E+00_JPRB, 0.52842E+00_JPRB, 0.51065E+00_JPRB, 0.49213E+00_JPRB, 0.47630E+00_JPRB ,&
&  0.47421E+00_JPRB, 0.50157E+00_JPRB, 0.47139E+00_JPRB, 0.47215E+00_JPRB, 0.59520E+00_JPRB, 0.59261E+00_JPRB, 0.61940E+00_JPRB ,&
&  0.74230E+00_JPRB, 0.91986E+00_JPRB, 0.12389E+01_JPRB, 0.17217E+01_JPRB, 0.21723E+01_JPRB, 0.25943E+01_JPRB, 0.27955E+01_JPRB ,&
&  0.27994E+01_JPRB, 0.24141E+01_JPRB, 0.20341E+01_JPRB, 0.17183E+01_JPRB, 0.14088E+01_JPRB, 0.11746E+01_JPRB, 0.10082E+01_JPRB ,&
&  0.92055E+00_JPRB, 0.88061E+00_JPRB, 0.84929E+00_JPRB, 0.81230E+00_JPRB, 0.79545E+00_JPRB, 0.76033E+00_JPRB, 0.72696E+00_JPRB ,&
&  0.71261E+00_JPRB, 0.71312E+00_JPRB, 0.69978E+00_JPRB, 0.72695E+00_JPRB, 0.72837E+00_JPRB, 0.76886E+00_JPRB, 0.87856E+00_JPRB ,&
&  0.10421E+01_JPRB, 0.11743E+01_JPRB, 0.12684E+01_JPRB, 0.15796E+01_JPRB, 0.20008E+01_JPRB, 0.19194E+01_JPRB, 0.20470E+01_JPRB ,&
&  0.17500E+01_JPRB, 0.14216E+01_JPRB, 0.11310E+01_JPRB, 0.87642E+00_JPRB, 0.90065E+00_JPRB, 0.12357E+01_JPRB, 0.12661E+01_JPRB ,&
&  0.11307E+01_JPRB, 0.10813E+01_JPRB, 0.10328E+01_JPRB, 0.88123E+00_JPRB, 0.68851E+00_JPRB, 0.55008E+00_JPRB, 0.65007E+00_JPRB ,&
&  0.68752E+00_JPRB, 0.68322E+00_JPRB, 0.64227E+00_JPRB, 0.57773E+00_JPRB, 0.50276E+00_JPRB, 0.55273E+00_JPRB, 0.55139E+00_JPRB ,&
&  0.53986E+00_JPRB, 0.52901E+00_JPRB, 0.57044E+00_JPRB, 0.64583E+00_JPRB, 0.66233E+00_JPRB, 0.66773E+00_JPRB, 0.70223E+00_JPRB ,&
&  0.72818E+00_JPRB, 0.78187E+00_JPRB, 0.79850E+00_JPRB, 0.78929E+00_JPRB, 0.77731E+00_JPRB, 0.75053E+00_JPRB, 0.72584E+00_JPRB ,&
&  0.70807E+00_JPRB, 0.69593E+00_JPRB, 0.67936E+00_JPRB, 0.66332E+00_JPRB, 0.64919E+00_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.63758E+00_JPRB, 0.62940E+00_JPRB, 0.62894E+00_JPRB, 0.63862E+00_JPRB, 0.65724E+00_JPRB, 0.65967E+00_JPRB, 0.65052E+00_JPRB ,&
&  0.63515E+00_JPRB, 0.61731E+00_JPRB, 0.59039E+00_JPRB, 0.55721E+00_JPRB, 0.53431E+00_JPRB, 0.50777E+00_JPRB, 0.48730E+00_JPRB ,&
&  0.47724E+00_JPRB, 0.47772E+00_JPRB, 0.43324E+00_JPRB, 0.47285E+00_JPRB, 0.52172E+00_JPRB, 0.51341E+00_JPRB, 0.57716E+00_JPRB ,&
&  0.63458E+00_JPRB, 0.77190E+00_JPRB, 0.99148E+00_JPRB, 0.12778E+01_JPRB, 0.15930E+01_JPRB, 0.20377E+01_JPRB, 0.23674E+01_JPRB ,&
&  0.23502E+01_JPRB, 0.22820E+01_JPRB, 0.20775E+01_JPRB, 0.18589E+01_JPRB, 0.15664E+01_JPRB, 0.13344E+01_JPRB, 0.11730E+01_JPRB ,&
&  0.10495E+01_JPRB, 0.97640E+00_JPRB, 0.93508E+00_JPRB, 0.89872E+00_JPRB, 0.86805E+00_JPRB, 0.83623E+00_JPRB, 0.81063E+00_JPRB ,&
&  0.76932E+00_JPRB, 0.76325E+00_JPRB, 0.74971E+00_JPRB, 0.76539E+00_JPRB, 0.75211E+00_JPRB, 0.79635E+00_JPRB, 0.87826E+00_JPRB ,&
&  0.11445E+01_JPRB, 0.13306E+01_JPRB, 0.13953E+01_JPRB, 0.18249E+01_JPRB, 0.17036E+01_JPRB, 0.16878E+01_JPRB, 0.19346E+01_JPRB ,&
&  0.21700E+01_JPRB, 0.21624E+01_JPRB, 0.20216E+01_JPRB, 0.15432E+01_JPRB, 0.12282E+01_JPRB, 0.15920E+01_JPRB, 0.15246E+01_JPRB ,&
&  0.13444E+01_JPRB, 0.12304E+01_JPRB, 0.11042E+01_JPRB, 0.10472E+01_JPRB, 0.83763E+00_JPRB, 0.72315E+00_JPRB, 0.61967E+00_JPRB ,&
&  0.64079E+00_JPRB, 0.60254E+00_JPRB, 0.66052E+00_JPRB, 0.61451E+00_JPRB, 0.58612E+00_JPRB, 0.57538E+00_JPRB, 0.55064E+00_JPRB ,&
&  0.53923E+00_JPRB, 0.53061E+00_JPRB, 0.53069E+00_JPRB, 0.57943E+00_JPRB, 0.61528E+00_JPRB, 0.60108E+00_JPRB, 0.62747E+00_JPRB ,&
&  0.65595E+00_JPRB, 0.69701E+00_JPRB, 0.73024E+00_JPRB, 0.76016E+00_JPRB, 0.76778E+00_JPRB, 0.76201E+00_JPRB, 0.74497E+00_JPRB ,&
&  0.72428E+00_JPRB, 0.71118E+00_JPRB, 0.69357E+00_JPRB, 0.67199E+00_JPRB, 0.65200E+00_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.61140E+00_JPRB, 0.60738E+00_JPRB, 0.60860E+00_JPRB, 0.62433E+00_JPRB, 0.65112E+00_JPRB, 0.66546E+00_JPRB, 0.65605E+00_JPRB ,&
&  0.64451E+00_JPRB, 0.62968E+00_JPRB, 0.62034E+00_JPRB, 0.59057E+00_JPRB, 0.56041E+00_JPRB, 0.52492E+00_JPRB, 0.49804E+00_JPRB ,&
&  0.49394E+00_JPRB, 0.54829E+00_JPRB, 0.47568E+00_JPRB, 0.43583E+00_JPRB, 0.41107E+00_JPRB, 0.45960E+00_JPRB, 0.51998E+00_JPRB ,&
&  0.59366E+00_JPRB, 0.65575E+00_JPRB, 0.82090E+00_JPRB, 0.92854E+00_JPRB, 0.10608E+01_JPRB, 0.13735E+01_JPRB, 0.17742E+01_JPRB ,&
&  0.22116E+01_JPRB, 0.18721E+01_JPRB, 0.18876E+01_JPRB, 0.17537E+01_JPRB, 0.16227E+01_JPRB, 0.14580E+01_JPRB, 0.12988E+01_JPRB ,&
&  0.11608E+01_JPRB, 0.10424E+01_JPRB, 0.97588E+00_JPRB, 0.92250E+00_JPRB, 0.87503E+00_JPRB, 0.83209E+00_JPRB, 0.79668E+00_JPRB ,&
&  0.77553E+00_JPRB, 0.79040E+00_JPRB, 0.82159E+00_JPRB, 0.83075E+00_JPRB, 0.88312E+00_JPRB, 0.96748E+00_JPRB, 0.10466E+01_JPRB ,&
&  0.11171E+01_JPRB, 0.11664E+01_JPRB, 0.16128E+01_JPRB, 0.18309E+01_JPRB, 0.19904E+01_JPRB, 0.20680E+01_JPRB, 0.21791E+01_JPRB ,&
&  0.23138E+01_JPRB, 0.23241E+01_JPRB, 0.22454E+01_JPRB, 0.18806E+01_JPRB, 0.19318E+01_JPRB, 0.18857E+01_JPRB, 0.17540E+01_JPRB ,&
&  0.15879E+01_JPRB, 0.14822E+01_JPRB, 0.12992E+01_JPRB, 0.11675E+01_JPRB, 0.10883E+01_JPRB, 0.10463E+01_JPRB, 0.90093E+00_JPRB ,&
&  0.78120E+00_JPRB, 0.80532E+00_JPRB, 0.72782E+00_JPRB, 0.62655E+00_JPRB, 0.54276E+00_JPRB, 0.54036E+00_JPRB, 0.53905E+00_JPRB ,&
&  0.53473E+00_JPRB, 0.52917E+00_JPRB, 0.51530E+00_JPRB, 0.53047E+00_JPRB, 0.56166E+00_JPRB, 0.56507E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.56670E+00_JPRB, 0.61189E+00_JPRB, 0.63661E+00_JPRB, 0.64884E+00_JPRB, 0.68918E+00_JPRB, 0.69456E+00_JPRB, 0.67967E+00_JPRB ,&
&  0.65662E+00_JPRB, 0.63998E+00_JPRB, 0.63387E+00_JPRB, 0.62582E+00_JPRB, 0.61847E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.57164E+00_JPRB, 0.57567E+00_JPRB, 0.58451E+00_JPRB, 0.60561E+00_JPRB, 0.63928E+00_JPRB, 0.66294E+00_JPRB, 0.65838E+00_JPRB ,&
&  0.65235E+00_JPRB, 0.63984E+00_JPRB, 0.62757E+00_JPRB, 0.60974E+00_JPRB, 0.57704E+00_JPRB, 0.54101E+00_JPRB, 0.51702E+00_JPRB ,&
&  0.52061E+00_JPRB, 0.66927E+00_JPRB, 0.55436E+00_JPRB, 0.51650E+00_JPRB, 0.46446E+00_JPRB, 0.49572E+00_JPRB, 0.53127E+00_JPRB ,&
&  0.59764E+00_JPRB, 0.61692E+00_JPRB, 0.66457E+00_JPRB, 0.72363E+00_JPRB, 0.80712E+00_JPRB, 0.10048E+01_JPRB, 0.13241E+01_JPRB ,&
&  0.15192E+01_JPRB, 0.16558E+01_JPRB, 0.15647E+01_JPRB, 0.16634E+01_JPRB, 0.15324E+01_JPRB, 0.13474E+01_JPRB, 0.13430E+01_JPRB ,&
&  0.12273E+01_JPRB, 0.11169E+01_JPRB, 0.10064E+01_JPRB, 0.93477E+00_JPRB, 0.88365E+00_JPRB, 0.83500E+00_JPRB, 0.79440E+00_JPRB ,&
&  0.77444E+00_JPRB, 0.78947E+00_JPRB, 0.83797E+00_JPRB, 0.87356E+00_JPRB, 0.89433E+00_JPRB, 0.10080E+01_JPRB, 0.11417E+01_JPRB ,&
&  0.15198E+01_JPRB, 0.15294E+01_JPRB, 0.14103E+01_JPRB, 0.18863E+01_JPRB, 0.23945E+01_JPRB, 0.24011E+01_JPRB, 0.22533E+01_JPRB ,&
&  0.23235E+01_JPRB, 0.23075E+01_JPRB, 0.22516E+01_JPRB, 0.21648E+01_JPRB, 0.20732E+01_JPRB, 0.19689E+01_JPRB, 0.18503E+01_JPRB ,&
&  0.17380E+01_JPRB, 0.16330E+01_JPRB, 0.14929E+01_JPRB, 0.13007E+01_JPRB, 0.11523E+01_JPRB, 0.10662E+01_JPRB, 0.10259E+01_JPRB ,&
&  0.92337E+00_JPRB, 0.79436E+00_JPRB, 0.60639E+00_JPRB, 0.60152E+00_JPRB, 0.50384E+00_JPRB, 0.49045E+00_JPRB, 0.51680E+00_JPRB ,&
&  0.50624E+00_JPRB, 0.51707E+00_JPRB, 0.51109E+00_JPRB, 0.49350E+00_JPRB, 0.51028E+00_JPRB, 0.52935E+00_JPRB, 0.52327E+00_JPRB ,&
&  0.52749E+00_JPRB, 0.52300E+00_JPRB, 0.56101E+00_JPRB, 0.56426E+00_JPRB, 0.56891E+00_JPRB, 0.57591E+00_JPRB, 0.57617E+00_JPRB ,&
&  0.56359E+00_JPRB, 0.54889E+00_JPRB, 0.54440E+00_JPRB, 0.55212E+00_JPRB, 0.56211E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.51471E+00_JPRB, 0.52818E+00_JPRB, 0.54604E+00_JPRB, 0.57100E+00_JPRB, 0.60796E+00_JPRB, 0.63799E+00_JPRB, 0.64244E+00_JPRB ,&
&  0.63918E+00_JPRB, 0.63325E+00_JPRB, 0.62123E+00_JPRB, 0.60851E+00_JPRB, 0.58485E+00_JPRB, 0.54848E+00_JPRB, 0.52737E+00_JPRB ,&
&  0.58180E+00_JPRB, 0.62527E+00_JPRB, 0.54665E+00_JPRB, 0.47089E+00_JPRB, 0.53147E+00_JPRB, 0.51653E+00_JPRB, 0.53346E+00_JPRB ,&
&  0.52425E+00_JPRB, 0.57418E+00_JPRB, 0.60159E+00_JPRB, 0.60604E+00_JPRB, 0.63560E+00_JPRB, 0.77669E+00_JPRB, 0.95809E+00_JPRB ,&
&  0.11068E+01_JPRB, 0.12018E+01_JPRB, 0.12500E+01_JPRB, 0.12739E+01_JPRB, 0.13269E+01_JPRB, 0.11922E+01_JPRB, 0.11786E+01_JPRB ,&
&  0.11091E+01_JPRB, 0.10330E+01_JPRB, 0.95556E+00_JPRB, 0.89601E+00_JPRB, 0.84392E+00_JPRB, 0.79834E+00_JPRB, 0.77027E+00_JPRB ,&
&  0.78110E+00_JPRB, 0.80195E+00_JPRB, 0.83936E+00_JPRB, 0.88773E+00_JPRB, 0.92270E+00_JPRB, 0.11192E+01_JPRB, 0.15340E+01_JPRB ,&
&  0.20528E+01_JPRB, 0.22443E+01_JPRB, 0.25088E+01_JPRB, 0.27560E+01_JPRB, 0.27163E+01_JPRB, 0.26736E+01_JPRB, 0.26499E+01_JPRB ,&
&  0.25399E+01_JPRB, 0.24039E+01_JPRB, 0.22641E+01_JPRB, 0.21436E+01_JPRB, 0.20693E+01_JPRB, 0.20139E+01_JPRB, 0.19261E+01_JPRB ,&
&  0.17252E+01_JPRB, 0.15883E+01_JPRB, 0.15294E+01_JPRB, 0.13543E+01_JPRB, 0.12181E+01_JPRB, 0.10774E+01_JPRB, 0.10556E+01_JPRB ,&
&  0.96823E+00_JPRB, 0.63143E+00_JPRB, 0.53801E+00_JPRB, 0.58775E+00_JPRB, 0.50959E+00_JPRB, 0.49721E+00_JPRB, 0.52280E+00_JPRB ,&
&  0.51224E+00_JPRB, 0.49898E+00_JPRB, 0.49700E+00_JPRB, 0.47801E+00_JPRB, 0.48079E+00_JPRB, 0.51587E+00_JPRB, 0.48959E+00_JPRB ,&
&  0.46437E+00_JPRB, 0.47308E+00_JPRB, 0.49373E+00_JPRB, 0.48696E+00_JPRB, 0.48105E+00_JPRB, 0.48565E+00_JPRB, 0.48352E+00_JPRB ,&
&  0.47431E+00_JPRB, 0.47169E+00_JPRB, 0.47055E+00_JPRB, 0.47955E+00_JPRB, 0.49859E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.48415E+00_JPRB, 0.50565E+00_JPRB, 0.52205E+00_JPRB, 0.54433E+00_JPRB, 0.57676E+00_JPRB, 0.61328E+00_JPRB, 0.62637E+00_JPRB ,&
&  0.62630E+00_JPRB, 0.62239E+00_JPRB, 0.61377E+00_JPRB, 0.59936E+00_JPRB, 0.58393E+00_JPRB, 0.55503E+00_JPRB, 0.54169E+00_JPRB ,&
&  0.46177E+00_JPRB, 0.50008E+00_JPRB, 0.43670E+00_JPRB, 0.45035E+00_JPRB, 0.54442E+00_JPRB, 0.51070E+00_JPRB, 0.49904E+00_JPRB ,&
&  0.48785E+00_JPRB, 0.50942E+00_JPRB, 0.53773E+00_JPRB, 0.55868E+00_JPRB, 0.55084E+00_JPRB, 0.62333E+00_JPRB, 0.76238E+00_JPRB ,&
&  0.81928E+00_JPRB, 0.82616E+00_JPRB, 0.86274E+00_JPRB, 0.87247E+00_JPRB, 0.90733E+00_JPRB, 0.99026E+00_JPRB, 0.10213E+01_JPRB ,&
&  0.96989E+00_JPRB, 0.91213E+00_JPRB, 0.85716E+00_JPRB, 0.81651E+00_JPRB, 0.78031E+00_JPRB, 0.74003E+00_JPRB, 0.71449E+00_JPRB ,&
&  0.70811E+00_JPRB, 0.71382E+00_JPRB, 0.76482E+00_JPRB, 0.85023E+00_JPRB, 0.91847E+00_JPRB, 0.12602E+01_JPRB, 0.19513E+01_JPRB ,&
&  0.23260E+01_JPRB, 0.24099E+01_JPRB, 0.26502E+01_JPRB, 0.29371E+01_JPRB, 0.28993E+01_JPRB, 0.28274E+01_JPRB, 0.27056E+01_JPRB ,&
&  0.26212E+01_JPRB, 0.25145E+01_JPRB, 0.23370E+01_JPRB, 0.22424E+01_JPRB, 0.20644E+01_JPRB, 0.19800E+01_JPRB, 0.18175E+01_JPRB ,&
&  0.15609E+01_JPRB, 0.14401E+01_JPRB, 0.14263E+01_JPRB, 0.13580E+01_JPRB, 0.12866E+01_JPRB, 0.12378E+01_JPRB, 0.11748E+01_JPRB ,&
&  0.10616E+01_JPRB, 0.88374E+00_JPRB, 0.67915E+00_JPRB, 0.62118E+00_JPRB, 0.51099E+00_JPRB, 0.50608E+00_JPRB, 0.54892E+00_JPRB ,&
&  0.53566E+00_JPRB, 0.49419E+00_JPRB, 0.46825E+00_JPRB, 0.45548E+00_JPRB, 0.45461E+00_JPRB, 0.47859E+00_JPRB, 0.45050E+00_JPRB ,&
&  0.41925E+00_JPRB, 0.42379E+00_JPRB, 0.42797E+00_JPRB, 0.43510E+00_JPRB, 0.44362E+00_JPRB, 0.44350E+00_JPRB, 0.43054E+00_JPRB ,&
&  0.43898E+00_JPRB, 0.43391E+00_JPRB, 0.42985E+00_JPRB, 0.43782E+00_JPRB, 0.45792E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.46557E+00_JPRB, 0.48975E+00_JPRB, 0.50839E+00_JPRB, 0.52385E+00_JPRB, 0.54708E+00_JPRB, 0.58127E+00_JPRB, 0.59720E+00_JPRB ,&
&  0.59969E+00_JPRB, 0.59903E+00_JPRB, 0.59555E+00_JPRB, 0.58554E+00_JPRB, 0.56560E+00_JPRB, 0.53878E+00_JPRB, 0.47887E+00_JPRB ,&
&  0.40910E+00_JPRB, 0.44765E+00_JPRB, 0.43390E+00_JPRB, 0.44686E+00_JPRB, 0.45384E+00_JPRB, 0.44658E+00_JPRB, 0.43807E+00_JPRB ,&
&  0.45212E+00_JPRB, 0.46775E+00_JPRB, 0.50127E+00_JPRB, 0.54316E+00_JPRB, 0.54798E+00_JPRB, 0.57741E+00_JPRB, 0.67154E+00_JPRB ,&
&  0.69287E+00_JPRB, 0.70035E+00_JPRB, 0.67772E+00_JPRB, 0.67001E+00_JPRB, 0.70520E+00_JPRB, 0.76414E+00_JPRB, 0.80936E+00_JPRB ,&
&  0.80539E+00_JPRB, 0.76226E+00_JPRB, 0.72232E+00_JPRB, 0.69877E+00_JPRB, 0.68179E+00_JPRB, 0.66189E+00_JPRB, 0.64133E+00_JPRB ,&
&  0.62069E+00_JPRB, 0.61457E+00_JPRB, 0.63858E+00_JPRB, 0.68634E+00_JPRB, 0.76790E+00_JPRB, 0.10600E+01_JPRB, 0.13303E+01_JPRB ,&
&  0.14506E+01_JPRB, 0.16095E+01_JPRB, 0.18319E+01_JPRB, 0.20338E+01_JPRB, 0.21606E+01_JPRB, 0.22330E+01_JPRB, 0.22671E+01_JPRB ,&
&  0.23168E+01_JPRB, 0.22306E+01_JPRB, 0.22223E+01_JPRB, 0.21242E+01_JPRB, 0.19417E+01_JPRB, 0.17991E+01_JPRB, 0.16160E+01_JPRB ,&
&  0.14435E+01_JPRB, 0.12791E+01_JPRB, 0.13051E+01_JPRB, 0.12927E+01_JPRB, 0.12634E+01_JPRB, 0.12294E+01_JPRB, 0.11662E+01_JPRB ,&
&  0.10852E+01_JPRB, 0.99051E+00_JPRB, 0.82873E+00_JPRB, 0.70948E+00_JPRB, 0.60932E+00_JPRB, 0.59769E+00_JPRB, 0.54992E+00_JPRB ,&
&  0.49369E+00_JPRB, 0.44199E+00_JPRB, 0.43971E+00_JPRB, 0.43621E+00_JPRB, 0.43647E+00_JPRB, 0.42188E+00_JPRB, 0.40964E+00_JPRB ,&
&  0.40924E+00_JPRB, 0.41168E+00_JPRB, 0.40323E+00_JPRB, 0.40336E+00_JPRB, 0.41549E+00_JPRB, 0.41119E+00_JPRB, 0.39659E+00_JPRB ,&
&  0.39668E+00_JPRB, 0.40032E+00_JPRB, 0.39895E+00_JPRB, 0.41158E+00_JPRB, 0.43543E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.44599E+00_JPRB, 0.46841E+00_JPRB, 0.48732E+00_JPRB, 0.50386E+00_JPRB, 0.52380E+00_JPRB, 0.54496E+00_JPRB, 0.54478E+00_JPRB ,&
&  0.56051E+00_JPRB, 0.56623E+00_JPRB, 0.56377E+00_JPRB, 0.55299E+00_JPRB, 0.53082E+00_JPRB, 0.46453E+00_JPRB, 0.38111E+00_JPRB ,&
&  0.36218E+00_JPRB, 0.37949E+00_JPRB, 0.40250E+00_JPRB, 0.41260E+00_JPRB, 0.41746E+00_JPRB, 0.42243E+00_JPRB, 0.42175E+00_JPRB ,&
&  0.42676E+00_JPRB, 0.43841E+00_JPRB, 0.46516E+00_JPRB, 0.50682E+00_JPRB, 0.54435E+00_JPRB, 0.55974E+00_JPRB, 0.59756E+00_JPRB ,&
&  0.59740E+00_JPRB, 0.61332E+00_JPRB, 0.61188E+00_JPRB, 0.56223E+00_JPRB, 0.58342E+00_JPRB, 0.58939E+00_JPRB, 0.61395E+00_JPRB ,&
&  0.63480E+00_JPRB, 0.62481E+00_JPRB, 0.60149E+00_JPRB, 0.58883E+00_JPRB, 0.58269E+00_JPRB, 0.57310E+00_JPRB, 0.56344E+00_JPRB ,&
&  0.54991E+00_JPRB, 0.54294E+00_JPRB, 0.55889E+00_JPRB, 0.58661E+00_JPRB, 0.62931E+00_JPRB, 0.82911E+00_JPRB, 0.94456E+00_JPRB ,&
&  0.10521E+01_JPRB, 0.12525E+01_JPRB, 0.15509E+01_JPRB, 0.16347E+01_JPRB, 0.18736E+01_JPRB, 0.19921E+01_JPRB, 0.21136E+01_JPRB ,&
&  0.21645E+01_JPRB, 0.20113E+01_JPRB, 0.19991E+01_JPRB, 0.18900E+01_JPRB, 0.17704E+01_JPRB, 0.16210E+01_JPRB, 0.14530E+01_JPRB ,&
&  0.13284E+01_JPRB, 0.12588E+01_JPRB, 0.12495E+01_JPRB, 0.12031E+01_JPRB, 0.11282E+01_JPRB, 0.10807E+01_JPRB, 0.10326E+01_JPRB ,&
&  0.98828E+00_JPRB, 0.92658E+00_JPRB, 0.83606E+00_JPRB, 0.74088E+00_JPRB, 0.65500E+00_JPRB, 0.58766E+00_JPRB, 0.53460E+00_JPRB ,&
&  0.49197E+00_JPRB, 0.44852E+00_JPRB, 0.43354E+00_JPRB, 0.43008E+00_JPRB, 0.42476E+00_JPRB, 0.40613E+00_JPRB, 0.39859E+00_JPRB ,&
&  0.39507E+00_JPRB, 0.39164E+00_JPRB, 0.39012E+00_JPRB, 0.38872E+00_JPRB, 0.39551E+00_JPRB, 0.39854E+00_JPRB, 0.38802E+00_JPRB ,&
&  0.38056E+00_JPRB, 0.38556E+00_JPRB, 0.39101E+00_JPRB, 0.40314E+00_JPRB, 0.42289E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.43261E+00_JPRB, 0.45829E+00_JPRB, 0.47203E+00_JPRB, 0.48463E+00_JPRB, 0.49773E+00_JPRB, 0.49447E+00_JPRB, 0.49363E+00_JPRB ,&
&  0.52158E+00_JPRB, 0.48882E+00_JPRB, 0.52657E+00_JPRB, 0.41928E+00_JPRB, 0.37822E+00_JPRB, 0.37038E+00_JPRB, 0.36096E+00_JPRB ,&
&  0.36340E+00_JPRB, 0.37499E+00_JPRB, 0.38646E+00_JPRB, 0.39413E+00_JPRB, 0.40683E+00_JPRB, 0.40922E+00_JPRB, 0.40987E+00_JPRB ,&
&  0.41351E+00_JPRB, 0.42201E+00_JPRB, 0.44502E+00_JPRB, 0.45886E+00_JPRB, 0.47386E+00_JPRB, 0.48786E+00_JPRB, 0.50017E+00_JPRB ,&
&  0.49766E+00_JPRB, 0.51502E+00_JPRB, 0.54573E+00_JPRB, 0.53289E+00_JPRB, 0.51651E+00_JPRB, 0.50269E+00_JPRB, 0.51000E+00_JPRB ,&
&  0.50858E+00_JPRB, 0.46344E+00_JPRB, 0.47638E+00_JPRB, 0.47803E+00_JPRB, 0.48021E+00_JPRB, 0.48251E+00_JPRB, 0.48486E+00_JPRB ,&
&  0.48692E+00_JPRB, 0.48993E+00_JPRB, 0.50622E+00_JPRB, 0.52638E+00_JPRB, 0.56476E+00_JPRB, 0.64315E+00_JPRB, 0.69907E+00_JPRB ,&
&  0.84298E+00_JPRB, 0.87179E+00_JPRB, 0.11926E+01_JPRB, 0.14695E+01_JPRB, 0.18457E+01_JPRB, 0.19486E+01_JPRB, 0.20563E+01_JPRB ,&
&  0.20815E+01_JPRB, 0.19117E+01_JPRB, 0.18340E+01_JPRB, 0.16832E+01_JPRB, 0.15407E+01_JPRB, 0.14114E+01_JPRB, 0.13044E+01_JPRB ,&
&  0.12129E+01_JPRB, 0.10983E+01_JPRB, 0.10932E+01_JPRB, 0.10250E+01_JPRB, 0.94731E+00_JPRB, 0.89742E+00_JPRB, 0.85960E+00_JPRB ,&
&  0.81847E+00_JPRB, 0.76905E+00_JPRB, 0.70276E+00_JPRB, 0.60737E+00_JPRB, 0.56398E+00_JPRB, 0.52862E+00_JPRB, 0.48858E+00_JPRB ,&
&  0.46991E+00_JPRB, 0.45046E+00_JPRB, 0.43696E+00_JPRB, 0.43272E+00_JPRB, 0.42071E+00_JPRB, 0.40477E+00_JPRB, 0.39674E+00_JPRB ,&
&  0.38239E+00_JPRB, 0.37317E+00_JPRB, 0.37144E+00_JPRB, 0.37171E+00_JPRB, 0.37208E+00_JPRB, 0.37461E+00_JPRB, 0.37802E+00_JPRB ,&
&  0.37639E+00_JPRB, 0.37799E+00_JPRB, 0.38362E+00_JPRB, 0.39037E+00_JPRB, 0.40760E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.40688E+00_JPRB, 0.43971E+00_JPRB, 0.45529E+00_JPRB, 0.46392E+00_JPRB, 0.47122E+00_JPRB, 0.46860E+00_JPRB, 0.44290E+00_JPRB ,&
&  0.44288E+00_JPRB, 0.39911E+00_JPRB, 0.37972E+00_JPRB, 0.36534E+00_JPRB, 0.35963E+00_JPRB, 0.35396E+00_JPRB, 0.34572E+00_JPRB ,&
&  0.34787E+00_JPRB, 0.35809E+00_JPRB, 0.37256E+00_JPRB, 0.38229E+00_JPRB, 0.39080E+00_JPRB, 0.39661E+00_JPRB, 0.40111E+00_JPRB ,&
&  0.40721E+00_JPRB, 0.41477E+00_JPRB, 0.42553E+00_JPRB, 0.43419E+00_JPRB, 0.43812E+00_JPRB, 0.44011E+00_JPRB, 0.43930E+00_JPRB ,&
&  0.44663E+00_JPRB, 0.46167E+00_JPRB, 0.48496E+00_JPRB, 0.49667E+00_JPRB, 0.49351E+00_JPRB, 0.47758E+00_JPRB, 0.44927E+00_JPRB ,&
&  0.38360E+00_JPRB, 0.37992E+00_JPRB, 0.40218E+00_JPRB, 0.41202E+00_JPRB, 0.42204E+00_JPRB, 0.43144E+00_JPRB, 0.43898E+00_JPRB ,&
&  0.44529E+00_JPRB, 0.45434E+00_JPRB, 0.47364E+00_JPRB, 0.49581E+00_JPRB, 0.53171E+00_JPRB, 0.58763E+00_JPRB, 0.62867E+00_JPRB ,&
&  0.70101E+00_JPRB, 0.71542E+00_JPRB, 0.85772E+00_JPRB, 0.11487E+01_JPRB, 0.17432E+01_JPRB, 0.17748E+01_JPRB, 0.18006E+01_JPRB ,&
&  0.17884E+01_JPRB, 0.17124E+01_JPRB, 0.16538E+01_JPRB, 0.15388E+01_JPRB, 0.13938E+01_JPRB, 0.12603E+01_JPRB, 0.11562E+01_JPRB ,&
&  0.10461E+01_JPRB, 0.98154E+00_JPRB, 0.91107E+00_JPRB, 0.87305E+00_JPRB, 0.82596E+00_JPRB, 0.78916E+00_JPRB, 0.74846E+00_JPRB ,&
&  0.69912E+00_JPRB, 0.64978E+00_JPRB, 0.58979E+00_JPRB, 0.53368E+00_JPRB, 0.49460E+00_JPRB, 0.47560E+00_JPRB, 0.45844E+00_JPRB ,&
&  0.44849E+00_JPRB, 0.44290E+00_JPRB, 0.43424E+00_JPRB, 0.42841E+00_JPRB, 0.41362E+00_JPRB, 0.39261E+00_JPRB, 0.38188E+00_JPRB ,&
&  0.36716E+00_JPRB, 0.35626E+00_JPRB, 0.34959E+00_JPRB, 0.34651E+00_JPRB, 0.34815E+00_JPRB, 0.35022E+00_JPRB, 0.35367E+00_JPRB ,&
&  0.35938E+00_JPRB, 0.35836E+00_JPRB, 0.35872E+00_JPRB, 0.36307E+00_JPRB, 0.37925E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.37774E+00_JPRB, 0.40104E+00_JPRB, 0.42097E+00_JPRB, 0.43552E+00_JPRB, 0.43804E+00_JPRB, 0.43679E+00_JPRB, 0.42387E+00_JPRB ,&
&  0.42451E+00_JPRB, 0.40362E+00_JPRB, 0.38917E+00_JPRB, 0.37335E+00_JPRB, 0.36134E+00_JPRB, 0.35330E+00_JPRB, 0.35392E+00_JPRB ,&
&  0.35896E+00_JPRB, 0.36546E+00_JPRB, 0.37212E+00_JPRB, 0.37653E+00_JPRB, 0.38260E+00_JPRB, 0.38979E+00_JPRB, 0.39847E+00_JPRB ,&
&  0.40684E+00_JPRB, 0.41172E+00_JPRB, 0.41619E+00_JPRB, 0.42015E+00_JPRB, 0.42441E+00_JPRB, 0.42685E+00_JPRB, 0.42176E+00_JPRB ,&
&  0.42476E+00_JPRB, 0.43102E+00_JPRB, 0.45032E+00_JPRB, 0.46791E+00_JPRB, 0.47675E+00_JPRB, 0.45894E+00_JPRB, 0.39525E+00_JPRB ,&
&  0.35454E+00_JPRB, 0.33913E+00_JPRB, 0.34301E+00_JPRB, 0.36827E+00_JPRB, 0.38044E+00_JPRB, 0.39173E+00_JPRB, 0.40242E+00_JPRB ,&
&  0.40187E+00_JPRB, 0.40562E+00_JPRB, 0.42585E+00_JPRB, 0.45597E+00_JPRB, 0.48588E+00_JPRB, 0.53917E+00_JPRB, 0.58736E+00_JPRB ,&
&  0.66408E+00_JPRB, 0.78360E+00_JPRB, 0.10027E+01_JPRB, 0.10671E+01_JPRB, 0.12053E+01_JPRB, 0.15366E+01_JPRB, 0.14903E+01_JPRB ,&
&  0.14621E+01_JPRB, 0.14509E+01_JPRB, 0.14526E+01_JPRB, 0.13921E+01_JPRB, 0.12558E+01_JPRB, 0.11342E+01_JPRB, 0.10192E+01_JPRB ,&
&  0.90215E+00_JPRB, 0.82699E+00_JPRB, 0.76519E+00_JPRB, 0.74424E+00_JPRB, 0.71389E+00_JPRB, 0.68260E+00_JPRB, 0.65735E+00_JPRB ,&
&  0.61435E+00_JPRB, 0.56351E+00_JPRB, 0.51335E+00_JPRB, 0.47308E+00_JPRB, 0.45317E+00_JPRB, 0.44276E+00_JPRB, 0.43628E+00_JPRB ,&
&  0.43000E+00_JPRB, 0.42431E+00_JPRB, 0.41792E+00_JPRB, 0.40590E+00_JPRB, 0.38581E+00_JPRB, 0.36298E+00_JPRB, 0.35386E+00_JPRB ,&
&  0.34815E+00_JPRB, 0.34325E+00_JPRB, 0.34081E+00_JPRB, 0.33708E+00_JPRB, 0.33694E+00_JPRB, 0.33616E+00_JPRB, 0.33313E+00_JPRB ,&
&  0.33554E+00_JPRB, 0.33567E+00_JPRB, 0.33325E+00_JPRB, 0.33787E+00_JPRB, 0.35550E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.35840E+00_JPRB, 0.36695E+00_JPRB, 0.37866E+00_JPRB, 0.39340E+00_JPRB, 0.40100E+00_JPRB, 0.39426E+00_JPRB, 0.37919E+00_JPRB ,&
&  0.37039E+00_JPRB, 0.36644E+00_JPRB, 0.36868E+00_JPRB, 0.37246E+00_JPRB, 0.37453E+00_JPRB, 0.37510E+00_JPRB, 0.37480E+00_JPRB ,&
&  0.37293E+00_JPRB, 0.37308E+00_JPRB, 0.37617E+00_JPRB, 0.38161E+00_JPRB, 0.38846E+00_JPRB, 0.39508E+00_JPRB, 0.40230E+00_JPRB ,&
&  0.40743E+00_JPRB, 0.40897E+00_JPRB, 0.40917E+00_JPRB, 0.40735E+00_JPRB, 0.40819E+00_JPRB, 0.41107E+00_JPRB, 0.41082E+00_JPRB ,&
&  0.40744E+00_JPRB, 0.40913E+00_JPRB, 0.41961E+00_JPRB, 0.43323E+00_JPRB, 0.44908E+00_JPRB, 0.43564E+00_JPRB, 0.39007E+00_JPRB ,&
&  0.34948E+00_JPRB, 0.32018E+00_JPRB, 0.31052E+00_JPRB, 0.31378E+00_JPRB, 0.32720E+00_JPRB, 0.34861E+00_JPRB, 0.36423E+00_JPRB ,&
&  0.37709E+00_JPRB, 0.39064E+00_JPRB, 0.40393E+00_JPRB, 0.41512E+00_JPRB, 0.43379E+00_JPRB, 0.46363E+00_JPRB, 0.50155E+00_JPRB ,&
&  0.56532E+00_JPRB, 0.65582E+00_JPRB, 0.77575E+00_JPRB, 0.80516E+00_JPRB, 0.85239E+00_JPRB, 0.95875E+00_JPRB, 0.10943E+01_JPRB ,&
&  0.11508E+01_JPRB, 0.10813E+01_JPRB, 0.10832E+01_JPRB, 0.10762E+01_JPRB, 0.10459E+01_JPRB, 0.99233E+00_JPRB, 0.89928E+00_JPRB ,&
&  0.81668E+00_JPRB, 0.74290E+00_JPRB, 0.68331E+00_JPRB, 0.64808E+00_JPRB, 0.63070E+00_JPRB, 0.61229E+00_JPRB, 0.58899E+00_JPRB ,&
&  0.55678E+00_JPRB, 0.51710E+00_JPRB, 0.47464E+00_JPRB, 0.44187E+00_JPRB, 0.43188E+00_JPRB, 0.43009E+00_JPRB, 0.42749E+00_JPRB ,&
&  0.41901E+00_JPRB, 0.40944E+00_JPRB, 0.39987E+00_JPRB, 0.38553E+00_JPRB, 0.37254E+00_JPRB, 0.35646E+00_JPRB, 0.34735E+00_JPRB ,&
&  0.34352E+00_JPRB, 0.34241E+00_JPRB, 0.34591E+00_JPRB, 0.34126E+00_JPRB, 0.33972E+00_JPRB, 0.34131E+00_JPRB, 0.33775E+00_JPRB ,&
&  0.33529E+00_JPRB, 0.33492E+00_JPRB, 0.33294E+00_JPRB, 0.33790E+00_JPRB, 0.34609E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.35719E+00_JPRB, 0.36537E+00_JPRB, 0.37179E+00_JPRB, 0.37473E+00_JPRB, 0.37209E+00_JPRB, 0.37125E+00_JPRB, 0.36539E+00_JPRB ,&
&  0.36250E+00_JPRB, 0.36400E+00_JPRB, 0.36285E+00_JPRB, 0.36427E+00_JPRB, 0.36617E+00_JPRB, 0.36829E+00_JPRB, 0.37090E+00_JPRB ,&
&  0.37427E+00_JPRB, 0.37828E+00_JPRB, 0.38263E+00_JPRB, 0.38735E+00_JPRB, 0.39234E+00_JPRB, 0.39765E+00_JPRB, 0.40264E+00_JPRB ,&
&  0.40524E+00_JPRB, 0.40474E+00_JPRB, 0.40583E+00_JPRB, 0.40435E+00_JPRB, 0.40087E+00_JPRB, 0.39523E+00_JPRB, 0.38981E+00_JPRB ,&
&  0.39096E+00_JPRB, 0.39826E+00_JPRB, 0.40044E+00_JPRB, 0.40636E+00_JPRB, 0.41757E+00_JPRB, 0.40552E+00_JPRB, 0.37150E+00_JPRB ,&
&  0.33465E+00_JPRB, 0.30443E+00_JPRB, 0.29090E+00_JPRB, 0.28639E+00_JPRB, 0.29206E+00_JPRB, 0.30390E+00_JPRB, 0.31855E+00_JPRB ,&
&  0.33332E+00_JPRB, 0.35265E+00_JPRB, 0.37177E+00_JPRB, 0.38487E+00_JPRB, 0.39702E+00_JPRB, 0.41148E+00_JPRB, 0.42321E+00_JPRB ,&
&  0.44046E+00_JPRB, 0.47048E+00_JPRB, 0.52454E+00_JPRB, 0.59199E+00_JPRB, 0.65477E+00_JPRB, 0.68973E+00_JPRB, 0.77875E+00_JPRB ,&
&  0.94058E+00_JPRB, 0.86893E+00_JPRB, 0.83574E+00_JPRB, 0.79305E+00_JPRB, 0.74840E+00_JPRB, 0.70929E+00_JPRB, 0.68166E+00_JPRB ,&
&  0.66528E+00_JPRB, 0.65151E+00_JPRB, 0.62495E+00_JPRB, 0.59192E+00_JPRB, 0.56492E+00_JPRB, 0.54609E+00_JPRB, 0.53771E+00_JPRB ,&
&  0.52723E+00_JPRB, 0.52225E+00_JPRB, 0.54571E+00_JPRB, 0.48219E+00_JPRB, 0.44822E+00_JPRB, 0.43193E+00_JPRB, 0.41741E+00_JPRB ,&
&  0.40609E+00_JPRB, 0.39790E+00_JPRB, 0.38904E+00_JPRB, 0.37754E+00_JPRB, 0.37013E+00_JPRB, 0.36113E+00_JPRB, 0.35684E+00_JPRB ,&
&  0.35501E+00_JPRB, 0.35477E+00_JPRB, 0.35777E+00_JPRB, 0.35871E+00_JPRB, 0.35458E+00_JPRB, 0.35308E+00_JPRB, 0.35185E+00_JPRB ,&
&  0.34757E+00_JPRB, 0.34426E+00_JPRB, 0.34379E+00_JPRB, 0.34739E+00_JPRB, 0.35099E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.36884E+00_JPRB, 0.37280E+00_JPRB, 0.37466E+00_JPRB, 0.37700E+00_JPRB, 0.37509E+00_JPRB, 0.37457E+00_JPRB, 0.37018E+00_JPRB ,&
&  0.36660E+00_JPRB, 0.36466E+00_JPRB, 0.36394E+00_JPRB, 0.36592E+00_JPRB, 0.36967E+00_JPRB, 0.37430E+00_JPRB, 0.37876E+00_JPRB ,&
&  0.38334E+00_JPRB, 0.38357E+00_JPRB, 0.38658E+00_JPRB, 0.39178E+00_JPRB, 0.39447E+00_JPRB, 0.39828E+00_JPRB, 0.40049E+00_JPRB ,&
&  0.40091E+00_JPRB, 0.39833E+00_JPRB, 0.39649E+00_JPRB, 0.39170E+00_JPRB, 0.38811E+00_JPRB, 0.38580E+00_JPRB, 0.38692E+00_JPRB ,&
&  0.38113E+00_JPRB, 0.37846E+00_JPRB, 0.39028E+00_JPRB, 0.39713E+00_JPRB, 0.38146E+00_JPRB, 0.36828E+00_JPRB, 0.34267E+00_JPRB ,&
&  0.31691E+00_JPRB, 0.29183E+00_JPRB, 0.28282E+00_JPRB, 0.27789E+00_JPRB, 0.28106E+00_JPRB, 0.28693E+00_JPRB, 0.29841E+00_JPRB ,&
&  0.30988E+00_JPRB, 0.32590E+00_JPRB, 0.34046E+00_JPRB, 0.35638E+00_JPRB, 0.36705E+00_JPRB, 0.37946E+00_JPRB, 0.38860E+00_JPRB ,&
&  0.40042E+00_JPRB, 0.41697E+00_JPRB, 0.44176E+00_JPRB, 0.47239E+00_JPRB, 0.50063E+00_JPRB, 0.53404E+00_JPRB, 0.58239E+00_JPRB ,&
&  0.65368E+00_JPRB, 0.64365E+00_JPRB, 0.64310E+00_JPRB, 0.62583E+00_JPRB, 0.60220E+00_JPRB, 0.58282E+00_JPRB, 0.56502E+00_JPRB ,&
&  0.55753E+00_JPRB, 0.55944E+00_JPRB, 0.54760E+00_JPRB, 0.53567E+00_JPRB, 0.52102E+00_JPRB, 0.50811E+00_JPRB, 0.50144E+00_JPRB ,&
&  0.49576E+00_JPRB, 0.49180E+00_JPRB, 0.49784E+00_JPRB, 0.47204E+00_JPRB, 0.45182E+00_JPRB, 0.43334E+00_JPRB, 0.41850E+00_JPRB ,&
&  0.40700E+00_JPRB, 0.39787E+00_JPRB, 0.38906E+00_JPRB, 0.38092E+00_JPRB, 0.37644E+00_JPRB, 0.37012E+00_JPRB, 0.36903E+00_JPRB ,&
&  0.36851E+00_JPRB, 0.36873E+00_JPRB, 0.37014E+00_JPRB, 0.36938E+00_JPRB, 0.36268E+00_JPRB, 0.35694E+00_JPRB, 0.35514E+00_JPRB ,&
&  0.35318E+00_JPRB, 0.35480E+00_JPRB, 0.35741E+00_JPRB, 0.36171E+00_JPRB, 0.36502E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.38304E+00_JPRB, 0.38665E+00_JPRB, 0.38897E+00_JPRB, 0.38897E+00_JPRB, 0.38712E+00_JPRB, 0.38576E+00_JPRB, 0.38349E+00_JPRB ,&
&  0.38186E+00_JPRB, 0.38040E+00_JPRB, 0.37981E+00_JPRB, 0.38098E+00_JPRB, 0.38415E+00_JPRB, 0.38730E+00_JPRB, 0.38938E+00_JPRB ,&
&  0.39047E+00_JPRB, 0.39208E+00_JPRB, 0.39241E+00_JPRB, 0.39360E+00_JPRB, 0.39441E+00_JPRB, 0.39705E+00_JPRB, 0.39788E+00_JPRB ,&
&  0.39913E+00_JPRB, 0.39598E+00_JPRB, 0.38783E+00_JPRB, 0.38372E+00_JPRB, 0.38434E+00_JPRB, 0.37575E+00_JPRB, 0.37783E+00_JPRB ,&
&  0.38221E+00_JPRB, 0.38339E+00_JPRB, 0.37959E+00_JPRB, 0.37796E+00_JPRB, 0.36818E+00_JPRB, 0.31947E+00_JPRB, 0.31789E+00_JPRB ,&
&  0.30737E+00_JPRB, 0.29220E+00_JPRB, 0.28480E+00_JPRB, 0.27909E+00_JPRB, 0.27764E+00_JPRB, 0.28417E+00_JPRB, 0.29436E+00_JPRB ,&
&  0.30050E+00_JPRB, 0.32125E+00_JPRB, 0.33005E+00_JPRB, 0.34087E+00_JPRB, 0.35044E+00_JPRB, 0.36327E+00_JPRB, 0.37177E+00_JPRB ,&
&  0.38321E+00_JPRB, 0.39494E+00_JPRB, 0.41399E+00_JPRB, 0.43760E+00_JPRB, 0.46144E+00_JPRB, 0.48437E+00_JPRB, 0.50810E+00_JPRB ,&
&  0.55215E+00_JPRB, 0.55493E+00_JPRB, 0.55829E+00_JPRB, 0.55549E+00_JPRB, 0.53892E+00_JPRB, 0.52526E+00_JPRB, 0.51051E+00_JPRB ,&
&  0.50628E+00_JPRB, 0.50604E+00_JPRB, 0.50628E+00_JPRB, 0.49975E+00_JPRB, 0.49272E+00_JPRB, 0.48684E+00_JPRB, 0.48279E+00_JPRB ,&
&  0.47859E+00_JPRB, 0.47310E+00_JPRB, 0.46606E+00_JPRB, 0.45153E+00_JPRB, 0.44337E+00_JPRB, 0.43079E+00_JPRB, 0.42157E+00_JPRB ,&
&  0.41222E+00_JPRB, 0.40447E+00_JPRB, 0.39349E+00_JPRB, 0.38972E+00_JPRB, 0.38681E+00_JPRB, 0.38523E+00_JPRB, 0.38324E+00_JPRB ,&
&  0.38050E+00_JPRB, 0.37874E+00_JPRB, 0.37749E+00_JPRB, 0.37490E+00_JPRB, 0.36914E+00_JPRB, 0.36390E+00_JPRB, 0.36389E+00_JPRB ,&
&  0.36561E+00_JPRB, 0.36809E+00_JPRB, 0.37069E+00_JPRB, 0.37467E+00_JPRB, 0.37816E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.38449E+00_JPRB, 0.38633E+00_JPRB, 0.38821E+00_JPRB, 0.38901E+00_JPRB, 0.38963E+00_JPRB, 0.39044E+00_JPRB, 0.39123E+00_JPRB ,&
&  0.39123E+00_JPRB, 0.39074E+00_JPRB, 0.39039E+00_JPRB, 0.38991E+00_JPRB, 0.39121E+00_JPRB, 0.39291E+00_JPRB, 0.39539E+00_JPRB ,&
&  0.39610E+00_JPRB, 0.39626E+00_JPRB, 0.39532E+00_JPRB, 0.39443E+00_JPRB, 0.39370E+00_JPRB, 0.39381E+00_JPRB, 0.39284E+00_JPRB ,&
&  0.39304E+00_JPRB, 0.39020E+00_JPRB, 0.38066E+00_JPRB, 0.37917E+00_JPRB, 0.37568E+00_JPRB, 0.37399E+00_JPRB, 0.37120E+00_JPRB ,&
&  0.36578E+00_JPRB, 0.34373E+00_JPRB, 0.35272E+00_JPRB, 0.34314E+00_JPRB, 0.33337E+00_JPRB, 0.32471E+00_JPRB, 0.31401E+00_JPRB ,&
&  0.30778E+00_JPRB, 0.29957E+00_JPRB, 0.29373E+00_JPRB, 0.29189E+00_JPRB, 0.29251E+00_JPRB, 0.29807E+00_JPRB, 0.30533E+00_JPRB ,&
&  0.30948E+00_JPRB, 0.32226E+00_JPRB, 0.32962E+00_JPRB, 0.33819E+00_JPRB, 0.34837E+00_JPRB, 0.36080E+00_JPRB, 0.36852E+00_JPRB ,&
&  0.38016E+00_JPRB, 0.38627E+00_JPRB, 0.40016E+00_JPRB, 0.41423E+00_JPRB, 0.43672E+00_JPRB, 0.45650E+00_JPRB, 0.47664E+00_JPRB ,&
&  0.50288E+00_JPRB, 0.51504E+00_JPRB, 0.52107E+00_JPRB, 0.52040E+00_JPRB, 0.51026E+00_JPRB, 0.49999E+00_JPRB, 0.48862E+00_JPRB ,&
&  0.48028E+00_JPRB, 0.47398E+00_JPRB, 0.47089E+00_JPRB, 0.46810E+00_JPRB, 0.46356E+00_JPRB, 0.46151E+00_JPRB, 0.46008E+00_JPRB ,&
&  0.45825E+00_JPRB, 0.45263E+00_JPRB, 0.44864E+00_JPRB, 0.43866E+00_JPRB, 0.43143E+00_JPRB, 0.41984E+00_JPRB, 0.41049E+00_JPRB ,&
&  0.40357E+00_JPRB, 0.39552E+00_JPRB, 0.38820E+00_JPRB, 0.38336E+00_JPRB, 0.38093E+00_JPRB, 0.38063E+00_JPRB, 0.38200E+00_JPRB ,&
&  0.38355E+00_JPRB, 0.38535E+00_JPRB, 0.38701E+00_JPRB, 0.38355E+00_JPRB, 0.37642E+00_JPRB, 0.37130E+00_JPRB, 0.36993E+00_JPRB ,&
&  0.37116E+00_JPRB, 0.37388E+00_JPRB, 0.37681E+00_JPRB, 0.38107E+00_JPRB, 0.38348E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.38787E+00_JPRB, 0.38666E+00_JPRB, 0.38534E+00_JPRB, 0.38349E+00_JPRB, 0.38259E+00_JPRB, 0.38207E+00_JPRB, 0.38356E+00_JPRB ,&
&  0.38515E+00_JPRB, 0.38728E+00_JPRB, 0.38854E+00_JPRB, 0.38906E+00_JPRB, 0.38985E+00_JPRB, 0.38968E+00_JPRB, 0.39061E+00_JPRB ,&
&  0.39204E+00_JPRB, 0.39346E+00_JPRB, 0.39440E+00_JPRB, 0.39438E+00_JPRB, 0.39371E+00_JPRB, 0.39326E+00_JPRB, 0.38946E+00_JPRB ,&
&  0.38661E+00_JPRB, 0.38394E+00_JPRB, 0.37024E+00_JPRB, 0.37252E+00_JPRB, 0.36882E+00_JPRB, 0.36683E+00_JPRB, 0.36534E+00_JPRB ,&
&  0.35594E+00_JPRB, 0.35189E+00_JPRB, 0.34138E+00_JPRB, 0.33232E+00_JPRB, 0.33229E+00_JPRB, 0.32467E+00_JPRB, 0.32477E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32012E+00_JPRB, 0.31955E+00_JPRB, 0.31977E+00_JPRB, 0.32305E+00_JPRB, 0.32575E+00_JPRB, 0.32870E+00_JPRB ,&
&  0.33364E+00_JPRB, 0.33911E+00_JPRB, 0.34796E+00_JPRB, 0.35145E+00_JPRB, 0.36242E+00_JPRB, 0.37465E+00_JPRB, 0.38741E+00_JPRB ,&
&  0.39665E+00_JPRB, 0.39751E+00_JPRB, 0.40219E+00_JPRB, 0.40889E+00_JPRB, 0.41634E+00_JPRB, 0.42580E+00_JPRB, 0.43827E+00_JPRB ,&
&  0.45469E+00_JPRB, 0.46781E+00_JPRB, 0.47588E+00_JPRB, 0.47630E+00_JPRB, 0.47398E+00_JPRB, 0.46872E+00_JPRB, 0.46381E+00_JPRB ,&
&  0.45989E+00_JPRB, 0.45599E+00_JPRB, 0.45257E+00_JPRB, 0.44910E+00_JPRB, 0.44475E+00_JPRB, 0.44168E+00_JPRB, 0.43821E+00_JPRB ,&
&  0.43415E+00_JPRB, 0.42908E+00_JPRB, 0.42727E+00_JPRB, 0.42022E+00_JPRB, 0.41593E+00_JPRB, 0.41094E+00_JPRB, 0.40292E+00_JPRB ,&
&  0.39531E+00_JPRB, 0.38814E+00_JPRB, 0.38152E+00_JPRB, 0.37613E+00_JPRB, 0.37235E+00_JPRB, 0.37155E+00_JPRB, 0.37303E+00_JPRB ,&
&  0.37694E+00_JPRB, 0.38206E+00_JPRB, 0.38691E+00_JPRB, 0.38698E+00_JPRB, 0.38611E+00_JPRB, 0.38416E+00_JPRB, 0.38318E+00_JPRB ,&
&  0.38249E+00_JPRB, 0.38365E+00_JPRB, 0.38485E+00_JPRB, 0.38675E+00_JPRB, 0.38838E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.37619E+00_JPRB, 0.37637E+00_JPRB, 0.37671E+00_JPRB, 0.37703E+00_JPRB, 0.37781E+00_JPRB, 0.37884E+00_JPRB, 0.37998E+00_JPRB ,&
&  0.38127E+00_JPRB, 0.38308E+00_JPRB, 0.38469E+00_JPRB, 0.38594E+00_JPRB, 0.38667E+00_JPRB, 0.38760E+00_JPRB, 0.38842E+00_JPRB ,&
&  0.38824E+00_JPRB, 0.38809E+00_JPRB, 0.38846E+00_JPRB, 0.38893E+00_JPRB, 0.38919E+00_JPRB, 0.38939E+00_JPRB, 0.38669E+00_JPRB ,&
&  0.38455E+00_JPRB, 0.38265E+00_JPRB, 0.38027E+00_JPRB, 0.37653E+00_JPRB, 0.37423E+00_JPRB, 0.36169E+00_JPRB, 0.36330E+00_JPRB ,&
&  0.37396E+00_JPRB, 0.37283E+00_JPRB, 0.37052E+00_JPRB, 0.36623E+00_JPRB, 0.35235E+00_JPRB, 0.34541E+00_JPRB, 0.34393E+00_JPRB ,&
&  0.33340E+00_JPRB, 0.34142E+00_JPRB, 0.34391E+00_JPRB, 0.34520E+00_JPRB, 0.34811E+00_JPRB, 0.34680E+00_JPRB, 0.34829E+00_JPRB ,&
&  0.35690E+00_JPRB, 0.35879E+00_JPRB, 0.36486E+00_JPRB, 0.37243E+00_JPRB, 0.38067E+00_JPRB, 0.38818E+00_JPRB, 0.39573E+00_JPRB ,&
&  0.40136E+00_JPRB, 0.40502E+00_JPRB, 0.40716E+00_JPRB, 0.40777E+00_JPRB, 0.40927E+00_JPRB, 0.41214E+00_JPRB, 0.41752E+00_JPRB ,&
&  0.42677E+00_JPRB, 0.43727E+00_JPRB, 0.44609E+00_JPRB, 0.45099E+00_JPRB, 0.45670E+00_JPRB, 0.46018E+00_JPRB, 0.46278E+00_JPRB ,&
&  0.46395E+00_JPRB, 0.46248E+00_JPRB, 0.45783E+00_JPRB, 0.45349E+00_JPRB, 0.44935E+00_JPRB, 0.44412E+00_JPRB, 0.43751E+00_JPRB ,&
&  0.43053E+00_JPRB, 0.42329E+00_JPRB, 0.41640E+00_JPRB, 0.41014E+00_JPRB, 0.40481E+00_JPRB, 0.39953E+00_JPRB, 0.39515E+00_JPRB ,&
&  0.39028E+00_JPRB, 0.38600E+00_JPRB, 0.38125E+00_JPRB, 0.37668E+00_JPRB, 0.37239E+00_JPRB, 0.36971E+00_JPRB, 0.36794E+00_JPRB ,&
&  0.36809E+00_JPRB, 0.36930E+00_JPRB, 0.37025E+00_JPRB, 0.37065E+00_JPRB, 0.37199E+00_JPRB, 0.37374E+00_JPRB, 0.37510E+00_JPRB ,&
&  0.37601E+00_JPRB, 0.37613E+00_JPRB, 0.37603E+00_JPRB, 0.37609E+00_JPRB, 0.37612E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.36557E+00_JPRB, 0.36603E+00_JPRB, 0.36645E+00_JPRB, 0.36692E+00_JPRB, 0.36770E+00_JPRB, 0.36863E+00_JPRB, 0.36955E+00_JPRB ,&
&  0.37045E+00_JPRB, 0.37165E+00_JPRB, 0.37293E+00_JPRB, 0.37406E+00_JPRB, 0.37491E+00_JPRB, 0.37597E+00_JPRB, 0.37701E+00_JPRB ,&
&  0.37775E+00_JPRB, 0.37827E+00_JPRB, 0.37847E+00_JPRB, 0.37883E+00_JPRB, 0.37924E+00_JPRB, 0.37953E+00_JPRB, 0.38049E+00_JPRB ,&
&  0.38141E+00_JPRB, 0.38185E+00_JPRB, 0.38168E+00_JPRB, 0.37896E+00_JPRB, 0.37708E+00_JPRB, 0.37533E+00_JPRB, 0.37379E+00_JPRB ,&
&  0.37170E+00_JPRB, 0.37146E+00_JPRB, 0.37149E+00_JPRB, 0.37152E+00_JPRB, 0.37323E+00_JPRB, 0.37300E+00_JPRB, 0.37254E+00_JPRB ,&
&  0.37161E+00_JPRB, 0.36885E+00_JPRB, 0.36938E+00_JPRB, 0.37021E+00_JPRB, 0.37102E+00_JPRB, 0.37310E+00_JPRB, 0.37506E+00_JPRB ,&
&  0.37734E+00_JPRB, 0.37980E+00_JPRB, 0.38375E+00_JPRB, 0.38813E+00_JPRB, 0.39206E+00_JPRB, 0.39562E+00_JPRB, 0.39828E+00_JPRB ,&
&  0.40061E+00_JPRB, 0.40307E+00_JPRB, 0.40573E+00_JPRB, 0.40833E+00_JPRB, 0.41069E+00_JPRB, 0.41389E+00_JPRB, 0.41719E+00_JPRB ,&
&  0.42048E+00_JPRB, 0.42411E+00_JPRB, 0.42993E+00_JPRB, 0.43690E+00_JPRB, 0.44421E+00_JPRB, 0.44893E+00_JPRB, 0.45236E+00_JPRB ,&
&  0.45442E+00_JPRB, 0.45484E+00_JPRB, 0.45302E+00_JPRB, 0.45024E+00_JPRB, 0.44690E+00_JPRB, 0.44206E+00_JPRB, 0.43678E+00_JPRB ,&
&  0.43147E+00_JPRB, 0.42624E+00_JPRB, 0.42073E+00_JPRB, 0.41479E+00_JPRB, 0.40931E+00_JPRB, 0.40407E+00_JPRB, 0.39955E+00_JPRB ,&
&  0.39473E+00_JPRB, 0.39043E+00_JPRB, 0.38628E+00_JPRB, 0.38265E+00_JPRB, 0.37877E+00_JPRB, 0.37540E+00_JPRB, 0.37239E+00_JPRB ,&
&  0.37022E+00_JPRB, 0.36798E+00_JPRB, 0.36578E+00_JPRB, 0.36416E+00_JPRB, 0.36364E+00_JPRB, 0.36370E+00_JPRB, 0.36394E+00_JPRB ,&
&  0.36415E+00_JPRB, 0.36448E+00_JPRB, 0.36491E+00_JPRB, 0.36514E+00_JPRB, 0.36526E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.37440E+00_JPRB, 0.37457E+00_JPRB, 0.37463E+00_JPRB, 0.37470E+00_JPRB, 0.37473E+00_JPRB, 0.37486E+00_JPRB, 0.37497E+00_JPRB ,&
&  0.37506E+00_JPRB, 0.37526E+00_JPRB, 0.37561E+00_JPRB, 0.37595E+00_JPRB, 0.37632E+00_JPRB, 0.37662E+00_JPRB, 0.37697E+00_JPRB ,&
&  0.37720E+00_JPRB, 0.37732E+00_JPRB, 0.37742E+00_JPRB, 0.37782E+00_JPRB, 0.37820E+00_JPRB, 0.37850E+00_JPRB, 0.37891E+00_JPRB ,&
&  0.37910E+00_JPRB, 0.37891E+00_JPRB, 0.37865E+00_JPRB, 0.37883E+00_JPRB, 0.37896E+00_JPRB, 0.37911E+00_JPRB, 0.37924E+00_JPRB ,&
&  0.37933E+00_JPRB, 0.37947E+00_JPRB, 0.37951E+00_JPRB, 0.37951E+00_JPRB, 0.37979E+00_JPRB, 0.38023E+00_JPRB, 0.38075E+00_JPRB ,&
&  0.38133E+00_JPRB, 0.38195E+00_JPRB, 0.38280E+00_JPRB, 0.38363E+00_JPRB, 0.38450E+00_JPRB, 0.38504E+00_JPRB, 0.38572E+00_JPRB ,&
&  0.38634E+00_JPRB, 0.38697E+00_JPRB, 0.38777E+00_JPRB, 0.38871E+00_JPRB, 0.38936E+00_JPRB, 0.39016E+00_JPRB, 0.39074E+00_JPRB ,&
&  0.39195E+00_JPRB, 0.39350E+00_JPRB, 0.39522E+00_JPRB, 0.39699E+00_JPRB, 0.39868E+00_JPRB, 0.40050E+00_JPRB, 0.40256E+00_JPRB ,&
&  0.40455E+00_JPRB, 0.40645E+00_JPRB, 0.40883E+00_JPRB, 0.41158E+00_JPRB, 0.41445E+00_JPRB, 0.41648E+00_JPRB, 0.41848E+00_JPRB ,&
&  0.42042E+00_JPRB, 0.42206E+00_JPRB, 0.42213E+00_JPRB, 0.42203E+00_JPRB, 0.42162E+00_JPRB, 0.42057E+00_JPRB, 0.41899E+00_JPRB ,&
&  0.41711E+00_JPRB, 0.41509E+00_JPRB, 0.41229E+00_JPRB, 0.40915E+00_JPRB, 0.40625E+00_JPRB, 0.40350E+00_JPRB, 0.40078E+00_JPRB ,&
&  0.39814E+00_JPRB, 0.39583E+00_JPRB, 0.39368E+00_JPRB, 0.39153E+00_JPRB, 0.38923E+00_JPRB, 0.38705E+00_JPRB, 0.38488E+00_JPRB ,&
&  0.38301E+00_JPRB, 0.38122E+00_JPRB, 0.37956E+00_JPRB, 0.37802E+00_JPRB, 0.37692E+00_JPRB, 0.37619E+00_JPRB, 0.37558E+00_JPRB ,&
&  0.37506E+00_JPRB, 0.37478E+00_JPRB, 0.37469E+00_JPRB, 0.37456E+00_JPRB, 0.37439E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB, 0.38924E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB, 0.21129E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.23742E+00_JPRB, 0.23747E+00_JPRB, 0.23752E+00_JPRB, 0.23717E+00_JPRB, 0.23572E+00_JPRB, 0.23466E+00_JPRB, 0.23376E+00_JPRB ,&
&  0.23337E+00_JPRB, 0.23260E+00_JPRB, 0.23222E+00_JPRB, 0.23210E+00_JPRB, 0.23179E+00_JPRB, 0.23130E+00_JPRB, 0.23067E+00_JPRB ,&
&  0.22985E+00_JPRB, 0.22876E+00_JPRB, 0.22774E+00_JPRB, 0.22766E+00_JPRB, 0.22794E+00_JPRB, 0.22823E+00_JPRB, 0.22848E+00_JPRB ,&
&  0.22862E+00_JPRB, 0.22870E+00_JPRB, 0.22830E+00_JPRB, 0.22834E+00_JPRB, 0.22838E+00_JPRB, 0.22839E+00_JPRB, 0.22854E+00_JPRB ,&
&  0.22880E+00_JPRB, 0.22899E+00_JPRB, 0.22873E+00_JPRB, 0.22824E+00_JPRB, 0.22732E+00_JPRB, 0.22565E+00_JPRB, 0.22306E+00_JPRB ,&
&  0.22013E+00_JPRB, 0.21718E+00_JPRB, 0.21482E+00_JPRB, 0.21265E+00_JPRB, 0.21066E+00_JPRB, 0.20884E+00_JPRB, 0.20732E+00_JPRB ,&
&  0.20553E+00_JPRB, 0.20376E+00_JPRB, 0.20215E+00_JPRB, 0.20103E+00_JPRB, 0.19981E+00_JPRB, 0.19876E+00_JPRB, 0.19783E+00_JPRB ,&
&  0.19721E+00_JPRB, 0.19666E+00_JPRB, 0.19618E+00_JPRB, 0.19594E+00_JPRB, 0.19575E+00_JPRB, 0.19556E+00_JPRB, 0.19535E+00_JPRB ,&
&  0.19515E+00_JPRB, 0.19499E+00_JPRB, 0.19486E+00_JPRB, 0.19479E+00_JPRB, 0.19483E+00_JPRB, 0.19468E+00_JPRB, 0.19451E+00_JPRB ,&
&  0.19443E+00_JPRB, 0.19464E+00_JPRB, 0.19481E+00_JPRB, 0.19511E+00_JPRB, 0.19558E+00_JPRB, 0.19624E+00_JPRB, 0.19696E+00_JPRB ,&
&  0.19769E+00_JPRB, 0.19853E+00_JPRB, 0.19947E+00_JPRB, 0.20059E+00_JPRB, 0.20178E+00_JPRB, 0.20297E+00_JPRB, 0.20411E+00_JPRB ,&
&  0.20534E+00_JPRB, 0.20647E+00_JPRB, 0.20752E+00_JPRB, 0.20831E+00_JPRB, 0.20934E+00_JPRB, 0.21038E+00_JPRB, 0.21158E+00_JPRB ,&
&  0.21293E+00_JPRB, 0.21470E+00_JPRB, 0.21706E+00_JPRB, 0.21984E+00_JPRB, 0.22321E+00_JPRB, 0.22577E+00_JPRB, 0.22843E+00_JPRB ,&
&  0.23039E+00_JPRB, 0.23186E+00_JPRB, 0.23293E+00_JPRB, 0.23433E+00_JPRB, 0.23587E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.27276E+00_JPRB, 0.27853E+00_JPRB, 0.28270E+00_JPRB, 0.29191E+00_JPRB, 0.30663E+00_JPRB, 0.40197E+00_JPRB, 0.39694E+00_JPRB ,&
&  0.39579E+00_JPRB, 0.35309E+00_JPRB, 0.34478E+00_JPRB, 0.33250E+00_JPRB, 0.31715E+00_JPRB, 0.29813E+00_JPRB, 0.28629E+00_JPRB ,&
&  0.27643E+00_JPRB, 0.26507E+00_JPRB, 0.25431E+00_JPRB, 0.24809E+00_JPRB, 0.24314E+00_JPRB, 0.23886E+00_JPRB, 0.23655E+00_JPRB ,&
&  0.23595E+00_JPRB, 0.23673E+00_JPRB, 0.23785E+00_JPRB, 0.23969E+00_JPRB, 0.24328E+00_JPRB, 0.24711E+00_JPRB, 0.25089E+00_JPRB ,&
&  0.25301E+00_JPRB, 0.25696E+00_JPRB, 0.25755E+00_JPRB, 0.25430E+00_JPRB, 0.24883E+00_JPRB, 0.24179E+00_JPRB, 0.23468E+00_JPRB ,&
&  0.22732E+00_JPRB, 0.22040E+00_JPRB, 0.21471E+00_JPRB, 0.20952E+00_JPRB, 0.20372E+00_JPRB, 0.19840E+00_JPRB, 0.19505E+00_JPRB ,&
&  0.19210E+00_JPRB, 0.18913E+00_JPRB, 0.18683E+00_JPRB, 0.18565E+00_JPRB, 0.18431E+00_JPRB, 0.18343E+00_JPRB, 0.18280E+00_JPRB ,&
&  0.18263E+00_JPRB, 0.18254E+00_JPRB, 0.18262E+00_JPRB, 0.18291E+00_JPRB, 0.18289E+00_JPRB, 0.18282E+00_JPRB, 0.18280E+00_JPRB ,&
&  0.18333E+00_JPRB, 0.18334E+00_JPRB, 0.18322E+00_JPRB, 0.18296E+00_JPRB, 0.18349E+00_JPRB, 0.18358E+00_JPRB, 0.18383E+00_JPRB ,&
&  0.18436E+00_JPRB, 0.18515E+00_JPRB, 0.18577E+00_JPRB, 0.18651E+00_JPRB, 0.18722E+00_JPRB, 0.18804E+00_JPRB, 0.18900E+00_JPRB ,&
&  0.18996E+00_JPRB, 0.19138E+00_JPRB, 0.19283E+00_JPRB, 0.19413E+00_JPRB, 0.19529E+00_JPRB, 0.19686E+00_JPRB, 0.19863E+00_JPRB ,&
&  0.20131E+00_JPRB, 0.20393E+00_JPRB, 0.20663E+00_JPRB, 0.20841E+00_JPRB, 0.21111E+00_JPRB, 0.21337E+00_JPRB, 0.21571E+00_JPRB ,&
&  0.21835E+00_JPRB, 0.22246E+00_JPRB, 0.22819E+00_JPRB, 0.23427E+00_JPRB, 0.24078E+00_JPRB, 0.24704E+00_JPRB, 0.25122E+00_JPRB ,&
&  0.25501E+00_JPRB, 0.25905E+00_JPRB, 0.26518E+00_JPRB, 0.26848E+00_JPRB, 0.27116E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.48997E+00_JPRB, 0.49196E+00_JPRB, 0.49417E+00_JPRB, 0.49792E+00_JPRB, 0.50187E+00_JPRB, 0.50602E+00_JPRB, 0.50983E+00_JPRB ,&
&  0.49179E+00_JPRB, 0.46864E+00_JPRB, 0.44975E+00_JPRB, 0.42736E+00_JPRB, 0.40696E+00_JPRB, 0.39061E+00_JPRB, 0.37058E+00_JPRB ,&
&  0.35071E+00_JPRB, 0.33079E+00_JPRB, 0.30822E+00_JPRB, 0.28974E+00_JPRB, 0.27155E+00_JPRB, 0.25680E+00_JPRB, 0.24747E+00_JPRB ,&
&  0.24266E+00_JPRB, 0.24147E+00_JPRB, 0.24313E+00_JPRB, 0.24677E+00_JPRB, 0.25008E+00_JPRB, 0.25502E+00_JPRB, 0.26170E+00_JPRB ,&
&  0.27094E+00_JPRB, 0.28493E+00_JPRB, 0.29761E+00_JPRB, 0.30242E+00_JPRB, 0.29930E+00_JPRB, 0.29126E+00_JPRB, 0.28437E+00_JPRB ,&
&  0.27590E+00_JPRB, 0.26360E+00_JPRB, 0.24900E+00_JPRB, 0.23461E+00_JPRB, 0.22124E+00_JPRB, 0.21148E+00_JPRB, 0.20356E+00_JPRB ,&
&  0.19695E+00_JPRB, 0.19119E+00_JPRB, 0.18685E+00_JPRB, 0.18388E+00_JPRB, 0.18135E+00_JPRB, 0.17927E+00_JPRB, 0.17734E+00_JPRB ,&
&  0.17630E+00_JPRB, 0.17520E+00_JPRB, 0.17452E+00_JPRB, 0.17413E+00_JPRB, 0.17369E+00_JPRB, 0.17359E+00_JPRB, 0.17350E+00_JPRB ,&
&  0.17320E+00_JPRB, 0.17261E+00_JPRB, 0.17226E+00_JPRB, 0.17202E+00_JPRB, 0.17235E+00_JPRB, 0.17260E+00_JPRB, 0.17304E+00_JPRB ,&
&  0.17377E+00_JPRB, 0.17538E+00_JPRB, 0.17644E+00_JPRB, 0.17741E+00_JPRB, 0.17792E+00_JPRB, 0.17832E+00_JPRB, 0.17887E+00_JPRB ,&
&  0.17980E+00_JPRB, 0.18095E+00_JPRB, 0.18203E+00_JPRB, 0.18345E+00_JPRB, 0.18491E+00_JPRB, 0.18692E+00_JPRB, 0.18904E+00_JPRB ,&
&  0.19157E+00_JPRB, 0.19473E+00_JPRB, 0.19868E+00_JPRB, 0.20238E+00_JPRB, 0.20785E+00_JPRB, 0.21334E+00_JPRB, 0.21942E+00_JPRB ,&
&  0.22555E+00_JPRB, 0.23215E+00_JPRB, 0.24061E+00_JPRB, 0.25059E+00_JPRB, 0.26283E+00_JPRB, 0.27168E+00_JPRB, 0.28367E+00_JPRB ,&
&  0.30189E+00_JPRB, 0.34046E+00_JPRB, 0.47504E+00_JPRB, 0.48846E+00_JPRB, 0.49272E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.50244E+00_JPRB, 0.51124E+00_JPRB, 0.52562E+00_JPRB, 0.53726E+00_JPRB, 0.56188E+00_JPRB, 0.57568E+00_JPRB, 0.62246E+00_JPRB ,&
&  0.57653E+00_JPRB, 0.49712E+00_JPRB, 0.46523E+00_JPRB, 0.43088E+00_JPRB, 0.39137E+00_JPRB, 0.36308E+00_JPRB, 0.34306E+00_JPRB ,&
&  0.33127E+00_JPRB, 0.32123E+00_JPRB, 0.31164E+00_JPRB, 0.29956E+00_JPRB, 0.28708E+00_JPRB, 0.27292E+00_JPRB, 0.26208E+00_JPRB ,&
&  0.25550E+00_JPRB, 0.25342E+00_JPRB, 0.25310E+00_JPRB, 0.25758E+00_JPRB, 0.26378E+00_JPRB, 0.27210E+00_JPRB, 0.28153E+00_JPRB ,&
&  0.30234E+00_JPRB, 0.30790E+00_JPRB, 0.30731E+00_JPRB, 0.30756E+00_JPRB, 0.30894E+00_JPRB, 0.31100E+00_JPRB, 0.30135E+00_JPRB ,&
&  0.31190E+00_JPRB, 0.33497E+00_JPRB, 0.34118E+00_JPRB, 0.32329E+00_JPRB, 0.26883E+00_JPRB, 0.23576E+00_JPRB, 0.21332E+00_JPRB ,&
&  0.20447E+00_JPRB, 0.19512E+00_JPRB, 0.18928E+00_JPRB, 0.18389E+00_JPRB, 0.18065E+00_JPRB, 0.17883E+00_JPRB, 0.17709E+00_JPRB ,&
&  0.17573E+00_JPRB, 0.17456E+00_JPRB, 0.17261E+00_JPRB, 0.17167E+00_JPRB, 0.17072E+00_JPRB, 0.17022E+00_JPRB, 0.16992E+00_JPRB ,&
&  0.16977E+00_JPRB, 0.16945E+00_JPRB, 0.16959E+00_JPRB, 0.17008E+00_JPRB, 0.17095E+00_JPRB, 0.17164E+00_JPRB, 0.17228E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.17366E+00_JPRB, 0.17375E+00_JPRB, 0.17407E+00_JPRB, 0.17438E+00_JPRB, 0.17533E+00_JPRB, 0.17572E+00_JPRB ,&
&  0.17615E+00_JPRB, 0.17647E+00_JPRB, 0.17640E+00_JPRB, 0.17685E+00_JPRB, 0.17765E+00_JPRB, 0.17931E+00_JPRB, 0.18127E+00_JPRB ,&
&  0.18381E+00_JPRB, 0.18673E+00_JPRB, 0.19002E+00_JPRB, 0.19326E+00_JPRB, 0.19757E+00_JPRB, 0.20258E+00_JPRB, 0.20890E+00_JPRB ,&
&  0.21584E+00_JPRB, 0.22417E+00_JPRB, 0.23514E+00_JPRB, 0.24877E+00_JPRB, 0.26526E+00_JPRB, 0.28338E+00_JPRB, 0.30493E+00_JPRB ,&
&  0.34088E+00_JPRB, 0.44226E+00_JPRB, 0.50234E+00_JPRB, 0.48770E+00_JPRB, 0.49712E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.57960E+00_JPRB, 0.59560E+00_JPRB, 0.62106E+00_JPRB, 0.61374E+00_JPRB, 0.63419E+00_JPRB, 0.62070E+00_JPRB, 0.59832E+00_JPRB ,&
&  0.55103E+00_JPRB, 0.49366E+00_JPRB, 0.44737E+00_JPRB, 0.39525E+00_JPRB, 0.35512E+00_JPRB, 0.32549E+00_JPRB, 0.31117E+00_JPRB ,&
&  0.30454E+00_JPRB, 0.30252E+00_JPRB, 0.30180E+00_JPRB, 0.30119E+00_JPRB, 0.29708E+00_JPRB, 0.29406E+00_JPRB, 0.28864E+00_JPRB ,&
&  0.28272E+00_JPRB, 0.27786E+00_JPRB, 0.27456E+00_JPRB, 0.27956E+00_JPRB, 0.28590E+00_JPRB, 0.29266E+00_JPRB, 0.29658E+00_JPRB ,&
&  0.30027E+00_JPRB, 0.30159E+00_JPRB, 0.30416E+00_JPRB, 0.30243E+00_JPRB, 0.31334E+00_JPRB, 0.33490E+00_JPRB, 0.35384E+00_JPRB ,&
&  0.37215E+00_JPRB, 0.38755E+00_JPRB, 0.38387E+00_JPRB, 0.36053E+00_JPRB, 0.29518E+00_JPRB, 0.26090E+00_JPRB, 0.23140E+00_JPRB ,&
&  0.21302E+00_JPRB, 0.19947E+00_JPRB, 0.19055E+00_JPRB, 0.18541E+00_JPRB, 0.18155E+00_JPRB, 0.17923E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17436E+00_JPRB, 0.17279E+00_JPRB, 0.17115E+00_JPRB, 0.17037E+00_JPRB, 0.16952E+00_JPRB, 0.16841E+00_JPRB, 0.16748E+00_JPRB ,&
&  0.16753E+00_JPRB, 0.16730E+00_JPRB, 0.16830E+00_JPRB, 0.16923E+00_JPRB, 0.17107E+00_JPRB, 0.17266E+00_JPRB, 0.17478E+00_JPRB ,&
&  0.17644E+00_JPRB, 0.17776E+00_JPRB, 0.17758E+00_JPRB, 0.17700E+00_JPRB, 0.17609E+00_JPRB, 0.17549E+00_JPRB, 0.17452E+00_JPRB ,&
&  0.17340E+00_JPRB, 0.17344E+00_JPRB, 0.17352E+00_JPRB, 0.17374E+00_JPRB, 0.17466E+00_JPRB, 0.17607E+00_JPRB, 0.17835E+00_JPRB ,&
&  0.18133E+00_JPRB, 0.18448E+00_JPRB, 0.18711E+00_JPRB, 0.18983E+00_JPRB, 0.19338E+00_JPRB, 0.19750E+00_JPRB, 0.20270E+00_JPRB ,&
&  0.20760E+00_JPRB, 0.21556E+00_JPRB, 0.22637E+00_JPRB, 0.23741E+00_JPRB, 0.25295E+00_JPRB, 0.26961E+00_JPRB, 0.29751E+00_JPRB ,&
&  0.34762E+00_JPRB, 0.52322E+00_JPRB, 0.80783E+00_JPRB, 0.57966E+00_JPRB, 0.56630E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.61589E+00_JPRB, 0.62645E+00_JPRB, 0.64045E+00_JPRB, 0.64320E+00_JPRB, 0.65325E+00_JPRB, 0.65212E+00_JPRB, 0.63521E+00_JPRB ,&
&  0.60701E+00_JPRB, 0.58078E+00_JPRB, 0.56633E+00_JPRB, 0.54983E+00_JPRB, 0.52255E+00_JPRB, 0.46182E+00_JPRB, 0.43847E+00_JPRB ,&
&  0.42018E+00_JPRB, 0.41996E+00_JPRB, 0.41105E+00_JPRB, 0.41135E+00_JPRB, 0.40057E+00_JPRB, 0.38939E+00_JPRB, 0.37126E+00_JPRB ,&
&  0.35434E+00_JPRB, 0.33698E+00_JPRB, 0.32515E+00_JPRB, 0.31904E+00_JPRB, 0.31698E+00_JPRB, 0.31290E+00_JPRB, 0.30144E+00_JPRB ,&
&  0.29312E+00_JPRB, 0.29150E+00_JPRB, 0.29427E+00_JPRB, 0.30512E+00_JPRB, 0.34987E+00_JPRB, 0.40566E+00_JPRB, 0.40576E+00_JPRB ,&
&  0.42903E+00_JPRB, 0.44300E+00_JPRB, 0.45654E+00_JPRB, 0.44607E+00_JPRB, 0.41394E+00_JPRB, 0.37635E+00_JPRB, 0.32192E+00_JPRB ,&
&  0.27008E+00_JPRB, 0.24154E+00_JPRB, 0.21842E+00_JPRB, 0.20259E+00_JPRB, 0.19212E+00_JPRB, 0.18768E+00_JPRB, 0.18208E+00_JPRB ,&
&  0.17832E+00_JPRB, 0.17552E+00_JPRB, 0.17435E+00_JPRB, 0.17363E+00_JPRB, 0.17291E+00_JPRB, 0.17097E+00_JPRB, 0.17039E+00_JPRB ,&
&  0.17087E+00_JPRB, 0.17111E+00_JPRB, 0.17104E+00_JPRB, 0.17179E+00_JPRB, 0.17277E+00_JPRB, 0.17412E+00_JPRB, 0.17527E+00_JPRB ,&
&  0.17696E+00_JPRB, 0.17787E+00_JPRB, 0.17940E+00_JPRB, 0.18100E+00_JPRB, 0.18010E+00_JPRB, 0.17886E+00_JPRB, 0.17605E+00_JPRB ,&
&  0.17303E+00_JPRB, 0.17250E+00_JPRB, 0.17187E+00_JPRB, 0.17236E+00_JPRB, 0.17377E+00_JPRB, 0.17578E+00_JPRB, 0.17846E+00_JPRB ,&
&  0.18105E+00_JPRB, 0.18374E+00_JPRB, 0.18583E+00_JPRB, 0.18788E+00_JPRB, 0.19127E+00_JPRB, 0.19494E+00_JPRB, 0.19959E+00_JPRB ,&
&  0.20445E+00_JPRB, 0.20989E+00_JPRB, 0.21750E+00_JPRB, 0.22735E+00_JPRB, 0.24109E+00_JPRB, 0.25697E+00_JPRB, 0.28160E+00_JPRB ,&
&  0.30572E+00_JPRB, 0.40759E+00_JPRB, 0.64483E+00_JPRB, 0.57085E+00_JPRB, 0.58860E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.69061E+00_JPRB, 0.70153E+00_JPRB, 0.71109E+00_JPRB, 0.71270E+00_JPRB, 0.71192E+00_JPRB, 0.70700E+00_JPRB, 0.69501E+00_JPRB ,&
&  0.67538E+00_JPRB, 0.64916E+00_JPRB, 0.62372E+00_JPRB, 0.60343E+00_JPRB, 0.59079E+00_JPRB, 0.59432E+00_JPRB, 0.60335E+00_JPRB ,&
&  0.62131E+00_JPRB, 0.64728E+00_JPRB, 0.65291E+00_JPRB, 0.64672E+00_JPRB, 0.61126E+00_JPRB, 0.57435E+00_JPRB, 0.53985E+00_JPRB ,&
&  0.50938E+00_JPRB, 0.48100E+00_JPRB, 0.47021E+00_JPRB, 0.45157E+00_JPRB, 0.43836E+00_JPRB, 0.42192E+00_JPRB, 0.39896E+00_JPRB ,&
&  0.36142E+00_JPRB, 0.33060E+00_JPRB, 0.31222E+00_JPRB, 0.32702E+00_JPRB, 0.41747E+00_JPRB, 0.47807E+00_JPRB, 0.48664E+00_JPRB ,&
&  0.52158E+00_JPRB, 0.54678E+00_JPRB, 0.54229E+00_JPRB, 0.52280E+00_JPRB, 0.50160E+00_JPRB, 0.47836E+00_JPRB, 0.45944E+00_JPRB ,&
&  0.44064E+00_JPRB, 0.41429E+00_JPRB, 0.36525E+00_JPRB, 0.29575E+00_JPRB, 0.25999E+00_JPRB, 0.24645E+00_JPRB, 0.23200E+00_JPRB ,&
&  0.22109E+00_JPRB, 0.21191E+00_JPRB, 0.20840E+00_JPRB, 0.20541E+00_JPRB, 0.20463E+00_JPRB, 0.20058E+00_JPRB, 0.19721E+00_JPRB ,&
&  0.19326E+00_JPRB, 0.19041E+00_JPRB, 0.18595E+00_JPRB, 0.18153E+00_JPRB, 0.17842E+00_JPRB, 0.17790E+00_JPRB, 0.17759E+00_JPRB ,&
&  0.17852E+00_JPRB, 0.18245E+00_JPRB, 0.19330E+00_JPRB, 0.19936E+00_JPRB, 0.19971E+00_JPRB, 0.19463E+00_JPRB, 0.18752E+00_JPRB ,&
&  0.18128E+00_JPRB, 0.17920E+00_JPRB, 0.17774E+00_JPRB, 0.17819E+00_JPRB, 0.18072E+00_JPRB, 0.18420E+00_JPRB, 0.18718E+00_JPRB ,&
&  0.18955E+00_JPRB, 0.19177E+00_JPRB, 0.19390E+00_JPRB, 0.19619E+00_JPRB, 0.19893E+00_JPRB, 0.20218E+00_JPRB, 0.20662E+00_JPRB ,&
&  0.21144E+00_JPRB, 0.21815E+00_JPRB, 0.22801E+00_JPRB, 0.24217E+00_JPRB, 0.25449E+00_JPRB, 0.26212E+00_JPRB, 0.27220E+00_JPRB ,&
&  0.28787E+00_JPRB, 0.33419E+00_JPRB, 0.46789E+00_JPRB, 0.69565E+00_JPRB, 0.72767E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.75041E+00_JPRB, 0.76818E+00_JPRB, 0.78898E+00_JPRB, 0.80001E+00_JPRB, 0.80101E+00_JPRB, 0.79806E+00_JPRB, 0.78939E+00_JPRB ,&
&  0.77451E+00_JPRB, 0.75654E+00_JPRB, 0.74336E+00_JPRB, 0.73453E+00_JPRB, 0.73080E+00_JPRB, 0.73401E+00_JPRB, 0.73987E+00_JPRB ,&
&  0.73973E+00_JPRB, 0.73611E+00_JPRB, 0.72441E+00_JPRB, 0.70915E+00_JPRB, 0.69302E+00_JPRB, 0.68047E+00_JPRB, 0.67076E+00_JPRB ,&
&  0.66114E+00_JPRB, 0.64643E+00_JPRB, 0.62341E+00_JPRB, 0.58819E+00_JPRB, 0.54664E+00_JPRB, 0.50855E+00_JPRB, 0.47992E+00_JPRB ,&
&  0.45623E+00_JPRB, 0.38860E+00_JPRB, 0.35101E+00_JPRB, 0.36399E+00_JPRB, 0.45845E+00_JPRB, 0.48814E+00_JPRB, 0.51193E+00_JPRB ,&
&  0.54952E+00_JPRB, 0.58946E+00_JPRB, 0.60849E+00_JPRB, 0.60913E+00_JPRB, 0.60376E+00_JPRB, 0.59128E+00_JPRB, 0.57177E+00_JPRB ,&
&  0.54696E+00_JPRB, 0.51652E+00_JPRB, 0.49073E+00_JPRB, 0.46214E+00_JPRB, 0.43255E+00_JPRB, 0.41063E+00_JPRB, 0.39529E+00_JPRB ,&
&  0.38468E+00_JPRB, 0.37524E+00_JPRB, 0.36785E+00_JPRB, 0.35819E+00_JPRB, 0.34902E+00_JPRB, 0.32800E+00_JPRB, 0.30653E+00_JPRB ,&
&  0.28267E+00_JPRB, 0.26555E+00_JPRB, 0.24695E+00_JPRB, 0.22888E+00_JPRB, 0.21406E+00_JPRB, 0.20449E+00_JPRB, 0.19759E+00_JPRB ,&
&  0.19527E+00_JPRB, 0.20058E+00_JPRB, 0.21859E+00_JPRB, 0.25165E+00_JPRB, 0.29134E+00_JPRB, 0.25970E+00_JPRB, 0.23851E+00_JPRB ,&
&  0.21839E+00_JPRB, 0.20653E+00_JPRB, 0.20210E+00_JPRB, 0.20158E+00_JPRB, 0.20285E+00_JPRB, 0.20519E+00_JPRB, 0.20749E+00_JPRB ,&
&  0.21048E+00_JPRB, 0.21325E+00_JPRB, 0.21623E+00_JPRB, 0.21834E+00_JPRB, 0.21971E+00_JPRB, 0.22231E+00_JPRB, 0.22685E+00_JPRB ,&
&  0.23372E+00_JPRB, 0.24523E+00_JPRB, 0.26467E+00_JPRB, 0.28996E+00_JPRB, 0.31593E+00_JPRB, 0.33965E+00_JPRB, 0.39259E+00_JPRB ,&
&  0.47999E+00_JPRB, 0.59275E+00_JPRB, 0.64831E+00_JPRB, 0.70276E+00_JPRB, 0.72954E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.81047E+00_JPRB, 0.83653E+00_JPRB, 0.86561E+00_JPRB, 0.88226E+00_JPRB, 0.88299E+00_JPRB, 0.87971E+00_JPRB, 0.86465E+00_JPRB ,&
&  0.83362E+00_JPRB, 0.80797E+00_JPRB, 0.80350E+00_JPRB, 0.80725E+00_JPRB, 0.81900E+00_JPRB, 0.83774E+00_JPRB, 0.83980E+00_JPRB ,&
&  0.81407E+00_JPRB, 0.79526E+00_JPRB, 0.78603E+00_JPRB, 0.77962E+00_JPRB, 0.76336E+00_JPRB, 0.73929E+00_JPRB, 0.72371E+00_JPRB ,&
&  0.71948E+00_JPRB, 0.71977E+00_JPRB, 0.71098E+00_JPRB, 0.69699E+00_JPRB, 0.66553E+00_JPRB, 0.62681E+00_JPRB, 0.59671E+00_JPRB ,&
&  0.58696E+00_JPRB, 0.57911E+00_JPRB, 0.53798E+00_JPRB, 0.55585E+00_JPRB, 0.55941E+00_JPRB, 0.55967E+00_JPRB, 0.55760E+00_JPRB ,&
&  0.57067E+00_JPRB, 0.60424E+00_JPRB, 0.63721E+00_JPRB, 0.64702E+00_JPRB, 0.65663E+00_JPRB, 0.65847E+00_JPRB, 0.66017E+00_JPRB ,&
&  0.66491E+00_JPRB, 0.65694E+00_JPRB, 0.62732E+00_JPRB, 0.58348E+00_JPRB, 0.54023E+00_JPRB, 0.50655E+00_JPRB, 0.47783E+00_JPRB ,&
&  0.45790E+00_JPRB, 0.44544E+00_JPRB, 0.43577E+00_JPRB, 0.43233E+00_JPRB, 0.43975E+00_JPRB, 0.45452E+00_JPRB, 0.44946E+00_JPRB ,&
&  0.43159E+00_JPRB, 0.41540E+00_JPRB, 0.39374E+00_JPRB, 0.36432E+00_JPRB, 0.33467E+00_JPRB, 0.29556E+00_JPRB, 0.27579E+00_JPRB ,&
&  0.28013E+00_JPRB, 0.31908E+00_JPRB, 0.37060E+00_JPRB, 0.38921E+00_JPRB, 0.41605E+00_JPRB, 0.44205E+00_JPRB, 0.43153E+00_JPRB ,&
&  0.38553E+00_JPRB, 0.31544E+00_JPRB, 0.29253E+00_JPRB, 0.28056E+00_JPRB, 0.27375E+00_JPRB, 0.26871E+00_JPRB, 0.26804E+00_JPRB ,&
&  0.27212E+00_JPRB, 0.28352E+00_JPRB, 0.29267E+00_JPRB, 0.28928E+00_JPRB, 0.27884E+00_JPRB, 0.27208E+00_JPRB, 0.27239E+00_JPRB ,&
&  0.28079E+00_JPRB, 0.30835E+00_JPRB, 0.37199E+00_JPRB, 0.48898E+00_JPRB, 0.52360E+00_JPRB, 0.55591E+00_JPRB, 0.59352E+00_JPRB ,&
&  0.62955E+00_JPRB, 0.66233E+00_JPRB, 0.69047E+00_JPRB, 0.72530E+00_JPRB, 0.77195E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.80524E+00_JPRB, 0.82645E+00_JPRB, 0.84495E+00_JPRB, 0.85748E+00_JPRB, 0.85905E+00_JPRB, 0.85218E+00_JPRB, 0.83640E+00_JPRB ,&
&  0.80657E+00_JPRB, 0.78965E+00_JPRB, 0.79871E+00_JPRB, 0.81341E+00_JPRB, 0.82241E+00_JPRB, 0.82236E+00_JPRB, 0.79097E+00_JPRB ,&
&  0.73725E+00_JPRB, 0.71700E+00_JPRB, 0.71413E+00_JPRB, 0.72393E+00_JPRB, 0.74257E+00_JPRB, 0.75014E+00_JPRB, 0.73870E+00_JPRB ,&
&  0.72690E+00_JPRB, 0.72892E+00_JPRB, 0.73199E+00_JPRB, 0.73898E+00_JPRB, 0.72818E+00_JPRB, 0.70729E+00_JPRB, 0.69853E+00_JPRB ,&
&  0.69630E+00_JPRB, 0.68942E+00_JPRB, 0.68223E+00_JPRB, 0.63332E+00_JPRB, 0.65878E+00_JPRB, 0.64735E+00_JPRB, 0.62941E+00_JPRB ,&
&  0.61324E+00_JPRB, 0.61617E+00_JPRB, 0.65767E+00_JPRB, 0.70381E+00_JPRB, 0.71514E+00_JPRB, 0.71797E+00_JPRB, 0.71349E+00_JPRB ,&
&  0.71034E+00_JPRB, 0.71072E+00_JPRB, 0.71206E+00_JPRB, 0.70385E+00_JPRB, 0.68340E+00_JPRB, 0.65711E+00_JPRB, 0.62820E+00_JPRB ,&
&  0.61053E+00_JPRB, 0.58532E+00_JPRB, 0.54165E+00_JPRB, 0.52132E+00_JPRB, 0.51923E+00_JPRB, 0.52703E+00_JPRB, 0.53083E+00_JPRB ,&
&  0.52777E+00_JPRB, 0.52052E+00_JPRB, 0.52501E+00_JPRB, 0.52252E+00_JPRB, 0.50100E+00_JPRB, 0.48824E+00_JPRB, 0.48241E+00_JPRB ,&
&  0.47079E+00_JPRB, 0.46922E+00_JPRB, 0.47345E+00_JPRB, 0.48463E+00_JPRB, 0.49447E+00_JPRB, 0.50770E+00_JPRB, 0.51285E+00_JPRB ,&
&  0.51482E+00_JPRB, 0.51167E+00_JPRB, 0.50620E+00_JPRB, 0.49450E+00_JPRB, 0.47638E+00_JPRB, 0.46951E+00_JPRB, 0.47091E+00_JPRB ,&
&  0.47948E+00_JPRB, 0.48499E+00_JPRB, 0.48402E+00_JPRB, 0.48264E+00_JPRB, 0.47103E+00_JPRB, 0.45834E+00_JPRB, 0.44844E+00_JPRB ,&
&  0.44358E+00_JPRB, 0.44581E+00_JPRB, 0.46028E+00_JPRB, 0.48441E+00_JPRB, 0.51571E+00_JPRB, 0.55357E+00_JPRB, 0.59796E+00_JPRB ,&
&  0.62969E+00_JPRB, 0.64867E+00_JPRB, 0.67308E+00_JPRB, 0.72022E+00_JPRB, 0.76533E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.76734E+00_JPRB, 0.77546E+00_JPRB, 0.78517E+00_JPRB, 0.79811E+00_JPRB, 0.80970E+00_JPRB, 0.80627E+00_JPRB, 0.79383E+00_JPRB ,&
&  0.77024E+00_JPRB, 0.76170E+00_JPRB, 0.76663E+00_JPRB, 0.76911E+00_JPRB, 0.77082E+00_JPRB, 0.76847E+00_JPRB, 0.75245E+00_JPRB ,&
&  0.71584E+00_JPRB, 0.68649E+00_JPRB, 0.67242E+00_JPRB, 0.66516E+00_JPRB, 0.66289E+00_JPRB, 0.67016E+00_JPRB, 0.67919E+00_JPRB ,&
&  0.69693E+00_JPRB, 0.70784E+00_JPRB, 0.72037E+00_JPRB, 0.72998E+00_JPRB, 0.73273E+00_JPRB, 0.73675E+00_JPRB, 0.73811E+00_JPRB ,&
&  0.72445E+00_JPRB, 0.72511E+00_JPRB, 0.72717E+00_JPRB, 0.71803E+00_JPRB, 0.69732E+00_JPRB, 0.67649E+00_JPRB, 0.65062E+00_JPRB ,&
&  0.63364E+00_JPRB, 0.62524E+00_JPRB, 0.65081E+00_JPRB, 0.71227E+00_JPRB, 0.75524E+00_JPRB, 0.76330E+00_JPRB, 0.75576E+00_JPRB ,&
&  0.75811E+00_JPRB, 0.76563E+00_JPRB, 0.76911E+00_JPRB, 0.75746E+00_JPRB, 0.73764E+00_JPRB, 0.72023E+00_JPRB, 0.70341E+00_JPRB ,&
&  0.69738E+00_JPRB, 0.69608E+00_JPRB, 0.67818E+00_JPRB, 0.64572E+00_JPRB, 0.61815E+00_JPRB, 0.59872E+00_JPRB, 0.58860E+00_JPRB ,&
&  0.58273E+00_JPRB, 0.57357E+00_JPRB, 0.56817E+00_JPRB, 0.57346E+00_JPRB, 0.55387E+00_JPRB, 0.53599E+00_JPRB, 0.53981E+00_JPRB ,&
&  0.55616E+00_JPRB, 0.56134E+00_JPRB, 0.57013E+00_JPRB, 0.57265E+00_JPRB, 0.56889E+00_JPRB, 0.56450E+00_JPRB, 0.56518E+00_JPRB ,&
&  0.57329E+00_JPRB, 0.58260E+00_JPRB, 0.58939E+00_JPRB, 0.59422E+00_JPRB, 0.59922E+00_JPRB, 0.58767E+00_JPRB, 0.56669E+00_JPRB ,&
&  0.54890E+00_JPRB, 0.54526E+00_JPRB, 0.54946E+00_JPRB, 0.54860E+00_JPRB, 0.54685E+00_JPRB, 0.54680E+00_JPRB, 0.54513E+00_JPRB ,&
&  0.54197E+00_JPRB, 0.53422E+00_JPRB, 0.52516E+00_JPRB, 0.52551E+00_JPRB, 0.53699E+00_JPRB, 0.55919E+00_JPRB, 0.59376E+00_JPRB ,&
&  0.64565E+00_JPRB, 0.68511E+00_JPRB, 0.72262E+00_JPRB, 0.73739E+00_JPRB, 0.74024E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.73700E+00_JPRB, 0.74507E+00_JPRB, 0.75081E+00_JPRB, 0.75484E+00_JPRB, 0.76305E+00_JPRB, 0.77337E+00_JPRB, 0.77297E+00_JPRB ,&
&  0.75831E+00_JPRB, 0.74483E+00_JPRB, 0.73556E+00_JPRB, 0.73301E+00_JPRB, 0.73468E+00_JPRB, 0.73684E+00_JPRB, 0.73116E+00_JPRB ,&
&  0.71089E+00_JPRB, 0.68910E+00_JPRB, 0.67573E+00_JPRB, 0.66240E+00_JPRB, 0.65428E+00_JPRB, 0.64923E+00_JPRB, 0.65647E+00_JPRB ,&
&  0.66710E+00_JPRB, 0.68364E+00_JPRB, 0.69724E+00_JPRB, 0.70721E+00_JPRB, 0.71148E+00_JPRB, 0.70059E+00_JPRB, 0.70235E+00_JPRB ,&
&  0.70997E+00_JPRB, 0.71264E+00_JPRB, 0.72822E+00_JPRB, 0.72813E+00_JPRB, 0.71540E+00_JPRB, 0.70533E+00_JPRB, 0.69516E+00_JPRB ,&
&  0.69251E+00_JPRB, 0.70385E+00_JPRB, 0.72924E+00_JPRB, 0.76558E+00_JPRB, 0.78618E+00_JPRB, 0.79381E+00_JPRB, 0.79099E+00_JPRB ,&
&  0.79409E+00_JPRB, 0.80612E+00_JPRB, 0.81279E+00_JPRB, 0.80442E+00_JPRB, 0.78596E+00_JPRB, 0.77071E+00_JPRB, 0.75715E+00_JPRB ,&
&  0.74692E+00_JPRB, 0.73467E+00_JPRB, 0.72644E+00_JPRB, 0.71638E+00_JPRB, 0.69474E+00_JPRB, 0.66907E+00_JPRB, 0.64680E+00_JPRB ,&
&  0.63047E+00_JPRB, 0.61447E+00_JPRB, 0.59489E+00_JPRB, 0.58905E+00_JPRB, 0.59349E+00_JPRB, 0.57054E+00_JPRB, 0.54557E+00_JPRB ,&
&  0.54904E+00_JPRB, 0.57990E+00_JPRB, 0.60385E+00_JPRB, 0.61310E+00_JPRB, 0.61331E+00_JPRB, 0.61709E+00_JPRB, 0.61999E+00_JPRB ,&
&  0.62590E+00_JPRB, 0.63399E+00_JPRB, 0.64013E+00_JPRB, 0.65089E+00_JPRB, 0.66218E+00_JPRB, 0.66074E+00_JPRB, 0.64692E+00_JPRB ,&
&  0.63152E+00_JPRB, 0.62248E+00_JPRB, 0.61832E+00_JPRB, 0.62016E+00_JPRB, 0.62330E+00_JPRB, 0.62103E+00_JPRB, 0.61512E+00_JPRB ,&
&  0.60350E+00_JPRB, 0.58498E+00_JPRB, 0.56923E+00_JPRB, 0.56086E+00_JPRB, 0.56085E+00_JPRB, 0.56769E+00_JPRB, 0.58384E+00_JPRB ,&
&  0.61502E+00_JPRB, 0.66257E+00_JPRB, 0.69723E+00_JPRB, 0.71608E+00_JPRB, 0.72027E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.70949E+00_JPRB, 0.71564E+00_JPRB, 0.72449E+00_JPRB, 0.73732E+00_JPRB, 0.75056E+00_JPRB, 0.75895E+00_JPRB, 0.75814E+00_JPRB ,&
&  0.74716E+00_JPRB, 0.73408E+00_JPRB, 0.72994E+00_JPRB, 0.72958E+00_JPRB, 0.72684E+00_JPRB, 0.72567E+00_JPRB, 0.72152E+00_JPRB ,&
&  0.71110E+00_JPRB, 0.69488E+00_JPRB, 0.68739E+00_JPRB, 0.67401E+00_JPRB, 0.66137E+00_JPRB, 0.65116E+00_JPRB, 0.65171E+00_JPRB ,&
&  0.66081E+00_JPRB, 0.67118E+00_JPRB, 0.68394E+00_JPRB, 0.69868E+00_JPRB, 0.70716E+00_JPRB, 0.70653E+00_JPRB, 0.70399E+00_JPRB ,&
&  0.70203E+00_JPRB, 0.69978E+00_JPRB, 0.71144E+00_JPRB, 0.72093E+00_JPRB, 0.72727E+00_JPRB, 0.73857E+00_JPRB, 0.74761E+00_JPRB ,&
&  0.75493E+00_JPRB, 0.76573E+00_JPRB, 0.78392E+00_JPRB, 0.79735E+00_JPRB, 0.80674E+00_JPRB, 0.81697E+00_JPRB, 0.82090E+00_JPRB ,&
&  0.82735E+00_JPRB, 0.83847E+00_JPRB, 0.84073E+00_JPRB, 0.83159E+00_JPRB, 0.81700E+00_JPRB, 0.80737E+00_JPRB, 0.80169E+00_JPRB ,&
&  0.79200E+00_JPRB, 0.77261E+00_JPRB, 0.75812E+00_JPRB, 0.74676E+00_JPRB, 0.72831E+00_JPRB, 0.70848E+00_JPRB, 0.68516E+00_JPRB ,&
&  0.65852E+00_JPRB, 0.63867E+00_JPRB, 0.62329E+00_JPRB, 0.60943E+00_JPRB, 0.60630E+00_JPRB, 0.59795E+00_JPRB, 0.59133E+00_JPRB ,&
&  0.59585E+00_JPRB, 0.61378E+00_JPRB, 0.62700E+00_JPRB, 0.63109E+00_JPRB, 0.63301E+00_JPRB, 0.64819E+00_JPRB, 0.66673E+00_JPRB ,&
&  0.68106E+00_JPRB, 0.68675E+00_JPRB, 0.68227E+00_JPRB, 0.68413E+00_JPRB, 0.68893E+00_JPRB, 0.69319E+00_JPRB, 0.68740E+00_JPRB ,&
&  0.67900E+00_JPRB, 0.67344E+00_JPRB, 0.67224E+00_JPRB, 0.66869E+00_JPRB, 0.67022E+00_JPRB, 0.67010E+00_JPRB, 0.66727E+00_JPRB ,&
&  0.66147E+00_JPRB, 0.65188E+00_JPRB, 0.63971E+00_JPRB, 0.63284E+00_JPRB, 0.62283E+00_JPRB, 0.60776E+00_JPRB, 0.60909E+00_JPRB ,&
&  0.62062E+00_JPRB, 0.63902E+00_JPRB, 0.65611E+00_JPRB, 0.67598E+00_JPRB, 0.69893E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.68789E+00_JPRB, 0.70006E+00_JPRB, 0.71515E+00_JPRB, 0.73058E+00_JPRB, 0.74002E+00_JPRB, 0.74573E+00_JPRB, 0.74336E+00_JPRB ,&
&  0.73266E+00_JPRB, 0.72205E+00_JPRB, 0.72373E+00_JPRB, 0.72373E+00_JPRB, 0.72300E+00_JPRB, 0.72166E+00_JPRB, 0.71556E+00_JPRB ,&
&  0.70679E+00_JPRB, 0.69948E+00_JPRB, 0.69659E+00_JPRB, 0.69397E+00_JPRB, 0.68449E+00_JPRB, 0.67728E+00_JPRB, 0.66959E+00_JPRB ,&
&  0.66997E+00_JPRB, 0.67319E+00_JPRB, 0.67681E+00_JPRB, 0.67794E+00_JPRB, 0.68683E+00_JPRB, 0.69792E+00_JPRB, 0.70298E+00_JPRB ,&
&  0.69811E+00_JPRB, 0.70457E+00_JPRB, 0.62369E+00_JPRB, 0.69365E+00_JPRB, 0.71920E+00_JPRB, 0.73419E+00_JPRB, 0.74879E+00_JPRB ,&
&  0.75805E+00_JPRB, 0.76867E+00_JPRB, 0.77589E+00_JPRB, 0.77949E+00_JPRB, 0.79567E+00_JPRB, 0.80987E+00_JPRB, 0.81388E+00_JPRB ,&
&  0.81517E+00_JPRB, 0.81441E+00_JPRB, 0.81565E+00_JPRB, 0.81527E+00_JPRB, 0.81913E+00_JPRB, 0.81835E+00_JPRB, 0.81198E+00_JPRB ,&
&  0.80376E+00_JPRB, 0.79227E+00_JPRB, 0.77543E+00_JPRB, 0.75616E+00_JPRB, 0.72547E+00_JPRB, 0.69870E+00_JPRB, 0.68170E+00_JPRB ,&
&  0.66737E+00_JPRB, 0.66300E+00_JPRB, 0.65757E+00_JPRB, 0.65178E+00_JPRB, 0.64941E+00_JPRB, 0.65158E+00_JPRB, 0.65032E+00_JPRB ,&
&  0.65692E+00_JPRB, 0.66569E+00_JPRB, 0.67006E+00_JPRB, 0.67723E+00_JPRB, 0.68643E+00_JPRB, 0.70000E+00_JPRB, 0.71269E+00_JPRB ,&
&  0.72457E+00_JPRB, 0.73248E+00_JPRB, 0.73327E+00_JPRB, 0.73300E+00_JPRB, 0.73073E+00_JPRB, 0.72979E+00_JPRB, 0.72289E+00_JPRB ,&
&  0.71735E+00_JPRB, 0.71447E+00_JPRB, 0.70836E+00_JPRB, 0.69677E+00_JPRB, 0.68997E+00_JPRB, 0.68424E+00_JPRB, 0.68298E+00_JPRB ,&
&  0.68411E+00_JPRB, 0.68515E+00_JPRB, 0.68247E+00_JPRB, 0.68329E+00_JPRB, 0.67717E+00_JPRB, 0.65737E+00_JPRB, 0.64199E+00_JPRB ,&
&  0.64145E+00_JPRB, 0.64686E+00_JPRB, 0.65194E+00_JPRB, 0.66125E+00_JPRB, 0.67592E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.72506E+00_JPRB, 0.73037E+00_JPRB, 0.73895E+00_JPRB, 0.74877E+00_JPRB, 0.75165E+00_JPRB, 0.75080E+00_JPRB, 0.74837E+00_JPRB ,&
&  0.74911E+00_JPRB, 0.74111E+00_JPRB, 0.73531E+00_JPRB, 0.73207E+00_JPRB, 0.73181E+00_JPRB, 0.72934E+00_JPRB, 0.72131E+00_JPRB ,&
&  0.71479E+00_JPRB, 0.71290E+00_JPRB, 0.71826E+00_JPRB, 0.72041E+00_JPRB, 0.70591E+00_JPRB, 0.70149E+00_JPRB, 0.69418E+00_JPRB ,&
&  0.69338E+00_JPRB, 0.69426E+00_JPRB, 0.69797E+00_JPRB, 0.70498E+00_JPRB, 0.72216E+00_JPRB, 0.73456E+00_JPRB, 0.74146E+00_JPRB ,&
&  0.74102E+00_JPRB, 0.66151E+00_JPRB, 0.68917E+00_JPRB, 0.70214E+00_JPRB, 0.72501E+00_JPRB, 0.74722E+00_JPRB, 0.76585E+00_JPRB ,&
&  0.78999E+00_JPRB, 0.80252E+00_JPRB, 0.80679E+00_JPRB, 0.80672E+00_JPRB, 0.80803E+00_JPRB, 0.80832E+00_JPRB, 0.80213E+00_JPRB ,&
&  0.79775E+00_JPRB, 0.80446E+00_JPRB, 0.82023E+00_JPRB, 0.82773E+00_JPRB, 0.83137E+00_JPRB, 0.83324E+00_JPRB, 0.82973E+00_JPRB ,&
&  0.82280E+00_JPRB, 0.80896E+00_JPRB, 0.78604E+00_JPRB, 0.75871E+00_JPRB, 0.72779E+00_JPRB, 0.70681E+00_JPRB, 0.69519E+00_JPRB ,&
&  0.68724E+00_JPRB, 0.68687E+00_JPRB, 0.68658E+00_JPRB, 0.68680E+00_JPRB, 0.68921E+00_JPRB, 0.69685E+00_JPRB, 0.69962E+00_JPRB ,&
&  0.70701E+00_JPRB, 0.71460E+00_JPRB, 0.72680E+00_JPRB, 0.74642E+00_JPRB, 0.76627E+00_JPRB, 0.77753E+00_JPRB, 0.77836E+00_JPRB ,&
&  0.78389E+00_JPRB, 0.78628E+00_JPRB, 0.78237E+00_JPRB, 0.78221E+00_JPRB, 0.77851E+00_JPRB, 0.77982E+00_JPRB, 0.77705E+00_JPRB ,&
&  0.76856E+00_JPRB, 0.76248E+00_JPRB, 0.75522E+00_JPRB, 0.74072E+00_JPRB, 0.72836E+00_JPRB, 0.71964E+00_JPRB, 0.71242E+00_JPRB ,&
&  0.71193E+00_JPRB, 0.71529E+00_JPRB, 0.72037E+00_JPRB, 0.71540E+00_JPRB, 0.70139E+00_JPRB, 0.69320E+00_JPRB, 0.68349E+00_JPRB ,&
&  0.67699E+00_JPRB, 0.67960E+00_JPRB, 0.68042E+00_JPRB, 0.68868E+00_JPRB, 0.70798E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.75384E+00_JPRB, 0.75738E+00_JPRB, 0.76199E+00_JPRB, 0.76258E+00_JPRB, 0.75880E+00_JPRB, 0.75666E+00_JPRB, 0.75858E+00_JPRB ,&
&  0.75993E+00_JPRB, 0.75217E+00_JPRB, 0.74433E+00_JPRB, 0.73882E+00_JPRB, 0.73739E+00_JPRB, 0.73046E+00_JPRB, 0.71965E+00_JPRB ,&
&  0.71228E+00_JPRB, 0.71274E+00_JPRB, 0.71940E+00_JPRB, 0.72358E+00_JPRB, 0.71616E+00_JPRB, 0.70798E+00_JPRB, 0.70565E+00_JPRB ,&
&  0.70725E+00_JPRB, 0.71396E+00_JPRB, 0.72490E+00_JPRB, 0.73192E+00_JPRB, 0.74162E+00_JPRB, 0.75668E+00_JPRB, 0.75895E+00_JPRB ,&
&  0.75886E+00_JPRB, 0.62389E+00_JPRB, 0.66083E+00_JPRB, 0.67558E+00_JPRB, 0.70439E+00_JPRB, 0.74498E+00_JPRB, 0.77739E+00_JPRB ,&
&  0.80732E+00_JPRB, 0.82422E+00_JPRB, 0.83184E+00_JPRB, 0.82541E+00_JPRB, 0.81107E+00_JPRB, 0.79815E+00_JPRB, 0.79432E+00_JPRB ,&
&  0.79248E+00_JPRB, 0.79899E+00_JPRB, 0.81326E+00_JPRB, 0.82339E+00_JPRB, 0.82715E+00_JPRB, 0.82831E+00_JPRB, 0.83091E+00_JPRB ,&
&  0.82813E+00_JPRB, 0.81298E+00_JPRB, 0.79160E+00_JPRB, 0.76589E+00_JPRB, 0.74634E+00_JPRB, 0.73809E+00_JPRB, 0.72644E+00_JPRB ,&
&  0.70986E+00_JPRB, 0.69905E+00_JPRB, 0.69621E+00_JPRB, 0.69809E+00_JPRB, 0.70611E+00_JPRB, 0.71531E+00_JPRB, 0.72349E+00_JPRB ,&
&  0.73309E+00_JPRB, 0.74838E+00_JPRB, 0.76471E+00_JPRB, 0.78056E+00_JPRB, 0.79745E+00_JPRB, 0.81364E+00_JPRB, 0.82652E+00_JPRB ,&
&  0.83523E+00_JPRB, 0.83394E+00_JPRB, 0.82259E+00_JPRB, 0.81349E+00_JPRB, 0.80963E+00_JPRB, 0.81349E+00_JPRB, 0.81379E+00_JPRB ,&
&  0.80592E+00_JPRB, 0.79974E+00_JPRB, 0.79753E+00_JPRB, 0.78800E+00_JPRB, 0.77721E+00_JPRB, 0.76766E+00_JPRB, 0.75771E+00_JPRB ,&
&  0.74973E+00_JPRB, 0.74650E+00_JPRB, 0.74365E+00_JPRB, 0.73792E+00_JPRB, 0.72917E+00_JPRB, 0.72596E+00_JPRB, 0.72361E+00_JPRB ,&
&  0.72118E+00_JPRB, 0.71747E+00_JPRB, 0.71898E+00_JPRB, 0.72918E+00_JPRB, 0.74379E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.76658E+00_JPRB, 0.77370E+00_JPRB, 0.77828E+00_JPRB, 0.78311E+00_JPRB, 0.78367E+00_JPRB, 0.78209E+00_JPRB, 0.78603E+00_JPRB ,&
&  0.79032E+00_JPRB, 0.78562E+00_JPRB, 0.77790E+00_JPRB, 0.77590E+00_JPRB, 0.76796E+00_JPRB, 0.75602E+00_JPRB, 0.74714E+00_JPRB ,&
&  0.73568E+00_JPRB, 0.73246E+00_JPRB, 0.73098E+00_JPRB, 0.72922E+00_JPRB, 0.72406E+00_JPRB, 0.71711E+00_JPRB, 0.72158E+00_JPRB ,&
&  0.72883E+00_JPRB, 0.73641E+00_JPRB, 0.73911E+00_JPRB, 0.74416E+00_JPRB, 0.75127E+00_JPRB, 0.75925E+00_JPRB, 0.77315E+00_JPRB ,&
&  0.66531E+00_JPRB, 0.54203E+00_JPRB, 0.59183E+00_JPRB, 0.64226E+00_JPRB, 0.68361E+00_JPRB, 0.74568E+00_JPRB, 0.78943E+00_JPRB ,&
&  0.82263E+00_JPRB, 0.85236E+00_JPRB, 0.85842E+00_JPRB, 0.83848E+00_JPRB, 0.81779E+00_JPRB, 0.81044E+00_JPRB, 0.80954E+00_JPRB ,&
&  0.80733E+00_JPRB, 0.80879E+00_JPRB, 0.81510E+00_JPRB, 0.82128E+00_JPRB, 0.82718E+00_JPRB, 0.83416E+00_JPRB, 0.83683E+00_JPRB ,&
&  0.83506E+00_JPRB, 0.82331E+00_JPRB, 0.81165E+00_JPRB, 0.79358E+00_JPRB, 0.77804E+00_JPRB, 0.77399E+00_JPRB, 0.76538E+00_JPRB ,&
&  0.75029E+00_JPRB, 0.73984E+00_JPRB, 0.73778E+00_JPRB, 0.74115E+00_JPRB, 0.75051E+00_JPRB, 0.76144E+00_JPRB, 0.77952E+00_JPRB ,&
&  0.79991E+00_JPRB, 0.81466E+00_JPRB, 0.82474E+00_JPRB, 0.83710E+00_JPRB, 0.84408E+00_JPRB, 0.85343E+00_JPRB, 0.87055E+00_JPRB ,&
&  0.88483E+00_JPRB, 0.87610E+00_JPRB, 0.86762E+00_JPRB, 0.85856E+00_JPRB, 0.85500E+00_JPRB, 0.85213E+00_JPRB, 0.84301E+00_JPRB ,&
&  0.83452E+00_JPRB, 0.82979E+00_JPRB, 0.82783E+00_JPRB, 0.83207E+00_JPRB, 0.83059E+00_JPRB, 0.82576E+00_JPRB, 0.82433E+00_JPRB ,&
&  0.82382E+00_JPRB, 0.81708E+00_JPRB, 0.80870E+00_JPRB, 0.80493E+00_JPRB, 0.79714E+00_JPRB, 0.79248E+00_JPRB, 0.78690E+00_JPRB ,&
&  0.78182E+00_JPRB, 0.76638E+00_JPRB, 0.77042E+00_JPRB, 0.76437E+00_JPRB, 0.76146E+00_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.75955E+00_JPRB, 0.78345E+00_JPRB, 0.79004E+00_JPRB, 0.80282E+00_JPRB, 0.81034E+00_JPRB, 0.81483E+00_JPRB, 0.82261E+00_JPRB ,&
&  0.82773E+00_JPRB, 0.82394E+00_JPRB, 0.81785E+00_JPRB, 0.80553E+00_JPRB, 0.78490E+00_JPRB, 0.76267E+00_JPRB, 0.74511E+00_JPRB ,&
&  0.73383E+00_JPRB, 0.73494E+00_JPRB, 0.73020E+00_JPRB, 0.72298E+00_JPRB, 0.71891E+00_JPRB, 0.71715E+00_JPRB, 0.72055E+00_JPRB ,&
&  0.72947E+00_JPRB, 0.73445E+00_JPRB, 0.73046E+00_JPRB, 0.73042E+00_JPRB, 0.74035E+00_JPRB, 0.75766E+00_JPRB, 0.76646E+00_JPRB ,&
&  0.77010E+00_JPRB, 0.57704E+00_JPRB, 0.61447E+00_JPRB, 0.65766E+00_JPRB, 0.68964E+00_JPRB, 0.73888E+00_JPRB, 0.79516E+00_JPRB ,&
&  0.83133E+00_JPRB, 0.85505E+00_JPRB, 0.85908E+00_JPRB, 0.85333E+00_JPRB, 0.83783E+00_JPRB, 0.82388E+00_JPRB, 0.80868E+00_JPRB ,&
&  0.79649E+00_JPRB, 0.79524E+00_JPRB, 0.80212E+00_JPRB, 0.80897E+00_JPRB, 0.81317E+00_JPRB, 0.81879E+00_JPRB, 0.81470E+00_JPRB ,&
&  0.81357E+00_JPRB, 0.81041E+00_JPRB, 0.80804E+00_JPRB, 0.80735E+00_JPRB, 0.80526E+00_JPRB, 0.80328E+00_JPRB, 0.79623E+00_JPRB ,&
&  0.78066E+00_JPRB, 0.76943E+00_JPRB, 0.76320E+00_JPRB, 0.75967E+00_JPRB, 0.76567E+00_JPRB, 0.77899E+00_JPRB, 0.79872E+00_JPRB ,&
&  0.81956E+00_JPRB, 0.82371E+00_JPRB, 0.82968E+00_JPRB, 0.83717E+00_JPRB, 0.84855E+00_JPRB, 0.85992E+00_JPRB, 0.87424E+00_JPRB ,&
&  0.87860E+00_JPRB, 0.87150E+00_JPRB, 0.86553E+00_JPRB, 0.85488E+00_JPRB, 0.85402E+00_JPRB, 0.84060E+00_JPRB, 0.82416E+00_JPRB ,&
&  0.82276E+00_JPRB, 0.82375E+00_JPRB, 0.82547E+00_JPRB, 0.83481E+00_JPRB, 0.84173E+00_JPRB, 0.84247E+00_JPRB, 0.85036E+00_JPRB ,&
&  0.86248E+00_JPRB, 0.87161E+00_JPRB, 0.87080E+00_JPRB, 0.87320E+00_JPRB, 0.86578E+00_JPRB, 0.85898E+00_JPRB, 0.84002E+00_JPRB ,&
&  0.81533E+00_JPRB, 0.79767E+00_JPRB, 0.62105E+00_JPRB, 0.70383E+00_JPRB, 0.72853E+00_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.79083E+00_JPRB, 0.82951E+00_JPRB, 0.84807E+00_JPRB, 0.85054E+00_JPRB, 0.85436E+00_JPRB, 0.85400E+00_JPRB, 0.84868E+00_JPRB ,&
&  0.84583E+00_JPRB, 0.84081E+00_JPRB, 0.82871E+00_JPRB, 0.81055E+00_JPRB, 0.79239E+00_JPRB, 0.77327E+00_JPRB, 0.75983E+00_JPRB ,&
&  0.75905E+00_JPRB, 0.76068E+00_JPRB, 0.75266E+00_JPRB, 0.74492E+00_JPRB, 0.73682E+00_JPRB, 0.73032E+00_JPRB, 0.72881E+00_JPRB ,&
&  0.72596E+00_JPRB, 0.72565E+00_JPRB, 0.73238E+00_JPRB, 0.74092E+00_JPRB, 0.75151E+00_JPRB, 0.76297E+00_JPRB, 0.77810E+00_JPRB ,&
&  0.77956E+00_JPRB, 0.59277E+00_JPRB, 0.61495E+00_JPRB, 0.68270E+00_JPRB, 0.72212E+00_JPRB, 0.77397E+00_JPRB, 0.82598E+00_JPRB ,&
&  0.84997E+00_JPRB, 0.87853E+00_JPRB, 0.87956E+00_JPRB, 0.87270E+00_JPRB, 0.87742E+00_JPRB, 0.86697E+00_JPRB, 0.83599E+00_JPRB ,&
&  0.81262E+00_JPRB, 0.81306E+00_JPRB, 0.82711E+00_JPRB, 0.83811E+00_JPRB, 0.84020E+00_JPRB, 0.83625E+00_JPRB, 0.83351E+00_JPRB ,&
&  0.82870E+00_JPRB, 0.82223E+00_JPRB, 0.81392E+00_JPRB, 0.80253E+00_JPRB, 0.80437E+00_JPRB, 0.81644E+00_JPRB, 0.82241E+00_JPRB ,&
&  0.81919E+00_JPRB, 0.80850E+00_JPRB, 0.79918E+00_JPRB, 0.79470E+00_JPRB, 0.79906E+00_JPRB, 0.81697E+00_JPRB, 0.83622E+00_JPRB ,&
&  0.84608E+00_JPRB, 0.85037E+00_JPRB, 0.85847E+00_JPRB, 0.85945E+00_JPRB, 0.86427E+00_JPRB, 0.86978E+00_JPRB, 0.87974E+00_JPRB ,&
&  0.88358E+00_JPRB, 0.87605E+00_JPRB, 0.86715E+00_JPRB, 0.85480E+00_JPRB, 0.83863E+00_JPRB, 0.82928E+00_JPRB, 0.82713E+00_JPRB ,&
&  0.82439E+00_JPRB, 0.83071E+00_JPRB, 0.84690E+00_JPRB, 0.86700E+00_JPRB, 0.87542E+00_JPRB, 0.87831E+00_JPRB, 0.88238E+00_JPRB ,&
&  0.89172E+00_JPRB, 0.89724E+00_JPRB, 0.90479E+00_JPRB, 0.85897E+00_JPRB, 0.89990E+00_JPRB, 0.90097E+00_JPRB, 0.88297E+00_JPRB ,&
&  0.86019E+00_JPRB, 0.83201E+00_JPRB, 0.82270E+00_JPRB, 0.69128E+00_JPRB, 0.75738E+00_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.85127E+00_JPRB, 0.88551E+00_JPRB, 0.88384E+00_JPRB, 0.88076E+00_JPRB, 0.87508E+00_JPRB, 0.85754E+00_JPRB, 0.83640E+00_JPRB ,&
&  0.81569E+00_JPRB, 0.80545E+00_JPRB, 0.79054E+00_JPRB, 0.77100E+00_JPRB, 0.75999E+00_JPRB, 0.75533E+00_JPRB, 0.75149E+00_JPRB ,&
&  0.75285E+00_JPRB, 0.75363E+00_JPRB, 0.75516E+00_JPRB, 0.75563E+00_JPRB, 0.75083E+00_JPRB, 0.74660E+00_JPRB, 0.73728E+00_JPRB ,&
&  0.73196E+00_JPRB, 0.73371E+00_JPRB, 0.73624E+00_JPRB, 0.74156E+00_JPRB, 0.74961E+00_JPRB, 0.75835E+00_JPRB, 0.76418E+00_JPRB ,&
&  0.75722E+00_JPRB, 0.64125E+00_JPRB, 0.66152E+00_JPRB, 0.71705E+00_JPRB, 0.77621E+00_JPRB, 0.83436E+00_JPRB, 0.85864E+00_JPRB ,&
&  0.87732E+00_JPRB, 0.88852E+00_JPRB, 0.88993E+00_JPRB, 0.88567E+00_JPRB, 0.87601E+00_JPRB, 0.87075E+00_JPRB, 0.84936E+00_JPRB ,&
&  0.83795E+00_JPRB, 0.83763E+00_JPRB, 0.84736E+00_JPRB, 0.85345E+00_JPRB, 0.85285E+00_JPRB, 0.84758E+00_JPRB, 0.84538E+00_JPRB ,&
&  0.83735E+00_JPRB, 0.81422E+00_JPRB, 0.78808E+00_JPRB, 0.76756E+00_JPRB, 0.76076E+00_JPRB, 0.78482E+00_JPRB, 0.81248E+00_JPRB ,&
&  0.83513E+00_JPRB, 0.82693E+00_JPRB, 0.81058E+00_JPRB, 0.80295E+00_JPRB, 0.80554E+00_JPRB, 0.82250E+00_JPRB, 0.84739E+00_JPRB ,&
&  0.85572E+00_JPRB, 0.85738E+00_JPRB, 0.86895E+00_JPRB, 0.87017E+00_JPRB, 0.87150E+00_JPRB, 0.86900E+00_JPRB, 0.86618E+00_JPRB ,&
&  0.85888E+00_JPRB, 0.85068E+00_JPRB, 0.84607E+00_JPRB, 0.84036E+00_JPRB, 0.83947E+00_JPRB, 0.83226E+00_JPRB, 0.82112E+00_JPRB ,&
&  0.80844E+00_JPRB, 0.81864E+00_JPRB, 0.85010E+00_JPRB, 0.87918E+00_JPRB, 0.87916E+00_JPRB, 0.87066E+00_JPRB, 0.86631E+00_JPRB ,&
&  0.87278E+00_JPRB, 0.86808E+00_JPRB, 0.86298E+00_JPRB, 0.90406E+00_JPRB, 0.90112E+00_JPRB, 0.89243E+00_JPRB, 0.88774E+00_JPRB ,&
&  0.88719E+00_JPRB, 0.87435E+00_JPRB, 0.86088E+00_JPRB, 0.84714E+00_JPRB, 0.79226E+00_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.93466E+00_JPRB, 0.92768E+00_JPRB, 0.91573E+00_JPRB, 0.90753E+00_JPRB, 0.89029E+00_JPRB, 0.85964E+00_JPRB, 0.82997E+00_JPRB ,&
&  0.81153E+00_JPRB, 0.79082E+00_JPRB, 0.76012E+00_JPRB, 0.74643E+00_JPRB, 0.74514E+00_JPRB, 0.74848E+00_JPRB, 0.74733E+00_JPRB ,&
&  0.74530E+00_JPRB, 0.75251E+00_JPRB, 0.75116E+00_JPRB, 0.75653E+00_JPRB, 0.75433E+00_JPRB, 0.75703E+00_JPRB, 0.74369E+00_JPRB ,&
&  0.73678E+00_JPRB, 0.73197E+00_JPRB, 0.73154E+00_JPRB, 0.73873E+00_JPRB, 0.74473E+00_JPRB, 0.74911E+00_JPRB, 0.75279E+00_JPRB ,&
&  0.73273E+00_JPRB, 0.63928E+00_JPRB, 0.69929E+00_JPRB, 0.79057E+00_JPRB, 0.84789E+00_JPRB, 0.90865E+00_JPRB, 0.90271E+00_JPRB ,&
&  0.90384E+00_JPRB, 0.90257E+00_JPRB, 0.92018E+00_JPRB, 0.91591E+00_JPRB, 0.89386E+00_JPRB, 0.87719E+00_JPRB, 0.87018E+00_JPRB ,&
&  0.87724E+00_JPRB, 0.87880E+00_JPRB, 0.88305E+00_JPRB, 0.88196E+00_JPRB, 0.87580E+00_JPRB, 0.86793E+00_JPRB, 0.86786E+00_JPRB ,&
&  0.85435E+00_JPRB, 0.82067E+00_JPRB, 0.78637E+00_JPRB, 0.76274E+00_JPRB, 0.76874E+00_JPRB, 0.79366E+00_JPRB, 0.82939E+00_JPRB ,&
&  0.85370E+00_JPRB, 0.85301E+00_JPRB, 0.84598E+00_JPRB, 0.84184E+00_JPRB, 0.84303E+00_JPRB, 0.85800E+00_JPRB, 0.86611E+00_JPRB ,&
&  0.87316E+00_JPRB, 0.86986E+00_JPRB, 0.86831E+00_JPRB, 0.86292E+00_JPRB, 0.85875E+00_JPRB, 0.85772E+00_JPRB, 0.85283E+00_JPRB ,&
&  0.84632E+00_JPRB, 0.84735E+00_JPRB, 0.83822E+00_JPRB, 0.83645E+00_JPRB, 0.82597E+00_JPRB, 0.80974E+00_JPRB, 0.79619E+00_JPRB ,&
&  0.80365E+00_JPRB, 0.82977E+00_JPRB, 0.84862E+00_JPRB, 0.87354E+00_JPRB, 0.88038E+00_JPRB, 0.87040E+00_JPRB, 0.87254E+00_JPRB ,&
&  0.86640E+00_JPRB, 0.85860E+00_JPRB, 0.81468E+00_JPRB, 0.84251E+00_JPRB, 0.93774E+00_JPRB, 0.93033E+00_JPRB, 0.92373E+00_JPRB ,&
&  0.92268E+00_JPRB, 0.90868E+00_JPRB, 0.87513E+00_JPRB, 0.85317E+00_JPRB, 0.91576E+00_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.86306E+00_JPRB, 0.85057E+00_JPRB, 0.85891E+00_JPRB, 0.86323E+00_JPRB, 0.85053E+00_JPRB, 0.82233E+00_JPRB, 0.80046E+00_JPRB ,&
&  0.79453E+00_JPRB, 0.76893E+00_JPRB, 0.74560E+00_JPRB, 0.73130E+00_JPRB, 0.73666E+00_JPRB, 0.74062E+00_JPRB, 0.73963E+00_JPRB ,&
&  0.73009E+00_JPRB, 0.73038E+00_JPRB, 0.72189E+00_JPRB, 0.71834E+00_JPRB, 0.72011E+00_JPRB, 0.72507E+00_JPRB, 0.72264E+00_JPRB ,&
&  0.73152E+00_JPRB, 0.73533E+00_JPRB, 0.74106E+00_JPRB, 0.74880E+00_JPRB, 0.74635E+00_JPRB, 0.75017E+00_JPRB, 0.74043E+00_JPRB ,&
&  0.70969E+00_JPRB, 0.70014E+00_JPRB, 0.80449E+00_JPRB, 0.95549E+00_JPRB, 0.99382E+00_JPRB, 0.99212E+00_JPRB, 0.96542E+00_JPRB ,&
&  0.93889E+00_JPRB, 0.93485E+00_JPRB, 0.95444E+00_JPRB, 0.94649E+00_JPRB, 0.93258E+00_JPRB, 0.92894E+00_JPRB, 0.92372E+00_JPRB ,&
&  0.91631E+00_JPRB, 0.90500E+00_JPRB, 0.90099E+00_JPRB, 0.88620E+00_JPRB, 0.86489E+00_JPRB, 0.85653E+00_JPRB, 0.85854E+00_JPRB ,&
&  0.81909E+00_JPRB, 0.79052E+00_JPRB, 0.76731E+00_JPRB, 0.74678E+00_JPRB, 0.77767E+00_JPRB, 0.84109E+00_JPRB, 0.89582E+00_JPRB ,&
&  0.87865E+00_JPRB, 0.88514E+00_JPRB, 0.88492E+00_JPRB, 0.88327E+00_JPRB, 0.89855E+00_JPRB, 0.88129E+00_JPRB, 0.87326E+00_JPRB ,&
&  0.85449E+00_JPRB, 0.85962E+00_JPRB, 0.86541E+00_JPRB, 0.87624E+00_JPRB, 0.87535E+00_JPRB, 0.87274E+00_JPRB, 0.84439E+00_JPRB ,&
&  0.82998E+00_JPRB, 0.81795E+00_JPRB, 0.81081E+00_JPRB, 0.80534E+00_JPRB, 0.80368E+00_JPRB, 0.78952E+00_JPRB, 0.77866E+00_JPRB ,&
&  0.78834E+00_JPRB, 0.80290E+00_JPRB, 0.85014E+00_JPRB, 0.91067E+00_JPRB, 0.91413E+00_JPRB, 0.89054E+00_JPRB, 0.87435E+00_JPRB ,&
&  0.86367E+00_JPRB, 0.84443E+00_JPRB, 0.81930E+00_JPRB, 0.86561E+00_JPRB, 0.89944E+00_JPRB, 0.97554E+00_JPRB, 0.95864E+00_JPRB ,&
&  0.92441E+00_JPRB, 0.87655E+00_JPRB, 0.85256E+00_JPRB, 0.84372E+00_JPRB, 0.87060E+00_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.81615E+00_JPRB, 0.81623E+00_JPRB, 0.82887E+00_JPRB, 0.83636E+00_JPRB, 0.82327E+00_JPRB, 0.79777E+00_JPRB, 0.77443E+00_JPRB ,&
&  0.76778E+00_JPRB, 0.76751E+00_JPRB, 0.76589E+00_JPRB, 0.74812E+00_JPRB, 0.73297E+00_JPRB, 0.72420E+00_JPRB, 0.72063E+00_JPRB ,&
&  0.72102E+00_JPRB, 0.71210E+00_JPRB, 0.70090E+00_JPRB, 0.68999E+00_JPRB, 0.69585E+00_JPRB, 0.70884E+00_JPRB, 0.72868E+00_JPRB ,&
&  0.72978E+00_JPRB, 0.73132E+00_JPRB, 0.73844E+00_JPRB, 0.74655E+00_JPRB, 0.75664E+00_JPRB, 0.74897E+00_JPRB, 0.73387E+00_JPRB ,&
&  0.71632E+00_JPRB, 0.79973E+00_JPRB, 0.10152E+01_JPRB, 0.11322E+01_JPRB, 0.11151E+01_JPRB, 0.10338E+01_JPRB, 0.99242E+00_JPRB ,&
&  0.95556E+00_JPRB, 0.97475E+00_JPRB, 0.10129E+01_JPRB, 0.10029E+01_JPRB, 0.97911E+00_JPRB, 0.96029E+00_JPRB, 0.94886E+00_JPRB ,&
&  0.92757E+00_JPRB, 0.90079E+00_JPRB, 0.87175E+00_JPRB, 0.86024E+00_JPRB, 0.85140E+00_JPRB, 0.85096E+00_JPRB, 0.83008E+00_JPRB ,&
&  0.80464E+00_JPRB, 0.78269E+00_JPRB, 0.76354E+00_JPRB, 0.74801E+00_JPRB, 0.76450E+00_JPRB, 0.75873E+00_JPRB, 0.84820E+00_JPRB ,&
&  0.10158E+01_JPRB, 0.95766E+00_JPRB, 0.93659E+00_JPRB, 0.94224E+00_JPRB, 0.94339E+00_JPRB, 0.92866E+00_JPRB, 0.91390E+00_JPRB ,&
&  0.91387E+00_JPRB, 0.92655E+00_JPRB, 0.96046E+00_JPRB, 0.96544E+00_JPRB, 0.93046E+00_JPRB, 0.88092E+00_JPRB, 0.85777E+00_JPRB ,&
&  0.85016E+00_JPRB, 0.84110E+00_JPRB, 0.83294E+00_JPRB, 0.82128E+00_JPRB, 0.81021E+00_JPRB, 0.79000E+00_JPRB, 0.78351E+00_JPRB ,&
&  0.78986E+00_JPRB, 0.82016E+00_JPRB, 0.88861E+00_JPRB, 0.91786E+00_JPRB, 0.94382E+00_JPRB, 0.94972E+00_JPRB, 0.89804E+00_JPRB ,&
&  0.83455E+00_JPRB, 0.80413E+00_JPRB, 0.80834E+00_JPRB, 0.84380E+00_JPRB, 0.90572E+00_JPRB, 0.98370E+00_JPRB, 0.94961E+00_JPRB ,&
&  0.89412E+00_JPRB, 0.84244E+00_JPRB, 0.81653E+00_JPRB, 0.80473E+00_JPRB, 0.82648E+00_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.78536E+00_JPRB, 0.77789E+00_JPRB, 0.78015E+00_JPRB, 0.79243E+00_JPRB, 0.78111E+00_JPRB, 0.76355E+00_JPRB, 0.74323E+00_JPRB ,&
&  0.76591E+00_JPRB, 0.77993E+00_JPRB, 0.78689E+00_JPRB, 0.75646E+00_JPRB, 0.73535E+00_JPRB, 0.71083E+00_JPRB, 0.69587E+00_JPRB ,&
&  0.69467E+00_JPRB, 0.68719E+00_JPRB, 0.67722E+00_JPRB, 0.67440E+00_JPRB, 0.68286E+00_JPRB, 0.69369E+00_JPRB, 0.71387E+00_JPRB ,&
&  0.70846E+00_JPRB, 0.71658E+00_JPRB, 0.74144E+00_JPRB, 0.75630E+00_JPRB, 0.76831E+00_JPRB, 0.75093E+00_JPRB, 0.73598E+00_JPRB ,&
&  0.74414E+00_JPRB, 0.77652E+00_JPRB, 0.11575E+01_JPRB, 0.13704E+01_JPRB, 0.11546E+01_JPRB, 0.10154E+01_JPRB, 0.93420E+00_JPRB ,&
&  0.97086E+00_JPRB, 0.10083E+01_JPRB, 0.10423E+01_JPRB, 0.10233E+01_JPRB, 0.10083E+01_JPRB, 0.97444E+00_JPRB, 0.94870E+00_JPRB ,&
&  0.91486E+00_JPRB, 0.88997E+00_JPRB, 0.87488E+00_JPRB, 0.87225E+00_JPRB, 0.86152E+00_JPRB, 0.84342E+00_JPRB, 0.81652E+00_JPRB ,&
&  0.80230E+00_JPRB, 0.78912E+00_JPRB, 0.76917E+00_JPRB, 0.76125E+00_JPRB, 0.78929E+00_JPRB, 0.86410E+00_JPRB, 0.85566E+00_JPRB ,&
&  0.87022E+00_JPRB, 0.10262E+01_JPRB, 0.99789E+00_JPRB, 0.10048E+01_JPRB, 0.99305E+00_JPRB, 0.98618E+00_JPRB, 0.95581E+00_JPRB ,&
&  0.94089E+00_JPRB, 0.96732E+00_JPRB, 0.96303E+00_JPRB, 0.92839E+00_JPRB, 0.89175E+00_JPRB, 0.87590E+00_JPRB, 0.85681E+00_JPRB ,&
&  0.85232E+00_JPRB, 0.86260E+00_JPRB, 0.86080E+00_JPRB, 0.84085E+00_JPRB, 0.82302E+00_JPRB, 0.80559E+00_JPRB, 0.80351E+00_JPRB ,&
&  0.82023E+00_JPRB, 0.84238E+00_JPRB, 0.89858E+00_JPRB, 0.95178E+00_JPRB, 0.93819E+00_JPRB, 0.90432E+00_JPRB, 0.86591E+00_JPRB ,&
&  0.83209E+00_JPRB, 0.80897E+00_JPRB, 0.80379E+00_JPRB, 0.85887E+00_JPRB, 0.88046E+00_JPRB, 0.94228E+00_JPRB, 0.90435E+00_JPRB ,&
&  0.83980E+00_JPRB, 0.79664E+00_JPRB, 0.77808E+00_JPRB, 0.77679E+00_JPRB, 0.78821E+00_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.77802E+00_JPRB, 0.75805E+00_JPRB, 0.76273E+00_JPRB, 0.76750E+00_JPRB, 0.76415E+00_JPRB, 0.74660E+00_JPRB, 0.73634E+00_JPRB ,&
&  0.75715E+00_JPRB, 0.76684E+00_JPRB, 0.76577E+00_JPRB, 0.73873E+00_JPRB, 0.72248E+00_JPRB, 0.69657E+00_JPRB, 0.67688E+00_JPRB ,&
&  0.67678E+00_JPRB, 0.67966E+00_JPRB, 0.67409E+00_JPRB, 0.67583E+00_JPRB, 0.68200E+00_JPRB, 0.70080E+00_JPRB, 0.71404E+00_JPRB ,&
&  0.72693E+00_JPRB, 0.73928E+00_JPRB, 0.75942E+00_JPRB, 0.77227E+00_JPRB, 0.76905E+00_JPRB, 0.75438E+00_JPRB, 0.74546E+00_JPRB ,&
&  0.79987E+00_JPRB, 0.16962E+01_JPRB, 0.11132E+01_JPRB, 0.15045E+01_JPRB, 0.11406E+01_JPRB, 0.96503E+00_JPRB, 0.80264E+00_JPRB ,&
&  0.84356E+00_JPRB, 0.10046E+01_JPRB, 0.10774E+01_JPRB, 0.10443E+01_JPRB, 0.10282E+01_JPRB, 0.10002E+01_JPRB, 0.94728E+00_JPRB ,&
&  0.91223E+00_JPRB, 0.90756E+00_JPRB, 0.88256E+00_JPRB, 0.87260E+00_JPRB, 0.85543E+00_JPRB, 0.83615E+00_JPRB, 0.82576E+00_JPRB ,&
&  0.81402E+00_JPRB, 0.80182E+00_JPRB, 0.78296E+00_JPRB, 0.82369E+00_JPRB, 0.83458E+00_JPRB, 0.93336E+00_JPRB, 0.92791E+00_JPRB ,&
&  0.84440E+00_JPRB, 0.10727E+01_JPRB, 0.10373E+01_JPRB, 0.10127E+01_JPRB, 0.10228E+01_JPRB, 0.99961E+00_JPRB, 0.98009E+00_JPRB ,&
&  0.95257E+00_JPRB, 0.95814E+00_JPRB, 0.91581E+00_JPRB, 0.89524E+00_JPRB, 0.89836E+00_JPRB, 0.89208E+00_JPRB, 0.86410E+00_JPRB ,&
&  0.83549E+00_JPRB, 0.83903E+00_JPRB, 0.85133E+00_JPRB, 0.84670E+00_JPRB, 0.83139E+00_JPRB, 0.80601E+00_JPRB, 0.80312E+00_JPRB ,&
&  0.82457E+00_JPRB, 0.83906E+00_JPRB, 0.87191E+00_JPRB, 0.94466E+00_JPRB, 0.92009E+00_JPRB, 0.86078E+00_JPRB, 0.82107E+00_JPRB ,&
&  0.83194E+00_JPRB, 0.81993E+00_JPRB, 0.82197E+00_JPRB, 0.85256E+00_JPRB, 0.87348E+00_JPRB, 0.90086E+00_JPRB, 0.83324E+00_JPRB ,&
&  0.79795E+00_JPRB, 0.77013E+00_JPRB, 0.76002E+00_JPRB, 0.76288E+00_JPRB, 0.77907E+00_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.80211E+00_JPRB, 0.77432E+00_JPRB, 0.75157E+00_JPRB, 0.75320E+00_JPRB, 0.75583E+00_JPRB, 0.74672E+00_JPRB, 0.73052E+00_JPRB ,&
&  0.72714E+00_JPRB, 0.72108E+00_JPRB, 0.73484E+00_JPRB, 0.72313E+00_JPRB, 0.71450E+00_JPRB, 0.69978E+00_JPRB, 0.68854E+00_JPRB ,&
&  0.68801E+00_JPRB, 0.69355E+00_JPRB, 0.68024E+00_JPRB, 0.68540E+00_JPRB, 0.71437E+00_JPRB, 0.73975E+00_JPRB, 0.74951E+00_JPRB ,&
&  0.76143E+00_JPRB, 0.77533E+00_JPRB, 0.78032E+00_JPRB, 0.78255E+00_JPRB, 0.77175E+00_JPRB, 0.77111E+00_JPRB, 0.77337E+00_JPRB ,&
&  0.84869E+00_JPRB, 0.18203E+01_JPRB, 0.13822E+01_JPRB, 0.15518E+01_JPRB, 0.11589E+01_JPRB, 0.93967E+00_JPRB, 0.75077E+00_JPRB ,&
&  0.77128E+00_JPRB, 0.95566E+00_JPRB, 0.10537E+01_JPRB, 0.10961E+01_JPRB, 0.10467E+01_JPRB, 0.99337E+00_JPRB, 0.94250E+00_JPRB ,&
&  0.91154E+00_JPRB, 0.90762E+00_JPRB, 0.88837E+00_JPRB, 0.87172E+00_JPRB, 0.84602E+00_JPRB, 0.84316E+00_JPRB, 0.83814E+00_JPRB ,&
&  0.82843E+00_JPRB, 0.82446E+00_JPRB, 0.82943E+00_JPRB, 0.83652E+00_JPRB, 0.93918E+00_JPRB, 0.99734E+00_JPRB, 0.93856E+00_JPRB ,&
&  0.89565E+00_JPRB, 0.10854E+01_JPRB, 0.10758E+01_JPRB, 0.10291E+01_JPRB, 0.10183E+01_JPRB, 0.97837E+00_JPRB, 0.97085E+00_JPRB ,&
&  0.93675E+00_JPRB, 0.91399E+00_JPRB, 0.89111E+00_JPRB, 0.88218E+00_JPRB, 0.89589E+00_JPRB, 0.89294E+00_JPRB, 0.85311E+00_JPRB ,&
&  0.82809E+00_JPRB, 0.83745E+00_JPRB, 0.84585E+00_JPRB, 0.84867E+00_JPRB, 0.83534E+00_JPRB, 0.81851E+00_JPRB, 0.82454E+00_JPRB ,&
&  0.84502E+00_JPRB, 0.86611E+00_JPRB, 0.87753E+00_JPRB, 0.92805E+00_JPRB, 0.90337E+00_JPRB, 0.85767E+00_JPRB, 0.81281E+00_JPRB ,&
&  0.82107E+00_JPRB, 0.83141E+00_JPRB, 0.83764E+00_JPRB, 0.84040E+00_JPRB, 0.85539E+00_JPRB, 0.84672E+00_JPRB, 0.79838E+00_JPRB ,&
&  0.79859E+00_JPRB, 0.80350E+00_JPRB, 0.81351E+00_JPRB, 0.80117E+00_JPRB, 0.82139E+00_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.78773E+00_JPRB, 0.74678E+00_JPRB, 0.73690E+00_JPRB, 0.74639E+00_JPRB, 0.73600E+00_JPRB, 0.72006E+00_JPRB, 0.69443E+00_JPRB ,&
&  0.67321E+00_JPRB, 0.69511E+00_JPRB, 0.72560E+00_JPRB, 0.74143E+00_JPRB, 0.74817E+00_JPRB, 0.75343E+00_JPRB, 0.73149E+00_JPRB ,&
&  0.70193E+00_JPRB, 0.70201E+00_JPRB, 0.71702E+00_JPRB, 0.74554E+00_JPRB, 0.76577E+00_JPRB, 0.78597E+00_JPRB, 0.81737E+00_JPRB ,&
&  0.82680E+00_JPRB, 0.82427E+00_JPRB, 0.81406E+00_JPRB, 0.80378E+00_JPRB, 0.79994E+00_JPRB, 0.81155E+00_JPRB, 0.83385E+00_JPRB ,&
&  0.99616E+00_JPRB, 0.23341E+01_JPRB, 0.23313E+01_JPRB, 0.15866E+01_JPRB, 0.11946E+01_JPRB, 0.95221E+00_JPRB, 0.80546E+00_JPRB ,&
&  0.75545E+00_JPRB, 0.75717E+00_JPRB, 0.10754E+01_JPRB, 0.11093E+01_JPRB, 0.10361E+01_JPRB, 0.98678E+00_JPRB, 0.98405E+00_JPRB ,&
&  0.95391E+00_JPRB, 0.94760E+00_JPRB, 0.91320E+00_JPRB, 0.88740E+00_JPRB, 0.86614E+00_JPRB, 0.87045E+00_JPRB, 0.87765E+00_JPRB ,&
&  0.87466E+00_JPRB, 0.88516E+00_JPRB, 0.93759E+00_JPRB, 0.97584E+00_JPRB, 0.95495E+00_JPRB, 0.96765E+00_JPRB, 0.91035E+00_JPRB ,&
&  0.93701E+00_JPRB, 0.10456E+01_JPRB, 0.10587E+01_JPRB, 0.10418E+01_JPRB, 0.95720E+00_JPRB, 0.99602E+00_JPRB, 0.95956E+00_JPRB ,&
&  0.93113E+00_JPRB, 0.88734E+00_JPRB, 0.88632E+00_JPRB, 0.89891E+00_JPRB, 0.90855E+00_JPRB, 0.90827E+00_JPRB, 0.88508E+00_JPRB ,&
&  0.86684E+00_JPRB, 0.86402E+00_JPRB, 0.86042E+00_JPRB, 0.85668E+00_JPRB, 0.85188E+00_JPRB, 0.85277E+00_JPRB, 0.87824E+00_JPRB ,&
&  0.89523E+00_JPRB, 0.92689E+00_JPRB, 0.93584E+00_JPRB, 0.97508E+00_JPRB, 0.95430E+00_JPRB, 0.89225E+00_JPRB, 0.83588E+00_JPRB ,&
&  0.83063E+00_JPRB, 0.85950E+00_JPRB, 0.87348E+00_JPRB, 0.87123E+00_JPRB, 0.85459E+00_JPRB, 0.82657E+00_JPRB, 0.82278E+00_JPRB ,&
&  0.84055E+00_JPRB, 0.86937E+00_JPRB, 0.86693E+00_JPRB, 0.82019E+00_JPRB, 0.83319E+00_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.76416E+00_JPRB, 0.72737E+00_JPRB, 0.71778E+00_JPRB, 0.70793E+00_JPRB, 0.68962E+00_JPRB, 0.68113E+00_JPRB, 0.65936E+00_JPRB ,&
&  0.67783E+00_JPRB, 0.73736E+00_JPRB, 0.80662E+00_JPRB, 0.83317E+00_JPRB, 0.84721E+00_JPRB, 0.84361E+00_JPRB, 0.81313E+00_JPRB ,&
&  0.77474E+00_JPRB, 0.77726E+00_JPRB, 0.79194E+00_JPRB, 0.80189E+00_JPRB, 0.81900E+00_JPRB, 0.83293E+00_JPRB, 0.85856E+00_JPRB ,&
&  0.87981E+00_JPRB, 0.86111E+00_JPRB, 0.83612E+00_JPRB, 0.82745E+00_JPRB, 0.86424E+00_JPRB, 0.91496E+00_JPRB, 0.97754E+00_JPRB ,&
&  0.13611E+01_JPRB, 0.26353E+01_JPRB, 0.14414E+01_JPRB, 0.12579E+01_JPRB, 0.11416E+01_JPRB, 0.99716E+00_JPRB, 0.88182E+00_JPRB ,&
&  0.85067E+00_JPRB, 0.78101E+00_JPRB, 0.92765E+00_JPRB, 0.10901E+01_JPRB, 0.10484E+01_JPRB, 0.10103E+01_JPRB, 0.10072E+01_JPRB ,&
&  0.99933E+00_JPRB, 0.98715E+00_JPRB, 0.97397E+00_JPRB, 0.93599E+00_JPRB, 0.92661E+00_JPRB, 0.92173E+00_JPRB, 0.92823E+00_JPRB ,&
&  0.93605E+00_JPRB, 0.95078E+00_JPRB, 0.10143E+01_JPRB, 0.97238E+00_JPRB, 0.99494E+00_JPRB, 0.94012E+00_JPRB, 0.80274E+00_JPRB ,&
&  0.74738E+00_JPRB, 0.88909E+00_JPRB, 0.10128E+01_JPRB, 0.10968E+01_JPRB, 0.10663E+01_JPRB, 0.92564E+00_JPRB, 0.98536E+00_JPRB ,&
&  0.91270E+00_JPRB, 0.90177E+00_JPRB, 0.91397E+00_JPRB, 0.91655E+00_JPRB, 0.93823E+00_JPRB, 0.93659E+00_JPRB, 0.93910E+00_JPRB ,&
&  0.92136E+00_JPRB, 0.90439E+00_JPRB, 0.89268E+00_JPRB, 0.88283E+00_JPRB, 0.87733E+00_JPRB, 0.89269E+00_JPRB, 0.92286E+00_JPRB ,&
&  0.94986E+00_JPRB, 0.99516E+00_JPRB, 0.10462E+01_JPRB, 0.10625E+01_JPRB, 0.10432E+01_JPRB, 0.98279E+00_JPRB, 0.91267E+00_JPRB ,&
&  0.89550E+00_JPRB, 0.90917E+00_JPRB, 0.88426E+00_JPRB, 0.85337E+00_JPRB, 0.86215E+00_JPRB, 0.86085E+00_JPRB, 0.91594E+00_JPRB ,&
&  0.10034E+01_JPRB, 0.11429E+01_JPRB, 0.12452E+01_JPRB, 0.80585E+00_JPRB, 0.80035E+00_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.69390E+00_JPRB, 0.69135E+00_JPRB, 0.66516E+00_JPRB, 0.64621E+00_JPRB, 0.63925E+00_JPRB, 0.66116E+00_JPRB, 0.70632E+00_JPRB ,&
&  0.77730E+00_JPRB, 0.86612E+00_JPRB, 0.94227E+00_JPRB, 0.96615E+00_JPRB, 0.95251E+00_JPRB, 0.95387E+00_JPRB, 0.90003E+00_JPRB ,&
&  0.84366E+00_JPRB, 0.83273E+00_JPRB, 0.84418E+00_JPRB, 0.85387E+00_JPRB, 0.86664E+00_JPRB, 0.88689E+00_JPRB, 0.91098E+00_JPRB ,&
&  0.92389E+00_JPRB, 0.93098E+00_JPRB, 0.93558E+00_JPRB, 0.98586E+00_JPRB, 0.10663E+01_JPRB, 0.11912E+01_JPRB, 0.14142E+01_JPRB ,&
&  0.19336E+01_JPRB, 0.18559E+01_JPRB, 0.10143E+01_JPRB, 0.10713E+01_JPRB, 0.93224E+00_JPRB, 0.94347E+00_JPRB, 0.89889E+00_JPRB ,&
&  0.93293E+00_JPRB, 0.92328E+00_JPRB, 0.10218E+01_JPRB, 0.11304E+01_JPRB, 0.11084E+01_JPRB, 0.10961E+01_JPRB, 0.10751E+01_JPRB ,&
&  0.10679E+01_JPRB, 0.10429E+01_JPRB, 0.10187E+01_JPRB, 0.98750E+00_JPRB, 0.97420E+00_JPRB, 0.97663E+00_JPRB, 0.98070E+00_JPRB ,&
&  0.10140E+01_JPRB, 0.10286E+01_JPRB, 0.11017E+01_JPRB, 0.99829E+00_JPRB, 0.10394E+01_JPRB, 0.93002E+00_JPRB, 0.76332E+00_JPRB ,&
&  0.70582E+00_JPRB, 0.81117E+00_JPRB, 0.99177E+00_JPRB, 0.11411E+01_JPRB, 0.11254E+01_JPRB, 0.92344E+00_JPRB, 0.96431E+00_JPRB ,&
&  0.91070E+00_JPRB, 0.91108E+00_JPRB, 0.91314E+00_JPRB, 0.92941E+00_JPRB, 0.96121E+00_JPRB, 0.96425E+00_JPRB, 0.96935E+00_JPRB ,&
&  0.94144E+00_JPRB, 0.91787E+00_JPRB, 0.90769E+00_JPRB, 0.90450E+00_JPRB, 0.91036E+00_JPRB, 0.94275E+00_JPRB, 0.96234E+00_JPRB ,&
&  0.98665E+00_JPRB, 0.10241E+01_JPRB, 0.10708E+01_JPRB, 0.11222E+01_JPRB, 0.11136E+01_JPRB, 0.10542E+01_JPRB, 0.97856E+00_JPRB ,&
&  0.95807E+00_JPRB, 0.96305E+00_JPRB, 0.89694E+00_JPRB, 0.85500E+00_JPRB, 0.82307E+00_JPRB, 0.84381E+00_JPRB, 0.83907E+00_JPRB ,&
&  0.84083E+00_JPRB, 0.84322E+00_JPRB, 0.80420E+00_JPRB, 0.69744E+00_JPRB, 0.69785E+00_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.66967E+00_JPRB, 0.67251E+00_JPRB, 0.66452E+00_JPRB, 0.68040E+00_JPRB, 0.68193E+00_JPRB, 0.74295E+00_JPRB, 0.81138E+00_JPRB ,&
&  0.91492E+00_JPRB, 0.98983E+00_JPRB, 0.10340E+01_JPRB, 0.10389E+01_JPRB, 0.10160E+01_JPRB, 0.95368E+00_JPRB, 0.89535E+00_JPRB ,&
&  0.88989E+00_JPRB, 0.89093E+00_JPRB, 0.90276E+00_JPRB, 0.92187E+00_JPRB, 0.93912E+00_JPRB, 0.96733E+00_JPRB, 0.99458E+00_JPRB ,&
&  0.10280E+01_JPRB, 0.10769E+01_JPRB, 0.11335E+01_JPRB, 0.12162E+01_JPRB, 0.13193E+01_JPRB, 0.14610E+01_JPRB, 0.15966E+01_JPRB ,&
&  0.11998E+01_JPRB, 0.61562E+00_JPRB, 0.87597E+00_JPRB, 0.92143E+00_JPRB, 0.86247E+00_JPRB, 0.97190E+00_JPRB, 0.11176E+01_JPRB ,&
&  0.10270E+01_JPRB, 0.99944E+00_JPRB, 0.10050E+01_JPRB, 0.11634E+01_JPRB, 0.11510E+01_JPRB, 0.11395E+01_JPRB, 0.11283E+01_JPRB ,&
&  0.11058E+01_JPRB, 0.10770E+01_JPRB, 0.10485E+01_JPRB, 0.10424E+01_JPRB, 0.10323E+01_JPRB, 0.10431E+01_JPRB, 0.10517E+01_JPRB ,&
&  0.10752E+01_JPRB, 0.11177E+01_JPRB, 0.12020E+01_JPRB, 0.99609E+00_JPRB, 0.10309E+01_JPRB, 0.97478E+00_JPRB, 0.77238E+00_JPRB ,&
&  0.73695E+00_JPRB, 0.74882E+00_JPRB, 0.83716E+00_JPRB, 0.11295E+01_JPRB, 0.11545E+01_JPRB, 0.93433E+00_JPRB, 0.96494E+00_JPRB ,&
&  0.92364E+00_JPRB, 0.95039E+00_JPRB, 0.92737E+00_JPRB, 0.97754E+00_JPRB, 0.99947E+00_JPRB, 0.10110E+01_JPRB, 0.97966E+00_JPRB ,&
&  0.94751E+00_JPRB, 0.91422E+00_JPRB, 0.90157E+00_JPRB, 0.90849E+00_JPRB, 0.91635E+00_JPRB, 0.94764E+00_JPRB, 0.95568E+00_JPRB ,&
&  0.96328E+00_JPRB, 0.98983E+00_JPRB, 0.10387E+01_JPRB, 0.10891E+01_JPRB, 0.11178E+01_JPRB, 0.10667E+01_JPRB, 0.10201E+01_JPRB ,&
&  0.98544E+00_JPRB, 0.95153E+00_JPRB, 0.87562E+00_JPRB, 0.78357E+00_JPRB, 0.77481E+00_JPRB, 0.73308E+00_JPRB, 0.68096E+00_JPRB ,&
&  0.69018E+00_JPRB, 0.68686E+00_JPRB, 0.67061E+00_JPRB, 0.66767E+00_JPRB, 0.67246E+00_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.75339E+00_JPRB, 0.74497E+00_JPRB, 0.73685E+00_JPRB, 0.73893E+00_JPRB, 0.74685E+00_JPRB, 0.79268E+00_JPRB, 0.86671E+00_JPRB ,&
&  0.93608E+00_JPRB, 0.97666E+00_JPRB, 0.10133E+01_JPRB, 0.10268E+01_JPRB, 0.10237E+01_JPRB, 0.97448E+00_JPRB, 0.96343E+00_JPRB ,&
&  0.94950E+00_JPRB, 0.94291E+00_JPRB, 0.95858E+00_JPRB, 0.98601E+00_JPRB, 0.10076E+01_JPRB, 0.10313E+01_JPRB, 0.10543E+01_JPRB ,&
&  0.10848E+01_JPRB, 0.11331E+01_JPRB, 0.11792E+01_JPRB, 0.12555E+01_JPRB, 0.13023E+01_JPRB, 0.13116E+01_JPRB, 0.12360E+01_JPRB ,&
&  0.56239E+00_JPRB, 0.67343E+00_JPRB, 0.10582E+01_JPRB, 0.10484E+01_JPRB, 0.10959E+01_JPRB, 0.11507E+01_JPRB, 0.12096E+01_JPRB ,&
&  0.11227E+01_JPRB, 0.99298E+00_JPRB, 0.10142E+01_JPRB, 0.11940E+01_JPRB, 0.11708E+01_JPRB, 0.11830E+01_JPRB, 0.11755E+01_JPRB ,&
&  0.11505E+01_JPRB, 0.11229E+01_JPRB, 0.10981E+01_JPRB, 0.10890E+01_JPRB, 0.10930E+01_JPRB, 0.11017E+01_JPRB, 0.11200E+01_JPRB ,&
&  0.11298E+01_JPRB, 0.11651E+01_JPRB, 0.12240E+01_JPRB, 0.99892E+00_JPRB, 0.99911E+00_JPRB, 0.10127E+01_JPRB, 0.86049E+00_JPRB ,&
&  0.67078E+00_JPRB, 0.65040E+00_JPRB, 0.78863E+00_JPRB, 0.10880E+01_JPRB, 0.11022E+01_JPRB, 0.11404E+01_JPRB, 0.96271E+00_JPRB ,&
&  0.94810E+00_JPRB, 0.90006E+00_JPRB, 0.91044E+00_JPRB, 0.95851E+00_JPRB, 0.99355E+00_JPRB, 0.10110E+01_JPRB, 0.98129E+00_JPRB ,&
&  0.92441E+00_JPRB, 0.89120E+00_JPRB, 0.87496E+00_JPRB, 0.87864E+00_JPRB, 0.89649E+00_JPRB, 0.90911E+00_JPRB, 0.90298E+00_JPRB ,&
&  0.89747E+00_JPRB, 0.90334E+00_JPRB, 0.94537E+00_JPRB, 0.97368E+00_JPRB, 0.10074E+01_JPRB, 0.10458E+01_JPRB, 0.10812E+01_JPRB ,&
&  0.10512E+01_JPRB, 0.10478E+01_JPRB, 0.11517E+01_JPRB, 0.10752E+01_JPRB, 0.97839E+00_JPRB, 0.84282E+00_JPRB, 0.70871E+00_JPRB ,&
&  0.72418E+00_JPRB, 0.73223E+00_JPRB, 0.72777E+00_JPRB, 0.73373E+00_JPRB, 0.75069E+00_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.84051E+00_JPRB, 0.82332E+00_JPRB, 0.81057E+00_JPRB, 0.79878E+00_JPRB, 0.81022E+00_JPRB, 0.84286E+00_JPRB, 0.87965E+00_JPRB ,&
&  0.91471E+00_JPRB, 0.95507E+00_JPRB, 0.10106E+01_JPRB, 0.10239E+01_JPRB, 0.10559E+01_JPRB, 0.10471E+01_JPRB, 0.10270E+01_JPRB ,&
&  0.99513E+00_JPRB, 0.10013E+01_JPRB, 0.10165E+01_JPRB, 0.10338E+01_JPRB, 0.10335E+01_JPRB, 0.10476E+01_JPRB, 0.10662E+01_JPRB ,&
&  0.10971E+01_JPRB, 0.11254E+01_JPRB, 0.11675E+01_JPRB, 0.12469E+01_JPRB, 0.12582E+01_JPRB, 0.12391E+01_JPRB, 0.12211E+01_JPRB ,&
&  0.65910E+00_JPRB, 0.10840E+01_JPRB, 0.11698E+01_JPRB, 0.11380E+01_JPRB, 0.11789E+01_JPRB, 0.11499E+01_JPRB, 0.11116E+01_JPRB ,&
&  0.11184E+01_JPRB, 0.10801E+01_JPRB, 0.10693E+01_JPRB, 0.11401E+01_JPRB, 0.12000E+01_JPRB, 0.12140E+01_JPRB, 0.12083E+01_JPRB ,&
&  0.11850E+01_JPRB, 0.11580E+01_JPRB, 0.11524E+01_JPRB, 0.11732E+01_JPRB, 0.11793E+01_JPRB, 0.11904E+01_JPRB, 0.12105E+01_JPRB ,&
&  0.12021E+01_JPRB, 0.11975E+01_JPRB, 0.12502E+01_JPRB, 0.11211E+01_JPRB, 0.10340E+01_JPRB, 0.10601E+01_JPRB, 0.95955E+00_JPRB ,&
&  0.65887E+00_JPRB, 0.54931E+00_JPRB, 0.73370E+00_JPRB, 0.97704E+00_JPRB, 0.90954E+00_JPRB, 0.93012E+00_JPRB, 0.87690E+00_JPRB ,&
&  0.78533E+00_JPRB, 0.75893E+00_JPRB, 0.82825E+00_JPRB, 0.89178E+00_JPRB, 0.94360E+00_JPRB, 0.98188E+00_JPRB, 0.96123E+00_JPRB ,&
&  0.90886E+00_JPRB, 0.86575E+00_JPRB, 0.83293E+00_JPRB, 0.82574E+00_JPRB, 0.84743E+00_JPRB, 0.85818E+00_JPRB, 0.84432E+00_JPRB ,&
&  0.84013E+00_JPRB, 0.83221E+00_JPRB, 0.87606E+00_JPRB, 0.90973E+00_JPRB, 0.95800E+00_JPRB, 0.10162E+01_JPRB, 0.11611E+01_JPRB ,&
&  0.14021E+01_JPRB, 0.16266E+01_JPRB, 0.19037E+01_JPRB, 0.18750E+01_JPRB, 0.17109E+01_JPRB, 0.12240E+01_JPRB, 0.81601E+00_JPRB ,&
&  0.83256E+00_JPRB, 0.85145E+00_JPRB, 0.85285E+00_JPRB, 0.85993E+00_JPRB, 0.84997E+00_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.97177E+00_JPRB, 0.95517E+00_JPRB, 0.95102E+00_JPRB, 0.94660E+00_JPRB, 0.94409E+00_JPRB, 0.95130E+00_JPRB, 0.97310E+00_JPRB ,&
&  0.98211E+00_JPRB, 0.10153E+01_JPRB, 0.10652E+01_JPRB, 0.10943E+01_JPRB, 0.11321E+01_JPRB, 0.11273E+01_JPRB, 0.10869E+01_JPRB ,&
&  0.10743E+01_JPRB, 0.11005E+01_JPRB, 0.11242E+01_JPRB, 0.11297E+01_JPRB, 0.10868E+01_JPRB, 0.10562E+01_JPRB, 0.10541E+01_JPRB ,&
&  0.10755E+01_JPRB, 0.10931E+01_JPRB, 0.11111E+01_JPRB, 0.11457E+01_JPRB, 0.11378E+01_JPRB, 0.10574E+01_JPRB, 0.70732E+00_JPRB ,&
&  0.10361E+01_JPRB, 0.12304E+01_JPRB, 0.12420E+01_JPRB, 0.11729E+01_JPRB, 0.11361E+01_JPRB, 0.10643E+01_JPRB, 0.10591E+01_JPRB ,&
&  0.10976E+01_JPRB, 0.11399E+01_JPRB, 0.11201E+01_JPRB, 0.10663E+01_JPRB, 0.11684E+01_JPRB, 0.11711E+01_JPRB, 0.11713E+01_JPRB ,&
&  0.11651E+01_JPRB, 0.11624E+01_JPRB, 0.11656E+01_JPRB, 0.11724E+01_JPRB, 0.11704E+01_JPRB, 0.11785E+01_JPRB, 0.11773E+01_JPRB ,&
&  0.11537E+01_JPRB, 0.11269E+01_JPRB, 0.11452E+01_JPRB, 0.10644E+01_JPRB, 0.10367E+01_JPRB, 0.10602E+01_JPRB, 0.92640E+00_JPRB ,&
&  0.60227E+00_JPRB, 0.51117E+00_JPRB, 0.72639E+00_JPRB, 0.93659E+00_JPRB, 0.83413E+00_JPRB, 0.79646E+00_JPRB, 0.80958E+00_JPRB ,&
&  0.77188E+00_JPRB, 0.71617E+00_JPRB, 0.70218E+00_JPRB, 0.75197E+00_JPRB, 0.82388E+00_JPRB, 0.87955E+00_JPRB, 0.89390E+00_JPRB ,&
&  0.86080E+00_JPRB, 0.83326E+00_JPRB, 0.82820E+00_JPRB, 0.84037E+00_JPRB, 0.86855E+00_JPRB, 0.93807E+00_JPRB, 0.10371E+01_JPRB ,&
&  0.97551E+00_JPRB, 0.78735E+00_JPRB, 0.90983E+00_JPRB, 0.97842E+00_JPRB, 0.11024E+01_JPRB, 0.12723E+01_JPRB, 0.15711E+01_JPRB ,&
&  0.20447E+01_JPRB, 0.25590E+01_JPRB, 0.25315E+01_JPRB, 0.21196E+01_JPRB, 0.26858E+01_JPRB, 0.17885E+01_JPRB, 0.93971E+00_JPRB ,&
&  0.96610E+00_JPRB, 0.98339E+00_JPRB, 0.99861E+00_JPRB, 0.99069E+00_JPRB, 0.98371E+00_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.99342E+00_JPRB, 0.99396E+00_JPRB, 0.98450E+00_JPRB, 0.10125E+01_JPRB, 0.10079E+01_JPRB, 0.10211E+01_JPRB, 0.10206E+01_JPRB ,&
&  0.10418E+01_JPRB, 0.10589E+01_JPRB, 0.10789E+01_JPRB, 0.11137E+01_JPRB, 0.11192E+01_JPRB, 0.11320E+01_JPRB, 0.11580E+01_JPRB ,&
&  0.11833E+01_JPRB, 0.12162E+01_JPRB, 0.12113E+01_JPRB, 0.11928E+01_JPRB, 0.11531E+01_JPRB, 0.11235E+01_JPRB, 0.10887E+01_JPRB ,&
&  0.10603E+01_JPRB, 0.10520E+01_JPRB, 0.10514E+01_JPRB, 0.10587E+01_JPRB, 0.10596E+01_JPRB, 0.10690E+01_JPRB, 0.71777E+00_JPRB ,&
&  0.11387E+01_JPRB, 0.12753E+01_JPRB, 0.11926E+01_JPRB, 0.11182E+01_JPRB, 0.10317E+01_JPRB, 0.10288E+01_JPRB, 0.10422E+01_JPRB ,&
&  0.10857E+01_JPRB, 0.11303E+01_JPRB, 0.11190E+01_JPRB, 0.10884E+01_JPRB, 0.10611E+01_JPRB, 0.10695E+01_JPRB, 0.10598E+01_JPRB ,&
&  0.10536E+01_JPRB, 0.10466E+01_JPRB, 0.10363E+01_JPRB, 0.10209E+01_JPRB, 0.10076E+01_JPRB, 0.10045E+01_JPRB, 0.98577E+00_JPRB ,&
&  0.95723E+00_JPRB, 0.92917E+00_JPRB, 0.93539E+00_JPRB, 0.93966E+00_JPRB, 0.92879E+00_JPRB, 0.83130E+00_JPRB, 0.67558E+00_JPRB ,&
&  0.47027E+00_JPRB, 0.48219E+00_JPRB, 0.67190E+00_JPRB, 0.93076E+00_JPRB, 0.85201E+00_JPRB, 0.77718E+00_JPRB, 0.72879E+00_JPRB ,&
&  0.71893E+00_JPRB, 0.67645E+00_JPRB, 0.62434E+00_JPRB, 0.61760E+00_JPRB, 0.64005E+00_JPRB, 0.69123E+00_JPRB, 0.69655E+00_JPRB ,&
&  0.70996E+00_JPRB, 0.70561E+00_JPRB, 0.68326E+00_JPRB, 0.66635E+00_JPRB, 0.69685E+00_JPRB, 0.69370E+00_JPRB, 0.61237E+00_JPRB ,&
&  0.71958E+00_JPRB, 0.67972E+00_JPRB, 0.72816E+00_JPRB, 0.82170E+00_JPRB, 0.90359E+00_JPRB, 0.10925E+01_JPRB, 0.13894E+01_JPRB ,&
&  0.18406E+01_JPRB, 0.18222E+01_JPRB, 0.24361E+01_JPRB, 0.37346E+01_JPRB, 0.29247E+01_JPRB, 0.20658E+01_JPRB, 0.10940E+01_JPRB ,&
&  0.10749E+01_JPRB, 0.10430E+01_JPRB, 0.10468E+01_JPRB, 0.10233E+01_JPRB, 0.10047E+01_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.90864E+00_JPRB, 0.91734E+00_JPRB, 0.93346E+00_JPRB, 0.95775E+00_JPRB, 0.96961E+00_JPRB, 0.98994E+00_JPRB, 0.99746E+00_JPRB ,&
&  0.10072E+01_JPRB, 0.10349E+01_JPRB, 0.10606E+01_JPRB, 0.10889E+01_JPRB, 0.11134E+01_JPRB, 0.11377E+01_JPRB, 0.11603E+01_JPRB ,&
&  0.11797E+01_JPRB, 0.11960E+01_JPRB, 0.12010E+01_JPRB, 0.11915E+01_JPRB, 0.11685E+01_JPRB, 0.11550E+01_JPRB, 0.11320E+01_JPRB ,&
&  0.11133E+01_JPRB, 0.11124E+01_JPRB, 0.11181E+01_JPRB, 0.11203E+01_JPRB, 0.11238E+01_JPRB, 0.11600E+01_JPRB, 0.81346E+00_JPRB ,&
&  0.11823E+01_JPRB, 0.12475E+01_JPRB, 0.11792E+01_JPRB, 0.10277E+01_JPRB, 0.95190E+00_JPRB, 0.94050E+00_JPRB, 0.95114E+00_JPRB ,&
&  0.98723E+00_JPRB, 0.10414E+01_JPRB, 0.10717E+01_JPRB, 0.98889E+00_JPRB, 0.91109E+00_JPRB, 0.91563E+00_JPRB, 0.90156E+00_JPRB ,&
&  0.90166E+00_JPRB, 0.89426E+00_JPRB, 0.88383E+00_JPRB, 0.86629E+00_JPRB, 0.84801E+00_JPRB, 0.82683E+00_JPRB, 0.80117E+00_JPRB ,&
&  0.76638E+00_JPRB, 0.72944E+00_JPRB, 0.67038E+00_JPRB, 0.67017E+00_JPRB, 0.66106E+00_JPRB, 0.58506E+00_JPRB, 0.47101E+00_JPRB ,&
&  0.40643E+00_JPRB, 0.53619E+00_JPRB, 0.68217E+00_JPRB, 0.93665E+00_JPRB, 0.86964E+00_JPRB, 0.82099E+00_JPRB, 0.76118E+00_JPRB ,&
&  0.66376E+00_JPRB, 0.61838E+00_JPRB, 0.57851E+00_JPRB, 0.56457E+00_JPRB, 0.56328E+00_JPRB, 0.57700E+00_JPRB, 0.57215E+00_JPRB ,&
&  0.55393E+00_JPRB, 0.52501E+00_JPRB, 0.51471E+00_JPRB, 0.50936E+00_JPRB, 0.51859E+00_JPRB, 0.45496E+00_JPRB, 0.56715E+00_JPRB ,&
&  0.56145E+00_JPRB, 0.50019E+00_JPRB, 0.65136E+00_JPRB, 0.48382E+00_JPRB, 0.63669E+00_JPRB, 0.72833E+00_JPRB, 0.85143E+00_JPRB ,&
&  0.89290E+00_JPRB, 0.11897E+01_JPRB, 0.14973E+01_JPRB, 0.13200E+01_JPRB, 0.11665E+01_JPRB, 0.10424E+01_JPRB, 0.99210E+00_JPRB ,&
&  0.97544E+00_JPRB, 0.95963E+00_JPRB, 0.94956E+00_JPRB, 0.93419E+00_JPRB, 0.91573E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.85642E+00_JPRB, 0.86255E+00_JPRB, 0.87293E+00_JPRB, 0.88783E+00_JPRB, 0.90329E+00_JPRB, 0.91743E+00_JPRB, 0.92860E+00_JPRB ,&
&  0.93717E+00_JPRB, 0.95068E+00_JPRB, 0.97173E+00_JPRB, 0.99747E+00_JPRB, 0.10074E+01_JPRB, 0.10233E+01_JPRB, 0.10270E+01_JPRB ,&
&  0.10478E+01_JPRB, 0.10706E+01_JPRB, 0.10792E+01_JPRB, 0.10886E+01_JPRB, 0.10949E+01_JPRB, 0.11075E+01_JPRB, 0.11234E+01_JPRB ,&
&  0.11382E+01_JPRB, 0.11743E+01_JPRB, 0.12275E+01_JPRB, 0.12710E+01_JPRB, 0.13300E+01_JPRB, 0.14250E+01_JPRB, 0.11884E+01_JPRB ,&
&  0.12408E+01_JPRB, 0.12611E+01_JPRB, 0.11343E+01_JPRB, 0.95756E+00_JPRB, 0.89905E+00_JPRB, 0.85774E+00_JPRB, 0.83215E+00_JPRB ,&
&  0.82567E+00_JPRB, 0.78731E+00_JPRB, 0.79248E+00_JPRB, 0.78466E+00_JPRB, 0.77792E+00_JPRB, 0.80244E+00_JPRB, 0.79618E+00_JPRB ,&
&  0.79567E+00_JPRB, 0.79913E+00_JPRB, 0.80324E+00_JPRB, 0.79999E+00_JPRB, 0.80275E+00_JPRB, 0.78962E+00_JPRB, 0.76588E+00_JPRB ,&
&  0.72880E+00_JPRB, 0.68918E+00_JPRB, 0.62979E+00_JPRB, 0.60755E+00_JPRB, 0.59719E+00_JPRB, 0.54788E+00_JPRB, 0.50219E+00_JPRB ,&
&  0.53192E+00_JPRB, 0.58189E+00_JPRB, 0.63463E+00_JPRB, 0.87868E+00_JPRB, 0.88294E+00_JPRB, 0.87895E+00_JPRB, 0.74774E+00_JPRB ,&
&  0.68860E+00_JPRB, 0.60651E+00_JPRB, 0.56806E+00_JPRB, 0.56843E+00_JPRB, 0.56368E+00_JPRB, 0.55671E+00_JPRB, 0.53340E+00_JPRB ,&
&  0.48988E+00_JPRB, 0.45592E+00_JPRB, 0.45491E+00_JPRB, 0.46404E+00_JPRB, 0.49005E+00_JPRB, 0.46926E+00_JPRB, 0.52674E+00_JPRB ,&
&  0.49567E+00_JPRB, 0.36496E+00_JPRB, 0.43135E+00_JPRB, 0.53597E+00_JPRB, 0.54575E+00_JPRB, 0.60960E+00_JPRB, 0.74144E+00_JPRB ,&
&  0.95129E+00_JPRB, 0.10150E+01_JPRB, 0.10295E+01_JPRB, 0.96079E+00_JPRB, 0.94088E+00_JPRB, 0.93309E+00_JPRB, 0.91011E+00_JPRB ,&
&  0.89820E+00_JPRB, 0.89032E+00_JPRB, 0.88261E+00_JPRB, 0.86845E+00_JPRB, 0.85796E+00_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.81817E+00_JPRB, 0.81924E+00_JPRB, 0.82344E+00_JPRB, 0.83119E+00_JPRB, 0.84053E+00_JPRB, 0.85091E+00_JPRB, 0.85634E+00_JPRB ,&
&  0.85308E+00_JPRB, 0.85619E+00_JPRB, 0.86533E+00_JPRB, 0.87989E+00_JPRB, 0.89270E+00_JPRB, 0.90374E+00_JPRB, 0.91536E+00_JPRB ,&
&  0.91282E+00_JPRB, 0.91304E+00_JPRB, 0.92670E+00_JPRB, 0.94408E+00_JPRB, 0.95366E+00_JPRB, 0.95245E+00_JPRB, 0.95376E+00_JPRB ,&
&  0.94865E+00_JPRB, 0.95628E+00_JPRB, 0.97941E+00_JPRB, 0.99308E+00_JPRB, 0.10145E+01_JPRB, 0.10733E+01_JPRB, 0.12508E+01_JPRB ,&
&  0.11100E+01_JPRB, 0.12044E+01_JPRB, 0.10429E+01_JPRB, 0.90553E+00_JPRB, 0.88654E+00_JPRB, 0.84826E+00_JPRB, 0.81991E+00_JPRB ,&
&  0.78837E+00_JPRB, 0.75789E+00_JPRB, 0.71564E+00_JPRB, 0.70785E+00_JPRB, 0.68759E+00_JPRB, 0.70546E+00_JPRB, 0.71778E+00_JPRB ,&
&  0.74377E+00_JPRB, 0.75601E+00_JPRB, 0.77638E+00_JPRB, 0.78987E+00_JPRB, 0.82508E+00_JPRB, 0.82214E+00_JPRB, 0.78408E+00_JPRB ,&
&  0.77406E+00_JPRB, 0.80159E+00_JPRB, 0.76675E+00_JPRB, 0.78344E+00_JPRB, 0.76676E+00_JPRB, 0.73078E+00_JPRB, 0.71180E+00_JPRB ,&
&  0.69227E+00_JPRB, 0.68914E+00_JPRB, 0.78693E+00_JPRB, 0.80406E+00_JPRB, 0.87646E+00_JPRB, 0.90437E+00_JPRB, 0.82343E+00_JPRB ,&
&  0.75522E+00_JPRB, 0.67875E+00_JPRB, 0.66932E+00_JPRB, 0.66154E+00_JPRB, 0.68158E+00_JPRB, 0.66840E+00_JPRB, 0.65320E+00_JPRB ,&
&  0.59634E+00_JPRB, 0.54016E+00_JPRB, 0.54066E+00_JPRB, 0.53072E+00_JPRB, 0.51479E+00_JPRB, 0.58763E+00_JPRB, 0.56909E+00_JPRB ,&
&  0.53152E+00_JPRB, 0.48652E+00_JPRB, 0.42288E+00_JPRB, 0.58054E+00_JPRB, 0.60760E+00_JPRB, 0.67032E+00_JPRB, 0.74812E+00_JPRB ,&
&  0.85820E+00_JPRB, 0.95162E+00_JPRB, 0.10058E+01_JPRB, 0.95276E+00_JPRB, 0.87226E+00_JPRB, 0.85166E+00_JPRB, 0.83616E+00_JPRB ,&
&  0.83426E+00_JPRB, 0.83138E+00_JPRB, 0.82743E+00_JPRB, 0.82148E+00_JPRB, 0.81263E+00_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.75988E+00_JPRB, 0.76071E+00_JPRB, 0.77373E+00_JPRB, 0.77421E+00_JPRB, 0.77425E+00_JPRB, 0.78231E+00_JPRB, 0.78611E+00_JPRB ,&
&  0.78198E+00_JPRB, 0.79030E+00_JPRB, 0.80134E+00_JPRB, 0.81313E+00_JPRB, 0.82664E+00_JPRB, 0.83836E+00_JPRB, 0.84419E+00_JPRB ,&
&  0.83469E+00_JPRB, 0.82429E+00_JPRB, 0.84422E+00_JPRB, 0.86220E+00_JPRB, 0.86438E+00_JPRB, 0.86648E+00_JPRB, 0.89288E+00_JPRB ,&
&  0.94454E+00_JPRB, 0.97073E+00_JPRB, 0.96689E+00_JPRB, 0.93447E+00_JPRB, 0.94834E+00_JPRB, 0.11305E+01_JPRB, 0.17661E+01_JPRB ,&
&  0.15774E+01_JPRB, 0.10589E+01_JPRB, 0.98886E+00_JPRB, 0.89693E+00_JPRB, 0.89362E+00_JPRB, 0.90991E+00_JPRB, 0.89367E+00_JPRB ,&
&  0.84245E+00_JPRB, 0.83525E+00_JPRB, 0.82686E+00_JPRB, 0.81695E+00_JPRB, 0.82180E+00_JPRB, 0.82853E+00_JPRB, 0.82765E+00_JPRB ,&
&  0.81128E+00_JPRB, 0.81080E+00_JPRB, 0.82436E+00_JPRB, 0.83041E+00_JPRB, 0.86069E+00_JPRB, 0.97068E+00_JPRB, 0.96752E+00_JPRB ,&
&  0.96398E+00_JPRB, 0.93146E+00_JPRB, 0.89366E+00_JPRB, 0.89597E+00_JPRB, 0.90279E+00_JPRB, 0.86686E+00_JPRB, 0.82491E+00_JPRB ,&
&  0.81905E+00_JPRB, 0.78302E+00_JPRB, 0.67690E+00_JPRB, 0.76458E+00_JPRB, 0.85378E+00_JPRB, 0.94398E+00_JPRB, 0.90998E+00_JPRB ,&
&  0.87543E+00_JPRB, 0.80100E+00_JPRB, 0.77881E+00_JPRB, 0.81362E+00_JPRB, 0.86176E+00_JPRB, 0.84997E+00_JPRB, 0.81823E+00_JPRB ,&
&  0.74106E+00_JPRB, 0.67624E+00_JPRB, 0.66298E+00_JPRB, 0.61648E+00_JPRB, 0.59687E+00_JPRB, 0.53705E+00_JPRB, 0.58019E+00_JPRB ,&
&  0.58293E+00_JPRB, 0.60677E+00_JPRB, 0.51519E+00_JPRB, 0.66378E+00_JPRB, 0.68824E+00_JPRB, 0.70537E+00_JPRB, 0.72586E+00_JPRB ,&
&  0.74499E+00_JPRB, 0.76916E+00_JPRB, 0.77126E+00_JPRB, 0.84422E+00_JPRB, 0.82811E+00_JPRB, 0.81295E+00_JPRB, 0.81040E+00_JPRB ,&
&  0.77401E+00_JPRB, 0.76124E+00_JPRB, 0.75678E+00_JPRB, 0.76190E+00_JPRB, 0.76102E+00_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.68689E+00_JPRB, 0.68364E+00_JPRB, 0.68330E+00_JPRB, 0.69842E+00_JPRB, 0.71208E+00_JPRB, 0.70508E+00_JPRB, 0.71400E+00_JPRB ,&
&  0.69605E+00_JPRB, 0.68188E+00_JPRB, 0.69351E+00_JPRB, 0.70031E+00_JPRB, 0.71681E+00_JPRB, 0.73052E+00_JPRB, 0.73991E+00_JPRB ,&
&  0.74110E+00_JPRB, 0.74040E+00_JPRB, 0.71735E+00_JPRB, 0.73462E+00_JPRB, 0.82336E+00_JPRB, 0.96469E+00_JPRB, 0.12151E+01_JPRB ,&
&  0.14743E+01_JPRB, 0.15747E+01_JPRB, 0.16278E+01_JPRB, 0.15838E+01_JPRB, 0.14279E+01_JPRB, 0.12170E+01_JPRB, 0.15163E+01_JPRB ,&
&  0.13438E+01_JPRB, 0.10345E+01_JPRB, 0.10851E+01_JPRB, 0.10267E+01_JPRB, 0.10278E+01_JPRB, 0.99665E+00_JPRB, 0.97669E+00_JPRB ,&
&  0.93845E+00_JPRB, 0.93019E+00_JPRB, 0.93655E+00_JPRB, 0.96333E+00_JPRB, 0.99338E+00_JPRB, 0.10263E+01_JPRB, 0.10424E+01_JPRB ,&
&  0.10411E+01_JPRB, 0.10411E+01_JPRB, 0.10746E+01_JPRB, 0.10366E+01_JPRB, 0.10042E+01_JPRB, 0.10300E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.10642E+01_JPRB, 0.10585E+01_JPRB, 0.92192E+00_JPRB, 0.90503E+00_JPRB, 0.93580E+00_JPRB, 0.85848E+00_JPRB, 0.84581E+00_JPRB ,&
&  0.88022E+00_JPRB, 0.82778E+00_JPRB, 0.64046E+00_JPRB, 0.68159E+00_JPRB, 0.76316E+00_JPRB, 0.88096E+00_JPRB, 0.98371E+00_JPRB ,&
&  0.95550E+00_JPRB, 0.90502E+00_JPRB, 0.89737E+00_JPRB, 0.93948E+00_JPRB, 0.96687E+00_JPRB, 0.95963E+00_JPRB, 0.94884E+00_JPRB ,&
&  0.93138E+00_JPRB, 0.80366E+00_JPRB, 0.74281E+00_JPRB, 0.67405E+00_JPRB, 0.64084E+00_JPRB, 0.62791E+00_JPRB, 0.60370E+00_JPRB ,&
&  0.61524E+00_JPRB, 0.62775E+00_JPRB, 0.63403E+00_JPRB, 0.63265E+00_JPRB, 0.61260E+00_JPRB, 0.74322E+00_JPRB, 0.76171E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76291E+00_JPRB, 0.77274E+00_JPRB, 0.76010E+00_JPRB, 0.73802E+00_JPRB, 0.71674E+00_JPRB, 0.69646E+00_JPRB ,&
&  0.70477E+00_JPRB, 0.71401E+00_JPRB, 0.71655E+00_JPRB, 0.70177E+00_JPRB, 0.68997E+00_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.66118E+00_JPRB, 0.66515E+00_JPRB, 0.66437E+00_JPRB, 0.67783E+00_JPRB, 0.67520E+00_JPRB, 0.68980E+00_JPRB, 0.67987E+00_JPRB ,&
&  0.66948E+00_JPRB, 0.67065E+00_JPRB, 0.67874E+00_JPRB, 0.68692E+00_JPRB, 0.71257E+00_JPRB, 0.76308E+00_JPRB, 0.82769E+00_JPRB ,&
&  0.89582E+00_JPRB, 0.92368E+00_JPRB, 0.97890E+00_JPRB, 0.10956E+01_JPRB, 0.13173E+01_JPRB, 0.15209E+01_JPRB, 0.15445E+01_JPRB ,&
&  0.16042E+01_JPRB, 0.17045E+01_JPRB, 0.17660E+01_JPRB, 0.17928E+01_JPRB, 0.17032E+01_JPRB, 0.15562E+01_JPRB, 0.13811E+01_JPRB ,&
&  0.12091E+01_JPRB, 0.10474E+01_JPRB, 0.10482E+01_JPRB, 0.10630E+01_JPRB, 0.10705E+01_JPRB, 0.10386E+01_JPRB, 0.10094E+01_JPRB ,&
&  0.98600E+00_JPRB, 0.98194E+00_JPRB, 0.97761E+00_JPRB, 0.99051E+00_JPRB, 0.10211E+01_JPRB, 0.10625E+01_JPRB, 0.10703E+01_JPRB ,&
&  0.10547E+01_JPRB, 0.10440E+01_JPRB, 0.10455E+01_JPRB, 0.95851E+00_JPRB, 0.98405E+00_JPRB, 0.99219E+00_JPRB, 0.98222E+00_JPRB ,&
&  0.96108E+00_JPRB, 0.91504E+00_JPRB, 0.91679E+00_JPRB, 0.89882E+00_JPRB, 0.87563E+00_JPRB, 0.82571E+00_JPRB, 0.80662E+00_JPRB ,&
&  0.82013E+00_JPRB, 0.79151E+00_JPRB, 0.65877E+00_JPRB, 0.70730E+00_JPRB, 0.77975E+00_JPRB, 0.80459E+00_JPRB, 0.10271E+01_JPRB ,&
&  0.10040E+01_JPRB, 0.96932E+00_JPRB, 0.96635E+00_JPRB, 0.95306E+00_JPRB, 0.98200E+00_JPRB, 0.98158E+00_JPRB, 0.99202E+00_JPRB ,&
&  0.10433E+01_JPRB, 0.91315E+00_JPRB, 0.82714E+00_JPRB, 0.76683E+00_JPRB, 0.71307E+00_JPRB, 0.67430E+00_JPRB, 0.65728E+00_JPRB ,&
&  0.67512E+00_JPRB, 0.69904E+00_JPRB, 0.69297E+00_JPRB, 0.68392E+00_JPRB, 0.65630E+00_JPRB, 0.75037E+00_JPRB, 0.75661E+00_JPRB ,&
&  0.77584E+00_JPRB, 0.76446E+00_JPRB, 0.74467E+00_JPRB, 0.71942E+00_JPRB, 0.70347E+00_JPRB, 0.69237E+00_JPRB, 0.68809E+00_JPRB ,&
&  0.69260E+00_JPRB, 0.69666E+00_JPRB, 0.68800E+00_JPRB, 0.67818E+00_JPRB, 0.66661E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.63725E+00_JPRB, 0.63438E+00_JPRB, 0.63458E+00_JPRB, 0.66357E+00_JPRB, 0.66219E+00_JPRB, 0.65368E+00_JPRB, 0.66613E+00_JPRB ,&
&  0.68238E+00_JPRB, 0.70314E+00_JPRB, 0.72807E+00_JPRB, 0.75230E+00_JPRB, 0.78547E+00_JPRB, 0.83567E+00_JPRB, 0.89328E+00_JPRB ,&
&  0.95978E+00_JPRB, 0.10403E+01_JPRB, 0.11730E+01_JPRB, 0.13418E+01_JPRB, 0.15380E+01_JPRB, 0.16384E+01_JPRB, 0.16312E+01_JPRB ,&
&  0.15120E+01_JPRB, 0.15398E+01_JPRB, 0.15752E+01_JPRB, 0.15475E+01_JPRB, 0.12885E+01_JPRB, 0.12620E+01_JPRB, 0.12336E+01_JPRB ,&
&  0.12042E+01_JPRB, 0.11290E+01_JPRB, 0.10798E+01_JPRB, 0.10375E+01_JPRB, 0.99824E+00_JPRB, 0.99859E+00_JPRB, 0.99402E+00_JPRB ,&
&  0.97383E+00_JPRB, 0.96767E+00_JPRB, 0.96398E+00_JPRB, 0.96761E+00_JPRB, 0.98683E+00_JPRB, 0.10248E+01_JPRB, 0.10304E+01_JPRB ,&
&  0.10236E+01_JPRB, 0.10264E+01_JPRB, 0.10006E+01_JPRB, 0.94958E+00_JPRB, 0.94645E+00_JPRB, 0.93905E+00_JPRB, 0.92687E+00_JPRB ,&
&  0.93628E+00_JPRB, 0.90142E+00_JPRB, 0.90282E+00_JPRB, 0.89585E+00_JPRB, 0.84945E+00_JPRB, 0.77885E+00_JPRB, 0.76756E+00_JPRB ,&
&  0.78628E+00_JPRB, 0.81848E+00_JPRB, 0.68611E+00_JPRB, 0.83091E+00_JPRB, 0.93220E+00_JPRB, 0.89932E+00_JPRB, 0.94475E+00_JPRB ,&
&  0.94484E+00_JPRB, 0.95418E+00_JPRB, 0.96646E+00_JPRB, 0.97046E+00_JPRB, 0.96627E+00_JPRB, 0.96208E+00_JPRB, 0.91893E+00_JPRB ,&
&  0.10064E+01_JPRB, 0.91380E+00_JPRB, 0.83658E+00_JPRB, 0.80489E+00_JPRB, 0.77102E+00_JPRB, 0.71590E+00_JPRB, 0.66840E+00_JPRB ,&
&  0.65386E+00_JPRB, 0.84919E+00_JPRB, 0.92365E+00_JPRB, 0.10459E+01_JPRB, 0.11090E+01_JPRB, 0.75649E+00_JPRB, 0.79359E+00_JPRB ,&
&  0.78417E+00_JPRB, 0.76947E+00_JPRB, 0.74333E+00_JPRB, 0.71482E+00_JPRB, 0.70953E+00_JPRB, 0.70888E+00_JPRB, 0.70539E+00_JPRB ,&
&  0.69833E+00_JPRB, 0.68343E+00_JPRB, 0.67360E+00_JPRB, 0.65406E+00_JPRB, 0.64431E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.65345E+00_JPRB, 0.64881E+00_JPRB, 0.64793E+00_JPRB, 0.66229E+00_JPRB, 0.67465E+00_JPRB, 0.67634E+00_JPRB, 0.67037E+00_JPRB ,&
&  0.67832E+00_JPRB, 0.70406E+00_JPRB, 0.73497E+00_JPRB, 0.75074E+00_JPRB, 0.75625E+00_JPRB, 0.76581E+00_JPRB, 0.80832E+00_JPRB ,&
&  0.88121E+00_JPRB, 0.98967E+00_JPRB, 0.11258E+01_JPRB, 0.12357E+01_JPRB, 0.13327E+01_JPRB, 0.14096E+01_JPRB, 0.15348E+01_JPRB ,&
&  0.15474E+01_JPRB, 0.15896E+01_JPRB, 0.15309E+01_JPRB, 0.11860E+01_JPRB, 0.10698E+01_JPRB, 0.12417E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.10888E+01_JPRB, 0.10669E+01_JPRB, 0.10436E+01_JPRB, 0.10003E+01_JPRB, 0.95710E+00_JPRB, 0.96261E+00_JPRB, 0.95528E+00_JPRB ,&
&  0.92968E+00_JPRB, 0.92093E+00_JPRB, 0.92038E+00_JPRB, 0.92701E+00_JPRB, 0.93884E+00_JPRB, 0.94827E+00_JPRB, 0.94083E+00_JPRB ,&
&  0.95247E+00_JPRB, 0.94875E+00_JPRB, 0.95320E+00_JPRB, 0.94902E+00_JPRB, 0.93144E+00_JPRB, 0.93889E+00_JPRB, 0.93015E+00_JPRB ,&
&  0.93055E+00_JPRB, 0.91267E+00_JPRB, 0.89845E+00_JPRB, 0.89819E+00_JPRB, 0.88112E+00_JPRB, 0.78368E+00_JPRB, 0.76518E+00_JPRB ,&
&  0.78251E+00_JPRB, 0.78742E+00_JPRB, 0.74110E+00_JPRB, 0.82926E+00_JPRB, 0.62943E+00_JPRB, 0.73928E+00_JPRB, 0.88236E+00_JPRB ,&
&  0.91725E+00_JPRB, 0.93167E+00_JPRB, 0.93793E+00_JPRB, 0.94350E+00_JPRB, 0.94001E+00_JPRB, 0.87511E+00_JPRB, 0.87962E+00_JPRB ,&
&  0.88295E+00_JPRB, 0.85003E+00_JPRB, 0.81906E+00_JPRB, 0.80988E+00_JPRB, 0.79232E+00_JPRB, 0.75994E+00_JPRB, 0.71995E+00_JPRB ,&
&  0.66615E+00_JPRB, 0.80793E+00_JPRB, 0.82739E+00_JPRB, 0.84276E+00_JPRB, 0.83568E+00_JPRB, 0.72933E+00_JPRB, 0.76459E+00_JPRB ,&
&  0.78346E+00_JPRB, 0.75660E+00_JPRB, 0.72450E+00_JPRB, 0.71884E+00_JPRB, 0.71315E+00_JPRB, 0.71161E+00_JPRB, 0.69726E+00_JPRB ,&
&  0.69435E+00_JPRB, 0.69057E+00_JPRB, 0.68480E+00_JPRB, 0.67347E+00_JPRB, 0.66231E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.65217E+00_JPRB, 0.65103E+00_JPRB, 0.65592E+00_JPRB, 0.67024E+00_JPRB, 0.69670E+00_JPRB, 0.71901E+00_JPRB, 0.67942E+00_JPRB ,&
&  0.66654E+00_JPRB, 0.66018E+00_JPRB, 0.68762E+00_JPRB, 0.69893E+00_JPRB, 0.70729E+00_JPRB, 0.71567E+00_JPRB, 0.75479E+00_JPRB ,&
&  0.82660E+00_JPRB, 0.93036E+00_JPRB, 0.10322E+01_JPRB, 0.10992E+01_JPRB, 0.11702E+01_JPRB, 0.12691E+01_JPRB, 0.13912E+01_JPRB ,&
&  0.12546E+01_JPRB, 0.10949E+01_JPRB, 0.14656E+01_JPRB, 0.15241E+01_JPRB, 0.13653E+01_JPRB, 0.12706E+01_JPRB, 0.12568E+01_JPRB ,&
&  0.11114E+01_JPRB, 0.10935E+01_JPRB, 0.10923E+01_JPRB, 0.10359E+01_JPRB, 0.93129E+00_JPRB, 0.91613E+00_JPRB, 0.90874E+00_JPRB ,&
&  0.89407E+00_JPRB, 0.87528E+00_JPRB, 0.87213E+00_JPRB, 0.87940E+00_JPRB, 0.89012E+00_JPRB, 0.87946E+00_JPRB, 0.86301E+00_JPRB ,&
&  0.85857E+00_JPRB, 0.85925E+00_JPRB, 0.86836E+00_JPRB, 0.86664E+00_JPRB, 0.87535E+00_JPRB, 0.89216E+00_JPRB, 0.90195E+00_JPRB ,&
&  0.90898E+00_JPRB, 0.89766E+00_JPRB, 0.91099E+00_JPRB, 0.93150E+00_JPRB, 0.94421E+00_JPRB, 0.85800E+00_JPRB, 0.85069E+00_JPRB ,&
&  0.82355E+00_JPRB, 0.78815E+00_JPRB, 0.76395E+00_JPRB, 0.79978E+00_JPRB, 0.65035E+00_JPRB, 0.72340E+00_JPRB, 0.76948E+00_JPRB ,&
&  0.88955E+00_JPRB, 0.89961E+00_JPRB, 0.91429E+00_JPRB, 0.95021E+00_JPRB, 0.97572E+00_JPRB, 0.90781E+00_JPRB, 0.87373E+00_JPRB ,&
&  0.85617E+00_JPRB, 0.84952E+00_JPRB, 0.82712E+00_JPRB, 0.81782E+00_JPRB, 0.77327E+00_JPRB, 0.77118E+00_JPRB, 0.78581E+00_JPRB ,&
&  0.81739E+00_JPRB, 0.82239E+00_JPRB, 0.82599E+00_JPRB, 0.81045E+00_JPRB, 0.76466E+00_JPRB, 0.73661E+00_JPRB, 0.74525E+00_JPRB ,&
&  0.75762E+00_JPRB, 0.76955E+00_JPRB, 0.71928E+00_JPRB, 0.68824E+00_JPRB, 0.68155E+00_JPRB, 0.69596E+00_JPRB, 0.68304E+00_JPRB ,&
&  0.68087E+00_JPRB, 0.67168E+00_JPRB, 0.67137E+00_JPRB, 0.66564E+00_JPRB, 0.66113E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.65552E+00_JPRB, 0.67326E+00_JPRB, 0.70284E+00_JPRB, 0.73899E+00_JPRB, 0.77670E+00_JPRB, 0.83424E+00_JPRB, 0.80079E+00_JPRB ,&
&  0.76756E+00_JPRB, 0.60221E+00_JPRB, 0.61754E+00_JPRB, 0.62901E+00_JPRB, 0.64430E+00_JPRB, 0.64531E+00_JPRB, 0.66589E+00_JPRB ,&
&  0.73849E+00_JPRB, 0.84684E+00_JPRB, 0.93724E+00_JPRB, 0.10114E+01_JPRB, 0.10878E+01_JPRB, 0.11897E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.90337E+00_JPRB, 0.10769E+01_JPRB, 0.16615E+01_JPRB, 0.16182E+01_JPRB, 0.14971E+01_JPRB, 0.13882E+01_JPRB, 0.12716E+01_JPRB ,&
&  0.11614E+01_JPRB, 0.10993E+01_JPRB, 0.11205E+01_JPRB, 0.10255E+01_JPRB, 0.90852E+00_JPRB, 0.88590E+00_JPRB, 0.87887E+00_JPRB ,&
&  0.85746E+00_JPRB, 0.83808E+00_JPRB, 0.82488E+00_JPRB, 0.82569E+00_JPRB, 0.82681E+00_JPRB, 0.81714E+00_JPRB, 0.80473E+00_JPRB ,&
&  0.80069E+00_JPRB, 0.80426E+00_JPRB, 0.80708E+00_JPRB, 0.80428E+00_JPRB, 0.82650E+00_JPRB, 0.85092E+00_JPRB, 0.86529E+00_JPRB ,&
&  0.87464E+00_JPRB, 0.89998E+00_JPRB, 0.92093E+00_JPRB, 0.91876E+00_JPRB, 0.87839E+00_JPRB, 0.93727E+00_JPRB, 0.88715E+00_JPRB ,&
&  0.87109E+00_JPRB, 0.82433E+00_JPRB, 0.83220E+00_JPRB, 0.71872E+00_JPRB, 0.71272E+00_JPRB, 0.75849E+00_JPRB, 0.80516E+00_JPRB ,&
&  0.83543E+00_JPRB, 0.87262E+00_JPRB, 0.88819E+00_JPRB, 0.93923E+00_JPRB, 0.99279E+00_JPRB, 0.97336E+00_JPRB, 0.96052E+00_JPRB ,&
&  0.89611E+00_JPRB, 0.92461E+00_JPRB, 0.86465E+00_JPRB, 0.81082E+00_JPRB, 0.73725E+00_JPRB, 0.71817E+00_JPRB, 0.75213E+00_JPRB ,&
&  0.80824E+00_JPRB, 0.83860E+00_JPRB, 0.85418E+00_JPRB, 0.86126E+00_JPRB, 0.83195E+00_JPRB, 0.80139E+00_JPRB, 0.73935E+00_JPRB ,&
&  0.72310E+00_JPRB, 0.68411E+00_JPRB, 0.67093E+00_JPRB, 0.67002E+00_JPRB, 0.65993E+00_JPRB, 0.67103E+00_JPRB, 0.66360E+00_JPRB ,&
&  0.64986E+00_JPRB, 0.64939E+00_JPRB, 0.64818E+00_JPRB, 0.64546E+00_JPRB, 0.64647E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.61354E+00_JPRB, 0.62885E+00_JPRB, 0.65547E+00_JPRB, 0.66953E+00_JPRB, 0.66457E+00_JPRB, 0.68125E+00_JPRB, 0.71336E+00_JPRB ,&
&  0.71032E+00_JPRB, 0.57053E+00_JPRB, 0.55273E+00_JPRB, 0.55959E+00_JPRB, 0.57891E+00_JPRB, 0.59078E+00_JPRB, 0.59893E+00_JPRB ,&
&  0.63277E+00_JPRB, 0.72188E+00_JPRB, 0.85857E+00_JPRB, 0.91537E+00_JPRB, 0.96034E+00_JPRB, 0.10646E+01_JPRB, 0.99793E+00_JPRB ,&
&  0.82812E+00_JPRB, 0.14647E+01_JPRB, 0.14483E+01_JPRB, 0.16221E+01_JPRB, 0.15819E+01_JPRB, 0.14861E+01_JPRB, 0.13114E+01_JPRB ,&
&  0.11284E+01_JPRB, 0.10349E+01_JPRB, 0.10146E+01_JPRB, 0.96639E+00_JPRB, 0.91964E+00_JPRB, 0.87131E+00_JPRB, 0.85835E+00_JPRB ,&
&  0.83279E+00_JPRB, 0.79874E+00_JPRB, 0.77394E+00_JPRB, 0.77271E+00_JPRB, 0.76738E+00_JPRB, 0.76024E+00_JPRB, 0.75591E+00_JPRB ,&
&  0.76260E+00_JPRB, 0.77002E+00_JPRB, 0.76996E+00_JPRB, 0.78409E+00_JPRB, 0.80179E+00_JPRB, 0.83228E+00_JPRB, 0.85523E+00_JPRB ,&
&  0.84327E+00_JPRB, 0.83159E+00_JPRB, 0.89435E+00_JPRB, 0.92120E+00_JPRB, 0.92791E+00_JPRB, 0.97105E+00_JPRB, 0.90821E+00_JPRB ,&
&  0.91602E+00_JPRB, 0.84340E+00_JPRB, 0.85205E+00_JPRB, 0.71303E+00_JPRB, 0.71431E+00_JPRB, 0.75847E+00_JPRB, 0.79167E+00_JPRB ,&
&  0.79180E+00_JPRB, 0.86019E+00_JPRB, 0.88835E+00_JPRB, 0.94313E+00_JPRB, 0.99642E+00_JPRB, 0.10579E+01_JPRB, 0.10122E+01_JPRB ,&
&  0.98875E+00_JPRB, 0.99580E+00_JPRB, 0.98404E+00_JPRB, 0.84797E+00_JPRB, 0.77186E+00_JPRB, 0.70678E+00_JPRB, 0.73984E+00_JPRB ,&
&  0.78996E+00_JPRB, 0.80934E+00_JPRB, 0.83955E+00_JPRB, 0.87981E+00_JPRB, 0.87746E+00_JPRB, 0.89725E+00_JPRB, 0.85316E+00_JPRB ,&
&  0.75925E+00_JPRB, 0.71130E+00_JPRB, 0.65409E+00_JPRB, 0.65079E+00_JPRB, 0.65274E+00_JPRB, 0.64273E+00_JPRB, 0.64460E+00_JPRB ,&
&  0.63755E+00_JPRB, 0.62548E+00_JPRB, 0.61987E+00_JPRB, 0.61280E+00_JPRB, 0.60513E+00_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.57977E+00_JPRB, 0.58999E+00_JPRB, 0.60777E+00_JPRB, 0.61798E+00_JPRB, 0.61866E+00_JPRB, 0.60894E+00_JPRB, 0.63648E+00_JPRB ,&
&  0.66242E+00_JPRB, 0.56034E+00_JPRB, 0.51605E+00_JPRB, 0.50022E+00_JPRB, 0.50945E+00_JPRB, 0.53553E+00_JPRB, 0.55762E+00_JPRB ,&
&  0.57838E+00_JPRB, 0.61509E+00_JPRB, 0.75104E+00_JPRB, 0.83235E+00_JPRB, 0.88814E+00_JPRB, 0.10236E+01_JPRB, 0.75439E+00_JPRB ,&
&  0.82652E+00_JPRB, 0.12344E+01_JPRB, 0.13960E+01_JPRB, 0.15694E+01_JPRB, 0.16752E+01_JPRB, 0.16119E+01_JPRB, 0.14175E+01_JPRB ,&
&  0.11831E+01_JPRB, 0.10386E+01_JPRB, 0.95685E+00_JPRB, 0.90550E+00_JPRB, 0.86750E+00_JPRB, 0.84147E+00_JPRB, 0.81373E+00_JPRB ,&
&  0.79840E+00_JPRB, 0.77136E+00_JPRB, 0.73698E+00_JPRB, 0.72887E+00_JPRB, 0.72084E+00_JPRB, 0.70550E+00_JPRB, 0.71472E+00_JPRB ,&
&  0.73398E+00_JPRB, 0.74870E+00_JPRB, 0.76322E+00_JPRB, 0.75273E+00_JPRB, 0.78293E+00_JPRB, 0.81146E+00_JPRB, 0.84278E+00_JPRB ,&
&  0.81798E+00_JPRB, 0.79981E+00_JPRB, 0.90881E+00_JPRB, 0.10084E+01_JPRB, 0.10470E+01_JPRB, 0.10026E+01_JPRB, 0.94483E+00_JPRB ,&
&  0.93274E+00_JPRB, 0.88104E+00_JPRB, 0.81397E+00_JPRB, 0.70992E+00_JPRB, 0.70875E+00_JPRB, 0.75778E+00_JPRB, 0.79733E+00_JPRB ,&
&  0.83821E+00_JPRB, 0.86989E+00_JPRB, 0.91045E+00_JPRB, 0.93182E+00_JPRB, 0.97743E+00_JPRB, 0.10678E+01_JPRB, 0.10831E+01_JPRB ,&
&  0.11777E+01_JPRB, 0.13074E+01_JPRB, 0.11536E+01_JPRB, 0.97935E+00_JPRB, 0.84602E+00_JPRB, 0.73719E+00_JPRB, 0.71238E+00_JPRB ,&
&  0.75154E+00_JPRB, 0.75421E+00_JPRB, 0.74324E+00_JPRB, 0.79382E+00_JPRB, 0.84149E+00_JPRB, 0.97300E+00_JPRB, 0.10113E+01_JPRB ,&
&  0.86915E+00_JPRB, 0.79977E+00_JPRB, 0.71837E+00_JPRB, 0.67752E+00_JPRB, 0.65452E+00_JPRB, 0.65189E+00_JPRB, 0.64517E+00_JPRB ,&
&  0.64031E+00_JPRB, 0.62815E+00_JPRB, 0.61065E+00_JPRB, 0.59802E+00_JPRB, 0.58457E+00_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.57670E+00_JPRB, 0.56735E+00_JPRB, 0.56737E+00_JPRB, 0.57075E+00_JPRB, 0.58439E+00_JPRB, 0.58277E+00_JPRB, 0.58306E+00_JPRB ,&
&  0.62062E+00_JPRB, 0.53735E+00_JPRB, 0.50294E+00_JPRB, 0.47822E+00_JPRB, 0.48276E+00_JPRB, 0.50702E+00_JPRB, 0.53484E+00_JPRB ,&
&  0.55608E+00_JPRB, 0.58074E+00_JPRB, 0.66610E+00_JPRB, 0.76950E+00_JPRB, 0.77654E+00_JPRB, 0.70750E+00_JPRB, 0.73375E+00_JPRB ,&
&  0.93506E+00_JPRB, 0.12040E+01_JPRB, 0.13723E+01_JPRB, 0.15551E+01_JPRB, 0.17794E+01_JPRB, 0.18805E+01_JPRB, 0.16212E+01_JPRB ,&
&  0.14012E+01_JPRB, 0.11728E+01_JPRB, 0.10002E+01_JPRB, 0.90477E+00_JPRB, 0.86006E+00_JPRB, 0.81574E+00_JPRB, 0.78282E+00_JPRB ,&
&  0.75198E+00_JPRB, 0.72665E+00_JPRB, 0.70924E+00_JPRB, 0.69549E+00_JPRB, 0.69219E+00_JPRB, 0.67249E+00_JPRB, 0.67116E+00_JPRB ,&
&  0.68935E+00_JPRB, 0.70986E+00_JPRB, 0.73434E+00_JPRB, 0.72880E+00_JPRB, 0.75604E+00_JPRB, 0.79031E+00_JPRB, 0.84581E+00_JPRB ,&
&  0.86925E+00_JPRB, 0.92273E+00_JPRB, 0.97927E+00_JPRB, 0.10111E+01_JPRB, 0.10690E+01_JPRB, 0.10798E+01_JPRB, 0.10253E+01_JPRB ,&
&  0.96725E+00_JPRB, 0.88564E+00_JPRB, 0.77604E+00_JPRB, 0.74238E+00_JPRB, 0.75533E+00_JPRB, 0.79235E+00_JPRB, 0.72084E+00_JPRB ,&
&  0.69871E+00_JPRB, 0.69650E+00_JPRB, 0.76479E+00_JPRB, 0.85707E+00_JPRB, 0.93089E+00_JPRB, 0.10388E+01_JPRB, 0.12156E+01_JPRB ,&
&  0.12931E+01_JPRB, 0.85671E+00_JPRB, 0.73270E+00_JPRB, 0.82412E+00_JPRB, 0.71355E+00_JPRB, 0.60119E+00_JPRB, 0.65371E+00_JPRB ,&
&  0.69268E+00_JPRB, 0.73926E+00_JPRB, 0.75421E+00_JPRB, 0.70331E+00_JPRB, 0.75083E+00_JPRB, 0.92780E+00_JPRB, 0.11397E+01_JPRB ,&
&  0.94768E+00_JPRB, 0.87919E+00_JPRB, 0.81687E+00_JPRB, 0.75435E+00_JPRB, 0.70349E+00_JPRB, 0.67018E+00_JPRB, 0.65079E+00_JPRB ,&
&  0.64344E+00_JPRB, 0.63756E+00_JPRB, 0.62786E+00_JPRB, 0.61103E+00_JPRB, 0.59240E+00_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.58968E+00_JPRB, 0.57292E+00_JPRB, 0.56203E+00_JPRB, 0.55374E+00_JPRB, 0.55542E+00_JPRB, 0.55362E+00_JPRB, 0.54565E+00_JPRB ,&
&  0.57998E+00_JPRB, 0.53478E+00_JPRB, 0.49676E+00_JPRB, 0.47471E+00_JPRB, 0.47354E+00_JPRB, 0.48823E+00_JPRB, 0.51757E+00_JPRB ,&
&  0.55618E+00_JPRB, 0.58465E+00_JPRB, 0.66141E+00_JPRB, 0.74335E+00_JPRB, 0.72328E+00_JPRB, 0.62818E+00_JPRB, 0.71838E+00_JPRB ,&
&  0.81496E+00_JPRB, 0.10818E+01_JPRB, 0.15134E+01_JPRB, 0.17718E+01_JPRB, 0.18360E+01_JPRB, 0.19253E+01_JPRB, 0.17624E+01_JPRB ,&
&  0.16300E+01_JPRB, 0.14075E+01_JPRB, 0.11512E+01_JPRB, 0.99010E+00_JPRB, 0.88277E+00_JPRB, 0.83369E+00_JPRB, 0.78246E+00_JPRB ,&
&  0.75717E+00_JPRB, 0.71819E+00_JPRB, 0.68463E+00_JPRB, 0.66750E+00_JPRB, 0.65798E+00_JPRB, 0.65115E+00_JPRB, 0.64059E+00_JPRB ,&
&  0.64605E+00_JPRB, 0.66298E+00_JPRB, 0.69563E+00_JPRB, 0.72466E+00_JPRB, 0.64801E+00_JPRB, 0.75748E+00_JPRB, 0.82428E+00_JPRB ,&
&  0.89980E+00_JPRB, 0.97225E+00_JPRB, 0.96753E+00_JPRB, 0.10912E+01_JPRB, 0.11311E+01_JPRB, 0.10824E+01_JPRB, 0.10507E+01_JPRB ,&
&  0.10466E+01_JPRB, 0.93848E+00_JPRB, 0.83605E+00_JPRB, 0.84533E+00_JPRB, 0.83221E+00_JPRB, 0.82689E+00_JPRB, 0.63293E+00_JPRB ,&
&  0.61180E+00_JPRB, 0.69586E+00_JPRB, 0.72437E+00_JPRB, 0.72192E+00_JPRB, 0.81113E+00_JPRB, 0.96339E+00_JPRB, 0.88562E+00_JPRB ,&
&  0.50754E+00_JPRB, 0.44705E+00_JPRB, 0.46935E+00_JPRB, 0.47345E+00_JPRB, 0.45955E+00_JPRB, 0.48140E+00_JPRB, 0.64977E+00_JPRB ,&
&  0.64067E+00_JPRB, 0.70097E+00_JPRB, 0.72580E+00_JPRB, 0.70616E+00_JPRB, 0.69938E+00_JPRB, 0.76966E+00_JPRB, 0.11812E+01_JPRB ,&
&  0.10480E+01_JPRB, 0.92235E+00_JPRB, 0.85030E+00_JPRB, 0.80502E+00_JPRB, 0.75140E+00_JPRB, 0.70455E+00_JPRB, 0.66670E+00_JPRB ,&
&  0.64141E+00_JPRB, 0.62915E+00_JPRB, 0.62381E+00_JPRB, 0.61715E+00_JPRB, 0.60517E+00_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.58108E+00_JPRB, 0.56529E+00_JPRB, 0.55492E+00_JPRB, 0.54729E+00_JPRB, 0.54051E+00_JPRB, 0.54044E+00_JPRB, 0.52911E+00_JPRB ,&
&  0.54144E+00_JPRB, 0.53575E+00_JPRB, 0.50228E+00_JPRB, 0.47976E+00_JPRB, 0.47165E+00_JPRB, 0.48421E+00_JPRB, 0.52112E+00_JPRB ,&
&  0.55007E+00_JPRB, 0.60374E+00_JPRB, 0.73040E+00_JPRB, 0.69834E+00_JPRB, 0.71026E+00_JPRB, 0.62764E+00_JPRB, 0.66532E+00_JPRB ,&
&  0.75077E+00_JPRB, 0.92785E+00_JPRB, 0.12684E+01_JPRB, 0.15493E+01_JPRB, 0.18706E+01_JPRB, 0.19919E+01_JPRB, 0.19491E+01_JPRB ,&
&  0.18120E+01_JPRB, 0.16578E+01_JPRB, 0.14204E+01_JPRB, 0.11796E+01_JPRB, 0.10147E+01_JPRB, 0.90296E+00_JPRB, 0.83117E+00_JPRB ,&
&  0.79336E+00_JPRB, 0.76510E+00_JPRB, 0.70158E+00_JPRB, 0.66523E+00_JPRB, 0.64489E+00_JPRB, 0.63347E+00_JPRB, 0.61282E+00_JPRB ,&
&  0.60886E+00_JPRB, 0.62388E+00_JPRB, 0.65430E+00_JPRB, 0.68712E+00_JPRB, 0.64209E+00_JPRB, 0.61252E+00_JPRB, 0.75138E+00_JPRB ,&
&  0.89795E+00_JPRB, 0.10705E+01_JPRB, 0.11350E+01_JPRB, 0.13344E+01_JPRB, 0.12581E+01_JPRB, 0.11518E+01_JPRB, 0.11502E+01_JPRB ,&
&  0.11357E+01_JPRB, 0.11552E+01_JPRB, 0.93189E+00_JPRB, 0.94335E+00_JPRB, 0.95518E+00_JPRB, 0.75367E+00_JPRB, 0.66019E+00_JPRB ,&
&  0.71569E+00_JPRB, 0.69308E+00_JPRB, 0.68285E+00_JPRB, 0.64199E+00_JPRB, 0.74546E+00_JPRB, 0.78286E+00_JPRB, 0.50088E+00_JPRB ,&
&  0.43815E+00_JPRB, 0.42503E+00_JPRB, 0.41089E+00_JPRB, 0.39495E+00_JPRB, 0.39388E+00_JPRB, 0.41355E+00_JPRB, 0.51609E+00_JPRB ,&
&  0.55390E+00_JPRB, 0.61618E+00_JPRB, 0.65811E+00_JPRB, 0.69093E+00_JPRB, 0.67878E+00_JPRB, 0.74122E+00_JPRB, 0.90696E+00_JPRB ,&
&  0.10364E+01_JPRB, 0.10071E+01_JPRB, 0.91431E+00_JPRB, 0.84222E+00_JPRB, 0.77586E+00_JPRB, 0.72311E+00_JPRB, 0.68487E+00_JPRB ,&
&  0.65632E+00_JPRB, 0.63967E+00_JPRB, 0.62723E+00_JPRB, 0.61611E+00_JPRB, 0.59842E+00_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.58933E+00_JPRB, 0.57116E+00_JPRB, 0.55060E+00_JPRB, 0.53707E+00_JPRB, 0.53134E+00_JPRB, 0.52516E+00_JPRB, 0.51661E+00_JPRB ,&
&  0.51121E+00_JPRB, 0.52557E+00_JPRB, 0.50527E+00_JPRB, 0.48876E+00_JPRB, 0.47625E+00_JPRB, 0.48092E+00_JPRB, 0.50951E+00_JPRB ,&
&  0.54905E+00_JPRB, 0.62841E+00_JPRB, 0.68780E+00_JPRB, 0.63390E+00_JPRB, 0.60402E+00_JPRB, 0.56955E+00_JPRB, 0.56455E+00_JPRB ,&
&  0.68943E+00_JPRB, 0.80860E+00_JPRB, 0.10163E+01_JPRB, 0.13774E+01_JPRB, 0.18269E+01_JPRB, 0.19861E+01_JPRB, 0.21603E+01_JPRB ,&
&  0.20215E+01_JPRB, 0.18255E+01_JPRB, 0.16579E+01_JPRB, 0.14438E+01_JPRB, 0.12051E+01_JPRB, 0.10504E+01_JPRB, 0.94883E+00_JPRB ,&
&  0.87924E+00_JPRB, 0.82864E+00_JPRB, 0.76911E+00_JPRB, 0.70346E+00_JPRB, 0.65370E+00_JPRB, 0.62751E+00_JPRB, 0.60397E+00_JPRB ,&
&  0.59189E+00_JPRB, 0.60651E+00_JPRB, 0.63395E+00_JPRB, 0.67567E+00_JPRB, 0.72267E+00_JPRB, 0.73754E+00_JPRB, 0.76143E+00_JPRB ,&
&  0.86577E+00_JPRB, 0.98157E+00_JPRB, 0.12923E+01_JPRB, 0.14036E+01_JPRB, 0.13908E+01_JPRB, 0.13696E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.12947E+01_JPRB, 0.13615E+01_JPRB, 0.11184E+01_JPRB, 0.11578E+01_JPRB, 0.10305E+01_JPRB, 0.79168E+00_JPRB, 0.80916E+00_JPRB ,&
&  0.82714E+00_JPRB, 0.77163E+00_JPRB, 0.69381E+00_JPRB, 0.59049E+00_JPRB, 0.59112E+00_JPRB, 0.51949E+00_JPRB, 0.40606E+00_JPRB ,&
&  0.38703E+00_JPRB, 0.37548E+00_JPRB, 0.36469E+00_JPRB, 0.36510E+00_JPRB, 0.36159E+00_JPRB, 0.38865E+00_JPRB, 0.45066E+00_JPRB ,&
&  0.51656E+00_JPRB, 0.56201E+00_JPRB, 0.60181E+00_JPRB, 0.63618E+00_JPRB, 0.64702E+00_JPRB, 0.66053E+00_JPRB, 0.86331E+00_JPRB ,&
&  0.86930E+00_JPRB, 0.89902E+00_JPRB, 0.89570E+00_JPRB, 0.84722E+00_JPRB, 0.78853E+00_JPRB, 0.73770E+00_JPRB, 0.69330E+00_JPRB ,&
&  0.66369E+00_JPRB, 0.64960E+00_JPRB, 0.63937E+00_JPRB, 0.62499E+00_JPRB, 0.60821E+00_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.58781E+00_JPRB, 0.56991E+00_JPRB, 0.55314E+00_JPRB, 0.54197E+00_JPRB, 0.53344E+00_JPRB, 0.52510E+00_JPRB, 0.51299E+00_JPRB ,&
&  0.49728E+00_JPRB, 0.50251E+00_JPRB, 0.49990E+00_JPRB, 0.47725E+00_JPRB, 0.46754E+00_JPRB, 0.47557E+00_JPRB, 0.50242E+00_JPRB ,&
&  0.54484E+00_JPRB, 0.67714E+00_JPRB, 0.58054E+00_JPRB, 0.53866E+00_JPRB, 0.56059E+00_JPRB, 0.56292E+00_JPRB, 0.52378E+00_JPRB ,&
&  0.64687E+00_JPRB, 0.76322E+00_JPRB, 0.93842E+00_JPRB, 0.13530E+01_JPRB, 0.20699E+01_JPRB, 0.22081E+01_JPRB, 0.23353E+01_JPRB ,&
&  0.23905E+01_JPRB, 0.21550E+01_JPRB, 0.19386E+01_JPRB, 0.16945E+01_JPRB, 0.14070E+01_JPRB, 0.11914E+01_JPRB, 0.10495E+01_JPRB ,&
&  0.94536E+00_JPRB, 0.88420E+00_JPRB, 0.80752E+00_JPRB, 0.74182E+00_JPRB, 0.68950E+00_JPRB, 0.65079E+00_JPRB, 0.62430E+00_JPRB ,&
&  0.60894E+00_JPRB, 0.61071E+00_JPRB, 0.62936E+00_JPRB, 0.67397E+00_JPRB, 0.72738E+00_JPRB, 0.74702E+00_JPRB, 0.10028E+01_JPRB ,&
&  0.12000E+01_JPRB, 0.12969E+01_JPRB, 0.13406E+01_JPRB, 0.16146E+01_JPRB, 0.17696E+01_JPRB, 0.17140E+01_JPRB, 0.17783E+01_JPRB ,&
&  0.14948E+01_JPRB, 0.13865E+01_JPRB, 0.13721E+01_JPRB, 0.13252E+01_JPRB, 0.98482E+00_JPRB, 0.96546E+00_JPRB, 0.11821E+01_JPRB ,&
&  0.90845E+00_JPRB, 0.89888E+00_JPRB, 0.91867E+00_JPRB, 0.85246E+00_JPRB, 0.47271E+00_JPRB, 0.41356E+00_JPRB, 0.48101E+00_JPRB ,&
&  0.48479E+00_JPRB, 0.39263E+00_JPRB, 0.35780E+00_JPRB, 0.38520E+00_JPRB, 0.35327E+00_JPRB, 0.38508E+00_JPRB, 0.45480E+00_JPRB ,&
&  0.46764E+00_JPRB, 0.48394E+00_JPRB, 0.55322E+00_JPRB, 0.58728E+00_JPRB, 0.61871E+00_JPRB, 0.63366E+00_JPRB, 0.71350E+00_JPRB ,&
&  0.76621E+00_JPRB, 0.89280E+00_JPRB, 0.93399E+00_JPRB, 0.87785E+00_JPRB, 0.80426E+00_JPRB, 0.74015E+00_JPRB, 0.69244E+00_JPRB ,&
&  0.65588E+00_JPRB, 0.63578E+00_JPRB, 0.62462E+00_JPRB, 0.61052E+00_JPRB, 0.59875E+00_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.55269E+00_JPRB, 0.54018E+00_JPRB, 0.52880E+00_JPRB, 0.51958E+00_JPRB, 0.51600E+00_JPRB, 0.51283E+00_JPRB, 0.50755E+00_JPRB ,&
&  0.49543E+00_JPRB, 0.48682E+00_JPRB, 0.49242E+00_JPRB, 0.47303E+00_JPRB, 0.45854E+00_JPRB, 0.46156E+00_JPRB, 0.49692E+00_JPRB ,&
&  0.53782E+00_JPRB, 0.58018E+00_JPRB, 0.48795E+00_JPRB, 0.45887E+00_JPRB, 0.54025E+00_JPRB, 0.52140E+00_JPRB, 0.54145E+00_JPRB ,&
&  0.63203E+00_JPRB, 0.72902E+00_JPRB, 0.88592E+00_JPRB, 0.12479E+01_JPRB, 0.16334E+01_JPRB, 0.20448E+01_JPRB, 0.23460E+01_JPRB ,&
&  0.24524E+01_JPRB, 0.23643E+01_JPRB, 0.21629E+01_JPRB, 0.19238E+01_JPRB, 0.15894E+01_JPRB, 0.13328E+01_JPRB, 0.11534E+01_JPRB ,&
&  0.10191E+01_JPRB, 0.91015E+00_JPRB, 0.82492E+00_JPRB, 0.72260E+00_JPRB, 0.67198E+00_JPRB, 0.63480E+00_JPRB, 0.61366E+00_JPRB ,&
&  0.60768E+00_JPRB, 0.61666E+00_JPRB, 0.63324E+00_JPRB, 0.68228E+00_JPRB, 0.66944E+00_JPRB, 0.70132E+00_JPRB, 0.89476E+00_JPRB ,&
&  0.12980E+01_JPRB, 0.13695E+01_JPRB, 0.14707E+01_JPRB, 0.16888E+01_JPRB, 0.18906E+01_JPRB, 0.18022E+01_JPRB, 0.19103E+01_JPRB ,&
&  0.14909E+01_JPRB, 0.12202E+01_JPRB, 0.10453E+01_JPRB, 0.83354E+00_JPRB, 0.86095E+00_JPRB, 0.11894E+01_JPRB, 0.11818E+01_JPRB ,&
&  0.10749E+01_JPRB, 0.10086E+01_JPRB, 0.95045E+00_JPRB, 0.74717E+00_JPRB, 0.56313E+00_JPRB, 0.45306E+00_JPRB, 0.48984E+00_JPRB ,&
&  0.50051E+00_JPRB, 0.50058E+00_JPRB, 0.46304E+00_JPRB, 0.42908E+00_JPRB, 0.39967E+00_JPRB, 0.44998E+00_JPRB, 0.45275E+00_JPRB ,&
&  0.43794E+00_JPRB, 0.43433E+00_JPRB, 0.47085E+00_JPRB, 0.51609E+00_JPRB, 0.56102E+00_JPRB, 0.55368E+00_JPRB, 0.61960E+00_JPRB ,&
&  0.64592E+00_JPRB, 0.69873E+00_JPRB, 0.72308E+00_JPRB, 0.71641E+00_JPRB, 0.69176E+00_JPRB, 0.65760E+00_JPRB, 0.62867E+00_JPRB ,&
&  0.61011E+00_JPRB, 0.59601E+00_JPRB, 0.58287E+00_JPRB, 0.57161E+00_JPRB, 0.56314E+00_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.51439E+00_JPRB, 0.50917E+00_JPRB, 0.50364E+00_JPRB, 0.50118E+00_JPRB, 0.49877E+00_JPRB, 0.49929E+00_JPRB, 0.49942E+00_JPRB ,&
&  0.49290E+00_JPRB, 0.47974E+00_JPRB, 0.48512E+00_JPRB, 0.47700E+00_JPRB, 0.46249E+00_JPRB, 0.45831E+00_JPRB, 0.47787E+00_JPRB ,&
&  0.51718E+00_JPRB, 0.50988E+00_JPRB, 0.41819E+00_JPRB, 0.43496E+00_JPRB, 0.47928E+00_JPRB, 0.46535E+00_JPRB, 0.51172E+00_JPRB ,&
&  0.57978E+00_JPRB, 0.67189E+00_JPRB, 0.76772E+00_JPRB, 0.97688E+00_JPRB, 0.12361E+01_JPRB, 0.15691E+01_JPRB, 0.18829E+01_JPRB ,&
&  0.19762E+01_JPRB, 0.21162E+01_JPRB, 0.21572E+01_JPRB, 0.19774E+01_JPRB, 0.17465E+01_JPRB, 0.15100E+01_JPRB, 0.13089E+01_JPRB ,&
&  0.11644E+01_JPRB, 0.10407E+01_JPRB, 0.89761E+00_JPRB, 0.78119E+00_JPRB, 0.70857E+00_JPRB, 0.66774E+00_JPRB, 0.64201E+00_JPRB ,&
&  0.62823E+00_JPRB, 0.62031E+00_JPRB, 0.62271E+00_JPRB, 0.67367E+00_JPRB, 0.66382E+00_JPRB, 0.74720E+00_JPRB, 0.93447E+00_JPRB ,&
&  0.15190E+01_JPRB, 0.16215E+01_JPRB, 0.16346E+01_JPRB, 0.19535E+01_JPRB, 0.15621E+01_JPRB, 0.15202E+01_JPRB, 0.17326E+01_JPRB ,&
&  0.18302E+01_JPRB, 0.17510E+01_JPRB, 0.16597E+01_JPRB, 0.12010E+01_JPRB, 0.95054E+00_JPRB, 0.12704E+01_JPRB, 0.11992E+01_JPRB ,&
&  0.10692E+01_JPRB, 0.99610E+00_JPRB, 0.91391E+00_JPRB, 0.89653E+00_JPRB, 0.63497E+00_JPRB, 0.52046E+00_JPRB, 0.44441E+00_JPRB ,&
&  0.44440E+00_JPRB, 0.44149E+00_JPRB, 0.47175E+00_JPRB, 0.46368E+00_JPRB, 0.45541E+00_JPRB, 0.45715E+00_JPRB, 0.45182E+00_JPRB ,&
&  0.45429E+00_JPRB, 0.44656E+00_JPRB, 0.44312E+00_JPRB, 0.47048E+00_JPRB, 0.51369E+00_JPRB, 0.50724E+00_JPRB, 0.53577E+00_JPRB ,&
&  0.55415E+00_JPRB, 0.58740E+00_JPRB, 0.61186E+00_JPRB, 0.61574E+00_JPRB, 0.61090E+00_JPRB, 0.59631E+00_JPRB, 0.57282E+00_JPRB ,&
&  0.56014E+00_JPRB, 0.55281E+00_JPRB, 0.54257E+00_JPRB, 0.53238E+00_JPRB, 0.52271E+00_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.46348E+00_JPRB, 0.46207E+00_JPRB, 0.46369E+00_JPRB, 0.46840E+00_JPRB, 0.47391E+00_JPRB, 0.47891E+00_JPRB, 0.48254E+00_JPRB ,&
&  0.47834E+00_JPRB, 0.46548E+00_JPRB, 0.46508E+00_JPRB, 0.46739E+00_JPRB, 0.45532E+00_JPRB, 0.45158E+00_JPRB, 0.46476E+00_JPRB ,&
&  0.49954E+00_JPRB, 0.53290E+00_JPRB, 0.44934E+00_JPRB, 0.39738E+00_JPRB, 0.38269E+00_JPRB, 0.42779E+00_JPRB, 0.47467E+00_JPRB ,&
&  0.55832E+00_JPRB, 0.62668E+00_JPRB, 0.71908E+00_JPRB, 0.83134E+00_JPRB, 0.99587E+00_JPRB, 0.12075E+01_JPRB, 0.15202E+01_JPRB ,&
&  0.19531E+01_JPRB, 0.18041E+01_JPRB, 0.19088E+01_JPRB, 0.18300E+01_JPRB, 0.17514E+01_JPRB, 0.15707E+01_JPRB, 0.13874E+01_JPRB ,&
&  0.12162E+01_JPRB, 0.10901E+01_JPRB, 0.95987E+00_JPRB, 0.84328E+00_JPRB, 0.75838E+00_JPRB, 0.69699E+00_JPRB, 0.66122E+00_JPRB ,&
&  0.64739E+00_JPRB, 0.63244E+00_JPRB, 0.61744E+00_JPRB, 0.63821E+00_JPRB, 0.74820E+00_JPRB, 0.89592E+00_JPRB, 0.11494E+01_JPRB ,&
&  0.13758E+01_JPRB, 0.13508E+01_JPRB, 0.17741E+01_JPRB, 0.19337E+01_JPRB, 0.19701E+01_JPRB, 0.19217E+01_JPRB, 0.19208E+01_JPRB ,&
&  0.20116E+01_JPRB, 0.18922E+01_JPRB, 0.17589E+01_JPRB, 0.14310E+01_JPRB, 0.15120E+01_JPRB, 0.14063E+01_JPRB, 0.12499E+01_JPRB ,&
&  0.10960E+01_JPRB, 0.10180E+01_JPRB, 0.91299E+00_JPRB, 0.84313E+00_JPRB, 0.78040E+00_JPRB, 0.72608E+00_JPRB, 0.58034E+00_JPRB ,&
&  0.49642E+00_JPRB, 0.51964E+00_JPRB, 0.49396E+00_JPRB, 0.46917E+00_JPRB, 0.44071E+00_JPRB, 0.45441E+00_JPRB, 0.46564E+00_JPRB ,&
&  0.47546E+00_JPRB, 0.47608E+00_JPRB, 0.47037E+00_JPRB, 0.48390E+00_JPRB, 0.50654E+00_JPRB, 0.51837E+00_JPRB, 0.51347E+00_JPRB ,&
&  0.50019E+00_JPRB, 0.50933E+00_JPRB, 0.51915E+00_JPRB, 0.51340E+00_JPRB, 0.54536E+00_JPRB, 0.53358E+00_JPRB, 0.51594E+00_JPRB ,&
&  0.50247E+00_JPRB, 0.49414E+00_JPRB, 0.48671E+00_JPRB, 0.47736E+00_JPRB, 0.46792E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.41995E+00_JPRB, 0.42331E+00_JPRB, 0.42680E+00_JPRB, 0.43314E+00_JPRB, 0.44006E+00_JPRB, 0.45047E+00_JPRB, 0.45767E+00_JPRB ,&
&  0.45917E+00_JPRB, 0.45058E+00_JPRB, 0.44558E+00_JPRB, 0.45275E+00_JPRB, 0.44895E+00_JPRB, 0.44607E+00_JPRB, 0.45730E+00_JPRB ,&
&  0.49150E+00_JPRB, 0.60060E+00_JPRB, 0.50093E+00_JPRB, 0.46069E+00_JPRB, 0.42177E+00_JPRB, 0.44966E+00_JPRB, 0.48122E+00_JPRB ,&
&  0.57152E+00_JPRB, 0.61006E+00_JPRB, 0.63480E+00_JPRB, 0.68801E+00_JPRB, 0.81527E+00_JPRB, 0.10101E+01_JPRB, 0.12441E+01_JPRB ,&
&  0.13775E+01_JPRB, 0.15451E+01_JPRB, 0.15619E+01_JPRB, 0.17434E+01_JPRB, 0.16465E+01_JPRB, 0.14106E+01_JPRB, 0.13717E+01_JPRB ,&
&  0.12223E+01_JPRB, 0.10836E+01_JPRB, 0.94977E+00_JPRB, 0.84682E+00_JPRB, 0.76930E+00_JPRB, 0.71197E+00_JPRB, 0.66882E+00_JPRB ,&
&  0.63821E+00_JPRB, 0.61196E+00_JPRB, 0.58410E+00_JPRB, 0.57275E+00_JPRB, 0.63560E+00_JPRB, 0.88678E+00_JPRB, 0.12139E+01_JPRB ,&
&  0.18543E+01_JPRB, 0.17118E+01_JPRB, 0.14423E+01_JPRB, 0.18907E+01_JPRB, 0.22926E+01_JPRB, 0.22180E+01_JPRB, 0.20464E+01_JPRB ,&
&  0.21652E+01_JPRB, 0.21211E+01_JPRB, 0.19840E+01_JPRB, 0.18883E+01_JPRB, 0.17389E+01_JPRB, 0.15481E+01_JPRB, 0.13338E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.96360E+00_JPRB, 0.88926E+00_JPRB, 0.82035E+00_JPRB, 0.73829E+00_JPRB, 0.68075E+00_JPRB, 0.64033E+00_JPRB ,&
&  0.57573E+00_JPRB, 0.52905E+00_JPRB, 0.45079E+00_JPRB, 0.46733E+00_JPRB, 0.41907E+00_JPRB, 0.42911E+00_JPRB, 0.46453E+00_JPRB ,&
&  0.46817E+00_JPRB, 0.48999E+00_JPRB, 0.49377E+00_JPRB, 0.48218E+00_JPRB, 0.51269E+00_JPRB, 0.50619E+00_JPRB, 0.48900E+00_JPRB ,&
&  0.47350E+00_JPRB, 0.44220E+00_JPRB, 0.45103E+00_JPRB, 0.45365E+00_JPRB, 0.45050E+00_JPRB, 0.44350E+00_JPRB, 0.43612E+00_JPRB ,&
&  0.43086E+00_JPRB, 0.42786E+00_JPRB, 0.42716E+00_JPRB, 0.42238E+00_JPRB, 0.41911E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.38410E+00_JPRB, 0.38725E+00_JPRB, 0.38881E+00_JPRB, 0.39555E+00_JPRB, 0.40524E+00_JPRB, 0.41768E+00_JPRB, 0.43054E+00_JPRB ,&
&  0.43976E+00_JPRB, 0.43937E+00_JPRB, 0.43365E+00_JPRB, 0.43597E+00_JPRB, 0.43659E+00_JPRB, 0.43662E+00_JPRB, 0.44701E+00_JPRB ,&
&  0.49174E+00_JPRB, 0.54572E+00_JPRB, 0.48857E+00_JPRB, 0.43320E+00_JPRB, 0.45196E+00_JPRB, 0.45669E+00_JPRB, 0.48828E+00_JPRB ,&
&  0.51471E+00_JPRB, 0.56820E+00_JPRB, 0.59271E+00_JPRB, 0.61524E+00_JPRB, 0.67032E+00_JPRB, 0.81299E+00_JPRB, 0.10000E+01_JPRB ,&
&  0.11032E+01_JPRB, 0.11711E+01_JPRB, 0.12703E+01_JPRB, 0.13655E+01_JPRB, 0.13772E+01_JPRB, 0.12290E+01_JPRB, 0.12122E+01_JPRB ,&
&  0.11170E+01_JPRB, 0.10021E+01_JPRB, 0.87840E+00_JPRB, 0.77737E+00_JPRB, 0.70703E+00_JPRB, 0.66045E+00_JPRB, 0.63122E+00_JPRB ,&
&  0.61304E+00_JPRB, 0.59175E+00_JPRB, 0.56189E+00_JPRB, 0.55750E+00_JPRB, 0.60480E+00_JPRB, 0.97036E+00_JPRB, 0.16406E+01_JPRB ,&
&  0.23208E+01_JPRB, 0.24101E+01_JPRB, 0.24810E+01_JPRB, 0.26334E+01_JPRB, 0.26383E+01_JPRB, 0.26617E+01_JPRB, 0.26418E+01_JPRB ,&
&  0.24896E+01_JPRB, 0.23181E+01_JPRB, 0.20645E+01_JPRB, 0.17868E+01_JPRB, 0.15975E+01_JPRB, 0.14600E+01_JPRB, 0.12829E+01_JPRB ,&
&  0.10628E+01_JPRB, 0.92884E+00_JPRB, 0.87353E+00_JPRB, 0.80010E+00_JPRB, 0.74820E+00_JPRB, 0.68385E+00_JPRB, 0.66987E+00_JPRB ,&
&  0.64560E+00_JPRB, 0.46355E+00_JPRB, 0.42310E+00_JPRB, 0.46267E+00_JPRB, 0.42637E+00_JPRB, 0.44084E+00_JPRB, 0.48025E+00_JPRB ,&
&  0.48206E+00_JPRB, 0.48227E+00_JPRB, 0.49256E+00_JPRB, 0.47666E+00_JPRB, 0.48031E+00_JPRB, 0.48377E+00_JPRB, 0.45890E+00_JPRB ,&
&  0.43809E+00_JPRB, 0.42704E+00_JPRB, 0.43250E+00_JPRB, 0.42159E+00_JPRB, 0.40775E+00_JPRB, 0.39742E+00_JPRB, 0.39090E+00_JPRB ,&
&  0.38667E+00_JPRB, 0.38590E+00_JPRB, 0.38784E+00_JPRB, 0.38344E+00_JPRB, 0.38054E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.35683E+00_JPRB, 0.35736E+00_JPRB, 0.36082E+00_JPRB, 0.36916E+00_JPRB, 0.38011E+00_JPRB, 0.39459E+00_JPRB, 0.40990E+00_JPRB ,&
&  0.42437E+00_JPRB, 0.43031E+00_JPRB, 0.42475E+00_JPRB, 0.42132E+00_JPRB, 0.42240E+00_JPRB, 0.42535E+00_JPRB, 0.44126E+00_JPRB ,&
&  0.40849E+00_JPRB, 0.43178E+00_JPRB, 0.39474E+00_JPRB, 0.40331E+00_JPRB, 0.44746E+00_JPRB, 0.45253E+00_JPRB, 0.46611E+00_JPRB ,&
&  0.48164E+00_JPRB, 0.51019E+00_JPRB, 0.52809E+00_JPRB, 0.54708E+00_JPRB, 0.58035E+00_JPRB, 0.66270E+00_JPRB, 0.81688E+00_JPRB ,&
&  0.86992E+00_JPRB, 0.86063E+00_JPRB, 0.91373E+00_JPRB, 0.96989E+00_JPRB, 0.96761E+00_JPRB, 0.10124E+01_JPRB, 0.10384E+01_JPRB ,&
&  0.97920E+00_JPRB, 0.90573E+00_JPRB, 0.82620E+00_JPRB, 0.74889E+00_JPRB, 0.68276E+00_JPRB, 0.62711E+00_JPRB, 0.58930E+00_JPRB ,&
&  0.56279E+00_JPRB, 0.54359E+00_JPRB, 0.53001E+00_JPRB, 0.56264E+00_JPRB, 0.62318E+00_JPRB, 0.11548E+01_JPRB, 0.20979E+01_JPRB ,&
&  0.25849E+01_JPRB, 0.26040E+01_JPRB, 0.27533E+01_JPRB, 0.30519E+01_JPRB, 0.29579E+01_JPRB, 0.28242E+01_JPRB, 0.26391E+01_JPRB ,&
&  0.24180E+01_JPRB, 0.21495E+01_JPRB, 0.18836E+01_JPRB, 0.16837E+01_JPRB, 0.14472E+01_JPRB, 0.13306E+01_JPRB, 0.11620E+01_JPRB ,&
&  0.97534E+00_JPRB, 0.88583E+00_JPRB, 0.85041E+00_JPRB, 0.81528E+00_JPRB, 0.77631E+00_JPRB, 0.76961E+00_JPRB, 0.73868E+00_JPRB ,&
&  0.68635E+00_JPRB, 0.60112E+00_JPRB, 0.49941E+00_JPRB, 0.48507E+00_JPRB, 0.43563E+00_JPRB, 0.45325E+00_JPRB, 0.50068E+00_JPRB ,&
&  0.49613E+00_JPRB, 0.47138E+00_JPRB, 0.45270E+00_JPRB, 0.44369E+00_JPRB, 0.44487E+00_JPRB, 0.45915E+00_JPRB, 0.43526E+00_JPRB ,&
&  0.41523E+00_JPRB, 0.41967E+00_JPRB, 0.41842E+00_JPRB, 0.40946E+00_JPRB, 0.39747E+00_JPRB, 0.38369E+00_JPRB, 0.37315E+00_JPRB ,&
&  0.37211E+00_JPRB, 0.37136E+00_JPRB, 0.36660E+00_JPRB, 0.35857E+00_JPRB, 0.35579E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.33935E+00_JPRB, 0.33945E+00_JPRB, 0.34198E+00_JPRB, 0.35120E+00_JPRB, 0.36352E+00_JPRB, 0.37614E+00_JPRB, 0.39126E+00_JPRB ,&
&  0.40732E+00_JPRB, 0.41700E+00_JPRB, 0.41549E+00_JPRB, 0.40991E+00_JPRB, 0.40794E+00_JPRB, 0.41153E+00_JPRB, 0.39290E+00_JPRB ,&
&  0.35782E+00_JPRB, 0.37860E+00_JPRB, 0.37352E+00_JPRB, 0.38119E+00_JPRB, 0.40438E+00_JPRB, 0.41144E+00_JPRB, 0.42118E+00_JPRB ,&
&  0.44243E+00_JPRB, 0.45400E+00_JPRB, 0.46879E+00_JPRB, 0.48749E+00_JPRB, 0.51268E+00_JPRB, 0.57117E+00_JPRB, 0.67082E+00_JPRB ,&
&  0.69747E+00_JPRB, 0.71383E+00_JPRB, 0.72823E+00_JPRB, 0.76249E+00_JPRB, 0.80787E+00_JPRB, 0.83664E+00_JPRB, 0.82189E+00_JPRB ,&
&  0.78480E+00_JPRB, 0.74260E+00_JPRB, 0.71085E+00_JPRB, 0.68157E+00_JPRB, 0.64097E+00_JPRB, 0.59640E+00_JPRB, 0.56077E+00_JPRB ,&
&  0.53219E+00_JPRB, 0.51551E+00_JPRB, 0.51633E+00_JPRB, 0.53906E+00_JPRB, 0.62732E+00_JPRB, 0.10438E+01_JPRB, 0.16146E+01_JPRB ,&
&  0.18863E+01_JPRB, 0.20976E+01_JPRB, 0.21692E+01_JPRB, 0.22190E+01_JPRB, 0.22458E+01_JPRB, 0.22155E+01_JPRB, 0.21300E+01_JPRB ,&
&  0.19868E+01_JPRB, 0.17410E+01_JPRB, 0.15770E+01_JPRB, 0.14509E+01_JPRB, 0.12995E+01_JPRB, 0.11916E+01_JPRB, 0.10323E+01_JPRB ,&
&  0.89955E+00_JPRB, 0.80841E+00_JPRB, 0.80288E+00_JPRB, 0.78425E+00_JPRB, 0.75525E+00_JPRB, 0.74976E+00_JPRB, 0.71900E+00_JPRB ,&
&  0.69297E+00_JPRB, 0.65924E+00_JPRB, 0.58563E+00_JPRB, 0.53458E+00_JPRB, 0.48772E+00_JPRB, 0.49846E+00_JPRB, 0.47031E+00_JPRB ,&
&  0.43458E+00_JPRB, 0.39854E+00_JPRB, 0.40138E+00_JPRB, 0.40440E+00_JPRB, 0.41384E+00_JPRB, 0.41909E+00_JPRB, 0.40717E+00_JPRB ,&
&  0.40490E+00_JPRB, 0.41050E+00_JPRB, 0.40144E+00_JPRB, 0.39210E+00_JPRB, 0.38189E+00_JPRB, 0.36596E+00_JPRB, 0.35489E+00_JPRB ,&
&  0.35129E+00_JPRB, 0.34919E+00_JPRB, 0.34456E+00_JPRB, 0.33860E+00_JPRB, 0.33785E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.33386E+00_JPRB, 0.33986E+00_JPRB, 0.34249E+00_JPRB, 0.34744E+00_JPRB, 0.35587E+00_JPRB, 0.36431E+00_JPRB, 0.37053E+00_JPRB ,&
&  0.38916E+00_JPRB, 0.39738E+00_JPRB, 0.40024E+00_JPRB, 0.39905E+00_JPRB, 0.39406E+00_JPRB, 0.36392E+00_JPRB, 0.31576E+00_JPRB ,&
&  0.30688E+00_JPRB, 0.32116E+00_JPRB, 0.34131E+00_JPRB, 0.35346E+00_JPRB, 0.36471E+00_JPRB, 0.37427E+00_JPRB, 0.38115E+00_JPRB ,&
&  0.38624E+00_JPRB, 0.39185E+00_JPRB, 0.40653E+00_JPRB, 0.42921E+00_JPRB, 0.45387E+00_JPRB, 0.48699E+00_JPRB, 0.54584E+00_JPRB ,&
&  0.56233E+00_JPRB, 0.58943E+00_JPRB, 0.62405E+00_JPRB, 0.60392E+00_JPRB, 0.67123E+00_JPRB, 0.68497E+00_JPRB, 0.67953E+00_JPRB ,&
&  0.66730E+00_JPRB, 0.64118E+00_JPRB, 0.61031E+00_JPRB, 0.58510E+00_JPRB, 0.56221E+00_JPRB, 0.54265E+00_JPRB, 0.51482E+00_JPRB ,&
&  0.49344E+00_JPRB, 0.48048E+00_JPRB, 0.48014E+00_JPRB, 0.49208E+00_JPRB, 0.53751E+00_JPRB, 0.80156E+00_JPRB, 0.10367E+01_JPRB ,&
&  0.12228E+01_JPRB, 0.14043E+01_JPRB, 0.14715E+01_JPRB, 0.14580E+01_JPRB, 0.16647E+01_JPRB, 0.18274E+01_JPRB, 0.17756E+01_JPRB ,&
&  0.16945E+01_JPRB, 0.14748E+01_JPRB, 0.13484E+01_JPRB, 0.12402E+01_JPRB, 0.11533E+01_JPRB, 0.10367E+01_JPRB, 0.89975E+00_JPRB ,&
&  0.80256E+00_JPRB, 0.77305E+00_JPRB, 0.73768E+00_JPRB, 0.70025E+00_JPRB, 0.66254E+00_JPRB, 0.64369E+00_JPRB, 0.61982E+00_JPRB ,&
&  0.60931E+00_JPRB, 0.59631E+00_JPRB, 0.56733E+00_JPRB, 0.53782E+00_JPRB, 0.50729E+00_JPRB, 0.47779E+00_JPRB, 0.44982E+00_JPRB ,&
&  0.42322E+00_JPRB, 0.39286E+00_JPRB, 0.38484E+00_JPRB, 0.38967E+00_JPRB, 0.39442E+00_JPRB, 0.38963E+00_JPRB, 0.38804E+00_JPRB ,&
&  0.38247E+00_JPRB, 0.38621E+00_JPRB, 0.38573E+00_JPRB, 0.37533E+00_JPRB, 0.36125E+00_JPRB, 0.34660E+00_JPRB, 0.33703E+00_JPRB ,&
&  0.32930E+00_JPRB, 0.32928E+00_JPRB, 0.32987E+00_JPRB, 0.32831E+00_JPRB, 0.32869E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.32465E+00_JPRB, 0.33429E+00_JPRB, 0.33873E+00_JPRB, 0.34040E+00_JPRB, 0.34523E+00_JPRB, 0.34753E+00_JPRB, 0.35042E+00_JPRB ,&
&  0.39327E+00_JPRB, 0.37011E+00_JPRB, 0.38217E+00_JPRB, 0.33661E+00_JPRB, 0.31914E+00_JPRB, 0.31420E+00_JPRB, 0.30586E+00_JPRB ,&
&  0.30760E+00_JPRB, 0.31597E+00_JPRB, 0.32430E+00_JPRB, 0.33145E+00_JPRB, 0.34293E+00_JPRB, 0.34873E+00_JPRB, 0.35224E+00_JPRB ,&
&  0.35764E+00_JPRB, 0.36704E+00_JPRB, 0.38209E+00_JPRB, 0.39483E+00_JPRB, 0.41468E+00_JPRB, 0.44327E+00_JPRB, 0.48672E+00_JPRB ,&
&  0.50388E+00_JPRB, 0.54129E+00_JPRB, 0.60580E+00_JPRB, 0.62751E+00_JPRB, 0.64136E+00_JPRB, 0.63467E+00_JPRB, 0.62108E+00_JPRB ,&
&  0.60005E+00_JPRB, 0.52130E+00_JPRB, 0.53646E+00_JPRB, 0.51819E+00_JPRB, 0.50063E+00_JPRB, 0.48228E+00_JPRB, 0.46516E+00_JPRB ,&
&  0.45141E+00_JPRB, 0.44307E+00_JPRB, 0.44184E+00_JPRB, 0.44345E+00_JPRB, 0.45460E+00_JPRB, 0.53011E+00_JPRB, 0.63248E+00_JPRB ,&
&  0.83652E+00_JPRB, 0.83058E+00_JPRB, 0.10742E+01_JPRB, 0.11592E+01_JPRB, 0.13897E+01_JPRB, 0.15135E+01_JPRB, 0.14912E+01_JPRB ,&
&  0.14046E+01_JPRB, 0.12459E+01_JPRB, 0.10981E+01_JPRB, 0.99588E+00_JPRB, 0.94783E+00_JPRB, 0.87906E+00_JPRB, 0.80381E+00_JPRB ,&
&  0.73012E+00_JPRB, 0.66272E+00_JPRB, 0.64971E+00_JPRB, 0.61553E+00_JPRB, 0.58116E+00_JPRB, 0.56395E+00_JPRB, 0.54613E+00_JPRB ,&
&  0.52980E+00_JPRB, 0.51174E+00_JPRB, 0.49000E+00_JPRB, 0.45617E+00_JPRB, 0.44520E+00_JPRB, 0.43376E+00_JPRB, 0.41608E+00_JPRB ,&
&  0.40866E+00_JPRB, 0.39719E+00_JPRB, 0.38909E+00_JPRB, 0.39279E+00_JPRB, 0.38774E+00_JPRB, 0.38550E+00_JPRB, 0.38132E+00_JPRB ,&
&  0.36888E+00_JPRB, 0.36261E+00_JPRB, 0.35816E+00_JPRB, 0.35280E+00_JPRB, 0.34384E+00_JPRB, 0.33745E+00_JPRB, 0.32877E+00_JPRB ,&
&  0.32759E+00_JPRB, 0.32639E+00_JPRB, 0.32511E+00_JPRB, 0.32101E+00_JPRB, 0.32130E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.32349E+00_JPRB, 0.32959E+00_JPRB, 0.33220E+00_JPRB, 0.33120E+00_JPRB, 0.33219E+00_JPRB, 0.33345E+00_JPRB, 0.32918E+00_JPRB ,&
&  0.34525E+00_JPRB, 0.32031E+00_JPRB, 0.30656E+00_JPRB, 0.30130E+00_JPRB, 0.30190E+00_JPRB, 0.30024E+00_JPRB, 0.29610E+00_JPRB ,&
&  0.29813E+00_JPRB, 0.30567E+00_JPRB, 0.31546E+00_JPRB, 0.32278E+00_JPRB, 0.32940E+00_JPRB, 0.33423E+00_JPRB, 0.33840E+00_JPRB ,&
&  0.34470E+00_JPRB, 0.35323E+00_JPRB, 0.36423E+00_JPRB, 0.37944E+00_JPRB, 0.39893E+00_JPRB, 0.42505E+00_JPRB, 0.45092E+00_JPRB ,&
&  0.49052E+00_JPRB, 0.54427E+00_JPRB, 0.58647E+00_JPRB, 0.62554E+00_JPRB, 0.63478E+00_JPRB, 0.61758E+00_JPRB, 0.57938E+00_JPRB ,&
&  0.41462E+00_JPRB, 0.39181E+00_JPRB, 0.45343E+00_JPRB, 0.45007E+00_JPRB, 0.44169E+00_JPRB, 0.42978E+00_JPRB, 0.42283E+00_JPRB ,&
&  0.41920E+00_JPRB, 0.41519E+00_JPRB, 0.41463E+00_JPRB, 0.41394E+00_JPRB, 0.41701E+00_JPRB, 0.43227E+00_JPRB, 0.47027E+00_JPRB ,&
&  0.56861E+00_JPRB, 0.58871E+00_JPRB, 0.64306E+00_JPRB, 0.73197E+00_JPRB, 0.99781E+00_JPRB, 0.10375E+01_JPRB, 0.10224E+01_JPRB ,&
&  0.10257E+01_JPRB, 0.95587E+00_JPRB, 0.89253E+00_JPRB, 0.83722E+00_JPRB, 0.80182E+00_JPRB, 0.76988E+00_JPRB, 0.73630E+00_JPRB ,&
&  0.68534E+00_JPRB, 0.63733E+00_JPRB, 0.59100E+00_JPRB, 0.56556E+00_JPRB, 0.54320E+00_JPRB, 0.52600E+00_JPRB, 0.51073E+00_JPRB ,&
&  0.49559E+00_JPRB, 0.47667E+00_JPRB, 0.45108E+00_JPRB, 0.42723E+00_JPRB, 0.40974E+00_JPRB, 0.40178E+00_JPRB, 0.39690E+00_JPRB ,&
&  0.39305E+00_JPRB, 0.39072E+00_JPRB, 0.38846E+00_JPRB, 0.38673E+00_JPRB, 0.38305E+00_JPRB, 0.37539E+00_JPRB, 0.36711E+00_JPRB ,&
&  0.35643E+00_JPRB, 0.34421E+00_JPRB, 0.33513E+00_JPRB, 0.32709E+00_JPRB, 0.32265E+00_JPRB, 0.32098E+00_JPRB, 0.32380E+00_JPRB ,&
&  0.32272E+00_JPRB, 0.31560E+00_JPRB, 0.31497E+00_JPRB, 0.31493E+00_JPRB, 0.31946E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.32339E+00_JPRB, 0.32601E+00_JPRB, 0.32542E+00_JPRB, 0.32076E+00_JPRB, 0.31726E+00_JPRB, 0.31753E+00_JPRB, 0.31821E+00_JPRB ,&
&  0.32793E+00_JPRB, 0.31681E+00_JPRB, 0.30556E+00_JPRB, 0.29691E+00_JPRB, 0.29245E+00_JPRB, 0.29118E+00_JPRB, 0.29406E+00_JPRB ,&
&  0.29919E+00_JPRB, 0.30421E+00_JPRB, 0.30875E+00_JPRB, 0.31345E+00_JPRB, 0.31808E+00_JPRB, 0.32167E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.33216E+00_JPRB, 0.33866E+00_JPRB, 0.34509E+00_JPRB, 0.35610E+00_JPRB, 0.37225E+00_JPRB, 0.39669E+00_JPRB, 0.41566E+00_JPRB ,&
&  0.43996E+00_JPRB, 0.47316E+00_JPRB, 0.52118E+00_JPRB, 0.58839E+00_JPRB, 0.61848E+00_JPRB, 0.59260E+00_JPRB, 0.45454E+00_JPRB ,&
&  0.35368E+00_JPRB, 0.32470E+00_JPRB, 0.33723E+00_JPRB, 0.38531E+00_JPRB, 0.38767E+00_JPRB, 0.38784E+00_JPRB, 0.38886E+00_JPRB ,&
&  0.37677E+00_JPRB, 0.37073E+00_JPRB, 0.37602E+00_JPRB, 0.38905E+00_JPRB, 0.39051E+00_JPRB, 0.39407E+00_JPRB, 0.41310E+00_JPRB ,&
&  0.45125E+00_JPRB, 0.49840E+00_JPRB, 0.55260E+00_JPRB, 0.55496E+00_JPRB, 0.63384E+00_JPRB, 0.87467E+00_JPRB, 0.88675E+00_JPRB ,&
&  0.83098E+00_JPRB, 0.77282E+00_JPRB, 0.75069E+00_JPRB, 0.70390E+00_JPRB, 0.66807E+00_JPRB, 0.64309E+00_JPRB, 0.61584E+00_JPRB ,&
&  0.58224E+00_JPRB, 0.54653E+00_JPRB, 0.51028E+00_JPRB, 0.49661E+00_JPRB, 0.48843E+00_JPRB, 0.48120E+00_JPRB, 0.47691E+00_JPRB ,&
&  0.46715E+00_JPRB, 0.44436E+00_JPRB, 0.41905E+00_JPRB, 0.39511E+00_JPRB, 0.38352E+00_JPRB, 0.37748E+00_JPRB, 0.37443E+00_JPRB ,&
&  0.37126E+00_JPRB, 0.37067E+00_JPRB, 0.37216E+00_JPRB, 0.36958E+00_JPRB, 0.35814E+00_JPRB, 0.34454E+00_JPRB, 0.33761E+00_JPRB ,&
&  0.33402E+00_JPRB, 0.32704E+00_JPRB, 0.32147E+00_JPRB, 0.31477E+00_JPRB, 0.31318E+00_JPRB, 0.31071E+00_JPRB, 0.30862E+00_JPRB ,&
&  0.31165E+00_JPRB, 0.30871E+00_JPRB, 0.30758E+00_JPRB, 0.30884E+00_JPRB, 0.31650E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.32273E+00_JPRB, 0.32693E+00_JPRB, 0.32314E+00_JPRB, 0.31706E+00_JPRB, 0.30632E+00_JPRB, 0.29971E+00_JPRB, 0.29573E+00_JPRB ,&
&  0.29645E+00_JPRB, 0.29500E+00_JPRB, 0.29424E+00_JPRB, 0.29468E+00_JPRB, 0.29581E+00_JPRB, 0.29815E+00_JPRB, 0.30024E+00_JPRB ,&
&  0.30191E+00_JPRB, 0.30426E+00_JPRB, 0.30701E+00_JPRB, 0.30958E+00_JPRB, 0.31229E+00_JPRB, 0.31646E+00_JPRB, 0.32122E+00_JPRB ,&
&  0.32634E+00_JPRB, 0.32892E+00_JPRB, 0.33048E+00_JPRB, 0.33428E+00_JPRB, 0.34336E+00_JPRB, 0.35939E+00_JPRB, 0.37940E+00_JPRB ,&
&  0.40057E+00_JPRB, 0.42124E+00_JPRB, 0.46318E+00_JPRB, 0.53474E+00_JPRB, 0.57118E+00_JPRB, 0.54053E+00_JPRB, 0.42266E+00_JPRB ,&
&  0.34034E+00_JPRB, 0.29877E+00_JPRB, 0.28830E+00_JPRB, 0.29709E+00_JPRB, 0.31789E+00_JPRB, 0.34263E+00_JPRB, 0.34907E+00_JPRB ,&
&  0.35336E+00_JPRB, 0.35574E+00_JPRB, 0.35811E+00_JPRB, 0.36119E+00_JPRB, 0.36300E+00_JPRB, 0.36637E+00_JPRB, 0.37864E+00_JPRB ,&
&  0.40129E+00_JPRB, 0.42935E+00_JPRB, 0.45720E+00_JPRB, 0.48973E+00_JPRB, 0.51837E+00_JPRB, 0.57916E+00_JPRB, 0.63588E+00_JPRB ,&
&  0.61727E+00_JPRB, 0.56467E+00_JPRB, 0.57576E+00_JPRB, 0.58401E+00_JPRB, 0.58486E+00_JPRB, 0.56962E+00_JPRB, 0.54405E+00_JPRB ,&
&  0.51849E+00_JPRB, 0.48487E+00_JPRB, 0.45396E+00_JPRB, 0.43402E+00_JPRB, 0.43035E+00_JPRB, 0.43867E+00_JPRB, 0.44551E+00_JPRB ,&
&  0.44725E+00_JPRB, 0.43300E+00_JPRB, 0.40670E+00_JPRB, 0.37758E+00_JPRB, 0.36790E+00_JPRB, 0.36826E+00_JPRB, 0.36999E+00_JPRB ,&
&  0.36714E+00_JPRB, 0.36383E+00_JPRB, 0.36174E+00_JPRB, 0.35594E+00_JPRB, 0.34531E+00_JPRB, 0.33815E+00_JPRB, 0.33193E+00_JPRB ,&
&  0.32628E+00_JPRB, 0.32143E+00_JPRB, 0.32342E+00_JPRB, 0.31952E+00_JPRB, 0.31641E+00_JPRB, 0.31392E+00_JPRB, 0.31009E+00_JPRB ,&
&  0.30815E+00_JPRB, 0.30804E+00_JPRB, 0.30671E+00_JPRB, 0.30857E+00_JPRB, 0.31399E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.32083E+00_JPRB, 0.32294E+00_JPRB, 0.32353E+00_JPRB, 0.31818E+00_JPRB, 0.30844E+00_JPRB, 0.30057E+00_JPRB, 0.29595E+00_JPRB ,&
&  0.29582E+00_JPRB, 0.29766E+00_JPRB, 0.29708E+00_JPRB, 0.29783E+00_JPRB, 0.29903E+00_JPRB, 0.30065E+00_JPRB, 0.30286E+00_JPRB ,&
&  0.30555E+00_JPRB, 0.30797E+00_JPRB, 0.30933E+00_JPRB, 0.30997E+00_JPRB, 0.31226E+00_JPRB, 0.31631E+00_JPRB, 0.32055E+00_JPRB ,&
&  0.32304E+00_JPRB, 0.32109E+00_JPRB, 0.32170E+00_JPRB, 0.32478E+00_JPRB, 0.32969E+00_JPRB, 0.33677E+00_JPRB, 0.34592E+00_JPRB ,&
&  0.36297E+00_JPRB, 0.38888E+00_JPRB, 0.43502E+00_JPRB, 0.48939E+00_JPRB, 0.51437E+00_JPRB, 0.47696E+00_JPRB, 0.38976E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.28964E+00_JPRB, 0.27431E+00_JPRB, 0.27134E+00_JPRB, 0.27778E+00_JPRB, 0.29211E+00_JPRB, 0.31189E+00_JPRB ,&
&  0.32527E+00_JPRB, 0.33091E+00_JPRB, 0.33869E+00_JPRB, 0.34530E+00_JPRB, 0.35015E+00_JPRB, 0.35309E+00_JPRB, 0.35768E+00_JPRB ,&
&  0.36493E+00_JPRB, 0.37594E+00_JPRB, 0.39453E+00_JPRB, 0.42454E+00_JPRB, 0.46464E+00_JPRB, 0.46180E+00_JPRB, 0.47587E+00_JPRB ,&
&  0.50159E+00_JPRB, 0.48499E+00_JPRB, 0.49413E+00_JPRB, 0.48945E+00_JPRB, 0.48423E+00_JPRB, 0.46955E+00_JPRB, 0.45869E+00_JPRB ,&
&  0.44873E+00_JPRB, 0.43555E+00_JPRB, 0.41845E+00_JPRB, 0.40032E+00_JPRB, 0.38751E+00_JPRB, 0.38341E+00_JPRB, 0.38702E+00_JPRB ,&
&  0.39543E+00_JPRB, 0.40762E+00_JPRB, 0.45064E+00_JPRB, 0.40563E+00_JPRB, 0.38735E+00_JPRB, 0.38187E+00_JPRB, 0.37358E+00_JPRB ,&
&  0.36632E+00_JPRB, 0.36123E+00_JPRB, 0.35759E+00_JPRB, 0.35176E+00_JPRB, 0.34491E+00_JPRB, 0.33994E+00_JPRB, 0.33657E+00_JPRB ,&
&  0.33240E+00_JPRB, 0.32787E+00_JPRB, 0.32473E+00_JPRB, 0.32279E+00_JPRB, 0.32100E+00_JPRB, 0.31996E+00_JPRB, 0.31835E+00_JPRB ,&
&  0.31565E+00_JPRB, 0.31548E+00_JPRB, 0.31728E+00_JPRB, 0.31976E+00_JPRB, 0.32088E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.32605E+00_JPRB, 0.32508E+00_JPRB, 0.32074E+00_JPRB, 0.31778E+00_JPRB, 0.31205E+00_JPRB, 0.30746E+00_JPRB, 0.30246E+00_JPRB ,&
&  0.29958E+00_JPRB, 0.29867E+00_JPRB, 0.29781E+00_JPRB, 0.29863E+00_JPRB, 0.29982E+00_JPRB, 0.30086E+00_JPRB, 0.30286E+00_JPRB ,&
&  0.30481E+00_JPRB, 0.30408E+00_JPRB, 0.30568E+00_JPRB, 0.30847E+00_JPRB, 0.31112E+00_JPRB, 0.31359E+00_JPRB, 0.31521E+00_JPRB ,&
&  0.31398E+00_JPRB, 0.31107E+00_JPRB, 0.31201E+00_JPRB, 0.31492E+00_JPRB, 0.31951E+00_JPRB, 0.32736E+00_JPRB, 0.34289E+00_JPRB ,&
&  0.36292E+00_JPRB, 0.39073E+00_JPRB, 0.42202E+00_JPRB, 0.45093E+00_JPRB, 0.45411E+00_JPRB, 0.42344E+00_JPRB, 0.35069E+00_JPRB ,&
&  0.31202E+00_JPRB, 0.28729E+00_JPRB, 0.27668E+00_JPRB, 0.27226E+00_JPRB, 0.27558E+00_JPRB, 0.28449E+00_JPRB, 0.29992E+00_JPRB ,&
&  0.31056E+00_JPRB, 0.31716E+00_JPRB, 0.32151E+00_JPRB, 0.32563E+00_JPRB, 0.32855E+00_JPRB, 0.33181E+00_JPRB, 0.33434E+00_JPRB ,&
&  0.33949E+00_JPRB, 0.34194E+00_JPRB, 0.34337E+00_JPRB, 0.34768E+00_JPRB, 0.35511E+00_JPRB, 0.35981E+00_JPRB, 0.36601E+00_JPRB ,&
&  0.38195E+00_JPRB, 0.38420E+00_JPRB, 0.38606E+00_JPRB, 0.38595E+00_JPRB, 0.39094E+00_JPRB, 0.38786E+00_JPRB, 0.38440E+00_JPRB ,&
&  0.37990E+00_JPRB, 0.38102E+00_JPRB, 0.37641E+00_JPRB, 0.37153E+00_JPRB, 0.36593E+00_JPRB, 0.36097E+00_JPRB, 0.35871E+00_JPRB ,&
&  0.35465E+00_JPRB, 0.35718E+00_JPRB, 0.37467E+00_JPRB, 0.37548E+00_JPRB, 0.36639E+00_JPRB, 0.35585E+00_JPRB, 0.34605E+00_JPRB ,&
&  0.33903E+00_JPRB, 0.33358E+00_JPRB, 0.32967E+00_JPRB, 0.32671E+00_JPRB, 0.32552E+00_JPRB, 0.32444E+00_JPRB, 0.32472E+00_JPRB ,&
&  0.32276E+00_JPRB, 0.31992E+00_JPRB, 0.31721E+00_JPRB, 0.31646E+00_JPRB, 0.31761E+00_JPRB, 0.31846E+00_JPRB, 0.31853E+00_JPRB ,&
&  0.31851E+00_JPRB, 0.32132E+00_JPRB, 0.32622E+00_JPRB, 0.32905E+00_JPRB, 0.32815E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.32557E+00_JPRB, 0.32504E+00_JPRB, 0.32165E+00_JPRB, 0.31778E+00_JPRB, 0.31305E+00_JPRB, 0.30985E+00_JPRB, 0.30660E+00_JPRB ,&
&  0.30487E+00_JPRB, 0.30372E+00_JPRB, 0.30266E+00_JPRB, 0.30200E+00_JPRB, 0.30183E+00_JPRB, 0.30166E+00_JPRB, 0.30228E+00_JPRB ,&
&  0.30335E+00_JPRB, 0.30491E+00_JPRB, 0.30622E+00_JPRB, 0.30744E+00_JPRB, 0.30855E+00_JPRB, 0.31019E+00_JPRB, 0.31071E+00_JPRB ,&
&  0.31107E+00_JPRB, 0.30837E+00_JPRB, 0.30717E+00_JPRB, 0.31458E+00_JPRB, 0.32325E+00_JPRB, 0.32510E+00_JPRB, 0.34277E+00_JPRB ,&
&  0.36303E+00_JPRB, 0.38985E+00_JPRB, 0.40615E+00_JPRB, 0.41370E+00_JPRB, 0.40892E+00_JPRB, 0.33496E+00_JPRB, 0.31976E+00_JPRB ,&
&  0.30312E+00_JPRB, 0.28984E+00_JPRB, 0.28354E+00_JPRB, 0.27958E+00_JPRB, 0.27885E+00_JPRB, 0.28392E+00_JPRB, 0.29220E+00_JPRB ,&
&  0.29654E+00_JPRB, 0.31376E+00_JPRB, 0.31420E+00_JPRB, 0.31273E+00_JPRB, 0.31284E+00_JPRB, 0.31411E+00_JPRB, 0.31237E+00_JPRB ,&
&  0.31276E+00_JPRB, 0.31160E+00_JPRB, 0.30966E+00_JPRB, 0.31110E+00_JPRB, 0.31187E+00_JPRB, 0.31019E+00_JPRB, 0.31066E+00_JPRB ,&
&  0.31220E+00_JPRB, 0.31238E+00_JPRB, 0.31170E+00_JPRB, 0.31185E+00_JPRB, 0.31393E+00_JPRB, 0.31711E+00_JPRB, 0.32279E+00_JPRB ,&
&  0.32287E+00_JPRB, 0.32770E+00_JPRB, 0.33051E+00_JPRB, 0.33391E+00_JPRB, 0.33697E+00_JPRB, 0.33881E+00_JPRB, 0.34196E+00_JPRB ,&
&  0.34136E+00_JPRB, 0.34277E+00_JPRB, 0.34449E+00_JPRB, 0.34893E+00_JPRB, 0.34594E+00_JPRB, 0.33998E+00_JPRB, 0.33606E+00_JPRB ,&
&  0.33351E+00_JPRB, 0.33270E+00_JPRB, 0.32768E+00_JPRB, 0.32401E+00_JPRB, 0.32083E+00_JPRB, 0.31903E+00_JPRB, 0.31692E+00_JPRB ,&
&  0.31611E+00_JPRB, 0.31501E+00_JPRB, 0.31419E+00_JPRB, 0.31420E+00_JPRB, 0.31752E+00_JPRB, 0.32056E+00_JPRB, 0.32177E+00_JPRB ,&
&  0.32263E+00_JPRB, 0.32452E+00_JPRB, 0.32678E+00_JPRB, 0.32742E+00_JPRB, 0.32586E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.32190E+00_JPRB, 0.31940E+00_JPRB, 0.31721E+00_JPRB, 0.31381E+00_JPRB, 0.31037E+00_JPRB, 0.30787E+00_JPRB, 0.30516E+00_JPRB ,&
&  0.30376E+00_JPRB, 0.30349E+00_JPRB, 0.30369E+00_JPRB, 0.30418E+00_JPRB, 0.30474E+00_JPRB, 0.30513E+00_JPRB, 0.30614E+00_JPRB ,&
&  0.30634E+00_JPRB, 0.30722E+00_JPRB, 0.30769E+00_JPRB, 0.30829E+00_JPRB, 0.30845E+00_JPRB, 0.30856E+00_JPRB, 0.30821E+00_JPRB ,&
&  0.30649E+00_JPRB, 0.30288E+00_JPRB, 0.30388E+00_JPRB, 0.31475E+00_JPRB, 0.31859E+00_JPRB, 0.32473E+00_JPRB, 0.33345E+00_JPRB ,&
&  0.35238E+00_JPRB, 0.34443E+00_JPRB, 0.35137E+00_JPRB, 0.33682E+00_JPRB, 0.32513E+00_JPRB, 0.30899E+00_JPRB, 0.29535E+00_JPRB ,&
&  0.29005E+00_JPRB, 0.28372E+00_JPRB, 0.28046E+00_JPRB, 0.28051E+00_JPRB, 0.28225E+00_JPRB, 0.28764E+00_JPRB, 0.29426E+00_JPRB ,&
&  0.29719E+00_JPRB, 0.30889E+00_JPRB, 0.30960E+00_JPRB, 0.30836E+00_JPRB, 0.30883E+00_JPRB, 0.30871E+00_JPRB, 0.30515E+00_JPRB ,&
&  0.30387E+00_JPRB, 0.30110E+00_JPRB, 0.29847E+00_JPRB, 0.29841E+00_JPRB, 0.30115E+00_JPRB, 0.30160E+00_JPRB, 0.30170E+00_JPRB ,&
&  0.30164E+00_JPRB, 0.29995E+00_JPRB, 0.29711E+00_JPRB, 0.29503E+00_JPRB, 0.29382E+00_JPRB, 0.29366E+00_JPRB, 0.29396E+00_JPRB ,&
&  0.29560E+00_JPRB, 0.29923E+00_JPRB, 0.30229E+00_JPRB, 0.30852E+00_JPRB, 0.31439E+00_JPRB, 0.32249E+00_JPRB, 0.32919E+00_JPRB ,&
&  0.33425E+00_JPRB, 0.33802E+00_JPRB, 0.34098E+00_JPRB, 0.34470E+00_JPRB, 0.34808E+00_JPRB, 0.34575E+00_JPRB, 0.33983E+00_JPRB ,&
&  0.33620E+00_JPRB, 0.33057E+00_JPRB, 0.32612E+00_JPRB, 0.32179E+00_JPRB, 0.31816E+00_JPRB, 0.31601E+00_JPRB, 0.31404E+00_JPRB ,&
&  0.31309E+00_JPRB, 0.31210E+00_JPRB, 0.31204E+00_JPRB, 0.31320E+00_JPRB, 0.31711E+00_JPRB, 0.31985E+00_JPRB, 0.32138E+00_JPRB ,&
&  0.32240E+00_JPRB, 0.32456E+00_JPRB, 0.32558E+00_JPRB, 0.32508E+00_JPRB, 0.32367E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.31913E+00_JPRB, 0.31586E+00_JPRB, 0.31316E+00_JPRB, 0.30984E+00_JPRB, 0.30660E+00_JPRB, 0.30341E+00_JPRB, 0.30126E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29917E+00_JPRB, 0.29947E+00_JPRB, 0.30001E+00_JPRB, 0.30115E+00_JPRB, 0.30209E+00_JPRB, 0.30336E+00_JPRB ,&
&  0.30418E+00_JPRB, 0.30485E+00_JPRB, 0.30554E+00_JPRB, 0.30580E+00_JPRB, 0.30568E+00_JPRB, 0.30528E+00_JPRB, 0.30384E+00_JPRB ,&
&  0.30227E+00_JPRB, 0.30188E+00_JPRB, 0.29814E+00_JPRB, 0.30571E+00_JPRB, 0.30729E+00_JPRB, 0.30889E+00_JPRB, 0.31297E+00_JPRB ,&
&  0.31429E+00_JPRB, 0.31831E+00_JPRB, 0.31902E+00_JPRB, 0.31580E+00_JPRB, 0.31266E+00_JPRB, 0.30283E+00_JPRB, 0.29818E+00_JPRB ,&
&  0.29284E+00_JPRB, 0.28994E+00_JPRB, 0.28959E+00_JPRB, 0.28966E+00_JPRB, 0.29364E+00_JPRB, 0.29687E+00_JPRB, 0.30066E+00_JPRB ,&
&  0.30572E+00_JPRB, 0.30922E+00_JPRB, 0.31266E+00_JPRB, 0.30855E+00_JPRB, 0.30814E+00_JPRB, 0.30711E+00_JPRB, 0.30638E+00_JPRB ,&
&  0.30588E+00_JPRB, 0.30450E+00_JPRB, 0.30378E+00_JPRB, 0.30287E+00_JPRB, 0.30272E+00_JPRB, 0.30326E+00_JPRB, 0.30386E+00_JPRB ,&
&  0.30462E+00_JPRB, 0.30472E+00_JPRB, 0.30417E+00_JPRB, 0.30245E+00_JPRB, 0.30067E+00_JPRB, 0.29888E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29805E+00_JPRB, 0.29864E+00_JPRB, 0.30025E+00_JPRB, 0.30180E+00_JPRB, 0.30339E+00_JPRB, 0.30537E+00_JPRB, 0.30790E+00_JPRB ,&
&  0.31163E+00_JPRB, 0.31522E+00_JPRB, 0.32006E+00_JPRB, 0.32217E+00_JPRB, 0.32717E+00_JPRB, 0.33280E+00_JPRB, 0.33532E+00_JPRB ,&
&  0.33396E+00_JPRB, 0.33153E+00_JPRB, 0.32879E+00_JPRB, 0.32613E+00_JPRB, 0.32356E+00_JPRB, 0.32033E+00_JPRB, 0.31752E+00_JPRB ,&
&  0.31467E+00_JPRB, 0.31231E+00_JPRB, 0.31117E+00_JPRB, 0.31160E+00_JPRB, 0.31374E+00_JPRB, 0.31566E+00_JPRB, 0.31750E+00_JPRB ,&
&  0.31945E+00_JPRB, 0.32181E+00_JPRB, 0.32351E+00_JPRB, 0.32346E+00_JPRB, 0.32206E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.31332E+00_JPRB, 0.31226E+00_JPRB, 0.31072E+00_JPRB, 0.30865E+00_JPRB, 0.30686E+00_JPRB, 0.30543E+00_JPRB, 0.30443E+00_JPRB ,&
&  0.30323E+00_JPRB, 0.30134E+00_JPRB, 0.30036E+00_JPRB, 0.30026E+00_JPRB, 0.30049E+00_JPRB, 0.30113E+00_JPRB, 0.30200E+00_JPRB ,&
&  0.30284E+00_JPRB, 0.30341E+00_JPRB, 0.30426E+00_JPRB, 0.30480E+00_JPRB, 0.30545E+00_JPRB, 0.30571E+00_JPRB, 0.30469E+00_JPRB ,&
&  0.30381E+00_JPRB, 0.30292E+00_JPRB, 0.30198E+00_JPRB, 0.30170E+00_JPRB, 0.30267E+00_JPRB, 0.29509E+00_JPRB, 0.29650E+00_JPRB ,&
&  0.30383E+00_JPRB, 0.30790E+00_JPRB, 0.31310E+00_JPRB, 0.31646E+00_JPRB, 0.31488E+00_JPRB, 0.31221E+00_JPRB, 0.31139E+00_JPRB ,&
&  0.30216E+00_JPRB, 0.30629E+00_JPRB, 0.30663E+00_JPRB, 0.30704E+00_JPRB, 0.30889E+00_JPRB, 0.30813E+00_JPRB, 0.30951E+00_JPRB ,&
&  0.31623E+00_JPRB, 0.31537E+00_JPRB, 0.31476E+00_JPRB, 0.31338E+00_JPRB, 0.31165E+00_JPRB, 0.31014E+00_JPRB, 0.30898E+00_JPRB ,&
&  0.30761E+00_JPRB, 0.30666E+00_JPRB, 0.30633E+00_JPRB, 0.30522E+00_JPRB, 0.30459E+00_JPRB, 0.30456E+00_JPRB, 0.30480E+00_JPRB ,&
&  0.30449E+00_JPRB, 0.30438E+00_JPRB, 0.30444E+00_JPRB, 0.30411E+00_JPRB, 0.30381E+00_JPRB, 0.30311E+00_JPRB, 0.30263E+00_JPRB ,&
&  0.30184E+00_JPRB, 0.30066E+00_JPRB, 0.30052E+00_JPRB, 0.30031E+00_JPRB, 0.30008E+00_JPRB, 0.29940E+00_JPRB, 0.29973E+00_JPRB ,&
&  0.30000E+00_JPRB, 0.30015E+00_JPRB, 0.29992E+00_JPRB, 0.30007E+00_JPRB, 0.30108E+00_JPRB, 0.30284E+00_JPRB, 0.30638E+00_JPRB ,&
&  0.30933E+00_JPRB, 0.31168E+00_JPRB, 0.31229E+00_JPRB, 0.31264E+00_JPRB, 0.31291E+00_JPRB, 0.31205E+00_JPRB, 0.31083E+00_JPRB ,&
&  0.30913E+00_JPRB, 0.30736E+00_JPRB, 0.30605E+00_JPRB, 0.30505E+00_JPRB, 0.30432E+00_JPRB, 0.30385E+00_JPRB, 0.30445E+00_JPRB ,&
&  0.30670E+00_JPRB, 0.30934E+00_JPRB, 0.31201E+00_JPRB, 0.31318E+00_JPRB, 0.31377E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.30736E+00_JPRB, 0.30756E+00_JPRB, 0.30739E+00_JPRB, 0.30694E+00_JPRB, 0.30613E+00_JPRB, 0.30518E+00_JPRB, 0.30426E+00_JPRB ,&
&  0.30327E+00_JPRB, 0.30238E+00_JPRB, 0.30147E+00_JPRB, 0.30067E+00_JPRB, 0.30002E+00_JPRB, 0.29968E+00_JPRB, 0.29967E+00_JPRB ,&
&  0.30033E+00_JPRB, 0.30120E+00_JPRB, 0.30252E+00_JPRB, 0.30325E+00_JPRB, 0.30386E+00_JPRB, 0.30404E+00_JPRB, 0.30425E+00_JPRB ,&
&  0.30418E+00_JPRB, 0.30384E+00_JPRB, 0.30328E+00_JPRB, 0.30213E+00_JPRB, 0.30134E+00_JPRB, 0.30082E+00_JPRB, 0.30061E+00_JPRB ,&
&  0.30048E+00_JPRB, 0.30036E+00_JPRB, 0.30064E+00_JPRB, 0.30144E+00_JPRB, 0.30390E+00_JPRB, 0.30563E+00_JPRB, 0.30691E+00_JPRB ,&
&  0.30797E+00_JPRB, 0.30794E+00_JPRB, 0.30794E+00_JPRB, 0.30803E+00_JPRB, 0.30822E+00_JPRB, 0.30847E+00_JPRB, 0.30797E+00_JPRB ,&
&  0.30754E+00_JPRB, 0.30708E+00_JPRB, 0.30664E+00_JPRB, 0.30602E+00_JPRB, 0.30546E+00_JPRB, 0.30516E+00_JPRB, 0.30368E+00_JPRB ,&
&  0.30277E+00_JPRB, 0.30268E+00_JPRB, 0.30306E+00_JPRB, 0.30352E+00_JPRB, 0.30400E+00_JPRB, 0.30439E+00_JPRB, 0.30475E+00_JPRB ,&
&  0.30505E+00_JPRB, 0.30570E+00_JPRB, 0.30638E+00_JPRB, 0.30686E+00_JPRB, 0.30709E+00_JPRB, 0.30737E+00_JPRB, 0.30721E+00_JPRB ,&
&  0.30669E+00_JPRB, 0.30605E+00_JPRB, 0.30580E+00_JPRB, 0.30538E+00_JPRB, 0.30469E+00_JPRB, 0.30396E+00_JPRB, 0.30388E+00_JPRB ,&
&  0.30353E+00_JPRB, 0.30300E+00_JPRB, 0.30242E+00_JPRB, 0.30279E+00_JPRB, 0.30299E+00_JPRB, 0.30300E+00_JPRB, 0.30266E+00_JPRB ,&
&  0.30344E+00_JPRB, 0.30451E+00_JPRB, 0.30595E+00_JPRB, 0.30794E+00_JPRB, 0.30922E+00_JPRB, 0.30947E+00_JPRB, 0.30903E+00_JPRB ,&
&  0.30775E+00_JPRB, 0.30617E+00_JPRB, 0.30464E+00_JPRB, 0.30336E+00_JPRB, 0.30240E+00_JPRB, 0.30219E+00_JPRB, 0.30251E+00_JPRB ,&
&  0.30312E+00_JPRB, 0.30378E+00_JPRB, 0.30482E+00_JPRB, 0.30593E+00_JPRB, 0.30697E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.30232E+00_JPRB, 0.30240E+00_JPRB, 0.30240E+00_JPRB, 0.30229E+00_JPRB, 0.30203E+00_JPRB, 0.30169E+00_JPRB, 0.30137E+00_JPRB ,&
&  0.30100E+00_JPRB, 0.30077E+00_JPRB, 0.30038E+00_JPRB, 0.29999E+00_JPRB, 0.29960E+00_JPRB, 0.29927E+00_JPRB, 0.29897E+00_JPRB ,&
&  0.29870E+00_JPRB, 0.29847E+00_JPRB, 0.29844E+00_JPRB, 0.29842E+00_JPRB, 0.29837E+00_JPRB, 0.29835E+00_JPRB, 0.29835E+00_JPRB ,&
&  0.29845E+00_JPRB, 0.29850E+00_JPRB, 0.29851E+00_JPRB, 0.29844E+00_JPRB, 0.29856E+00_JPRB, 0.29850E+00_JPRB, 0.29857E+00_JPRB ,&
&  0.29856E+00_JPRB, 0.29868E+00_JPRB, 0.29874E+00_JPRB, 0.29889E+00_JPRB, 0.29888E+00_JPRB, 0.29913E+00_JPRB, 0.29919E+00_JPRB ,&
&  0.29934E+00_JPRB, 0.29930E+00_JPRB, 0.29929E+00_JPRB, 0.29927E+00_JPRB, 0.29932E+00_JPRB, 0.29922E+00_JPRB, 0.29921E+00_JPRB ,&
&  0.29922E+00_JPRB, 0.29933E+00_JPRB, 0.29948E+00_JPRB, 0.29960E+00_JPRB, 0.29964E+00_JPRB, 0.29975E+00_JPRB, 0.29973E+00_JPRB ,&
&  0.30037E+00_JPRB, 0.30125E+00_JPRB, 0.30219E+00_JPRB, 0.30319E+00_JPRB, 0.30418E+00_JPRB, 0.30498E+00_JPRB, 0.30572E+00_JPRB ,&
&  0.30628E+00_JPRB, 0.30680E+00_JPRB, 0.30737E+00_JPRB, 0.30801E+00_JPRB, 0.30847E+00_JPRB, 0.30895E+00_JPRB, 0.30933E+00_JPRB ,&
&  0.30955E+00_JPRB, 0.30948E+00_JPRB, 0.30951E+00_JPRB, 0.30950E+00_JPRB, 0.30953E+00_JPRB, 0.30946E+00_JPRB, 0.30950E+00_JPRB ,&
&  0.30947E+00_JPRB, 0.30955E+00_JPRB, 0.30961E+00_JPRB, 0.30965E+00_JPRB, 0.30958E+00_JPRB, 0.30947E+00_JPRB, 0.30924E+00_JPRB ,&
&  0.30904E+00_JPRB, 0.30879E+00_JPRB, 0.30851E+00_JPRB, 0.30798E+00_JPRB, 0.30730E+00_JPRB, 0.30658E+00_JPRB, 0.30581E+00_JPRB ,&
&  0.30509E+00_JPRB, 0.30422E+00_JPRB, 0.30341E+00_JPRB, 0.30269E+00_JPRB, 0.30199E+00_JPRB, 0.30153E+00_JPRB, 0.30120E+00_JPRB ,&
&  0.30106E+00_JPRB, 0.30113E+00_JPRB, 0.30137E+00_JPRB, 0.30173E+00_JPRB, 0.30209E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB ,&
&  0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB, 0.30129E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1930:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_OBS1930
